-- This file creates the space schema for Oracle databases.
-- Version 1.2.0, generated on 2018-09-26.
-- DO NOT MODIFY THIS FILE.  It is automatically generated.

-- Session settings.
-- Don't prompt for any ampersands:
set define off;
-- We don't need to see every message.
set feedback off;
-- But we do want to see a few specific messages indicating status.
set serveroutput on
-- Use the same date format to make file smaller:
alter session set nls_date_format = 'YYYY-MM-DD HH24:MI:SS';
-- Use CHAR semantics for UTF8 data.
alter session set nls_length_semantics='CHAR';


-- Print intro message.
exec dbms_output.put_line('------------------------------------------------------------------------');
exec dbms_output.put_line('-- Installing space database version 1.2.0, generated on 2018-09-26.');
exec dbms_output.put_line('-- ');
exec dbms_output.put_line('-- This data is from JSR Launch Vehicle Database, 2017 Dec 28 Edition.');
exec dbms_output.put_line('-- This file was generated by Jon Heller, jon@jonheller.org.');
exec dbms_output.put_line('-- The database installs 20 tables and uses about 27MB of space.');
exec dbms_output.put_line('--');
exec dbms_output.put_line('-- The installation will run for about a minute and will stop on any');
exec dbms_output.put_line('-- errors.  You should see a "done" message at the end.');
exec dbms_output.put_line('------------------------------------------------------------------------');

-- Stop at the first error.
whenever sqlerror exit sql.sqlcode;

--------------------------------------------------------------------------------
-- Checking for existing tables.
--------------------------------------------------------------------------------
exec dbms_output.put_line('Checking for existing tables...');

--Raise an error and stop installation if any tables exist.
declare
	v_tables_already_exist varchar2(32767);
begin
	select listagg(table_name, ',') within group (order by table_name)
	into v_tables_already_exist
	from all_tables
	where owner = sys_context('userenv', 'CURRENT_SCHEMA')
		and table_name in ('ORGANIZATION','PLATFORM','ORGANIZATION_ORG_TYPE','SITE','SITE_ORG','LAUNCH_VEHICLE_FAMILY','LAUNCH_VEHICLE','LAUNCH_VEHICLE_MANUFACTURER','LAUNCH','LAUNCH_PAYLOAD_ORG','LAUNCH_AGENCY','SATELLITE','SATELLITE_ORG','ENGINE','STAGE','LAUNCH_VEHICLE_STAGE','STAGE_MANUFACTURER','PROPELLANT','ENGINE_PROPELLANT','ENGINE_MANUFACTURER')
	;

	if v_tables_already_exist is not null then
		raise_application_error(-20000, 'These tables already exist in the schema ' ||
		sys_context('userenv', 'CURRENT_SCHEMA') || ': ' || v_tables_already_exist);
	end if;
end;
/


--------------------------------------------------------------------------------
-- ORGANIZATION
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing ORGANIZATION...');

CREATE TABLE "ORGANIZATION" 
 (	"ORG_CODE" VARCHAR2(9), 
	"ORG_NAME" VARCHAR2(81), 
	"ORG_CLASS" VARCHAR2(29), 
	"PARENT_ORG_CODE" VARCHAR2(13), 
	"ORG_STATE_CODE" VARCHAR2(7), 
	"ORG_LOCATION" VARCHAR2(53), 
	"ORG_START_DATE" DATE, 
	"ORG_STOP_DATE" DATE, 
	"ORG_UTF8_NAME" VARCHAR2(230), 
	 CONSTRAINT "ORGANIZATION_PK" PRIMARY KEY ("ORG_CODE")
USING INDEX  ENABLE, 
	 CONSTRAINT "ORGANIZATION_FK" FOREIGN KEY ("PARENT_ORG_CODE")
	  REFERENCES "ORGANIZATION" ("ORG_CODE") DISABLE
 ) ;

insert into ORGANIZATION
select '10ADS','USAF Aerospace Defence Command, 10th Aerospace Defence Sqn.','defense','AFADC','US','Vandenberg AFB, California','1966-01-01 00:00:00','','USAF Aerospace Defence Command, 10th Aerospace Defence Sqn.' from dual union all
select '11TMS','11th Tactical Missile Sqn, USAF','defense','USAF','US','Patrick AFB, Florida','1954-01-01 00:00:00','','11th Tactical Missile Sqn, USAF' from dual union all
select '1GMR','1st Guided Missile Regiment, US Army, White Sands','defense','USA','US','White Sands Proving Ground, New Mexico','1948-01-01 00:00:00','1950-01-01 00:00:00','1st Guided Missile Regiment, US Army, White Sands' from dual union all
select '1GMS','1st Guided Missile Squadron, 6540th Missile Test Wing (Mat/Snark)','defense','USAF','US','Patrick AFB, Florida','1950-01-01 00:00:00','','1st Guided Missile Squadron, 6540th Missile Test Wing (Mat/Snark)' from dual union all
select '1PBS','1st Pilotless Bomber Sqn, 6555 Guided Missile Group [TAC after 54Jan]','defense','USAF','US','Patrick AFB, Florida','1950-01-01 00:00:00','','1st Pilotless Bomber Sqn, 6555 Guided Missile Group [TAC after 54Jan]' from dual union all
select '1STRAD','1st Strategic Aerospace Division, SAC','defense','USAF','US','Vandenberg AFB, California','1961-07-21 00:00:00','1990-07-31 00:00:00','1st Strategic Aerospace Division, SAC' from dual union all
select '209FA','US 209th Field Artillery, 333rd Artillery, 4th Missile Battalion','defense','USA','US','Ft. Sill, Oklahoma','1958-01-01 00:00:00','1964-01-01 00:00:00','US 209th Field Artillery, 333rd Artillery, 4th Missile Battalion' from dual union all
select '21AT','TwentyFirst Century Aerospace Technology Co.','academic/non-profit','','CN','Beijing:Haidian','2011-01-01 00:00:00','','二十一世纪空间技术应用股份有限公司' from dual union all
select '2GMS','2nd Guided Missile Squadron, 6540th Missile Test Wing (Mat/Snark)','defense','USAF','US','Holloman AFB, New Mexico','1950-01-01 00:00:00','','2nd Guided Missile Squadron, 6540th Missile Test Wing (Mat/Snark)' from dual union all
select '2SLS','Boeing/USAF 2SLS','defense','USAF','US','Patrick AFB, Florida','1990-01-01 00:00:00','','Boeing/USAF 2SLS' from dual union all
select '30FA2','US Army 30th Field Artillery Bttn 2','defense','USA','US','Ft Sill, Oklahoma','1970-01-01 00:00:00','','US 30th Field Artillery Bttn 2' from dual union all
select '30SW','USAF 30th Space Wing','defense','USAF','US','Vandenberg AFB, California','1997-01-01 00:00:00','','USAF 30th Space Wing' from dual union all
select '321SMW','USAF 321 Strategic Missile Wing','defense','USAF','US','Grand Forks AFB, N Dakota','','','USAF 321 Strategic Missile Wing' from dual union all
select '341MW','USAF 341 Missile Wing','defense','USAF','US','Malstrom AFB, Montana','','1993-01-01 00:00:00','USAF 341 Missile Wing' from dual union all
select '341SMW','USAF 341 Missile Wing','defense','USAF','US','Malstrom AFB, Montana','1993-01-01 00:00:00','2000-01-01 00:00:00','USAF 341 Missile Wing' from dual union all
select '341SW','USAF 341 Space Wing','defense','USAF','US','Malstrom AFB, Montana','2000-01-01 00:00:00','','USAF 341 Space Wing' from dual union all
select '351SMW','USAF 351 Strategic Missile Wing','defense','USAF','US','Whiteman AFB, Missouri','1962-01-01 00:00:00','1995-01-01 00:00:00','USAF 351 Strategic Missile Wing' from dual union all
select '36AIS','36th Strategic Interdiction Squadron','defense','AMI','I','Gioia del Colle','1956-01-01 00:00:00','1963-01-01 00:00:00','36 Aerobrigata interdizione strategica (36 Strategic Interdict.Sqn)' from dual union all
select '3GMS','3rd Guided Missile Squadron, 550th Guided Missile Wing [Lark]','defense','USAF','US','Patrick AFB, Florida','1950-01-01 00:00:00','1950-01-01 00:00:00','3rd Guided Missile Squadron, 550th Guided Missile Wing [Lark]' from dual union all
select '3SLS','USAF 3SLS','defense','USAF','US','Patrick AFB, Florida','1990-01-01 00:00:00','','USAF 3SLS' from dual union all
select '40FA','US 40th Field Artillery, 217th Field Artillery Missile Battalion','defense','USA','US','Huntsville, Alabama','1958-01-01 00:00:00','1959-01-01 00:00:00','US 40th Field Artillery, 217th Field Artillery Missile Battalion' from dual union all
select '40FA1','US 40th Field Artillery, 333rd Artillery, 1st Missile Battalion','defense','USA','US','Huntsville, Alabama','1959-01-01 00:00:00','1964-01-01 00:00:00','US 40th Field Artillery, 333rd Artillery, 1st Missile Battalion' from dual union all
select '41FA1','US 41st Field Artillery, Battalion 1, US 7th Army','defense','USA','US','Schwabisch Gmund, Baden:Wurttemberg','1964-01-01 00:00:00','1991-01-01 00:00:00','US 41st Field Artillery, Battalion 1, US 7th Army' from dual union all
select '41FA4','US 41st Field Artillery, Battalion 4','defense','USA','US','Ft. Sill, Oklahoma','1962-01-01 00:00:00','1964-01-01 00:00:00','US 41st Field Artillery, Battalion 4' from dual union all
select '44FA2','US 44th Field Artillery, Battalion 2','defense','USA','US','Ft. Sill, Oklahoma','','1991-01-01 00:00:00','US 44th Field Artillery, Battalion 2' from dual union all
select '44SMW','USAF 44th Strategic Missile Wing','defense','USAF','US','Ellsworth AFB, Rapid City, S Dakota','1962-01-01 00:00:00','1991-04-01 00:00:00','USAF 44th Strategic Missile Wing' from dual union all
select '4504MTW','USAF 4504 Missile Training Wing','defense','USAF','US','Patrick AFB, Florida','1961-01-01 00:00:00','','USAF 4504 Missile Training Wing' from dual union all
select '4505M','USAF 4505 Missile Training Wing','defense','USAF','US','Patrick AFB, Florida','1961-01-01 00:00:00','','USAF 4505 Missile Training Wing' from dual union all
select '455SMW','USAF 455th Strategic Missile Wing','defense','USAF','US','Minot AFB, N Dakota','1962-01-01 00:00:00','1968-01-01 00:00:00','USAF 455th Strategic Missile Wing' from dual union all
select '46FA2','US 46th Field Artillery, 333rd Artillery, 2nd Missile Battalion','defense','USA','US','Ft. Sill, Oklahoma','1958-01-01 00:00:00','1964-01-01 00:00:00','US 46th Field Artillery, 333rd Artillery, 2nd Missile Battalion' from dual union all
select '4803GMS','4803rd Guided Missile Squadron, 4800 Guided Missile Wing [Lark]','defense','USAF','US','Patrick AFB, Florida','1951-01-01 00:00:00','1953-01-01 00:00:00','4803rd Guided Missile Squadron, 4800 Guided Missile Wing [Lark]' from dual union all
select '4SLS','USAF 4SLS','defense','USAF','US','Patrick AFB, Florida','1990-01-01 00:00:00','','USAF 4SLS' from dual union all
select '4WG','USAF 4th Weather Group, Ft Churchill (AWS)','defense','AWS','US','Churchill, Manitoba','','','USAF 4th Weather Group, Ft Churchill (AWS)' from dual union all
select '556SMS','USAF 556th Strategic Missile Squadron','defense','USAF','US','Cape Canaveral AFS, Florida','1958-01-01 00:00:00','','USAF 556th Strategic Missile Squadron' from dual union all
select '564SMS','USAF 564th Strategic Missile Squadron','defense','USAF','US','Malstrom AFB, Montana','','','USAF 564th Strategic Missile Squadron' from dual union all
select '576FTS','USAF 576 Flight Test Squadron, SAC','defense','SAC','US','Vandenberg AFB, California','1991-09-01 00:00:00','','USAF 576 Flight Test Squadron, SAC' from dual union all
select '576SMS','USAF 576th Strategic Missile Squadron, SAC','defense','SAC','US','Vandenberg AFB, California','1958-03-06 00:00:00','1966-04-02 00:00:00','USAF 576th Strategic Missile Squadron, SAC' from dual union all
select '6555A','USAF 6555 Aerospace Test Wing','defense','USAF','US','Cape Canaveral AFS, Florida','','','USAF 6555 Aerospace Test Wing' from dual union all
select '6555GMS','USAF 6555th Guided Missile Sqn','defense','USAF','US','Cape Canaveral AFS, Florida','1951-01-01 00:00:00','1955-01-01 00:00:00','USAF 6555th Guided Missile Sqn' from dual union all
select '69PBS','69th Pilotless Bomber Sqn, 6555 Guided Missile Group [TAC after 54Jan]','defense','USAF','US','Patrick AFB, Florida','1950-01-01 00:00:00','','69th Pilotless Bomber Sqn, 6555 Guided Missile Group [TAC after 54Jan]' from dual union all
select '702SMW','USAF 702 Strategic Missile WIng, SAC','defense','SAC','US','Presque Isle AFB, Maine','1959-01-01 00:00:00','1961-01-01 00:00:00','USAF 702 Strategic Missile WIng, SAC' from dual union all
select '79FA2','US 79th Field Artillery, Battalion 2','defense','USA','US','Fort Sill, Oklahoma','','1965-01-01 00:00:00','US 79th Field Artillery, Battalion 2' from dual union all
select '81FA1','US 81st Field Artillery, Battalion 1, US 7th Army','defense','USA','US','Neu Ulm, Bayern','','','US 81st Field Artillery, Battalion 1, US 7th Army' from dual union all
select '84FA3','US 84th Field Artillery, Battalion 3, US 7th Army','defense','USA','US','Heilbronn, Baden:Wurttemburg','','','US 84th Field Artillery, Battalion 3, US 7th Army' from dual union all
select '90MW','USAF 90th Space Wing','defense','SAC','US','Warren AFB, Cheyenne, Wyoming','2000-01-01 00:00:00','','USAF 90th Space Wing' from dual union all
select '90SMW','USAF 90th Strategic Missile Wing','defense','SAC','US','Warren AFB, Cheyenne, Wyoming','1963-07-01 00:00:00','1997-09-04 00:00:00','USAF 90th Strategic Missile Wing' from dual union all
select '90SW','USAF 90th Space Wing','defense','SAC','US','Warren AFB, Cheyenne, Wyoming','1997-09-04 00:00:00','2000-01-01 00:00:00','USAF 90th Space Wing' from dual union all
select '91MW','USAF 91st Missile Wing','defense','SAC','US','Minot AFB, N Dakota','2008-01-01 00:00:00','','USAF 91st Missile Wing' from dual union all
select '91SW','USAF 91st Space Wing','defense','SAC','US','Minot AFB, N Dakota','1968-01-01 00:00:00','2008-01-01 00:00:00','USAF 91st Missile Wing' from dual union all
select '9FA3','US 9th Field Artiller, Batallion 3','defense','USA','US','Fort Sill, Oklahoma','','','US 9th Field Artiller, Batallion 3' from dual union all
select 'AAAF','Alamagordo Army Air Field','defense','USAF','US','Alamagordo, New Mexico','1941-04-13 00:00:00','1948-01-13 00:00:00','Alamagordo Army Air Field' from dual union all
select 'AACH','Aachen University of Applied Sciences','academic/non-profit','','D','Aachen','1971-01-01 00:00:00','','Fachhochschule Aachen' from dual union all
select 'AACHR','RWTH Aachen University','academic/non-profit','','D','Aachen','1870-01-01 00:00:00','','Rheinisch-Westfälische Technische Hochschule Aachen' from dual union all
select 'AACM','AAC Microtec AB','business','','S','Uppsala','2005-01-01 00:00:00','','ÅAC Microtec' from dual union all
select 'AADC','Alaska Aerospace Development Corporation','business','','US','Anchorage, Alaska','1991-01-01 00:00:00','2009-10-01 00:00:00','Alaska Aerospace Development Corporation' from dual union all
select 'AALTO','Aalto University','academic/non-profit','','FI','Helsinki','2010-01-01 00:00:00','','Aalto-universitetet' from dual union all
select 'AAMV','Max Valier Amateur Astronomers, South Tyrol','academic/non-profit','','I','Bolzano, Alto Adige (Sudtirol)','2012-01-01 00:00:00','','Amateurastronomen Max Valier' from dual union all
select 'AAP','Americom Asia Pacific (LMGT/GE Americom)','business','LMGT','SG','Singapore','1998-01-01 00:00:00','','Americom Asia Pacific (LMGT/GE Americom)' from dual union all
select 'AAT','AAT','','AU','AAT','Davis Staton, Princess Elizabeth Land','1841-01-01 00:00:00','','Australian Antarctic Territory' from dual union all
select 'AAU','Aalborg University','academic/non-profit','','DK','Aalborg, Denmark','1974-01-01 00:00:00','','Aalborg Universitet' from dual union all
select 'ABAE','Bolivarian Agency for Space Activities','civilian','','VE','Caracas','2008-01-01 00:00:00','','Agencia Bolivariana para Activadades Espaciales' from dual union all
select 'ABE','Bolivian Space Agency','civilian','','BO','La Paz, Bolivia','2010-04-01 00:00:00','','Agencia Boliviana spacial' from dual union all
select 'ABERU','Prifysgol Aberystwyth (Aberystwyth University)','academic/non-profit','','UK','Aberystwyth, Wales','2007-09-01 00:00:00','','Prifysgol Aberystwyth (Aberystwyth University)' from dual union all
select 'ABL','Alleghany Ballistic Laboratories (USN/Hercules Powder Co)','defense','USN','US','Cumberland:Rocket Center, West Virginia','1945-01-01 00:00:00','1995-01-01 00:00:00','Alleghany Ballistic Laboratories (USN/Hercules Powder Co)' from dual union all
select 'ABLN','Alleghany Ballistic Laboratories (NRDC/GWU)','defense','','US','Cumberland:Rocket Center, West Virginia','1944-01-01 00:00:00','1945-01-01 00:00:00','Alleghany Ballistic Laboratories (NRDC/GWU)' from dual union all
select 'ABMA','U.S. Army Ballistic Missile Agency','defense','USA','US','Huntsville, Alabama','1956-02-01 00:00:00','1961-12-01 00:00:00','U.S. Army Ballistic Missile Agency' from dual union all
select 'ABMDA','Aegis Ballistic Missile Defense Agency','defense','USN','US','Dahlgren, Virginia','2007-01-01 00:00:00','','Aegis Ballistic Missile Defense Agency' from dual union all
select 'ABMDO','US Army Ballistic Missile Defense Organization','defense','USA','US','Huntsville, Alabama?','1974-05-20 00:00:00','1985-07-01 00:00:00','US Army Ballistic Missile Defense Organization' from dual union all
select 'ABS','Asia Broadcast Satellite Ltd.','business','','HK','Hong Kong','2006-01-01 00:00:00','2012-01-01 00:00:00','Asia Broadcast Satellite Ltd.' from dual union all
select 'ABSB','Asia Broadcast Satellite Ltd.','business','','BM','Bermuda','2012-01-01 00:00:00','','Asia Broadcast Satellite Ltd.' from dual union all
select 'ABSCA','Asia Broadcast Satellite Holdings Ltd.','business','ABS','US','Sunnyvale, California?','2006-09-01 00:00:00','2006-09-01 00:00:00','Asia Broadcast Satellite Holdings Ltd.' from dual union all
select 'ABSHK','Asia Broadcast Satellite (Hong Kong) Ltd.','business','ABSB','HK','Hong Kong','2012-01-01 00:00:00','','Asia Broadcast Satellite (Hong Kong) Ltd.' from dual union all
select 'ABSL','Asia Broadcast Satellite Ltd.','business','ABS','UK','London','2006-01-01 00:00:00','2006-01-01 00:00:00','Asia Broadcast Satellite Ltd.' from dual union all
select 'ACAE','Central Americal Association of Aeronautics and Space','academic/non-profit','','CR','San Jose, Costa Rica','2010-01-01 00:00:00','','Asociación Centroamericana de Aeronáutica y del Espacio' from dual union all
select 'ACC','Air Combat Command','defense','USAF','US','Langley AFB, Virginia','1992-06-01 00:00:00','','Air Combat Command' from dual union all
select 'ACE','Army Map Service, US Army Corps of Engineers','defense','USA','US','Bethesda, Maryland','1942-07-01 00:00:00','1968-09-01 00:00:00','Army Map Service, US Army Corps of Engineers' from dual union all
select 'ACES','Asia Cellular Satellite','business','PSN','ID','Jakarta','1995-01-01 00:00:00','2006-01-01 00:00:00','Asia Cellular Satellite' from dual union all
select 'ACESI','Inmarsat/Asia Cellular Satellite','business','INMRL','ID','Jakarta','2006-01-01 00:00:00','2015-01-01 00:00:00','Inmarsat/Asia Cellular Satellite' from dual union all
select 'ACF','French Aero Club','academic/non-profit','','F','Paris, France','1898-01-01 00:00:00','','Aéro-Club de France' from dual union all
select 'ADAST','Ad Astra Rocket Company','business','','US','Houston:Webster, Texas','2005-01-01 00:00:00','','Ad Astra Rocket Company' from dual union all
select 'ADD','Agency for Defense Development','defense','KR','KR','Daejon','1983-01-01 00:00:00','','Agency for Defense Development (Daejon)' from dual union all
select 'ADEL','University of Adelaide','academic/non-profit','','AU','Adelaide, South Australia','1874-01-01 00:00:00','','University of Adelaide' from dual union all
select 'ADIG','Astro Digital','business','','US','Mountain View, California','2015-01-01 00:00:00','','Astro Digital' from dual union all
select 'ADS','Airbus Defense and Space','business','','NL','Schiphol:Rijk, Amsterdam','2014-01-01 00:00:00','','Airbus Defense and Space' from dual union all
select 'ADSAQ','Airbus Defense and Space/Aquitaine','business','ADS','F','Bordeaux:St-Medard-en-Jalles','2014-01-01 00:00:00','','Airbus Defense and Space/Aquitaine' from dual union all
select 'ADSB','Airbus Defense and Space, Bremen','business','ADS','D','Bremen','2014-01-01 00:00:00','','Airbus Defense and Space, Bremen' from dual union all
select 'ADSDF','Airbus Defense and Space Satellites (D), Friedrichsafen','business','ADS','D','Friedrichsafen','2014-01-01 00:00:00','','Airbus Defense and Space Satellites (D), Friedrichsafen' from dual union all
select 'ADSFL','Airbus Defense and Space Inc (US)','business','ADS','US','Cape Canaveral, Florida','2018-01-01 00:00:00','','Airbus Defense and Space Inc (US)' from dual union all
select 'ADSG','Airbus Defense and Space Launch Vehicles  CSG','business','ADS','F','CSG, Kourou, Guyane Francaise','2014-01-01 00:00:00','','Airbus Defense and Space Launch Vehicles - CSG' from dual union all
select 'ADSL','Airbus Defense and Space (D), Lampoldshausen','business','ADS','D','Lampoldshausen','2014-01-01 00:00:00','','Airbus Defense and Space (D), Lampoldshausen' from dual union all
select 'ADSLV','Airbus Defense and Space Launch Vehicles (ADS LV)','business','ADS','F','Les Mureaux, Paris, France','2014-01-01 00:00:00','2015-01-01 00:00:00','Airbus Defense and Space Launch Vehicles (ADS LV)' from dual union all
select 'ADSM','Airbus Defense and Space Satellites/Barajas  (CASA Espacio)','business','ADS','E','Madrid:Barajas','2014-01-01 00:00:00','','Airbus Defense and Space Satellites/Barajas  (CASA Espacio)' from dual union all
select 'ADSNA','Airbus DS Space Systems Inc.','business','ADS','US','Houston, Texas','2014-01-01 00:00:00','','Airbus DS Space Systems Inc.' from dual union all
select 'ADSO','Airbus Defense and Space Propulsion/Ottobrunn','business','ADS','D','Munchen:Ottobrunn','2014-01-01 00:00:00','','Airbus Defense and Space Propulsion/Ottobrunn' from dual union all
select 'ADSP','Airbus Defense and Space (UK) (Corsham)','business','','UK','Corsham, Wiltshire','2015-01-01 00:00:00','','Airbus Defense and Space (UK) (Corsham)' from dual union all
select 'ADST','Airbus Defense and Space (F) Toulouse','business','ADS','F','Toulouse','2014-01-01 00:00:00','','Airbus Defense and Space (F) Toulouse' from dual union all
select 'ADSUK','Airbus Defense and Space (UK)','business','ADS','UK','Stevenage','2014-01-01 00:00:00','','Airbus Defense and Space (UK)' from dual union all
select 'AE','Arianespace','business','','F','Paris:Evry, France','1980-03-26 00:00:00','2017-03-01 00:00:00','Arianespace, Inc.' from dual union all
select 'AEB','BSA','civilian','BR','BR','San Jose dos Campos, Brasil','','','Agência Espacial Brasileira' from dual union all
select 'AEC','U.S. Atomic Energy Commission','defense','','US','Washington, D.C.','1947-01-01 00:00:00','1975-01-19 00:00:00','U.S. Atomic Energy Commission' from dual;
insert into ORGANIZATION
select 'AECAB','AEC/Able Engineering Inc.','business','','US','Goleta, California','1975-01-01 00:00:00','2004-01-01 00:00:00','AEC/Able Engineering Inc.' from dual union all
select 'AECSG','Aerospatiale/Kourou','business','','F','CSG, Kourou, Guyane Francaise','1995-01-01 00:00:00','2000-07-10 00:00:00','Aérospatiale - Centre spatial guyanais' from dual union all
select 'AEDC','Arnold Engineering Development Center, Arnold AFB','defense','USAF','US','Tullahoma, Tennessee','1951-01-01 00:00:00','','Arnold Engineering Development Center, Arnold AFB' from dual union all
select 'AELM','Aerospatiale/Les Mureaux','business','','F','Paris:Les Mureaux, Yvelines, France','1970-01-01 00:00:00','1999-07-01 00:00:00','Aérospatiale Les Mureaux' from dual union all
select 'AEML','Aerospatiale Matra Lanceurs','business','','F','Paris:Les Mureaux, Yvelines, France','1999-07-01 00:00:00','2000-07-10 00:00:00','Aérospatiale Matra Lanceurs' from dual union all
select 'AENUT','Aeronutronic Co.','business','','US','Los Angeles:Glendale, California','1955-01-01 00:00:00','1956-01-01 00:00:00','Aeronutronic Co.' from dual union all
select 'AERAR','Argentine Air Force','defense','','AR','Buenos Aires:El Palomar','1945-01-01 00:00:00','','Fuerza Aérea Argentina' from dual union all
select 'AERIT','Aeritalia Space Systems','business','','I','Torino','1969-01-01 00:00:00','1990-01-01 00:00:00','Aeritalia Sistemi Spaziali (Torino)' from dual union all
select 'AERO','Aerospace Corp.','defense','','US','El Segundo, California','1960-06-01 00:00:00','','Aerospace Corp.' from dual union all
select 'AEROA','AeroAstro, Inc','business','','US','Ashburn, Virginia','1988-01-01 00:00:00','2008-01-01 00:00:00','AeroAstro, Inc' from dual union all
select 'AEROAC','Comtech AeroAstro, Inc','business','','US','Ashburn, Virginia','2008-01-01 00:00:00','2012-07-01 00:00:00','Comtech AeroAstro, Inc' from dual union all
select 'AEROACL','Comtech AeroAstro/Littleton','business','AEROA','US','Littleton, Colorado','2011-01-01 00:00:00','2012-07-01 00:00:00','Comtech AeroAstro/Littleton' from dual union all
select 'AEROL','Aerolab Development Co.','business','','US','Pasadena, California','','1960-01-01 00:00:00','Aerolab Development Co.' from dual union all
select 'AEROLR','Ryan Aerolab Development Co.','business','','US','Pasadena/Monrovia, California','1960-01-01 00:00:00','1967-01-01 00:00:00','Ryan Aerolab Development Co.' from dual union all
select 'AERON','Aeronomy Service, CNRS','civilian','CNRS','F','Paris:Verrieres:le:Buisson','1961-01-01 00:00:00','2009-01-01 00:00:00','Service d''Aéronomie du CNRS, Verrières-le-Buisson' from dual union all
select 'AERONM','Aeronomy Service, CNRS','civilian','CNRS','F','Paris:Meudon','1958-01-01 00:00:00','1961-01-01 00:00:00','Service d''Aéronomie du CNRS, Meudon' from dual union all
select 'AEROPH','Aerophysics Development Corporation','business','','US','Santa Barbara:Goleta, California','1955-01-01 00:00:00','1960-01-01 00:00:00','Aerophysics Development Corporation' from dual union all
select 'AES','Atmospheric Environment Service, Dept. of the Environment','civilian','CA','CA','Toronto:Downsview','1971-01-01 00:00:00','2002-01-01 00:00:00','Atmospheric Environment Service, Dept. of the Environment' from dual union all
select 'AESCO','Advanced Engineering Services Co. Ltd.','business','','J','Tsukuba','1986-04-01 00:00:00','','エイ･イー･エス' from dual union all
select 'AESP','Arianespace','business','ARIG','F','Paris:Evry, France','2017-03-01 00:00:00','','Arianespace SAS' from dual union all
select 'AESPC','AerospatialeCannes','business','','F','Cannes','1970-01-01 00:00:00','1998-06-01 00:00:00','Société nationale industrielle aérospatiale, Groupe technique de Cannes' from dual union all
select 'AESPP','Aerospatiale, Ch. Courbevoie  (SEREB)','business','','F','Paris:Courbevoie','1970-01-01 00:00:00','1973-01-01 00:00:00','Aérospatiale - Courbevoie' from dual union all
select 'AETE','Aerospace Engineering Test Establishment, CFB Cold Lake','defense','CFCA','CA','Cold Lake, Alberta','1971-01-01 00:00:00','','Aerospace Engineering Test Establishment, CFB Cold Lake' from dual union all
select 'AF','Afganistan','','','AF','Afghanistan','2004-01-01 00:00:00','','د افغانستان اسلامي جمهوریت' from dual union all
select 'AFADC','US Air Force Aerospace Defense Command','defense','USAF','US','Cheyenne Mountain, Colorado','1968-01-01 00:00:00','1980-01-01 00:00:00','US Air Force Aerospace Defense Command' from dual union all
select 'AFADCA','US Air Force Air Defense Command','defense','USAF','US','Colorado Springs, Colorado','1946-01-01 00:00:00','1968-01-01 00:00:00','US Air Force Air Defense Command' from dual union all
select 'AFAL','US Air Force Avionics Laboratory, USAF ASD','defense','AFASD','US','Wright-Patterson AFB, Dayton, Ohio','1963-01-01 00:00:00','1992-01-01 00:00:00','US Air Force Avionics Laboratory, USAF ASD' from dual union all
select 'AFAMC','Air Materiel Command','defense','USAF','US','Wright-Patterson AFB, Dayton, Ohio','1946-03-01 00:00:00','1961-04-01 00:00:00','Air Materiel Command' from dual union all
select 'AFAPL','US Air Force Aero Propulsion Laboratory, USAF ASD','defense','USAF','US','Wright-Patterson AFB, Dayton, Ohio','1963-01-01 00:00:00','1992-01-01 00:00:00','US Air Force Aero Propulsion Laboratory, USAF ASD' from dual union all
select 'AFASD','Aeronautical Systems Division, ARDC/AFSC','defense','USAF','US','Wright-Patterson AFB, Dayton, Ohio','1961-01-01 00:00:00','1992-01-01 00:00:00','Aeronautical Systems Division, ARDC/AFSC' from dual union all
select 'AFBMD','US Air Force Ballistic Missile Division','defense','ARDC','US','El Segundo, California','1957-06-01 00:00:00','1961-04-01 00:00:00','US Air Force Ballistic Missile Division' from dual union all
select 'AFBMO','US Air Force Ballistic Missile Office AFSC','defense','AFSC','US','Norton AFB, San Bernardino, California','1979-10-01 00:00:00','1989-03-01 00:00:00','US Air Force Ballistic Missile Office AFSC' from dual union all
select 'AFBMO2','US Air Force Ballistic Missile Organization  AFSC','defense','AFSC','US','Norton AFB, San Bernardino, California','1990-05-01 00:00:00','1993-09-01 00:00:00','US Air Force Ballistic Missile Organization  AFSC' from dual union all
select 'AFBSD','US Air Force Ballistic Systems Division, AFSC','defense','AFSC','US','Norton AFB, San Bernardino, California','1962-09-01 00:00:00','1967-07-01 00:00:00','US Air Force Ballistic Systems Division, AFSC' from dual union all
select 'AFBSD2','US Air Force Ballistic Systems Division, AFSC','defense','AFSC','US','Norton AFB, San Bernardino, California','1989-03-01 00:00:00','1990-05-01 00:00:00','US Air Force Ballistic Systems Division, AFSC' from dual union all
select 'AFBSL','US Air Force Ballistic Systems Division','defense','AFSC','US','El Segundo, California','1961-04-01 00:00:00','1962-09-01 00:00:00','US Air Force Ballistic Systems Division' from dual union all
select 'AFCFS','AFCFS','defense','USAF','US','Cambridge, MA  (224 Albany St)','1945-09-20 00:00:00','1949-07-05 00:00:00','US Air Force, Cambridge Field Station' from dual union all
select 'AFCRCA','AFCRC','defense','USAF','US','Cambridge, MA','1951-06-28 00:00:00','1959-01-01 00:00:00','US Air Force Cambridge Research Center' from dual union all
select 'AFCRCH','AFCRC Hanscom','defense','USAF','US','Bedford, MA','1951-06-28 00:00:00','1960-08-01 00:00:00','US Air Force Cambridge Research Center' from dual union all
select 'AFCRH','AFCRL Hanscom','defense','USAF','US','Bedford, MA','1949-07-05 00:00:00','1951-06-28 00:00:00','US Air Force Cambridge Research Laboratories (Hanscom)' from dual union all
select 'AFCRL','AFCRL','defense','USAF','US','Bedford, MA','1960-08-01 00:00:00','1976-01-15 00:00:00','US Air Force Cambridge Research Laboratories' from dual union all
select 'AFCRL1','AFCRL Cambridge','defense','USAF','US','Cambridge, MA','1949-07-05 00:00:00','1951-06-28 00:00:00','US Air Force Cambridge Research Laboratories' from dual union all
select 'AFETR','USAF ETR','defense','USAF','US','Patrick AFB, Cape Canaveral, Florida','1964-01-01 00:00:00','1977-01-01 00:00:00','US Air Force Eastern Test Range' from dual union all
select 'AFFTC','AFFTC','defense','USAF','US','Edwards AFB, California','1947-01-01 00:00:00','','US Air Force Flight Test Center' from dual union all
select 'AFGL','AFGL','defense','USAF','US','Bedford, MA','1976-01-15 00:00:00','1989-01-01 00:00:00','US Air Force Geophysics Laboratory' from dual union all
select 'AFGL2','USAF GL','defense','USAF','US','Bedford, MA','1989-01-01 00:00:00','1990-12-01 00:00:00','US Air Force Geophysics Laboratory (AFSC)' from dual union all
select 'AFGSC','USAF GSC','defense','USAF','US','Barksdale AFB, Bossier City, Louisiana','2009-12-01 00:00:00','','US Air Force Global Strike Command' from dual union all
select 'AFIT','AFIT','defense','USAF','US','Wright-Patterson AFB, Dayton, Ohio','1919-01-01 00:00:00','','US Air Force Institute of Technology, Wright-Patterson AFB' from dual union all
select 'AFLC','AFLC','defense','USAF','US','Wright-Patterson AFB, Dayton, Ohio','1961-04-01 00:00:00','1992-07-01 00:00:00','US Air Force Logistics Command' from dual union all
select 'AFMC','USAF AFMC','defense','USAF','US','Wright-Patterson AFB, Dayton, Ohio','1992-07-01 00:00:00','','US Air Force Materiel Command' from dual union all
select 'AFMCIT','Afghan MCIT','civilian','','AF','Kabul','2004-01-01 00:00:00','','Ministry of Communciations and Information Technology' from dual union all
select 'AFMCSW','US Air Force Military Satellite Communications Wing, AFSMC, LAAFB','defense','AFSMC','US','Los Angeles:El Segundo, California','2006-01-01 00:00:00','','US Air Force Military Satellite Communications Wing, AFSMC, LAAFB' from dual union all
select 'AFMDC','Air Force Missile Development Center, Holloman Air Force Base','defense','HAFB','US','Alamagordo, New Mexico','1957-09-01 00:00:00','1970-08-01 00:00:00','Air Force Missile Development Center, Holloman Air Force Base' from dual union all
select 'AFMTC','AFMTC','defense','USAF','US','Patrick AFB, Cape Canaveral, Florida','1951-01-01 00:00:00','1964-01-01 00:00:00','US Air Force Missile Test Center' from dual union all
select 'AFORS','DoD ORS','defense','USAF','US','Kirtland AFB, Albuquerque, New Mexico','1997-10-01 00:00:00','','Operationally Responsive Space Office, Dept. of Defense' from dual union all
select 'AFPLH','AF Phillips/GD','defense','USAF','US','Bedford, MA','1990-12-01 00:00:00','1997-10-01 00:00:00','US Air Force Phillips Laboratory, Geophysics Directorate' from dual union all
select 'AFRC','NASA Neil A. Armstrong Flight Research Center','civilian','NASA','US','Edwards AFB, California','2014-01-08 00:00:00','','NASA Neil A. Armstrong Flight Research Center' from dual union all
select 'AFRCO','AF RCO','defense','USAF','US','Washington, DC','2003-01-01 00:00:00','','US Air Force Rapid Capabilities Office' from dual union all
select 'AFRL','AFRL/RV','defense','USAF','US','Kirtland AFB, Albuquerque, New Mexico','1997-10-01 00:00:00','','US Air Force Research Laboratories/RV (Space Vehicles Directorate)' from dual union all
select 'AFRLH','AFRLVS/Hanscom','defense','AFRL','US','Bedford, MA','1997-10-01 00:00:00','2006-01-01 00:00:00','US Air Force Research Lab, Space Vehicles Dir., Hanscom' from dual union all
select 'AFRLRZ','AFRL Propulsion','defense','USAF','US','Wright-Patterson AFB, Dayton, Ohio','1997-10-01 00:00:00','','US Air Force Research Lab, Propulsion Directorate' from dual union all
select 'AFRLW','AFRL Det 1','defense','USAF','US','Wright-Patterson AFB, Dayton, Ohio','1997-10-01 00:00:00','','US Air Force Research Lab, Wright' from dual union all
select 'AFROM','USAF Rome','defense','AFMC','US','Rome, New York','1951-01-01 00:00:00','1997-01-01 00:00:00','US Air Force Rome Labs, Griffiss AFB' from dual union all
select 'AFRPL','US Air Force Rocket Propulsion Laboratory, USAF ASD','defense','AFASD','US','Edwards AFB, California','1963-01-01 00:00:00','1992-01-01 00:00:00','US Air Force Rocket Propulsion Laboratory, USAF ASD' from dual union all
select 'AFSAM','US Air Force School of Aerospace Medicine (Brooks)','defense','AFMC','US','Brooks AFB, San Antonio, Texas','1959-01-01 00:00:00','2011-01-01 00:00:00','US Air Force School of Aerospace Medicine (Brooks)' from dual union all
select 'AFSC','US Air Force Systems Command, Los Angeles AFS','defense','USAF','US','El Segundo, California','1961-04-01 00:00:00','1992-07-01 00:00:00','US Air Force Systems Command, Los Angeles AFS' from dual union all
select 'AFSD','US Air Force Space Division','defense','USAF','US','El Segundo, California','1979-10-01 00:00:00','1989-03-01 00:00:00','US Air Force Space Division' from dual union all
select 'AFSMB','US Air Force Space and Missile Center (Norton)','defense','AFSC','US','Norton AFB, San Bernardino, California','1993-09-01 00:00:00','1995-09-01 00:00:00','US Air Force Space and Missile Center (Norton)' from dual union all
select 'AFSMC','US Air Force Space and Missile Center','defense','USAF','US','El Segundo, California','1992-07-01 00:00:00','','US Air Force Space and Missile Center' from dual union all
select 'AFSMS','US Air Force Sec. AF, Office of Satellite and Missile Systems','defense','USAF','US','Arlington, Virginia?  / El Segundo','1960-09-01 00:00:00','','US Air Force Sec. AF, Office of Satellite and Missile Systems' from dual union all
select 'AFSOC','US Air Force Special Operations Command','defense','USAF','US','Eglin, Florida','1990-05-22 00:00:00','','US Air Force Special Operations Command' from dual union all
select 'AFSPC','US Air Force Space Command','defense','USAF','US','Peterson AFB, Colorado','1982-09-01 00:00:00','','US Air Force Space Command' from dual union all
select 'AFSSD','US Air Force Space Systems Division','defense','AFSC','US','El Segundo, California','1961-04-01 00:00:00','1967-07-01 00:00:00','US Air Force Space Systems Division' from dual union all
select 'AFSSD2','US Air Force Space Systems Division','defense','USAF','US','El Segundo, California','1989-03-01 00:00:00','1992-07-01 00:00:00','US Air Force Space Systems Division' from dual union all
select 'AFSWC','US Air Force Special Weapons Center','defense','USAF','US','Kirtland AFB, Albequerque, New Mexico','1952-04-01 00:00:00','1976-04-01 00:00:00','US Air Force Special Weapons Center' from dual union all
select 'AFSWP','US Air Force Special Weapons Project, Sandia Base','defense','USAF','US','Sandia AFB, Albequerque, New Mexico','1947-01-01 00:00:00','1959-01-01 00:00:00','US Air Force Special Weapons Project, Sandia Base' from dual union all
select 'AFTAC','US Air Force Technical Applications Center','defense','USAF','US','Patrick AFB, Florida','1947-01-01 00:00:00','','US Air Force Technical Applications Center' from dual union all
select 'AFWDD','AF WDD','defense','USAF','US','El Segundo, California','1954-07-01 00:00:00','1957-06-01 00:00:00','US Air Force Western Development Division' from dual union all
select 'AFWL','AFWL','defense','USAF','US','Kirtland AFB, Albequerque, New Mexico','1963-01-01 00:00:00','1990-01-01 00:00:00','US Air Force Weapons Lab' from dual union all
select 'AG','Antigua','','','AG','Antigua','1981-01-01 00:00:00','','Antigua and Barbuda' from dual union all
select 'AGILS','AgilSpace','business','STESS','SG','Singapore','2015-01-01 00:00:00','','AgilSpace' from dual union all
select 'AGSL','ABS Global Satellite Ltd.','business','','PG','Port Moresby','2017-01-01 00:00:00','','ABS Global Satellite Ltd.' from dual union all
select 'AGUK','Antigua','','UK','AGUK','Antigua','1832-01-01 00:00:00','1981-01-01 00:00:00','Antigua colony' from dual union all
select 'AIL','Airborne Instruments Laboratory','business','','US','Mineola New York','1945-01-01 00:00:00','1958-01-01 00:00:00','Airborne Instruments Laboratory' from dual union all
select 'AJ','Aerojet Engineering','business','GTRC','US','Azusa, California','1947-01-01 00:00:00','2001-01-01 00:00:00','Aerojet Engineering' from dual union all
select 'AJCA','Aerojet/Camden','engine/motor manufacturer','GTRC','US','East Camden, Arkansas','2003-01-01 00:00:00','','Aerojet/Camden' from dual union all
select 'AJGC','Aerojet General Corp.','business','GTRC','US','Sacramento:Rancho Cordova, California','1953-01-01 00:00:00','2013-06-01 00:00:00','Aerojet General Corp.' from dual union all
select 'AJP','Aerojet Engineering, Pasadena','business','GTRC','US','Pasadena, California','1942-01-01 00:00:00','1947-01-01 00:00:00','Aerojet Engineering, Pasadena' from dual union all
select 'AJR','Aerojet Rocketdyne','business','GTRC','US','Sacramento:Rancho Cordova, California','2013-06-01 00:00:00','','Aerojet Rocketdyne' from dual union all
select 'AJRCP','Aerojet Rocketdyne/Canoga Park','business','GTRC','US','Canoga Park, California','2013-06-01 00:00:00','','Aerojet Rocketdyne/Canoga Park' from dual union all
select 'AJRED','Aerojet/Redmond','business','AJ','US','Redmond, Washington','2002-08-01 00:00:00','2013-06-01 00:00:00','Aerojet/Redmond' from dual union all
select 'AJRFL','Aerojet Rocketdyne/West Palm Beach','business','AJR','US','West Palm Beach, Florida','2013-06-01 00:00:00','','Aerojet Rocketdyne/West Palm Beach' from dual union all
select 'AJRRED','Aerojet Rocketdyne/Redmond','business','AJR','US','Redmond, Washington','2013-06-01 00:00:00','','Aerojet Rocketdyne/Redmond' from dual union all
select 'AJRSP','Aerojet Solid Propulsion Co., AF Plant 70','business','AJ','US','Sacramento:Rancho Cordova, California','2013-06-01 00:00:00','','Aerojet Solid Propulsion Co., AF Plant 70' from dual union all
select 'AJSP','Aerojet Rocketdyne Solid Propulsion Co., AF Plant 70','business','AJR','US','Sacramento:Rancho Cordova, California','1953-01-01 00:00:00','2013-06-01 00:00:00','Aerojet Rocketdyne Solid Propulsion Co., AF Plant 70' from dual union all
select 'AKAC','Alaska Aerospace Corporation','business','','US','Anchorage, Alaska','2009-10-01 00:00:00','','Alaska Aerospace Corporation' from dual union all
select 'AKATT','AT&T Alascom','business','','US','Anchorage, Alaska','1995-01-01 00:00:00','','AT&T Alascom' from dual union all
select 'AKCM','Alascom, Inc. (Pacific Telecom)','business','','US','Anchorage, Alaska','1979-01-01 00:00:00','1995-01-01 00:00:00','Alascom, Inc. (Pacific Telecom)' from dual union all
select 'AKCMR','RCA Alaska Communications Inc. (Alascom)','business','','US','Anchorage, Alaska','1970-01-01 00:00:00','1979-01-01 00:00:00','RCA Alaska Communications Inc. (Alascom)' from dual union all
select 'ALAER','Allied Aerospace','business','','US','Newport News, Virginia            a','','2006-11-01 00:00:00','Allied Aerospace' from dual;
insert into ORGANIZATION
select 'ALB','U. Alberta','academic/non-profit','','CA','Edmonton, Alberta','1908-01-01 00:00:00','','U. Alberta' from dual union all
select 'ALC','Alcatel Space, Centre spatiale de CannesMandelieu','business','','F','Cannes','1998-07-01 00:00:00','2005-06-01 00:00:00','Alcatel Space, Centre spatiale de Cannes-Mandelieu' from dual union all
select 'ALCAF','Alcatel Alenia Space (F)','business','','F','Cannes','2005-06-01 00:00:00','2007-04-10 00:00:00','Alcatel Alenia Space (F)' from dual union all
select 'ALCR','Alcatel Alenia Space (I) Roma','business','','I','Roma','2005-06-01 00:00:00','2007-04-10 00:00:00','Alcatel Alenia Space (I) Roma' from dual union all
select 'ALCS','USAF Airborne Launch Control System','defense','','US','Vandenberg AFB, California','1967-01-01 00:00:00','','USAF Airborne Launch Control System' from dual union all
select 'ALCT','Alcatel Alenia Space (I)','business','','I','Torino','2005-06-01 00:00:00','2007-04-10 00:00:00','Alcatel Alenia Space (I)' from dual union all
select 'ALEN','Alenia Spazio','business','','I','Torino','1990-01-01 00:00:00','2005-06-01 00:00:00','Alenia Spazio' from dual union all
select 'ALENR','Alenia Spazio (Roma)','business','','I','Roma','1990-01-01 00:00:00','2005-06-01 00:00:00','Alenia Spazio (Roma)' from dual union all
select 'ALMAN','AO Kontsern VKO AlmazAntey','business','','RU','Moskva','2002-01-01 00:00:00','','Концерн ПВО «Алмаз – Антей»' from dual union all
select 'ALPSP','Alpha Space','business','','US','Houston, Texas','2015-01-01 00:00:00','','Alpha Space' from dual union all
select 'ALYAH','Al Yah Satellite Communications Co. PrJsc','business','','UAE','Abu Dhabi','2009-01-01 00:00:00','','Al Yah Satellite Communications Co. PrJsc' from dual union all
select 'AM','AMSATBrazil','academic/non-profit','','US','Washington DC','','','Radio Amateur Satellite Corporation' from dual union all
select 'AMAI','Adcole Maryland Aerospace Inc','business','','US','Crofton, Maryland','2017-01-01 00:00:00','','Adcole Maryland Aerospace Inc.' from dual union all
select 'AMBR','AMSATBrasil','academic/non-profit','AMSAT','BR','Rio de Janeiro, Argentina','','','AMSAT-Brasil' from dual union all
select 'AMC','RCA Americom','business','','US','Princeton, New Jersey','1976-03-01 00:00:00','1986-01-01 00:00:00','RCA Americom' from dual union all
select 'AMCEL','Amcel Propulsion','business','','US','Swannanoa, North Carolina','1959-01-01 00:00:00','1965-01-01 00:00:00','Amcel Propulsion' from dual union all
select 'AMCG','GE Americom','business','','US','Princeton, New Jersey','1986-01-01 00:00:00','2001-11-21 00:00:00','GE Americom' from dual union all
select 'AMCOM','US Army Aviation and Missile Command','defense','','US','Huntsville, Alabama','1997-10-01 00:00:00','','US Army Aviation and Missile Command' from dual union all
select 'AMCR','RCA Global Communications Inc.','business','','US','Piscataway, New Jersey','1975-01-01 00:00:00','1976-03-01 00:00:00','RCA Global Communications Inc.' from dual union all
select 'AMCS','SES Americom','business','SES','US','Princeton, New Jersey','2001-11-21 00:00:00','2009-09-01 00:00:00','SES Americom' from dual union all
select 'AMDL','AMSATGermany','academic/non-profit','AMSAT','D','Marburg','','','AMSAT-Deutschland' from dual union all
select 'AMF','AMSATFrance','academic/non-profit','AMSAT','F','Paris','','','AMSAT-France' from dual union all
select 'AMI','Italian Air Force','defense','I','I','Roma?','','','Aeronautica Militare Italiana' from dual union all
select 'AMIL','AMSATIsrael','academic/non-profit','AMSAT','IL','Israel','','','AMSAT-Israel' from dual union all
select 'AMIT','AMSATItalia','academic/non-profit','AMSAT','I','Roma','','','AMSAT-Italia' from dual union all
select 'AMLU','AMSATArgentina','academic/non-profit','AMSAT','AR','Buenos Aires?','1987-06-01 00:00:00','','AMSAT-Argentina' from dual union all
select 'AMNA','AMSAT North America  (Radio Amateur Satellite Corp)','academic/non-profit','','US','Washington, D.C.','1969-03-03 00:00:00','','AMSAT North America  (Radio Amateur Satellite Corp)' from dual union all
select 'AMNL','AMSATNL','academic/non-profit','AMSAT','NL','Delft?','','','AMSAT-NL' from dual union all
select 'AMPCS','AMPAC InSpace Propulsion','defense','','US','Niagara Falls, New York','2004-10-01 00:00:00','2012-08-01 00:00:00','AMPAC In-Space Propulsion' from dual union all
select 'AMPCW','AMPAC InSpace Propulsion (UK)','defense','','UK','Westcott, Buckinghamshire','2004-10-01 00:00:00','2012-08-01 00:00:00','AMPAC In-Space Propulsion (UK)' from dual union all
select 'AMROC','American Rocket Company','business','','US','Camarillo, California','1985-01-01 00:00:00','1996-01-01 00:00:00','American Rocket Company' from dual union all
select 'AMRU','AMSATRussia','academic/non-profit','','RU','Moskva','','','AMSAT-Russia' from dual union all
select 'AMSAT','AMSAT','academic/non-profit','','US','Washington, D.C.','1969-03-03 00:00:00','','AMSAT' from dual union all
select 'AMSC','American Mobile Satellite Corp.','business','','US','Reston, Virginia','1988-05-01 00:00:00','2000-06-01 00:00:00','American Mobile Satellite Corp.' from dual union all
select 'AMUK','AMSATUK','academic/non-profit','','UK','Milton Keynes?','1975-01-01 00:00:00','','AMSAT-UK' from dual union all
select 'AMVU','AMSAT India (AMSATVU)','academic/non-profit','','IN','Bangalore','2001-10-01 00:00:00','','AMSAT India (AMSAT-VU)' from dual union all
select 'AN','Soviet Academy of Sciences','civilian','','SU','Moskva, Rossiya','','1991-01-01 00:00:00','Академия наук СССР' from dual union all
select 'ANBEL','Belarus Academy of Sciences','civilian','','BY','Minsk, Belarus','1929-01-01 00:00:00','','Natsionalnaya Akademia Navuk Belarusi' from dual union all
select 'ANCZ','Czech Academy of Sciences','civilian','','CZ','Praha','1992-01-01 00:00:00','','Akademie věd České republiky' from dual union all
select 'ANDRW','Andrews Space Inc.','business','','US','Tukwila, Washington','1999-01-01 00:00:00','2015-01-01 00:00:00','Andrews Space Inc.' from dual union all
select 'ANNAU','Anna University, Chennai','academic/non-profit','','IN','Chennai','1978-01-01 00:00:00','','Anna University, Chennai' from dual union all
select 'ANRF','Russian Academy of Sciences','civilian','','RU','Moskva','1991-01-01 00:00:00','','Российская академия наук' from dual union all
select 'ANSAL','Ansar Allah (Houthi) Revolutionary Committee Forces','defense','','YE','Sa''dah, Yemen','2004-01-01 00:00:00','','أنصار الله' from dual union all
select 'ANSP','Analytical Space, Inc.','business','','US','Cambridge, Massachusetts','2016-01-01 00:00:00','','Analytical Space, Inc.' from dual union all
select 'ANTEL','National Administration of Telecommunications','civilian','','UY','Montevideo','1974-01-01 00:00:00','','Administración Nacional de Telecomunicaciones' from dual union all
select 'ANTN','Neth. Antilles','','NL','ANTN','Nederlandse Antillen','1954-01-01 00:00:00','2010-10-01 00:00:00','Nederlandse Antillen' from dual union all
select 'ANTX','Antrix, India Dept. of Space','civilian','','IN','Bangalore','1992-09-01 00:00:00','','Antrix, India Dept. of Space' from dual union all
select 'ANUC','All Nations University College','academic/non-profit','','GH','Koforidua, Eastern Region','2002-01-01 00:00:00','','All Nations University College' from dual union all
select 'AO','Angola','','','AO','Luanda','1975-01-01 00:00:00','','República de Angola' from dual union all
select 'AOFTB','US Army Ordnance, Missile Group?, Ft Bliss','defense','USA','US','El Paso:Ft Bliss, Texas','1945-01-01 00:00:00','1950-10-01 00:00:00','US Army Ordnance, Missile Group?, Ft Bliss' from dual union all
select 'AOGMC','US Army Ordnance Guided Missile Center','defense','USA','US','Huntsville, Alabama','1950-04-01 00:00:00','1954-01-01 00:00:00','US Army Ordnance Guided Missile Center' from dual union all
select 'AOMC','Army Ordnance Missile Command','defense','USA','US','Huntsville, Alabama','1961-12-01 00:00:00','1963-05-01 00:00:00','Army Ordnance Missile Command' from dual union all
select 'APAERO','Asia Pacific (AP) Aerospace Inc.','business','','KR','Seoul','2012-02-01 00:00:00','','Asia Pacific (AP) Aerospace Inc.' from dual union all
select 'APAR','Aprize Argentina','academic/non-profit','','AR','Buenos Aires?','2001-01-01 00:00:00','','Aprize Argentina' from dual union all
select 'APGC','US Air Force Air Proving Ground Center, Eglin AFB','defense','USAF','US','Eglin AFB, Florida','1957-12-01 00:00:00','1968-08-01 00:00:00','US Air Force Air Proving Ground Center, Eglin AFB' from dual union all
select 'APL','JHUApplied Physics Lab','business','','US','Laurel, Maryland','1953-01-01 00:00:00','','JHU-Applied Physics Lab' from dual union all
select 'APLS','JHUApplied Physics Lab (Silver Spring)','business','','US','DC:Silver Spring, Maryland','1942-01-01 00:00:00','1977-01-01 00:00:00','JHU-Applied Physics Lab (Silver Spring)' from dual union all
select 'APMT','APMT','civilian','CALT','CN','Beijing','1998-03-01 00:00:00','','中国亚太移动通信卫星有限责任公司' from dual union all
select 'APPL','Appleton Laboratory','civilian','SERC','UK','Ditton Park, Buckinghamshire','1974-01-01 00:00:00','1979-01-01 00:00:00','Appleton Laboratory' from dual union all
select 'APRZ','Aprize Satellite Inc.','academic/non-profit','SPQ','US','Fairfax, Virginia','2001-05-01 00:00:00','','Aprize Satellite Inc.' from dual union all
select 'APT','Asia Pacific Telecom','civilian','','HK','Hong Kong','1992-01-01 00:00:00','','Asia Pacific Telecom' from dual union all
select 'APW','Working Group for Space Physics Research','academic/non-profit','','D','Freiburg, Baden:Wurttemberg','1964-01-01 00:00:00','1973-01-01 00:00:00','Arbeitsgruppe für Physikalische Weltraumforschung, Freiburg' from dual union all
select 'AQ','Antarctica','','','AQ',':','','','Antartica' from dual union all
select 'AR','Argentina','','','AR','Argentina','1816-07-09 00:00:00','','República Argentina' from dual union all
select 'ARAB','Arabsat, Arab League','business','','SA','Riyadh, Saudi Arabia','1976-01-01 00:00:00','','عرب سات' from dual union all
select 'ARC','NASA Ames Research Center, Moffett Field','civilian','NASA','US','San Francisco, California','1939-12-01 00:00:00','','NASA Ames Research Center, Moffett Field' from dual union all
select 'ARDC','USAF Air Research and Development Command (AFCRL division)','defense','USAF','US','Cambridge, Mass.','1950-01-01 00:00:00','1961-04-01 00:00:00','USAF Air Research and Development Command (AFCRL division)' from dual union all
select 'ARGMA','Army Rocket and Guided Missile Agency','defense','AOMC','US','Huntsville, Alabama','1958-03-31 00:00:00','1961-12-01 00:00:00','Army Rocket and Guided Missile Agency' from dual union all
select 'ARH','Aerojet Rocketdyne Holdings','business','','US','Rancho Cordova, California','2015-04-27 00:00:00','','Aerojet Rocketdyne Holdings' from dual union all
select 'ARIG','ArianeGroup','business','ADS','F','Paris/Les Mureaux','2017-07-01 00:00:00','','ArianeGroup' from dual union all
select 'ARMAD','Armadillo Aerospace','business','','US','Dallas:Mesquite, Texas','2000-01-01 00:00:00','2015-01-01 00:00:00','Armadillo Aerospace' from dual union all
select 'ARMF','French Armed Forces','defense','F','F','Paris','1445-01-01 00:00:00','','Forces armées françaises' from dual union all
select 'ARMT','Academy of Rocket Motors Technology (CASC 4th Academy)','civilian','CASC','CN','Xi''an, Shaanxi','1962-07-01 00:00:00','','航天动力技术研究院' from dual union all
select 'ARPA','Advanced Research Projects Agency, U.S. DoD','defense','DOD','US','Washington, DC','1958-02-07 00:00:00','1972-03-23 00:00:00','Advanced Research Projects Agency, U.S. DoD' from dual union all
select 'ARPA2','Advanced Research Projects Agency, U.S. DoD','defense','DOD','US','Washington, DC','1993-02-22 00:00:00','1996-02-10 00:00:00','Advanced Research Projects Agency, U.S. DoD' from dual union all
select 'ARR','Andoya Research Range','civilian','','N','Andenes, Norway','1962-01-01 00:00:00','2014-04-01 00:00:00','Andøya Rakettskytefelt' from dual union all
select 'ARRS','Andoya Space Center','civilian','','N','Andenes, Norway','2014-04-01 00:00:00','','Andøya Space Center' from dual union all
select 'ARSAT','ARSAT','business','','AR','Buenos Aires','2006-01-01 00:00:00','','Empresa Argentina de Soluciones Satelitales S.A.' from dual union all
select 'ARSL','Arsenal Design Bureau','business','','SU','Sankt:Peterburg (Leningrad)','1965-01-01 00:00:00','2014-03-01 00:00:00','ФГУП «Конструкторское бюро „Арсенал“ имени М. В. Фрунзе»' from dual union all
select 'ARSLO','Arsenal Design Bureau','business','ORKK','SU','Sankt:Peterburg (Leningrad)','2014-03-01 00:00:00','','ФГУП «Конструкторское бюро „Арсенал“ имени М. В. Фрунзе»' from dual union all
select 'ARTEC','US Army Topographic Engineering Center','defense','USA','US','Fort Belvoir, Virginia','1991-05-06 00:00:00','','US Army Topographic Engineering Center' from dual union all
select 'ARTOPL','US Army Engineer Topographic Laboratories  (TOPOCOM, later ACE)','defense','USA','US','Fort Belvoir, Virginia','1967-07-31 00:00:00','1991-05-06 00:00:00','US Army Engineer Topographic Laboratories  (TOPOCOM, later ACE)' from dual union all
select 'ARTOPO','US Army Topographic Command','defense','USA','US','Bethesda, Maryland?','1968-09-01 00:00:00','1972-07-01 00:00:00','US Army Topographic Command' from dual union all
select 'ARV','Arg. Antarctic','','AR','ARV','Ushuaia, Tierra del Fuego','1900-01-01 00:00:00','','Provincia de Tierra del Fuego, Antártida e Islas del Atlántico Sur' from dual union all
select 'ASAL','Algerian Space Agency','civilian','','DZ','Dzayer (Algiers)','2002-01-01 00:00:00','','Agence Spatiale Algérienne' from dual union all
select 'ASBC','Advanced Space Business Corporation','business','','J','Tokyo:Hiyoda','2002-11-01 00:00:00','2007-01-01 00:00:00','新衛星ビジネス株式会社' from dual union all
select 'ASCO','American Satellite Corp','business','','US','Germantown, Maryland','1982-01-01 00:00:00','1985-01-01 00:00:00','American Satellite Corp' from dual union all
select 'ASE','American Science and Engineering','academic/non-profit','','US','Cambridge, MA','1958-01-01 00:00:00','','American Science and Engineering' from dual union all
select 'ASFL','Airbus Safran Launchers','business','ADS','F','Paris/Les Mureaux','2015-01-01 00:00:00','2017-07-01 00:00:00','Airbus Safran Launchers' from dual union all
select 'ASGARD','Asgardia Space','academic/non-profit','','AT','Vienna','2016-01-01 00:00:00','','Asgardia Space' from dual union all
select 'ASI','Italian Space Agency','civilian','I','I','Roma','1988-01-01 00:00:00','','Agenzia Spaziale Italiana' from dual union all
select 'ASIA','Asia Satellite Telecom, Hong Kong','business','','HK','Hong Kong','1988-01-01 00:00:00','','亞洲衛星有限公司' from dual union all
select 'ASID','Armstrong Siddeley','business','','UK','Coventry','1927-01-01 00:00:00','1960-01-01 00:00:00','Armstrong Siddeley' from dual union all
select 'ASL','US Army Atmospheric Sciences Laboratory','defense','WSMR','US','White Sands, New Mexico','1958-01-01 00:00:00','','US Army Atmospheric Sciences Laboratory' from dual union all
select 'ASTA','Spar Astro Aerospace, Carpenteria','business','','US','Carpenteria, California','1972-01-01 00:00:00','1999-01-01 00:00:00','Spar Astro Aerospace, Carpenteria' from dual union all
select 'ASTD','Astrium (D), Bremen','business','','D','Bremen','2000-07-10 00:00:00','2003-06-16 00:00:00','Astrium (D), Bremen' from dual union all
select 'ASTDF','Astrium (D), Friedrichshafen','business','','D','Friedrichsafen','2000-07-10 00:00:00','2003-06-16 00:00:00','Astrium (D), Friedrichshafen' from dual union all
select 'ASTDYN','Astrodyne Inc.   (Phillips/NAA)','business','','US','McGregor, Texas','1958-01-01 00:00:00','1959-09-30 00:00:00','Astrodyne Inc.   (Phillips/NAA)' from dual union all
select 'ASTFN','AstroundFeinwerktechnik Adlershof GmbH (Astrofein Technik)','business','','D','Berlin','2006-01-01 00:00:00','','Astro-und-Feinwerktechnik Adlershof GmbH (Astrofein Technik)' from dual union all
select 'ASTL','Astrium (D), Lampoldshausen','business','','D','Lampoldshausen','1996-01-01 00:00:00','2014-01-01 00:00:00','Astrium (D), Lampoldshausen' from dual;
insert into ORGANIZATION
select 'ASTM','EADS Astrium Satellites/Barajas  (CASA Espacio)','business','EADS','E','Madrid:Barajas','2006-07-01 00:00:00','2014-01-01 00:00:00','EADS Astrium Satellites/Barajas  (CASA Espacio)' from dual union all
select 'ASTNA','Astrium North America','business','ADS','US','Houston:Webster, Texas','2000-01-01 00:00:00','2014-01-01 00:00:00','Astrium North America' from dual union all
select 'ASTO','Astrium Space Propulsion/Ottobrunn','business','','D','Munchen:Ottobrunn','2006-07-01 00:00:00','2014-01-01 00:00:00','Astrium Space Propulsion/Ottobrunn' from dual union all
select 'ASTRA','Atmospheric and Space Technology Research Associates LLC','business','','US','Boulder, Colorado','2016-01-01 00:00:00','','Atmospheric and Space Technology Research Associates LLC' from dual union all
select 'ASTRAN','Astranis Space Technologies Corp.','business','','US','San Francisco, California','2015-01-01 00:00:00','','Astranis Space Technologies Corp.' from dual union all
select 'ASTRV','Astra Space','business','','US','Alameda, California','2016-01-01 00:00:00','','Astra Space' from dual union all
select 'ASTSC','Astroscale Japan Inc.','business','','J','Tokyo','2015-01-01 00:00:00','','Astroscale Japan Inc.' from dual union all
select 'ASTT','Astrium (F) Toulouse','business','','F','Toulouse','2000-01-01 00:00:00','2014-01-01 00:00:00','Astrium (F) Toulouse' from dual union all
select 'ASTUK','Astrium Ltd (UK)','business','EADS','UK','Stevenage','2000-01-01 00:00:00','2014-01-01 00:00:00','Astrium Ltd (UK)' from dual union all
select 'ASU','Arizona State University','academic/non-profit','','US','Phoenix:Tempe, Arizona','1885-01-01 00:00:00','','Arizona State University' from dual union all
select 'AT','Austria','','','AT','Wien, Osterreich','1955-07-27 00:00:00','','Republik Österreich' from dual union all
select 'ATECH','Astrotech Space Operations','business','SHAB','US','Titusville, Florida','1985-01-01 00:00:00','2014-01-01 00:00:00','Astrotech Space Operations' from dual union all
select 'ATECHS','Astrotech Space Operations (Suborbital)','business','SHAB','US','Washington, DC:Greenbelt','1985-01-01 00:00:00','2000-01-01 00:00:00','Astrotech Space Operations (Suborbital)' from dual union all
select 'ATECHT','Astrotech Corp.','business','','US','Austin, Texas','1984-01-01 00:00:00','2009-01-01 00:00:00','Astrotech Corp.' from dual union all
select 'ATEF','Technical Association for Rocket Studies','business','','F','Paris?','1954-01-01 00:00:00','1960-01-01 00:00:00','Association Technique pour l''Etude des Fusées' from dual union all
select 'ATHERM','Aerotherm Corp.','business','','US','Alberquerque, New Mexico','','','Aerotherm Corp.' from dual union all
select 'ATI','Advanced Technology, Inc. (ATI)','business','','US','Palo Alto, California','1959-01-01 00:00:00','','Advanced Technology, Inc. (ATI)' from dual union all
select 'ATK','Alliant Techsystems','business','','US','Minneapolis:Eden Prairie, Minnesota','1990-01-01 00:00:00','','Alliant Techsystems' from dual union all
select 'ATKAB','ATK Able Engineering','business','ATK','US','Goleta, California','2004-01-01 00:00:00','2015-02-10 00:00:00','ATK Able Engineering' from dual union all
select 'ATKABL','ATK Tactical Systems, Alleghany Ballistic Laboratories (USN)','defense','ATK','US','Cumberland:Rocket Center, West Virginia','1995-01-01 00:00:00','2015-02-10 00:00:00','ATK Tactical Systems, Alleghany Ballistic Laboratories (USN)' from dual union all
select 'ATKB','ATK Defense Group / Missile Products','business','ATK','US','Baltimore, Maryland','2014-01-01 00:00:00','2015-02-10 00:00:00','ATK Defense Group / Missile Products' from dual union all
select 'ATKE','Alliant Thiokol','business','ATK','US','Elkton, Maryland','2001-01-01 00:00:00','2015-02-10 00:00:00','Alliant Thiokol' from dual union all
select 'ATKG','ATK GASL','business','ATK','US','Ronkonkoma, New York','2003-01-01 00:00:00','2015-02-10 00:00:00','ATK GASL' from dual union all
select 'ATKH','Alliant (Hercules)','business','ATK','US','Salt Lake City:Magna/Bacchus, Utah','1995-03-01 00:00:00','2015-02-10 00:00:00','Alliant (Hercules)' from dual union all
select 'ATKL','ATK Launch Systems','business','ATK','US','Brigham City:Promonotory, Utah','1995-01-01 00:00:00','2015-02-01 00:00:00','ATK Launch Systems' from dual union all
select 'ATKP','Alliant Thiokol (Promontory)','business','ATK','US','Brigham City:Promontory, Utah','2001-01-01 00:00:00','2015-02-10 00:00:00','Alliant Thiokol (Promontory)' from dual union all
select 'ATKS','ATK Space Division','business','ATK','US','Beltsville, Maryland','2007-06-01 00:00:00','2015-02-10 00:00:00','ATK Space Division' from dual union all
select 'ATKTN','ATK Propulsion and Controls Div.','business','ATK','US','Tullahoma, Tennessee','2003-11-01 00:00:00','2011-03-01 00:00:00','ATK Propulsion and Controls Div.' from dual union all
select 'ATLA','Atlantic Research Corp. (Alexandria)','business','','US','Alexandria, Virginia','1951-01-01 00:00:00','','Atlantic Research Corp. (Alexandria)' from dual union all
select 'ATLC','Atlantic Research Corp. Missile Systems Div.','business','','US','LA:Costa Mesa, California','1967-01-01 00:00:00','','Atlantic Research Corp. Missile Systems Div.' from dual union all
select 'ATLCA','Atlantic Research Corp. (Camden)','engine/motor manufacturer','','US','East Camden, Arkansas','','2003-01-01 00:00:00','Atlantic Research Corp. (Camden)' from dual union all
select 'ATLL','Atlantic Research Corp. (Liquid Propulsion Div).','civilian','','US','Niagara Falls, New York','1998-01-01 00:00:00','2004-01-01 00:00:00','Atlantic Research Corp. (Liquid Propulsion Div).' from dual union all
select 'ATLM','Atlantic Research Corp. Missile Systems (Aerolab)','business','','US','Pasadena:Monrovia, California','1967-01-01 00:00:00','1968-01-01 00:00:00','Atlantic Research Corp. Missile Systems (Aerolab)' from dual union all
select 'ATLR','Atlantic Research Corp. (Solid Propulsion Div).','civilian','','US','Gainesville, Virginia','1951-01-01 00:00:00','2005-01-01 00:00:00','Atlantic Research Corp. (Solid Propulsion Div).' from dual union all
select 'ATLU','Atlantic Research Corp. UK','defense','','UK','Westcott, Buckinghamshire','1998-01-01 00:00:00','2004-01-01 00:00:00','Atlantic Research Corp. UK' from dual union all
select 'ATSB','Astronautic Technology (M) Sdn Bhd','business','','MY','Shah Alam, Selangor, Malaysia','','','Astronautic Technology (M) Sdn Bhd' from dual union all
select 'ATT','American Telephone and Telegraph Co.','business','','US','Bedminster, New Jersey','1962-01-01 00:00:00','1996-01-01 00:00:00','American Telephone and Telegraph Co.' from dual union all
select 'ATTS','AT&T Skynet Satellite Services','business','ATT','US','Bedminster, New Jersey','1962-01-01 00:00:00','1996-01-01 00:00:00','AT&T Skynet Satellite Services' from dual union all
select 'AU','Australia','','','AU','Australia','1901-01-01 00:00:00','','Commonwealth of Australia' from dual union all
select 'AUBU','Auburn University','academic/non-profit','','US','Auburn, Alabama','','','Auburn University' from dual union all
select 'AUCZ','Astronomical Institute of the Czech Academy of Sciences','civilian','ANCZ','CZ','Ondrejov','1992-01-01 00:00:00','','Astronomický ústav Akademie věd České republiky' from dual union all
select 'AUDACY','Audacy','business','','US','San Francisco:Mountain View, California','2015-01-01 00:00:00','','Audacy' from dual union all
select 'AUDISR','Space Policy Unit, Dept of Innovation, Industry, Science and Research','civilian','AU','AU','Canberra, ACT','2009-01-01 00:00:00','','Space Policy Unit, Dept of Innovation, Industry, Science and Research' from dual union all
select 'AUSP','Auspace Pty','business','','AU','Canberra:Fyshwick','1983-01-01 00:00:00','2007-01-01 00:00:00','Auspace Pty' from dual union all
select 'AUSS','Aussat Pty, Australia','business','','AU','Sydney','1982-01-01 00:00:00','1992-01-01 00:00:00','Aussat Pty, Australia' from dual union all
select 'AVANTI','Avanti Communications Group plc','business','','UK','London','2007-01-01 00:00:00','','Avanti Communications Group plc' from dual union all
select 'AVCO','Avco Corporation','business','','US','Wilmington, Mass.','1956-01-01 00:00:00','1986-01-01 00:00:00','Avco Corporation' from dual union all
select 'AVIB','Avibras Industria Aeroespacial','business','','BR','Sao Jose dos Campos','1961-01-01 00:00:00','','Avibras Indústria Aeroespacial' from dual union all
select 'AVIO','Avio s.p.A.','business','','I','Colleferro','2003-01-01 00:00:00','','Avio s.p.A.' from dual union all
select 'AVKO','Altenwalde Versuchskommando','defense','','DR','Cuxhaven:Altenwalde, Germany','1945-01-01 00:00:00','1945-01-01 00:00:00','Altenwalde Versuchskommando' from dual union all
select 'AWS','USAF Air Weather Service, Scott AFB','defense','USAF','US','St Louis:Belleville, Missouri','1947-01-01 00:00:00','1991-01-01 00:00:00','USAF Air Weather Service, Scott AFB' from dual union all
select 'AXEL','AXELSPACE Corporation','business','','J','Tokyo:Chiyoda','2008-01-01 00:00:00','','アクセルスペース' from dual union all
select 'AZ','Azerbaijan','','','AZ','Baku','1991-10-01 00:00:00','','Azərbaycan Respublikası' from dual union all
select 'AZER','Azerkosmos','civilian','AZRITN','AZ','Baku','1991-01-01 00:00:00','','Azərkosmos' from dual union all
select 'AZRITN','Azeri Min. Comms','civilian','','AZ','Baku','1991-01-01 00:00:00','','Azərbaycan Respublikası Rabitə və Yüksək Texnologiyalar Nazirliyi' from dual union all
select 'B','Belgium','','','B','Belgique','1830-10-04 00:00:00','','Koninkrijk België, Royaume de Belgique' from dual union all
select 'BABTS','Babakin Science and Research Space Center','business','NPOL','RU','Moskva:Khimki','1965-01-01 00:00:00','','научно-испытательный центр имени Г.Н. Бабакина' from dual union all
select 'BAC','British Aircraft Corp.','business','','UK','Bristol:Filton, Avon','1910-01-01 00:00:00','1977-04-01 00:00:00','British Aircraft Corp.' from dual union all
select 'BACB','Bristol Aeroplane Co., Banwell Factory','business','','UK','Bristol:Banwell, Avon','1956-01-01 00:00:00','1959-01-01 00:00:00','Bristol Aeroplane Co., Banwell Factory' from dual union all
select 'BAEF','British Aerospace (BAe) Systems','business','','UK','Bristol:Filton, Avon','1977-04-01 00:00:00','','British Aerospace (BAe) Systems' from dual union all
select 'BAENH','BAe Electronic Systems, Nashua','business','','US','Nashua, New Hampshire','2000-01-01 00:00:00','','BAe Electronic Systems, Nashua' from dual union all
select 'BAES','British Aerospace','business','','UK','Stevenage, Hertfordshire','1977-04-01 00:00:00','1994-01-01 00:00:00','British Aerospace' from dual union all
select 'BAJ','Canadian Bristol Aerojet Ltd','business','','CA','Winnipeg, Manitoba','1962-01-01 00:00:00','1967-01-01 00:00:00','Canadian Bristol Aerojet Ltd' from dual union all
select 'BAJUK','Bristol Aerojet UK, Banwell Factory','business','','UK','Bristol:Banwell, Avon','1959-01-01 00:00:00','1991-01-01 00:00:00','Bristol Aerojet UK, Banwell Factory' from dual union all
select 'BAL','Bristol Aerospace','business','','CA','Winnipeg, Manitoba','1967-01-01 00:00:00','1997-01-01 00:00:00','Bristol Aerospace' from dual union all
select 'BALL','Ball Aerospace','business','','US','Boulder, Colorado','1956-01-01 00:00:00','','Ball Aerospace' from dual union all
select 'BALM','Bristol Aerospace Ltd., Magellan Aerospace Corporation','business','','CA','Winnipeg, Manitoba','1997-01-01 00:00:00','','Bristol Aerospace Ltd., Magellan Aerospace Corporation' from dual union all
select 'BAS','British Antarctic Survey','civilian','','UK','Cambridge, UK','1962-01-01 00:00:00','','British Antarctic Survey' from dual union all
select 'BAT','BAT','','UK','BAT','Rothera','1908-01-01 00:00:00','','British Antarctic Territory' from dual union all
select 'BAUM','Baumann Moscow State Technical University','academic/non-profit','','RU','Moskva','1917-01-01 00:00:00','','Московский государственный технический университет им. Н. Э. Баумана' from dual union all
select 'BAYI','Beijing Bayi School','academic/non-profit','','CN','Beijing','','','北京市八一学校' from dual union all
select 'BB','Barbados','','','BB','Barbados','1966-01-01 00:00:00','','Barbados' from dual union all
select 'BBRI','BlackBridge','business','','D','Brandenburg an der Havel, Brandenburg','2013-11-06 00:00:00','2015-08-01 00:00:00','BlackBridge' from dual union all
select 'BBUK','Barbados','','UK','BBUK','Barbados','1628-01-01 00:00:00','1966-01-01 00:00:00','Barbados, British self governing colony' from dual union all
select 'BCSCL','Bangladesh Communications Satellite Company Ltd','business','BTRC','BD','Dhaka','2017-01-01 00:00:00','','Bangladesh Communications Satellite Company Ltd' from dual union all
select 'BD','Bangladesh','','','BD','Dhaka','1971-01-01 00:00:00','','গণপ্রজাতন্ত্রী বাংলাদেশ' from dual union all
select 'BELINT','Belintersat','business','ZTZM','BY','Minsk, Belarus','2014-01-01 00:00:00','','Belintersat' from dual union all
select 'BELL','Bell Aircraft (Air Force Plant 38)','business','','US','Niagara Falls, New York','1935-01-01 00:00:00','1960-01-01 00:00:00','Bell Aircraft (Air Force Plant 38)' from dual union all
select 'BELT','Bell Textron (Bell Aerospace)','business','','US','Niagara Falls, New York','1960-01-01 00:00:00','1994-01-01 00:00:00','Bell Textron (Bell Aerospace)' from dual union all
select 'BENDX','Bendix Systems Div.','business','','US','Ann Arbor, Michigan','','','Bendix Systems Div.' from dual union all
select 'BERG','University of Bergen','academic/non-profit','','N','Bergen','1946-01-01 00:00:00','','Universitetet i Bergen (UiB)' from dual union all
select 'BERN','Bern University','academic/non-profit','','CH','Berne, Berne','1834-01-01 00:00:00','','Universität Bern' from dual union all
select 'BG','Bulgaria','','','BG','Balgariya','1944-01-01 00:00:00','1989-01-01 00:00:00','Република България' from dual union all
select 'BGN','PR Bulgaria','','','BG','Balgariya','1989-01-01 00:00:00','','Народна република България' from dual union all
select 'BGSAT','Bulgaria Sat AD','business','BULSAT','BGN','Stara Zagora','2004-01-01 00:00:00','','Bulgaria Sat AD' from dual union all
select 'BGU','BenGurion University of the Negev','academic/non-profit','','IL','Beersheba','1969-01-01 00:00:00','','אוניברסיטת בן-גוריון בנגב' from dual union all
select 'BHSJ','SpaceView','business','','CN','Beijing','2015-01-01 00:00:00','','北京航天世景信息技术有限公司' from dual union all
select 'BIGEL','Bigelow Aerospace','business','','US','N Las Vegas, Nevada','1999-01-01 00:00:00','','Bigelow Aerospace' from dual union all
select 'BINA','Binariang Satellite Systems Sdn. Bhd.','business','','MY','Kuala Lumpur','1993-01-01 00:00:00','2001-01-01 00:00:00','Binariang Satellite Systems Sdn. Bhd.' from dual union all
select 'BIR','Birmingham University','academic/non-profit','','UK','Birmingham','','','Birmingham University' from dual union all
select 'BISME','Beijing Inst. of Space Machinery and Electricity','civilian','CAST','CN','Beijing','1958-08-21 00:00:00','','北京空间机电研究所' from dual union all
select 'BKSA','Space Science Division','civilian','','MY','Kuala Lumpur?','1992-01-01 00:00:00','','Bahagian Kajian Sains Angkasa (BAKSA)' from dual union all
select 'BLCAN','Blue Canyon Technologies','business','','US','Boulder, Colorado','2008-01-01 00:00:00','','Blue Canyon Technologies' from dual union all
select 'BLMIT','Beijing Landview Mapping Information Technology Co.','academic/non-profit','21AT','CN','Beijing:Haidian','2001-06-01 00:00:00','2011-01-01 00:00:00','北京宇视蓝图信息技术有限公司' from dual union all
select 'BLOR','Blue Origin LLC','business','','US','Kent, Washington','2000-01-01 00:00:00','','Blue Origin LLC' from dual union all
select 'BLORTX','Blue Origin LLC, West Texas operations','business','BLOR','US','Van Horn, Texas','2000-01-01 00:00:00','','Blue Origin LLC, West Texas operations' from dual union all
select 'BLS','Boeing Launch Services (?)','business','BOHB','US','Huntington Beach, California','1990-01-01 00:00:00','','Boeing Launch Services (?)' from dual union all
select 'BM','Bermuda','','UK','BM','Hamilton, Bermuda','1684-01-01 00:00:00','','UK Overseas Territory of Bermuda' from dual union all
select 'BMBF','Ministry of Education and Research','civilian','D','D','Bonn?','1994-01-01 00:00:00','','Bundesministerium für Bildung und Forschung' from dual union all
select 'BMBW','Ministry of Education and Science','civilian','D','D','Bonn','1969-09-01 00:00:00','1994-01-01 00:00:00','Bundesministerium für Bildung und Wissenschaft' from dual;
insert into ORGANIZATION
select 'BMDO','Ballistic Missile Defense Organization','defense','','US','Washington,DC:Pentagon','1993-01-01 00:00:00','2002-01-01 00:00:00','Ballistic Missile Defense Organization' from dual union all
select 'BME','BUTE','academic/non-profit','','HU','Budapest','1782-01-01 00:00:00','','Budapesti Műszaki és Gazdaságtudományi Egyetem' from dual union all
select 'BMFT','Ministry of Research and Technology [Merged with BMBW?]','civilian','D','D','Bonn','1972-01-01 00:00:00','1994-01-01 00:00:00','Bundesministerium für Forschung und Technologie' from dual union all
select 'BMVG','Federal Ministry of Defense','defense','D','D','Bonn:Hardthohe','1955-01-01 00:00:00','','Bundesministerium der Verteidigung' from dual union all
select 'BMWF','Federal Ministry for Scienfitic Research','civilian','D','D','Bonn','1962-01-01 00:00:00','1969-09-01 00:00:00','Bundesministerium für wissenschaftliche Forschung' from dual union all
select 'BNAP','Boeing North American/Palmdale','business','BOE','US','Plant 42, Palmdale, California','1996-12-01 00:00:00','','Boeing North American/Palmdale' from dual union all
select 'BNC','British National Committee for Space Research, Royal Society','civilian','','UK','London','1958-01-01 00:00:00','1965-01-01 00:00:00','British National Committee for Space Research, Royal Society' from dual union all
select 'BNSC','British National Space Center','civilian','','UK','Swindon','1985-01-01 00:00:00','2010-04-01 00:00:00','British National Space Center' from dual union all
select 'BO','Bolivia','','','BO','La Paz','1825-01-01 00:00:00','','Estado Plurinacional de Bolivia (Bulivya Mamallaqta)' from dual union all
select 'BO2','Boeing Airplane Co Plant 2 (E Marginal Way)','business','BOE','US','Seattle:Boeing Field, Washington','1936-01-01 00:00:00','','Boeing Airplane Co Plant 2 (E Marginal Way)' from dual union all
select 'BOC','Boeing Airplane Co Plant 1, 27th Ave S','business','BOE','US','Seattle:King, Washington','1916-01-01 00:00:00','1972-01-01 00:00:00','Boeing Airplane Co Plant 1, 27th Ave S' from dual union all
select 'BOC3PF','Boeing Comm. Crew Processing Facility, OFF3','business','BOE','US','Kennedy Space Center, Florida','2011-01-01 00:00:00','','Boeing Comm. Crew Processing Facility, OFF-3' from dual union all
select 'BOCH','Bochum Ruhr University','academic/non-profit','','D','Bochum, NordRhein:Westfalen','1962-01-01 00:00:00','','Ruhr-Universität Bochum' from dual union all
select 'BOCP','Boeing Rocketdyne','business','BOE','US','Canoga Park, California','1996-12-01 00:00:00','2005-08-01 00:00:00','Boeing Rocketdyne' from dual union all
select 'BOE','Boeing Co.','business','','US','Seattle, Washington','1916-01-01 00:00:00','','Boeing Co.' from dual union all
select 'BOEI','Japan Defense Ministry','defense','','J','Tokyo:Ichigaya','2007-01-09 00:00:00','','防衛省' from dual union all
select 'BOES','Boeing Satellite Development Center, El Segundo','business','BOE','US','El Segundo, California','2000-10-01 00:00:00','','Boeing Satellite Development Center, El Segundo' from dual union all
select 'BOHB','Boeing Space Systems (HB)','business','BOE','US','Huntington Beach, California','1996-12-01 00:00:00','','Boeing Space Systems (HB)' from dual union all
select 'BOHPS','Boeing Houston Product Support Center (HPSC)','business','BOE','US','Houston, Texas','2014-01-01 00:00:00','','Boeing Houston Product Support Center (HPSC)' from dual union all
select 'BOHSM','Boeing Huntsville (Cummings)','business','BOE','US','Huntsville, Alabama','1997-01-01 00:00:00','','Boeing Huntsville (Cummings)' from dual union all
select 'BOHSV','Boeing Huntsville (HIA)','business','BOE','US','Huntsville, Alabama','1984-01-01 00:00:00','','Boeing Huntsville (HIA)' from dual union all
select 'BOKT','Boeing Space Center, Kent','business','BOE','US','Kent, Washington','1964-01-01 00:00:00','','Boeing Space Center, Kent' from dual union all
select 'BOL','Alma Mater Studiorum Universita di Bologna','academic/non-profit','','I','Bologna, Emilia:Romagna','1088-01-01 00:00:00','','Università di Bologna' from dual union all
select 'BOLK','B\"olkowEntwicklungen KG','business','','D','Munchen:Ottobrunn','1956-01-01 00:00:00','1968-01-01 00:00:00','Bölkow Entwicklungen KG' from dual union all
select 'BOMI','Boeing, NASA Michoud Assembly Facility','business','BOE','US','Michoud, Louisiana','1961-01-01 00:00:00','1973-01-01 00:00:00','Boeing, NASA Michoud Assembly Facility' from dual union all
select 'BONN','Rhenische FriedrichWilhelmsUniversitat Bonn','academic/non-profit','','D','Bonn, Nordrhein:Westfalen','1818-01-01 00:00:00','','Rheinische Friedrich-Wilhelms-Universität Bonn' from dual union all
select 'BONUM','BONUM1 Co., Media Most','business','','RU','Moskva','1996-01-01 00:00:00','2003-01-01 00:00:00','BONUM-1 Co., Media Most' from dual union all
select 'BOOZA','Booz Allen Hamilton','business','','US','McLean, Virginia','1914-01-01 00:00:00','','Booz Allen Hamilton' from dual union all
select 'BOPBL','Boeing/Pueblo','business','BOE','US','Pueblo, Colorado','1987-04-01 00:00:00','','Boeing/Pueblo' from dual union all
select 'BOPW','Boeing Phantom Works, St Louis','business','BOE','US','St Louis, Missouri','1996-01-01 00:00:00','','Boeing Phantom Works, St Louis' from dual union all
select 'BOSB','Boeing/Seal Beach','business','BOE','US','Seal Beach, California','1996-12-01 00:00:00','','Boeing/Seal Beach' from dual union all
select 'BOSE','Boeing Space Exploration','business','BOE','US','Houston, Texas','1996-01-01 00:00:00','2014-01-01 00:00:00','Boeing Space Exploration' from dual union all
select 'BOSL','Sea Launch Inc.','business','SEALP','US','Long Beach, California','1997-01-01 00:00:00','2010-01-01 00:00:00','Sea Launch Inc.' from dual union all
select 'BOTUL','Boeing/Tulsa, Air Force Plant 3','business','BOE','US','Tulsa, Oklahoma','1962-01-01 00:00:00','2003-01-01 00:00:00','Boeing/Tulsa, Air Force Plant 77' from dual union all
select 'BOUT','Boeing/Ogden, Air Force Plant 77','business','BOE','US','Ogden, Utah','1960-01-01 00:00:00','1978-11-30 00:00:00','Boeing/Ogden, Air Force Plant 77' from dual union all
select 'BPD','BPD Defense and Space','business','','I','Colleferro','1912-10-26 00:00:00','1994-01-01 00:00:00','BPD Difesa e Spazio' from dual union all
select 'BPT','DFVLRBPT Bereich fur Projecttragerschaften','civilian','DFVLR','D','Koln','1975-01-01 00:00:00','','DFVLR Bereich für Projektträgerschaften (BPT)' from dual union all
select 'BR','Brazil','','','BR','Brasil','1822-09-07 00:00:00','','República Federativa do Brasil' from dual union all
select 'BRAC','BRAC University','academic/non-profit','','BD','Dhaka:Mohakhali','2001-01-01 00:00:00','','BRAC University' from dual union all
select 'BRAE','Bristol Aero Engines','business','','UK','Bristol:Filton','1956-01-01 00:00:00','1958-01-01 00:00:00','Bristol Aero Engines' from dual union all
select 'BRCTA','General Command for Aerospace Technology','defense','','BR','Sao Jose dos Campos, Brasil','1953-01-01 00:00:00','','Comando-Geral de Tecnologia Aerospacial' from dual union all
select 'BREC','Bristol Engine Co.','business','','UK','Bristol:Filton','1918-01-01 00:00:00','1956-01-01 00:00:00','Bristol Engine Co.' from dual union all
select 'BREI','Ionosphere Research Insitute','academic/non-profit','','D','Breisach, Baden:Wutternberg','1947-01-01 00:00:00','','Ionosphäreninstitut Breisach' from dual union all
select 'BREM','U. Bremen','academic/non-profit','','D','Bremen, FH Bremen','1971-01-01 00:00:00','','Universität Bremen' from dual union all
select 'BRIC','Bristol Aeroplane Co. (Canada)','business','','CA','Winnipeg, Manitoba','1954-01-01 00:00:00','1962-01-01 00:00:00','Bristol Aeroplane Co. (Canada)' from dual union all
select 'BRID','PT Bank Rakyat Indonesia (Persero) Tbk.','business','','ID','Jakarta','1895-01-01 00:00:00','','PT Bank Rakyat Indonesia (Persero) Tbk.' from dual union all
select 'BRIS','Bristol University','academic/non-profit','','UK','Bristol','1909-01-01 00:00:00','','Bristol University' from dual union all
select 'BRL','US Army Ballistic Research Lab, Aberdeen Proving Ground','defense','USA','US','Aberdeen, Maryland','1941-01-01 00:00:00','1992-01-01 00:00:00','US Army Ballistic Research Lab, Aberdeen Proving Ground' from dual union all
select 'BROWN','Brown University','academic/non-profit','','US','Providence, Rhode Island','1764-01-01 00:00:00','','Brown University' from dual union all
select 'BRSW','Braunschweig Technical University','academic/non-profit','','D','Braunschweig, Niedersachsen','1968-01-01 00:00:00','','Technische Universität Carolo-Wilhelmina zu Braunschweig' from dual union all
select 'BRUNSW','Brunswick Corp.','business','','US','Costa Mesa, California','1980-01-01 00:00:00','','Brunswick Corp.' from dual union all
select 'BRVCC','Brevard Community College','academic/non-profit','','US','Melbourne, Florida','1960-01-01 00:00:00','','Brevard Community College' from dual union all
select 'BS','Bahamas','','','BS','Nassau','1964-01-01 00:00:00','','Bahamas, Commonwealth of the' from dual union all
select 'BSAT','Broadcasting Satellite System Corp.','business','','J','Shibuya-ku, Tokyo','1993-04-13 00:00:00','','株式会社　放送衛星システム' from dual union all
select 'BSB','British Satellite Broadcasting','business','','UK','Wandsworth, London','1986-01-01 00:00:00','1990-12-01 00:00:00','British Satellite Broadcasting' from dual union all
select 'BSC','NASA Beltsville Space Center','civilian','NASA','US','Beltsville (Greenbelt), Maryland','1959-01-15 00:00:00','1959-05-01 00:00:00','NASA Beltsville Space Center' from dual union all
select 'BSEL','Bristol Siddeley (Filton)','business','','UK','Bristol:Filton','1958-01-01 00:00:00','1966-01-01 00:00:00','Bristol Siddeley (Filton)' from dual union all
select 'BSELC','Bristol Siddeley (Coventry)','business','','UK','Coventry:Anstey','1960-01-01 00:00:00','1966-01-01 00:00:00','Bristol Siddeley (Coventry)' from dual union all
select 'BSELL','Bristol Siddeley Engines Ltd (BSEL) Leavesden','business','','UK','Watford:Leavesden, Hertfordshire','1959-01-01 00:00:00','1966-01-01 00:00:00','Bristol Siddeley Engines Ltd (BSEL) Leavesden' from dual union all
select 'BSKB','British Sky Broadcasting','business','','UK','Isleworth, London','1990-12-01 00:00:00','','British Sky Broadcasting' from dual union all
select 'BSKG','BlackSky Global','business','SFLIN','US','Tukwila, Washington','2015-03-01 00:00:00','','BlackSky Global' from dual union all
select 'BST','Berlin Space Technologies GmbH','business','TUB1','D','Berlin:Aldershof','2014-01-01 00:00:00','2014-01-01 00:00:00','Berlin Space Technologies GmBH' from dual union all
select 'BTL','American Telephone and Telegraph Co. Bell Telephone Laboratories','business','','US','Murray Hill, New Jersey','1925-01-01 00:00:00','1996-01-01 00:00:00','American Telephone and Telegraph Co. Bell Telephone Laboratories' from dual union all
select 'BTLW','American Telephone and Telegraph Co. Bell Telephone Laboratories','business','BTL','US','Whippany, New Jersey','1925-01-01 00:00:00','1996-01-01 00:00:00','American Telephone and Telegraph Co. Bell Telephone Laboratories' from dual union all
select 'BTRC','Bangladesh Telecommunications Regulatory Commission','civilian','','BD','Dhaka','2002-01-01 00:00:00','','বাংলাদেশ টেলিযোগাযোগ নিয়ন্ত্রণ কমিশন' from dual union all
select 'BU','Boston University','academic/non-profit','','US','Boston, Mass.','1839-01-01 00:00:00','','Boston University' from dual union all
select 'BUAA','Beijing University of Aeronautics and Astronautics','academic/non-profit','','CN','Beijing','1952-10-01 00:00:00','','北京航空航天大学' from dual union all
select 'BUCU','University of Bucharest','academic/non-profit','','RO','Bucuresti','1864-01-01 00:00:00','','Universitatea din Bucuresti' from dual union all
select 'BULSAT','Bulsatcom AD','business','','BGN','Stara Zagora','2004-01-01 00:00:00','','Bulsatcom AD' from dual union all
select 'BUORD','Bureau of Ordnance, US Navy','defense','USN','US','Washington, DC','1862-01-01 00:00:00','1959-09-01 00:00:00','Bureau of Ordnance, US Navy' from dual union all
select 'BUSEK','Busek Co. Inc.','business','','US','Natick, Mass.','1985-01-01 00:00:00','','Busek Co. Inc.' from dual union all
select 'BUWEPS','Bureau of Naval Weapons, US Navy','defense','USN','US','Washington, DC','1959-09-01 00:00:00','1966-05-01 00:00:00','Bureau of Naval Weapons, US Navy' from dual union all
select 'BWB','Federal Office of Defense Technology and Procurement','defense','BWEHR','D','Koblenz','1957-01-01 00:00:00','','Bundesamt für Wehrtechnik und Beschaffung' from dual union all
select 'BWEHR','Bundeswehr','defense','','D','Bonn','1955-11-12 00:00:00','','Bundeswehr' from dual union all
select 'BWYIC','[Same as CALT?]','business','CALT','CN','Beijing:WanYuan','1957-11-11 00:00:00','','北京万源工业公司' from dual union all
select 'BY','Belarus','','','BY','Belarus''','1990-07-27 00:00:00','','Рэспубліка Беларусь' from dual union all
select 'BYXYS','Beijing Institute of Remote Sensing Info. (GS Aerospace Reco','defense','','CN','Beijing:Qinghe','','','北京遥感信息研究所 (总参航天技术侦察局)' from dual union all
select 'C360','Cinema 360, Inc.','business','','US','?','1980-01-01 00:00:00','1986-01-01 00:00:00','Cinema 360, Inc.' from dual union all
select 'CA','Canada','','','CA','Ottawa, Ontario','1867-07-01 00:00:00','','Canada' from dual union all
select 'CABVS','Cablevision Systems Corp.','business','','US','Bethpage, Long Island, New York','1973-01-01 00:00:00','','Cablevision Systems Corp.' from dual union all
select 'CAL','Columbia Astrophysics Lab, Columbia University','academic/non-profit','','US','New York, New York','1967-01-01 00:00:00','','Columbia Astrophysics Lab, Columbia University' from dual union all
select 'CALET','CALET Collaboration, Science Data Analysis Center','academic/non-profit','WASEDA','J','Tokyo:Shinjuku','2014-01-01 00:00:00','','カロリメータ型宇宙電子線望遠鏡' from dual union all
select 'CALG','University of Calgary','academic/non-profit','','CA','Calgary, Alberta','1966-01-01 00:00:00','','University of Calgary' from dual union all
select 'CALP','California Polytechnic State University','academic/non-profit','','US','San Luis Obispo, California','1901-01-01 00:00:00','','California Polytechnic State University' from dual union all
select 'CALT','CALT','civilian','CASC','CN','Beijing:Nanyuan','1957-11-11 00:00:00','','中国运载火箭技术研究院' from dual union all
select 'CAM','University of Cambridge','academic/non-profit','','UK','Cambridge','1209-01-01 00:00:00','','University of Cambridge' from dual union all
select 'CAMSAT','AMSATChina (CAMSAT)','academic/non-profit','','CN','Beijing','2009-01-01 00:00:00','','AMSAT-China (CAMSAT)' from dual union all
select 'CANON','Canon Electronics Ltd.','business','','J','Tokyo','2015-01-01 00:00:00','','キヤノン電子㈱' from dual union all
select 'CANOP','Canopus Systems US LLC','business','DAUR','US','Mountain View, California','2011-01-01 00:00:00','2015-01-01 00:00:00','Canopus Systems US LLC' from dual union all
select 'CAPSP','Capella Space Corp.','business','','US','Palo Alto, California','2016-01-01 00:00:00','','Capella Space Corp.' from dual union all
select 'CARDE','Canadian Armament Researcha and Development Establishment','defense','','CA','Quebec City:Valcartier, Quebec','1945-01-01 00:00:00','1969-01-01 00:00:00','Canadian Armament Researcha and Development Establishment' from dual union all
select 'CARDU','Cardiff University','academic/non-profit','','UK','Cardiff, Wales','1883-01-01 00:00:00','','Cardiff University' from dual union all
select 'CARTH','Carthage College','academic/non-profit','','US','Kenosha, Wisconsin','1847-01-01 00:00:00','','Carthage College' from dual union all
select 'CAS','Chinese Academy of Sciences','civilian','','CN','Beijing','1949-01-01 00:00:00','','中国科学院' from dual union all
select 'CAS581','CAS 581 Group (space science)','civilian','','CN','Beijing?','1958-01-01 00:00:00','1959-12-01 00:00:00','中国科学院‘581’组' from dual union all
select 'CASA','Construcciones Aeronauticas S.A. (CASA) Espacio','business','','E','Madrid:Barajas','1986-01-01 00:00:00','1999-01-01 00:00:00','Construcciones Aeronáuticas SA' from dual union all
select 'CASAG','Construcciones Aeronauticas S.A. (CASA) Espacio','business','','E','Madrid:Getafe','1924-01-01 00:00:00','1986-01-01 00:00:00','Construcciones Aeronáuticas SA' from dual union all
select 'CASC','China Aerospace Corp','business','','CN','Beijing, Zhongguo','1999-07-01 00:00:00','','中国航天科技集团公司' from dual union all
select 'CASC1','China Aerospace Sci/Tech Corp','business','','CN','Beijing','1993-06-01 00:00:00','1999-07-01 00:00:00','中国航天工业总公司' from dual union all
select 'CASC2A','2nd Academy of 7th Machine Building Ministry','civilian','CASC','CN','Beijing','1957-01-01 00:00:00','2002-01-01 00:00:00','二研究院七机械工业部' from dual;
insert into ORGANIZATION
select 'CASC6A','Academy of Aerospace Propulsion Technology','civilian','CASC','CN','Fengxian region, Xi''an, Shaanxi','1965-01-01 00:00:00','1993-01-01 00:00:00','第七机械工业部 六七基地' from dual union all
select 'CASC6A1','Academy of Aerospace Propulsion Technology','civilian','CASC','CN','Xi''an, Shaanxi','1993-01-01 00:00:00','2002-01-01 00:00:00','第七机械工业部 六七基地' from dual union all
select 'CASDN','National Defense Science Action Committee','defense','','F','Paris?','1948-01-01 00:00:00','1959-01-01 00:00:00','Comite'' d''Action Scientifique de Defense Nationale' from dual union all
select 'CASG2','CAS Geophysics 2nd Office','civilian','','CN','Beijing?','1959-12-01 00:00:00','1966-01-01 00:00:00','中国科学院地球物理所二部' from dual union all
select 'CASIC','CASIC','business','','CN','Beijing:Haidian','1999-07-01 00:00:00','','中国航天科工集团' from dual union all
select 'CASIC2A','CASIC 2nd Academy','civilian','CASIC','CN','Beijing','2002-01-01 00:00:00','2002-01-01 00:00:00','中国航天科工集团第二研究院' from dual union all
select 'CASIC4A','CASIC 4th Academy','civilian','CASIC','CN','Beijing?','2002-01-01 00:00:00','','中国航天科工集团第四研究院' from dual union all
select 'CASIE','CAS Institute of Electronics','civilian','CAS','CN','Beijing:Haidian,','','','中国科学院电子学研究所' from dual union all
select 'CASIOM','CAS Institute of Mechanics','civilian','CAS','CN','Beijing:Haidian','1956-01-01 00:00:00','','中国科学院力学研究所' from dual union all
select 'CAST','Chinese Academy of Space Technology, Beijing (CASC 5th Acad)','civilian','CASC','CN','Beijing:Haidian','1968-02-20 00:00:00','','中国空间技术研究院' from dual union all
select 'CASTBZ','Beijing Satellite Factory','civilian','CAST','CN','Beijing:Haidian','1968-02-27 00:00:00','','北京卫星制造厂' from dual union all
select 'CASTIX','CAST International Exchange Center','civilian','CAST','CN','Beijing','','','中国航天科技国际交流中心' from dual union all
select 'CASTL','CASIC Space Technology Ltd (CASIC 1st Acad?)','academic/non-profit','CASIC','CN','Beijing:Haidian','2005-01-01 00:00:00','2010-01-01 00:00:00','中国航天科工信息技术研究院' from dual union all
select 'CATHU','Catholic University of America','academic/non-profit','','US','Washington, D.C','1887-01-01 00:00:00','','Catholic University of America' from dual union all
select 'CATON','Beijing Caton Universal Technology Co. Ltd.','business','','CN','Beijing','2016-01-01 00:00:00','','北京凯盾环宇科技有限公司' from dual union all
select 'CBKPL','Space Research Center, Polish Academy of Sciences','civilian','','PL','Warszawa','1976-01-01 00:00:00','','Centrum Badań Kosmicznych' from dual union all
select 'CBMET','Commonwealth Bureau of Meteorology','civilian','','AU','Melbourne, Victoria','1955-06-01 00:00:00','','Commonwealth Bureau of Meteorology' from dual union all
select 'CCSST','Center for Space Science and Technology, CAS','civilian','CAS','CN','Beijing?  (beceame CSSAR)','1978-01-01 00:00:00','1987-01-01 00:00:00','空间科学技术中心' from dual union all
select 'CD','DR Congo','','','ZR','Kinshasa','1997-01-01 00:00:00','','République démocratique du Congo' from dual union all
select 'CEA','Commissariat a l''Energie Atomique (CEA)  Saclay','civilian','','F','Paris:Saclay, Essonne','1945-01-01 00:00:00','','Commissariat à l''énergie atomique, Saclay' from dual union all
select 'CECOM','US Army CommuncationsElectronics Materiel Readiness Command','defense','USA','US','Ft Monmouth, New Jersey','1981-05-01 00:00:00','2010-01-01 00:00:00','US Army Communcations-Electronics Materiel Readiness Command' from dual union all
select 'CEL','Centre d''Essais de Landes, DGA','defense','DGA','F','Biscarosse, France','1967-01-01 00:00:00','2005-01-01 00:00:00','Centre d''Essais de Landes, DGA' from dual union all
select 'CELES','Celesco Industries','business','','US','Costa Mesa, California','','1980-01-01 00:00:00','Celesco Industries' from dual union all
select 'CELML','Centre d''essais de Lancement de missiles, Site Landes','defense','DGA','F','Biscarosse, France','2005-01-01 00:00:00','2009-01-01 00:00:00','Centre d''essais de Lancement de missiles, Site Landes' from dual union all
select 'CERCOM','US Army CommuncationsElectronics Command','defense','USA','US','Ft Monmouth, New Jersey','1978-01-01 00:00:00','1981-05-01 00:00:00','US Army Communcations-Electronics Command' from dual union all
select 'CERMA','Centre d''Enseignement et de Recherches en Medecine Aeronautique','civilian','','F','Bretigny','1955-01-01 00:00:00','1988-01-01 00:00:00','Centre d''Enseignement et de Recherches de Médecine Aéronautique' from dual union all
select 'CERN','European Organization for Nuclear Research (CERN)','civilian','','F','Geneve, Suisse','1952-01-01 00:00:00','','Conseil Européen pour la Recherche Nucléaire' from dual union all
select 'CESBIO','Centre d''Etudes Spatiales de la Biosphere','civilian','TLS','F','Toulouse:Rangueil','1990-01-01 00:00:00','','Centre d''Études spatiales de la Biosphère' from dual union all
select 'CESR','Center for Radiation Space Studies','civilian','','F','Toulouse','','','Centre d''étude spatiale des rayonnements' from dual union all
select 'CETEF','Center for Physical Tech. L Torres Quevedo','civilian','','E','Madrid','1994-01-01 00:00:00','','Centro de Tecnologias Fisicas "Leonardo Torres Quevedo"' from dual union all
select 'CFCA','Canadian Forces','defense','','CA','Ottawa','','','Canadian Forces' from dual union all
select 'CFTH','Compagnie Francaise ThomsonHouston','business','','F','Paris','1893-01-01 00:00:00','1966-01-01 00:00:00','Compagnie Française Thomson-Houston' from dual union all
select 'CFTHB','CFTHHotchkissBrant','business','','F','Paris','1966-01-01 00:00:00','1968-01-01 00:00:00','CFTH-Hotchkiss-Brant' from dual union all
select 'CGS','Carlo Gavazzi Space','business','FUCHS','I','Milano','1981-01-01 00:00:00','2011-01-01 00:00:00','Carlo Gavazzi Space' from dual union all
select 'CGSO','OHB CGS S.p.A., Compagnia Generale per lo Spazio','business','FUCHS','I','Milano','2011-01-01 00:00:00','','OHB CGS S.p.A., Compagnia Generale per lo Spazio' from dual union all
select 'CGSTL','Chang Guang','business','CIOMP','CN','Changchun, Jilin','2014-12-01 00:00:00','','长光卫星技术有限公司' from dual union all
select 'CH','Switzerland','','','CH','Bern','1291-01-01 00:00:00','','Confédération suisse' from dual union all
select 'CHAIL','CutlerHammer Airborne Instruments Laboratory','business','','US','Mineola and Deer Park, New York','1958-01-01 00:00:00','2000-01-01 00:00:00','Cutler-Hammer Airborne Instruments Laboratory' from dual union all
select 'CHDB','China Direct Broadcast Satellite Co.','business','CHIS','CN','Beijing','2008-01-01 00:00:00','','中国直播星有限公司' from dual union all
select 'CHFEN','CCMETA','civilian','CASIC','CN','Beijing','2002-01-01 00:00:00','','中国长峰机电技术研究设计院;' from dual union all
select 'CHIBA','Chiba Institute of Technology','academic/non-profit','','J','Narashino, Chiba','1942-01-01 00:00:00','','千葉工業大学' from dual union all
select 'CHIC','University of Chicago','academic/non-profit','','US','Hyde Park, Chicago, Illinois','1891-01-01 00:00:00','','University of Chicago' from dual union all
select 'CHIS','China Satellite Communications Crop.','business','','CN','Beijing:Haidian','1983-01-01 00:00:00','2001-12-19 00:00:00','中国卫星通信 公司' from dual union all
select 'CHISA','China Satellite Communications Corp. Group (China Satcom)','business','','CN','Beijing:Haidian','2001-12-19 00:00:00','2009-04-01 00:00:00','中国卫星通信集团公司' from dual union all
select 'CHISAE','China Satellite Communications Corp. Group (China Satcom)','civilian','CASC','CN','Beijing:Haidian','2009-04-01 00:00:00','','中国卫星通信集团公司' from dual union all
select 'CHORI','China Orient Telecom Satellite Co. Ltd.','business','CHIS','CN','Beijing:Dong Cheng','1995-04-01 00:00:00','2008-04-01 00:00:00','中国东方通信卫星有限公司' from dual union all
select 'CHOSU','Choson U','academic/non-profit','','KR','Gwangju','1946-01-01 00:00:00','','조선대학교' from dual union all
select 'CHRMI','Chrysler Operations, NASA Michoud Assembly Facility','business','','US','Michoud, Louisiana','1962-01-01 00:00:00','1975-01-01 00:00:00','Chrysler Operations, NASA Michoud Assembly Facility' from dual union all
select 'CHRY','Chrysler Michigan Ordnance Missile Plant','business','','US','Detroit:Sterling Heights, Michigan','1952-01-01 00:00:00','1970-01-01 00:00:00','Chrysler Michigan Ordnance Missile Plant' from dual union all
select 'CHUNG','Chunghaw Telecom','business','','TW','Taipei','1996-01-01 00:00:00','','中華電信' from dual union all
select 'CI','Ivory Coast','','','CI','Cote d''Ivoire','1960-08-10 00:00:00','','République de Côte d''Ivoire' from dual union all
select 'CIA','Central Intelligence Agency, USA','defense','US','US','Langley, Virginia','1947-07-26 00:00:00','','Central Intelligence Agency, USA' from dual union all
select 'CIAB','CIA Office of Development and Engineering, Dir of Sci.Tech. (NRO Program B)','defense','NRO','US','Langley, Virginia','1965-01-01 00:00:00','1992-12-31 00:00:00','CIA Office of Development and Engineering, Dir of Sci.Tech. (NRO Program B)' from dual union all
select 'CIAG','Institute for Applied Geophysics','civilian','','CN','Beijing?  (beceame CSSAR)','1966-01-01 00:00:00','1968-01-01 00:00:00','应用地球物理所' from dual union all
select 'CIEL','Ciel Satellite Group','business','','CA','Ottawa:Kanata, Ontario','2004-01-01 00:00:00','','Ciel Satellite Group' from dual union all
select 'CIFAS','Consortium Industrial FranceAllemand pour le Satellite Symphonie','business','','F','Les Mureaux','1968-01-01 00:00:00','1975-01-01 00:00:00','Consortium Industrial France-Allemand pour le Satellite Symphonie' from dual union all
select 'CIFTQ','Center for Physics Investigations. L Torres Quevedo','civilian','','E','Madrid','','1994-01-01 00:00:00','Centro de Investigaciones Fiscas "L. Torres Quevedo"' from dual union all
select 'CIN','University of Cincinatti','academic/non-profit','','US','Cincinatti, Ohio','1819-01-01 00:00:00','','University of Cincinatti' from dual union all
select 'CIOMP','CIOMP','academic/non-profit','CAS','CN','Changchun, Jilin','1952-01-01 00:00:00','','长春光学精密机械与物理研究所' from dual union all
select 'CISIR','Ceylon Institute for Scientific and Industrial Research','civilian','','LK','Colombo','1955-01-01 00:00:00','1988-01-01 00:00:00','Ceylon Institute for Scientific and Industrial Research' from dual union all
select 'CISP','Institute for Space Physics','civilian','','CN','Beijing?  (beceame CSSAR)','1980-01-01 00:00:00','1987-01-01 00:00:00','Institute for Space Physics, CAS' from dual union all
select 'CISP5','5th Institute for Space Physics','civilian','','CN','Beijing?  (beceame CSSAR)','1969-01-01 00:00:00','1978-11-01 00:00:00','第五研究院空间物理及探测技术研究所' from dual union all
select 'CIT','California Institute of Technology','academic/non-profit','','US','Pasadena, California','1891-01-01 00:00:00','','California Institute of Technology' from dual union all
select 'CITED','Centro de Investigaciones Cientificas y Tecnologicas para la Defensa','defense','','AR','Villa Maria, Cordoba','1954-01-01 00:00:00','','Instituto de Investigaciones Científicas y Técnicas para la Defensa' from dual union all
select 'CITEF','Inst.de Investigaciones Cientificas y Tech. de las Fuerzas Armadas','civilian','','AR','Falda del Carmen, Cordoba','1982-01-01 00:00:00','1991-05-28 00:00:00','Instituto de Investigaciones Científicas y Técnicas de las Fuerzas Armadas' from dual union all
select 'CK','Cook Islands','','','CK','Avarua, Rarotonga Island','1965-08-04 00:00:00','','Cook Islands (Kuki ''Airani)' from dual union all
select 'CL','Chile','','','CL','Chile','1818-02-12 00:00:00','','República de Chile' from dual union all
select 'CLAAC','AAC Microtec/Clyde Space Ltd.','business','','UK','Glasgow, Scotland','2018-01-01 00:00:00','','AAC Microtec/Clyde Space Ltd.' from dual union all
select 'CLARK','David Clark Co.','business','','US','Worcester, Massachusetts','1935-01-01 00:00:00','','David Clark Co.' from dual union all
select 'CLASP','University of ColoradoLab for Atmospheric and Space Physics','academic/non-profit','UCO','US','Boulder, Colorado','1965-01-01 00:00:00','','University of Colorado-Lab for Atmospheric and Space Physics' from dual union all
select 'CLEM','Clemson University','academic/non-profit','','US','Clemson, S Carolina','1889-01-01 00:00:00','','Clemson University' from dual union all
select 'CLSR','Central Lab. for Space Research, Sofia','academic/non-profit','','BGN','Sofia','1975-01-01 00:00:00','1987-01-01 00:00:00','Централна лаборатория за космически изследвания към БАН' from dual union all
select 'CLYDE','Clyde Space Ltd.','business','','UK','Glasgow, Scotland','2006-01-01 00:00:00','2018-01-01 00:00:00','Clyde Space Ltd.' from dual union all
select 'CMA','China Meteorological Administration','civilian','','CN','Beijing','1941-01-01 00:00:00','','中国气象局' from dual union all
select 'CMIK','Choson MIK','defense','KPA','KP','Pyongyang','','','Choson MIK' from dual union all
select 'CMMB','CMMB Vision Holdings (China Mobile Multimedia Broadcasting)','business','','HK','Hong Kong','2008-01-01 00:00:00','','中國移動多媒體廣播控股有限公司' from dual union all
select 'CMSEO','CMSEO','defense','ZZB','CN','Beijing','1999-01-01 00:00:00','','中国载人航天工程办公室' from dual union all
select 'CN','China','','','CN','Beijing','1949-10-01 00:00:00','','中华人民共和国' from dual union all
select 'CNAE','Space Activities Comission','civilian','','BR','Rio de Janeiro?','1963-01-01 00:00:00','1971-01-01 00:00:00','Comissão Nacional de Atividades Espaciais' from dual union all
select 'CNCOS','COSTIND','defense','','CN','Beijing','1982-01-01 00:00:00','2008-01-01 00:00:00','国防科学技术工业委员会' from dual union all
select 'CNES','National Space Research Center','civilian','','F','Paris','1962-01-01 00:00:00','','Centre national d''études spatiales' from dual union all
select 'CNESB','Bretigny Space Center','civilian','CNES','F','Bretigny','1962-01-01 00:00:00','1974-01-01 00:00:00','CNES  Centre Spatial de Brétigny' from dual union all
select 'CNEST','Toulouse Space Center','civilian','CNES','F','Toulouse','1973-01-01 00:00:00','','Centre Spatiale de Toulouse' from dual union all
select 'CNET','National Telecom Research Center','civilian','','F','Paris:Bagneux (Issy)','1944-01-01 00:00:00','2000-03-01 00:00:00','Centre national d''études des télécommunications' from dual union all
select 'CNIE','National Space Research Commission','civilian','','AR','Buenos Aires','1961-01-01 00:00:00','1991-05-28 00:00:00','Comisión Nacional de Investigaciones Espaciales' from dual union all
select 'CNR','National Research Council (Italy)','civilian','','I','Roma','1923-01-01 00:00:00','','Consiglio Nazionale delle Richerche' from dual union all
select 'CNRS','National Scientific Research Council (France)','civilian','','F','Paris','1939-01-01 00:00:00','','Conseil National de Recherches Scientifiques' from dual union all
select 'CNSA','China National Space Administration','civilian','','CN','Beijing','1993-06-01 00:00:00','','国家航天局' from dual union all
select 'CNSAS','SASTIND','defense','','CN','Beijing','2008-01-01 00:00:00','','国家国防科技工业局' from dual union all
select 'CNSSC','National Space Science Center','civilian','CAS','CN','Beijing:Haidian','2015-06-01 00:00:00','','中国科学院国家空间科学中心' from dual union all
select 'CNTS','National Space Technology Center (Algeria)','civilian','ASAL','DZ','Dzayer (Algiers)','2002-01-01 00:00:00','','Centre National des Techniques Spatiales' from dual union all
select 'CNU','Chungnam National University','academic/non-profit','','KR','Daejeon','1952-01-01 00:00:00','','충남대학교' from dual union all
select 'CO','Colombia','','','CO','Colombia','1810-07-20 00:00:00','','República de Colombia' from dual union all
select 'COEP','College of Engineering, Pune','academic/non-profit','','IN','Pune, Maharashtra','1854-01-01 00:00:00','','College of Engineering, Pune' from dual union all
select 'COLC','Columbia Communications Corp.','business','','US','Honolulu, Hawaii','1983-01-01 00:00:00','2000-09-01 00:00:00','Columbia Communications Corp.' from dual union all
select 'COLCG','Columbia Communications Corp. (GE Americom)','business','AMCG','US','Honolulu, Hawaii','2000-09-01 00:00:00','2003-01-01 00:00:00','Columbia Communications Corp. (GE Americom)' from dual union all
select 'COLE','Coleman Research Corporation','business','','US','Orlando, Florida','1980-01-01 00:00:00','1996-01-01 00:00:00','Coleman Research Corporation' from dual union all
select 'COLEA','Coleman Aerospace','business','','US','Orlando, Florida','1996-01-01 00:00:00','2017-01-01 00:00:00','Coleman Aerospace' from dual union all
select 'COLEAR','Coleman Aerospace/Aerojet Rocketdyne','business','AJR','US','Orlando, Florida','2017-01-01 00:00:00','','Coleman Aerospace' from dual union all
select 'COLP','Congo','','','ZR','Leopoldville','1965-11-24 00:00:00','1971-01-01 00:00:00','République du Congo-Léopoldville' from dual;
insert into ORGANIZATION
select 'COM','Comsat General Corp., Washington D.C.','business','','US','Washington, DC','1963-01-01 00:00:00','1998-09-01 00:00:00','Comsat General Corp., Washington D.C.' from dual union all
select 'COMAH','Comahue National University','academic/non-profit','','AR','Neuquen, Neuquen, Patagonia','1971-01-01 00:00:00','','Universidad Nacional del Comahue (UNCOMA)' from dual union all
select 'COMDEV','Com Dev International','business','','CA','Cambridge, Ontario','1974-01-01 00:00:00','','Com Dev International' from dual union all
select 'CONAB','National Space Activities Commission','civilian','','AR','Buenos Aires','1991-05-28 00:00:00','','Comisión Nacional de Actividades Espaciales' from dual union all
select 'CONAE','CONAE Tabanera Space Center','civilian','CONAB','AR','Falda del Carmen, Cordoba','1991-05-28 00:00:00','','CONAE Centro Espacial Teófilo Tabanera' from dual union all
select 'CONCOR','Concordia University','academic/non-profit','','CA','Montreal, Quebec','1974-01-01 00:00:00','','Concordia University' from dual union all
select 'CONIE','National Space Investigation Commission','civilian','','E','Madrid:Torrejon','1963-07-08 00:00:00','1986-01-01 00:00:00','Comisión Nacional de Actividades Espaciales' from dual union all
select 'CONT','Contel ASC Inc.','business','CONTL','US','Rockville, Maryland','1985-01-01 00:00:00','1991-01-01 00:00:00','Contel ASC Inc.' from dual union all
select 'CONTF','Contel Federal Systems','business','CONTL','US','Fairfax, Virginia','1976-01-01 00:00:00','1993-01-01 00:00:00','Contel Federal Systems' from dual union all
select 'CONTL','Contel Corporation','business','','US','Atlanta, Georgia','1976-01-01 00:00:00','1993-01-01 00:00:00','Contel Corporation' from dual union all
select 'CONTR','Oerlikon Contraves AG','business','','CH','Zurich','1936-01-01 00:00:00','1999-01-01 00:00:00','Oerlikon Contraves AG' from dual union all
select 'CONTS','Oerlikon Contraves Space','business','','CH','Zurich','1999-01-01 00:00:00','2007-01-01 00:00:00','Oerlikon Contraves Space' from dual union all
select 'CONV','Consolidated Vultee Aircraft Corp.','business','','US','San Diego, California','1943-01-01 00:00:00','1953-03-01 00:00:00','Consolidated Vultee Aircraft Corp.' from dual union all
select 'CONVA','Convair Astronautics Division','business','GD','US','San Diego:Kearny Mesa, California','1957-01-01 00:00:00','1961-06-01 00:00:00','Convair Astronautics Division' from dual union all
select 'CONVPO','Convair Aircraft, Pomona','business','','US','Pomona, California','1952-01-01 00:00:00','1954-04-01 00:00:00','Convair Aircraft, Pomona' from dual union all
select 'COOP','Cooper Development Corp. (CDC)','business','MARQ','US','Monrovia, California','1955-01-01 00:00:00','1964-01-01 00:00:00','Cooper Development Corp. (CDC)' from dual union all
select 'COPERN','European Commission DGGROW Directorate I/2, Copernicus Unit','civilian','DGGROW','I-EU','Brussels','2014-01-01 00:00:00','','European Commission DG-GROW Directorate I/2, Copernicus Unit' from dual union all
select 'CORAT','Coratec (Cordoba Alta Tecnologia S.E.)','business','','AR','Cordoba, Argentina','','','Coratec (Cordoba Alta Tecnologia S.E.)' from dual union all
select 'CORD','University Aeronautical Institute','academic/non-profit','','AR','Cordoba, Argentina','1991-01-01 00:00:00','','Instituto Universitario Aeronáutico,  Fuerza Aérea Argentina' from dual union all
select 'CORN','Cornell University','academic/non-profit','','US','Ithaca, New York','1865-01-01 00:00:00','','Cornell University' from dual union all
select 'COSG','Colorado Space Grant Consortium, U Colorado at Boulder','academic/non-profit','UCO','US','Boulder, Colorado','1989-01-01 00:00:00','','Colorado Space Grant Consortium, U Colorado at Boulder' from dual union all
select 'COSM','COSMOS consortium (Aerospatiale/CASA/MBB/Selenia/MSDS)','business','','F','Cannes','','','COSMOS consortium (Aerospatiale/CASA/MBB/Selenia/MSDS)' from dual union all
select 'COSMOG','Cosmogia Inc.','business','','US','San Francisco, California','2011-01-01 00:00:00','2013-06-01 00:00:00','Cosmogia Inc.' from dual union all
select 'COUAL','Upper Atmosphere Lab, University of Colorado','academic/non-profit','UCO','US','Boulder, Colorado','1948-01-01 00:00:00','1965-01-01 00:00:00','Upper Atmosphere Lab, University of Colorado' from dual union all
select 'CPUT','Cape Peninsula University of Technology, Belleville Campus','academic/non-profit','','ZA','Cape Town:Belleville','2005-01-01 00:00:00','','Cape Peninsula University of Technology, Belleville Campus' from dual union all
select 'CR','Costa Rica','','','CR','San Jose','1838-01-01 00:00:00','','República de Costa Rica' from dual union all
select 'CRA','Aerospace Research Center','civilian','CRPSM','I','Roma','1959-01-01 00:00:00','','Centro Richerchi Aerospaziale, Roma la Sapienza' from dual union all
select 'CRC','Communications Research Center','civilian','','CA','Ottawa, Canada','1969-01-01 00:00:00','','Communications Research Center' from dual union all
select 'CRCSS','Cooperative Research Centre for Satellite Systems','civilian','','AU','Canberra:Yarralumla','1998-01-01 00:00:00','2005-12-31 00:00:00','Cooperative Research Centre for Satellite Systems' from dual union all
select 'CRI','Computer Resources International','business','','DK','Kobenhavn:Rudersdal/Birkerod','','1997-01-01 00:00:00','Computer Resources International' from dual union all
select 'CRLI','Communications Research Lab','civilian','','J','Kashima, Ibaraki, Japan','1988-01-01 00:00:00','2004-01-01 00:00:00','通信総合研究所' from dual union all
select 'CRPSM','San Marco Project Research Center','academic/non-profit','ROSAP','I','Roma','1962-01-01 00:00:00','','Centro di Ricerca Progetto San Marco' from dual union all
select 'CRR','Churchill Research Range (USAF)','civilian','OAR','CA','Churchill, Manitoba','1962-11-01 00:00:00','1970-06-01 00:00:00','Churchill Research Range (USAF)' from dual union all
select 'CRRC','Churchill Research Range (NRC)','civilian','NRCC','CA','Churchill, Manitoba','1970-06-01 00:00:00','','Churchill Research Range (NRC)' from dual union all
select 'CRS','ISRC','civilian','CNR','I','Roma','1959-01-01 00:00:00','','Commissione per le Richerche Spaziali (CRS)' from dual union all
select 'CRSEA','Centro de Referencia en Sistemas Embarcados e Aerosepaciais','civilian','','BR','Sao Joao da Barra, Rio de Janeiro province','2009-01-01 00:00:00','','Centro de Referencia en Sistemas Embarcados e Aerosepaciais' from dual union all
select 'CRSF','Comite des Recherches Spatiales','civilian','','F','Paris','1959-01-01 00:00:00','1962-01-01 00:00:00','Comité des recherches spatiales' from dual union all
select 'CRTS','Centre Royal de Teledetection Spatiale','civilian','','MA','Hay Ryad, Rabat','','','Centre royal de télédétection spatiale du Maroc' from dual union all
select 'CSA','Canadian Space Agency','civilian','','CA','Montreal:Saint-Hubert','1989-12-14 00:00:00','','Canadian Space Agency' from dual union all
select 'CSAE','CSA Engineering','business','','US','Mountain View, California','1982-01-01 00:00:00','2008-01-01 00:00:00','CSA Engineering' from dual union all
select 'CSD','United Technology Corp Chemical Systems Division (UTC CSD)','business','UTC2','US','San Jose, California','1975-05-01 00:00:00','1996-01-01 00:00:00','United Technology Corp Chemical Systems Division (UTC CSD)' from dual union all
select 'CSFR','Czechoslovakia','','','CZ','Praha, Ceskoslovensko','1990-01-01 00:00:00','1993-01-01 00:00:00','Česká a Slovenská Federativní Republika' from dual union all
select 'CSG','Centre Spatial Guyanais','civilian','','F','Kourou, Guyane','1967-01-01 00:00:00','','Centre Spatial Guyanais' from dual union all
select 'CSIAP','CSIRO Division of Atmospheric Physics','civilian','CSIRO','AU','Melbourne:Aspendale','1971-01-01 00:00:00','1988-01-01 00:00:00','CSIRO Division of Atmospheric Physics' from dual union all
select 'CSIAR','CSIRO Division of Atmospheric Research','civilian','CSIRO','AU','Melbourne:Aspendale','1988-01-01 00:00:00','','CSIRO Division of Atmospheric Research' from dual union all
select 'CSICE','Cabinet Satellite Intelligence Center','defense','','J','Tokyo','2003-01-01 00:00:00','','内閣衛星情報センター' from dual union all
select 'CSIMP','CSIRO Division of Meteorological Physics','civilian','CSIRO','AU','Melbourne:Aspendale','1949-01-01 00:00:00','1971-01-01 00:00:00','CSIRO Division of Meteorological Physics' from dual union all
select 'CSIR','Council on Scientific and Industrial Research, South Africa','civilian','','ZA','Pretoria','1945-01-01 00:00:00','','Council on Scientific and Industrial Research, South Africa' from dual union all
select 'CSIRO','Commonwealth Scientific and Industrial Research Organization','civilian','AU','AU','Canberra','1949-01-01 00:00:00','','Commonwealth Scientific and Industrial Research Organization' from dual union all
select 'CSSAR','Space Science and Application Research Center, CAS','civilian','CAS','CN','Beijing:Haidian','1987-01-01 00:00:00','2015-06-01 00:00:00','中国科学院空间科学与应用研究中心' from dual union all
select 'CSSR','Czechoslovakia','','','CZ','Praha, Ceskoslovensko','1940-01-01 00:00:00','1990-01-01 00:00:00','Československá socialistická republika' from dual union all
select 'CSU','Colorado State University','academic/non-profit','','US','Fort Collins, Colorado','1870-01-01 00:00:00','','Colorado State Universitya' from dual union all
select 'CSUN','California State University, Northridge','academic/non-profit','','US','Los Angeles:Northridge, California','1958-01-01 00:00:00','','California State University, Northridge' from dual union all
select 'CSXT','Civilian Space Exploration Team (CSXT)','business','','US','Minneapolis, Minnesota','1995-01-01 00:00:00','','Civilian Space Exploration Team (CSXT)' from dual union all
select 'CTA','CTA Space Systems Inc','business','','US','McLean, Virginia','1992-07-01 00:00:00','1997-01-01 00:00:00','CTA Space Systems Inc' from dual union all
select 'CTEL','China Telecommunications Corporation','civilian','','CN','Beijing','','','中国电信集团公司' from dual union all
select 'CTYK','Changsha Research Inst. Space Instrument Tech. Co. Ltd.','business','','CN','Changsha, Hunan','2016-01-01 00:00:00','','长沙天仪空间科技研究院有限公司' from dual union all
select 'CUBESP','CubeSpace','business','','ZA','Stellenbosch','2015-01-01 00:00:00','','CubeSpace' from dual union all
select 'CUBIC','Cubic Corp.','business','','US','San Diego, California','1951-01-01 00:00:00','','Cubic Corp.' from dual union all
select 'CULH','Culham Laboratory','civilian','','UK','Abingdon, Oxfordshire','1963-01-01 00:00:00','','Culham Laboratory' from dual union all
select 'CUNYME','Medgar Evers College, City University of New York','academic/non-profit','','US','New York:Brooklyn, New York','1970-01-01 00:00:00','','Medgar Evers College, City University of New York' from dual union all
select 'CYM','Cayman Is.','','UK','CYM','Georgetown','1962-01-01 00:00:00','','Cayman Islands British Overseas Territory' from dual union all
select 'CZ','Czech Republic','','','CZ','Ceska Rep.','1993-01-01 00:00:00','','Česká republika' from dual union all
select 'D','Germany','','','D','Bonn, Deutschland  (later Berlin)','1949-05-23 00:00:00','','Bundesrepublik Deutschland' from dual union all
select 'DAC','Douglas Aircraft Corp.','business','','US','Santa Monica, California','1921-07-01 00:00:00','1967-04-28 00:00:00','Douglas Aircraft Corp.' from dual union all
select 'DACC','Douglas Aircraft Corp. Charlotte Ordnance Missile Plant (COMP)','business','DAC','US','Charlotte, North Carolina','1956-01-01 00:00:00','1967-05-01 00:00:00','Douglas Aircraft Corp. Charlotte Ordnance Missile Plant (COMP)' from dual union all
select 'DACHB','Douglas Aircraft Corp. Missiles and Space Systems Division (HB)','business','DAC','US','Huntington Beach, California','1963-01-01 00:00:00','1967-04-28 00:00:00','Douglas Aircraft Corp. Missiles and Space Systems Division (HB)' from dual union all
select 'DACM','Douglas Aircraft Corp. Missiles and Space Systems Division (MSSD)','business','DAC','US','Santa Monica, California','1961-01-01 00:00:00','1967-01-01 00:00:00','Douglas Aircraft Corp. Missiles and Space Systems Division (MSSD)' from dual union all
select 'DARA','German Space Affairs Agency','civilian','','D','Bonn:Oberkassel?','1989-01-01 00:00:00','1997-01-01 00:00:00','Deutsche Agentur für Raumfahrtangelegenheiten' from dual union all
select 'DARE','Delft Aerospace Rocket Engineering','academic/non-profit','DELFT','NL','Delft','2001-01-01 00:00:00','','Delft Aerospace Rocket Engineering' from dual union all
select 'DARPA','Defense Advanced Research Projects Agency','defense','DOD','US','Washington DC','1972-03-23 00:00:00','1993-02-22 00:00:00','Defense Advanced Research Projects Agency' from dual union all
select 'DARPA2','Defense Advanced Research Projects Agency','defense','DOD','US','Washington DC','1996-02-10 00:00:00','','Defense Advanced Research Projects Agency' from dual union all
select 'DART','Dartmouth University','academic/non-profit','','US','Hanover, New Hampshire','1769-12-13 00:00:00','','Dartmouth University' from dual union all
select 'DASA','US Defense Atomic Support Agency','defense','DOD','US','Albequerque, New Mexico','1959-05-06 00:00:00','1971-11-24 00:00:00','US Defense Atomic Support Agency' from dual union all
select 'DASAF','Deutsche Aerospace AG (DASA)Friedrichsafen (Dornier GmBH)','business','','D','Friedrichsafen','1989-03-01 00:00:00','1995-01-01 00:00:00','Deutsche Aerospace AG (DASA)-Friedrichsafen (Dornier GmBH)' from dual union all
select 'DASAO','Deutsche Aerospace AG (DASA)Ottobrunn','business','','D','Munchen:Ottobrunn','1989-12-01 00:00:00','2000-07-01 00:00:00','Deutsche Aerospace AG (DASA)-Ottobrunn' from dual union all
select 'DASARI','DaimlerBenz Aerospace RaumfahrtInfrastruktur (DASA RI)','business','','D','Bremen, BRD','1989-12-01 00:00:00','2000-07-10 00:00:00','Daimler-Benz Aerospace Raumfahrt-Infrastruktur (DASA RI)' from dual union all
select 'DASBF','DaimlerBenz Aerospace (DASA)Friedrichsafen (Dornier GmBH)','business','','D','Friedrichsafen','1995-01-01 00:00:00','1998-11-07 00:00:00','Daimler-Benz Aerospace (DASA)-Friedrichsafen (Dornier GmBH)' from dual union all
select 'DASCF','DaimlerChrysler Aerospace (DASA)Friedrichsafen','business','','D','Friedrichsafen','1998-11-07 00:00:00','2000-07-10 00:00:00','Daimler-Chrysler Aerospace (DASA)-Friedrichsafen' from dual union all
select 'DASST','Avions Marcel Dassault','business','','F','Bordeaux:Martignas:sur:Jalle, Gironde','1951-01-01 00:00:00','','Avions Marcel Dassault' from dual union all
select 'DAUR','Dauria Aerospace','business','','RU','Moskva:Skolkovo','2012-04-01 00:00:00','','Dauria Aerospace' from dual union all
select 'DBP','DBP Telecom','civilian','','D','Darmstadt','','','Deutsche Bundespost Telekom, Fernmeldetechnisches Zentralamt' from dual union all
select 'DBSD','DBSD North America Inc.','business','DISH','US','Reston, Virginia','2009-08-01 00:00:00','2012-03-01 00:00:00','DBSD North America Inc.' from dual union all
select 'DBSDD','Dish Network, DBSD assets','business','DISH','US','Reston, Virginia','2012-03-01 00:00:00','','Dish Network, DBSD assets' from dual union all
select 'DCA','Defense Communications Agency','defense','DOD','US','Arlington, Virginia','1960-01-01 00:00:00','1991-06-25 00:00:00','Defense Communications Agency' from dual union all
select 'DD','East Germany','','','DD','Berlin','1949-10-01 00:00:00','1990-10-03 00:00:00','Deutsche Demokratische Republik' from dual union all
select 'DEHAVE','De Havilland Engine Co.','business','','UK','Watford:Leavesden?','1946-01-01 00:00:00','1959-01-01 00:00:00','De Havilland Engine Co.' from dual union all
select 'DEHAVS','de Havilland Aircraft','business','','UK','Stevenage','1953-01-01 00:00:00','1963-01-01 00:00:00','de Havilland Aircraft' from dual union all
select 'DEHAVT','De Havilland, Toronto (Special Products/Appl. Research Div)','business','','CA','Toronto:Downsview, Ontario','1928-01-01 00:00:00','1967-01-01 00:00:00','De Havilland, Toronto (Special Products/Appl. Research Div)' from dual union all
select 'DEIMI','Deimos Imaging','business','DEIMOS','E','Boecillo, Valladolid','2006-01-01 00:00:00','2009-09-01 00:00:00','Deimos Imaging' from dual union all
select 'DEIMOS','Deimos Space S.L.','business','','E','Madrid:Tres Cantos','2001-06-01 00:00:00','2009-09-01 00:00:00','Deimos Space S.L.' from dual union all
select 'DELFT','Technische Universiteit Delft','academic/non-profit','','NL','Delft','1986-01-01 00:00:00','','Technische Universiteit Delft' from dual union all
select 'DENV','University of Denver','academic/non-profit','','US','Denver, Colorado','1864-01-01 00:00:00','','University of Denver' from dual union all
select 'DERA','Defense Evaluation and Research Agency','defense','UK','UK','Farnborough, Hants','1995-04-01 00:00:00','2001-07-01 00:00:00','Defense Evaluation and Research Agency' from dual union all
select 'DFF','DeutscheFranzosische Forschungsinstitut SaintLouis (ISL)','academic/non-profit','','F','Saint:Louis, Haut:Rhin, Alsace','1958-03-31 00:00:00','','Deutsch-Französisches Forschungsinstitut Saint-Louis' from dual union all
select 'DFH','Spaceflight East Is Red Satellite Co','business','CAST','CN','Beijing','2001-08-01 00:00:00','2006-01-01 00:00:00','航天东方红卫星有限公司' from dual union all
select 'DFHS','Shenzhen Hangtian DongFangHong HIT Satellite Co','business','CAST','CN','Shenzhen, Guangdong','2008-01-01 00:00:00','','深圳航天东方红海特卫星有限公司' from dual union all
select 'DFHZ','Spaceflight East Is Red Satellite Co','business','ZDFH','CN','Beijing','2006-01-01 00:00:00','','航天东方红卫星有限公司' from dual union all
select 'DFRC','NASA Dryden Flight Research Center','civilian','NASA','US','Edwards AFB, California','1976-03-26 00:00:00','2014-01-08 00:00:00','NASA Dryden Flight Research Center' from dual union all
select 'DFVLR','DFVLR','civilian','','D','Koln','1969-01-01 00:00:00','1989-01-01 00:00:00','Deutsche Forschungs- und Versuchsanstalt für Luft- und Raumfahrt' from dual;
insert into ORGANIZATION
select 'DGA','General Delegation for Ordnance','defense','F','F','Paris:Bagneux','1977-01-01 00:00:00','','Direction générale de l''armement' from dual union all
select 'DGAEM','DGA Essais de missiles, Site Landes','defense','DGA','F','Biscarosse, France','2009-01-01 00:00:00','','DGA Essais de missiles, Site Landes' from dual union all
select 'DGGROW','DirectorateGeneral for Internal Market, Industry, Entrepreneurship and SMEs','civilian','','I-EU','Brussel','','','Directorate-General for Internal Market, Industry, Entrepreneurship and SMEs' from dual union all
select 'DGLO','DigitalGlobe','business','','US','Longmont, Colorado','2002-01-01 00:00:00','','DigitalGlobe' from dual union all
select 'DGLOH','DigitalGlobe (Herndon)','business','DGLO','US','Herndon, Virginia','2012-07-01 00:00:00','2017-10-01 00:00:00','DigitalGlobe (Herndon)' from dual union all
select 'DGLOT','DigitalGlobe (Thornton)','business','DGLO','US','Thornton, Colorado','2012-07-01 00:00:00','','DigitalGlobe (Thornton)' from dual union all
select 'DHSST','US Department of Homeland Security, Science and Technology Directorate','defense','','US','Washington, DC','2003-01-01 00:00:00','','US Department of Homeland Security, Science and Technology Directorate' from dual union all
select 'DISA','Defense Information Systems Agency','defense','DOD','US','Arlington, Virginia','1991-06-25 00:00:00','','Defense Information Systems Agency' from dual union all
select 'DISH','Dish Network','business','','US','Meridian, Colorado','2008-01-01 00:00:00','','Dish Network' from dual union all
select 'DK','Denmark','','','DK','Danmark','0700-01-01 00:00:00','','Kongeriget Danmark' from dual union all
select 'DKAU','Ukranian State Space Agency','civilian','','UA','Kiev, Ukraine','2011-01-01 00:00:00','','Державне космічне агентство України' from dual union all
select 'DLR','German Research Inst. for Aviation and Spaceflight','civilian','','D','Koln','1989-01-01 00:00:00','1997-01-01 00:00:00','Deutsche Forschungsanstalt für Luft- und Raumfahrt' from dual union all
select 'DLR2','German Aerospace Center','civilian','','D','Koln','1997-01-01 00:00:00','','Deutsches Zentrum für Luft- und Raumfahrt' from dual union all
select 'DLRB','DLR Institute for Space Systems','civilian','DLR','D','Bremen','2006-01-01 00:00:00','','DLR Institute fur Raumfahrtsysteme' from dual union all
select 'DLRRA','DLR Institute fur Raumfahrtantriebe (Space propulsion)','civilian','DLR','D','Lampoldshausen','','','DLR Institute fur Raumfahrtantriebe (Space propulsion)' from dual union all
select 'DLRRD','DLR RaumfahrtAgentur','civilian','DLR','D','Bonn:Oberkassel','1997-01-01 00:00:00','','DLR Raumfahrt-Agentur' from dual union all
select 'DMA','D\''el\''egation Minist\''erielle pour l''Armement','defense','','F','Paris:Bagneux','1961-01-01 00:00:00','1977-01-01 00:00:00','Délégation Ministérielle pour l''Armement' from dual union all
select 'DMC','DMC International Imaging','business','','UK','Guildford','2004-01-01 00:00:00','','DMC International Imaging' from dual union all
select 'DMI','Danish Meteorological Institute','civilian','','DK','Kobenhavn','1872-01-01 00:00:00','','Danmarks Meteorologiske Institut' from dual union all
select 'DML','Queen Maud Land','','','DML','Troll, Princess Martha Coast','1939-01-14 00:00:00','','Dronning Maud Land' from dual union all
select 'DNA','Defense Nuclear Agency, DoD, USA','defense','DOD','US','Albequerque, New Mexico [and DC]','1971-11-24 00:00:00','1996-05-31 00:00:00','Defense Nuclear Agency, DoD, USA' from dual union all
select 'DND','Directorate of Space Development (DSpaceD), Department of National Defense','defense','','CA','Ottawa','1997-01-01 00:00:00','','Directorate of Space Development (DSpaceD), Department of National Defense' from dual union all
select 'DNE','Design Net Engineering','business','','US','Denver:Golden, Colorado','1996-01-01 00:00:00','','Design Net Engineering' from dual union all
select 'DNSC','Danish National Space Center','civilian','DTU','DK','Kobenhavn:Osterbro','2005-01-01 00:00:00','','Danmarks Rumcenter' from dual union all
select 'DOC','Department of Commerce','civilian','','US','Washington, DC','1903-01-01 00:00:00','','Department of Commerce' from dual union all
select 'DOD','Department of Defense','defense','','US','Washington, DC','1947-09-18 00:00:00','','Department of Defense' from dual union all
select 'DOE','Department of Energy','civilian','','US','Washington, DC','1977-08-04 00:00:00','','Department of Energy' from dual union all
select 'DORBIT','DORBIT','business','','I','Lomazzo, Como','2011-01-01 00:00:00','','D-Orbit' from dual union all
select 'DORN','Dornier Flugzeugwerke (Freidrichshafen)','business','','D','Friedrichsafen','1914-01-01 00:00:00','1985-01-01 00:00:00','Dornier Flugzeugwerke (Freidrichshafen)' from dual union all
select 'DORND','Dornier GmBH (DaimlerBenz)','business','','D','Friedrichsafen','1985-01-01 00:00:00','1989-05-01 00:00:00','Dornier GmBH (Daimler-Benz)' from dual union all
select 'DOSAAF','Voluntary Society of Assistance to the Army, AF and Navy','academic/non-profit','','SU','Moskva','1951-08-01 00:00:00','1991-09-25 00:00:00','Добровольное Общество Содействия Армии, Авиации и Флоту - ДОСААФ' from dual union all
select 'DOSH','Doshisha University','civilian','','J','Kyoto','1875-01-01 00:00:00','','同志社大学' from dual union all
select 'DOST','Department of Science and Technology','civilian','','PH','Taguig City','1987-01-01 00:00:00','','Kagawaran ng Agham at Teknolohiya' from dual union all
select 'DR','Germany (Reich)','','','DR','Berlin','1933-02-27 00:00:00','1949-05-23 00:00:00','Deutsches Reich' from dual union all
select 'DRA','Defense Research Agency','defense','','UK','Farnborough, Hants.','1991-04-01 00:00:00','1995-04-01 00:00:00','Defense Research Agency' from dual union all
select 'DRAP','Charles Stark Draper Laboratory, Inc.','business','','US','Cambridge, Mass.','1973-01-01 00:00:00','','Charles Stark Draper Laboratory, Inc.' from dual union all
select 'DRDC','Defense Research and Development Canada','defense','','CA','Quebec City:Valcartier, Quebec','2000-04-01 00:00:00','','Defense Research and Development Canada' from dual union all
select 'DRDL','Defense Research and Development Laboratory','defense','DRDO','IN','Hyderabad','','','Defense Research and Development Laboratory' from dual union all
select 'DRDO','Defense Research and Development Organization','defense','','IN','New Delhi','1958-01-01 00:00:00','','Defense Research and Development Organization' from dual union all
select 'DREV','DRE/Valcartier','defense','','CA','Quebec City:Valcartier, Quebec','1969-01-01 00:00:00','2000-04-01 00:00:00','DRE/Valcartier' from dual union all
select 'DREXEL','Drexel University','academic/non-profit','','US','Philadelphia, Pennsylvania','1891-01-01 00:00:00','','Drexel University' from dual union all
select 'DRG','German Rocket Society','business','','D','Cuxhaven?','1952-01-01 00:00:00','1961-01-01 00:00:00','Deutsche Raketen Gesellschaft (Seliger)' from dual union all
select 'DRTE','Defense Research Telecommunications Establishment, Canada','defense','','CA','Ottawa, Ontario','1951-01-01 00:00:00','1969-01-01 00:00:00','Defense Research Telecommunications Establishment, Canada' from dual union all
select 'DSI','Defense Systems Inc','business','','US','McLean, Virginia','1984-01-01 00:00:00','1992-07-01 00:00:00','Defense Systems Inc' from dual union all
select 'DSIR','UK Dept. of Scientific and Industrial Research','civilian','','UK','London','1917-01-01 00:00:00','1965-01-01 00:00:00','UK Dept. of Scientific and Industrial Research' from dual union all
select 'DSNC','DSN Corporation','business','','J','Tokyo','2012-01-01 00:00:00','','株式会社ディー・エス・エヌ' from dual union all
select 'DSRI','Danish Space Research Institute','civilian','','DK','Kobenhavn:Osterbro?','1968-01-01 00:00:00','2005-01-01 00:00:00','Dansk Rumforskningsinstitut (Danish Space Research Inst)' from dual union all
select 'DSSG','Deployable Space Systems','business','','US','Goleta, California','2008-04-01 00:00:00','','Deployable Space Systems' from dual union all
select 'DSTO','Defense Science and Technology Organization, Weapons Sys Div.','defense','','AU','Adelaide:Edinburgh (Salisbury), South Australia','1978-01-01 00:00:00','','Defense Science and Technology Organization, Weapons Sys Div.' from dual union all
select 'DTI','DTI Associates (ex Astrotech assets)','business','','US','Arlington, Virginia','2000-01-01 00:00:00','','DTI Associates (ex Astrotech assets)' from dual union all
select 'DTU','Technical University of Denmark','academic/non-profit','','DK','Kobenhavn:Lyngby','1829-01-01 00:00:00','','Danmarks Tekniske Universitet' from dual union all
select 'DTV','DirecTV','business','','US','El Segundo, California','1994-01-01 00:00:00','2015-01-01 00:00:00','DirecTV' from dual union all
select 'DTVA','AT&T/DirecTV','business','','US','El Segundo, California','2015-01-01 00:00:00','','AT&T/DirecTV' from dual union all
select 'DTVLA','DirecTV Latin America','business','DTVA','BR','Rio De Janeiro','2015-01-01 00:00:00','','DirecTV Latin America' from dual union all
select 'DUCOMM','Miltec Missiles and Space Co.','business','','US','Huntsville, Alabama','2012-01-01 00:00:00','2016-05-01 00:00:00','Miltec Missiles and Space Co.' from dual union all
select 'DUD','Dudley Observatory','academic/non-profit','','US','Schenectady, New York','1852-01-01 00:00:00','','Dudley Observatory' from dual union all
select 'DUTCH','Dutch Space, an EADS Astrium Company','business','ASTD','NL','Amsterdam','2002-01-01 00:00:00','','Dutch Space, an EADS Astrium Company' from dual union all
select 'DUTEC','DeltaUTEC Space Research','business','','NL','Leiden','1996-01-01 00:00:00','','Delta-UTEC Space Research' from dual union all
select 'DUTH','Democritus University of Thrace','academic/non-profit','','GR','Xanthi','1973-01-01 00:00:00','','Δημοκρίτειο Πανεπιστήμιο Θράκης' from dual union all
select 'DX','Germany (occ.)','','','DX','Berlin','1945-05-08 00:00:00','1949-10-01 00:00:00','Deutschland (occupied)' from dual union all
select 'DYN','Dynacon Inc. Space Division','business','','CA','Toronto:Mississauga, Ontario','1975-01-01 00:00:00','2009-01-01 00:00:00','Dynacon Inc. Space Division' from dual union all
select 'DYNT','Dynetics','business','','US','Huntsville, Alabama','2008-01-01 00:00:00','','Dynetics' from dual union all
select 'DZ','Algeria','','','DZ','Al Jazair','1962-07-05 00:00:00','','الجمهورية الجزائرية الديمقراطية الشعبية' from dual union all
select 'E','Spain','','','E','Espana','1580-01-01 00:00:00','','Reino de España' from dual union all
select 'EAC','Environmental Aeroscience Corporation','business','','US','Miami, Florida','1994-01-01 00:00:00','','Environmental Aeroscience Corporation' from dual union all
select 'EADS','European Aeronautic Defense and Space Company EADS N.V.','business','','NL','Schiphol:Rijk, Amsterdam','2000-07-10 00:00:00','2014-01-01 00:00:00','European Aeronautic Defense and Space Company EADS N.V.' from dual union all
select 'EADSA','EADS Astrium Aquitaine','business','EADS','F','Bordeaux:St-Medard-en-Jalles','2000-07-10 00:00:00','2014-01-01 00:00:00','EADS Astrium Aquitaine' from dual union all
select 'EADSB','EADS Bremen','business','EADS','D','Bremen','2003-06-16 00:00:00','2014-01-01 00:00:00','EADS Bremen' from dual union all
select 'EADSCA','EADS Astrium Construcciones Aeronauticas S.A. (CASA) Espacio','business','EADS','E','Madrid:Barajas','2004-01-01 00:00:00','2006-07-01 00:00:00','EADS Construcciones Aeronáuticas SA' from dual union all
select 'EADSG','EADS Launch Vehicles  CSG','business','EADS','F','CSG, Kourou, Guyane Francaise','2000-07-10 00:00:00','2014-01-01 00:00:00','EADS Launch Vehicles - CSG' from dual union all
select 'EADSL','EADS Launch Vehicles (EADS LV)','business','EADS','F','Les Mureaux, Paris, France','2000-07-10 00:00:00','2014-01-01 00:00:00','EADS Launch Vehicles (EADS LV)' from dual union all
select 'EADSO','EADS Ottobrunn','business','','D','Munchen:Ottobrunn','2000-07-01 00:00:00','2006-06-01 00:00:00','EADS Ottobrunn' from dual union all
select 'EARI','Earthi Ltd.','business','','UK','Guildford','2013-01-01 00:00:00','','Earth-i Ltd.' from dual union all
select 'EARTH','Earth (Terra)','','','EARTH','Earth','','','Earth' from dual union all
select 'EASF','EADS Astrium Satellites (D), Friedrichsafen','business','EADS','D','Friedrichsafen','2003-06-16 00:00:00','2014-01-01 00:00:00','EADS Astrium Satellites (D), Friedrichsafen' from dual union all
select 'EC','Ecuador','','','EC','Quito','1830-01-01 00:00:00','','República del Ecuador' from dual union all
select 'ECHO','Echostar Communications Corp','business','','US','Englewood, Colorado','1980-01-01 00:00:00','2007-12-01 00:00:00','Echostar Communications Corp' from dual union all
select 'ECHOC','Echostar Corp','business','','US','Englewood, Colorado','2007-12-01 00:00:00','','Echostar Corp' from dual union all
select 'ECHOM','Echostar Mobile','business','ECHO','IE','Dublin','2015-03-01 00:00:00','','Solaris Mobile Ltd.' from dual union all
select 'ECLIP','Ecliptic Enterprises','business','','US','Pasadena, California','2001-01-01 00:00:00','','Ecliptic Enterprises' from dual union all
select 'ECOM','US Army Electronics Command','defense','USA','US','Ft Monmouth, NJ','1962-08-01 00:00:00','1978-01-01 00:00:00','US Army Electronics Command' from dual union all
select 'EDLS','Electronics Defense LaboratorySylvania','business','','US','San Jose:Mountain View, California','1953-01-01 00:00:00','','Electronics Defense Laboratory-Sylvania' from dual union all
select 'EE','Estonia','','','EE','Tallinn','1991-01-01 00:00:00','','Eesti Vabariik' from dual union all
select 'EELEC','English Electric Co., Stevenage','business','','UK','Stevenage','1953-01-01 00:00:00','1960-01-01 00:00:00','English Electric Co., Stevenage' from dual union all
select 'EER','EER Systems Inc. Aerospace Div.','business','','US','Seabrook, Maryland','1990-12-01 00:00:00','2001-01-01 00:00:00','EER Systems Inc. Aerospace Div.' from dual union all
select 'EERSSI','EER Systems Inc. SSI Division','business','','US','Houston, Texas','1990-12-01 00:00:00','2001-01-01 00:00:00','EER Systems Inc. SSI Division' from dual union all
select 'EG','Egypt','','','EG','al-Qahira, Misr','1922-02-22 00:00:00','','جمهورية مصر العربية' from dual union all
select 'EGA','Egyptian Army, Missile Division','defense','','EG','Jabal Hamzah, Egypt','1960-01-01 00:00:00','','Egyptian Army, Missile Division' from dual union all
select 'EGAO','Egyptian General Aero Organization','civilian','RKKE','EG','Al Qahira','1960-01-01 00:00:00','1967-01-01 00:00:00','Egyptian General Aero Organization' from dual union all
select 'EGS','EnergiyaGPISpace','business','GPI','GE','Tbilisi','1998-01-01 00:00:00','','Energiya-GPI-Space' from dual union all
select 'EGSA','European GNSS Supervisory Authority','civilian','','I-EU','Paris','2004-01-01 00:00:00','2010-01-01 00:00:00','European GNSS Supervisory Authority' from dual union all
select 'EHNS','Echostar/Hughes Satellite Systems Corp','business','ECHO','US','Germantown, Maryland','2011-01-01 00:00:00','','Echostar/Hughes Satellite Systems Corp' from dual union all
select 'EIAST','Emirates Institution for Advanced Science and Technology','civilian','','UAE','Dubai','2008-01-01 00:00:00','','Emirates Institution for Advanced Science and Technology' from dual union all
select 'ELDE','Elecnor Deimos (Tres Cantos)','business','','E','Madrid:Tres Cantos','2009-09-01 00:00:00','','Elecnor Deimos (Tres Cantos)' from dual union all
select 'ELDEC','Elecnor Deimos (CastillaLa Mancha)','business','','E','Puertollano, Castilla:La Mancha','2010-09-01 00:00:00','2015-08-01 00:00:00','Elecnor Deimos (Castilla-La Mancha)' from dual union all
select 'ELDECI','Deimos Imaging, Elecnor Deimos','business','ELDE','E','Boecillo, Valladolid','2009-09-01 00:00:00','2015-08-01 00:00:00','Deimos Imaging, Elecnor Deimos' from dual union all
select 'ELDO','European Launcher Development Organization','civilian','I-ELDO','I-ELDO','Paris','1962-01-01 00:00:00','1975-01-01 00:00:00','Centre européen pour la construction de lanceurs d''engins spatiaux' from dual union all
select 'ELLT','Elliott Bros (London) Space and Weapon Research Labs (Marconi)','business','','UK','Frimley, Surrey','1804-01-01 00:00:00','1966-01-01 00:00:00','Elliott Bros (London) Space and Weapon Research Labs (Marconi)' from dual union all
select 'ELUS','Energia Logistics Ltd, (ELUS)','business','RKKE','US','Long Beach, California','2010-01-01 00:00:00','','Energia Logistics Ltd, (ELUS)' from dual union all
select 'ELV','ELV SpA','business','','I','Colleferro','2008-01-01 00:00:00','','ELV SpA' from dual;
insert into ORGANIZATION
select 'ELYS','Elysium Space','business','','US','San Francisco, California','2015-01-01 00:00:00','','Elysium Space' from dual union all
select 'EMASH','NPO Energomash','business','','RU','Moskva:Khimki','1990-01-01 00:00:00','','НПО Энергомаш имени академика В. П. Глушко' from dual union all
select 'EMBR','Embratel (Empresa Brasileira de Telecomunicacoes) Satellite Div','civilian','TELB','BR','Rio de Janeiro','1965-01-01 00:00:00','1998-01-01 00:00:00','Embratel, Empresa Brasileira de Telecomunicações S/A.' from dual union all
select 'EMBRM','Embratel (Empresa Brasileira de Telecomunicacoes) Satellite Div','civilian','MCI','BR','Rio de Janeiro','1998-01-01 00:00:00','2003-01-01 00:00:00','Embratel, Empresa Brasileira de Telecomunicações S/A.' from dual union all
select 'EMBRT','Embratel (Empresa Brasileira de Telecomunicacoes) Satellite Div','civilian','TELMX','BR','Rio de Janeiro','2003-01-01 00:00:00','','Embratel, Empresa Brasileira de Telecomunicações S/A.' from dual union all
select 'EMST','EMS Technologies (Canada)','business','','CA','Montreal:St-Anne-de-Bellevue, Quebec','1999-01-01 00:00:00','2005-01-01 00:00:00','EMS Technologies (Canada)' from dual union all
select 'ENDUR','EnduroSat','academic/non-profit','','BGN','Sofia','2015-01-01 00:00:00','','EnduroSat' from dual union all
select 'EOS','ElectroOptical Systems, Inc.','business','','US','Pasadena, California','','1980-01-01 00:00:00','Electro-Optical Systems, Inc.' from dual union all
select 'EOSAT','EOSAT Inc (Earth Observation Satellite Company; Hughes/RCA)','business','','US','Washington, DC/Lanham, Md.','1985-01-01 00:00:00','1996-11-01 00:00:00','EOSAT Inc (Earth Observation Satellite Company; Hughes/RCA)' from dual union all
select 'EOSAU','ElectroOptical Systems Pty Ltd.','business','','AU','Queanbeyan, New South Wales','1983-01-01 00:00:00','','Electro-Optical Systems Pty Ltd.' from dual union all
select 'EPFL','Ecole Polytechnique F\''ed\''erale de Lausanne, Ecublens campus','academic/non-profit','','CH','Lausanne:Ecublens','1978-01-01 00:00:00','','École Polytechnique Fédérale de Lausanne, Écublens' from dual union all
select 'ERAU','EmbryRiddle Aeronautical University (ERAU)','academic/non-profit','','US','Daytona Beach, Florida','1965-01-01 00:00:00','','Embry-Riddle Aeronautical University (ERAU)' from dual union all
select 'ERDA','Energy Research and Development Agency','defense','','US','Washington, DC','1975-01-19 00:00:00','1977-08-04 00:00:00','Energy Research and Development Agency' from dual union all
select 'ERDL','US Army Engineer Research and Development Lab','defense','','US','Springfield:Ft Belvoir, Virginia','1942-01-01 00:00:00','','US Army Engineer Research and Development Lab' from dual union all
select 'ERNO','ERNOVFW (Entwicklungsring Nord)','business','VFW','D','Bremen','1961-01-01 00:00:00','1969-01-01 00:00:00','ERNO-VFW (Entwicklungsring Nord)' from dual union all
select 'ERNOF','ERNOVFW Fokker (Entwicklungsring Nord)','business','VFWF','D','Bremen','1969-01-01 00:00:00','1980-01-01 00:00:00','ERNO-VFW Fokker (Entwicklungsring Nord)' from dual union all
select 'ERNOV','ERNOVFW (Entwicklungsring Nord)','business','VFW','D','Bremen','1980-01-01 00:00:00','1981-01-01 00:00:00','ERNO-VFW (Entwicklungsring Nord)' from dual union all
select 'ERTU','Egyptian Radio and Television Union','civilian','EG','EG','6th of October City','1960-01-01 00:00:00','','اتحاد الاذاعة و التليفزيون المصري‎' from dual union all
select 'ESA','European Space Agency','civilian','','I-ESA','Paris','1975-04-30 00:00:00','','European Space Agency' from dual union all
select 'ESCN','Canary Is','','E','ESCN','Santa Cruz de Tenerife','1479-01-01 00:00:00','','Islas Canarias' from dual union all
select 'ESHAIL','Es''hailsat, Qatar Satellite Compnay','academic/non-profit','ICTQ','QA','Doha','2010-01-01 00:00:00','','Es''hailsat, Qatar Satellite Compnay' from dual union all
select 'ESIE','ESIEESPACE, Ecole Sup. d''Ingenieurs en Electron. et Electrotechnique','academic/non-profit','TARTU','F','Paris','1990-01-01 00:00:00','','ESIEESPACE, École Supérieure d''Ingénieurs en Électronique et Électrotechnique' from dual union all
select 'ESL','Electromagnetic Systems Laboratory, Inc','business','','US','Palo Alto, California','1964-01-01 00:00:00','1969-01-01 00:00:00','Electronic Systems Lab. Inc.' from dual union all
select 'ESL1','Electromagnetic Systems Laboratory, Inc','business','','US','Sunnyvale, California','1969-01-01 00:00:00','1978-01-01 00:00:00','Electronic Systems Lab. Inc.' from dual union all
select 'ESRO','European Space Research Organization','civilian','','I-ESRO','Paris:Neuilly:sur:Seine','1962-01-01 00:00:00','1975-04-30 00:00:00','Conseil Europeen de Recherches Spatiales' from dual union all
select 'ESRWA','Earth and Space Research','civilian','','US','Seattle, Washington','','','Earth and Space Research' from dual union all
select 'ESSA','Environmental Science Services Administration','civilian','','US','Suitland, Maryland','1965-01-01 00:00:00','1970-10-01 00:00:00','Environmental Science Services Administration' from dual union all
select 'ESSP','Estonian Student Satellite Program','academic/non-profit','','EE','Tartu','2012-01-01 00:00:00','','Eesti tudengisatelliidi programm' from dual union all
select 'ESTEC','ESTEC','civilian','I-ESA','I-ESA','Noordwijik, Nederland','1964-01-01 00:00:00','','ESTEC' from dual union all
select 'ESYS','ESystems Inc.','business','','US','Dallas:Garland, Texas','1972-01-01 00:00:00','1995-01-01 00:00:00','E-Systems Inc' from dual union all
select 'ETAC','Environmental Technical Applications Center, USAF','defense','USAF','US','Washington, DC','1964-12-01 00:00:00','1975-08-01 00:00:00','Environmental Technical Applications Center, USAF' from dual union all
select 'ETACIL','Environmental Technical Applications Center, USAF','defense','USAF','US','St. Louis:Scott AFB, Illinois','1975-08-01 00:00:00','1995-10-01 00:00:00','Environmental Technical Applications Center, USAF' from dual union all
select 'ETEC','Escola Municipal Presidente Tancredo de Almeida Neves','academic/non-profit','','BR','Ubatuba, Sao Paolo','','','Escola Municipal Presidente Tancredo de Almeida Neves' from dual union all
select 'EUMET','European Meteorological Satellite Organization','civilian','I-EUM','I-EUM','Darmstadt','1986-01-01 00:00:00','','European Meteorological Satellite Organization' from dual union all
select 'EUPST','EuropeStar Ltd.','business','','UK','Uxbridge, Middlesex','1998-01-01 00:00:00','2005-07-01 00:00:00','EuropeStar Ltd.' from dual union all
select 'EURAS','Eurasspace Gesellschaft fur Raumfahrttechnik mbH','business','','D','Munchen','1994-01-01 00:00:00','','EURASSPACE Gesellschaft für Raumfahrttechnik mbH' from dual union all
select 'EUROK','Eurockot Launch Services GmbH (D for RU lanch vehicle)','business','','RU','Bremen','1995-01-01 00:00:00','','Eurockot Launch Services GmbH (D for RU lanch vehicle)' from dual union all
select 'EUROL','Eurolaunch (DLR/SSC)','business','SSC','S','Kiruna','2003-12-01 00:00:00','','Eurolaunch (DLR/SSC)' from dual union all
select 'EUROP','Europropulsion (SEP/BPD)','business','','F','Suresnes','1991-01-01 00:00:00','','Europropulsion (SEP/BPD)' from dual union all
select 'EUROPK','Europropulsion (SEP/BPD)','business','EUROP','F','Kourou','1991-01-01 00:00:00','','Europropulsion (SEP/BPD)' from dual union all
select 'EURSA','Eurasiasat','business','','MC','Monaco','1996-01-01 00:00:00','','Eurasiasat' from dual union all
select 'EURSG','EurasiasatGolbasi (Satellite control center)','business','EURSA','TR','Golbasi','1996-01-01 00:00:00','','Eurasiasat-Golbasi (Satellite control center)' from dual union all
select 'EUTAM','Eutelsat Americas','business','EUTSA','MX','Ciudad de Mexico:Lomas de Sotelo','2014-03-07 00:00:00','','Eutelsat Americas' from dual union all
select 'EUTEL','European Telecommunications Satellite Organization','civilian','I-EUT','I-EUT','Paris','1977-01-01 00:00:00','2001-07-01 00:00:00','European Telecommunications Satellite Organization' from dual union all
select 'EUTSA','Eutelsat SA','business','','F','Paris','2001-07-02 00:00:00','','Eutelsat SA' from dual union all
select 'EWAT','Earth Watch Inc','business','','US','Longmont, Colorado','1995-01-01 00:00:00','2002-01-01 00:00:00','Earth Watch Inc' from dual union all
select 'EXA','Ecuadorian Civilian Space Agency','civilian','','EC','Guayaquil','2007-11-01 00:00:00','','Agencia Especial Civil Ecuatoriana' from dual union all
select 'EXEA','exactEarth Ltd.','business','COMDEV','CA','Cambridge, Ontario','2009-01-01 00:00:00','','exactEarth Ltd.' from dual union all
select 'EXOSA','Exos Aerospace','business','','US','Dallas:Caddo Mills, Texas','2015-01-01 00:00:00','','Exos Aerospace' from dual union all
select 'EXPACE','Expace','business','CASIC','CN','Beijing:Haidian','2016-01-01 00:00:00','','航天科工火箭技术有限公司' from dual union all
select 'F','France','','','F','France','0509-01-01 00:00:00','','République française' from dual union all
select 'FACH','Fuerza A\''{e}rea de Chile, ECS Fasat, Los Cerrillos AFB','defense','','CL','Santiago:Los Cerrillos','1930-01-01 00:00:00','','Fuerza Aérea de Chile, Aeródromo Los Cerrillos' from dual union all
select 'FAI','Final Analysis Inc','business','','US','Greenbelt, Maryland','1993-01-01 00:00:00','2001-01-01 00:00:00','Final Analysis Inc' from dual union all
select 'FAIR','Fairchild Space and Electronics','business','','US','Germantown, Maryland','1971-01-01 00:00:00','1994-01-01 00:00:00','Fairchild Space and Electronics' from dual union all
select 'FAIRA','Fairchild Aircraft Division','business','','US','Hagerstown, Maryland','1950-01-01 00:00:00','1961-01-01 00:00:00','Fairchild Aircraft Division' from dual union all
select 'FAIRH','Fairchild Hiller Space Systems Division','business','','US','Hagerstown, Maryland','1964-01-01 00:00:00','1971-01-01 00:00:00','Fairchild Hiller Space Systems Division' from dual union all
select 'FAIRHG','Fairchild Hiller Space Systems (Rockville/Germantown)','business','','US','Germantown, Maryland','1964-01-01 00:00:00','1971-01-01 00:00:00','Fairchild Hiller Space Systems (Rockville/Germantown)' from dual union all
select 'FAIRS','Fairchild Stratos Space Systems Division','business','','US','Hagerstown, Maryland','1961-01-01 00:00:00','1964-01-01 00:00:00','Fairchild Stratos Space Systems Division' from dual union all
select 'FAKEL','OKB Fakel','business','','SU','Kaliningrad','1971-01-01 00:00:00','2014-03-01 00:00:00','ОКБ "Факел"' from dual union all
select 'FAKELO','OKB Fakel, fil. ORKK','business','ORKK','RU','Kaliningrad','2014-03-01 00:00:00','','ОКБ "Факел"' from dual union all
select 'FASTC','USAF Foreign Aerospace Science Technology Center (FTD)','defense','USAF','US','Wright-Patterson AFB, Dayton, Ohio','1961-01-01 00:00:00','1993-10-01 00:00:00','USAF Foreign Aerospace Science Technology Center (FTD)' from dual union all
select 'FATAH','Al Fatah Co., Baghdad','business','','IQ','Baghdad','1999-01-01 00:00:00','2003-01-01 00:00:00','Al Fatah Co., Baghdad' from dual union all
select 'FCCO','4C Polito Space, 4C Controls','business','','I','Torino','2009-01-01 00:00:00','','4C Polito Space, 4C Controls' from dual union all
select 'FHWN','University of Applied Sciences Wiener Neustadt','academic/non-profit','','AT','Wiener Neustadt','1994-01-01 00:00:00','','Fachhochschule Wiener Neustadt' from dual union all
select 'FI','Finland','','','FI','Helsinki','1917-12-06 00:00:00','','Suomen tasavalta' from dual union all
select 'FIAN','Lebedev Physical Inst','academic/non-profit','AN','RU','Moskva','1934-01-01 00:00:00','','Физический институт имени П.Н.Лебедева (ФИАН)' from dual union all
select 'FIAT','Fiat','business','','I','Torino','1899-07-01 00:00:00','1969-01-01 00:00:00','Fiat' from dual union all
select 'FIATA','FiatAvio spA','business','','I','Colleferro','1994-01-01 00:00:00','2003-01-01 00:00:00','FiatAvio spA' from dual union all
select 'FING','Faculty of Engineering, University of the Republic','academic/non-profit','','UY','Montevideo:Parque Rodo','1888-01-01 00:00:00','','Facultad de Ingeniería (Universidad de la República)' from dual union all
select 'FIPM','FraunhoferInstitut fur Physikalische Messtechnik','academic/non-profit','','D','Freiburg im Breslau','1980-01-01 00:00:00','','Fraunhofer-Institut für Physikalische Messtechnik' from dual union all
select 'FIRES','Firestone Guided Missile Division Los Angeles plant','business','','US','Los Angeles:South Gate, California','1954-01-01 00:00:00','1962-01-01 00:00:00','Firestone Guided Missile Division Los Angeles plant' from dual union all
select 'FIRETR','Firestone Tire and Rubber','business','','US','Akron, Ohio??','1951-01-01 00:00:00','1954-01-01 00:00:00','Firestone Tire and Rubber' from dual union all
select 'FIT','Florida Inst of Technology','academic/non-profit','','US','Melbourne, Florida','1966-01-01 00:00:00','','Florida Inst of Technology' from dual union all
select 'FKA','Roskosmos','civilian','','RU','Moskva','2004-01-01 00:00:00','2016-01-01 00:00:00','Федеральное космическое агентство (Роскосмос)' from dual union all
select 'FKG','Flugkorpergeschwader 1 or 2 (unknown)','defense','LW','D','Landsberg am Lech, Bayern','1964-01-01 00:00:00','1991-01-01 00:00:00','Flugkorpergeschwader 1 or 2 (unknown)' from dual union all
select 'FKG1','Flugkorpergeschwader 1','defense','LW','D','Landsberg am Lech, Bayern','1964-01-01 00:00:00','1991-01-01 00:00:00','Flugkorpergeschwader 1' from dual union all
select 'FKG2','Flugkorpergeschwader 2','defense','LW','D','Geilenkirchen, Nordrhein:Westphalen','1964-01-01 00:00:00','1991-01-01 00:00:00','Flugkorpergeschwader 2' from dual union all
select 'FKGR11','Flugkorpergruppe 11, FKG 1','defense','LW','D','Landsberg am Lech, Bayern','1964-01-01 00:00:00','1991-01-01 00:00:00','Flugkorpergruppe 11, FKG 1' from dual union all
select 'FKGR12','Flugkorpergruppe 12, FKG 1','defense','LW','D','Landsberg am Lech, Bayern','1964-01-01 00:00:00','1991-01-01 00:00:00','Flugkorpergruppe 12, FKG 1' from dual union all
select 'FKGR13','Flugkorpergruppe 13, FKG 1','defense','LW','D','Landsberg am Lech, Bayern','1964-01-01 00:00:00','1991-01-01 00:00:00','Flugkorpergruppe 13, FKG 1' from dual union all
select 'FKGR21','Flugkorpergruppe 21, FKG 2','defense','LW','D','Geilenkirchen, Nordrhein:Westphalen','1964-01-01 00:00:00','1991-01-01 00:00:00','Flugkorpergruppe 21, FKG 2' from dual union all
select 'FKGR22','Flugkorpergruppe 22, FKG 2','defense','LW','D','Geilenkirchen, Nordrhein:Westphalen','1964-01-01 00:00:00','1991-01-01 00:00:00','Flugkorpergruppe 22, FKG 2' from dual union all
select 'FLASP','Florida Space Authority','civilian','','US','Kennedy Space Center, Florida','2000-01-01 00:00:00','2006-05-01 00:00:00','Florida Space Authority' from dual union all
select 'FLSPA','Space Florida','civilian','','US','Kennedy Space Center, Florida','2006-05-01 00:00:00','','Space Florida' from dual union all
select 'FMI','DALO','defense','','DK','Kobenhavn:Ballerup','2014-01-01 00:00:00','','Forsvarets Materiel- og Indkøbsstyrelse' from dual union all
select 'FOKK','Fokker Aircraft','business','','NL','Amsterdam','1919-01-01 00:00:00','1996-01-01 00:00:00','Fokker Aircraft' from dual union all
select 'FOKKS','Fokker Space','business','FOKK','NL','Amsterdam','1970-01-01 00:00:00','1996-01-01 00:00:00','Fokker Space' from dual union all
select 'FOKKSS','Fokker Space and Systems','business','','NL','Amsterdam','1996-01-01 00:00:00','2002-01-01 00:00:00','Fokker Space and Systems' from dual union all
select 'FORDA','Ford Aerospace, Palo Alto','business','','US','Palo Alto, California','1976-12-01 00:00:00','1990-10-01 00:00:00','Ford Aerospace, Palo Alto' from dual union all
select 'FORDAN','Ford Motor Co. Aeronutronic Div.','business','','US','Los Angeles:Glendale, California','1956-01-01 00:00:00','1960-01-01 00:00:00','Ford Motor Co. Aeronutronic Div.' from dual union all
select 'FORDP','Aeronutronic Ford Western Development Labs','business','','US','Palo Alto, California','1975-01-01 00:00:00','1976-12-01 00:00:00','Aeronutronic Ford Western Development Labs' from dual union all
select 'FORNB','Aeronutronic Ford, Newport Beach','business','','US','Newport Beach, California','1975-01-01 00:00:00','','Aeronutronic Ford, Newport Beach' from dual union all
select 'FORNBA','Ford Aeronutronic Division','business','','US','Newport Beach, California','1960-01-01 00:00:00','1962-01-01 00:00:00','Ford Aeronutronic Division' from dual union all
select 'FORSK','Research Council of Norway','civilian','','N','Oslo','1993-01-01 00:00:00','','Norges Forskingradet' from dual union all
select 'FPTU','F Space Lab, Financing and Promoting Technology Corp. University','academic/non-profit','','VN','Ha Noi','2006-09-01 00:00:00','','Trường Đại học FPT' from dual union all
select 'FRANK','Frankfurt University','academic/non-profit','','D','Frankfurt am Main','1912-01-01 00:00:00','','Johann Wolfgang Goethe-Universität Frankfurt am Main' from dual union all
select 'FRASC','CNRFrascati','academic/non-profit','','I','Frascati','','','CNR-Frascati' from dual union all
select 'FRC','NASA Flight Research Center, Edwards AFB','civilian','NASA','US','Edwards, California','1958-01-01 00:00:00','1976-03-26 00:00:00','NASA Flight Research Center, Edwards AFB' from dual union all
select 'FREIB','Freiburg University','academic/non-profit','','D','Freiburg im Breisgau','1457-01-01 00:00:00','','Albert-Ludwigs-Universität Freiburg' from dual union all
select 'FRMET','MeteoFrance','civilian','','F','Paris','1993-01-01 00:00:00','','Météo-France' from dual;
insert into ORGANIZATION
select 'FSB','Federal''naya sluzhba bezopasnosti Rossiyskoy Federatsii','defense','','RU','Moskva','1995-01-01 00:00:00','','Федеральная служба безопасности Российской Федерации' from dual union all
select 'FSU','Florida State University','academic/non-profit','','US','Tallahassee, Florida','1851-01-01 00:00:00','','Florida State University' from dual union all
select 'FTB','Arny Air Artillery Defense School, Fort Bliss, Texas','defense','','US','Fort Bliss, Texas','1945-01-01 00:00:00','','Arny Air Artillery Defense School, Fort Bliss, Texas' from dual union all
select 'FTEL','France Telecom','business','','F','Paris','1988-01-01 00:00:00','2013-07-01 00:00:00','France Télécom S.A.' from dual union all
select 'FUCHS','FuchsGruppe','business','','D','Bremen','1985-01-01 00:00:00','','Fuchs-Gruppe' from dual union all
select 'FUDAN','Fudan University','academic/non-profit','','CN','Shanghai','1905-01-01 00:00:00','','复旦大学' from dual union all
select 'FUKU','Fukuoka Institute of Technology','academic/non-profit','','J','Fukuoka','1954-01-01 00:00:00','','福岡工業大学' from dual union all
select 'FUSEI','Fuji Precision Industries Co Ltd','business','','J','Tokyo:Suginami (Ogikubo)','1950-01-01 00:00:00','1961-01-01 00:00:00','富士精密工業（株）' from dual union all
select 'FUTA','Federal University of Technology Akure','academic/non-profit','','NG','Akure, Ondo','1981-01-01 00:00:00','','Federal University of Technology Akure' from dual union all
select 'G2P','G2P (SNECMA Propulsion Solide/SNPEME)','business','SNECPS','F','Bordeaux:Le Haillan','2000-01-01 00:00:00','','G2P (SNECMA Propulsion Solide/SNPE-ME)' from dual union all
select 'G2SS','Panamsat G2 Satellite Solutions','business','','US','Washington, DC','2004-01-01 00:00:00','2006-07-03 00:00:00','Panamsat G2 Satellite Solutions' from dual union all
select 'GACO','General Atomics Electromagnetic System Group/Englewood','business','GATOM','US','Denver:Englewood, Colorado','2017-11-13 00:00:00','','General Atomics Electromagnetic Systems Group/Englewood' from dual union all
select 'GAHSV','General Atomics, Huntsville','business','GATOM','US','Huntsville, Alabama','2016-05-01 00:00:00','','General Atomics, Huntsville' from dual union all
select 'GAISH','Shternberg Astronomical Institute, MGU','academic/non-profit','MGU','RU','Moskva:MGU','1831-01-01 00:00:00','','Государственный астрономический институт имени Штернберга' from dual union all
select 'GAREF','Groupe Amateur de Recherches et d''\''Etudes par Fus\''ees XV','academic/non-profit','','F','Paris','1964-01-01 00:00:00','','Groupe Amateur de Recherches et d''Études par Fusées XV' from dual union all
select 'GASL','GASL, Inc.','business','','US','Ronkonkoma, New Yrok','1956-01-01 00:00:00','2000-01-01 00:00:00','GASL, Inc.' from dual union all
select 'GASLA','GASL, Inc. Allied Aerospace','business','ALAER','US','Ronkonkoma, New Yrok','2000-01-01 00:00:00','2003-01-01 00:00:00','GASL, Inc. Allied Aerospace' from dual union all
select 'GATCO','General Atomic Co.','business','','US','San Diego, California','1973-01-01 00:00:00','1982-01-01 00:00:00','General Atomic Co.' from dual union all
select 'GATGD','General Dynamics General Atomics Division','business','','US','San Diego, California','1955-07-01 00:00:00','1967-01-01 00:00:00','General Dynamics General Atomics Division' from dual union all
select 'GATGU','Gulf General Atomic','business','','US','San Diego, California','1967-01-01 00:00:00','1973-01-01 00:00:00','Gulf General Atomic' from dual union all
select 'GATGU2','GA Technologies Inc.','business','','US','San Diego, California','1982-01-01 00:00:00','1986-01-01 00:00:00','GA Technologies Inc' from dual union all
select 'GATOM','General Atomics','business','','US','San Diego, California','1986-01-01 00:00:00','','General Atomics' from dual union all
select 'GAU','Main Artillery Dept','defense','MVS','SU','Moskva','','','Главное артиллерийское управление' from dual union all
select 'GAU4','4th Directorate, Main Artillery Dept','defense','GAU','SU','Moskva','1946-01-01 00:00:00','1953-04-01 00:00:00','4-го управления ГАУ ВС СССР' from dual union all
select 'GAUSS','Group of Astrodynamics for the Use of Space Systems, GAUSS SrI','business','','I','Roma','2011-01-01 00:00:00','','Group of Astrodynamics for the Use of Space Systems, GAUSS SrI' from dual union all
select 'GAZ1','State Aviation Plant No. 1','business','','SU','Samara','1941-01-01 00:00:00','1961-01-01 00:00:00','Государственный авиационный завод №.1' from dual union all
select 'GAZC','OAO Gazkom','business','','RU','Moskva:Korolev, Rossiya','1992-01-01 00:00:00','2008-12-01 00:00:00','ОАО «Газком»' from dual union all
select 'GAZS','Gazprom Space Systems','business','','RU','Moskva:Korolev, Rossiya','2008-12-01 00:00:00','','ОАО «Газпром космические системы»' from dual union all
select 'GB5','5th Academy','civilian','CN','CN','Beijing','1956-10-08 00:00:00','1964-11-23 00:00:00','国防部第五研究院' from dual union all
select 'GCA','Geophysics Corporation of America','business','','US','Boston, Mass.','1958-01-01 00:00:00','','Geophysics Corporation of America' from dual union all
select 'GCDX','State Surveying and Mapping Geographic Information Bureau','civilian','GZB','CN','Beijing','1959-01-01 00:00:00','','国家测绘地理信息局' from dual union all
select 'GCR','Grand Central Rocket, Redlands','business','','US','Redlands, California','1954-01-01 00:00:00','1961-01-01 00:00:00','Grand Central Rocket, Redlands' from dual union all
select 'GCRG','Grand Central Rocket, Grand Central Airport','business','','US','Los Angeles:Glendale, California','1954-01-01 00:00:00','1961-01-01 00:00:00','Grand Central Rocket, Grand Central Airport' from dual union all
select 'GD','General Dynamics','business','','US','West Falls Church, Virginia','','','General Dynamics' from dual union all
select 'GDA','General Dynamics Astronautics','business','GD','US','San Diego:Kearny Mesa, California','1961-06-01 00:00:00','1965-02-01 00:00:00','General Dynamics Astronautics' from dual union all
select 'GDC','General Dynamics/Convair Division  (Plant 19)','business','GD','US','San Diego:SAN, California','1953-03-01 00:00:00','1985-03-01 00:00:00','General Dynamics/Convair Division  (Plant 19)' from dual union all
select 'GDCKM','General Dynamics/Convair Division  (Plant 71)','business','GD','US','San Diego:Kearny Mesa, California','1965-02-01 00:00:00','1985-03-01 00:00:00','General Dynamics/Convair Division  (Plant 71)' from dual union all
select 'GDCLS','General Dynamics Commercial Launch Service','business','GD','US','San Diego, California','1988-01-01 00:00:00','1994-05-01 00:00:00','General Dynamics Commercial Launch Service' from dual union all
select 'GDG','General Dynamics C4 Systems (Adv Info Systems)','business','GD','US','Gilbert, Arizona','2004-01-01 00:00:00','2010-04-01 00:00:00','General Dynamics C4 Systems (Adv Info Systems)' from dual union all
select 'GDOTS','General Dynamics Ordnance and Tactical Systems','business','GD','US','Redmond, Washington','2001-01-01 00:00:00','2002-08-01 00:00:00','General Dynamics Ordnance and Tactical Systems' from dual union all
select 'GDPO','General Dynamics Convair  Pomona','business','GD','US','Pomona, California','1954-04-01 00:00:00','1992-05-01 00:00:00','General Dynamics Convair - Pomona' from dual union all
select 'GDSSD','General Dynamics Space Systems Division','business','GD','US','San Diego:Kearny Mesa, California','1985-03-01 00:00:00','1994-05-02 00:00:00','General Dynamics Space Systems Division' from dual union all
select 'GE','Georgia','','','GE','Tbilist','1991-12-25 00:00:00','','საქართველო' from dual union all
select 'GECAP','GE Capital Satellites (Gibraltar) Ltd.','business','GE','UK','Gibraltar','1997-01-01 00:00:00','2001-01-01 00:00:00','GE Capital Satellites (Gibraltar) Ltd.' from dual union all
select 'GECO','General Electric Co.','business','','US','Schenectady, New York','1878-01-01 00:00:00','','General Electric Co.' from dual union all
select 'GEEW','GE Astro Space, East Windsor','business','GE','US','East Windsor, New Jersey','1986-01-01 00:00:00','1993-01-01 00:00:00','GE Astro Space, East Windsor' from dual union all
select 'GEMSD','GE Missles and Space Division','business','GE','US','Philadelphia, Pennsylvania','1956-01-01 00:00:00','1961-01-01 00:00:00','GE Missles and Space Division' from dual union all
select 'GEMTS','GE Malta Rocket Test Station','business','GESNY','US','Malta, New York','1945-01-01 00:00:00','1984-01-01 00:00:00','GE Malta Rocket Test Station' from dual union all
select 'GENC','GenCorp.','business','','US','Cleveland, Ohio','1984-01-01 00:00:00','2015-04-27 00:00:00','GenCorp.' from dual union all
select 'GEOOPT','GeoOptics Inc.','business','','US','Pasadena, California','2015-01-01 00:00:00','','GeoOptics Inc.' from dual union all
select 'GEOST','Geostar Corp.','business','','US','Washington, DC','1983-03-01 00:00:00','1991-05-01 00:00:00','Geostar Corp.' from dual union all
select 'GERSD','GE Reentry Systems Div','business','GE','US','Philadelphia, Pennsylvania','1961-01-01 00:00:00','1993-01-01 00:00:00','GE Reentry Systems Div' from dual union all
select 'GESNY','General Electric Co.','business','GE','US','Schenectady, New York','','','General Electric Co.' from dual union all
select 'GESPC','GE Americom/SpacenetMcLean','business','AMC','US','McLean, Virginia','1994-07-01 00:00:00','1995-01-01 00:00:00','GE Americom/Spacenet-McLean' from dual union all
select 'GEVF','GE Missiles and Space Vehicles Division (MSVD)','business','GE','US','Philadelphia:King of Prussia/Valley Forge, Penn.','1961-01-01 00:00:00','1993-01-01 00:00:00','GE Missiles and Space Vehicles Division (MSVD)' from dual union all
select 'GEYE','GeoEye Inc. (on Orbital campus)','business','','US','Dulles, Virginia','2006-01-01 00:00:00','2011-01-01 00:00:00','GeoEye Inc. (on Orbital campus)' from dual union all
select 'GEYEH','GeoEye Inc.','business','','US','Herndon, Virginia','2011-01-01 00:00:00','2012-07-01 00:00:00','GeoEye Inc.' from dual union all
select 'GEYET','GeoEye Inc. (Thornton)','business','','US','Thornton, Colorado','2006-01-01 00:00:00','2012-07-01 00:00:00','GeoEye Inc. (Thornton)' from dual union all
select 'GFW','Gesellschaft fur Weltraumforschung','civilian','BMFT','D','Bonn','1962-08-01 00:00:00','1975-01-01 00:00:00','Gesellschaft für Weltraumforschung' from dual union all
select 'GFZ','Geoforschungszentrum Potzdam','academic/non-profit','','D','Berlin:Potsdam','1992-01-01 00:00:00','','Geoforschungszentrum Potzdam' from dual union all
select 'GGPEN','National Space Program Management','civilian','','AO','Luanda','2016-01-01 00:00:00','','Gabinete de Gestão do Programa Espacial Nacional' from dual union all
select 'GH','Ghana','','','GH','Accra','1957-03-01 00:00:00','','Republic of Ghana' from dual union all
select 'GHALAM','JV Ghalam LLP','business','','KZ','Astana','2010-01-01 00:00:00','','JV Ghalam LLP' from dual union all
select 'GHG','GHGSat Inc.','business','','CA','Montreal','2015-01-01 00:00:00','','GGHSat Inc.' from dual union all
select 'GHJ','China State Oceanic Administration','civilian','','CN','Beijing?','','','国家海洋局' from dual union all
select 'GI','Gibraltar','','','GI','Gibraltar','','','Gibraltar' from dual union all
select 'GICZ','Geophysical Inst. of the Czech Academy of Sciences','business','','CZ','Praha','1920-01-01 00:00:00','','Geofyzikální ústav Akademie věd České republiky' from dual union all
select 'GIMR','US Army Engineer Geodesy, Intelligence and Mapping R and D Agency (under ACE)','defense','USA','US','Fort Belvoir, Virginia','1960-08-02 00:00:00','1967-07-31 00:00:00','US Army Engineer Geodesy, Intelligence and Mapping R and D Agency (under ACE)' from dual union all
select 'GIRD','Group for the Study of Reactive Motion','defense','','SU','Moskva','1931-01-01 00:00:00','1933-01-01 00:00:00','Группа изучения реактивного движения' from dual union all
select 'GISTDA','GeoInformatics and Space Technology Development Agency','civilian','','T','Bangkok','2000-01-01 00:00:00','','Geo-Informatics and Space Technology Development Agency' from dual union all
select 'GIT','Georgia Institute of Technology','academic/non-profit','','US','Atlanta, Georgia','1885-10-01 00:00:00','','Georgia Institute of Technology' from dual union all
select 'GK','Glavkosmos','civilian','','SU','Moskva','1985-01-01 00:00:00','','Главкосмос' from dual union all
select 'GL','Greenland','','','GL','Nuuk (Godthab)','1979-01-01 00:00:00','','Kalaallit Nunaat (Gronland)' from dual union all
select 'GLOB','Globalstar LLC','business','','US','San Jose, California','1991-01-01 00:00:00','','Globalstar LLC' from dual union all
select 'GMS','Meteorological Service','civilian','','SU','Moskva','','1991-01-01 00:00:00','Главное Управление гидрометеорологической службы при СНК СССР' from dual union all
select 'GOBBZ','Institute of Technology "Max Valier", Bozen','academic/non-profit','','I','Bolzano, Alto Adige (Sudtirol)','2012-01-01 00:00:00','','Gewerbeoberschule Max Valier, Bozen' from dual union all
select 'GOBME','Institute of Technology "Oskar von Miller", Meran','academic/non-profit','','I','Merano, Alto Adige (Sudtirol)','2012-01-01 00:00:00','','Gewerbeoberschule Oskar von Miller, Meran' from dual union all
select 'GOCNAE','Space Activities Comission','civilian','','BR','Rio de Janeiro?','1961-01-01 00:00:00','1963-01-01 00:00:00','Grupo de Organização da Comissão Nacional de Atividades Espaciais' from dual union all
select 'GOEOS','Goodrich ElectroOptical Systems (former PerkinElmer)','business','','US','Danbury, Connecticut','2001-01-01 00:00:00','2012-07-01 00:00:00','Goodrich Electro-Optical Systems (former Perkin-Elmer)' from dual union all
select 'GOL2','GITsIP KS (GTsIU153?), Golitsyno2','defense','','SU','Golitsyno-2 (Krasnoznamensk)','','','Голицыно-2' from dual union all
select 'GOMSP','GomSpace','business','','DK','Aalborg, Denmark','2007-01-01 00:00:00','','GomSpace' from dual union all
select 'GOS','German Orbital Systems GmbH','business','','D','Berlin','2015-01-01 00:00:00','','German Orbital Systems GmbH' from dual union all
select 'GOYR','Goodyear Aerospace','business','','US','Akron, Ohio','1939-01-01 00:00:00','1987-01-01 00:00:00','Goodyear Aerospace' from dual union all
select 'GOYRP','Goodyear Aerospace','business','','US','Phoenix, Arizona','1941-01-01 00:00:00','1987-01-01 00:00:00','Goodyear Aerospace' from dual union all
select 'GPEW','Guided Projective Establishment, UK Ministry of Supply','defense','','UK','Westcott, Buckinghamshire','1946-01-01 00:00:00','1947-01-01 00:00:00','Guided Projective Establishment, UK Ministry of Supply' from dual union all
select 'GPI','Georgian Polytechnical Intellect','business','','GE','Tbilisi','1996-08-27 00:00:00','','Грузинский политехнический интеллект' from dual union all
select 'GPKS','Satellite Communications Co.','civilian','','SU','Moskva','1992-01-01 00:00:00','2001-04-19 00:00:00','Государственное предприятие «Космическая связь»' from dual union all
select 'GR','Greece','','','GR','Ellas','1821-03-01 00:00:00','','Ελληνική Δημοκρατία' from dual union all
select 'GRC','NASA Glenn Research Center','civilian','NASA','US','Cleveland, Ohio','1999-03-01 00:00:00','','NASA Glenn Research Center' from dual union all
select 'GRI','Group de Recherches Ionospheriques','civilian','CNET','F','Paris:St Maur, Val-de-Marne','1961-01-01 00:00:00','','Groupe de Recherches Ionosphériques' from dual union all
select 'GROC','Geofysica en Ruimtonderzoek Commissie','civilian','','NL','Leiden?','1960-01-01 00:00:00','1983-01-01 00:00:00','Geofysica en Ruimtonderzoek Commissie' from dual union all
select 'GRU','Glavnoye Razvedyvatelnoe Upravleniye','defense','SOVA','SU','Mosvka NW','1918-01-01 00:00:00','1991-01-01 00:00:00','Главное Разведывательное Управление' from dual union all
select 'GRUM','Grummann Aerospace Corporation','business','','US','Bethpage, New York','1929-12-06 00:00:00','1994-01-01 00:00:00','Grummann Aerospace Corporation' from dual union all
select 'GRUR','Glavnoye Razvedyvatelnoe Upravleniye','defense','MORF','RU','Mosvka NW','1991-01-01 00:00:00','','Главное Разведывательное Управление' from dual union all
select 'GSAEU','European Global Navigation Satellite Systems Agency','civilian','','I-EU','Praha','2010-01-01 00:00:00','','European Global Navigation Satellite Systems Agency' from dual union all
select 'GSFC','NASA Goddard Space Flight Center','civilian','NASA','US','Greenbelt, Maryland','1959-05-01 00:00:00','','NASA Goddard Space Flight Center' from dual union all
select 'GSOC','Deutsches RaumfahrtKontrollzentrum (German Space Ops Center)','civilian','DFVLR','D','Munchen:Oberpfaffenhofen, Bayern','1969-01-01 00:00:00','','Deutsches Raumfahrt-Kontrollzentrum (German Space Ops Center)' from dual union all
select 'GSYU','GS Yuasa Corporation','business','','J','Kyoto','2004-01-01 00:00:00','','株式会社ジーエス・ユアサ コーポレーション' from dual union all
select 'GTC','Group Technique de Cannes  (SNCASE/Aerospatiale)','business','SUD','F','Cannes','1941-04-01 00:00:00','','Group Technique de Cannes  (SNCASE/Aerospatiale)' from dual union all
select 'GTE','GTE Spacenet Corp','business','','US','McLean, Virginia','1983-01-01 00:00:00','1994-06-30 00:00:00','GTE Spacenet Corp' from dual;
insert into ORGANIZATION
select 'GTRC','General Tire and Rubber Company','business','','US','Cleveland:Akron, Ohio','1915-01-01 00:00:00','1984-01-01 00:00:00','General Tire and Rubber Company' from dual union all
select 'GU','Guam','','US','GU','Hagatna, Guam','1898-01-01 00:00:00','','Guahan (Guam)' from dual union all
select 'GUF','French Guiana','','F','GUF','Cayenne','1763-01-01 00:00:00','','Guyane française' from dual union all
select 'GUGK','Main Geodesy and Cartography Administration of the USSR','civilian','MVD','SU','Moskva','','1991-01-01 00:00:00','Главное управление геодезии и картографии при Совете министров СССР (ГУГК при СМ СССР)' from dual union all
select 'GUKOS','Main Space Forces Directorate','defense','MO','SU','Moskva?','1981-01-01 00:00:00','1986-04-24 00:00:00','Главное управление космических средств' from dual union all
select 'GUKOSR','Main Space Forces Directorate','defense','RVSN','SU','Moskva?','1970-03-01 00:00:00','1981-01-01 00:00:00','Glavnoye Upravleniye Kosmicheskhikh Sredstv' from dual union all
select 'GURVO','Main Rocket Armaments Directorate','defense','RVSN','SU','Moskva?','1959-12-01 00:00:00','1970-01-01 00:00:00','Главное управление ракетного вооружения' from dual union all
select 'GWEST','Global Western','business','','US','Denver, Colorado','2015-01-01 00:00:00','','Global Western' from dual union all
select 'GWHYZ','National Satellite Oceanic Applications Center','civilian','GHJ','CN','Mudanjiang, Heliongang','','','国家卫星海洋应用中心' from dual union all
select 'GYZ','State Remote Sensing Center','civilian','','CN','Beijing','','','国家遥感中心' from dual union all
select 'GZB','Minstry of Resources and Land','civilian','','CN','Beijing','1998-01-01 00:00:00','','中华人民共和国国土资源部' from dual union all
select 'HAC','Hughes Aircraft Co., Culver City','business','','US','Culver City, California','1948-01-01 00:00:00','1985-01-01 00:00:00','Hughes Aircraft Co., Culver City' from dual union all
select 'HACES','Hughes Aircraft Co., El Segundo','business','HAC','US','El Segundo, California','1955-01-01 00:00:00','1985-01-01 00:00:00','Hughes Aircraft Co., El Segundo' from dual union all
select 'HADC','Holloman Air Development Center','defense','HAFB','US','Alamagordo, New Mexico','1942-01-01 00:00:00','1957-09-01 00:00:00','Holloman Air Development Center' from dual union all
select 'HAFB','Holloman Air Force Base','defense','USAF','US','Alamagordo, New Mexico','1948-01-13 00:00:00','','Holloman Air Force Base' from dual union all
select 'HAL','Hindustan Aeronautics Ltd','business','','IN','Bangalore','1940-01-01 00:00:00','','Hindustan Aeronautics Ltd' from dual union all
select 'HAL5','Huntsville, Alabama L5 Society','academic/non-profit','','US','Huntsville, Alabama','1997-01-01 00:00:00','','Huntsville, Alabama L5 Society' from dual union all
select 'HAM','Holloman Aeromedical Lab, Holloman Air Force Base','defense','HADC','US','Alamagordo, New Mexico','1945-01-01 00:00:00','','Holloman Aeromedical Lab, Holloman Air Force Base' from dual union all
select 'HANAU','Hankuk Aviation University, Space System Research Lab','academic/non-profit','','KR','Seoul:Goyang','2000-01-01 00:00:00','2007-01-01 00:00:00','한국항공대학교' from dual union all
select 'HANWHA','Hanwha Corporation','business','','KR','Seoul','1952-01-01 00:00:00','','한화' from dual union all
select 'HAO','High Altitude Observatory (NCAR, U Colorado)','academic/non-profit','NCAR','US','Boulder, Colorado','1947-01-01 00:00:00','','High Altitude Observatory (NCAR, U Colorado)' from dual union all
select 'HARB','Harbin Institute of Technology','business','','CN','Harbin, Heilongjiang','1920-01-01 00:00:00','','哈尔滨工业大学' from dual union all
select 'HARK','NPP KhartronArkos Ltd  (OKB692)','business','','UA','Khar''kov, Ukraine','1959-01-01 00:00:00','','НПП «Хартрон-Аркос»' from dual union all
select 'HARP','High Altitude Research Project (Project HARP), McGill University','business','','CA','Montreal','1964-01-01 00:00:00','1966-01-01 00:00:00','High Altitude Research Project (Project HARP), McGill University' from dual union all
select 'HASTD','Hamilton Standard Corp.','business','UAC','US','Windsor Locks, Connecticut','1929-01-01 00:00:00','1999-06-01 00:00:00','Hamilton Standard Corp.' from dual union all
select 'HASUN','Hamilton Sunstrand (UTC)','business','UTC2','US','Windsor Locks, Connecticut','1999-06-01 00:00:00','2012-07-01 00:00:00','Hamilton Sunstrand (UTC)' from dual union all
select 'HAVEL','HAVELSAN','business','','TR','Ankara','1982-01-01 00:00:00','','HAVELSAN A.Ş' from dual union all
select 'HAWK','Hawk Institute for Space Sciences','academic/non-profit','','US','Pocomoke City, Maryland','2006-01-01 00:00:00','','Hawk Institute for Space Sciences' from dual union all
select 'HCG','Hughes Communications Galaxy Inc','business','HCI','US','El Segundo','1985-01-01 00:00:00','1997-05-01 00:00:00','Hughes Communications Galaxy Inc' from dual union all
select 'HCI','Hughes Communications Inc.','business','HEC','US','El Segundo','','1997-05-01 00:00:00','Hughes Communications Inc.' from dual union all
select 'HCO','Harvard College Observatory','academic/non-profit','','US','Cambridge, Mass.','1839-01-01 00:00:00','','Harvard College Observatory' from dual union all
select 'HCS','Hughes Communications Services Inc.','business','HEC','US','El Segundo','','2011-01-01 00:00:00','Hughes Communications Services Inc.' from dual union all
select 'HDOS','HughesDanbury Optical Systems','business','','US','Danbury, Connecticut','1990-05-01 00:00:00','2001-01-01 00:00:00','Hughes-Danbury Optical Systems' from dual union all
select 'HE360','HawkEye 360 Inc.','business','','US','Herndon, Virginia','2016-01-01 00:00:00','','HawkEye 360, Inc.' from dual union all
select 'HEAD','Beijing HEAD Aerospace Technology Co. Ltd.','business','','CN','Beijing','2007-01-01 00:00:00','','北京和德宇航技术有限公司' from dual union all
select 'HEC','Hughes Electronics Corp.','business','','US','El Segundo, California','1985-01-01 00:00:00','2003-01-01 00:00:00','Hughes Electronics Corp.' from dual union all
select 'HEID','Heidelberg Observatory','academic/non-profit','','D','Heidelberg:Konigstuhl','1898-01-01 00:00:00','','Landessternwarte Heidelberg-Königstuhl' from dual union all
select 'HELSA','HellasSat Consortium Ltd., OTE Group','business','','GR','Athenai:Maroussi','2001-08-01 00:00:00','2013-02-01 00:00:00','HellasSat Consortium Ltd, OTE Group (Org. Telepikoinonion Ellados)' from dual union all
select 'HELSAR','HellasSat Consortium Ltd., OTE Group','business','ARAB','GR','Athenai:Maroussi','2013-02-01 00:00:00','','HellasSat Consortium Ltd, Arabsat' from dual union all
select 'HERAK','Herakles  (SAFRAN/SNECMA Prop. Solide)','business','SAFRAN','F','Bordeaux:Le Haillan','2012-04-01 00:00:00','','Herakles  (SAFRAN/SNECMA Prop. Solide)' from dual union all
select 'HERAKS','Herakles SME (former SNPE)','business','SAFRAN','F','Saint:Medard:en:Jalles, Gironde','2012-04-01 00:00:00','','Herakles SME (former SNPE)' from dual union all
select 'HERC','Hercules Powder Co.','business','','US','Salt Lake City:Magna/Bacchus, Utah','1912-01-01 00:00:00','1995-03-01 00:00:00','Hercules Powder Co.' from dual union all
select 'HERCM','Hercules Powder Co. McGregor Plant','business','HERC','US','McGregor, Texas','1978-01-01 00:00:00','1996-01-01 00:00:00','Hercules Powder Co. McGregor Plant' from dual union all
select 'HERCR','Hercules Powder Co. Radford Arsenal plant','business','HERC','US','Radford, Virginia','1950-01-01 00:00:00','','Hercules Powder Co. Radford Arsenal plant' from dual union all
select 'HEXI','China Hexi Chemical and Machinery Co. (CASIC6)','business','CASIC','CN','Hohhot, Nei Menggu Zizhiqu','1999-01-01 00:00:00','','中国河西化工机械公司' from dual union all
select 'HEXI4A','4th Academy','business','JGB7','CN','Hohhot, Nei Menggu Zizhiqu','1962-07-01 00:00:00','1999-01-01 00:00:00','第七机械工业部第四研究院' from dual union all
select 'HGS','Hughes Global Services','business','','US','Los Angeles:El Segundo, California','1997-09-01 00:00:00','2004-01-01 00:00:00','Hughes Global Services' from dual union all
select 'HGS1','Hughes Government Services','business','','US','Los Angeles:El Segundo, California','1996-11-01 00:00:00','1997-09-01 00:00:00','Hughes Government Services' from dual union all
select 'HHJ','Space Star Technology Ltd','business','','CN','Beijing:Haidian','2017-01-01 00:00:00','','航天恒星技术有限公司' from dual union all
select 'HIBER','Hiber','business','','NL','Delft','2015-01-01 00:00:00','','Hiber' from dual union all
select 'HIN','Narvik University College','academic/non-profit','','N','Narvik','1994-08-01 00:00:00','','Høgskolen i Narvik' from dual union all
select 'HISD','Hisdesat Servicios Estrategicos SA','business','HISP','E','Madrid','2001-07-17 00:00:00','','Hisdesat Servicios Estratégicos S.A' from dual union all
select 'HISP','Hispasat SA','business','','E','Madrid','1989-01-01 00:00:00','','Hispasat SA' from dual union all
select 'HISPM','Hispamar Satelites S.A.','business','HISP','BR','Rio de Janeiro','2002-01-01 00:00:00','','Hispamar Satélites S.A' from dual union all
select 'HITS','HIT Shenzhen Graduate School','academic/non-profit','HARB','CN','Shenzhen, Guangdong','2002-01-01 00:00:00','','哈尔滨工业大学深圳研究生院' from dual union all
select 'HK','China(Hong Kong)','','CN','HK','Xianggang','1997-07-01 00:00:00','','中華人民共和國香港特別行政區' from dual union all
select 'HKUK','Hong Kong','','UK','HK','Xianggang','1842-08-29 00:00:00','1997-07-01 00:00:00','Hong Kong Crown Colony' from dual union all
select 'HNS','Hughes Network Systems','business','HEC','US','Germantown, Maryland','1987-01-01 00:00:00','2011-01-01 00:00:00','Hughes Network Systems' from dual union all
select 'HOCO','HorningCooper Inc.','business','','US','Monrovia, California','1952-01-01 00:00:00','1955-01-01 00:00:00','Horning-Cooper Inc.' from dual union all
select 'HOG','HermannOberthGesellschaft','academic/non-profit','','D','Cuxhaven','1961-01-01 00:00:00','1993-01-01 00:00:00','Hermann-Oberth-Gesellschaft' from dual union all
select 'HOKIT','Hokkaido Institute of Technology','academic/non-profit','','J','Sapporo, Hokkaido','1967-04-01 00:00:00','','北海道科学大学' from dual union all
select 'HOKK','Hokkaido University','academic/non-profit','','J','Sapporo, Hokkaido','1876-01-01 00:00:00','','北海道大学' from dual union all
select 'HONEY','Honeywell Aerospace, Defense and Space Systems','business','','US','Clearwater, Florida','','','Honeywell Aerospace, Defense and Space Systems' from dual union all
select 'HORZ','Horizons Satellite LLC (Intelsat/JSAT)','business','','US','Homer, Alaska ?','2005-01-01 00:00:00','','Horizons Satellite LLC (Intelsat/JSAT)' from dual union all
select 'HOUS','University of Houston','academic/non-profit','','US','Houston, Texas','1927-01-01 00:00:00','','University of Houston' from dual union all
select 'HOUW','Houwteq (ARMSCOR)','business','','ZA','Grabouw, Western Cape','1970-01-01 00:00:00','2004-01-01 00:00:00','Houwteq (ARMSCOR)' from dual union all
select 'HPS','High Performance Space Structure Systems GmbH','business','','D','Munchen','2012-01-01 00:00:00','','High Performance Space Structure Systems GmbH' from dual union all
select 'HQZS','Huai''an Youth Comprehensive Development Base','academic/non-profit','','CN','Huai''an, Jiangsu','2010-01-01 00:00:00','','淮安市青少年综合实践基地' from dual union all
select 'HRBS','HRBSinger Co.','business','','US','State College, Pennsylvania','1958-01-01 00:00:00','1990-01-01 00:00:00','HRB-Singer Co.' from dual union all
select 'HRBSE','ESystems HRBS','business','ESYS','US','State College, Pennsylvania','1990-01-01 00:00:00','','HRB Systems' from dual union all
select 'HSCIL','Herzliya Science Center','academic/non-profit','','IL','Herzliya','1994-01-01 00:00:00','','Herzliya Science Center' from dual union all
select 'HSD','Hawker Siddeley Dynamics','business','','UK','Stevenage','1963-01-01 00:00:00','1977-04-01 00:00:00','Hawker Siddeley Dynamics' from dual union all
select 'HSES','Hughes Space and Comms. Co (Hughes Satellite Systems)','business','HEC','US','El Segundo, California','1961-01-01 00:00:00','2000-10-01 00:00:00','Hughes Space and Comms. Co (Hughes Satellite Systems)' from dual union all
select 'HTSTL','HangtianTsinghua Satellite Technology Ltd.','academic/non-profit','TSHUA','CN','Beijing:Haidian','2000-01-01 00:00:00','2005-01-01 00:00:00','航天清华卫星技术有限公司' from dual union all
select 'HU','Hungary','','','HU','Budapest','','','Magyar Köztársaság' from dual union all
select 'HUMS','Hughes Missile Systems, AF Plant 44','business','','US','Tucson, Arizona','1951-01-01 00:00:00','1997-01-01 00:00:00','Hughes Missile Systems, AF Plant 44' from dual union all
select 'HUNTE','Hunting Engineering','business','','UK','Ampthill, Bedfordshire','1957-01-01 00:00:00','2001-10-01 00:00:00','Hunting Engineering' from dual union all
select 'HUPO','Hughes Missile Systems Co. Pomona','business','','US','Pomona, California','1992-05-01 00:00:00','1994-01-01 00:00:00','Hughes Missile Systems Co. Pomona' from dual union all
select 'HVP','HV Peenemunde','defense','WEHR','DR','Peenemunde','1939-01-01 00:00:00','1945-01-01 00:00:00','Heeresversuchsanstalt Peenemünde' from dual union all
select 'HYSIQ','HySpecIQ','business','','US','Washington, DC','2014-01-01 00:00:00','','HySpecIQ' from dual union all
select 'I','Italy','','','I','Italia','1861-03-17 00:00:00','','Repubblica Italiana' from dual union all
select 'I-CSC','COSPASSARSAT','','','I-CSC','Montreal, Quebec','1988-07-01 00:00:00','','COSPAS-SARSAT Council' from dual union all
select 'I-CSC1','COSPASSARSAT','','','I-CSC1','Montreal, Quebec?','1979-01-01 00:00:00','1988-07-01 00:00:00','COSPAS-SARSAT interim Memo of Understanding' from dual union all
select 'I-ELDO','ELDO','','','I-ELDO','Paris','1964-01-01 00:00:00','1975-04-30 00:00:00','Centre européen pour la construction de lanceurs d''engins spatiaux' from dual union all
select 'I-ESA','ESA','','','I-ESA','Paris','1975-04-30 00:00:00','','European Space Agency' from dual union all
select 'I-ESRO','ESRO','','','I-ESRO','Paris','1964-01-01 00:00:00','1975-04-30 00:00:00','Conseil européen de recherches spatiales' from dual union all
select 'I-EU','EU','','','I-EU','Bruxelles','1993-11-01 00:00:00','','European Union' from dual union all
select 'I-EUM','EUMETSAT','','','I-EUM','Darmstadt','1983-01-01 00:00:00','','European Meteorological Satellite Organization' from dual union all
select 'I-EUT','EUTELSAT','','','I-EUT','Paris','1985-01-01 00:00:00','2001-07-02 00:00:00','European Telecommunications Satellite Organization' from dual union all
select 'I-INM','INMARSAT','','','I-INM','London','1979-01-01 00:00:00','1999-04-01 00:00:00','International Maritime Satellite Organization' from dual union all
select 'I-INT','INTELSAT','','','I-INT','Washington, DC','1964-01-01 00:00:00','2001-07-01 00:00:00','International Telecommunications Satellite Organization' from dual union all
select 'I-RASC','Regional African Satellite Comms. Organization','','','I-RASC','Abidjan, Cote d''Ivoire','1993-01-01 00:00:00','','Regional African Satellite Comms. Organization' from dual union all
select 'IAA','Instituto de Astrofisica de Andalusia (CSIC)','academic/non-profit','','E','Granada','1975-01-01 00:00:00','','Instituto de Astrofísica de Andalucía (CSIC)' from dual union all
select 'IACG','GLONASS InformationAnalytic Center','civilian','FKA','SU','Korolev:Podlipki','1995-01-01 00:00:00','','Информационно-аналитического центр ЦНИИмаш' from dual union all
select 'IADN','Israeli Air Defense Network, Israel Defense Forces','defense','','IL','Palmachim AFB, Yavne:Rishin LeZion','1948-01-01 00:00:00','','Israeli Air Defense Network, Israel Defense Forces' from dual union all
select 'IAE','Aeronautics and Space Institute','defense','BRCTA','BR','Sao Jose dos Campos, Brasil','1969-10-01 00:00:00','','Instituto de Aeronáutica e Espaço' from dual union all
select 'IAF167','Wing 167, Israeli Air Force','defense','','IL','Palmachim AFB, Yavne:Rishin LeZion','2011-01-01 00:00:00','','Wing 167, Israeli Air Force' from dual union all
select 'IAFE','Astronomy and Space Physics Instiute','civilian','','AR','Buenos Aires','1971-01-01 00:00:00','','Instituto de Astronomía y Física del Espacio' from dual union all
select 'IAI','Israeli Aircraft Industries, Mabat factory','business','','IL','Yehud','1968-01-01 00:00:00','','Israeli Aircraft Industries, Mabat factory' from dual union all
select 'IAIS','IAI Space Communications Ltd.','business','IAI','IL','Tel Aviv:Ramat Gan','1993-01-01 00:00:00','','IAI Space Communications Ltd.' from dual;
insert into ORGANIZATION
select 'IAM','State Research Experimental Inst. of Aviation Medicine','civilian','','SU','Moskva','1935-01-01 00:00:00','','Институт авиационной медицины Министерства обороны СССР' from dual union all
select 'IAP','Leibnitz Institute for Atmospheric Physics','academic/non-profit','','D','K\"ulungsborn (Rostock), Mecklenburg:Vorpommern','1991-01-01 00:00:00','','Leibniz-Instituts für Atmosphärenphysik e.V. an der Universität Rostock (IAP)' from dual union all
select 'IASB','BIRAIASB Institut d''Aeronomie Spatiale de Belgique','academic/non-profit','','B','Brussel:Uccle','1964-01-01 00:00:00','','Institut d''Aéronomie Spatiale de Belgique' from dual union all
select 'IBM','IBM Inc.','business','','US','Armonk, New York','1964-01-01 00:00:00','','IBM Inc.' from dual union all
select 'IBMH','IBM Inc.Huntsville','business','IBM','US','Huntsville, Alabama','1964-01-01 00:00:00','','IBM Inc.-Huntsville' from dual union all
select 'IBSI','Innovative Business Solutions Inc.','business','','US','Largo, Florida','1992-01-01 00:00:00','','Innovative Business Solutions Inc.' from dual union all
select 'ICANS','Industral Consortium Astronomical Netherlands Satellite','business','','NL','Amsterdam','1966-09-01 00:00:00','1977-01-01 00:00:00','Industral Consortium Astronomical Netherlands Satellite' from dual union all
select 'ICEYE','ICEYE','business','','FI','Helsinki','2015-01-01 00:00:00','','ICEYE' from dual union all
select 'ICO','ICO Global Communications','business','','UK','London, England','1995-01-01 00:00:00','2000-05-01 00:00:00','ICO Global Communications' from dual union all
select 'ICOH','ICO Global Communications (Holdings) Ltd','business','','US','Reston, Virginia','2000-05-01 00:00:00','2010-01-01 00:00:00','ICO Global Communications (Holdings) Ltd' from dual union all
select 'ICOK','ICO Global Communications (Holdings) Ltd','business','','US','Kirkland, Washington','2010-01-01 00:00:00','','ICO Global Communications (Holdings) Ltd' from dual union all
select 'ICONA','ICO North America','business','ICOH','US','Reston, Virginia','2000-01-01 00:00:00','2009-08-01 00:00:00','ICO North America' from dual union all
select 'ICST','Imperial College of Science and Technology','academic/non-profit','','UK','South Kensington, London, England','1907-01-01 00:00:00','','Imperial College of Science and Technology' from dual union all
select 'ICTQ','Supreme Council of Information and Communication Technology, Qatar','civilian','','QA','Doha, Qatar','','','Supreme Council of Information and Communication Technology, Qatar' from dual union all
select 'ID','Indonesia','','','ID','Indonesia','1945-08-17 00:00:00','','Republik Indonesia' from dual union all
select 'IDF','Israel Defense Forces (IAF Israeli Air Force)','defense','','IL','Tel Aviv','1948-01-01 00:00:00','','Israel Defense Forces (IAF Israeli Air Force)' from dual union all
select 'IDG','Insitute for the Dynamics of Geospheres','civilian','','RU','Moskva','1991-01-01 00:00:00','','Институт динамики геосфер Российской академии наук' from dual union all
select 'IDRDL','Defense Research and Development Lab','defense','','IN','Hyderabad','1962-01-01 00:00:00','','Defense Research and Development Lab' from dual union all
select 'IE','Ireland, Republic of','','','IE','Dublin (Baile Atha Cliath)','1922-01-01 00:00:00','','Republic of Ireland (Poblacht na hEireann)' from dual union all
select 'IEI','Iran Electronics Industries','defense','','IR','Tehran','1972-01-01 00:00:00','','شركت صنايع الكترونيك ايران' from dual union all
select 'IEM','Institute of Experimental Meteorology','civilian','','SU','Obninsk, Kaluga Oblast','1968-01-01 00:00:00','1980-01-01 00:00:00','Института экспериментальной метеорологии' from dual union all
select 'IFAMR','DLR Institut fur Physik der Atmosphare','civilian','','D','Munchen:Riem','','','DLR-Institut für Physik der Atmosphäre' from dual union all
select 'IGC','Intelsat General Corp.','business','INTELB','US','Washington, DC','2004-10-01 00:00:00','','Intelsat General Corp.' from dual union all
select 'IHI','IshikawajimaHarima Heavy Industries Co. Ltd','business','','J','Tokyo:Koto','1853-12-08 00:00:00','2007-01-01 00:00:00','(株）アイ・エイチ・アイ・エアロスペース' from dual union all
select 'IHIA','IHI Aerospace (former Nissan)','business','','J','Tomioka, Gunma','2000-06-01 00:00:00','','（株）アイ・エイチ・アイ・エアロスペース 富岡事業所' from dual union all
select 'IHIC','IHI Corporation','business','','J','Tokyo:Koto','2007-01-01 00:00:00','','（株）ＩＨＩエアロスペース' from dual union all
select 'IHIO','IHI Aerospace, Ogikubo plant','business','IHI','J','Tokyo:Suginami (Ogikubo)','2000-06-01 00:00:00','','（株）アイ・エイチ・アイ・エアロスペース 荻窪' from dual union all
select 'IIAE','Institute for Aerospace Research','civilian','','AR','Cordoba, Argentina','1961-01-01 00:00:00','1991-01-01 00:00:00','Instituto de Investigación Aeronáutica y Espacial' from dual union all
select 'IIST','Tokyo Univ. Inst of Industrial Sci.','academic/non-profit','TOK','J','Tokyo:Komaba','1955-01-01 00:00:00','1964-04-01 00:00:00','生産技術研究所(東京大学)' from dual union all
select 'IISTC','Institute of Industrial Science, Tokyo','civilian','TOK','J','Tokyo:Chiba','1955-01-01 00:00:00','1964-04-01 00:00:00','生産技術研究所' from dual union all
select 'IITB','Indian Institute of Technology, Bombay','academic/non-profit','','IN','Mumbai:Powai, Maharashtra','1958-01-01 00:00:00','','Indian Institute of Technology, Bombay' from dual union all
select 'IITK','Indian Institute of Technology, Kanpur','academic/non-profit','','IN','Kanpur:Kalyanpur, Uttar Pradesh','1959-01-01 00:00:00','','Indian Institute of Technology, Kanpur' from dual union all
select 'IITM','Indian Institute of Tropical Meteorology','academic/non-profit','','IN','Pune:Pashan','1971-04-01 00:00:00','','Indian Institute of Tropical Meteorology' from dual union all
select 'IITMC','Indian Institute of Technology, Madras','academic/non-profit','','IN','Chennai, Tamil Nadu','1959-01-01 00:00:00','','Indian Institute of Technology, Madras' from dual union all
select 'IK','Interkosmos','civilian','','SU','Moskva?','1967-04-01 00:00:00','','Интеркосмос' from dual union all
select 'IKI','Institut Kosmicheksiye Issledovaniya','civilian','','SU','Moskva','1965-01-01 00:00:00','','Институт космических исследований РАН' from dual union all
select 'IL','Israel','','','IL','Tel Aviv, Yisra''el','1948-05-14 00:00:00','','מְדִינַת יִשְׂרָאֵל' from dual union all
select 'ILC','Ionosphere Lab, Copenhagen (DTU)','civilian','DTU','DK','Kobenhavn:Lyngby','1962-04-01 00:00:00','1968-01-01 00:00:00','Ionosphere Laboratory, DTU' from dual union all
select 'ILS','International Launch Services','business','','US','San Diego, California','1995-01-01 00:00:00','2000-01-01 00:00:00','International Launch Services' from dual union all
select 'ILSK','International Launch Services, Khrunichev','business','KHRU','RU','Moskva','1995-01-01 00:00:00','','International Launch Services, Khrunichev' from dual union all
select 'ILSKR','International Launch Services, Khrunichev','business','KHRU','US','Reston, Virginia','2008-01-01 00:00:00','','International Launch Services, Khrunichev' from dual union all
select 'ILSL','International Launch Services, Lockheed Martin','business','LM','US','McLean, Virginia','2000-01-01 00:00:00','2008-01-01 00:00:00','International Launch Services, Lockheed Martin' from dual union all
select 'IMAG','ImageSat International N.V.','business','','KORS','Willemstad, Curacao, Netherlands Antilles','1997-01-01 00:00:00','','ImageSat International N.V.' from dual union all
select 'IMAGT','ImageSat (TelAviv)','business','IMAG','IL','Tel:Aviv','1997-01-01 00:00:00','','ImageSat (Tel-Aviv)' from dual union all
select 'IMASSA','Aerospace Medical Institute Defense research centre','defense','','F','Bretigny','1988-01-01 00:00:00','','Institut de médecine aérospatiale du service de santé des armées' from dual union all
select 'IMAX','IMAX Corporation','business','','US','Toronto:Missisauga, Ontario','1967-01-01 00:00:00','','IMAX Corporation' from dual union all
select 'IMBP','Insitute of MedicalBiological Problems','civilian','','SU','Moskva','1963-01-01 00:00:00','','Института медико-биологических проблем' from dual union all
select 'IMDP','India Meteorological Department, Pune','civilian','','IN','Pune','1928-01-01 00:00:00','','India Meteorological Department, Pune' from dual union all
select 'IMGW','Institute of Meteorology and Water Management','civilian','','PL','Warsawa','1973-01-01 00:00:00','','Instytut Meteorologii i Gospodarki Wodnej' from dual union all
select 'IMICAS','Institute of Microsatellite Innovation, CAS','civilian','SECM','CN','Shanghai','2017-01-01 00:00:00','','中国科学院微小卫星创新研究院' from dual union all
select 'IMIS','IMI Summerfield Rocket Research Station','defense','','UK','Kidderminster:Summerfield','1952-01-01 00:00:00','','IMI Summerfield Rocket Research Station' from dual union all
select 'IN','India','','','IN','India','1947-08-15 00:00:00','','Republic of India' from dual union all
select 'INASAN','Insitute of Astronomy, Russian Acad. Sci.','civilian','ANRF','RU','Moskva','1991-01-01 00:00:00','','Институт астрономии Российской академии наук' from dual union all
select 'INAV','Indian Navy (Bharatiya Nau Sena)','defense','','IN','New Delhi','1947-01-01 00:00:00','','Indian Navy (Bharatiya Nau Sena)' from dual union all
select 'INAVC','Assistant Chief of Naval Staff (Comms, Space and NetworkCentric Ops)','defense','INAV','IN','New Delhi','2012-06-01 00:00:00','','Assistant Chief of Naval Staff (Comms, Space and Network-Centric Ops)' from dual union all
select 'INCO','INCOSPAR (Indian Committee on Space Research), TIFR','civilian','','IN','Mumbai:Colaba','1962-01-01 00:00:00','1969-01-01 00:00:00','INCOSPAR (Indian Committee on Space Research), TIFR' from dual union all
select 'INCOT','INCOSPAR, TERLS','civilian','','IN','Trivandrum','1962-01-01 00:00:00','1969-01-01 00:00:00','INCOSPAR, TERLS' from dual union all
select 'INDO','PT. Media Citra Indostar','business','','ID','Jakarta, West Java','1991-07-22 00:00:00','','PT. Media Citra Indostar' from dual union all
select 'INDOS','PT Indosat','business','','ID','Jakarta','2003-01-01 00:00:00','','PT Indosat' from dual union all
select 'INETI','Instituto Nacional de Engenharia, Tecnologia e Inova\,cao','academic/non-profit','','P','Lisboa:Alfragide, Portugal','1970-01-01 00:00:00','','Instituto Nacional de Engenharia, Tecnologia e Inovação' from dual union all
select 'INFER','Interferometrics Inc','business','','US','Chantilly, Virginia','','','Interferometrics Inc' from dual union all
select 'INFK','AO Informkosmos','business','','RU','Moskva','1992-01-01 00:00:00','2000-01-01 00:00:00','`ИНФОРМКОСМОС` АО' from dual union all
select 'INFOC','InFocus, Edgewood, New Mexico (40 Blanco Drive)','business','','US','Edgewood, New Mexico','2015-01-01 00:00:00','','InFocus, Edgewood, New Mexico (40 Blanco Drive)' from dual union all
select 'INMAR','International Maritime Satellite Organization','civilian','I-INM','I-INM','London','','1999-04-15 00:00:00','International Maritime Satellite Organization' from dual union all
select 'INMRL','International Maritime Satellite Organization Ltd,','business','','UK','London','1999-04-15 00:00:00','','International Maritime Satellite Organization Ltd,' from dual union all
select 'INPE','National Institute of Space Research','civilian','','BR','San Jose dos Campos, Brasil','1971-01-01 00:00:00','','Instituto Nacional de Pesquisas Espacias,' from dual union all
select 'INPES','INPE Southern Regional Space Center','civilian','','BR','Santa Maria, Rio Grande do Sul','1996-01-01 00:00:00','','Centro Regional Sul de Pequisas Espaciais (CRS/INPE-MCT)' from dual union all
select 'INSSET','INSSET','academic/non-profit','','F','Saint:Quentin, Aisne','1960-01-01 00:00:00','','Institut supérieur des sciences et techniques' from dual union all
select 'INSYS','INSYS Ltd.','business','','UK','Ampthill, Bedfordshire','2001-10-01 00:00:00','2005-08-01 00:00:00','INSYS Ltd.' from dual union all
select 'INTA','National Institute of Aerospace Technology','civilian','','E','Madrid:Torrejon','1942-01-01 00:00:00','','Instituto Nacional de Técnica Aeroespacial' from dual union all
select 'INTEL','INTELSAT','civilian','I-INT','I-INT','Washington, DC','1964-01-01 00:00:00','2001-07-18 00:00:00','INTELSAT' from dual union all
select 'INTELB','Intelsat Ltd.','business','','BM','Hamilton:Pembroke, Bermuda','2001-07-18 00:00:00','2009-12-15 00:00:00','Intelsat Ltd.' from dual union all
select 'INTELD','Intelsat Ltd, US headquarters','business','INTELB','US','Washington, DC','2001-07-18 00:00:00','2009-12-15 00:00:00','Intelsat Ltd, US headquarters' from dual union all
select 'INTELS','Intelsat S.A.','business','','L','Luxembourg','2009-12-15 00:00:00','','Intelsat S.A.' from dual union all
select 'INTELU','Intelsat S.A., US headquarters','business','INTELS','US','Washington, DC','2009-12-15 00:00:00','','Intelsat S.A., US headquarters' from dual union all
select 'INVAP','Investigaciones Aplicads Sociedad del Estado (INVAP SE)','business','','AR','San Carlos de Bariloche, Rio Negro','1976-01-01 00:00:00','','Investigaciones Aplicads Sociedad del Estado (INVAP SE)' from dual union all
select 'IOS','Interorbital Systems Corporation','business','INTELS','US','Mojave, California','1996-01-01 00:00:00','','Interorbital Systems Corporation' from dual union all
select 'IPSF','Irvine Public Schools Foundation','academic/non-profit','','US','Irvine, California','1976-01-01 00:00:00','','Irvine Public Schools Foundation' from dual union all
select 'IPW','Institut fur Physikalische Weltraumforschung','academic/non-profit','','D','Freiburg im Breslau','1973-01-01 00:00:00','1980-01-01 00:00:00','Institut für  Physikalische Weltraumforschung' from dual union all
select 'IQ','Iraq','','','IQ','Al Iraq','1919-10-01 00:00:00','','جمهورية العـراق' from dual union all
select 'IQ223','Brigade 223, Iraqi Army','defense','IQA','IQ','Al Taji, Baghdad','1989-01-01 00:00:00','2003-01-01 00:00:00','Brigade 223, Iraqi Army' from dual union all
select 'IQ224','Brigade 224, Iraqi Army','defense','IQA','IQ','Al Taji, Baghdad','1976-01-01 00:00:00','2003-01-01 00:00:00','Brigade 224, Iraqi Army' from dual union all
select 'IQA','Iraqi Army','defense','','IQ','Baghdad','1924-01-01 00:00:00','2003-01-01 00:00:00','Iraqi Army' from dual union all
select 'IQMIC','Military Industrial Commission, Iraq','defense','','IQ','Baghdad','1987-08-01 00:00:00','2003-01-01 00:00:00','Military Industrial Commission, Iraq' from dual union all
select 'IQMOST','Ministry of Science and Technology','civilian','','IQ','Baghdad','2003-01-01 00:00:00','','وزارة العلوم والتكنولوجيا' from dual union all
select 'IR','Iran','','','IR','Iran','1501-01-01 00:00:00','','جمهوری اسلامی ایران' from dual union all
select 'IRF','Swedish Inst of Space Physics','civilian','','S','Kiruna','1987-01-01 00:00:00','','Institutet för rymdfysik' from dual union all
select 'IRFU','Swedish Inst of Space Physics','civilian','','S','Uppsala','1986-01-01 00:00:00','','Institutet för rymdfysik, Uppsala' from dual union all
select 'IRID','Iridium Inc.','business','','US','Washington, DC','1991-01-01 00:00:00','2000-12-01 00:00:00','Iridium Inc.' from dual union all
select 'IRIDS','Iridium Satellite LLC','business','','US','Washington:Bethesda, Maryland','2000-12-01 00:00:00','','Iridium Satellite LLC' from dual union all
select 'IRMB','Royal Meteorological Inst. of Belgium','academic/non-profit','','B','Brussel:Uccle','1913-01-01 00:00:00','','Institut Royal Meteorologique (Koninklijk Meteorologisch Instituut)' from dual union all
select 'IROST','Iranian Research Organization for Science and Technology (OSIRI)','civilian','','IR','Tehran','','','سازمان پژوهشهای علمی و صنعتی ايران' from dual union all
select 'IRSA','Iranian Space Agency','defense','','IR','Tehran','2004-01-01 00:00:00','','سازمان فضایی ایران' from dual union all
select 'IRSS','Institut fur Raumfahrtsysteme, Universitat Stuttgart','academic/non-profit','','D','Stuttgart','1988-01-01 00:00:00','','Institut für Raumfahrtsysteme, Universität Stuttgart' from dual union all
select 'IS','Iceland','','','IS','Reykjavik','0930-01-01 00:00:00','','Ísland' from dual union all
select 'ISA','Israeli Space Agency','civilian','IL','IL','Yisra''el','1983-01-01 00:00:00','','Israeli Space Agency' from dual union all
select 'ISAC','ISRO Satellite Center','civilian','','IN','Bangalore','','','ISRO Satellite Center' from dual union all
select 'ISACA','ISRO Space Applications Center','civilian','','IN','Ahmedabad','','','ISRO Space Applications Center' from dual union all
select 'ISAE','National Higher School of Aeronautics and Space','academic/non-profit','','F','Toulouse','2007-01-01 00:00:00','','Institut supérieur de l''aéronautique et de l''espace' from dual union all
select 'ISAEV','Factory 293 (Isaev)','business','','SU','Moskva:Korolev (Podlipki)','1943-01-01 00:00:00','1952-01-01 00:00:00','КБ завода № 293 им. А.М. Исаева' from dual;
insert into ORGANIZATION
select 'ISAS','Institute of Space and Aeronautical Sci','civilian','','J','Tokyo:Komaba','1964-04-01 00:00:00','1981-04-01 00:00:00','宇宙航空研究所' from dual union all
select 'ISASJ','Institute of Space and Astronautical Sci','civilian','JAXA','J','Tokyo:Sagamihara','2003-10-01 00:00:00','','宇宙科学研究所/宇宙航空研究開発機構' from dual union all
select 'ISASM','Institute of Space and Astronautical Sci','civilian','MONBU','J','Tokyo:Komaba','1981-04-01 00:00:00','1989-04-01 00:00:00','宇宙科学研究所' from dual union all
select 'ISASS','Institute of Space and Astronautical Sci','civilian','','J','Tokyo:Sagamihara','1989-04-01 00:00:00','2003-10-01 00:00:00','宇宙科学研究所' from dual union all
select 'ISFC','Strategic Forces Command, Nuclear Command Authority','defense','','IN','New Delhi','2003-01-01 00:00:00','','Strategic Forces Command, Nuclear Command Authority' from dual union all
select 'ISISBV','Innovative Solutions In Space B.V.','business','','NL','Delft','2006-01-01 00:00:00','','Innovative Solutions In Space B.V.' from dual union all
select 'ISKRM','MKB Iskra im. I.I. Kartukov (KB2 Zav. 81)','business','','RU','Moskva','1946-03-01 00:00:00','','МКБ «Искра» имени И.И.Картукова' from dual union all
select 'ISKRP','OAO NPO Iskra','business','','RU','Perm'', Permskiy Krai','','','НПО «Искра»' from dual union all
select 'ISPUT','Intersputnik Intl. Org of Space Comms.','civilian','','SU','Moskva','1971-11-15 00:00:00','','Интерспутник' from dual union all
select 'ISRO','Indian Space Research Organization','civilian','','IN','Bangalore','1969-01-01 00:00:00','','Indian Space Research Organization' from dual union all
select 'ISROT','ISRO/Thumba Equatorial Rocket Launch Station','civilian','','IN','Trivandrum','1969-01-01 00:00:00','1971-01-01 00:00:00','ISRO/Thumba Equatorial Rocket Launch Station' from dual union all
select 'IST','Interstellar Technologies Inc','business','','J','Taiki, Tokachi, Hokkaido','2003-01-01 00:00:00','','Interstellar Technologies Inc.' from dual union all
select 'ISTPK','Institute of Space Technology','academic/non-profit','','PK','Islamabad','2002-01-01 00:00:00','','Institute of Space Technology' from dual union all
select 'ITA','Instituto Tecnologico de Aeronautica','business','','BR','Sao Jose dos Campos','1950-01-01 00:00:00','','Instituto Tecnológico de Aeronáutica' from dual union all
select 'ITBW','Bundesamt f. Inf.management und Informationstechnik der Bundeswehr','defense','BWEHR','D','Koblenz','','','Bundesamt für Informationsmanagement und Informationstechnik der Bundeswehr' from dual union all
select 'ITCR','Costa Rica Institute of Technology','academic/non-profit','','CR','Cartago','1971-01-01 00:00:00','','Instituto Tecnológico de Costa Rica' from dual union all
select 'ITRC','Iran Telecommunication Research Center','civilian','','IR','Tehran','1970-01-01 00:00:00','','مرکز تحقیقات مخابرات ایران' from dual union all
select 'ITSK','KeRC','civilian','','RU','Moskva:Khovrino','1992-01-01 00:00:00','','Исследовательский центр имени М.В. Келдыша' from dual union all
select 'ITT','ITT Federal Laboratories','business','','US','Nutley, New Jersey? (San Fernando,CA?)','','','ITT Federal Laboratories' from dual union all
select 'ITUTR','Istanbul Technical University','academic/non-profit','','TR','Istanbul','2007-01-01 00:00:00','','İstanbul Teknik Üniversitesi,(Space Systems Design and Test Laboratory)' from dual union all
select 'IUST','IUST','academic/non-profit','','IR','Tehran','1929-01-01 00:00:00','','دانشگاه علم و صنعت ایران' from dual union all
select 'IZM','Inst. of Terrestrial Magnetism, Ionos. and RW Prop.','civilian','','SU','Moskva:Troitsk','1940-01-01 00:00:00','','Институт земного магнетизма, ионосферы и распространения радиоволн' from dual union all
select 'J','Japan','','','J','Tokyo-to, Honshu, Nippon','0538-01-01 00:00:00','','日本国' from dual union all
select 'J061','CASIC Base 061, CASIC 4th Acad?','defense','CASIC4A','CN','Zunyi, Guizhou','2010-01-01 00:00:00','','061基地' from dual union all
select 'JARL','Japanese Amateur Radio League','academic/non-profit','','J','Tokyo:Toshima','1926-01-01 00:00:00','','日本アマチュア無線連盟' from dual union all
select 'JAXA','Japan Aerospace Exploration Agency','civilian','','J','Tokyo:Chofu','2003-10-01 00:00:00','','宇宙航空研究開発機構' from dual union all
select 'JAXAC','Chofu Aerospace Center','civilian','','J','Tokyo:Chofu','2003-10-01 00:00:00','','調布航空宇宙センター' from dual union all
select 'JBO','Jodrell Bank Observatory, Univ. of Manchester','academic/non-profit','','UK','Jodrell Bank, Cheshire, England','1957-01-01 00:00:00','','Jodrell Bank Observatory, Univ. of Manchester' from dual union all
select 'JCAB','Japan Civil Aviation Bureau (Board)','civilian','','J','Tokyo:Chiyoda','','','航空局' from dual union all
select 'JCSAT','JCSAT (Japan Communications Satellite Corp), Nippon','business','','J','Tokyo:Minato','1985-04-01 00:00:00','1993-08-01 00:00:00','日本通信衛星（株）' from dual union all
select 'JDA','Japan Defense Agency','defense','','J','Tokyo:Minato/Akasaka','1954-07-01 00:00:00','2007-01-09 00:00:00','防衛庁' from dual union all
select 'JENA','DASA Jena Optronic GmbH (DJO)','business','','D','Jena, Thuringia','1991-01-01 00:00:00','2010-01-01 00:00:00','DASA Jena Optronic GmbH (DJO)' from dual union all
select 'JENAA','DASA Jena Optronic GmbH (DJO)/ Astrium','business','ASTD','D','Jena, Thuringia','2010-01-01 00:00:00','','DASA Jena Optronic GmbH (DJO)/ Astrium' from dual union all
select 'JGB7','7th Machine Bld.','civilian','CN','CN','Beijing','1964-11-23 00:00:00','1982-04-01 00:00:00','七机械工业部' from dual union all
select 'JHU','Johns Hopkins Univ.','academic/non-profit','','US','Baltimore, Maryland','1876-01-01 00:00:00','','Johns Hopkins Univ.' from dual union all
select 'JMA','Japan Meteorological Agency','civilian','','J','Tokyo:Chiyoda','1956-01-01 00:00:00','','気象庁' from dual union all
select 'JMASC','JMA Meteorological Satellite Center','civilian','JMA','J','Tokyo:Kiyose  [MTSAT GMS]','1978-01-01 00:00:00','','気象衛星センター' from dual union all
select 'JMSDF','Maritime Self Defense Force','defense','','J','Tokyo:Yososuka','1954-01-01 00:00:00','','海上自衛隊' from dual union all
select 'JNIC','MDA Joint National Integration Center, Schreiver AFB','defense','','US','Colorado Springs, Colorado','1988-01-01 00:00:00','2011-01-01 00:00:00','MDA Joint National Integration Center, Schreiver AFB' from dual union all
select 'JNSIC','China Jiangnan Space Industry Corp. (CASIC 061)','defense','CASIC','CN','Zunyi, Guizhou','1964-01-01 00:00:00','2010-01-01 00:00:00','中国江南航天工业集团公司' from dual union all
select 'JO','Jordan','','','JO','Amman','1946-01-01 00:00:00','','المملكة الأردنية الهاشمية' from dual union all
select 'JPL','Jet Propulsion Lab','civilian','','US','Pasadena, California','1944-01-01 00:00:00','','Jet Propulsion Lab' from dual union all
select 'JRG','Strategic Rocket Division, Korean People''s Army','defense','KPA','KP','Songchong-kun','2012-01-01 00:00:00','','조선인민군 전략로케트군' from dual union all
select 'JSAT','Japan Satellite Systems Inc','business','','J','Tokyo:Minato','1993-08-01 00:00:00','2008-08-01 00:00:00','日本サテライトシステムズ' from dual union all
select 'JSC','NASA Johnson Space Flight Center','civilian','NASA','US','Houston:Clear Lake, Texas','1973-02-17 00:00:00','','NASA Johnson Space Flight Center' from dual union all
select 'JSDO','Space Development Office, STA','civilian','STA','J','Tokyo, Nippon','1963-04-01 00:00:00','1964-06-01 00:00:00','宇宙開発室' from dual union all
select 'JSS','Japan Space Systems','business','','J','Tokyo:Minato','2012-03-01 00:00:00','','日本サテライトシステムズ' from dual union all
select 'JTF7','Joint Task Force 7 HQ','defense','','US','Arlington, Virginia','1962-01-01 00:00:00','1963-01-01 00:00:00','Joint Task Force 7 HQ' from dual union all
select 'JUNK','Junkers Flugzeug und Motorenwerke GmBH','business','','D','Munchen','1958-01-01 00:00:00','1969-01-01 00:00:00','Junkers Flugzeug- und Motorenwerke GmBH' from dual union all
select 'JUST','Jordan University of Science and Technology','academic/non-profit','','JO','Al Ramtha, Irbid','1986-01-01 00:00:00','','جامعة العلوم والتكنولوجيا الأردنية' from dual union all
select 'KACST','King Abdul Aziz City for Science and Technology','civilian','','SA','Riyadh','1985-01-01 00:00:00','','King Abdul Aziz City for Science and Technology' from dual union all
select 'KAGA','Kagawa Daigaku (Kagawa U.)','academic/non-profit','','J','Takamatsu, Kagawa','1949-01-01 00:00:00','','香川大学' from dual union all
select 'KAGO','Kagoshima Daigaku','academic/non-profit','','J','Kagoshima','1949-01-01 00:00:00','','鹿児島大学' from dual union all
select 'KAI','Kuibyshev Aviation Institute','business','','SU','Kubyshev (Samara)','1942-01-01 00:00:00','1967-01-01 00:00:00','Куйбышевский авиационный институт' from dual union all
select 'KAIFW','KaiserFleetwings','business','','US','Bristol, Pennsylvania','1943-01-01 00:00:00','','Kaiser-Fleetwings' from dual union all
select 'KAIK','Korolev Aviation Institute','business','','SU','Samara (Kubyshev)','1967-01-01 00:00:00','1992-01-01 00:00:00','Куйбышевский Авиационный Институт Им. С.П. Королева' from dual union all
select 'KAISR','KAIST Satellite Research Center (SatRec)','civilian','KAIST','KR','Daejeon','1989-08-01 00:00:00','','한국과학기술원 인공위성연구센터' from dual union all
select 'KAIST','Korea Advanced Institute for Space Technology','civilian','','KR','Daejeon','1971-01-01 00:00:00','','한국과학기술원' from dual union all
select 'KAIT','Kanagawa Institute of Technology','academic/non-profit','','J','Atsugi, Kanagawa','1975-01-01 00:00:00','','神奈川工科大学' from dual union all
select 'KARI','Korea Aerospace Research Institute','civilian','','KR','Daejeon','1989-10-01 00:00:00','','한국항공우주연구원' from dual union all
select 'KASH','National Inst. Communications Tech., Kashima Space Research','civilian','','J','Kashima, Ibaraki, Japan','2004-01-01 00:00:00','','鹿島宇宙技術センター' from dual union all
select 'KAWA','Kawasaki Heavy Industries Aerospace Co.','business','','J','Gifu','1954-01-01 00:00:00','','川崎重工業航空宇宙カンパニー' from dual union all
select 'KAZK','Kazakh National Space Agnecy','civilian','','KZ','Astana','1993-09-25 00:00:00','','Қазақстан Республикасы Ұлттық ғарыш агенттiгi' from dual union all
select 'KAZNU','AlFarabi Kazakh National University','academic/non-profit','','KZ','Almaty','1934-01-01 00:00:00','','Əл Фараби атындағы Қазақ Ұлттық Университеті,' from dual union all
select 'KAZOKB','Kazanskoye OKB Soyuz  (Zubets, MKB Soyuz)','business','NKE','SU','Kazan, Tartarstan','1990-01-01 00:00:00','','Казанское ОКБ «Союз»' from dual union all
select 'KAZSOY','Kazanskoye motorostroitel''niy zavod Soyuz','business','NKE','SU','Kazan, Tartarstan','1978-01-01 00:00:00','1990-01-01 00:00:00','ОАО «Казанский моторостроительный завод «Союз»' from dual union all
select 'KB1','KB1 OKB41 (Savin)','business','','SU','Moskva (SE)','1962-01-01 00:00:00','1973-01-01 00:00:00','«Конструкторское бюро № 1» (КБ-1)' from dual union all
select 'KBEM','KB Energomash','business','','RU','Moskva:Khimki','1967-01-01 00:00:00','1974-07-11 00:00:00','КБ Энергомаш' from dual union all
select 'KBEME','KB Energomash, NPO Energiya','business','NPOE','RU','Moskva:Khimki','1974-07-11 00:00:00','1990-01-01 00:00:00','КБ Энергомаш НПО “Энергия” МОМ' from dual union all
select 'KBKHA','KB KhimAvtomatiki','business','','RU','Voronezh','1967-01-01 00:00:00','','ОАО КБ химической автоматики (КБХА)' from dual union all
select 'KBM','KB Mashinostroeniye (SKB101)','business','','SU','Mosvka:Kolomna','1942-01-01 00:00:00','','КБ машиностроения' from dual union all
select 'KBPT','KBP (Instrument Design Bureau) (TSKB14)','business','','SU','Tula','1946-01-01 00:00:00','','Конструкторского бюро приборостроения (ЦКБ-14)' from dual union all
select 'KBSAL','KB Salyut','business','','RU','Moskva:Fili','1981-01-01 00:00:00','1994-01-01 00:00:00','Конструкторское бюро “Салют”' from dual union all
select 'KCHT','Guangdong Aerospace Tech Co Ltd.','business','','CN','Guangdong','2015-01-01 00:00:00','','广东科创航天科技有限公司' from dual union all
select 'KCST','Korean Committee of Space Technology','defense','','KP','Pyongyang?','1998-01-01 00:00:00','2013-01-01 00:00:00','조선우주공간기술위원회' from dual union all
select 'KE','Kenya','','','KE','Nairobi','1963-12-12 00:00:00','','Jamhuri ya Kenya' from dual union all
select 'KEPLER','Kepler Communications Inc.','business','','CA','Toronto','2015-01-01 00:00:00','','Kepler Communications Inc.' from dual union all
select 'KEPWU','Kepler Satellite Technology (Wuhan) Co. Ltd.','business','SECM','CN','Wuhan','2016-01-01 00:00:00','','开普勒卫星科技（武汉）有限公司' from dual union all
select 'KGB','Komitet gosudarstvennoy besopasnosti SSSR','defense','','SU','Moskva','1954-01-01 00:00:00','1995-01-01 00:00:00','Комитет государственной безопасности' from dual union all
select 'KGI','Kiruna Geophysical Institute','civilian','','S','Kiruna','1973-01-01 00:00:00','1987-01-01 00:00:00','Kiruna Geofysiska Institut' from dual union all
select 'KGO','Kiruna Geophysical Observatory','civilian','','S','Kiruna','1957-01-01 00:00:00','1973-01-01 00:00:00','Kiruna Geofysiska Observatorium' from dual union all
select 'KH','Cambodia','','','KH','Phnom Penh','1993-01-01 00:00:00','','ព្រះរាជាណាចក្រកម្ពុជា' from dual union all
select 'KHMM','KB Khimmash','business','','SU','Moskva:Korolev (Podlipki)','1966-01-01 00:00:00','','Конструкторское бюро химического машиностроения имени А.М.Исаева' from dual union all
select 'KHRO','Khrunichev State Research and Production Center','business','ORKK','RU','Moskva:Fili','2014-03-01 00:00:00','','ФГУП Государственный космический научно-производственный центр имени М. В. Хруничева' from dual union all
select 'KHRR','Khrunichev State Research and Production Center','business','','RU','Moskva:Fili','1998-01-01 00:00:00','2014-03-01 00:00:00','Государственный космический научно-производственный центр имени М.В.Хруничева' from dual union all
select 'KHRS','GKNPTs im. M.V. Khrunichev, KB Salyut','business','KHRU','RU','Moskva:Fili','1994-01-01 00:00:00','','ГКНПЦ им. М.В.Хруничева, Конструкторское бюро “Салют”' from dual union all
select 'KHRU','Khrunichev State Research and Production Center','business','','RU','Moskva:Fili','1994-01-01 00:00:00','1998-01-01 00:00:00','ГКНПЦ им. М.В.Хруничева' from dual union all
select 'KHUS','KHU','academic/non-profit','','KR','Seoul','1949-01-01 00:00:00','','경희대학교' from dual union all
select 'KI','Kiribati','','','KI','South Tarawa, Tarawa Atoll','1979-01-01 00:00:00','','Kiribati' from dual union all
select 'KIEL','Kiel University','academic/non-profit','','D','Kiel, Schleswig-Holstein','1665-01-01 00:00:00','','Christian-Albrechts-Universität zu Kiel' from dual union all
select 'KLIMOV','OAO Klimov / OKB im.. Klimov','business','','SU','Sankt:Peterburg (Leningrad)','1965-01-01 00:00:00','','ОАО «Климов»' from dual union all
select 'KMARQ','Kaiser Marquardt','business','','US','Van Nuys, California','1990-01-01 00:00:00','2000-06-01 00:00:00','Kaiser Marquardt' from dual union all
select 'KMIT','MITT','business','','RU','Moskva (NE)','1992-01-01 00:00:00','','НТЦ «КОМПЛЕКС-МИТ»' from dual union all
select 'KMUTNB','King Mongkut''s University of Technology North Bangkok','academic/non-profit','','T','Bangkok','1959-01-01 00:00:00','','มหาวิทยาลัยเทคโนโลยีพระจอมเกล้าพระนครเหนือ' from dual union all
select 'KOBE','Kobe University','academic/non-profit','','J','Kobe','1949-01-01 00:00:00','','神戸大学' from dual union all
select 'KOBN','Copenhagen University','academic/non-profit','','DK','Kobenhavn','1479-01-01 00:00:00','','Københavns Universitet' from dual union all
select 'KOKT','Krasniy Oktyabr Mash, StPeterberg im. Klimov','business','','RU','Sankt:Peterberg/Leningrad','1912-01-01 00:00:00','','СПб ОАО «Красный Октябрь»' from dual union all
select 'KOLN','University of Cologne','academic/non-profit','','D','Koln','1919-01-01 00:00:00','','Universität zu Köln' from dual union all
select 'KOMET','TsNII Kometa','business','','SU','Moskva (SE)','1973-01-01 00:00:00','','ЦНИИ «Комета»' from dual union all
select 'KORS','Neth. Antilles','','NL','KORS','Korsou','2010-10-01 00:00:00','','Land Curaçao' from dual;
insert into ORGANIZATION
select 'KORSA','Neth. Antilles','','ANTN','KORS','Curacao (Korsou)','1954-01-01 00:00:00','2010-10-01 00:00:00','Curaçao, Nederlandse Antillen' from dual union all
select 'KOSMT','Tsentr Kosmonit OAO RKS (Rossiyskoe Kosmicheskiye Sistemi)','business','','RU','Moskva','','','НТЦ "КОСМОНИТ" ОАО "Российские космические системы"' from dual union all
select 'KOUKO','TMCAE','academic/non-profit','','J','Tokyo:Arakawa-ku','1962-01-01 00:00:00','','航空高専 (東京都立航空工業高等専門学校)' from dual union all
select 'KP','North Korea','','','KP','Choson','1948-09-09 00:00:00','','조선민주주의인민공화국' from dual union all
select 'KPA','Korean People''s Army','defense','','KP','Pyongyang','1948-01-01 00:00:00','','조선인민군' from dual union all
select 'KPAMJ','Strategic Rocket Division, KPA','defense','KPA','KP','Songchong-kun','1999-01-01 00:00:00','2012-01-01 00:00:00','미사일지도국' from dual union all
select 'KPI','Kyiv Polytechnic Institute','academic/non-profit','','UA','Kiev','1898-01-01 00:00:00','','Київський політехнічний інститут' from dual union all
select 'KPNO','Kitt Peak National Observatory, Cherry St Building','civilian','','US','Tucson, Arizona','1960-01-01 00:00:00','','Kitt Peak National Observatory, Cherry St Building' from dual union all
select 'KR','South Korea','','','KR','Hanguk','1948-08-15 00:00:00','','대한민국' from dual union all
select 'KRAS','Krasnoyarksiy Mash Zavod','business','','RU','Krasnoyarsk','1935-01-01 00:00:00','','Красноярский машиностроительный завод' from dual union all
select 'KRAT','Kratos Defense and Security Solutions','business','','US','San Diego, California','2003-01-01 00:00:00','','Kratos Defense and Security Solutions' from dual union all
select 'KRAU','Korea Aerospace University','academic/non-profit','','KR','Seoul:Goyang','2007-01-01 00:00:00','','한국항공대학교' from dual union all
select 'KRLP','Khan Research Laboratories, Kahuta','defense','','PK','Kahuta','1984-01-01 00:00:00','','Khan Research Laboratories, Kahuta' from dual union all
select 'KS','RSCC','business','','RU','Moskva','2001-04-19 00:00:00','','ФГУП «Космическая связь»' from dual union all
select 'KSC','NASA John F. Kennedy Space Center','civilian','NASA','US','Cape Canaveral, Florida','1963-11-01 00:00:00','','NASA John F. Kennedy Space Center' from dual union all
select 'KTEL','Korea Telecom','business','','KR','Hanguk','1981-01-01 00:00:00','2001-01-01 00:00:00','Korea Telecom' from dual union all
select 'KTELT','KT Corp. (Korea Telecom)','business','','KR','Hanguk','2001-01-01 00:00:00','','케이티 주식회사' from dual union all
select 'KTH','Royal Inst. of Technology','academic/non-profit','','S','Stockholm:Roslagstull?','1827-01-01 00:00:00','','Kungliga Tekniska högskolan Rymd- och plasmafysik' from dual union all
select 'KTHR','KayserThrede GmbH','business','','D','Munchen','1967-01-01 00:00:00','','Kayser-Threde GmbH' from dual union all
select 'KTRAS','MKK Kosmotras','business','','RU','Moskva','1997-01-01 00:00:00','','ЗАО Международная космическая компания “Космотрас”' from dual union all
select 'KTSAT','KT Sat','business','KTELT','KR','Seoul','2012-01-01 00:00:00','','KT Sat' from dual union all
select 'KTU','Kaunas University of Technology','academic/non-profit','','LT','Kaunas','1920-01-01 00:00:00','','Kauno technologijos universitetas' from dual union all
select 'KUBOS','Kubos Corp.','business','','IS','Denton, Texas','2016-01-01 00:00:00','','Kubos, Texas' from dual union all
select 'KUZ','OAO Kuznetsov','business','','SU','Samara','1996-01-01 00:00:00','','ОАО «Кузнецов»' from dual union all
select 'KVR','Russian Space Forces','defense','','RU','Moskva:Solnechnogorsk','2001-06-01 00:00:00','2011-12-01 00:00:00','Космические войска России' from dual union all
select 'KYO','Kyoto University','academic/non-profit','','J','Kyoto','1869-01-01 00:00:00','','京都大学' from dual union all
select 'KYSP','Kentucky Space, Kentucky Science and Technology Corp.','academic/non-profit','','US','Lexington, Kentucky','2008-01-01 00:00:00','','Kentucky Space, Kentucky Science and Technology Corp.' from dual union all
select 'KYUD','Kyushu University','academic/non-profit','','J','Fukuoaka, Fukuoka','1903-01-01 00:00:00','','九州大学' from dual union all
select 'KYUT','Kyushu Instiute of Technology','academic/non-profit','','J','Kitakyushu:Tobata, Fukuoka','2004-01-01 00:00:00','','九州工業大学' from dual union all
select 'KZ','Kazakhstan','','','KZ','Qazaqstan','1991-12-16 00:00:00','','Қазақстан Республикасы' from dual union all
select 'L','Luxembourg','','','L','Luxembourg','0963-01-01 00:00:00','','Grand-Duché de Luxembourg' from dual union all
select 'LA','Laos','','','LA','Vientiane','1953-01-01 00:00:00','','ສາທາລະນະລັດ ປະຊາທິປະໄຕ ປະຊາຊົນລາວ' from dual union all
select 'LAC','Lockheed Aircraft Corporation','business','','US','LA:Burbank, California','1932-01-01 00:00:00','1995-03-15 00:00:00','Lockheed Aircraft Corporation' from dual union all
select 'LAMBDA','Lambda Team','academic/non-profit','','US','San Jose, California','2014-01-01 00:00:00','','Lambda Team' from dual union all
select 'LANAST','National Authority for Science and Technology, Laos','civilian','','LA','Vientiane','','','ອົງການວິທະຍາສາດ ແລະ ເຕັກໂນໂລຊີແຫ່ງຊາດ (ອວຕຊ)' from dual union all
select 'LAND','Land Group','business','','CN','Beijing?','1990-01-01 00:00:00','','Land Group' from dual union all
select 'LANL','Los Alamos National Laboratory','defense','','US','Los Alamos, New Mexico','1980-01-01 00:00:00','','Los Alamos National Laboratory' from dual union all
select 'LAOJV','Lao Satellite Joint Venture Company','business','LANAST','LA','Vientiane','2015-01-01 00:00:00','','Lao Satellite Joint Venture Co.' from dual union all
select 'LAPAN','Lembaga Penerbangan dan Antariksa Nasional/LAPAN','civilian','','ID','Jakarta','1964-11-27 00:00:00','','Lembaga Penerbangan dan Antariksa Nasional/LAPAN' from dual union all
select 'LAPANR','LAPAN Rocket Motor Lab','civilian','','ID','Tarogong, W Java','','','LAPAN Rocket Motor Lab' from dual union all
select 'LARC','NACA Langley Research Center','civilian','NACA','US','Langley Field, Hampton, Virginia','1917-01-01 00:00:00','1958-10-01 00:00:00','NACA Langley Research Center' from dual union all
select 'LARCN','NASA Langley Research Center','civilian','NASA','US','Langley Field, Hampton, Virginia','1958-10-01 00:00:00','','NASA Langley Research Center' from dual union all
select 'LARCNE','NASA Engineering and Safety Center, LaRC','civilian','NASA','US','Langley Field, Hampton, Virginia','2003-01-01 00:00:00','','NASA Engineering and Safety Center, LaRC' from dual union all
select 'LAS','Labo. d''Astronomie Spatiale','academic/non-profit','CNRS','F','Marseille','','','Labo. d''Astronomie Spatiale' from dual union all
select 'LASL','Los Alamos Scientific Laboratory','defense','','US','Los Alamos, New Mexico','1947-01-01 00:00:00','1980-01-01 00:00:00','Los Alamos Scientific Laboratory' from dual union all
select 'LATMOS','Aeronomy Service, CNRS','civilian','CNRS','F','Paris:Verrieres:le:Buisson','2009-01-01 00:00:00','','Laboratoire Atmosphères, Milieux, Observations spatiales' from dual union all
select 'LB','Lebanon','','','LB','Beirut','1943-01-01 00:00:00','','الجمهورية اللبنانية' from dual union all
select 'LCT','Laboratoire Centrale de Telecommunications','civilian','','F','Paris','1927-01-01 00:00:00','','Laboratoire Centrale de Telecommunications' from dual union all
select 'LEEDS','University of Leeds','academic/non-profit','','UK','Leeds','1887-01-01 00:00:00','','University of Leeds' from dual union all
select 'LEID','Universiteit Leiden','academic/non-profit','','NL','Leiden','1575-01-01 00:00:00','','Universiteit Leiden' from dual union all
select 'LEOS','Loral ElectroOptical Systems, Inc.','business','LORC','US','Pasadena, California','1980-01-01 00:00:00','','Loral Electro-Optical Systems, Inc.' from dual union all
select 'LEOSTL','LeoStella LLC (SFLIN/THALES)','business','SFLIN','US','Seattle','2018-03-01 00:00:00','','LeoStella LLC' from dual union all
select 'LERC','NASA Lewis Research Center','civilian','NASA','US','Cleveland, Ohio','1958-01-01 00:00:00','1999-03-01 00:00:00','NASA Lewis Research Center' from dual union all
select 'LGAR','L''Garde Inc.','business','','US','Tustin, California','1971-01-01 00:00:00','','L''Garde Inc.' from dual union all
select 'LIEGE','Universite'' de Liege','academic/non-profit','','B','Liege, Wallonie','1817-01-01 00:00:00','','Université de Liège' from dual union all
select 'LII','Letnoissledovatel''skiy institut im. M.M. Gromov','defense','','RU','Zhukovsky:Ramenskoye','1941-01-01 00:00:00','','Лётно-исследовательский институт имени М. М. Громова' from dual union all
select 'LINAO','Aerologisches Observatorium Lindenberg, Met. Dienst DDR','civilian','','DD','Lindenberg, Bayern e','1905-01-01 00:00:00','','Aerologisches Observatorium Lindenberg, Met. Dienst DDR' from dual union all
select 'LINK','Link Space Aerospace Technology','business','','CN','Beijing','2014-01-01 00:00:00','','深圳市翎客航天技术有限公司' from dual union all
select 'LK','Sri Lanka','','','LK','Colombo/Sri Jayawardenapura Kotte','1972-01-01 00:00:00','','ශ්‍රී ලංකා ප්‍රජාතාන්ත්‍රික සමාජවාදී ජනරජය' from dual union all
select 'LKA','Lithuanian Space Association','academic/non-profit','','LT','Vilnius','2010-01-01 00:00:00','','Lietuvos Kosmoso Asociacija' from dual union all
select 'LLL','Lawrence Livermore Lab','defense','','US','Livermore, California','1971-06-01 00:00:00','1980-01-01 00:00:00','Lawrence Livermore Lab' from dual union all
select 'LLNL','Lawrence Livermore National Lab','defense','','US','Livermore, California','1980-01-01 00:00:00','','Lawrence Livermore National Lab' from dual union all
select 'LM','Lockheed Martin','business','','US','Sunnyvale, California','1995-03-15 00:00:00','','Lockheed Martin' from dual union all
select 'LMA','Lockheed Martin Astronautics','business','LM','US','Denver:Littleton, Colorado','1995-03-15 00:00:00','2003-01-01 00:00:00','Lockheed Martin Astronautics' from dual union all
select 'LMASD','LMA/San Diego (Kearny Mesa)','business','LM','US','San Diego:Kearny Mesa, California','1995-03-15 00:00:00','1996-01-01 00:00:00','LMA/San Diego (Kearny Mesa)' from dual union all
select 'LMATC','Lockheed Martin Advanced Technology Center','business','LM','US','Palo Alto, California','1997-01-01 00:00:00','','Lockheed Martin Advanced Technology Center' from dual union all
select 'LMCAM','Lockheed Martin Missiles and Fire Control/Camden','business','LM','US','Camden, Arkansas','1995-01-01 00:00:00','','Lockheed Martin Missiles and Fire Control/Camden' from dual union all
select 'LMCLS','Lockheed Martin Commercial Launch Services','business','LM','US','McLean, Virginia','2008-01-01 00:00:00','','Lockheed Martin Commercial Launch Services' from dual union all
select 'LMCO','Lockheed Martin Courtland (Lawrence County)','business','LM','US','Courtland, Alabama','1994-01-01 00:00:00','','Lockheed Martin Courtland (Lawrence County)' from dual union all
select 'LMCOM','Lockheed Martin Global Telecom/Comsat General','business','LM','US','Washington, DC','1998-09-01 00:00:00','2004-10-01 00:00:00','Lockheed Martin Global Telecom/Comsat General' from dual union all
select 'LMCSS','Lockheed Martin Commercial Space Systems/Sunnyvale','business','LM','US','Sunnyvale, California','1996-01-01 00:00:00','','Lockheed Martin Commercial Space Systems/Sunnyvale' from dual union all
select 'LMEW','Lockheed Martin/East Windsor','business','LM','US','East Windsor, New Jersey','1995-03-15 00:00:00','1998-01-01 00:00:00','Lockheed Martin/East Windsor' from dual union all
select 'LMGT','Lockheed Martin Global Telecommunications','business','LM','US','Sunnyvale, California ?','1995-03-01 00:00:00','','Lockheed Martin Global Telecommunications' from dual union all
select 'LMI','Lockheed Martin Intersputnik','business','LMGT','UK','London','1997-01-01 00:00:00','2006-01-01 00:00:00','Lockheed Martin Intersputnik' from dual union all
select 'LMINS','Lockheed Martin INSYS','business','LM','UK','Ampthill, Bedfordshire','2005-08-01 00:00:00','','Lockheed Martin INSYS' from dual union all
select 'LMKSC','Lockheed Martin, Operations/Checkout Bldg, Kennedy Space Center','business','LM','US','Kennedy Space Center, Florida','2009-01-01 00:00:00','','Lockheed Martin, Operations/Checkout Bldg, Kennedy Space Center' from dual union all
select 'LMMF','Lockheed Martin Missiles and Fire Control  Dallas','business','LM','US','Dallas:Grand Prairie, Texas','1999-01-01 00:00:00','','Lockheed Martin Missiles and Fire Control - Dallas' from dual union all
select 'LMMI','Lockheed Martin/Michoud','business','LM','US','Michoud, Louisiana','1995-03-15 00:00:00','','Lockheed Martin/Michoud' from dual union all
select 'LMMS','Lockheed Martin Missiles and Space, Sunnyvale','business','LM','US','Sunnyvale, California','1995-03-15 00:00:00','2005-01-01 00:00:00','Lockheed Martin Missiles and Space, Sunnyvale' from dual union all
select 'LMN','Lockheed Martin Commercial Space Systems/Newton','business','LM','US','Newton, Pennsylvania','1997-01-01 00:00:00','2015-01-01 00:00:00','Lockheed Martin Commercial Space Systems/Newton' from dual union all
select 'LMO','Lockheed Martin Missiles and Fire Control/Orlando','business','LM','US','Orlando, Florida','1995-01-01 00:00:00','','Lockheed Martin Missiles and Fire Control/Orlando' from dual union all
select 'LMPCO','Lockheed Martin Pike County Operations','business','LM','US','Troy, Alabama','1995-01-01 00:00:00','','Lockheed Martin Pike County Operations' from dual union all
select 'LMSAL','Lockheed Martin Solar and Astrophysics Laboratory','business','LMATC','US','Palo Alto, California','1997-01-01 00:00:00','','Lockheed Martin Solar and Astrophysics Laboratory' from dual union all
select 'LMSAN','Lockheed Martin Aerospace Electronics Systems/Nashua','business','','US','Nashua, New Hampshire','1995-01-01 00:00:00','2000-01-01 00:00:00','Lockheed Martin Aerospace Electronics Systems/Nashua' from dual union all
select 'LMSC','Lockheed Missiles and Space Co.','business','LAC','US','Sunnyvale, California','1961-06-01 00:00:00','1995-03-15 00:00:00','Lockheed Missiles and Space Co.' from dual union all
select 'LMSCV','Lockheed Martin Space Communications Ventures','business','LM','US','Sunnyvale, California?','1997-01-01 00:00:00','2006-09-01 00:00:00','Lockheed Martin Space Communications Ventures' from dual union all
select 'LMSD','Lockheed Missiles and Space Division','business','LAC','US','Sunnyvale, California','1955-01-01 00:00:00','1961-06-01 00:00:00','Lockheed Missiles and Space Division' from dual union all
select 'LMSO','LM Space Operations','business','SHAB','US','Titusville, Florida','2014-08-01 00:00:00','','LM Space Operations' from dual union all
select 'LMSS','Lockheed Martin Space Systems (Sunnyvale)','business','LAC','US','Sunnyvale, California','2005-01-01 00:00:00','','Lockheed Martin Space Systems (Sunnyvale)' from dual union all
select 'LMSSD','Lockheed Martin Space Systems (Denver), Astronautics Operations','business','LM','US','Denver:Littleton, Colorado','2003-01-01 00:00:00','','Lockheed Martin Space Systems (Denver), Astronautics Operations' from dual union all
select 'LMT','Lockheed Martin Telecommunications','business','LM','US','Sunnyvale, California','1995-01-01 00:00:00','','Lockheed Martin Telecommunications' from dual union all
select 'LMTEW','Lockheed Martin Telecommunications/East Windsor','business','LM','US','East Windsor, New Jersey','1995-01-01 00:00:00','1998-01-01 00:00:00','Lockheed Martin Telecommunications/East Windsor' from dual union all
select 'LMV','Lockheed Martin Vought','business','','US','Dallas:Grand Prairie, Texas','1996-01-01 00:00:00','1999-01-01 00:00:00','Lockheed Martin Vought' from dual union all
select 'LMVF','Lockheed Martin Space Systems/Valley Forge','business','LM','US','Valley Forge, Penn.','1995-01-01 00:00:00','','Lockheed Martin Space Systems/Valley Forge' from dual union all
select 'LOR','SS/Loral','business','LORC','US','Palo Alto, California','1990-10-01 00:00:00','2012-11-01 00:00:00','SS/Loral' from dual union all
select 'LORC','Loral Corporation','business','','US','New York, New York','1948-01-01 00:00:00','1996-01-01 00:00:00','Loral Corporation' from dual union all
select 'LORI','Loral Orion','business','LORC','US','Rockville, Maryland','1998-03-20 00:00:00','2005-11-21 00:00:00','Loral Orion' from dual union all
select 'LORS','Loral Space and Communications Inc.','business','','US','New York, New York','1996-01-01 00:00:00','','Loral Space and Communications Inc.' from dual union all
select 'LPA','Laboratoire de Physique de L''Atmosphere, Univ. Paris','civilian','','F','Paris','1955-01-01 00:00:00','','Laboratoire de Physique de l''Atmosphère' from dual union all
select 'LPARL','Lockheed Palo Alto Research Labs','business','LAC','US','Palo Alto, California','1956-01-01 00:00:00','1997-01-01 00:00:00','Lockheed Palo Alto Research Labs' from dual;
insert into ORGANIZATION
select 'LPC','Lockheed Propulsion Co','business','','US','Redlands, California','1961-01-01 00:00:00','1974-01-01 00:00:00','Lockheed Propulsion Co' from dual union all
select 'LPCE','Laboratoire de Physique et Chimie de l''Enivronnement et de l''Espace','academic/non-profit','','F','Orleans','','','Laboratoire de Physique et Chimie de l''Enivronnement et de l''Espace' from dual union all
select 'LPI','Lunar and Planetary Institute','civilian','','US','Houston, Texas','1968-01-01 00:00:00','','Lunar and Planetary Institute' from dual union all
select 'LPSC','VSSC Liquid Propulsion Systems Center','civilian','','IN','Trivandrum, Kerala','','','VSSC Liquid Propulsion Systems Center' from dual union all
select 'LPSCM','VSSC Liquid Propulsion Systems Center, Mahendragiri Facilities','civilian','','IN','Mahendragiri, Tamil Nadu','','','VSSC Liquid Propulsion Systems Center, Mahendragiri Facilities' from dual union all
select 'LPSP','Laboratoire de Physique Stellaire et Planetaire (CNRS)','civilian','CNRS','F','Paris:Verrieres:le:Buisson, Essonne','','','Laboratoire de Physique Stellaire et Planétaire du CNRS' from dual union all
select 'LRBA','Laboratoire de Recherches Ballistiques et Aerodynamiques','civilian','DGA','F','Vernon, Hte Normandie','1946-05-01 00:00:00','','Laboratoire de Recherches Balistiques et Aérodynamiques' from dual union all
select 'LRL','Lawrence Radiation Laboratory','defense','','US','Livermore, California','1958-01-01 00:00:00','1971-06-01 00:00:00','Lawrence Radiation Laboratory' from dual union all
select 'LRPG','Long Range Proving Ground','generic organization','','US','Cape Canaveral, Florida','1949-01-01 00:00:00','1950-01-01 00:00:00','Long Range Proving Ground' from dual union all
select 'LRRC','Lunar Rocket and Rover Co.','generic organization','','US','Los Alamitos, California','2002-03-01 00:00:00','2010-01-01 00:00:00','Lunar Rocket and Rover Co.' from dual union all
select 'LRS','LRS','civilian','','LB','Beirut:Dbayeh','1960-01-01 00:00:00','1966-01-01 00:00:00','L''Association Libanaise pour l''Étude des Engins Spatiaux' from dual union all
select 'LRWE','Long Range Weapons Establishment','defense','','AU','Adelaide:Salisbury, S Australia','1947-01-01 00:00:00','1955-01-01 00:00:00','Long Range Weapons Establishment' from dual union all
select 'LSKY','Loral Skynet','business','LORC','US','Bedminster, New Jersey','1996-01-01 00:00:00','2007-10-31 00:00:00','Loral Skynet' from dual union all
select 'LSKYB','Loral Skynet do Brazil','business','LSKY','BR','Rio de Janeiro','2000-01-01 00:00:00','2007-10-31 00:00:00','Loral Skynet do Brazil' from dual union all
select 'LSQ','LightSquared (Harbinger Capital Partners)','business','','US','New York, New York','2010-07-01 00:00:00','','LightSquared (Harbinger Capital Partners)' from dual union all
select 'LT','Lithuania','','','LT','Vilnius','1990-03-11 00:00:00','','Lietuvos Respublika' from dual union all
select 'LTU','Lulea Technical University','academic/non-profit','','S','Lulea','1971-01-01 00:00:00','','Luleå tekniska universitet' from dual union all
select 'LTV','LingTemcoVought Missile Systems','business','','US','Dallas:Grand Prarie, Texas','1962-01-01 00:00:00','1976-01-01 00:00:00','Ling-Temco-Vought Missile Systems' from dual union all
select 'LTVCAM','LTV Camden','business','LTV','US','Camden, Arkansas','','1995-01-01 00:00:00','LTV Camdebn' from dual union all
select 'LTVE','LTV Electronic Systems Division','business','','US','Dallas:Garland, Texas','1964-01-01 00:00:00','1972-01-01 00:00:00','LTV Electronic Systems Division' from dual union all
select 'LTVM','LTV, Michigan Army Missile Plant','business','','US','Detroit:Sterling Heights, Michigan','1962-01-01 00:00:00','1980-01-01 00:00:00','LTV, Michigan Army Missile Plant' from dual union all
select 'LTVV','LTV Missiles and Space (Vought Corporation)','business','','US','Dallas:Grand Prairie, Texas','1976-01-01 00:00:00','1992-08-31 00:00:00','LTV Missiles and Space (Vought Corporation)' from dual union all
select 'LULRZ','Luliang Municipal People''s Government','civilian','','CN','Luliang, Shanxi','','','吕梁市人民政府' from dual union all
select 'LUNA','Moon','','','LUNA','Luna','','','Luna' from dual union all
select 'LUND','Lund University','academic/non-profit','','S','Lund, Scania','1666-01-01 00:00:00','','Lunds universitat' from dual union all
select 'LUX','Leicester University Xray group','academic/non-profit','','UK','Leicester','1961-01-01 00:00:00','','Leicester University X-ray group' from dual union all
select 'LUXG','LuxGovSat S.A.','business','SES','L','Betzdorf, Luxembourg','2016-01-01 00:00:00','','LuxGovSat S.A.' from dual union all
select 'LUXS','LuxSpace Sarl (OHB)','business','OHB','L','Betzdorf, Luxembourg','2004-11-01 00:00:00','','LuxSpace Sarl (OHB)' from dual union all
select 'LV','Latvia','','','LV','Riga','1991-01-01 00:00:00','','Latvijas Republika' from dual union all
select 'LVMS','Loral Vought','business','LORC','US','Dallas:Grand Prairie, Texas','1992-08-31 00:00:00','1996-01-01 00:00:00','Loral Vought' from dual union all
select 'LVN','Lockheed Aircraft Corp, Plant B9, Missile Systems Division (X7)','business','LAC','US','Van Nuys, California','1954-01-01 00:00:00','1960-01-01 00:00:00','Lockheed Aircraft Corp, Plant B-9, Missile Systems Division (X-7)' from dual union all
select 'LW','German Air Force','defense','','D','Koln?','1956-01-01 00:00:00','','Luftwaffe (Bundeswehr)' from dual union all
select 'LY','Libya','','','LY','Al Libiyyah','1951-12-24 00:00:00','','الجماهيرية العربية الليبية الشعبية الاشتراكية العظمى' from dual union all
select 'LYK','OneSpace','business','','CN','Chongqing Liangjian New Area','2015-01-01 00:00:00','','北京零壹空间科技有限公司' from dual union all
select 'M42','M42 Technologies','business','','US','Seattle, Washington','2014-08-01 00:00:00','','M42 Technologies' from dual union all
select 'MA','Morocco','','','MA','Al Maghrib','1956-03-02 00:00:00','','المملكة المغربية' from dual union all
select 'MAI','Ministry of Astronautics Industry','civilian','','CN','Beijing','1982-04-01 00:00:00','1988-07-01 00:00:00','航天工业部' from dual union all
select 'MAI2','Ministry of Aerospace Industry','civilian','','CN','Beijing','1988-07-01 00:00:00','1993-06-01 00:00:00','航空航天工业部' from dual union all
select 'MAIMD','Maryland Aerospace','business','','US','Crofton, Maryland','2004-01-01 00:00:00','','Maryland Aerospace' from dual union all
select 'MAIO','Moscow Aviation Institute, Student KB Iskra','civilian','','SU','Moskva','1933-01-01 00:00:00','','Московский авиационный институт им. Серго Орджоникидзе, СКБ «Искра»' from dual union all
select 'MAK','Makeev Design Bureau','business','','RU','Miass, Chelyabinsk','1993-01-01 00:00:00','','Государственный ракетный центр имени академика В.П. Макеева' from dual union all
select 'MARCO','Marconi','business','','UK','Portsmouth','','','Marconi' from dual union all
select 'MARQ','Marquardt/Van Nuys','business','','US','Van Nuys, California','1948-01-01 00:00:00','1990-01-01 00:00:00','Marquardt/Van Nuys' from dual union all
select 'MARQO','Marquardt/Ogden','business','','US','Ogden, Utah','1957-01-01 00:00:00','','Marquardt/Ogden' from dual union all
select 'MARTB','MartinBaltimore','business','','US','Baltimore, Maryland','','1961-01-01 00:00:00','Martin-Baltimore' from dual union all
select 'MARTD','MartinDenver','business','','US','Denver:Littleton, Colorado','1955-01-01 00:00:00','1961-01-01 00:00:00','Martin-Denver' from dual union all
select 'MARTO','Martin/Orlando','business','','US','Orlando, Florida','1957-01-01 00:00:00','1961-01-01 00:00:00','Martin/Orlando' from dual union all
select 'MASTEN','Masten Space Systems, R&D Shop','business','','US','Mojave, California','2004-01-01 00:00:00','','Masten Space Systems, R&D Shop' from dual union all
select 'MATRA','MATRA Space','business','','F','Paris:Velizy?','','1989-01-01 00:00:00','MATRA Espace' from dual union all
select 'MATT','MATRA Space','business','','F','Toulouse','1968-01-01 00:00:00','1989-01-01 00:00:00','MATRA Espace' from dual union all
select 'MATV','MATRA Space','business','','F','Paris:Velizy','','1989-01-01 00:00:00','MATRA Espace (Velizy)' from dual union all
select 'MAXAR','Maxar Technologies, Ltd','business','','CA','Richmond, British Columbia','2017-10-01 00:00:00','','MacDonald, Dettwiler and Associates' from dual union all
select 'MAXARB','MDA Robotics','business','MAXAR','CA','Brampton, Ontario','2017-10-01 00:00:00','','MDA (MacDonald Dettwiler Assoc.) Robotics' from dual union all
select 'MAXARM','Maxar Satellite Systems','business','MAXAR','CA','Montreal:St-Anne-de-Bellevue, Quebec','2017-10-01 00:00:00','','MDA Satellite Systems' from dual union all
select 'MBAK','MartinBaker Aircraft Ltd','business','','UK','Higher Denham, Buckinghamshire','','','Martin-Baker Aircraft Ltd' from dual union all
select 'MBB','MBB','business','','D','Munchen:Ottobrunn','1968-01-01 00:00:00','1989-12-01 00:00:00','Messerschmitt-Bölkow-Blohm GMbH' from dual union all
select 'MBBE','MBBERNO','business','MBB','D','Bremen','1980-12-01 00:00:00','1989-12-01 00:00:00','MBB-ERNO (Entwicklungsring Nord)' from dual union all
select 'MBBM','MBB (Munchen, Junkers plant)','business','','D','Munchen','1969-01-01 00:00:00','1975-01-01 00:00:00','MBB München' from dual union all
select 'MBCO','Mobile Broadcasting Corp','business','','J','Tokyo:Shinagawa','1998-05-01 00:00:00','2009-03-01 00:00:00','モバイル放送株式会社' from dual union all
select 'MBRSC','Mohammed bin Rashid Space Centre, EIAST','civilian','EIAST','UAE','Dubai','2015-05-01 00:00:00','','Mohammed bin Rashid Space Centre, EIAST' from dual union all
select 'MC','Monaco','','','MC','Monaco','1297-01-01 00:00:00','','Principauté de Monaco' from dual union all
select 'MCDA','McDonnell Aircraft','business','','US','St Louis, Missouri','1938-01-01 00:00:00','1967-01-01 00:00:00','McDonnell Aircraft' from dual union all
select 'MCI','MCI Communications Corp.','business','','US','Washington, DC','1963-01-01 00:00:00','2003-01-01 00:00:00','MCI Communications Corp.' from dual union all
select 'MDA','Missile Defense Agency','defense','','US','Washington,DC:Pentagon','2002-01-01 00:00:00','','Missile Defense Agency' from dual union all
select 'MDABC','MacDonald, Dettwiler and Associates','business','','CA','Richmond, British Columbia','1990-01-01 00:00:00','2017-10-01 00:00:00','MacDonald, Dettwiler and Associates' from dual union all
select 'MDAC','McDonnell Douglas Astronautics/HB','business','MDC','US','Huntington Beach, California','1967-01-01 00:00:00','1996-12-01 00:00:00','McDonnell Douglas Astronautics Co.' from dual union all
select 'MDAIOC','MDA Missile Defense Integration and Operations Center, Schreiver AFB','defense','','US','Colorado Springs, Colorado','2011-01-01 00:00:00','','MDA Missile Defense Integration and Operations Center, Schreiver AFB' from dual union all
select 'MDAM','MDA Satellite Systems','business','MDABC','CA','Montreal:St-Anne-de-Bellevue, Quebec','2005-01-01 00:00:00','2017-10-01 00:00:00','MDA Satellite Systems' from dual union all
select 'MDASL','MDA/St Louis','business','MDC','US','St Louis, Missouri','1967-01-01 00:00:00','1996-01-01 00:00:00','MDA/St Louis' from dual union all
select 'MDC','McDonnell Douglas','business','','US','St Louis, Missouri','1967-01-01 00:00:00','1996-01-01 00:00:00','McDonnell Douglas' from dual union all
select 'MDD','Met Service of the DDR','civilian','','DD','Berlin:Potsdam?','1950-01-01 00:00:00','1991-01-01 00:00:00','Meteorologischer Dienst der DDR' from dual union all
select 'MDDI','Italian Defense Administration','defense','','I','Roma','','','Ministero della Difesa, Amministrazione Difesa' from dual union all
select 'MDHSV','McDonnell Douglas/Huntsville','business','MDC','US','Huntsville, Alabama','1962-01-01 00:00:00','1997-01-01 00:00:00','McDonnell Douglas Aerospace - Huntsville' from dual union all
select 'MDPE','Ministry of Defense of Peru','defense','','PE','Lima','1987-01-01 00:00:00','','Ministerio de Defensa del Perú' from dual union all
select 'MDR','MDA Robotics','business','MDABC','CA','Brampton, Ontario','1999-03-01 00:00:00','2017-10-01 00:00:00','MDA (MacDonald Dettwiler Assoc.) Robotics' from dual union all
select 'MDSSC','McDonnell Douglas/HB','business','MDC','US','Huntington Beach, California','','','McDonnell Douglas Space Systems Corp. (MDSSC-HB)' from dual union all
select 'MEAS','Measat','business','','MY','Cyberjaya, Sepang, Selangor','2001-01-01 00:00:00','','MEASAT Satellite Systems Sdn Bhd.' from dual union all
select 'MEGS','Meggiorin Group','business','','I','Brescia, Italia','','','Meggiorin Group, Brescia' from dual union all
select 'MEI','Moscow Power Engineering Institute','academic/non-profit','','SU','Moskva','1930-01-01 00:00:00','','Московский энергетический институт' from dual union all
select 'MEISEI','Meisei Electric Co. Ltd','business','','J','Tokyo','1938-01-01 00:00:00','','明星電気株式会社' from dual union all
select 'MEIT','MEI Technologies','business','','US','Houston, Texas','2005-01-01 00:00:00','','MEI Technologies' from dual union all
select 'MELB','Melbourne Uni./Australis','academic/non-profit','','AU','Melbourne, Victoria','1853-01-01 00:00:00','','Project Australis, Melbourne University' from dual union all
select 'MELCO','Mitsubishi Electric Co.','business','','J','Kanagawa','','','三菱電機（株） 鎌倉製作所' from dual union all
select 'METCA','Meteorological Service of Canada, Dept. of the Environment','civilian','','CA','Toronto:Downsview','2002-01-01 00:00:00','','Meteorological Service of Canada, Dept. of the Environment' from dual union all
select 'METP','Ministry of the Electrotechnical Industry','civilian','','SU','Mosvka','1965-01-01 00:00:00','1991-01-01 00:00:00','Министерство электротехнической промышленности СССР' from dual union all
select 'MEWTA','White Sands/MEWTA','defense','WSMR','US','White Sands, New Mexico','1960-01-01 00:00:00','1970-01-01 00:00:00','Miissile Electronic Warfare Techincal Area, WSMR' from dual union all
select 'MGON','MGON','defense','','SU','Moskva','1953-01-01 00:00:00','1991-12-01 00:00:00','Министерство геологии и охраны недр' from dual union all
select 'MGU','Moscow State University','academic/non-profit','','RU','Moskva','1755-01-01 00:00:00','','Московский государственный университет имени М. В. Ломоносова' from dual union all
select 'MH','Marshall Is.','','','MH','Majuro, Marshall Is.','1979-01-01 00:00:00','','Republic of the Marshall Islands' from dual union all
select 'MHI','Mitsubishi Heavy Industries (Launch Services)','business','','J','Tokyo:Minato','1964-01-01 00:00:00','','三菱重工業株式会社' from dual union all
select 'MHIKO','MHI Aerospace Systems, Komaki Minami Plant','business','','J','Nagoya:Komaki (Nagoya Airport)','','','三菱重工業 小牧南工場' from dual union all
select 'MHIME','MHI Guidance and Propulsion (Meiyo)','business','','J','Nagoya:Komaki','','','名古屋誘導推進システム製作所' from dual union all
select 'MHITO','MHI Aerospace Systems, Tobishima Plant','business','','J','Nagoya:Tobishima','','','三菱重工業名古屋航空宇宙システム製作所   飛島工場' from dual union all
select 'MICOM','US Army Missile Command','defense','','US','Huntsville, Alabama','1963-05-01 00:00:00','1997-10-01 00:00:00','US Army Missile Command' from dual union all
select 'MICRC','MicroCraft Inc.(Allied Aerospace)','business','','US','Tullahoma, Tennessee','1958-01-01 00:00:00','1999-01-01 00:00:00','MicroCraft Inc.(Allied Aerospace)' from dual union all
select 'MICRC2','MicroCraft Inc','business','','US','Tullahoma, Tennessee','2011-03-01 00:00:00','','MicroCraft Inc' from dual union all
select 'MICRCA','MicroCraft Inc.(Allied Aerospace)','business','','US','Tullahoma, Tennessee','1999-01-01 00:00:00','2003-11-01 00:00:00','MicroCraft Inc.(Allied Aerospace)' from dual union all
select 'MICRL','MicroSat Lt  (IAI/Rafael)','business','','IL','Tel Aviv?','2009-01-01 00:00:00','','MicroSat Lt  (IAI/Rafael)' from dual union all
select 'MICROC','Microcosm Inc.','business','','US','Hawthorne, California','','','Microcosm Inc.' from dual union all
select 'MICSG','MicroSpace RAPID Pte Ltd.','business','','SG','Singapore','2002-01-01 00:00:00','','MicroSpace RAPID Pte Ltd.' from dual;
insert into ORGANIZATION
select 'MICSP','MicroSpace Inc.','business','','US','Denver, Colorado','1998-01-01 00:00:00','2012-01-01 00:00:00','Micro-Space Inc.' from dual union all
select 'MICSY','Microsat Systems Inc (MSI)','business','SIENV','US','Denver:Littleton, Colorado','2001-01-01 00:00:00','','Microsat Systems Inc (MSI)' from dual union all
select 'MILAN','University of Milan','academic/non-profit','','I','Milano','1924-01-01 00:00:00','','Università degli Studi di Milano' from dual union all
select 'MILJPO','US Air Force Milsatcom Joint Project Office, AFSMC, LAAFB','defense','AFSMC','US','Los Angeles:El Segundo, California','1990-01-01 00:00:00','2006-01-01 00:00:00','US Air Force Milsatcom Joint Project Office, AFSMC, LAAFB' from dual union all
select 'MILLEI','Millenium Engeering and Integration Services, LLC','business','','US','Arlington, Virginia','1995-01-01 00:00:00','','Millenium Engineering and Integration Services, LLC' from dual union all
select 'MILLEN','Millenium Space Systems','business','','US','Torrance, California','2001-01-01 00:00:00','','Millenium Space Systems' from dual union all
select 'MILLES','Millenium Space Systems/El Segundo factory','business','MILLEN','US','El Segundo, California','2014-01-01 00:00:00','','Millenium Space Systems/El Segundo factory' from dual union all
select 'MILTEC','Miltec Missiles and Space Co.','business','','US','Huntsville, Alabama','2010-01-01 00:00:00','2012-01-01 00:00:00','Miltec Missiles and Space Co.' from dual union all
select 'MINCYT','Ministry of Science Technology and Productive Innovation','civilian','','AR','Buenos Aires','','','Ministerio de Ciencia, Tecnología e Innovación Productiva' from dual union all
select 'MINES','Ecole Nationale Superieure des Mines','academic/non-profit','','F','Paris','1783-01-01 00:00:00','','École Nationale Supérieure des Mines de Paris (MINES ParisTech)' from dual union all
select 'MIRC','Mission Research Corp (NH)','business','','US','Nashua, New Hampshire','','2004-01-01 00:00:00','Mission Research Corp (NH)' from dual union all
select 'MISU','Meteorological Institute Stockholm Univ.','academic/non-profit','STO','S','Stockholm','1947-01-01 00:00:00','1973-01-01 00:00:00','Meteorologiska Institutionen Stockholms Universitet' from dual union all
select 'MISUF','Meteorological Institute Stockholm Univ.','academic/non-profit','STO','S','Stockholm:Frescati','1973-01-01 00:00:00','','Met Inst. Stockholm  (Frescati campus)' from dual union all
select 'MIT','Mass. Inst of Technology','academic/non-profit','','US','Cambridge, Massachusetts','1916-01-01 00:00:00','','Mass. Inst of Technology' from dual union all
select 'MITLL','Lincoln Labs, Mass. Inst of Technology','academic/non-profit','','US','Lexington, Massachusetts','1951-01-01 00:00:00','','Lincoln Labs, Mass. Inst of Technology' from dual union all
select 'MITT','Moscow Inst of Thermal Technology','business','','SU','Mosvka (NE)','1946-01-01 00:00:00','','Московский институт теплотехники' from dual union all
select 'MM','Martin Marietta','business','','US','Denver, Colorado','1961-01-01 00:00:00','1995-01-01 00:00:00','Martin Marietta' from dual union all
select 'MMA','Martin Marietta Astronautics','business','MM','US','Denver, Colorado','1961-01-01 00:00:00','1995-03-15 00:00:00','Martin Marietta Astronautics' from dual union all
select 'MMAS','Martin Marietta Astro Space/East Windsor','business','MM','US','East Windsor, New Jersey','1993-01-01 00:00:00','1995-03-15 00:00:00','Martin Marietta Astro Space/East Windsor' from dual union all
select 'MMCLS','Martin Marietta Commercial Launch Services','business','MM','US','San Diego, California','1994-05-01 00:00:00','1995-03-15 00:00:00','Martin Marietta Commercial Launch Services' from dual union all
select 'MMD','Martin Marietta/Denver','business','MM','US','Denver:Littleton, Colorado','1961-01-01 00:00:00','1995-03-15 00:00:00','Martin Marietta/Denver' from dual union all
select 'MMEW','Martin Marietta Astro Space/East Windsor','business','MM','US','East Windsor, New Jersey','1993-01-01 00:00:00','1995-03-15 00:00:00','Martin Marietta Astro Space/East Windsor' from dual union all
select 'MMMI','Martin Marietta/Michoud','business','MM','US','Michoud, Louisiana','1973-01-01 00:00:00','1995-03-15 00:00:00','Martin Marietta/Michoud' from dual union all
select 'MMO','Martin Marietta/Orlando','business','','US','Orlando, Florida','1961-01-01 00:00:00','1995-03-15 00:00:00','Martin Marietta/Orlando' from dual union all
select 'MMPCO','Martin Marietta Pike County Operations','business','LM','US','Troy, Alabama','1994-01-01 00:00:00','1995-03-15 00:00:00','Martin Marietta Pike County Operations' from dual union all
select 'MMSD','Martin Marietta (San Diego), AF Plant 19','business','MM','US','San Diego:Kearny Mesa, California','1994-05-02 00:00:00','1995-03-15 00:00:00','Martin Marietta (San Diego), AF Plant 19' from dual union all
select 'MMSS','Matra Marconi Space (Stevenage)','business','','UK','Stevenage','1994-01-01 00:00:00','2000-01-01 00:00:00','Matra Marconi Space (Stevenage)' from dual union all
select 'MMST','Matra Marconi Space (Toulouse)','business','','F','Toulouse','1989-01-01 00:00:00','2000-01-01 00:00:00','Matra Marconi Space (Toulouse)' from dual union all
select 'MMSV','Matra Marconi Space','business','','F','Paris:Velizy','1989-01-01 00:00:00','2000-01-01 00:00:00','Matra Marconi Space' from dual union all
select 'MMVF','Martin Marietta/Valley Forge','business','MM','US','Vally Forge, Penn.','1993-01-01 00:00:00','1995-01-01 00:00:00','Martin Marietta/Valley Forge' from dual union all
select 'MN','Mongolia','','','MN','Ulan Bator','1911-01-01 00:00:00','','Монгол Улс' from dual union all
select 'MNF','Meteorologie Nationale, France','civilian','','F','Paris','1945-01-01 00:00:00','1993-01-01 00:00:00','La Météorologie nationale' from dual union all
select 'MO','MinOboroni','defense','','SU','Moskva','1953-03-01 00:00:00','1991-12-01 00:00:00','Министерство обороны СССР' from dual union all
select 'MOD','Ministry of Defence, London','defense','','UK','London:Whitehall','1964-01-01 00:00:00','','Ministry of Defence, London' from dual union all
select 'MOLN','NPO Molniya, Tushino Mash. Zavod.','business','','SU','Moskva:Tushino','1976-01-01 00:00:00','','НПО Молния' from dual union all
select 'MOM','MOM','defense','','SU','Moskva','1965-03-01 00:00:00','1992-02-01 00:00:00','Министерство общего машиностроения СССР' from dual union all
select 'MONBU','Ministry of Education, Science and Culture','civilian','J','J','Tokyo','1950-01-01 00:00:00','2001-01-01 00:00:00','文部省' from dual union all
select 'MONKA','Ministry of Education, Culture, Sports, Science and Tech.','civilian','J','J','Tokyo','2001-01-01 00:00:00','','文部科学省' from dual union all
select 'MONTP2','Montpellier 2 University','academic/non-profit','','F','Montpellier, Herault','1970-01-01 00:00:00','','Université Montpellier 2' from dual union all
select 'MOOG','Moog Inc.','business','','US','Buffalo/East Aurora, New York','1951-01-01 00:00:00','','Moog Inc.' from dual union all
select 'MOOGC','Moog CSA','business','MOOG','US','Mountain View, California','2008-01-01 00:00:00','','Moog CSA' from dual union all
select 'MOOGNF','Moog InSpace Propulsion','defense','MOOG','US','Niagara Falls, New York','2012-08-01 00:00:00','','Moog In-Space Propulsion' from dual union all
select 'MOOGW','Moog InSpace Propulsion (UK)','defense','MOOG','UK','Westcott, Buckinghamshire','2012-08-01 00:00:00','','Moog In-Space Propulsion (UK)' from dual union all
select 'MORF','Russian Ministry of Defense','defense','','RU','Moskva','1991-12-01 00:00:00','','Министерство обороны Российской Федерации' from dual union all
select 'MOTI','Motient Corporation','business','','US','Reston, Virginia','2000-01-01 00:00:00','2007-01-01 00:00:00','Motient Corporation' from dual union all
select 'MOTO','Motorola Satellite Communications','business','','US','Phoenix:Chandler, Arizona','1990-01-01 00:00:00','2008-09-01 00:00:00','Motorola Satellite Communications' from dual union all
select 'MOTOS','Motor Construction','engine/motor manufacturer','','RU','Samara','1912-01-01 00:00:00','','ОАО "Моторостроитель"' from dual union all
select 'MOZH','Mozhaiskiy Red Banner Military Space Engineering Institute','academic/non-profit','','RU','Santk:Peterburg','1955-01-01 00:00:00','1990-01-01 00:00:00','Военный инженерный Краснознаменный институт имени А. Ф. Можайского' from dual union all
select 'MOZHA','Mozhaiskiy Military Space Academy','academic/non-profit','','RU','Santk:Peterburg','1990-01-01 00:00:00','','Военно-космическая академия им. А. Ф. Можайского' from dual union all
select 'MPAE','MPI Aeronomy','academic/non-profit','','D','Lindau','1957-01-01 00:00:00','2004-07-01 00:00:00','Max-Planck-Institut für Aeronomie' from dual union all
select 'MPE','Max Planck Institut fur Extraterrestrische Physik','academic/non-profit','','D','Garching','','','Max-Planck-Institut für extraterrestrische Physik' from dual union all
select 'MPIH','Max Planck Institut fur Astronomie','academic/non-profit','','D','Heidelberg','','','Max-Planck-Institut für Astronomie' from dual union all
select 'MPIK','MaxPlanckInstitut fur Kernphysik','academic/non-profit','','D','Heidelberg','1958-01-01 00:00:00','','Max-Planck-Institut für Kernphysik' from dual union all
select 'MPIS','Max Planck Institut fur Sonnensystemforschung','academic/non-profit','','D','Lindau','2004-07-01 00:00:00','','Max-Planck-Institut für Sonnensystemforschung' from dual union all
select 'MPSC','Mabuhay Philippines Satellite Corp','business','','PH','Subic Bay','1996-01-01 00:00:00','2009-11-01 00:00:00','Mabuhay Philippines Satellite Corp' from dual union all
select 'MPSC1','Asia Broadcast Satellite (MPSC division)','business','ABS','PH','Subic Bay','2009-11-01 00:00:00','2009-12-01 00:00:00','Asia Broadcast Satellite (MPSC division)' from dual union all
select 'MPU','Moscow Polytechnic University','academic/non-profit','','RU','Moskva','2016-03-01 00:00:00','','Московский политехнический университет' from dual union all
select 'MR','Mauritania','','','MR','Nouakchott','1960-11-28 00:00:00','','الجمهورية الإسلامية  الموريتانية' from dual union all
select 'MRI','JMA Meteorological Research Institute','civilian','JMA','J','Tokyo:Suginami','1945-01-01 00:00:00','1980-01-01 00:00:00','気象研究所' from dual union all
select 'MRITS','JMA Meteorological Research Institute','civilian','JMA','J','Tsukuba','1980-01-01 00:00:00','','気象研究所 筑波研究学園都市' from dual union all
select 'MRN','Meteorological Rocket Network','defense','','US','White Sands, New Mexico','1959-01-01 00:00:00','','Meteorological Rocket Network' from dual union all
select 'MSBTR','MND','defense','','TR','Ankara','1970-01-01 00:00:00','','Millî Savunma Bakanlığı' from dual union all
select 'MSC','NASA Manned Spacecraft Center','civilian','NASA','US','Houston:Clear Lake, Texas','1962-03-01 00:00:00','1973-02-17 00:00:00','NASA Manned Spacecraft Center' from dual union all
select 'MSCI','Microsat Systems Canada Inc','business','','CA','Toronto:Mississauga, Ontario','2009-01-01 00:00:00','','Microsat Systems Canada Inc' from dual union all
select 'MSFC','NASA Marshall Space Flight Center','civilian','NASA','US','Huntsville, Alabama','1960-01-01 00:00:00','','NASA Marshall Space Flight Center' from dual union all
select 'MSGC','Montana Space Grant Consortium, Montana State University, Bozeman','academic/non-profit','MSU','US','Bozeman, Montana','2010-01-01 00:00:00','','Montana Space Grant Consortium, Montana State University, Bozeman' from dual union all
select 'MSSL','Mullard Space Science Laboratory','academic/non-profit','UCL','UK','Dorking, Surrey','1966-01-01 00:00:00','','Mullard Space Science Laboratory' from dual union all
select 'MSU','Montana State University, Bozeman','academic/non-profit','','US','Bozeman, Montana','1893-01-01 00:00:00','','Montana State University, Bozeman' from dual union all
select 'MSUKY','Morehead State University','academic/non-profit','','US','Morehead, Kentucky','1887-01-01 00:00:00','','Morehead State University' from dual union all
select 'MSV','Mobile Satellite Ventures L.P.','business','MOTI','US','Reston, Virginia','2000-06-01 00:00:00','2008-12-01 00:00:00','Mobile Satellite Ventures L.P.' from dual union all
select 'MSVYAZ','Ministry of Comms. Union Radio Comms.','civilian','','SU','Moskva','1968-01-01 00:00:00','1992-01-01 00:00:00','Министерство связи СССР Союзный узел радиовещания и радиосвязи №9' from dual union all
select 'MTF','NASA Mississippi Test Facility','civilian','NASA','US','Pearl River, Mississippi','1962-11-01 00:00:00','1988-01-01 00:00:00','NASA Mississippi Test Facility' from dual union all
select 'MTHK','Morton Thiokol Inc.','business','','US','Chicago, Illinois','1982-01-01 00:00:00','1989-07-01 00:00:00','Morton Thiokol Inc.' from dual union all
select 'MU','Mauritius','','','MU','Port Louis, Maurice','1968-03-12 00:00:00','','République de Maurice' from dual union all
select 'MUNIZ','Muniz Engineering','business','','US','Houston, Texas','1992-01-01 00:00:00','2005-01-01 00:00:00','Muniz Engineering' from dual union all
select 'MV','Maldives','','','MV','Male, Male Island','1965-07-26 00:00:00','','ދިވެހިރާއްޖޭގެ ޖުމްހޫރިއްޔާ' from dual union all
select 'MV7','Miinistry of Armaments 7th Directorate','defense','','SU','Moskva','1954-01-01 00:00:00','1959-01-01 00:00:00','7 главное управление Министерство вооружения СССР' from dual union all
select 'MVD','Ministry of Internal Affairs','civilian','','SU','Mosvka','1946-01-01 00:00:00','1992-01-01 00:00:00','Министерство внутренних дел' from dual union all
select 'MVS','Miinistry of the Armed Forces','defense','','SU','Moskva','1946-03-01 00:00:00','1950-02-01 00:00:00','Министерство Вооруженных Сил СССР' from dual union all
select 'MVSM','Voenniy Ministerstvo SSSR (?)','defense','','SU','Moskva','1950-02-01 00:00:00','1953-03-01 00:00:00','Военное министерство СССР' from dual union all
select 'MX','Mexico','','','MX','Mexico','1819-09-16 00:00:00','','Estados Unidos Mexicanos' from dual union all
select 'MY','Malaysia','','','MY','Malaysia','1957-08-31 00:00:00','','Malaysia' from dual union all
select 'MYA','Eksperimental''niy KB im. Myasishchev','defense','','SU','Moskva:Zhukovskiy','1976-01-01 00:00:00','','Экспери­мен­тальный Машин­ост­роительный Завод им. В. М. Мясищева' from dual union all
select 'N','Norway','','','N','Oslo, Norge','0872-01-01 00:00:00','','Kongeriket Norge' from dual union all
select 'NAA','North American Aviation','business','','US','Los Angeles:Inglewood, California','1936-01-01 00:00:00','1973-01-01 00:00:00','North American Aviation' from dual union all
select 'NAAA','North America Aviation  Aerophysics Laboratory','business','NAA','US','Los Angeles:Inglewood, California','1946-01-01 00:00:00','1948-01-01 00:00:00','North America Aviation - Aerophysics Laboratory' from dual union all
select 'NAAD','North American Aviation, Missile Division, Downey','business','NAA','US','Downey, California','1948-07-01 00:00:00','1973-01-01 00:00:00','North American Aviation, Missile Division, Downey' from dual union all
select 'NAADR','North American Aviation, Missile Division, Propulsion Research','business','NAAD','US','Downey, California','1948-07-01 00:00:00','1955-11-01 00:00:00','North American Aviation, Missile Division, Propulsion Research' from dual union all
select 'NAASB','North American Aviation, Seal Beach','business','NAA','US','Seal Beach, California','1965-01-01 00:00:00','1973-01-01 00:00:00','North American Aviation, Seal Beach' from dual union all
select 'NACA','National Advisory Council on Aeronautics','civilian','','US','Washington, DC','1915-01-01 00:00:00','1958-10-01 00:00:00','National Advisory Council on Aeronautics' from dual union all
select 'NACI','NATO Communications and Information Systems Agency','defense','NATO','US','Brussel:Haren','1986-01-01 00:00:00','1996-07-01 00:00:00','NATO Communications and Information Systems Agency' from dual union all
select 'NADA','National Aerospace Development Administration','defense','','KP','Pyongyang','2013-01-01 00:00:00','','국가우주개발국' from dual union all
select 'NAFI','Naval Avionics Facility, Indianapolis','defense','USN','US','Indianapolis, Indiana','1956-01-01 00:00:00','1977-01-01 00:00:00','Naval Avionics Facility, Indianapolis' from dual union all
select 'NAG','Nagoya University','academic/non-profit','','J','Nagoya','1939-01-01 00:00:00','','名古屋大学' from dual union all
select 'NAHU','Nahuelsat SA','business','','AR','Buenos Aires, Argentina','1994-01-01 00:00:00','2010-01-01 00:00:00','Nahuelsat SA' from dual union all
select 'NAHUAR','Nahuelsat SA assets, transferred to ARSAT','business','ARSAT','AR','Buenos Aires, Argentina','2010-01-01 00:00:00','2011-01-01 00:00:00','Nahuelsat SA assets, transferred to AR-SAT' from dual union all
select 'NAIC','National Air Intelligence Center','defense','USAF','US','Wright-Patterson AFB, Dayton, Ohio','1993-10-01 00:00:00','2003-02-01 00:00:00','National Air Intelligence Center' from dual union all
select 'NAIR','University of Nairobi','academic/non-profit','','KE','Nairobi','1970-01-01 00:00:00','','University of Nairobi' from dual union all
select 'NAKA','Nakashimada Engineering Works Ltd.','business','','J','Hirokawa, Fukuoka','','','(株)中島田鉄工所' from dual union all
select 'NAL','Koku Ucyu Gijyutsu Kenkyujyo (National Aerospace Lab)','civilian','','J','Tokyo:Chofu','1961-02-01 00:00:00','2003-10-01 00:00:00','航空宇宙技術研究所' from dual;
insert into ORGANIZATION
select 'NAMFI','NATO Missile Firing Installation, Hellenic Natural Defense General Staff','defense','NATO','GR','Chania, Crete','','','NATO Missile Firing Installation, Hellenic Natural Defense General Staff' from dual union all
select 'NAMMO','Nammo (Nordic Ammunition Group) Raufoss','business','','N','Raufoss','1998-01-01 00:00:00','','Nammo (Nordic Ammunition Group) Raufoss' from dual union all
select 'NAMTC','Naval Air Missile Test Center, Point Mugu','defense','USN','US','Pt. Mugu, California','1946-10-01 00:00:00','1958-06-01 00:00:00','Naval Air Missile Test Center, Point Mugu' from dual union all
select 'NANR','NanoRacks','business','','US','Houston, Texas','2008-01-01 00:00:00','','NanoRacks' from dual union all
select 'NANSFI','NanoSatisfi','civilian','','US','San Francisco, California','2011-01-01 00:00:00','2014-08-01 00:00:00','NanoSatisfi' from dual union all
select 'NAOJ','National Astronomical Observatory of Japan (Mitaka)','civilian','','J','Tokyo:Mitaka','1988-01-01 00:00:00','','国立天文台' from dual union all
select 'NARSS','National Authority for Remote Sensing and Space Science','civilian','','EG','Al-Qahira (Cairo):El Nozha El Gedida','1994-01-01 00:00:00','','National Authority for Remote Sensing and Space Science' from dual union all
select 'NASA','National Aeronautics and Space Administration','civilian','','US','Washington, D.C.','1958-10-01 00:00:00','','National Aeronautics and Space Administration' from dual union all
select 'NASAHQ','National Aeronautics and Space Administration','civilian','NASA','US','Washington, D.C.','1958-10-01 00:00:00','','National Aeronautics and Space Administration' from dual union all
select 'NASDA','National Space Development Adminstration','civilian','','J','Tokyo, Nippon','1969-10-01 00:00:00','2003-10-01 00:00:00','宇宙開発事業団' from dual union all
select 'NASDM','NASDA Micro Space System Lab','civilian','NASDA','J','Tokyo, Nippon','2000-01-01 00:00:00','','宇宙開発事業団 マイクロスペースシステム研究室' from dual union all
select 'NASIC','National Air and Space Intelligence Center','defense','USAF','US','Wright-Patterson AFB, Dayton, Ohio','2003-02-01 00:00:00','','National Air and Space Intelligence Center' from dual union all
select 'NATO','North Atlantic Treaty Organization','defense','','US','Brussel:Haren','','','North Atlantic Treaty Organization' from dual union all
select 'NAVAIR','USN Naval Air Systems Command (NAVAIR)','defense','USN','US','Patuxent River, Maryland','1966-05-01 00:00:00','','USN Naval Air Systems Command (NAVAIR)' from dual union all
select 'NAVAST','USN Naval Astronautics Group','defense','USN','US','Pt Mugu, California','1962-01-01 00:00:00','1990-06-12 00:00:00','USN Naval Astronautics Group' from dual union all
select 'NAVMAT','USN Naval Material Command','defense','USN','US','Washington, DC','1966-05-01 00:00:00','1985-05-06 00:00:00','USN Naval Material Command' from dual union all
select 'NAVSOC','USN Naval Satellite Operations Center','defense','NAVSPA','US','Pt Mugu, California','1990-06-12 00:00:00','','USN Naval Satellite Operations Center' from dual union all
select 'NAVSPA','US Naval Space Command (NAVSPACECOM)','defense','USN','US','Dahlgren, Virginia','1983-10-01 00:00:00','','US Naval Space Command (NAVSPACECOM)' from dual union all
select 'NAVSPOC','Naval Space Operations Center','defense','NAVSPA','US','Dahlgren, Virginia','','','Naval Space Operations Center' from dual union all
select 'NAWC','Naval Air Warfare Center','defense','USN','US','China Lake, California','1992-01-22 00:00:00','','Naval Air Warfare Center' from dual union all
select 'NAXS','NASA HQXS (Office of Space Access and Tech., OSAT/Code X)','civilian','NASA','US','Washington, DC','1994-09-01 00:00:00','1996-09-01 00:00:00','NASA HQ-XS (Office of Space Access and Tech., OSAT/Code X)' from dual union all
select 'NBN','NBN Co. Ltd. (nbn, National Broadband Network Co.)','business','AU','AU','Melbourne, Victoria','2009-04-01 00:00:00','','NBN Co. Ltd. (nbn, National Broadband Network Co.)' from dual union all
select 'NBS','National Bureau of Standards, Central Radio Propagation Lab','civilian','DOC','US','Boulder, Colorado','1954-01-01 00:00:00','1988-01-01 00:00:00','National Bureau of Standards, Central Radio Propagation Lab' from dual union all
select 'NC3A','NATO C3 Agency (NC3A)','defense','NATO','US','Brussel:Haren','1996-07-01 00:00:00','','NATO C3 Agency (NC3A)' from dual union all
select 'NCAR','National Center for Atmospheric Research','civilian','','US','Boulder, Colorado','1960-01-01 00:00:00','','National Center for Atmospheric Research' from dual union all
select 'NCKU','National Cheng Kung University','academic/non-profit','','TW','Tainan City','1931-01-01 00:00:00','','國立成功大學' from dual union all
select 'NCST','Naval Center for Space Technology','defense','NRL','US','Washington, DC','1980-01-01 00:00:00','','Naval Center for Space Technology' from dual union all
select 'NCU','National Central University  Shuanglianpo','academic/non-profit','','TW','Jhongli:Shuanglianpo','1968-01-01 00:00:00','','國立中央大學' from dual union all
select 'NDAWN','New Dawn','business','INTELS','MU','Port Louis, Maurice','2009-01-01 00:00:00','','New Dawn Satellite Co. (Intelsat/Convergence Partners)' from dual union all
select 'NDRE','Norwegian Defense Research Establishment','defense','','N','Oslo:Kjeller','1946-01-01 00:00:00','','Forsvarets forskningsinstitutt (FFI)' from dual union all
select 'NEC','Nippon Electric Corp., Space Div., Kamoi Works','business','','J','Kamoi, Shin:Yokohama','','2001-01-01 00:00:00','日本電気 (NEC) 横浜事業場' from dual union all
select 'NECST','NEC Space Technology Co. Ltd.','business','','J','Kawasaki City, Kanagawa','2015-05-15 00:00:00','','NECスペーステクノロジー株式会社' from dual union all
select 'NECT','NEC Toshiba Space','business','','J','Kawasaki CIty, Kanagawa','2001-01-01 00:00:00','2015-05-15 00:00:00','NEC東芝スペースシステム株式会社' from dual union all
select 'NECTF','NEC Toshiba Space, Fuchu works','business','NECT','J','Tokyo:Fuchu','2005-01-01 00:00:00','','ＮＥＣ東芝スペースシステム株式会社 府中事業所' from dual union all
select 'NECTKA','NEC Toshiba Space, Kamoi works','business','NECT','J','Kamoi, Tokyo:Shin:Yokohama','2001-01-01 00:00:00','2005-01-01 00:00:00','NEC東芝スペースシステム株式会社  鴨居事業所' from dual union all
select 'NECTKE','NEC Space, Keihin works','business','NECT','J','Kawasaki CIty, Kanagawa','2001-01-01 00:00:00','','NEC東芝スペースシステム京浜工場' from dual union all
select 'NESC','ESSA National Environmental Satellite Center','civilian','ESSA','US','Suitland, Maryland','1965-01-01 00:00:00','1970-10-01 00:00:00','ESSA National Environmental Satellite Center' from dual union all
select 'NESCOM','Pakistan National Engineering and Scientific Commission','defense','','PK','Islamabad','2001-01-01 00:00:00','','Pakistan National Engineering and Scientific Commission' from dual union all
select 'NESCOMN','NESCOM, Pakistan National Defense Complex','defense','NESCOM','PK','Fateh Jang','2001-01-01 00:00:00','','NESCOM, Pakistan National Defense Complex' from dual union all
select 'NESCOMP','NESCOM, Project Management Organization, Pakistan','civilian','NESCOM','PK','Khanpur','2001-01-01 00:00:00','','NESCOM, Project Management Organization, Pakistan' from dual union all
select 'NESD','National Environmental Satellite, Data and Information Service','civilian','NOAA','US','Suitland, Maryland','1980-08-01 00:00:00','','National Environmental Satellite, Data and Information Service' from dual union all
select 'NESS','National Environmental Satellite Service','civilian','NOAA','US','Suitland, Maryland','1970-10-01 00:00:00','1980-08-01 00:00:00','National Environmental Satellite Service' from dual union all
select 'NEU','Northeastern University','academic/non-profit','','US','Boston, MA','1898-01-01 00:00:00','','Northeastern University' from dual union all
select 'NEWSAT','NewSat USA LLC','business','','US','Allendale, New Jersey','2000-01-01 00:00:00','2004-05-01 00:00:00','NewSat USA LLC' from dual union all
select 'NEWSAU','NewSat','business','','AU','Melbourne, Victoria','2005-01-01 00:00:00','','NewSat' from dual union all
select 'NG','Nigeria','','','NG','Abuja, Federal Capital Territory','1960-10-01 00:00:00','','Federal Republic of Nigeria' from dual union all
select 'NGA','National Geospatial Intelligence Agency, Ft Belvoir','defense','','US','Springfield:Ft Belvoir, Virginia','2009-01-01 00:00:00','','National Geospatial Intelligence Agency, Ft Belvoir' from dual union all
select 'NGAA','NG Astro Aerospace, Carpenteria','business','NGST','US','Carpenteria, California','2002-01-01 00:00:00','','NG Astro Aerospace, Carpenteria' from dual union all
select 'NGAS','Northrop Grummann Aerospace Systems','business','','US','Redondo Beach, California','2009-01-01 00:00:00','','Northrop Grummann Aerospace Systems' from dual union all
select 'NGCOM','Nigcomsat (Nigerian Comm. Sat. Ltd, Federal Ministry of Science and Tech)','business','','NG','Abuja, Federal Capital Territory','2004-01-01 00:00:00','','Nigcomsat (Nigerian Comm. Sat. Ltd, Federal Ministry of Science and Tech)' from dual union all
select 'NGIS','Northrop Grumman Innovation Systems','business','','US','Washington:Dulles, Virginia','2018-06-07 00:00:00','','Orbital ATK' from dual union all
select 'NGISABL','NGIS/Orbital ATK DSG Missile Products (Rocket Center, WV)','business','OATK','US','Cumberland:Rocket Center, West Virginia','2018-06-07 00:00:00','','NGIS Missile Products (Rocket Center, WV)' from dual union all
select 'NGISB','NGIS Defense Systems Group / Missile Products','business','OATK','US','Baltimore, Maryland','2018-06-07 00:00:00','','NGIS Defense Systems Group / Missile Products' from dual union all
select 'NGISC','NGIS Flight Systems Group','business','OATK','US','Phoenix:Chandler, Arizona','2018-06-07 00:00:00','','NGIS Flight Systems Group' from dual union all
select 'NGISD','Northrop Grumman Space Systems Group (SSG)','business','NGIS','US','Washington:Dulles, Virginia','2018-06-07 00:00:00','','Northrop Grumman Space Systems Group (SSG)' from dual union all
select 'NGISE','NGIS DSG Missile Products (Elkton)','business','NGIS','US','Elkton, Maryland','2018-06-07 00:00:00','','NGIS DSG Missile Products (Elkton)' from dual union all
select 'NGISG','Northrop Grumman DSG Missile Products (Ronkonkoma)','business','NGIS','US','Ronkonkoma, New York','2018-06-07 00:00:00','','Northrop Grumman DSG Missile Products (Ronkonkoma)' from dual union all
select 'NGISGA','Northrop Grumman Civil Sats. and Nat. Security Systems','business','NGIS','US','Gilbert, Arizona','2018-06-07 00:00:00','','Northrop Grumman Civil Sats. and Nat. Security Systems' from dual union all
select 'NGISGO','Northrop Grumman Space Components (Goleta)','business','NGIS','US','Goleta, California','2018-06-07 00:00:00','','Northrop Grumman Space Components (Goleta)' from dual union all
select 'NGISH','NGIS / ATK Aerospace Group Headquarters','business','NGIS','US','Salt Lake City:West Valley City (Magna), Utah','2018-06-07 00:00:00','','NGIS / ATK Aerospace Group Headquarters' from dual union all
select 'NGISP','NGIS FSG Test Services (Promontory)','business','NGIS','US','Brigham City:Promontory, Utah','2018-06-07 00:00:00','','NGIS FSG Test Services (Promontory)' from dual union all
select 'NGISS','Northrop Grumman Space Systems (Beltsville)','business','NGIS','US','Beltsville, Maryland','2018-06-07 00:00:00','','Northrop Grumman Space Systems (Beltsville)' from dual union all
select 'NGISW','NGIS Launch Systems (Wallops)','business','NGIS','US','Wallops Island, Virginia','2018-06-07 00:00:00','','NGIS Launch Systems (Wallops)' from dual union all
select 'NGNW','Northrop Grumman NovaWorks','business','NGST','US','Redondo Beach?, California','2010-01-01 00:00:00','','Northrop Grumman NovaWorks' from dual union all
select 'NGST','Northrop Grummann Space Technology','business','','US','Redondo Beach, California','2002-07-01 00:00:00','2009-01-01 00:00:00','Northrop Grummann Space Technology' from dual union all
select 'NHK','Japan Broadcasting Corp.','business','','J','Tokyo','1925-03-01 00:00:00','','日本放送協会' from dual union all
select 'NICP','Norwegian Institute for Cosmic Physics','civilian','','N','Tromso','1927-01-01 00:00:00','1972-01-01 00:00:00','Norsk Institutt for Kosmisk Fysikk' from dual union all
select 'NICS','NATO Integrated Communications Management Agency','defense','NATO','US','Brussel:Haren','1971-03-01 00:00:00','1986-01-01 00:00:00','NATO Integrated Communications Management Agency' from dual union all
select 'NIHON','Nihon Daigaku','academic/non-profit','','J','Tokyo:Chiyoda','1889-01-01 00:00:00','','日本大学' from dual union all
select 'NII1','Nauchnoissledovatel''niy institut 1  (RNII)','civilian','','SU','Moskva:Khovrino','1950-01-01 00:00:00','1965-01-01 00:00:00','НИИ-1' from dual union all
select 'NII125','NII125 (Zhukov)','business','','SU','Mosvka:Lyubertsy/Dzerzhinskiy','1947-01-01 00:00:00','1980-01-01 00:00:00','НИИ-125' from dual union all
select 'NII1M','NII1 Fil. (Sverdlovsk)','engine/motor manufacturer','','SU','Nishnaya Salda, Sverdlovsk Oblast','1958-01-01 00:00:00','1981-01-01 00:00:00','НИИ-1 филиал' from dual union all
select 'NII88','NII88 (NauchnoIssledovatel''niy Institut 88)','defense','','SU','Moskva:Podlipki','1946-05-01 00:00:00','1967-01-01 00:00:00','НИИ-88' from dual union all
select 'NII88A','NII88 civilian launches (not in RVSN list)','defense','','SU','Moskva:Podlipki','1946-05-01 00:00:00','1967-01-01 00:00:00','НИИ-88' from dual union all
select 'NIIEM','FGUP NII Elektromekhaniki','civilian','VNIEM','SU','Mosvka:Istra','1960-01-01 00:00:00','','НИИ электромеханики' from dual union all
select 'NIIKP','RISDE','civilian','','SU','Moskva:Lefortovo','1978-01-01 00:00:00','2009-01-01 00:00:00','НИИ космического приборостроения' from dual union all
select 'NIIM','NII Mashinostroeniya','engine/motor manufacturer','','SU','Nishnaya Salda, Sverdlovsk Oblast','1981-01-01 00:00:00','','НИИ машиностроения' from dual union all
select 'NIIP','Instute for Instrument Building','civilian','','SU','Moskva:Lefortovo','1963-01-01 00:00:00','1978-01-01 00:00:00','НИИ приборостроения' from dual union all
select 'NIIPP','IPIE','civilian','','SU','Mosvka','1986-01-01 00:00:00','','НИИ прецизионного приборостроения' from dual union all
select 'NIIRP','NII RadioPriborostroeniya','defense','','SU','Mosvka:Sokol','1975-01-01 00:00:00','','НИИ Радиоприбор' from dual union all
select 'NIITP','NII Thermal Processes (Keldysh)','civilian','','SU','Moskva:Khovrino','1965-01-01 00:00:00','1992-01-01 00:00:00','НИИ тепловых процессов (НИИТП)' from dual union all
select 'NIKEX','NIKEX Project Office','defense','USA','US','Huntsville, Alabama?','1964-02-01 00:00:00','1967-01-01 00:00:00','NIKE-X Project Office' from dual union all
select 'NIKEZ','NikeZeus Project Office','defense','USAMC','US','Huntsville, Alabama?','1961-12-01 00:00:00','1964-02-01 00:00:00','Nike-Zeus Project Office' from dual union all
select 'NIKUS','Nikon USA','business','','US','Melville, New York','1946-01-01 00:00:00','','Nikon USA' from dual union all
select 'NILE','Nilesat Inc','civilian','ERTU','EG','6th of October City','1996-07-01 00:00:00','','Nilesat Inc' from dual union all
select 'NIMHB','National Institute of Meteorology and Hydrology, Bulgarian A','academic/non-profit','','BG','Akhtopol','','','Националният Институт по Метеорология и Хидрология' from dual union all
select 'NIPR','National Institute for Polar Research','civilian','MONBU','J','Tokyo:Itabashi','1973-09-29 00:00:00','','国立極地研究所' from dual union all
select 'NIRFI','RRI','civilian','','SU','Gorky (Nizhniy Novgorod)','1956-01-01 00:00:00','1977-01-01 00:00:00','Научно-исследовательский радиофизический институт' from dual union all
select 'NISS','Nissan Motor Co Ltd','business','NISSY','J','Tokyo:Suginami (Ogikubo)','1966-01-01 00:00:00','2000-06-01 00:00:00','日産自動車（株）' from dual union all
select 'NISST','Nissan (Solid propulsion plant)','business','NISSY','J','Tomioka','1998-04-01 00:00:00','2000-06-01 00:00:00','日産自動車宇宙航空事業部' from dual union all
select 'NISSY','Nissan','business','','J','Yokohama','1949-01-01 00:00:00','','日産 横浜市' from dual union all
select 'NIUTN','Noorul Islam University, Kanyakumari, Tiruvannamalai','academic/non-profit','','IN','Tiruvannamalai, Tamil Nadu','2004-01-01 00:00:00','','Noorul Islam University' from dual union all
select 'NIVR','Netherlands Institute for Aeronautics and Spaceflight','civilian','','NL','Delft','','','Nederlands Instituut voor Vliegfulgontwikkeling en Ruimtevaart' from dual union all
select 'NJUST','Nanjing University of Science and Technology','academic/non-profit','','CN','Nanjing, Jiangsu','1953-01-01 00:00:00','','南京理工大学' from dual union all
select 'NKAU','Ukranian National Space Agency','civilian','','UA','Kiev, Ukraine','1992-01-01 00:00:00','2011-01-01 00:00:00','Національне космічне агентство України' from dual union all
select 'NKE','NK Engines','business','','RU','Samara','1994-01-01 00:00:00','1996-01-01 00:00:00','АО СНТК «Двигатели НК»' from dual union all
select 'NKO','People''s Commissariat of Defense of the USSR','defense','','SU','Moskva','1934-01-01 00:00:00','1946-03-01 00:00:00','Народный комиссариат обороны' from dual union all
select 'NL','Netherlands','','','NL','Amsterdam, Nederland','1581-07-26 00:00:00','','Koninkrijk der Nederlanden' from dual union all
select 'NLR','National Aerospace Lab','generic organization','','NL','Amsterdam','1961-01-01 00:00:00','','Nationaal Lucht-en Ruimtevaartlaboratorium (National Aerospace Lab)' from dual union all
select 'NMC','US Naval Missile Center','defense','USN','US','Pt Mugu. California','1959-01-07 00:00:00','1975-04-26 00:00:00','US Naval Missile Center' from dual;
insert into ORGANIZATION
select 'NMCOL','New Mexico College','academic/non-profit','','US','Las Cruces, New Mexico','1889-01-01 00:00:00','1960-01-01 00:00:00','New Mexico College' from dual union all
select 'NMIT','Nitte Meenakshi Institute of Technology','academic/non-profit','','IN','Bengaluru (Bangalore:Govindapura)','2001-01-01 00:00:00','','Nitte Meenakshi Institute of Technology' from dual union all
select 'NMOL','USN Manned Orbiting Laboratory Field Office','defense','USN','US','Los Angeles:El Segundo, California','1964-01-01 00:00:00','1968-01-01 00:00:00','USN Manned Orbiting Laboratory Field Office' from dual union all
select 'NMSA','National Meteorological Service','civilian','','AR','La Rioja/Buenos Aires?','1932-01-01 00:00:00','','Servicio Meteorológico Nacional- Argentina' from dual union all
select 'NMSPA','New Mexico Spaceport Authority','civilian','','US','Las Cruces, New Mexico','2005-01-01 00:00:00','','New Mexico Spaceport Authority' from dual union all
select 'NMSU','New Mexico State University','academic/non-profit','','US','Las Cruces, New Mexico','1960-01-01 00:00:00','','New Mexico State University' from dual union all
select 'NOAA','National Oceanic and Atmospheric Administration','civilian','','US','Suitland, Maryland','1970-10-03 00:00:00','','National Oceanic and Atmospheric Administration' from dual union all
select 'NOBY','Nobeyama Radio Observatory','academic/non-profit','','J','Nobeyama','1978-01-01 00:00:00','','国立天文台野辺山' from dual union all
select 'NOL','Naval Ordnance Lab','defense','USN','US','White Oak, Maryland','1929-01-01 00:00:00','1993-01-01 00:00:00','Naval Ordnance Lab' from dual union all
select 'NONAN','NovaNano','business','','F','Lyon/Saint:Didier','2009-01-01 00:00:00','2015-01-01 00:00:00','国立天文台野辺山' from dual union all
select 'NORD','Nord Aviation','business','','F','Bourges, Cher','1954-10-01 00:00:00','1970-01-01 00:00:00','Nord Aviation' from dual union all
select 'NORDS','Societe Nationale de Constructions Aeronautiques du Nord','business','','F','Bourges, Cher','1921-01-01 00:00:00','1954-10-01 00:00:00','Société nationale de constructions aéronautiques du Nord' from dual union all
select 'NORLM','Nord Aviation','business','NORD','F','Paris:Les Mureaux, Yvelines','1921-01-01 00:00:00','1970-01-01 00:00:00','Nord Aviation' from dual union all
select 'NORTH','Northrop Corporation','business','','US','Hawthorne, California','1939-01-01 00:00:00','1962-07-01 00:00:00','Northrop Corporation' from dual union all
select 'NORTHC','Northrop Carolina, Chemtronics Site','business','NORTH','US','Asheville:Swannanoa, North Carolina','1965-01-01 00:00:00','1971-01-01 00:00:00','Northrop Carolina, Chemtronics Site' from dual union all
select 'NORTHS','Northrop Space Laboratories, Northrop Corp.','business','','US','Hawthorne, California','1962-07-01 00:00:00','2002-07-01 00:00:00','Northrop Space Laboratories, Northrop Corp.' from dual union all
select 'NOTS','Naval Ordnance Test Station','defense','USN','US','China Lake, California','1943-01-01 00:00:00','1967-01-01 00:00:00','Naval Ordnance Test Station' from dual union all
select 'NOVTR','OKB Novator','business','','SU','Yekaterinburg (Sverdlovsk)','1947-01-01 00:00:00','','ОКБ «Новатор»' from dual union all
select 'NPL','National Physical Lab','civilian','','IN','New Delhi','1947-01-01 00:00:00','','National Physical Lab' from dual union all
select 'NPOE','NPO Energiya','business','','SU','Korolev:Podlipki, Moskva, Rossiya','1974-07-01 00:00:00','1994-01-01 00:00:00','НПО "Энергия"' from dual union all
select 'NPOL','NPO Lavochkin','business','','RU','Moskva:Khimki','1965-01-01 00:00:00','2014-03-01 00:00:00','НПО им. Лавочкина' from dual union all
select 'NPOLO','NPO Lavochkin, fil. ORKK','business','ORKK','RU','Moskva:Khimki','2014-03-01 00:00:00','','НПО им. Лавочкина' from dual union all
select 'NPOMA','NPO Mashinostroenie','defense','','SU','Moskva:Reutov','1983-01-01 00:00:00','2014-03-01 00:00:00','НПО машиностроения' from dual union all
select 'NPOMAO','NPO Mashinostroenie, fil. ORKK','defense','ORKK','SU','Moskva:Reutov','2014-03-01 00:00:00','','НПО машиностроения' from dual union all
select 'NPOPM','NPO Applied Mechanics','business','MOM','SU','Zheleznogorsk (Krasnoyarsk-26)','1977-01-01 00:00:00','1994-01-01 00:00:00','НПО прикладной механики имени академика М.Ф. Решетнёва' from dual union all
select 'NPOPMR','NPO Applied Mechanics','business','FKA','RU','Zheleznogorsk (Krasnoyarsk-26)','1994-01-01 00:00:00','2006-01-01 00:00:00','НПО прикладной механики имени академика М.Ф. Решетнёва' from dual union all
select 'NPOT','NPO Trud','business','','SU','Samara','1981-01-01 00:00:00','1994-01-01 00:00:00','НПО Труд' from dual union all
select 'NRCC','National Research Council','civilian','','CA','Ottawa','1932-01-01 00:00:00','','National Research Council' from dual union all
select 'NRL','Naval Research Lab','defense','USN','US','Washington, D.C.','1923-01-01 00:00:00','','Naval Research Lab' from dual union all
select 'NRLC','NRO Program C, NRL','defense','USN','US','Washington, D.C.','1962-01-01 00:00:00','1992-12-31 00:00:00','NRO Program C, NRL' from dual union all
select 'NRO','National Reconnaissance Office','defense','','US','Washington, D.C.','1960-08-25 00:00:00','1994-01-01 00:00:00','National Reconnaissance Office' from dual union all
select 'NROC','National Reconnaissance Office','defense','','US','Chantilly, Virginia','1994-01-01 00:00:00','','National Reconnaissance Office' from dual union all
select 'NSA','National Security Agency','defense','DOD','US','Ft. Meade, Maryland','1952-11-04 00:00:00','','National Security Agency' from dual union all
select 'NSAB','Nordiska Satellite AB','business','','S','Stockholm:Solna','1982-01-01 00:00:00','2005-12-01 00:00:00','Nordiska Satellite AB' from dual union all
select 'NSC','Norwegian Space Centre','civilian','','N','Oslo','1987-01-01 00:00:00','','Nordsk Romsenter' from dual union all
select 'NSDC','National Space Development Center, STA','civilian','STA','J','Tokyo, Nippon','1964-06-01 00:00:00','1969-10-01 00:00:00','宇宙開発推進本部' from dual union all
select 'NSF','National Science Foundation','civilian','','US','Washington, DC:Arlington','1950-05-01 00:00:00','','National Science Foundation' from dual union all
select 'NSL','NearSpaceLaunch','business','','US','Upland, Indiana','2012-01-01 00:00:00','','NearSpaceLaunch' from dual union all
select 'NSMC','National Satellite Meteorological Center','civilian','CMA','CN','Beijing','1971-01-01 00:00:00','','国家卫星气象中心' from dual union all
select 'NSPO','National Space Organization','civilian','','TW','Taipei:HsinChu, Taiwan','2005-04-01 00:00:00','','國家太空中心' from dual union all
select 'NSPT','National Space Program Office','civilian','','TW','Taipei:HsinChu, Taiwan','1991-10-01 00:00:00','2005-04-01 00:00:00','國家太空計畫室' from dual union all
select 'NSRDA','NationalSpace Research Development Agency, Nigeria','civilian','','NG','Abuju:Garki, Nigeria','1998-01-01 00:00:00','','NationalSpace Research Development Agency, Nigeria' from dual union all
select 'NSROC','NASA Sounding Rocket Operations Contract','business','NASA','US','Wallops Island, Virginia','1999-01-01 00:00:00','','NASA Sounding Rocket Operations Contract' from dual union all
select 'NSS','New Skies Satellites N.V.','business','','NL','''s:Gravenhage','1998-10-01 00:00:00','2006-09-01 00:00:00','New Skies Satellites N.V.' from dual union all
select 'NSSA','USN Navy Space Systems Activity (NSSA, NAVSPASYSACT)','defense','USN','US','Los Angeles:El Segundo, California','1968-01-01 00:00:00','','USN Navy Space Systems Activity (NSSA, NAVSPASYSACT)' from dual union all
select 'NSWCPH','Naval Surface Warfare Center Port Hueneme','defense','USN','US','Port Hueneme, California','1963-01-01 00:00:00','','Naval Surface Warfare Center Port Hueneme' from dual union all
select 'NSWCWS','Naval Surface Warfare Center Port Hueneme, White Sands Det.','defense','USN','US','White Sands, New Mexico','1963-01-01 00:00:00','','Naval Surface Warfare Center Port Hueneme, White Sands Det.' from dual union all
select 'NTNF','Norwegian Technical Research Council','civilian','','N','Oslo','1946-01-01 00:00:00','1993-01-01 00:00:00','Norges Teknisk-Naturvitenskapelige Forskningsrad' from dual union all
select 'NTNU','Norwegian U of Science and Technology','civilian','','N','Trondheim','1996-01-01 00:00:00','','Norges teknisk-naturvitenskapelige universitet' from dual union all
select 'NTSOMZ','Research Center for Earth Operative Monitoring','civilian','FKA','RU','Moskva','1999-01-01 00:00:00','','Научный центр оперативного мониторинга Земли (НЦ ОМЗ)' from dual union all
select 'NTT','Nippon Telephone and Telegraph Corp','business','','J','Chiyoda-ku, Tokyo','1985-01-01 00:00:00','','日本電信電話株式会社' from dual union all
select 'NTTDM','Nippon Telepone and Telegraph DoMoCo','business','NTT','J','Nagata-cho, Chiyoda-ku, Tokyo','1992-07-01 00:00:00','','株式会社NTTドコモ' from dual union all
select 'NTTP','Nippon Telephone and Telegraph Public Corp','business','NTT','J','Chiyoda-ku, Tokyo','1953-01-01 00:00:00','1985-01-01 00:00:00','日本電信電話公社' from dual union all
select 'NTU','Nanyang Technological University','academic/non-profit','','SG','Singapore','1991-01-01 00:00:00','','Nanyang Technological University' from dual union all
select 'NUAA','Nanjing University of Aeronautics and Astronautics','academic/non-profit','','CN','Nanjing, Jiangsu','1952-01-01 00:00:00','','南京航空航天大学' from dual union all
select 'NUDTC','National University of Defense Technology','business','','CN','Changsha, Hunan','1970-01-01 00:00:00','','国防科技大学研' from dual union all
select 'NUM','National University of Mongolia','academic/non-profit','','MN','Ulaanbaatar','1942-01-01 00:00:00','','Монгол Улсын Их Сургууль' from dual union all
select 'NUS','National University of Singapore','academic/non-profit','','SG','Singapore:Kent Ridge','1980-01-01 00:00:00','','National University of Singapore' from dual union all
select 'NUSS','Cabinet Space Strategy Office','civilian','','J','Tokyo','2012-01-01 00:00:00','','内閣府宇宙戦略室' from dual union all
select 'NVART','Nevada Museum of Art','academic/non-profit','','US','Reno, Nevada','1931-01-01 00:00:00','','Nevada Museum of Art' from dual union all
select 'NWC','Naval Weapons Center','defense','USN','US','China Lake, California','1967-01-01 00:00:00','1992-01-01 00:00:00','Naval Weapons Center' from dual union all
select 'NWNAZ','Northwest Nazarene University','academic/non-profit','','US','Nampa, Idaho','1913-01-01 00:00:00','','Northwest Nazarene University' from dual union all
select 'NWO','Netherlands Organization for Scientific Research','civilian','','NL','''s:Gravenhage','1988-01-01 00:00:00','','Nederlandse Organisatie voor Wetenschappelijk Onderzoek' from dual union all
select 'NWPU','Northwestern Polytechnical University','academic/non-profit','','CN','Xi''an, Shaanxi','1938-01-01 00:00:00','','西北工业大学' from dual union all
select 'NWURKS','NovaWurks','business','','US','Los Alamitos, California','2012-01-01 00:00:00','','NovaWurks' from dual union all
select 'NYBB2','New York Broadband II','business','','US','New York, New York','','','New York Broadband II' from dual union all
select 'NYSH','New York Satellite Holdings LLC','business','NYBB2','US','New York:Queens, New York','2014-08-01 00:00:00','2016-04-01 00:00:00','New York Satellite Holdings' from dual union all
select 'NYU','New York University','academic/non-profit','','US','New York, New York','1831-01-01 00:00:00','','New York University' from dual union all
select 'NZ','New Zealand','','','NZ','Wellingon, North Island, New Zealand','1907-09-26 00:00:00','','New Zealand' from dual union all
select 'NZRD','Ross Dep.','','NZ','NZRD','Scott Base','1841-01-01 00:00:00','','Ross Dependency, New Zealand Antarctic Territory' from dual union all
select 'O3B','O3B Networks Ltd.','business','','UK','St John, Jersey, Channel Islands','2007-01-01 00:00:00','2017-05-01 00:00:00','O3B Networks Ltd.' from dual union all
select 'O3BS','SES O3B Networks Ltd.','business','SES','UK','St John, Jersey, Channel Islands','2017-05-01 00:00:00','','O3B Networks Ltd.' from dual union all
select 'OAKAE','Oakman Aerospace Inc.','business','','US','Littleton, Colorado','2012-07-01 00:00:00','','Oakman Aerospace Inc.' from dual union all
select 'OAR','US Air Force Office of Aerospace Research','defense','USAF','US','Washington, D.C.','1961-04-01 00:00:00','1970-07-01 00:00:00','US Air Force Office of Aerospace Research' from dual union all
select 'OATK','Orbital ATK','business','','US','Washington:Dulles, Virginia','2015-02-10 00:00:00','2018-06-07 00:00:00','Orbital ATK' from dual union all
select 'OATKABL','Orbital ATK DSG Missile Products (Rocket Center, WV)','business','OATK','US','Cumberland:Rocket Center, West Virginia','2015-02-10 00:00:00','2018-06-07 00:00:00','Orbital ATK DSG Missile Products (Rocket Center, WV)' from dual union all
select 'OATKB','OATK Defense Systems Group / Missile Products','business','OATK','US','Baltimore, Maryland','2015-02-10 00:00:00','2018-06-07 00:00:00','OATK Defense Systems Group / Missile Products' from dual union all
select 'OATKC','Orbital ATK  Flight Systems Group (FSG)','business','OATK','US','Phoenix:Chandler, Arizona','2015-02-10 00:00:00','2018-06-07 00:00:00','Orbital ATK  Flight Systems Group (FSG)' from dual union all
select 'OATKD','Orbital ATK Space Systems Group (SSG)','business','OATK','US','Washington:Dulles, Virginia','2015-02-10 00:00:00','2018-06-07 00:00:00','Orbital ATK Space Systems Group (SSG)' from dual union all
select 'OATKE','OATK DSG Missile Products (Elkton)','business','OATK','US','Elkton, Maryland','2015-02-10 00:00:00','2018-06-07 00:00:00','OATK DSG Missile Products (Elkton)' from dual union all
select 'OATKG','Orbital ATK DSG Missile Products (Ronkonkoma)','business','OATK','US','Ronkonkoma, New York','2015-02-10 00:00:00','2018-06-07 00:00:00','Orbital ATK DSG Missile Products (Ronkonkoma)' from dual union all
select 'OATKGA','Orbital ATK Civil Sats. and Nat. Security Systems','business','OATK','US','Gilbert, Arizona','2015-02-10 00:00:00','2018-06-07 00:00:00','Orbital ATK Civil Sats. and Nat. Security Systems' from dual union all
select 'OATKGO','Orbital ATK Space Components (Goleta)','business','OATK','US','Goleta, California','2015-02-10 00:00:00','2018-06-07 00:00:00','Orbital ATK Space Components (Goleta)' from dual union all
select 'OATKH','OATK / ATK Aerospace Group Headquarters','business','OATK','US','Salt Lake City:West Valley City (Magna), Utah','2015-02-10 00:00:00','2018-06-07 00:00:00','OATK / ATK Aerospace Group Headquarters' from dual union all
select 'OATKP','OATK FSG Test Services (Promontory)','business','OATK','US','Brigham City:Promontory, Utah','2015-02-10 00:00:00','2018-06-07 00:00:00','OATK FSG Test Services (Promontory)' from dual union all
select 'OATKS','Orbital ATK Space Systems (Beltsville)','business','OATK','US','Beltsville, Maryland','2015-02-10 00:00:00','2018-06-07 00:00:00','Orbital ATK Space Systems (Beltsville)' from dual union all
select 'OATKW','OATK Launch Systems (Wallops)','business','OATK','US','Wallops Island, Virginia','2015-02-10 00:00:00','2018-06-07 00:00:00','OATK Launch Systems (Wallops)' from dual union all
select 'OAZ','Omsk Aviation Plant','business','','SU','Omsk','1965-01-01 00:00:00','1990-01-01 00:00:00','Омские авиационные завод' from dual union all
select 'OCO','Office of the Chief of Ordnance','defense','USA','US','Washington, DC','1832-01-01 00:00:00','1962-01-01 00:00:00','Office of the Chief of Ordnance' from dual union all
select 'OCOS','Open Cosmos Ltd.','business','','UK','Didcot, Oxfordshire','2015-01-01 00:00:00','','Open Cosmos, Ltd.' from dual union all
select 'OERL','Oerlikon Space','business','CONTR','CH','Zurich','2007-01-01 00:00:00','2009-07-01 00:00:00','Oerlikon Space' from dual union all
select 'OHB','OHB (Orbital Hochtechnologie Bremen System GmbH)','business','FUCHS','D','Bremen','1985-01-01 00:00:00','','OHB (Orbital Hochtechnologie Bremen System GmbH)' from dual union all
select 'OHBSE','OHB Sweden AB','business','OHB','S','Stockholm:Solna','2011-07-01 00:00:00','','OHB Sweden AB' from dual union all
select 'OIT','Osaka Inst. of Technology','academic/non-profit','','J','Osaka CIty/Asahi','1922-01-01 00:00:00','','大阪工業大学' from dual union all
select 'OKB1','OKB1 im. S.P. Korolev','defense','','SU','Korolev:Podlipki','1946-01-01 00:00:00','1966-03-06 00:00:00','ОКБ-1 им. С.П. Королева' from dual union all
select 'OKB10','OKB10 (Reshetnev)','business','','SU','Zheleznogorsk (Krasnoyarsk-26)','1961-12-01 00:00:00','1977-01-01 00:00:00','ОКБ-10' from dual union all
select 'OKB117','OKB117 (Klimov/Izotov)','business','','SU','Sankt:Peterburg (Leningrad)','1942-01-01 00:00:00','1965-01-01 00:00:00','ОКБ-117' from dual union all
select 'OKB154','Design Bureau No. 154','business','','SU','Voronezh','1957-01-01 00:00:00','1967-01-01 00:00:00','ОКБ-154' from dual union all
select 'OKB16','Kazan OKB16    (Zubets)','business','NKE','SU','Kazan, Tartarstan','1953-01-01 00:00:00','1978-01-01 00:00:00','ОКБ-16' from dual union all
select 'OKB1F2','OKB1 Filial No. 2','business','','SU','Zheleznogorsk (Krasnoyarsk-26)','1959-06-04 00:00:00','1961-12-01 00:00:00','филиал № 2 ОКБ-1' from dual;
insert into ORGANIZATION
select 'OKB1F3','OKB1 Filial No. 3','business','','SU','Samara','1959-07-23 00:00:00','1967-01-01 00:00:00','филиал № 3 ОКБ-1' from dual union all
select 'OKB2','OKB2 (Isaev)','business','','SU','Moskva:Korolev (Podlipki)','1952-01-01 00:00:00','1966-01-01 00:00:00','ОКБ-2' from dual union all
select 'OKB23','OKB23 (Myasishchev)','defense','','SU','Moskva:Fili','1951-03-01 00:00:00','1960-01-01 00:00:00','ОКБ-23 В. М. Мясищева' from dual union all
select 'OKB276','OKBS276 (N.D. Kuznetsov)','business','','SU','Samara','1946-04-01 00:00:00','1981-01-01 00:00:00','ОКБ-276' from dual union all
select 'OKB456','OKB456 (Glushko)','business','','SU','Moskva:Khimki','1946-07-03 00:00:00','1967-01-01 00:00:00','ОКБ-456' from dual union all
select 'OKB52','OKB52 (Chelomey)','defense','','SU','Moskva:Reutov','1954-01-01 00:00:00','1966-01-01 00:00:00','ОКБ-52' from dual union all
select 'OKB52F1','OKB52 Fil. 1 (Chelomey)','defense','','SU','Moskva:Fili','1960-01-01 00:00:00','1981-01-01 00:00:00','филиал № 1 ОКБ-52' from dual union all
select 'OKB586','Yangel KB','defense','','SU','Dnepropetrovsk','1954-01-01 00:00:00','1965-01-01 00:00:00','ОКБ-586' from dual union all
select 'OKB586F1','OKB586 Filial No. 1','defense','','SU','Omsk','1961-01-01 00:00:00','1965-01-01 00:00:00','филиал № 1 ОКБ-586' from dual union all
select 'OKB9','OKB9 (V.F. Petrov) of Uralmashzavod','defense','','SU','Sverdlovsk (Ekaterinburg)','1942-01-01 00:00:00','1970-01-01 00:00:00','ОКБ-9' from dual union all
select 'OKBZ154','Factory No. 154 Design Bureau','business','','SU','Voronezh','1946-04-01 00:00:00','1957-01-01 00:00:00','ОКБ завода №154' from dual union all
select 'OKSU','Oklahoma State University','academic/non-profit','','US','Stillwater, Oklahoma','1890-01-01 00:00:00','','Oklahoma State University' from dual union all
select 'OLIN','Olin Aerospace','business','','US','Redmond, Washington','1985-01-01 00:00:00','1997-01-01 00:00:00','Olin Aerospace' from dual union all
select 'OMLGM','Redstone Arsenal, Ordnance Missle Lab, Guided Missle Div','defense','ORDRSA','US','Huntsville, Alabama','1954-01-01 00:00:00','1956-02-01 00:00:00','Redstone Arsenal, Ordnance Missle Lab, Guided Missle Div' from dual union all
select 'OMNIE','OmniEarth LLC','business','','US','Arlington, Virginia','2014-05-01 00:00:00','','OmniEarth LLC' from dual union all
select 'ONDAS','Ondas Media','business','','E','Madrid, Espana','2008-01-01 00:00:00','','Ondas Media' from dual union all
select 'ONERA','Office National d''Etudes et de Recherches Aerospatiales','defense','','F','Paris:Chatillon (?)','1946-01-01 00:00:00','','Office National d’Études et de Recherches Aérospatiales' from dual union all
select 'ONEWEB','One Web','business','','US','Cape Canaveral, Florida','2015-01-01 00:00:00','','One Web' from dual union all
select 'ONM','Office of Naval Material, US Navy','defense','USN','US','Washington, DC','1942-01-01 00:00:00','1966-05-01 00:00:00','Office of Naval Material, US Navy' from dual union all
select 'ONR','Office of Naval Research','defense','','US','Arlington, Virginia','1946-01-01 00:00:00','','Office of Naval Research' from dual union all
select 'ONUSJ','Cuyo National Univ., San Juan','academic/non-profit','','AR','Mendoza:San Juan','1947-01-01 00:00:00','1973-01-01 00:00:00','Universidad Nacional de Cuyo, Fac. of Engineering, San Juan' from dual union all
select 'OPM','Observatoire de ParisMeudon','academic/non-profit','','F','Paris:Meudon','1874-01-01 00:00:00','','Observatoire de Paris-Meudon' from dual union all
select 'OPT','Optus Communications Pty.','business','','AU','Sydney','1992-01-01 00:00:00','2001-01-01 00:00:00','Optus Communications Pty.' from dual union all
select 'OPTS','SingTel Optus Communications Pty','business','','AU','Sydney','2001-01-01 00:00:00','','SingTel Optus Communications Pty' from dual union all
select 'OPU','Osaka Prefectural University','academic/non-profit','','J','Osaka:Sakai','1883-01-01 00:00:00','','大阪府立大学' from dual union all
select 'ORANGE','Orange S.A.','business','','F','Paris','2013-07-01 00:00:00','','Orange S.A.' from dual union all
select 'ORBC','Orbcomm Inc.','business','','US','New York:Fort Lee, New Jersey','1993-01-01 00:00:00','','Orbcomm Inc.' from dual union all
select 'ORBI','Orbimage Inc (Orbital Imaging Corp)','business','','US','Washington:Dulles, Virginia','1992-01-01 00:00:00','2006-01-01 00:00:00','Orbimage Inc (Orbital Imaging Corp)' from dual union all
select 'ORBTEC','Orbitec (Orbital Technologies Corporation)','business','','US','Madison, Wisconsin','1988-01-01 00:00:00','','Orbitec (Orbital Technologies Corporation)' from dual union all
select 'ORD','US Army Ordnance Dept. HQ, White Sands Proving Ground','defense','OCO','US','White Sands, New Mexico','1944-01-01 00:00:00','1949-10-01 00:00:00','US Army Ordnance Dept. HQ, White Sands Proving Ground' from dual union all
select 'ORDRSA','US Army Ordnance, Redstone Arsenal Ordnance Missile Lab','defense','OCO','US','Huntsville, Alabama','1950-01-01 00:00:00','1958-03-31 00:00:00','US Army Ordnance, Redstone Arsenal Ordnance Missile Lab' from dual union all
select 'OREN','Orenburgskiy Mashinostroitelniy Zavod','business','','SU','Orenburg','1961-10-15 00:00:00','1986-04-07 00:00:00','Оренбургский машиностроительный завод' from dual union all
select 'ORI','Orion Satellite Corp.','business','','US','Rockville, Maryland','1982-01-01 00:00:00','1998-03-01 00:00:00','Orion Satellite Corp.' from dual union all
select 'ORIAS','Institut d''Astrophysique Spatiale (IAS), Uni. Paris XI','academic/non-profit','','F','Paris:Orsay, Essonne','1960-01-01 00:00:00','','Institut d''Astrophysique Spatiale (IAS), Uni. Paris XI' from dual union all
select 'ORKK','United RocketSpace Corporation','business','','RU','Moskva','2014-03-04 00:00:00','2016-01-01 00:00:00','ОАО "Объединенная ракетно-космическая корпорация"' from dual union all
select 'ORL','Universite'' d''Orleans','academic/non-profit','','F','Orleans','1230-01-01 00:00:00','','Université d''Orléans' from dual union all
select 'OSAKA','Osaka University','academic/non-profit','','J','Osaka','1931-01-01 00:00:00','','大阪大学' from dual union all
select 'OSC','Orbital Sciences Corp.','business','','US','Fairfax, Virginia','1986-01-01 00:00:00','1993-01-01 00:00:00','Orbital Sciences Corp.' from dual union all
select 'OSCAR','Project OSCAR Inc.','academic/non-profit','','US','San Jose, California','1960-01-01 00:00:00','','Project OSCAR Inc.' from dual union all
select 'OSCC','Orbital Sciences Corp./Chandler','business','OSC','US','Phoenix:Chandler, Arizona','1988-01-01 00:00:00','2015-02-10 00:00:00','Orbital Sciences Corp./Chandler' from dual union all
select 'OSCD','Orbital Sciences Corp.','business','','US','Washington:Dulles, Virginia','1993-01-01 00:00:00','2015-02-10 00:00:00','Orbital Sciences Corp.' from dual union all
select 'OSCG','OSC Germantown','business','OSC','US','Germantown, Maryland','1994-01-01 00:00:00','2000-01-01 00:00:00','OSC Germantown' from dual union all
select 'OSCGA','Orbital Sciences Corp./Gilbert','business','OSC','US','Gilbert, Arizona','2010-04-01 00:00:00','2015-02-10 00:00:00','Orbital Sciences Corp./Gilbert' from dual union all
select 'OSCM','OSC McLean','business','OSC','US','Washington:McLean, Virginia','1997-01-01 00:00:00','2000-01-01 00:00:00','OSC McLean' from dual union all
select 'OSCV','Orbital Sciences Corp.','business','OSC','US','Vienna, Virginia','1982-01-01 00:00:00','1986-01-01 00:00:00','Orbital Sciences Corp.' from dual union all
select 'OSCW','Orbital Sciences HIF, MidAtlantic Regional Spaceport','business','OSC','US','Wallops Island, Virginia','2011-01-01 00:00:00','2015-02-10 00:00:00','Orbital Sciences HIF, Mid-Atlantic Regional Spaceport' from dual union all
select 'OSLO','Oslo University','academic/non-profit','','N','Oslo','1811-01-01 00:00:00','','Universitetet i Oslo' from dual union all
select 'OSN','Open Space Network','business','','US','San Jose:Mountain View, California','2015-01-01 00:00:00','','Open Space Network' from dual union all
select 'OSSI','Open Source Satellite Initiative','academic/non-profit','','KR','Seoul','2011-01-01 00:00:00','','Open Source Satellite Initiative' from dual union all
select 'OSSS','One Stop Satellite Solutions (Weber CAST)','business','WEB','US','Ogden, Utah','1996-01-01 00:00:00','2002-01-01 00:00:00','One Stop Satellite Solutions (Weber CAST)' from dual union all
select 'OSU','Ohio State University','academic/non-profit','','US','Columbus, Ohio','1870-01-01 00:00:00','','Ohio State University' from dual union all
select 'OTR','Orbital Transport und Raketen AG, Stuttgart factory','business','','D','Stuttgart:Vaihingen, Baden:Wurttemberg','1975-01-01 00:00:00','1980-01-01 00:00:00','Orbital Transport und Raketen AG, Stuttgart factory' from dual union all
select 'OTRG','Orbital Transport und Raketen AG, Garching factory','business','','D','Garching bei Munchen','1980-01-01 00:00:00','1987-01-01 00:00:00','Orbital Transport und Raketen AG, Garching factory' from dual union all
select 'OVHO','OverHorizon AB','business','','S','Solna','2009-01-01 00:00:00','','OverHorizon AB' from dual union all
select 'OX','Oxford University','academic/non-profit','','UK','Oxford','1096-01-01 00:00:00','','Oxford University' from dual union all
select 'P','Portugal','','','P','Lisboa, Portugal','1139-07-25 00:00:00','','República Portuguesa' from dual union all
select 'PACSCI','Pacific Scientific Energetic Materials Co.','business','','US','Chandler, Arizona','1986-01-01 00:00:00','','Pacific Scientific Energetic Materials Co.' from dual union all
select 'PAEC','Pakistan Atomic Energy Commission','defense','','PK','Islamabad','1955-01-01 00:00:00','','Pakistan Atomic Energy Commission' from dual union all
select 'PAI','Polish Aviation Institute','civilian','','PL','Warsawa','1926-01-01 00:00:00','','Instytut Lotnictwa' from dual union all
select 'PAN','Panamsat Corp','business','','US','Greenwich/Wilton, Connecticut','1984-01-01 00:00:00','2006-07-01 00:00:00','Panamsat Corp' from dual union all
select 'PAR','Panama','','','PAR','Panama, Panama province','1903-01-01 00:00:00','','República de Panamá' from dual union all
select 'PARA','Paradigm Secure Comms, Astrium Services','business','','UK','Corsham, Wiltshire','1999-01-01 00:00:00','2015-01-01 00:00:00','Paradigm Secure Comms, Astrium Services' from dual union all
select 'PARAC','Paracom SA','business','','AR','Buenos Aires, Argentina','1992-01-01 00:00:00','1994-01-01 00:00:00','Paracom SA' from dual union all
select 'PARD','NACA Pilotless Aircraft Research Division, Langley Aero. Lab.','civilian','NACA','US','Wallops Island, Virginia','1945-01-01 00:00:00','1959-05-01 00:00:00','NACA Pilotless Aircraft Research Division, Langley Aero. Lab.' from dual union all
select 'PARIS','Universite de Parisla Sorbonne, Faculte des Sciences','academic/non-profit','','F','Paris','1808-01-01 00:00:00','1970-01-01 00:00:00','Université Paris-Sorbonne,Faculté des sciences' from dual union all
select 'PASI','Philippine Agila Satellite Inc.','business','','PH','Manila','1994-01-01 00:00:00','','Philippine Agila Satellite Inc.' from dual union all
select 'PATRAS','University of Patras','academic/non-profit','','GR','Patras','1968-01-01 00:00:00','','Πανεπιστήμιο Πατρών' from dual union all
select 'PAVKH','Pavlograd Khimmash','business','','RU','Pavlograd','','','Завод "химмаш" Павлоград' from dual union all
select 'PCZ','Canal Zone','','US','PCZ','Balboa','1903-01-01 00:00:00','1979-01-01 00:00:00','Panama Canal Zone, US territory' from dual union all
select 'PE','Peru','','','PE','Lima','1821-01-01 00:00:00','','República del Perú' from dual union all
select 'PEND','Pendrell Corp.','business','','US','Kirkland, Washington','2011-07-01 00:00:00','','Pendrell Corp.' from dual union all
select 'PEOSS','USN Program Executive Office for Space Systems','defense','USN','US','San Diego, California','','','USN Program Executive Office for Space Systems' from dual union all
select 'PER','PT Perusahaan Umum Telekomunikasi (PT Perumtel)','business','','ID','Jakarta','1974-01-01 00:00:00','1991-01-01 00:00:00','PT Perusahaan Umum Telekomunikasi (PT Perumtel)' from dual union all
select 'PERC','Planetary Exploration Research Center','academic/non-profit','CHIBA','J','Narashino, Chiba','','','千葉工業大学 惑星探査研究センター' from dual union all
select 'PERM','KB Perm  Motorostroitel','business','','RU','Perm'', Permskiy Krai','1992-01-01 00:00:00','1997-01-01 00:00:00','Пермское государственное предприятие «Моторостроитель»' from dual union all
select 'PERM2','KB Perm  Motorostroitel','business','','RU','Perm'', Permskiy Krai','1997-01-01 00:00:00','','OAO Пермский моторный завод' from dual union all
select 'PERME','Rocket Propulsion Establishment','defense','MOD','UK','Westcott, Buckinghamshire','1977-02-01 00:00:00','1984-01-01 00:00:00','Rocket Propulsion Establishment' from dual union all
select 'PESU','PES (People''s Education Society) University','academic/non-profit','','IN','Bengaluru, Karnataka','1988-01-01 00:00:00','','PES University, Bangalore' from dual union all
select 'PFAB','Process Fab Inc','business','','US','Santa Fe Springs, California','','','Process Fab Inc' from dual union all
select 'PFRR','Poker Flat Research Range','civilian','','US','Poker Flat, Fairbanks, Alaska','1968-01-01 00:00:00','','Poker Flat Research Range' from dual union all
select 'PG','Papua New Guinea','','','PG','Port Moresby','1949-01-01 00:00:00','','Independen Stet bilong Papua Niugini' from dual union all
select 'PH','Phillipines','','','PH','Lungsod ng Maynila, Pilipinas','1521-03-16 00:00:00','','Republika ng Pilipinas' from dual union all
select 'PH4','Phase Four, Inc.','business','','US','Los Angeles:El Segundo, California','2015-01-01 00:00:00','','Phase Four, Inc.' from dual union all
select 'PHFO','PhilcoFord','business','','US','Newport Beach, California','1966-01-01 00:00:00','1975-01-01 00:00:00','Philco-Ford' from dual union all
select 'PHFP','PhilcoFord WDL','business','PHFO','US','Palo Alto, California','1966-01-01 00:00:00','1975-01-01 00:00:00','Philco-Ford WDL' from dual union all
select 'PHIL','Phillips','business','','NL','Eindhoven','1891-01-01 00:00:00','','Phillips' from dual union all
select 'PHILR','Phillips Petroleum, Rocket Fuels Division, Air Force Plant 66','business','','US','McGregor, Texas','1952-08-01 00:00:00','1958-01-01 00:00:00','Phillips Petroleum, Rocket Fuels Division, Air Force Plant 66' from dual union all
select 'PHLO','Philco','business','','US','Newport Beach, California','1962-01-01 00:00:00','1966-01-01 00:00:00','Philco' from dual union all
select 'PHLP','Philco WDL','business','PHLO','US','Palo Alto, California','1957-01-01 00:00:00','1966-01-01 00:00:00','Philco WDL' from dual union all
select 'PICA','Picatinny Arsenal, USA ARSEC','defense','USA','US','Dover:Lake Denmark, New Jersey','1880-01-01 00:00:00','','Picatinny Arsenal, USA ARSEC' from dual union all
select 'PIHM','National Institute of Hydrology and Meteorology','civilian','','PL','Warsawa','1945-01-01 00:00:00','1973-01-01 00:00:00','Państwowy Instytut Hydrologiczno-Meteorologiczny' from dual union all
select 'PITT','University of Pittsburgh','academic/non-profit','','US','Pittsburg:Oakland, Pennsylvania','1908-01-01 00:00:00','','University of Pittsburgh' from dual union all
select 'PK','Pakistan','','','PK','Islamabad, Pakistan','1947-08-14 00:00:00','','اسلامی جمہوریۂ پاكستان' from dual union all
select 'PKAFSC','Air Force Strategic Command, Pakistan Air Force','defense','','PK','Islamabad','2000-03-01 00:00:00','','Air Force Strategic Command, Pakistan Air Force' from dual union all
select 'PKEL','PerkinElmer Corp.','business','','US','Danbury, Connecticut','1968-01-01 00:00:00','1990-05-01 00:00:00','Perkin-Elmer Corp.' from dual union all
select 'PKNDC','Pakistan National Defense Complex','defense','PAEC','PK','Fateh Jang','1993-01-01 00:00:00','2001-01-01 00:00:00','Pakistan National Defense Complex' from dual union all
select 'PKO','Space Defence Force, USSR Armed Forces','defense','PVO','SU','Moskva?','1963-01-01 00:00:00','','войска противокосмической обороны' from dual union all
select 'PKPMO','Project Management Organization, Pakistan','civilian','PAEC','PK','Khanpur','1994-01-01 00:00:00','2001-01-01 00:00:00','Project Management Organization, Pakistan' from dual union all
select 'PL','Poland','','','PL','Warszawa, Polska','1989-01-01 00:00:00','','Rzeczpospolita Polska' from dual union all
select 'PLA','Chinese People''s Liberation Army','defense','','CN','Beijing','1927-08-01 00:00:00','','中国人民解放军' from dual;
insert into ORGANIZATION
select 'PLA2AC','PLA Second Artillery Corps','defense','','CN','Beijing','1966-07-01 00:00:00','2016-01-01 00:00:00','第二炮兵部队' from dual union all
select 'PLABS','Planet Labs','business','','US','San Francisco, California','2013-06-01 00:00:00','2016-06-01 00:00:00','Planet Labs' from dual union all
select 'PLABSB','Planet Labs/Berlin','business','PLABS','D','Brandenburg an der Havel, Brandenburg','2015-08-01 00:00:00','','Planet Labs/Berlin' from dual union all
select 'PLABST','Terra Bella (Planet)','business','PLABS','US','San Jose:Mountain View, California','2017-04-01 00:00:00','','Skybox Imaging' from dual union all
select 'PLAN','Planet','business','','US','San Francisco, California','2016-06-01 00:00:00','','Planet Labs' from dual union all
select 'PLANQ','PlanetiQ','business','','US','Washington/Bethesda, Maryland','2012-01-01 00:00:00','','PlanetiQ' from dual union all
select 'PLANS','North Sea Fleet, People''s Liberation Army Navy','defense','PLA','CN','Qingdao','1949-01-01 00:00:00','','北海舰队' from dual union all
select 'PLARF','PLA Rocket Forces','defense','','CN','Beijing','2016-01-01 00:00:00','','中国人民解放军火箭军' from dual union all
select 'PLRES','Planetary Resources Inc.','business','','US','Redmond, Washington','2010-01-01 00:00:00','','Planetary Resources Inc.' from dual union all
select 'PLRL','Poland','','','PL','Warszawa, Polska','1952-01-01 00:00:00','1989-01-01 00:00:00','Polska Rzeczpospolita Ludowa' from dual union all
select 'PMOD','World Radiation Center','academic/non-profit','','CH','Davos, Dorf','','','Physikalisch-Meteorologisches Observatorium Davos, World Radiation Center' from dual union all
select 'PMR','Pacific Missile Range  Pt Mugu','defense','USN','US','Pt Mugu. California','1958-06-01 00:00:00','1959-01-01 00:00:00','Pacific Missile Range - Pt Mugu' from dual union all
select 'PMTC','Pacific Missile Test Center','defense','USN','US','Pt Mugu. California','1975-04-26 00:00:00','1992-01-01 00:00:00','Pacific Missile Test Center' from dual union all
select 'PMW146','SPAWAR PMW 146 Com Sat Program Office','defense','PEOSS','US','San Diego, California','','','SPAWAR PMW 146 Com Sat Program Office' from dual union all
select 'POL','PO Polyot','business','FKA','SU','Omsk','1990-01-01 00:00:00','2007-01-01 00:00:00','ПО «Полёт»' from dual union all
select 'POLK','PO Polyot, Filial GKNPTs Khrunichev','business','FKA','SU','Omsk','2007-01-01 00:00:00','','ПО «Полёт» филиал ФГУП "ГКНПЦ им. М.В. Хруничева"' from dual union all
select 'POLY','Ecole Polytechnique','academic/non-profit','','F','Paris:Saclay','1794-01-01 00:00:00','','École Polytechnique' from dual union all
select 'POPACS','Project POPACS (Gil Moore)','academic/non-profit','','US','Monument, Colorado','2013-01-01 00:00:00','','Project POPACS (Gil Moore)' from dual union all
select 'PORTO','University of Porto','academic/non-profit','','PT','Porto','1911-01-01 00:00:00','','Universidade do Porto' from dual union all
select 'POUDR','Poudrerie Royale (Poudrerie Nationale)','civilian','','F','Saint:Medard:en:Jalles, Gironde','1660-01-01 00:00:00','1873-01-01 00:00:00','Poudrerie Royale (Poudrerie Nationale)' from dual union all
select 'PPARC','UK Particle Physics and Astronomy Research Council','civilian','','UK','Swindon, England','1994-01-01 00:00:00','2007-01-01 00:00:00','UK Particle Physics and Astronomy Research Council' from dual union all
select 'PR','Puerto Rico','','US','PR','San Juan','1898-01-01 00:00:00','','Estado Libre Asociado de Puerto Rico' from dual union all
select 'PRIMVN','PRIMEX Technologies/Van Nuys (transfer to Redmond)','business','','US','Van Nuys, California','2000-01-01 00:00:00','2000-01-01 00:00:00','PRIMEX Technologies/Van Nuys (transfer to Redmond)' from dual union all
select 'PRIMX','PRIMEX Technologies','business','','US','Redmond, Washington','1997-01-01 00:00:00','2001-01-01 00:00:00','PRIMEX Technologies' from dual union all
select 'PRIN','Princeton University','academic/non-profit','','US','Princeton, New Jersey','1756-01-01 00:00:00','','Princeton University' from dual union all
select 'PRINCE','Prince Motor Co Ltd','business','','J','Tokyo:Suginami (Ogikubo)','1961-01-01 00:00:00','1966-01-01 00:00:00','プリンス自動車工業株式会社' from dual union all
select 'PRIR','Priroda Center GUGK','civilian','GUGK','SU','Moscow','','','центр "Природа"' from dual union all
select 'PRL','Physical Research Lab, Ahmedabad','civilian','','IN','Ahmedabad, Gujarat','1947-11-11 00:00:00','','Physical Research Lab, Ahmedabad' from dual union all
select 'PRO','Missile Defense Force','defense','PVO','SU','Moskva?','1963-01-01 00:00:00','','войска противоракетной обороны' from dual union all
select 'PROG','Progress Factory','business','','RU','Samara','1961-01-01 00:00:00','1996-04-01 00:00:00','Завод «Прогресс»' from dual union all
select 'PROGT','CSDBProgress Factory','business','TSSKBP','RU','Samara (Kubyshev), Samara Oblast','1996-04-12 00:00:00','','Завод ЦСКБ-Прогресс' from dual union all
select 'PROTO','Protostar Ltd','business','','BM','Bermuda','2001-09-01 00:00:00','','Protostar Ltd' from dual union all
select 'PROTS','Protostar Satellite Systems','business','PROTO','US','San Francisco, California','2005-03-01 00:00:00','','Protostar Satellite Systems' from dual union all
select 'PSC','Planetary Systems Corporation','business','','US','Washington/Silver Spring, Maryland','','','Planetary Systems Corporation' from dual union all
select 'PSCI','Physical Sciences, Inc.','business','','US','Andover, MA','','','Physical Sciences, Inc.' from dual union all
select 'PSN','PT Pasifik Satelite Nusantara','business','','ID','Jakarta:Bekasi, West Java','1992-01-01 00:00:00','','PT Pasifik Satelite Nusantara' from dual union all
select 'PSRC','Pacific Sierra Research Corp.','business','','US','Los Angeles, California','','1998-08-01 00:00:00','Pacific Sierra Research Corp.' from dual union all
select 'PSU','Penn State University','academic/non-profit','','US','State College, Pennsylvania','1855-01-01 00:00:00','','Penn State University' from dual union all
select 'PT','Portugal','','','PT','Lisboa','0868-01-01 00:00:00','','República Portuguesa' from dual union all
select 'PU','ZAO Puskovie Uslugi','business','','RU','Moskva','1992-01-01 00:00:00','','ЗАО «Пусковые услуги»' from dual union all
select 'PUCP','Pontifical Catholic University of Peru','academic/non-profit','','PE','Lima','1917-01-01 00:00:00','','Pontificia Universidad Católica del Perú' from dual union all
select 'PUMPK','Pumpkin Inc.','business','','US','San Francisco, California','2009-01-01 00:00:00','','Pumpkin Inc.' from dual union all
select 'PUSTEK','Center for Electronics Technology Aerospace Indonesia','business','','ID','Jakarta','','','Pusat Teknology Elektronika, Dirgantara Indonesia' from dual union all
select 'PVO','ProtoVozdhusnoy Oborony','defense','','SU','Moskva','1949-01-01 00:00:00','','Войска́ противовозду́шной оборо́ны' from dual union all
select 'PWAR','Warsaw University of Technology','academic/non-profit','','PL','Warszawa','1826-01-01 00:00:00','','Politechnika Warszawska' from dual union all
select 'PWCSD','UTC Pratt and Whitney Space Propulsion Chemical Systems Div.','business','UTC2','US','San Jose, California','1996-01-01 00:00:00','2004-01-01 00:00:00','UTC Pratt and Whitney Space Propulsion Chemical Systems Div.' from dual union all
select 'PWFL','Pratt & Whitney Florida Research and Development Center','business','UAC','US','West Palm Beach, Florida','1958-01-01 00:00:00','2013-06-01 00:00:00','Pratt & Whitney Florida Research and Development Center' from dual union all
select 'PWRD','Pratt & Whitney Rocketdyne Inc','business','UTC2','US','Canoga Park, California','2005-08-01 00:00:00','2013-06-01 00:00:00','Pratt & Whitney Rocketdyne Inc' from dual union all
select 'QA','Qatar','','','QA','Doha, Qatar','1878-01-01 00:00:00','','دولة قطر' from dual union all
select 'QB50','QB50 Consortium','academic/non-profit','','B','Brussels/Sint:Genesius:Rode, Brabant','2012-01-01 00:00:00','','QB50 Consortium' from dual union all
select 'QBUS','QBUS Consortium (Stanford/APL/UC Boulder/U MIch/U del Turabo)','academic/non-profit','','US','Laurel, Maryland','2014-01-01 00:00:00','','QBUS Consortium' from dual union all
select 'QINQ','Qinetiq','business','','UK','Farnborough, Hants','2001-07-01 00:00:00','','Qinetiq' from dual union all
select 'QINS','QinetiQ Space nv','business','QINQ','B','Kruibeke','2010-01-01 00:00:00','','QinetiQ Space nv' from dual union all
select 'QMW','Queen Mary Westfield College, University of London','academic/non-profit','','UK','London:Mile End','','','Queen Mary Westfield College, University of London' from dual union all
select 'QTT','Full Map Network Co.','business','','CN','Beijing:Yizhuang','2016-01-01 00:00:00','','全图通公司' from dual union all
select 'QUAFI','QuakeFinder LLC, Palo Alto','business','','US','Palo Alto, California','2000-01-01 00:00:00','','QuakeFinder LLC, Palo Alto' from dual union all
select 'QUB','Queen''s University, Belfast','academic/non-profit','','UK','Belfast, Northern Ireland','1849-01-01 00:00:00','','Queen''s University, Belfast' from dual union all
select 'QUETZ','Quetzsat S.A. de C.V','business','','MX','Mexico CIty?','2011-01-01 00:00:00','','Quetzsat S.A. de C.V' from dual union all
select 'QUK','Queen''s University (Kingston, Ontario)','academic/non-profit','','CA','Kingston, Ontario','1841-01-01 00:00:00','','Queen''s University (Kingston, Ontario)' from dual union all
select 'RACE','Radio Amateur Club de L''Espace','academic/non-profit','','F','Paris?','','','Radio Amateur Club de L''Espace' from dual union all
select 'RADS','Radiant Solutions (Herndon)','business','MAXAR','US','Herndon, Virginia','2017-10-01 00:00:00','','DigitalGlobe (Herndon)' from dual union all
select 'RAE','Royal Aircraft Establishment','defense','','UK','Farnborough, Hants.','1918-04-01 00:00:00','1991-04-01 00:00:00','Royal Aircraft Establishment' from dual union all
select 'RAERPD','RAE Rocket Propulsion Department','defense','RAE','UK','Westcott, Buckinghamshire','1947-01-01 00:00:00','1958-08-01 00:00:00','RAE Rocket Propulsion Department' from dual union all
select 'RAF','Royal Air Force Bomber Command','defense','','UK','High Wycombe','1940-01-01 00:00:00','1968-01-01 00:00:00','Royal Air Force Bomber Command' from dual union all
select 'RAFL','Rafael Advanced Defense Systems Ltd.','business','','IL','Haifa','1965-01-01 00:00:00','','Rafael Advanced Defense Systems Ltd.' from dual union all
select 'RAFSPO','UK Space Operations Coordination Centre, RAF High Wycombe','defense','','UK','High Wycombe','1957-01-01 00:00:00','2016-01-01 00:00:00','UK Space Operations Centre, Royal Air Force High Wycombe' from dual union all
select 'RAKA','Rosaviakosmos (Rossiskoe aviatsionnokosmicheskoe agentstvo)','civilian','','RU','Moskva, Rossiya','1999-01-01 00:00:00','2004-01-01 00:00:00','Российское авиационно-космическое агентство' from dual union all
select 'RAL','Rutherford Appleton Laboratory','civilian','SERC','UK','Chilton, Oxfordshire','1979-01-01 00:00:00','','Rutherford Appleton Laboratory' from dual union all
select 'RAYES','Raytheon IIS (Dallas)','business','RAYTH','US','Dallas:Garland, Texas','1995-01-01 00:00:00','','Raytheon IIS (Dallas)' from dual union all
select 'RAYHSV','Raytheon Missile Systems/Huntsville','business','RAYTH','US','Huntsville, Alabama','2010-01-01 00:00:00','','Raytheon Missile Systems/Huntsville' from dual union all
select 'RAYM','Raymond Engineering Laboratories','business','','US','Middletown, Connecticut','1957-01-01 00:00:00','','Raymond Engineering Laboratories' from dual union all
select 'RAYMS','Raytheon Missile Systems','business','RAYTH','US','Tucson, Arizona','1997-01-01 00:00:00','','Raytheon Missile Systems' from dual union all
select 'RAYTH','Raytheon Space and Information Systems','business','','US','Sudbury, Mass.','','','Raytheon Space and Information Systems' from dual union all
select 'RBOW','Rainbow DBS Company LLC','business','CABVS','US','Bethpage:Jericho, New York','2003-01-01 00:00:00','2005-01-01 00:00:00','Rainbow DBS Company LLC' from dual union all
select 'RCA','RCA Astro Electronics (Astro Space)','business','','US','East Windsor, New Jersey  (Princeton)','1957-01-01 00:00:00','1986-01-01 00:00:00','RCA Astro Electronics (Astro Space)' from dual union all
select 'RCAF','Royal Canadian Air Force','defense','','CA','North Bay, Ontario  (or RCAR?)','1924-01-01 00:00:00','1968-01-01 00:00:00','Royal Canadian Air Force' from dual union all
select 'RCAR','Royal Canadian Artillery','defense','','CA','Shilo, Manitoba?','1750-01-01 00:00:00','','Royal Canadian Artillery' from dual union all
select 'RCAV','RCA Victor Ltd.','business','','CA','Montreal:St Henri','1958-01-01 00:00:00','1977-01-01 00:00:00','RCA Victor Ltd.' from dual union all
select 'RDYNE','Rocketdyne','business','NAA','US','Canoga Park, California','1955-11-07 00:00:00','1996-12-01 00:00:00','Rocketdyne' from dual union all
select 'RDYNM','Rocketdyne Solid Propulsion','business','RDYNE','US','McGregor, Texas','1959-09-30 00:00:00','1978-01-01 00:00:00','Rocketdyne Solid Propulsion' from dual union all
select 'RDYNN','Rocketdyne, Neosho Plant','business','RDYNE','US','Neosho, Missouri','1956-01-01 00:00:00','1968-01-01 00:00:00','Rocketdyne, Neosho Plant' from dual union all
select 'REP','Republic Aircraft','business','','US','Farmingdale, New York','1939-01-01 00:00:00','1965-01-01 00:00:00','Republic Aircraft' from dual union all
select 'REPF','Fairchild Republic Aircraft Division','business','FAIR','US','Farmingdale, New York','1965-01-01 00:00:00','1987-01-01 00:00:00','Fairchild Republic Aircraft Division' from dual union all
select 'RESH','OAO M. Reshetnev Information Satellite Systems','business','FKA','RU','Zheleznogorsk (Krasnoyarsk-26)','2006-01-01 00:00:00','','ОАО «Информационные спутниковые системы» имени академика М. Ф. Решетнёва' from dual union all
select 'REYE','RapidEye AG','business','','D','Brandenburg an der Havel, Brandenburg','1998-01-01 00:00:00','2013-11-06 00:00:00','RapidEye AG' from dual union all
select 'RGMS','Rosgidromet NITs Planeta','civilian','','RU','Moskva','1991-01-01 00:00:00','','Росгидромет Научно-исследовательский центр космической гидрометеорологии "Планета"' from dual union all
select 'RHI','Rheinmetall Italia SpA.','business','','I','Roma','','','Rheinmetall Italia SpA.' from dual union all
select 'RICE','Rice University','academic/non-profit','','US','Houston, Texas','1912-01-01 00:00:00','','Rice University' from dual union all
select 'RIKEN','RIKEN, Wako site','civilian','','J','Tokyo:Wako','1963-01-01 00:00:00','','理化学研究所' from dual union all
select 'RIKK','Rikkyo University','academic/non-profit','','J','Tokyo:Toshima','1874-01-01 00:00:00','','立教大学' from dual union all
select 'RIMSAT','Rimsat','business','','US','Fort Wayne, Indiana','1992-04-01 00:00:00','1995-01-01 00:00:00','Rimsat' from dual union all
select 'RIU','Rhode Island University','academic/non-profit','','US','Kingston, Rhode Island','1951-01-01 00:00:00','','Rhode Island University' from dual union all
select 'RKA','Rossiskoe aviatsionnokosmicheskoe agentsvo (Rosaviakosmos)','civilian','','RU','Moskva','1992-02-01 00:00:00','1999-01-01 00:00:00','Российское космическое агентство' from dual union all
select 'RKKE','RKK Energiya','business','FKA','RU','Korolev:Podlipki, Moskva, Rossiya','1994-01-01 00:00:00','','ОАО "Ракетно-космическая корпорация "Энергия" имени С.П. Королёва"' from dual union all
select 'RKS','Russian Space Systems','civilian','','SU','Moskva','2009-01-01 00:00:00','','АО «Российские космические системы»' from dual union all
select 'RLABN','Rocket Lab Ltd.','academic/non-profit','RLABU','NZ','Parnell, Auckland, North Island','2009-01-01 00:00:00','','Rocket Lab Ltd.' from dual union all
select 'RLABU','Rocket Lab USA','business','','US','Los Angeles, California','2014-01-01 00:00:00','','Rocket Lab USA' from dual union all
select 'RMAF','Royal Moroccan Armed Forces','defense','','MA','Rabat','1956-01-01 00:00:00','','القوات المسلحة الملكية المغربية' from dual union all
select 'RMD','Thiokol Chemical Corp. Reaction Motors Division','business','','US','Denville, New Jersey','1958-04-30 00:00:00','1972-06-01 00:00:00','Thiokol Chemical Corp. Reaction Motors Division' from dual union all
select 'RMI','Reaction Motors Incorporated (Denville)','business','','US','Denville, New Jersey','1955-01-01 00:00:00','1958-04-30 00:00:00','Reaction Motors Incorporated (Denville)' from dual;
insert into ORGANIZATION
select 'RMILD','Reaction Motors Incorporated, Picatinny Arsenal','business','','US','Dover:Lake Denmark, New Jersey','1946-01-01 00:00:00','1970-01-01 00:00:00','Reaction Motors Incorporated, Picatinny Arsenal' from dual union all
select 'RMIPL','Reaction Motors Incorporated, Pompton Plains','business','','US','Pequannock:Pompton Plains, New Jersey','1942-01-01 00:00:00','1946-01-01 00:00:00','Reaction Motors Incorporated, Pompton Plains' from dual union all
select 'RN','Royal Navy Submarine Service, HMS Neptune, Faslane Naval Base','defense','UKRN','UK','Gare Loch:Faslane, Argyll and Bute, Scotland','1964-01-01 00:00:00','','Royal Navy Submarine Service, HMS Neptune, Faslane Naval Base' from dual union all
select 'RNAF','Royal Netherlands Air Force, Guided Missile Group','defense','','NL','Venray','1954-01-01 00:00:00','','Koninklijke Luchtmacht (KLu), Groep Geleide Wapens, LMB De Peel' from dual union all
select 'RNII','Reaktivniy nauchnoissledovatel''niy institut','civilian','','SU','Moskva:Khovrino','1945-01-01 00:00:00','1950-01-01 00:00:00','Реактивный научно-исследовательский институт' from dual union all
select 'RO','Romania','','','RO','Bucuresti','1859-01-01 00:00:00','','România' from dual union all
select 'ROB','Koninklijke Sterrenwacht van Belgie (Observatoire Royal de Belgique)','civilian','','B','Brussel:Uccle','1890-01-01 00:00:00','','Koninklijke Sterrenwacht van Belgie (Observatoire Royal de Belgique)' from dual union all
select 'ROE','Jet Propulsion Science Research Institute','civilian','','UK','Edinburgh, Scotland','1896-01-01 00:00:00','','Royal Observatory, Edinburgh, Blackford Hill' from dual union all
select 'ROGAU','Gruppo di Astrodinamica dell''Universita degli Studi di Roma ''La Sapienza''','academic/non-profit','ROSAP','I','Roma','','2011-01-01 00:00:00','Gruppo di Astrodinamica dell''Universita degli Studi di Roma ''La Sapienza''' from dual union all
select 'ROHR','Rohr Corp.','business','','US','Chula Vista, California','1940-01-01 00:00:00','','Rohr Corp.' from dual union all
select 'ROISS','Institute of Space Science (Romania)','civilian','','RO','Bucharest:Magurele','1990-01-01 00:00:00','','Institutul de Științe Spațiale' from dual union all
select 'RORD','Royal Ordnance','defense','','UK','Westcott, Buckinghamshire','1984-01-01 00:00:00','1998-01-01 00:00:00','Royal Ordnance' from dual union all
select 'ROSAP','Universita di Roma la Sapienaza (Scoula di Ingengneria Aerospaziale)','academic/non-profit','','I','Roma','1303-01-01 00:00:00','','Universita di Roma la Sapienaza (Scoula di Ingengneria Aerospaziale)' from dual union all
select 'ROSK','Korporatsii Roskosmos','business','','RU','Moskva, Rossiya','2016-01-01 00:00:00','','Государственная корпорация по космической деятельности "Роскосмос"' from dual union all
select 'ROSTO','Rossiskaya oboronnaya sportivnotekhnicheskaya organizatsiya','academic/non-profit','','RU','Moskva','1991-09-25 00:00:00','','Российская оборонная спортивно-техническая организация – РОСТО' from dual union all
select 'ROWAN','Rowan University','academic/non-profit','','US','Camden, New Jersey','1923-01-01 00:00:00','','Rowan University' from dual union all
select 'ROYAL','The Royal Group, Cambodia','business','','KH','Phnom Penh','1991-01-01 00:00:00','','The Royal Group, Cambodia' from dual union all
select 'RPEW','Propellants, Explosives and Rocket Motor Establishment','defense','MOD','UK','Westcott, Buckinghamshire','1958-08-01 00:00:00','1977-02-01 00:00:00','Propellants, Explosives and Rocket Motor Establishment' from dual union all
select 'RPI','Rocket Power Inc.','business','','US','Tempe, Arizona','1959-01-01 00:00:00','1964-01-01 00:00:00','Rocket Power Inc.' from dual union all
select 'RRC','Rocket Research Corp.','business','','US','Redmond, Washington','1960-01-01 00:00:00','1985-01-01 00:00:00','Rocket Research Corp.' from dual union all
select 'RRF','Rocket Research Facility, Fort Churchill','defense','AOMC','CA','Churchill, Manitoba','1957-01-01 00:00:00','1962-11-01 00:00:00','Rocket Research Facility, Fort Churchill' from dual union all
select 'RRL','Radio Research Laboratory, NTT','civilian','','J','Kashima, Ibaraki, Japan','1952-01-01 00:00:00','1985-01-01 00:00:00','電波研究所' from dual union all
select 'RROY','RollsRoyce Ltd.','civilian','','UK','Derby','1906-01-01 00:00:00','','Rolls-Royce Ltd.' from dual union all
select 'RROYC','Rolls Royce Engines (Coventry)','business','','UK','Coventry','1966-01-01 00:00:00','1994-01-01 00:00:00','Rolls Royce Engines (Coventry)' from dual union all
select 'RROYL','RollsRoyce Engines, Leavesden','business','RROY','UK','Watford:Leavesden, Hertfordshire','1966-01-01 00:00:00','1993-01-01 00:00:00','Rolls-Royce Engines, Leavesden' from dual union all
select 'RRS','Radio Research Station','civilian','SERC','UK','Ditton Park, Buckinghamshire','1924-01-01 00:00:00','1965-01-01 00:00:00','Radio Research Station' from dual union all
select 'RSC','Rocket Systems Corporation','business','','J','Tokyo','1990-07-05 00:00:00','2006-03-31 00:00:00','株)ロケットシステム' from dual union all
select 'RSQ','RASCOM Star QAF','business','I-RASC','MU','Port:Louis, Maurice','2002-01-01 00:00:00','','RASCOMStar-QAF' from dual union all
select 'RSRS','Radio and Space Research Station','civilian','SERC','UK','Ditton Park, Buckinghamshire','1965-01-01 00:00:00','1974-01-01 00:00:00','Radio and Space Research Station' from dual union all
select 'RTG','STG','civilian','','S','Stockholm:Solna','1961-01-01 00:00:00','1972-07-01 00:00:00','Rymdtekniska Gruppen' from dual union all
select 'RU','Russia','','','RU','Moskva, Rossiya','1991-12-26 00:00:00','','Российская Федерация' from dual union all
select 'RUAF','Russian Astronautical Federation','academic/non-profit','','RU','Moskva, Rossiya','1992-01-01 00:00:00','','Международная спутниковая программа для школьников' from dual union all
select 'RUAG','RUAG Aerospace Sweden AB, Limkoping factory','business','','S','Linkoping','2008-09-01 00:00:00','','RUAG Aerospace Sweden AB, Limkoping factory' from dual union all
select 'RUAGZ','RUAG Space, Zurich','business','','CH','Zurich','2009-07-01 00:00:00','','RUAG Space, Zurich' from dual union all
select 'RUL','Rutherford Laboratory','civilian','SERC','UK','Chilton, Oxfordshire','1957-01-01 00:00:00','1957-01-01 00:00:00','Rutherford Laboratory' from dual union all
select 'RVSN','Strategic Rocket Forces','defense','','SU','Mosvka?','1960-01-01 00:00:00','1991-12-01 00:00:00','Ракетные войска стратегического назначения' from dual union all
select 'RVSNR','Russian Strategic Rocket Forces','defense','','RU','Mosvka?','1991-12-01 00:00:00','','Ракетные войска стратегического назначения Российской Федерации' from dual union all
select 'RWI','Rockwell International (Air Force Plant 42, Palmdale)','business','','US','Palmdale, California','1973-01-01 00:00:00','1996-12-01 00:00:00','Rockwell International (Air Force Plant 42, Palmdale)' from dual union all
select 'RWID','Rockwell/Downey','business','','US','Downey, California','1973-01-01 00:00:00','1996-12-01 00:00:00','Rockwell/Downey' from dual union all
select 'RWISB','Rockwell/Seal Beach','business','','US','Seal Beach, California','1973-01-01 00:00:00','1996-12-01 00:00:00','Rockwell/Seal Beach' from dual union all
select 'S','Sweden','','','S','Stockholms stad, Sverige','0970-01-01 00:00:00','','Konungariket Sverige' from dual union all
select 'S394I','USAF Strategic Air Command, 394th ICBM Test Maintenance Sqn','defense','USAF','US','Vandenberg AFB, California','','','USAF Strategic Air Command, 394th ICBM Test Maintenance Sqn' from dual union all
select 'S4300','USAF Strategic Air Command, 4300th Support Sqn.','defense','USAF','US','Vandenberg AFB, California','','','USAF Strategic Air Command, 4300th Support Sqn.' from dual union all
select 'S7','S7 Space','business','','RU','Moskva','2016-01-01 00:00:00','','S7 Space' from dual union all
select 'S801','801st Research Institute','defense','','CN','Shanghai','1965-01-01 00:00:00','2007-01-01 00:00:00','航天工业总公司８０１所' from dual union all
select 'SA','Saudi Arabia','','','SA','ar-Riyad, Al Arabiyah','1926-01-08 00:00:00','','المملكة العربية السعودية' from dual union all
select 'SAAB','Swedish Aeroplane Co.','business','','S','Linkoping','1937-01-01 00:00:00','1969-01-01 00:00:00','Svenska Aeroplan Aktiebolaget' from dual union all
select 'SAABE','Saab Space, Linkoping factory','business','','S','Linkoping','1983-01-01 00:00:00','1992-01-01 00:00:00','Saab Space, Linkoping factory' from dual union all
select 'SAABS','SAABSCANIA AB','business','','S','Linkoping','1969-01-01 00:00:00','1983-01-01 00:00:00','SAAB-SCANIA AB' from dual union all
select 'SAABSA','Saab Ericsson Space, Linkoping factory','business','','S','Linkoping','1992-01-01 00:00:00','2006-01-01 00:00:00','Saab Ericsson Space, Linkoping factory' from dual union all
select 'SAABSP','Saab Space AB, Limkoping factory','business','','S','Linkoping','2006-01-01 00:00:00','2008-09-01 00:00:00','Saab Space AB, Limkoping factory' from dual union all
select 'SAAF','South African Air Force, AFB Waterkloof?','defense','','ZA','Pretoria:Verwoerdburg (Tshwane:Centurion)','1957-01-01 00:00:00','1994-01-01 00:00:00','South African Air Force, AFB Waterkloof?' from dual union all
select 'SABCA','SA Belge de Construction Aeronautique','business','','B','Brussel:Haren','1920-12-16 00:00:00','','SA Belge de Construction Aeronautique' from dual union all
select 'SAC','USAF Strategic Air Command, Offutt AFB','defense','USAF','US','Omaha, Nebraska','1948-11-09 00:00:00','1992-01-01 00:00:00','USAF Strategic Air Command, Offutt AFB' from dual union all
select 'SAFRAN','SAFRAN','business','','F','Paris:Evry?','2005-05-01 00:00:00','','SAFRAN' from dual union all
select 'SAFSC','Safeguard Systems Command','defense','USA','US','Huntsville, Alabama?','1969-01-01 00:00:00','1974-05-20 00:00:00','Safeguard Systems Command' from dual union all
select 'SAFSP','Secretary of the Air Force, Office of Special Projects (NRO Program A)','defense','NRO','US','Los Angeles:El Segundo, California','1961-09-01 00:00:00','1992-12-31 00:00:00','Secretary of the Air Force, Office of Special Projects (NRO Program A)' from dual union all
select 'SAIC','Science Applications International Corp.','business','','US','La Jolla, California','1969-01-01 00:00:00','','Science Applications International Corp.' from dual union all
select 'SAMSB','USAF Space and Missile Systems Organization (Norton AFB)','defense','USAF','US','Norton AFB, San Bernardino, California','1967-07-01 00:00:00','1979-10-01 00:00:00','USAF Space and Missile Systems Organization (Norton AFB)' from dual union all
select 'SAMSO','USAF Space and Missile Systems Organization','defense','USAF','US','El Segundo, California','1967-07-01 00:00:00','1979-10-01 00:00:00','USAF Space and Missile Systems Organization' from dual union all
select 'SAMX','Satelites Mexicanos SA de CV','business','','MX','Ciudad de Mexico:Lomas de Sotelo','1997-01-01 00:00:00','2014-03-07 00:00:00','Satélites Mexicanos S.A. de C. V.' from dual union all
select 'SAND','Sandia Laboratories','defense','','US','Albequerque, New Mexico','1948-04-01 00:00:00','1979-01-01 00:00:00','Sandia Laboratories' from dual union all
select 'SANDF','South African National Defense Force','defense','','ZA','Pretoria','1994-01-01 00:00:00','','South African National Defense Force' from dual union all
select 'SANH','Sanders Associates','business','','US','Nashua, New Hampshire','1952-01-01 00:00:00','1986-01-01 00:00:00','Sanders Associates' from dual union all
select 'SANHL','Lockheed/Sanders Associates','business','','US','Nashua, New Hampshire','1986-01-01 00:00:00','1995-01-01 00:00:00','Lockheed/Sanders Associates' from dual union all
select 'SANJ','Sanjiang hangtian jituan (Base 066)','business','','CN','Hankou, Wuhan, Hubei','1969-08-01 00:00:00','2011-01-01 00:00:00','三江航天集团' from dual union all
select 'SANJ9','Sanjiang Space Group (CASIC 9th Acad)','business','CASIC','CN','Hankou, Wuhan, Hubei','2011-01-01 00:00:00','','中国三江航天集团' from dual union all
select 'SANL','Sandia National Laboratories (SNLNM)','defense','','US','Albequerque, New Mexico','1979-01-01 00:00:00','','Sandia National Laboratories (SNL-NM)' from dual union all
select 'SAO','Smithsonian Astrophysical Observatory','civilian','','US','Cambridge, MA','1955-01-01 00:00:00','','Smithsonian Astrophysical Observatory' from dual union all
select 'SASG','Sky and Space Global (UK)','business','SASGA','UK','London','2015-01-01 00:00:00','','Sky and Space Global (UK)' from dual union all
select 'SASGA','Sky and Space Global Ltd (Australia)','business','','AU','Perth, W Australia','2016-01-01 00:00:00','','Sky and Space Global Ltd (Australia)' from dual union all
select 'SASK','University of Saskatchewan','academic/non-profit','','CA','Saskatoon, Saskatchewan','1907-01-01 00:00:00','','University of Saskatchewan' from dual union all
select 'SASMAC','Satellite Surveying and Mapping Applications Center','civilian','GCDX','CN','Beijing:Haidian','2009-01-01 00:00:00','','卫星测绘应用中心' from dual union all
select 'SAST','Shanghai Academy of Space Technology (CASC 8th Acad)','business','CASC','CN','Shanghai:Minghan','1993-01-01 00:00:00','','上海航天技术研究院' from dual union all
select 'SATBYUL','Satbyul','business','','KR','Suwon-si, Gyoenggi-do','2009-01-01 00:00:00','','' from dual union all
select 'SATEL','Satellife Inc','business','','US','Boston, MA','1986-01-01 00:00:00','','Satellife Inc' from dual union all
select 'SATELOG','Satellogic S.A.','business','','AR','San Carlos de Bariloche, Rio Negro','2011-01-01 00:00:00','','Satellogic S.A.' from dual union all
select 'SATGE','SATGE, GE International Holdings','business','','SG','Singapore','2007-06-01 00:00:00','','SAT-GE, GE International Holdings' from dual union all
select 'SATHYA','Sathyabama University','academic/non-profit','','IN','Chennai:Sholinganallur','1987-01-01 00:00:00','','சத்யபாமா ப‌ல்கலைக்கழகம்' from dual union all
select 'SBA','Shanghai Bureau of Astronautics','defense','','CN','Shanghai:Minhang','1980-01-01 00:00:00','1993-01-01 00:00:00','上海航天局' from dual union all
select 'SBA2','2nd Bureau, Shanghai Mech./Elec Industry','defense','','CN','Shanghai:Minhang','1961-08-01 00:00:00','1980-01-01 00:00:00','上海市第二机电工业局' from dual union all
select 'SBEL','Spacebel','business','','B','Liege','2010-01-01 00:00:00','','Spacebel' from dual union all
select 'SBRC','Santa Barbara Research Corp.','business','','US','Santa Barbara:Goleta','','1960-01-01 00:00:00','Santa Barbara Research Corp.' from dual union all
select 'SBRCH','Hughes Santa Barbara Research Corp.','business','','US','Santa Barbara:Goleta','1960-01-01 00:00:00','1997-01-01 00:00:00','Hughes Santa Barbara Research Corp.' from dual union all
select 'SBRCR','Raytheon Santa Barbara Research Corp.','business','','US','Santa Barbara:Goleta','1997-01-01 00:00:00','','Raytheon Santa Barbara Research Corp.' from dual union all
select 'SBS','Satellite Business Systems, Inc. (IBM/Aetna/Comsat)','business','','US','McLean, Virginia','1975-12-15 00:00:00','1985-07-01 00:00:00','Satellite Business Systems, Inc. (IBM/Aetna/Comsat)' from dual union all
select 'SCAL','Scaled Composites','business','','US','Mojave, California','1982-01-01 00:00:00','','Scaled Composites' from dual union all
select 'SCC','Space Communications Corp','business','','J','Tokyo:Shinagawa','1985-03-22 00:00:00','2008-08-01 00:00:00','宇宙通信（株）' from dual union all
select 'SCCJ','Space Communications Corp (Sky Perfect JSAT)','business','SKPJ','J','Tokyo:Shinagawa','2008-08-01 00:00:00','2009-01-01 00:00:00','（株）スカイパーフェクト・コミュニケーションズ' from dual union all
select 'SCEL','US Army Signal Corps Engineering Labs','defense','USA','US','Fort Monmouth, New Jersey','1949-01-01 00:00:00','1962-01-01 00:00:00','US Army Signal Corps Engineering Labs' from dual union all
select 'SCHJ','G T Schjeldahl Inc.','business','','US','Northfield, Minnesota','1955-01-01 00:00:00','1974-01-01 00:00:00','G T Schjeldahl Inc.' from dual union all
select 'SCSS','SCSSpace','business','','ZA','Cape Town','','','SCS-Space' from dual union all
select 'SCT','Secretaria de Comunicaciones y Transportes','business','','MX','Ciudad de Mexico','1891-01-01 00:00:00','','Secretaria de Comunicaciones y Transportes' from dual union all
select 'SCU','Santa Clara University','business','','US','Santa Clara, California','1851-01-01 00:00:00','','Santa Clara University' from dual union all
select 'SDC','US Army Strategic Defense Command (USASDC)','defense','','US','Huntsville, Alabama','1985-07-01 00:00:00','1992-08-01 00:00:00','US Army Strategic Defense Command (USASDC)' from dual union all
select 'SDCO','Space Data Corp.','business','','US','Phoenix:Tempe/Chandler, Arizona','1963-01-01 00:00:00','1988-01-01 00:00:00','Space Data Corp.' from dual union all
select 'SDIO','Strategic Defense Initiative Organization, DoD','defense','','US','Washington, DC:Pentagon','1984-01-01 00:00:00','1993-01-01 00:00:00','Strategic Defense Initiative Organization, DoD' from dual union all
select 'SDLP','Space Devices Ltd.','business','','CZ','Praha','1991-01-01 00:00:00','','Space Devices s.r.o.' from dual union all
select 'SDP','Services des Poudres','civilian','','F','Saint:Medard:en:Jalles, Gironde','1873-01-01 00:00:00','1971-01-01 00:00:00','Services des Poudres' from dual union all
select 'SDSC','Satish Dhawan Space Center','civilian','ISRO','IN','Sriharikota','2002-01-01 00:00:00','','Satish Dhawan Space Center' from dual;
insert into ORGANIZATION
select 'SEAL','Sea Launch AG','business','RKKE','CH','Bern','2010-01-01 00:00:00','2014-04-01 00:00:00','Sea Launch AG' from dual union all
select 'SEALC','Sea Launch Limited Partnership','business','','CYM','Georgetown, Cayman Islands','1997-01-01 00:00:00','2000-09-01 00:00:00','Sea Launch Limited Partnership' from dual union all
select 'SEALN','Sea Launch AG','business','RKKE','CH','Nyon, Vaud','2014-04-01 00:00:00','','Sea Launch AG' from dual union all
select 'SEALP','Sea Launch Limited Partnership','business','','US','Long Beach, California','2000-09-01 00:00:00','2010-01-01 00:00:00','Sea Launch Limited Partnership' from dual union all
select 'SECM','Shanghai Engineering Center for Microsatellites, SIMIT','academic/non-profit','SIMIT','CN','Shanghai','2003-01-01 00:00:00','','上海微小卫星工程中心' from dual union all
select 'SECT','Service d''Equipement de Champs de Tir','defense','','F','Paris?','','','Service des Équipements des Champs de Tir' from dual union all
select 'SEDSH','Students for the Exploration and Development of Space (HSV)','academic/non-profit','','US','Huntsville, Alabama','','','Students for the Exploration and Development of Space (HSV)' from dual union all
select 'SEL','Stanford Electronics Laboratory','business','','US','Palo Alto, California','1958-01-01 00:00:00','1969-01-01 00:00:00','Stanford Electronics Laboratory' from dual union all
select 'SELEN','Selenia Spazio SpA','business','','I','Roma','1968-01-01 00:00:00','1990-01-01 00:00:00','Selenia Spazio SpA' from dual union all
select 'SELGR','Berthold Seliger Forschungs und Entwicklungsgesellschaft mbH','business','','D','Cuxhaven?','1961-01-01 00:00:00','1964-01-01 00:00:00','Berthold Seliger Forschungs- und Entwicklungsgesellschaft mbH' from dual union all
select 'SELM','Shaanxi Engineering Laboratory for Microsatellites','academic/non-profit','NWPU','CN','Xi''an','2016-01-01 00:00:00','','陕西省微小卫星工程实验室' from dual union all
select 'SENER','SENER','business','','E','Bilbao?','1956-01-01 00:00:00','','SENER' from dual union all
select 'SENSC','Sentinel System Command','defense','USA','US','Huntsville, Alabama?','1967-01-01 00:00:00','1969-01-01 00:00:00','Sentinel System Command' from dual union all
select 'SEP','Soci\''et\''e Europ\''eene de Propulsion (Vernon)','business','','F','Vernon, Hte Normandie','1971-01-01 00:00:00','1997-01-01 00:00:00','Société européenne de propulsion' from dual union all
select 'SEPB','Soci\''et\''e Europ\''eene de Propulsion','business','','F','Bordeaux:Le Haillan','1969-01-01 00:00:00','1997-01-01 00:00:00','Soci\''et\''e Europ\''eene de Propulsion' from dual union all
select 'SEPBL','SEP Division Moteurs Spatiaux  Blanquefort','business','','F','Bordeaux:Blanquefort','1969-01-01 00:00:00','1997-01-01 00:00:00','SEP Division Moteurs Spatiaux - Blanquefort' from dual union all
select 'SEPR','Soci\''et\''e d''Etudes de la Propulsion par Reaction','business','','F','Bordeaux:Le Haillan','1964-01-01 00:00:00','1969-01-01 00:00:00','Société d''Études de la Propulsion par Réaction' from dual union all
select 'SEPRV','Soci\''et\''e d''Etudes de la Propulsion par Reaction (Villejuif)','business','','F','Paris:Villejuif','1944-01-01 00:00:00','1969-01-01 00:00:00','Société d''Études de la Propulsion par Réaction(Villejuif)' from dual union all
select 'SEPVR','Soci\''et\''e Europ\''eene de Propulsion (Villaroche)','business','','F','Villaroche, Seine-et-Marne','1947-01-01 00:00:00','1997-01-01 00:00:00','Société européenne de propulsion(Villaroche)' from dual union all
select 'SERC','UK Science and Engineering Research Council','civilian','','UK','Swindon, England','1981-01-01 00:00:00','1994-01-01 00:00:00','UK Science and Engineering Research Council' from dual union all
select 'SEREB','Soci\''et\''e pour l''etude et la r\''ealisation d''engins ballistiques','business','','F','Paris:Courbevoie, Hauts-de-Seine','1959-09-01 00:00:00','1970-07-01 00:00:00','Société pour l''étude et la réalisation d''engins balistiques' from dual union all
select 'SES','Soci\''et\''e Europeene des Satellites, Luxembourg','business','','L','Betzdorf','1985-01-01 00:00:00','2001-01-01 00:00:00','Société Européenne des Satellites' from dual union all
select 'SESA','SES Astra','business','SES','L','Betzdorf','1985-01-01 00:00:00','2011-01-01 00:00:00','SES Astra' from dual union all
select 'SESBM','SES Satellites (Bermuda) Ltd.','business','SES','BM','Hamilton','2010-11-02 00:00:00','','SES Satellites (Bermuda) Ltd.' from dual union all
select 'SESE','SES Engineering (Luxembourg)','business','SES','L','Betzdorf','2005-12-01 00:00:00','','SES Engineering (Luxembourg)' from dual union all
select 'SESG','SES Satellites (Gibraltar) Ltd.','business','SES','UK','Gibraltar','2001-01-01 00:00:00','','SES Satellites (Gibraltar) Ltd.' from dual union all
select 'SESGL','SES Global','business','','L','Betzdorf','2001-01-01 00:00:00','2006-01-01 00:00:00','SES Global' from dual union all
select 'SESNS','SES New Skies Satellites B.V.','business','SES','NL','''s:Gravenhage','2006-09-01 00:00:00','2009-09-01 00:00:00','SES New Skies Satellites B.V.' from dual union all
select 'SESS','SES Sirius AB','business','SES','S','Stockholm:Solna','2005-12-01 00:00:00','','SES Sirius AB' from dual union all
select 'SESSA','SES S.A.','business','','L','Betzdorf','2006-01-01 00:00:00','','SES S.A.' from dual union all
select 'SESSL','SES Satellite Leasing Ltd. (Isle of Man)','business','SES','UK','Douglas, Isle of Man','2005-12-01 00:00:00','','SES Satellite Leasing Ltd. (Isle of Man)' from dual union all
select 'SESW','Worldsat LLC','civilian','SES','US','Princeton, New Jersey?','2003-01-01 00:00:00','2005-02-01 00:00:00','Worldsat LLC' from dual union all
select 'SESWS','SES World Skies (The Hague)','business','SES','NL','''s:Gravenhage','2009-09-01 00:00:00','','SES World Skies (The Hague)' from dual union all
select 'SESWSP','SES World Skies (Princeton)','business','SES','US','Princeton, New Jersey','2009-09-01 00:00:00','','SES World Skies (Princeton)' from dual union all
select 'SFLIN','Spaceflight Industries Inc.','business','','US','Seattle:Tukwila, Washington','2015-01-01 00:00:00','','Spaceflight Industries Inc.' from dual union all
select 'SG','Singapore','','','SG','Singapore','1959-06-03 00:00:00','','Republic of Singapore (Republik Singapura)' from dual union all
select 'SGAU','Samara State Aerospace University','business','','RU','Samara (Kubyshev)','1992-01-01 00:00:00','','Самарский государственный аэрокосмический университет имени академика С.П.Королева' from dual union all
select 'SGSP','Sci. Group of Space Physics, Bulg. Acad. Sci.','academic/non-profit','','BGN','Sofia','1969-01-01 00:00:00','1975-01-01 00:00:00','Група по физика на космоса' from dual union all
select 'SH','St Helena','','','SH','St Helena','1834-01-01 00:00:00','','St Helena, British colony/overseas territory' from dual union all
select 'SHAB','Spacehab Inc.','business','','US','Vienna, Virginia','1984-01-01 00:00:00','2009-01-01 00:00:00','Spacehab Inc.' from dual union all
select 'SHABH','Spacehab Inc.','business','','US','Webster, Texas','1984-01-01 00:00:00','2009-01-01 00:00:00','Spacehab Inc.' from dual union all
select 'SHABT','Spacehab Inc.','business','','US','Austin, Texas','1984-01-01 00:00:00','2009-01-01 00:00:00','Spacehab Inc.' from dual union all
select 'SHANGT','Shanghai kexue daxue','academic/non-profit','','CN','Shanghai','2013-01-01 00:00:00','','上海科学大学' from dual union all
select 'SHANT','Shantou Institute of Electronics Technology','business','','CN','Shantou','','','汕头电子技术研究所' from dual union all
select 'SHAOX','China Junior Satellite Project','business','','CN','Beijing?','2017-01-01 00:00:00','','中国少年微星计划' from dual union all
select 'SHAR','Sriharikota Range','civilian','ISRO','IN','Sriharikota','1971-10-01 00:00:00','2002-01-01 00:00:00','Sriharikota Range' from dual union all
select 'SHEF','Sheffield University','academic/non-profit','','UK','Sheffield, South Yorks.','1905-01-01 00:00:00','','Sheffield University' from dual union all
select 'SHELD','Sheldahl Inc. (former GT Schjeldahl)','business','','US','Northfield, Minnesota','1974-01-01 00:00:00','','Sheldahl Inc. (former GT Schjeldahl)' from dual union all
select 'SHIN','Shin Satellite Plc.','business','','T','Bangkok:Nonthaburi','1999-08-01 00:00:00','2008-04-10 00:00:00','บริษัท ชินแซทเทลไลท์ จำกัด' from dual union all
select 'SHINSHU','Shinshu University','academic/non-profit','','J','Nagano:Wakasato','1949-01-01 00:00:00','','信州大学' from dual union all
select 'SHINW','Shinawatra Satellite Co. Ltd.','business','','T','Bangkok:Nonthaburi','1991-01-01 00:00:00','1999-08-01 00:00:00','บริษัท ชินแซทเทลไลท์ จำกัด' from dual union all
select 'SHIZ','Shizuoka University','academic/non-profit','','J','Shizuoka, Shizuoka','1949-01-01 00:00:00','','静岡大学' from dual union all
select 'SIBGAU','Siberian State Aerospace University','academic/non-profit','','RU','Krasnoyarsk','1960-01-01 00:00:00','','Сибирский государственный аэрокосмический университет имени академика М. Ф. Решетнёва' from dual union all
select 'SIENV','Sierra Nevada Corp.','business','','US','Sparks, Nevada','2008-01-01 00:00:00','','Sierra Nevada Corp.' from dual union all
select 'SII','Space Industries Inc','business','','US','Houston, Texas','1982-01-01 00:00:00','1993-07-01 00:00:00','Space Industries Inc' from dual union all
select 'SIIC','Space Industries International/Calspan','business','','US','Houston, Texas','1993-07-01 00:00:00','','Space Industries International/Calspan' from dual union all
select 'SIL','Space Innovations Ltd','business','','UK','Newbury, Berkshire','1997-01-01 00:00:00','','Space Innovations Ltd' from dual union all
select 'SILK','Silkwave Holdings Ltd','business','CMMB','CYM','George Town, Grand Cayman','2016-04-01 00:00:00','','Silkwave Holdings Ltd' from dual union all
select 'SIMED','Shanghai Inst of Machine and Electrical Design','academic/non-profit','','CN','Shanghai','1958-01-01 00:00:00','1968-01-01 00:00:00','上海机电设计院' from dual union all
select 'SIMG','Space Imaging Inc','business','','US','Thornton, Colorado','1994-01-01 00:00:00','2006-01-01 00:00:00','Space Imaging Inc' from dual union all
select 'SIMGEO','Space Imaging EOSAT','business','SIMG','US','Washington, DC/Lanham, Md.','1996-11-01 00:00:00','','Space Imaging EOSAT' from dual union all
select 'SIMIT','Shanghai Inst of Microsystem and Information Technology','academic/non-profit','','CN','Shanghai','1928-01-01 00:00:00','','上海微系统与信息技术研究' from dual union all
select 'SINDO','PT Satellit Palapa Indonesia (Satelindo)','business','','ID','Jakarta','1993-01-01 00:00:00','2003-01-01 00:00:00','PT Satellit Palapa Indonesia (Satelindo)' from dual union all
select 'SINO','Sino Satellite Communications Co. (Sinosat)','business','','CN','Beijing:Haidian','1994-05-01 00:00:00','','鑫诺卫星通信公司' from dual union all
select 'SIR','Sirius Satellite Radio Inc','business','','US','New York, New York','1999-11-18 00:00:00','2008-08-05 00:00:00','Sirius Satellite Radio Inc' from dual union all
select 'SIRX','Sirius XM Radio Inc','business','','US','New York, New York','2008-08-05 00:00:00','','Sirius XM Radio Inc' from dual union all
select 'SIS','Space International Services','business','','RU','Moskva','2005-01-01 00:00:00','','Space International Services' from dual union all
select 'SISE','Shanghai Inst. of Satellite Engineering (509 Res.Inst.)','business','SAST','CN','Shanghai','','','上海卫星工程研究所' from dual union all
select 'SISP','Shanghai Int. of Space Propulsion','defense','','CN','Shanghai','2007-01-01 00:00:00','','上海空间推进研究所' from dual union all
select 'SITAEL','SITAEL SpA.','business','','I','Forli','2014-01-01 00:00:00','','Sitael SpA' from dual union all
select 'SITP','Shanghai Inst. of Technical Physics, CAS','academic/non-profit','CAS','CN','Shanghai','1958-01-01 00:00:00','','中国科学院上海技术物理研究所' from dual union all
select 'SJSU','San Jose State University','academic/non-profit','','US','San Jose, California','1857-01-01 00:00:00','','San Jose State University' from dual union all
select 'SK','Slovakia','','','SK','Bratislava','1993-01-01 00:00:00','','Slovenská republika' from dual union all
select 'SKB30','SKB30 (G B Kisunko)','defense','','SU','Mosvka:Sokol','1960-01-01 00:00:00','1975-01-01 00:00:00','СКБ-30' from dual union all
select 'SKB385','SKB385 (Makeev)','civilian','','SU','Miass','1959-01-01 00:00:00','1993-01-01 00:00:00','СКБ-385' from dual union all
select 'SKB385Z','SKB385, Zlatoust','civilian','','SU','Zlatoust','1949-01-01 00:00:00','1959-01-01 00:00:00','Специальном конструкторском бюро N385, Златоуст' from dual union all
select 'SKC','Salish Kootenai College','academic/non-profit','','US','Pablo, Montana','1977-01-01 00:00:00','','Salish Kootenai College' from dual union all
select 'SKPJ','Sky Perfect JSAT Corp.','business','','J','Tokyo:Minato','2008-08-01 00:00:00','','スカパーJSAT株式会社' from dual union all
select 'SKTEL','SK Telecom','business','','KR','Seoul:Jung:gu','1997-03-01 00:00:00','','에스케이텔레콤' from dual union all
select 'SKTELI','SK Telink','business','SKTEL','KR','Seoul','2010-01-01 00:00:00','','SK Telink' from dual union all
select 'SKYB','DirecTV Latin America/Sky Brasil','business','DTV','BR','Rio de Janeiro','1996-01-01 00:00:00','','DirecTV Latin America/SKY Brasil' from dual union all
select 'SKYBOX','Skybox Imaging','business','','US','San Jose:Mountain View, California','2012-01-01 00:00:00','2016-03-01 00:00:00','Skybox Imaging' from dual union all
select 'SKYM','SKY Mexico','business','DTV','MX','Mexico City','1996-01-01 00:00:00','','SKY México' from dual union all
select 'SKYTC','SkyTerra Comms','business','','US','Reston, Virginia','2003-01-01 00:00:00','2010-07-01 00:00:00','SkyTerra Comms' from dual union all
select 'SKYTE','SkyTerra LP','business','SKYTC','US','Reston, Virginia','2008-12-01 00:00:00','','SkyTerra LP' from dual union all
select 'SLII','Southwest Louisiana Industrial Institute','academic/non-profit','','US','Lafayette, Louisiana','1900-01-01 00:00:00','1960-01-01 00:00:00','Southwest Louisiana Industrial Institute' from dual union all
select 'SLREC','Academy of Aerospace Propulsion Technology (CASC 6th Acad)','business','CASC','CN','Xi''an, Shaanxi','2002-01-01 00:00:00','','航天推进技术研究院' from dual union all
select 'SLU','Saint Louis University','academic/non-profit','','US','St Louis, Missouri','1818-01-01 00:00:00','','Saint Louis University' from dual union all
select 'SMCTE','USAF Space and Missile Center, TE  directorate','defense','USAF','US','Kirtland AFB, Albequerque, New Mexico','1995-07-01 00:00:00','2001-06-29 00:00:00','USAF Space and Missile Center, TE  directorate' from dual union all
select 'SMCTV','USAF Space and Missile Center, TEV directorate','defense','USAF','US','Kirtland AFB, Albequerque, New Mexico','1995-07-01 00:00:00','2001-06-29 00:00:00','USAF Space and Missile Center, TEV directorate' from dual union all
select 'SMCXR','USAF Space and Missile Center, Development Planning Directorate','defense','USAF','US','Kirtland AFB, Albequerque, New Mexico','2005-01-01 00:00:00','','USAF Space and Missile Center, Development Planning Directorate' from dual union all
select 'SMDC','US Army Space and Missile Defense Command (USASMDC)','defense','','US','Huntsville, Alabama','1997-10-01 00:00:00','','US Army Space and Missile Defense Command (USASMDC)' from dual union all
select 'SMSA','US Army Signal Missile Support Agency','defense','USA','US','White Sands, New Mexico','','1963-12-01 00:00:00','US Army Signal Missile Support Agency' from dual union all
select 'SNCASE','SN Southeast Aero Construction','business','','F','Cannes','1937-02-01 00:00:00','1957-03-01 00:00:00','Société nationale de constructions aéronautiques du sud-est' from dual union all
select 'SNCASO','SN Southwest Aero Construction','business','','F','Paris:Courbevoie','1945-01-01 00:00:00','1957-03-01 00:00:00','Société nationale des constructions aéronautiques du sud-ouest' from dual union all
select 'SNECB','SNECMA SEP Division','business','','F','Bordeaux:Le Haillan/St-Medard','1997-01-01 00:00:00','2002-02-28 00:00:00','SNECMA Division SEP' from dual union all
select 'SNECBL','SNECMA Space Motors','business','','F','Bordeaux:Blanquefort','1965-01-01 00:00:00','1969-01-01 00:00:00','SNECMA Division Moteurs Spatiaux - Blanquefort' from dual union all
select 'SNECDE','SNECMA','business','SNECMA','F','Paris:Evry?','1962-01-01 00:00:00','1969-01-01 00:00:00','SNECMA Divsion des Engins et de l''Espace' from dual union all
select 'SNECMA','SNECMA Motors','business','','F','Paris:Evry','1966-01-01 00:00:00','2005-04-01 00:00:00','Société nationale d''études et de construction de moteurs d''aviation' from dual union all
select 'SNECMS','SNECMA Motors','business','SAFRAN','F','Paris:Evry','2005-04-01 00:00:00','','SAFRAN SNECMA Moteurs' from dual;
insert into ORGANIZATION
select 'SNECPS','SNECMA Solid Propulsion','business','','F','Bordeaux:Le Haillan','2002-02-28 00:00:00','2012-04-01 00:00:00','SNECMA Propulsion Solide' from dual union all
select 'SNECV','SNECMA Space Motors','business','','F','Vernon, Eure','1997-01-01 00:00:00','','SNECMA Moteurs Spatiaux (Vernon)' from dual union all
select 'SNECVR','SNECMA','business','','F','Villaroche, Seine-et-Marne','1997-01-01 00:00:00','2006-01-01 00:00:00','SNECMA-Villaroche' from dual union all
select 'SNPE','Societe Nationale de Poudres et Explosifs (SNPE)','business','','F','Saint-Medard-en-Jalles, Gironde','1971-03-08 00:00:00','2012-04-01 00:00:00','Société nationale des poudres et des explosifs' from dual union all
select 'SNSB','Swedish National Space Board','civilian','','S','Stockholm:Solna','1972-07-01 00:00:00','','Rymdstyrelsen' from dual union all
select 'SNU','Seoul National University','academic/non-profit','','KR','Seoul:Gwanak','1946-01-01 00:00:00','','서울대학교' from dual union all
select 'SNVL','Sierra Nevada Corp. Space Systems (Louisville)','business','','US','Boulder:Louisville, Colorado','2009-01-01 00:00:00','','Sierra Nevada Corp. Space Systems (Louisville)' from dual union all
select 'SNVP','Sierra Nevada Corp. Space Systems (SpaceDev)','business','','US','San Diego:Poway, California','2008-01-01 00:00:00','','Sierra Nevada Corp. Space Systems (SpaceDev)' from dual union all
select 'SOCOM','US Special Operations Command','defense','DOD','US','MacDill AFB, Tampa, Florida','1987-01-01 00:00:00','','US Special Operations Command' from dual union all
select 'SOHLA','Space Oriented Higashiosaka Leading Association','business','','J','Osaka:Higashiosaka (E Osaka)','2003-01-01 00:00:00','','宇宙開発協同組合 SOHLA' from dual union all
select 'SOKA','Soka Daigaku','academic/non-profit','','J','Tokyo:Hachioji','1969-01-01 00:00:00','','創価大学' from dual union all
select 'SOLM','Solaris Mobile','business','','IE','Dublin','2008-01-01 00:00:00','2015-03-01 00:00:00','Solaris Mobile Ltd.' from dual union all
select 'SOR','USAF Starfire Optical Range, AFRL','defense','USAF','US','Kirtland AFB, Albuquerque, New Mexico','1993-01-01 00:00:00','','USAF Starfire Optical Range, AFRL' from dual union all
select 'SORUN','Sorun Corporation','business','','J','Tokyo:Minato','2001-01-01 00:00:00','','ソラン株式会社' from dual union all
select 'SOSA','Slovak Organization for Space Activities','academic/non-profit','','SK','Bratislava/Vel''ka Paka','2013-01-01 00:00:00','','Slovenská organizácia pre vesmírne aktivity' from dual union all
select 'SOTI','State Organization for Technical Industries (SOTI)','defense','','IQ','Baghdad, Iraq','1974-01-01 00:00:00','1987-08-01 00:00:00','State Organization for Technical Industries (SOTI)' from dual union all
select 'SOTON','University of Southampton','academic/non-profit','','UK','Southampton, Hampshire','1952-01-01 00:00:00','','University of Southampton' from dual union all
select 'SOVA','Soviet Army','defense','VS','SU','Moskva','1946-01-01 00:00:00','1992-01-01 00:00:00','Советская Армия' from dual union all
select 'SOYUZ','NPO Soyuz','business','','RU','Mosvka:Lyubertsy/Dzerzhinskiy','1980-01-01 00:00:00','','НПО Союз' from dual union all
select 'SPA','Spectrum Astro','business','','US','Gilbert, Arizona','1988-01-01 00:00:00','2004-01-01 00:00:00','Spectrum Astro' from dual union all
select 'SPAC','Satellite Positioning Research and Application Center','civilian','','J','Tokyo:Chiyoda','2007-01-01 00:00:00','','衛星測位利用推進センター' from dual union all
select 'SPAR','Spar Aerospace Robotics Div. (Brampton)','business','','CA','Brampton, Ontario','1992-01-01 00:00:00','1999-03-01 00:00:00','Spar Aerospace Robotics Div. (Brampton)' from dual union all
select 'SPARD','Spar Aerospace','business','','CA','Toronto:Downsview, Ontario','1967-01-01 00:00:00','','Spar Aerospace' from dual union all
select 'SPARM','Spar Aerospace Space Systems','business','','CA','Montreal:St-Anne-de-Bellevue, Quebec','1967-01-01 00:00:00','1999-01-01 00:00:00','Spar Aerospace Space Systems' from dual union all
select 'SPARV','Spar Aerospace (MontrealSt Henri)','business','','CA','Montreal:St Henri','1977-01-01 00:00:00','1985-01-01 00:00:00','Spar Aerospace (Montreal-St Henri)' from dual union all
select 'SPAWAR','Space and Naval Warfare Systems Command, Space Field Activity','defense','USN','US','Chantilly, Virginia','1987-01-01 00:00:00','','Space and Naval Warfare Systems Command, Space Field Activity' from dual union all
select 'SPAWSD','Space and Naval Warfare Systems Command, San Diego','defense','SPAWAR','US','San Diego, California','2004-01-01 00:00:00','','Space and Naval Warfare Systems Command, San Diego' from dual union all
select 'SPCOM','Contel Space Communications Co. (Spacecom)','business','CONTF','US','Gaithersburg, Maryland','1976-01-01 00:00:00','1993-01-01 00:00:00','Contel Space Communications Co. (Spacecom)' from dual union all
select 'SPDEV','SpaceDev','business','','US','San Diego:Poway, California','1997-01-01 00:00:00','2008-01-01 00:00:00','SpaceDev' from dual union all
select 'SPER','Sperry Utah Co.','business','','US','Salt Lake City, Utah','1960-01-01 00:00:00','1986-01-01 00:00:00','Sperry Utah Co.' from dual union all
select 'SPETZ','AO Spetztekhnika','defense','','SU','Sverdlovsk (Ekaterinburg)','1970-01-01 00:00:00','','АО спецтехника' from dual union all
select 'SPFL','Spaceport Florida Authority','civilian','','US','Cape Canaveral, Florida','1989-01-01 00:00:00','2000-01-01 00:00:00','Spaceport Florida Authority' from dual union all
select 'SPG','Space General Div, AerojetGeneral Corp.','business','AJ','US','El Monte, California','1961-01-01 00:00:00','','Space General Div, Aerojet-General Corp.' from dual union all
select 'SPHARM','SpacePharma R&D group','business','SPHARS','IL','Herzliya','2012-01-01 00:00:00','','SpacePharma R&D group, Israel' from dual union all
select 'SPHARS','SpacePharma','business','','CL','Basel:Courgenay','2012-01-01 00:00:00','','SpacePharma' from dual union all
select 'SPIRE','Spire','civilian','','US','San Francisco, California','2014-08-01 00:00:00','','Spire' from dual union all
select 'SPIREG','Spire/Glasgow','civilian','','UK','Glasgow','2015-06-01 00:00:00','','Spire/Glasgow' from dual union all
select 'SPMIC','Space Micro','business','','US','San Diego','2002-01-01 00:00:00','','Space Micro' from dual union all
select 'SPOG','Special Projectiles Operation Group','defense','UKWO','UK','Cuxhaven, Germany','1945-01-01 00:00:00','1945-01-01 00:00:00','Special Projectiles Operation Group' from dual union all
select 'SPOT','SPOT Image','business','','F','Toulouse','1982-01-01 00:00:00','','SPOT Image' from dual union all
select 'SPQ','SpaceQuest','academic/non-profit','','US','Fairfax, Virginia','1994-01-01 00:00:00','','SpaceQuest' from dual union all
select 'SPUT','SPUTNIX Ltd','business','','RU','Moskva:Rumyantsevo','2010-01-01 00:00:00','','СПУТНИКС (ООО Спутниковые инновационные космические системы)' from dual union all
select 'SPVR','SpaceVR','business','','US','San Francisco, California','2015-01-01 00:00:00','','SpaceVR' from dual union all
select 'SPX','SpaceX','business','','US','Hawthorne, California','2007-08-01 00:00:00','','SpaceX' from dual union all
select 'SPXES','SpaceX (El Segundo)','business','','US','El Segundo, California','2002-01-01 00:00:00','2007-08-01 00:00:00','SpaceX (El Segundo)' from dual union all
select 'SPXS','SpaceX (Seattle)','business','','US','Seattle:Redmond, Washington','2015-06-01 00:00:00','','SpaceX (Seattle)' from dual union all
select 'SPXTX','SpaceX (McGregor)','business','','US','McGregor, Texas','2003-01-01 00:00:00','','SpaceX (McGregor)' from dual union all
select 'SR','Suriname','','','SR','Paramaribo','1975-01-01 00:00:00','','Republiek Suriname' from dual union all
select 'SRC','UK Science Research Council','civilian','','UK','Swindon, England','1965-01-01 00:00:00','1981-01-01 00:00:00','UK Science Research Council' from dual union all
select 'SRCI','Space Research Coporation','business','','CA','Highwater, Quebec','1966-01-01 00:00:00','1980-01-01 00:00:00','Space Research Coporation' from dual union all
select 'SRDL','US Army Signal Corps R&D Labs','defense','USA','US','Ft Monmouth, New Jersey','1929-01-01 00:00:00','1962-08-01 00:00:00','US Army Signal Corps R&D Labs' from dual union all
select 'SRI','SRI','business','','US','Menlo Park, California','1970-01-01 00:00:00','1977-01-01 00:00:00','SRI' from dual union all
select 'SRIBAS','Space Research Inst.','academic/non-profit','','BG','Sofia','1987-01-01 00:00:00','','Институт за космически изследвания и технологии' from dual union all
select 'SRII','SRI International','business','','US','Menlo Park, California','1977-01-01 00:00:00','','SRI International' from dual union all
select 'SRIS','Stanford Research Institute','business','STAN','US','Menlo Park, California','1946-01-01 00:00:00','1970-01-01 00:00:00','Stanford Research Institute' from dual union all
select 'SRMEC','SRM (Sri Ramaswamy Memorial) Engineering College, Anna University','academic/non-profit','ANNAU','IN','Chennai','1985-01-01 00:00:00','2006-01-01 00:00:00','SRM (Sri Ramaswamy Memorial) Engineering College, Anna University' from dual union all
select 'SRMU','SRM (Sri Ramaswamy Memorial) University','academic/non-profit','','IN','Chennai','2006-01-01 00:00:00','','SRM (Sri Ramaswamy Memorial) University' from dual union all
select 'SROE','SaundersRoe (Saro)','business','','UK','Cowes, Isle of Wight','1929-01-01 00:00:00','1965-01-01 00:00:00','Saunders-Roe (Saro)' from dual union all
select 'SRON','Space Research Org of the Netherlands','civilian','','NL','Utrecht','1983-01-01 00:00:00','','Space Research Org. of the Netherlands (SRON)' from dual union all
select 'SRSES','Space Recovery Systems','business','','US','El Segundo, California','1957-01-01 00:00:00','','Space Recovery Systems' from dual union all
select 'SS500','SS Battery 500','defense','','DR','Den Haag','1944-01-01 00:00:00','1945-01-01 00:00:00','SS Werfer-Abteilung 500' from dual union all
select 'SSC','Swedish Space Corporation','civilian','','S','Stockholm:Solna','1972-07-01 00:00:00','','Svenska Rymdaktiebolaget (Rymd AB, Swedish Space Corporation)' from dual union all
select 'SSCO','Seattle Scientific Corp.','business','','US','Seattle, Washington','','','Seattle Scientific Corp.' from dual union all
select 'SSCS','Swedish Space Corporation','business','SSC','S','Stockholm:Solna','1972-07-01 00:00:00','2011-06-01 00:00:00','Svenska rymdaktiebolaget' from dual union all
select 'SSDC','US Army Space and Strategic Defense Command','defense','USA','US','Huntsville, Alabama','1992-08-01 00:00:00','1997-08-01 00:00:00','US Army Space and Strategic Defense Command' from dual union all
select 'SSI','Space Services Inc.','business','','US','Houston, Texas','1980-01-01 00:00:00','1990-12-01 00:00:00','Space Services Inc.' from dual union all
select 'SSICE','Celestis, Inc. (SSI)','academic/non-profit','SSI','US','Houston, Texas','1994-01-01 00:00:00','','Celestis, Inc. (SSI)' from dual union all
select 'SSIH','Space Services Inc.','business','','US','Houston, Texas','2001-01-01 00:00:00','','Space Services Inc.' from dual union all
select 'SSIM','Space Systems International  Monaco S.A.M.','business','','MC','Monaco','2009-01-01 00:00:00','','Space Systems International - Monaco S.A.M.' from dual union all
select 'SSL','SS/Loral  (MDA)','business','MDABC','US','Palo Alto, California','2012-11-01 00:00:00','2017-10-01 00:00:00','SS/Loral  (MDA)' from dual union all
select 'SSLMAX','SS/Loral  (Maxar)','business','MAXAR','US','Palo Alto, California','2017-10-01 00:00:00','','SS/Loral  (MDA)' from dual union all
select 'SSPO','USN Strategic Systems Programs Office','defense','USN','US','Sunnyvale, California','1968-01-01 00:00:00','1987-01-01 00:00:00','USN Strategic Systems Programs Office' from dual union all
select 'SSTARS','Southern Stars Group LLC','business','','US','San Francisco, California','1993-01-01 00:00:00','','Southern Stars Group LLC' from dual union all
select 'SSTL','Surrey Satellite Technology Ltd','business','SURREY','UK','Guildford, Surrey, England','1985-01-01 00:00:00','2009-01-01 00:00:00','Surrey Satellite Technology Ltd' from dual union all
select 'SSTLA','Surrey Satellite Technology Ltd/Astrium','business','ASTUK','UK','Guildford, Surrey, England','2009-01-01 00:00:00','','Surrey Satellite Technology Ltd/Astrium' from dual union all
select 'SSTUS','Surrey Satellite Technology US LLC','business','SSTL','US','Denver:Englewood, Colorado','2011-01-01 00:00:00','2017-11-13 00:00:00','Surrey Satellite Technology US LLC' from dual union all
select 'ST2V','ST2 Satellite Ventures','business','STEL','SG','Singapore','2008-01-01 00:00:00','','ST-2 Satellite Ventures Pte Ltd (STS) (STEL/CHUNG)' from dual union all
select 'STA','Science and Technology Agency','academic/non-profit','','J','Tokyo','1958-01-01 00:00:00','2001-01-01 00:00:00','科学技術庁' from dual union all
select 'STADK','STADOKO UG','business','','D','Aachen','2012-01-01 00:00:00','2014-06-01 00:00:00','STADOKO UG' from dual union all
select 'STAN','Stanford University','academic/non-profit','','US','Stanford, California','1885-01-01 00:00:00','','Stanford University' from dual union all
select 'STBO','Stellenbosch University','academic/non-profit','','ZA','Cape Town, South Africa','1866-01-01 00:00:00','','Stellenbosch University' from dual union all
select 'STEL','Singapore Telecom','business','','SG','Singapore','1992-03-01 00:00:00','','Singapore Telecom' from dual union all
select 'STELEC','Singapore Technologies Electronics Ltd','business','','SG','Singapore','1969-01-01 00:00:00','','Singapore Technologies Electronics Ltd' from dual union all
select 'STELLT','Stellat','business','','F','Paris?','2001-02-01 00:00:00','2002-08-01 00:00:00','Stellat' from dual union all
select 'STEN','STENSAT','academic/non-profit','','US','Washington, DC','1999-01-01 00:00:00','','STENSAT' from dual union all
select 'STENN','NASA Stennis Space Center','civilian','NASA','US','Pearl River, Mississippi','1988-01-01 00:00:00','','NASA Stennis Space Center' from dual union all
select 'STESS','Singapore Technologies Electronics (Satellite Systems)','business','STELEC','SG','Singapore','2011-05-01 00:00:00','','Singapore Technologies Electronics (Satellite Systems)' from dual union all
select 'STFC','UK Science and Technology Facilities Council','civilian','','UK','Swindon, England','2007-01-01 00:00:00','','UK Science and Technology Facilities Council' from dual union all
select 'STG','NASA Space Task Group','civilian','NASA','US','Langley Field, Virginia','1963-01-03 00:00:00','','NASA Space Task Group' from dual union all
select 'STGL','NASA Langley Research Center, Space Task Group','civilian','NASA','US','Langley Field, Virginia','1958-10-01 00:00:00','1963-01-03 00:00:00','NASA Langley Research Center, Space Task Group' from dual union all
select 'STL','Space Technology Laboratories','business','','US','LA:Redondo Beach, California','1957-01-01 00:00:00','1958-01-01 00:00:00','Space Technology Laboratories' from dual union all
select 'STLC','IBM Satellite Transponder Leasing Corp.','business','','US','McLean, Virginia','1986-01-01 00:00:00','1990-04-01 00:00:00','IBM Satellite Transponder Leasing Corp.' from dual union all
select 'STLCH','Satellite Transponder Leasing Corp.','business','HCI','US','McLean, Virginia','1990-04-01 00:00:00','2001-01-01 00:00:00','Satellite Transponder Leasing Corp.' from dual union all
select 'STMS','St. Thomas More Cathedral School','academic/non-profit','','US','Arlington, Virginia','1944-01-01 00:00:00','','St. Thomas More Cathedral School' from dual union all
select 'STO','Stockholm University','academic/non-profit','','S','Stockholm','1878-01-01 00:00:00','','Stockholms universitet' from dual union all
select 'STONE','Star One','business','EMBR','BR','Rio de Janeiro','2000-12-01 00:00:00','','Star One' from dual union all
select 'STP','USAF SMC/DoD Space Test Program','defense','USAF','US','Kirtland AFB, Albuquerque, New Mexico','1967-01-01 00:00:00','','USAF SMC/DoD Space Test Program' from dual union all
select 'STRELA','PO Strela','business','','SU','Orenburg','1986-04-07 00:00:00','','Производственное объединение «Стрела»' from dual union all
select 'STSM','Starsem','business','','F','Paris:Evry','1996-08-06 00:00:00','','Starsem' from dual union all
select 'STST','Starstruck Inc.','business','','US','San Francisco:Redwood City, California','1980-01-01 00:00:00','1985-01-01 00:00:00','Starstruck Inc.' from dual;
insert into ORGANIZATION
select 'SU','USSR','','','SU','Moskva, RSFSR, SSSR','1922-12-30 00:00:00','1991-12-26 00:00:00','Союз Советских Социалистических Республик' from dual union all
select 'SU190','Unknown USSR manufacturer 190','business','','SU','::','','','Unknown USSR' from dual union all
select 'SU523','Unknown USSR manufacturer 523','business','','SU','::','','','Unknown USSR' from dual union all
select 'SU782','Unknown USSR manufacturer 782','business','','SU','::','','','Unknown USSR' from dual union all
select 'SUD','Sud Aviation (South Aviation)','business','','F','Cannes','1957-03-01 00:00:00','1970-01-01 00:00:00','Sud Aviation' from dual union all
select 'SUDC','Sud Aviation, Ch. Courbevoie  (SEREB)','business','','F','Paris:Courbevoie','1957-03-01 00:00:00','1970-01-01 00:00:00','Sud Aviation, Ch. Courbevoie  (SEREB)' from dual union all
select 'SUEL','Sperry Utah Engineering Labs','business','','US','Salt Lake City, Utah','1956-01-01 00:00:00','1960-01-01 00:00:00','Sperry Utah Engineering Labs' from dual union all
select 'SUI','State University of Iowa (Iowa University)','academic/non-profit','','US','Iowa City, Iowa','1847-01-01 00:00:00','1964-01-01 00:00:00','State University of Iowa (Iowa University)' from dual union all
select 'SUPA','Space and Upper Atmosphere Research Commission','civilian','','PK','Islamabad','1961-09-01 00:00:00','','خلائی و بالائی فضائی تحقیقاتی مأموریہ' from dual union all
select 'SUPAER','National Higher School of Aeronautics and Space','academic/non-profit','','F','Toulouse','1909-01-01 00:00:00','2007-01-01 00:00:00','École nationale supérieure de l''aéronautique et de l''espace' from dual union all
select 'SUPSAT','SupremeSAT (Pvt) Ltd.','business','','LK','Colombo?','2012-01-01 00:00:00','','SupremeSAT (Pvt) Ltd.' from dual union all
select 'SUPSI','Scuola Universitaria Professionale della Svizzera, Dipart. Tecn. Innovative','academic/non-profit','','CH','Manno, Repubblica e Cantone Ticino','1997-01-01 00:00:00','','Scuola Universitaria Professionale della Svizzera, Dipart. Tecn. Innovative' from dual union all
select 'SURREY','Surrey Space Centre, University of Surrey','academic/non-profit','','UK','Guildford, Surrey','1985-01-01 00:00:00','','Surrey Space Centre, University of Surrey' from dual union all
select 'SUSX','University of Sussex','academic/non-profit','','UK','Brighton:Falmer, Sussex, England','1961-08-01 00:00:00','','University of Sussex' from dual union all
select 'SUT','SUT','academic/non-profit','','IR','Tehran','1966-01-01 00:00:00','','دانشگاه صنعتی شریف' from dual union all
select 'SVC','Space Vector Corp','business','','US','LA:Chatsworth/Canoga Park, California','1969-01-01 00:00:00','','Space Vector Corp' from dual union all
select 'SVEC','Space Vacuum Epitaxy Center','business','HOUS','US','Houston, Texas','1985-01-01 00:00:00','2002-01-01 00:00:00','Space Vacuum Epitaxy Center' from dual union all
select 'SWALES','Swales Aerospace','business','','US','Beltsville, Maryland','1978-01-01 00:00:00','2007-06-01 00:00:00','Swales Aerospace' from dual union all
select 'SWARM','Swarm Technologies','business','','US','Pasadena, California','2016-01-01 00:00:00','','Swarm Technologies' from dual union all
select 'SWCAS','Southwest Center for Advanced Studies','academic/non-profit','','US','Dallas, Texas','1961-01-01 00:00:00','1969-01-01 00:00:00','Southwest Center for Advanced Studies' from dual union all
select 'SWRI','SouthWest Research Institute','academic/non-profit','','US','San Antonio, Texas','1947-01-01 00:00:00','','SouthWest Research Institute' from dual union all
select 'SWRIB','SouthWest Research Institute (Boulder)','academic/non-profit','SWRI','US','Boulder, Colorado','1994-01-01 00:00:00','','SouthWest Research Institute (Boulder)' from dual union all
select 'SY','Syria','','','SY','Dimashq, Suriya','1944-01-01 00:00:00','','الجمهورية العربية السورية' from dual union all
select 'SYMC','Syrian Missile Command','defense','','SY','Aleppo','1980-01-01 00:00:00','','Syrian Missile Command' from dual union all
select 'SYRU','Syracuse University','academic/non-profit','','US','Syracuse, New York','1870-01-01 00:00:00','','Syracuse University' from dual union all
select 'SZRTC','Shaanxi Zhongtian Rocket Technology Ltd','business','ARMT','CN','Xi''an, Shaanxi','2012-01-01 00:00:00','','陕西中天火箭技术股份有限公司' from dual union all
select 'T','Thailand','','','T','Bangkok, Prathet Thai','1238-01-01 00:00:00','','ประเทศไทย' from dual union all
select 'TAIFUN','NPO Typhoon','civilian','','SU','Obninsk, Kaluga Oblast','1980-01-01 00:00:00','','НПО «Тайфун» (Институт экспериментальной метеорологии)' from dual union all
select 'TAMA','Tama Art University, Hachioji campus','academic/non-profit','','J','Tokyo:Hachioji','1950-01-01 00:00:00','','多摩美術大学' from dual union all
select 'TAMU','Texas A&M University','academic/non-profit','','US','College Station, Texas','1876-01-01 00:00:00','','Texas A&M University' from dual union all
select 'TAO','Tokyo Astronomical Observatory','academic/non-profit','','J','Tokyo:Mitaka','1924-01-01 00:00:00','1988-01-01 00:00:00','東京天文台' from dual union all
select 'TAOJ','Telecommunications Advancement Organization of Japan','academic/non-profit','','J','Tokyo:Minato','1992-10-01 00:00:00','','通信・放送機構' from dual union all
select 'TARTU','University of Tartu','academic/non-profit','','EE','Tartu','1632-01-01 00:00:00','','Tartu Ülikool' from dual union all
select 'TAYLU','Taylor University','academic/non-profit','','US','Upland, Indiana','1846-01-01 00:00:00','','Taylor University' from dual union all
select 'TBELLA','Google Terra Bella','business','','US','San Jose:Mountain View, California','2016-03-01 00:00:00','2017-04-01 00:00:00','Skybox Imaging' from dual union all
select 'TC','Turks and Caicos','','UK','TC','Cockburn Town, Grand Turk Island','1799-01-01 00:00:00','','Turks and Caicos Islands, British Overseas Territory' from dual union all
select 'TCAN','Telesat Canada','business','','CA','Ottawa, Ontario','1969-09-01 00:00:00','2007-10-05 00:00:00','Telesat Canada' from dual union all
select 'TCANL','Telesat Canada (Loral)','business','LORC','CA','Ottawa, Ontario','2007-10-05 00:00:00','','Telesat Canada (Loral)' from dual union all
select 'TCI','Telecommunications Satellite Inc. (Tempo Satellite Inc)','business','','US','Denver:Littleton/Englewood, Colorado','1990-01-01 00:00:00','1998-01-01 00:00:00','Telecommunications Satellite Inc. (Tempo Satellite Inc)' from dual union all
select 'TCSF','ThomsonCSF','business','','F','Paris','1968-01-01 00:00:00','2000-01-01 00:00:00','Thomson-CSF' from dual union all
select 'TECN','Technion','academic/non-profit','','IL','Haifa','1924-01-01 00:00:00','','Technion University' from dual union all
select 'TEIK','Teikyo Scientific University','academic/non-profit','','J','Tokyo:Itabashi','1931-01-01 00:00:00','','帝京大学' from dual union all
select 'TEKEV','TEKEVER','business','','P','Lisboa','','','TEKEVER Group' from dual union all
select 'TELB','Brazilian Telecoms SA','business','','BR','Brasilia','1972-01-01 00:00:00','1998-06-01 00:00:00','Telebras (Telecomunicacoes Brasileiras SA)' from dual union all
select 'TELBR','Teledyne Brown Engineering, Huntsville','business','','US','Huntsville, Alabama','1966-01-01 00:00:00','','Teledyne Brown Engineering, Huntsville' from dual union all
select 'TELD','Teledesic LLC.','business','','US','Seattle:Bellevue, Washington','1990-06-01 00:00:00','2002-01-01 00:00:00','Teledesic LLC.' from dual union all
select 'TELED','Directorate of Telecoms, Informatics and Adv. Tech.','defense','MDDI','I','Roma','','','Dir. Gen.delle Telecom. dell''Informatica e delle Tecnologie Avanzate' from dual union all
select 'TELK','PT Telecommunications of Indonesia','business','','ID','Jakarta','1991-01-01 00:00:00','','PT Telekomunikasi Indonesia Tbk (Telkom)' from dual union all
select 'TELMX','Tele Mexico','business','','MX','Ciudad de Mexico','1989-01-01 00:00:00','','Telecomunicaciones de Mexico' from dual union all
select 'TEMIS','Temis S.r.l.','business','','I','Milano:Corbetta','2010-01-01 00:00:00','','Temis S.r.l.' from dual union all
select 'TERMAB','Terma','business','','DK','Kobenhavn:Rudersdal/Birkerod','1997-01-01 00:00:00','','Terma A/S, Birkerod' from dual union all
select 'TERRD','Terrestar Networks Inc. (Dish Network Corp)','business','DISH','US','Reston, Virginia','2011-07-07 00:00:00','','Terrestar Networks Inc. (Dish Network Corp)' from dual union all
select 'TERRE','Terrestar Corp.','business','','US','Reston, Virginia','2007-01-01 00:00:00','','Terrestar Corp.' from dual union all
select 'TERRN','Terrestar Networks Inc.','business','TERRE','US','Reston, Virginia','2001-01-01 00:00:00','2011-07-07 00:00:00','Terrestar Networks Inc.' from dual union all
select 'TESLA','Tesla, Inc.','business','','US','Fremont, California','2003-01-01 00:00:00','','Tesla, Inc.' from dual union all
select 'TETHA','Tether Applications','business','','US','San Diego:Chula Vista, California','1989-01-01 00:00:00','','Tether Applications' from dual union all
select 'TF','French SAL','','F','TF','Port-aux-francais, Kerguelen','1955-01-01 00:00:00','','Territoire des Terres australes et antarctiques françaises' from dual union all
select 'TF88','USN TF88','defense','USN','US','USS Norton Sound, Atlantic Ocean','1958-04-28 00:00:00','1958-09-06 00:00:00','USN TF88' from dual union all
select 'THAIC','Thaicom Public Company Ltd.','business','','T','Bangkok:Nonthaburi','2008-04-10 00:00:00','','Thaicom Public Company Ltd.' from dual union all
select 'THALA','Thales Alenia Space','business','THALP','I','Torino','2007-04-10 00:00:00','','Thales Alenia Space/Torino (TAS-I)' from dual union all
select 'THALES','Thales Alenia Space','business','THALP','F','Cannes','2007-04-10 00:00:00','','Thales Alenia Space/Cannes (TAS-F)' from dual union all
select 'THALP','Thales Group','business','','F','Paris:Neuilly','2000-01-01 00:00:00','','Thales Group' from dual union all
select 'THALR','Thales Alenia Space','business','THALP','I','Roma','2007-04-10 00:00:00','','Thales Alenia Space/Roma' from dual union all
select 'THK','Thiokol Chemical Co. (THKE or THKX)','business','','US','Elkton, Maryland','1929-01-01 00:00:00','1982-01-01 00:00:00','Thiokol Chemical Co.' from dual union all
select 'THKAIC','Alcoa Industrial Components/Thiokol','business','','US','Salt Lake City, Utah','2000-01-01 00:00:00','2001-01-01 00:00:00','Alcoa Industrial Components/Thiokol' from dual union all
select 'THKC','Thiokol Corp.','business','','US','Ogden, Utah','1989-07-01 00:00:00','1998-01-01 00:00:00','Thiokol Corp.' from dual union all
select 'THKCOR','Cordant Technologies, ThiokolPropulsion','business','','US','Salt Lake CIty, Utah','1998-01-01 00:00:00','2000-01-01 00:00:00','Cordant Technologies, ThiokolPropulsion' from dual union all
select 'THKE','ThiokolElkton','business','THK','US','Elkton, Maryland','1947-01-01 00:00:00','2001-01-01 00:00:00','Thiokol-Elkton' from dual union all
select 'THKU','Thiokol Wasatch Div/Air Force Plant 78','business','THK','US','Brigham City/Promontory, Utah','1957-01-01 00:00:00','2001-01-01 00:00:00','Thiokol Wasatch Div/Air Force Plant 78' from dual union all
select 'THKX','Thiokol/Huntsville','business','THK','US','Huntsville, Alabama','1949-01-01 00:00:00','1996-01-01 00:00:00','Thiokol/Huntsville' from dual union all
select 'THKXM','Thiokol Longhorn Division','business','THK','US','Marshall, Texas','1952-01-01 00:00:00','1971-01-01 00:00:00','Thiokol Longhorn Division' from dual union all
select 'THUR','Thuraya Satellite','business','','UAE','Abu Dhabi','1997-01-01 00:00:00','2018-04-01 00:00:00','Thuraya Satellite' from dual union all
select 'THURY','Thuraya Satellite','business','ALYAH','UAE','Abu Dhabi','2018-04-01 00:00:00','','Thuraya Satellite (Yahsat)' from dual union all
select 'TIANJ','Tianjin Aerospace Long March Rocket Manufacturing Co.Ltd.','business','','CN','Tianjin','2009-01-01 00:00:00','','天津航天长征火箭制造有限公司' from dual union all
select 'TIFR','Tata Inst.','civilian','','IN','Mumbai','1945-06-01 00:00:00','','Tata Inst. for Fundamental Research' from dual union all
select 'TIT','Tokyo Inst of Technology','academic/non-profit','','J','Tokyo:Meguro','1881-01-01 00:00:00','','東京工業大学' from dual union all
select 'TITV','Titov Main Test Center for Space Control','defense','RVSN','RU','Kraznosnamensk, Rossiya','2001-08-14 00:00:00','','153-й главный испытательный космический центр имени Г. С. Титова' from dual union all
select 'TJHS','Thomas Jefferson High School for Science and Technology','academic/non-profit','','US','Alexandria, Virginia','1985-01-01 00:00:00','','Thomas Jefferson High School for Science and Technology' from dual union all
select 'TLS','Paul Sabatier University, Toulouse','academic/non-profit','','F','Toulouse:Rangueil','1961-01-01 00:00:00','','Universite de Toulouse-Paul Sabatier (Rangueil)' from dual union all
select 'TM','Turkmenistan','','','TM','Ashgabat','1991-12-01 00:00:00','','Türkmenistan' from dual union all
select 'TMI','Telesat Mobile Inc.','business','','CA','Ottawa:Gloucester, Ontario','1988-01-01 00:00:00','2002-01-01 00:00:00','Telesat Mobile Inc.' from dual union all
select 'TMSAT','Thai Microsatellite','academic/non-profit','','T','Bangkok','','','Thai Microsatellite Co. Ltd. (Mahanakorn UT)' from dual union all
select 'TMTEL','Ministry of Communications of Turkmenistan','civilian','','TM','Ashgabat','1991-01-01 00:00:00','','Türkmenistanyň Aragatnaşyk ministrligi' from dual union all
select 'TMU','Tokyo Metropolitan University','academic/non-profit','','J','Tokyo:Hachioji','2005-01-01 00:00:00','','首都大学東京' from dual union all
select 'TMUT','Tokyo Metropolitan University','academic/non-profit','','J','Tokyo:Hachioji','1949-01-01 00:00:00','2005-01-01 00:00:00','東京都立大学' from dual union all
select 'TNO','TNO','civilian','','NL','Den Haag','1932-01-01 00:00:00','','Nederlandse Organisatie voor Toegepast Natuurwetenschappelijk Onderzoek' from dual union all
select 'TNOR','Telenor','business','','N','Oslo:Fornebu','1855-01-01 00:00:00','','Telenor' from dual union all
select 'TNSA','Turkmenistan National Space Agency','civilian','','TM','Ashgabat','2011-01-01 00:00:00','','Türkmenistanyň prezidentiň ýanynda Milli kosmos agentligi' from dual union all
select 'TO','Tonga','','','TO','Nuku''alofa','1970-01-01 00:00:00','','Pule''anga Fakatu''i ''o Tonga' from dual union all
select 'TOHO','Tohoku University','academic/non-profit','','J','Sendai, Tohoku','1907-01-01 00:00:00','','東北大学' from dual union all
select 'TOK','Tokyo University','academic/non-profit','','J','Tokyo:Komaba','1949-01-01 00:00:00','','東京大学' from dual union all
select 'TOKAI','Tokai Unviersity','academic/non-profit','','J','Tokyo:Shibuya','1924-01-01 00:00:00','','東海大学' from dual union all
select 'TONG','Tongasat  (Friendly Islands Satellite Co.)','business','','TO','Nuku''alofa','1993-01-01 00:00:00','','Tongasat  (Friendly Islands Satellite Co.)' from dual union all
select 'TORP','Polytechnic University of Turin','academic/non-profit','','I','Torino','1906-01-01 00:00:00','','Politecnico di Torino' from dual union all
select 'TOSH','Toshiba Komukai Works','business','','J','Kawasaki City, Kanagawa','1966-01-01 00:00:00','','東芝 小向工場' from dual union all
select 'TOSHKE','Toshiba Keihin Works','business','','J','Kawasaki City, Kanagawa','1966-01-01 00:00:00','2001-01-01 00:00:00','東芝 京浜工場' from dual union all
select 'TOYA','Toyama Prefectural University','academic/non-profit','','J','Imizu, Toyama','1990-01-01 00:00:00','','富山県立大学' from dual union all
select 'TPS','The Planetary Society','academic/non-profit','','US','LA:Pasadena, California','1980-01-01 00:00:00','','The Planetary Society' from dual union all
select 'TPU','Tomsk Polytechnic Unviersity','academic/non-profit','','RU','Tomsk','1896-01-01 00:00:00','','Томский политехнический университет' from dual union all
select 'TR','Turkey','','','TR','Ankara, Turkiye','1919-05-19 00:00:00','','Türkiye Cumhuriyeti' from dual;
insert into ORGANIZATION
select 'TRACR','Tracor Aerospace','business','','US','Austin, Texas','','','Tracor Aerospace' from dual union all
select 'TROM','University of Tromso','academic/non-profit','','N','Tromso','1972-01-01 00:00:00','','Universitetet i Tromsø' from dual union all
select 'TRORB','TransOrbital, Inc','business','','US','San Diego:La Jolla,California','1999-01-01 00:00:00','','TransOrbital, Inc' from dual union all
select 'TRW','TRW Inc.','business','','US','Redondo Beach, California','1958-01-01 00:00:00','2002-07-01 00:00:00','TRW Inc.' from dual union all
select 'TRWAA','TRW Astro Aerospace, Carpenteria','business','TRW','US','Carpenteria, California','1999-01-01 00:00:00','2002-01-01 00:00:00','TRW Astro Aerospace, Carpenteria' from dual union all
select 'TRWC','TRW Inc. Chantilly','business','','US','Chantilly, Virginia','','','TRW Inc. Chantilly' from dual union all
select 'TRWESL','TRW ESL Inc.','business','TRW','US','Sunnyvale, California','1978-01-01 00:00:00','2001-01-01 00:00:00','Electronic Systems Lab. Inc.' from dual union all
select 'TRWRC','TRW Radio Club, Redondo Beach','academic/non-profit','','US','Los Angeles:Redondo Beach, California','1965-01-01 00:00:00','1965-01-01 00:00:00','TRW Radio Club, Redondo Beach' from dual union all
select 'TSAO','Central Aerological Observatory','academic/non-profit','GMS','RU','Moskva:Dolgoprudny','1941-01-01 00:00:00','','Центральной аэрологической обсерваторией Гидрометеослужбы СССР' from dual union all
select 'TSBM','Central Bank, Russian Federation','business','','RU','Moskva','1990-07-13 00:00:00','','Центральный банк Российской Федерации' from dual union all
select 'TSCJ','Telecommunications Satellite Corp of Japan','business','','J','Kimitsu?','1979-01-01 00:00:00','1992-10-01 00:00:00','Telecommunications Satellite Corp of Japan' from dual union all
select 'TSCO','The Spaceship Co. (Virgin Galactic/NGASScaled)','business','','US','Mojave, California','2005-01-01 00:00:00','','The Spaceship Co. (Virgin Galactic/NGAS-Scaled)' from dual union all
select 'TSHUA','Tsinghua University','academic/non-profit','','CN','Beijing:Haidian','1911-01-01 00:00:00','','清华大学' from dual union all
select 'TSKB7','Arsenal Design Bureau','business','','SU','Sankt:Peterburg (Leningrad)','1949-11-21 00:00:00','1965-01-01 00:00:00','онструкторское бюро «Арсенал» (ЦКБ-7)' from dual union all
select 'TSKBE3','TsKBEM Filial No. 3','business','TSKBEM','SU','Samara','1967-01-01 00:00:00','1974-01-01 00:00:00','филиал № 3 ЦКБЭМ' from dual union all
select 'TSKBEM','Central KB for Experimental Machine Building','defense','','SU','Korolev:Podlipki','1966-03-06 00:00:00','1974-01-01 00:00:00','Центральное Конструкторское Бюро Экспериментального Машиностроения (ЦКБЭМ)' from dual union all
select 'TSKBM','Machine Building Special Design Bureau','defense','','SU','Moskva:Reutov','1966-03-01 00:00:00','1983-01-01 00:00:00','ЦКБ машиностроения (ЦКБМ)' from dual union all
select 'TSKY','Telesat','business','TCAN','CA','Ottawa','2007-10-01 00:00:00','','Telesat (former Loral Skynet), Ottawa' from dual union all
select 'TSKYB','Telesat','business','TCAN','US','Bedminster, New Jersey','2007-10-01 00:00:00','','Telesat (former Loral Skynet)' from dual union all
select 'TSKYBR','Telesat Brasil Capacidade de Satelites Ltda.','business','TCAN','BR','Rio de Janeiro','2007-10-31 00:00:00','','Telesat Brasil Capacidade de Satelites Ltda.' from dual union all
select 'TSMASH','TsNIIMash (Tsenral''niy NII Mashinostroeniya)','civilian','','SU','Moskva:Podlipki','1967-01-01 00:00:00','1992-01-01 00:00:00','Центральный научно-исследовательский институт машиностроения (ЦНИИмаш)' from dual union all
select 'TSMASHF','TsNIIMash (Tsenral''niy NII Mashinostroeniya)','civilian','FKA','RU','Moskva:Podlipki','1992-01-01 00:00:00','','Центральный научно-исследовательский институт машиностроения (ЦНИИмаш)' from dual union all
select 'TSN108','TsNII108 GKRE (Zaslavskiy)','defense','','SU','Moskva','1946-01-01 00:00:00','1991-01-01 00:00:00','ЦНИИ-108 Министерства обороны СССР' from dual union all
select 'TSNIRTI','TsNIRTI (Zaslavskiy)','defense','','SU','Moskva','1991-01-01 00:00:00','','ЦНИИ РТИ' from dual union all
select 'TSPZ','Telespazio','business','','I','Roma','1965-01-01 00:00:00','','Telespazio SpA' from dual union all
select 'TSROK','Rokot Center, Golitsyno2','defense','RVSN','SU','Moskva:Golitsyno','1972-06-01 00:00:00','2001-08-14 00:00:00','Главный испытательный центр испытаний и управления космическими средствами' from dual union all
select 'TSSKB','Central Specialized Design Bureau','business','','SU','Samara (Kubyshev), Samara Oblast','1974-01-01 00:00:00','1996-04-12 00:00:00','Центрального специализированного конструкторского бюро (ЦСКБ)' from dual union all
select 'TSSKBP','Central Specialized Design Bureau  Progress','business','FKA','SU','Samara (Kubyshev), Samara Oblast','1996-04-12 00:00:00','','ЦСКБ-Прогресс' from dual union all
select 'TSUK','Tsukuba Space Center','academic/non-profit','','J','Tsukuba, Ibaraki','1972-01-01 00:00:00','','筑波宇宙センタ' from dual union all
select 'TSUKOS','Central Space Forces Directorate','defense','MO','SU','Moskva?','1964-10-01 00:00:00','1970-03-01 00:00:00','Центральное управление космических средств' from dual union all
select 'TSUKU','Tsukuba University','academic/non-profit','','J','Tsukuba, Ibaraki','1872-01-01 00:00:00','','筑波大学' from dual union all
select 'TSVS','Topographics Service of the Russian Armed Froces','defense','','RU','Moskva','2010-01-01 00:00:00','','Топографическая служба Вооружённых Сил Российской Федерации' from dual union all
select 'TTPI','TTPI','','','TTPI','Saipan','1947-07-18 00:00:00','1979-01-01 00:00:00','Trust Territory of the Pacific Islands' from dual union all
select 'TTY','Tampere University of Technology','academic/non-profit','','FI','Tampere:Hervanta, Pirkanmaa, Lansi:Suomen','1965-01-01 00:00:00','','Tampereen teknillinen yliopisto' from dual union all
select 'TUAF','Turkish Air Force','defense','','TR','Izmir','','','Türk Hava Kuvvetleri,  Çiğli Hava Üssü' from dual union all
select 'TUAFA','Turkish Air Force Academy','defense','TUAF','TR','Istanbul:Yesilyurt','1951-01-01 00:00:00','','Hava Harp Okulu' from dual union all
select 'TUB','Technical University of Berlin','academic/non-profit','TUB1','D','Berlin','1989-01-01 00:00:00','2014-01-01 00:00:00','TUBSAT group, Technische Universität Berlin' from dual union all
select 'TUB1','Technical University of Berlin','academic/non-profit','','D','Berlin','1946-04-09 00:00:00','','Technische Universität Berlin' from dual union all
select 'TUBAFA','AstroundFeinwerktechnik Adlershof (TUB spinoff)','business','','D','Berlin','2009-01-01 00:00:00','','Astro-und-Feinwerktechnik Adlershof (TUB spinoff)' from dual union all
select 'TUBI','University of Tubingen','academic/non-profit','','D','Tubingen, Baden:Wuttemberg','1477-01-01 00:00:00','','Eberhard Karls Universität Tübingen' from dual union all
select 'TUBT','Scientific and Technological Research Council (Turkey)','academic/non-profit','','TR','Ankara','1963-07-01 00:00:00','','Türkiye Bilimsel ve Teknolojik Araştırma Kurumu (TÜBİTAK)' from dual union all
select 'TUBTK','STRC Information Technologies and Electronics Institute','academic/non-profit','TUBT','TR','Ankara','1985-01-01 00:00:00','2006-05-01 00:00:00','TÜBİTAK - BİLTEN Bilgi Teknolojileri ve Elektronik Araştırma Enstitüsü' from dual union all
select 'TUBTU','STRC Space Technologies Research Institute','academic/non-profit','TUBT','TR','Ankara','2006-05-01 00:00:00','','TÜBİTAK Uzay Teknolojileri Araştırma Enstitüsü' from dual union all
select 'TUC','National Tucuman University','academic/non-profit','','AR','San Miguel de Tucuman, Tucuman','1914-01-01 00:00:00','','Universidad Nacional de Tucumán' from dual union all
select 'TUD','Dresden University of Technology','academic/non-profit','','D','Dresden, Sachsen','1828-01-01 00:00:00','','Technische Universität Dresden' from dual union all
select 'TUG','Graz University of Technology','academic/non-profit','','AT','Graz, Steiermark','1811-01-01 00:00:00','','Technische Universität Graz' from dual union all
select 'TUI','Tethers Unlimited Inc (TUI)','business','','US','Seattle:Bothell, Washington','1994-01-01 00:00:00','','Tethers Unlimited Inc (TUI)' from dual union all
select 'TUM','Technical University of Munich','academic/non-profit','','D','Munchen','1868-01-01 00:00:00','','Technische Universität München' from dual union all
select 'TUMED','SK Telecom TU Media','business','SKTEL','KR','Seoul','2004-01-01 00:00:00','2010-01-01 00:00:00','SK Telecom TU Media' from dual union all
select 'TURAB','University of Turabo','business','','PR','Gurabo, Puerto Rico','1972-01-01 00:00:00','','Universidad del Turabo' from dual union all
select 'TURKP','Turkish PTT','business','','TR','Ankara','1989-01-01 00:00:00','1995-01-01 00:00:00','Posta Telgraf Telefon' from dual union all
select 'TURKS','Turksat Satellite Comm. and Cable TV Operation AS','business','','TR','Ankara','2004-07-01 00:00:00','','Türksat Uydu Haberleşme Kablo TV ve İşletme A.Ş.' from dual union all
select 'TURKT','Turk Telecom','business','','TR','Ankara','1995-01-01 00:00:00','2004-07-01 00:00:00','Türk Telekomünikasyon A.Ş.' from dual union all
select 'TUSAS','TAI','business','','TR','Ankara','1984-01-01 00:00:00','','Türk Havacılık ve Uzay Sanayi A.Ş (TUSAŞ)' from dual union all
select 'TUUT','Taiwan Universities United Team , c/o China Inst. of Technology','academic/non-profit','','TW','Taipei','1998-01-01 00:00:00','','Taiwan Universities United Team , c/o China Inst. of Technology' from dual union all
select 'TW','Taiwan','','','TW','Taipeo, Taiwan','1949-12-07 00:00:00','','中華民國' from dual union all
select 'TXWC','Schellinger Labs, Texas Western College, El Paso','academic/non-profit','','US','El Paso, Texas','1914-01-01 00:00:00','1967-01-01 00:00:00','Schellinger Labs, Texas Western College, El Paso' from dual union all
select 'TYVAK','Tyvak Nanosatellite Systems LLC','business','','US','San Luis Obispo, California','2011-01-01 00:00:00','','Tyvak Nanosatellite Systems LLC' from dual union all
select 'UA','Ukraine','','','UA','Kiev, Ukraina','1991-12-26 00:00:00','','Україна' from dual union all
select 'UAC','United Aircraft Corp.','business','','US','Hartford, Connecticut','1934-01-01 00:00:00','1975-05-01 00:00:00','United Aircraft Corp.' from dual union all
select 'UAE','UAE','','','UAE','Abu Dhabi','1971-12-02 00:00:00','','الإمارات العربية المتحدة' from dual union all
select 'UAF','U. Alaska Fairbanks','academic/non-profit','','US','Fairbanks, Alaska','1917-01-01 00:00:00','','U. Alaska Fairbanks' from dual union all
select 'UAH','University of AlabamaHuntsville','academic/non-profit','','US','Huntsville, Alabama','1969-01-01 00:00:00','','University of Alabama-Huntsville' from dual union all
select 'UAHCM','UAH Consortium for Materials Development in Space','academic/non-profit','UAH','US','Huntsville, Alabama','1985-01-01 00:00:00','','UAH Consortium for Materials Development in Space' from dual union all
select 'UAP','Peruvian Aviation University','academic/non-profit','','PE','Lima','1996-01-01 00:00:00','','Universidad Alas Peruanas' from dual union all
select 'UAZ','University of ArizonaTucson','academic/non-profit','','US','Tucson, Arizona','1885-01-01 00:00:00','','University of Arizona-Tucson' from dual union all
select 'UBC','University of British Columbia','academic/non-profit','','CA','Vancouver:Point Grey, British Columbia','1915-01-01 00:00:00','','University of British Columbia' from dual union all
select 'UCAN','University of Cantebury','academic/non-profit','','NZ','Christchurgh:Ilam, New Zealand','1873-01-01 00:00:00','','University of Cantebury' from dual union all
select 'UCB','University of California Berkeley','academic/non-profit','','US','Berkeley, California','1868-03-23 00:00:00','','University of California Berkeley' from dual union all
select 'UCD','University of California Davis','academic/non-profit','','US','Davis, California','','','University of California Davis' from dual union all
select 'UCH','Space and Planetary Exploration Lab, University of Chile','academic/non-profit','','CL','Santiago, Chile','','','University of Chile' from dual union all
select 'UCL','University College, London','academic/non-profit','','UK','London:GowerSt','1826-01-01 00:00:00','','University College, London' from dual union all
select 'UCO','University of Colorado','academic/non-profit','','US','Boulder, Colorado','1876-01-01 00:00:00','','University of Colorado' from dual union all
select 'UCSD','University of California San Diego','academic/non-profit','','US','San Diego:La Jolla, California','1960-01-01 00:00:00','','University of California San Diego' from dual union all
select 'UCW','University College of Wales','academic/non-profit','','UK','Aberystwyth, Wales','1872-01-01 00:00:00','1996-01-01 00:00:00','University College of Wales' from dual union all
select 'UDAYT','University of Dayton','academic/non-profit','','US','Dayton, Ohio','1850-01-01 00:00:00','','University of Dayton' from dual union all
select 'UFA','Institute of Atmospheric Physics','academic/non-profit','ANCZ','CZ','Praha','1964-01-01 00:00:00','','Ústav fyziky atmosféry AV ČR' from dual union all
select 'UFL','University of Florida','academic/non-profit','','US','Gainesville, Florida','1853-01-01 00:00:00','','University of Florida' from dual union all
select 'UFSM','Federal University of Santa Maria','academic/non-profit','','BR','Santa Maria, Rio Grande do Sul','1960-01-01 00:00:00','','Universidade Federal de Santa Maria' from dual union all
select 'UGATU','USATU','academic/non-profit','','RU','Ufa, Bashkortostan (RU:BA)','1932-01-01 00:00:00','','Уфимский государственный авиационный технический университет' from dual union all
select 'UHI','University of Hawaii, Manoa','academic/non-profit','','US','Honolulu:Manoa, Oahu','1907-01-01 00:00:00','','University of Hawaii, Manoa' from dual union all
select 'UIA','University of Iowa (=SUI)','academic/non-profit','','US','Iowa CIty, Iowa','1964-01-01 00:00:00','','University of Iowa (=SUI)' from dual union all
select 'UIO','Uppsala Ionospheric Observatory','academic/non-profit','','N','Uppsala','1952-01-01 00:00:00','1986-01-01 00:00:00','Uppsala Ionospheric Observatory' from dual union all
select 'UIUC','University of Illinois UrbanaChampaign','academic/non-profit','','US','Urbana/Champaign, Illinois','1867-01-01 00:00:00','','University of Illinois Urbana-Champaign' from dual union all
select 'UK','UK','','','UK','London','1707-05-01 00:00:00','','United Kingdom of Great Britain and Northern Ireland' from dual union all
select 'UKMO','UK Meteorological Office, Air Ministry','civilian','','UK','Bracknell','1962-01-01 00:00:00','2003-01-01 00:00:00','UK Meteorological Office, Air Ministry' from dual union all
select 'UKNR','Navy Royal','defense','','UK','London, Admiralty','1540-01-01 00:00:00','1660-01-01 00:00:00','Navy Royal' from dual union all
select 'UKRN','Royal Navy','defense','','UK','London, Admiralty','1660-01-01 00:00:00','','Royal Navy' from dual union all
select 'UKS','University of Kansas','academic/non-profit','','US','Lawrence, Kansas','1865-01-01 00:00:00','','University of Kansas' from dual union all
select 'UKSA','UK Space Agency','civilian','','UK','Swindon','2010-04-01 00:00:00','','UK Space Agency' from dual union all
select 'UKSPOC','UK Space Operations Centre, Royal Air Force High Wycombe','defense','','UK','High Wycombe','2016-01-01 00:00:00','','UK Space Operations Centre, Royal Air Force High Wycombe' from dual union all
select 'UKTSB','Technology Strategy Board, Dept. for Business, Innovation and Skills','civilian','','UK','Swindon','2007-07-01 00:00:00','','Technology Strategy Board, Dept. for Business, Innovation and Skills' from dual union all
select 'UKWO','UK War Office','defense','','UK','London:Whitehall','1684-01-01 00:00:00','1964-01-01 00:00:00','UK War Office' from dual union all
select 'UKY','University of Kentucky','academic/non-profit','','US','Lexington, Kentucky','1865-01-01 00:00:00','','University of Kentucky' from dual union all
select 'ULAB','United Launch Alliance/Boeing','business','BOE','US','Denver:Centennial, Colorado','2006-12-01 00:00:00','','United Launch Alliance/Boeing' from dual union all
select 'ULAL','United Launch Alliance/Lockheed Martin Astronautics','business','LM','US','Denver:Centennial, Colorado','2006-12-01 00:00:00','','United Launch Alliance/Lockheed Martin Astronautics' from dual union all
select 'ULLAF','University of LouisianaLafayette','academic/non-profit','','US','Lafayette, Louisiana','1999-01-01 00:00:00','','University of Louisiana-Lafayette' from dual union all
select 'ULM','University of Ulm','academic/non-profit','','D','Ulm, Baden:Wurttemburg','1967-01-01 00:00:00','','Universitat Ulm' from dual union all
select 'UM67','Johnston I.','','US','UM67','Johnston Island','1945-01-01 00:00:00','','Johnston Atoll, US Minor Outlying Islands' from dual union all
select 'UM79','Wake I.','','US','UM79','Wake Island','1945-01-01 00:00:00','','Wake Island, US Minor Outlying Islands' from dual;
insert into ORGANIZATION
select 'UMAL','University of Massachusetts Lowell','academic/non-profit','','US','Lowell, Massachusetts','1975-01-01 00:00:00','','University of Massachusetts Lowell' from dual union all
select 'UMBC','University of Maryland, Baltimore County','academic/non-profit','','US','Baltimore:Catonsville, Maryland','1966-01-01 00:00:00','','University of Maryland, Baltimore County' from dual union all
select 'UMD','University of Maryland','academic/non-profit','','US','College Park, Maryland','1856-01-01 00:00:00','','University of Maryland' from dual union all
select 'UMI','University of Michigan','academic/non-profit','','US','Ann Arbor, Michigan','1837-01-01 00:00:00','','University of Michigan' from dual union all
select 'UMIAMI','University of Miami','academic/non-profit','','US','Coral Gables, Florida','1925-01-01 00:00:00','','University of Miami' from dual union all
select 'UMN','University of Minnesota','academic/non-profit','','US','Minneapolis:St. Paul, Minnesota','1861-01-01 00:00:00','','University of Minnesota' from dual union all
select 'UNAM','National Autonomous University of Mexico','academic/non-profit','','MX','Ciudad de Mexico','1910-01-01 00:00:00','','Universidad Nacional Autonoma de Mexico (UNAM)' from dual union all
select 'UNB','University of Brasilia','academic/non-profit','','BR','Brasilia','1962-01-01 00:00:00','','Universidade de Brasilia' from dual union all
select 'UNCW','University of North Carolina, Wilmington','academic/non-profit','','US','Wilmington, North Carolina','1947-01-01 00:00:00','','University of North Carolina, Wilmington' from dual union all
select 'UNH','University of New Hampshire','academic/non-profit','','US','Durham, New Hampshire','1866-01-01 00:00:00','','University of New Hampshire' from dual union all
select 'UNIPE','National University of Engineering','academic/non-profit','','PE','Lima:Rimac','1876-01-01 00:00:00','','Universidad Nacional de Ingenieria' from dual union all
select 'UNIS','University Space Engineering Consortium','academic/non-profit','','J','Tokyo:Yayoi','2008-01-01 00:00:00','','University Space Engineering Consortium' from dual union all
select 'UNITS','Trieste University','academic/non-profit','','I','Trieste, Friuli:Venezia Giulia','1924-01-01 00:00:00','','Universita degli Studi di Trieste' from dual union all
select 'UNK','UNK','','','X',':','','','-            -' from dual union all
select 'UNKS','Upravleniye Nachalnika Kosmicheskikh Sredstv','defense','MO','SU','Moskva','1986-04-24 00:00:00','1991-01-01 00:00:00','Управление начальника космических средств МО СССР' from dual union all
select 'UNKSR','Upravleniye Nachalnika Kosmicheskikh Sredstv RF','defense','RVSNR','RU','Moskva','1991-01-01 00:00:00','1992-01-01 00:00:00','Управление начальника космических средств' from dual union all
select 'UNKUS','Unknown US manufacturer (classified satellite)','business','','US','::','','','Unknown US manufacturer (classified satellite)' from dual union all
select 'UNKUS1','Unknown US commercial operator','business','','US','::','2017-01-01 00:00:00','','Unknown US commercial operator' from dual union all
select 'UNMC','COSMIAC, University of New MexicoAlbequerque','academic/non-profit','','US','Albequerque, New Mexico','','','COSMIAC, University of New Mexico-Albequerque' from dual union all
select 'UNRV','Directorate of Chief of RocketPropelled Weapons','defense','GAU','SU','Moskva','1955-03-01 00:00:00','1959-01-01 00:00:00','Управление начальника реактивного вооружения' from dual union all
select 'UNSJ','National University of San Juan','academic/non-profit','','AR','Mendoza:San Juan','1973-01-01 00:00:00','','Universidad Nacional de San Juan' from dual union all
select 'UNSW','University of New South Wales Center for Space Eng. Research','academic/non-profit','','AU','Sydney','2011-01-01 00:00:00','','University of New South Wales' from dual union all
select 'UNSWC','University of New South Wales/Canberra','academic/non-profit','','AU','Canberra','2011-01-01 00:00:00','','University of New South Wales/Canberra' from dual union all
select 'UOS','University of Surrey Dept. of Electronic and Electrical Eng.','academic/non-profit','','UK','Guildford, Surrey','1968-01-01 00:00:00','1985-01-01 00:00:00','University of Surrey Dept. of Electronic and Electrical Eng.' from dual union all
select 'UPAE','UP Aerospace','business','','US','Denver:Highlands Ranch, Colorado','2005-01-01 00:00:00','','UP Aerospace' from dual union all
select 'UPC','Polytechnic U. of Catalonia','academic/non-profit','','E','Barcelona, prov. Barcelona, CA Catalonia','1971-01-01 00:00:00','','Universita Politecnica de Catalunya' from dual union all
select 'UPD','U. of the Phillipines Diliman','academic/non-profit','','PH','Quezon City/Diliman','1949-01-01 00:00:00','','Pamantasan ng Pilipinas Diliman' from dual union all
select 'UPM','Polytechnic U. of Madrid','academic/non-profit','','E','Madrid','1971-01-01 00:00:00','','Universita Politecnica de Madrid' from dual union all
select 'UPMIM','UPM Microgravity Institute','academic/non-profit','UPM','E','Madrid','1971-01-01 00:00:00','','Instituto Universitario de Microgravidad "Ingacio Da Riva", UPM' from dual union all
select 'UQU','University of Queensland, Centre for Hypersonics','academic/non-profit','','AU','Brisbane:St Lucia, Queensland','1997-01-01 00:00:00','','University ofQueensland, Centre for Hypersonics' from dual union all
select 'URDEI','UrtheCast Deimos Imaging','business','URTHE','E','Boecillo, Valladolid','2015-08-01 00:00:00','','UrtheCast Deimos Imaging' from dual union all
select 'URTHE','UrtheCast','business','','CA','Vancouver, British Columbia','2013-01-01 00:00:00','','UrtheCast' from dual union all
select 'URUGUS','URUGUS S.A., Satellogic S.A., Montevideo satellite factory','business','SATELOG','UY','Montevideo','2015-01-01 00:00:00','','URUGUS S.A.' from dual union all
select 'US','USA','','','US','Washington, DC','1776-07-04 00:00:00','','United States of America' from dual union all
select 'US7A','US 7th Army (Germany), Pershing battalions','defense','','US','Schwabisch Gmuend, Baden:Wurttemberg, Germany','','','US 7th Army (Germany), Pershing battalions' from dual union all
select 'US8A','US 8th Army (Korea), ATACMS group','defense','','US','Seoul:Yongsan','','','US 8th Army (Korea)' from dual union all
select 'USA','United States Army','defense','','US','Washington, DC','1775-06-14 00:00:00','','United States Army' from dual union all
select 'USAF','United States Air Force','defense','','US','Washington, DC:Pentagon','1947-09-18 00:00:00','','United States Air Force' from dual union all
select 'USAFA','United States Air Force Academy','defense','USAF','US','Colorado Springs, Colorado','1954-01-01 00:00:00','','United States Air Force Academy' from dual union all
select 'USAMC','United States Army Materiel Command','defense','USA','US','Washington, DC','1962-01-01 00:00:00','2009-01-01 00:00:00','United States Army Materiel Command' from dual union all
select 'USASC','US Army Signal Corps, Ft Monmouth','defense','USA','US','Ft Monmouth, New Jersey','1917-01-01 00:00:00','','US Army Signal Corps, Ft Monmouth' from dual union all
select 'USCG','US Coast Guard Research and Development Center','defense','','US','New London, Connecticut','','','US Coast Guard Research and Development Center' from dual union all
select 'USCLA','University of Southern California','academic/non-profit','','US','Los Angeles:University Park, California','1880-01-01 00:00:00','','University of Southern California' from dual union all
select 'USCMR','ISISERC, USC Marina Del Rey Campus','academic/non-profit','USCLA','US','Los Angeles:Marina Del Rey, California','1880-01-01 00:00:00','','ISI-SERC, USC Marina Del Rey Campus' from dual union all
select 'USDI','US Dept of the Interior, Bureau of Land Management','civilian','','US','Washington DC','','','US Dept of the Interior, Bureau of Land Management' from dual union all
select 'USDMA','Defense Mapping Agency, Topographic Center (DMATC)','defense','DOD','US','Bethesda, Maryland','1972-07-01 00:00:00','1996-10-01 00:00:00','Defense Mapping Agency, Topographic Center (DMATC)' from dual union all
select 'USEF','Institute for Unmanned Space Experiment Free Flyer','business','','J','Tokyo:Chiyoda','1986-01-01 00:00:00','2012-03-01 00:00:00','財団法人無人宇宙実験システム研究開発機構' from dual union all
select 'USERAR','Sergio Arboleda University','academic/non-profit','','CO','Bogota','1984-01-01 00:00:00','','Universidad Sergio Arboleda' from dual union all
select 'USGS','US Geological Survey EROS Data Center','civilian','USGSR','US','Sioux Falls, South Dakota','1973-01-01 00:00:00','','US Geological Survey EROS Data Center' from dual union all
select 'USGSR','US Geological Survey (Reston)','civilian','','US','Reston, Virginia','1973-01-01 00:00:00','','US Geological Survey (Reston)' from dual union all
select 'USMA','US Military Academy, West Point','defense','USA','US','West Point, New York','','','US Military Academy, West Point' from dual union all
select 'USN','United States Navy','defense','','US','Washington, DC:Pentagon','1775-01-01 00:00:00','','United States Navy' from dual union all
select 'USNA','US Naval Academy','defense','USN','US','Annapolis, Maryland','1845-01-01 00:00:00','','US Naval Academy' from dual union all
select 'USNESC','Navy Electronic Systems Command','defense','USN','US','Washington, DC','1966-05-01 00:00:00','1987-01-01 00:00:00','Navy Electronic Systems Command' from dual union all
select 'USNPS','US Naval Postgraduate School','defense','USN','US','Monterey, California','1909-01-01 00:00:00','','US Naval Postgraduate School' from dual union all
select 'USNSP','Navy Space Project Office, NAVMAT PM16','defense','NAVMAT','US','Washington, DC','1971-01-14 00:00:00','1973-06-01 00:00:00','Navy Space Project Office, NAVMAT PM-16' from dual union all
select 'USNSP2','Navy Space Project Office, NESC PME106  (NAVELEXSYSCOM)','defense','USNESC','US','Washington, DC','1973-06-01 00:00:00','','Navy Space Project Office, NESC PME-106  (NAVELEXSYSCOM)' from dual union all
select 'USNSPO','USN Special Projects Office (FBM)','defense','USN','US','Washington, DC:Pentagon?','1955-01-01 00:00:00','1968-01-01 00:00:00','USN Special Projects Office (FBM)' from dual union all
select 'USNSSD','USN Space Systems Division, OP943','defense','USN','US','Washington, DC','1981-01-01 00:00:00','','USN Space Systems Division, OP-943' from dual union all
select 'USNSSP','USN Strategic Systems Programs PMO (NAVPMOSSP)','defense','USN','US','Sunnyvale, California','1987-01-01 00:00:00','','USN Strategic Systems Programs PMO (NAVPMOSSP)' from dual union all
select 'USNTB','USN TBMD office, OCNO','defense','USN','US','Washington, DC:Pentagon','','','USN TBMD office, OCNO' from dual union all
select 'USPS','US Postal Service, Philatelic Division','civilian','','US','Washington, DC','1971-07-01 00:00:00','','US Postal Service, Philatelic Division' from dual union all
select 'USTC','University of Science and Technology of China','academic/non-profit','CAS','CN','Hefei, Anhui','1970-01-01 00:00:00','','中国科学技术大学' from dual union all
select 'USU','Utah State University, Space Dynamics Lab, (SDL)','academic/non-profit','','US','Logan, Utah','1959-01-01 00:00:00','','Utah State University, Space Dynamics Lab, (SDL)' from dual union all
select 'USWL','University of Southwest Louisiana','academic/non-profit','','US','Lafayette, Louisiana','1960-01-01 00:00:00','1999-01-01 00:00:00','University of Southwest Louisiana' from dual union all
select 'USYD','University of Sydney','academic/non-profit','','AU','Sydney, New South Wales','1850-01-01 00:00:00','','University of Sydney' from dual union all
select 'UT','University of Texas  Austin','academic/non-profit','','US','Austin, Texas','1883-01-01 00:00:00','','University of Texas - Austin' from dual union all
select 'UTC','United Technology Corp','business','UAC','US','San Jose, California','1959-01-01 00:00:00','1975-05-01 00:00:00','United Technology Corp' from dual union all
select 'UTC2','United Technologies Corp.','business','','US','Hartford, Connecticut','1975-05-01 00:00:00','','United Technologies Corp.' from dual union all
select 'UTCISR','ISR Systems, UTC Aerospace Systems, Danbury','business','UTC2','US','Danbury, Connecticut','2012-07-01 00:00:00','','ISR Systems, UTC Aerospace Systems, Danbury' from dual union all
select 'UTCWL','UTC Aerospace Systems (Windsor Locks)','business','UTC2','US','Windsor Locks, Connecticut','2012-07-01 00:00:00','','UTC Aerospace Systems (Windsor Locks)' from dual union all
select 'UTD','University of TexasDallas','academic/non-profit','','US','Dallas, Texas','1969-01-01 00:00:00','','University of Texas-Dallas' from dual union all
select 'UTE','Equinoctial University of Ecuador','academic/non-profit','','EC','Quito','1971-01-01 00:00:00','','Universidad Tecnológica Equinoccial' from dual union all
select 'UTEP','University of Texas El Paso','academic/non-profit','','US','El Paso, Texas','1967-01-01 00:00:00','','University of Texas -El Paso' from dual union all
select 'UTIAS','UTIAS Space Flight Laboratory','academic/non-profit','UTIAS0','CA','Toronto:York U. Heights','1998-01-01 00:00:00','','UTIAS Space Flight Laboratory' from dual union all
select 'UTIAS0','University of Toronto Inst. of Aerospace Studies','academic/non-profit','','CA','Toronto:York U. Heights','1963-01-01 00:00:00','','University of Toronto Inst. of Aerospace Studies' from dual union all
select 'UTMB','University of Texas Medical Branch','academic/non-profit','','US','Galveston, Texas','','','University of Texas Medical Branch' from dual union all
select 'UTOR','University of Toronto Inst. of Aerophysics','academic/non-profit','','CA','Toronto:York U. Heights','1949-01-01 00:00:00','1963-01-01 00:00:00','University of Toronto Inst. of Aerophysics' from dual union all
select 'UTR','Universiteit Utrecht','academic/non-profit','','NL','Utrecht:De Uithof','1636-01-01 00:00:00','1974-01-01 00:00:00','Universiteit Utrecht' from dual union all
select 'UTRU','Universiteit Utrecht, Dept. Natuuren Sterrenkunde','academic/non-profit','','NL','Utrecht:De Uithof','1974-01-01 00:00:00','','Universiteit Utrecht, Dept. Natuur-en Sterrenkunde' from dual union all
select 'UUT','University of Utah','academic/non-profit','','US','Salt Lake City','1850-01-01 00:00:00','','University of Utah' from dual union all
select 'UVA','University of Virgina','academic/non-profit','','US','Charlottesville, Virginia','1819-01-01 00:00:00','','University of Virgina' from dual union all
select 'UVIE','University of Vienna Inst. for Astronomy','academic/non-profit','','AT','Wien','','','Institut für Astrophysik Universität Wien' from dual union all
select 'UVT','University of Vermont','academic/non-profit','','US','Burlington, Vermont','1791-01-01 00:00:00','','University of Vermont' from dual union all
select 'UWA','University of Washington','academic/non-profit','','US','Seattle, Washington','1861-01-01 00:00:00','','University of Washington' from dual union all
select 'UWAB','University of Wales, Aberystwyth','academic/non-profit','','UK','Aberystwyth, Wales','1996-01-01 00:00:00','2007-09-01 00:00:00','University of Wales, Aberystwyth' from dual union all
select 'UWI','University of Wisconsin','academic/non-profit','','US','Madison, WIsconsin','1848-01-01 00:00:00','','University of Wisconsin' from dual union all
select 'UWO','University of Western Ontario','academic/non-profit','','CA','London, Ontario','1878-01-01 00:00:00','','University of Western Ontario' from dual union all
select 'UY','Uruguay','','','UY','Montevideo','1825-01-01 00:00:00','','República Oriental del Uruguay' from dual union all
select 'UZKA','Directorate of the Deputy Cdr. of Artillery for Special Eqpt','defense','GAU','SU','Moskva','1953-04-01 00:00:00','1955-03-01 00:00:00','Управления заместителя командующего артиллерией (УЗКА)' from dual union all
select 'VANDU','Vanderbilt University','academic/non-profit','','US','Nashville, Tennessee','1873-01-01 00:00:00','','Vanderbilt University' from dual union all
select 'VANST','Vanguard Space Technologies','business','','US','San Diego, California','1995-01-01 00:00:00','','Vanguard Space Technologies' from dual union all
select 'VAP','Virginia Tech (VA Tech)','academic/non-profit','','US','Blacksburg, Virginia','1872-01-01 00:00:00','','Virginia Polytechnic Institute and State University' from dual union all
select 'VAST','Vietnam Academy of Science and Technology','civilian','','VN','Ho Chi Minh CIty','1975-01-01 00:00:00','','Viện Hàn lâm Khoa học và Công nghệ Việt Nam' from dual union all
select 'VATP','VHTP','business','','LV','Ventspils','2005-01-01 00:00:00','','Ventspils Augsto Tehnologiju Parks' from dual union all
select 'VE','Venezuela','','','VE','Caracas, Venezuela','1830-01-13 00:00:00','','República Bolivariana de Venezuela' from dual union all
select 'VENT','Ventions LLC','business','','US','San Francisco','2004-01-01 00:00:00','2016-01-01 00:00:00','Ventions LLC' from dual union all
select 'VER','Verheart Design and Development NV (VDD)','business','','B','Kruibeke','2005-01-01 00:00:00','2010-01-01 00:00:00','Verheart Design and Development NV (VDD)' from dual union all
select 'VERSI','Verhaert/SI','business','','B','Kruibeke','1984-01-01 00:00:00','2005-01-01 00:00:00','Verhaert/SI' from dual union all
select 'VFW','Vereinigte Flugtechnische Werke (VFW)','business','','D','Bremen','1964-01-01 00:00:00','1969-01-01 00:00:00','Vereinigte Flugtechnische Werke (VFW)' from dual;
insert into ORGANIZATION
select 'VFW2','Vereinigte Flugtechnische Werke (VFW)','business','','D','Bremen','1980-01-01 00:00:00','1981-01-01 00:00:00','Vereinigte Flugtechnische Werke (VFW)' from dual union all
select 'VFWF','Zentralgesellschaft VFWFokker GmbH','business','','D','Bremen','1969-01-01 00:00:00','1980-01-01 00:00:00','Zentralgesellschaft VFW-Fokker GmbH' from dual union all
select 'VGLB','VGX','business','','US','Long Beach, California','2015-01-01 00:00:00','','Virgin Galactic, Long Beach Facility' from dual union all
select 'VGX','Virgin Galactic','business','','US','Pasadena, California','2004-01-01 00:00:00','','Virgin Galactic' from dual union all
select 'VIA','ViaSat Inc.','business','','US','Carlsbad, California','2007-01-01 00:00:00','','ViaSat Inc.' from dual union all
select 'VIGO','University of Vigo','academic/non-profit','','E','Vigo, Galicia','1990-01-01 00:00:00','','Universidade de Vigo' from dual union all
select 'VILN','Vilnius University','academic/non-profit','','LT','Vilnius','1579-01-01 00:00:00','','Vilniaus universitetas' from dual union all
select 'VISTE','VISIONA Tecnologia Espacial S.A.  (Embraer/Telebras)','business','TELB','BR','Sao Jose de Campos','2012-01-01 00:00:00','','VISIONA Tecnologia Espacial S.A.' from dual union all
select 'VITA','Volunteers in Technical Assistance','business','','US','Washington, DC:Arlington, Virginia','1960-01-01 00:00:00','2005-01-01 00:00:00','Volunteers in Technical Assistance' from dual union all
select 'VKI','Von Karman Institute for Fluid Dynamics','academic/non-profit','','B','Brussels/Sint:Genesius:Rode, Brabant','1956-10-01 00:00:00','','Von Karman Institute for Fluid Dynamics' from dual union all
select 'VKS','Russian Military Space Forces','defense','','RU','Mosvka:Solnechnogorsk, Rossiya','1992-08-10 00:00:00','1997-07-01 00:00:00','Военно-космические силы (ВКС)' from dual union all
select 'VKSR','Russian Military Space Forces','defense','RVSN','RU','Mosvka:Solnechnogorsk, Rossiya','1997-07-01 00:00:00','2001-06-01 00:00:00','Военно-космические силы РВСН' from dual union all
select 'VMF','Soviet Navy','defense','','SU','Moskva, Rossiya','1917-01-01 00:00:00','1991-12-01 00:00:00','Военно-Морской Флот СССР' from dual union all
select 'VMFR','Russian Navy','defense','','RU','Moskva, Rossiya','1991-12-01 00:00:00','','Военно-Морской Флот Российской Федерации' from dual union all
select 'VN','Vietnam','','','VN','Hanoi, Viet Nam','1945-09-02 00:00:00','','Cộng hòa Xã hội chủ nghĩa Việt Nam' from dual union all
select 'VNIEM','AllUnion Electromechanical Scientific Research Institute','business','METP','SU','Moskva, Rossiya','1959-01-01 00:00:00','1992-01-01 00:00:00','Всесоюзный научно-исследовательский институт электромеханики (ВНИИЭМ)' from dual union all
select 'VNIEMR','AllUnion Electromechanical Scientific Research Institute','business','FKA','RU','Moskva, Rossiya','1992-01-01 00:00:00','2011-12-30 00:00:00','Всероссийский научно-исследовательский институт электромеханики с заводом (НПП ВНИИЭМ)' from dual union all
select 'VNIIEMI','Joint Stock Corp. VNIIEM','business','','RU','Moskva, Rossiya','2011-12-30 00:00:00','','ОАО «Корпорация «ВНИИЭМ» имени А.Г. Иосифьяна»' from dual union all
select 'VNPT','Vietnam P&T Corp','civilian','','VN','Dong Da, Ha Noi','1992-01-01 00:00:00','','Tập đoàn Bưu chính Viễn thông Việt Nam' from dual union all
select 'VNSC','National Satellite Center, Vietnam Academy of Science and Te','civilian','VAST','VN','Ha Noi:Hoa Lac','2011-09-01 00:00:00','','Trung tâm Vệ tinh Quốc gia - Viện Hàn lâm Khoa học và Công nghệ Việt Nam' from dual union all
select 'VOENT','Voentelekom','defense','','RU','Moskva','2012-01-01 00:00:00','','Воентелеком' from dual union all
select 'VOKS','Russian Aerospace Forces','defense','','RU','Moskva','2015-08-01 00:00:00','','Воздушно-космические силы BC РФ' from dual union all
select 'VORO','Voronezh Mech. Zavod','engine/motor manufacturer','','RU','Voronezh','','','Воронежский механический завод' from dual union all
select 'VOTK','Votkinskiy Zavod','engine/motor manufacturer','','RU','Votkinsk, Udmurt Oblast','','','Федеральное государственное унитарное предприятие «Воткинский завод»' from dual union all
select 'VOUA','Chance Vought Aircraft (Missiles Div)','business','','US','Dallas:Grand Prairie, Texas','1949-01-01 00:00:00','1962-01-01 00:00:00','Chance Vought Aircraft (Missiles Div)' from dual union all
select 'VS','Soviet Armed Forces','defense','','SU','Moskva','1946-01-01 00:00:00','1992-01-01 00:00:00','Вооружённые Силы СССР' from dual union all
select 'VSSC','Vikram Sarabhai Space Center','civilian','ISRO','IN','Trivandrum, Kerala','1971-01-01 00:00:00','','Vikram Sarabhai Space Center' from dual union all
select 'VTC','Vermont Technical College','academic/non-profit','','US','Randolph Center, Vermont','','','Vermont Technical College' from dual union all
select 'VTS','MilitaryTopographic Service of the Soviet Army','defense','','SU','Moskva','1941-01-01 00:00:00','1991-01-01 00:00:00','Военно-топографическая служба Красной Армии' from dual union all
select 'VTSR','Topographics Service of the Russian Armed Froces','defense','','RU','Moskva','1991-01-01 00:00:00','2010-01-01 00:00:00','Военно-топографическая служба Вооружённых сил Российской Федерации' from dual union all
select 'VULT','Consolidated Vultee Aircraft Corp., Downey (former Vultee AC)','business','CONV','US','Downey, California','1943-01-01 00:00:00','1948-07-01 00:00:00','Consolidated Vultee Aircraft Corp., Downey (former Vultee AC)' from dual union all
select 'VVKO','Aerospace Defence Forces','defense','','RU','Moskva','2011-12-01 00:00:00','2015-01-01 00:00:00','Войска воздушно-космической обороны' from dual union all
select 'VVKOV','Aerospace Defence Forces','defense','VOKS','RU','Moskva','2015-01-01 00:00:00','','Войска воздушно-космической обороны' from dual union all
select 'VVS','VoennoVosdushniye Sili','defense','','SU','Moskva','1918-05-24 00:00:00','1992-05-07 00:00:00','Военно-воздушные силы' from dual union all
select 'VVSR','VoennoVosdushniye SiliRossii','defense','','RU','Moskva','1992-05-07 00:00:00','','Военно-воздушные силы России' from dual union all
select 'VZLU','Aerospace Research and Test Establishment','civilian','','CZ','Praha','1922-01-01 00:00:00','','Výzkumný a zkušební letecký ústav' from dual union all
select 'WADC','Wright Air Development Center/Equipment Laboratory','defense','USAF','US','Wright-Patterson AFB, Dayton, Ohio','1948-01-01 00:00:00','1951-01-01 00:00:00','Wright Air Development Center/Equipment Laboratory' from dual union all
select 'WADD','Wright Air Development Division, ARDC','defense','USAF','US','Wright-Patterson AFB, Dayton, Ohio','1951-01-01 00:00:00','1961-01-01 00:00:00','Wright Air Development Division, ARDC' from dual union all
select 'WAKA','Wakayama University','academic/non-profit','','J','Wakayama, Wakayama-ken','1949-01-01 00:00:00','','和歌山大学' from dual union all
select 'WAML','Wright Aeromedical Laboratory, WADC','defense','USAF','US','Wright-Patterson AFB, Dayton, Ohio','1940-01-01 00:00:00','1951-01-01 00:00:00','Wright Aeromedical Laboratory, WADC' from dual union all
select 'WASEDA','Waseda Daigaku (Sodai)','academic/non-profit','','J','Tokyo:Shinjuku','1902-01-01 00:00:00','','早稲田大学' from dual union all
select 'WBLU','WildBlue Communications Inc.','business','','US','Denver:Greenwood Village, Colorado','1998-01-01 00:00:00','','WildBlue Communications Inc.' from dual union all
select 'WEB','Weber State Univ., Utah','academic/non-profit','','US','Ogden, Utah','1889-01-01 00:00:00','','Weber State Univ., Utah' from dual union all
select 'WEBC','CAST, Weber State Univ., Utah','academic/non-profit','WEB','US','Ogden, Utah','1982-01-01 00:00:00','2002-01-01 00:00:00','CAST, Weber State Univ., Utah' from dual union all
select 'WEHR','Wehrmacht','defense','','DR','Berlin','1939-01-01 00:00:00','1945-01-01 00:00:00','Wehrmacht' from dual union all
select 'WEINA','MicroNano Star Space','business','','CN','Beijing','2017-08-01 00:00:00','','微纳星空' from dual union all
select 'WELEC','Western Electric Co. Burlington Works','business','','US','Burlington, North Carolina','1942-01-01 00:00:00','1991-01-01 00:00:00','Western Electric Co. Burlington Works' from dual union all
select 'WER271-1','Wehrmacht unit 2711','defense','WEHR','DR',':','1939-01-01 00:00:00','1945-01-01 00:00:00','Wehrmacht unit 271-1' from dual union all
select 'WER444','Wehrmacht unit 444','defense','WEHR','DR',':','1939-01-01 00:00:00','1945-01-01 00:00:00','Wehrmacht unit 444' from dual union all
select 'WER485','Wehrmacht unit 485','defense','WEHR','DR',':','1939-01-01 00:00:00','1945-01-01 00:00:00','Wehrmacht unit 485' from dual union all
select 'WER485-1','Wehrmacht unit 4851','defense','WEHR','DR',':','1939-01-01 00:00:00','1945-01-01 00:00:00','Wehrmacht unit 485-1' from dual union all
select 'WER485-2','Wehrmacht unit 4852','defense','WEHR','DR',':','1939-01-01 00:00:00','1945-01-01 00:00:00','Wehrmacht unit 485-2' from dual union all
select 'WER485-3','Wehrmacht unit 4853','defense','WEHR','DR',':','1939-01-01 00:00:00','1945-01-01 00:00:00','Wehrmacht unit 485-3' from dual union all
select 'WER500','Wehrmacht unit 500','defense','WEHR','DR',':','1939-01-01 00:00:00','1945-01-01 00:00:00','Wehrmacht unit 500' from dual union all
select 'WER836','Wehrmacht ArtillerieAbteilung 836','defense','WEHR','DR',':','1939-01-01 00:00:00','1945-01-01 00:00:00','Wehrmacht Artillerie-Abteilung 836' from dual union all
select 'WER836-1','Wehrmacht ArtillerieAbteilung 1./836','defense','WEHR','DR',':','1939-01-01 00:00:00','1945-01-01 00:00:00','Wehrmacht Artillerie-Abteilung 1./836' from dual union all
select 'WER836-2','Wehrmacht ArtillerieAbteilung 2./836','defense','WEHR','DR',':','1939-01-01 00:00:00','1945-01-01 00:00:00','Wehrmacht Artillerie-Abteilung 2./836' from dual union all
select 'WER836-3','Wehrmacht ArtillerieAbteilung 3./836','defense','WEHR','DR',':','1939-01-01 00:00:00','1945-01-01 00:00:00','Wehrmacht Artillerie-Abteilung 3./836' from dual union all
select 'WER902','Wehrmacht unit 902','defense','WEHR','DR',':','1939-01-01 00:00:00','1945-01-01 00:00:00','Wehrmacht unit 902' from dual union all
select 'WESTL','Westland/Osborne Works (SaundersRoe)','business','','UK','Cowes, Isle of Wight','1965-01-01 00:00:00','','Westland/Osborne Works (Saunders-Roe)' from dual union all
select 'WFC','NASA Wallops Flight Center','civilian','NASA','US','Wallops Island, Virginia','1974-04-01 00:00:00','1981-01-01 00:00:00','NASA Wallops Flight Center' from dual union all
select 'WFF','NASA Wallops Flight Facility','civilian','NASA','US','Wallops Island, Virginia','1981-01-01 00:00:00','','NASA Wallops Flight Facility' from dual union all
select 'WI','NASA Wallops Station','civilian','NASA','US','Wallops Island, Virginia','1959-05-01 00:00:00','1974-04-01 00:00:00','NASA Wallops Station' from dual union all
select 'WNI','Weathernews Inc.','business','','J','Tokyo:Chiba','1996-01-01 00:00:00','','ウェザーニューズ' from dual union all
select 'WORSP','Worldspace Inc.','business','','US','Washington D.C.','1990-01-01 00:00:00','2008-07-01 00:00:00','Worldspace Inc.' from dual union all
select 'WORSP1','1worldspace Inc.','business','','US','Washington D.C.','2008-07-01 00:00:00','2010-01-01 00:00:00','Worldspace Inc.' from dual union all
select 'WRE','Weapons Research Establishment','defense','','AU','Adelaide:Salisbury, S Australia','1955-01-01 00:00:00','1978-01-01 00:00:00','Weapons Research Establishment' from dual union all
select 'WSMC','USAF Western Space and Missile Center','defense','USAF','US','Vandenberg AFB, California','1979-01-01 00:00:00','1997-01-01 00:00:00','USAF Western Space and Missile Center' from dual union all
select 'WSMR','White Sands Missile Range','defense','USA','US','White Sands, New Mexico','1958-05-01 00:00:00','','White Sands Missile Range' from dual union all
select 'WSPG','White Sands Proving Ground','defense','USA','US','White Sands, New Mexico','1945-07-09 00:00:00','1958-05-01 00:00:00','White Sands Proving Ground' from dual union all
select 'WTA','Washington Technology Associates','business','','US','Washington, D.C.','','','Washington Technology Associates' from dual union all
select 'WTR','USAF Western Test Range','defense','USAF','US','Vandenberg AFB, California','1964-01-01 00:00:00','1979-01-01 00:00:00','USAF Western Test Range' from dual union all
select 'WUHAN','Wuhan University Aerospace Science and Tech Research Inst','academic/non-profit','','CN','Wuhan, Hubei','2015-01-01 00:00:00','','武汉大学宇航科学与技术研究院' from dual union all
select 'WUPP','Bergische Universit\"at Gesamthochschule Wuppertal (BUGW)','academic/non-profit','','D','Wuppertal, NordRhein:Westphalen','1972-01-01 00:00:00','','Bergische Universität Wuppertal' from dual union all
select 'WURZ','JuliusMaximiliansUniversitat Wurzburg','academic/non-profit','','D','Wurzburg, Bayern','1582-01-01 00:00:00','','Julius-Maximilians-Universität Würzburg' from dual union all
select 'WUTC','Western Union Telegraph Corp.','business','','US','Upper Saddle River, New Jersey','1971-01-01 00:00:00','','Western Union Telegraph Corp.' from dual union all
select 'WXB','US Weather Bureau, Meteorological Satellite Lab','civilian','DOC','US','Suitland, Maryland','1958-01-01 00:00:00','1965-01-01 00:00:00','US Weather Bureau, Meteorological Satellite Lab' from dual union all
select 'X','UNKNOWN','','','X',':','','','-' from dual union all
select 'XCOR','XCOR Aerospace','business','','US','Mojave, California','1999-01-01 00:00:00','2013-01-01 00:00:00','XCOR Aerospace' from dual union all
select 'XCORC','XCOR Aerospace (California)','business','XCORT','US','Mojave, California','2013-01-01 00:00:00','','XCOR Aerospace (California)' from dual union all
select 'XCORT','XCOR Aerospace (Texas)','business','','US','Midland, Texas','2013-01-01 00:00:00','','XCOR Aerospace (Texas)' from dual union all
select 'XCYS','Xian Bureau of Surveying and Mapping','civilian','','CN','Xi''an, Shaaxni','2017-01-01 00:00:00','','西安测绘研究所' from dual union all
select 'XIDI','Xian University of Electronic Science and Tech.','academic/non-profit','','CN','Xi''an, Shaanxi','1931-01-01 00:00:00','','西安电子科技大学' from dual union all
select 'XINWEI','Xinwei tongxin jishu gufen youxian gongsi','business','','CN','Beijing','','','信威通信技术股份有限公司' from dual union all
select 'XIOPM','Xian Institute of Optics and Precision Mechanics','civilian','','CN','Xi''an, Shaanxi','','','西安光学精密机械研究所' from dual union all
select 'XJRY','iSpace','business','','CN','Beijing','2016-10-01 00:00:00','','北京星际荣耀空间科技有限公司' from dual union all
select 'XMR','XM Satellite Radio Holdings (XM)','business','','US','Washington, DC','1988-01-01 00:00:00','2008-07-01 00:00:00','XM Satellite Radio Holdings (XM)' from dual union all
select 'XMRS','XM Satellite Radio Holdings (Sirius XM)','business','SIRX','US','Washington, DC','2008-07-01 00:00:00','','XM Satellite Radio Holdings (Sirius XM)' from dual union all
select 'XTAR','XTAR LLC  (Hisdesat/Loral)','business','LOR','US','Rockville, Maryland','2001-01-01 00:00:00','','XTAR LLC  (Hisdesat/Loral)' from dual union all
select 'YAZMI','Yamzi USA','business','','US','Washington D.C.','2009-01-01 00:00:00','','Yamzi USA' from dual union all
select 'YE','Yemen','','','YE','Sana''a, al-Yamaniyyah','1990-05-22 00:00:00','','الجمهورية اليمنية' from dual union all
select 'YERK','Yerkes Observatory','academic/non-profit','CHIC','US','Williams Bay, Wisconsin','1897-01-01 00:00:00','','Yerkes Observatory' from dual union all
select 'YKVZ','UstKatav Carriage Works','engine/motor manufacturer','','RU','Ust''-Katav (Chelyabinsk)','1758-01-01 00:00:00','','Усть-Катавский вагоностроительный завод' from dual union all
select 'YONS','Yonsei University','academic/non-profit','','KR','Seoul','1885-01-01 00:00:00','','연세대학교' from dual union all
select 'YORK','York University','academic/non-profit','','CA','Toronto, Ontario','1959-01-01 00:00:00','','York University' from dual union all
select 'YORKSS','York Space Systems','business','','US','Denver, Colorado','2015-01-01 00:00:00','','York Space Systems' from dual union all
select 'YSP','Yemen Socialist Party','defense','','YE','South Yemen','1994-01-01 00:00:00','1994-01-01 00:00:00','الحزب الاشتراكي اليمني' from dual union all
select 'YUZGU','SWSU','academic/non-profit','','RU','Kursk','1964-01-01 00:00:00','','Юго-Западный государственный университет' from dual union all
select 'YUZH','Southern SDB','business','','UA','Dnepropetrovsk','1965-01-01 00:00:00','','КБ Южное имени академика М. К. Янгеля' from dual union all
select 'YUZHM','PO Yuzhniy Mashinostroitel''niy Zavod','business','','UA','Dnepropetrovsk','1966-01-01 00:00:00','','Государственное предприятие «Производственное объединение „Южный машиностроительный завод“ имени А. М. Макарова» (Южмаш)' from dual;
insert into ORGANIZATION
select 'ZA','South Africa','','','ZA','Pretoria, Gauteng Province, South Africa','1910-05-31 00:00:00','','Republic of South Africa' from dual union all
select 'ZARM','CASTM Bremen','academic/non-profit','BREM','D','Bremen, FH Bremen','1985-01-01 00:00:00','','Zentrum fur Angewandte Raumfahrttechnologie und Mikrogravitation' from dual union all
select 'ZAV23','Zavod 23, OKB Myasishchev','business','OKB23','SU','Moskva:Fili','1951-01-01 00:00:00','1960-10-01 00:00:00','Завод № 23' from dual union all
select 'ZAV47','Zavod No. 47','business','','SU','Orenburg','1941-01-01 00:00:00','1961-10-15 00:00:00','Завод № 47' from dual union all
select 'ZAV586','Zavod No. 586','business','','SU','Dnepropetrovsk','1954-01-01 00:00:00','1966-01-01 00:00:00','Завод № 586' from dual union all
select 'ZAV88','Zavod No. 88','business','','SU','Moskva:Podlipki','1946-01-01 00:00:00','1966-01-01 00:00:00','Завод № 88' from dual union all
select 'ZC3B','GSD Third Department (SIGINT)','defense','','CN','Beijing:Xishan?','','','解放军总参三部' from dual union all
select 'ZDFH','China SpaceSat','business','','CN','Beijing:Haidian','2006-01-01 00:00:00','','中国东方红卫星股份有限公司' from dual union all
select 'ZEM','Zavod Eksperimental''novo Mashinostroeniya','business','','SU','Moskva:Podlipki','1966-01-01 00:00:00','','Завод экспериментального машиностроения' from dual union all
select 'ZHEJ','Zhejiang U.','academic/non-profit','','CN','Hangzhou, Zhejiang','1897-01-01 00:00:00','','浙江大学' from dual union all
select 'ZHELZ','Zhejiang LiTong Electronic Tech Co. Ltd.','business','','CN','Zhejiang','2012-01-01 00:00:00','','浙江利骓电子科技有限公司' from dual union all
select 'ZHUORB','Zhuhai Orbital Control Engineering Ltd','business','','CN','Zhuhai, Guangdong','2015-01-01 00:00:00','','珠海欧比特控制工程股份有限公司' from dual union all
select 'ZIKH','ZIKh (Zavod im. M.V. Khrunichev)','business','KBSAL','SU','Moskva:Fili','1960-10-01 00:00:00','1994-01-01 00:00:00','Машиностроительный завод им. М. В. Хруничева (ЗИХ)' from dual union all
select 'ZIMN','Zimney Corp.','business','','US','Monrovia, California','1959-01-01 00:00:00','1963-01-01 00:00:00','Zimney Corp.' from dual union all
select 'ZLZB','Strategic Support Troops','defense','PLA','CN','Beijing','2016-01-01 00:00:00','','战略支持部队' from dual union all
select 'ZR','Zaire','','','ZR','Kinshasa','1971-01-01 00:00:00','1997-05-16 00:00:00','République du Zaïre' from dual union all
select 'ZTZM','PEMF','business','','BY','Minsk','2009-01-01 00:00:00','','Завод точной электромеханики' from dual union all
select 'ZVEZ','Zvezda','business','MOM','SU','Tomilino, Moskov. Oblast','1952-01-01 00:00:00','','ОАО «НПП «Звезда»' from dual union all
select 'ZWO','Organization for Pure Scientific Research','civilian','','NL','''s-Gravenhage','1950-01-01 00:00:00','1988-01-01 00:00:00','Organisatie voor Zuiver-Wetenschappelijk Onderzoek' from dual union all
select 'ZZB','General Armaments Dept. of the PLA','defense','PLA','CN','Beijing','1998-04-01 00:00:00','','中国人民解放军总装备部' from dual union all
select 'ZZWYZ','China Resources Satellite Applications Center (CRESDA)','civilian','GZB','CN','Beijing','','','中国资源卫星应用中心' from dual;


ALTER TABLE "ORGANIZATION" ENABLE CONSTRAINT "ORGANIZATION_FK";


--------------------------------------------------------------------------------
-- PLATFORM
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing PLATFORM...');

CREATE TABLE "PLATFORM" 
 (	"PLATFORM_CODE" VARCHAR2(11), 
	"PLATFORM_UCODE" VARCHAR2(12), 
	"PLATFORM_STATE_ORG_CODE" VARCHAR2(8), 
	"PLATFORM_TYPE" VARCHAR2(17), 
	"PLATFORM_CLASS" VARCHAR2(29), 
	"PLATFORM_SHORT_NAME" VARCHAR2(18), 
	"PLATFORM_NAME" VARCHAR2(81), 
	"PLATFORM_PARENT_ORG_CODE" VARCHAR2(13), 
	 CONSTRAINT "PLATFORM_PK" PRIMARY KEY ("PLATFORM_CODE")
USING INDEX  ENABLE, 
	 CONSTRAINT "PLATFORM_STATE_FK" FOREIGN KEY ("PLATFORM_STATE_ORG_CODE")
	  REFERENCES "ORGANIZATION" ("ORG_CODE") ENABLE, 
	 CONSTRAINT "PLATFORM_PARENT_FK" FOREIGN KEY ("PLATFORM_PARENT_ORG_CODE")
	  REFERENCES "ORGANIZATION" ("ORG_CODE") ENABLE
 ) ;


CREATE INDEX "PLATFORM_IDX1" ON "PLATFORM" ("PLATFORM_STATE_ORG_CODE") 
;

CREATE INDEX "PLATFORM_IDX2" ON "PLATFORM" ("PLATFORM_PARENT_ORG_CODE") 
;
insert into PLATFORM
select 'AEGIS','AEGIS','US','SHIP','defense','AEGIS ship','Unknown AEGIS class ship','USN' from dual union all
select 'AGB-4','AGB-4','US','SHIP','defense','SHIP  AGB-4','USS Glacier','USN' from dual union all
select 'AGB-5','AGB-5','US','SHIP','defense','SHIP  AGB-5','USS Staten Island','USN' from dual union all
select 'AVM1','AVM1','US','SHIP','defense','SHIP  AVM1','USS Norton Sound','USN' from dual union all
select 'B-29','B-29','US','AIR','defense','AIR   B-29','B-29 Superfortress','USAF' from dual union all
select 'B-47','B-47','US','AIR','defense','AIR   B-47','B-47','USAF' from dual union all
select 'B-52','B-52','US','AIR','defense','AIR   B-52','B-52 Stratofortress','USAF' from dual union all
select 'B-52G','B-52G','US','AIR','defense','AIR   B-52G','B-52 Stratofortress','USAF' from dual union all
select 'B-52H','B-52H','US','AIR','defense','AIR   B-52H','B-52 Stratofortress','USAF' from dual union all
select 'B-57A','B-57A','US','AIR','defense','AIR   B-57A','B-57 Canberra','USAF' from dual union all
select 'B-57B','B-57B','US','AIR','defense','AIR   B-57B','B-57 Canberra','USAF' from dual union all
select 'B-58','B-58','US','AIR','defense','AIR   B-58','B-58 Hustler','USAF' from dual union all
select 'B-62','B-62','SU','SUB','defense','SUB   B-62','B-62 submarine','VMF' from dual union all
select 'B-67','B-67','SU','SUB','defense','SUB   B-67','B-67 submarine','VMF' from dual union all
select 'BUGAEV','BUGAEV','SU','SHIP','defense','SHIP  BUGAEV','Bugaev','VMF' from dual union all
select 'Balloon','Balloon','US','AIR','defense','AIR   Balloon','Balloon','USAF' from dual union all
select 'C-130','C-130','US','AIR','defense','AIR   C-130','C-130 Hercules','USAF' from dual union all
select 'C-17','C-17','US','AIR','defense','AIR   C-17','C-17 cargo plane','USAF' from dual union all
select 'C-5A','C-5A','US','AIR','defense','AIR   C-5A','C-5A Galaxy','USAF' from dual union all
select 'CG-20','DLG-20','US','SHIP','defense','SHIP  CG-20','USS Richmond K. Turner (was DLG-20 prior to 1975)','USN' from dual union all
select 'CG-29','CG-29','US','SHIP','defense','SHIP  CG-29','USS Jouett','USN' from dual union all
select 'CG-67','CG-67','US','SHIP','defense','SHIP  CG-67','USS Shiloh','USN' from dual union all
select 'CG-70','CG-70','US','SHIP','defense','SHIP  CG-70','USS Lake Erie','USN' from dual union all
select 'CV-41','CV-41','US','SHIP','defense','SHIP  CV-41','USS Midway','USN' from dual union all
select 'CVE-25','CVE-25','US','SHIP','defense','SHIP  CVE-25','USNS Croatan, T-AKV 43 (former CVE-25), Wallops Mobile Range Facility','USN' from dual union all
select 'DD-787','DD-787','US','SHIP','defense','SHIP  DD-787','USS Kyes','USN' from dual union all
select 'DD-790','DD-790','US','SHIP','defense','SHIP  DD-790','USS Shelton','USN' from dual union all
select 'DD-843','DD-843','US','SHIP','defense','SHIP  DD-843','USS Warrington','USN' from dual union all
select 'DDG-1','DDG-1','US','SHIP','defense','SHIP  DDG-1','USS Hazelwood','USN' from dual union all
select 'DDG-173','DDG-173','J','SHIP','defense','SHIP  DDG-173','JS Kongo','JMSDF' from dual union all
select 'DDG-174','DDG-174','J','SHIP','defense','SHIP  DDG-174','JS Kirishima','JMSDF' from dual union all
select 'DDG-175','DDG-175','J','SHIP','defense','SHIP  DDG-175','JS Myoko','JMSDF' from dual union all
select 'DDG-176','DDG-176','J','SHIP','defense','SHIP  DDG-176','JS Chokai','JMSDF' from dual union all
select 'DDG-53','DDG-53','US','SHIP','defense','SHIP  DDG-53','USS John Paul Jones','USN' from dual union all
select 'DDG-60','DDG-60','US','SHIP','defense','SHIP  DDG-60','USS Paul Hamilton','USN' from dual union all
select 'DDG-62','DDG-62','US','SHIP','defense','SHIP  DDG-62','USS Fitzgerald','USN' from dual union all
select 'DDG-65','DDG-65','US','SHIP','defense','SHIP  DDG-65','USS Benfold','USN' from dual union all
select 'DDG-68','DDG-68','US','SHIP','defense','SHIP  DDG-68','USS The Sullivans','USN' from dual union all
select 'DDG-70','DDG-70','US','SHIP','defense','SHIP  DDG-70','USS Hopper','USN' from dual union all
select 'DDG-71','DDG-71','US','SHIP','defense','SHIP  DDG-71','USS Ross','USN' from dual union all
select 'DDG-73','DDG-73','US','SHIP','defense','SHIP  DDG-73','USS Decatur','USN' from dual union all
select 'DDG-74','DDG-74','US','SHIP','defense','SHIP  DDG-74','USS McFaul','USN' from dual union all
select 'DDG-75','DDG-75','US','SHIP','defense','SHIP  DDG-75','USS Donald Cook','USN' from dual union all
select 'DDG-77','DDG-77','US','SHIP','defense','SHIP  DDG-77','USS O/Kane','USN' from dual union all
select 'DELTA2','DELTA2','SU','SUB','defense','SUB   DELTA2','Unknown Delta 2 class submarine','VMF' from dual union all
select 'DELTA3','DELTA3','SU','SUB','defense','SUB   DELTA3','Unknown Delta 3 class submarine','VMF' from dual union all
select 'DELTA4','DELTA4','SU','SUB','defense','SUB   DELTA4','Unknown Delta 4 class submarine','VMF' from dual union all
select 'DLG-20','DLG-20','US','SHIP','defense','SHIP  CG-20','USS Richmond K. Turner (was DLG-20 prior to 1975)','USN' from dual union all
select 'EAG-154','EAG-154','US','SHIP','defense','SHIP  EAG-154','USNS Observation Island (became AGM-23 in 1977)','USN' from dual union all
select 'F-104','F-104','US','AIR','defense','AIR   F-104','F-104 Starfighter S/N 60749','USAF' from dual union all
select 'F-15','F-15','US','AIR','defense','AIR   F-15','F-15 Eagle, ASAT program.','USAF' from dual union all
select 'F-15D','F-15D','IL','AIR','defense','AIR   F-15','F-15D Eagle, Black Sparrow program.','IDF' from dual union all
select 'F-3B','F-3B','US','AIR','defense','AIR   F-3B','F-3B Demon','USAF' from dual union all
select 'F-4','F-4','US','AIR','defense','AIR   F-4','F-4 Phantom?','USAF' from dual union all
select 'F-4B','F-4B','US','AIR','defense','AIR   F-4B','F-4B Phantom','USAF' from dual union all
select 'F-4C','F-4C','US','AIR','defense','AIR   F-4C','F-4C Phantom','USAF' from dual union all
select 'F-4D','F-4D','US','AIR','defense','AIR   F-4D','F-4D Phantom','USAF' from dual union all
select 'F-4H','F-4H','US','AIR','defense','AIR   F-4H','F-4H Phantom','USN' from dual union all
select 'F-86D','F-86D','US','AIR','defense','AIR   F-86D','F-86D, Rockaire program','USAF' from dual union all
select 'F2H2','F2H2','US','AIR','defense','AIR   F2H2','F2H2 Banshee','USAF' from dual union all
select 'F4D-1 747','F4D-1 747','US','AIR','defense','AIR   F4D-1 747','F4D-1 Skyray S/N 130747','USAF' from dual union all
select 'FB-111','FB-111','US','AIR','defense','AIR   FB-111','FB-111','USAF' from dual union all
select 'GOLF','GOLF','SU','SUB','defense','SUB   GOLF','Unknown Golf submarine','VMF' from dual union all
select 'GOLF-C','GOLF-C','SU','SUB','defense','SUB   GOLF-C','Unknown Golf class submarine, Chinese navy','PLANS' from dual union all
select 'GOLF2','GOLF2','SU','SUB','defense','SUB   GOLF2','Unknown Golf 2 submarine','VMF' from dual union all
select 'HOTEL2','HOTEL2','SU','SUB','defense','SUB   HOTEL2','Unknown Hotel 2 submarine','VMF' from dual union all
select 'HULU','HULU','CN','SUB','defense','SUB   HULU','Huludao, submarine','PLANS' from dual union all
select 'INS-D15','INS-D15','IN','SHIP','defense','SHIP  INS-D15','INS Rajput (D-15)','INAV' from dual union all
select 'INS-OPV','INS-OPV','IN','SHIP','defense','SHIP  INS','Unknown INS Offshore Patrol Vessel (INS-P51 and similar)','INAV' from dual union all
select 'INS-P51','INS-P51','IN','SHIP','defense','SHIP  INS-P51','INS Subhadra (P-51)','INAV' from dual union all
select 'INS-P52','INS-P52','IN','SHIP','defense','SHIP  INS-P52','INS Suvarna (P-52)','INAV' from dual union all
select 'INS-S73','INS-S73','IN','SUB','defense','SUB   INS-S73','INS Arihant (S-73)','INAV' from dual union all
select 'JRTI','JRTI','US','SHIP','civilian','ASDS  JRTI','Just Read The Instructions, SpaceX Autonomous Spaceport Drone Ship','SPX' from dual union all
select 'K-102','K-102','SU','SUB','defense','SUB   K-102','K-102 Project 605 Submarine (was B-121)','VMF' from dual union all
select 'K-114','K-114','SU','SUB','defense','SUB   K-114','K-114 Tula Submarine 667BRDRM (Sev. Flot)','VMF' from dual union all
select 'K-117','K-117','SU','SUB','defense','SUB   K-117','K-117 Bryansk Submarine 667BDRM (Sev. Flot)','VMF' from dual union all
select 'K-140','K-140','SU','SUB','defense','SUB   K-140','K-140 Submarine','VMF' from dual union all
select 'K-145','K-145','SU','SUB','defense','SUB   K-145','K-145 Submarine','VMF' from dual union all
select 'K-170','K-170','SU','SUB','defense','SUB   K-170','Unknown Project 675 (Echo II) submarine','VMF' from dual union all
select 'K-18','K-18','SU','SUB','defense','SUB   K-18','K-18 Karelia submarine 667BDRM (Sev. Flot)','VMF' from dual union all
select 'K-211','K-211','SU','SUB','defense','SUB   K-211','K-211 Petropavlovsk-Kamchatkskiy submarine 667BDR Pac. Flt.  (Delta III)','VMF' from dual union all
select 'K-223','K-223','SU','SUB','defense','SUB   K-223','K-223 Podolsk submarine, 667BDR Pac. Flt','VMF' from dual union all
select 'K-407','K-407','SU','SUB','defense','SUB   K-407','K-407 Novomoskvosk Submarine 667BDRM  (Sev. Flot)','VMF' from dual union all
select 'K-433','K-433','SU','SUB','defense','SUB   K-433','K-433 Svyaity Georgiy Pobedonosets Submarine 667BDR (Pac. flt)','VMF' from dual union all
select 'K-44','K-44','SU','SUB','defense','SUB   K-44','K-44 Ryazan Submarine 667BDR (Sev. Flot)','VMF' from dual union all
select 'K-496','K-496','SU','SUB','defense','SUB   K-496','K-496 Borisoglebsk, submarine 667BDR (Sev. Flot)','VMF' from dual union all
select 'K-506','K-506','SU','SUB','defense','SUB   K-506','K-506 Zelenograd submarine 667BDR (Pac. Flt)','VMF' from dual union all
select 'K-51','K-51','SU','SUB','defense','SUB   K-51','K-51 Verkhoturiye Sumbarine 667BDRM (Sev. Flot)','VMF' from dual union all
select 'K-535','K-535','RU','SUB','defense','SUB   K-535','K-535 Yuriy Dolgorukiy submarine, class 955','VMF' from dual union all
select 'K-550','K-550','RU','SUB','defense','SUB   K-550','K-550 Aleksandry Nevskiy submarine, class 955','VMF' from dual union all
select 'K-551','K-551','RU','SUB','defense','SUB   K-551','K-551 Vladimir Monomakh, class 955','VMF' from dual union all
select 'K-84','K-84','SU','SUB','defense','SUB   K-84','K-84 Ekaterinburg Submarine 667BDRM (Sev. Flot)','VMF' from dual union all
select 'KALMAR','KALMAR','SU','SUB','defense','SUB   KALMAR','Unknown Kalmar (Delta III) class submarine','VMF' from dual union all
select 'KORO','KORO','SU','SHIP','defense','SHIP  KORO','Akademik Korolev','VMF' from dual union all
select 'KPN PLAT','KPN PLAT','KP','SUB','defense','SUB KPN Barge','Submerged barge,Korean People''s Navy','KPA' from dual union all
select 'KPN UNK','KPN UNK','KP','SUB','defense','SUB   SINPO','Unknown Sinpo class submarine, Korean People''s Navy','KPA' from dual union all
select 'KRENKEL''','KRENKEL''','SU','SHIP','defense','SHIP  KRENKEL''','Krenkel''','VMF' from dual union all
select 'L-1011','L-1011','US','AIR','defense','AIR   L-1011','L-1011 Stargazer','OSC' from dual union all
select 'LSD-14','LSD-14','US','SHIP','defense','SHIP  LSD-14','USS Rushmore','USN' from dual union all
select 'LSD-18','LSD-18','US','SHIP','defense','SHIP  LSD-18','USS Colonial','USN' from dual;
insert into PLATFORM
select 'LSD-29','LSD-29','US','SHIP','defense','SHIP  LSD-29','USS Plymouth Rock','USN' from dual union all
select 'LSD-31','LSD-31','US','SHIP','defense','SHIP  LSD-31','USS Point Defiance','USN' from dual union all
select 'LUT1','LUT1','US','PLAT','defense','PLAT  LUT1','Launch Umbilical Tower 1','NASA' from dual union all
select 'LUT2','LUT2','US','PLAT','defense','PLAT  LUT2','Launch Umbilical Tower 2','NASA' from dual union all
select 'LUT3','LUT3','US','PLAT','defense','PLAT  LUT3','Launch Umbilical Tower 3','NASA' from dual union all
select 'MLP','MLP','US','SHIP','defense','SHIP  MLP','US Army Mobile Launch Platform (USS Tripoli)','USA' from dual union all
select 'MLP1','LUT3','US','PLAT','defense','PLAT  MLP1','Mobile Launch Platform 1','NASA' from dual union all
select 'MLP2','LUT2','US','PLAT','defense','PLAT  MLP2','Mobile Launch Platform 2','NASA' from dual union all
select 'MLP3','LUT1','US','PLAT','defense','PLAT  MLP3','Mobile Launch Platform 3','NASA' from dual union all
select 'MUSSON','MUSSON','SU','SHIP','defense','SHIP  MUSSON','Musson','VMF' from dual union all
select 'MiG31D-72','MiG31D-72','SU','AIR','defense','Mig-31D 07-2','Mig-31D 07-2','PVO' from dual union all
select 'NB-52 003','NB-52 003','US','AIR','defense','AIR   NB-52 003','NB-52A Stratofortress S/N 003','USAF' from dual union all
select 'NB-52 008','NB-52 008','US','AIR','defense','AIR   NB-52 008','NB-52B Stratofortress S/N 008','USAF' from dual union all
select 'NB-52','NB-52','US','AIR','defense','AIR   NB-52','NB-52 Stratofortress (S/N 003 or 008)','USAF' from dual union all
select 'NOKV','NOKV','SU','SHIP','defense','SHIP  NOKV','Noyekov','VMF' from dual union all
select 'OCISLY','OCISLY','US','SHIP','civilian','ASDS  OCISLY','Of Course I Still Love You, SpaceX Autonomous Spaceport Drone Ship','SPX' from dual union all
select 'ODYSSEY','ODYSSEY','US','SHIP','defense','SHIP  ODYSSEY','Sea Launch Odyssey','BOSL' from dual union all
select 'PASSAT','PASSAT','SU','SHIP','defense','SHIP  PASSAT','Passat','VMF' from dual union all
select 'PLBR','PLBR','SU','SUB','defense','SUB   PLBR','Unknown Soviet/Russian ballistic missile submarine','VMF' from dual union all
select 'PRILIV','PRILIV','SU','SHIP','defense','SHIP  PRILIV','Priliv','VMF' from dual union all
select 'RB-57B','RB-57B','US','AIR','defense','AIR   RB-57B','B-57 Canberra','USAF' from dual union all
select 'RB-57C','RB-57C','US','AIR','defense','AIR   RB-57C','B-57 Canberra','USAF' from dual union all
select 'RREC','RREC','US','SHIP','defense','SHIP  RREC','USNS Range Recoverer, T-AG-161','USN' from dual union all
select 'S-229','S-229','SU','SUB','defense','SUB   S-229','S-229 Submarine','VMF' from dual union all
select 'S610','S610','F','SUB','defense','SUB   S610','SNLE S610 Le Foudroyant','ARMF' from dual union all
select 'S611','S611','F','SUB','defense','SUB   S611','SNLE S611 Le Redoutable','ARMF' from dual union all
select 'S612','S612','F','SUB','defense','SUB   S612','SNLE S612 Le Terrible','ARMF' from dual union all
select 'S613','S613','F','SUB','defense','SUB   S613','SNLE S613 L''Indomptable','ARMF' from dual union all
select 'S614','S614','F','SUB','defense','SUB   S614','SNLE S614 Le Tonnant','ARMF' from dual union all
select 'S615','S615','F','SUB','defense','SUB   S615','SNLE S615 L''Inflexible','ARMF' from dual union all
select 'S616','S616','F','SUB','defense','SUB   S616','SNLE-NG S616 Le Triomphant','ARMF' from dual union all
select 'S617','S617','F','SUB','defense','SUB   S617','SNLE-NG S617 Le Temeraire','ARMF' from dual union all
select 'S618','S618','F','SUB','defense','SUB   S618','SNLE-NG S618 Le Vigilant','ARMF' from dual union all
select 'S655','S655','F','SUB','defense','SUB   S655','SNLE S655 Gymnote','ARMF' from dual union all
select 'SHIP-ISR','SHIP-ISR','IL','SHIP','defense','SHIP  SHIP-ISR','Unknown Israeli ship','IDF' from dual union all
select 'SHIP-S','SHIP-S','US','SHIP','defense','SHIP  SHIP-S','Ship Sierra, MRN  (maybe Range Recoverer?)','USN' from dual union all
select 'SHIP-SOV','SHIP-SOV','SU','SHIP','defense','SHIP  SHIP-SOV','Unknown Soviet ship','VMF' from dual union all
select 'SHIP-US','SHIP-US','US','SHIP','defense','SHIP  SHIP-US','Unknown US ship','USN' from dual union all
select 'SHIR','SHIR','SU','SHIP','defense','SHIP  SHIR','Shirshov','VMF' from dual union all
select 'SHOK','SHOK','SU','SHIP','defense','SHIP  SHOK','Shokalski','VMF' from dual union all
select 'SIR','SIR','KR','SHIP','defense','SHIP  SIRIUS','Sirius barge, Anhueng','ADD' from dual union all
select 'SNLE','SNLE','F','SUB','defense','SUB   SNLE','Unknown SNLE submarine, Marine Francaise','ARMF' from dual union all
select 'SSBN 598','SSBN 598','US','SUB','defense','SUB   SSBN 598','George Washington','USN' from dual union all
select 'SSBN 599','SSBN 599','US','SUB','defense','SUB   SSBN 599','Patrick Henry','USN' from dual union all
select 'SSBN 600','SSBN 600','US','SUB','defense','SUB   SSBN 600','Theodore Roosevelt','USN' from dual union all
select 'SSBN 601','SSBN 601','US','SUB','defense','SUB   SSBN 601','Robert E Lee','USN' from dual union all
select 'SSBN 602','SSBN 602','US','SUB','defense','SUB   SSBN 602','Abraham Lincoln','USN' from dual union all
select 'SSBN 608','SSBN 608','US','SUB','defense','SUB   SSBN 608','Ethan Allen','USN' from dual union all
select 'SSBN 609','SSBN 609','US','SUB','defense','SUB   SSBN 609','Sam Houston','USN' from dual union all
select 'SSBN 610','SSBN 610','US','SUB','defense','SUB   SSBN 610','Thomas A Edison','USN' from dual union all
select 'SSBN 611','SSBN 611','US','SUB','defense','SUB   SSBN 611','John Marshall','USN' from dual union all
select 'SSBN 616','SSBN 616','US','SUB','defense','SUB   SSBN 616','Lafayette','USN' from dual union all
select 'SSBN 617','SSBN 617','US','SUB','defense','SUB   SSBN 617','Alexander Hamilton','USN' from dual union all
select 'SSBN 618','SSBN 618','US','SUB','defense','SUB   SSBN 618','Thomas Jefferson','USN' from dual union all
select 'SSBN 619','SSBN 619','US','SUB','defense','SUB   SSBN 619','Andrew Jackson','USN' from dual union all
select 'SSBN 620','SSBN 620','US','SUB','defense','SUB   SSBN 620','John Adams','USN' from dual union all
select 'SSBN 622','SSBN 622','US','SUB','defense','SUB   SSBN 622','James Monroe','USN' from dual union all
select 'SSBN 623','SSBN 623','US','SUB','defense','SUB   SSBN 623','Nathan Hale','USN' from dual union all
select 'SSBN 624','SSBN 624','US','SUB','defense','SUB   SSBN 624','Woodrow Wilson','USN' from dual union all
select 'SSBN 625','SSBN 625','US','SUB','defense','SUB   SSBN 625','Henry Clay','USN' from dual union all
select 'SSBN 626','SSBN 626','US','SUB','defense','SUB   SSBN 626','Daniel Webster','USN' from dual union all
select 'SSBN 627','SSBN 627','US','SUB','defense','SUB   SSBN 627','James Madison','USN' from dual union all
select 'SSBN 628','SSBN 628','US','SUB','defense','SUB   SSBN 628','Tecumseh','USN' from dual union all
select 'SSBN 629','SSBN 629','US','SUB','defense','SUB   SSBN 629','Daniel Boone','USN' from dual union all
select 'SSBN 630','SSBN 630','US','SUB','defense','SUB   SSBN 630','John C Calhoun','USN' from dual union all
select 'SSBN 631','SSBN 631','US','SUB','defense','SUB   SSBN 631','Ulysses S Grant','USN' from dual union all
select 'SSBN 632','SSBN 632','US','SUB','defense','SUB   SSBN 632','Von Steuben','USN' from dual union all
select 'SSBN 633','SSBN 633','US','SUB','defense','SUB   SSBN 633','Casimir Pulaski','USN' from dual union all
select 'SSBN 634','SSBN 634','US','SUB','defense','SUB   SSBN 634','Stonewall Jacskon','USN' from dual union all
select 'SSBN 635','SSBN 635','US','SUB','defense','SUB   SSBN 635','Sam Rayburn','USN' from dual union all
select 'SSBN 636','SSBN 636','US','SUB','defense','SUB   SSBN 636','Nathanael Greene','USN' from dual union all
select 'SSBN 640','SSBN 640','US','SUB','defense','SUB   SSBN 640','Benjamin Franklin','USN' from dual union all
select 'SSBN 641','SSBN 641','US','SUB','defense','SUB   SSBN 641','Simon Bolivar','USN' from dual union all
select 'SSBN 642','SSBN 642','US','SUB','defense','SUB   SSBN 642','Kamehameha','USN' from dual union all
select 'SSBN 643','SSBN 643','US','SUB','defense','SUB   SSBN 643','George Bancroft','USN' from dual union all
select 'SSBN 644','SSBN 644','US','SUB','defense','SUB   SSBN 644','Lewis and Clark','USN' from dual union all
select 'SSBN 645','SSBN 645','US','SUB','defense','SUB   SSBN 645','James K Polk','USN' from dual union all
select 'SSBN 654','SSBN 654','US','SUB','defense','SUB   SSBN 654','George C Marshall','USN' from dual union all
select 'SSBN 655','SSBN 655','US','SUB','defense','SUB   SSBN 655','Henry L Stimson','USN' from dual union all
select 'SSBN 656','SSBN 656','US','SUB','defense','SUB   SSBN 656','George Washington Carver','USN' from dual union all
select 'SSBN 657','SSBN 657','US','SUB','defense','SUB   SSBN 657','Francis Scott Key','USN' from dual union all
select 'SSBN 658','SSBN 658','US','SUB','defense','SUB   SSBN 658','Mariano G Vallejo','USN' from dual union all
select 'SSBN 659','SSBN 659','US','SUB','defense','SUB   SSBN 659','Will Rogers','USN' from dual union all
select 'SSBN 726','SSBN 726','US','SUB','defense','SUB   SSBN 726','Ohio','USN' from dual union all
select 'SSBN 727','SSBN 727','US','SUB','defense','SUB   SSBN 727','Michigan','USN' from dual union all
select 'SSBN 728','SSBN 728','US','SUB','defense','SUB   SSBN 728','Florida','USN' from dual union all
select 'SSBN 729','SSBN 729','US','SUB','defense','SUB   SSBN 729','Georgia','USN' from dual union all
select 'SSBN 730','SSBN 730','US','SUB','defense','SUB   SSBN 730','Henry M. Jackson','USN' from dual union all
select 'SSBN 731','SSBN 731','US','SUB','defense','SUB   SSBN 731','Alabama','USN' from dual union all
select 'SSBN 732','SSBN 732','US','SUB','defense','SUB   SSBN 732','Alaska','USN' from dual union all
select 'SSBN 733','SSBN 733','US','SUB','defense','SUB   SSBN 733','Nevada','USN' from dual union all
select 'SSBN 734','SSBN 734','US','SUB','defense','SUB   SSBN 734','Tennessee','USN' from dual union all
select 'SSBN 735','SSBN 735','US','SUB','defense','SUB   SSBN 735','Pennsylvania','USN' from dual union all
select 'SSBN 736','SSBN 736','US','SUB','defense','SUB   SSBN 736','West Virginia','USN' from dual union all
select 'SSBN 737','SSBN 737','US','SUB','defense','SUB   SSBN 737','Kentucky','USN' from dual union all
select 'SSBN 738','SSBN 738','US','SUB','defense','SUB   SSBN 738','Maryland','USN' from dual union all
select 'SSBN 739','SSBN 739','US','SUB','defense','SUB   SSBN 739','Nebraska','USN' from dual union all
select 'SSBN 740','SSBN 740','US','SUB','defense','SUB   SSBN 740','Rhode Island','USN' from dual union all
select 'SSBN 741','SSBN 741','US','SUB','defense','SUB   SSBN 741','Maine','USN' from dual union all
select 'SSBN 742','SSBN 742','US','SUB','defense','SUB   SSBN 742','Wyoming','USN' from dual;
insert into PLATFORM
select 'SSBN 743','SSBN 743','US','SUB','defense','SUB   SSBN 743','Louisiana','USN' from dual union all
select 'SSBN UNK','SSBN UNK','US','SUB','defense','SUB   SSBN','Unknown SSBN submarine, USN','ARMF' from dual union all
select 'T-AGM-12','T-AGM-12','US','SHIP','defense','SHIP  T-AGM-12','USAS American Mariner','USN' from dual union all
select 'T-AGM-2','T-AGM-2','US','SHIP','defense','SHIP  T-AGM-2','USNS Range Recoverer','USN' from dual union all
select 'T-AGM-8','T-AGM-8','US','SHIP','defense','SHIP  T-AGM-8','USNS Wheeling','USN' from dual union all
select 'T094','T094','CN','SUB','defense','SUB   Type 094','Unknown Jinji Qianting (Type 094) class submarine','PLANS' from dual union all
select 'TAIFUN','TAIFUN','SU','SUB','defense','SUB   TAIFUN','Unknown 941 class submarine','VMF' from dual union all
select 'TK-20','TK-20','SU','SUB','defense','SUB   TK-20','Severstal TK-20  submarine','VMF' from dual union all
select 'TK-208','TK-208','SU','SUB','defense','SUB   TK-208','Dmitry Donskoy TK-208 submarine 941 class','VMF' from dual union all
select 'UK S22','UK S22','UK','SUB','defense','SUB   UK S22','HMS Resolution   (UK01)','RN' from dual union all
select 'UK S23','UK S23','UK','SUB','defense','SUB   UK S23','HMS Repulse      (UK03)','RN' from dual union all
select 'UK S26','UK S26','UK','SUB','defense','SUB   UK S26','HMS Renown       (UK02)','RN' from dual union all
select 'UK S27','UK S27','UK','SUB','defense','SUB   UK S27','HMS Revenge      (UK04)','RN' from dual union all
select 'UK S28','UK S28','UK','SUB','defense','SUB   UK S28','HMS Vanguard     (UK05)','RN' from dual union all
select 'UK S29','UK S29','UK','SUB','defense','SUB   UK S29','HMS Victorious   (UK06)','RN' from dual union all
select 'UK S30','UK S30','UK','SUB','defense','SUB   UK S30','HMS Vigilant','RN' from dual union all
select 'UK S31','UK S31','UK','SUB','defense','SUB   UK S31','HMS Vengeance','RN' from dual union all
select 'USHAKOV','USHAKOV','SU','SHIP','defense','SHIP  USHAKOV','Ushakov','VMF' from dual union all
select 'VISE','VISE','SU','SHIP','defense','SHIP  VISE','Prof. Vise','VMF' from dual union all
select 'VOKV','VOKV','SU','SHIP','defense','SHIP  VOKV','Voiekov','VMF' from dual union all
select 'VOLNA','VOLNA','SU','SHIP','defense','SHIP  VOLNA','Volna','VMF' from dual union all
select 'WK1','WK1','US','AIR','business','White Knight 1','White Knight 1','SCAL' from dual union all
select 'WK2','WK2','US','AIR','business','White Knight 2','White Knight 2','TSCO' from dual union all
select 'WYG-279','WYG-279','US','SHIP','defense','SHIP  WYG-279','USCGC Eastwind','USCG' from dual union all
select 'XIA','XIA','CN','SUB','defense','SUB   XIA','Unknown Xia class submarine','PLANS' from dual union all
select 'YANKEE','YANKEE','SU','SUB','defense','SUB   YANKEE','Unknown Yankee class submarine','VMF' from dual union all
select 'ZUB','ZUB','SU','SHIP','defense','SHIP  ZUB','Prof. Zubov','VMF' from dual union all
select 'ZULUV','ZULUV','SU','SUB','defense','SUB   ZULUV','Unknown Zulu V class submarine','VMF' from dual;


--------------------------------------------------------------------------------
-- ORGANIZATION_ORG_TYPE
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing ORGANIZATION_ORG_TYPE...');

CREATE TABLE "ORGANIZATION_ORG_TYPE" 
 (	"ORG_CODE" VARCHAR2(9), 
	"ORG_TYPE" VARCHAR2(4000), 
	 CONSTRAINT "ORGANIZATION_ORG_TYPE_PK" PRIMARY KEY ("ORG_CODE", "ORG_TYPE")
USING INDEX  ENABLE, 
	 CONSTRAINT "ORGANIZATION_ORG_TYPE_KK" FOREIGN KEY ("ORG_CODE")
	  REFERENCES "ORGANIZATION" ("ORG_CODE") ENABLE
 ) ;

insert into ORGANIZATION_ORG_TYPE
select '10ADS','launch agency' from dual union all
select '11TMS','launch agency' from dual union all
select '1GMR','launch agency' from dual union all
select '1GMS','launch agency' from dual union all
select '1PBS','launch agency' from dual union all
select '1STRAD','launch agency' from dual union all
select '209FA','launch agency' from dual union all
select '2GMS','launch agency' from dual union all
select '2SLS','launch agency' from dual union all
select '30FA2','launch agency' from dual union all
select '30SW','launch agency' from dual union all
select '321SMW','launch agency' from dual union all
select '341MW','launch agency' from dual union all
select '341SMW','launch agency' from dual union all
select '341SW','launch agency' from dual union all
select '351SMW','launch agency' from dual union all
select '36AIS','launch agency' from dual union all
select '3GMS','launch agency' from dual union all
select '3SLS','launch agency' from dual union all
select '40FA','launch agency' from dual union all
select '40FA1','launch agency' from dual union all
select '41FA1','launch agency' from dual union all
select '41FA4','launch agency' from dual union all
select '44FA2','launch agency' from dual union all
select '44SMW','launch agency' from dual union all
select '4504MTW','launch agency' from dual union all
select '4505M','launch agency' from dual union all
select '455SMW','launch agency' from dual union all
select '46FA2','launch agency' from dual union all
select '4803GMS','launch agency' from dual union all
select '4SLS','launch agency' from dual union all
select '4WG','launch agency' from dual union all
select '4WG','school' from dual union all
select '556SMS','launch agency' from dual union all
select '564SMS','launch agency' from dual union all
select '576FTS','launch agency' from dual union all
select '576SMS','launch agency' from dual union all
select '6555A','launch agency' from dual union all
select '6555GMS','launch agency' from dual union all
select '69PBS','launch agency' from dual union all
select '702SMW','launch agency' from dual union all
select '79FA2','launch agency' from dual union all
select '81FA1','launch agency' from dual union all
select '84FA3','launch agency' from dual union all
select '90MW','launch agency' from dual union all
select '90SMW','launch agency' from dual union all
select '90SW','launch agency' from dual union all
select '91MW','launch agency' from dual union all
select '91SW','launch agency' from dual union all
select '9FA3','launch agency' from dual union all
select 'AAAF','launch site' from dual union all
select 'AACH','organization' from dual union all
select 'AACH','satellite manufacturer' from dual union all
select 'AACH','school' from dual union all
select 'AACHR','organization' from dual union all
select 'AACHR','satellite manufacturer' from dual union all
select 'AACM','satellite manufacturer' from dual union all
select 'AADC','launch agency' from dual union all
select 'AALTO','organization' from dual union all
select 'AALTO','satellite manufacturer' from dual union all
select 'AAMV','satellite manufacturer' from dual union all
select 'AAP','organization' from dual union all
select 'AAT','country' from dual union all
select 'AAU','organization' from dual union all
select 'AAU','satellite manufacturer' from dual union all
select 'ABAE','organization' from dual union all
select 'ABE','organization' from dual union all
select 'ABL','engine/motor manufacturer' from dual union all
select 'ABL','launch vehicle manufacturer' from dual union all
select 'ABLN','engine/motor manufacturer' from dual union all
select 'ABMA','engine/motor manufacturer' from dual union all
select 'ABMA','launch agency' from dual union all
select 'ABMA','launch vehicle manufacturer' from dual union all
select 'ABMA','organization' from dual union all
select 'ABMA','satellite manufacturer' from dual union all
select 'ABMDA','launch agency' from dual union all
select 'ABS','organization' from dual union all
select 'ACAE','organization' from dual union all
select 'ACC','launch agency' from dual union all
select 'ACE','organization' from dual union all
select 'ACES','organization' from dual union all
select 'ACESI','organization' from dual union all
select 'ACF','organization' from dual union all
select 'ACF','satellite manufacturer' from dual union all
select 'ADD','engine/motor manufacturer' from dual union all
select 'ADD','launch agency' from dual union all
select 'ADD','launch vehicle manufacturer' from dual union all
select 'ADD','organization' from dual union all
select 'ADEL','school' from dual union all
select 'ADIG','organization' from dual union all
select 'ADIG','satellite manufacturer' from dual union all
select 'ADSAQ','launch vehicle manufacturer' from dual union all
select 'ADSB','satellite manufacturer' from dual union all
select 'ADSDF','satellite manufacturer' from dual union all
select 'ADSFL','satellite manufacturer' from dual union all
select 'ADSG','launch vehicle manufacturer' from dual union all
select 'ADSL','engine/motor manufacturer' from dual union all
select 'ADSLV','launch vehicle manufacturer' from dual union all
select 'ADSM','satellite manufacturer' from dual union all
select 'ADSO','engine/motor manufacturer' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'ADSO','satellite manufacturer' from dual union all
select 'ADST','satellite manufacturer' from dual union all
select 'ADSUK','satellite manufacturer' from dual union all
select 'AE','launch agency' from dual union all
select 'AE','organization' from dual union all
select 'AEB','launch agency' from dual union all
select 'AEB','organization' from dual union all
select 'AEB','school' from dual union all
select 'AEC','launch agency' from dual union all
select 'AEC','organization' from dual union all
select 'AEC','school' from dual union all
select 'AECAB','satellite manufacturer' from dual union all
select 'AECSG','launch vehicle manufacturer' from dual union all
select 'AEDC','engine/motor manufacturer' from dual union all
select 'AELM','launch vehicle manufacturer' from dual union all
select 'AELM','satellite manufacturer' from dual union all
select 'AEML','launch vehicle manufacturer' from dual union all
select 'AEML','satellite manufacturer' from dual union all
select 'AENUT','launch vehicle manufacturer' from dual union all
select 'AERAR','school' from dual union all
select 'AERIT','launch vehicle manufacturer' from dual union all
select 'AERIT','satellite manufacturer' from dual union all
select 'AERO','engine/motor manufacturer' from dual union all
select 'AERO','launch agency' from dual union all
select 'AERO','organization' from dual union all
select 'AERO','satellite manufacturer' from dual union all
select 'AERO','school' from dual union all
select 'AEROA','satellite manufacturer' from dual union all
select 'AEROAC','satellite manufacturer' from dual union all
select 'AEROACL','satellite manufacturer' from dual union all
select 'AEROL','launch vehicle manufacturer' from dual union all
select 'AEROLR','launch vehicle manufacturer' from dual union all
select 'AERON','satellite manufacturer' from dual union all
select 'AERONM','satellite manufacturer' from dual union all
select 'AEROPH','launch vehicle manufacturer' from dual union all
select 'AES','school' from dual union all
select 'AESCO','satellite manufacturer' from dual union all
select 'AESP','launch agency' from dual union all
select 'AESPC','launch vehicle manufacturer' from dual union all
select 'AESPC','satellite manufacturer' from dual union all
select 'AESPC','school' from dual union all
select 'AETE','launch agency' from dual union all
select 'AETE','meteorological' from dual union all
select 'AF','country' from dual union all
select 'AFAL','school' from dual union all
select 'AFAMC','launch agency' from dual union all
select 'AFAPL','school' from dual union all
select 'AFBMD','launch agency' from dual union all
select 'AFBMD','organization' from dual union all
select 'AFBMO','launch agency' from dual union all
select 'AFBMO2','launch agency' from dual union all
select 'AFCRL','launch agency' from dual union all
select 'AFCRL','launch vehicle manufacturer' from dual union all
select 'AFCRL','organization' from dual union all
select 'AFCRL','satellite manufacturer' from dual union all
select 'AFCRL','school' from dual union all
select 'AFETR','launch agency' from dual union all
select 'AFGL','launch agency' from dual union all
select 'AFGL','school' from dual union all
select 'AFGL2','launch agency' from dual union all
select 'AFIT','launch vehicle manufacturer' from dual union all
select 'AFIT','organization' from dual union all
select 'AFIT','satellite manufacturer' from dual union all
select 'AFMC','launch agency' from dual union all
select 'AFMCIT','organization' from dual union all
select 'AFMCSW','organization' from dual union all
select 'AFMDC','launch agency' from dual union all
select 'AFMTC','launch site' from dual union all
select 'AFORS','organization' from dual union all
select 'AFORS','satellite manufacturer' from dual union all
select 'AFORS','school' from dual union all
select 'AFRCO','organization' from dual union all
select 'AFRL','launch agency' from dual union all
select 'AFRL','organization' from dual union all
select 'AFRL','satellite manufacturer' from dual union all
select 'AFRL','school' from dual union all
select 'AFRLRZ','launch agency' from dual union all
select 'AFRLRZ','school' from dual union all
select 'AFROM','organization' from dual union all
select 'AFSC','launch agency' from dual union all
select 'AFSD','organization' from dual union all
select 'AFSMB','launch agency' from dual union all
select 'AFSMC','launch agency' from dual union all
select 'AFSMC','organization' from dual union all
select 'AFSMS','organization' from dual union all
select 'AFSOC','organization' from dual union all
select 'AFSPC','launch agency' from dual union all
select 'AFSPC','organization' from dual union all
select 'AFSSD','organization' from dual union all
select 'AFSSD2','organization' from dual union all
select 'AFSWC','launch agency' from dual union all
select 'AFSWC','school' from dual union all
select 'AFSWP','school' from dual union all
select 'AFTAC','launch agency' from dual union all
select 'AFTAC','school' from dual union all
select 'AFWDD','launch agency' from dual union all
select 'AFWL','school' from dual union all
select 'AG','country' from dual union all
select 'AGSL','organization' from dual union all
select 'AGUK','country' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'AIL','satellite manufacturer' from dual union all
select 'AJ','engine/motor manufacturer' from dual union all
select 'AJ','launch vehicle manufacturer' from dual union all
select 'AJ','satellite manufacturer' from dual union all
select 'AJCA','engine/motor manufacturer' from dual union all
select 'AJGC','engine/motor manufacturer' from dual union all
select 'AJGC','launch vehicle manufacturer' from dual union all
select 'AJGC','satellite manufacturer' from dual union all
select 'AJP','engine/motor manufacturer' from dual union all
select 'AJRED','engine/motor manufacturer' from dual union all
select 'AJRFL','engine/motor manufacturer' from dual union all
select 'AJRRED','engine/motor manufacturer' from dual union all
select 'AJRSP','engine/motor manufacturer' from dual union all
select 'AJSP','engine/motor manufacturer' from dual union all
select 'AJSP','launch vehicle manufacturer' from dual union all
select 'AKAC','launch agency' from dual union all
select 'AKATT','organization' from dual union all
select 'AKCM','organization' from dual union all
select 'ALB','school' from dual union all
select 'ALC','satellite manufacturer' from dual union all
select 'ALCAF','satellite manufacturer' from dual union all
select 'ALCR','satellite manufacturer' from dual union all
select 'ALCS','launch agency' from dual union all
select 'ALCT','satellite manufacturer' from dual union all
select 'ALEN','satellite manufacturer' from dual union all
select 'ALENR','satellite manufacturer' from dual union all
select 'ALMAN','launch vehicle manufacturer' from dual union all
select 'ALPSP','satellite manufacturer' from dual union all
select 'ALYAH','organization' from dual union all
select 'AM','organization' from dual union all
select 'AM','satellite manufacturer' from dual union all
select 'AMAI','organization' from dual union all
select 'AMBR','organization' from dual union all
select 'AMBR','satellite manufacturer' from dual union all
select 'AMC','organization' from dual union all
select 'AMCEL','engine/motor manufacturer' from dual union all
select 'AMCG','organization' from dual union all
select 'AMCR','organization' from dual union all
select 'AMCS','organization' from dual union all
select 'AMDL','organization' from dual union all
select 'AMDL','satellite manufacturer' from dual union all
select 'AMF','organization' from dual union all
select 'AMF','satellite manufacturer' from dual union all
select 'AMIL','organization' from dual union all
select 'AMIL','satellite manufacturer' from dual union all
select 'AMIT','organization' from dual union all
select 'AMIT','satellite manufacturer' from dual union all
select 'AMLU','organization' from dual union all
select 'AMLU','satellite manufacturer' from dual union all
select 'AMNA','organization' from dual union all
select 'AMNA','satellite manufacturer' from dual union all
select 'AMNL','organization' from dual union all
select 'AMPCS','engine/motor manufacturer' from dual union all
select 'AMPCW','engine/motor manufacturer' from dual union all
select 'AMROC','engine/motor manufacturer' from dual union all
select 'AMROC','launch agency' from dual union all
select 'AMROC','launch vehicle manufacturer' from dual union all
select 'AMROC','school' from dual union all
select 'AMRU','organization' from dual union all
select 'AMRU','satellite manufacturer' from dual union all
select 'AMSAT','satellite manufacturer' from dual union all
select 'AMSC','organization' from dual union all
select 'AMUK','organization' from dual union all
select 'AMVU','organization' from dual union all
select 'AMVU','satellite manufacturer' from dual union all
select 'AN','launch agency' from dual union all
select 'AN','school' from dual union all
select 'ANBEL','organization' from dual union all
select 'ANNAU','satellite manufacturer' from dual union all
select 'ANSAL','launch agency' from dual union all
select 'ANSP','satellite manufacturer' from dual union all
select 'ANTEL','organization' from dual union all
select 'ANTN','country' from dual union all
select 'ANTX','satellite manufacturer' from dual union all
select 'ANUC','satellite manufacturer' from dual union all
select 'AO','country' from dual union all
select 'AOGMC','school' from dual union all
select 'AOMC','launch agency' from dual union all
select 'APAERO','satellite manufacturer' from dual union all
select 'APGC','launch agency' from dual union all
select 'APGC','school' from dual union all
select 'APL','launch agency' from dual union all
select 'APL','launch vehicle manufacturer' from dual union all
select 'APL','organization' from dual union all
select 'APL','satellite manufacturer' from dual union all
select 'APL','school' from dual union all
select 'APLS','satellite manufacturer' from dual union all
select 'APRZ','organization' from dual union all
select 'APT','organization' from dual union all
select 'APW','school' from dual union all
select 'AQ','country' from dual union all
select 'AR','country' from dual union all
select 'ARAB','organization' from dual union all
select 'ARC','engine/motor manufacturer' from dual union all
select 'ARC','launch agency' from dual union all
select 'ARC','organization' from dual union all
select 'ARC','satellite manufacturer' from dual union all
select 'ARC','school' from dual union all
select 'ARDC','launch agency' from dual union all
select 'ARGMA','launch agency' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'ARIG','launch vehicle manufacturer' from dual union all
select 'ARMAD','engine/motor manufacturer' from dual union all
select 'ARMAD','launch agency' from dual union all
select 'ARMAD','launch vehicle manufacturer' from dual union all
select 'ARMAD','school' from dual union all
select 'ARMF','launch agency' from dual union all
select 'ARMT','engine/motor manufacturer' from dual union all
select 'ARMT','launch vehicle manufacturer' from dual union all
select 'ARMT','school' from dual union all
select 'ARPA','launch agency' from dual union all
select 'ARPA','organization' from dual union all
select 'ARR','launch agency' from dual union all
select 'ARR','organization' from dual union all
select 'ARR','school' from dual union all
select 'ARRS','launch agency' from dual union all
select 'ARSAT','organization' from dual union all
select 'ARSL','engine/motor manufacturer' from dual union all
select 'ARSL','launch vehicle manufacturer' from dual union all
select 'ARSL','satellite manufacturer' from dual union all
select 'ARSLO','satellite manufacturer' from dual union all
select 'ARTOPL','organization' from dual union all
select 'ARV','country' from dual union all
select 'ASAL','organization' from dual union all
select 'ASE','school' from dual union all
select 'ASFL','launch vehicle manufacturer' from dual union all
select 'ASGARD','organization' from dual union all
select 'ASI','launch agency' from dual union all
select 'ASI','organization' from dual union all
select 'ASI','school' from dual union all
select 'ASIA','organization' from dual union all
select 'ASID','engine/motor manufacturer' from dual union all
select 'ASL','launch agency' from dual union all
select 'ASL','school' from dual union all
select 'ASTA','satellite manufacturer' from dual union all
select 'ASTD','launch agency' from dual union all
select 'ASTD','satellite manufacturer' from dual union all
select 'ASTD','school' from dual union all
select 'ASTDF','satellite manufacturer' from dual union all
select 'ASTDYN','engine/motor manufacturer' from dual union all
select 'ASTFN','satellite manufacturer' from dual union all
select 'ASTL','engine/motor manufacturer' from dual union all
select 'ASTM','satellite manufacturer' from dual union all
select 'ASTNA','satellite manufacturer' from dual union all
select 'ASTO','engine/motor manufacturer' from dual union all
select 'ASTO','satellite manufacturer' from dual union all
select 'ASTRA','organization' from dual union all
select 'ASTRAN','organization' from dual union all
select 'ASTRV','organization' from dual union all
select 'ASTSC','satellite manufacturer' from dual union all
select 'ASTT','satellite manufacturer' from dual union all
select 'ASTUK','satellite manufacturer' from dual union all
select 'ASU','organization' from dual union all
select 'ASU','satellite manufacturer' from dual union all
select 'AT','country' from dual union all
select 'ATECH','school' from dual union all
select 'ATECHS','launch vehicle manufacturer' from dual union all
select 'ATEF','engine/motor manufacturer' from dual union all
select 'ATEF','launch agency' from dual union all
select 'ATEF','launch vehicle manufacturer' from dual union all
select 'ATI','satellite manufacturer' from dual union all
select 'ATK','engine/motor manufacturer' from dual union all
select 'ATKAB','satellite manufacturer' from dual union all
select 'ATKABL','engine/motor manufacturer' from dual union all
select 'ATKE','engine/motor manufacturer' from dual union all
select 'ATKE','launch vehicle manufacturer' from dual union all
select 'ATKG','satellite manufacturer' from dual union all
select 'ATKG','school' from dual union all
select 'ATKH','engine/motor manufacturer' from dual union all
select 'ATKP','engine/motor manufacturer' from dual union all
select 'ATKS','satellite manufacturer' from dual union all
select 'ATKTN','satellite manufacturer' from dual union all
select 'ATLA','school' from dual union all
select 'ATLC','launch vehicle manufacturer' from dual union all
select 'ATLR','engine/motor manufacturer' from dual union all
select 'ATLR','launch vehicle manufacturer' from dual union all
select 'ATLR','school' from dual union all
select 'ATSB','organization' from dual union all
select 'ATSB','satellite manufacturer' from dual union all
select 'ATT','organization' from dual union all
select 'ATTS','organization' from dual union all
select 'AU','country' from dual union all
select 'AUBU','organization' from dual union all
select 'AUBU','satellite manufacturer' from dual union all
select 'AUCZ','organization' from dual union all
select 'AUDACY','organization' from dual union all
select 'AUSP','satellite manufacturer' from dual union all
select 'AUSS','organization' from dual union all
select 'AVANTI','organization' from dual union all
select 'AVCO','satellite manufacturer' from dual union all
select 'AVIB','engine/motor manufacturer' from dual union all
select 'AVIB','launch vehicle manufacturer' from dual union all
select 'AVIB','satellite manufacturer' from dual union all
select 'AVIO','engine/motor manufacturer' from dual union all
select 'AVIO','satellite manufacturer' from dual union all
select 'AVKO','launch agency' from dual union all
select 'AWS','launch agency' from dual union all
select 'AWS','meteorological' from dual union all
select 'AXEL','satellite manufacturer' from dual union all
select 'AZ','country' from dual union all
select 'AZER','organization' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'B','country' from dual union all
select 'BABTS','organization' from dual union all
select 'BABTS','satellite manufacturer' from dual union all
select 'BAC','launch agency' from dual union all
select 'BAC','launch vehicle manufacturer' from dual union all
select 'BAC','satellite manufacturer' from dual union all
select 'BAC','school' from dual union all
select 'BACB','launch vehicle manufacturer' from dual union all
select 'BAES','satellite manufacturer' from dual union all
select 'BAJ','school' from dual union all
select 'BAJUK','engine/motor manufacturer' from dual union all
select 'BAJUK','launch vehicle manufacturer' from dual union all
select 'BAL','engine/motor manufacturer' from dual union all
select 'BAL','launch agency' from dual union all
select 'BAL','launch vehicle manufacturer' from dual union all
select 'BAL','school' from dual union all
select 'BALL','satellite manufacturer' from dual union all
select 'BALM','satellite manufacturer' from dual union all
select 'BAS','school' from dual union all
select 'BAT','country' from dual union all
select 'BAUM','organization' from dual union all
select 'BAUM','satellite manufacturer' from dual union all
select 'BAYI','organization' from dual union all
select 'BB','country' from dual union all
select 'BBUK','country' from dual union all
select 'BCSCL','organization' from dual union all
select 'BD','country' from dual union all
select 'BELINT','organization' from dual union all
select 'BELL','engine/motor manufacturer' from dual union all
select 'BELT','engine/motor manufacturer' from dual union all
select 'BENDX','satellite manufacturer' from dual union all
select 'BERG','school' from dual union all
select 'BERN','school' from dual union all
select 'BG','country' from dual union all
select 'BGN','country' from dual union all
select 'BGU','satellite manufacturer' from dual union all
select 'BHSJ','organization' from dual union all
select 'BIGEL','organization' from dual union all
select 'BIGEL','satellite manufacturer' from dual union all
select 'BINA','organization' from dual union all
select 'BIR','school' from dual union all
select 'BKSA','organization' from dual union all
select 'BLMIT','organization' from dual union all
select 'BLOR','engine/motor manufacturer' from dual union all
select 'BLOR','launch vehicle manufacturer' from dual union all
select 'BLORTX','engine/motor manufacturer' from dual union all
select 'BLS','launch agency' from dual union all
select 'BLS','organization' from dual union all
select 'BM','country' from dual union all
select 'BMBW','organization' from dual union all
select 'BMDO','launch agency' from dual union all
select 'BMDO','launch vehicle manufacturer' from dual union all
select 'BMDO','organization' from dual union all
select 'BMDO','school' from dual union all
select 'BME','organization' from dual union all
select 'BME','satellite manufacturer' from dual union all
select 'BNAP','launch vehicle manufacturer' from dual union all
select 'BNAP','satellite manufacturer' from dual union all
select 'BNC','organization' from dual union all
select 'BNSC','organization' from dual union all
select 'BO','country' from dual union all
select 'BOC','launch vehicle manufacturer' from dual union all
select 'BOCH','school' from dual union all
select 'BOCP','satellite manufacturer' from dual union all
select 'BOES','satellite manufacturer' from dual union all
select 'BOHB','launch vehicle manufacturer' from dual union all
select 'BOHB','organization' from dual union all
select 'BOHB','satellite manufacturer' from dual union all
select 'BOHPS','satellite manufacturer' from dual union all
select 'BOHSM','satellite manufacturer' from dual union all
select 'BOHSV','launch vehicle manufacturer' from dual union all
select 'BOHSV','satellite manufacturer' from dual union all
select 'BOKT','launch vehicle manufacturer' from dual union all
select 'BOKT','satellite manufacturer' from dual union all
select 'BOL','organization' from dual union all
select 'BOL','satellite manufacturer' from dual union all
select 'BOL','school' from dual union all
select 'BOLK','satellite manufacturer' from dual union all
select 'BOMI','launch vehicle manufacturer' from dual union all
select 'BONN','school' from dual union all
select 'BONUM','organization' from dual union all
select 'BOOZA','organization' from dual union all
select 'BOOZA','satellite manufacturer' from dual union all
select 'BOPBL','launch vehicle manufacturer' from dual union all
select 'BOPW','satellite manufacturer' from dual union all
select 'BOSB','satellite manufacturer' from dual union all
select 'BOSL','launch agency' from dual union all
select 'BOSL','organization' from dual union all
select 'BOTUL','satellite manufacturer' from dual union all
select 'BOUT','launch vehicle manufacturer' from dual union all
select 'BPD','engine/motor manufacturer' from dual union all
select 'BPD','launch vehicle manufacturer' from dual union all
select 'BR','country' from dual union all
select 'BRAC','satellite manufacturer' from dual union all
select 'BRAE','engine/motor manufacturer' from dual union all
select 'BREC','engine/motor manufacturer' from dual union all
select 'BREI','school' from dual union all
select 'BREM','organization' from dual union all
select 'BREM','satellite manufacturer' from dual union all
select 'BRIS','satellite manufacturer' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'BRIS','school' from dual union all
select 'BRL','launch agency' from dual union all
select 'BRL','launch vehicle manufacturer' from dual union all
select 'BRL','school' from dual union all
select 'BROWN','satellite manufacturer' from dual union all
select 'BRSW','school' from dual union all
select 'BRUNSW','launch vehicle manufacturer' from dual union all
select 'BRVCC','school' from dual union all
select 'BS','country' from dual union all
select 'BSAT','organization' from dual union all
select 'BSB','organization' from dual union all
select 'BSC','organization' from dual union all
select 'BSEL','engine/motor manufacturer' from dual union all
select 'BSELC','engine/motor manufacturer' from dual union all
select 'BSELL','engine/motor manufacturer' from dual union all
select 'BST','organization' from dual union all
select 'BST','satellite manufacturer' from dual union all
select 'BST','school' from dual union all
select 'BTL','launch agency' from dual union all
select 'BTL','satellite manufacturer' from dual union all
select 'BTRC','organization' from dual union all
select 'BU','school' from dual union all
select 'BUAA','satellite manufacturer' from dual union all
select 'BUCU','organization' from dual union all
select 'BUCU','satellite manufacturer' from dual union all
select 'BUSEK','engine/motor manufacturer' from dual union all
select 'BWB','organization' from dual union all
select 'BWEHR','launch agency' from dual union all
select 'BWYIC','engine/motor manufacturer' from dual union all
select 'BWYIC','satellite manufacturer' from dual union all
select 'BY','country' from dual union all
select 'CA','country' from dual union all
select 'CAL','school' from dual union all
select 'CALG','school' from dual union all
select 'CALP','organization' from dual union all
select 'CALP','satellite manufacturer' from dual union all
select 'CALP','school' from dual union all
select 'CALT','engine/motor manufacturer' from dual union all
select 'CALT','launch agency' from dual union all
select 'CALT','launch vehicle manufacturer' from dual union all
select 'CALT','satellite manufacturer' from dual union all
select 'CAM','school' from dual union all
select 'CAMSAT','organization' from dual union all
select 'CAMSAT','satellite manufacturer' from dual union all
select 'CANON','satellite manufacturer' from dual union all
select 'CANOP','organization' from dual union all
select 'CANOP','satellite manufacturer' from dual union all
select 'CAPSP','organization' from dual union all
select 'CARDE','launch agency' from dual union all
select 'CARDE','launch vehicle manufacturer' from dual union all
select 'CARDE','school' from dual union all
select 'CARDU','satellite manufacturer' from dual union all
select 'CARTH','satellite manufacturer' from dual union all
select 'CASA','satellite manufacturer' from dual union all
select 'CASAG','satellite manufacturer' from dual union all
select 'CASC','launch agency' from dual union all
select 'CASC','organization' from dual union all
select 'CASC','satellite manufacturer' from dual union all
select 'CASC6A','engine/motor manufacturer' from dual union all
select 'CASC6A1','engine/motor manufacturer' from dual union all
select 'CASDN','launch agency' from dual union all
select 'CASIC4A','launch vehicle manufacturer' from dual union all
select 'CASIOM','organization' from dual union all
select 'CAST','organization' from dual union all
select 'CAST','satellite manufacturer' from dual union all
select 'CAST','school' from dual union all
select 'CASTBZ','satellite manufacturer' from dual union all
select 'CASTIX','organization' from dual union all
select 'CATHU','satellite manufacturer' from dual union all
select 'CATON','satellite manufacturer' from dual union all
select 'CBKPL','organization' from dual union all
select 'CBKPL','satellite manufacturer' from dual union all
select 'CBMET','school' from dual union all
select 'CD','country' from dual union all
select 'CEA','school' from dual union all
select 'CEL','launch agency' from dual union all
select 'CEL','launch site' from dual union all
select 'CEL','school' from dual union all
select 'CELES','launch vehicle manufacturer' from dual union all
select 'CELML','launch site' from dual union all
select 'CERMA','school' from dual union all
select 'CERN','organization' from dual union all
select 'CERN','satellite manufacturer' from dual union all
select 'CESR','school' from dual union all
select 'CFTH','school' from dual union all
select 'CFTHB','engine/motor manufacturer' from dual union all
select 'CFTHB','launch vehicle manufacturer' from dual union all
select 'CGS','satellite manufacturer' from dual union all
select 'CGSO','satellite manufacturer' from dual union all
select 'CGSTL','organization' from dual union all
select 'CH','country' from dual union all
select 'CHAIL','satellite manufacturer' from dual union all
select 'CHAIL','school' from dual union all
select 'CHFEN','launch agency' from dual union all
select 'CHFEN','school' from dual union all
select 'CHIBA','organization' from dual union all
select 'CHIBA','satellite manufacturer' from dual union all
select 'CHIC','school' from dual union all
select 'CHIS','organization' from dual union all
select 'CHISA','organization' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'CHISAE','organization' from dual union all
select 'CHORI','organization' from dual union all
select 'CHOSU','organization' from dual union all
select 'CHRY','launch vehicle manufacturer' from dual union all
select 'CI','country' from dual union all
select 'CIA','organization' from dual union all
select 'CIA','satellite manufacturer' from dual union all
select 'CIEL','organization' from dual union all
select 'CIFAS','satellite manufacturer' from dual union all
select 'CIFTQ','school' from dual union all
select 'CIN','school' from dual union all
select 'CIOMP','satellite manufacturer' from dual union all
select 'CISIR','school' from dual union all
select 'CIT','engine/motor manufacturer' from dual union all
select 'CIT','satellite manufacturer' from dual union all
select 'CIT','school' from dual union all
select 'CITED','engine/motor manufacturer' from dual union all
select 'CITED','launch agency' from dual union all
select 'CITED','launch vehicle manufacturer' from dual union all
select 'CITED','school' from dual union all
select 'CITEF','engine/motor manufacturer' from dual union all
select 'CITEF','launch agency' from dual union all
select 'CITEF','launch vehicle manufacturer' from dual union all
select 'CITEF','school' from dual union all
select 'CK','country' from dual union all
select 'CL','country' from dual union all
select 'CLAAC','satellite manufacturer' from dual union all
select 'CLARK','satellite manufacturer' from dual union all
select 'CLASP','organization' from dual union all
select 'CLEM','school' from dual union all
select 'CLSR','school' from dual union all
select 'CLYDE','satellite manufacturer' from dual union all
select 'CMMB','organization' from dual union all
select 'CMSEO','organization' from dual union all
select 'CN','country' from dual union all
select 'CNAE','launch agency' from dual union all
select 'CNES','launch agency' from dual union all
select 'CNES','launch vehicle manufacturer' from dual union all
select 'CNES','organization' from dual union all
select 'CNES','satellite manufacturer' from dual union all
select 'CNES','school' from dual union all
select 'CNEST','satellite manufacturer' from dual union all
select 'CNET','launch agency' from dual union all
select 'CNET','school' from dual union all
select 'CNIE','launch agency' from dual union all
select 'CNIE','school' from dual union all
select 'CNR','organization' from dual union all
select 'CNR','satellite manufacturer' from dual union all
select 'CNRS','school' from dual union all
select 'CNSA','organization' from dual union all
select 'CNSA','satellite manufacturer' from dual union all
select 'CNSAS','organization' from dual union all
select 'CNTS','organization' from dual union all
select 'CNU','organization' from dual union all
select 'CO','country' from dual union all
select 'COEP','organization' from dual union all
select 'COLC','organization' from dual union all
select 'COLE','launch agency' from dual union all
select 'COLE','launch vehicle manufacturer' from dual union all
select 'COLEA','launch vehicle manufacturer' from dual union all
select 'COLEAR','launch vehicle manufacturer' from dual union all
select 'COLP','country' from dual union all
select 'COM','organization' from dual union all
select 'COMAH','satellite manufacturer' from dual union all
select 'COMDEV','organization' from dual union all
select 'CONAE','launch agency' from dual union all
select 'CONAE','organization' from dual union all
select 'CONAE','school' from dual union all
select 'CONIE','school' from dual union all
select 'CONT','organization' from dual union all
select 'CONTR','engine/motor manufacturer' from dual union all
select 'CONTR','launch vehicle manufacturer' from dual union all
select 'CONTR','satellite manufacturer' from dual union all
select 'CONTS','satellite manufacturer' from dual union all
select 'CONV','launch vehicle manufacturer' from dual union all
select 'CONVPO','launch vehicle manufacturer' from dual union all
select 'COOP','engine/motor manufacturer' from dual union all
select 'COOP','launch vehicle manufacturer' from dual union all
select 'COPERN','organization' from dual union all
select 'CORAT','satellite manufacturer' from dual union all
select 'CORD','organization' from dual union all
select 'CORN','organization' from dual union all
select 'CORN','satellite manufacturer' from dual union all
select 'CORN','school' from dual union all
select 'COSG','organization' from dual union all
select 'COSG','satellite manufacturer' from dual union all
select 'COSG','school' from dual union all
select 'COSM','satellite manufacturer' from dual union all
select 'COSMOG','organization' from dual union all
select 'COSMOG','satellite manufacturer' from dual union all
select 'CPUT','organization' from dual union all
select 'CPUT','satellite manufacturer' from dual union all
select 'CR','country' from dual union all
select 'CRA','launch agency' from dual union all
select 'CRA','satellite manufacturer' from dual union all
select 'CRA','school' from dual union all
select 'CRC','organization' from dual union all
select 'CRC','satellite manufacturer' from dual union all
select 'CRC','school' from dual union all
select 'CRCSS','organization' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'CRLI','launch agency' from dual union all
select 'CRLI','school' from dual union all
select 'CRR','school' from dual union all
select 'CRS','organization' from dual union all
select 'CRSEA','organization' from dual union all
select 'CRSF','launch agency' from dual union all
select 'CRTS','organization' from dual union all
select 'CSA','launch agency' from dual union all
select 'CSA','organization' from dual union all
select 'CSA','satellite manufacturer' from dual union all
select 'CSA','school' from dual union all
select 'CSD','engine/motor manufacturer' from dual union all
select 'CSFR','country' from dual union all
select 'CSG','launch site' from dual union all
select 'CSG','school' from dual union all
select 'CSICE','organization' from dual union all
select 'CSIMP','school' from dual union all
select 'CSIR','organization' from dual union all
select 'CSSAR','launch agency' from dual union all
select 'CSSAR','launch vehicle manufacturer' from dual union all
select 'CSSAR','school' from dual union all
select 'CSSR','country' from dual union all
select 'CSU','satellite manufacturer' from dual union all
select 'CSUN','satellite manufacturer' from dual union all
select 'CSXT','engine/motor manufacturer' from dual union all
select 'CSXT','launch agency' from dual union all
select 'CSXT','launch vehicle manufacturer' from dual union all
select 'CSXT','school' from dual union all
select 'CTA','organization' from dual union all
select 'CTA','satellite manufacturer' from dual union all
select 'CTEL','organization' from dual union all
select 'CTYK','organization' from dual union all
select 'CUBESP','organization' from dual union all
select 'CUBIC','satellite manufacturer' from dual union all
select 'CULH','school' from dual union all
select 'CUNYME','organization' from dual union all
select 'CUNYME','satellite manufacturer' from dual union all
select 'CYM','country' from dual union all
select 'CZ','country' from dual union all
select 'D','country' from dual union all
select 'DAC','engine/motor manufacturer' from dual union all
select 'DAC','launch vehicle manufacturer' from dual union all
select 'DAC','school' from dual union all
select 'DACC','launch vehicle manufacturer' from dual union all
select 'DACM','launch vehicle manufacturer' from dual union all
select 'DARA','organization' from dual union all
select 'DARE','engine/motor manufacturer' from dual union all
select 'DARE','launch agency' from dual union all
select 'DARE','launch vehicle manufacturer' from dual union all
select 'DARE','school' from dual union all
select 'DARPA','launch agency' from dual union all
select 'DARPA','organization' from dual union all
select 'DARPA','school' from dual union all
select 'DARPA2','organization' from dual union all
select 'DART','school' from dual union all
select 'DASA','launch agency' from dual union all
select 'DASA','organization' from dual union all
select 'DASA','school' from dual union all
select 'DASAF','satellite manufacturer' from dual union all
select 'DASAO','engine/motor manufacturer' from dual union all
select 'DASAO','satellite manufacturer' from dual union all
select 'DASARI','organization' from dual union all
select 'DASARI','satellite manufacturer' from dual union all
select 'DASARI','school' from dual union all
select 'DASBF','satellite manufacturer' from dual union all
select 'DASCF','satellite manufacturer' from dual union all
select 'DASST','launch agency' from dual union all
select 'DASST','launch vehicle manufacturer' from dual union all
select 'DAUR','organization' from dual union all
select 'DAUR','satellite manufacturer' from dual union all
select 'DBP','organization' from dual union all
select 'DBSD','organization' from dual union all
select 'DCA','organization' from dual union all
select 'DD','country' from dual union all
select 'DEHAVE','engine/motor manufacturer' from dual union all
select 'DEHAVS','launch vehicle manufacturer' from dual union all
select 'DEHAVS','satellite manufacturer' from dual union all
select 'DEHAVT','satellite manufacturer' from dual union all
select 'DEIMOS','organization' from dual union all
select 'DELFT','organization' from dual union all
select 'DELFT','satellite manufacturer' from dual union all
select 'DENV','satellite manufacturer' from dual union all
select 'DENV','school' from dual union all
select 'DERA','organization' from dual union all
select 'DERA','satellite manufacturer' from dual union all
select 'DFF','school' from dual union all
select 'DFH','organization' from dual union all
select 'DFH','satellite manufacturer' from dual union all
select 'DFHS','organization' from dual union all
select 'DFHS','satellite manufacturer' from dual union all
select 'DFHZ','satellite manufacturer' from dual union all
select 'DFRC','school' from dual union all
select 'DFVLR','launch agency' from dual union all
select 'DFVLR','organization' from dual union all
select 'DFVLR','school' from dual union all
select 'DGA','launch agency' from dual union all
select 'DGA','organization' from dual union all
select 'DGA','school' from dual union all
select 'DGAEM','launch site' from dual union all
select 'DGGROW','organization' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'DGLO','organization' from dual union all
select 'DHSST','organization' from dual union all
select 'DISA','organization' from dual union all
select 'DISH','organization' from dual union all
select 'DK','country' from dual union all
select 'DLR','launch agency' from dual union all
select 'DLR','organization' from dual union all
select 'DLR','satellite manufacturer' from dual union all
select 'DLR','school' from dual union all
select 'DLRB','satellite manufacturer' from dual union all
select 'DLRRD','organization' from dual union all
select 'DLRRD','satellite manufacturer' from dual union all
select 'DMA','launch agency' from dual union all
select 'DMA','school' from dual union all
select 'DMC','organization' from dual union all
select 'DMI','launch agency' from dual union all
select 'DMI','organization' from dual union all
select 'DMI','school' from dual union all
select 'DML','country' from dual union all
select 'DNA','launch agency' from dual union all
select 'DNA','launch vehicle manufacturer' from dual union all
select 'DNA','school' from dual union all
select 'DND','organization' from dual union all
select 'DNE','satellite manufacturer' from dual union all
select 'DOD','launch agency' from dual union all
select 'DOE','satellite manufacturer' from dual union all
select 'DOE','school' from dual union all
select 'DORN','satellite manufacturer' from dual union all
select 'DORN','school' from dual union all
select 'DOSH','school' from dual union all
select 'DR','country' from dual union all
select 'DRA','organization' from dual union all
select 'DRA','satellite manufacturer' from dual union all
select 'DRDC','organization' from dual union all
select 'DRDL','launch agency' from dual union all
select 'DRDL','satellite manufacturer' from dual union all
select 'DRDO','engine/motor manufacturer' from dual union all
select 'DRDO','launch agency' from dual union all
select 'DRDO','launch vehicle manufacturer' from dual union all
select 'DRDO','satellite manufacturer' from dual union all
select 'DREXEL','organization' from dual union all
select 'DREXEL','satellite manufacturer' from dual union all
select 'DRG','engine/motor manufacturer' from dual union all
select 'DRG','launch agency' from dual union all
select 'DRG','launch vehicle manufacturer' from dual union all
select 'DRG','school' from dual union all
select 'DRTE','launch agency' from dual union all
select 'DRTE','organization' from dual union all
select 'DRTE','school' from dual union all
select 'DSI','satellite manufacturer' from dual union all
select 'DSIR','organization' from dual union all
select 'DSNC','organization' from dual union all
select 'DSRI','school' from dual union all
select 'DSSG','satellite manufacturer' from dual union all
select 'DSTO','launch agency' from dual union all
select 'DSTO','school' from dual union all
select 'DTI','launch vehicle manufacturer' from dual union all
select 'DTU','organization' from dual union all
select 'DTU','satellite manufacturer' from dual union all
select 'DTU','school' from dual union all
select 'DTV','organization' from dual union all
select 'DTVA','organization' from dual union all
select 'DTVLA','organization' from dual union all
select 'DUCOMM','satellite manufacturer' from dual union all
select 'DUD','school' from dual union all
select 'DUTCH','satellite manufacturer' from dual union all
select 'DUTEC','satellite manufacturer' from dual union all
select 'DUTH','satellite manufacturer' from dual union all
select 'DX','country' from dual union all
select 'DYN','satellite manufacturer' from dual union all
select 'DYNT','satellite manufacturer' from dual union all
select 'DZ','country' from dual union all
select 'E','country' from dual union all
select 'EAC','engine/motor manufacturer' from dual union all
select 'EAC','launch vehicle manufacturer' from dual union all
select 'EAC','school' from dual union all
select 'EADS','launch vehicle manufacturer' from dual union all
select 'EADS','satellite manufacturer' from dual union all
select 'EADS','school' from dual union all
select 'EADSA','launch vehicle manufacturer' from dual union all
select 'EADSB','organization' from dual union all
select 'EADSB','satellite manufacturer' from dual union all
select 'EADSCA','satellite manufacturer' from dual union all
select 'EADSG','launch vehicle manufacturer' from dual union all
select 'EADSL','launch vehicle manufacturer' from dual union all
select 'EADSO','engine/motor manufacturer' from dual union all
select 'EADSO','satellite manufacturer' from dual union all
select 'EARI','organization' from dual union all
select 'EARTH','astronomical polity' from dual union all
select 'EASF','organization' from dual union all
select 'EASF','satellite manufacturer' from dual union all
select 'EC','country' from dual union all
select 'ECHO','organization' from dual union all
select 'ECHOC','organization' from dual union all
select 'ECHOM','organization' from dual union all
select 'ECLIP','satellite manufacturer' from dual union all
select 'EDLS','satellite manufacturer' from dual union all
select 'EE','country' from dual union all
select 'EELEC','launch vehicle manufacturer' from dual union all
select 'EER','launch agency' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'EER','organization' from dual union all
select 'EERSSI','launch vehicle manufacturer' from dual union all
select 'EG','country' from dual union all
select 'EGA','launch agency' from dual union all
select 'EGAO','engine/motor manufacturer' from dual union all
select 'EGAO','launch agency' from dual union all
select 'EGAO','launch vehicle manufacturer' from dual union all
select 'EGAO','school' from dual union all
select 'EGS','satellite manufacturer' from dual union all
select 'EHNS','organization' from dual union all
select 'EIAST','organization' from dual union all
select 'ELDEC','organization' from dual union all
select 'ELDEC','satellite manufacturer' from dual union all
select 'ELDO','launch agency' from dual union all
select 'ELDO','launch vehicle manufacturer' from dual union all
select 'ELDO','organization' from dual union all
select 'ELLT','satellite manufacturer' from dual union all
select 'ELUS','launch agency' from dual union all
select 'ELV','launch vehicle manufacturer' from dual union all
select 'ELYS','satellite manufacturer' from dual union all
select 'EMASH','engine/motor manufacturer' from dual union all
select 'EMBR','organization' from dual union all
select 'EMST','satellite manufacturer' from dual union all
select 'ENDUR','satellite manufacturer' from dual union all
select 'EOS','school' from dual union all
select 'EOSAT','organization' from dual union all
select 'EOSAU','organization' from dual union all
select 'EOSAU','satellite manufacturer' from dual union all
select 'EPFL','organization' from dual union all
select 'EPFL','satellite manufacturer' from dual union all
select 'ERAU','engine/motor manufacturer' from dual union all
select 'ERAU','launch agency' from dual union all
select 'ERAU','launch vehicle manufacturer' from dual union all
select 'ERAU','school' from dual union all
select 'ERDA','launch agency' from dual union all
select 'ERDA','school' from dual union all
select 'ERDL','launch agency' from dual union all
select 'ERDL','organization' from dual union all
select 'ERDL','school' from dual union all
select 'ERNO','engine/motor manufacturer' from dual union all
select 'ERNO','launch vehicle manufacturer' from dual union all
select 'ERNO','school' from dual union all
select 'ESA','launch agency' from dual union all
select 'ESA','organization' from dual union all
select 'ESA','school' from dual union all
select 'ESCN','country' from dual union all
select 'ESIE','organization' from dual union all
select 'ESIE','satellite manufacturer' from dual union all
select 'ESL','satellite manufacturer' from dual union all
select 'ESL1','satellite manufacturer' from dual union all
select 'ESRO','launch agency' from dual union all
select 'ESRO','organization' from dual union all
select 'ESRO','school' from dual union all
select 'ESSA','launch agency' from dual union all
select 'ESSA','organization' from dual union all
select 'ESSA','school' from dual union all
select 'ESSP','organization' from dual union all
select 'ESSP','satellite manufacturer' from dual union all
select 'ESTEC','launch agency' from dual union all
select 'ESTEC','organization' from dual union all
select 'ESTEC','satellite manufacturer' from dual union all
select 'ESTEC','school' from dual union all
select 'ESYS','satellite manufacturer' from dual union all
select 'ETAC','school' from dual union all
select 'ETEC','organization' from dual union all
select 'EUMET','organization' from dual union all
select 'EUPST','organization' from dual union all
select 'EURAS','organization' from dual union all
select 'EUROK','organization' from dual union all
select 'EUROL','launch agency' from dual union all
select 'EUROL','school' from dual union all
select 'EUROPK','engine/motor manufacturer' from dual union all
select 'EURSA','organization' from dual union all
select 'EURSG','organization' from dual union all
select 'EUTAM','organization' from dual union all
select 'EUTEL','organization' from dual union all
select 'EUTSA','organization' from dual union all
select 'EWAT','organization' from dual union all
select 'EXA','organization' from dual union all
select 'EXA','satellite manufacturer' from dual union all
select 'EXEA','organization' from dual union all
select 'EXOSA','launch vehicle manufacturer' from dual union all
select 'F','country' from dual union all
select 'FACH','organization' from dual union all
select 'FAI','organization' from dual union all
select 'FAI','satellite manufacturer' from dual union all
select 'FAIR','satellite manufacturer' from dual union all
select 'FAIRA','launch vehicle manufacturer' from dual union all
select 'FAIRA','satellite manufacturer' from dual union all
select 'FAIRH','satellite manufacturer' from dual union all
select 'FAIRHG','satellite manufacturer' from dual union all
select 'FAIRS','satellite manufacturer' from dual union all
select 'FAKEL','engine/motor manufacturer' from dual union all
select 'FAKEL','launch vehicle manufacturer' from dual union all
select 'FAKELO','engine/motor manufacturer' from dual union all
select 'FATAH','engine/motor manufacturer' from dual union all
select 'FATAH','launch vehicle manufacturer' from dual union all
select 'FHWN','satellite manufacturer' from dual union all
select 'FI','country' from dual union all
select 'FIAN','launch agency' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'FIAN','launch vehicle manufacturer' from dual union all
select 'FIAN','organization' from dual union all
select 'FIAN','school' from dual union all
select 'FIAT','satellite manufacturer' from dual union all
select 'FIATA','engine/motor manufacturer' from dual union all
select 'FIATA','satellite manufacturer' from dual union all
select 'FING','organization' from dual union all
select 'FING','satellite manufacturer' from dual union all
select 'FIRES','satellite manufacturer' from dual union all
select 'FIRETR','satellite manufacturer' from dual union all
select 'FIT','school' from dual union all
select 'FKA','launch agency' from dual union all
select 'FKA','organization' from dual union all
select 'FKG','launch agency' from dual union all
select 'FKG1','launch agency' from dual union all
select 'FKG2','launch agency' from dual union all
select 'FKGR11','launch agency' from dual union all
select 'FKGR12','launch agency' from dual union all
select 'FKGR13','launch agency' from dual union all
select 'FKGR21','launch agency' from dual union all
select 'FKGR22','launch agency' from dual union all
select 'FLASP','launch agency' from dual union all
select 'FLASP','school' from dual union all
select 'FMI','organization' from dual union all
select 'FOKK','satellite manufacturer' from dual union all
select 'FOKKS','satellite manufacturer' from dual union all
select 'FOKKSS','satellite manufacturer' from dual union all
select 'FORDA','satellite manufacturer' from dual union all
select 'FORDAN','launch vehicle manufacturer' from dual union all
select 'FORDP','satellite manufacturer' from dual union all
select 'FORNB','satellite manufacturer' from dual union all
select 'FORSK','launch agency' from dual union all
select 'FPTU','organization' from dual union all
select 'FPTU','satellite manufacturer' from dual union all
select 'FRANK','school' from dual union all
select 'FRASC','school' from dual union all
select 'FRC','launch vehicle manufacturer' from dual union all
select 'FRC','school' from dual union all
select 'FREIB','school' from dual union all
select 'FSB','organization' from dual union all
select 'FSU','school' from dual union all
select 'FTB','launch site' from dual union all
select 'FTEL','organization' from dual union all
select 'FUDAN','organization' from dual union all
select 'FUKU','organization' from dual union all
select 'FUKU','satellite manufacturer' from dual union all
select 'FUSEI','engine/motor manufacturer' from dual union all
select 'FUTA','satellite manufacturer' from dual union all
select 'G2P','engine/motor manufacturer' from dual union all
select 'GAHSV','satellite manufacturer' from dual union all
select 'GAISH','organization' from dual union all
select 'GASL','satellite manufacturer' from dual union all
select 'GASL','school' from dual union all
select 'GASLA','satellite manufacturer' from dual union all
select 'GATCO','organization' from dual union all
select 'GATGD','organization' from dual union all
select 'GATGU','organization' from dual union all
select 'GATGU2','organization' from dual union all
select 'GATOM','organization' from dual union all
select 'GAU4','launch agency' from dual union all
select 'GAUSS','organization' from dual union all
select 'GAUSS','satellite manufacturer' from dual union all
select 'GAZ1','launch vehicle manufacturer' from dual union all
select 'GAZ1','satellite manufacturer' from dual union all
select 'GAZC','organization' from dual union all
select 'GAZS','organization' from dual union all
select 'GB5','launch agency' from dual union all
select 'GCA','school' from dual union all
select 'GCDX','organization' from dual union all
select 'GCR','engine/motor manufacturer' from dual union all
select 'GCRG','engine/motor manufacturer' from dual union all
select 'GDA','launch vehicle manufacturer' from dual union all
select 'GDC','satellite manufacturer' from dual union all
select 'GDCLS','launch agency' from dual union all
select 'GDG','satellite manufacturer' from dual union all
select 'GDOTS','engine/motor manufacturer' from dual union all
select 'GDPO','launch vehicle manufacturer' from dual union all
select 'GE','country' from dual union all
select 'GECAP','organization' from dual union all
select 'GEEW','satellite manufacturer' from dual union all
select 'GEMSD','satellite manufacturer' from dual union all
select 'GEMTS','engine/motor manufacturer' from dual union all
select 'GEMTS','launch agency' from dual union all
select 'GEMTS','launch vehicle manufacturer' from dual union all
select 'GEMTS','school' from dual union all
select 'GEOOPT','organization' from dual union all
select 'GERSD','satellite manufacturer' from dual union all
select 'GESNY','engine/motor manufacturer' from dual union all
select 'GEVF','satellite manufacturer' from dual union all
select 'GEYE','organization' from dual union all
select 'GFW','satellite manufacturer' from dual union all
select 'GFZ','organization' from dual union all
select 'GGPEN','organization' from dual union all
select 'GH','country' from dual union all
select 'GHALAM','organization' from dual union all
select 'GHG','organization' from dual union all
select 'GHJ','organization' from dual union all
select 'GI','country' from dual union all
select 'GICZ','satellite manufacturer' from dual union all
select 'GICZ','school' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'GIMR','organization' from dual union all
select 'GISTDA','organization' from dual union all
select 'GIT','school' from dual union all
select 'GL','country' from dual union all
select 'GLOB','organization' from dual union all
select 'GMS','engine/motor manufacturer' from dual union all
select 'GMS','launch agency' from dual union all
select 'GMS','organization' from dual union all
select 'GMS','school' from dual union all
select 'GOBBZ','satellite manufacturer' from dual union all
select 'GOBME','satellite manufacturer' from dual union all
select 'GOEOS','satellite manufacturer' from dual union all
select 'GOMSP','organization' from dual union all
select 'GOMSP','satellite manufacturer' from dual union all
select 'GOS','organization' from dual union all
select 'GOYR','satellite manufacturer' from dual union all
select 'GOYR','school' from dual union all
select 'GOYRP','satellite manufacturer' from dual union all
select 'GPI','satellite manufacturer' from dual union all
select 'GPKS','organization' from dual union all
select 'GR','country' from dual union all
select 'GRC','engine/motor manufacturer' from dual union all
select 'GRI','school' from dual union all
select 'GRU','organization' from dual union all
select 'GRUM','launch vehicle manufacturer' from dual union all
select 'GRUM','satellite manufacturer' from dual union all
select 'GRUR','organization' from dual union all
select 'GSAEU','organization' from dual union all
select 'GSFC','launch agency' from dual union all
select 'GSFC','launch vehicle manufacturer' from dual union all
select 'GSFC','organization' from dual union all
select 'GSFC','satellite manufacturer' from dual union all
select 'GSFC','school' from dual union all
select 'GSOC','school' from dual union all
select 'GTC','satellite manufacturer' from dual union all
select 'GTE','organization' from dual union all
select 'GU','country' from dual union all
select 'GUF','country' from dual union all
select 'GUKOS','launch agency' from dual union all
select 'GUKOS','organization' from dual union all
select 'GUKOSR','launch agency' from dual union all
select 'GUKOSR','organization' from dual union all
select 'GURVO','launch agency' from dual union all
select 'GURVO','organization' from dual union all
select 'GWEST','satellite manufacturer' from dual union all
select 'GWHYZ','organization' from dual union all
select 'GYZ','organization' from dual union all
select 'HAC','launch agency' from dual union all
select 'HADC','launch agency' from dual union all
select 'HADC','launch site' from dual union all
select 'HADC','school' from dual union all
select 'HAFB','launch site' from dual union all
select 'HAL','launch vehicle manufacturer' from dual union all
select 'HAL','satellite manufacturer' from dual union all
select 'HAL5','engine/motor manufacturer' from dual union all
select 'HAL5','launch agency' from dual union all
select 'HAL5','launch vehicle manufacturer' from dual union all
select 'HAL5','school' from dual union all
select 'HAM','school' from dual union all
select 'HANAU','organization' from dual union all
select 'HANAU','satellite manufacturer' from dual union all
select 'HANWHA','launch vehicle manufacturer' from dual union all
select 'HAO','school' from dual union all
select 'HARB','organization' from dual union all
select 'HARB','satellite manufacturer' from dual union all
select 'HARK','satellite manufacturer' from dual union all
select 'HARP','launch agency' from dual union all
select 'HARP','school' from dual union all
select 'HASTD','engine/motor manufacturer' from dual union all
select 'HASTD','satellite manufacturer' from dual union all
select 'HASUN','satellite manufacturer' from dual union all
select 'HAVEL','satellite manufacturer' from dual union all
select 'HAWK','organization' from dual union all
select 'HAWK','satellite manufacturer' from dual union all
select 'HCG','organization' from dual union all
select 'HCI','organization' from dual union all
select 'HCO','satellite manufacturer' from dual union all
select 'HCO','school' from dual union all
select 'HDOS','satellite manufacturer' from dual union all
select 'HE360','organization' from dual union all
select 'HEAD','organization' from dual union all
select 'HEID','satellite manufacturer' from dual union all
select 'HELSA','organization' from dual union all
select 'HERAK','engine/motor manufacturer' from dual union all
select 'HERAKS','engine/motor manufacturer' from dual union all
select 'HERC','engine/motor manufacturer' from dual union all
select 'HERCM','engine/motor manufacturer' from dual union all
select 'HERCR','engine/motor manufacturer' from dual union all
select 'HEXI','engine/motor manufacturer' from dual union all
select 'HEXI4A','engine/motor manufacturer' from dual union all
select 'HGS','organization' from dual union all
select 'HHJ','satellite manufacturer' from dual union all
select 'HIBER','organization' from dual union all
select 'HIN','organization' from dual union all
select 'HIN','satellite manufacturer' from dual union all
select 'HISD','organization' from dual union all
select 'HISP','organization' from dual union all
select 'HISPM','organization' from dual union all
select 'HITS','satellite manufacturer' from dual union all
select 'HK','country' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'HKUK','country' from dual union all
select 'HNS','organization' from dual union all
select 'HOG','school' from dual union all
select 'HOKIT','organization' from dual union all
select 'HOKIT','satellite manufacturer' from dual union all
select 'HOKK','school' from dual union all
select 'HONEY','satellite manufacturer' from dual union all
select 'HORZ','organization' from dual union all
select 'HOUS','school' from dual union all
select 'HOUW','engine/motor manufacturer' from dual union all
select 'HOUW','launch agency' from dual union all
select 'HOUW','launch vehicle manufacturer' from dual union all
select 'HPS','satellite manufacturer' from dual union all
select 'HQZS','organization' from dual union all
select 'HRBS','satellite manufacturer' from dual union all
select 'HRBSE','satellite manufacturer' from dual union all
select 'HSCIL','organization' from dual union all
select 'HSCIL','satellite manufacturer' from dual union all
select 'HSD','satellite manufacturer' from dual union all
select 'HSES','satellite manufacturer' from dual union all
select 'HTSTL','organization' from dual union all
select 'HTSTL','satellite manufacturer' from dual union all
select 'HU','country' from dual union all
select 'HUPO','launch vehicle manufacturer' from dual union all
select 'HVP','engine/motor manufacturer' from dual union all
select 'HVP','launch site' from dual union all
select 'I','country' from dual union all
select 'I-CSC','country' from dual union all
select 'I-CSC','intergovernmental organization' from dual union all
select 'I-CSC1','country' from dual union all
select 'I-CSC1','intergovernmental organization' from dual union all
select 'I-ELDO','country' from dual union all
select 'I-ELDO','intergovernmental organization' from dual union all
select 'I-ESA','country' from dual union all
select 'I-ESA','intergovernmental organization' from dual union all
select 'I-ESRO','country' from dual union all
select 'I-ESRO','intergovernmental organization' from dual union all
select 'I-EU','country' from dual union all
select 'I-EU','intergovernmental organization' from dual union all
select 'I-EUM','country' from dual union all
select 'I-EUM','intergovernmental organization' from dual union all
select 'I-EUT','country' from dual union all
select 'I-EUT','intergovernmental organization' from dual union all
select 'I-INM','country' from dual union all
select 'I-INM','intergovernmental organization' from dual union all
select 'I-INT','country' from dual union all
select 'I-INT','intergovernmental organization' from dual union all
select 'I-RASC','country' from dual union all
select 'I-RASC','intergovernmental organization' from dual union all
select 'IAA','school' from dual union all
select 'IACG','organization' from dual union all
select 'IAE','engine/motor manufacturer' from dual union all
select 'IAE','launch agency' from dual union all
select 'IAE','launch vehicle manufacturer' from dual union all
select 'IAE','school' from dual union all
select 'IAFE','school' from dual union all
select 'IAI','engine/motor manufacturer' from dual union all
select 'IAI','launch agency' from dual union all
select 'IAI','launch vehicle manufacturer' from dual union all
select 'IAI','satellite manufacturer' from dual union all
select 'IAIS','organization' from dual union all
select 'IAIS','satellite manufacturer' from dual union all
select 'IAM','school' from dual union all
select 'IAP','school' from dual union all
select 'IASB','school' from dual union all
select 'IBMH','satellite manufacturer' from dual union all
select 'IBSI','school' from dual union all
select 'ICANS','satellite manufacturer' from dual union all
select 'ICEYE','organization' from dual union all
select 'ICO','organization' from dual union all
select 'ICOH','organization' from dual union all
select 'ICONA','organization' from dual union all
select 'ICST','school' from dual union all
select 'ICTQ','organization' from dual union all
select 'ID','country' from dual union all
select 'IDF','launch agency' from dual union all
select 'IDF','organization' from dual union all
select 'IDG','launch agency' from dual union all
select 'IDG','school' from dual union all
select 'IDRDL','launch agency' from dual union all
select 'IE','country' from dual union all
select 'IEM','school' from dual union all
select 'IFAMR','school' from dual union all
select 'IGC','organization' from dual union all
select 'IHI','engine/motor manufacturer' from dual union all
select 'IHIA','engine/motor manufacturer' from dual union all
select 'IHIC','engine/motor manufacturer' from dual union all
select 'IHIC','satellite manufacturer' from dual union all
select 'IHIO','engine/motor manufacturer' from dual union all
select 'IHIO','satellite manufacturer' from dual union all
select 'IIAE','engine/motor manufacturer' from dual union all
select 'IIAE','launch agency' from dual union all
select 'IIAE','launch vehicle manufacturer' from dual union all
select 'IIAE','school' from dual union all
select 'IITB','organization' from dual union all
select 'IITB','satellite manufacturer' from dual union all
select 'IITK','organization' from dual union all
select 'IITK','satellite manufacturer' from dual union all
select 'IITM','school' from dual union all
select 'IITMC','organization' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'IITMC','satellite manufacturer' from dual union all
select 'IK','organization' from dual union all
select 'IKI','organization' from dual union all
select 'IKI','satellite manufacturer' from dual union all
select 'IKI','school' from dual union all
select 'IL','country' from dual union all
select 'ILC','school' from dual union all
select 'ILS','launch agency' from dual union all
select 'ILSK','launch agency' from dual union all
select 'ILSKR','launch agency' from dual union all
select 'ILSL','launch agency' from dual union all
select 'IMAGT','organization' from dual union all
select 'IMBP','organization' from dual union all
select 'IMDP','school' from dual union all
select 'IMICAS','organization' from dual union all
select 'IMICAS','satellite manufacturer' from dual union all
select 'IMIS','engine/motor manufacturer' from dual union all
select 'IN','country' from dual union all
select 'INAVC','organization' from dual union all
select 'INCO','launch agency' from dual union all
select 'INCOT','launch agency' from dual union all
select 'INDO','organization' from dual union all
select 'INDOS','organization' from dual union all
select 'INETI','organization' from dual union all
select 'INFER','organization' from dual union all
select 'INFER','satellite manufacturer' from dual union all
select 'INFK','organization' from dual union all
select 'INFOC','organization' from dual union all
select 'INFOC','satellite manufacturer' from dual union all
select 'INMAR','organization' from dual union all
select 'INMRL','organization' from dual union all
select 'INPE','launch agency' from dual union all
select 'INPE','launch vehicle manufacturer' from dual union all
select 'INPE','organization' from dual union all
select 'INPE','satellite manufacturer' from dual union all
select 'INPE','school' from dual union all
select 'INPES','organization' from dual union all
select 'INPES','satellite manufacturer' from dual union all
select 'INSSET','satellite manufacturer' from dual union all
select 'INTA','engine/motor manufacturer' from dual union all
select 'INTA','launch agency' from dual union all
select 'INTA','launch vehicle manufacturer' from dual union all
select 'INTA','organization' from dual union all
select 'INTA','satellite manufacturer' from dual union all
select 'INTA','school' from dual union all
select 'INTEL','organization' from dual union all
select 'INTELB','organization' from dual union all
select 'INTELS','organization' from dual union all
select 'INTELU','organization' from dual union all
select 'INVAP','satellite manufacturer' from dual union all
select 'IOS','satellite manufacturer' from dual union all
select 'IPSF','satellite manufacturer' from dual union all
select 'IQ','country' from dual union all
select 'IQ','engine/motor manufacturer' from dual union all
select 'IQ223','launch agency' from dual union all
select 'IQ224','launch agency' from dual union all
select 'IQA','launch agency' from dual union all
select 'IQMIC','launch agency' from dual union all
select 'IQMIC','launch vehicle manufacturer' from dual union all
select 'IQMOST','organization' from dual union all
select 'IR','country' from dual union all
select 'IR','engine/motor manufacturer' from dual union all
select 'IRF','organization' from dual union all
select 'IRF','satellite manufacturer' from dual union all
select 'IRF','school' from dual union all
select 'IRID','organization' from dual union all
select 'IRIDS','organization' from dual union all
select 'IROST','organization' from dual union all
select 'IRSA','launch agency' from dual union all
select 'IRSA','organization' from dual union all
select 'IRSA','satellite manufacturer' from dual union all
select 'IRSA','school' from dual union all
select 'IRSS','school' from dual union all
select 'IS','country' from dual union all
select 'ISA','launch agency' from dual union all
select 'ISA','organization' from dual union all
select 'ISAC','satellite manufacturer' from dual union all
select 'ISAE','satellite manufacturer' from dual union all
select 'ISAEV','engine/motor manufacturer' from dual union all
select 'ISAS','engine/motor manufacturer' from dual union all
select 'ISAS','launch agency' from dual union all
select 'ISAS','launch vehicle manufacturer' from dual union all
select 'ISAS','organization' from dual union all
select 'ISAS','satellite manufacturer' from dual union all
select 'ISAS','school' from dual union all
select 'ISASJ','organization' from dual union all
select 'ISASJ','satellite manufacturer' from dual union all
select 'ISASM','organization' from dual union all
select 'ISASM','satellite manufacturer' from dual union all
select 'ISASS','organization' from dual union all
select 'ISASS','satellite manufacturer' from dual union all
select 'ISFC','launch agency' from dual union all
select 'ISISBV','organization' from dual union all
select 'ISISBV','satellite manufacturer' from dual union all
select 'ISKRM','engine/motor manufacturer' from dual union all
select 'ISKRP','engine/motor manufacturer' from dual union all
select 'ISPUT','organization' from dual union all
select 'ISRO','engine/motor manufacturer' from dual union all
select 'ISRO','launch agency' from dual union all
select 'ISRO','launch vehicle manufacturer' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'ISRO','organization' from dual union all
select 'ISRO','satellite manufacturer' from dual union all
select 'ISRO','school' from dual union all
select 'IST','launch vehicle manufacturer' from dual union all
select 'ISTPK','organization' from dual union all
select 'ISTPK','satellite manufacturer' from dual union all
select 'ITA','satellite manufacturer' from dual union all
select 'ITBW','organization' from dual union all
select 'ITCR','organization' from dual union all
select 'ITRC','organization' from dual union all
select 'ITT','satellite manufacturer' from dual union all
select 'ITUTR','organization' from dual union all
select 'ITUTR','satellite manufacturer' from dual union all
select 'IUST','satellite manufacturer' from dual union all
select 'IZM','launch agency' from dual union all
select 'IZM','organization' from dual union all
select 'IZM','school' from dual union all
select 'J','country' from dual union all
select 'JARL','organization' from dual union all
select 'JARL','satellite manufacturer' from dual union all
select 'JAXA','engine/motor manufacturer' from dual union all
select 'JAXA','launch agency' from dual union all
select 'JAXA','launch vehicle manufacturer' from dual union all
select 'JAXA','organization' from dual union all
select 'JAXA','satellite manufacturer' from dual union all
select 'JAXA','school' from dual union all
select 'JAXAC','engine/motor manufacturer' from dual union all
select 'JAXAC','launch agency' from dual union all
select 'JAXAC','launch vehicle manufacturer' from dual union all
select 'JAXAC','organization' from dual union all
select 'JAXAC','satellite manufacturer' from dual union all
select 'JBO','school' from dual union all
select 'JCAB','organization' from dual union all
select 'JCSAT','organization' from dual union all
select 'JDA','launch agency' from dual union all
select 'JENA','satellite manufacturer' from dual union all
select 'JENAA','satellite manufacturer' from dual union all
select 'JHU','school' from dual union all
select 'JMA','launch agency' from dual union all
select 'JMA','organization' from dual union all
select 'JMA','school' from dual union all
select 'JMSDF','launch agency' from dual union all
select 'JO','country' from dual union all
select 'JPL','engine/motor manufacturer' from dual union all
select 'JPL','launch agency' from dual union all
select 'JPL','launch vehicle manufacturer' from dual union all
select 'JPL','organization' from dual union all
select 'JPL','satellite manufacturer' from dual union all
select 'JPL','school' from dual union all
select 'JRG','engine/motor manufacturer' from dual union all
select 'JRG','launch agency' from dual union all
select 'JRG','launch vehicle manufacturer' from dual union all
select 'JSAT','organization' from dual union all
select 'JSC','organization' from dual union all
select 'JSC','satellite manufacturer' from dual union all
select 'JSC','school' from dual union all
select 'JSDO','launch agency' from dual union all
select 'JTF7','launch agency' from dual union all
select 'JUNK','satellite manufacturer' from dual union all
select 'JUST','organization' from dual union all
select 'KACST','organization' from dual union all
select 'KACST','satellite manufacturer' from dual union all
select 'KAGA','organization' from dual union all
select 'KAGA','satellite manufacturer' from dual union all
select 'KAGO','organization' from dual union all
select 'KAGO','satellite manufacturer' from dual union all
select 'KAI','satellite manufacturer' from dual union all
select 'KAIFW','satellite manufacturer' from dual union all
select 'KAIK','organization' from dual union all
select 'KAIK','satellite manufacturer' from dual union all
select 'KAISR','satellite manufacturer' from dual union all
select 'KAIST','organization' from dual union all
select 'KAIST','satellite manufacturer' from dual union all
select 'KAIT','school' from dual union all
select 'KARI','engine/motor manufacturer' from dual union all
select 'KARI','launch agency' from dual union all
select 'KARI','launch vehicle manufacturer' from dual union all
select 'KARI','organization' from dual union all
select 'KARI','satellite manufacturer' from dual union all
select 'KARI','school' from dual union all
select 'KASH','organization' from dual union all
select 'KAWA','satellite manufacturer' from dual union all
select 'KAZK','organization' from dual union all
select 'KAZNU','satellite manufacturer' from dual union all
select 'KAZOKB','engine/motor manufacturer' from dual union all
select 'KAZSOY','engine/motor manufacturer' from dual union all
select 'KB1','organization' from dual union all
select 'KB1','satellite manufacturer' from dual union all
select 'KBEM','engine/motor manufacturer' from dual union all
select 'KBEME','engine/motor manufacturer' from dual union all
select 'KBKHA','engine/motor manufacturer' from dual union all
select 'KBM','engine/motor manufacturer' from dual union all
select 'KBM','launch vehicle manufacturer' from dual union all
select 'KBPT','engine/motor manufacturer' from dual union all
select 'KBPT','launch vehicle manufacturer' from dual union all
select 'KBSAL','satellite manufacturer' from dual union all
select 'KCHT','organization' from dual union all
select 'KE','country' from dual union all
select 'KEPLER','organization' from dual union all
select 'KEPWU','satellite manufacturer' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'KGI','school' from dual union all
select 'KGO','school' from dual union all
select 'KH','country' from dual union all
select 'KHMM','engine/motor manufacturer' from dual union all
select 'KHRO','launch agency' from dual union all
select 'KHRO','launch vehicle manufacturer' from dual union all
select 'KHRO','satellite manufacturer' from dual union all
select 'KHRR','launch agency' from dual union all
select 'KHRR','launch vehicle manufacturer' from dual union all
select 'KHRR','organization' from dual union all
select 'KHRR','satellite manufacturer' from dual union all
select 'KHRS','launch agency' from dual union all
select 'KHRS','launch vehicle manufacturer' from dual union all
select 'KHRS','satellite manufacturer' from dual union all
select 'KHRU','launch agency' from dual union all
select 'KHRU','launch vehicle manufacturer' from dual union all
select 'KHRU','organization' from dual union all
select 'KHRU','satellite manufacturer' from dual union all
select 'KHRU','school' from dual union all
select 'KHUS','organization' from dual union all
select 'KHUS','satellite manufacturer' from dual union all
select 'KI','country' from dual union all
select 'KIEL','school' from dual union all
select 'KLIMOV','engine/motor manufacturer' from dual union all
select 'KMARQ','engine/motor manufacturer' from dual union all
select 'KMIT','satellite manufacturer' from dual union all
select 'KMUTNB','satellite manufacturer' from dual union all
select 'KOBE','school' from dual union all
select 'KOBN','school' from dual union all
select 'KOKT','engine/motor manufacturer' from dual union all
select 'KOLN','school' from dual union all
select 'KOMET','satellite manufacturer' from dual union all
select 'KORS','country' from dual union all
select 'KORSA','country' from dual union all
select 'KOSMT','satellite manufacturer' from dual union all
select 'KOUKO','organization' from dual union all
select 'KOUKO','satellite manufacturer' from dual union all
select 'KP','country' from dual union all
select 'KPA','engine/motor manufacturer' from dual union all
select 'KPA','launch agency' from dual union all
select 'KPA','launch vehicle manufacturer' from dual union all
select 'KPAMJ','engine/motor manufacturer' from dual union all
select 'KPAMJ','launch agency' from dual union all
select 'KPAMJ','launch vehicle manufacturer' from dual union all
select 'KPI','organization' from dual union all
select 'KPI','satellite manufacturer' from dual union all
select 'KPNO','launch agency' from dual union all
select 'KPNO','school' from dual union all
select 'KR','country' from dual union all
select 'KRAS','engine/motor manufacturer' from dual union all
select 'KRAT','launch vehicle manufacturer' from dual union all
select 'KRLP','launch agency' from dual union all
select 'KRLP','launch vehicle manufacturer' from dual union all
select 'KS','organization' from dual union all
select 'KSC','launch agency' from dual union all
select 'KSC','launch site' from dual union all
select 'KSC','school' from dual union all
select 'KTEL','organization' from dual union all
select 'KTH','school' from dual union all
select 'KTHR','satellite manufacturer' from dual union all
select 'KTRAS','launch agency' from dual union all
select 'KTRAS','organization' from dual union all
select 'KTU','organization' from dual union all
select 'KTU','satellite manufacturer' from dual union all
select 'KUBOS','organization' from dual union all
select 'KUZ','engine/motor manufacturer' from dual union all
select 'KVR','launch agency' from dual union all
select 'KVR','organization' from dual union all
select 'KYO','school' from dual union all
select 'KYSP','launch agency' from dual union all
select 'KYSP','organization' from dual union all
select 'KYSP','satellite manufacturer' from dual union all
select 'KYSP','school' from dual union all
select 'KYUD','organization' from dual union all
select 'KYUD','satellite manufacturer' from dual union all
select 'KYUT','organization' from dual union all
select 'KYUT','satellite manufacturer' from dual union all
select 'KZ','country' from dual union all
select 'L','country' from dual union all
select 'LA','country' from dual union all
select 'LAMBDA','organization' from dual union all
select 'LAMBDA','satellite manufacturer' from dual union all
select 'LAND','organization' from dual union all
select 'LANL','organization' from dual union all
select 'LANL','satellite manufacturer' from dual union all
select 'LANL','school' from dual union all
select 'LAPAN','engine/motor manufacturer' from dual union all
select 'LAPAN','launch agency' from dual union all
select 'LAPAN','launch vehicle manufacturer' from dual union all
select 'LAPAN','organization' from dual union all
select 'LAPAN','satellite manufacturer' from dual union all
select 'LAPAN','school' from dual union all
select 'LARC','engine/motor manufacturer' from dual union all
select 'LARC','launch agency' from dual union all
select 'LARC','launch vehicle manufacturer' from dual union all
select 'LARC','school' from dual union all
select 'LARCN','launch agency' from dual union all
select 'LARCN','organization' from dual union all
select 'LARCN','satellite manufacturer' from dual union all
select 'LARCN','school' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'LARCNE','school' from dual union all
select 'LAS','school' from dual union all
select 'LASL','launch agency' from dual union all
select 'LASL','satellite manufacturer' from dual union all
select 'LASL','school' from dual union all
select 'LATMOS','satellite manufacturer' from dual union all
select 'LB','country' from dual union all
select 'LCT','satellite manufacturer' from dual union all
select 'LEEDS','school' from dual union all
select 'LEID','school' from dual union all
select 'LEOSTL','satellite manufacturer' from dual union all
select 'LERC','launch agency' from dual union all
select 'LERC','organization' from dual union all
select 'LERC','satellite manufacturer' from dual union all
select 'LERC','school' from dual union all
select 'LGAR','satellite manufacturer' from dual union all
select 'LIEGE','school' from dual union all
select 'LINAO','meteorological' from dual union all
select 'LINAO','school' from dual union all
select 'LINK','satellite manufacturer' from dual union all
select 'LK','country' from dual union all
select 'LKA','organization' from dual union all
select 'LKA','satellite manufacturer' from dual union all
select 'LLL','launch agency' from dual union all
select 'LLL','launch vehicle manufacturer' from dual union all
select 'LLNL','launch vehicle manufacturer' from dual union all
select 'LLNL','organization' from dual union all
select 'LLNL','satellite manufacturer' from dual union all
select 'LLNL','school' from dual union all
select 'LM','launch agency' from dual union all
select 'LM','satellite manufacturer' from dual union all
select 'LMA','launch agency' from dual union all
select 'LMA','launch vehicle manufacturer' from dual union all
select 'LMA','satellite manufacturer' from dual union all
select 'LMCAM','launch vehicle manufacturer' from dual union all
select 'LMCLS','launch agency' from dual union all
select 'LMCO','launch vehicle manufacturer' from dual union all
select 'LMCSS','organization' from dual union all
select 'LMCSS','satellite manufacturer' from dual union all
select 'LMEW','satellite manufacturer' from dual union all
select 'LMI','organization' from dual union all
select 'LMKSC','satellite manufacturer' from dual union all
select 'LMMI','engine/motor manufacturer' from dual union all
select 'LMMI','satellite manufacturer' from dual union all
select 'LMMS','launch vehicle manufacturer' from dual union all
select 'LMMS','satellite manufacturer' from dual union all
select 'LMN','satellite manufacturer' from dual union all
select 'LMPCO','launch vehicle manufacturer' from dual union all
select 'LMSAL','satellite manufacturer' from dual union all
select 'LMSC','launch agency' from dual union all
select 'LMSC','launch vehicle manufacturer' from dual union all
select 'LMSC','satellite manufacturer' from dual union all
select 'LMSC','school' from dual union all
select 'LMSCV','organization' from dual union all
select 'LMSD','satellite manufacturer' from dual union all
select 'LMSS','satellite manufacturer' from dual union all
select 'LMSSD','launch vehicle manufacturer' from dual union all
select 'LMSSD','satellite manufacturer' from dual union all
select 'LMT','satellite manufacturer' from dual union all
select 'LMTEW','satellite manufacturer' from dual union all
select 'LMVF','satellite manufacturer' from dual union all
select 'LOR','organization' from dual union all
select 'LOR','satellite manufacturer' from dual union all
select 'LORI','organization' from dual union all
select 'LPA','school' from dual union all
select 'LPARL','satellite manufacturer' from dual union all
select 'LPARL','school' from dual union all
select 'LPC','engine/motor manufacturer' from dual union all
select 'LPSC','engine/motor manufacturer' from dual union all
select 'LPSC','satellite manufacturer' from dual union all
select 'LPSCM','engine/motor manufacturer' from dual union all
select 'LPSP','school' from dual union all
select 'LRBA','engine/motor manufacturer' from dual union all
select 'LRBA','launch agency' from dual union all
select 'LRBA','launch vehicle manufacturer' from dual union all
select 'LRBA','school' from dual union all
select 'LRL','launch agency' from dual union all
select 'LRL','satellite manufacturer' from dual union all
select 'LRL','school' from dual union all
select 'LRPG','launch site' from dual union all
select 'LRRC','launch vehicle manufacturer' from dual union all
select 'LRRC','school' from dual union all
select 'LRS','engine/motor manufacturer' from dual union all
select 'LRS','launch agency' from dual union all
select 'LRS','launch vehicle manufacturer' from dual union all
select 'LRS','school' from dual union all
select 'LSKY','organization' from dual union all
select 'LSKYB','organization' from dual union all
select 'LT','country' from dual union all
select 'LTU','satellite manufacturer' from dual union all
select 'LTV','launch vehicle manufacturer' from dual union all
select 'LTV','satellite manufacturer' from dual union all
select 'LTVCAM','launch vehicle manufacturer' from dual union all
select 'LTVE','satellite manufacturer' from dual union all
select 'LTVM','launch vehicle manufacturer' from dual union all
select 'LUNA','astronomical polity' from dual union all
select 'LUND','school' from dual union all
select 'LUX','school' from dual union all
select 'LUXG','organization' from dual union all
select 'LUXS','organization' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'LUXS','satellite manufacturer' from dual union all
select 'LV','country' from dual union all
select 'LVN','launch vehicle manufacturer' from dual union all
select 'LY','country' from dual union all
select 'LYK','launch vehicle manufacturer' from dual union all
select 'M42','satellite manufacturer' from dual union all
select 'MA','country' from dual union all
select 'MAI','organization' from dual union all
select 'MAIMD','satellite manufacturer' from dual union all
select 'MAIO','organization' from dual union all
select 'MAIO','satellite manufacturer' from dual union all
select 'MAK','engine/motor manufacturer' from dual union all
select 'MAK','launch agency' from dual union all
select 'MAK','launch vehicle manufacturer' from dual union all
select 'MAK','organization' from dual union all
select 'MAK','satellite manufacturer' from dual union all
select 'MARCO','satellite manufacturer' from dual union all
select 'MARQ','engine/motor manufacturer' from dual union all
select 'MARQO','engine/motor manufacturer' from dual union all
select 'MARTB','engine/motor manufacturer' from dual union all
select 'MARTB','launch vehicle manufacturer' from dual union all
select 'MARTB','satellite manufacturer' from dual union all
select 'MARTD','launch vehicle manufacturer' from dual union all
select 'MARTO','launch vehicle manufacturer' from dual union all
select 'MASTEN','engine/motor manufacturer' from dual union all
select 'MASTEN','launch vehicle manufacturer' from dual union all
select 'MATRA','launch agency' from dual union all
select 'MATRA','launch vehicle manufacturer' from dual union all
select 'MATRA','school' from dual union all
select 'MATT','satellite manufacturer' from dual union all
select 'MATV','satellite manufacturer' from dual union all
select 'MAXAR','satellite manufacturer' from dual union all
select 'MAXARB','satellite manufacturer' from dual union all
select 'MAXARM','satellite manufacturer' from dual union all
select 'MBAK','satellite manufacturer' from dual union all
select 'MBB','engine/motor manufacturer' from dual union all
select 'MBB','launch agency' from dual union all
select 'MBB','organization' from dual union all
select 'MBB','satellite manufacturer' from dual union all
select 'MBB','school' from dual union all
select 'MBBE','satellite manufacturer' from dual union all
select 'MBBM','satellite manufacturer' from dual union all
select 'MBCO','organization' from dual union all
select 'MC','country' from dual union all
select 'MCDA','launch vehicle manufacturer' from dual union all
select 'MCDA','satellite manufacturer' from dual union all
select 'MCI','organization' from dual union all
select 'MDA','launch agency' from dual union all
select 'MDA','organization' from dual union all
select 'MDA','school' from dual union all
select 'MDABC','satellite manufacturer' from dual union all
select 'MDAC','engine/motor manufacturer' from dual union all
select 'MDAC','launch vehicle manufacturer' from dual union all
select 'MDAC','satellite manufacturer' from dual union all
select 'MDAC','school' from dual union all
select 'MDAM','satellite manufacturer' from dual union all
select 'MDASL','satellite manufacturer' from dual union all
select 'MDD','launch agency' from dual union all
select 'MDD','meteorological' from dual union all
select 'MDHSV','satellite manufacturer' from dual union all
select 'MDR','satellite manufacturer' from dual union all
select 'MDSSC','launch agency' from dual union all
select 'MEAS','organization' from dual union all
select 'MEGS','organization' from dual union all
select 'MEGS','satellite manufacturer' from dual union all
select 'MEI','satellite manufacturer' from dual union all
select 'MEISEI','organization' from dual union all
select 'MEISEI','satellite manufacturer' from dual union all
select 'MEIT','satellite manufacturer' from dual union all
select 'MELB','organization' from dual union all
select 'MELB','satellite manufacturer' from dual union all
select 'MELCO','satellite manufacturer' from dual union all
select 'MEWTA','school' from dual union all
select 'MGON','organization' from dual union all
select 'MGU','organization' from dual union all
select 'MGU','satellite manufacturer' from dual union all
select 'MH','country' from dual union all
select 'MHI','engine/motor manufacturer' from dual union all
select 'MHI','launch agency' from dual union all
select 'MHI','satellite manufacturer' from dual union all
select 'MHIME','engine/motor manufacturer' from dual union all
select 'MHITO','launch vehicle manufacturer' from dual union all
select 'MHITO','satellite manufacturer' from dual union all
select 'MICOM','launch agency' from dual union all
select 'MICRC','satellite manufacturer' from dual union all
select 'MICRC2','satellite manufacturer' from dual union all
select 'MICRCA','satellite manufacturer' from dual union all
select 'MICROC','satellite manufacturer' from dual union all
select 'MICSG','organization' from dual union all
select 'MICSG','satellite manufacturer' from dual union all
select 'MICSP','satellite manufacturer' from dual union all
select 'MICSY','satellite manufacturer' from dual union all
select 'MILAN','satellite manufacturer' from dual union all
select 'MILAN','school' from dual union all
select 'MILLEI','satellite manufacturer' from dual union all
select 'MILLEN','satellite manufacturer' from dual union all
select 'MILTEC','satellite manufacturer' from dual union all
select 'MINCYT','organization' from dual union all
select 'MINES','satellite manufacturer' from dual union all
select 'MIRC','school' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'MISU','school' from dual union all
select 'MIT','launch vehicle manufacturer' from dual union all
select 'MIT','satellite manufacturer' from dual union all
select 'MIT','school' from dual union all
select 'MITLL','launch agency' from dual union all
select 'MITLL','organization' from dual union all
select 'MITLL','satellite manufacturer' from dual union all
select 'MITLL','school' from dual union all
select 'MITT','engine/motor manufacturer' from dual union all
select 'MITT','launch vehicle manufacturer' from dual union all
select 'MM','launch vehicle manufacturer' from dual union all
select 'MM','satellite manufacturer' from dual union all
select 'MMA','launch agency' from dual union all
select 'MMAS','satellite manufacturer' from dual union all
select 'MMCLS','launch agency' from dual union all
select 'MMD','launch vehicle manufacturer' from dual union all
select 'MMD','satellite manufacturer' from dual union all
select 'MMEW','satellite manufacturer' from dual union all
select 'MMMI','satellite manufacturer' from dual union all
select 'MMPCO','launch vehicle manufacturer' from dual union all
select 'MMSS','satellite manufacturer' from dual union all
select 'MMST','satellite manufacturer' from dual union all
select 'MMSV','satellite manufacturer' from dual union all
select 'MMVF','satellite manufacturer' from dual union all
select 'MN','country' from dual union all
select 'MNF','launch agency' from dual union all
select 'MNF','meteorological' from dual union all
select 'MNF','school' from dual union all
select 'MO','organization' from dual union all
select 'MO','school' from dual union all
select 'MOD','launch agency' from dual union all
select 'MOD','organization' from dual union all
select 'MOLN','organization' from dual union all
select 'MOLN','satellite manufacturer' from dual union all
select 'MOM','organization' from dual union all
select 'MONTP2','organization' from dual union all
select 'MONTP2','satellite manufacturer' from dual union all
select 'MOOGNF','engine/motor manufacturer' from dual union all
select 'MOOGW','engine/motor manufacturer' from dual union all
select 'MORF','organization' from dual union all
select 'MOTO','satellite manufacturer' from dual union all
select 'MOTOS','engine/motor manufacturer' from dual union all
select 'MOZHA','organization' from dual union all
select 'MOZHA','satellite manufacturer' from dual union all
select 'MPAE','school' from dual union all
select 'MPE','launch agency' from dual union all
select 'MPE','organization' from dual union all
select 'MPE','satellite manufacturer' from dual union all
select 'MPE','school' from dual union all
select 'MPIH','school' from dual union all
select 'MPIK','launch agency' from dual union all
select 'MPIK','school' from dual union all
select 'MPSC','organization' from dual union all
select 'MPU','satellite manufacturer' from dual union all
select 'MR','country' from dual union all
select 'MRI','school' from dual union all
select 'MRN','launch agency' from dual union all
select 'MRN','school' from dual union all
select 'MSBTR','organization' from dual union all
select 'MSC','organization' from dual union all
select 'MSC','school' from dual union all
select 'MSCI','satellite manufacturer' from dual union all
select 'MSFC','engine/motor manufacturer' from dual union all
select 'MSFC','launch agency' from dual union all
select 'MSFC','launch vehicle manufacturer' from dual union all
select 'MSFC','organization' from dual union all
select 'MSFC','satellite manufacturer' from dual union all
select 'MSFC','school' from dual union all
select 'MSGC','organization' from dual union all
select 'MSSL','school' from dual union all
select 'MSU','organization' from dual union all
select 'MSU','satellite manufacturer' from dual union all
select 'MSU','school' from dual union all
select 'MSUKY','organization' from dual union all
select 'MSUKY','satellite manufacturer' from dual union all
select 'MSV','organization' from dual union all
select 'MSVYAZ','organization' from dual union all
select 'MU','country' from dual union all
select 'MUNIZ','satellite manufacturer' from dual union all
select 'MV','country' from dual union all
select 'MV7','launch agency' from dual union all
select 'MVS','launch agency' from dual union all
select 'MVSM','launch agency' from dual union all
select 'MX','country' from dual union all
select 'MY','country' from dual union all
select 'N','country' from dual union all
select 'NAA','engine/motor manufacturer' from dual union all
select 'NAA','launch vehicle manufacturer' from dual union all
select 'NAA','satellite manufacturer' from dual union all
select 'NAAD','launch vehicle manufacturer' from dual union all
select 'NAAD','satellite manufacturer' from dual union all
select 'NAADR','engine/motor manufacturer' from dual union all
select 'NACA','launch agency' from dual union all
select 'NADA','launch agency' from dual union all
select 'NAFI','satellite manufacturer' from dual union all
select 'NAG','organization' from dual union all
select 'NAG','satellite manufacturer' from dual union all
select 'NAG','school' from dual union all
select 'NAHU','organization' from dual union all
select 'NAIC','launch agency' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'NAIC','school' from dual union all
select 'NAIR','organization' from dual union all
select 'NAKA','satellite manufacturer' from dual union all
select 'NAL','engine/motor manufacturer' from dual union all
select 'NAL','launch agency' from dual union all
select 'NAL','launch vehicle manufacturer' from dual union all
select 'NAL','organization' from dual union all
select 'NAL','satellite manufacturer' from dual union all
select 'NAMFI','launch agency' from dual union all
select 'NAMMO','engine/motor manufacturer' from dual union all
select 'NAMMO','launch agency' from dual union all
select 'NAMMO','launch vehicle manufacturer' from dual union all
select 'NAMMO','school' from dual union all
select 'NAMTC','launch agency' from dual union all
select 'NANSFI','organization' from dual union all
select 'NANSFI','satellite manufacturer' from dual union all
select 'NAOJ','school' from dual union all
select 'NARSS','organization' from dual union all
select 'NASA','launch agency' from dual union all
select 'NASA','launch vehicle manufacturer' from dual union all
select 'NASA','organization' from dual union all
select 'NASA','satellite manufacturer' from dual union all
select 'NASA','school' from dual union all
select 'NASAHQ','launch agency' from dual union all
select 'NASAHQ','launch vehicle manufacturer' from dual union all
select 'NASAHQ','satellite manufacturer' from dual union all
select 'NASDA','engine/motor manufacturer' from dual union all
select 'NASDA','launch agency' from dual union all
select 'NASDA','launch vehicle manufacturer' from dual union all
select 'NASDA','organization' from dual union all
select 'NASDA','satellite manufacturer' from dual union all
select 'NASDA','school' from dual union all
select 'NASDM','satellite manufacturer' from dual union all
select 'NATO','organization' from dual union all
select 'NAVAIR','organization' from dual union all
select 'NAWC','launch agency' from dual union all
select 'NAWC','launch vehicle manufacturer' from dual union all
select 'NAWC','school' from dual union all
select 'NAXS','school' from dual union all
select 'NBN','organization' from dual union all
select 'NBS','school' from dual union all
select 'NCAR','school' from dual union all
select 'NCKU','organization' from dual union all
select 'NCKU','satellite manufacturer' from dual union all
select 'NCU','school' from dual union all
select 'NDAWN','organization' from dual union all
select 'NDRE','launch agency' from dual union all
select 'NDRE','organization' from dual union all
select 'NDRE','school' from dual union all
select 'NEC','satellite manufacturer' from dual union all
select 'NECST','satellite manufacturer' from dual union all
select 'NECT','satellite manufacturer' from dual union all
select 'NECTF','satellite manufacturer' from dual union all
select 'NECTKA','satellite manufacturer' from dual union all
select 'NECTKE','satellite manufacturer' from dual union all
select 'NESCOMN','launch agency' from dual union all
select 'NESCOMN','launch vehicle manufacturer' from dual union all
select 'NESCOMP','launch agency' from dual union all
select 'NEU','school' from dual union all
select 'NEWSAT','organization' from dual union all
select 'NG','country' from dual union all
select 'NGAA','satellite manufacturer' from dual union all
select 'NGAS','organization' from dual union all
select 'NGAS','satellite manufacturer' from dual union all
select 'NGCOM','organization' from dual union all
select 'NGISABL','engine/motor manufacturer' from dual union all
select 'NGISD','launch agency' from dual union all
select 'NGISG','satellite manufacturer' from dual union all
select 'NGISGO','satellite manufacturer' from dual union all
select 'NGISH','engine/motor manufacturer' from dual union all
select 'NGISW','launch vehicle manufacturer' from dual union all
select 'NGNW','satellite manufacturer' from dual union all
select 'NGST','satellite manufacturer' from dual union all
select 'NHK','organization' from dual union all
select 'NICP','school' from dual union all
select 'NICS','organization' from dual union all
select 'NIHON','organization' from dual union all
select 'NIHON','satellite manufacturer' from dual union all
select 'NII125','engine/motor manufacturer' from dual union all
select 'NII88','launch agency' from dual union all
select 'NII88','launch vehicle manufacturer' from dual union all
select 'NII88','organization' from dual union all
select 'NII88A','launch agency' from dual union all
select 'NII88A','launch vehicle manufacturer' from dual union all
select 'NIIM','engine/motor manufacturer' from dual union all
select 'NIIPP','organization' from dual union all
select 'NIIPP','satellite manufacturer' from dual union all
select 'NIKUS','satellite manufacturer' from dual union all
select 'NILE','organization' from dual union all
select 'NIPR','launch agency' from dual union all
select 'NIPR','school' from dual union all
select 'NIRFI','school' from dual union all
select 'NISS','engine/motor manufacturer' from dual union all
select 'NISS','satellite manufacturer' from dual union all
select 'NISST','engine/motor manufacturer' from dual union all
select 'NISST','launch vehicle manufacturer' from dual union all
select 'NIUTN','satellite manufacturer' from dual union all
select 'NIVR','organization' from dual union all
select 'NJUST','organization' from dual union all
select 'NJUST','satellite manufacturer' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'NKAU','organization' from dual union all
select 'NKE','engine/motor manufacturer' from dual union all
select 'NKO','launch agency' from dual union all
select 'NL','country' from dual union all
select 'NLR','satellite manufacturer' from dual union all
select 'NMC','engine/motor manufacturer' from dual union all
select 'NMC','launch agency' from dual union all
select 'NMC','launch vehicle manufacturer' from dual union all
select 'NMC','school' from dual union all
select 'NMCOL','satellite manufacturer' from dual union all
select 'NMIT','organization' from dual union all
select 'NMIT','satellite manufacturer' from dual union all
select 'NMSA','school' from dual union all
select 'NMSU','launch agency' from dual union all
select 'NMSU','launch vehicle manufacturer' from dual union all
select 'NMSU','school' from dual union all
select 'NOAA','launch agency' from dual union all
select 'NOAA','organization' from dual union all
select 'NOAA','school' from dual union all
select 'NOBY','school' from dual union all
select 'NOL','launch agency' from dual union all
select 'NOL','school' from dual union all
select 'NONAN','launch agency' from dual union all
select 'NORD','launch vehicle manufacturer' from dual union all
select 'NORD','satellite manufacturer' from dual union all
select 'NORDS','satellite manufacturer' from dual union all
select 'NORLM','launch vehicle manufacturer' from dual union all
select 'NORTH','launch vehicle manufacturer' from dual union all
select 'NORTH','satellite manufacturer' from dual union all
select 'NORTHC','engine/motor manufacturer' from dual union all
select 'NORTHS','satellite manufacturer' from dual union all
select 'NOTS','engine/motor manufacturer' from dual union all
select 'NOTS','launch agency' from dual union all
select 'NOTS','launch vehicle manufacturer' from dual union all
select 'NOTS','organization' from dual union all
select 'NOTS','satellite manufacturer' from dual union all
select 'NOTS','school' from dual union all
select 'NOVTR','launch vehicle manufacturer' from dual union all
select 'NPL','school' from dual union all
select 'NPOE','launch vehicle manufacturer' from dual union all
select 'NPOE','organization' from dual union all
select 'NPOE','satellite manufacturer' from dual union all
select 'NPOL','engine/motor manufacturer' from dual union all
select 'NPOL','launch agency' from dual union all
select 'NPOL','launch vehicle manufacturer' from dual union all
select 'NPOL','organization' from dual union all
select 'NPOL','satellite manufacturer' from dual union all
select 'NPOL','school' from dual union all
select 'NPOLO','launch agency' from dual union all
select 'NPOMA','launch vehicle manufacturer' from dual union all
select 'NPOMA','organization' from dual union all
select 'NPOMA','satellite manufacturer' from dual union all
select 'NPOPM','organization' from dual union all
select 'NPOPM','satellite manufacturer' from dual union all
select 'NPOPMR','organization' from dual union all
select 'NPOPMR','satellite manufacturer' from dual union all
select 'NPOT','engine/motor manufacturer' from dual union all
select 'NRCC','launch agency' from dual union all
select 'NRCC','organization' from dual union all
select 'NRCC','satellite manufacturer' from dual union all
select 'NRCC','school' from dual union all
select 'NRL','engine/motor manufacturer' from dual union all
select 'NRL','launch agency' from dual union all
select 'NRL','launch vehicle manufacturer' from dual union all
select 'NRL','organization' from dual union all
select 'NRL','satellite manufacturer' from dual union all
select 'NRL','school' from dual union all
select 'NRO','organization' from dual union all
select 'NROC','organization' from dual union all
select 'NSA','launch agency' from dual union all
select 'NSA','organization' from dual union all
select 'NSA','school' from dual union all
select 'NSAB','organization' from dual union all
select 'NSC','organization' from dual union all
select 'NSC','satellite manufacturer' from dual union all
select 'NSDC','launch agency' from dual union all
select 'NSF','launch agency' from dual union all
select 'NSF','organization' from dual union all
select 'NSL','organization' from dual union all
select 'NSL','satellite manufacturer' from dual union all
select 'NSMC','organization' from dual union all
select 'NSPO','engine/motor manufacturer' from dual union all
select 'NSPO','launch agency' from dual union all
select 'NSPO','launch vehicle manufacturer' from dual union all
select 'NSPO','organization' from dual union all
select 'NSPO','school' from dual union all
select 'NSPT','organization' from dual union all
select 'NSRDA','organization' from dual union all
select 'NSRDA','satellite manufacturer' from dual union all
select 'NSROC','school' from dual union all
select 'NSS','organization' from dual union all
select 'NSWCPH','school' from dual union all
select 'NSWCWS','school' from dual union all
select 'NTNF','launch agency' from dual union all
select 'NTNU','organization' from dual union all
select 'NTNU','satellite manufacturer' from dual union all
select 'NTT','organization' from dual union all
select 'NTTDM','organization' from dual union all
select 'NTU','organization' from dual union all
select 'NTU','satellite manufacturer' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'NUAA','organization' from dual union all
select 'NUAA','satellite manufacturer' from dual union all
select 'NUDTC','engine/motor manufacturer' from dual union all
select 'NUDTC','launch vehicle manufacturer' from dual union all
select 'NUDTC','organization' from dual union all
select 'NUDTC','satellite manufacturer' from dual union all
select 'NUM','satellite manufacturer' from dual union all
select 'NUS','organization' from dual union all
select 'NVART','organization' from dual union all
select 'NWC','launch vehicle manufacturer' from dual union all
select 'NWC','school' from dual union all
select 'NWNAZ','satellite manufacturer' from dual union all
select 'NWPU','satellite manufacturer' from dual union all
select 'NWURKS','satellite manufacturer' from dual union all
select 'NYSH','organization' from dual union all
select 'NYU','school' from dual union all
select 'NZ','country' from dual union all
select 'NZRD','country' from dual union all
select 'O3B','organization' from dual union all
select 'O3BS','organization' from dual union all
select 'OAKAE','satellite manufacturer' from dual union all
select 'OAR','organization' from dual union all
select 'OATKABL','engine/motor manufacturer' from dual union all
select 'OATKD','launch agency' from dual union all
select 'OATKG','satellite manufacturer' from dual union all
select 'OATKGO','satellite manufacturer' from dual union all
select 'OATKH','engine/motor manufacturer' from dual union all
select 'OATKW','launch vehicle manufacturer' from dual union all
select 'OAZ','satellite manufacturer' from dual union all
select 'OCOS','satellite manufacturer' from dual union all
select 'OERL','satellite manufacturer' from dual union all
select 'OHB','organization' from dual union all
select 'OHB','satellite manufacturer' from dual union all
select 'OIT','organization' from dual union all
select 'OIT','satellite manufacturer' from dual union all
select 'OKB1','engine/motor manufacturer' from dual union all
select 'OKB1','launch vehicle manufacturer' from dual union all
select 'OKB1','organization' from dual union all
select 'OKB1','satellite manufacturer' from dual union all
select 'OKB10','organization' from dual union all
select 'OKB10','satellite manufacturer' from dual union all
select 'OKB117','engine/motor manufacturer' from dual union all
select 'OKB154','engine/motor manufacturer' from dual union all
select 'OKB16','engine/motor manufacturer' from dual union all
select 'OKB1F2','satellite manufacturer' from dual union all
select 'OKB1F3','organization' from dual union all
select 'OKB1F3','satellite manufacturer' from dual union all
select 'OKB2','engine/motor manufacturer' from dual union all
select 'OKB23','launch vehicle manufacturer' from dual union all
select 'OKB276','engine/motor manufacturer' from dual union all
select 'OKB456','engine/motor manufacturer' from dual union all
select 'OKB52','organization' from dual union all
select 'OKB52','satellite manufacturer' from dual union all
select 'OKB586','launch vehicle manufacturer' from dual union all
select 'OKB586','organization' from dual union all
select 'OKB586','satellite manufacturer' from dual union all
select 'OKBZ154','engine/motor manufacturer' from dual union all
select 'OKSU','school' from dual union all
select 'OLIN','engine/motor manufacturer' from dual union all
select 'OMLGM','launch vehicle manufacturer' from dual union all
select 'ONERA','engine/motor manufacturer' from dual union all
select 'ONERA','launch agency' from dual union all
select 'ONERA','launch vehicle manufacturer' from dual union all
select 'ONERA','school' from dual union all
select 'ONR','launch agency' from dual union all
select 'ONR','school' from dual union all
select 'ONUSJ','school' from dual union all
select 'OPM','school' from dual union all
select 'OPT','organization' from dual union all
select 'OPTS','organization' from dual union all
select 'OPU','organization' from dual union all
select 'OPU','satellite manufacturer' from dual union all
select 'ORBC','organization' from dual union all
select 'ORBI','organization' from dual union all
select 'ORD','launch agency' from dual union all
select 'ORD','school' from dual union all
select 'ORDRSA','launch agency' from dual union all
select 'ORDRSA','launch vehicle manufacturer' from dual union all
select 'ORDRSA','school' from dual union all
select 'OREN','launch vehicle manufacturer' from dual union all
select 'ORI','organization' from dual union all
select 'ORIAS','school' from dual union all
select 'ORL','school' from dual union all
select 'OSAKA','school' from dual union all
select 'OSC','launch agency' from dual union all
select 'OSC','launch vehicle manufacturer' from dual union all
select 'OSC','organization' from dual union all
select 'OSCAR','organization' from dual union all
select 'OSCAR','satellite manufacturer' from dual union all
select 'OSCC','engine/motor manufacturer' from dual union all
select 'OSCC','launch agency' from dual union all
select 'OSCC','launch vehicle manufacturer' from dual union all
select 'OSCC','satellite manufacturer' from dual union all
select 'OSCD','engine/motor manufacturer' from dual union all
select 'OSCD','launch agency' from dual union all
select 'OSCD','organization' from dual union all
select 'OSCD','satellite manufacturer' from dual union all
select 'OSCG','organization' from dual union all
select 'OSCG','satellite manufacturer' from dual union all
select 'OSCGA','satellite manufacturer' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'OSCM','satellite manufacturer' from dual union all
select 'OSCV','launch vehicle manufacturer' from dual union all
select 'OSCW','launch vehicle manufacturer' from dual union all
select 'OSLO','school' from dual union all
select 'OSN','organization' from dual union all
select 'OSSI','organization' from dual union all
select 'OSSI','satellite manufacturer' from dual union all
select 'OSU','school' from dual union all
select 'OTR','engine/motor manufacturer' from dual union all
select 'OTR','launch agency' from dual union all
select 'OTR','launch vehicle manufacturer' from dual union all
select 'OTR','school' from dual union all
select 'OTRG','launch agency' from dual union all
select 'OTRG','launch vehicle manufacturer' from dual union all
select 'OX','school' from dual union all
select 'P','country' from dual union all
select 'PACSCI','organization' from dual union all
select 'PAI','launch vehicle manufacturer' from dual union all
select 'PAN','organization' from dual union all
select 'PAR','country' from dual union all
select 'PARA','organization' from dual union all
select 'PARAC','organization' from dual union all
select 'PARD','engine/motor manufacturer' from dual union all
select 'PARD','launch agency' from dual union all
select 'PARD','launch vehicle manufacturer' from dual union all
select 'PARD','school' from dual union all
select 'PARIS','school' from dual union all
select 'PASI','organization' from dual union all
select 'PATRAS','satellite manufacturer' from dual union all
select 'PAVKH','engine/motor manufacturer' from dual union all
select 'PCZ','country' from dual union all
select 'PE','country' from dual union all
select 'PER','organization' from dual union all
select 'PERM','engine/motor manufacturer' from dual union all
select 'PERM2','engine/motor manufacturer' from dual union all
select 'PESU','satellite manufacturer' from dual union all
select 'PFAB','satellite manufacturer' from dual union all
select 'PFRR','launch agency' from dual union all
select 'PG','country' from dual union all
select 'PH','country' from dual union all
select 'PH4','organization' from dual union all
select 'PHFO','satellite manufacturer' from dual union all
select 'PHFP','satellite manufacturer' from dual union all
select 'PHILR','engine/motor manufacturer' from dual union all
select 'PHLO','satellite manufacturer' from dual union all
select 'PHLP','satellite manufacturer' from dual union all
select 'PIHM','engine/motor manufacturer' from dual union all
select 'PIHM','launch agency' from dual union all
select 'PIHM','school' from dual union all
select 'PITT','school' from dual union all
select 'PK','country' from dual union all
select 'PKAFSC','launch agency' from dual union all
select 'PKEL','satellite manufacturer' from dual union all
select 'PKNDC','engine/motor manufacturer' from dual union all
select 'PKNDC','launch vehicle manufacturer' from dual union all
select 'PKO','launch agency' from dual union all
select 'PKO','organization' from dual union all
select 'PKPMO','launch vehicle manufacturer' from dual union all
select 'PL','country' from dual union all
select 'PLA','launch agency' from dual union all
select 'PLA2AC','launch agency' from dual union all
select 'PLABS','organization' from dual union all
select 'PLABS','satellite manufacturer' from dual union all
select 'PLABST','organization' from dual union all
select 'PLABST','satellite manufacturer' from dual union all
select 'PLANS','launch agency' from dual union all
select 'PLARF','launch agency' from dual union all
select 'PLRES','organization' from dual union all
select 'PLRES','satellite manufacturer' from dual union all
select 'PLRL','country' from dual union all
select 'PMR','launch agency' from dual union all
select 'PMR','school' from dual union all
select 'POL','launch vehicle manufacturer' from dual union all
select 'POL','organization' from dual union all
select 'POL','satellite manufacturer' from dual union all
select 'POLK','organization' from dual union all
select 'POLK','satellite manufacturer' from dual union all
select 'POLY','satellite manufacturer' from dual union all
select 'POPACS','organization' from dual union all
select 'PORTO','satellite manufacturer' from dual union all
select 'POUDR','engine/motor manufacturer' from dual union all
select 'PR','country' from dual union all
select 'PRIMVN','engine/motor manufacturer' from dual union all
select 'PRIMX','engine/motor manufacturer' from dual union all
select 'PRIN','launch agency' from dual union all
select 'PRIN','school' from dual union all
select 'PRINCE','engine/motor manufacturer' from dual union all
select 'PRIR','organization' from dual union all
select 'PRL','school' from dual union all
select 'PRO','launch agency' from dual union all
select 'PROG','engine/motor manufacturer' from dual union all
select 'PROG','launch vehicle manufacturer' from dual union all
select 'PROG','satellite manufacturer' from dual union all
select 'PROGT','launch vehicle manufacturer' from dual union all
select 'PROGT','satellite manufacturer' from dual union all
select 'PROTO','organization' from dual union all
select 'PSC','satellite manufacturer' from dual union all
select 'PSN','organization' from dual union all
select 'PSRC','school' from dual union all
select 'PSU','launch agency' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'PSU','school' from dual union all
select 'PT','country' from dual union all
select 'PU','launch agency' from dual union all
select 'PUCP','organization' from dual union all
select 'PUCP','satellite manufacturer' from dual union all
select 'PUMPK','organization' from dual union all
select 'PUMPK','satellite manufacturer' from dual union all
select 'PVO','launch agency' from dual union all
select 'PVO','organization' from dual union all
select 'PWAR','organization' from dual union all
select 'PWAR','satellite manufacturer' from dual union all
select 'PWCSD','engine/motor manufacturer' from dual union all
select 'PWFL','engine/motor manufacturer' from dual union all
select 'PWRD','engine/motor manufacturer' from dual union all
select 'PWRD','satellite manufacturer' from dual union all
select 'QA','country' from dual union all
select 'QB50','organization' from dual union all
select 'QBUS','organization' from dual union all
select 'QINQ','satellite manufacturer' from dual union all
select 'QINS','satellite manufacturer' from dual union all
select 'QTT','organization' from dual union all
select 'QUAFI','organization' from dual union all
select 'QUB','school' from dual union all
select 'QUETZ','organization' from dual union all
select 'QUK','school' from dual union all
select 'RACE','organization' from dual union all
select 'RAE','launch agency' from dual union all
select 'RAE','launch vehicle manufacturer' from dual union all
select 'RAE','organization' from dual union all
select 'RAE','satellite manufacturer' from dual union all
select 'RAE','school' from dual union all
select 'RAF','launch agency' from dual union all
select 'RAFL','engine/motor manufacturer' from dual union all
select 'RAFL','satellite manufacturer' from dual union all
select 'RAFSPO','organization' from dual union all
select 'RAKA','launch agency' from dual union all
select 'RAL','organization' from dual union all
select 'RAL','satellite manufacturer' from dual union all
select 'RAL','school' from dual union all
select 'RAYES','satellite manufacturer' from dual union all
select 'RAYM','satellite manufacturer' from dual union all
select 'RAYTH','launch vehicle manufacturer' from dual union all
select 'RAYTH','satellite manufacturer' from dual union all
select 'RBOW','organization' from dual union all
select 'RCA','satellite manufacturer' from dual union all
select 'RCAF','school' from dual union all
select 'RCAR','launch agency' from dual union all
select 'RCAV','satellite manufacturer' from dual union all
select 'RDYNE','engine/motor manufacturer' from dual union all
select 'RDYNM','engine/motor manufacturer' from dual union all
select 'RDYNN','engine/motor manufacturer' from dual union all
select 'REP','launch vehicle manufacturer' from dual union all
select 'REP','satellite manufacturer' from dual union all
select 'REPF','satellite manufacturer' from dual union all
select 'RESH','organization' from dual union all
select 'RESH','satellite manufacturer' from dual union all
select 'REYE','organization' from dual union all
select 'RGMS','launch agency' from dual union all
select 'RGMS','organization' from dual union all
select 'RGMS','school' from dual union all
select 'RICE','satellite manufacturer' from dual union all
select 'RICE','school' from dual union all
select 'RIKEN','school' from dual union all
select 'RIKK','school' from dual union all
select 'RIMSAT','organization' from dual union all
select 'RIU','school' from dual union all
select 'RKA','launch agency' from dual union all
select 'RKA','organization' from dual union all
select 'RKKE','organization' from dual union all
select 'RKKE','satellite manufacturer' from dual union all
select 'RLABN','engine/motor manufacturer' from dual union all
select 'RLABN','launch agency' from dual union all
select 'RLABN','launch vehicle manufacturer' from dual union all
select 'RLABN','school' from dual union all
select 'RLABU','engine/motor manufacturer' from dual union all
select 'RMAF','organization' from dual union all
select 'RMD','engine/motor manufacturer' from dual union all
select 'RMI','engine/motor manufacturer' from dual union all
select 'RMILD','engine/motor manufacturer' from dual union all
select 'RMIPL','engine/motor manufacturer' from dual union all
select 'RN','launch agency' from dual union all
select 'RNAF','launch agency' from dual union all
select 'RNII','school' from dual union all
select 'RO','country' from dual union all
select 'ROE','school' from dual union all
select 'ROGAU','organization' from dual union all
select 'ROGAU','satellite manufacturer' from dual union all
select 'ROHR','satellite manufacturer' from dual union all
select 'ROISS','satellite manufacturer' from dual union all
select 'RORD','engine/motor manufacturer' from dual union all
select 'RORD','launch vehicle manufacturer' from dual union all
select 'ROSAP','organization' from dual union all
select 'ROSAP','satellite manufacturer' from dual union all
select 'ROSAP','school' from dual union all
select 'ROSTO','organization' from dual union all
select 'ROWAN','satellite manufacturer' from dual union all
select 'ROYAL','organization' from dual union all
select 'RPEW','engine/motor manufacturer' from dual union all
select 'RPI','engine/motor manufacturer' from dual union all
select 'RPI','launch vehicle manufacturer' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'RRC','engine/motor manufacturer' from dual union all
select 'RRF','launch agency' from dual union all
select 'RRL','launch agency' from dual union all
select 'RRL','school' from dual union all
select 'RROY','engine/motor manufacturer' from dual union all
select 'RROYC','engine/motor manufacturer' from dual union all
select 'RROYL','engine/motor manufacturer' from dual union all
select 'RSC','launch agency' from dual union all
select 'RSQ','organization' from dual union all
select 'RSRS','school' from dual union all
select 'RTG','launch agency' from dual union all
select 'RTG','school' from dual union all
select 'RU','country' from dual union all
select 'RUAF','organization' from dual union all
select 'RUAF','satellite manufacturer' from dual union all
select 'RUAG','satellite manufacturer' from dual union all
select 'RUAGZ','satellite manufacturer' from dual union all
select 'RVSN','launch agency' from dual union all
select 'RVSN','organization' from dual union all
select 'RVSNR','launch agency' from dual union all
select 'RWI','launch vehicle manufacturer' from dual union all
select 'RWI','satellite manufacturer' from dual union all
select 'RWID','satellite manufacturer' from dual union all
select 'RWISB','satellite manufacturer' from dual union all
select 'S','country' from dual union all
select 'S394I','launch agency' from dual union all
select 'S4300','launch agency' from dual union all
select 'S7','launch agency' from dual union all
select 'S801','engine/motor manufacturer' from dual union all
select 'SA','country' from dual union all
select 'SAAB','satellite manufacturer' from dual union all
select 'SAAB','school' from dual union all
select 'SAABE','satellite manufacturer' from dual union all
select 'SAABS','satellite manufacturer' from dual union all
select 'SAABSA','satellite manufacturer' from dual union all
select 'SAABSP','satellite manufacturer' from dual union all
select 'SAAF','launch agency' from dual union all
select 'SABCA','satellite manufacturer' from dual union all
select 'SAC','launch agency' from dual union all
select 'SAC','school' from dual union all
select 'SAFRAN','engine/motor manufacturer' from dual union all
select 'SAFSP','organization' from dual union all
select 'SAIC','school' from dual union all
select 'SAMSO','launch agency' from dual union all
select 'SAMSO','launch vehicle manufacturer' from dual union all
select 'SAMSO','organization' from dual union all
select 'SAMSO','school' from dual union all
select 'SAMX','organization' from dual union all
select 'SAND','launch agency' from dual union all
select 'SAND','launch vehicle manufacturer' from dual union all
select 'SAND','school' from dual union all
select 'SANDF','organization' from dual union all
select 'SANH','satellite manufacturer' from dual union all
select 'SANJ','engine/motor manufacturer' from dual union all
select 'SANJ','launch vehicle manufacturer' from dual union all
select 'SANJ9','launch vehicle manufacturer' from dual union all
select 'SANL','launch agency' from dual union all
select 'SANL','launch vehicle manufacturer' from dual union all
select 'SANL','organization' from dual union all
select 'SANL','satellite manufacturer' from dual union all
select 'SAO','satellite manufacturer' from dual union all
select 'SAO','school' from dual union all
select 'SASK','school' from dual union all
select 'SASMAC','organization' from dual union all
select 'SAST','launch agency' from dual union all
select 'SAST','launch vehicle manufacturer' from dual union all
select 'SAST','organization' from dual union all
select 'SAST','satellite manufacturer' from dual union all
select 'SATBYUL','satellite manufacturer' from dual union all
select 'SATEL','organization' from dual union all
select 'SATELOG','organization' from dual union all
select 'SATELOG','satellite manufacturer' from dual union all
select 'SATGE','organization' from dual union all
select 'SATHYA','organization' from dual union all
select 'SBA','launch vehicle manufacturer' from dual union all
select 'SBRC','school' from dual union all
select 'SBS','organization' from dual union all
select 'SCAL','engine/motor manufacturer' from dual union all
select 'SCAL','launch agency' from dual union all
select 'SCAL','launch vehicle manufacturer' from dual union all
select 'SCAL','school' from dual union all
select 'SCC','organization' from dual union all
select 'SCCJ','organization' from dual union all
select 'SCEL','launch agency' from dual union all
select 'SCEL','school' from dual union all
select 'SCHJ','satellite manufacturer' from dual union all
select 'SCSS','satellite manufacturer' from dual union all
select 'SCT','organization' from dual union all
select 'SCU','organization' from dual union all
select 'SCU','satellite manufacturer' from dual union all
select 'SDC','engine/motor manufacturer' from dual union all
select 'SDC','launch agency' from dual union all
select 'SDCO','engine/motor manufacturer' from dual union all
select 'SDCO','launch vehicle manufacturer' from dual union all
select 'SDCO','satellite manufacturer' from dual union all
select 'SDCO','school' from dual union all
select 'SDIO','launch agency' from dual union all
select 'SDIO','organization' from dual union all
select 'SDIO','school' from dual union all
select 'SDLP','satellite manufacturer' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'SDP','engine/motor manufacturer' from dual union all
select 'SDSC','school' from dual union all
select 'SEAL','launch agency' from dual union all
select 'SEALC','launch agency' from dual union all
select 'SEALN','launch agency' from dual union all
select 'SEALP','launch agency' from dual union all
select 'SECM','organization' from dual union all
select 'SECM','satellite manufacturer' from dual union all
select 'SECT','launch agency' from dual union all
select 'SECT','school' from dual union all
select 'SEDSH','organization' from dual union all
select 'SEDSH','satellite manufacturer' from dual union all
select 'SEL','satellite manufacturer' from dual union all
select 'SELEN','satellite manufacturer' from dual union all
select 'SELM','satellite manufacturer' from dual union all
select 'SENER','satellite manufacturer' from dual union all
select 'SEP','engine/motor manufacturer' from dual union all
select 'SEPB','engine/motor manufacturer' from dual union all
select 'SEPBL','engine/motor manufacturer' from dual union all
select 'SEPR','engine/motor manufacturer' from dual union all
select 'SEPRV','engine/motor manufacturer' from dual union all
select 'SEPVR','engine/motor manufacturer' from dual union all
select 'SEREB','launch vehicle manufacturer' from dual union all
select 'SES','organization' from dual union all
select 'SESA','organization' from dual union all
select 'SESBM','organization' from dual union all
select 'SESE','organization' from dual union all
select 'SESG','organization' from dual union all
select 'SESNS','organization' from dual union all
select 'SESS','organization' from dual union all
select 'SESSL','organization' from dual union all
select 'SESW','organization' from dual union all
select 'SESWS','organization' from dual union all
select 'SESWSP','organization' from dual union all
select 'SG','country' from dual union all
select 'SGAU','organization' from dual union all
select 'SGAU','satellite manufacturer' from dual union all
select 'SH','country' from dual union all
select 'SHAB','organization' from dual union all
select 'SHANGT','organization' from dual union all
select 'SHANGT','satellite manufacturer' from dual union all
select 'SHANT','satellite manufacturer' from dual union all
select 'SHAOX','satellite manufacturer' from dual union all
select 'SHAR','launch site' from dual union all
select 'SHAR','launch vehicle manufacturer' from dual union all
select 'SHEF','school' from dual union all
select 'SHIN','organization' from dual union all
select 'SHINSHU','organization' from dual union all
select 'SHINSHU','satellite manufacturer' from dual union all
select 'SHIZ','satellite manufacturer' from dual union all
select 'SIBGAU','satellite manufacturer' from dual union all
select 'SII','organization' from dual union all
select 'SII','satellite manufacturer' from dual union all
select 'SIL','satellite manufacturer' from dual union all
select 'SILK','organization' from dual union all
select 'SIMED','engine/motor manufacturer' from dual union all
select 'SIMED','launch agency' from dual union all
select 'SIMED','launch vehicle manufacturer' from dual union all
select 'SIMED','school' from dual union all
select 'SIMG','organization' from dual union all
select 'SINDO','organization' from dual union all
select 'SINO','organization' from dual union all
select 'SIR','organization' from dual union all
select 'SIRX','organization' from dual union all
select 'SIS','launch agency' from dual union all
select 'SISE','satellite manufacturer' from dual union all
select 'SISP','engine/motor manufacturer' from dual union all
select 'SITAEL','satellite manufacturer' from dual union all
select 'SITP','satellite manufacturer' from dual union all
select 'SJSU','organization' from dual union all
select 'SJSU','satellite manufacturer' from dual union all
select 'SK','country' from dual union all
select 'SKB30','launch agency' from dual union all
select 'SKB30','launch vehicle manufacturer' from dual union all
select 'SKB385','launch vehicle manufacturer' from dual union all
select 'SKB385','satellite manufacturer' from dual union all
select 'SKB385Z','satellite manufacturer' from dual union all
select 'SKC','organization' from dual union all
select 'SKC','satellite manufacturer' from dual union all
select 'SKPJ','organization' from dual union all
select 'SKTEL','organization' from dual union all
select 'SKYB','organization' from dual union all
select 'SKYBOX','organization' from dual union all
select 'SKYBOX','satellite manufacturer' from dual union all
select 'SKYM','organization' from dual union all
select 'SKYTE','organization' from dual union all
select 'SLREC','engine/motor manufacturer' from dual union all
select 'SLU','organization' from dual union all
select 'SLU','satellite manufacturer' from dual union all
select 'SMCTV','organization' from dual union all
select 'SMCXR','organization' from dual union all
select 'SMDC','launch agency' from dual union all
select 'SMDC','organization' from dual union all
select 'SMDC','satellite manufacturer' from dual union all
select 'SMDC','school' from dual union all
select 'SMSA','launch agency' from dual union all
select 'SMSA','school' from dual union all
select 'SNCASE','satellite manufacturer' from dual union all
select 'SNECB','engine/motor manufacturer' from dual union all
select 'SNECBL','engine/motor manufacturer' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'SNECDE','engine/motor manufacturer' from dual union all
select 'SNECMA','engine/motor manufacturer' from dual union all
select 'SNECMA','launch vehicle manufacturer' from dual union all
select 'SNECMS','engine/motor manufacturer' from dual union all
select 'SNECPS','engine/motor manufacturer' from dual union all
select 'SNECV','engine/motor manufacturer' from dual union all
select 'SNECVR','engine/motor manufacturer' from dual union all
select 'SNPE','engine/motor manufacturer' from dual union all
select 'SNU','satellite manufacturer' from dual union all
select 'SNVL','satellite manufacturer' from dual union all
select 'SNVP','satellite manufacturer' from dual union all
select 'SOCOM','organization' from dual union all
select 'SOHLA','organization' from dual union all
select 'SOHLA','satellite manufacturer' from dual union all
select 'SOKA','organization' from dual union all
select 'SOKA','satellite manufacturer' from dual union all
select 'SOLM','organization' from dual union all
select 'SOR','organization' from dual union all
select 'SORUN','organization' from dual union all
select 'SORUN','satellite manufacturer' from dual union all
select 'SOSA','organization' from dual union all
select 'SOTON','school' from dual union all
select 'SOYUZ','engine/motor manufacturer' from dual union all
select 'SPA','satellite manufacturer' from dual union all
select 'SPAC','organization' from dual union all
select 'SPAR','satellite manufacturer' from dual union all
select 'SPARD','satellite manufacturer' from dual union all
select 'SPARM','satellite manufacturer' from dual union all
select 'SPAWSD','organization' from dual union all
select 'SPDEV','engine/motor manufacturer' from dual union all
select 'SPDEV','satellite manufacturer' from dual union all
select 'SPER','satellite manufacturer' from dual union all
select 'SPFL','launch agency' from dual union all
select 'SPFL','launch site' from dual union all
select 'SPFL','launch vehicle manufacturer' from dual union all
select 'SPFL','school' from dual union all
select 'SPG','satellite manufacturer' from dual union all
select 'SPHARM','satellite manufacturer' from dual union all
select 'SPHARS','satellite manufacturer' from dual union all
select 'SPIRE','organization' from dual union all
select 'SPIREG','satellite manufacturer' from dual union all
select 'SPMIC','satellite manufacturer' from dual union all
select 'SPOG','launch agency' from dual union all
select 'SPOT','organization' from dual union all
select 'SPQ','organization' from dual union all
select 'SPQ','satellite manufacturer' from dual union all
select 'SPUT','organization' from dual union all
select 'SPUT','satellite manufacturer' from dual union all
select 'SPVR','organization' from dual union all
select 'SPX','launch agency' from dual union all
select 'SPX','launch vehicle manufacturer' from dual union all
select 'SPX','organization' from dual union all
select 'SPX','satellite manufacturer' from dual union all
select 'SPX','school' from dual union all
select 'SPXES','launch agency' from dual union all
select 'SPXES','satellite manufacturer' from dual union all
select 'SPXS','satellite manufacturer' from dual union all
select 'SPXTX','engine/motor manufacturer' from dual union all
select 'SPXTX','launch agency' from dual union all
select 'SR','country' from dual union all
select 'SRC','launch agency' from dual union all
select 'SRC','organization' from dual union all
select 'SRC','satellite manufacturer' from dual union all
select 'SRCI','launch vehicle manufacturer' from dual union all
select 'SRCI','school' from dual union all
select 'SRDL','launch agency' from dual union all
select 'SRDL','organization' from dual union all
select 'SRDL','satellite manufacturer' from dual union all
select 'SRDL','school' from dual union all
select 'SRI','school' from dual union all
select 'SRII','organization' from dual union all
select 'SRII','satellite manufacturer' from dual union all
select 'SRMU','organization' from dual union all
select 'SRMU','satellite manufacturer' from dual union all
select 'SROE','launch vehicle manufacturer' from dual union all
select 'SS500','launch agency' from dual union all
select 'SSC','launch agency' from dual union all
select 'SSC','launch vehicle manufacturer' from dual union all
select 'SSC','organization' from dual union all
select 'SSC','satellite manufacturer' from dual union all
select 'SSC','school' from dual union all
select 'SSCO','organization' from dual union all
select 'SSDC','launch agency' from dual union all
select 'SSDC','school' from dual union all
select 'SSI','launch agency' from dual union all
select 'SSI','school' from dual union all
select 'SSICE','organization' from dual union all
select 'SSICE','satellite manufacturer' from dual union all
select 'SSICE','school' from dual union all
select 'SSIM','organization' from dual union all
select 'SSL','satellite manufacturer' from dual union all
select 'SSLMAX','satellite manufacturer' from dual union all
select 'SSTARS','organization' from dual union all
select 'SSTARS','satellite manufacturer' from dual union all
select 'SSTL','organization' from dual union all
select 'SSTL','satellite manufacturer' from dual union all
select 'SSTLA','organization' from dual union all
select 'SSTLA','satellite manufacturer' from dual union all
select 'ST2V','organization' from dual union all
select 'STA','engine/motor manufacturer' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'STA','launch agency' from dual union all
select 'STA','launch vehicle manufacturer' from dual union all
select 'STA','school' from dual union all
select 'STADK','organization' from dual union all
select 'STADK','satellite manufacturer' from dual union all
select 'STAN','organization' from dual union all
select 'STAN','satellite manufacturer' from dual union all
select 'STAN','school' from dual union all
select 'STBO','organization' from dual union all
select 'STBO','satellite manufacturer' from dual union all
select 'STEL','organization' from dual union all
select 'STELLT','organization' from dual union all
select 'STEN','satellite manufacturer' from dual union all
select 'STGL','organization' from dual union all
select 'STL','satellite manufacturer' from dual union all
select 'STLC','organization' from dual union all
select 'STMS','satellite manufacturer' from dual union all
select 'STO','school' from dual union all
select 'STONE','organization' from dual union all
select 'STP','organization' from dual union all
select 'STRELA','launch vehicle manufacturer' from dual union all
select 'STSM','launch agency' from dual union all
select 'STSM','organization' from dual union all
select 'STST','engine/motor manufacturer' from dual union all
select 'STST','launch agency' from dual union all
select 'STST','launch vehicle manufacturer' from dual union all
select 'STST','school' from dual union all
select 'SU','country' from dual union all
select 'SU190','satellite manufacturer' from dual union all
select 'SU523','satellite manufacturer' from dual union all
select 'SU782','satellite manufacturer' from dual union all
select 'SUD','engine/motor manufacturer' from dual union all
select 'SUD','launch vehicle manufacturer' from dual union all
select 'SUD','school' from dual union all
select 'SUEL','launch vehicle manufacturer' from dual union all
select 'SUEL','satellite manufacturer' from dual union all
select 'SUI','launch agency' from dual union all
select 'SUI','launch vehicle manufacturer' from dual union all
select 'SUI','satellite manufacturer' from dual union all
select 'SUI','school' from dual union all
select 'SUPA','launch agency' from dual union all
select 'SUPA','launch vehicle manufacturer' from dual union all
select 'SUPA','organization' from dual union all
select 'SUPA','satellite manufacturer' from dual union all
select 'SUPA','school' from dual union all
select 'SUPAER','satellite manufacturer' from dual union all
select 'SUPSI','organization' from dual union all
select 'SUPSI','satellite manufacturer' from dual union all
select 'SURREY','organization' from dual union all
select 'SURREY','satellite manufacturer' from dual union all
select 'SUSX','school' from dual union all
select 'SUT','satellite manufacturer' from dual union all
select 'SVC','launch agency' from dual union all
select 'SVC','launch vehicle manufacturer' from dual union all
select 'SWALES','satellite manufacturer' from dual union all
select 'SWARM','satellite manufacturer' from dual union all
select 'SWRI','school' from dual union all
select 'SWRIB','school' from dual union all
select 'SY','country' from dual union all
select 'SYMC','launch agency' from dual union all
select 'SYMC','launch vehicle manufacturer' from dual union all
select 'SZRTC','engine/motor manufacturer' from dual union all
select 'T','country' from dual union all
select 'TAIFUN','engine/motor manufacturer' from dual union all
select 'TAIFUN','launch vehicle manufacturer' from dual union all
select 'TAMA','organization' from dual union all
select 'TAMA','satellite manufacturer' from dual union all
select 'TAMU','organization' from dual union all
select 'TAMU','satellite manufacturer' from dual union all
select 'TAO','school' from dual union all
select 'TARTU','satellite manufacturer' from dual union all
select 'TAYLU','organization' from dual union all
select 'TAYLU','satellite manufacturer' from dual union all
select 'TBELLA','organization' from dual union all
select 'TBELLA','satellite manufacturer' from dual union all
select 'TC','country' from dual union all
select 'TCAN','organization' from dual union all
select 'TCI','organization' from dual union all
select 'TECN','organization' from dual union all
select 'TECN','satellite manufacturer' from dual union all
select 'TEIK','organization' from dual union all
select 'TEIK','satellite manufacturer' from dual union all
select 'TEIK','school' from dual union all
select 'TEKEV','organization' from dual union all
select 'TELB','organization' from dual union all
select 'TELBR','satellite manufacturer' from dual union all
select 'TELD','organization' from dual union all
select 'TELED','organization' from dual union all
select 'TELK','organization' from dual union all
select 'TELMX','organization' from dual union all
select 'TEMIS','satellite manufacturer' from dual union all
select 'TERMAB','satellite manufacturer' from dual union all
select 'TERRE','organization' from dual union all
select 'TESLA','satellite manufacturer' from dual union all
select 'TETHA','satellite manufacturer' from dual union all
select 'TF','country' from dual union all
select 'TF88','launch agency' from dual union all
select 'TF88','school' from dual union all
select 'THAIC','organization' from dual union all
select 'THALA','satellite manufacturer' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'THALES','satellite manufacturer' from dual union all
select 'THALR','satellite manufacturer' from dual union all
select 'THK','engine/motor manufacturer' from dual union all
select 'THKE','engine/motor manufacturer' from dual union all
select 'THKE','launch vehicle manufacturer' from dual union all
select 'THKU','engine/motor manufacturer' from dual union all
select 'THKU','satellite manufacturer' from dual union all
select 'THKX','engine/motor manufacturer' from dual union all
select 'THKX','launch vehicle manufacturer' from dual union all
select 'THKX','satellite manufacturer' from dual union all
select 'THKXM','engine/motor manufacturer' from dual union all
select 'THUR','organization' from dual union all
select 'THURY','organization' from dual union all
select 'TIANJ','engine/motor manufacturer' from dual union all
select 'TIFR','school' from dual union all
select 'TIT','organization' from dual union all
select 'TIT','satellite manufacturer' from dual union all
select 'TITV','control center' from dual union all
select 'TJHS','organization' from dual union all
select 'TJHS','satellite manufacturer' from dual union all
select 'TLS','school' from dual union all
select 'TM','country' from dual union all
select 'TMI','organization' from dual union all
select 'TMSAT','organization' from dual union all
select 'TMTEL','organization' from dual union all
select 'TMU','school' from dual union all
select 'TNO','satellite manufacturer' from dual union all
select 'TNOR','organization' from dual union all
select 'TO','country' from dual union all
select 'TOHO','organization' from dual union all
select 'TOHO','satellite manufacturer' from dual union all
select 'TOHO','school' from dual union all
select 'TOK','organization' from dual union all
select 'TOK','satellite manufacturer' from dual union all
select 'TOK','school' from dual union all
select 'TOKAI','satellite manufacturer' from dual union all
select 'TOKAI','school' from dual union all
select 'TONG','organization' from dual union all
select 'TORP','organization' from dual union all
select 'TORP','satellite manufacturer' from dual union all
select 'TOSH','satellite manufacturer' from dual union all
select 'TOYA','school' from dual union all
select 'TPS','organization' from dual union all
select 'TPS','satellite manufacturer' from dual union all
select 'TPS','school' from dual union all
select 'TR','country' from dual union all
select 'TROM','school' from dual union all
select 'TRORB','organization' from dual union all
select 'TRW','engine/motor manufacturer' from dual union all
select 'TRW','launch vehicle manufacturer' from dual union all
select 'TRW','organization' from dual union all
select 'TRW','satellite manufacturer' from dual union all
select 'TRW','school' from dual union all
select 'TRWAA','satellite manufacturer' from dual union all
select 'TRWC','satellite manufacturer' from dual union all
select 'TRWESL','satellite manufacturer' from dual union all
select 'TRWRC','satellite manufacturer' from dual union all
select 'TSAO','launch agency' from dual union all
select 'TSAO','school' from dual union all
select 'TSBM','organization' from dual union all
select 'TSCJ','organization' from dual union all
select 'TSHUA','organization' from dual union all
select 'TSHUA','satellite manufacturer' from dual union all
select 'TSKB7','engine/motor manufacturer' from dual union all
select 'TSKB7','satellite manufacturer' from dual union all
select 'TSKBE3','organization' from dual union all
select 'TSKBE3','satellite manufacturer' from dual union all
select 'TSKBEM','organization' from dual union all
select 'TSKBEM','satellite manufacturer' from dual union all
select 'TSKBM','organization' from dual union all
select 'TSKBM','satellite manufacturer' from dual union all
select 'TSKY','organization' from dual union all
select 'TSKYBR','organization' from dual union all
select 'TSN108','organization' from dual union all
select 'TSPZ','organization' from dual union all
select 'TSPZ','satellite manufacturer' from dual union all
select 'TSROK','control center' from dual union all
select 'TSSKB','organization' from dual union all
select 'TSSKB','satellite manufacturer' from dual union all
select 'TSSKBP','organization' from dual union all
select 'TSSKBP','satellite manufacturer' from dual union all
select 'TSUK','satellite manufacturer' from dual union all
select 'TSUK','school' from dual union all
select 'TSUKOS','launch agency' from dual union all
select 'TSUKOS','organization' from dual union all
select 'TSUKU','organization' from dual union all
select 'TSUKU','satellite manufacturer' from dual union all
select 'TSVS','organization' from dual union all
select 'TTPI','country' from dual union all
select 'TTY','school' from dual union all
select 'TUAF','launch agency' from dual union all
select 'TUAFA','satellite manufacturer' from dual union all
select 'TUB','organization' from dual union all
select 'TUB','satellite manufacturer' from dual union all
select 'TUB','school' from dual union all
select 'TUBAFA','satellite manufacturer' from dual union all
select 'TUBI','school' from dual union all
select 'TUBTK','organization' from dual union all
select 'TUBTU','organization' from dual union all
select 'TUBTU','satellite manufacturer' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'TUC','school' from dual union all
select 'TUD','organization' from dual union all
select 'TUD','satellite manufacturer' from dual union all
select 'TUG','organization' from dual union all
select 'TUG','satellite manufacturer' from dual union all
select 'TUI','organization' from dual union all
select 'TUI','satellite manufacturer' from dual union all
select 'TUM','organization' from dual union all
select 'TUM','satellite manufacturer' from dual union all
select 'TUMED','organization' from dual union all
select 'TURAB','organization' from dual union all
select 'TURKP','organization' from dual union all
select 'TURKS','organization' from dual union all
select 'TURKT','organization' from dual union all
select 'TUSAS','satellite manufacturer' from dual union all
select 'TW','country' from dual union all
select 'TXWC','launch agency' from dual union all
select 'TXWC','school' from dual union all
select 'TYVAK','satellite manufacturer' from dual union all
select 'UA','country' from dual union all
select 'UAE','country' from dual union all
select 'UAF','launch agency' from dual union all
select 'UAF','organization' from dual union all
select 'UAF','satellite manufacturer' from dual union all
select 'UAF','school' from dual union all
select 'UAH','organization' from dual union all
select 'UAH','satellite manufacturer' from dual union all
select 'UAH','school' from dual union all
select 'UAHCM','school' from dual union all
select 'UAP','organization' from dual union all
select 'UAP','satellite manufacturer' from dual union all
select 'UAZ','organization' from dual union all
select 'UAZ','satellite manufacturer' from dual union all
select 'UAZ','school' from dual union all
select 'UBC','school' from dual union all
select 'UCAN','launch agency' from dual union all
select 'UCAN','school' from dual union all
select 'UCB','organization' from dual union all
select 'UCB','satellite manufacturer' from dual union all
select 'UCB','school' from dual union all
select 'UCL','launch agency' from dual union all
select 'UCL','school' from dual union all
select 'UCO','organization' from dual union all
select 'UCO','satellite manufacturer' from dual union all
select 'UCO','school' from dual union all
select 'UCSD','school' from dual union all
select 'UCW','school' from dual union all
select 'UDAYT','school' from dual union all
select 'UFA','satellite manufacturer' from dual union all
select 'UFL','organization' from dual union all
select 'UFL','satellite manufacturer' from dual union all
select 'UFL','school' from dual union all
select 'UFSM','organization' from dual union all
select 'UFSM','satellite manufacturer' from dual union all
select 'UGATU','organization' from dual union all
select 'UGATU','satellite manufacturer' from dual union all
select 'UHI','organization' from dual union all
select 'UHI','satellite manufacturer' from dual union all
select 'UHI','school' from dual union all
select 'UIA','satellite manufacturer' from dual union all
select 'UIA','school' from dual union all
select 'UIO','launch agency' from dual union all
select 'UIO','school' from dual union all
select 'UIUC','organization' from dual union all
select 'UIUC','satellite manufacturer' from dual union all
select 'UIUC','school' from dual union all
select 'UK','country' from dual union all
select 'UKMO','launch agency' from dual union all
select 'UKMO','school' from dual union all
select 'UKNR','launch agency' from dual union all
select 'UKRN','launch agency' from dual union all
select 'UKS','organization' from dual union all
select 'UKS','satellite manufacturer' from dual union all
select 'UKSA','organization' from dual union all
select 'UKSPOC','organization' from dual union all
select 'UKTSB','organization' from dual union all
select 'UKY','school' from dual union all
select 'ULAB','launch agency' from dual union all
select 'ULAL','launch agency' from dual union all
select 'ULLAF','organization' from dual union all
select 'ULLAF','satellite manufacturer' from dual union all
select 'ULM','school' from dual union all
select 'UM67','country' from dual union all
select 'UM79','country' from dual union all
select 'UMAL','school' from dual union all
select 'UMBC','organization' from dual union all
select 'UMBC','satellite manufacturer' from dual union all
select 'UMD','launch agency' from dual union all
select 'UMD','launch vehicle manufacturer' from dual union all
select 'UMD','school' from dual union all
select 'UMI','launch agency' from dual union all
select 'UMI','launch vehicle manufacturer' from dual union all
select 'UMI','organization' from dual union all
select 'UMI','satellite manufacturer' from dual union all
select 'UMI','school' from dual union all
select 'UMIAMI','school' from dual union all
select 'UMN','school' from dual union all
select 'UNAM','organization' from dual union all
select 'UNAM','satellite manufacturer' from dual union all
select 'UNAM','school' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'UNB','organization' from dual union all
select 'UNB','satellite manufacturer' from dual union all
select 'UNH','organization' from dual union all
select 'UNH','satellite manufacturer' from dual union all
select 'UNH','school' from dual union all
select 'UNIPE','organization' from dual union all
select 'UNIPE','satellite manufacturer' from dual union all
select 'UNIS','organization' from dual union all
select 'UNIS','satellite manufacturer' from dual union all
select 'UNK','engine/motor manufacturer' from dual union all
select 'UNK','launch agency' from dual union all
select 'UNK','launch vehicle manufacturer' from dual union all
select 'UNK','organization' from dual union all
select 'UNK','satellite manufacturer' from dual union all
select 'UNK','school' from dual union all
select 'UNKS','launch agency' from dual union all
select 'UNKS','organization' from dual union all
select 'UNKSR','launch agency' from dual union all
select 'UNKSR','organization' from dual union all
select 'UNKUS','satellite manufacturer' from dual union all
select 'UNKUS1','satellite manufacturer' from dual union all
select 'UNMC','organization' from dual union all
select 'UNMC','satellite manufacturer' from dual union all
select 'UNRV','launch agency' from dual union all
select 'UNSW','satellite manufacturer' from dual union all
select 'UNSWC','satellite manufacturer' from dual union all
select 'UOS','organization' from dual union all
select 'UOS','satellite manufacturer' from dual union all
select 'UPAE','engine/motor manufacturer' from dual union all
select 'UPAE','launch agency' from dual union all
select 'UPAE','launch vehicle manufacturer' from dual union all
select 'UPAE','school' from dual union all
select 'UPC','school' from dual union all
select 'UPD','satellite manufacturer' from dual union all
select 'UPM','organization' from dual union all
select 'UPM','satellite manufacturer' from dual union all
select 'UQU','launch agency' from dual union all
select 'UQU','school' from dual union all
select 'URDEI','organization' from dual union all
select 'URUGUS','satellite manufacturer' from dual union all
select 'US','country' from dual union all
select 'US7A','launch agency' from dual union all
select 'US8A','launch agency' from dual union all
select 'USA','launch agency' from dual union all
select 'USA','organization' from dual union all
select 'USA','school' from dual union all
select 'USAF','launch agency' from dual union all
select 'USAF','organization' from dual union all
select 'USAF','school' from dual union all
select 'USAFA','launch agency' from dual union all
select 'USAFA','launch vehicle manufacturer' from dual union all
select 'USAFA','organization' from dual union all
select 'USAFA','satellite manufacturer' from dual union all
select 'USAFA','school' from dual union all
select 'USCG','organization' from dual union all
select 'USCLA','organization' from dual union all
select 'USCLA','satellite manufacturer' from dual union all
select 'USCLA','school' from dual union all
select 'USCMR','organization' from dual union all
select 'USCMR','satellite manufacturer' from dual union all
select 'USDMA','organization' from dual union all
select 'USEF','organization' from dual union all
select 'USERAR','organization' from dual union all
select 'USERAR','satellite manufacturer' from dual union all
select 'USGS','organization' from dual union all
select 'USMA','organization' from dual union all
select 'USMA','satellite manufacturer' from dual union all
select 'USN','engine/motor manufacturer' from dual union all
select 'USN','launch agency' from dual union all
select 'USN','launch vehicle manufacturer' from dual union all
select 'USN','organization' from dual union all
select 'USN','school' from dual union all
select 'USNA','organization' from dual union all
select 'USNA','satellite manufacturer' from dual union all
select 'USNPS','organization' from dual union all
select 'USNPS','satellite manufacturer' from dual union all
select 'USNTB','school' from dual union all
select 'USTC','satellite manufacturer' from dual union all
select 'USU','organization' from dual union all
select 'USU','satellite manufacturer' from dual union all
select 'USU','school' from dual union all
select 'USYD','satellite manufacturer' from dual union all
select 'UT','organization' from dual union all
select 'UT','satellite manufacturer' from dual union all
select 'UTC','engine/motor manufacturer' from dual union all
select 'UTC','satellite manufacturer' from dual union all
select 'UTCISR','satellite manufacturer' from dual union all
select 'UTCWL','satellite manufacturer' from dual union all
select 'UTD','school' from dual union all
select 'UTE','satellite manufacturer' from dual union all
select 'UTEP','school' from dual union all
select 'UTIAS','organization' from dual union all
select 'UTIAS','satellite manufacturer' from dual union all
select 'UTIAS0','organization' from dual union all
select 'UTIAS0','satellite manufacturer' from dual union all
select 'UTOR','school' from dual union all
select 'UTR','school' from dual union all
select 'UUT','school' from dual union all
select 'UVA','school' from dual union all
select 'UVIE','organization' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'UVT','school' from dual union all
select 'UWA','school' from dual union all
select 'UWI','satellite manufacturer' from dual union all
select 'UWI','school' from dual union all
select 'UWO','school' from dual union all
select 'UY','country' from dual union all
select 'UZKA','launch agency' from dual union all
select 'VANDU','organization' from dual union all
select 'VAP','school' from dual union all
select 'VAST','organization' from dual union all
select 'VATP','satellite manufacturer' from dual union all
select 'VE','country' from dual union all
select 'VENT','engine/motor manufacturer' from dual union all
select 'VENT','launch vehicle manufacturer' from dual union all
select 'VER','satellite manufacturer' from dual union all
select 'VERSI','satellite manufacturer' from dual union all
select 'VFW','satellite manufacturer' from dual union all
select 'VFW2','satellite manufacturer' from dual union all
select 'VFWF','satellite manufacturer' from dual union all
select 'VIA','organization' from dual union all
select 'VIGO','organization' from dual union all
select 'VIGO','satellite manufacturer' from dual union all
select 'VILN','organization' from dual union all
select 'VILN','satellite manufacturer' from dual union all
select 'VISTE','organization' from dual union all
select 'VITA','organization' from dual union all
select 'VKI','organization' from dual union all
select 'VKS','launch agency' from dual union all
select 'VKS','organization' from dual union all
select 'VKSR','launch agency' from dual union all
select 'VKSR','organization' from dual union all
select 'VMF','launch agency' from dual union all
select 'VMF','organization' from dual union all
select 'VMFR','launch agency' from dual union all
select 'VMFR','organization' from dual union all
select 'VN','country' from dual union all
select 'VNIEM','organization' from dual union all
select 'VNIEM','satellite manufacturer' from dual union all
select 'VNIEMR','organization' from dual union all
select 'VNIEMR','satellite manufacturer' from dual union all
select 'VNIIEMI','satellite manufacturer' from dual union all
select 'VNPT','organization' from dual union all
select 'VNSC','organization' from dual union all
select 'VNSC','satellite manufacturer' from dual union all
select 'VOENT','organization' from dual union all
select 'VORO','engine/motor manufacturer' from dual union all
select 'VOTK','engine/motor manufacturer' from dual union all
select 'VOTK','launch vehicle manufacturer' from dual union all
select 'VOUA','engine/motor manufacturer' from dual union all
select 'VOUA','launch vehicle manufacturer' from dual union all
select 'VSSC','engine/motor manufacturer' from dual union all
select 'VSSC','launch vehicle manufacturer' from dual union all
select 'VSSC','satellite manufacturer' from dual union all
select 'VSSC','school' from dual union all
select 'VTC','organization' from dual union all
select 'VTC','satellite manufacturer' from dual union all
select 'VTS','organization' from dual union all
select 'VTSR','organization' from dual union all
select 'VULT','launch vehicle manufacturer' from dual union all
select 'VVKO','launch agency' from dual union all
select 'VVKOV','launch agency' from dual union all
select 'VVS','launch agency' from dual union all
select 'VVSR','launch agency' from dual union all
select 'VZLU','satellite manufacturer' from dual union all
select 'WADC','school' from dual union all
select 'WADD','launch agency' from dual union all
select 'WADD','school' from dual union all
select 'WAKA','organization' from dual union all
select 'WAKA','satellite manufacturer' from dual union all
select 'WAML','school' from dual union all
select 'WASEDA','organization' from dual union all
select 'WASEDA','satellite manufacturer' from dual union all
select 'WBLU','organization' from dual union all
select 'WEB','organization' from dual union all
select 'WEB','satellite manufacturer' from dual union all
select 'WEBC','organization' from dual union all
select 'WEBC','satellite manufacturer' from dual union all
select 'WEHR','launch agency' from dual union all
select 'WEHR','launch vehicle manufacturer' from dual union all
select 'WEINA','organization' from dual union all
select 'WELEC','launch vehicle manufacturer' from dual union all
select 'WER271-1','launch agency' from dual union all
select 'WER444','launch agency' from dual union all
select 'WER485','launch agency' from dual union all
select 'WER485-1','launch agency' from dual union all
select 'WER485-2','launch agency' from dual union all
select 'WER485-3','launch agency' from dual union all
select 'WER500','launch agency' from dual union all
select 'WER836','launch agency' from dual union all
select 'WER836-1','launch agency' from dual union all
select 'WER836-2','launch agency' from dual union all
select 'WER836-3','launch agency' from dual union all
select 'WER902','launch agency' from dual union all
select 'WFC','launch agency' from dual union all
select 'WFC','launch vehicle manufacturer' from dual union all
select 'WFC','organization' from dual union all
select 'WFF','launch agency' from dual union all
select 'WFF','launch vehicle manufacturer' from dual union all
select 'WFF','school' from dual union all
select 'WI','launch site' from dual;
insert into ORGANIZATION_ORG_TYPE
select 'WI','organization' from dual union all
select 'WNI','organization' from dual union all
select 'WORSP','organization' from dual union all
select 'WORSP1','organization' from dual union all
select 'WRE','engine/motor manufacturer' from dual union all
select 'WRE','launch agency' from dual union all
select 'WRE','launch vehicle manufacturer' from dual union all
select 'WRE','organization' from dual union all
select 'WRE','satellite manufacturer' from dual union all
select 'WRE','school' from dual union all
select 'WSMC','launch agency' from dual union all
select 'WSMC','organization' from dual union all
select 'WSMR','launch agency' from dual union all
select 'WSMR','school' from dual union all
select 'WSPG','launch agency' from dual union all
select 'WTA','satellite manufacturer' from dual union all
select 'WTR','launch agency' from dual union all
select 'WTR','launch site' from dual union all
select 'WUHAN','satellite manufacturer' from dual union all
select 'WUPP','school' from dual union all
select 'WURZ','organization' from dual union all
select 'WURZ','satellite manufacturer' from dual union all
select 'WUTC','organization' from dual union all
select 'WXB','launch agency' from dual union all
select 'X','country' from dual union all
select 'XCOR','engine/motor manufacturer' from dual union all
select 'XCYS','satellite manufacturer' from dual union all
select 'XIDI','organization' from dual union all
select 'XIDI','satellite manufacturer' from dual union all
select 'XJRY','launch vehicle manufacturer' from dual union all
select 'XMR','organization' from dual union all
select 'XTAR','organization' from dual union all
select 'YE','country' from dual union all
select 'YERK','school' from dual union all
select 'YKVZ','engine/motor manufacturer' from dual union all
select 'YONS','satellite manufacturer' from dual union all
select 'YORK','school' from dual union all
select 'YORKSS','satellite manufacturer' from dual union all
select 'YSP','launch agency' from dual union all
select 'YUZGU','satellite manufacturer' from dual union all
select 'YUZH','engine/motor manufacturer' from dual union all
select 'YUZH','launch vehicle manufacturer' from dual union all
select 'YUZH','organization' from dual union all
select 'YUZH','satellite manufacturer' from dual union all
select 'YUZHM','engine/motor manufacturer' from dual union all
select 'ZA','country' from dual union all
select 'ZAV23','launch vehicle manufacturer' from dual union all
select 'ZAV23','satellite manufacturer' from dual union all
select 'ZAV47','launch vehicle manufacturer' from dual union all
select 'ZDFH','satellite manufacturer' from dual union all
select 'ZHEJ','organization' from dual union all
select 'ZHEJ','satellite manufacturer' from dual union all
select 'ZHELZ','organization' from dual union all
select 'ZHUORB','organization' from dual union all
select 'ZIKH','launch vehicle manufacturer' from dual union all
select 'ZIKH','organization' from dual union all
select 'ZIKH','satellite manufacturer' from dual union all
select 'ZIMN','launch vehicle manufacturer' from dual union all
select 'ZLZB','launch agency' from dual union all
select 'ZR','country' from dual union all
select 'ZTZM','organization' from dual union all
select 'ZVEZ','satellite manufacturer' from dual union all
select 'ZZB','launch agency' from dual union all
select 'ZZB','organization' from dual union all
select 'ZZWYZ','organization' from dual;


--------------------------------------------------------------------------------
-- SITE
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing SITE...');

CREATE TABLE "SITE" 
 (	"SITE_ID" NUMBER, 
	"SITE_NAME" VARCHAR2(9), 
	"SITE_CODE" VARCHAR2(13), 
	"SITE_TYPE" VARCHAR2(32), 
	"STATE_ORG_CODE" VARCHAR2(9), 
	"START_DATE" DATE, 
	"STOP_DATE" DATE, 
	"SITE_SHORT_NAME" VARCHAR2(18), 
	"SITE_FULL_NAME" VARCHAR2(81), 
	"SITE_LOCATION" VARCHAR2(53), 
	"LONGITUDE" NUMBER, 
	"LATITUDE" NUMBER, 
	"DEGREES_UNCERTAINTY" NUMBER, 
	 CONSTRAINT "SITE_PK" PRIMARY KEY ("SITE_ID")
USING INDEX  ENABLE, 
	 CONSTRAINT "SITE_ORG_FK" FOREIGN KEY ("STATE_ORG_CODE")
	  REFERENCES "ORGANIZATION" ("ORG_CODE") ENABLE
 ) ;


CREATE INDEX "SITE_IDX" ON "SITE" ("STATE_ORG_CODE") 
;
insert into SITE
select 1,'AAAF','','launch site','US','1941-04-13 00:00:00','1949-12-02 00:00:00','AAAF','Alamagordo Army Air Field','',-106.103,32.8496,.0001 from dual union all
select 10,'ALCA','RAG','launch point','','1994-08-19 00:00:00','','ALCA  RAG','RAG launcher, CL Alcantara','Alcantara, Maranhao, Brasil',-44.3675,-2.3148,.0001 from dual union all
select 100,'AOR13','LP3','launch point','','1955-09-24 00:00:00','1955-09-24 00:00:00','AOR13 LP3','Atlantic Ocean Launch Site 13 Launch Point 3','Atlantic Ocean',-59.42,45.28,.01 from dual union all
select 1000,'NOVOSY','','launch site','SU','1969-01-01 00:00:00','1969-01-01 00:00:00','NOVOSY','Novosysoyevka missile base','',133.36,44.24,.2 from dual union all
select 1001,'NPOLE','','launch zone','RU','1995-01-01 00:00:00','1995-01-01 00:00:00','NPOLE -','North polar submarine launch area','',0,90,1 from dual union all
select 1002,'NTAG','','launch site','SU','1960-01-01 00:00:00','1990-01-01 00:00:00','NTAG','Nizhniy Tagil missile base, Russia  (Verhknaya Salda)','',60.364,58.096,.01 from dual union all
select 1003,'NTRURO','','launch site','US','1969-11-01 00:00:00','1970-02-01 00:00:00','NTRURO','North Truro Air Force Station','',-70.05,42.03,.01 from dual union all
select 1004,'NTS','Area 26','launch point','','1997-02-01 00:00:00','','NTS   Area 26','Sandia Wahmonie Launch Facility, Area 26, Nevada Test Site','',-116.11,36.75,.04 from dual union all
select 1005,'NTS','','launch site','US','1997-02-01 00:00:00','','NTS   Area 26','Sandia Wahmonie Launch Facility, Area 26, Nevada Test Site','',-116.11,36.75,.04 from dual union all
select 1006,'NZEM','','launch site','SU','1961-01-01 00:00:00','1961-01-01 00:00:00','NZ    -','Novaya Zemlya test site','',54.6,70.7,.1 from dual union all
select 1007,'OBA','','launch site','J','1961-06-01 00:00:00','1961-01-01 00:00:00','OBA   -','Obuchi (Obachi?), Aomori, Honshu, Nippon','',141.73,40.7,.02 from dual union all
select 1008,'OCEAN','','launch site','EARTH','','','Ocean','Ocean launch sites','',null,null,null from dual union all
select 1009,'OK','KRE','launch point','','','','OK    KRE','Krenkel Polar Station, Ostrov Kheysa','',58.03,80.37,.01 from dual union all
select 101,'AOR13','LP4','launch point','','1955-09-25 00:00:00','1955-09-25 00:00:00','AOR13 LP4','Atlantic Ocean Launch Site 13 Launch Point 4','Atlantic Ocean',-52.5,47.73,.01 from dual union all
select 1010,'OK','','launch site','SU','1956-01-01 00:00:00','1980-01-01 00:00:00','OK    -','Ostrov Kheysa, Zemlya Frantsa-Iosifa','',58.05,80.45,.02 from dual union all
select 1011,'OMLA','','launch zone','SU','1960-01-01 00:00:00','','OMLA  -','Okhotsk Launch Area, Okhotskoye More','',148,57,2 from dual union all
select 1012,'OMSK','','launch site','SU','1960-01-01 00:00:00','1990-01-01 00:00:00','OMSK','Omsk missile base, Russia','',73.635,55.16,.01 from dual union all
select 1013,'ORDZH','','launch site','SU','1961-01-01 00:00:00','1969-01-01 00:00:00','ORDZH','Ordzhonikidze missile base, Ordzhonikidze (Vladikavkaz), S Ossetia','',44.65,43.02,.2 from dual union all
select 1014,'OTB','','launch site','ZA','1980-01-01 00:00:00','1990-01-01 00:00:00','OTB   -','Overberg Toersbaan, Arniston, Waenhuiskrans, Western Cape, South Africa','',20.32,-34.58,.02 from dual union all
select 1015,'PA','LC-A','launch point','','1960-09-19 00:00:00','1964-12-21 00:00:00','PA    LC-A','Launch Complex A, Naval Missile Facility, Point Arguello, California','',-120.5992,34.6591,.005 from dual union all
select 1016,'PA','LC-B','launch point','','1960-07-01 00:00:00','1963-05-11 00:00:00','PA    LC-B','Launch Complex B, Naval Missile Facility, Point Arguello, California','',-120.6017,34.6523,.005 from dual union all
select 1017,'PA','LC-C','launch point','','1962-01-01 00:00:00','1964-07-01 00:00:00','PA    LC-C','Launch Complex C, Naval Missile Facility, Point Arguello, California','',-120.6269,34.6057,.005 from dual union all
select 1018,'PA','LC-D','launch point','','1962-04-26 00:00:00','1964-07-01 00:00:00','PA    LC-D','Launch Complex D, Naval Missile Facility, Point Arguello, California','',-120.6234,34.6043,.005 from dual union all
select 1019,'PA','LC1-1','launch point','','1960-10-11 00:00:00','1964-07-10 00:00:00','PA    LC1-1','Launch Complex 1, Pad 1, Naval Missile Facility, Point Arguello','',-120.5911,34.64,.005 from dual union all
select 102,'AOR13','LP5','launch point','','1955-09-27 00:00:00','1955-09-27 00:00:00','AOR13 LP5','Atlantic Ocean Launch Site 13 Launch Point 5','Atlantic Ocean',-54.23,57.85,.01 from dual union all
select 1020,'PA','LC1-2','launch point','','1961-07-12 00:00:00','1964-07-01 00:00:00','PA    LC1-2','Launch Complex 1, Pad 2, Naval Missile Facility, Point Arguello','',-120.5878,34.6359,.005 from dual union all
select 1021,'PA','LC2-3','launch point','','1963-07-12 00:00:00','1964-07-06 00:00:00','PA    LC2-3','Launch Complex 2, Pad 3, Naval Missile Facility, Point Arguello','',-120.6139,34.629,.005 from dual union all
select 1022,'PA','LC2-4','launch point','','1964-01-01 00:00:00','1964-07-01 00:00:00','PA    LC2-4','Launch Complex 2, Pad 4, Naval Missile Facility, Point Arguello','',-120.6099,34.6276,.005 from dual union all
select 1023,'PA','','launch site','US','1958-01-01 00:00:00','1965-01-01 00:00:00','PA    -','Naval Missile Facility, Point Arguello, California','',-120.6,34.62,.02 from dual union all
select 1024,'PALB','IAFTR','launch point','','1988-01-01 00:00:00','','PALB  IAFTR','Sea Launch Area, Israeli Air Force Test Range, Palmachim Beach, Israel','',34.6,31.9,.02 from dual union all
select 1025,'PALB','SH','launch point','','1988-01-01 00:00:00','','PALB  SH','Shaviyt pad, Israeli Air Force Test Range, Palmachim Beach, Israel','',34.7,31.9,.02 from dual union all
select 1026,'PALB','','launch site','IL','1988-01-01 00:00:00','','PALB  -','Israeli Air Force Test Range, Palmachim Beach, Israel','',34.7,31.9,.02 from dual union all
select 1027,'PASS80A','','launch cruise','SU','1980-02-01 00:00:00','1980-03-01 00:00:00','Passat 80','Passat cruise 1980','',null,null,null from dual union all
select 1028,'PASS82A','','launch cruise','SU','1982-02-01 00:00:00','1982-02-01 00:00:00','Passat 82','Passat cruise 1982','',null,null,null from dual union all
select 1029,'PASS82B','','launch cruise','SU','1982-05-01 00:00:00','1982-05-01 00:00:00','Passat 82','Passat cruise 1982','',null,null,null from dual union all
select 103,'AOR13','LP6','launch point','','1955-09-27 00:00:00','1955-09-27 00:00:00','AOR13 LP6','Atlantic Ocean Launch Site 13 Launch Point 6','Atlantic Ocean',-53.95,58.45,.01 from dual union all
select 1030,'PASS86A','','launch cruise','SU','1986-02-01 00:00:00','1986-03-01 00:00:00','Passat 86','Passat cruise 1986','',null,null,null from dual union all
select 1031,'PASS88A','','launch cruise','SU','1988-03-01 00:00:00','1988-04-01 00:00:00','Passat 88','Passat cruise 1988','',null,null,null from dual union all
select 1032,'PASS88B','','launch cruise','SU','1988-11-01 00:00:00','1988-11-01 00:00:00','Passat 88','Passat cruise 1988','',null,null,null from dual union all
select 1033,'PASS89A','','launch cruise','SU','1989-02-01 00:00:00','1989-02-01 00:00:00','Passat 89','Passat cruise 1989','',null,null,null from dual union all
select 1034,'PASS89B','','launch cruise','SU','1989-05-01 00:00:00','1989-05-01 00:00:00','Passat 89','Passat cruise 1989','',null,null,null from dual union all
select 1035,'PASS90A','','launch cruise','SU','1990-01-01 00:00:00','1990-01-01 00:00:00','Passat 90','Passat cruise 1990','',null,null,null from dual union all
select 1036,'PASS90B','','launch cruise','SU','1990-11-01 00:00:00','1990-11-01 00:00:00','Passat 90','Passat cruise 1990','',null,null,null from dual union all
select 1037,'PAVL','','launch site','SU','1971-01-01 00:00:00','1972-01-01 00:00:00','PAVL','PMZ Pavlograd test site, Ukraine','',38.5,48.9,.2 from dual union all
select 1038,'PAWA','','launch zone','US','1990-01-01 00:00:00','','PAWA  -','Point Arguello Warning Area Drop Zone, Pacific Ocean','',-123,36,.02 from dual union all
select 1039,'PBA','','launch site','US','1965-01-01 00:00:00','1972-01-01 00:00:00','PBA   -','Point Barrow, Alaska','',-156.78,71.03,.02 from dual union all
select 104,'AOR13','LP7','launch point','','1955-09-28 00:00:00','1955-09-28 00:00:00','AOR13 LP7','Atlantic Ocean Launch Site 13 Launch Point 7','Atlantic Ocean',-55.25,61.17,.01 from dual union all
select 1040,'PDI','','launch site','AR','','','PDI   -','Puente del Inca, Mendoza, Argentina','',-69.91,-32.82,.05 from dual union all
select 1041,'PF','LC1','launch point','','1969-01-01 00:00:00','','PF    LC1','MRL 7.5K launcher, Pad 1, Poker Flat Research Range, Alaska','',-147.4882,65.1296,.0001 from dual union all
select 1042,'PF','LC2','launch point','','1969-01-01 00:00:00','','PF    LC2','MRL 7.5K launcher, Pad 2, Poker Flat Research Range, Alaska','',-147.4861,65.1298,.0001 from dual union all
select 1043,'PF','LC3','launch point','','1969-01-01 00:00:00','','PF    LC3','AML 20K launcher, Pad 3, Poker Flat Research Range, Alaska','',-147.4855,65.1298,.0001 from dual union all
select 1044,'PF','LC4','launch point','','1969-01-01 00:00:00','','PF    LC4','AML 20K launcher, Pad 4, Poker Flat Research Range, Alaska','',-147.4832,65.1302,.0001 from dual union all
select 1045,'PF','LC5','launch point','','1969-01-01 00:00:00','','PF    LC5','AML 4K3 launcher, Pad 5, Poker Flat Research Range, Alaska','',-147.4833,65.1292,.0001 from dual union all
select 1046,'PF','LC6','launch point','','1969-01-01 00:00:00','','PF    LC6','Hilltop 7.5K launcher, Pad 6, Poker Flat Research Range, Alaska','',-147.4609,65.117,.0001 from dual union all
select 1047,'PF','','launch site','US','1969-01-01 00:00:00','','PF    -','Poker Flat Research Range, Ft Wainwright, Alaska','',-147.467,65.117,.005 from dual union all
select 1048,'PISQ','','launch site','I','2000-01-01 00:00:00','','PISQ','Poligono Sperimentale e di Addestramento Interforze del Salto di Quirra','',9.43,39.6,.02 from dual union all
select 1049,'PLATT','','launch site','US','','','PLATT -','Plattsburgh AFB, NY','',-73.46,44.66,.01 from dual union all
select 105,'AOR13','LP8','launch point','','1955-09-28 00:00:00','1955-09-28 00:00:00','AOR13 LP8','Atlantic Ocean Launch Site 13 Launch Point 8','Atlantic Ocean',-56.08,62.03,.01 from dual union all
select 1050,'PLM','','launch zone','US','1959-01-01 00:00:00','1967-01-01 00:00:00','PLM   -','Palmdale Omni Drop Zone, Palmdale, California','',-117.97,34.68,.02 from dual union all
select 1051,'PM','B1','launch point','','1957-01-01 00:00:00','','PM    B1','Complex Bravo Pad 1, Point Mugu (Tomahawk/HARPOON)','Pt. Mugu, California',-119.1216,34.0996,.0001 from dual union all
select 1052,'PM','B2','launch point','','1957-01-01 00:00:00','','PM    B2','Complex Bravo Pad 2, Point Mugu','Pt. Mugu, California',-119.1209,34.0992,.0001 from dual union all
select 1053,'PM','C1','launch point','','1957-01-01 00:00:00','','PM    C1','Complex Charlie Pad 1, Point Mugu (Met rocket/AMRAAM)','Pt. Mugu, California',-119.1184,34.0986,.05 from dual union all
select 1054,'PM','C2','launch point','','1957-01-01 00:00:00','','PM    C2','Complex Charlie Pad 2, Point Mugu','Pt. Mugu, California',-119.1184,34.0986,.05 from dual union all
select 1055,'PM','C3','launch point','','1957-01-01 00:00:00','','PM    C3','Complex Charlie Pad 3, Point Mugu','Pt. Mugu, California',-119.1184,34.0986,.05 from dual union all
select 1056,'PM','SLB','launch point','','1957-01-01 00:00:00','','PM    -','Surface Launch Building, Naval Missile and Astronautics Center','Point Mugu, California',-119.1286,34.1103,.0001 from dual union all
select 1057,'PM','','launch site','US','1957-01-01 00:00:00','','PM    -','Naval Missile and Astronautics Center, Point Mugu, California','',-119.12,34.12,.02 from dual union all
select 1058,'PMFA','','launch site','US','1960-03-01 00:00:00','','PMFA  -','POMFLANT, Polaris Missile Facility, Atlantic','Charleston, S Carolina',-79.95,32.85,.01 from dual union all
select 1059,'PMFP','','launch site','US','1966-01-01 00:00:00','','PMFP  -','POMFPAC,  Polaris Missile Facility, Pacific','Bangor, Washington',-122.72,42.72,.01 from dual union all
select 106,'AOR13','LP9','launch point','','1955-09-29 00:00:00','1955-09-29 00:00:00','AOR13 LP9','Atlantic Ocean Launch Site 13 Launch Point 9','Atlantic Ocean',-58,65.65,.01 from dual union all
select 1060,'PMLA','','launch zone','US','1955-01-01 00:00:00','','PMLA  -','Point Mugu Launch Area, Western Test Range, Point Mugu, California','',-119.2,34,.5 from dual union all
select 1061,'PMR','','launch zone','US','1960-01-01 00:00:00','1974-01-01 00:00:00','WTR   -','SLBM Launch Area, Western Test Range, Pacific Ocean','',-120,35,3 from dual union all
select 1062,'POLA','','launch zone','SU','','','POLA  -','Pacific Ocean Launch Area for Soviet/Russian submarines','',null,null,null from dual union all
select 1063,'POR1','','launch cruise','US','1965-10-28 00:00:00','1965-10-28 00:00:00','POR1  -','Hydra Launch Site 2','',-120,30,.5 from dual union all
select 1064,'POR10','','launch cruise','US','1958-10-01 00:00:00','1958-10-01 00:00:00','POR10 -','1958 NRL Cruise','',-165.5,-10.5,1 from dual union all
select 1065,'POR11','LP1','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP1','1965 Croatan MRF Expedition LP1','',-79.38,37.78,.01 from dual union all
select 1066,'POR11','LP10','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP10','1965 Croatan MRF Expedition LP10','',-77.83,-12.5,.01 from dual union all
select 1067,'POR11','LP11','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP11','1965 Croatan MRF Expedition LP11','',-78.25,-12.82,.01 from dual union all
select 1068,'POR11','LP12','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP12','1965 Croatan MRF Expedition LP12','',-77.83,-12.83,.01 from dual union all
select 1069,'POR11','LP13','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP13','1965 Croatan MRF Expedition LP13','',-77.93,-12.8,.01 from dual union all
select 107,'AOR13','','launch cruise','US','1955-09-23 00:00:00','1955-09-23 00:00:00','Staten I1955','USS Staten Island, 1955 Arctic expedition','Atlantic Ocean',-66.18,42.67,.01 from dual union all
select 1070,'POR11','LP14','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP14','1965 Croatan MRF Expedition LP14','',-77.67,-12.67,.01 from dual union all
select 1071,'POR11','LP15','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP15','1965 Croatan MRF Expedition LP15','',-77.98,-12.78,.01 from dual union all
select 1072,'POR11','LP16','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP16','1965 Croatan MRF Expedition LP16','',-77,-12,.01 from dual union all
select 1073,'POR11','LP17','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP17','1965 Croatan MRF Expedition LP17','',-78,-12.92,.01 from dual union all
select 1074,'POR11','LP18','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP18','1965 Croatan MRF Expedition LP18','',-77.97,-12.95,.01 from dual union all
select 1075,'POR11','LP19','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP19','1965 Croatan MRF Expedition LP19','',-77.9,-12.9,.01 from dual union all
select 1076,'POR11','LP2','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP2','1965 Croatan MRF Expedition LP2','',-84,.01,.1 from dual union all
select 1077,'POR11','LP20','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP20','1965 Croatan MRF Expedition LP20','',-77.83,-12.95,.01 from dual union all
select 1078,'POR11','LP21','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP21','1965 Croatan MRF Expedition LP21','',-78.6,-11.73,.01 from dual union all
select 1079,'POR11','LP22','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP22','1965 Croatan MRF Expedition LP22','',-78.33,-11.83,.01 from dual union all
select 108,'AOR14','LP1','launch point','','1956-10-27 00:00:00','1956-11-10 00:00:00','AOR14 LP1','Michigan Cruise 1956, Launch Point 1','Atlantic Ocean',-71,40,.5 from dual union all
select 1080,'POR11','LP23','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP23','1965 Croatan MRF Expedition LP23','',-78.42,-11.4,.01 from dual union all
select 1081,'POR11','LP24','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP24','1965 Croatan MRF Expedition LP24','',-80,-9.17,.01 from dual union all
select 1082,'POR11','LP25','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP25','1965 Croatan MRF Expedition LP25','',-79.83,-10.17,.01 from dual union all
select 1083,'POR11','LP26','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP26','1965 Croatan MRF Expedition LP26','',-79.45,-10.17,.01 from dual union all
select 1084,'POR11','LP27','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP27','1965 Croatan MRF Expedition LP27','',-78.75,-12.18,.01 from dual union all
select 1085,'POR11','LP28','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP28','1965 Croatan MRF Expedition LP28','',-77.83,-14,.01 from dual union all
select 1086,'POR11','LP29','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP29','1965 Croatan MRF Expedition LP29','',-77.98,-15.98,.01 from dual union all
select 1087,'POR11','LP3','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP3','1965 Croatan MRF Expedition LP3','',-84,-1,.01 from dual union all
select 1088,'POR11','LP30','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP30','1965 Croatan MRF Expedition LP30','',-78.23,-12.08,.01 from dual;
insert into SITE
select 1089,'POR11','LP31','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP31','1965 Croatan MRF Expedition LP31','',-78.22,-12.12,.01 from dual union all
select 109,'AOR14','LP2','launch point','','1956-10-27 00:00:00','1956-11-10 00:00:00','AOR14 LP2','Michigan Cruise 1956, Launch Point 2','Atlantic Ocean',-48,49,.5 from dual union all
select 1090,'POR11','LP32','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP32','1965 Croatan MRF Expedition LP32','',-76.13,-24.08,.01 from dual union all
select 1091,'POR11','LP33','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP33','1965 Croatan MRF Expedition LP33','',-75.22,-29.57,.01 from dual union all
select 1092,'POR11','LP34','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP34','1965 Croatan MRF Expedition LP34','',-75.03,-30.63,.01 from dual union all
select 1093,'POR11','LP35','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP35','1965 Croatan MRF Expedition LP35','',-74.25,-35.23,.01 from dual union all
select 1094,'POR11','LP36','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP36','1965 Croatan MRF Expedition LP36','',-77.67,-44.25,.01 from dual union all
select 1095,'POR11','LP37','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP37','1965 Croatan MRF Expedition LP37','',-77.78,-44.38,.01 from dual union all
select 1096,'POR11','LP38','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP38','1965 Croatan MRF Expedition LP38','',-78,-58.32,.01 from dual union all
select 1097,'POR11','LP39','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP39','1965 Croatan MRF Expedition LP39','',-78,-60,.01 from dual union all
select 1098,'POR11','LP4','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP4','1965 Croatan MRF Expedition LP4','',-84.37,-3.12,.01 from dual union all
select 1099,'POR11','LP40','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP40','1965 Croatan MRF Expedition LP40','',-77.97,-59.92,.01 from dual union all
select 11,'ALCA','UL','launch point','','1994-08-19 00:00:00','','ALCA  UL','UL launcher, CL Alcantara','Alcantara, Maranhao, Brasil',-44.3676,-2.316,.0001 from dual union all
select 110,'AOR14','LP3','launch point','','1956-10-27 00:00:00','1956-11-10 00:00:00','AOR14 LP3','Michigan Cruise 1956, Launch Point 3','Atlantic Ocean',-47,58,.5 from dual union all
select 1100,'POR11','LP41','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP41','1965 Croatan MRF Expedition LP41','',-78.33,-52.58,.01 from dual union all
select 1101,'POR11','LP42','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP42','1965 Croatan MRF Expedition LP42','',-78.22,-52.52,.01 from dual union all
select 1102,'POR11','LP43','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP43','1965 Croatan MRF Expedition LP43','',-82.77,3.92,.01 from dual union all
select 1103,'POR11','LP44','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP44','1965 Croatan MRF Expedition LP44','',-84.38,-3.07,.01 from dual union all
select 1104,'POR11','LP45','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP45','1965 Croatan MRF Expedition LP45','',-82.17,-9.53,.01 from dual union all
select 1105,'POR11','LP46','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP46','1965 Croatan MRF Expedition LP46','',-78,-12.92,.01 from dual union all
select 1106,'POR11','LP47','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP47','1965 Croatan MRF Expedition LP47','',-78.05,-12.63,.01 from dual union all
select 1107,'POR11','LP48','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP48','1965 Croatan MRF Expedition LP48','',-77.97,-12.82,.01 from dual union all
select 1108,'POR11','LP49','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP49','1965 Croatan MRF Expedition LP49','',-78,-13.1,.01 from dual union all
select 1109,'POR11','LP5','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP5','1965 Croatan MRF Expedition LP5','',-84.53,-6.5,.01 from dual union all
select 111,'AOR14','LP4','launch point','','1956-10-27 00:00:00','1956-11-10 00:00:00','AOR14 LP4','Michigan Cruise 1956, Launch Point 4','Atlantic Ocean',-58,64,.5 from dual union all
select 1110,'POR11','LP50','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP50','1965 Croatan MRF Expedition LP50','',-78.05,-12.95,.01 from dual union all
select 1111,'POR11','LP51','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP51','1965 Croatan MRF Expedition LP51','',-77.9,-12.47,.01 from dual union all
select 1112,'POR11','LP52','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP52','1965 Croatan MRF Expedition LP52','',-78.38,-11.57,.01 from dual union all
select 1113,'POR11','LP53','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP53','1965 Croatan MRF Expedition LP53','',-79.93,-9.03,.01 from dual union all
select 1114,'POR11','LP54','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP54','1965 Croatan MRF Expedition LP54','',-77.98,-14.17,.01 from dual union all
select 1115,'POR11','LP55','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP55','1965 Croatan MRF Expedition LP55','',-78,-16.02,.01 from dual union all
select 1116,'POR11','LP56','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP56','1965 Croatan MRF Expedition LP56','',-78.18,-12.32,.01 from dual union all
select 1117,'POR11','LP57','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP57','1965 Croatan MRF Expedition LP57','',-77.78,-14.57,.01 from dual union all
select 1118,'POR11','LP58','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP58','1965 Croatan MRF Expedition LP58','',-76.02,-24.6,.01 from dual union all
select 1119,'POR11','LP59','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP59','1965 Croatan MRF Expedition LP59','',-75.15,-29.87,.01 from dual union all
select 112,'AOR14','LP5','launch point','','1956-10-27 00:00:00','1956-11-10 00:00:00','AOR14 LP5','Michigan Cruise 1956, Launch Point 5','Atlantic Ocean',-58,66,.5 from dual union all
select 1120,'POR11','LP6','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP6','1965 Croatan MRF Expedition LP6','',-82.43,-9.45,.01 from dual union all
select 1121,'POR11','LP60','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP60','1965 Croatan MRF Expedition LP60','',-75,-30.87,.01 from dual union all
select 1122,'POR11','LP61','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP61','1965 Croatan MRF Expedition LP61','',-74.27,-35.2,.01 from dual union all
select 1123,'POR11','LP62','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP62','1965 Croatan MRF Expedition LP62','',-75.18,-39.12,.01 from dual union all
select 1124,'POR11','LP63','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP63','1965 Croatan MRF Expedition LP63','',-77.75,-47.03,.01 from dual union all
select 1125,'POR11','LP64','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP64','1965 Croatan MRF Expedition LP64','',-77.7,-48.58,.01 from dual union all
select 1126,'POR11','LP65','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP65','1965 Croatan MRF Expedition LP65','',-77.82,-52.18,.01 from dual union all
select 1127,'POR11','LP66','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP66','1965 Croatan MRF Expedition LP66','',-77.97,-59.87,.01 from dual union all
select 1128,'POR11','LP67','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP67','1965 Croatan MRF Expedition LP67','',-77.83,-59.77,.01 from dual union all
select 1129,'POR11','LP68','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP68','1965 Croatan MRF Expedition LP68','',-78.15,-52.47,.01 from dual union all
select 113,'AOR14','','launch cruise','US','1956-10-27 00:00:00','1956-11-10 00:00:00','AOR14','USS Rushmore U. Michigan Cruise 1956','Atlantic Ocean',-71,40,.5 from dual union all
select 1130,'POR11','LP7','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP7','1965 Croatan MRF Expedition LP7','',-82.35,-9.6,.01 from dual union all
select 1131,'POR11','LP8','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP8','1965 Croatan MRF Expedition LP8','',-81.23,-9.53,.01 from dual union all
select 1132,'POR11','LP9','launch point','','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP9','1965 Croatan MRF Expedition LP9','',-81.33,-11.42,.01 from dual union all
select 1133,'POR11','','launch cruise','US','1965-03-01 00:00:00','1965-03-01 00:00:00','POR11 LP1','1965 Croatan MRF Expedition LP1','',-79.38,37.78,.01 from dual union all
select 1134,'POR12','','launch cruise','US','1968-01-01 00:00:00','1968-01-01 00:00:00','POR12 -','1968 LRL Cruise, Pacific Ocean unknown location','',null,null,null from dual union all
select 1135,'POR13','','launch cruise','US','1958-01-01 00:00:00','1958-01-01 00:00:00','POR13 -','1958 Point Defiance Expedition, Danger Is., Pacific Ocean','',-165.48,-10.33,.01 from dual union all
select 1136,'POR15','LP1','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP1','1965 Voyekov expedition LP1','',156.0358,40.0514,.0002 from dual union all
select 1137,'POR15','LP10','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP10','1965 Voyekov expedition LP10 M9053','',180,13.4189,.002 from dual union all
select 1138,'POR15','LP11','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP11','1965 Voyekov expedition LP11 M9054','',180,10.1678,.002 from dual union all
select 1139,'POR15','LP12','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP12','1965 Voyekov expedition LP12 M9055','',180,5.95,.002 from dual union all
select 114,'AOR15','','launch zone','US','1992-01-01 00:00:00','1995-01-01 00:00:00','AOR15 -','USS Turner, Aegis NTW test launch area, 150 km off N Carolina','Atlantic Ocean',-73.689,33.754,.001 from dual union all
select 1140,'POR15','LP13','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP13','1965 Voyekov expedition LP13 M9056','',180,-.7342,.002 from dual union all
select 1141,'POR15','LP14','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP14','1965 Voyekov expedition LP14 M9057','',180,-5.0342,.002 from dual union all
select 1142,'POR15','LP15','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP15','1965 Voyekov expedition LP15 M9058','',-169.0669,-19.8356,.002 from dual union all
select 1143,'POR15','LP16','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP16','1965 Voyekov expedition LP16 M9059','',-162.7833,-19.5014,.002 from dual union all
select 1144,'POR15','LP17','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP17','1965 Voyekov expedition LP17','',-158.4189,-19.6842,.002 from dual union all
select 1145,'POR15','LP18','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP18','1965 Voyekov expedition LP18','',-158.6856,-18.8011,.002 from dual union all
select 1146,'POR15','LP19','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP19','1965 Voyekov expedition LP19','',-158.6856,-18.8011,.002 from dual union all
select 1147,'POR15','LP2','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP2','1965 Voyekov expedition LP2','',161.0672,40.1192,.0002 from dual union all
select 1148,'POR15','LP20','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP20','1965 Voyekov expedition LP20','',-153.0853,-19.2183,.002 from dual union all
select 1149,'POR15','LP21','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP21','1965 Voyekov expedition LP21','',-150.1506,-15.0667,.002 from dual union all
select 115,'AOR16','LP1','launch point','','1985-01-01 00:00:00','1985-01-01 00:00:00','AOR16 LP1','Zubov Expedition 8, 1985A Feb-Apr, 68-18N 0-30W','Atlantic Ocean',-30,18,1 from dual union all
select 1150,'POR15','LP22','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP22','1965 Voyekov expedition LP22','',-150,-11.2022,.002 from dual union all
select 1151,'POR15','LP23','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP23','1965 Voyekov expedition LP23','',-150,-6.95,.002 from dual union all
select 1152,'POR15','LP24','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP24','1965 Voyekov expedition LP24','',-150,-2.9833,.002 from dual union all
select 1153,'POR15','LP25','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP25','1965 Voyekov expedition LP25','',-150,.9167,.002 from dual union all
select 1154,'POR15','LP26','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP26','1965 Voyekov expedition LP26','',-150,5.05,.002 from dual union all
select 1155,'POR15','LP27','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP27','1965 Voyekov expedition LP27','',-150,8.9167,.002 from dual union all
select 1156,'POR15','LP28','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP28','1965 Voyekov expedition LP28','',-150,13.0022,.002 from dual union all
select 1157,'POR15','LP29','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP29','1965 Voyekov expedition LP29','',-150,16.6667,.002 from dual union all
select 1158,'POR15','LP3','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP3','1965 Voyekov expedition LP3','',166.5,40,.0002 from dual union all
select 1159,'POR15','LP30','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP30','1965 Voyekov expedition LP30','',-150,21.4847,.002 from dual union all
select 116,'AOR16','','launch cruise','SU','1985-02-01 00:00:00','1985-04-01 00:00:00','AOR16 -','Zubov Expedition 8, 1985A Feb-Apr, 68-18N 0-30W','Atlantic Ocean',null,null,null from dual union all
select 1160,'POR15','LP31','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP31','1965 Voyekov expedition LP31','',-150,25.0167,.002 from dual union all
select 1161,'POR15','LP32','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP32','1965 Voyekov expedition LP32','',-150,28.9856,.002 from dual union all
select 1162,'POR15','LP33','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP33','1965 Voyekov expedition LP33','',-150.0033,33.0011,.002 from dual union all
select 1163,'POR15','LP34','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP34','1965 Voyekov expedition LP34','',-170.0008,35.8175,.002 from dual union all
select 1164,'POR15','LP4','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP4','1965 Voyekov expedition LP4 M9047','',178.0667,39.8519,.0002 from dual union all
select 1165,'POR15','LP5','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP5','1965 Voyekov expedition LP5 M9048','',179.85,37.8667,.0002 from dual union all
select 1166,'POR15','LP6','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP6','1965 Voyekov expedition LP6 M9049','',180,28.9525,.0002 from dual union all
select 1167,'POR15','LP7','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP7','1965 Voyekov expedition LP7 M9050','',180,24.3167,.002 from dual union all
select 1168,'POR15','LP8','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP8','1965 Voyekov expedition LP8 M9051','',180,19.1178,.002 from dual union all
select 1169,'POR15','LP9','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15 LP9','1965 Voyekov expedition LP9 M9052','',179.485,16.4011,.002 from dual union all
select 117,'AOR17','LP1','launch point','','1968-01-01 00:00:00','1968-01-01 00:00:00','AOR17 LP1','Vise Expedition 1968, Launch Point 1','Atlantic Ocean',-33.33,21.68,.01 from dual union all
select 1170,'POR15','','launch cruise','SU','1965-01-01 00:00:00','1965-01-01 00:00:00','POR15','1965 Voyekov expedition','',156.0358,40.0514,.0002 from dual union all
select 1171,'POR16','LP1','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP1','1965 Shokalskiy expedition LP1','',131.7686,41.8017,.002 from dual union all
select 1172,'POR16','LP10','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP10','1965 Shokalskiy expedition LP10','',150,4.1503,.002 from dual union all
select 1173,'POR16','LP11','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP11','1965 Shokalskiy expedition LP11','',150.15,.0022,.002 from dual union all
select 1174,'POR16','LP12','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP12','1965 Shokalskiy expedition LP12','',154.5,.0181,.002 from dual union all
select 1175,'POR16','LP13','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP13','1965 Shokalskiy expedition LP13','',157.65,.15,.002 from dual union all
select 1176,'POR16','LP14','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP14','1965 Shokalskiy expedition LP14','',162.1692,.0006,.002 from dual union all
select 1177,'POR16','LP15','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP15','1965 Shokalskiy expedition LP15','',166.6,.0006,.002 from dual union all
select 1178,'POR16','LP16','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP16','1965 Shokalskiy expedition LP16','',171.2186,.0019,.002 from dual;
insert into SITE
select 1179,'POR16','LP17','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP17','1965 Shokalskiy expedition LP17','',175.1856,.0347,.002 from dual union all
select 118,'AOR17','','launch cruise','SU','1968-01-01 00:00:00','1968-01-01 00:00:00','AOR17 -','Vise Expedition 1968, 10 to 24N 20 to 35W','Atlantic Ocean',-20,10,1 from dual union all
select 1180,'POR16','LP18','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP18','1965 Shokalskiy expedition LP18','',179.9858,10,.002 from dual union all
select 1181,'POR16','LP19','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP19','1965 Shokalskiy expedition LP19','',179.8667,6.7333,.002 from dual union all
select 1182,'POR16','LP2','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP2','1965 Shokalskiy expedition LP2','',132.0514,41.7667,.002 from dual union all
select 1183,'POR16','LP20','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP20','1965 Shokalskiy expedition LP20','',179.45,11.3333,.002 from dual union all
select 1184,'POR16','LP21','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP21','1965 Shokalskiy expedition LP21','',172.5172,17.9844,.002 from dual union all
select 1185,'POR16','LP22','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP22','1965 Shokalskiy expedition LP22','',167.8356,18.1008,.002 from dual union all
select 1186,'POR16','LP23','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP23','1965 Shokalskiy expedition LP23','',163.7514,17.6167,.002 from dual union all
select 1187,'POR16','LP24','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP24','1965 Shokalskiy expedition LP24','',164.6167,18.0333,.002 from dual union all
select 1188,'POR16','LP25','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP25','1965 Shokalskiy expedition LP25','',164.9356,10.9022,.002 from dual union all
select 1189,'POR16','LP26','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP26','1965 Shokalskiy expedition LP26','',166.0014,6.4525,.002 from dual union all
select 119,'AOR18','','launch cruise','SU','1970-01-01 00:00:00','1970-01-01 00:00:00','AOR18 -','Vise Expedition 1970, 39 to 0 N, 30 to 18W','Atlantic Ocean',-30,39,1 from dual union all
select 1190,'POR16','LP27','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP27','1965 Shokalskiy expedition LP27','',166.0014,-1.0328,.002 from dual union all
select 1191,'POR16','LP28','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP28','1965 Shokalskiy expedition LP28','',165,5.6169,.002 from dual union all
select 1192,'POR16','LP29','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP29','1965 Shokalskiy expedition LP29','',165,8.9181,.002 from dual union all
select 1193,'POR16','LP3','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP3','1965 Shokalskiy expedition LP3','',149.9847,39.9167,.002 from dual union all
select 1194,'POR16','LP30','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP30','1965 Shokalskiy expedition LP30','',165.0005,3.3006,.002 from dual union all
select 1195,'POR16','LP31','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP31','1965 Shokalskiy expedition LP31','',165.0669,21.2003,.002 from dual union all
select 1196,'POR16','LP32','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP32','1965 Shokalskiy expedition LP32','',165.0167,25.1525,.002 from dual union all
select 1197,'POR16','LP33','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP33','1965 Shokalskiy expedition LP33','',165,29.6,.002 from dual union all
select 1198,'POR16','LP34','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP34','1965 Shokalskiy expedition LP34','',165.0167,33.0686,.002 from dual union all
select 1199,'POR16','LP4','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP4','1965 Shokalskiy expedition LP4','',150.1167,31,.002 from dual union all
select 12,'ALCA','VLS','launch point','','1997-11-02 00:00:00','','ALCA  VLS','VLS pad, Alcantara','Alcantara, Maranhao, Brasil',-44.367,-2.3185,.002 from dual union all
select 120,'AOR19','LP1','launch point','','1973-01-01 00:00:00','1973-06-12 00:00:00','AOR19 LP1','Vise Expedition 1973, 50 N to 3 S, 30 to 25W , Launch Point 1','Atlantic Ocean',-25.2,-2.33,.01 from dual union all
select 1200,'POR16','LP5','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP5','1965 Shokalskiy expedition LP5','',150.0014,26.4669,.002 from dual union all
select 1201,'POR16','LP6','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP6','1965 Shokalskiy expedition LP6','',149.9333,21.9833,.002 from dual union all
select 1202,'POR16','LP7','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP7','1965 Shokalskiy expedition LP7','',150,17.3342,.002 from dual union all
select 1203,'POR16','LP8','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP8','1965 Shokalskiy expedition LP8','',149.9514,12.2353,.002 from dual union all
select 1204,'POR16','LP9','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16 LP9','1965 Shokalskiy expedition LP9','',149.5833,8.2681,.002 from dual union all
select 1205,'POR16','M9072','launch point','','','','UNKO  M9072','Unknown Soviet cruise meteo launch site','',150,21.5,.1 from dual union all
select 1206,'POR16','','launch cruise','SU','1965-01-01 00:00:00','1965-01-01 00:00:00','POR16','1965 Shokalskiy expedition','',131.7686,41.8017,.002 from dual union all
select 1207,'POR17','','launch cruise','SU','','','POR17 -','1988 Vise Expedition, Japan and Chukotsk seas','',null,null,null from dual union all
select 1208,'POR18','','launch zone','US','2005-01-01 00:00:00','','POR18 -','MRT test point','',-175,22,.5 from dual union all
select 1209,'POR19','','launch zone','US','1961-01-01 00:00:00','1961-01-01 00:00:00','POR19 -','HYDRA launch point, 1961 HYDRA POR','',-120,0,1 from dual union all
select 121,'AOR19','LP2','launch point','','1973-01-01 00:00:00','1973-06-13 00:00:00','AOR19 LP2','Vise Expedition 1973, 50 N to 3 S, 30 to 25W , Launch Point 2','Atlantic Ocean',-25.2,-1.8,.01 from dual union all
select 1210,'POR2','','launch cruise','US','1966-05-31 00:00:00','1966-05-31 00:00:00','POR2  -','Hydra Launch Site 3','',-115.8847,9.9511,.0002 from dual union all
select 1211,'POR20','M9002','launch point','','','','POR20 M9002','1960 Soviet Pacific   meteo launch site','',175.7,36.7,.1 from dual union all
select 1212,'POR20','M9003','launch point','','','','POR20 M9003','1960 Soviet Pacific   meteo launch site','',169.4,26,.1 from dual union all
select 1213,'POR20','M9004','launch point','','','','POR20 M9004','1960 Soviet Pacific   meteo launch site','',167,23.1,.1 from dual union all
select 1214,'POR20','M9005','launch point','','','','POR20 M9005','1960 Soviet Pacific   meteo launch site','',163.4,20.5,.1 from dual union all
select 1215,'POR20','M9006','launch point','','','','POR20 M9006','1960 Soviet Pacific   meteo launch site','',160.1,18.1,.1 from dual union all
select 1216,'POR20','M9007','launch point','','','','POR20 M9007','1960 Soviet Pacific   meteo launch site','',157.2,16.1,.1 from dual union all
select 1217,'POR20','M9008','launch point','','','','POR20 M9008','1960 Soviet Pacific   meteo launch site','',155.2,14.4,.1 from dual union all
select 1218,'POR20','M9009','launch point','','','','POR20 M9009','1960 Soviet Pacific   meteo launch site','',154.4,13.9,.1 from dual union all
select 1219,'POR20','M9010','launch point','','','','POR20 M9010','1960 Soviet Pacific   meteo launch site','',151.7,11.6,.1 from dual union all
select 122,'AOR19','LP3','launch point','','1973-01-01 00:00:00','1973-06-14 00:00:00','AOR19 LP3','Vise Expedition 1973, 50 N to 3 S, 30 to 25W , Launch Point 3','Atlantic Ocean',-24.97,-2.4,.01 from dual union all
select 1220,'POR20','M9011','launch point','','','','POR20 M9011','1960 Soviet Pacific   meteo launch site','',149.9,10,.1 from dual union all
select 1221,'POR20','M9012','launch point','','','','POR20 M9012','1960 Soviet Pacific   meteo launch site','',149.5,10.4,.1 from dual union all
select 1222,'POR20','M9013','launch point','','','','POR20 M9013','1960 Soviet Pacific   meteo launch site','',149.2,14.1,.1 from dual union all
select 1223,'POR20','M9014','launch point','','','','POR20 M9014','1960 Soviet Pacific   meteo launch site','',149,14.5,.1 from dual union all
select 1224,'POR20','M9015','launch point','','','','POR20 M9015','1960 Soviet Pacific   meteo launch site','',148.6,17.1,.1 from dual union all
select 1225,'POR20','M9016','launch point','','','','POR20 M9016','1960 Soviet Pacific   meteo launch site','',148.1,19.5,.1 from dual union all
select 1226,'POR20','M9017','launch point','','','','POR20 M9017','1960 Soviet Pacific   meteo launch site','',179.7,42.4,.1 from dual union all
select 1227,'POR20','M9018','launch point','','','','POR20 M9018','1960 Soviet Pacific   meteo launch site','',179.6,42.2,.1 from dual union all
select 1228,'POR20','M9019','launch point','','','','POR20 M9019','1960 Soviet Pacific   meteo launch site','',179.9,42,.1 from dual union all
select 1229,'POR20','M9020','launch point','','','','POR20 M9020','1960 Soviet Pacific   meteo launch site','',179.4,42.2,.1 from dual union all
select 123,'AOR19','LP4','launch point','','1973-01-01 00:00:00','1973-06-14 00:00:00','AOR19 LP4','Vise Expedition 1973, 50 N to 3 S, 30 to 25W , Launch Point 4','Atlantic Ocean',-24.98,-2.57,.01 from dual union all
select 1230,'POR20','M9021','launch point','','','','POR20 M9021','1960 Soviet Pacific   meteo launch site','',179.7,42.1,.1 from dual union all
select 1231,'POR20','M9022','launch point','','','','POR20 M9022','1960 Soviet Pacific   meteo launch site','',180,35.7,.1 from dual union all
select 1232,'POR20','M9023','launch point','','','','POR20 M9023','1960 Soviet Pacific   meteo launch site','',180,32,.1 from dual union all
select 1233,'POR20','','launch cruise','SU','1960-01-01 00:00:00','1960-01-01 00:00:00','POR20','1960 Soviet Pacific   meteo launch site','',175.7,36.7,.1 from dual union all
select 1234,'POR21','M9024','launch point','','','','POR21 M9024','1960 POR Soviet cruise meteo launch site','',180,28.3,.1 from dual union all
select 1235,'POR21','M9025','launch point','','','','POR21 M9025','1960 POR Soviet cruise meteo launch site','',180,24.7,.1 from dual union all
select 1236,'POR21','M9026','launch point','','','','POR21 M9026','1960 POR Soviet cruise meteo launch site','',180,21.2,.1 from dual union all
select 1237,'POR21','M9027','launch point','','','','POR21 M9027','1960 POR Soviet cruise meteo launch site','',180,13.5,.1 from dual union all
select 1238,'POR21','M9028','launch point','','','','POR21 M9028','1960 POR Soviet cruise meteo launch site','',180,9.9,.1 from dual union all
select 1239,'POR21','M9029','launch point','','','','POR21 M9029','1960 POR Soviet cruise meteo launch site','',179,-1.3,.1 from dual union all
select 124,'AOR19','LP5','launch point','','1973-01-01 00:00:00','1973-01-01 00:00:00','AOR19 LP5','Vise Expedition 1973, 50 N to 3 S, 30 to 25W , Launch Point 5','Atlantic Ocean',-30,49.63,.01 from dual union all
select 1240,'POR21','M9030','launch point','','','','POR21 M9030','1960 POR Soviet cruise meteo launch site','',180,5.3,.1 from dual union all
select 1241,'POR21','M9031','launch point','','','','POR21 M9031','1960 POR Soviet cruise meteo launch site','',180,9.6,.1 from dual union all
select 1242,'POR21','M9032','launch point','','','','POR21 M9032','1960 POR Soviet cruise meteo launch site','',177.4,12.2,.1 from dual union all
select 1243,'POR21','M9033','launch point','','','','POR21 M9033','1960 POR Soviet cruise meteo launch site','',175.4,13.4,.1 from dual union all
select 1244,'POR21','M9034','launch point','','','','POR21 M9034','1960 POR Soviet cruise meteo launch site','',179.2,41.8,.1 from dual union all
select 1245,'POR21','M9035','launch point','','','','POR21 M9035','1960 POR Soviet cruise meteo launch site','',179.5,42,.1 from dual union all
select 1246,'POR21','M9036','launch point','','','','POR21 M9036','1960 POR Soviet cruise meteo launch site','',179.5,39,.1 from dual union all
select 1247,'POR21','M9037','launch point','','','','POR21 M9037','1960 POR Soviet cruise meteo launch site','',179.8,38,.1 from dual union all
select 1248,'POR21','M9038','launch point','','','','POR21 M9038','1960 POR Soviet cruise meteo launch site','',179.8,34,.1 from dual union all
select 1249,'POR21','M9039','launch point','','','','POR21 M9039','1960 POR Soviet cruise meteo launch site','',179.8,27.1,.1 from dual union all
select 125,'AOR19','LP6','launch point','','1973-01-01 00:00:00','1973-01-01 00:00:00','AOR19 LP6','Vise Expedition 1973, 50 N to 3 S, 30 to 25W , Launch Point 6','Atlantic Ocean',-28.183,17,.01 from dual union all
select 1250,'POR21','M9040','launch point','','','','POR21 M9040','1960 POR Soviet cruise meteo launch site','',179.8,23.5,.1 from dual union all
select 1251,'POR21','M9041','launch point','','','','POR21 M9041','1960 POR Soviet cruise meteo launch site','',179.8,19.9,.1 from dual union all
select 1252,'POR21','M9042','launch point','','','','POR21 M9042','1960 POR Soviet cruise meteo launch site','',179.8,12.9,.1 from dual union all
select 1253,'POR21','M9043','launch point','','','','POR21 M9043','1960 POR Soviet cruise meteo launch site','',179.8,9.7,.1 from dual union all
select 1254,'POR21','M9044','launch point','','','','POR21 M9044','1960 POR Soviet cruise meteo launch site','',179.8,3.2,.1 from dual union all
select 1255,'POR21','M9045','launch point','','','','POR21 M9045','1960 POR Soviet cruise meteo launch site POR','',179.8,.1,.1 from dual union all
select 1256,'POR21','','launch cruise','SU','1960-01-01 00:00:00','1960-01-01 00:00:00','POR21','1960 Soviet Pacific   meteo launch site','',175.7,36.7,.1 from dual union all
select 1257,'POR22','LP1','launch point','','1949-03-22 00:00:00','1949-03-22 00:00:00','POR22 LP1','Pacific Ocean Launch Site 22 1','',-88.43,-11.1,.02 from dual union all
select 1258,'POR22','LP2','launch point','','1949-03-24 00:00:00','1949-03-24 00:00:00','POR22 LP2','Pacific Ocean Launch Site 22 2','',-82.13,-11.27,.02 from dual union all
select 1259,'POR22','','launch cruise','US','1949-03-01 00:00:00','1949-03-01 00:00:00','POR22','APL Aerobee Pacific Ocean cruise','',-88.43,-11.1,.02 from dual union all
select 126,'AOR19','LP7','launch point','','1973-01-01 00:00:00','1973-06-12 00:00:00','AOR19 LP7','Vise Expedition 1973, 50 N to 3 S, 30 to 25W , Launch Point 7','Atlantic Ocean',-24.97,-2.18,.01 from dual union all
select 1260,'POR23','LP01','launch point','','1977-07-01 00:00:00','1977-07-01 00:00:00','POR23 LP1','1977 Korolev expedition','',-149.2,18.42,.02 from dual union all
select 1261,'POR23','LP02','launch point','','1977-07-01 00:00:00','1977-07-01 00:00:00','POR23 LP2','1977 Korolev expedition','',-148.85,18.42,.02 from dual union all
select 1262,'POR23','LP03','launch point','','1977-07-01 00:00:00','1977-07-01 00:00:00','POR23 LP3','1977 Korolev expedition','',-126.25,14.28,.02 from dual union all
select 1263,'POR23','LP04','launch point','','1977-07-01 00:00:00','1977-07-01 00:00:00','POR23 LP4','1977 Korolev expedition','',-126.12,14.28,.02 from dual union all
select 1264,'POR23','LP05','launch point','','1977-07-01 00:00:00','1977-07-01 00:00:00','POR23 LP5','1977 Korolev expedition','',-125.9,14.25,.02 from dual union all
select 1265,'POR23','LP06','launch point','','1977-07-01 00:00:00','1977-07-01 00:00:00','POR23 LP6','1977 Korolev expedition','',-104.63,10.35,.02 from dual union all
select 1266,'POR23','LP07','launch point','','1977-07-01 00:00:00','1977-07-01 00:00:00','POR23 LP7','1977 Korolev expedition','',-104.2,10.28,.02 from dual union all
select 1267,'POR23','LP08','launch point','','1977-07-01 00:00:00','1977-07-01 00:00:00','POR23 LP8','1977 Korolev expedition','',-103.8,10.22,.02 from dual union all
select 1268,'POR23','LP09','launch point','','1977-07-01 00:00:00','1977-07-01 00:00:00','POR23 LP9','1977 Korolev expedition','',-93.05,7.88,.02 from dual;
insert into SITE
select 1269,'POR23','LP10','launch point','','1977-07-01 00:00:00','1977-07-01 00:00:00','POR23 LP10','1977 Korolev expedition','',-92.75,7.77,.02 from dual union all
select 127,'AOR19','','launch cruise','SU','1973-01-01 00:00:00','1973-01-01 00:00:00','AOR19 -','Vise Expedition 1973, 50 N to 3 S, 30 to 25W','Atlantic Ocean',-30,50,1 from dual union all
select 1270,'POR23','','launch cruise','SU','1977-01-01 00:00:00','1977-01-01 00:00:00','POR23 -','Pacific Ocean Sites, Korolev 1977 cruise','',null,null,null from dual union all
select 1271,'POR24','','launch zone','US','2009-01-01 00:00:00','','POR24 -','Pacific Ocean Launch Site off San Nicolas Island?','',-121.25,32.4,.1 from dual union all
select 1272,'POR25','','launch zone','US','2012-01-01 00:00:00','','POR25 -','Pacific Ocean Launch Site N of Wake Island','',167,21,1.5 from dual union all
select 1273,'POR26','','launch zone','US','2012-01-01 00:00:00','','POR26 -','Pacific Ocean Launch Site SE of Wake Island','',168,17,1.5 from dual union all
select 1274,'POR27','','launch zone','US','2012-01-01 00:00:00','','POR27 -','Pacific Ocean Launch Site SE of Wake Island','',177.421,12.9245,.001 from dual union all
select 1275,'POR28','','launch zone','US','2015-01-01 00:00:00','','POR28 -','Pacific Ocean Launch Site BOA SW of Hawaii','',null,null,null from dual union all
select 1276,'POR29','','launch zone','US','2017-01-01 00:00:00','','POR29 -','Pacific Ocean Launch Site for Alaska intercept tests','',null,null,null from dual union all
select 1277,'POR3','','launch cruise','US','1966-10-30 00:00:00','1966-10-30 00:00:00','POR3  -','Hydra Launch Site 4','',-133,31,.5 from dual union all
select 1278,'POR4','FL-B','launch point','','1968-03-20 00:00:00','1968-03-20 00:00:00','POR4  FL-B','Pacific Ocean Launch Site 4, Floating Launcher B','',-128,32.25,.2 from dual union all
select 1279,'POR4','','launch cruise','US','1968-03-20 00:00:00','1968-03-20 00:00:00','POR4  -','Hydra Launch Site 5','',-128,32.25,.2 from dual union all
select 128,'AOR20','','launch cruise','SU','1976-01-01 00:00:00','1976-01-01 00:00:00','AOR20 -','Vise Expedition 1976, 60N-8S, 35-30W, then IOR 15S 63E','Atlantic Ocean',-35,60,1 from dual union all
select 1280,'POR5','','launch cruise','US','1968-11-03 00:00:00','1968-11-03 00:00:00','POR5  -','Hydra Launch Site 6','',-125,27.5,.5 from dual union all
select 1281,'POR6','','launch cruise','US','1964-08-10 00:00:00','1964-08-10 00:00:00','POR6  -','Hydra Launch Site 1','',-123,34,.5 from dual union all
select 1282,'POR7','LP1','launch point','','1971-11-17 00:00:00','1971-11-17 00:00:00','POR7  LP1','Pacific Ocean Launch Site 7 LP1','',-125,27.38,.5 from dual union all
select 1283,'POR7','','launch cruise','US','1971-11-17 00:00:00','1971-11-17 00:00:00','POR7  -','Hydra Launch Site 6','',-125,27,.5 from dual union all
select 1284,'POR8','','launch cruise','US','1950-05-12 00:00:00','1950-05-12 00:00:00','POR8  -','Viking/Project Reach','',-161.4167,.1866,.02 from dual union all
select 1285,'POR9','LP1','launch point','','1956-07-17 00:00:00','1956-07-27 00:00:00','POR9  LP1','1956 NRL Cruise Launch Point 1','',-121.57,29.68,.02 from dual union all
select 1286,'POR9','LP10','launch point','','1956-07-17 00:00:00','1956-07-27 00:00:00','POR9  LP10','1956 NRL Cruise Launch Point 10','',-121.62,29.53,.02 from dual union all
select 1287,'POR9','LP2','launch point','','1956-07-17 00:00:00','1956-07-27 00:00:00','POR9  LP2','1956 NRL Cruise Launch Point 2','',-121.58,29.7,.02 from dual union all
select 1288,'POR9','LP3','launch point','','1956-07-17 00:00:00','1956-07-27 00:00:00','POR9  LP3','1956 NRL Cruise Launch Point 3','',-121.67,30.25,.02 from dual union all
select 1289,'POR9','LP4','launch point','','1956-07-17 00:00:00','1956-07-27 00:00:00','POR9  LP4','1956 NRL Cruise Launch Point 4','',-121.53,29.72,.02 from dual union all
select 129,'AOR21','','launch cruise','SU','1976-01-01 00:00:00','1977-01-01 00:00:00','AOR21 -','Vise Expedition 1977, 20-14N 61-30W','Atlantic Ocean',-61,20,1 from dual union all
select 1290,'POR9','LP5','launch point','','1956-07-17 00:00:00','1956-07-27 00:00:00','POR9  LP5','1956 NRL Cruise Launch Point 5','',-121.5,29.75,.02 from dual union all
select 1291,'POR9','LP6','launch point','','1956-07-17 00:00:00','1956-07-27 00:00:00','POR9  LP6','1956 NRL Cruise Launch Point 6','',-121.82,30.17,.02 from dual union all
select 1292,'POR9','LP7','launch point','','1956-07-17 00:00:00','1956-07-27 00:00:00','POR9  LP7','1956 NRL Cruise Launch Point 7','',-121.75,29.75,.02 from dual union all
select 1293,'POR9','LP8','launch point','','1956-07-17 00:00:00','1956-07-27 00:00:00','POR9  LP8','1956 NRL Cruise Launch Point 8','',-121.58,29.67,.02 from dual union all
select 1294,'POR9','LP9','launch point','','1956-07-17 00:00:00','1956-07-27 00:00:00','POR9  LP9','1956 NRL Cruise Launch Point 9','',-121.5,29.72,.02 from dual union all
select 1295,'POR9','','launch cruise','US','1956-07-17 00:00:00','1956-07-27 00:00:00','POR9','1956 NRL Cruise Launch Point 1','',-121.57,29.68,.02 from dual union all
select 1296,'PRCT','','launch site','CN','1995-01-01 00:00:00','1996-01-01 00:00:00','PRCT  -','Leping missile base, Jianxi, China','',117.12,28.97,.02 from dual union all
select 1297,'PRIL76A','','launch cruise','SU','1976-10-01 00:00:00','1977-01-01 00:00:00','Priliv 76','Priliv cruise 1976-77','',null,null,null from dual union all
select 1298,'PRIL79A','','launch cruise','SU','1979-05-01 00:00:00','1979-07-01 00:00:00','Priliv 79','Priliv cruise 1979','',null,null,null from dual union all
select 1299,'PRIL80A','','launch cruise','SU','1980-04-01 00:00:00','1980-06-01 00:00:00','Priliv 80','Priliv cruise 1980','',null,null,null from dual union all
select 13,'ALCA','','launch site','BR','1994-08-19 00:00:00','','ALCA  -','CLA, Alcantara Space Center','Alcantara, Maranhao, Brasil',-44.3671,-2.317,.001 from dual union all
select 130,'AOR22','','launch cruise','SU','1978-01-01 00:00:00','1978-01-01 00:00:00','AOR22 -','Vise Expedition 1978  38-20N 75-30W','Atlantic Ocean',-75,38,1 from dual union all
select 1300,'PRIL81A','','launch cruise','SU','1981-01-01 00:00:00','1981-04-01 00:00:00','Priliv 81','Priliv cruise 1981','',null,null,null from dual union all
select 1301,'PRIL87A','','launch cruise','SU','1987-01-01 00:00:00','1987-01-01 00:00:00','Priliv 87','Priliv cruise 1987','',null,null,null from dual union all
select 1302,'PRIL87B','','launch cruise','SU','1987-11-01 00:00:00','1987-12-01 00:00:00','Priliv 87','Priliv cruise 1987','',null,null,null from dual union all
select 1303,'PRIL88A','','launch cruise','SU','1988-02-01 00:00:00','1988-02-01 00:00:00','Priliv 88','Priliv cruise 1988','',null,null,null from dual union all
select 1304,'PRIL88B','','launch cruise','SU','1988-04-01 00:00:00','1988-04-01 00:00:00','Priliv 88','Priliv cruise 1988','',null,null,null from dual union all
select 1305,'PRIL88C','','launch cruise','SU','1988-12-01 00:00:00','1989-03-01 00:00:00','Priliv 88','Priliv cruise 1988','',null,null,null from dual union all
select 1306,'PRIL90A','','launch cruise','SU','1990-01-01 00:00:00','1990-03-01 00:00:00','Priliv 90','Priliv cruise 1990','',null,null,null from dual union all
select 1307,'PRIM','','launch site','CA','1970-01-01 00:00:00','1980-01-01 00:00:00','PRIM  -','Primrose Lake, Alberta','',-110.05,54.75,.02 from dual union all
select 1308,'PSCA','','launch site','US','2016-08-01 00:00:00','','PSCA','Pacific Spaceport Complex Alaska, Kodiak Island, Alaska','',-152.3378,57.4359,.0002 from dual union all
select 1309,'PUKCH','','launch site','KP','2017-01-01 00:00:00','','Pukchang','Pukchang airfield, Sunchon, Pyongan-namdo','',125.9643,39.0544,.01 from dual union all
select 131,'AOR23','','launch cruise','SU','1982-01-01 00:00:00','1982-01-01 00:00:00','AOR23 -','Vise Expedition 1982  68N-25S 30-0W','Atlantic Ocean',-30,68,1 from dual union all
select 1310,'PYONGS','','launch site','KP','2017-01-01 00:00:00','','Pyongsong','Pyongsong region','',125.9,39.25,.1 from dual union all
select 1311,'QOM','','launch site','IR','1991-03-01 00:00:00','','QOM   -','Qom test site, Markazi, Iran','',50.9,34.65,.2 from dual union all
select 1312,'RAMEY','','launch site','PR','1958-08-01 00:00:00','1958-09-01 00:00:00','RAMEY -','Ramey AFB, Aguadilla, Puerto Rico','',-67.13,18.45,.05 from dual union all
select 1313,'RARO','','launch site','CK','1965-01-01 00:00:00','1965-01-01 00:00:00','RARO  -','Rutaki, Rarotonga, Cook Islands, Pacific Ocean','',-159.8,-21.26,.05 from dual union all
select 1314,'REG','','launch site','DZ','1961-01-01 00:00:00','1965-01-01 00:00:00','REG   -','Reggane, Algerie','',.28,26.72,.01 from dual union all
select 1315,'RES','','launch site','CA','1966-01-01 00:00:00','1971-01-01 00:00:00','RES   -','Resolute Bay, Magnetic N Pole Station, Cornwallis I., Nunavut, Canada','',-94.9,74.68,.02 from dual union all
select 1316,'RI','','launch site','US','1999-01-01 00:00:00','1999-01-01 00:00:00','RI    -','Ninigret State Park (Charlestown NAS), Charlestown, Rhode Island','',-71.67,41.36,.02 from dual union all
select 1317,'RLAKE','CHK','launch point','','1979-02-26 00:00:00','1979-02-27 00:00:00','RLAKE CHK','Red Lake, Chukuni Range, Ontario, Canada','',-93.4538,50.9007,.0001 from dual union all
select 1318,'RLAKE','COCH','launch point','','1979-02-26 00:00:00','1979-02-27 00:00:00','RLAKE COCH','Red Lake, Cocheour Site, Ontario, Canada','',-93.47,50.88,.2 from dual union all
select 1319,'RLAKE','MCM','launch point','','1979-02-26 00:00:00','1979-02-27 00:00:00','RLAKE MCM','Red Lake, McMarmac Site, Ontario, Canada','',-93.47,50.88,.2 from dual union all
select 132,'AOR24','','launch cruise','SU','1982-01-01 00:00:00','1982-01-01 00:00:00','AOR24 -','Vise Expedition 1983  68-0N    22-0W','Atlantic Ocean',-30,68,1 from dual union all
select 1320,'RLAKE','','launch site','CA','1979-02-26 00:00:00','1979-02-27 00:00:00','RLAKE CHK','Red Lake, Chukuni Range, Ontario, Canada','',-93.47,50.88,.2 from dual union all
select 1321,'ROBI','','launch site','US','','','ROB   -','Robins AFB, Georgia','',-83.592,32.64,.01 from dual union all
select 1322,'RODR','M9046','launch point','','1968-01-01 00:00:00','1969-01-01 00:00:00','IOR4  M9046','Soviet meteo ship launch site near Rodrigues Island','Indian Ocean',65,-20,.1 from dual union all
select 1323,'RODR','','launch cruise','SU','1968-01-01 00:00:00','1969-01-01 00:00:00','IOR4  M9046','Soviet meteo ship launch site near Rodrigues Island','Indian Ocean',65,-20,.1 from dual union all
select 1324,'ROS','A','launch point','','1964-01-01 00:00:00','1965-01-01 00:00:00','ROS   A','Rosamund Dry Lake, California (drop point A)','',-118.05,35,.01 from dual union all
select 1325,'ROS','B','launch point','','1960-01-01 00:00:00','1960-01-01 00:00:00','ROS   B','Rosamund Dry Lake, California (drop point B)','',-118.33,34.72,.01 from dual union all
select 1326,'ROS','C','launch point','','1959-01-01 00:00:00','1960-01-01 00:00:00','ROS   C','Rosamund Dry Lake, California (drop point C)','',-118.02,34.76,.01 from dual union all
select 1327,'ROS','D','launch point','','1959-01-01 00:00:00','1959-01-01 00:00:00','ROS   D','Rosamund Dry Lake, California (drop point D)','',-118.1,34.76,.01 from dual union all
select 1328,'ROS','WAL','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','ROS   WAL','Waldrip drop zone, Rosamund Dry Lake, California','',-118.1,34.72,.01 from dual union all
select 1329,'ROS','','launch site','US','1959-01-01 00:00:00','1967-01-01 00:00:00','ROS   -','Rosamund Dry Lake, California','',-118.05,34.83,.01 from dual union all
select 133,'AOR25','','launch cruise','SU','1986-01-01 00:00:00','1986-01-01 00:00:00','AOR25 -','Vise Expedition 1986, Atlantic','Atlantic Ocean',null,null,null from dual union all
select 1330,'ROTH','','launch site','BAT','1998-01-01 00:00:00','1998-01-01 00:00:00','ROTH  -','Rothera, British Antarctic Territory','',-68.117,-67.567,.01 from dual union all
select 1331,'RRV','','launch site','US','1959-01-01 00:00:00','1967-01-01 00:00:00','RRV   -','Railroad Valley Lake drop zone, Nevada','',-116.18,38.22,.1 from dual union all
select 1332,'RYO','','launch site','J','1970-04-01 00:00:00','','RYO   -','Ryori, Sanrikucho, Japan','',141.83,39.03,.02 from dual union all
select 1333,'SADAH','','launch site','YE','2015-01-01 00:00:00','2015-01-01 00:00:00','Sa''dah','Sa''dah (Saa''da), Yemen and Dammaj Valley area','',43.85,16.668,.2 from dual union all
select 1334,'SAIN','','launch site','MN','1989-01-01 00:00:00','1989-01-01 00:00:00','SAIN  -','Sainshand meteorological rocket station','Sainshand, Dornogovi, Mongolia',110.14,44.89,.1 from dual union all
select 1335,'SALKH','','launch site','SU','1961-01-01 00:00:00','1962-01-01 00:00:00','Salekhard','Salekhard R-12 missile base','Salekhard, Yamalo-Nenets',66.59,66.59,.02 from dual union all
select 1336,'SANAA','','launch site','YE','2016-01-01 00:00:00','','Sanaa','Sana''a, Yemen','',44.215,15.3014,.2 from dual union all
select 1337,'SCL','E','launch point','','1957-01-01 00:00:00','1960-01-01 00:00:00','SCL   E','Starstruck offshore site, 16 km E of San Clemente Island, California','',-118.5,32.92,.04 from dual union all
select 1338,'SCL','','launch site','US','1957-01-01 00:00:00','1960-01-01 00:00:00','SCL   -','San Clemente Island, Pacific Missile Range, California','',-118.5,32.92,.02 from dual union all
select 1339,'SDQ','BASE','launch point','','','','PISQ','Base camp, Perdasdefogu','',9.4432,39.6777,.0001 from dual union all
select 134,'AOR26','','launch cruise','SU','1969-01-01 00:00:00','1969-01-01 00:00:00','AOR26 -','Zubov Expedition 1969, 45-0N 46-18W','Atlantic Ocean',-46,45,1 from dual union all
select 1340,'SDQ','NHER','launch point','','','','PISQ','Nike Hercules pad, Salto di Quirra','',9.6229,39.5038,.0001 from dual union all
select 1341,'SDQ','SL','launch point','','1959-01-01 00:00:00','','SDQ   SL','San Lorenzo pad, Salto di Quirra, Perdas de Fogu, Sardegna','',9.65,39.5,.02 from dual union all
select 1342,'SDQ','UNK1','launch point','','','','PISQ','Hawk missile site at Capo San Lorenzo, Salto di Quirra','',9.6361,39.5418,.0001 from dual union all
select 1343,'SDQ','','launch site','I','1959-01-01 00:00:00','2000-01-01 00:00:00','SDQ   -','Salto di Quirra (San Lorenzo), Perdas de Fogu, Sardegna','',9.43,39.6,.02 from dual union all
select 1344,'SDSC','','launch site','IN','2002-01-01 00:00:00','','SHAR  -','Satish Dhawan Space Ctr, Sriharikota, Andhra Pradesh, India','',80.303,13.621,.02 from dual union all
select 1345,'SEBA','','launch site','LY','1981-03-01 00:00:00','1981-03-01 00:00:00','SEBA  -','Seba Oasis, Sabha?, Camp Tawiwa, Jarmash, Libya','',14.4585,26.9879,.05 from dual union all
select 1346,'SEM','IKSC','launch point','','2013-01-01 00:00:00','','SEM   IKSC','Imam Khomeini Space Launch Center, Semnan, Iran','',53.9522,35.2369,.0002 from dual union all
select 1347,'SEM','SAF','launch point','','2013-01-01 00:00:00','','SEM   SAF','Safir pad, Semnan, Iran','',53.921,35.2347,.0002 from dual union all
select 1348,'SEM','','launch site','IR','2000-01-01 00:00:00','','SEM   -','Semnan missile launch site, Iran','',53.921,35.2347,.0002 from dual union all
select 1349,'SEMX','','launch site','IR','1980-01-01 00:00:00','','SHAHX -','Unknown mobile launch sites in Semnan province, poss. near Shahrud','',55.33,36.23,.2 from dual union all
select 135,'AOR27','','launch cruise','SU','1969-01-01 00:00:00','1969-01-01 00:00:00','AOR27 -','Zubov Expedition 1969/2, 65-41N 38-17W','Atlantic Ocean',-46,45,1 from dual union all
select 1350,'SEND','','launch site','CA','1980-02-15 00:00:00','1980-02-23 00:00:00','SEND  -','Southend, Saskatchewan, Canada','',-103.36,56.26,.03 from dual union all
select 1351,'SENDAI','','launch site','J','','','SENDA -','Sendai, Japan','',140.9,38.27,.01 from dual union all
select 1352,'SERRE','','launch site','AR','2009-01-01 00:00:00','2009-01-01 00:00:00','Serrezuela','Poligono de Salinas Grandes, Serrezuela, Cordoba','',-65.5,-30.65,.1 from dual union all
select 1353,'SFO','','launch site','US','2011-01-01 00:00:00','2011-01-01 00:00:00','SFO','San Francisco International Airport (SFO)','',-122.37,37.62,.01 from dual union all
select 1354,'SGAV','','launch site','RU','1963-01-01 00:00:00','1965-01-01 00:00:00','SGAV  -','Sovetskaya Gavan, Rossiya','',140.3,48.97,.01 from dual union all
select 1355,'SHABA','','launch site','ZR','1977-01-01 00:00:00','1978-01-01 00:00:00','SHABA -','Shaba North, Kapani Tonneo, Shaba, Zaire','',28.5328,-7.9236,.001 from dual union all
select 1356,'SHAHR','','launch site','IR','1980-01-01 00:00:00','','SHAHR -','Damghan (Shahroud), Iran (Dasht-E-Kabir?)','',55.33,36.23,.01 from dual union all
select 1357,'SHAR','FLP','launch point','','1990-01-01 00:00:00','','SHAR  PSLV','PSLV pad, Sriharikota Range','',80.293,13.576,.02 from dual union all
select 1358,'SHAR','RH','launch point','','1971-10-09 00:00:00','','SHAR  RH','Sounding rocket complex, Sriharikota Range','',80.303,13.621,.02 from dual;
insert into SITE
select 1359,'SHAR','SLP','launch point','','1990-01-01 00:00:00','','SHAR  SLP','GSLV pad 2, Sriharikota Range','',80.293,13.576,.1 from dual union all
select 136,'AOR28','LP1','launch point','','1971-01-01 00:00:00','1971-01-01 00:00:00','AOR28 LP1','Zubov Expedition 1971-2  24-0N 53-24W','Atlantic Ocean',-24.05,0,.5 from dual union all
select 1360,'SHAR','SLV','launch point','','1979-08-01 00:00:00','','SHAR  SLV','SLV pad, Sriharikota Range','',80.299,13.542,.02 from dual union all
select 1361,'SHAR','','launch site','IN','1971-10-09 00:00:00','2002-01-01 00:00:00','SHAR  -','Satish Dhawan Space Ctr, Sriharikota, Andhra Pradesh, India','',80.303,13.621,.02 from dual union all
select 1362,'SHEM','','launch site','US','1988-01-01 00:00:00','1988-01-01 00:00:00','SHEM  -','Eareckson Air Station, Shemya Island, Alaska','',174.08,52.72,.01 from dual union all
select 1363,'SHER','','launch site','PCZ','1969-01-20 00:00:00','1970-11-13 00:00:00','SHER  -','Fort Sherman, Panama Canal Zone','',-79.98,9.33,.02 from dual union all
select 1364,'SHIJ','','launch site','CN','1960-09-01 00:00:00','1960-01-01 00:00:00','SHIJ  -','Shijiedu, Anhui, Zhongguo','',119.22,30.95,.02 from dual union all
select 1365,'SHIR72A','','launch cruise','SU','1972-01-01 00:00:00','1972-03-01 00:00:00','SHIR72A','Shirsov cruise Jan-Mar 1972','',null,null,null from dual union all
select 1366,'SHIR72B','','launch cruise','SU','1972-07-01 00:00:00','1972-08-01 00:00:00','SHIR72B','Shirsov cruise Jul-Aug 1972','',null,null,null from dual union all
select 1367,'SHIR72C','','launch cruise','SU','1972-12-01 00:00:00','1973-03-01 00:00:00','SHIR72C','Shirsov cruise Dec 1972  - Mar 1973','',null,null,null from dual union all
select 1368,'SHIR73A','','launch cruise','SU','1973-01-01 00:00:00','1973-01-01 00:00:00','SHIR73A','Shirsov cruise Jul-Sep 1973','',null,null,null from dual union all
select 1369,'SHIR73B','','launch cruise','SU','1973-12-01 00:00:00','1974-03-01 00:00:00','SHIR73B','Shirsov cruise Dec 1973','',null,null,null from dual union all
select 137,'AOR28','','launch cruise','SU','','','AOR28 -','Zubov Expedition 1971-2  24-0N 53-24W','Atlantic Ocean',null,null,null from dual union all
select 1370,'SHIR74A','','launch cruise','SU','1974-12-01 00:00:00','1975-01-01 00:00:00','SHIR74','Shirsov cruise Dec 1974-Jan 1975','',null,null,null from dual union all
select 1371,'SHIR75A','','launch cruise','SU','1975-03-01 00:00:00','1975-03-01 00:00:00','SHIR75A','Shirsov cruise Mar 1975','',null,null,null from dual union all
select 1372,'SHIR75B','','launch cruise','SU','1975-05-01 00:00:00','1975-06-01 00:00:00','SHIR75B','Shirsov cruise May-Jun 1975','',null,null,null from dual union all
select 1373,'SHIR76A','','launch cruise','SU','1976-01-01 00:00:00','1976-02-01 00:00:00','SHIR76A','Shirsov cruise Jan-Feb 1976','',null,null,null from dual union all
select 1374,'SHIR76B','','launch cruise','SU','1976-10-01 00:00:00','1977-02-01 00:00:00','SHIR76B','Shirsov cruise Oct 1976-Feb 1977','',null,null,null from dual union all
select 1375,'SHIR78A','','launch cruise','SU','1978-03-01 00:00:00','1978-03-01 00:00:00','SHIR78A','Shirsov cruise Mar 1978','',null,null,null from dual union all
select 1376,'SHIR79A','','launch cruise','SU','1979-04-01 00:00:00','1979-05-01 00:00:00','SHIR79A','Shirsov cruise May 1979','',null,null,null from dual union all
select 1377,'SHIR80A','','launch cruise','SU','1980-02-01 00:00:00','1980-03-01 00:00:00','SHIR80A','Shirsov cruise 1980','',null,null,null from dual union all
select 1378,'SHIR80B','','launch cruise','SU','1980-04-01 00:00:00','1980-06-01 00:00:00','SHIR80B','Shirsov cruise 1980','',null,null,null from dual union all
select 1379,'SHIR81A','','launch cruise','SU','1981-01-01 00:00:00','1981-03-01 00:00:00','SHIR81A','Shirsov cruise 1981','',null,null,null from dual union all
select 138,'AOR29','','launch cruise','SU','','','AOR29 -','Zubov Expedition 1975   60N-10S, 30-24W','Atlantic Ocean',null,null,null from dual union all
select 1380,'SHIR82A','','launch cruise','SU','1982-05-01 00:00:00','1982-07-01 00:00:00','SHIR82A','Shirsov cruise 1982','',null,null,null from dual union all
select 1381,'SHIR83A','','launch cruise','SU','1983-02-01 00:00:00','1983-03-01 00:00:00','SHIR83A','Shirsov cruise 1983','',null,null,null from dual union all
select 1382,'SHIR85A','','launch cruise','SU','1985-10-01 00:00:00','1986-01-01 00:00:00','SHIR85A','Shirsov cruise 1985','',null,null,null from dual union all
select 1383,'SHIR86A','','launch cruise','SU','1986-04-01 00:00:00','1986-06-01 00:00:00','SHIR86A','Shirsov cruise 1986','',null,null,null from dual union all
select 1384,'SHIR87A','','launch cruise','SU','1987-01-01 00:00:00','1987-02-01 00:00:00','SHIR87A','Shirsov cruise 1987','',null,null,null from dual union all
select 1385,'SHIR87B','','launch cruise','SU','1987-10-01 00:00:00','1987-12-01 00:00:00','SHIR87B','Shirsov cruise 1987','',null,null,null from dual union all
select 1386,'SHIR88A','','launch cruise','SU','1988-03-01 00:00:00','1988-05-01 00:00:00','SHIR88A','Shirsov cruise 1988','',null,null,null from dual union all
select 1387,'SHIR89A','','launch cruise','SU','1989-08-01 00:00:00','1989-10-01 00:00:00','SHIR89A','Shirsov cruise 1989','',null,null,null from dual union all
select 1388,'SHIR90A','','launch cruise','SU','1990-01-01 00:00:00','1990-03-01 00:00:00','SHIR90A','Shirsov cruise 1990','',null,null,null from dual union all
select 1389,'SHOK74A','','launch cruise','SU','1974-01-01 00:00:00','1974-03-01 00:00:00','SHOK74A','Shokalskiy cruise 1974','',null,null,null from dual union all
select 139,'AOR30','','launch cruise','SU','','','AOR30 -','Zubov Expedition 1975-76 60N-10S, 30W','Atlantic Ocean',null,null,null from dual union all
select 1390,'SHOK74B','','launch cruise','SU','1974-08-01 00:00:00','1974-11-01 00:00:00','SHOK74B','Shokalskiy cruise 1974','',null,null,null from dual union all
select 1391,'SHOK75A','','launch cruise','SU','1975-04-01 00:00:00','1975-06-01 00:00:00','SHOK75A','Shokalskiy cruise 1975','',null,null,null from dual union all
select 1392,'SHOK76A','','launch cruise','SU','1976-02-01 00:00:00','1976-04-01 00:00:00','SHOK76A','Shokalskiy cruise 1976','',null,null,null from dual union all
select 1393,'SHOK76B','','launch cruise','SU','1976-06-01 00:00:00','1976-08-01 00:00:00','SHOK76B','Shokalskiy cruise 1976','',null,null,null from dual union all
select 1394,'SHOK76C','','launch cruise','SU','1976-12-01 00:00:00','1977-01-01 00:00:00','SHOK76C','Shokalskiy cruise 1976','',null,null,null from dual union all
select 1395,'SHOK77A','','launch cruise','SU','1977-05-01 00:00:00','1977-07-01 00:00:00','SHOK77A','Shokalskiy cruise 1977','',null,null,null from dual union all
select 1396,'SHOK77B','','launch cruise','SU','1977-11-01 00:00:00','1978-01-01 00:00:00','SHOK77B','Shokalskiy cruise 1977','',null,null,null from dual union all
select 1397,'SHOK79A','','launch cruise','SU','1979-02-01 00:00:00','1979-03-01 00:00:00','SHOK79A','Shokalskiy cruise 1979','',null,null,null from dual union all
select 1398,'SHOK79B','','launch cruise','SU','1979-06-01 00:00:00','1979-08-01 00:00:00','SHOK79B','Shokalskiy cruise 1979','',null,null,null from dual union all
select 1399,'SHOK79C','','launch cruise','SU','1979-11-01 00:00:00','1980-01-01 00:00:00','SHOK79C','Shokalskiy cruise 1979','',null,null,null from dual union all
select 14,'ALNASR','','launch site','SY','1992-07-01 00:00:00','','SYRIA ALNASR','Al Nasriyah missile base, Syria','',36.87,33.9285,.01 from dual union all
select 140,'AOR31','','launch cruise','SU','','','AOR31 -','Zubov Expedition 1980 Apr-Jul, 68N-27S, 0-35W','Atlantic Ocean',null,null,null from dual union all
select 1400,'SIL','A','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','SIL   A','Silver Lake 1 drop zone, California','',-115.8,35.53,.1 from dual union all
select 1401,'SIL','B','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','SIL   B','Silver Lake 2 drop zone, California','',-115.95,35.58,.1 from dual union all
select 1402,'SIL','C','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','SIL   C','Silver Lake drop zone, California','',-115.85,35.12,.1 from dual union all
select 1403,'SIL','RW15/33','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','SIL   RW15/33','RW15/33, Silver Lake lakebed, California','',-116.1,35.33,.1 from dual union all
select 1404,'SIL','','launch site','US','1960-01-01 00:00:00','','SIL   -','Silver Lake 1 drop zone, California','',-115.8,35.53,.1 from dual union all
select 1405,'SIMLA','','launch zone','SU','','','SIMLA -','Simushir Island launch area, Pacific Ocean','',152,47,1 from dual union all
select 1406,'SINPO','','launch site','KP','2014-01-01 00:00:00','','Shinpo','Shinpo Shipyard, S Hamgyong Prov (Hamgyong-namdo), North Korea','',128.158,40.019,.001 from dual union all
select 1407,'SIP','A','launch point','','1980-12-12 00:00:00','1981-01-11 00:00:00','SIP   A','Siple Station, Ellsworth Land, Antarctica (Arcas launcher)','',-84.2481,-75.9391,.02 from dual union all
select 1408,'SIP','RAG','launch point','','1980-12-12 00:00:00','1981-01-11 00:00:00','SIP   RAG','Siple Station, Ellsworth Land, Antarctica (RAG launcher)','',-84.251,-75.9384,.02 from dual union all
select 1409,'SIP','','launch site','AQ','1980-12-12 00:00:00','1981-01-11 00:00:00','SIP   -','Siple Station, Ellsworth Land, Antarctica','',-84.2481,-75.94,.0005 from dual union all
select 141,'AOR32','','launch cruise','SU','','','AOR32 -','Zubov Expedition 1981 May-Aug, 76-20N 0-35W','Atlantic Ocean',null,null,null from dual union all
select 1410,'SMLC','Rita','launch point','','1964-01-01 00:00:00','1965-01-01 00:00:00','SMLC  Rita','Santa Rita Platform, San Marco Range, Formosa Bay, Kenya','',40.21,-2.9386,.0002 from dual union all
select 1411,'SMLC','SMLP','launch point','','1964-01-01 00:00:00','','SMLC  -','San Marco Platform, San Marco Launch Complex, Formosa Bay, Kenya','',40.2134,-2.9408,.0002 from dual union all
select 1412,'SMLC','','launch site','KE','1964-01-01 00:00:00','','SMLC  -','San Marco Launch Complex, Formosa Bay, Kenya','',40.2134,-2.9408,.0002 from dual union all
select 1413,'SNI','PC1','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','SNI PC1','Probe Complex No. 1, San Nicolas I.','',-119.571,33.2596,.01 from dual union all
select 1414,'SNI','RS','launch point','','1965-01-01 00:00:00','1965-01-01 00:00:00','SNI RS','Redstone launch pad, San Nicolas I.','',-119.571,33.2596,.0001 from dual union all
select 1415,'SNI','','launch site','US','1960-01-01 00:00:00','1973-01-01 00:00:00','SNI   -','San Nicolas Island, Pacific Missile Range, California','',-119.5,33.25,.02 from dual union all
select 1416,'SOHAE','LC1','launch point','','2008-01-01 00:00:00','','TONG  -','Sohae Launch Site, Tongch''ang-dong, Pyongang-bukdo (N Pyongan Prov), N Korea','',124.7053,39.6601,.0001 from dual union all
select 1417,'SOHAE','LP1','launch point','','2016-01-01 00:00:00','','TONG  -','Scud launch point, Tongch''ang-dong, Pyongang-bukdo (N Pyongan Prov), N Korea','',124.7117,39.6811,.0001 from dual union all
select 1418,'SOHAE','','launch site','KP','2008-01-01 00:00:00','','TONG  -','Sohae Launch Site, Tongch''ang-dong, Pyongang-bukdo (N Pyongan Prov), N Korea','',124.706,39.66,.1 from dual union all
select 1419,'SOM','','launch site','PK','1962-01-01 00:00:00','','SOM   -','Sonmiani Range, Sonmiani Beach, Baluchistan, Pakistan','',66.75,25.2,.02 from dual union all
select 142,'AOR33','','launch cruise','SU','','','AOR33 -','Zubov Expedition 9,1985 Aug-Nov, 72N-27S, 5E-35W','Atlantic Ocean',null,null,null from dual union all
select 1420,'SOZEK','','launch site','SU','1968-01-01 00:00:00','1969-01-01 00:00:00','SOZEK','Sary Ozek missile base, Kazakhstan','',67.65,43.3,.2 from dual union all
select 1421,'SPAM','RW 16/34','launch point','','2010-10-01 00:00:00','','SWRS  GBRS','Governor Bill Richardson Spaceway, RW  16/34  Spaceport America, New Mexico','',-106.9722,32.9909,.0001 from dual union all
select 1422,'SPAM','VLA LC1','launch point','','2010-10-01 00:00:00','','SPAM  LC1','Launch Complex 1, Vertical Launch Area, Spaceport America','',-106.9066,32.9403,.0001 from dual union all
select 1423,'SPAM','VLA LLP','launch point','','2010-10-01 00:00:00','','SPAM  LLP','Lunar Lander Pads, Vertical Launch Area, Spaceport America','',-106.9224,32.9436,.0001 from dual union all
select 1424,'SPAM','','launch site','US','2006-01-01 00:00:00','','SPAM  -','Spaceport America, New Mexico','Upham, New Mexico',-106.97,32.99,.01 from dual union all
select 1425,'SPFL','SLC46','launch point','','1987-01-15 00:00:00','','SPFL  SLC46','Space Launch Complex 46, Spaceport Florida, Cape Canaveral','',-80.5272,28.4583,.0002 from dual union all
select 1426,'SPFL','SLC47','launch point','','2006-01-01 00:00:00','','SPFL  SLC47','Space Launch Complex 47, Spaceport Florida, Cape Canaveral','',-80.53,28.46,.01 from dual union all
select 1427,'SPFL','','launch site','US','1998-01-01 00:00:00','','SPFL  -','Spaceport Florida, Cape Canaveral','',-80.5272,28.4583,.0002 from dual union all
select 1428,'SPXMCG','','launch site','US','2012-01-01 00:00:00','','SpaceX McGregor','SpaceX McGregor Test Site','McGregor, Texas',-97.46,31.4,.01 from dual union all
select 1429,'SRL','RW03/21','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','SRL   RW03/21','RW03/21, Smith Ranch Lake, Nevada','',-117.5,39.25,.02 from dual union all
select 143,'AOR34','','launch cruise','SU','1986-09-01 00:00:00','1986-11-01 00:00:00','AOR34 -','Zubov Expedition 10, 66-0N 0-20W','Atlantic Ocean',null,null,null from dual union all
select 1430,'SRL','','launch zone','US','1959-01-01 00:00:00','1967-01-01 00:00:00','SRL   -','Smith Ranch Lake drop zone, Nevada','',-117.17,39.5,.02 from dual union all
select 1431,'SRMSC','','launch site','US','1971-01-01 00:00:00','1975-01-01 00:00:00','SRMSC','Stanley R. Mickelson Safeguard Complex (SRMSC)','Nekoma, North Dakota',-98.3562,48.5931,.0001 from dual union all
select 1432,'SSG','DL','launch point','','1971-01-01 00:00:00','1992-01-01 00:00:00','SSG   DL','Danish Launcher, Sondestrom','',-50.5965,67.0218,.0001 from dual union all
select 1433,'SSG','HAD','launch point','','1971-01-01 00:00:00','1992-01-01 00:00:00','SSG   HAD','HAD Launcher, Sondestrom','',-50.5974,67.0237,.0001 from dual union all
select 1434,'SSG','HAD3','launch point','','1986-01-01 00:00:00','1992-01-01 00:00:00','SSG   HAD3','HAD Launcher No 3, Sondestrom','',-50.5972,67.0232,.0001 from dual union all
select 1435,'SSG','MORABA','launch point','','1974-01-01 00:00:00','1975-01-01 00:00:00','SSG   MORABA','MORABA Launcher, Sondestrom','',-50.5974,67.0237,.01 from dual union all
select 1436,'SSG','MRL','launch point','','1971-01-01 00:00:00','1992-01-01 00:00:00','SSG   MRL','MRL Launcher, Sondestrom','',-50.5974,67.0235,.0001 from dual union all
select 1437,'SSG','T','launch point','','1986-01-01 00:00:00','1992-01-01 00:00:00','SSG   T','Test Launcher, Sondestrom','',-50.5965,67.023,.0001 from dual union all
select 1438,'SSG','','launch site','GL','1971-01-01 00:00:00','1992-01-01 00:00:00','SSG   -','Sonde Stromfjord, Kangerlussuaq, Kalaallit Nunaat (Gronland)','',-50.597,67.023,.002 from dual union all
select 1439,'SUIP','','launch site','F','1950-08-02 00:00:00','1951-10-06 00:00:00','SUIP  -','Camp de Suippes, Marne, France','',4.53,49.15,.03 from dual union all
select 144,'AOR35','','launch cruise','SU','1987-06-01 00:00:00','1987-09-01 00:00:00','AOR35 -','Zubov Expedition 11, 72N-27S 5-35W','Atlantic Ocean',null,null,null from dual union all
select 1440,'SUIST','Bornish','launch point','','1962-01-01 00:00:00','','SUIST -','Bornish Launch Area, Guided Weapons Range, South Uist, Hebrides','',-7.4568,57.2415,.001 from dual union all
select 1441,'SUIST','C1','launch point','','1962-01-01 00:00:00','','SUIST -','Corporal Launch Area, Guided Weapons Range, South Uist, Hebrides','',-7.4033,57.3609,.001 from dual union all
select 1442,'SUIST','C2','launch point','','1962-01-01 00:00:00','','SUIST -','Corporal Launch Area, Guided Weapons Range, South Uist, Hebrides','',-7.405,57.3635,.001 from dual union all
select 1443,'SUIST','C3','launch point','','1962-01-01 00:00:00','','SUIST -','Corporal Launch Area, Guided Weapons Range, South Uist, Hebrides','',-7.4068,57.366,.001 from dual union all
select 1444,'SUIST','LA1','launch point','','1962-01-01 00:00:00','','SUIST -','Launch Area 1, Royal Artillery Guided Weapons Range, South Uist, Hebrides','',-7.411,57.3697,.001 from dual union all
select 1445,'SUIST','LA2','launch point','','1962-01-01 00:00:00','','SUIST -','Launch Area 2, Royal Artillery Guided Weapons Range, South Uist, Hebrides','',-7.3982,57.3458,.001 from dual union all
select 1446,'SUIST','T1','launch point','','1962-01-01 00:00:00','','SUIST -','Target Launch Area, Guided Weapons Range, South Uist, Hebrides','',-7.3945,57.3579,.001 from dual union all
select 1447,'SUIST','','launch site','UK','1962-01-01 00:00:00','','SUIST -','Royal Artillery Guided Weapons Range, South Uist, Hebrides','',-7.33,57.33,.02 from dual union all
select 1448,'SUKCH','','launch site','KP','2014-01-01 00:00:00','','SUKCHON','Sukchon County launch site, South Pyongan Prov. (Pyong''annam-do), CMIK','',125.6,39.4,.1 from dual;
insert into SITE
select 1449,'SUNAN','','launch site','KP','2017-01-01 00:00:00','','SUNAN','Sunan Airport, Pyonyang','',125.6752,39.2674,.0005 from dual union all
select 145,'AOR36','','launch cruise','SU','1989-07-01 00:00:00','1989-11-01 00:00:00','AOR36 -','Zubov Expedition 12, 72N-27S 5-35W','Atlantic Ocean',null,null,null from dual union all
select 1450,'SV3','LM6-DS','launch point','','1969-11-20 00:00:00','1969-11-20 00:00:00','SV3   LM6-DS','LM6 Descent Stage, Surveyor 3 Site, Mare Insularum, Montes Riphaeus A2','Luna',23.38,-3.2,.01 from dual union all
select 1451,'SV3','','launch site','LUNA','1969-11-20 00:00:00','1969-11-20 00:00:00','SV3   LM6-DS','LM6 Descent Stage, Surveyor 3 Site, Mare Insularum, Montes Riphaeus A2','Luna',23.38,-3.2,.01 from dual union all
select 1452,'SVAL','LONG','launch point','','1997-11-01 00:00:00','','SVAL  LONG','Longyearbyen, Svalbard, Spitzbergen','',15.7,78.2,.0001 from dual union all
select 1453,'SVAL','','launch site','N','1997-11-01 00:00:00','','SVAL  -','SvalRak, Ny Alesund, Svalbard, Spitsbergen','',11.8504,78.9315,.0001 from dual union all
select 1454,'SVER','','launch site','SU','1960-01-01 00:00:00','','SVER  -','Sverdlovsk air defence base','',39.65,48.9,.01 from dual union all
select 1455,'SVOB','','launch site','RU','1990-01-01 00:00:00','','GIK-2 -','Svobodniy, Amurskaya Oblast'', Rossiya','',128,51.7,.5 from dual union all
select 1456,'SWRS','','launch site','US','2006-01-01 00:00:00','2006-01-01 00:00:00','SWRS  -','SouthWest Regional Spaceport (Spaceport America) New Mexico','',-106.97,32.99,.01 from dual union all
select 1457,'SYRIA','','launch site','SY','1992-07-01 00:00:00','','SYRIA -','Syrian missile base, unknown, N of Halab/Alleppo?, Syria','',38,35,3 from dual union all
select 1458,'SYW','','launch site','DML','1970-02-01 00:00:00','','SYW   -','Syowa (Showa) Base, East Ongul I., Queen Maud Land, Antarctica','',39.58,-69,.03 from dual union all
select 1459,'TAIKI','','launch site','J','2017-07-01 00:00:00','','Taiki IST','Interstellar Technologies Test Site, Taiki, Hokkaido','',143.4572,42.5059,.0001 from dual union all
select 146,'AOR37','','launch cruise','SU','1990-08-01 00:00:00','1990-12-01 00:00:00','AOR37 -','Zubov Expedition 13, 72N-27S 5-35W','Atlantic Ocean',null,null,null from dual union all
select 1460,'TART','','launch site','AR','1966-11-18 00:00:00','1966-11-18 00:00:00','TART  -','Tartagal, Salta, Argentina','',-63.82,-22.77,.02 from dual union all
select 1461,'TATS','','launch site','SU','1980-01-01 00:00:00','1980-01-01 00:00:00','TATS  -','Tatishchevo ICBM base, Russia','',45.57,51.67,.02 from dual union all
select 1462,'TERLS','Pad 1','launch point','','1963-01-01 00:00:00','','TERLS Pad 1','Pad 1, Thumba Equatorial Rocket Launch Station, Trivandrum, Kerala, India','',76.87,8.53,.02 from dual union all
select 1463,'TERLS','Pad 2','launch point','','1963-01-01 00:00:00','','TERLS Pad 2','Pad 2, Thumba Equatorial Rocket Launch Station, Trivandrum, Kerala, India','',76.87,8.53,.02 from dual union all
select 1464,'TERLS','Pad 3','launch point','','1963-01-01 00:00:00','','TERLS Pad 3','Pad 3, Thumba Equatorial Rocket Launch Station, Trivandrum, Kerala, India','',76.87,8.53,.02 from dual union all
select 1465,'TERLS','Pad 4','launch point','','1963-01-01 00:00:00','','TERLS Pad 4','Pad 4, Thumba Equatorial Rocket Launch Station, Trivandrum, Kerala, India','',76.87,8.53,.02 from dual union all
select 1466,'TERLS','Pad 5','launch point','','1963-01-01 00:00:00','','TERLS Pad 5','LC5,  Thumba Equatorial Rocket Launch Station, Trivandrum, Kerala, India','',76.87,8.53,.02 from dual union all
select 1467,'TERLS','','launch site','IN','1963-01-01 00:00:00','','TERLS -','Thumba Equatorial Rocket Launch Station, Trivandrum, Kerala, India','',76.87,8.53,.02 from dual union all
select 1468,'TEYK','','launch site','SU','1960-01-01 00:00:00','1990-01-01 00:00:00','TEYK','Teykovo Missile Base, Russia','',40.44,56.7,.01 from dual union all
select 1469,'THU','TUTO','launch point','','1964-01-01 00:00:00','1966-01-01 00:00:00','THU   TUTO','Thule/Camp Tuto (Geopole Station), Halvo, Kalaallit Nunaat','',-68.32,76.43,.02 from dual union all
select 147,'AOR38','','launch cruise','SU','1991-07-01 00:00:00','1991-10-01 00:00:00','AOR38 -','Zubov Expedition 14, Caribbean and equatorial','Atlantic Ocean',null,null,null from dual union all
select 1470,'THU','','launch site','GL','1970-01-01 00:00:00','1976-01-01 00:00:00','THU   -','Thule AFB, Halvo, Kalaallit Nunaat','',-68.75,76.53,.02 from dual union all
select 1471,'THUTH','','launch site','YE','2015-01-01 00:00:00','2015-01-01 00:00:00','TussenHuhen','Tussen Huthen, Yemen','',44.1,16.42,.2 from dual union all
select 1472,'TIKSI','','launch site','RU','2015-01-01 00:00:00','','TIKSI','Tiksi Hydrometeorological Observatory','',71.586,128.925,.003 from dual union all
select 1473,'TILLA','','launch site','PK','1998-04-08 00:00:00','','TILLA -','Tilla Firing Range, Malute, Jhelum region, Punjab, Pakistan','',73.3,33.4,.5 from dual union all
select 1474,'TNSC','N','launch point','','1975-01-01 00:00:00','','TNSC  N','N Launch Complex, Osaki Launch Site, Tanegashima Space Center','',130.9703,30.3995,.0001 from dual union all
select 1475,'TNSC','Q','launch point','','1975-01-01 00:00:00','1975-01-01 00:00:00','TNSC  Q','Q Launch Complex, Osaki Launch Site, Tanegashima Space Center','',130.9685,30.3985,.0005 from dual union all
select 1476,'TNSC','T','launch point','','1975-01-01 00:00:00','','TNSC  T','Takesaki Launch Site, Tanegashima Space Center','',130.9625,30.376,.0001 from dual union all
select 1477,'TNSC','Y','launch point','','1992-01-01 00:00:00','','TNSC  Y','Yoshinobu Launch Complex, Osaki Site, Tanegashima Space Center','',130.9776,30.4008,.0001 from dual union all
select 1478,'TNSC','Y2','launch point','','1992-01-01 00:00:00','','TNSC  Y','Yoshinobu Launch Complex, Osaki Site, Tanegashima Space Center','',130.9754,30.4009,.0001 from dual union all
select 1479,'TNSC','','launch site','J','1975-01-01 00:00:00','','TNSC  -','Tanegashima Space Center, Tanegashima, Nippon','',130.97,30.4,.02 from dual union all
select 148,'AOR39','','launch cruise','US','1958-01-01 00:00:00','1958-01-01 00:00:00','AOR39 -','USS Warrington, TF88 cruise from Virginia to S Atlantic, Argus 1958','Atlantic Ocean',null,null,null from dual union all
select 1480,'TONGH','','launch site','KP','1984-04-01 00:00:00','','TONGH -','Tonghae SLG, Musudan-ri, Hwadae-gun, Hamgyong Pukdo, Choson','',129.6666,40.85,.002 from dual union all
select 1481,'TORT','HAD','launch point','','1992-05-25 00:00:00','','TORT  HAD','HAD launcher, Camp Tortuguero, Arecibo, Puerto Rico','',-66.4395,18.4829,.0001 from dual union all
select 1482,'TORT','MRL','launch point','','1992-05-25 00:00:00','','TORT  MRL','MRL launcher, Camp Tortuguero, Arecibo, Puerto Rico','',-66.4392,18.4831,.0001 from dual union all
select 1483,'TORT','VB','launch point','','1966-02-01 00:00:00','1968-05-01 00:00:00','VB    -','Camp Tortuguero, Arecibo, Vega Baja, Puerto Rico','',-66.4389,18.4826,.001 from dual union all
select 1484,'TORT','','launch site','PR','1992-05-25 00:00:00','','TORT  -','Camp Tortuguero, Vega Baja, Arecibo, Puerto Rico','',-66.4395,18.4829,.02 from dual union all
select 1485,'TRAN','LM5-DS','launch point','','1969-07-21 00:00:00','1969-07-21 00:00:00','TRAN  LM5-DS','LM 5 DS, Tranquility Base, Mare Tranquillitatis, Julius Caesar C4','Luna',23.43,.69,.01 from dual union all
select 1486,'TRAN','','launch site','LUNA','1969-07-21 00:00:00','1969-07-21 00:00:00','TRAN  LM5-DS','LM 5 DS, Tranquility Base, Mare Tranquillitatis, Julius Caesar C4','Luna',23.43,.69,.01 from dual union all
select 1487,'TRIP','','launch site','LY','','','TRIP  -','Tripoli missile base, Tripoli, Libya','',13,32.5,2 from dual union all
select 1488,'TTR','HAD4','launch point','','1957-01-01 00:00:00','','TTR   HAD4','HAD Launcher No. 4, Tonopah Test Range, Nevada','',-116.75,37.75,.03 from dual union all
select 1489,'TTR','UL3','launch point','','1957-01-01 00:00:00','','TTR   UL3','Universal Launcher No. 3, Tonopah Test Range, Nevada','',-116.75,37.75,.03 from dual union all
select 149,'AOR4','LP1','launch point','','','','AOR4  LP1','Atlantic Ocean, Croatan 1964 cruise 120 km off WI','Atlantic Ocean',null,null,null from dual union all
select 1490,'TTR','','launch site','US','1957-01-01 00:00:00','','TTR   -','Tonopah Test Range, Nevada','',-116.75,37.75,.03 from dual union all
select 1491,'TYSC','LC16','launch point','','2015-01-01 00:00:00','','TYSC  LC16','Pad 16, Taiyuan Space Center, Wuzhai, Shanxi, China','',111.5802,38.8681,.0002 from dual union all
select 1492,'TYSC','LC7','launch point','','1988-01-01 00:00:00','','TYSC  LC7','Pad 7, Taiyuan Space Center, Wuzhai, Shanxi, China','',111.6081,38.8488,.0002 from dual union all
select 1493,'TYSC','LC9','launch point','','2008-01-01 00:00:00','','TYSC  LC9','Pad 9, Taiyuan Space Center, Wuzhai, Shanxi, China','',111.5895,38.8632,.0002 from dual union all
select 1494,'TYSC','','launch site','CN','1988-01-01 00:00:00','','TYSC  -','Taiyuan weixing fashe zhongxin','Wuzhai, Shanxi, China',112.6,37.5,.1 from dual union all
select 1495,'TYUM','','launch site','SU','1960-01-01 00:00:00','1980-01-01 00:00:00','TYUM','Tyumen'' missile base, Russia','',65.57,56.85,.03 from dual union all
select 1496,'UNKMR1','M9104','launch point','','','','UNKO  M9104','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1497,'UNKMR1','M9105','launch point','','','','UNKO  M9105','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1498,'UNKMR1','M9106','launch point','','','','UNKO  M9106','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1499,'UNKMR1','M9107','launch point','','','','UNKO  M9107','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 15,'ALQAL','','launch site','SY','1992-07-01 00:00:00','','SYRIA ALQAL','Al Qalamon, near Damascus, Syria','',36.87,33.9285,.1 from dual union all
select 150,'AOR4','LP2','launch point','','1964-11-11 00:00:00','1964-11-11 00:00:00','AOR4  LP2','Atlantic Ocean, Croatan 1964 cruise 300 km off WI','Atlantic Ocean',-74.67,34.45,.02 from dual union all
select 1500,'UNKMR1','M9108','launch point','','','','UNKO  M9108','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1501,'UNKMR1','M9109','launch point','','','','UNKO  M9109','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1502,'UNKMR1','M9110','launch point','','','','UNKO  M9110','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1503,'UNKMR1','M9111','launch point','','','','UNKO  M9111','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1504,'UNKMR1','M9112','launch point','','','','UNKO  M9112','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1505,'UNKMR1','M9113','launch point','','','','UNKO  M9113','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1506,'UNKMR1','M9114','launch point','','','','UNKO  M9114','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1507,'UNKMR1','M9115','launch point','','','','UNKO  M9115','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1508,'UNKMR1','M9116','launch point','','','','UNKO  M9116','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1509,'UNKMR1','M9117','launch point','','','','UNKO  M9117','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 151,'AOR4','LP3','launch point','','','','AOR4  LP3','Atlantic Ocean, Croatan 1964 cruise off N Carolina','Atlantic Ocean',null,null,null from dual union all
select 1510,'UNKMR1','M9118','launch point','','','','UNKO  M9118','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1511,'UNKMR1','M9119','launch point','','','','UNKO  M9119','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1512,'UNKMR1','M9154','launch point','','','','UNKO  M9154','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1513,'UNKMR1','M9155','launch point','','','','UNKO  M9155','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1514,'UNKMR1','M9156','launch point','','','','UNKO  M9156','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1515,'UNKMR1','M9157','launch point','','','','UNKO  M9157','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1516,'UNKMR1','M9158','launch point','','','','UNKO  M9158','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1517,'UNKMR1','M9159','launch point','','','','UNKO  M9159','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1518,'UNKMR1','M9160','launch point','','','','UNKO  M9160','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1519,'UNKMR1','M9169','launch point','','','','UNKO  M9169','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 152,'AOR4','LP4','launch point','','','','AOR4  LP4','Atlantic Ocean, Croatan 1964 cruise','Atlantic Ocean',null,null,null from dual union all
select 1520,'UNKMR1','','launch cruise','SU','1966-12-01 00:00:00','1967-02-01 00:00:00','UNKMR1','Unknown Soviet ship cruise, Dec 1966-Feb 1967','',null,null,null from dual union all
select 1521,'UNKMR10','','launch cruise','SU','1970-10-01 00:00:00','1971-02-01 00:00:00','UNKMR10','Unknown Soviet ship cruise, Oct 1970-Feb 1971','',null,null,null from dual union all
select 1522,'UNKMR11','','launch cruise','SU','1971-05-01 00:00:00','1971-07-01 00:00:00','UNKMR11','Unknown Soviet ship cruise, May-Jul 1971','',null,null,null from dual union all
select 1523,'UNKMR2','M9170','launch point','','','','UNKO  M9170','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1524,'UNKMR2','M9173','launch point','','','','UNKO  M9173','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1525,'UNKMR2','M9174','launch point','','','','UNKO  M9174','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1526,'UNKMR2','M9175','launch point','','','','UNKO  M9175','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1527,'UNKMR2','M9176','launch point','','','','UNKO  M9176','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1528,'UNKMR2','M9177','launch point','','','','UNKO  M9177','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1529,'UNKMR2','M9178','launch point','','','','UNKO  M9178','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 153,'AOR4','LP5','launch point','','','','AOR4  LP5','Atlantic Ocean, Croatan 1964 cruise','Atlantic Ocean',null,null,null from dual union all
select 1530,'UNKMR2','M9179','launch point','','','','UNKO  M9179','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1531,'UNKMR2','M9180','launch point','','','','UNKO  M9180','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1532,'UNKMR2','M9181','launch point','','','','UNKO  M9181','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1533,'UNKMR2','M9182','launch point','','','','UNKO  M9182','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1534,'UNKMR2','M9183','launch point','','','','UNKO  M9183','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1535,'UNKMR2','M9184','launch point','','','','UNKO  M9184','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1536,'UNKMR2','M9185','launch point','','','','UNKO  M9185','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1537,'UNKMR2','M9186','launch point','','','','UNKO  M9186','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1538,'UNKMR2','M9187','launch point','','','','UNKO  M9187','Unknown Soviet cruise meteo launch site','',null,null,null from dual;
insert into SITE
select 1539,'UNKMR2','M9188','launch point','','','','UNKO  M9188','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 154,'AOR4','LP6','launch point','','','','AOR4  LP6','Atlantic Ocean, Croatan 1964 cruise','Atlantic Ocean',null,null,null from dual union all
select 1540,'UNKMR2','M9189','launch point','','','','UNKO  M9189','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1541,'UNKMR2','M9190','launch point','','','','UNKO  M9190','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1542,'UNKMR2','M9191','launch point','','','','UNKO  M9191','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1543,'UNKMR2','M9192','launch point','','','','UNKO  M9192','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1544,'UNKMR2','M9193','launch point','','','','UNKO  M9193','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1545,'UNKMR2','M9194','launch point','','','','UNKO  M9194','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1546,'UNKMR2','M9195','launch point','','','','UNKO  M9195','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1547,'UNKMR2','M9196','launch point','','','','UNKO  M9196','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1548,'UNKMR2','M9197','launch point','','','','UNKO  M9197','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1549,'UNKMR2','M9198','launch point','','','','UNKO  M9198','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 155,'AOR4','LP7','launch point','','','','AOR4  LP7','Atlantic Ocean, Croatan 1964 cruise','Atlantic Ocean',null,null,null from dual union all
select 1550,'UNKMR2','M9199','launch point','','','','UNKO  M9199','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1551,'UNKMR2','M9200','launch point','','','','UNKO  M9200','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1552,'UNKMR2','M9207','launch point','','','','UNKO  M9207','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1553,'UNKMR2','M9208','launch point','','','','UNKO  M9208','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1554,'UNKMR2','M9209','launch point','','','','UNKO  M9209','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1555,'UNKMR2','M9210','launch point','','','','UNKO  M9210','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1556,'UNKMR2','','launch cruise','SU','1966-12-01 00:00:00','1967-02-01 00:00:00','UNKMR2','Unknown Soviet ship cruise, Dec 1966-Feb 1967','',null,null,null from dual union all
select 1557,'UNKMR3','M9132','launch point','','','','UNKO  M9132','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1558,'UNKMR3','M9135','launch point','','','','UNKO  M9135','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1559,'UNKMR3','M9136','launch point','','','','UNKO  M9136','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 156,'AOR4','','launch cruise','US','','','AOR4','USS Croatan, 1964 Atlantic Ocean cruise','Atlantic Ocean',null,null,null from dual union all
select 1560,'UNKMR3','M9161','launch point','','','','UNKO  M9161','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1561,'UNKMR3','M9162','launch point','','','','UNKO  M9162','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1562,'UNKMR3','M9163','launch point','','','','UNKO  M9163','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1563,'UNKMR3','M9164','launch point','','','','UNKO  M9164','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1564,'UNKMR3','M9165','launch point','','','','UNKO  M9165','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1565,'UNKMR3','M9166','launch point','','','','UNKO  M9166','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1566,'UNKMR3','M9167','launch point','','','','UNKO  M9167','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1567,'UNKMR3','M9168','launch point','','','','UNKO  M9168','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1568,'UNKMR3','M9171','launch point','','','','UNKO  M9171','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1569,'UNKMR3','M9172','launch point','','','','UNKO  M9172','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 157,'AOR40','','launch cruise','US','1966-02-01 00:00:00','1966-02-01 00:00:00','AOR40 -','Ship Sierra point, Feb 1966 [Unknown USN vessel maybe Range Rec.]','Atlantic Ocean',-63.5,26.6,.1 from dual union all
select 1570,'UNKMR3','','launch cruise','SU','1968-07-01 00:00:00','1968-09-01 00:00:00','UNKMR3','Unknown Soviet ship cruise, Jul-Sep 1968','',null,null,null from dual union all
select 1571,'UNKMR4','M9120','launch point','','','','UNKO  M9120','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1572,'UNKMR4','M9121','launch point','','','','UNKO  M9121','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1573,'UNKMR4','M9122','launch point','','','','UNKO  M9122','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1574,'UNKMR4','M9123','launch point','','','','UNKO  M9123','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1575,'UNKMR4','M9124','launch point','','','','UNKO  M9124','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1576,'UNKMR4','M9125','launch point','','','','UNKO  M9125','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1577,'UNKMR4','M9126','launch point','','','','UNKO  M9126','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1578,'UNKMR4','M9127','launch point','','','','UNKO  M9127','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1579,'UNKMR4','M9128','launch point','','','','UNKO  M9128','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 158,'AOR41','','launch cruise','US','1958-01-01 00:00:00','1958-01-01 00:00:00','AOR41 -','Terrier intercept test area off US Atlantic coast (RI/Mass?)','Atlantic Ocean',null,null,null from dual union all
select 1580,'UNKMR4','M9133','launch point','','','','UNKO  M9133','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1581,'UNKMR4','M9135','launch point','','','','UNKO  M9135','Unknown Soviet cruise meteo launch site  - see also UNKMR3','',null,null,null from dual union all
select 1582,'UNKMR4','M9138','launch point','','','','UNKO  M9138','Unknown Soviet cruise meteo launch site 1969','',null,null,null from dual union all
select 1583,'UNKMR4','M9166','launch point','','','','UNKO  M9166','Unknown Soviet cruise meteo launch site  - see also UNKMR3','',null,null,null from dual union all
select 1584,'UNKMR4','M9201','launch point','','','','UNKO  M9201','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1585,'UNKMR4','M9202','launch point','','','','UNKO  M9202','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1586,'UNKMR4','M9203','launch point','','','','UNKO  M9203','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1587,'UNKMR4','M9204','launch point','','','','UNKO  M9204','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1588,'UNKMR4','M9205','launch point','','','','UNKO  M9205','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1589,'UNKMR4','M9206','launch point','','','','UNKO  M9206','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 159,'AOR5','LP1','launch point','','1965-01-07 00:00:00','1965-01-07 00:00:00','AOR5  LP1','USNS Wheeling, 1965 Hydra-Iris SAA expedition','Atlantic Ocean',-27.42,-56.53,.02 from dual union all
select 1590,'UNKMR4','','launch cruise','SU','1969-01-01 00:00:00','1969-02-01 00:00:00','UNKMR4','Unknown Soviet ship cruise, Jan 1969','',null,null,null from dual union all
select 1591,'UNKMR5','M9127','launch point','','','','UNKO  M9127','Unknown Soviet cruise meteo launch site see also UNKMR4','',null,null,null from dual union all
select 1592,'UNKMR5','M9128','launch point','','','','UNKO  M9128','Unknown Soviet cruise meteo launch site see also UNKMR4','',null,null,null from dual union all
select 1593,'UNKMR5','M9129','launch point','','','','UNKO  M9129','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1594,'UNKMR5','M9130','launch point','','','','UNKO  M9130','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1595,'UNKMR5','M9131','launch point','','','','UNKO  M9131','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1596,'UNKMR5','M9132','launch point','','','','UNKO  M9132','Unknown Soviet cruise meteo launch site see also UNKMR3','',null,null,null from dual union all
select 1597,'UNKMR5','M9133','launch point','','','','UNKO  M9133','Unknown Soviet cruise meteo launch site see also UNKMR4','',null,null,null from dual union all
select 1598,'UNKMR5','M9134','launch point','','','','UNKO  M9134','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1599,'UNKMR5','M9135','launch point','','','','UNKO  M9135','Unknown Soviet cruise meteo launch site see also UNKMR3','',null,null,null from dual union all
select 16,'ALSAFIR','','launch site','SY','1992-07-01 00:00:00','','MINAK -','Syrian missile base, Al Safir, Aleppo, Syria (Base at Al Safir)','',37.3508,36.0339,.001 from dual union all
select 160,'AOR5','LP2','launch point','','1965-01-26 00:00:00','1965-01-26 00:00:00','AOR5  LP2','USNS Wheeling, 1965 Hydra-Iris SAA expedition  (LP2, ex AOR6)','Atlantic Ocean',-35.53,-35.05,.02 from dual union all
select 1600,'UNKMR5','M9136','launch point','','','','UNKO  M9136','Unknown Soviet cruise meteo launch site see also UNKMR3','',null,null,null from dual union all
select 1601,'UNKMR5','M9137','launch point','','','','UNKO  M9137','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1602,'UNKMR5','M9138','launch point','','','','UNKO  M9138','Unknown Soviet cruise meteo launch site 1969 - see also UNKMR4','',null,null,null from dual union all
select 1603,'UNKMR5','M9139','launch point','','','','UNKO  M9139','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1604,'UNKMR5','M9140','launch point','','','','UNKO  M9140','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1605,'UNKMR5','M9141','launch point','','','','UNKO  M9141','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1606,'UNKMR5','M9142','launch point','','','','UNKO  M9142','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1607,'UNKMR5','M9143','launch point','','','','UNKO  M9143','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1608,'UNKMR5','M9144','launch point','','','','UNKO  M9144','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1609,'UNKMR5','M9145','launch point','','','','UNKO  M9145','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 161,'AOR5','','launch cruise','US','1965-01-07 00:00:00','1965-01-07 00:00:00','AOR5','USNS Wheeling, 1965 Hydra-Iris SAA expedition','Atlantic Ocean',-27.42,-56.53,.02 from dual union all
select 1610,'UNKMR5','M9146','launch point','','','','UNKO  M9146','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1611,'UNKMR5','M9147','launch point','','','','UNKO  M9147','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1612,'UNKMR5','M9148','launch point','','','','UNKO  M9148','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1613,'UNKMR5','M9149','launch point','','','','UNKO  M9149','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1614,'UNKMR5','M9150','launch point','','','','UNKO  M9150','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1615,'UNKMR5','M9151','launch point','','','','UNKO  M9151','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1616,'UNKMR5','M9152','launch point','','','','UNKO  M9152','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1617,'UNKMR5','M9153','launch point','','','','UNKO  M9153','Unknown Soviet cruise meteo launch site','',null,null,null from dual union all
select 1618,'UNKMR5','','launch cruise','SU','1969-05-01 00:00:00','1969-07-01 00:00:00','UNKMR5','Unknown Soviet ship cruise, May-Jun 1969','',null,null,null from dual union all
select 1619,'UNKMR6','M9133','launch point','','','','UNKO  M9133','Unknown Soviet cruise meteo launch site see also UNKMR4/UNKMR5','',null,null,null from dual union all
select 162,'AOR7','LP1','launch point','','1958-08-27 00:00:00','1958-08-27 00:00:00','AOR7  LP1','USS Norton Sound, 1958 Argus expedition','Tristan de Cunha, Atlantic Ocean',-11.5,-38.5,.1 from dual union all
select 1620,'UNKMR6','M9137','launch point','','','','UNKO  M9137','Unknown Soviet cruise meteo launch site see also UNKMR5','',null,null,null from dual union all
select 1621,'UNKMR6','M9145','launch point','','','','UNKO  M9145','Unknown Soviet cruise meteo launch site see also UNKMR5','',null,null,null from dual union all
select 1622,'UNKMR6','','launch cruise','SU','1969-09-01 00:00:00','1969-10-01 00:00:00','UNKMR6','Unknown Soviet ship cruise, Sep-Oct 1969','',null,null,null from dual union all
select 1623,'UNKMR7','','launch cruise','SU','1970-01-01 00:00:00','1970-02-01 00:00:00','UNKMR7','Unknown Soviet ship cruise, Jan-Feb 1970','',null,null,null from dual union all
select 1624,'UNKMR8','','launch cruise','SU','1970-03-01 00:00:00','1970-04-01 00:00:00','UNKMR8','Unknown Soviet ship cruise, Mar-Apr 1970','',null,null,null from dual union all
select 1625,'UNKMR9','','launch cruise','SU','1970-06-01 00:00:00','1970-07-01 00:00:00','UNKMR9','Unknown Soviet ship cruise, Jun-Jul 1970','',null,null,null from dual union all
select 1626,'UNKPL','','launch zone','SU','','','UNKPL -','Unknown Soviet/Russian submarine launch area','',null,null,null from dual union all
select 1627,'UNKV2','','launch site','DR','','','UNKV2 -','Unknown V2 WWII launch site, Europe','',6,50,2.5 from dual union all
select 1628,'URDA','','launch site','SU','1957-01-01 00:00:00','1957-01-01 00:00:00','Urda','Tactical launch site, Urda, Kazakhstan','',47.48,48.76,.05 from dual;
insert into SITE
select 1629,'URUM','','launch site','CN','','','URLM','Korla Missile Test Center, Launch site near Urumqi, China','',87.6,43.8,2 from dual union all
select 163,'AOR7','LP2','launch point','','1958-08-30 00:00:00','1958-08-30 00:00:00','AOR7  LP2','Atlantic Ocean, Argus cruise LP2 (ex AOR8)','Atlantic Ocean',-8.2,-49.5,.1 from dual union all
select 1630,'USC','','launch site','J','2003-10-01 00:00:00','','USC   -','Uchinoura Space Center, Kagoshima (formerly Kagoshima)','',131.079,31.25,.001 from dual union all
select 1631,'USHA76A','','launch cruise','SU','1976-04-01 00:00:00','1976-06-01 00:00:00','Ushakov 76','Ushakov cruise 1976','',null,null,null from dual union all
select 1632,'USHA76B','','launch cruise','SU','1976-08-01 00:00:00','1976-10-01 00:00:00','Ushakov 76','Ushakov cruise 1976','',null,null,null from dual union all
select 1633,'USHA77A','','launch cruise','SU','1977-04-01 00:00:00','1977-06-01 00:00:00','Ushakov 77','Ushakov cruise 1977','',null,null,null from dual union all
select 1634,'USHA77B','','launch cruise','SU','1977-07-01 00:00:00','1977-09-01 00:00:00','Ushakov 77','Ushakov cruise 1977','',null,null,null from dual union all
select 1635,'USHA78A','','launch cruise','SU','1978-02-01 00:00:00','1978-02-01 00:00:00','Ushakov 78','Ushakov cruise 1978','',null,null,null from dual union all
select 1636,'USHA78B','','launch cruise','SU','1978-04-01 00:00:00','1978-04-01 00:00:00','Ushakov 78','Ushakov cruise 1978','',null,null,null from dual union all
select 1637,'USHA79A','','launch cruise','SU','1979-11-01 00:00:00','1979-12-01 00:00:00','Ushakov 79','Ushakov cruise 1979','',null,null,null from dual union all
select 1638,'USHA80A','','launch cruise','SU','1980-03-01 00:00:00','1980-03-01 00:00:00','Ushakov 80','Ushakov cruise 1980','',null,null,null from dual union all
select 1639,'USHA86A','','launch cruise','SU','1986-09-01 00:00:00','1986-09-01 00:00:00','Ushakov 86','Ushakov cruise 1986','',null,null,null from dual union all
select 164,'AOR7','LP3','launch point','','1958-09-06 00:00:00','1958-09-06 00:00:00','AOR7  LP3','Atlantic Ocean, Argus cruise LP3 (ex AOR9)','Atlantic Ocean',-9.7,-48.5,.1 from dual union all
select 1640,'USHA87A','','launch cruise','SU','1987-11-01 00:00:00','1987-12-01 00:00:00','Ushakov 87','Ushakov cruise 1987','',null,null,null from dual union all
select 1641,'USHA88A','','launch cruise','SU','1988-02-01 00:00:00','1988-03-01 00:00:00','Ushakov 88','Ushakov cruise 1988','',null,null,null from dual union all
select 1642,'USHA88B','','launch cruise','SU','1988-05-01 00:00:00','1988-06-01 00:00:00','Ushakov 88','Ushakov cruise 1988','',null,null,null from dual union all
select 1643,'USHA88C','','launch cruise','SU','1988-08-01 00:00:00','1988-09-01 00:00:00','Ushakov 88','Ushakov cruise 1988','',null,null,null from dual union all
select 1644,'USHA89A','','launch cruise','SU','1989-10-01 00:00:00','1989-10-01 00:00:00','Ushakov 89','Ushakov cruise 1989','',null,null,null from dual union all
select 1645,'USHA90A','','launch cruise','SU','1990-01-01 00:00:00','1990-01-01 00:00:00','Ushakov 90','Ushakov cruise 1990','',null,null,null from dual union all
select 1646,'UZHUR','','launch site','SU','1960-01-01 00:00:00','1990-01-01 00:00:00','UZHUR','Uzhur missile site, Russia','',90.253,55.2075,.01 from dual union all
select 1647,'V','394A1','launch point','','1962-09-28 00:00:00','1963-12-20 00:00:00','V     394A1','Silo 394-A1, Vandenberg AFB','',-120.574,34.842,.005 from dual union all
select 1648,'V','394A2','launch point','','1962-09-28 00:00:00','1963-12-20 00:00:00','V     394A2','Silo 394-A2, Vandenberg AFB','',-120.5699,34.8427,.005 from dual union all
select 1649,'V','394A3','launch point','','1962-09-28 00:00:00','1963-12-20 00:00:00','V     394A3','Silo 394-A3, Vandenberg AFB','',-120.5968,34.8567,.005 from dual union all
select 165,'AOR7','','launch cruise','US','1958-08-27 00:00:00','1958-09-01 00:00:00','AOR7','USS Norton Sound, 1958 Argus expedition','Tristan de Cunha, Atlantic Ocean',-11.5,-38.5,.1 from dual union all
select 1650,'V','394A4','launch point','','1962-09-28 00:00:00','1963-12-20 00:00:00','V     394A4','Silo 394-A4, Vandenberg AFB','',-120.5984,34.8592,.005 from dual union all
select 1651,'V','394A5','launch point','','1962-09-28 00:00:00','1963-12-20 00:00:00','V     394A5','Silo 394-A5, Vandenberg AFB','',-120.622,34.8811,.005 from dual union all
select 1652,'V','394A6','launch point','','1962-09-28 00:00:00','1963-12-20 00:00:00','V     394A6','Silo 394-A6, Vandenberg AFB','',-120.6212,34.8847,.005 from dual union all
select 1653,'V','394A7','launch point','','1962-09-28 00:00:00','1963-12-20 00:00:00','V     394A7','Silo 394-A7, Vandenberg AFB','',-120.5862,34.8468,.005 from dual union all
select 1654,'V','395-A1','launch point','','1961-09-23 00:00:00','1965-03-05 00:00:00','V     395-A1','Silo 395-A1, Vandenberg AFB','',-120.5348,34.8016,.005 from dual union all
select 1655,'V','395-A2','launch point','','1961-09-23 00:00:00','1965-03-05 00:00:00','V     395-A2','Silo 395-A2, Vandenberg AFB','',-120.5373,34.8003,.005 from dual union all
select 1656,'V','395-A3','launch point','','1961-09-23 00:00:00','1965-03-05 00:00:00','V     395-A3','Silo 395-A3, Vandenberg AFB','',-120.5365,34.8023,.005 from dual union all
select 1657,'V','395-B','launch point','','1964-02-17 00:00:00','1969-05-20 00:00:00','V     395-B','Silo 395-B, Vandenberg AFB','',-120.5992,34.7783,.005 from dual union all
select 1658,'V','395-C','launch point','','1963-02-16 00:00:00','1976-06-27 00:00:00','V     395-C','Silo 395-C, Vandenberg AFB','',-120.5898,34.7304,.005 from dual union all
select 1659,'V','395-D','launch point','','1963-05-13 00:00:00','1966-04-05 00:00:00','V     395-D','Silo 395-D, Vandenberg AFB','',-120.5846,34.703,.005 from dual union all
select 166,'APC','KT-408','launch point','','1972-02-22 00:00:00','1972-02-22 00:00:00','APC   KT-408','KT No. 408, Luna-20 Site, Apollonius C, Mare Undarum B3','Luna',56.55,3.53,.01 from dual union all
select 1660,'V','4300B6','launch point','','1965-01-19 00:00:00','1967-02-08 00:00:00','V     4300B6','Complex 4300-B6, Vandenberg AFB','',-120.6172,34.7612,.05 from dual union all
select 1661,'V','4300C','launch point','','1963-12-01 00:00:00','1967-02-01 00:00:00','V     4300C','Complex 4300C, Vandenberg AFB','',-120.6085,34.7667,.005 from dual union all
select 1662,'V','576A1','launch point','','1959-09-09 00:00:00','1965-03-26 00:00:00','V     576A1','Complex 576-A1, Vandenberg AFB','',-120.593,34.7741,.005 from dual union all
select 1663,'V','576A2','launch point','','1959-09-09 00:00:00','1965-03-26 00:00:00','V     576A2','Complex 576-A2, Vandenberg AFB','',-120.5886,34.774,.005 from dual union all
select 1664,'V','576A3','launch point','','1959-09-09 00:00:00','1965-03-26 00:00:00','V     576A3','Complex 576-A3, Vandenberg AFB','',-120.5919,34.7783,.005 from dual union all
select 1665,'V','576B1','launch point','','1960-04-22 00:00:00','1965-05-06 00:00:00','V     576B1','Complex 576-B1, Vandenberg AFB','',-120.587,34.7896,.005 from dual union all
select 1666,'V','576B2','launch point','','1960-04-22 00:00:00','1965-05-06 00:00:00','V     576B2','Complex 576-B2, Vandenberg AFB','',-120.5829,34.7868,.005 from dual union all
select 1667,'V','576B3','launch point','','1960-04-22 00:00:00','1965-05-06 00:00:00','V     576B3','Complex 576-B3, Vandenberg AFB','',-120.5865,34.7893,.005 from dual union all
select 1668,'V','576C','launch point','','1963-07-03 00:00:00','1963-09-25 00:00:00','V     576C','Complex 576-C, Vandenberg AFB','',-120.5745,34.8051,.005 from dual union all
select 1669,'V','576D','launch point','','1963-03-16 00:00:00','1964-08-31 00:00:00','V     576D','Complex 576-D, Vandenberg AFB','',-120.5471,34.8167,.005 from dual union all
select 167,'APC','','launch site','LUNA','1972-02-22 00:00:00','1972-02-22 00:00:00','APC   KT-408','KT No. 408, Luna-20 Site, Apollonius C, Mare Undarum B3','Luna',56.55,3.53,.01 from dual union all
select 1670,'V','576E','launch point','','1962-08-01 00:00:00','','V     576E','Complex 576-E, Vandenberg AFB','',-120.6123,34.7372,.005 from dual union all
select 1671,'V','576F','launch point','','1963-08-24 00:00:00','1964-08-27 00:00:00','V     576F','Complex 576-F, Vandenberg AFB','',-120.5797,34.7937,.005 from dual union all
select 1672,'V','576G','launch point','','1963-10-04 00:00:00','1965-01-08 00:00:00','V     576G','Complex 576-G, Vandenberg AFB','',-120.5511,34.8192,.005 from dual union all
select 1673,'V','75-1-1','launch point','','1958-12-16 00:00:00','1966-06-24 00:00:00','V     75-1-1','Complex 75-1, Pad 1, Vandenberg AFB','',-120.6106,34.7468,.005 from dual union all
select 1674,'V','75-1-2','launch point','','1959-09-17 00:00:00','1966-02-09 00:00:00','V     75-1-2','Complex 75-1, Pad 2, Vandenberg AFB','',-120.6155,34.7523,.005 from dual union all
select 1675,'V','75-2-6','launch point','','1959-06-16 00:00:00','1959-08-14 00:00:00','V     75-2-6','Complex 75-2, Pad 6, Vandenberg AFB','',-120.6172,34.7612,.05 from dual union all
select 1676,'V','75-2-7','launch point','','1960-06-22 00:00:00','1961-06-20 00:00:00','V     75-2-7','Complex 75-2, Pad 7, Vandenberg AFB','',-120.6139,34.7592,.05 from dual union all
select 1677,'V','75-2-8','launch point','','1959-04-16 00:00:00','1960-12-13 00:00:00','V     75-2-8','Complex 75-2, Pad 8, Vandenberg AFB','',-120.6147,34.7633,.005 from dual union all
select 1678,'V','75-3-4','launch point','','1959-01-21 00:00:00','1966-03-09 00:00:00','V     75-3-4','Complex 75-3, Pad 4, Vandenberg AFB','',-120.6237,34.755,.005 from dual union all
select 1679,'V','75-3-5','launch point','','1959-06-25 00:00:00','1966-06-21 00:00:00','V     75-3-5','Complex 75-3, Pad 5, Vandenberg AFB','',-120.6188,34.7523,.005 from dual union all
select 168,'ARO1','LP1','launch point','','1952-08-21 00:00:00','1952-08-21 00:00:00','ARO1  LP1','Arctic Ocean Launch Site 1 Launch Point 1','Arctic Ocean',-68.47,80.1,.1 from dual union all
select 1680,'V','ABRESA1','launch point','','1965-06-08 00:00:00','1971-04-05 00:00:00','V     ABRESA1','Complex ABRES-A, Pad 1, Vandenberg AFB','',-120.593,34.7741,.005 from dual union all
select 1681,'V','ABRESA2','launch point','','1965-08-05 00:00:00','1969-03-18 00:00:00','V     ABRESA2','Complex ABRES-A, Pad 2, Vandenberg AFB','',-120.5886,34.774,.005 from dual union all
select 1682,'V','ABRESA3','launch point','','1965-06-10 00:00:00','1971-06-29 00:00:00','V     ABRESA3','Complex ABRES-A, Pad 3, Vandenberg AFB','',-120.5919,34.7783,.005 from dual union all
select 1683,'V','ABRESB1','launch point','','1965-07-01 00:00:00','1966-06-10 00:00:00','V     ABRESB1','Complex ABRES-B, Pad 1, Vandenberg AFB','',-120.587,34.7896,.005 from dual union all
select 1684,'V','ABRESB2','launch point','','1965-06-03 00:00:00','1967-11-07 00:00:00','V     ABRESB2','Complex ABRES-B, Pad 2, Vandenberg AFB','',-120.5829,34.7868,.005 from dual union all
select 1685,'V','ABRESB3','launch point','','1965-05-28 00:00:00','1967-10-11 00:00:00','V     ABRESB3','Complex ABRES-B, Pad 3, Vandenberg AFB','',-120.5865,34.7893,.005 from dual union all
select 1686,'V','BMRSA1','launch point','','1971-09-01 00:00:00','1974-09-08 00:00:00','V     BMRSA1','Complex BMRS-A, Pad 1, Vandenberg AFB','',-120.593,34.7741,.005 from dual union all
select 1687,'V','BMRSA2','launch point','','1971-08-07 00:00:00','1971-08-07 00:00:00','V     BMRSA2','Complex BMRS-A, Pad 2, Vandenberg AFB','',-120.5886,34.774,.005 from dual union all
select 1688,'V','BMRSA3','launch point','','1973-08-29 00:00:00','1989-10-05 00:00:00','V     BMRSA3','Complex BMRS-A, Pad 3, Vandenberg AFB','',-120.5919,34.7783,.005 from dual union all
select 1689,'V','BOM-1','launch point','','','','V     BOM-1','BOMARC launcher 1, Vandenberg AFB','',null,null,null from dual union all
select 169,'ARO1','LP2','launch point','','1952-08-24 00:00:00','1952-08-29 00:00:00','ARO1  LP2','Arctic Ocean Launch Site 1 Launch Point 2','Arctic Ocean',-73.5,77.52,.1 from dual union all
select 1690,'V','BOM-2','launch point','','','','V     BOM-2','BOMARC launcher 2, Vandenberg AFB','',null,null,null from dual union all
select 1691,'V','BOM1','launch point','','1966-01-01 00:00:00','1982-01-01 00:00:00','V     BOM1','BOMARC Complex 1, Vandenberg AFB','',-120.5911,34.7961,.005 from dual union all
select 1692,'V','BOM2','launch point','','1966-01-01 00:00:00','1982-01-01 00:00:00','V     BOM2','BOMARC Complex 2, Vandenberg AFB','',-120.5911,34.7982,.005 from dual union all
select 1693,'V','HP-06','launch point','','','','V     HP-06','HP-06 GLCM launcher, Vandenberg AFB','',-120.6,34.8,.1 from dual union all
select 1694,'V','LE-6','launch point','','1967-06-29 00:00:00','1967-10-11 00:00:00','V     LE-6','Launch Emplacement 6, Vandenberg AFB','',-120.6172,34.7612,.05 from dual union all
select 1695,'V','LE-7','launch point','','1961-09-06 00:00:00','1962-03-19 00:00:00','V     LE-7','Launch Emplacement 7, Vandenberg AFB','',-120.6139,34.7592,.05 from dual union all
select 1696,'V','LE-8','launch point','','1961-12-06 00:00:00','1962-06-19 00:00:00','V     LE-8','Launch Emplacement 8, Vandenberg AFB','',-120.6147,34.7633,.005 from dual union all
select 1697,'V','LF02','launch point','','1964-01-16 00:00:00','','V     LF02','Launch Facility 02, Vandenberg AFB','',-120.574,34.842,.005 from dual union all
select 1698,'V','LF03','launch point','','1964-01-29 00:00:00','','V     LF03','Launch Facility 03, Vandenberg AFB','',-120.5699,34.8427,.005 from dual union all
select 1699,'V','LF04','launch point','','1964-01-23 00:00:00','','V     LF04','Launch Facility 04, Vandenberg AFB','',-120.5968,34.8567,.005 from dual union all
select 17,'ALSULAY','','launch site','SA','','','Al Sulayil','Al Sulayil missile base, Saudi Arabia','al-Sulayil, Sauid Arabia',45.5836,20.7186,.001 from dual union all
select 170,'ARO1','LP3','launch point','','1952-08-24 00:00:00','1952-08-29 00:00:00','ARO1  LP3','Arctic Ocean Launch Site 1 Launch Point 3','Arctic Ocean',-73.5,77.55,.1 from dual union all
select 1700,'V','LF05','launch point','','1964-02-29 00:00:00','','V     LF05','Launch Facility 05, Vandenberg AFB','',-120.5984,34.8592,.005 from dual union all
select 1701,'V','LF06','launch point','','1964-02-11 00:00:00','','V     LF06','Launch Facility 06, Vandenberg AFB','',-120.622,34.8811,.005 from dual union all
select 1702,'V','LF07','launch point','','1964-01-10 00:00:00','1987-11-09 00:00:00','V     LF07','Launch Facility 07, Vandenberg AFB','Vandenberg AFB, California',-120.6212,34.8847,.005 from dual union all
select 1703,'V','LF08','launch point','','1964-08-21 00:00:00','1991-06-11 00:00:00','V     LF08','Launch Facility 08, Vandenberg AFB','',-120.5862,34.8468,.005 from dual union all
select 1704,'V','LF08/Rail','launch point','','1994-02-04 00:00:00','1994-02-04 00:00:00','V     LF08/Rail','Launch Facility 08 (Rail), Vandenberg AFB','',-120.586,34.847,.009 from dual union all
select 1705,'V','LF09','launch point','','1964-06-29 00:00:00','','V     LF09','Launch Facility 09, Vandenberg AFB','',-120.622,34.8777,.005 from dual union all
select 1706,'V','LF10','launch point','','1987-07-12 00:00:00','','V     LF10','Launch Facility 10, Vandenberg AFB','',-120.6057,34.8715,.02 from dual union all
select 1707,'V','LF21','launch point','','1965-08-18 00:00:00','1980-03-27 00:00:00','V     LF21','Launch Facility 21, Vandenberg AFB','',-120.5821,34.8578,.005 from dual union all
select 1708,'V','LF22','launch point','','1965-10-06 00:00:00','1975-12-17 00:00:00','V     LF22','Launch Facility 22, Vandenberg AFB  (=LF10?)','',-120.5829,34.8468,.005 from dual union all
select 1709,'V','LF23','launch point','','1966-08-26 00:00:00','1966-08-26 00:00:00','V     LF23','Launch Facility 23, Vandenberg AFB','',-120.5862,34.8523,.005 from dual union all
select 171,'ARO1','LP4','launch point','','1952-08-24 00:00:00','1952-08-29 00:00:00','ARO1  LP4','Arctic Ocean Launch Site 1 Launch Point 4','Arctic Ocean',-73.5,77.53,.1 from dual union all
select 1710,'V','LF24','launch point','','1965-12-15 00:00:00','1971-05-18 00:00:00','V     LF24','Launch Facility 24, Vandenberg AFB','',-120.5903,34.8537,.005 from dual union all
select 1711,'V','LF25','launch point','','1966-02-16 00:00:00','1976-03-06 00:00:00','V     LF25','Launch Facility 25, Vandenberg AFB','',-120.6172,34.8811,.005 from dual union all
select 1712,'V','LF26','launch point','','1966-01-18 00:00:00','','V     LF26','Launch Facility 26, Vandenberg AFB','',-120.6237,34.8879,.005 from dual union all
select 1713,'V','LF7632','launch point','','','','V     LF7632','Launch Facility 7632, Vandenberg AFB','',null,null,null from dual union all
select 1714,'V','OSTF','launch point','','1960-12-03 00:00:00','1963-07-26 00:00:00','V     OSTF','Operational Silo Test Facility, Vandenberg AFB','',-120.58,34.79,.02 from dual union all
select 1715,'V','OSTF1','launch point','','1960-12-03 00:00:00','1963-07-26 00:00:00','V     OSTF1','Operational Silo Test Facility 1, Vandenberg AFB','',-120.5797,34.7937,.005 from dual union all
select 1716,'V','OSTF2','launch point','','1962-08-10 00:00:00','1963-12-18 00:00:00','V     OSTF2','Operational Silo Test Facility 2, Vandenberg AFB','',-120.5511,34.8192,.005 from dual union all
select 1717,'V','RW30','launch point','','1990-01-01 00:00:00','','V     RW30','RW30,    Vandenberg AFB, California','',-120.5658,34.7321,.005 from dual union all
select 1718,'V','RW30/12','launch point','','1990-01-01 00:00:00','','V     RW30/12','RW30/12, Vandenberg AFB, California','',-120.5658,34.7321,.005 from dual;
insert into SITE
select 1719,'V','SLC10E','launch point','','1968-05-23 00:00:00','1968-05-23 00:00:00','V     SLC10E','Space Launch Complex 10-East, Vandenberg AFB','',-120.6139,34.7592,.05 from dual union all
select 172,'ARO1','LP5','launch point','','1952-08-24 00:00:00','1952-08-29 00:00:00','ARO1  LP5','Arctic Ocean Launch Site 1 Launch Point 5','Arctic Ocean',-73.48,77.35,.1 from dual union all
select 1720,'V','SLC10W','launch point','','1968-05-23 00:00:00','1980-07-15 00:00:00','V     SLC10W','Space Launch Complex 10-West, Vandenberg AFB','',-120.6172,34.7612,.05 from dual union all
select 1721,'V','SLC1E','launch point','','1967-05-09 00:00:00','1968-09-18 00:00:00','V     SLC1E','Space Launch Complex 1-East, Vandenberg AFB','',-120.6188,34.7523,.005 from dual union all
select 1722,'V','SLC1W','launch point','','1966-08-09 00:00:00','1971-12-14 00:00:00','V     SLC1W','Space Launch Complex 1-West, Vandenberg AFB','',-120.6237,34.755,.005 from dual union all
select 1723,'V','SLC2E','launch point','','1966-10-02 00:00:00','1972-03-12 00:00:00','V     SLC2E','Space Launch Complex 2-East, Vandenberg AFB','',-120.6106,34.7468,.005 from dual union all
select 1724,'V','SLC2W','launch point','','1966-12-29 00:00:00','','V     SLC2W','Space Launch Complex 2-West, Vandenberg AFB','',-120.6155,34.7523,.005 from dual union all
select 1725,'V','SLTF','launch point','','1961-05-03 00:00:00','1961-05-03 00:00:00','V     SLTF','Silo Lift Test Facility, Vandenberg AFB','',-120.5405,34.8016,.005 from dual union all
select 1726,'V','TP-01','launch point','','1983-06-17 00:00:00','1991-04-18 00:00:00','V     TP-01','Silo TP-01, Vandenberg AFB','',-120.6,34.8,.1 from dual union all
select 1727,'V','','launch site','US','1958-01-01 00:00:00','','V     -','Vandenberg AFB, California','',-120.62,34.75,.02 from dual union all
select 1728,'V2BURG','','launch site','DR','','','V2BUR -','V-2 launch area, Burgsteinfurt','',7.3,52.1,.1 from dual union all
select 1729,'V2ENS','','launch site','DR','1944-01-01 00:00:00','1944-01-01 00:00:00','V2 Enschede','V-2 launch area, Enschede, Nederlanden','',6.89,52.22,.05 from dual union all
select 173,'ARO1','LP6','launch point','','1952-08-31 00:00:00','1952-08-31 00:00:00','ARO1  LP6','Arctic Ocean Launch Site 1 Launch Point 6','Arctic Ocean',-72.83,77.33,.1 from dual union all
select 1730,'V2EUS','','launch site','DR','','','V2EUS -','V-2 launch area, Euskirchen, Deutschland','',6.8,50.65,.1 from dual union all
select 1731,'V2HAAG','LP1','launch point','DR','','','V2 Haag','V-2 launch area near Den Haag-Wassenaar','',4.3799,52.1302,.01 from dual union all
select 1732,'V2HAAG','LP2','launch point','DR','','','V2 Haag','V-2 launch area near Den Haag-Wassenaar','',4.3693,52.12,.01 from dual union all
select 1733,'V2HAAG','LP3','launch point','DR','1945-03-03 00:00:00','1945-03-01 00:00:00','V2 Haag','V-2 launch area at Duindigt Race Course, Den Haag','',4.3448,52.1067,.0002 from dual union all
select 1734,'V2HAAG','','launch site','DR','','','V2HAA -','V-2 launch area near Den Haag-Wassenaar','',4.3799,52.1302,.01 from dual union all
select 1735,'V2HACH','','launch site','DR','','','V2HAC -','V-2 launch area near Hachenburg/Montaburg','',7.8,50.5,.2 from dual union all
select 1736,'V2HELL','','launch site','DR','','','V2 Hell.','V-2 launch area at Hellendoorn/Eelerberg','',6.43,52.41,.1 from dual union all
select 1737,'V2HERM','','launch site','DR','','','V2HER -','V-2 launch area at Hermeskeil','',6.9,49.65,.1 from dual union all
select 1738,'V2HOUF','','launch site','DR','','','V2VIE -','V-2 launch areas, Houffalize, Belgium','',5.79,50.04,.02 from dual union all
select 1739,'V2MERZ','','launch site','DR','','','V2MER -','V-2 launch area, Merzig, N of Saar','',6.6,49.45,.1 from dual union all
select 174,'ARO1','LP7','launch point','','1952-08-31 00:00:00','1952-08-31 00:00:00','ARO1  LP7','Arctic Ocean Launch Site 1 Launch Point 7','Arctic Ocean',-70.22,77.18,.1 from dual union all
select 1740,'V2REM','','launch site','DR','','','V2 Remscheid','V-2 launch area, Remscheid','',7.2,51.18,.1 from dual union all
select 1741,'V2STAV','LP1','launch point','DR','','','V2 Rijs','V-2 launch area, Rijs wood, Friesland','',5.4838,52.8563,.0002 from dual union all
select 1742,'V2STAV','','launch site','DR','','','V2STA -','V-2 launch area, Stavorem, Friesland','',5.4,52.9,.1 from dual union all
select 1743,'V2VIEL','','launch site','DR','','','V2VIE -','V-2 launch areas, Fraiture, Vielsalm, Ardennes','',5.752,50.258,.02 from dual union all
select 1744,'V2VUGHT','','launch site','DR','1944-01-01 00:00:00','1944-01-01 00:00:00','V2 Vught','V-2 launch area, Beukenhorst mansion, Vught, Nederlanden','',5.2885,51.6241,.0002 from dual union all
select 1745,'V2WAL','','launch site','DR','','','V2WAL -','V-2 launch area, Serooskerke, Walcheren, Holland','',3.5899,51.5515,.0002 from dual union all
select 1746,'V2ZWOL','','launch site','DR','','','V2ZWO -','V-2 launch area, Zwolle-Hellendoorn-Enschede','',6.45,52.4,.2 from dual union all
select 1747,'VCM','','launch site','ARV','1969-01-01 00:00:00','1984-01-01 00:00:00','VCM   -','Vicecomodoro Marambio Station, Seymour I, Graham Land, Antarctica','',-56.72,-64.23,.02 from dual union all
select 1748,'VERNON','','launch site','F','1951-04-01 00:00:00','1951-04-01 00:00:00','VERNO -','Vernon, Eure, France','',1.48,49.08,.02 from dual union all
select 1749,'VIK','','launch site','IS','1964-08-01 00:00:00','1965-09-03 00:00:00','VIK   -','Vik, Myrdalsandir, Island','',-19,63.42,.02 from dual union all
select 175,'ARO1','','launch cruise','US','1952-08-21 00:00:00','1952-08-21 00:00:00','ARO1','Eastwind 1952 Arctic expedition','Arctic Ocean',-68.47,80.1,.1 from dual union all
select 1750,'VISKH','','launch site','IN','','','VISKH -','DRDO-ENESTIEL Naval Sci/Tech Lab, Visakhapatnam, Andhra Pradesh','',83.25,17.7,.1 from dual union all
select 1751,'VLAD','PL1','launch point','','1950-01-01 00:00:00','','VLAD  PL1','PL 1, Vladimirovka Poligon','',46.2,48.4,.1 from dual union all
select 1752,'VLAD','','launch site','SU','1950-01-01 00:00:00','','VLAD  -','Vladimirovka Poligon','',46.2,48.3,.1 from dual union all
select 1753,'VOKV82A','','launch cruise','SU','1982-11-01 00:00:00','1983-01-01 00:00:00','Voyekov 82','Vokeyov cruise 1982','',null,null,null from dual union all
select 1754,'VOKV83A','','launch cruise','SU','1983-08-01 00:00:00','1983-09-01 00:00:00','Voyekov 83','Vokeyov cruise 1983','',null,null,null from dual union all
select 1755,'VOKV86A','','launch cruise','SU','1986-08-01 00:00:00','1986-08-01 00:00:00','Voyekov 86','Vokeyov cruise 1986','',null,null,null from dual union all
select 1756,'VOKV87A','','launch cruise','SU','1987-05-01 00:00:00','1987-06-01 00:00:00','Voyekov 87','Vokeyov cruise 1987','',null,null,null from dual union all
select 1757,'VOLN76A','','launch cruise','SU','1976-02-01 00:00:00','1976-03-01 00:00:00','Volna 76','Volna cruise 1976','',null,null,null from dual union all
select 1758,'VOLN77A','','launch cruise','SU','1977-03-01 00:00:00','1977-05-01 00:00:00','Volna 77','Volna cruise 1977','',null,null,null from dual union all
select 1759,'VOLN78A','','launch cruise','SU','1978-03-01 00:00:00','1978-03-01 00:00:00','Volna 78','Volna cruise 1978','',null,null,null from dual union all
select 176,'ASC','','launch site','SH','1964-01-29 00:00:00','1968-01-01 00:00:00','ASC   -','Ascension Island, Atlantic Ocean','Ascension Island, Atlantic Ocean',-14.4142,-7.9762,.0001 from dual union all
select 1760,'VOLN79A','','launch cruise','SU','1979-12-01 00:00:00','1979-12-01 00:00:00','Volna 79','Volna cruise 1979','',null,null,null from dual union all
select 1761,'VOLN80A','','launch cruise','SU','1980-02-01 00:00:00','1980-02-01 00:00:00','Volna 80','Volna cruise 1980','',null,null,null from dual union all
select 1762,'VOLN81A','','launch cruise','SU','1981-01-01 00:00:00','1981-03-01 00:00:00','Volna 81','Volna cruise 1981','',null,null,null from dual union all
select 1763,'VORK','','launch site','SU','1960-01-01 00:00:00','1990-01-01 00:00:00','Vorkuta','Vorkuta Sovetskiy, R-12 missile deployment','Vorkuta, Komi Republic',64.3,67.46,.01 from dual union all
select 1764,'VOST','PU1S','launch point','','2016-01-01 00:00:00','','Vostochniy PU1S','PU1S (371SK14) Soyuz pad, Vostochniy, Svobodniy, Amurskaya Oblast'', Rossiya','',128.3322,51.8769,.001 from dual union all
select 1765,'VOST','','launch site','RU','2016-01-01 00:00:00','','Vostochniy','Vostochniy, Svobodniy, Amurskaya Oblast'', Rossiya','',128,51.7,.5 from dual union all
select 1766,'VS','PALC-A','launch point','','1964-07-01 00:00:00','1965-01-01 00:00:00','V     PALC-A','Point Arguello Launch Complex A, Vandenberg AFB','',-120.5992,34.6591,.005 from dual union all
select 1767,'VS','PALC-B','launch point','','1964-07-01 00:00:00','1965-01-01 00:00:00','V     PALC-B','Point Arguello Launch Complex B, Vandenberg AFB','',-120.6017,34.6523,.005 from dual union all
select 1768,'VS','PALC-C','launch point','','1964-07-01 00:00:00','1972-10-11 00:00:00','V     PALC-C','Point Arguello Launch Complex C, Vandenberg AFB','',-120.6269,34.6057,.005 from dual union all
select 1769,'VS','PALC-D','launch point','','1964-07-01 00:00:00','1966-05-19 00:00:00','V     PALC-D','Point Arguello Launch Complex D, Vandenberg AFB','',-120.6234,34.6043,.005 from dual union all
select 177,'ASHU','','launch site','SU','','','Ashuluk','Ashuluk, Rossiya','',48.0343,47.4224,.0001 from dual union all
select 1770,'VS','PALC1-1','launch point','','1964-07-01 00:00:00','1966-05-24 00:00:00','V     PALC1-1','Point Arguello Launch Complex 1-1, Vandenberg AFB','',-120.5911,34.64,.005 from dual union all
select 1771,'VS','PALC1-2','launch point','','1964-07-01 00:00:00','1966-06-09 00:00:00','V     PALC1-2','Point Arguello Launch Complex 1-2, Vandenberg AFB','',-120.5878,34.6359,.005 from dual union all
select 1772,'VS','PALC2-3','launch point','','1964-07-01 00:00:00','1965-03-12 00:00:00','V     PALC2-3','Point Arguello Launch Complex 2-3, Vandenberg AFB','',-120.6139,34.629,.005 from dual union all
select 1773,'VS','PALC2-4','launch point','','1964-07-01 00:00:00','1966-06-03 00:00:00','V     PALC2-4','Point Arguello Launch Complex 2-4, Vandenberg AFB','',-120.609,34.6276,.005 from dual union all
select 1774,'VS','PLC-C','launch point','','1975-08-14 00:00:00','1975-12-12 00:00:00','V     PLC-C','Probe Launch Complex C, Vandenberg AFB','',-120.6269,34.6057,.005 from dual union all
select 1775,'VS','SLC3E','launch point','','1966-08-19 00:00:00','','V     SLC3E','Space Launch Complex 3-East, Vandenberg AFB','',-120.5878,34.6359,.005 from dual union all
select 1776,'VS','SLC3W','launch point','','1966-09-20 00:00:00','1995-03-24 00:00:00','V     SLC3W','Space Launch Complex 3-West, Vandenberg AFB','',-120.5911,34.64,.005 from dual union all
select 1777,'VS','SLC4E','launch point','','1966-07-12 00:00:00','','V     SLC4E','Space Launch Complex 4-East, Vandenberg AFB','Vandenberg AFB, California',-120.609,34.6276,.005 from dual union all
select 1778,'VS','SLC4W','launch point','','1966-07-29 00:00:00','','V     SLC4W','Space Launch Complex 4-West, Vandenberg AFB','',-120.6139,34.629,.005 from dual union all
select 1779,'VS','SLC5','launch point','','1966-08-04 00:00:00','1994-05-09 00:00:00','V     SLC5','Space Launch Complex 5, Vandenberg AFB','',-120.6234,34.6043,.005 from dual union all
select 178,'AUR','','launch site','MH','1997-02-01 00:00:00','','AUR   -','Aur Atoll launch site','Bigen Island, Aur Atoll, Marshall Is.',171.04,8.36,.01 from dual union all
select 1780,'VS','SLC6','launch point','','1966-01-01 00:00:00','','V     SLC6','Space Launch Complex 6, Vandenberg AFB','',-120.6253,34.5783,.05 from dual union all
select 1781,'VS','SLC8','launch point','','1999-12-01 00:00:00','','V     SLC8','Commercial Space Launch Facility, California Spaceport, Vandenberg AFB','',-120.6324,34.5764,.001 from dual union all
select 1782,'VS','','launch site','US','1964-12-31 00:00:00','','V     -','South Vandenberg AFB, California','',-120.61,34.63,.02 from dual union all
select 1783,'WAKE','T','launch point','','2015-01-01 00:00:00','','WAKE  -','THAAD Launcher, Wake Island','',166.6539,19.2728,.0001 from dual union all
select 1784,'WAKE','','launch site','UM79','1974-02-13 00:00:00','','WAKE  -','Wake Island','',166.618,19.29,.005 from dual union all
select 1785,'WC','CB','launch point','','1964-04-01 00:00:00','1964-12-11 00:00:00','WC    CB','Little Carter Bay, Bahama Is. (Same as Walker Cay?)','',-78.42,27.28,.02 from dual union all
select 1786,'WC','','launch site','BS','1964-04-01 00:00:00','1964-12-11 00:00:00','WC    -','Walker Cay, Bahama Islands','',-78.42,27.28,.02 from dual union all
select 1787,'WEN','LC101','launch point','','2016-01-01 00:00:00','','WEN','CZ-5 pad, Wenchang Space Center, Hainan','',110.9511,19.6143,.0001 from dual union all
select 1788,'WEN','LC201','launch point','','2016-01-01 00:00:00','','WEN','CZ-7 pad, Wenchang Space Center, Hainan','',110.9555,19.6185,.0001 from dual union all
select 1789,'WEN','','launch site','CN','2016-01-01 00:00:00','','WEN','Wenchang Space Center, Hainan','',110.9511,19.6142,.0001 from dual union all
select 179,'BALKH','LC A','launch point','','1960-01-01 00:00:00','','SSATC LC A','Launch Complex A, Sary Shagan [US designation]','Priozyorsk (Sary Shagan-2), Kazakhstan',72.87,46.38,.02 from dual union all
select 1790,'WI','5IN','launch point','','','','WI    5IN','5 Inch Naval Gun','',null,null,null from dual union all
select 1791,'WI','7IN','launch point','','','','WI    7IN','7-inch naval gun, WI','',null,null,null from dual union all
select 1792,'WI','AMGD','launch point','','1990-01-01 00:00:00','','WI    AMGD','Advanced Gun, Wallops Island','',-75.5,37.85,.05 from dual union all
select 1793,'WI','LA0','launch point','','1980-01-01 00:00:00','1995-01-01 00:00:00','WI    LA0','Pad 0, Wallops Flight Facility','',-75.4894,37.8316,.0001 from dual union all
select 1794,'WI','LA0 ARC','launch point','','1995-01-01 00:00:00','','WI    LA0 ARC','Launch Area 0, ARC launcher','',-75.4894,37.8316,.001 from dual union all
select 1795,'WI','LA0 HAD','launch point','','1995-01-01 00:00:00','','WI    LA0 HAD','Launch Area 0, HAD launcher','',-75.4894,37.8316,.0001 from dual union all
select 1796,'WI','LA0A','launch point','','1995-01-01 00:00:00','2006-01-01 00:00:00','WI    LA0A','Launch Area 0A, Wallops Flight Facility','',-75.4901,37.8315,.0001 from dual union all
select 1797,'WI','LA1','launch point','','1950-01-01 00:00:00','','WI    LA1','Aerobee Launcher, Launch Area 1, Wallops Flight Facility','',-75.4861,37.8352,.0001 from dual union all
select 1798,'WI','LA1 AML','launch point','','1950-01-01 00:00:00','','WI    LA1 AML','AML 50K Launcher, Launch Area 1, Wallops Flight Facility','',-75.4871,37.8352,.0001 from dual union all
select 1799,'WI','LA1 HAD','launch point','','1950-01-01 00:00:00','','WI    LA4 HAD','HAD launcher, Launch Area 4, Wallops Flight Facility','',-75.4696,37.8511,.0001 from dual union all
select 18,'ALT','6','launch point','','','','ALT   6','Altus AFB Site 6','Atlus, Oklahoma',-99.2743,34.663,.1 from dual union all
select 180,'BALKH','LC B','launch point','','1960-01-01 00:00:00','','SSATC LC B','Launch Complex B, Sary Shagan [US designation]','Priozyorsk (Sary Shagan-2), Kazakhstan',72.6,45.95,.05 from dual union all
select 1800,'WI','LA1 ML','launch point','','1950-01-01 00:00:00','','WI    LA4 ML','Military Launcher, Launch Area 4, Wallops Flight Facility','',-75.4698,37.851,.0001 from dual union all
select 1801,'WI','LA2','launch point','','1950-01-01 00:00:00','','WI    LA2','Launch Area 2, Wallops Flight Facility','',-75.4834,37.8376,.0001 from dual union all
select 1802,'WI','LA2 AML1','launch point','','1950-01-01 00:00:00','','WI    LA2 AML-1','Thiokol AML-1 North, Launch Area 2, Wallops Flight Facility','',-75.4828,37.8381,.0001 from dual union all
select 1803,'WI','LA2 AML2','launch point','','1950-01-01 00:00:00','','WI    LA2 AML-2','Thiokol AML-2 South, Launch Area 2, Wallops Flight Facility','',-75.483,37.8379,.0001 from dual union all
select 1804,'WI','LA2 ARC','launch point','','1950-01-01 00:00:00','','WI    LA2 ARC','ARC Launcher, Launch Area 2, Wallops Flight Facility','',-75.4841,37.838,.0001 from dual union all
select 1805,'WI','LA2 HAD','launch point','','1950-01-01 00:00:00','','WI    LA2 HAD','HAD Launcher, Launch Area 2, Wallops Flight Facility','',-75.4826,37.8383,.0001 from dual union all
select 1806,'WI','LA2 JUP','launch point','','1950-01-01 00:00:00','','WI    LA2 JUP','Jupiter Launcher, Launch Area 2A, Wallops Flight Facility','',-75.482,37.8394,.0001 from dual union all
select 1807,'WI','LA2 MLN','launch point','','1950-01-01 00:00:00','','WI    LA2 MLN','Military Launcher North, Launch Area 2, Wallops Flight Facility','',-75.4834,37.8376,.0001 from dual union all
select 1808,'WI','LA2 MLS','launch point','','1950-01-01 00:00:00','','WI    LA2 MLS','Military Launcher South, Launch Area 2, Wallops Flight Facility','',-75.4835,37.8374,.0001 from dual;
insert into SITE
select 1809,'WI','LA2 MRL','launch point','','1950-01-01 00:00:00','','WI    LA2 MRL','MRL Launcher, Launch Area 2, Wallops Flight Facility','',-75.4834,37.8376,.002 from dual union all
select 181,'BALKH','LC35','launch point','','1960-01-01 00:00:00','','Balkash LC35','Launch Complex 35, Sary Shagan','Priozyorsk (Sary Shagan-2), Kazakhstan',72.87,46.38,.1 from dual union all
select 1810,'WI','LA2 RAG','launch point','','1950-01-01 00:00:00','','WI    LA2 RAG','RAG Launcher, Launch Area 2A, Wallops Flight Facility','',-75.4824,37.8386,.0001 from dual union all
select 1811,'WI','LA2A','launch point','','1950-01-01 00:00:00','','WI    LA2','Launch Area 2A, Wallops Flight Facility','',-75.4834,37.8376,.0001 from dual union all
select 1812,'WI','LA3','launch point','','1960-01-01 00:00:00','1970-01-01 00:00:00','WI    LA3','Mk I Launcher, Launch Area 3, Wallops Flight Facility','',-75.4725,37.8503,.001 from dual union all
select 1813,'WI','LA3 HAD','launch point','','1970-01-01 00:00:00','','WI    LA3 HAD','HAD Launcher, Launch Area 3, Wallops Flight Facility','',-75.4725,37.8494,.0001 from dual union all
select 1814,'WI','LA3A','launch point','','1964-01-01 00:00:00','1995-01-01 00:00:00','WI    LA3A','Mk II Launcher, Launch Area 3, Wallops Flight Facility','',-75.4736,37.848,.0001 from dual union all
select 1815,'WI','LA3B','launch point','','1970-01-01 00:00:00','','WI    LA3B','20K AML launcher, Pad 3B, Launch Area 3, Wallops Flight Facility','',-75.4725,37.8494,.0001 from dual union all
select 1816,'WI','LA4','launch point','','1950-01-01 00:00:00','','WI    LA4','Launch Area 4, Wallops Flight Facility','',-75.4702,37.8508,.0001 from dual union all
select 1817,'WI','LA4 MAST','launch point','','1950-01-01 00:00:00','','WI    LA4 MAST','MAST launcher, Launch Area 4, Wallops Flight Facility','',-75.4702,37.8508,.0001 from dual union all
select 1818,'WI','LA5','launch point','','1950-01-01 00:00:00','','WI    LA5','Launch Area 5, Wallops Flight Facility','',-75.4681,37.8529,.0001 from dual union all
select 1819,'WI','RW04/22','launch point','','1958-01-01 00:00:00','','WI    RW04/22','RW04/22, WFF Research Airport, Wallops Main Base, Virginia','',-75.4633,37.9383,.0001 from dual union all
select 182,'BALKH','LC6','launch point','','1960-01-01 00:00:00','','Balkash LC6','Launch Complex 6, Sary Shagan','Priozyorsk (Sary Shagan-2), Kazakhstan',72.87,46.38,.02 from dual union all
select 1820,'WI','RW22','launch point','','1958-01-01 00:00:00','','WI    RW04/22','RW04/22, WFF Research Airport, Wallops Main Base, Virginia','',-75.4633,37.9383,.0001 from dual union all
select 1821,'WI','','launch site','US','1946-01-01 00:00:00','','WI    -','Wallops Flight Facility, Wallops Island, Virginia','',-75.48,37.83,.02 from dual union all
select 1822,'WILA','','launch site','US','1977-01-01 00:00:00','','WILA  -','Wallops Island Launch Area, 8 km offshore','',-75.27,37.45,.01 from dual union all
select 1823,'WLK','579-1','launch point','','','','WLK   579-1','Silo 579-1, Walker AFB','',null,null,null from dual union all
select 1824,'WLK','579-2','launch point','','','','WLK   579-2','Silo 579-2, Walker AFB','',null,null,null from dual union all
select 1825,'WLK','579-5','launch point','','','','WLK   579-5','Silo 579-5, Walker AFB','',null,null,null from dual union all
select 1826,'WLK','','launch site','US','','','WLK   -','Walker AFB, Roswell, New Mexico','Roswell, New Mexico',-104.5305,33.3017,.001 from dual union all
select 1827,'WONSAN','LP1','launch point','','','','WONSAN LP1','Kalma Airport, Wonsan/Kittaeryong (Gitdaeryung), Kangwon-do','',127.495,39.157,.001 from dual union all
select 1828,'WONSAN','LP2','launch point','','','','WONSAN LP2','Masikryon Ski Resort, Wonsan/Kittaeryong (Gitdaeryung), Kangwon-do','',127.2503,39.066,.001 from dual union all
select 1829,'WONSAN','LP3','launch point','','','','WONSAN LP3','North Wonsan launch aera, Wonsan, Kangwon-do','',127.5369,39.4017,.001 from dual union all
select 183,'BALKH','','launch site','SU','1960-01-01 00:00:00','','GNIIP-10 SSATC','Sary Shagan Antimissile Test Center (NIIP-A, GNIIP-10), Balkash','Priozyorsk (Sary Shagan-2), Kazakhstan',72.87,46.38,.02 from dual union all
select 1830,'WONSAN','','launch site','KP','2006-01-01 00:00:00','','KITT  -','Kalma Airport, Wonsan/Kittaeryong (Gitdaeryung), Kangwon-do','',127.495,39.157,.001 from dual union all
select 1831,'WOO','LA1','launch point','','1948-01-01 00:00:00','','WOO   LA1','LA1, Woomera Range, South Australia','',136.5,-30.925,.1 from dual union all
select 1832,'WOO','LA2','launch point','','1948-01-01 00:00:00','','WOO   LA2','LA2, Woomera Range, South Australia','',136.5,-30.925,.002 from dual union all
select 1833,'WOO','LA2 AH','launch point','','1957-01-01 00:00:00','','WOO   LA2 AH','Aero High launcher, Woomera','',136.5205,-30.9431,.0001 from dual union all
select 1834,'WOO','LA2 D','launch point','','1957-01-01 00:00:00','','WOO   LA2 D','Moraba mobile launcher, LA2, Woomera Range, South Australia','',136.5,-30.925,.002 from dual union all
select 1835,'WOO','LA2 HAD','launch point','','1957-01-01 00:00:00','','WOO   HAD','HAD launcher, Woomera','',136.5205,-30.9431,.0001 from dual union all
select 1836,'WOO','LA2 HRV','launch point','','1957-01-01 00:00:00','1980-01-01 00:00:00','WOO   LA2 HRV','Hypersonic launcher, LA2, Woomera Range, South Australia','',136.5,-30.925,.002 from dual union all
select 1837,'WOO','LA2 LT','launch point','','1957-01-01 00:00:00','','WOO   LA2 LT','Long Tom launcher, LA2, Woomera Range, South Australia','',136.5,-30.925,.002 from dual union all
select 1838,'WOO','LA2 N','launch point','','1957-01-01 00:00:00','','WOO   LA2 N','Nike Launcher, LA2, Woomera Range, South Australia','',136.5,-30.925,.002 from dual union all
select 1839,'WOO','LA2 SL','launch point','','1957-01-01 00:00:00','','WOO   LA2 SL','Skylark launcher, LA2, Woomera Range, South Australia','',136.5,-30.925,.002 from dual union all
select 184,'BARB','5IN','launch point','','1963-05-18 00:00:00','1968-01-01 00:00:00','BARB  5IN','HARP 5-in Gun, Seawell, Barbados','',-59.483,13.067,.04 from dual union all
select 1840,'WOO','LA3','launch point','','1957-01-01 00:00:00','1970-01-01 00:00:00','WOO   LA3','Launch Area 3, Woomera Range, South Australia','',136.5022,-30.9201,.002 from dual union all
select 1841,'WOO','LA4','launch point','','1957-01-01 00:00:00','1970-01-01 00:00:00','WOO   LA4','Launch Area 4, Woomera Range, South Australia','',136.5034,-30.9152,.002 from dual union all
select 1842,'WOO','LA5','launch point','','1958-01-01 00:00:00','1965-01-01 00:00:00','WOO   LA5','Launch Area 5A or 5B, Woomera Range','',136.4592,-30.95,.005 from dual union all
select 1843,'WOO','LA5A','launch point','','1958-01-01 00:00:00','1965-01-01 00:00:00','WOO   LA5A','Launch Area 5A, Woomera Range, South Australia','',136.4592,-30.95,.005 from dual union all
select 1844,'WOO','LA5B','launch point','','1961-01-01 00:00:00','1971-01-01 00:00:00','WOO   LA5B','Launch Area 5B, Woomera Range, South Australia','',136.4602,-30.95,.005 from dual union all
select 1845,'WOO','LA6A','launch point','','1964-01-01 00:00:00','1970-01-01 00:00:00','WOO   LA6A','Launch Area 6A, Woomera Range, South Australia','',136.431,-31.051,.005 from dual union all
select 1846,'WOO','LA8','launch point','','1967-01-01 00:00:00','1970-01-01 00:00:00','WOO   LA8','Launch Area 8, Woomera Range, South Australia','',136.437,-31.011,.02 from dual union all
select 1847,'WOO','LA9','launch point','','1960-01-01 00:00:00','1970-01-01 00:00:00','WOO   LA9','Launch Area 9, Woomera Range, South Australia','',136.4872,-30.9072,.0002 from dual union all
select 1848,'WOO','MRL','launch point','','1957-01-01 00:00:00','','WOO   MRL','MRL launcher, Woomera','',136.5207,-30.9429,.0001 from dual union all
select 1849,'WOO','','launch site','AU','1948-01-01 00:00:00','','WOO   -','Woomera Instrumented Range, Woomera, South Australia','',136.5,-30.925,.002 from dual union all
select 185,'BARB','HARP16','launch point','','1963-05-18 00:00:00','1968-01-01 00:00:00','BARB  HARP16','HARP 40-cm Gun, Seawell, Barbados','',-59.483,13.067,.04 from dual union all
select 1850,'WS','5IN','launch point','','1977-01-01 00:00:00','','WS    5IN','Mk 39 5-inch naval gun, WSMR','',null,null,null from dual union all
select 1851,'WS','7IN','launch point','','','','WS    7IN','7-inch naval gun, WSMR','',null,null,null from dual union all
select 1852,'WS','ALA1','launch point','','1945-09-01 00:00:00','1960-01-01 00:00:00','WS    ALA1','Army Launch Area 1, WSPG','',-106.3782,32.401,.0001 from dual union all
select 1853,'WS','ALA2','launch point','','1946-01-01 00:00:00','1960-01-01 00:00:00','WS    ALA2','Army Launch Area 2, WSMR','',-106.38,32.4,.1 from dual union all
select 1854,'WS','ALA3','launch point','','1958-01-01 00:00:00','1960-01-01 00:00:00','WS    ALA3','Army Launch Area 3, WSPG','',-106.3215,32.4179,.001 from dual union all
select 1855,'WS','ALA4','launch point','','1946-01-01 00:00:00','1960-01-01 00:00:00','WS    LC32','Launch Complex 32, WSMR','',-106.4067,32.4066,.0001 from dual union all
select 1856,'WS','ALA5','launch point','','1946-01-01 00:00:00','1960-01-01 00:00:00','WS    ALA5','Army Launch Area 5, WSPG','',-106.28,32.41,.05 from dual union all
select 1857,'WS','FIX','launch point','','1990-01-01 00:00:00','','WS    FIX','Launch Complex 94 (FIX), WSMR','',-106.4589,34.2049,.0001 from dual union all
select 1858,'WS','HARP5','launch point','','','','WS    5IN','HARP 5-inch gun, WSMR','',null,null,null from dual union all
select 1859,'WS','LC31','launch point','','1960-01-01 00:00:00','','WS    LC31','Launch Complex 31, WSMR','',-106.4067,32.4066,.0001 from dual union all
select 186,'BARB','','launch site','BB','1963-05-18 00:00:00','1968-01-01 00:00:00','BARB  -','Barbados, Windward Islands','',-59.483,13.067,.04 from dual union all
select 1860,'WS','LC32','launch point','','1960-01-01 00:00:00','','WS    LC32','Launch Complex 32, WSMR','',-106.4067,32.4066,.0001 from dual union all
select 1861,'WS','LC32E','launch point','','2009-01-01 00:00:00','','WS    LC32E','Launch Complex 32-East, WSMR','',-106.4057,32.4066,.0002 from dual union all
select 1862,'WS','LC32W','launch point','','2009-01-01 00:00:00','','WS    LC32W','Launch Complex 32-West, WSMR','',-106.4057,32.4066,.0002 from dual union all
select 1863,'WS','LC33','launch point','','1960-01-01 00:00:00','','WS    LC33','Launch Complex 33, WSMR','',-106.3782,32.401,.0001 from dual union all
select 1864,'WS','LC33 L-314','launch point','','1960-01-01 00:00:00','','WS    LC33','Launcher 314, Launch Complex 33, WSMR','',-106.3782,32.401,.0001 from dual union all
select 1865,'WS','LC33 L-361','launch point','','1960-01-01 00:00:00','','WS    LC33','Launcher 361 Launch Complex 33, WSMR','',-106.3782,32.401,.0001 from dual union all
select 1866,'WS','LC33 L-361L','launch point','','1960-01-01 00:00:00','','WS    LC33','Launcher 361 Left Boom, Launch Complex 33, WSMR','',-106.3782,32.401,.0001 from dual union all
select 1867,'WS','LC33 L-361R','launch point','','1960-01-01 00:00:00','','WS    LC33','Launcher 361 Right Boom, Launch Complex 33, WSMR','',-106.3782,32.401,.0001 from dual union all
select 1868,'WS','LC34','launch point','','1946-01-01 00:00:00','','WS    LC34','Launch Complex 34, WSMR','',-106.3658,32.4039,.001 from dual union all
select 1869,'WS','LC35','launch point','','1960-01-01 00:00:00','','WS    LC35','Launch Complex 35, WSMR','',-106.343,32.404,.0001 from dual union all
select 187,'BARK','','launch site','US','','','BARK  -','Barksdale AFB','Louisiana',-93.73,32.52,.02 from dual union all
select 1870,'WS','LC35 L-21A','launch point','','1960-01-01 00:00:00','','WS    LC35 L-21A','Aerobee Tower L-21A, Launch Complex 35, WSMR','',-106.3425,32.4043,.0001 from dual union all
select 1871,'WS','LC35 L-21B','launch point','','1960-01-01 00:00:00','','WS    LC35 L-21B','Aerobee Tower L-21B, Launch Complex 35, WSMR','',-106.3419,32.4043,.0001 from dual union all
select 1872,'WS','LC35 L-457','launch point','','1960-01-01 00:00:00','','WS    LC35 L-457','Navy Launcher, Launch Complex 35?, WSMR','',-106.3297,32.4133,.0001 from dual union all
select 1873,'WS','LC36','launch point','','1960-01-01 00:00:00','','WS    LC36','Launch Complex 36, WSMR','',-106.322,32.417,.005 from dual union all
select 1874,'WS','LC36 L-455','launch point','','1960-01-01 00:00:00','','WS    LC36 L-455','Aerobee Launcher (Trainable Tower) 455, Launch Complex 36, WSMR','',-106.3215,32.4179,.0001 from dual union all
select 1875,'WS','LC36 L-462','launch point','','1960-01-01 00:00:00','','WS    LC36 L-462','Launcher 462, Launch Complex 36, WSMR','',-106.3199,32.4181,.0001 from dual union all
select 1876,'WS','LC36 L-479','launch point','','1960-01-01 00:00:00','','WS    LC36 L-479','Launcher 479, Launch Complex 36, WSMR','',-106.3258,32.4169,.0001 from dual union all
select 1877,'WS','LC36 L-536','launch point','','1960-01-01 00:00:00','','WS    LC36 L-536','BB Launcher 536 (Aries), Launch Complex 36, WSMR','',-106.3256,32.4168,.0001 from dual union all
select 1878,'WS','LC36 L-580','launch point','','1960-01-01 00:00:00','','WS    LC36 L-580','Launcher 580, Launch Complex 36, WSMR','',-106.32,32.4181,.0001 from dual union all
select 1879,'WS','LC36 L-630','launch point','','1960-01-01 00:00:00','','WS    LC36 L-630','MRL Launcher 630, Launch Complex 36, WSMR','',-106.321,32.4178,.0001 from dual union all
select 188,'BART','','launch site','US','1970-03-02 00:00:00','1970-03-05 00:00:00','BART  -','Barter Island, Alaska','',-143.63,70.11,.02 from dual union all
select 1880,'WS','LC36 L-702','launch point','','1960-01-01 00:00:00','','WS    LC36 L-702','Aries 54K launcher, Launch Complex 36, WSMR','',-106.32,32.418,.01 from dual union all
select 1881,'WS','LC36 L-738','launch point','','1960-01-01 00:00:00','','WS    LC36 L-738','BB Athena Launcher 738, Launch Complex 36, WSMR','',-106.32,32.418,.0001 from dual union all
select 1882,'WS','LC36 L-763','launch point','','1960-01-01 00:00:00','','WS    LC36 L-763','Starbird 42ft 50K launcher, Launch Complex 36, WSMR','',-106.32,32.418,.01 from dual union all
select 1883,'WS','LC37','launch point','','1960-01-01 00:00:00','','WS    LC37','Launch Complex 37, WSMR','',-106.3097,32.4112,.001 from dual union all
select 1884,'WS','LC38','launch point','','1960-01-01 00:00:00','','WS    LC38','Launch Complex 38, WSMR  = ALA5','',-106.277,32.4162,.0001 from dual union all
select 1885,'WS','LC38 Pit 207','launch point','','1960-01-01 00:00:00','','WS    LC38','Launch Complex 38, WSMR','',-106.277,32.4162,.0001 from dual union all
select 1886,'WS','LC38S','launch point','','1960-01-01 00:00:00','','WS    LC38 S','Old pad south of Launch Complex 38, WSMR  = ALA5?','',-106.2791,32.405,.0001 from dual union all
select 1887,'WS','LC39','launch point','','1960-01-01 00:00:00','','WS    LC39','Launch Complex 39, WSMR','',-106.2365,32.4161,.0001 from dual union all
select 1888,'WS','LC50','launch point','','1960-01-01 00:00:00','','WS    LC50','Launch Complex 50, WSMR','',-106.3236,32.4722,.0001 from dual union all
select 1889,'WS','NLA','launch point','','1946-01-01 00:00:00','1960-01-01 00:00:00','WS    NLA','Navy Launch Area, WSPG','',-106.343,32.404,.0001 from dual union all
select 189,'BCB','','launch site','US','2016-01-01 00:00:00','','Boca Chica','Texas Launch Site, Boca Chica Beach, Texas','',-97.1536,25.9964,.001 from dual union all
select 1890,'WS','RC','launch point','','1956-01-01 00:00:00','','WS    RCRC','Red Canyon Range Camp','',-106.1658,33.6828,.0005 from dual union all
select 1891,'WS','SEUS?','launch point','','1960-01-01 00:00:00','','WS    LC39?','Launch Complex 39, WSMR (or SUES complex?)','',-106.2558,32.4077,.0001 from dual union all
select 1892,'WS','SULF','launch point','','1960-01-01 00:00:00','','WS    SULF','Speedball Uprange Launch Facility, WSMR','',-106.7364,33.7211,.0001 from dual union all
select 1893,'WS','WSSH','launch point','','1981-01-01 00:00:00','','WS    WSSH','White Sands Space Harbor, New Mexico','',-106.414,32.946,.002 from dual union all
select 1894,'WS','ZURF','launch point','','1960-01-01 00:00:00','','WS    ZURF','Zeus Uprange Facility, Stallion Site, WSMR','White Sands, New Mexico',-106.595,33.7581,.0005 from dual union all
select 1895,'WS','','launch site','US','1946-01-01 00:00:00','','WS    -','White Sands Missile Range, New Mexico','',-106.53,32.4,.1 from dual union all
select 1896,'WTEX','','launch site','US','2015-01-01 00:00:00','','WTEX','Blue Origin, West Texas Launch Site','',-104.7572,31.4228,.0002 from dual union all
select 1897,'WTR','','launch zone','US','1974-01-01 00:00:00','','WTR   -','SLBM Launch Area, Western Test Range, Pacific Ocean','',-120,35,3 from dual union all
select 1898,'WTR-A','','launch zone','US','1974-01-01 00:00:00','','WTR   -','SLBM Launch Area, Western Test Range, Pacific Ocean','',-117.75,32.25,.2 from dual union all
select 1899,'WTR-B','','launch zone','US','1974-01-01 00:00:00','','WTR   -','SLBM Launch Area, Western Test Range, Pacific Ocean','',-123.2,34.75,.2 from dual;
insert into SITE
select 19,'ALT','','launch site','US','','','ALT   -','Altus AFB','Atlus, Oklahoma',-99.2743,34.663,.1 from dual union all
select 190,'BELA','','launch zone','SU','1960-01-01 00:00:00','','BELA  -','White Sea Launch Area, Beloye More, Archangel''sk Oblast''','',38,65.5,1 from dual union all
select 1900,'WTR-JI','','launch zone','US','1964-01-01 00:00:00','','WTR   -','Johnston Island SLBM Launch Area, Western Test Range, Pacific Ocean','',-120,35,3 from dual union all
select 1901,'WURT','','launch site','US','','','WURT  -','Wurtsmith AFB','Oscoda, Michigan',-83.3803,44.4525,.002 from dual union all
select 1902,'XSC','LC','launch point','','1984-01-01 00:00:00','','XSC   LC','Launch Complex, Xichang Space Center (later LC3)','',102.0291,28.2472,.0001 from dual union all
select 1903,'XSC','LC2','launch point','','1990-01-01 00:00:00','','XSC   LC2','Launch Complex 2, Xichang Space Center','',102.0271,28.2455,.0001 from dual union all
select 1904,'XSC','LC3','launch point','','1984-01-01 00:00:00','','XSC   LC3','Launch Complex 3, Xichang Space Center','',102.0291,28.2472,.0001 from dual union all
select 1905,'XSC','','launch site','CN','1984-01-01 00:00:00','','XSC   -','Xichang Space Center (Songlin), Sichuan, China','',102.0291,28.2472,.0001 from dual union all
select 1906,'YAS','Sh370/11','launch point','RU','2004-01-01 00:00:00','','YAS   -','Shast'' (silo) 370/11, Yasniy site','',59.6975,51.0655,.0002 from dual union all
select 1907,'YAS','Sh370/13','launch point','RU','2004-01-01 00:00:00','','YAS   -','Shast'' (silo) 370/13, Yasniy site','',59.5503,50.9729,.0002 from dual union all
select 1908,'YAS','','launch site','RU','2004-01-01 00:00:00','2004-01-01 00:00:00','YAS   -','Yasniy site, Dombarovskiy Air Base, Orensburgskaya Oblast'', Rossiya','',59.5,50.75,.25 from dual union all
select 1909,'YEL','','launch zone','CN','','','YEL   -','Yellow Sea launch area','',123,35,3 from dual union all
select 191,'BER1','LP1','launch point','','1950-01-01 00:00:00','1950-01-01 00:00:00','BER1  LP1','USS Norton Sound, 1950 Bering Sea Launch Site 1 (APL Aerobee/Norton Sound)','',-170,58,9 from dual union all
select 1910,'YEMEN','','launch site','YE','2015-01-01 00:00:00','','Yemen','Various locations in Yemen','',43.7307,16.9691,4 from dual union all
select 1911,'YSNYA','','launch site','RU','','','YASNAYA','Yasnaya missile base, Chita Oblast','',115.9,50.8,.2 from dual union all
select 1912,'YUK','','launch site','US','1984-02-07 00:00:00','1984-02-07 00:00:00','YUK   -','Fort Yukon, Alaska','',-145.1972,66.5623,.0001 from dual union all
select 1913,'YUMA','5IN','launch point','','1966-11-17 00:00:00','1966-11-20 00:00:00','YUMA  5IN','5-inch HARP Gun, Yuma Proving Ground, Arizona','',-114.32,32.87,.02 from dual union all
select 1914,'YUMA','HARP16','launch point','','1966-11-17 00:00:00','1966-11-20 00:00:00','YUMA  HARP16','16-inch HARP Gun, Yuma Proving Ground, Arizona','',-114.32,32.87,.02 from dual union all
select 1915,'YUMA','','launch site','US','1966-11-17 00:00:00','1966-11-20 00:00:00','YUMA  -','Yuma Proving Ground, Arizona','',-114.32,32.87,.02 from dual union all
select 1916,'YURYA','','launch site','SU','1960-01-01 00:00:00','1990-01-01 00:00:00','YURYA','Yurya missile site, Russia','',49.44,59.17,.1 from dual union all
select 1917,'ZINGST','','launch site','DD','1988-10-21 00:00:00','1992-04-01 00:00:00','ZINGS -','Zingst Range, MMR-06 pad, DDR','',12.7,54.2,.1 from dual union all
select 192,'BER1','LP2','launch point','','1950-01-01 00:00:00','1950-01-01 00:00:00','BER1  LP2','USS Norton Sound, 1950 Bering Sea Launch Site 2 (APL Aerobee/Norton Sound)','',-170,50,9 from dual union all
select 193,'BER1','','launch cruise','US','1950-01-01 00:00:00','1950-01-01 00:00:00','BER1','USS Norton Sound, 1950 Bering Sea expedition','',-170,58,9 from dual union all
select 194,'BERM','','launch site','BM','1962-01-01 00:00:00','1971-01-01 00:00:00','BERM  -','NASA Bermuda, Kindley AFB, Coopers Island, Bermuda','',-64.6566,32.3483,.001 from dual union all
select 195,'BERSH','','launch site','SU','1960-01-01 00:00:00','1990-01-01 00:00:00','BERSH','Bershet missile base, Russia','',56.306,57.692,.001 from dual union all
select 196,'BFJL','','launch zone','SU','2009-01-01 00:00:00','','BFJL','Franz Josef Land Launch Area, Barentsovo More','',58,80.5,1 from dual union all
select 197,'BFL','','launch site','NZ','1963-01-01 00:00:00','1963-01-01 00:00:00','BFL   -','Birdling''s Flat, Canterbury Plains, New Zealand','',172.68,-43.8167,.01 from dual union all
select 198,'BHOT','','launch site','CN','1990-01-01 00:00:00','','BHOT','Bayan Hot test site','Alxa, Nei Monggol',105.7,38.8,.5 from dual union all
select 199,'BIK','','launch site','MH','1956-01-01 00:00:00','1956-01-01 00:00:00','BIK   -','Pacific Proving Grounds, Bikini Island, Pacific Ocean','',165.33,11.58,.04 from dual union all
select 2,'ABER','','launch site','UK','1959-06-29 00:00:00','1972-07-20 00:00:00','ABER  -','Aberporth Range','Aberporth, Cardigan, Wales',-4.5548,52.1379,.0001 from dual union all
select 20,'AMAR','','launch site','IQ','1985-01-01 00:00:00','1990-01-01 00:00:00','AMAR  -','Al ''Amarah missile launch site','Al ''Amarah, Iraq',47.15,31.83,.1 from dual union all
select 200,'BLA','LP1','launch point','','1998-07-07 00:00:00','','BLA   LP1','K-407 submarine, Barents Launch Area, Barentsovo More, Murmansk Oblast''','',34.2,69.5,.1 from dual union all
select 201,'BLA','','launch zone','SU','1960-01-01 00:00:00','','BLA   -','Barents Launch Area, Barentsovo More, Murmansk Oblast''  (Os. Kildin)','',34.2,69.5,.5 from dual union all
select 202,'BLAN','BM','launch point','','1967-01-01 00:00:00','1980-01-01 00:00:00','BLAN  BM','Black Mesa, Blanding, Utah','',-109.5883,37.5038,.02 from dual union all
select 203,'BLAN','WM','launch point','','1967-01-01 00:00:00','1980-01-01 00:00:00','BLAN  WM','White Mesa, Blanding, Utah','',-109.5165,37.538,.02 from dual union all
select 204,'BLAN','','launch site','US','1967-01-01 00:00:00','1980-01-01 00:00:00','BLAN  -','Black Mesa, Blanding, Utah','',-109.29,37.37,.02 from dual union all
select 205,'BOHAI','','launch zone','CN','','','Bohai -','Bohai launch area  (Some Yellow Sea launches may also be here)','',120,38.7,320000 from dual union all
select 206,'BRET','','launch site','F','1983-01-01 00:00:00','','BRET  -','Zone de lancement SNLE, Golfe de Gascogne, Bretagne','',-4.5,47.5,.7 from dual union all
select 207,'BRETA','','launch site','F','2009-01-01 00:00:00','','BRETA','Zone de lancement SNLE, Baie d''Audierne','Baie D''Audierne, Bretagne',-4.05,47.63,.05 from dual union all
select 208,'BROCK','','launch site','US','2004-01-01 00:00:00','','BROCK -','Black Rock Desert, Nevada','',-119.14,40.83,.05 from dual union all
select 209,'BUGA80A','','launch cruise','SU','1980-10-01 00:00:00','1980-10-01 00:00:00','Bugaev 80','Bugaev cruise 1980','',null,null,null from dual union all
select 21,'AMR','','launch site','US','1958-01-01 00:00:00','1962-01-01 00:00:00','AMR   -','Atlantic Missile Range Drop Zone, N Atlantic Ocean','Atlantic Ocean',-79,29,1 from dual union all
select 210,'BUGA81A','','launch cruise','SU','1981-06-01 00:00:00','1981-06-01 00:00:00','Bugaev 81','Bugaev cruise 1981','',null,null,null from dual union all
select 211,'BUGA81B','','launch cruise','SU','1981-10-01 00:00:00','1981-11-01 00:00:00','Bugaev 81','Bugaev cruise 1981','',null,null,null from dual union all
select 212,'BUGA82A','','launch cruise','SU','1982-12-01 00:00:00','1982-12-01 00:00:00','Bugaev 82','Bugaev cruise 1982','',null,null,null from dual union all
select 213,'BUGA83A','','launch cruise','SU','1983-03-01 00:00:00','1983-03-01 00:00:00','Bugaev 83','Bugaev cruise 1983','',null,null,null from dual union all
select 214,'BUGA87A','','launch cruise','SU','1987-01-01 00:00:00','1987-01-01 00:00:00','Bugaev 87','Bugaev cruise 1987','',null,null,null from dual union all
select 215,'BUGA87B','','launch cruise','SU','1987-04-01 00:00:00','1987-04-01 00:00:00','Bugaev 87','Bugaev cruise 1987','',null,null,null from dual union all
select 216,'BUGA89A','','launch cruise','SU','1989-01-01 00:00:00','1989-01-01 00:00:00','Bugaev 89','Bugaev cruise 1989','',null,null,null from dual union all
select 217,'BUGA90A','','launch cruise','SU','1990-12-01 00:00:00','1990-12-01 00:00:00','Bugaev 90','Bugaev cruise 1990','',null,null,null from dual union all
select 218,'CARD','','launch site','F','1952-01-01 00:00:00','1952-01-01 00:00:00','CARD  -','Le Cardonnet, Montpelier, France','',3.64,43.58,.05 from dual union all
select 219,'CARN','','launch site','AU','1964-07-16 00:00:00','1964-10-01 00:00:00','CARN  -','Carnarvon, Western Australia','',113.4,-24.5,.02 from dual union all
select 22,'ANBAR','','launch site','IQ','1989-12-05 00:00:00','','ANBAR -','Al-Anbar base, Al-Anbar province, Iraq','Al-Anbar, Iraq',43.56,33.37,.5 from dual union all
select 220,'CARN2','','launch site','AU','1965-05-01 00:00:00','1965-05-11 00:00:00','CARN2 -','Carnarvon north site, Western Australia','',113.25,-24.1,.2 from dual union all
select 221,'CASS','','launch site','BR','1966-11-05 00:00:00','1966-11-12 00:00:00','CASS  -','Cassino, Rio Grande Do Sul, Brasil','',-52.1689,-32.1927,.0001 from dual union all
select 222,'CC','LA','launch point','','1950-01-01 00:00:00','1960-01-01 00:00:00','CC    LA','Launch Area, Cape Canaveral','',-80.537,28.465,.005 from dual union all
select 223,'CC','LC1','launch point','','1957-01-01 00:00:00','1960-01-01 00:00:00','CC    LC1','Launch Complex 1, Cape Canaveral','',-80.5372,28.4652,.0004 from dual union all
select 224,'CC','LC1/2','launch point','','1957-01-01 00:00:00','1960-01-01 00:00:00','CC    LC1/2?','Launch Complex 1 or 2, Cape Canaveral','',-80.5372,28.4652,.0004 from dual union all
select 225,'CC','LC10','launch point','','1957-03-22 00:00:00','1959-04-27 00:00:00','CC    LC10','Launch Complex 10, Cape Canaveral','',-80.5565,28.4501,.001 from dual union all
select 226,'CC','LC11','launch point','','1958-07-19 00:00:00','1964-04-01 00:00:00','CC    LC11','Launch Complex 11, Cape Canaveral','',-80.5397,28.4751,.0002 from dual union all
select 227,'CC','LC12','launch point','','1958-01-10 00:00:00','1967-11-05 00:00:00','CC    LC12','Launch Complex 12, Cape Canaveral','',-80.5421,28.4802,.0002 from dual union all
select 228,'CC','LC13','launch point','','1958-08-02 00:00:00','1978-04-07 00:00:00','CC    LC13','Launch Complex 13, Cape Canaveral','',-80.5446,28.4859,.0002 from dual union all
select 229,'CC','LC14','launch point','','1957-06-11 00:00:00','1966-11-11 00:00:00','CC    LC14','Launch Complex 14, Cape Canaveral','',-80.547,28.49,.0002 from dual union all
select 23,'AND','Athena','launch point','','1993-01-01 00:00:00','','AND   Athena','Athena launcher, Pad 5, Andoya','Andoya',16.0207,69.2943,.05 from dual union all
select 230,'CC','LC15','launch point','','1959-02-06 00:00:00','1964-04-09 00:00:00','CC    LC15','Launch Complex 15, Cape Canaveral','',-80.5496,28.4923,.0002 from dual union all
select 231,'CC','LC16','launch point','','1959-12-12 00:00:00','1988-03-21 00:00:00','CC    LC16','Launch Complex 16, Cape Canaveral','',-80.552,28.5015,.0002 from dual union all
select 232,'CC','LC17','launch point','','1957-08-30 00:00:00','','CC    LC17','Launch Complex 17A/17B, Cape Canaveral','',-80.5649,28.4472,.0001 from dual union all
select 233,'CC','LC17A','launch point','','1957-08-30 00:00:00','1998-10-01 00:00:00','CC    LC17A','Space Launch Complex 17A, Cape Canaveral','',-80.5649,28.4472,.0001 from dual union all
select 234,'CC','LC17B','launch point','','1957-01-25 00:00:00','1998-10-01 00:00:00','CC    LC17B','Space Launch Complex 17B, Cape Canaveral','',-80.5656,28.4458,.0001 from dual union all
select 235,'CC','LC18A','launch point','','1956-12-08 00:00:00','1965-06-09 00:00:00','CC    LC18A','Launch Complex 18A, Cape Canaveral','',-80.5625,28.4501,.0002 from dual union all
select 236,'CC','LC18B','launch point','','1958-06-04 00:00:00','1962-04-12 00:00:00','CC    LC18B','Launch Complex 18B, Cape Canaveral','',-80.562,28.4485,.0002 from dual union all
select 237,'CC','LC19','launch point','','1959-08-14 00:00:00','1966-11-11 00:00:00','CC    LC19','Launch Complex 19, Cape Canaveral','',-80.5546,28.5067,.0002 from dual union all
select 238,'CC','LC2','launch point','','1954-01-01 00:00:00','1960-01-01 00:00:00','CC    LC2','Launch Complex 2, Cape Canaveral','',-80.5368,28.4655,.0004 from dual union all
select 239,'CC','LC20','launch point','','1960-07-01 00:00:00','','CC    LC20','Launch Complex 20, Cape Canaveral','',-80.557,28.5119,.0002 from dual union all
select 24,'AND','Haugnes','launch point','','1980-01-01 00:00:00','1983-01-01 00:00:00','AND   Haugnes','Haugnes site, Andoy, Nordland','Haugenes, Andoy',16.18,69.28,.02 from dual union all
select 240,'CC','LC21','launch point','','1958-07-24 00:00:00','1963-07-17 00:00:00','CC    LC21','Launch Complex 21, Cape Canaveral','',-80.5416,28.4616,.0002 from dual union all
select 241,'CC','LC21/1','launch point','','1958-07-24 00:00:00','1963-07-17 00:00:00','CC    LC21/1','Launch Complex 21, Cape Canaveral (Launcher 1)','',-80.5416,28.4616,.0002 from dual union all
select 242,'CC','LC21/2','launch point','','1958-07-24 00:00:00','1963-07-17 00:00:00','CC    LC21/2','Launch Complex 21, Cape Canaveral (Launcher 2)','',-80.5416,28.4616,.0002 from dual union all
select 243,'CC','LC22','launch point','','1957-03-13 00:00:00','1960-06-24 00:00:00','CC    LC22','Launch Complex 22, Cape Canaveral','',-80.5411,28.4614,.0002 from dual union all
select 244,'CC','LC25A','launch point','','1958-04-18 00:00:00','1965-03-05 00:00:00','CC    LC25A','Launch Complex 25A, Cape Canaveral','',-80.574,28.4315,.0002 from dual union all
select 245,'CC','LC25B','launch point','','1959-08-14 00:00:00','1960-08-02 00:00:00','CC    LC25B','Launch Complex 25B, Cape Canaveral','',-80.5749,28.4297,.0002 from dual union all
select 246,'CC','LC25C','launch point','','1968-08-16 00:00:00','1979-01-23 00:00:00','CC    LC25C','Launch Complex 25C, Cape Canaveral','',-80.5766,28.431,.0002 from dual union all
select 247,'CC','LC25D','launch point','','1969-09-17 00:00:00','1969-09-17 00:00:00','CC    LC25D','Launch Complex 25D, Cape Canaveral','',-80.5782,28.4298,.0002 from dual union all
select 248,'CC','LC26A','launch point','','1957-08-28 00:00:00','1963-01-22 00:00:00','CC    LC26A','Launch Complex 26A, Cape Canaveral','',-80.5704,28.444,.0002 from dual union all
select 249,'CC','LC26B','launch point','','1957-10-23 00:00:00','1961-05-24 00:00:00','CC    LC26B','Launch Complex 26B, Cape Canaveral','',-80.5714,28.4428,.0002 from dual union all
select 25,'AND','LC10','launch point','','1993-01-01 00:00:00','','AND   LC10','Pad 10, Andoya','Andoya',16.0185,69.294,.0001 from dual union all
select 250,'CC','LC29','launch point','','1959-09-21 00:00:00','1980-05-19 00:00:00','CC    LC29','Launch Complex 29, Cape Canaveral','',-80.5766,28.4291,.0002 from dual union all
select 251,'CC','LC29A','launch point','','1959-09-21 00:00:00','1980-05-19 00:00:00','CC    LC29A','Launch Complex 29A, Cape Canaveral','',-80.5766,28.4291,.0002 from dual union all
select 252,'CC','LC29B','launch point','','1959-09-21 00:00:00','1980-05-19 00:00:00','CC    LC29B','Launch Complex 29B, Cape Canaveral','',-80.5766,28.4291,.0002 from dual union all
select 253,'CC','LC3','launch point','','1950-01-01 00:00:00','1958-01-01 00:00:00','CC    LC3','Launch Complex 3, Cape Canaveral','',-80.5362,28.4658,.0004 from dual union all
select 254,'CC','LC3/4','launch point','','1950-01-01 00:00:00','1958-01-01 00:00:00','CC    LC3/4?','Launch Complex 3 or 4, Cape Canaveral','',-80.5362,28.4658,.0004 from dual union all
select 255,'CC','LC30','launch point','','1960-02-25 00:00:00','1963-04-24 00:00:00','CC    LC30','Launch Complex 30 pad A or B, Cape Canaveral','',-80.5792,28.4391,.0002 from dual union all
select 256,'CC','LC30A','launch point','','1960-02-25 00:00:00','1963-04-24 00:00:00','CC    LC30A','Launch Complex 30A, Cape Canaveral','',-80.5792,28.4391,.0002 from dual union all
select 257,'CC','LC30A TEL','launch point','','1960-02-25 00:00:00','1963-04-24 00:00:00','CC    LC30A TEL','Transporter launcher, Launch Complex 30, Cape Canaveral','',-80.5792,28.4391,.0002 from dual union all
select 258,'CC','LC30B','launch point','','1960-02-25 00:00:00','1963-04-24 00:00:00','CC    LC30B','Launch Complex 30B, Cape Canaveral','',-80.58,28.4376,.0002 from dual union all
select 259,'CC','LC30D','launch point','','1960-02-25 00:00:00','1963-04-24 00:00:00','CC    LC30D','Launch Complex 30D, Cape Canaveral','',-80.58,28.4376,.001 from dual union all
select 26,'AND','LC5','launch point','','1993-01-01 00:00:00','','AND   LC5','Pad 5, Andoya','Andoya',16.0198,69.2944,.0001 from dual union all
select 260,'CC','LC30E','launch point','','1960-02-25 00:00:00','1963-04-24 00:00:00','CC    LC30E','Launch Complex 30E, Cape Canaveral','',-80.58,28.4376,.001 from dual union all
select 261,'CC','LC31A','launch point','','1961-02-01 00:00:00','1973-01-01 00:00:00','CC    LC31A','Launch Complex 31A, Cape Canaveral','',-80.5565,28.4514,.0002 from dual union all
select 262,'CC','LC31A TEL','launch point','','1961-02-01 00:00:00','1973-01-01 00:00:00','CC    LC31A Tel','TEL, Launch Complex 31A, Cape Canaveral','',-80.5565,28.4514,.0002 from dual;
insert into SITE
select 263,'CC','LC31A/Truck','launch point','','1961-02-01 00:00:00','1973-01-01 00:00:00','CC    LC31A/Truck','Truck launcher, LC31A, Cape Canaveral','',-80.5565,28.4514,.0002 from dual union all
select 264,'CC','LC31B','launch point','','1961-12-18 00:00:00','1969-09-23 00:00:00','CC    LC31B','Launch Complex 31B, Cape Canaveral','',-80.5552,28.4511,.0002 from dual union all
select 265,'CC','LC32A','launch point','','1961-01-01 00:00:00','1961-08-01 00:00:00','CC    LC32A','Launch Complex 32A, Cape Canaveral','',-80.5557,28.4534,.0002 from dual union all
select 266,'CC','LC32B','launch point','','1961-08-30 00:00:00','1970-12-14 00:00:00','CC    LC32B','Launch Complex 32B, Cape Canaveral','',-80.5546,28.4531,.0002 from dual union all
select 267,'CC','LC34','launch point','','1961-10-27 00:00:00','1968-10-11 00:00:00','CC    LC34','Launch Complex 34, Cape Canaveral','',-80.561,28.5218,.0002 from dual union all
select 268,'CC','LC36A','launch point','','1962-05-08 00:00:00','','CC    LC36A','Space Launch Complex 36A, Cape Canaveral','',-80.5377,28.471,.0002 from dual union all
select 269,'CC','LC36B','launch point','','1965-08-11 00:00:00','','CC    LC36B','Space Launch Complex 36B, Cape Canaveral','',-80.5411,28.4681,.0002 from dual union all
select 27,'AND','LC9','launch point','','1993-01-01 00:00:00','','AND   LC9','Pad 9, Andoya','Andoya',16.0188,69.2941,.0001 from dual union all
select 270,'CC','LC37A','launch point','','1964-01-29 00:00:00','1968-01-22 00:00:00','CC    LC37A','Launch Complex 37A, Cape Canaveral','',-80.5682,28.5334,.0002 from dual union all
select 271,'CC','LC37B','launch point','','1964-01-29 00:00:00','1968-01-22 00:00:00','CC    LC37B','Launch Complex 37B, Cape Canaveral','',-80.565,28.5315,.0002 from dual union all
select 272,'CC','LC4','launch point','','1952-01-01 00:00:00','1960-01-01 00:00:00','CC    LC4','Launch Complex 4, Cape Canaveral','',-80.536,28.4665,.0004 from dual union all
select 273,'CC','LC4/10','launch point','','1950-01-01 00:00:00','1958-01-01 00:00:00','CC    LC4/10','Launch Complex 4 or 10, Cape Canaveral','',-80.5362,28.4658,.0004 from dual union all
select 274,'CC','LC40','launch point','','1965-06-18 00:00:00','','CC    LC40','Space Launch Complex 40, Cape Canaveral','',-80.5772,28.562,.0001 from dual union all
select 275,'CC','LC41','launch point','','1965-12-21 00:00:00','','CC    LC41','Space Launch Complex 41, Cape Canaveral','',-80.5831,28.5833,.0002 from dual union all
select 276,'CC','LC43','launch point','','1979-01-01 00:00:00','','CC LC43','Launch Complex 43, Cape Canaveral','',-80.5,28.5,.5 from dual union all
select 277,'CC','LC43D','launch point','','1979-01-01 00:00:00','','CC LC43D','Launch Complex 43D, Cape Canaveral','',-80.5,28.5,.5 from dual union all
select 278,'CC','LC4A','launch point','','1952-01-01 00:00:00','1960-01-01 00:00:00','CC    LC4A','Launch Complex 4A, Cape Canaveral','',-80.536,28.4665,.0004 from dual union all
select 279,'CC','LC5','launch point','','1956-07-19 00:00:00','1961-07-21 00:00:00','CC    LC5','Launch Complex 5, Cape Canaveral','',-80.5733,28.4394,.0001 from dual union all
select 28,'AND','U3','launch point','','1993-01-01 00:00:00','','AND   U3','Universal Launcher 3, Pad 7, Andoya','Andoya',16.0207,69.2943,.0001 from dual union all
select 280,'CC','LC6','launch point','','1955-04-20 00:00:00','1961-06-26 00:00:00','CC    LC6','Launch Complex 6, Cape Canaveral','',-80.5725,28.44,.0004 from dual union all
select 281,'CC','LC9','launch point','','1956-11-06 00:00:00','1958-11-18 00:00:00','CC    LC9','Launch Complex 9, Cape Canaveral','',-80.5565,28.4528,.001 from dual union all
select 282,'CC','LaC1','launch point','','2015-07-01 00:00:00','','CC    LaC1','Landing Complex 1, Cape Canaveral','',-80.5446,28.4859,.0002 from dual union all
select 283,'CC','PAFB/800','launch point','','','','CC    PAFB/800','Patrick AFB/800, Cape Canaveral, Florida','',-80.6,28.24,.02 from dual union all
select 284,'CC','RW30/12','launch point','','1952-05-01 00:00:00','','CC    RW 30/12','Skid Strip, Cape Canaveral Air Station','',-80.5665,28.4675,.0004 from dual union all
select 285,'CC','SLC17A','launch point','','1998-10-01 00:00:00','','CC    SLC17A','Space Launch Complex 17A, Cape Canaveral','',-80.5649,28.4472,.0001 from dual union all
select 286,'CC','SLC17B','launch point','','1998-10-01 00:00:00','','CC    SLC17B','Space Launch Complex 17B, Cape Canaveral','',-80.5656,28.4458,.0001 from dual union all
select 287,'CC','SLC20','launch point','','1960-07-01 00:00:00','','CC    SLC20','Space Launch Complex 20, Cape Canaveral','',-80.557,28.5119,.0002 from dual union all
select 288,'CC','SLC36A','launch point','','1962-05-08 00:00:00','','CC    SLC36A','Space Launch Complex 36A, Cape Canaveral','',-80.5377,28.471,.0002 from dual union all
select 289,'CC','SLC36B','launch point','','1965-08-11 00:00:00','','CC    SLC36B','Space Launch Complex 36B, Cape Canaveral','',-80.5411,28.4681,.0002 from dual union all
select 29,'AND','','launch site','N','1962-08-18 00:00:00','','AND   -','Andoya Rocket Range, Andoy, Nordland, Norge','Andoya, Nordland',16.0207,69.2943,.002 from dual union all
select 290,'CC','SLC37B','launch point','','2000-01-01 00:00:00','','CC    SLC37B','Space Launch Complex 37B, Cape Canaveral','',-80.565,28.5315,.0002 from dual union all
select 291,'CC','SLC40','launch point','','1965-06-18 00:00:00','','CC    SLC40','Space Launch Complex 40, Cape Canaveral','',-80.5772,28.562,.0001 from dual union all
select 292,'CC','SLC41','launch point','','1965-12-21 00:00:00','','CC    SLC41','Space Launch Complex 41, Cape Canaveral','',-80.5831,28.5833,.0002 from dual union all
select 293,'CC','','launch site','US','1992-01-01 00:00:00','','CC    -','Cape Canaveral Air Station, Florida','',-80.54,28.46,.05 from dual union all
select 294,'CCA','LC46','launch point','','1987-01-15 00:00:00','','CC LC46','Launch Complex 46, Cape Canaveral','',-80.5272,28.4583,.0002 from dual union all
select 295,'CCA','LC47','launch point','','2006-01-01 00:00:00','','CC LC47','Launch Complex 47, Cape Canaveral','',-80.53,28.46,.01 from dual union all
select 296,'CCA','','launch site','US','1984-01-01 00:00:00','1998-01-01 00:00:00','CC (A)','Cape Canaveral Air Force Station, Eastern Test Range','Cape Canaveral, Florida',-80.54,28.46,.05 from dual union all
select 297,'CCAFS','','launch site','US','1974-03-01 00:00:00','1992-01-01 00:00:00','CCAFS','Cape Canaveral Air Force Station, Eastern Test Range','Cape Canaveral, Florida',-80.54,28.46,.05 from dual union all
select 298,'CCMTA','','launch site','US','1955-12-16 00:00:00','1964-01-22 00:00:00','CCMTA','Cape Canaveral Missile Test Annex, Air Force Missile Test Center','Cape Canaveral, Florida',-80.54,28.46,.05 from dual union all
select 299,'CEL','BE','launch point','','1966-01-01 00:00:00','','CEL   BE','Base Espace, CEL, Biscarosse, France','',-1.259,44.255,.002 from dual union all
select 3,'ACHINSK','','launch site','SU','1962-01-01 00:00:00','1964-01-01 00:00:00','Achinsk','Achinsk Missile Base','Achinsk, Krasnoyarsk Krai',90.58,56.27,.2 from dual union all
select 30,'ANHU','LP1','launch point','','2005-01-01 00:00:00','','Anhueng LP1','LP1, ADD Anhueng','Anhueng Island 1',126.2375,36.6048,.0001 from dual union all
select 300,'CEL','BESA','launch point','','1964-01-01 00:00:00','','CEL   BESA','Base Experimentale Sol-Air, CEL, Biscarosse, France','',-1.245,44.393,.002 from dual union all
select 301,'CEL','BLB','launch point','','1966-02-01 00:00:00','','CEL   BLB','Base Lancement Balistique, CEL, Biscarosse, France','',-1.25,44.362,.002 from dual union all
select 302,'CEL','BP','launch point','','1969-12-01 00:00:00','','CEL   BP','Base Pluton, CEL, Biscarosse, France','',-1.246,44.372,.002 from dual union all
select 303,'CEL','BS','launch point','','1966-01-01 00:00:00','','CEL   BS','Base Scientifique, CEL, Biscarosse, France','',-1.262,44.282,.002 from dual union all
select 304,'CEL','CE','launch point','','1964-03-11 00:00:00','','CEL   CE','Probable launch from CEL (Site "CE" in Chesser''s BAL list)','',-1.25,44.45,.02 from dual union all
select 305,'CEL','SUD','launch point','','1966-01-01 00:00:00','','CEL   SUD','Pas de tir sud, CEL, Biscarosse, France','',-1.263,44.3,.002 from dual union all
select 306,'CEL','ZLS','launch point','','1968-11-01 00:00:00','','CEL   ZLS','Zone de lancement SNLE, Golfe de Gascogne, off CEL','',-1.7,44.36,.2 from dual union all
select 307,'CEL','','launch site','F','1964-03-11 00:00:00','','CEL   -','Centre d''Essais de Landes, Biscarosse, Landes, France','',-1.25,44.45,.02 from dual union all
select 308,'CELAT','','launch site','AR','1968-01-01 00:00:00','1970-01-01 00:00:00','CELAT -','CELPA Atlantica, Mar Chiquita, Buenos Aires, Argentina','',-57.42,-37.75,.02 from dual union all
select 309,'CELM','','launch site','F','1966-02-01 00:00:00','','CELM  -','Centre d''Essais de Lancement de Missiles  (former CEL)','',-1.25,44.362,.002 from dual union all
select 31,'ANHU','LP2','launch point','','2005-01-01 00:00:00','','Anhueng LP2','Barge launch point, ADD Anhueng','Anhueng Island 2',125.6944,36.6422,.0001 from dual union all
select 310,'CELPA','','launch site','AR','1962-01-01 00:00:00','1973-01-01 00:00:00','CELPA -','Centro de Ensayo y Lanzamiento de Proyectiles Autopropulsados','Chamical, La Rioja, Argentina',-66.317,-30.13,.02 from dual union all
select 311,'CEM','','launch site','F','1968-09-01 00:00:00','','CEM   -','Centre d''Essais de la Mediterranee, Ile du Levant, Var, France','',6.47,43.05,.02 from dual union all
select 312,'CERES','Nemo','launch point','','1966-07-01 00:00:00','','CERES Nemo','Caisson Nemo, CERES/CEM, Ile du Levant, Var, France','',6.47,43.05,.02 from dual union all
select 313,'CERES','','launch site','F','1948-01-01 00:00:00','1968-09-01 00:00:00','CERES -','Centre d''Essais et de Recherches d''Engins Speciaux, Ile du Levant, Var','',6.47,43.05,.02 from dual union all
select 314,'CHAC','','launch site','AR','1966-11-12 00:00:00','1966-11-12 00:00:00','CHAC  -','Las Palmas, Chaco, Argentina','',-58.75,-27.13,.03 from dual union all
select 315,'CHAN','IC3','launch point','','1999-01-01 00:00:00','','CHAN  IC3','IC-3,Balasore Range, Orissa, India','',86.6,21.3,.1 from dual union all
select 316,'CHAN','IC4','launch point','','1999-01-01 00:00:00','2015-01-01 00:00:00','CHAN  IC4','IC-4, Inner Wheeler Island, Balasore Range, Orissa, India','',87.0884,20.7553,.0001 from dual union all
select 317,'CHAN','IC4/TEL','launch point','','1999-01-01 00:00:00','','CHAN  IC4/TEL','Tactical launcher, IC-4, Inner Wheeler Island, Orissa, India','',87.07,20.75,.01 from dual union all
select 318,'CHAN','IC4AK','launch point','','2015-01-01 00:00:00','','CHAN  IC4','IC-4, Abdul Kalam Island, Balasore Range, Orissa, India','',87.0884,20.7553,.0001 from dual union all
select 319,'CHAN','ITR','launch point','','1989-01-01 00:00:00','','CHAN  ITR','Interim Test Range, Balasore Range, Chandipur-On-Sea, Orissa, India','',86.6,21.3,.05 from dual union all
select 32,'ANHU','LP3','launch point','','2005-01-01 00:00:00','','Anhueng LP3','LP3, ADD Anhueng','South of Anhueng',125.9086,35.1355,.0001 from dual union all
select 320,'CHAN','','launch site','IN','1989-01-01 00:00:00','','CHAN  -','Integrated Test Range, Balasore Range, Chandipur-On-Sea, Orissa, India','',86.6,21.3,.05 from dual union all
select 321,'CHEL','','launch site','SU','1957-01-01 00:00:00','1960-01-01 00:00:00','CHEL  -','SP-5 Tactical missile site, Chelkar, Aktyubinsk','',59.61,47.85,.02 from dual union all
select 322,'CHI','PLOB','launch point','','1980-01-01 00:00:00','1990-01-01 00:00:00','CHI   PLOB','Punta Lobos Range, Peru (same as Chilca?)','',-76.8,-12.5,.05 from dual union all
select 323,'CHI','','launch site','PE','1974-03-29 00:00:00','1983-03-23 00:00:00','CHI   -','Chilca Launch Range, Lima, Peru','',-76.8,-12.5,.02 from dual union all
select 324,'CHIHA','','launch site','KP','1991-07-01 00:00:00','','CHIHA -','Chiha-ri, Ich''on, Kangwon do, Choson','',126.685,38.623,.1 from dual union all
select 325,'CHIOHE','','launch site','TW','1999-01-01 00:00:00','','CHIOH -','Chio He air base, Pingtung, Formosa (Taiwan)   [== JP?]','',120.5,22.6,.1 from dual union all
select 326,'CHM','PS','launch point','','','','CHM   PS','Chernoye More (Black Sea), PS submerged pad','',null,null,null from dual union all
select 327,'CHM','','launch zone','SU','','','CHM   -','Chernoye More (Black Sea) submarine test launch area','',34,44,.5 from dual union all
select 328,'CKAFS','','launch site','US','1964-01-22 00:00:00','1974-03-01 00:00:00','CKAFS','Cape Kennedy Air Force Station, Eastern Test Range','Cape Kennedy, Florida',-80.54,28.46,.05 from dual union all
select 329,'CLBI','MAN','launch point','','1965-12-15 00:00:00','','CLBI  MAN','CLBI Natal, MAN site','',-35.383,-5.866,.02 from dual union all
select 33,'ANHU','','launch site','KR','1993-06-01 00:00:00','','ANHU  -','Anhueng, Taean missile site, Ch''ungch''ong Namdo, Korea','Anhueng, Ch''ungch''ong Namdo',126.1,36.41,.02 from dual union all
select 330,'CLBI','','launch site','BR','1965-12-15 00:00:00','','CLBI  -','Centro de Lancamento da Barreira do Inferno, Natal, Rio Grande N, Brasil','',-35.383,-5.866,.02 from dual union all
select 331,'COLDLK','','launch site','CA','1954-01-01 00:00:00','1990-01-01 00:00:00','COLDL -','Cold Lake Air Weapons Range, Alberta (meteorological rockets)','',-110,54,.5 from dual union all
select 332,'CORO','','launch site','SR','1965-09-18 00:00:00','1965-09-27 00:00:00','CORO  -','Coronie, Surinam','',-56.3,5.85,.02 from dual union all
select 333,'CP','RAG','launch point','','1971-03-15 00:00:00','1971-03-22 00:00:00','CP    -','Thiokol RAG launcher at Cape Parry?','',-124.717,70.17,.02 from dual union all
select 334,'CP','','launch site','CA','1969-03-07 00:00:00','1982-01-21 00:00:00','CP    -','Cape Parry, PIN Main DEW, Inuvik Region, Northwest Territory, Canada','',-124.717,70.17,.02 from dual union all
select 335,'CRI23','KT-410','launch point','','1974-11-06 00:00:00','1974-11-06 00:00:00','CRI23 KT-410','KT No. 410, Luna-23 Site, Mare Crisium, Mare Undarum A3','Luna',56.5,13.5,.01 from dual union all
select 336,'CRI23','','launch site','LUNA','1974-11-06 00:00:00','1974-11-06 00:00:00','CRI23 KT-410','KT No. 410, Luna-23 Site, Mare Crisium, Mare Undarum A3','Luna',56.5,13.5,.01 from dual union all
select 337,'CRI24','KT-413','launch point','','1974-08-19 00:00:00','1974-08-19 00:00:00','CRI24 KT-413','KT No. 413, Luna-24 Site, Mare Crisium, Mare Undarum B1','Luna',62.2,12.75,.01 from dual union all
select 338,'CRI24','','launch site','LUNA','1974-08-19 00:00:00','1974-08-19 00:00:00','CRI24 KT-413','KT No. 413, Luna-24 Site, Mare Crisium, Mare Undarum B1','Luna',62.2,12.75,.01 from dual union all
select 339,'CSB','','launch site','US','1960-01-01 00:00:00','','CSB   -','Site D3, Cape San Blas, Eglin AFB, Florida','',-85.37,29.67,.02 from dual union all
select 34,'ANNA','','launch site','AU','1997-09-02 00:00:00','1997-09-11 00:00:00','ANNA  -','Anna Plains, Western Australia','Anna Plains',121.57,-19.28,.05 from dual union all
select 340,'CSG','ALFS','launch point','','1968-01-01 00:00:00','1979-01-01 00:00:00','CSG   ALFS','Aire de Lancement Fusee-Sondes, CSG','',-52.7317,5.208,.001 from dual union all
select 341,'CSG','ALFS V','launch point','','1968-01-01 00:00:00','1979-01-01 00:00:00','CSG   ALFS V','Ensemble Veronique, ALFS, CSG','',-52.7317,5.208,.001 from dual union all
select 342,'CSG','CECLES','launch point','','1971-01-01 00:00:00','1979-01-01 00:00:00','CSG   CECLES','Aire de Lancement CECLES, CSG','',-52.7747,5.2358,.001 from dual union all
select 343,'CSG','Diamant','launch point','','1970-01-01 00:00:00','1975-01-01 00:00:00','CSG   Diamant','Aire de Lancement Diamant, CSG','',-52.7517,5.2333,.001 from dual union all
select 344,'CSG','ELA1','launch point','','1979-01-01 00:00:00','1989-01-01 00:00:00','CSG   ELA1','Ensemble de Lancement Ariane 1, CSG','',-52.7747,5.2358,.001 from dual union all
select 345,'CSG','ELA2','launch point','','1986-01-01 00:00:00','','CSG   ELA2','Ensemble de Lancement Ariane 2, CSG','',-52.7755,5.2322,.001 from dual union all
select 346,'CSG','ELA3','launch point','','1996-01-01 00:00:00','','CSG   ELA3','Ensemble de Lancement Ariane 3, CSG','',-52.7687,5.2375,.001 from dual union all
select 347,'CSG','ELS','launch point','','2006-01-01 00:00:00','','CSG   ELS','Ensemble de Lancement Soyuz','',-52.8344,5.305,.0001 from dual union all
select 348,'CSG','MER','launch point','','1973-01-01 00:00:00','','CSG   MER','Offshore launch area CSG','',-52.8,5.3,.1 from dual union all
select 349,'CSG','ZLV','launch point','','2012-01-01 00:00:00','','CSG   ZLS','Zone de Lancement Vega','',-52.7747,5.2358,.001 from dual union all
select 35,'ANT','','launch site','AG','1972-05-16 00:00:00','1974-01-31 00:00:00','ANT   -','Antigua Air Force Station','Antigua',-61.7752,17.1371,.0005 from dual union all
select 350,'CSG','','launch site','GUF','1968-01-01 00:00:00','','CSG   -','Centre Spatial Guyanais, Kourou, Guyane Francaise','',-52.7517,5.2333,.02 from dual union all
select 351,'CUD','RW01/19','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','CUD   RW01/19','RW01/19, Cuddeback Dry Lake, California','',-117.47,35.31,.1 from dual union all
select 352,'CUD','','launch site','US','1959-01-01 00:00:00','1967-01-01 00:00:00','CUD   -','Cuddeback Dry Lake drop zone, California','',-117.27,35.18,.1 from dual;
insert into SITE
select 353,'CUX','','launch site','D','1945-01-01 00:00:00','1963-01-01 00:00:00','CUX   -','Cuxhaven, Niedersachsen, Deutschland','',8.7,53.87,.02 from dual union all
select 354,'CYCL','','launch site','GR','2008-01-01 00:00:00','','Pachia?','NAMFI target launch site: either Ftena, Pachia or Makra','Cyclades Islands',36.3,25.8,.15 from dual union all
select 355,'DAT','','launch site','US','1963-01-01 00:00:00','1963-01-01 00:00:00','DAT   -','Datil, New Mexico','',-107.51,34.08,.02 from dual union all
select 356,'DBAY','','launch site','LB','1960-01-01 00:00:00','1966-01-01 00:00:00','Dbayeh','Dbayeh launch site, LRS','Dbayeh, Liban',35.6,33.93,.1 from dual union all
select 357,'DDL','1','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','DDL   1','Delamar Dry Lake drop zone #1, Nevada','',-114.6,37.3,.02 from dual union all
select 358,'DDL','2','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','DDL   2','Delamar Dry Lake drop zone #2, Nevada','',-115,37.65,.02 from dual union all
select 359,'DDL','RW00/18','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','DDL   RW00/18','RW00/18, Delamar Dry Lake, Nevada','',-114.93,37.36,.02 from dual union all
select 36,'AOR10','','launch cruise','US','1947-09-06 00:00:00','1947-09-06 00:00:00','AOR10 -','USS Midway, Operation Sandy 1947 "several hundred mi SE Bermuda"','Atlantic Ocean',-60,27,5 from dual union all
select 360,'DDL','','launch site','US','1959-01-01 00:00:00','1967-01-01 00:00:00','DDL   RW00/18','RW00/18, Delamar Dry Lake, Nevada','',-114.93,37.36,.02 from dual union all
select 361,'DDU','','launch site','TF','1967-01-26 00:00:00','1967-01-29 00:00:00','DDU   -','Dumont d''Urville, Terre Adelie','Dumont d''Urville, Terre Adelie',140.02,-64.67,.02 from dual union all
select 362,'DEK','','launch site','IR','1990-01-01 00:00:00','1990-01-01 00:00:00','DEK   -','Dasht-E-Kabir, Iran (Khatami Air Base?)','',51.85,32.75,.05 from dual union all
select 363,'DERZH','','launch site','SU','1960-01-01 00:00:00','1990-01-01 00:00:00','DERZH','Derzhavinsk missile site, Russia','',66.1,51.3,.1 from dual union all
select 364,'DES','LM11-DS','launch point','','1972-04-24 00:00:00','1972-04-24 00:00:00','DES   LM11-DS','LM 11 Descent Stage, Descartes Base, Descartes Plain, Theophilus D2','Luna',15.51,-8.9,.01 from dual union all
select 365,'DES','','launch site','LUNA','1972-04-24 00:00:00','1972-04-24 00:00:00','DES   LM11-DS','LM 11 Descent Stage, Descartes Base, Descartes Plain, Theophilus D2','Luna',15.51,-8.9,.01 from dual union all
select 366,'DEVER','','launch site','EG','1973-01-01 00:00:00','1973-01-01 00:00:00','Deversoir','Deversoir Air Base','Egypt',32.3353,30.4227,.01 from dual union all
select 367,'DOMFB','','launch cruise','US','1962-04-19 00:00:00','1962-05-01 00:00:00','DOMFB -','DOMINIC/FRIGATE BIRD launch point, Pacific Ocean','',-134.03,12.53,.01 from dual union all
select 368,'DROV','','launch site','RU','1988-01-01 00:00:00','1989-01-01 00:00:00','DROV','Drovyanaya, Chitinsk oblast''','',113,51.5,.1 from dual union all
select 369,'DZGC','','launch zone','US','1997-01-01 00:00:00','1997-01-01 00:00:00','DZGC  -','Gran Canaria Drop Zone, North Atlantic Ocean','',-15.3,27,.1 from dual union all
select 37,'AOR11','LP1','launch point','','1953-07-18 00:00:00','1953-07-18 00:00:00','AOR11 LP1','Atlantic Ocean Launch Site 11 Launch Point 1','Atlantic Ocean',-70.37,42.43,.01 from dual union all
select 370,'DZK2','','launch zone','US','2008-01-01 00:00:00','','DZK2  -','Kwajalein Drop Zone (CNOFS), Pacific Ocean','',167.7562,10.5229,.0001 from dual union all
select 371,'DZK3','','launch zone','US','2012-01-01 00:00:00','','DZK3  -','Kwajalein Drop Zone (Nustar), Pacific Ocean','',167.7,6.743,.001 from dual union all
select 372,'DZKAU1','','launch zone','US','2014-01-01 00:00:00','2014-01-01 00:00:00','DZKAU1-','LDSD Drop Zone, Kauai, Pacific Ocean','',-160.714,22.194,.01 from dual union all
select 373,'DZKW','','launch zone','US','2000-01-01 00:00:00','','DZKW  -','Kwajalein Drop Zone, Pacific Ocean','',167.7,7.65,.05 from dual union all
select 374,'DZNC','','launch zone','US','1955-01-01 00:00:00','1955-01-01 00:00:00','DZNC  -','NAOTS Chincoteage Drop Zone, Wallops Island, Atlantic Ocean','',-75,37.5,1 from dual union all
select 375,'DZSB','','launch zone','US','1958-01-01 00:00:00','1958-01-01 00:00:00','DZSB  -','Santa Barbara Channel Drop Zone, California','',-120,34.2,.1 from dual union all
select 376,'DZWI','','launch zone','US','1996-01-01 00:00:00','','DZWI  -','Wallops Island Drop Zone, North Atlantic Ocean','',-72,37,.1 from dual union all
select 377,'DZWTR','','launch zone','US','1958-01-01 00:00:00','','DZWTR -','Western Test Range Drop Zone, Pacific Ocean','',-123,35,2 from dual union all
select 378,'EAFB','EDT','launch point','','1957-01-01 00:00:00','','EAFB  RW04L/22R','EDT (Temporary Runway) RW04L/22R, Edwards AFB, California','',-117.8,34.9,.1 from dual union all
select 379,'EAFB','RW04/22','launch point','','1957-01-01 00:00:00','','EAFB  RW04/22','RW04/22, Edwards AFB, California','',-117.8,34.9,.1 from dual union all
select 38,'AOR11','LP10','launch point','','1953-08-09 00:00:00','1953-08-09 00:00:00','AOR11 LP10','Atlantic Ocean Launch Site 11 Launch Point 10','Atlantic Ocean',-71.93,74.38,.01 from dual union all
select 380,'EAFB','RW18/36','launch point','','1957-01-01 00:00:00','','EAFB  RW18/36','RW18/36, Rogers Dry Lake, Edwards AFB, California','',-117.8,34.9,.1 from dual union all
select 381,'EAFB','TL1','launch point','','1957-01-01 00:00:00','','EAFB  TL1','Tethered Launch test silo, Edwards AFB','',-117.8,34.9,.1 from dual union all
select 382,'EAFB','TL2','launch point','','1957-01-01 00:00:00','','EAFB  TL2','Tethered Launch test silo, Edwards AFB','',-117.8,34.9,.1 from dual union all
select 383,'EAFB','','launch site','US','1957-01-01 00:00:00','','EAFB  -','Edwards AFB, California','',-117.8,34.9,.1 from dual union all
select 384,'EAO','','launch site','RU','2016-01-01 00:00:00','','EAO','Iskander mobile launch site, Everyskaya avtonomnaya oblast','',133,48.75,2 from dual union all
select 385,'EGL','A-11','launch point','','1959-09-18 00:00:00','1980-12-12 00:00:00','EGL   A-11','Site A-11, Santa Rosa Island, Eglin AFB, Florida','',-86.735,30.3958,.02 from dual union all
select 386,'EGL','A-15','launch point','','1959-09-18 00:00:00','1980-01-01 00:00:00','EGL   A-15','Site A-15, Santa Rosa Island, Eglin AFB, Florida','',-86.8077,30.387,.02 from dual union all
select 387,'EGL','A-15A','launch point','','1959-09-18 00:00:00','1980-01-01 00:00:00','EGL   A-15A','Site A-15A, Santa Rosa Island, Eglin AFB, Florida','',-86.8033,30.3875,.02 from dual union all
select 388,'EGL','A-15A Pad 2','launch point','','1959-09-18 00:00:00','1980-01-01 00:00:00','EGL   A-15A-2','Site A-15A Pad 2, Santa Rosa Island, Eglin AFB, Florida','',-86.8034,30.3875,.0001 from dual union all
select 389,'EGL','A-15A Pad 3','launch point','','1959-09-18 00:00:00','1980-01-01 00:00:00','EGL   A-15A-3','Site A-15A Pad 3, Santa Rosa Island, Eglin AFB, Florida','',-86.8037,30.3874,.0001 from dual union all
select 39,'AOR11','LP2','launch point','','1953-07-19 00:00:00','1953-07-19 00:00:00','AOR11 LP2','Atlantic Ocean Launch Site 11 Launch Point 2','Atlantic Ocean',-65.12,43.07,.01 from dual union all
select 390,'EGL','AT','launch point','','1959-09-18 00:00:00','1980-12-12 00:00:00','EGL   AT','Aerobee Tower, Santa Rosa Island, Eglin AFB, Florida','',-86.7165,30.3948,.0001 from dual union all
select 391,'EGL','Pad 1','launch point','','1959-09-18 00:00:00','1980-12-12 00:00:00','EGL   Pad 1','Launcher 1, Santa Rosa Island, Eglin AFB, Florida','',-86.712,30.3948,.0001 from dual union all
select 392,'EGL','Pad 2','launch point','','1959-09-18 00:00:00','1980-12-12 00:00:00','EGL   Pad 2','Launcher 2, Santa Rosa Island, Eglin AFB, Florida','',-86.7111,30.3948,.0001 from dual union all
select 393,'EGL','Pad 3','launch point','','1959-09-18 00:00:00','1980-12-12 00:00:00','EGL   Pad 3','Launcher 3, Santa Rosa Island, Eglin AFB, Florida','',-86.7103,30.3948,.0001 from dual union all
select 394,'EGL','','launch site','US','1959-09-18 00:00:00','1980-12-12 00:00:00','EGL   -','Santa Rosa Island, Eglin AFB, Florida','',-86.74,30.38,.02 from dual union all
select 395,'ELAR','MORABA','launch point','','1974-10-01 00:00:00','1974-10-01 00:00:00','ELAR  MORABA','MORABA launcher, Huelva 3 campaign, El Arenosillo','',-6.7417,37.0958,.0002 from dual union all
select 396,'ELAR','','launch site','E','1966-10-15 00:00:00','','ELAR  -','Estacion de sondeos atmosfericos de El Arenosillo, Huelva, Espana','',-6.7417,37.0958,.0002 from dual union all
select 397,'ELLS','N2','launch point','','','','ELLS  N2','Ellsworth AFB, Silo N2','',null,null,null from dual union all
select 398,'ELLS','','launch site','US','','','ELLS  -','Ellsworth AFB','',-103.104,44.145,.001 from dual union all
select 399,'ELM','B','launch point','','1959-01-01 00:00:00','1979-01-01 00:00:00','Elmendorf B','Site Bay, Battery C, Elmendorf AFB','Anchorage, Alaska',-149.8824,61.4039,.001 from dual union all
select 4,'AFMDC','','launch site','US','1957-09-01 00:00:00','1970-08-01 00:00:00','AFMDC','Air Force Missile Development Center, Holloman Air Force Base','Alamagordo, New Mexico',-106.103,32.8496,.0001 from dual union all
select 40,'AOR11','LP21','launch point','','1953-08-05 00:00:00','1953-08-05 00:00:00','AOR11 LP21','Atlantic Ocean Launch Site 11 Launch Point 21','Atlantic Ocean',-63.92,62.07,.01 from dual union all
select 400,'ELM','P','launch point','','1959-01-01 00:00:00','1979-01-01 00:00:00','Elmendorf P','Site Point, Elmendorf AFB','Anchorage, Alaska',-150.0541,61.1552,.001 from dual union all
select 401,'ELM','S','launch point','','1959-01-01 00:00:00','1979-01-01 00:00:00','Elmendorf S','Site Summit LS1, Battery B, Elmendorf AFB','Anchorage, Alaska',-149.547,61.2474,.001 from dual union all
select 402,'ELM','S2','launch point','','1959-01-01 00:00:00','1979-01-01 00:00:00','Elmendorf S','Site Summit LS2, Battery B, Elmendorf AFB','Anchorage, Alaska',-149.5467,61.2464,.001 from dual union all
select 403,'ELM','','launch site','US','1959-01-01 00:00:00','1979-01-01 00:00:00','Elmendorf AFB','Elmendorf AFB','Anchorage, Alaska',61.25,-149.8,.001 from dual union all
select 404,'EMBA','','launch site','KZ','','1999-01-01 00:00:00','EMBA  -','Emba test range, Kazakshtan','',55.42,47.1,.1 from dual union all
select 405,'ENW','','launch site','TTPI','1957-10-03 00:00:00','1957-10-22 00:00:00','ENW   -','EPG (Eniwetok Proving Grounds), Eniwetok (Enewetak) Atoll, Marshall Is.','',162.25,11.5,.02 from dual union all
select 406,'EQUOD','','launch site','CA','1970-03-07 00:00:00','1972-07-10 00:00:00','EQUOD -','East Quoddy, Newfoundland','',-63.42,44.9,.02 from dual union all
select 407,'ESR','A','launch point','','1976-01-01 00:00:00','','ESR   A','Aries/Maxus tower, ESRANGE','',21.1045,67.8934,.001 from dual union all
select 408,'ESR','C','launch point','','1965-02-01 00:00:00','','ESR   C','Centaure launcher, ESRANGE','',21.1042,67.8922,.001 from dual union all
select 409,'ESR','L','launch point','','1965-02-01 00:00:00','','ESR   L','Super Loki/Viper Launchers, ESRANGE','',21.1065,67.8932,.0001 from dual union all
select 41,'AOR11','LP22','launch point','','1953-08-08 00:00:00','1953-08-08 00:00:00','AOR11 LP22','Atlantic Ocean Launch Site 11 Launch Point 22','Atlantic Ocean',-63.92,73.62,.01 from dual union all
select 410,'ESR','MAN','launch point','','1965-02-01 00:00:00','','ESR   MAN','MAN Launcher, ESRANGE','',21.1053,67.8932,.0001 from dual union all
select 411,'ESR','MAN/MRL','launch point','','1965-02-01 00:00:00','','ESR   MRL','MAN or MRL Launcher, ESRANGE','',21.106,67.8932,.0001 from dual union all
select 412,'ESR','MRL','launch point','','1965-02-01 00:00:00','','ESR   MRL','MRL Launcher, ESRANGE','',21.106,67.8932,.0001 from dual union all
select 413,'ESR','N','launch point','','1965-02-01 00:00:00','','ESR   N','Nike launcher W or E, ESRANGE','',21.109,67.8933,.0002 from dual union all
select 414,'ESR','N1','launch point','','1965-02-01 00:00:00','','ESR   N2','Nike launcher W, ESRANGE','',21.1102,67.8933,.0002 from dual union all
select 415,'ESR','N2','launch point','','1965-02-01 00:00:00','','ESR   N2','Nike launcher E, ESRANGE','',21.1102,67.8933,.0002 from dual union all
select 416,'ESR','S','launch point','','1965-02-01 00:00:00','','ESR   S','Skylark Tower (Skylarktornet), ESRANGE','',21.107,67.8933,.0001 from dual union all
select 417,'ESR','','launch site','S','1965-02-01 00:00:00','','ESR   -','ESRANGE, Kiruna','',21.107,67.894,.002 from dual union all
select 418,'ETR','500NM','launch point','','','','ETR   500NM','930 km downrange test area for operational conditions','',null,null,null from dual union all
select 419,'ETR','D4','launch point','','1960-01-01 00:00:00','','ETR   D4','Launch Area 87 km NE of Port Canaveral, ETR,  Atlantic Ocean','',-79,28.5,2 from dual union all
select 42,'AOR11','LP23','launch point','','1953-08-09 00:00:00','1953-08-09 00:00:00','AOR11 LP23','Atlantic Ocean Launch Site 11 Launch Point 23','Atlantic Ocean',-63.92,74.48,.01 from dual union all
select 420,'ETR','LP5','launch point','','1960-01-01 00:00:00','','ETR   LP5','Unknown - Launch Point 5, Eastern Test Range, Atlantic Ocean','',-79,28.5,2 from dual union all
select 421,'ETR','','launch zone','US','1960-01-01 00:00:00','','ETR   -','SLBM Launch Area, Eastern Test Range, Atlantic Ocean','',-79,28.5,2 from dual union all
select 422,'EUNYUL','','launch site','KP','2016-01-01 00:00:00','','Hwangju','Scud launch site near Eunyul, N Hwanghae Prov.','',125.183,38.519,1 from dual union all
select 423,'FC','LC1','launch point','','1956-01-01 00:00:00','1998-01-01 00:00:00','FC    LC1','Universal Launcher, Pad 1, Churchill Research Range, Canada','',-93.8161,58.7322,.0001 from dual union all
select 424,'FC','LC3','launch point','','1956-01-01 00:00:00','1998-01-01 00:00:00','FC    LC3','Aerobee Launcher, Pad 3, Churchill Research Range, Canada','',-93.8206,58.7345,.0001 from dual union all
select 425,'FC','LC4AE','launch point','','1956-01-01 00:00:00','1998-01-01 00:00:00','FC    LC4AE','Arcas East Launcher, Pad 4A, Churchill Research Range, Canada','',-93.822,58.7339,.0001 from dual union all
select 426,'FC','LC4AW','launch point','','1956-01-01 00:00:00','1998-01-01 00:00:00','FC    LC4AW','Arcas West Launcher, Pad 4A, Churchill Research Range, Canada','',-93.8221,58.7339,.0001 from dual union all
select 427,'FC','LC7','launch point','','1968-08-01 00:00:00','1998-01-01 00:00:00','FC    LC7','Auroral Launcher, Pad 7, Churchill Research Range, Canada','',-93.815,58.7327,.0001 from dual union all
select 428,'FC','','launch site','CA','1956-01-01 00:00:00','1998-01-01 00:00:00','FC    -','Churchill Research Range, Fort Churchill, Manitoba, Canada','',-93.8203,58.7343,.001 from dual union all
select 429,'FEC','KT-406','launch point','','1970-09-21 00:00:00','1970-09-21 00:00:00','FEC   KT-406','KT No. 406, Luna-16 Site, Mare Fecunditatis, Langrenus A2','Luna',56.3,-.68,.01 from dual union all
select 43,'AOR11','LP24','launch point','','1953-08-11 00:00:00','1953-08-11 00:00:00','AOR11 LP24','Atlantic Ocean Launch Site 11 Launch Point 24','Atlantic Ocean',-63.92,74.57,.01 from dual union all
select 430,'FEC','','launch site','LUNA','1970-09-21 00:00:00','1970-09-21 00:00:00','FEC   KT-406','KT No. 406, Luna-16 Site, Mare Fecunditatis, Langrenus A2','Luna',56.3,-.68,.01 from dual union all
select 431,'FMR','LM8-DS','launch point','','1971-02-06 00:00:00','1971-02-06 00:00:00','FMR   LM8-DS','LM 8 Descent Stage, Fra Mauro Base, Fra Mauro, Montes Riphaeus B1','Luna',-19.27,-3.67,.01 from dual union all
select 432,'FMR','','launch site','LUNA','1971-02-06 00:00:00','1971-02-06 00:00:00','FMR   LM8-DS','LM 8 Descent Stage, Fra Mauro Base, Fra Mauro, Montes Riphaeus B1','Luna',-19.27,-3.67,.01 from dual union all
select 433,'FTB','MCG','launch point','','1959-01-01 00:00:00','','FTB   MCG','McGregor Range, Fort Bliss, New Mexico (White Sands)','',-106.1524,32.0736,.01 from dual union all
select 434,'FTB','ORO','launch point','','1960-01-01 00:00:00','1962-01-01 00:00:00','WS    ORO','Orogrande Range Camp, White Sands, NM','',-106.1486,32.404,.001 from dual union all
select 435,'FTB','','launch site','US','1945-01-01 00:00:00','1970-01-01 00:00:00','FTB   -','Hueco Range, Fort Bliss, Texas','',-106.65,31.8,.1 from dual union all
select 436,'FW','LC96','launch point','','1997-11-17 00:00:00','','FW    LC96','Launch Complex 96, Fort Wingate, New Mexico','',-108.5995,35.4487,.0001 from dual union all
select 437,'FW','','launch site','US','1958-01-01 00:00:00','','FW    -','Fort Wingate Depot Activity, New Mexico','',-108.6,35.45,.005 from dual union all
select 438,'FWAK','J1','launch point','','1959-01-01 00:00:00','1971-01-01 00:00:00','FWAK Site Jig 1','Site Jig LS1, Nike Battery D, Ft Wainwright, Alaska','Fairbanks, Alaska',-146.964,64.5259,.0001 from dual union all
select 439,'FWAK','J2','launch point','','1959-01-01 00:00:00','1971-01-01 00:00:00','FWAK Site Jig 2','Site Jig LS2, Nike Battery D, Ft Wainwright, Alaska','Fairbanks, Alaska',-146.9615,64.5256,.0001 from dual union all
select 44,'AOR11','LP3','launch point','','1953-07-19 00:00:00','1953-07-19 00:00:00','AOR11 LP3','Atlantic Ocean Launch Site 11 Launch Point 3','Atlantic Ocean',-63.47,43.68,.01 from dual union all
select 440,'FWAK','L1','launch point','','1959-01-01 00:00:00','1971-01-01 00:00:00','FWAK Site Love 1','Site Love LS1, Nike Battery E, Ft Wainwright, Alaska','Fairbanks, Alaska',-147.8815,64.9834,.0001 from dual union all
select 441,'FWAK','L2','launch point','','1959-01-01 00:00:00','1971-01-01 00:00:00','FWAK Site Love 2','Site Love LS1, Nike Battery E, Ft Wainwright, Alaska','Fairbanks, Alaska',-147.8841,64.9834,.0001 from dual union all
select 442,'FWAK','M','launch point','','1959-01-01 00:00:00','1971-01-01 00:00:00','FWAK Site Mike','Site Mike, Nike Battery C, Ft Wainwright, Alaska','Fairbanks, Alaska',-146.7281,64.5864,.0001 from dual;
insert into SITE
select 443,'FWAK','P','launch point','','1959-01-01 00:00:00','1971-01-01 00:00:00','FWAK Site Peter 1','Site Peter LS1, Nike Battery B, Ft Wainwright, Alaska','Fairbanks, Alaska',-146.7494,64.6734,.0001 from dual union all
select 444,'FWAK','P2','launch point','','1959-01-01 00:00:00','1971-01-01 00:00:00','FWAK Site Peter 2','Site Peter LS2, Nike Battery B, Ft Wainwright, Alaska','Fairbanks, Alaska',-146.7477,64.6726,.0001 from dual union all
select 445,'FWAK','T1','launch point','','1959-01-01 00:00:00','1971-01-01 00:00:00','FWAK Site Tare 1','Site Tare LS1, Nike Battery A, Ft Wainwright, Alaska','Fairbanks, Alaska',-147.2141,64.7567,.0001 from dual union all
select 446,'FWAK','T2','launch point','','1959-01-01 00:00:00','1971-01-01 00:00:00','FWAK Site Tare 2','Site Tare LS2, Nike Battery A, Ft Wainwright, Alaska','Fairbanks, Alaska',-147.2166,64.7567,.0001 from dual union all
select 447,'FWAK','','launch site','US','1959-01-01 00:00:00','1971-01-01 00:00:00','Ft Wainwright','Ft Wainwright, Alaska','',-147.633,64.83,.02 from dual union all
select 448,'FXM','RAG','launch point','','1971-03-15 00:00:00','1971-03-22 00:00:00','FXM   -','Thiokol RAG launcher at FOX-MAIN DEW','',-81.2,68.7,.2 from dual union all
select 449,'FXM','','launch site','CA','1971-03-15 00:00:00','1971-03-22 00:00:00','FXM   -','FOX-MAIN DEW site, Hall Beach, Nunavut, Canada','',-81.224,68.757,.002 from dual union all
select 45,'AOR11','LP4','launch point','','1953-07-19 00:00:00','1953-07-19 00:00:00','AOR11 LP4','Atlantic Ocean Launch Site 11 Launch Point 4','Atlantic Ocean',-62.17,44.27,.01 from dual union all
select 450,'GAN','','launch site','MV','','','GANI  -','Gan Island, Addu Atoll, Maldives, Indian Ocean','',73.15,-.6833,.01 from dual union all
select 451,'GANC','RW03/21','launch point','','1997-04-01 00:00:00','1997-04-01 00:00:00','GAN   RW03/21','RW03/21L and RW03/21R, Base Aerea de Gando, Gran Canaria','',-15.32,27.92,.02 from dual union all
select 452,'GANC','','launch site','ESCN','1997-04-01 00:00:00','1997-04-01 00:00:00','GAN   -','Base Aerea de Gando, Gran Canaria','',-15.32,27.92,.02 from dual union all
select 453,'GF','','launch site','US','','','GF    -','Grand Forks AFB','',-97.4,47.96,.05 from dual union all
select 454,'GIK-1','LC131/1','launch point','','1967-01-01 00:00:00','','GIK-1 LC131','Ploshchadka 131, GIK-1, Plesetsk, Rossiya','',40.8,62.9,.2 from dual union all
select 455,'GIK-1','LC132','launch point','','1967-01-01 00:00:00','','GIK-1 LC132','Ploshchadka 132 PU No 1 or 2, GIK-1, Plesetsk, Rossiya','',40.887,62.893,.1 from dual union all
select 456,'GIK-1','LC132/1','launch point','','1967-01-01 00:00:00','','GIK-1 LC132/1','Ploshchadka 132/1, GIK-1, Plesetsk, Rossiya','',40.887,62.893,.1 from dual union all
select 457,'GIK-1','LC132/2','launch point','','1967-01-01 00:00:00','','GIK-1 LC132/2','Ploshchadka 132/1, GIK-1, Plesetsk, Rossiya','',40.888,62.893,.1 from dual union all
select 458,'GIK-1','LC133','launch point','','1967-01-01 00:00:00','','GIK-1 LC133','PL 133 PU No. 1 or 3, GIK-1','',40.867,62.895,.1 from dual union all
select 459,'GIK-1','LC133/1','launch point','','1967-01-01 00:00:00','','GIK-1 LC133/1','Ploshchadka 133/1, GIK-1, Plesetsk, Rossiya','',40.867,62.895,.1 from dual union all
select 46,'AOR11','LP5','launch point','','1953-07-24 00:00:00','1953-07-24 00:00:00','AOR11 LP5','Atlantic Ocean Launch Site 11 Launch Point 5','Atlantic Ocean',-61.92,58.53,.01 from dual union all
select 460,'GIK-1','LC133/3','launch point','','1967-01-01 00:00:00','','GIK-1 LC133/3','Ploshchadka 133/3, GIK-1, Plesetsk, Rossiya','',40.868,62.895,.1 from dual union all
select 461,'GIK-1','LC157','launch point','','1967-01-01 00:00:00','','NIIP-53 LC157','LC157, 53 NIIP, Plesetsk','',40.8,62.9,.2 from dual union all
select 462,'GIK-1','LC16/2','launch point','','1960-01-01 00:00:00','','GIK-1 LC16/2','Ploshchadka 16/2, GIK-1, Plesetsk, Rossiya','',40.68,62.95,.05 from dual union all
select 463,'GIK-1','LC161/1','launch point','','1967-01-01 00:00:00','','NIIP-53 LC161','LC161 PU No. 1, 53 NIIP, Plesetsk','',40.8,62.9,.2 from dual union all
select 464,'GIK-1','LC161/ShPU-2','launch point','','1967-01-01 00:00:00','','NIIP-53 LC161','LC161 ShPU No. 2, 53 NIIP, Plesetsk','',40.8,62.9,.2 from dual union all
select 465,'GIK-1','LC163','launch point','','1967-01-01 00:00:00','','NIIP-53 LC163','LC163 , 53 NIIP, Plesetsk','',40.8,62.9,.2 from dual union all
select 466,'GIK-1','LC163/ShPU-2','launch point','','1967-01-01 00:00:00','','NIIP-53 LC163','LC163 ShPU No. 2, 53 NIIP, Plesetsk','',40.8,62.9,.2 from dual union all
select 467,'GIK-1','LC167','launch point','','1967-01-01 00:00:00','','NIIP-53 LC167','LC167, 53 NIIP, Plesetsk','',40.8,62.9,.2 from dual union all
select 468,'GIK-1','LC31/ShPu-12','launch point','','1967-01-01 00:00:00','','GIK-1 LC31/12','Ploshchadka 32/ShPu-12, GIK-1, Plesetsk, Rossiya','',40.8,62.9,.5 from dual union all
select 469,'GIK-1','LC32/1','launch point','','1977-01-01 00:00:00','','GIK-1 LC32/1','Ploshchadka 32/1, GIK-1, Plesetsk, Rossiya','',40.8,62.915,.08 from dual union all
select 47,'AOR11','LP6','launch point','','1953-07-28 00:00:00','1953-07-28 00:00:00','AOR11 LP6','Atlantic Ocean Launch Site 11 Launch Point 6','Atlantic Ocean',-64.23,62.5,.01 from dual union all
select 470,'GIK-1','LC32/2','launch point','','1977-01-01 00:00:00','','GIK-1 LC32/2','Ploshchadka 32/2, GIK-1, Plesetsk, Rossiya','',40.801,62.915,.08 from dual union all
select 471,'GIK-1','LC35','launch point','','2014-01-01 00:00:00','','GIK-1 LC35','Ploshchadka 35, GIK-1, Plesetsk, Rossiya','',40.58,62.92,.01 from dual union all
select 472,'GIK-1','LC35/1','launch point','','2014-01-01 00:00:00','','GIK-1 LC35','Ploshchadka 35, GIK-1, Plesetsk, Rossiya','',40.58,62.92,.01 from dual union all
select 473,'GIK-1','LC41/1','launch point','','1960-01-01 00:00:00','','GIK-1 LC41/1','Ploshchadka 41/1, GIK-1, Plesetsk, Rossiya','',40.513,62.93,.05 from dual union all
select 474,'GIK-1','LC43/3','launch point','','1960-01-01 00:00:00','','GIK-1 LC43/3','Ploshchadka 43/3, GIK-1, Plesetsk, Rossiya','',40.45,62.93,.02 from dual union all
select 475,'GIK-1','LC43/4','launch point','','1960-01-01 00:00:00','','GIK-1 LC43/4','Ploshchadka 43/4 ''Sankt-Peterburg'', GIK-1, Plesetsk, Rossiya','',40.47,62.93,.02 from dual union all
select 476,'GIK-1','PU11','launch point','','1960-01-01 00:00:00','','GIK-1 PU11','PU 11, NIIP-53, Plesetsk, Rossiya','',40.5,62.9,.1 from dual union all
select 477,'GIK-1','','launch site','RU','1994-11-01 00:00:00','','GIK-1 -','1-y Gosudarstvenniy Ispitatelniy Kosmodrom MO RF','Plesetsk, Rossiya',40.5,62.9,.02 from dual union all
select 478,'GIK-2','LC5','launch point','','1990-01-01 00:00:00','','GIK-2 LC5','Ploshchadka 5, GIK-2, Svobodniy','',128,51.7,.5 from dual union all
select 479,'GIK-2','','launch site','RU','1990-01-01 00:00:00','2016-01-01 00:00:00','GIK-2 -','GIK-2, Svobodniy, Amurskaya Oblast'', Rossiya','',128,51.7,.5 from dual union all
select 48,'AOR11','LP7','launch point','','1953-08-03 00:00:00','1953-08-03 00:00:00','AOR11 LP7','Atlantic Ocean Launch Site 11 Launch Point 7','Atlantic Ocean',-66.25,62.75,.01 from dual union all
select 480,'GIK-5','LC1','launch point','','1957-01-01 00:00:00','','GIK-5 LC1','Ploshchadka 1 PU 17P32-5, GIK-5, Baykonur, Kazakstan','',63.35,45.912,.015 from dual union all
select 481,'GIK-5','LC1/LC31','launch point','','1957-01-01 00:00:00','','GIK-5 LC1/LC31','Ploshchadka 1 or 31, PU 5 or 6, GIK-5, Baykonur, Kazakstan','',63.35,45.912,.1 from dual union all
select 482,'GIK-5','LC101','launch point','','1986-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC101','Ploshchadka 101, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 483,'GIK-5','LC101/93','launch point','','1986-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC101','Ploshchadka 101, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 484,'GIK-5','LC102','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC102','Ploshchadka 102, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 485,'GIK-5','LC102/32','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC102','Ploshchadka 102, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 486,'GIK-5','LC103','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC103','Ploshchadka 103, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 487,'GIK-5','LC103/94','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC103','Ploshchadka 103, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 488,'GIK-5','LC104','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC104','Ploshchadka 104, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 489,'GIK-5','LC104/92','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC104','Ploshchadka 104, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 49,'AOR11','LP8','launch point','','1953-08-06 00:00:00','1953-08-06 00:00:00','AOR11 LP8','Atlantic Ocean Launch Site 11 Launch Point 8','Atlantic Ocean',-59.1,64.33,.01 from dual union all
select 490,'GIK-5','LC105','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC105','Ploshchadka 105, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 491,'GIK-5','LC105/99','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC105','Ploshchadka 105, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 492,'GIK-5','LC106','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC106','Ploshchadka 106, GIK-5, Baykonur, Kazakstan','',63.441,45.944,.02 from dual union all
select 493,'GIK-5','LC106/100','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC106','Ploshchadka 106, GIK-5, Baykonur, Kazakstan','',63.441,45.944,.02 from dual union all
select 494,'GIK-5','LC107','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC107','Ploshchadka 107, GIK-5, Baykonur, Kazakstan','',63.81,46.046,.02 from dual union all
select 495,'GIK-5','LC108','launch point','','1965-01-01 00:00:00','','GIK-5 LC108','Ploshchadka 108, GIK-5, Baykonur, Kazakstan','',63.815,46.046,.02 from dual union all
select 496,'GIK-5','LC109','launch point','','1965-01-01 00:00:00','','GIK-5 LC109','Ploshchadka 109, GIK-5, Baykonur, Kazakstan','',63.442,45.944,.02 from dual union all
select 497,'GIK-5','LC109/95','launch point','','1965-01-01 00:00:00','','GIK-5 LC109/95','PU95, Ploshchadka 109, GIK-5, Baykonur, Kazakstan','',63.442,45.944,.02 from dual union all
select 498,'GIK-5','LC110L','launch point','','1969-01-01 00:00:00','','GIK-5 LC110L','Ploshchadka 110/37, GIK-5, Baykonur, Kazakstan','',63.309,45.955,.015 from dual union all
select 499,'GIK-5','LC110R','launch point','','1969-01-01 00:00:00','','GIK-5 LC110R','Ploshchadka 110/38, GIK-5, Baykonur, Kazakstan','',63.314,45.95,.015 from dual union all
select 5,'AFMTC','','launch site','US','1951-06-30 00:00:00','1955-12-16 00:00:00','AFMTC','Air Force Missile Test Center','Cape Canaveral, Florida',-80.54,28.46,.05 from dual union all
select 50,'AOR11','LP9','launch point','','1953-08-06 00:00:00','1953-08-06 00:00:00','AOR11 LP9','Atlantic Ocean Launch Site 11 Launch Point 9','Atlantic Ocean',-58.58,65.22,.01 from dual union all
select 500,'GIK-5','LC130','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC130','Ploshchadka 130, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 501,'GIK-5','LC130/26','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC130','Ploshchadka 130, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 502,'GIK-5','LC130/27','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC130','Ploshchadka 130, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 503,'GIK-5','LC131','launch point','','1965-01-01 00:00:00','','GIK-5 LC131','Ploshchadka 131, GIK-5, Baykonur, Kazakstan','',62.968,46.069,.2 from dual union all
select 504,'GIK-5','LC131/28','launch point','','1965-01-01 00:00:00','','GIK-5 LC131','Ploshchadka 131, GIK-5, Baykonur, Kazakstan','',62.968,46.069,.2 from dual union all
select 505,'GIK-5','LC131/29','launch point','','1965-01-01 00:00:00','','GIK-5 LC131','Ploshchadka 131, GIK-5, Baykonur, Kazakstan','',62.968,46.069,.2 from dual union all
select 506,'GIK-5','LC131N/57','launch point','','1965-01-01 00:00:00','','GIK-5 LC131','Ploshchadka 131, GIK-5, Baykonur, Kazakstan','',62.968,46.069,.2 from dual union all
select 507,'GIK-5','LC132','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC132','Ploshchadka 132, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 508,'GIK-5','LC132/30','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC132','ShPU-30,Ploshchadka 132, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 509,'GIK-5','LC133','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC133','Ploshchadka 133, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 51,'AOR11','','launch cruise','US','1953-07-18 00:00:00','1953-09-01 00:00:00','AOR11','USS Staten Island, 1953 Arctic expedition (Operation MUSHRAT)','Atlantic Ocean',-70.37,42.43,.01 from dual union all
select 510,'GIK-5','LC140/18','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC140/18','PU18, Ploshchadka 140, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 511,'GIK-5','LC140/33','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC140/18','PU33, Ploshchadka 140, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 512,'GIK-5','LC140/96','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC140/96','PU96, Ploshchadka 140, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 513,'GIK-5','LC141','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC141','Ploshchadka 141, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 514,'GIK-5','LC141/31','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC141','Ploshchadka 141, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 515,'GIK-5','LC142/34','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC142/34','Ploshchadka 142, GIK-5, Baykonur, Kazakstan (Also PL142-1a)','',63,46,.2 from dual union all
select 516,'GIK-5','LC145/75','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC145/75','Ploshchadka 145, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 517,'GIK-5','LC160','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC160','Ploshchadka 160, GIK-5, Baykonur, Kazakstan','',62.946,46.073,.015 from dual union all
select 518,'GIK-5','LC161/35','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC161/35','Ploshchadka 161, GIK-5, Baykonur, Kazakstan (Also PL142-16)','',63.077,46.038,.03 from dual union all
select 519,'GIK-5','LC162/36','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC162/36','Ploshchadka 162, GIK-5, Baykonur, Kazakstan (Also PL142-1v)','',63.079,46.038,.03 from dual union all
select 52,'AOR11B','LP11','launch point','','1953-08-30 00:00:00','1953-08-30 00:00:00','AOR11 LP11','Atlantic Ocean Launch Site 11 Launch Point 11','Atlantic Ocean',-55.08,53.1,.01 from dual union all
select 520,'GIK-5','LC163','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC163','Ploshchadka 163, GIK-5, Baykonur, Kazakstan','',63.172,46.002,.015 from dual union all
select 521,'GIK-5','LC164','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC164','Ploshchadka 164, GIK-5, Baykonur, Kazakstan','',63.014,45.957,.03 from dual union all
select 522,'GIK-5','LC165','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC165','Ploshchadka 165, GIK-5, Baykonur, Kazakstan','',62.912,45.993,.03 from dual union all
select 523,'GIK-5','LC170','launch point','','1965-01-01 00:00:00','','GIK-5 LC170','Ploshchadka 170, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 524,'GIK-5','LC170/43','launch point','','1965-01-01 00:00:00','','GIK-5 LC170','Ploshchadka 170, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 525,'GIK-5','LC171','launch point','','1965-01-01 00:00:00','','GIK-5 LC171','Ploshchadka 171, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 526,'GIK-5','LC171/48','launch point','','1965-01-01 00:00:00','','GIK-5 LC171','Ploshchadka 171, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 527,'GIK-5','LC172','launch point','','1965-01-01 00:00:00','','GIK-5 LC172','Ploshchadka 172, GIK-5, Baykonur, Kazakstan','',63.092,46.065,.015 from dual union all
select 528,'GIK-5','LC172/49','launch point','','1965-01-01 00:00:00','','GIK-5 LC172','Ploshchadka 172, GIK-5, Baykonur, Kazakstan','',63.092,46.065,.015 from dual union all
select 529,'GIK-5','LC173','launch point','','1965-01-01 00:00:00','','GIK-5 LC173','Ploshchadka 173, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 53,'AOR11B','LP12','launch point','','1953-08-30 00:00:00','1953-08-30 00:00:00','AOR11 LP12','Atlantic Ocean Launch Site 11 Launch Point 12','Atlantic Ocean',-55.4,52.78,.01 from dual union all
select 530,'GIK-5','LC173/50','launch point','','1965-01-01 00:00:00','','GIK-5 LC173','Ploshchadka 173, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 531,'GIK-5','LC174','launch point','','1965-01-01 00:00:00','','GIK-5 LC174','Ploshchadka 174, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 532,'GIK-5','LC174/44','launch point','','1965-01-01 00:00:00','','GIK-5 LC174','Ploshchadka 174, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual;
insert into SITE
select 533,'GIK-5','LC175','launch point','','1965-01-01 00:00:00','','GIK-5 LC175','Ploshchadka 175, GIK-5, Baykonur, Kazakstan','',62.98,46.045,.015 from dual union all
select 534,'GIK-5','LC175/1/58','launch point','','1965-01-01 00:00:00','','GIK-5 LC175/2','PU58 (1?)Ploshchadka 175, GIK-5, Baykonur, Kazakstan','',62.98,46.045,.015 from dual union all
select 535,'GIK-5','LC175/2/59','launch point','','1965-01-01 00:00:00','','GIK-5 LC175/59','PU59 (2?), Ploshchadka 175, GIK-5, Baykonur, Kazakstan','',62.98,46.045,.015 from dual union all
select 536,'GIK-5','LC175/45','launch point','','1965-01-01 00:00:00','','GIK-5 LC175/2','PU45, Ploshchadka 175, GIK-5, Baykonur, Kazakstan','',62.98,46.045,.015 from dual union all
select 537,'GIK-5','LC175/58','launch point','','1965-01-01 00:00:00','','GIK-5 LC175/2','PU58 (1?)Ploshchadka 175, GIK-5, Baykonur, Kazakstan','',62.98,46.045,.015 from dual union all
select 538,'GIK-5','LC175/59','launch point','','1965-01-01 00:00:00','','GIK-5 LC175/59','PU59 (2?), Ploshchadka 175, GIK-5, Baykonur, Kazakstan','',62.98,46.045,.015 from dual union all
select 539,'GIK-5','LC176','launch point','','1965-01-01 00:00:00','','GIK-5 LC176','Ploshchadka 176, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 54,'AOR11B','LP13','launch point','','1953-08-30 00:00:00','1953-08-30 00:00:00','AOR11 LP13','Atlantic Ocean Launch Site 11 Launch Point 13','Atlantic Ocean',-54.75,53.13,.01 from dual union all
select 540,'GIK-5','LC176/46','launch point','','1965-01-01 00:00:00','','GIK-5 LC176','Ploshchadka 176, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 541,'GIK-5','LC177','launch point','','1965-01-01 00:00:00','','GIK-5 LC177','Ploshchadka 177, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 542,'GIK-5','LC177/47','launch point','','1965-01-01 00:00:00','','GIK-5 LC177','Ploshchadka 177, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 543,'GIK-5','LC179','launch point','','1965-01-01 00:00:00','','GIK-5 LC179','Ploshchadka 179, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 544,'GIK-5','LC179/42','launch point','','1965-01-01 00:00:00','','GIK-5 LC179','Ploshchadka 179, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 545,'GIK-5','LC181','launch point','','1965-01-01 00:00:00','','GIK-5 LC181','Ploshchadka 181, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 546,'GIK-5','LC181/52','launch point','','1965-01-01 00:00:00','','GIK-5 LC181','Ploshchadka 181, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 547,'GIK-5','LC191/66','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC191/66','Ploshchadka 191, GIK-5, Baykonur, Kazakstan','',63.223,45.967,.015 from dual union all
select 548,'GIK-5','LC192','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC192','Ploshchadka 192, GIK-5, Baykonur, Kazakstan','',63.306,46.023,.015 from dual union all
select 549,'GIK-5','LC193','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC193','Ploshchadka 193, GIK-5, Baykonur, Kazakstan','',63.382,45.946,.015 from dual union all
select 55,'AOR11B','LP14','launch point','','1953-09-03 00:00:00','1953-09-03 00:00:00','AOR11 LP14','Atlantic Ocean Launch Site 11 Launch Point 14','Atlantic Ocean',-57.22,44.83,.01 from dual union all
select 550,'GIK-5','LC194','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC194','Ploshchadka 194, GIK-5, Baykonur, Kazakstan','',63.301,45.854,.015 from dual union all
select 551,'GIK-5','LC195','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC195','Ploshchadka 195, GIK-5, Baykonur, Kazakstan','',63.235,45.792,.025 from dual union all
select 552,'GIK-5','LC196','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC196','Ploshchadka 196, GIK-5, Baykonur, Kazakstan','',63.129,45.81,.025 from dual union all
select 553,'GIK-5','LC200/39','launch point','','1965-01-01 00:00:00','','GIK-5 LC200/39','PU39, Ploshchadka 200L, GIK-5, Baykonur, Kazakstan','',63.037,46.035,.015 from dual union all
select 554,'GIK-5','LC200/40','launch point','','1965-01-01 00:00:00','','GIK-5 LC200/40','PU40, Ploshchadka 200P, GIK-5, Baykonur, Kazakstan','',63.044,46.031,.015 from dual union all
select 555,'GIK-5','LC241','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC241','Ploshchadka 241, GIK-5, Baykonur, Kazakstan','',63.452,45.849,.03 from dual union all
select 556,'GIK-5','LC242','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC242','Ploshchadka 242, GIK-5, Baykonur, Kazakstan','',63.452,45.932,.03 from dual union all
select 557,'GIK-5','LC243','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC243','Ploshchadka 243, GIK-5, Baykonur, Kazakstan','',63.692,45.842,.03 from dual union all
select 558,'GIK-5','LC244','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC244','Ploshchadka 244, GIK-5, Baykonur, Kazakstan','',63.612,45.826,.03 from dual union all
select 559,'GIK-5','LC245','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC245','Ploshchadka 245, GIK-5, Baykonur, Kazakstan','',63.506,45.813,.02 from dual union all
select 56,'AOR11B','LP15','launch point','','1953-09-03 00:00:00','1953-09-03 00:00:00','AOR11 LP15','Atlantic Ocean Launch Site 11 Launch Point 15','Atlantic Ocean',-57.17,44.75,.01 from dual union all
select 560,'GIK-5','LC246','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC246','Ploshchadka 246, GIK-5, Baykonur, Kazakstan','',63.414,45.752,.02 from dual union all
select 561,'GIK-5','LC250','launch point','','1987-01-01 00:00:00','','GIK-5 LC250','Ploshchadka 250, GIK-5, Baykonur, Kazakstan','',63.309,45.995,.015 from dual union all
select 562,'GIK-5','LC31','launch point','','1957-01-01 00:00:00','','GIK-5 LC31','Ploshchadka 31 PU 17P32-6, GIK-5, Baykonur, Kazakstan','',63.556,45.982,.015 from dual union all
select 563,'GIK-5','LC41','launch point','','1964-01-01 00:00:00','1968-01-01 00:00:00','GIK-5 LC41','Ploshchadka 41, GIK-5, Baykonur, Kazakstan','',63.642,45.958,.02 from dual union all
select 564,'GIK-5','LC41/15','launch point','','1964-01-01 00:00:00','1968-01-01 00:00:00','GIK-5 LC41/15','PU15, Ploshchadka 41, GIK-5, Baykonur, Kazakstan','',63.643,45.955,.02 from dual union all
select 565,'GIK-5','LC41/3','launch point','','1964-01-01 00:00:00','1968-01-01 00:00:00','GIK-5 LC41/3','PU3, Ploshchadka 41L, GIK-5, Baykonur, Kazakstan','',63.642,45.958,.02 from dual union all
select 566,'GIK-5','LC41/4','launch point','','1964-01-01 00:00:00','1968-01-01 00:00:00','GIK-5 LC41/4','PU4, Ploshchadka 41P, GIK-5, Baykonur, Kazakstan','',63.643,45.958,.02 from dual union all
select 567,'GIK-5','LC45/1','launch point','','1977-01-01 00:00:00','','GIK-5 LC45/1','PU1, Ploshchadka 45L, GIK-5, Baykonur, Kazakstan','',63.622,45.926,.02 from dual union all
select 568,'GIK-5','LC45/2','launch point','','1977-01-01 00:00:00','','GIK-5 LC45/2','PU2, Ploshchadka 45P, GIK-5, Baykonur, Kazakstan','',63.627,45.93,.02 from dual union all
select 569,'GIK-5','LC51','launch point','','1961-01-01 00:00:00','1962-01-01 00:00:00','GIK-5 LC51','Ploshchadka 51, GIK-5, Baykonur, Kazakstan','',63.347,45.916,.015 from dual union all
select 57,'AOR11B','LP16','launch point','','1953-09-03 00:00:00','1953-09-03 00:00:00','AOR11 LP16','Atlantic Ocean Launch Site 11 Launch Point 16','Atlantic Ocean',-57.05,44.55,.01 from dual union all
select 570,'GIK-5','LC60/6','launch point','','1962-01-01 00:00:00','1966-01-01 00:00:00','GIK-5 LC60/6','PU6, Ploshchadka 60, GIK-5, Baykonur, Kazakstan','',63.88,46.002,.02 from dual union all
select 571,'GIK-5','LC60/7','launch point','','1962-01-01 00:00:00','1966-01-01 00:00:00','GIK-5 LC60/7','PU7, Ploshchadka 60, GIK-5, Baykonur, Kazakstan','',63.88,46.002,.02 from dual union all
select 572,'GIK-5','LC60/8','launch point','','1962-01-01 00:00:00','1966-01-01 00:00:00','GIK-5 LC60/8','PU8, Ploshchadka 60, GIK-5, Baykonur, Kazakstan','',63.88,46.002,.02 from dual union all
select 573,'GIK-5','LC67/21','launch point','','1965-01-01 00:00:00','1968-01-01 00:00:00','GIK-5 LC67/21','PU 21, Ploshchadka 67L, GIK-5, Baykonur, Kazakstan','',63.683,45.972,.02 from dual union all
select 574,'GIK-5','LC67/22','launch point','','1965-01-01 00:00:00','1968-01-01 00:00:00','GIK-5 LC67/22','PU 22, Ploshchadka 67P, GIK-5, Baykonur, Kazakstan','',63.683,45.972,.02 from dual union all
select 575,'GIK-5','LC69L','launch point','','1965-01-01 00:00:00','1968-01-01 00:00:00','GIK-5 LC69L','Ploshchadka 69L, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 576,'GIK-5','LC69P','launch point','','1965-01-01 00:00:00','1968-01-01 00:00:00','GIK-5 LC69P','Ploshchadka 69P, GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 577,'GIK-5','LC70','launch point','','1961-01-01 00:00:00','1965-01-01 00:00:00','GIK-5 LC70','Ploshchadka 70, GIK-5, Baykonur, Kazakstan','',63.102,46.029,.015 from dual union all
select 578,'GIK-5','LC70/10','launch point','','1961-01-01 00:00:00','1965-01-01 00:00:00','GIK-5 LC70','Ploshchadka 70, GIK-5, Baykonur, Kazakstan','',63.102,46.029,.015 from dual union all
select 579,'GIK-5','LC70/11','launch point','','1961-01-01 00:00:00','1965-01-01 00:00:00','GIK-5 LC70','Ploshchadka 70, GIK-5, Baykonur, Kazakstan','',63.102,46.029,.015 from dual union all
select 58,'AOR11B','LP25','launch point','','1953-09-04 00:00:00','1953-09-04 00:00:00','AOR11 LP25','Atlantic Ocean Launch Site 11 Launch Point 25','Atlantic Ocean',-63.92,43.17,.01 from dual union all
select 580,'GIK-5','LC70/9','launch point','','1961-01-01 00:00:00','1965-01-01 00:00:00','GIK-5 LC70','Ploshchadka 70, GIK-5, Baykonur, Kazakstan','',63.102,46.029,.015 from dual union all
select 581,'GIK-5','LC75','launch point','','1961-01-01 00:00:00','1965-01-01 00:00:00','GIK-5 LC75','Ploshchadka 75, GIK-5, Baykonur, Kazakstan','',63.207,45.956,.015 from dual union all
select 582,'GIK-5','LC75/12','launch point','','1961-01-01 00:00:00','1965-01-01 00:00:00','GIK-5 LC75','Ploshchadka 75, GIK-5, Baykonur, Kazakstan','',63.207,45.956,.015 from dual union all
select 583,'GIK-5','LC75/13','launch point','','1961-01-01 00:00:00','1965-01-01 00:00:00','GIK-5 LC75','Ploshchadka 75, GIK-5, Baykonur, Kazakstan','',63.207,45.956,.015 from dual union all
select 584,'GIK-5','LC75/14','launch point','','1961-01-01 00:00:00','1965-01-01 00:00:00','GIK-5 LC75','Ploshchadka 75, GIK-5, Baykonur, Kazakstan','',63.207,45.956,.015 from dual union all
select 585,'GIK-5','LC80/17','launch point','','1965-01-01 00:00:00','1970-01-01 00:00:00','GIK-5 LC80/17','Ploshchadka 80, GIK-5, Baykonur, Kazakstan','',64,46,.04 from dual union all
select 586,'GIK-5','LC80/18','launch point','','1965-01-01 00:00:00','1980-01-01 00:00:00','GIK-5 LC80/18','PU18, Ploshchadka 80 (later 140), GIK-5, Baykonur, Kazakstan','',63,46,.2 from dual union all
select 587,'GIK-5','LC81/23','launch point','','1965-01-01 00:00:00','','GIK-5 LC81/23','PU23, Ploshchadka 81L, GIK-5, Baykonur, Kazakstan','',62.983,46.059,.015 from dual union all
select 588,'GIK-5','LC81/24','launch point','','1965-01-01 00:00:00','','GIK-5 LC81/24','PU24, Ploshchadka 81P, GIK-5, Baykonur, Kazakstan','',62.99,46.067,.015 from dual union all
select 589,'GIK-5','LC90','launch point','','1965-01-01 00:00:00','','GIK-5 LC90','PU19 or PU20, PL 90, GIK-5, Baykonur, Kazakstan','',62.935,46.081,.015 from dual union all
select 59,'AOR11B','LP26','launch point','','1953-09-04 00:00:00','1953-09-04 00:00:00','AOR11 LP26','Atlantic Ocean Launch Site 11 Launch Point 26','Atlantic Ocean',-63.92,43,.01 from dual union all
select 590,'GIK-5','LC90/19','launch point','','1965-01-01 00:00:00','','GIK-5 LC90/19','PU19, Ploshchadka 90L, GIK-5, Baykonur, Kazakstan','',62.935,46.081,.015 from dual union all
select 591,'GIK-5','LC90/20','launch point','','1965-01-01 00:00:00','','GIK-5 LC90/20','PU20, Ploshchadka 90P, GIK-5, Baykonur, Kazakstan','',62.936,46.081,.015 from dual union all
select 592,'GIK-5','PL251','launch point','','1987-01-01 00:00:00','','GIK-5 PL251','Buran runway, GIK-5 Baykonur','',63.324,46.029,.02 from dual union all
select 593,'GIK-5','PU31','launch point','','1956-01-01 00:00:00','','GIK-5 PU31','PU31 (Ploshchadka unknown), GIK-5, Baykonur, Kazakstan','',63,46,.04 from dual union all
select 594,'GIK-5','PU32','launch point','','1956-01-01 00:00:00','','GIK-5 PU32','PU32 (Ploshchadka unknown), GIK-5, Baykonur, Kazakstan','',63,46,.04 from dual union all
select 595,'GIK-5','PU33','launch point','','1956-01-01 00:00:00','','GIK-5 PU33','PU33 (Ploshchadka unknown), GIK-5, Baykonur, Kazakstan','',63,46,.04 from dual union all
select 596,'GIK-5','','launch site','RU','1989-01-01 00:00:00','','GIK-5','GIK-5, Baykonur, Kazakstan','',63,46,.02 from dual union all
select 597,'GILB','','launch site','US','1964-01-01 00:00:00','1970-01-01 00:00:00','GILB  -','Gilson Butte, Utah','',-110.58,38.59,.05 from dual union all
select 598,'GILLAM','','launch site','CA','1972-01-15 00:00:00','1973-02-02 00:00:00','GILLA -','Gillam, Manitoba','',-96,56.1,1 from dual union all
select 599,'GIOIA','','launch site','I','','','GIOIA -','Jupiter sites, Gioia del Colle, Italy','',16.93,40.76,.01 from dual union all
select 6,'AKHT','','launch site','BG','1984-01-01 00:00:00','','Akhtopol','Akhtopol Meteorological Rocket Station','Akhtopol, Bulgaria',27.9537,42.0846,.0002 from dual union all
select 60,'AOR11B','','launch cruise','US','1953-08-01 00:00:00','1953-09-01 00:00:00','Eastwind 1953','USS Eastwind, 1953 Arctic expedition','Atlantic Ocean',-55.08,53.1,.01 from dual union all
select 600,'GIP-53','','launch site','RU','1994-06-01 00:00:00','1995-01-01 00:00:00','GIP-53','GIP-53, Plesetsk, Rossiya','',41.1,63,.5 from dual union all
select 601,'GMI','','launch site','NZ','2009-01-01 00:00:00','','GMI','Great Mercury Island','Great Mercury I., New Zealand',175.92,-36.58,.02 from dual union all
select 602,'GNIIP','LC(T2)','launch point','','1982-01-01 00:00:00','','GNIIP LC(T2)','15Zh44/15Zh60 SP, GNIIP, Plesetsk','',41.083,63.083,.3 from dual union all
select 603,'GNIIP','LC(T3)','launch point','','1982-01-01 00:00:00','','GNIIP LC(T3)','Topol''-M SP, GNIIP, Plesetsk','',41.092,63.1,.3 from dual union all
select 604,'GNIIP','LC(T4)','launch point','','1982-01-01 00:00:00','','GNIIP LC(T4)','15Zh44 SP, GNIIP, Plesetsk','',41.12,63.11,.3 from dual union all
select 605,'GNIIP','LC158','launch point','','1982-01-01 00:00:00','','GNIIP LC158','Ploshchadka 158, GNIIP, Plesetsk, Rossiya','',41.08,63.05,.3 from dual union all
select 606,'GNIIP','LC167','launch point','','1982-01-01 00:00:00','','GNIIP LC167','15Zh58 SP, GNIIP, Plesetsk     (ID uncertain)','',41.13,62.98,.3 from dual union all
select 607,'GNIIP','Yu-1','launch point','','1994-01-01 00:00:00','','GNIIP Yu-1','ShPU Ploshch. Yuzhnaya-1, , Plesetsk, Rossiya','',null,null,null from dual union all
select 608,'GNIIP','','launch site','RU','1995-01-01 00:00:00','','GNIIP -','GNIIP, Plesetsk, Rossiya','',41.1,63,.5 from dual union all
select 609,'GNIIP1','','launch site','RU','1993-01-01 00:00:00','1994-06-01 00:00:00','GNIIP -','GNIIP, Plesetsk, Rossiya','',41.1,63,.5 from dual union all
select 61,'AOR12','LP1','launch point','','1954-07-16 00:00:00','1954-07-16 00:00:00','AOR12 LP1','Atlantic Ocean Launch Site 12 Launch Point 1','Atlantic Ocean',-66.7,42.1,.01 from dual union all
select 610,'GNIIPV','','launch site','RU','1993-01-01 00:00:00','1994-11-01 00:00:00','GNIIP','GNIIP, VKS section, Plesetsk, Rossiya','',40.5,62.9,.02 from dual union all
select 611,'GOSEONG','','launch site','KR','2017-07-01 00:00:00','','Goseong','Goseong, South Korea','',128.4183,38.5226,.0001 from dual union all
select 612,'GRLC','Pad 1','launch point','','1964-02-10 00:00:00','1973-08-24 00:00:00','GRLC  Pad 1','Pad 1, Green River, Utah','',-110.1,38.9,.1 from dual union all
select 613,'GRLC','Pad 2','launch point','','1964-02-10 00:00:00','1973-08-24 00:00:00','GRLC  Pad 2','Pad 2, Green River, Utah','',-110.1,38.9,.1 from dual union all
select 614,'GRLC','Pad 3','launch point','','1964-02-10 00:00:00','1973-08-24 00:00:00','GRLC  Pad 3','Pad 3, Green River, Utah','',-110.1,38.9,.1 from dual union all
select 615,'GRLC','','launch site','US','1964-02-10 00:00:00','1973-08-24 00:00:00','GRU   -','Green River, Utah','',-110.13,38.97,.05 from dual union all
select 616,'GRLY','5IN','launch point','','1971-08-07 00:00:00','1971-08-14 00:00:00','GRLY  5IN','HARP 5-inch Gun, Fort Greely, Alaska','',-146,64,.25 from dual union all
select 617,'GRLY','','launch site','US','1971-08-07 00:00:00','1971-08-14 00:00:00','GRLY  -','Fort Greely, Alaska','',-146,64,.25 from dual union all
select 618,'GTI','','launch site','TC','','','GTI','Grand Turk Auxiliary AFB, Grand Turk Island','',-71.142,21.444,.005 from dual union all
select 619,'GTIDZ','','launch zone','US','','','GTIDZ -','Grand Turk Island drop zone','',-71,21,2 from dual union all
select 62,'AOR12','LP10','launch point','','1954-07-23 00:00:00','1954-07-24 00:00:00','AOR12 LP10','Atlantic Ocean Launch Site 12 Launch Point 10','Atlantic Ocean',-53.92,58.85,.01 from dual union all
select 620,'GTsMP-4','','launch site','SU','1999-01-01 00:00:00','','GTsMP-4','GTsMP-4 MO RF, Kapustin Yar, Rossiya','',45.8,48.5,.1 from dual union all
select 621,'GTsP-4','LC107','launch point','','1973-01-01 00:00:00','','GTsP- LC107','Ploshchadka  107 (PU 1 or 2), GTsP-4, Kapustin Yar','',46.3,48.6,.02 from dual union all
select 622,'GTsP-4','LC107/1','launch point','','1973-01-01 00:00:00','','GTsP- LC107/1','Ploshchadka  107/1, GTsP-4, Kapustin Yar','',46.3001,48.5406,.0002 from dual;
insert into SITE
select 623,'GTsP-4','LC107/2','launch point','','1973-01-01 00:00:00','','GTsP- LC107/2','Ploshchadka  107/2, GTsP-4, Kapustin Yar','',46.2978,48.54,.0002 from dual union all
select 624,'GTsP-4','LC84','launch point','','1964-12-01 00:00:00','1967-12-19 00:00:00','GTsP- LC84','Ploshchadka  84 GTsP-4, Kapustin Yar','',46.25,48.55,.02 from dual union all
select 625,'GTsP-4','LC86','launch point','','1964-12-01 00:00:00','1967-12-19 00:00:00','GTsP- LC86','Ploshchadka  86 (PU 1 or 4), GTsP-4, Kapustin Yar','',46.25,48.55,.02 from dual union all
select 626,'GTsP-4','LC86/1','launch point','','1964-12-01 00:00:00','1967-12-19 00:00:00','GTsP- LC86/1','Ploshchadka  86/1, GTsP-4, Kapustin Yar','',46.2948,48.5688,.0002 from dual union all
select 627,'GTsP-4','LC86/4','launch point','','1967-12-26 00:00:00','1973-04-19 00:00:00','GTsP- LC86/4','Ploshchadka  86/4, GTsP-4, Kapustin Yar','',46.2957,48.5568,.0002 from dual union all
select 628,'GTsP-4','MR-100','launch point','','1960-01-01 00:00:00','','GTsP- MR-100','MR-100 launch area, GTsP-4, Kapustin Yar','',46.23,48.67,.02 from dual union all
select 629,'GTsP-4','Mayak-1','launch point','','1959-12-21 00:00:00','1964-10-24 00:00:00','GTsP- Mayak-1','Mayak-1 silo, GTsP-4, Kapustin Yar','',46.25,48.55,.1 from dual union all
select 63,'AOR12','LP11','launch point','','1954-07-23 00:00:00','1954-07-25 00:00:00','AOR12 LP11','Atlantic Ocean Launch Site 12 Launch Point 11','Atlantic Ocean',-53.72,59.2,.01 from dual union all
select 630,'GTsP-4','Mayak-2','launch point','','1959-12-21 00:00:00','1964-10-24 00:00:00','GTsP- Mayak-2','Mayak-2 silo, GTsP-4, Kapustin Yar','',46.2839,48.6841,.0002 from dual union all
select 631,'GTsP-4','OKB1','launch point','','1946-01-01 00:00:00','1970-01-01 00:00:00','GTsP- OKB1','Korolev launch area, GTsP-4, Kapustin Yar','',45.75,48.75,.02 from dual union all
select 632,'GTsP-4','PL87','launch point','','1946-01-01 00:00:00','','GTsP- PL87','Ploshchadka 87, GTsP-4, Kapustin Yar','',45.8,48.5,.1 from dual union all
select 633,'GTsP-4','SM-49','launch point','','1955-01-01 00:00:00','1960-01-01 00:00:00','GTsP- SM-49','SM-49 submarine simulator, Kapustin Yar','',45.8,48.5,.1 from dual union all
select 634,'GTsP-4','SP-2','launch point','','1955-01-01 00:00:00','1960-01-01 00:00:00','GTsP- SP-2','Unknown site for R-2A launches, near KY','',45.8,48.5,1 from dual union all
select 635,'GTsP-4','ShPU','launch point','','1960-01-01 00:00:00','1970-01-01 00:00:00','GTsP- silo','Unknown silo, GTsP-4, Kapustin Yar','',46.2,48.6,.2 from dual union all
select 636,'GTsP-4','','launch site','SU','1946-01-01 00:00:00','1999-01-01 00:00:00','GTsP-4','GTsP-4, Kapustin Yar, Volgograd, Rossiya','',45.8,48.5,.1 from dual union all
select 637,'GUAM','','launch site','GU','1957-02-12 00:00:00','1958-11-24 00:00:00','GUAM  -','Guam, Marianas Islands','',144.9,13.6,.25 from dual union all
select 638,'HAD','LM10-DS','launch point','','1971-08-02 00:00:00','1971-08-02 00:00:00','HAD   LM10-DS','LM 10 Descent Stage, Hadley Base, Montes Apenninus, Montes Apenninus B4','Luna',3.66,26.08,.01 from dual union all
select 639,'HAD','','launch site','LUNA','1971-08-02 00:00:00','1971-08-02 00:00:00','HAD   LM10-DS','LM 10 Descent Stage, Hadley Base, Montes Apenninus, Montes Apenninus B4','Luna',3.66,26.08,.01 from dual union all
select 64,'AOR12','LP12','launch point','','1954-07-23 00:00:00','1954-07-25 00:00:00','AOR12 LP12','Atlantic Ocean Launch Site 12 Launch Point 12','Atlantic Ocean',-59.37,59.37,.01 from dual union all
select 640,'HADC','A','launch point','','1949-12-02 00:00:00','','HADC  A','Holloman Air Development Center, Aerobee tower','',-106.1228,32.8931,.001 from dual union all
select 641,'HADC','LP1','launch point','','1949-01-01 00:00:00','1951-01-01 00:00:00','HADC  LP1','Unidentified pad at HADC, possibly NATIV? ACC property 1139','',-106.1272,32.8892,.0001 from dual union all
select 642,'HADC','LP2','launch point','','1949-01-01 00:00:00','1951-01-01 00:00:00','HADC  LP2','Unidentified pad at HADC, possibly NATIV? ACC property 1142','',-106.1277,32.8844,.0001 from dual union all
select 643,'HADC','NATIV','launch point','','1949-12-02 00:00:00','','HADC  NATIV','Holloman Air Development Center, NATIV pad (=LP1?)','',-106.1272,32.8892,.01 from dual union all
select 644,'HADC','SLED','launch point','','1949-12-02 00:00:00','','HADC  SLED','Holloman Air Development Center, Sled Test Track','',-106.1499,32.8848,.0001 from dual union all
select 645,'HADC','ZEL','launch point','','1949-12-02 00:00:00','','HADC  ZEL','Holloman Air Development Center, Able-51/ZEL for Matador/Mace','',-106.1475,32.8665,.0001 from dual union all
select 646,'HADC','','launch site','US','1949-12-02 00:00:00','1957-09-01 00:00:00','HADC  -','Holloman Air Development Center, New Mexico','',-106.103,32.8496,.0001 from dual union all
select 647,'HAI','','launch site','CN','1988-01-01 00:00:00','','HAI   -','Haikou Space Center, Hainan','',109.1239,19.5303,.0002 from dual union all
select 648,'HAMA','','launch site','SY','1992-07-01 00:00:00','','SYRIA HAMA','Hama missile base, Syria','',36.7107,35.1211,.001 from dual union all
select 649,'HAMNC','','launch site','US','1997-01-01 00:00:00','1997-01-01 00:00:00','HAM','Hampstead, North Carolina, 1 mi W of Topsail Beach','',-77.64,34.36,.05 from dual union all
select 65,'AOR12','LP13','launch point','','1954-07-23 00:00:00','1954-07-25 00:00:00','AOR12 LP13','Atlantic Ocean Launch Site 12 Launch Point 13','Atlantic Ocean',-54.62,59.3,.01 from dual union all
select 650,'HARBIN','','launch site','CN','1967-01-01 00:00:00','','Harbin','Harbin, Heilongjiang, Manchuria, China','',126.68,45.75,.02 from dual union all
select 651,'HH','A1','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','HH    A1','Hidden Hills drop zone A1, California','',-115.77,36.35,.02 from dual union all
select 652,'HH','A2','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','HH    A2','Hidden Hills drop zone A2, California','',-115.98,36.33,.02 from dual union all
select 653,'HH','A3','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','HH    A3','Hidden Hills drop zone A3, California','',-116.03,36.37,.02 from dual union all
select 654,'HH','A4','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','HH    A4','Hidden Hills drop zone A4, California','',-116.18,36.33,.02 from dual union all
select 655,'HH','A5','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','HH    A5','Hidden Hills drop zone A5, California','',-116.27,36.3,.02 from dual union all
select 656,'HH','B1','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','HH    B1','Hidden Hills drop zone B1, California','',-115.66,35.94,.02 from dual union all
select 657,'HH','B2','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','HH    B2','Hidden Hills drop zone B2, California','',-115.65,35.97,.02 from dual union all
select 658,'HH','B3','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','HH    B3','Hidden Hills drop zone B3, California','',-115.65,35.92,.02 from dual union all
select 659,'HH','RW14/32','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','HH    RW14/32','RW14/32, Hidden Hills Dry Lake, California','',-115.95,36,.02 from dual union all
select 66,'AOR12','LP14','launch point','','1954-07-19 00:00:00','1954-07-26 00:00:00','AOR12 LP14','Atlantic Ocean Launch Site 12 Launch Point 14','Atlantic Ocean',-53.5,54,.01 from dual union all
select 660,'HH','','launch site','US','1959-01-01 00:00:00','1967-01-01 00:00:00','Hidden Hills','RW14/32, Hidden Hills Dry Lake, California','',-115.95,36,.02 from dual union all
select 661,'HKR','','launch site','DR','1944-01-01 00:00:00','1945-01-01 00:00:00','HKR   -','Heidekraut, Tucheler Heide, Deutschland (now Polska)','',18,53.6,.3 from dual union all
select 662,'HLG','','launch site','PL','1943-11-01 00:00:00','1944-08-01 00:00:00','HLG   -','Truppenubungsplatz Heidelager, Blizna, Krakow, Polska','',21.8,50.2,.2 from dual union all
select 663,'HMG','Agate','launch point','','1952-01-01 00:00:00','1967-01-01 00:00:00','HMG   Brigitte/A','Agate launcher, Brigitte, CIEES B2, Hammaguir, Algerie','',-3.07,30.9,.05 from dual union all
select 664,'HMG','Bacchus','launch point','','1952-01-01 00:00:00','1967-01-01 00:00:00','HMG   Bacchus','Bacchus, CIEES B2, Hammaguir, Algerie','',-3.07,30.87,.05 from dual union all
select 665,'HMG','Beatrice','launch point','','1952-01-01 00:00:00','1967-01-01 00:00:00','HMG   Beatrice','Beatrice, CIEES B2, Hammaguir, Algerie','',-3.07,30.87,.05 from dual union all
select 666,'HMG','Bechar','launch point','','1952-05-20 00:00:00','1965-11-18 00:00:00','HMG   Bechar','Bou Hammadi, CIEES B0, Colomb-Bechar, Algerie','',-2.3,31.72,.01 from dual union all
select 667,'HMG','Beni','launch point','','1949-01-01 00:00:00','1967-01-01 00:00:00','HMG   Beni','Beni Abbes, Algerie','',-3.083,30.887,2 from dual union all
select 668,'HMG','Blandine','launch point','','1952-01-01 00:00:00','1967-01-01 00:00:00','HMG   Blandine','Blandine, CIEES B2, Hammaguir, Algerie','',-3.07,30.87,.05 from dual union all
select 669,'HMG','Brigitte','launch point','','1952-01-01 00:00:00','1967-01-01 00:00:00','HMG   Brigitte','Brigitte, CIEES B2, Hammaguir, Algerie','',-3.07,30.9,.05 from dual union all
select 67,'AOR12','LP15','launch point','','1954-07-19 00:00:00','1954-07-26 00:00:00','AOR12 LP15','Atlantic Ocean Launch Site 12 Launch Point 15','Atlantic Ocean',-53.38,55.22,.01 from dual union all
select 670,'HMG','Leger','launch point','','1949-01-01 00:00:00','1967-01-01 00:00:00','HMG   Leger','Base Georges Leger, CIEES B1, Hameida Hammada, Colomb-Bechar, Algerie','',-2.22,31.62,.01 from dual union all
select 671,'HMG','','launch site','DZ','1949-01-01 00:00:00','1967-01-01 00:00:00','HMG   -','Centre Interarmees d''Essais d''Engins Speciaux (CIEES), Algerie','',-3.083,30.887,.02 from dual union all
select 672,'HULU','','launch site','CN','1990-01-01 00:00:00','','Huludao','Huludao Shipyard test site','Huludao, Liaonning',120.8,40.7,.5 from dual union all
select 673,'HVP','GWO','launch point','','1944-01-01 00:00:00','1945-01-01 00:00:00','HVP   GWO','Greifswalder Oie, Mecklenburg-Vorpommern, Deutschland','',13.92,54.25,.02 from dual union all
select 674,'HVP','P10','launch point','','1942-01-01 00:00:00','1945-01-01 00:00:00','HVP   P10','Launch Site P10, Heersversuchsstelle Peenemunde','',13.815,54.162,.05 from dual union all
select 675,'HVP','P7','launch point','','1942-01-01 00:00:00','1945-01-01 00:00:00','HVP   P7','Test Stand P7, Heersversuchsstelle Peenemunde','',13.801,54.173,.01 from dual union all
select 676,'HVP','SK','launch point','','1942-01-01 00:00:00','1945-01-01 00:00:00','HVP   SK','Siedlung Kalrshagen, Heersversuchsstelle Peenemunde','',13.801,54.173,.05 from dual union all
select 677,'HVP','','launch site','DR','1942-01-01 00:00:00','1945-01-01 00:00:00','HVP   -','Heersversuchsstelle Peenemunde, Usedom, Rostock, Mecklenburg-Vorpommern','',13.801,54.173,.01 from dual union all
select 678,'HWJU','','launch site','KP','2016-01-01 00:00:00','','Hwangju','Scud launch site near Hwangju, N Hwanghae Prov.','',125.702,38.6868,.0001 from dual union all
select 679,'IGY01','LP1','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP1','IGY Rockoon Launch Site 1, Atlantic Ocean LP1','',-54,55,.02 from dual union all
select 68,'AOR12','LP16','launch point','','1954-07-19 00:00:00','1954-07-26 00:00:00','AOR12 LP16','Atlantic Ocean Launch Site 12 Launch Point 16','Atlantic Ocean',-53.02,59.55,.01 from dual union all
select 680,'IGY01','LP10','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP10','IGY Rockoon Launch Site 1, Atlantic Ocean LP10','',-57,70.4,.02 from dual union all
select 681,'IGY01','LP11','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP11','IGY Rockoon Launch Site 1, Atlantic Ocean LP11','',-54.93,66.67,.02 from dual union all
select 682,'IGY01','LP12','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP12','IGY Rockoon Launch Site 1, Atlantic Ocean LP12','',-53.95,64.68,.02 from dual union all
select 683,'IGY01','LP13','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP13','IGY Rockoon Launch Site 1, Atlantic Ocean LP13','',-53.72,63.05,.02 from dual union all
select 684,'IGY01','LP14','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP14','IGY Rockoon Launch Site 1, Atlantic Ocean LP14','',-49.27,60.58,.02 from dual union all
select 685,'IGY01','LP15','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP15','IGY Rockoon Launch Site 1, Atlantic Ocean LP15','',-47.8,59.3,.02 from dual union all
select 686,'IGY01','LP16','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP16','IGY Rockoon Launch Site 1, Atlantic Ocean LP16','',-48.15,57.73,.02 from dual union all
select 687,'IGY01','LP17','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP17','IGY Rockoon Launch Site 1, Atlantic Ocean LP17','',-48.77,56.22,.02 from dual union all
select 688,'IGY01','LP18','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP18','IGY Rockoon Launch Site 1, Atlantic Ocean LP18','',-48.93,55.67,.02 from dual union all
select 689,'IGY01','LP2','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP2','IGY Rockoon Launch Site 1, Atlantic Ocean LP2','',-54.42,56.62,.02 from dual union all
select 69,'AOR12','LP17','launch point','','1954-07-19 00:00:00','1954-07-26 00:00:00','AOR12 LP17','Atlantic Ocean Launch Site 12 Launch Point 17','Atlantic Ocean',-54.98,58.37,.01 from dual union all
select 690,'IGY01','LP3','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP3','IGY Rockoon Launch Site 1, Atlantic Ocean LP3','',-56.28,62.27,.02 from dual union all
select 691,'IGY01','LP4','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP4','IGY Rockoon Launch Site 1, Atlantic Ocean LP4','',-56.15,63.93,.02 from dual union all
select 692,'IGY01','LP5','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP5','IGY Rockoon Launch Site 1, Atlantic Ocean LP5','',-57.05,67.33,.02 from dual union all
select 693,'IGY01','LP6','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP6','IGY Rockoon Launch Site 1, Atlantic Ocean LP6','',-58.52,72.92,.02 from dual union all
select 694,'IGY01','LP7','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP7','IGY Rockoon Launch Site 1, Atlantic Ocean LP7','',-63.32,75.2,.02 from dual union all
select 695,'IGY01','LP8','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP8','IGY Rockoon Launch Site 1, Atlantic Ocean LP8','',-63.57,74.3,.02 from dual union all
select 696,'IGY01','LP9','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY01 LP9','IGY Rockoon Launch Site 1, Atlantic Ocean LP9','',-58.33,72.07,.02 from dual union all
select 697,'IGY01','','launch cruise','US','1957-08-01 00:00:00','1957-08-20 00:00:00','IGY01 -','1957 NRL USS Plymouth Rock Cruise, Atlantic Ocean','',-54,55,.02 from dual union all
select 698,'IGY02','LP1','launch point','','1957-09-23 00:00:00','1957-11-10 00:00:00','IGY02 -','1957 NRL USS Glacier cruise, Atlantic and Pacific Oceans','',-.99,.83,.02 from dual union all
select 699,'IGY02','LP10','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY11 -','IGY Rockoon Launch Site 11, Pacific Ocean','',-158.012,3.38,.02 from dual union all
select 7,'AKT','','launch site','J','1956-09-24 00:00:00','1962-05-24 00:00:00','AKT   -','Akita Rocket Range (ATC)','Michikawa, Akita City, Honshu, Nippon',140.06,39.57,.01 from dual union all
select 70,'AOR12','LP18','launch point','','1954-07-19 00:00:00','1954-07-26 00:00:00','AOR12 LP18','Atlantic Ocean Launch Site 12 Launch Point 18','Atlantic Ocean',-56.18,58.67,.01 from dual union all
select 700,'IGY02','LP11','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY12 -','IGY Rockoon Launch Site 12, Pacific Ocean','',-159.92,1.87,.02 from dual union all
select 701,'IGY02','LP12','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY13 -','IGY Rockoon Launch Site 13, Pacific Ocean','',-160.53,.8,.02 from dual union all
select 702,'IGY02','LP13','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY14 -','IGY Rockoon Launch Site 14, Pacific Ocean','',-160.87,.35,.02 from dual union all
select 703,'IGY02','LP14','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY15 -','IGY Rockoon Launch Site 15, Pacific Ocean','',-161.08,-2.08,.02 from dual union all
select 704,'IGY02','LP15','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY16 -','IGY Rockoon Launch Site 16, Pacific Ocean','',-161.42,-2.52,.02 from dual union all
select 705,'IGY02','LP16','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY17 -','IGY Rockoon Launch Site 17, Pacific Ocean','',-161.47,-2.83,.02 from dual union all
select 706,'IGY02','LP17','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY18 -','IGY Rockoon Launch Site 18, Pacific Ocean','',-162.17,-6.57,.02 from dual union all
select 707,'IGY02','LP18','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY19 -','IGY Rockoon Launch Site 19, Pacific Ocean','',-163.4,-17.7,.02 from dual union all
select 708,'IGY02','LP19','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY20 -','IGY Rockoon Launch Site 20, Pacific Ocean','',-165.98,-39.07,.02 from dual union all
select 709,'IGY02','LP2','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY03 -','IGY Rockoon Launch Site 3, Atlantic Ocean','',-74.4,25.33,.02 from dual union all
select 71,'AOR12','LP19','launch point','','1954-07-19 00:00:00','1954-07-26 00:00:00','AOR12 LP19','Atlantic Ocean Launch Site 12 Launch Point 19','Atlantic Ocean',-56.37,60.05,.01 from dual union all
select 710,'IGY02','LP20','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY21 -','IGY Rockoon Launch Site 21, Pacific Ocean','',-166.18,-40.45,.02 from dual union all
select 711,'IGY02','LP21','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY22 -','IGY Rockoon Launch Site 22, Pacific Ocean','',-169.65,-50.72,.02 from dual union all
select 712,'IGY02','LP22','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY23 -','IGY Rockoon Launch Site 23, Pacific Ocean','',-176.1,-60.08,.02 from dual;
insert into SITE
select 713,'IGY02','LP23','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY24 -','IGY Rockoon Launch Site 24, Pacific Ocean','',-176.63,-60.73,.02 from dual union all
select 714,'IGY02','LP24','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY25 -','IGY Rockoon Launch Site 25, Pacific Ocean','',-177.85,-65.05,.02 from dual union all
select 715,'IGY02','LP25','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY26 -','IGY Rockoon Launch Site 26, Pacific Ocean','',-177.63,-65.38,.02 from dual union all
select 716,'IGY02','LP26','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY27 -','IGY Rockoon Launch Site 27, Pacific Ocean','',175.7,-70.23,.02 from dual union all
select 717,'IGY02','LP27','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY28 -','IGY Rockoon Launch Site 28, Pacific Ocean','',175.83,-70.8,.02 from dual union all
select 718,'IGY02','LP28','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY29 -','IGY Rockoon Launch Site 29, Pacific Ocean','',176.13,-71.17,.02 from dual union all
select 719,'IGY02','LP29','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY30 -','IGY Rockoon Launch Site 30, Pacific Ocean','',176.75,-71.47,.02 from dual union all
select 72,'AOR12','LP2','launch point','','1954-07-19 00:00:00','1954-07-19 00:00:00','AOR12 LP2','Atlantic Ocean Launch Site 12 Launch Point 2','Atlantic Ocean',-53.5,54.63,.01 from dual union all
select 720,'IGY02','LP3','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY04 -','IGY Rockoon Launch Site 4, Atlantic Ocean','',-89.48,5.47,.02 from dual union all
select 721,'IGY02','LP30','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY31 -','IGY Rockoon Launch Site 31, Pacific Ocean','',176.67,-71.97,.02 from dual union all
select 722,'IGY02','LP31','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY32 -','IGY Rockoon Launch Site 32, Pacific Ocean','',173.55,-70.85,.02 from dual union all
select 723,'IGY02','LP32','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY33 -','IGY Rockoon Launch Site 33, Pacific Ocean','',173.63,-69.53,.02 from dual union all
select 724,'IGY02','LP33','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY34 -','IGY Rockoon Launch Site 34, Pacific Ocean','',174.62,-66.58,.02 from dual union all
select 725,'IGY02','LP34','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY35 -','IGY Rockoon Launch Site 35, Pacific Ocean','',174.68,-66.15,.02 from dual union all
select 726,'IGY02','LP35','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY36 -','IGY Rockoon Launch Site 36, Pacific Ocean','',173.92,-48.3,.02 from dual union all
select 727,'IGY02','LP36','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY37 -','IGY Rockoon Launch Site 37, Pacific Ocean','',173.83,-48.02,.02 from dual union all
select 728,'IGY02','LP4','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY05 -','IGY Rockoon Launch Site 5, Pacific Ocean','',-143.13,2.17,.02 from dual union all
select 729,'IGY02','LP5','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY06 -','IGY Rockoon Launch Site 6, Pacific Ocean','',-150.38,2.27,.02 from dual union all
select 73,'AOR12','LP3','launch point','','1954-07-21 00:00:00','1954-07-21 00:00:00','AOR12 LP3','Atlantic Ocean Launch Site 12 Launch Point 3','Atlantic Ocean',-54.22,59.38,.01 from dual union all
select 730,'IGY02','LP6','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY07 -','IGY Rockoon Launch Site 7, Pacific Ocean','',-156.63,7.12,.02 from dual union all
select 731,'IGY02','LP7','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY08 -','IGY Rockoon Launch Site 8, Pacific Ocean','',-156.92,6.47,.02 from dual union all
select 732,'IGY02','LP8','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY09 -','IGY Rockoon Launch Site 9, Pacific Ocean','',-157.02,6.18,.02 from dual union all
select 733,'IGY02','LP9','launch point','','1957-01-01 00:00:00','1958-01-01 00:00:00','IGY10 -','IGY Rockoon Launch Site 10, Pacific Ocean','',-158.012,3.93,.02 from dual union all
select 734,'IGY02','','launch cruise','US','1957-09-23 00:00:00','1957-11-10 00:00:00','IGY02 -','1957 NRL USS Glacier cruise, Atlantic and Pacific Oceans','',-.99,.83,.02 from dual union all
select 735,'IOR1','','launch cruise','SU','1976-01-01 00:00:00','1976-01-01 00:00:00','IOR1  -','Vise Expedition 1976,  IOR 15S 63E','',63,-15,1 from dual union all
select 736,'IOR2','LP1','launch point','','1967-01-01 00:00:00','1967-01-01 00:00:00','IOR2  LP1','Vokyekov Expedition 1967, IOR  Launch Position 1','',87.283,-.53,.01 from dual union all
select 737,'IOR2','LP2','launch point','','1967-01-01 00:00:00','1967-01-01 00:00:00','IOR2  LP2','Vokyekov Expedition 1967, IOR  Launch Position 2','',61.166,-13.02,.01 from dual union all
select 738,'IOR2','LP3','launch point','','1967-01-01 00:00:00','1967-01-01 00:00:00','IOR2  LP3','Vokyekov Expedition 1967, IOR  Launch Position 3','',60.083,-39.467,.01 from dual union all
select 739,'IOR2','LP4','launch point','','1967-01-01 00:00:00','1967-01-01 00:00:00','IOR2  LP4','Vokyekov Expedition 1967, IOR  Launch Position 4','',59.833,-47.717,.01 from dual union all
select 74,'AOR12','LP4','launch point','','1954-07-21 00:00:00','1954-07-21 00:00:00','AOR12 LP4','Atlantic Ocean Launch Site 12 Launch Point 4','Atlantic Ocean',-54.45,59.6,.01 from dual union all
select 740,'IOR2','LP5','launch point','','1967-01-01 00:00:00','1967-01-01 00:00:00','IOR2  LP5','Vokyekov Expedition 1967, IOR  Launch Position 5','',59.8,-56.133,.01 from dual union all
select 741,'IOR2','LP6','launch point','','1967-01-01 00:00:00','1967-01-01 00:00:00','IOR2  LP6','Vokyekov Expedition 1967, IOR  Launch Position 6','',60.167,-60.033,.01 from dual union all
select 742,'IOR2','LP7','launch point','','1967-01-01 00:00:00','1967-01-01 00:00:00','IOR2  LP7','Vokyekov Expedition 1967, IOR  Launch Position 7','',64.983,-20.033,.01 from dual union all
select 743,'IOR2','','launch cruise','SU','1967-01-01 00:00:00','1967-01-01 00:00:00','IOR2','Vokyekov Expedition 1967, IOR  Launch Position 1','',87.283,-.53,.01 from dual union all
select 744,'IOR3','','launch cruise','SU','1970-01-01 00:00:00','1970-01-01 00:00:00','IOR3  -','Shokalsky Expedition 1970, near Kerguelen','',65,-50,1 from dual union all
select 745,'IQ160','','launch site','IQ','1991-01-01 00:00:00','1991-01-01 00:00:00','Km 160','Kilometer 160, Ar Ramadi, Al Ambar, Iraq','Ar Ramadi, Al Anbar, Iraq',43.1,33.4,.5 from dual union all
select 746,'IQAN','','launch site','IQ','1991-01-01 00:00:00','1991-01-01 00:00:00','Al Nikheb','Al Nikheb (Al Nuqaub)','Al Nikheb, Al Anbar, Iraq',42.26,32.03,.01 from dual union all
select 747,'IQBA','N','launch point','','2003-01-01 00:00:00','2003-01-01 00:00:00','Basra N','Mobile launchers N of Basra','Basra, Iraq',47.82,30.5,.5 from dual union all
select 748,'IQBA','NQ','launch point','','2003-01-01 00:00:00','2003-01-01 00:00:00','Basra-Qurnah','Mobile launcher, Qurnah, N of Basra','Basra, Iraq',47.43,31.02,.1 from dual union all
select 749,'IQBA','NW','launch point','','2003-01-01 00:00:00','2003-01-01 00:00:00','Basra NW','Mobile launchers NW of Basra','Basra, Iraq',47.82,30.5,.5 from dual union all
select 75,'AOR12','LP5','launch point','','1954-07-21 00:00:00','1954-07-21 00:00:00','AOR12 LP5','Atlantic Ocean Launch Site 12 Launch Point 5','Atlantic Ocean',-52.47,59.55,.01 from dual union all
select 750,'IQBA','S','launch point','','2003-01-01 00:00:00','2003-01-01 00:00:00','Basra S','Mobile launchers S of Basra','Basra, Iraq',47.82,30.5,.5 from dual union all
select 751,'IQBA','W','launch point','','2003-01-01 00:00:00','2003-01-01 00:00:00','Basra W','Mobile launchers W of Basra','Basra, Iraq',47.82,30.5,.5 from dual union all
select 752,'IQBA','','launch site','IQ','2003-01-01 00:00:00','2003-01-01 00:00:00','Basra','Mobile launchers in Basra region','Basra, Iraq',47.8,30.5,.2 from dual union all
select 753,'IQH3','','launch site','IQ','2003-01-01 00:00:00','2003-01-01 00:00:00','H3 Airbase','H3 Airbase, Al Anbar','Al Anbar, Iraq',39.74,32.93,.01 from dual union all
select 754,'IQKK','','launch site','IQ','2003-01-01 00:00:00','2003-01-01 00:00:00','Kirkuk','Kirkuk region, Iraq','Kirkuk, Iraq',44.4,35.47,.2 from dual union all
select 755,'IQNJ','','launch site','IQ','2003-01-01 00:00:00','2003-01-01 00:00:00','IQNJ','Mobile launcher, S of An Najaf','An Najaf, Iraq',44.1,31,.2 from dual union all
select 756,'IQSA','','launch site','IQ','2003-01-01 00:00:00','2003-01-01 00:00:00','IQTA','Saddam Air Base','Al Qayyarah, Iraq',43.12,35.76,.01 from dual union all
select 757,'IQTA','','launch site','IQ','2003-01-01 00:00:00','2003-01-01 00:00:00','IQTA','Tall'' Afar Air Base','Tall'' Afar, Mosul, Iraq',42.41,36.28,.01 from dual union all
select 758,'IQUQ','','launch site','IQ','2003-01-01 00:00:00','2003-01-01 00:00:00','IQUQ','Umm Qasr (Um Qassir)','Umm Qasr (Um Qassir), Shatt Al Arab, Iraq',47.95,30.05,.05 from dual union all
select 759,'IRAN','','launch site','IR','','1985-01-01 00:00:00','IRAN  -','Mobile launchers in Iran?','',49,34,4 from dual union all
select 76,'AOR12','LP6','launch point','','1954-07-23 00:00:00','1954-07-23 00:00:00','AOR12 LP6','Atlantic Ocean Launch Site 12 Launch Point 6','Atlantic Ocean',-53.62,59.53,.01 from dual union all
select 760,'IRAQ','','launch site','IQ','','1991-01-01 00:00:00','IRAQ  -','Mobile launchers in Iraq','',44,33,4 from dual union all
select 761,'IRAQ-A','','launch site','IQ','','1991-01-01 00:00:00','IRAQ- -','Mobile launchers in Iraq, near Al Amarah','',47.1,31.8,.2 from dual union all
select 762,'IRAQ-W','','launch site','IQ','1991-01-01 00:00:00','1991-01-01 00:00:00','IRAQ- -','Mobile launchers in Iraq, western launch area','',41,34,1 from dual union all
select 763,'IRKURD','','launch site','IR','2017-01-01 00:00:00','','Kurdistan','Missile base, Kurdistan province, Iran                                          -','',46,36,1.5 from dual union all
select 764,'IRW','','launch site','US','1944-01-01 00:00:00','1945-01-01 00:00:00','IRW   -','Leach Spring, Camp Irwin, California','',-116.7,35.3,.5 from dual union all
select 765,'IXC','','launch site','MX','1991-07-11 00:00:00','1991-07-11 00:00:00','IXC   -','Santiago Ixcuintla, Nayarit, Mexico','',-105.37,21.83,.02 from dual union all
select 766,'JHAM','S','launch point','','2003-01-01 00:00:00','2003-01-01 00:00:00','JHAM South Site','South Site, Jabal Hamzah','Jabal Hamzah, Egypt',30.6167,30.1003,.0002 from dual union all
select 767,'JHAM','','launch site','EG','1962-01-01 00:00:00','1973-01-01 00:00:00','Jabal Hamzah','Jabal Hamzah','Egypt',30.607,30.1274,.0002 from dual union all
select 768,'JHIM','G350','launch point','','1990-01-01 00:00:00','1990-01-01 00:00:00','Jebel Himreen','0.35m  gun, Jebel Himreen','Iraq',45.3,34,.5 from dual union all
select 769,'JHIM','','launch site','IQ','1990-01-01 00:00:00','1990-01-01 00:00:00','Jebel Himreen','Jebel Himreen, Iraq','Iraq',45.3,34,.5 from dual union all
select 77,'AOR12','LP7','launch point','','1954-07-23 00:00:00','1954-07-23 00:00:00','AOR12 LP7','Atlantic Ocean Launch Site 12 Launch Point 7','Atlantic Ocean',-52.35,59.63,.01 from dual union all
select 770,'JI','HAD23','launch point','','1958-01-01 00:00:00','1958-01-01 00:00:00','JI    HAD23','HAD Launcher 23, Johnston Island','',-169.524,16.737,.02 from dual union all
select 771,'JI','LC1','launch point','','1958-01-01 00:00:00','1958-01-01 00:00:00','JI    LC1','Redstone Pad 1, Johnston Island east end','',-169.5256,16.7336,.0002 from dual union all
select 772,'JI','LE1','launch point','','1962-01-01 00:00:00','1975-01-01 00:00:00','JI    LE1','Launch Emplacement 1, Johnston Island NW end  [Former VAFB LE-7?]','',-169.5379,16.7296,.0001 from dual union all
select 773,'JI','LE2','launch point','','1962-01-01 00:00:00','1975-01-01 00:00:00','JI    LE2','Launch Emplacement 2, Johnston Island NW end  [Former VAFB LE-8?]','',-169.5344,16.7321,.0001 from dual union all
select 774,'JI','Pad 1','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 1','Launcher  1, Johnston Island','',-169.53,16.73,.01 from dual union all
select 775,'JI','Pad 10','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 10','Launcher 10, Johnston Island','',-169.53,16.73,.01 from dual union all
select 776,'JI','Pad 11','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 11','Launcher 11, Johnston Island','',-169.53,16.73,.01 from dual union all
select 777,'JI','Pad 12','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 12','Launcher 12, Johnston Island','',-169.53,16.73,.01 from dual union all
select 778,'JI','Pad 13','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 13','Launcher 13, Johnston Island','',-169.53,16.73,.01 from dual union all
select 779,'JI','Pad 14','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 14','Launcher 14, Johnston Island','',-169.53,16.73,.01 from dual union all
select 78,'AOR12','LP8','launch point','','1954-07-23 00:00:00','1954-07-23 00:00:00','AOR12 LP8','Atlantic Ocean Launch Site 12 Launch Point 8','Atlantic Ocean',-52.75,59.6,.01 from dual union all
select 780,'JI','Pad 15','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 15','Launcher 15, Johnston Island','',-169.53,16.73,.01 from dual union all
select 781,'JI','Pad 16','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 16','Launcher 16, Johnston Island','',-169.53,16.73,.01 from dual union all
select 782,'JI','Pad 17','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 17','Launcher 17, Johnston Island','',-169.53,16.73,.01 from dual union all
select 783,'JI','Pad 18','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 18','Launcher 18, Johnston Island','',-169.53,16.73,.01 from dual union all
select 784,'JI','Pad 19','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 19','Launcher 19, Johnston Island','',-169.53,16.73,.01 from dual union all
select 785,'JI','Pad 2','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 2','Launcher  2, Johnston Island','',-169.53,16.73,.01 from dual union all
select 786,'JI','Pad 20','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 20','Launcher 20, Johnston Island','',-169.53,16.73,.01 from dual union all
select 787,'JI','Pad 21','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 21','Launcher 21, Johnston Island','',-169.53,16.73,.01 from dual union all
select 788,'JI','Pad 22','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 22','Launcher 22, Johnston Island','',-169.53,16.73,.01 from dual union all
select 789,'JI','Pad 23','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 23','Launcher 23, Johnston Island','',-169.53,16.73,.01 from dual union all
select 79,'AOR12','LP9','launch point','','1954-07-23 00:00:00','1954-07-24 00:00:00','AOR12 LP9','Atlantic Ocean Launch Site 12 Launch Point 9','Atlantic Ocean',-54.2,59.07,.01 from dual union all
select 790,'JI','Pad 24','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 24','Launcher 24, Johnston Island','',-169.53,16.73,.01 from dual union all
select 791,'JI','Pad 25','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 25','Launcher 25, Johnston Island','',-169.53,16.73,.01 from dual union all
select 792,'JI','Pad 26','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 26','Launcher 26, Johnston Island','',-169.53,16.73,.01 from dual union all
select 793,'JI','Pad 27','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 27','Launcher 27, Johnston Island','',-169.53,16.73,.01 from dual union all
select 794,'JI','Pad 3','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 3','Launcher  3, Johnston Island','',-169.53,16.73,.01 from dual union all
select 795,'JI','Pad 4','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 4','Launcher  4, Johnston Island','',-169.53,16.73,.01 from dual union all
select 796,'JI','Pad 5','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 5','Launcher  5, Johnston Island','',-169.53,16.73,.01 from dual union all
select 797,'JI','Pad 6','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 6','Launcher  6, Johnston Island','',-169.53,16.73,.01 from dual union all
select 798,'JI','Pad 7','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 7','Launcher  7, Johnston Island','',-169.53,16.73,.01 from dual union all
select 799,'JI','Pad 8','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 8','Launcher  8, Johnston Island','',-169.53,16.73,.01 from dual union all
select 8,'ALCA','HAD','launch point','','1994-08-19 00:00:00','','ALCA  HAD','HAD launcher, CL Alcantara','Alcantara, Maranhao, Brasil',-44.3677,-2.3147,.0001 from dual union all
select 80,'AOR12','','launch cruise','US','1954-07-16 00:00:00','1954-07-16 00:00:00','Eastwind 1954','USS Eastwind, 1954 Arctic expedition','Atlantic Ocean',-66.7,42.1,.01 from dual union all
select 800,'JI','Pad 9','launch point','','1962-01-01 00:00:00','1963-01-01 00:00:00','JI    Pad 9','Launcher  9, Johnston Island','',-169.53,16.73,.01 from dual union all
select 801,'JI','S','launch point','','1958-01-01 00:00:00','1975-01-01 00:00:00','JI    S','Johnston Island DOMINIC south launchers','',-169.5238,16.7374,.0001 from dual union all
select 802,'JI','UL6','launch point','','1958-01-01 00:00:00','1958-01-01 00:00:00','JI    UL6','Universal Launcher 6, Johnston Island','',-169.524,16.737,.02 from dual;
insert into SITE
select 803,'JI','','launch site','UM67','1958-01-01 00:00:00','1975-01-01 00:00:00','JI    -','Johnston Island','',-169.5238,16.7374,.0001 from dual union all
select 804,'JIAN','','launch site','CN','1970-01-01 00:00:00','1972-01-01 00:00:00','Jianshui','Jianshui (Ching-yu)','',102.82,23.62,.01 from dual union all
select 805,'JIANGG','','launch site','CN','1990-01-01 00:00:00','','Jianggezhuang','Jianggezhuang Submarine Base','Jianggezhuang',120.5789,36.1041,.0001 from dual union all
select 806,'JINZH','','launch site','CN','1990-01-01 00:00:00','','Jimzhou','Jinzhou missile base','',121.17,41.19,.02 from dual union all
select 807,'JLRPG','','launch site','US','1949-10-01 00:00:00','1950-05-16 00:00:00','JLRPG','Joint Long Range Proving Ground','Cape Canaveral, Florida',-80.54,28.46,.05 from dual union all
select 808,'JP','','launch site','TW','1999-01-01 00:00:00','','JP    -','Jiu Peng air base 7, Pingtung, Formosa','',120.92,22.13,.1 from dual union all
select 809,'JQ','LA2A','launch point','','1968-01-01 00:00:00','','JQ    LA2A','Launch Area 2 Pad 5020, North Launch Site, Jiuquan Space Center','',100.3133,41.3062,.0001 from dual union all
select 81,'AOR13','LP1','launch point','','1955-09-23 00:00:00','1955-09-23 00:00:00','AOR13 LP1','Atlantic Ocean Launch Site 13 Launch Point 1','Atlantic Ocean',-66.18,42.67,.01 from dual union all
select 810,'JQ','LA2B','launch point','','1968-01-01 00:00:00','','JQ    LA2B','Launch Area 2 Pad 138, North Launch Site, Jiuquan Space Center','',100.3188,41.3065,.0001 from dual union all
select 811,'JQ','LA3','launch point','','1968-01-01 00:00:00','','JQ    LA3','Launch Area 3A/B, Jiuquan Space Center','',100.3047,41.2805,.001 from dual union all
select 812,'JQ','LA3B','launch point','','1968-01-01 00:00:00','','JQ    LA3','Launch Area 3B, Jiuquan Space Center','',100.3048,41.2831,.001 from dual union all
select 813,'JQ','Pad 603','launch point','','2008-01-01 00:00:00','','JQ    SLS-2','Pad 603, South Launch Site-2, Jiuquan Space Center','',100.298,40.9605,.0002 from dual union all
select 814,'JQ','Pad 921','launch point','','2008-01-01 00:00:00','','JQ    SLS-1','Pad 921, South Launch Site-1, Jiuquan Space Center','',100.2912,40.958,.0002 from dual union all
select 815,'JQ','SLS','launch point','','1999-01-01 00:00:00','','JQ    SLS','South Launch Site, Jiuquan Space Center','',100.2912,40.958,.002 from dual union all
select 816,'JQ','SLS-1','launch point','','2008-01-01 00:00:00','','JQ    SLS-1','Pad 921, South Launch Site-1, Jiuquan Space Center','',100.2912,40.958,.0002 from dual union all
select 817,'JQ','SLS-2','launch point','','2008-01-01 00:00:00','','JQ    SLS-2','Pad 603, South Launch Site-2, Jiuquan Space Center','',100.298,40.9605,.0002 from dual union all
select 818,'JQ','SLS-E','launch point','','2008-01-01 00:00:00','','JQ    SLS-E','KZ-1 pad, South Launch Site-East, Jiuquan Space Center','',100.3435,40.9691,.0002 from dual union all
select 819,'JQ','','launch site','CN','1968-01-01 00:00:00','','JQ    -','Jiuquan Space Center, Nei Monggol Zizhiqu, China','',100.31,41.31,.04 from dual union all
select 82,'AOR13','LP10','launch point','','1955-09-29 00:00:00','1955-09-29 00:00:00','AOR13 LP10','Atlantic Ocean Launch Site 13 Launch Point 10','Atlantic Ocean',-60.25,67.52,.01 from dual union all
select 820,'JUA','','launch site','US','1959-01-01 00:00:00','1959-01-01 00:00:00','JUA   -','Sierra de Juarez, Baja California','',-115.83,32.25,.02 from dual union all
select 821,'KAES','','launch site','KP','2014-01-01 00:00:00','','Kaesong','Kaesong, North Hwanghae Prov.','',126.5878,37.9382,.001 from dual union all
select 822,'KANSK','','launch site','RU','1988-01-01 00:00:00','1989-01-01 00:00:00','KANSK','Kansk, Krasnoyarsk region','',95.7,56.2,.1 from dual union all
select 823,'KARK','L1','launch point','','1965-05-25 00:00:00','1965-05-30 00:00:00','KARK  L1','Launcher 1, Cape Karikari, North Island, New Zealand','',173.4053,-34.825,.0001 from dual union all
select 824,'KARK','L2','launch point','','1965-05-25 00:00:00','1965-05-30 00:00:00','KARK  L2','Launcher 2, Cape Karikari, North Island, New Zealand','',173.4053,-34.825,.0001 from dual union all
select 825,'KARK','','launch site','NZ','1965-05-25 00:00:00','1965-05-30 00:00:00','KARK  -','Cape Karikari, North Island, New Zealand','',173.4053,-34.825,.0001 from dual union all
select 826,'KART','BRK2 PU2','launch point','','1981-01-01 00:00:00','1988-01-01 00:00:00','Kartaly-6 2-2','Silo PU2, Site BRK2, Kartaly missile base','Kartaly-6, Chelyabinsk',60.6,52.9,.5 from dual union all
select 827,'KART','BRK4 PU1','launch point','','1981-01-01 00:00:00','1988-01-01 00:00:00','Kartaly-6 4-1','Silo PU1, Site BRK4, Kartaly missile base','Kartaly-6, Chelyabinsk',60.6,52.9,.5 from dual union all
select 828,'KART','BRK4 PU2','launch point','','1981-01-01 00:00:00','1988-01-01 00:00:00','Kartaly-6 4-2','Silo PU2, Site BRK4, Kartaly missile base','Kartaly-6, Chelyabinsk',60.6,52.9,.5 from dual union all
select 829,'KART','BRK4 PU6','launch point','','1981-01-01 00:00:00','1988-01-01 00:00:00','Kartaly-6 4-6','Silo PU6, Site BRK4, Kartaly missile base','Kartaly-6, Chelyabinsk',60.6,52.9,.5 from dual union all
select 83,'AOR13','LP11','launch point','','1955-09-29 00:00:00','1955-09-29 00:00:00','AOR13 LP11','Atlantic Ocean Launch Site 13 Launch Point 11','Atlantic Ocean',-60,66.85,.01 from dual union all
select 830,'KART','','launch site','SU','1981-01-01 00:00:00','1988-01-01 00:00:00','Kartaly-6','Kartaly missile base','Kartaly-6, Chelyabinsk',60.6,52.9,.5 from dual union all
select 831,'KARY','','launch site','GR','1966-05-15 00:00:00','1966-05-20 00:00:00','KARY  -','Nissaki-Karystos, Evvoia, Hellas','',24.42,38.01,.02 from dual union all
select 832,'KASC','EP','launch point','','2013-01-01 00:00:00','','USC Epsilon','Epsilon Pad, Uchinoura Space Center','',131.0848,31.2474,.0001 from dual union all
select 833,'KASC','K','launch point','','1963-01-01 00:00:00','','KASC  K','Kappa Pad, Kagoshima Space Center','',131.082,31.247,.001 from dual union all
select 834,'KASC','L','launch point','','1963-01-01 00:00:00','1977-01-01 00:00:00','KASC  L','Lambda Pad, Kagoshima Space Center','',131.082,31.247,.001 from dual union all
select 835,'KASC','M','launch point','','1966-01-01 00:00:00','1995-01-01 00:00:00','KASC  M','Mu Pad, Kagoshima Space Center','',131.0848,31.2474,.0001 from dual union all
select 836,'KASC','M-V','launch point','','1997-01-01 00:00:00','','KASC  M-V','M-V Pad, Kagoshima Space Center','',131.085,31.247,.001 from dual union all
select 837,'KASC','M1','launch point','','1966-01-01 00:00:00','1995-01-01 00:00:00','KASC  M1','Refurbished Mu Pad, Kagoshima Space Center','',131.0848,31.2474,.0001 from dual union all
select 838,'KASC','TMP','launch point','','1960-01-01 00:00:00','','KASC  TMP','Temporary pad, Kagoshima Space Center, Kagoshima, Kyushu, Nippon','',131.079,31.25,.001 from dual union all
select 839,'KASC','','launch site','J','1960-01-01 00:00:00','2003-10-01 00:00:00','KASC  -','Kagoshima Space Center, Kagoshima, Kyushu, Nippon','',131.079,31.25,.001 from dual union all
select 84,'AOR13','LP12','launch point','','1955-09-30 00:00:00','1955-09-30 00:00:00','AOR13 LP12','Atlantic Ocean Launch Site 13 Launch Point 12','Atlantic Ocean',-62.45,70.2,.01 from dual union all
select 840,'KATTAK','','launch site','SU','1963-01-01 00:00:00','1971-01-01 00:00:00','Kattakurgan','Kattakurgan missile base, Samarkand region','',66.3,39.9,.2 from dual union all
select 841,'KAU','AAMDTC','launch point','','2014-01-01 00:00:00','','KAU   AAMDTC','Aegis Ashore Missile Defense Test Complex,Kauai Test Facility, Kauai','',-159.7672,22.0043,.0001 from dual union all
select 842,'KAU','KP','launch point','','2015-01-01 00:00:00','','KAU   KP','Super Strypi Rail, Kokole Point Launch Site, KTF','',-159.7622,21.985,.0002 from dual union all
select 843,'KAU','LC1','launch point','','1962-06-01 00:00:00','','KAU   LC1','Pad 1, Kauai Test Facility (KTF), Barking Sands, Kauai','',-159.777,22.058,.004 from dual union all
select 844,'KAU','LC10','launch point','','1962-01-01 00:00:00','','KAU   LC10','Launcher 10, Kauai Test Facility (KTF), Barking Sands, Kauai','',-159.78,22.06,.02 from dual union all
select 845,'KAU','LC12','launch point','','1962-01-01 00:00:00','','KAU   LC12','Launcher 12, Kauai Test Facility (KTF), Barking Sands, Kauai','',-159.7819,22.0604,.0001 from dual union all
select 846,'KAU','LC14','launch point','','1962-01-01 00:00:00','','KAU   LC14','Launcher 14, Kauai Test Facility (KTF), Barking Sands, Kauai','',-159.78,22.06,.02 from dual union all
select 847,'KAU','LC19','launch point','','1962-01-01 00:00:00','','KAU   LC19','Launcher 19, Kauai Test Facility (KTF), Barking Sands, Kauai','',-159.7817,22.0636,.02 from dual union all
select 848,'KAU','LC41','launch point','','1985-01-01 00:00:00','1992-01-01 00:00:00','KAU   Pad 41','MRL 7.5K, Kokole Point Launch Site, KTF','',-159.7622,21.985,.0002 from dual union all
select 849,'KAU','LC42','launch point','','1993-01-01 00:00:00','','KAU   LC42','PMR Pad, Kauai Test Facility, Kauai','',-159.772,22.068,.004 from dual union all
select 85,'AOR13','LP13','launch point','','1955-10-04 00:00:00','1955-10-04 00:00:00','AOR13 LP13','Atlantic Ocean Launch Site 13 Launch Point 13','Atlantic Ocean',-75.35,72.68,.01 from dual union all
select 850,'KAU','LP1','launch point','','1999-01-01 00:00:00','','KAU   LP1','Air Launch Zone, Kauai Test Facility, Kauai','',-160.1218,22.7982,.0001 from dual union all
select 851,'KAU','POA','launch point','','1962-06-01 00:00:00','','KAU   POA','Pacific Ocean launch area, off Kauai','',-159.777,22.058,.01 from dual union all
select 852,'KAU','POA2','launch point','','2006-01-01 00:00:00','','KAU   POA2','Pacific Ocean launch area, off Kauai','',-160.95,22.3,.01 from dual union all
select 853,'KAU','POA3','launch point','','2006-01-01 00:00:00','','KAU   POA3','Pacific Ocean launch area, off Kauai','',-164,21,1 from dual union all
select 854,'KAU','POA4','launch point','','2006-01-01 00:00:00','','KAU   POA4','Pacific Ocean launch area, off Kauai','',-163,20.5,.2 from dual union all
select 855,'KAU','','launch site','US','1962-06-01 00:00:00','','KAU   -','Kauai Test Facility (KTF), Barking Sands, Kauai','',-159.777,22.058,.004 from dual union all
select 856,'KER','','launch site','TF','1968-03-01 00:00:00','1975-01-01 00:00:00','KER   -','Kerguelen Range, Port-Aux-Francais, Kerguelen Island','',70.23,-49.37,.02 from dual union all
select 857,'KERM','','launch site','IR','1985-01-01 00:00:00','1985-01-01 00:00:00','KERM  -','Bakhtaran/Kermanshah Air Base, Iran','',47.2358,34.3575,.001 from dual union all
select 858,'KEW','','launch site','US','1964-08-06 00:00:00','1971-01-31 00:00:00','KEW   -','Keweenaw Rocket Launch Site (KRLS), Michigan','',-87.72,47.43,.02 from dual union all
select 859,'KITT','','launch site','KP','2006-01-01 00:00:00','','KITT  -','Kittaeryong (Gitdaeryung), Anbyun, Gangwon Province (Kangwon-do)','',127.6236,38.9908,.001 from dual union all
select 86,'AOR13','LP14','launch point','','1955-10-04 00:00:00','1955-10-04 00:00:00','AOR13 LP14','Atlantic Ocean Launch Site 13 Launch Point 14','Atlantic Ocean',-61.85,65.12,.01 from dual union all
select 860,'KLA','','launch site','KI','1998-01-01 00:00:00','','KLA   -','Sea Launch Kiritimati Launch Area, Pacific Ocean','',-154,0,.05 from dual union all
select 861,'KLC','LP1','launch point','','1998-01-01 00:00:00','','KLC LP1','Launch Pad 1, Kodiak Launch Complex, Kodiak Island, Alaska','',-152.3394,57.4353,.0002 from dual union all
select 862,'KLC','LP2','launch point','','1998-01-01 00:00:00','','KLC LP2','Launch Pad 2, Kodiak Launch Complex, Kodiak Island, Alaska','',-152.342,57.4348,.0002 from dual union all
select 863,'KLC','','launch site','US','1998-01-01 00:00:00','2016-08-01 00:00:00','KLC   -','Kodiak Launch Complex, Kodiak Island, Alaska','',-152.3378,57.4359,.0002 from dual union all
select 864,'KMR','BOA','launch point','','2009-01-01 00:00:00','','KMR   BOA N','Broad Ocean Area 75-150 km N of Roi Namur','Pacific Ocean',167.5,10.5,.5 from dual union all
select 865,'KMR','BOA-NE','launch point','','2009-01-01 00:00:00','','KMR   BOA NE','Broad Ocean Area NE of Kwajalein','Pacific Ocean',168,10.5,1 from dual union all
select 866,'KMR','IL','launch point','','1962-01-01 00:00:00','','KMR   IL','Illeginni Island, USAKA, Marshall Islands','',167.7,9,.0001 from dual union all
select 867,'KMR','KW','launch point','','1962-01-01 00:00:00','','KMR   KW','Kwajalein Island, US Army Kwajalein Atoll (USAKA), Marshall Islands','',167.718,8.7023,.005 from dual union all
select 868,'KMR','Meck','launch point','','1962-01-01 00:00:00','','KMR   Meck','Meck Island, USAKA, Marshall Islands','',167.7269,9.006,.0001 from dual union all
select 869,'KMR','Meck 1','launch point','','1962-01-01 00:00:00','','KMR   Meck 1','Silo 1, Meck Island, USAKA, Marshall Islands','',167.7269,9.006,.0001 from dual union all
select 87,'AOR13','LP15','launch point','','1955-10-04 00:00:00','1955-10-04 00:00:00','AOR13 LP15','Atlantic Ocean Launch Site 13 Launch Point 15','Atlantic Ocean',-64.6,61.58,.01 from dual union all
select 870,'KMR','OM','launch point','','1962-01-01 00:00:00','','KMR   OM','Omelek Island, USAKA, Marshall Islands','',167.7432,9.0494,.0001 from dual union all
select 871,'KMR','RN','launch point','','1962-01-01 00:00:00','','KMR   RN','Roi-Namur Island, USAKA, Marshall Islands','',167.4663,9.4012,.0001 from dual union all
select 872,'KMR','RW06/24','launch point','','2000-01-01 00:00:00','','KMR   RW06/24','RW06/24, BUCHOLZ ARMY AIRFIELD (PKWA), USAKA, Marshall Islands','',167.7267,8.715,.002 from dual union all
select 873,'KMR','','launch site','TTPI','1962-01-01 00:00:00','','KMR   -','US Army Kwajalein Atoll (USAKA), Marshall Islands','',167.72,8.9917,.02 from dual union all
select 874,'KOR','LP1','launch point','','1966-05-15 00:00:00','1966-05-21 00:00:00','KOR   LP1','Offshore launch point, Koroni','',21.95,36.77,.02 from dual union all
select 875,'KOR','LP2','launch point','','1966-05-15 00:00:00','1966-05-21 00:00:00','KOR   LP2','Koroni, Peloponnisos, Hellas (beach launch area)','',21.95,36.77,.02 from dual union all
select 876,'KOR','','launch site','GR','1966-05-15 00:00:00','1966-05-21 00:00:00','KOR   -','Koroni, Peloponnisos, Hellas (beach launch area)','',21.95,36.77,.02 from dual union all
select 877,'KORO72A','','launch cruise','SU','1972-01-01 00:00:00','1972-03-01 00:00:00','KORO72A','Korolev cruise Jan-Mar 1972','',null,null,null from dual union all
select 878,'KORO73A','','launch cruise','SU','1973-02-01 00:00:00','1973-03-01 00:00:00','KORO73A','Korolev cruise Feb - Mar 1973','',null,null,null from dual union all
select 879,'KORO73B','','launch cruise','SU','1973-09-01 00:00:00','1973-01-01 00:00:00','KORO73B','Korolev cruise Sept-Dec 1973 via CSG','',null,null,null from dual union all
select 88,'AOR13','LP16','launch point','','1955-10-04 00:00:00','1955-10-04 00:00:00','AOR13 LP16','Atlantic Ocean Launch Site 13 Launch Point 16','Atlantic Ocean',-58.7,56.78,.01 from dual union all
select 880,'KORO74A','','launch cruise','SU','1974-02-01 00:00:00','1974-05-01 00:00:00','KORO74A','Korolev cruise 1974','',null,null,null from dual union all
select 881,'KORO74B','','launch cruise','SU','1974-10-01 00:00:00','1974-11-01 00:00:00','KORO74B','Korolev cruise 1974','',null,null,null from dual union all
select 882,'KORO75A','','launch cruise','SU','1975-03-01 00:00:00','1975-03-01 00:00:00','KORO75A','Korolev cruise 1975','',null,null,null from dual union all
select 883,'KORO77A','','launch cruise','SU','1977-01-01 00:00:00','1977-01-01 00:00:00','KORO77A','Korolev cruise 1977','',null,null,null from dual union all
select 884,'KORO77B','','launch cruise','SU','1977-03-01 00:00:00','1977-03-01 00:00:00','KORO77B','Korolev cruise 1977','',null,null,null from dual union all
select 885,'KORO77C','','launch cruise','SU','1977-11-01 00:00:00','1978-03-01 00:00:00','KORO77C','Korolev cruise 1977','',null,null,null from dual union all
select 886,'KORO78A','','launch cruise','SU','1978-04-01 00:00:00','1978-04-01 00:00:00','KORO78A','Korolev cruise 1978','',null,null,null from dual union all
select 887,'KORO79A','','launch cruise','SU','1979-01-01 00:00:00','1979-02-01 00:00:00','KORO79A','Korolev cruise 1979','',null,null,null from dual union all
select 888,'KORO79B','','launch cruise','SU','1979-04-01 00:00:00','1979-05-01 00:00:00','KORO79B','Korolev cruise 1979','',null,null,null from dual union all
select 889,'KORO79C','','launch cruise','SU','1979-06-01 00:00:00','1979-08-01 00:00:00','KORO79C','Korolev firings mid 1979','',null,null,null from dual union all
select 89,'AOR13','LP17','launch point','','1955-10-04 00:00:00','1955-10-04 00:00:00','AOR13 LP17','Atlantic Ocean Launch Site 13 Launch Point 17','Atlantic Ocean',-58.18,56.37,.01 from dual union all
select 890,'KORO79D','','launch cruise','SU','1979-11-01 00:00:00','1980-02-01 00:00:00','KORO79D','Korolev cruise 1979-80','',null,null,null from dual union all
select 891,'KORO80A','','launch cruise','SU','1980-05-01 00:00:00','1980-06-01 00:00:00','KORO80A','Korolev cruise 1980','',null,null,null from dual union all
select 892,'KORO81A','','launch cruise','SU','1981-05-01 00:00:00','1981-06-01 00:00:00','KORO81A','Korolev cruise 1981','',null,null,null from dual union all
select 893,'KORO82A','','launch cruise','SU','1982-02-01 00:00:00','1982-03-01 00:00:00','KORO82A','Korolev cruise 1982','',null,null,null from dual;
insert into SITE
select 894,'KORO83A','','launch cruise','SU','1983-06-01 00:00:00','1983-08-01 00:00:00','KORO83A','Korolev cruise 1983','',null,null,null from dual union all
select 895,'KORO87A','','launch cruise','SU','1987-01-01 00:00:00','1987-03-01 00:00:00','KORO87A','Korolev cruise 1987','',null,null,null from dual union all
select 896,'KORO88A','','launch cruise','SU','1988-12-01 00:00:00','1989-03-01 00:00:00','KORO88A','Korolev cruise 1988-89','',null,null,null from dual union all
select 897,'KOSTR','','launch site','SU','1960-01-01 00:00:00','1980-01-01 00:00:00','KOSTR','Kostroma missile base, Russia','',41.38,57.67,.01 from dual union all
select 898,'KRAS','','launch site','SU','1960-01-01 00:00:00','1990-01-01 00:00:00','KRAS','Krasnoyarsk missile site, Russia','',92.29,56.23,.02 from dual union all
select 899,'KREN77A','','launch cruise','SU','1977-02-01 00:00:00','1977-04-01 00:00:00','Krenkel 77','Krenkel cruise 1977','',null,null,null from dual union all
select 9,'ALCA','MRL','launch point','','1994-08-19 00:00:00','','ALCA  MRL','MRL launcher, CL Alcantara','Alcantara, Maranhao, Brasil',-44.3677,-2.315,.0001 from dual union all
select 90,'AOR13','LP18','launch point','','1955-10-04 00:00:00','1955-10-04 00:00:00','AOR13 LP18','Atlantic Ocean Launch Site 13 Launch Point 18','Atlantic Ocean',-57.3,55.72,.01 from dual union all
select 900,'KREN77B','','launch cruise','SU','1977-06-01 00:00:00','1977-07-01 00:00:00','Krenkel 77','Krenkel cruise 1977','',null,null,null from dual union all
select 901,'KREN77C','','launch cruise','SU','1977-10-01 00:00:00','1977-10-01 00:00:00','Krenkel 77','Krenkel cruise 1977','',null,null,null from dual union all
select 902,'KREN78A','','launch cruise','SU','1978-07-01 00:00:00','1978-09-01 00:00:00','Krenkel 77','Krenkel cruise 1977','',null,null,null from dual union all
select 903,'KREN79A','','launch cruise','SU','1979-01-01 00:00:00','1979-01-01 00:00:00','Krenkel 79','Krenkel cruise 1979','',null,null,null from dual union all
select 904,'KREN79B','','launch cruise','SU','1979-04-01 00:00:00','1979-04-01 00:00:00','Krenkel 79','Krenkel cruise 1979','',null,null,null from dual union all
select 905,'KREN79C','','launch cruise','SU','1979-07-01 00:00:00','1979-07-01 00:00:00','Krenkel 79','Krenkel cruise 1979','',null,null,null from dual union all
select 906,'KREN79D','','launch cruise','SU','1979-11-01 00:00:00','1979-11-01 00:00:00','Krenkel 79','Krenkel cruise 1979','',null,null,null from dual union all
select 907,'KREN80A','','launch cruise','SU','1980-09-01 00:00:00','1980-10-01 00:00:00','Krenkel 80','Krenkel cruise 1980','',null,null,null from dual union all
select 908,'KREN81A','','launch cruise','SU','1981-02-01 00:00:00','1981-02-01 00:00:00','Krenkel 81','Krenkel cruise 1981','',null,null,null from dual union all
select 909,'KREN81B','','launch cruise','SU','1981-05-01 00:00:00','1981-05-01 00:00:00','Krenkel 81','Krenkel cruise 1981','',null,null,null from dual union all
select 91,'AOR13','LP19','launch point','','1955-09-27 00:00:00','1955-10-04 00:00:00','AOR13 LP19','Atlantic Ocean Launch Site 13 Launch Point 19','Atlantic Ocean',-54.57,57.4,.01 from dual union all
select 910,'KREN82A','','launch cruise','SU','1982-11-01 00:00:00','1982-11-01 00:00:00','Krenkel 82','Krenkel cruise 1982','',null,null,null from dual union all
select 911,'KREN83A','','launch cruise','SU','1983-02-01 00:00:00','1983-02-01 00:00:00','Krenkel 83','Krenkel cruise 1983','',null,null,null from dual union all
select 912,'KREN85A','','launch cruise','SU','1985-08-01 00:00:00','1985-09-01 00:00:00','Krenkel 85','Krenkel cruise 1985','',null,null,null from dual union all
select 913,'KREN86A','','launch cruise','SU','1986-03-01 00:00:00','1986-04-01 00:00:00','Krenkel 86','Krenkel cruise 1986','',null,null,null from dual union all
select 914,'KREN87A','','launch cruise','SU','1987-03-01 00:00:00','1987-03-01 00:00:00','Krenkel 87','Krenkel cruise 1987','',null,null,null from dual union all
select 915,'KREN87B','','launch cruise','SU','1987-06-01 00:00:00','1987-06-01 00:00:00','Krenkel 87','Krenkel cruise 1987','',null,null,null from dual union all
select 916,'KREN87C','','launch cruise','SU','1987-09-01 00:00:00','1987-10-01 00:00:00','Krenkel 87','Krenkel cruise 1987','',null,null,null from dual union all
select 917,'KREN89A','','launch cruise','SU','1989-11-01 00:00:00','1989-11-01 00:00:00','Krenkel 89','Krenkel cruise 1989','',null,null,null from dual union all
select 918,'KREN90A','','launch cruise','SU','1990-02-01 00:00:00','1990-03-01 00:00:00','Krenkel 90','Krenkel cruise 1990','',null,null,null from dual union all
select 919,'KRON','J','launch point','','1961-01-01 00:00:00','1961-08-01 00:00:00','KRON  J','Jokkmokk (Vidsel), Sweden','',19.2767,66.4147,.0001 from dual union all
select 92,'AOR13','LP2','launch point','','1955-09-24 00:00:00','1955-09-24 00:00:00','AOR13 LP2','Atlantic Ocean Launch Site 13 Launch Point 2','Atlantic Ocean',-60.73,44.58,.01 from dual union all
select 920,'KRON','','launch site','S','1961-01-01 00:00:00','1964-01-01 00:00:00','KRON  -','Kronogard (Jokkmokk/Vidsel), Norrbottens Lan, Sverige','',19.78,66.22,.02 from dual union all
select 921,'KSC','LC39A','launch point','','1966-12-01 00:00:00','','KSC   LC39A','Launch Complex 39A, NASA Kennedy Space Center, Florida','',-80.6041,28.6083,.0001 from dual union all
select 922,'KSC','LC39B','launch point','','1966-12-01 00:00:00','','KSC   LC39B','Launch Complex 39B, NASA Kennedy Space Center, Florida','',-80.6208,28.6272,.0001 from dual union all
select 923,'KSC','RW15/33','launch point','','1981-04-01 00:00:00','','KSC   RW15/33','Shuttle Landing Facility, NASA Kennedy Space Center, Florida','',-80.6944,28.6149,.0002 from dual union all
select 924,'KSC','','launch site','US','1966-12-01 00:00:00','','KSC   -','NASA John F. Kennedy Space Center, Florida','',-80.6041,28.6083,.0001 from dual union all
select 925,'KUSONG','LP1','launch point','','2016-01-01 00:00:00','','Kusong LP1','Musudan launch site, Panghyon Airport, Kusong City','',125.2079,39.9275,.001 from dual union all
select 926,'KUSONG','LP2','launch point','','2016-01-01 00:00:00','','Kusong LP2','Bukgeoksong launch site, Kusong City','',125.223,40.0133,.001 from dual union all
select 927,'KUSONG','LP3','launch point','','2016-01-01 00:00:00','','Kusong LP3','Hwasong 12 launch site, Kusong City','',125.2099,40.066,.001 from dual union all
select 928,'KUSONG','LP4','launch point','','2016-01-01 00:00:00','','Kusong LP4','Hwasong 14 launch site, Kusong City','',125.2689,39.8722,.001 from dual union all
select 929,'KUSONG','','launch site','KP','2016-01-01 00:00:00','','Kusong','Musudan launch site, Banghyon Airport, Kusong City, N Pyongan Prov.','',125.2,39.9,.1 from dual union all
select 93,'AOR13','LP20','launch point','','1955-09-27 00:00:00','1955-10-13 00:00:00','AOR13 LP20','Atlantic Ocean Launch Site 13 Launch Point 20','Atlantic Ocean',-53.97,58.05,.01 from dual union all
select 930,'LANC','','launch site','AU','1974-06-01 00:00:00','1974-06-01 00:00:00','LANC  -','Lancelin, Perth, W. Australia','',115.19,-31.01,.02 from dual union all
select 931,'LAOG','','launch site','CN','1960-02-19 00:00:00','1960-01-01 00:00:00','LAOG  -','Laogang, Shanghai Shi, Zhongguo','',121.82,31.02,.02 from dual union all
select 932,'LAP','','launch site','ID','1965-08-11 00:00:00','','LAP   -','STASPRO, Pameungpeuk site, Cilauteurum, Lembe I, Garut, Jawa Barat','',107.6917,-7.6689,.001 from dual union all
select 933,'LEBA','','launch site','PL','1969-10-03 00:00:00','1973-06-15 00:00:00','LEBA  -','Leba (Ustka), Slupsk, Pomerania, Polska','',17.55,54.78,.02 from dual union all
select 934,'LIT','LM12-DS','launch point','','1972-12-14 00:00:00','1972-12-14 00:00:00','LIT   LM12-DS','LM 12 Descent Stage, Taurus-Littrow Base, Littrow Valley, Macrobius B1','Luna',30.75,20.16,.01 from dual union all
select 935,'LIT','','launch site','LUNA','1972-12-14 00:00:00','1972-12-14 00:00:00','LIT   LM12-DS','LM 12 Descent Stage, Taurus-Littrow Base, Littrow Valley, Macrobius B1','Luna',30.75,20.16,.01 from dual union all
select 936,'LROCK','374-7','launch point','','1962-01-01 00:00:00','1986-01-01 00:00:00','Little Rock','Launch Complex 374-7, Little Rock Air Force Base, Arkansas','',-92.3973,35.414,.0001 from dual union all
select 937,'LROCK','','launch site','US','1960-01-01 00:00:00','1990-01-01 00:00:00','Little Rock','Little Rock Air Force Base, Arkansas','',-92.15,34.9,.02 from dual union all
select 938,'LRPG','','launch site','US','1950-05-16 00:00:00','1951-06-30 00:00:00','LRPG','Long Range Proving Ground','Cape Canaveral, Florida',-80.54,28.46,.05 from dual union all
select 939,'LUNA','','launch site','LUNA','','','Luna','Luna','',null,null,null from dual union all
select 94,'AOR13','LP21','launch point','','1955-09-27 00:00:00','1955-10-13 00:00:00','AOR13 LP21','Atlantic Ocean Launch Site 13 Launch Point 21','Atlantic Ocean',-55.43,61.45,.01 from dual union all
select 940,'MAHIA','LC1','launch point','','2017-05-01 00:00:00','','RLLC1','Rocket Lab Launch Complex 1, Mahia Peninsula','Mahia Peninsula, N Island',177.8655,-39.2609,.0001 from dual union all
select 941,'MAHIA','','launch site','NZ','2017-05-01 00:00:00','','RLLC1','Rocket Lab Launch Complex 1, Onenui Station, Mahia Peninsula','Mahia Peninsula, N Island',177.8655,-39.2609,.0001 from dual union all
select 942,'MAKAT','','launch site','KZ','1957-01-01 00:00:00','1960-01-01 00:00:00','MAKAT -','Makat tactical missile base, Atyrau, Kazakstan','',53.32,47.65,.02 from dual union all
select 943,'MAKHCH','','launch site','SU','1960-01-01 00:00:00','1970-01-01 00:00:00','Makhch','Makhachkala missile base, Makhachkala, Dagestan','',47.5,42.7,.2 from dual union all
select 944,'MARS','Pad 0A','launch point','','2006-01-01 00:00:00','','WI    LA0A','Launch Area 0A, Wallops Flight Facility','',-75.4901,37.8315,.0001 from dual union all
select 945,'MARS','Pad 0B','launch point','','2006-01-01 00:00:00','','WI    LA0B','Launch Area 0B, Mid-Atlantic Regional Spaceport, Wallops Island','',-75.49,37.83,.02 from dual union all
select 946,'MARS','','launch site','US','2006-01-01 00:00:00','','MARS','Mid-Atlantic Regional Spaceport, Wallops Island, Virginia','',-75.48,37.83,.02 from dual union all
select 947,'MASHH','','launch site','IR','2000-01-01 00:00:00','','Mashhad','14th Tactical Air Base, Mashhad','Mashhad, Iran',59.6403,36.2342,.0005 from dual union all
select 948,'MATI','','launch site','US','1981-08-05 00:00:00','1982-09-09 00:00:00','MATI  -','Matagorda Island, Texas','',-96.5,28.25,.02 from dual union all
select 949,'MATZ','','launch site','ARV','1965-01-01 00:00:00','1965-01-01 00:00:00','MATZ  -','Base Aerea Teniente Benjamin Matienzo, Nunatak Larsen, Antarctica','Antartida Argentina',-60.07,-64.96,.05 from dual union all
select 95,'AOR13','LP22','launch point','','1955-09-27 00:00:00','1955-10-13 00:00:00','AOR13 LP22','Atlantic Ocean Launch Site 13 Launch Point 22','Atlantic Ocean',-56.72,62.48,.01 from dual union all
select 950,'MCM','','launch site','NZRD','1962-01-01 00:00:00','1963-01-01 00:00:00','MCM   -','McMurdo Station, Ross Ice Shelf, New Zealand Antarctic Territory','',166.66,-77.85,.02 from dual union all
select 951,'MEDR','','launch site','IL','2010-01-01 00:00:00','','Med Range','Target drop point, Mediterranean Sea','',34,18.5,1 from dual union all
select 952,'MER','','launch site','AR','1973-01-01 00:00:00','1973-01-01 00:00:00','MER   -','Villa Reynolds Air Base, Mercedes, San Luis province, Argentina','',-65.38,-33.73,.02 from dual union all
select 953,'MFWA','','launch zone','US','1993-01-01 00:00:00','','MFWA  -','Drop Zone, Mayport, Florida, Warning Area, North Atlantic Ocean','',-78.5,29,.1 from dual union all
select 954,'MFWA1','','launch zone','US','1993-01-01 00:00:00','','MFWA1 -','Drop Zone, Mayport, Florida, Warning Area, North Atlantic Ocean','',-78.3,29,.1 from dual union all
select 955,'MHV','RW12/30','launch point','','2004-01-01 00:00:00','','MHV   RW12/30','RW12/30, Mojave Spaceport, California','',-118.1357,35.0487,.001 from dual union all
select 956,'MHV','RW30','launch point','','2004-01-01 00:00:00','','MHV   RW30','RW12/30, Mojave Spaceport, California','',-118.1357,35.0487,.001 from dual union all
select 957,'MHV','','launch site','US','2004-01-01 00:00:00','','MHV','Mojave Spaceport, California','',-118.1357,35.0487,.001 from dual union all
select 958,'MID','','launch zone','US','2004-01-01 00:00:00','','MID   -','Air drop zone south of Midway Island','',-177,27,3 from dual union all
select 959,'MID2','','launch zone','US','2004-01-01 00:00:00','','MID2  -','Air drop zone north of Midway?, 1500 km from Shemya','',180,38,5 from dual union all
select 96,'AOR13','LP23','launch point','','1955-09-27 00:00:00','1955-10-13 00:00:00','AOR13 LP23','Atlantic Ocean Launch Site 13 Launch Point 23','Atlantic Ocean',-61.4,65.92,.01 from dual union all
select 960,'MINAKH','','launch site','SY','1992-07-01 00:00:00','','MINAK -','Syrian missile base, Minakh, N of Halab/Alleppo, Syria (Base at Al Safir)','',37.0391,36.519,.001 from dual union all
select 961,'MLD','','launch site','AAT','1970-01-01 00:00:00','1984-01-01 00:00:00','MLD   -','Molodezhnaya, Enderby Land, Australian Antarctic Territory','',45.85,-67.66,.02 from dual union all
select 962,'MRL','','launch site','N','1983-11-30 00:00:00','1984-01-15 00:00:00','MRL   -','Marka Range, Lista, Norway','',7.3,58.2,.1 from dual union all
select 963,'MRNV','','launch site','US','1957-01-01 00:00:00','1957-01-01 00:00:00','MRNV  -','Mercury, Nevada','',-115.98,36.67,.04 from dual union all
select 964,'MUD','A','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','MUD   A','Mud Lake 2 drop zone, Nevada','',-117.3,38.1,.1 from dual union all
select 965,'MUD','B','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','MUD   B','Mud Lake 1 drop zone, Nevada','',-116.8,37.97,.1 from dual union all
select 966,'MUD','L','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','MUD   L','Mud Lake, Nevada','',-117.09,37.87,.1 from dual union all
select 967,'MUD','RW08/26','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','MUD   RW08/26','RW08/26, Mud Lake, Nevada','',-117.09,37.87,.1 from dual union all
select 968,'MUD','RW18/36','launch point','','1959-01-01 00:00:00','1967-01-01 00:00:00','MUD   RW18/36','RW18/36, Mud Lake, Nevada','',-117.09,37.87,.1 from dual union all
select 969,'MUD','','launch site','US','1959-01-01 00:00:00','1967-01-01 00:00:00','MUD','Mud Lake, Nevada','',-117.3,38.1,.1 from dual union all
select 97,'AOR13','LP24','launch point','','1955-09-27 00:00:00','1955-10-13 00:00:00','AOR13 LP24','Atlantic Ocean Launch Site 13 Launch Point 24','Atlantic Ocean',-63.53,60.58,.01 from dual union all
select 970,'MUPYONG','','launch site','KP','2017-01-01 00:00:00','','Mupyong-ni','Mupyong-ni, Chagang province','',126.4257,40.6112,.0001 from dual union all
select 971,'MUSS77A','','launch cruise','SU','1977-12-01 00:00:00','1977-12-01 00:00:00','Musson 77','Musson cruise 1977','',null,null,null from dual union all
select 972,'MUSS78A','','launch cruise','SU','1978-03-01 00:00:00','1978-04-01 00:00:00','Musson 78','Musson cruise 1978','',null,null,null from dual union all
select 973,'MUSS78B','','launch cruise','SU','1978-11-01 00:00:00','1978-11-01 00:00:00','Musson 78','Musson cruise 1978','',null,null,null from dual union all
select 974,'MUSS79A','','launch cruise','SU','1979-02-01 00:00:00','1979-02-01 00:00:00','Musson 79','Musson cruise 1979','',null,null,null from dual union all
select 975,'MUSS79B','','launch cruise','SU','1979-05-01 00:00:00','1979-06-01 00:00:00','Musson 79','Musson cruise 1979','',null,null,null from dual union all
select 976,'MUSS81A','','launch cruise','SU','1981-09-01 00:00:00','1981-10-01 00:00:00','Musson 81','Musson cruise 1981','',null,null,null from dual union all
select 977,'MUSS82A','','launch cruise','SU','1982-01-01 00:00:00','1982-01-01 00:00:00','Musson 82','Musson cruise 1982','',null,null,null from dual union all
select 978,'MUSS82B','','launch cruise','SU','1982-04-01 00:00:00','1982-05-01 00:00:00','Musson 82','Musson cruise 1982','',null,null,null from dual union all
select 979,'MUSS83A','','launch cruise','SU','1983-05-01 00:00:00','1983-05-01 00:00:00','Musson 83','Musson cruise 1983','',null,null,null from dual union all
select 98,'AOR13','LP25','launch point','','1955-09-27 00:00:00','1955-10-13 00:00:00','AOR13 LP25','Atlantic Ocean Launch Site 13 Launch Point 25','Atlantic Ocean',-59,57.17,.01 from dual union all
select 980,'MUSS83B','','launch cruise','SU','1983-08-01 00:00:00','1983-08-01 00:00:00','Musson 83','Musson cruise 1983','',null,null,null from dual union all
select 981,'MUSS85A','','launch cruise','SU','1985-10-01 00:00:00','1985-11-01 00:00:00','Musson 85','Musson cruise 1985','',null,null,null from dual union all
select 982,'MUSS87A','','launch cruise','SU','1987-07-01 00:00:00','1987-07-01 00:00:00','Musson 87','Musson cruise 1987','',null,null,null from dual union all
select 983,'MUSS89A','','launch cruise','SU','1989-06-01 00:00:00','1989-06-01 00:00:00','Musson 89','Musson cruise 1989','',null,null,null from dual;
insert into SITE
select 984,'NAMFI','','launch site','GR','1994-01-01 00:00:00','','NAMFI','NATO Missile Firing Installation (NAMFI)','Chania, Crete',24.2,35.6,.15 from dual union all
select 985,'NAMPO','','launch site','KP','2015-01-01 00:00:00','','Nampo','Scud launch site near Nampo, South Pyongan Prov. (Pyong''annam-do)','',125.5,38.75,.3 from dual union all
select 986,'NAOTS','','launch site','US','1955-01-01 00:00:00','1955-01-01 00:00:00','NAOTS -','Naval Air Ordnance Test Station, Chincoteague, Virginia','',-75.36,37.92,.03 from dual union all
select 987,'NARO','','launch site','KR','2007-01-01 00:00:00','','NARO  -','Naro Space Center (Naro Uju Senteo),Oenaro I,GoHeung, Jeollanam-do,Korea','',127.54,34.43,.02 from dual union all
select 988,'NEGEV','','launch site','IL','1980-01-01 00:00:00','1995-01-01 00:00:00','NEGEV -','Negev Desert test site, Hadarom Hanegev, Israel','',35,31,1 from dual union all
select 989,'NEN','','launch site','SU','1960-01-01 00:00:00','','NEN   -','GTsMP?, Nenoksa Test Range, Rossiya','',39.2,64.63,.1 from dual union all
select 99,'AOR13','LP26','launch point','','1955-09-27 00:00:00','1955-10-13 00:00:00','AOR13 LP26','Atlantic Ocean Launch Site 13 Launch Point 26','Atlantic Ocean',-57.65,55.98,.01 from dual union all
select 990,'NIIJ','','launch site','J','1963-01-01 00:00:00','1966-01-01 00:00:00','NIIJ  -','Niijima, Nippon','',139.27,34.33,.02 from dual union all
select 991,'NIIP-5','','launch site','SU','1956-01-01 00:00:00','1989-01-01 00:00:00','NIIP-5','NIIP-5, Baykonur, Kazakstan','',63,46,.02 from dual union all
select 992,'NIIP-53','','launch site','SU','1960-01-01 00:00:00','1993-01-01 00:00:00','NIIP-53','53-y Nauchno-Issledovatelskiy Ispitatelniy Poligon,','Plesetsk',40.5,62.9,.02 from dual union all
select 993,'NOKV73A','','launch cruise','SU','1973-01-01 00:00:00','1973-01-01 00:00:00','NOKV73A','Noyekov cruise May-Jul 1973','',null,null,null from dual union all
select 994,'NOKV73B','','launch cruise','SU','1973-01-01 00:00:00','1973-01-01 00:00:00','NOKV73B','Noyekov cruise Oct-Dec 1973','',null,null,null from dual union all
select 995,'NOSSET','','launch site','RU','2008-01-01 00:00:00','2008-01-01 00:00:00','N Ossetia','North Ossetia? Possible Iskander launch point','',44,43,1 from dual union all
select 996,'NOTS','G-2','launch point','','1958-01-01 00:00:00','1958-01-01 00:00:00','NOTS  G-2','Test Stand G-2, Naval Ordnance Test Station, China Lake, California','',-117.17,35.58,.04 from dual union all
select 997,'NOTS','RW','launch point','','1958-01-01 00:00:00','1958-01-01 00:00:00','NOTS  RW','RW ??, Naval Ordnance Test Station, China Lake, California','',-117.17,35.58,.02 from dual union all
select 998,'NOTS','','launch site','US','1958-01-01 00:00:00','1958-01-01 00:00:00','NOTS  -','Naval Ordnance Test Station, China Lake, California','',-117.17,35.58,.02 from dual union all
select 999,'NOUA','','launch site','MR','1973-06-30 00:00:00','1973-06-30 00:00:00','NOUA  -','Nouadhibou, Dakhlet Nouadhibou, Mauritanie','',-17.0314,20.9208,.0002 from dual;


--------------------------------------------------------------------------------
-- SITE_ORG
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing SITE_ORG...');

CREATE TABLE "SITE_ORG" 
 (	"SITE_ID" NUMBER, 
	"ORG_CODE" VARCHAR2(4000), 
	 CONSTRAINT "SITE_ORG_PK" PRIMARY KEY ("SITE_ID", "ORG_CODE")
USING INDEX  ENABLE, 
	 CONSTRAINT "SITE_ORG_SITE_FK" FOREIGN KEY ("SITE_ID")
	  REFERENCES "SITE" ("SITE_ID") ENABLE, 
	 CONSTRAINT "SITE_ORG_ORG_FK" FOREIGN KEY ("ORG_CODE")
	  REFERENCES "ORGANIZATION" ("ORG_CODE") ENABLE
 ) ;

insert into SITE_ORG
select 1,'USAF' from dual union all
select 10,'AEB' from dual union all
select 100,'SUI' from dual union all
select 1000,'RVSN' from dual union all
select 1001,'VMF' from dual union all
select 1002,'RVSN' from dual union all
select 1003,'USAF' from dual union all
select 1004,'DOE' from dual union all
select 1005,'DOE' from dual union all
select 1006,'RVSN' from dual union all
select 1007,'ISAS' from dual union all
select 1009,'SU' from dual union all
select 101,'SUI' from dual union all
select 1010,'SU' from dual union all
select 1011,'VMF' from dual union all
select 1012,'SU' from dual union all
select 1013,'SU' from dual union all
select 1014,'ZA' from dual union all
select 1015,'USAF' from dual union all
select 1016,'USAF' from dual union all
select 1017,'USN' from dual union all
select 1018,'USAF' from dual union all
select 1019,'USAF' from dual union all
select 102,'SUI' from dual union all
select 1020,'USAF' from dual union all
select 1021,'USAF' from dual union all
select 1022,'USAF' from dual union all
select 1023,'USN' from dual union all
select 1024,'IL' from dual union all
select 1025,'IL' from dual union all
select 1026,'IL' from dual union all
select 1027,'SU' from dual union all
select 1028,'SU' from dual union all
select 1029,'SU' from dual union all
select 103,'SUI' from dual union all
select 1030,'SU' from dual union all
select 1031,'SU' from dual union all
select 1032,'SU' from dual union all
select 1033,'SU' from dual union all
select 1034,'SU' from dual union all
select 1035,'SU' from dual union all
select 1036,'SU' from dual union all
select 1037,'SU' from dual union all
select 1038,'OSC' from dual union all
select 1039,'NASA' from dual union all
select 104,'SUI' from dual union all
select 1040,'AR' from dual union all
select 1041,'USAF' from dual union all
select 1042,'USAF' from dual union all
select 1043,'USAF' from dual union all
select 1044,'USAF' from dual union all
select 1045,'USAF' from dual union all
select 1046,'USAF' from dual union all
select 1047,'USAF' from dual union all
select 1048,'I' from dual union all
select 1049,'USAF' from dual union all
select 105,'SUI' from dual union all
select 1050,'NASA' from dual union all
select 1050,'USAF' from dual union all
select 1051,'USN' from dual union all
select 1052,'USN' from dual union all
select 1053,'USN' from dual union all
select 1054,'USN' from dual union all
select 1055,'USN' from dual union all
select 1056,'USN' from dual union all
select 1057,'USN' from dual union all
select 1058,'USN' from dual union all
select 1059,'USN' from dual union all
select 106,'SUI' from dual union all
select 1060,'USN' from dual union all
select 1061,'USN' from dual union all
select 1062,'VMF' from dual union all
select 1063,'USN' from dual union all
select 1064,'NRL' from dual union all
select 1065,'NASA' from dual union all
select 1066,'NASA' from dual union all
select 1067,'NASA' from dual union all
select 1068,'NASA' from dual union all
select 1069,'NASA' from dual union all
select 107,'SUI' from dual union all
select 1070,'NASA' from dual union all
select 1071,'NASA' from dual union all
select 1072,'NASA' from dual union all
select 1073,'NASA' from dual union all
select 1074,'NASA' from dual union all
select 1075,'NASA' from dual union all
select 1076,'NASA' from dual union all
select 1077,'NASA' from dual union all
select 1078,'NASA' from dual union all
select 1079,'NASA' from dual union all
select 108,'UMI' from dual union all
select 108,'USN' from dual union all
select 1080,'NASA' from dual union all
select 1081,'NASA' from dual union all
select 1082,'NASA' from dual union all
select 1083,'NASA' from dual union all
select 1084,'NASA' from dual union all
select 1085,'NASA' from dual union all
select 1086,'NASA' from dual union all
select 1087,'NASA' from dual;
insert into SITE_ORG
select 1088,'NASA' from dual union all
select 1089,'NASA' from dual union all
select 109,'UMI' from dual union all
select 109,'USN' from dual union all
select 1090,'NASA' from dual union all
select 1091,'NASA' from dual union all
select 1092,'NASA' from dual union all
select 1093,'NASA' from dual union all
select 1094,'NASA' from dual union all
select 1095,'NASA' from dual union all
select 1096,'NASA' from dual union all
select 1097,'NASA' from dual union all
select 1098,'NASA' from dual union all
select 1099,'NASA' from dual union all
select 11,'AEB' from dual union all
select 110,'UMI' from dual union all
select 110,'USN' from dual union all
select 1100,'NASA' from dual union all
select 1101,'NASA' from dual union all
select 1102,'NASA' from dual union all
select 1103,'NASA' from dual union all
select 1104,'NASA' from dual union all
select 1105,'NASA' from dual union all
select 1106,'NASA' from dual union all
select 1107,'NASA' from dual union all
select 1108,'NASA' from dual union all
select 1109,'NASA' from dual union all
select 111,'UMI' from dual union all
select 111,'USN' from dual union all
select 1110,'NASA' from dual union all
select 1111,'NASA' from dual union all
select 1112,'NASA' from dual union all
select 1113,'NASA' from dual union all
select 1114,'NASA' from dual union all
select 1115,'NASA' from dual union all
select 1116,'NASA' from dual union all
select 1117,'NASA' from dual union all
select 1118,'NASA' from dual union all
select 1119,'NASA' from dual union all
select 112,'UMI' from dual union all
select 112,'USN' from dual union all
select 1120,'NASA' from dual union all
select 1121,'NASA' from dual union all
select 1122,'NASA' from dual union all
select 1123,'NASA' from dual union all
select 1124,'NASA' from dual union all
select 1125,'NASA' from dual union all
select 1126,'NASA' from dual union all
select 1127,'NASA' from dual union all
select 1128,'NASA' from dual union all
select 1129,'NASA' from dual union all
select 113,'UMI' from dual union all
select 113,'USN' from dual union all
select 1130,'NASA' from dual union all
select 1131,'NASA' from dual union all
select 1132,'NASA' from dual union all
select 1133,'NASA' from dual union all
select 1134,'LRL' from dual union all
select 1134,'USN' from dual union all
select 1135,'USN' from dual union all
select 1136,'AN' from dual union all
select 1137,'AN' from dual union all
select 1138,'AN' from dual union all
select 1139,'AN' from dual union all
select 114,'USN' from dual union all
select 1140,'AN' from dual union all
select 1141,'AN' from dual union all
select 1142,'AN' from dual union all
select 1143,'AN' from dual union all
select 1144,'AN' from dual union all
select 1145,'AN' from dual union all
select 1146,'AN' from dual union all
select 1147,'AN' from dual union all
select 1148,'AN' from dual union all
select 1149,'AN' from dual union all
select 115,'GMS' from dual union all
select 1150,'AN' from dual union all
select 1151,'AN' from dual union all
select 1152,'AN' from dual union all
select 1153,'AN' from dual union all
select 1154,'AN' from dual union all
select 1155,'AN' from dual union all
select 1156,'AN' from dual union all
select 1157,'AN' from dual union all
select 1158,'AN' from dual union all
select 1159,'AN' from dual union all
select 116,'GMS' from dual union all
select 1160,'AN' from dual union all
select 1161,'AN' from dual union all
select 1162,'AN' from dual union all
select 1163,'AN' from dual union all
select 1164,'AN' from dual union all
select 1165,'AN' from dual union all
select 1166,'AN' from dual union all
select 1167,'AN' from dual union all
select 1168,'AN' from dual union all
select 1169,'AN' from dual union all
select 117,'GMS' from dual union all
select 1170,'AN' from dual union all
select 1171,'AN' from dual;
insert into SITE_ORG
select 1172,'AN' from dual union all
select 1173,'AN' from dual union all
select 1174,'AN' from dual union all
select 1175,'AN' from dual union all
select 1176,'AN' from dual union all
select 1177,'AN' from dual union all
select 1178,'AN' from dual union all
select 1179,'AN' from dual union all
select 118,'GMS' from dual union all
select 1180,'AN' from dual union all
select 1181,'AN' from dual union all
select 1182,'AN' from dual union all
select 1183,'AN' from dual union all
select 1184,'AN' from dual union all
select 1185,'AN' from dual union all
select 1186,'AN' from dual union all
select 1187,'AN' from dual union all
select 1188,'AN' from dual union all
select 1189,'AN' from dual union all
select 119,'GMS' from dual union all
select 1190,'AN' from dual union all
select 1191,'AN' from dual union all
select 1192,'AN' from dual union all
select 1193,'AN' from dual union all
select 1194,'AN' from dual union all
select 1195,'AN' from dual union all
select 1196,'AN' from dual union all
select 1197,'AN' from dual union all
select 1198,'AN' from dual union all
select 1199,'AN' from dual union all
select 12,'AEB' from dual union all
select 120,'GMS' from dual union all
select 1200,'AN' from dual union all
select 1201,'AN' from dual union all
select 1202,'AN' from dual union all
select 1203,'AN' from dual union all
select 1204,'AN' from dual union all
select 1206,'AN' from dual union all
select 1207,'AN' from dual union all
select 1208,'SMDC' from dual union all
select 1209,'USN' from dual union all
select 121,'GMS' from dual union all
select 1210,'USN' from dual union all
select 122,'GMS' from dual union all
select 123,'GMS' from dual union all
select 1233,'AN' from dual union all
select 124,'GMS' from dual union all
select 125,'GMS' from dual union all
select 1256,'AN' from dual union all
select 1257,'USN' from dual union all
select 1258,'USN' from dual union all
select 1259,'USN' from dual union all
select 126,'GMS' from dual union all
select 1260,'GMS' from dual union all
select 1261,'GMS' from dual union all
select 1262,'GMS' from dual union all
select 1263,'GMS' from dual union all
select 1264,'GMS' from dual union all
select 1265,'GMS' from dual union all
select 1266,'GMS' from dual union all
select 1267,'GMS' from dual union all
select 1268,'GMS' from dual union all
select 1269,'GMS' from dual union all
select 127,'GMS' from dual union all
select 1270,'AN' from dual union all
select 1271,'USN' from dual union all
select 1272,'USN' from dual union all
select 1273,'USN' from dual union all
select 1274,'USN' from dual union all
select 1275,'USN' from dual union all
select 1276,'MDA' from dual union all
select 1277,'USN' from dual union all
select 1278,'USN' from dual union all
select 1279,'USN' from dual union all
select 128,'GMS' from dual union all
select 1280,'USN' from dual union all
select 1281,'USN' from dual union all
select 1282,'USN' from dual union all
select 1283,'USN' from dual union all
select 1284,'USN' from dual union all
select 1285,'NRL' from dual union all
select 1286,'NRL' from dual union all
select 1287,'NRL' from dual union all
select 1288,'NRL' from dual union all
select 1289,'NRL' from dual union all
select 129,'GMS' from dual union all
select 1290,'NRL' from dual union all
select 1291,'NRL' from dual union all
select 1292,'NRL' from dual union all
select 1293,'NRL' from dual union all
select 1294,'NRL' from dual union all
select 1295,'NRL' from dual union all
select 1296,'CN' from dual union all
select 1297,'SU' from dual union all
select 1298,'SU' from dual union all
select 1299,'SU' from dual union all
select 13,'AEB' from dual union all
select 130,'GMS' from dual union all
select 1300,'SU' from dual union all
select 1301,'SU' from dual;
insert into SITE_ORG
select 1302,'SU' from dual union all
select 1303,'SU' from dual union all
select 1304,'SU' from dual union all
select 1305,'SU' from dual union all
select 1306,'SU' from dual union all
select 1307,'NRCC' from dual union all
select 1308,'AADC' from dual union all
select 1309,'KP' from dual union all
select 131,'GMS' from dual union all
select 1310,'KP' from dual union all
select 1311,'IR' from dual union all
select 1312,'USAF' from dual union all
select 1313,'SAND' from dual union all
select 1314,'F' from dual union all
select 1315,'NRCC' from dual union all
select 1316,'SPFL' from dual union all
select 1317,'NRCC' from dual union all
select 1318,'NRCC' from dual union all
select 1319,'NRCC' from dual union all
select 132,'GMS' from dual union all
select 1320,'NRCC' from dual union all
select 1321,'USAF' from dual union all
select 1322,'SU' from dual union all
select 1323,'SU' from dual union all
select 1324,'NASA' from dual union all
select 1324,'USAF' from dual union all
select 1325,'NASA' from dual union all
select 1325,'USAF' from dual union all
select 1326,'NASA' from dual union all
select 1326,'USAF' from dual union all
select 1327,'NASA' from dual union all
select 1327,'USAF' from dual union all
select 1328,'NASA' from dual union all
select 1328,'USAF' from dual union all
select 1329,'NASA' from dual union all
select 1329,'USAF' from dual union all
select 133,'GMS' from dual union all
select 1330,'BAS' from dual union all
select 1331,'NASA' from dual union all
select 1331,'USAF' from dual union all
select 1332,'JMA' from dual union all
select 1333,'YE' from dual union all
select 1334,'GMS' from dual union all
select 1335,'RVSN' from dual union all
select 1336,'YE' from dual union all
select 1337,'USN' from dual union all
select 1338,'USN' from dual union all
select 1339,'I' from dual union all
select 134,'GMS' from dual union all
select 1340,'I' from dual union all
select 1341,'I' from dual union all
select 1342,'I' from dual union all
select 1343,'I' from dual union all
select 1344,'ISRO' from dual union all
select 1345,'OTRG' from dual union all
select 1346,'IR' from dual union all
select 1347,'IR' from dual union all
select 1348,'IR' from dual union all
select 1349,'IR' from dual union all
select 135,'GMS' from dual union all
select 1350,'NRCC' from dual union all
select 1351,'J' from dual union all
select 1352,'CITED' from dual union all
select 1353,'SCAL' from dual union all
select 1354,'SU' from dual union all
select 1355,'OTRG' from dual union all
select 1356,'IR' from dual union all
select 1357,'ISRO' from dual union all
select 1358,'ISRO' from dual union all
select 1359,'ISRO' from dual union all
select 136,'GMS' from dual union all
select 1360,'ISRO' from dual union all
select 1361,'ISRO' from dual union all
select 1362,'USAF' from dual union all
select 1363,'NASA' from dual union all
select 1364,'CN' from dual union all
select 1365,'SU' from dual union all
select 1366,'SU' from dual union all
select 1367,'SU' from dual union all
select 1368,'SU' from dual union all
select 1369,'SU' from dual union all
select 137,'GMS' from dual union all
select 1370,'SU' from dual union all
select 1371,'SU' from dual union all
select 1372,'SU' from dual union all
select 1373,'SU' from dual union all
select 1374,'SU' from dual union all
select 1375,'SU' from dual union all
select 1376,'SU' from dual union all
select 1377,'SU' from dual union all
select 1378,'SU' from dual union all
select 1379,'SU' from dual union all
select 138,'GMS' from dual union all
select 1380,'SU' from dual union all
select 1381,'SU' from dual union all
select 1382,'SU' from dual union all
select 1383,'SU' from dual union all
select 1384,'SU' from dual union all
select 1385,'SU' from dual union all
select 1386,'SU' from dual;
insert into SITE_ORG
select 1387,'SU' from dual union all
select 1388,'SU' from dual union all
select 1389,'SU' from dual union all
select 139,'GMS' from dual union all
select 1390,'SU' from dual union all
select 1391,'SU' from dual union all
select 1392,'SU' from dual union all
select 1393,'SU' from dual union all
select 1394,'SU' from dual union all
select 1395,'SU' from dual union all
select 1396,'SU' from dual union all
select 1397,'SU' from dual union all
select 1398,'SU' from dual union all
select 1399,'SU' from dual union all
select 14,'SYMC' from dual union all
select 140,'GMS' from dual union all
select 1400,'NASA' from dual union all
select 1400,'USAF' from dual union all
select 1401,'NASA' from dual union all
select 1401,'USAF' from dual union all
select 1402,'NASA' from dual union all
select 1402,'USAF' from dual union all
select 1403,'NASA' from dual union all
select 1403,'USAF' from dual union all
select 1404,'USAF' from dual union all
select 1405,'VMF' from dual union all
select 1406,'KP' from dual union all
select 1407,'NASA' from dual union all
select 1408,'NASA' from dual union all
select 1409,'NASA' from dual union all
select 141,'GMS' from dual union all
select 1410,'I' from dual union all
select 1411,'I' from dual union all
select 1412,'I' from dual union all
select 1413,'USN' from dual union all
select 1414,'USN' from dual union all
select 1415,'USN' from dual union all
select 1416,'CMIK' from dual union all
select 1417,'CMIK' from dual union all
select 1418,'CMIK' from dual union all
select 1419,'SUPA' from dual union all
select 142,'GMS' from dual union all
select 1420,'RVSN' from dual union all
select 1421,'NMSPA' from dual union all
select 1422,'NMSPA' from dual union all
select 1423,'NMSPA' from dual union all
select 1424,'NMSPA' from dual union all
select 1425,'SPFL' from dual union all
select 1426,'SPFL' from dual union all
select 1427,'SPFL' from dual union all
select 1428,'SPX' from dual union all
select 1429,'NASA' from dual union all
select 1429,'USAF' from dual union all
select 143,'GMS' from dual union all
select 1430,'NASA' from dual union all
select 1430,'USAF' from dual union all
select 1431,'SMDC' from dual union all
select 1432,'USAF' from dual union all
select 1433,'USAF' from dual union all
select 1434,'USAF' from dual union all
select 1435,'DLR' from dual union all
select 1436,'USAF' from dual union all
select 1437,'USAF' from dual union all
select 1438,'USAF' from dual union all
select 1439,'ARMF' from dual union all
select 144,'GMS' from dual union all
select 1440,'MOD' from dual union all
select 1441,'MOD' from dual union all
select 1442,'MOD' from dual union all
select 1443,'MOD' from dual union all
select 1444,'MOD' from dual union all
select 1445,'MOD' from dual union all
select 1446,'MOD' from dual union all
select 1447,'MOD' from dual union all
select 1448,'CMIK' from dual union all
select 1449,'CMIK' from dual union all
select 145,'GMS' from dual union all
select 1450,'NASA' from dual union all
select 1451,'NASA' from dual union all
select 1452,'N' from dual union all
select 1453,'N' from dual union all
select 1454,'SU' from dual union all
select 1455,'RVSN' from dual union all
select 1456,'NMSPA' from dual union all
select 1457,'SYMC' from dual union all
select 1458,'J' from dual union all
select 1459,'IST' from dual union all
select 146,'GMS' from dual union all
select 1460,'CONAE' from dual union all
select 1461,'RVSN' from dual union all
select 1462,'ISRO' from dual union all
select 1463,'ISRO' from dual union all
select 1464,'ISRO' from dual union all
select 1465,'ISRO' from dual union all
select 1466,'ISRO' from dual union all
select 1467,'ISRO' from dual union all
select 1468,'RVSN' from dual union all
select 1469,'USAF' from dual union all
select 147,'GMS' from dual union all
select 1470,'USAF' from dual;
insert into SITE_ORG
select 1471,'YE' from dual union all
select 1472,'GMS' from dual union all
select 1473,'PK' from dual union all
select 1474,'NASDA' from dual union all
select 1475,'NASDA' from dual union all
select 1476,'NASDA' from dual union all
select 1477,'NASDA' from dual union all
select 1478,'NASDA' from dual union all
select 1479,'NASDA' from dual union all
select 148,'USN' from dual union all
select 1480,'KPA' from dual union all
select 1481,'NASA' from dual union all
select 1482,'NASA' from dual union all
select 1483,'USAF' from dual union all
select 1484,'NASA' from dual union all
select 1485,'NASA' from dual union all
select 1486,'NASA' from dual union all
select 1487,'LY' from dual union all
select 1488,'SAND' from dual union all
select 1489,'SAND' from dual union all
select 149,'USN' from dual union all
select 1490,'SAND' from dual union all
select 1491,'CN' from dual union all
select 1492,'CN' from dual union all
select 1493,'CN' from dual union all
select 1494,'CN' from dual union all
select 1495,'RVSN' from dual union all
select 15,'SYMC' from dual union all
select 150,'USN' from dual union all
select 151,'USN' from dual union all
select 152,'USN' from dual union all
select 1520,'SU' from dual union all
select 1521,'SU' from dual union all
select 1522,'SU' from dual union all
select 153,'USN' from dual union all
select 154,'USN' from dual union all
select 155,'USN' from dual union all
select 1556,'SU' from dual union all
select 156,'USN' from dual union all
select 157,'USA' from dual union all
select 1570,'SU' from dual union all
select 158,'USN' from dual union all
select 159,'USN' from dual union all
select 1590,'SU' from dual union all
select 16,'SYMC' from dual union all
select 160,'USN' from dual union all
select 161,'USN' from dual union all
select 1618,'SU' from dual union all
select 162,'USN' from dual union all
select 1622,'SU' from dual union all
select 1623,'SU' from dual union all
select 1624,'SU' from dual union all
select 1625,'SU' from dual union all
select 1626,'VMF' from dual union all
select 1627,'WEHR' from dual union all
select 1628,'RVSN' from dual union all
select 1629,'CN' from dual union all
select 163,'USN' from dual union all
select 1630,'ISAS' from dual union all
select 1631,'SU' from dual union all
select 1632,'SU' from dual union all
select 1633,'SU' from dual union all
select 1634,'SU' from dual union all
select 1635,'SU' from dual union all
select 1636,'SU' from dual union all
select 1637,'SU' from dual union all
select 1638,'SU' from dual union all
select 1639,'SU' from dual union all
select 164,'USN' from dual union all
select 1640,'SU' from dual union all
select 1641,'SU' from dual union all
select 1642,'SU' from dual union all
select 1643,'SU' from dual union all
select 1644,'SU' from dual union all
select 1645,'SU' from dual union all
select 1646,'RVSN' from dual union all
select 1647,'USAF' from dual union all
select 1648,'USAF' from dual union all
select 1649,'USAF' from dual union all
select 165,'USN' from dual union all
select 1650,'USAF' from dual union all
select 1651,'USAF' from dual union all
select 1652,'USAF' from dual union all
select 1653,'USAF' from dual union all
select 1654,'USAF' from dual union all
select 1655,'USAF' from dual union all
select 1656,'USAF' from dual union all
select 1657,'USAF' from dual union all
select 1658,'USAF' from dual union all
select 1659,'USAF' from dual union all
select 166,'NPOL' from dual union all
select 1660,'USAF' from dual union all
select 1661,'USAF' from dual union all
select 1662,'USAF' from dual union all
select 1663,'USAF' from dual union all
select 1664,'USAF' from dual union all
select 1665,'USAF' from dual union all
select 1666,'USAF' from dual union all
select 1667,'USAF' from dual union all
select 1668,'USAF' from dual;
insert into SITE_ORG
select 1669,'USAF' from dual union all
select 167,'NPOL' from dual union all
select 1670,'USAF' from dual union all
select 1671,'USAF' from dual union all
select 1672,'USAF' from dual union all
select 1673,'USAF' from dual union all
select 1674,'USAF' from dual union all
select 1675,'USAF' from dual union all
select 1676,'USAF' from dual union all
select 1677,'USAF' from dual union all
select 1678,'USAF' from dual union all
select 1679,'USAF' from dual union all
select 168,'SUI' from dual union all
select 1680,'USAF' from dual union all
select 1681,'USAF' from dual union all
select 1682,'USAF' from dual union all
select 1683,'USAF' from dual union all
select 1684,'USAF' from dual union all
select 1685,'USAF' from dual union all
select 1686,'USAF' from dual union all
select 1687,'USAF' from dual union all
select 1688,'USAF' from dual union all
select 1689,'USAF' from dual union all
select 169,'SUI' from dual union all
select 1690,'USAF' from dual union all
select 1691,'USAF' from dual union all
select 1692,'USAF' from dual union all
select 1693,'USAF' from dual union all
select 1694,'USAF' from dual union all
select 1695,'USAF' from dual union all
select 1696,'USAF' from dual union all
select 1697,'USAF' from dual union all
select 1698,'USAF' from dual union all
select 1699,'USAF' from dual union all
select 17,'SA' from dual union all
select 170,'SUI' from dual union all
select 1700,'USAF' from dual union all
select 1701,'USAF' from dual union all
select 1702,'USAF' from dual union all
select 1703,'USAF' from dual union all
select 1704,'USAF' from dual union all
select 1705,'USAF' from dual union all
select 1706,'USAF' from dual union all
select 1707,'USAF' from dual union all
select 1708,'USAF' from dual union all
select 1709,'USAF' from dual union all
select 171,'SUI' from dual union all
select 1710,'USAF' from dual union all
select 1711,'USAF' from dual union all
select 1712,'USAF' from dual union all
select 1713,'USAF' from dual union all
select 1714,'USAF' from dual union all
select 1715,'USAF' from dual union all
select 1716,'USAF' from dual union all
select 1717,'USAF' from dual union all
select 1718,'USAF' from dual union all
select 1719,'USAF' from dual union all
select 172,'SUI' from dual union all
select 1720,'USAF' from dual union all
select 1721,'USAF' from dual union all
select 1722,'USAF' from dual union all
select 1723,'NASA' from dual union all
select 1724,'USAF' from dual union all
select 1725,'USAF' from dual union all
select 1726,'USAF' from dual union all
select 1727,'USAF' from dual union all
select 1728,'WEHR' from dual union all
select 1729,'WEHR' from dual union all
select 173,'SUI' from dual union all
select 1730,'WEHR' from dual union all
select 1731,'WEHR' from dual union all
select 1732,'WEHR' from dual union all
select 1733,'WEHR' from dual union all
select 1734,'WEHR' from dual union all
select 1735,'WEHR' from dual union all
select 1736,'WEHR' from dual union all
select 1737,'WEHR' from dual union all
select 1738,'WEHR' from dual union all
select 1739,'WEHR' from dual union all
select 174,'SUI' from dual union all
select 1740,'WEHR' from dual union all
select 1741,'WEHR' from dual union all
select 1742,'WEHR' from dual union all
select 1743,'WEHR' from dual union all
select 1744,'WEHR' from dual union all
select 1745,'WEHR' from dual union all
select 1746,'WEHR' from dual union all
select 1747,'AR' from dual union all
select 1748,'ARMF' from dual union all
select 1749,'CNES' from dual union all
select 175,'SUI' from dual union all
select 1750,'DRDO' from dual union all
select 1751,'VVS' from dual union all
select 1752,'VVS' from dual union all
select 1753,'SU' from dual union all
select 1754,'SU' from dual union all
select 1755,'SU' from dual union all
select 1756,'SU' from dual union all
select 1757,'SU' from dual union all
select 1758,'SU' from dual;
insert into SITE_ORG
select 1759,'SU' from dual union all
select 176,'USAF' from dual union all
select 1760,'SU' from dual union all
select 1761,'SU' from dual union all
select 1762,'SU' from dual union all
select 1763,'RVSN' from dual union all
select 1764,'RVSN' from dual union all
select 1765,'RVSN' from dual union all
select 1766,'USAF' from dual union all
select 1767,'USAF' from dual union all
select 1768,'USAF' from dual union all
select 1769,'USAF' from dual union all
select 177,'PVO' from dual union all
select 1770,'USAF' from dual union all
select 1771,'USAF' from dual union all
select 1772,'USAF' from dual union all
select 1773,'USAF' from dual union all
select 1774,'USAF' from dual union all
select 1775,'USAF' from dual union all
select 1776,'USAF' from dual union all
select 1777,'USAF' from dual union all
select 1778,'USAF' from dual union all
select 1779,'NASA' from dual union all
select 178,'SMDC' from dual union all
select 1780,'USAF' from dual union all
select 1781,'SSI' from dual union all
select 1782,'USAF' from dual union all
select 1783,'USAF' from dual union all
select 1784,'USAF' from dual union all
select 1785,'USAF' from dual union all
select 1786,'USAF' from dual union all
select 1787,'CN' from dual union all
select 1788,'CN' from dual union all
select 1789,'CN' from dual union all
select 179,'MO' from dual union all
select 1790,'USN' from dual union all
select 1791,'USN' from dual union all
select 1792,'NASA' from dual union all
select 1793,'NASA' from dual union all
select 1794,'NASA' from dual union all
select 1795,'NASA' from dual union all
select 1796,'NASA' from dual union all
select 1797,'NASA' from dual union all
select 1798,'NASA' from dual union all
select 1799,'NASA' from dual union all
select 18,'SAC' from dual union all
select 180,'MO' from dual union all
select 1800,'NASA' from dual union all
select 1801,'NASA' from dual union all
select 1802,'NASA' from dual union all
select 1803,'NASA' from dual union all
select 1804,'NASA' from dual union all
select 1805,'NASA' from dual union all
select 1806,'NASA' from dual union all
select 1807,'NASA' from dual union all
select 1808,'NASA' from dual union all
select 1809,'NASA' from dual union all
select 181,'MO' from dual union all
select 1810,'NASA' from dual union all
select 1811,'NASA' from dual union all
select 1812,'NASA' from dual union all
select 1813,'NASA' from dual union all
select 1814,'NASA' from dual union all
select 1815,'NASA' from dual union all
select 1816,'NASA' from dual union all
select 1817,'NASA' from dual union all
select 1818,'NASA' from dual union all
select 1819,'NASA' from dual union all
select 182,'MO' from dual union all
select 1820,'NASA' from dual union all
select 1821,'NASA' from dual union all
select 1822,'NASA' from dual union all
select 1823,'USAF' from dual union all
select 1824,'USAF' from dual union all
select 1825,'USAF' from dual union all
select 1826,'USAF' from dual union all
select 1827,'CMIK' from dual union all
select 1828,'CMIK' from dual union all
select 1829,'CMIK' from dual union all
select 183,'MO' from dual union all
select 1830,'CMIK' from dual union all
select 1831,'WRE' from dual union all
select 1832,'WRE' from dual union all
select 1833,'WRE' from dual union all
select 1834,'WRE' from dual union all
select 1835,'WRE' from dual union all
select 1836,'WRE' from dual union all
select 1837,'WRE' from dual union all
select 1838,'WRE' from dual union all
select 1839,'WRE' from dual union all
select 184,'USAF' from dual union all
select 1840,'WRE' from dual union all
select 1841,'WRE' from dual union all
select 1842,'WRE' from dual union all
select 1843,'WRE' from dual union all
select 1844,'WRE' from dual union all
select 1845,'WRE' from dual union all
select 1846,'WRE' from dual union all
select 1847,'WRE' from dual union all
select 1848,'WRE' from dual;
insert into SITE_ORG
select 1849,'WRE' from dual union all
select 185,'USAF' from dual union all
select 1850,'USN' from dual union all
select 1851,'USN' from dual union all
select 1852,'SMDC' from dual union all
select 1853,'SMDC' from dual union all
select 1854,'SMDC' from dual union all
select 1855,'SMDC' from dual union all
select 1856,'SMDC' from dual union all
select 1857,'SMDC' from dual union all
select 1858,'USN' from dual union all
select 1859,'SMDC' from dual union all
select 186,'USAF' from dual union all
select 1860,'SMDC' from dual union all
select 1861,'NASA' from dual union all
select 1862,'NASA' from dual union all
select 1863,'SMDC' from dual union all
select 1864,'SMDC' from dual union all
select 1865,'SMDC' from dual union all
select 1866,'SMDC' from dual union all
select 1867,'SMDC' from dual union all
select 1868,'SMDC' from dual union all
select 1869,'SMDC' from dual union all
select 187,'USAF' from dual union all
select 1870,'SMDC' from dual union all
select 1871,'SMDC' from dual union all
select 1872,'SMDC' from dual union all
select 1873,'SMDC' from dual union all
select 1874,'SMDC' from dual union all
select 1875,'SMDC' from dual union all
select 1876,'SMDC' from dual union all
select 1877,'SMDC' from dual union all
select 1878,'SMDC' from dual union all
select 1879,'SMDC' from dual union all
select 188,'NASA' from dual union all
select 188,'USAF' from dual union all
select 1880,'SMDC' from dual union all
select 1881,'SMDC' from dual union all
select 1882,'SMDC' from dual union all
select 1883,'SMDC' from dual union all
select 1884,'SMDC' from dual union all
select 1885,'SMDC' from dual union all
select 1886,'SMDC' from dual union all
select 1887,'SMDC' from dual union all
select 1888,'SMDC' from dual union all
select 1889,'SMDC' from dual union all
select 189,'SPX' from dual union all
select 1890,'USA' from dual union all
select 1891,'SMDC' from dual union all
select 1892,'SMDC' from dual union all
select 1893,'NASA' from dual union all
select 1894,'USA' from dual union all
select 1895,'SMDC' from dual union all
select 1896,'BLOR' from dual union all
select 1897,'USN' from dual union all
select 1898,'USN' from dual union all
select 1899,'USN' from dual union all
select 19,'SAC' from dual union all
select 190,'VMF' from dual union all
select 1900,'USN' from dual union all
select 1901,'USAF' from dual union all
select 1902,'CN' from dual union all
select 1903,'CN' from dual union all
select 1904,'CN' from dual union all
select 1905,'CN' from dual union all
select 1906,'RVSN' from dual union all
select 1907,'RVSN' from dual union all
select 1908,'RVSN' from dual union all
select 1909,'CN' from dual union all
select 191,'APL' from dual union all
select 1910,'YE' from dual union all
select 1911,'RVSN' from dual union all
select 1912,'NASA' from dual union all
select 1913,'BRL' from dual union all
select 1914,'BRL' from dual union all
select 1915,'USA' from dual union all
select 1916,'RVSN' from dual union all
select 1917,'DD' from dual union all
select 192,'APL' from dual union all
select 193,'APL' from dual union all
select 194,'NASA' from dual union all
select 195,'SU' from dual union all
select 196,'VMF' from dual union all
select 197,'UCAN' from dual union all
select 198,'PLA' from dual union all
select 199,'USN' from dual union all
select 2,'RAE' from dual union all
select 20,'IQ' from dual union all
select 200,'VMF' from dual union all
select 201,'VMF' from dual union all
select 202,'USA' from dual union all
select 203,'USA' from dual union all
select 204,'USA' from dual union all
select 205,'CN' from dual union all
select 206,'DGA' from dual union all
select 207,'DGA' from dual union all
select 208,'USDI' from dual union all
select 209,'SU' from dual union all
select 21,'USAF' from dual union all
select 210,'SU' from dual;
insert into SITE_ORG
select 211,'SU' from dual union all
select 212,'SU' from dual union all
select 213,'SU' from dual union all
select 214,'SU' from dual union all
select 215,'SU' from dual union all
select 216,'SU' from dual union all
select 217,'SU' from dual union all
select 218,'DGA' from dual union all
select 219,'WRE' from dual union all
select 22,'IQ' from dual union all
select 220,'WRE' from dual union all
select 221,'INPE' from dual union all
select 221,'NASA' from dual union all
select 222,'USAF' from dual union all
select 223,'USAF' from dual union all
select 224,'USAF' from dual union all
select 225,'USAF' from dual union all
select 226,'USAF' from dual union all
select 227,'USAF' from dual union all
select 228,'USAF' from dual union all
select 229,'USAF' from dual union all
select 23,'ARR' from dual union all
select 230,'USAF' from dual union all
select 231,'USAF' from dual union all
select 232,'USAF' from dual union all
select 233,'USAF' from dual union all
select 234,'USAF' from dual union all
select 235,'USAF' from dual union all
select 236,'USAF' from dual union all
select 237,'USAF' from dual union all
select 238,'USAF' from dual union all
select 239,'USAF' from dual union all
select 24,'ARR' from dual union all
select 240,'USAF' from dual union all
select 241,'USAF' from dual union all
select 242,'USAF' from dual union all
select 243,'USAF' from dual union all
select 244,'USN' from dual union all
select 245,'USN' from dual union all
select 246,'USN' from dual union all
select 247,'USN' from dual union all
select 248,'USAF' from dual union all
select 249,'USAF' from dual union all
select 25,'ARR' from dual union all
select 250,'USN' from dual union all
select 251,'USN' from dual union all
select 252,'USN' from dual union all
select 253,'USAF' from dual union all
select 254,'USAF' from dual union all
select 255,'USA' from dual union all
select 256,'USA' from dual union all
select 257,'USA' from dual union all
select 258,'USA' from dual union all
select 259,'USA' from dual union all
select 26,'ARR' from dual union all
select 260,'USA' from dual union all
select 261,'USAF' from dual union all
select 262,'USAF' from dual union all
select 263,'USAF' from dual union all
select 264,'USAF' from dual union all
select 265,'USAF' from dual union all
select 266,'USAF' from dual union all
select 267,'NASA' from dual union all
select 268,'USAF' from dual union all
select 269,'USAF' from dual union all
select 27,'ARR' from dual union all
select 270,'NASA' from dual union all
select 271,'NASA' from dual union all
select 272,'USAF' from dual union all
select 273,'USAF' from dual union all
select 274,'USAF' from dual union all
select 275,'USAF' from dual union all
select 276,'USAF' from dual union all
select 277,'USAF' from dual union all
select 278,'USAF' from dual union all
select 279,'USAF' from dual union all
select 28,'ARR' from dual union all
select 280,'USAF' from dual union all
select 281,'USAF' from dual union all
select 282,'USAF' from dual union all
select 283,'USAF' from dual union all
select 284,'USAF' from dual union all
select 285,'USAF' from dual union all
select 286,'USAF' from dual union all
select 287,'USAF' from dual union all
select 288,'USAF' from dual union all
select 289,'USAF' from dual union all
select 29,'ARR' from dual union all
select 290,'NASA' from dual union all
select 291,'USAF' from dual union all
select 292,'USAF' from dual union all
select 293,'USAF' from dual union all
select 294,'SPFL' from dual union all
select 295,'SPFL' from dual union all
select 296,'USAF' from dual union all
select 297,'USAF' from dual union all
select 298,'USAF' from dual union all
select 299,'F' from dual union all
select 3,'RVSN' from dual union all
select 30,'ADD' from dual;
insert into SITE_ORG
select 300,'F' from dual union all
select 301,'F' from dual union all
select 302,'F' from dual union all
select 303,'F' from dual union all
select 304,'F' from dual union all
select 305,'F' from dual union all
select 306,'F' from dual union all
select 307,'F' from dual union all
select 308,'AR' from dual union all
select 309,'F' from dual union all
select 31,'ADD' from dual union all
select 310,'AR' from dual union all
select 311,'F' from dual union all
select 312,'F' from dual union all
select 313,'F' from dual union all
select 314,'CNES' from dual union all
select 315,'DRDO' from dual union all
select 316,'DRDO' from dual union all
select 317,'DRDO' from dual union all
select 318,'DRDO' from dual union all
select 319,'DRDO' from dual union all
select 32,'ADD' from dual union all
select 320,'DRDO' from dual union all
select 321,'RVSN' from dual union all
select 322,'NASA' from dual union all
select 323,'PE' from dual union all
select 324,'KPA' from dual union all
select 325,'NSPO' from dual union all
select 326,'VMF' from dual union all
select 327,'VMF' from dual union all
select 328,'USAF' from dual union all
select 329,'BR' from dual union all
select 33,'ADD' from dual union all
select 330,'IAE' from dual union all
select 331,'RCAR' from dual union all
select 332,'ZWO' from dual union all
select 333,'CA' from dual union all
select 334,'CA' from dual union all
select 335,'NPOL' from dual union all
select 336,'NPOL' from dual union all
select 337,'NPOL' from dual union all
select 338,'NPOL' from dual union all
select 339,'USAF' from dual union all
select 34,'NASA' from dual union all
select 340,'CNES' from dual union all
select 341,'CNES' from dual union all
select 342,'CNES' from dual union all
select 343,'CNES' from dual union all
select 344,'CNES' from dual union all
select 345,'CNES' from dual union all
select 346,'CNES' from dual union all
select 347,'STSM' from dual union all
select 348,'CNES' from dual union all
select 349,'CNES' from dual union all
select 35,'USAF' from dual union all
select 350,'CNES' from dual union all
select 351,'NASA' from dual union all
select 351,'USAF' from dual union all
select 352,'NASA' from dual union all
select 352,'USAF' from dual union all
select 353,'UK' from dual union all
select 354,'NATO' from dual union all
select 355,'USA' from dual union all
select 356,'LRS' from dual union all
select 357,'NASA' from dual union all
select 357,'USAF' from dual union all
select 358,'NASA' from dual union all
select 358,'USAF' from dual union all
select 359,'NASA' from dual union all
select 359,'USAF' from dual union all
select 36,'USN' from dual union all
select 360,'NASA' from dual union all
select 360,'USAF' from dual union all
select 361,'CNES' from dual union all
select 362,'IR' from dual union all
select 363,'RVSN' from dual union all
select 364,'NASA' from dual union all
select 365,'NASA' from dual union all
select 366,'EG' from dual union all
select 367,'USN' from dual union all
select 368,'RVSN' from dual union all
select 369,'OSC' from dual union all
select 37,'SUI' from dual union all
select 370,'OSC' from dual union all
select 371,'OSC' from dual union all
select 372,'JPL' from dual union all
select 373,'OSC' from dual union all
select 374,'USN' from dual union all
select 375,'NOTS' from dual union all
select 376,'OSC' from dual union all
select 377,'USAF' from dual union all
select 378,'USAF' from dual union all
select 379,'USAF' from dual union all
select 38,'SUI' from dual union all
select 380,'USAF' from dual union all
select 381,'USAF' from dual union all
select 382,'USAF' from dual union all
select 383,'USAF' from dual union all
select 384,'RVSN' from dual union all
select 385,'USAF' from dual;
insert into SITE_ORG
select 386,'USAF' from dual union all
select 387,'USAF' from dual union all
select 388,'USAF' from dual union all
select 389,'USAF' from dual union all
select 39,'SUI' from dual union all
select 390,'USAF' from dual union all
select 391,'USAF' from dual union all
select 392,'USAF' from dual union all
select 393,'USAF' from dual union all
select 394,'USAF' from dual union all
select 395,'E' from dual union all
select 396,'E' from dual union all
select 397,'SAC' from dual union all
select 398,'SAC' from dual union all
select 399,'SAC' from dual union all
select 4,'USAF' from dual union all
select 40,'NRL' from dual union all
select 400,'SAC' from dual union all
select 401,'SAC' from dual union all
select 402,'SAC' from dual union all
select 403,'SAC' from dual union all
select 404,'MO' from dual union all
select 405,'USAF' from dual union all
select 406,'NRCC' from dual union all
select 407,'ESRO' from dual union all
select 408,'ESRO' from dual union all
select 409,'ESRO' from dual union all
select 41,'NRL' from dual union all
select 410,'ESRO' from dual union all
select 411,'ESRO' from dual union all
select 412,'ESRO' from dual union all
select 413,'ESRO' from dual union all
select 414,'ESRO' from dual union all
select 415,'ESRO' from dual union all
select 416,'ESRO' from dual union all
select 417,'ESRO' from dual union all
select 418,'USN' from dual union all
select 419,'USN' from dual union all
select 42,'NRL' from dual union all
select 420,'USN' from dual union all
select 421,'USN' from dual union all
select 422,'CMIK' from dual union all
select 423,'NRCC' from dual union all
select 424,'NRCC' from dual union all
select 425,'NRCC' from dual union all
select 426,'NRCC' from dual union all
select 427,'NRCC' from dual union all
select 428,'NRCC' from dual union all
select 429,'NPOL' from dual union all
select 43,'NRL' from dual union all
select 430,'NPOL' from dual union all
select 431,'NASA' from dual union all
select 432,'NASA' from dual union all
select 433,'USA' from dual union all
select 434,'USA' from dual union all
select 435,'USA' from dual union all
select 436,'USA' from dual union all
select 437,'SMDC' from dual union all
select 438,'USA' from dual union all
select 439,'USA' from dual union all
select 44,'SUI' from dual union all
select 440,'USA' from dual union all
select 441,'USA' from dual union all
select 442,'USA' from dual union all
select 443,'USA' from dual union all
select 444,'USA' from dual union all
select 445,'USA' from dual union all
select 446,'USA' from dual union all
select 447,'USAF' from dual union all
select 448,'NASA' from dual union all
select 449,'NASA' from dual union all
select 45,'SUI' from dual union all
select 450,'UKMO' from dual union all
select 451,'OSC' from dual union all
select 452,'OSC' from dual union all
select 453,'USAF' from dual union all
select 454,'RVSN' from dual union all
select 455,'RVSN' from dual union all
select 456,'RVSN' from dual union all
select 457,'RVSN' from dual union all
select 458,'RVSN' from dual union all
select 459,'RVSN' from dual union all
select 46,'SUI' from dual union all
select 460,'RVSN' from dual union all
select 461,'RVSN' from dual union all
select 462,'RVSN' from dual union all
select 463,'RVSN' from dual union all
select 464,'RVSN' from dual union all
select 465,'RVSN' from dual union all
select 466,'RVSN' from dual union all
select 467,'RVSN' from dual union all
select 468,'RVSN' from dual union all
select 469,'RVSN' from dual union all
select 47,'SUI' from dual union all
select 470,'RVSN' from dual union all
select 471,'RVSN' from dual union all
select 472,'RVSN' from dual union all
select 473,'RVSN' from dual union all
select 474,'RVSN' from dual union all
select 475,'RVSN' from dual;
insert into SITE_ORG
select 476,'RVSN' from dual union all
select 477,'RVSN' from dual union all
select 478,'RVSN' from dual union all
select 479,'RVSN' from dual union all
select 48,'SUI' from dual union all
select 480,'RVSN' from dual union all
select 481,'RVSN' from dual union all
select 482,'RVSN' from dual union all
select 483,'RVSN' from dual union all
select 484,'RVSN' from dual union all
select 485,'RVSN' from dual union all
select 486,'RVSN' from dual union all
select 487,'RVSN' from dual union all
select 488,'RVSN' from dual union all
select 489,'RVSN' from dual union all
select 49,'SUI' from dual union all
select 490,'RVSN' from dual union all
select 491,'RVSN' from dual union all
select 492,'RVSN' from dual union all
select 493,'RVSN' from dual union all
select 494,'RVSN' from dual union all
select 495,'RVSN' from dual union all
select 496,'RVSN' from dual union all
select 497,'RVSN' from dual union all
select 498,'RVSN' from dual union all
select 499,'RVSN' from dual union all
select 5,'USAF' from dual union all
select 50,'SUI' from dual union all
select 500,'RVSN' from dual union all
select 501,'RVSN' from dual union all
select 502,'RVSN' from dual union all
select 503,'RVSN' from dual union all
select 504,'RVSN' from dual union all
select 505,'RVSN' from dual union all
select 506,'RVSN' from dual union all
select 507,'RVSN' from dual union all
select 508,'RVSN' from dual union all
select 509,'RVSN' from dual union all
select 51,'SUI' from dual union all
select 510,'RVSN' from dual union all
select 511,'RVSN' from dual union all
select 512,'RVSN' from dual union all
select 513,'RVSN' from dual union all
select 514,'RVSN' from dual union all
select 515,'RVSN' from dual union all
select 516,'RVSN' from dual union all
select 517,'RVSN' from dual union all
select 518,'RVSN' from dual union all
select 519,'RVSN' from dual union all
select 52,'SUI' from dual union all
select 520,'RVSN' from dual union all
select 521,'RVSN' from dual union all
select 522,'RVSN' from dual union all
select 523,'RVSN' from dual union all
select 524,'RVSN' from dual union all
select 525,'RVSN' from dual union all
select 526,'RVSN' from dual union all
select 527,'RVSN' from dual union all
select 528,'RVSN' from dual union all
select 529,'RVSN' from dual union all
select 53,'SUI' from dual union all
select 530,'RVSN' from dual union all
select 531,'RVSN' from dual union all
select 532,'RVSN' from dual union all
select 533,'RVSN' from dual union all
select 534,'RVSN' from dual union all
select 535,'RVSN' from dual union all
select 536,'RVSN' from dual union all
select 537,'RVSN' from dual union all
select 538,'RVSN' from dual union all
select 539,'RVSN' from dual union all
select 54,'SUI' from dual union all
select 540,'RVSN' from dual union all
select 541,'RVSN' from dual union all
select 542,'RVSN' from dual union all
select 543,'RVSN' from dual union all
select 544,'RVSN' from dual union all
select 545,'RVSN' from dual union all
select 546,'RVSN' from dual union all
select 547,'RVSN' from dual union all
select 548,'RVSN' from dual union all
select 549,'RVSN' from dual union all
select 55,'SUI' from dual union all
select 550,'RVSN' from dual union all
select 551,'RVSN' from dual union all
select 552,'RVSN' from dual union all
select 553,'RVSN' from dual union all
select 554,'RVSN' from dual union all
select 555,'RVSN' from dual union all
select 556,'RVSN' from dual union all
select 557,'RVSN' from dual union all
select 558,'RVSN' from dual union all
select 559,'RVSN' from dual union all
select 56,'SUI' from dual union all
select 560,'RVSN' from dual union all
select 561,'RVSN' from dual union all
select 562,'RVSN' from dual union all
select 563,'RVSN' from dual union all
select 564,'RVSN' from dual union all
select 565,'RVSN' from dual;
insert into SITE_ORG
select 566,'RVSN' from dual union all
select 567,'RVSN' from dual union all
select 568,'RVSN' from dual union all
select 569,'RVSN' from dual union all
select 57,'SUI' from dual union all
select 570,'RVSN' from dual union all
select 571,'RVSN' from dual union all
select 572,'RVSN' from dual union all
select 573,'RVSN' from dual union all
select 574,'RVSN' from dual union all
select 575,'RVSN' from dual union all
select 576,'RVSN' from dual union all
select 577,'RVSN' from dual union all
select 578,'RVSN' from dual union all
select 579,'RVSN' from dual union all
select 58,'NRL' from dual union all
select 580,'RVSN' from dual union all
select 581,'RVSN' from dual union all
select 582,'RVSN' from dual union all
select 583,'RVSN' from dual union all
select 584,'RVSN' from dual union all
select 585,'RVSN' from dual union all
select 586,'RVSN' from dual union all
select 587,'RVSN' from dual union all
select 588,'RVSN' from dual union all
select 589,'RVSN' from dual union all
select 59,'NRL' from dual union all
select 590,'RVSN' from dual union all
select 591,'RVSN' from dual union all
select 592,'RVSN' from dual union all
select 593,'RVSN' from dual union all
select 594,'RVSN' from dual union all
select 595,'RVSN' from dual union all
select 596,'RVSN' from dual union all
select 597,'USA' from dual union all
select 598,'NRCC' from dual union all
select 599,'I' from dual union all
select 6,'NIMHB' from dual union all
select 60,'SUI' from dual union all
select 600,'RVSN' from dual union all
select 601,'RLABN' from dual union all
select 602,'RVSN' from dual union all
select 603,'RVSN' from dual union all
select 604,'RVSN' from dual union all
select 605,'RVSN' from dual union all
select 606,'RVSN' from dual union all
select 607,'RVSN' from dual union all
select 608,'RVSN' from dual union all
select 609,'RVSN' from dual union all
select 61,'SUI' from dual union all
select 610,'RVSN' from dual union all
select 611,'ADD' from dual union all
select 612,'USAF' from dual union all
select 613,'USAF' from dual union all
select 614,'USAF' from dual union all
select 615,'USAF' from dual union all
select 616,'USA' from dual union all
select 617,'USA' from dual union all
select 618,'USAF' from dual union all
select 619,'USAF' from dual union all
select 62,'SUI' from dual union all
select 620,'RVSN' from dual union all
select 621,'RVSN' from dual union all
select 622,'RVSN' from dual union all
select 623,'RVSN' from dual union all
select 624,'RVSN' from dual union all
select 625,'RVSN' from dual union all
select 626,'RVSN' from dual union all
select 627,'RVSN' from dual union all
select 628,'RVSN' from dual union all
select 629,'RVSN' from dual union all
select 63,'SUI' from dual union all
select 630,'RVSN' from dual union all
select 631,'RVSN' from dual union all
select 632,'RVSN' from dual union all
select 633,'RVSN' from dual union all
select 634,'RVSN' from dual union all
select 635,'RVSN' from dual union all
select 636,'RVSN' from dual union all
select 637,'USA' from dual union all
select 638,'NASA' from dual union all
select 639,'NASA' from dual union all
select 64,'SUI' from dual union all
select 640,'USAF' from dual union all
select 641,'USAF' from dual union all
select 642,'USAF' from dual union all
select 643,'USAF' from dual union all
select 644,'USAF' from dual union all
select 645,'USAF' from dual union all
select 646,'USAF' from dual union all
select 647,'CN' from dual union all
select 648,'SYMC' from dual union all
select 649,'HAL5' from dual union all
select 65,'SUI' from dual union all
select 650,'CN' from dual union all
select 651,'NASA' from dual union all
select 651,'USAF' from dual union all
select 652,'NASA' from dual union all
select 652,'USAF' from dual union all
select 653,'NASA' from dual;
insert into SITE_ORG
select 653,'USAF' from dual union all
select 654,'NASA' from dual union all
select 654,'USAF' from dual union all
select 655,'NASA' from dual union all
select 655,'USAF' from dual union all
select 656,'NASA' from dual union all
select 656,'USAF' from dual union all
select 657,'NASA' from dual union all
select 657,'USAF' from dual union all
select 658,'NASA' from dual union all
select 658,'USAF' from dual union all
select 659,'NASA' from dual union all
select 659,'USAF' from dual union all
select 66,'NRL' from dual union all
select 660,'NASA' from dual union all
select 660,'USAF' from dual union all
select 661,'WEHR' from dual union all
select 662,'WEHR' from dual union all
select 663,'F' from dual union all
select 664,'F' from dual union all
select 665,'F' from dual union all
select 666,'F' from dual union all
select 667,'F' from dual union all
select 668,'F' from dual union all
select 669,'F' from dual union all
select 67,'NRL' from dual union all
select 670,'F' from dual union all
select 671,'F' from dual union all
select 672,'PLANS' from dual union all
select 673,'WEHR' from dual union all
select 674,'WEHR' from dual union all
select 675,'WEHR' from dual union all
select 676,'WEHR' from dual union all
select 677,'WEHR' from dual union all
select 678,'CMIK' from dual union all
select 679,'USN' from dual union all
select 68,'NRL' from dual union all
select 680,'USN' from dual union all
select 681,'USN' from dual union all
select 682,'USN' from dual union all
select 683,'USN' from dual union all
select 684,'USN' from dual union all
select 685,'USN' from dual union all
select 686,'USN' from dual union all
select 687,'USN' from dual union all
select 688,'USN' from dual union all
select 689,'USN' from dual union all
select 69,'NRL' from dual union all
select 690,'USN' from dual union all
select 691,'USN' from dual union all
select 692,'USN' from dual union all
select 693,'USN' from dual union all
select 694,'USN' from dual union all
select 695,'USN' from dual union all
select 696,'USN' from dual union all
select 697,'USN' from dual union all
select 698,'USN' from dual union all
select 699,'USN' from dual union all
select 7,'ISAS' from dual union all
select 70,'NRL' from dual union all
select 700,'USN' from dual union all
select 701,'USN' from dual union all
select 702,'USN' from dual union all
select 703,'USN' from dual union all
select 704,'USN' from dual union all
select 705,'USN' from dual union all
select 706,'USN' from dual union all
select 707,'USN' from dual union all
select 708,'USN' from dual union all
select 709,'USN' from dual union all
select 71,'NRL' from dual union all
select 710,'USN' from dual union all
select 711,'USN' from dual union all
select 712,'USN' from dual union all
select 713,'USN' from dual union all
select 714,'USN' from dual union all
select 715,'USN' from dual union all
select 716,'USN' from dual union all
select 717,'USN' from dual union all
select 718,'USN' from dual union all
select 719,'USN' from dual union all
select 72,'SUI' from dual union all
select 720,'USN' from dual union all
select 721,'USN' from dual union all
select 722,'USN' from dual union all
select 723,'USN' from dual union all
select 724,'USN' from dual union all
select 725,'USN' from dual union all
select 726,'USN' from dual union all
select 727,'USN' from dual union all
select 728,'USN' from dual union all
select 729,'USN' from dual union all
select 73,'SUI' from dual union all
select 730,'USN' from dual union all
select 731,'USN' from dual union all
select 732,'USN' from dual union all
select 733,'USN' from dual union all
select 734,'USN' from dual union all
select 735,'SU' from dual union all
select 736,'SU' from dual;
insert into SITE_ORG
select 737,'SU' from dual union all
select 738,'SU' from dual union all
select 739,'SU' from dual union all
select 74,'SUI' from dual union all
select 740,'SU' from dual union all
select 741,'SU' from dual union all
select 742,'SU' from dual union all
select 743,'SU' from dual union all
select 744,'SU' from dual union all
select 745,'IQ' from dual union all
select 746,'IQ' from dual union all
select 747,'IQ' from dual union all
select 748,'IQ' from dual union all
select 749,'IQ' from dual union all
select 75,'SUI' from dual union all
select 750,'IQ' from dual union all
select 751,'IQ' from dual union all
select 752,'IQ' from dual union all
select 753,'IQ' from dual union all
select 754,'IQ' from dual union all
select 755,'IQ' from dual union all
select 756,'IQ' from dual union all
select 757,'IQ' from dual union all
select 758,'IQ' from dual union all
select 759,'IR' from dual union all
select 76,'SUI' from dual union all
select 760,'IQ' from dual union all
select 761,'IQ' from dual union all
select 762,'IQ' from dual union all
select 763,'IR' from dual union all
select 764,'JPL' from dual union all
select 765,'SPFL' from dual union all
select 766,'EGAO' from dual union all
select 767,'EGAO' from dual union all
select 768,'IQ' from dual union all
select 769,'IQ' from dual union all
select 77,'SUI' from dual union all
select 770,'USAF' from dual union all
select 771,'USAF' from dual union all
select 772,'USAF' from dual union all
select 773,'USAF' from dual union all
select 774,'USAF' from dual union all
select 775,'USAF' from dual union all
select 776,'USAF' from dual union all
select 777,'USAF' from dual union all
select 778,'USAF' from dual union all
select 779,'USAF' from dual union all
select 78,'SUI' from dual union all
select 780,'USAF' from dual union all
select 781,'USAF' from dual union all
select 782,'USAF' from dual union all
select 783,'USAF' from dual union all
select 784,'USAF' from dual union all
select 785,'USAF' from dual union all
select 786,'USAF' from dual union all
select 787,'USAF' from dual union all
select 788,'USAF' from dual union all
select 789,'USAF' from dual union all
select 79,'SUI' from dual union all
select 790,'USAF' from dual union all
select 791,'USAF' from dual union all
select 792,'USAF' from dual union all
select 793,'USAF' from dual union all
select 794,'USAF' from dual union all
select 795,'USAF' from dual union all
select 796,'USAF' from dual union all
select 797,'USAF' from dual union all
select 798,'USAF' from dual union all
select 799,'USAF' from dual union all
select 8,'AEB' from dual union all
select 80,'SUI' from dual union all
select 800,'USAF' from dual union all
select 801,'USAF' from dual union all
select 802,'USAF' from dual union all
select 803,'USAF' from dual union all
select 804,'CN' from dual union all
select 805,'PLANS' from dual union all
select 806,'PLA' from dual union all
select 807,'USAF' from dual union all
select 808,'NSPO' from dual union all
select 809,'CN' from dual union all
select 81,'SUI' from dual union all
select 810,'CN' from dual union all
select 811,'CN' from dual union all
select 812,'CN' from dual union all
select 813,'CN' from dual union all
select 814,'CN' from dual union all
select 815,'CN' from dual union all
select 816,'CN' from dual union all
select 817,'CN' from dual union all
select 818,'CN' from dual union all
select 819,'CN' from dual union all
select 82,'SUI' from dual union all
select 820,'USA' from dual union all
select 821,'KP' from dual union all
select 822,'RVSN' from dual union all
select 823,'NASA' from dual union all
select 824,'NASA' from dual union all
select 825,'NASA' from dual union all
select 826,'RVSN' from dual;
insert into SITE_ORG
select 827,'RVSN' from dual union all
select 828,'RVSN' from dual union all
select 829,'RVSN' from dual union all
select 83,'SUI' from dual union all
select 830,'RVSN' from dual union all
select 831,'ESRO' from dual union all
select 832,'ISAS' from dual union all
select 833,'ISAS' from dual union all
select 834,'ISAS' from dual union all
select 835,'ISAS' from dual union all
select 836,'ISAS' from dual union all
select 837,'ISAS' from dual union all
select 838,'ISAS' from dual union all
select 839,'ISAS' from dual union all
select 84,'SUI' from dual union all
select 840,'RVSN' from dual union all
select 841,'USN' from dual union all
select 842,'USN' from dual union all
select 843,'USN' from dual union all
select 844,'USN' from dual union all
select 845,'USN' from dual union all
select 846,'USN' from dual union all
select 847,'USN' from dual union all
select 848,'USN' from dual union all
select 849,'USN' from dual union all
select 85,'SUI' from dual union all
select 850,'USN' from dual union all
select 851,'USN' from dual union all
select 852,'USN' from dual union all
select 853,'USN' from dual union all
select 854,'USN' from dual union all
select 855,'USN' from dual union all
select 856,'CNES' from dual union all
select 857,'IR' from dual union all
select 858,'UMI' from dual union all
select 859,'CMIK' from dual union all
select 86,'SUI' from dual union all
select 860,'BOSL' from dual union all
select 861,'AADC' from dual union all
select 862,'AADC' from dual union all
select 863,'AADC' from dual union all
select 864,'USAF' from dual union all
select 865,'USAF' from dual union all
select 866,'SMDC' from dual union all
select 867,'SMDC' from dual union all
select 868,'SMDC' from dual union all
select 869,'SMDC' from dual union all
select 87,'SUI' from dual union all
select 870,'SMDC' from dual union all
select 871,'SMDC' from dual union all
select 872,'SMDC' from dual union all
select 873,'SMDC' from dual union all
select 874,'ESRO' from dual union all
select 874,'NASA' from dual union all
select 875,'ESRO' from dual union all
select 875,'NASA' from dual union all
select 876,'ESRO' from dual union all
select 876,'NASA' from dual union all
select 877,'SU' from dual union all
select 878,'SU' from dual union all
select 879,'SU' from dual union all
select 88,'SUI' from dual union all
select 880,'SU' from dual union all
select 881,'SU' from dual union all
select 882,'SU' from dual union all
select 883,'SU' from dual union all
select 884,'SU' from dual union all
select 885,'SU' from dual union all
select 886,'SU' from dual union all
select 887,'SU' from dual union all
select 888,'SU' from dual union all
select 889,'SU' from dual union all
select 89,'SUI' from dual union all
select 890,'SU' from dual union all
select 891,'SU' from dual union all
select 892,'SU' from dual union all
select 893,'SU' from dual union all
select 894,'SU' from dual union all
select 895,'SU' from dual union all
select 896,'SU' from dual union all
select 897,'RVSN' from dual union all
select 898,'RVSN' from dual union all
select 899,'SU' from dual union all
select 9,'AEB' from dual union all
select 90,'SUI' from dual union all
select 900,'SU' from dual union all
select 901,'SU' from dual union all
select 902,'SU' from dual union all
select 903,'SU' from dual union all
select 904,'SU' from dual union all
select 905,'SU' from dual union all
select 906,'SU' from dual union all
select 907,'SU' from dual union all
select 908,'SU' from dual union all
select 909,'SU' from dual union all
select 91,'SUI' from dual union all
select 910,'SU' from dual union all
select 911,'SU' from dual union all
select 912,'SU' from dual union all
select 913,'SU' from dual;
insert into SITE_ORG
select 914,'SU' from dual union all
select 915,'SU' from dual union all
select 916,'SU' from dual union all
select 917,'SU' from dual union all
select 918,'SU' from dual union all
select 919,'NASA' from dual union all
select 919,'SSC' from dual union all
select 92,'SUI' from dual union all
select 920,'SSC' from dual union all
select 921,'NASA' from dual union all
select 922,'NASA' from dual union all
select 923,'NASA' from dual union all
select 924,'NASA' from dual union all
select 925,'KP' from dual union all
select 926,'KP' from dual union all
select 927,'KP' from dual union all
select 928,'KP' from dual union all
select 929,'KP' from dual union all
select 93,'SUI' from dual union all
select 930,'SAND' from dual union all
select 931,'SIMED' from dual union all
select 932,'LAPAN' from dual union all
select 933,'PL' from dual union all
select 934,'NASA' from dual union all
select 935,'NASA' from dual union all
select 936,'SAC' from dual union all
select 937,'SAC' from dual union all
select 938,'USAF' from dual union all
select 939,'LUNA' from dual union all
select 94,'SUI' from dual union all
select 940,'RLABN' from dual union all
select 941,'RLABN' from dual union all
select 942,'RVSN' from dual union all
select 943,'RVSN' from dual union all
select 944,'NASA' from dual union all
select 945,'NASA' from dual union all
select 946,'NASA' from dual union all
select 947,'IR' from dual union all
select 948,'SSI' from dual union all
select 949,'AR' from dual union all
select 95,'SUI' from dual union all
select 950,'USAF' from dual union all
select 951,'IDF' from dual union all
select 952,'AR' from dual union all
select 953,'OSC' from dual union all
select 954,'OSC' from dual union all
select 955,'SCAL' from dual union all
select 956,'SCAL' from dual union all
select 957,'SCAL' from dual union all
select 958,'USA' from dual union all
select 959,'USA' from dual union all
select 96,'SUI' from dual union all
select 960,'SYMC' from dual union all
select 961,'SU' from dual union all
select 962,'N' from dual union all
select 963,'AEC' from dual union all
select 963,'USN' from dual union all
select 964,'NASA' from dual union all
select 964,'USAF' from dual union all
select 965,'NASA' from dual union all
select 965,'USAF' from dual union all
select 966,'NASA' from dual union all
select 966,'USAF' from dual union all
select 967,'NASA' from dual union all
select 967,'USAF' from dual union all
select 968,'NASA' from dual union all
select 968,'USAF' from dual union all
select 969,'NASA' from dual union all
select 969,'USAF' from dual union all
select 97,'SUI' from dual union all
select 970,'KP' from dual union all
select 971,'SU' from dual union all
select 972,'SU' from dual union all
select 973,'SU' from dual union all
select 974,'SU' from dual union all
select 975,'SU' from dual union all
select 976,'SU' from dual union all
select 977,'SU' from dual union all
select 978,'SU' from dual union all
select 979,'SU' from dual union all
select 98,'SUI' from dual union all
select 980,'SU' from dual union all
select 981,'SU' from dual union all
select 982,'SU' from dual union all
select 983,'SU' from dual union all
select 984,'NATO' from dual union all
select 985,'CMIK' from dual union all
select 986,'USN' from dual union all
select 987,'KARI' from dual union all
select 988,'IL' from dual union all
select 989,'VMF' from dual union all
select 99,'SUI' from dual union all
select 990,'STA' from dual union all
select 991,'RVSN' from dual union all
select 992,'RVSN' from dual union all
select 993,'SU' from dual union all
select 994,'SU' from dual union all
select 995,'RVSN' from dual union all
select 996,'USN' from dual union all
select 997,'USN' from dual;
insert into SITE_ORG
select 998,'USN' from dual union all
select 999,'KPNO' from dual;


--------------------------------------------------------------------------------
-- LAUNCH_VEHICLE_FAMILY
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing LAUNCH_VEHICLE_FAMILY...');

CREATE TABLE "LAUNCH_VEHICLE_FAMILY" 
 (	"LV_FAMILY_CODE" VARCHAR2(21), 
	"LV_FAMILY_CLASS" VARCHAR2(28), 
	 CONSTRAINT "LAUNCH_VEHICLE_FAMILY_PK" PRIMARY KEY ("LV_FAMILY_CODE")
USING INDEX  ENABLE
 ) ;

insert into LAUNCH_VEHICLE_FAMILY
select '10KS2500','sounding rocket' from dual union all
select '10SM','intermediate stage 1' from dual union all
select '11A52B','intermediate stage 1' from dual union all
select '11A52V','intermediate stage 1' from dual union all
select '20SM','intermediate stage 1' from dual union all
select '25SM','intermediate stage 1' from dual union all
select '40SM','intermediate stage 1' from dual union all
select '48N6','sounding rocket' from dual union all
select '51T6B','intermediate stage 1' from dual union all
select '5inSM','intermediate stage 1' from dual union all
select '8K75B','intermediate stage 1' from dual union all
select 'A-350','sounding rocket' from dual union all
select 'AAD','sounding rocket' from dual union all
select 'AJ60','intermediate stage 1' from dual union all
select 'AMROC','sounding rocket' from dual union all
select 'ASLV','orbital family' from dual union all
select 'ASLV.St2','intermediate stage 1' from dual union all
select 'ASLV.St3','intermediate stage 1' from dual union all
select 'ASLV.St4','orbital' from dual union all
select 'ATACMS','intermediate stage 1' from dual union all
select 'AUS-51','orbital' from dual union all
select 'AVUM','orbital' from dual union all
select 'Abid','missile' from dual union all
select 'Abid.St3','intermediate stage 1' from dual union all
select 'Able','orbital' from dual union all
select 'Ablestar','orbital' from dual union all
select 'AeroHTV','sounding rocket' from dual union all
select 'AeroHTV2','sounding rocket' from dual union all
select 'Aerobee','sounding rocket' from dual union all
select 'AerobeeB','missile' from dual union all
select 'Agate','missile' from dual union all
select 'Agena','orbital' from dual union all
select 'Agni3','missile' from dual union all
select 'AgniII.St2','intermediate stage 1' from dual union all
select 'Air','astronaut suborbital' from dual union all
select 'Ajax','intermediate stage 1' from dual union all
select 'AlKahir','missile test rocket' from dual union all
select 'Alcor','intermediate stage 1' from dual union all
select 'Alfa','missile' from dual union all
select 'Altair','orbital' from dual union all
select 'Angara','orbital family' from dual union all
select 'Antares','intermediate stage 1' from dual union all
select 'Apache','intermediate stage 1' from dual union all
select 'Apollo.LES','test rocket' from dual union all
select 'Arcas','sounding rocket' from dual union all
select 'ArcasB','missile' from dual union all
select 'Arcon','sounding rocket' from dual union all
select 'Ariane','orbital family' from dual union all
select 'Ariane.St2','intermediate stage 1' from dual union all
select 'Ariane.St3','orbital' from dual union all
select 'Ariane5','orbital family' from dual union all
select 'Aries','sounding rocket' from dual union all
select 'Arrow','intermediate stage 1' from dual union all
select 'Arrow2','missile' from dual union all
select 'Arrow3','missile' from dual union all
select 'Asp','intermediate stage 1' from dual union all
select 'Astra','sounding rocket' from dual union all
select 'Astrid','test rocket' from dual union all
select 'Astris','orbital' from dual union all
select 'AstrobeeD','sounding rocket' from dual union all
select 'AstrobeeF','sounding rocket' from dual union all
select 'Atea','missile' from dual union all
select 'Atlas','orbital family' from dual union all
select 'Atlas.Booster','intermediate stage 1' from dual union all
select 'Atlas5','orbital family' from dual union all
select 'AtlasVSRM','intermediate stage 1' from dual union all
select 'Aurore','intermediate stage 1' from dual union all
select 'BB3','sounding rocket' from dual union all
select 'BB5','sounding rocket' from dual union all
select 'BB6','sounding rocket' from dual union all
select 'BB7','weather (small)' from dual union all
select 'BE3','orbital' from dual union all
select 'BLDT','test rocket' from dual union all
select 'BOZ','intermediate stage 1' from dual union all
select 'BR','orbital' from dual union all
select 'BabySergeant','orbital' from dual union all
select 'Belenos','weather (small)' from dual union all
select 'Belier','sounding rocket' from dual union all
select 'Belisama','weather (small)' from dual union all
select 'Berenice','intermediate stage 1' from dual union all
select 'BlackArrow','orbital family' from dual union all
select 'BlackArrow.St2','intermediate stage 1' from dual union all
select 'BlackKnight','sounding rocket' from dual union all
select 'BlackSparrow','intermediate stage 1' from dual union all
select 'Blok-A','intermediate stage 1' from dual union all
select 'Blok-D','orbital' from dual union all
select 'Blok-E','orbital' from dual union all
select 'Blok-I','orbital' from dual union all
select 'Blok-L','orbital' from dual union all
select 'BlokG','intermediate stage 1' from dual union all
select 'BlueStreak','orbital family' from dual union all
select 'Bomarc','missile' from dual union all
select 'Briz','orbital' from dual union all
select 'Bukguekseong','missile' from dual union all
select 'BullGoose','missile' from dual union all
select 'Bullpup','sounding rocket' from dual union all
select 'Burya','missile' from dual union all
select 'CB-250','sounding rocket' from dual union all
select 'CSXT','sounding rocket' from dual union all
select 'CZ.LB','intermediate stage 1' from dual;
insert into LAUNCH_VEHICLE_FAMILY
select 'CZ.St2','orbital' from dual union all
select 'CZ1.St2','intermediate stage 1' from dual union all
select 'CZ11-3','intermediate stage 1' from dual union all
select 'CZ11-4','intermediate stage 1' from dual union all
select 'CZ3.St3','orbital' from dual union all
select 'CZ5','orbital family' from dual union all
select 'Castor4','sounding rocket' from dual union all
select 'Cedre','sounding rocket' from dual union all
select 'Cedre.St2','sounding rocket' from dual union all
select 'Cedre.St3','sounding rocket' from dual union all
select 'Centaur','orbital' from dual union all
select 'Cetus','orbital' from dual union all
select 'Chick','intermediate stage 1' from dual union all
select 'Cirrus','sounding rocket' from dual union all
select 'Cluster','orbital' from dual union all
select 'Coralie','intermediate stage 1' from dual union all
select 'Corporal','missile' from dual union all
select 'Cuckoo','missile' from dual union all
select 'Cygnus','intermediate stage 1' from dual union all
select 'D3SS','orbital' from dual union all
select 'D4SS','orbital' from dual union all
select 'DACRoc','sounding rocket' from dual union all
select 'DC-X','test rocket' from dual union all
select 'DF-11','missile' from dual union all
select 'DF-15','missile' from dual union all
select 'DF-16','missile' from dual union all
select 'DF-21','missile' from dual union all
select 'DF-31','missile' from dual union all
select 'DF3','orbital family' from dual union all
select 'DF5','orbital family' from dual union all
select 'DIM','sounding rocket' from dual union all
select 'DS','orbital' from dual union all
select 'Daniel','sounding rocket' from dual union all
select 'Deacon','missile' from dual union all
select 'Delta2','orbital' from dual union all
select 'Delta4','orbital family' from dual union all
select 'Demon','sounding rocket' from dual union all
select 'Dolphin','test rocket' from dual union all
select 'Dragon','test rocket' from dual union all
select 'Dummy','intermediate stage 1' from dual union all
select 'E8','orbital family' from dual union all
select 'EPBS','orbital' from dual union all
select 'EPS','orbital' from dual union all
select 'Electron','orbital family' from dual union all
select 'Emeraude','orbital family' from dual union all
select 'Emma','weather (small)' from dual union all
select 'Energiya','orbital family' from dual union all
select 'Excalibur','missile' from dual union all
select 'FFAR','sounding rocket' from dual union all
select 'FFK','sounding rocket' from dual union all
select 'FTB','sounding rocket' from dual union all
select 'FTI','sounding rocket' from dual union all
select 'FW4','orbital' from dual union all
select 'Falcon','intermediate stage 1' from dual union all
select 'Falcon1','orbital family' from dual union all
select 'Falcon9','orbital family' from dual union all
select 'Falcon9.St2','intermediate stage 1' from dual union all
select 'Fatah','sounding rocket' from dual union all
select 'Fregat','orbital' from dual union all
select 'GCR','orbital' from dual union all
select 'GEM','missile' from dual union all
select 'GEM60','missile' from dual union all
select 'GEM63','missile' from dual union all
select 'GF-02','orbital' from dual union all
select 'GLCM','missile' from dual union all
select 'GS3','orbital' from dual union all
select 'GSLV','orbital family' from dual union all
select 'GSLV3','orbital family' from dual union all
select 'GSLV3.St2','intermediate stage 1' from dual union all
select 'GSOM','intermediate stage 1' from dual union all
select 'GammaCen','sounding rocket' from dual union all
select 'GammaCenS','sounding rocket' from dual union all
select 'Generic','sounding rocket' from dual union all
select 'Genie','missile' from dual union all
select 'Gosling','sounding rocket' from dual union all
select 'Grad','missile' from dual union all
select 'Gradicom','sounding rocket' from dual union all
select 'Gun05','sounding rocket' from dual union all
select 'Gun07','sounding rocket' from dual union all
select 'Gun16','sounding rocket' from dual union all
select 'H2','orbital family' from dual union all
select 'H2.SRB','intermediate stage 1' from dual union all
select 'H2.SSB','intermediate stage 1' from dual union all
select 'H2.St2','orbital' from dual union all
select 'H2B','orbital family' from dual union all
select 'HALO','sounding rocket' from dual union all
select 'HAPS','orbital' from dual union all
select 'HIBEX','astronaut suborbital' from dual union all
select 'HJ','missile' from dual union all
select 'HOTROC','orbital family' from dual union all
select 'HTR','test rocket' from dual union all
select 'HVAR','sounding rocket' from dual union all
select 'HYSR','test rocket' from dual union all
select 'Hades','missile' from dual union all
select 'Hercules','intermediate stage 1' from dual union all
select 'Hermes','astronaut suborbital' from dual union all
select 'HermesA3','astronaut suborbital' from dual union all
select 'Hopi','missile' from dual union all
select 'HoundDog','missile' from dual union all
select 'Hwasong-12','missile' from dual;
insert into LAUNCH_VEHICLE_FAMILY
select 'Hwasong-15','missile' from dual union all
select 'Hyunmoo','missile' from dual union all
select 'IAI.Arrow.St2','intermediate stage 1' from dual union all
select 'IIAE.Orion','sounding rocket' from dual union all
select 'INTA-100','sounding rocket' from dual union all
select 'IRIS.Fiat','orbital' from dual union all
select 'ISRO-TV','sounding rocket' from dual union all
select 'Icarus','sounding rocket' from dual union all
select 'Icarus2','sounding rocket' from dual union all
select 'Ikar','orbital' from dual union all
select 'Interstage','intermediate stage 1' from dual union all
select 'IraqGun350','sounding rocket' from dual union all
select 'Iris','sounding rocket' from dual union all
select 'Iroquois','intermediate stage 1' from dual union all
select 'Ivanov','sounding rocket' from dual union all
select 'JCR','sounding rocket' from dual union all
select 'Javelin','intermediate stage 1' from dual union all
select 'Jericho','orbital family' from dual union all
select 'Jupiter','orbital family' from dual union all
select 'K-15','missile' from dual union all
select 'K-4','missile' from dual union all
select 'K10S','orbital' from dual union all
select 'K150','sounding rocket' from dual union all
select 'K250','sounding rocket' from dual union all
select 'K420','sounding rocket' from dual union all
select 'KM-D','intermediate stage 1' from dual union all
select 'KM-H','intermediate stage 1' from dual union all
select 'KM-L','intermediate stage 1' from dual union all
select 'KM-V','orbital' from dual union all
select 'KM-V2','orbital' from dual union all
select 'KSLV.St2','intermediate stage 1' from dual union all
select 'KSR','sounding rocket' from dual union all
select 'KSR-3','sounding rocket' from dual union all
select 'KT1-3','orbital' from dual union all
select 'KZ2-3','intermediate stage 1' from dual union all
select 'Kappa','sounding rocket' from dual union all
select 'Kappa4','missile' from dual union all
select 'Kavoshgar','sounding rocket' from dual union all
select 'Kiva','sounding rocket' from dual union all
select 'Kontakt','missile' from dual union all
select 'Kosmos-2.St2','orbital' from dual union all
select 'Kosmos-3.St2','orbital' from dual union all
select 'Kuryer','missile' from dual union all
select 'Kuryer.St3','missile' from dual union all
select 'L14','orbital' from dual union all
select 'L480S','orbital' from dual union all
select 'L500','intermediate stage 1' from dual union all
select 'LAPS','orbital' from dual union all
select 'LAPSTAR','intermediate stage 1' from dual union all
select 'LE-3','orbital' from dual union all
select 'LE-5','orbital' from dual union all
select 'LEO-46','missile' from dual union all
select 'LEO-7','missile' from dual union all
select 'LJ1','test rocket' from dual union all
select 'LJ2','sounding rocket' from dual union all
select 'LMAS','orbital family' from dual union all
select 'LPR-006','sounding rocket' from dual union all
select 'LS-C','sounding rocket' from dual union all
select 'Lacrosse','sounding rocket' from dual union all
select 'Lambda','orbital family' from dual union all
select 'Lambda.St2','intermediate stage 1' from dual union all
select 'Lance','intermediate stage 1' from dual union all
select 'LanceMotor','intermediate stage 1' from dual union all
select 'Lark','missile' from dual union all
select 'Lex','sounding rocket' from dual union all
select 'Lobster','intermediate stage 1' from dual union all
select 'Loki','missile' from dual union all
select 'LongTom','missile' from dual union all
select 'Luna','missile' from dual union all
select 'Lupus','sounding rocket' from dual union all
select 'Luzou','sounding rocket' from dual union all
select 'M-100','sounding rocket' from dual union all
select 'M-20','intermediate stage 1' from dual union all
select 'M-24','intermediate stage 1' from dual union all
select 'M-30','orbital' from dual union all
select 'M-34','orbital' from dual union all
select 'M-35','intermediate stage 1' from dual union all
select 'M-40','orbital' from dual union all
select 'M51','missile' from dual union all
select 'MLRS','intermediate stage 1' from dual union all
select 'MMR-06','weather (small)' from dual union all
select 'MR-12','sounding rocket' from dual union all
select 'MR-UR-100','missile' from dual union all
select 'MR-UR-100.St2','intermediate stage 1' from dual union all
select 'MSBS','missile' from dual union all
select 'MSBS.St2','intermediate stage 1' from dual union all
select 'MSBS.St3','intermediate stage 1' from dual union all
select 'MT-135','weather (small)' from dual union all
select 'MV','orbital family' from dual union all
select 'MX','orbital family' from dual union all
select 'MX-774','astronaut suborbital' from dual union all
select 'MX.St2','intermediate stage 1' from dual union all
select 'MX.St3','intermediate stage 1' from dual union all
select 'MX.St4','intermediate stage 1' from dual union all
select 'Malemute','sounding rocket' from dual union all
select 'Matador','missile' from dual union all
select 'Mera','weather (small)' from dual union all
select 'Meteo','weather (small)' from dual union all
select 'Meteor','weather (small)' from dual union all
select 'Meteor2','sounding rocket' from dual;
insert into LAUNCH_VEHICLE_FAMILY
select 'Mimosa','intermediate stage 1' from dual union all
select 'Minuteman','orbital family' from dual union all
select 'Minuteman.St3','intermediate stage 1' from dual union all
select 'Mk72','missile' from dual union all
select 'Momo','sounding rocket' from dual union all
select 'Monica','sounding rocket' from dual union all
select 'Mu','orbital family' from dual union all
select 'Musca','intermediate stage 1' from dual union all
select 'N-1','orbital family' from dual union all
select 'NAL','sounding rocket' from dual union all
select 'NOTS.3inSM','intermediate stage 1' from dual union all
select 'NOTS.8in','intermediate stage 1' from dual union all
select 'NOTS400','sounding rocket' from dual union all
select 'NOTS500','sounding rocket' from dual union all
select 'NOTS551','intermediate stage 1' from dual union all
select 'NSPO','sounding rocket' from dual union all
select 'NSPOB','missile' from dual union all
select 'Navaho','missile' from dual union all
select 'Neptune','intermediate stage 1' from dual union all
select 'NewShepard','intermediate stage 1' from dual union all
select 'Nike','missile' from dual union all
select 'Nike-46','missile' from dual union all
select 'Nodong','orbital family' from dual union all
select 'OAM','orbital' from dual union all
select 'OTRAG','test rocket' from dual union all
select 'OV','orbital family' from dual union all
select 'Oka','missile' from dual union all
select 'Orbus1','intermediate stage 1' from dual union all
select 'Orbus21','orbital' from dual union all
select 'Orbus6','orbital' from dual union all
select 'Oriole','intermediate stage 1' from dual union all
select 'Orion','sounding rocket' from dual union all
select 'Orion.LAS','missile' from dual union all
select 'P-10','missile' from dual union all
select 'P-16','missile' from dual union all
select 'P068','orbital' from dual union all
select 'PAL','intermediate stage 1' from dual union all
select 'PAM-D2','orbital' from dual union all
select 'PAP','intermediate stage 1' from dual union all
select 'PBS','intermediate stage 1' from dual union all
select 'PRS-1','missile' from dual union all
select 'PSLV','orbital family' from dual union all
select 'PSLV.St2','intermediate stage 1' from dual union all
select 'PSLV.St3','intermediate stage 1' from dual union all
select 'PSLV.St4','orbital' from dual union all
select 'Pedro','sounding rocket' from dual union all
select 'Pegasus','orbital family' from dual union all
select 'Pegasus.St2','intermediate stage 1' from dual union all
select 'Pegasus.St3','orbital' from dual union all
select 'Peregrine','sounding rocket' from dual union all
select 'Perekhodnik','intermediate stage 1' from dual union all
select 'Pershing','missile' from dual union all
select 'Pershing.St2','intermediate stage 1' from dual union all
select 'Petrel','sounding rocket' from dual union all
select 'Plastic','intermediate stage 1' from dual union all
select 'Polaris','missile' from dual union all
select 'Polaris.St2','intermediate stage 1' from dual union all
select 'PolarisA3.St2','intermediate stage 1' from dual union all
select 'Poseidon','missile' from dual union all
select 'Poseidon.St2','intermediate stage 1' from dual union all
select 'Prithvi','missile' from dual union all
select 'PrivateA','missile' from dual union all
select 'PrivateBooster','missile' from dual union all
select 'Proson','sounding rocket' from dual union all
select 'Proson2','intermediate stage 1' from dual union all
select 'Proton','orbital family' from dual union all
select 'Proton.St3','intermediate stage 1' from dual union all
select 'R-11','missile' from dual union all
select 'R-12','orbital family' from dual union all
select 'R-13','missile' from dual union all
select 'R-14','orbital family' from dual union all
select 'R-16','missile' from dual union all
select 'R-16.St2','intermediate stage 1' from dual union all
select 'R-21','missile' from dual union all
select 'R-27','missile' from dual union all
select 'R-29','orbital family' from dual union all
select 'R-29.St2','intermediate stage 1' from dual union all
select 'R-29.St3','intermediate stage 1' from dual union all
select 'R-31','missile' from dual union all
select 'R-31.St2','intermediate stage 1' from dual union all
select 'R-36','orbital family' from dual union all
select 'R-36.St2','orbital' from dual union all
select 'R-39','missile' from dual union all
select 'R-39.St2','intermediate stage 1' from dual union all
select 'R-39.St3','intermediate stage 1' from dual union all
select 'R-5','missile' from dual union all
select 'R-7','orbital family' from dual union all
select 'R-9','missile' from dual union all
select 'RH-125','weather (small)' from dual union all
select 'RH-200','sounding rocket' from dual union all
select 'RH-300','sounding rocket' from dual union all
select 'RH-560','sounding rocket' from dual union all
select 'RH-75','weather (small)' from dual union all
select 'RT-1.St3','intermediate stage 1' from dual union all
select 'RT-15','missile' from dual union all
select 'RT-15.St2','intermediate stage 1' from dual union all
select 'RT-15.St3','intermediate stage 1' from dual union all
select 'RT-2','missile' from dual union all
select 'RT-2.St3','intermediate stage 1' from dual union all
select 'RT-20P','missile' from dual;
insert into LAUNCH_VEHICLE_FAMILY
select 'RT-20P.St2','intermediate stage 1' from dual union all
select 'RT-21.St2','intermediate stage 1' from dual union all
select 'RT-23.St2','intermediate stage 1' from dual union all
select 'RT-23.St3','intermediate stage 1' from dual union all
select 'RTV','astronaut suborbital' from dual union all
select 'RX-250','sounding rocket' from dual union all
select 'RX-320','sounding rocket' from dual union all
select 'RX-420','sounding rocket' from dual union all
select 'Raven','sounding rocket' from dual union all
select 'Recruit','sounding rocket' from dual union all
select 'Redstone','orbital family' from dual union all
select 'Regulus','missile' from dual union all
select 'Rigel','sounding rocket' from dual union all
select 'Rita','intermediate stage 1' from dual union all
select 'Rocketsonde','sounding rocket' from dual union all
select 'Rocketsonde2','sounding rocket' from dual union all
select 'Rubis','intermediate stage 1' from dual union all
select 'S-160','sounding rocket' from dual union all
select 'S-210','sounding rocket' from dual union all
select 'S-300','sounding rocket' from dual union all
select 'S-310','sounding rocket' from dual union all
select 'S-40','orbital family' from dual union all
select 'S-44','intermediate stage 1' from dual union all
select 'S-520','sounding rocket' from dual union all
select 'S5M','orbital' from dual union all
select 'SB-310','intermediate stage 1' from dual union all
select 'SB-735','test rocket' from dual union all
select 'SD','orbital' from dual union all
select 'SEPR167','intermediate stage 1' from dual union all
select 'SICBM','missile' from dual union all
select 'SICBM2','intermediate stage 1' from dual union all
select 'SICBM3','intermediate stage 1' from dual union all
select 'SICBM4','intermediate stage 1' from dual union all
select 'SM-3-II','sounding rocket' from dual union all
select 'SM3.TSRM','intermediate stage 1' from dual union all
select 'SOZ','intermediate stage 1' from dual union all
select 'SR19','missile' from dual union all
select 'SR73','intermediate stage 1' from dual union all
select 'SRAM','missile' from dual union all
select 'SRB','orbital family' from dual union all
select 'SRMU','intermediate stage 1' from dual union all
select 'SS-520-4','intermediate stage 1' from dual union all
select 'SS1','sounding rocket' from dual union all
select 'SS2','intermediate stage 1' from dual union all
select 'STA','sounding rocket' from dual union all
select 'STIG','sounding rocket' from dual union all
select 'STIG-B','sounding rocket' from dual union all
select 'STS','orbital family' from dual union all
select 'STS.OV','orbital' from dual union all
select 'Safir','intermediate stage 1' from dual union all
select 'Samoud','sounding rocket' from dual union all
select 'Sandhawk','sounding rocket' from dual union all
select 'Saturn','orbital family' from dual union all
select 'SaturnII','orbital' from dual union all
select 'SaturnIV','orbital' from dual union all
select 'SaturnIVB','orbital' from dual union all
select 'SaturnV','orbital family' from dual union all
select 'Scout','missile' from dual union all
select 'ScoutD','orbital family' from dual union all
select 'Sejjil','missile' from dual union all
select 'Seliger','sounding rocket' from dual union all
select 'Sergeant','missile' from dual union all
select 'Shaviyt','intermediate stage 1' from dual union all
select 'Sidewinder','missile' from dual union all
select 'Simorgh.St2','intermediate stage 1' from dual union all
select 'Sirocco','weather (small)' from dual union all
select 'Skua','sounding rocket' from dual union all
select 'Skybolt','missile' from dual union all
select 'Skybolt.St2','intermediate stage 1' from dual union all
select 'Snark','missile' from dual union all
select 'Sonda','sounding rocket' from dual union all
select 'Sonda3','sounding rocket' from dual union all
select 'SpaceLoft','sounding rocket' from dual union all
select 'Sparrow','missile' from dual union all
select 'Spartan','missile' from dual union all
select 'Sprint','missile' from dual union all
select 'Squirt','missile' from dual union all
select 'Standard','intermediate stage 1' from dual union all
select 'Star20','orbital' from dual union all
select 'Star26','orbital' from dual union all
select 'Star27','orbital' from dual union all
select 'Star37','orbital' from dual union all
select 'Star48','orbital' from dual union all
select 'Start','intermediate stage 1' from dual union all
select 'Stonechat','sounding rocket' from dual union all
select 'Stratos2','sounding rocket' from dual union all
select 'Stromboli','sounding rocket' from dual union all
select 'SuperLoki','missile' from dual union all
select 'T-40','astronaut suborbital' from dual union all
select 'T-55','intermediate stage 1' from dual union all
select 'T-7','sounding rocket' from dual union all
select 'T-7B','sounding rocket' from dual union all
select 'T48','missile' from dual union all
select 'TD3','intermediate stage 1' from dual union all
select 'THAAD','missile' from dual union all
select 'TR-1','sounding rocket' from dual union all
select 'TR-1.St2','intermediate stage 1' from dual union all
select 'TT-210','sounding rocket' from dual union all
select 'TTB','missile' from dual union all
select 'TX-238','intermediate stage 1' from dual;
insert into LAUNCH_VEHICLE_FAMILY
select 'TX-239','intermediate stage 1' from dual union all
select 'Talos','missile' from dual union all
select 'Temp','missile' from dual union all
select 'Temp-2S','missile' from dual union all
select 'Terrier','missile' from dual union all
select 'Thor','orbital family' from dual union all
select 'Tianying','sounding rocket' from dual union all
select 'Titan','orbital family' from dual union all
select 'Titan.SRM','intermediate stage 1' from dual union all
select 'Titan.St2','orbital' from dual union all
select 'Tochka','missile' from dual union all
select 'Tomahawk','sounding rocket' from dual union all
select 'Topol','orbital family' from dual union all
select 'Topol.St2','intermediate stage 1' from dual union all
select 'Topol.St3','intermediate stage 1' from dual union all
select 'Transtage','orbital' from dual union all
select 'Trident','missile' from dual union all
select 'Trident.St2','intermediate stage 1' from dual union all
select 'Trident.St3','intermediate stage 1' from dual union all
select 'Trident2','missile' from dual union all
select 'UM129A','orbital' from dual union all
select 'UNK','sounding rocket' from dual union all
select 'UR-100','orbital family' from dual union all
select 'UR-100.St2','intermediate stage 1' from dual union all
select 'UR-100N','missile' from dual union all
select 'UR-100N.St2','intermediate stage 1' from dual union all
select 'UR-100NU.St3','intermediate stage 1' from dual union all
select 'UR-200','missile' from dual union all
select 'URM-2','orbital' from dual union all
select 'Unha','missile' from dual union all
select 'Unha.St2','missile' from dual union all
select 'Unha.St3','missile' from dual union all
select 'Unknown','sounding rocket' from dual union all
select 'V-1000','missile' from dual union all
select 'V-1000.Booster','missile' from dual union all
select 'V-2','missile' from dual union all
select 'V-750','missile' from dual union all
select 'V-825','missile' from dual union all
select 'Vega','orbital family' from dual union all
select 'Vela','intermediate stage 1' from dual union all
select 'Venus','sounding rocket' from dual union all
select 'Veronique','sounding rocket' from dual union all
select 'Vesta','sounding rocket' from dual union all
select 'Viking','orbital family' from dual union all
select 'Viper','missile' from dual union all
select 'Viper3','missile' from dual union all
select 'Viper5','missile' from dual union all
select 'WAC','sounding rocket' from dual union all
select 'Waxwing','orbital' from dual union all
select 'X-10','missile' from dual union all
select 'X-15','test rocket' from dual union all
select 'X-241','intermediate stage 1' from dual union all
select 'X-271','intermediate stage 1' from dual union all
select 'X-51','missile' from dual union all
select 'X-7','missile' from dual union all
select 'Xaero','sounding rocket' from dual union all
select 'YZ-1','intermediate stage 1' from dual union all
select 'ZN','weather (small)' from dual union all
select 'Zefiro','intermediate stage 1' from dual union all
select 'Zelzal','sounding rocket' from dual union all
select 'Zenit','orbital family' from dual union all
select 'Zenit.St2','orbital' from dual union all
select 'Zenith','sounding rocket' from dual union all
select 'Zeus-A','intermediate stage 1' from dual;


--------------------------------------------------------------------------------
-- LAUNCH_VEHICLE
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing LAUNCH_VEHICLE...');

CREATE TABLE "LAUNCH_VEHICLE" 
 (	"LV_ID" NUMBER, 
	"LV_NAME" VARCHAR2(33), 
	"LV_VARIANT" VARCHAR2(11), 
	"LV_CLASS" VARCHAR2(37), 
	"LV_FAMILY_CODE" VARCHAR2(21), 
	"LV_ALIAS" VARCHAR2(20), 
	"MIN_STAGE" NUMBER, 
	"MAX_STAGE" NUMBER, 
	"LENGTH" NUMBER, 
	"DIAMETER" NUMBER, 
	"LAUNCH_MASS" NUMBER, 
	"LEO_CAPACITY" NUMBER, 
	"GTO_CAPACITY" NUMBER, 
	"TAKEOFF_THRUST" NUMBER, 
	"APOGEE" NUMBER, 
	"RANGE" NUMBER, 
	 CONSTRAINT "LAUNCH_VEHICLE_PK" PRIMARY KEY ("LV_ID")
USING INDEX  ENABLE, 
	 CONSTRAINT "LAUNCH_VEHICLE_UQ" UNIQUE ("LV_NAME", "LV_VARIANT")
USING INDEX  ENABLE, 
	 CONSTRAINT "LAUNCH_VEHICLE_FAMILY_FK" FOREIGN KEY ("LV_FAMILY_CODE")
	  REFERENCES "LAUNCH_VEHICLE_FAMILY" ("LV_FAMILY_CODE") ENABLE
 ) ;


CREATE INDEX "LAUNCH_VEHICLE_IDX1" ON "LAUNCH_VEHICLE" ("LV_FAMILY_CODE") 
;
insert into LAUNCH_VEHICLE
select 1,'1Ya2TA','','endoatmospheric rocket','V-1000','',0,1,14.5,1,8.8,0,0,null,30,null from dual union all
select 10,'A-925','','missile','A-350','',1,2,19.8,2.57,33,0,0,null,350,350 from dual union all
select 100,'Ariane 42P','42P','orbital launch vehicle','Ariane','',0,3,58.4,3.8,320,null,2600,4008,40000,null from dual union all
select 1000,'R-36 8K67M','','missile','R-36','',1,2,34.6,3,185,0,0,2360,1000,1100 from dual union all
select 1001,'R-36 8K67MA','','missile','R-36','',1,2,34.6,3,185,0,0,2360,1000,1100 from dual union all
select 1002,'R-36 8K67P','','missile','R-36','',1,2,33.9,3,183.9,0,0,2360,1000,1100 from dual union all
select 1003,'R-36 8K67PM','','missile','R-36','',1,2,33.9,3,183.9,0,0,2360,1000,1100 from dual union all
select 1004,'R-36M 15A14','','missile','R-36','',1,2,33.6,3,209.6,0,0,4500,1000,150 from dual union all
select 1005,'R-36M2 15A18M','','missile','R-36','',1,3,37,3,211.1,0,0,4500,1000,null from dual union all
select 1006,'R-36MU 15A18','','missile','R-36','',1,3,34.3,3,211,0,0,4525,1000,110 from dual union all
select 1007,'R-36O 8K69','','orbital launch vehicle','R-36','',1,2,35,3,180,0,0,2360,200,null from dual union all
select 1008,'R-36O 8K69M','','orbital launch vehicle','R-36','',1,2,35,3,180,0,0,2360,200,null from dual union all
select 1009,'R-5','','missile','R-5','',1,1,20.7,1.65,24.4,0,0,430,300,1200 from dual union all
select 101,'Ariane 42P','42P+','orbital launch vehicle','Ariane','',0,3,58.4,3.8,324,null,2740,4028,40000,null from dual union all
select 1010,'R-5 VAO','','missile','R-5','',1,1,21.2,1.65,24.4,0,0,430,500,null from dual union all
select 1011,'R-5A','','missile','R-5','',1,1,20.7,1.65,24.4,0,0,430,300,null from dual union all
select 1012,'R-5B','','missile','R-5','',1,1,20.7,1.65,24.4,0,0,430,300,null from dual union all
select 1013,'R-5M','','missile','R-5','',1,1,20.3,1.65,24.4,0,0,430,500,null from dual union all
select 1014,'R-5V','','missile','R-5','',1,1,21.2,1.65,24.4,0,0,430,500,null from dual union all
select 1015,'R-7','71/III','missile','R-7','',1,2,33.5,2.95,279.1,0,0,3904,1000,null from dual union all
select 1016,'R-7','','missile','R-7','',1,2,34.2,2.95,274.3,0,0,3904,1000,null from dual union all
select 1017,'R-7A','','missile','R-7','',1,2,31.1,2.95,276.6,0,0,4020,1000,null from dual union all
select 1018,'R-9A','','missile','R-9','',1,2,24,2.68,100,0,0,1385,1000,16000 from dual union all
select 1019,'R-UNK','','missile','UNK','',1,1,22.5,1.65,42,0,0,null,402,null from dual union all
select 102,'Ariane 42P','42P-3','orbital launch vehicle','Ariane','',0,3,58.4,3.8,324,null,2930,4028,40000,null from dual union all
select 1020,'RAM A','','research rocket','Sergeant','',0,4,null,.79,null,0,0,null,100,null from dual union all
select 1021,'RAM B','','research rocket','Sergeant','',1,3,12.6,.79,6,0,0,235,100,null from dual union all
select 1022,'RH-125','','endoatmospheric weather rocket <80 km','RH-125','',1,1,2.5,.12,.1,0,0,8,30,null from dual union all
select 1023,'RH-200','','research rocket','RH-200','',1,2,3.6,.2,.1,0,0,17,80,null from dual union all
select 1024,'RH-200SV','','research rocket','RH-200','',1,1,3,.2,.1,0,0,17,80,null from dual union all
select 1025,'RH-300','','research rocket','Belier','',1,1,4.1,.31,.3,0,0,33,150,null from dual union all
select 1026,'RH-300 Mk II','','research rocket','Belier','',1,1,5.9,.31,.5,0,0,39,150,null from dual union all
select 1027,'RH-300B','','research rocket','Venus','',1,2,6.5,.28,.6,0,0,38,null,null from dual union all
select 1028,'RH-560','','research rocket','Stromboli','',1,2,8.4,.56,1.3,0,0,76,400,null from dual union all
select 1029,'RH-560/300 Mk II','','research rocket','Stromboli','',1,2,9.1,.56,1.3,0,0,76,400,null from dual union all
select 103,'Ariane 44L','44L','orbital launch vehicle','Ariane','',0,3,58.4,3.8,470,null,4200,5708,40000,null from dual union all
select 1030,'RH-560/LPR-3T','','research rocket','Stromboli','',1,2,10,.56,2.5,0,0,119,420,null from dual union all
select 1031,'RH-560S','','research rocket','Stromboli','',1,1,5.9,.56,1.5,0,0,119,250,null from dual union all
select 1032,'RH-75','','endoatmospheric weather rocket <80 km','RH-75','',1,1,1.5,.08,0,0,0,null,10,null from dual union all
select 1033,'RSA-3','d','missile','Jericho','',1,3,17.6,1.35,23.6,330,0,451,300,null from dual union all
select 1034,'RSA-3','','missile','Jericho','',1,3,17.6,1.35,23.6,330,0,451,300,null from dual union all
select 1035,'RT-1','','missile','RT-2','',1,3,18,1.84,34,0,0,890,500,2000 from dual union all
select 1036,'RT-15','','missile','RT-15','',1,2,11.9,1.49,16,0,0,412,500,2500 from dual union all
select 1037,'RT-2','','missile','RT-2','',1,3,21.3,1.84,46,0,0,890,1000,9500 from dual union all
select 1038,'RT-20P','','missile','RT-20P','',1,2,17.5,1.8,30,0,0,600,1000,8000 from dual union all
select 1039,'RT-23 15Zh44','','missile','R-39','',1,3,22.4,2.4,104.5,0,0,2060,1000,1000 from dual union all
select 104,'Ariane 44L','44L+','orbital launch vehicle','Ariane','',0,3,58.4,3.8,484,null,4460,5728,40000,null from dual union all
select 1040,'RT-23 15Zh52','','missile','R-39','',1,3,22.4,2.4,104.5,0,0,2060,1000,1000 from dual union all
select 1041,'RT-23U 15Zh60','','missile','R-39','',1,3,22.6,2.4,104.5,0,0,2060,1000,1000 from dual union all
select 1042,'RT-23U 15Zh61','','missile','R-39','',1,3,22.6,2.4,104.5,0,0,2060,1000,1000 from dual union all
select 1043,'RT-2M','','missile','RT-2','',1,3,21.3,1.84,51,0,0,980,1000,1000 from dual union all
select 1044,'RT-2P','','missile','RT-2','',1,3,21.3,1.84,51,0,0,980,1000,1000 from dual union all
select 1045,'RTV-A-2','','endoatmoshpheric test/research','MX-774','',1,1,9.6,.76,1.3,0,0,35,50,null from dual union all
select 1046,'RV-A-10','','endoatmoshpheric test/research','Sergeant','',1,1,6.2,.79,3.5,0,0,140,58,110 from dual union all
select 1047,'RX-250-LPN','','endoatmoshpheric test/research','RX-250','',1,1,5.3,.25,.3,0,0,52,70,null from dual union all
select 1048,'RX-320','','endoatmoshpheric test/research','RX-320','',1,1,4.7,.32,.5,0,0,null,70,null from dual union all
select 1049,'RX-420','','research rocket','RX-420','',1,1,6.2,.42,1,0,0,null,50,null from dual union all
select 105,'Ariane 44L','44L-3','orbital launch vehicle','Ariane','',0,3,58.4,3.8,484,null,4820,5728,40000,null from dual union all
select 1050,'Raven','','endoatmospheric weather rocket <80 km','Hopi','',1,1,3.3,.11,0,0,0,8,60,null from dual union all
select 1051,'Recruit','','research rocket','Recruit','',1,1,3,.23,.3,0,0,156,10,null from dual union all
select 1052,'Recruit T55','','research rocket','Recruit','',0,2,5,.23,.4,0,0,156,10,null from dual union all
select 1053,'Redstone','','missile','Redstone','',1,1,20,1.78,28.4,0,0,357,95,null from dual union all
select 1054,'Redstone MRLV','','missile','Redstone','',1,1,20,1.78,28.4,0,0,357,200,null from dual union all
select 1055,'Regulus','','cruise missile','Regulus','',1,1,10.1,1.4,6.6,0,0,293,15,null from dual union all
select 1056,'Regulus II','','cruise missile','Regulus','',1,1,20.5,1.3,10.2,0,0,null,20,null from dual union all
select 1057,'Rif','','missile','R-39','',1,3,16,2.4,84,0,0,2060,1000,830 from dual union all
select 1058,'Rigel','','research rocket','Rigel','',1,2,6.3,.28,1,0,0,null,310,null from dual union all
select 1059,'Rockair','','endoatmoshpheric test/research','FFAR','',0,1,1.2,.07,0,0,0,3,50,null from dual union all
select 106,'Ariane 44LP','44LP','orbital launch vehicle','Ariane','',-1,3,58.4,3.8,418.5,null,3700,5528,40000,null from dual union all
select 1060,'Rockaire','','endoatmoshpheric test/research','Deacon','',0,1,2.7,.16,.1,0,0,30,40,null from dual union all
select 1061,'Rocketsonde','','endoatmospheric weather rocket <80 km','Rocketsonde','',1,1,3,.11,0,0,0,1,100,null from dual union all
select 1062,'Rocketsonde2','','endoatmospheric weather rocket <80 km','Rocketsonde2','',1,1,3,.11,0,0,0,1,100,null from dual union all
select 1063,'Rokot','K','orbital launch vehicle','UR-100N','',1,3,29,2.5,107,1850,0,1870,1500,null from dual union all
select 1064,'Rokot','','orbital launch vehicle','UR-100N','',1,3,29,2.5,107,1850,0,1870,1500,null from dual union all
select 1065,'Rook','','endoatmospheric research test vehicle','Raven','',1,1,5,.44,1.2,0,0,null,20,null from dual union all
select 1066,'Rook IIIA','','endoatmospheric research test vehicle','Raven','',1,1,5,.44,1.2,0,0,null,20,null from dual union all
select 1067,'Rubezh','','missile','Topol','',1,2,20.5,1.8,47.1,0,0,890,1000,1050 from dual union all
select 1068,'Rubis','','research rocket','Agate','',1,2,9.6,.8,4,0,0,186,2000,null from dual union all
select 1069,'S-160','','research rocket','S-160','',1,1,4,.16,.1,0,0,null,80,null from dual union all
select 107,'Ariane 44LP','44LP+','orbital launch vehicle','Ariane','',-1,3,58.4,3.8,421,null,4030,5508,40000,null from dual union all
select 1070,'S-210','','research rocket','S-210','',1,1,5.2,.21,.3,0,0,null,110,null from dual union all
select 1071,'S-250','','endoatmoshpheric test/research','K250','',1,1,5,.25,.2,0,0,null,40,null from dual union all
select 1072,'S-300','','research rocket','S-300','',1,1,6,.3,.6,0,0,null,160,null from dual union all
select 1073,'S-310','','research rocket','S-310','',1,1,6.8,.31,.7,0,0,null,190,null from dual union all
select 1074,'S-520','','research rocket','S-520','',1,1,9.4,.52,2.2,0,0,143,430,null from dual union all
select 1075,'S-A','','endoatmoshpheric test/research','STA','',1,1,2.8,.16,.04,0,0,null,50,null from dual union all
select 1076,'S-B','','endoatmoshpheric test/research','STA','',1,1,2.8,.16,.1,0,0,null,75,null from dual union all
select 1077,'S-C','','research rocket','STA','',1,1,2.9,.17,.1,0,0,null,85,null from dual union all
select 1078,'SA-II','','endoatmoshpheric test/research','STA','',1,1,null,.1,0,0,0,null,20,null from dual union all
select 1079,'SARGE','','research rocket','STIG-B','',1,1,11,.51,1.1,0,0,24,100,null from dual union all
select 108,'Ariane 44LP','44LP-3','orbital launch vehicle','Ariane','',-1,3,58.4,3.8,421,null,4220,5528,40000,null from dual union all
select 1080,'SE4400','','endoatmospheric rocket','Stromboli','',1,1,8.1,.56,1.2,0,0,null,60,null from dual union all
select 1081,'SEGPRO','','endoatmoshpheric test/research','ISRO-TV','',1,1,null,null,0,0,0,null,10,null from dual union all
select 1082,'SICBM','','missile','SICBM','',1,4,16.2,1.17,17,0,0,979,1000,11000 from dual union all
select 1083,'SLV-3','D4','orbital launch vehicle','ASLV','',1,4,23.6,1,17,42,0,503,900,null from dual union all
select 1084,'SLV-3','','orbital launch vehicle','ASLV','',1,4,23.6,1,17,42,0,503,900,null from dual union all
select 1085,'SM-1ER','','endoatmospheric rocket','Terrier','',1,2,8,.46,1.3,0,0,258,20,null from dual union all
select 1086,'SM-1MR','','endoatmospheric rocket','Standard','',1,1,4.5,.34,.6,0,0,null,20,null from dual union all
select 1087,'SM-2','','endoatmospheric rocket','Terrier','',1,2,7.9,.53,1.3,0,0,null,30,null from dual union all
select 1088,'SM-2-IV','','endoatmospheric rocket','Mk72','',1,2,7.9,.53,1.3,0,0,null,30,null from dual;
insert into LAUNCH_VEHICLE
select 1089,'SM-2MR','','endoatmospheric rocket','Standard','',1,1,5,.35,1.3,0,0,null,30,null from dual union all
select 109,'Ariane 44P','44P','orbital launch vehicle','Ariane','',0,3,58.4,3.8,355,null,3000,5308,40000,null from dual union all
select 1090,'SM-3','','research rocket','Mk72','',1,3,7.9,.53,1.3,0,0,null,100,null from dual union all
select 1091,'SM-3-IB','','research rocket','Mk72','',1,3,7.9,.53,1.3,0,0,null,100,null from dual union all
select 1092,'SM-3-IIA','','research rocket','Mk72','',1,3,7.9,.53,1.3,0,0,null,100,null from dual union all
select 1093,'SM-6','','endoatmospheric rocket','Mk72','',1,2,7.9,.53,1.3,0,0,null,30,null from dual union all
select 1094,'SO-300-200','','research rocket','Belier','',0,1,6,.31,.5,0,0,38,300,null from dual union all
select 1095,'SPARTA','','orbital launch vehicle','Redstone','',1,3,21.8,1.78,30,45,0,357,1100,null from dual union all
select 1096,'SR19','','research rocket','SR19','',1,1,6,1.32,8,0,0,267,100,null from dual union all
select 1097,'SR19/SR19','','missile','SR19','',0,2,15.5,1.32,15,0,0,267,300,null from dual union all
select 1098,'SRALT','','missile','SR19','',0,1,12,1.32,12,0,0,267,300,null from dual union all
select 1099,'SRAM','','cruise missile','SRAM','',1,1,4.3,.46,1,0,0,null,10,180 from dual union all
select 11,'AAD','','tactical ballistic missile','AAD','',1,1,7.5,.42,1.3,0,0,null,40,350 from dual union all
select 110,'Ariane 44P','44P-3','orbital launch vehicle','Ariane','',0,3,58.4,3.8,356,null,3465,5328,40000,null from dual union all
select 1100,'SS-520','4','orbital launch vehicle','S-520','',1,3,9.7,.52,2.6,0,0,null,1000,null from dual union all
select 1101,'SS-520','','research rocket','S-520','',1,2,9.7,.52,2.6,0,0,null,1000,null from dual union all
select 1102,'SSBS S01','','missile','P-10','',1,2,15.1,1.5,25,0,0,440,500,null from dual union all
select 1103,'SSBS S02','','missile','P-16','',1,2,14.8,1.5,31.9,0,0,540,600,3000 from dual union all
select 1104,'SSBS S112','','endoatmoshpheric test/research','P-10','',1,2,12.6,1.5,25.1,0,0,440,50,null from dual union all
select 1105,'SSBS S2','','missile','P-16','',1,2,14.8,1.5,31.9,0,0,540,600,3000 from dual union all
select 1106,'SSBS S3','','missile','P-16','',1,2,13.8,1.5,25.8,0,0,540,600,3500 from dual union all
select 1107,'ST','','endoatmoshpheric test/research','STA','',1,1,null,.1,0,0,0,null,20,null from dual union all
select 1108,'ST-735','1','research rocket','SB-735','',0,1,14.6,.74,7.4,0,0,null,120,null from dual union all
select 1109,'ST-735','','research rocket','SB-735','',1,2,14,.74,7.4,0,0,null,120,null from dual union all
select 111,'Ariane 5ECA','','orbital launch vehicle','Ariane5','',0,2,57.7,5.4,780,10000,10000,11400,40000,null from dual union all
select 1110,'STAR','','exoatmospheric weather rocket','SuperLoki','',1,1,4,.11,0,0,0,null,90,null from dual union all
select 1111,'STARS','','missile','Polaris','',1,3,11.5,1.37,16.7,0,0,null,2500,5500 from dual union all
select 1112,'STIG-A','','research rocket','STIG','',1,1,9.8,.39,.7,0,0,22,80,null from dual union all
select 1113,'STIG-B','','research rocket','STIG-B','',1,1,10.4,.51,.7,0,0,22,80,null from dual union all
select 1114,'Safir','','tactical ballistic missile','Nodong','',1,2,22,1.25,26,0,0,255,250,null from dual union all
select 1115,'Sandhawk','','research rocket','Sandhawk','',1,1,5.1,.33,.7,0,0,80,200,null from dual union all
select 1116,'Sandhawk Tomahawk','','research rocket','Sandhawk','',1,2,9,.33,1,0,0,80,550,null from dual union all
select 1117,'Saphir VE231','','rocket test vehicle','Emeraude','',1,2,17.8,1.4,18,0,0,275,1000,null from dual union all
select 1118,'Saturn C-1','','orbital launch vehicle','Saturn','',1,3,49.6,6.6,420,9000,0,5768,200,null from dual union all
select 1119,'Saturn I','Blk2','orbital launch vehicle','Saturn','',1,2,58,6.53,508,9000,0,5768,200,null from dual union all
select 112,'Ariane 5ES','','orbital launch vehicle','Ariane5','',0,2,45.7,5.4,746,20000,6200,11400,40000,null from dual union all
select 1120,'Saturn I','','orbital launch vehicle','Saturn','',1,3,50.4,6.6,420,9000,0,5768,200,null from dual union all
select 1121,'Saturn IB','','orbital launch vehicle','Saturn','',1,2,68.1,6.53,587.3,null,null,7290,250,null from dual union all
select 1122,'Saturn V','2','orbital launch vehicle','SaturnV','',1,3,104.8,10.06,2822,75000,0,34030,500,null from dual union all
select 1123,'Saturn V','','orbital launch vehicle','SaturnV','',1,4,110.6,10.06,2913,118000,0,34030,400000,null from dual union all
select 1124,'Scanner','','research rocket','Sergeant','',0,3,null,.79,null,0,0,550,600,null from dual union all
select 1125,'Scout A','','orbital launch vehicle','Scout','',1,4,22,1.02,17.9,121,0,400,1000,null from dual union all
select 1126,'Scout A-1','','orbital launch vehicle','Scout','',1,4,22,1.02,17.9,121,0,440,1000,null from dual union all
select 1127,'Scout B','2A','orbital launch vehicle','Scout','',1,4,22,1.02,17.9,143,0,400,1000,null from dual union all
select 1128,'Scout B','','orbital launch vehicle','Scout','',1,4,22.4,1.02,17.9,143,0,400,1000,null from dual union all
select 1129,'Scout B-1','F','orbital launch vehicle','Scout','',1,4,22,1.02,17.9,143,0,440,1000,null from dual union all
select 113,'Ariane 5ES/ATV','','orbital launch vehicle','Ariane5','',0,2,45.7,5.4,746,20000,6200,11400,40000,null from dual union all
select 1130,'Scout B-1','','orbital launch vehicle','Scout','',1,4,22,1.02,17.5,143,0,440,1000,null from dual union all
select 1131,'Scout D-1','F','orbital launch vehicle','ScoutD','',1,4,25,1.14,21.3,183,0,464,1000,null from dual union all
select 1132,'Scout D-1','','orbital launch vehicle','ScoutD','',1,4,25,1.14,21.3,183,0,464,10000,null from dual union all
select 1133,'Scout E-1','','orbital launch vehicle','ScoutD','',1,5,25,1.14,20.6,200,0,464,130000,null from dual union all
select 1134,'Scout F-1','','orbital launch vehicle','ScoutD','',1,4,25,1.14,20.6,192,0,464,500,null from dual union all
select 1135,'Scout G-1','','orbital launch vehicle','ScoutD','',1,4,25,1.14,23.8,205,0,464,1000,null from dual union all
select 1136,'Scout II TV','','endoatmoshpheric test/research','Zefiro','',1,1,9,1.14,17,0,0,590,50,null from dual union all
select 1137,'Scout X','','big test rocket','Scout','',1,4,21.6,1.02,16.7,0,0,556,100,null from dual union all
select 1138,'Scout X-1','','orbital launch vehicle','Scout','',1,4,21.6,1.02,16.7,59,0,556,7000,null from dual union all
select 1139,'Scout X-1A','','research rocket','Scout','',1,5,22,1.02,17,0,0,440,220,null from dual union all
select 114,'Ariane 5G','','orbital launch vehicle','Ariane5','',0,2,45.7,5.4,746,10000,6200,11400,40000,null from dual union all
select 1140,'Scout X-2','1C','orbital launch vehicle','Scout','',1,4,22,1.02,17,76,0,440,7000,null from dual union all
select 1141,'Scout X-2','','orbital launch vehicle','Scout','',1,4,22,1.02,17,76,0,440,900,null from dual union all
select 1142,'Scout X-2B','','orbital launch vehicle','Scout','',1,4,22,1.02,17,90,0,440,900,null from dual union all
select 1143,'Scout X-2M','','orbital launch vehicle','Scout','',1,4,22,1.02,17,90,0,440,900,null from dual union all
select 1144,'Scout X-3','','orbital launch vehicle','Scout','',1,4,22,1.02,17,87,0,380,900,null from dual union all
select 1145,'Scout X-3A','','research rocket','Scout','',1,5,22,1.02,17,0,0,380,220,null from dual union all
select 1146,'Scout X-3C','','research rocket','Scout','',1,3,22,1.02,17,0,0,380,1300,null from dual union all
select 1147,'Scout X-3M','','orbital launch vehicle','Scout','',1,4,22,1.02,17,90,0,380,900,null from dual union all
select 1148,'Scout X-4','','orbital launch vehicle','Scout','',1,4,22,1.02,17.5,103,0,380,4000,null from dual union all
select 1149,'Scout X-4A','','research rocket','Scout','',1,5,22,1.02,17.5,0,0,380,180,null from dual union all
select 115,'Ariane 5G+','','orbital launch vehicle','Ariane5','',0,2,45.7,5.4,746,10000,6200,11400,40000,null from dual union all
select 1150,'Scout X-5C','','research rocket','Scout','',1,3,22,1.02,17.5,0,0,380,180,null from dual union all
select 1151,'Scud','','tactical ballistic missile','R-11','',1,1,11.2,.88,5.5,0,0,90,72,null from dual union all
select 1152,'Scud B','','tactical ballistic missile','R-11','',1,1,11.3,.88,6.3,0,0,90,76,300 from dual union all
select 1153,'Scud D','','tactical ballistic missile','Nodong','',1,1,16,1.25,16,0,0,255,150,null from dual union all
select 1154,'Seabee','','research rocket','AerobeeB','',1,1,7,.38,.7,0,0,18,1,null from dual union all
select 1155,'Seagull','','endoatmoshpheric test/research','Unknown','',1,1,null,.13,null,0,0,null,50,null from dual union all
select 1156,'Sejjil','','tactical ballistic missile','Sejjil','',1,2,17.6,1.25,23.6,0,0,255,150,2000 from dual union all
select 1157,'Sejjil-2','','tactical ballistic missile','Sejjil','',1,2,17.6,1.25,23.6,0,0,560,150,2000 from dual union all
select 1158,'Seliger 1','','research rocket','Seliger','',1,1,3.4,.6,.5,0,0,49,52,null from dual union all
select 1159,'Seliger 2','','research rocket','Seliger','',1,2,6,.6,1.5,0,0,49,80,null from dual union all
select 116,'Ariane 5GS','','orbital launch vehicle','Ariane5','',0,2,45.7,5.4,746,10000,6200,11400,40000,null from dual union all
select 1160,'Seliger 3','','research rocket','Seliger','',1,3,12.8,.6,3,0,0,49,120,null from dual union all
select 1161,'Sergeant','2R','research rocket','Sergeant','',0,1,5.1,.79,2.3,0,0,220,80,null from dual union all
select 1162,'Sergeant','','research rocket','Sergeant','',1,1,5.1,.79,2.3,0,0,220,80,null from dual union all
select 1163,'Sergeant 5-stage','','endoatmospheric research test vehicle','Sergeant','',1,5,14,.79,null,0,0,230,35,null from dual union all
select 1164,'Sergeant Hydac','','research rocket','Sergeant','',1,2,13.8,.79,4.8,0,0,null,500,null from dual union all
select 1165,'Shadow 1','','exoatmospheric weather rocket','SuperLoki','',1,1,3.3,.102,0,0,0,18,90,null from dual union all
select 1166,'Shahab 1','','tactical ballistic missile','R-11','',1,1,11.3,.88,6.3,0,0,90,76,300 from dual union all
select 1167,'Shahab 2','','tactical ballistic missile','R-11','',1,1,11.3,.88,6.1,0,0,90,76,500 from dual union all
select 1168,'Shahab 3','','tactical ballistic missile','Nodong','',1,1,15.6,1.25,16,0,0,255,150,2000 from dual union all
select 1169,'Shaheen 1','','tactical ballistic missile','DF-15','',1,1,9.1,1,6.2,0,0,null,100,750 from dual union all
select 117,'Aries','','research rocket','Aries','',1,1,9.2,1.16,6.3,0,0,200,500,null from dual union all
select 1170,'Shaheen 1A','','tactical ballistic missile','DF-15','',1,1,9.1,1,6.2,0,0,null,100,750 from dual union all
select 1171,'Shaheen 2','','missile','DF-21','',1,2,18,1.4,25,0,0,null,300,2000 from dual union all
select 1172,'Shaheen 3','','missile','DF-21','',1,2,18,1.4,25,0,0,null,300,2000 from dual union all
select 1173,'Shaviyt','','orbital launch vehicle','Jericho','',1,3,17,1.35,26,200,0,610,1400,null from dual union all
select 1174,'Shaviyt 1','','orbital launch vehicle','Jericho','',1,3,18,1.35,30,230,0,610,1400,null from dual union all
select 1175,'Shian Quxian 1S','','tactical ballistic missile','DF-11','',1,1,8.4,.88,4.6,0,0,90,150,300 from dual union all
select 1176,'Short Al Hussein','','tactical ballistic missile','R-11','',1,1,11.6,.88,6.3,0,0,90,150,600 from dual union all
select 1177,'Shotput','','big test rocket','Sergeant','',0,2,9.9,.79,5.1,0,0,220,400,null from dual union all
select 1178,'Shourya','','research rocket','K-15','',1,2,10,.74,6.2,0,0,null,200,600 from dual;
insert into LAUNCH_VEHICLE
select 1179,'Shtil''','','missile','R-29','',1,3,14.8,1.9,40.3,0,0,682,1000,8300 from dual union all
select 118,'Arrow 1','','endoatmospheric rocket','Jericho','',1,1,7.5,1.35,null,0,0,null,40,null from dual union all
select 1180,'Shtil''-1','','orbital launch vehicle','R-29','',1,3,14.8,1.9,40,430,0,810,1000,null from dual union all
select 1181,'Sidewinder-Arcas','','research rocket','Sidewinder','',1,2,4.3,.13,.08,0,0,null,90,null from dual union all
select 1182,'Sidewinder-Raven','','exoatmospheric weather rocket','Sidewinder','',1,2,5.2,.13,.1,0,0,null,112,null from dual union all
select 1183,'Silver Sparrow','','tactical ballistic missile','BlackSparrow','',0,1,8.4,.53,3.1,0,0,null,100,null from dual union all
select 1184,'Simorgh','','orbital launch vehicle','Unha','',1,2,27,2.4,85,50,0,1000,2000,null from dual union all
select 1185,'Sineva','','missile','R-29','',1,3,14.8,1.9,40.3,0,0,682,1000,8300 from dual union all
select 1186,'Sirocco','','endoatmospheric weather rocket <80 km','Sirocco','',1,1,3.9,.14,.1,0,0,null,60,null from dual union all
select 1187,'Skorost''','','missile','Topol','',1,2,18,1.8,45,0,0,890,1000,1050 from dual union all
select 1188,'Skua','','endoatmospheric weather rocket <80 km','Skua','',0,1,2.2,.13,0,0,0,null,70,null from dual union all
select 1189,'Skua 2','','research rocket','Skua','',0,1,2.8,.13,.1,0,0,null,100,null from dual union all
select 119,'Arrow 2','','endoatmospheric rocket','Arrow2','',1,2,6.3,.8,2.5,0,0,null,40,50 from dual union all
select 1190,'Skua 3','','research rocket','Skua','',0,1,2.8,.13,.1,0,0,null,120,null from dual union all
select 1191,'Skua 4','','research rocket','Skua','',0,1,2.8,.13,.1,0,0,null,120,null from dual union all
select 1192,'Skybolt ALBM','','endoatmospheric rocket','Skybolt','',1,2,12,.91,5.1,0,0,111,390,1200 from dual union all
select 1193,'Skylark','1','research rocket','Raven','',1,1,7.5,.44,1.1,0,0,44,150,null from dual union all
select 1194,'Skylark','2','research rocket','Raven','',1,1,7.5,.44,1.1,0,0,44,150,null from dual union all
select 1195,'Skylark','2C','research rocket','Raven','',1,2,9.1,.44,1.4,0,0,80,200,null from dual union all
select 1196,'Skylark','5','research rocket','Raven','',1,1,7.5,.44,1.1,0,0,44,110,null from dual union all
select 1197,'Skylark','5C','research rocket','Raven','',1,2,9.1,.44,1.4,0,0,80,200,null from dual union all
select 1198,'Skylark','7','research rocket','Raven','',1,1,7.5,.44,1.1,0,0,44,140,null from dual union all
select 1199,'Skylark','7C','research rocket','Raven','',1,2,9.1,.44,1.4,0,0,80,200,null from dual union all
select 12,'ADTV','','endoatmoshpheric test/research','Lupus','',1,1,2.2,.13,.04,0,0,9,20,null from dual union all
select 120,'Arrow 3','','endoatmospheric rocket','Arrow3','',1,2,11,.7,1.5,0,0,null,100,50 from dual union all
select 1200,'Skylark 1','','research rocket','Raven','',1,1,10.8,.44,1.2,0,0,50,125,null from dual union all
select 1201,'Skylark 10A','','research rocket','Raven','',1,2,6,.44,.8,0,0,130,270,null from dual union all
select 1202,'Skylark 12','','research rocket','Raven','',1,3,13,.44,2.1,0,0,191,750,null from dual union all
select 1203,'Skylark 12 AC','','research rocket','Raven','',1,3,13,.44,2.1,0,0,191,830,null from dual union all
select 1204,'Skylark 2','','research rocket','Raven','',1,2,12.7,.44,1.5,0,0,80,190,null from dual union all
select 1205,'Skylark 2 AC','','research rocket','Raven','',1,2,12.7,.44,1.5,0,0,80,190,null from dual union all
select 1206,'Skylark 3','','research rocket','Raven','',1,2,12.7,.44,1.7,0,0,80,210,null from dual union all
select 1207,'Skylark 3 AC','','research rocket','Raven','',1,2,12.7,.44,1.7,0,0,80,210,null from dual union all
select 1208,'Skylark 4','','research rocket','Raven','',1,2,13.2,.44,1.9,0,0,191,240,null from dual union all
select 1209,'Skylark 4 AC','','research rocket','Raven','',1,2,13.2,.44,1.9,0,0,191,240,null from dual union all
select 121,'Ashura','','tactical ballistic missile','Sejjil','Sejjil',1,2,10,1.25,20,0,0,560,150,2000 from dual union all
select 1210,'Skylark 6','','research rocket','Raven','',1,2,13.2,.44,1.9,0,0,191,270,null from dual union all
select 1211,'Skylark 6 AC','','research rocket','Raven','',1,2,13.2,.44,1.9,0,0,191,270,null from dual union all
select 1212,'Skylark 7','','research rocket','Raven','',1,2,12.2,.44,2.1,0,0,191,270,null from dual union all
select 1213,'Skylark 7 AC','','research rocket','Raven','',1,2,12.2,.44,2,0,0,191,270,null from dual union all
select 1214,'Snark','','cruise missile','Snark','',1,1,21.3,1.4,22.5,0,0,null,10,null from dual union all
select 1215,'Sonda 1','','endoatmospheric weather rocket <80 km','Sonda','',1,2,4.5,.11,.1,0,0,27,65,null from dual union all
select 1216,'Sonda 2','','research rocket','Sonda','',1,1,5.6,.3,.4,0,0,36,180,null from dual union all
select 1217,'Sonda 3','M1','research rocket','Sonda3','',1,2,8,.56,1.4,0,0,102,600,null from dual union all
select 1218,'Sonda 3','M2','research rocket','Sonda3','',1,2,8,.56,1.4,0,0,102,600,null from dual union all
select 1219,'Sonda 3','','research rocket','Sonda3','',1,2,8,.56,1.5,0,0,102,600,null from dual union all
select 122,'Asp','','endoatmospheric weather rocket <80 km','Asp','',1,1,2.7,.17,0,0,0,42,70,null from dual union all
select 1220,'Sonda 4','','research rocket','S-40','',1,2,11,1.01,7.2,0,0,203,800,null from dual union all
select 1221,'Sounding Rocket','','research rocket','NSPOB','',1,2,8,.55,2,0,0,null,200,null from dual union all
select 1222,'Soyuz 11A510','','orbital launch vehicle','R-7','',1,2,37.9,2.95,280.7,3600,0,4054,250,null from dual union all
select 1223,'Soyuz 11A511','','orbital launch vehicle','R-7','',1,3,50,2.95,300,6500,0,4054,400,null from dual union all
select 1224,'Soyuz 11A511L','','orbital launch vehicle','R-7','',1,3,50,2.95,300,6500,0,4054,400,null from dual union all
select 1225,'Soyuz 11A511M','','orbital launch vehicle','R-7','',1,3,50,2.95,300,6500,0,4054,400,null from dual union all
select 1226,'Soyuz-2-1A','Fregat','orbital launch vehicle','R-7','',1,4,50.7,2.95,313,7600,0,4146,500,null from dual union all
select 1227,'Soyuz-2-1A','','orbital launch vehicle','R-7','',1,3,50.7,2.95,313,7600,0,4146,500,null from dual union all
select 1228,'Soyuz-2-1B','Fregat','orbital launch vehicle','R-7','',1,4,50.7,2.95,313,7600,0,4146,500,null from dual union all
select 1229,'Soyuz-2-1B','','orbital launch vehicle','R-7','',1,3,50.7,2.95,313,7600,0,4146,500,null from dual union all
select 123,'Asp II','','endoatmospheric weather rocket <80 km','Asp','',1,1,2.7,.17,0,0,0,42,50,null from dual union all
select 1230,'Soyuz-2-1V','Volga','orbital launch vehicle','Blok-A','',1,3,41.1,2.95,160,2800,0,null,null,null from dual union all
select 1231,'Soyuz-2-1V','','orbital launch vehicle','Blok-A','',1,2,41.1,2.95,160,2800,0,null,null,null from dual union all
select 1232,'Soyuz-FG','Fregat','orbital launch vehicle','R-7','',1,4,46,2.95,308,7200,0,4146,500,null from dual union all
select 1233,'Soyuz-FG','','orbital launch vehicle','R-7','',1,3,46,2.95,308,7200,0,4146,500,null from dual union all
select 1234,'Soyuz-ST-A','Fregat','orbital launch vehicle','R-7','',1,4,50.7,2.95,313,7600,0,4146,500,null from dual union all
select 1235,'Soyuz-ST-A','','orbital launch vehicle','R-7','',1,3,50.7,2.95,313,7600,0,4146,500,null from dual union all
select 1236,'Soyuz-ST-B','Fregat','orbital launch vehicle','R-7','',1,4,50.7,2.95,313,7600,0,4146,500,null from dual union all
select 1237,'Soyuz-ST-B','','orbital launch vehicle','R-7','',1,3,50.7,2.95,313,7600,0,4146,500,null from dual union all
select 1238,'Soyuz-U','','orbital launch vehicle','R-7','',1,3,50.7,2.95,310,7200,0,4032,500,null from dual union all
select 1239,'Soyuz-U-PVB','Fregat','orbital launch vehicle','R-7','',1,4,42.5,2.95,308,5000,0,4146,500,null from dual union all
select 124,'Asp III','4L','endoatmospheric weather rocket <80 km','Asp','',0,1,2.7,.17,0,0,0,42,50,null from dual union all
select 1240,'Soyuz-U-PVB','Ikar','orbital launch vehicle','R-7','',1,4,43.4,2.95,305,4100,0,4032,500,null from dual union all
select 1241,'Soyuz-U-PVB','','orbital launch vehicle','R-7','',1,3,50.7,2.95,310,7200,0,4032,500,null from dual union all
select 1242,'Soyuz-U2','','orbital launch vehicle','R-7','',1,3,50.7,2.95,310,7200,0,4062,500,null from dual union all
select 1243,'Space Shuttle','','orbital launch vehicle','SRB','',1,3,38.1,8.4,2040,24400,0,28200,600,null from dual union all
select 1244,'SpaceLoft XL','','research rocket','SpaceLoft','',1,1,6.1,.27,.4,0,0,null,100,null from dual union all
select 1245,'Spaceship One','','research rocket','SS1','',0,1,5,1.5,4,0,0,null,130,null from dual union all
select 1246,'Spaceship Two','','research rocket','SS2','',0,1,18.3,2.3,9.7,0,0,null,130,null from dual union all
select 1247,'Sparoair I','','research rocket','Sparrow','',0,2,3.7,.2,.1,0,0,35,100,null from dual union all
select 1248,'Sparoair II','','research rocket','Sparrow','',0,2,3.9,.2,.1,0,0,35,100,null from dual union all
select 1249,'Sparoair III','','research rocket','Sparrow','',0,2,3.9,.2,.14,0,0,35,100,null from dual union all
select 125,'Asp III','','endoatmospheric weather rocket <80 km','Asp','',0,1,2.7,.17,0,0,0,42,50,null from dual union all
select 1250,'Sparrow Arcas','','research rocket','Sparrow','',1,2,3.8,.2,.1,0,0,35,175,null from dual union all
select 1251,'Spartan ABM','','missile','Spartan','',1,3,16.8,1.09,13.2,0,0,2000,560,740 from dual union all
select 1252,'Sprint ABM','','endoatmospheric rocket','Sprint','',1,2,8.2,1.37,3.4,0,0,3000,30,null from dual union all
select 1253,'Sputnik 11A59','','orbital launch vehicle','R-7','',1,2,34.5,2.95,275.3,2250,0,4020,1500,null from dual union all
select 1254,'Sputnik 8A91','','orbital launch vehicle','R-7','',1,2,29.2,2.95,268.6,500,0,3806,1500,null from dual union all
select 1255,'Sputnik 8K71PS','','orbital launch vehicle','R-7','',1,2,29.2,2.95,265,500,0,3904,1500,null from dual union all
select 1256,'Squirt','','endoatmospheric rocket','Squirt','',1,2,8.2,1.37,3.4,0,0,3000,1,null from dual union all
select 1257,'Stable Super Loki','','exoatmospheric weather rocket','SuperLoki','',1,1,3.3,.102,0,0,0,18,90,null from dual union all
select 1258,'Starbird','','research rocket','Talos','',1,4,17.5,.76,7.4,0,0,null,500,null from dual union all
select 1259,'Start','','orbital launch vehicle','Topol','',1,6,28.9,1.8,60,650,0,890,1000,null from dual union all
select 126,'Asp IV','','exoatmospheric weather rocket','Asp','',1,1,2.7,.17,0,0,0,12,109,null from dual union all
select 1260,'Start-1','','orbital launch vehicle','Topol','',1,5,22.7,1.8,47,500,0,890,1000,null from dual union all
select 1261,'Start-1.2','','orbital launch vehicle','Topol','',1,5,22.7,1.8,47,500,0,890,1000,null from dual union all
select 1262,'Storm','','missile','Sergeant','',1,2,null,.79,6,0,0,null,200,null from dual union all
select 1263,'Storm-2','','missile','SR19','',1,1,9.9,1.32,10,0,0,267,200,600 from dual union all
select 1264,'Stratos II+','','research rocket','Stratos2','',1,1,7,.2,.19,0,0,8,50,null from dual union all
select 1265,'Strela','','orbital launch vehicle','UR-100N','',1,3,29.2,2.5,105,1400,0,null,500,null from dual union all
select 1266,'Strongarm','1','research rocket','HJ','',1,4,17.1,.58,3.3,0,0,365,1000,null from dual union all
select 1267,'Strongarm','','research rocket','HJ','',1,5,17.1,.58,3.3,0,0,365,1000,null from dual union all
select 1268,'Strypi','','research rocket','Sergeant','',0,1,null,.79,null,0,0,null,200,null from dual;
insert into LAUNCH_VEHICLE
select 1269,'Strypi II','','research rocket','Sergeant','',0,1,11.3,.79,5.5,0,0,586,208,null from dual union all
select 127,'Astra','','research rocket','Astra','',1,2,10,1,6,0,0,null,100,null from dual union all
select 1270,'Strypi IIAR','','research rocket','Sergeant','',0,1,null,.79,null,0,0,586,250,null from dual union all
select 1271,'Strypi IIR','','research rocket','Sergeant','',0,1,11.3,.79,5.5,0,0,586,208,null from dual union all
select 1272,'Strypi IV','','research rocket','Sergeant','',0,2,9.3,.79,null,0,0,586,500,null from dual union all
select 1273,'Strypi IVA','','research rocket','Sergeant','',0,2,9.3,.79,null,0,0,586,500,null from dual union all
select 1274,'Strypi IVR','','research rocket','Sergeant','',0,2,9.3,.79,null,0,0,586,500,null from dual union all
select 1275,'Strypi IX','','research rocket','Sergeant','',1,2,null,.79,null,0,0,null,200,null from dual union all
select 1276,'Strypi VI','','research rocket','Sergeant','',0,3,null,.79,null,0,0,null,200,null from dual union all
select 1277,'Strypi VIIAR','','research rocket','Sergeant','',0,3,null,.79,null,0,0,null,200,null from dual union all
select 1278,'Strypi VIIR','','research rocket','Sergeant','',0,3,11.2,.79,4.6,0,0,499,170,null from dual union all
select 1279,'Strypi XI','','research rocket','Sergeant','',0,3,null,.79,null,0,0,null,120,null from dual union all
select 128,'Astrid','','endoatmoshpheric test/research','Astrid','',1,1,1.9,.17,0,0,0,null,10,null from dual union all
select 1280,'Strypi XIIR','','research rocket','Sergeant','',0,2,16,1.02,13.5,0,0,430,500,null from dual union all
select 1281,'Super Arcas','','exoatmospheric weather rocket','Arcas','',1,1,2.7,.11,0,0,0,1,100,null from dual union all
select 1282,'Super Chief I','','research rocket','Talos','',1,2,14.7,.76,4.7,0,0,516,300,null from dual union all
select 1283,'Super Chief II','','research rocket','Talos','',1,2,14.7,.76,4.7,0,0,516,300,null from dual union all
select 1284,'Super Loki','','exoatmospheric weather rocket','SuperLoki','',1,1,3.3,.102,0,0,0,18,90,null from dual union all
select 1285,'Super Strypi','','orbital launch vehicle','LEO-46','',1,3,20.6,1.17,24,300,0,null,250,250 from dual union all
select 1286,'Swik A','','research rocket','Sergeant','',0,2,11,.79,5.9,0,0,null,1200,null from dual union all
select 1287,'T-7','','endoatmospheric weather rocket <80 km','T-7','',0,1,9.6,.45,1.1,0,0,null,60,null from dual union all
select 1288,'T-7/GF-01A','','research rocket','T-7','',0,2,10,.45,1.5,0,0,null,311,null from dual union all
select 1289,'T-7A','','research rocket','T-7','',0,1,10.3,.45,1.1,0,0,null,115,null from dual union all
select 129,'Astrobee 1500','','research rocket','Sergeant','',0,2,10.4,.79,5.2,0,0,566,2000,null from dual union all
select 1290,'T-7A-S','','research rocket','T-7','',0,1,10.3,.45,1.2,0,0,null,100,null from dual union all
select 1291,'T-7A-S2','','research rocket','T-7','',0,1,10,.45,1.3,0,0,null,100,null from dual union all
select 1292,'T-7M','','endoatmoshpheric test/research','T-7','',0,1,5.3,.45,.2,0,0,2,10,null from dual union all
select 1293,'THAAD','1','tactical ballistic missile','THAAD','',1,1,6.2,.34,.9,0,0,null,100,null from dual union all
select 1294,'THAAD','','tactical ballistic missile','THAAD','',1,1,6.2,.34,.9,0,0,null,100,null from dual union all
select 1295,'TR-1','','research rocket','TR-1','',1,1,14.3,1.13,11.8,0,0,608,80,null from dual union all
select 1296,'TR-1A','','research rocket','TR-1','',1,1,13.4,1.13,10.3,0,0,608,270,null from dual union all
select 1297,'TT-200','','research rocket','TT-210','',1,1,6,.2,.2,0,0,null,100,null from dual union all
select 1298,'TT-210','','research rocket','TT-210','',1,1,6,.21,.2,0,0,null,100,null from dual union all
select 1299,'TT-500','','research rocket','JCR','',1,2,10.5,.5,2.2,0,0,null,200,null from dual union all
select 13,'ALARR','','research rocket','Genie','',0,1,3.1,.38,.5,0,0,null,100,null from dual union all
select 130,'Astrobee 200','','research rocket','Nike','',1,2,6.3,.42,.8,0,0,161,350,null from dual union all
select 1300,'TT-500A','','research rocket','JCR','',1,2,10.5,.5,2.2,0,0,null,290,null from dual union all
select 1301,'Tacite','','rocket test vehicle','Stromboli','',1,1,7.8,.56,2,0,0,170,150,null from dual union all
select 1302,'Talos','','endoatmospheric rocket','Talos','',1,1,9.8,.76,3.5,0,0,null,20,null from dual union all
select 1303,'Talos Castor','','research rocket','Talos','',1,2,16.9,.76,8.2,0,0,null,300,null from dual union all
select 1304,'Talos Sergeant Hydac','','research rocket','Talos','',1,3,17.3,.76,6.1,0,0,null,500,null from dual union all
select 1305,'Talos Terrier','','research rocket','Talos','',1,2,10,.76,null,0,0,null,12,null from dual union all
select 1306,'Talos Terrier Oriole','','research rocket','Talos','',1,3,19.8,.76,null,0,0,null,12,null from dual union all
select 1307,'Talos Terrier Peregrine','','research rocket','Talos','',1,3,14,.46,1,0,0,null,500,null from dual union all
select 1308,'Tandem Double Deacon','','endoatmospheric research test vehicle','Deacon','',1,2,10,.15,.1,0,0,54,20,null from dual union all
select 1309,'Tater','','endoatmospheric research test vehicle','Talos','',1,3,12.3,.76,null,0,0,null,5,null from dual union all
select 131,'Astrobee 500','','research rocket','Genie','',1,3,7.8,.38,.9,0,0,161,1000,null from dual union all
select 1310,'Taurion','','endoatmoshpheric test/research','HJ','',1,1,5,.58,1.5,0,0,7,2,null from dual union all
select 1311,'Tauro','','research rocket','Rigel','',1,2,7.7,.28,1.2,0,0,null,150,null from dual union all
select 1312,'Taurus','S','endoatmoshpheric test/research','HJ','',1,1,5,.58,1.5,0,0,457,10,null from dual union all
select 1313,'Taurus 1110','','orbital launch vehicle','MX','ARPA Taurus',1,4,27.9,2.35,73,1320,0,1711,1000,null from dual union all
select 1314,'Taurus 2110','','orbital launch vehicle','MX','',1,4,27.9,2.35,73,1380,0,1711,1000,null from dual union all
select 1315,'Taurus 2210','','orbital launch vehicle','MX','',1,4,29.7,2.35,73,1380,0,1711,1000,null from dual union all
select 1316,'Taurus 3110','','orbital launch vehicle','MX','',1,4,31.6,2.35,73,1380,0,1711,1000,null from dual union all
select 1317,'Taurus 3210','','orbital launch vehicle','MX','',1,4,31.6,2.35,73,1380,0,1711,1000,null from dual union all
select 1318,'Taurus Nike Tomahawk','','research rocket','HJ','',1,3,12,.58,2.2,0,0,457,700,null from dual union all
select 1319,'Taurus Orion','','research rocket','HJ','',1,2,10.1,.58,2,0,0,457,200,null from dual union all
select 132,'Astrobee D','','research rocket','AstrobeeD','',1,1,3.9,.15,.1,0,0,23,140,null from dual union all
select 1320,'Taurus Tomahawk','','research rocket','HJ','',1,2,9.4,.58,1.6,0,0,457,550,null from dual union all
select 1321,'Temp','','missile','Temp','',1,1,12.4,1.01,9.7,0,0,null,150,900 from dual union all
select 1322,'Temp-2S','','missile','Temp-2S','',1,3,18.5,1.79,41.5,0,0,null,1000,1050 from dual union all
select 1323,'Temp-S','','missile','Temp','',1,1,12.4,1.01,9.3,0,0,null,150,900 from dual union all
select 1324,'Terrapin','','research rocket','Deacon','',1,2,4.5,.16,.1,0,0,15,140,null from dual union all
select 1325,'Terrier','','endoatmoshpheric test/research','Terrier','',1,1,6,.46,1.3,0,0,258,30,null from dual union all
select 1326,'Terrier ASAS','','endoatmospheric rocket','Terrier','',1,2,12,.46,1.3,0,0,258,275,null from dual union all
select 1327,'Terrier ASROC Cajun','','research rocket','Terrier','',1,3,11,.46,1.4,0,0,258,100,null from dual union all
select 1328,'Terrier Asp','','research rocket','Terrier','',1,2,10,.46,1.1,0,0,258,100,null from dual union all
select 1329,'Terrier Asp IV','','research rocket','Terrier','',1,2,10,.46,1.1,0,0,258,100,null from dual union all
select 133,'Astrobee F','','research rocket','AstrobeeF','',1,1,11.5,.38,1.5,0,0,178,375,null from dual union all
select 1330,'Terrier Hydac','','research rocket','Terrier','',1,2,10,.46,null,0,0,258,500,null from dual union all
select 1331,'Terrier Imp Malemute','1','research rocket','Terrier','',1,2,12.7,.46,1.6,0,0,258,500,null from dual union all
select 1332,'Terrier Imp Malemute','2','research rocket','Terrier','',1,2,12.7,.46,1.6,0,0,258,500,null from dual union all
select 1333,'Terrier Imp Malemute','','research rocket','Terrier','',1,2,12.7,.46,1.6,0,0,258,500,null from dual union all
select 1334,'Terrier Improved Orion','','research rocket','Terrier','',1,2,12.8,.46,1.3,0,0,258,300,null from dual union all
select 1335,'Terrier LEAP','','endoatmospheric rocket','Terrier','',1,3,10,.46,1.3,0,0,258,30,null from dual union all
select 1336,'Terrier Lynx','','research rocket','Terrier','',1,2,9,.46,1.3,0,0,258,200,null from dual union all
select 1337,'Terrier Malemute','','research rocket','Terrier','',1,2,12.7,.46,1.6,0,0,258,500,null from dual union all
select 1338,'Terrier Mk 12 Imp Orion','','research rocket','Terrier','',1,2,12.8,.46,1.3,0,0,258,300,null from dual union all
select 1339,'Terrier Mk 70 Imp Orion','','research rocket','Terrier','',1,2,12.8,.46,1.3,0,0,258,300,null from dual union all
select 134,'Atea-1','','research rocket','Atea','',1,1,5,.15,.06,0,0,null,100,null from dual union all
select 1340,'Terrier Mk 70 Oriole','','research rocket','Terrier','',1,2,11,.46,1,0,0,258,370,null from dual union all
select 1341,'Terrier Mk 70 Orion','','research rocket','Terrier','',1,2,12.8,.46,1.3,0,0,258,300,null from dual union all
select 1342,'Terrier Mk 70 Peregrine','','research rocket','Terrier','',1,2,17,.46,2.2,0,0,null,300,null from dual union all
select 1343,'Terrier Nike','','endoatmospheric rocket','Terrier','',1,2,9.8,.46,1.1,0,0,258,2,null from dual union all
select 1344,'Terrier Oriole','','research rocket','Terrier','',1,2,11,.46,1,0,0,258,370,null from dual union all
select 1345,'Terrier Orion','','research rocket','Terrier','',1,2,12.8,.46,1.3,0,0,258,300,null from dual union all
select 1346,'Terrier Recruit','','research rocket','Terrier','',1,2,5,.46,1,0,0,258,10,null from dual union all
select 1347,'Terrier Sandhawk','','research rocket','Terrier','',1,2,11.4,.46,null,0,0,258,430,null from dual union all
select 1348,'Terrier Terrier Oriole','','research rocket','Terrier','',1,3,14,.46,1,0,0,258,370,null from dual union all
select 1349,'Terrier Tomahawk','','research rocket','Terrier','',1,2,9,.46,1.1,0,0,258,300,null from dual union all
select 135,'Athena','','missile','Sergeant','',0,4,15.2,.79,7.2,0,0,586,200,null from dual union all
select 1350,'Terrier Viper I','','research rocket','Terrier','',1,2,8,.42,.6,0,0,217,80,null from dual union all
select 1351,'Terrier/551','','research rocket','Terrier','',1,2,8.4,.46,1.3,0,0,258,450,null from dual union all
select 1352,'Thor Able','','missile','Thor','',1,2,24.8,2.44,48.3,0,0,668,1600,null from dual union all
select 1353,'Thor Able I','','orbital launch vehicle','Thor','',1,3,26.9,2.44,52,50,0,710,100000,null from dual union all
select 1354,'Thor Able II','M1','orbital launch vehicle','Thor','',1,3,27.8,2.44,52,50,0,668,1600,null from dual union all
select 1355,'Thor Able II','','missile','Thor','',1,2,27.3,2.44,52,null,0,668,1600,null from dual union all
select 1356,'Thor Able III','','orbital launch vehicle','Thor','',1,3,27.4,2.44,52,100,0,668,100000,null from dual union all
select 1357,'Thor Able IV','','orbital launch vehicle','Thor','',1,3,27.2,2.44,52,100,0,668,400000,null from dual union all
select 1358,'Thor Ablestar','2','orbital launch vehicle','Thor','',1,2,23.8,2.44,53,150,0,756,1500,null from dual;
insert into LAUNCH_VEHICLE
select 1359,'Thor Ablestar','','orbital launch vehicle','Thor','',1,2,23.5,2.44,53,150,0,668,1500,null from dual union all
select 136,'Athena H','','rocket test vehicle','Castor4','',0,3,18.6,1.02,14,0,0,422,200,null from dual union all
select 1360,'Thor Agena A','','orbital launch vehicle','Thor','',1,2,24,2.44,53.1,250,0,670,1000,null from dual union all
select 1361,'Thor Agena B','','orbital launch vehicle','Thor','',1,2,23.4,2.44,56.5,null,0,670,1000,null from dual union all
select 1362,'Thor Agena D','','orbital launch vehicle','Thor','',1,2,23.4,2.44,56.5,null,0,743,5500,null from dual union all
select 1363,'Thor Burner 1','','orbital launch vehicle','Thor','',1,2,23,2.44,50,200,0,668,1000,null from dual union all
select 1364,'Thor Burner 2','','orbital launch vehicle','Thor','',1,2,22.4,2.44,50,500,0,750,1000,null from dual union all
select 1365,'Thor Burner 2A','','orbital launch vehicle','Thor','',1,3,23,2.44,50,500,0,750,1000,null from dual union all
select 1366,'Thor DM-18','','missile','Thor','',1,1,19.8,2.44,48.5,0,0,710,520,null from dual union all
select 1367,'Thor DM-18A','','missile','Thor','',1,1,19.4,2.44,48.9,0,0,668,520,null from dual union all
select 1368,'Thor DM-18C','','missile','Thor','',1,1,22,2.44,48.5,0,0,734,500,null from dual union all
select 1369,'Thor DSV-2D','','big test rocket','Thor','',1,1,21.7,2.44,50,0,0,743,1500,null from dual union all
select 137,'Athena-1','','orbital launch vehicle','MX','',1,3,18.9,2.3,66.3,820,0,1450,1000,null from dual union all
select 1370,'Thor DSV-2E','','missile','Thor','',1,1,19.4,2.44,49.3,0,0,668,500,null from dual union all
select 1371,'Thor DSV-2F','','rocket test vehicle','Thor','',1,1,20.8,2.44,50,0,0,668,50,null from dual union all
select 1372,'Thor DSV-2G','','rocket test vehicle','Thor','',1,2,26.9,2.44,55,0,0,668,75,null from dual union all
select 1373,'Thor DSV-2J','','missile','Thor','',1,1,19.4,2.44,55,0,0,668,1500,null from dual union all
select 1374,'Thor DSV-2U','','orbital launch vehicle','Thor','',1,2,23,2.44,49.5,500,0,750,1000,null from dual union all
select 1375,'Thor Delta','','orbital launch vehicle','Thor','',1,3,27.5,2.44,50.7,226,0,734,100000,null from dual union all
select 1376,'Thor Delta A','','orbital launch vehicle','Thor','',1,3,27.5,2.44,51,250,0,756,100000,null from dual union all
select 1377,'Thor Delta B','','orbital launch vehicle','Thor','',1,3,27.5,2.44,51.8,370,0,756,40000,null from dual union all
select 1378,'Thor Delta C','','orbital launch vehicle','Thor','',1,3,27.5,2.44,52,400,81,756,100000,null from dual union all
select 1379,'Thor Delta C1','','orbital launch vehicle','Thor','',1,3,27.5,2.44,52,400,81,756,1000,null from dual union all
select 138,'Athena-2','','orbital launch vehicle','MX','',1,4,28.2,2.3,120.7,2065,0,1450,400000,null from dual union all
select 1380,'Thor Delta D','','orbital launch vehicle','Thor','',0,3,27.5,2.44,64.7,450,104,756,40000,null from dual union all
select 1381,'Thor Delta E','','orbital launch vehicle','Thor','',0,3,28,2.44,68,540,150,1450,400000,null from dual union all
select 1382,'Thor Delta E1','','orbital launch vehicle','Thor','',0,3,28,2.44,68,540,150,1450,400000,null from dual union all
select 1383,'Thor Delta G','','orbital launch vehicle','Thor','',0,2,28,2.44,65,null,0,1590,400,null from dual union all
select 1384,'Thor Delta J','','orbital launch vehicle','Thor','',0,3,28,2.44,65,null,260,1590,6000,null from dual union all
select 1385,'Thor Delta L','','orbital launch vehicle','Thor','',0,3,34,2.44,89.9,null,300,1590,400000,null from dual union all
select 1386,'Thor Delta M','','orbital launch vehicle','Thor','',0,3,32.4,2.44,91,null,355,1590,40000,null from dual union all
select 1387,'Thor Delta M6','','orbital launch vehicle','Thor','',0,3,32.4,2.44,103.7,null,450,2411,400000,null from dual union all
select 1388,'Thor Delta N','','orbital launch vehicle','Thor','',0,2,33,2.44,90,null,null,1590,1500,null from dual union all
select 1389,'Thor Delta N6','','orbital launch vehicle','Thor','',0,2,33,2.44,102.9,null,null,2411,1500,null from dual union all
select 139,'Atlas 3A','','orbital launch vehicle','Atlas','',1,2,52.8,3.05,225,8620,4060,3827,40000,null from dual union all
select 1390,'Thor MG-18','','orbital launch vehicle','Thor','',1,2,23,2.44,50,200,0,668,1000,null from dual union all
select 1391,'Thor SLV-2 Agena B','','orbital launch vehicle','Thor','',1,2,27.2,2.44,56.5,null,0,756,1000,null from dual union all
select 1392,'Thor SLV-2 Agena D','','orbital launch vehicle','Thor','',1,2,27.2,2.44,56.5,null,0,756,1000,null from dual union all
select 1393,'Thor SLV-2A Agena B','','orbital launch vehicle','Thor','',0,2,27.2,2.44,67.8,null,0,1475,2000,null from dual union all
select 1394,'Thor SLV-2A Agena D','','orbital launch vehicle','Thor','',0,2,27.2,2.44,67.8,null,0,1475,1000,null from dual union all
select 1395,'Thorad SLV-2G Agena D','','orbital launch vehicle','Thor','',0,2,32.9,2.44,91.4,1500,0,1444,1000,null from dual union all
select 1396,'Thorad SLV-2H Agena D','','orbital launch vehicle','Thor','',0,2,32.9,2.44,91.4,1500,0,1444,1000,null from dual union all
select 1397,'Tianying-3C','','research rocket','Tianying','',1,1,6.4,.45,1.1,0,0,null,200,null from dual union all
select 1398,'Tianying-3E','','research rocket','Tianying','',1,1,6.4,.45,1.1,0,0,null,200,null from dual union all
select 1399,'Tianying-3F','','research rocket','Tianying','',1,2,9.1,.45,1.6,0,0,null,320,null from dual union all
select 14,'ALSOR','','research rocket','Deacon','',0,1,3.6,.16,.12,0,0,null,100,null from dual union all
select 140,'Atlas 3B','DEC','orbital launch vehicle','Atlas','',1,2,53.6,3.05,225,8620,4060,3827,40000,null from dual union all
select 1400,'Tibere','','rocket test vehicle','Stromboli','',1,3,14.2,.56,4.6,0,0,170,1000,null from dual union all
select 1401,'Tiny Tim','','endoatmoshpheric test/research','WAC','',1,1,2.4,.3,.1,0,0,null,10,null from dual union all
select 1402,'Titan 23B','','orbital launch vehicle','Titan','',1,3,45,3.05,165,3600,null,2300,1000,null from dual union all
select 1403,'Titan 24B','','orbital launch vehicle','Titan','',1,3,45,3.05,170,3600,null,2410,1000,null from dual union all
select 1404,'Titan 33B','','orbital launch vehicle','Titan','',1,3,45,3.05,185,3600,null,2300,40000,null from dual union all
select 1405,'Titan 34B','','orbital launch vehicle','Titan','',1,3,45.5,3.05,188.5,3600,0,2410,40000,null from dual union all
select 1406,'Titan 34D','','orbital launch vehicle','Titan','',0,2,50,3.05,723.5,15350,0,11280,40000,null from dual union all
select 1407,'Titan 34D/IUS','','orbital launch vehicle','Titan','',0,4,52.4,3.05,725,0,5000,11280,40000,null from dual union all
select 1408,'Titan 34D/Transtage','','orbital launch vehicle','Titan','',0,3,50,3.05,720,0,5000,11280,40000,null from dual union all
select 1409,'Titan 401A/Centaur','','orbital launch vehicle','Titan','',0,3,63.1,3.05,868,0,null,14200,40000,null from dual union all
select 141,'Atlas 3B','SEC','orbital launch vehicle','Atlas','',1,2,53.6,3.05,225,8620,4060,3827,40000,null from dual union all
select 1410,'Titan 401B/Centaur','','orbital launch vehicle','Titan','',0,3,62.2,3.05,939,0,9000,15000,400000,null from dual union all
select 1411,'Titan 402A/IUS','','orbital launch vehicle','Titan','',0,4,54,3.05,910,0,null,14200,40000,null from dual union all
select 1412,'Titan 402B/IUS','','orbital launch vehicle','Titan','',0,4,54,3.05,925,0,null,15000,40000,null from dual union all
select 1413,'Titan 403A','','orbital launch vehicle','Titan','',0,2,54,3.05,906,17700,0,14200,1000,null from dual union all
select 1414,'Titan 403B','','orbital launch vehicle','Titan','',0,2,54,3.05,910,21680,0,15000,1000,null from dual union all
select 1415,'Titan 404A','','orbital launch vehicle','Titan','',0,2,54,3.05,906,17700,0,14200,1000,null from dual union all
select 1416,'Titan 404B','','orbital launch vehicle','Titan','',0,2,54,3.05,910,21680,0,15000,1000,null from dual union all
select 1417,'Titan 405A','','orbital launch vehicle','Titan','',0,2,54,3.05,906,17700,0,14200,1000,null from dual union all
select 1418,'Titan 405B','','orbital launch vehicle','Titan','',0,2,54,3.05,910,21680,0,15000,1000,null from dual union all
select 1419,'Titan I','','missile','Titan','',1,2,31,3.05,105.1,0,0,1450,1000,null from dual union all
select 142,'Atlas A','','big test rocket','Atlas','',0,1,22.9,3.05,82,0,0,1210,120,900 from dual union all
select 1420,'Titan II','','missile','Titan','',1,2,31.1,3.05,150,0,0,2000,1380,null from dual union all
select 1421,'Titan II GLV','','orbital launch vehicle','Titan','',1,2,32.8,3.05,150,3600,0,2090,300,null from dual union all
select 1422,'Titan II SLV','','orbital launch vehicle','Titan','',1,2,34.6,3.05,155,1900,0,2090,1000,null from dual union all
select 1423,'Titan IIIA','','orbital launch vehicle','Titan','',1,3,42,3.05,161.7,4100,null,2300,4000,null from dual union all
select 1424,'Titan IIIB','','orbital launch vehicle','Titan','',1,3,45,3.05,156.5,3700,0,2300,1000,null from dual union all
select 1425,'Titan IIIC','','orbital launch vehicle','Titan','',0,3,38.3,3.05,626.2,13300,3000,10600,40000,null from dual union all
select 1426,'Titan IIID','','orbital launch vehicle','Titan','',0,2,36,3.05,613,11200,0,10600,1000,null from dual union all
select 1427,'Titan IIIE','','orbital launch vehicle','Titan','',0,3,48.8,3.05,633,14000,0,10600,400000,null from dual union all
select 1428,'Titus','','rocket test vehicle','Stromboli','',1,2,11.6,.56,3.4,0,0,170,250,null from dual union all
select 1429,'Tochka','','missile','Tochka','',1,1,6.4,.65,2,0,0,null,20,null from dual union all
select 143,'Atlas Able','','orbital launch vehicle','Atlas','',0,3,35,3.05,120,null,0,1629,400000,null from dual union all
select 1430,'Tochka-U','','missile','Tochka','',1,1,6.4,.65,2,0,0,null,26,null from dual union all
select 1431,'Tomahawk','','research rocket','Tomahawk','',1,1,5,.23,.3,0,0,44,80,null from dual union all
select 1432,'Topaze VE111','','rocket test vehicle','Agate','',1,1,7.1,.8,2.8,0,0,120,80,null from dual union all
select 1433,'Topaze VE111L','','rocket test vehicle','Agate','',1,1,7.9,.8,3.4,0,0,147,110,null from dual union all
select 1434,'Topol''','','missile','Topol','',1,3,21.5,1.8,45,0,0,890,1000,1050 from dual union all
select 1435,'Topol''-E','','missile','Topol','',1,3,21.5,1.8,45,0,0,890,1000,1050 from dual union all
select 1436,'Topol''-M','','missile','Topol','',1,3,20.5,1.8,47.1,0,0,890,1000,1050 from dual union all
select 1437,'Topol''-M 15Zh55','','missile','Topol','',1,3,20.5,1.8,47.1,0,0,890,1000,1050 from dual union all
select 1438,'Trailblazer 1','','research rocket','HJ','',1,6,17.1,.58,3.4,0,0,365,280,null from dual union all
select 1439,'Trailblazer 2','M','research rocket','Sergeant','',0,5,15.7,.79,6.1,0,0,547,300,null from dual union all
select 144,'Atlas Agena A','','orbital launch vehicle','Atlas','',0,2,30,3.05,124,null,0,1629,600,null from dual union all
select 1440,'Trailblazer 2','','research rocket','Sergeant','',0,4,15.7,.79,6.1,0,0,547,300,null from dual union all
select 1441,'Tri-Deacon Deacon HPAG','','endoatmospheric research test vehicle','Deacon','',1,3,7,.15,.2,0,0,75,20,null from dual union all
select 1442,'Trident C-4','','missile','Trident','',1,3,10.4,1.88,33.1,0,0,null,1000,7300 from dual union all
select 1443,'Trident D-5','','missile','Trident','',1,3,13.6,2.11,58.5,0,0,null,1000,11000 from dual union all
select 1444,'Triple Deacon','','endoatmospheric research test vehicle','Deacon','',1,1,7,.15,.1,0,0,75,20,null from dual union all
select 1445,'Triple Deacon HPAG','','endoatmospheric research test vehicle','Deacon','',1,2,7,.15,.1,0,0,75,20,null from dual union all
select 1446,'Tsiklon-2','','orbital launch vehicle','R-36','',1,2,35.5,3,180,3350,0,2652,1000,null from dual union all
select 1447,'Tsiklon-2A','','orbital launch vehicle','R-36','',1,2,39.7,3,180,3000,0,2652,1000,null from dual union all
select 1448,'Tsiklon-3','','orbital launch vehicle','R-36','',1,3,39.3,3,190,4100,0,2745,2000,null from dual;
insert into LAUNCH_VEHICLE
select 1449,'UR-100','','missile','UR-100','',1,2,16.9,2,42.3,0,0,2070,1000,10600 from dual union all
select 145,'Atlas Agena B','','orbital launch vehicle','Atlas','',0,2,32.1,3.05,130,null,0,1629,400000,null from dual union all
select 1450,'UR-100K','','missile','UR-100','',1,2,18.9,2,50,0,0,2070,1000,12000 from dual union all
select 1451,'UR-100M','','missile','UR-100','',1,2,16.7,2,42.3,0,0,2070,1000,10600 from dual union all
select 1452,'UR-100N','','missile','UR-100N','',1,2,18.9,2.5,50,0,0,null,1000,12000 from dual union all
select 1453,'UR-100NU 15A35','','missile','UR-100N','',1,3,24.3,2.5,105.6,0,0,1870,1000,10000 from dual union all
select 1454,'UR-100NU 15A35P','','missile','UR-100N','',1,3,24.3,2.5,105.6,0,0,1870,1000,10000 from dual union all
select 1455,'UR-100NU 15A35S','','missile','UR-100N','',1,3,24.3,2.5,105.6,0,0,1870,1000,10000 from dual union all
select 1456,'UR-100U','','missile','UR-100','',1,2,18.9,2,50,0,0,2070,1000,12000 from dual union all
select 1457,'UR-200','','missile','UR-200','',1,2,26.4,3,138,0,0,2001,1000,120 from dual union all
select 1458,'UR-500','','orbital launch vehicle','Proton','',1,2,39,7.4,595,8400,0,9500,300,null from dual union all
select 1459,'UR-500K','','orbital launch vehicle','Proton','Proton-K',1,3,50,7.4,694,19000,0,9500,300,null from dual union all
select 146,'Atlas Agena D','','orbital launch vehicle','Atlas','',0,2,32.1,3.05,130,null,800,1590,400000,null from dual union all
select 1460,'UR-500K/Blok D','','orbital launch vehicle','Proton','Proton-K/D',1,4,55.4,7.4,691.5,5000,4350,9500,400000,null from dual union all
select 1461,'Unha-1','','missile','Unha','',1,2,29,2.4,85,0,0,1000,1000,4000 from dual union all
select 1462,'Unha-2','','orbital launch vehicle','Unha','',1,3,30,2.4,85,50,0,1000,2000,null from dual union all
select 1463,'Unha-3','','orbital launch vehicle','Unha','',1,3,30,2.4,85,50,0,1000,2000,null from dual union all
select 1464,'Unknown','','research rocket','Unknown','',1,1,null,null,null,null,null,null,null,null from dual union all
select 1465,'Uprated Saturn I','','orbital launch vehicle','Saturn','Saturn IB',1,2,68.1,6.53,587.3,null,null,7290,250,null from dual union all
select 1466,'Upstage','','endoatmoshpheric test/research','HIBEX','',1,2,5.2,.79,6,0,0,null,20,null from dual union all
select 1467,'Ute Apache','','research rocket','Genie','',1,2,4.2,.38,.5,0,0,60,200,null from dual union all
select 1468,'Ute Tomahawk','','research rocket','Genie','',1,2,5.2,.38,.5,0,0,60,230,null from dual union all
select 1469,'V-1000','','endoatmospheric rocket','V-1000','',0,1,14.5,1,8.8,0,0,null,30,null from dual union all
select 147,'Atlas B','','orbital launch vehicle','Atlas','',0,1,22,3.05,112.5,68,0,1615,900,6000 from dual union all
select 1470,'V-2','','research rocket','V-2','',1,1,13.6,1.65,12.8,0,0,312,200,null from dual union all
select 1471,'V-750','','research rocket','V-750','',0,1,10.7,.65,2.3,0,0,490,27,null from dual union all
select 1472,'V-825','','tactical ballistic missile','V-825','',0,1,18.5,1.3,16.6,0,0,null,180,null from dual union all
select 1473,'VE10 Aigle','','endoatmospheric research test vehicle','Stromboli','',1,1,8,.55,2.3,0,0,null,20,null from dual union all
select 1474,'VE10A Aigle','','endoatmospheric research test vehicle','Stromboli','',1,1,8,.55,2.3,0,0,null,20,null from dual union all
select 1475,'VLS-1','','orbital launch vehicle','S-40','',0,3,19.5,1.01,50.7,120,0,1172,750,null from dual union all
select 1476,'VLS-R1','','endoatmoshpheric test/research','Sonda','',1,2,6,.72,1,0,0,120,10,null from dual union all
select 1477,'VLS-R2','','endoatmoshpheric test/research','Sonda','',0,1,4.3,.3,1,0,0,120,50,null from dual union all
select 1478,'VS-30','','research rocket','Sonda3','',1,1,5,.56,1.4,0,0,102,140,null from dual union all
select 1479,'VS-30/Orion','','research rocket','Sonda3','',1,2,7,.56,1.8,0,0,102,320,null from dual union all
select 148,'Atlas Burner 2','','orbital launch vehicle','Atlas','',0,2,28.9,3.05,130,null,0,1700,800,null from dual union all
select 1480,'VS-31/Imp. Malemute','','research rocket','Sonda3','',1,2,8,.56,2.7,0,0,102,400,null from dual union all
select 1481,'VS-31/Orion','','research rocket','Sonda3','',1,2,8,.56,1.8,0,0,102,400,null from dual union all
select 1482,'VS-40','','research rocket','S-40','',1,2,9.5,1.01,6.8,0,0,190,1000,null from dual union all
select 1483,'VS-40M','','research rocket','S-40','',1,2,9.5,1.01,6.8,0,0,190,1000,null from dual union all
select 1484,'VSB-30','','research rocket','Sonda3','',1,2,12,.56,2.6,0,0,102,265,null from dual union all
select 1485,'Vanguard','X-248','orbital launch vehicle','Viking','',1,3,22.1,1.14,10,20,0,120,3000,null from dual union all
select 1486,'Vanguard','','orbital launch vehicle','Viking','',1,3,22.1,1.14,10,20,0,120,3000,null from dual union all
select 1487,'Vega','1','orbital launch vehicle','Vega','',1,4,30.1,3,136.7,1500,null,2974,1000,null from dual union all
select 1488,'Vega','','orbital launch vehicle','Vega','',1,4,30.1,3,136.7,1500,null,2974,1000,null from dual union all
select 1489,'Vega Statoreacteur','','cruise missile','Stromboli','',1,1,9.9,.56,1.7,0,0,null,30,null from dual union all
select 149,'Atlas Burner 2A','','orbital launch vehicle','Atlas','',0,3,30,3.05,130,null,0,1700,800,null from dual union all
select 1490,'Veronique','','research rocket','Veronique','',1,1,7.3,.55,1.3,0,0,40,100,null from dual union all
select 1491,'Veronique 61','','research rocket','Veronique','',1,1,9.5,.55,1.9,0,0,60,300,null from dual union all
select 1492,'Veronique 61M','','research rocket','Veronique','',1,1,9.5,.55,1.9,0,0,60,300,null from dual union all
select 1493,'Veronique AGI','','research rocket','Veronique','',1,1,7.1,.55,1.3,0,0,40,100,null from dual union all
select 1494,'Veronique P2','','endoatmoshpheric test/research','Veronique','',1,1,6,.55,.5,0,0,20,2,null from dual union all
select 1495,'Veronique P6','','endoatmoshpheric test/research','Veronique','',1,1,6,.55,.5,0,0,20,2,null from dual union all
select 1496,'Veronique R','','endoatmoshpheric test/research','Veronique','',1,1,7.3,.55,1.3,0,0,40,2,null from dual union all
select 1497,'Vertikal'' K65UP','','big test rocket','R-14','',1,1,27.6,2.4,90,0,0,1485,1500,null from dual union all
select 1498,'Vesta','','research rocket','Vesta','',1,1,10.2,1,6,0,0,140,250,null from dual union all
select 1499,'Viking','','research rocket','Viking','',1,1,14.9,.81,4.5,0,0,93,200,null from dual union all
select 15,'ALV','','research rocket','Pegasus','',1,2,16.8,1.28,15,null,null,580,200,null from dual union all
select 150,'Atlas C','','missile','Atlas','',0,1,24,3.05,112.5,0,0,1615,900,8000 from dual union all
select 1500,'Viking Type 9','','research rocket','Viking','',1,1,12.8,1.14,6.8,0,0,93,250,null from dual union all
select 1501,'Viper','','exoatmospheric weather rocket','Viper','',1,1,2.4,.16,0,0,0,null,100,null from dual union all
select 1502,'Viper 3A','','exoatmospheric weather rocket','Viper3','',1,1,2.4,.114,0,0,0,null,120,null from dual union all
select 1503,'Viper Falcon','','exoatmospheric weather rocket','Viper','',1,2,3.5,.17,0,0,0,null,80,null from dual union all
select 1504,'Viper V/Dart','','exoatmospheric weather rocket','Viper5','',1,1,3.4,.18,0,0,0,27,150,null from dual union all
select 1505,'Viper-Arrow','','research rocket','Viper','',1,2,5,.17,.1,0,0,24,100,null from dual union all
select 1506,'Viper-Dart','','exoatmospheric weather rocket','Viper','',1,1,3.4,.16,0,0,0,null,120,null from dual union all
select 1507,'Volna','','missile','R-29','',1,2,14.1,1.9,35.3,0,0,682,1000,6500 from dual union all
select 1508,'Voskhod 11A57','','orbital launch vehicle','R-7','',1,3,44,2.95,305.1,5600,0,4054,500,null from dual union all
select 1509,'Vostok 8A92','','orbital launch vehicle','R-7','',1,3,38.4,2.95,285.9,4750,0,4020,500,null from dual union all
select 151,'Atlas C Able','','orbital launch vehicle','Atlas','',0,3,28,3.05,120,170,0,1615,400000,null from dual union all
select 1510,'Vostok 8A92M','','orbital launch vehicle','R-7','',1,3,38.8,2.95,283.1,1280,0,4020,1000,null from dual union all
select 1511,'Vostok 8K72','','orbital launch vehicle','R-7','',1,3,38.4,2.95,283.5,4500,0,3998,500,null from dual union all
select 1512,'Vostok 8K72K','','orbital launch vehicle','R-7','',1,3,38.4,2.95,287,4700,0,3998,70000,null from dual union all
select 1513,'Vostok-L 8K72','','orbital launch vehicle','R-7','',1,3,33.5,2.95,279.1,4000,0,3998,400000,null from dual union all
select 1514,'Vysota','','missile','R-29','',1,2,13,1.9,33.3,0,0,682,1000,9100 from dual union all
select 1515,'WAC','','endoatmoshpheric test/research','WAC','',0,1,7.3,.3,.3,0,0,null,10,null from dual union all
select 1516,'WAC A','','endoatmoshpheric test/research','WAC','',0,1,7.3,.3,.3,0,0,null,75,null from dual union all
select 1517,'WAC B','','endoatmoshpheric test/research','WAC','',0,1,7.3,.3,.3,0,0,null,75,null from dual union all
select 1518,'WASP','','research rocket','Sergeant','',0,2,14.7,.79,6.8,0,0,580,240,null from dual union all
select 1519,'Wasserfall','','endoatmospheric rocket','Hermes','',1,1,7.8,.88,3.5,0,0,80,18,29 from dual union all
select 152,'Atlas Centaur','','orbital launch vehicle','Atlas','',0,2,33,3.05,136.1,null,1700,1629,400000,null from dual union all
select 1520,'X-15','','big test rocket','X-15','',0,1,15.3,3.55,14.2,0,0,257,100,null from dual union all
select 1521,'X-17','','rocket test vehicle','Sergeant','',1,3,12.3,.79,3.4,0,0,220,500,null from dual union all
select 1522,'X-17 HTV','1','endoatmospheric research test vehicle','Nike','',1,3,5,.2,.1,0,0,209,10,null from dual union all
select 1523,'X-17 HTV','','endoatmospheric research test vehicle','Nike','',1,3,5,.2,.1,0,0,209,10,null from dual union all
select 1524,'X-17 QTV','','endoatmospheric research test vehicle','T-40','',1,3,2.5,.2,.1,0,0,13,10,null from dual union all
select 1525,'X-51A','','endoatmoshpheric test/research','ATACMS','',0,2,7.7,.61,1.8,0,0,180,20,null from dual union all
select 1526,'X-7A-1','','cruise missile','HJ','',1,2,2.1,.61,3.6,0,0,null,30,null from dual union all
select 1527,'X-7A-3','','cruise missile','X-7','',0,1,2.1,.61,3.6,0,0,null,30,null from dual union all
select 1528,'X-7B','','cruise missile','X-7','',0,1,2.1,.61,3.6,0,0,null,30,null from dual union all
select 1529,'Xaero','','research rocket','Xaero','',1,1,4.6,.77,1,0,0,null,30,null from dual union all
select 153,'Atlas Centaur D','','orbital launch vehicle','Atlas','',0,2,33,3.05,136.1,null,1700,1629,400000,null from dual union all
select 1530,'Yars','','missile','Topol','',1,3,20.5,1.8,47.1,0,0,890,1000,1050 from dual union all
select 1531,'Yars 15Zh65M','','missile','Topol','',1,3,20.5,1.8,47.1,0,0,890,1000,1050 from dual union all
select 1532,'Yars-M','','missile','Topol','',1,3,20.5,1.8,47.1,0,0,890,1000,1050 from dual union all
select 1533,'ZUR-215','','research rocket','V-750','',1,1,10.7,.65,2.3,0,0,490,27,null from dual union all
select 1534,'Zelzal','','research rocket','Zelzal','',1,1,10,.61,.9,0,0,null,50,209 from dual union all
select 1535,'Zenit','C','research rocket','Zenith','',0,1,5.9,.42,.9,0,0,45,190,null from dual union all
select 1536,'Zenit','','research rocket','Zenith','',1,1,4.6,.42,.7,0,0,45,140,null from dual union all
select 1537,'Zenit-2','','orbital launch vehicle','Zenit','',1,2,57,3.9,459,13500,0,7259,1000,null from dual union all
select 1538,'Zenit-2 11K77.05','','orbital launch vehicle','Zenit','',1,2,57,3.9,459,13500,0,7259,1000,null from dual;
insert into LAUNCH_VEHICLE
select 1539,'Zenit-2M','','orbital launch vehicle','Zenit','',1,2,57,3.9,459,13500,0,7259,1000,null from dual union all
select 154,'Atlas D','','orbital launch vehicle','Atlas','',0,1,24,3.05,119,1400,0,1629,1855,13000 from dual union all
select 1540,'Zenit-2SB','','orbital launch vehicle','Zenit','',1,2,59.6,3.9,471,0,5000,7259,40000,null from dual union all
select 1541,'Zenit-3F','','orbital launch vehicle','Zenit','',1,3,59,3.9,471,0,5000,7259,40000,null from dual union all
select 1542,'Zenit-3SL','','orbital launch vehicle','Zenit','',1,3,59.6,3.9,471,0,5000,7259,40000,null from dual union all
select 1543,'Zenit-3SLB','','orbital launch vehicle','Zenit','',1,3,59.6,3.9,471,0,5000,7259,40000,null from dual union all
select 1544,'Zenit-3SLBF','','orbital launch vehicle','Zenit','Zenit-3F',1,3,59.6,3.9,471,0,5000,7259,40000,null from dual union all
select 1545,'Zhinui','','research rocket','ZN','',1,1,4.8,.15,.1,0,0,10,120,null from dual union all
select 1546,'Zombie','','research rocket','ATACMS','',1,1,5,.61,1,0,0,57,40,null from dual union all
select 1547,'Zulfiqar','','research rocket','Zelzal','',1,1,10,.61,.9,0,0,null,50,700 from dual union all
select 1548,'Zyb','','missile','R-27','',1,1,9.7,1.5,14.2,0,0,null,500,3000 from dual union all
select 1549,'Zyb R-27K','','missile','R-27','',1,1,9.7,1.5,14.2,0,0,null,500,3000 from dual union all
select 155,'Atlas E','','orbital launch vehicle','Atlas','',0,1,20,3.05,121,1700,0,1731,1570,13000 from dual union all
select 1550,'Zyb R-27U','','missile','R-27','',1,1,9.7,1.5,14.2,0,0,null,500,3000 from dual union all
select 1551,'eMRBM','','missile','SR19','',0,2,15.5,1.32,15,0,0,267,300,null from dual union all
select 1552,'eMRBM-T1','','missile','SR19','',0,2,15.5,1.32,15,0,0,267,300,null from dual union all
select 156,'Atlas E Altair','','orbital launch vehicle','Atlas','',0,2,27.3,3.05,125,null,0,1731,1000,null from dual union all
select 157,'Atlas E/MSD','','orbital launch vehicle','Atlas','',0,2,27.3,3.05,122,1200,0,1731,1100,null from dual union all
select 158,'Atlas E/OIS','','orbital launch vehicle','Atlas','',0,2,28.7,3.05,120,null,0,1731,800,null from dual union all
select 159,'Atlas E/SGS-2','','orbital launch vehicle','Atlas','',0,3,32.1,3.05,125,861,0,1731,20000,null from dual union all
select 16,'AMROC IRR','','research rocket','AMROC','',1,1,17.7,1.29,12.2,0,0,315,100,null from dual union all
select 160,'Atlas E/SVS','','orbital launch vehicle','Atlas','',0,3,29.2,3.05,125,700,0,1731,20000,null from dual union all
select 161,'Atlas E/Trident','','missile','Atlas','',0,2,25,3.05,130,0,0,1731,1500,null from dual union all
select 162,'Atlas F','','orbital launch vehicle','Atlas','',0,1,20,3.05,121,1700,0,1731,1413,13000 from dual union all
select 163,'Atlas F/Agena D','','orbital launch vehicle','Atlas','',0,2,34.6,3.05,130,null,0,1731,1000,null from dual union all
select 164,'Atlas F/MSD','','orbital launch vehicle','Atlas','',0,2,27.3,3.05,122,1200,0,1731,1100,null from dual union all
select 165,'Atlas F/OIS','','orbital launch vehicle','Atlas','',0,2,32.1,3.05,120,null,0,1731,550,null from dual union all
select 166,'Atlas F/PTS','','orbital launch vehicle','Atlas','',0,2,29.2,3.05,125,295,295,1731,20000,null from dual union all
select 167,'Atlas F/SVS','','orbital launch vehicle','Atlas','',0,3,29.2,3.05,125,700,0,1731,20000,null from dual union all
select 168,'Atlas F/Trident','','missile','Atlas','',0,2,25,3.05,130,0,0,1731,1500,null from dual union all
select 169,'Atlas G Centaur','','orbital launch vehicle','Atlas','',0,2,41.8,3.05,166.1,3630,2400,1987,40000,null from dual union all
select 17,'ARAV-ER','','research rocket','Terrier','',1,3,14,.46,1,0,0,258,370,null from dual union all
select 170,'Atlas H','','orbital launch vehicle','Atlas','',0,1,27.4,3.05,150.5,2000,0,1954,1100,null from dual union all
select 171,'Atlas I','','orbital launch vehicle','Atlas','',0,2,41.8,3.05,166.1,null,2250,1987,400000,null from dual union all
select 172,'Atlas II','','orbital launch vehicle','Atlas','',0,2,45,3.05,187,6580,2770,2140,40000,null from dual union all
select 173,'Atlas IIA','','orbital launch vehicle','Atlas','',0,2,47.4,3.05,187.5,7316,3066,2140,100000,null from dual union all
select 174,'Atlas IIAS','','orbital launch vehicle','Atlas','',-1,2,47.4,3.05,237.2,8618,3719,3000,400000,null from dual union all
select 175,'Atlas SLV-3','','orbital launch vehicle','Atlas','',0,1,21,3.05,120,0,0,1650,1500,null from dual union all
select 176,'Atlas SLV-3 Agena B','','orbital launch vehicle','Atlas','',0,2,32.1,3.05,140,null,0,1629,400000,null from dual union all
select 177,'Atlas SLV-3 Agena D','','orbital launch vehicle','Atlas','',0,2,32.1,3.05,140,null,850,1629,400000,null from dual union all
select 178,'Atlas SLV-3A Agena D','','orbital launch vehicle','Atlas','',0,2,36,3.05,155,null,1000,1700,150000,null from dual union all
select 179,'Atlas SLV-3C Centaur','','orbital launch vehicle','Atlas','',0,2,40.2,3.05,148.4,null,1800,1700,400000,null from dual union all
select 18,'ARPA Taurus','','orbital launch vehicle','MX','',1,4,27.2,2.35,73,1320,0,1711,1000,null from dual union all
select 180,'Atlas SLV-3D Centaur','','orbital launch vehicle','Atlas','',0,2,40.3,3.05,149,null,1900,1741,400000,null from dual union all
select 181,'Atlas V 401','C','orbital launch vehicle','Atlas5','',1,2,58.3,3.8,334.5,null,4750,3800,40000,null from dual union all
select 182,'Atlas V 401','','orbital launch vehicle','Atlas5','',1,2,58.3,3.8,334.5,null,4750,3800,40000,null from dual union all
select 183,'Atlas V 411','C','orbital launch vehicle','Atlas5','',0,2,58.3,3.8,386.8,null,5950,null,40000,null from dual union all
select 184,'Atlas V 411','','orbital launch vehicle','Atlas5','',0,2,58.3,3.8,386.8,null,5950,null,40000,null from dual union all
select 185,'Atlas V 421','C','orbital launch vehicle','Atlas5','',0,2,58.3,3.8,433.8,null,6890,null,40000,null from dual union all
select 186,'Atlas V 421','','orbital launch vehicle','Atlas5','',0,2,58.3,3.8,433.8,null,6890,null,40000,null from dual union all
select 187,'Atlas V 431','C','orbital launch vehicle','Atlas5','',0,2,59.1,3.8,479.7,null,7700,null,40000,null from dual union all
select 188,'Atlas V 431','','orbital launch vehicle','Atlas5','',0,2,59.1,3.8,479.7,null,7700,null,40000,null from dual union all
select 189,'Atlas V 501','C','orbital launch vehicle','Atlas5','',1,2,58.3,3.8,334.5,null,3775,null,40000,null from dual union all
select 19,'ARPAT','','endoatmoshpheric test/research','Sergeant','',0,2,8,.79,4,0,0,null,30,null from dual union all
select 190,'Atlas V 501','','orbital launch vehicle','Atlas5','',1,2,58.3,3.8,334.5,null,3775,null,40000,null from dual union all
select 191,'Atlas V 521','','orbital launch vehicle','Atlas5','',0,2,59.7,3.8,433.5,14000,6475,6520,40000,null from dual union all
select 192,'Atlas V 531','','orbital launch vehicle','Atlas5','',0,2,60,3.8,479.7,null,7700,null,40000,null from dual union all
select 193,'Atlas V 541','C','orbital launch vehicle','Atlas5','',0,2,60,3.8,479.7,null,7700,null,40000,null from dual union all
select 194,'Atlas V 541','','orbital launch vehicle','Atlas5','',0,2,60,3.8,479.7,null,7700,null,40000,null from dual union all
select 195,'Atlas V 551','C','orbital launch vehicle','Atlas5','',0,2,59.7,3.8,573.2,null,8900,null,40000,null from dual union all
select 196,'Atlas V 551','','orbital launch vehicle','Atlas5','',0,2,59.7,3.8,573.2,null,8900,null,40000,null from dual union all
select 197,'Aurore','','endoatmospheric weather rocket <80 km','Aurore','',1,1,3.3,.13,.04,0,0,8,60,null from dual union all
select 198,'B-611','','tactical ballistic missile','DF-11','',1,1,13.3,.88,8,0,0,90,150,600 from dual union all
select 199,'BLDT','','cruise missile','BLDT','',1,1,0,0,0,0,0,null,10,null from dual union all
select 2,'48N6','','research rocket','48N6','',1,1,7.5,.5,1.8,0,0,null,40,null from dual union all
select 20,'ASAT','','missile','SRAM','',1,2,5.4,.46,1.2,0,0,null,1000,null from dual union all
select 200,'BT-310','','research rocket','S-310','',1,1,6,.31,.7,0,0,null,5,null from dual union all
select 201,'Badr-1','','missile','Grad','',1,1,5,.6,2,0,0,null,20,null from dual union all
select 202,'Belenos','','research rocket','Belenos','',1,2,4.9,.21,.1,0,0,null,116,null from dual union all
select 203,'Belier','','research rocket','Belier','',1,1,3.8,.31,.4,0,0,null,109,null from dual union all
select 204,'Belier III','','research rocket','Belier','',1,1,3.8,.31,.4,0,0,null,109,null from dual union all
select 205,'Belisama','','research rocket','Belenos','',1,2,4.6,.21,.1,0,0,null,92,null from dual union all
select 206,'Berenice','','rocket test vehicle','Stromboli','',1,4,12.3,.56,3.4,0,0,170,250,null from dual union all
select 207,'Black Arrow','BA0','big test rocket','BlackArrow','',1,3,12.9,2,18.1,0,0,222,550,null from dual union all
select 208,'Black Arrow','','orbital launch vehicle','BlackArrow','',1,3,12.9,2,18.1,75,0,222,1600,null from dual union all
select 209,'Black Brant 10CM1','','research rocket','Terrier','',1,3,14.5,.46,2.6,0,0,257,900,null from dual union all
select 21,'ASCAMP','','research rocket','Asp','',1,2,5,.17,.1,0,0,42,110,null from dual union all
select 210,'Black Brant 8B','','research rocket','Nike','',1,2,11.9,.44,2,0,0,196,340,null from dual union all
select 211,'Black Brant 8C','','research rocket','Nike','',1,2,11.9,.44,2,0,0,196,340,null from dual union all
select 212,'Black Brant 9BM1','','research rocket','Terrier','',1,2,12.2,.46,2.2,0,0,257,300,null from dual union all
select 213,'Black Brant 9CM1','','research rocket','Terrier','',1,2,12.2,.46,2.2,0,0,257,300,null from dual union all
select 214,'Black Brant I','','research rocket','BB5','',1,1,8.4,.44,1,0,0,111,160,null from dual union all
select 215,'Black Brant II','','research rocket','BB5','',1,1,8.4,.44,1,0,0,111,200,null from dual union all
select 216,'Black Brant IIB','','research rocket','BB5','',1,1,8.4,.44,1,0,0,89,200,null from dual union all
select 217,'Black Brant III','','research rocket','BB3','',1,1,5.5,.26,.3,0,0,49,170,null from dual union all
select 218,'Black Brant IIIB','','research rocket','BB3','',1,1,5.5,.26,.3,0,0,44,220,null from dual union all
select 219,'Black Brant IVA','','research rocket','BB5','',1,2,12,.44,1.4,0,0,111,850,null from dual union all
select 22,'ASLV','','orbital launch vehicle','ASLV','',0,4,23.6,1,39,150,0,1383,900,null from dual union all
select 220,'Black Brant IVB','','research rocket','BB5','',1,2,12,.44,1.4,0,0,111,950,null from dual union all
select 221,'Black Brant IX','2','research rocket','Terrier','',1,2,12.2,.46,2.2,0,0,257,300,null from dual union all
select 222,'Black Brant IX','3','research rocket','Terrier','',1,2,12.2,.46,2.2,0,0,257,300,null from dual union all
select 223,'Black Brant IX','4','research rocket','Terrier','',1,2,12.2,.46,2.2,0,0,257,300,null from dual union all
select 224,'Black Brant IX','','research rocket','Terrier','',1,2,12.2,.46,2.2,0,0,257,300,null from dual union all
select 225,'Black Brant IXB','','research rocket','Terrier','',1,2,12.2,.46,2.2,0,0,257,300,null from dual union all
select 226,'Black Brant VA','','research rocket','BB5','',1,1,8.3,.44,1.1,0,0,111,170,null from dual union all
select 227,'Black Brant VB','','research rocket','BB5','',1,1,8.3,.44,1.2,0,0,68,400,null from dual union all
select 228,'Black Brant VC','','research rocket','BB5','',1,1,8.3,.44,1.3,0,0,68,350,null from dual union all
select 229,'Black Brant VI','','endoatmospheric weather rocket <80 km','BB6','',1,1,2.8,.12,.1,0,0,7,72,null from dual union all
select 23,'ATACMS','','research rocket','ATACMS','',1,1,5,.61,1,0,0,57,40,null from dual union all
select 230,'Black Brant VII','','endoatmospheric weather rocket <80 km','BB7','',1,1,2.1,.09,0,0,9,null,40,null from dual;
insert into LAUNCH_VEHICLE
select 231,'Black Brant X','','research rocket','Terrier','',1,3,14.5,.46,2.6,0,0,257,900,null from dual union all
select 232,'Black Brant XB','','research rocket','Terrier','',1,3,14.5,.46,2.6,0,0,257,900,null from dual union all
select 233,'Black Brant XI','1','research rocket','Talos','',1,3,17,.76,5.3,0,0,489,1400,null from dual union all
select 234,'Black Brant XI','','research rocket','Talos','',1,3,17,.76,5.3,0,0,489,1400,null from dual union all
select 235,'Black Brant XIA','','research rocket','Talos','',1,3,17,.76,5.3,0,0,489,1400,null from dual union all
select 236,'Black Brant XII','1','research rocket','Talos','',1,4,17,.76,5.3,0,0,489,1500,null from dual union all
select 237,'Black Brant XII','','research rocket','Talos','',1,4,17,.76,5.3,0,0,489,1500,null from dual union all
select 238,'Black Brant XIIA','','research rocket','Talos','',1,4,17,.76,5.3,0,0,489,1500,null from dual union all
select 239,'Black Knight','201','rocket test vehicle','BlackKnight','',1,1,10.2,.91,5.4,0,0,75,500,null from dual union all
select 24,'ATV','','research rocket','Stromboli','',1,2,10.3,.56,3,0,0,120,400,null from dual union all
select 240,'Black Knight','201/C','rocket test vehicle','BlackKnight','',1,2,11.6,.91,6.4,0,0,95,500,null from dual union all
select 241,'Black Knight','201/C2','rocket test vehicle','BlackKnight','',1,2,11.6,.91,6.4,0,0,95,500,null from dual union all
select 242,'Black Knight','301/C','rocket test vehicle','BlackKnight','',1,2,11.6,.91,6.4,0,0,95,500,null from dual union all
select 243,'Black Knight','301/C2','rocket test vehicle','BlackKnight','',1,2,11.6,.91,6.4,0,0,95,500,null from dual union all
select 244,'Black Sparrow','','tactical ballistic missile','BlackSparrow','',0,1,4.8,.53,1.2,0,0,null,40,null from dual union all
select 245,'Blue Scout I','','research rocket','Scout','',1,3,21.7,1.02,16.7,0,0,556,5000,null from dual union all
select 246,'Blue Scout II','','orbital launch vehicle','Scout','',1,4,21.7,1.02,16.9,50,0,556,2500,null from dual union all
select 247,'Blue Scout Jr','','research rocket','Sergeant','',1,4,12.4,.79,5.8,0,0,222,200000,null from dual union all
select 248,'Blue Scout Jr SLV-1B(m)','','research rocket','Sergeant','',1,3,12,.79,5.8,0,0,222,1000,null from dual union all
select 249,'Blue Scout Jr SLV-1C','','research rocket','Sergeant','',1,3,12.4,.79,5.8,0,0,222,1000,null from dual union all
select 25,'Ababeel','','missile','DF-21','',1,3,22,1.4,25,0,0,null,300,2000 from dual union all
select 250,'Blue Sparrow','','tactical ballistic missile','BlackSparrow','',0,1,6.5,.53,1.9,0,0,null,100,null from dual union all
select 251,'Blue Streak','','big test rocket','BlueStreak','',1,1,31.7,3.05,100,0,0,1512,250,null from dual union all
select 252,'Blue Water','','missile','Raven','',1,2,7.6,.44,1.4,0,0,196,20,null from dual union all
select 253,'Bold Orion','1','research rocket','Sergeant','',0,1,11.3,.79,null,0,0,null,100,null from dual union all
select 254,'Bold Orion','2','research rocket','Sergeant','',0,2,11.3,.79,null,0,0,null,200,null from dual union all
select 255,'Bomarc A','','cruise missile','Bomarc','',1,1,14.2,.89,6.8,0,0,null,20,null from dual union all
select 256,'Bomarc B','','cruise missile','Bomarc','',1,1,14.2,.89,6.8,0,0,null,20,null from dual union all
select 257,'Boosted Arcas','','exoatmospheric weather rocket','ArcasB','',1,2,3.4,.11,0,0,0,1,90,null from dual union all
select 258,'Boosted Arcas 2','','exoatmospheric weather rocket','ArcasB','',1,2,3.4,.11,0,0,0,1,90,null from dual union all
select 259,'Bukguekseong','','missile','Bukguekseong','',1,2,9.5,1.5,18,0,0,null,500,3000 from dual union all
select 26,'Aeolus','','endoatmoshpheric test/research','LongTom','',1,2,6.6,.41,.5,0,0,80,50,null from dual union all
select 260,'Bukguekseong-2','','missile','Bukguekseong','',1,2,9.5,1.5,18,0,0,null,500,3000 from dual union all
select 261,'Bulava','','missile','Topol','',1,3,20.5,1.8,47.1,0,0,890,1000,1050 from dual union all
select 262,'Bull Goose','','cruise missile','BullGoose','',1,1,10.2,.85,3.5,0,0,null,20,null from dual union all
select 263,'Bullpup Apache','','endoatmoshpheric test/research','Bullpup','',1,2,7,.3,.4,0,0,111,50,null from dual union all
select 264,'Bullpup Cajun','','endoatmoshpheric test/research','Bullpup','',1,2,7,.3,.4,0,0,111,41,null from dual union all
select 265,'Bumper-WAC','','big test rocket','V-2','',1,2,17.2,1.65,12.9,0,0,267,250,null from dual union all
select 266,'Burkan-1H','','tactical ballistic missile','R-11','',1,1,10,.88,5,0,0,90,150,null from dual union all
select 267,'Burkan-2H','','tactical ballistic missile','R-11','',1,1,10,.88,5,0,0,90,150,null from dual union all
select 268,'Burya','','cruise missile','Burya','',1,2,22,2.4,120,0,0,1530,50,null from dual union all
select 269,'CB-250/LPR-006','','endoatmospheric weather rocket <80 km','CB-250','',0,1,3.5,.25,.1,0,0,8,30,null from dual union all
select 27,'Aero High','','research rocket','Gosling','',1,2,6.8,.26,.43,0,0,99,200,null from dual union all
select 270,'CB-250/RH-125','','endoatmospheric weather rocket <80 km','CB-250','',0,1,3.5,.25,.1,0,0,8,30,null from dual union all
select 271,'CSXT','','research rocket','CSXT','',1,1,6.5,.25,.3,0,0,null,100,null from dual union all
select 272,'Cajun','','endoatmospheric research test vehicle','Deacon','',1,1,4,.17,.1,0,0,36,20,null from dual union all
select 273,'Cajun Dart','','endoatmospheric weather rocket <80 km','Deacon','',1,1,4.1,.17,.1,0,0,36,74,null from dual union all
select 274,'Caleb','','big test rocket','NOTS500','',0,2,4.9,.61,1.3,0,0,null,1200,null from dual union all
select 275,'Canopus 1','','research rocket','Rigel','',1,1,4,.28,.3,0,0,null,150,null from dual union all
select 276,'Canopus 2','','research rocket','Rigel','',1,1,4,.28,.3,0,0,null,150,null from dual union all
select 277,'Castor','2R','research rocket','Sergeant','',0,1,10,.79,5.2,0,0,586,240,null from dual union all
select 278,'Castor','','research rocket','Sergeant','',1,1,8,.79,null,0,0,235,300,null from dual union all
select 279,'Castor (A)','','research rocket','Rigel','',1,2,7,.28,.3,0,0,null,400,null from dual union all
select 28,'Aerobee','','research rocket','AerobeeB','',1,2,7.9,.38,.7,0,0,18,130,null from dual union all
select 280,'Castor 4A','','research rocket','Castor4','',1,1,16,1.02,13.5,0,0,430,500,null from dual union all
select 281,'Castor 4B','','research rocket','Castor4','',1,1,14,1.02,13,0,0,500,500,null from dual union all
select 282,'Castor Lance','','research rocket','Sergeant','',0,2,15.1,.79,5.8,0,0,null,600,null from dual union all
select 283,'Castor-M57A1','','research rocket','Castor4','',1,2,16,1.02,14,0,0,500,1100,null from dual union all
select 284,'Castor-Recruit','','research rocket','Sergeant','',1,2,null,.79,null,0,0,null,150,null from dual union all
select 285,'Cedre 3','','research rocket','Cedre','',1,3,6.8,.52,1.2,0,0,null,200,null from dual union all
select 286,'Cedre 4','','research rocket','Cedre','',1,3,7,.52,1.2,0,0,null,200,null from dual union all
select 287,'Centaure','','research rocket','Venus','Centaure 1',1,2,5.9,.28,.6,0,0,44,130,null from dual union all
select 288,'Centaure 1','','research rocket','Venus','Centaure 1',1,2,5.9,.28,.6,0,0,44,130,null from dual union all
select 289,'Centaure 2A','','research rocket','Venus','',1,2,5.9,.28,.6,0,0,44,146,null from dual union all
select 29,'Aerobee 100','','research rocket','AerobeeB','',1,2,7.8,.38,.7,0,0,18,100,null from dual union all
select 290,'Centaure 2B','','research rocket','Venus','',1,2,6.3,.28,.5,0,0,44,146,null from dual union all
select 291,'Centaure 2C','','research rocket','Venus','',1,2,5.9,.28,.6,0,0,44,146,null from dual union all
select 292,'Centenario','','research rocket','Rigel','',1,1,4.6,.28,.3,0,0,null,50,null from dual union all
select 293,'Chang Zheng 1','','orbital launch vehicle','DF3','',1,3,29.5,2.25,81.5,250,0,1224,2500,null from dual union all
select 294,'Chang Zheng 11','','orbital launch vehicle','DF-31','',1,4,20,2,58,500,0,1177,200,200 from dual union all
select 295,'Chang Zheng 1D','','orbital launch vehicle','DF3','',1,3,29.5,2.25,81.5,250,0,1224,2500,null from dual union all
select 296,'Chang Zheng 2','','orbital launch vehicle','DF5','',1,2,31.2,3.35,190,2200,0,2786,500,null from dual union all
select 297,'Chang Zheng 2C','SM','orbital launch vehicle','DF5','',1,2,35.2,3.35,192,2800,0,2962,500,null from dual union all
select 298,'Chang Zheng 2C','SMA','orbital launch vehicle','DF5','',1,2,35.2,3.35,192,2800,0,2962,500,null from dual union all
select 299,'Chang Zheng 2C','j','orbital launch vehicle','DF5','',1,2,35.2,3.35,192,2800,0,2962,500,null from dual union all
select 3,'53T6','','tactical ballistic missile','PRS-1','',1,1,10,2,12,0,0,null,40,80 from dual union all
select 30,'Aerobee 150','','research rocket','AerobeeB','',1,2,9.3,.38,.9,0,0,18,270,null from dual union all
select 300,'Chang Zheng 2C','zj','orbital launch vehicle','DF5','',1,2,35.2,3.35,192,2800,0,2962,500,null from dual union all
select 301,'Chang Zheng 2C','','orbital launch vehicle','DF5','',1,2,35.2,3.35,192,2800,0,2962,500,null from dual union all
select 302,'Chang Zheng 2C-III/SD','','orbital launch vehicle','DF5','',1,3,39.9,3.35,213,2800,0,2962,1000,null from dual union all
select 303,'Chang Zheng 2D','j','orbital launch vehicle','DF5','',1,2,38.3,3.35,232,3400,0,2962,500,null from dual union all
select 304,'Chang Zheng 2D','','orbital launch vehicle','DF5','',1,2,38.3,3.35,232,3400,0,2962,500,null from dual union all
select 305,'Chang Zheng 2E','','orbital launch vehicle','DF5','',0,2,51.2,3.35,464,9200,3500,5923,500,null from dual union all
select 306,'Chang Zheng 2F','','orbital launch vehicle','DF5','',0,2,51.2,3.35,464,9200,3500,5923,500,null from dual union all
select 307,'Chang Zheng 2FT1','','orbital launch vehicle','DF5','',0,2,51.2,3.35,480,9200,3500,5923,500,null from dual union all
select 308,'Chang Zheng 3','','orbital launch vehicle','DF5','',1,3,43.9,3.35,202,0,1400,2962,40000,null from dual union all
select 309,'Chang Zheng 3A','','orbital launch vehicle','DF5','',1,3,52.5,3.35,240,0,2300,2962,40000,null from dual union all
select 31,'Aerobee 150 MI','17','research rocket','AerobeeB','',1,2,9.3,.38,.9,0,0,18,270,null from dual union all
select 310,'Chang Zheng 3B','G1','orbital launch vehicle','DF5','',0,3,56.5,3.35,457,0,5400,5923,40000,null from dual union all
select 311,'Chang Zheng 3B','G2','orbital launch vehicle','DF5','',0,3,56.5,3.35,457,0,5400,5923,40000,null from dual union all
select 312,'Chang Zheng 3B','G3','orbital launch vehicle','DF5','',0,3,56.5,3.35,457,0,5400,5923,40000,null from dual union all
select 313,'Chang Zheng 3B','','orbital launch vehicle','DF5','',0,3,54.8,3.35,425.8,0,4800,5923,40000,null from dual union all
select 314,'Chang Zheng 3B/YZ-1','','orbital launch vehicle','DF5','',0,4,54.8,3.35,345,0,4800,4441,40000,null from dual union all
select 315,'Chang Zheng 3C','G2','orbital launch vehicle','DF5','',0,3,54.8,3.35,345,0,4800,4441,40000,null from dual union all
select 316,'Chang Zheng 3C','','orbital launch vehicle','DF5','',0,3,54.8,3.35,345,0,4800,4441,40000,null from dual union all
select 317,'Chang Zheng 3C/YZ-1','','orbital launch vehicle','DF5','',0,4,54.8,3.35,345,0,4800,4441,40000,null from dual union all
select 318,'Chang Zheng 4','','orbital launch vehicle','DF5','',1,3,42,3.35,249,1650,0,2962,1000,null from dual union all
select 319,'Chang Zheng 4B','','orbital launch vehicle','DF5','',1,3,43.1,3.35,254.4,2800,1500,2962,1000,null from dual union all
select 32,'Aerobee 150 MI','20','research rocket','AerobeeB','',1,2,9.3,.38,.9,0,0,18,270,null from dual union all
select 320,'Chang Zheng 4C','','orbital launch vehicle','DF5','',1,3,43.1,3.35,254.4,2800,1500,2962,1000,null from dual;
insert into LAUNCH_VEHICLE
select 321,'Chang Zheng 5','522','orbital launch vehicle','CZ5','',-1,2,59.5,5,643,25000,14000,1180,null,null from dual union all
select 322,'Chang Zheng 5','540','orbital launch vehicle','CZ5','',0,2,59.5,5,643,25000,14000,1180,null,null from dual union all
select 323,'Chang Zheng 5/YZ-2','','orbital launch vehicle','CZ5','',0,3,59.5,5,643,25000,14000,1180,null,null from dual union all
select 324,'Chang Zheng 6','','orbital launch vehicle','DF5','',1,3,29,3.35,103,1000,0,null,null,null from dual union all
select 325,'Chang Zheng 7','','orbital launch vehicle','DF5','',0,2,53.1,3.35,594,13500,7000,null,null,null from dual union all
select 326,'Chang Zheng 7/YZ-1A','','orbital launch vehicle','DF5','',0,3,53.1,3.35,594,13500,7000,null,null,null from dual union all
select 327,'Cirrus I','','research rocket','Cirrus','',1,2,4.1,.2,.1,0,0,5,35,null from dual union all
select 328,'Cirrus II','','research rocket','Cirrus','',1,2,4.1,.2,.1,0,0,5,35,null from dual union all
select 329,'Cleansweep III','','endoatmospheric weather rocket <80 km','Asp','',1,1,3.6,.17,.1,0,0,16,10,null from dual union all
select 33,'Aerobee 150 MI','','research rocket','AerobeeB','',1,2,9.3,.38,.9,0,0,18,270,null from dual union all
select 330,'Cleansweep IIIB','','research rocket','Sparrow','',0,1,3,.2,.1,0,0,16,51,null from dual union all
select 331,'Cockatoo','','research rocket','Gosling','',1,2,6.2,.26,.3,0,0,111,140,null from dual union all
select 332,'Cockatoo Mk 2','','research rocket','Gosling','',1,2,6.3,.26,.32,0,0,111,140,null from dual union all
select 333,'Cockatoo Mk 3','','research rocket','Gosling','',1,2,6.3,.26,.32,0,0,99,140,null from dual union all
select 334,'Cockatoo Mk 4','','research rocket','Gosling','',1,2,6.3,.26,.32,0,0,99,140,null from dual union all
select 335,'Commercial Titan 3','','orbital launch vehicle','Titan','',0,2,47.3,3.05,680,14515,1850,12450,40000,null from dual union all
select 336,'Conestoga 1620','','orbital launch vehicle','Castor4','',1,4,15.2,1.02,87.4,900,0,1580,450,null from dual union all
select 337,'Cora','','endoatmoshpheric test/research','Coralie','',1,2,11,2.01,16,0,0,268,55,null from dual union all
select 338,'Corella','','research rocket','Gosling','',1,2,8,.26,.44,0,0,99,220,null from dual union all
select 339,'Corporal','','endoatmospheric rocket','Corporal','',1,1,14,.76,5.4,0,0,89,50,null from dual union all
select 34,'Aerobee 150 MII','20','research rocket','AerobeeB','',1,2,9.3,.38,.9,0,0,18,270,null from dual union all
select 340,'Corporal E','','endoatmoshpheric test/research','Corporal','',1,1,14,.76,4.5,0,0,89,50,null from dual union all
select 341,'Corporal Type 1','','endoatmospheric rocket','Corporal','',1,1,14,.76,5.4,0,0,89,50,null from dual union all
select 342,'Coyote','','research rocket','Terrier','',1,1,9.5,.46,1,0,0,258,1,null from dual union all
select 343,'D-Region Tomahawk','','research rocket','Tomahawk','',1,1,5,.23,.3,0,0,44,120,null from dual union all
select 344,'DAC Roc','','endoatmoshpheric test/research','DACRoc','',1,1,null,.13,null,0,0,null,50,null from dual union all
select 345,'DC-X','','endoatmoshpheric test/research','DC-X','',1,1,12.6,4.95,18.8,0,0,242,10,null from dual union all
select 346,'DC-XA','','endoatmoshpheric test/research','DC-X','',1,1,12.6,4.95,19,0,0,242,10,null from dual union all
select 347,'DF-1','','tactical ballistic missile','V-2','',1,1,17.6,1.65,20.4,0,0,250,100,590 from dual union all
select 348,'DF-11','','tactical ballistic missile','DF-11','',1,1,11.3,.88,6.3,0,0,90,150,300 from dual union all
select 349,'DF-11M','','tactical ballistic missile','DF-11','',1,1,13.3,.88,8,0,0,90,150,600 from dual union all
select 35,'Aerobee 150A','','research rocket','AerobeeB','',1,2,9.3,.38,.9,0,0,18,270,null from dual union all
select 350,'DF-15','','tactical ballistic missile','DF-15','',1,1,9.1,1,6.2,0,0,null,100,600 from dual union all
select 351,'DF-16','','missile','DF-16','',1,1,12,1.2,18,0,0,null,200,1000 from dual union all
select 352,'DF-17','','missile','DF-16','',1,1,12,1.2,18,0,0,null,200,1000 from dual union all
select 353,'DF-2','','tactical ballistic missile','V-2','',1,1,20.6,1.65,32,0,0,300,200,1050 from dual union all
select 354,'DF-21','','missile','DF-21','',1,2,10.7,1.4,14.7,0,0,null,500,1800 from dual union all
select 355,'DF-2A','','tactical ballistic missile','V-2','',1,1,20.6,1.65,32,0,0,300,200,1250 from dual union all
select 356,'DF-3','','missile','DF3','',1,1,24,2.25,64,0,0,1224,500,2650 from dual union all
select 357,'DF-31','','missile','DF-31','',1,3,11,2,20,0,0,null,1000,8000 from dual union all
select 358,'DF-3A','','missile','DF3','',1,1,24,2.25,64,0,0,1224,500,2650 from dual union all
select 359,'DF-4','','missile','DF3','',1,2,28,2.25,82,0,0,1224,500,4750 from dual union all
select 36,'Aerobee 150A MII','','research rocket','AerobeeB','',1,2,9.3,.38,.9,0,0,18,270,null from dual union all
select 360,'DF-41','','missile','DF-31','',1,3,11,2,20,0,0,null,1000,8000 from dual union all
select 361,'DIM','','endoatmospheric weather rocket <80 km','DIM','',1,1,2.9,.12,0,0,0,null,70,null from dual union all
select 362,'DOT','','research rocket','Sergeant','',0,2,11.6,.79,5.5,0,0,586,300,null from dual union all
select 363,'Daniel','','research rocket','Daniel','',1,3,8.4,.4,1,0,0,null,130,null from dual union all
select 364,'Dauphin','','research rocket','Stromboli','',1,1,5.3,.56,1.1,0,0,97,116,null from dual union all
select 365,'Deacon','','endoatmospheric research test vehicle','Deacon','',1,1,4,.15,.1,0,0,27,20,null from dual union all
select 366,'Deacon Arrow II','','research rocket','Deacon','',1,2,5.1,.15,.1,0,0,27,85,null from dual union all
select 367,'Deacon Deacon','','endoatmospheric research test vehicle','Deacon','',1,2,5,.15,.1,0,0,27,20,null from dual union all
select 368,'Deacon HVAR','','endoatmospheric research test vehicle','Deacon','',1,2,4,.15,.1,0,0,27,20,null from dual union all
select 369,'Deacon Judi','','research rocket','Deacon','',1,2,4.2,.16,.1,0,0,21,146,null from dual union all
select 37,'Aerobee 170','','research rocket','Nike','',1,2,12.6,.42,1.4,0,0,225,200,null from dual union all
select 370,'Deacon Rockoon','','research rocket','Deacon','',0,1,3.7,.16,.1,0,0,15,100,null from dual union all
select 371,'Deacon Sidewinder','','endoatmospheric research test vehicle','Deacon','',1,2,4,.15,.1,0,0,27,20,null from dual union all
select 372,'Deacon-Loki Rockoon','','research rocket','Deacon','',0,2,5.2,.16,.1,0,0,15,100,null from dual union all
select 373,'Delta 0300','','orbital launch vehicle','Thor','',0,2,34,2.44,90.1,1000,0,1590,1500,null from dual union all
select 374,'Delta 0900','','orbital launch vehicle','Thor','',0,2,34,2.44,116.5,1300,0,2500,1500,null from dual union all
select 375,'Delta 1410','','orbital launch vehicle','Thor','',0,2,35,2.44,110,1000,0,1863,900,null from dual union all
select 376,'Delta 1604','','orbital launch vehicle','Thor','',0,3,35,2.44,118,null,600,2410,200000,null from dual union all
select 377,'Delta 1900','','orbital launch vehicle','Thor','',0,2,35,2.44,130,null,0,2410,4000,null from dual union all
select 378,'Delta 1910','','orbital launch vehicle','Thor','',0,2,35,2.44,130,null,0,2410,550,null from dual union all
select 379,'Delta 1913','','orbital launch vehicle','Thor','',0,3,35,2.44,130,null,null,2410,400000,null from dual union all
select 38,'Aerobee 170A','','research rocket','Nike','',1,2,12.4,.42,1.5,0,0,217,200,null from dual union all
select 380,'Delta 1914','','orbital launch vehicle','Thor','',0,3,35,2.44,130.3,null,null,2410,40000,null from dual union all
select 381,'Delta 2310','','orbital launch vehicle','Thor','',0,2,35,2.44,131,null,0,1737,1500,null from dual union all
select 382,'Delta 2313','','orbital launch vehicle','Thor','',0,3,35,2.44,131,1540,500,1737,40000,null from dual union all
select 383,'Delta 2910','','orbital launch vehicle','Thor','',0,2,35,2.44,131,null,0,2560,3000,null from dual union all
select 384,'Delta 2913','','orbital launch vehicle','Thor','',0,3,35,2.44,131.8,2000,700,2560,100000,null from dual union all
select 385,'Delta 2914','C','orbital launch vehicle','Thor','',0,3,35,2.44,130.4,2000,724,2560,40000,null from dual union all
select 386,'Delta 2914','','orbital launch vehicle','Thor','',0,3,35,2.44,130.4,2000,724,2560,400000,null from dual union all
select 387,'Delta 3910','','orbital launch vehicle','Thor','',0,2,35,2.44,190,null,0,3188,1000,null from dual union all
select 388,'Delta 3910/PAM','','orbital launch vehicle','Thor','',0,3,35,2.44,191.6,0,1087,3188,40000,null from dual union all
select 389,'Delta 3913','','orbital launch vehicle','Thor','',0,3,35,2.44,190,null,null,3188,25000,null from dual union all
select 39,'Aerobee 170B','','research rocket','Nike','',1,2,12.4,.42,1.5,0,0,225,200,null from dual union all
select 390,'Delta 3914','','orbital launch vehicle','Thor','',0,3,35,2.44,190.3,2700,951,3188,200000,null from dual union all
select 391,'Delta 3920','','orbital launch vehicle','Thor','',0,2,36,2.44,191,null,0,3188,700,null from dual union all
select 392,'Delta 3920-8','','orbital launch vehicle','Thor','',0,2,36,2.44,190,null,0,3188,500,null from dual union all
select 393,'Delta 3920/PAM','','orbital launch vehicle','Thor','',0,3,36,2.44,191.6,0,1280,3188,40000,null from dual union all
select 394,'Delta 3924','','orbital launch vehicle','Thor','',0,3,36,2.44,191,null,null,3188,120000,null from dual union all
select 395,'Delta 4925-8','','orbital launch vehicle','Thor','',0,3,34,2.44,200.7,null,1200,3470,40000,null from dual union all
select 396,'Delta 4H','A','orbital launch vehicle','Delta4','',0,2,70.7,5,733.4,28800,14200,9372,40000,null from dual union all
select 397,'Delta 4H','','orbital launch vehicle','Delta4','',0,2,70.7,5,733.4,null,null,8674,40000,null from dual union all
select 398,'Delta 4M','','orbital launch vehicle','Delta4','',1,2,63,5,249.5,8120,4210,2891,40000,null from dual union all
select 399,'Delta 4M+(4,2)','A','orbital launch vehicle','Delta4','',0,2,66.2,5,292.7,10430,5845,4200,40000,null from dual union all
select 4,'?','','research rocket','Unknown','',1,1,null,null,null,null,null,null,null,null from dual union all
select 40,'Aerobee 200','','research rocket','Nike','',1,2,12.6,.42,1.6,0,0,225,250,null from dual union all
select 400,'Delta 4M+(4,2)','','orbital launch vehicle','Delta4','',0,2,66.2,5,292.7,10430,5845,4200,40000,null from dual union all
select 401,'Delta 4M+(5,2)','A','orbital launch vehicle','Delta4','',0,2,70.7,5,404.6,11700,6565,5860,40000,null from dual union all
select 402,'Delta 4M+(5,2)','','orbital launch vehicle','Delta4','',0,2,70.7,5,404.6,11700,6565,5860,40000,null from dual union all
select 403,'Delta 4M+(5,4)','A','orbital launch vehicle','Delta4','',0,2,70.7,5,404.6,11700,6565,5860,40000,null from dual union all
select 404,'Delta 4M+(5,4)','','orbital launch vehicle','Delta4','',0,2,70.7,5,404.6,11700,6565,5860,40000,null from dual union all
select 405,'Delta 5920-8','','orbital launch vehicle','Thor','',0,2,34,2.44,201.6,3850,0,3470,1000,null from dual union all
select 406,'Delta 6920-10','','orbital launch vehicle','Thor','',0,2,39,2.44,219,null,0,3790,1000,null from dual union all
select 407,'Delta 6920-8','','orbital launch vehicle','Thor','',0,2,39,2.44,219,null,0,3790,1000,null from dual union all
select 408,'Delta 6925','','orbital launch vehicle','Thor','',0,3,39,2.44,219,0,1445,3790,400000,null from dual union all
select 409,'Delta 6925-8','','orbital launch vehicle','Thor','',0,3,39,2.44,219,0,1500,3790,40000,null from dual union all
select 41,'Aerobee 200A','','research rocket','Nike','',1,2,12.6,.42,1.6,0,0,225,250,null from dual union all
select 410,'Delta 7320-10','','orbital launch vehicle','Thor','',0,2,38.9,2.44,151.7,null,0,2500,1000,null from dual;
insert into LAUNCH_VEHICLE
select 411,'Delta 7320-10C','','orbital launch vehicle','Thor','',0,2,38.9,2.44,151.7,null,0,2500,1000,null from dual union all
select 412,'Delta 7326-9.5','','orbital launch vehicle','Thor','',0,3,38.4,2.44,155,null,null,2500,400000,null from dual union all
select 413,'Delta 7420-10C','','orbital launch vehicle','Thor','',0,2,39,2.44,165,null,0,3020,1500,null from dual union all
select 414,'Delta 7425-10','','orbital launch vehicle','Thor','',0,3,39,2.44,170,null,null,3020,400000,null from dual union all
select 415,'Delta 7425-9.5','','orbital launch vehicle','Thor','',0,3,39,2.44,170,null,null,3020,400000,null from dual union all
select 416,'Delta 7426-9.5','','orbital launch vehicle','Thor','',0,3,39,2.44,170,null,null,3020,400000,null from dual union all
select 417,'Delta 7920-10','','orbital launch vehicle','Thor','',0,2,39,2.44,220,5118,0,4000,1000,null from dual union all
select 418,'Delta 7920-10C','','orbital launch vehicle','Thor','',0,2,39,2.44,232,null,0,4000,1000,null from dual union all
select 419,'Delta 7920-10L','','orbital launch vehicle','Thor','',0,2,40,2.44,232,null,0,4000,1000,null from dual union all
select 42,'Aerobee 300','','research rocket','AerobeeB','',1,3,9.9,.38,.9,0,0,18,300,null from dual union all
select 420,'Delta 7920-8','','orbital launch vehicle','Thor','',0,2,39,2.44,220,5118,0,4000,400000,null from dual union all
select 421,'Delta 7920H','','orbital launch vehicle','Thor','',0,2,39.7,2.44,283,null,1816,5030,null,null from dual union all
select 422,'Delta 7925','','orbital launch vehicle','Thor','',0,3,39.7,2.44,219.9,null,1816,4000,400000,null from dual union all
select 423,'Delta 7925-10','','orbital launch vehicle','Thor','',0,3,38.9,2.44,231.9,null,null,4000,400000,null from dual union all
select 424,'Delta 7925-10C','','orbital launch vehicle','Thor','',0,3,38.9,2.44,231.9,null,null,4000,400000,null from dual union all
select 425,'Delta 7925-10L','','orbital launch vehicle','Thor','',0,3,38.9,2.44,231.9,null,null,4000,400000,null from dual union all
select 426,'Delta 7925-8','','orbital launch vehicle','Thor','',0,3,38.2,2.44,220,null,null,4000,400000,null from dual union all
select 427,'Delta 7925-9.5','','orbital launch vehicle','Thor','',0,3,39,2.44,231,null,null,4000,400000,null from dual union all
select 428,'Delta 7925H','','orbital launch vehicle','Thor','',0,3,39.7,2.44,283,null,1816,5030,null,null from dual union all
select 429,'Delta 8930','','orbital launch vehicle','Thor','',0,2,39.1,2.44,301.5,8280,3800,4891,40000,null from dual union all
select 43,'Aerobee 300A','','research rocket','AerobeeB','',1,3,9.9,.38,.9,0,0,18,300,null from dual union all
select 430,'Dhanush','','endoatmospheric rocket','Prithvi','',1,1,8.5,1.1,4.5,0,0,null,50,150 from dual union all
select 431,'Diamant A','','orbital launch vehicle','Emeraude','',1,3,18.9,1.4,18.5,40,0,275,2000,null from dual union all
select 432,'Diamant B','','orbital launch vehicle','Emeraude','',1,3,23.5,1.4,24.6,115,0,348,2000,null from dual union all
select 433,'Diamant BP.4','','orbital launch vehicle','Emeraude','',1,3,21.6,1.4,26.3,150,0,316,2000,null from dual union all
select 434,'Dnepr','','orbital launch vehicle','R-36','',1,3,34.3,3,211,4500,0,4525,1500,null from dual union all
select 435,'Dolphin','','endoatmoshpheric test/research','Dolphin','',1,1,15.5,1.07,7.5,0,0,155,2,null from dual union all
select 436,'Dong Feng 5','','missile','DF5','',1,2,32.6,3.35,183,0,0,2500,1000,13000 from dual union all
select 437,'Dong Feng 5A','','missile','DF5','',1,2,32.6,3.35,183,0,0,2500,1000,13000 from dual union all
select 438,'Dong Feng 5C','','missile','DF5','',1,2,32.6,3.35,183,0,0,2500,1000,13000 from dual union all
select 439,'Doorknob','1','endoatmoshpheric test/research','Lacrosse','',1,1,3.6,.4,.5,0,0,168,24,null from dual union all
select 44,'Aerobee 350','','research rocket','Nike','',1,2,15.9,.42,3.6,0,0,217,450,null from dual union all
select 440,'Doorknob','2','research rocket','Lacrosse','',1,2,5.7,.4,.9,0,0,168,84,null from dual union all
select 441,'Double Cajun T40','','endoatmospheric research test vehicle','Deacon','',1,2,5,.17,.1,0,0,36,20,null from dual union all
select 442,'Double Deacon','','endoatmospheric research test vehicle','Deacon','',1,1,5,.15,.1,0,0,54,20,null from dual union all
select 443,'Double FFAR','','endoatmospheric research test vehicle','FFAR','',1,2,2.4,.08,0,0,0,null,10,null from dual union all
select 444,'Double HPAG Deacon','','endoatmospheric research test vehicle','Sidewinder','',1,2,3,.13,0,0,0,12,10,null from dual union all
select 445,'Draco','','endoatmospheric research test vehicle','Sergeant','',1,2,14,.79,null,0,0,null,30,null from dual union all
select 446,'Dragon','','research rocket','Stromboli','',1,2,7,.56,1.2,0,0,75,440,null from dual union all
select 447,'Dragon 2B','','research rocket','Stromboli','',1,2,7,.56,1.2,0,0,97,440,null from dual union all
select 448,'Dragon 3','','research rocket','Stromboli','',1,2,7.3,.56,1.3,0,0,97,520,null from dual union all
select 449,'Dragon V2','','big test rocket','Dragon','',1,1,6.1,3.7,4.2,0,0,584,1,null from dual union all
select 45,'Aerobee 75','','research rocket','Orion','',1,1,6,.35,.4,0,0,7,80,null from dual union all
select 450,'E-8-5 VA','','deep space launch vehicle','E8','',1,1,2.3,.5,.8,0,0,null,99000,null from dual union all
select 451,'E-LRALT','','missile','SR19','',0,3,15.5,1.32,15,0,0,267,300,null from dual union all
select 452,'ELDO A','','orbital launch vehicle','BlueStreak','',1,3,31.7,3.05,104.6,1000,200,1512,2000,null from dual union all
select 453,'ERIS','','missile','Aries','',1,2,4.1,1.16,14,0,0,200,300,null from dual union all
select 454,'ETV','','big test rocket','Mu','',0,2,21.3,1.41,39.4,0,0,1932,150,null from dual union all
select 455,'Elbrus','','tactical ballistic missile','R-11','',1,1,11.3,.88,6.3,0,0,90,150,300 from dual union all
select 456,'Electron','','orbital launch vehicle','Electron','',1,3,17,1.2,12.6,150,0,162,200,200 from dual union all
select 457,'Emad','','tactical ballistic missile','Nodong','',1,1,15.6,1.25,16,0,0,255,150,2000 from dual union all
select 458,'Emeraude VE121','','rocket test vehicle','Emeraude','',1,1,17.9,1.4,18.2,0,0,275,200,null from dual union all
select 459,'Emma','','endoatmospheric weather rocket <80 km','Emma','',0,1,3.9,.13,.07,0,0,null,69,null from dual union all
select 46,'Aerobee AJ10-25','','research rocket','AerobeeB','',1,2,7.9,.38,.7,0,0,18,130,null from dual union all
select 460,'Energiya','','orbital launch vehicle','Energiya','',0,1,58.7,7.75,2524,88000,0,35139,250,null from dual union all
select 461,'Energiya/Buran','','orbital launch vehicle','Energiya','',0,1,58.7,7.75,2524,88000,0,35139,250,null from dual union all
select 462,'Epona','','endoatmospheric weather rocket <80 km','Belisama','',1,2,4.4,.1,.1,0,0,null,60,null from dual union all
select 463,'Epsilon','2','orbital launch vehicle','H2.SRB','',1,3,26,2.5,95.4,0,500,2250,500,null from dual union all
select 464,'Epsilon','2PBS','orbital launch vehicle','H2.SRB','',1,4,26,2.5,95.7,0,500,2250,500,null from dual union all
select 465,'Epsilon','','orbital launch vehicle','H2.SRB','',1,4,24.4,2.5,91,0,450,2250,500,null from dual union all
select 466,'Eridan','','research rocket','Stromboli','',1,2,8.3,.56,2.1,0,0,97,480,null from dual union all
select 467,'Europa I','','orbital launch vehicle','BlueStreak','',1,3,31.7,3.05,104.6,1000,200,1512,2000,null from dual union all
select 468,'Europa II','','orbital launch vehicle','BlueStreak','',1,4,31.7,3.05,111.7,1160,360,1512,40000,null from dual union all
select 469,'Excalibur','','exoatmospheric weather rocket','Excalibur','',1,1,3,.11,0,0,0,1,100,null from dual union all
select 47,'Aerobee AJ10-27','','research rocket','AerobeeB','',1,2,7.9,.38,.7,0,0,18,130,null from dual union all
select 470,'Exos','','research rocket','HJ','',1,3,12.9,.58,2.6,0,0,365,600,null from dual union all
select 471,'FFAR','','endoatmospheric research test vehicle','FFAR','',1,1,1.2,.08,0,0,0,null,10,null from dual union all
select 472,'FTB','','research rocket','FTB','',1,1,3,.13,.07,0,0,null,30,null from dual union all
select 473,'FTI','','research rocket','FTI','',1,1,5,.3,.2,0,0,null,60,null from dual union all
select 474,'FTV','','endoatmoshpheric test/research','ISRO-TV','',1,1,null,null,0,0,0,null,10,null from dual union all
select 475,'Falcon 1','','orbital launch vehicle','Falcon1','',1,2,21.3,1.7,27.2,600,0,378,500,null from dual union all
select 476,'Falcon 9','1.1','orbital launch vehicle','Falcon9','',1,2,71,3.65,480,13150,4850,5886,200,null from dual union all
select 477,'Falcon 9','FT3','orbital launch vehicle','Falcon9','',1,2,71,3.65,480,22800,8300,5886,200,null from dual union all
select 478,'Falcon 9','FT3/4','orbital launch vehicle','Falcon9','',1,2,71,3.65,480,13150,4850,5886,200,null from dual union all
select 479,'Falcon 9','FT4','orbital launch vehicle','Falcon9','',1,2,71,3.65,480,22800,8300,5886,200,null from dual union all
select 48,'Aerobee AJ10-34','','research rocket','AerobeeB','',1,2,7.9,.38,.7,0,0,18,130,null from dual union all
select 480,'Falcon 9','FT4/5','orbital launch vehicle','Falcon9','',1,2,68.4,3.65,480,13150,4850,5886,200,null from dual union all
select 481,'Falcon 9','FT5','orbital launch vehicle','Falcon9','',1,2,68.4,3.65,480,13150,4850,5886,200,null from dual union all
select 482,'Falcon 9','','orbital launch vehicle','Falcon9','',1,2,54.9,3.65,333.4,9300,3400,4940,200,null from dual union all
select 483,'Falcon 9R','1','orbital launch vehicle','Falcon9','',1,1,49,3.65,480,13150,4850,5886,10,null from dual union all
select 484,'Falcon Heavy','','orbital launch vehicle','Falcon9','',0,2,69.2,3.65,1400,54400,22200,17000,200,null from dual union all
select 485,'FalconLaunch','','exoatmospheric weather rocket','SuperLoki','',1,1,3.3,.1,0,0,0,22,108,null from dual union all
select 486,'Falstaff','1','endoatmoshpheric test/research','Stonechat','',1,1,5.3,.92,5.1,0,0,240,30,null from dual union all
select 487,'Falstaff','','endoatmoshpheric test/research','Stonechat','',1,1,5.3,.92,5.1,0,0,240,30,null from dual union all
select 488,'Farside','','research rocket','Recruit','',1,4,7.3,.46,.9,0,0,667,4500,null from dual union all
select 489,'Fateh-110','','missile','Zelzal','',1,1,9,.61,1,0,0,null,60,200 from dual union all
select 49,'Aerobee Hi','','research rocket','AerobeeB','',1,2,9.5,.38,.9,0,0,18,270,null from dual union all
select 490,'Father John','1','endoatmoshpheric test/research','HJ','',0,1,5,.58,1.5,0,0,365,10,null from dual union all
select 491,'Father John','2','endoatmoshpheric test/research','HJ','',0,1,5,.58,1.5,0,0,365,10,null from dual union all
select 492,'Feng Bao 1','','orbital launch vehicle','DF5','',1,2,32.6,3.35,192,2200,0,2500,1000,null from dual union all
select 493,'Forschungsflugkorper','','research rocket','FFK','',1,1,4,.25,.3,0,0,null,120,null from dual union all
select 494,'Frangible Arcas','','exoatmospheric weather rocket','Arcas','',1,1,2.3,.11,0,0,0,1,90,null from dual union all
select 495,'Fulmar','','research rocket','Gosling','',1,2,7.5,.26,.5,0,0,137,250,null from dual union all
select 496,'GBI','2','research rocket','Pegasus','',1,2,15.5,1.28,18,0,0,580,2000,null from dual union all
select 497,'GBI','','research rocket','Pegasus','',1,3,15.5,1.28,18,0,0,580,2000,null from dual union all
select 498,'GBI BV','','research rocket','GEM','',1,3,15.5,1.02,14,0,0,500,200,null from dual union all
select 499,'GBI BV+','','research rocket','GEM','',1,3,15.5,1.02,14,0,0,500,200,null from dual union all
select 5,'A-1','','research rocket','V-2','',1,1,14.1,1.65,13.4,0,0,267,100,null from dual union all
select 50,'Aerobee RTV-A-1','','research rocket','AerobeeB','',1,2,7.9,.38,.7,0,0,18,130,null from dual union all
select 500,'GLCM','','cruise missile','GLCM','',1,1,6.4,.53,1.8,0,0,null,20,2500 from dual;
insert into LAUNCH_VEHICLE
select 501,'GSLV Mk I','a','orbital launch vehicle','PSLV','',0,3,49,2.8,401,0,2500,7420,40000,null from dual union all
select 502,'GSLV Mk I','b','orbital launch vehicle','PSLV','',0,3,49,2.8,401,0,2500,7420,40000,null from dual union all
select 503,'GSLV Mk I','c','orbital launch vehicle','PSLV','',0,3,49,2.8,401,0,2500,7420,40000,null from dual union all
select 504,'GSLV Mk II','','orbital launch vehicle','PSLV','',0,3,49,2.8,401,0,2500,7420,40000,null from dual union all
select 505,'GSLV Mk III','','orbital launch vehicle','GSLV3','',0,2,42.4,4,629,null,4000,null,40000,null from dual union all
select 506,'Gamma Centauro','','endoatmospheric weather rocket <80 km','GammaCen','',1,2,2.4,.13,.03,0,0,null,30,null from dual union all
select 507,'Genie','','research rocket','Genie','',1,1,3.1,.38,.5,0,0,null,100,null from dual union all
select 508,'Ghadr 1','','tactical ballistic missile','Nodong','',1,1,15.6,1.25,16,0,0,255,150,2000 from dual union all
select 509,'Ghauri','','tactical ballistic missile','Nodong','',1,1,16,1.25,16,0,0,255,150,null from dual union all
select 51,'Aerobee RTV-A-1a','','research rocket','AerobeeB','',1,2,7.9,.38,.7,0,0,18,130,null from dual union all
select 510,'Ghaznavi','','tactical ballistic missile','DF-11','',1,1,9.1,.88,6.2,0,0,null,100,750 from dual union all
select 511,'Goose','','cruise missile','BullGoose','',1,1,10.2,.85,3.5,0,0,null,20,null from dual union all
select 512,'Gosling-Lobster','','research rocket','Gosling','',1,2,5,.26,.3,0,0,null,40,null from dual union all
select 513,'Grad','','missile','Grad','',1,1,5,.6,2,0,0,null,20,null from dual union all
select 514,'Gradicom 1','','endoatmoshpheric test/research','Gradicom','',1,1,4.5,.32,.5,0,0,null,40,null from dual union all
select 515,'Gradicom 2','','endoatmoshpheric test/research','Gradicom','',1,2,7.7,.32,.9,0,0,null,100,null from dual union all
select 516,'Grannos','','research rocket','Belenos','',1,2,4.9,.21,.2,0,0,null,143,null from dual union all
select 517,'Grasshopper','','big test rocket','Falcon9','',1,1,34,3.65,200,0,0,540,1,null from dual union all
select 518,'Grom','','missile','R-39','',1,3,16,2.4,84,0,0,2060,1000,830 from dual union all
select 519,'Gun Projectile 16-inch','','research rocket','Gun16','',1,1,1.7,.41,.2,0,0,null,100,null from dual union all
select 52,'Aerobee RTV-A-1b','','research rocket','AerobeeB','',1,2,7.9,.38,.7,0,0,18,130,null from dual union all
select 520,'Gun Projectile 5-inch','','research rocket','Gun05','',1,1,1,.13,.2,0,0,null,100,null from dual union all
select 521,'Gun Projectile 7-inch','','research rocket','Gun07','',1,1,1,.18,.2,0,0,null,100,null from dual union all
select 522,'H-1','(2)','orbital launch vehicle','Thor','',0,2,42,2.44,142,null,null,2130,1500,null from dual union all
select 523,'H-1','6R','orbital launch vehicle','Thor','',0,3,42,2.44,145,null,null,1350,40000,null from dual union all
select 524,'H-1','','orbital launch vehicle','Thor','',0,3,42,2.44,142,3200,1100,2130,40000,null from dual union all
select 525,'H-II','(2S)','orbital launch vehicle','H2','',-1,2,49,4,281,10060,3930,4600,40000,null from dual union all
select 526,'H-II','','orbital launch vehicle','H2','',0,2,49,4,260,10060,3930,3960,40000,null from dual union all
select 527,'H-IIA 202','1','orbital launch vehicle','H2','',0,2,53,4,285,10000,4100,5600,40000,null from dual union all
select 528,'H-IIA 202','','orbital launch vehicle','H2','',0,2,53,4,285,10000,4100,5600,40000,null from dual union all
select 529,'H-IIA 2022','','orbital launch vehicle','H2','',-1,2,57,4,348,null,5000,8580,40000,null from dual union all
select 53,'Aerobee RTV-A-1c','','research rocket','Aerobee','',1,1,7.5,.38,.7,0,0,18,130,null from dual union all
select 530,'H-IIA 2024','1','orbital launch vehicle','H2','',-1,2,57,4,348,null,5000,8580,40000,null from dual union all
select 531,'H-IIA 2024','','orbital launch vehicle','H2','',-1,2,57,4,348,null,5000,8580,40000,null from dual union all
select 532,'H-IIA 204','2','orbital launch vehicle','H2','',0,2,53,4,285,10000,4100,5600,40000,null from dual union all
select 533,'H-IIA 204','','orbital launch vehicle','H2','',0,2,53,4,285,10000,4100,5600,40000,null from dual union all
select 534,'H-IIB','','orbital launch vehicle','H2B','',0,2,56.6,5.2,551,19000,8000,9000,null,null from dual union all
select 535,'HAD','','research rocket','Gosling','',1,2,6.1,.26,.28,0,0,130,130,null from dual union all
select 536,'HAEC','','endoatmoshpheric test/research','Demon','',1,2,3.5,.18,.05,0,0,9,75,null from dual union all
select 537,'HALO','','endoatmoshpheric test/research','HALO','',1,1,2.2,.13,0,0,0,null,60,null from dual union all
select 538,'HAT','','endoatmospheric weather rocket <80 km','LongTom','',1,2,5,.47,.5,0,0,null,60,null from dual union all
select 539,'HEDI','','endoatmospheric rocket','Sprint','',1,2,8.2,1.37,3.5,0,0,3000,15,null from dual union all
select 54,'Aerobee RTV-N-10','','research rocket','AerobeeB','',1,2,7.9,.38,.7,0,0,18,130,null from dual union all
select 540,'HIBEX','','endoatmoshpheric test/research','HIBEX','',1,1,5.2,.79,6,0,0,null,20,null from dual union all
select 541,'HIMES Rockoon','','endoatmoshpheric test/research','KM-D','',0,1,3.5,.7,.5,0,0,null,70,null from dual union all
select 542,'HJ','','endoatmoshpheric test/research','HJ','',1,1,5,.58,1.5,0,0,365,10,null from dual union all
select 543,'HJ Hydac','','research rocket','HJ','',1,2,11.3,.58,2.6,0,0,365,150,null from dual union all
select 544,'HJ Javelin','','research rocket','HJ','',1,2,9.1,.58,1.7,0,0,365,100,null from dual union all
select 545,'HJ Nike','','research rocket','HJ','',1,2,14.3,.58,2.7,0,0,365,100,null from dual union all
select 546,'HJ Nike Gosling','','endoatmospheric research test vehicle','HJ','',1,3,10,.58,null,0,0,365,20,null from dual union all
select 547,'HJ Nike Hydac','','research rocket','HJ','',1,3,14.3,.58,2.9,0,0,365,500,null from dual union all
select 548,'HJ Nike Javelin','','research rocket','HJ','',1,3,13.4,.58,2.8,0,0,365,300,null from dual union all
select 549,'HJ Nike Nike','','research rocket','HJ','',1,3,10,.58,null,0,0,365,170,null from dual union all
select 55,'Aerobee RTV-N-10a','','research rocket','AerobeeB','',1,2,7.9,.38,.7,0,0,18,130,null from dual union all
select 550,'HJ Nike Nike 20"SM','','research rocket','HJ','',1,4,12,.58,3.2,0,0,365,1380,null from dual union all
select 551,'HJ Nike Nike Recruit','','endoatmospheric research test vehicle','HJ','',1,4,10,.58,1.3,0,0,365,30,null from dual union all
select 552,'HJ Nike Nike Recruit T55','','endoatmospheric research test vehicle','HJ','',1,5,10.9,.58,1.3,0,0,365,30,null from dual union all
select 553,'HJ Nike T40','','endoatmospheric research test vehicle','HJ','',1,3,12,.58,null,0,0,365,30,null from dual union all
select 554,'HJ Nike T40 T55','','endoatmospheric research test vehicle','HJ','',1,4,12,.58,null,0,0,365,30,null from dual union all
select 555,'HJ Nike Tri-Deacon T40','','endoatmospheric research test vehicle','HJ','',1,4,12,.58,null,0,0,365,30,null from dual union all
select 556,'HJ Orion','','research rocket','HJ','',1,2,12.6,.58,2,0,0,365,120,null from dual union all
select 557,'HJ PRV','','research rocket','HJ','',1,4,12,.58,null,0,0,365,100,null from dual union all
select 558,'HOE','','missile','Minuteman','',1,2,19,1.68,35,0,0,935,200,null from dual union all
select 559,'HPAG','','endoatmospheric research test vehicle','Sidewinder','',1,1,1.8,.13,0,0,0,12,10,null from dual union all
select 56,'Aerobee RTV-N-10b','','research rocket','AerobeeB','',1,2,7.9,.38,.7,0,0,18,130,null from dual union all
select 560,'HPAG Deacon','','endoatmospheric research test vehicle','Sidewinder','',1,2,3,.13,0,0,0,12,10,null from dual union all
select 561,'HPB','','research rocket','Talos','',1,2,14.4,.76,9,0,0,null,400,null from dual union all
select 562,'HS9','','research rocket','ASLV','',1,1,19,1,9,0,0,503,100,null from dual union all
select 563,'HTV','','research rocket','AeroHTV','',1,2,3.9,.23,.1,0,0,null,10,null from dual union all
select 564,'HVAR','','endoatmospheric research test vehicle','HVAR','',1,1,2,.13,0,0,0,25,10,null from dual union all
select 565,'HVAR FFAR','','endoatmospheric research test vehicle','HVAR','',1,2,3,.13,0,0,0,25,10,null from dual union all
select 566,'HYSR','','research rocket','HYSR','',1,1,18,.61,null,0,0,270,70,null from dual union all
select 567,'Hades','','tactical ballistic missile','Hades','',1,1,7.5,.53,1.9,0,0,null,150,300 from dual union all
select 568,'Hatf I','','research rocket','Stromboli','',1,1,5.3,.56,1.1,0,0,97,116,null from dual union all
select 569,'Hatf II','','research rocket','Stromboli','',1,2,8.3,.56,2.1,0,0,97,250,null from dual union all
select 57,'Aerobee RTV-N-10c','','research rocket','AerobeeB','',1,2,7.9,.38,.7,0,0,18,130,null from dual union all
select 570,'Hera','','missile','SR19','',1,2,11.2,1.32,10.8,0,0,267,300,null from dual union all
select 571,'Hermes A-1','','endoatmospheric rocket','Hermes','',1,1,7.8,.86,3,0,0,71,24,60 from dual union all
select 572,'Hermes A-3A','','endoatmospheric rocket','HermesA3','',1,1,8.7,1.02,3.8,0,0,80,30,null from dual union all
select 573,'Hermes A-3B','','endoatmospheric rocket','HermesA3','',1,1,10.1,1.19,5.4,0,0,100,37,100 from dual union all
select 574,'Hermes II','','research rocket','V-2','',1,1,14,1.65,13,0,0,312,200,null from dual union all
select 575,'High Virgo WS199C','','missile','Sergeant','',0,1,9.4,.79,3,0,0,220,250,null from dual union all
select 576,'Honest John','','endoatmoshpheric test/research','HJ','',1,1,5,.58,1.5,0,0,365,10,null from dual union all
select 577,'Hopi Dart','','exoatmospheric weather rocket','Hopi','',1,1,3.3,.11,0,0,0,null,116,null from dual union all
select 578,'Hugo','','endoatmospheric rocket','Terrier','',1,2,10,.46,1.3,0,0,null,160,null from dual union all
select 579,'Hwasong 5','','tactical ballistic missile','R-11','',1,1,10,.88,5,0,0,90,150,null from dual union all
select 58,'Aerobee RTV-N-8','','research rocket','AerobeeB','',1,2,7.9,.38,.7,0,0,18,130,null from dual union all
select 580,'Hwasong 6','','tactical ballistic missile','R-11','',1,1,10,.88,5,0,0,90,200,null from dual union all
select 581,'Hwasong 9','','tactical ballistic missile','R-11','-',1,1,16,.88,5,0,0,255,150,null from dual union all
select 582,'Hwasong-10','','missile','R-27','',1,1,9.7,1.5,14.2,0,0,null,500,null from dual union all
select 583,'Hwasong-12','','missile','Hwasong-12','',1,1,14,1.75,16,0,0,null,2000,null from dual union all
select 584,'Hwasong-14','','missile','Hwasong-12','',1,2,17.9,1.75,20,0,0,null,3000,null from dual union all
select 585,'Hwasong-15','','missile','Hwasong-15','',1,2,22,2.4,85,0,0,null,3000,null from dual union all
select 586,'Hwasong-7','','tactical ballistic missile','Nodong','',1,1,16,1.25,16,0,0,255,150,null from dual union all
select 587,'Hybrid Test Rocket','','endoatmoshpheric test/research','HTR','',1,1,10.6,.25,.47,0,0,30,30,null from dual union all
select 588,'Hydra Sandhawk','','research rocket','NOTS400','',1,2,10,.5,1.2,0,0,null,280,null from dual union all
select 589,'Hydra-Iris','','research rocket','Iris','',0,1,6,.31,.8,0,0,17,300,null from dual union all
select 59,'Aerobee XASR-SC-1','','research rocket','AerobeeB','',1,2,7.9,.38,.7,0,0,18,130,null from dual union all
select 590,'Hyper X','','endoatmoshpheric test/research','Pegasus','',0,1,14.9,1.28,18.8,0,0,726,40,null from dual union all
select 591,'Hyperion','','endoatmoshpheric test/research','AMROC','',1,1,5.8,.15,.1,0,0,6,36,null from dual;
insert into LAUNCH_VEHICLE
select 592,'Hyunmoo-1','','endoatmospheric rocket','Hyunmoo','',1,2,11.9,1.2,4.9,0,0,775,45,null from dual union all
select 593,'Hyunmoo-2A','','endoatmospheric rocket','Hyunmoo','',1,1,7,1.2,5,0,0,null,80,null from dual union all
select 594,'Hyunmoo-2B','','endoatmospheric rocket','Hyunmoo','',1,1,7,1.2,5,0,0,null,80,null from dual union all
select 595,'Hyunmoo-2C','','endoatmospheric rocket','Hyunmoo','',1,1,7,1.2,5,0,0,null,80,null from dual union all
select 596,'ICBM-T2','','missile','Trident','',1,3,17.6,1.68,33.1,0,0,935,1300,null from dual union all
select 597,'INTA-100','','endoatmospheric weather rocket <80 km','INTA-100','',1,2,4.1,.12,.07,0,0,17,100,null from dual union all
select 598,'INTA-255','','research rocket','Gosling','',0,1,6.1,.26,.3,0,0,22,150,null from dual union all
select 599,'INTA-300','','research rocket','Gosling','',1,2,7.3,.26,.5,0,0,137,250,null from dual union all
select 6,'A-350','','missile','A-350','',1,2,20,2.57,33,0,0,null,300,300 from dual union all
select 60,'Aerobee XASR-SC-2','','research rocket','AerobeeB','',1,2,7.9,.38,.7,0,0,18,130,null from dual union all
select 600,'INTA-300B','','research rocket','Gosling','',1,2,7.3,.26,.5,0,0,137,250,null from dual union all
select 601,'IRBM-T1','','missile','Pegasus','',1,2,15,1.28,23,350,0,726,4000,null from dual union all
select 602,'Icarus','','research rocket','Icarus','',1,2,4.9,.15,.05,0,0,16,60,null from dual union all
select 603,'Improved Honest John','','missile','HJ','',1,1,4.1,.58,1.5,0,0,null,10,null from dual union all
select 604,'Improved Malemute','','research rocket','Malemute','',1,1,2.4,.41,.8,0,0,57,165,null from dual union all
select 605,'Improved Orion','','research rocket','Orion','',1,1,5.6,.35,.4,0,0,7,85,null from dual union all
select 606,'Iraqi Gun Projectile','','research rocket','IraqGun350','',1,1,1.5,.35,.1,0,0,null,100,null from dual union all
select 607,'Iris','','research rocket','Iris','',0,1,5,.31,.6,0,0,17,250,null from dual union all
select 608,'Iskander','','tactical ballistic missile','Oka','',1,1,7.3,.92,3.8,0,0,null,50,400 from dual union all
select 609,'Iskander-E','','tactical ballistic missile','Oka','',1,1,7.3,.92,3.8,0,0,null,50,200 from dual union all
select 61,'Agate','','endoatmospheric research test vehicle','Agate','',1,1,8.5,.8,3.2,0,0,186,20,null from dual union all
select 610,'Iskander-M','','tactical ballistic missile','Oka','',1,1,7.3,.92,3.8,0,0,null,50,400 from dual union all
select 611,'Ivanov','','endoatmospheric weather rocket <80 km','Ivanov','',0,3,4.3,.14,.1,0,0,null,40,null from dual union all
select 612,'J-1','','big test rocket','H2.SRB','',1,2,33.1,1.8,88.5,0,0,1550,110,null from dual union all
select 613,'JCR','','big test rocket','JCR','',1,2,10.3,.51,2.3,0,0,133,200,null from dual union all
select 614,'JL-1','','missile','DF-21','',1,2,10.7,1.4,14.7,0,0,null,500,1800 from dual union all
select 615,'JL-2','','missile','DF-31','',1,3,11,2,20,0,0,null,1000,8000 from dual union all
select 616,'Jabiru','','endoatmospheric research test vehicle','Raven','',1,3,11.9,.44,1.7,0,0,null,24,null from dual union all
select 617,'Jabiru 2','','endoatmospheric research test vehicle','Raven','',1,3,12.9,.44,2.2,0,0,null,30,null from dual union all
select 618,'Jabiru 3','','endoatmospheric research test vehicle','Raven','',1,2,12.7,.44,2,0,0,null,30,null from dual union all
select 619,'Jaguar','','research rocket','Recruit','',0,3,8.8,.38,.8,0,0,440,800,null from dual union all
select 62,'Agni 1','','missile','ASLV','',1,1,15,1,12,0,0,503,200,700 from dual union all
select 620,'Jaguar 1','','endoatmospheric research test vehicle','Raven','',1,3,11.9,.44,1.7,0,0,null,24,null from dual union all
select 621,'Jaguar 2','','endoatmospheric research test vehicle','Raven','',1,3,12.9,.44,2.2,0,0,null,30,null from dual union all
select 622,'Janus','','missile','SR19','',0,2,11.2,1.32,10.8,0,0,267,300,null from dual union all
select 623,'Jason','','research rocket','HJ','',1,5,17.5,.58,3.4,0,0,365,800,null from dual union all
select 624,'Javelin','','research rocket','HJ','',1,4,14.8,.58,3.4,0,0,365,1100,null from dual union all
select 625,'Javelin 3','','research rocket','Javelin','',1,1,4.5,.23,.2,0,0,86,60,null from dual union all
select 626,'Jericho','','tactical ballistic missile','Agate','',1,2,13.4,.8,6.5,0,0,null,100,500 from dual union all
select 627,'Jericho II','','missile','Jericho','',1,1,15,1.35,20,0,0,610,300,null from dual union all
select 628,'Jericho III','','missile','Jericho','',1,2,15,1.35,20,0,0,610,300,null from dual union all
select 629,'Jericho-1','','tactical ballistic missile','Arrow2','',1,1,9,.8,6,0,0,null,100,null from dual union all
select 63,'Agni 2','','missile','ASLV','',1,2,20,1,16,0,0,503,405,2000 from dual union all
select 630,'Journeyman','','research rocket','Sergeant','',0,4,18.9,.79,6.3,0,0,null,3000,null from dual union all
select 631,'Judi-Dart','','endoatmospheric weather rocket <80 km','Loki','',1,1,2.7,.08,0,0,0,9,65,null from dual union all
select 632,'Juno','','missile','SR19','',1,2,11.2,1.32,10.8,0,0,267,300,null from dual union all
select 633,'Juno II','(3)','orbital launch vehicle','Jupiter','',1,3,23.4,2.67,55.3,50,0,667,1000,null from dual union all
select 634,'Juno II','','orbital launch vehicle','Jupiter','',1,4,23.4,2.67,55.3,50,0,667,100000,null from dual union all
select 635,'Jupiter A','','missile','Redstone','',1,1,20,1.78,28.4,0,0,357,95,null from dual union all
select 636,'Jupiter C','JunoI','orbital launch vehicle','Redstone','',1,4,21.2,1.78,28.5,11,0,357,5000,null from dual union all
select 637,'Jupiter C','','rocket test vehicle','Redstone','',1,3,21.2,1.78,28.5,11,0,357,1000,null from dual union all
select 638,'Jupiter IRBM','','missile','Jupiter','',1,1,18.4,2.67,54.4,0,0,667,500,3000 from dual union all
select 639,'K-15','','research rocket','K-15','',1,2,11,.5,4,0,0,null,300,null from dual union all
select 64,'Agni 2P','','missile','ASLV','Agni 4',1,2,20,1,16,0,0,503,405,2000 from dual union all
select 640,'K-4','','missile','K-4','',1,2,12,1.3,17,0,0,null,500,3500 from dual union all
select 641,'K-BATS','','missile','ATACMS','',1,1,4,.6,1.5,0,0,null,50,null from dual union all
select 642,'K150','','endoatmoshpheric test/research','K150','',1,1,4,.15,.1,0,0,null,7,null from dual union all
select 643,'K245','','endoatmoshpheric test/research','K250','',1,1,5.6,.25,.3,0,0,null,10,null from dual union all
select 644,'K63','','missile','R-12','',1,1,22.5,1.65,42,0,0,730,402,null from dual union all
select 645,'K63D','','missile','R-12','',1,1,22.5,1.65,42,0,0,730,402,null from dual union all
select 646,'K65M-R','','missile','R-14','',1,2,32.4,2.4,109,0,0,1485,500,null from dual union all
select 647,'K65M-RB','','orbital launch vehicle','R-14','',1,2,32.4,2.4,109,1500,0,1485,250,null from dual union all
select 648,'KN-02','','missile','Tochka','',1,1,6.4,.65,2,0,0,null,20,null from dual union all
select 649,'KSR-I','','endoatmoshpheric test/research','KSR','',1,1,6.7,.42,1.2,0,0,86,50,null from dual union all
select 65,'Agni 3','','missile','Agni3','',1,2,16,2,48,0,0,null,100,2000 from dual union all
select 650,'KSR-II','','research rocket','KSR','',1,2,6.7,.42,2,0,0,298,130,null from dual union all
select 651,'KSR-III','','research rocket','KSR-3','',1,1,13.5,1,5.6,0,0,126,100,null from dual union all
select 652,'KT-1','','orbital launch vehicle','DF-21','',1,3,11,1.4,20,50,0,null,1000,8000 from dual union all
select 653,'KT-2','','orbital launch vehicle','DF-31','',1,3,20,1.4,30,250,0,null,1000,8000 from dual union all
select 654,'Kappa 1','','endoatmoshpheric test/research','Kappa','',1,1,2.7,.13,0,0,0,10,40,null from dual union all
select 655,'Kappa 10','','research rocket','K420','',1,2,9.8,.42,1.8,0,0,null,250,null from dual union all
select 656,'Kappa 10C','','research rocket','K420','',1,2,10,.42,null,0,0,null,250,null from dual union all
select 657,'Kappa 10S','','research rocket','K420','',1,3,10,.42,null,0,0,null,750,null from dual union all
select 658,'Kappa 2','','endoatmoshpheric test/research','K250','',1,2,5,.22,.2,0,0,null,20,null from dual union all
select 659,'Kappa 3','','endoatmoshpheric test/research','K250','',1,2,5,.22,.2,0,0,null,40,null from dual union all
select 66,'Agni 4','','missile','ASLV','',1,2,20,1,16,0,0,503,405,2000 from dual union all
select 660,'Kappa 4','','research rocket','Kappa4','',1,2,5.9,.33,.2,0,0,105,80,null from dual union all
select 661,'Kappa 5','','research rocket','Kappa4','',1,2,5.9,.33,.2,0,0,105,80,null from dual union all
select 662,'Kappa 6','','endoatmoshpheric test/research','K250','',1,2,5.6,.25,.3,0,0,null,60,null from dual union all
select 663,'Kappa 6H','','research rocket','K250','',1,2,6.9,.25,.3,0,0,null,85,null from dual union all
select 664,'Kappa 7','','endoatmoshpheric test/research','K420','',1,1,6.7,.42,1.2,0,0,null,50,null from dual union all
select 665,'Kappa 8','','research rocket','K420','',1,2,10.9,.42,1.5,0,0,null,200,null from dual union all
select 666,'Kappa 8L','','research rocket','K250','',1,2,7.3,.25,.3,0,0,null,200,null from dual union all
select 667,'Kappa 9L','','research rocket','K420','',1,3,12.5,.42,1.6,0,0,null,350,null from dual union all
select 668,'Kappa 9M','','research rocket','K420','',1,2,11.1,.42,1.5,0,0,null,350,null from dual union all
select 669,'Kavoshgar K110','','research rocket','Zelzal','',1,1,10,.61,.9,0,0,null,50,null from dual union all
select 67,'Agni 5','','missile','Agni3','',1,3,17.2,2,50,0,0,null,100,5000 from dual union all
select 670,'Kavoshgar M5','','research rocket','Kavoshgar','',1,1,6,.36,.9,0,0,null,150,null from dual union all
select 671,'Khorramshahr','','missile','R-27','',1,1,9.7,1.5,14.2,0,0,null,500,null from dual union all
select 672,'Kisha Judi','','research rocket','Deacon','',1,2,4.2,.16,.1,0,0,21,146,null from dual union all
select 673,'Kiva/Hopi','','research rocket','Kiva','',1,2,5.5,.17,.1,0,0,27,365,null from dual union all
select 674,'Kontakt','','research rocket','Kontakt','',1,3,10,.74,4.5,0,0,null,500,null from dual union all
select 675,'Kookaburra','','endoatmoshpheric test/research','Lupus','',1,2,3.5,.13,.05,0,0,9,75,null from dual union all
select 676,'Kookaburra 2','','endoatmoshpheric test/research','Lupus','',1,2,3.5,.13,.05,0,0,9,75,null from dual union all
select 677,'Kookaburra 3','','endoatmoshpheric test/research','Lupus','',1,2,3.5,.13,.05,0,0,9,75,null from dual union all
select 678,'Kosmos 11K63','','orbital launch vehicle','R-12','',1,2,31.5,1.65,48.1,500,0,730,2000,null from dual union all
select 679,'Kosmos 11K65','','orbital launch vehicle','R-14','',1,2,32.4,2.4,109,1500,0,1485,1500,null from dual union all
select 68,'Agni TD','','missile','ASLV','',1,2,21,1,19,0,0,503,300,1200 from dual union all
select 680,'Kosmos 11K65M','','orbital launch vehicle','R-14','',1,2,32.4,2.4,109,1500,0,1485,2000,null from dual union all
select 681,'Kosmos 63S1','','orbital launch vehicle','R-12','',1,2,31.5,1.65,48.1,500,0,730,2000,null from dual;
insert into LAUNCH_VEHICLE
select 682,'Kosmos 65S3','','orbital launch vehicle','R-14','',1,2,32.4,2.4,109,1500,0,1480,1500,null from dual union all
select 683,'Kuaizhou','','orbital launch vehicle','DF-21','',1,4,20,1.4,30,250,0,null,1000,8000 from dual union all
select 684,'Kuryer','','missile','Kuryer','',1,3,11.2,1.36,15,0,0,null,1000,1000 from dual union all
select 685,'Kwangmyongsong','','orbital launch vehicle','Unha','Unha-3',1,3,30,2.4,85,50,0,1000,2000,null from dual union all
select 686,'LCLV','','research rocket','Talos','',1,3,16.5,.76,6.9,0,0,null,400,null from dual union all
select 687,'LDSD','','endoatmospheric rocket','Star48','',0,1,2,1.22,3,0,0,null,60,null from dual union all
select 688,'LLV-1','','orbital launch vehicle','MX','Athena-1',1,3,18.9,2.3,66.3,820,0,1450,1000,null from dual union all
select 689,'LM AS','','deep space launch vehicle','LMAS','',1,1,2.5,3.76,4.9,2000,0,null,100,null from dual union all
select 69,'Al Abbas','','tactical ballistic missile','R-11','',1,1,13.2,.88,7.8,0,0,90,150,900 from dual union all
select 690,'LMLV-1','','orbital launch vehicle','MX','Athena-1',1,3,18.9,2.3,66.3,820,0,1450,1000,null from dual union all
select 691,'LRALT','','missile','SR19','',0,2,15.5,1.32,15,0,0,267,300,null from dual union all
select 692,'LS-A','','research rocket','LS-C','',1,2,7.5,.35,.8,0,0,60,150,null from dual union all
select 693,'LS-C','7','research rocket','LS-C','',1,2,10.3,.6,2.3,0,0,null,100,null from dual union all
select 694,'LS-C','','research rocket','LS-C','',1,2,10.3,.6,2.3,0,0,null,100,null from dual union all
select 695,'LV-2','','missile','Trident','',1,2,10.4,1.88,30,null,null,null,null,null from dual union all
select 696,'Lambda 2','','research rocket','Lambda','',0,2,15.5,.74,6.3,0,0,970,500,null from dual union all
select 697,'Lambda 3','','research rocket','Lambda','',0,3,17.4,.74,7,0,0,970,1000,null from dual union all
select 698,'Lambda 3H','','research rocket','Lambda','',0,3,16.5,.74,8.2,0,0,970,2000,null from dual union all
select 699,'Lambda 4S','','orbital launch vehicle','Lambda','',0,4,16.5,.74,9.4,26,0,970,5000,null from dual union all
select 7,'A-350Zh','','missile','A-350','',1,2,17.9,2.57,32,0,0,null,350,350 from dual union all
select 70,'Al Abid','','big test rocket','Abid','',1,3,25,2.64,48,0,0,650,100,2000 from dual union all
select 700,'Lambda 4SC','','big test rocket','Lambda','',0,4,16.5,.74,9.4,0,0,970,2000,null from dual union all
select 701,'Lambda 4T','','big test rocket','Lambda','',0,4,16.5,.74,9.4,0,0,970,2000,null from dual union all
select 702,'Lance','','missile','Lance','',1,1,6.2,.56,null,0,0,null,46,null from dual union all
select 703,'Lark','','cruise missile','Lark','',1,1,5.6,.46,.9,0,0,null,8,null from dual union all
select 704,'Layner','','missile','R-29','',1,3,14.8,1.9,40.3,0,0,682,1000,8300 from dual union all
select 705,'Leopard','','endoatmospheric research test vehicle','Raven','',1,2,6,.44,1.5,0,0,null,20,null from dual union all
select 706,'Lex','','research rocket','Lex','',1,1,3.4,.16,.08,0,0,10,130,null from dual union all
select 707,'Little Joe 1','2C','endoatmoshpheric test/research','LJ1','',0,1,15.2,2.03,12.7,0,0,650,60,null from dual union all
select 708,'Little Joe 1','2P','endoatmoshpheric test/research','LJ1','',0,1,15.2,2.03,12.7,0,0,650,60,null from dual union all
select 709,'Little Joe 1','4C','big test rocket','LJ1','',0,1,15.2,2.03,12.7,0,0,1044,90,null from dual union all
select 71,'Al Fatah','','research rocket','Fatah','',1,1,6.5,.5,1.6,0,0,null,50,null from dual union all
select 710,'Little Joe 1','4P','endoatmoshpheric test/research','LJ1','',0,1,15.2,2.03,12.7,0,0,650,60,null from dual union all
select 711,'Little Joe II','0-3-3','endoatmoshpheric test/research','LJ2','',1,1,26.2,3.96,80.3,0,0,3276,35,null from dual union all
select 712,'Little Joe II','4-2-0','endoatmoshpheric test/research','LJ2','',0,1,26.2,3.96,42.7,0,0,1716,10,null from dual union all
select 713,'Little Joe II','5-2-2','endoatmoshpheric test/research','LJ2','',0,1,26.2,3.96,63.3,0,0,2964,25,null from dual union all
select 714,'Little Joe II','6-1-0','endoatmoshpheric test/research','LJ2','',0,1,26.2,3.96,26.2,0,0,1482,10,null from dual union all
select 715,'Loki','','endoatmospheric weather rocket <80 km','Loki','',1,1,2.6,.08,0,0,0,null,75,null from dual union all
select 716,'Loki Dart','','endoatmospheric weather rocket <80 km','Loki','',1,1,2.6,.08,0,0,0,null,75,null from dual union all
select 717,'Loki Dart PWN-8B','','endoatmospheric weather rocket <80 km','Loki','',1,1,2.6,.08,0,0,0,null,75,null from dual union all
select 718,'Loki Rockoon','','research rocket','Loki','',0,1,2.4,.08,0,0,0,10,100,null from dual union all
select 719,'Long Tom','1','research rocket','LongTom','',1,2,8.3,.47,.9,0,0,190,120,null from dual union all
select 72,'Al Hijara','','tactical ballistic missile','R-11','',1,1,11.9,.88,6.3,0,0,90,150,600 from dual union all
select 720,'Long Tom','','research rocket','LongTom','',1,2,8.3,.47,.9,0,0,190,120,null from dual union all
select 721,'Lorikeet','','research rocket','Demon','',1,2,5,.18,.1,0,0,null,100,null from dual union all
select 722,'Lorikeet Mk 2','','research rocket','Demon','',1,2,5,.18,.1,0,0,null,100,null from dual union all
select 723,'Luna-M','','missile','Luna','',1,1,9.4,.54,2.5,0,0,null,70,null from dual union all
select 724,'Lynx','','endoatmospheric rocket','Standard','',1,1,5,.35,1.3,0,0,null,30,null from dual union all
select 725,'M-100','','exoatmospheric weather rocket','M-100','',1,2,8.3,.25,.5,0,0,null,90,null from dual union all
select 726,'M-100 (A-1)','','exoatmospheric weather rocket','M-100','',1,2,8.3,.25,.5,0,0,null,90,null from dual union all
select 727,'M-100A','','exoatmospheric weather rocket','M-100','',1,2,8.3,.25,.5,0,0,null,90,null from dual union all
select 728,'M-100B','','exoatmospheric weather rocket','M-100','',1,2,8.3,.25,.5,0,0,null,90,null from dual union all
select 729,'M-600','','missile','Zelzal','',1,1,9,.61,1,0,0,null,60,241 from dual union all
select 73,'Al Hussein','','tactical ballistic missile','R-11','',1,1,12.2,.88,6.3,0,0,90,150,600 from dual union all
select 730,'M-V','1','orbital launch vehicle','MV','',1,3,30.7,2.5,139,1800,0,3786,1000,null from dual union all
select 731,'M-V','','orbital launch vehicle','MV','',1,3,30.7,2.5,139,1800,0,3786,1000,null from dual union all
select 732,'M56A1','','research rocket','Aries','',1,1,9.2,1.16,6.3,0,0,200,500,null from dual union all
select 733,'MCR-3/RH-125','','endoatmospheric weather rocket <80 km','RH-125','',0,1,3.5,.12,.1,0,0,8,30,null from dual union all
select 734,'MD-01','','endoatmospheric weather rocket <80 km','Meteo','',1,1,3.4,.15,.06,0,0,null,20,null from dual union all
select 735,'MD-620','2','tactical ballistic missile','Agate','',1,2,13.4,.8,6.5,0,0,null,100,500 from dual union all
select 736,'MD-620','','tactical ballistic missile','Agate','',1,1,13.4,.8,6.5,0,0,null,100,500 from dual union all
select 737,'MERA','','exoatmospheric weather rocket','Mera','',1,2,3.2,.15,.1,0,0,null,85,null from dual union all
select 738,'MMR-06','','endoatmospheric weather rocket <80 km','MMR-06','',1,1,3.5,.2,.1,0,0,8,60,null from dual union all
select 739,'MN-300','','research rocket','MR-12','MR-30',1,1,8,.45,1.6,0,0,100,300,null from dual union all
select 74,'Al Kahir','','endoatmospheric rocket','AlKahir','',1,1,12.5,1.22,10,0,0,null,60,null from dual union all
select 740,'MR-12','','research rocket','MR-12','',1,1,8.8,.45,1.4,0,0,100,150,null from dual union all
select 741,'MR-20','','research rocket','MR-12','',1,1,9,.45,1.6,0,0,100,240,null from dual union all
select 742,'MR-25','','research rocket','MR-12','',1,1,9,.45,1.6,0,0,100,240,null from dual union all
select 743,'MR-30','','research rocket','MR-12','',1,1,8,.45,1.6,0,0,100,300,null from dual union all
select 744,'MR-UR-100','','missile','MR-UR-100','',1,3,20.9,2.25,71,0,0,1236,1000,10000 from dual union all
select 745,'MR-UR-100U 15A16','','missile','MR-UR-100','',1,3,21.6,2.25,71.1,0,0,1236,1000,10000 from dual union all
select 746,'MRBM Target','','missile','SR19','',1,2,15.5,1.32,15,0,0,267,300,null from dual union all
select 747,'MRBM-T3','','missile','SR19','',1,2,15,1.32,15,0,0,267,300,null from dual union all
select 748,'MRT','','research rocket','Castor4','',1,1,14,1.02,13,0,0,500,500,null from dual union all
select 749,'MSBS M011','','endoatmoshpheric test/research','P-10','',1,2,10,1.5,15,0,0,440,50,null from dual union all
select 75,'Al Samoud','','research rocket','V-750','',1,1,7.7,.5,1.5,0,0,null,50,null from dual union all
select 750,'MSBS M012','','missile','P-10','',1,2,10,1.5,15,0,0,440,500,null from dual union all
select 751,'MSBS M013','','missile','P-10','',1,2,10,1.5,15,0,0,440,500,null from dual union all
select 752,'MSBS M1','','missile','P-10','',1,2,10.4,1.5,18,0,0,440,500,null from dual union all
select 753,'MSBS M112','','endoatmoshpheric test/research','P-10','',1,2,10.4,1.5,18,0,0,440,50,null from dual union all
select 754,'MSBS M2','','missile','P-10','',1,2,10.7,1.5,20,0,0,440,600,null from dual union all
select 755,'MSBS M20','','missile','P-10','',1,2,10.7,1.5,20,0,0,440,600,3000 from dual union all
select 756,'MSBS M4','','missile','MSBS','',1,3,11,1.93,35,0,0,900,800,null from dual union all
select 757,'MSBS M45','','missile','MSBS','',1,3,11,1.93,35,0,0,900,800,null from dual union all
select 758,'MSBS M51','','missile','M51','',1,3,12,2.3,56,0,0,null,1000,10000 from dual union all
select 759,'MT-135','','endoatmospheric weather rocket <80 km','MT-135','',1,1,3.3,.14,0,0,0,12,50,null from dual union all
select 76,'Al Samoud 2','','research rocket','Samoud','',1,1,7.2,.76,2,0,0,null,50,null from dual union all
select 760,'MT-135JA','','endoatmospheric weather rocket <80 km','MT-135','',1,1,3.3,.14,0,0,0,12,50,null from dual union all
select 761,'MT-135P','','endoatmospheric weather rocket <80 km','MT-135','',1,1,3.3,.14,0,0,0,12,50,null from dual union all
select 762,'MX-2227','','research rocket','AeroHTV','',1,2,3.9,.23,.1,0,0,null,10,null from dual union all
select 763,'MX-775','','cruise missile','Snark','',1,1,15.8,1.4,12.7,0,0,null,10,null from dual union all
select 764,'Mace','','cruise missile','Matador','',1,1,14.3,1.4,8.5,0,0,null,12,null from dual union all
select 765,'Malemute','','research rocket','Malemute','',1,1,2.4,.41,.8,0,0,57,165,null from dual union all
select 766,'Martlet 1','','research rocket','Gun16','',1,1,1.7,.41,.2,0,0,null,150,null from dual union all
select 767,'Martlet 2','','research rocket','Gun16','',1,1,1.7,.41,.2,0,0,null,150,null from dual union all
select 768,'Martlet 2C','','research rocket','Gun16','',1,1,1.7,.41,.2,0,0,null,150,null from dual union all
select 769,'Martlet 3A','','research rocket','Gun16','',1,1,1.7,.41,.2,0,0,null,150,null from dual union all
select 77,'Alfa','','missile','Alfa','',1,1,6.5,1.37,8,0,0,245,300,1000 from dual union all
select 770,'Martlet 3B','','research rocket','Gun16','',1,1,1.7,.41,.2,0,0,null,150,null from dual union all
select 771,'Matador','','cruise missile','Matador','',1,1,12.1,1.4,5.2,0,0,null,12,null from dual;
insert into LAUNCH_VEHICLE
select 772,'Maxus','','research rocket','Castor4','',1,1,15.5,1.02,12.3,0,0,500,850,null from dual union all
select 773,'Menaka II','','endoatmospheric weather rocket <80 km','RH-125','',1,2,3.8,.12,.1,0,0,8,60,null from dual union all
select 774,'Mesquito','','research rocket','MLRS','',1,1,5,.23,.2,0,0,null,60,null from dual union all
select 775,'Meteo','','endoatmospheric weather rocket <80 km','Meteo','',1,1,3.4,.15,0,0,0,null,50,null from dual union all
select 776,'Meteor-1','','endoatmospheric weather rocket <80 km','Meteor','',1,1,2.5,.12,0,0,0,14,40,null from dual union all
select 777,'Meteor-2H','','endoatmospheric weather rocket <80 km','Meteor2','',1,1,4.5,.35,.4,0,0,24,68,null from dual union all
select 778,'Meteor-2K','','research rocket','Meteor2','',0,1,4.3,.35,.4,0,0,52,90,null from dual union all
select 779,'Meteor-3','','endoatmospheric weather rocket <80 km','Meteor','',1,2,4.3,.12,0,0,0,14,65,null from dual union all
select 78,'AltAir','','research rocket','SR19','',1,1,6,1.32,8,0,0,267,100,null from dual union all
select 780,'Meteor-3B','','endoatmospheric weather rocket <80 km','Meteor','',1,2,4.3,.12,0,0,0,14,65,null from dual union all
select 781,'Metroc','','endoatmospheric weather rocket <80 km','Loki','',1,1,1.4,.08,0,0,0,1,30,null from dual union all
select 782,'Microstar','','endoatmospheric weather rocket <80 km','SuperLoki','',1,1,3.3,.1,0,0,0,null,75,null from dual union all
select 783,'Minotaur 1','','orbital launch vehicle','Minuteman','',1,4,19.2,1.68,36.3,340,0,892,1000,null from dual union all
select 784,'Minotaur 2','','missile','Minuteman','',1,3,17.6,1.68,33.1,0,0,935,1300,null from dual union all
select 785,'Minotaur C','','orbital launch vehicle','MX','',1,4,31.6,2.35,73,1380,0,1711,1000,null from dual union all
select 786,'Minotaur IV','2','orbital launch vehicle','MX','',1,5,23.9,2.35,36.2,1735,0,1600,500,500 from dual union all
select 787,'Minotaur IV','H','orbital launch vehicle','MX','',1,5,23.9,2.35,36.2,1735,0,1600,500,500 from dual union all
select 788,'Minotaur IV','','orbital launch vehicle','MX','',1,4,23.9,2.35,36.2,1735,0,1600,500,500 from dual union all
select 789,'Minotaur IV Lite','','missile','MX','',1,3,23,2.35,36.2,1735,0,1600,500,null from dual union all
select 79,'Angara A5','','orbital launch vehicle','Angara','',0,3,42.8,2.9,773,24500,5400,2095,200,null from dual union all
select 790,'Minotaur IV+','','orbital launch vehicle','MX','',1,4,23.9,2.35,36.2,1735,0,1600,500,500 from dual union all
select 791,'Minotaur V','','orbital launch vehicle','MX','',1,5,23.9,2.35,36.2,1735,0,1600,500,500 from dual union all
select 792,'Minuteman 1','','endoatmoshpheric test/research','Minuteman','',1,3,16,1.68,30,0,0,935,50,null from dual union all
select 793,'Minuteman 1A','T','endoatmoshpheric test/research','Minuteman','',1,1,16.4,1.68,30,0,0,935,10,null from dual union all
select 794,'Minuteman 1A','','missile','Minuteman','',1,3,16.4,1.68,31.3,0,0,935,1300,null from dual union all
select 795,'Minuteman 1B','','missile','Minuteman','',1,3,16.8,1.68,31.7,0,0,935,1300,null from dual union all
select 796,'Minuteman 2','','missile','Minuteman','',1,3,17.6,1.68,33.1,0,0,935,1300,null from dual union all
select 797,'Minuteman 2 PLV','','missile','SR19','',1,2,10.3,1.32,15,0,0,267,225,null from dual union all
select 798,'Minuteman 3','PRP','missile','Minuteman','',1,4,18.3,1.68,35.3,0,0,935,1600,null from dual union all
select 799,'Minuteman 3','','missile','Minuteman','',1,4,18.3,1.68,35.3,0,0,935,1600,null from dual union all
select 8,'A-4','','tactical ballistic missile','V-2','',1,1,13.6,1.65,12.8,0,0,270,85,null from dual union all
select 80,'Angara-1.2PP','','orbital launch vehicle','Angara','',1,2,42.8,2.9,171,0,0,2095,200,null from dual union all
select 800,'Molniya 8K78','E6','orbital launch vehicle','R-7','',1,5,44.4,2.95,306.2,null,0,4054,400000,null from dual union all
select 801,'Molniya 8K78','M','orbital launch vehicle','R-7','',1,5,44.4,2.95,306.2,null,0,4054,400000,null from dual union all
select 802,'Molniya 8K78','','orbital launch vehicle','R-7','',1,5,44,2.95,305.5,null,0,4020,400000,null from dual union all
select 803,'Molniya 8K78M','2BL','orbital launch vehicle','R-7','',1,5,43.4,2.95,305,0,2000,4054,40000,null from dual union all
select 804,'Molniya 8K78M','M1M','orbital launch vehicle','R-7','',1,5,42.8,2.95,305.5,null,0,4054,400000,null from dual union all
select 805,'Molniya 8K78M','ML','orbital launch vehicle','R-7','',1,5,43.4,2.95,305,0,2000,4054,40000,null from dual union all
select 806,'Molniya 8K78M','NVL','orbital launch vehicle','R-7','',1,5,43.4,2.95,305,0,2000,4054,40000,null from dual union all
select 807,'Molniya 8K78M','SOL','orbital launch vehicle','R-7','',1,5,43.4,2.95,305,0,2000,4054,400000,null from dual union all
select 808,'Molniya 8K78M','VL','orbital launch vehicle','R-7','',1,5,43.4,2.95,305,0,2000,4054,40000,null from dual union all
select 809,'Molniya 8K78M','','orbital launch vehicle','R-7','',1,5,42.8,2.95,305.5,null,0,4054,400000,null from dual union all
select 81,'Antares','','research rocket','Stromboli','OPd-56-39-22D',1,4,12.2,.56,1.8,0,0,200,280,null from dual union all
select 810,'Molniya 8K78M-PVB','2BL','orbital launch vehicle','R-7','',1,5,43.4,2.95,305,0,2000,4054,40000,null from dual union all
select 811,'Molniya 8K78M-PVB','ML','orbital launch vehicle','R-7','',1,5,43.4,2.95,305,0,2000,4054,40000,null from dual union all
select 812,'Momo','','research rocket','Momo','',1,1,8.5,.5,.9,0,0,null,100,null from dual union all
select 813,'Monica I','','research rocket','Monica','',1,3,3.1,.16,.1,0,0,null,50,null from dual union all
select 814,'Monica IV','','research rocket','Monica','',1,3,5.1,.16,.1,0,0,null,100,null from dual union all
select 815,'Mu-1','','endoatmoshpheric test/research','Mu','',0,4,21.9,1.41,40,0,0,1932,50,null from dual union all
select 816,'Mu-3C','','orbital launch vehicle','Mu','',0,3,20.2,1.41,41.6,195,0,1932,3500,null from dual union all
select 817,'Mu-3D','','big test rocket','Mu','',0,4,23.6,1.41,40,0,0,1932,160,null from dual union all
select 818,'Mu-3H','','orbital launch vehicle','Mu','',0,4,23.8,1.41,44.7,300,0,2150,1000,null from dual union all
select 819,'Mu-3S','','orbital launch vehicle','Mu','',0,3,23.8,1.41,49.3,300,0,2150,1000,null from dual union all
select 82,'Antares 110','','orbital launch vehicle','Zenit','',1,2,40.5,3.9,282,4600,1800,3631,300,null from dual union all
select 820,'Mu-3S-II','','orbital launch vehicle','Mu','',0,3,27.8,1.41,61.7,800,0,1930,1400,null from dual union all
select 821,'Mu-4S','','orbital launch vehicle','Mu','',0,4,23.6,1.41,43.6,180,0,1932,6500,null from dual union all
select 822,'N-1','','orbital launch vehicle','Thor','',0,3,33,2.44,90,1200,360,2290,40000,null from dual union all
select 823,'N-1 11A52','','orbital launch vehicle','N-1','',1,3,105.3,14,2788,70000,0,45300,200,null from dual union all
select 824,'N-2','(2)','orbital launch vehicle','Thor','',0,2,35,2.44,133,null,null,2290,1000,null from dual union all
select 825,'N-2','','orbital launch vehicle','Thor','',0,3,35,2.44,135,2000,730,2290,40000,null from dual union all
select 826,'NAL-16','','endoatmoshpheric test/research','STA','',1,1,4,.16,.1,0,0,10,20,null from dual union all
select 827,'NAL-25','','endoatmoshpheric test/research','STA','',1,1,null,.25,.1,0,0,null,20,null from dual union all
select 828,'NAL-7','','endoatmoshpheric test/research','STA','',1,1,null,.07,0,0,0,null,20,null from dual union all
select 829,'NAL-735','','endoatmoshpheric test/research','SB-735','',1,1,10,.74,4.7,0,0,250,18,null from dual union all
select 83,'Antares 120','','orbital launch vehicle','Zenit','',1,2,40.5,3.9,282,4600,1800,3631,300,null from dual union all
select 830,'NATIV','','endoatmoshpheric test/research','RTV','',1,1,4.1,.46,.6,0,0,12,19,50 from dual union all
select 831,'Naro-1','','orbital launch vehicle','Angara','',1,2,33.5,2.9,140,100,0,2095,200,null from dual union all
select 832,'Navaho II SM-64','','cruise missile','Navaho','',1,2,26.6,null,54.6,0,0,null,25,null from dual union all
select 833,'Nazeat 6H','','research rocket','Kavoshgar','',1,1,6.3,.36,.9,0,0,null,150,null from dual union all
select 834,'New Shepard','','orbital launch vehicle','NewShepard','',1,1,15,3,75,0,0,1020,100,null from dual union all
select 835,'Nike','','endoatmospheric rocket','Nike','',1,1,6,.42,.7,0,0,217,23,null from dual union all
select 836,'Nike Ajax','','endoatmospheric rocket','Nike','',1,2,10.8,.42,1.1,0,0,217,6,null from dual union all
select 837,'Nike Apache','','research rocket','Nike','',1,2,8.6,.42,.8,0,0,217,200,null from dual union all
select 838,'Nike Asp','','research rocket','Nike','',1,2,7.9,.42,.7,0,0,217,220,null from dual union all
select 839,'Nike Cajun','','research rocket','Nike','',1,2,8.1,.42,.7,0,0,217,120,null from dual union all
select 84,'Antares 130','','orbital launch vehicle','Zenit','',1,2,40.5,3.9,282,5100,1800,3631,300,null from dual union all
select 840,'Nike Deacon','1','research rocket','Nike','',1,2,7.8,.42,null,0,0,265,50,null from dual union all
select 841,'Nike Deacon','','research rocket','Nike','',1,2,7.8,.42,null,0,0,217,100,null from dual union all
select 842,'Nike Hawk','','research rocket','Nike','',1,2,9,.42,1.1,0,0,217,160,null from dual union all
select 843,'Nike Hercules','','endoatmospheric rocket','Hercules','',1,2,11.9,1.06,4.9,0,0,775,45,null from dual union all
select 844,'Nike Hercules 1810','','endoatmospheric rocket','Hercules','',1,2,11.9,1.06,4.9,0,0,775,45,null from dual union all
select 845,'Nike Hercules R&D','','endoatmospheric rocket','Hercules','',1,2,11.9,1.06,4.9,0,0,775,45,null from dual union all
select 846,'Nike Hydac','','research rocket','Nike','',1,2,9.1,.42,.9,0,0,217,150,null from dual union all
select 847,'Nike Improved Orion','','research rocket','Nike','',1,2,9,.42,1.1,0,0,217,140,null from dual union all
select 848,'Nike Iroquois','','research rocket','Nike','',1,2,8,.42,.7,0,0,217,290,null from dual union all
select 849,'Nike J','','endoatmospheric rocket','Hercules','',1,2,11.9,1.06,4.9,0,0,775,45,null from dual union all
select 85,'Antares 230','','orbital launch vehicle','Zenit','',1,2,42.5,3.9,290,6250,1800,3844,300,null from dual union all
select 850,'Nike Javelin','','research rocket','Nike','',1,2,8.2,.42,.9,0,0,217,130,null from dual union all
select 851,'Nike Malemute','','research rocket','Nike','',1,2,8.6,.42,1,0,0,217,500,null from dual union all
select 852,'Nike Nike','','endoatmoshpheric test/research','Nike','',1,2,11.9,.42,1.4,0,0,217,50,null from dual union all
select 853,'Nike Nike Cajun','','research rocket','Nike','',1,3,8.1,.42,.7,0,0,217,120,null from dual union all
select 854,'Nike Nike Deacon','','endoatmospheric research test vehicle','Nike','',1,3,9,.42,null,0,0,217,10,null from dual union all
select 855,'Nike Nike HPAG','','endoatmospheric research test vehicle','Nike','',1,3,9,.42,null,0,0,217,10,null from dual union all
select 856,'Nike Nike Recruit','','research rocket','Nike','',1,3,9,.42,null,0,0,217,120,null from dual union all
select 857,'Nike Nike T40','','endoatmospheric research test vehicle','Nike','',1,3,9,.42,null,0,0,217,10,null from dual union all
select 858,'Nike Nike T40 T55','','research rocket','Nike','',1,4,9,.42,null,0,0,217,100,null from dual union all
select 859,'Nike Nike Tri-Deacon T40','','endoatmospheric research test vehicle','Nike','',1,4,9,.42,null,0,0,217,10,null from dual union all
select 86,'Apache','','research rocket','Deacon','',1,1,4,.17,.1,0,0,null,50,null from dual union all
select 860,'Nike Orion','','research rocket','Nike','',1,2,9,.42,1.1,0,0,217,140,null from dual union all
select 861,'Nike Recruit','','endoatmospheric research test vehicle','Nike','',1,2,8,.42,1.1,0,0,217,5,null from dual;
insert into LAUNCH_VEHICLE
select 862,'Nike T40 T55','','endoatmospheric research test vehicle','Nike','',1,3,9,.42,null,0,0,217,10,null from dual union all
select 863,'Nike T48 Ajax','','endoatmospheric rocket','T48','',1,2,8,.46,1.3,0,0,190,20,null from dual union all
select 864,'Nike Tomahawk','12','research rocket','Nike','',1,2,10.8,.42,1,0,0,217,270,null from dual union all
select 865,'Nike Tomahawk','9','research rocket','Nike','',1,2,10.8,.42,1,0,0,217,270,null from dual union all
select 866,'Nike Tomahawk','9a','research rocket','Nike','',1,2,10.8,.42,1,0,0,217,270,null from dual union all
select 867,'Nike Tomahawk','','research rocket','Nike','',1,2,10.8,.42,1,0,0,217,270,null from dual union all
select 868,'Nike Viper I','','research rocket','Nike','',1,2,8,.42,.6,0,0,217,80,null from dual union all
select 869,'Nike X201 Ajax','','endoatmospheric rocket','Nike','',1,2,10.8,.42,1.1,0,0,217,6,null from dual union all
select 87,'Apollo LES','','endoatmoshpheric test/research','Apollo.LES','',1,1,10.2,.66,2.5,0,0,690,2,null from dual union all
select 870,'Nike Yardbird','','research rocket','Nike','',1,2,8.1,.42,null,0,0,217,120,null from dual union all
select 871,'Nike Zeus','3','missile','Spartan','',1,3,14.6,1.09,10.3,0,0,2002,280,400 from dual union all
select 872,'Nike Zeus','','missile','Spartan','',1,2,14.7,1.09,10.3,0,0,2002,200,400 from dual union all
select 873,'Nike Zeus A','','endoatmospheric rocket','Spartan','',1,2,13.3,.91,5,0,0,2002,50,320 from dual union all
select 874,'Nike Zeus DM-15S','','missile','Spartan','',1,3,14.6,1.09,10.3,0,0,2002,280,400 from dual union all
select 875,'Nike-46','','endoatmospheric rocket','Nike-46','',1,2,6,.4,.7,0,0,400,10,null from dual union all
select 876,'Nike-47','','endoatmospheric rocket','Nike-46','',1,2,6,.4,.7,0,0,400,10,null from dual union all
select 877,'Nike-480','','endoatmospheric rocket','Nike','',1,2,6,.42,.7,0,0,217,20,null from dual union all
select 878,'Nike-481','','endoatmospheric rocket','Nike-46','',1,2,6,.42,.7,0,0,217,20,null from dual union all
select 879,'Nike-482','','endoatmospheric rocket','Nike','',1,2,6,.42,.7,0,0,217,20,null from dual union all
select 88,'Arcas','','exoatmospheric weather rocket','Arcas','',1,1,2.3,.11,0,0,0,1,90,null from dual union all
select 880,'Nike-484','','endoatmospheric rocket','Nike','',1,2,6,.42,.7,0,0,217,20,null from dual union all
select 881,'Nike-490','','endoatmospheric rocket','Nike','',1,2,6,.42,.7,0,0,217,20,null from dual union all
select 882,'Nudol''','','missile','A-350','',1,2,19,2.6,33,0,0,null,350,350 from dual union all
select 883,'OPd-56-39-22D','','research rocket','Stromboli','',1,4,12.2,.56,1.8,0,0,200,280,null from dual union all
select 884,'OS-X0','','tactical ballistic missile','DF-11','',1,1,8.4,.88,4.6,0,0,90,150,300 from dual union all
select 885,'OTRAG','','endoatmoshpheric test/research','OTRAG','',1,1,10.3,.76,null,0,0,null,20,null from dual union all
select 886,'Oka','','tactical ballistic missile','Oka','',1,1,7.5,.92,4.7,0,0,null,80,400 from dual union all
select 887,'Oriole II','','research rocket','Terrier','Terrier Oriole',1,2,11,.46,1,0,0,258,370,null from dual union all
select 888,'Oriole III','','research rocket','Talos','',1,3,14,.46,1,0,0,258,370,null from dual union all
select 889,'Oriole IIIA','','research rocket','Terrier','',1,3,14,.46,1,0,0,258,370,null from dual union all
select 89,'Arcon','','research rocket','Arcon','',0,1,4,.15,.1,0,0,14,100,null from dual union all
select 890,'Oriole IV','','research rocket','Talos','',1,4,19.8,.76,null,0,0,null,12,null from dual union all
select 891,'Orion','','research rocket','Orion','',1,1,5.6,.35,.4,0,0,7,85,null from dual union all
select 892,'Orion LAS','','missile','Orion.LAS','',1,1,11.6,1,6.1,0,0,null,1,null from dual union all
select 893,'Orion-1','','research rocket','IIAE.Orion','',1,1,4,.21,.1,0,0,null,95,null from dual union all
select 894,'Orion-2','','research rocket','IIAE.Orion','',1,1,4,.21,.1,0,0,null,95,null from dual union all
select 895,'PDV','','tactical ballistic missile','Prithvi','',1,1,9,1.1,4.5,0,0,null,100,350 from dual union all
select 896,'PDV Target','','tactical ballistic missile','Prithvi','',1,2,9,1.1,4.5,0,0,null,100,350 from dual union all
select 897,'PRS-1 5Ya26','','tactical ballistic missile','PRS-1','',1,1,10,2,12,0,0,null,40,80 from dual union all
select 898,'PRS-1M','','tactical ballistic missile','PRS-1','',1,1,10,2,12,0,0,null,40,80 from dual union all
select 899,'PSLV','C','orbital launch vehicle','PSLV','',0,4,44.4,2.8,294,1600,1000,5290,1000,null from dual union all
select 9,'A-4b','','tactical ballistic missile','V-2','',1,1,13.6,1.65,12.8,0,0,312,85,null from dual union all
select 90,'Ares I-X','','endoatmoshpheric test/research','SRB','',1,1,100,3.77,834,null,0,0,45,null from dual union all
select 900,'PSLV','CA','orbital launch vehicle','PSLV','',1,4,44.4,2.8,230,1300,1000,4430,1000,null from dual union all
select 901,'PSLV','','orbital launch vehicle','PSLV','',0,4,44.4,2.8,294,1600,800,5290,1000,null from dual union all
select 902,'PSLV-XL','','orbital launch vehicle','PSLV','',0,4,44.4,2.8,230,1800,1000,9230,1000,null from dual union all
select 903,'Paekkom','','endoatmospheric rocket','Hercules','',1,2,11.9,1.06,4.9,0,0,775,45,null from dual union all
select 904,'Paektusan 1','','orbital launch vehicle','Nodong','',1,3,25.8,1.25,33.4,50,0,570,2000,null from dual union all
select 905,'Paiute Apache','','research rocket','Genie','',1,2,4.8,.38,.6,0,0,140,200,null from dual union all
select 906,'Paiute Tomahawk','','research rocket','Genie','',1,2,5.8,.38,.6,0,0,140,350,null from dual union all
select 907,'Peacekeeper','','missile','MX','',1,4,21.6,2.3,88.5,0,0,1711,1000,null from dual union all
select 908,'Pedro Recruit','','endoatmoshpheric test/research','Pedro','',1,2,7.8,.71,null,0,0,244,20,null from dual union all
select 909,'Pegasus','','orbital launch vehicle','Pegasus','',0,3,15.5,1.28,18.5,350,0,580,4000,null from dual union all
select 91,'Ariane 1','','orbital launch vehicle','Ariane','',1,3,47.4,3.8,210,null,1400,2462,40000,null from dual union all
select 910,'Pegasus H','','orbital launch vehicle','Pegasus','',0,3,15.5,1.28,18.5,350,0,580,4000,null from dual union all
select 911,'Pegasus XL','','orbital launch vehicle','Pegasus','',0,3,17.4,1.28,23,440,0,726,4000,null from dual union all
select 912,'Pegasus XL/HAPS','','orbital launch vehicle','Pegasus','',0,4,17.4,1.28,23,350,0,726,4000,null from dual union all
select 913,'Pegasus/HAPS','','orbital launch vehicle','Pegasus','',0,4,15.5,1.28,18.5,250,0,580,4000,null from dual union all
select 914,'Perimetr 15A11','','missile','MR-UR-100','',1,2,21.6,2.25,71,0,0,1236,1000,null from dual union all
select 915,'Perimetr-RTs','','missile','Topol','',1,3,20.5,1.8,47.1,0,0,890,1000,1050 from dual union all
select 916,'Pershing 1','','missile','Pershing','',1,2,10.7,1.02,4.7,0,0,116,250,600 from dual union all
select 917,'Pershing 1A','','missile','Pershing','',1,2,10.7,1.02,4.7,0,0,116,250,600 from dual union all
select 918,'Pershing 2','1','missile','Pershing','',1,2,9,1.02,7.5,0,0,null,100,200 from dual union all
select 919,'Pershing 2','','missile','Pershing','',1,2,10.7,1.02,7.5,0,0,null,250,1800 from dual union all
select 92,'Ariane 2','','orbital launch vehicle','Ariane','',1,3,49,3.8,217,null,2175,2689,40000,null from dual union all
select 920,'Petrel','','research rocket','Petrel','',0,1,3.3,.19,.2,0,0,null,140,null from dual union all
select 921,'Petrel 2','','research rocket','Petrel','',0,1,3.7,.19,.2,0,0,null,175,null from dual union all
select 922,'Pioner','','missile','Temp-2S','',1,2,16.5,1.79,37,0,0,null,500,5000 from dual union all
select 923,'Pioner UTTKh','','missile','Temp-2S','',1,2,16.5,1.79,37,0,0,null,500,5000 from dual union all
select 924,'Pioner-3','','missile','Temp-2S','',1,2,16.5,1.79,37,0,0,null,500,5000 from dual union all
select 925,'Pogo Hi','','endoatmospheric research test vehicle','Deacon','',1,1,4,.17,.1,0,0,36,25,null from dual union all
select 926,'Pogo Hi II','','endoatmospheric research test vehicle','Deacon','',1,1,4,.17,.1,0,0,36,25,null from dual union all
select 927,'Pogo Hi III','','endoatmospheric research test vehicle','Asp','',1,1,4,.17,.1,0,0,36,25,null from dual union all
select 928,'Polaris A1','','missile','Polaris','',1,2,8.7,1.37,13,0,0,null,500,1850 from dual union all
select 929,'Polaris A2','','missile','Polaris','',1,2,9.4,1.37,14.7,0,0,null,1000,4600 from dual union all
select 93,'Ariane 3','','orbital launch vehicle','Ariane','',0,3,49,3.8,237,null,2700,4021,40000,null from dual union all
select 930,'Polaris A2E','','missile','Polaris','',1,2,9.4,1.37,14.7,0,0,null,1000,4600 from dual union all
select 931,'Polaris A3','A3T','missile','Polaris','',1,2,9.4,1.37,16.2,0,0,null,1000,4600 from dual union all
select 932,'Polaris A3','','missile','Polaris','',1,2,9.4,1.37,16.2,0,0,null,1000,4600 from dual union all
select 933,'Polaris AX','','missile','Polaris','',1,2,7.6,1.37,10,0,0,null,500,null from dual union all
select 934,'Polaris FTV-1','','research rocket','Sergeant','',1,1,10.1,.79,5,0,0,null,40,null from dual union all
select 935,'Polaris FTV-3','','rocket test vehicle','Sergeant','',1,3,12.3,.79,3.4,0,0,220,500,null from dual union all
select 936,'Polaris FTV-4','','research rocket','10KS2500','',1,1,2.9,.35,.1,0,0,11,10,null from dual union all
select 937,'Polaris FTV-5','','research rocket','Sergeant','',1,2,10.1,.79,5,0,0,null,40,null from dual union all
select 938,'Polaris TV','','endoatmoshpheric test/research','Polaris','',1,1,8,.79,5,0,0,220,30,null from dual union all
select 939,'Poseidon C3','','missile','Poseidon','',1,2,10.4,1.88,29.4,0,0,null,500,4600 from dual union all
select 94,'Ariane 40','40','orbital launch vehicle','Ariane','',1,3,58.4,3.8,243,2700,1900,2708,1000,null from dual union all
select 940,'Prahaar','','tactical ballistic missile','AAD','',1,1,7.5,.42,1.3,0,0,null,40,350 from dual union all
select 941,'Prithvi','','tactical ballistic missile','Prithvi','',1,1,9,1.1,4.5,0,0,null,100,350 from dual union all
select 942,'Prithvi 2','','tactical ballistic missile','Prithvi','',1,1,9,1.1,4.5,0,0,null,100,350 from dual union all
select 943,'Private A','','research rocket','PrivateA','',0,1,3.4,.25,.3,0,0,98,1,null from dual union all
select 944,'Private F','','research rocket','PrivateA','',0,1,3.4,.25,.3,0,0,98,1,null from dual union all
select 945,'Project Pilot','1','endoatmoshpheric test/research','HOTROC','',1,5,4.4,.76,.9,0,0,125,10,null from dual union all
select 946,'Project Pilot','','orbital launch vehicle','HOTROC','',0,5,4.4,.76,.9,1,0,125,5000,null from dual union all
select 947,'Proson M1','','research rocket','Proson','',1,2,3.1,.2,.1,0,0,null,50,null from dual union all
select 948,'Prospector','','research rocket','Castor4','',1,1,14.1,1.02,12.2,0,0,430,500,null from dual union all
select 949,'Proton-K','','orbital launch vehicle','Proton','',1,3,50,7.4,694,21000,0,9500,300,null from dual union all
select 95,'Ariane 40','40+','orbital launch vehicle','Ariane','',1,3,58.4,3.8,245,2740,null,2718,1000,null from dual union all
select 950,'Proton-K/17S40','DM-5','orbital launch vehicle','Proton','',1,6,57.2,7.4,691.5,5000,4350,9500,40000,null from dual union all
select 951,'Proton-K/17S40','DM2','orbital launch vehicle','Proton','',1,6,57.2,7.4,691.5,5000,4350,9500,40000,null from dual;
insert into LAUNCH_VEHICLE
select 952,'Proton-K/Briz-M','','orbital launch vehicle','Proton','',1,4,53,7.4,700,21000,4500,9500,40000,null from dual union all
select 953,'Proton-K/D','','orbital launch vehicle','Proton','',1,6,55.4,7.4,691.5,5000,4350,9500,400000,null from dual union all
select 954,'Proton-K/D-1','','orbital launch vehicle','Proton','',1,6,55.4,7.4,691.5,5000,4350,9500,400000,null from dual union all
select 955,'Proton-K/D-2','','orbital launch vehicle','Proton','',1,6,55.4,7.4,691.5,5000,4350,9500,400000,null from dual union all
select 956,'Proton-K/DM','','orbital launch vehicle','Proton','',1,6,57.2,7.4,691.5,5000,4350,9500,40000,null from dual union all
select 957,'Proton-K/DM-2','DM1','orbital launch vehicle','Proton','',1,6,57.2,7.4,691.5,5000,4350,9500,40000,null from dual union all
select 958,'Proton-K/DM-2','','orbital launch vehicle','Proton','',1,6,57.2,7.4,691.5,5000,4350,9500,40000,null from dual union all
select 959,'Proton-K/DM-2M','DM-2M','orbital launch vehicle','Proton','',1,6,57.2,7.4,691.5,5000,4350,9500,40000,null from dual union all
select 96,'Ariane 40','40-3','orbital launch vehicle','Ariane','',1,3,58.4,3.8,245,2740,null,2718,1000,null from dual union all
select 960,'Proton-K/DM-2M','DM3','orbital launch vehicle','Proton','',1,6,57.2,7.4,691.5,5000,4350,9500,40000,null from dual union all
select 961,'Proton-K/DM-2M','DM4','orbital launch vehicle','Proton','',1,6,57.2,7.4,691.5,5000,4350,9500,40000,null from dual union all
select 962,'Proton-M/Briz-M','','orbital launch vehicle','Proton','',1,4,53,7.4,712.8,21000,4500,10000,40000,null from dual union all
select 963,'Proton-M/DM-2','','orbital launch vehicle','Proton','',1,6,53,7.4,712.8,21000,4500,10000,40000,null from dual union all
select 964,'Proton-M/DM-3','','orbital launch vehicle','Proton','',1,6,53,7.4,712.8,21000,4500,10000,40000,null from dual union all
select 965,'Qahir-1','','research rocket','V-750','',0,1,11,.65,2.3,0,0,490,27,250 from dual union all
select 966,'Qahir-M2','','research rocket','V-750','',0,1,11,.65,2.3,0,0,490,27,250 from dual union all
select 967,'Qiam 1','','tactical ballistic missile','R-11','',1,1,10.6,.88,6.1,0,0,90,76,800 from dual union all
select 968,'Quad Deacon','','endoatmospheric research test vehicle','Deacon','',1,1,7,.15,.1,0,0,100,20,null from dual union all
select 969,'R-1','','tactical ballistic missile','V-2','',1,1,14.1,1.65,13.4,0,0,267,100,null from dual union all
select 97,'Ariane 42L','42L','orbital launch vehicle','Ariane','',0,3,58.4,3.8,363,null,2800,4208,40000,null from dual union all
select 970,'R-1 8A11','','tactical ballistic missile','V-2','R-1',1,1,14.1,1.65,13.4,0,0,267,100,null from dual union all
select 971,'R-11','','tactical ballistic missile','R-11','',1,1,7.4,.88,4.6,0,0,80,160,150 from dual union all
select 972,'R-11A','','tactical ballistic missile','R-11','',1,1,10.2,.88,4.6,0,0,80,200,null from dual union all
select 973,'R-11A-MV','','tactical ballistic missile','R-11','',1,1,10,.88,5,0,0,80,200,null from dual union all
select 974,'R-11FM','','tactical ballistic missile','R-11','',1,1,8.4,.88,4.6,0,0,93,160,150 from dual union all
select 975,'R-11M','','tactical ballistic missile','R-11','',1,1,7.4,.88,4.6,0,0,80,160,150 from dual union all
select 976,'R-12','','missile','R-12','',1,1,22.5,1.65,42,0,0,730,402,null from dual union all
select 977,'R-12U','','missile','R-12','',1,1,22.5,1.65,42,0,0,730,402,null from dual union all
select 978,'R-13','','tactical ballistic missile','R-13','',1,1,11.8,1.3,13.7,0,0,null,150,600 from dual union all
select 979,'R-14','','missile','R-14','',1,1,24.4,2.4,86.3,0,0,1480,676,400 from dual union all
select 98,'Ariane 42L','42L+','orbital launch vehicle','Ariane','',0,3,58.4,3.8,363,null,3350,4228,40000,null from dual union all
select 980,'R-14U','','missile','R-14','',1,1,24.4,2.4,86.4,0,0,1480,676,400 from dual union all
select 981,'R-16','','missile','R-16','',1,2,34.3,3,140.6,0,0,2610,1000,110 from dual union all
select 982,'R-16U','','missile','R-16','',1,2,34.3,3,140.6,0,0,2610,1000,130 from dual union all
select 983,'R-17','','tactical ballistic missile','R-11','',1,1,9,.88,5.4,0,0,93,150,null from dual union all
select 984,'R-17VTO','','tactical ballistic missile','R-11','',1,1,9,.88,5.4,0,0,93,150,null from dual union all
select 985,'R-1A','','tactical ballistic missile','V-2','R-1',1,1,14.1,1.65,13.4,0,0,267,100,null from dual union all
select 986,'R-1B','','tactical ballistic missile','V-2','R-1',1,1,17.5,1.65,13.4,0,0,267,100,null from dual union all
select 987,'R-1D','','tactical ballistic missile','V-2','R-1',1,1,14.1,1.65,13.4,0,0,267,100,null from dual union all
select 988,'R-1E','','tactical ballistic missile','V-2','',1,1,14.1,1.65,13.4,0,0,267,100,null from dual union all
select 989,'R-1E (A-1)','','tactical ballistic missile','V-2','',1,1,14.1,1.65,13.4,0,0,267,100,null from dual union all
select 99,'Ariane 42L','42L-3','orbital launch vehicle','Ariane','',0,3,58.4,3.8,363,null,3480,4228,40000,null from dual union all
select 990,'R-1UK','','tactical ballistic missile','V-2','',1,1,14.1,1.65,13.4,0,0,267,100,null from dual union all
select 991,'R-1V','','tactical ballistic missile','V-2','R-1',1,1,17.5,1.65,13.4,0,0,267,100,null from dual union all
select 992,'R-2','','research rocket','V-2','',1,1,17,1.65,20.3,0,0,363,150,550 from dual union all
select 993,'R-21','','missile','R-21','',1,1,13,1.3,16.6,0,0,null,300,1400 from dual union all
select 994,'R-2A','','research rocket','V-2','',1,1,17,1.65,20.3,0,0,363,212,null from dual union all
select 995,'R-2R','','research rocket','V-2','',1,1,17,1.65,20.3,0,0,363,200,null from dual union all
select 996,'R-2e','','research rocket','V-2','',1,1,17,1.65,20.3,0,0,363,200,null from dual union all
select 997,'R-31','','missile','R-31','',1,3,11.4,1.54,26.9,0,0,580,500,4200 from dual union all
select 998,'R-36','','missile','R-36','',1,2,34.1,3,183.9,0,0,2360,1000,1100 from dual union all
select 999,'R-36 8K67','','missile','R-36','',1,2,34.1,3,183.9,0,0,2360,1000,1100 from dual;


--------------------------------------------------------------------------------
-- LAUNCH_VEHICLE_MANUFACTURER
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing LAUNCH_VEHICLE_MANUFACTURER...');

CREATE TABLE "LAUNCH_VEHICLE_MANUFACTURER" 
 (	"LV_ID" NUMBER, 
	"LV_MANUFACTURER_ORG_CODE" VARCHAR2(4000), 
	 CONSTRAINT "LAUNCH_VEHICLE_MAN_ORG_PK" PRIMARY KEY ("LV_ID", "LV_MANUFACTURER_ORG_CODE")
USING INDEX  ENABLE, 
	 CONSTRAINT "LAUNCH_VEHICLE_MAN_ORG_FK1" FOREIGN KEY ("LV_MANUFACTURER_ORG_CODE")
	  REFERENCES "ORGANIZATION" ("ORG_CODE") ENABLE, 
	 CONSTRAINT "LAUNCH_VEHICLE_MAN_ORG_FK2" FOREIGN KEY ("LV_ID")
	  REFERENCES "LAUNCH_VEHICLE" ("LV_ID") ENABLE
 ) ;


CREATE INDEX "LAUNCH_VEHICLE_MANUFACT_IDX1" ON "LAUNCH_VEHICLE_MANUFACTURER" ("LV_MANUFACTURER_ORG_CODE") 
;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 1,'SKB30' from dual union all
select 10,'FAKEL' from dual union all
select 100,'AELM' from dual union all
select 1000,'OKB586' from dual union all
select 1001,'OKB586' from dual union all
select 1002,'OKB586' from dual union all
select 1003,'OKB586' from dual union all
select 1004,'YUZH' from dual union all
select 1005,'YUZH' from dual union all
select 1006,'YUZH' from dual union all
select 1007,'OKB586' from dual union all
select 1008,'OKB586' from dual union all
select 1009,'OKB1' from dual union all
select 101,'AELM' from dual union all
select 1010,'OKB1' from dual union all
select 1011,'OKB1' from dual union all
select 1012,'OKB1' from dual union all
select 1013,'OKB1' from dual union all
select 1014,'OKB1' from dual union all
select 1015,'OKB1' from dual union all
select 1016,'OKB1' from dual union all
select 1017,'OKB1' from dual union all
select 1018,'OKB1' from dual union all
select 102,'AELM' from dual union all
select 1020,'UNK' from dual union all
select 1021,'UNK' from dual union all
select 1022,'VSSC' from dual union all
select 1023,'VSSC' from dual union all
select 1024,'VSSC' from dual union all
select 1025,'VSSC' from dual union all
select 1026,'VSSC' from dual union all
select 1027,'VSSC' from dual union all
select 1028,'VSSC' from dual union all
select 1029,'VSSC' from dual union all
select 103,'AELM' from dual union all
select 1030,'VSSC' from dual union all
select 1031,'VSSC' from dual union all
select 1032,'VSSC' from dual union all
select 1033,'HOUW' from dual union all
select 1034,'HOUW' from dual union all
select 1035,'OKB1' from dual union all
select 1036,'ARSL' from dual union all
select 1037,'OKB1' from dual union all
select 1038,'YUZH' from dual union all
select 1039,'YUZH' from dual union all
select 104,'AELM' from dual union all
select 1040,'YUZH' from dual union all
select 1041,'YUZH' from dual union all
select 1042,'YUZH' from dual union all
select 1043,'MITT' from dual union all
select 1044,'ARSL' from dual union all
select 1045,'CONV' from dual union all
select 1046,'GEMTS' from dual union all
select 1046,'JPL' from dual union all
select 1047,'LAPAN' from dual union all
select 1048,'LAPAN' from dual union all
select 1049,'LAPAN' from dual union all
select 105,'AELM' from dual union all
select 1050,'RPI' from dual union all
select 1051,'PARD' from dual union all
select 1052,'PARD' from dual union all
select 1053,'ABMA' from dual union all
select 1053,'CHRY' from dual union all
select 1054,'ABMA' from dual union all
select 1054,'CHRY' from dual union all
select 1055,'VOUA' from dual union all
select 1056,'VOUA' from dual union all
select 1057,'MAK' from dual union all
select 1058,'IIAE' from dual union all
select 1059,'UMD' from dual union all
select 106,'AELM' from dual union all
select 1060,'NRL' from dual union all
select 1061,'SDCO' from dual union all
select 1062,'SDCO' from dual union all
select 1063,'KHRU' from dual union all
select 1064,'KHRU' from dual union all
select 1065,'RAE' from dual union all
select 1066,'RAE' from dual union all
select 1067,'MITT' from dual union all
select 1067,'VOTK' from dual union all
select 1068,'SEREB' from dual union all
select 1069,'NISST' from dual union all
select 107,'AELM' from dual union all
select 1070,'NISST' from dual union all
select 1071,'ISAS' from dual union all
select 1072,'ISAS' from dual union all
select 1073,'ISAS' from dual union all
select 1074,'NISST' from dual union all
select 1075,'MHITO' from dual union all
select 1076,'MHITO' from dual union all
select 1077,'MHITO' from dual union all
select 1078,'STA' from dual union all
select 1079,'EXOSA' from dual union all
select 108,'AELM' from dual union all
select 1080,'SUD' from dual union all
select 1081,'VSSC' from dual union all
select 1082,'MMD' from dual union all
select 1083,'ISRO' from dual union all
select 1084,'ISRO' from dual union all
select 1085,'GDPO' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 1086,'GDPO' from dual union all
select 1087,'USN' from dual union all
select 1088,'USN' from dual union all
select 1089,'USN' from dual union all
select 109,'AELM' from dual union all
select 1090,'USN' from dual union all
select 1091,'USN' from dual union all
select 1092,'USN' from dual union all
select 1093,'USN' from dual union all
select 1094,'VSSC' from dual union all
select 1095,'CHRY' from dual union all
select 1095,'TRW' from dual union all
select 1096,'COLE' from dual union all
select 1097,'OSCC' from dual union all
select 1098,'COLE' from dual union all
select 1099,'BOUT' from dual union all
select 11,'HAL' from dual union all
select 110,'AELM' from dual union all
select 1100,'NISST' from dual union all
select 1101,'NISST' from dual union all
select 1102,'SEREB' from dual union all
select 1103,'SEREB' from dual union all
select 1104,'SEREB' from dual union all
select 1105,'SEREB' from dual union all
select 1106,'AELM' from dual union all
select 1107,'STA' from dual union all
select 1108,'ISAS' from dual union all
select 1109,'ISAS' from dual union all
select 111,'EADSL' from dual union all
select 1110,'UNK' from dual union all
select 1111,'LMSC' from dual union all
select 1111,'SAND' from dual union all
select 1112,'ARMAD' from dual union all
select 1113,'ARMAD' from dual union all
select 1114,'IR' from dual union all
select 1115,'SAND' from dual union all
select 1116,'SAND' from dual union all
select 1117,'SEREB' from dual union all
select 1118,'MSFC' from dual union all
select 1119,'MSFC' from dual union all
select 112,'EADSL' from dual union all
select 1120,'MSFC' from dual union all
select 1121,'MSFC' from dual union all
select 1122,'MSFC' from dual union all
select 1123,'MSFC' from dual union all
select 1124,'UNK' from dual union all
select 1125,'LTV' from dual union all
select 1126,'LTV' from dual union all
select 1127,'LTV' from dual union all
select 1128,'LTV' from dual union all
select 1129,'LTV' from dual union all
select 113,'EADSL' from dual union all
select 1130,'LTV' from dual union all
select 1131,'LTV' from dual union all
select 1132,'LTV' from dual union all
select 1133,'LTV' from dual union all
select 1134,'LTV' from dual union all
select 1135,'LTV' from dual union all
select 1136,'BPD' from dual union all
select 1137,'LTV' from dual union all
select 1138,'LTV' from dual union all
select 1139,'LTV' from dual union all
select 114,'EADSL' from dual union all
select 1140,'LTV' from dual union all
select 1141,'LTV' from dual union all
select 1142,'LTV' from dual union all
select 1143,'LTV' from dual union all
select 1144,'LTV' from dual union all
select 1145,'LTV' from dual union all
select 1146,'LTV' from dual union all
select 1147,'LTV' from dual union all
select 1148,'LTV' from dual union all
select 1149,'LTV' from dual union all
select 115,'EADSL' from dual union all
select 1150,'LTV' from dual union all
select 1151,'VOTK' from dual union all
select 1152,'MAK' from dual union all
select 1153,'KPA' from dual union all
select 1154,'AJ' from dual union all
select 1156,'IR' from dual union all
select 1157,'IR' from dual union all
select 1158,'DRG' from dual union all
select 1159,'DRG' from dual union all
select 116,'EADSL' from dual union all
select 1160,'DRG' from dual union all
select 1161,'DNA' from dual union all
select 1162,'JPL' from dual union all
select 1163,'PARD' from dual union all
select 1164,'SDCO' from dual union all
select 1165,'LRRC' from dual union all
select 1166,'MAK' from dual union all
select 1167,'MAK' from dual union all
select 1168,'IR' from dual union all
select 1169,'PKNDC' from dual union all
select 117,'SVC' from dual union all
select 1170,'PKNDC' from dual union all
select 1171,'PKNDC' from dual union all
select 1172,'PKNDC' from dual union all
select 1173,'IAI' from dual union all
select 1174,'IAI' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 1175,'SANJ' from dual union all
select 1176,'IQ' from dual union all
select 1177,'PARD' from dual union all
select 1178,'DRDO' from dual union all
select 1179,'MAK' from dual union all
select 118,'IAI' from dual union all
select 1180,'MAK' from dual union all
select 1181,'ATLR' from dual union all
select 1182,'RPI' from dual union all
select 1183,'IAI' from dual union all
select 1184,'IR' from dual union all
select 1185,'MAK' from dual union all
select 1186,'SDCO' from dual union all
select 1187,'MITT' from dual union all
select 1187,'VOTK' from dual union all
select 1188,'BAJUK' from dual union all
select 1189,'BAJUK' from dual union all
select 119,'IAI' from dual union all
select 1190,'BAJUK' from dual union all
select 1191,'BAJUK' from dual union all
select 1192,'DAC' from dual union all
select 1193,'RAE' from dual union all
select 1194,'RAE' from dual union all
select 1195,'RAE' from dual union all
select 1196,'RAE' from dual union all
select 1197,'RAE' from dual union all
select 1198,'RAE' from dual union all
select 1199,'RAE' from dual union all
select 12,'WRE' from dual union all
select 120,'IAI' from dual union all
select 1200,'BAC' from dual union all
select 1201,'RAE' from dual union all
select 1202,'BAC' from dual union all
select 1203,'BAC' from dual union all
select 1204,'BAC' from dual union all
select 1205,'BAC' from dual union all
select 1206,'BAC' from dual union all
select 1207,'BAC' from dual union all
select 1208,'BAC' from dual union all
select 1209,'BAC' from dual union all
select 121,'IR' from dual union all
select 1210,'BAC' from dual union all
select 1211,'BAC' from dual union all
select 1212,'BAC' from dual union all
select 1213,'BAC' from dual union all
select 1214,'NORTH' from dual union all
select 1215,'IAE' from dual union all
select 1216,'IAE' from dual union all
select 1217,'IAE' from dual union all
select 1218,'IAE' from dual union all
select 1219,'IAE' from dual union all
select 122,'COOP' from dual union all
select 1220,'INPE' from dual union all
select 1221,'NSPO' from dual union all
select 1222,'OKB1' from dual union all
select 1223,'OKB1' from dual union all
select 1223,'PROG' from dual union all
select 1224,'OKB1' from dual union all
select 1224,'PROG' from dual union all
select 1225,'OKB1' from dual union all
select 1225,'PROG' from dual union all
select 1226,'PROG' from dual union all
select 1227,'PROG' from dual union all
select 1228,'PROG' from dual union all
select 1229,'PROG' from dual union all
select 123,'COOP' from dual union all
select 1230,'PROG' from dual union all
select 1231,'PROG' from dual union all
select 1232,'PROG' from dual union all
select 1233,'PROG' from dual union all
select 1234,'PROG' from dual union all
select 1235,'PROG' from dual union all
select 1236,'PROG' from dual union all
select 1237,'PROG' from dual union all
select 1238,'PROG' from dual union all
select 1239,'PROG' from dual union all
select 124,'COOP' from dual union all
select 1240,'PROG' from dual union all
select 1241,'PROG' from dual union all
select 1242,'PROG' from dual union all
select 1243,'RWI' from dual union all
select 1244,'UPAE' from dual union all
select 1245,'SCAL' from dual union all
select 1246,'SCAL' from dual union all
select 1247,'NMC' from dual union all
select 1248,'NMC' from dual union all
select 1249,'NMC' from dual union all
select 125,'COOP' from dual union all
select 1250,'ATLR' from dual union all
select 1251,'MDAC' from dual union all
select 1251,'WELEC' from dual union all
select 1252,'MDAC' from dual union all
select 1253,'OKB1' from dual union all
select 1254,'OKB1' from dual union all
select 1255,'OKB1' from dual union all
select 1256,'MDAC' from dual union all
select 1257,'SDCO' from dual union all
select 1258,'SDCO' from dual union all
select 1259,'MITT' from dual union all
select 126,'COOP' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 1260,'MITT' from dual union all
select 1261,'MITT' from dual union all
select 1262,'OSCC' from dual union all
select 1263,'OSCC' from dual union all
select 1264,'DARE' from dual union all
select 1265,'NPOMA' from dual union all
select 1266,'UMI' from dual union all
select 1267,'UMI' from dual union all
select 1268,'SAND' from dual union all
select 1269,'SAND' from dual union all
select 127,'ASTRV' from dual union all
select 1270,'SAND' from dual union all
select 1271,'SAND' from dual union all
select 1272,'SAND' from dual union all
select 1273,'SAND' from dual union all
select 1274,'SAND' from dual union all
select 1275,'SAND' from dual union all
select 1276,'SAND' from dual union all
select 1277,'SAND' from dual union all
select 1278,'SAND' from dual union all
select 1279,'SAND' from dual union all
select 128,'LLL' from dual union all
select 1280,'SAND' from dual union all
select 1281,'ATLR' from dual union all
select 1282,'AJSP' from dual union all
select 1283,'AJSP' from dual union all
select 1284,'SDCO' from dual union all
select 1285,'SAND' from dual union all
select 1286,'SAND' from dual union all
select 1287,'SIMED' from dual union all
select 1288,'SIMED' from dual union all
select 1289,'SIMED' from dual union all
select 129,'AJSP' from dual union all
select 1290,'SIMED' from dual union all
select 1291,'SIMED' from dual union all
select 1292,'SIMED' from dual union all
select 1293,'LMCO' from dual union all
select 1294,'LMPCO' from dual union all
select 1295,'NISST' from dual union all
select 1296,'NISST' from dual union all
select 1297,'NASDA' from dual union all
select 1298,'NASDA' from dual union all
select 1299,'NASDA' from dual union all
select 13,'AFIT' from dual union all
select 130,'AJSP' from dual union all
select 1300,'NASDA' from dual union all
select 1301,'ONERA' from dual union all
select 1302,'APL' from dual union all
select 1303,'SDCO' from dual union all
select 1304,'SDCO' from dual union all
select 1305,'DNA' from dual union all
select 1306,'NASA' from dual union all
select 1307,'MSFC' from dual union all
select 1308,'PARD' from dual union all
select 1309,'SAMSO' from dual union all
select 131,'AJSP' from dual union all
select 1310,'WFF' from dual union all
select 1311,'IIAE' from dual union all
select 1312,'WFF' from dual union all
select 1313,'OSCC' from dual union all
select 1314,'OSCC' from dual union all
select 1315,'OSCC' from dual union all
select 1316,'OSCC' from dual union all
select 1317,'OSCC' from dual union all
select 1318,'WFF' from dual union all
select 1319,'WFF' from dual union all
select 132,'AJSP' from dual union all
select 1320,'WFF' from dual union all
select 1321,'MIT' from dual union all
select 1321,'VOTK' from dual union all
select 1322,'MITT' from dual union all
select 1322,'VOTK' from dual union all
select 1323,'MIT' from dual union all
select 1323,'VOTK' from dual union all
select 1324,'REP' from dual union all
select 1325,'ORDRSA' from dual union all
select 1326,'BMDO' from dual union all
select 1327,'NOTS' from dual union all
select 1328,'UNK' from dual union all
select 1329,'UNK' from dual union all
select 133,'AJSP' from dual union all
select 1330,'DACC' from dual union all
select 1331,'GSFC' from dual union all
select 1332,'GSFC' from dual union all
select 1333,'GSFC' from dual union all
select 1334,'UNK' from dual union all
select 1335,'BMDO' from dual union all
select 1336,'NAWC' from dual union all
select 1337,'SAND' from dual union all
select 1338,'UNK' from dual union all
select 1339,'UNK' from dual union all
select 134,'RLABN' from dual union all
select 1340,'DTI' from dual union all
select 1341,'UNK' from dual union all
select 1342,'MSFC' from dual union all
select 1343,'SAND' from dual union all
select 1344,'DTI' from dual union all
select 1345,'UNK' from dual union all
select 1346,'WFC' from dual union all
select 1347,'SAND' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 1348,'DTI' from dual union all
select 1349,'SAND' from dual union all
select 135,'ATLR' from dual union all
select 1350,'NOTS' from dual union all
select 1351,'NOTS' from dual union all
select 1352,'DACM' from dual union all
select 1353,'DACM' from dual union all
select 1354,'DACM' from dual union all
select 1355,'DACM' from dual union all
select 1356,'DACM' from dual union all
select 1357,'DACM' from dual union all
select 1358,'DACM' from dual union all
select 1359,'DACM' from dual union all
select 136,'ATLC' from dual union all
select 136,'CELES' from dual union all
select 1360,'DACM' from dual union all
select 1361,'DACM' from dual union all
select 1362,'DACM' from dual union all
select 1363,'DACM' from dual union all
select 1364,'DACM' from dual union all
select 1365,'MDAC' from dual union all
select 1366,'DACM' from dual union all
select 1367,'DACM' from dual union all
select 1368,'DACM' from dual union all
select 1369,'DACM' from dual union all
select 137,'LMSC' from dual union all
select 1370,'DACM' from dual union all
select 1371,'DACM' from dual union all
select 1372,'DACM' from dual union all
select 1373,'DACM' from dual union all
select 1374,'DACM' from dual union all
select 1375,'DACM' from dual union all
select 1376,'DACM' from dual union all
select 1377,'DACM' from dual union all
select 1378,'DACM' from dual union all
select 1379,'DACM' from dual union all
select 138,'LMSC' from dual union all
select 1380,'DACM' from dual union all
select 1381,'DACM' from dual union all
select 1382,'DACM' from dual union all
select 1383,'DACM' from dual union all
select 1384,'DACM' from dual union all
select 1385,'DACM' from dual union all
select 1386,'DACM' from dual union all
select 1387,'DACM' from dual union all
select 1388,'DACM' from dual union all
select 1389,'DACM' from dual union all
select 139,'LMA' from dual union all
select 1390,'DACM' from dual union all
select 1391,'DACM' from dual union all
select 1392,'DACM' from dual union all
select 1393,'DACM' from dual union all
select 1394,'DACM' from dual union all
select 1395,'DACM' from dual union all
select 1396,'DACM' from dual union all
select 1397,'CSSAR' from dual union all
select 1398,'CSSAR' from dual union all
select 1399,'CSSAR' from dual union all
select 14,'FRC' from dual union all
select 14,'ZIMN' from dual union all
select 140,'LMA' from dual union all
select 1400,'ONERA' from dual union all
select 1401,'DAC' from dual union all
select 1401,'JPL' from dual union all
select 1402,'MMD' from dual union all
select 1403,'MMD' from dual union all
select 1404,'MMD' from dual union all
select 1405,'MMD' from dual union all
select 1406,'MMD' from dual union all
select 1407,'MMD' from dual union all
select 1408,'MMD' from dual union all
select 1409,'LMA' from dual union all
select 141,'LMA' from dual union all
select 1410,'LMA' from dual union all
select 1411,'LMA' from dual union all
select 1412,'LMA' from dual union all
select 1413,'LMA' from dual union all
select 1414,'LMA' from dual union all
select 1415,'LMA' from dual union all
select 1416,'LMA' from dual union all
select 1417,'LMA' from dual union all
select 1418,'LMA' from dual union all
select 1419,'MARTD' from dual union all
select 142,'GDA' from dual union all
select 1420,'MARTD' from dual union all
select 1421,'MARTD' from dual union all
select 1422,'MARTD' from dual union all
select 1423,'MARTD' from dual union all
select 1424,'MMD' from dual union all
select 1425,'MMD' from dual union all
select 1426,'MMD' from dual union all
select 1427,'MMD' from dual union all
select 1428,'ONERA' from dual union all
select 1429,'KBM' from dual union all
select 143,'GDA' from dual union all
select 1430,'KBM' from dual union all
select 1431,'SAND' from dual union all
select 1432,'SEREB' from dual union all
select 1433,'SEREB' from dual union all
select 1434,'MITT' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 1434,'VOTK' from dual union all
select 1435,'MITT' from dual union all
select 1435,'VOTK' from dual union all
select 1436,'MITT' from dual union all
select 1436,'VOTK' from dual union all
select 1437,'MITT' from dual union all
select 1437,'VOTK' from dual union all
select 1438,'PARD' from dual union all
select 1439,'UNK' from dual union all
select 144,'GDA' from dual union all
select 1440,'UNK' from dual union all
select 1441,'PARD' from dual union all
select 1442,'LMSC' from dual union all
select 1443,'LMSC' from dual union all
select 1444,'PARD' from dual union all
select 1445,'PARD' from dual union all
select 1446,'OKB586' from dual union all
select 1447,'OKB586' from dual union all
select 1448,'YUZH' from dual union all
select 1449,'KHRU' from dual union all
select 145,'GDA' from dual union all
select 1450,'KHRU' from dual union all
select 1451,'KHRU' from dual union all
select 1452,'KHRU' from dual union all
select 1453,'KHRU' from dual union all
select 1454,'KHRU' from dual union all
select 1455,'KHRU' from dual union all
select 1456,'KHRU' from dual union all
select 1457,'KHRU' from dual union all
select 1458,'KHRU' from dual union all
select 1459,'KHRU' from dual union all
select 146,'GDA' from dual union all
select 1460,'KHRU' from dual union all
select 1461,'KPA' from dual union all
select 1462,'KPA' from dual union all
select 1463,'KPA' from dual union all
select 1465,'MSFC' from dual union all
select 1466,'MDAC' from dual union all
select 1467,'THKE' from dual union all
select 1468,'THKE' from dual union all
select 1469,'SKB30' from dual union all
select 147,'GDA' from dual union all
select 1470,'GEMTS' from dual union all
select 1471,'FAKEL' from dual union all
select 1472,'FAKEL' from dual union all
select 1473,'UNK' from dual union all
select 1474,'UNK' from dual union all
select 1475,'INPE' from dual union all
select 1476,'IAE' from dual union all
select 1477,'IAE' from dual union all
select 1478,'INPE' from dual union all
select 1479,'INPE' from dual union all
select 148,'GDA' from dual union all
select 1480,'IAE' from dual union all
select 1481,'IAE' from dual union all
select 1482,'INPE' from dual union all
select 1483,'INPE' from dual union all
select 1484,'INPE' from dual union all
select 1485,'NRL' from dual union all
select 1486,'NRL' from dual union all
select 1487,'ELV' from dual union all
select 1488,'ELV' from dual union all
select 1489,'NORD' from dual union all
select 149,'GDA' from dual union all
select 1490,'LRBA' from dual union all
select 1491,'LRBA' from dual union all
select 1492,'LRBA' from dual union all
select 1493,'LRBA' from dual union all
select 1494,'LRBA' from dual union all
select 1495,'LRBA' from dual union all
select 1496,'LRBA' from dual union all
select 1497,'POL' from dual union all
select 1498,'LRBA' from dual union all
select 1499,'NRL' from dual union all
select 15,'ATKE' from dual union all
select 150,'GDA' from dual union all
select 1500,'NRL' from dual union all
select 1501,'SDCO' from dual union all
select 1502,'SDCO' from dual union all
select 1503,'SDCO' from dual union all
select 1504,'OSCC' from dual union all
select 1505,'SAND' from dual union all
select 1506,'SDCO' from dual union all
select 1507,'MAK' from dual union all
select 1508,'OKB1' from dual union all
select 1509,'OKB1' from dual union all
select 151,'GDA' from dual union all
select 1510,'PROG' from dual union all
select 1511,'OKB1' from dual union all
select 1512,'OKB1' from dual union all
select 1513,'OKB1' from dual union all
select 1514,'MAK' from dual union all
select 1515,'DAC' from dual union all
select 1515,'JPL' from dual union all
select 1516,'DAC' from dual union all
select 1516,'JPL' from dual union all
select 1517,'DAC' from dual union all
select 1517,'JPL' from dual union all
select 1518,'LARC' from dual union all
select 1519,'WEHR' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 152,'GDA' from dual union all
select 1520,'NAA' from dual union all
select 1521,'LVN' from dual union all
select 1522,'LVN' from dual union all
select 1523,'LVN' from dual union all
select 1524,'LVN' from dual union all
select 1525,'BNAP' from dual union all
select 1526,'LVN' from dual union all
select 1527,'LVN' from dual union all
select 1528,'LVN' from dual union all
select 1529,'MASTEN' from dual union all
select 153,'GDA' from dual union all
select 1530,'MITT' from dual union all
select 1530,'VOTK' from dual union all
select 1531,'MITT' from dual union all
select 1531,'VOTK' from dual union all
select 1532,'MITT' from dual union all
select 1532,'VOTK' from dual union all
select 1533,'NPOL' from dual union all
select 1534,'IR' from dual union all
select 1535,'CONTR' from dual union all
select 1536,'CONTR' from dual union all
select 1537,'YUZH' from dual union all
select 1538,'YUZH' from dual union all
select 1539,'YUZH' from dual union all
select 154,'GDA' from dual union all
select 1540,'YUZH' from dual union all
select 1541,'YUZH' from dual union all
select 1542,'YUZH' from dual union all
select 1543,'YUZH' from dual union all
select 1544,'YUZH' from dual union all
select 1545,'NUDTC' from dual union all
select 1546,'SMDC' from dual union all
select 1547,'IR' from dual union all
select 1548,'MAK' from dual union all
select 1549,'MAK' from dual union all
select 155,'GDA' from dual union all
select 1550,'MAK' from dual union all
select 1551,'LMCO' from dual union all
select 1552,'OSCC' from dual union all
select 156,'GDA' from dual union all
select 157,'GDA' from dual union all
select 158,'GDA' from dual union all
select 159,'GDA' from dual union all
select 16,'AMROC' from dual union all
select 160,'GDA' from dual union all
select 161,'GDA' from dual union all
select 162,'GDA' from dual union all
select 163,'GDA' from dual union all
select 164,'GDA' from dual union all
select 165,'GDA' from dual union all
select 166,'GDA' from dual union all
select 167,'GDA' from dual union all
select 168,'GDA' from dual union all
select 169,'GDA' from dual union all
select 17,'KRAT' from dual union all
select 170,'GDA' from dual union all
select 171,'LMA' from dual union all
select 172,'LMA' from dual union all
select 173,'LMA' from dual union all
select 174,'LMA' from dual union all
select 175,'GDA' from dual union all
select 176,'GDA' from dual union all
select 177,'GDA' from dual union all
select 178,'GDA' from dual union all
select 179,'GDA' from dual union all
select 18,'OSCC' from dual union all
select 180,'GDA' from dual union all
select 181,'LMA' from dual union all
select 182,'LMA' from dual union all
select 183,'LMA' from dual union all
select 184,'LMA' from dual union all
select 185,'LMA' from dual union all
select 186,'LMA' from dual union all
select 187,'LMA' from dual union all
select 188,'LMA' from dual union all
select 189,'LMA' from dual union all
select 19,'RAYTH' from dual union all
select 190,'LMA' from dual union all
select 191,'LMA' from dual union all
select 192,'LMA' from dual union all
select 193,'LMA' from dual union all
select 194,'LMA' from dual union all
select 195,'LMA' from dual union all
select 196,'LMA' from dual union all
select 197,'SNECMA' from dual union all
select 198,'SANJ' from dual union all
select 199,'LARCN' from dual union all
select 2,'FAKEL' from dual union all
select 20,'LTV' from dual union all
select 200,'ISAS' from dual union all
select 201,'FAKEL' from dual union all
select 202,'CFTHB' from dual union all
select 203,'AESPC' from dual union all
select 204,'AESPC' from dual union all
select 205,'CFTHB' from dual union all
select 206,'ONERA' from dual union all
select 207,'RAE' from dual union all
select 208,'RAE' from dual union all
select 209,'BAL' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 21,'COOP' from dual union all
select 210,'BAL' from dual union all
select 211,'BAL' from dual union all
select 212,'BAL' from dual union all
select 213,'BAL' from dual union all
select 214,'CARDE' from dual union all
select 215,'CARDE' from dual union all
select 216,'CARDE' from dual union all
select 217,'BAL' from dual union all
select 218,'BAL' from dual union all
select 219,'BAL' from dual union all
select 22,'ISRO' from dual union all
select 220,'BAL' from dual union all
select 221,'BAL' from dual union all
select 222,'BAL' from dual union all
select 223,'BAL' from dual union all
select 224,'BAL' from dual union all
select 225,'BAL' from dual union all
select 226,'BAL' from dual union all
select 227,'BAL' from dual union all
select 228,'BAL' from dual union all
select 229,'BAL' from dual union all
select 23,'SMDC' from dual union all
select 230,'BAL' from dual union all
select 231,'BAL' from dual union all
select 232,'BAL' from dual union all
select 233,'BAL' from dual union all
select 234,'BAL' from dual union all
select 235,'BAL' from dual union all
select 236,'BAL' from dual union all
select 237,'BAL' from dual union all
select 238,'BAL' from dual union all
select 239,'SROE' from dual union all
select 24,'VSSC' from dual union all
select 240,'SROE' from dual union all
select 241,'SROE' from dual union all
select 242,'SROE' from dual union all
select 243,'SROE' from dual union all
select 244,'IAI' from dual union all
select 245,'LTV' from dual union all
select 246,'LTV' from dual union all
select 247,'LTV' from dual union all
select 248,'LTV' from dual union all
select 249,'LTV' from dual union all
select 25,'PKNDC' from dual union all
select 250,'IAI' from dual union all
select 251,'ELDO' from dual union all
select 252,'EELEC' from dual union all
select 253,'MARTB' from dual union all
select 254,'MARTB' from dual union all
select 255,'BOC' from dual union all
select 256,'BOC' from dual union all
select 257,'ATLR' from dual union all
select 258,'ATLR' from dual union all
select 259,'KPA' from dual union all
select 26,'WRE' from dual union all
select 260,'KPA' from dual union all
select 261,'MITT' from dual union all
select 261,'VOTK' from dual union all
select 262,'FAIRA' from dual union all
select 263,'THKE' from dual union all
select 264,'THKE' from dual union all
select 265,'GEMTS' from dual union all
select 266,'ANSAL' from dual union all
select 267,'ANSAL' from dual union all
select 268,'OKB23' from dual union all
select 269,'VSSC' from dual union all
select 27,'WRE' from dual union all
select 270,'VSSC' from dual union all
select 271,'CSXT' from dual union all
select 272,'PARD' from dual union all
select 273,'SDCO' from dual union all
select 274,'NOTS' from dual union all
select 275,'IIAE' from dual union all
select 276,'IIAE' from dual union all
select 277,'THKX' from dual union all
select 278,'THKX' from dual union all
select 279,'IIAE' from dual union all
select 28,'AJ' from dual union all
select 280,'OSCC' from dual union all
select 281,'OSCC' from dual union all
select 282,'UNK' from dual union all
select 283,'OSCC' from dual union all
select 284,'UNK' from dual union all
select 285,'LRS' from dual union all
select 286,'LRS' from dual union all
select 287,'SUD' from dual union all
select 288,'SUD' from dual union all
select 289,'SUD' from dual union all
select 29,'AJ' from dual union all
select 290,'SUD' from dual union all
select 291,'SUD' from dual union all
select 292,'IIAE' from dual union all
select 293,'CALT' from dual union all
select 294,'CALT' from dual union all
select 294,'SLREC' from dual union all
select 295,'CALT' from dual union all
select 296,'CALT' from dual union all
select 297,'CALT' from dual union all
select 298,'CALT' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 299,'CALT' from dual union all
select 3,'NOVTR' from dual union all
select 30,'AJGC' from dual union all
select 300,'CALT' from dual union all
select 301,'CALT' from dual union all
select 302,'CALT' from dual union all
select 303,'SAST' from dual union all
select 304,'SAST' from dual union all
select 305,'CALT' from dual union all
select 306,'CALT' from dual union all
select 307,'CALT' from dual union all
select 308,'CALT' from dual union all
select 309,'CALT' from dual union all
select 31,'AJGC' from dual union all
select 310,'CALT' from dual union all
select 311,'CALT' from dual union all
select 312,'CALT' from dual union all
select 313,'CALT' from dual union all
select 314,'CALT' from dual union all
select 315,'CALT' from dual union all
select 316,'CALT' from dual union all
select 317,'CALT' from dual union all
select 318,'SAST' from dual union all
select 319,'SAST' from dual union all
select 32,'AJGC' from dual union all
select 320,'SAST' from dual union all
select 321,'CALT' from dual union all
select 322,'CALT' from dual union all
select 323,'CALT' from dual union all
select 324,'SAST' from dual union all
select 325,'CALT' from dual union all
select 326,'CALT' from dual union all
select 327,'DRG' from dual union all
select 328,'DRG' from dual union all
select 329,'ZIMN' from dual union all
select 33,'AJGC' from dual union all
select 330,'ZIMN' from dual union all
select 331,'WRE' from dual union all
select 332,'WRE' from dual union all
select 333,'WRE' from dual union all
select 334,'WRE' from dual union all
select 335,'MMD' from dual union all
select 336,'EERSSI' from dual union all
select 337,'LRBA' from dual union all
select 337,'NORD' from dual union all
select 338,'WRE' from dual union all
select 339,'JPL' from dual union all
select 339,'SUEL' from dual union all
select 34,'AJGC' from dual union all
select 340,'JPL' from dual union all
select 341,'JPL' from dual union all
select 341,'SUEL' from dual union all
select 342,'OSCC' from dual union all
select 343,'SAND' from dual union all
select 344,'DAC' from dual union all
select 345,'MDAC' from dual union all
select 346,'MDAC' from dual union all
select 347,'CALT' from dual union all
select 348,'SANJ' from dual union all
select 349,'SANJ' from dual union all
select 35,'AJGC' from dual union all
select 350,'ARMT' from dual union all
select 351,'CASIC' from dual union all
select 352,'CASIC' from dual union all
select 353,'CALT' from dual union all
select 354,'CASIC4A' from dual union all
select 355,'CALT' from dual union all
select 356,'CALT' from dual union all
select 357,'ARMT' from dual union all
select 358,'CALT' from dual union all
select 359,'CALT' from dual union all
select 36,'AJGC' from dual union all
select 360,'ARMT' from dual union all
select 361,'IIAE' from dual union all
select 362,'UNK' from dual union all
select 363,'ONERA' from dual union all
select 364,'SUD' from dual union all
select 365,'PARD' from dual union all
select 366,'SAND' from dual union all
select 367,'PARD' from dual union all
select 368,'PARD' from dual union all
select 369,'RPI' from dual union all
select 37,'AJGC' from dual union all
select 370,'SUI' from dual union all
select 371,'PARD' from dual union all
select 372,'SUI' from dual union all
select 373,'MDAC' from dual union all
select 374,'MDAC' from dual union all
select 375,'MDAC' from dual union all
select 376,'MDAC' from dual union all
select 377,'MDAC' from dual union all
select 378,'MDAC' from dual union all
select 379,'MDAC' from dual union all
select 38,'AJGC' from dual union all
select 380,'MDAC' from dual union all
select 381,'MDAC' from dual union all
select 382,'MDAC' from dual union all
select 383,'MDAC' from dual union all
select 384,'MDAC' from dual union all
select 385,'MDAC' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 386,'MDAC' from dual union all
select 387,'MDAC' from dual union all
select 388,'MDAC' from dual union all
select 389,'MDAC' from dual union all
select 39,'AJGC' from dual union all
select 390,'MDAC' from dual union all
select 391,'MDAC' from dual union all
select 392,'MDAC' from dual union all
select 393,'MDAC' from dual union all
select 394,'MDAC' from dual union all
select 395,'MDAC' from dual union all
select 396,'BOHB' from dual union all
select 397,'BOHB' from dual union all
select 398,'BOHB' from dual union all
select 399,'BOHB' from dual union all
select 40,'AJGC' from dual union all
select 400,'BOHB' from dual union all
select 401,'BOHB' from dual union all
select 402,'BOHB' from dual union all
select 403,'BOHB' from dual union all
select 404,'BOHB' from dual union all
select 405,'MDAC' from dual union all
select 406,'MDAC' from dual union all
select 407,'MDAC' from dual union all
select 408,'MDAC' from dual union all
select 409,'MDAC' from dual union all
select 41,'AJGC' from dual union all
select 410,'MDAC' from dual union all
select 411,'MDAC' from dual union all
select 412,'MDAC' from dual union all
select 413,'MDAC' from dual union all
select 414,'MDAC' from dual union all
select 415,'MDAC' from dual union all
select 416,'MDAC' from dual union all
select 417,'MDAC' from dual union all
select 418,'MDAC' from dual union all
select 419,'MDAC' from dual union all
select 42,'AJGC' from dual union all
select 420,'MDAC' from dual union all
select 421,'MDAC' from dual union all
select 422,'MDAC' from dual union all
select 423,'MDAC' from dual union all
select 424,'MDAC' from dual union all
select 425,'MDAC' from dual union all
select 426,'MDAC' from dual union all
select 427,'MDAC' from dual union all
select 428,'MDAC' from dual union all
select 429,'BOHB' from dual union all
select 43,'AJGC' from dual union all
select 430,'HAL' from dual union all
select 431,'SEREB' from dual union all
select 432,'SEREB' from dual union all
select 433,'SEREB' from dual union all
select 434,'YUZH' from dual union all
select 435,'STST' from dual union all
select 436,'CALT' from dual union all
select 437,'CALT' from dual union all
select 438,'CALT' from dual union all
select 439,'SAND' from dual union all
select 44,'AJGC' from dual union all
select 440,'SAND' from dual union all
select 441,'PARD' from dual union all
select 442,'PARD' from dual union all
select 443,'PARD' from dual union all
select 444,'PARD' from dual union all
select 445,'MCDA' from dual union all
select 446,'SUD' from dual union all
select 447,'SUD' from dual union all
select 448,'SUD' from dual union all
select 449,'SPX' from dual union all
select 45,'AJ' from dual union all
select 450,'NPOL' from dual union all
select 451,'COLE' from dual union all
select 452,'ELDO' from dual union all
select 453,'LMSC' from dual union all
select 454,'NISST' from dual union all
select 455,'MAK' from dual union all
select 456,'RLABN' from dual union all
select 457,'IR' from dual union all
select 458,'SEREB' from dual union all
select 459,'MATRA' from dual union all
select 46,'AJ' from dual union all
select 460,'NPOE' from dual union all
select 461,'NPOE' from dual union all
select 462,'CFTHB' from dual union all
select 463,'JAXA' from dual union all
select 464,'JAXA' from dual union all
select 465,'JAXA' from dual union all
select 466,'CNES' from dual union all
select 467,'ELDO' from dual union all
select 468,'ELDO' from dual union all
select 469,'BAL' from dual union all
select 47,'AJ' from dual union all
select 470,'UMI' from dual union all
select 471,'PARD' from dual union all
select 472,'AVIB' from dual union all
select 473,'AVIB' from dual union all
select 474,'VSSC' from dual union all
select 475,'SPX' from dual union all
select 476,'SPX' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 477,'SPX' from dual union all
select 478,'SPX' from dual union all
select 479,'SPX' from dual union all
select 48,'AJ' from dual union all
select 480,'SPX' from dual union all
select 481,'SPX' from dual union all
select 482,'SPX' from dual union all
select 483,'SPX' from dual union all
select 484,'SPX' from dual union all
select 485,'USAFA' from dual union all
select 486,'RAE' from dual union all
select 486,'RORD' from dual union all
select 487,'RAE' from dual union all
select 487,'RORD' from dual union all
select 488,'AENUT' from dual union all
select 489,'IR' from dual union all
select 49,'AJ' from dual union all
select 490,'ORDRSA' from dual union all
select 491,'ORDRSA' from dual union all
select 492,'SBA' from dual union all
select 493,'ERNO' from dual union all
select 494,'ATLR' from dual union all
select 495,'BAJUK' from dual union all
select 496,'OSCC' from dual union all
select 497,'OSCC' from dual union all
select 498,'BOHSV' from dual union all
select 499,'LMSC' from dual union all
select 5,'NII88' from dual union all
select 50,'AJ' from dual union all
select 500,'GDA' from dual union all
select 501,'SHAR' from dual union all
select 501,'VSSC' from dual union all
select 502,'SHAR' from dual union all
select 502,'VSSC' from dual union all
select 503,'SHAR' from dual union all
select 503,'VSSC' from dual union all
select 504,'SHAR' from dual union all
select 504,'VSSC' from dual union all
select 505,'ISRO' from dual union all
select 506,'IIAE' from dual union all
select 507,'SAND' from dual union all
select 508,'IR' from dual union all
select 509,'KRLP' from dual union all
select 51,'AJ' from dual union all
select 510,'PKPMO' from dual union all
select 511,'FAIRA' from dual union all
select 512,'WRE' from dual union all
select 513,'FAKEL' from dual union all
select 514,'CITED' from dual union all
select 515,'CITED' from dual union all
select 516,'CFTHB' from dual union all
select 517,'SPX' from dual union all
select 518,'MAK' from dual union all
select 519,'SRCI' from dual union all
select 52,'AJ' from dual union all
select 520,'BRL' from dual union all
select 521,'BRL' from dual union all
select 522,'MHITO' from dual union all
select 523,'MHITO' from dual union all
select 524,'MHITO' from dual union all
select 525,'MHITO' from dual union all
select 526,'MHITO' from dual union all
select 527,'NASDA' from dual union all
select 528,'NASDA' from dual union all
select 529,'NASDA' from dual union all
select 53,'AJ' from dual union all
select 530,'NASDA' from dual union all
select 531,'NASDA' from dual union all
select 532,'NASDA' from dual union all
select 533,'NASDA' from dual union all
select 534,'JAXA' from dual union all
select 535,'WRE' from dual union all
select 536,'WRE' from dual union all
select 537,'HAL5' from dual union all
select 538,'WRE' from dual union all
select 539,'MDAC' from dual union all
select 54,'AJ' from dual union all
select 540,'BOKT' from dual union all
select 541,'NISST' from dual union all
select 542,'ORDRSA' from dual union all
select 543,'SDCO' from dual union all
select 544,'SDCO' from dual union all
select 545,'SDCO' from dual union all
select 546,'PARD' from dual union all
select 547,'SDCO' from dual union all
select 548,'SDCO' from dual union all
select 549,'PARD' from dual union all
select 55,'AJ' from dual union all
select 550,'PARD' from dual union all
select 551,'PARD' from dual union all
select 552,'PARD' from dual union all
select 553,'PARD' from dual union all
select 554,'PARD' from dual union all
select 555,'PARD' from dual union all
select 556,'UNK' from dual union all
select 557,'PARD' from dual union all
select 558,'LMMS' from dual union all
select 559,'PARD' from dual union all
select 56,'AJ' from dual union all
select 560,'PARD' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 561,'SDCO' from dual union all
select 562,'ISRO' from dual union all
select 563,'AEROPH' from dual union all
select 564,'PARD' from dual union all
select 565,'PARD' from dual union all
select 566,'LMSC' from dual union all
select 567,'AELM' from dual union all
select 568,'SUPA' from dual union all
select 569,'SUPA' from dual union all
select 57,'AJ' from dual union all
select 570,'COLE' from dual union all
select 571,'GEMTS' from dual union all
select 572,'GEMTS' from dual union all
select 573,'GEMTS' from dual union all
select 574,'GEMTS' from dual union all
select 575,'CONV' from dual union all
select 576,'ORDRSA' from dual union all
select 577,'RPI' from dual union all
select 578,'NMSU' from dual union all
select 579,'KPA' from dual union all
select 58,'AJ' from dual union all
select 580,'KPA' from dual union all
select 581,'KPA' from dual union all
select 582,'KPA' from dual union all
select 583,'KPA' from dual union all
select 584,'KPA' from dual union all
select 585,'KPA' from dual union all
select 586,'KPA' from dual union all
select 587,'NAMMO' from dual union all
select 588,'LLNL' from dual union all
select 588,'NOTS' from dual union all
select 589,'LLNL' from dual union all
select 589,'NWC' from dual union all
select 59,'AJ' from dual union all
select 590,'OSCC' from dual union all
select 591,'EAC' from dual union all
select 592,'ADD' from dual union all
select 593,'ADD' from dual union all
select 594,'ADD' from dual union all
select 595,'ADD' from dual union all
select 596,'OATK' from dual union all
select 597,'INTA' from dual union all
select 598,'INTA' from dual union all
select 599,'INTA' from dual union all
select 6,'SKB30' from dual union all
select 60,'AJ' from dual union all
select 600,'INTA' from dual union all
select 601,'OSCC' from dual union all
select 602,'ERAU' from dual union all
select 603,'DAC' from dual union all
select 604,'GSFC' from dual union all
select 605,'WFF' from dual union all
select 606,'SRCI' from dual union all
select 607,'ATLR' from dual union all
select 607,'NRL' from dual union all
select 608,'KBM' from dual union all
select 609,'KBM' from dual union all
select 61,'SEREB' from dual union all
select 610,'KBM' from dual union all
select 611,'FIAN' from dual union all
select 612,'NISST' from dual union all
select 613,'NASDA' from dual union all
select 614,'CASIC4A' from dual union all
select 615,'ARMT' from dual union all
select 616,'RAE' from dual union all
select 616,'WRE' from dual union all
select 617,'RAE' from dual union all
select 617,'WRE' from dual union all
select 618,'RAE' from dual union all
select 618,'WRE' from dual union all
select 619,'WFF' from dual union all
select 62,'DRDO' from dual union all
select 620,'RAE' from dual union all
select 620,'WRE' from dual union all
select 621,'RAE' from dual union all
select 621,'WRE' from dual union all
select 622,'OSCC' from dual union all
select 623,'AEROL' from dual union all
select 624,'AEROL' from dual union all
select 625,'SDCO' from dual union all
select 626,'DASST' from dual union all
select 626,'IAI' from dual union all
select 627,'IAI' from dual union all
select 628,'IAI' from dual union all
select 629,'DASST' from dual union all
select 629,'IAI' from dual union all
select 63,'DRDO' from dual union all
select 630,'AEROL' from dual union all
select 631,'RPI' from dual union all
select 632,'OSCC' from dual union all
select 633,'ABMA' from dual union all
select 633,'CHRY' from dual union all
select 634,'ABMA' from dual union all
select 634,'CHRY' from dual union all
select 635,'ABMA' from dual union all
select 635,'CHRY' from dual union all
select 636,'ABMA' from dual union all
select 637,'ABMA' from dual union all
select 638,'ABMA' from dual union all
select 638,'CHRY' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 639,'DRDO' from dual union all
select 64,'DRDO' from dual union all
select 640,'DRDO' from dual union all
select 641,'HANWHA' from dual union all
select 642,'ISAS' from dual union all
select 643,'ISAS' from dual union all
select 644,'OKB586' from dual union all
select 645,'OKB586' from dual union all
select 646,'POL' from dual union all
select 647,'POL' from dual union all
select 648,'CMIK' from dual union all
select 649,'KARI' from dual union all
select 65,'DRDO' from dual union all
select 650,'KARI' from dual union all
select 651,'KARI' from dual union all
select 652,'CASIC4A' from dual union all
select 653,'CASIC4A' from dual union all
select 654,'ISAS' from dual union all
select 655,'ISAS' from dual union all
select 656,'ISAS' from dual union all
select 657,'ISAS' from dual union all
select 658,'ISAS' from dual union all
select 659,'ISAS' from dual union all
select 66,'DRDO' from dual union all
select 660,'ISAS' from dual union all
select 661,'ISAS' from dual union all
select 662,'ISAS' from dual union all
select 663,'ISAS' from dual union all
select 664,'ISAS' from dual union all
select 665,'ISAS' from dual union all
select 666,'ISAS' from dual union all
select 667,'ISAS' from dual union all
select 668,'ISAS' from dual union all
select 669,'IR' from dual union all
select 67,'DRDO' from dual union all
select 670,'IR' from dual union all
select 671,'IR' from dual union all
select 672,'RPI' from dual union all
select 673,'RPI' from dual union all
select 674,'FAKEL' from dual union all
select 675,'WRE' from dual union all
select 676,'WRE' from dual union all
select 677,'WRE' from dual union all
select 678,'OKB586' from dual union all
select 679,'OKB586' from dual union all
select 68,'DRDO' from dual union all
select 680,'POL' from dual union all
select 681,'OKB586' from dual union all
select 682,'OKB586' from dual union all
select 683,'CASIC4A' from dual union all
select 684,'MITT' from dual union all
select 684,'VOTK' from dual union all
select 685,'KPA' from dual union all
select 686,'SDCO' from dual union all
select 687,'JPL' from dual union all
select 688,'LMSC' from dual union all
select 689,'GRUM' from dual union all
select 69,'IQ' from dual union all
select 690,'LMSC' from dual union all
select 691,'COLE' from dual union all
select 692,'MHITO' from dual union all
select 693,'MHITO' from dual union all
select 694,'MHITO' from dual union all
select 695,'LMSC' from dual union all
select 696,'ISAS' from dual union all
select 697,'ISAS' from dual union all
select 698,'ISAS' from dual union all
select 699,'ISAS' from dual union all
select 7,'SKB30' from dual union all
select 70,'IQ' from dual union all
select 700,'ISAS' from dual union all
select 701,'ISAS' from dual union all
select 702,'LTVM' from dual union all
select 703,'FAIRA' from dual union all
select 704,'MAK' from dual union all
select 705,'RAE' from dual union all
select 706,'ONERA' from dual union all
select 706,'SNECMA' from dual union all
select 707,'NAA' from dual union all
select 708,'NAA' from dual union all
select 709,'NAA' from dual union all
select 71,'FATAH' from dual union all
select 710,'NAA' from dual union all
select 711,'GDA' from dual union all
select 712,'GDA' from dual union all
select 713,'GDA' from dual union all
select 714,'GDA' from dual union all
select 715,'SDCO' from dual union all
select 716,'SDCO' from dual union all
select 717,'SDCO' from dual union all
select 718,'SUI' from dual union all
select 719,'WRE' from dual union all
select 72,'IQ' from dual union all
select 720,'WRE' from dual union all
select 721,'WRE' from dual union all
select 722,'WRE' from dual union all
select 723,'MIT' from dual union all
select 724,'USN' from dual union all
select 725,'TAIFUN' from dual union all
select 726,'TAIFUN' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 727,'TAIFUN' from dual union all
select 728,'TAIFUN' from dual union all
select 729,'SYMC' from dual union all
select 73,'IQ' from dual union all
select 730,'NISST' from dual union all
select 731,'NISST' from dual union all
select 732,'OSCC' from dual union all
select 733,'VSSC' from dual union all
select 734,'SUD' from dual union all
select 735,'DASST' from dual union all
select 735,'IAI' from dual union all
select 736,'DASST' from dual union all
select 736,'IAI' from dual union all
select 737,'KBPT' from dual union all
select 738,'TAIFUN' from dual union all
select 739,'TAIFUN' from dual union all
select 74,'EGAO' from dual union all
select 740,'TAIFUN' from dual union all
select 741,'TAIFUN' from dual union all
select 742,'TAIFUN' from dual union all
select 743,'TAIFUN' from dual union all
select 744,'YUZH' from dual union all
select 745,'YUZH' from dual union all
select 746,'LMCO' from dual union all
select 747,'OSCC' from dual union all
select 748,'OSCC' from dual union all
select 749,'SEREB' from dual union all
select 75,'IQMIC' from dual union all
select 750,'SEREB' from dual union all
select 751,'SEREB' from dual union all
select 752,'SEREB' from dual union all
select 753,'SEREB' from dual union all
select 754,'SEREB' from dual union all
select 755,'AELM' from dual union all
select 756,'AELM' from dual union all
select 757,'AELM' from dual union all
select 758,'EADS' from dual union all
select 759,'NISST' from dual union all
select 76,'IQMIC' from dual union all
select 760,'NISST' from dual union all
select 761,'NISST' from dual union all
select 762,'AEROPH' from dual union all
select 763,'NORTH' from dual union all
select 764,'MARTB' from dual union all
select 765,'SAND' from dual union all
select 766,'SRCI' from dual union all
select 767,'SRCI' from dual union all
select 768,'SRCI' from dual union all
select 769,'SRCI' from dual union all
select 77,'AERIT' from dual union all
select 770,'SRCI' from dual union all
select 771,'MARTB' from dual union all
select 772,'SSC' from dual union all
select 773,'VSSC' from dual union all
select 774,'WFF' from dual union all
select 775,'SUD' from dual union all
select 776,'PAI' from dual union all
select 777,'PAI' from dual union all
select 778,'PAI' from dual union all
select 779,'PAI' from dual union all
select 78,'SVC' from dual union all
select 780,'PAI' from dual union all
select 781,'ATLR' from dual union all
select 782,'SPFL' from dual union all
select 783,'OSCC' from dual union all
select 784,'BOUT' from dual union all
select 785,'OSCC' from dual union all
select 786,'OSCC' from dual union all
select 787,'OSCC' from dual union all
select 788,'OSCC' from dual union all
select 789,'OSCC' from dual union all
select 79,'KHRU' from dual union all
select 790,'OSCC' from dual union all
select 791,'OSCC' from dual union all
select 792,'BOUT' from dual union all
select 793,'BOUT' from dual union all
select 794,'BOUT' from dual union all
select 795,'BOUT' from dual union all
select 796,'BOUT' from dual union all
select 797,'LMMS' from dual union all
select 798,'BOUT' from dual union all
select 799,'BOUT' from dual union all
select 8,'WEHR' from dual union all
select 80,'KHRU' from dual union all
select 800,'OKB1' from dual union all
select 801,'OKB1' from dual union all
select 802,'OKB1' from dual union all
select 803,'PROG' from dual union all
select 804,'PROG' from dual union all
select 805,'PROG' from dual union all
select 806,'PROG' from dual union all
select 807,'PROG' from dual union all
select 808,'PROG' from dual union all
select 809,'PROG' from dual union all
select 81,'ONERA' from dual union all
select 810,'PROG' from dual union all
select 811,'PROG' from dual union all
select 812,'IST' from dual union all
select 813,'ATEF' from dual union all
select 814,'ATEF' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 815,'NISST' from dual union all
select 816,'NISST' from dual union all
select 817,'NISST' from dual union all
select 818,'NISST' from dual union all
select 819,'NISST' from dual union all
select 82,'OSCW' from dual union all
select 820,'NISST' from dual union all
select 821,'NISST' from dual union all
select 822,'MHITO' from dual union all
select 823,'OKB1' from dual union all
select 824,'MHITO' from dual union all
select 825,'MHITO' from dual union all
select 826,'STA' from dual union all
select 827,'STA' from dual union all
select 828,'STA' from dual union all
select 829,'ISAS' from dual union all
select 829,'NAL' from dual union all
select 83,'OSCW' from dual union all
select 830,'NAAD' from dual union all
select 831,'KARI' from dual union all
select 832,'NAAD' from dual union all
select 833,'IR' from dual union all
select 834,'BLOR' from dual union all
select 835,'ABL' from dual union all
select 836,'WELEC' from dual union all
select 837,'SAND' from dual union all
select 838,'COOP' from dual union all
select 839,'UMI' from dual union all
select 84,'OSCW' from dual union all
select 840,'PARD' from dual union all
select 841,'PARD' from dual union all
select 842,'UNK' from dual union all
select 843,'DACC' from dual union all
select 843,'WELEC' from dual union all
select 844,'DACC' from dual union all
select 844,'WELEC' from dual union all
select 845,'DACC' from dual union all
select 845,'WELEC' from dual union all
select 846,'SDCO' from dual union all
select 847,'UNK' from dual union all
select 848,'AJSP' from dual union all
select 849,'MHITO' from dual union all
select 85,'OSCW' from dual union all
select 850,'UNK' from dual union all
select 851,'UNK' from dual union all
select 852,'AFCRL' from dual union all
select 853,'UMI' from dual union all
select 854,'PARD' from dual union all
select 855,'PARD' from dual union all
select 856,'LARC' from dual union all
select 857,'PARD' from dual union all
select 858,'PARD' from dual union all
select 859,'PARD' from dual union all
select 86,'NMSU' from dual union all
select 860,'UNK' from dual union all
select 861,'PARD' from dual union all
select 862,'PARD' from dual union all
select 863,'DAC' from dual union all
select 864,'SAND' from dual union all
select 865,'SAND' from dual union all
select 866,'SAND' from dual union all
select 867,'SAND' from dual union all
select 868,'SAND' from dual union all
select 869,'WELEC' from dual union all
select 87,'NAA' from dual union all
select 870,'UMI' from dual union all
select 871,'DACC' from dual union all
select 871,'WELEC' from dual union all
select 872,'DACC' from dual union all
select 872,'WELEC' from dual union all
select 873,'DACC' from dual union all
select 873,'WELEC' from dual union all
select 874,'DACC' from dual union all
select 874,'WELEC' from dual union all
select 875,'AJ' from dual union all
select 875,'DAC' from dual union all
select 876,'AJ' from dual union all
select 876,'DAC' from dual union all
select 877,'AJ' from dual union all
select 877,'DACC' from dual union all
select 878,'AJ' from dual union all
select 878,'DAC' from dual union all
select 879,'AJ' from dual union all
select 879,'DACC' from dual union all
select 88,'ATLR' from dual union all
select 880,'AJ' from dual union all
select 880,'DACC' from dual union all
select 881,'AJ' from dual union all
select 881,'DACC' from dual union all
select 882,'ALMAN' from dual union all
select 883,'ONERA' from dual union all
select 884,'LYK' from dual union all
select 885,'OTR' from dual union all
select 886,'KBM' from dual union all
select 887,'DTI' from dual union all
select 888,'DTI' from dual union all
select 889,'DTI' from dual union all
select 89,'ATLR' from dual union all
select 89,'NRL' from dual union all
select 890,'NASA' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 891,'WFF' from dual union all
select 892,'OSCC' from dual union all
select 893,'IAE' from dual union all
select 894,'IAE' from dual union all
select 895,'HAL' from dual union all
select 896,'HAL' from dual union all
select 897,'NOVTR' from dual union all
select 898,'NOVTR' from dual union all
select 899,'SHAR' from dual union all
select 899,'VSSC' from dual union all
select 9,'WEHR' from dual union all
select 90,'MSFC' from dual union all
select 900,'SHAR' from dual union all
select 900,'VSSC' from dual union all
select 901,'SHAR' from dual union all
select 901,'VSSC' from dual union all
select 902,'SHAR' from dual union all
select 902,'VSSC' from dual union all
select 903,'ADD' from dual union all
select 904,'KPA' from dual union all
select 905,'THKE' from dual union all
select 906,'THKE' from dual union all
select 907,'MMD' from dual union all
select 908,'UNK' from dual union all
select 909,'OSCC' from dual union all
select 91,'AELM' from dual union all
select 910,'OSCC' from dual union all
select 911,'OSCC' from dual union all
select 912,'OSCC' from dual union all
select 913,'OSCC' from dual union all
select 914,'YUZH' from dual union all
select 915,'MITT' from dual union all
select 915,'VOTK' from dual union all
select 916,'MARTO' from dual union all
select 917,'MARTO' from dual union all
select 918,'MARTO' from dual union all
select 919,'MARTO' from dual union all
select 92,'AELM' from dual union all
select 920,'BAJUK' from dual union all
select 921,'BAJUK' from dual union all
select 922,'MITT' from dual union all
select 922,'VOTK' from dual union all
select 923,'MITT' from dual union all
select 923,'VOTK' from dual union all
select 924,'MITT' from dual union all
select 924,'VOTK' from dual union all
select 925,'NMSU' from dual union all
select 926,'NMSU' from dual union all
select 927,'NMSU' from dual union all
select 928,'LMSC' from dual union all
select 929,'LMSC' from dual union all
select 93,'AELM' from dual union all
select 930,'LMSC' from dual union all
select 931,'LMSC' from dual union all
select 932,'LMSC' from dual union all
select 933,'LMSC' from dual union all
select 934,'LMSC' from dual union all
select 935,'LVN' from dual union all
select 936,'LMSC' from dual union all
select 937,'LMSC' from dual union all
select 938,'LVN' from dual union all
select 939,'LMSC' from dual union all
select 94,'AELM' from dual union all
select 940,'HAL' from dual union all
select 941,'HAL' from dual union all
select 942,'HAL' from dual union all
select 943,'JPL' from dual union all
select 944,'JPL' from dual union all
select 945,'NOTS' from dual union all
select 946,'NOTS' from dual union all
select 947,'CITEF' from dual union all
select 948,'SDCO' from dual union all
select 949,'KHRU' from dual union all
select 95,'AELM' from dual union all
select 950,'KHRU' from dual union all
select 951,'KHRU' from dual union all
select 952,'KHRU' from dual union all
select 953,'KHRU' from dual union all
select 954,'KHRU' from dual union all
select 955,'KHRU' from dual union all
select 956,'KHRU' from dual union all
select 957,'KHRU' from dual union all
select 958,'KHRU' from dual union all
select 959,'KHRU' from dual union all
select 96,'AELM' from dual union all
select 960,'KHRU' from dual union all
select 961,'KHRU' from dual union all
select 962,'KHRU' from dual union all
select 963,'KHRU' from dual union all
select 964,'KHRU' from dual union all
select 965,'FAKEL' from dual union all
select 966,'FAKEL' from dual union all
select 967,'MAK' from dual union all
select 968,'PARD' from dual union all
select 969,'NII88' from dual union all
select 97,'AELM' from dual union all
select 970,'NII88' from dual union all
select 971,'MAK' from dual union all
select 971,'OKB1' from dual union all
select 972,'OKB1' from dual;
insert into LAUNCH_VEHICLE_MANUFACTURER
select 973,'OKB1' from dual union all
select 974,'OKB1' from dual union all
select 975,'OKB1' from dual union all
select 976,'OKB586' from dual union all
select 977,'OKB586' from dual union all
select 978,'SKB385' from dual union all
select 979,'OKB586' from dual union all
select 98,'AELM' from dual union all
select 980,'OKB586' from dual union all
select 981,'OKB586' from dual union all
select 982,'OKB586' from dual union all
select 983,'MAK' from dual union all
select 983,'VOTK' from dual union all
select 984,'MAK' from dual union all
select 984,'VOTK' from dual union all
select 985,'OKB1' from dual union all
select 986,'OKB1' from dual union all
select 987,'OKB1' from dual union all
select 988,'OKB1' from dual union all
select 989,'OKB1' from dual union all
select 99,'AELM' from dual union all
select 990,'OKB1' from dual union all
select 991,'OKB1' from dual union all
select 992,'OKB1' from dual union all
select 993,'SKB385' from dual union all
select 994,'OKB1' from dual union all
select 995,'OKB1' from dual union all
select 996,'OKB1' from dual union all
select 997,'ARSL' from dual union all
select 998,'OKB586' from dual union all
select 999,'OKB586' from dual;


--------------------------------------------------------------------------------
-- LAUNCH
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing LAUNCH...');

CREATE TABLE "LAUNCH" 
 (	"LAUNCH_ID" NUMBER, 
	"LAUNCH_TAG" VARCHAR2(15), 
	"LAUNCH_DATE" DATE, 
	"LAUNCH_CATEGORY" VARCHAR2(31), 
	"LAUNCH_STATUS" VARCHAR2(31), 
	"LV_ID" NUMBER, 
	"FLIGHT_ID1" VARCHAR2(21), 
	"FLIGHT_ID2" VARCHAR2(25), 
	"MISSION" VARCHAR2(25), 
	"FLIGHT_CODE" VARCHAR2(25), 
	"FLIGHT_TYPE" VARCHAR2(25), 
	"SITE_ID" NUMBER, 
	"PLATFORM_CODE" VARCHAR2(10), 
	"APOGEE" NUMBER, 
	 CONSTRAINT "LAUNCH_PK" PRIMARY KEY ("LAUNCH_ID")
USING INDEX  ENABLE, 
	 CONSTRAINT "LAUNCH_LV_FK" FOREIGN KEY ("LV_ID")
	  REFERENCES "LAUNCH_VEHICLE" ("LV_ID") ENABLE, 
	 CONSTRAINT "LAUNCH_SITE_FK" FOREIGN KEY ("SITE_ID")
	  REFERENCES "SITE" ("SITE_ID") ENABLE, 
	 CONSTRAINT "LAUNCH_PLATFORM_FK" FOREIGN KEY ("PLATFORM_CODE")
	  REFERENCES "PLATFORM" ("PLATFORM_CODE") ENABLE
 ) ;


CREATE INDEX "LAUNCH_IDX1" ON "LAUNCH" ("LV_ID") 
;

CREATE INDEX "LAUNCH_IDX2" ON "LAUNCH" ("SITE_ID") 
;

CREATE INDEX "LAUNCH_IDX3" ON "LAUNCH" ("PLATFORM_CODE") 
;

CREATE INDEX "LAUNCH_IDX4" ON "LAUNCH" ("LAUNCH_TAG") 
;
insert into LAUNCH
select 1,'1942-A01','1942-06-13 10:52:00','military missile','failure',8,'2','','','','Test',675,'',1 from dual union all
select 10,'1943-A03','1943-02-17 00:00:00','military missile','success',8,'12','','','','Test',675,'',30 from dual union all
select 100,'1944-M02','1944-09-13 16:47:00','military missile','success',8,'18181','M140','','','Test',676,'',64 from dual union all
select 1000,'1951-A11','1951-04-12 00:00:00','atmospheric rocket','success',869,'490-65','Nike 490A 60R','','','Test',1895,'',10 from dual union all
select 10000,'1963-S479','1963-10-05 00:00:00','military missile','success',1053,'CC-2005','ATBM-11 target','','','OT',437,'',90 from dual union all
select 10001,'1963-S48','1963-02-15 00:45:00','military missile','success',916,'403','P-47','','','Test',256,'',180 from dual union all
select 10002,'1963-S480','1963-10-07 20:00:00','suborbital rocket','success',837,'NASA 14.36DI','B field probe','','','Magnetospheric',428,'',165 from dual union all
select 10003,'1963-S481','1963-10-09 23:14:00','suborbital rocket','success',837,'NASA 14.102NA','Release','','','Aeronomy',1821,'',158 from dual union all
select 10004,'1963-S482','1963-10-10 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 10005,'1963-S483','1963-10-10 23:13:00','suborbital rocket','success',837,'NASA 14.103NA','Release','','','Aeronomy',1821,'',158 from dual union all
select 10006,'1963-S484','1963-10-11 00:00:00','military missile','success',976,'','8K63 KY63-26','','','Test',636,'',402 from dual union all
select 10007,'1963-S485','1963-10-12 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 10008,'1963-S486','1963-10-14 18:18:00','military missile','success',916,'','P-64 63 Fall 08','','','OT',437,'',180 from dual union all
select 10009,'1963-S487','1963-10-14 13:56:40','military missile','success',1017,'L2-5','8K74 UBP-14','','','OT',480,'',1350 from dual union all
select 1001,'1951-A12','1951-04-12 17:26:00','suborbital rocket','success',50,'USAF 11','Colorado UVS','','','Solar UV',640,'',29 from dual union all
select 10010,'1963-S488','1963-10-14 05:34:00','suborbital rocket','success',215,'BB-2A-20','','','','Ionos/Auroral',428,'',145 from dual union all
select 10011,'1963-S489','1963-10-15 00:00:00','military missile','success',976,'','8K63 KY63-27','','','Test',636,'',402 from dual union all
select 10012,'1963-S49','1963-02-14 00:00:00','military missile','success',976,'','8K63 PRO-15','','','Test',942,'',402 from dual union all
select 10013,'1963-S490','1963-10-15 09:51:00','suborbital rocket','success',1199,'SL171','UCL GS-3/QUB Na-10','','','Aeronomy',1839,'',210 from dual union all
select 10014,'1963-S491','1963-10-15 11:48:00','suborbital rocket','success',1198,'SL169','UCL GC-24','','','Aeronomy',1839,'',139 from dual union all
select 10015,'1963-S492','1963-10-15 15:09:00','suborbital rocket','success',1198,'SL170','UCL GC-25','','','Aeronomy',1839,'',140 from dual union all
select 10016,'1963-S493','1963-10-15 16:05:00','suborbital rocket','success',30,'NASA 04.33GS','ASE XRT','','','Solar X',1869,'',197 from dual union all
select 10017,'1963-S494','1963-10-15 19:22:00','suborbital rocket','success',1199,'SL168','UCL GS-4/QUB Na-11','','','Aeronomy',1839,'',180 from dual union all
select 10018,'1963-S495','1963-10-16 15:15:00','military missile','success',916,'ATBM Target','P-65 63 Fall 09','','','OT',437,'',180 from dual union all
select 10019,'1963-S496','1963-10-16 16:30:02','military missile','success',929,'A2P-221','PY-1 DASO','','','OT',421,'SSBN 619',1000 from dual union all
select 1002,'1951-A13','1951-04-16 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10020,'1963-S497','1963-10-17 00:00:00','military missile','failure',794,'672','Mk 5 RV','OT, flew long','CEDAR LAKE','OT',1649,'',900 from dual union all
select 10021,'1963-S498','1963-10-17 10:52:00','military missile','success',239,'BK.11','Blue Streak Test','','','Test',1842,'',527 from dual union all
select 10022,'1963-S499','1963-10-17 16:50:00','suborbital rocket','success',35,'NASA 04.93II','FR-1 test','','','Ionos',1821,'',187 from dual union all
select 10023,'1963-S50','1963-02-14 00:00:00','military missile','success',976,'','8K63 KY63-3','','','Test',636,'',402 from dual union all
select 10024,'1963-S500','1963-10-18 14:00:00','military missile','success',916,'','P-66 63 Fall 10','','','OT',437,'',180 from dual union all
select 10025,'1963-S501','1963-10-18 07:09:00','suborbital rocket','success',1490,'AGI47','Bio (Cat)','','','Life Sci',668,'',155 from dual union all
select 10026,'1963-S502','1963-10-18 09:20:00','suborbital rocket','success',215,'CC-2A-39','NO2 release','','','Aeron',428,'',140 from dual union all
select 10027,'1963-S503','1963-10-21 00:00:00','military missile','success',916,'','P-67 63 Fall 11','','','OT',437,'',180 from dual union all
select 10028,'1963-S504','1963-10-22 00:00:00','military missile','success',982,'','8K64U UBP-P1 Groza','','','Test',476,'',1210 from dual union all
select 10029,'1963-S505','1963-10-22 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 1003,'1951-A14','1951-04-17 00:00:00','military missile','success',703,'232','','','','Test',222,'',8 from dual union all
select 10030,'1963-S506','1963-10-23 03:29:00','military missile','success',982,'G 22504-15T','8K64U UBP-34','','','OT',565,'',1210 from dual union all
select 10031,'1963-S507','1963-10-23 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 10032,'1963-S508','1963-10-24 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 609',1000 from dual union all
select 10033,'1963-S509','1963-10-24 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 609',1000 from dual union all
select 10034,'1963-S51','1963-02-14 00:30:00','suborbital rocket','success',30,'AB3.345','Sphere','','','Aeron',394,'',234 from dual union all
select 10035,'1963-S510','1963-10-24 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 609',1000 from dual union all
select 10036,'1963-S511','1963-10-24 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 609',1000 from dual union all
select 10037,'1963-S512','1963-10-24 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 609',1000 from dual union all
select 10038,'1963-S513','1963-10-24 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 609',1000 from dual union all
select 10039,'1963-S514','1963-10-24 05:30:00','suborbital rocket','failure',1490,'AGI50','Bio (Cat)','','','Life Sci',668,'',88 from dual union all
select 1004,'1951-A15','1951-04-26 00:00:00','military missile','success',571,'6','','','','Test',1863,'',24 from dual union all
select 10040,'1963-S515','1963-10-25 00:00:00','suborbital rocket','success',1012,'','R-5B Geophysical','VAO','','Aeron',636,'',500 from dual union all
select 10041,'1963-S516','1963-10-26 16:14:04','military missile','success',932,'A3X-43','','','','Test',421,'SSBN 619',1000 from dual union all
select 10042,'1963-S517','1963-10-26 00:00:00','military missile','success',976,'','8K63 KY63-28','','','Test',636,'',402 from dual union all
select 10043,'1963-S518','1963-10-26 02:00:00','suborbital rocket','success',215,'CC-2A-19','','','','Ionos/Auroral',428,'',145 from dual union all
select 10044,'1963-S519','1963-10-28 00:00:00','military missile','success',916,'','P-69 63 Fall 13','','','OT',437,'',180 from dual union all
select 10045,'1963-S52','1963-02-16 00:30:00','military missile','success',874,'ZW-54 MUDFLAP 2','DM-15S Test 2','','','ASAT Test',1884,'',240 from dual union all
select 10046,'1963-S520','1963-10-28 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 10047,'1963-S521','1963-10-28 03:15:00','ballistic missile test','failure',162,'136F','ABRES WAC-1/Pod 21','','','Reentry Test/Plume',226,'',500 from dual union all
select 10048,'1963-S522','1963-10-29 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 10049,'1963-S523','1963-10-29 02:53:00','suborbital rocket','success',215,'CC-2A-40','NO2 release','','','Aeron',428,'',140 from dual union all
select 1005,'1951-A16','1951-05-03 00:00:00','military missile','success',703,'234','','','','Test',222,'',8 from dual union all
select 10050,'1963-S524','1963-10-30 00:00:00','suborbital rocket','success',446,'D-05','','','','Test',664,'',470 from dual union all
select 10051,'1963-S525','1963-10-30 14:26:00','suborbital rocket','success',1199,'SL134','RAE Test 20','','','Test',1839,'',186 from dual union all
select 10052,'1963-S526','1963-10-31 00:00:00','military missile','success',794,'664','Mk 5 RV','OT','DRAG CHUTE','OT',1651,'',900 from dual union all
select 10053,'1963-S527','1963-10-31 16:50:00','suborbital rocket','success',35,'NASA 04.94II','FR-1 test','','','Ionos',1797,'',187 from dual union all
select 10054,'1963-S528','1963-10-31 18:00:00','suborbital rocket','success',839,'AC6.341','D-region composition','','','Ionos',394,'',112 from dual union all
select 10055,'1963-S529','1963-10-31 21:17:00','suborbital rocket','success',837,'NASA 14.111GT','Vibration test','','','Test',1801,'',134 from dual union all
select 10056,'1963-S53','1963-02-15 16:34:00','suborbital rocket','success',30,'NC3.134','Bennett Mass Spec','','','Aeron',1869,'',239 from dual union all
select 10057,'1963-S530','1963-11-01 00:00:00','military missile','success',871,'ZW-70','','','','Test',1884,'',50 from dual union all
select 10058,'1963-S531','1963-11-01 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 10059,'1963-S532','1963-11-01 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 1006,'1951-A17','1951-05-07 00:00:00','atmospheric rocket','success',365,'','E2 Model','','','Test',1821,'',10 from dual union all
select 10060,'1963-S533','1963-11-01 20:14:54','ballistic missile test','success',1420,'N-25','Mk 6 RV/Pod T','R&D','','Test/Plume study',230,'',1300 from dual union all
select 10061,'1963-S534','1963-11-02 00:00:00','military missile','success',794,'613','Mk 5 RV','OT','HARD LINE','OT',1648,'',900 from dual union all
select 10062,'1963-S535','1963-11-04 09:34:00','military missile','success',154,'232D','ABRES REX-III','','HICKORY HOLLOW','Reentry Test',1662,'',1800 from dual union all
select 10063,'1963-S536','1963-11-04 10:00:00','suborbital rocket','success',837,'Sphere','','','','Aeron',1895,'',215 from dual union all
select 10064,'1963-S537','1963-11-04 10:31:00','suborbital rocket','success',839,'Falling Sphere 5','','','','Aeronomy',871,'',150 from dual union all
select 10065,'1963-S538','1963-11-04 12:26:00','suborbital rocket','success',839,'Falling Sphere 6','','','','Aeronomy',871,'',150 from dual union all
select 10066,'1963-S539','1963-11-04 15:10:00','suborbital rocket','success',30,'KP3.3','Dayglow','','','Aeron',1869,'',200 from dual union all
select 10067,'1963-S54','1963-02-16 00:00:00','suborbital rocket','success',1438,'','','','','Reentry Test',1821,'',280 from dual union all
select 10068,'1963-S540','1963-11-05 00:00:00','suborbital rocket','success',839,'Sphere','','','','Aeronomy',1895,'',164 from dual union all
select 10069,'1963-S541','1963-11-05 00:00:00','military missile','success',976,'','8K63 KY63-29','','','Test',636,'',402 from dual union all
select 1007,'1951-A18','1951-05-08 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10070,'1963-S542','1963-11-05 00:00:00','military missile','success',1018,'E 15001-12','8K75 LKI 3-13','','','OT',584,'',1160 from dual union all
select 10071,'1963-S543','1963-11-06 00:00:00','suborbital rocket','success',839,'Mass spectrometer','','','','Aeronomy',394,'',100 from dual union all
select 10072,'1963-S544','1963-11-07 00:00:00','suborbital rocket','success',217,'AA-3-8','USN Eval 3','','','Test',1015,'',116 from dual union all
select 10073,'1963-S545','1963-11-09 00:00:00','military missile','success',1420,'N-27','Mk 6 RV','R&D','FIRE TRUCK','Test',1658,'',1300 from dual union all
select 10074,'1963-S546','1963-11-09 16:26:00','suborbital rocket','success',839,'Falling Sphere 7','','','','Aeronomy',871,'',141 from dual union all
select 10075,'1963-S547','1963-11-11 18:48:05','military missile','success',932,'A3X-42','','','','Test',421,'SSBN 619',1000 from dual union all
select 10076,'1963-S548','1963-11-12 18:58:00','suborbital rocket','success',35,'NASA 04.76UA','JHU Airglow 6','','','Aeronomy',1821,'',223 from dual union all
select 10077,'1963-S549','1963-11-13 16:27:57','military missile','success',795,'446','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 10078,'1963-S55','1963-02-18 17:43:00','suborbital rocket','success',30,'NE3.140','','','','Solar EUV',1869,'',306 from dual union all
select 10079,'1963-S550','1963-11-13 22:35:39','military missile','failure',154,'158D','Mk 11-5B','ST COOL WATER VI','COOL WATER VI','Test',1666,'',100 from dual union all
select 1008,'1951-A19','1951-05-10 00:00:00','atmospheric rocket','success',869,'490-66','Nike 490A 61R','','','Test',1895,'',10 from dual union all
select 10080,'1963-S551','1963-11-13 23:00:00','suborbital rocket','success',30,'AD3.324','Dayglow','','','Aeron',1869,'',262 from dual union all
select 10081,'1963-S552','1963-11-14 14:30:00','military missile','success',916,'','P-70 63 Winter 1 C','','','OT',437,'',180 from dual union all
select 10082,'1963-S553','1963-11-14 00:00:00','military missile','success',871,'ZK-28','K-24 Titan Intercept','','','Interceptor',873,'',80 from dual union all
select 10083,'1963-S554','1963-11-14 13:42:00','military missile','success',1419,'SM-68','Nike TVX-32','DASO/NTMP KX-24','FAST RIDE','OT',1654,'',1000 from dual union all
select 10084,'1963-S555','1963-11-14 14:58:00','suborbital rocket','success',839,'Falling Sphere 8','','','','Aeronomy',871,'',127 from dual union all
select 10085,'1963-S556','1963-11-16 16:54:42','military missile','success',929,'A2P-233','','','','OT',421,'SSBN 619',1000 from dual union all
select 10086,'1963-S557','1963-11-16 00:00:00','military missile','success',976,'','8K63 KY63-30','','','Test',636,'',402 from dual union all
select 10087,'1963-S558','1963-11-17 18:15:00','suborbital rocket','success',35,'NASA 04.85NA','Airglow','','','Aeronomy',1821,'',370 from dual;
insert into LAUNCH
select 10088,'1963-S559','1963-11-18 00:00:00','suborbital rocket','success',549,'HAD-1','LRL HAD-1','','','Test',1015,'',163 from dual union all
select 10089,'1963-S56','1963-02-19 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 1009,'1951-A20','1951-05-11 00:00:00','military missile','success',703,'235','','','','Test',222,'',8 from dual union all
select 10090,'1963-S560','1963-11-20 17:31:00','military missile','success',916,'','P-71 63 Winter 2 A','','','OT',437,'',180 from dual union all
select 10091,'1963-S561','1963-11-27 06:10:59','military missile','failure',932,'A3X-44','PX-2,PY-2','','','Test',421,'EAG-154',40 from dual union all
select 10092,'1963-S562','1963-11-21 00:00:00','suborbital rocket','success',866,'Sandia 152-7','Ballistic','','','Test',855,'',292 from dual union all
select 10093,'1963-S563','1963-11-21 12:55:00','suborbital rocket','success',837,'NASA 14.128IA','ISRO 10.01  (Na)','TERLS-001','','Aeronomy',1467,'',208 from dual union all
select 10094,'1963-S564','1963-11-22 03:31:00','military missile','success',982,'G 22604-20T','8K64U UBP-35','','','OT',565,'',1210 from dual union all
select 10095,'1963-S565','1963-11-22 21:00:00','suborbital rocket','success',866,'Sandia 152-9','HRT','','','Test',855,'',292 from dual union all
select 10096,'1963-S566','1963-11-23 00:00:00','military missile','success',976,'','8K63 KY63-31','','','Test',636,'',402 from dual union all
select 10097,'1963-S568','1963-11-26 18:16:00','suborbital rocket','success',837,'NASA 14.10UA','UM Composition 3/Den','','','Aeronomy',1821,'',172 from dual union all
select 10098,'1963-S569','1963-11-26 18:44:00','suborbital rocket','success',839,'NASA 10.131UA','Sphere/Structure','','','Aeronomy',1821,'',151 from dual union all
select 10099,'1963-S57','1963-02-21 01:00:04','military missile','success',794,'421B','Mk 5 RV','R&D','','Test',266,'',900 from dual union all
select 101,'1944-S01','1944-01-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1010,'1951-A21','1951-05-17 00:00:00','atmospheric rocket','success',365,'','X-5 E17 Model','','','Test',1821,'',10 from dual union all
select 10100,'1963-S570','1963-11-27 00:00:00','military missile','success',795,'842','Mk 11 RV','DASO-1','BIG CIRCLE','Test',1652,'',900 from dual union all
select 10101,'1963-S571','1963-11-27 00:00:00','military missile','success',976,'','8K63 KY63-32','','','Test',636,'',402 from dual union all
select 10102,'1963-S572','1963-11-27 17:50:00','suborbital rocket','success',30,'NE3.153','','','','Solar EUV',1869,'',216 from dual union all
select 10103,'1963-S573','1963-11-28 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 10104,'1963-S574','1963-11-28 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 10105,'1963-S575','1963-11-29 00:00:00','military missile','failure',794,'642','Mk 5 RV','OT','ARM CHAIR','OT',1651,'',50 from dual union all
select 10106,'1963-S576','1963-11-29 00:00:00','military missile','success',976,'','8K63 KY63-33','','','Test',636,'',402 from dual union all
select 10107,'1963-S577','1963-12-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 10108,'1963-S578','1963-12-01 00:00:00','suborbital rocket','success',1289,'','','','','Test',1364,'',115 from dual union all
select 10109,'1963-S579','1963-12-02 04:15:00','suborbital rocket','success',1199,'SL135','RAE Test 21','','','Test',1839,'',168 from dual union all
select 1011,'1951-A22','1951-05-24 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10110,'1963-S58','1963-02-20 00:00:00','military missile','success',871,'ZW-55','','','','Test',1884,'',50 from dual union all
select 10111,'1963-S580','1963-12-03 00:00:00','military missile','success',976,'','8K63 KY63-34','','','Test',636,'',402 from dual union all
select 10112,'1963-S581','1963-12-03 00:00:00','military missile','success',976,'','8K63 KY63-35','','','Test',636,'',402 from dual union all
select 10113,'1963-S582','1963-12-03 04:59:00','suborbital rocket','failure',1199,'SL104','LUX-16/UCL I-17SUV8','','','Iono/Solar X/UCW-7/Bir-8',1839,'',106 from dual union all
select 10114,'1963-S583','1963-12-03 08:00:29','military missile','success',999,'G 22500-04L','8K67 LKI-1','','','Test',573,'',903 from dual union all
select 10115,'1963-S584','1963-12-04 00:00:00','military missile','success',976,'','8K63 KY63-36','','','Test',636,'',402 from dual union all
select 10116,'1963-S585','1963-12-05 00:00:00','suborbital rocket','success',30,'','HR solar spectrum','','','Solar UV',1869,'',200 from dual union all
select 10117,'1963-S586','1963-12-05 00:00:00','military missile','success',916,'','P-73 63 Winter 4 B','','','OT',437,'',180 from dual union all
select 10118,'1963-S587','1963-12-07 13:11:00','suborbital rocket','success',839,'NASA 10.55GA','GSFC Grenade 35','','','Aeronomy',1821,'',103 from dual union all
select 10119,'1963-S588','1963-12-07 13:43:00','suborbital rocket','success',837,'NASA 14.21UA','UM Pitot 3','','','Aeronomy',1821,'',141 from dual union all
select 1012,'1951-A23','1951-05-29 00:00:00','military missile','failure',703,'242','','','','Test',222,'',0 from dual union all
select 10120,'1963-S589','1963-12-09 00:00:00','military missile','success',979,'','8K65 UBP-48','','','Test',636,'',675 from dual union all
select 10121,'1963-S59','1963-02-20 23:18:00','suborbital rocket','success',837,'NASA 14.35CA','GCA Grenade 1/Rel Na','','','Aeronomy',1821,'',164 from dual union all
select 10122,'1963-S590','1963-12-09 05:24:00','suborbital rocket','failure',130,'AC15.610','Ionization detector','','','Aeron/Auroral',428,'',242 from dual union all
select 10123,'1963-S591','1963-12-10 03:11:17','military missile','success',932,'A3X-48','PX-2,PY-2','','','Test',244,'',1000 from dual union all
select 10124,'1963-S592','1963-12-10 00:00:00','military missile','success',976,'','8K63 KY63-37','','','Test',636,'',402 from dual union all
select 10125,'1963-S593','1963-12-11 01:00:06','military missile','failure',932,'A3X-46','PX-2,PY-2','','','Test',421,'EAG-154',10 from dual union all
select 10126,'1963-S594','1963-12-11 05:00:00','suborbital rocket','success',696,'L-2-2','','','','Ionos',834,'',410 from dual union all
select 10127,'1963-S595','1963-12-12 04:51:00','military missile','success',982,'E 22502-21T','8K64U UBP-36','','','OT',565,'',1210 from dual union all
select 10128,'1963-S596','1963-12-12 08:50:00','suborbital rocket','success',666,'K-8L-2','','','','Aeron/Fields',839,'',103 from dual union all
select 10129,'1963-S597','1963-12-12 16:50:00','suborbital rocket','success',30,'AD3.513','EUV Mon/XR/RPA','','','Solar EUV',1869,'',325 from dual union all
select 1013,'1951-A24','1951-05-29 19:46:00','suborbital rocket','failure',50,'USAF 13','Ionos Propagation','','','Ionos',640,'',4 from dual union all
select 10130,'1963-S598','1963-12-12 19:06:00','suborbital rocket','success',535,'HAD 122','Sphere','','','Aeron',1835,'',100 from dual union all
select 10131,'1963-S599','1963-12-12 20:00:01','ballistic missile test','success',1420,'N-29','Mk 6 RV/Pod T','R&D','','Test/Plume study',230,'',1300 from dual union all
select 10132,'1963-S60','1963-02-20 23:34:00','suborbital rocket','success',839,'NASA 10.58GA','GSFC Grenade 29','','','Aeronomy',428,'',113 from dual union all
select 10133,'1963-S600','1963-12-13 00:00:00','military missile','success',795,'854','Mk 11 RV','DASO-2','ANSWER MAN','Test',1647,'',900 from dual union all
select 10134,'1963-S601','1963-12-13 00:00:00','military missile','success',794,'708','Mk 5 RV','OT','BAMBOO SHOOT','OT',1649,'',900 from dual union all
select 10135,'1963-S602','1963-12-11 00:00:00','military missile','success',871,'ZW-71','Nike X R&D,last WS','','','Test',1884,'',50 from dual union all
select 10136,'1963-S603','1963-12-17 00:00:00','ballistic missile test','success',1420,'N-28','Mk 6 RV/Pod T','R&D','USEFUL TASK','Test/Plume study',1659,'',1300 from dual union all
select 10137,'1963-S604','1963-12-17 00:00:00','military missile','success',976,'','8K63 KY63-39','','','Test',636,'',402 from dual union all
select 10138,'1963-S605','1963-12-17 17:17:34','test rocket','success',249,'302','ERCS 279L','','','Comms',1661,'',1000 from dual union all
select 10139,'1963-S606','1963-12-18 03:00:00','military missile','success',795,'447','Mk 11 RV','R&D','','Test',266,'',900 from dual union all
select 1014,'1951-A25','1951-06-07 00:00:00','military missile','success',703,'236','','','','Test',222,'',8 from dual union all
select 10140,'1963-S607','1963-12-18 00:00:00','suborbital rocket','success',839,'Falling Sphere 9','','','','Aeronomy',871,'',100 from dual union all
select 10141,'1963-S608','1963-12-18 00:00:00','suborbital rocket','success',839,'Falling Sphere 10','','','','Aeronomy',871,'',100 from dual union all
select 10142,'1963-S609','1963-12-18 09:52:27','military missile','success',154,'233D','ABRES LORV-8','','LENS COVER','Reentry Test',1662,'',1800 from dual union all
select 10143,'1963-S61','1963-02-20 23:47:00','suborbital rocket','success',839,'NASA 10.48GA','GSFC Grenade 30','','','Aeronomy',1821,'',113 from dual union all
select 10144,'1963-S610','1963-12-18 22:56:22','military missile','success',162,'109F','Mk 4-5A','R&D','DAY BOOK','Test',1672,'',1400 from dual union all
select 10145,'1963-S611','1963-12-20 00:00:00','military missile','success',794,'515','Mk 5 RV','OT','BENT HOOK','OT',1651,'',900 from dual union all
select 10146,'1963-S612','1963-12-20 04:49:00','military missile','success',982,'T 22501-01T','8K64U UBP-37','','','OT',565,'',1210 from dual union all
select 10147,'1963-S613','1963-12-21 00:00:00','test rocket','success',1433,'L1','','','','Test',664,'',74 from dual union all
select 10148,'1963-S614','1963-12-21 00:00:00','military missile','success',1018,'E 15001-06','8K75 LKI 3-16','','','OT',584,'',1160 from dual union all
select 10149,'1963-S615','1963-12-24 00:00:00','suborbital rocket','success',1012,'','R-5B Geophysical','VAO','','Aeron',636,'',500 from dual union all
select 1015,'1951-A26','1951-06-07 00:00:00','military missile','failure',703,'238','','','','Test',222,'',0 from dual union all
select 10150,'1963-S616','1963-12-25 00:00:00','military missile','success',1018,'G 15001-08 (5TS)','8K75 LKI 3-17','','','OT',584,'',1160 from dual union all
select 10151,'1963-S617','1963-12-29 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 10152,'1963-S618','1963-12-30 00:00:00','military missile','success',979,'','8K65 UBP-49','','','Test',636,'',675 from dual union all
select 10153,'1963-S619','1963-03-18 00:00:00','military missile','success',1035,'','','','','Test',636,'',500 from dual union all
select 10154,'1963-S62','1963-02-21 17:56:07','military missile','success',929,'A2P-117','','','','OT',421,'SSBN 618',1000 from dual union all
select 10155,'1963-S620','1963-10-23 00:00:00','military missile','success',871,'ZW-69','','','','Test',1884,'',50 from dual union all
select 10156,'1963-S621','1963-04-01 00:00:00','suborbital rocket','success',767,'','2A II-1','','','Test',185,'',80 from dual union all
select 10157,'1963-S622','1963-06-01 00:00:00','suborbital rocket','success',767,'','2A II-3','','','Test',185,'',80 from dual union all
select 10158,'1963-S623','1963-08-28 00:00:00','suborbital rocket','success',767,'','2B III-1','','','Test',185,'',80 from dual union all
select 10159,'1963-S624','1963-08-28 00:00:00','suborbital rocket','success',767,'','2B III-2','','','Test',185,'',80 from dual union all
select 1016,'1951-A27','1951-06-07 00:00:00','atmospheric rocket','success',869,'490-70','Nike 490A 62R','','','Test',1895,'',10 from dual union all
select 10160,'1963-S625','1963-09-05 00:00:00','suborbital rocket','success',769,'','3A III-2','','','Test',185,'',80 from dual union all
select 10161,'1963-S626','1963-09-05 00:00:00','suborbital rocket','success',769,'','3A III-3','','','Test',185,'',80 from dual union all
select 10162,'1963-S627','1963-09-05 00:00:00','suborbital rocket','success',769,'','3A III-4','','','Test',185,'',80 from dual union all
select 10163,'1963-S628','1963-10-01 00:00:00','suborbital rocket','success',1250,'','Denpro 01','','','Test',1895,'',100 from dual union all
select 10164,'1963-S629','1963-10-01 00:00:00','suborbital rocket','success',1250,'','Denpro 02','','','Test',1895,'',100 from dual union all
select 10165,'1963-S63','1963-02-21 23:16:00','suborbital rocket','success',837,'NASA 14.39CA','GCA Grenade 2','','','Aeronomy',1821,'',172 from dual union all
select 10166,'1963-S630','1963-02-06 00:00:00','military missile','success',871,'ZW-53','Rail launch test','','','Test',1895,'',50 from dual union all
select 10167,'1963-S631','1963-05-31 00:00:00','military missile','failure',871,'ZW-62','','','','Test',1895,'',50 from dual union all
select 10168,'1963-S632','1963-11-27 00:00:00','military missile','success',916,'','P-72 63 Winter 3 D','','','OT',437,'',180 from dual union all
select 10169,'1963-S633','1963-10-28 03:15:00','suborbital rocket','success',30,'HITAB','TSRA','','','IR Background',1786,'',200 from dual union all
select 1017,'1951-A28','1951-06-08 01:18:00','suborbital rocket','failure',59,'SC 19','Probe','','','Aeronomy',1869,'',6 from dual union all
select 10170,'1963-S634','1963-10-28 03:15:00','suborbital rocket','success',850,'','TRUMP','','','Test',293,'',130 from dual union all
select 10171,'1963-S635','1963-10-01 00:00:00','suborbital rocket','success',1247,'','NITE OWL 2','','','UV Astron',1057,'F-3B',100 from dual union all
select 10172,'1963-S636','1963-11-27 00:00:00','suborbital rocket','success',1350,'HITAB-BWSS-03','','','','IR background',394,'',140 from dual union all
select 10173,'1963-S637','1963-07-01 00:00:00','military missile','failure',1321,'','9K71 Phase 2 No. 12','','','Test',636,'',100 from dual union all
select 10174,'1963-S638','1963-03-19 01:30:00','suborbital rocket','success',839,'TRUMP','','','','Tech',277,'',100 from dual union all
select 10175,'1963-S639','1963-02-12 00:00:00','suborbital rocket','success',839,'TRUMP','','','','Tech',277,'',100 from dual union all
select 10176,'1963-S64','1963-02-22 00:00:00','military missile','success',1018,'T 15002-04','8K75 LKI 3-3','','','OT',584,'',1160 from dual union all
select 10177,'1963-S640','1963-05-18 05:14:00','suborbital rocket','success',839,'TRUMP','','','','Tech',277,'',100 from dual;
insert into LAUNCH
select 10178,'1963-S641','1963-06-28 00:00:00','military missile','success',982,'','8K64U 54 RD UBP-1','','','OT',1468,'',1210 from dual union all
select 10179,'1963-S642','1963-08-30 00:00:00','military missile','success',982,'','8K64U 54 RD UBP-2','','','OT',1468,'',1210 from dual union all
select 1018,'1951-A29','1951-06-14 13:48:00','suborbital rocket','failure',1470,'55','','','','Solar UV,X/CR',1863,'',0 from dual union all
select 10180,'1963-S643','1963-01-18 00:00:00','military missile','success',979,'','8K65 68 RB','Climate test','','Test',3,'',675 from dual union all
select 10181,'1963-S644','1963-02-01 00:00:00','military missile','success',979,'','8K65 68 RB','Climate test','','Test',3,'',675 from dual union all
select 10182,'1963-S645','1963-10-22 00:00:00','military missile','success',976,'','8K63U UBP 51 RB','Groza','','Test',840,'',402 from dual union all
select 10183,'1963-S646','1963-10-01 00:00:00','military missile','success',980,'','8K65U 35 RD UBP-1','','','Test',943,'',675 from dual union all
select 10184,'1963-S647','1963-07-12 00:00:00','military missile','success',982,'','8K64 42 RD UBP-1','','','OT',1002,'',1210 from dual union all
select 10185,'1963-S648','1963-08-24 00:00:00','military missile','success',982,'','8K64 42 RD UBP-2','','','OT',1002,'',1210 from dual union all
select 10186,'1963-S649','1963-10-22 00:00:00','military missile','success',976,'','8K63U UBP 35 RB','','','Test',1013,'',402 from dual union all
select 10187,'1963-S65','1963-02-26 00:00:00','suborbital rocket','failure',30,'','Mass spectrometer','','','Aeron',1869,'',200 from dual union all
select 10188,'1963-S650','1963-07-15 00:00:00','military missile','success',976,'','8K63 KY63-15 UBP 43 RD','','','Test',636,'',402 from dual union all
select 10189,'1963-S651','1963-07-28 00:00:00','military missile','success',977,'','8K63U UBP','Salvo','','Test',636,'',402 from dual union all
select 1019,'1951-A30','1951-06-15 00:00:00','atmospheric rocket','success',869,'490-69','Nike 490A 63R','','','Test',1895,'',10 from dual union all
select 10190,'1963-S652','1963-07-28 00:00:00','military missile','success',977,'','8K63U UBP','Salvo','','Test',636,'',402 from dual union all
select 10191,'1963-S653','1963-07-18 00:00:00','military missile','success',979,'','8K65 UBP-45 43 RD','','','Test',636,'',675 from dual union all
select 10192,'1963-S66','1963-02-26 01:31:43','military missile','success',916,'406','P-48','','','Test',256,'',180 from dual union all
select 10193,'1963-S67','1963-02-27 19:30:00','suborbital rocket','success',837,'NASA 14.86CI','GCA Ionos 11','','','Ionos/Eclipse test',1821,'',151 from dual union all
select 10194,'1963-S70','1963-03-01 00:00:00','military missile','success',837,'BMRS','','','','Reentry test',1895,'',100 from dual union all
select 10195,'1963-S71','1963-03-01 00:00:00','military missile','success',837,'BMRS','','','','Reentry test',1895,'',100 from dual union all
select 10196,'1963-S72','1963-02-28 09:02:48','military missile','success',154,'188D','Nike TV-22','NTMP K-10','PITCH PINE','ABM Test',1664,'',1800 from dual union all
select 10197,'1963-S73','1963-02-28 09:30:00','military missile','success',871,'ZK-13','K-10 Atlas Intercept','','','Test',873,'',70 from dual union all
select 10198,'1963-S74','1963-02-28 21:48:00','suborbital rocket','success',839,'NASA 10.59GA','GSFC Grenade 31','','','Aeronomy',428,'',113 from dual union all
select 10199,'1963-S75','1963-02-28 22:11:00','suborbital rocket','success',839,'NASA 10.53GA','GSFC Grenade 32','','','Aeronomy',1821,'',103 from dual union all
select 102,'1944-S02','1944-01-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1020,'1951-A31','1951-06-20 00:00:00','military missile','success',771,'GM-544','','','','Test',222,'',12 from dual union all
select 10200,'1963-S76','1963-03-01 00:00:00','military missile','success',976,'','8K63 PRO-16','','','Test',942,'',402 from dual union all
select 10201,'1963-S77','1963-03-01 01:20:00','suborbital rocket','success',1199,'SL84','LUX-9/UCL SUV-4/UCW-4','','','Ionos/Solar X/Bir-5',1839,'',209 from dual union all
select 10202,'1963-S78','1963-03-01 21:00:00','ballistic missile test','success',162,'134F','ABRES REX-I/Pod 1','','','Reentry Test',226,'',1800 from dual union all
select 10203,'1963-S79','1963-03-05 01:00:01','military missile','success',916,'405','P-49','','','Test',259,'',180 from dual union all
select 10204,'1963-S80','1963-03-04 12:42:00','military missile','success',982,'G 225   01-20T','8K64U UBP-13 KO','','','Test',566,'',1210 from dual union all
select 10205,'1963-S81','1963-03-06 22:30:00','suborbital rocket','success',30,'AC3.372','Mass spectrometer','','','Aeron',1869,'',228 from dual union all
select 10206,'1963-S82','1963-03-07 00:00:00','military missile','success',976,'','8K63 PRO-17','','','Test',942,'',402 from dual union all
select 10207,'1963-S83','1963-03-07 18:48:15','military missile','success',929,'A2P-120','','','','OT',421,'SSBN 618',1000 from dual union all
select 10208,'1963-S84','1963-03-08 19:30:00','suborbital rocket','success',837,'NASA 14.107GI','','','','Ionos',1821,'',155 from dual union all
select 10209,'1963-S85','1963-03-09 00:01:00','suborbital rocket','success',839,'NASA 10.60GA','GSFC Grenade 33','','','Aeronomy',428,'',117 from dual union all
select 1021,'1951-A32','1951-06-25 00:00:00','atmospheric rocket','success',365,'','X-3 Model','','','Test',1821,'',10 from dual union all
select 10210,'1963-S86','1963-03-09 00:01:00','suborbital rocket','success',839,'NASA 10.54GA','GSFC Grenade 34','','','Aeronomy',1821,'',119 from dual union all
select 10211,'1963-S87','1963-03-11 04:30:00','suborbital rocket','success',1199,'SL85','LUX-10/UCL SUV-5/UCW','','','Iono/Solar X/UCW-5/Bir-6',1839,'',209 from dual union all
select 10212,'1963-S88','1963-03-12 00:00:00','military missile','success',976,'','8K63 PRO-18','','','Test',942,'',402 from dual union all
select 10213,'1963-S89','1963-03-12 05:21:00','military missile','success',154,'64D','Mk 3 RV','OT','TALL TREE 2','OT',1666,'',1800 from dual union all
select 10214,'1963-S90','1963-03-12 09:50:00','suborbital rocket','success',535,'HAD 116','Sphere','','','Aeron',1835,'',124 from dual union all
select 10215,'1963-S91','1963-03-14 01:46:04','military missile','success',916,'407','P-50','','','Test',256,'',180 from dual union all
select 10216,'1963-S92','1963-03-14 00:00:00','military missile','failure',929,'A2','ORRT','','','OT',421,'SSBN 600',500 from dual union all
select 10217,'1963-S93','1963-03-14 00:00:00','military missile','success',929,'A2','ORRT','','','OT',421,'SSBN 600',1000 from dual union all
select 10218,'1963-S94','1963-03-14 00:00:00','military missile','success',929,'A2','ORRT','','','OT',421,'SSBN 600',1000 from dual union all
select 10219,'1963-S95','1963-03-14 00:00:00','military missile','success',1018,'E 15001-35','8K75 LKI 3-5','','','OT',584,'',1160 from dual union all
select 1022,'1951-A33','1951-06-26 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10220,'1963-S96','1963-03-14 01:01:49','test rocket','success',249,'203','ERCS 279L','','','Comms',1015,'',1000 from dual union all
select 10221,'1963-S97','1963-03-15 00:00:00','military missile','success',976,'','8K63 PRO-19','','','Test',942,'',402 from dual union all
select 10222,'1963-S98','1963-03-15 11:38:00','military missile','failure',154,'46D','Mk 3 RV','OT TALL TREE 1','TALL TREE 1','OT',1665,'',300 from dual union all
select 10223,'1963-S99','1963-03-16 00:00:00','military missile','success',976,'','8K63 KY63-4','','','Test',636,'',402 from dual union all
select 10224,'1963-U01','1963-07-30 16:16:08','ballistic missile test','success',247,'22-1/AD-622','OAR 22-1/AFCRL-1/21','AFCRL-1/21 (AD-622)','','Plasma/RF Astron',235,'',11100 from dual union all
select 10225,'1963-W01','1963-01-01 00:00:00','suborbital rocket','success',88,'','Parachute test','','','Test',1895,'',65 from dual union all
select 10226,'1963-W02','1963-01-01 00:00:00','suborbital rocket','success',88,'','Parachute test','','','Test',1895,'',65 from dual union all
select 10227,'1963-W03','1963-01-01 00:00:00','suborbital rocket','success',88,'','Parachute test','','','Test',1895,'',65 from dual union all
select 10228,'1963-W04','1963-01-09 03:20:00','suborbital rocket','success',88,'','McMurdo 16','','','Aeron',950,'',60 from dual union all
select 10229,'1963-W05','1963-01-16 04:45:00','suborbital rocket','success',88,'','McMurdo 17','','','Aeron',950,'',60 from dual union all
select 1023,'1951-A34','1951-06-27 00:00:00','atmospheric rocket','success',365,'','XF4D-1 Model','','','Test',1821,'',10 from dual union all
select 10230,'1963-W06','1963-01-18 17:18:00','suborbital rocket','success',88,'','Robin','','','Aeron',428,'',60 from dual union all
select 10231,'1963-W07','1963-02-12 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',51 from dual union all
select 10232,'1963-W08','1963-02-19 00:00:00','suborbital rocket','success',459,'Emma 05','','','','Aeron',671,'',60 from dual union all
select 10233,'1963-W09','1963-02-20 00:00:00','suborbital rocket','success',459,'Emma 06','','','','Aeron',671,'',60 from dual union all
select 10234,'1963-W10','1963-02-28 00:00:00','suborbital rocket','success',459,'Emma 01','','','','Aeron',671,'',55 from dual union all
select 10235,'1963-W11','1963-03-02 00:15:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 10236,'1963-W12','1963-04-01 00:00:00','suborbital rocket','success',1050,'5','','','','Weather',1895,'',67 from dual union all
select 10237,'1963-W13','1963-04-01 00:00:00','suborbital rocket','success',1050,'6','','','','Weather',1895,'',66 from dual union all
select 10238,'1963-W14','1963-05-01 00:00:00','suborbital rocket','success',1050,'7','','','','Weather',1895,'',63 from dual union all
select 10239,'1963-W15','1963-05-10 00:00:00','suborbital rocket','success',459,'Emma','','','','Aeron',671,'',18 from dual union all
select 1024,'1951-A35','1951-06-28 00:00:00','atmospheric rocket','success',869,'490-71','Nike 490A 64R','','','Test',1895,'',10 from dual union all
select 10240,'1963-W16','1963-05-10 00:00:00','suborbital rocket','success',459,'Emma','','','','Aeron',671,'',25 from dual union all
select 10241,'1963-W17','1963-05-10 00:00:00','suborbital rocket','success',577,'F5','','','','Test',1821,'',64 from dual union all
select 10242,'1963-W18','1963-05-16 03:40:00','suborbital rocket','success',88,'','McMurdo 21','','','Aeron',950,'',60 from dual union all
select 10243,'1963-W19','1963-05-23 00:00:00','suborbital rocket','success',88,'','','','','Ionos',197,'',60 from dual union all
select 10244,'1963-W20','1963-05-24 00:00:00','suborbital rocket','success',734,'MD-01/8 (108)','','','','Aeron',671,'',20 from dual union all
select 10245,'1963-W21','1963-05-29 00:00:00','suborbital rocket','success',734,'Meteo M1A 01','','','','Aeron',671,'',20 from dual union all
select 10246,'1963-W22','1963-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 10247,'1963-W23','1963-06-05 00:00:00','suborbital rocket','success',577,'F6','','','','Test',1821,'',63 from dual union all
select 10248,'1963-W24','1963-07-02 02:57:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 10249,'1963-W25','1963-07-29 03:48:00','suborbital rocket','success',88,'','McMurdo 22','','','Aeron',950,'',60 from dual union all
select 1025,'1951-A36','1951-06-28 21:43:00','suborbital rocket','failure',1470,'52','Blossom IVF','','','Solar UV,X/Aeron',1863,'',6 from dual union all
select 10250,'1963-W26','1963-08-23 03:03:00','suborbital rocket','success',88,'','McMurdo 24','','','Aeron',950,'',60 from dual union all
select 10251,'1963-W27','1963-08-28 18:48:00','suborbital rocket','success',88,'','Robin','','','Aeron',428,'',60 from dual union all
select 10252,'1963-W28','1963-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 10253,'1963-W29','1963-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 10254,'1963-W30','1963-09-17 03:30:00','suborbital rocket','success',88,'','McMurdo 25','','','Aeron',950,'',60 from dual union all
select 10255,'1963-W31','1963-09-18 02:30:00','suborbital rocket','success',88,'','McMurdo 26','','','Aeron',950,'',60 from dual union all
select 10256,'1963-W32','1963-09-18 18:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 10257,'1963-W33','1963-09-27 00:00:00','suborbital rocket','success',88,'','Arcas-Margo (Meteor)','','','Reentry Test',1821,'',51 from dual union all
select 10258,'1963-W34','1963-09-27 03:00:00','suborbital rocket','success',88,'','McMurdo 27','','','Aeron',950,'',60 from dual union all
select 10259,'1963-W35','1963-10-02 18:04:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 1026,'1951-A37','1951-06-29 00:00:00','atmospheric rocket','success',576,'1','','','','Test',1895,'',2 from dual union all
select 10260,'1963-W36','1963-10-07 18:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 10261,'1963-W37','1963-10-08 00:00:00','suborbital rocket','failure',459,'Emma 03','','','','Aeron',671,'',2 from dual union all
select 10262,'1963-W38','1963-10-10 00:00:00','suborbital rocket','success',459,'Emma 04','','','','Aeron',671,'',55 from dual union all
select 10263,'1963-W39','1963-11-13 20:45:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 10264,'1963-W40','1963-12-13 00:45:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 10265,'1963-W41','1963-02-06 05:35:00','suborbital rocket','success',88,'','McMurdo 18','','','Aeron',950,'',60 from dual union all
select 10266,'1963-W42','1963-02-27 04:30:00','suborbital rocket','success',88,'','McMurdo 19','','','Aeron',950,'',60 from dual union all
select 10267,'1963-W43','1963-04-24 01:00:00','suborbital rocket','success',88,'','McMurdo 20','','','Aeron',950,'',60 from dual;
insert into LAUNCH
select 10268,'1963-W44','1963-08-14 02:30:00','suborbital rocket','success',88,'','McMurdo 23','','','Aeron',950,'',60 from dual union all
select 10269,'1963-W45','1963-10-03 03:30:00','suborbital rocket','success',88,'','McMurdo 28','','','Aeron',950,'',60 from dual union all
select 1027,'1951-A38','1951-06-29 00:00:00','military missile','success',771,'GM-545','','','','Test',222,'',12 from dual union all
select 10270,'1963-W46','1963-01-01 00:00:00','suborbital rocket','success',1050,'8','','','','Weather',1895,'',65 from dual union all
select 10271,'1963-W47','1963-01-01 00:00:00','suborbital rocket','success',1050,'9','','','','Weather',1895,'',61 from dual union all
select 10272,'1963-W48','1963-01-01 00:00:00','suborbital rocket','success',1050,'10','','','','Weather',1895,'',67 from dual union all
select 10273,'1963-W49','1963-01-01 00:00:00','suborbital rocket','success',1050,'15','','','','Weather',1895,'',64 from dual union all
select 10274,'1963-Y01','1963-06-18 04:30:00','suborbital rocket','unknown',538,'','','','','Ionos',1837,'',88 from dual union all
select 10275,'1963-Y02','1963-07-18 00:00:00','suborbital rocket','success',577,'F7','','','','Test',1821,'',95 from dual union all
select 10276,'1963-Y03','1963-08-07 00:00:00','suborbital rocket','success',577,'F8','','','','Test',1821,'',95 from dual union all
select 10277,'1963-Y04','1963-08-09 00:00:00','suborbital rocket','success',577,'F9','','','','Test',1821,'',93 from dual union all
select 10278,'1963-Y05','1963-08-22 00:00:00','suborbital rocket','success',577,'F10','','','','Test',1821,'',90 from dual union all
select 10279,'1963-Y06','1963-10-08 19:00:00','suborbital rocket','success',257,'','UJO-1','','','Ionos',1895,'',89 from dual union all
select 1028,'1951-A39','1951-07-03 00:00:00','military missile','success',703,'240','','','','Test',222,'',8 from dual union all
select 10280,'1963-Y07','1963-10-10 16:00:00','suborbital rocket','success',257,'','UJO-2','','','Ionos',1895,'',81 from dual union all
select 10281,'1963-Y08','1963-10-15 15:09:00','suborbital rocket','unknown',538,'','','','','Aeron',1837,'',80 from dual union all
select 10282,'1963-Y09','1963-10-15 19:22:00','suborbital rocket','unknown',538,'','','','','Aeron',1837,'',80 from dual union all
select 10283,'1963-Y10','1963-10-29 00:00:00','suborbital rocket','success',577,'F12','','','','Aeron',1490,'',90 from dual union all
select 10284,'1963-Y11','1963-10-29 00:00:00','suborbital rocket','success',577,'F13','','','','Aeron',1490,'',90 from dual union all
select 10285,'1963-Y12','1963-10-29 00:00:00','suborbital rocket','success',577,'F14','','','','Aeron',1490,'',90 from dual union all
select 10286,'1963-Y13','1963-12-13 21:00:00','suborbital rocket','success',257,'','UJO-3','','','Ionos',1895,'',99 from dual union all
select 10287,'1964-001','1964-01-11 20:07:00','orbital','success',1394,'390/FTV 2354','POPPY 3/NRL Comp 4','POPPY 3/HAYLOFT','EMPTY POCKET','Sat TAG83',1679,'',null from dual union all
select 10288,'1964-002','1964-01-19 10:59:54','orbital','success',1392,'384/FTV 2303','OPS 3367A/3367B','P35-6','BENCH TOP','Sat TAG84',1674,'',null from dual union all
select 10289,'1964-003','1964-01-21 21:14:59','orbital','success',1377,'373/D23','Relay 2','Relay B','','Sat',234,'',null from dual union all
select 1029,'1951-A40','1951-07-09 00:00:00','atmospheric rocket','success',564,'','D27 Model','','','Test',1821,'',5 from dual union all
select 10290,'1964-004','1964-01-25 13:59:04','orbital','success',1391,'397/FTV 6301 (TA2)','Echo 2','Echo C','RAMS HORN','Sat TAG85',1673,'',null from dual union all
select 10291,'1964-005','1964-01-29 16:25:01','orbital','success',1119,'SA-5','SA-5','Jupiter Nosecone/Ballast','','Sat',271,'',null from dual union all
select 10292,'1964-006','1964-01-30 09:45:09','orbital','success',1512,'G103-18','Elektron-1/2','2D No. 1','','Sat',480,'',null from dual union all
select 10293,'1964-007','1964-01-30 15:49:09','deep space','success',145,'199D (AA8)','Ranger 6','RA-6','','Sat',227,'',null from dual union all
select 10294,'1964-008','1964-02-15 21:38:23','orbital','success',1394,'389/FTV 1174','CORONA 76/1004','CORONA J-5','GARDEN PARTY','Sat TAG86',1678,'',null from dual union all
select 10295,'1964-009','1964-02-25 18:59:47','orbital','success',146,'285D','AFP-206','GAMBIT SV 955','UPPER OCTANE','Sat',1021,'',null from dual union all
select 10296,'1964-010','1964-02-27 13:26:00','orbital','success',681,'','Kosmos-25','DS-P1 No. 4','','Sat',630,'',null from dual union all
select 10297,'1964-011','1964-02-28 03:20:00','orbital','success',1394,'402/FTV 2316','OPS 3722','Ferret 5','FIRST QUARTER','Sat TAG87',1679,'',null from dual union all
select 10298,'1964-012','1964-03-11 20:14:24','orbital','success',146,'296D','AFP-206','GAMBIT SV 956','INK BLOTTER','Sat',1021,'',null from dual union all
select 10299,'1964-013','1964-03-18 15:07:00','orbital','success',681,'','Kosmos-26','DS-MG No. 1','','Sat',630,'',null from dual union all
select 103,'1944-S03','1944-01-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1030,'1951-A41','1951-07-14 00:00:00','atmospheric rocket','success',869,'490-72','Nike 490A 65R','','','Test',1895,'',10 from dual union all
select 10300,'1964-014','1964-03-27 03:24:42','orbital','failure',801,'T 15000- 22','Kosmos-27','3MV-1 No. 3','','Sat',480,'',208 from dual union all
select 10301,'1964-015','1964-03-27 17:25:23','orbital','success',1144,'S127R','Ariel 2','UK-C','','Sat',1812,'',null from dual union all
select 10302,'1964-016','1964-04-02 02:42:40','deep space','success',801,'G 15000- 28/23','Zond','3MV-1 No. 4','','Sat',480,'',null from dual union all
select 10303,'1964-017','1964-04-04 09:45:00','orbital','success',1509,'G 15001- 04','Kosmos-28','Zenit-2 No. 16','','Sat',562,'',null from dual union all
select 10304,'1964-018','1964-04-08 16:00:01','orbital','success',1421,'GT-1','Gemini 1','Gemini SC1','','Sat',237,'',null from dual union all
select 10305,'1964-019','1964-04-12 09:30:00','orbital','success',1253,'T 15001- 01A','Polyot-2','I-1B No. 112','','Sat',562,'',null from dual union all
select 10306,'1964-020','1964-04-23 18:48:00','orbital','success',146,'351D','AFP-206','GAMBIT SV 957','ANCHOR DAN','Sat',1021,'',null from dual union all
select 10307,'1964-021','1964-04-25 10:21:00','orbital','success',1509,'R 15001- 01','Kosmos-29','Zenit-2 No. 19','','Sat',562,'',null from dual union all
select 10308,'1964-022','1964-04-27 23:23:43','orbital','success',1394,'395/FTV 1604','CORONA 78/1005','CORONA J-8','NICE BIRD','Sat TAG89',1678,'',null from dual union all
select 10309,'1964-023','1964-05-18 09:42:00','orbital','success',1508,'G 15000- 12','Kosmos-30','Zenit-4 No. 2','','Sat',480,'',null from dual union all
select 1031,'1951-A42','1951-07-17 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10310,'1964-024','1964-05-19 19:21:14','orbital','success',146,'350D','AFP-206','GAMBIT SV 958','BIG FRED','Sat',1021,'',null from dual union all
select 10311,'1964-025','1964-05-28 17:07:00','orbital','success',1119,'SA-6','AS-106','Apollo BP-13 CM','','Sat',271,'',null from dual union all
select 10312,'1964-026','1964-06-04 03:50:55','orbital','success',1148,'S125R','Transit VC (N-4)','Transit VC','ARROW PLANT','Sat',1018,'',null from dual union all
select 10313,'1964-027','1964-06-04 22:59:00','orbital','success',1394,'403/FTV 1176','CORONA 79/1006','CORONA J-9','KICK BALL','Sat TAG90',1019,'',null from dual union all
select 10314,'1964-028','1964-06-06 06:00:00','orbital','success',681,'','Kosmos-31','DS-MT No. 2','','Sat',630,'',null from dual union all
select 10315,'1964-029','1964-06-10 11:00:00','orbital','success',1509,'R 15001- 02','Kosmos-32','Zenit-2 No. 18','','Sat',562,'',null from dual union all
select 10316,'1964-030','1964-06-13 15:47:00','orbital','success',1394,'408/FTV 1606','CORONA 80/9063A','ARGON 21','BEAGLE HOUND','Sat TAG91',1674,'',null from dual union all
select 10317,'1964-031','1964-06-18 04:56:08','orbital','success',1392,'407/FTV 2304','OPS 4467A/4467B','P35-8','INDIO IKE?','Sat TAG92',1678,'',null from dual union all
select 10318,'1964-032','1964-06-19 23:18:07','orbital','success',1394,'410/FTV 1609','CORONA 81/1007','CORONA J-7','GREEN DOOR','Sat TAG93',1673,'',null from dual union all
select 10319,'1964-033','1964-06-23 10:00:00','orbital','success',1509,'G 15001- 05','Kosmos-33','Zenit-2 No. 20','','Sat',562,'',null from dual union all
select 1032,'1951-A43','1951-07-18 00:00:00','atmospheric rocket','failure',442,'Double Deacon','X-2 Model','','','Test',1821,'',0 from dual union all
select 10320,'1964-034','1964-07-01 11:00:00','orbital','success',1508,'T 15000- 04','Kosmos-34','Zenit-4 No. 3','','Sat',480,'',null from dual union all
select 10321,'1964-035','1964-07-02 23:59:56','orbital','success',1394,'409/FTV 2315','OPS 3395','Ferret 6','FIRING ORDER','Sat TAG94',1679,'',null from dual union all
select 10322,'1964-036','1964-07-06 18:51:18','orbital','success',146,'352D','AFP-206','GAMBIT SV 959','QUARTER ROUND','Sat',1021,'',null from dual union all
select 10323,'1964-037','1964-07-10 23:14:04','orbital','success',1394,'404/FTV 1177','CORONA 82/1008','CORONA J-10','OLD HAT','Sat TAG95',1019,'',null from dual union all
select 10324,'1964-038','1964-07-10 21:51:02','orbital','success',1512,'G103-19','Elektron-3/4','2D No. 3','','Sat',480,'',null from dual union all
select 10325,'1964-039','1964-07-15 11:27:00','orbital','success',1509,'R 15001- 03','Kosmos-35','Zenit-2 No. 21','','Sat',562,'',null from dual union all
select 10326,'1964-040','1964-07-17 08:22:00','orbital','success',146,'216D','Vela 2A/2B','Vela 2A','','Sat',228,'',null from dual union all
select 10327,'1964-041','1964-07-28 16:50:07','deep space','success',145,'250D (AA9)','Ranger 7','Ranger 7','','Sat',227,'',null from dual union all
select 10328,'1964-042','1964-07-30 03:36:00','orbital','success',681,'','Kosmos-36','DS-P1-Yu No. 1','','Sat',630,'',null from dual union all
select 10329,'1964-043','1964-08-05 23:15:35','orbital','success',1394,'413/FTV 1605','CORONA 83/1009','CORONA J-12','LONG LOOP','Sat TAG96',1678,'',null from dual union all
select 1033,'1951-A44','1951-07-18 00:00:00','atmospheric rocket','unknown',576,'2','','','','Test',1895,'',2 from dual union all
select 10330,'1964-044','1964-08-14 09:30:00','orbital','success',1509,'R 15001- 04','Kosmos-37','Zenit-2 No. 22','','Sat',562,'',null from dual union all
select 10331,'1964-045','1964-08-14 22:00:13','orbital','success',177,'7101','AFP-206','GAMBIT SV 960','BIG SICKLE','Sat',1773,'',null from dual union all
select 10332,'1964-046','1964-08-18 09:15:00','orbital','success',682,'4000-000-02L','Kosmos-38/40','Strela-1','','Sat',564,'',null from dual union all
select 10333,'1964-047','1964-08-19 12:15:02','orbital','success',1380,'417/D25','Syncom 3','Syncom C','','Sat',233,'',null from dual union all
select 10334,'1964-048','1964-08-21 15:45:00','orbital','success',1394,'412/FTV 1603','CORONA 84/9064A','ARGON 22','KILO KATE','Sat TAG97',1674,'',null from dual union all
select 10335,'1964-049','1964-08-22 07:21:00','orbital','success',802,'G 15000- 19/R10336','Kosmos-41','Molniya-1 11F67 No. 1','','Sat',480,'',null from dual union all
select 10336,'1964-050','1964-08-22 11:00:00','orbital','success',681,'','Kosmos-42/43','Strela-1','','Sat',630,'',null from dual union all
select 10337,'1964-051','1964-08-25 13:43:00','orbital','success',1148,'S134R','Explorer 20','NASA S-48','HURRY BABY','Sat',1769,'',null from dual union all
select 10338,'1964-052','1964-08-28 07:56:57','orbital','success',1391,'399/FTV 6201 (TA3)','Nimbus 1','Nimbus A','HULU MOON','Sat TAG98',1673,'',null from dual union all
select 10339,'1964-053','1964-08-28 16:00:00','orbital','success',1510,'T 15000- 05','Kosmos-44','Meteor 11F614 No. 1','','Sat',562,'',null from dual union all
select 1034,'1951-A45','1951-07-18 00:00:00','military missile','success',771,'GM-546','','','','Test',222,'',12 from dual union all
select 10340,'1964-054','1964-09-05 01:23:00','orbital','success',145,'195D (AA10)','OGO 1','OGO A','','Sat',227,'',null from dual union all
select 10341,'1964-055','1964-09-13 09:45:00','orbital','success',1508,'R 15001- 01','Kosmos-45','Zenit-4 No. 5','','Sat',480,'',null from dual union all
select 10342,'1964-056','1964-09-14 22:53:43','orbital','success',1394,'405/FTV 1178','CORONA 85/1010','CORONA J-11','QUIT CLAIM','Sat TAG99',1770,'',null from dual union all
select 10343,'1964-057','1964-09-18 16:22:43','orbital','success',1119,'SA-7','AS-107','Apollo BP-15 CM','','Sat',271,'',null from dual union all
select 10344,'1964-058','1964-09-23 20:06:00','orbital','success',177,'7102','AFP-206','GAMBIT SV 962','SLOW PACE','Sat',1773,'',null from dual union all
select 10345,'1964-059','1964-09-24 12:04:00','orbital','success',1509,'R 15001- 05','Kosmos-46','Zenit-2 No. 23','','Sat',562,'',null from dual union all
select 10346,'1964-060','1964-10-04 03:45:00','orbital','success',1378,'392/D26','Explorer 21','IMP B','','Sat',233,'',null from dual union all
select 10347,'1964-061','1964-10-05 21:50:14','orbital','success',1394,'421/FTV 1170','CORONA 86/1011','CORONA J-3','SOLID PACK','Sat TAG100',1678,'',null from dual union all
select 10348,'1964-062','1964-10-06 07:00:00','orbital','success',1508,'R 15000- 02','Kosmos-47','Voskhod 3KV No. 2','','Sat',480,'',null from dual union all
select 10349,'1964-063','1964-10-06 17:04:21','orbital','success',1358,'423/AB016','NNS O-1','NNS O-1','AIR ALARM','Sat',1674,'',null from dual union all
select 1035,'1951-A46','1951-07-20 00:00:00','military missile','failure',703,'239','','','','Test',222,'',0 from dual union all
select 10350,'1964-064','1964-10-10 03:01:00','orbital','success',1148,'S123R','Explorer 22','Beacon Explorer B','GUS GOOSE','Sat',1769,'',null from dual union all
select 10351,'1964-065','1964-10-12 07:30:01','orbital','success',1508,'R 15000- 04','Voskhod','Voskhod 3KV No. 3','','Sat',480,'',null from dual union all
select 10352,'1964-066','1964-10-14 10:00:00','orbital','success',1509,'R 15002- 01','Kosmos-48','Zenit-2 No. 24','','Sat',562,'',null from dual union all
select 10353,'1964-067','1964-10-17 22:02:23','orbital','success',1394,'418/FTV 1179','CORONA 87/1012','CORONA J-13','MOOSE HORN','Sat TAG101',1770,'',null from dual union all
select 10354,'1964-068','1964-10-23 18:27:00','orbital','success',146,'353D','AFP-206','GAMBIT SV 963','BOON DOCKER','Sat',1772,'',null from dual union all
select 10355,'1964-069','1964-10-24 05:17:00','orbital','success',681,'','Kosmos-49','DS-MG No. 2','','Sat',630,'',null from dual union all
select 10356,'1964-070','1964-10-28 10:40:00','orbital','success',1509,'R 15002- 02','Kosmos-50','Zenit-2 No. 25','','Sat',562,'',null from dual union all
select 10357,'1964-071','1964-11-02 21:30:20','orbital','success',1394,'420/FTV 1173','CORONA 88/1013','CORONA J-15','BROWN MOOSE','Sat TAG102',1678,'',null from dual;
insert into LAUNCH
select 10358,'1964-072','1964-11-04 02:12:11','orbital','success',1394,'430/FTV 2317','OPS 3062','Ferret 7','ECHO HOLE','Sat TAG103',1679,'',null from dual union all
select 10359,'1964-073','1964-11-05 19:22:05','deep space','success',146,'289D (AA11)','Mariner 3','Mariner C-2','','Sat',228,'',null from dual union all
select 1036,'1951-A47','1951-07-25 00:00:00','atmospheric rocket','unknown',576,'3','','','','Test',1895,'',2 from dual union all
select 10360,'1964-074','1964-11-06 12:02:01','orbital','success',1148,'S133R','Explorer 23','NASA S-55C','','Sat',1812,'',null from dual union all
select 10361,'1964-075','1964-11-18 20:35:54','orbital','success',1394,'416/FTV 1180','CORONA 89/1014','CORONA J-16','VERBAL VENTURE','Sat TAG104',1673,'',null from dual union all
select 10362,'1964-076','1964-11-21 17:09:39','orbital','success',1148,'S135R','Explorer 24/25','AD-B','IMA BIRD','Sat',1769,'',null from dual union all
select 10363,'1964-077','1964-11-28 14:22:01','deep space','success',146,'288D (AA12)','Mariner 4','Mariner C-3','','Sat',227,'',null from dual union all
select 10364,'1964-078','1964-11-30 13:25:00','deep space','success',802,'G 15000- 29','Zond-2','3MV-4A No. 2','','Sat',480,'',null from dual union all
select 10365,'1964-079','1964-12-04 18:57:00','orbital','success',177,'7105','AFP-206','GAMBIT SV 964','BATTLE ROYAL','Sat',1773,'',null from dual union all
select 10366,'1964-080','1964-12-09 23:00:00','orbital','success',681,'','Kosmos-51','DS-MT No. 3','','Sat',626,'',null from dual union all
select 10367,'1964-081','1964-12-10 16:53:33','orbital','success',1423,'3A-1','Transtage','Lead Dummy Payload','','Sat',239,'',null from dual union all
select 10368,'1964-082','1964-12-11 14:25:02','orbital','success',152,'AC-4','Surveyor Model','Surveyor Model','','Sat',268,'',null from dual union all
select 10369,'1964-083','1964-12-13 00:08:10','orbital','success',1358,'427/AB017','NNS O-2','APL SN-43','ASTRO ANNIE','Sat',1674,'',null from dual union all
select 1037,'1951-A48','1951-07-27 00:00:00','atmospheric rocket','success',564,'','Hermes A3A Model','','','Test',1821,'',5 from dual union all
select 10370,'1964-084','1964-12-15 20:20:04','orbital','success',1148,'S137R','San Marco 1','San Marco 1','','Sat',1814,'',null from dual union all
select 10371,'1964-085','1964-12-19 21:10:16','orbital','success',1394,'424/FTV 1607','CORONA 90/1015','CORONA J-17','UTILITY TOOL','Sat TAG105',1678,'',null from dual union all
select 10372,'1964-086','1964-12-21 09:00:03','orbital','success',1378,'393/D27','Explorer 26','EPE-D','','Sat',233,'',null from dual union all
select 10373,'1964-087','1964-12-21 19:08:56','orbital','success',1394,'425/FTV 2355','QUILL','QUILL','BARN OWL','Sat TAG106',1673,'',null from dual union all
select 10374,'1964-A01','1964-01-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 10375,'1964-A02','1964-01-08 20:10:31','suborbital spaceplane','success',1520,'1-43-69','SAS Off Stability','','','Test',379,'NB-52 008',42 from dual union all
select 10376,'1964-A03','1964-01-13 23:39:00','military missile','failure',929,'A2P-226','PY-1 DASO','','','OT',421,'SSBN 623',2 from dual union all
select 10377,'1964-A04','1964-01-15 22:34:00','suborbital rocket','failure',837,'NASA 14.38CA','GCA Release 29 Na','','','Aeronomy',1821,'',45 from dual union all
select 10378,'1964-A05','1964-01-16 18:03:30','suborbital spaceplane','success',1520,'3-25-42','Sharp Upper Vert','','','Test',379,'NB-52 008',21 from dual union all
select 10379,'1964-A06','1964-01-22 05:36:00','military missile','success',1018,'G 15001-02 (6TS)','8K75 LKI 3-23','','','OT',580,'',1160 from dual union all
select 1038,'1951-A49','1951-08-01 00:00:00','atmospheric rocket','unknown',576,'4','','','','Test',1895,'',2 from dual union all
select 10380,'1964-A07','1964-01-22 18:52:00','atmospheric rocket','success',835,'Smoke 381','','','','Aeronomy',293,'',18 from dual union all
select 10381,'1964-A08','1964-01-22 20:27:00','atmospheric rocket','success',835,'Smoke 040','','','','Aeronomy',1821,'',22 from dual union all
select 10382,'1964-A09','1964-01-23 00:00:00','suborbital rocket','failure',864,'Sandia 152-11','LRL 12" Payload Test','','','Test',855,'',5 from dual union all
select 10383,'1964-A10','1964-01-24 00:36:00','suborbital rocket','failure',839,'NASA 10.86GA','GSFC Grenade 37 (1C)','','','Aeronomy',428,'',21 from dual union all
select 10384,'1964-A100','1964-07-02 00:00:00','suborbital spaceplane','failure',1520,'3- A-49','','','','Test',379,'NB-52',14 from dual union all
select 10385,'1964-A102','1964-07-06 19:45:00','atmospheric rocket','success',835,'Smoke 403','','','','Aeronomy',293,'',17 from dual union all
select 10386,'1964-A103','1964-07-07 19:30:00','military missile','failure',794,'663','Mk 5 RV','OT','FIVE POINTS','OT',1700,'',10 from dual union all
select 10387,'1964-A104','1964-07-08 00:00:00','suborbital rocket','failure',1349,'Sandia 152-63','Terihawk-12','','','Aeron?',855,'',1 from dual union all
select 10388,'1964-A105','1964-07-15 00:00:00','suborbital rocket','failure',1186,'LW-4','','','','Test',1895,'',9 from dual union all
select 10389,'1964-A106','1964-07-15 18:20:00','atmospheric rocket','success',835,'Smoke 404','','','','Aeronomy',293,'',16 from dual union all
select 1039,'1951-A50','1951-08-02 00:00:00','atmospheric rocket','success',365,'','Boojum Model','','','Test',1821,'',10 from dual union all
select 10390,'1964-A107','1964-07-21 00:00:00','military missile','failure',976,'','8K63 60 RD UBP-7','','','Test',1354,'',0 from dual union all
select 10391,'1964-A108','1964-07-24 02:07:00','suborbital rocket','success',759,'MT-135-001','','','','Aeron',839,'',49 from dual union all
select 10392,'1964-A109','1964-07-27 00:00:00','suborbital rocket','success',506,'','','','','Meteo',1040,'',35 from dual union all
select 10393,'1964-A11','1964-01-24 19:00:00','atmospheric rocket','success',835,'Smoke 382','','','','Aeronomy',293,'',18 from dual union all
select 10394,'1964-A110','1964-07-28 00:00:00','suborbital spaceplane','failure',1520,'3- A-51','','','','Test',379,'NB-52',14 from dual union all
select 10395,'1964-A111','1964-07-28 02:08:00','suborbital rocket','success',1069,'PT-160-1','Propellant Test','','','Test',839,'',44 from dual union all
select 10396,'1964-A112','1964-07-28 18:00:00','atmospheric rocket','success',835,'Smoke 405','','','','Aeronomy',293,'',14 from dual union all
select 10397,'1964-A113','1964-07-29 18:00:00','atmospheric rocket','success',835,'Smoke 406','','','','Aeronomy',293,'',19 from dual union all
select 10398,'1964-A114','1964-07-29 19:55:19','suborbital spaceplane','success',1520,'3-31-52','Heat tran/Local Flow','','','Test',379,'NB-52 003',23 from dual union all
select 10399,'1964-A115','1964-07-30 21:17:00','atmospheric rocket','success',835,'Smoke 056','','','','Aeronomy',1821,'',16 from dual union all
select 104,'1944-S04','1944-01-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1040,'1951-A51','1951-08-07 00:00:00','atmospheric rocket','unknown',576,'5','','','','Test',1895,'',2 from dual union all
select 10400,'1964-A116','1964-07-31 00:00:00','suborbital rocket','failure',1186,'LW-5','','','','Test',1895,'',5 from dual union all
select 10401,'1964-A117','1964-08-05 02:07:00','suborbital rocket','success',839,'NASA 10.114GA','GSFC Grenade 50','','','Aeronomy',176,'',28 from dual union all
select 10402,'1964-A118','1964-08-06 00:00:00','suborbital rocket','failure',1186,'LW-6','','','','Test',1895,'',1 from dual union all
select 10403,'1964-A119','1964-08-11 02:00:05','military missile','failure',999,'G 22500-15L','8K67 LKI-12','','','Test',574,'',0 from dual union all
select 10404,'1964-A12','1964-01-25 17:15:00','military missile','failure',999,'G 22500-06L','8K67 LKI-3','','','Test',573,'',0 from dual union all
select 10405,'1964-A120','1964-08-12 18:12:33','suborbital spaceplane','success',1520,'3-32-53','Heat tran/LF/BLN','','','Test',379,'NB-52 003',24 from dual union all
select 10406,'1964-A121','1964-08-14 17:54:19','suborbital spaceplane','success',1520,'2-33-56','Stability','','','Test',379,'NB-52 003',31 from dual union all
select 10407,'1964-A122','1964-08-14 22:54:00','atmospheric rocket','success',835,'Smoke 057','','','','Aeronomy',1821,'',17 from dual union all
select 10408,'1964-A123','1964-08-19 18:11:00','atmospheric rocket','success',835,'Smoke 058','','','','Aeronomy',1821,'',17 from dual union all
select 10409,'1964-A124','1964-08-25 00:30:00','suborbital rocket','success',1349,'Sandia 152-68','TT-12 no igniter','','','Test',1490,'',30 from dual union all
select 1041,'1951-A52','1951-08-08 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10410,'1964-A125','1964-08-26 18:42:07','suborbital spaceplane','success',1520,'3-33-54','HT/LF/BLN','','','Test',379,'NB-52 003',27 from dual union all
select 10411,'1964-A126','1964-09-02 16:45:00','military missile','failure',983,'KY-2','','','','Test',636,'',0 from dual union all
select 10412,'1964-A127','1964-09-03 14:54:00','suborbital rocket','success',88,'T1-1676','Biospace 5','','','Bio',1821,'',35 from dual union all
select 10413,'1964-A128','1964-09-03 17:54:54','suborbital spaceplane','success',1520,'3-34-55','HT/BLN','','','Test',379,'NB-52 003',23 from dual union all
select 10414,'1964-A129','1964-09-03 18:36:00','atmospheric rocket','success',835,'Smoke 059','','','','Aeronomy',1821,'',16 from dual union all
select 10415,'1964-A13','1964-01-26 16:47:03','military missile','failure',929,'A2P-227','','','','OT',421,'SSBN 623',5 from dual union all
select 10416,'1964-A130','1964-09-04 17:01:00','atmospheric rocket','success',835,'Smoke 060','','','','Aeronomy',1821,'',16 from dual union all
select 10417,'1964-A131','1964-09-08 03:43:00','suborbital rocket','failure',215,'AK-2A-59','','','','Ionos',428,'',20 from dual union all
select 10418,'1964-A132','1964-09-09 17:35:00','atmospheric rocket','success',835,'Smoke 061','','','','Aeronomy',1821,'',15 from dual union all
select 10419,'1964-A133','1964-09-15 17:58:00','atmospheric rocket','success',835,'Smoke 062','','','','Aeronomy',1821,'',19 from dual union all
select 1042,'1951-A53','1951-08-10 00:00:00','military missile','success',771,'GM-548','','','','Test',222,'',12 from dual union all
select 10420,'1964-A134','1964-09-16 00:00:00','suborbital rocket','failure',1186,'LW-10','','','','Test',1895,'',2 from dual union all
select 10421,'1964-A135','1964-09-23 00:00:00','suborbital spaceplane','failure',1520,'3- A-56','','','','Test/Tech',379,'NB-52',14 from dual union all
select 10422,'1964-A136','1964-09-23 21:16:00','suborbital spaceplane','success',1520,'3-35-57','MA-45R TPS, BLN','','','Test/Tech',379,'NB-52 003',29 from dual union all
select 10423,'1964-A137','1964-09-25 20:12:00','atmospheric rocket','success',835,'Smoke 063','','','','Aeronomy',1821,'',18 from dual union all
select 10424,'1964-A138','1964-09-26 16:35:00','suborbital rocket','success',88,'T1-1746','Biospace 6','','','Bio',1821,'',35 from dual union all
select 10425,'1964-A139','1964-09-29 21:22:49','military missile','failure',795,'445','Mk 11 RV','R&D','','Test',264,'',1 from dual union all
select 10426,'1964-A14','1964-01-28 20:11:36','suborbital spaceplane','success',1520,'1-44-70','Stability','','','Test',379,'NB-52 008',32 from dual union all
select 10427,'1964-A140','1964-09-29 21:00:13','suborbital spaceplane','success',1520,'2-34-57','Stab.,Star tracker','','','Test',379,'NB-52 008',29 from dual union all
select 10428,'1964-A141','1964-10-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 10429,'1964-A142','1964-10-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 1043,'1951-A54','1951-08-17 00:00:00','atmospheric rocket','success',365,'','E15 Model','','','Test',1821,'',10 from dual union all
select 10430,'1964-A143','1964-10-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 10431,'1964-A144','1964-10-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 10432,'1964-A145','1964-10-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 10433,'1964-A146','1964-10-02 00:00:00','atmospheric rocket','failure',617,'J201','HRV 920/Dummy St3','','','Test',1836,'',30 from dual union all
select 10434,'1964-A147','1964-10-02 00:00:00','suborbital spaceplane','failure',1520,'1- A-78','','','','Test',379,'NB-52',14 from dual union all
select 10435,'1964-A148','1964-10-04 08:00:00','military missile','failure',135,'D001','Subscale RV','ABRES C-1 (USAF)','','Reentry Test',613,'',0 from dual union all
select 10436,'1964-A15','1964-01-29 19:19:00','atmospheric rocket','success',835,'Smoke 041','','','','Aeronomy',1821,'',20 from dual union all
select 10437,'1964-A150','1964-10-09 00:00:00','suborbital rocket','failure',1186,'LW-14','','','','Test',1895,'',23 from dual union all
select 10438,'1964-A151','1964-10-13 19:26:00','atmospheric rocket','success',835,'Smoke 064','','','','Aeronomy',1821,'',18 from dual union all
select 10439,'1964-A152','1964-10-15 21:15:40','suborbital spaceplane','success',1520,'1-50-79','IFDS/Tip Pod/AD-SB','','','Test/Aeron',379,'NB-52 008',25 from dual union all
select 1044,'1951-A55','1951-08-21 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10440,'1964-A153','1964-10-16 00:00:00','suborbital rocket','failure',273,'05','','','','Test',394,'',0 from dual union all
select 10441,'1964-A154','1964-10-16 00:00:00','suborbital rocket','failure',1349,'Sandia 152-70','Terihawk-12','','','Aeron?',855,'',0 from dual union all
select 10442,'1964-A155','1964-10-20 00:00:00','military missile','failure',458,'','','','','Test',669,'',0 from dual union all
select 10443,'1964-A156','1964-10-20 00:00:00','suborbital rocket','failure',1431,'Sandia 152-69','Ballistic','','','Test/Aeron?',1490,'',0 from dual union all
select 10444,'1964-A157','1964-10-22 16:56:00','atmospheric rocket','success',835,'Smoke 065','','','','Aeronomy',1821,'',19 from dual union all
select 10445,'1964-A158','1964-10-29 00:00:00','suborbital spaceplane','success',1520,'3- C-58','Gear mod check','','','Test',379,'NB-52',14 from dual union all
select 10446,'1964-A159','1964-10-30 17:51:52','suborbital spaceplane','success',1520,'3-36-59','Gear mod, BLN','','','Test',379,'NB-52 008',25 from dual union all
select 10447,'1964-A16','1964-01-29 23:57:00','suborbital rocket','failure',88,'','Deltasonde for 10.89','','','Aeron',428,'',1 from dual;
insert into LAUNCH
select 10448,'1964-A160','1964-10-30 18:28:00','atmospheric rocket','success',835,'Smoke 066','','','','Aeronomy',1821,'',15 from dual union all
select 10449,'1964-A161','1964-11-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 1045,'1951-A56','1951-08-22 00:00:00','atmospheric rocket','success',564,'','D32 Model','','','Test',1821,'',5 from dual union all
select 10450,'1964-A162','1964-11-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 10451,'1964-A163','1964-11-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 10452,'1964-A164','1964-11-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 10453,'1964-A165','1964-11-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 10454,'1964-A166','1964-11-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 10455,'1964-A167','1964-11-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 10456,'1964-A168','1964-11-02 00:00:00','atmospheric rocket','success',1474,'','','','','Test',669,'',20 from dual union all
select 10457,'1964-A169','1964-11-04 00:00:00','suborbital rocket','failure',1186,'LW-12','','','','Test',1895,'',1 from dual union all
select 10458,'1964-A17','1964-02-04 18:20:00','atmospheric rocket','success',835,'Smoke 042','','','','Aeronomy',1821,'',19 from dual union all
select 10459,'1964-A170','1964-11-06 00:00:00','suborbital spaceplane','success',1520,'2- C-58','','','','Test',379,'NB-52',14 from dual union all
select 1046,'1951-A57','1951-08-30 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10460,'1964-A171','1964-11-13 20:10:00','atmospheric rocket','success',835,'Smoke 068','','','','Aeronomy',1821,'',19 from dual union all
select 10461,'1964-A172','1964-11-16 00:00:00','suborbital spaceplane','success',1520,'2- C-59','','','','Test',379,'NB-52',14 from dual union all
select 10462,'1964-A173','1964-11-19 18:34:00','suborbital rocket','failure',837,'NASA 14.29GA','UM Pitot 7','','','Aeronomy',154,'CVE-25',14 from dual union all
select 10463,'1964-A174','1964-11-24 00:00:00','suborbital rocket','failure',850,'','','','','Aeron?',394,'',1 from dual union all
select 10464,'1964-A175','1964-11-24 18:10:00','atmospheric rocket','success',835,'Smoke 069','','','','Aeronomy',1821,'',18 from dual union all
select 10465,'1964-A176','1964-11-28 00:00:00','military missile','failure',135,'C003','Mk 11 decoy','ABRES B-4 (USAF)','','Reentry Test',614,'',0 from dual union all
select 10466,'1964-A177','1964-11-30 20:09:32','suborbital spaceplane','success',1520,'2-35-60','Gear mod, Stab,ST','','','Test',379,'NB-52 008',26 from dual union all
select 10467,'1964-A178','1964-12-01 20:53:00','atmospheric rocket','success',835,'Smoke 070','','','','Aeronomy',1821,'',17 from dual union all
select 10468,'1964-A179','1964-12-04 00:00:00','suborbital spaceplane','failure',1520,'1- A-80','','','','Test',379,'NB-52',14 from dual union all
select 10469,'1964-A18','1964-02-05 06:00:00','military missile','failure',983,'KY-2','','','','Test',636,'',0 from dual union all
select 1047,'1951-A58','1951-08-31 00:00:00','military missile','success',771,'GM-549','','','','Test',222,'',12 from dual union all
select 10470,'1964-A180','1964-12-08 15:00:00','test rocket','success',712,'LJII-3 (12-51-1)','A-002','','','SC Test',1873,'',5 from dual union all
select 10471,'1964-A181','1964-12-08 18:13:00','atmospheric rocket','success',835,'Smoke 071','','','','Aeronomy',1821,'',20 from dual union all
select 10472,'1964-A182','1964-12-09 18:36:17','suborbital spaceplane','success',1520,'3-37-60','3D Flow Field,MA-45R','','','Test/Tech',379,'NB-52 008',28 from dual union all
select 10473,'1964-A183','1964-12-10 00:00:00','atmospheric rocket','success',1065,'Ra28','Ranger 28','','','Test',2,'',20 from dual union all
select 10474,'1964-A184','1964-12-10 19:10:26','suborbital spaceplane','success',1520,'1-51-81','IFDS/MIT/Air Den','','','Test/Aeron',379,'NB-52 003',34 from dual union all
select 10475,'1964-A185','1964-12-11 18:00:00','suborbital rocket','success',44,'NASA 12.02GT','Dummy sustainer','','','Test',1821,'',2 from dual union all
select 10476,'1964-A186','1964-12-22 18:44:52','suborbital spaceplane','success',1520,'3-38-61','Friction,GE TPS.,3D','','','Test/Tech',379,'NB-52 003',24 from dual union all
select 10477,'1964-A187','1964-04-25 00:00:00','suborbital rocket','failure',706,'01','LEX-01','','','Test',313,'',1 from dual union all
select 10478,'1964-A188','1963-11-06 00:00:00','military missile','success',1256,'1','Squirt MTV-1','','','Test',1883,'',3 from dual union all
select 10479,'1964-A189','1964-03-17 00:00:00','military missile','success',1256,'2','Squirt MTV-2','','','Test',1883,'',3 from dual union all
select 1048,'1951-A59','1951-09-05 00:00:00','atmospheric rocket','success',365,'','E7 Model','','','Test',1821,'',10 from dual union all
select 10480,'1964-A19','1964-02-06 00:00:00','suborbital rocket','failure',549,'HAD-2','LRL HAD-2','','','Solar X',1015,'',5 from dual union all
select 10481,'1964-A190','1964-07-14 00:00:00','military missile','success',1256,'3','Squirt MTV-3','','','Test',1883,'',3 from dual union all
select 10482,'1964-A191','1964-01-23 23:00:00','suborbital rocket','success',1188,'','Wind,Temp','','','Aeron',1447,'',40 from dual union all
select 10483,'1964-A192','1964-01-01 00:00:00','suborbital rocket','failure',1050,'16','','','','Weather',1895,'',33 from dual union all
select 10484,'1964-A193','1964-01-01 00:00:00','suborbital rocket','failure',1050,'19','','','','Weather',1895,'',11 from dual union all
select 10485,'1964-A194','1964-01-01 00:00:00','suborbital rocket','failure',1050,'21','','','','Weather',1895,'',4 from dual union all
select 10486,'1964-A195','1964-01-01 00:00:00','suborbital rocket','failure',1050,'23','','','','Weather',1895,'',18 from dual union all
select 10487,'1964-A196','1964-01-01 00:00:00','suborbital rocket','failure',1050,'25','','','','Weather',1895,'',13 from dual union all
select 10488,'1964-A197','1964-01-06 00:00:00','military missile','success',843,'','ASF','','','Test',401,'',10 from dual union all
select 10489,'1964-A198','1964-01-21 00:00:00','military missile','success',843,'','ASF (Last Anchorage)','','','Test',401,'',10 from dual union all
select 1049,'1951-A60','1951-09-06 00:00:00','military missile','failure',771,'GM-550','','','','Test',222,'',12 from dual union all
select 10490,'1964-A199','1964-01-01 00:00:00','atmospheric rocket','success',536,'1','Test','','','Test',1849,'',45 from dual union all
select 10491,'1964-A20','1964-02-10 18:02:00','military missile','failure',135,'A001 (Colo.)','Decoy RV','ABRES A-1 (USAF)','','Reentry Test',613,'',40 from dual union all
select 10492,'1964-A200','1964-01-01 00:00:00','atmospheric rocket','success',536,'2','Test','','','Test',1849,'',45 from dual union all
select 10493,'1964-A201','1964-02-14 01:32:00','suborbital rocket','success',88,'','Mighty Mouse','','','Meteor test',1821,'',40 from dual union all
select 10494,'1964-A202','1964-01-01 00:00:00','suborbital rocket','success',329,'','LRL Sampling','','','Aeron',1057,'B-57B',40 from dual union all
select 10495,'1964-A203','1964-01-01 00:00:00','suborbital rocket','success',329,'','LRL Sampling','','','Aeron',1057,'B-57B',40 from dual union all
select 10496,'1964-A204','1964-02-12 00:00:00','suborbital rocket','failure',88,'','Paetzold ozonesonde','','','Aeron',1057,'',0 from dual union all
select 10497,'1964-A205','1964-07-30 00:00:00','suborbital rocket','failure',578,'Hugo IV','Diagnostic','','','Test',1057,'',10 from dual union all
select 10498,'1964-A206','1964-04-01 00:00:00','military missile','failure',1323,'','9M76B No. 2','','','Test',636,'',10 from dual union all
select 10499,'1964-A207','1964-05-01 00:00:00','military missile','failure',1323,'','9M76B No. 3','','','Test',636,'',10 from dual union all
select 105,'1944-S05','1944-01-06 01:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1050,'1951-A61','1951-09-12 00:00:00','atmospheric rocket','success',564,'','E2 Model','','','Test',1821,'',5 from dual union all
select 10500,'1964-A208','1964-07-14 00:00:00','suborbital rocket','failure',535,'HAD 132','Sphere','','','Aeron',219,'',1 from dual union all
select 10501,'1964-A209','1964-07-15 00:00:00','suborbital rocket','failure',535,'HAD 135','Sphere','','','Aeron',219,'',1 from dual union all
select 10502,'1964-A21','1964-02-11 12:00:00','military missile','failure',983,'KY-2','','','','Test',636,'',0 from dual union all
select 10503,'1964-A210','1964-06-05 00:00:00','suborbital rocket','failure',837,'ARPA PRESS','Mass Spec','','','Ionos',1895,'',43 from dual union all
select 10504,'1964-A22','1964-02-12 00:00:00','atmospheric rocket','success',620,'J9','HRV 916','','','Test',1836,'',24 from dual union all
select 10505,'1964-A23','1964-02-12 19:59:40','military missile','failure',155,'48E','Mk 11 RV','ST','BLUE BAY','OT',1671,'',0 from dual union all
select 10506,'1964-A24','1964-02-13 18:11:00','military missile','destroyed before launch',162,'88F','Mk 4 RV','','','Accident',1825,'',0 from dual union all
select 10507,'1964-A25','1964-02-19 17:57:24','suborbital spaceplane','success',1520,'3-26-43','SUV,Boundary Layer','','','Test',379,'NB-52 003',23 from dual union all
select 10508,'1964-A26','1964-02-19 21:00:00','atmospheric rocket','success',835,'Smoke 383','','','','Aeronomy',293,'',14 from dual union all
select 10509,'1964-A27','1964-02-20 18:30:00','atmospheric rocket','success',835,'Smoke 384','','','','Aeronomy',293,'',19 from dual union all
select 1051,'1951-A62','1951-09-13 00:00:00','military missile','failure',703,'593','','','','Test',222,'',0 from dual union all
select 10510,'1964-A28','1964-02-26 12:29:00','military missile','failure',982,'R 22601-20T','8K64U UBP-40 L','','','OT',565,'',0 from dual union all
select 10511,'1964-A29','1964-02-26 20:55:00','atmospheric rocket','success',835,'Smoke 385','','','','Aeronomy',293,'',15 from dual union all
select 10512,'1964-A30','1964-02-27 02:30:28','military missile','failure',999,'G 22500-08L','8K67 LKI-5','','','Test',573,'',0 from dual union all
select 10513,'1964-A31','1964-02-28 00:00:00','test rocket','success',61,'R','','','','Test',313,'',20 from dual union all
select 10514,'1964-A32','1964-03-09 21:54:00','military missile','destroyed before launch',162,'90F','Mk 4 RV','','','Accident',1824,'',0 from dual union all
select 10515,'1964-A33','1964-03-13 00:00:00','military missile','failure',979,'','8K65 UBP-51','','','Test',636,'',0 from dual union all
select 10516,'1964-A34','1964-03-13 17:46:27','suborbital spaceplane','success',1520,'3-27-44','SUV,Boundary Layer','','','Test',379,'NB-52 003',23 from dual union all
select 10517,'1964-A35','1964-03-13 18:30:00','atmospheric rocket','success',835,'Smoke 386','','','','Aeronomy',293,'',17 from dual union all
select 10518,'1964-A36','1964-03-13 18:36:00','atmospheric rocket','success',835,'Smoke 043','','','','Aeronomy',1821,'',21 from dual union all
select 10519,'1964-A37','1964-03-17 00:00:00','suborbital spaceplane','failure',1520,'1- A-71','Optical Degradation','','','Tech',379,'NB-52',14 from dual union all
select 1052,'1951-A63','1951-09-19 00:00:00','military missile','success',703,'595','','','','Test',222,'',8 from dual union all
select 10520,'1964-A38','1964-03-17 19:40:00','atmospheric rocket','success',835,'Smoke 044','','','','Aeronomy',1821,'',17 from dual union all
select 10521,'1964-A39','1964-03-18 18:30:00','atmospheric rocket','success',835,'Smoke 387','','','','Aeronomy',293,'',17 from dual union all
select 10522,'1964-A40','1964-03-20 20:05:00','atmospheric rocket','success',835,'Smoke 388','','','','Aeronomy',293,'',15 from dual union all
select 10523,'1964-A41','1964-03-25 00:00:00','military missile','failure',795,'738','Mk 11 RV','OT','CHROME PLATE','OT',1698,'',1 from dual union all
select 10524,'1964-A42','1964-03-23 00:00:00','suborbital rocket','failure',864,'Sandia 152-12','LRL 12" Payload Test','','','Test/Aeron',855,'',20 from dual union all
select 10525,'1964-A43','1964-03-27 00:00:00','military missile','failure',976,'','8K63 KY64-11','','','Test',636,'',0 from dual union all
select 10526,'1964-A44','1964-03-27 18:10:18','suborbital spaceplane','success',1520,'1-45-72','Optical Degradation','','','Tech',379,'NB-52 003',30 from dual union all
select 10527,'1964-A45','1964-03-27 19:04:00','atmospheric rocket','success',835,'Smoke 046','','','','Aeronomy',1821,'',14 from dual union all
select 10528,'1964-A46','1964-03-31 00:00:00','suborbital spaceplane','failure',1520,'3- A-45','','','','Test',379,'NB-52',14 from dual union all
select 10529,'1964-A47','1964-03-31 18:30:00','atmospheric rocket','success',835,'Smoke 389','','','','Aeronomy',293,'',15 from dual union all
select 1053,'1951-A64','1951-09-25 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10530,'1964-A48','1964-04-01 18:30:00','atmospheric rocket','success',835,'Smoke 390','','','','Aeronomy',293,'',14 from dual union all
select 10531,'1964-A49','1964-04-03 18:00:00','atmospheric rocket','success',835,'Smoke 391','','','','Aeronomy',293,'',18 from dual union all
select 10532,'1964-A50','1964-04-03 20:26:22','military missile','failure',162,'3F','Mk 4-5B','DASO','HIGH BALL','OT',1672,'',0 from dual union all
select 10533,'1964-A51','1964-04-09 21:29:00','atmospheric rocket','success',835,'Smoke 047','','','','Aeronomy',1821,'',19 from dual union all
select 10534,'1964-A52','1964-04-11 07:05:54','military missile','failure',1457,'R10701-601','8K81 LKI-2','GCh 8F114','','Test',590,'',70 from dual union all
select 10535,'1964-A53','1964-04-13 19:45:00','atmospheric rocket','success',835,'Smoke 392','','','','Aeronomy',293,'',19 from dual union all
select 10536,'1964-A54','1964-04-14 18:30:00','suborbital rocket','failure',30,'NASA 04.86NA','Airglow','','','Aeronomy',1869,'',31 from dual union all
select 10537,'1964-A55','1964-04-17 18:44:00','atmospheric rocket','success',835,'Smoke 048','','','','Aeronomy',1821,'',17 from dual;
insert into LAUNCH
select 10538,'1964-A56','1964-04-20 00:00:00','test rocket','success',61,'R','','','','Test',313,'',20 from dual union all
select 10539,'1964-A57','1964-04-21 16:18:00','suborbital rocket','failure',30,'NASA 04.113GA-GI','Astrochemistry','','','Meteorites/Ionos/Solar X',1869,'',11 from dual union all
select 1054,'1951-A65','1951-10-02 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10540,'1964-A58','1964-04-22 20:00:00','atmospheric rocket','success',835,'Smoke 393','','','','Aeronomy',293,'',16 from dual union all
select 10541,'1964-A59','1964-04-25 06:05:26','military missile','failure',983,'KY-2','','','','Test',636,'',0 from dual union all
select 10542,'1964-A60','1964-04-26 06:50:00','military missile','failure',999,'G 22500-11L','8K67 LKI-6','','','Test',574,'',0 from dual union all
select 10543,'1964-A61','1964-04-29 18:00:27','suborbital spaceplane','success',1520,'1-47-74','Opt Deg/Att Indic.','','','Tech/Test',379,'NB-52 003',31 from dual union all
select 10544,'1964-A62','1964-05-04 18:15:00','atmospheric rocket','success',835,'Smoke 049','','','','Aeronomy',1821,'',18 from dual union all
select 10545,'1964-A63','1964-05-05 17:34:00','atmospheric rocket','success',835,'Smoke 050','','','','Aeronomy',1821,'',18 from dual union all
select 10546,'1964-A64','1964-05-06 18:18:00','atmospheric rocket','success',835,'Smoke 051','','','','Aeronomy',1821,'',19 from dual union all
select 10547,'1964-A65','1964-05-06 18:43:00','atmospheric rocket','success',835,'Smoke 052','','','','Aeronomy',1821,'',14 from dual union all
select 10548,'1964-A66','1964-05-11 00:00:00','suborbital spaceplane','failure',1520,'3- A-46','','','','Test',379,'NB-52',14 from dual union all
select 10549,'1964-A67','1964-05-12 17:51:46','suborbital spaceplane','success',1520,'3-28-47','SUV/BLN','','','Test',379,'NB-52 003',22 from dual union all
select 1055,'1951-A66','1951-10-02 00:00:00','suborbital rocket','success',1496,'R3','','','','Test',1439,'',0 from dual union all
select 10550,'1964-A68','1964-05-13 12:59:59','test rocket','success',714,'LJII-2 (12-50-2)','A-001','','','SC Test',1873,'',5 from dual union all
select 10551,'1964-A69','1964-05-14 16:50:00','military missile','destroyed before launch',162,'61F','Mk 4','','','Accident',18,'',0 from dual union all
select 10552,'1964-A70','1964-05-19 18:22:00','atmospheric rocket','success',835,'Smoke 053','','','','Aeronomy',1821,'',13 from dual union all
select 10553,'1964-A71','1964-05-20 18:30:00','atmospheric rocket','success',835,'Smoke 394','','','','Aeronomy',293,'',17 from dual union all
select 10554,'1964-A72','1964-05-21 17:39:34','suborbital spaceplane','failure',1520,'3-29-48','SUV/BLN','','','Test',379,'NB-52 003',19 from dual union all
select 10555,'1964-A73','1964-05-21 18:30:00','atmospheric rocket','success',835,'Smoke 395','','','','Aeronomy',293,'',14 from dual union all
select 10556,'1964-A74','1964-05-22 19:43:00','atmospheric rocket','success',835,'Smoke 054','','','','Aeronomy',1821,'',15 from dual union all
select 10557,'1964-A75','1964-05-25 20:05:00','atmospheric rocket','success',835,'Smoke 396','','','','Aeronomy',293,'',18 from dual union all
select 10558,'1964-A76','1964-05-27 19:30:00','atmospheric rocket','success',835,'Smoke 397','','','','Aeronomy',293,'',17 from dual union all
select 10559,'1964-A77','1964-05-28 02:15:00','suborbital rocket','failure',837,'NASA 14.54DA','Air sampling','','','Aeronomy',1895,'',48 from dual union all
select 1056,'1951-A67','1951-10-04 00:00:00','military missile','failure',703,'597','','','','Test',222,'',0 from dual union all
select 10560,'1964-A78','1964-05-28 23:15:00','military missile','failure',135,'A002 (Shiprock)','Decoy RV','ABRES A-2 (USAF)','','Reentry Test',613,'',40 from dual union all
select 10561,'1964-A79','1964-06-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 10562,'1964-A80','1964-06-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 10563,'1964-A81','1964-06-09 19:00:00','atmospheric rocket','success',835,'Smoke 398','','','','Aeronomy',293,'',16 from dual union all
select 10564,'1964-A82','1964-06-10 18:15:00','atmospheric rocket','success',835,'Smoke 399','','','','Aeronomy',293,'',16 from dual union all
select 10565,'1964-A83','1964-06-11 00:00:00','suborbital spaceplane','failure',1520,'1- A-76','','','','Tech/Test',379,'NB-52',14 from dual union all
select 10566,'1964-A84','1964-06-11 18:34:00','atmospheric rocket','success',835,'Smoke 055','','','','Aeronomy',1821,'',15 from dual union all
select 10567,'1964-A85','1964-06-12 19:00:00','atmospheric rocket','success',835,'Smoke 400','','','','Aeronomy',293,'',17 from dual union all
select 10568,'1964-A86','1964-06-15 00:00:00','military missile','failure',458,'','','','','Test',669,'',0 from dual union all
select 10569,'1964-A87','1964-06-15 00:00:00','suborbital spaceplane','success',1520,'2- C-53','Carry Test','','','Test',379,'NB-52',14 from dual union all
select 1057,'1951-A68','1951-10-05 00:00:00','suborbital rocket','success',1496,'R4','','','','Test',1439,'',2 from dual union all
select 10570,'1964-A88','1964-06-15 19:00:00','atmospheric rocket','success',835,'Smoke 401','','','','Aeronomy',293,'',17 from dual union all
select 10571,'1964-A89','1964-06-17 00:00:00','military missile','failure',458,'','','','','Test',669,'',0 from dual union all
select 10572,'1964-A90','1964-06-17 19:00:00','atmospheric rocket','success',835,'Smoke 402','','','','Aeronomy',293,'',15 from dual union all
select 10573,'1964-A91','1964-06-23 00:00:00','suborbital spaceplane','failure',1520,'2- A-54','Stability','','','Test',379,'NB-52',14 from dual union all
select 10574,'1964-A93','1964-06-25 17:34:47','suborbital spaceplane','success',1520,'2-32-55','Stability','','','Test',379,'NB-52 003',25 from dual union all
select 10575,'1964-A94','1964-07-01 00:00:00','suborbital rocket','failure',27,'','','','','Test?',1833,'',0 from dual union all
select 10576,'1964-A95','1964-07-01 00:00:00','suborbital rocket','failure',27,'','','','','Test?',1833,'',0 from dual union all
select 10577,'1964-A96','1964-07-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 10578,'1964-A97','1964-07-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 10579,'1964-A98','1964-07-16 00:00:00','military missile','success',19,'','','','','Test',1895,'',30 from dual union all
select 1058,'1951-A69','1951-10-06 00:00:00','suborbital rocket','success',1496,'R5','','','','Test',1439,'',2 from dual union all
select 10580,'1964-A99','1964-06-30 17:49:40','suborbital spaceplane','success',1520,'1-49-77','Opt Deg/Alt Build','','','Tech/Test',379,'NB-52 003',30 from dual union all
select 10581,'1964-F01','1964-02-19 05:47:40','orbital','failure',801,'T 15000- 19','','3MV-1 No. 2','','Sat',480,'',200 from dual union all
select 10582,'1964-F02','1964-03-19 11:13:41','orbital','failure',1377,'391/D24','BE-A','BE-A','','Sat',233,'',1200 from dual union all
select 10583,'1964-F03','1964-03-21 08:15:35','orbital','failure',800,'T 15000- 20','[8K78/E6]','E-6 No. 6','','Sat',480,'',200 from dual union all
select 10584,'1964-F04','1964-03-24 22:22:48','orbital','failure',1394,'396/FTV 1175','CORONA 77/1003','CORONA J-6','HEALTH FARM','Sat TAG88',1019,'',183 from dual union all
select 10585,'1964-F05','1964-04-20 08:08:28','orbital','failure',800,'T 15000- 21','[8K78/E6]','E-6 No. 5','','Sat',480,'',200 from dual union all
select 10586,'1964-F07','1964-06-04 04:00:00','orbital','failure',802,'G 15000- 18/R10334','','Molniya-1 11F67 No 2','','Sat',480,'',50 from dual union all
select 10587,'1964-F08','1964-06-25 01:40:24','orbital','failure',1148,'S128R','ESRS (PL-94, CRL-2)','AFCRL B','CHERRY PIE','Sat',1018,'',20 from dual union all
select 10588,'1964-F09','1964-06-30 14:04:22','orbital','failure',152,'AC-3','Centaur R&D','AC-3','','Sat',268,'',500 from dual union all
select 10589,'1964-F10','1964-09-01 15:00:06','orbital','failure',1423,'3A-2','Transtage','Lead Dummy Payload','','Sat',239,'',160 from dual union all
select 1059,'1951-A70','1951-10-10 00:00:00','military missile','failure',340,'11-10','','','','Test',1863,'',0 from dual union all
select 10590,'1964-F11','1964-10-07 19:30:00','orbital','failure',177,'7103','AFP-206','GAMBIT SV 961','BUSY LINE','Sat',1773,'',140 from dual union all
select 10591,'1964-F12','1964-10-23 07:30:00','orbital','failure',682,'4000-000-01L','','Strela-1','','Sat',564,'',250 from dual union all
select 10592,'1964-F13','1964-12-01 08:45:00','orbital','failure',681,'','','DS-2 No. 2','','Sat',626,'',20 from dual union all
select 10593,'1964-M01','1964-01-04 00:00:00','suborbital rocket','success',767,'','2A IV-1','','','Test',185,'',80 from dual union all
select 10594,'1964-M02','1964-01-17 07:21:00','suborbital rocket','success',215,'AD-2-42','','','','Auroral/Ionos/Micromet',428,'',75 from dual union all
select 10595,'1964-M03','1964-02-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 10596,'1964-M04','1964-02-17 00:00:00','military missile','success',1162,'','','','','OT?',1860,'',50 from dual union all
select 10597,'1964-M05','1964-02-12 09:46:00','suborbital rocket','success',535,'HAD 124','','','','Aeron',1835,'',75 from dual union all
select 10598,'1964-M06','1964-03-13 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 10599,'1964-M07','1964-03-24 12:15:00','test rocket','failure',1372,'240','ASSET 2','','','Reentry Test',234,'',55 from dual union all
select 106,'1944-S06','1944-02-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1060,'1951-A71','1951-10-16 00:00:00','atmospheric rocket','success',869,'490B','Nike 490B Elec Test','','','Test',1895,'',10 from dual union all
select 10600,'1964-M08','1964-04-08 18:02:27','suborbital spaceplane','success',1520,'1-46-73','Opt Deg/Alt Build','','','Tech/Test',379,'NB-52 003',53 from dual union all
select 10601,'1964-M09','1964-04-10 04:31:00','suborbital rocket','failure',30,'NASA 04.81GG','','','','UV Astron',1869,'',74 from dual union all
select 10602,'1964-M10','1964-05-19 18:26:28','suborbital spaceplane','success',1520,'1-48-75','Opt Deg/Alt Build','','','Tech/Test',379,'NB-52 003',59 from dual union all
select 10603,'1964-M100','1964-04-27 16:48:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10604,'1964-M101','1964-04-28 10:28:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10605,'1964-M102','1964-04-28 11:00:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10606,'1964-M103','1964-04-28 14:03:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10607,'1964-M104','1964-04-28 15:59:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10608,'1964-M105','1964-04-28 16:46:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10609,'1964-M106','1964-04-29 10:42:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 1061,'1951-A72','1951-10-16 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10610,'1964-M107','1964-04-29 14:08:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10611,'1964-M108','1964-04-29 16:34:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10612,'1964-M109','1964-04-30 11:29:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10613,'1964-M11','1964-06-10 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 10614,'1964-M110','1964-04-30 14:27:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10615,'1964-M111','1964-04-30 16:19:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10616,'1964-M112','1964-05-04 10:45:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10617,'1964-M113','1964-05-04 11:09:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10618,'1964-M114','1964-05-04 14:17:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10619,'1964-M115','1964-05-04 16:43:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 1062,'1951-A73','1951-10-17 00:00:00','atmospheric rocket','unknown',576,'7','','','','Test',1895,'',10 from dual union all
select 10620,'1964-M116','1964-05-04 21:43:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10621,'1964-M117','1964-05-05 11:09:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10622,'1964-M118','1964-05-05 11:29:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10623,'1964-M119','1964-05-05 16:35:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10624,'1964-M12','1964-06-09 00:00:00','military missile','success',1162,'099','','','','OT?',1895,'',50 from dual union all
select 10625,'1964-M120','1964-05-05 17:00:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10626,'1964-M121','1964-05-06 11:20:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10627,'1964-M122','1964-05-06 14:00:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual;
insert into LAUNCH
select 10628,'1964-M123','1964-05-06 16:22:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10629,'1964-M124','1964-05-08 10:39:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 1063,'1951-A74','1951-10-17 00:00:00','atmospheric rocket','unknown',576,'6','','','','Test',1895,'',10 from dual union all
select 10630,'1964-M125','1964-05-08 11:03:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10631,'1964-M126','1964-05-08 14:22:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10632,'1964-M127','1964-05-08 14:48:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10633,'1964-M128','1964-05-08 16:47:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10634,'1964-M129','1964-02-14 00:52:00','suborbital rocket','success',88,'','Arcas-Margo','','','Reentry Test',1821,'',50 from dual union all
select 10635,'1964-M13','1964-06-09 00:00:00','military missile','success',1162,'100','1stBtn 68thArt','','','OT?',1895,'',50 from dual union all
select 10636,'1964-M130','1964-07-09 00:00:00','military missile','success',871,'ZK-43 20158','Sim Target Intercept','','','Interceptor',873,'',70 from dual union all
select 10637,'1964-M131','1964-12-09 00:00:00','military missile','success',871,'ZW-72 20061','Special Comms Round','','','Test',1884,'',50 from dual union all
select 10638,'1964-M132','1964-05-01 00:00:00','military missile','success',871,'ZK-44','','','','Interceptor',873,'',80 from dual union all
select 10639,'1964-M133','1964-06-01 00:00:00','military missile','success',871,'ZK-45','','','','Interceptor',873,'',80 from dual union all
select 1064,'1951-A75','1951-10-24 00:00:00','atmospheric rocket','unknown',576,'8','','','','Test',1895,'',10 from dual union all
select 10640,'1964-M134','1964-06-01 00:00:00','military missile','success',871,'ZK-46','','','','Interceptor',873,'',80 from dual union all
select 10641,'1964-M135','1964-06-01 00:00:00','military missile','success',871,'ZK-47','','','','Interceptor',873,'',80 from dual union all
select 10642,'1964-M136','1964-10-01 00:00:00','military missile','success',871,'ZK-48','','','','Interceptor',873,'',80 from dual union all
select 10643,'1964-M137','1964-10-01 00:00:00','military missile','success',871,'ZK-49','','','','Interceptor',873,'',80 from dual union all
select 10644,'1964-M138','1964-02-18 00:00:00','military missile','success',871,'ZK-34','','','','Interceptor',873,'',80 from dual union all
select 10645,'1964-M139','1964-02-01 00:00:00','military missile','success',871,'ZK-35','','','','Interceptor',873,'',80 from dual union all
select 10646,'1964-M14','1964-06-04 00:00:00','test rocket','failure',1432,'Ci1 (11)','MSBS test','','','Test',669,'',1 from dual union all
select 10647,'1964-M140','1964-02-01 00:00:00','military missile','success',874,'ZK-36','ASAT training','','','Interceptor',873,'',80 from dual union all
select 10648,'1964-M141','1964-03-01 00:00:00','military missile','success',871,'ZK-37','','','','Interceptor',873,'',80 from dual union all
select 10649,'1964-M142','1964-03-01 00:00:00','military missile','success',871,'ZK-38','','','','Interceptor',873,'',80 from dual union all
select 1065,'1951-A76','1951-10-24 00:00:00','atmospheric rocket','unknown',576,'9','','','','Test',1895,'',10 from dual union all
select 10650,'1964-M143','1964-04-01 00:00:00','military missile','success',871,'ZK-39','','','','Interceptor',873,'',80 from dual union all
select 10651,'1964-M144','1964-05-01 00:00:00','military missile','success',871,'ZK-40','','','','Interceptor',873,'',80 from dual union all
select 10652,'1964-M145','1964-05-01 00:00:00','military missile','success',871,'ZK-41','','','','Interceptor',873,'',80 from dual union all
select 10653,'1964-M147','1964-03-16 00:00:00','suborbital rocket','failure',88,'','Paetzold ozonesonde','','','Aeron',1895,'',50 from dual union all
select 10654,'1964-M148','1964-08-16 00:00:00','suborbital rocket','failure',88,'','Paetzold ozonesonde','','','Aeron',1057,'',50 from dual union all
select 10655,'1964-M149','1964-10-08 00:00:00','suborbital rocket','success',88,'','ROCOZ','','','Aeron',1057,'',50 from dual union all
select 10656,'1964-M15','1964-07-08 21:02:52','suborbital spaceplane','success',1520,'3-30-50','Hor Scan/MH-96','','','Test',379,'NB-52 003',51 from dual union all
select 10657,'1964-M150','1964-01-31 00:00:00','suborbital rocket','success',716,'HASP II','White Oak HI-LO','','','Aeron',1057,'',58 from dual union all
select 10658,'1964-M151','1964-08-26 00:00:00','suborbital rocket','success',88,'','Paetzold ozonesonde','','','Aeron',1057,'',50 from dual union all
select 10659,'1964-M152','1964-09-25 00:00:00','suborbital rocket','success',631,'HASP/JUDI','','','','Aeron',1057,'',76 from dual union all
select 1066,'1951-A77','1951-10-24 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10660,'1964-M153','1964-06-11 00:00:00','suborbital rocket','success',625,'','Test','','','Test',1895,'',68 from dual union all
select 10661,'1964-M154','1964-01-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 10662,'1964-M155','1964-01-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 10663,'1964-M16','1964-07-17 07:20:00','suborbital rocket','success',1076,'S-B1','','','','Test',990,'',60 from dual union all
select 10664,'1964-M17','1964-07-19 00:00:00','suborbital rocket','success',1290,'','Life Sci (mice)','','','Biology',1364,'',70 from dual union all
select 10665,'1964-M18','1964-07-19 03:47:00','suborbital rocket','success',1076,'S-B2','','','','Test',990,'',60 from dual union all
select 10666,'1964-M19','1964-07-22 15:39:00','test rocket','success',1372,'250','ASSET 3','','','Reentry Test',234,'',71 from dual union all
select 10667,'1964-M20','1964-07-23 03:37:00','suborbital rocket','success',1076,'S-B3','','','','Test',990,'',60 from dual union all
select 10668,'1964-M21','1964-10-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 10669,'1964-M22','1964-10-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 1067,'1951-A78','1951-10-26 00:00:00','military missile','success',771,'GM-551','','','','Test',222,'',12 from dual union all
select 10670,'1964-M23','1964-10-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 10671,'1964-M24','1964-10-21 00:00:00','test rocket','success',1432,'Ci2 (12)','MSBS test','','','Test',669,'',49 from dual union all
select 10672,'1964-M25','1964-10-29 03:35:00','test rocket','success',1371,'260','ASSET 4','','','Reentry Test',234,'',50 from dual union all
select 10673,'1964-M26','1964-11-10 00:00:00','suborbital rocket','success',203,'B69','CNES FU-113','','','Aeronomy',313,'',68 from dual union all
select 10674,'1964-M27','1964-11-10 23:26:00','suborbital rocket','success',837,'AE5.551 IQSY','IQSY Ionos,Sphere','','','Aeron',394,'',75 from dual union all
select 10675,'1964-M28','1964-12-10 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 10676,'1964-M29','1964-12-09 02:00:00','test rocket','success',1371,'247','ASSET 5','','','Reentry Test',234,'',50 from dual union all
select 10677,'1964-M30','1964-12-11 00:00:00','test rocket','success',1432,'Ci3 (13)','MSBS test','','','Test',669,'',80 from dual union all
select 10678,'1964-M31','1964-12-15 00:00:00','test rocket','success',1432,'Ci4 (14)','MSBS test','','','Test',669,'',80 from dual union all
select 10679,'1964-M32','1964-10-12 00:00:00','test rocket','failure',1068,'R3','A-1 capsule','','','Test',664,'',60 from dual union all
select 1068,'1951-A79','1951-11-13 00:00:00','atmospheric rocket','unknown',576,'13','','','','Test',1895,'',10 from dual union all
select 10680,'1964-M33','1964-12-18 00:00:00','test rocket','failure',1068,'R4','A-1 capsule','','','Test',664,'',60 from dual union all
select 10681,'1964-M84','1964-04-20 15:15:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10682,'1964-M85','1964-04-20 16:07:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10683,'1964-M86','1964-04-21 09:32:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10684,'1964-M87','1964-04-21 14:28:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10685,'1964-M88','1964-04-21 16:06:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10686,'1964-M89','1964-04-22 10:59:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10687,'1964-M90','1964-04-22 14:03:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10688,'1964-M91','1964-04-22 16:33:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10689,'1964-M92','1964-04-23 10:38:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 1069,'1951-A80','1951-11-14 00:00:00','atmospheric rocket','unknown',576,'11','','','','Test',1895,'',10 from dual union all
select 10690,'1964-M93','1964-04-23 14:03:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10691,'1964-M94','1964-04-23 14:24:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10692,'1964-M95','1964-04-23 16:23:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10693,'1964-M96','1964-04-24 10:02:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10694,'1964-M97','1964-04-24 14:04:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10695,'1964-M98','1964-04-27 11:20:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10696,'1964-M99','1964-04-27 14:02:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',1343,'',75 from dual union all
select 10697,'1964-S01','1964-01-01 00:00:00','military missile','success',7,'6TYa','F3','','','Test',182,'',100 from dual union all
select 10698,'1964-S02','1964-01-01 00:00:00','military missile','success',7,'6TYa','F4','','','Test',182,'',100 from dual union all
select 10699,'1964-S05','1964-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',150 from dual union all
select 107,'1944-S07','1944-02-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1070,'1951-A81','1951-11-14 00:00:00','atmospheric rocket','unknown',576,'10','','','','Test',1895,'',10 from dual union all
select 10700,'1964-S06','1964-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',150 from dual union all
select 10701,'1964-S07','1964-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',150 from dual union all
select 10702,'1964-S08','1964-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',150 from dual union all
select 10703,'1964-S09','1964-01-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 10704,'1964-S10','1964-01-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 10705,'1964-S100','1964-02-17 17:03:04','military missile','success',929,'A2P-257','PY-1 DASO','','','OT',421,'SSBN 622',1000 from dual union all
select 10706,'1964-S101','1964-02-17 16:15:00','military missile','success',1420,'B-15','Mk 6 RV','R&D','SAFE CONDUCT','Test',1657,'',1300 from dual union all
select 10707,'1964-S102','1964-02-17 10:00:00','suborbital rocket','success',30,'ND3.143','','','','Aeron',1895,'',268 from dual union all
select 10708,'1964-S103','1964-02-18 00:00:00','military missile','success',976,'','8K63 KY64-8','','','Test',636,'',402 from dual union all
select 10709,'1964-S104','1964-02-18 21:50:00','suborbital rocket','success',837,'ERDA 004','Sphere','','','Aeron',1895,'',205 from dual union all
select 1071,'1951-A82','1951-11-15 00:00:00','atmospheric rocket','success',869,'490B','System Test 67R','','','Test',1895,'',10 from dual union all
select 10710,'1964-S105','1964-02-19 15:15:07','military missile','success',999,'G 22500-07L','8K67 LKI-4','','','Test',573,'',906 from dual union all
select 10711,'1964-S106','1964-02-19 21:30:00','suborbital rocket','success',839,'ERDA 005','Sphere','','','Aeron',1895,'',154 from dual union all
select 10712,'1964-S107','1964-02-20 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 10713,'1964-S108','1964-02-20 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 10714,'1964-S109','1964-02-20 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 10715,'1964-S110','1964-02-20 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 10716,'1964-S111','1964-02-20 03:00:00','suborbital rocket','failure',1198,'SL30','RAE Test 23','','','Test',1839,'',112 from dual union all
select 10717,'1964-S112','1964-02-20 07:23:00','suborbital rocket','success',837,'NASA 14.43GE','Auroral particles','','','Auroral',428,'',204 from dual;
insert into LAUNCH
select 10718,'1964-S113','1964-02-25 02:00:00','military missile','success',795,'813','Mk 11 RV','R&D','SNAP ROLL','OT',1697,'',900 from dual union all
select 10719,'1964-S114','1964-02-25 02:45:00','military missile','success',795,'688','Mk 11 RV','OT','FAST ORBIT','OT',1698,'',900 from dual union all
select 1072,'1951-A83','1951-11-15 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10720,'1964-S115','1964-02-24 14:57:00','military missile','success',916,'','P-74','','','OT',437,'',180 from dual union all
select 10721,'1964-S116','1964-02-25 00:17:00','military missile','success',916,'','P-75','','','OT',437,'',180 from dual union all
select 10722,'1964-S117','1964-02-26 01:29:58','military missile','success',795,'437','Mk 11 RV/Decoys','R&D','','Test',264,'',900 from dual union all
select 10723,'1964-S118','1964-02-25 20:22:00','ballistic missile test','success',155,'5E','ABRES WAC-3/Pod 18','','','Reentry Test',226,'',1600 from dual union all
select 10724,'1964-S119','1964-02-26 20:14:44','military missile','success',1420,'N-32','Mk 6 RV','R&D','','Test',230,'',1300 from dual union all
select 10725,'1964-S12','1964-01-01 00:00:00','military missile','success',1035,'','','','','Test',636,'',500 from dual union all
select 10726,'1964-S120','1964-02-26 21:18:00','suborbital rocket','failure',30,'NASA 04.124UA','JHU Aurora 2','','','Aeronomy',428,'',161 from dual union all
select 10727,'1964-S121','1964-02-28 02:04:14','military missile','success',795,'439','Mk 11 RV/Decoys','R&D','','Test',266,'',900 from dual union all
select 10728,'1964-S122','1964-02-29 00:00:00','military missile','success',794,'581','Mk 5 RV','OT Ripple','BRASS RING','OT',1699,'',900 from dual union all
select 10729,'1964-S123','1964-02-29 00:00:00','military missile','success',794,'636','Mk 5 RV','OT Ripple','BOX SEAT','OT',1700,'',900 from dual union all
select 1073,'1951-A84','1951-11-16 00:00:00','atmospheric rocket','success',869,'490B','System Test 68R','','','Test',1895,'',10 from dual union all
select 10730,'1964-S124','1964-03-01 00:00:00','military missile','success',837,'BMRS','','','','Target',1895,'',100 from dual union all
select 10731,'1964-S125','1964-02-29 00:00:00','military missile','success',976,'','8K63 PRO-29 Target','','','Test',942,'',402 from dual union all
select 10732,'1964-S126','1964-02-29 05:32:00','suborbital rocket','success',837,'NASA 14.44GE','Auroral particles','','','Auroral',428,'',204 from dual union all
select 10733,'1964-S127','1964-03-01 06:40:00','military missile','success',1373,'209/20006','AFP-437 Demo','','','Interceptor',772,'',674 from dual union all
select 10734,'1964-S128','1964-03-02 16:30:03','military missile','success',929,'A2P-294','PY-1 DASO','','','OT',421,'SSBN 624',1000 from dual union all
select 10735,'1964-S13','1964-01-01 00:00:00','military missile','success',1035,'','','','','Test',636,'',500 from dual union all
select 10736,'1964-S130','1964-03-04 00:00:00','military missile','success',976,'','8K63 PRO-30 Target','','','Test',942,'',402 from dual union all
select 10737,'1964-S131','1964-04-03 09:14:00','military missile','success',982,'R 22502-20T','8K64U UBP-41 K','','','OT',565,'',1210 from dual union all
select 10738,'1964-S132','1964-03-04 09:53:00','suborbital rocket','success',535,'HAD 127','Sphere','','','Aeron',1835,'',111 from dual union all
select 10739,'1964-S133','1964-03-06 00:00:00','military missile','success',795,'686','Mk 11 RV','OT','KITE TAIL','OT',1702,'',900 from dual union all
select 1074,'1951-A85','1951-11-27 00:00:00','atmospheric rocket','success',869,'490B','System Test 69R','','','Test',1895,'',10 from dual union all
select 10740,'1964-S134','1964-03-07 02:45:00','suborbital rocket','success',839,'NASA 10.137GA','GSFC Grenade 47','','','Aeronomy',1821,'',114 from dual union all
select 10741,'1964-S135','1964-03-10 00:00:00','military missile','success',976,'','8K63 PRO-31 Target','','','Test',942,'',402 from dual union all
select 10742,'1964-S136','1964-03-10 00:00:00','suborbital rocket','success',1431,'Sandia 152-5','Tomahawk-Dart 7"','','','Test/Aeron',1490,'',105 from dual union all
select 10743,'1964-S137','1964-03-10 02:06:00','suborbital rocket','success',1199,'SL129','LUX-17/SoG-1/UCL I-18','','','Gamma Astron/Solar X',1839,'',175 from dual union all
select 10744,'1964-S138','1964-03-11 00:00:00','test rocket','success',1433,'L2','','','','Test',664,'',110 from dual union all
select 10745,'1964-S139','1964-03-11 00:00:00','military missile','success',241,'BK.12','Cuckoo II Test','','','Reentry Test',1842,'',500 from dual union all
select 10746,'1964-S14','1964-01-01 00:00:00','military missile','success',206,'007','','','','Reentry Test',313,'',270 from dual union all
select 10747,'1964-S140','1964-03-11 12:30:05','military missile','failure',932,'A3X-58 (A3Y-2)','','','','Test',244,'',100 from dual union all
select 10748,'1964-S141','1964-03-11 00:00:00','military missile','success',976,'','8K63 PRO-32 Target','','','Test',942,'',402 from dual union all
select 10749,'1964-S142','1964-03-11 18:00:00','suborbital rocket','success',839,'AC6.342','Ion composition','','','Ionos',394,'',100 from dual union all
select 1075,'1951-A86','1951-11-28 00:00:00','atmospheric rocket','unknown',576,'12','','','','Test',1895,'',10 from dual union all
select 10750,'1964-S143','1964-03-12 02:07:00','suborbital rocket','success',1199,'SL128','LUX-18/SoG-2/UCL I-19','','','Gamma Astron/Solar X',1839,'',176 from dual union all
select 10751,'1964-S144','1964-03-12 02:58:40','suborbital rocket','success',837,'Ferdinand 6','Ferdinand 6','','','Auroral/Plasma/Solar',29,'',146 from dual union all
select 10752,'1964-S145','1964-03-13 17:04:47','military missile','success',795,'440','Mk 11 RV','R&D','','Test',266,'',900 from dual union all
select 10753,'1964-S146','1964-03-13 11:40:56','ballistic missile test','failure',247,'22-2 (OAR Probe 2)','OAR 22-2/AFCRL-9','','','Magnetospheric AD-623',235,'',600 from dual union all
select 10754,'1964-S147','1964-03-13 00:00:00','military missile','success',1420,'N-30','Mk 6 RV','R&D','APPLE PIE','Test',1658,'',1300 from dual union all
select 10755,'1964-S148','1964-03-13 18:20:00','suborbital rocket','success',839,'Falling Sphere 12','','','','Aeronomy',871,'',174 from dual union all
select 10756,'1964-S149','1964-03-14 00:00:00','suborbital rocket','success',549,'HAD-3','LRL HAD-3','','','Solar X',1015,'',143 from dual union all
select 10757,'1964-S15','1964-01-01 00:00:00','military missile','success',206,'008','','','','Reentry Test',313,'',270 from dual union all
select 10758,'1964-S150','1964-03-15 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 10759,'1964-S151','1964-03-15 02:44:00','suborbital rocket','success',837,'Ferdinand 7','Ferdinand 7','','','Ionos/Plasma',29,'',134 from dual union all
select 1076,'1951-A87','1951-11-29 00:00:00','atmospheric rocket','success',442,'','D3 Model','','','Test',1821,'',10 from dual union all
select 10760,'1964-S152','1964-03-15 14:12:54','military missile','success',983,'KY-2','','','','Test',636,'',278 from dual union all
select 10761,'1964-S153','1964-03-16 16:30:14','military missile','success',929,'A2P-288','PY-1 DASO','','','OT',421,'SSBN 624',1000 from dual union all
select 10762,'1964-S154','1964-03-16 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 10763,'1964-S155','1964-03-16 19:29:00','suborbital rocket','success',30,'HITAB','BIRA-1','','','IR/UV Background',1895,'',200 from dual union all
select 10764,'1964-S156','1964-03-18 00:00:00','suborbital rocket','success',865,'Sandia 152-45','Ballistic','','','Test/Aeron',855,'',315 from dual union all
select 10765,'1964-S157','1964-03-18 06:11:00','suborbital rocket','success',837,'NASA 14.151UE','Rice Airglow/Aur. 2','','','Auroral',428,'',151 from dual union all
select 10766,'1964-S158','1964-03-19 15:42:00','suborbital rocket','success',837,'Ferdinand 8','Ferdinand 8','','','Ionos/Plasma/Solar',29,'',139 from dual union all
select 10767,'1964-S159','1964-03-20 17:20:11','military missile','success',795,'442','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 10768,'1964-S16','1964-01-04 00:00:00','military missile','success',1018,'G 15001-03','8K75 LKI 3-18 SLI','','','OT',584,'',1160 from dual union all
select 10769,'1964-S160','1964-03-20 05:23:00','suborbital rocket','success',837,'NASA 14.152UE','Rice Airglow/Aur. 3','','','Auroral',428,'',151 from dual union all
select 1077,'1951-A88','1951-11-29 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10770,'1964-S161','1964-03-24 00:00:00','military missile','success',795,'812','Mk 11 RV','OT','BIG TREE','OT',1697,'',900 from dual union all
select 10771,'1964-S162','1964-03-23 00:00:00','military missile','success',976,'','8K63 KY64-9','','','Test',636,'',402 from dual union all
select 10772,'1964-S163','1964-03-23 10:37:00','suborbital rocket','success',837,'NASA 14.153UE','Rice Airglow/Aur. 4','','','Auroral',428,'',153 from dual union all
select 10773,'1964-S164','1964-03-24 00:00:00','military missile','success',976,'','8K63 KY64-10','','','Test',636,'',402 from dual union all
select 10774,'1964-S165','1964-03-23 20:41:57','ballistic missile test','success',1420,'N-33','Mk 6 RV/Pod T-207','R&D','','Test/Plume study',230,'',1300 from dual union all
select 10775,'1964-S166','1964-03-24 04:22:54','suborbital rocket','success',837,'NASA 14.118GE','Auroral Electrons','','','Auroral',428,'',184 from dual union all
select 10776,'1964-S167','1964-03-25 00:00:00','suborbital rocket','success',837,'','','','','Test',1410,'',200 from dual union all
select 10777,'1964-S169','1964-03-25 03:25:00','suborbital rocket','success',837,'NASA 14.120GE','','','','Auroral',428,'',179 from dual union all
select 10778,'1964-S17','1964-01-06 00:00:00','military missile','success',874,'ZK-33','ASAT training','','','Interceptor',873,'',80 from dual union all
select 10779,'1964-S170','1964-03-26 00:00:00','military missile','success',794,'597','Mk 5 RV','OT-12','CIGAR SMOKE','OT',1700,'',900 from dual union all
select 1078,'1951-A89','1951-12-04 00:00:00','atmospheric rocket','success',869,'490B','System Test 70R','','','Test',1895,'',10 from dual union all
select 10780,'1964-S171','1964-03-26 00:00:00','military missile','success',979,'','8K65 UBP-52 43 RD','','','Test',636,'',675 from dual union all
select 10781,'1964-S172','1964-03-26 03:30:00','suborbital rocket','success',837,'NASA 14.119GE','','','','Auroral',428,'',193 from dual union all
select 10782,'1964-S174','1964-03-31 02:33:11','military missile','success',795,'441','Mk 11 RV','R&D/TRUMP target','','Test',266,'',900 from dual union all
select 10783,'1964-S175','1964-03-31 02:33:30','suborbital rocket','success',850,'','TRUMP (MM+30s)','','','Test',293,'',131 from dual union all
select 10784,'1964-S176','1964-03-30 00:00:00','suborbital rocket','success',837,'','','','','Test',1410,'',200 from dual union all
select 10785,'1964-S177','1964-03-30 19:00:00','suborbital rocket','success',30,'AD3.514','EUV Mon 3/RPA','','','Solar EUV',1869,'',248 from dual union all
select 10786,'1964-S178','1964-03-31 00:00:00','military missile','success',794,'557','Mk 5 RV','OT','CRACKER BOX','OT',1699,'',900 from dual union all
select 10787,'1964-S179','1964-04-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual union all
select 10788,'1964-S18','1964-01-08 00:00:10','military missile','success',932,'A3X-50','PX-2,PY-2','','','Test',251,'',1000 from dual union all
select 10789,'1964-S182','1964-04-01 00:00:00','suborbital rocket','success',30,'HITAB','TSRA-2','','','IR Background',1786,'',200 from dual union all
select 1079,'1951-A90','1951-12-06 00:00:00','military missile','unknown',339,'12R','Corporal I','','','Test',1895,'',30 from dual union all
select 10790,'1964-S183','1964-04-01 03:10:00','suborbital rocket','success',666,'K-8L-3','','','','Aeron/Fields?',839,'',100 from dual union all
select 10791,'1964-S184','1964-04-01 20:22:00','ballistic missile test','success',162,'137F','ABRES WAC-2/Pod 2','','','Reentry Test/Plume',226,'',1400 from dual union all
select 10792,'1964-S185','1964-04-02 00:00:00','suborbital rocket','success',837,'','','','','Test',1410,'',200 from dual union all
select 10793,'1964-S186','1964-04-02 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 10794,'1964-S187','1964-04-02 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 10795,'1964-S188','1964-04-03 03:48:00','suborbital rocket','success',30,'NASA 04.15GG','','','','UV Astron',1869,'',190 from dual union all
select 10796,'1964-S189','1964-04-06 16:30:03','military missile','success',929,'A2P-320','','','','OT',421,'SSBN 625',1000 from dual union all
select 10797,'1964-S19','1964-01-08 13:15:00','suborbital rocket','success',837,'NASA 14.129IA','ISRO 10.02','TERLS-002','','Aeronomy Na',1467,'',187 from dual union all
select 10798,'1964-S190','1964-04-08 01:12:56','military missile','success',795,'443','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 10799,'1964-S191','1964-04-07 05:29:00','suborbital rocket','success',215,'AA-2A-43','Auroral particles','','','Auroral/Ionos/Micromet',428,'',155 from dual union all
select 108,'1944-S08','1944-02-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1080,'1951-A91','1951-12-07 00:00:00','military missile','success',771,'GM-547','First all military','','','Test',222,'',12 from dual union all
select 10800,'1964-S192','1964-04-07 14:13:00','suborbital rocket','success',30,'KP3.4','Dayglow','','','Aeron',1869,'',223 from dual union all
select 10801,'1964-S193','1964-04-09 07:47:00','suborbital rocket','success',215,'AD-2A-44','Auroral study  Apr9','','','Auroral/Aeron/Ionos',428,'',154 from dual union all
select 10802,'1964-S194','1964-04-08 08:41:00','military missile','success',983,'KY-2','','','','Test',636,'',278 from dual union all
select 10803,'1964-S195','1964-04-09 21:04:50','ballistic missile test','success',1420,'N-3A','Mk 6 RV/Pod RVIP','R&D','','Test/Plume study',230,'',1300 from dual union all
select 10804,'1964-S196','1964-04-09 14:22:00','suborbital rocket','success',837,'NASA 14.134IA','Rehbar 4  (Na)','','','Aeronomy',1419,'',160 from dual union all
select 10805,'1964-S197','1964-04-10 17:40:00','suborbital rocket','success',1021,'RAM B-3','RAM B-3','','','Reentry Test',1821,'',90 from dual union all
select 10806,'1964-S198','1964-04-11 06:29:00','suborbital rocket','success',837,'NASA 14.121GE','Auroral emissions','','','Auroral',428,'',172 from dual union all
select 10807,'1964-S199','1964-04-11 00:00:00','suborbital rocket','success',446,'D-09','FU-104B Probe/Na','','','Aeronomy',664,'',400 from dual;
insert into LAUNCH
select 10808,'1964-S20','1964-01-09 00:00:00','military missile','success',976,'','8K63 KY64-1','','','Test',636,'',402 from dual union all
select 10809,'1964-S200','1964-04-11 00:00:00','suborbital rocket','success',288,'C58','Na/TMA','','','Aeronomy',664,'',188 from dual union all
select 1081,'1951-A92','1951-12-11 00:00:00','military missile','success',703,'598','','','','Test',222,'',8 from dual union all
select 10810,'1964-S201','1964-04-11 09:19:00','suborbital rocket','success',1199,'SL136','MO-4 Ozone dist','','','Aeronomy',1839,'',200 from dual union all
select 10811,'1964-S202','1964-04-13 19:00:00','military missile','success',795,'734','Mk 11 RV','OT','COPPER FACE','OT',1702,'',900 from dual union all
select 10812,'1964-S203','1964-04-13 00:00:00','suborbital rocket','success',446,'D-06','FU-104B Probe/Na','','','Aeronomy',664,'',425 from dual union all
select 10813,'1964-S204','1964-04-13 00:00:00','suborbital rocket','success',288,'C59','Na/TMA','','','Aeronomy',664,'',190 from dual union all
select 10814,'1964-S205','1964-04-14 00:00:00','suborbital rocket','success',1490,'AGI51','Solar La','','','Solar UV',668,'',119 from dual union all
select 10815,'1964-S206','1964-04-16 06:39:00','suborbital rocket','success',837,'NASA 14.122GE','Auroral emissions','','','Auroral',428,'',157 from dual union all
select 10816,'1964-S207','1964-04-14 08:32:00','suborbital rocket','success',535,'HAD 129','Sphere','','','Aeron',1835,'',121 from dual union all
select 10817,'1964-S208','1964-04-15 01:21:42','suborbital rocket','success',837,'NASA 14.24UA','UM Pitot 5/Fire cal','','','Aeronomy',176,'',158 from dual union all
select 10818,'1964-S209','1964-04-15 15:56:00','suborbital rocket','success',837,'NASA 14.23UA','UM Pitot 6/Fire cal','','','Aeronomy',176,'',158 from dual union all
select 10819,'1964-S21','1964-01-09 00:00:00','military missile','success',1018,'G 103   42','8K75 LKI 3-19 SLI','','','OT',578,'',1160 from dual union all
select 1082,'1951-A93','1951-12-11 00:00:00','atmospheric rocket','success',869,'490B','System Test 71R','','','Test',1895,'',10 from dual union all
select 10820,'1964-S210','1964-04-15 23:25:00','suborbital rocket','success',624,'NASA 12.03GT/GI','X-258 Test','','','Ionospheric/Test',1821,'',727 from dual union all
select 10821,'1964-S211','1964-04-16 07:16:00','suborbital rocket','success',215,'AD-2A-36','','','','Ionos/Particles/Micromet',428,'',152 from dual union all
select 10822,'1964-S212','1964-04-16 21:05:00','suborbital rocket','success',837,'NASA 14.143UI','UIUC-GCA Ionos 1','','','Ionos',1821,'',168 from dual union all
select 10823,'1964-S213','1964-04-17 00:44:00','suborbital rocket','success',215,'AF-2A-35/03','Daytime ne','','','Test/Ionos',428,'',172 from dual union all
select 10824,'1964-S214','1964-04-17 23:15:00','suborbital rocket','success',839,'NASA 10.142GA','Sphere','','','Aeronomy',1821,'',149 from dual union all
select 10825,'1964-S215','1964-04-18 00:38:00','suborbital rocket','success',839,'NASA 10.73GA','GSFC Grenade 48','','','Aeronomy',428,'',119 from dual union all
select 10826,'1964-S216','1964-04-18 00:59:00','suborbital rocket','success',839,'NASA 10.83GA','GSFC Grenade 49','','','Aeronomy',1821,'',119 from dual union all
select 10827,'1964-S217','1964-04-20 16:30:03','military missile','success',929,'A2P-286','Surface launch (1st)','','','OT',421,'SSBN 625',1000 from dual union all
select 10828,'1964-S218','1964-04-20 17:00:09','military missile','success',929,'A2P-119','Submerged launch','','','OT',421,'SSBN 625',1000 from dual union all
select 10829,'1964-S219','1964-04-21 23:00:00','military missile','success',916,'AO-12','P-78','','','OT',435,'',180 from dual union all
select 1083,'1951-A94','1951-12-11 00:00:00','atmospheric rocket','success',869,'490B','System Test 72R','','','Test',1895,'',10 from dual union all
select 10830,'1964-S22','1964-01-09 05:22:00','suborbital rocket','success',215,'AA-2-41','Auroral particles','','','Auroral/Ionos/Micromet',428,'',115 from dual union all
select 10831,'1964-S220','1964-04-21 06:15:00','military missile','success',1373,'290/20003','AFP-437 CTL','','','Interceptor',773,'',778 from dual union all
select 10832,'1964-S221','1964-04-21 23:56:00','suborbital rocket','success',217,'FF-3-09','BB3 09','','','Test',428,'',150 from dual union all
select 10833,'1964-S222','1964-04-22 06:54:00','suborbital rocket','success',837,'NASA 14.123GE','Auroral emissions','','','Auroral',428,'',166 from dual union all
select 10834,'1964-S223','1964-04-23 11:00:00','military missile','success',795,'917','Mk 11 RV','OT','DARK LADY','OT',1698,'',900 from dual union all
select 10835,'1964-S224','1964-04-24 00:30:00','military missile','success',916,'AO-13','P-79','','','OT',435,'',180 from dual union all
select 10836,'1964-S225','1964-04-24 20:14:17','military missile','success',795,'444','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 10837,'1964-S227','1964-04-27 00:00:00','military missile','success',794,'605','Mk 5 RV','OT-13','DRAG OUT','OT',1700,'',900 from dual union all
select 10838,'1964-S228','1964-04-27 00:00:00','military missile','success',976,'','8K63 KY64-12 UBP 51 RB','','','Test',636,'',402 from dual union all
select 10839,'1964-S229','1964-05-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual union all
select 1084,'1951-A95','1951-12-13 00:00:00','military missile','success',771,'GM-552','','','','Test',222,'',12 from dual union all
select 10840,'1964-S23','1964-01-10 00:00:00','military missile','success',795,'883','Mk 11 RV','DASO-3','CLOCK WATCH','Test',1702,'',900 from dual union all
select 10841,'1964-S230','1964-05-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual union all
select 10842,'1964-S231','1964-05-04 00:00:00','suborbital rocket','success',288,'C60','CNET A','','','Ionos',664,'',148 from dual union all
select 10843,'1964-S232','1964-05-06 16:00:00','suborbital rocket','success',30,'NE3.154','','','','Solar EUV',1869,'',238 from dual union all
select 10844,'1964-S233','1964-05-07 00:00:00','military missile','success',795,'1002','Mk 11 RV','OT','BLACK WASP','OT',1697,'',900 from dual union all
select 10845,'1964-S234','1964-05-07 02:31:00','suborbital rocket','unknown',839,'','','','','Aeron',1057,'',110 from dual union all
select 10846,'1964-S235','1964-05-08 18:22:00','suborbital rocket','success',288,'C33','CNET A','','','Ionos',664,'',151 from dual union all
select 10847,'1964-S236','1964-05-10 00:00:00','suborbital rocket','failure',288,'C22','Seddon','','','Aeronomy',664,'',28 from dual union all
select 10848,'1964-S237','1964-05-10 00:00:00','suborbital rocket','success',288,'C46','Na','','','Aeronomy',664,'',179 from dual union all
select 10849,'1964-S238','1964-05-11 00:00:00','military missile','success',794,'718','Mk 5 RV','OT','DIP NET','OT',1701,'',900 from dual union all
select 1085,'1951-A96','1951-12-14 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 10850,'1964-S239','1964-05-12 00:00:00','suborbital rocket','success',865,'Sandia 152-52','Ballistic','','','Aeron?',855,'',313 from dual union all
select 10851,'1964-S24','1964-01-12 00:00:00','military missile','success',1018,'T 103   45','8K75 LKI 3-20 SLI','','','OT',580,'',1160 from dual union all
select 10852,'1964-S240','1964-05-12 11:25:00','suborbital rocket','success',839,'Falling Sphere 13','','','','Aeronomy',871,'',144 from dual union all
select 10853,'1964-S242','1964-05-14 00:00:00','suborbital rocket','success',865,'Sandia 152-50','Ballistic','','','Aeron?',855,'',313 from dual union all
select 10854,'1964-S243','1964-05-14 00:00:00','suborbital rocket','success',288,'C23','Seddon','','','Aeronomy',664,'',129 from dual union all
select 10855,'1964-S244','1964-05-14 04:31:00','suborbital rocket','success',1199,'SL20','RAE Test 24','','','Test',1839,'',182 from dual union all
select 10856,'1964-S245','1964-05-14 08:37:00','suborbital rocket','success',535,'HAD 126','Sphere','','','Aeron',1835,'',88 from dual union all
select 10857,'1964-S246','1964-05-15 00:00:00','suborbital rocket','success',865,'Sandia 152-51','Ballistic','','','Aeron?',855,'',325 from dual union all
select 10858,'1964-S247','1964-05-15 12:01:00','military missile','success',1018,'R 15001-03','8K75 UBP-1','','','OT',584,'',1160 from dual union all
select 10859,'1964-S248','1964-05-15 06:56:00','military missile','success',1457,'','8K81 LKI-3','','','Test',590,'',689 from dual union all
select 1086,'1951-A97','1951-12-17 00:00:00','atmospheric rocket','unknown',576,'14','','','','Test',1895,'',10 from dual union all
select 10860,'1964-S249','1964-05-18 00:00:00','military missile','success',795,'997','Mk 11 RV','OT','DRESS BLUES','OT',1698,'',900 from dual union all
select 10861,'1964-S25','1964-01-12 00:26:00','suborbital rocket','success',837,'NASA 14.130IA','ISRO 10.03','TERLS-003','','Aeronomy Na',1467,'',187 from dual union all
select 10862,'1964-S250','1964-05-18 00:00:00','suborbital rocket','success',865,'Sandia 152-46','Ballistic HRT','','','Aeron?',855,'',300 from dual union all
select 10863,'1964-S251','1964-05-18 06:20:00','suborbital rocket','success',837,'AE5.479','','','','Test',394,'',208 from dual union all
select 10864,'1964-S252','1964-05-18 06:20:00','suborbital rocket','success',837,'','REDLAMP TMA','','','Aeron',293,'',152 from dual union all
select 10865,'1964-S253','1964-05-18 18:35:00','suborbital rocket','success',837,'','REDLAMP SF6','','','Aeron',293,'',190 from dual union all
select 10866,'1964-S254','1964-05-19 06:45:00','suborbital rocket','success',837,'AE5.481 -','REDLAMP TMA','','','Aeron/Test',293,'',150 from dual union all
select 10867,'1964-S255','1964-05-19 18:00:00','suborbital rocket','success',837,'AE5.482 -','REDLAMP SF6','','','Aeron/Test',293,'',267 from dual union all
select 10868,'1964-S256','1964-05-20 00:00:00','suborbital rocket','success',837,'','Noctilucent clouds','','','Aeron',1895,'',200 from dual union all
select 10869,'1964-S257','1964-05-20 00:00:00','suborbital rocket','success',864,'Sandia 152-22','Stovepipe Ballistic','','','Test?',855,'',142 from dual union all
select 1087,'1951-A98','1951-12-17 00:00:00','atmospheric rocket','unknown',576,'15','','','','Test',1895,'',10 from dual union all
select 10870,'1964-S258','1964-05-20 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 10871,'1964-S259','1964-05-20 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 10872,'1964-S26','1964-01-13 16:30:04','military missile','failure',929,'A2P-243','PY-1 DASO','','','OT',421,'SSBN 623',100 from dual union all
select 10873,'1964-S260','1964-05-21 00:00:00','suborbital rocket','success',866,'Sandia 152-10','Density','','','Aeron',855,'',329 from dual union all
select 10874,'1964-S261','1964-05-23 04:59:50','military missile','failure',999,'G 22500-09L','8K67 LKI-7','','','Test',573,'',870 from dual union all
select 10875,'1964-S262','1964-05-25 00:00:00','military missile','success',795,'889','Mk 11 RV','OT','DEER HORN','OT',1702,'',900 from dual union all
select 10876,'1964-S263','1964-05-25 00:00:00','suborbital rocket','success',865,'Sandia 152-28','LASL Sun','','','Solar',855,'',290 from dual union all
select 10877,'1964-S264','1964-05-25 18:29:05','military missile','success',932,'A3P-6','','','','Test',421,'SSBN 626',1000 from dual union all
select 10878,'1964-S265','1964-05-25 19:15:06','military missile','success',932,'A3P-1','','','','Test',421,'SSBN 626',1000 from dual union all
select 10879,'1964-S266','1964-05-27 00:00:00','suborbital rocket','success',865,'Sandia 152-30','LASL Sun','','','Solar',855,'',290 from dual union all
select 1088,'1951-A99','1951-12-18 00:00:00','atmospheric rocket','success',869,'490B','System Test 73R','','','Test',1895,'',10 from dual union all
select 10880,'1964-S267','1964-05-27 04:30:00','suborbital rocket','success',30,'AD3.355','Lunar X-ray','','','Test/XR Astron',1869,'',245 from dual union all
select 10881,'1964-S268','1964-05-28 00:00:00','suborbital rocket','success',30,'HITAB','TSRA-1','','','IR Background',1786,'',200 from dual union all
select 10882,'1964-S269','1964-05-28 07:32:00','military missile','failure',1373,'227/20007','AFP-437 CTL','','','Interceptor',773,'',932 from dual union all
select 10883,'1964-S27','1964-01-13 21:50:01','military missile','success',929,'A2P-244','PY-1 DASO','','','OT',421,'SSBN 623',1000 from dual union all
select 10884,'1964-S270','1964-05-30 06:31:00','military missile','success',982,'R 22503-20T','8K64U UBP-42 K','','','OT',566,'',1210 from dual union all
select 10885,'1964-S271','1964-05-30 02:30:30','military missile','success',999,'G 22500-10L','8K67 LKI-8','','','Test',574,'',864 from dual union all
select 10886,'1964-S272','1964-05-30 04:02:51','military missile','success',1457,'','8K81 LKI-4','','','Test',590,'',695 from dual union all
select 10887,'1964-S273','1964-06-01 00:00:00','military missile','success',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 10888,'1964-S274','1964-06-04 00:00:00','suborbital rocket','success',837,'','Propagation study','','','Ionos',1895,'',189 from dual union all
select 10889,'1964-S275','1964-06-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual union all
select 1089,'1951-M01','1951-01-16 00:00:00','military missile','success',340,'7R','','','','Test',1863,'',50 from dual union all
select 10890,'1964-S276','1964-06-08 00:00:00','military missile','success',980,'','8K65U UBP-2 29 RD','','','Test',636,'',675 from dual union all
select 10891,'1964-S277','1964-06-03 06:04:00','military missile','success',1017,'E 15003- 15','8K74A UBP-15','','','OT',562,'',1350 from dual union all
select 10892,'1964-S278','1964-06-03 18:45:00','suborbital rocket','success',837,'NASA 14.33GI','','','','Ionos',1821,'',140 from dual union all
select 10893,'1964-S279','1964-06-04 23:44:00','test rocket','success',251,'F-1','','','','-',1845,'',157 from dual union all
select 10894,'1964-S28','1964-01-14 00:00:00','suborbital rocket','success',866,'Sandia 152-21','Ballistic','','','Test',855,'',292 from dual union all
select 10895,'1964-S280','1964-06-05 00:00:00','military missile','success',976,'','8K63 KY64-13 UBP 29 RD','','','Test',636,'',402 from dual union all
select 10896,'1964-S282','1964-06-08 16:34:07','military missile','success',932,'A3P-2','','','','Test',421,'SSBN 626',1000 from dual union all
select 10897,'1964-S283','1964-06-08 17:09:07','military missile','success',932,'A3P-4','DASO','','','Test',421,'SSBN 626',1000 from dual union all
select 10898,'1964-S284','1964-06-08 00:00:00','suborbital rocket','success',1491,'VA75','','','','Test',668,'',260 from dual;
insert into LAUNCH
select 10899,'1964-S285','1964-06-09 00:00:00','military missile','success',794,'532','Mk 5 RV','OT','GOLD CLUB','OT',1700,'',900 from dual union all
select 109,'1944-S09','1944-02-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1090,'1951-M02','1951-03-22 00:00:00','military missile','success',340,'8R','','','','Test',1863,'',50 from dual union all
select 10900,'1964-S286','1964-06-10 05:30:00','sounding rocket','success',1068,'R1','A-1 capsule','','','Test',664,'',1800 from dual union all
select 10901,'1964-S287','1964-06-10 05:03:00','suborbital rocket','success',130,'CRL AC15.373','Sphere','','','Aeron',1023,'',343 from dual union all
select 10902,'1964-S288','1964-06-10 12:44:59','suborbital rocket','success',30,'NASA 04.67NP','Paraglider','','','Tech/Meteoroid',1869,'',155 from dual union all
select 10903,'1964-S289','1964-06-10 15:10:00','suborbital rocket','success',837,'NASA 14.155GE','B Field 1','','','Magnetospheric',1821,'',146 from dual union all
select 10904,'1964-S29','1964-01-14 00:00:00','military missile','success',976,'','8K63 KY64-2','','','Test',636,'',402 from dual union all
select 10905,'1964-S290','1964-06-11 00:00:00','military missile','success',794,'715','Mk 5 RV','OT','HIGH OCTANE','OT',1701,'',900 from dual union all
select 10906,'1964-S291','1964-06-11 00:00:00','military missile','success',976,'','8K63 KY64-14','','','Test',636,'',402 from dual union all
select 10907,'1964-S292','1964-06-11 08:31:00','suborbital rocket','success',535,'HAD 128','Sphere','','','Aeron',1835,'',113 from dual union all
select 10908,'1964-S293','1964-06-12 05:03:00','sounding rocket','success',1068,'R2','A-1 capsule','','','Test',664,'',1800 from dual union all
select 10909,'1964-S294','1964-06-13 00:00:00','suborbital rocket','success',1491,'VA76','','','','Test',668,'',260 from dual union all
select 1091,'1951-M03','1951-03-28 23:14:00','suborbital rocket','success',50,'USAF 10','GRD-2 NaCl release','','','Aeronomy',640,'',66 from dual union all
select 10910,'1964-S295','1964-06-15 12:40:00','military missile','success',795,'910','Mk 11 RV','OT','CRUSH PROOF','OT',1697,'',900 from dual union all
select 10911,'1964-S296','1964-06-16 09:00:00','suborbital rocket','success',30,'NB3.147','','','','UV Astron/XR Astron',1869,'',127 from dual union all
select 10912,'1964-S297','1964-06-17 01:01:00','suborbital rocket','success',839,'Falling Sphere 14','','','','Aeronomy',871,'',140 from dual union all
select 10913,'1964-S298','1964-06-17 04:00:31','military missile','success',1457,'','8K81 LKI-5','','','Test',590,'',694 from dual union all
select 10914,'1964-S299','1964-06-18 02:19:00','suborbital rocket','success',839,'Falling Sphere 15','','','','Aeronomy',871,'',149 from dual union all
select 10915,'1964-S30','1964-01-14 06:05:00','suborbital rocket','success',215,'AA-2-25','Auroral particles','','','Auroral/Ionos/Micromet',428,'',155 from dual union all
select 10916,'1964-S300','1964-06-18 14:56:13','ballistic missile test','success',154,'243D','ABRES LORV-1/Pod 31/GTV1','','IRON LUNG','Reentry Test',1662,'',1800 from dual union all
select 10917,'1964-S301','1964-06-18 16:30:00','suborbital rocket','success',839,'Falling Sphere 16','','','','Aeronomy',871,'',148 from dual union all
select 10918,'1964-S302','1964-06-19 01:29:00','suborbital rocket','success',839,'PMR-15','','','','Aeronomy',873,'',100 from dual union all
select 10919,'1964-S303','1964-06-19 13:46:00','suborbital rocket','success',30,'KP3.6','Dayglow','','','Aeron',1869,'',124 from dual union all
select 1092,'1951-M04','1951-04-18 18:39:00','suborbital rocket','success',50,'USAF 12','Aeromed 1','','','Biology',640,'',61 from dual union all
select 10920,'1964-S304','1964-06-19 15:40:00','suborbital rocket','success',839,'PMR-16','','','','Aeronomy',873,'',100 from dual union all
select 10921,'1964-S305','1964-06-22 00:00:00','military missile','success',979,'','8K65 UBP-54','','','Test',636,'',675 from dual union all
select 10922,'1964-S306','1964-06-23 18:16:03','military missile','success',929,'A2P-296','','','','OT',421,'SSBN 620',1000 from dual union all
select 10923,'1964-S307','1964-06-23 19:53:03','military missile','success',929,'A2P-297','','','','OT',421,'SSBN 620',1000 from dual union all
select 10924,'1964-S308','1964-06-23 21:31:02','military missile','success',929,'A2P-319','','','','OT',421,'SSBN 620',1000 from dual union all
select 10925,'1964-S309','1964-06-24 04:42:27','military missile','success',999,'G 22500-12L','8K67 LKI-9','','','Test',574,'',874 from dual union all
select 10926,'1964-S31','1964-01-15 20:30:19','ballistic missile test','success',1420,'N-31','Mk 6 RV/Pod T','R&D','','Test',230,'',1300 from dual union all
select 10927,'1964-S310','1964-06-25 14:57:39','suborbital rocket','success',837,'NASA 14.156GE','B Field 2','','','Magnetospheric',1821,'',150 from dual union all
select 10928,'1964-S311','1964-06-26 00:52:30','suborbital rocket','success',837,'NASA 14.157GE','B Field 3','','','Magnetospheric',1821,'',156 from dual union all
select 10929,'1964-S312','1964-06-29 21:00:00','military missile','success',795,'1141','Mk 11 RV','R&D','ELM BRANCH','Test',1705,'',900 from dual union all
select 1093,'1951-M05','1951-06-09 06:11:00','suborbital rocket','success',59,'SC 18','Grenades','','','Aeronomy',1869,'',67 from dual union all
select 10930,'1964-S313','1964-06-30 00:00:00','military missile','success',794,'631','Mk 5 RV','OT','BLACK PEPPER','OT',1699,'',900 from dual union all
select 10931,'1964-S314','1964-06-29 00:00:00','suborbital rocket','failure',865,'Sandia 152-57','Ballistic','','','Aeron?',855,'',0 from dual union all
select 10932,'1964-S315','1964-06-29 00:00:00','military missile','success',353,'','','','','Test',811,'',200 from dual union all
select 10933,'1964-S318','1964-07-01 01:30:00','military missile','success',999,'G 22500-13L','8K67 LKI-10','','','Test',574,'',872 from dual union all
select 10934,'1964-S319','1964-07-01 00:00:00','military missile','success',976,'','8K63 KY64-15','','','Test',636,'',402 from dual union all
select 10935,'1964-S32','1964-01-15 02:31:00','suborbital rocket','success',837,'NASA 14.150UE Sammy','Rice Airglow/Aur. 1','','','Airglow',1821,'',90 from dual union all
select 10936,'1964-S321','1964-07-02 00:00:00','suborbital rocket','success',865,'Sandia 152-58','Ballistic','','','Aeron?',855,'',298 from dual union all
select 10937,'1964-S322','1964-07-02 00:24:03','military missile','success',932,'A3X-10','PY-2','','','Test',244,'',1000 from dual union all
select 10938,'1964-S323','1964-07-02 00:00:00','military missile','success',976,'','8K63 KY64-16 UBP 29 RD','','','Test',636,'',402 from dual union all
select 10939,'1964-S324','1964-07-05 00:00:00','suborbital rocket','unknown',837,'','','','','Aeron',1895,'',200 from dual union all
select 1094,'1951-M06','1951-07-12 00:00:00','military missile','success',340,'9R','','','','Test',1863,'',50 from dual union all
select 10940,'1964-S325','1964-07-06 00:00:00','suborbital rocket','success',865,'Sandia 152-62','Ballistic (Sun)','','','Solar?',855,'',288 from dual union all
select 10941,'1964-S326','1964-07-06 19:23:00','suborbital rocket','success',1199,'ESRO S01/1','ESRO S01/1 MPE Ba-7','','','Aeronomy',1343,'',198 from dual union all
select 10942,'1964-S327','1964-07-07 19:00:00','military missile','success',795,'1007','Mk 11 RV','OT','NORMAL TIMES','OT',1697,'',900 from dual union all
select 10943,'1964-S328','1964-07-07 00:00:00','military missile','success',976,'','8K63 KY64-17','','','Test',636,'',402 from dual union all
select 10944,'1964-S329','1964-07-07 22:43:00','suborbital rocket','success',30,'NB3.164','Crab lunar occ.','','','XR Astron',1895,'',295 from dual union all
select 10945,'1964-S33','1964-01-15 22:40:00','suborbital rocket','success',837,'NASA 14.106CA','GCA Release 30 Na','','','Aeronomy',1821,'',114 from dual union all
select 10946,'1964-S330','1964-07-08 19:15:03','military missile','success',929,'A2P-295','','','','OT',421,'SSBN 620',1000 from dual union all
select 10947,'1964-S331','1964-07-08 20:15:03','military missile','success',929,'A2P-155','','','','OT',421,'SSBN 620',1000 from dual union all
select 10948,'1964-S332','1964-07-08 21:16:02','military missile','success',929,'A2P-232','','','','OT',421,'SSBN 620',1000 from dual union all
select 10949,'1964-S333','1964-07-08 00:00:00','military missile','success',976,'','8K63 KY64-18','','','Test',636,'',402 from dual union all
select 1095,'1951-M07','1951-07-25 16:26:00','suborbital rocket','success',50,'USAF 15','Airglow/NLC Imager','','','Aeron',640,'',71 from dual union all
select 10950,'1964-S334','1964-07-08 19:22:00','suborbital rocket','success',1199,'ESRO S01/2','ESRO S01/2 MPE Ba-8','','','Aeronomy',1343,'',183 from dual union all
select 10951,'1964-S335','1964-07-08 21:23:00','military missile','success',135,'A003','Mk 11 decoy','ABRES B-1 (USAF)','','Reentry Test',612,'',260 from dual union all
select 10952,'1964-S336','1964-07-09 19:00:00','military missile','success',795,'973','Mk 11 RV','OT','GEORGIA BOY','OT',1702,'',900 from dual union all
select 10953,'1964-S337','1964-07-09 00:00:00','military missile','success',353,'','','','','Test',811,'',200 from dual union all
select 10954,'1964-S338','1964-07-09 04:13:00','suborbital rocket','success',837,'NASA 14.154UE','Rice Airglow/Aur. 5','','','Aeronomy',1821,'',167 from dual union all
select 10955,'1964-S339','1964-07-09 08:39:00','suborbital rocket','success',535,'HAD 141','Sphere','','','Aeron',1835,'',111 from dual union all
select 10956,'1964-S34','1964-01-17 00:29:59','military missile','success',795,'438','Mk 11 RV','R&D','','Test',266,'',900 from dual union all
select 10957,'1964-S340','1964-07-10 00:00:00','military missile','success',976,'','8K63 KY64-19','','','Test',636,'',402 from dual union all
select 10958,'1964-S341','1964-07-11 00:00:00','military missile','success',353,'','','','','Test',811,'',200 from dual union all
select 10959,'1964-S342','1964-07-11 02:02:00','suborbital rocket','success',697,'L-3-1','','','','Ionos',834,'',857 from dual union all
select 1096,'1951-M08','1951-08-30 22:40:00','suborbital rocket','success',52,'USAF 17','Rocket test','','','Test',640,'',75 from dual union all
select 10960,'1964-S343','1964-07-13 00:00:00','military missile','success',795,'861','Mk 11 RV','OT','OLD FOX','OT',1698,'',900 from dual union all
select 10961,'1964-S344','1964-07-13 03:04:00','suborbital rocket','success',535,'HAD 130','Sphere','','','Aeron',1835,'',122 from dual union all
select 10962,'1964-S345','1964-07-14 00:00:00','military missile','success',976,'','8K63 KY64-20','','','Test',636,'',402 from dual union all
select 10963,'1964-S346','1964-07-14 03:00:00','suborbital rocket','success',535,'HAD 131','Sphere','','','Aeron',1835,'',122 from dual union all
select 10964,'1964-S347','1964-07-15 00:47:00','suborbital rocket','success',1198,'SL31','RAE Test 25','','','Test',1839,'',81 from dual union all
select 10965,'1964-S348','1964-07-15 00:58:00','suborbital rocket','success',837,'NASA 14.49CA','GCA Release 33 Na','','','Aeronomy',1821,'',191 from dual union all
select 10966,'1964-S349','1964-07-15 02:19:00','suborbital rocket','success',535,'HAD 133','Sphere','','','Aeron',1835,'',106 from dual union all
select 10967,'1964-S35','1964-01-16 00:00:00','military missile','success',795,'870','Mk 11 RV','DASO-4','DOUBLE BARREL','Test',1697,'',900 from dual union all
select 10968,'1964-S350','1964-07-15 04:09:00','suborbital rocket','success',837,'NASA 14.50CA','GCA Release 34 Na','','','Aeronomy',1821,'',192 from dual union all
select 10969,'1964-S351','1964-07-15 08:00:00','suborbital rocket','success',837,'NASA 14.144UI','UIUC-GCA Ionos 2/2a','','','Ionos',1821,'',154 from dual union all
select 1097,'1951-M09','1951-09-13 11:37:00','suborbital rocket','success',50,'USAF 18','UM T/p','','','Aeronomy',640,'',76 from dual union all
select 10970,'1964-S352','1964-07-15 08:05:00','suborbital rocket','success',837,'NASA 14.51CA','GCA Release 35 Na','','','Aeronomy',1821,'',190 from dual union all
select 10971,'1964-S353','1964-07-15 09:06:00','suborbital rocket','success',837,'NASA 14.52CA','GCA Release 36 Na','','','Aeronomy',1821,'',191 from dual union all
select 10972,'1964-S354','1964-07-15 09:20:00','suborbital rocket','success',837,'NASA 14.145UI','UIUC-GCA Ionos 3/2b','','','Ionos',1821,'',161 from dual union all
select 10973,'1964-S355','1964-07-15 10:25:00','suborbital rocket','success',837,'NASA 14.146UI','UIUC-GCA Ionos 4/2c','','','Ionos',1821,'',171 from dual union all
select 10974,'1964-S356','1964-07-15 17:30:00','suborbital rocket','success',865,'Sandia 152-18','LRL SAP Camera','','','Test/Tech',855,'',270 from dual union all
select 10975,'1964-S357','1964-07-16 16:54:04','military missile','success',932,'A3P-19','','','','OT',421,'SSBN 629',1000 from dual union all
select 10976,'1964-S358','1964-07-16 17:24:07','military missile','failure',932,'A3P-3','','','','OT',421,'SSBN 629',500 from dual union all
select 10977,'1964-S359','1964-07-16 02:13:00','suborbital rocket','success',535,'HAD 134','Sphere','','','Aeron',1835,'',106 from dual union all
select 10978,'1964-S36','1964-01-16 04:28:58','military missile','success',999,'G 22500-05L','8K67 LKI-2','','','Test',573,'',908 from dual union all
select 10979,'1964-S360','1964-07-16 05:30:00','suborbital rocket','success',535,'HAD 136','','','','Aeron',219,'',114 from dual union all
select 1098,'1951-M10','1951-09-20 16:31:00','suborbital rocket','success',50,'USAF 19','Aeromed 2','','','Biology',640,'',71 from dual union all
select 10980,'1964-S361','1964-07-16 16:22:00','suborbital rocket','success',837,'NASA 14.127GI','Antenna impedance','','','Ionos/RF Astron',1821,'',137 from dual union all
select 10981,'1964-S362','1964-07-17 00:00:00','suborbital rocket','success',30,'HITAB','TIRA-2','','','Test/IR Background',1786,'',90 from dual union all
select 10982,'1964-S363','1964-07-17 00:00:00','military missile','success',916,'AO-14','P-80','','','OT',203,'',180 from dual union all
select 10983,'1964-S364','1964-07-17 00:00:00','military missile','success',976,'','8K63 60 RD UBP-6','','','Test',1354,'',402 from dual union all
select 10984,'1964-S365','1964-07-17 00:00:00','military missile','success',976,'','8K63 KY64-21','','','Test',636,'',402 from dual union all
select 10985,'1964-S366','1964-07-17 04:35:00','suborbital rocket','success',1198,'SL32','RAE Test 26','','','Test',1839,'',90 from dual union all
select 10986,'1964-S367','1964-07-17 05:15:00','suborbital rocket','success',535,'HAD 137','','','','Aeron',219,'',120 from dual union all
select 10987,'1964-S368','1964-07-17 08:24:00','suborbital rocket','success',837,'Missile compression','REDLAMP','','','Test/Aeron',293,'',105 from dual union all
select 10988,'1964-S369','1964-07-18 04:30:00','military missile','success',983,'KY-2','','','','Test',636,'',250 from dual;
insert into LAUNCH
select 10989,'1964-S37','1964-01-16 05:00:00','suborbital rocket','success',837,'NASA 14.125CA','GCA Release 31 Na','','','Aeronomy',1821,'',170 from dual union all
select 1099,'1951-M11','1951-09-27 00:06:00','suborbital rocket','success',59,'SC 21','Sampling','','','Aeron',1869,'',69 from dual union all
select 10990,'1964-S370','1964-07-18 12:22:00','suborbital rocket','success',130,'CRL AC15.363','Dayglow spectra','','','Aeron',428,'',207 from dual union all
select 10991,'1964-S371','1964-07-20 10:53:00','ballistic missile test','success',1148,'S124R','SERT 1','','','Tech',1814,'',4002 from dual union all
select 10992,'1964-S372','1964-07-20 17:30:00','suborbital rocket','success',865,'Sandia 152-20','LRL SAP Camera','','','Tech',855,'',287 from dual union all
select 10993,'1964-S373','1964-07-21 00:00:00','suborbital rocket','success',865,'Sandia 152-31','LASL Sun','','','Solar',855,'',270 from dual union all
select 10994,'1964-S375','1964-07-22 00:00:00','suborbital rocket','success',865,'Sandia 152-33','LASL Sun','','','Solar',855,'',321 from dual union all
select 10995,'1964-S376','1964-07-22 00:00:00','military missile','success',916,'AO-15','P-81','','','OT',203,'',180 from dual union all
select 10996,'1964-S377','1964-07-22 02:50:00','suborbital rocket','success',692,'LS-A-1','','','','Test',990,'',60 from dual union all
select 10997,'1964-S378','1964-07-22 08:55:00','suborbital rocket','success',692,'LS-A-2','','','','Test',990,'',60 from dual union all
select 10998,'1964-S379','1964-07-23 18:43:00','suborbital rocket','success',30,'NASA 04.107GE','CR nuclei','','','Cosmic rays',428,'',233 from dual union all
select 10999,'1964-S38','1964-01-16 11:34:00','suborbital rocket','success',837,'NASA 14.126CA','GCA Release 32 Na','','','Aeronomy',1821,'',200 from dual union all
select 11,'1943-A04','1943-02-19 00:00:00','military missile','failure',8,'13','','','','Test',675,'',1 from dual union all
select 110,'1944-S10','1944-02-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1100,'1951-M12','1951-11-01 09:45:00','suborbital rocket','success',59,'SC 20','Grenades','','','Aeronomy',1869,'',66 from dual union all
select 11000,'1964-S380','1964-07-25 17:35:00','suborbital rocket','success',30,'NASA 04.108GE','CR nuclei','','','Cosmic rays',428,'',215 from dual union all
select 11001,'1964-S381','1964-07-26 09:07:00','suborbital rocket','success',666,'K-8L-4','','','','Aeron',839,'',124 from dual union all
select 11002,'1964-S382','1964-07-26 10:51:00','suborbital rocket','success',666,'K-8L-5','','','','Aeron',839,'',112 from dual union all
select 11003,'1964-S383','1964-07-27 00:00:00','military missile','success',794,'617','Mk 5 RV','OT','GINGER FOOT','OT',1701,'',900 from dual union all
select 11004,'1964-S384','1964-07-27 14:00:00','military missile','success',1017,'L1-7','8K74 UBP-16','','','OT',562,'',1350 from dual union all
select 11005,'1964-S385','1964-07-28 20:06:00','suborbital rocket','success',837,'','Sphere','','','Aeron',1863,'',200 from dual union all
select 11006,'1964-S386','1964-07-28 21:14:00','suborbital rocket','success',42,'NASA 06.10GA','Thermosphere probe','','','Aeronomy',428,'',322 from dual union all
select 11007,'1964-S387','1964-07-29 09:22:56','military missile','success',154,'248D','Nike TVX-30/TV-32 x2','NTMP KX-13','KNOCK WOOD','Target',1664,'',1800 from dual union all
select 11008,'1964-S388','1964-07-29 09:30:00','suborbital rocket','success',837,'','Sphere','','','Aeron',1895,'',200 from dual union all
select 11009,'1964-S389','1964-07-29 10:07:00','suborbital rocket','success',668,'K-9M-5','','','','CR',839,'',300 from dual union all
select 1101,'1951-S01','1951-01-22 22:55:00','suborbital rocket','success',54,'A19','Mass spectrometer','','','Aeronomy',1869,'',88 from dual union all
select 11010,'1964-S39','1964-01-17 05:01:00','suborbital rocket','success',624,'NASA 8.31DA','Composition/Airglow','','','Aeronomy',1821,'',925 from dual union all
select 11011,'1964-S390','1964-07-30 00:00:00','suborbital rocket','success',865,'Sandia 152-48','Density','','','Aeron',855,'',311 from dual union all
select 11012,'1964-S391','1964-07-30 16:30:03','military missile','success',932,'A3P-14','','','','OT',421,'SSBN 629',1000 from dual union all
select 11013,'1964-S392','1964-07-30 17:00:03','military missile','success',932,'A3P-32','','','','OT',421,'SSBN 629',1000 from dual union all
select 11014,'1964-S393','1964-07-30 15:20:36','military missile','success',1420,'B-28  (62-009)','Mk 6 RV','DASO-1','COBRA SKIN','OT',1659,'',1300 from dual union all
select 11015,'1964-S394','1964-08-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',150 from dual union all
select 11016,'1964-S395','1964-08-01 00:00:00','military missile','success',837,'BMTS','SPEEDBALL II 55 SN24','','','Target',1895,'',150 from dual union all
select 11017,'1964-S396','1964-07-31 00:00:00','military missile','success',976,'','8K63 KY64-22','','','Test',636,'',402 from dual union all
select 11018,'1964-S397','1964-07-31 08:05:00','military missile','success',982,'R 22504-20T','8K64U UBP-43','','','OT',566,'',1210 from dual union all
select 11019,'1964-S398','1964-08-01 01:13:00','suborbital rocket','success',446,'D-10','FU-125 Ions/Part.','','','Aeronomy/Ionos',1749,'',440 from dual union all
select 1102,'1951-S02','1951-01-25 15:00:00','suborbital rocket','success',54,'A20','Ozone','','','Aeron',1869,'',90 from dual union all
select 11020,'1964-S399','1964-08-01 05:54:49','military missile','success',1457,'','8K81 LKI-6','','','Test',590,'',686 from dual union all
select 11021,'1964-S40','1964-01-20 17:02:01','military missile','success',932,'A3X-57 (A3Y-1)','A hit SS','','','Test',421,'EAG-154',1000 from dual union all
select 11022,'1964-S400','1964-08-04 00:00:00','military missile','success',976,'','8K63 KY64-23','','','Test',636,'',402 from dual union all
select 11023,'1964-S401','1964-08-05 00:00:00','suborbital rocket','success',865,'Sandia 152-43','LASL Sun','','','Solar',855,'',300 from dual union all
select 11024,'1964-S402','1964-08-05 02:00:29','military missile','success',999,'G 22500-14L','8K67 LKI-11','','','Test',574,'',1493 from dual union all
select 11025,'1964-S403','1964-08-05 17:15:05','military missile','success',243,'BK.19','H-210/Dazzle 1','','','Reentry Test',1842,'',602 from dual union all
select 11026,'1964-S404','1964-08-06 00:00:00','military missile','success',976,'','8K63 KY64-24','','','Test',636,'',402 from dual union all
select 11027,'1964-S405','1964-08-06 00:00:00','military missile','success',976,'','8K63 60 RD UBP-8','','','Test',1354,'',402 from dual union all
select 11028,'1964-S406','1964-08-06 23:29:00','suborbital rocket','success',837,'NASA 14.55DA','RTG K64-S-1 Air samp','','','Aeronomy',920,'',125 from dual union all
select 11029,'1964-S407','1964-08-07 00:00:00','military missile','success',795,'998','Mk 11 RV','OT','GOLD REEF','OT',1698,'',900 from dual union all
select 1103,'1951-S03','1951-01-29 00:00:00','military missile','success',969,'IIIA-1','KI','','','OT',636,'',100 from dual union all
select 11030,'1964-S408','1964-08-06 16:30:00','military missile','success',932,'A3P-7','','','','OT',421,'SSBN 628',1000 from dual union all
select 11031,'1964-S409','1964-08-07 00:04:00','suborbital rocket','success',446,'D-11','FU-125 Ions/Part.','','','Aeronomy/Ionos',1749,'',420 from dual union all
select 11032,'1964-S41','1964-01-20 18:00:00','suborbital rocket','success',215,'CC-2A-29','Fin test','','','Test',428,'',204 from dual union all
select 11033,'1964-S410','1964-08-07 00:16:00','suborbital rocket','success',839,'NASA 10.138GA','RTG K64-G-1 Grenades','','','Aeronomy',920,'',130 from dual union all
select 11034,'1964-S411','1964-08-07 01:00:00','suborbital rocket','success',839,'NASA 10.78GA','GSFC Grenade 51 (W)','','','Aeronomy',1821,'',119 from dual union all
select 11035,'1964-S412','1964-08-07 20:12:00','military missile','success',162,'110F','Mk 4-5B','DASO','LARGE CHARGE','OT',1670,'',1400 from dual union all
select 11036,'1964-S413','1964-08-08 00:00:00','suborbital rocket','success',865,'Sandia 152-42','LASL Sun','','','Solar',855,'',307 from dual union all
select 11037,'1964-S414','1964-08-08 04:00:00','suborbital rocket','success',839,'NASA 10.104GA','GSFC Grenade 52 (C)','','','Aeronomy',428,'',139 from dual union all
select 11038,'1964-S416','1964-08-10 00:00:00','suborbital rocket','success',589,'GSP-1','LRL GSP-1','','','Magnetospheric',1281,'T-AGM-8',200 from dual union all
select 11039,'1964-S417','1964-08-11 15:22:43','military missile','success',1420,'B-9   (61-2763)','Mk 6 RV','DASO-2','DOUBLE TALLEY','OT',1658,'',1300 from dual union all
select 1104,'1951-S04','1951-01-30 00:00:00','military missile','success',969,'IIIA-5','','','','OT',636,'',100 from dual union all
select 11040,'1964-S418','1964-08-11 05:40:00','suborbital rocket','success',1199,'SL301','LUX-19AC1/Cul-1','','','Solar X,UV',1839,'',145 from dual union all
select 11041,'1964-S419','1964-08-11 10:30:00','suborbital rocket','success',30,'NASA 04.82GG','','','','UV Astron',1869,'',172 from dual union all
select 11042,'1964-S42','1964-01-20 21:15:00','suborbital rocket','success',215,'CC-2A-30','Fin test','','','Test',428,'',204 from dual union all
select 11043,'1964-S420','1964-08-12 00:43:00','suborbital rocket','success',837,'NASA 14.56DA','RTG K64-S-2 Air samp','','','Aeronomy',920,'',121 from dual union all
select 11044,'1964-S421','1964-08-12 01:11:00','suborbital rocket','success',839,'NASA 10.139GA','RTG K64-G-2 Grenades','','','Aeronomy',920,'',130 from dual union all
select 11045,'1964-S422','1964-08-12 01:49:00','suborbital rocket','success',839,'NASA 10.84GA','GSFC Grenade 53 (W)','','','Aeronomy',1821,'',114 from dual union all
select 11046,'1964-S423','1964-08-12 02:15:00','suborbital rocket','success',839,'NASA 10.105GA','GSFC Grenade 54 (C)','','','Aeronomy',428,'',126 from dual union all
select 11047,'1964-S424','1964-08-12 03:00:12','military missile','success',983,'KY-2','','','','Test',636,'',246 from dual union all
select 11048,'1964-S425','1964-08-13 14:36:32','military missile','success',1420,'B-7   (61-2761)','Mk 6 RV','DASO-3','GENTLE ANNIE','OT',1657,'',1300 from dual union all
select 11049,'1964-S426','1964-08-16 00:53:00','suborbital rocket','success',837,'NASA 14.57DA','RTG K64-S-3 Air samp','','','Aeronomy',920,'',119 from dual union all
select 1105,'1951-S05','1951-01-31 00:00:00','military missile','success',969,'IIIA-2','','','','OT',636,'',100 from dual union all
select 11050,'1964-S427','1964-08-16 01:13:00','suborbital rocket','success',839,'NASA 10.140GA','RTG K64-G-3 Grenades','','','Aeronomy',920,'',135 from dual union all
select 11051,'1964-S428','1964-08-16 03:15:00','suborbital rocket','success',839,'NASA 10.85GA','GSFC Grenade 55 (W)','','','Aeronomy',1821,'',122 from dual union all
select 11052,'1964-S429','1964-08-16 05:53:00','suborbital rocket','success',839,'NASA 10.115GA','GSFC Grenade 56 (A)','','','Aeronomy',176,'',123 from dual union all
select 11053,'1964-S43','1964-01-22 05:06:00','military missile','success',1018,'G 15001-04 (7TS)','8K75 LKI 3-21 SLI','','','OT',579,'',1160 from dual union all
select 11054,'1964-S430','1964-08-17 00:00:00','military missile','success',794,'556','Mk 5 RV','OT','LIMBER POLE','OT',1700,'',900 from dual union all
select 11055,'1964-S431','1964-08-17 00:29:00','suborbital rocket','success',837,'NASA 14.58DA','RTG K64-S-4 Air samp','','','Aeronomy',920,'',124 from dual union all
select 11056,'1964-S432','1964-08-17 00:49:00','suborbital rocket','success',839,'NASA 10.141GA','RTG K64-G-4 Grenades','','','Aeronomy',920,'',132 from dual union all
select 11057,'1964-S433','1964-08-17 12:55:00','suborbital rocket','success',839,'NASA 10.116GA','GSFC Grenade 57 (A)','','','Aeronomy',176,'',122 from dual union all
select 11058,'1964-S434','1964-08-18 01:15:00','suborbital rocket','success',839,'NASA 10.106GA','GSFC Grenade 58 (C)','','','Aeronomy',428,'',124 from dual union all
select 11059,'1964-S435','1964-08-18 01:25:00','suborbital rocket','success',839,'NASA 10.113GA','GSFC Grenade 59 (W)','','','Aeronomy',1821,'',120 from dual union all
select 1106,'1951-S06','1951-02-01 00:00:00','military missile','success',969,'IIIA-6','','','','OT',636,'',100 from dual union all
select 11060,'1964-S436','1964-08-20 16:30:02','military missile','success',932,'A3P-33','','','','OT',421,'SSBN 628',1000 from dual union all
select 11061,'1964-S437','1964-08-20 00:00:00','military missile','success',976,'','8K63 KY64-25','','','Test',636,'',402 from dual union all
select 11062,'1964-S438','1964-08-20 05:36:00','suborbital rocket','success',1199,'SL121','LUX-20 PHC/UCW-8','','','Ionos/Solar X',1839,'',192 from dual union all
select 11063,'1964-S439','1964-08-20 09:02:00','suborbital rocket','success',535,'HAD 135','Sphere','','','Aeron',1835,'',119 from dual union all
select 11064,'1964-S44','1964-01-22 05:16:00','military missile','failure',1018,'G 15001-09 (10TS)','8K75 LKI 3-22 SLI','','','OT',578,'',0 from dual union all
select 11065,'1964-S440','1964-08-21 00:00:00','military missile','success',795,'1072','Mk 11 RV','OT','LIGHT HORSE','OT',1703,'',900 from dual union all
select 11066,'1964-S441','1964-08-22 10:15:00','suborbital rocket','success',30,'NASA 04.126GG','UV Spec (Venus)','','','UV Astron',1869,'',123 from dual union all
select 11067,'1964-S442','1964-08-25 00:00:00','military missile','success',795,'989','Mk 11 RV','OT','IVY TOWER','OT',1702,'',900 from dual union all
select 11068,'1964-S443','1964-08-26 00:00:00','suborbital rocket','success',839,'','EM Absorption','','','Aeron',394,'',100 from dual union all
select 11069,'1964-S444','1964-08-26 00:00:00','military missile','success',976,'','8K63 60 RD UBP-9','','','Test',1354,'',402 from dual union all
select 1107,'1951-S07','1951-02-02 00:00:00','military missile','success',969,'IIIA-3','','','','OT',636,'',100 from dual union all
select 11070,'1964-S445','1964-08-26 17:01:00','suborbital rocket','success',837,'NASA 14.34GI','','','','Ionos',1821,'',154 from dual union all
select 11071,'1964-S446','1964-08-27 09:54:00','military missile','success',155,'57E','Mk 4E-5B','NTMP KX-48','GALLANT GAL','Target',1671,'',1600 from dual union all
select 11072,'1964-S447','1964-08-29 05:30:00','suborbital rocket','success',30,'NASA 04.122GG','AS/E/MIT-1','','','XR Astron',1869,'',179 from dual union all
select 11073,'1964-S448','1964-08-29 09:36:53','ballistic missile test','success',248,'21-2','Ion Engine Test B','','CANDY BAG','Tech',1015,'',2000 from dual union all
select 11074,'1964-S449','1964-09-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 11075,'1964-S45','1964-01-24 01:30:00','military missile','success',794,'551','Mk 5 RV','OT','BLACK BUSH','OT',1699,'',900 from dual union all
select 11076,'1964-S450','1964-09-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 11077,'1964-S451','1964-09-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual;
insert into LAUNCH
select 11078,'1964-S452','1964-09-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual union all
select 11079,'1964-S453','1964-09-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual union all
select 1108,'1951-S08','1951-02-06 17:20:00','suborbital rocket','success',54,'A21','Earth photography','','','Imaging',1869,'',98 from dual union all
select 11080,'1964-S454','1964-08-31 15:46:17','military missile','success',162,'36F','Mk 4-5B','DASO','BIG DEAL','OT',1669,'',1400 from dual union all
select 11081,'1964-S455','1964-09-01 21:51:00','military missile','success',794,'668','Mk 5 RV','OT','DAWN PATROL','OT',1699,'',900 from dual union all
select 11082,'1964-S456','1964-09-01 09:19:00','suborbital rocket','success',1199,'SL137','MO-5 Ozone dist','','','Aeronomy',1839,'',212 from dual union all
select 11083,'1964-S457','1964-09-02 03:08:00','suborbital rocket','success',35,'NASA 04.55UG','','','','UV Astron',1821,'',156 from dual union all
select 11084,'1964-S458','1964-09-04 01:20:00','suborbital rocket','failure',1197,'SL87A','RAE Test 27','','','Test',1839,'',182 from dual union all
select 11085,'1964-S459','1964-09-05 16:30:03','military missile','success',932,'A3P-13','','','','OT',421,'SSBN 631',1000 from dual union all
select 11086,'1964-S46','1964-01-23 00:00:00','military missile','success',976,'','8K63 KY64-3','','','Test',636,'',402 from dual union all
select 11087,'1964-S460','1964-09-08 23:40:00','military missile','success',794,'576','Mk 5 RV','OT','LONG LINE','OT',1701,'',900 from dual union all
select 11088,'1964-S461','1964-09-09 02:13:03','military missile','success',999,'G 22500-16L','8K67 LKI-13','','','Test',574,'',1495 from dual union all
select 11089,'1964-S462','1964-09-11 00:50:00','military missile','success',795,'1024','Mk 11 RV','OT','HOT ROOF','OT',1697,'',900 from dual union all
select 1109,'1951-S09','1951-06-08 00:11:00','suborbital rocket','success',50,'USAF 14','Solar X/NaCl release','','','Solar X/Aeron',640,'',88 from dual union all
select 11090,'1964-S463','1964-09-10 17:25:00','military missile','success',982,'R 22603-20T','8K64U UBP-44','','','OT',565,'',1210 from dual union all
select 11091,'1964-S464','1964-09-11 00:00:00','military missile','success',979,'','8K65 UBP-55 43 RD','','','Test',636,'',675 from dual union all
select 11092,'1964-S465','1964-09-13 02:57:00','suborbital rocket','success',5,'','','','','Test',636,'',100 from dual union all
select 11093,'1964-S466','1964-09-15 19:10:00','military missile','success',795,'1179','Mk 11 RV','OT/ECP VER','QUICK LAUNCH','OT',1703,'',900 from dual union all
select 11094,'1964-S467','1964-09-15 06:59:00','suborbital rocket','success',215,'AD-2A-58','','','','Ionos',428,'',166 from dual union all
select 11095,'1964-S468','1964-09-15 09:35:00','suborbital rocket','success',30,'KP3.5','Zodiacal light','','','IR Astron',1869,'',200 from dual union all
select 11096,'1964-S469','1964-09-15 15:27:00','ballistic missile test','success',154,'245D','ABRES LORV-3/Pod 26/GTV3','','BUTTERFLY NET','Reentry Test',1662,'',1800 from dual union all
select 11097,'1964-S47','1964-01-23 00:00:00','military missile','success',1420,'N-26','Mk 6 RV','R&D','RED SAILS','Test',1658,'',1300 from dual union all
select 11098,'1964-S470','1964-09-16 00:00:00','military missile','success',976,'','8K63 KY64-26','','','Test',636,'',402 from dual union all
select 11099,'1964-S471','1964-09-17 05:26:00','suborbital rocket','success',1199,'SL120','LUX-21 PHC/UCW-9','','','Ionos/Solar X',1839,'',192 from dual union all
select 111,'1944-S100','1944-09-14 02:48:00','military missile','success',1470,'','London','','','Weapon',1732,'',95 from dual union all
select 1110,'1951-S10','1951-06-13 00:00:00','military missile','success',969,'IIIB-11','','','','OT',636,'',100 from dual union all
select 11100,'1964-S472','1964-09-17 09:17:00','suborbital rocket','success',535,'HAD 144','Sphere','','','Aeron',1835,'',99 from dual union all
select 11101,'1964-S473','1964-09-18 17:17:00','suborbital rocket','success',35,'NASA 04.115NA','Dayglow','','','Aeronomy',1821,'',168 from dual union all
select 11102,'1964-S474','1964-09-18 17:25:00','suborbital rocket','success',837,'','REDLAMP/ne,particles','','','Test/Aeron/Ionos',293,'',105 from dual union all
select 11103,'1964-S475','1964-09-21 19:16:00','military missile','success',795,'1108','Mk 11 RV','OT','LONG RANGE','OT',1705,'',900 from dual union all
select 11104,'1964-S476','1964-09-21 16:30:03','military missile','success',932,'A3P-38','','','','OT',421,'SSBN 631',1000 from dual union all
select 11105,'1964-S477','1964-09-22 13:10:00','military missile','success',154,'247D','Nike TVX-30/TV-32 x2','NTMP KX-19','BUZZING BEE','Target',1664,'',1800 from dual union all
select 11106,'1964-S478','1964-09-24 03:50:00','military missile','success',795,'906','Mk 11 RV','OT','MOUNT UP','OT',1698,'',900 from dual union all
select 11107,'1964-S479','1964-09-24 16:04:15','military missile','success',796,'449','Mk 11 RV','R&D','','Test',266,'',900 from dual union all
select 11108,'1964-S48','1964-01-23 18:25:00','suborbital rocket','success',839,'Falling Sphere 11','','','','Aeronomy',871,'',172 from dual union all
select 11109,'1964-S480','1964-09-23 21:00:00','military missile','success',1018,'G 15001-13','8K75 UBP-2','','','OT',584,'',1160 from dual union all
select 1111,'1951-S11','1951-06-14 00:00:00','military missile','success',969,'IIIB-10','','','','OT',636,'',100 from dual union all
select 11110,'1964-S481','1964-09-24 04:42:00','suborbital rocket','success',1199,'SL133','LUX-22/RSRS-3','','','Ionos/Solar X',1839,'',180 from dual union all
select 11111,'1964-S482','1964-09-24 05:56:15','military missile','success',1457,'','8K81 LKI-7','','','Test',591,'',686 from dual union all
select 11112,'1964-S483','1964-09-25 05:59:00','military missile','success',982,'G 22501-13T','8K64U UBP-45','Kedr','','OT',572,'',1210 from dual union all
select 11113,'1964-S484','1964-09-25 00:00:00','suborbital rocket','success',1431,'IQSY Tomahawk','','','','Test',1821,'',100 from dual union all
select 11114,'1964-S485','1964-09-25 06:00:00','military missile','success',999,'G 22500-17L','8K67 LKI-14','Kedr','','Test',574,'',1000 from dual union all
select 11115,'1964-S486','1964-09-28 02:15:00','military missile','failure',135,'C001','Mk 11 decoy','ABRES B-2 (USAF)','','Reentry Test',613,'',200 from dual union all
select 11116,'1964-S487','1964-09-26 03:50:00','suborbital rocket','failure',1014,'','R-5  Geophysical','VAO','','Ionos?',624,'',500 from dual union all
select 11117,'1964-S488','1964-09-26 12:04:00','suborbital rocket','success',35,'NASA 04.13GP-GT','Attitude control test','','','Test',1821,'',120 from dual union all
select 11118,'1964-S489','1964-09-26 05:19:00','military missile','success',982,'R 22505-20T','8K64U UBP-46','Climate Test','','OT',570,'',1210 from dual union all
select 11119,'1964-S49','1964-01-24 00:16:00','suborbital rocket','success',839,'NASA 10.61GA','GSFC Grenade 36 (1W)','','','Aeronomy',1821,'',119 from dual union all
select 1112,'1951-S12','1951-06-18 00:00:00','military missile','success',969,'IIIB-1','','','','OT',636,'',100 from dual union all
select 11120,'1964-S490','1964-09-28 16:30:03','military missile','success',932,'A3P-57','','','','OT',421,'SSBN 627',1000 from dual union all
select 11121,'1964-S491','1964-09-29 17:05:00','military missile','success',795,'994','Mk 11 RV','OT','PAINTED WARRIOR','OT',1702,'',900 from dual union all
select 11122,'1964-S493','1964-09-29 04:31:00','suborbital rocket','success',1199,'SL132','LUX-23/RSRS-4','','','Ionos/Solar X',1839,'',174 from dual union all
select 11123,'1964-S494','1964-10-01 00:00:00','military missile','success',7,'6TYa','F5','','','Test',182,'',100 from dual union all
select 11124,'1964-S495','1964-10-01 00:00:00','military missile','success',837,'BMTS','SPEEDBALL II 61 SN28','','','Target?',1895,'',150 from dual union all
select 11125,'1964-S496','1964-09-30 00:00:00','military missile','success',976,'','8K63 KY64-27','','','Test',636,'',402 from dual union all
select 11126,'1964-S497','1964-10-01 00:00:00','military missile','success',206,'009','','','','Reentry Test',313,'',250 from dual union all
select 11127,'1964-S498','1964-10-01 00:00:00','military missile','success',206,'010','','','','Reentry Test',313,'',250 from dual union all
select 11128,'1964-S499','1964-10-02 16:43:57','military missile','success',1420,'B-1   (61-2755)','Mk 6 RV','DASO-4','BLACK WIDOW','OT',1658,'',1300 from dual union all
select 11129,'1964-S50','1964-01-25 00:00:00','military missile','success',1018,'G 15001-03A (1TS)','8K75 LKI 3-24 SLI','','','OT',579,'',1160 from dual union all
select 1113,'1951-S13','1951-06-19 00:00:00','military missile','success',969,'IIIB-2','','','','OT',636,'',100 from dual union all
select 11130,'1964-S500','1964-10-02 02:35:00','suborbital rocket','success',30,'NASA 04.120CG','Lockheed XR Survey','','','XR Astron',1869,'',144 from dual union all
select 11131,'1964-S501','1964-10-02 03:35:14','military missile','success',1457,'','8K81 LKI-8','','','Test',591,'',681 from dual union all
select 11132,'1964-S502','1964-10-05 15:30:00','suborbital rocket','success',288,'C62','CNET B','','','Ionos',664,'',148 from dual union all
select 11133,'1964-S503','1964-10-06 00:00:00','military missile','success',976,'','8K63 KY64-28','','','Test',636,'',402 from dual union all
select 11134,'1964-S504','1964-10-06 20:20:00','suborbital rocket','success',216,'CC-2B-49','','','','Test',428,'',211 from dual union all
select 11135,'1964-S505','1964-10-07 15:42:00','suborbital rocket','success',288,'C63','CNET B','','','Ionos',664,'',118 from dual union all
select 11136,'1964-S506','1964-10-07 18:35:00','suborbital rocket','success',216,'CC-2B-61','','','','Test',428,'',213 from dual union all
select 11137,'1964-S507','1964-10-07 23:04:00','suborbital rocket','success',837,'NASA 14.195CA','GCA Release 37 TMA','','','Aeronomy',1821,'',172 from dual union all
select 11138,'1964-S508','1964-10-08 00:00:00','suborbital rocket','success',1431,'Sandia 152-37','Darthawk 6.5"','','','Test/Aeron?',855,'',92 from dual union all
select 11139,'1964-S509','1964-10-08 03:52:00','suborbital rocket','success',624,'NASA 8.03CA/CI','Ion composition','','','Ionos/Aeron',1821,'',1004 from dual union all
select 1114,'1951-S14','1951-06-20 00:00:00','military missile','success',969,'IIIB-3','','','','OT',636,'',100 from dual union all
select 11140,'1964-S51','1964-01-25 05:14:00','suborbital rocket','success',837,'NASA 14.79UE UNH64-1','ISRO 20.01','TERLS-004','','Magnetospheric',1467,'',164 from dual union all
select 11141,'1964-S510','1964-10-08 10:23:00','suborbital rocket','success',837,'NASA 14.194CA','GCA Release 38 Na','','','Aeronomy',1821,'',160 from dual union all
select 11142,'1964-S511','1964-10-08 10:34:00','suborbital rocket','success',837,'NASA 14.159GE','B Field 5','','','Magnetospheric',1821,'',141 from dual union all
select 11143,'1964-S512','1964-10-09 00:00:00','military missile','success',976,'','8K63 KY64-29','','','Test',636,'',402 from dual union all
select 11144,'1964-S513','1964-10-09 04:04:56','test rocket','success',1146,'S130R','RFD-2','','','Reentry Test',1814,'',132 from dual union all
select 11145,'1964-S514','1964-10-10 04:00:00','military missile','success',999,'G 22500-19L','8K67 LKI-15','','','Test',574,'',865 from dual union all
select 11146,'1964-S515','1964-10-10 07:00:04','military missile','success',983,'KY-2','','','','Test',636,'',254 from dual union all
select 11147,'1964-S519','1964-10-13 16:50:04','military missile','success',932,'A3P-17','','','','OT',421,'SSBN 627',1000 from dual union all
select 11148,'1964-S52','1964-01-26 19:59:03','military missile','success',929,'A2P-191','','','','OT',421,'SSBN 623',1000 from dual union all
select 11149,'1964-S520','1964-10-14 00:00:00','suborbital rocket','success',35,'HITAB','BIRA-2','','','IR Background',1821,'',200 from dual union all
select 1115,'1951-S15','1951-06-22 00:00:00','military missile','success',969,'IIIB-7','Short range test','','','OT',636,'',100 from dual union all
select 11150,'1964-S521','1964-10-14 00:00:00','suborbital rocket','success',1431,'Sandia 152-38','Darthawk 6.5"','','','Test/Aeron?',855,'',91 from dual union all
select 11151,'1964-S522','1964-10-15 09:36:00','suborbital rocket','success',535,'HAD 146','Sphere','','','Aeron',1835,'',102 from dual union all
select 11152,'1964-S523','1964-10-16 00:00:00','military missile','success',977,'','8K63U 23 RD UBP','ShPU','','Test',636,'',402 from dual union all
select 11153,'1964-S524','1964-10-16 12:10:00','military missile','success',983,'KY-2','','','','Test',636,'',254 from dual union all
select 11154,'1964-S525','1964-10-17 10:43:00','suborbital rocket','success',837,'AE5.487','Ionos wind,T (IQSY)','','','Aeron',394,'',156 from dual union all
select 11155,'1964-S526','1964-10-17 10:45:00','suborbital rocket','success',850,'','BLOSSOM wind,T','','','Aeron',394,'',177 from dual union all
select 11156,'1964-S527','1964-10-19 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 617',1000 from dual union all
select 11157,'1964-S528','1964-10-19 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 617',1000 from dual union all
select 11158,'1964-S529','1964-10-19 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 617',1000 from dual union all
select 11159,'1964-S530','1964-10-19 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 617',1000 from dual union all
select 1116,'1951-S16','1951-06-23 00:00:00','military missile','success',969,'IIIB-9','Short range test','','','OT',636,'',100 from dual union all
select 11160,'1964-S531','1964-10-19 15:42:00','suborbital rocket','success',624,'NASA 8.24GI-II','Japan-US Ionos 6','','','Ionos',1821,'',840 from dual union all
select 11161,'1964-S532','1964-10-19 23:00:00','test rocket','success',251,'F-2','','','','-',1845,'',243 from dual union all
select 11162,'1964-S533','1964-10-20 02:29:00','military missile','success',1457,'','8K81 LKI-9','','','Test',591,'',1129 from dual union all
select 11163,'1964-S534','1964-10-21 19:44:00','sounding rocket','success',129,'NASA 16.02GT','','','','Test',1811,'',1950 from dual union all
select 11164,'1964-S535','1964-10-21 21:12:00','suborbital rocket','success',535,'HAD 148','','','','Aeron',219,'',100 from dual union all
select 11165,'1964-S536','1964-10-22 11:03:00','suborbital rocket','success',535,'HAD 140','','','','Aeron',219,'',100 from dual union all
select 11166,'1964-S537','1964-10-23 16:00:00','suborbital rocket','success',624,'NASA 8.33GR','RAE development','','','RF Astron',1821,'',1100 from dual union all
select 11167,'1964-S538','1964-10-24 04:33:00','suborbital rocket','success',837,'','Sandia 154-30','TMA','','Aeron',855,'',160 from dual;
insert into LAUNCH
select 11168,'1964-S539','1964-10-27 00:58:00','suborbital rocket','success',30,'NASA 04.123CG','AS/E/MIT-2','','','XR Astron',1869,'',192 from dual union all
select 11169,'1964-S54','1964-01-27 04:30:00','suborbital rocket','success',837,'NASA 14.80UE UNH64-2','ISRO 20.02','TERLS-005','','Magnetospheric',1467,'',164 from dual union all
select 1117,'1951-S17','1951-06-24 00:00:00','military missile','success',969,'IIIB-8','Short range test','','','OT',636,'',100 from dual union all
select 11170,'1964-S540','1964-10-27 10:50:00','suborbital rocket','success',1199,'SL47','LUX XRS1/UCL UV-3I','','','UV/XR Astronomy',1839,'',146 from dual union all
select 11171,'1964-S541','1964-10-27 16:57:00','suborbital rocket','success',215,'AC17.606','D region','','','Ionos',428,'',149 from dual union all
select 11172,'1964-S542','1964-10-27 16:57:00','suborbital rocket','success',839,'AB6.197','RPA','','','Auroral/Ionos',428,'',136 from dual union all
select 11173,'1964-S543','1964-10-27 07:25:00','suborbital rocket','success',837,'','Sandia 154-31','TMA','','Aeron',855,'',160 from dual union all
select 11174,'1964-S544','1964-10-28 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 616',1000 from dual union all
select 11175,'1964-S545','1964-10-28 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 616',1000 from dual union all
select 11176,'1964-S546','1964-10-28 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 616',1000 from dual union all
select 11177,'1964-S547','1964-10-28 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 616',1000 from dual union all
select 11178,'1964-S548','1964-10-29 16:28:13','military missile','success',796,'450','Mk 11A RV','R&D','','Test',266,'',900 from dual union all
select 11179,'1964-S549','1964-10-29 01:59:00','military missile','success',999,'G 22500-18L','8K67 LKI-16','','','Test',574,'',868 from dual union all
select 1118,'1951-S18','1951-06-25 00:00:00','military missile','success',969,'IIIB-4','','','','OT',636,'',100 from dual union all
select 11180,'1964-S55','1964-01-28 23:01:38','military missile','success',795,'448','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 11181,'1964-S550','1964-10-29 12:15:02','military missile','success',983,'KY-2','','','','Test',636,'',248 from dual union all
select 11182,'1964-S551','1964-10-29 23:18:00','suborbital rocket','success',288,'','ESRO C02/1','','','Aeronomy',313,'',150 from dual union all
select 11183,'1964-S552','1964-10-30 19:30:00','suborbital rocket','success',30,'NASA 04.116GS','EUV spectrum/XR flux','','','Solar X',1869,'',189 from dual union all
select 11184,'1964-S553','1964-11-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual union all
select 11185,'1964-S554','1964-11-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual union all
select 11186,'1964-S555','1964-11-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual union all
select 11187,'1964-S556','1964-11-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual union all
select 11188,'1964-S557','1964-11-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual union all
select 11189,'1964-S558','1964-11-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual union all
select 1119,'1951-S19','1951-06-26 00:00:00','military missile','success',969,'IIIB-5','','','','OT',636,'',100 from dual union all
select 11190,'1964-S559','1964-10-31 18:04:03','military missile','success',932,'A3P-72','','','','OT',421,'SSBN 633',1000 from dual union all
select 11191,'1964-S56','1964-01-28 00:00:00','suborbital rocket','success',866,'Sandia 152-8','Density','','','Test/Aeron',855,'',290 from dual union all
select 11192,'1964-S560','1964-11-01 06:00:00','suborbital rocket','success',837,'NASA 14.197CA','GCA Ionos 13/Rel Na','','','Aeronomy',428,'',182 from dual union all
select 11193,'1964-S561','1964-11-01 05:10:00','suborbital rocket','success',839,'','Sandia 154-32','Balloon','','Aeron',855,'',140 from dual union all
select 11194,'1964-S562','1964-11-01 06:18:00','suborbital rocket','success',837,'','Sandia 154-34','TMA','','Aeron',855,'',160 from dual union all
select 11195,'1964-S563','1964-11-03 00:00:00','military missile','success',795,'853','Mk 11 RV','OT','PARIS ROYAL','OT',1703,'',900 from dual union all
select 11196,'1964-S564','1964-11-03 00:35:00','military missile','success',795,'644','Mk 11 RV','OT [+35m]','NET GAIN','OT',1698,'',900 from dual union all
select 11197,'1964-S565','1964-11-02 09:03:00','suborbital rocket','success',666,'K-8L-7','','','','Aeron',839,'',156 from dual union all
select 11198,'1964-S567','1964-11-03 07:57:00','suborbital rocket','success',30,'NASA 04.52UG','Princeton UVS','','','UV Astron',1869,'',127 from dual union all
select 11199,'1964-S568','1964-11-03 10:02:00','suborbital rocket','success',837,'AE5.550','IQSY Ionos,Sphere','','','Aeron/Ionos',394,'',209 from dual union all
select 112,'1944-S101','1944-09-16 00:00:00','military missile','success',1470,'','Lille  x6','','','Weapon',1730,'',95 from dual union all
select 1120,'1951-S20','1951-06-27 00:00:00','military missile','success',969,'IIIB-14','','','','OT',636,'',100 from dual union all
select 11200,'1964-S569','1964-11-03 04:32:00','suborbital rocket','success',837,'','Sandia 154-36','TMA','','Aeron',855,'',160 from dual union all
select 11201,'1964-S57','1964-01-28 19:00:00','suborbital rocket','success',30,'NASA 04.88GT','Attitude control','','','Test',1869,'',126 from dual union all
select 11202,'1964-S570','1964-11-03 17:38:00','suborbital rocket','success',839,'NASA 10.132GA','GSFC Grenade 60','','','Aeronomy',1821,'',122 from dual union all
select 11203,'1964-S571','1964-11-04 21:20:00','military missile','success',794,'537','Mk 5 RV','OT','BLACK FROST','OT',1701,'',900 from dual union all
select 11204,'1964-S572','1964-11-04 16:58:59','military missile','success',1420,'B-32  (62-013)','Mk 6 RV','DASO-5','HIGH RIDER','OT',1659,'',1300 from dual union all
select 11205,'1964-S573','1964-11-04 08:31:00','suborbital rocket','success',288,'C78','','','','Aeron',310,'',175 from dual union all
select 11206,'1964-S574','1964-11-04 16:35:00','suborbital rocket','success',1490,'AGI53','Solar La','','','Solar UV',668,'',150 from dual union all
select 11207,'1964-S575','1964-11-05 00:00:00','military missile','success',795,'873','Mk 11 RV','OT','LONG SHOT','OT',1705,'',900 from dual union all
select 11208,'1964-S576','1964-11-05 00:00:00','suborbital rocket','success',288,'C69','TMA','','','Aeronomy',664,'',175 from dual union all
select 11209,'1964-S577','1964-11-05 00:00:00','suborbital rocket','success',288,'C47','Na','','','Aeronomy',664,'',185 from dual union all
select 1121,'1951-S21','1951-07-04 00:00:00','military missile','success',992,'II-2','8Zh38 EDI-2-2','','','Test',636,'',150 from dual union all
select 11210,'1964-S578','1964-11-05 00:00:00','suborbital rocket','success',288,'C64','TMA','','','Aeronomy',313,'',150 from dual union all
select 11211,'1964-S579','1964-11-05 00:00:00','suborbital rocket','success',288,'C72','Na','','','Aeronomy',1314,'',150 from dual union all
select 11212,'1964-S58','1964-01-30 00:04:49','military missile','success',795,'808/64-111','Mk 11 Mod 5B RV','DASO-5','ECHO HILL','OT',1698,'',921 from dual union all
select 11213,'1964-S580','1964-11-05 00:00:00','suborbital rocket','success',288,'C67','Na','','','Aeronomy',1314,'',150 from dual union all
select 11214,'1964-S581','1964-11-05 03:01:00','suborbital rocket','success',668,'K-9M-4','','','','Ionos',839,'',356 from dual union all
select 11215,'1964-S582','1964-11-05 17:00:00','suborbital rocket','success',839,'NASA 10.107GA','GSFC Grenade 61','','','Aeronomy',1821,'',122 from dual union all
select 11216,'1964-S583','1964-11-05 19:03:00','suborbital rocket','success',624,'NASA 8.34CA','JHU Airglow 7','','','Aeronomy',1821,'',867 from dual union all
select 11217,'1964-S584','1964-11-05 20:38:00','suborbital rocket','success',837,'NASA 14.104DI','Ionos/B field','','','Ionos/Fields',428,'',145 from dual union all
select 11218,'1964-S585','1964-11-05 20:39:00','suborbital rocket','success',624,'NASA 8.19DI','Ionos/B field','','','Ionos',428,'',840 from dual union all
select 11219,'1964-S586','1964-11-05 23:16:00','suborbital rocket','success',288,'C80','','','','Aeron',310,'',176 from dual union all
select 1122,'1951-S22','1951-07-07 00:00:00','military missile','success',992,'II-3','8Zh38 EDI-2-3','','','Test',636,'',150 from dual union all
select 11220,'1964-S587','1964-11-06 00:00:00','military missile','failure',795,'960','Mk 11 RV','OT','POP FLY','OT',1697,'',0 from dual union all
select 11221,'1964-S588','1964-11-06 00:00:00','military missile','success',795,'848','Mk 11 RV','OT+45m','ORANGE CHUTE','OT',1702,'',900 from dual union all
select 11222,'1964-S589','1964-11-06 00:00:00','suborbital rocket','success',865,'Sandia 152-59','Ballistic HRT','','','Aeron?',855,'',289 from dual union all
select 11223,'1964-S59','1964-01-29 03:09:00','suborbital rocket','success',43,'NASA 06.09GA','Thermosphere probe','','','Aeronomy',1821,'',309 from dual union all
select 11224,'1964-S590','1964-11-06 00:00:00','suborbital rocket','success',288,'C66','Na/TMA','','','Aeronomy',1314,'',150 from dual union all
select 11225,'1964-S591','1964-11-06 00:00:00','suborbital rocket','success',288,'C71','TMA','','','Aeronomy',313,'',103 from dual union all
select 11226,'1964-S592','1964-11-06 00:02:00','suborbital rocket','success',839,'NASA 10.133GA','GSFC Grenade 62','','','Aeronomy',1821,'',120 from dual union all
select 11227,'1964-S593','1964-11-06 00:08:00','suborbital rocket','success',837,'NASA 14.131IA','ISRO 10.04','TERLS-012','','Aeronomy Na',1467,'',192 from dual union all
select 11228,'1964-S594','1964-11-06 05:20:00','suborbital rocket','success',839,'NASA 10.134GA','GSFC Grenade 63','','','Aeronomy',1821,'',120 from dual union all
select 11229,'1964-S595','1964-11-06 08:31:00','suborbital rocket','success',288,'C79','','','','Aeron',310,'',179 from dual union all
select 1123,'1951-S23','1951-07-09 00:00:00','military missile','success',992,'II-4','8Zh38 EDI-2-4','','','Test',636,'',150 from dual union all
select 11230,'1964-S596','1964-11-06 10:00:00','suborbital rocket','success',839,'NASA 10.135GA','GSFC Grenade 64','','','Aeronomy',1821,'',125 from dual union all
select 11231,'1964-S597','1964-11-06 13:45:06','suborbital rocket','success',243,'BK.20','H-206/Dazzle 2','','','Reentry Test/Aeron',1842,'',629 from dual union all
select 11232,'1964-S598','1964-11-07 03:21:00','military missile','success',135,'C004','Mk 11 Mod B Decoys','ABRES B-5 (USAF)','','Reentry Test',612,'',260 from dual union all
select 11233,'1964-S599','1964-11-07 05:13:00','suborbital rocket','success',837,'NASA 14.105DI','Ionos/B field','','','Ionos',428,'',149 from dual union all
select 11234,'1964-S60','1964-01-29 04:11:00','suborbital rocket','success',839,'NASA 10.71GA','GSFC Grenade 38 (2W)','','','Aeronomy',1821,'',254 from dual union all
select 11235,'1964-S600','1964-11-07 05:14:00','suborbital rocket','success',624,'NASA 8.20DI','Ionos/B field','','','Ionos',428,'',714 from dual union all
select 11236,'1964-S601','1964-11-07 10:31:00','suborbital rocket','success',30,'NASA 04.109GG','','','','UV Astron',1869,'',213 from dual union all
select 11237,'1964-S602','1964-11-08 08:22:00','suborbital rocket','failure',1490,'AGI52','UV Spectrum/CU point','','','Solar UV',668,'',98 from dual union all
select 11238,'1964-S603','1964-11-09 00:00:00','military missile','success',794,'661','Mk 5 RV','OT','QUICK JUMP','OT',1700,'',900 from dual union all
select 11239,'1964-S604','1964-11-09 00:00:00','suborbital rocket','success',865,'Sandia 152-67','Ballistic HRT','','','Aeron?',855,'',289 from dual union all
select 1124,'1951-S24','1951-07-10 00:00:00','military missile','success',992,'II-5','8Zh38 EDI-2-5','','','Test',636,'',150 from dual union all
select 11240,'1964-S605','1964-11-09 12:54:00','suborbital rocket','success',837,'NASA 14.204IA','ISRO 10.05','TERLS-013','','Aeronomy Na',1467,'',192 from dual union all
select 11241,'1964-S606','1964-11-10 00:00:00','suborbital rocket','success',837,'','','','','Aeron?',394,'',200 from dual union all
select 11242,'1964-S607','1964-11-10 00:00:00','suborbital rocket','success',837,'','','','','Aeron?',394,'',200 from dual union all
select 11243,'1964-S608','1964-11-10 00:05:00','suborbital rocket','success',837,'NASA 14.205IA','ISRO 10.06','TERLS-014','','Aeronomy Na',1467,'',192 from dual union all
select 11244,'1964-S609','1964-11-10 02:05:00','suborbital rocket','success',666,'K-8L-8','','','','Ionos',839,'',145 from dual union all
select 11245,'1964-S61','1964-01-29 04:18:00','suborbital rocket','success',839,'NASA 10.81GA','GSFC Grenade 40 (2A)','','','Aeronomy',176,'',124 from dual union all
select 11246,'1964-S610','1964-11-10 11:07:00','suborbital rocket','success',837,'NASA 14.147UI','UIUC-GCA Ionos 5/3a','','','Ionos',1821,'',251 from dual union all
select 11247,'1964-S611','1964-11-10 22:25:00','suborbital rocket','success',837,'NASA 14.114CA','GCA Release 39 Na','','','Aeronomy',149,'CVE-25',198 from dual union all
select 11248,'1964-S612','1964-11-10 22:28:00','suborbital rocket','success',837,'NASA 14.53CA','GCA Release 40 Na','','','Aeronomy',1821,'',198 from dual union all
select 11249,'1964-S613','1964-11-11 01:19:00','suborbital rocket','success',837,'AE5.552 IQSY','IQSY Ionos,Sphere','','','Aeron',394,'',81 from dual union all
select 1125,'1951-S25','1951-07-11 00:00:00','military missile','success',992,'II-6','8Zh38 EDI-2-6','','','Test',636,'',150 from dual union all
select 11250,'1964-S614','1964-11-11 09:59:00','suborbital rocket','success',535,'HAD 145','Sphere','','','Aeron',1835,'',100 from dual union all
select 11251,'1964-S615','1964-11-11 22:24:00','suborbital rocket','success',837,'NASA 14.115CA','GCA Release 41 Na','','','Aeronomy',150,'CVE-25',156 from dual union all
select 11252,'1964-S616','1964-11-11 22:27:00','suborbital rocket','success',837,'NASA 14.112CA','GCA Release 42 Na','','','Aeronomy',1821,'',202 from dual union all
select 11253,'1964-S617','1964-11-12 03:05:00','suborbital rocket','success',666,'K-8L-6','','','','Aeron',839,'',100 from dual union all
select 11254,'1964-S618','1964-11-12 10:00:00','suborbital rocket','success',535,'HAD 157','Sphere','','','Aeron',1835,'',103 from dual union all
select 11255,'1964-S619','1964-11-12 10:53:00','suborbital rocket','success',837,'NASA 14.116CA','GCA Release 43 Na','','','Aeronomy',151,'CVE-25',204 from dual union all
select 11256,'1964-S62','1964-01-29 10:00:00','suborbital rocket','success',837,'NASA 14.81UE UNH64-3','ISRO 20.03','TERLS-006','','Magnetospheric',1467,'',167 from dual union all
select 11257,'1964-S620','1964-11-12 10:56:00','suborbital rocket','success',837,'NASA 14.113CA','GCA Release 44 Na','','','Aeronomy',1821,'',204 from dual;
insert into LAUNCH
select 11258,'1964-S621','1964-11-12 19:30:00','suborbital rocket','success',30,'NE3.155','NRL UV Echelle','','','Solar UV',1869,'',189 from dual union all
select 11259,'1964-S622','1964-11-13 00:00:00','suborbital rocket','success',865,'Sandia 152-65','Ballistic','','','Aeron?',855,'',308 from dual union all
select 1126,'1951-S26','1951-07-12 00:00:00','military missile','failure',992,'II-7','8Zh38 EDI-2-7','','','Test',636,'',0 from dual union all
select 11260,'1964-S623','1964-11-13 16:57:03','military missile','success',932,'A3P-87','','','','OT',421,'SSBN 633',1000 from dual union all
select 11261,'1964-S624','1964-11-14 10:23:00','suborbital rocket','success',30,'NASA 04.110GG','','','','UV Astron',1869,'',207 from dual union all
select 11262,'1964-S625','1964-11-16 06:57:00','military missile','success',1373,'236','AFP-437 CTL','','','Interceptor',772,'',1148 from dual union all
select 11263,'1964-S626','1964-11-16 18:18:00','suborbital rocket','success',35,'NASA 04.45GA','Thermosphere probe','','','Aeronomy',1821,'',188 from dual union all
select 11264,'1964-S627','1964-11-16 18:31:00','suborbital rocket','success',30,'NASA 04.118NA','Luster 1964','','','Meteorites',1869,'',157 from dual union all
select 11265,'1964-S628','1964-11-17 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 11266,'1964-S629','1964-11-17 17:49:00','suborbital rocket','success',837,'NASA 14.233UA','Density','','','Aeronomy',152,'CVE-25',194 from dual union all
select 11267,'1964-S63','1964-01-29 04:17:00','suborbital rocket','success',839,'NASA 10.89GA','GSFC Grenade 39 (2C)','','','Aeronomy',428,'',122 from dual union all
select 11268,'1964-S630','1964-11-17 21:10:00','suborbital rocket','success',839,'NASA 10.153GA','Density','','','Aeronomy',153,'CVE-25',160 from dual union all
select 11269,'1964-S631','1964-11-18 00:00:00','suborbital rocket','success',865,'Sandia 152-66','Ballistic','','','Aeron?',855,'',309 from dual union all
select 1127,'1951-S27','1951-07-14 00:00:00','military missile','success',992,'II-8','8Zh38 EDI-2-8','','','Test',636,'',150 from dual union all
select 11270,'1964-S632','1964-11-18 14:55:00','military missile','success',916,'AO-16','P-82','','','OT',435,'',180 from dual union all
select 11271,'1964-S633','1964-11-18 22:30:00','military missile','success',916,'AO-17','P-83','','','OT',435,'',180 from dual union all
select 11272,'1964-S634','1964-11-18 11:23:00','suborbital rocket','failure',837,'','LILA','','','Aeron',394,'',143 from dual union all
select 11273,'1964-S635','1964-11-19 15:00:00','military missile','success',916,'AO-18','P-84','','','OT',435,'',180 from dual union all
select 11274,'1964-S636','1964-11-19 22:30:00','military missile','failure',916,'AO-19','P-85','(Colo. overshoot)','','OT',435,'',180 from dual union all
select 11275,'1964-S637','1964-11-19 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 11276,'1964-S638','1964-11-19 05:34:00','suborbital rocket','success',837,'AE5.490','TMA release','','','Aeron',394,'',142 from dual union all
select 11277,'1964-S639','1964-11-19 17:15:00','suborbital rocket','success',30,'NE3.154R','','','','Solar UV',1869,'',219 from dual union all
select 11278,'1964-S64','1964-02-28 18:50:00','military missile','success',916,'AO-11','P-77','','','OT',437,'',180 from dual union all
select 11279,'1964-S640','1964-11-19 19:02:00','suborbital rocket','success',839,'NASA 10.117GA','GSFC Grenade 65','','','Aeronomy',1821,'',125 from dual union all
select 1128,'1951-S28','1951-07-16 00:00:00','military missile','success',992,'II-9','8Zh38 EDI-2-9','','','Test',636,'',150 from dual union all
select 11280,'1964-S641','1964-11-19 20:20:00','suborbital rocket','success',837,'NASA 14.149UI','UIUC-GCA Ionos 6/3b','','','Ionos',1821,'',167 from dual union all
select 11281,'1964-S642','1964-11-19 22:02:00','suborbital rocket','success',837,'NASA 14.148UI','UIUC IQSY Ionos','','','Ionos',155,'CVE-25',170 from dual union all
select 11282,'1964-S643','1964-11-19 23:43:00','suborbital rocket','success',837,'AE5.491 IQSY','IQSY release','','','Aeron',394,'',200 from dual union all
select 11283,'1964-S644','1964-11-21 00:00:00','suborbital rocket','success',865,'Sandia 115-5','Sampler','','','Aeron',855,'',270 from dual union all
select 11284,'1964-S645','1964-11-22 16:30:00','suborbital rocket','success',839,'AD6.811','EM absorption','','','Ionos',394,'',110 from dual union all
select 11285,'1964-S646','1964-11-23 17:07:00','suborbital rocket','success',837,'NASA 14.117GI','','','','Ionos',1821,'',178 from dual union all
select 11286,'1964-S647','1964-11-24 00:00:00','suborbital rocket','success',865,'Sandia 115-6','Sampler','','','Aeron',855,'',300 from dual union all
select 11287,'1964-S648','1964-11-25 11:34:00','suborbital rocket','success',30,'NB3.161','','','','XR Astron',1869,'',200 from dual union all
select 11288,'1964-S649','1964-11-27 08:02:00','suborbital rocket','success',215,'AA-2A-37','','','','Auroral/Ionos/Micromet',428,'',170 from dual union all
select 11289,'1964-S65','1964-02-25 18:00:00','military missile','failure',916,'','P-76','','','OT',437,'',10 from dual union all
select 1129,'1951-S29','1951-07-20 00:00:00','military missile','success',992,'II-10','8Zh38 EDI-2-10','','','Test',636,'',150 from dual union all
select 11290,'1964-S650','1964-11-27 13:31:00','suborbital rocket','success',848,'AE07.312','Vehicle Test','','','Test',394,'',201 from dual union all
select 11291,'1964-S651','1964-11-27 17:38:00','suborbital rocket','success',288,'C75/EXP-9','MPE Ba-9','','','Aeronomy',664,'',166 from dual union all
select 11292,'1964-S652','1964-11-28 11:14:26','military missile','success',983,'KY-2','','','','Test',636,'',250 from dual union all
select 11293,'1964-S653','1964-11-30 00:00:00','suborbital rocket','success',549,'Sandia 152-13','LRL Payload Test','','','Test',855,'',168 from dual union all
select 11294,'1964-S654','1964-11-30 00:00:00','suborbital rocket','success',865,'Sandia 152-71','HHRT','','','Test',855,'',293 from dual union all
select 11295,'1964-S655','1964-12-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 11296,'1964-S656','1964-12-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual union all
select 11297,'1964-S657','1964-12-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeronomy',1895,'',100 from dual union all
select 11298,'1964-S659','1964-11-30 13:11:00','suborbital rocket','success',837,'NASA 14.135IA','Rehbar 5  (Na)','','','Aeronomy',1419,'',207 from dual union all
select 11299,'1964-S66','1964-01-31 00:00:00','military missile','success',976,'','8K63 KY64-4','','','Test',636,'',402 from dual union all
select 113,'1944-S102','1944-09-16 00:00:00','military missile','success',1470,'','Mons   x3','','','Weapon',1730,'',95 from dual union all
select 1130,'1951-S30','1951-07-23 00:00:00','military missile','success',992,'II-11','8Zh38 EDI-2-11','','','Test',636,'',150 from dual union all
select 11300,'1964-S660','1964-11-30 17:37:00','suborbital rocket','success',288,'C76/EXP-10','MPE Ba-10','','','Aeronomy',664,'',207 from dual union all
select 11301,'1964-S661','1964-12-01 00:00:00','suborbital rocket','failure',865,'Sandia 152-55','ACS','','','Test?',855,'',270 from dual union all
select 11302,'1964-S662','1964-12-01 01:20:00','suborbital rocket','success',837,'NASA 14.136IA','Rehbar 6  (Na)','','','Aeronomy',1419,'',200 from dual union all
select 11303,'1964-S663','1964-12-01 04:20:00','suborbital rocket','success',839,'ION 64-1','','','','Ionos',310,'',134 from dual union all
select 11304,'1964-S664','1964-12-01 06:15:00','suborbital rocket','success',30,'NASA 04.83GA','UV Airglow','','','Aeronomy',1869,'',182 from dual union all
select 11305,'1964-S665','1964-12-01 08:45:04','military missile','success',154,'210D','ABRES LORV-5/GTV4','','BROOK TROUT','Reentry Test',1662,'',1800 from dual union all
select 11306,'1964-S666','1964-12-02 16:30:04','military missile','success',932,'A3P-100','','','','OT',421,'SSBN 634',1000 from dual union all
select 11307,'1964-S667','1964-12-03 17:40:00','suborbital rocket','success',446,'D-16','FU-103B MPE Ba-11','','','Aeronomy',664,'',390 from dual union all
select 11308,'1964-S668','1964-12-04 11:09:17','military missile','success',154,'300D','Nike RMV-303','NTMP KX-11','OPERA GLASS','Target',1664,'',1800 from dual union all
select 11309,'1964-S669','1964-12-04 18:06:00','suborbital rocket','success',839,'ION 64-2','','','','Ionos',310,'',123 from dual union all
select 1131,'1951-S31','1951-07-25 00:00:00','military missile','success',992,'II-12','8Zh38 EDI-2-12','','','Test',636,'',150 from dual union all
select 11310,'1964-S67','1964-01-31 13:30:00','suborbital rocket','success',837,'NASA 14.82UE UNH64-4','ISRO 20.04','TERLS-007','','Magnetospheric',1467,'',167 from dual union all
select 11311,'1964-S670','1964-12-05 04:30:00','military missile','success',135,'C002','Mk 11 Mod A Decoy','ABRES B-3 (USAF)','','Reentry Test',613,'',200 from dual union all
select 11312,'1964-S671','1964-12-07 17:56:00','suborbital rocket','success',837,'NASA 14.60UE','UNH Particles','','','Magnetospheric',1821,'',145 from dual union all
select 11313,'1964-S672','1964-12-08 00:00:00','military missile','success',1419,'SM-85','Mk 4 RV','ST WEST WIND I','WEST WIND I','OT',1654,'',1000 from dual union all
select 11314,'1964-S673','1964-12-08 10:26:00','suborbital rocket','success',535,'HAD 159','Sphere','','','Aeron',1835,'',100 from dual union all
select 11315,'1964-S674','1964-12-08 22:55:00','suborbital rocket','success',30,'AD3.515','EUV Mon','','','Solar EUV',1869,'',200 from dual union all
select 11316,'1964-S675','1964-12-09 08:00:00','military missile','failure',795,'1101','Mk 11 RV','OT','NICKED BLADE','OT',1703,'',0 from dual union all
select 11317,'1964-S676','1964-12-09 20:00:00','military missile','success',795,'1163','Mk 11 RV','OT','TOP RAIL','OT',1697,'',900 from dual union all
select 11318,'1964-S677','1964-12-10 10:26:00','suborbital rocket','success',535,'HAD 161','Sphere','','','Aeron',1835,'',100 from dual union all
select 11319,'1964-S678','1964-12-11 00:00:00','suborbital rocket','success',30,'HITAB','TSRA-3','','','IR Background',1786,'',200 from dual union all
select 1132,'1951-S32','1951-07-02 00:00:00','military missile','success',992,'II-1','8Zh38 EDI-2-1','','','Test',636,'',150 from dual union all
select 11320,'1964-S679','1964-12-15 17:59:59','military missile','success',796,'451','Mk 11 RV','R&D','','Test',266,'',900 from dual union all
select 11321,'1964-S68','1964-02-01 00:00:00','military missile','success',976,'','8K63 KY64-5','','','Test',636,'',402 from dual union all
select 11322,'1964-S680','1964-12-15 00:00:00','military missile','success',976,'','8K63 KY64-31','','','Test',636,'',402 from dual union all
select 11323,'1964-S681','1964-12-15 00:00:00','military missile','success',999,'G 22500-21L','8K67 LKI-17','','','Test',574,'',870 from dual union all
select 11324,'1964-S682','1964-12-16 16:34:03','military missile','success',932,'A3P-45','','','','OT',421,'SSBN 634',1000 from dual union all
select 11325,'1964-S683','1964-12-17 00:00:00','suborbital rocket','success',1250,'OP 417104','Denpro II','','','Meteo',1415,'',100 from dual union all
select 11326,'1964-S684','1964-12-16 02:02:00','suborbital rocket','success',1206,'SL88A','RAE Test 28','','','Test',1839,'',220 from dual union all
select 11327,'1964-S685','1964-12-16 14:30:00','suborbital rocket','success',30,'NASA 04.132GA-GI','Meteoroids','','','Meteorites',1869,'',207 from dual union all
select 11328,'1964-S686','1964-12-16 14:57:00','suborbital rocket','success',837,'NASA 14.209GI','','','','Ionos',1895,'',195 from dual union all
select 11329,'1964-S687','1964-12-17 00:00:00','suborbital rocket','success',1250,'OP 417158B','Denpro III','','','Meteo',1415,'',100 from dual union all
select 1133,'1951-S33','1951-07-22 00:00:00','suborbital rocket','success',991,'1','1-RB','','','Test/Bio',636,'',100 from dual union all
select 11330,'1964-S688','1964-12-17 03:05:00','suborbital rocket','success',1199,'SL302','LUX-25AC2/Cul-2','','','Solar X,UV',1839,'',167 from dual union all
select 11331,'1964-S689','1964-12-17 06:55:00','suborbital rocket','success',30,'NASA 04.125UA','JHU Airglow 8','','','Aeronomy',1869,'',235 from dual union all
select 11332,'1964-S69','1964-02-02 00:00:00','military missile','success',979,'','8K65 UBP-50','','','Test',636,'',675 from dual union all
select 11333,'1964-S690','1964-12-17 19:54:00','suborbital rocket','unknown',839,'','','','','Aeron',1057,'',100 from dual union all
select 11334,'1964-S692','1964-12-19 01:06:12','military missile','success',796,'452','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 11335,'1964-S693','1964-12-18 00:00:00','military missile','success',795,'1057','Mk 11 RV','OT','ROSY FUTURE','OT',1702,'',900 from dual union all
select 11336,'1964-S694','1964-12-18 00:00:00','military missile','success',980,'','8K65U UBP-3 68 RB','','','Test',636,'',675 from dual union all
select 11337,'1964-S695','1964-12-18 04:59:00','military missile','success',982,'R 22604-20T','8K64U UBP-47','','','OT',565,'',1210 from dual union all
select 11338,'1964-S696','1964-12-22 04:00:00','ballistic missile test','failure',248,'21-3','Ion Engine Test C','Program 661A','QUAKER TOWN','Tech',1015,'',500 from dual union all
select 11339,'1964-S697','1964-12-21 09:44:59','military missile','success',983,'KY-2','','','','Test',636,'',250 from dual union all
select 1134,'1951-S34','1951-07-27 00:00:00','military missile','success',992,'II-13','8Zh38 EDI-2-13','','','Test',636,'',150 from dual union all
select 11340,'1964-S698','1964-12-22 16:30:04','military missile','success',932,'A3P-46','','','','OT',421,'SSBN 632',1000 from dual union all
select 11341,'1964-S699','1964-12-22 00:00:00','military missile','success',976,'','8K63 KY64-32','','','Test',636,'',402 from dual union all
select 11342,'1964-S70','1964-02-02 17:00:00','military missile','success',982,'G 22503-09','8K64U UBP-38','','','OT',571,'',1210 from dual union all
select 11343,'1964-S700','1964-12-22 19:15:20','military missile','success',162,'111F','Mk 4-5B','ST','STEP OVER','Test',1670,'',1400 from dual union all
select 11344,'1964-S701','1964-12-23 00:00:00','military missile','success',976,'','8K63 KY64-33','','','Test',636,'',402 from dual union all
select 11345,'1964-S702','1964-12-26 00:00:00','military missile','success',976,'','8K63 KY64-34','','','Test',636,'',402 from dual union all
select 11346,'1964-S703','1964-12-27 00:00:00','military missile','success',976,'','8K63 KY64-35 UBP 40 RD','','','Test',636,'',402 from dual union all
select 11347,'1964-S704','1964-12-26 05:00:00','military missile','success',982,'R 22602-20T','8K64U UBP-48','','','OT',565,'',1210 from dual;
insert into LAUNCH
select 11348,'1964-S705','1964-06-24 20:34:00','suborbital rocket','failure',219,'NRC FF-4-01','','','','Test',428,'',467 from dual union all
select 11349,'1964-S706','1964-07-02 21:00:00','suborbital rocket','failure',219,'NRC FF-4-02','','','','Test',428,'',400 from dual union all
select 1135,'1951-S35','1951-07-29 00:00:00','suborbital rocket','success',986,'1','','','','Test/Bio',636,'',100 from dual union all
select 11350,'1964-S707','1964-03-16 00:00:00','military missile','success',929,'A2P','PY-1 DASO','','','OT',421,'SSBN 625',1000 from dual union all
select 11351,'1964-S708','1964-03-20 00:00:00','military missile','success',929,'A2P','PY-1 DASO','','','OT',421,'SSBN 625',1000 from dual union all
select 11352,'1964-S709','1964-01-01 00:00:00','suborbital rocket','success',767,'','2A IV-2','','','Test',185,'',80 from dual union all
select 11353,'1964-S71','1964-02-02 00:00:00','military missile','success',1018,'','8K75 LKI 3-25 UBP','','','OT',584,'',1160 from dual union all
select 11354,'1964-S710','1964-01-01 00:00:00','suborbital rocket','success',767,'','2A IV-3','','','Test',185,'',80 from dual union all
select 11355,'1964-S711','1964-01-01 00:00:00','suborbital rocket','success',767,'','2A IV-4','','','Test',185,'',80 from dual union all
select 11356,'1964-S712','1964-01-01 00:00:00','suborbital rocket','success',767,'','2A IV-5','','','Test',185,'',80 from dual union all
select 11357,'1964-S713','1964-01-01 00:00:00','suborbital rocket','success',767,'','2A IV-6','','','Test',185,'',80 from dual union all
select 11358,'1964-S714','1964-01-01 00:00:00','suborbital rocket','success',767,'','2A IV-7','','','Test',185,'',80 from dual union all
select 11359,'1964-S715','1964-01-01 00:00:00','suborbital rocket','success',767,'','2A IV-8','','','Test',185,'',80 from dual union all
select 1136,'1951-S36','1951-08-07 16:36:00','suborbital rocket','success',50,'USAF 16','Ionos Propagation','','','Ionos',640,'',83 from dual union all
select 11360,'1964-S716','1964-01-01 00:00:00','suborbital rocket','success',767,'','2A IV-9','','','Test',185,'',80 from dual union all
select 11361,'1964-S717','1964-01-01 00:00:00','suborbital rocket','success',769,'','3A IV-1','','','Test',185,'',80 from dual union all
select 11362,'1964-S718','1964-01-01 00:00:00','suborbital rocket','success',769,'','3A IV-2','','','Test',185,'',80 from dual union all
select 11363,'1964-S719','1964-01-01 00:00:00','suborbital rocket','success',769,'','3A IV-3','','','Test',185,'',80 from dual union all
select 11364,'1964-S72','1964-02-03 16:41:07','military missile','success',929,'A2P-255','PY-1 DASO','','','OT',421,'SSBN 622',1000 from dual union all
select 11365,'1964-S720','1964-01-01 00:00:00','suborbital rocket','success',769,'','3A IV-4','','','Test',185,'',80 from dual union all
select 11366,'1964-S721','1964-01-01 00:00:00','suborbital rocket','success',769,'','3A IV-5','','','Test',185,'',80 from dual union all
select 11367,'1964-S722','1964-01-01 00:00:00','suborbital rocket','success',769,'','3A IV-6','','','Test',185,'',80 from dual union all
select 11368,'1964-S723','1964-01-15 00:00:00','suborbital rocket','success',769,'','3A IV-7','','','Test',185,'',80 from dual union all
select 11369,'1964-S724','1964-06-23 00:00:00','suborbital rocket','success',767,'','2A V-1','','','Test',185,'',80 from dual union all
select 1137,'1951-S37','1951-08-07 18:00:00','suborbital rocket','success',1499,'7','Viking 7','','','Iono-solar',1852,'',219 from dual union all
select 11370,'1964-S725','1964-06-25 00:00:00','suborbital rocket','success',767,'','2A V-2','','','Test',185,'',80 from dual union all
select 11371,'1964-S726','1964-06-25 00:00:00','suborbital rocket','success',767,'','2A V-3','','','Test',185,'',80 from dual union all
select 11372,'1964-S727','1964-06-26 00:00:00','suborbital rocket','success',767,'','2A V-4','','','Test',185,'',80 from dual union all
select 11373,'1964-S728','1964-07-06 00:00:00','suborbital rocket','success',767,'','2A VI-1','','','Test',185,'',80 from dual union all
select 11374,'1964-S729','1964-07-07 23:02:00','suborbital rocket','success',767,'42 TMA-1','2A VI-2   IRIS','','','Aeron TMA',185,'',109 from dual union all
select 11375,'1964-S73','1964-02-04 01:46:00','suborbital rocket','success',839,'NASA 10.62GA','GSFC Grenade 41 (3W)','','','Aeronomy',1821,'',116 from dual union all
select 11376,'1964-S730','1964-07-08 01:10:00','suborbital rocket','success',767,'43 TMA-2','2A VI-3   JANET','','','Aeron TMA',185,'',95 from dual union all
select 11377,'1964-S731','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-4','','','Test',185,'',80 from dual union all
select 11378,'1964-S732','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-5','','','Test',185,'',80 from dual union all
select 11379,'1964-S733','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-6','','','Test',185,'',80 from dual union all
select 1138,'1951-S38','1951-08-15 00:00:00','suborbital rocket','success',986,'2','','','','Test/Solar UV/Bio',636,'',100 from dual union all
select 11380,'1964-S734','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-7','','','Test',185,'',80 from dual union all
select 11381,'1964-S735','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-8','','','Test',185,'',80 from dual union all
select 11382,'1964-S736','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-9','','','Test',185,'',80 from dual union all
select 11383,'1964-S737','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-10','','','Test',185,'',80 from dual union all
select 11384,'1964-S738','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-11','','','Test',185,'',80 from dual union all
select 11385,'1964-S739','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-12','','','Test',185,'',80 from dual union all
select 11386,'1964-S74','1964-02-04 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 11387,'1964-S740','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-13','','','Test',185,'',80 from dual union all
select 11388,'1964-S741','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-14','','','Test',185,'',80 from dual union all
select 11389,'1964-S742','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-15','','','Test',185,'',80 from dual union all
select 1139,'1951-S39','1951-08-19 00:00:00','suborbital rocket','success',991,'2','','','','Test/Bio',636,'',100 from dual union all
select 11390,'1964-S743','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-16','','','Test',185,'',80 from dual union all
select 11391,'1964-S744','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-17','','','Test',185,'',80 from dual union all
select 11392,'1964-S745','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-18','','','Test',185,'',80 from dual union all
select 11393,'1964-S746','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-19','','','Test',185,'',80 from dual union all
select 11394,'1964-S747','1964-07-01 00:00:00','suborbital rocket','success',767,'','2A VI-20','','','Test',185,'',80 from dual union all
select 11395,'1964-S748','1964-07-01 00:00:00','suborbital rocket','success',770,'','3B VI-5','','','Test',185,'',80 from dual union all
select 11396,'1964-S749','1964-07-22 23:00:00','suborbital rocket','success',767,'62 TMA-3','2A VI-21  SHARON','','','Aeron TMA',185,'',98 from dual union all
select 11397,'1964-S75','1964-02-04 01:05:00','suborbital rocket','success',1197,'SL88','RAE Test 22','','','Aeron',1839,'',118 from dual union all
select 11398,'1964-S750','1964-07-23 00:00:00','suborbital rocket','success',767,'','2A VI-22','','','Test',185,'',80 from dual union all
select 11399,'1964-S751','1964-07-23 00:00:00','suborbital rocket','success',770,'','3B VI-1','','','Test',185,'',80 from dual union all
select 114,'1944-S103','1944-09-16 00:00:00','military missile','success',1470,'','London','','','Weapon',1745,'',95 from dual union all
select 1140,'1951-S40','1951-08-22 19:00:00','suborbital rocket','success',1470,'TF1','','','','Test',1863,'',213 from dual union all
select 11400,'1964-S752','1964-07-23 00:00:00','suborbital rocket','success',770,'','3B VI-2','','','Test',185,'',80 from dual union all
select 11401,'1964-S753','1964-07-23 00:00:00','suborbital rocket','success',770,'','3B VI-3','','','Test',185,'',80 from dual union all
select 11402,'1964-S754','1964-07-24 23:45:00','suborbital rocket','success',767,'67 TMA-4','2A VI-23  QUEENIE','','','Aeron TMA',185,'',96 from dual union all
select 11403,'1964-S755','1964-07-01 00:00:00','suborbital rocket','success',770,'','3B VI-5','','','Test',185,'',80 from dual union all
select 11404,'1964-S756','1964-07-01 00:00:00','suborbital rocket','success',770,'','3B VI-6','','','Test',185,'',80 from dual union all
select 11405,'1964-S757','1964-07-01 00:00:00','suborbital rocket','success',770,'','3B VI-7','','','Test',185,'',80 from dual union all
select 11406,'1964-S758','1964-07-01 00:00:00','suborbital rocket','success',770,'','3B VI-8','','','Test',185,'',80 from dual union all
select 11407,'1964-S759','1964-07-01 00:00:00','suborbital rocket','success',770,'','3B VI-9','','','Test',185,'',80 from dual union all
select 11408,'1964-S76','1964-02-04 01:35:00','suborbital rocket','success',837,'NASA 14.22UA','UM Pitot 4','','','Aeronomy',176,'',158 from dual union all
select 11409,'1964-S760','1964-07-25 00:00:00','suborbital rocket','success',770,'','3B VI-10','','','Test',185,'',80 from dual union all
select 1141,'1951-S41','1951-08-28 00:00:00','suborbital rocket','success',986,'3','','','','Test/Bio',636,'',100 from dual union all
select 11410,'1964-S761','1964-09-08 00:00:00','suborbital rocket','success',770,'','3B VII-1','','','Test',185,'',80 from dual union all
select 11411,'1964-S762','1964-09-08 00:00:00','suborbital rocket','success',770,'','3B VII-2','','','Test',185,'',80 from dual union all
select 11412,'1964-S763','1964-09-09 00:00:00','suborbital rocket','success',770,'','3B VII-3','','','Test',185,'',80 from dual union all
select 11413,'1964-S764','1964-09-09 00:00:00','suborbital rocket','success',770,'77','3B VII-4','','','Test',185,'',80 from dual union all
select 11414,'1964-S765','1964-12-08 00:00:00','suborbital rocket','success',767,'84','2A IX-1','','','Test',185,'',80 from dual union all
select 11415,'1964-S766','1964-12-01 00:00:00','suborbital rocket','success',767,'','2A IX-2','','','Test',185,'',80 from dual union all
select 11416,'1964-S767','1964-12-01 00:00:00','suborbital rocket','success',767,'','2A IX-3','','','Test',185,'',80 from dual union all
select 11417,'1964-S768','1964-12-01 00:00:00','suborbital rocket','success',767,'','2A IX-4','','','Test',185,'',80 from dual union all
select 11418,'1964-S769','1964-12-01 00:00:00','suborbital rocket','success',767,'','2C IX-1','','','Test',185,'',80 from dual union all
select 11419,'1964-S77','1964-02-05 00:39:00','suborbital rocket','success',839,'NASA 10.87GA','GSFC Grenade 42 (4C)','','','Aeronomy',428,'',126 from dual union all
select 1142,'1951-S42','1951-09-03 00:00:00','suborbital rocket','success',986,'4','','','','Test/Bio',636,'',100 from dual union all
select 11420,'1964-S770','1964-12-01 00:00:00','suborbital rocket','success',767,'','2C IX-2','','','Test',185,'',80 from dual union all
select 11421,'1964-S771','1964-12-01 00:00:00','suborbital rocket','success',767,'','2C IX-3','','','Test',185,'',80 from dual union all
select 11422,'1964-S772','1964-12-01 00:00:00','suborbital rocket','success',767,'','2C IX-4','','','Test',185,'',80 from dual union all
select 11423,'1964-S773','1964-12-01 00:00:00','suborbital rocket','success',770,'','3B IX-1','','','Test',185,'',80 from dual union all
select 11424,'1964-S774','1964-12-01 00:00:00','suborbital rocket','success',770,'','3B IX-2','','','Test',185,'',80 from dual union all
select 11425,'1964-S775','1964-12-14 00:00:00','suborbital rocket','success',770,'','3B IX-3','','','Test',185,'',80 from dual union all
select 11426,'1964-S776','1964-10-07 23:19:00','suborbital rocket','success',837,'NASA 14.158GE','B Field 4','','','Magnetospheric',1821,'',140 from dual union all
select 11427,'1964-S777','1964-12-01 00:00:00','suborbital rocket','success',1181,'','Test 1 (of 10)','','','Test',1057,'',100 from dual union all
select 11428,'1964-S778','1964-01-23 00:00:00','suborbital rocket','success',1250,'','Denpro 03','','','Meteo',1413,'',150 from dual union all
select 11429,'1964-S779','1964-01-23 00:00:00','suborbital rocket','success',1250,'','Denpro 04','','','Meteo',1413,'',150 from dual union all
select 1143,'1951-S43','1951-10-17 18:17:00','suborbital rocket','success',51,'USAF 20','Ionos Propagation','','','Ionos',640,'',114 from dual union all
select 11430,'1964-S78','1964-02-05 03:20:00','suborbital rocket','success',839,'NASA 10.63GA','GSFC Grenade 43 (4W)','','','Aeronomy',1821,'',119 from dual union all
select 11431,'1964-S780','1964-01-23 00:00:00','suborbital rocket','success',1250,'','Denpro 05','','','Meteo',1413,'',150 from dual union all
select 11432,'1964-S781','1964-01-23 00:00:00','suborbital rocket','success',1250,'','Denpro 06','','','Meteo',1413,'',150 from dual union all
select 11433,'1964-S782','1964-05-01 00:00:00','suborbital rocket','success',1250,'OP 411176','Denpro 07','','','Meteo',1413,'',100 from dual union all
select 11434,'1964-S783','1964-05-08 00:00:00','suborbital rocket','success',1250,'OP 411391','Denpro 08','','','Meteo',1413,'',100 from dual union all
select 11435,'1964-S784','1964-12-16 00:00:00','suborbital rocket','success',1250,'','Denpro 09 Sphere','','','Meteo',1415,'',100 from dual union all
select 11436,'1964-S785','1964-12-16 00:00:00','suborbital rocket','success',1250,'','Denpro 10 Sphere','','','Meteo',1415,'',100 from dual union all
select 11437,'1964-S786','1964-12-17 00:00:00','suborbital rocket','success',1250,'OP 417158A','Denpro','','','Meteo',1415,'',100 from dual;
insert into LAUNCH
select 11438,'1964-S787','1964-04-22 00:00:00','military missile','success',837,'','SPEEDBALL 048 SN16','','','Target',1895,'',150 from dual union all
select 11439,'1964-S788','1964-01-01 00:00:00','suborbital rocket','unknown',720,'LT15','','','','Aeron',1837,'',80 from dual union all
select 1144,'1951-S44','1951-10-29 21:04:00','suborbital rocket','success',1470,'60','Probe','','','Aeron',1863,'',137 from dual union all
select 11440,'1964-S789','1964-05-14 00:00:00','military missile','success',837,'','SPEEDBALL 050 SN17','','','Target',1895,'',150 from dual union all
select 11441,'1964-S79','1964-02-06 13:01:00','military missile','success',982,'R 22501-20T','8K64U UBP-39 K','','','OT',566,'',1210 from dual union all
select 11442,'1964-S790','1964-01-01 00:00:00','suborbital rocket','success',286,'Cedar 4','','','','Test',356,'',40 from dual union all
select 11443,'1964-S791','1964-01-01 00:00:00','suborbital rocket','success',286,'Cedar 5','','','','Test',356,'',40 from dual union all
select 11444,'1964-S792','1964-09-23 00:00:00','suborbital rocket','success',286,'Cedar 6','','','','Test',356,'',60 from dual union all
select 11445,'1964-S793','1964-07-09 00:00:00','military missile','success',871,'ZK-42 20157','Sim Target Intercept','','','Interceptor',873,'',80 from dual union all
select 11446,'1964-S794','1964-04-01 00:00:00','military missile','success',932,'A3X-53','PX-2 First PMR A3X','','','Test',1900,'EAG-154',1000 from dual union all
select 11447,'1964-S795','1964-04-08 00:00:00','military missile','failure',932,'A3X-55','PX-2','','','Test',1900,'EAG-154',10 from dual union all
select 11448,'1964-S796','1964-05-12 00:00:00','military missile','success',932,'A3X-60','PY-2','','','Test',1900,'EAG-154',1000 from dual union all
select 11449,'1964-S797','1964-09-24 00:00:00','military missile','success',837,'','SPEEDBALL II 59 SN26','','','Target?',1895,'',150 from dual union all
select 1145,'1951-S45','1951-11-03 00:35:00','suborbital rocket','success',59,'SC 22','Grenades','','','Aeronomy',1869,'',82 from dual union all
select 11450,'1964-S798','1964-09-30 00:00:00','military missile','success',837,'','SPEEDBALL II 60 SN27','','','Target?',1895,'',150 from dual union all
select 11451,'1964-S799','1964-11-20 00:00:00','military missile','success',932,'A3X','PX-2','','','OT',1900,'EAG-154',1000 from dual union all
select 11452,'1964-S80','1964-02-08 00:00:00','military missile','success',976,'','8K63 KY64-6','','','Test',636,'',402 from dual union all
select 11453,'1964-S800','1964-11-25 00:00:00','military missile','success',932,'A3X','PX-2','','','OT',1900,'EAG-154',1000 from dual union all
select 11454,'1964-S801','1964-05-15 00:00:00','suborbital rocket','success',839,'Falling sphere','','','','Aeron',871,'',110 from dual union all
select 11455,'1964-S802','1964-12-17 00:00:00','suborbital rocket','success',1250,'','Denpro','','','Meteo',1415,'',100 from dual union all
select 11456,'1964-S803','1964-06-08 00:00:00','suborbital rocket','success',578,'Hugo IV','Diagnostic','','','Test',1057,'',117 from dual union all
select 11457,'1964-S804','1964-09-30 00:00:00','suborbital rocket','success',578,'Hugo IV','TV payload','','','Meteo',1057,'',120 from dual union all
select 11458,'1964-S805','1964-01-18 00:00:00','suborbital rocket','success',1247,'','NITE OWL 3','','','UV Astron',1057,'F-3B',91 from dual union all
select 11459,'1964-S806','1964-10-15 00:00:00','suborbital rocket','success',1330,'','','','','Test',1415,'',500 from dual union all
select 1146,'1952-A01','1952-01-01 00:00:00','atmospheric rocket','failure',442,'Double Deacon','X-2 Model','','','Test',1821,'',0 from dual union all
select 11460,'1964-S807','1964-06-01 00:00:00','suborbital rocket','success',837,'ARPA PRESS','Mass Spec','','','Aeron',873,'',100 from dual union all
select 11461,'1964-S808','1964-06-01 00:00:00','suborbital rocket','success',837,'ARPA PRESS','Mass Spec','','','Aeron',873,'',100 from dual union all
select 11462,'1964-S809','1964-03-14 00:00:00','military missile','success',1323,'OTR-22S','9M76B No. 1','','','Test',636,'',100 from dual union all
select 11463,'1964-S81','1964-02-09 18:23:00','suborbital rocket','success',288,'C13','FU-104A','','','Aeronomy',664,'',180 from dual union all
select 11464,'1964-S810','1964-06-01 00:00:00','military missile','success',1323,'','9M76B No. 4','','','Test',636,'',100 from dual union all
select 11465,'1964-S811','1964-07-18 00:00:00','military missile','success',1323,'','9M76B No. 5','','','Test',636,'',100 from dual union all
select 11466,'1964-S812','1964-03-25 00:00:00','military missile','success',982,'','8K64U 54 RD UBP-3','','','OT',1468,'',1210 from dual union all
select 11467,'1964-S813','1964-11-01 00:00:00','military missile','success',982,'','8K64U 54 RD UBP-4','','','OT',1468,'',1210 from dual union all
select 11468,'1964-S814','1964-04-15 00:00:00','military missile','success',982,'','8K64 42 RD UBP-3','','','OT',1002,'',1210 from dual union all
select 11469,'1964-S815','1964-05-30 00:00:00','military missile','success',982,'','8K64 36 RD UBP-1','','','OT',898,'',1210 from dual union all
select 1147,'1952-A02','1952-01-04 00:00:00','atmospheric rocket','unknown',576,'16','','','','Test',1895,'',10 from dual union all
select 11470,'1964-S816','1964-06-05 00:00:00','military missile','success',1013,'','8K51 29 RD UBP-1','','','Test',636,'',500 from dual union all
select 11471,'1964-S817','1964-06-05 00:00:00','military missile','success',1013,'','8K51 29 RD UBP-2','','','Test',636,'',500 from dual union all
select 11472,'1964-S818','1964-10-12 00:00:00','military missile','success',1013,'','8K51 29 RD UBP-3','','','Test',636,'',500 from dual union all
select 11473,'1964-S819','1964-02-02 00:00:00','military missile','success',977,'','8K63U 29 RD UBP','ShPU','','Test',636,'',402 from dual union all
select 11474,'1964-S82','1964-02-09 04:12:00','suborbital rocket','success',30,'AC3.603','Utah ionos probe','','','Ionos/Auroral',428,'',228 from dual union all
select 11475,'1964-S820','1964-06-05 00:00:00','suborbital rocket','success',837,'','Propagation study','','','Ionos',1895,'',189 from dual union all
select 11476,'1964-S83','1964-02-10 00:00:00','suborbital rocket','success',446,'D-07','','','','Test',664,'',470 from dual union all
select 11477,'1964-S84','1964-02-11 00:00:00','military missile','success',794,'692','Mk 5 RV','OT','BLUE PAINT','OT',1701,'',900 from dual union all
select 11478,'1964-S85','1964-02-13 01:36:22','military missile','success',795,'436','Mk 11 RV/Decoys','R&D','','Test',266,'',900 from dual union all
select 11479,'1964-S86','1964-02-12 00:00:00','suborbital rocket','failure',288,'C52','FU-106','','','Aeronomy',664,'',23 from dual union all
select 1148,'1952-A03','1952-01-04 00:00:00','atmospheric rocket','unknown',576,'17','','','','Test',1895,'',10 from dual union all
select 11480,'1964-S87','1964-02-12 00:00:00','suborbital rocket','success',288,'C44','','','','Aeronomy',664,'',190 from dual union all
select 11481,'1964-S88','1964-02-12 00:00:00','suborbital rocket','success',288,'C56','','','','Aeronomy',664,'',190 from dual union all
select 11482,'1964-S89','1964-02-12 20:30:00','suborbital rocket','success',837,'NASA 14.28GT','Fin test','','','Test',1821,'',145 from dual union all
select 11483,'1964-S90','1964-02-13 00:00:00','military missile','success',976,'','8K63 KY64-7','','','Test',636,'',402 from dual union all
select 11484,'1964-S91','1964-02-13 05:00:00','suborbital rocket','success',288,'C57','','','','Aeronomy',664,'',170 from dual union all
select 11485,'1964-S92','1964-02-13 00:00:00','suborbital rocket','success',288,'C40','','','','Aeronomy',664,'',170 from dual union all
select 11486,'1964-S93','1964-02-13 04:29:00','suborbital rocket','success',839,'NASA 10.88GA','GSFC Grenade 44 (5C)','','','Aeronomy',428,'',119 from dual union all
select 11487,'1964-S94','1964-02-13 04:30:00','suborbital rocket','success',839,'NASA 10.136GA','GSFC Grenade 45 (5W)','','','Aeronomy',1821,'',119 from dual union all
select 11488,'1964-S95','1964-02-13 04:55:00','suborbital rocket','success',839,'NASA 10.82GA','GSFC Grenade 46 (5A)','','','Aeronomy',176,'',119 from dual union all
select 11489,'1964-S96','1964-02-14 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 1149,'1952-A04','1952-01-09 00:00:00','atmospheric rocket','unknown',576,'18','','','','Test',1895,'',10 from dual union all
select 11490,'1964-S97','1964-02-14 04:37:00','suborbital rocket','success',288,'C38','MPE Ba-5','','','Aeronomy',664,'',186 from dual union all
select 11491,'1964-S98','1964-02-14 07:47:00','military missile','success',1373,'299/20002','AFP-437 Demo','','','Interceptor',772,'',1000 from dual union all
select 11492,'1964-S99','1964-02-15 04:37:00','suborbital rocket','success',288,'C34','MPE Ba-6','','','Aeronomy',664,'',189 from dual union all
select 11493,'1964-U01','1964-03-12 04:17:00','suborbital rocket','success',839,'Meteor 2/3','','Meteor 2','','Test',1821,'',110 from dual union all
select 11494,'1964-U02','1964-04-14 21:42:25','test rocket','success',154,'263D','FIRE 1','FIRE 1','','Reentry Test',227,'',837 from dual union all
select 11495,'1964-U03','1964-04-21 18:50:00','orbital','failure',1358,'379/AB014','Transit VBN-3','Transit VBN-3','USEFUL TOOL','Sat',1673,'',500 from dual union all
select 11496,'1964-U04','1964-08-18 06:05:00','test rocket','success',1149,'S129R','Reentry 4','Reentry 4','','Reentry Test',1812,'',183 from dual union all
select 11497,'1964-U05','1964-11-07 01:38:00','suborbital rocket','success',839,'Meteor 4/5','','Meteor 4','','Test',1821,'',110 from dual union all
select 11498,'1964-W01','1964-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 11499,'1964-W02','1964-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 115,'1944-S104','1944-09-16 00:00:00','military missile','success',1470,'','London','','','Weapon',1745,'',95 from dual union all
select 1150,'1952-A05','1952-01-09 00:00:00','atmospheric rocket','unknown',576,'19','','','','Test',1895,'',10 from dual union all
select 11500,'1964-W03','1964-01-17 03:37:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11501,'1964-W04','1964-01-29 02:05:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11502,'1964-W05','1964-02-05 01:45:00','suborbital rocket','success',88,'','Deltasonde for 10.87','','','Aeron',428,'',60 from dual union all
select 11503,'1964-W06','1964-02-05 19:00:00','suborbital rocket','success',716,'','','','','Aeron',615,'',60 from dual union all
select 11504,'1964-W07','1964-02-05 22:30:00','suborbital rocket','success',716,'','','','','Aeron',615,'',60 from dual union all
select 11505,'1964-W08','1964-02-13 04:40:00','suborbital rocket','success',88,'','Deltasonde for 10.88','','','Aeron',428,'',60 from dual union all
select 11506,'1964-W09','1964-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 11507,'1964-W10','1964-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 11508,'1964-W100','1964-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 11509,'1964-W101','1964-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 1151,'1952-A06','1952-01-15 00:00:00','military missile','success',771,'GM-554','','','','Test',222,'',12 from dual union all
select 11510,'1964-W102','1964-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 11511,'1964-W103','1964-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 11512,'1964-W104','1964-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 11513,'1964-W105','1964-11-10 00:00:00','suborbital rocket','success',631,'','','','','Aeron',1895,'',75 from dual union all
select 11514,'1964-W106','1964-11-14 02:10:00','suborbital rocket','success',759,'MT-135-002','','','','Aeron',839,'',55 from dual union all
select 11515,'1964-W107','1964-11-15 07:10:00','suborbital rocket','success',759,'MT-135-003','','','','Aeron',839,'',55 from dual union all
select 11516,'1964-W108','1964-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 11517,'1964-W109','1964-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 11518,'1964-W11','1964-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 11519,'1964-W110','1964-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 1152,'1952-A07','1952-01-22 00:00:00','atmospheric rocket','success',869,'490B','System Test 74R','','','Test',1895,'',10 from dual union all
select 11520,'1964-W111','1964-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 11521,'1964-W112','1964-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 11522,'1964-W113','1964-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 11523,'1964-W114','1964-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 11524,'1964-W115','1964-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 11525,'1964-W116','1964-12-01 00:00:00','suborbital rocket','success',88,'','','','','Ionos',197,'',60 from dual union all
select 11526,'1964-W117','1964-12-02 00:00:00','suborbital rocket','success',631,'','','','','Aeron',1895,'',75 from dual union all
select 11527,'1964-W118','1964-12-04 00:00:00','suborbital rocket','success',1186,'LW-19','','','','Test',1895,'',56 from dual;
insert into LAUNCH
select 11528,'1964-W119','1964-12-05 06:55:00','suborbital rocket','success',631,'TERLS-15','ISRO 11.05','TERLS-015','','Aeron',1467,'',77 from dual union all
select 11529,'1964-W12','1964-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 1153,'1952-A08','1952-01-23 00:00:00','atmospheric rocket','success',442,'','X-3 Model','','','Test',1821,'',10 from dual union all
select 11530,'1964-W120','1964-12-07 11:15:00','suborbital rocket','success',88,'','ROBIN','','','Aeron',276,'',60 from dual union all
select 11531,'1964-W121','1964-12-07 17:05:00','suborbital rocket','success',88,'','Chute','','','Aeron',276,'',60 from dual union all
select 11532,'1964-W122','1964-12-11 18:05:00','suborbital rocket','success',88,'','PSU Arcas #2','','','Ionos',1895,'',75 from dual union all
select 11533,'1964-W123','1964-12-15 06:30:00','suborbital rocket','success',631,'TERLS-16','ISRO 11.06','TERLS-016','','Aeron',1467,'',77 from dual union all
select 11534,'1964-W124','1964-12-16 07:38:00','suborbital rocket','success',631,'TERLS-17','ISRO 11.07','TERLS-017','','Aeron',1467,'',77 from dual union all
select 11535,'1964-W125','1964-12-16 00:00:00','suborbital rocket','failure',494,'FAD-1','AFFT-1','','','Test',1415,'',58 from dual union all
select 11536,'1964-W126','1964-12-16 00:00:00','suborbital rocket','success',1186,'LW-21','PSU D-region','','','Test/Ionos',1895,'',67 from dual union all
select 11537,'1964-W127','1964-12-17 00:00:00','suborbital rocket','success',1186,'LW-22','','','','Test',1895,'',60 from dual union all
select 11538,'1964-W128','1964-07-01 00:00:00','suborbital rocket','unknown',197,'1','','','','Aeron',671,'',60 from dual union all
select 11539,'1964-W129','1964-10-21 13:58:00','suborbital rocket','success',631,'','SUPARCO','','','Aeron',1419,'',65 from dual union all
select 1154,'1952-A09','1952-01-24 00:00:00','military missile','unknown',339,'14-12R','','','','Test',1895,'',30 from dual union all
select 11540,'1964-W13','1964-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 11541,'1964-W130','1964-11-18 14:08:00','suborbital rocket','success',631,'','SUPARCO','','','Aeron',1419,'',65 from dual union all
select 11542,'1964-W131','1964-12-01 02:15:00','suborbital rocket','success',631,'','SUPARCO','','','Aeron',1419,'',65 from dual union all
select 11543,'1964-W132','1964-12-16 13:03:00','suborbital rocket','success',631,'','SUPARCO','','','Aeron',1419,'',65 from dual union all
select 11544,'1964-W133','1964-01-12 16:00:00','suborbital rocket','success',1188,'','Wind,Temp','','','Aeron',1447,'',50 from dual union all
select 11545,'1964-W134','1964-01-15 19:00:00','suborbital rocket','success',1188,'','Wind,Temp','','','Aeron',1447,'',57 from dual union all
select 11546,'1964-W135','1964-01-17 20:00:00','suborbital rocket','success',1188,'','Wind,Temp','','','Aeron',1447,'',65 from dual union all
select 11547,'1964-W136','1964-01-01 00:00:00','suborbital rocket','success',1050,'17','','','','Weather',1895,'',53 from dual union all
select 11548,'1964-W137','1964-01-01 00:00:00','suborbital rocket','success',1050,'18','','','','Weather',1895,'',61 from dual union all
select 11549,'1964-W138','1964-01-01 00:00:00','suborbital rocket','success',1050,'20','','','','Weather',1895,'',61 from dual union all
select 1155,'1952-A10','1952-01-25 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 11550,'1964-W139','1964-01-01 00:00:00','suborbital rocket','success',1050,'22','','','','Weather',1895,'',62 from dual union all
select 11551,'1964-W14','1964-03-13 20:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11552,'1964-W140','1964-01-01 00:00:00','suborbital rocket','success',1050,'24','','','','Weather',1895,'',63 from dual union all
select 11553,'1964-W141','1964-09-01 00:00:00','suborbital rocket','success',88,'','PSU Arcas #1','','','Ionos',1895,'',70 from dual union all
select 11554,'1964-W15','1964-03-25 18:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11555,'1964-W16','1964-03-29 02:12:00','suborbital rocket','success',759,'PT-135-1','Propellant Test','','','Test',839,'',51 from dual union all
select 11556,'1964-W17','1964-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 11557,'1964-W18','1964-03-31 07:15:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',64 from dual union all
select 11558,'1964-W19','1964-04-02 01:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11559,'1964-W20','1964-04-08 04:09:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',64 from dual union all
select 1156,'1952-A100','1952-06-19 00:00:00','military missile','unknown',339,'22R','','','','Test',1895,'',30 from dual union all
select 11560,'1964-W21','1964-04-08 18:25:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',64 from dual union all
select 11561,'1964-W22','1964-04-15 16:45:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',64 from dual union all
select 11562,'1964-W23','1964-04-22 18:07:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11563,'1964-W24','1964-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11564,'1964-W25','1964-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11565,'1964-W26','1964-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11566,'1964-W27','1964-05-06 18:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11567,'1964-W28','1964-05-20 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11568,'1964-W29','1964-05-27 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11569,'1964-W30','1964-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 1157,'1952-A101','1952-06-24 00:00:00','military missile','success',703,'230','','','','Test',222,'',8 from dual union all
select 11570,'1964-W31','1964-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11571,'1964-W32','1964-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11572,'1964-W33','1964-06-10 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11573,'1964-W34','1964-06-17 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',64 from dual union all
select 11574,'1964-W35','1964-06-25 00:00:00','suborbital rocket','failure',88,'T1-1673','BioSpace 1','','','Bio',1821,'',20 from dual union all
select 11575,'1964-W36','1964-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11576,'1964-W37','1964-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11577,'1964-W38','1964-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11578,'1964-W39','1964-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11579,'1964-W40','1964-07-10 00:00:00','suborbital rocket','failure',88,'T1-1674','BioSpace 2','','','Bio',1821,'',20 from dual union all
select 1158,'1952-A102','1952-06-25 00:00:00','atmospheric rocket','success',836,'1249-107 XM5 test','Nike 1249 R&D 98R','','','Test',1895,'',10 from dual union all
select 11580,'1964-W41','1964-07-14 06:54:00','suborbital rocket','success',631,'','ISRO 11.01','TERLS-008','','Aeron',1467,'',76 from dual union all
select 11581,'1964-W42','1964-07-15 09:55:00','suborbital rocket','success',631,'','ISRO 11.02','TERLS-009','','Aeron',1467,'',76 from dual union all
select 11582,'1964-W43','1964-07-15 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11583,'1964-W44','1964-07-16 09:10:00','suborbital rocket','success',631,'','ISRO 11.03','TERLS-010','','Aeron',1467,'',76 from dual union all
select 11584,'1964-W45','1964-07-23 00:00:00','suborbital rocket','success',88,'T1-1675','BioSpace 3','','','Bio',1821,'',27 from dual union all
select 11585,'1964-W46','1964-07-29 00:00:00','suborbital rocket','success',88,'T1-1676','BioSpace 4','','','Bio',1821,'',32 from dual union all
select 11586,'1964-W47','1964-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11587,'1964-W48','1964-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11588,'1964-W49','1964-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11589,'1964-W50','1964-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 1159,'1952-A103','1952-06-27 00:00:00','military missile','unknown',339,'24R','','','','Test',1895,'',30 from dual union all
select 11590,'1964-W51','1964-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11591,'1964-W52','1964-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron?',858,'',60 from dual union all
select 11592,'1964-W53','1964-08-05 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11593,'1964-W54','1964-08-06 12:07:00','suborbital rocket','success',88,'Weather','','','','Aeron',858,'',66 from dual union all
select 11594,'1964-W55','1964-08-10 18:30:00','suborbital rocket','success',716,'','','','','Aeron',1469,'',75 from dual union all
select 11595,'1964-W56','1964-08-12 18:52:00','suborbital rocket','success',716,'','','','','Aeron',1469,'',75 from dual union all
select 11596,'1964-W57','1964-08-17 18:34:00','suborbital rocket','success',716,'','','','','Aeron',1469,'',75 from dual union all
select 11597,'1964-W58','1964-08-18 00:00:00','suborbital rocket','success',88,'','Reentry 4 Cal (1of6)','','','Meteo',194,'',60 from dual union all
select 11598,'1964-W59','1964-08-18 00:00:00','suborbital rocket','success',88,'','Reentry 4 Cal','','','Meteo',194,'',60 from dual union all
select 11599,'1964-W60','1964-08-19 06:01:00','suborbital rocket','success',631,'','ISRO 11.04','TERLS-011','','Aeron',1467,'',78 from dual union all
select 116,'1944-S105','1944-09-16 11:50:00','military missile','success',8,'18746','Ma161','','','Training',661,'',90 from dual union all
select 1160,'1952-A104','1952-06-30 00:00:00','atmospheric rocket','unknown',576,'38','','','','Test',1895,'',10 from dual union all
select 11600,'1964-W61','1964-08-19 19:07:00','suborbital rocket','success',716,'','','','','Aeron',1469,'',75 from dual union all
select 11601,'1964-W62','1964-08-19 19:51:00','suborbital rocket','success',273,'01','','','','Test',394,'',75 from dual union all
select 11602,'1964-W63','1964-08-20 00:00:00','suborbital rocket','success',631,'','','','','Aeron',1895,'',75 from dual union all
select 11603,'1964-W64','1964-08-20 00:00:00','suborbital rocket','success',631,'','','','','Aeron',1895,'',75 from dual union all
select 11604,'1964-W65','1964-08-21 19:39:00','suborbital rocket','success',273,'02','','','','Test',394,'',75 from dual union all
select 11605,'1964-W66','1964-08-24 17:45:00','suborbital rocket','success',273,'03','','','','Test',394,'',75 from dual union all
select 11606,'1964-W67','1964-08-28 18:46:00','suborbital rocket','success',273,'04','','','','Test',394,'',75 from dual union all
select 11607,'1964-W68','1964-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11608,'1964-W69','1964-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 11609,'1964-W70','1964-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Aeron',428,'',60 from dual union all
select 1161,'1952-A105','1952-06-30 00:00:00','atmospheric rocket','unknown',576,'37','','','','Test',1895,'',10 from dual union all
select 11610,'1964-W71','1964-09-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron?',858,'',60 from dual union all
select 11611,'1964-W72','1964-09-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron?',858,'',60 from dual union all
select 11612,'1964-W73','1964-09-04 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11613,'1964-W74','1964-09-14 00:00:00','suborbital rocket','success',88,'','','','','Aeron?',858,'',60 from dual union all
select 11614,'1964-W75','1964-09-17 00:00:00','suborbital rocket','success',631,'','','','','Aeron',1895,'',75 from dual union all
select 11615,'1964-W76','1964-09-28 00:00:00','suborbital rocket','success',1186,'LW-9','','','','Test',1895,'',59 from dual union all
select 11616,'1964-W77','1964-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde I','','','Aeron',428,'',60 from dual union all
select 11617,'1964-W78','1964-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde I','','','Aeron',428,'',60 from dual;
insert into LAUNCH
select 11618,'1964-W79','1964-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde I','','','Aeron',428,'',60 from dual union all
select 11619,'1964-W80','1964-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde I','','','Aeron',428,'',60 from dual union all
select 1162,'1952-A106','1952-07-02 00:00:00','military missile','unknown',339,'99R','Special','','','Test',1895,'',30 from dual union all
select 11620,'1964-W81','1964-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde I','','','Aeron',428,'',60 from dual union all
select 11621,'1964-W82','1964-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde I','','','Aeron',428,'',60 from dual union all
select 11622,'1964-W83','1964-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde I','','','Aeron',428,'',60 from dual union all
select 11623,'1964-W84','1964-09-30 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11624,'1964-W85','1964-10-01 10:47:00','suborbital rocket','unknown',273,'NO!','','','','Aeron',394,'',75 from dual union all
select 11625,'1964-W86','1964-10-02 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11626,'1964-W87','1964-10-05 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11627,'1964-W88','1964-10-07 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11628,'1964-W89','1964-10-08 00:00:00','suborbital rocket','success',1186,'LW-13','','','','Test',1895,'',54 from dual union all
select 11629,'1964-W90','1964-10-12 16:07:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 1163,'1952-A107','1952-07-07 00:00:00','atmospheric rocket','unknown',576,'39','','','','Test',1895,'',10 from dual union all
select 11630,'1964-W91','1964-10-14 16:02:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11631,'1964-W92','1964-10-16 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11632,'1964-W93','1964-10-21 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11633,'1964-W94','1964-10-22 00:00:00','suborbital rocket','success',273,'06','','','','Test',394,'',75 from dual union all
select 11634,'1964-W95','1964-10-23 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11635,'1964-W96','1964-10-29 19:03:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 11636,'1964-W97','1964-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 11637,'1964-W98','1964-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 11638,'1964-W99','1964-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 11639,'1964-Y01','1964-01-01 00:00:00','suborbital rocket','success',1182,'','','','','Test',1895,'',113 from dual union all
select 1164,'1952-A108','1952-07-07 00:00:00','military missile','success',703,'592','','','','Test',222,'',8 from dual union all
select 11640,'1964-Y02','1964-01-01 00:00:00','suborbital rocket','success',1182,'','','','','Test',1895,'',113 from dual union all
select 11641,'1964-Y03','1964-01-05 22:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11642,'1964-Y04','1964-01-13 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11643,'1964-Y05','1964-01-15 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11644,'1964-Y06','1964-01-17 22:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11645,'1964-Y07','1964-02-13 18:28:00','suborbital rocket','success',577,'F15','','','','Aeron',276,'',90 from dual union all
select 11646,'1964-Y08','1964-02-13 21:45:00','suborbital rocket','success',577,'F16','','','','Aeron',276,'',90 from dual union all
select 11647,'1964-Y09','1964-02-14 21:03:00','suborbital rocket','success',577,'F18','','','','Aeron',276,'',90 from dual union all
select 11648,'1964-Y10','1964-03-06 23:25:00','suborbital rocket','success',257,'','UJO-4 (Na/Li)','','','Aeron',1821,'',97 from dual union all
select 11649,'1964-Y100','1964-10-22 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 1165,'1952-A109','1952-07-08 00:00:00','atmospheric rocket','failure',442,'','B-58 Model','','','Test',1821,'',0 from dual union all
select 11650,'1964-Y101','1964-10-23 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11651,'1964-Y102','1964-10-23 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11652,'1964-Y103','1964-10-26 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11653,'1964-Y104','1964-10-27 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11654,'1964-Y105','1964-10-28 22:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11655,'1964-Y106','1964-10-29 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11656,'1964-Y107','1964-10-30 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11657,'1964-Y108','1964-11-15 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11658,'1964-Y109','1964-11-18 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11659,'1964-Y11','1964-03-18 14:36:00','suborbital rocket','success',631,'','SUPARCO','','','Aeron',1419,'',80 from dual union all
select 1166,'1952-A11','1952-01-25 00:00:00','suborbital rocket','success',1495,'P6-1','','','','Test',218,'',1 from dual union all
select 11660,'1964-Y110','1964-11-23 21:48:00','suborbital rocket','success',577,'F21','','','','Aeron',1821,'',90 from dual union all
select 11661,'1964-Y111','1964-11-23 22:11:00','suborbital rocket','success',577,'F22','','','','Aeron',1821,'',90 from dual union all
select 11662,'1964-Y112','1964-02-14 18:30:00','suborbital rocket','success',577,'F17','','','','Aeron',276,'',90 from dual union all
select 11663,'1964-Y12','1964-04-18 22:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11664,'1964-Y13','1964-04-19 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11665,'1964-Y14','1964-04-20 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11666,'1964-Y15','1964-04-22 22:30:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11667,'1964-Y16','1964-04-24 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11668,'1964-Y17','1964-04-26 20:08:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11669,'1964-Y18','1964-04-27 17:32:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 1167,'1952-A110','1952-07-09 00:00:00','military missile','success',771,'GM-565','','','','Test',222,'',12 from dual union all
select 11670,'1964-Y19','1964-04-27 20:08:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11671,'1964-Y20','1964-04-28 17:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11672,'1964-Y21','1964-04-29 17:33:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11673,'1964-Y22','1964-04-29 20:08:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11674,'1964-Y23','1964-05-04 17:29:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11675,'1964-Y24','1964-05-04 20:08:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11676,'1964-Y25','1964-05-06 19:08:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11677,'1964-Y26','1964-05-07 20:08:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11678,'1964-Y27','1964-05-08 20:08:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11679,'1964-Y28','1964-05-10 20:08:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 1168,'1952-A111','1952-07-11 00:00:00','military missile','unknown',339,'25R','','','','Test',1895,'',30 from dual union all
select 11680,'1964-Y29','1964-05-12 17:06:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11681,'1964-Y30','1964-05-12 20:08:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11682,'1964-Y31','1964-05-12 22:30:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11683,'1964-Y32','1964-05-13 20:08:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11684,'1964-Y33','1964-05-14 17:36:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11685,'1964-Y34','1964-05-15 20:08:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11686,'1964-Y35','1964-05-17 20:08:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11687,'1964-Y36','1964-05-18 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11688,'1964-Y37','1964-05-19 17:02:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11689,'1964-Y38','1964-05-19 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 1169,'1952-A112','1952-07-15 00:00:00','military missile','failure',703,'606','','','','Test',222,'',0 from dual union all
select 11690,'1964-Y39','1964-05-20 17:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11691,'1964-Y40','1964-05-20 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11692,'1964-Y41','1964-05-21 17:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11693,'1964-Y42','1964-05-21 18:05:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11694,'1964-Y43','1964-05-21 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11695,'1964-Y44','1964-05-22 22:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11696,'1964-Y45','1964-06-07 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11697,'1964-Y46','1964-06-08 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11698,'1964-Y47','1964-06-10 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11699,'1964-Y48','1964-06-12 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 117,'1944-S106','1944-09-16 16:55:00','military missile','success',8,'18750','Ma166','','','Training',661,'',90 from dual union all
select 1170,'1952-A113','1952-07-17 00:00:00','military missile','unknown',339,'26R','','','','Test',1895,'',30 from dual union all
select 11700,'1964-Y49','1964-06-14 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11701,'1964-Y50','1964-06-16 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11702,'1964-Y51','1964-06-17 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11703,'1964-Y52','1964-06-18 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11704,'1964-Y53','1964-07-08 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11705,'1964-Y54','1964-07-09 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11706,'1964-Y55','1964-07-10 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11707,'1964-Y56','1964-07-12 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual;
insert into LAUNCH
select 11708,'1964-Y57','1964-07-13 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11709,'1964-Y58','1964-07-14 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 1171,'1952-A114','1952-07-17 00:00:00','atmospheric rocket','success',836,'1249-111 XM5 test','Nike 1249 R&D 100R','','','Test',1895,'',10 from dual union all
select 11710,'1964-Y59','1964-07-15 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11711,'1964-Y60','1964-07-16 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11712,'1964-Y61','1964-07-17 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11713,'1964-Y62','1964-07-20 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11714,'1964-Y63','1964-07-22 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11715,'1964-Y64','1964-07-24 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11716,'1964-Y65','1964-08-10 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11717,'1964-Y66','1964-08-12 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11718,'1964-Y67','1964-08-14 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11719,'1964-Y68','1964-08-16 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 1172,'1952-A115','1952-07-19 00:00:00','military missile','success',771,'GM-566','','','','Test',222,'',12 from dual union all
select 11720,'1964-Y69','1964-08-18 16:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11721,'1964-Y70','1964-08-18 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11722,'1964-Y71','1964-08-19 14:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11723,'1964-Y72','1964-08-19 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11724,'1964-Y73','1964-08-20 17:11:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11725,'1964-Y74','1964-08-20 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11726,'1964-Y75','1964-08-22 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11727,'1964-Y76','1964-08-24 17:36:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11728,'1964-Y77','1964-08-24 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11729,'1964-Y78','1964-08-26 18:08:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 1173,'1952-A116','1952-07-22 00:00:00','atmospheric rocket','success',836,'1249B-1001','Nike 1249 R&D 101R','','','Test',1895,'',10 from dual union all
select 11730,'1964-Y79','1964-08-26 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11731,'1964-Y80','1964-08-31 17:31:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11732,'1964-Y81','1964-09-01 17:15:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11733,'1964-Y82','1964-09-02 17:53:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11734,'1964-Y83','1964-09-09 17:05:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 11735,'1964-Y84','1964-09-09 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11736,'1964-Y85','1964-09-10 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11737,'1964-Y86','1964-09-12 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11738,'1964-Y87','1964-09-14 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11739,'1964-Y88','1964-09-24 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 1174,'1952-A117','1952-07-23 00:00:00','atmospheric rocket','unknown',576,'40','','','','Test',1895,'',10 from dual union all
select 11740,'1964-Y89','1964-10-12 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11741,'1964-Y90','1964-10-14 21:23:00','suborbital rocket','success',577,'F19','','','','Aeron',1821,'',90 from dual union all
select 11742,'1964-Y91','1964-10-14 22:00:00','suborbital rocket','success',577,'F20','','','','Aeron',1821,'',90 from dual union all
select 11743,'1964-Y92','1964-10-14 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11744,'1964-Y93','1964-10-16 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11745,'1964-Y94','1964-10-17 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11746,'1964-Y95','1964-10-19 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11747,'1964-Y96','1964-10-19 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11748,'1964-Y97','1964-10-20 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11749,'1964-Y98','1964-10-21 20:30:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 1175,'1952-A118','1952-07-23 00:00:00','atmospheric rocket','unknown',576,'41','','','','Test',1895,'',10 from dual union all
select 11750,'1964-Y99','1964-10-21 21:40:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 11751,'1965-001','1965-01-11 09:29:58','orbital','success',1509,'R 15002- 03','Kosmos-52','Zenit-2 No. 26','','Sat',562,'',null from dual union all
select 11752,'1965-002','1965-01-15 21:00:44','orbital','success',1394,'414/FTV 1608','CORONA 91/1016','CORONA J-18','BUCKET FACTORY','Sat TAG107',1679,'',null from dual union all
select 11753,'1965-003','1965-01-19 05:03:42','orbital','success',1390,'224','DAPP/OPS 7040','DAPP 10','ASTRAL LAMP','Sat',1660,'',null from dual union all
select 11754,'1965-004','1965-01-22 07:52:00','orbital','failure',1378,'374/D28','Tiros 9','Tiros I','','Sat',233,'',null from dual union all
select 11755,'1965-005','1965-01-23 20:09:00','orbital','success',177,'7106','AFP-206','GAMBIT SV 965','SAND LARK','Sat',1772,'',null from dual union all
select 11756,'1965-006','1965-01-30 09:36:00','orbital','success',681,'','Kosmos-53','DS-A1 No. 5','','Sat',626,'',null from dual union all
select 11757,'1965-007','1965-02-03 16:36:00','orbital','success',1378,'411/D29','OSO 2','OSO B2','','Sat',234,'',null from dual union all
select 11758,'1965-008','1965-02-11 15:19:05','orbital','success',1423,'3A-3','LES 1','Payload Truss','','Sat',239,'',null from dual union all
select 11759,'1965-009','1965-02-16 14:37:03','orbital','success',1119,'SA-9','AS-109/Pegasus 1','Pegasus 1','','Sat',271,'',null from dual union all
select 1176,'1952-A119','1952-07-28 00:00:00','military missile','unknown',339,'27R','','','','Test',1895,'',30 from dual union all
select 11760,'1965-010','1965-02-17 17:05:00','deep space','success',145,'196D (AA13)','Ranger 8','RA-8','','Sat',227,'',null from dual union all
select 11761,'1965-011','1965-02-21 11:00:00','orbital','success',682,'4000-000-03L','Kosmos-54/56','Strela-1','','Sat',564,'',null from dual union all
select 11762,'1965-012','1965-02-22 07:30:00','orbital','success',1508,'R 15000- 03','Kosmos-57','Voskhod 3KD No. 1','','Sat',562,'',null from dual union all
select 11763,'1965-013','1965-02-25 21:44:55','orbital','success',1394,'432/FTV 1611','CORONA 92/1017','CORONA J-14','BOAT CAMP','Sat TAG108',1770,'',null from dual union all
select 11764,'1965-014','1965-02-26 05:01:32','orbital','success',1510,'R 15000- 09','Kosmos-58','Meteor 11F614 No. 2','','Sat',562,'',null from dual union all
select 11765,'1965-015','1965-03-07 08:59:28','orbital','success',1508,'R 15001- 05','Kosmos-59','Zenit-4 No. 11','','Sat',562,'',null from dual union all
select 11766,'1965-016','1965-03-09 18:29:47','orbital','success',1392,'419/FTV 2701','POPPY 4/NRL Comp 5','Secor Type II S/N 1','DEUCE SPOT','Sat TAG109',1674,'',null from dual union all
select 11767,'1965-017','1965-03-11 13:39:59','orbital','success',1358,'440/AB018','NNS O-3','Secor Type II S/N 3','BUSH CATTLE','Sat',1673,'',null from dual union all
select 11768,'1965-018','1965-03-12 09:25:31','orbital','failure',800,'G 15000- 24/R10325','Kosmos-60 [8K78/E6 ]','E-6 No. 9','','Sat',480,'',270 from dual union all
select 11769,'1965-019','1965-03-12 19:25:00','orbital','success',177,'7104','AFP-206','GAMBIT SV 966','SHIP RAIL','Sat',1772,'',null from dual union all
select 1177,'1952-A12','1952-01-26 00:00:00','suborbital rocket','success',1496,'R7','','','','Test',218,'',1 from dual union all
select 11770,'1965-020','1965-03-15 11:00:00','orbital','success',682,'4000-000-04L','Kosmos-61/63','Strela-1','','Sat',564,'',null from dual union all
select 11771,'1965-021','1965-03-18 04:43:46','orbital','success',1390,'306','DAPP/OPS 7353','DAPP 11','ASTRAL BODY','Sat',1660,'',null from dual union all
select 11772,'1965-022','1965-03-18 07:00:00','orbital','success',1508,'R 15000- 05','Voskhod-2','Voskhod 3KD No. 4','','Sat',480,'',null from dual union all
select 11773,'1965-023','1965-03-21 21:37:02','deep space','success',145,'204D (AA14)','Ranger 9','RA-9','','Sat',227,'',null from dual union all
select 11774,'1965-024','1965-03-23 14:24:00','orbital','success',1421,'GT-3','Gemini 3','Gemini SC3','','Sat',237,'',null from dual union all
select 11775,'1965-025','1965-03-25 10:00:00','orbital','success',1509,'G 15001- 06','Kosmos-64','Zenit-2 No. 17','','Sat',562,'',null from dual union all
select 11776,'1965-026','1965-03-25 21:11:17','orbital','success',1394,'429/FTV 1612','CORONA 93/1018','CORONA J-19','PAPER ROUTE','Sat TAG110',1678,'',null from dual union all
select 11777,'1965-027','1965-04-03 21:25:00','orbital','success',177,'7401','SNAPSHOT','SNAP-10A FS-4','AIR PUMP','Sat',1773,'',null from dual union all
select 11778,'1965-028','1965-04-06 23:47:50','orbital','success',1380,'426/D30','Early Bird','INTELSAT I','','Sat',233,'',null from dual union all
select 11779,'1965-029','1965-04-17 09:50:00','orbital','success',1508,'G 15000- 11','Kosmos-65','Zenit-4 No. 6','','Sat',562,'',null from dual union all
select 1178,'1952-A120','1952-07-28 00:00:00','atmospheric rocket','unknown',576,'43','','','','Test',1895,'',10 from dual union all
select 11780,'1965-030','1965-04-23 01:55:00','orbital','success',802,'U 15000 -35/U10335','Molniya-1','Molniya-1 11F67 No. 3','','Sat',480,'',null from dual union all
select 11781,'1965-031','1965-04-28 20:17:00','orbital','success',177,'7107','AFP-206','GAMBIT SV 967','DWARF TREE','Sat',1773,'',null from dual union all
select 11782,'1965-032','1965-04-29 14:17:00','orbital','success',1148,'S136R','Explorer 27','Beacon Explorer C','','Sat',1814,'',null from dual union all
select 11783,'1965-033','1965-04-29 21:44:56','orbital','success',1394,'437/FTV 1614','CORONA 94/1019','CORONA J-4','MUSK OX','Sat TAG111',1770,'',null from dual union all
select 11784,'1965-034','1965-05-06 15:00:03','orbital','success',1423,'3A-6','LES 2','','','Sat',239,'',null from dual union all
select 11785,'1965-035','1965-05-07 09:29:59','orbital','success',1509,'R 15002- 04','Kosmos-66','Zenit-2 No. 27','','Sat',562,'',null from dual union all
select 11786,'1965-036','1965-05-09 07:49:37','deep space','success',801,'U 15000- 24/U10330','Luna-5    [8K78/E6]','E-6 No. 10','','Sat',480,'',null from dual union all
select 11787,'1965-037','1965-05-18 18:02:18','orbital','success',1394,'438/FTV 1615','CORONA 95/1021','CORONA J-21','IVY VINE','Sat TAG112',1678,'',null from dual union all
select 11788,'1965-038','1965-05-20 16:30:53','orbital','success',1363,'282','DAPP/OPS 8386','DAPP 12 (Block 3)','ROYAL EAGLE','Sat',1660,'',null from dual union all
select 11789,'1965-039','1965-05-25 07:35:01','orbital','success',1119,'SA-8','AS-108/Pegasus 2','Pegasus 2','','Sat',271,'',null from dual union all
select 1179,'1952-A121','1952-07-28 00:00:00','atmospheric rocket','unknown',576,'42','','','','Test',1895,'',10 from dual union all
select 11790,'1965-040','1965-05-25 10:50:00','orbital','success',1508,'R 15001- 04','Kosmos-67','Zenit-4 No. 10','','Sat',562,'',null from dual union all
select 11791,'1965-041','1965-05-27 19:30:00','orbital','success',177,'7108','AFP-206','GAMBIT SV 968','BOTTOM LAND','Sat',1773,'',null from dual union all
select 11792,'1965-042','1965-05-29 12:00:00','orbital','success',1378,'441/D31','Explorer 28','IMP C','','Sat',234,'',null from dual union all
select 11793,'1965-043','1965-06-03 15:16:00','orbital','success',1421,'GT-4','Gemini 4','Gemini SC4','','Sat',237,'',null from dual union all
select 11794,'1965-044','1965-06-08 07:40:23','deep space','success',801,'U 15000- 33/U10331','Luna-6    [8K78/E6]','E-6 No. 7','','Sat',480,'',null from dual union all
select 11795,'1965-045','1965-06-09 21:58:16','orbital','success',1394,'444/FTV 1613','CORONA 96/1020','CORONA J-20','FEMALE LOGIC','Sat TAG113',1679,'',null from dual union all
select 11796,'1965-046','1965-06-15 09:43:02','orbital','success',1509,'U 15001- 01','Kosmos-68','Zenit-2 No. 29','','Sat',562,'',null from dual union all
select 11797,'1965-047','1965-06-18 14:00:04','orbital','success',1425,'3C-7','Transtage','Lead Ballast','','Sat',274,'',null from dual union all
select 11798,'1965-048','1965-06-24 22:35:29','orbital','success',1358,'447/AB019','NNS O-4','NNS O-4','PARADISE TREE','Sat',1673,'',null from dual;
insert into LAUNCH
select 11799,'1965-049','1965-06-25 09:45:00','orbital','success',1508,'G 15000- 10','Kosmos-69','Zenit-4 No. 4','','Sat',480,'',null from dual union all
select 118,'1944-S107','1944-09-17 00:00:00','military missile','success',1470,'','London','','','Weapon',1745,'',95 from dual union all
select 1180,'1952-A122','1952-07-29 00:00:00','atmospheric rocket','success',836,'1249B-1002','Nike 1249 R&D 102R','','','Test',1895,'',10 from dual union all
select 11800,'1965-050','1965-06-25 19:30:00','orbital','success',177,'7109','AFP-206','P-11 No. 4402','WORN FACE','Sat',1773,'',null from dual union all
select 11801,'1965-051','1965-07-02 04:07:00','orbital','success',1378,'415/D32','Tiros 10','Tiros OT1','','Sat',234,'',null from dual union all
select 11802,'1965-052','1965-07-02 06:29:00','orbital','success',681,'','Kosmos-70','DS-A1 No. 7','','Sat',626,'',null from dual union all
select 11803,'1965-053','1965-07-16 03:31:00','orbital','success',682,'4000-000-05L','Kosmos-71/75','Strela-1','','Sat',564,'',null from dual union all
select 11804,'1965-054','1965-07-16 11:16:00','orbital','success',1458,'107207-01 (207)','Proton','N-4 No. 1','','Sat',587,'',null from dual union all
select 11805,'1965-055','1965-07-17 05:55:01','orbital','success',1394,'422/FTV 2702','OPS 8411','Ferret 8','LOST NOVEMBER','Sat TAG114',1674,'',null from dual union all
select 11806,'1965-056','1965-07-18 14:32:05','deep space','success',802,'U 15000- 32','Zond-3','3MV-4 No. 3','','Sat',480,'',null from dual union all
select 11807,'1965-057','1965-07-19 22:01:12','orbital','success',1394,'446/FTV 1617','CORONA 97/1022','CORONA J-22','ROCKY RIVER','Sat TAG115',1770,'',null from dual union all
select 11808,'1965-058','1965-07-20 08:27:00','orbital','success',146,'225D','Vela 3A/3B','Vela 3A','','Sat',228,'',null from dual union all
select 11809,'1965-059','1965-07-23 04:34:00','orbital','success',681,'','Kosmos-76','DS-P1-Yu No. 3','','Sat',626,'',null from dual union all
select 1181,'1952-A123','1952-07-31 00:00:00','atmospheric rocket','success',836,'1249-109','Nike 1249 R&D 103R','','','Test',1895,'',10 from dual union all
select 11810,'1965-060','1965-07-30 13:00:00','orbital','success',1119,'SA-10','AS-110/Pegasus 3','Pegasus 3','','Sat',271,'',null from dual union all
select 11811,'1965-061','1965-08-03 11:00:00','orbital','success',1508,'U 15001- 01','Kosmos-77','Zenit-4 No. 13','','Sat',562,'',null from dual union all
select 11812,'1965-062','1965-08-03 19:12:00','orbital','success',177,'7111','AFP-206','GAMBIT SV 971','WATER TOWER','Sat',1773,'',null from dual union all
select 11813,'1965-063','1965-08-10 17:54:00','orbital','success',1128,'S131R','SEV/Secor 5','Secor Type I S/N 2','','Sat',1814,'',null from dual union all
select 11814,'1965-064','1965-08-11 14:31:04','deep space','success',153,'AC-6','SD-2','Surveyor SD-2','','Sat',269,'',null from dual union all
select 11815,'1965-065','1965-08-13 22:11:24','orbital','success',1358,'455/AB020','NNS O-5','Dodecapole 2','BEAUTY SHOP','Sat',1673,'',null from dual union all
select 11816,'1965-066','1965-08-14 10:56:46','orbital','success',1509,'U 15001- 02','Kosmos-78','Zenit-2 No. 30','','Sat',562,'',null from dual union all
select 11817,'1965-067','1965-08-17 20:59:57','orbital','success',1394,'449/FTV 1618','CORONA 98/1023','CORONA J-23','LIGHTS OUT','Sat TAG116',1770,'',null from dual union all
select 11818,'1965-068','1965-08-21 14:00:00','orbital','success',1421,'GT-5','Gemini 5','Gemini SC5','','Sat',237,'',null from dual union all
select 11819,'1965-069','1965-08-25 10:10:02','orbital','success',1508,'R 15001- 06','Kosmos-79','Zenit-4 No. 7','','Sat',480,'',null from dual union all
select 1182,'1952-A124','1952-08-04 00:00:00','atmospheric rocket','unknown',576,'44','','','','Test',1895,'',10 from dual union all
select 11820,'1965-070','1965-09-03 14:00:00','orbital','success',682,'4000-000-07LS','Kosmos-80/84','Strela-1','','Sat',564,'',null from dual union all
select 11821,'1965-071','1965-09-09 09:30:00','orbital','success',1508,'R 15001- 02','Kosmos-85','Zenit-4 No. 8','','Sat',562,'',null from dual union all
select 11822,'1965-072','1965-09-10 04:41:38','orbital','success',1363,'213','DAPP/OPS 8068','DAPP 13','VICTORIA CROSS','Sat',1660,'',null from dual union all
select 11823,'1965-073','1965-09-18 07:59:00','orbital','success',682,'4000-000-08LS','Kosmos-86/90','Strela-1','','Sat',564,'',null from dual union all
select 11824,'1965-074','1965-09-22 21:31:14','orbital','success',1394,'458/FTV 1619','CORONA 100/1024','CORONA J-24','NICKLE SILVER','Sat TAG118',1770,'',null from dual union all
select 11825,'1965-075','1965-09-23 09:00:00','orbital','success',1508,'R 15001- 03','Kosmos-91','Zenit-4 No. 9','','Sat',562,'',null from dual union all
select 11826,'1965-076','1965-09-30 19:20:00','orbital','success',177,'7110','AFP-206','GAMBIT SV 972','LOG FOG','Sat',1773,'',null from dual union all
select 11827,'1965-077','1965-10-04 07:36:48','deep space','success',809,'U 15000- 54/U10327','Luna-7   [8K78/E6]','E-6 No. 11','','Sat',480,'',null from dual union all
select 11828,'1965-078','1965-10-05 09:07:08','orbital','success',154,'34D   NB20.170  RS','OV1-2/Pod 29','OV1-2S','SEETHING CITY','Sat/Aeron/Ionos',1685,'',null from dual union all
select 11829,'1965-079','1965-10-05 17:45:57','orbital','success',1394,'433/FTV 1616','CORONA 101/1025','CORONA JX-28','UNION LEADER','Sat TAG119',1679,'',null from dual union all
select 1183,'1952-A125','1952-08-04 00:00:00','atmospheric rocket','unknown',576,'45','','','','Test',1895,'',10 from dual union all
select 11830,'1965-080','1965-10-14 06:00:00','orbital','success',802,'U 15000- 34','Molniya-1','Molniya-1 11F67 No. 4','','Sat',480,'',null from dual union all
select 11831,'1965-081','1965-10-14 13:11:55','orbital','success',1394,'435/FTV 6801 (TA4)','OGO 2','OGO C','OLD ABBEY','Sat TAG120',1673,'',null from dual union all
select 11832,'1965-082','1965-10-15 17:23:59','orbital','success',1425,'3C-4','OV2-1','LCS 2','','Sat',274,'',null from dual union all
select 11833,'1965-083','1965-10-16 08:14:59','orbital','success',1508,'U 15001- 04','Kosmos-92','Zenit-4 No. 16','','Sat',562,'',null from dual union all
select 11834,'1965-084','1965-10-19 05:44:00','orbital','success',678,'','Kosmos-93','DS-U2-V No. 1','','Sat',626,'',null from dual union all
select 11835,'1965-085','1965-10-28 08:15:01','orbital','success',1508,'U 15001- 03','Kosmos-94','Zenit-4 No. 15','','Sat',562,'',null from dual union all
select 11836,'1965-086','1965-10-28 21:17:12','orbital','success',1394,'439/FTV 1620','CORONA 102/1026','CORONA J-25','HIGH JOURNEY','Sat TAG121',1770,'',null from dual union all
select 11837,'1965-087','1965-11-02 12:28:00','orbital','success',1458,'209','Proton-2','N-4 No. 2','','Sat',587,'',null from dual union all
select 11838,'1965-088','1965-11-04 05:31:00','orbital','success',678,'','Kosmos-95','DS-U2-V No. 2','','Sat',626,'',null from dual union all
select 11839,'1965-089','1965-11-06 18:38:43','orbital','success',1381,'457/D34','Explorer 29','GEOS A','','Sat',233,'',null from dual union all
select 1184,'1952-A126','1952-08-05 00:00:00','atmospheric rocket','success',367,'','Hermes Ramjet Model','','','Test',1821,'',10 from dual union all
select 11840,'1965-090','1965-11-08 19:26:00','orbital','success',177,'7113','AFP-206','GAMBIT SV 973','SHOP DEGREE','Sat',1773,'',null from dual union all
select 11841,'1965-091','1965-11-12 04:46:48','deep space','success',801,'U 15000- 42','Venera-2','3MV-4 No. 4','','Sat',562,'',null from dual union all
select 11842,'1965-092','1965-11-16 04:13:35','deep space','success',801,'U 15000- 31','Venera-3','3MV-3 No. 1','','Sat',562,'',null from dual union all
select 11843,'1965-093','1965-11-19 04:48:27','orbital','success',1148,'S138R','Explorer 30','Solar Explorer A','','Sat',1814,'',null from dual union all
select 11844,'1965-094','1965-11-23 03:14:10','orbital','failure',801,'U 15000- 30','Kosmos-96','3MV-4 No. 6','','Sat',562,'',262 from dual union all
select 11845,'1965-095','1965-11-26 12:14:00','orbital','success',678,'','Kosmos-97','DS-U2-M No. 1','','Sat',626,'',null from dual union all
select 11846,'1965-096','1965-11-26 14:47:21','orbital','success',431,'No. 1','A-1','A-1','','Sat',669,'',null from dual union all
select 11847,'1965-097','1965-11-27 08:20:00','orbital','success',1509,'U 15001- 05','Kosmos-98','Zenit-2 No. 31','','Sat',562,'',null from dual union all
select 11848,'1965-098','1965-11-29 04:48:47','orbital','success',1391,'453/FTV 6102 (TA5)','ISIS-X','Alouette 2','RE HEAT','Sat TAG122',1673,'',null from dual union all
select 11849,'1965-099','1965-12-03 10:46:14','deep space','success',802,'U 15000- 48/U10328','Luna-8  [8K78/E6]','E-6 No. 12','','Sat',562,'',null from dual union all
select 1185,'1952-A127','1952-08-06 00:00:00','atmospheric rocket','success',442,'','B-58 Model','','','Test',1821,'',10 from dual union all
select 11850,'1965-100','1965-12-04 19:30:04','orbital','success',1421,'GT-7','Gemini 7','Gemini SC7','','Sat',237,'',null from dual union all
select 11851,'1965-101','1965-12-06 21:05:47','orbital','success',1148,'S139R','FR-1','FR-1','SQUEEKY HUB?','Sat',1769,'',null from dual union all
select 11852,'1965-102','1965-12-09 21:10:19','orbital','success',1394,'448/FTV 1621','CORONA 103/1027','CORONA JX-27','LUCKY FELLOW','Sat TAG123',1679,'',null from dual union all
select 11853,'1965-103','1965-12-10 08:10:00','orbital','success',1509,'U 15001- 04','Kosmos-99','Zenit-2 No. 32','','Sat',562,'',null from dual union all
select 11854,'1965-104','1965-12-15 13:37:26','orbital','success',1421,'GT-6','Gemini 6','Gemini SC6','','Sat',237,'',null from dual union all
select 11855,'1965-105','1965-12-16 07:31:21','deep space','success',1381,'460/D35','Pioneer 6','Pioneer A','','Sat',233,'',null from dual union all
select 11856,'1965-106','1965-12-17 02:20:00','orbital','success',1510,'R 15000- 31','Kosmos-100','Meteor 11F614 No. 3','','Sat',562,'',null from dual union all
select 11857,'1965-107','1965-12-21 06:14:00','orbital','success',681,'','Kosmos-101','DS-P1-Yu No. 4','','Sat',626,'',null from dual union all
select 11858,'1965-108','1965-12-21 14:00:01','orbital','success',1425,'3C-8','OV2-3/LES 3/LES 4','LES 4','','Sat',275,'',null from dual union all
select 11859,'1965-109','1965-12-22 04:33:04','orbital','success',1125,'S140C','NNS O-6  (N-5)','NNS O-6','SOCIAL CIRCLE','Sat',1769,'',null from dual union all
select 1186,'1952-A128','1952-08-06 00:00:00','atmospheric rocket','success',365,'','D-558-II Model','','','Test',1821,'',10 from dual union all
select 11860,'1965-110','1965-12-24 21:06:15','orbital','success',1394,'451/FTV 1610','CORONA 104/1028','CORONA J-26','TALL STORY','Sat TAG124',1678,'',null from dual union all
select 11861,'1965-111','1965-12-27 22:24:56','orbital','success',1222,'G 15000- 01','Kosmos-102','4Ya11 (US-A)','','Sat',562,'',null from dual union all
select 11862,'1965-112','1965-12-28 12:30:00','orbital','success',682,'4000-000-09LP','Kosmos-103','Strela-2','','Sat',564,'',null from dual union all
select 11863,'1965-A01','1965-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 11864,'1965-A02','1965-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 11865,'1965-A03','1965-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 11866,'1965-A04','1965-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 11867,'1965-A05','1965-01-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 11868,'1965-A06','1965-01-05 20:00:00','atmospheric rocket','success',835,'Smoke 072','','','','Aeronomy',1821,'',16 from dual union all
select 11869,'1965-A07','1965-01-07 00:00:00','suborbital rocket','failure',589,'SAP-1','LRL SAP-1','','','Ionos/Mag',159,'T-AGM-8',2 from dual union all
select 1187,'1952-A129','1952-08-07 00:00:00','military missile','unknown',339,'28R','Firestone','','','Test',1895,'',30 from dual union all
select 11870,'1965-A08','1965-01-13 05:02:00','military missile','failure',999,'R 22500-20L','8K67 LKI-18','Silo test','','Test',585,'',0 from dual union all
select 11871,'1965-A09','1965-01-13 18:51:06','suborbital spaceplane','success',1520,'3-39-62','3D,HT,Fric,BLN','','','Test',379,'NB-52 003',30 from dual union all
select 11872,'1965-A10','1965-01-14 18:19:00','atmospheric rocket','success',835,'Smoke 074','','','','Aeronomy',1821,'',15 from dual union all
select 11873,'1965-A100','1965-07-14 07:57:00','suborbital rocket','failure',30,'AF3.265/VFT65-S-2','Noctilucent clouds','','','Aeron   AF3.285?',428,'',0 from dual union all
select 11874,'1965-A101','1965-07-14 18:09:00','atmospheric rocket','success',835,'Smoke 086','','','','Aeronomy',1821,'',16 from dual union all
select 11875,'1965-A102','1965-07-17 06:40:00','military missile','destroyed before launch',1449,'22942-001','8K84 LKI','GCh 8F121','','Test',505,'',0 from dual union all
select 11876,'1965-A103','1965-07-17 15:05:00','suborbital rocket','success',837,'AE5.814','EM Absorption','','','Ionos',394,'',17 from dual union all
select 11877,'1965-A104','1965-07-20 00:00:00','military missile','failure',135,'C007','Chaff Payload E-1','ABRES E-1 (USAF)','','Reentry Test',614,'',0 from dual union all
select 11878,'1965-A105','1965-07-20 17:59:28','suborbital spaceplane','success',1520,'3-45-69','Bound.Layer Noise','','','Test',379,'NB-52 008',32 from dual union all
select 11879,'1965-A106','1965-07-21 02:23:00','suborbital rocket','success',1069,'ST-160-1','Spin Test','','','Test',839,'',9 from dual union all
select 1188,'1952-A13','1952-01-28 00:00:00','military missile','success',703,'600','','','','Test',222,'',8 from dual union all
select 11880,'1965-A107','1965-07-21 05:20:00','suborbital rocket','success',1069,'ST-160-2','','','','Test',839,'',14 from dual union all
select 11881,'1965-A108','1965-07-21 17:08:00','atmospheric rocket','success',835,'Smoke 087','','','','Aeronomy',1821,'',15 from dual union all
select 11882,'1965-A109','1965-07-23 00:00:00','atmospheric rocket','success',1065,'Ra30','Ranger 30','','','Test',2,'',20 from dual union all
select 11883,'1965-A11','1965-01-21 00:00:00','suborbital rocket','failure',1186,'LW-20','','','','Test',1895,'',2 from dual union all
select 11884,'1965-A110','1965-07-23 00:00:00','suborbital spaceplane','failure',1520,'1- A-90','','','','Test',379,'NB-52',14 from dual union all
select 11885,'1965-A111','1965-07-26 00:00:00','suborbital rocket','failure',1186,'EX-1.8','','','','Aeron',394,'',1 from dual union all
select 11886,'1965-A112','1965-07-27 00:00:00','military missile','failure',135,'D005','ABRES P-5 (USAF)','ABRES P-5 (USAF)','','Reentry Test',612,'',0 from dual union all
select 11887,'1965-A113','1965-07-27 00:00:00','suborbital spaceplane','failure',1520,'1- A-91','','','','Test',379,'NB-52',14 from dual union all
select 11888,'1965-A114','1965-07-28 00:00:00','suborbital spaceplane','failure',1520,'1- A-92','','','','Test',379,'NB-52',14 from dual;
insert into LAUNCH
select 11889,'1965-A115','1965-07-29 01:15:00','suborbital rocket','failure',1199,'SL105','LUX-29/UCL I-23','','','Ionos/Solar X',1839,'',0 from dual union all
select 1189,'1952-A130','1952-08-08 00:00:00','atmospheric rocket','success',836,'1249-112','Nike 1249 R&D 104R','','','Test',1895,'',10 from dual union all
select 11890,'1965-A116','1965-07-31 00:00:00','military missile','failure',1449,'','8K84 LKI-8','','','Test',505,'',0 from dual union all
select 11891,'1965-A117','1965-08-03 04:15:00','military missile','failure',999,'U 22500-31L','8K67 LKI-23','','','Test',485,'',0 from dual union all
select 11892,'1965-A118','1965-08-04 00:00:00','military missile','success',1162,'','SIGS Test','','','Tech',1895,'',10 from dual union all
select 11893,'1965-A119','1965-08-06 17:41:46','suborbital spaceplane','success',1520,'1-56-93','IR Scan,Stability','','','Test',379,'NB-52 008',31 from dual union all
select 11894,'1965-A12','1965-01-21 18:11:00','atmospheric rocket','success',835,'Smoke 075','','','','Aeronomy',1821,'',18 from dual union all
select 11895,'1965-A120','1965-08-12 19:57:00','atmospheric rocket','success',835,'Smoke 088','','','','Aeronomy',1821,'',16 from dual union all
select 11896,'1965-A121','1965-08-20 00:00:00','military missile','failure',135,'D009','Subscale RV','ABRES C-2 (USAF)','','Reentry Test',612,'',0 from dual union all
select 11897,'1965-A122','1965-08-20 00:00:00','suborbital spaceplane','failure',1520,'1- A-94','','','','Test',379,'NB-52',14 from dual union all
select 11898,'1965-A123','1965-08-21 03:10:00','military missile','failure',999,'R 22500-25L','8K67 LKI-25','','','Test',485,'',0 from dual union all
select 11899,'1965-A124','1965-08-24 00:00:00','suborbital spaceplane','failure',1520,'1- A-95','','','','Test',379,'NB-52',14 from dual union all
select 119,'1944-S108','1944-09-17 00:00:00','military missile','success',1470,'','London','','','Weapon',1745,'',95 from dual union all
select 1190,'1952-A131','1952-08-08 00:00:00','atmospheric rocket','success',836,'1249B-1003','Nike 1249 R&D 105R','','','Test',1895,'',10 from dual union all
select 11900,'1965-A125','1965-08-25 00:00:00','atmospheric rocket','failure',617,'J202','HRV 921/UK-2F','','','Test',1836,'',10 from dual union all
select 11901,'1965-A126','1965-08-25 00:15:00','suborbital rocket','success',1069,'BT-160-1','Booster test','','','Test',839,'',2 from dual union all
select 11902,'1965-A127','1965-08-25 02:09:00','suborbital rocket','success',1069,'BT-160-2','Booster test','','','Test',839,'',9 from dual union all
select 11903,'1965-A128','1965-09-22 00:00:00','military missile','success',540,'4','HIBEX D-1','','','Test',1895,'',20 from dual union all
select 11904,'1965-A129','1965-09-01 00:00:00','military missile','failure',1035,'RT-1-1963','','','','Test',636,'',10 from dual union all
select 11905,'1965-A13','1965-01-25 20:53:00','suborbital rocket','failure',216,'CC-2B-56','','','','Test',428,'',20 from dual union all
select 11906,'1965-A130','1965-09-09 17:55:50','suborbital spaceplane','success',1520,'1-58-97','IR scan,Abl TPS','','','Test',379,'NB-52 008',29 from dual union all
select 11907,'1965-A131','1965-09-16 14:04:00','suborbital rocket','failure',837,'NASA 14.234UE','Auroral electrons','','','Aurora',428,'',27 from dual union all
select 11908,'1965-A132','1965-09-17 00:00:00','military missile','failure',1449,'','8K84 LKI-10','','','Test',504,'',0 from dual union all
select 11909,'1965-A133','1965-09-20 14:10:00','suborbital rocket','failure',837,'NASA 14.237UE','Auroral electrons','','','Aurora',428,'',20 from dual union all
select 1191,'1952-A132','1952-08-11 00:00:00','atmospheric rocket','unknown',576,'46','','','','Test',1895,'',10 from dual union all
select 11910,'1965-A134','1965-09-22 18:58:39','suborbital spaceplane','success',1520,'1-59-98','IR Scan','','','Test',379,'NB-52 003',30 from dual union all
select 11911,'1965-A135','1965-09-23 19:31:00','atmospheric rocket','success',835,'Smoke 089','','','','Aeronomy',1821,'',18 from dual union all
select 11912,'1965-A136','1965-09-25 00:00:00','military missile','destroyed before launch',1449,'[Spurious]','8K84 LKI','','','Test',596,'',0 from dual union all
select 11913,'1965-A137','1965-09-29 02:00:00','suborbital rocket','success',759,'MT-135-010','','','','Aeron',839,'',48 from dual union all
select 11914,'1965-A138','1965-10-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 11915,'1965-A139','1965-09-30 17:43:55','suborbital spaceplane','success',1520,'1-60-99','IR Scan','','','Test',379,'NB-52 003',23 from dual union all
select 11916,'1965-A14','1965-01-26 00:00:00','suborbital spaceplane','failure',1520,'1- A-82','','','','Test',379,'NB-52',14 from dual union all
select 11917,'1965-A140','1965-10-05 17:52:00','atmospheric rocket','success',835,'Smoke 090','','','','Aeronomy',1821,'',14 from dual union all
select 11918,'1965-A141','1965-10-06 03:00:00','suborbital rocket','failure',837,'NASA 14.203CA','GCA Ionos-TMA 8','','','Aeronomy',428,'',2 from dual union all
select 11919,'1965-A142','1965-10-08 00:00:00','suborbital spaceplane','failure',1520,'1- A-100','','','','Test',379,'NB-52',14 from dual union all
select 1192,'1952-A133','1952-08-11 00:00:00','atmospheric rocket','unknown',576,'47','','','','Test',1895,'',10 from dual union all
select 11920,'1965-A143','1965-10-10 01:25:00','suborbital rocket','failure',624,'NASA 8.42UI','M/DTR 4','','','Ionos',1821,'',32 from dual union all
select 11921,'1965-A144','1965-10-12 17:43:13','suborbital spaceplane','success',1520,'3-50-74','Checkout','','','Test',379,'NB-52 008',28 from dual union all
select 11922,'1965-A145','1965-10-06 00:00:00','military missile','failure',916,'AO-29','P-95','','','OT',597,'',10 from dual union all
select 11923,'1965-A146','1965-10-15 00:00:00','suborbital rocket','success',557,'Pacemaker','Apollo ablator test','','','Reentry test',1821,'',23 from dual union all
select 11924,'1965-A147','1965-10-20 00:00:00','suborbital rocket','failure',1351,'','Flight test 1','','','Test',1415,'',5 from dual union all
select 11925,'1965-A148','1965-10-28 00:00:00','military missile','success',540,'5','HIBEX D-2','','','Test',1888,'',1 from dual union all
select 11926,'1965-A149','1965-10-29 18:23:00','atmospheric rocket','success',835,'Smoke 091','','','','Aeronomy',1821,'',13 from dual union all
select 11927,'1965-A150','1965-11-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 11928,'1965-A151','1965-11-01 00:00:00','military missile','failure',1035,'RT-1-1963','','','','Test',636,'',10 from dual union all
select 11929,'1965-A152','1965-11-02 00:00:00','suborbital spaceplane','failure',1520,'1- A-102','','','','Test',379,'NB-52',14 from dual union all
select 1193,'1952-A134','1952-08-12 00:00:00','military missile','unknown',339,'29R','Firestone','','','Test',1895,'',30 from dual union all
select 11930,'1965-A153','1965-11-02 15:31:00','atmospheric rocket','success',835,'Smoke 092','','','','Aeronomy',1821,'',12 from dual union all
select 11931,'1965-A154','1965-11-03 00:00:00','military missile','failure',1449,'','8K84 LKI-16','','','Test',508,'',0 from dual union all
select 11932,'1965-A155','1965-11-03 17:09:10','suborbital spaceplane','success',1520,'2-43-75','External Tank Test','','','Test',379,'NB-52 003',21 from dual union all
select 11933,'1965-A156','1965-11-04 07:25:00','suborbital rocket','success',1069,'DS-160-1','Despin Test','','','Test',839,'',35 from dual union all
select 11934,'1965-A157','1965-11-04 17:11:31','suborbital spaceplane','success',1520,'1-62-103','Checkout','','','Test',379,'NB-52 008',24 from dual union all
select 11935,'1965-A158','1965-11-05 00:00:00','military missile','failure',1449,'','8K84 LKI-17','','','Test',505,'',0 from dual union all
select 11936,'1965-A159','1965-11-10 06:30:00','suborbital rocket','failure',837,'NASA 14.169UM','UM Pitot 19','','','Aeronomy',428,'',2 from dual union all
select 11937,'1965-A16','1965-02-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 11938,'1965-A160','1965-11-17 00:00:00','military missile','success',1252,'FLA-1','','','','Test',1888,'',3 from dual union all
select 11939,'1965-A161','1965-11-16 02:09:00','suborbital rocket','success',1107,'ST-I F1','','','','Test',990,'',20 from dual union all
select 1194,'1952-A135','1952-08-15 00:00:00','atmospheric rocket','success',836,'1249-113','Nike 1249 R&D 106R','','','Test',1895,'',10 from dual union all
select 11940,'1965-A162','1965-11-17 06:34:00','suborbital rocket','success',826,'NAL-16TR','','','','Test',990,'',45 from dual union all
select 11941,'1965-A163','1965-11-17 20:42:00','atmospheric rocket','success',835,'Smoke 093','','','','Aeronomy',1821,'',15 from dual union all
select 11942,'1965-A164','1965-11-18 18:18:00','atmospheric rocket','success',835,'Smoke 094','','','','Aeronomy',1821,'',14 from dual union all
select 11943,'1965-A165','1965-11-22 00:00:00','military missile','failure',135,'B007','Ionizing pellets','ABRES N-3 (USN)','','Reentry Test',614,'',0 from dual union all
select 11944,'1965-A166','1965-11-22 22:15:00','suborbital rocket','failure',864,'Sandia 152-83','LRL BOX-3','','','XR Astron',855,'',10 from dual union all
select 11945,'1965-A167','1965-11-23 19:24:00','atmospheric rocket','success',835,'Smoke 095','','','','Aeronomy',1821,'',12 from dual union all
select 11946,'1965-A168','1965-12-02 00:00:00','military missile','failure',540,'6','HIBEX D-3','','','Test',1888,'',20 from dual union all
select 11947,'1965-A169','1965-12-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 11948,'1965-A17','1965-02-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 11949,'1965-A170','1965-11-30 00:00:00','military missile','failure',1449,'','8K84 LKI-18','','','Test',503,'',0 from dual union all
select 1195,'1952-A136','1952-08-15 00:00:00','atmospheric rocket','success',836,'1249B-1004','Nike 1249 R&D 107R','','','Test',1895,'',10 from dual union all
select 11950,'1965-A171','1965-11-30 16:10:00','atmospheric rocket','success',835,'Smoke 096','','','','Aeronomy',1821,'',14 from dual union all
select 11951,'1965-A172','1965-12-07 18:12:00','atmospheric rocket','success',835,'Smoke 097','','','','Aeronomy',1821,'',16 from dual union all
select 11952,'1965-A173','1965-12-19 00:00:00','military missile','failure',1449,'','8K84 LKI-20','','','Test',508,'',0 from dual union all
select 11953,'1965-A174','1965-12-30 00:00:00','military missile','failure',1449,'','8K84 LKI-22','Salvo','','Test',503,'',0 from dual union all
select 11954,'1965-A175','1964-09-23 00:00:00','military missile','success',1256,'4','Squirt MTV-4','','','Test',1883,'',3 from dual union all
select 11955,'1965-A176','1965-03-26 00:00:00','military missile','success',1252,'PTV-1','','','','Test',1883,'',3 from dual union all
select 11956,'1965-A177','1965-01-01 00:00:00','suborbital rocket','failure',839,'Sandia 276-1','Dart/Parachute','','','Parachute test',1490,'',2 from dual union all
select 11957,'1965-A178','1965-11-15 16:00:00','suborbital rocket','success',770,'154','3B FOAM','','','Test',185,'',11 from dual union all
select 11958,'1965-A179','1965-11-15 20:03:00','suborbital rocket','failure',770,'155','3B GOOSE','','','Test',185,'',10 from dual union all
select 11959,'1965-A18','1965-02-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 1196,'1952-A137','1952-08-19 00:00:00','atmospheric rocket','success',836,'1249-115','Nike 1249 R&D 108R','','','Test',1895,'',10 from dual union all
select 11960,'1965-A180','1965-11-17 18:31:00','suborbital rocket','success',770,'158','3B HAHA','','','Test',185,'',12 from dual union all
select 11961,'1965-A181','1965-11-18 05:58:00','suborbital rocket','failure',767,'165','2C2 HASTINGS','','','Aeron TMA',185,'',32 from dual union all
select 11962,'1965-A182','1965-11-23 00:55:00','suborbital rocket','failure',767,'175','2C2 OLDBURY','','','Aeron TMA',185,'',31 from dual union all
select 11963,'1965-A183','1965-02-06 00:00:00','suborbital rocket','success',506,'','','','','Meteo',310,'',35 from dual union all
select 11964,'1965-A184','1965-02-08 00:00:00','suborbital rocket','success',506,'','','','','Meteo',310,'',35 from dual union all
select 11965,'1965-A185','1965-02-06 22:00:00','suborbital rocket','success',506,'16','Matienzo Ionos XR','','','Aeron/Ionos',949,'',35 from dual union all
select 11966,'1965-A186','1965-02-08 22:03:00','suborbital rocket','success',506,'15','Matienzo Ionos XR','','','Aeron/Ionos',949,'',35 from dual union all
select 11967,'1965-A187','1965-05-20 00:00:00','suborbital rocket','failure',494,'FAS-1','AFFT-3/Arcasonde 1A','','','Test',1415,'',21 from dual union all
select 11968,'1965-A188','1965-12-21 00:00:00','suborbital rocket','success',494,'FAS-2','AFFT-4/Arcasonde 1A','','','Test',1415,'',40 from dual union all
select 11969,'1965-A189','1965-01-20 00:00:00','suborbital rocket','failure',494,'FAD-2','AFFT-2','','','Test',1057,'',35 from dual union all
select 1197,'1952-A138','1952-08-20 00:00:00','atmospheric rocket','unknown',576,'48','','','','Test',1895,'',10 from dual union all
select 11970,'1965-A190','1965-12-01 01:28:20','military missile','success',1325,'','DAT-371 Intercept','','','Intercept',1415,'SHIP-US',8 from dual union all
select 11971,'1965-A191','1965-12-01 01:28:00','military missile','success',1053,'CC-2036','DAT-371 Target','','','Target',1415,'',8 from dual union all
select 11972,'1965-A20','1965-02-02 06:30:00','military missile','failure',135,'D003','ABRES P-1 (USAF)','ABRES P-1 (USAF)','','Reentry Test',613,'',50 from dual union all
select 11973,'1965-A209','1956-08-01 00:00:00','military missile','success',845,'1810-32','','','','Test',1895,'',10 from dual union all
select 11974,'1965-A21','1965-02-02 19:53:00','atmospheric rocket','success',835,'Smoke 076','','','','Aeronomy',1821,'',11 from dual union all
select 11975,'1965-A210','1956-08-01 00:00:00','military missile','success',845,'2032','','','','Test',1895,'',10 from dual union all
select 11976,'1965-A211','1956-08-16 00:00:00','military missile','success',845,'1810-21','','','','Test',1895,'',10 from dual union all
select 11977,'1965-A212','1956-09-26 00:00:00','military missile','success',845,'1810-41','','','','Test',1895,'',10 from dual union all
select 11978,'1965-A213','1956-09-26 00:00:00','military missile','success',845,'1810-43','','','','Test',1895,'',10 from dual;
insert into LAUNCH
select 11979,'1965-A214','1956-10-03 00:00:00','military missile','success',845,'1810-34','','','','Test',1895,'',10 from dual union all
select 1198,'1952-A139','1952-08-20 00:00:00','atmospheric rocket','unknown',576,'49','','','','Test',1895,'',10 from dual union all
select 11980,'1965-A215','1956-10-11 00:00:00','military missile','success',845,'1810-42','','','','Test',1895,'',10 from dual union all
select 11981,'1965-A216','1956-11-07 00:00:00','military missile','success',845,'1810-47','','','','Test',1895,'',10 from dual union all
select 11982,'1965-A217','1956-01-01 00:00:00','military missile','success',845,'','[B45]','','','Test',1895,'',10 from dual union all
select 11983,'1965-A218','1956-01-01 00:00:00','military missile','success',845,'','[B46]','','','Test',1895,'',10 from dual union all
select 11984,'1965-A219','1965-01-01 00:00:00','atmospheric rocket','success',536,'3','Test','','','Test',1849,'',45 from dual union all
select 11985,'1965-A22','1965-02-02 20:50:14','suborbital spaceplane','success',1520,'3-40-63','MA-25S TPS,MH-96,BLN','','','Test/Tech',379,'NB-52 008',29 from dual union all
select 11986,'1965-A220','1965-01-01 00:00:00','atmospheric rocket','success',536,'4','Test','','','Test',1849,'',45 from dual union all
select 11987,'1965-A221','1965-06-09 00:00:00','military missile','failure',1252,'PTV-2','','','','Test',1883,'',3 from dual union all
select 11988,'1965-A222','1965-08-20 16:42:00','suborbital rocket','success',88,'','BioSpace','','','Bio',1821,'',40 from dual union all
select 11989,'1965-A223','1965-10-21 15:00:00','military missile','failure',916,'','P-97','','','OT',597,'',40 from dual union all
select 1199,'1952-A14','1952-01-28 00:00:00','suborbital rocket','success',1495,'P6-2','','','','Test',218,'',1 from dual union all
select 11990,'1965-A224','1965-10-21 00:00:00','military missile','unknown',339,'','Anti-SLBM Target','','','Test',1415,'',40 from dual union all
select 11991,'1965-A225','1965-11-01 00:00:00','military missile','unknown',339,'','Anti-SLBM Target','','','Test',1415,'',40 from dual union all
select 11992,'1965-A226','1965-11-01 00:00:00','military missile','unknown',339,'','Anti-SLBM Target','','','Test',1415,'',40 from dual union all
select 11993,'1965-A227','1965-05-10 00:00:00','suborbital rocket','failure',1181,'','','Boosted Rooster','','Test',1415,'',10 from dual union all
select 11994,'1965-A228','1965-05-10 00:00:00','suborbital rocket','failure',1181,'','','Boosted Rooster','','Test',1415,'',10 from dual union all
select 11995,'1965-A229','1965-05-10 00:00:00','suborbital rocket','failure',1181,'','','Boosted Rooster','','Test',1415,'',10 from dual union all
select 11996,'1965-A23','1965-02-05 20:29:00','atmospheric rocket','success',835,'Smoke 077','','','','Aeronomy',1821,'',17 from dual union all
select 11997,'1965-A230','1965-11-22 00:00:00','suborbital rocket','failure',867,'Sandia','AEC SKOL','','','Aeron',855,'',2 from dual union all
select 11998,'1965-A231','1965-11-22 00:00:00','suborbital rocket','failure',867,'Sandia','AEC SKOL','','','Aeron',855,'',2 from dual union all
select 11999,'1965-A232','1965-11-20 13:30:00','suborbital rocket','failure',545,'','','','','Ionos',394,'',1 from dual union all
select 12,'1943-A05','1943-03-03 00:00:00','military missile','failure',8,'16','','','','Test',675,'',0 from dual union all
select 120,'1944-S109','1944-09-17 10:36:00','military missile','success',8,'18752','Ma162','','','Training',661,'',90 from dual union all
select 1200,'1952-A140','1952-08-21 00:00:00','atmospheric rocket','success',836,'1249-110','Nike 1249 R&D 109R','','','Test',1895,'',10 from dual union all
select 12000,'1965-A233','1965-08-25 15:47:00','suborbital rocket','success',88,'','BioSpace','','','Bio',1821,'',40 from dual union all
select 12001,'1965-A234','1964-10-22 00:00:00','suborbital rocket','failure',535,'HAD 143','','','','Aeron',219,'',1 from dual union all
select 12002,'1965-A235','1965-03-15 00:00:00','military missile','success',702,'XMGM-52A FT-1','','','','Test',1895,'',10 from dual union all
select 12003,'1965-A24','1965-02-06 00:00:00','suborbital rocket','success',506,'','','','','Meteo',310,'',35 from dual union all
select 12004,'1965-A25','1965-02-06 14:30:00','suborbital rocket','success',506,'14','Matienzo Ionos XR','','','Aeron/Ionos',949,'',35 from dual union all
select 12005,'1965-A26','1965-02-12 00:00:00','military missile','failure',135,'D002','ABRES P-2 (USAF)','ABRES P-2 (USAF)','','Reentry Test',614,'',0 from dual union all
select 12006,'1965-A28','1965-02-15 00:00:00','suborbital spaceplane','success',1520,'2- C-61','','','','Test',379,'NB-52',14 from dual union all
select 12007,'1965-A29','1965-02-15 00:00:00','suborbital spaceplane','success',1520,'2- C-62','','','','Test',379,'NB-52',14 from dual union all
select 12008,'1965-A30','1965-02-16 05:51:00','military missile','failure',135,'D008','ABRES P-4 (USAF)','ABRES P-4 (USAF)','','Reentry Test',612,'',10 from dual union all
select 12009,'1965-A31','1965-02-17 18:44:59','suborbital spaceplane','success',1520,'2-36-63','Stability','','','Test',379,'NB-52 008',29 from dual union all
select 1201,'1952-A141','1952-08-21 06:25:00','suborbital rocket','failure',370,'','SUI 1','','','Test',168,'WYG-279',11 from dual union all
select 12010,'1965-A32','1965-02-19 00:00:00','suborbital spaceplane','failure',1520,'1- A-83','','','','Test',379,'NB-52',14 from dual union all
select 12011,'1965-A33','1965-02-20 06:30:02','military missile','failure',983,'KY-2','','','','Test',636,'',0 from dual union all
select 12012,'1965-A34','1965-02-25 15:44:00','military missile','success',540,'1','HIBEX A-1','','','Test',1895,'',88 from dual union all
select 12013,'1965-A35','1965-02-25 00:00:00','suborbital spaceplane','failure',1520,'1- A-84','','','','Test',379,'NB-52',14 from dual union all
select 12014,'1965-A36','1965-02-26 19:45:55','suborbital spaceplane','success',1520,'1-52-85','MIT/AirD/Sky Bright','','','Aeron',379,'NB-52 008',46 from dual union all
select 12015,'1965-A37','1965-03-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 12016,'1965-A38','1965-02-28 00:30:00','suborbital rocket','success',837,'AE5.493','Ionos winds,T','','','Aeron',394,'',46 from dual union all
select 12017,'1965-A39','1965-02-28 03:30:00','suborbital rocket','failure',837,'NASA 14.198GA','GCA Ionos-TMA 2','','','Aeronomy',428,'',5 from dual union all
select 12018,'1965-A40','1965-02-28 06:00:00','suborbital rocket','failure',837,'NASA 14.199GA','GCA Ionos-TMA 3','','','Aeronomy',428,'',7 from dual union all
select 12019,'1965-A41','1965-02-28 11:30:00','suborbital rocket','failure',837,'AE5.554','TINA    (TMA/Gren)','','','Ionos',394,'',28 from dual union all
select 1202,'1952-A142','1952-08-24 03:34:00','suborbital rocket','failure',370,'','SUI 2','','','Test',169,'WYG-279',11 from dual union all
select 12020,'1965-A42','1965-02-28 13:15:00','suborbital rocket','failure',837,'AE5.555','Neutral/Ion/ImpProbe','','','Ionos',394,'',28 from dual union all
select 12021,'1965-A43','1965-03-01 19:00:00','military missile','success',795,'','Mk 11 RV','LONG LIFE','LONG LIFE','Test',397,'',1 from dual union all
select 12022,'1965-A44','1965-03-03 22:28:00','suborbital rocket','failure',837,'Ferdinand 9','Ferdinand 9','','','Aeronomy',29,'',22 from dual union all
select 12023,'1965-A45','1965-03-11 18:11:00','atmospheric rocket','success',835,'Smoke 078','','','','Aeronomy',1821,'',16 from dual union all
select 12024,'1965-A46','1965-03-15 20:41:00','suborbital rocket','failure',837,'Ferdinand 10','Ferdinand 10','','','Ionos/Auroral',29,'',35 from dual union all
select 12025,'1965-A47','1965-03-17 18:08:00','suborbital rocket','failure',1199,'SL140','ROE-1/SHF-3/UCL I-20','','','UV Astronomy',1839,'',19 from dual union all
select 12026,'1965-A48','1965-03-22 22:19:00','suborbital rocket','failure',837,'Ferdinand 11','Ferdinand 11','','','Aeronomy',29,'',20 from dual union all
select 12027,'1965-A49','1965-03-26 19:01:59','suborbital spaceplane','success',1520,'1-53-86','IR Scanner/IFDS','','','Tech/Test',379,'NB-52 008',31 from dual union all
select 12028,'1965-A52','1965-04-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 12029,'1965-A53','1965-04-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 1203,'1952-A143','1952-08-26 18:53:00','suborbital rocket','failure',51,'USAF 29','AF/Utah Ionos 2','','','Ionos',640,'',32 from dual union all
select 12030,'1965-A54','1965-03-31 14:34:00','suborbital rocket','failure',1198,'S03/1','ESRO S03/1','','','Ionospheric',1343,'',45 from dual union all
select 12031,'1965-A55','1965-03-31 18:31:00','atmospheric rocket','success',835,'Smoke 079','','','','Aeronomy',1821,'',12 from dual union all
select 12032,'1965-A56','1965-04-14 06:09:19','military missile','failure',928,'C2X-1','STAFF 1 (USAF)','','','Test',251,'',1 from dual union all
select 12033,'1965-A57','1965-04-23 17:44:16','suborbital spaceplane','success',1520,'3-41-64','HT/Abl TPS/BLN','','','Test/Tech',379,'NB-52 008',24 from dual union all
select 12034,'1965-A58','1965-04-28 12:58:00','military missile','failure',982,'U 22502-20T','8K64U UBP-54 K','','','OT',566,'',0 from dual union all
select 12035,'1965-A59','1965-04-28 20:26:20','suborbital spaceplane','success',1520,'2-37-64','Landing loads,Stab','','','Test',379,'NB-52 008',28 from dual union all
select 12036,'1965-A60','1965-05-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 12037,'1965-A61','1965-04-30 18:37:00','suborbital rocket','success',839,'NASA 10.95IA','Rehbar 8 (Grenades)','','','Aeronomy',1419,'',40 from dual union all
select 12038,'1965-A62','1965-04-30 19:53:00','atmospheric rocket','success',835,'Smoke 080','','','','Aeronomy',1821,'',23 from dual union all
select 12039,'1965-A63','1965-05-01 00:00:00','military missile','success',540,'2','HIBEX B-1','','','Test',1895,'',20 from dual union all
select 1204,'1952-A144','1952-08-27 00:00:00','military missile','unknown',339,'30R','Firestone','','','Test',1895,'',30 from dual union all
select 12040,'1965-A64','1965-05-04 07:30:00','suborbital rocket','failure',30,'NF3.162R','','','','IR Astron',1869,'',17 from dual union all
select 12041,'1965-A65','1965-05-11 00:00:00','suborbital spaceplane','failure',1520,'1- A-87','','','','Test',379,'NB-52',14 from dual union all
select 12042,'1965-A66','1965-05-11 07:30:00','suborbital rocket','success',535,'HAD 163','Falling sphere','','','Aeron',220,'',44 from dual union all
select 12043,'1965-A67','1965-05-12 00:00:00','atmospheric rocket','success',1066,'Ly4','Lynx 4','','','Test',2,'',20 from dual union all
select 12044,'1965-A68','1965-05-13 00:00:00','suborbital spaceplane','failure',1520,'2- A-65','','','','Test',379,'NB-52',14 from dual union all
select 12045,'1965-A69','1965-05-13 18:13:00','atmospheric rocket','success',835,'Smoke 081','','','','Aeronomy',1821,'',11 from dual union all
select 12046,'1965-A70','1965-05-14 15:41:00','atmospheric rocket','success',835,'Smoke 082','','','','Aeronomy',1821,'',15 from dual union all
select 12047,'1965-A71','1965-05-18 17:56:38','suborbital spaceplane','success',1520,'2-38-66','Stab, landing, ST','','','Test',379,'NB-52 008',31 from dual union all
select 12048,'1965-A72','1965-05-18 18:30:00','atmospheric rocket','success',835,'Smoke 083','','','','Aeronomy',1821,'',14 from dual union all
select 12049,'1965-A73','1965-05-19 13:01:04','test rocket','failure',711,'LJII-4 (12-51-2)','A-003','','','SC Test',1873,'',6 from dual union all
select 1205,'1952-A145','1952-08-29 00:00:00','military missile','success',763,'Dyn Model #2','','','','Test',222,'',10 from dual union all
select 12050,'1965-A74','1965-05-20 03:45:00','military missile','failure',1449,'','8K84 LKI-2','','','Test',502,'',0 from dual union all
select 12051,'1965-A76','1965-05-27 23:56:00','suborbital rocket','failure',215,'BB-2A-51','','','','Ionos',428,'',27 from dual union all
select 12052,'1965-A77','1965-06-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 12053,'1965-A78','1965-06-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 12054,'1965-A79','1965-06-01 18:44:00','atmospheric rocket','success',835,'Smoke 084','','','','Aeronomy',1821,'',13 from dual union all
select 12055,'1965-A80','1965-06-04 00:00:00','suborbital spaceplane','failure',1520,'2- A-67','','','','Test',379,'NB-52',14 from dual union all
select 12056,'1965-A81','1965-06-08 00:00:00','suborbital spaceplane','failure',1520,'2- A-68','','','','Test',379,'NB-52',14 from dual union all
select 12057,'1965-A82','1965-06-11 00:00:00','suborbital spaceplane','failure',1520,'2- A-69','','','','Test',379,'NB-52',14 from dual union all
select 12058,'1965-A83','1965-06-12 12:24:59','suborbital rocket','success',725,'','Prob. SHOK','','','Meteo',1205,'SHOK',48 from dual union all
select 12059,'1965-A84','1965-06-16 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 1206,'1952-A146','1952-09-01 00:00:00','atmospheric rocket','success',367,'','Hermes Ramjet Model','','','Test',1821,'',10 from dual union all
select 12060,'1965-A85','1965-06-17 17:40:31','suborbital spaceplane','success',1520,'1-55-89','IR Scanner, INS','','','Test',379,'NB-52 008',33 from dual union all
select 12061,'1965-A86','1965-06-18 07:43:00','suborbital rocket','success',826,'HM-16-IT','','','','Test',990,'',38 from dual union all
select 12062,'1965-A87','1965-06-19 00:00:00','military missile','failure',1449,'','8K84 LKI-3','','','Test',501,'',0 from dual union all
select 12063,'1965-A88','1965-06-22 17:44:43','suborbital spaceplane','success',1520,'2-39-70','Stability,ST,gear','','','Test',379,'NB-52 008',47 from dual union all
select 12064,'1965-A89','1965-06-29 05:11:00','military missile','failure',982,'T 22601-14T','8K64U UBP-57','','','OT',571,'',0 from dual union all
select 12065,'1965-A90','1965-06-29 13:00:01','test rocket','success',87,'PA-2','','','','Test',1873,'',2 from dual union all
select 12066,'1965-A91','1965-07-14 00:00:00','military missile','failure',540,'3','HIBEX B-2','','','Test',1895,'',1 from dual union all
select 12067,'1965-A92','1965-07-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual;
insert into LAUNCH
select 12068,'1965-A93','1965-07-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 12069,'1965-A94','1965-07-01 17:40:00','atmospheric rocket','success',835,'Smoke 085','','','','Aeronomy',1821,'',19 from dual union all
select 1207,'1952-A147','1952-09-05 00:00:00','atmospheric rocket','success',365,'','Rigel Model','','','Test',1821,'',10 from dual union all
select 12070,'1965-A95','1965-07-02 00:00:00','suborbital spaceplane','failure',1520,'2- A-71','','','','Test',379,'NB-52',14 from dual union all
select 12071,'1965-A96','1965-07-08 00:00:00','suborbital rocket','failure',1249,'','','','','Test',1057,'F-4B',27 from dual union all
select 12072,'1965-A97','1965-07-10 00:00:00','military missile','success',1449,'','8K84 LKI-6','','','Test',501,'',1000 from dual union all
select 12073,'1965-A98','1965-07-13 00:00:00','suborbital rocket','failure',1186,'EX-1.8','','','','Aeron',1895,'',1 from dual union all
select 12074,'1965-A99','1965-07-13 00:00:00','suborbital spaceplane','failure',1520,'3- A-68','','','','Test',379,'NB-52',14 from dual union all
select 12075,'1965-F01','1965-01-21 21:34:54','orbital','failure',154,'172D','ABRES MTV-1/OV1-1','OV1-1','BEAVERS DAM','Sat',1667,'',560 from dual union all
select 12076,'1965-F02','1965-02-12 12:00:00','orbital','failure',681,'','','DS-P1-Yu No. 2','','Sat',626,'',40 from dual union all
select 12077,'1965-F03','1965-02-20 06:30:00','orbital','failure',681,'','','DS-A1 No. 6','','Sat',626,'',15 from dual union all
select 12078,'1965-F04','1965-03-02 13:25:00','orbital','failure',152,'AC-5','SD-1','Surveyor SD-1','','Sat',268,'',0 from dual union all
select 12079,'1965-F05','1965-04-10 08:39:01','orbital','failure',802,'U 15000- 22/R10326','','E-6 No. 8','','Sat',480,'',200 from dual union all
select 1208,'1952-A148','1952-09-05 00:00:00','atmospheric rocket','success',564,'','Rascal Model','','','Test',1821,'',5 from dual union all
select 12080,'1965-F06','1965-05-28 02:54:56','orbital','failure',154,'68D','OV1-3/Mk 3 RV','OV1-3','TENNIS MATCH','Sat',1685,'',150 from dual union all
select 12081,'1965-F07','1965-07-12 19:00:00','orbital','failure',177,'7112','AFP-206','GAMBIT SV 970','WHITE PINE','Sat',1773,'',170 from dual union all
select 12082,'1965-F08','1965-07-13 00:00:00','orbital','failure',1509,'R 15002- 05','','Zenit-2 No. 28','','Sat',562,'',100 from dual union all
select 12083,'1965-F09','1965-08-25 15:17:00','orbital','failure',1378,'434/D33','OSO C','OSO C','','Sat',234,'',562 from dual union all
select 12084,'1965-F10','1965-09-02 20:00:16','orbital','failure',1392,'401/FTV 1602','CORONA 99/R&D','CORONA R&D FTV 1602','WORD SCRAMBLE','Sat TAG117',1679,'',11 from dual union all
select 12085,'1965-F11','1965-10-25 15:00:00','orbital','failure',177,'5301','GATV 5002','TDA-2','','Sat',229,'',294 from dual union all
select 12086,'1965-F12','1965-12-28 08:10:00','orbital','failure',681,'','','DS-K-40 No. 1','','Sat',626,'',10 from dual union all
select 12087,'1965-M01','1965-02-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 12088,'1965-M02','1965-02-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 12089,'1965-M03','1965-02-23 14:36:00','test rocket','success',1372,'248','ASSET 6','','','Reentry Test',234,'',70 from dual union all
select 1209,'1952-A149','1952-09-10 00:00:00','military missile','failure',255,'621-1','','','','Test',272,'',0 from dual union all
select 12090,'1965-M04','1965-03-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 12091,'1965-M05','1965-03-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 12092,'1965-M06','1965-02-28 06:39:00','suborbital rocket','success',837,'AE5.495','','','','Aeron',394,'',61 from dual union all
select 12093,'1965-M07','1965-02-28 12:11:00','suborbital rocket','failure',837,'NASA 14.200CA','GCA Ionos-TMA 4','','','Aeronomy',428,'',55 from dual union all
select 12094,'1965-M08','1965-03-01 00:00:00','suborbital rocket','success',1431,'Sandia 152-40','Magnus','','','Test/Aeron?',855,'',73 from dual union all
select 12095,'1965-M09','1965-04-20 22:04:00','suborbital rocket','success',839,'AD6.841','Antenna breakdown','','','Test',1821,'',80 from dual union all
select 12096,'1965-M10','1965-05-25 18:12:07','suborbital spaceplane','success',1520,'1-54-88','IFDS/MIT/RAS Mod','','','Tech/Test',379,'NB-52 008',54 from dual union all
select 12097,'1965-M11','1965-05-27 00:00:00','suborbital rocket','failure',1491,'VA79','FU-144 Ly alpha','','','Solar UV?',668,'',70 from dual union all
select 12098,'1965-M12','1965-05-28 17:43:51','suborbital spaceplane','success',1520,'3-42-65','Radiometer/Scan/BLN','','','Test/Aeron',379,'NB-52 008',63 from dual union all
select 12099,'1965-M13','1965-06-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 121,'1944-S11','1944-03-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1210,'1952-A15','1952-01-28 00:00:00','suborbital rocket','success',1496,'R6','','','','Test',218,'',1 from dual union all
select 12100,'1965-M14','1965-06-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 12101,'1965-M15','1965-06-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 12102,'1965-M16','1965-06-01 00:00:00','suborbital rocket','success',1290,'','Life Sci (mice)','','','Biology',1364,'',70 from dual union all
select 12103,'1965-M17','1965-06-05 00:00:00','suborbital rocket','success',1290,'','Life Sci (mice)','','','Biology',1364,'',70 from dual union all
select 12104,'1965-M18','1965-06-15 02:30:00','suborbital rocket','success',1076,'S-B4','','','','Test',990,'',60 from dual union all
select 12105,'1965-M19','1965-06-16 04:00:00','suborbital rocket','success',1076,'S-B5','','','','Test',990,'',60 from dual union all
select 12106,'1965-M20','1965-06-16 18:26:33','suborbital spaceplane','success',1520,'3-43-66','Radiometer/BLN','','','Test',379,'NB-52 003',74 from dual union all
select 12107,'1965-M21','1965-06-17 07:30:00','suborbital rocket','success',826,'HM-16-D','','','','Test',990,'',52 from dual union all
select 12108,'1965-M22','1965-07-08 17:16:55','suborbital spaceplane','success',1520,'2-40-72','ST,Landing,Alt','','','Test',379,'NB-52 003',64 from dual union all
select 12109,'1965-M23','1965-07-10 00:00:00','military missile','failure',1117,'P2','','','','Reentry Test',669,'',50 from dual union all
select 1211,'1952-A150','1952-09-10 00:00:00','atmospheric rocket','unknown',576,'51','','','','Test',1895,'',10 from dual union all
select 12110,'1965-M24','1965-08-03 20:40:05','suborbital spaceplane','success',1520,'2-41-73','RAS,ST,Landing','','','Test',379,'NB-52 008',63 from dual union all
select 12111,'1965-M25','1965-08-25 17:54:46','suborbital spaceplane','success',1520,'1-57-96','MIT Scan,Stability','','','Test',379,'NB-52 003',65 from dual union all
select 12112,'1965-M26','1965-08-26 17:52:12','suborbital spaceplane','success',1520,'3-47-71','Northrop Radiom,BLN','','','Test/Aeron?',379,'NB-52 008',73 from dual union all
select 12113,'1965-M27','1965-09-02 17:40:05','suborbital spaceplane','success',1520,'2-42-74','RAS,ST,Landing','','','Test',379,'NB-52 008',73 from dual union all
select 12114,'1965-M28','1965-09-14 18:01:06','suborbital spaceplane','success',1520,'3-48-72','N.Radiom,BLN','','','Test/Aeron?',379,'NB-52 008',72 from dual union all
select 12115,'1965-M34','1965-11-18 00:38:00','suborbital rocket','failure',767,'162','2C2 ENTERPRISE','','','Aeron TMA',185,'',71 from dual union all
select 12116,'1965-M35','1965-11-19 17:30:00','suborbital rocket','success',767,'168','2C3 KENDALL','','','Aeron Chaff',185,'',72 from dual union all
select 12117,'1965-M36','1965-11-20 20:47:00','suborbital rocket','failure',767,'171','2C3 LANCASTER','','','Aeron Chaff',185,'',58 from dual union all
select 12118,'1965-M39','1965-10-23 00:00:00','military missile','success',1104,'2601','Essai 1','','','Test',669,'',50 from dual union all
select 12119,'1965-M40','1965-10-27 18:49:10','suborbital spaceplane','success',1520,'3-51-75','N.Scan/BLN','','','Test',379,'NB-52 003',72 from dual union all
select 1212,'1952-A151','1952-09-10 00:00:00','atmospheric rocket','unknown',576,'50','','','','Test',1895,'',10 from dual union all
select 12120,'1965-M41','1965-11-03 00:00:00','military missile','success',1104,'2602','Essai 2','','','Test',669,'',50 from dual union all
select 12121,'1965-M42','1965-11-18 06:42:00','suborbital rocket','success',1076,'SB-II F6','','','','Test',990,'',75 from dual union all
select 12122,'1965-M43','1965-11-19 01:44:00','suborbital rocket','success',1076,'SB-II F7','','','','Test',990,'',75 from dual union all
select 12123,'1965-M45','1965-06-01 00:00:00','suborbital rocket','success',706,'02/1','','','','Test',313,'',68 from dual union all
select 12124,'1965-M46','1965-06-01 00:00:00','suborbital rocket','success',706,'02/2','','','','Test',313,'',60 from dual union all
select 12125,'1965-M47','1965-06-01 00:00:00','suborbital rocket','success',706,'02/3','','','','Test',313,'',60 from dual union all
select 12126,'1965-M48','1965-02-02 00:00:00','suborbital rocket','success',520,'','BRL WS  5-1','','','Aeron',1850,'',61 from dual union all
select 12127,'1965-M49','1965-02-05 00:00:00','suborbital rocket','success',520,'','BRL WS  5-2','','','Aeron',1850,'',61 from dual union all
select 12128,'1965-M50','1965-02-05 00:00:00','suborbital rocket','success',520,'','BRL WS  5-3','','','Aeron',1850,'',61 from dual union all
select 12129,'1965-M51','1965-02-06 00:00:00','suborbital rocket','success',520,'','BRL WS  5-4','','','Aeron',1850,'',61 from dual union all
select 1213,'1952-A152','1952-09-10 00:00:00','military missile','success',771,'GM-568','','','','Test',222,'',12 from dual union all
select 12130,'1965-M52','1965-02-09 00:00:00','suborbital rocket','success',520,'','BRL WS  5-5','','','Aeron',1850,'',61 from dual union all
select 12131,'1965-M53','1965-02-09 00:00:00','suborbital rocket','success',520,'','BRL WS  5-6','','','Aeron',1850,'',61 from dual union all
select 12132,'1965-M54','1965-02-18 00:00:00','suborbital rocket','success',520,'','BRL WS  5-7','','','Aeron',1850,'',61 from dual union all
select 12133,'1965-M55','1965-02-23 00:00:00','suborbital rocket','success',520,'','BRL WS  5-8','','','Aeron',1850,'',61 from dual union all
select 12134,'1965-M56','1965-03-02 00:00:00','suborbital rocket','success',520,'','BRL WS  5-9','','','Aeron',1850,'',61 from dual union all
select 12135,'1965-M57','1965-03-09 00:00:00','suborbital rocket','success',520,'','BRL WS  5-10','','','Aeron',1850,'',61 from dual union all
select 12136,'1965-M58','1965-05-14 00:00:00','suborbital rocket','success',520,'','BRL WS  5-11','','','Aeron',1850,'',61 from dual union all
select 12137,'1965-M59','1965-05-20 00:00:00','suborbital rocket','success',520,'','BRL WS  5-12','','','Aeron',1850,'',61 from dual union all
select 12138,'1965-M60','1965-06-29 00:00:00','suborbital rocket','success',520,'','BRL WS  5-13','','','Aeron',1850,'',61 from dual union all
select 12139,'1965-M61','1965-07-22 00:00:00','suborbital rocket','success',520,'','BRL WS 5-14','','','Aeron',1850,'',61 from dual union all
select 1214,'1952-A153','1952-09-11 00:00:00','atmospheric rocket','success',442,'','B-58 Model','','','Test',1821,'',10 from dual union all
select 12140,'1965-M62','1965-08-12 00:00:00','suborbital rocket','success',520,'','BRL WS 5-15','','','Aeron',1850,'',61 from dual union all
select 12141,'1965-M63','1965-08-18 00:00:00','suborbital rocket','success',520,'','BRL WS 5-16','','','Aeron',1850,'',61 from dual union all
select 12142,'1965-M64','1965-08-23 00:00:00','suborbital rocket','success',520,'','BRL WS 5-17','','','Aeron',1850,'',61 from dual union all
select 12143,'1965-M65','1965-09-01 00:00:00','suborbital rocket','success',520,'','BRL WS 5-18','','','Aeron',1850,'',61 from dual union all
select 12144,'1965-M66','1965-09-02 00:00:00','suborbital rocket','success',520,'','BRL WS 5-19','','','Aeron',1850,'',61 from dual union all
select 12145,'1965-M67','1965-09-08 00:00:00','suborbital rocket','success',520,'','BRL WS 5-20','','','Aeron',1850,'',61 from dual union all
select 12146,'1965-M68','1965-09-08 00:00:00','suborbital rocket','success',520,'','BRL WS 5-21','','','Aeron',1850,'',61 from dual union all
select 12147,'1965-M69','1965-09-11 00:00:00','suborbital rocket','success',520,'','BRL WS 5-22','','','Aeron',1850,'',61 from dual union all
select 12148,'1965-M70','1965-09-14 00:00:00','suborbital rocket','success',520,'','BRL WS 5-23','','','Aeron',1850,'',61 from dual union all
select 12149,'1965-M71','1965-09-16 00:00:00','suborbital rocket','success',520,'','BRL WS 5-24','','','Aeron',1850,'',61 from dual union all
select 1215,'1952-A154','1952-09-11 00:00:00','atmospheric rocket','success',836,'1249-114','Nike 1249 R&D 110R','','','Test',1895,'',10 from dual union all
select 12150,'1965-M72','1965-09-28 00:00:00','suborbital rocket','success',520,'','BRL WS 5-25','','','Aeron',1850,'',61 from dual union all
select 12151,'1965-M73','1965-10-04 00:00:00','suborbital rocket','success',520,'','BRL WS 5-26','','','Aeron',1850,'',61 from dual union all
select 12152,'1965-M74','1965-10-05 00:00:00','suborbital rocket','success',520,'','BRL WS 5-27','','','Aeron',1850,'',61 from dual union all
select 12153,'1965-M75','1965-10-11 00:00:00','suborbital rocket','success',520,'','BRL WS 5-28','','','Aeron',1850,'',61 from dual union all
select 12154,'1965-M76','1965-10-13 00:00:00','suborbital rocket','success',520,'','BRL WS 5-29','','','Aeron',1850,'',61 from dual union all
select 12155,'1965-M77','1965-10-28 00:00:00','suborbital rocket','success',520,'','BRL WS 5-30','','','Aeron',1850,'',61 from dual union all
select 12156,'1965-M78','1965-10-28 00:00:00','suborbital rocket','success',520,'','BRL WS 5-31','','','Aeron',1850,'',61 from dual union all
select 12157,'1965-M79','1965-10-28 00:00:00','suborbital rocket','success',520,'','BRL WS 5-32','','','Aeron',1850,'',61 from dual;
insert into LAUNCH
select 12158,'1965-M80','1965-10-28 00:00:00','suborbital rocket','success',520,'','BRL WS 5-33','','','Aeron',1850,'',61 from dual union all
select 12159,'1965-M81','1965-11-10 00:00:00','suborbital rocket','success',520,'','BRL WS 5-34','','','Aeron',1850,'',61 from dual union all
select 1216,'1952-A155','1952-09-12 00:00:00','military missile','unknown',339,'31R','','','','Test',1895,'',30 from dual union all
select 12160,'1965-M82','1965-10-20 00:00:00','suborbital rocket','success',330,'','Shrike motor','','','Atmos',1057,'RB-57B',52 from dual union all
select 12161,'1965-M83','1965-12-20 00:00:00','suborbital rocket','success',330,'','','','','Atmos',803,'RB-57B',50 from dual union all
select 12162,'1965-M86','1965-06-16 00:00:00','military missile','success',1162,'','','RB350 Training','','OT',1447,'',50 from dual union all
select 12163,'1965-S01','1965-01-01 00:00:00','suborbital rocket','success',625,'','Test','','','Test',1895,'',80 from dual union all
select 12164,'1965-S05','1965-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',150 from dual union all
select 12165,'1965-S06','1965-01-01 00:00:00','military missile','success',837,'BMTS','','','','Aeronomy',1895,'',100 from dual union all
select 12166,'1965-S07','1965-01-01 00:00:00','military missile','success',837,'BMTS','','','','Aeronomy',1895,'',100 from dual union all
select 12167,'1965-S08','1965-01-01 00:00:00','military missile','success',979,'','8K65 UBP-57','','','Test',636,'',675 from dual union all
select 12168,'1965-S09','1965-01-01 00:00:00','military missile','success',1548,'','','','','Test',636,'',620 from dual union all
select 12169,'1965-S10','1965-09-24 02:49:00','military missile','success',1018,'U 15001-10','8K75 UBP-5','','','OT',583,'',1160 from dual union all
select 1217,'1952-A156','1952-09-16 00:00:00','military missile','success',771,'GM-569','','','','Test',222,'',12 from dual union all
select 12170,'1965-S100','1965-02-19 04:51:42','suborbital rocket','success',865,'Sandia 152-72','LASL Aurora','','','Auroral',428,'',320 from dual union all
select 12171,'1965-S101','1965-02-19 09:17:31','suborbital rocket','success',837,'NASA 14.95UA','UM Composition 5','','','Aeronomy',428,'',187 from dual union all
select 12172,'1965-S102','1965-02-23 17:15:00','suborbital rocket','success',850,'Sphere 14','Sphere','','','Aeronomy',394,'',200 from dual union all
select 12173,'1965-S103','1965-02-25 00:00:00','suborbital rocket','success',837,'','Ionos winds,T','','','Aeronomy',394,'',200 from dual union all
select 12174,'1965-S104','1965-02-26 03:00:00','military missile','success',982,'G 22503-15T','8K64U UBP-49 PBG','','','OT',565,'',1210 from dual union all
select 12175,'1965-S105','1965-02-26 11:10:00','suborbital rocket','success',837,'Sphere 24','Sphere','','','Aeronomy',394,'',150 from dual union all
select 12176,'1965-S106','1965-02-27 00:00:00','military missile','success',458,'','','','','Test',669,'',200 from dual union all
select 12177,'1965-S107','1965-02-27 11:11:16','military missile','success',154,'211D','ABRES LORV-4','','DRAG BAR','Reentry Test',1662,'',1800 from dual union all
select 12178,'1965-S108','1965-02-27 21:00:00','suborbital rocket','success',837,'AE5.556','Neutral/Ion/Sphere','','','Aeron',394,'',157 from dual union all
select 12179,'1965-S109','1965-02-27 21:33:00','suborbital rocket','success',839,'NASA 10.155UA','Density','','','Aeronomy',1065,'CVE-25',100 from dual union all
select 1218,'1952-A157','1952-09-16 00:00:00','atmospheric rocket','success',836,'1249-117','Nike 1249 R&D 111R','','','Test',1895,'',10 from dual union all
select 12180,'1965-S11','1965-09-28 09:00:00','military missile','success',1018,'U 15001-07','8K75 UBP-6 K','','','OT',582,'',1160 from dual union all
select 12181,'1965-S110','1965-02-27 22:30:00','suborbital rocket','success',837,'AE5.486','Neutral/Ion/Sphere','','','Ionos',394,'',132 from dual union all
select 12182,'1965-S111','1965-03-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 12183,'1965-S112','1965-03-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 12184,'1965-S113','1965-03-01 00:00:00','military missile','success',871,'ZK-52','Sim target','','','Interceptor',873,'',80 from dual union all
select 12185,'1965-S114','1965-03-01 00:00:00','military missile','success',871,'ZK-53','Sim target','','','Interceptor',873,'',80 from dual union all
select 12186,'1965-S115','1965-02-28 00:21:00','suborbital rocket','success',837,'NASA 14.196CA','GCA Ionos-TMA 1','','','Aeron/Ionos',428,'',170 from dual union all
select 12187,'1965-S116','1965-02-28 04:15:00','suborbital rocket','success',837,'AE5.494','PENNY   (TMA/Gren)','','','Aeron',394,'',145 from dual union all
select 12188,'1965-S117','1965-02-28 09:15:00','suborbital rocket','success',837,'AE5.496','SHIRLEY (TMA/Gren)','','','Aeron',394,'',200 from dual union all
select 12189,'1965-S118','1965-02-28 09:15:00','suborbital rocket','success',837,'AE5.553','Neutral/Ion/Sphere','','','Aeron',394,'',143 from dual union all
select 1219,'1952-A158','1952-09-17 00:00:00','atmospheric rocket','unknown',576,'52','','','','Test',1895,'',10 from dual union all
select 12190,'1965-S119','1965-02-28 11:30:00','suborbital rocket','success',837,'AE5.497','Neutral/Ion/ImpProbe','','','Aeron',394,'',139 from dual union all
select 12191,'1965-S12','1965-10-18 21:00:00','military missile','success',1018,'U 15001-03','8K75 UBP-7 K','','','OT',580,'',1160 from dual union all
select 12192,'1965-S120','1965-03-02 01:10:00','military missile','success',795,'1093','Mk 11 RV','OT','GOLDEN ROSE','OT',1703,'',900 from dual union all
select 12193,'1965-S121','1965-03-01 17:18:06','military missile','success',932,'A3P-172','','','','OT',421,'SSBN 630',1000 from dual union all
select 12194,'1965-S122','1965-03-01 20:43:00','suborbital rocket','success',837,'NASA 14.253IA','UJO-6','','','Aeronomy',29,'',185 from dual union all
select 12195,'1965-S123','1965-03-02 00:00:00','suborbital rocket','success',865,'Sandia 152-49','ACS','','','Tech?',855,'',270 from dual union all
select 12196,'1965-S124','1965-03-02 00:00:00','suborbital rocket','success',864,'Sandia 152-53','Ballistic','','','Aeron?',855,'',175 from dual union all
select 12197,'1965-S125','1965-03-02 09:52:31','military missile','success',154,'301D','Nike RMV-301','NTMP KX-30','PORK BARREL','Target',1664,'',1800 from dual union all
select 12198,'1965-S126','1965-03-03 00:00:00','suborbital rocket','success',1431,'Sandia 152-39','Darthawk 6.5"','','','Test/Aeron?',855,'',90 from dual union all
select 12199,'1965-S127','1965-03-03 00:09:00','suborbital rocket','success',30,'AE3.516','EUV Mon','','','Solar UV',1869,'',241 from dual union all
select 122,'1944-S110','1944-09-17 13:55:00','military missile','success',8,'18754','Ma170','','','Training',661,'',90 from dual union all
select 1220,'1952-A159','1952-09-17 00:00:00','military missile','success',703,'608','','','','Test',222,'',8 from dual union all
select 12200,'1965-S128','1965-03-03 04:42:00','suborbital rocket','success',1206,'SL87B','RAE Test 29','','','Test',1839,'',216 from dual union all
select 12201,'1965-S129','1965-03-03 14:15:00','suborbital rocket','unknown',30,'AE3.517','EUV Mon','','','Solar UV',1895,'',252 from dual union all
select 12202,'1965-S13','1965-12-23 05:30:00','military missile','success',1018,'U 15002-03','8K75 UBP-8 ZP','','','OT',580,'',1160 from dual union all
select 12203,'1965-S130','1965-03-04 04:28:00','military missile','success',1018,'G 15000-15','8K75 UBP-3','','','OT',584,'',1160 from dual union all
select 12204,'1965-S131','1965-03-04 00:00:00','suborbital rocket','success',288,'C85','SINTRA pointing','','','Solar?',664,'',120 from dual union all
select 12205,'1965-S132','1965-03-04 18:32:00','suborbital rocket','success',549,'Sandia 152-16','LRL Payload Test','','','Test',855,'',181 from dual union all
select 12206,'1965-S133','1965-03-05 01:12:59','military missile','success',930,'A2EA-193','','','','OT',244,'',1000 from dual union all
select 12207,'1965-S134','1965-03-05 00:00:00','military missile','success',1419,'SM-80','Nike TVX-30-3','ST/ABRES NT KX-97','WEST WIND II','OT',1655,'',1000 from dual union all
select 12208,'1965-S135','1965-03-05 00:11:00','suborbital rocket','success',30,'AE3.518','EUV Mon','','','Solar UV',1869,'',245 from dual union all
select 12209,'1965-S136','1965-03-05 11:00:21','military missile','success',983,'KY-2','','','','Test',636,'',250 from dual union all
select 1221,'1952-A16','1952-01-29 00:00:00','military missile','unknown',339,'15-13 R','','','','Test',1895,'',30 from dual union all
select 12210,'1965-S137','1965-03-05 20:57:50','suborbital rocket','success',837,'NASA 14.254IA','UJO-8','','','Aeronomy',29,'',180 from dual union all
select 12211,'1965-S138','1965-03-06 00:00:00','suborbital rocket','success',865,'Sandia 152-73','Aurora','','','Auroral',428,'',184 from dual union all
select 12212,'1965-S139','1965-03-06 07:00:00','military missile','success',1018,'T 15001-11','8K75 UBP-4 K','','','OT',584,'',1160 from dual union all
select 12213,'1965-S14','1965-01-01 00:00:00','military missile','success',1036,'','','','','Test',636,'',1000 from dual union all
select 12214,'1965-S140','1965-03-06 03:41:00','suborbital rocket','success',30,'NASA 04.133UG','Princeton UVS','','','UV Astron',1869,'',189 from dual union all
select 12215,'1965-S141','1965-03-06 07:32:00','suborbital rocket','success',30,'AD3.613','D-region comp/aurora','','','Aeron/Ionos',428,'',193 from dual union all
select 12216,'1965-S142','1965-03-08 00:00:00','military missile','success',795,'1132','Mk 11 RV','OT','DOCK BELL','OT',1702,'',900 from dual union all
select 12217,'1965-S143','1965-03-08 15:35:00','suborbital rocket','success',837,'NASA 14.160GE','B Field','','','Plasma',1076,'CVE-25',151 from dual union all
select 12218,'1965-S144','1965-03-08 17:48:08','suborbital rocket','success',837,'NASA 14.64UA','UM Pitot 8','','','Aeronomy',1076,'CVE-25',141 from dual union all
select 12219,'1965-S145','1965-03-09 06:26:26','suborbital rocket','success',837,'NASA 14.65UA','UM Pitot 9','','','Aeronomy',1087,'CVE-25',143 from dual union all
select 1222,'1952-A160','1952-09-19 00:00:00','atmospheric rocket','success',365,'','D109/HPAG seeker','','','Test',1821,'',10 from dual union all
select 12220,'1965-S146','1965-03-09 16:25:00','suborbital rocket','success',837,'NASA 14.85UE','UNH 65-2','','','Plasma',1098,'CVE-25',200 from dual union all
select 12221,'1965-S147','1965-03-10 06:00:00','military missile','success',982,'U 22501-20T','8K64U UBP-50','','','OT',566,'',1210 from dual union all
select 12222,'1965-S148','1965-03-10 16:00:00','suborbital rocket','success',837,'NASA 14.83UE','UNH 65-3','','','Plasma',1109,'CVE-25',200 from dual union all
select 12223,'1965-S149','1965-03-11 09:35:14','suborbital rocket','success',837,'NASA 14.98UA','Composition(Tropics)','','','Aeronomy',1120,'CVE-25',197 from dual union all
select 12224,'1965-S15','1966-11-01 00:00:00','military missile','success',1036,'1','LKI 1','','','Test',636,'',1000 from dual union all
select 12225,'1965-S150','1965-03-11 10:38:00','suborbital rocket','success',839,'NASA 10.156UA','','','','Aeronomy',1130,'CVE-25',100 from dual union all
select 12226,'1965-S151','1965-03-11 20:07:00','suborbital rocket','success',837,'NASA 14.99UA','','','','Aeronomy',1131,'CVE-25',200 from dual union all
select 12227,'1965-S152','1965-03-12 00:00:00','suborbital rocket','success',865,'Sandia 152-79','ACS','','','Aeron?',855,'',280 from dual union all
select 12228,'1965-S153','1965-03-12 13:30:00','suborbital rocket','success',837,'NASA 14.07UE','UNH 65-4','','','Plasma',1132,'CVE-25',178 from dual union all
select 12229,'1965-S154','1965-03-12 16:00:00','suborbital rocket','success',837,'NASA 14.84UE','UNH 65-5','','','Plasma',1132,'CVE-25',175 from dual union all
select 1223,'1952-A161','1952-09-19 15:49:00','suborbital rocket','failure',1470,'TF5','','','','Test/CR',1863,'',7 from dual union all
select 12230,'1965-S155','1965-03-12 23:21:52','military missile','success',154,'154D','MTV-2','ABRES MTV-2','ANGEL CAMP','Reentry Test',1667,'',1800 from dual union all
select 12231,'1965-S156','1965-03-13 03:38:00','suborbital rocket','success',30,'NASA 04.56GG','UV spectra','','','UV Astron',1869,'',155 from dual union all
select 12232,'1965-S157','1965-03-13 03:55:00','suborbital rocket','success',30,'AE3.516','[uncertain]','','','UV Astron??',1869,'',105 from dual union all
select 12233,'1965-S158','1965-03-13 07:42:00','suborbital rocket','success',30,'AE3.612','Auroral Input/Output','','','Meteorites/Auroral',428,'',184 from dual union all
select 12234,'1965-S159','1965-03-13 08:20:31','military missile','success',983,'KY-2','','','','Test',636,'',207 from dual union all
select 12235,'1965-S16','1965-01-01 00:00:00','military missile','success',1035,'','','','','Test',636,'',500 from dual union all
select 12236,'1965-S160','1965-03-15 18:22:02','military missile','success',932,'A3P-158','Surface launch','','','OT',421,'SSBN 636',1000 from dual union all
select 12237,'1965-S161','1965-03-15 19:09:01','military missile','success',932,'A3P-21','','','','OT',421,'SSBN 636',1000 from dual union all
select 12238,'1965-S162','1965-03-16 00:00:00','military missile','success',795,'801','Mk 11 RV','OT','TAIL FIN','OT',1697,'',900 from dual union all
select 12239,'1965-S163','1965-03-16 00:00:00','military missile','success',976,'','8K63 UBP 44 RD','','','Test',636,'',402 from dual union all
select 1224,'1952-A162','1952-09-24 00:00:00','atmospheric rocket','success',365,'','D35 Model','','','Test',1821,'',10 from dual union all
select 12240,'1965-S164','1965-03-16 16:15:00','suborbital rocket','success',837,'NASA 14.171GE','B field','','','Plasma',1066,'CVE-25',172 from dual union all
select 12241,'1965-S165','1965-03-16 16:51:00','suborbital rocket','success',837,'NASA 14.177GI','','','','Ionos',1067,'CVE-25',200 from dual union all
select 12242,'1965-S166','1965-03-17 15:10:00','suborbital rocket','success',30,'NASA 04.63GS','ASE XRT','','','Solar X',1869,'',156 from dual union all
select 12243,'1965-S167','1965-03-17 21:30:00','suborbital rocket','success',215,'AE-2A-24','','','','Aeron',428,'',153 from dual union all
select 12244,'1965-S168','1965-03-18 06:04:00','suborbital rocket','success',837,'NASA 14.172GE','B field','','','Plasma',1068,'CVE-25',150 from dual union all
select 12245,'1965-S169','1965-03-18 06:41:00','suborbital rocket','success',837,'NASA 14.178GI','','','','Ionos',1069,'CVE-25',200 from dual union all
select 12246,'1965-S17','1965-01-01 00:00:00','military missile','success',206,'011','','','','Reentry Test',313,'',270 from dual union all
select 12247,'1965-S170','1965-03-18 10:07:00','suborbital rocket','success',697,'L-3-3','','','','XR Astron',834,'',1085 from dual;
insert into LAUNCH
select 12248,'1965-S171','1965-03-18 16:01:00','suborbital rocket','success',837,'NASA 14.176GE','B field','','','Plasma',1070,'CVE-25',149 from dual union all
select 12249,'1965-S172','1965-03-18 16:38:00','suborbital rocket','success',837,'NASA 14.179GI','','','','Ionos',1071,'CVE-25',200 from dual union all
select 1225,'1952-A163','1952-09-24 00:00:00','atmospheric rocket','success',836,'1249-116','Nike 1249 R&D 112R','','','Test',1895,'',10 from dual union all
select 12250,'1965-S173','1965-03-18 20:43:00','suborbital rocket','success',837,'NASA 14.62UA','Composition','','','Aeronomy',1821,'',158 from dual union all
select 12251,'1965-S174','1965-03-19 03:00:00','suborbital rocket','success',30,'NASA 04.57GG','UV spectra','','','UV Astron',1869,'',155 from dual union all
select 12252,'1965-S175','1965-03-19 06:24:00','suborbital rocket','success',215,'AA-2A-52','','','','Fields',428,'',168 from dual union all
select 12253,'1965-S176','1965-03-19 18:09:00','suborbital rocket','success',867,'NASA 12.05/18.01GA','UM Thermosphere 1','','','Aeronomy',1821,'',315 from dual union all
select 12254,'1965-S177','1965-03-20 05:42:00','suborbital rocket','success',42,'NASA 06.11GA','Thermosphere probe','','','Aeronomy',1821,'',326 from dual union all
select 12255,'1965-S178','1965-03-20 13:20:09','suborbital rocket','success',837,'NASA 14.228UI','','','','Ionos',1072,'CVE-25',174 from dual union all
select 12256,'1965-S179','1965-03-20 16:28:00','suborbital rocket','success',837,'AE5.293','B field','','','Ionos',1073,'CVE-25',201 from dual union all
select 12257,'1965-S18','1965-06-24 00:00:00','military missile','success',1449,'','8K84 LKI-4','','','Test',502,'',1000 from dual union all
select 12258,'1965-S180','1965-03-20 17:04:13','suborbital rocket','success',839,'AE6.252','Electron density','','','Ionos',1074,'CVE-25',122 from dual union all
select 12259,'1965-S181','1965-03-21 23:08:00','test rocket','success',251,'F-3','','','','-',1845,'',263 from dual union all
select 1226,'1952-A164','1952-09-25 00:00:00','military missile','unknown',339,'32R','','','','Test',1895,'',30 from dual union all
select 12260,'1965-S182','1965-03-22 00:00:00','military missile','success',979,'','8K65 UBP-59','','','Test',636,'',675 from dual union all
select 12261,'1965-S183','1965-03-22 07:44:00','suborbital rocket','success',837,'AE5.294','B field','','','Ionos',1075,'CVE-25',206 from dual union all
select 12262,'1965-S184','1965-03-22 08:49:10','suborbital rocket','success',839,'AE6.253','Electron density','','','Ionos',1077,'CVE-25',120 from dual union all
select 12263,'1965-S185','1965-03-23 09:31:09','suborbital rocket','success',837,'NASA 14.229UI','','','','Ionos',1074,'CVE-25',165 from dual union all
select 12264,'1965-S186','1965-03-23 06:40:00','suborbital rocket','success',215,'AA-2A-53','[or Mar 23]','','','Fields',428,'',166 from dual union all
select 12265,'1965-S187','1965-03-24 16:34:00','suborbital rocket','success',837,'NASA 14.174GE','B field','','','Plasma',1079,'CVE-25',200 from dual union all
select 12266,'1965-S188','1965-03-24 17:07:00','suborbital rocket','success',837,'NASA 14.180GI','','','','Ionos',1080,'CVE-25',200 from dual union all
select 12267,'1965-S189','1965-03-25 23:11:00','military missile','success',795,'756','Mk 11 RV','OT','QUICK NOTE','OT',1698,'',900 from dual union all
select 12268,'1965-S19','1965-06-25 10:03:00','military missile','failure',1449,'U 22943-201','8K84 LKI-5','GCh 8F121','','Test',504,'',100 from dual union all
select 12269,'1965-S190','1965-03-25 00:06:00','suborbital rocket','success',1199,'SL141','ROE-2/SHF-4/UCL I-21','','','UV Astronomy/Ionos',1839,'',174 from dual union all
select 1227,'1952-A165','1952-09-25 00:00:00','atmospheric rocket','failure',365,'','F-100 Model','','','Test',1821,'',10 from dual union all
select 12270,'1965-S191','1965-03-25 02:15:16','military missile','success',1420,'B-60','Mk 6 RV','OT','ARCTIC SUN','OT',1657,'',1300 from dual union all
select 12271,'1965-S192','1965-03-24 05:44:00','suborbital rocket','success',215,'AA-2A-60','[or Mar 24]','','','Ionos/Micromet',428,'',155 from dual union all
select 12272,'1965-S193','1965-03-25 11:21:00','suborbital rocket','success',668,'K-9M-8','Airglow','','','Aeron/Ionos',839,'',320 from dual union all
select 12273,'1965-S194','1965-03-25 16:00:00','suborbital rocket','success',837,'AE5.295','B field','','','Ionos',1081,'CVE-25',100 from dual union all
select 12274,'1965-S195','1965-03-26 00:00:00','military missile','success',976,'','8K63 UBP','','','Test',636,'',402 from dual union all
select 12275,'1965-S196','1965-03-26 09:01:11','military missile','success',154,'297D','LORV-7','ABRES LORV-7','FRESH FROG','Reentry Test',1662,'',1800 from dual union all
select 12276,'1965-S197','1965-03-26 15:41:00','suborbital rocket','success',837,'NASA 14.173GE','B field','','','Plasma',1082,'CVE-25',155 from dual union all
select 12277,'1965-S198','1965-03-26 16:13:00','suborbital rocket','success',837,'NASA 14.181GI','','','','Ionos',1083,'CVE-25',200 from dual union all
select 12278,'1965-S199','1965-03-27 03:00:00','suborbital rocket','success',837,'NASA 14.182GI','','','','Ionos',1084,'CVE-25',200 from dual union all
select 12279,'1965-S20','1965-07-24 00:00:00','military missile','success',1449,'','8K84 LKI-7','','','Test',501,'',1000 from dual union all
select 1228,'1952-A166','1952-09-25 00:00:00','atmospheric rocket','success',836,'1249A-120','Nike 1249 R&D 113R','','','Test',1895,'',10 from dual union all
select 12280,'1965-S200','1965-03-27 05:01:00','suborbital rocket','success',668,'K-9M-9','Te probe','','','Ionos',839,'',335 from dual union all
select 12281,'1965-S201','1965-03-27 18:08:00','suborbital rocket','success',837,'NASA 14.175GE','B field','','','Plasma',1085,'CVE-25',200 from dual union all
select 12282,'1965-S202','1965-03-28 07:01:00','suborbital rocket','success',668,'K-9M-10','','','','Ionos/Plasma',839,'',349 from dual union all
select 12283,'1965-S203','1965-03-28 15:50:00','suborbital rocket','success',837,'AE5.296','B field','','','Ionos',1086,'CVE-25',129 from dual union all
select 12284,'1965-S204','1965-03-29 18:30:07','military missile','success',932,'A3P-11','','','','OT',421,'SSBN 636',1000 from dual union all
select 12285,'1965-S205','1965-03-29 15:47:00','suborbital rocket','success',837,'NASA 14.70GE','B field','','','Plasma',1088,'CVE-25',172 from dual union all
select 12286,'1965-S206','1965-03-29 15:47:00','suborbital rocket','success',839,'NASA 10.143GE','','','','Plasma',1088,'CVE-25',172 from dual union all
select 12287,'1965-S207','1965-03-30 04:01:00','military missile','success',982,'R 22601-02T','8K64U UBP-51','','','OT',566,'',1210 from dual union all
select 12288,'1965-S208','1965-03-30 05:50:00','suborbital rocket','success',30,'ND3.159','','','','Aeron/UV Astron',1869,'',217 from dual union all
select 12289,'1965-S209','1965-03-30 09:30:00','suborbital rocket','success',535,'HAD 153','Sphere','','','Aeron',1835,'',112 from dual union all
select 1229,'1952-A167','1952-09-26 00:00:00','atmospheric rocket','unknown',576,'53','','','','Test',1895,'',10 from dual union all
select 12290,'1965-S21','1965-08-31 00:00:00','military missile','success',1449,'','8K84 LKI-9','','','Test',504,'',1000 from dual union all
select 12291,'1965-S210','1965-04-28 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 12292,'1965-S211','1965-04-01 02:07:00','suborbital rocket','success',837,'NASA 14.132NA','Airglow','','','Aeronomy',1821,'',139 from dual union all
select 12293,'1965-S212','1965-04-02 13:22:00','suborbital rocket','success',837,'NASA 14.185UE','UNH Particles','','','Plasma',1089,'CVE-25',200 from dual union all
select 12294,'1965-S213','1965-04-03 12:54:00','suborbital rocket','success',1198,'S03/2','ESRO S03/2 (SE5)','','','Ionospheric',1343,'',172 from dual union all
select 12295,'1965-S214','1965-04-03 21:59:00','suborbital rocket','success',837,'NASA 14.207UE','Auroral fluxes','','','Plasma',428,'',205 from dual union all
select 12296,'1965-S215','1965-04-04 16:06:35','suborbital rocket','success',837,'NASA 14.66UA','UM Pitot 10','','','Aeronomy',1090,'CVE-25',139 from dual union all
select 12297,'1965-S216','1965-04-05 07:20:00','military missile','success',1373,'155','AFP-437 CTL','','','Interceptor',773,'',826 from dual union all
select 12298,'1965-S217','1965-04-05 13:45:53','suborbital rocket','success',837,'NASA 14.230UI','','','','Ionos',1091,'CVE-25',177 from dual union all
select 12299,'1965-S218','1965-04-05 19:42:00','suborbital rocket','success',837,'NASA 14.184UE','UNH Particles','','','Plasma',1092,'CVE-25',200 from dual union all
select 123,'1944-S111','1944-09-17 00:00:00','military missile','success',1470,'','London','','','Weapon',1745,'',95 from dual union all
select 1230,'1952-A168','1952-09-30 00:00:00','atmospheric rocket','success',836,'1249A-118','Nike 1249 R&D 114R','','','Test',1895,'',10 from dual union all
select 12300,'1965-S219','1965-04-06 13:34:59','military missile','success',154,'150D','WAC IV','ABRES WAC-4','FLIP SIDE','Reentry Test',1665,'',1800 from dual union all
select 12301,'1965-S22','1965-10-04 00:00:00','military missile','success',1449,'','8K84 LKI-11','','','Test',503,'',1000 from dual union all
select 12302,'1965-S220','1965-04-06 16:34:05','suborbital rocket','success',837,'NASA 14.26UA','UM Pitot 11','','','Aeronomy',1093,'CVE-25',142 from dual union all
select 12303,'1965-S221','1965-04-09 01:01:00','suborbital rocket','success',1199,'SL303','LUX-26AC3/Cul-3','','','Solar X,UV',1839,'',160 from dual union all
select 12304,'1965-S222','1965-04-09 19:18:15','suborbital rocket','success',837,'NASA 14.231UI','','','','Ionos',1094,'CVE-25',191 from dual union all
select 12305,'1965-S223','1965-04-09 20:26:10','suborbital rocket','success',837,'NASA 14.63UA','UM Pitot 12','','','Aeronomy',1095,'CVE-25',147 from dual union all
select 12306,'1965-S224','1965-04-10 08:10:00','military missile','success',795,'1150','Mk 11 RV','OT','SMOKEY RIVER','OT',1705,'',900 from dual union all
select 12307,'1965-S225','1965-04-12 07:30:00','military missile','success',982,'T 22601-12TB','8K64U UBP-52','','','OT',570,'',1210 from dual union all
select 12308,'1965-S226','1965-04-12 14:47:00','suborbital rocket','success',30,'NASA 04.49GS (128-3)','Mg II heliogram 1','','','Solar UV',1869,'',201 from dual union all
select 12309,'1965-S227','1965-04-12 17:14:02','suborbital rocket','success',837,'NASA 14.232UI','UIUC-GCA Ionos 7','','','Ionos',1096,'CVE-25',187 from dual union all
select 1231,'1952-A169','1952-10-01 00:00:00','military missile','success',763,'Dyn Model #1','','','','Test',222,'',10 from dual union all
select 12310,'1965-S228','1965-04-12 18:37:00','suborbital rocket','success',848,'AE07.313','Vehicle Test','','','Test',394,'',143 from dual union all
select 12311,'1965-S229','1965-04-13 21:00:00','military missile','success',795,'851','Mk 11 RV','OT','SEA POINT','OT',1697,'',900 from dual union all
select 12312,'1965-S23','1965-10-06 00:00:00','military missile','success',1449,'','8K84 LKI-12','','','Test',503,'',1000 from dual union all
select 12313,'1965-S230','1965-04-13 21:00:00','military missile','success',795,'1165','Mk 11 RV','OT','YELLOW LIGHT','OT',1703,'',900 from dual union all
select 12314,'1965-S231','1965-04-13 04:05:06','suborbital rocket','success',837,'NASA 14.67UA','UM Pitot 13','','','Aeronomy',1097,'CVE-25',150 from dual union all
select 12315,'1965-S232','1965-04-13 16:00:09','suborbital rocket','success',837,'NASA 14.27UA','UM Pitot 14','','','Aeronomy',1097,'CVE-25',146 from dual union all
select 12316,'1965-S233','1965-04-13 17:10:00','suborbital rocket','success',837,'NASA 14.186UE','UNH Particles','','','Plasma',1097,'CVE-25',200 from dual union all
select 12317,'1965-S234','1965-04-13 19:06:00','suborbital rocket','success',837,'NASA 14.101UA','Composition','','','Aeronomy',1099,'CVE-25',200 from dual union all
select 12318,'1965-S235','1965-04-14 14:39:00','sounding rocket','success',630,'NASA 11.07UE','Minnesota Particles','','','Plasma',1821,'',1660 from dual union all
select 12319,'1965-S236','1965-04-15 10:45:00','suborbital rocket','success',30,'NASA 04.127UA','AF-142 Thermo. comp.','','','Aeronomy',1869,'',198 from dual union all
select 1232,'1952-A17','1952-01-29 00:00:00','atmospheric rocket','success',869,'490B','System Test 76R','','','Test',1895,'',10 from dual union all
select 12320,'1965-S237','1965-04-15 16:00:04','suborbital rocket','success',837,'NASA 14.25UA','UM Pitot 15','','','Aeronomy',1100,'CVE-25',140 from dual union all
select 12321,'1965-S238','1965-04-15 18:02:00','suborbital rocket','success',837,'NASA 14.100UA','Composition','','','Aeronomy',1101,'CVE-25',200 from dual union all
select 12322,'1965-S239','1965-04-16 00:00:00','military missile','success',976,'','8K63 PRO-33','','','Test',942,'',402 from dual union all
select 12323,'1965-S24','1965-10-19 00:00:00','military missile','success',1449,'','8K84 LKI-13','Salvo','','Test',503,'',1000 from dual union all
select 12324,'1965-S240','1965-04-16 19:19:55','military missile','success',1420,'B-45','Mk 6 RV','OT','BEAR HUG','OT',1658,'',1300 from dual union all
select 12325,'1965-S241','1965-04-19 02:20:00','military missile','success',1449,'U 10741-601','8K84 LKI-1','GCh 8F121','','Test',502,'',1000 from dual union all
select 12326,'1965-S242','1965-04-20 15:40:00','suborbital rocket','success',30,'NE3.156','','','','Solar UV,X',1869,'',201 from dual union all
select 12327,'1965-S243','1965-04-21 11:01:47','suborbital rocket','success',243,'BK.21','H-211/Dazzle 3','','','Reentry Test/Ionos',1842,'',651 from dual union all
select 12328,'1965-S244','1965-04-22 00:00:00','military missile','success',1013,'','8K51 29 RD UBP-4','','','OT',636,'',500 from dual union all
select 12329,'1965-S245','1965-04-22 05:14:00','suborbital rocket','success',217,'AA-3-11','','','','Auroral/Ionos',428,'',136 from dual union all
select 1233,'1952-A170','1952-10-02 00:00:00','military missile','unknown',339,'33R','','','','Test',1895,'',30 from dual union all
select 12330,'1965-S246','1965-04-23 03:01:00','suborbital rocket','success',839,'NASA 10.171NA','Release','','','Aeronomy',1821,'',128 from dual union all
select 12331,'1965-S247','1965-04-23 04:02:00','suborbital rocket','success',837,'NASA 14.255NA','Release','','','Aeronomy',1821,'',145 from dual union all
select 12332,'1965-S248','1965-04-24 00:00:00','military missile','success',976,'','8K63 PRO-34','','','Test',942,'',402 from dual union all
select 12333,'1965-S249','1965-04-24 00:00:00','military missile','success',1013,'','8K51 29 RD UBP-5','','','OT',636,'',500 from dual union all
select 12334,'1965-S25','1965-10-19 00:00:00','military missile','success',1449,'','8K84 LKI-14','Salvo','','Test',508,'',1000 from dual union all
select 12335,'1965-S250','1965-04-24 05:15:00','suborbital rocket','failure',30,'NASA 04.114GG','','','','UV Astron',1869,'',82 from dual union all
select 12336,'1965-S251','1965-04-24 10:00:25','military missile','success',983,'KY-2','','','','Test',636,'',250 from dual union all
select 12337,'1965-S252','1965-04-26 13:00:00','military missile','success',982,'U 22601-20T','8K64U UBP-53','','','OT',566,'',1210 from dual;
insert into LAUNCH
select 12338,'1965-S253','1965-04-26 14:13:00','military missile','success',1013,'','8K51 29 RD UBP-6','','','OT',636,'',500 from dual union all
select 12339,'1965-S255','1965-04-27 05:00:00','military missile','success',999,'R 22500-28L','8K67 LKI-20','','','Test',586,'',870 from dual union all
select 1234,'1952-A171','1952-10-02 00:00:00','atmospheric rocket','success',836,'1249A-122','Nike 1249 R&D 115R','','','Test',1895,'',10 from dual union all
select 12340,'1965-S256','1965-04-27 06:30:00','suborbital rocket','success',30,'NB3.176','','','','XR Astron',1869,'',193 from dual union all
select 12341,'1965-S257','1965-04-28 22:27:00','suborbital rocket','success',839,'NASA 10.150GA','GSFC Grenade 75','','','Aeronomy',1039,'',100 from dual union all
select 12342,'1965-S258','1965-04-28 23:57:00','suborbital rocket','success',288,'C99','ISRO 15.02','TERLS-022','','Aeron/Ionos',1467,'',150 from dual union all
select 12343,'1965-S259','1965-04-29 09:03:00','suborbital rocket','failure',1199,'SL464','UCL GC-26/QUB Li-1','','','Aeronomy',1839,'',140 from dual union all
select 12344,'1965-S26','1965-11-01 00:00:00','military missile','success',1449,'','8K84 LKI-15','','','Test',504,'',1000 from dual union all
select 12345,'1965-S260','1965-04-29 11:09:00','suborbital rocket','success',1198,'SL363','UCL GC-27','','','Aeronomy',1839,'',136 from dual union all
select 12346,'1965-S261','1965-04-29 12:25:00','suborbital rocket','success',1198,'SL364','UCL GC-28','','','Aeronomy',1839,'',132 from dual union all
select 12347,'1965-S262','1965-04-29 13:56:00','suborbital rocket','success',1198,'SL461','UCL GC-29','','','Aeronomy',1839,'',124 from dual union all
select 12348,'1965-S263','1965-04-29 16:19:00','suborbital rocket','success',1198,'SL462','UCL GC-30','','','Aeronomy',1839,'',140 from dual union all
select 12349,'1965-S264','1965-04-29 17:32:00','suborbital rocket','success',839,'NASA 10.94IA','Rehbar 7 (Grenades)','','','Aeronomy',1419,'',100 from dual union all
select 1235,'1952-A172','1952-10-03 00:00:00','atmospheric rocket','unknown',576,'54','','','','Test',1895,'',10 from dual union all
select 12350,'1965-S265','1965-04-29 17:56:00','suborbital rocket','success',1198,'SL463','UCL GC-31','','','Aeronomy',1839,'',135 from dual union all
select 12351,'1965-S266','1965-04-29 20:40:00','suborbital rocket','success',1199,'SL362','UCL GS-5','','','Aeronomy',1839,'',175 from dual union all
select 12352,'1965-S267','1965-04-30 22:00:00','military missile','success',795,'730','Mk 11 RV','OT','WINTER BREW','OT',1698,'',900 from dual union all
select 12353,'1965-S268','1965-05-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeron?',1895,'',100 from dual union all
select 12354,'1965-S269','1965-05-27 00:00:00','military missile','success',916,'AO-25 (overshot)','P-91','','','OT',597,'',180 from dual union all
select 12355,'1965-S27','1965-12-17 00:00:00','military missile','success',1449,'','8K84 LKI-19','','','Test',503,'',1000 from dual union all
select 12356,'1965-S270','1965-05-04 00:00:00','suborbital rocket','success',1440,'D58-3539','Reentry expt.','','','Reentry test',1821,'',300 from dual union all
select 12357,'1965-S271','1965-04-30 19:05:07','military missile','success',1420,'B-54','Mk 6 RV','OT','CARD DECK','OT',1659,'',1300 from dual union all
select 12358,'1965-S272','1965-05-01 23:43:00','suborbital rocket','success',288,'C100','ISRO 15.01','TERLS-023','','Ionos',1467,'',140 from dual union all
select 12359,'1965-S273','1965-05-03 18:23:00','suborbital rocket','success',839,'NASA 10.127GA','GSFC Grenade 76','','','Aeronomy',1821,'',100 from dual union all
select 1236,'1952-A173','1952-10-03 00:00:00','atmospheric rocket','unknown',576,'55','','','','Test',1895,'',10 from dual union all
select 12360,'1965-S274','1965-05-05 03:23:00','suborbital rocket','success',30,'NASA 04.89GG','STRAP','','','UV Astron',1821,'',145 from dual union all
select 12361,'1965-S275','1965-05-07 07:28:00','military missile','success',971,'','','','','OT',636,'',86 from dual union all
select 12362,'1965-S276','1965-05-08 01:59:59','military missile','success',796,'455','Mk 11 RV','R&D','','Test',266,'',900 from dual union all
select 12363,'1965-S277','1965-05-08 10:47:00','military missile','success',971,'','','','','OT',636,'',86 from dual union all
select 12364,'1965-S278','1965-05-10 00:00:00','military missile','success',795,'1003','Mk 11 RV','OT','VIOLET RAY','OT',1705,'',900 from dual union all
select 12365,'1965-S279','1965-05-10 07:30:00','suborbital rocket','success',535,'HAD 165','Falling sphere','','','Aeron',220,'',115 from dual union all
select 12366,'1965-S28','1965-12-29 00:00:00','military missile','success',1449,'','8K84 LKI-21','','','Test',503,'',1000 from dual union all
select 12367,'1965-S280','1965-05-10 10:20:00','suborbital rocket','success',535,'HAD 164','Falling sphere','','','Aeron',220,'',116 from dual union all
select 12368,'1965-S281','1965-05-10 13:20:00','suborbital rocket','success',535,'HAD 162','Falling sphere','','','Aeron',220,'',116 from dual union all
select 12369,'1965-S282','1965-05-10 16:20:00','suborbital rocket','success',535,'HAD 158','Falling sphere','','','Aeron',220,'',109 from dual union all
select 1237,'1952-A174','1952-10-03 00:00:00','atmospheric rocket','success',565,'','E39 Model','','','Test',1821,'',1 from dual union all
select 12370,'1965-S283','1965-05-10 19:20:00','suborbital rocket','success',535,'HAD 170','Falling sphere','','','Aeron',220,'',115 from dual union all
select 12371,'1965-S284','1965-05-11 00:45:00','suborbital rocket','success',535,'HAD 154','Falling sphere','','','Aeron',220,'',111 from dual union all
select 12372,'1965-S285','1965-05-11 03:10:00','suborbital rocket','success',535,'HAD 155','Falling sphere','','','Aeron',220,'',113 from dual union all
select 12373,'1965-S286','1965-05-11 08:39:00','suborbital rocket','success',535,'HAD 160','Sphere','','','Aeron',1835,'',114 from dual union all
select 12374,'1965-S287','1965-05-11 20:40:00','suborbital rocket','success',1199,'SL361','UCL FS-6/QUB Li-2','','','Aeronomy',1839,'',181 from dual union all
select 12375,'1965-S288','1965-05-12 12:30:00','military missile','success',976,'','8K63 UBP','','','Test',636,'',402 from dual union all
select 12376,'1965-S289','1965-05-13 00:00:00','military missile','success',458,'','','','','Test',669,'',200 from dual union all
select 12377,'1965-S29','1965-01-07 03:50:00','suborbital rocket','success',837,'NASA 14.142NA','Airglow','','','Aeronomy',1821,'',146 from dual union all
select 12378,'1965-S290','1965-05-13 16:05:00','suborbital rocket','success',1199,'SL139','RSRS-4','','','Ionos',1839,'',187 from dual union all
select 12379,'1965-S292','1965-05-14 19:12:00','military missile','success',971,'','','','','OT',636,'',86 from dual union all
select 1238,'1952-A175','1952-10-03 00:00:00','atmospheric rocket','success',565,'','E39 Model','','','Test',1821,'',2 from dual union all
select 12380,'1965-S293','1965-05-15 08:00:00','military missile','success',971,'','','','','OT',636,'',86 from dual union all
select 12381,'1965-S294','1965-05-18 00:00:00','test rocket','success',1433,'LG1 (5)','','','','Test',669,'',100 from dual union all
select 12382,'1965-S295','1965-05-18 00:00:00','military missile','success',795,'1115','Mk 11 RV','OT','SILVER CLOUD','OT',1702,'',900 from dual union all
select 12383,'1965-S296','1965-05-18 05:00:00','military missile','success',999,'R 22500-29L','8K67 LKI-21','','','Test',586,'',870 from dual union all
select 12384,'1965-S297','1965-05-18 09:07:00','military missile','success',971,'','','','','OT',636,'',86 from dual union all
select 12385,'1965-S298','1965-05-19 05:25:00','suborbital rocket','success',1199,'SL138','LUX-27/RSRS-5','','','Ionos/Solar X',1839,'',179 from dual union all
select 12386,'1965-S299','1965-05-19 13:00:00','military missile','success',978,'','','','','Test',201,'K-170',150 from dual union all
select 12387,'1965-S30','1965-01-08 00:00:00','military missile','success',979,'','8K65 UBP-56','','','Test',636,'',675 from dual union all
select 12388,'1965-S300','1965-05-19 20:11:00','suborbital rocket','success',624,'NASA 8.29UI','M/DTR 2','','','Ionos',1821,'',947 from dual union all
select 12389,'1965-S301','1965-05-21 00:00:00','test rocket','success',1433,'LG2 (6)','','','','Test',669,'',100 from dual union all
select 1239,'1952-A176','1952-10-03 00:00:00','atmospheric rocket','success',836,'1249A-121','Nike 1249 R&D 116R','','','Test',1895,'',10 from dual union all
select 12390,'1965-S302','1965-05-21 23:53:25','military missile','success',1420,'B-51','Mk 6 RV','OT','FRONT SIGHT','OT',1657,'',1300 from dual union all
select 12391,'1965-S303','1965-05-22 02:02:01','suborbital rocket','success',837,'NASA 14.47UA','UM Pitot 16/FIRE cal','','','Aeronomy',176,'',147 from dual union all
select 12392,'1965-S304','1965-05-22 06:29:00','military missile','success',982,'G 22503-08T','8K64U UBP-55','','','OT',565,'',920 from dual union all
select 12393,'1965-S305','1965-05-22 14:00:00','suborbital rocket','success',837,'NASA 14.48UA','UM Pitot 17/FIRE cal','','','Aeronomy',176,'',158 from dual union all
select 12394,'1965-S306','1965-05-24 10:00:00','military missile','success',978,'','','','','Test',201,'GOLF',150 from dual union all
select 12395,'1965-S307','1965-05-25 00:00:00','suborbital rocket','success',535,'HAD','','','','Aeron',1835,'',110 from dual union all
select 12396,'1965-S308','1965-05-25 00:00:00','military missile','success',1013,'','8K51 29 RD UBP-7','','','OT',636,'',500 from dual union all
select 12397,'1965-S309','1965-05-25 13:00:00','military missile','success',978,'','','','','Test',201,'GOLF',150 from dual union all
select 12398,'1965-S31','1965-01-08 18:59:54','military missile','success',162,'106F','Mk 4-5B','ST','PILOT LIGHT','Test',1672,'',1400 from dual union all
select 12399,'1965-S310','1965-05-25 23:20:00','suborbital rocket','success',470,'CRL AD13.815','EM Absorption','','','Ionos',394,'',488 from dual union all
select 124,'1944-S112','1944-09-18 00:00:00','military missile','success',1470,'','London/W Norwood','','','Weapon',1745,'',95 from dual union all
select 1240,'1952-A177','1952-10-03 00:00:00','atmospheric rocket','success',836,'1249A-125','Nike 1249 R&D 117R','','','Test',1895,'',10 from dual union all
select 12400,'1965-S311','1965-05-26 02:00:00','military missile','success',796,'456','Mk 11 RV','R&D','','Test',266,'',900 from dual union all
select 12401,'1965-S312','1965-05-26 00:00:00','military missile','success',916,'AO-24','P-90','','','OT',597,'',180 from dual union all
select 12402,'1965-S313','1965-05-26 00:00:00','military missile','success',1013,'','8K51 29 RD UBP-8','','','OT',636,'',500 from dual union all
select 12403,'1965-S314','1965-05-26 13:36:00','military missile','success',978,'','','','','Test',201,'K-170',150 from dual union all
select 12404,'1965-S315','1965-05-26 17:30:00','military missile','success',983,'KY-2','','','','Test',636,'',231 from dual union all
select 12405,'1965-S316','1965-05-26 19:01:00','suborbital rocket','failure',624,'NASA 8.37GI','','','','Ionos',1821,'',320 from dual union all
select 12406,'1965-S317','1965-05-26 23:20:00','suborbital rocket','success',470,'CRL AD13.812','EM Absorption','','','Ionos',394,'',488 from dual union all
select 12407,'1965-S318','1965-05-28 02:23:00','military missile','success',982,'T 22601-17T','8K64U UBP-56','','','OT',570,'',743 from dual union all
select 12408,'1965-S319','1965-05-30 20:00:00','suborbital rocket','success',865,'Sandia 152-74','LASL Eclipse','','','Solar X',1313,'',270 from dual union all
select 12409,'1965-S32','1965-01-11 17:13:03','military missile','success',932,'A3P-67','','','','OT',421,'SSBN 632',1000 from dual union all
select 1241,'1952-A178','1952-10-07 00:00:00','atmospheric rocket','success',836,'1249A-123','Nike 1249 R&D 118R','','','Test',1895,'',10 from dual union all
select 12410,'1965-S320','1965-05-30 20:00:00','suborbital rocket','success',865,'Sandia 152-75','LASL Eclipse','','','Solar X',1313,'',270 from dual union all
select 12411,'1965-S321','1965-05-30 20:00:00','suborbital rocket','success',865,'Sandia 152-76','LASL Eclipse','','','Solar X',1313,'',270 from dual union all
select 12412,'1965-S322','1965-05-30 20:00:00','suborbital rocket','success',865,'Sandia 152-77','LASL Eclipse','','','Solar X',1313,'',270 from dual union all
select 12413,'1965-S323','1965-05-30 20:00:00','suborbital rocket','success',865,'Sandia 152-78','LASL Eclipse','','','Solar X',1313,'',270 from dual union all
select 12414,'1965-S324','1965-05-31 00:00:00','sounding rocket','success',1068,'R5','A-1 capsule','','','Test/Ionos?',664,'',2000 from dual union all
select 12415,'1965-S325','1965-06-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeron?',1895,'',100 from dual union all
select 12416,'1965-S326','1965-06-01 00:00:00','military missile','success',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 12417,'1965-S327','1965-06-01 00:00:00','military missile','success',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 12418,'1965-S328','1965-05-31 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 623',1000 from dual union all
select 12419,'1965-S329','1965-05-31 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 623',1000 from dual union all
select 1242,'1952-A179','1952-10-07 00:00:00','atmospheric rocket','success',836,'1249A-126','Nike 1249 R&D 119R','','','Test',1895,'',10 from dual union all
select 12420,'1965-S33','1965-01-12 14:32:26','military missile','success',154,'166D','Nike RMV-302','NTMP KX-25','PENCIL SET','Target',1665,'',1800 from dual union all
select 12421,'1965-S330','1965-04-01 00:00:00','military missile','success',874,'ZK-55','Exoatm test','','','Interceptor',873,'',80 from dual union all
select 12422,'1965-S331','1965-06-01 00:00:00','military missile','failure',874,'ZK-56','DM-15S2 Qual-0','','','Interceptor',873,'',80 from dual union all
select 12423,'1965-S332','1965-05-31 13:30:00','military missile','success',978,'','','','','Test',201,'GOLF',150 from dual union all
select 12424,'1965-S333','1965-06-02 10:40:00','military missile','success',795,'1054','Mk 11 RV','OT','SURF SPRAY','OT',1703,'',900 from dual union all
select 12425,'1965-S334','1965-06-02 00:00:00','military missile','success',916,'AO-26','P-92','','','OT',597,'',180 from dual union all
select 12426,'1965-S335','1965-06-02 06:22:00','suborbital rocket','success',30,'NASA 04.17UG','Princeton UVS','','','UV Astron',1869,'',181 from dual union all
select 12427,'1965-S336','1965-06-02 10:19:00','suborbital rocket','success',839,'AD6.850','Ionos winds,T','','','Aeron',394,'',121 from dual;
insert into LAUNCH
select 12428,'1965-S337','1965-06-02 10:30:00','suborbital rocket','success',839,'AE6.498','BETH','','','Aeron',394,'',121 from dual union all
select 12429,'1965-S338','1965-06-02 14:19:00','suborbital rocket','success',215,'BB-2A-50','','','','Auroral/Ionos',428,'',150 from dual union all
select 1243,'1952-A18','1952-01-29 00:00:00','atmospheric rocket','success',869,'490B','System Test 75R','','','Test',1895,'',10 from dual union all
select 12430,'1965-S339','1965-06-03 00:00:00','sounding rocket','success',1068,'R6','A-1 capsule','','','Test/Ionos?',664,'',2000 from dual union all
select 12431,'1965-S34','1965-01-13 15:49:00','suborbital rocket','success',30,'KP3.9','Dayglow','','','Aeron',1869,'',228 from dual union all
select 12432,'1965-S340','1965-06-03 10:38:09','military missile','success',154,'177D','Nike RMV-304','NTMP KX-40','OLD FOGEY','Reentry Test',1684,'',1800 from dual union all
select 12433,'1965-S341','1965-06-03 20:54:00','suborbital rocket','success',768,'111 Telem','2C XI-1 IRE','','','Ionos',185,'',105 from dual union all
select 12434,'1965-S342','1965-06-05 05:40:00','sounding rocket','success',1068,'R7','Rubis 01/D-1','','','Test',664,'',2020 from dual union all
select 12435,'1965-S343','1965-06-05 20:20:00','suborbital rocket','success',767,'118','2C XI-2 BRUTUS','','','Ionos',185,'',120 from dual union all
select 12436,'1965-S344','1965-06-05 22:43:00','suborbital rocket','success',767,'119','2C XI-3 JANUS','','','Ionos',185,'',117 from dual union all
select 12437,'1965-S345','1965-06-06 17:45:00','suborbital rocket','success',545,'','DASA D6.4','','','Ionos',394,'',100 from dual union all
select 12438,'1965-S346','1965-06-07 00:00:00','military missile','success',795,'810','Mk 11 RV','OT','WHEEL HORSE','OT',1698,'',900 from dual union all
select 12439,'1965-S348','1965-06-08 08:31:00','suborbital rocket','success',535,'HAD 172','Sphere','','','Aeron',1835,'',106 from dual union all
select 1244,'1952-A180','1952-10-09 00:00:00','military missile','success',771,'GM-570','','','','Test',222,'',12 from dual union all
select 12440,'1965-S349','1965-06-08 15:17:03','military missile','success',154,'299D','LORV 6','ABRES LORV-6','LEA RING','Reentry Test',1680,'',1800 from dual union all
select 12441,'1965-S35','1965-01-13 17:12:00','suborbital rocket','success',624,'NASA 8.28UI','M/DTR 1','','','Ionos',1821,'',1006 from dual union all
select 12442,'1965-S350','1965-06-09 08:32:00','suborbital rocket','success',535,'HAD 171','Sphere','','','Aeron',1835,'',115 from dual union all
select 12443,'1965-S351','1965-06-10 00:00:00','military missile','failure',795,'669','Mk 11 RV','OT','SPEED KING','OT',1697,'',1 from dual union all
select 12444,'1965-S352','1965-06-10 12:26:40','military missile','success',154,'302D','Nike RMV-303','NTMP KX-29','STOCK BOY','Reentry Test',1682,'',1800 from dual union all
select 12445,'1965-S353','1965-06-12 15:15:00','suborbital rocket','success',549,'Sandia 152-80','LRL BOX-1','','','XR Astron',855,'',172 from dual union all
select 12446,'1965-S354','1965-06-12 19:14:00','suborbital rocket','success',227,'FF-VB-01','Test 1','','','Test',428,'',378 from dual union all
select 12447,'1965-S355','1965-06-14 09:13:30','suborbital rocket','success',837,'NASA 14.245UI','UIUC-GCA D/E-4a','','','Ionos',1821,'',177 from dual union all
select 12448,'1965-S356','1965-06-14 13:31:30','military missile','success',1420,'B-22','Mk 6 RV','OT','GOLD FISH','OT',1658,'',1300 from dual union all
select 12449,'1965-S357','1965-06-15 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 620',1000 from dual union all
select 1245,'1952-A181','1952-10-09 00:00:00','atmospheric rocket','success',836,'1249A-127','Nike 1249 R&D 120R','','','Test',1895,'',10 from dual union all
select 12450,'1965-S358','1965-06-15 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 620',1000 from dual union all
select 12451,'1965-S359','1965-06-15 00:00:00','suborbital rocket','success',1301,'TACITE 01','IR Horizon Study','','','Aeron',307,'',182 from dual union all
select 12452,'1965-S36','1965-01-13 23:48:00','suborbital rocket','success',30,'NASA 04.111NA','Airglow','','','Aeronomy',1821,'',178 from dual union all
select 12453,'1965-S360','1965-06-16 00:00:00','military missile','success',916,'AO-27','P-93','','','OT',597,'',180 from dual union all
select 12454,'1965-S361','1965-06-16 13:56:04','military missile','success',928,'C2X-2','STAFF 2','','','Test',251,'',650 from dual union all
select 12455,'1965-S362','1965-06-17 21:07:00','suborbital rocket','success',30,'NASA 04.140GE','CR fluxes','','','Magnetospheric',428,'',233 from dual union all
select 12456,'1965-S363','1965-06-17 21:41:00','suborbital rocket','success',837,'NASA 14.246UI','UIUC-GCA D/E-4b','','','Ionos',1821,'',177 from dual union all
select 12457,'1965-S364','1965-06-18 08:06:00','suborbital rocket','unknown',545,'','','','','Aeron',855,'',100 from dual union all
select 12458,'1965-S365','1965-06-18 16:12:00','suborbital rocket','success',227,'FF-VB-02','Test 2','','','Test',428,'',364 from dual union all
select 12459,'1965-S366','1965-06-18 17:56:00','suborbital rocket','success',837,'NASA 14.215AI','Topside Sounder','','','Ionos',1821,'',186 from dual union all
select 1246,'1952-A182','1952-10-10 00:00:00','atmospheric rocket','success',836,'1249A-128','Nike 1249 R&D 121R','','','Test',1895,'',10 from dual union all
select 12460,'1965-S367','1965-06-18 23:11:00','suborbital rocket','success',44,'NASA 17.01GT','','','','Test',1821,'',374 from dual union all
select 12461,'1965-S368','1965-06-22 00:00:00','suborbital rocket','failure',288,'C106','Cs','','','Aeronomy',664,'',4 from dual union all
select 12462,'1965-S369','1965-06-22 04:15:00','military missile','failure',135,'B002','USN subscale Mk 2 RV','ABRES Q-2 (USN)','','Reentry Test',614,'',200 from dual union all
select 12463,'1965-S37','1965-01-14 12:00:00','military missile','success',1419,'SM-33','Mk 4 RV','ST/ABRES','WEST WIND III','OT',1656,'',1000 from dual union all
select 12464,'1965-S370','1965-06-22 06:00:00','suborbital rocket','unknown',545,'','','','','Aeron',855,'',100 from dual union all
select 12465,'1965-S371','1965-06-22 07:37:00','military missile','success',135,'B001','0.6-scale USN Mk 2 RV','ABRES Q-1 (USN)','','Reentry Test',613,'',200 from dual union all
select 12466,'1965-S372','1965-06-23 00:00:00','military missile','success',795,'1033','Mk 11 RV','OT','WHITE GLOVE','OT',1705,'',900 from dual union all
select 12467,'1965-S373','1965-06-23 00:00:00','suborbital rocket','success',288,'C61','Na','','','Aeronomy',664,'',150 from dual union all
select 12468,'1965-S374','1965-06-23 00:00:00','suborbital rocket','success',288,'C65','Na','','','Aeronomy',313,'',150 from dual union all
select 12469,'1965-S375','1965-06-23 00:00:00','suborbital rocket','success',288,'C50','TMA','','','Aeronomy',664,'',150 from dual union all
select 1247,'1952-A183','1952-10-14 00:00:00','atmospheric rocket','success',836,'1249A-129','Nike 1249 R&D 122R','','','Test',1895,'',10 from dual union all
select 12470,'1965-S376','1965-06-23 00:00:00','suborbital rocket','failure',288,'C95','TMA','','','Aeronomy',313,'',150 from dual union all
select 12471,'1965-S377','1965-06-23 02:03:00','suborbital rocket','success',837,'NASA 14.72CM','GCA Release 46 Na','','','Aeronomy/Na',1821,'',204 from dual union all
select 12472,'1965-S378','1965-06-23 01:49:00','suborbital rocket','success',837,'NASA 14.71CM','GCA Release 45 Na','','','Aeronomy/Na',1821,'',191 from dual union all
select 12473,'1965-S379','1965-06-23 04:00:00','suborbital rocket','success',837,'NASA 14.201CA','GCA Ionos-TMA 5','','','Aeronomy/Na',1821,'',164 from dual union all
select 12474,'1965-S38','1965-01-15 00:00:00','military missile','success',135,'C006','Mk 11 decoy','ABRES B-7 (USAF)','','Reentry Test',613,'',200 from dual union all
select 12475,'1965-S380','1965-06-23 08:47:00','suborbital rocket','success',837,'NASA 14.73CM','GCA Release 47 Na','','','Aeronomy',1821,'',196 from dual union all
select 12476,'1965-S381','1965-06-23 08:52:00','suborbital rocket','success',837,'NASA 14.74CM','GCA Ionos-TMA 6','','','Aeronomy',1821,'',143 from dual union all
select 12477,'1965-S382','1965-06-23 14:32:00','suborbital rocket','success',30,'KP3.10','Soft X solar photo','','','Solar X',1869,'',186 from dual union all
select 12478,'1965-S383','1965-06-23 15:47:00','suborbital rocket','success',30,'NASA 04.141GE','CR fluxes','','','Magnetospheric',428,'',215 from dual union all
select 12479,'1965-S384','1965-06-24 00:00:00','suborbital rocket','failure',288,'C84','TMA','','','Aeronomy',313,'',150 from dual union all
select 1248,'1952-A184','1952-10-15 00:00:00','atmospheric rocket','unknown',576,'56','','','','Test',1895,'',10 from dual union all
select 12480,'1965-S385','1965-06-24 00:00:00','suborbital rocket','failure',288,'C74','TMA','','','Aeronomy',664,'',4 from dual union all
select 12481,'1965-S386','1965-06-24 00:00:00','suborbital rocket','success',288,'C86','','','','Aeronomy',1314,'',150 from dual union all
select 12482,'1965-S387','1965-06-24 00:00:00','suborbital rocket','success',288,'C83','Na','','','Aeronomy',313,'',150 from dual union all
select 12483,'1965-S388','1965-06-24 06:07:00','suborbital rocket','unknown',545,'','','','','Aeron',855,'',100 from dual union all
select 12484,'1965-S389','1965-06-29 12:00:00','military missile','success',795,'1148','Mk 11 RV','OT','MAPLE GROVE','OT',1703,'',900 from dual union all
select 12485,'1965-S39','1965-01-15 15:00:03','military missile','success',983,'KY-2','','','','Test',636,'',250 from dual union all
select 12486,'1965-S390','1965-06-29 00:00:00','suborbital rocket','success',850,'','Composition','','','Aeron',394,'',130 from dual union all
select 12487,'1965-S391','1965-06-29 17:45:00','suborbital rocket','success',548,'DASA 6.4','','','','Aeron?',394,'',300 from dual union all
select 12488,'1965-S392','1965-06-29 18:21:17','suborbital spaceplane','success',1520,'3-44-67','LaRC Scan,Radiom,BLN','','','Test/Aeron',379,'NB-52 008',85 from dual union all
select 12489,'1965-S393','1965-06-29 19:00:00','suborbital rocket','success',30,'NASA 04.112NA','Airglow','','','Aeronomy',1869,'',195 from dual union all
select 1249,'1952-A185','1952-10-15 00:00:00','atmospheric rocket','unknown',576,'57','','','','Test',1895,'',10 from dual union all
select 12490,'1965-S394','1965-06-05 15:00:00','suborbital rocket','success',767,'116','2C XI   GRACCHUS','','','Test',185,'',115 from dual union all
select 12491,'1965-S395','1965-07-22 00:00:00','military missile','success',837,'Capache','NICAP 018','','','Aeron?',1895,'',100 from dual union all
select 12492,'1965-S396','1965-07-22 00:00:00','military missile','success',837,'Capache','NICAP 025','','','Aeron?',1895,'',100 from dual union all
select 12493,'1965-S397','1965-07-22 00:00:00','military missile','success',837,'Capache','NICAP 026','','','Aeron?',1895,'',100 from dual union all
select 12494,'1965-S398','1965-07-14 00:00:00','military missile','success',839,'Capache','NICAP/1 015','','','Aeron?',1895,'',100 from dual union all
select 12495,'1965-S399','1965-07-15 00:00:00','military missile','success',839,'','NICAP/1 014','','','Aeron?',1895,'',100 from dual union all
select 12496,'1965-S40','1965-01-20 00:00:00','military missile','success',795,'777','Mk 11 RV','OT','PURPLE LIGHT','OT',1698,'',900 from dual union all
select 12497,'1965-S400','1965-07-15 00:00:00','military missile','success',839,'','NICAP/1 016','','','Aeron?',1895,'',100 from dual union all
select 12498,'1965-S401','1965-07-15 00:00:00','military missile','success',839,'','NICAP/1 017','','','Aeron?',1895,'',100 from dual union all
select 12499,'1965-S402','1965-07-16 00:00:00','military missile','success',839,'','NICAP/1 019','','','Aeron?',1895,'',100 from dual union all
select 125,'1944-S113','1944-09-18 09:58:00','military missile','success',8,'18755','Ma169','','','Training',661,'',90 from dual union all
select 1250,'1952-A186','1952-10-17 00:00:00','atmospheric rocket','unknown',576,'58','','','','Test',1895,'',10 from dual union all
select 12500,'1965-S403','1965-07-16 00:00:00','military missile','success',839,'','NICAP/1 020','','','Aeron?',1895,'',100 from dual union all
select 12501,'1965-S404','1965-07-16 00:00:00','military missile','success',839,'','NICAP/1 021','','','Aeron?',1895,'',100 from dual union all
select 12502,'1965-S405','1965-07-21 00:00:00','military missile','success',839,'Capache','NICAP 024','','','Aeron?',1895,'',100 from dual union all
select 12503,'1965-S406','1965-07-23 00:00:00','military missile','success',839,'Capache','NICAP 027','','','Aeron?',1895,'',100 from dual union all
select 12504,'1965-S407','1965-07-23 00:00:00','military missile','success',839,'Capache','NICAP 028','','','Aeron?',1895,'',100 from dual union all
select 12505,'1965-S408','1965-07-20 00:00:00','military missile','success',839,'Capache','NICAP 022','','','Aeron?',1895,'',100 from dual union all
select 12506,'1965-S409','1965-07-20 00:00:00','military missile','success',839,'Capache','NICAP 023','','','Aeron?',1895,'',100 from dual union all
select 12507,'1965-S41','1965-01-20 00:00:00','suborbital rocket','success',1431,'Sandia 152-14','Magnus Stovepipe','','','Test/Aeron?',1490,'',106 from dual union all
select 12508,'1965-S410','1965-06-01 00:00:00','military missile','success',874,'ZK-57 20170','DM-15S2 Qual-1','','','ASAT Test',873,'',100 from dual union all
select 12509,'1965-S411','1965-07-08 00:00:00','military missile','success',874,'ZK-58 20173','DM-15S2 Qual-2','','','ASAT Test',873,'',100 from dual union all
select 1251,'1952-A187','1952-10-17 00:00:00','atmospheric rocket','unknown',576,'59','','','','Test',1895,'',10 from dual union all
select 12510,'1965-S412','1965-06-30 05:33:00','sounding rocket','success',630,'NASA 11.03UR','','','','RF Astron',1816,'',1770 from dual union all
select 12511,'1965-S413','1965-06-30 14:29:30','military missile','success',1420,'B-30','Mk 6 RV','OT','BUSY BEE','OT',1659,'',1300 from dual union all
select 12512,'1965-S414','1965-06-30 14:30:00','suborbital rocket','success',30,'NASA 04.105NP','JPL Rocket Radar 3','','','Tech',1869,'',158 from dual union all
select 12513,'1965-S415','1965-07-01 01:45:00','suborbital rocket','success',1199,'SL106','LUX-28/UCL I-22','','','Ionos/Solar X',1839,'',130 from dual union all
select 12514,'1965-S416','1965-07-01 09:54:11','military missile','success',154,'59D','Nike RMV-301','NTMP KX-32','BLIND SPOT','Reentry Test',1683,'',1800 from dual union all
select 12515,'1965-S417','1965-07-02 00:00:00','military missile','success',795,'799','Mk 11A RV','OT','SWEET TALK','OT',1698,'',900 from dual union all
select 12516,'1965-S418','1965-07-05 00:00:00','military missile','success',1117,'P1','','','','Reentry Test',669,'',1000 from dual union all
select 12517,'1965-S419','1965-07-06 00:00:00','military missile','success',795,'1069','Mk 11 RV','OT','STAR DUST','OT',1702,'',900 from dual;
insert into LAUNCH
select 12518,'1965-S42','1965-01-21 20:30:00','suborbital rocket','success',30,'NA3.163','','','','Solar EUV/X',1869,'',235 from dual union all
select 12519,'1965-S420','1965-07-07 07:25:00','suborbital rocket','success',30,'AE3.117/VFT65-S-1','Noctilucent clouds','','','Aeron',428,'',166 from dual union all
select 1252,'1952-A188','1952-10-17 00:00:00','atmospheric rocket','success',836,'1249A-130','Nike 1249 R&D 123R','','','Test',1895,'',10 from dual union all
select 12520,'1965-S421','1965-07-07 08:37:00','suborbital rocket','success',535,'HAD 174','Sphere','','','Aeron',1835,'',105 from dual union all
select 12521,'1965-S422','1965-07-09 00:00:00','military missile','success',999,'R 22500-27L','8K67 LKI-22','','','Test',511,'',1000 from dual union all
select 12522,'1965-S423','1965-07-14 11:08:00','suborbital rocket','success',1199,'SL48','UCL UV-4','','','UV Astronomy',1839,'',182 from dual union all
select 12523,'1965-S424','1965-07-15 20:50:00','suborbital rocket','success',30,'NASA 04.128UA','Thermosphere comp.','','','Aeronomy',1869,'',196 from dual union all
select 12524,'1965-S425','1965-07-16 00:00:00','military missile','success',135,'B003','USN subscale Mk 2 RV','ABRES Q-3 (USN)','','Reentry Test',612,'',200 from dual union all
select 12525,'1965-S426','1965-07-16 10:50:00','suborbital rocket','success',665,'K-8-12','','','','Aeron/Ionos',839,'',160 from dual union all
select 12526,'1965-S427','1965-07-21 04:12:00','military missile','success',982,'G 22601-03T','8K64U UBP-58','','','OT',570,'',1210 from dual union all
select 12527,'1965-S428','1965-07-21 10:33:00','suborbital rocket','success',848,'AE07.347','Sphere/Vehicle test','','','Test/Aeron',394,'',178 from dual union all
select 12528,'1965-S429','1965-07-21 18:18:46','military missile','success',1420,'B-62','Mk 6 RV','OT','LONG BALL','OT',1657,'',1300 from dual union all
select 12529,'1965-S43','1965-01-22 05:14:00','suborbital rocket','success',668,'K-9M-3','','','','Ionos?',839,'',350 from dual union all
select 1253,'1952-A189','1952-10-21 00:00:00','atmospheric rocket','success',836,'1249A-131','Nike 1249 R&D 124R','','','Test',1895,'',10 from dual union all
select 12530,'1965-S430','1965-07-21 20:00:00','suborbital rocket','success',30,'CRL AD3.375','Mass spec/Imp. probe','','','Ionos',1869,'',193 from dual union all
select 12531,'1965-S431','1965-07-23 17:05:00','suborbital rocket','success',839,'NASA 10.128GM','GSFC Grenade 77','','','Aeronomy',1821,'',116 from dual union all
select 12532,'1965-S432','1965-07-24 08:36:00','suborbital rocket','success',30,'AF3.266/VFT65-S-3','Noctilucent clouds','','','Aeron',428,'',179 from dual union all
select 12533,'1965-S433','1965-07-25 00:00:00','suborbital rocket','success',1431,'Sandia 152-15','Dart/EMP  [or 1966]','','','Test',855,'',90 from dual union all
select 12534,'1965-S434','1965-07-26 00:00:00','military missile','success',135,'C011','Chaff Payload E-2','ABRES E-2 (USAF)','','Reentry Test',613,'',200 from dual union all
select 12535,'1965-S435','1965-07-26 12:01:00','suborbital rocket','success',668,'K-9M-12','','','','Opt Astron/XR Astron',839,'',350 from dual union all
select 12536,'1965-S436','1965-07-26 17:59:38','military missile','success',243,'BK.23','H-207 II/Dazzle 4','','','Reentry Test',1842,'',492 from dual union all
select 12537,'1965-S437','1965-07-27 03:10:00','suborbital rocket','success',668,'K-9M-13','','','','Ionos',839,'',317 from dual union all
select 12538,'1965-S438','1965-08-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeron?',1895,'',100 from dual union all
select 12539,'1965-S439','1965-07-31 00:00:00','military missile','success',135,'D004','Small Blunt RV','ABRES P-6 (USAF)','','Reentry Test',613,'',200 from dual union all
select 1254,'1952-A19','1952-01-30 00:00:00','suborbital rocket','success',1496,'R8','','','','Test',218,'',1 from dual union all
select 12540,'1965-S44','1965-01-22 21:46:00','suborbital rocket','success',219,'NRC FF-4-03','','','','Test',428,'',670 from dual union all
select 12541,'1965-S440','1965-08-03 04:47:00','military missile','success',135,'C014','Chaff Payload E-3','ABRES E-3 (USAF)','','Reentry Test',614,'',170 from dual union all
select 12542,'1965-S441','1965-08-03 08:52:00','suborbital rocket','success',535,'HAD 177','Sphere','','','Aeron',1835,'',115 from dual union all
select 12543,'1965-S442','1965-08-04 03:41:10','military missile','success',796,'457','Mk 11 RV','R&D','','Test',266,'',900 from dual union all
select 12544,'1965-S443','1965-08-04 12:43:08','military missile','success',154,'183D','WAC-5','ABRES WAC-5','PIANO WIRE','Reentry Test',1683,'',1800 from dual union all
select 12545,'1965-S444','1965-08-05 13:21:28','military missile','success',162,'147F','LORV L-2/GTV5','ABRES LORV-2A','SEA TRAMP','Reentry Test',1681,'',1400 from dual union all
select 12546,'1965-S445','1965-08-07 00:00:00','suborbital rocket','success',665,'K-8(L)','','','','Aeron',932,'',100 from dual union all
select 12547,'1965-S446','1965-08-07 00:00:00','military missile','success',135,'C005','Mk 11 decoy','ABRES B-6 (USAF)','','Reentry Test',613,'',200 from dual union all
select 12548,'1965-S447','1965-08-07 00:00:00','military missile','failure',135,'C009','Chaff Payload E-4','ABRES E-4 (USAF)','','Reentry Test',612,'',200 from dual union all
select 12549,'1965-S448','1965-08-07 11:13:00','suborbital rocket','success',839,'NASA 10.151GM','GSFC Grenade 78','','','Aeronomy',1039,'',120 from dual union all
select 1255,'1952-A190','1952-10-21 00:00:00','atmospheric rocket','success',836,'1249A-132','Nike 1249 R&D 125R','','','Test',1895,'',10 from dual union all
select 12550,'1965-S449','1965-08-07 12:00:00','suborbital rocket','success',839,'NASA 10.96GM','GSFC Grenade 79','','','Aeronomy',428,'',127 from dual union all
select 12551,'1965-S45','1965-01-23 16:30:08','military missile','success',932,'A3P-157','','','','OT',421,'SSBN 635',1000 from dual union all
select 12552,'1965-S450','1965-08-07 18:30:00','suborbital rocket','success',839,'NASA 10.154UA','UM Sphere','','','Aeronomy',1821,'',167 from dual union all
select 12553,'1965-S451','1965-08-07 19:39:00','suborbital rocket','success',839,'NASA 10.162GM','GSFC Grenade 80','','','Aeronomy',1039,'',127 from dual union all
select 12554,'1965-S452','1965-08-07 19:45:00','suborbital rocket','success',839,'NASA 10.165GM','GSFC Grenade 81','','','Aeronomy',428,'',118 from dual union all
select 12555,'1965-S453','1965-08-07 20:06:00','suborbital rocket','success',839,'NASA 10.168GM','Grenades/UM Sphere','','','Aeronomy',1821,'',120 from dual union all
select 12556,'1965-S454','1965-08-08 03:40:00','suborbital rocket','success',839,'NASA 10.169GM','Grenades/UM Sphere','','','Aeronomy',1821,'',122 from dual union all
select 12557,'1965-S455','1965-08-08 04:00:00','suborbital rocket','success',839,'NASA 10.166GM','GSFC Grenade 82 C','','','Aeronomy',428,'',116 from dual union all
select 12558,'1965-S456','1965-08-08 04:15:00','suborbital rocket','success',839,'NASA 10.163GM','GSFC Grenade 83 PB','','','Aeronomy',1039,'',124 from dual union all
select 12559,'1965-S457','1965-08-08 06:52:00','suborbital rocket','success',839,'NASA 10.157UA','UM Sphere','','','Aeronomy',1821,'',165 from dual union all
select 1256,'1952-A191','1952-10-24 00:00:00','atmospheric rocket','success',836,'1249A-133','Nike 1249 R&D 126R','','','Test',1895,'',10 from dual union all
select 12560,'1965-S458','1965-08-08 10:02:00','suborbital rocket','success',839,'NASA 10.167GM','GSFC Grenade 84 C','','','Aeronomy',428,'',125 from dual union all
select 12561,'1965-S459','1965-08-08 10:15:00','suborbital rocket','success',839,'NASA 10.170GM','Grenades/UM Sphere','','','Aeronomy',1821,'',120 from dual union all
select 12562,'1965-S46','1965-01-23 00:00:00','military missile','success',979,'','8K65 UBP-58','','','Test',636,'',675 from dual union all
select 12563,'1965-S460','1965-08-09 10:10:00','suborbital rocket','success',839,'NASA 10.164GM','GSFC Grenade 85 PB','','','Aeronomy',1039,'',124 from dual union all
select 12564,'1965-S461','1965-08-10 19:24:21','suborbital spaceplane','success',1520,'3-46-70','Northrop Scan,BLN','','','Test',379,'NB-52 003',82 from dual union all
select 12565,'1965-S462','1965-08-10 23:13:00','suborbital rocket','success',1199,'ESRO S05/1','ESRO S05/1 ROE-3','','','UV Astronomy',1343,'',215 from dual union all
select 12566,'1965-S463','1965-08-11 19:01:01','military missile','success',928,'C2X-3','STAFF 3','','','Test',251,'',500 from dual union all
select 12567,'1965-S464','1965-08-11 01:35:00','suborbital rocket','success',665,'K-8(L)-4','SK-8-4','','','Aeron/Ionos/Fields',932,'',191 from dual union all
select 12568,'1965-S465','1965-08-11 11:49:00','suborbital rocket','success',839,'NASA 10.144GM','Meteoroids','','','Aeronomy/Meteorites',1895,'',129 from dual union all
select 12569,'1965-S466','1965-08-12 17:15:00','suborbital rocket','success',30,'CRL AE3.519','EUV Mon','','','Solar EUV',1869,'',241 from dual union all
select 1257,'1952-A192','1952-10-24 00:00:00','atmospheric rocket','success',836,'1249A-134','Nike 1249 R&D 127R','','','Test',1895,'',10 from dual union all
select 12570,'1965-S467','1965-08-13 09:45:00','military missile','success',999,'R 22500-26L','8K67 LKI-24','','','Test',511,'',1000 from dual union all
select 12571,'1965-S468','1965-08-16 20:04:56','military missile','success',1420,'B-6','Mk 6 RV','OT','MAGIC LAMP','OT',1658,'',1300 from dual union all
select 12572,'1965-S469','1965-08-17 00:05:00','suborbital rocket','success',665,'K-8(L)','SK-8-3 (Grenades)','','','Aeron',932,'',100 from dual union all
select 12573,'1965-S47','1965-01-23 18:15:00','suborbital rocket','success',219,'NRC FF-4-04','','','','Test',428,'',735 from dual union all
select 12574,'1965-S470','1965-08-18 11:30:00','military missile','success',796,'2001','Mk 11A RV','R&D','REBEL RANGER','Test',1707,'',900 from dual union all
select 12575,'1965-S471','1965-08-19 01:40:00','suborbital rocket','success',837,'NASA 14.133NA','Airglow','','','Aeronomy',1821,'',144 from dual union all
select 12576,'1965-S472','1965-08-20 00:00:00','military missile','success',135,'C008','Mk 11 decoy','ABRES B-8 (USAF)','','Reentry Test',614,'',200 from dual union all
select 12577,'1965-S473','1965-08-23 20:00:27','military missile','success',796,'454','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 12578,'1965-S474','1965-08-24 00:00:00','suborbital rocket','success',446,'D-17','FU-146/VLF','','','Aeronomy/Ionos',1749,'',440 from dual union all
select 12579,'1965-S475','1965-08-24 16:37:28','military missile','success',794,'677','Mk 5 RV','FOT-1','SHUTTLE TRAIN','Test',1701,'',1000 from dual union all
select 1258,'1952-A193','1952-10-28 00:00:00','atmospheric rocket','success',836,'1249A-135','Nike 1249 R&D 128R','','','Test',1895,'',10 from dual union all
select 12580,'1965-S476','1965-08-24 21:59:00','suborbital rocket','success',837,'NASA 14.210GI','US/Brazil Test','','','Ionos',1821,'',162 from dual union all
select 12581,'1965-S477','1965-08-25 16:28:07','military missile','failure',794,'509','Mk 5 RV','FOT-2','PILOT ROCK','OT',1699,'',500 from dual union all
select 12582,'1965-S478','1965-08-25 18:31:00','suborbital rocket','success',624,'NASA 8.11UA','He ionization','','','Aeronomy',1821,'',902 from dual union all
select 12583,'1965-S479','1965-08-26 00:04:43','military missile','success',1420,'B-19','Mk 6 RV','OT','NEW ROLE','OT',1659,'',1300 from dual union all
select 12584,'1965-S48','1965-01-25 16:00:00','suborbital rocket','success',216,'CC-2B-48','','','','Test',428,'',230 from dual union all
select 12585,'1965-S480','1965-08-26 11:20:14','military missile','success',154,'61D','Nike RMV-304','NTMP KX-41','TONTO RIM','Reentry Test',1684,'',1800 from dual union all
select 12586,'1965-S481','1965-08-26 19:03:00','suborbital rocket','success',30,'CRL AE3.614','Gyro Resonance','','','Ionos',394,'',109 from dual union all
select 12587,'1965-S482','1965-08-27 00:00:00','military missile','success',135,'D012','ABRES P-9 (USAF)','ABRES P-9 (USAF)','','Reentry Test',613,'',200 from dual union all
select 12588,'1965-S483','1965-08-28 02:02:00','suborbital rocket','success',657,'K-10S-1','Te probe','','','Ionos',839,'',742 from dual union all
select 12589,'1965-S484','1965-08-31 05:31:00','military missile','success',982,'U 22602-20T','8K64U UBP-59 K','','','OT',570,'',1210 from dual union all
select 1259,'1952-A194','1952-10-28 00:00:00','atmospheric rocket','success',836,'1249A-136','Nike 1249 R&D 129R','','','Test',1895,'',10 from dual union all
select 12590,'1965-S485','1965-09-01 00:00:00','military missile','success',1036,'','','','','Test',624,'',1000 from dual union all
select 12591,'1965-S486','1965-09-01 11:17:00','suborbital rocket','success',837,'NASA 14.213UI','MIP 1 (day)','','','Ionos',1821,'',150 from dual union all
select 12592,'1965-S487','1965-09-03 00:00:00','suborbital rocket','success',446,'D-18','FU-146/VLF','','','Aeronomy/Ionos',1749,'',451 from dual union all
select 12593,'1965-S488','1965-09-03 05:17:00','suborbital rocket','success',837,'NASA 14.214UI','MIP 2 (night)','','','Ionos',1821,'',153 from dual union all
select 12594,'1965-S489','1965-09-03 19:30:00','suborbital rocket','success',740,'','','','','Aeron',636,'',165 from dual union all
select 12595,'1965-S49','1965-01-26 09:40:26','suborbital rocket','success',589,'SAP-2','LRL SAP-2 (SAA)','','','Ionos/Mag',160,'T-AGM-8',289 from dual union all
select 12596,'1965-S490','1965-09-09 04:15:00','suborbital rocket','success',740,'','','','','Aeron',636,'',165 from dual union all
select 12597,'1965-S491','1965-09-09 04:20:00','suborbital rocket','success',1206,'SL88B','RAE Test 30','','','Test',1839,'',221 from dual union all
select 12598,'1965-S492','1965-09-09 23:58:00','suborbital rocket','success',837,'NASA 14.75GR','RF propagation','','','RF Astron',1821,'',171 from dual union all
select 12599,'1965-S493','1965-09-14 05:45:00','military missile','success',135,'B004','WIRE Ionizing Pellets','ABRES N-1 (USN)','','Reentry Test',612,'',200 from dual union all
select 126,'1944-S114','1944-09-18 10:48:00','military missile','success',8,'18747','Ma164','','','Training',661,'',90 from dual union all
select 1260,'1952-A195','1952-10-28 00:00:00','military missile','success',1302,'','X-SAM-N-6','','','Test',1895,'',5 from dual union all
select 12600,'1965-S494','1965-09-14 07:45:00','military missile','success',135,'D033','Subscale Low Obs RVs','ABRES C-3 (USAF)','','Reentry Test',614,'',200 from dual union all
select 12601,'1965-S495','1965-09-15 20:28:00','suborbital rocket','success',837,'NASA 14.244UI','UIUC-GCA D/E-5','','','Ionos',1821,'',180 from dual union all
select 12602,'1965-S496','1965-09-16 23:05:00','suborbital rocket','success',227,'VB-03','CRL AE18.315','','','Test',394,'',344 from dual union all
select 12603,'1965-S497','1965-09-17 15:44:00','suborbital rocket','success',837,'NASA 14.235UE','Auroral electrons','','','Aurora',428,'',160 from dual union all
select 12604,'1965-S498','1965-09-17 19:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',165 from dual union all
select 12605,'1965-S499','1965-09-17 21:51:00','suborbital rocket','success',35,'NASA 04.138II','FR-1 test','','','Ionos',1821,'',182 from dual union all
select 12606,'1965-S50','1965-01-27 00:00:00','suborbital rocket','success',866,'Sandia 152-23','Ballistic (TTR)','','','Test',855,'',302 from dual union all
select 12607,'1965-S500','1965-09-18 02:55:00','military missile','failure',982,'G 22601-07T','8K64U UBP-60 PBG','','','OT',565,'',100 from dual;
insert into LAUNCH
select 12608,'1965-S501','1965-09-18 22:02:00','suborbital rocket','success',837,'NASA 14.224IA','Na release','','','Aeronomy',332,'',205 from dual union all
select 12609,'1965-S502','1965-09-20 00:00:00','military missile','success',135,'D006','ABRES P-7 (USAF)','ABRES P-7 (USAF)','','Reentry Test',613,'',200 from dual union all
select 1261,'1952-A196','1952-10-29 00:00:00','atmospheric rocket','unknown',576,'60','','','','Test',1895,'',10 from dual union all
select 12610,'1965-S503','1965-09-20 03:27:00','suborbital rocket','success',1014,'','R-5V Geophysical','','','Test/Ionos/Solar X',636,'',480 from dual union all
select 12611,'1965-S504','1965-09-20 16:08:00','suborbital rocket','success',837,'NASA 14.236UE','Auroral electrons','','','Aurora',428,'',146 from dual union all
select 12612,'1965-S505','1965-09-21 14:04:23','military missile','success',1420,'B-58','Mk 6 RV','OT','BOLD GUY','OT',1657,'',1300 from dual union all
select 12613,'1965-S506','1965-09-21 22:00:00','suborbital rocket','success',837,'NASA 14.225IA','Na release','','','Aeronomy',332,'',205 from dual union all
select 12614,'1965-S507','1965-09-22 20:07:00','suborbital rocket','success',30,'NASA 04.147CG','AS/E','','','XR Astron',1869,'',199 from dual union all
select 12615,'1965-S508','1965-09-23 14:36:00','suborbital rocket','success',624,'NASA 8.36GI','Alouette underflight','','','Ionos',1821,'',805 from dual union all
select 12616,'1965-S51','1965-01-27 21:32:00','suborbital rocket','success',839,'NASA 10.124GA','GSFC Grenade 66 (PB)','','','Aeronomy',1039,'',100 from dual union all
select 12617,'1965-S510','1965-09-25 02:20:13','military missile','success',796,'458','Mk 11 RV','R&D','','Test',266,'',900 from dual union all
select 12618,'1965-S511','1965-09-24 08:52:00','suborbital rocket','success',837,'NASA 14.226IA','Na release','','','Aeronomy',332,'',205 from dual union all
select 12619,'1965-S512','1965-09-25 05:59:00','military missile','success',982,'T 22601-09T','8K64U UBP-61','','','OT',565,'',1210 from dual union all
select 1262,'1952-A197','1952-10-29 00:00:00','atmospheric rocket','unknown',576,'61','','','','Test',1895,'',10 from dual union all
select 12620,'1965-S513','1965-09-25 04:00:00','suborbital rocket','success',30,'CRL AD3.722','IR Horizon 1','','','Test',1869,'',199 from dual union all
select 12621,'1965-S514','1965-09-25 21:54:00','suborbital rocket','success',35,'NASA 04.139II','FR-1 test','','','Ionos',1821,'',192 from dual union all
select 12622,'1965-S515','1965-09-27 00:00:00','military missile','success',135,'C012','MARCAS','ABRES W-1 (USAF)','','Reentry Test',612,'',200 from dual union all
select 12623,'1965-S516','1965-09-27 08:54:00','suborbital rocket','success',837,'NASA 14.227IA','Na release','','','Aeronomy',332,'',205 from dual union all
select 12624,'1965-S517','1965-09-28 09:09:58','suborbital rocket','success',30,'NASA 04.150GA-GI-GB','Meteors/Ionos','','','Meteorites/Biology',1870,'',183 from dual union all
select 12625,'1965-S518','1965-09-28 14:35:00','military missile','success',243,'BK.24','H-208 V/Dazzle 5','','','Reentry Test',1842,'',605 from dual union all
select 12626,'1965-S519','1965-09-28 18:07:37','suborbital spaceplane','success',1520,'3-49-73','N.Scan/BLN/Tail','','','Test',379,'NB-52 003',90 from dual union all
select 12627,'1965-S52','1965-01-27 22:24:00','suborbital rocket','success',839,'NASA 10.118GA','GSFC Grenade 67 (W)','','','Aeronomy',1821,'',117 from dual union all
select 12628,'1965-S520','1965-09-29 00:00:00','suborbital rocket','success',30,'','','','','Aeron',1869,'',200 from dual union all
select 12629,'1965-S521','1965-09-29 10:40:00','military missile','success',154,'125D','Nike RMV-303','NTMP KX-45','WATER SNAKE','Reentry Test',1683,'',1800 from dual union all
select 1263,'1952-A198','1952-10-30 00:00:00','atmospheric rocket','success',442,'','B-58 Model','','','Test',1821,'',10 from dual union all
select 12630,'1965-S522','1965-10-20 00:00:00','suborbital rocket','success',893,'No. 1','','','','Test',310,'',90 from dual union all
select 12631,'1965-S523','1965-10-10 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 12632,'1965-S524','1965-10-06 00:00:00','military missile','success',916,'AO-28','P-94','','','OT',597,'',180 from dual union all
select 12633,'1965-S528','1965-09-30 07:30:00','military missile','success',982,'T 22602-09T','8K64U UBP-62','','','OT',565,'',1210 from dual union all
select 12634,'1965-S529','1965-10-01 00:00:00','military missile','success',1035,'RT-1-1963','','','','Test',636,'',500 from dual union all
select 12635,'1965-S53','1965-01-28 22:46:00','military missile','success',916,'AO-20','P-86','','','OT',437,'',180 from dual union all
select 12636,'1965-S530','1965-09-30 04:20:00','suborbital rocket','success',1199,'ESRO S04/1 (SEO8)','ESRO S04/1 MPE Ba-12','','','Aeronomy grenades',1343,'',225 from dual union all
select 12637,'1965-S531','1965-09-30 04:20:00','sounding rocket','success',1068,'R8','Rubis 02','','','RF Astron/Particles',664,'',1760 from dual union all
select 12638,'1965-S532','1965-10-02 00:59:59','military missile','success',796,'459','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 12639,'1965-S533','1965-10-01 04:40:00','military missile','success',999,'R 22500-24L','8K67 LKI-26','','','Test',514,'',1000 from dual union all
select 1264,'1952-A199','1952-10-30 00:00:00','atmospheric rocket','success',836,'1249A-138','Nike 1249 R&D 130R','','','Test',1895,'',10 from dual union all
select 12640,'1965-S534','1965-10-01 00:00:00','military missile','success',135,'B006','WIRE Ionizing Pellets','ABRES N-2 (USN)','','Reentry Test',614,'',200 from dual union all
select 12641,'1965-S535','1965-10-01 01:30:00','suborbital rocket','success',30,'NASA 04.121CG','LPARL 1D XRT','','','XR Astron',1869,'',148 from dual union all
select 12642,'1965-S536','1965-10-01 03:57:00','suborbital rocket','success',1012,'','R-5B Geophysical','','','Ionos/Solar X',636,'',480 from dual union all
select 12643,'1965-S537','1965-10-02 04:22:00','suborbital rocket','success',1199,'ESRO S04/2 (SEO9)','ESRO S04/2 MPE Ba-13','','','Aeronomy grenades',1343,'',250 from dual union all
select 12644,'1965-S539','1965-10-04 03:00:00','suborbital rocket','success',668,'K-9M-14','','','','Ionos',839,'',299 from dual union all
select 12645,'1965-S54','1965-01-28 12:51:10','ballistic missile test','failure',247,'22-3 (OAR Probe 3)','OAR 22-3/AFCRL-309','','','Magnetospheric',235,'',200 from dual union all
select 12646,'1965-S540','1965-10-05 00:42:00','suborbital rocket','success',624,'NASA 8.30UI','M/DTR 3','','','Ionos',1821,'',960 from dual union all
select 12647,'1965-S541','1965-10-05 03:00:00','suborbital rocket','success',30,'NRL NB3.184','','','','UV/XR Astron',1869,'',171 from dual union all
select 12648,'1965-S542','1965-10-05 09:27:00','suborbital rocket','success',535,'HAD 178','Sphere','','','Aeron',1835,'',100 from dual union all
select 12649,'1965-S543','1965-10-05 23:21:00','suborbital rocket','success',837,'NASA 14.202CA','GCA Ionos-TMA 7','','','Aeronomy',428,'',206 from dual union all
select 1265,'1952-A20','1952-01-30 20:45:00','suborbital rocket','failure',51,'USAF 21','AF/Utah Ionos 1','','','Ionos',640,'',0 from dual union all
select 12650,'1965-S544','1965-10-06 00:00:00','military missile','success',796,'2002','Mk 11 RV','R&D','DICE SPOT','Test',1708,'',900 from dual union all
select 12651,'1965-S545','1965-10-06 04:00:00','suborbital rocket','success',666,'K-8L-9','','','','Aeron',839,'',141 from dual union all
select 12652,'1965-S546','1965-10-09 00:00:00','military missile','success',1117,'P3','','','','Reentry Test',669,'',1150 from dual union all
select 12653,'1965-S547','1965-10-09 05:05:00','military missile','success',999,'U 22500-32L','8K67 LKI-27','','','Test',511,'',1000 from dual union all
select 12654,'1965-S548','1965-10-11 00:00:00','military missile','success',135,'C016','MARCAS 1','ABRES W-2 (USAF)','','Reentry Test',613,'',200 from dual union all
select 12655,'1965-S549','1965-10-13 00:00:00','suborbital rocket','success',1014,'','R-5V Geophysical','VAO','','Ionos?',636,'',500 from dual union all
select 12656,'1965-S55','1965-01-27 22:24:00','suborbital rocket','success',839,'NASA 10.121GA','GSFC Grenade 68 (C)','','','Aeronomy',428,'',125 from dual union all
select 12657,'1965-S550','1965-10-13 03:13:00','suborbital rocket','success',30,'NASA 04.151UG','','','','UV Astron',1869,'',183 from dual union all
select 12658,'1965-S551','1965-10-13 16:01:00','suborbital rocket','success',839,'NASA 10.152GM','GSFC Grenade 86 PB','','','Aeronomy',1039,'',100 from dual union all
select 12659,'1965-S552','1965-10-13 16:11:00','suborbital rocket','success',839,'NASA 10.97GM','GSFC Grenade 87 C','','','Aeronomy',428,'',124 from dual union all
select 1266,'1952-A200','1952-10-30 00:00:00','atmospheric rocket','success',836,'1249A-137','Nike 1249 R&D 131R','','','Test',1895,'',10 from dual union all
select 12660,'1965-S553','1965-10-13 16:51:00','suborbital rocket','success',839,'NASA 10.129GM','GSFC Grenade 88/UM Sph','','','Aeronomy',1821,'',110 from dual union all
select 12661,'1965-S554','1965-10-14 20:46:32','suborbital spaceplane','success',1520,'1-61-101','MIT,Pace transducer','','','Test',379,'NB-52 003',81 from dual union all
select 12662,'1965-S555','1965-10-15 04:05:00','military missile','success',999,'R 22500-23L','8K67 LKI-28','','','Test',485,'',1000 from dual union all
select 12663,'1965-S556','1965-10-15 00:00:00','suborbital rocket','success',1498,'001','FU-123','','','Test',668,'',150 from dual union all
select 12664,'1965-S557','1965-10-18 08:30:11','suborbital rocket','success',848,'CRL AE07.618D','Auroral I/O','','','Aeron',447,'',103 from dual union all
select 12665,'1965-S558','1965-10-18 11:23:00','suborbital rocket','success',1199,'SL39','QUB AGP-3/UCL I-24','','','Aeronomy',1839,'',205 from dual union all
select 12666,'1965-S559','1965-10-19 17:30:00','suborbital rocket','success',839,'NASA 10.177GM','GSFC Grenade 89 PB','','','Aeronomy',1039,'',100 from dual union all
select 12667,'1965-S56','1965-01-29 01:21:15','military missile','success',796,'453','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 12668,'1965-S560','1965-10-19 17:30:00','suborbital rocket','success',839,'NASA 10.98GM','GSFC Grenade 90 FC','','','Aeronomy',428,'',124 from dual union all
select 12669,'1965-S561','1965-10-19 20:00:00','suborbital rocket','success',35,'NASA 04.142NA','Colorado/Ikeya-Seki','','','UV Astron',1821,'',179 from dual union all
select 1267,'1952-A201','1952-10-30 00:00:00','military missile','success',763,'Dyn Model #3','','','','Test',222,'',10 from dual union all
select 12670,'1965-S562','1965-10-19 23:10:00','suborbital rocket','success',839,'NASA 10.174GM','GSFC Grenade 91 W','','','Aeronomy',1821,'',114 from dual union all
select 12671,'1965-S563','1965-10-20 05:20:00','suborbital rocket','success',1207,'SL306','LUX-30AC4/Cul-4','','','Solar X,UV',1839,'',214 from dual union all
select 12672,'1965-S565','1965-10-20 18:09:22','military missile','success',1420,'B-33','Mk 6 RV','OT','POWER BOX','OT',1658,'',1300 from dual union all
select 12673,'1965-S566','1965-10-20 20:35:00','suborbital rocket','success',30,'NASA 04.146DS','Comet study/Solar','','','UV Astron/Solar UV',1869,'',184 from dual union all
select 12674,'1965-S567','1965-10-21 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 12675,'1965-S568','1965-10-21 16:00:00','suborbital rocket','success',30,'NASA 04.164UA','JHU Ikeya-Seki','','','UV Astron',1821,'',191 from dual union all
select 12676,'1965-S569','1965-10-22 07:54:28','suborbital rocket','success',848,'CRL AE07.619D','Auroral I/O','','','Ionos',447,'',137 from dual union all
select 12677,'1965-S57','1965-01-30 05:15:00','military missile','success',983,'KY-2','','','','Test',636,'',206 from dual union all
select 12678,'1965-S570','1965-10-22 20:18:00','suborbital rocket','success',1493,'AGI/V54','FU-115 Electron den.','','','Ionos',668,'',210 from dual union all
select 12679,'1965-S571','1965-10-23 05:30:00','suborbital rocket','success',30,'NRL ND3.167','','','','Aeron',1869,'',254 from dual union all
select 1268,'1952-A202','1952-10-31 00:00:00','atmospheric rocket','failure',442,'','Navaho Model','','','Test',1821,'',10 from dual union all
select 12680,'1965-S572','1965-10-23 15:38:00','suborbital rocket','success',839,'NASA 10.178GM','GSFC Grenade 92 PB','','','Aeronomy',1039,'',100 from dual union all
select 12681,'1965-S573','1965-10-23 16:14:00','suborbital rocket','success',839,'NASA 10.175GM','GSFC Grenade 93/UM Sph','','','Aeronomy',1821,'',118 from dual union all
select 12682,'1965-S574','1965-10-23 16:38:00','suborbital rocket','success',839,'NASA 10.172GM','GSFC Grenade 94 C','','','Aeronomy',428,'',114 from dual union all
select 12683,'1965-S575','1965-10-25 00:00:00','suborbital rocket','success',1498,'002','FU-123','','','Test',668,'',105 from dual union all
select 12684,'1965-S576','1965-10-26 12:03:00','suborbital rocket','success',740,'','','','','Aeron/Ionos',636,'',170 from dual union all
select 12685,'1965-S577','1965-10-26 15:25:00','suborbital rocket','success',30,'NASA 04.53GS','','','','Solar',1869,'',195 from dual union all
select 12686,'1965-S578','1965-10-27 23:48:00','suborbital rocket','success',839,'NASA 10.173GM','GSFC Grenade 95 C','','','Aeronomy',428,'',118 from dual union all
select 12687,'1965-S579','1965-10-27 23:48:00','suborbital rocket','success',839,'NASA 10.179GM','GSFC Grenade 96 PB','','','Aeronomy',1039,'',100 from dual union all
select 12688,'1965-S58','1965-01-30 07:45:00','suborbital rocket','success',30,'NF3.162','','','','IR Astron',1869,'',211 from dual union all
select 12689,'1965-S580','1965-10-28 00:10:00','suborbital rocket','success',839,'NASA 10.176GM','GSFC Grenade 97 W','','','Aeronomy',1821,'',117 from dual union all
select 1269,'1952-A203','1952-11-01 00:00:00','atmospheric rocket','success',365,'','Rigel Model','','','Test',1821,'',10 from dual union all
select 12690,'1965-S581','1965-10-28 06:55:00','suborbital rocket','success',1493,'AGI/V55','FU-115 electron den.','','','Ionos',668,'',210 from dual union all
select 12691,'1965-S582','1965-10-28 18:01:50','suborbital rocket','success',589,'BOX-2','LRL BOX-2','','','XR Astron',1063,'T-AGM-8',194 from dual union all
select 12692,'1965-S583','1965-10-29 03:12:00','military missile','success',999,'U 22500-35L','8K67 LKI-29','','','Test',511,'',1000 from dual union all
select 12693,'1965-S584','1965-10-29 11:30:00','suborbital rocket','success',30,'NRL NF3.162R','IR Telescope','','','IR Astron',1869,'',205 from dual union all
select 12694,'1965-S585','1965-11-19 00:00:00','military missile','success',837,'BMTS','SR-019','','','Target',1895,'',100 from dual union all
select 12695,'1965-S586','1965-11-10 00:00:00','military missile','success',837,'BMRS','SPEEDBALL II 79 SN32','','','Target',1895,'',150 from dual union all
select 12696,'1965-S587','1965-11-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeron?',1895,'',100 from dual union all
select 12697,'1965-S588','1965-11-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeron?',1895,'',100 from dual union all
select 12698,'1965-S589','1965-11-01 02:15:00','suborbital rocket','success',1069,'PT-160-2','Propellant Test','','','Test',839,'',93 from dual;
insert into LAUNCH
select 12699,'1965-S59','1965-01-30 08:11:00','military missile','success',999,'R 22500-22L','8K67 LKI-19','','','Test',574,'',1493 from dual union all
select 127,'1944-S115','1944-09-18 14:22:00','military missile','success',8,'18766','Ma183','','','Training',661,'',90 from dual union all
select 1270,'1952-A204','1952-11-04 00:00:00','military missile','success',771,'GM-2353','New tail config','','','Test',222,'',12 from dual union all
select 12700,'1965-S590','1965-11-02 04:18:00','military missile','success',999,'U 22500-37L','8K67 LKI-30','','','Test',514,'',1000 from dual union all
select 12701,'1965-S591','1965-11-02 20:00:00','suborbital rocket','success',470,'CRL AE13.559','IQSY Sphere','','','Aeron',394,'',686 from dual union all
select 12702,'1965-S592','1965-11-02 23:35:00','suborbital rocket','success',1250,'S-1','IQSY release','','','Aeron/Ionos',394,'',134 from dual union all
select 12703,'1965-S593','1965-11-03 09:17:00','suborbital rocket','success',839,'CRL AE6.557','IQSY 7-inch Sphere','','','Aeron',394,'',137 from dual union all
select 12704,'1965-S594','1965-11-03 19:00:00','suborbital rocket','success',30,'CRL AE3.520','Solar XUVT','','','Solar SX',1869,'',210 from dual union all
select 12705,'1965-S595','1965-11-04 04:32:00','military missile','success',982,'U 22603-20T','8K64U UBP-63 K','','','OT',566,'',1210 from dual union all
select 12706,'1965-S596','1965-11-05 00:00:00','military missile','success',932,'A3','OT','','','OT',421,'SSBN 627',1000 from dual union all
select 12707,'1965-S597','1965-11-05 00:00:00','military missile','success',932,'A3','OT','','','OT',421,'SSBN 627',1000 from dual union all
select 12708,'1965-S598','1965-11-05 00:00:00','military missile','success',932,'A3','OT','','','OT',421,'SSBN 627',1000 from dual union all
select 12709,'1965-S599','1965-11-05 00:00:00','military missile','success',932,'A3','OT','','','OT',421,'SSBN 627',1000 from dual union all
select 1271,'1952-A205','1952-11-06 00:00:00','atmospheric rocket','success',836,'1249A-139','Nike 1249 R&D 132R','','','Test',1895,'',10 from dual union all
select 12710,'1965-S60','1965-02-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 12711,'1965-S600','1965-11-05 00:00:00','military missile','success',135,'D007','ABRES P-8 (USAF)','ABRES P-8 (USAF)','','Reentry Test',612,'',200 from dual union all
select 12712,'1965-S601','1965-11-08 05:05:00','suborbital rocket','success',655,'K-10-1','','','','Test/Ionos',839,'',228 from dual union all
select 12713,'1965-S602','1965-11-09 00:00:00','suborbital rocket','success',547,'','','','','Aeron?',394,'',521 from dual union all
select 12714,'1965-S603','1965-11-09 00:00:00','military missile','success',796,'2009','Mk 11A RV','R&D','LOW TREE','Test',1707,'',900 from dual union all
select 12715,'1965-S604','1965-11-09 09:58:00','suborbital rocket','success',535,'HAD 179','Sphere','','','Aeron',1835,'',100 from dual union all
select 12716,'1965-S605','1965-11-09 18:40:00','suborbital rocket','success',837,'NASA 14.168UA','UM Pitot 18','','','Aeronomy',428,'',144 from dual union all
select 12717,'1965-S606','1965-11-09 19:00:00','suborbital rocket','success',30,'CRL AF3.523','EUV Mon','','','Solar EUV',1869,'',238 from dual union all
select 12718,'1965-S607','1965-11-09 19:16:00','suborbital rocket','success',867,'NASA 18.03GA','UM Thermosphere 2','','','Aeronomy',428,'',328 from dual union all
select 12719,'1965-S608','1965-11-10 02:30:00','suborbital rocket','success',30,'CRL AD3.356','Lunar IR','','','IR Astron',1869,'',180 from dual union all
select 1272,'1952-A206','1952-11-06 00:00:00','atmospheric rocket','success',836,'1249A-140','Nike 1249 R&D 133R','','','Test',1895,'',10 from dual union all
select 12720,'1965-S609','1965-11-10 07:00:00','suborbital rocket','success',867,'NASA 18.02GA','UM Thermosphere 3','','','Aeronomy',428,'',328 from dual union all
select 12721,'1965-S61','1965-02-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 12722,'1965-S610','1965-11-16 18:22:00','suborbital rocket','success',839,'CRL AE6.379','D-region comp.','','','Ionos/Aeron',394,'',114 from dual union all
select 12723,'1965-S611','1965-11-11 04:10:00','suborbital rocket','success',839,'CRL AE6.378','DAPHNE','','','Aeron',394,'',113 from dual union all
select 12724,'1965-S612','1965-11-11 04:15:00','suborbital rocket','success',1250,'S-2','','','','Aeron/Ionos',394,'',137 from dual union all
select 12725,'1965-S613','1965-11-12 00:00:00','suborbital rocket','success',865,'Sandia 152-84','AEC SKOL wind/density','','','Aeron',855,'',300 from dual union all
select 12726,'1965-S614','1965-11-12 03:58:00','military missile','success',982,'G 22601-06T','8K64U UBP-64','','','OT',566,'',1210 from dual union all
select 12727,'1965-S615','1965-11-12 22:28:14','military missile','success',928,'C2X-4','STAFF 4','','','Test',251,'',500 from dual union all
select 12728,'1965-S616','1965-11-14 03:00:00','military missile','success',999,'U 22500-36L','8K67 LKI-31','','','Test',485,'',1000 from dual union all
select 12729,'1965-S617','1965-11-16 16:00:00','suborbital rocket','success',30,'NASA 04.119NA','Luster 1965-B','','','Meteorites',1869,'',144 from dual union all
select 1273,'1952-A207','1952-11-06 00:00:00','atmospheric rocket','success',836,'1249A-141','Nike 1249 R&D 134R','','','Test',1895,'',10 from dual union all
select 12730,'1965-S618','1965-11-16 22:15:00','suborbital rocket','unknown',839,'','YORKSHIRE','','','Aeron',394,'',130 from dual union all
select 12731,'1965-S619','1965-11-16 23:00:00','suborbital rocket','success',549,'Sandia 152-90','LRL RCS-1','','','Aeron',855,'',183 from dual union all
select 12732,'1965-S62','1965-02-01 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeron?',1895,'',100 from dual union all
select 12733,'1965-S620','1965-11-17 04:25:00','suborbital rocket','success',837,'NASA 14.238UE','Auroral emissions','','','Aurora',428,'',178 from dual union all
select 12734,'1965-S621','1965-11-17 23:36:00','suborbital rocket','success',839,'CRL AE6.484','EDNA','','','Aeron',394,'',155 from dual union all
select 12735,'1965-S622','1965-11-18 00:00:00','suborbital rocket','success',850,'','IQSY Na, TMA','','','Aeron Na/TMA',394,'',130 from dual union all
select 12736,'1965-S623','1965-11-18 02:20:00','suborbital rocket','unknown',1250,'NA-FLORA       /S-4','','','','Aeron',394,'',130 from dual union all
select 12737,'1965-S624','1965-11-18 05:20:00','suborbital rocket','success',839,'CRL AE6.382','IQSY D-region comp.','','','Aeron',394,'',116 from dual union all
select 12738,'1965-S625','1965-11-18 05:20:00','suborbital rocket','success',839,'CRL AE6.383','IQSY Ion composition','','','Aeron',394,'',107 from dual union all
select 12739,'1965-S626','1965-11-18 05:20:00','suborbital rocket','unknown',1250,'S-3','','','','Aeron',394,'',130 from dual union all
select 1274,'1952-A208','1952-11-06 00:00:00','atmospheric rocket','success',836,'1249A-142','Nike 1249 R&D 135R','','','Test',1895,'',10 from dual union all
select 12740,'1965-S627','1965-11-18 08:01:00','suborbital rocket','success',1250,'NA-HANNAH      /S-6','','','','Aeron/Ionos',394,'',137 from dual union all
select 12741,'1965-S628','1965-11-18 11:21:00','suborbital rocket','success',839,'CRL AE6.499','POLLY','','','Aeron',394,'',155 from dual union all
select 12742,'1965-S629','1965-11-18 14:31:00','suborbital rocket','success',837,'NASA 14.78UA','Pandora 1 (Leonids)','','','Meteorites',1895,'',149 from dual union all
select 12743,'1965-S63','1965-02-03 15:35:00','military missile','success',916,'AO-21','P-88','','','OT',437,'',180 from dual union all
select 12744,'1965-S630','1965-11-18 17:45:00','suborbital rocket','success',288,'C119','MPE Ba-14','','','Aeronomy',666,'',170 from dual union all
select 12745,'1965-S631','1965-11-19 04:54:00','suborbital rocket','success',1199,'SL130','LUX-31/QUB Li-3/Bir','','','Iono/Solar X/UCL I-24/IC',1839,'',138 from dual union all
select 12746,'1965-S632','1965-11-19 17:45:00','suborbital rocket','success',288,'C120','MPE Ba-15','','','Aeronomy',666,'',190 from dual union all
select 12747,'1965-S633','1965-11-20 00:37:53','suborbital rocket','success',837,'Ferdinand 12','Ferdinand 12','','','Ionos',29,'',122 from dual union all
select 12748,'1965-S634','1965-11-20 05:20:00','suborbital rocket','success',837,'NASA 14.239UE','Auroral emissions','','','Aurora',428,'',175 from dual union all
select 12749,'1965-S635','1965-11-20 08:28:00','suborbital rocket','success',30,'KP3.8','Stellar Spectra','','','UV Astron',1869,'',150 from dual union all
select 1275,'1952-A209','1952-11-07 00:00:00','atmospheric rocket','success',365,'','F-100 Model','','','Test',1821,'',10 from dual union all
select 12750,'1965-S636','1965-11-20 17:53:00','suborbital rocket','success',446,'D-08','MPE Ba-16','','','Aeronomy',666,'',404 from dual union all
select 12751,'1965-S637','1965-11-22 00:00:00','suborbital rocket','success',865,'Sandia 152-27','Sampler','','','Aeron?',855,'',200 from dual union all
select 12752,'1965-S638','1965-11-22 00:00:00','suborbital rocket','failure',864,'Sandia 152-89','HRT','','','Aeron?',855,'',270 from dual union all
select 12753,'1965-S639','1965-11-22 06:35:00','suborbital rocket','success',692,'LS-A-3','','','','Test',990,'',60 from dual union all
select 12754,'1965-S64','1965-02-03 15:30:00','military missile','success',916,'AO-22','P-87','','','OT',437,'',180 from dual union all
select 12755,'1965-S640','1965-11-22 23:09:00','suborbital rocket','success',767,'173','2C2 MAXWELL','','','Aeron TMA',185,'',142 from dual union all
select 12756,'1965-S641','1965-11-23 00:30:00','suborbital rocket','unknown',839,'','NEEDHAMS POINT','','','Aeron',394,'',100 from dual union all
select 12757,'1965-S642','1965-11-23 02:59:00','suborbital rocket','success',767,'176','2C2 PARAGON','','','Aeron TMA',185,'',135 from dual union all
select 12758,'1965-S643','1965-11-23 05:31:00','suborbital rocket','success',767,'177','2C2 QUEENS FORT','','','Aeron TMA',185,'',133 from dual union all
select 12759,'1965-S644','1965-11-23 07:28:00','suborbital rocket','success',767,'178','2C2 ROCKLEY','','','Aeron TMA',185,'',143 from dual union all
select 1276,'1952-A21','1952-02-01 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 12760,'1965-S645','1965-11-23 09:16:00','suborbital rocket','success',767,'179','2C2 SEAWELL','','','Aeron TMA',185,'',141 from dual union all
select 12761,'1965-S646','1965-11-24 04:04:00','suborbital rocket','success',837,'NASA 14.124UE','Auroral emissions','','','Aurora',428,'',175 from dual union all
select 12762,'1965-S647','1965-11-24 15:37:00','suborbital rocket','success',837,'NASA 14.242UE','B Field','','','Plasma',1821,'',169 from dual union all
select 12763,'1965-S648','1965-11-25 04:40:00','suborbital rocket','success',288,'C91','','','','Aeron',310,'',175 from dual union all
select 12764,'1965-S649','1965-11-25 08:24:00','suborbital rocket','success',288,'C93','','','','Aeron',310,'',175 from dual union all
select 12765,'1965-S65','1965-01-31 05:01:00','suborbital rocket','success',697,'L-3-2','','','','Ionos',834,'',1040 from dual union all
select 12766,'1965-S650','1965-11-25 13:20:00','military missile','success',243,'BK.25','H-212 II/Dazzle 6','','','Reentry Test',1842,'',632 from dual union all
select 12767,'1965-S651','1965-11-26 08:24:00','suborbital rocket','success',288,'C96','','','','Aeron',310,'',175 from dual union all
select 12768,'1965-S652','1965-11-27 00:00:00','military missile','success',999,'U 22500-33L','8K67 LKI-32','','','Test',511,'',1000 from dual union all
select 12769,'1965-S653','1965-11-27 10:00:50','military missile','success',1420,'B-20','Mk 6 RV','OT','RED WAGON','OT',1659,'',1300 from dual union all
select 1277,'1952-A210','1952-11-07 00:00:00','atmospheric rocket','unknown',576,'62','','','','Test',1895,'',10 from dual union all
select 12770,'1965-S654','1965-11-29 14:30:13','ballistic missile test','success',154,'200D','Nike RMV-302/Pod 27','NTMP KX-33','WILD GOAT','Reentry Test',1680,'',1800 from dual union all
select 12771,'1965-S655','1965-12-01 00:00:00','military missile','success',7,'6TYa','','','','Test',182,'',100 from dual union all
select 12772,'1965-S656','1965-12-08 00:00:00','military missile','success',837,'BMRS','SR-020','','','Target',1895,'',100 from dual union all
select 12773,'1965-S657','1965-12-21 00:00:00','military missile','success',982,'E 22501-10','8K64U UBP-P2','Verification','','OT',477,'',1210 from dual union all
select 12774,'1965-S658','1965-12-24 00:00:00','military missile','success',982,'T 22602-04','8K64U UBP-P3','Verification','','OT',477,'',1210 from dual union all
select 12775,'1965-S659','1965-11-30 07:48:00','suborbital rocket','success',30,'NASA 04.155GG','','','','UV Astron',1869,'',179 from dual union all
select 12776,'1965-S66','1965-02-01 00:00:00','military missile','success',736,'','Tir 1 (Dummy St1)','','','Test',313,'',100 from dual union all
select 12777,'1965-S660','1965-11-30 14:44:33','military missile','success',1420,'B-4','Mk 6 RV','OT','CROSS FIRE','OT',1657,'',1300 from dual union all
select 12778,'1965-S661','1965-12-02 17:30:00','suborbital rocket','success',30,'NASA 04.145GS','Mg II heliogram 2','','','Solar UV',1869,'',185 from dual union all
select 12779,'1965-S662','1965-12-06 19:25:41','military missile','success',932,'A3P-42','OT/GT7 target','','','OT',421,'SSBN 640',1000 from dual union all
select 1278,'1952-A211','1952-11-07 00:00:00','atmospheric rocket','unknown',576,'63','','','','Test',1895,'',10 from dual union all
select 12780,'1965-S663','1965-12-06 00:00:00','military missile','success',135,'C017','MARCAS','ABRES W-3 (USAF)','','Reentry Test',614,'',200 from dual union all
select 12781,'1965-S664','1965-12-06 10:24:00','suborbital rocket','success',535,'HAD 151','Sphere','','','Aeron',1835,'',100 from dual union all
select 12782,'1965-S665','1965-12-07 04:05:00','military missile','success',999,'U 22500-34L','8K67 LKI-33','','','Test',485,'',1000 from dual union all
select 12783,'1965-S666','1965-12-07 03:53:00','suborbital rocket','success',288,'','ESRO C07/1','','','Aeron',1343,'',150 from dual union all
select 12784,'1965-S667','1965-12-08 02:28:00','test rocket','success',1373,'188/20011','AFP-437(AP)','','','Interceptor',773,'',487 from dual union all
select 12785,'1965-S668','1965-12-08 13:21:00','suborbital rocket','success',217,'NC7.179','','','','Aeron',394,'',140 from dual union all
select 12786,'1965-S669','1965-12-09 13:05:00','suborbital rocket','success',535,'HAD 305','Lunar UV Ozone','','','UV Astron/Aeron',1835,'',85 from dual union all
select 12787,'1965-S67','1965-02-02 00:00:00','military missile','success',795,'714','Mk 11 RV','OT','RED BRIDGE','OT',1703,'',900 from dual union all
select 12788,'1965-S670','1965-12-09 20:33:00','suborbital rocket','success',839,'CRL AA6.174','Sphere','','','Aeron',428,'',139 from dual;
insert into LAUNCH
select 12789,'1965-S671','1965-12-10 00:00:00','military missile','success',135,'C023','ECM Decoy','ABRES H-1 (USAF)','','Reentry Test',612,'',200 from dual union all
select 1279,'1952-A212','1952-11-07 00:00:00','military missile','success',771,'GM-567','','','','Test',222,'',12 from dual union all
select 12790,'1965-S672','1965-12-10 05:00:00','suborbital rocket','success',839,'CRL AA6.173','Sphere','','','Aeron',428,'',141 from dual union all
select 12791,'1965-S673','1965-12-11 00:00:00','suborbital rocket','success',850,'','','','','Aeron',394,'',130 from dual union all
select 12792,'1965-S674','1965-12-11 04:05:00','suborbital rocket','success',288,'','ESRO C07/2','','','Aeronomy',1343,'',159 from dual union all
select 12793,'1965-S675','1965-12-11 18:55:00','suborbital rocket','success',837,'CRL AE5.558','Sphere','','','Aeron',428,'',155 from dual union all
select 12794,'1965-S676','1965-12-12 04:51:00','suborbital rocket','success',837,'CRL AE5.172','Sphere','','','Aeron',428,'',155 from dual union all
select 12795,'1965-S677','1965-12-13 06:20:00','suborbital rocket','success',668,'K-9M-16','Ozone profile','','','Ionos/Aeron',839,'',318 from dual union all
select 12796,'1965-S678','1965-12-13 10:08:00','suborbital rocket','success',1199,'SL131','LUX-32/QUB Li-4/Bir','','','Iono/Solar X/UCL I-25/IC',1839,'',130 from dual union all
select 12797,'1965-S679','1965-12-14 00:00:00','military missile','success',1017,'T 15001- 09','8K74 UBP-P1','','','Test',473,'',1350 from dual union all
select 12798,'1965-S68','1965-02-02 00:00:00','military missile','success',795,'1041','Mk 11 RV','OT','SIDE TRIP','OT',1697,'',900 from dual union all
select 12799,'1965-S680','1965-12-14 12:49:00','suborbital rocket','success',288,'CE-6','ESRO C06/1','','','Solar X',1343,'',159 from dual union all
select 128,'1944-S116','1944-09-18 16:25:00','military missile','success',8,'18762','Ma176','','','Training',661,'',90 from dual union all
select 1280,'1952-A213','1952-11-07 00:00:00','atmospheric rocket','success',836,'1249B-1010','Nike Eng IE','','','Test',1895,'',10 from dual union all
select 12800,'1965-S681','1965-12-14 17:23:00','military missile','success',794,'516','Mk 5 RV','FOT-3','GRAND RIVER','OT',1699,'',900 from dual union all
select 12801,'1965-S682','1965-12-16 00:00:00','military missile','success',796,'2017','Mk 11 RV','R&D','PUSH PULL','Test',1710,'',900 from dual union all
select 12802,'1965-S683','1965-12-15 17:00:00','suborbital rocket','success',837,'NASA 14.247UI','UIUC-GCA D/E-6','','','Ionos',1821,'',184 from dual union all
select 12803,'1965-S684','1965-12-15 19:19:00','suborbital rocket','success',837,'NASA 14.68II','','','','Ionos',330,'',185 from dual union all
select 12804,'1965-S685','1965-12-16 00:00:00','suborbital rocket','success',288,'C131','CNES FU-148','','','Ionos',664,'',145 from dual union all
select 12805,'1965-S686','1965-12-16 00:00:00','suborbital rocket','success',288,'C128','CNES FU-148','','','Ionos',664,'',128 from dual union all
select 12806,'1965-S687','1965-12-16 00:00:00','suborbital rocket','success',288,'C94','CNES FU-146','','','Aeronomy',664,'',170 from dual union all
select 12807,'1965-S688','1965-12-18 04:59:00','suborbital rocket','success',837,'NASA 14.69II','','','','Ionos',330,'',188 from dual union all
select 12808,'1965-S689','1965-12-18 06:00:00','suborbital rocket','success',668,'K-9M-17','','','','Ionos',839,'',316 from dual union all
select 12809,'1965-S69','1965-02-02 00:00:00','suborbital rocket','success',850,'DASA HAR','','','','Aeron?',1895,'',100 from dual union all
select 1281,'1952-A214','1952-11-08 00:00:00','suborbital rocket','failure',1490,'N4','','','','Test',666,'',10 from dual union all
select 12810,'1965-S690','1965-12-20 16:33:03','military missile','success',932,'A3P-188','','','','OT',421,'SSBN 640',1000 from dual union all
select 12811,'1965-S691','1965-12-20 00:00:00','suborbital rocket','success',288,'C130','CNES FU-148','','','Ionos',664,'',145 from dual union all
select 12812,'1965-S692','1965-12-20 13:39:56','military missile','success',154,'85D','Nike RMV-303','NTMP KX-31','TAG DAY','Reentry Test',1684,'',1800 from dual union all
select 12813,'1965-S693','1965-12-21 00:00:00','military missile','success',1017,'T 15003- 14','8K74 UBP-P2','','','Test',473,'',1350 from dual union all
select 12814,'1965-S694','1965-12-22 14:10:45','military missile','success',1420,'B-73','Mk 6 RV','OT','SEA ROVER','OT',1658,'',1300 from dual union all
select 12815,'1965-S695','1965-12-23 00:00:00','military missile','failure',735,'','Bietage 1','','','Test',313,'',100 from dual union all
select 12816,'1965-S696','1965-12-23 07:39:00','military missile','success',982,'G 22602-10T','8K64U UBP-65','','','OT',565,'',1210 from dual union all
select 12817,'1965-S697','1965-12-25 07:16:00','military missile','success',982,'T 22602-06TB','8K64U UBP-66','','','OT',566,'',1210 from dual union all
select 12818,'1965-S698','1965-12-25 04:00:00','military missile','success',999,'U 22502-09T','8K67 LKI-34','Salvo','','Test',511,'',1000 from dual union all
select 12819,'1965-S699','1965-12-25 04:01:00','military missile','success',999,'U 22503-09T','8K67 LKI-35','Salvo','','Test',485,'',1000 from dual union all
select 1282,'1952-A215','1952-11-09 00:00:00','suborbital rocket','failure',1490,'N5','','','','Test',666,'',1 from dual union all
select 12820,'1965-S70','1965-02-02 00:00:00','suborbital rocket','success',1431,'Sandia 152-17','Magnus','','','Test/Aeron?',1490,'',106 from dual union all
select 12821,'1965-S700','1965-12-26 12:43:00','military missile','success',982,'U 22601-06T','8K64U UBP-67','','','OT',565,'',1210 from dual union all
select 12822,'1965-S701','1965-12-26 10:00:00','military missile','success',999,'U 22504-09T','8K67 LKI-36','','','Test',514,'',1000 from dual union all
select 12823,'1965-S702','1965-02-03 22:55:00','suborbital rocket','success',535,'HAD','La absorption','','','Aeron',1835,'',100 from dual union all
select 12824,'1965-S703','1965-01-01 00:00:00','military missile','success',983,'KY-3 (Scud C)','Long range R-17','','','Test',636,'',250 from dual union all
select 12825,'1965-S704','1965-01-01 00:00:00','suborbital rocket','success',1431,'Sandia 276-2','Dart/Parachute','','','Parachute test',1490,'',90 from dual union all
select 12826,'1965-S705','1965-03-23 00:00:00','suborbital rocket','success',767,'','2A X-1','','','Test',185,'',80 from dual union all
select 12827,'1965-S706','1965-03-23 00:00:00','suborbital rocket','success',767,'','2A X-2','','','Test',185,'',80 from dual union all
select 12828,'1965-S707','1965-03-24 01:24:00','suborbital rocket','success',767,'97 TMA-5','2A X-3  LUPACA','','','Aeron TMA',185,'',114 from dual union all
select 12829,'1965-S708','1965-03-24 05:03:00','suborbital rocket','success',767,'98 TMA-6','2A X-4  MIAMI','','','Aeron TMA',185,'',110 from dual union all
select 1283,'1952-A216','1952-11-10 00:00:00','military missile','unknown',339,'34R','','','','Test',1895,'',30 from dual union all
select 12830,'1965-S709','1965-03-24 00:00:00','suborbital rocket','success',767,'','2C X-1','','','Test',185,'',80 from dual union all
select 12831,'1965-S71','1965-03-02 09:57:00','suborbital rocket','success',535,'HAD 152','Sphere','','','Aeron',1835,'',123 from dual union all
select 12832,'1965-S710','1965-03-24 00:00:00','suborbital rocket','success',767,'99','2C X-2','','','Test',185,'',80 from dual union all
select 12833,'1965-S711','1965-03-24 00:00:00','suborbital rocket','success',767,'100','2C X-3','','','Test',185,'',80 from dual union all
select 12834,'1965-S712','1965-03-27 06:20:00','suborbital rocket','success',767,'103 TMA-7','2C X-4  NOOTKA','','','Aeron TMA',185,'',111 from dual union all
select 12835,'1965-S713','1965-03-29 00:02:00','suborbital rocket','success',767,'104 TMA-8','2C X-5  OTTAWA','','','Aeron TMA',185,'',119 from dual union all
select 12836,'1965-S714','1965-03-29 02:20:00','suborbital rocket','success',767,'105 TMA-9','2C X-6  PUEBLO','','','Aeron TMA',185,'',117 from dual union all
select 12837,'1965-S715','1965-03-25 00:00:00','suborbital rocket','success',767,'106','2D X-1','','','Test',185,'',80 from dual union all
select 12838,'1965-S716','1965-03-29 00:00:00','suborbital rocket','success',770,'107','3B X-3','','','Test',185,'',80 from dual union all
select 12839,'1965-S717','1965-05-31 16:47:00','suborbital rocket','success',767,'108 Chaff','2A XI-1 APPIUS','','','Test',185,'',84 from dual union all
select 1284,'1952-A217','1952-11-12 00:00:00','atmospheric rocket','success',365,'','XF2Y-1 Model','','','Test',1821,'',10 from dual union all
select 12840,'1965-S718','1965-06-02 18:15:00','suborbital rocket','success',767,'109 Chaff','2A XI-2 RUFUS','','','Test',185,'',79 from dual union all
select 12841,'1965-S719','1965-06-05 17:22:00','suborbital rocket','success',767,'117 Chaff','2A XI-3 SPARTA','','','Test',185,'',76 from dual union all
select 12842,'1965-S72','1965-02-03 00:00:00','suborbital rocket','success',1431,'Sandia 152-47','Magnus','','','Test/Aeron?',1490,'',106 from dual union all
select 12843,'1965-S720','1965-06-03 23:57:00','suborbital rocket','success',767,'112 TMA-10','2C XI-2 MARIUS','','','Aeron TMA',185,'',111 from dual union all
select 12844,'1965-S721','1965-06-04 02:41:00','suborbital rocket','success',767,'113 TMA-11','2C XI-5 NERO','','','Aeron TMA',185,'',94 from dual union all
select 12845,'1965-S722','1965-06-04 05:34:56','suborbital rocket','success',767,'114 TMA-12 B-12','2C XI-6 ELAGABULUS','','','Aeron TMA',185,'',119 from dual union all
select 12846,'1965-S723','1965-06-04 07:17:00','suborbital rocket','success',767,'115 TMA-13','2C XI-7 FABIUS','','','Aeron TMA',185,'',110 from dual union all
select 12847,'1965-S724','1965-06-10 01:57:00','suborbital rocket','success',767,'122 TMA-14','2C XI-8  OVID','','','Aeron TMA',185,'',105 from dual union all
select 12848,'1965-S725','1965-06-10 03:58:00','suborbital rocket','success',767,'123 TMA-15','2C XI-9  CICERO','','','Aeron TMA',185,'',107 from dual union all
select 12849,'1965-S726','1965-06-10 06:35:00','suborbital rocket','success',767,'124 TMA','2C XI-10 DIANA','','','Aeron TMA',185,'',109 from dual union all
select 1285,'1952-A218','1952-11-13 00:00:00','military missile','unknown',339,'35R','','','','Test',1895,'',30 from dual union all
select 12850,'1965-S727','1965-06-11 01:07:00','suborbital rocket','success',767,'125 TMA-16','2C XI-11 PLINY','','','Aeron TMA',185,'',133 from dual union all
select 12851,'1965-S728','1965-06-09 22:51:00','suborbital rocket','success',767,'121 TMA','2C XI-12 LUCRETIA','','','Ionos?',185,'',118 from dual union all
select 12852,'1965-S729','1965-06-12 07:00:00','suborbital rocket','success',767,'126 TMA','2C XI-13 QUINTUS','','','Aeron',185,'',135 from dual union all
select 12853,'1965-S73','1965-02-03 18:08:00','suborbital rocket','success',837,'NASA 14.61UE','UNH Particles','','','Plasma',1821,'',141 from dual union all
select 12854,'1965-S730','1965-06-12 08:31:00','suborbital rocket','success',767,'127 TMA','2C XI-14 HADRIAN','','','Aeron',185,'',133 from dual union all
select 12855,'1965-S731','1965-06-03 17:32:00','suborbital rocket','success',770,'110 Grain test','3B XI-1 DEVILS','','','Test',185,'',1 from dual union all
select 12856,'1965-S732','1965-06-07 16:11:00','suborbital rocket','success',770,'120 Grain test','3B XI-2 EAGLE','','','Test',185,'',1 from dual union all
select 12857,'1965-S733','1965-08-04 00:00:00','suborbital rocket','success',767,'128','2C XII-1','','','Aeron?',185,'',100 from dual union all
select 12858,'1965-S734','1965-08-06 00:20:00','suborbital rocket','success',767,'129 TMA-17','2C XII-2','','','Aeron TMA',185,'',107 from dual union all
select 12859,'1965-S735','1965-08-06 06:44:00','suborbital rocket','success',767,'130 TMA-18','2C XII-3','','','Aeron TMA',185,'',106 from dual union all
select 1286,'1952-A219','1952-11-13 00:00:00','atmospheric rocket','success',836,'1249A-143','Nike 1249 R&D 136R','','','Test',1895,'',10 from dual union all
select 12860,'1965-S736','1965-09-14 00:00:00','suborbital rocket','success',767,'','2C XIII-1','','','Aeron?',185,'',100 from dual union all
select 12861,'1965-S737','1965-09-01 00:00:00','suborbital rocket','success',767,'','2C XIII-2','','','Aeron?',185,'',100 from dual union all
select 12862,'1965-S738','1965-09-01 00:00:00','suborbital rocket','success',767,'','2C XIII-3','','','Aeron?',185,'',100 from dual union all
select 12863,'1965-S739','1965-09-01 00:00:00','suborbital rocket','success',767,'','2C XIII-4','','','Aeron?',185,'',100 from dual union all
select 12864,'1965-S74','1965-02-04 04:45:00','suborbital rocket','success',839,'NASA 10.125GA','GSFC Grenade 69 (PB)','','','Aeronomy',1039,'',121 from dual union all
select 12865,'1965-S740','1965-09-01 00:00:00','suborbital rocket','success',767,'','2C XIII-5','','','Aeron?',185,'',100 from dual union all
select 12866,'1965-S741','1965-09-01 00:00:00','suborbital rocket','success',767,'','2C XIII-6','','','Aeron?',185,'',100 from dual union all
select 12867,'1965-S742','1965-09-01 00:00:00','suborbital rocket','success',767,'','2C XIII-7','','','Aeron?',185,'',100 from dual union all
select 12868,'1965-S744','1965-03-25 00:00:00','suborbital rocket','success',770,'101','3B X-1','','','Test',185,'',80 from dual union all
select 12869,'1965-S745','1965-03-25 00:00:00','suborbital rocket','success',770,'102','3B X-2','','','Test',185,'',80 from dual union all
select 1287,'1952-A22','1952-02-05 00:00:00','atmospheric rocket','success',869,'490B','System Test 77R','','','Test',1895,'',10 from dual union all
select 12870,'1965-S746','1965-09-01 00:00:00','suborbital rocket','success',767,'','2A XIII-1','','','Aeron?',185,'',100 from dual union all
select 12871,'1965-S747','1965-09-25 00:00:00','suborbital rocket','success',767,'','2A XIII-2','','','Aeron?',185,'',100 from dual union all
select 12872,'1965-S748','1965-09-20 23:30:00','suborbital rocket','success',767,'133 TMA-19 B-19','2C XIII-8  BELAIR','','','Aeron',185,'',132 from dual union all
select 12873,'1965-S749','1965-09-21 00:52:00','suborbital rocket','success',767,'134 TMA-20','2C XIII-9  CONOWINGO','','','Aeron',185,'',136 from dual union all
select 12874,'1965-S75','1965-02-04 05:10:00','suborbital rocket','success',839,'NASA 10.119GA','GSFC Grenade 70 (W)','','','Aeronomy',1821,'',118 from dual union all
select 12875,'1965-S750','1965-09-21 04:24:00','suborbital rocket','success',767,'136 TMA-21','2C XIII-10 EDGEWOOD','','','Aeron',185,'',123 from dual union all
select 12876,'1965-S751','1965-09-21 06:21:00','suborbital rocket','success',767,'137 TMA-22','2C XIII-11 FO. GREEN','','','Aeron',185,'',128 from dual union all
select 12877,'1965-S752','1965-09-21 07:35:00','suborbital rocket','success',767,'138 TMA-23','2C XIII-12 GALENA','','','Aeron',185,'',138 from dual union all
select 12878,'1965-S753','1965-09-21 09:00:00','suborbital rocket','success',767,'139 TMA-24','2C XIII-13 HAVRE DE','','','Aeron',185,'',123 from dual;
insert into LAUNCH
select 12879,'1965-S754','1965-09-22 23:24:00','suborbital rocket','success',767,'142 TMA-25','2C XIII-14 LINTHICUM','','','Aeron',185,'',127 from dual union all
select 1288,'1952-A220','1952-11-13 00:00:00','atmospheric rocket','success',836,'1249A-144','Nike 1249 R&D 137R','','','Test',1895,'',10 from dual union all
select 12880,'1965-S755','1965-09-23 05:50:00','suborbital rocket','success',767,'144 TMA-26','2C XIII-15 MAGNOLIA','','','Aeron',185,'',126 from dual union all
select 12881,'1965-S756','1965-09-23 07:15:00','suborbital rocket','success',767,'145 TMA-27','2C XIII-16 NANTICOKE','','','Aeron',185,'',133 from dual union all
select 12882,'1965-S757','1965-09-23 08:07:00','suborbital rocket','success',767,'146 TMA-28','2C XIII-17 OLNEY','','','Aeron',185,'',126 from dual union all
select 12883,'1965-S758','1965-09-23 09:05:00','suborbital rocket','success',767,'147 TMA-29','2C XIII-18 PERRYMAN','','','Aeron',185,'',124 from dual union all
select 12884,'1965-S759','1965-11-16 17:18:00','suborbital rocket','success',767,'156','2C2 ZENMAC','','','Test',185,'',110 from dual union all
select 12885,'1965-S76','1965-02-04 17:34:00','suborbital rocket','success',839,'NASA 10.122GA','GSFC Grenade 71 (C)','','','Aeronomy',428,'',150 from dual union all
select 12886,'1965-S760','1965-11-16 22:15:00','suborbital rocket','success',767,'157','2C3 CESIUM','','','Ionos',185,'',119 from dual union all
select 12887,'1965-S761','1965-11-17 20:16:00','suborbital rocket','success',767,'159','2C3 BRIDGETOWN Chaff','','','Aeron',185,'',85 from dual union all
select 12888,'1965-S763','1965-11-17 22:15:00','suborbital rocket','success',767,'160 B-31','2C2 CHRISTCHURCH','','','Aeron TMA',185,'',120 from dual union all
select 12889,'1965-S764','1965-11-17 23:34:00','suborbital rocket','success',767,'161 B-32','2C2 DOVER','','','Aeron TMA',185,'',122 from dual union all
select 1289,'1952-A221','1952-11-13 00:00:00','suborbital rocket','failure',1490,'N7','','','','Test',666,'',4 from dual union all
select 12890,'1965-S765','1965-11-18 03:15:00','suborbital rocket','success',767,'163 B-33','2C2 FOULBAY','','','Aeron TMA',185,'',123 from dual union all
select 12891,'1965-S766','1965-11-18 04:45:00','suborbital rocket','success',767,'164','2C2 GUNHILL','','','Aeron TMA',185,'',126 from dual union all
select 12892,'1965-S767','1965-11-18 07:30:00','suborbital rocket','success',767,'166','2C2 INDIAN GROUND','','','Aeron TMA',185,'',122 from dual union all
select 12893,'1965-S768','1965-11-18 09:08:00','suborbital rocket','success',767,'167','2C2 JAMESTOWN','','','Aeron TMA',185,'',119 from dual union all
select 12894,'1965-S769','1965-11-20 16:00:00','suborbital rocket','success',767,'170','2C2 WORTHING','','','Test',185,'',104 from dual union all
select 12895,'1965-S77','1965-02-06 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 624',1000 from dual union all
select 12896,'1965-S770','1965-11-21 16:10:00','suborbital rocket','success',767,'172','2C2 VAUXHALL','','','Test',185,'',128 from dual union all
select 12897,'1965-S771','1965-11-22 23:30:00','suborbital rocket','success',767,'174','2C2 NEEDHAM''S POINT','','','Aeron TMA',185,'',133 from dual union all
select 12898,'1965-S772','1965-12-08 00:00:00','military missile','success',837,'BMRS','SR-021','','','Target',1895,'',100 from dual union all
select 12899,'1965-S773','1965-09-29 00:00:00','military missile','success',837,'','SPEEDBALL II 75 SN32','','','Target',1895,'',150 from dual union all
select 129,'1944-S117','1944-09-19 00:00:00','military missile','success',1470,'','Tournai','','','Weapon',1728,'',95 from dual union all
select 1290,'1952-A222','1952-11-14 00:00:00','atmospheric rocket','success',565,'','E31 Model','','','Test',1821,'',5 from dual union all
select 12900,'1965-S774','1965-10-05 00:00:00','military missile','success',837,'','SPEEDBALL II 76 SN33','','','Target',1895,'',150 from dual union all
select 12901,'1965-S775','1965-10-14 00:00:00','military missile','success',837,'','SPEEDBALL II 77 SN31','','','Target',1895,'',150 from dual union all
select 12902,'1965-S776','1965-10-27 00:00:00','military missile','success',837,'','SPEEDBALL II 78 SN34','','','Target',1895,'',150 from dual union all
select 12903,'1965-S777','1965-12-10 00:00:00','military missile','success',837,'','SPEEDBALL II 81 SN36','','','Target',1895,'',150 from dual union all
select 12904,'1965-S778','1965-12-14 00:00:00','military missile','success',837,'','SPEEDBALL II 82 SN37','','','Target',1895,'',150 from dual union all
select 12905,'1965-S779','1965-12-15 00:00:00','military missile','success',837,'','SPEEDBALL II 83 SN38','','','Target',1895,'',150 from dual union all
select 12906,'1965-S78','1965-02-06 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 624',1000 from dual union all
select 12907,'1965-S780','1965-12-16 00:00:00','military missile','success',837,'','SPEEDBALL II 84 SN39','','','Target',1895,'',150 from dual union all
select 12908,'1965-S781','1965-12-17 00:00:00','military missile','success',837,'','SPEEDBALL II 85 SN40','','','Target',1895,'',150 from dual union all
select 12909,'1965-S782','1965-12-17 00:00:00','military missile','success',837,'','SPEEDBALL II 86 SN41','','','Target',1895,'',150 from dual union all
select 1291,'1952-A223','1952-11-16 00:00:00','suborbital rocket','failure',1490,'N6','','','','Test',666,'',1 from dual union all
select 12910,'1965-S783','1965-06-27 05:24:22','suborbital rocket','success',1440,'No. 40','','','','Reentry Test',1821,'',300 from dual union all
select 12911,'1965-S784','1965-01-30 00:00:00','military missile','success',871,'ZK-50 20155','Sim SLBM Target','','','Interceptor',873,'',80 from dual union all
select 12912,'1965-S785','1965-01-30 00:00:00','military missile','success',871,'ZK-51 20162','Sim SLBM Target','','','Interceptor',873,'',80 from dual union all
select 12913,'1965-S787','1965-07-29 00:00:00','military missile','success',874,'ZK-59 20174','DM-15S2 Qual-3','','','ASAT Test',873,'',100 from dual union all
select 12914,'1965-S788','1965-01-01 00:00:00','military missile','success',871,'ZK-54','','','','Interceptor',873,'',80 from dual union all
select 12915,'1965-S789','1965-10-21 16:00:00','military missile','failure',916,'','P-98','','','OT',597,'',100 from dual union all
select 12916,'1965-S79','1965-02-06 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 624',1000 from dual union all
select 12917,'1965-S790','1965-10-20 00:00:00','military missile','success',916,'','P-96','','','OT',597,'',180 from dual union all
select 12918,'1965-S791','1965-09-22 00:00:00','military missile','success',932,'A3X-61','PX-2','','','Test',1900,'EAG-154',1000 from dual union all
select 12919,'1965-S792','1965-03-09 00:00:00','suborbital rocket','success',1330,'','','','','Test',1415,'',500 from dual union all
select 1292,'1952-A224','1952-11-17 00:00:00','military missile','unknown',339,'36R','','','','Test',1895,'',30 from dual union all
select 12920,'1965-S793','1965-11-24 00:00:00','suborbital rocket','failure',867,'Sandia','AEC SKOL','','','Aeron',855,'',80 from dual union all
select 12921,'1965-S794','1965-11-16 00:00:00','suborbital rocket','success',549,'Sandia','AEC SKOL','','','Aeron',855,'',150 from dual union all
select 12922,'1965-S795','1965-12-25 00:00:00','military missile','success',982,'T 22602-05','8K64U UBP-P4','','','OT',477,'',1210 from dual union all
select 12923,'1965-S796','1965-12-30 00:00:00','military missile','success',1449,'','8K84 LKI-23','Salvo','','Test',508,'',1000 from dual union all
select 12924,'1965-S797','1965-11-04 00:00:00','military missile','success',982,'','8K64U 54 RD UBP-5','','','OT',1468,'',1210 from dual union all
select 12925,'1965-S798','1965-12-20 00:00:00','military missile','success',982,'','8K64U 54 RD UBP-6','','','OT',1468,'',1210 from dual union all
select 12926,'1965-S799','1965-07-01 00:00:00','military missile','success',1018,'','8K75 UBP Omsk-1','','','OT',1012,'',1160 from dual union all
select 12927,'1965-S80','1965-02-06 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 624',1000 from dual union all
select 12928,'1965-S800','1965-12-25 00:00:00','military missile','success',982,'','8K64U 8 RD UBP-1','','','OT',1916,'',1210 from dual union all
select 12929,'1965-S801','1964-10-21 00:00:00','suborbital rocket','success',535,'HAD 142','','','','Aeron',219,'',100 from dual union all
select 1293,'1952-A225','1952-11-17 00:00:00','military missile','success',771,'GM-2354','','','','Test',222,'',12 from dual union all
select 12930,'1965-S802','1965-02-25 00:00:00','military missile','success',977,'','8K63U UBP 51 RB','','','Test',840,'',402 from dual union all
select 12931,'1965-S803','1965-02-25 00:00:00','military missile','success',977,'','8K63U UBP 51 RB','','','Test',840,'',402 from dual union all
select 12932,'1965-S804','1965-09-24 00:00:00','military missile','success',1013,'','8K51 29 RD UBP-8','','','OT',636,'',500 from dual union all
select 12933,'1965-S805','1965-08-05 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 12934,'1965-S806','1965-09-01 00:00:00','military missile','success',976,'','8K63 43 RD UBP','','','Test',636,'',402 from dual union all
select 12935,'1965-S807','1965-06-04 00:00:00','military missile','success',977,'','8K63U 29 RD UBP','','','Test',636,'',402 from dual union all
select 12936,'1965-S808','1965-07-14 00:00:00','military missile','success',980,'','8K65U UBP-4 29 RD','','','Test',636,'',675 from dual union all
select 12937,'1965-S809','1965-10-01 00:00:00','military missile','success',980,'','8K65U UBP-5 32 RD','','','Test',636,'',675 from dual union all
select 12938,'1965-S81','1965-02-06 05:01:00','suborbital rocket','success',668,'K-9M-7','','','','Ionos',839,'',325 from dual union all
select 12939,'1965-S82','1965-02-06 10:02:00','suborbital rocket','success',668,'K-9M-6','','','','Solar RF',839,'',335 from dual union all
select 1294,'1952-A226','1952-11-17 00:00:00','military missile','success',771,'GM-11042','','','','Test',222,'',12 from dual union all
select 12940,'1965-S83','1965-02-06 10:19:00','suborbital rocket','success',839,'AP6.850','','','','Aeron',394,'',100 from dual union all
select 12941,'1965-S84','1965-02-08 20:21:00','military missile','success',795,'1142','Mk 11 RV','OT','PRONTO ROSE','OT',1705,'',900 from dual union all
select 12942,'1965-S85','1965-02-08 17:25:05','military missile','success',932,'A3P-5','','','','OT',421,'SSBN 635',1000 from dual union all
select 12943,'1965-S86','1965-02-08 22:15:00','suborbital rocket','success',839,'NASA 10.126GA','GSFC Grenade 72 (PB)','','','Aeronomy',1039,'',124 from dual union all
select 12944,'1965-S87','1965-02-08 22:53:00','suborbital rocket','success',839,'NASA 10.120GA','GSFC Grenade 73 (W)','','','Aeronomy',1821,'',107 from dual union all
select 12945,'1965-S88','1965-02-08 22:58:00','suborbital rocket','success',839,'NASA 10.123GA','GSFC Grenade 74 (C)','','','Aeronomy',428,'',165 from dual union all
select 12946,'1965-S89','1965-02-11 00:00:00','military missile','success',1019,'','','','','Test',636,'',200 from dual union all
select 12947,'1965-S90','1965-02-11 17:55:00','suborbital rocket','success',30,'AE3.153','Solar spectrum','','','Solar UV/Solar EUV',1869,'',200 from dual union all
select 12948,'1965-S91','1965-02-12 00:59:00','military missile','success',916,'AO-23','P-89','','','OT',437,'',180 from dual union all
select 12949,'1965-S92','1965-02-12 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 618',1000 from dual union all
select 1295,'1952-A227','1952-11-17 00:00:00','suborbital rocket','failure',1490,'N9','','','','Test',666,'',10 from dual union all
select 12950,'1965-S93','1965-02-12 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 618',1000 from dual union all
select 12951,'1965-S94','1965-02-12 00:00:00','suborbital rocket','success',1490,'AGI56','','','','Solar?',668,'',95 from dual union all
select 12952,'1965-S95','1965-02-12 12:00:06','military missile','success',983,'KY-2','','','','Test',636,'',250 from dual union all
select 12953,'1965-S96','1965-02-15 16:30:03','military missile','success',932,'A3P-171','','','','OT',421,'SSBN 630',1000 from dual union all
select 12954,'1965-S97','1965-02-18 20:09:00','suborbital rocket','success',837,'NASA 14.11UA','UM Composition 4','','','Aeronomy',428,'',195 from dual union all
select 12955,'1965-S98','1965-02-19 00:25:00','suborbital rocket','success',837,'AE5.492','TMA/Grenades','','','Aeron',394,'',161 from dual union all
select 12956,'1965-S99','1965-02-19 04:51:02','suborbital rocket','success',30,'NASA 04.129CA','JHU Aurora 3','','','Aeronomy',428,'',159 from dual union all
select 12957,'1965-U01','1965-12-16 14:09:22','orbital','success',1007,'U 22502-01L','OGCh No. 01L','OGCh No. 01L','','Sat',573,'',200 from dual union all
select 12958,'1965-U02','1965-01-19 14:04:00','test rocket','success',1421,'GT-2','Gemini 2','Gemini GT-2','','SC Test',237,'',169 from dual union all
select 12959,'1965-U03','1965-05-04 05:50:00','suborbital rocket','success',839,'Meteor 6/7','Reentry physics','Meteor 6','','Test',1821,'',110 from dual union all
select 1296,'1952-A228','1952-11-18 00:00:00','atmospheric rocket','success',836,'1249A-119','Nike 1249 R&D 138R','','','Test',1895,'',10 from dual union all
select 12960,'1965-U04','1965-05-22 21:55:00','test rocket','success',154,'264D','FIRE 2','FIRE 2','','Reentry Test',227,'',817 from dual union all
select 12961,'1965-U05','1965-03-30 16:09:31','ballistic missile test','success',247,'22-4 (OAR Probe 4)','OAR 22-4/AFCRL-35','AFCRL-35','','Magnetospheric',235,'',16500 from dual union all
select 12962,'1965-U06','1965-04-09 18:10:37','ballistic missile test','success',247,'22-9 (OAR Probe 5)','OAR 22-9/AFWL-14','AWFL-14','','Magnetospheric',235,'',25422 from dual union all
select 12963,'1965-U07','1965-05-12 16:02:35','ballistic missile test','success',247,'22-8 (OAR Probe 6)','OAR 22-8/AFCRL-335','AFCRL-335','','Magnetospheric',235,'',13586 from dual union all
select 12964,'1965-U08','1965-06-09 16:26:15','ballistic missile test','success',247,'22-5 (OAR Probe 7)','OAR 22-5/AFWL-304','AFWL-304','','Magnetospheric',235,'',17533 from dual union all
select 12965,'1965-W01','1965-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12966,'1965-W02','1965-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12967,'1965-W03','1965-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12968,'1965-W04','1965-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual;
insert into LAUNCH
select 12969,'1965-W05','1965-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 1297,'1952-A229','1952-11-18 00:00:00','atmospheric rocket','success',836,'1249A-147','Nike 1249 R&D 139R','','','Test',1895,'',10 from dual union all
select 12970,'1965-W06','1965-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12971,'1965-W07','1965-01-12 10:59:00','suborbital rocket','success',631,'TERLS-18','ISRO 11.08','TERLS-018','','Aeron',1467,'',77 from dual union all
select 12972,'1965-W08','1965-01-13 10:15:00','suborbital rocket','success',631,'TERLS-19','ISRO 11.09','TERLS-019','','Aeron',1467,'',77 from dual union all
select 12973,'1965-W09','1965-01-19 00:00:00','suborbital rocket','success',1186,'LW-27','','','','Test',1895,'',74 from dual union all
select 12974,'1965-W10','1965-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12975,'1965-W100','1965-07-21 02:50:00','suborbital rocket','success',631,'TERLS-26','ISRO 11.14','TERLS-026','','Aeron',1467,'',77 from dual union all
select 12976,'1965-W101','1965-07-21 17:00:00','suborbital rocket','success',273,'28','','','','Aeron',293,'',75 from dual union all
select 12977,'1965-W102','1965-07-23 17:00:00','suborbital rocket','success',273,'29','','','','Aeron',293,'',75 from dual union all
select 12978,'1965-W103','1965-07-28 14:00:00','suborbital rocket','success',273,'30','','','','Aeron',293,'',75 from dual union all
select 12979,'1965-W104','1965-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 1298,'1952-A23','1952-02-07 00:00:00','atmospheric rocket','success',869,'490B','System Test 78R','','','Test',1895,'',10 from dual union all
select 12980,'1965-W105','1965-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12981,'1965-W106','1965-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12982,'1965-W107','1965-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12983,'1965-W108','1965-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12984,'1965-W109','1965-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12985,'1965-W11','1965-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12986,'1965-W110','1965-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12987,'1965-W111','1965-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12988,'1965-W112','1965-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12989,'1965-W113','1965-08-07 13:00:00','suborbital rocket','success',88,'','Arcasonde IA / 10.96','','','Aeron',428,'',60 from dual union all
select 1299,'1952-A230','1952-11-18 00:00:00','suborbital rocket','failure',1490,'N8','','','','Test',666,'',0 from dual union all
select 12990,'1965-W114','1965-08-07 20:45:00','suborbital rocket','success',88,'','Arcasonde IA /10.165','','','Aeron',428,'',60 from dual union all
select 12991,'1965-W115','1965-08-08 05:00:00','suborbital rocket','success',88,'','Arcasonde IA /10.166','','','Aeron',428,'',60 from dual union all
select 12992,'1965-W116','1965-08-08 11:11:00','suborbital rocket','success',88,'','Arcasonde IA /10.167','','','Aeron',428,'',60 from dual union all
select 12993,'1965-W117','1965-08-13 17:22:00','suborbital rocket','unknown',273,'31','','','','Aeron',293,'',75 from dual union all
select 12994,'1965-W118','1965-08-18 07:10:00','suborbital rocket','success',631,'TERLS-27','ISRO 11.15','TERLS-027','','Aeron',1467,'',77 from dual union all
select 12995,'1965-W119','1965-08-18 17:00:00','suborbital rocket','unknown',273,'32','','','','Aeron',293,'',75 from dual union all
select 12996,'1965-W12','1965-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12997,'1965-W120','1965-08-25 17:00:00','suborbital rocket','unknown',273,'33','','','','Aeron',293,'',75 from dual union all
select 12998,'1965-W121','1965-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 12999,'1965-W122','1965-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13,'1943-A06','1943-03-25 00:00:00','military missile','failure',8,'19','','','','Test',675,'',0 from dual union all
select 130,'1944-S118','1944-09-19 14:11:00','military missile','success',8,'18753','Ma159','','','Training',661,'',90 from dual union all
select 1300,'1952-A231','1952-11-19 00:00:00','atmospheric rocket','success',365,'','XF10F-1 Model','','','Test',1821,'',10 from dual union all
select 13000,'1965-W123','1965-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13001,'1965-W124','1965-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13002,'1965-W125','1965-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13003,'1965-W126','1965-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13004,'1965-W127','1965-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13005,'1965-W128','1965-09-01 17:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 13006,'1965-W129','1965-09-08 22:35:00','suborbital rocket','success',88,'EXAMETNET A65-01','Arcasonde IA','','','Aeron',1821,'',53 from dual union all
select 13007,'1965-W13','1965-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13008,'1965-W130','1965-09-09 01:26:00','suborbital rocket','success',88,'EXAMETNET A65-02','Arcasonde IA','','','Aeron',1821,'',52 from dual union all
select 13009,'1965-W131','1965-09-09 00:00:00','suborbital rocket','unknown',273,'34','','','','Aeron',293,'',75 from dual union all
select 1301,'1952-A232','1952-11-19 00:00:00','atmospheric rocket','unknown',576,'64','','','','Test',1895,'',10 from dual union all
select 13010,'1965-W133','1965-09-15 09:31:00','suborbital rocket','success',631,'TERLS-28','ISRO 11.16','TERLS-028','','Aeron',1467,'',77 from dual union all
select 13011,'1965-W134','1965-09-17 17:00:00','suborbital rocket','unknown',273,'35','','','','Aeron',293,'',75 from dual union all
select 13012,'1965-W135','1965-09-22 17:00:00','suborbital rocket','unknown',273,'36','','','','Aeron',293,'',75 from dual union all
select 13013,'1965-W136','1965-09-29 07:00:00','suborbital rocket','success',759,'MT-135-011','','','','Aeron',839,'',51 from dual union all
select 13014,'1965-W137','1965-09-29 16:00:00','suborbital rocket','unknown',273,'37','','','','Aeron',293,'',75 from dual union all
select 13015,'1965-W138','1965-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13016,'1965-W139','1965-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13017,'1965-W14','1965-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13018,'1965-W140','1965-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13019,'1965-W141','1965-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 1302,'1952-A233','1952-11-20 00:00:00','military missile','unknown',339,'37R','','','','Test',1895,'',30 from dual union all
select 13020,'1965-W142','1965-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13021,'1965-W143','1965-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13022,'1965-W144','1965-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13023,'1965-W145','1965-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13024,'1965-W146','1965-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13025,'1965-W147','1965-10-06 17:18:00','suborbital rocket','unknown',273,'38','','','','Aeron',293,'',75 from dual union all
select 13026,'1965-W148','1965-10-13 00:00:00','suborbital rocket','unknown',273,'39','','','','Aeron',293,'',75 from dual union all
select 13027,'1965-W149','1965-10-20 07:05:00','suborbital rocket','success',631,'TERLS-29','ISRO 11.17','TERLS-029','','Aeron',1467,'',77 from dual union all
select 13028,'1965-W15','1965-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13029,'1965-W150','1965-10-20 17:00:00','suborbital rocket','unknown',273,'40','','','','Aeron',293,'',75 from dual union all
select 1303,'1952-A234','1952-11-20 00:00:00','atmospheric rocket','unknown',576,'66','','','','Test',1895,'',10 from dual union all
select 13030,'1965-W151','1965-10-25 04:00:00','suborbital rocket','unknown',88,'','','','','Aeron',1895,'',60 from dual union all
select 13031,'1965-W152','1965-10-27 00:00:00','suborbital rocket','unknown',273,'41','','','','Aeron',293,'',75 from dual union all
select 13032,'1965-W153','1965-10-28 00:00:00','suborbital rocket','success',1186,'EX-1.8','','','','Aeron',1895,'',70 from dual union all
select 13033,'1965-W154','1965-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13034,'1965-W155','1965-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13035,'1965-W156','1965-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13036,'1965-W157','1965-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13037,'1965-W158','1965-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13038,'1965-W159','1965-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13039,'1965-W16','1965-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 1304,'1952-A235','1952-11-20 00:00:00','atmospheric rocket','unknown',576,'65','','','','Test',1895,'',10 from dual union all
select 13040,'1965-W160','1965-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13041,'1965-W161','1965-11-01 17:00:00','suborbital rocket','unknown',273,'42','','','','Aeron',293,'',75 from dual union all
select 13042,'1965-W162','1965-11-08 00:00:00','suborbital rocket','unknown',273,'43','','','','Aeron',293,'',75 from dual union all
select 13043,'1965-W163','1965-11-17 08:36:00','suborbital rocket','success',631,'TERLS-30','ISRO 11.18','TERLS-030','','Aeron',1467,'',77 from dual union all
select 13044,'1965-W164','1965-11-17 17:00:00','suborbital rocket','unknown',273,'44','','','','Aeron',293,'',75 from dual union all
select 13045,'1965-W165','1965-11-18 00:34:00','suborbital rocket','unknown',716,'','DOVER','','','Aeron',394,'',75 from dual union all
select 13046,'1965-W166','1965-11-18 04:15:00','suborbital rocket','unknown',716,'','FOUL BAY','','','Aeron',394,'',75 from dual union all
select 13047,'1965-W167','1965-11-18 05:45:00','suborbital rocket','unknown',716,'','GUN HILL','','','Aeron',394,'',75 from dual union all
select 13048,'1965-W168','1965-11-18 08:30:00','suborbital rocket','unknown',716,'','INDIAN GRND','','','Aeron',394,'',75 from dual union all
select 13049,'1965-W169','1965-11-18 10:08:00','suborbital rocket','unknown',716,'','JAMESTOWN','','','Aeron',394,'',75 from dual union all
select 1305,'1952-A236','1952-11-20 00:00:00','atmospheric rocket','success',836,'1249A-146','Nike 1249 R&D 141R','','','Test',1895,'',10 from dual union all
select 13050,'1965-W17','1965-02-03 07:39:00','suborbital rocket','success',631,'TERLS-20','ISRO 11.10','TERLS-020','','Aeron',1467,'',77 from dual union all
select 13051,'1965-W170','1965-11-18 11:34:00','suborbital rocket','unknown',716,'','INEZ','','','Aeron',394,'',75 from dual union all
select 13052,'1965-W171','1965-11-23 02:27:00','suborbital rocket','success',631,'TERLS-31','ISRO 19.01','TERLS-031','','Aeron',1467,'',69 from dual union all
select 13053,'1965-W172','1965-11-24 03:06:00','suborbital rocket','unknown',88,'','','','','Aeron',1895,'',60 from dual union all
select 13054,'1965-W173','1965-11-24 17:00:00','suborbital rocket','unknown',273,'45','','','','Aeron',293,'',75 from dual union all
select 13055,'1965-W174','1965-11-29 17:00:00','suborbital rocket','unknown',273,'46','','','','Aeron',293,'',75 from dual union all
select 13056,'1965-W175','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13057,'1965-W176','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual;
insert into LAUNCH
select 13058,'1965-W177','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13059,'1965-W178','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 1306,'1952-A237','1952-11-20 00:00:00','atmospheric rocket','success',836,'1249A-145','Nike 1249 R&D 140R','','','Test',1895,'',10 from dual union all
select 13060,'1965-W179','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13061,'1965-W18','1965-02-17 19:04:00','suborbital rocket','success',273,'07','','','','Aeron',293,'',75 from dual union all
select 13062,'1965-W180','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13063,'1965-W181','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13064,'1965-W182','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13065,'1965-W183','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13066,'1965-W184','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13067,'1965-W185','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13068,'1965-W186','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13069,'1965-W187','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 1307,'1952-A238','1952-11-21 00:00:00','atmospheric rocket','unknown',576,'67','','','','Test',1895,'',10 from dual union all
select 13070,'1965-W188','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13071,'1965-W189','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13072,'1965-W19','1965-02-24 15:08:00','suborbital rocket','success',273,'08','','','','Aeron',293,'',75 from dual union all
select 13073,'1965-W190','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13074,'1965-W191','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13075,'1965-W192','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13076,'1965-W193','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13077,'1965-W194','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13078,'1965-W195','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13079,'1965-W196','1965-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 1308,'1952-A239','1952-11-24 00:00:00','atmospheric rocket','success',564,'','D32 Model','','','Test',1821,'',5 from dual union all
select 13080,'1965-W197','1965-12-09 22:15:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13081,'1965-W198','1965-12-08 17:00:00','suborbital rocket','unknown',273,'47','','','','Aeron',293,'',75 from dual union all
select 13082,'1965-W199','1965-12-11 00:00:00','suborbital rocket','success',88,'','','','','Aeron',394,'',60 from dual union all
select 13083,'1965-W20','1965-02-26 17:30:00','suborbital rocket','success',273,'09','','','','Aeron',293,'',75 from dual union all
select 13084,'1965-W200','1965-12-15 03:33:00','suborbital rocket','success',631,'','','','','Weather',873,'',60 from dual union all
select 13085,'1965-W201','1965-12-15 07:35:00','suborbital rocket','success',631,'TERLS-32','ISRO 11.20','TERLS-032','','Aeron',1467,'',77 from dual union all
select 13086,'1965-W202','1965-12-17 06:36:00','suborbital rocket','success',631,'TERLS-33','ISRO 11.21','TERLS-033','','Aeron',1467,'',75 from dual union all
select 13087,'1965-W203','1965-12-17 17:00:00','suborbital rocket','unknown',273,'48','','','','Aeron',293,'',75 from dual union all
select 13088,'1965-W204','1965-12-18 06:47:00','suborbital rocket','success',631,'TERLS-34','ISRO 11.22','TERLS-034','','Aeron',1467,'',75 from dual union all
select 13089,'1965-W205','1965-12-22 16:40:00','suborbital rocket','unknown',273,'49','','','','Aeron',293,'',75 from dual union all
select 1309,'1952-A24','1952-02-07 00:00:00','atmospheric rocket','success',869,'490B','System Test 79R','','','Test',1895,'',10 from dual union all
select 13090,'1965-W206','1965-07-23 00:00:00','suborbital rocket','success',88,'','PSU-Arcas #7','','','Ionos',1895,'',70 from dual union all
select 13091,'1965-W207','1965-01-13 14:10:00','suborbital rocket','success',631,'','SUPARCO','','','Aeron',1419,'',65 from dual union all
select 13092,'1965-W208','1965-02-17 13:25:00','suborbital rocket','success',631,'','SUPARCO 15/65','','','Aeron',1419,'',65 from dual union all
select 13093,'1965-W209','1965-03-17 17:32:00','suborbital rocket','success',631,'','SUPARCO 16/65','','','Aeron',1419,'',65 from dual union all
select 13094,'1965-W21','1965-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13095,'1965-W210','1965-02-16 15:37:00','suborbital rocket','success',88,'','Chute','','','Aeron',276,'',60 from dual union all
select 13096,'1965-W211','1965-02-17 17:00:00','suborbital rocket','success',88,'','Chute','','','Aeron',276,'',60 from dual union all
select 13097,'1965-W212','1965-05-25 17:40:00','suborbital rocket','success',88,'','Chute','','','Aeron',276,'',60 from dual union all
select 13098,'1965-W213','1965-07-30 13:41:00','suborbital rocket','success',88,'','Chute','','','Aeron',276,'',60 from dual union all
select 13099,'1965-W214','1965-10-25 19:38:00','suborbital rocket','success',88,'EXAMETNET A65-03','Arcasonde IA','','','Meteo',1821,'',59 from dual union all
select 131,'1944-S119','1944-09-19 16:09:00','military missile','success',8,'18764','Ma179','','','Training',661,'',90 from dual union all
select 1310,'1952-A240','1952-11-25 00:00:00','atmospheric rocket','success',836,'1249B-1020','Nike Eng 3E','','','Test',1895,'',10 from dual union all
select 13100,'1965-W215','1965-10-26 18:34:00','suborbital rocket','success',88,'EXAMETNET A65-05','Arcasonde IA','','','Meteo',1821,'',55 from dual union all
select 13101,'1965-W216','1965-10-27 19:42:00','suborbital rocket','success',88,'EXAMETNET A65-06','Arcasonde IA','','','Meteo',1821,'',55 from dual union all
select 13102,'1965-W217','1965-11-08 20:13:00','suborbital rocket','success',88,'EXAMETNET A65-09','Arcasonde IA','','','Meteo',1821,'',61 from dual union all
select 13103,'1965-W218','1965-11-10 18:19:00','suborbital rocket','success',88,'EXAMETNET A65-10','Arcasonde IA','','','Meteo',1821,'',56 from dual union all
select 13104,'1965-W219','1965-11-24 17:16:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 13105,'1965-W22','1965-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13106,'1965-W220','1965-12-01 17:34:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 13107,'1965-W221','1965-12-08 20:31:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 13108,'1965-W222','1965-12-15 19:21:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 13109,'1965-W223','1965-09-03 15:25:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 1311,'1952-A241','1952-11-25 00:00:00','atmospheric rocket','success',836,'1249B-1014','Nike Eng 2E','','','Test',1895,'',10 from dual union all
select 13110,'1965-W224','1965-11-11 04:40:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 13111,'1965-W225','1965-11-29 04:29:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 13112,'1965-W226','1965-12-02 06:23:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 13113,'1965-W227','1965-12-09 04:54:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 13114,'1965-W228','1965-12-16 06:01:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 13115,'1965-W229','1965-09-08 21:45:00','suborbital rocket','success',631,'EXAMETNET J65-01','HASP/WOX-1A','','','Meteo',1821,'',54 from dual union all
select 13116,'1965-W23','1965-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13117,'1965-W230','1965-09-08 23:07:00','suborbital rocket','success',631,'EXAMETNET J65-02','HASP/WOX-1A','','','Meteo',1821,'',56 from dual union all
select 13118,'1965-W231','1965-09-09 00:47:00','suborbital rocket','success',631,'EXAMETNET J65-03','HASP/WOX-1A','','','Meteo',1821,'',53 from dual union all
select 13119,'1965-W232','1965-09-09 01:41:00','suborbital rocket','success',631,'EXAMETNET J65-04','HASP/WOX-1A','','','Meteo',1821,'',49 from dual union all
select 1312,'1952-A242','1952-11-26 00:00:00','atmospheric rocket','unknown',576,'68','','','','Test',1895,'',10 from dual union all
select 13120,'1965-W233','1965-09-09 02:25:00','suborbital rocket','success',631,'EXAMETNET J65-05','HASP/WOX-1A','','','Meteo',1821,'',53 from dual union all
select 13121,'1965-W234','1965-09-09 05:01:00','suborbital rocket','success',631,'EXAMETNET J65-06','HASP/WOX-1A','','','Meteo',1821,'',53 from dual union all
select 13122,'1965-W235','1965-09-09 08:05:00','suborbital rocket','success',631,'EXAMETNET J65-07','HASP/WOX-1A','','','Meteo',1821,'',55 from dual union all
select 13123,'1965-W237','1965-09-09 16:48:00','suborbital rocket','success',631,'EXAMETNET J65-10','HASP/WOX-1A','','','Meteo',1821,'',54 from dual union all
select 13124,'1965-W238','1965-09-09 22:34:00','suborbital rocket','success',631,'EXAMETNET J65-11','HASP/WOX-1A','','','Meteo',1821,'',56 from dual union all
select 13125,'1965-W239','1965-09-10 01:15:00','suborbital rocket','success',631,'EXAMETNET J65-12','HASP/WOX-1A','','','Meteo',1821,'',52 from dual union all
select 13126,'1965-W24','1965-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13127,'1965-W240','1965-09-10 05:00:00','suborbital rocket','success',631,'EXAMETNET J65-13','HASP/WOX-1A','','','Meteo',1821,'',52 from dual union all
select 13128,'1965-W241','1965-09-10 08:00:00','suborbital rocket','success',631,'EXAMETNET J65-14','HASP/WOX-1A','','','Meteo',1821,'',54 from dual union all
select 13129,'1965-W242','1965-09-10 12:38:00','suborbital rocket','success',631,'EXAMETNET J65-15','HASP/WOX-1A','','','Meteo',1821,'',53 from dual union all
select 1313,'1952-A243','1952-11-26 00:00:00','atmospheric rocket','unknown',576,'70','','','','Test',1895,'',10 from dual union all
select 13130,'1965-W243','1965-03-11 00:00:00','suborbital rocket','success',716,'HASP','','','','Aeron',1104,'CVE-25',60 from dual union all
select 13131,'1965-W244','1965-03-17 00:00:00','suborbital rocket','success',716,'HASP','','','','Aeron',1106,'CVE-25',60 from dual union all
select 13132,'1965-W245','1965-03-19 00:00:00','suborbital rocket','success',716,'HASP','','','','Aeron',1108,'CVE-25',60 from dual union all
select 13133,'1965-W246','1965-03-22 00:00:00','suborbital rocket','success',716,'HASP','','','','Aeron',1111,'CVE-25',60 from dual union all
select 13134,'1965-W247','1965-03-25 00:00:00','suborbital rocket','success',716,'HASP','','','','Aeron',1113,'CVE-25',60 from dual union all
select 13135,'1965-W248','1965-03-28 00:00:00','suborbital rocket','success',716,'HASP','','','','Aeron',1115,'CVE-25',60 from dual union all
select 13136,'1965-W249','1965-04-08 00:00:00','suborbital rocket','success',716,'HASP','','','','Aeron',1123,'CVE-25',60 from dual union all
select 13137,'1965-W25','1965-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13138,'1965-W250','1965-03-18 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1107,'CVE-25',60 from dual union all
select 13139,'1965-W251','1965-03-21 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1110,'CVE-25',60 from dual union all
select 1314,'1952-A244','1952-11-26 00:00:00','atmospheric rocket','unknown',576,'69','','','','Test',1895,'',10 from dual union all
select 13140,'1965-W252','1965-03-24 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1112,'CVE-25',60 from dual union all
select 13141,'1965-W253','1965-03-27 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1114,'CVE-25',60 from dual union all
select 13142,'1965-W254','1965-04-02 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1116,'CVE-25',60 from dual union all
select 13143,'1965-W255','1965-04-03 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1117,'CVE-25',60 from dual union all
select 13144,'1965-W256','1965-04-04 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1118,'CVE-25',60 from dual union all
select 13145,'1965-W257','1965-04-05 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1119,'CVE-25',60 from dual union all
select 13146,'1965-W258','1965-04-05 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1121,'CVE-25',60 from dual union all
select 13147,'1965-W259','1965-04-06 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1122,'CVE-25',60 from dual;
insert into LAUNCH
select 13148,'1965-W26','1965-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13149,'1965-W260','1965-04-10 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1124,'CVE-25',60 from dual union all
select 1315,'1952-A245','1952-11-26 00:00:00','atmospheric rocket','unknown',576,'71','','','','Test',1895,'',10 from dual union all
select 13150,'1965-W261','1965-04-11 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1125,'CVE-25',60 from dual union all
select 13151,'1965-W262','1965-04-11 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1126,'CVE-25',60 from dual union all
select 13152,'1965-W263','1965-04-13 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1127,'CVE-25',60 from dual union all
select 13153,'1965-W264','1965-04-14 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1128,'CVE-25',60 from dual union all
select 13154,'1965-W265','1965-04-15 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1129,'CVE-25',60 from dual union all
select 13155,'1965-W266','1965-09-09 16:30:00','suborbital rocket','success',631,'EXAMETNET J65-09','WOX-1A','','','Meteo',1821,'',52 from dual union all
select 13156,'1965-W267','1965-10-26 00:15:00','suborbital rocket','success',88,'EXAMETNET A65-04','Arcasonde 1A','','','Meteo',1821,'',59 from dual union all
select 13157,'1965-W268','1965-11-04 19:11:00','suborbital rocket','success',88,'EXAMETNET A65-07','Arcasonde IA','','','Meteo',1821,'',54 from dual union all
select 13158,'1965-W269','1965-10-05 18:51:00','suborbital rocket','success',631,'EXAMETNET J65-16','Chaff','','','Meteo',1821,'',63 from dual union all
select 13159,'1965-W27','1965-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 1316,'1952-A246','1952-11-26 00:00:00','atmospheric rocket','success',836,'1249A-149','Nike 1249 R&D 143R','','','Test',1895,'',10 from dual union all
select 13160,'1965-W270','1965-10-08 19:34:00','suborbital rocket','success',631,'EXAMETNET J65-17','Chaff','','','Meteo',1821,'',60 from dual union all
select 13161,'1965-W271','1965-10-12 18:49:00','suborbital rocket','success',631,'EXAMETNET J65-18','Chaff','','','Meteo',1821,'',65 from dual union all
select 13162,'1965-W272','1965-10-14 19:29:00','suborbital rocket','success',631,'EXAMETNET J65-19','Chaff','','','Meteo',1821,'',60 from dual union all
select 13163,'1965-W273','1965-10-15 20:16:00','suborbital rocket','success',631,'EXAMETNET J65-20','Chaff','','','Meteo',1821,'',63 from dual union all
select 13164,'1965-W274','1965-10-19 18:27:00','suborbital rocket','success',631,'EXAMETNET J65-21','Chaff','','','Meteo',1821,'',59 from dual union all
select 13165,'1965-W275','1965-10-20 18:32:00','suborbital rocket','success',631,'EXAMETNET J65-22','Chaff','','','Meteo',1821,'',65 from dual union all
select 13166,'1965-W276','1965-10-27 23:31:00','suborbital rocket','success',631,'EXAMETNET J65-23','WOX-1A','','','Meteo',1821,'',56 from dual union all
select 13167,'1965-W277','1965-10-29 18:45:00','suborbital rocket','success',631,'EXAMETNET J65-24','WOX-1A','','','Meteo',1821,'',66 from dual union all
select 13168,'1965-W278','1965-11-03 19:27:00','suborbital rocket','success',631,'EXAMETNET J65-25','WOX-1A','','','Meteo',1821,'',52 from dual union all
select 13169,'1965-W279','1965-11-08 19:05:00','suborbital rocket','success',88,'EXAMETNET A65-08','Arcasonde IA','','','Meteo',1821,'',58 from dual union all
select 1317,'1952-A247','1952-11-26 00:00:00','atmospheric rocket','success',836,'1249A-148','Nike 1249 R&D 142R','','','Test',1895,'',10 from dual union all
select 13170,'1965-W28','1965-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13171,'1965-W280','1965-10-20 12:58:00','suborbital rocket','success',631,'','SUPARCO 17/65','','','Aeron',1419,'',65 from dual union all
select 13172,'1965-W281','1965-11-17 12:10:00','suborbital rocket','success',631,'','SUPARCO 18/65','','','Aeron',1419,'',65 from dual union all
select 13173,'1965-W282','1965-09-09 12:01:00','suborbital rocket','success',631,'EXAMETNET J65-08','HASP/WOX-1A','','','Meteo',1821,'',54 from dual union all
select 13174,'1965-W29','1965-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13175,'1965-W30','1965-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13176,'1965-W31','1965-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13177,'1965-W32','1965-03-05 00:00:00','suborbital rocket','success',273,'10','','','','Aeron',293,'',75 from dual union all
select 13178,'1965-W33','1965-03-06 22:00:00','suborbital rocket','success',88,'','ROBIN 933','','','Aeron',1895,'',64 from dual union all
select 13179,'1965-W34','1965-03-07 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1102,'CVE-25',60 from dual union all
select 1318,'1952-A248','1952-11-26 00:00:00','military missile','success',763,'N-25 GM-246','','','','Test',222,'',10 from dual union all
select 13180,'1965-W35','1965-03-09 02:10:00','suborbital rocket','success',759,'MT-135-004','','','','Aeron',839,'',53 from dual union all
select 13181,'1965-W36','1965-03-09 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1103,'CVE-25',60 from dual union all
select 13182,'1965-W37','1965-03-10 16:00:00','suborbital rocket','success',273,'11','','','','Aeron',293,'',75 from dual union all
select 13183,'1965-W38','1965-03-10 19:42:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 13184,'1965-W39','1965-03-11 02:07:00','suborbital rocket','success',759,'MT-135-005','','','','Aeron',839,'',60 from dual union all
select 13185,'1965-W40','1965-03-11 07:07:00','suborbital rocket','success',759,'MT-135-006','','','','Aeron',839,'',60 from dual union all
select 13186,'1965-W41','1965-03-16 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1105,'CVE-25',60 from dual union all
select 13187,'1965-W42','1965-03-16 04:52:30','suborbital rocket','success',257,'NASA 15.01GI','Sylvester I','','','Ionos',29,'',74 from dual union all
select 13188,'1965-W43','1965-03-17 06:55:00','suborbital rocket','success',631,'TERLS-21','ISRO 11.11','TERLS-021','','Aeron',1467,'',77 from dual union all
select 13189,'1965-W44','1965-03-17 17:10:00','suborbital rocket','success',273,'12','','','','Aeron',293,'',75 from dual union all
select 1319,'1952-A249','1952-11-28 00:00:00','military missile','success',703,'610','','','','Test',222,'',8 from dual union all
select 13190,'1965-W45','1965-03-22 10:00:00','suborbital rocket','success',88,'','PSU-Arcas #4','','','Ionos',1895,'',70 from dual union all
select 13191,'1965-W46','1965-03-23 05:59:30','suborbital rocket','success',257,'NASA 15.02GI','Sylvester II','','','Ionos',29,'',76 from dual union all
select 13192,'1965-W47','1965-03-24 17:45:00','suborbital rocket','success',273,'13','','','','Aeron',293,'',75 from dual union all
select 13193,'1965-W48','1965-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13194,'1965-W49','1965-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13195,'1965-W50','1965-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13196,'1965-W51','1965-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13197,'1965-W52','1965-03-31 00:00:00','suborbital rocket','success',1186,'EX-1.8','','','','Aeron',394,'',70 from dual union all
select 13198,'1965-W53','1965-04-01 00:00:00','suborbital rocket','success',1215,'1','','','','Test',330,'',65 from dual union all
select 13199,'1965-W54','1965-03-31 17:01:00','suborbital rocket','success',273,'14','','','','Aeron',293,'',75 from dual union all
select 132,'1944-S12','1944-03-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1320,'1952-A25','1952-02-11 00:00:00','atmospheric rocket','unknown',576,'20','','','','Test',1895,'',10 from dual union all
select 13200,'1965-W55','1965-04-07 17:00:00','suborbital rocket','success',273,'15','','','','Aeron',293,'',75 from dual union all
select 13201,'1965-W56','1965-04-14 17:44:00','suborbital rocket','success',273,'16','','','','Aeron',293,'',75 from dual union all
select 13202,'1965-W57','1965-04-17 17:00:00','suborbital rocket','success',273,'','','','','Aeron',293,'',75 from dual union all
select 13203,'1965-W58','1965-04-21 17:15:00','suborbital rocket','success',273,'17','','','','Aeron',293,'',75 from dual union all
select 13204,'1965-W59','1965-04-28 00:00:00','suborbital rocket','success',273,'18','','','','Aeron',293,'',75 from dual union all
select 13205,'1965-W60','1965-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13206,'1965-W61','1965-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13207,'1965-W62','1965-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13208,'1965-W63','1965-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13209,'1965-W64','1965-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 1321,'1952-A250','1952-12-03 00:00:00','military missile','unknown',339,'38R','','','','Test',1895,'',30 from dual union all
select 13210,'1965-W65','1965-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13211,'1965-W66','1965-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13212,'1965-W67','1965-05-04 15:07:00','suborbital rocket','success',725,'','Error for May 5','','','Aeron',1203,'SHOK',85 from dual union all
select 13213,'1965-W68','1965-05-12 19:00:00','suborbital rocket','success',273,'19','','','','Aeron',293,'',75 from dual union all
select 13214,'1965-W69','1965-05-19 07:04:00','suborbital rocket','success',631,'TERLS-24','ISRO 11.12','TERLS-024','','Aeron',1467,'',77 from dual union all
select 13215,'1965-W70','1965-05-23 02:02:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 13216,'1965-W71','1965-05-23 14:00:00','suborbital rocket','unknown',88,'','','','','Aeron',176,'',60 from dual union all
select 13217,'1965-W72','1965-05-26 17:00:00','suborbital rocket','success',273,'20','','','','Aeron',293,'',75 from dual union all
select 13218,'1965-W73','1965-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13219,'1965-W74','1965-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 1322,'1952-A251','1952-12-03 00:00:00','atmospheric rocket','unknown',576,'72','','','','Test',1895,'',10 from dual union all
select 13220,'1965-W75','1965-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13221,'1965-W76','1965-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13222,'1965-W77','1965-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13223,'1965-W78','1965-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13224,'1965-W79','1965-06-02 17:00:00','suborbital rocket','success',273,'21','','','','Aeron',293,'',75 from dual union all
select 13225,'1965-W80','1965-06-11 17:00:00','suborbital rocket','success',273,'22','','','','Aeron',293,'',75 from dual union all
select 13226,'1965-W81','1965-06-16 07:04:00','suborbital rocket','success',631,'TERLS-25','ISRO 11.13','TERLS-025','','Aeron',1467,'',77 from dual union all
select 13227,'1965-W82','1965-06-16 18:00:00','suborbital rocket','success',273,'23','','','','Aeron',293,'',75 from dual union all
select 13228,'1965-W83','1965-06-23 17:25:00','suborbital rocket','success',273,'24','','','','Aeron',293,'',75 from dual union all
select 13229,'1965-W84','1965-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 1323,'1952-A252','1952-12-03 00:00:00','atmospheric rocket','success',836,'1249B-1037','Nike 1249 R&D 145R','','','Test',1895,'',10 from dual union all
select 13230,'1965-W85','1965-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13231,'1965-W86','1965-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13232,'1965-W87','1965-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13233,'1965-W88','1965-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13234,'1965-W89','1965-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13235,'1965-W90','1965-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13236,'1965-W91','1965-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 13237,'1965-W92','1965-06-30 15:03:00','suborbital rocket','success',273,'25','','','','Aeron',293,'',75 from dual;
insert into LAUNCH
select 13238,'1965-W93','1965-07-07 15:00:00','suborbital rocket','success',273,'26','','','','Aeron',293,'',75 from dual union all
select 13239,'1965-W94','1965-07-08 02:26:00','suborbital rocket','success',759,'MT-135-007','','','','Aeron',839,'',50 from dual union all
select 1324,'1952-A253','1952-12-03 00:00:00','atmospheric rocket','success',836,'1249A-150','Nike 1249 R&D 144R','','','Test',1895,'',10 from dual union all
select 13240,'1965-W95','1965-07-08 11:10:00','suborbital rocket','success',759,'MT-135-008','','','','Aeron',839,'',50 from dual union all
select 13241,'1965-W96','1965-07-12 02:07:00','suborbital rocket','failure',759,'MT-135-009','','','','Aeron',839,'',50 from dual union all
select 13242,'1965-W97','1965-07-13 07:50:00','suborbital rocket','success',88,'','PSU-Arcas #5','','','Ionos',1895,'',74 from dual union all
select 13243,'1965-W98','1965-07-14 17:09:00','suborbital rocket','failure',273,'27','','','','Aeron',293,'',75 from dual union all
select 13244,'1965-W99','1965-07-15 05:00:00','suborbital rocket','success',88,'','PSU-Arcas #6','','','Ionos',1895,'',74 from dual union all
select 13245,'1965-Y01','1965-01-01 00:00:00','suborbital rocket','success',1287,'','DFH test','','','Tech',819,'',83 from dual union all
select 13246,'1965-Y02','1965-01-11 12:15:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13247,'1965-Y03','1965-01-13 13:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13248,'1965-Y04','1965-01-15 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13249,'1965-Y05','1965-01-18 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 1325,'1952-A254','1952-12-04 00:00:00','military missile','success',703,'599','','','','Test',222,'',8 from dual union all
select 13250,'1965-Y06','1965-01-20 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13251,'1965-Y07','1965-01-20 16:46:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13252,'1965-Y08','1965-01-22 12:30:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13253,'1965-Y09','1965-01-22 17:42:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13254,'1965-Y10','1965-01-22 18:42:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13255,'1965-Y100','1965-06-09 11:02:00','suborbital rocket','success',725,'','','','','Aeron',1191,'SHOK',85 from dual union all
select 13256,'1965-Y101','1965-06-09 12:16:00','suborbital rocket','success',725,'','','','','Aeron',1155,'VOKV',85 from dual union all
select 13257,'1965-Y102','1965-06-10 11:00:00','suborbital rocket','success',725,'','','','','Aeron',1192,'SHOK',85 from dual union all
select 13258,'1965-Y103','1965-06-10 12:01:00','suborbital rocket','success',725,'','','','','Aeron',1156,'VOKV',85 from dual union all
select 13259,'1965-Y104','1965-06-11 11:00:00','suborbital rocket','success',725,'','','','','Aeron',1194,'SHOK',85 from dual union all
select 1326,'1952-A255','1952-12-04 00:00:00','military missile','success',771,'GM-553','','','','Test',222,'',12 from dual union all
select 13260,'1965-Y105','1965-06-11 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1157,'VOKV',85 from dual union all
select 13261,'1965-Y106','1965-06-13 12:05:00','suborbital rocket','success',725,'','','','','Aeron',1160,'VOKV',85 from dual union all
select 13262,'1965-Y107','1965-06-13 12:21:00','suborbital rocket','success',725,'','','','','Aeron',1195,'SHOK',85 from dual union all
select 13263,'1965-Y108','1965-06-13 15:25:00','suborbital rocket','success',725,'','','','','Aeron',1159,'VOKV',85 from dual union all
select 13264,'1965-Y109','1965-06-14 11:00:00','suborbital rocket','success',725,'','','','','Aeron',1196,'SHOK',85 from dual union all
select 13265,'1965-Y11','1965-02-10 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13266,'1965-Y110','1965-06-14 12:05:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13267,'1965-Y111','1965-06-14 12:05:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13268,'1965-Y112','1965-06-14 12:17:00','suborbital rocket','success',725,'','','','','Aeron',1161,'VOKV',85 from dual union all
select 13269,'1965-Y113','1965-06-15 11:27:00','suborbital rocket','success',725,'','','','','Aeron',1162,'VOKV',85 from dual union all
select 1327,'1952-A256','1952-12-04 00:00:00','atmospheric rocket','success',836,'1249A-153','Nike 1249 R&D 147R','','','Test',1895,'',10 from dual union all
select 13270,'1965-Y114','1965-06-15 11:51:00','suborbital rocket','success',725,'','','','','Aeron',1197,'SHOK',85 from dual union all
select 13271,'1965-Y115','1965-06-15 12:25:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13272,'1965-Y116','1965-06-15 15:09:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13273,'1965-Y117','1965-06-16 11:53:00','suborbital rocket','success',725,'','','','','Aeron',1198,'SHOK',85 from dual union all
select 13274,'1965-Y118','1965-06-16 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13275,'1965-Y119','1965-06-18 12:10:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13276,'1965-Y12','1965-02-16 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13277,'1965-Y120','1965-06-18 17:05:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13278,'1965-Y121','1965-06-19 12:51:00','suborbital rocket','success',725,'','','','','Aeron',1163,'VOKV',85 from dual union all
select 13279,'1965-Y122','1965-06-21 15:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 1328,'1952-A257','1952-12-04 00:00:00','atmospheric rocket','success',836,'1249A-154','Nike 1249 R&D 148R','','','Test',1895,'',10 from dual union all
select 13280,'1965-Y123','1965-06-21 21:15:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13281,'1965-Y124','1965-06-23 11:53:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13282,'1965-Y125','1965-06-23 12:10:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13283,'1965-Y126','1965-06-23 12:10:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13284,'1965-Y127','1965-07-06 09:30:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13285,'1965-Y128','1965-07-09 08:15:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13286,'1965-Y129','1965-07-13 08:15:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13287,'1965-Y13','1965-02-18 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13288,'1965-Y130','1965-07-21 08:02:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13289,'1965-Y131','1965-07-22 08:17:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 1329,'1952-A258','1952-12-04 00:00:00','atmospheric rocket','success',836,'1249B-1043','Nike 1249 R&D 149R','','','Test',1895,'',10 from dual union all
select 13290,'1965-Y132','1965-07-22 13:15:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13291,'1965-Y133','1965-07-23 08:16:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13292,'1965-Y134','1965-07-26 12:15:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13293,'1965-Y135','1965-07-27 08:17:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13294,'1965-Y136','1965-07-28 12:15:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13295,'1965-Y137','1965-08-12 08:16:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13296,'1965-Y138','1965-08-13 12:15:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13297,'1965-Y139','1965-08-17 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13298,'1965-Y14','1965-02-19 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13299,'1965-Y140','1965-08-18 12:15:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 133,'1944-S120','1944-09-20 00:00:00','military missile','success',1470,'','Lille','','','Weapon',1730,'',95 from dual union all
select 1330,'1952-A259','1952-12-04 00:00:00','atmospheric rocket','success',836,'1249A-124','Nike 1249 R&D 146R','','','Test',1895,'',10 from dual union all
select 13300,'1965-Y141','1965-09-08 08:17:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13301,'1965-Y142','1965-09-15 08:17:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13302,'1965-Y143','1965-09-19 12:35:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13303,'1965-Y144','1965-09-20 12:40:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13304,'1965-Y145','1965-09-22 10:13:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13305,'1965-Y146','1965-09-22 12:25:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13306,'1965-Y147','1965-09-23 09:16:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13307,'1965-Y148','1965-09-24 12:05:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13308,'1965-Y149','1965-10-06 11:16:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13309,'1965-Y15','1965-03-01 13:12:00','suborbital rocket','success',257,'NASA 15.03II','UJO-5 (Uppsala)','','','Ionos',29,'',95 from dual union all
select 1331,'1952-A26','1952-02-11 00:00:00','atmospheric rocket','unknown',576,'21','','','','Test',1895,'',10 from dual union all
select 13310,'1965-Y150','1965-10-19 12:11:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13311,'1965-Y151','1965-10-20 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13312,'1965-Y152','1965-10-27 12:06:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13313,'1965-Y153','1965-10-28 11:07:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13314,'1965-Y154','1965-10-29 08:36:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13315,'1965-Y155','1965-11-16 12:03:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13316,'1965-Y156','1965-11-17 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13317,'1965-Y157','1965-11-18 13:44:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13318,'1965-Y158','1965-11-24 12:02:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13319,'1965-Y159','1965-12-02 02:19:00','suborbital rocket','success',257,'NASA 15.20GI-II','Arcas I','','','Ionos',29,'',80 from dual union all
select 1332,'1952-A260','1952-12-05 00:00:00','atmospheric rocket','unknown',576,'73','','','','Test',1895,'',10 from dual union all
select 13320,'1965-Y16','1965-03-02 20:09:00','suborbital rocket','success',257,'NASA 15.04II','UJO-7','','','Ionos',29,'',85 from dual union all
select 13321,'1965-Y160','1965-12-06 13:58:30','suborbital rocket','success',257,'NASA 15.19GI','Sylvester III','','','Ionos',29,'',85 from dual union all
select 13322,'1965-Y161','1965-12-13 12:35:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13323,'1965-Y162','1965-12-15 12:10:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13324,'1965-Y163','1965-12-17 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13325,'1965-Y17','1965-03-08 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13326,'1965-Y18','1965-03-08 15:17:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13327,'1965-Y19','1965-03-10 07:45:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual;
insert into LAUNCH
select 13328,'1965-Y20','1965-03-10 12:05:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13329,'1965-Y21','1965-03-10 15:15:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 1333,'1952-A261','1952-12-05 00:00:00','atmospheric rocket','unknown',576,'74','','','','Test',1895,'',10 from dual union all
select 13330,'1965-Y22','1965-03-10 17:15:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13331,'1965-Y23','1965-03-12 12:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13332,'1965-Y24','1965-03-15 07:45:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13333,'1965-Y25','1965-03-15 12:05:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13334,'1965-Y26','1965-03-17 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13335,'1965-Y27','1965-03-17 14:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13336,'1965-Y28','1965-03-18 12:05:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13337,'1965-Y29','1965-03-18 12:30:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 13338,'1965-Y30','1965-03-19 12:05:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13339,'1965-Y31','1965-03-19 14:41:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 1334,'1952-A262','1952-12-09 00:00:00','military missile','unknown',339,'39R','','','','Test',1895,'',30 from dual union all
select 13340,'1965-Y32','1965-04-14 12:15:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13341,'1965-Y33','1965-04-17 09:44:00','suborbital rocket','success',725,'','','','','Aeron',1171,'SHOK',85 from dual union all
select 13342,'1965-Y34','1965-04-18 14:16:00','suborbital rocket','success',725,'','','','','Aeron',1182,'SHOK',85 from dual union all
select 13343,'1965-Y35','1965-04-20 12:15:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13344,'1965-Y36','1965-04-22 12:10:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13345,'1965-Y37','1965-04-28 13:03:00','suborbital rocket','success',725,'','','','','Aeron',1193,'SHOK',85 from dual union all
select 13346,'1965-Y38','1965-04-30 10:30:00','suborbital rocket','success',725,'','','','','Aeron',1147,'VOKV',85 from dual union all
select 13347,'1965-Y39','1965-04-30 12:00:00','suborbital rocket','success',725,'','Ion density','','','Aeron',1199,'SHOK',85 from dual union all
select 13348,'1965-Y40','1965-04-30 14:17:30','suborbital rocket','success',725,'','','','','Aeron',1136,'VOKV',85 from dual union all
select 13349,'1965-Y41','1965-05-01 11:24:00','suborbital rocket','success',725,'','','','','Aeron',1158,'VOKV',85 from dual union all
select 1335,'1952-A263','1952-12-09 00:00:00','atmospheric rocket','success',836,'1249A-156','Nike 1249 R&D 150R','','','Test',1895,'',10 from dual union all
select 13350,'1965-Y42','1965-05-01 11:59:00','suborbital rocket','success',725,'','','','','Aeron',1200,'SHOK',85 from dual union all
select 13351,'1965-Y43','1965-05-02 12:09:00','suborbital rocket','success',725,'','','','','Aeron',1201,'SHOK',85 from dual union all
select 13352,'1965-Y44','1965-05-03 12:11:00','suborbital rocket','success',725,'','','','','Aeron',1202,'SHOK',85 from dual union all
select 13353,'1965-Y45','1965-05-04 09:41:00','suborbital rocket','success',725,'','','','','Aeron',1165,'VOKV',85 from dual union all
select 13354,'1965-Y46','1965-05-04 12:11:00','suborbital rocket','success',725,'','','','','Aeron',1164,'VOKV',85 from dual union all
select 13355,'1965-Y47','1965-05-05 13:05:00','suborbital rocket','success',725,'','','','','Aeron',1204,'SHOK',85 from dual union all
select 13356,'1965-Y48','1965-05-05 15:07:00','suborbital rocket','success',725,'','','','','Aeron',1203,'SHOK',85 from dual union all
select 13357,'1965-Y49','1965-05-06 08:58:00','suborbital rocket','success',725,'','','','','Aeron',1166,'VOKV',85 from dual union all
select 13358,'1965-Y50','1965-05-06 11:58:00','suborbital rocket','success',725,'','','','','Aeron',1172,'SHOK',85 from dual union all
select 13359,'1965-Y51','1965-05-07 09:57:00','suborbital rocket','success',725,'','','','','Aeron',1167,'VOKV',85 from dual union all
select 1336,'1952-A264','1952-12-11 00:00:00','atmospheric rocket','success',836,'1249A-155','Nike 1249 R&D 152R','','','Test',1895,'',10 from dual union all
select 13360,'1965-Y52','1965-05-07 12:22:00','suborbital rocket','success',725,'','','','','Aeron',1173,'SHOK',85 from dual union all
select 13361,'1965-Y53','1965-05-08 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1174,'SHOK',85 from dual union all
select 13362,'1965-Y54','1965-05-08 13:51:00','suborbital rocket','success',725,'','Error for May 9','','','Aeron',1168,'VOKV',85 from dual union all
select 13363,'1965-Y55','1965-05-09 08:59:00','suborbital rocket','success',725,'','','','','Aeron',1169,'VOKV',85 from dual union all
select 13364,'1965-Y56','1965-05-09 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1175,'SHOK',85 from dual union all
select 13365,'1965-Y57','1965-05-09 13:51:00','suborbital rocket','success',725,'','','','','Aeron',1168,'VOKV',85 from dual union all
select 13366,'1965-Y58','1965-05-10 09:43:00','suborbital rocket','success',725,'','','','','Aeron',1137,'VOKV',85 from dual union all
select 13367,'1965-Y59','1965-05-10 12:42:00','suborbital rocket','success',725,'','','','','Aeron',1176,'SHOK',85 from dual union all
select 13368,'1965-Y60','1965-05-11 09:59:00','suborbital rocket','success',725,'','','','','Aeron',1138,'VOKV',85 from dual union all
select 13369,'1965-Y61','1965-05-11 11:00:00','suborbital rocket','success',725,'','','','','Aeron',1177,'SHOK',85 from dual union all
select 1337,'1952-A265','1952-12-11 00:00:00','atmospheric rocket','success',836,'1249A-151','Nike 1249 R&D 151R','','','Test',1895,'',10 from dual union all
select 13370,'1965-Y62','1965-05-12 09:05:00','suborbital rocket','success',725,'','','','','Aeron',1139,'VOKV',85 from dual union all
select 13371,'1965-Y63','1965-05-12 11:32:00','suborbital rocket','success',725,'','','','','Aeron',1178,'SHOK',85 from dual union all
select 13372,'1965-Y64','1965-05-12 12:15:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13373,'1965-Y65','1965-05-13 10:00:00','suborbital rocket','success',725,'','0230LT','','','Aeron',1179,'SHOK',85 from dual union all
select 13374,'1965-Y66','1965-05-14 08:59:00','suborbital rocket','success',725,'','','','','Aeron',1140,'VOKV',85 from dual union all
select 13375,'1965-Y67','1965-05-15 09:13:00','suborbital rocket','success',725,'','','','','Aeron',1141,'VOKV',85 from dual union all
select 13376,'1965-Y68','1965-05-15 10:41:00','suborbital rocket','success',725,'','','','','Aeron',1180,'SHOK',85 from dual union all
select 13377,'1965-Y69','1965-05-16 10:00:00','suborbital rocket','success',725,'','','','','Aeron',1181,'SHOK',85 from dual union all
select 13378,'1965-Y70','1965-05-17 10:00:00','suborbital rocket','success',725,'','','','','Aeron',1183,'SHOK',85 from dual union all
select 13379,'1965-Y71','1965-05-18 12:15:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 1338,'1952-A266','1952-12-12 00:00:00','atmospheric rocket','success',365,'','D109/HPAG seeker','','','Test',1821,'',10 from dual union all
select 13380,'1965-Y72','1965-05-19 12:15:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13381,'1965-Y73','1965-05-20 12:05:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 13382,'1965-Y74','1965-05-25 19:59:00','suborbital rocket','success',257,'NASA 15.18GI','Kiwi 1 range test','','','Test',823,'',88 from dual union all
select 13383,'1965-Y75','1965-05-26 10:58:00','suborbital rocket','success',725,'','','','','Aeron',1142,'VOKV',85 from dual union all
select 13384,'1965-Y76','1965-05-27 10:53:00','suborbital rocket','success',725,'','','','','Aeron',1184,'SHOK',85 from dual union all
select 13385,'1965-Y77','1965-05-28 10:42:00','suborbital rocket','success',725,'','','','','Aeron',1185,'SHOK',85 from dual union all
select 13386,'1965-Y78','1965-05-28 22:03:00','suborbital rocket','success',725,'','','','','Aeron',1143,'VOKV',85 from dual union all
select 13387,'1965-Y79','1965-05-29 10:58:00','suborbital rocket','success',725,'','','','','Aeron',1186,'SHOK',85 from dual union all
select 13388,'1965-Y80','1965-05-29 22:06:00','suborbital rocket','success',725,'','','','','Aeron',1144,'VOKV',85 from dual union all
select 13389,'1965-Y81','1965-05-30 19:00:00','suborbital rocket','success',257,'NASA 15.05GI','Kiwi 2','','','Ionos/Eclipse',823,'',88 from dual union all
select 1339,'1952-A267','1952-12-15 00:00:00','atmospheric rocket','unknown',576,'76','','','','Test',1895,'',10 from dual union all
select 13390,'1965-Y82','1965-05-30 19:20:24','suborbital rocket','success',257,'NASA 15.06GI','Kiwi 3','','','Ionos/Eclipse',824,'',88 from dual union all
select 13391,'1965-Y83','1965-05-30 19:40:05','suborbital rocket','success',257,'NASA 15.07GI','Kiwi 4','','','Ionos/Eclipse',823,'',88 from dual union all
select 13392,'1965-Y84','1965-05-30 20:00:01','suborbital rocket','success',257,'NASA 15.08GI','Kiwi 5','','','Ionos/Eclipse',824,'',88 from dual union all
select 13393,'1965-Y85','1965-05-30 20:20:01','suborbital rocket','success',257,'NASA 15.09GI','Kiwi 6','','','Ionos/Eclipse',823,'',88 from dual union all
select 13394,'1965-Y86','1965-05-30 21:00:02','suborbital rocket','success',257,'NASA 15.10GI','Kiwi 7','','','Ionos/Eclipse',824,'',88 from dual union all
select 13395,'1965-Y87','1965-05-30 22:01:00','suborbital rocket','success',725,'','','','','Aeron',1145,'VOKV',85 from dual union all
select 13396,'1965-Y88','1965-05-30 23:09:00','suborbital rocket','success',725,'','','','','Aeron',1146,'VOKV',85 from dual union all
select 13397,'1965-Y89','1965-06-02 14:45:00','suborbital rocket','success',725,'','','','','Aeron',1148,'VOKV',85 from dual union all
select 13398,'1965-Y90','1965-06-03 11:00:00','suborbital rocket','success',725,'','','','','Aeron',1187,'SHOK',85 from dual union all
select 13399,'1965-Y91','1965-06-03 12:32:00','suborbital rocket','success',725,'','','','','Aeron',1149,'VOKV',85 from dual union all
select 134,'1944-S121','1944-09-20 00:00:00','military missile','success',1470,'','Arras','','','Weapon',1730,'',95 from dual union all
select 1340,'1952-A268','1952-12-15 00:00:00','atmospheric rocket','unknown',576,'75','','','','Test',1895,'',10 from dual union all
select 13400,'1965-Y92','1965-06-04 12:06:00','suborbital rocket','success',725,'','','','','Aeron',1150,'VOKV',85 from dual union all
select 13401,'1965-Y93','1965-06-05 11:02:00','suborbital rocket','success',725,'','','','','Aeron',1188,'SHOK',85 from dual union all
select 13402,'1965-Y94','1965-06-05 11:55:00','suborbital rocket','success',725,'','','','','Aeron',1151,'VOKV',85 from dual union all
select 13403,'1965-Y95','1965-06-06 11:00:00','suborbital rocket','success',725,'','','','','Aeron',1189,'SHOK',85 from dual union all
select 13404,'1965-Y96','1965-06-06 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1152,'VOKV',85 from dual union all
select 13405,'1965-Y97','1965-06-07 11:30:00','suborbital rocket','success',725,'','','','','Aeron',1190,'SHOK',85 from dual union all
select 13406,'1965-Y98','1965-06-07 11:58:00','suborbital rocket','success',725,'','','','','Aeron',1153,'VOKV',85 from dual union all
select 13407,'1965-Y99','1965-06-08 12:35:00','suborbital rocket','success',725,'','','','','Aeron',1154,'VOKV',85 from dual union all
select 13408,'1966-001','1966-01-07 08:20:00','orbital','success',1509,'U 15001- 08','Kosmos-104','Zenit-2 No. 36','','Sat',562,'',null from dual union all
select 13409,'1966-002','1966-01-19 20:10:00','orbital','success',177,'7114','AFP-206','GAMBIT SV 974','BLANKET PARTY','Sat',1773,'',null from dual union all
select 1341,'1952-A269','1952-12-15 00:00:00','military missile','success',771,'GM-2356','','','','Test',222,'',12 from dual union all
select 13410,'1966-003','1966-01-22 08:30:00','orbital','success',1509,'U 15001- 10','Kosmos-105','Zenit-2 No. 38','','Sat',562,'',null from dual union all
select 13411,'1966-004','1966-01-25 12:28:00','orbital','success',678,'','Kosmos-106','DS-P1-I No. 1','','Sat',626,'',null from dual union all
select 13412,'1966-005','1966-01-28 17:06:00','orbital','success',1125,'S142C','NNS O-7  (N-6)','NNS O-7','INVENTORY AID','Sat',1769,'',null from dual union all
select 13413,'1966-006','1966-01-31 11:41:37','deep space','success',809,'U 15000- 49','Luna-9','E-6M No. 202 (13)','','Sat',562,'',null from dual union all
select 13414,'1966-007','1966-02-02 21:32:13','orbital','success',1394,'450/FTV 1623','CORONA 105/1029','CORONA J-27','SEA LEVEL','Sat TAG125',1770,'',null from dual union all
select 13415,'1966-008','1966-02-03 07:41:23','orbital','success',1378,'445/D36','ESSA 1','Tiros OT3','','Sat',233,'',null from dual union all
select 13416,'1966-009','1966-02-09 19:45:01','orbital','success',1394,'428/FTV 2703','OPS 1439','Ferret 9','IRON BACK','Sat TAG126',1674,'',null from dual union all
select 13417,'1966-010','1966-02-10 08:40:00','orbital','success',1509,'U 15001- 06','Kosmos-107','Zenit-2 No. 34','','Sat',562,'',null from dual;
insert into LAUNCH
select 13418,'1966-011','1966-02-11 18:00:00','orbital','success',681,'','Kosmos-108','DS-U1-G No. 1','','Sat',626,'',null from dual union all
select 13419,'1966-012','1966-02-15 20:32:00','orbital','success',177,'7115','AFP-206','GAMBIT SV 975','MUCHO GRANDE','Sat',1773,'',null from dual union all
select 1342,'1952-A27','1952-02-11 00:00:00','atmospheric rocket','unknown',576,'22','','','','Test',1895,'',10 from dual union all
select 13420,'1966-013','1966-02-17 08:33:36','orbital','success',431,'No. 2','D-1A','Diapason D-1A','','Sat',669,'',null from dual union all
select 13421,'1966-014','1966-02-19 08:49:00','orbital','success',1508,'U 15001- 02','Kosmos-109','Zenit-4 No. 14','','Sat',562,'',null from dual union all
select 13422,'1966-015','1966-02-22 20:09:36','orbital','success',1508,'R 15000- 06','Kosmos-110','Voskhod 3KV No. 5','','Sat',562,'',null from dual union all
select 13423,'1966-016','1966-02-28 13:58:00','orbital','success',1381,'461/D37','ESSA 2','Tiros OT2','','Sat',234,'',null from dual union all
select 13424,'1966-017','1966-03-01 11:03:49','orbital','failure',809,'U 15000- 50','Kosmos-111','E-6S No. 204 ISL','','Sat',562,'',194 from dual union all
select 13425,'1966-018','1966-03-09 22:02:03','orbital','success',1394,'452/FTV 1622','CORONA 106/1030','CORONA J-29','EASY CHAIR','Sat TAG127',1678,'',null from dual union all
select 13426,'1966-019','1966-03-16 15:00:03','orbital','success',177,'5302','GATV 5003','TDA 3','','Sat',229,'',null from dual union all
select 13427,'1966-020','1966-03-16 16:41:02','orbital','success',1421,'GT-8','Gemini 8','Gemini SC8','','Sat',237,'',null from dual union all
select 13428,'1966-021','1966-03-17 10:28:42','orbital','success',1509,'U 15001- 09','Kosmos-112','Zenit-2 No. 37','','Sat',473,'',null from dual union all
select 13429,'1966-022','1966-03-18 20:30:00','orbital','success',177,'7116','AFP-206','GAMBIT SV 976','DUMB DORA','Sat',1773,'',null from dual union all
select 1343,'1952-A270','1952-12-16 00:00:00','military missile','unknown',339,'40R','','','','Test',1895,'',30 from dual union all
select 13430,'1966-023','1966-03-21 09:40:00','orbital','success',1508,'U 15001- 05','Kosmos-113','Zenit-4 No. 17','','Sat',562,'',null from dual union all
select 13431,'1966-024','1966-03-26 03:31:00','orbital','success',1125,'S143C','NNS O-8  (N-7)','NNS O-8','BEST GIRL','Sat',1769,'',null from dual union all
select 13432,'1966-025','1966-03-30 09:20:12','orbital','success',154,'72D   NB20.152  RS','OV1-4/OV1-5/Pod 28','OV1-4S','BRONZE BELL','Sat',1685,'',null from dual union all
select 13433,'1966-026','1966-03-31 05:41:04','orbital','success',1363,'147','DAPP/OPS 0340','DAPP 15','RESORT HOTEL','Sat',1660,'',null from dual union all
select 13434,'1966-027','1966-03-31 10:46:59','deep space','success',809,'N 15000- 51/N10351','Luna-10','E-6S No. 206 ISL','','Sat',562,'',null from dual union all
select 13435,'1966-028','1966-04-06 11:40:00','orbital','success',1508,'U 15001- 07 *','Kosmos-114','Zenit-4 No. 19','','Sat',473,'',null from dual union all
select 13436,'1966-029','1966-04-07 22:02:55','orbital','success',1394,'474/FTV 1627','CORONA 107/1031','CORONA J-30','GAPING WOUND','Sat TAG128',1770,'',null from dual union all
select 13437,'1966-030','1966-04-08 01:00:02','orbital','success',153,'AC-8','SM-2','Surveyor Model 2 (SM-2)','','Sat',269,'',null from dual union all
select 13438,'1966-031','1966-04-08 19:35:00','orbital','success',177,'5001 (AA15)','OAO A1','OAO A1','','Sat',227,'',null from dual union all
select 13439,'1966-032','1966-04-19 19:12:00','orbital','success',177,'7117','AFP-206','GAMBIT SV 977','SHALLOW STREAM','Sat',1773,'',null from dual union all
select 1344,'1952-A271','1952-12-16 00:00:00','military missile','failure',703,'605','','','','Test',222,'',0 from dual union all
select 13440,'1966-033','1966-04-20 10:40:00','orbital','success',1509,'U 15001- 07','Kosmos-115','Zenit-2 No. 35','','Sat',562,'',null from dual union all
select 13441,'1966-034','1966-04-22 09:45:00','orbital','success',1128,'S145C','OV3-1','OV3-1','LABRADOR RETRIEVER','Sat',1769,'',null from dual union all
select 13442,'1966-035','1966-04-25 07:10:00','orbital','success',801,'N 15000- 37','Molniya-1','Molniya-1 11F67 No. 6','','Sat',562,'',null from dual union all
select 13443,'1966-036','1966-04-26 10:04:00','orbital','success',678,'','Kosmos-116','DS-P1-Yu No. 6','','Sat',626,'',null from dual union all
select 13444,'1966-037','1966-05-06 10:58:00','orbital','success',1509,'N 15001- 01','Kosmos-117','Zenit-2 No. 39','','Sat',562,'',null from dual union all
select 13445,'1966-038','1966-05-11 14:10:00','orbital','success',1510,'R 15000- 32','Kosmos-118','Meteor 11F614 No. 4','','Sat',562,'',null from dual union all
select 13446,'1966-039','1966-05-14 18:30:00','orbital','success',177,'7118','AFP-206','GAMBIT SV 978','PUMP HANDLE','Sat',1773,'',null from dual union all
select 13447,'1966-040','1966-05-15 07:55:34','orbital','success',1393,'456/FTV 6202 (TA6)','Nimbus 2','Nimbus C','BAD ULCER','Sat TAG130',1673,'',null from dual union all
select 13448,'1966-041','1966-05-19 02:27:00','orbital','success',1125,'S146C','NNS O-9  (N-8)','NNS O-9','DANCE LESSON','Sat',1769,'',null from dual union all
select 13449,'1966-042','1966-05-24 02:00:33','orbital','success',1394,'469/FTV 1630','CORONA 109/1033','CORONA J-33','SHORT TON','Sat TAG131',1770,'',null from dual union all
select 1345,'1952-A272','1952-12-17 00:00:00','atmospheric rocket','success',836,'1249A-157','Nike 1249 R&D 155R','','','Test',1895,'',10 from dual union all
select 13450,'1966-043','1966-05-24 05:31:00','orbital','success',678,'','Kosmos-119','DS-U2-I No. 1','','Sat',626,'',null from dual union all
select 13451,'1966-044','1966-05-25 14:00:00','orbital','success',1379,'436/D38','Explorer 32','AE-B','','Sat',234,'',null from dual union all
select 13452,'1966-045','1966-05-30 14:41:01','deep space','success',153,'AC-10','Surveyor 1','Surveyor SC-1','','Sat',268,'',null from dual union all
select 13453,'1966-046','1966-06-01 15:00:02','orbital','success',175,'5304','GATV ATDA','TDA 4','','Sat',229,'',288 from dual union all
select 13454,'1966-047','1966-06-03 13:39:33','orbital','success',1421,'GT-9','Gemini 9','Gemini SC9','','Sat',237,'',null from dual union all
select 13455,'1966-048','1966-06-03 19:25:00','orbital','success',177,'7119','AFP-206','GAMBIT SV 979','POWER DRILL','Sat',1773,'',null from dual union all
select 13456,'1966-049','1966-06-07 02:48:00','orbital','success',176,'5601 (AA16)','OGO B','OGO B','','Sat',227,'',null from dual union all
select 13457,'1966-050','1966-06-08 11:00:00','orbital','success',1508,'N 15001- 13','Kosmos-120','Zenit-2 No. 41','','Sat',562,'',null from dual union all
select 13458,'1966-051','1966-06-09 20:15:00','orbital','success',177,'7201','FTV 1351','Midas RTS-I F1','MAMAS BOY','Sat',1771,'',null from dual union all
select 13459,'1966-052','1966-06-10 04:15:00','orbital','success',1128,'S147C','OV3-4','OV3-4 PHASR','','Sat',1814,'',null from dual union all
select 1346,'1952-A273','1952-12-17 00:00:00','atmospheric rocket','success',836,'1249A-158','Nike 1249 R&D 153R','','','Test',1895,'',10 from dual union all
select 13460,'1966-053','1966-06-16 14:00:01','orbital','success',1425,'3C-11','IDCSP','GGTS','','Sat',275,'',null from dual union all
select 13461,'1966-054','1966-06-17 11:00:00','orbital','success',1508,'U 15001- 08','Kosmos-121','Zenit-4 No. 20','','Sat',473,'',null from dual union all
select 13462,'1966-055','1966-06-21 21:31:30','orbital','success',1394,'466/FTV 1626','CORONA 110/1034','CORONA J-31','GAME LEG','Sat TAG132',1679,'',null from dual union all
select 13463,'1966-056','1966-06-24 00:12:02','orbital','success',1394,'473/FTV 6311 (TA7)','Pageos','Pageos A','CLOTH COAT','Sat TAG133',1673,'',null from dual union all
select 13464,'1966-057','1966-06-25 10:30:00','orbital','success',1510,'R 15000- 21','Kosmos-122','Meteor 11F614 No. 5','','Sat',562,'',null from dual union all
select 13465,'1966-058','1966-07-01 16:02:25','deep space','success',1382,'467/D39','Explorer 33','AIMP D','','Sat',233,'',null from dual union all
select 13466,'1966-059','1966-07-05 14:53:17','orbital','success',1465,'SA-203','AS-203','Subcritical Cryo LN2 Exp','','Sat',271,'',null from dual union all
select 13467,'1966-060','1966-07-06 12:57:00','orbital','success',1458,'212','Proton-3','N-4 No. 4','','Sat',587,'',null from dual union all
select 13468,'1966-061','1966-07-08 05:31:00','orbital','success',681,'','Kosmos-123','DS-P1-Yu No. 5','','Sat',626,'',null from dual union all
select 13469,'1966-062','1966-07-12 17:57:00','orbital','success',177,'7120','AFP-206','GAMBIT SV 980','SNAKE CREEK','Sat',1777,'',null from dual union all
select 1347,'1952-A274','1952-12-17 00:00:00','atmospheric rocket','success',836,'1249A-152','Nike 1249 R&D 154R','','','Test',1895,'',10 from dual union all
select 13470,'1966-063','1966-07-14 02:10:02','orbital','success',154,'58D','OV1-7/8/NC20.189','OV1-8 PasComSat','STONY ISLAND','Sat/Ionos',1685,'',1000 from dual union all
select 13471,'1966-064','1966-07-14 10:25:00','orbital','success',1508,'N 15001- 14','Kosmos-124','Zenit-2 No. 42','','Sat',562,'',null from dual union all
select 13472,'1966-065','1966-07-18 20:39:46','orbital','success',177,'5305','GATV','TDA 1A','','Sat',229,'',null from dual union all
select 13473,'1966-066','1966-07-18 22:20:27','orbital','success',1421,'GT-10','Gemini 10','Gemini SC10','','Sat',237,'',null from dual union all
select 13474,'1966-067','1966-07-20 08:58:00','orbital','success',1222,'G 15000- 02','Kosmos-125','US-A','','Sat',562,'',null from dual union all
select 13475,'1966-068','1966-07-28 10:50:00','orbital','success',1508,'N 15001- 01','Kosmos-126','Zenit-4 No. 21','','Sat',562,'',null from dual union all
select 13476,'1966-069','1966-07-29 18:30:20','orbital','success',1424,'3B-1','KH-8','GAMBIT-3 Mission 4301','DAILY MAIL','Sat',1778,'',null from dual union all
select 13477,'1966-070','1966-08-04 10:45:01','orbital','success',1128,'S148C','OV3-3','OV3-3','RUBBER MAT','Sat',1779,'',null from dual union all
select 13478,'1966-071','1966-08-08 11:29:00','orbital','success',1508,'N 15001- 13','Kosmos-127','Zenit-4 No. 22','','Sat',562,'',null from dual union all
select 13479,'1966-072','1966-08-09 20:46:03','orbital','success',1395,'506/FTV 1631','CORONA 111/1036','CORONA J-32','CURLY TOP','Sat TAG134',1722,'',null from dual union all
select 1348,'1952-A275','1952-12-18 00:00:00','atmospheric rocket','success',365,'','D28 Model','','','Test',1821,'',10 from dual union all
select 13480,'1966-073','1966-08-10 19:26:00','deep space','success',177,'5801 (AA17)','Lunar Orbiter 1','Lunar Orbiter A','','Sat',228,'',null from dual union all
select 13481,'1966-074','1966-08-16 18:30:00','orbital','success',177,'7121','AFP-206','GAMBIT SV 981','SILVER DOLL','Sat',1777,'',null from dual union all
select 13482,'1966-075','1966-08-17 15:20:17','deep space','success',1382,'462/D40','Pioneer 7','Pioneer B','','Sat',233,'',null from dual union all
select 13483,'1966-076','1966-08-18 02:25:02','orbital','success',1125,'S149C','NNS O-10 (N-9)','NNS O-10','MARBLE HALL','Sat',1779,'',null from dual union all
select 13484,'1966-077','1966-08-19 19:30:00','orbital','success',177,'7202','FTV 1352','Midas RTS-I F2','HAPPY MOUNTAIN','Sat',1775,'',null from dual union all
select 13485,'1966-078','1966-08-24 08:03:21','deep space','success',809,'N 15000- 52','Luna-11','E-6LF No. 101','','Sat',562,'',null from dual union all
select 13486,'1966-079','1966-08-27 09:50:00','orbital','success',1508,'N 15001- 03','Kosmos-128','Zenit-4 No. 23','','Sat',480,'',null from dual union all
select 13487,'1966-080','1966-09-12 13:05:02','orbital','success',177,'5306','GATV','TDA 6','','Sat',229,'',null from dual union all
select 13488,'1966-081','1966-09-12 14:42:27','orbital','success',1421,'GT-11','Gemini 11','Gemini SC11','','Sat',237,'',null from dual union all
select 13489,'1966-082','1966-09-16 04:36:09','orbital','success',1364,'167/BII-1','DAPP 1416','DAPP 1416','IRISH DUKE','Sat',1660,'',null from dual union all
select 1349,'1952-A276','1952-12-18 00:00:00','atmospheric rocket','success',365,'','D18 F100 Model','','','Test',1821,'',10 from dual union all
select 13490,'1966-083','1966-09-16 17:59:00','orbital','success',177,'7123','AFP-206','GAMBIT SV 982','TAXI DRIVER','Sat',1777,'',null from dual union all
select 13491,'1966-084','1966-09-20 12:32:00','deep space','success',153,'AC-7','Surveyor 2','Surveyor SC-2','','Sat',268,'',null from dual union all
select 13492,'1966-085','1966-09-20 21:14:05','orbital','success',1394,'477/FTV 1628','CORONA 112/1035','CORONA J-36','BIG BADGE','Sat TAG135',1776,'',null from dual union all
select 13493,'1966-086','1966-09-28 19:07:00','orbital','success',1424,'3B-2','KH-8','GAMBIT-3 Mission 4302','BUSY SCHEME','Sat',1778,'',null from dual union all
select 13494,'1966-087','1966-10-02 10:39:03','orbital','success',1381,'463/D41','ESSA 3','TOS A','BUSY MALLET','Sat',1723,'',null from dual union all
select 13495,'1966-088','1966-09-17 22:35:00','orbital','success',1007,'U 22500-05L','OGCh No. 05L','OGCh No. 05L?','','Sat',519,'',null from dual union all
select 13496,'1966-089','1966-10-05 22:00:00','orbital','success',177,'7203','FTV 1353','Midas RTS-I F3','DWARF KILLER','Sat',1775,'',null from dual union all
select 13497,'1966-090','1966-10-12 19:15:00','orbital','success',177,'7122','AFP-206','GAMBIT SV 983','GLEAMING STAR','Sat',1777,'',null from dual union all
select 13498,'1966-091','1966-10-14 12:13:08','orbital','success',1509,'U 15001- 05','Kosmos-129','Zenit-2 No. 33','','Sat',473,'',null from dual union all
select 13499,'1966-092','1966-10-20 07:50:00','orbital','success',801,'N 15000- 40','Molniya-1','Molniya-1 11F67 No. 7','','Sat',480,'',null from dual union all
select 135,'1944-S122','1944-09-20 00:00:00','military missile','success',1470,'','Cambrai','','','Weapon',1730,'',95 from dual union all
select 1350,'1952-A277','1952-12-19 00:00:00','atmospheric rocket','unknown',576,'78','','','','Test',1895,'',10 from dual union all
select 13500,'1966-093','1966-10-20 08:46:00','orbital','success',1508,'N 15001- 04','Kosmos-130','Zenit-4 No. 24','','Sat',562,'',null from dual union all
select 13501,'1966-094','1966-10-22 08:42:26','deep space','success',809,'N 15000- 53','Luna-12','E-6LF No. 102','','Sat',562,'',null from dual union all
select 13502,'1966-095','1966-10-26 11:12:02','deep space','success',153,'AC-9','SM-3','Surveyor SM-3','','Sat',269,'',null from dual union all
select 13503,'1966-096','1966-10-26 23:05:00','orbital','success',1382,'464/D42','Intelsat II F-1','INTELSAT II F-1','','Sat',234,'',null from dual union all
select 13504,'1966-097','1966-10-28 11:56:02','orbital','success',1128,'S150C','OV3-2','OV3-2','BUSY SERVICE','Sat',1779,'',null from dual union all
select 13505,'1966-098','1966-11-02 20:23:00','orbital','success',177,'7124','AFP-206','GAMBIT SV 984','RED CABOOSE','Sat',1777,'',null from dual union all
select 13506,'1966-099','1966-11-03 13:50:42','orbital','success',1425,'3C-9','Gemini B/MOL','OV4-3','','Sat',274,'',null from dual union all
select 13507,'1966-100','1966-11-06 23:21:00','deep space','success',177,'5802 (AA18)','Lunar Orbiter 2','Lunar Orbiter B','','Sat',228,'',null from dual;
insert into LAUNCH
select 13508,'1966-101','1966-11-02 00:45:03','orbital','success',1007,'N 22500-06L','OGCh','OGCh No. 06L?','','Sat',519,'',null from dual union all
select 13509,'1966-102','1966-11-08 19:53:02','orbital','success',1395,'507/FTV 1632','CORONA 113/1037','CORONA J-38','BUSY MEETING','Sat TAG136',1722,'',null from dual union all
select 1351,'1952-A278','1952-12-19 00:00:00','atmospheric rocket','unknown',576,'77','','','','Test',1895,'',10 from dual union all
select 13510,'1966-103','1966-11-11 19:07:59','orbital','success',177,'5307','GATV','TDA 7A','','Sat',229,'',null from dual union all
select 13511,'1966-104','1966-11-11 20:46:33','orbital','success',1421,'GT-12','Gemini 12','Gemini SC12','','Sat',237,'',null from dual union all
select 13512,'1966-105','1966-11-12 09:50:00','orbital','success',1508,'U 15001- 05','Kosmos-131','Zenit-4 No. 25','','Sat',473,'',null from dual union all
select 13513,'1966-106','1966-11-19 08:00:00','orbital','success',1509,'N 15001- 08','Kosmos-132','Zenit-2 No. 46','','Sat',562,'',null from dual union all
select 13514,'1966-107','1966-11-28 11:00:00','orbital','success',1223,'U 15000- 02','Kosmos-133','Soyuz 7K-OK No. 2','','Sat',562,'',null from dual union all
select 13515,'1966-108','1966-12-03 08:15:00','orbital','success',1508,'N 15001- 06','Kosmos-134','Zenit-4 No. 26','','Sat',562,'',null from dual union all
select 13516,'1966-109','1966-12-05 21:09:00','orbital','success',177,'7125','AFP-206','GAMBIT SV 985','BUSY MERMAID','Sat',1777,'',null from dual union all
select 13517,'1966-110','1966-12-07 02:12:01','orbital','success',177,'5101 (AA19)','ATS 1','ATS B','','Sat',227,'',null from dual union all
select 13518,'1966-111','1966-12-11 21:09:57','orbital','success',154,'89D','OV1-9/10','OV1-9','BUSY PANAMA','Sat',1685,'',null from dual union all
select 13519,'1966-112','1966-12-12 20:38:00','orbital','success',678,'','Kosmos-135','DS-U2-MP No. 1','','Sat',626,'',null from dual union all
select 1352,'1952-A279','1952-12-19 00:00:00','military missile','success',771,'GM-11043','','','','Test',222,'',12 from dual union all
select 13520,'1966-113','1966-12-14 18:14:00','orbital','success',1424,'3B-3','KH-8','GAMBIT-3 Mission 4303','BUSY SKYROCKET','Sat',1778,'',null from dual union all
select 13521,'1966-114','1966-12-14 19:20:03','orbital','success',1383,'471/D43','Biosatellite 1','Biosatellite A Adapter','','Sat',233,'',null from dual union all
select 13522,'1966-115','1966-12-19 12:00:01','orbital','success',1509,'N 15001- 09','Kosmos-136','Zenit-2 No. 47','','Sat',473,'',null from dual union all
select 13523,'1966-116','1966-12-21 10:17:08','deep space','success',809,'N 15000- 55','Luna-13','E-6M No. 205','','Sat',480,'',null from dual union all
select 13524,'1966-117','1966-12-21 13:12:00','orbital','success',681,'','Kosmos-137','DS-U2-D No. 1','','Sat',626,'',null from dual union all
select 13525,'1966-118','1966-12-29 12:00:06','orbital','success',1394,'459/FTV 2731','OPS 1584','MULTIGROUP 1','FRONT ROW','Sat TAG137',1724,'',null from dual union all
select 13526,'1966-A01','1966-01-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 13527,'1966-A02','1966-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 13528,'1966-A03','1966-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 13529,'1966-A04','1966-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 1353,'1952-A28','1952-02-11 00:00:00','atmospheric rocket','unknown',576,'23','','','','Test',1895,'',10 from dual union all
select 13530,'1966-A05','1966-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 13531,'1966-A06','1966-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 13532,'1966-A07','1966-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 13533,'1966-A08','1966-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 13534,'1966-A09','1966-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 13535,'1966-A10','1966-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 13536,'1966-A100','1966-08-25 00:00:00','military missile','failure',255,'A','Target','','AUGUST CORN','Target',1690,'',20 from dual union all
select 13537,'1966-A101','1966-08-30 17:51:37','suborbital spaceplane','success',1520,'2-49-86','V-On,Maurer,TPS','','','Test/Tech/UV Ast',379,'NB-52 008',31 from dual union all
select 13538,'1966-A102','1966-09-08 18:39:16','suborbital spaceplane','failure',1520,'1-68-113','HS/Elec loads','','','Test',379,'NB-52 008',22 from dual union all
select 13539,'1966-A103','1966-09-13 00:00:00','suborbital spaceplane','failure',1520,'3- A-81','','','','Test',379,'NB-52',14 from dual union all
select 1354,'1952-A280','1952-12-19 00:00:00','atmospheric rocket','success',836,'1249A-159','Nike 1249 R&D 157R','','','Test',1895,'',10 from dual union all
select 13540,'1966-A104','1966-09-13 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',42 from dual union all
select 13541,'1966-A105','1966-09-15 00:00:00','suborbital rocket','failure',1115,'Sandia 281-2','','','','Test',1490,'',0 from dual union all
select 13542,'1966-A106','1966-09-16 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',37 from dual union all
select 13543,'1966-A107','1966-09-20 17:05:00','suborbital rocket','success',30,'NASA 12.06GT','Booster test','','','Test',1869,'',3 from dual union all
select 13544,'1966-A108','1966-09-22 05:27:00','military missile','failure',982,'G 22602-03T','8K64U UBP-82','','','OT',566,'',10 from dual union all
select 13545,'1966-A109','1966-09-23 19:51:00','atmospheric rocket','success',835,'Smoke 104','','','','Aeronomy',1821,'',16 from dual union all
select 13546,'1966-A11','1966-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 13547,'1966-A110','1966-09-27 02:32:00','military missile','failure',999,'N 22504-10T','8K67 UBP-3 K','','','OT',514,'',0 from dual union all
select 13548,'1966-A111','1966-09-28 00:00:00','suborbital spaceplane','failure',1520,'1- A-114','','','','Test',379,'NB-52',14 from dual union all
select 13549,'1966-A112','1966-10-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 1355,'1952-A281','1952-12-19 00:00:00','atmospheric rocket','success',836,'1249A-160','Nike 1249 R&D 156R','','','Test',1895,'',10 from dual union all
select 13550,'1966-A113','1966-10-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 13551,'1966-A114','1966-10-04 00:00:00','suborbital spaceplane','failure',1520,'1- A-115','','','','Test',379,'NB-52',14 from dual union all
select 13552,'1966-A115','1966-10-06 20:16:59','suborbital spaceplane','failure',1520,'1-69-116','Checkout','','','Test',379,'NB-52 003',23 from dual union all
select 13553,'1966-A116','1966-10-07 00:00:00','suborbital spaceplane','failure',1520,'2- A-87','','','','Test',379,'NB-52',14 from dual union all
select 13554,'1966-A117','1966-10-07 19:12:00','atmospheric rocket','success',835,'Smoke 105','','','','Aeronomy',1821,'',22 from dual union all
select 13555,'1966-A118','1966-10-10 17:20:00','suborbital rocket','failure',848,'CRL AF07.349','','','','Aeron',394,'',48 from dual union all
select 13556,'1966-A119','1966-10-11 18:17:00','atmospheric rocket','success',835,'Smoke 106','','','','Aeronomy',1821,'',23 from dual union all
select 13557,'1966-A12','1966-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 13558,'1966-A120','1966-10-13 02:25:00','suborbital rocket','unknown',759,'MT-135-019','','','','Aeron',839,'',44 from dual union all
select 13559,'1966-A121','1966-10-14 20:00:00','military missile','unknown',255,'A','Target','','FALL HARVEST','Target',1689,'',20 from dual union all
select 1356,'1952-A282','1952-12-19 00:00:00','military missile','success',763,'N-25 GM-972','','','','Test',222,'',10 from dual union all
select 13560,'1966-A122','1966-10-17 07:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',42 from dual union all
select 13561,'1966-A123','1966-10-19 00:00:00','suborbital spaceplane','failure',1520,'2- A-88','','','','Test',379,'NB-52',14 from dual union all
select 13562,'1966-A124','1966-10-19 11:00:00','suborbital rocket','failure',1188,'INTA S-6602','','','','Aeron/Test',396,'',0 from dual union all
select 13563,'1966-A125','1966-10-28 18:09:00','atmospheric rocket','success',835,'Smoke 107','','','','Aeronomy',1821,'',16 from dual union all
select 13564,'1966-A126','1966-10-30 17:45:00','suborbital rocket','failure',589,'BOX-5','LRL BOX-5/FL-B','','','XR Astron',1277,'T-AGM-8',12 from dual union all
select 13565,'1966-A127','1966-10-31 00:00:00','suborbital rocket','success',557,'Pacemaker','Apollo ablator test','','','Reentry test',1821,'',23 from dual union all
select 13566,'1966-A128','1966-11-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 13567,'1966-A129','1966-11-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 13568,'1966-A13','1966-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 13569,'1966-A130','1966-11-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 1357,'1952-A283','1952-06-26 13:45:00','suborbital rocket','success',365,'','SUI 0 payload test','','','CR',1895,'',14 from dual union all
select 13570,'1966-A131','1966-11-10 00:00:00','atmospheric rocket','success',617,'J204','Dummy St 3','','','Test',1836,'',30 from dual union all
select 13571,'1966-A132','1966-10-31 05:04:00','test rocket','success',815,'M-1-1','','','','Test',835,'',34 from dual union all
select 13572,'1966-A133','1966-11-01 00:00:00','military missile','unknown',255,'A','Target','','BLUE HAWK','Target',1690,'',20 from dual union all
select 13573,'1966-A134','1966-11-03 00:00:00','military missile','failure',1252,'FLA-6','','','','Test',1888,'',0 from dual union all
select 13574,'1966-A135','1966-11-05 00:00:00','suborbital rocket','success',545,'PEPP [F1]','Voyager parachute','','','Test',1895,'',32 from dual union all
select 13575,'1966-A136','1966-11-07 00:00:00','suborbital rocket','failure',894,'','','','','Test',1821,'',1 from dual union all
select 13576,'1966-A137','1966-11-09 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1727,'',30 from dual union all
select 13577,'1966-A138','1966-11-18 00:00:00','suborbital spaceplane','failure',1520,'3- A-84','','','','Test',379,'NB-52',14 from dual union all
select 13578,'1966-A139','1966-11-18 21:24:07','suborbital spaceplane','success',1520,'2-50-89','ET/TPS/Maurer/Mach 6','','','Test/UV Ast',379,'NB-52 008',30 from dual union all
select 13579,'1966-A14','1966-01-11 18:17:00','atmospheric rocket','success',835,'Smoke 099','','','','Aeronomy',1821,'',16 from dual union all
select 1358,'1952-A284','1952-07-22 00:00:00','atmospheric rocket','success',869,'1249-108','Nike 1249 R&D 99R','','','Test',1895,'',10 from dual union all
select 13580,'1966-A140','1966-11-21 00:00:00','suborbital rocket','success',545,'PEPP [F2] ','[Error for Nov 24]','','','Test',1895,'',37 from dual union all
select 13581,'1966-A141','1966-11-21 06:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',47 from dual union all
select 13582,'1966-A142','1966-11-23 00:00:00','suborbital spaceplane','failure',1520,'3- A-85','','','','Test',379,'NB-52',14 from dual union all
select 13583,'1966-A143','1966-11-24 13:45:00','suborbital rocket','success',545,'PEPP 02 [F2]','31.2-ft Ringsail','','','Test',1895,'',39 from dual union all
select 13584,'1966-A144','1966-11-26 19:11:00','suborbital rocket','failure',1206,'ISRC-SK-16','ESRO S11/1 ROE4/BXR1','','','UV Astron/XR Astron',1343,'',3 from dual union all
select 13585,'1966-A145','1966-11-27 09:21:14','test rocket','failure',337,'G1','','','','Test',665,'',12 from dual union all
select 13586,'1966-A146','1966-11-29 19:38:32','suborbital spaceplane','success',1520,'3-57-86','Checkout','','','Test',379,'NB-52 003',28 from dual union all
select 13587,'1966-A147','1966-12-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 13588,'1966-A148','1966-12-08 14:57:00','suborbital rocket','failure',1207,'SL405','LUX-35AC6/Cul-6','','','Solar X,UV',1839,'',0 from dual union all
select 13589,'1966-A149','1966-12-10 01:53:00','military missile','failure',999,'N 22500-03T','8K67 UBP-8','GCh 8F674 LKI','','Test',485,'',0 from dual union all
select 1359,'1952-A285','1952-11-07 17:34:00','atmospheric rocket','success',1325,'','Booster Test 5','','','Test',1895,'',1 from dual union all
select 13590,'1966-A15','1966-01-12 12:21:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',44 from dual union all
select 13591,'1966-A150','1966-12-10 06:28:00','suborbital rocket','success',88,'','Scanner 2 support','','','Weather',394,'',46 from dual union all
select 13592,'1966-A151','1966-12-10 17:12:00','suborbital rocket','success',545,'PEPP [F3]','30-ft DGP Para','','','Test',1895,'',40 from dual union all
select 13593,'1966-A152','1966-12-13 08:37:00','suborbital rocket','failure',226,'AAD-VA-107','Geminids','','','Auroral/Ionos/Meteo',428,'',48 from dual union all
select 13594,'1966-A153','1966-12-17 06:15:00','suborbital rocket','failure',30,'KP3.19','','','','UV Astron',1869,'',1 from dual union all
select 13595,'1966-A154','1966-12-19 06:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',48 from dual union all
select 13596,'1966-A155','1966-12-19 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1507,'SHIP-SOV',45 from dual union all
select 13597,'1966-A156','1966-12-21 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1506,'SHIP-SOV',40 from dual union all
select 13598,'1966-A157','1966-12-21 14:10:00','suborbital rocket','success',725,'','','','','Meteo',636,'',47 from dual;
insert into LAUNCH
select 13599,'1966-A158','1966-12-22 00:00:00','suborbital spaceplane','success',1520,'2- C-90','','','','Test',379,'NB-52',14 from dual union all
select 136,'1944-S123','1944-09-20 00:00:00','military missile','success',1470,'','Liege','','','Weapon',1730,'',95 from dual union all
select 1360,'1952-A286','1952-12-17 00:00:00','atmospheric rocket','success',1526,'','','','','Test',1895,'B-29',10 from dual union all
select 13600,'1966-A159','1966-12-23 12:05:00','military missile','failure',982,'T 22502-04','8K64U UBP-95','','','OT',566,'',0 from dual union all
select 13601,'1966-A16','1966-01-05 00:00:00','military missile','success',540,'7','HIBEX D-4','10','','Test',1895,'',20 from dual union all
select 13602,'1966-A160','1966-12-30 08:30:00','suborbital rocket','success',725,'','','','','Meteo',1549,'SHIP-SOV',42 from dual union all
select 13603,'1966-A161','1966-01-01 00:00:00','atmospheric rocket','success',852,'Sandia','SPTV-9 Para test','','','Test',1490,'',4 from dual union all
select 13604,'1966-A162','1966-10-25 22:32:00','suborbital rocket','success',520,'','HARP 5003','','','Aeron',1913,'',48 from dual union all
select 13605,'1966-A163','1966-10-27 17:15:00','suborbital rocket','success',520,'','HARP 5005','','','Aeron',1913,'',47 from dual union all
select 13606,'1966-A164','1966-11-19 09:37:00','suborbital rocket','success',520,'','HARP 5018','','','Aeron',1913,'',40 from dual union all
select 13607,'1966-A165','1966-11-19 13:11:00','suborbital rocket','success',520,'','HARP 5020','','','Aeron',1913,'',40 from dual union all
select 13608,'1966-A166','1966-03-01 16:30:00','suborbital rocket','success',520,'','28','','','Aeron',184,'',10 from dual union all
select 13609,'1966-A167','1966-03-03 16:00:00','suborbital rocket','success',520,'','29','','','Aeron',184,'',10 from dual union all
select 1361,'1952-A287','1952-06-24 00:00:00','military missile','success',339,'23R','','','','Test',1895,'',9 from dual union all
select 13610,'1966-A168','1966-02-20 00:00:00','suborbital rocket','success',520,'','G03','','','Aeron',616,'',20 from dual union all
select 13611,'1966-A169','1966-02-20 03:00:00','suborbital rocket','success',520,'','G04','','','Aeron',616,'',20 from dual union all
select 13612,'1966-A17','1966-01-19 10:30:00','suborbital rocket','unknown',759,'MT-135-014','','','','Aeron',839,'',49 from dual union all
select 13613,'1966-A170','1966-02-23 00:00:00','suborbital rocket','success',520,'','G06','','','Aeron',616,'',20 from dual union all
select 13614,'1966-A171','1966-02-24 17:38:00','suborbital rocket','success',520,'','G09','','','Aeron',616,'',20 from dual union all
select 13615,'1966-A172','1966-02-24 20:38:00','suborbital rocket','success',520,'','G10','','','Aeron',616,'',43 from dual union all
select 13616,'1966-A173','1966-02-24 23:55:00','suborbital rocket','success',520,'','G11','','','Aeron',616,'',40 from dual union all
select 13617,'1966-A174','1966-02-25 01:33:00','suborbital rocket','success',520,'','G12','','','Aeron',616,'',45 from dual union all
select 13618,'1966-A175','1966-02-25 20:10:00','suborbital rocket','success',520,'','G13','','','Aeron',616,'',42 from dual union all
select 13619,'1966-A176','1966-03-03 00:00:00','suborbital rocket','success',520,'','G14','','','Aeron',616,'',41 from dual union all
select 1362,'1952-A29','1952-02-12 00:00:00','military missile','failure',703,'604','','','','Test',222,'',0 from dual union all
select 13620,'1966-A177','1966-03-03 01:00:00','suborbital rocket','success',520,'','G15','','','Aeron',616,'',42 from dual union all
select 13621,'1966-A178','1966-03-03 02:00:00','suborbital rocket','success',520,'','G16','','','Aeron',616,'',45 from dual union all
select 13622,'1966-A179','1966-03-03 03:00:00','suborbital rocket','success',520,'','G17','','','Aeron',616,'',45 from dual union all
select 13623,'1966-A18','1966-01-20 15:17:01','test rocket','success',713,'LJII-5 (12-51-3)','A-004','','','SC Test',1873,'',23 from dual union all
select 13624,'1966-A180','1966-03-03 23:00:00','suborbital rocket','success',520,'','G20','','','Aeron',616,'',20 from dual union all
select 13625,'1966-A181','1966-03-04 00:00:00','suborbital rocket','success',520,'','G21','','','Aeron',616,'',20 from dual union all
select 13626,'1966-A182','1966-03-04 00:33:00','suborbital rocket','success',520,'','G22','','','Aeron',616,'',33 from dual union all
select 13627,'1966-A183','1966-03-04 01:30:00','suborbital rocket','success',520,'','G23','','','Aeron',616,'',39 from dual union all
select 13628,'1966-A19','1966-01-21 12:22:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',47 from dual union all
select 13629,'1966-A20','1966-01-22 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',45 from dual union all
select 1363,'1952-A30','1952-02-13 00:00:00','military missile','success',771,'GM-555','','','','Test',222,'',12 from dual union all
select 13630,'1966-A21','1966-01-23 12:57:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',44 from dual union all
select 13631,'1966-A22','1966-01-25 00:00:00','military missile','failure',1252,'FLA-2','','','','Test',1888,'',0 from dual union all
select 13632,'1966-A23','1966-01-26 02:10:00','atmospheric rocket','success',1071,'BT-250-1','','','','Test',839,'',6 from dual union all
select 13633,'1966-A24','1966-02-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 13634,'1966-A25','1966-02-01 15:41:00','suborbital rocket','success',631,'','','','','Weather',276,'',40 from dual union all
select 13635,'1966-A27','1966-02-07 16:00:00','suborbital rocket','success',767,'','','','','Aeron',185,'',46 from dual union all
select 13636,'1966-A28','1966-02-08 07:15:00','suborbital rocket','success',88,'','Delta sensor','','','Weather',855,'',48 from dual union all
select 13637,'1966-A29','1966-02-08 16:41:00','suborbital rocket','success',716,'','','','','Weather',276,'',46 from dual union all
select 13638,'1966-A30','1966-02-08 23:07:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',35,'',45 from dual union all
select 13639,'1966-A31','1966-02-08 23:07:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',41 from dual union all
select 1364,'1952-A31','1952-02-19 00:00:00','atmospheric rocket','success',869,'490B','System Test 80R','','','Test',1895,'',10 from dual union all
select 13640,'1966-A32','1966-02-09 18:20:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',41 from dual union all
select 13641,'1966-A33','1966-02-09 21:12:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',428,'',44 from dual union all
select 13642,'1966-A34','1966-02-10 01:20:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',40 from dual union all
select 13643,'1966-A35','1966-02-10 15:00:00','suborbital rocket','success',716,'','','','','Weather',276,'',48 from dual union all
select 13644,'1966-A36','1966-02-11 02:03:15','suborbital rocket','failure',288,'CE-15','ESRO C15/2','','','Auroral/Ionos',29,'',1 from dual union all
select 13645,'1966-A37','1966-02-11 16:00:00','suborbital rocket','success',767,'','','','','Aeron',185,'',46 from dual union all
select 13646,'1966-A38','1966-02-11 18:57:00','suborbital rocket','success',631,'','','','','Weather',276,'',49 from dual union all
select 13647,'1966-A39','1966-02-17 15:00:00','suborbital rocket','success',631,'','','','','Weather',276,'',49 from dual union all
select 13648,'1966-A40','1966-02-20 11:08:00','suborbital rocket','failure',850,'CRL AFS.508','REDLAMP','','','Ionos/Aeron',1483,'',10 from dual union all
select 13649,'1966-A41','1966-02-21 18:54:00','atmospheric rocket','success',835,'Smoke 100','','','','Aeronomy',1821,'',22 from dual union all
select 1365,'1952-A32','1952-02-19 14:49:00','suborbital rocket','failure',53,'USAF 22','Airglow','','','Aeronomy SB',640,'',0 from dual union all
select 13650,'1966-A42','1966-02-23 15:57:00','suborbital rocket','success',631,'','','','','Weather',276,'',49 from dual union all
select 13651,'1966-A44','1966-02-28 04:58:00','suborbital rocket','failure',837,'NASA 14.252UM','UM Pitot 21/AS-201','','','Aeronomy',176,'',20 from dual union all
select 13652,'1966-A45','1966-03-11 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',45 from dual union all
select 13653,'1966-A46','1966-03-16 20:35:00','atmospheric rocket','success',835,'Smoke 101','','','','Aeronomy',1821,'',16 from dual union all
select 13654,'1966-A47','1966-03-17 18:02:00','atmospheric rocket','success',835,'Smoke 102','','','','Aeronomy',1821,'',13 from dual union all
select 13655,'1966-A48','1966-03-28 00:00:00','suborbital rocket','success',13,'','','','','Test',1895,'F-4C',46 from dual union all
select 13656,'1966-A49','1966-04-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 13657,'1966-A50','1966-04-04 08:07:00','suborbital rocket','failure',1492,'VA78','FU-145 ACS test','','','Test/[Astron?]',668,'',23 from dual union all
select 13658,'1966-A51','1966-04-05 00:00:00','suborbital rocket','failure',1431,'Sandia 152-41','Despin','','','Test',1490,'',0 from dual union all
select 13659,'1966-A52','1966-04-13 00:00:00','suborbital spaceplane','failure',1520,'2- A-76','','','','Test',379,'NB-52',14 from dual union all
select 1366,'1952-A33','1952-02-20 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 13660,'1966-A53','1966-04-20 00:00:00','suborbital spaceplane','failure',1520,'2- A-77','','','','Test',379,'NB-52',14 from dual union all
select 13661,'1966-A54','1966-04-20 12:13:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',30 from dual union all
select 13662,'1966-A55','1966-04-21 00:00:00','military missile','failure',1449,'N 22944-306','8K84 UBP-4 SLI','','','OT',505,'',0 from dual union all
select 13663,'1966-A56','1966-04-25 12:07:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',28 from dual union all
select 13664,'1966-A567','1955-06-16 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13665,'1966-A568','1955-06-17 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13666,'1966-A569','1955-06-20 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13667,'1966-A57','1966-04-28 02:00:00','suborbital rocket','success',200,'BT-310-1','','','','Test',839,'',1 from dual union all
select 13668,'1966-A570','1955-06-20 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13669,'1966-A571','1955-06-21 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 1367,'1952-A34','1952-02-25 00:00:00','atmospheric rocket','success',836,'1249-100','Nike 1249 R&D 81R','','','Test',1895,'',10 from dual union all
select 13670,'1966-A572','1955-06-21 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13671,'1966-A573','1955-06-28 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13672,'1966-A574','1955-06-30 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13673,'1966-A575','1955-06-30 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13674,'1966-A576','1955-07-05 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13675,'1966-A577','1955-07-05 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13676,'1966-A578','1955-07-13 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13677,'1966-A579','1955-07-13 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13678,'1966-A58','1966-04-29 00:00:00','suborbital rocket','success',200,'BT-310-2','','','','Test',839,'',1 from dual union all
select 13679,'1966-A580','1955-07-18 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 1368,'1952-A35','1952-02-27 00:00:00','atmospheric rocket','unknown',576,'24','','','','Test',1895,'',10 from dual union all
select 13680,'1966-A581','1955-07-18 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13681,'1966-A582','1955-07-26 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13682,'1966-A583','1955-07-28 00:00:00','atmospheric rocket','unknown',576,'','','','','Test',1895,'',10 from dual union all
select 13683,'1966-A584','1966-01-01 00:00:00','atmospheric rocket','success',536,'5','Test','','','Test',1849,'',45 from dual union all
select 13684,'1966-A585','1966-01-01 00:00:00','atmospheric rocket','success',536,'6','Test','','','Test',1849,'',45 from dual union all
select 13685,'1966-A586','1966-05-12 00:00:00','suborbital rocket','success',286,'Cedar 7','','','','Test',356,'',60 from dual union all
select 13686,'1966-A587','1966-01-01 00:00:00','atmospheric rocket','success',852,'Sandia','SPTV-8 Para test','','','Test',1490,'',4 from dual union all
select 13687,'1966-A588','1966-03-11 21:02:00','military missile','failure',837,'','STV SR-025','','','Target',1895,'',2 from dual union all
select 13688,'1966-A589','1966-11-01 00:00:00','suborbital rocket','success',275,'','','','','Test',310,'',40 from dual;
insert into LAUNCH
select 13689,'1966-A59','1966-05-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 1369,'1952-A36','1952-02-27 00:00:00','atmospheric rocket','unknown',576,'25','','','','Test',1895,'',10 from dual union all
select 13690,'1966-A590','1966-01-01 00:00:00','military missile','failure',1037,'','8K98 LKI-1 5','','','Test',632,'',0 from dual union all
select 13691,'1966-A591','1966-09-28 00:00:00','military missile','success',702,'XRL Feasibility 1','','','','Test',1895,'',10 from dual union all
select 13692,'1966-A592','1966-09-28 00:00:00','military missile','success',702,'XRL Feasibility 2','','','','Test',1895,'',10 from dual union all
select 13693,'1966-A593','1966-10-03 00:00:00','military missile','success',702,'EM Test 6','','','','Test',1895,'',10 from dual union all
select 13694,'1966-A60','1966-05-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 13695,'1966-A61','1966-05-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 13696,'1966-A62','1966-05-03 00:00:00','military missile','failure',1252,'FLA-4','','','','Test',1888,'',0 from dual union all
select 13697,'1966-A63','1966-05-04 00:00:00','military missile','failure',837,'BMTS-22','Target','','','Test',437,'',0 from dual union all
select 13698,'1966-A64','1966-05-05 00:00:00','suborbital spaceplane','failure',1520,'2- A-78','','','','Test',379,'NB-52',14 from dual union all
select 13699,'1966-A65','1966-05-06 21:30:12','suborbital spaceplane','failure',1520,'1-63-104','AD/MuMet/Scan/SB','','','Aeron/Meteor/Tech',379,'NB-52 003',20 from dual union all
select 137,'1944-S124','1944-09-20 00:00:00','military missile','success',1470,'','Maastricht','','','Weapon',1730,'',95 from dual union all
select 1370,'1952-A37','1952-02-28 00:00:00','military missile','failure',703,'596','','','','Test',222,'',0 from dual union all
select 13700,'1966-A66','1966-05-15 10:43:00','suborbital rocket','failure',257,'NASA 15.25GI','Zorba I (Test)','','','Iono-solar Eclipse',874,'RREC',20 from dual union all
select 13701,'1966-A67','1966-05-18 18:24:00','suborbital spaceplane','success',1520,'2-44-79','V-On Stab','','','Test',379,'NB-52 003',30 from dual union all
select 13702,'1966-A68','1966-05-20 10:10:00','suborbital rocket','failure',1250,'A005','ESRO A23/4','','','Iono-solar Eclipse',831,'',7 from dual union all
select 13703,'1966-A69','1966-05-21 04:40:00','military missile','failure',982,'G 22601-12T','8K64U UBP-74 PBG','','','OT',565,'',0 from dual union all
select 13704,'1966-A70','1966-05-24 12:08:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',46 from dual union all
select 13705,'1966-A71','1966-05-26 00:00:00','suborbital rocket','failure',1249,'','','','','Test',1057,'F-4B',30 from dual union all
select 13706,'1966-A72','1966-06-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 13707,'1966-A73','1966-06-02 00:00:00','suborbital spaceplane','failure',1520,'1- A-105','','','','Test',379,'NB-52',14 from dual union all
select 13708,'1966-A74','1966-06-09 00:00:00','military missile','failure',135,'D021','ECM Decoy F-4','ABRES F-4 (USAF)','','Reentry Test',614,'',0 from dual union all
select 13709,'1966-A75','1966-06-10 00:00:00','suborbital spaceplane','failure',1520,'1- A-106','','','','Test',379,'NB-52',14 from dual union all
select 1371,'1952-A38','1952-02-29 00:00:00','atmospheric rocket','success',869,'490B','System Test 82R','','','Test',1895,'',10 from dual union all
select 13710,'1966-A76','1966-06-11 00:00:00','military missile','failure',1449,'','8K84 UBP-10 SLI','[Not in RPCA list]','','OT',596,'',0 from dual union all
select 13711,'1966-A77','1966-06-15 12:01:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',48 from dual union all
select 13712,'1966-A78','1966-06-18 10:28:00','suborbital rocket','unknown',716,'NA-JOHANNA','','','','Aeron',394,'',38 from dual union all
select 13713,'1966-A79','1966-06-20 00:00:00','suborbital spaceplane','failure',1520,'3- A-76','','','','Test',379,'NB-52',14 from dual union all
select 13714,'1966-A80','1966-06-27 00:00:00','suborbital spaceplane','success',1520,'2- C-80','External Tank Test','','','Test',379,'NB-52',14 from dual union all
select 13715,'1966-A81','1966-07-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 13716,'1966-A82','1966-07-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 13717,'1966-A83','1966-07-01 19:02:36','suborbital spaceplane','failure',1520,'2-45-81','External Tank Test','','','Test',379,'NB-52 008',13 from dual union all
select 13718,'1966-A84','1966-07-12 19:32:15','suborbital spaceplane','success',1520,'1-64-107','Checkout, Systems','','','Test',379,'NB-52 003',39 from dual union all
select 13719,'1966-A85','1966-07-13 00:00:00','suborbital spaceplane','failure',1520,'3- A-77','','','','Test',379,'NB-52',14 from dual union all
select 1372,'1952-A39','1952-03-04 00:00:00','atmospheric rocket','success',869,'490B','System Test 83R','','','Test',1895,'',10 from dual union all
select 13720,'1966-A86','1966-07-13 06:20:00','suborbital rocket','success',631,'TERLS-45','ISRO 11.27','TERLS-045','','Aeron',1467,'',45 from dual union all
select 13721,'1966-A87','1966-07-18 19:38:24','suborbital spaceplane','success',1520,'3-52-78','Checkout, Systems','','','Test',379,'NB-52 003',29 from dual union all
select 13722,'1966-A88','1966-07-20 00:00:00','suborbital spaceplane','failure',1520,'2- A-82','','','','Test',379,'NB-52',14 from dual union all
select 13723,'1966-A89','1966-07-22 19:19:00','atmospheric rocket','success',835,'Smoke 103','','','','Aeronomy',1821,'',17 from dual union all
select 13724,'1966-A90','1966-08-02 00:00:00','military missile','failure',1252,'FLA-5','','','','Test',1888,'',0 from dual union all
select 13725,'1966-A91','1966-08-04 17:54:43','suborbital spaceplane','success',1520,'3-53-79','Heat Tran,BLN,OBE','','','Test/Tech',379,'NB-52 008',40 from dual union all
select 13726,'1966-A92','1966-08-06 02:00:00','suborbital rocket','success',200,'MM-310-1','Mu Scale Model','','','Test',839,'',2 from dual union all
select 13727,'1966-A93','1966-08-06 06:30:00','suborbital rocket','unknown',1070,'PT-210-1','','','','Test',839,'',1 from dual union all
select 13728,'1966-A94','1966-08-09 00:00:00','suborbital spaceplane','failure',1520,'1- A-109','','','','Test',379,'NB-52',14 from dual union all
select 13729,'1966-A96','1966-08-10 00:00:00','suborbital rocket','failure',1115,'Sandia 281-1','','','','Test',1490,'',0 from dual union all
select 1373,'1952-A40','1952-03-05 00:00:00','atmospheric rocket','success',365,'','XF3H-1 Model','','','Test',1821,'',10 from dual union all
select 13730,'1966-A97','1966-08-10 00:00:00','suborbital spaceplane','failure',1520,'1- A-110','','','','Test',379,'NB-52',14 from dual union all
select 13731,'1966-A98','1966-08-11 05:30:00','suborbital rocket','unknown',759,'IT-135-1','','','','Test',839,'',45 from dual union all
select 13732,'1966-A99','1966-08-15 00:00:00','military missile','failure',1449,'','8K84 UBP-21','','','OT',508,'',0 from dual union all
select 13733,'1966-E01','1966-03-15 22:00:00','orbital','destroyed before launch',1007,'U 22502-03L','OGCh No. 03L','OGCh No. 03L','','Sat',573,'',0 from dual union all
select 13734,'1966-F01','1966-01-07 04:48:23','orbital','failure',1363,'251','DAPP/OPS 2394','DAPP 14','PERSIAN LAMB','Sat',1660,'',850 from dual union all
select 13735,'1966-F02','1966-02-21 13:30:00','orbital','failure',681,'','','DS-K-40 No. 2','','Sat',626,'',20 from dual union all
select 13736,'1966-F03','1966-03-24 14:39:00','orbital','failure',1458,'211','','N-4 No. 3','','Sat',587,'',180 from dual union all
select 13737,'1966-F04','1966-03-27 07:20:00','orbital','failure',801,'U 15000- 40','','Molniya-1 11F67 No 5','','Sat',562,'',200 from dual union all
select 13738,'1966-F05','1966-05-03 19:25:25','orbital','failure',1394,'465/FTV 1625','CORONA 108/1032','CORONA J-28','CARGO NET','Sat TAG129',1679,'',180 from dual union all
select 13739,'1966-F06','1966-05-17 11:00:00','orbital','failure',1508,'U 15001- 06','','Zenit-4 No. 18','','Sat',473,'',10 from dual union all
select 1374,'1952-A41','1952-03-05 00:00:00','atmospheric rocket','success',836,'1249-101','Nike 1249 R&D 84R','','','Test',1895,'',10 from dual union all
select 13740,'1966-F07','1966-05-17 15:15:03','orbital','failure',177,'5303','GATV 5004','TDA 5','','Sat',229,'',116 from dual union all
select 13741,'1966-F08','1966-08-26 13:59:56','orbital','failure',1425,'3C-12','IDCSP','IDCSP','','Sat',275,'',25 from dual union all
select 13742,'1966-F09','1966-09-16 09:30:00','orbital','failure',1509,'N 15001- 02','','Zenit-2 No. 40','','Sat',562,'',1 from dual union all
select 13743,'1966-F10','1966-09-26 02:58:00','orbital','failure',699,'L-4S-1','','L-4S-1','','Sat',834,'',400 from dual union all
select 13744,'1966-F11','1966-11-16 13:00:00','orbital','failure',679,'4000-000-10LP','','Strela-2 No. 1','','Sat',564,'',500 from dual union all
select 13745,'1966-F12','1966-12-14 11:00:00','orbital','failure',1223,'U 15000- 01','','Soyuz No. 1','','Sat',562,'',0 from dual union all
select 13746,'1966-F13','1966-12-20 02:20:00','orbital','failure',699,'L-4S-2','','L-4S-2','','Sat',834,'',400 from dual union all
select 13747,'1966-M01','1966-01-01 00:00:00','military missile','unknown',753,'','Caisson 2','','','Test',312,'',50 from dual union all
select 13748,'1966-M02','1966-01-01 00:00:00','suborbital rocket','success',330,'','','','','Atmos',1057,'RB-57B',52 from dual union all
select 13749,'1966-M03','1966-01-19 04:39:00','suborbital rocket','failure',30,'KP3.11','Stellar Spectra','','','UV Astron',1869,'',50 from dual union all
select 1375,'1952-A42','1952-03-10 00:00:00','military missile','success',771,'GM-556','','','','Test',222,'',12 from dual union all
select 13750,'1966-M04','1966-01-28 06:43:00','suborbital rocket','failure',130,'CRL AC15.733','Electron flux','','','Ionos/Auroral',428,'',61 from dual union all
select 13751,'1966-M05','1966-02-02 17:35:00','suborbital rocket','success',767,'','','','','Aeron',185,'',53 from dual union all
select 13752,'1966-M06','1966-02-08 19:16:00','suborbital rocket','success',520,'','','','','Aeron',1790,'',60 from dual union all
select 13753,'1966-M07','1966-02-08 20:58:00','suborbital rocket','success',520,'','','','','Aeron',1790,'',60 from dual union all
select 13754,'1966-M08','1966-02-15 00:00:00','military missile','success',1104,'2603','Silo 1','','','Test',301,'',50 from dual union all
select 13755,'1966-M09','1966-02-16 17:00:00','suborbital rocket','success',767,'','','','','Aeron',185,'',60 from dual union all
select 13756,'1966-M10','1966-02-20 21:53:00','suborbital rocket','success',767,'','','','','Aeron',185,'',60 from dual union all
select 13757,'1966-M100','1966-04-20 00:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 13758,'1966-M101','1966-04-22 00:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 13759,'1966-M102','1966-05-23 00:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 1376,'1952-A43','1952-03-12 00:00:00','military missile','unknown',339,'17-14R','','','','Test',1895,'',30 from dual union all
select 13760,'1966-M103','1966-05-25 00:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 13761,'1966-M104','1966-07-07 00:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 13762,'1966-M105','1966-07-08 15:09:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 13763,'1966-M106','1966-07-27 00:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 13764,'1966-M107','1966-07-29 00:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 13765,'1966-M108','1966-08-15 15:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 13766,'1966-M109','1966-08-16 15:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 13767,'1966-M11','1966-02-23 22:38:00','suborbital rocket','success',767,'','','','','Aeron',185,'',60 from dual union all
select 13768,'1966-M110','1966-08-18 15:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 13769,'1966-M111','1966-09-14 18:30:00','suborbital rocket','success',88,'Ballute','DMQ-9/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 1377,'1952-A44','1952-03-13 00:00:00','atmospheric rocket','success',365,'','XF4D-1 Model','','','Test',1821,'',10 from dual union all
select 13770,'1966-M112','1966-09-15 16:00:00','suborbital rocket','success',88,'','DMQ-9/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 13771,'1966-M113','1966-09-19 15:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 13772,'1966-M114','1966-09-20 15:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 13773,'1966-M115','1966-04-20 00:00:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 13774,'1966-M116','1966-04-22 16:06:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 13775,'1966-M117','1966-05-25 15:00:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 13776,'1966-M118','1966-05-26 18:38:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 13777,'1966-M119','1966-05-27 14:45:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 13778,'1966-M12','1966-02-24 03:23:00','suborbital rocket','success',767,'','','','','Aeron',185,'',60 from dual;
insert into LAUNCH
select 13779,'1966-M120','1966-07-06 16:15:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 1378,'1952-A45','1952-03-14 00:00:00','atmospheric rocket','success',836,'1249-98','Nike 1249 R&D 85R','','','Test',1895,'',10 from dual union all
select 13780,'1966-M121','1966-07-08 16:30:00','suborbital rocket','success',716,'','Chute','','','Test',293,'',60 from dual union all
select 13781,'1966-M122','1966-09-21 00:00:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 13782,'1966-M123','1966-09-21 00:00:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 13783,'1966-M124','1966-09-22 00:00:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 13784,'1966-M125','1966-11-14 19:00:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 13785,'1966-M126','1966-11-14 20:07:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 13786,'1966-M127','1966-11-15 15:20:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 13787,'1966-M128','1966-11-15 00:00:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 13788,'1966-M129','1966-08-16 07:30:00','suborbital rocket','success',88,'','','','','Meteo',428,'',58 from dual union all
select 13789,'1966-M13','1966-02-24 06:16:00','suborbital rocket','success',767,'','','','','Aeron',185,'',60 from dual union all
select 1379,'1952-A46','1952-03-17 00:00:00','military missile','unknown',339,'18-15','','','','Test',1895,'',30 from dual union all
select 13790,'1966-M130','1966-08-16 07:50:00','suborbital rocket','success',88,'','','','','Meteo',394,'',55 from dual union all
select 13791,'1966-M131','1966-08-16 07:30:00','suborbital rocket','success',88,'','','','','Meteo',1895,'',61 from dual union all
select 13792,'1966-M132','1966-10-21 11:00:00','suborbital rocket','unknown',631,'INTA H-6602','','','','Met',396,'',70 from dual union all
select 13793,'1966-M133','1966-10-26 11:40:00','suborbital rocket','unknown',631,'INTA H-6603','','','','Met',396,'',70 from dual union all
select 13794,'1966-M134','1966-11-02 10:54:00','suborbital rocket','unknown',631,'INTA H-6604','','','','Met',396,'',70 from dual union all
select 13795,'1966-M135','1966-11-09 11:39:00','suborbital rocket','unknown',631,'INTA H-6605','','','','Met',396,'',70 from dual union all
select 13796,'1966-M136','1966-11-18 11:09:00','suborbital rocket','unknown',631,'INTA H-6606','','','','Met',396,'',70 from dual union all
select 13797,'1966-M137','1966-11-23 11:05:00','suborbital rocket','unknown',631,'INTA H-6607','','','','Met',396,'',70 from dual union all
select 13798,'1966-M138','1966-12-02 11:06:00','suborbital rocket','unknown',631,'INTA H-6608','','','','Met',396,'',70 from dual union all
select 13799,'1966-M139','1966-12-07 11:00:00','suborbital rocket','unknown',631,'INTA H-6609','','','','Met',396,'',70 from dual union all
select 138,'1944-S125','1944-09-20 00:00:00','military missile','success',1470,'','Lille','','','Weapon',1730,'',95 from dual union all
select 1380,'1952-A47','1952-03-18 00:00:00','atmospheric rocket','success',365,'','F29 Ramjet','','','Test',1821,'',10 from dual union all
select 13800,'1966-M14','1966-02-24 16:00:00','suborbital rocket','success',767,'','','','','Aeron',185,'',60 from dual union all
select 13801,'1966-M140','1966-12-09 11:35:00','suborbital rocket','unknown',631,'INTA H-6610','','','','Met',396,'',70 from dual union all
select 13802,'1966-M141','1966-12-12 00:00:00','suborbital rocket','success',88,'','PSU-Arcas','','','Ionos',1895,'',70 from dual union all
select 13803,'1966-M142','1966-08-29 00:00:00','suborbital rocket','success',88,'','PSU-Arcas','','','Ionos',1895,'',70 from dual union all
select 13804,'1966-M143','1966-02-11 00:00:00','suborbital rocket','failure',273,'','','','','Aeron',293,'',75 from dual union all
select 13805,'1966-M144','1966-07-05 16:45:00','suborbital rocket','success',273,'','','','','Aeron',293,'',75 from dual union all
select 13806,'1966-M145','1966-08-14 00:00:00','military missile','success',1162,'','','RB650 Training','','OT',1447,'',50 from dual union all
select 13807,'1966-M15','1966-02-25 16:00:00','suborbital rocket','success',767,'','','','','Aeron',185,'',60 from dual union all
select 13808,'1966-M16','1966-04-01 13:57:00','suborbital rocket','success',1181,'','AC30.752','','','Aeron',428,'',103 from dual union all
select 13809,'1966-M17','1966-04-25 04:15:00','suborbital rocket','unknown',1069,'IT-160-1','Instrument test','','','Test',839,'',68 from dual union all
select 1381,'1952-A48','1952-03-19 00:00:00','atmospheric rocket','success',442,'','Navaho Model','','','Test',1821,'',10 from dual union all
select 13810,'1966-M18','1966-05-11 23:20:00','suborbital rocket','failure',549,'Sandia 152-81','LRL ROX-1','','','XR Astron',855,'',80 from dual union all
select 13811,'1966-M19','1966-05-26 00:00:00','military missile','success',753,'','Essai 1','','','Test',669,'',50 from dual union all
select 13812,'1966-M20','1966-06-01 00:00:00','military missile','success',753,'','Essai 2','','','Test',669,'',50 from dual union all
select 13813,'1966-M21','1966-06-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 13814,'1966-M22','1966-06-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 13815,'1966-M23','1966-06-07 00:00:00','military missile','unknown',1104,'2604','Silo 2','','','Test',301,'',50 from dual union all
select 13816,'1966-M24','1966-06-16 00:14:36','suborbital rocket','failure',446,'D-19','FU-151 Ions/part.','','','Ionos',29,'',60 from dual union all
select 13817,'1966-M25','1966-07-12 00:00:00','military missile','unknown',753,'','Caisson 1','','','Test',312,'',50 from dual union all
select 13818,'1966-M26','1966-07-20 17:01:07','suborbital rocket','success',203,'ESRO B12/1','ESRO B12/1  Neutrons','','','Ionospheric',1343,'',65 from dual union all
select 13819,'1966-M27','1966-07-21 20:02:03','suborbital spaceplane','success',1520,'2-46-83','Alt,ST,Abl TPS','','','Test',379,'NB-52 003',58 from dual union all
select 1382,'1952-A49','1952-03-20 00:00:00','military missile','success',771,'GM-558','','','','Test',222,'',12 from dual union all
select 13820,'1966-M28','1966-07-25 18:10:00','suborbital rocket','success',203,'ESRO B12/2','ESRO B12/2  Neutrons','','','Ionospheric',1343,'',67 from dual union all
select 13821,'1966-M29','1966-07-28 18:01:12','suborbital spaceplane','success',1520,'1-65-108','MuMet/HS/SB/RAS','','','Tech/Meteor/Aeron',379,'NB-52 008',73 from dual union all
select 13822,'1966-M30','1966-08-03 16:45:26','suborbital spaceplane','success',1520,'2-47-84','ST, Base drag','','','Test',379,'NB-52 008',75 from dual union all
select 13823,'1966-M31','1966-08-11 17:44:13','suborbital spaceplane','success',1520,'1-66-111','HS,MuMet,SB,Gas','','','Tech/Meteor/Aeron',379,'NB-52 003',76 from dual union all
select 13824,'1966-M32','1966-08-12 18:25:33','suborbital spaceplane','success',1520,'2-48-85','ST,Pitot,Base drag','','','Test',379,'NB-52 003',70 from dual union all
select 13825,'1966-M33','1966-08-19 18:04:35','suborbital spaceplane','success',1520,'3-54-80','BLN,OBE,H','','','Test/Aeron?',379,'NB-52 003',54 from dual union all
select 13826,'1966-M34','1966-08-25 17:49:11','suborbital spaceplane','success',1520,'1-67-112','HS/MuMet/Gas','','','Tech/Meteor/Aeron',379,'NB-52 003',78 from dual union all
select 13827,'1966-M35','1966-09-14 20:01:29','suborbital spaceplane','success',1520,'3-55-82','MuMet/Solar spec','','','Tech/Meteor/Solar',379,'NB-52 003',77 from dual union all
select 13828,'1966-M36','1966-09-28 18:12:00','suborbital rocket','success',1181,'','AC30.753','','','Ionos',428,'',103 from dual union all
select 13829,'1966-M37','1966-10-11 19:59:41','military missile','failure',162,'115F','Simulated BGRV 1','ABRES BGRV-1','LOW HILL','Reentry Test',1681,'',40 from dual union all
select 1383,'1952-A50','1952-03-21 00:00:00','military missile','unknown',339,'19-16','','','','Test',1895,'',30 from dual union all
select 13830,'1966-M38','1966-10-30 14:06:00','suborbital rocket','success',839,'CRL AC6.710','Optics test','','','Tech',394,'',80 from dual union all
select 13831,'1966-M39','1966-11-05 15:35:00','suborbital rocket','success',850,'Cert. Round','','','','Ionos',221,'',80 from dual union all
select 13832,'1966-M40','1966-11-10 00:00:00','military missile','unknown',1104,'2605','Silo 3','','','Test',301,'',50 from dual union all
select 13833,'1966-M41','1966-11-12 13:38:00','suborbital rocket','success',850,'D-003C','','','','Ionos',221,'',80 from dual union all
select 13834,'1966-M42','1966-11-12 14:08:00','suborbital rocket','success',850,'D-010C','','','','Ionos',221,'',80 from dual union all
select 13835,'1966-M43','1966-11-12 13:50:00','suborbital rocket','success',850,'D-015C','','','','Ionos',221,'',80 from dual union all
select 13836,'1966-M44','1966-11-12 15:35:00','suborbital rocket','success',850,'D-016C','','','','Ionos',221,'',80 from dual union all
select 13837,'1966-M45','1966-12-02 00:00:00','military missile','failure',1117,'R5','','','','Reentry Test',669,'',50 from dual union all
select 13838,'1966-M46','1966-12-18 09:15:00','test rocket','success',337,'G2','','','','Test',665,'',55 from dual union all
select 13839,'1966-M47','1966-10-14 23:03:00','suborbital rocket','success',520,'','HARP 5001','','','Aeron',1913,'',62 from dual union all
select 1384,'1952-A51','1952-03-21 00:00:00','military missile','failure',703,'594','','','','Test',222,'',0 from dual union all
select 13840,'1966-M48','1966-10-25 21:12:00','suborbital rocket','success',520,'','HARP 5002','','','Aeron',1913,'',65 from dual union all
select 13841,'1966-M49','1966-10-26 05:01:00','suborbital rocket','success',520,'','HARP 5004','','','Aeron',1913,'',66 from dual union all
select 13842,'1966-M50','1966-11-16 22:10:00','suborbital rocket','success',520,'','HARP 5006','','','Aeron',1913,'',68 from dual union all
select 13843,'1966-M51','1966-11-17 01:44:00','suborbital rocket','success',520,'','HARP 5007','','','Aeron',1913,'',64 from dual union all
select 13844,'1966-M52','1966-11-17 03:44:00','suborbital rocket','success',520,'','HARP 5008','','','Aeron',1913,'',71 from dual union all
select 13845,'1966-M53','1966-11-17 05:44:00','suborbital rocket','success',520,'','HARP 5009','','','Aeron',1913,'',60 from dual union all
select 13846,'1966-M54','1966-11-17 10:19:00','suborbital rocket','success',520,'','HARP 5010','','','Aeron',1913,'',67 from dual union all
select 13847,'1966-M55','1966-11-17 11:10:00','suborbital rocket','success',520,'','HARP 5011','','','Aeron',1913,'',66 from dual union all
select 13848,'1966-M56','1966-11-18 20:06:00','suborbital rocket','success',520,'','HARP 5012','','','Aeron',1913,'',68 from dual union all
select 13849,'1966-M57','1966-11-19 01:23:00','suborbital rocket','success',520,'','HARP 5013','','','Aeron',1913,'',52 from dual union all
select 1385,'1952-A52','1952-03-28 00:00:00','atmospheric rocket','success',869,'490B','System Test 86R','','','Test',1895,'',10 from dual union all
select 13850,'1966-M58','1966-11-19 03:15:00','suborbital rocket','success',520,'','HARP 5014','','','Aeron',1913,'',67 from dual union all
select 13851,'1966-M59','1966-11-19 04:52:00','suborbital rocket','success',520,'','HARP 5015','','','Aeron',1913,'',68 from dual union all
select 13852,'1966-M60','1966-11-19 06:45:00','suborbital rocket','success',520,'','HARP 5016','','','Aeron',1913,'',63 from dual union all
select 13853,'1966-M61','1966-11-19 08:03:00','suborbital rocket','success',520,'','HARP 5017','','','Aeron',1913,'',71 from dual union all
select 13854,'1966-M62','1966-11-19 11:54:00','suborbital rocket','success',520,'','HARP 5019','','','Aeron',1913,'',60 from dual union all
select 13855,'1966-M63','1966-11-20 02:47:00','suborbital rocket','success',520,'','HARP 5021','','','Aeron',1913,'',69 from dual union all
select 13856,'1966-M64','1966-01-17 20:21:00','suborbital rocket','success',520,'','1','','','Aeron',184,'',67 from dual union all
select 13857,'1966-M65','1966-01-18 14:11:00','suborbital rocket','success',520,'','2','','','Aeron',184,'',69 from dual union all
select 13858,'1966-M66','1966-01-18 19:21:00','suborbital rocket','success',520,'','3','','','Aeron',184,'',68 from dual union all
select 13859,'1966-M67','1966-01-18 20:07:00','suborbital rocket','success',520,'','4','','','Aeron',184,'',67 from dual union all
select 1386,'1952-A53','1952-03-28 00:00:00','atmospheric rocket','success',869,'490B','System Test 87R','','','Test',1895,'',10 from dual union all
select 13860,'1966-M68','1966-01-19 15:00:00','suborbital rocket','success',520,'','5','','','Aeron',184,'',67 from dual union all
select 13861,'1966-M69','1966-01-19 18:45:00','suborbital rocket','success',520,'','6','','','Aeron',184,'',65 from dual union all
select 13862,'1966-M70','1966-01-19 19:57:00','suborbital rocket','success',520,'','7','','','Aeron',184,'',65 from dual union all
select 13863,'1966-M71','1966-01-20 16:07:00','suborbital rocket','success',520,'','8','','','Aeron',184,'',61 from dual union all
select 13864,'1966-M72','1966-01-20 19:25:00','suborbital rocket','success',520,'','9','','','Aeron',184,'',72 from dual union all
select 13865,'1966-M73','1966-01-21 16:02:00','suborbital rocket','success',520,'','10','','','Aeron',184,'',66 from dual union all
select 13866,'1966-M74','1966-01-21 19:50:00','suborbital rocket','success',520,'','11','','','Aeron',184,'',64 from dual union all
select 13867,'1966-M75','1966-01-24 16:00:00','suborbital rocket','success',520,'','12','','','Aeron',184,'',61 from dual union all
select 13868,'1966-M76','1966-01-26 16:10:00','suborbital rocket','success',520,'','13','','','Aeron',184,'',64 from dual;
insert into LAUNCH
select 13869,'1966-M77','1966-01-28 16:00:00','suborbital rocket','success',520,'','14','','','Aeron',184,'',64 from dual union all
select 1387,'1952-A54','1952-03-28 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 13870,'1966-M78','1966-01-31 16:40:00','suborbital rocket','success',520,'','15','','','Aeron',184,'',67 from dual union all
select 13871,'1966-M79','1966-02-02 19:35:00','suborbital rocket','success',520,'','16','','','Aeron',184,'',60 from dual union all
select 13872,'1966-M80','1966-02-02 16:00:00','suborbital rocket','success',520,'','17','','','Aeron',184,'',57 from dual union all
select 13873,'1966-M81','1966-02-07 16:00:00','suborbital rocket','success',520,'','18','','','Aeron',184,'',64 from dual union all
select 13874,'1966-M82','1966-02-09 16:03:00','suborbital rocket','success',520,'','19','','','Aeron',184,'',61 from dual union all
select 13875,'1966-M83','1966-02-11 16:00:00','suborbital rocket','success',520,'','20','','','Aeron',184,'',61 from dual union all
select 13876,'1966-M84','1966-02-16 16:00:00','suborbital rocket','success',520,'','21','','','Aeron',184,'',58 from dual union all
select 13877,'1966-M85','1966-02-17 17:36:00','suborbital rocket','success',520,'','22','','','Aeron',184,'',61 from dual union all
select 13878,'1966-M86','1966-02-18 15:01:00','suborbital rocket','success',520,'','23','','','Aeron',184,'',54 from dual union all
select 13879,'1966-M87','1966-02-20 21:53:00','suborbital rocket','success',520,'','24','','','Aeron',184,'',61 from dual union all
select 1388,'1952-A55','1952-04-01 00:00:00','atmospheric rocket','success',365,'','Rigel Model','','','Test',1821,'',10 from dual union all
select 13880,'1966-M88','1966-02-23 22:38:00','suborbital rocket','success',520,'','25','','','Aeron',184,'',66 from dual union all
select 13881,'1966-M89','1966-02-24 03:23:00','suborbital rocket','success',520,'','26','','','Aeron',184,'',56 from dual union all
select 13882,'1966-M90','1966-02-25 16:06:00','suborbital rocket','success',520,'','27','','','Aeron',184,'',56 from dual union all
select 13883,'1966-M91','1966-03-04 16:08:00','suborbital rocket','success',520,'','30','','','Aeron',184,'',59 from dual union all
select 13884,'1966-M92','1966-02-19 05:09:00','suborbital rocket','success',520,'','G01','','','Aeron',616,'',61 from dual union all
select 13885,'1966-M93','1966-02-19 16:07:00','suborbital rocket','success',520,'','G02','','','Aeron',616,'',61 from dual union all
select 13886,'1966-M94','1966-02-20 04:31:00','suborbital rocket','success',520,'','G05','','','Aeron',616,'',61 from dual union all
select 13887,'1966-M95','1966-02-24 03:00:00','suborbital rocket','success',520,'','G07','','','Aeron',616,'',50 from dual union all
select 13888,'1966-M96','1966-02-24 07:00:00','suborbital rocket','success',520,'','G08','','','Aeron',616,'',60 from dual union all
select 13889,'1966-M97','1966-03-03 21:03:00','suborbital rocket','success',520,'','G18','','','Aeron',616,'',61 from dual union all
select 1389,'1952-A56','1952-03-31 00:00:00','atmospheric rocket','unknown',576,'26','','','','Test',1895,'',10 from dual union all
select 13890,'1966-M98','1966-03-03 22:01:00','suborbital rocket','success',520,'','G19','','','Aeron',616,'',60 from dual union all
select 13891,'1966-M99','1966-03-04 02:30:00','suborbital rocket','success',520,'','G24','','','Aeron',616,'',60 from dual union all
select 13892,'1966-S01','1966-01-01 00:00:00','military missile','success',7,'6TYa','','','','Test',182,'',100 from dual union all
select 13893,'1966-S02','1966-01-01 00:00:00','military missile','success',7,'6TYa','','','','Test',182,'',100 from dual union all
select 13894,'1966-S03','1966-01-01 00:00:00','military missile','success',7,'6TYa','','','','Test',182,'',100 from dual union all
select 13895,'1966-S04','1966-01-01 00:00:00','military missile','success',7,'6TYa','','','','Test',182,'',100 from dual union all
select 13896,'1966-S05','1966-01-01 00:00:00','military missile','success',7,'6TYa','','','','Test',182,'',100 from dual union all
select 13897,'1966-S06','1966-01-01 00:00:00','military missile','failure',736,'','','','','Test',313,'',100 from dual union all
select 13898,'1966-S07','1966-01-01 00:00:00','military missile','success',736,'','','','','Test',313,'',100 from dual union all
select 13899,'1966-S08','1966-01-01 00:00:00','military missile','success',736,'','','','','Test',313,'',100 from dual union all
select 139,'1944-S126','1944-09-20 00:00:00','military missile','success',1470,'','Arras','','','Weapon',1730,'',95 from dual union all
select 1390,'1952-A57','1952-03-31 00:00:00','atmospheric rocket','unknown',576,'27','','','','Test',1895,'',10 from dual union all
select 13900,'1966-S09','1966-01-18 20:06:00','military missile','success',837,'BMRS','STV SR-022','','','Target',1895,'',100 from dual union all
select 13901,'1966-S10','1966-01-24 20:04:00','military missile','success',837,'','STV SR-023','','','Target',1895,'',100 from dual union all
select 13902,'1966-S100','1966-02-10 05:51:00','suborbital rocket','success',837,'NASA 14.188GE','Auroral electrons 1','','','Auroral',428,'',178 from dual union all
select 13903,'1966-S101','1966-02-10 07:09:00','suborbital rocket','success',839,'NASA 10.187GM','GSFC Grenade 102 C','','','Aeronomy',428,'',117 from dual union all
select 13904,'1966-S102','1966-02-10 07:48:00','suborbital rocket','success',839,'NASA 10.148GM','GSFC Grenade 103 W','','','Aeronomy',1821,'',114 from dual union all
select 13905,'1966-S103','1966-02-10 08:00:00','suborbital rocket','success',839,'NASA 10.183GM','GSFC Grenade 104 PB','','','Aeronomy',1039,'',100 from dual union all
select 13906,'1966-S104','1966-02-10 09:48:46','ballistic missile test','success',154,'305D','Nike RMV-303/Pod 32','NTMP KX-38','YEAST CAKE','Reentry Test/Plume',1680,'',1800 from dual union all
select 13907,'1966-S105','1966-02-10 18:00:00','suborbital rocket','success',839,'NASA 10.145GM','GSFC Grenade 105 C','','','Aeronomy',428,'',122 from dual union all
select 13908,'1966-S106','1966-02-10 18:41:00','suborbital rocket','success',839,'NASA 10.149GM','GSFC Grenade 106 W','','','Aeronomy',1821,'',114 from dual union all
select 13909,'1966-S107','1966-02-10 21:30:00','suborbital rocket','success',839,'NASA 10.184GM','GSFC Grenade 107 PB','','','Aeronomy',1039,'',100 from dual union all
select 1391,'1952-A58','1952-04-02 00:00:00','atmospheric rocket','unknown',576,'28','','','','Test',1895,'',10 from dual union all
select 13910,'1966-S108','1966-02-11 01:00:04','military missile','success',796,'460','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 13911,'1966-S109','1966-02-12 00:39:00','military missile','success',795,'1387','Mk 11 RV','FOT-2','CREEK BED','OT',1702,'',900 from dual union all
select 13912,'1966-S11','1966-03-26 06:17:00','military missile','success',1018,'T 15001-08','8K75 UBP-9','','','OT',580,'',1160 from dual union all
select 13913,'1966-S110','1966-02-11 13:04:53','military missile','success',154,'86D','Nike RMV-304','NTMP KX-51','LONELY MOUNTAIN','Reentry Test',1684,'',1800 from dual union all
select 13914,'1966-S111','1966-02-14 16:52:00','military missile','success',916,'AO-31','P-100','','','OT',435,'',180 from dual union all
select 13915,'1966-S112','1966-02-16 00:00:00','military missile','success',796,'2033','Mk 11A RV','R&D','CALAMITY JANE','Test',1711,'',900 from dual union all
select 13916,'1966-S113','1966-02-17 16:30:03','military missile','unknown',932,'A3P-14','','','','OT',421,'SSBN 644',1000 from dual union all
select 13917,'1966-S114','1966-02-17 00:00:00','military missile','success',135,'D014','ECM Decoy F-1','ABRES F-1 (USAF)','','Reentry Test',613,'',200 from dual union all
select 13918,'1966-S115','1966-02-17 09:45:52','military missile','success',1420,'B-61','Mk 6 RV','OT','BLACK HAWK','OT',1657,'',1300 from dual union all
select 13919,'1966-S116','1966-02-17 16:16:00','suborbital rocket','success',837,'NASA 14.243UE','E/B currents','','','Plasma',1821,'',186 from dual union all
select 1392,'1952-A59','1952-04-02 00:00:00','atmospheric rocket','unknown',576,'29','','','','Test',1895,'',10 from dual union all
select 13920,'1966-S117','1966-02-18 04:56:00','suborbital rocket','success',837,'NASA 14.189GE','Auroral electrons 2','','','Auroral',428,'',209 from dual union all
select 13921,'1966-S118','1966-02-19 00:05:00','suborbital rocket','success',850,'CRL AFS.502','REDLAMP ROBIN','','','Ionos/Aeron',1483,'',192 from dual union all
select 13922,'1966-S119','1966-02-19 09:56:42','military missile','success',154,'73D','Nike RMV-301','NTMP KX-47','SYCAMORE RIDGE','Reentry Test',1683,'',1800 from dual union all
select 13923,'1966-S12','1966-07-10 21:00:00','military missile','success',1018,'N 15001-05','8K75 UBP-10 ZP','','','OT',584,'',1160 from dual union all
select 13924,'1966-S120','1966-02-20 00:10:00','suborbital rocket','success',850,'CRL AFS.504','REDLAMP','','','Ionos/Aeron',1483,'',192 from dual union all
select 13925,'1966-S121','1966-02-20 05:30:00','suborbital rocket','success',30,'NASA 04.162UA','JHU Aurora 4','','','Aeronomy',428,'',166 from dual union all
select 13926,'1966-S122','1966-02-20 05:45:00','suborbital rocket','success',850,'CRL AFS.505','REDLAMP TONI','','','Ionos/Aeron',1483,'',192 from dual union all
select 13927,'1966-S123','1966-02-21 13:10:00','suborbital rocket','success',217,'NC7.181','','','','Aeron',394,'',175 from dual union all
select 13928,'1966-S124','1966-02-21 19:57:00','suborbital rocket','success',217,'NC7.180','','','','Aeron',394,'',188 from dual union all
select 13929,'1966-S125','1966-02-21 20:23:00','suborbital rocket','success',217,'NC7.178','O2 density','','','Aeron',428,'',196 from dual union all
select 1393,'1952-A60','1952-04-02 00:00:00','atmospheric rocket','success',869,'491','Warhead Test 88R','','','Test',1895,'',10 from dual union all
select 13930,'1966-S126','1966-02-22 18:28:56','suborbital rocket','success',288,'CE-13','ESRO C14/1','','','Auroral',29,'',115 from dual union all
select 13931,'1966-S127','1966-02-23 10:06:00','suborbital rocket','success',535,'HAD 186','Sphere','','','Aeron',1835,'',100 from dual union all
select 13932,'1966-S128','1966-02-23 23:14:52','suborbital rocket','success',288,'CE-14','ESRO C14/2','','','Auroral',29,'',117 from dual union all
select 13933,'1966-S129','1966-02-24 19:40:00','military missile','success',794,'529 (63-005)','Mk 5 RV','FOT-4 Salvo','BROAD ARROW','OT',1701,'',900 from dual union all
select 13934,'1966-S13','1966-01-01 00:00:00','military missile','success',1037,'','8K98 LKI-1 2','','','Test',632,'',1000 from dual union all
select 13935,'1966-S130','1966-02-24 19:40:00','military missile','success',794,'629 (63-112)','Mk 5 RV','FOT-5 Salvo','SEA DEVIL','OT',1699,'',900 from dual union all
select 13936,'1966-S131','1966-02-24 17:04:04','military missile','unknown',932,'A3P-380','','','','OT',421,'SSBN 642',1000 from dual union all
select 13937,'1966-S132','1966-02-25 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 625',1000 from dual union all
select 13938,'1966-S133','1966-02-25 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 625',1000 from dual union all
select 13939,'1966-S134','1966-02-25 13:59:00','suborbital rocket','success',837,'NASA 14.211IA','Rehbar 9  (Na)','','','Aeronomy',1419,'',200 from dual union all
select 1394,'1952-A61','1952-04-02 00:00:00','atmospheric rocket','success',869,'491','Warhead Test 90R','','','Test',1895,'',10 from dual union all
select 13940,'1966-S135','1966-02-26 00:00:00','military missile','success',1037,'','8K98 LKI-1 1','','','Test',632,'',1000 from dual union all
select 13941,'1966-S136','1966-02-26 13:59:00','suborbital rocket','success',837,'NASA 14.212IA','Rehbar 10 (Na)','','','Aeronomy',1419,'',200 from dual union all
select 13942,'1966-S137','1966-02-27 16:51:59','suborbital rocket','success',837,'NASA 14.251UM','UM Pitot 20/AS-201','','','Aeronomy',176,'',155 from dual union all
select 13943,'1966-S138','1966-03-01 00:00:00','military missile','success',837,'BMTS','Target','','','Target',437,'',120 from dual union all
select 13944,'1966-S139','1966-03-01 00:00:00','military missile','success',837,'BMTS','Target','','','Target',437,'',120 from dual union all
select 13945,'1966-S14','1966-01-01 00:00:00','military missile','success',1037,'','8K98 LKI-1 3','','','Test',632,'',1000 from dual union all
select 13946,'1966-S140','1966-03-11 19:00:00','military missile','success',837,'','STV SR-024','','','Target',1895,'',100 from dual union all
select 13947,'1966-S141','1966-03-29 19:14:00','military missile','success',837,'BMRS','STV SR-026','','','Target',1895,'',100 from dual union all
select 13948,'1966-S142','1966-03-15 00:00:00','suborbital rocket','success',1330,'','DAR-3','','','Test',1415,'',500 from dual union all
select 13949,'1966-S143','1966-03-01 00:00:00','military missile','unknown',929,'A2','','','','OT',421,'SSBN 611',1000 from dual union all
select 1395,'1952-A62','1952-04-02 00:00:00','atmospheric rocket','success',869,'491','Warhead Test 89R','','','Test',1895,'',10 from dual union all
select 13950,'1966-S144','1966-03-01 00:00:00','military missile','unknown',929,'A2','','','','OT',421,'SSBN 611',1000 from dual union all
select 13951,'1966-S145','1966-03-02 17:55:00','suborbital rocket','success',35,'NASA 04.99DS','Te, Solar UV','','','Solar EUV',1821,'',228 from dual union all
select 13952,'1966-S146','1966-03-02 17:59:00','suborbital rocket','success',624,'NASA 8.25GA-GI','Geoprobe','','','Aeron/Ionos',1821,'',536 from dual union all
select 13953,'1966-S147','1966-03-03 16:50:02','military missile','unknown',932,'A3P-135','-   (SSBN 642)','','','OT',421,'SSBN 644',1000 from dual union all
select 13954,'1966-S148','1966-03-03 21:54:00','suborbital rocket','success',35,'NASA 04.100DS','AFCRL Solar','','','Solar EUV',1821,'',219 from dual union all
select 13955,'1966-S149','1966-03-03 22:26:00','suborbital rocket','success',288,'C77','CNES FU-129','','','Solar UV',664,'',125 from dual union all
select 13956,'1966-S15','1966-01-01 00:00:00','military missile','success',1037,'','8K98 LKI-1 4','','','Test',632,'',1000 from dual union all
select 13957,'1966-S150','1966-03-04 12:29:55','military missile','failure',154,'303D','Nike RMV-303','NTMP KX-35','ETERNAL CAMP','Reentry Test',1680,'',500 from dual union all
select 13958,'1966-S151','1966-03-05 04:50:00','sounding rocket','success',698,'L-3H-1','','','','XR Astron',834,'',1829 from dual;
insert into LAUNCH
select 13959,'1966-S152','1966-03-08 00:00:00','military missile','success',795,'1362','Mk 11 RV','FOT-3','BAIT CAN','OT',1697,'',900 from dual union all
select 1396,'1952-A63','1952-04-04 00:00:00','military missile','success',771,'GM-561','','','','Test',222,'',12 from dual union all
select 13960,'1966-S153','1966-03-08 07:42:00','suborbital rocket','success',30,'NASA 04.148CG','AS/E/MIT-3','','','XR Astron',1869,'',165 from dual union all
select 13961,'1966-S154','1966-03-09 02:22:00','suborbital rocket','success',30,'KP3.15','Stellar spectra','','','UV Astron',1869,'',150 from dual union all
select 13962,'1966-S155','1966-03-09 22:20:21','suborbital rocket','success',288,'CE-12','ESRO C13/1','','','Auroral',29,'',94 from dual union all
select 13963,'1966-S156','1966-03-10 00:00:00','suborbital rocket','unknown',850,'','DASA HAR','','','Aeron?',1863,'',200 from dual union all
select 13964,'1966-S157','1966-03-10 16:00:02','military missile','unknown',932,'A3P-383','','','','OT',421,'SSBN 642',1000 from dual union all
select 13965,'1966-S158','1966-03-10 08:45:00','suborbital rocket','success',848,'CRL AF07.327','Bremss density','','','Ionos',394,'',205 from dual union all
select 13966,'1966-S159','1966-03-10 22:25:32','suborbital rocket','success',288,'CE-11','ESRO C13/2','','','Auroral',29,'',114 from dual union all
select 13967,'1966-S16','1966-01-01 00:00:00','suborbital rocket','success',206,'012','Titus Test','','','Test/Solar',313,'',270 from dual union all
select 13968,'1966-S160','1966-03-11 00:00:00','military missile','success',795,'1211','Mk 11 RV','FOT-4','CLEAN SLATE','OT',1698,'',900 from dual union all
select 13969,'1966-S161','1966-03-12 00:42:00','military missile','success',1373,'196','AFP-437(AP)','','','Interceptor',773,'',472 from dual union all
select 1397,'1952-A64','1952-04-09 00:00:00','military missile','success',771,'GM-559','','','','Test',222,'',12 from dual union all
select 13970,'1966-S162','1966-03-13 00:00:00','military missile','success',1117,'G1','','','','Reentry Test',669,'',1000 from dual union all
select 13971,'1966-S163','1966-03-13 05:13:00','suborbital rocket','success',837,'NASA 14.190GE','Auroral electrons 3','','','Auroral',428,'',200 from dual union all
select 13972,'1966-S164','1966-03-14 00:00:00','military missile','unknown',929,'A2','','','','OT',421,'SSBN 617',1000 from dual union all
select 13973,'1966-S165','1966-03-14 00:00:00','military missile','unknown',929,'A2','','','','OT',421,'SSBN 617',1000 from dual union all
select 13974,'1966-S166','1966-03-14 00:00:00','military missile','unknown',929,'A2','','','','OT',421,'SSBN 617',1000 from dual union all
select 13975,'1966-S167','1966-03-14 00:00:00','military missile','unknown',929,'A2','','','','OT',421,'SSBN 617',1000 from dual union all
select 13976,'1966-S168','1966-03-16 00:00:00','military missile','success',735,'','Bietage 2','','','Test',313,'',100 from dual union all
select 13977,'1966-S169','1966-03-16 23:50:00','suborbital rocket','success',1206,'SL581','RAE Test 31','','','Test',1839,'',221 from dual union all
select 13978,'1966-S17','1966-03-28 00:00:00','military missile','success',1449,'N 10744-308','8K84 UBP-1 SLI','','','OT',503,'',1000 from dual union all
select 13979,'1966-S170','1966-03-17 00:00:00','military missile','success',796,'2049','Mk 12 RV','Force Mod/NTMP KX-80','FAINT CLICK','Test',1703,'',900 from dual union all
select 1398,'1952-A65','1952-04-10 00:00:00','military missile','failure',703,'602','','','','Test',222,'',0 from dual union all
select 13980,'1966-S171','1966-03-17 00:00:00','military missile','unknown',929,'A2','','','','OT',421,'SSBN 616',1000 from dual union all
select 13981,'1966-S172','1966-03-17 00:00:00','military missile','unknown',929,'A2','','','','OT',421,'SSBN 616',1000 from dual union all
select 13982,'1966-S173','1966-03-17 16:05:00','military missile','success',982,'G 22604-13T','8K64U UBP-69','','','OT',571,'',1210 from dual union all
select 13983,'1966-S174','1966-03-18 00:00:00','military missile','success',1117,'G2','','','','Reentry Test',669,'',1000 from dual union all
select 13984,'1966-S175','1966-03-19 12:20:00','military missile','success',154,'304D','Nike RMV-301','NTMP KX-43','WHITE BEAR','Reentry Test',1680,'',1800 from dual union all
select 13985,'1966-S176','1966-03-19 12:20:30','suborbital rocket','success',850,'','HITAB 1','','SAGE HEN','Tech',1015,'',130 from dual union all
select 13986,'1966-S177','1966-03-20 12:15:00','suborbital rocket','success',668,'K-9M-15','XRA/Airglow','','','XR Astron',839,'',300 from dual union all
select 13987,'1966-S178','1966-03-21 00:00:00','military missile','success',794,'569 (63-061)','Mk 5 RV','FOT-6','TULIP TREE','OT',1701,'',900 from dual union all
select 13988,'1966-S179','1966-03-21 23:04:02','suborbital rocket','success',837,'NASA 14.109GI','D-region composition','','','Ionos',29,'',133 from dual union all
select 13989,'1966-S18','1966-04-01 00:00:00','military missile','success',1449,'N 10744-303','8K84 UBP-2 SLI','','','OT',508,'',1000 from dual union all
select 1399,'1952-A66','1952-04-17 00:00:00','military missile','failure',703,'601','','','','Test',222,'',0 from dual union all
select 13990,'1966-S180','1966-03-22 09:38:00','suborbital rocket','success',535,'HAD 181','Sphere','','','Aeron',1835,'',100 from dual union all
select 13991,'1966-S181','1966-03-23 04:07:00','suborbital rocket','success',867,'NASA 18.07GE','Auroral','','','Auroral',428,'',290 from dual union all
select 13992,'1966-S182','1966-03-24 05:49:00','military missile','success',982,'N 22601-20T','8K64U UBP-70','','','OT',566,'',1210 from dual union all
select 13993,'1966-S183','1966-03-24 13:30:00','suborbital rocket','success',837,'NASA 14.257IA','ISRO 10.07','TERLS-039','','Aeron',1467,'',201 from dual union all
select 13994,'1966-S184','1966-03-24 15:31:00','suborbital rocket','success',839,'NASA 10.180IM','Rehbar 11 (Grenades)','','','Aeronomy',1419,'',135 from dual union all
select 13995,'1966-S185','1966-03-24 16:45:00','suborbital rocket','success',288,'C101','ISRO 15.04','TERLS-040','','Aeron',1467,'',163 from dual union all
select 13996,'1966-S186','1966-03-24 17:02:00','suborbital rocket','success',1492,'VA80','FU-155','','','Test',668,'',209 from dual union all
select 13997,'1966-S187','1966-03-24 21:00:00','suborbital rocket','success',288,'C102','ISRO 15.05','TERLS-041','','Aeron',1467,'',159 from dual union all
select 13998,'1966-S188','1966-03-25 22:00:00','military missile','success',794,'649 (63-130)','Mk 5 RV','FOT-7','WHITE BOOK','OT',1699,'',900 from dual union all
select 13999,'1966-S189','1966-03-25 00:19:00','suborbital rocket','success',837,'NASA 14.258IA','ISRO 10.08','TERLS-042','','Aeron',1467,'',201 from dual union all
select 14,'1943-A07','1943-05-26 16:30:00','military missile','failure',8,'25','','','','Test',675,'',10 from dual union all
select 140,'1944-S127','1944-09-20 00:00:00','military missile','success',1470,'','Liege','','','Weapon',1730,'',95 from dual union all
select 1400,'1952-A67','1952-04-23 00:00:00','atmospheric rocket','failure',365,'','D109/FFAR seeker','','','Test',1821,'',1 from dual union all
select 14000,'1966-S19','1966-04-12 00:00:00','military missile','success',1449,'N 10744-311','8K84 UBP-3 SLI','','','OT',508,'',1000 from dual union all
select 14001,'1966-S190','1966-03-25 09:42:03','military missile','success',1420,'B-16','Mk 6 RV','OT/NTMP KX-52','CLOSE TOUCH','OT',1658,'',1300 from dual union all
select 14002,'1966-S191','1966-03-25 09:42:10','suborbital rocket','success',850,'','HITAB 2','','LEAD PENCIL','Tech',1015,'',130 from dual union all
select 14003,'1966-S192','1966-03-26 07:07:00','suborbital rocket','success',215,'AA-2A-103','','','','Auroral/Solar UV',428,'',153 from dual union all
select 14004,'1966-S193','1966-03-27 17:12:00','suborbital rocket','success',837,'NASA 14.165IM','Rehbar 12 (Grenades)','','','Aeronomy',1419,'',190 from dual union all
select 14005,'1966-S194','1966-03-28 00:00:00','military missile','success',1117,'R2','','','','Reentry Test',669,'',1000 from dual union all
select 14006,'1966-S195','1966-03-30 01:32:00','military missile','success',999,'N 22501-10T','8K67 LKI-37 SLI K','','','OT',514,'',1000 from dual union all
select 14007,'1966-S196','1966-03-30 04:19:00','suborbital rocket','success',215,'AK-2A-104','','','','Auroral/Ionos/Plasm',428,'',154 from dual union all
select 14008,'1966-S197','1966-03-30 18:35:00','suborbital rocket','success',30,'CRL AF3.524','EUV Mon/RPA','','','Solar EUV',1869,'',229 from dual union all
select 14009,'1966-S198','1966-04-01 00:00:00','military missile','success',837,'BMTS','Target','','','Target',437,'',120 from dual union all
select 1401,'1952-A68','1952-04-24 00:00:00','military missile','unknown',339,'21-17','','','','Test',1895,'',30 from dual union all
select 14010,'1966-S199','1966-04-23 02:10:00','military missile','success',839,'RN026','NICAP/1 026','','','Target',1895,'',120 from dual union all
select 14011,'1966-S20','1966-04-24 05:43:00','military missile','success',1449,'N 10744-312','8K84 UBP-5 SLI','GCh 8F121P1','','OT',504,'',1000 from dual union all
select 14012,'1966-S200','1966-04-21 18:45:00','military missile','success',837,'BMRS','STV SR-027','','','Target',1895,'',100 from dual union all
select 14013,'1966-S201','1966-04-21 21:03:00','military missile','success',837,'BMRS','STV SR-028','','','Target',1895,'',100 from dual union all
select 14014,'1966-S202','1966-03-31 16:00:00','military missile','success',932,'A3P-396','','','','OT',421,'SSBN 598',1000 from dual union all
select 14015,'1966-S203','1966-03-31 00:00:00','military missile','success',135,'D019','ECM Decoy F-3','ABRES F-3 (USAF)','','Reentry Test',614,'',200 from dual union all
select 14016,'1966-S204','1966-03-31 00:00:00','military missile','success',135,'B009','Delayed Chaff Payload','ABRES T-1 (USN)','','Reentry Test',612,'',200 from dual union all
select 14017,'1966-S205','1966-04-01 01:29:55','military missile','success',796,'462','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 14018,'1966-S206','1966-04-22 04:18:00','military missile','success',999,'N 22502-10T','8K67 LKI-38 SLI K','','','OT',514,'',1000 from dual union all
select 14019,'1966-S207','1966-04-04 00:00:00','military missile','success',795,'1258','Mk 11 RV','FOT-5','ARROW FEATHER','OT',1697,'',900 from dual union all
select 1402,'1952-A69','1952-04-24 00:00:00','atmospheric rocket','success',869,'491','Warhead Test 91R','','','Test',1895,'',10 from dual union all
select 14020,'1966-S208','1966-04-04 00:00:00','military missile','success',795,'1347','Mk 11 RV','FOT-6 /+7h','FLY BURNER','OT',1705,'',900 from dual union all
select 14021,'1966-S209','1966-04-06 16:52:00','suborbital rocket','success',837,'NASA 14.216AI','','','','Ionos',1821,'',174 from dual union all
select 14022,'1966-S21','1966-04-28 00:00:00','military missile','success',1449,'N 10701-01T','8K84 UBP-6 SLI','','','OT',540,'',1000 from dual union all
select 14023,'1966-S210','1966-04-05 00:00:00','military missile','success',1117,'R1','','','','Reentry Test',669,'',1000 from dual union all
select 14024,'1966-S211','1966-04-05 16:20:19','military missile','success',1420,'B-50','Mk 6 RV','OT','GOLD RING','OT',1659,'',1300 from dual union all
select 14025,'1966-S212','1966-04-06 15:30:00','suborbital rocket','success',1493,'AGI/57','FU-126 Solar corona','','','Solar UV',668,'',130 from dual union all
select 14026,'1966-S213','1966-04-07 16:00:00','military missile','failure',932,'A3P-445','1 of 2 failed','','','OT',421,'SSBN 643',20 from dual union all
select 14027,'1966-S214','1966-04-07 16:30:00','military missile','success',932,'A3P-400','','','','OT',421,'SSBN 643',1000 from dual union all
select 14028,'1966-S215','1966-04-08 04:01:00','suborbital rocket','success',837,'NASA 14.76UI','MIP 3','','','Ionos',1821,'',159 from dual union all
select 14029,'1966-S216','1966-04-12 07:47:00','military missile','success',982,'G 22602-03T','8K64U UBP-71','','','OT',566,'',1210 from dual union all
select 1403,'1952-A70','1952-04-24 00:00:00','atmospheric rocket','success',869,'491','Warhead Test 92R','','','Test',1895,'',10 from dual union all
select 14030,'1966-S217','1966-04-14 16:00:00','military missile','unknown',932,'A3P-375','','','','OT',421,'SSBN 598',1000 from dual union all
select 14031,'1966-S218','1966-04-14 05:55:00','suborbital rocket','success',867,'NASA 18.08GE','Auroral','','','Auroral',428,'',296 from dual union all
select 14032,'1966-S219','1966-04-14 15:00:00','suborbital rocket','success',30,'NASA 04.143NA','Dayglow','','','Aeronomy',1869,'',183 from dual union all
select 14033,'1966-S22','1966-04-29 00:00:00','military missile','success',1449,'N 22901-12P','8K84 UBP-7 SLI','','','OT',505,'',1000 from dual union all
select 14034,'1966-S220','1966-04-14 15:08:00','suborbital rocket','success',30,'NASA 04.24US','','','','Solar',1869,'',192 from dual union all
select 14035,'1966-S221','1966-04-15 00:00:00','military missile','success',795,'1312','Mk 11 RV','FOT-7','GAY CROWD','OT',1702,'',900 from dual union all
select 14036,'1966-S222','1966-04-18 06:49:00','military missile','success',982,'G 22602-04T','8K64U UBP-72','','','OT',566,'',1210 from dual union all
select 14037,'1966-S223','1966-04-20 02:30:00','suborbital rocket','success',1204,'SL582','RAE Test 32','','','Test',1839,'',172 from dual union all
select 14038,'1966-S224','1966-04-20 08:44:54','military missile','success',1420,'B-55','Mk 6 RV','OT','LONG LIGHT','OT',1657,'',1300 from dual union all
select 14039,'1966-S225','1966-04-20 12:05:00','suborbital rocket','success',665,'K-8-13','','','','Aeron/Plasma/Fields',839,'',155 from dual union all
select 1404,'1952-A71','1952-04-29 00:00:00','military missile','unknown',339,'22-18','','','','Test',1895,'',30 from dual union all
select 14040,'1966-S226','1966-04-21 16:00:00','military missile','unknown',932,'A3P-440','','','','OT',421,'SSBN 643',1000 from dual union all
select 14041,'1966-S227','1966-04-21 16:30:00','military missile','unknown',932,'A3P-431','','','','OT',421,'SSBN 643',1000 from dual union all
select 14042,'1966-S228','1966-04-21 19:15:02','military missile','unknown',932,'A3P-437','','','','OT',421,'SSBN 643',1000 from dual union all
select 14043,'1966-S229','1966-04-21 00:06:00','suborbital rocket','success',1204,'SL583','RAE Test 33','','','Test',1839,'',193 from dual union all
select 14044,'1966-S23','1966-05-30 00:00:00','military missile','success',1449,'N 10744-351','8K84 UBP-8 SLI','','','OT',540,'',1000 from dual union all
select 14045,'1966-S230','1966-04-21 02:10:00','suborbital rocket','success',837,'','Sandia 154-75','','','Aeron',1490,'',160 from dual union all
select 14046,'1966-S231','1966-04-21 11:25:00','suborbital rocket','success',30,'NASA 04.165UA','JHU UVT F1 (Venus)','','','UV Astron    STRAP?',1869,'',154 from dual union all
select 14047,'1966-S232','1966-04-22 00:00:00','military missile','success',795,'1252','Mk 11 RV','FOT-8','ECHO CANYON','OT',1698,'',900 from dual;
insert into LAUNCH
select 14048,'1966-S233','1966-04-22 20:00:00','sounding rocket','success',1068,'R9','Rubis 03/MPE Ba-17','','','Plasma/Ionos/Ba',664,'',2035 from dual union all
select 14049,'1966-S234','1966-04-26 09:59:00','military missile','success',982,'G 22602-01T','8K64U UBP-73','','','OT',566,'',1210 from dual union all
select 1405,'1952-A72','1952-05-02 00:00:00','military missile','success',771,'GM-560','','','','Test',222,'',12 from dual union all
select 14050,'1966-S235','1966-04-26 03:15:00','military missile','success',999,'U 22501-09T','8K67 LKI-39','','','Test',485,'',1000 from dual union all
select 14051,'1966-S236','1966-04-26 00:12:00','suborbital rocket','success',837,'NASA 14.249IM','Rehbar 13 (Gren/TMA)','','','Aeronomy',1419,'',190 from dual union all
select 14052,'1966-S237','1966-04-28 12:35:00','suborbital rocket','success',30,'NASA 04.189DS','NRL ND3.172R','','','Solar SX',1869,'',175 from dual union all
select 14053,'1966-S239','1966-04-29 03:49:00','suborbital rocket','success',30,'CRL AD3.723','IR Horizon 2','','','Tech IR',1869,'',160 from dual union all
select 14054,'1966-S24','1966-06-08 00:00:00','military missile','success',1449,'N 10744-352','8K84 UBP-9 SLI','','','OT',516,'',1000 from dual union all
select 14055,'1966-S240','1966-05-01 00:00:00','military missile','success',837,'BMTS','Target','','','Target',437,'',120 from dual union all
select 14056,'1966-S241','1966-05-02 14:26:00','military missile','success',837,'BMRS','STV SR-029','','','Target',1895,'',100 from dual union all
select 14057,'1966-S242','1966-05-02 16:47:00','military missile','success',837,'BMRS','STV SR-030','','','Target',1895,'',100 from dual union all
select 14058,'1966-S243','1966-05-24 22:00:00','military missile','success',837,'BMRS','STV SR-031','','','Target',1895,'',100 from dual union all
select 14059,'1966-S244','1966-05-01 05:17:00','suborbital rocket','success',624,'CRL AD19.286','B/p/e','','','Auroral/Fields',428,'',710 from dual union all
select 1406,'1952-A73','1952-05-02 00:00:00','atmospheric rocket','success',836,'1249-102','Nike 1249 R&D 93R','','','Test',1895,'',10 from dual union all
select 14060,'1966-S245','1966-05-01 22:10:00','suborbital rocket','success',839,'NASA 10.190GM','GSFC Grenade 108 PB','','','Aeronomy',1039,'',126 from dual union all
select 14061,'1966-S246','1966-05-02 20:35:00','military missile','success',795,'1388','Mk 11 RV','FOT-9','LACE STRAP','OT',1697,'',900 from dual union all
select 14062,'1966-S247','1966-05-02 01:14:00','suborbital rocket','success',839,'NASA 10.188GM','GSFC Grenade 109 W','','','Aeronomy',1821,'',131 from dual union all
select 14063,'1966-S248','1966-05-02 01:19:00','suborbital rocket','success',839,'NASA 10.194GM','GSFC Grenade 110 N','','','Aeronomy',330,'',126 from dual union all
select 14064,'1966-S249','1966-05-02 02:32:00','suborbital rocket','success',839,'NASA 10.192GM','GSFC Grenade 111 C','','','Aeronomy',428,'',126 from dual union all
select 14065,'1966-S25','1966-06-29 00:00:00','military missile','success',1449,'N 22944-307','8K84 UBP-11 SLI','','','OT',504,'',1000 from dual union all
select 14066,'1966-S250','1966-05-03 10:30:20','military missile','failure',154,'208D','Nike RMV-303','NTMP KX-37','CRAB CLAW','Reentry Test',1680,'',500 from dual union all
select 14067,'1966-S251','1966-05-03 22:01:00','suborbital rocket','success',839,'NASA 10.191GM','GSFC Grenade 112 PB','','','Aeronomy',1039,'',126 from dual union all
select 14068,'1966-S252','1966-05-04 00:08:00','suborbital rocket','success',839,'NASA 10.193GM','GSFC Grenade 113 C','','','Aeronomy',428,'',123 from dual union all
select 14069,'1966-S253','1966-05-04 00:37:00','suborbital rocket','success',839,'NASA 10.189GM','GSFC Grenade 114 W','','','Aeronomy',1821,'',119 from dual union all
select 1407,'1952-A74','1952-05-12 00:00:00','military missile','unknown',339,'23-19','','','','Test',1895,'',30 from dual union all
select 14070,'1966-S254','1966-05-04 02:00:00','suborbital rocket','failure',446,'D-12','FU-130 Ly alpha','','','Aeronomy/Lalpha',664,'',440 from dual union all
select 14071,'1966-S255','1966-05-04 01:20:00','suborbital rocket','success',839,'NASA 10.195GM','08 CNAE 6605/Gren.','','','Aeronomy GSFC Gren 115',330,'',141 from dual union all
select 14072,'1966-S256','1966-05-05 04:14:00','suborbital rocket','success',1207,'SL304','LUX-34/MSSL SX1-SUV9','','','Solar X,UV',1839,'',203 from dual union all
select 14073,'1966-S257','1966-05-06 00:00:00','military missile','success',874,'ZK-61 20139','','','','Interceptor',873,'',80 from dual union all
select 14074,'1966-S258','1966-05-09 15:20:00','suborbital rocket','success',30,'NASA 04.106NP','JPL Rocket Radar 4','','','Tech',1869,'',174 from dual union all
select 14075,'1966-S259','1966-05-11 00:00:00','suborbital rocket','success',865,'Sandia 152-24','Radiochemistry','','','Aeron',855,'',302 from dual union all
select 14076,'1966-S26','1966-07-05 00:00:00','military missile','success',1449,'N 229   001','8K84P UBP-12 SLI','','','OT',540,'',1000 from dual union all
select 14077,'1966-S260','1966-05-12 00:00:00','military missile','success',135,'D018','ECM Decoy F-2','ABRES F-2 (USAF)','','Reentry Test',612,'',200 from dual union all
select 14078,'1966-S261','1966-05-12 18:00:00','suborbital rocket','success',446,'D-13','FU-130 Ly alpha','','','Aeronomy',664,'',464 from dual union all
select 14079,'1966-S262','1966-05-13 11:43:52','military missile','success',154,'98D','ABRES WAC-5A/GTV-5A','','SUPPLY ROOM','Reentry Test',1683,'',1800 from dual union all
select 1408,'1952-A75','1952-05-15 00:00:00','military missile','failure',771,'GM-563','','','','Test',222,'',12 from dual union all
select 14080,'1966-S263','1966-05-15 05:02:00','suborbital rocket','success',288,'CE-5','ESRO C06/2','','','Solar X',831,'',143 from dual union all
select 14081,'1966-S264','1966-05-16 14:10:00','military missile','success',794,'710 (63-175)','Mk 5 RV','FOT-8','SAGE GREEN','OT',1701,'',900 from dual union all
select 14082,'1966-S265','1966-05-16 18:30:00','military missile','success',795,'1191','Mk 11 RV','FOT-10','DOCK WORKER','OT',1705,'',900 from dual union all
select 14083,'1966-S266','1966-05-17 00:00:00','military missile','success',795,'1314','Mk 11 RV','FOT-11','TIGHT DRUM','OT',1702,'',900 from dual union all
select 14084,'1966-S267','1966-05-17 08:42:00','suborbital rocket','success',535,'HAD 188','Sphere','','','Aeron',1835,'',100 from dual union all
select 14085,'1966-S268','1966-05-18 04:02:00','suborbital rocket','success',35,'NASA 04.171UG','','','','UV Astron',1821,'',210 from dual union all
select 14086,'1966-S269','1966-05-20 09:30:30','suborbital rocket','success',288,'CE-19','ESRO C22/1','','','Solar UV',831,'',118 from dual union all
select 14087,'1966-S27','1966-07-18 00:00:00','military missile','success',1449,'N 229   002','8K84P UBP-13 SLI','','','OT',542,'',1000 from dual union all
select 14088,'1966-S270','1966-05-20 11:40:00','suborbital rocket','success',288,'CE-20','ESRO C22/2','','','Solar UV',831,'',114 from dual union all
select 14089,'1966-S271','1966-05-20 15:10:00','suborbital rocket','success',30,'NASA 04.95GS','XRT (2)','','','Solar X',1869,'',172 from dual union all
select 1409,'1952-A76','1952-05-16 00:00:00','military missile','unknown',339,'24-20RSC','','','','Test',1895,'',30 from dual union all
select 14090,'1966-S272','1966-05-20 19:04:00','suborbital rocket','success',624,'NASA 8.44GR','RAE development','','','RF Astron',1821,'',998 from dual union all
select 14091,'1966-S273','1966-05-20 23:30:00','suborbital rocket','success',30,'NASA 04.92GS','','','','Solar',1869,'',195 from dual union all
select 14092,'1966-S274','1966-05-23 00:00:00','suborbital rocket','success',865,'Sandia 152-91','ACS','','','Tech',855,'',270 from dual union all
select 14093,'1966-S275','1966-05-23 05:26:00','suborbital rocket','success',446,'D-14','FU-131 Atomic H','','','Aeronomy',664,'',460 from dual union all
select 14094,'1966-S276','1966-05-23 23:36:00','test rocket','success',452,'F-4','','','','-',1845,'',86 from dual union all
select 14095,'1966-S277','1966-05-24 00:00:00','suborbital rocket','success',865,'Sandia 152-60','SPACS test','','','Tech',855,'',270 from dual union all
select 14096,'1966-S278','1966-05-24 00:00:00','military missile','success',1420,'B-91','Mk 6 RV','FOT','SILVER BULLET','OT',1658,'',1300 from dual union all
select 14097,'1966-S279','1966-05-24 05:07:00','suborbital rocket','failure',30,'NASA 04.51UG','','','','UV Astron',1869,'',132 from dual union all
select 14098,'1966-S28','1966-07-28 00:00:00','military missile','success',1449,'','8K84 UBP-14 SLI','','','OT',596,'',1000 from dual union all
select 14099,'1966-S280','1966-05-26 00:00:00','suborbital rocket','failure',865,'Sandia 152-92','ACS','','','Tech',855,'',270 from dual union all
select 141,'1944-S128','1944-09-20 00:00:00','military missile','success',1470,'','Maastricht','','','Weapon',1730,'',95 from dual union all
select 1410,'1952-A77','1952-05-16 00:00:00','atmospheric rocket','success',365,'','E15 Model','','','Test',1821,'',10 from dual union all
select 14100,'1966-S281','1966-05-26 12:04:27','military missile','success',154,'41D','Nike RMV-301','NTMP KX-27','SAND SHARK','Reentry Test',1684,'',1800 from dual union all
select 14101,'1966-S282','1966-05-26 19:05:12','suborbital rocket','success',446,'D-20','FU-151 Ions/part.','','','Ionos',29,'',230 from dual union all
select 14102,'1966-S283','1966-05-27 00:00:00','military missile','success',135,'C019','Chaff Payload E-7','ABRES E-7 (USAF)','','Reentry Test',613,'',200 from dual union all
select 14103,'1966-S284','1966-05-27 00:00:00','military missile','success',135,'D031','Chaff Payload E-6','ABRES E-6 (USAF)','','Reentry Test',612,'',200 from dual union all
select 14104,'1966-S285','1966-05-29 03:05:00','military missile','success',999,'U 22500-30L','8K67 LKI-40','GCh 8F675-6000','','Test',514,'',1000 from dual union all
select 14105,'1966-S286','1966-05-31 20:10:00','military missile','success',795,'1270','Mk 11 RV','FOT-12','GREEN PEA','OT',1698,'',900 from dual union all
select 14106,'1966-S287','1966-05-31 22:55:00','military missile','success',794,'622 (63-109)','Mk 5 RV','FOT-9','NIGHT STAND','OT',1699,'',900 from dual union all
select 14107,'1966-S288','1966-06-02 16:43:00','military missile','success',837,'BMTS','STV SR-032','','','Target',437,'',120 from dual union all
select 14108,'1966-S289','1966-06-01 00:00:00','military missile','success',837,'BMTS','Target','','','Target',437,'',120 from dual union all
select 14109,'1966-S29','1966-08-01 00:00:00','military missile','success',1449,'N 10744-310','8K84 UBP-15 SLI','','','OT',508,'',1000 from dual union all
select 1411,'1952-A78','1952-05-16 00:00:00','atmospheric rocket','success',564,'','D18 Model','','','Test',1821,'',5 from dual union all
select 14110,'1966-S290','1966-06-07 09:44:00','military missile','success',837,'BMRS Photometric','STV SR-033 Photometric','','','Target',1895,'',100 from dual union all
select 14111,'1966-S291','1966-06-27 00:00:00','military missile','success',916,'AO-34','P-103 66 Summer 02','','','OT',597,'',180 from dual union all
select 14112,'1966-S292','1966-07-19 00:00:00','military missile','success',916,'AO-35','P-105 66 Summer 04','','','OT',204,'',180 from dual union all
select 14113,'1966-S293','1966-06-01 00:00:00','military missile','success',1548,'','','','','Test',636,'',620 from dual union all
select 14114,'1966-S294','1966-05-31 17:16:00','suborbital rocket','success',589,'BOX-4','LRL BOX-4','','','XR Astron',1210,'T-AGM-8',188 from dual union all
select 14115,'1966-S295','1966-05-31 19:19:00','suborbital rocket','success',1199,'SL422','MSSL-1/SHF-5','','','Ionospheric',1839,'',161 from dual union all
select 14116,'1966-S296','1966-06-01 00:12:18','suborbital rocket','success',446,'D-23','FU-153 Ions/part.','','','Ionos/Plasma/Field',29,'',380 from dual union all
select 14117,'1966-S297','1966-06-02 22:41:00','military missile','success',795,'1240','Mk 11 RV','FOT-13','FOUR ACES','OT',1697,'',900 from dual union all
select 14118,'1966-S298','1966-06-02 08:34:00','suborbital rocket','success',1199,'SL327','MO UVO-1/UCW-10','','','Aeronomy/Solar UV',1839,'',176 from dual union all
select 14119,'1966-S299','1966-06-04 04:16:00','military missile','success',982,'N 22601-10T','8K64U UBP-75','','','OT',566,'',1210 from dual union all
select 1412,'1952-A79','1952-05-16 00:00:00','military missile','success',703,'603','','','','Test',222,'',8 from dual union all
select 14120,'1966-S30','1966-08-03 00:00:00','military missile','success',1449,'N 10744-357','8K84 UBP-16 SLI','','','OT',542,'',1000 from dual union all
select 14121,'1966-S300','1966-06-07 15:57:00','suborbital rocket','success',1518,'','WASP slosh test','','','Tech',1821,'',238 from dual union all
select 14122,'1966-S301','1966-06-08 02:41:00','military missile','success',982,'G 22603-07T','8K64U UBP-76','','','OT',566,'',1210 from dual union all
select 14123,'1966-S302','1966-06-08 14:50:00','suborbital rocket','success',837,'','Sandia 154-53','','','Aeron',855,'',173 from dual union all
select 14124,'1966-S303','1966-06-08 15:15:00','suborbital rocket','success',837,'','Sandia 154-66','','','Aeron',855,'',100 from dual union all
select 14125,'1966-S304','1966-06-09 08:33:00','suborbital rocket','success',1199,'SL328','MO UVO-2/UCW-11','','','Aeronomy/Solar UV',1839,'',184 from dual union all
select 14126,'1966-S305','1966-06-10 17:39:00','military missile','success',795,'1261','Mk 11 RV','FOT-14','EBONY ANGEL','OT',1705,'',900 from dual union all
select 14127,'1966-S306','1966-06-10 16:09:00','military missile','unknown',932,'A3P-421','','','','OT',421,'SSBN 645',1000 from dual union all
select 14128,'1966-S307','1966-06-10 04:17:00','military missile','success',999,'N 22503-10T','8K67 UBP-1 SLI K','','','OT',485,'',1000 from dual union all
select 14129,'1966-S308','1966-06-10 00:00:00','military missile','success',135,'D023','ABRES QQ-1 (USAF)','ABRES QQ-1 (USAF)','','Reentry Test',613,'',200 from dual union all
select 1413,'1952-A80','1952-05-16 00:00:00','atmospheric rocket','success',836,'1249-103','Nike 1249 R&D 94R','','','Test',1895,'',10 from dual union all
select 14130,'1966-S309','1966-06-10 11:15:00','military missile','success',154,'96D','Nike RMV-302','NTMP KX-42','VENEER PANEL','Reentry Test',1683,'',1800 from dual union all
select 14131,'1966-S31','1966-08-04 00:00:00','military missile','success',1449,'N 10744-304','8K84 UBP-17 SLI','','','OT',504,'',1000 from dual union all
select 14132,'1966-S310','1966-06-12 05:54:00','suborbital rocket','success',837,'','Sandia 154-55','','','Aeron',855,'',165 from dual union all
select 14133,'1966-S311','1966-06-12 07:37:00','suborbital rocket','success',837,'','Sandia 154-52','','','Aeron',855,'',160 from dual union all
select 14134,'1966-S312','1966-06-12 08:57:00','suborbital rocket','success',837,'','Sandia 154-56','','','Aeron',855,'',170 from dual union all
select 14135,'1966-S313','1966-06-12 09:18:00','suborbital rocket','success',837,'','Sandia 154-71','','','Aeron',855,'',100 from dual union all
select 14136,'1966-S314','1966-06-13 19:38:00','suborbital rocket','success',288,'C68','Na/Turbulence','','','Aeronomy',664,'',180 from dual union all
select 14137,'1966-S315','1966-06-14 09:18:00','suborbital rocket','success',837,'NASA 14.270UI','','','','Ionos',1821,'',186 from dual;
insert into LAUNCH
select 14138,'1966-S316','1966-06-14 14:27:00','suborbital rocket','success',446,'D-15','FU-131 Atomic H','','','Aeronomy',664,'',480 from dual union all
select 14139,'1966-S317','1966-06-15 08:35:00','suborbital rocket','success',535,'HAD 185','Sphere','','','Aeron',1835,'',100 from dual union all
select 1414,'1952-A81','1952-05-20 00:00:00','suborbital rocket','failure',1490,'N1','','','','Test',666,'',19 from dual union all
select 14140,'1966-S318','1966-06-15 19:51:00','suborbital rocket','success',446,'D-22','FU-131 Na release','','','Aeronomy',664,'',500 from dual union all
select 14141,'1966-S319','1966-06-16 00:00:00','suborbital rocket','success',288,'C70','Na/Turbulence','','','Aeronomy',664,'',183 from dual union all
select 14142,'1966-S32','1966-08-09 00:00:00','military missile','success',1449,'','8K84 UBP-18 SLI','','','OT',508,'',1000 from dual union all
select 14143,'1966-S320','1966-06-16 05:46:00','suborbital rocket','success',837,'','Sandia 154-84','','','Aeron',855,'',170 from dual union all
select 14144,'1966-S321','1966-06-16 06:25:00','suborbital rocket','success',837,'','Sandia 154-82','','','Aeron',855,'',110 from dual union all
select 14145,'1966-S322','1966-06-16 19:29:00','suborbital rocket','success',1199,'ESRO S17/1','ESRO S17/1 MPE Ba','','','Aeronomy/Ba',1343,'',230 from dual union all
select 14146,'1966-S323','1966-06-17 03:18:00','suborbital rocket','success',839,'NASA 10.196GM','GSFC Grenade 117 PB','','','Aeron',1039,'',100 from dual union all
select 14147,'1966-S324','1966-06-17 03:13:00','suborbital rocket','success',839,'NASA 10.198GM','GSFC Grenade 116 C','','','Aeron',428,'',123 from dual union all
select 14148,'1966-S325','1966-06-18 01:25:00','suborbital rocket','success',839,'CRL AF6.501','ANGELA  (TMA)','','','Aeron',394,'',217 from dual union all
select 14149,'1966-S326','1966-06-18 02:41:30','suborbital rocket','success',1199,'ESRO S17/2','ESRO S17/2 MPE Ba','','','Aeronomy/Ba',1343,'',220 from dual union all
select 1415,'1952-A82','1952-05-21 00:00:00','suborbital rocket','failure',1490,'N2','','','','Test',666,'',14 from dual union all
select 14150,'1966-S327','1966-06-20 19:39:00','suborbital rocket','success',1199,'ISRC-SK-7','ESRO S08/1 MPE Ba','','','Aeronomy/Ba/Ozone',1343,'',220 from dual union all
select 14151,'1966-S328','1966-06-22 00:00:00','military missile','success',916,'AO-33','P-102 66 Summer 01','','','OT',597,'',180 from dual union all
select 14152,'1966-S329','1966-06-23 00:00:00','military missile','success',1117,'R4','','','','Reentry Test',669,'',1000 from dual union all
select 14153,'1966-S33','1966-08-11 00:00:00','military missile','success',1449,'','8K84 UBP-19','','','OT',536,'',1000 from dual union all
select 14154,'1966-S330','1966-06-23 16:00:00','military missile','unknown',932,'A3P-484','','','','OT',421,'SSBN 645',1000 from dual union all
select 14155,'1966-S331','1966-06-23 00:00:00','military missile','success',135,'D034','RVTO','ABRES SS-1 (USAF)','','Reentry Test',614,'',200 from dual union all
select 14156,'1966-S332','1966-06-23 01:38:00','suborbital rocket','success',1199,'SL421','MSSL-2/SHF-6','','','Ionospheric',1839,'',170 from dual union all
select 14157,'1966-S333','1966-06-23 06:35:00','suborbital rocket','success',839,'NASA 10.199GM','GSFC Grenade 118 C','','','Aeronomy',428,'',123 from dual union all
select 14158,'1966-S334','1966-06-23 07:52:00','suborbital rocket','success',839,'NASA 10.197GM','GSFC Grenade 119 PB','','','Aeronomy',1039,'',100 from dual union all
select 14159,'1966-S335','1966-06-23 08:53:00','suborbital rocket','success',848,'CRL AF07.621','DALE (Niro TMA E1)','','','Aeron',394,'',158 from dual union all
select 1416,'1952-A83','1952-05-22 00:00:00','atmospheric rocket','success',365,'','F29 Ramjet','','','Test',1821,'',10 from dual union all
select 14160,'1966-S336','1966-06-24 23:10:00','military missile','success',796,'2083','Mk 12 RV','Force Mod','FOX TRAP','Test',1703,'',900 from dual union all
select 14161,'1966-S337','1966-06-25 13:29:00','military missile','success',982,'N 22601-04T','8K64U UBP-78','Pal''ma-1','','OT',572,'',1210 from dual union all
select 14162,'1966-S338','1966-06-25 01:30:00','suborbital rocket','success',848,'CRL AF07.506','','','','Aeron',394,'',130 from dual union all
select 14163,'1966-S339','1966-06-25 03:36:00','suborbital rocket','success',848,'CRL AF07.509','BABY','','','Aeron',394,'',157 from dual union all
select 14164,'1966-S34','1966-08-12 00:00:00','military missile','success',1449,'','8K84 UBP-20','','','OT',542,'',1000 from dual union all
select 14165,'1966-S340','1966-06-25 08:15:00','suborbital rocket','success',848,'CRL AF07.620','CORA','','','Aeron',394,'',140 from dual union all
select 14166,'1966-S341','1966-06-25 09:57:00','suborbital rocket','success',848,'CRL AF07.507','','','','Aeron',394,'',179 from dual union all
select 14167,'1966-S342','1966-06-25 08:52:00','military missile','success',982,'N 22601-11T','8K64U UBP-77','Pal''ma-1','','OT',571,'',1210 from dual union all
select 14168,'1966-S343','1966-06-26 15:34:47','military missile','success',154,'147D','Nike RMV-302','NTMP KX-20','GOLDEN MOUNTAIN','Reentry Test',1684,'',1800 from dual union all
select 14169,'1966-S344','1966-06-26 21:47:13','suborbital rocket','success',837,'Ferdinand 13','50/70 Hz probe','','','Ionos',29,'',138 from dual union all
select 1417,'1952-A84','1952-05-22 00:00:00','atmospheric rocket','success',442,'','Navaho Model','','','Test',1821,'',10 from dual union all
select 14170,'1966-S345','1966-06-27 17:05:00','suborbital rocket','success',1493,'AGI/60','FU-154 Recovery test','','','Test',668,'',123 from dual union all
select 14171,'1966-S346','1966-06-28 02:53:30','suborbital rocket','success',1199,'ISRC-SK-8','ESRO S08/2 MPE Ba','','','Aeronomy/Ba/Ozone',1343,'',228 from dual union all
select 14172,'1966-S347','1966-06-28 16:23:00','suborbital rocket','success',624,'CRL AC19.191','Pulse Phase Delay','','','Ionos',1821,'',546 from dual union all
select 14173,'1966-S348','1966-07-01 00:00:00','suborbital rocket','success',893,'No. 8','','','','Test',310,'',90 from dual union all
select 14174,'1966-S349','1966-07-21 00:00:00','military missile','success',837,'BMRS','STV SR-033','','','Target',1895,'',100 from dual union all
select 14175,'1966-S35','1966-08-17 00:00:00','military missile','success',1449,'','8K84 UBP-22','','','OT',596,'',1000 from dual union all
select 14176,'1966-S350','1966-07-28 17:00:00','military missile','success',837,'BMRS','STV SR-034','','','Target',1895,'',100 from dual union all
select 14177,'1966-S351','1966-07-24 06:05:00','military missile','success',839,'','NICAP/1 66-16','','','Target',1895,'',120 from dual union all
select 14178,'1966-S352','1966-07-24 07:30:00','military missile','success',839,'','NICAP/1 66-17','','','Target',1895,'',120 from dual union all
select 14179,'1966-S353','1966-07-24 09:00:00','military missile','success',839,'','NICAP/1 66-18','','','Target',1895,'',120 from dual union all
select 1418,'1952-A85','1952-05-22 00:00:00','atmospheric rocket','unknown',576,'30','','','','Test',1895,'',10 from dual union all
select 14180,'1966-S354','1966-07-16 00:00:00','military missile','success',837,'BMTS','Target','','','Target',437,'',120 from dual union all
select 14181,'1966-S355','1966-06-30 03:25:00','military missile','success',999,'N 22500-01T','8K67 UBP-2','','','Test',514,'',1000 from dual union all
select 14182,'1966-S356','1966-07-15 00:00:00','military missile','success',1037,'','8K98 LKI-1 7','','','Test',632,'',1000 from dual union all
select 14183,'1966-S357','1966-06-30 10:00:00','military missile','success',154,'298D','Nike RMV-303','NTMP KX-39/RMP-A','HEAVY ARTILLERY','Reentry Test',1680,'',1800 from dual union all
select 14184,'1966-S358','1966-07-02 06:22:00','military missile','success',982,'G 22602-02T','8K64U UBP-79','','','OT',572,'',1210 from dual union all
select 14185,'1966-S359','1966-07-02 20:53:00','military missile','success',1373,'289','AFP-437(AP)','','','Interceptor',773,'',802 from dual union all
select 14186,'1966-S36','1966-08-20 00:00:00','military missile','success',1449,'','8K84 UBP-23','','','OT',503,'',1000 from dual union all
select 14187,'1966-S360','1966-07-07 08:15:00','suborbital rocket','success',837,'NASA 14.59IE','ISRO 20.05','TERLS-044','','Plasma',1467,'',178 from dual union all
select 14188,'1966-S361','1966-07-08 19:47:00','suborbital rocket','success',1199,'ISRC-SK-18','ESRO S18/1','','','Ionospheric',1343,'',244 from dual union all
select 14189,'1966-S362','1966-07-09 02:30:15','military missile','success',796,'463','Mk 12 RV','R&D','','Test',266,'',900 from dual union all
select 1419,'1952-A86','1952-05-22 00:00:00','atmospheric rocket','unknown',576,'31','','','','Test',1895,'',10 from dual union all
select 14190,'1966-S363','1966-07-11 19:22:00','military missile','failure',795,'1250','Mk 11 RV','FOT','SOLID GOLD','OT',1698,'',10 from dual union all
select 14191,'1966-S364','1966-07-11 16:00:00','military missile','unknown',932,'A3P-399','','','','OT',421,'SSBN 654',1000 from dual union all
select 14192,'1966-S365','1966-07-11 19:18:00','suborbital rocket','success',837,'NASA 14.96UA','Density profile','','','Aeronomy',428,'',202 from dual union all
select 14193,'1966-S366','1966-07-12 07:45:00','military missile','success',795,'653','Mk 11 RV','FOT','SUGAR CANE','OT',1705,'',900 from dual union all
select 14194,'1966-S367','1966-07-12 21:00:00','suborbital rocket','success',624,'NASA 8.12UA','EUV Airglow','','','Aeronomy',1821,'',1015 from dual union all
select 14195,'1966-S368','1966-07-13 08:45:00','military missile','failure',795,'764','Mk 11 RV','LGM-30B FOT-17','YOUNG LION','OT',1697,'',10 from dual union all
select 14196,'1966-S369','1966-07-13 19:44:15','suborbital rocket','success',1199,'ESRO S18/2','ESRO S18/2','','','Ionospheric',1343,'',249 from dual union all
select 14197,'1966-S37','1966-08-24 00:00:00','military missile','success',1449,'','8K84 UBP-24','','','OT',536,'',1000 from dual union all
select 14198,'1966-S370','1966-07-14 20:23:00','military missile','success',135,'C015','ABRES GG-1 (ARPA)','ABRES GG-1 (ARPA)','','Reentry Test',613,'',200 from dual union all
select 14199,'1966-S371','1966-07-14 00:00:00','suborbital rocket','success',1291,'','Life Sci (dog)','','','Biology',1364,'',100 from dual union all
select 142,'1944-S129','1944-09-21 07:11:00','military missile','success',8,'18824','Ma214','','','Training',661,'',90 from dual union all
select 1420,'1952-A87','1952-05-22 00:00:00','military missile','success',771,'GM-562','','','','Test',222,'',12 from dual union all
select 14200,'1966-S372','1966-07-14 15:15:00','suborbital rocket','success',837,'NASA 14.166II','','','','Ionos',1821,'',195 from dual union all
select 14201,'1966-S373','1966-07-16 05:32:00','suborbital rocket','success',30,'NASA 04.159GG','STRAP','','','UV Astron',1869,'',145 from dual union all
select 14202,'1966-S374','1966-07-17 00:56:00','suborbital rocket','success',837,'NASA 14.291CM','GCA Release 53 Na','','','Aeronomy',1821,'',192 from dual union all
select 14203,'1966-S375','1966-07-17 02:10:00','suborbital rocket','success',668,'K-9M-18','','','','Ionos/Solar/Solar UV',839,'',326 from dual union all
select 14204,'1966-S376','1966-07-17 05:00:00','suborbital rocket','success',837,'NASA 14.292CM','GCA Release 54 TMA','','','Aeronomy',1821,'',211 from dual union all
select 14205,'1966-S377','1966-07-17 06:45:00','suborbital rocket','success',837,'NASA 14.293CM','GCA Release 55 TMA','','','Aeronomy',1821,'',199 from dual union all
select 14206,'1966-S378','1966-07-17 08:25:00','suborbital rocket','success',837,'NASA 14.294CM','GCA Release 56 TMA','','','Aeronomy',1821,'',200 from dual union all
select 14207,'1966-S379','1966-07-17 09:08:00','suborbital rocket','success',837,'NASA 14.295CM','GCA Release 57 Na','','','Aeronomy',1821,'',208 from dual union all
select 14208,'1966-S38','1966-08-25 00:00:00','military missile','success',1449,'','8K84 UBP-25','','','OT',532,'',1000 from dual union all
select 14209,'1966-S380','1966-07-19 03:36:00','suborbital rocket','success',30,'NASA 04.144DG','NRL NB3.165','','','UV Astron',1869,'',134 from dual union all
select 1421,'1952-A88','1952-05-23 00:00:00','military missile','unknown',339,'21R','','','','Test',1895,'',30 from dual union all
select 14210,'1966-S381','1966-07-19 05:00:00','suborbital rocket','success',217,'AKF-3-16','','','','Ionos/Solar X',1315,'',150 from dual union all
select 14211,'1966-S382','1966-07-19 08:48:00','suborbital rocket','success',535,'HAD 187','Sphere','','','Aeron',1835,'',100 from dual union all
select 14212,'1966-S383','1966-07-20 19:50:00','suborbital rocket','success',837,'NASA 14.218GE','SPICE 1  (PCA)','','','Solar particles',428,'',161 from dual union all
select 14213,'1966-S384','1966-07-21 03:27:00','suborbital rocket','success',217,'AKF-3-15','','','','Ionos/CR',1315,'',150 from dual union all
select 14214,'1966-S385','1966-07-22 00:00:00','military missile','success',1420,'B-95','Mk 6 RV','ST','GIANT TRAIN','OT',1657,'',1300 from dual union all
select 14215,'1966-S386','1966-07-22 21:02:00','suborbital rocket','success',30,'CRL AF3.525','EUV Mon/RPA','','','Solar UV',1869,'',247 from dual union all
select 14216,'1966-S387','1966-07-23 06:35:00','sounding rocket','success',698,'L-3H-2','Gyro-plasma probe','','','Ionos/Aeron',834,'',1800 from dual union all
select 14217,'1966-S388','1966-07-25 16:00:00','military missile','unknown',932,'A3P-419','','','','OT',421,'SSBN 654',1000 from dual union all
select 14218,'1966-S389','1966-07-26 00:00:00','military missile','failure',796,'2045','Mk 11 RV','Force Mod','TATTERED COAT','Test',1703,'',900 from dual union all
select 14219,'1966-S39','1966-08-27 00:00:00','military missile','success',1449,'','8K84 UBP-26','','','OT',508,'',1000 from dual union all
select 1422,'1952-A89','1952-05-23 00:00:00','atmospheric rocket','unknown',576,'32','','','','Test',1895,'',10 from dual union all
select 14220,'1966-S390','1966-07-26 00:00:00','military missile','success',795,'1219','Mk 11 RV','FOT','RED MAN','OT',1702,'',900 from dual union all
select 14221,'1966-S392','1966-07-25 15:30:00','military missile','success',916,'AO-37','P-106 66 Summer 05','','','OT',204,'',180 from dual union all
select 14222,'1966-S393','1966-07-25 16:00:00','military missile','success',916,'AO-38','P-107 66 Summer 06','','','OT',204,'',180 from dual union all
select 14223,'1966-S394','1966-07-25 17:00:00','military missile','success',916,'AO-39','P-108 66 Summer 07','','','OT',204,'',180 from dual union all
select 14224,'1966-S395','1966-07-26 16:00:00','military missile','success',916,'AO-40','P-109 66 Summer 08','','','OT',204,'',180 from dual union all
select 14225,'1966-S396','1966-07-26 17:00:00','military missile','success',916,'','P-110 66 Summer 09','','','OT',204,'',180 from dual union all
select 14226,'1966-S397','1966-07-26 18:00:00','military missile','success',916,'','P-111 66 Summer 10','','','OT',204,'',180 from dual union all
select 14227,'1966-S398','1966-07-27 15:00:00','suborbital rocket','success',30,'NRL NE3.190','Heliograph XUV-H1','','','Solar EUV',1869,'',211 from dual;
insert into LAUNCH
select 14228,'1966-S399','1966-07-28 00:00:00','suborbital rocket','success',1291,'','Life Sci (dog)','','','Biology',1364,'',100 from dual union all
select 14229,'1966-S40','1966-08-30 00:00:00','military missile','success',1449,'','8K84 UBP-27','','','OT',526,'',1000 from dual union all
select 1423,'1952-A90','1952-05-28 00:00:00','atmospheric rocket','success',836,'1249-104','Nike 1249 R&D 95R','','','Test',1895,'',10 from dual union all
select 14230,'1966-S400','1966-07-28 00:51:00','suborbital rocket','success',1206,'SL88C','RAE Test 34','','','Test',1839,'',213 from dual union all
select 14231,'1966-S401','1966-07-28 02:36:00','suborbital rocket','success',864,'Sandia 152-54','LRL BOX-6','','','XR Astron',855,'',173 from dual union all
select 14232,'1966-S402','1966-07-28 19:00:00','suborbital rocket','success',549,'Sandia 152-44','LRL RCS-2 (Daisy 2)','','','Aeronomy',855,'',173 from dual union all
select 14233,'1966-S403','1966-08-01 00:00:00','suborbital rocket','success',547,'','','','','Aeron?',394,'',544 from dual union all
select 14234,'1966-S404','1966-08-04 09:30:00','military missile','success',837,'BMTS','STV SR-035','','','Target',437,'',120 from dual union all
select 14235,'1966-S405','1966-08-30 14:40:00','military missile','success',837,'BMTS','STV SR-036','','','Target',437,'',120 from dual union all
select 14236,'1966-S406','1966-08-01 00:00:00','military missile','success',837,'BMTS','Target','','','Target',437,'',120 from dual union all
select 14237,'1966-S407','1966-08-01 00:00:00','military missile','success',796,'2061','Mk 11 RV','DASO','STAR BRIGHT','OT',1710,'',900 from dual union all
select 14238,'1966-S408','1966-08-01 18:11:00','suborbital rocket','success',837,'ERDA 66-22','Sphere','','','Aeron',1895,'',200 from dual union all
select 14239,'1966-S409','1966-08-01 21:05:00','suborbital rocket','success',837,'ERDA 66-23','Sphere','','','Aeron',1895,'',200 from dual union all
select 1424,'1952-A91','1952-05-29 00:00:00','atmospheric rocket','unknown',576,'33','','','','Test',1895,'',10 from dual union all
select 14240,'1966-S41','1966-09-02 00:00:00','military missile','success',1449,'','8K84 UBP-28','','','OT',503,'',1000 from dual union all
select 14241,'1966-S410','1966-08-02 00:00:00','suborbital rocket','success',865,'Sandia 152-29','Scan Converter','','','Tech',855,'',303 from dual union all
select 14242,'1966-S411','1966-08-02 00:00:00','suborbital rocket','success',865,'Sandia 152-102','ACS','','','Tech',855,'',306 from dual union all
select 14243,'1966-S412','1966-08-02 00:00:00','suborbital rocket','success',1351,'','Flight test 2','','','Test',1415,'',443 from dual union all
select 14244,'1966-S413','1966-08-03 02:00:00','suborbital rocket','success',668,'K-9M-11','','','','Weather?',839,'',328 from dual union all
select 14245,'1966-S414','1966-08-04 00:00:00','suborbital rocket','success',864,'Sandia 152-19','Airglow','','','Aeron',855,'',159 from dual union all
select 14246,'1966-S415','1966-08-04 00:00:00','suborbital rocket','success',865,'Sandia 152-32','Scan Converter','','','Tech',855,'',300 from dual union all
select 14247,'1966-S416','1966-08-04 00:00:00','suborbital rocket','success',865,'Sandia 152-85','ACS','','','Tech',855,'',300 from dual union all
select 14248,'1966-S417','1966-08-05 00:00:00','military missile','success',795,'1377','Mk 11 RV','FOT','PLAY BUSTER','OT',1698,'',900 from dual union all
select 14249,'1966-S418','1966-08-01 07:00:00','suborbital rocket','success',837,'ERDA 66-24','Sphere','','','Aeron',1895,'',200 from dual union all
select 1425,'1952-A92','1952-06-03 00:00:00','atmospheric rocket','success',564,'','F12 Model','','','Test',1821,'',5 from dual union all
select 14250,'1966-S42','1966-09-06 00:00:00','military missile','success',1449,'','8K84 UBP-29','','','OT',596,'',1000 from dual union all
select 14251,'1966-S420','1966-08-01 09:06:00','suborbital rocket','success',837,'ERDA 66-25','Sphere','','','Aeron',1895,'',200 from dual union all
select 14252,'1966-S421','1966-08-01 12:00:00','suborbital rocket','success',837,'ERDA 66-26','Sphere','','','Aeron',1895,'',200 from dual union all
select 14253,'1966-S422','1966-08-07 07:00:00','suborbital rocket','success',837,'NASA 14.296GM','GSFC Grenade NA1','','','Aeronomy',1821,'',200 from dual union all
select 14254,'1966-S423','1966-08-07 07:05:00','suborbital rocket','success',839,'NASA 10.204GM','GSFC Grenade 120 N','','','Aeronomy',330,'',100 from dual union all
select 14255,'1966-S424','1966-08-07 08:45:00','suborbital rocket','success',837,'','ISRO 20.05 TERLS-46','Spurious?','','Ionos',1467,'',175 from dual union all
select 14256,'1966-S425','1966-08-07 09:04:00','suborbital rocket','success',839,'NASA 10.202GM','GSFC Grenade 121 C','','','Aeronomy',428,'',123 from dual union all
select 14257,'1966-S426','1966-08-07 09:48:51','suborbital rocket','success',837,'NASA 14.289UM','UM Pitot 22/Noctil.','','','Aeronomy',428,'',152 from dual union all
select 14258,'1966-S427','1966-08-07 17:35:00','suborbital rocket','success',839,'NASA 10.203GM','GSFC Grenade 122 C','','','Aeronomy',428,'',123 from dual union all
select 14259,'1966-S428','1966-08-07 20:46:00','suborbital rocket','success',839,'NASA 10.206GM','GSFC Grenade 123 W','','','Aeronomy',1821,'',100 from dual union all
select 1426,'1952-A93','1952-06-06 17:30:00','suborbital rocket','failure',1500,'8','Viking 8','','','Test',1889,'',6 from dual union all
select 14260,'1966-S429','1966-08-07 23:26:00','suborbital rocket','success',839,'NASA 10.205GM','GSFC Grenade 124 N','','','Aeronomy',330,'',100 from dual union all
select 14261,'1966-S43','1966-09-12 00:00:00','military missile','success',1449,'','8K84 UBP-30','','','OT',596,'',1000 from dual union all
select 14262,'1966-S430','1966-08-08 17:47:38','military missile','failure',162,'149F','MBRV No. 5778600','ABRES MBRV-1','BUSY RAMROD','Reentry Test',1681,'',200 from dual union all
select 14263,'1966-S431','1966-08-08 18:22:00','suborbital rocket','success',30,'CRL AE3.734','Solar min particles','','','Cosmic Rays',428,'',134 from dual union all
select 14264,'1966-S432','1966-08-09 02:23:00','military missile','success',982,'N 22602-10T','8K64U UBP-80','','','OT',572,'',1210 from dual union all
select 14265,'1966-S433','1966-08-10 12:37:00','suborbital rocket','success',668,'K-9M-19','VLF Doppler','','','Ionos/Aeron',839,'',330 from dual union all
select 14266,'1966-S434','1966-08-11 00:00:00','suborbital rocket','success',865,'Sandia 152-86','ACS','','','Tech',855,'',300 from dual union all
select 14267,'1966-S435','1966-08-11 12:05:00','suborbital rocket','success',666,'K-8L-10','','','','Aeron',839,'',130 from dual union all
select 14268,'1966-S436','1966-08-11 15:53:00','suborbital rocket','success',30,'CRL AC3.364','Auroral emissions','','','Auroral',428,'',207 from dual union all
select 14269,'1966-S437','1966-08-13 00:00:00','suborbital rocket','success',894,'No. 1','','','','Test',310,'',114 from dual union all
select 1427,'1952-A94','1952-06-12 00:00:00','atmospheric rocket','success',836,'1249-105 XM5 test','Nike 1249 R&D 96R','','','Test',1895,'',10 from dual union all
select 14270,'1966-S438','1966-08-14 20:35:00','suborbital rocket','success',839,'NASA 10.200GM','GSFC Grenade 125 PB','','','Aeronomy',1039,'',100 from dual union all
select 14271,'1966-S439','1966-08-15 08:08:00','suborbital rocket','success',839,'NASA 10.201GM','GSFC Grenade 126 PB','','','Aeronomy',1039,'',100 from dual union all
select 14272,'1966-S44','1966-09-16 00:00:00','military missile','success',1449,'','8K84 UBP-31','','','OT',596,'',1000 from dual union all
select 14273,'1966-S440','1966-08-15 18:28:00','suborbital rocket','success',624,'NASA 8.32DA','Composition','','','Aeronomy',1821,'',698 from dual union all
select 14274,'1966-S441','1966-08-15 21:35:00','suborbital rocket','success',837,'NASA 14.170UA','Pandora 2','','','Aeronomy/Meteorites',1895,'',160 from dual union all
select 14275,'1966-S442','1966-08-16 00:00:00','military missile','failure',794,'522 (63-021)','Mk 5 RV','FOT','GIBSON GIRL','OT',1701,'',10 from dual union all
select 14276,'1966-S443','1966-08-16 00:00:00','military missile','success',795,'1342','Mk 11 RV','FOT+-3h','WHITE ARC','OT',1697,'',900 from dual union all
select 14277,'1966-S444','1966-08-16 06:18:00','suborbital rocket','success',1124,'','Scanner 1','','','Aeronomy',1821,'',612 from dual union all
select 14278,'1966-S445','1966-08-17 03:39:00','suborbital rocket','success',227,'ALF-VB-08','MPE Ba release','','','Aeron Ba',428,'',390 from dual union all
select 14279,'1966-S446','1966-08-17 19:50:00','suborbital rocket','success',44,'NASA 17.02GT-GI-GS','','','','Test/Iono-solar',1821,'',358 from dual union all
select 1428,'1952-A95','1952-06-13 00:00:00','atmospheric rocket','success',442,'','X-3 Model','','','Test',1821,'',10 from dual union all
select 14280,'1966-S447','1966-08-18 00:00:00','military missile','success',135,'B011','WIRE Ionizing Pellets','ABRES N-6 (USN)','','Reentry Test',612,'',200 from dual union all
select 14281,'1966-S448','1966-08-19 00:00:00','military missile','success',135,'E001','FATE IO-1','ABRES I-1 (USAF)','','Reentry Test',614,'',200 from dual union all
select 14282,'1966-S449','1966-08-19 07:44:00','suborbital rocket','success',227,'ALF-VB-09','MPE Ba release','','','Aeron Ba',428,'',391 from dual union all
select 14283,'1966-S45','1966-10-01 00:00:00','military missile','success',1449,'','8K84 UBP-32','','','OT',596,'',1000 from dual union all
select 14284,'1966-S450','1966-08-22 20:52:00','military missile','success',794,'621 (63-103)','Mk 5 RV','FOT','TOWN DOCTOR','OT',1699,'',900 from dual union all
select 14285,'1966-S452','1966-08-24 21:00:00','suborbital rocket','success',837,'NASA 14.183UE','UNH neutrons','','','Plasma',428,'',157 from dual union all
select 14286,'1966-S453','1966-08-24 23:15:00','suborbital rocket','success',837,'NASA 14.271UI','','','','Ionos',1821,'',164 from dual union all
select 14287,'1966-S454','1966-08-25 05:48:00','military missile','success',982,'G 22603-13T','8K64U UBP-81','','','OT',566,'',1210 from dual union all
select 14288,'1966-S455','1966-08-25 00:45:00','suborbital rocket','success',837,'NASA 14.272UI','','','','Ionos',1821,'',169 from dual union all
select 14289,'1966-S456','1966-08-26 21:36:00','military missile','success',796,'2056','Mk 11 RV','DASO','CAREER GIRL','OT',1709,'',900 from dual union all
select 1429,'1952-A96','1952-06-17 00:00:00','atmospheric rocket','unknown',576,'36','','','','Test',1895,'',10 from dual union all
select 14290,'1966-S457','1966-08-26 18:13:00','suborbital rocket','success',35,'NASA 04.101DS','EUV Mon/RPA','','','Solar',1821,'',212 from dual union all
select 14291,'1966-S458','1966-08-26 18:31:00','suborbital rocket','success',867,'NASA 18.05GA','UM Thermosphere 4','','','Aeronomy',1821,'',287 from dual union all
select 14292,'1966-S459','1966-08-26 18:51:00','suborbital rocket','success',867,'NASA 18.06GA','UM Thermosphere 5','','','Aeronomy',1821,'',308 from dual union all
select 14293,'1966-S46','1966-10-05 00:00:00','military missile','success',1449,'','8K84 UBP-33','','','OT',596,'',1000 from dual union all
select 14294,'1966-S460','1966-08-26 19:11:00','suborbital rocket','success',837,'NASA 14.285UM','UM Pitot 23/Thermo','','','Aeronomy',1821,'',149 from dual union all
select 14295,'1966-S461','1966-08-26 20:30:00','suborbital rocket','success',217,'AAF-3-13','','','','Test/Aeron',428,'',114 from dual union all
select 14296,'1966-S462','1966-08-28 04:23:00','suborbital rocket','success',837,'NASA 14.286UM','UM Pitot 24/Thermo','','','Aeronomy',1821,'',151 from dual union all
select 14297,'1966-S463','1966-08-28 04:03:00','suborbital rocket','success',867,'NASA 18.22GA','UM Thermosphere 6','','','Aeronomy',1821,'',323 from dual union all
select 14298,'1966-S464','1966-08-23 09:04:00','suborbital rocket','success',535,'HAD 189','Sphere','','','Aeron',1835,'',100 from dual union all
select 14299,'1966-S465','1966-08-29 19:05:00','military missile','success',795,'1197','Mk 11 RV','FOT','MOTHER CAT','OT',1702,'',900 from dual union all
select 143,'1944-S13','1944-03-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1430,'1952-A97','1952-06-17 00:00:00','atmospheric rocket','unknown',576,'34','','','','Test',1895,'',10 from dual union all
select 14300,'1966-S466','1966-08-29 15:20:00','suborbital rocket','success',30,'NASA 04.156GM','Nimbus MUSE test','','','Aeronomy/Solar UV',1869,'',206 from dual union all
select 14301,'1966-S467','1966-08-30 10:05:00','suborbital rocket','success',535,'HAD 312','Lunar UV Ozone','','','Aeron',1835,'',91 from dual union all
select 14302,'1966-S468','1966-09-01 00:00:00','suborbital rocket','success',894,'No. 4','','','','Test',310,'',90 from dual union all
select 14303,'1966-S469','1966-09-09 15:00:00','military missile','success',837,'BMTS','SPEEDBALL II 88 SN34','','','Target',437,'',150 from dual union all
select 14304,'1966-S47','1966-10-07 00:00:00','military missile','success',1449,'','8K84 UBP-34','','','OT',503,'',1000 from dual union all
select 14305,'1966-S470','1966-09-01 00:00:00','military missile','success',837,'BMTS','Target','','','Target',437,'',120 from dual union all
select 14306,'1966-S471','1966-09-01 00:00:00','military missile','success',837,'BMRS','','','','Target',1895,'',100 from dual union all
select 14307,'1966-S472','1966-09-01 00:00:00','military missile','success',837,'BMRS','','','','Target',1895,'',100 from dual union all
select 14308,'1966-S473','1966-09-01 00:00:00','military missile','success',837,'BMRS','','','','Target',1895,'',100 from dual union all
select 14309,'1966-S474','1966-09-01 00:00:00','suborbital rocket','success',1272,'','HALBERD RV','','','Reentry Test',855,'',200 from dual union all
select 1431,'1952-A98','1952-06-17 00:00:00','atmospheric rocket','unknown',576,'35','','','','Test',1895,'',10 from dual union all
select 14310,'1966-S475','1966-09-01 17:00:00','military missile','unknown',932,'A3P-386','','','','OT',421,'SSBN 656',1000 from dual union all
select 14311,'1966-S476','1966-09-01 00:00:00','military missile','success',135,'D029','ECM Decoy F-7','ABRES F-7 (USAF)','','Reentry Test',613,'',200 from dual union all
select 14312,'1966-S477','1966-09-01 18:58:00','suborbital rocket','success',867,'NASA 18.18UE','Auroral radiation','','','Plasma',428,'',242 from dual union all
select 14313,'1966-S478','1966-09-02 00:00:00','military missile','success',135,'D025','ABRES QQ-2 (USAF)','ABRES QQ-2 (USAF)','','Reentry Test',612,'',200 from dual union all
select 14314,'1966-S479','1966-09-02 14:37:00','suborbital rocket','success',837,'NASA 14.219GE','SPICE 2A','','','Solar particles',428,'',164 from dual union all
select 14315,'1966-S48','1966-10-21 00:00:00','military missile','success',1449,'','8K84 UBP-35','Pal''ma-2','','OT',504,'',1000 from dual union all
select 14316,'1966-S480','1966-09-02 22:27:00','suborbital rocket','success',837,'NASA 14.220GE','SPICE 2','','','Solar particles',428,'',159 from dual union all
select 14317,'1966-S481','1966-09-02 22:50:00','suborbital rocket','success',30,'NRL NA3.182','NRL NA3.182','','','Solar EUV',1869,'',206 from dual;
insert into LAUNCH
select 14318,'1966-S482','1966-09-03 17:24:00','suborbital rocket','success',837,'NASA 14.221GE','SPICE 3','','','Solar particles',428,'',160 from dual union all
select 14319,'1966-S483','1966-09-06 12:27:00','suborbital rocket','success',867,'NASA 18.20UE','Auroral radiation','','','Plasma',428,'',238 from dual union all
select 1432,'1952-A99','1952-06-18 00:00:00','atmospheric rocket','success',836,'1249-106 XM5 test','Nike 1249 R&D 97R','','','Test',1895,'',10 from dual union all
select 14320,'1966-S484','1966-09-06 18:02:00','suborbital rocket','success',867,'NASA 18.21UE','Auroral radiation','','','Plasma',428,'',243 from dual union all
select 14321,'1966-S485','1966-09-14 01:35:22','suborbital rocket','success',837,'NASA 14.278CA','GCA Ionos-TMA 9','','','Aeronomy',428,'',157 from dual union all
select 14322,'1966-S486','1966-09-14 04:23:00','suborbital rocket','success',837,'NASA 14.279CA','GCA Ionos-TMA 10','','','Aeronomy',428,'',186 from dual union all
select 14323,'1966-S487','1966-09-16 00:00:00','military missile','failure',795,'1291','Mk 11 RV','FOT','SNOW HILL','OT',1698,'',100 from dual union all
select 14324,'1966-S488','1966-09-16 00:00:00','military missile','success',1420,'B-40','Mk 6 RV','FOT/NTMP KX-53','BLACK RIVER','OT',1658,'',1300 from dual union all
select 14325,'1966-S489','1966-09-16 01:22:00','suborbital rocket','success',837,'NASA 14.280CA-CI','GCA Ionos-TMA 11','','','Ionos',428,'',185 from dual union all
select 14326,'1966-S49','1966-11-17 00:00:00','military missile','success',1449,'','8K84 UBP-36','','','OT',596,'',1000 from dual union all
select 14327,'1966-S490','1966-09-16 03:00:00','suborbital rocket','success',837,'NASA 14.281CA-CI','GCA Ionos-TMA 12','','','Ionos',428,'',185 from dual union all
select 14328,'1966-S491','1966-09-16 04:17:00','suborbital rocket','success',837,'NASA 14.282CA-CI','GCA Ionos-TMA 13','','','Ionos',428,'',206 from dual union all
select 14329,'1966-S492','1966-09-16 15:54:00','suborbital rocket','success',837,'NASA 14.164UI','','','','Ionos',1821,'',166 from dual union all
select 1433,'1952-M01','1952-05-15 01:15:00','suborbital rocket','success',59,'SC 23','Sphere','','','Aeronomy',1869,'',76 from dual union all
select 14330,'1966-S493','1966-09-17 02:53:00','suborbital rocket','success',867,'NASA 18.19UE','Auroral radiation','','','Plasma',428,'',243 from dual union all
select 14331,'1966-S494','1966-09-19 00:00:00','suborbital rocket','success',865,'Sandia 152-25','Sampler (Camera)','','','Aeron',803,'',300 from dual union all
select 14332,'1966-S495','1966-09-20 00:00:00','military missile','success',794,'571 (63-068)','Mk 5 RV','FOT','RED SPIDER','OT',1699,'',900 from dual union all
select 14333,'1966-S496','1966-09-20 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 624',1000 from dual union all
select 14334,'1966-S497','1966-09-20 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 624',1000 from dual union all
select 14335,'1966-S498','1966-09-20 05:45:00','suborbital rocket','success',30,'NASA 04.176UG','','','','UV Astron',1869,'',165 from dual union all
select 14336,'1966-S499','1966-09-20 15:34:00','suborbital rocket','success',864,'Sandia 152-99','LRL BOX-7','','','XR Astron',803,'',168 from dual union all
select 14337,'1966-S50','1966-11-19 00:00:00','military missile','success',1449,'','8K84 UBP-37','','','OT',596,'',1000 from dual union all
select 14338,'1966-S500','1966-09-22 14:30:00','military missile','success',795,'1262','Mk 11 RV','FOT/NTMP KX-66','HOT SPRINGS','OT',1705,'',900 from dual union all
select 14339,'1966-S501','1966-09-22 00:00:00','military missile','success',135,'B015','SFT-1','ABRES RR-1 (USN)','','Reentry Test',614,'',200 from dual union all
select 1434,'1952-M02','1952-05-21 15:15:00','suborbital rocket','success',50,'USAF 26','Aeromed 3','','','Biology',640,'',61 from dual union all
select 14340,'1966-S502','1966-09-22 17:10:00','suborbital rocket','success',864,'Sandia 152-98','LRL AXE-1','','','XR Astron',803,'',160 from dual union all
select 14341,'1966-S503','1966-09-24 09:13:00','suborbital rocket','success',624,'NASA 8.27IA','MPE Ba','','','Aeronomy',1821,'',930 from dual union all
select 14342,'1966-S504','1966-09-24 19:47:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',144 from dual union all
select 14343,'1966-S505','1966-09-25 09:51:00','suborbital rocket','success',867,'NASA 18.26IA','MPE Ba','','','Aeronomy',1821,'',280 from dual union all
select 14344,'1966-S506','1966-09-27 05:30:00','military missile','success',982,'G 22602-07T','8K64U UBP-83','','','OT',566,'',1210 from dual union all
select 14345,'1966-S507','1966-09-27 09:22:00','suborbital rocket','success',535,'HAD 190','Sphere','','','Aeron',1835,'',100 from dual union all
select 14346,'1966-S508','1966-09-28 00:00:00','military missile','unknown',929,'A2P','FOT','','','OT',421,'SSBN 619',1000 from dual union all
select 14347,'1966-S509','1966-09-28 00:00:00','military missile','unknown',929,'A2P','FOT','','','OT',421,'SSBN 619',1000 from dual union all
select 14348,'1966-S51','1966-11-22 00:00:00','military missile','success',1449,'','8K84 UBP-38','','','OT',596,'',1000 from dual union all
select 14349,'1966-S510','1966-09-28 18:16:00','suborbital rocket','success',215,'CRL AC17.604D','','','','Ionos',428,'',116 from dual union all
select 1435,'1952-M03','1952-05-22 00:00:00','suborbital rocket','success',1490,'N3','','','','Test',666,'',60 from dual union all
select 14350,'1966-S511','1966-10-05 20:04:00','military missile','success',837,'BMRS','SPEEDBALL II 90 SN36','','','Target',1895,'',150 from dual union all
select 14351,'1966-S512','1966-10-12 16:48:00','military missile','success',837,'BMRS','SPEEDBALL II 91 SN37','','','Target',1895,'',150 from dual union all
select 14352,'1966-S513','1966-10-26 00:00:00','military missile','success',916,'AO-23','P-115 66 Fall 04','','','OT',597,'',180 from dual union all
select 14353,'1966-S514','1966-09-30 09:07:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',174 from dual union all
select 14354,'1966-S515','1966-09-30 17:35:00','suborbital rocket','success',839,'NASA 10.146GM','GSFC Grenade 127 W','','','Aeronomy',1821,'',118 from dual union all
select 14355,'1966-S516','1966-09-30 23:50:00','suborbital rocket','success',837,'NASA 14.217GM','GSFC Grenade NA2','','','Aeronomy',1821,'',153 from dual union all
select 14356,'1966-S517','1966-10-01 06:44:00','military missile','success',982,'G 22603-09T','8K64U UBP-84','','','OT',566,'',1210 from dual union all
select 14357,'1966-S518','1966-10-01 05:30:00','suborbital rocket','success',839,'NASA 10.209GM','GSFC Grenade 128 W','','','Aeronomy',1821,'',115 from dual union all
select 14358,'1966-S519','1966-10-01 08:23:00','suborbital rocket','success',839,'NASA 10.211GM','GSFC Grenade 129 N','','','Aeronomy',330,'',100 from dual union all
select 14359,'1966-S52','1966-12-01 00:00:00','military missile','success',1449,'','8K84 UBP-39','','','OT',504,'',1000 from dual union all
select 1436,'1952-M04','1952-08-22 07:33:00','suborbital rocket','success',1470,'TF3','','','','Test/Photo/Solar X/Aeron',1863,'',79 from dual union all
select 14360,'1966-S520','1966-10-01 09:07:00','suborbital rocket','success',740,'','Thermosphere comp. 1','','','Aeron',1010,'',174 from dual union all
select 14361,'1966-S521','1966-10-01 11:27:00','suborbital rocket','success',839,'NASA 10.210GM','GSFC Grenade 130 W','','','Aeronomy',1821,'',126 from dual union all
select 14362,'1966-S522','1966-10-01 20:39:00','suborbital rocket','success',839,'NASA 10.212GM','GSFC Grenade 131 N','','','Aeronomy',330,'',100 from dual union all
select 14363,'1966-S523','1966-10-02 02:20:00','suborbital rocket','success',839,'NASA 10.213GM','GSFC Grenade 132 N','','','Aeronomy',330,'',100 from dual union all
select 14364,'1966-S524','1966-10-02 08:20:00','suborbital rocket','success',839,'NASA 10.214GM','GSFC Grenade 133 N','','','Aeronomy',330,'',100 from dual union all
select 14365,'1966-S525','1966-10-02 14:09:00','suborbital rocket','success',839,'NASA 10.215GM','GSFC Grenade 134 N','','','Aeronomy',330,'',100 from dual union all
select 14366,'1966-S526','1966-10-03 00:00:00','military missile','success',794,'578','Mk 5 RV','FOT','GROVE HILL','OT',1701,'',900 from dual union all
select 14367,'1966-S527','1966-10-03 00:00:00','suborbital rocket','success',1492,'VA77','FU-145 Ionos.','','','Mag/Ionos/Astron Zodi?',668,'',166 from dual union all
select 14368,'1966-S528','1966-10-04 20:29:00','suborbital rocket','success',30,'NRL NA3.160','KP3.18','','','Solar X/Test',1869,'',154 from dual union all
select 14369,'1966-S529','1966-10-05 00:00:00','military missile','success',1117,'R3','','','','Reentry Test',669,'',1000 from dual union all
select 1437,'1952-M05','1952-08-29 00:26:00','suborbital rocket','success',370,'','SUI 3','','','Test',170,'WYG-279',61 from dual union all
select 14370,'1966-S53','1966-12-08 00:00:00','military missile','success',1449,'','8K84 UBP-40','','','OT',508,'',1000 from dual union all
select 14371,'1966-S530','1966-10-06 02:42:00','military missile','success',999,'N 22505-10T','8K67 UBP-4','','','OT',485,'',1000 from dual union all
select 14372,'1966-S531','1966-10-06 00:00:00','military missile','success',135,'D024','SFT-2','ABRES RR-2 (USN)','','Reentry Test',614,'',200 from dual union all
select 14373,'1966-S532','1966-10-06 20:29:00','suborbital rocket','success',624,'NASA 8.38GI','','','','Ionos',1821,'',650 from dual union all
select 14374,'1966-S533','1966-10-07 00:00:00','military missile','success',135,'D049','ABRES WW-1 (USAF)','ABRES WW-1 (USAF)','','Reentry Test',613,'',200 from dual union all
select 14375,'1966-S534','1966-10-08 00:00:00','suborbital rocket','success',288,'C140','CNES FU-109','','','Test',664,'',170 from dual union all
select 14376,'1966-S535','1966-10-10 16:10:03','military missile','unknown',932,'A3P-447','','','','OT',421,'SSBN 601',1000 from dual union all
select 14377,'1966-S536','1966-10-10 07:15:00','suborbital rocket','success',740,'','','','','Ionos/Solar UV/Sola',636,'',200 from dual union all
select 14378,'1966-S537','1966-10-10 16:57:00','suborbital rocket','success',848,'CRL AF07.348','','','','Aeron',394,'',179 from dual union all
select 14379,'1966-S538','1966-10-11 23:22:00','suborbital rocket','success',30,'KP3.16','IR dayglow','','','Aeron',1869,'',150 from dual union all
select 1438,'1952-M06','1952-08-29 07:36:00','suborbital rocket','success',370,'','SUI 4','','','CR',171,'WYG-279',60 from dual union all
select 14380,'1966-S539','1966-10-12 01:52:00','suborbital rocket','success',30,'NASA 04.149CG','AS&E','','','XR Astron',1869,'',148 from dual union all
select 14381,'1966-S54','1966-12-12 00:00:00','military missile','success',1449,'','8K84 UBP-41','','','OT',505,'',1000 from dual union all
select 14382,'1966-S540','1966-10-13 22:10:00','military missile','success',795,'1247','Mk 11 RV','FOT','SEA RAVEN','OT',1698,'',900 from dual union all
select 14383,'1966-S541','1966-10-01 00:00:00','suborbital rocket','success',1,'','Yantar''-1','','','Tech/Ionos',636,'',400 from dual union all
select 14384,'1966-S542','1966-10-13 00:00:00','military missile','success',135,'D035','RVTO','ABRES SS-2 (USAF)','','Reentry Test',612,'',200 from dual union all
select 14385,'1966-S543','1966-10-13 00:00:00','military missile','success',135,'D036','RVTO','ABRES SS-3 (USAF)','','Reentry Test',614,'',200 from dual union all
select 14386,'1966-S544','1966-10-14 17:20:00','suborbital rocket','success',848,'CRL AE07.813','','','','Ionos',394,'',138 from dual union all
select 14387,'1966-S545','1966-10-17 22:51:24','suborbital rocket','success',446,'D-28','FU-165 Ions/E Field','','','Ionos',29,'',387 from dual union all
select 14388,'1966-S546','1966-10-18 09:17:00','suborbital rocket','success',666,'K-8L-11','','','','Aeron',839,'',150 from dual union all
select 14389,'1966-S547','1966-10-20 05:47:00','military missile','success',982,'G 22604-10T','8K64U UBP-85','Pal''ma-2','','OT',572,'',1210 from dual union all
select 1439,'1952-M07','1952-08-31 21:10:00','suborbital rocket','success',370,'','SUI 6','','','CR',173,'WYG-279',60 from dual union all
select 14390,'1966-S548','1966-10-20 06:35:00','military missile','success',999,'N 22500-02T','8K67 UBP-5','Pal''ma-2','','OT',485,'',1000 from dual union all
select 14391,'1966-S549','1966-10-20 02:20:00','suborbital rocket','success',665,'K-8-14','','','','Ionos/Fields/Solar UV',839,'',191 from dual union all
select 14392,'1966-S55','1966-12-16 00:00:00','military missile','success',1449,'','8K84 UBP-42','','','OT',504,'',1000 from dual union all
select 14393,'1966-S550','1966-10-20 08:25:00','suborbital rocket','success',668,'K-9M-20','Gyro-plasma probe','','','Ionos',839,'',353 from dual union all
select 14394,'1966-S551','1966-10-20 12:38:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',186 from dual union all
select 14395,'1966-S552','1966-10-21 20:35:00','military missile','success',795,'914','Mk 11 RV','FOT','MATCHED CLUBS','OT',1702,'',900 from dual union all
select 14396,'1966-S553','1966-10-21 00:00:00','military missile','success',916,'AO-41','P-112 66 Fall 01','','','OT',597,'',180 from dual union all
select 14397,'1966-S554','1966-10-21 00:00:00','military missile','success',916,'AO-42','P-113 66 Fall 02','','','OT',597,'',180 from dual union all
select 14398,'1966-S555','1966-10-21 00:00:00','military missile','success',916,'AO-43','P-114 66 Fall 03','','','OT',597,'',180 from dual union all
select 14399,'1966-S556','1966-10-22 04:33:00','military missile','success',982,'N 22601-24T','8K64U UBP-86','Pal''ma-2','','OT',570,'',1210 from dual union all
select 144,'1944-S130','1944-09-21 12:11:00','military missile','success',8,'18822','Ma211','','','Training',661,'',90 from dual union all
select 1440,'1952-M08','1952-09-04 09:17:00','suborbital rocket','success',370,'','SUI 7','','','CR',174,'WYG-279',60 from dual union all
select 14400,'1966-S557','1966-10-22 15:20:00','suborbital rocket','success',30,'NASA 04.161NA','Luster 1966','','','Meteorites',1869,'',149 from dual union all
select 14401,'1966-S558','1966-10-23 02:28:00','military missile','success',999,'N 22506-10T','8K67 UBP-6','','','OT',514,'',1000 from dual union all
select 14402,'1966-S559','1966-10-25 09:06:00','suborbital rocket','success',1199,'SL321','Bir-11','','','Ionospheric',1839,'',211 from dual union all
select 14403,'1966-S56','1966-12-26 00:00:00','military missile','success',1449,'','8K84 UBP-43','','','OT',505,'',1000 from dual union all
select 14404,'1966-S560','1966-10-25 11:11:00','suborbital rocket','success',1200,'SL322','Bir-12','','','Ionospheric',1839,'',119 from dual union all
select 14405,'1966-S561','1966-10-25 17:25:00','suborbital rocket','success',30,'NASA 04.195GA-GI','Meteoroids','','','Meteorites',1869,'',186 from dual union all
select 14406,'1966-S562','1966-10-25 18:10:00','suborbital rocket','success',839,'NASA 10.181AI','D-Region Probe','','','Ionos',1895,'',119 from dual union all
select 14407,'1966-S563','1966-10-26 00:00:00','suborbital rocket','success',1440,'','AFAL 2  Blackout R&D','','','Reentry Test',1821,'',300 from dual;
insert into LAUNCH
select 14408,'1966-S564','1966-10-26 14:16:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',176 from dual union all
select 14409,'1966-S565','1966-10-27 00:00:00','military missile','success',1449,'','8K84 LKI-24 (SLI)','','','OT',596,'',1000 from dual union all
select 1441,'1952-M09','1952-11-06 15:56:00','suborbital rocket','success',50,'USAF 32','AF Airglow 2','','','Aeronomy SB',640,'',76 from dual union all
select 14410,'1966-S566','1966-10-27 00:00:00','military missile','success',355,'','Nuclear test','','','Nuclear Test',811,'',200 from dual union all
select 14411,'1966-S567','1966-10-27 08:47:00','suborbital rocket','success',1199,'SL325','Bir-13','','','Ionospheric',1839,'',173 from dual union all
select 14412,'1966-S568','1966-10-27 10:25:00','suborbital rocket','success',1204,'SL323','Bir-14','','','Ionospheric',1839,'',212 from dual union all
select 14413,'1966-S569','1966-10-27 13:35:00','suborbital rocket','success',1199,'SL324','Bir-15','','','Ionospheric',1839,'',200 from dual union all
select 14414,'1966-S57','1966-12-27 00:00:00','military missile','success',1449,'','8K84 UBP-44 OT','','','OT',504,'',1000 from dual union all
select 14415,'1966-S570','1966-10-28 00:00:00','military missile','success',1117,'G3','','','','Reentry Test',669,'',1000 from dual union all
select 14416,'1966-S571','1966-10-28 04:42:00','military missile','success',982,'G 22604-09T','8K64U UBP-87','','','OT',566,'',1210 from dual union all
select 14417,'1966-S572','1966-11-01 00:00:00','military missile','success',837,'BMRS','','','','Target',1895,'',100 from dual union all
select 14418,'1966-S573','1966-11-04 00:00:00','military missile','failure',916,'AO-44','P-116 66 Fall 05','','','OT',597,'',2 from dual union all
select 14419,'1966-S574','1966-10-31 16:15:00','military missile','unknown',932,'A3P-397','','','','OT',421,'SSBN 599',1000 from dual union all
select 1442,'1952-S01','1952-02-19 17:00:00','suborbital rocket','success',54,'NRL 7','Sunfollower/SX','','','Solar/CR',1869,'',81 from dual union all
select 14420,'1966-S575','1966-10-31 16:45:00','military missile','unknown',932,'A3P-450','','','','OT',421,'SSBN 599',1000 from dual union all
select 14421,'1966-S576','1966-11-01 00:00:00','suborbital rocket','success',30,'','()','','','Solar EUV',1869,'',241 from dual union all
select 14422,'1966-S577','1966-11-01 04:34:00','military missile','success',982,'G 22603-14T','8K64U UBP-88','','','OT',566,'',1210 from dual union all
select 14423,'1966-S578','1966-11-01 09:48:00','suborbital rocket','success',535,'HAD 191','Sphere','','','Aeron',1835,'',100 from dual union all
select 14424,'1966-S579','1966-11-01 21:24:12','suborbital spaceplane','success',1520,'3-56-83','MuMet/DCR/SB','','','Meteor/Aeron',379,'NB-52 003',93 from dual union all
select 14425,'1966-S58','1966-01-05 00:30:01','military missile','success',796,'461','Mk 11 RV','R&D','','Test',266,'',900 from dual union all
select 14426,'1966-S580','1966-11-02 00:00:00','military missile','success',1117,'G4','','','','Reentry Test',669,'',1000 from dual union all
select 14427,'1966-S581','1966-11-02 00:00:00','military missile','failure',796,'2068','Mk 11 RV','DASO','GOLDEN AGE','OT',1708,'',10 from dual union all
select 14428,'1966-S582','1966-11-04 00:00:00','suborbital rocket','success',894,'','','','','Test',1821,'',90 from dual union all
select 14429,'1966-S583','1966-11-04 03:49:00','military missile','success',982,'N 22601-06T','8K64U UBP-89','','','OT',566,'',1210 from dual union all
select 1443,'1952-S02','1952-02-29 14:40:00','suborbital rocket','success',50,'USAF 23','Airglow','','','Aeronomy SB',640,'',88 from dual union all
select 14430,'1966-S584','1966-11-04 02:05:00','suborbital rocket','success',1072,'PT-300-1','','','','Test',839,'',160 from dual union all
select 14431,'1966-S585','1966-11-04 08:00:00','military missile','success',1037,'N10300-18L','8K98 LKI-2-1','','','Test',463,'',1000 from dual union all
select 14432,'1966-S586','1966-11-04 15:16:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',164 from dual union all
select 14433,'1966-S587','1966-11-05 14:55:21','suborbital rocket','success',846,'Cert. Round','','','','Test',221,'',96 from dual union all
select 14434,'1966-S588','1966-11-07 00:00:00','military missile','success',135,'D041','RVTO','ABRES SS-4 (USAF)','','Reentry Test',614,'',200 from dual union all
select 14435,'1966-S589','1966-11-07 00:00:00','military missile','success',135,'D039','SFT-3','ABRES RR-3 (USN)','','Reentry Test',612,'',200 from dual union all
select 14436,'1966-S59','1966-01-08 13:11:00','suborbital rocket','success',288,'C103','ISRO 15.03','TERLS-035','','Aeron/Ionos',1467,'',150 from dual union all
select 14437,'1966-S590','1966-11-08 17:32:00','suborbital rocket','success',867,'NASA 18.04GE','Fields','','','Plasma',1821,'',216 from dual union all
select 14438,'1966-S591','1966-11-09 17:43:00','suborbital rocket','success',894,'','','','','Test',1821,'',82 from dual union all
select 14439,'1966-S592','1966-11-11 05:07:00','military missile','success',135,'B017','FATE','ABRES I-2 (USAF)','','Reentry Test',613,'',200 from dual union all
select 1444,'1952-S03','1952-04-22 17:28:00','suborbital rocket','success',50,'USAF 24','Ionos Propagation','','','Ionos',640,'',116 from dual union all
select 14440,'1966-S593','1966-11-11 02:22:00','military missile','success',135,'D020','SFT-4','ABRES RR-4 (USN)','','Reentry Test',612,'',200 from dual union all
select 14441,'1966-S594','1966-11-10 23:05:00','suborbital rocket','success',719,'LT16','Solar UV','','','Solar UV',1837,'',100 from dual union all
select 14442,'1966-S595','1966-11-11 07:06:00','military missile','success',135,'D047','ABRES WW-2 (USAF)','ABRES WW-2 (USAF)','','Reentry Test',614,'',200 from dual union all
select 14443,'1966-S596','1966-11-11 04:33:00','suborbital rocket','success',837,'NASA 14.287IE','Sat payload test','','','Plasma',428,'',193 from dual union all
select 14444,'1966-S597','1966-11-12 00:00:00','suborbital rocket','success',894,'','','','','Eclipse',1460,'',90 from dual union all
select 14445,'1966-S598','1966-11-12 00:00:00','suborbital rocket','success',894,'','','','','Eclipse',1460,'',90 from dual union all
select 14446,'1966-S599','1966-11-12 00:00:00','suborbital rocket','success',894,'','','','','Eclipse',1460,'',90 from dual union all
select 14447,'1966-S60','1966-01-10 17:14:00','suborbital rocket','success',837,'NASA 14.248UI','UIUC-GCA D/E-7','','','Ionos',1821,'',186 from dual union all
select 14448,'1966-S600','1966-11-12 13:40:27','suborbital rocket','success',1428,'001','FU-150 Alpha','','','Solar Eclipse',314,'',274 from dual union all
select 14449,'1966-S601','1966-11-12 16:40:00','suborbital rocket','success',30,'NASA 04.191DS','NRL ND3.188 XUV-H2','','','Solar EUV/eclipse',1869,'',174 from dual union all
select 1445,'1952-S04','1952-04-30 13:30:00','suborbital rocket','success',54,'NRL 8','UV detectors','','','Solar UV/CR',1869,'',128 from dual union all
select 14450,'1966-S602','1966-11-12 12:00:00','suborbital rocket','success',865,'Sandia 152-93','','','','Solar X/Eclipse',221,'',297 from dual union all
select 14451,'1966-S603','1966-11-12 13:40:57','suborbital rocket','success',1428,'002','FU-150 Bravo','','','Solar Eclipse',314,'',270 from dual union all
select 14452,'1966-S604','1966-11-12 14:07:00','suborbital rocket','success',865,'Sandia 152-94','','','','Solar X/Eclipse',221,'',115 from dual union all
select 14453,'1966-S605','1966-11-12 14:01:59','suborbital rocket','success',837,'NASA 14.77CA','GCA Release 58 TMA','','','Aeronomy/Eclipse',221,'',185 from dual union all
select 14454,'1966-S606','1966-11-12 14:05:00','suborbital rocket','success',837,'NASA 14.274UI','','','','Ionos/Eclipse',221,'',197 from dual union all
select 14455,'1966-S607','1966-11-12 14:07:00','suborbital rocket','success',837,'NASA 14.302UI','Brazil EUV/Mass spec','','','Ionos/Eclipse',221,'',160 from dual union all
select 14456,'1966-S608','1966-11-12 14:10:00','suborbital rocket','success',837,'NASA 14.304UI','','','','Ionos/Eclipse',221,'',195 from dual union all
select 14457,'1966-S609','1966-11-12 14:43:00','suborbital rocket','success',865,'Sandia 152-95','','','','Solar X/Eclipse',221,'',115 from dual union all
select 14458,'1966-S61','1966-01-13 00:00:00','military missile','success',874,'ZK-60','','','','Interceptor',873,'',80 from dual union all
select 14459,'1966-S610','1966-11-12 13:45:00','suborbital rocket','success',846,'','','','','Solar',221,'',250 from dual union all
select 1446,'1952-S05','1952-05-01 14:59:00','suborbital rocket','success',54,'NRL 9','UV detectors','','','Solar X,UV',1869,'',126 from dual union all
select 14460,'1966-S611','1966-11-12 13:55:19','suborbital rocket','success',846,'D-4','','','','Ionos',221,'',115 from dual union all
select 14461,'1966-S612','1966-11-12 14:08:37','suborbital rocket','success',846,'D-11','','','','Ionos',221,'',250 from dual union all
select 14462,'1966-S613','1966-11-12 14:22:30','suborbital rocket','success',846,'D-13','','','','Ionos',221,'',115 from dual union all
select 14463,'1966-S614','1966-11-12 16:00:00','suborbital rocket','success',837,'NASA 14.303UI','','','','Ionos/Eclipse',221,'',200 from dual union all
select 14464,'1966-S615','1966-11-12 16:15:00','suborbital rocket','success',30,'NASA 04.153GS','','','','Solar eclipse',1869,'',151 from dual union all
select 14465,'1966-S616','1966-11-14 00:00:00','military missile','unknown',932,'A3P-512','','','','OT',421,'SSBN 655',1000 from dual union all
select 14466,'1966-S617','1966-11-14 16:50:02','military missile','unknown',932,'A3P-394','','','','OT',421,'SSBN 655',1000 from dual union all
select 14467,'1966-S618','1966-11-14 11:28:21','suborbital rocket','success',288,'C105','CNES FU-167','','','Aeronomy',664,'',170 from dual union all
select 14468,'1966-S619','1966-11-14 13:03:14','suborbital rocket','success',288,'C139','CNES FU-167','','','Aeronomy',664,'',170 from dual union all
select 14469,'1966-S62','1966-01-17 16:42:04','military missile','unknown',932,'A3P-393','','','','OT',421,'SSBN 641',1000 from dual union all
select 1447,'1952-S06','1952-05-01 15:42:00','suborbital rocket','success',50,'USAF 25','Solar flux','','','Solar UV',640,'',91 from dual union all
select 14470,'1966-S620','1966-11-14 20:38:58','test rocket','success',452,'F-5','','','','-',1845,'',120 from dual union all
select 14471,'1966-S621','1966-11-15 13:23:00','suborbital rocket','success',846,'','','','','Solar',221,'',150 from dual union all
select 14472,'1966-S622','1966-11-17 20:15:00','suborbital rocket','success',13,'','Leonids','','','Test/Meteorites',1895,'F-4D',100 from dual union all
select 14473,'1966-S623','1966-11-16 00:00:00','military missile','success',795,'1207','Mk 11 RV','FOT/NTMP KX-69','LONG DOZEN','OT',1705,'',900 from dual union all
select 14474,'1966-S624','1966-11-16 13:15:00','suborbital rocket','success',30,'CRL AF3.267','Leonids','','','Meteorites',1869,'',173 from dual union all
select 14475,'1966-S625','1966-11-17 16:09:14','military missile','failure',932,'A3TX-1 (A3P-492)','Antelope 1','','ANTELOPE 1','Test',251,'',500 from dual union all
select 14476,'1966-S626','1966-11-17 07:05:00','military missile','success',982,'G 22603-06T','8K64U UBP-90','','','OT',566,'',1210 from dual union all
select 14477,'1966-S627','1966-11-17 01:41:24','suborbital rocket','success',768,'','Y-10 SHOT-17','','','Aeron',1914,'',121 from dual union all
select 14478,'1966-S628','1966-11-17 03:41:52','suborbital rocket','success',768,'','Y-11 SHOT-18','','','Aeron',1914,'',120 from dual union all
select 14479,'1966-S629','1966-11-17 05:32:07','suborbital rocket','success',768,'','Y-12 SHOT-19','','','Aeron',1914,'',114 from dual union all
select 1448,'1952-S07','1952-05-05 13:44:00','suborbital rocket','success',54,'NRL 10','UV detectors','','','Solar X/UV/CR',1869,'',127 from dual union all
select 14480,'1966-S63','1966-01-17 17:56:00','suborbital rocket','success',288,'C127','CNES FU-148','','','Ionos',664,'',124 from dual union all
select 14481,'1966-S630','1966-11-17 07:16:13','suborbital rocket','success',768,'','Y-13 SHOT-20','','','Aeron',1914,'',126 from dual union all
select 14482,'1966-S631','1966-11-17 10:16:00','suborbital rocket','success',768,'','SHOT-21','','','Aeron',1914,'',100 from dual union all
select 14483,'1966-S632','1966-11-18 01:05:00','suborbital rocket','failure',1207,'SL481','RAE Test 35','','','Test',1839,'',142 from dual union all
select 14484,'1966-S633','1966-11-18 22:15:00','suborbital rocket','success',837,'NASA 14.299UA','Pandora A','','','Aeronomy',1895,'',161 from dual union all
select 14485,'1966-S634','1966-11-19 00:10:00','suborbital rocket','success',215,'AAD-2A-106','H line emission','','','Auroral/Ion/Field',428,'',144 from dual union all
select 14486,'1966-S635','1966-11-19 01:18:47','suborbital rocket','success',768,'','Y-14 SHOT-22','','','Aeron',1914,'',122 from dual union all
select 14487,'1966-S636','1966-11-19 03:12:21','suborbital rocket','success',768,'','Y-15 SHOT-23','','','Aeron',1914,'',125 from dual union all
select 14488,'1966-S637','1966-11-19 04:49:42','suborbital rocket','success',768,'','Y-16 SHOT-24','','','Aeron',1914,'',155 from dual union all
select 14489,'1966-S638','1966-11-19 06:43:09','suborbital rocket','success',768,'','Y-17 SHOT-25','','','Aeron',1914,'',149 from dual union all
select 1449,'1952-S08','1952-05-20 02:07:00','suborbital rocket','success',59,'SC 24','Grenades','','','Aeronomy',1869,'',90 from dual union all
select 14490,'1966-S639','1966-11-19 08:01:22','suborbital rocket','success',768,'','Y-18 SHOT-26','','','Aeron',1914,'',161 from dual union all
select 14491,'1966-S64','1966-01-17 22:39:00','suborbital rocket','success',837,'NASA 14.262CM','GCA Release 48 Na','','','Aeronomy',1821,'',206 from dual union all
select 14492,'1966-S640','1966-11-19 09:35:35','suborbital rocket','success',768,'','Y-19 SHOT-27','','','Aeron',1914,'',122 from dual union all
select 14493,'1966-S641','1966-11-19 11:52:53','suborbital rocket','success',768,'','Y-20 SHOT-28','','','Aeron',1914,'',180 from dual union all
select 14494,'1966-S642','1966-11-19 23:42:36','suborbital rocket','success',288,'','ESRO C21/1','','','Auroral/CR',408,'',123 from dual union all
select 14495,'1966-S643','1966-11-20 02:45:00','suborbital rocket','success',768,'','Y-21 SHOT-30','','','Aeron',1914,'',146 from dual union all
select 14496,'1966-S644','1966-11-20 04:21:29','suborbital rocket','success',768,'','Y-22 SHOT-31','','','Aeron',1914,'',112 from dual union all
select 14497,'1966-S645','1966-11-20 05:37:53','suborbital rocket','success',768,'','Y-23 SHOT-32','','','Aeron',1914,'',113 from dual union all
select 14498,'1966-S646','1966-11-20 06:59:14','suborbital rocket','success',768,'','Y-24 SHOT-33','','','Aeron',1914,'',168 from dual;
insert into LAUNCH
select 14499,'1966-S647','1966-11-21 02:08:00','military missile','success',999,'N 22500-04T','8K67 UBP-7','GCh 8F674 LKI','','Test',485,'',1000 from dual union all
select 145,'1944-S131','1944-09-22 07:26:00','military missile','success',8,'18825','Ma213','','','Training',661,'',90 from dual union all
select 1450,'1952-S09','1952-05-20 16:06:00','suborbital rocket','success',1470,'59/TF2','','','','Test/Photo/Aeron',1863,'',124 from dual union all
select 14500,'1966-S648','1966-11-21 10:00:00','suborbital rocket','success',30,'NASA 04.154GG','','','','UV Astron',1869,'',165 from dual union all
select 14501,'1966-S649','1966-11-22 21:40:00','military missile','success',795,'1183','Mk 11 RV','FOT','WORKING GIRL','OT',1698,'',900 from dual union all
select 14502,'1966-S65','1966-01-18 00:00:00','military missile','success',796,'2053','Mk 11A RV','R&D','RESTLESS DRIFTER','Test',1712,'',900 from dual union all
select 14503,'1966-S650','1966-11-23 21:44:00','suborbital rocket','success',740,'','Thermosphere comp. 2','','','Aeron',1010,'',163 from dual union all
select 14504,'1966-S651','1966-11-24 03:15:00','military missile','success',982,'G 22603-03T','8K64U UBP-91','','','OT',565,'',1210 from dual union all
select 14505,'1966-S652','1966-11-24 10:00:00','military missile','success',1420,'B-68','Mk 6 RV','FOT','BUBBLE GIRL','OT',1657,'',1300 from dual union all
select 14506,'1966-S653','1966-11-24 00:00:00','suborbital rocket','success',1492,'VA82','FU-158 Recovery test','','','Test',668,'',230 from dual union all
select 14507,'1966-S654','1966-11-26 04:15:00','military missile','success',982,'G 22602-08T','8K64U UBP-92','','','OT',566,'',1210 from dual union all
select 14508,'1966-S655','1966-11-27 02:16:00','suborbital rocket','success',30,'CRL AG3.357','A1 274-1L','Lunar IR','','IR Astron',428,'',169 from dual union all
select 14509,'1966-S656','1966-11-28 16:15:02','military missile','unknown',932,'A3P-389','','','','OT',421,'SSBN 655',1000 from dual union all
select 1451,'1952-S10','1952-06-18 17:50:00','suborbital rocket','success',50,'USAF 27','UV/Opt spectrographs','','','Solar UV',640,'',100 from dual union all
select 14510,'1966-S657','1966-11-28 16:45:02','military missile','unknown',932,'A3P-378','','','','OT',421,'SSBN 655',1000 from dual union all
select 14511,'1966-S658','1966-11-28 12:15:00','military missile','failure',1095,'CC-20xx/SV-1','SPARTA SV-1','','','Reentry Test',1846,'',90 from dual union all
select 14512,'1966-S659','1966-11-29 01:06:39','suborbital rocket','success',288,'CE-8','ESRO C9/1','','','Auroral',408,'',147 from dual union all
select 14513,'1966-S66','1966-01-18 00:32:00','suborbital rocket','success',837,'NASA 14.263CM','GCA Release 49 TMA','','','Aeronomy',1821,'',205 from dual union all
select 14514,'1966-S660','1966-11-29 06:01:00','suborbital rocket','success',217,'AAF-3-12','','','','Plasma/Fields',428,'',114 from dual union all
select 14515,'1966-S661','1966-11-29 06:08:00','suborbital rocket','success',219,'AAF-4-05','CR/ne probe','','','Plasma/Fields',428,'',800 from dual union all
select 14516,'1966-S662','1966-11-18 21:00:00','military missile','success',837,'BMRS','SPEEDBALL II 93 SN39','','','Target',1895,'',100 from dual union all
select 14517,'1966-S663','1966-11-30 11:45:00','suborbital rocket','success',30,'NASA 04.181UA','Composition','','','Aeronomy',1869,'',227 from dual union all
select 14518,'1966-S664','1966-11-30 19:06:00','suborbital rocket','success',867,'NASA 18.27UE','UNH/Particles','','','Plasma',1821,'',359 from dual union all
select 14519,'1966-S665','1966-11-30 22:04:16','suborbital rocket','success',288,'CE-7','ESRO C9/2','','','Auroral',408,'',149 from dual union all
select 1452,'1952-S11','1952-06-30 14:32:00','suborbital rocket','success',50,'USAF 28','AF Airglow 1','','','Aeronomy SB',640,'',101 from dual union all
select 14520,'1966-S666','1966-12-01 20:40:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',168 from dual union all
select 14521,'1966-S667','1966-12-02 21:09:00','suborbital rocket','success',30,'NASA 04.180UA','Composition','','','Aeronomy',1869,'',227 from dual union all
select 14522,'1966-S668','1966-12-05 02:00:00','suborbital rocket','success',668,'K-9M-21','Gyro-plasma probe','','','Aeron/Ionos/Solar UV',839,'',326 from dual union all
select 14523,'1966-S669','1966-12-06 03:35:00','suborbital rocket','success',30,'CRL AE3.724','','','','Tech IR',1869,'',188 from dual union all
select 14524,'1966-S67','1966-01-18 02:00:00','suborbital rocket','success',837,'NASA 14.264CM','GCA Release 50 TMA','','','Aeronomy',1821,'',167 from dual union all
select 14525,'1966-S670','1966-12-06 08:39:00','suborbital rocket','success',740,'','','','','Aeron',636,'',167 from dual union all
select 14526,'1966-S671','1966-12-08 07:56:00','military missile','success',982,'G 22603-18T','8K64U UBP-93','','','OT',571,'',1210 from dual union all
select 14527,'1966-S672','1966-12-09 06:54:00','military missile','success',982,'G 22602-12T','8K64U UBP-94','','','OT',566,'',1210 from dual union all
select 14528,'1966-S673','1966-12-09 15:20:00','suborbital rocket','success',1493,'AGI 59','FU-149 Ions/mass sp.','','','Ionos/Aeron',668,'',122 from dual union all
select 14529,'1966-S674','1966-12-09 11:00:00','suborbital rocket','success',839,'NASA 10.160GM','GSFC Ozone 1','','','Aeronomy',1895,'',120 from dual union all
select 1453,'1952-S12','1952-08-09 00:00:00','military missile','success',992,'KT-2','8Zh38 KI-2','','','Test',636,'',150 from dual union all
select 14530,'1966-S675','1966-12-10 02:00:00','suborbital rocket','success',666,'K-8L-12','','','','Aeron/Ionos/Solar',839,'',147 from dual union all
select 14531,'1966-S676','1966-12-10 02:42:00','suborbital rocket','success',1124,'','Scanner 2','','','Aeronomy',1821,'',709 from dual union all
select 14532,'1966-S677','1966-12-10 11:30:00','suborbital rocket','success',655,'K-10-2','','','','Aeron/Solar UV',839,'',253 from dual union all
select 14533,'1966-S678','1966-12-12 17:39:00','suborbital rocket','success',740,'','','','','Aeron/Ionos',636,'',160 from dual union all
select 14534,'1966-S679','1966-12-12 19:19:00','suborbital rocket','success',217,'ALF-3-18','BB3/1 Neutral dens.','','','Aeron',428,'',137 from dual union all
select 14535,'1966-S68','1966-01-18 03:19:00','suborbital rocket','success',35,'NASA 04.90GG','STRAP','','','UV Astron',1821,'',134 from dual union all
select 14536,'1966-S680','1966-12-13 00:00:00','military missile','success',1117,'R6','','','','Reentry Test',669,'',1000 from dual union all
select 14537,'1966-S681','1966-12-13 21:55:00','military missile','success',796,'2036','ERCS 1','494L (WATER TEST)','WATER TEST','Comms',1700,'',900 from dual union all
select 14538,'1966-S682','1966-12-13 06:30:00','suborbital rocket','success',30,'NASA 04.182UG','','','','XR Astron',330,'',169 from dual union all
select 14539,'1966-S683','1966-12-13 11:50:00','military missile','success',1095,'CC-2034/SV-2','SPARTA SV-2','','','Reentry Test',1846,'',90 from dual union all
select 1454,'1952-S13','1952-08-12 00:00:00','military missile','success',992,'KT-3','8Zh38 KI-3','','','Test',636,'',150 from dual union all
select 14540,'1966-S684','1966-12-14 06:33:00','suborbital rocket','success',1199,'SL581A','RAE Test 36','','','Test',1839,'',164 from dual union all
select 14541,'1966-S685','1966-12-14 08:16:00','suborbital rocket','success',30,'CRL AF3.615','A1 199-6L','','','Ionos',428,'',180 from dual union all
select 14542,'1966-S686','1966-12-14 08:16:00','suborbital rocket','success',130,'CRL AC15.735','AE15.735 elec flux','','','CR/Aurora',428,'',212 from dual union all
select 14543,'1966-S687','1966-12-14 11:30:00','suborbital rocket','success',30,'NRL NB3.194','','','','XR Astron?',1869,'',177 from dual union all
select 14544,'1966-S688','1966-12-15 00:00:00','military missile','success',135,'D038','RVTO','ABRES SS-6 (USAF)','','Reentry Test',613,'',200 from dual union all
select 14545,'1966-S689','1966-12-15 15:32:00','suborbital rocket','success',219,'AAF-4-08','','','','Test',428,'',871 from dual union all
select 14546,'1966-S69','1966-01-18 05:12:00','suborbital rocket','success',837,'NASA 14.265CM','GCA Release 51 TMA','','','Aeronomy',1821,'',207 from dual union all
select 14547,'1966-S690','1966-12-15 20:12:00','suborbital rocket','success',740,'','Mass spectrometer','','','Aeron',636,'',178 from dual union all
select 14548,'1966-S691','1966-12-15 21:32:00','suborbital rocket','success',219,'AAF-4-07','','','','Test',428,'',871 from dual union all
select 14549,'1966-S692','1966-12-16 00:00:00','military missile','success',135,'D042','ABRES QQ-3 (USAF)','ABRES QQ-3 (USAF)','','Reentry Test',614,'',200 from dual union all
select 1455,'1952-S14','1952-08-14 00:00:00','military missile','failure',992,'KT-4','8Zh38 KI-4','','','Test',636,'',150 from dual union all
select 14550,'1966-S693','1966-12-20 00:00:00','military missile','success',795,'1227','Mk 11 RV','FOT','SHELL BEACH','OT',1705,'',900 from dual union all
select 14551,'1966-S694','1966-12-20 16:00:00','suborbital rocket','success',740,'','','','','Aeron/Ionos',636,'',178 from dual union all
select 14552,'1966-S695','1966-12-21 12:30:00','suborbital rocket','success',740,'','','','','Aeron',636,'',166 from dual union all
select 14553,'1966-S696','1966-12-21 12:30:00','suborbital rocket','success',740,'','','','','Solar X',636,'',167 from dual union all
select 14554,'1966-S697','1966-12-22 21:50:00','military missile','success',795,'923','Mk 11 RV','FOT Salvo','WILD DUCK','OT',1698,'',900 from dual union all
select 14555,'1966-S698','1966-12-22 21:50:00','military missile','success',795,'1128','Mk 11 RV','FOT Salvo','STONE AXE','OT',1702,'',900 from dual union all
select 14556,'1966-S699','1966-12-26 03:00:00','military missile','failure',356,'601601','R&D','','','Test',809,'',500 from dual union all
select 14557,'1966-S70','1966-01-18 11:31:00','suborbital rocket','success',837,'NASA 14.266CM','GCA Release 52 Na','','','Aeronomy',1821,'',213 from dual union all
select 14558,'1966-S700','1966-05-19 00:00:00','suborbital rocket','success',894,'','','','','Test',310,'',100 from dual union all
select 14559,'1966-S701','1966-04-01 00:00:00','military missile','success',850,'Min 2 plume study','','','','Tech',1785,'',100 from dual union all
select 1456,'1952-S15','1952-08-25 00:00:00','military missile','success',992,'KT-5','8Zh38 KI-5','','','Test',636,'',150 from dual union all
select 14560,'1966-S702','1966-02-01 00:00:00','suborbital rocket','success',1440,'','AFAL 1','','','Reentry Test',1821,'',200 from dual union all
select 14561,'1966-S703','1966-01-01 00:00:00','suborbital rocket','success',1431,'Sandia 276-3','Dart/Parachute','','','Parachute test',1490,'',90 from dual union all
select 14562,'1966-S704','1966-01-01 00:00:00','suborbital rocket','success',1431,'Sandia 276-4','Dart/Parachute','','','Parachute test',1490,'',90 from dual union all
select 14563,'1966-S705','1966-01-01 00:00:00','suborbital rocket','success',1431,'Sandia 276-5','Dart/Parachute','','','Parachute test',1490,'',90 from dual union all
select 14564,'1966-S706','1966-02-18 01:03:00','suborbital rocket','success',767,'','B-43 INAUGUA','','','Aeron',185,'',123 from dual union all
select 14565,'1966-S707','1966-02-24 09:23:30','suborbital rocket','success',767,'','B-49 ST THOMAS','','','Aeron',185,'',118 from dual union all
select 14566,'1966-S708','1966-09-20 00:55:09','suborbital rocket','success',767,'','B-51 BETA','','','Aeron',185,'',128 from dual union all
select 14567,'1966-S709','1966-09-20 02:24:06','suborbital rocket','success',767,'','B-52 GAMMA','','','Aeron',185,'',134 from dual union all
select 14568,'1966-S71','1966-01-18 18:11:00','military missile','success',1373,'242','AFP-437(AP)','','','Interceptor',773,'',459 from dual union all
select 14569,'1966-S710','1966-09-20 04:10:00','suborbital rocket','success',767,'','B-53 DELTA','','','Aeron',185,'',122 from dual union all
select 1457,'1952-S16','1952-08-26 00:00:00','military missile','success',992,'KT-6','8Zh38 KI-6','','','Test',636,'',150 from dual union all
select 14570,'1966-S711','1966-09-20 06:24:00','suborbital rocket','success',767,'','B-55 ZETA','','','Aeron',185,'',117 from dual union all
select 14571,'1966-S712','1966-09-20 08:03:00','suborbital rocket','success',767,'','B-57 THETA','','','Aeron',185,'',128 from dual union all
select 14572,'1966-S713','1966-06-14 03:25:00','suborbital rocket','success',768,'','Y-1 SHOT-04','','','Aeron',1914,'',126 from dual union all
select 14573,'1966-S714','1966-06-14 05:15:00','suborbital rocket','success',768,'','Y-2 SHOT-05','','','Aeron',1914,'',121 from dual union all
select 14574,'1966-S715','1966-06-15 04:46:00','suborbital rocket','success',768,'','Y-3 SHOT-06','','','Aeron',1914,'',122 from dual union all
select 14575,'1966-S716','1966-06-15 06:17:00','suborbital rocket','success',768,'','SHOT-07','','','Aeron',1914,'',100 from dual union all
select 14576,'1966-S717','1966-06-15 08:27:00','suborbital rocket','success',767,'','Y-4 MCCONNELL SHOT-8','','','Aeron',1914,'',114 from dual union all
select 14577,'1966-S718','1966-06-15 10:05:00','suborbital rocket','success',768,'','Y-5 SHOT-09','','','Aeron',1914,'',125 from dual union all
select 14578,'1966-S719','1966-10-26 04:59:00','suborbital rocket','success',768,'','SHOT-10','','','Aeron',1914,'',94 from dual union all
select 14579,'1966-S72','1966-01-21 00:00:00','military missile','success',135,'C025','ECM Decoy','ABRES H-2 (USAF)','','Reentry Test',613,'',200 from dual union all
select 1458,'1952-S17','1952-08-27 00:00:00','military missile','success',992,'KT-7','8Zh38 KI-7','','','Test',636,'',150 from dual union all
select 14580,'1966-S720','1966-10-27 03:55:00','suborbital rocket','success',768,'','SHOT-11','','','Aeron',1914,'',125 from dual union all
select 14581,'1966-S721','1966-10-27 06:56:00','suborbital rocket','success',768,'','SHOT-12','','','Aeron',1914,'',165 from dual union all
select 14582,'1966-S722','1966-10-27 22:47:00','suborbital rocket','success',519,'Lahive 15-deg Cone','SHOT-14','','','Aeron',1914,'',80 from dual union all
select 14583,'1966-S723','1966-10-28 02:52:00','suborbital rocket','success',768,'','SHOT-13','','','Aeron',1914,'',126 from dual union all
select 14584,'1966-S724','1966-10-28 04:46:00','suborbital rocket','success',768,'','SHOT-15','','','Aeron',1914,'',163 from dual union all
select 14585,'1966-S725','1966-10-28 06:20:00','suborbital rocket','success',768,'','SHOT-16','','','Aeron',1914,'',100 from dual union all
select 14586,'1966-S726','1966-11-19 13:08:00','suborbital rocket','success',768,'','SHOT-29','','','Aeron',1914,'',100 from dual union all
select 14587,'1966-S727','1966-12-23 00:00:00','military missile','failure',1037,'','8K98 LKI-2-2','','','Test',477,'',100 from dual union all
select 14588,'1966-S728','1966-12-28 00:00:00','military missile','failure',1037,'','8K98 LKI-2-3','','','Test',477,'',100 from dual;
insert into LAUNCH
select 14589,'1966-S729','1966-11-22 16:02:00','suborbital rocket','success',846,'','STV SR-43','','','Aeron?',1864,'',100 from dual union all
select 1459,'1952-S18','1952-08-28 00:00:00','military missile','success',992,'KT-8','8Zh38 KI-8','','','Test',636,'',150 from dual union all
select 14590,'1966-S73','1966-01-21 08:14:00','suborbital rocket','success',446,'D-21','FU-133 Na release','','','Aeronomy',664,'',500 from dual union all
select 14591,'1966-S730','1966-12-01 15:23:00','suborbital rocket','success',846,'','STV SR-44','','','Aeron?',1895,'',100 from dual union all
select 14592,'1966-S731','1966-09-20 16:13:00','suborbital rocket','success',846,'STV SR-38','Mk 12 STV','','','Aeron?',1864,'',100 from dual union all
select 14593,'1966-S732','1966-10-04 14:02:00','suborbital rocket','success',846,'STV SR-40','','','','Aeron?',1895,'',100 from dual union all
select 14594,'1966-S733','1966-09-27 14:35:00','suborbital rocket','success',846,'STV SR-39','Mk 12 STV','','','Aeron?',1895,'',100 from dual union all
select 14595,'1966-S734','1966-10-25 19:20:00','suborbital rocket','success',846,'STV SR-41','','','','Aeron?',1895,'',100 from dual union all
select 14596,'1966-S735','1966-10-12 19:41:00','military missile','success',837,'BMRS','SPEEDBALL II 92 SN38','','','Target',1895,'',150 from dual union all
select 14597,'1966-S736','1966-11-18 23:00:00','military missile','success',837,'BMRS','SPEEDBALL II 94 SN40','','','Target',1895,'',100 from dual union all
select 14598,'1966-S737','1966-11-19 01:00:00','military missile','success',837,'BMRS','SPEEDBALL II 95 SN41','','','Target',1895,'',100 from dual union all
select 14599,'1966-S738','1966-07-24 10:15:00','military missile','success',839,'','NICAP/1 66-19','','','Target',1895,'',120 from dual union all
select 146,'1944-S132','1944-09-22 18:16:00','military missile','success',8,'18826','Ma215','','','Training',661,'',90 from dual union all
select 1460,'1952-S19','1952-08-08 00:00:00','military missile','success',992,'KT-1','8Zh38 KI-1','','','Test',636,'',150 from dual union all
select 14600,'1966-S739','1966-07-01 00:00:00','military missile','success',839,'','NICAP/1 66-02','','','Target',1895,'',120 from dual union all
select 14601,'1966-S74','1966-01-22 00:00:00','military missile','success',796,'2022','Mk 11 RV','Force Mod','SUPREME CHIEF','Test',1700,'',900 from dual union all
select 14602,'1966-S740','1966-07-24 11:45:00','military missile','success',839,'','NICAP/1 66-20','','','Target',1895,'',120 from dual union all
select 14603,'1966-S741','1966-07-24 13:00:00','military missile','success',839,'','NICAP/1 66-21','','','Target',1895,'',120 from dual union all
select 14604,'1966-S742','1966-07-07 10:38:00','military missile','success',839,'','NICAP/1 66-03','','','Target',1895,'',120 from dual union all
select 14605,'1966-S743','1966-07-15 19:09:00','military missile','success',839,'','NICAP/1 66-04','','','Target',1895,'',120 from dual union all
select 14606,'1966-S744','1966-07-18 23:40:00','military missile','success',839,'','NICAP/1 66-09','','','Target',1895,'',120 from dual union all
select 14607,'1966-S745','1966-08-04 00:00:00','suborbital rocket','success',286,'Cedar 8','','','','Test',356,'',140 from dual union all
select 14608,'1966-S746','1966-08-10 00:00:00','atmospheric rocket','success',617,'J203','HRV 923/UK-D','','','Test',1836,'',250 from dual union all
select 14609,'1966-S747','1966-05-27 00:00:00','military missile','success',1017,'','8K74 UBP','','','Test',480,'',1350 from dual union all
select 1461,'1952-S20','1952-08-20 00:00:00','military missile','success',970,'No 220','8A11','','','OT',636,'',100 from dual union all
select 14610,'1966-S748','1966-06-25 00:00:00','military missile','success',1017,'','8K74 UBP','Pal''ma','','Test',480,'',1350 from dual union all
select 14611,'1966-S749','1966-06-29 00:00:00','military missile','success',916,'','P-104 66 Summer 03','','','OT',597,'',180 from dual union all
select 14612,'1966-S75','1966-01-23 06:06:00','suborbital rocket','success',130,'CRL AC15.732','Electron flux','','','Ionos/Auroral',428,'',301 from dual union all
select 14613,'1966-S750','1966-11-15 15:45:00','suborbital rocket','success',846,'','Mk 12 STV SR-042','','','Aeron?',1864,'',100 from dual union all
select 14614,'1966-S751','1966-06-13 09:44:00','military missile','success',837,'BMRS Photometric','STV SR-034 Photometric','','','Target',1895,'',100 from dual union all
select 14615,'1966-S752','1966-08-03 17:00:00','military missile','success',837,'','SPEEDBALL II 87 SN33','','','Target',1895,'',150 from dual union all
select 14616,'1966-S753','1966-09-28 17:20:00','military missile','success',837,'BMTS','SPEEDBALL II 89 SN35','','','Target',1895,'',150 from dual union all
select 14617,'1966-S754','1966-03-29 00:00:00','military missile','failure',982,'','8K64U UBP-P5','','','OT',477,'',0 from dual union all
select 14618,'1966-S755','1966-12-26 00:00:00','military missile','success',982,'G 22604-18T','8K64U UBP-P17','','','OT',477,'',1210 from dual union all
select 14619,'1966-S756','1966-07-25 00:00:00','military missile','success',982,'G 22604-02T','8K64U UBP-P8','','','OT',477,'',1210 from dual union all
select 1462,'1952-S21','1952-08-21 00:00:00','military missile','success',970,'No 105','8A11','','','OT',636,'',100 from dual union all
select 14620,'1966-S757','1966-12-08 00:00:00','military missile','success',982,'G 22602-16T','8K64U UBP-P14','','','OT',477,'',1210 from dual union all
select 14621,'1966-S758','1966-12-24 00:00:00','military missile','success',982,'G 22604-17T','8K64U UBP-P16','','','OT',477,'',1210 from dual union all
select 14622,'1966-S759','1966-05-27 00:00:00','military missile','success',982,'G 22604-01T','8K64U UBP-P6','','','OT',477,'',1210 from dual union all
select 14623,'1966-S76','1966-01-24 00:00:00','suborbital rocket','success',740,'D-75M','','','','Aeron',1010,'',150 from dual union all
select 14624,'1966-S760','1966-07-02 00:00:00','military missile','success',982,'T 22502-11','8K64U UBP-P7','','','OT',477,'',1210 from dual union all
select 14625,'1966-S761','1966-09-16 00:00:00','military missile','success',982,'G 22603-03T','8K64U UBP-P9','','','OT',477,'',1210 from dual union all
select 14626,'1966-S762','1966-10-08 00:00:00','military missile','success',982,'G 22603-11T','8K64U UBP-P10','','','OT',477,'',1210 from dual union all
select 14627,'1966-S763','1966-10-24 00:00:00','military missile','success',982,'G 22604-04T','8K64U UBP-P11','','','OT',477,'',1210 from dual union all
select 14628,'1966-S764','1966-11-03 00:00:00','military missile','success',982,'G 22604-05T','8K64U UBP-P12','','','OT',477,'',1210 from dual union all
select 14629,'1966-S765','1966-11-25 00:00:00','military missile','success',982,'G 22602-17T','8K64U UBP-P13','','','OT',477,'',1210 from dual union all
select 1463,'1952-S22','1952-08-25 00:00:00','military missile','success',970,'No 320','8A11','','','OT',636,'',100 from dual union all
select 14630,'1966-S766','1966-12-10 00:00:00','military missile','success',982,'G 22602-14T','8K64U UBP-P15','','','OT',477,'',1210 from dual union all
select 14631,'1966-S767','1966-07-02 00:00:00','military missile','success',982,'','8K64U 54 RD UBP-7','','','OT',1468,'',1210 from dual union all
select 14632,'1966-S768','1966-09-26 00:00:00','military missile','success',982,'','8K64U 54 RD UBP-8','','','OT',1468,'',1210 from dual union all
select 14633,'1966-S769','1966-03-03 00:00:00','military missile','success',1018,'','8K75 UBP Omsk-2','','','OT',1012,'',1160 from dual union all
select 14634,'1966-S77','1966-01-24 05:42:00','suborbital rocket','success',839,'NASA 10.185GM','GSFC Grenade 98 C','','','Aeronomy',428,'',125 from dual union all
select 14635,'1966-S770','1966-07-17 00:00:00','military missile','success',1018,'','8K75 UBP Omsk-3','','','OT',1012,'',1160 from dual union all
select 14636,'1966-S771','1966-10-11 00:00:00','suborbital rocket','success',1250,'','Density sphere','','','Test/Aeron',394,'',90 from dual union all
select 14637,'1966-S772','1966-10-11 00:00:00','suborbital rocket','success',1250,'','Density sphere','','','Test/Aeron',394,'',90 from dual union all
select 14638,'1966-S773','1966-01-01 00:00:00','military missile','success',1037,'','8K98 LKI-1 6','','','Test',632,'',1000 from dual union all
select 14639,'1966-S774','1966-10-21 00:00:00','military missile','success',1323,'','9M76B','Pal''ma-2','','Test',596,'',100 from dual union all
select 1464,'1952-S23','1952-08-29 18:15:00','suborbital rocket','success',370,'','SUI 5','','','CR',172,'WYG-279',90 from dual union all
select 14640,'1966-S775','1966-01-01 00:00:00','military missile','success',977,'','8K63U UBP 40 RD','','','Test',840,'',402 from dual union all
select 14641,'1966-S776','1966-09-21 00:00:00','military missile','success',977,'','8K63U UBP 51 RD','','','Test',840,'',402 from dual union all
select 14642,'1966-S777','1966-10-12 00:00:00','military missile','success',977,'','8K63U UBP 40 RD','','','Test',840,'',402 from dual union all
select 14643,'1966-S778','1966-10-19 00:00:00','military missile','success',977,'','8K63U UBP 40 RD','','','Test',840,'',402 from dual union all
select 14644,'1966-S779','1966-12-06 00:00:00','military missile','success',982,'','8K64U 8 RD UBP-2','','','OT',1916,'',1210 from dual union all
select 14645,'1966-S78','1966-01-25 01:52:00','suborbital rocket','success',839,'NASA 10.158UA','UM Falling Spheres','','','Aeronomy',1821,'',167 from dual union all
select 14646,'1966-S780','1966-02-01 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 14647,'1966-S781','1966-04-01 00:00:00','military missile','success',976,'','8K63 44 RD UBP','','','Test',636,'',402 from dual union all
select 14648,'1966-S782','1966-04-01 00:00:00','military missile','success',976,'','8K63 32 RD UBP','','','Test',636,'',402 from dual union all
select 14649,'1966-S783','1966-06-23 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 1465,'1952-S24','1952-09-02 00:00:00','military missile','failure',992,'KT-9','8Zh38 KI-9','','','Test',636,'',150 from dual union all
select 14650,'1966-S784','1966-07-06 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 14651,'1966-S785','1966-09-01 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 14652,'1966-S786','1966-10-01 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 14653,'1966-S787','1966-11-30 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 14654,'1966-S788','1966-12-22 00:00:00','military missile','success',976,'','8K63 43 RD UBP','','','Test',636,'',402 from dual union all
select 14655,'1966-S789','1966-03-29 00:00:00','military missile','success',979,'','8K65 UBP-60 43 RD','','','Test',636,'',675 from dual union all
select 14656,'1966-S79','1966-01-26 10:32:00','suborbital rocket','success',535,'HAD 176','Sphere','','','Aeron',1835,'',100 from dual union all
select 14657,'1966-S790','1966-05-25 00:00:00','military missile','success',979,'','8K65 UBP-61 43 RD','','','Test',636,'',675 from dual union all
select 14658,'1966-S791','1966-06-29 00:00:00','military missile','success',979,'','8K65 UBP-62 43 RD','','','Test',636,'',675 from dual union all
select 14659,'1966-S792','1966-07-06 00:00:00','military missile','success',979,'','8K65 UBP-63 43 RD','','','Test',636,'',675 from dual union all
select 1466,'1952-S25','1952-09-15 00:00:00','military missile','success',992,'KT-10','8Zh38 KI-10','','','Test',636,'',150 from dual union all
select 14660,'1966-S793','1966-07-29 00:00:00','military missile','success',980,'','8K65U UBP-6 29 RD','','','Test',636,'',675 from dual union all
select 14661,'1966-S794','1966-10-21 00:00:00','military missile','success',979,'','8K65 UBP-64 43 RD','','','Test',636,'',675 from dual union all
select 14662,'1966-S795','1966-11-16 00:00:00','military missile','success',980,'','8K65U UBP-7 68 RD','','','Test',636,'',675 from dual union all
select 14663,'1966-S80','1966-01-26 17:45:00','suborbital rocket','success',837,'','Sandia 154-59','','','Aeron',855,'',92 from dual union all
select 14664,'1966-S81','1966-01-26 23:50:00','suborbital rocket','success',837,'','Sandia 154-60','','','Aeron',855,'',110 from dual union all
select 14665,'1966-S82','1966-01-27 00:00:00','military missile','success',795,'1243','Mk 5 RV','FOT-1','ANCHOR POLE','OT',1705,'',900 from dual union all
select 14666,'1966-S83','1966-01-27 01:29:00','suborbital rocket','success',30,'NRL NC3.186','','','','Aeron/Ionos',1869,'',243 from dual union all
select 14667,'1966-S84','1966-01-31 00:00:00','military missile','success',916,'AO-30','P-99','','','OT',435,'',180 from dual union all
select 14668,'1966-S85','1966-02-16 00:00:00','military missile','success',916,'AO-32','P-101','','','OT',435,'',180 from dual union all
select 14669,'1966-S86','1966-01-31 16:30:03','military missile','unknown',932,'A3P-388','','','','OT',421,'SSBN 641',1000 from dual union all
select 1467,'1952-S26','1952-09-16 00:00:00','military missile','success',992,'KT-11','8Zh38 KI-11','','','Test',636,'',150 from dual union all
select 14670,'1966-S87','1966-01-31 07:36:00','military missile','success',982,'G 22601-05T','8K64U UBP-68','','','OT',566,'',1210 from dual union all
select 14671,'1966-S88','1966-02-01 18:00:00','suborbital rocket','success',30,'NRL NE3.156R','Spectroheliograph','','','Solar SX',1869,'',209 from dual union all
select 14672,'1966-S89','1966-02-01 20:11:00','suborbital rocket','success',839,'NASA 10.182GM','GSFC Grenade 99 PB','','','Aeronomy',1039,'',117 from dual union all
select 14673,'1966-S90','1966-02-01 20:46:00','suborbital rocket','success',839,'NASA 10.147GM','GSFC Grenade 100 W','','','Aeronomy',1821,'',118 from dual union all
select 14674,'1966-S91','1966-02-02 02:02:00','suborbital rocket','success',839,'NASA 10.186GM','GSFC Grenade 101 C','','','Aeronomy',428,'',118 from dual union all
select 14675,'1966-S92','1966-02-02 10:04:00','suborbital rocket','success',30,'NASA 04.50UG','Spica UV spectra','','','UV Astron',1869,'',175 from dual union all
select 14676,'1966-S93','1966-02-02 00:26:00','suborbital rocket','success',1207,'SL307','LUX-33AC5/Cul-5','','','Solar X,UV',1839,'',216 from dual union all
select 14677,'1966-S94','1966-02-03 00:00:00','military missile','success',135,'C030','ECM Decoy H-3','ABRES H-3 (USAF)','','Reentry Test',614,'',200 from dual union all
select 14678,'1966-S95','1966-02-03 11:11:03','military missile','success',1420,'B-87','Mk 6 RV','OT','WINTER ICE','OT',1659,'',1300 from dual;
insert into LAUNCH
select 14679,'1966-S96','1966-02-03 18:31:00','suborbital rocket','success',839,'NASA 10.159UA','UM Falling Spheres','','','Aeronomy',1821,'',131 from dual union all
select 1468,'1952-S27','1952-09-16 00:00:00','military missile','success',992,'KT-12','8Zh38 KI-12','','','Test',636,'',150 from dual union all
select 14680,'1966-S97','1966-02-04 01:54:00','suborbital rocket','success',839,'NASA 10.143UA','UM Falling Sphere 1/2','','','Aeronomy',1821,'',152 from dual union all
select 14681,'1966-S98','1966-02-04 23:19:32','suborbital rocket','success',288,'CE-16','ESRO C15/1','','','Auroral/Ionos',29,'',120 from dual union all
select 14682,'1966-S99','1966-02-08 05:00:00','suborbital rocket','success',837,'','Sandia 154-70','','','Aeron',855,'',100 from dual union all
select 14683,'1966-U01','1966-02-05 12:19:49','orbital','success',1007,'U 22502-02L','OGCh No. 02L','OGCh No. 02L','','Sat',573,'',200 from dual union all
select 14684,'1966-U02','1966-05-19 19:30:00','orbital','success',1007,'U 22502-04L','OGCh No. 04L','OGCh No. 04L','','Sat',574,'',null from dual union all
select 14685,'1966-U03','1966-02-10 00:55:00','test rocket','success',1149,'S141C','Reentry 4B','Reentry E','','Reentry Test',1814,'',175 from dual union all
select 14686,'1966-U04','1966-02-26 16:12:01','test rocket','success',1121,'SA-201','AS-201','Apollo CM-009','','Reentry Test',267,'',488 from dual union all
select 14687,'1966-U05','1966-03-18 01:15:00','suborbital rocket','success',1439,'Meteor 8','','Meteor 8','','Reentry Test',1821,'',280 from dual union all
select 14688,'1966-U06','1966-08-25 17:15:32','test rocket','success',1465,'SA-202','AS-202','Apollo CM-011','','Reentry Test',267,'',1143 from dual union all
select 14689,'1966-U07','1966-09-17 01:57:00','suborbital rocket','success',839,'Meteor 9','','Meteor 9','','Reentry Test',1821,'',110 from dual union all
select 1469,'1952-S28','1952-09-17 00:00:00','military missile','success',992,'KT-13','8Zh38 KI-13','','','Test',636,'',150 from dual union all
select 14690,'1966-U08','1966-12-21 22:15:02','test rocket','success',175,'7001','SV-5D','SV-5D FV-1','BUSY PEACOCK','Reentry Test',1775,'',1500 from dual union all
select 14691,'1966-W01','1966-06-15 15:32:00','suborbital rocket','success',88,'EXAMETNET N09','Arcasonde 1A','','','Aeron',330,'',59 from dual union all
select 14692,'1966-W02','1966-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 14693,'1966-W03','1966-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 14694,'1966-W04','1966-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 14695,'1966-W05','1966-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 14696,'1966-W06','1966-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 14697,'1966-W07','1966-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeron',428,'',60 from dual union all
select 14698,'1966-W08','1966-01-04 12:14:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',60 from dual union all
select 14699,'1966-W09','1966-01-05 16:00:00','suborbital rocket','unknown',273,'50','','','','Aeron',293,'',75 from dual union all
select 147,'1944-S133','1944-09-23 15:44:00','military missile','success',8,'18821','Ma210','','','Training',661,'',90 from dual union all
select 1470,'1952-S29','1952-09-03 14:49:00','suborbital rocket','success',54,'NRL 11','Sunfollower/SX','','','Solar UV/X',1869,'',106 from dual union all
select 14700,'1966-W10','1966-01-06 03:00:00','suborbital rocket','success',88,'','PSU-Arcas #9','','','Ionos',1895,'',78 from dual union all
select 14701,'1966-W100','1966-02-10 23:00:00','suborbital rocket','success',716,'','','','','Weather',615,'',60 from dual union all
select 14702,'1966-W101','1966-02-10 23:40:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',35,'',60 from dual union all
select 14703,'1966-W102','1966-02-11 00:08:00','suborbital rocket','success',88,'','Sphere','','','Weather',194,'',60 from dual union all
select 14704,'1966-W103','1966-02-11 01:14:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',194,'',60 from dual union all
select 14705,'1966-W104','1966-02-11 02:14:00','suborbital rocket','success',88,'','Sphere','','','Weather',194,'',60 from dual union all
select 14706,'1966-W105','1966-02-11 03:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14707,'1966-W106','1966-02-11 03:11:00','suborbital rocket','success',88,'','Sphere','','','Weather',194,'',60 from dual union all
select 14708,'1966-W107','1966-02-11 15:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14709,'1966-W108','1966-02-11 17:13:00','suborbital rocket','success',716,'','HASP II','','','Weather',1821,'',60 from dual union all
select 1471,'1952-S30','1952-09-18 00:00:00','military missile','success',992,'KT-14','8Zh38 KI-14','','','Test',636,'',150 from dual union all
select 14710,'1966-W109','1966-02-11 18:11:00','suborbital rocket','success',716,'','Chaff','','','Weather',1895,'',60 from dual union all
select 14711,'1966-W11','1966-01-06 05:00:00','suborbital rocket','success',88,'','PSU-Arcas #10','','','Ionos',1895,'',70 from dual union all
select 14712,'1966-W110','1966-02-11 20:12:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 14713,'1966-W111','1966-02-11 22:11:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',194,'',60 from dual union all
select 14714,'1966-W112','1966-02-11 23:19:00','suborbital rocket','success',88,'','','','','Weather',194,'',60 from dual union all
select 14715,'1966-W113','1966-02-12 20:00:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',62 from dual union all
select 14716,'1966-W114','1966-02-14 15:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14717,'1966-W115','1966-02-14 15:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',60 from dual union all
select 14718,'1966-W116','1966-02-14 15:15:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',35,'',60 from dual union all
select 14719,'1966-W117','1966-02-14 16:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',618,'',60 from dual union all
select 1472,'1952-S31','1952-09-25 03:50:00','suborbital rocket','success',59,'SC 25','Grenades','','','Aeronomy',1869,'',117 from dual union all
select 14720,'1966-W118','1966-02-14 17:30:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',1895,'',60 from dual union all
select 14721,'1966-W119','1966-02-14 18:27:00','suborbital rocket','success',631,'','','','','Weather',1057,'',60 from dual union all
select 14722,'1966-W12','1966-01-08 12:14:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',50 from dual union all
select 14723,'1966-W120','1966-02-14 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 14724,'1966-W121','1966-02-14 20:00:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',62 from dual union all
select 14725,'1966-W122','1966-02-15 00:23:00','suborbital rocket','success',631,'','','','','Weather',855,'',60 from dual union all
select 14726,'1966-W123','1966-02-15 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 14727,'1966-W124','1966-02-15 20:00:00','suborbital rocket','success',716,'','Chute','','','Weather',1895,'',60 from dual union all
select 14728,'1966-W125','1966-02-16 03:30:00','suborbital rocket','success',631,'TERLS-37','ISRO 11.24','TERLS-037','','Aeron',1467,'',67 from dual union all
select 14729,'1966-W126','1966-02-16 04:30:00','suborbital rocket','success',631,'','SUPARCO 21/66','','','Aeron',1419,'',64 from dual union all
select 1473,'1952-S32','1952-10-10 14:24:00','suborbital rocket','success',50,'USAF 30','UV/Opt spectrographs','','','Solar UV',640,'',101 from dual union all
select 14730,'1966-W127','1966-02-16 13:36:00','suborbital rocket','success',631,'','SUPARCO 22/66','','','Aeron',1419,'',60 from dual union all
select 14731,'1966-W128','1966-02-16 15:10:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14732,'1966-W129','1966-02-16 15:45:00','suborbital rocket','success',88,'','','','','Weather',1895,'',60 from dual union all
select 14733,'1966-W13','1966-01-12 00:00:00','suborbital rocket','unknown',273,'51','','','','Aeron',293,'',75 from dual union all
select 14734,'1966-W130','1966-02-16 16:29:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',60 from dual union all
select 14735,'1966-W131','1966-02-16 18:00:00','suborbital rocket','success',88,'','','','','Weather',1895,'',60 from dual union all
select 14736,'1966-W132','1966-02-16 18:05:00','suborbital rocket','success',716,'','Chaff','','','Weather',1895,'',60 from dual union all
select 14737,'1966-W133','1966-02-16 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 14738,'1966-W134','1966-02-17 00:02:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',855,'',60 from dual union all
select 14739,'1966-W135','1966-02-17 00:32:00','suborbital rocket','success',631,'','','','','Weather',873,'',60 from dual union all
select 1474,'1952-S33','1952-10-23 03:45:00','suborbital rocket','success',60,'SC 26','Grenades','','','Aeronomy',1869,'',112 from dual union all
select 14740,'1966-W136','1966-02-17 16:51:00','suborbital rocket','success',631,'','','','','Weather',1057,'',60 from dual union all
select 14741,'1966-W137','1966-02-17 18:15:00','suborbital rocket','success',631,'EXAMETNET W02','Chaff','','','Weather',1821,'',67 from dual union all
select 14742,'1966-W138','1966-02-17 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 14743,'1966-W139','1966-02-17 21:04:00','suborbital rocket','success',716,'','Chute','','','Weather',1895,'',60 from dual union all
select 14744,'1966-W14','1966-01-12 15:23:00','suborbital rocket','success',631,'EXAMETNET W01','Chaff','','','Aeron',1821,'',60 from dual union all
select 14745,'1966-W140','1966-02-18 00:45:00','suborbital rocket','success',716,'','','','','Weather',615,'',60 from dual union all
select 14746,'1966-W141','1966-02-18 05:35:00','suborbital rocket','success',716,'','','','','Weather',615,'',60 from dual union all
select 14747,'1966-W142','1966-02-18 14:30:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14748,'1966-W143','1966-02-18 15:00:00','suborbital rocket','success',716,'','Chute','','','Weather',1895,'',60 from dual union all
select 14749,'1966-W144','1966-02-18 17:20:00','suborbital rocket','success',631,'','','','','Weather',1057,'',60 from dual union all
select 1475,'1952-S34','1952-10-22 14:35:00','suborbital rocket','success',50,'USAF 31','T-day','','','Aeronomy',640,'',100 from dual union all
select 14750,'1966-W145','1966-02-18 18:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',1895,'',60 from dual union all
select 14751,'1966-W146','1966-02-18 18:05:00','suborbital rocket','success',716,'','Chaff','','','Weather',1895,'',60 from dual union all
select 14752,'1966-W147','1966-02-18 18:35:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',618,'',60 from dual union all
select 14753,'1966-W148','1966-02-18 19:20:00','suborbital rocket','success',631,'','','','','Weather',855,'',60 from dual union all
select 14754,'1966-W149','1966-02-18 19:50:00','suborbital rocket','success',716,'','HASP II','','','Weather',1821,'',60 from dual union all
select 14755,'1966-W15','1966-01-12 12:30:00','suborbital rocket','success',631,'','SUPARCO 19/66','','','Aeron',1419,'',58 from dual union all
select 14756,'1966-W150','1966-02-18 20:00:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',64 from dual union all
select 14757,'1966-W151','1966-02-18 20:34:00','suborbital rocket','success',88,'','','','','Weather',617,'',60 from dual union all
select 14758,'1966-W152','1966-02-19 00:00:00','suborbital rocket','unknown',273,'54','','','','Aeron',293,'',75 from dual union all
select 14759,'1966-W153','1966-02-19 10:55:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',1057,'',60 from dual union all
select 1476,'1952-S35','1952-10-29 00:00:00','military missile','success',970,'No 420','','','','OT',636,'',100 from dual union all
select 14760,'1966-W154','1966-02-19 13:12:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',60 from dual union all
select 14761,'1966-W155','1966-02-19 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 14762,'1966-W156','1966-02-20 13:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',60 from dual union all
select 14763,'1966-W157','1966-02-20 20:00:00','suborbital rocket','success',88,'','','','','Weather',617,'',60 from dual union all
select 14764,'1966-W158','1966-02-21 13:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14765,'1966-W159','1966-02-21 13:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',60 from dual union all
select 14766,'1966-W16','1966-01-12 15:00:00','suborbital rocket','success',631,'EXAMETNET N01','04 CNAE 6601','','','Aeron',330,'',59 from dual union all
select 14767,'1966-W160','1966-02-21 14:30:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14768,'1966-W161','1966-02-21 14:45:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',35,'',60 from dual;
insert into LAUNCH
select 14769,'1966-W162','1966-02-21 15:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',618,'',60 from dual union all
select 1477,'1952-S36','1952-10-30 00:00:00','military missile','success',970,'No 501','','','','OT',636,'',100 from dual union all
select 14770,'1966-W163','1966-02-21 15:30:00','suborbital rocket','success',716,'','Chaff','','','Weather',1895,'',60 from dual union all
select 14771,'1966-W164','1966-02-21 22:00:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',64 from dual union all
select 14772,'1966-W165','1966-02-22 11:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 14773,'1966-W166','1966-02-22 13:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14774,'1966-W167','1966-02-22 13:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',60 from dual union all
select 14775,'1966-W168','1966-02-22 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 14776,'1966-W169','1966-02-22 22:45:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14777,'1966-W17','1966-01-15 00:00:00','suborbital rocket','success',631,'','','','','Weather',873,'',60 from dual union all
select 14778,'1966-W170','1966-02-22 22:55:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',60 from dual union all
select 14779,'1966-W171','1966-02-23 13:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 1478,'1952-S37','1952-10-30 00:00:00','military missile','success',970,'No 520','','','','OT',636,'',100 from dual union all
select 14780,'1966-W172','1966-02-23 14:46:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',35,'',60 from dual union all
select 14781,'1966-W173','1966-02-23 15:04:00','suborbital rocket','success',716,'','HASP II','','','Weather',1821,'',60 from dual union all
select 14782,'1966-W174','1966-02-23 18:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',1895,'',60 from dual union all
select 14783,'1966-W175','1966-02-23 18:05:00','suborbital rocket','success',716,'','Chaff','','','Weather',1895,'',60 from dual union all
select 14784,'1966-W176','1966-02-23 18:17:00','suborbital rocket','success',631,'','','','','Weather',1821,'',60 from dual union all
select 14785,'1966-W177','1966-02-23 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 14786,'1966-W178','1966-02-23 20:00:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',65 from dual union all
select 14787,'1966-W179','1966-02-23 20:22:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',855,'',60 from dual union all
select 14788,'1966-W18','1966-01-17 12:22:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',55 from dual union all
select 14789,'1966-W180','1966-02-23 23:20:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',1057,'',60 from dual union all
select 1479,'1952-S38','1952-11-21 00:00:00','military missile','success',970,'No 620','','','','OT',636,'',100 from dual union all
select 14790,'1966-W181','1966-02-24 00:06:00','suborbital rocket','success',631,'','','','','Weather',873,'',60 from dual union all
select 14791,'1966-W182','1966-02-24 12:45:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14792,'1966-W183','1966-02-24 12:45:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',60 from dual union all
select 14793,'1966-W184','1966-02-25 12:45:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14794,'1966-W185','1966-02-25 13:21:00','suborbital rocket','success',88,'','','','','Weather',276,'',60 from dual union all
select 14795,'1966-W186','1966-02-25 14:30:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',60 from dual union all
select 14796,'1966-W187','1966-02-25 16:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',35,'',60 from dual union all
select 14797,'1966-W188','1966-02-25 18:30:00','suborbital rocket','unknown',273,'55','','','','Aeron',293,'',75 from dual union all
select 14798,'1966-W189','1966-02-25 19:41:00','suborbital rocket','success',631,'','','','','Weather',1821,'',60 from dual union all
select 14799,'1966-W19','1966-01-18 02:10:00','suborbital rocket','unknown',759,'MT-135-012','','','','Aeron',839,'',54 from dual union all
select 148,'1944-S134','1944-09-23 18:26:00','military missile','success',8,'18831','Ma217','','','Training',661,'',90 from dual union all
select 1480,'1952-S39','1952-12-11 23:47:00','suborbital rocket','success',59,'SC 29','Sphere','','','Aeronomy',1869,'',105 from dual union all
select 14800,'1966-W190','1966-02-25 22:45:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14801,'1966-W191','1966-02-25 22:45:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',60 from dual union all
select 14802,'1966-W192','1966-02-26 00:45:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14803,'1966-W193','1966-02-26 02:45:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14804,'1966-W194','1966-02-26 17:12:00','suborbital rocket','success',88,'','Arcasonde 1A Chute','','','Weather',276,'',60 from dual union all
select 14805,'1966-W195','1966-02-26 18:12:00','suborbital rocket','unknown',273,'56','','','','Aeron',293,'',75 from dual union all
select 14806,'1966-W196','1966-02-26 18:50:00','suborbital rocket','success',88,'AS-201 support','Arcasonde 1A','','','Meteo',176,'',60 from dual union all
select 14807,'1966-W197','1966-02-26 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 14808,'1966-W198','1966-02-26 20:00:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',57 from dual union all
select 14809,'1966-W199','1966-02-27 18:45:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 1481,'1952-S40','1952-12-12 19:38:00','suborbital rocket','success',50,'USAF 33','Colorado/UVS','','','Solar UV',640,'',88 from dual union all
select 14810,'1966-W20','1966-01-19 00:00:00','suborbital rocket','unknown',273,'52','','','','Aeron',293,'',75 from dual union all
select 14811,'1966-W200','1966-02-27 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 14812,'1966-W201','1966-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14813,'1966-W202','1966-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14814,'1966-W203','1966-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14815,'1966-W204','1966-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14816,'1966-W205','1966-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14817,'1966-W206','1966-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14818,'1966-W207','1966-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14819,'1966-W208','1966-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1482,'1952-S41','1952-12-15 21:38:00','suborbital rocket','success',1500,'9','Viking 9','','','Solar X',1889,'',217 from dual union all
select 14820,'1966-W209','1966-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14821,'1966-W21','1966-01-19 02:00:00','suborbital rocket','unknown',759,'MT-135-013','','','','Aeron',839,'',59 from dual union all
select 14822,'1966-W210','1966-02-28 06:30:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14823,'1966-W211','1966-02-28 14:45:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',35,'',60 from dual union all
select 14824,'1966-W212','1966-02-28 15:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 14825,'1966-W213','1966-02-28 16:51:00','suborbital rocket','success',631,'','','','','Weather',1057,'',60 from dual union all
select 14826,'1966-W214','1966-02-28 18:00:00','suborbital rocket','success',716,'','Chaff','','','Weather',1895,'',60 from dual union all
select 14827,'1966-W215','1966-02-28 20:00:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',65 from dual union all
select 14828,'1966-W216','1966-02-28 22:30:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 14829,'1966-W217','1966-03-02 15:48:00','suborbital rocket','success',631,'EXAMETNET N03','07 CNAE 6604 Chaff','','','Aeron',330,'',60 from dual union all
select 1483,'1952-S42','1952-10-15 00:00:00','military missile','success',995,'RUD 1','','','','Test',636,'',150 from dual union all
select 14830,'1966-W218','1966-03-02 20:00:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',65 from dual union all
select 14831,'1966-W219','1966-03-04 20:00:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',64 from dual union all
select 14832,'1966-W22','1966-01-19 09:30:00','suborbital rocket','success',631,'TERLS-36','ISRO 11.23','TERLS-036','','Aeron',1467,'',67 from dual union all
select 14833,'1966-W220','1966-03-07 20:00:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',68 from dual union all
select 14834,'1966-W221','1966-03-08 12:07:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',55 from dual union all
select 14835,'1966-W222','1966-03-09 20:00:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',65 from dual union all
select 14836,'1966-W223','1966-03-16 12:00:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',59 from dual union all
select 14837,'1966-W224','1966-03-16 12:10:00','suborbital rocket','success',631,'TERLS-38','ISRO 11.25','TERLS-038','','Aeron',1467,'',62 from dual union all
select 14838,'1966-W225','1966-03-16 15:00:00','suborbital rocket','success',631,'EXAMETNET N04','08 CNAE 6605 Chaff','','','Aeron',330,'',64 from dual union all
select 14839,'1966-W226','1966-03-16 15:10:00','suborbital rocket','unknown',631,'SUPARCO  23','','','','Aeron',1419,'',62 from dual union all
select 1484,'1952-S43','1952-10-18 00:00:00','military missile','success',995,'RUD 2','','','','Test',636,'',150 from dual union all
select 14840,'1966-W227','1966-03-25 05:39:00','suborbital rocket','unknown',631,'SUPARCO  24','','','','Aeron',1419,'',66 from dual union all
select 14841,'1966-W228','1966-04-01 00:00:00','suborbital rocket','failure',88,'','Gerdien condenser','','','Aeron',428,'',10 from dual union all
select 14842,'1966-W229','1966-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14843,'1966-W23','1966-01-19 12:30:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',70 from dual union all
select 14844,'1966-W230','1966-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14845,'1966-W231','1966-04-15 19:03:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14846,'1966-W232','1966-04-13 07:30:00','suborbital rocket','success',631,'TERLS-43','ISRO 11.26','TERLS-043','','Aeron',1467,'',69 from dual union all
select 14847,'1966-W233','1966-04-13 12:57:00','suborbital rocket','unknown',631,'SUPARCO-025/66','','','','Aeron',1419,'',62 from dual union all
select 14848,'1966-W234','1966-04-18 21:53:00','suborbital rocket','unknown',1188,'','','','','Aeron',1447,'',69 from dual union all
select 14849,'1966-W235','1966-04-20 18:34:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',52 from dual union all
select 1485,'1952-S44','1952-10-19 00:00:00','military missile','success',995,'RUD 3','','','','Test',636,'',150 from dual union all
select 14850,'1966-W236','1966-04-21 22:03:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',68 from dual union all
select 14851,'1966-W237','1966-04-27 12:08:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',78 from dual union all
select 14852,'1966-W238','1966-04-27 21:54:00','suborbital rocket','success',631,'EXAMETNET C01','Chaff','','','Weather',310,'',60 from dual union all
select 14853,'1966-W239','1966-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14854,'1966-W24','1966-01-24 06:40:00','suborbital rocket','success',88,'','Arcasonde IA /10.185','','','Aeron',428,'',60 from dual union all
select 14855,'1966-W240','1966-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14856,'1966-W241','1966-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14857,'1966-W242','1966-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14858,'1966-W243','1966-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual;
insert into LAUNCH
select 14859,'1966-W244','1966-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1486,'1952-S45','1952-10-23 00:00:00','military missile','success',995,'RUD 4','','','','Test',636,'',150 from dual union all
select 14860,'1966-W245','1966-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14861,'1966-W246','1966-04-30 02:20:00','suborbital rocket','unknown',759,'MT-135-015','','','','Aeron',839,'',53 from dual union all
select 14862,'1966-W247','1966-04-30 10:40:00','suborbital rocket','unknown',759,'MT-135-016','','','','Aeron',839,'',54 from dual union all
select 14863,'1966-W248','1966-05-02 01:55:00','suborbital rocket','success',631,'EXAMETNET N06','13 CNAE 6610 WOX-1A','','','Aeron',330,'',52 from dual union all
select 14864,'1966-W249','1966-05-02 02:00:00','suborbital rocket','unknown',759,'MT-135-017','','','','Aeron',839,'',55 from dual union all
select 14865,'1966-W25','1966-01-25 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',75 from dual union all
select 14866,'1966-W250','1966-05-02 03:30:00','suborbital rocket','success',88,'','Arcasonde 1A /10.192','','','Aeron',428,'',60 from dual union all
select 14867,'1966-W251','1966-05-04 01:00:00','suborbital rocket','success',88,'','Arcasonde 1A /10.193','','','Aeron',428,'',60 from dual union all
select 14868,'1966-W252','1966-05-10 20:45:00','suborbital rocket','success',88,'','','','','Aeron',1821,'',60 from dual union all
select 14869,'1966-W253','1966-05-11 03:06:04','suborbital rocket','success',88,'','','','','Aeron',1821,'',60 from dual union all
select 1487,'1952-S46','1952-10-27 00:00:00','military missile','success',995,'RUD 5','','','','Test',636,'',150 from dual union all
select 14870,'1966-W254','1966-05-12 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1821,'',60 from dual union all
select 14871,'1966-W255','1966-05-13 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1821,'',60 from dual union all
select 14872,'1966-W256','1966-05-16 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',62 from dual union all
select 14873,'1966-W257','1966-05-18 14:15:00','suborbital rocket','unknown',631,'SUPARCO-026/66','','','','Aeron',1419,'',62 from dual union all
select 14874,'1966-W258','1966-05-18 15:50:00','suborbital rocket','success',631,'EXAMETNET N07','14 CNAE 6611 WOX-1A','','','Aeron',330,'',53 from dual union all
select 14875,'1966-W259','1966-05-18 18:42:00','suborbital rocket','unknown',631,'EXAMETNET C02','M-85 Chaff','','','Aeron',310,'',66 from dual union all
select 14876,'1966-W26','1966-01-26 12:18:00','suborbital rocket','success',631,'','SUPARCO 20/66','','','Aeron',1419,'',61 from dual union all
select 14877,'1966-W260','1966-05-18 20:22:00','suborbital rocket','success',631,'EXAMETNET C03','10-1449  WOX-1A','','','Aeron',310,'',53 from dual union all
select 14878,'1966-W261','1966-05-24 23:00:00','suborbital rocket','unknown',716,'SR-031','','','','Aeron',1895,'',75 from dual union all
select 14879,'1966-W262','1966-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1488,'1953-A01','1953-01-01 00:00:00','atmospheric rocket','success',365,'','F-100 Model','','','Test',1821,'',10 from dual union all
select 14880,'1966-W263','1966-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14881,'1966-W264','1966-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14882,'1966-W265','1966-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14883,'1966-W266','1966-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14884,'1966-W267','1966-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14885,'1966-W268','1966-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14886,'1966-W269','1966-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14887,'1966-W27','1966-01-27 00:00:00','suborbital rocket','success',631,'','','','','Weather',873,'',60 from dual union all
select 14888,'1966-W270','1966-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14889,'1966-W271','1966-06-01 16:31:00','suborbital rocket','success',88,'EXAMETNET N08','15 CNAE 6612 Arc-1A','','','Aeron',330,'',55 from dual union all
select 1489,'1953-A02','1953-01-01 00:00:00','atmospheric rocket','success',365,'','D35 Model','','','Test',1821,'',10 from dual union all
select 14890,'1966-W272','1966-06-03 08:48:00','suborbital rocket','success',1188,'SA-1','','','','Aeron',1447,'',70 from dual union all
select 14891,'1966-W273','1966-06-13 12:07:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',52 from dual union all
select 14892,'1966-W274','1966-06-14 12:06:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',60 from dual union all
select 14893,'1966-W275','1966-06-15 14:05:00','suborbital rocket','unknown',631,'SUPARCO-027/66','','','','Aeron',1419,'',64 from dual union all
select 14894,'1966-W276','1966-06-15 19:43:00','suborbital rocket','unknown',631,'EXAMETNET C04','10-1450 WOX-1A','','','Aeron',310,'',75 from dual union all
select 14895,'1966-W277','1966-06-17 04:23:00','suborbital rocket','success',88,'','Arcasonde 1A /10.198','','','Aeron',428,'',60 from dual union all
select 14896,'1966-W278','1966-06-18 00:55:00','suborbital rocket','unknown',716,'NA-HESTER','','','','Aeron',394,'',55 from dual union all
select 14897,'1966-W279','1966-06-23 07:24:00','suborbital rocket','success',88,'','Arcasonde 1A /10.199','','','Aeron',428,'',60 from dual union all
select 14898,'1966-W28','1966-01-27 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',50 from dual union all
select 14899,'1966-W280','1966-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 149,'1944-S135','1944-09-24 11:23:00','military missile','success',8,'18827','Ma216','','','Training',661,'',90 from dual union all
select 1490,'1953-A03','1953-01-01 00:00:00','atmospheric rocket','success',564,'','D32 Model','','','Test',1821,'',5 from dual union all
select 14900,'1966-W281','1966-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14901,'1966-W282','1966-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14902,'1966-W283','1966-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14903,'1966-W284','1966-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14904,'1966-W285','1966-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14905,'1966-W286','1966-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14906,'1966-W287','1966-07-03 19:53:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14907,'1966-W288','1966-07-04 00:00:00','suborbital rocket','unknown',273,'57','','','','Aeron',293,'',75 from dual union all
select 14908,'1966-W289','1966-07-06 00:00:00','suborbital rocket','unknown',273,'58','','','','Aeron',293,'',75 from dual union all
select 14909,'1966-W29','1966-01-27 17:05:00','suborbital rocket','unknown',273,'53','','','','Aeron',293,'',75 from dual union all
select 1491,'1953-A04','1953-02-03 00:00:00','atmospheric rocket','success',836,'303P','Nike CET 3','','','Test',1895,'',10 from dual union all
select 14910,'1966-W290','1966-07-13 07:30:00','suborbital rocket','unknown',631,'SUPARCO-028/66','','','','Aeron',1419,'',60 from dual union all
select 14911,'1966-W291','1966-07-13 22:44:00','suborbital rocket','unknown',631,'EXAMETNET C05','10-1448  WOX-1A','','','Aeron',310,'',75 from dual union all
select 14912,'1966-W292','1966-07-14 01:18:00','suborbital rocket','unknown',631,'EXAMETNET C06','P-467','','','Aeron',310,'',75 from dual union all
select 14913,'1966-W293','1966-07-14 15:17:00','suborbital rocket','unknown',631,'EXAMETNET C07','P-462    WOX-1A','','','Aeron',310,'',75 from dual union all
select 14914,'1966-W294','1966-07-15 05:14:00','suborbital rocket','success',725,'','','','','Meteo',636,'',74 from dual union all
select 14915,'1966-W295','1966-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14916,'1966-W296','1966-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14917,'1966-W297','1966-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14918,'1966-W298','1966-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14919,'1966-W299','1966-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1492,'1953-A05','1953-01-05 00:00:00','atmospheric rocket','unknown',576,'79','','','','Test',1895,'',10 from dual union all
select 14920,'1966-W30','1966-01-29 12:25:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',60 from dual union all
select 14921,'1966-W300','1966-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14922,'1966-W301','1966-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14923,'1966-W302','1966-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14924,'1966-W303','1966-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14925,'1966-W304','1966-08-07 10:55:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14926,'1966-W305','1966-08-07 18:31:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14927,'1966-W306','1966-08-17 14:30:00','suborbital rocket','unknown',631,'SUPARCO-029/66','','','','Aeron',1419,'',62 from dual union all
select 14928,'1966-W307','1966-08-18 02:00:00','suborbital rocket','unknown',759,'MT-135-018','','','','Aeron',839,'',52 from dual union all
select 14929,'1966-W308','1966-08-24 16:30:00','suborbital rocket','unknown',273,'59','','','','Aeron',293,'',75 from dual union all
select 1493,'1953-A06','1953-01-08 00:00:00','atmospheric rocket','success',836,'1249B','Nike Eng 5E','','','Test',1895,'',10 from dual union all
select 14930,'1966-W309','1966-08-25 19:08:00','suborbital rocket','unknown',273,'60','','','','Aeron',293,'',75 from dual union all
select 14931,'1966-W31','1966-02-01 08:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',428,'',60 from dual union all
select 14932,'1966-W310','1966-08-26 06:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',50 from dual union all
select 14933,'1966-W311','1966-09-07 05:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14934,'1966-W312','1966-09-07 09:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14935,'1966-W313','1966-09-07 13:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14936,'1966-W314','1966-09-07 17:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14937,'1966-W315','1966-09-07 21:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14938,'1966-W316','1966-09-08 01:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14939,'1966-W317','1966-09-08 05:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1494,'1953-A07','1953-01-08 00:00:00','atmospheric rocket','success',836,'1249A-163','Nike 1249 R&D 159R','','','Test',1895,'',10 from dual union all
select 14940,'1966-W318','1966-09-08 09:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14941,'1966-W319','1966-09-08 13:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14942,'1966-W32','1966-02-01 12:25:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',50 from dual union all
select 14943,'1966-W320','1966-09-08 17:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14944,'1966-W321','1966-09-08 21:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14945,'1966-W322','1966-09-09 01:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14946,'1966-W323','1966-09-09 05:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14947,'1966-W324','1966-09-09 09:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14948,'1966-W325','1966-09-09 13:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual;
insert into LAUNCH
select 14949,'1966-W326','1966-09-09 17:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1495,'1953-A08','1953-01-08 00:00:00','atmospheric rocket','success',836,'1249A-161','Nike 1249 R&D 158R','','','Test',1895,'',10 from dual union all
select 14950,'1966-W327','1966-09-09 21:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14951,'1966-W328','1966-09-10 01:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14952,'1966-W329','1966-09-10 05:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14953,'1966-W33','1966-02-01 21:15:00','suborbital rocket','success',1186,'','','','','Aeron',1895,'',60 from dual union all
select 14954,'1966-W330','1966-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14955,'1966-W331','1966-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14956,'1966-W332','1966-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14957,'1966-W333','1966-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14958,'1966-W334','1966-09-08 18:41:00','suborbital rocket','success',631,'EXAMETNET C09','WOX-1A','','','Aeron',310,'',60 from dual union all
select 14959,'1966-W335','1966-09-14 19:15:00','suborbital rocket','success',631,'EXAMETNET C10','WOX-1A','','','Aeron',310,'',61 from dual union all
select 1496,'1953-A09','1953-01-12 00:00:00','atmospheric rocket','unknown',576,'80','','','','Test',1895,'',10 from dual union all
select 14960,'1966-W336','1966-09-21 02:10:00','suborbital rocket','unknown',759,'DT-135-1','','','','Test',839,'',56 from dual union all
select 14961,'1966-W337','1966-09-21 16:40:00','suborbital rocket','success',631,'EXAMETNET C11','WOX-1A','','','Aeron',310,'',62 from dual union all
select 14962,'1966-W338','1966-09-27 20:37:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',59 from dual union all
select 14963,'1966-W339','1966-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14964,'1966-W34','1966-02-01 22:11:00','suborbital rocket','success',88,'','','','','Weather',1821,'',60 from dual union all
select 14965,'1966-W340','1966-09-28 00:00:00','suborbital rocket','success',88,'','Gerdien Condenser','','','Aeron',428,'',60 from dual union all
select 14966,'1966-W341','1966-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14967,'1966-W342','1966-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14968,'1966-W343','1966-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14969,'1966-W344','1966-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1497,'1953-A10','1953-01-12 00:00:00','atmospheric rocket','unknown',576,'81','','','','Test',1895,'',10 from dual union all
select 14970,'1966-W345','1966-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14971,'1966-W346','1966-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14972,'1966-W347','1966-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14973,'1966-W348','1966-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14974,'1966-W349','1966-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14975,'1966-W35','1966-02-02 03:00:00','suborbital rocket','success',88,'','Arcasonde 1A /10.186','','','Weather',428,'',60 from dual union all
select 14976,'1966-W350','1966-10-12 14:20:00','suborbital rocket','unknown',631,'SUPARCO-031/66','','','','Aeron',1419,'',69 from dual union all
select 14977,'1966-W351','1966-10-13 04:20:00','suborbital rocket','unknown',759,'MT-135-A -020','','','','Aeron',839,'',65 from dual union all
select 14978,'1966-W352','1966-10-13 23:30:00','suborbital rocket','success',631,'EXAMETNET C13','Chaff','','','Aeron',310,'',70 from dual union all
select 14979,'1966-W353','1966-10-24 07:36:59','suborbital rocket','success',725,'','','','','Meteo',636,'',58 from dual union all
select 1498,'1953-A100','1953-03-05 00:00:00','atmospheric rocket','success',836,'1052E','','','','Test',1895,'',10 from dual union all
select 14980,'1966-W354','1966-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14981,'1966-W355','1966-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14982,'1966-W356','1966-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14983,'1966-W357','1966-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14984,'1966-W358','1966-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14985,'1966-W359','1966-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14986,'1966-W36','1966-02-02 05:00:00','suborbital rocket','success',88,'','','','','Weather',1895,'',60 from dual union all
select 14987,'1966-W360','1966-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14988,'1966-W361','1966-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 14989,'1966-W362','1966-10-31 07:12:59','suborbital rocket','success',725,'','','','','Meteo',636,'',73 from dual union all
select 1499,'1953-A101','1953-03-06 00:00:00','atmospheric rocket','unknown',576,'340','','','','Test',1895,'',10 from dual union all
select 14990,'1966-W363','1966-11-05 13:55:00','suborbital rocket','success',1215,'D-4C','','','','Ionos',221,'',65 from dual union all
select 14991,'1966-W364','1966-11-12 13:38:00','suborbital rocket','success',1215,'D-3C','','','','Eclipse',221,'',65 from dual union all
select 14992,'1966-W365','1966-11-12 14:08:00','suborbital rocket','success',1215,'D-10C','','','','Eclipse',221,'',65 from dual union all
select 14993,'1966-W366','1966-11-12 14:50:00','suborbital rocket','success',1215,'D-15C','','','','Eclipse',221,'',65 from dual union all
select 14994,'1966-W367','1966-11-12 14:55:00','suborbital rocket','success',1215,'D-4','','','','Eclipse',221,'',65 from dual union all
select 14995,'1966-W368','1966-11-12 15:08:00','suborbital rocket','success',1215,'D-11','','','','Eclipse',221,'',65 from dual union all
select 14996,'1966-W369','1966-11-12 15:22:00','suborbital rocket','success',1215,'D-13','','','','Eclipse',221,'',65 from dual union all
select 14997,'1966-W37','1966-02-02 12:23:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',60 from dual union all
select 14998,'1966-W370','1966-11-12 15:35:00','suborbital rocket','success',1215,'D-16C','','','','Eclipse',221,'',65 from dual union all
select 14999,'1966-W371','1966-11-16 14:01:00','suborbital rocket','unknown',631,'SUPARCO-032/66','','','','Aeron',1419,'',64 from dual union all
select 15,'1943-A08','1943-06-22 00:00:00','military missile','failure',8,'28','','','','Test',675,'',5 from dual union all
select 150,'1944-S136','1944-09-25 00:00:00','military missile','success',1470,'','Lille','','','Weapon',1735,'',95 from dual union all
select 1500,'1953-A102','1953-03-10 00:00:00','atmospheric rocket','success',836,'1249A-189','Nike 1249 R&D 188R','','','Test',1895,'',10 from dual union all
select 15000,'1966-W372','1966-11-16 15:30:00','suborbital rocket','success',631,'EXAMETNET C14','WOX-1A','','','Aeron',1460,'',61 from dual union all
select 15001,'1966-W373','1966-11-18 16:44:00','suborbital rocket','unknown',197,'','','','','Aeron',671,'',50 from dual union all
select 15002,'1966-W374','1966-11-19 16:44:00','suborbital rocket','unknown',197,'','','','','Aeron',671,'',50 from dual union all
select 15003,'1966-W375','1966-11-20 16:53:00','suborbital rocket','unknown',197,'','','','','Aeron',671,'',50 from dual union all
select 15004,'1966-W376','1966-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 15005,'1966-W377','1966-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 15006,'1966-W378','1966-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 15007,'1966-W379','1966-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 15008,'1966-W38','1966-02-02 15:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 15009,'1966-W380','1966-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1501,'1953-A103','1953-03-10 00:00:00','military missile','success',763,'N-25 GM-2337','','','','Test',222,'',10 from dual union all
select 15010,'1966-W381','1966-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 15011,'1966-W382','1966-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 15012,'1966-W383','1966-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 15013,'1966-W384','1966-12-10 03:37:00','suborbital rocket','success',88,'','Scanner 2 support','','','Weather',35,'',56 from dual union all
select 15014,'1966-W385','1966-12-10 03:37:00','suborbital rocket','unknown',716,'','','','','Aeron',35,'',75 from dual union all
select 15015,'1966-W386','1966-12-10 03:52:00','suborbital rocket','success',88,'','Scanner 2 support','','','Weather',1821,'',55 from dual union all
select 15016,'1966-W387','1966-12-10 04:00:00','suborbital rocket','success',88,'','Scanner 2 support','','','Weather',1895,'',56 from dual union all
select 15017,'1966-W388','1966-12-10 06:28:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',75 from dual union all
select 15018,'1966-W389','1966-12-10 06:30:00','suborbital rocket','success',88,'','Scanner 2 support','','','Weather',1363,'',65 from dual union all
select 15019,'1966-W39','1966-02-02 15:25:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',35,'',60 from dual union all
select 1502,'1953-A104','1953-03-11 00:00:00','atmospheric rocket','unknown',576,'304','','','','Test',1895,'',10 from dual union all
select 15020,'1966-W390','1966-12-10 06:30:00','suborbital rocket','unknown',716,'','','','','Aeron',1363,'',75 from dual union all
select 15021,'1966-W391','1966-12-10 09:00:00','suborbital rocket','success',88,'','Scanner 2 support','','','Weather',428,'',54 from dual union all
select 15022,'1966-W392','1966-12-10 18:12:00','suborbital rocket','success',88,'','PEPP support','','','Weather',1895,'',60 from dual union all
select 15023,'1966-W393','1966-12-14 06:30:00','suborbital rocket','success',725,'','','','','Meteo',636,'',50 from dual union all
select 15024,'1966-W394','1966-12-14 14:07:00','suborbital rocket','success',631,'EXAMETNET C14A','','','','Aeron',310,'',60 from dual union all
select 15025,'1966-W395','1966-12-14 20:25:00','suborbital rocket','success',631,'EXAMETNET C15','WOX-1A','','','Aeron',310,'',60 from dual union all
select 15026,'1966-W396','1966-12-15 06:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',70 from dual union all
select 15027,'1966-W397','1966-12-19 15:01:59','suborbital rocket','success',725,'','','','','Meteo',1544,'SHIP-SOV',52 from dual union all
select 15028,'1966-W398','1966-12-21 11:21:00','suborbital rocket','success',725,'','','','','Meteo',1550,'SHIP-SOV',78 from dual union all
select 15029,'1966-W399','1966-12-22 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1505,'SHIP-SOV',50 from dual union all
select 1503,'1953-A105','1953-03-11 00:00:00','atmospheric rocket','unknown',576,'305','','','','Test',1895,'',10 from dual union all
select 15030,'1966-W40','1966-02-02 15:45:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',1895,'',60 from dual union all
select 15031,'1966-W400','1966-12-23 09:59:59','suborbital rocket','success',725,'','','','','Meteo',1504,'SHIP-SOV',79 from dual union all
select 15032,'1966-W401','1966-12-25 08:47:00','suborbital rocket','success',725,'','','','','Meteo',1545,'SHIP-SOV',50 from dual union all
select 15033,'1966-W402','1966-12-25 09:59:59','suborbital rocket','success',725,'','','','','Meteo',1503,'SHIP-SOV',78 from dual union all
select 15034,'1966-W403','1966-12-27 20:46:00','suborbital rocket','success',725,'','','','','Meteo',1547,'SHIP-SOV',78 from dual union all
select 15035,'1966-W404','1966-12-28 06:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',70 from dual union all
select 15036,'1966-W405','1966-12-29 08:30:00','suborbital rocket','success',725,'','','','','Meteo',1548,'SHIP-SOV',78 from dual union all
select 15037,'1966-W406','1966-12-29 09:59:59','suborbital rocket','success',725,'','','','','Meteo',1502,'SHIP-SOV',60 from dual;
insert into LAUNCH
select 15038,'1966-W407','1966-12-13 19:00:00','suborbital rocket','success',88,'','PSU-Arcas #16','','','Ionos',1895,'',70 from dual union all
select 15039,'1966-W408','1966-04-11 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 1504,'1953-A106','1953-03-12 00:00:00','military missile','unknown',339,'57RS','','','','Test',1895,'',30 from dual union all
select 15040,'1966-W409','1966-04-11 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15041,'1966-W41','1966-02-02 16:00:00','suborbital rocket','success',716,'','Chaff','','','Weather',1895,'',60 from dual union all
select 15042,'1966-W410','1966-04-11 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15043,'1966-W411','1966-04-11 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15044,'1966-W412','1966-04-11 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15045,'1966-W413','1966-04-12 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15046,'1966-W414','1966-04-12 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15047,'1966-W415','1966-04-12 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15048,'1966-W416','1966-04-12 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15049,'1966-W417','1966-04-12 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 1505,'1953-A107','1953-03-12 00:00:00','atmospheric rocket','unknown',576,'345','','','','Test',1895,'',10 from dual union all
select 15050,'1966-W418','1966-04-12 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15051,'1966-W419','1966-04-12 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15052,'1966-W42','1966-02-02 17:30:00','suborbital rocket','success',88,'','Sphere','','','Weather',1057,'',60 from dual union all
select 15053,'1966-W420','1966-04-12 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15054,'1966-W421','1966-04-12 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15055,'1966-W422','1966-04-12 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15056,'1966-W423','1966-04-12 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15057,'1966-W424','1966-04-12 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15058,'1966-W425','1966-04-12 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15059,'1966-W426','1966-04-13 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 1506,'1953-A108','1953-03-12 00:00:00','atmospheric rocket','success',836,'314P','Nike CET 14','','','Test',1895,'',10 from dual union all
select 15060,'1966-W427','1966-04-13 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15061,'1966-W428','1966-04-13 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15062,'1966-W429','1966-04-13 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15063,'1966-W43','1966-02-02 20:17:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 15064,'1966-W430','1966-04-13 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15065,'1966-W431','1966-04-13 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',176,'',60 from dual union all
select 15066,'1966-W432','1966-07-05 16:00:00','suborbital rocket','success',88,'','Chute','','','Weather',276,'',60 from dual union all
select 15067,'1966-W433','1966-08-26 15:00:00','suborbital rocket','success',88,'','Chute','','','Weather',276,'',60 from dual union all
select 15068,'1966-W434','1966-01-20 15:43:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15069,'1966-W435','1966-03-18 15:17:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 1507,'1953-A109','1953-03-12 00:00:00','atmospheric rocket','success',836,'315P','Nike CET 15','','','Test',1895,'',10 from dual union all
select 15070,'1966-W436','1966-11-09 20:31:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15071,'1966-W437','1966-11-16 16:06:00','suborbital rocket','success',88,'EXAMETNET W21','Arcasonde IA','','','Meteo',1821,'',54 from dual union all
select 15072,'1966-W438','1966-06-01 16:07:00','suborbital rocket','success',88,'EXAMETNET W09','Arcasonde IA','','','Meteo',1821,'',53 from dual union all
select 15073,'1966-W439','1966-06-03 14:46:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15074,'1966-W44','1966-02-02 21:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',428,'',60 from dual union all
select 15075,'1966-W440','1966-06-24 15:01:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15076,'1966-W441','1966-06-29 14:36:00','suborbital rocket','success',88,'EXAMETNET W11','Arcasonde IA','','','Meteo',1821,'',57 from dual union all
select 15077,'1966-W442','1966-07-13 19:14:00','suborbital rocket','success',88,'EXAMETNET W12','Arcasonde IA','','','Meteo',1821,'',63 from dual union all
select 15078,'1966-W443','1966-07-15 14:11:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15079,'1966-W444','1966-07-22 14:46:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 1508,'1953-A11','1953-01-12 00:00:00','military missile','failure',771,'GM-557','','','','Test',222,'',12 from dual union all
select 15080,'1966-W445','1966-08-19 15:38:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15081,'1966-W446','1966-08-26 19:54:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15082,'1966-W447','1966-08-29 16:31:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15083,'1966-W448','1966-09-07 15:35:00','suborbital rocket','success',88,'EXAMETNET W15','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15084,'1966-W449','1966-09-09 15:50:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15085,'1966-W45','1966-02-02 21:15:00','suborbital rocket','success',88,'','Delta T.F.','','','Weather',1895,'',60 from dual union all
select 15086,'1966-W450','1966-09-16 19:35:00','suborbital rocket','success',88,'EXAMETNET W16','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15087,'1966-W451','1966-09-23 18:48:00','suborbital rocket','success',88,'EXAMETNET W17','Arcasonde IA','','','Meteo',1821,'',63 from dual union all
select 15088,'1966-W452','1966-03-18 02:31:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15089,'1966-W453','1966-08-12 22:24:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 1509,'1953-A110','1953-03-12 00:00:00','atmospheric rocket','success',836,'316P','Nike CET 16','','','Test',1895,'',10 from dual union all
select 15090,'1966-W454','1966-08-16 07:06:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15091,'1966-W455','1966-08-17 23:40:00','suborbital rocket','success',88,'EXAMETNET W14','Arcasonde IA','','','Meteo',1821,'',57 from dual union all
select 15092,'1966-W456','1966-08-28 04:51:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15093,'1966-W457','1966-08-30 01:21:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15094,'1966-W458','1966-09-17 04:02:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15095,'1966-W459','1966-10-01 00:33:00','suborbital rocket','success',88,'EXAMETNET W18','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15096,'1966-W46','1966-02-03 00:09:00','suborbital rocket','success',631,'','','','','Weather',873,'',60 from dual union all
select 15097,'1966-W460','1966-10-05 00:07:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 15098,'1966-W461','1966-08-16 15:00:00','suborbital rocket','success',88,'','Arcasonde/Ballute','','','Meteo/Test',1821,'',60 from dual union all
select 15099,'1966-W462','1966-08-16 16:00:00','suborbital rocket','success',88,'','Arcasonde/Chute','','','Meteo/Test',1821,'',60 from dual union all
select 151,'1944-S137','1944-09-25 00:00:00','military missile','success',1470,'','Arras','','','Weapon',1735,'',95 from dual union all
select 1510,'1953-A111','1953-03-12 00:00:00','atmospheric rocket','success',836,'1076E','','','','Test',1895,'',10 from dual union all
select 15100,'1966-W463','1966-11-16 10:55:00','suborbital rocket','success',1188,'INTA S-6603','','','','Aeron/Test',396,'',75 from dual union all
select 15101,'1966-W464','1966-11-30 11:02:00','suborbital rocket','success',1188,'INTA S-6604','','','','Aeron/Test',396,'',75 from dual union all
select 15102,'1966-W465','1966-10-14 13:55:00','suborbital rocket','unknown',631,'INTA H-6601','','','','Met',396,'',70 from dual union all
select 15103,'1966-W467','1966-06-01 00:00:00','suborbital rocket','unknown',716,'HE test','','','','Test',1895,'',70 from dual union all
select 15104,'1966-W468','1966-06-01 00:00:00','suborbital rocket','unknown',716,'HE test','','','','Test',1895,'',70 from dual union all
select 15105,'1966-W469','1966-06-01 00:00:00','suborbital rocket','unknown',716,'HE test','','','','Test',1895,'',70 from dual union all
select 15106,'1966-W47','1966-02-03 12:36:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',70 from dual union all
select 15107,'1966-W470','1966-07-08 17:25:00','suborbital rocket','unknown',716,'HE test','','','','Test',1895,'',66 from dual union all
select 15108,'1966-W471','1966-07-08 19:36:00','suborbital rocket','unknown',716,'HE test','','','','Test',1895,'',89 from dual union all
select 15109,'1966-W472','1966-08-06 21:40:00','suborbital rocket','failure',716,'HE motor','HE G1','','','Aeron',617,'',20 from dual union all
select 1511,'1953-A112','1953-03-12 00:00:00','atmospheric rocket','success',836,'1249A-200','Nike 1249 R&D 189R','','','Test',1895,'',10 from dual union all
select 15110,'1966-W473','1966-08-06 23:40:00','suborbital rocket','success',716,'HE motor','HE G2','','','Aeron',617,'',82 from dual union all
select 15111,'1966-W474','1966-08-07 09:45:00','suborbital rocket','success',716,'HE motor','HE G3','','','Aeron',617,'',80 from dual union all
select 15112,'1966-W475','1966-08-07 11:40:00','suborbital rocket','success',716,'HE motor','HE G4','','','Aeron',617,'',80 from dual union all
select 15113,'1966-W476','1966-08-07 21:40:00','suborbital rocket','success',716,'HE motor','HE G5','','','Aeron',617,'',80 from dual union all
select 15114,'1966-W477','1966-08-07 23:40:00','suborbital rocket','success',716,'HE motor','HE G6','','','Aeron',617,'',82 from dual union all
select 15115,'1966-W478','1966-08-08 09:50:00','suborbital rocket','success',716,'HE motor','HE G7','','','Aeron',617,'',82 from dual union all
select 15116,'1966-W479','1966-08-08 11:40:00','suborbital rocket','success',716,'HE motor','HE G8','','','Aeron',617,'',80 from dual union all
select 15117,'1966-W48','1966-02-03 16:00:00','suborbital rocket','success',1186,'','','','','Aeron',1895,'',60 from dual union all
select 15118,'1966-W480','1966-08-08 21:40:00','suborbital rocket','success',716,'HE motor','HE G9','','','Aeron',617,'',80 from dual union all
select 15119,'1966-W481','1966-08-08 23:40:00','suborbital rocket','success',716,'HE motor','HE G10','','','Aeron',617,'',80 from dual union all
select 1512,'1953-A113','1953-03-12 00:00:00','atmospheric rocket','success',836,'1249A-172','Nike 1249 R&D 190R','','','Test',1895,'',10 from dual union all
select 15120,'1966-W482','1966-09-28 18:05:00','suborbital rocket','success',88,'EXAMETNET C12A','Arcasonde 2B','','','Aeron',310,'',69 from dual union all
select 15121,'1966-W483','1966-10-06 00:40:00','suborbital rocket','success',631,'EXAMETNET C12','Chaff','','','Aeron',310,'',67 from dual union all
select 15122,'1966-W484','1966-02-16 14:05:00','suborbital rocket','success',631,'EXAMETNET N02','Chaff','','','Aeron',330,'',60 from dual union all
select 15123,'1966-W485','1966-04-13 15:47:00','suborbital rocket','success',631,'EXAMETNET N05','WOX-1A','','','Aeron',330,'',54 from dual union all
select 15124,'1966-W486','1966-10-12 15:00:00','suborbital rocket','success',631,'EXAMETNET N15','WOX-1A','','','Aeron',330,'',55 from dual union all
select 15125,'1966-W487','1966-06-29 16:19:00','suborbital rocket','success',88,'EXAMETNET N10','Arcasonde 1A','','','Aeron',330,'',63 from dual union all
select 15126,'1966-W488','1966-07-15 16:05:00','suborbital rocket','success',88,'EXAMETNET N11','Arcasonde 1A','','','Aeron',330,'',62 from dual union all
select 15127,'1966-W489','1966-08-07 07:58:00','suborbital rocket','success',88,'EXAMETNET N12','Arcasonde 1A','','','Aeron',330,'',64 from dual;
insert into LAUNCH
select 15128,'1966-W49','1966-02-03 19:59:00','suborbital rocket','success',631,'','','','','Weather',1821,'',60 from dual union all
select 15129,'1966-W490','1966-08-17 15:37:00','suborbital rocket','success',88,'EXAMETNET N13','Arcasonde 1A','','','Aeron',330,'',63 from dual union all
select 1513,'1953-A114','1953-03-13 00:00:00','atmospheric rocket','unknown',576,'348','','','','Test',1895,'',10 from dual union all
select 15130,'1966-W491','1966-09-14 15:39:00','suborbital rocket','success',88,'EXAMETNET N14','Arcasonde 1A','','','Aeron',330,'',66 from dual union all
select 15131,'1966-W492','1966-11-16 15:05:00','suborbital rocket','success',88,'EXAMETNET N16','Arcasonde 1A','','','Aeron',330,'',60 from dual union all
select 15132,'1966-W493','1966-03-02 19:15:00','suborbital rocket','success',631,'EXAMETNET W03','WOX-1A','','','Aeron',1821,'',57 from dual union all
select 15133,'1966-W494','1966-03-16 16:23:00','suborbital rocket','success',631,'EXAMETNET W04','WOX-1A','','','Aeron',1821,'',57 from dual union all
select 15134,'1966-W495','1966-03-30 16:03:00','suborbital rocket','success',631,'EXAMETNET W05','Chaff','','','Aeron',1821,'',62 from dual union all
select 15135,'1966-W496','1966-04-14 15:09:00','suborbital rocket','success',631,'EXAMETNET W06','WOX-3A','','','Aeron',1821,'',55 from dual union all
select 15136,'1966-W497','1966-05-02 01:53:00','suborbital rocket','success',631,'EXAMETNET W07','WOX-3A','','','Aeron',1821,'',53 from dual union all
select 15137,'1966-W498','1966-05-19 17:01:00','suborbital rocket','success',631,'EXAMETNET W08','Chaff','','','Aeron',1821,'',65 from dual union all
select 15138,'1966-W499','1966-06-15 18:14:00','suborbital rocket','success',88,'EXAMETNET W10','Arcasonde 1A','','','Meteo',1821,'',59 from dual union all
select 15139,'1966-W50','1966-02-03 20:00:00','suborbital rocket','success',88,'','Ozone','','','Aeron',1895,'',60 from dual union all
select 1514,'1953-A115','1953-03-13 00:00:00','atmospheric rocket','unknown',576,'349','','','','Test',1895,'',10 from dual union all
select 15140,'1966-W500','1966-08-07 07:36:00','suborbital rocket','success',631,'EXAMETNET W13','WOX-3A','','','Aeron',1821,'',56 from dual union all
select 15141,'1966-W501','1966-10-06 14:40:00','suborbital rocket','success',631,'EXAMETNET W19','Chaff','','','Aeron',1821,'',66 from dual union all
select 15142,'1966-W502','1966-10-13 19:38:00','suborbital rocket','success',631,'EXAMETNET W20','WOX-1A','','','Aeron',1821,'',55 from dual union all
select 15143,'1966-W503','1966-12-14 20:39:00','suborbital rocket','success',631,'EXAMETNET W22','Robinette','','','Aeron',1821,'',66 from dual union all
select 15144,'1966-W51','1966-02-03 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 15145,'1966-W52','1966-02-03 22:00:00','suborbital rocket','success',631,'','','','','Weather',1057,'',60 from dual union all
select 15146,'1966-W53','1966-02-03 23:00:00','suborbital rocket','success',716,'','','','','Weather',615,'',60 from dual union all
select 15147,'1966-W54','1966-02-04 07:05:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',1895,'',60 from dual union all
select 15148,'1966-W55','1966-02-04 07:05:00','suborbital rocket','success',716,'','','','','Weather',615,'',60 from dual union all
select 15149,'1966-W56','1966-02-04 12:01:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',76 from dual union all
select 1515,'1953-A116','1953-03-13 00:00:00','military missile','success',572,'1','RV-A-8','','','Test',1863,'',30 from dual union all
select 15150,'1966-W57','1966-02-04 14:15:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',618,'',60 from dual union all
select 15151,'1966-W58','1966-02-04 15:07:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',35,'',60 from dual union all
select 15152,'1966-W59','1966-02-04 15:26:00','suborbital rocket','success',716,'','HASP II','','','Weather',1821,'',60 from dual union all
select 15153,'1966-W60','1966-02-04 16:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 15154,'1966-W61','1966-02-04 17:25:00','suborbital rocket','success',716,'','HASP III','','','Weather',1057,'',60 from dual union all
select 15155,'1966-W62','1966-02-04 18:05:00','suborbital rocket','success',716,'','Chaff','','','Weather',1895,'',60 from dual union all
select 15156,'1966-W63','1966-02-04 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 15157,'1966-W64','1966-02-05 01:01:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',428,'',60 from dual union all
select 15158,'1966-W65','1966-02-05 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 15159,'1966-W66','1966-02-06 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 1516,'1953-A117','1953-03-14 00:00:00','atmospheric rocket','unknown',576,'350','','','','Test',1895,'',10 from dual union all
select 15160,'1966-W67','1966-02-07 15:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 15161,'1966-W68','1966-02-07 15:01:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',35,'',60 from dual union all
select 15162,'1966-W69','1966-02-07 15:04:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',60 from dual union all
select 15163,'1966-W70','1966-02-07 17:05:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',618,'',60 from dual union all
select 15164,'1966-W71','1966-02-07 18:00:00','suborbital rocket','success',631,'','','','','Weather',1057,'',60 from dual union all
select 15165,'1966-W72','1966-02-07 22:19:00','suborbital rocket','success',716,'','Chaff','','','Weather',1895,'',60 from dual union all
select 15166,'1966-W73','1966-02-08 00:30:00','suborbital rocket','success',88,'','','','','Weather',617,'',60 from dual union all
select 15167,'1966-W74','1966-02-08 17:32:00','suborbital rocket','success',631,'','','','','Weather',1057,'',60 from dual union all
select 15168,'1966-W75','1966-02-08 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 15169,'1966-W76','1966-02-08 20:00:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',62 from dual union all
select 1517,'1953-A118','1953-03-15 00:00:00','military missile','failure',1009,'','8A62 LKI I','','','Test',636,'',0 from dual union all
select 15170,'1966-W77','1966-02-09 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',60 from dual union all
select 15171,'1966-W78','1966-02-09 00:07:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',35,'',60 from dual union all
select 15172,'1966-W79','1966-02-09 12:01:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',50 from dual union all
select 15173,'1966-W80','1966-02-09 15:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',176,'',60 from dual union all
select 15174,'1966-W81','1966-02-09 15:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',276,'',60 from dual union all
select 15175,'1966-W82','1966-02-09 15:08:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',35,'',60 from dual union all
select 15176,'1966-W83','1966-02-09 15:41:00','suborbital rocket','success',716,'','HASP II','','','Weather',1821,'',60 from dual union all
select 15177,'1966-W84','1966-02-09 18:01:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',1895,'',60 from dual union all
select 15178,'1966-W85','1966-02-10 00:14:00','suborbital rocket','success',88,'','Delta sensor','','','Weather',855,'',60 from dual union all
select 15179,'1966-W86','1966-02-10 02:07:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',35,'',60 from dual union all
select 1518,'1953-A119','1953-03-16 00:00:00','military missile','success',703,'621','','','','Test',222,'',8 from dual union all
select 15180,'1966-W87','1966-02-10 04:46:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',194,'',60 from dual union all
select 15181,'1966-W88','1966-02-10 06:06:00','suborbital rocket','success',88,'','Sphere','','','Weather',194,'',60 from dual union all
select 15182,'1966-W89','1966-02-10 06:54:00','suborbital rocket','success',631,'','','','','Weather',1821,'',60 from dual union all
select 15183,'1966-W90','1966-02-10 07:34:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',194,'',60 from dual union all
select 15184,'1966-W91','1966-02-10 08:09:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',428,'',60 from dual union all
select 15185,'1966-W92','1966-02-10 08:43:00','suborbital rocket','success',88,'','Sphere','','','Weather',194,'',60 from dual union all
select 15186,'1966-W93','1966-02-10 10:17:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',1057,'',60 from dual union all
select 15187,'1966-W94','1966-02-10 10:59:00','suborbital rocket','success',88,'','Arcasonde IIA','','','Weather',157,'SHIP-S',60 from dual union all
select 15188,'1966-W95','1966-02-10 19:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',428,'',60 from dual union all
select 15189,'1966-W96','1966-02-10 19:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',1895,'',60 from dual union all
select 1519,'1953-A12','1953-01-13 00:00:00','military missile','unknown',339,'41R','','','','Test',1895,'',30 from dual union all
select 15190,'1966-W97','1966-02-10 19:15:00','suborbital rocket','success',716,'','HASP III','','','Weather',1821,'',60 from dual union all
select 15191,'1966-W98','1966-02-10 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Weather',617,'',60 from dual union all
select 15192,'1966-W99','1966-02-10 20:00:00','suborbital rocket','success',1188,'','','','','Aeron',1447,'',62 from dual union all
select 15193,'1966-Y01','1966-01-24 12:33:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15194,'1966-Y02','1966-01-28 12:04:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15195,'1966-Y03','1966-01-31 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15196,'1966-Y04','1966-02-07 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15197,'1966-Y05','1966-02-08 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15198,'1966-Y06','1966-03-10 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15199,'1966-Y07','1966-03-14 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 152,'1944-S138','1944-09-25 00:00:00','military missile','success',1470,'','Cambrai','','','Weapon',1735,'',95 from dual union all
select 1520,'1953-A120','1953-03-17 00:00:00','atmospheric rocket','success',836,'1249A-178','Nike 1249 R&D 192R','','','Test',1895,'',10 from dual union all
select 15200,'1966-Y08','1966-04-15 00:00:00','suborbital rocket','success',273,'','B-1-2289 Robinette','','','Aeron',1821,'',93 from dual union all
select 15201,'1966-Y09','1966-04-22 12:04:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15202,'1966-Y10','1966-04-23 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15203,'1966-Y11','1966-05-05 00:00:00','suborbital rocket','success',273,'','B-1-2290 Robinette','','','Aeron',1821,'',94 from dual union all
select 15204,'1966-Y12','1966-05-15 09:40:00','suborbital rocket','success',1250,'AE001','ESRO A23/P','','','Iono-solar Eclipse',831,'',110 from dual union all
select 15205,'1966-Y13','1966-05-18 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15206,'1966-Y14','1966-05-20 08:00:00','suborbital rocket','success',257,'NASA 15.26GI','Zorba II','','','Iono-solar Eclipse',874,'RREC',90 from dual union all
select 15207,'1966-Y15','1966-05-20 08:35:00','suborbital rocket','failure',1250,'AE002','ESRO A23/1','','','Iono-solar Eclipse',831,'',10 from dual union all
select 15208,'1966-Y16','1966-05-20 08:45:00','suborbital rocket','success',257,'NASA 15.27GI','Zorba III','','','Iono-solar Eclipse',874,'RREC',99 from dual union all
select 15209,'1966-Y17','1966-05-20 09:00:00','suborbital rocket','success',1250,'A003','ESRO A23/2','','','Iono-solar Eclipse',831,'',99 from dual union all
select 1521,'1953-A121','1953-03-17 00:00:00','atmospheric rocket','success',836,'1034E','','','','Test',1895,'',10 from dual union all
select 15210,'1966-Y18','1966-05-20 09:30:00','suborbital rocket','success',257,'NASA 15.28GI','Zorba IV','','','Iono-solar Eclipse',874,'RREC',102 from dual union all
select 15211,'1966-Y19','1966-05-20 09:40:00','suborbital rocket','success',1250,'A004','ESRO A23/3','','','Iono-solar Eclipse',831,'',102 from dual union all
select 15212,'1966-Y20','1966-05-20 10:15:00','suborbital rocket','success',257,'NASA 15.29GI','Zorba V','','','Iono-solar Eclipse',874,'RREC',90 from dual union all
select 15213,'1966-Y21','1966-05-20 10:40:00','suborbital rocket','success',1250,'A006','ESRO A23/5','','','Iono-solar Eclipse',831,'',103 from dual union all
select 15214,'1966-Y22','1966-05-20 11:05:00','suborbital rocket','success',257,'NASA 15.30GI','Zorba VI','','','Iono-solar Eclipse',874,'RREC',101 from dual union all
select 15215,'1966-Y23','1966-05-20 12:04:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15216,'1966-Y24','1966-05-21 09:30:00','suborbital rocket','success',257,'NASA 15.31GI','Zorba VII','','','Ionos',874,'RREC',90 from dual union all
select 15217,'1966-Y25','1966-06-16 00:00:00','suborbital rocket','success',273,'','B-1-2291 Robinette','','','Aeron',1821,'',93 from dual;
insert into LAUNCH
select 15218,'1966-Y26','1966-06-16 12:12:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15219,'1966-Y27','1966-06-28 00:00:00','suborbital rocket','success',273,'','B-1-2292 Robinette','','','Aeron',1821,'',96 from dual union all
select 1522,'1953-A122','1953-03-17 00:00:00','atmospheric rocket','success',836,'1080E','','','','Test',1895,'',10 from dual union all
select 15220,'1966-Y28','1966-07-11 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15221,'1966-Y29','1966-07-12 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15222,'1966-Y30','1966-07-14 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15223,'1966-Y31','1966-07-15 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15224,'1966-Y32','1966-07-25 00:00:00','suborbital rocket','success',258,'','Test 1','','','Test',1895,'',145 from dual union all
select 15225,'1966-Y33','1966-08-17 12:01:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15226,'1966-Y34','1966-08-17 16:30:00','suborbital rocket','success',631,'EXAMETNET C08','WOX-1A','','','Aeron',310,'',61 from dual union all
select 15227,'1966-Y35','1966-08-19 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15228,'1966-Y36','1966-08-20 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15229,'1966-Y37','1966-08-29 12:38:00','suborbital rocket','success',257,'NASA 15.12','D-region','','','Ionos',1895,'',90 from dual union all
select 1523,'1953-A123','1953-03-17 00:00:00','atmospheric rocket','success',836,'1249A-201','Nike 1249 R&D 191R','','','Test',1895,'',10 from dual union all
select 15230,'1966-Y38','1966-08-29 13:37:00','suborbital rocket','success',257,'NASA 15.11','D-region','','','Ionos',1895,'',90 from dual union all
select 15231,'1966-Y39','1966-08-31 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15232,'1966-Y40','1966-09-07 09:30:00','suborbital rocket','success',88,'','PSU-Arcas #14','','','Ionos',617,'',85 from dual union all
select 15233,'1966-Y41','1966-09-15 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15234,'1966-Y42','1966-09-24 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15235,'1966-Y43','1966-10-05 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 15236,'1966-Y44','1966-10-15 10:56:00','suborbital rocket','success',1188,'INTA S-6601','Carabela 4','','','Aeron/Test',396,'',81 from dual union all
select 15237,'1966-Y45','1966-11-11 09:01:00','suborbital rocket','success',88,'','Eclipse','','','Aeron/Eclipse',1460,'',90 from dual union all
select 15238,'1966-Y46','1966-11-11 14:06:00','suborbital rocket','success',88,'','Eclipse','','','Aeron/Eclipse',1460,'',90 from dual union all
select 15239,'1966-Y47','1966-11-11 18:00:00','suborbital rocket','success',88,'','Eclipse','','','Aeron/Eclipse',1460,'',90 from dual union all
select 1524,'1953-A124','1953-03-18 00:00:00','military missile','success',771,'GM-11062','','','','Test',222,'',12 from dual union all
select 15240,'1966-Y48','1966-11-12 12:33:00','suborbital rocket','success',88,'','Eclipse','','','Aeron/Eclipse',1460,'',90 from dual union all
select 15241,'1966-Y49','1966-11-12 13:03:00','suborbital rocket','success',88,'','Eclipse','','','Aeron/Eclipse',1460,'',90 from dual union all
select 15242,'1966-Y50','1966-11-12 14:00:00','suborbital rocket','success',88,'','Eclipse','','','Aeron/Eclipse',1460,'',90 from dual union all
select 15243,'1966-Y51','1966-11-12 14:31:00','suborbital rocket','success',88,'','Eclipse','','','Aeron/Eclipse',1460,'',90 from dual union all
select 15244,'1966-Y52','1966-11-12 17:02:00','suborbital rocket','success',88,'','Eclipse','','','Aeron/Eclipse',1460,'',90 from dual union all
select 15245,'1966-Y53','1966-11-12 00:00:00','suborbital rocket','success',88,'','Eclipse','','','Aeron/Eclipse',1460,'',90 from dual union all
select 15246,'1966-Y54','1966-11-12 00:00:00','suborbital rocket','success',88,'','Eclipse','','','Aeron/Eclipse',1460,'',90 from dual union all
select 15247,'1966-Y55','1966-11-12 00:00:00','suborbital rocket','success',88,'','Eclipse','','','Aeron/Eclipse',1460,'',90 from dual union all
select 15248,'1966-Y56','1966-11-12 00:00:00','suborbital rocket','success',88,'','Eclipse','','','Aeron/Eclipse',1460,'',90 from dual union all
select 15249,'1966-Y57','1966-11-14 06:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 1525,'1953-A125','1953-03-18 00:00:00','military missile','failure',1009,'','8A62 LKI I','','','Test',636,'',0 from dual union all
select 15250,'1966-Y58','1966-11-30 10:07:00','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 15251,'1966-Y59','1966-12-14 07:30:00','suborbital rocket','success',740,'','Mass spectrometer','','','Aeron',636,'',167 from dual union all
select 15252,'1966-Y60','1966-12-24 06:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 15253,'1966-Y61','1966-12-26 09:12:59','suborbital rocket','success',725,'','','','','Meteo',1546,'SHIP-SOV',80 from dual union all
select 15254,'1966-Y62','1966-12-31 08:20:00','suborbital rocket','success',725,'','','','','Meteo',1519,'SHIP-SOV',80 from dual union all
select 15255,'1966-Y63','1966-12-31 09:59:59','suborbital rocket','success',725,'','','','','Meteo',1501,'SHIP-SOV',80 from dual union all
select 15256,'1967-001','1967-01-11 10:55:00','orbital','success',1382,'468/D44','Intelsat II F-2','INTELSAT II F-2','','Sat',234,'',null from dual union all
select 15257,'1967-002','1967-01-14 21:28:21','orbital','success',1394,'495/FTV 1629','CORONA 114/1038','CORONA J-34','LONG ROAD','Sat TAG138',1776,'',null from dual union all
select 15258,'1967-003','1967-01-18 14:19:08','orbital','success',1425,'3C-13','IDCSP','IDCSP 8','','Sat AMF',275,'',null from dual union all
select 15259,'1967-004','1967-01-19 12:39:59','orbital','success',1509,'N 15001- 05','Kosmos-138','Zenit-2 No. 43','','Sat',473,'',null from dual union all
select 1526,'1953-A126','1953-03-19 00:00:00','atmospheric rocket','success',836,'320P','Nike CET 20','','','Test',1895,'',10 from dual union all
select 15260,'1967-005','1967-01-25 13:55:01','orbital','success',1007,'N 22500-07L','Kosmos-139','OGCh No. 07L','','Sat',519,'',null from dual union all
select 15261,'1967-006','1967-01-26 17:31:00','orbital','success',1381,'472/D45','ESSA 4','TOS B','BUSY PENNY','Sat',1723,'',null from dual union all
select 15262,'1967-007','1967-02-02 20:00:00','orbital','success',177,'7126','AFP-206','GAMBIT SV 986','BUSY PARTY','Sat',1777,'',null from dual union all
select 15263,'1967-008','1967-02-05 01:17:01','deep space','success',177,'5803 (AA20)','Lunar Orbiter 3','Lunar Orbiter C','','Sat',228,'',null from dual union all
select 15264,'1967-009','1967-02-07 03:20:00','orbital','success',1223,'U 15000- 03','Kosmos-140','Soyuz No. 3','','Sat',480,'',null from dual union all
select 15265,'1967-010','1967-02-08 08:00:52','orbital','success',1364,'169/BII-2','DAPP 2418','DAPP 2418','ARROW POINT','Sat',1660,'',null from dual union all
select 15266,'1967-011','1967-02-08 09:39:39','orbital','success',431,'No. 3','D-1C','Diademe D-1C','','Sat',669,'',null from dual union all
select 15267,'1967-012','1967-02-08 10:19:59','orbital','success',1508,'N 15001- 09','Kosmos-141','Zenit-4 No. 29','','Sat',473,'',null from dual union all
select 15268,'1967-013','1967-02-14 10:04:56','orbital','success',678,'','Kosmos-142','DS-U2-I No. 2','','Sat',626,'',null from dual union all
select 15269,'1967-014','1967-02-15 10:06:57','orbital','success',431,'No. 4','D-1D','Diademe D-1D','','Sat',669,'',null from dual union all
select 1527,'1953-A127','1953-03-19 00:00:00','atmospheric rocket','success',836,'317P','Nike CET 17','','','Test',1895,'',10 from dual union all
select 15270,'1967-015','1967-02-22 22:02:15','orbital','success',1394,'493/FTV 1635','CORONA 115/1039','CORONA J-39','BUSY PAWNSHOP','Sat TAG139',1776,'',null from dual union all
select 15271,'1967-016','1967-02-24 19:59:00','orbital','success',1424,'3B-4','KH-8','GAMBIT-3 Mission 4304','BUSY PALEFACE','Sat',1778,'',null from dual union all
select 15272,'1967-017','1967-02-27 08:45:01','orbital','success',1509,'U 15001- 03','Kosmos-143','Zenit-2 No. 45','','Sat',480,'',null from dual union all
select 15273,'1967-018','1967-02-28 14:34:59','orbital','success',1510,'N 15000- 55','Kosmos-144','Meteor 11F614 No. 6','','Sat',473,'',null from dual union all
select 15274,'1967-019','1967-03-03 06:44:58','orbital','success',678,'','Kosmos-145','DS-U2-M No. 2','','Sat',626,'',null from dual union all
select 15275,'1967-020','1967-03-08 16:12:00','orbital','success',1378,'431/D46','OSO 3','OSO E1','','Sat',233,'',null from dual union all
select 15276,'1967-021','1967-03-10 11:30:33','deep space','success',1460,'N10722701','Kosmos-146','L-1 No. 2P','','Sat',587,'',null from dual union all
select 15277,'1967-022','1967-03-13 12:10:23','orbital','success',1509,'N 15001- 06','Kosmos-147','Zenit-2 No. 44','','Sat',473,'',null from dual union all
select 15278,'1967-023','1967-03-16 17:30:00','orbital','success',678,'U231-05','Kosmos-148','DS-P1-I No. 2','','Sat',460,'',null from dual union all
select 15279,'1967-024','1967-03-21 10:07:00','orbital','success',678,'','Kosmos-149','DS-MO No. 1','','Sat',626,'',null from dual union all
select 1528,'1953-A128','1953-03-19 00:00:00','atmospheric rocket','success',836,'1078E','','','','Test',1895,'',10 from dual union all
select 15280,'1967-025','1967-03-22 12:44:59','orbital','success',1508,'N 15001- 07','Kosmos-150','Zenit-4 No. 27','','Sat',473,'',null from dual union all
select 15281,'1967-026','1967-03-23 01:30:12','orbital','success',1382,'470/D47','Intelsat II F-3','INTELSAT II F-3','','Sat',234,'',null from dual union all
select 15282,'1967-027','1967-03-24 11:50:00','orbital','success',679,'4000-000-06LM','Kosmos-151','Strela-2','','Sat',564,'',null from dual union all
select 15283,'1967-028','1967-03-25 06:59:30','orbital','success',678,'U231-08','Kosmos-152','DS-P1-Yu No. 7','','Sat',460,'',null from dual union all
select 15284,'1967-029','1967-03-30 18:54:23','orbital','success',1394,'501/FTV 1636','CORONA 116/1040','CORONA J-35','GIANT BANANA','Sat TAG140',1776,'',null from dual union all
select 15285,'1967-030','1967-04-04 14:00:02','orbital','success',1509,'Ya15001- 01','Kosmos-153','Zenit-2 No. 48','','Sat',473,'',null from dual union all
select 15286,'1967-031','1967-04-06 03:23:01','orbital','success',177,'5102 (AA21)','ATS 2','ATS A','','Sat',227,'',null from dual union all
select 15287,'1967-032','1967-04-08 09:00:33','orbital','failure',953,'228-01','Kosmos-154','L-1 No. 3P','','Sat',587,'',null from dual union all
select 15288,'1967-033','1967-04-12 10:51:02','orbital','success',1508,'N 15001- 08','Kosmos-155','Zenit-4 No. 28','','Sat',480,'',null from dual union all
select 15289,'1967-034','1967-04-14 03:25:00','orbital','success',1125,'S154C','NNS O-12 (N-10)','NNS O-12','BUSY MINUTEMAN','Sat',1779,'',null from dual union all
select 1529,'1953-A129','1953-03-19 00:00:00','atmospheric rocket','success',836,'323P','Nike CET 23','','','Test',1895,'',10 from dual union all
select 15290,'1967-035','1967-04-17 07:05:01','deep space','success',153,'AC-12','Surveyor 3','Surveyor SC-3','','Sat',269,'',null from dual union all
select 15291,'1967-036','1967-04-20 11:21:10','orbital','success',1381,'484/D48','ESSA 5','TOS C','BUCKLE PAINTER','Sat',1723,'',null from dual union all
select 15292,'1967-037','1967-04-23 00:35:00','orbital','success',1223,'U 15000- 04','Soyuz-1','Soyuz No. 4','','Sat',480,'',null from dual union all
select 15293,'1967-038','1967-04-26 10:06:24','orbital','success',1128,'S153C','San Marco 2','San Marco B','','Sat',1412,'',null from dual union all
select 15294,'1967-039','1967-04-27 12:50:02','orbital','success',1510,'R 15000- 22','Kosmos-156','Meteor 11F614 No. 7','','Sat',473,'',null from dual union all
select 15295,'1967-040','1967-04-28 10:01:01','orbital','success',1425,'3C-10','Vela 4A/4B','Vela 4A','','Sat AMF',275,'',null from dual union all
select 15296,'1967-041','1967-05-04 22:25:00','deep space','success',177,'5804 (AA22)','Lunar Orbiter 4','Lunar Orbiter D','','Sat',228,'',null from dual union all
select 15297,'1967-042','1967-05-05 16:00:01','orbital','success',1125,'S155C','Ariel 3','UK-3','BUSY WIFE','Sat',1779,'',null from dual union all
select 15298,'1967-043','1967-05-09 21:50:42','orbital','success',1395,'508/FTV 1634','CORONA 117/1041','CORONA J-40','BUSY BANKER','Sat TAG141',1721,'',null from dual union all
select 15299,'1967-044','1967-05-12 10:30:01','orbital','success',1509,'Ya15001- 02','Kosmos-157','Zenit-2 No. 49','','Sat',480,'',null from dual union all
select 153,'1944-S139','1944-09-25 00:00:00','military missile','success',1470,'','Liege','','','Weapon',1735,'',95 from dual union all
select 1530,'1953-A13','1953-01-13 00:00:00','atmospheric rocket','unknown',576,'82','','','','Test',1895,'',10 from dual union all
select 15300,'1967-045','1967-05-15 11:00:00','orbital','success',680,'N149    -16LM','Kosmos-158','Tsiklon GVM','','Sat',457,'',null from dual union all
select 15301,'1967-046','1967-05-16 21:43:57','orbital','success',809,'N 15001- 58','Kosmos-159','E-6LS No. 111','','Sat',480,'',null from dual union all
select 15302,'1967-047','1967-05-17 16:05:02','orbital','success',1007,'Ya22500-09L','Kosmos-160','OGCh No. 09?','','Sat',518,'',null from dual union all
select 15303,'1967-048','1967-05-18 09:05:01','orbital','success',1125,'S156C','NNS O-13 (N-11)','NNS O-13','BUSY OCEAN','Sat',1779,'',null from dual union all
select 15304,'1967-049','1967-05-22 14:00:00','orbital','success',1508,'Ya15001- 12','Kosmos-161','Zenit-4 No. 31','','Sat',473,'',null from dual union all
select 15305,'1967-050','1967-05-22 18:30:00','orbital','success',177,'7127','AFP-206','GAMBIT SV 987','BUSY CAMPER','Sat',1777,'',null from dual union all
select 15306,'1967-051','1967-05-24 14:05:54','orbital','success',1382,'486/D49','Explorer 34','IMP F','','Sat',1723,'',null from dual union all
select 15307,'1967-052','1967-05-24 22:50:00','orbital','success',801,'N 15000- 41','Molniya-1','Molniya-1 11F67 No. 8','','Sat',480,'',null from dual;
insert into LAUNCH
select 15308,'1967-053','1967-05-31 09:30:48','orbital','success',1392,'443/FTV 2704','POPPY 5/NRL Comp 6','NRL PL160','COMIC STRIP','Sat TAG142',1724,'',null from dual union all
select 15309,'1967-054','1967-06-01 10:40:02','orbital','success',1508,'Ya15001- 11','Kosmos-162','Zenit-4 No. 30','','Sat',480,'',null from dual union all
select 1531,'1953-A130','1953-03-19 00:00:00','atmospheric rocket','success',836,'322','Nike CET 22','','','Test',1895,'',10 from dual union all
select 15310,'1967-055','1967-06-04 18:07:01','orbital','success',177,'7128','AFP-206','GAMBIT SV 988','BUSY WOLF','Sat',1777,'',null from dual union all
select 15311,'1967-056','1967-06-05 05:03:00','orbital','success',678,'','Kosmos-163','DS-U2-MP No. 2','','Sat',626,'',null from dual union all
select 15312,'1967-057','1967-06-08 13:00:01','orbital','success',1508,'Ya15001- 04 *(13)','Kosmos-164','Zenit-2 No. 50','','Sat',473,'',null from dual union all
select 15313,'1967-058','1967-06-12 02:39:45','deep space','success',808,'Ya15000- 71  L70','Venera-4','4V-1 No. 310','','Sat',480,'',null from dual union all
select 15314,'1967-059','1967-06-12 18:06:00','orbital','success',678,'N231-12','Kosmos-165','DS-P1-Yu No. 11','','Sat',460,'',null from dual union all
select 15315,'1967-060','1967-06-14 06:01:00','deep space','success',177,'5401 (AA23)','Mariner 5','Mariner 67-2','','Sat',227,'',null from dual union all
select 15316,'1967-061','1967-06-16 04:44:00','orbital','success',678,'','Kosmos-166','DS-U3-S No. 1','','Sat',626,'',null from dual union all
select 15317,'1967-062','1967-06-16 21:35:22','orbital','success',1395,'509/FTV 1633','CORONA 118/1042','CORONA J-37','','Sat TAG143',1722,'',null from dual union all
select 15318,'1967-063','1967-06-17 02:36:38','orbital','failure',808,'Ya15000- 70  L71','Kosmos-167','4V-1 No. 311','','Sat',480,'',262 from dual union all
select 15319,'1967-064','1967-06-20 16:15:00','orbital','success',1424,'3B-8','KH-8','GAMBIT-3 Mission 4306','BUSY PLAYMATE','Sat',1778,'',null from dual union all
select 1532,'1953-A131','1953-03-19 00:00:00','atmospheric rocket','success',836,'321P','Nike CET 21','','','Test',1895,'',10 from dual union all
select 15320,'1967-065','1967-06-29 21:01:44','orbital','success',1364,'171/BII-3','P67-1','Secor Type II S/N 10','DEER FOOT','Sat',1694,'',null from dual union all
select 15321,'1967-066','1967-07-01 13:15:01','orbital','success',1425,'3C-14','IDCSP','IDCSP 16','','Sat AMF',275,'',null from dual union all
select 15322,'1967-067','1967-07-04 05:59:59','orbital','success',1508,'Ya15001- 05','Kosmos-168','Zenit-2 No. 52','','Sat',562,'',null from dual union all
select 15323,'1967-068','1967-07-14 11:53:29','deep space','success',153,'AC-11','Surveyor 4','Surveyor SC-4','','Sat',268,'',null from dual union all
select 15324,'1967-069','1967-07-17 16:45:04','orbital','success',1007,'Ya22500-11L','Kosmos-169','OGCh No. 10?','','Sat',519,'',null from dual union all
select 15325,'1967-070','1967-07-19 14:19:02','deep space','success',1382,'488/D50','Explorer 35','AIMP E','','Sat',234,'',null from dual union all
select 15326,'1967-071','1967-07-25 03:48:10','orbital','success',1394,'496/FTV 2732','OPS 1879','MULTIGROUP 2','','Sat TAG144',1724,'',null from dual union all
select 15327,'1967-072','1967-07-27 19:00:03','orbital','failure',154,'92D','OV1-11/12/86','OV1-8S','','Sat',1685,'',604 from dual union all
select 15328,'1967-073','1967-07-28 14:21:07','orbital','success',1394,'478/FTV 6802 (TA8)','OGO 4','OGO D','','Sat TAG145',1723,'',null from dual union all
select 15329,'1967-074','1967-07-31 16:45:00','orbital','success',1007,'Ya22500-12L','Kosmos-170','OGCh No. 11?','','Sat',518,'',null from dual union all
select 1533,'1953-A132','1953-03-19 00:00:00','atmospheric rocket','success',836,'318P','Nike CET 18','','','Test',1895,'',10 from dual union all
select 15330,'1967-075','1967-08-01 22:33:00','deep space','success',177,'5805 (AA24)','Lunar Orbiter 5','Lunar Orbiter E','','Sat',228,'',null from dual union all
select 15331,'1967-076','1967-08-07 21:44:00','orbital','success',1395,'510/FTV 1637','CORONA 119/1043','CORONA J-42','','Sat TAG146',1721,'',null from dual union all
select 15332,'1967-077','1967-08-08 16:05:02','orbital','success',1007,'Ya22500-10L','Kosmos-171','OGCh No. 12?','','Sat',519,'',null from dual union all
select 15333,'1967-078','1967-08-09 05:45:18','orbital','success',1508,'Ya15001- 15','Kosmos-172','Zenit-4 No. 34','','Sat',480,'',null from dual union all
select 15334,'1967-079','1967-08-16 17:07:00','orbital','success',1424,'3B-9','KH-8','GAMBIT-3 Mission 4307','','Sat',1778,'',null from dual union all
select 15335,'1967-080','1967-08-23 04:41:04','orbital','success',1364,'266/BII-4','DAPP 3419','DAPP 3419','','Sat',1694,'',null from dual union all
select 15336,'1967-081','1967-08-24 04:59:49','orbital','success',678,'U231-10','Kosmos-173','DS-P1-Yu No. 8','','Sat',460,'',null from dual union all
select 15337,'1967-082','1967-08-31 08:00:03','orbital','success',801,'N 15000- 81  L81','Kosmos-174','Molniya-1Yu 11F67 No. 11','','Sat',480,'',null from dual union all
select 15338,'1967-083','1967-09-07 22:04:26','orbital','success',1383,'475/D51','Biosatellite 2','Biosatellite B Adapter','','Sat',234,'',null from dual union all
select 15339,'1967-084','1967-09-08 07:57:01','deep space','success',179,'AC-13','Surveyor 5','Surveyor 5','','Sat',269,'',null from dual union all
select 1534,'1953-A133','1953-03-19 00:00:00','atmospheric rocket','success',836,'319P','Nike CET 19','','','Test',1895,'',10 from dual union all
select 15340,'1967-085','1967-09-11 10:30:00','orbital','success',1508,'Ya15002- 11','Kosmos-175','Zenit-4 No. 37','','Sat',473,'',null from dual union all
select 15341,'1967-086','1967-09-12 17:00:00','orbital','success',678,'N231-08','Kosmos-176','DS-P1-Yu No. 10','','Sat',460,'',null from dual union all
select 15342,'1967-087','1967-09-15 19:41:15','orbital','success',1395,'512/FTV 1641','CORONA 120/1101','CORONA CR-1','','Sat TAG147',1722,'',null from dual union all
select 15343,'1967-088','1967-09-16 06:06:00','orbital','success',1508,'Ya15001- 06','Kosmos-177','Zenit-2 No. 53','','Sat',480,'',null from dual union all
select 15344,'1967-089','1967-09-19 14:45:03','orbital','success',1007,'Ya22500-14L','Kosmos-178','OGCh No. 13?','','Sat',518,'',null from dual union all
select 15345,'1967-090','1967-09-19 18:37:00','orbital','success',1424,'3B-10','KH-8','GAMBIT-3 Mission 4308','','Sat',1778,'',null from dual union all
select 15346,'1967-091','1967-09-22 14:05:05','orbital','success',1007,'Ya22500-15L','Kosmos-179','OGCh No. 14?','','Sat',519,'',null from dual union all
select 15347,'1967-092','1967-09-25 08:25:00','orbital','success',1125,'S157C','NNS O-14 (N-12)','NNS O-14','','Sat',1779,'',null from dual union all
select 15348,'1967-093','1967-09-26 10:20:00','orbital','success',1508,'Ya15001- 07','Kosmos-180','Zenit-2 No. 54','','Sat',473,'',null from dual union all
select 15349,'1967-094','1967-09-28 00:45:00','orbital','success',1382,'442/D52','Intelsat II F-4','INTELSAT II F-4','','Sat',234,'',null from dual union all
select 1535,'1953-A134','1953-03-20 00:00:00','atmospheric rocket','failure',442,'','B-58 Model','','','Test',1821,'',0 from dual union all
select 15350,'1967-095','1967-10-03 05:00:01','orbital','success',801,'Ya15000- 83  L83','Molniya-1','Molniya-1 11F67 No. 9','','Sat',480,'',null from dual union all
select 15351,'1967-096','1967-10-11 07:57:56','orbital','success',1364,'268/BII-5','DAPP 4417','DAPP 4417','','Sat',1694,'',null from dual union all
select 15352,'1967-097','1967-10-11 11:30:04','orbital','success',1508,'Ya15002- 01','Kosmos-181','Zenit-2 No. 55','','Sat',473,'',null from dual union all
select 15353,'1967-098','1967-10-16 08:00:00','orbital','success',1508,'Ya15001- 17','Kosmos-182','Zenit-4 No. 36','','Sat',562,'',null from dual union all
select 15354,'1967-099','1967-10-18 13:30:01','orbital','success',1007,'Ya22500-16L','Kosmos-183','OGCh No. 15?','','Sat',518,'',null from dual union all
select 15355,'1967-100','1967-10-18 15:58:00','orbital','success',1379,'490/D53','OSO 4','OSO D','','Sat',234,'',null from dual union all
select 15356,'1967-101','1967-10-22 08:40:01','orbital','success',801,'Ya15000- 84  L84','Molniya-1','Molniya-1 11F67 No. 12','','Sat',480,'',null from dual union all
select 15357,'1967-102','1967-10-24 22:49:09','orbital','success',1510,'N 15000- 56','Kosmos-184','Meteor 11F614 No. 8','','Sat',473,'',null from dual union all
select 15358,'1967-103','1967-10-25 19:15:00','orbital','success',1424,'3B-11','KH-8','GAMBIT-3 Mission 4309','','Sat',1778,'',null from dual union all
select 15359,'1967-104','1967-10-27 02:21:19','orbital','success',1447,'N 22500-03L','Kosmos-185','I2-BM','','Sat',590,'',null from dual union all
select 1536,'1953-A135','1953-03-20 00:00:00','atmospheric rocket','unknown',576,'351','','','','Test',1895,'',10 from dual union all
select 15360,'1967-105','1967-10-27 09:29:59','orbital','success',1223,'U 15000- 05','Kosmos-186','Soyuz No. 6','','Sat',562,'',null from dual union all
select 15361,'1967-106','1967-10-28 13:15:01','orbital','success',1007,'Ya22500-13L','Kosmos-187','OGCh No. 16?','','Sat',519,'',null from dual union all
select 15362,'1967-107','1967-10-30 08:12:41','orbital','success',1223,'N 15000- 07','Kosmos-188','Soyuz No. 5','','Sat',480,'',null from dual union all
select 15363,'1967-108','1967-10-30 17:59:59','orbital','success',680,'N149    -19LM','Kosmos-189','Tselina-O GVM','','Sat',457,'',null from dual union all
select 15364,'1967-109','1967-11-02 21:31:19','orbital','success',1395,'513/FTV 1639','CORONA 121/1044','CORONA J-41','','Sat TAG148',1721,'',null from dual union all
select 15365,'1967-110','1967-11-03 11:20:00','orbital','success',1508,'Ya15001- 16','Kosmos-190','Zenit-4 No. 35','','Sat',473,'',null from dual union all
select 15366,'1967-111','1967-11-05 23:37:00','orbital','success',177,'5103 (AA25)','ATS 3','ATS C','','Sat',227,'',null from dual union all
select 15367,'1967-112','1967-11-07 07:39:01','deep space','success',179,'AC-14','Surveyor 6','Surveyor SC-6','','Sat',269,'',null from dual union all
select 15368,'1967-113','1967-11-09 12:00:01','deep space','success',1123,'SA-501','Apollo 4','Apollo CM-017','','Sat',921,'LUT1',null from dual union all
select 15369,'1967-114','1967-11-10 17:53:08','orbital','success',1382,'480/D54','ESSA 6','TOS D','','Sat',1723,'',null from dual union all
select 1537,'1953-A136','1953-03-21 00:00:00','atmospheric rocket','unknown',576,'352','','','','Test',1895,'',10 from dual union all
select 15370,'1967-115','1967-11-21 14:29:48','orbital','success',678,'U231-11','Kosmos-191','DS-P1-Yu No. 9','','Sat',460,'',null from dual union all
select 15371,'1967-116','1967-11-23 15:00:00','orbital','success',680,'N149    -18LM','Kosmos-192','Tsiklon','','Sat',457,'',null from dual union all
select 15372,'1967-117','1967-11-25 11:30:00','orbital','success',1508,'Ya15002- 04','Kosmos-193','Zenit-2 No. 58','','Sat',473,'',null from dual union all
select 15373,'1967-118','1967-11-29 04:49:00','orbital','success',1095,'CC-2029/SV-10','WRESAT','WRESAT','','Sat',1846,'',1119 from dual union all
select 15374,'1967-119','1967-12-03 12:00:00','orbital','success',1508,'Ya15002- 13','Kosmos-194','Zenit-4 No. 39','','Sat',473,'',null from dual union all
select 15375,'1967-120','1967-12-05 01:03:46','orbital','success',1128,'S158C','OV3-6','OV3-6','','Sat',1779,'',null from dual union all
select 15376,'1967-121','1967-12-05 18:45:00','orbital','success',1424,'3B-12','KH-8','GAMBIT-3 Mission 4310','','Sat',1778,'',null from dual union all
select 15377,'1967-122','1967-12-09 22:25:41','orbital','success',1395,'514/FTV 1642','CORONA 122/1102','CORONA CR-2','','Sat TAG149',1722,'',null from dual union all
select 15378,'1967-123','1967-12-13 14:08:00','deep space','success',1382,'489/D55','Pioneer 8','Pioneer C','','Sat',234,'',null from dual union all
select 15379,'1967-124','1967-12-16 12:00:37','orbital','success',1508,'Ya15002- 03','Kosmos-195','Zenit-2 No. 57','','Sat',473,'',null from dual union all
select 1538,'1953-A137','1953-03-21 00:00:00','atmospheric rocket','unknown',576,'187','','','','Test',1895,'',10 from dual union all
select 15380,'1967-125','1967-12-19 06:30:07','orbital','success',681,'','Kosmos-196','DS-U1-G No. 2','','Sat',626,'',null from dual union all
select 15381,'1967-126','1967-12-26 09:01:59','orbital','success',678,'','Kosmos-197','DS-U2-V No. 3','','Sat',627,'',null from dual union all
select 15382,'1967-127','1967-12-27 11:28:51','orbital','success',1447,'N 22500-07L','Kosmos-198','US-A reactor','','Sat',590,'',null from dual union all
select 15383,'1967-A01','1967-01-01 00:00:00','military missile','success',545,'DASA 6.1','','','','Aeron?',1895,'',40 from dual union all
select 15384,'1967-A02','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15385,'1967-A03','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15386,'1967-A04','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15387,'1967-A05','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15388,'1967-A06','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15389,'1967-A07','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 1539,'1953-A138','1953-04-03 00:00:00','atmospheric rocket','success',836,'1249A-196','Nike 1249 R&D 199R','','','Test',1895,'',10 from dual union all
select 15390,'1967-A08','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15391,'1967-A09','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15392,'1967-A10','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15393,'1967-A100','1967-08-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 15394,'1967-A101','1967-08-24 00:00:00','atmospheric rocket','success',1066,'Ob1','Oberon 1','','','Test',1836,'',20 from dual union all
select 15395,'1967-A102','1967-08-07 00:00:00','suborbital spaceplane','success',1520,'2- C-93','Ramjet flow','','','Test',379,'NB-52',14 from dual union all
select 15396,'1967-A103','1967-08-08 00:00:00','military missile','success',1252,'FLA-11','','','','Test',1888,'',2 from dual union all
select 15397,'1967-A104','1967-08-09 11:27:00','suborbital rocket','success',88,'','Biospace  (Rat)','','','Bio',1821,'',40 from dual union all
select 15398,'1967-A105','1967-08-11 00:00:00','suborbital spaceplane','failure',1520,'2- A-94','Ramjet flow','','','Test',379,'NB-52',14 from dual;
insert into LAUNCH
select 15399,'1967-A106','1967-08-16 00:00:00','suborbital spaceplane','failure',1520,'2- A-95','','','','Test',379,'NB-52',14 from dual union all
select 154,'1944-S14','1944-03-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1540,'1953-A139','1953-03-23 00:00:00','military missile','unknown',339,'3EBN/1261','EU-3','','','Test',1895,'',30 from dual union all
select 15400,'1967-A107','1967-08-20 00:40:33','suborbital rocket','failure',1181,'','Arcas II','','','Ionos',29,'',1 from dual union all
select 15401,'1967-A108','1967-08-21 08:20:00','suborbital rocket','failure',839,'NASA 10.240GM','GSFC Grenade 158 N','','','Aeronomy',330,'',10 from dual union all
select 15402,'1967-A109','1967-08-21 17:59:16','suborbital spaceplane','success',1520,'2-52-96','Ramjet/Hycon/TPS','','','Test/Imaging?',379,'NB-52 008',27 from dual union all
select 15403,'1967-A11','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15404,'1967-A110','1967-08-24 00:00:00','military missile','failure',135,'D059','SFT-8','ABRES AC-1 (USN)','','Reentry Test',614,'',0 from dual union all
select 15405,'1967-A111','1967-08-25 20:27:28','suborbital spaceplane','success',1520,'3-62-92','Guid/HT/BLN','','','Test',379,'NB-52 003',25 from dual union all
select 15406,'1967-A112','1967-08-30 03:45:00','military missile','failure',982,'G 22604-11T','8K64U UBP-100','','','OT',566,'',0 from dual union all
select 15407,'1967-A113','1967-09-19 15:32:00','suborbital rocket','success',549,'PEPP/SPED-I','40-ft DGB Para','','','Test',1895,'',45 from dual union all
select 15408,'1967-A114','1967-09-01 00:00:00','military missile','success',545,'DASA 6.1','','','','Aeron?',1895,'',40 from dual union all
select 15409,'1967-A115','1967-09-07 00:00:00','atmospheric rocket','success',1065,'Ra32','Ranger 32','','','Test',2,'',20 from dual union all
select 1541,'1953-A14','1953-01-13 00:00:00','atmospheric rocket','success',836,'1249A-167','Nike 1249 R&D 160R','','','Test',1895,'',10 from dual union all
select 15410,'1967-A116','1967-09-08 10:03:00','suborbital rocket','failure',837,'NASA 14.309UI','UIUC Arecibo 3','','','Ionos',1483,'',20 from dual union all
select 15411,'1967-A117','1967-09-12 22:25:00','suborbital rocket','failure',30,'NASA 12.07GT','Rail launch test','','','Test',1869,'',0 from dual union all
select 15412,'1967-A118','1967-09-18 19:05:00','suborbital rocket','failure',837,'NASA 14.334UM','UM Pitot 33','','','Aeronomy',1821,'',20 from dual union all
select 15413,'1967-A119','1967-09-19 00:00:00','military missile','failure',1252,'FLA-12','','','','Test',1888,'',1 from dual union all
select 15414,'1967-A12','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15415,'1967-A120','1967-09-22 00:00:00','suborbital spaceplane','failure',1520,'3- A-93','','','','Test',379,'NB-52',14 from dual union all
select 15416,'1967-A121','1967-10-17 17:45:00','suborbital rocket','success',545,'PEPP/SPED-I','','','','Test',1895,'',55 from dual union all
select 15417,'1967-A122','1967-10-24 00:00:00','military missile','failure',1038,'4L','8K99 LKI-1','Mobile PU','','Test',461,'',0 from dual union all
select 15418,'1967-A123','1967-10-03 00:00:00','suborbital rocket','failure',257,'NASA 12.09GT','Booster test','','','Test',1821,'',2 from dual union all
select 15419,'1967-A124','1967-10-03 21:31:50','suborbital spaceplane','success',1520,'2-53-97','TPS/Ramjet/Mach 6.7','','','Test',379,'NB-52 008',31 from dual union all
select 1542,'1953-A140','1953-03-23 00:00:00','atmospheric rocket','unknown',576,'183','','','','Test',1895,'',10 from dual union all
select 15420,'1967-A125','1967-10-05 18:44:00','atmospheric rocket','success',835,'Smoke 204','','','','Aeronomy',1821,'',5 from dual union all
select 15421,'1967-A126','1967-10-05 19:59:00','atmospheric rocket','success',835,'Smoke 205','','','','Aeronomy',1821,'',6 from dual union all
select 15422,'1967-A127','1967-10-10 00:00:00','military missile','success',1252,'FLA-7','','','','Test',1888,'',6 from dual union all
select 15423,'1967-A128','1967-10-10 14:01:00','suborbital rocket','failure',446,'D-36','ESRO D34/2','','','Solar X/Aeronomy',29,'',1 from dual union all
select 15424,'1967-A129','1967-10-25 00:00:00','test rocket','failure',337,'G3','','','','Test',307,'',10 from dual union all
select 15425,'1967-A13','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15426,'1967-A130','1967-10-26 17:09:00','suborbital rocket','success',848,'CRL AG07.639','Arecibo cal','','','Ionos',1483,'',26 from dual union all
select 15427,'1967-A131','1967-10-27 13:10:00','military missile','failure',168,'81F','','ABRES RMP-B-3/NTMP KX103','','Reentry Test',1682,'',1 from dual union all
select 15428,'1967-A132','1967-11-01 00:00:00','military missile','success',545,'DASA 6.1','','','','Aeron?',1895,'',40 from dual union all
select 15429,'1967-A133','1967-10-31 00:00:00','military missile','success',1252,'FLA-14','','','','Test',1888,'',6 from dual union all
select 1543,'1953-A141','1953-03-24 00:00:00','atmospheric rocket','unknown',576,'182','','','','Test',1895,'',10 from dual union all
select 15430,'1967-A134','1967-10-31 00:00:00','suborbital spaceplane','failure',1520,'3- A-96','','','','Test',379,'NB-52',14 from dual union all
select 15431,'1967-A135','1967-11-20 00:00:00','military missile','failure',1252,'FLA-15','','','','Test',1888,'',6 from dual union all
select 15432,'1967-A136','1967-11-20 03:00:00','suborbital rocket','success',1032,'1','TERLS-54','','','Test',1467,'',9 from dual union all
select 15433,'1967-A137','1967-11-20 04:00:00','suborbital rocket','success',1032,'2','','','','Test',1467,'',9 from dual union all
select 15434,'1967-A138','1967-11-23 00:00:00','suborbital rocket','failure',1301,'002','FU-169','','','XR Astron',313,'',0 from dual union all
select 15435,'1967-A139','1967-12-01 00:00:00','suborbital rocket','success',1032,'3','','','','Test',1467,'',9 from dual union all
select 15436,'1967-A14','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15437,'1967-A140','1967-12-01 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 15438,'1967-A141','1967-12-06 23:03:00','suborbital rocket','success',88,'','Particles','','','Ionos',428,'',42 from dual union all
select 15439,'1967-A142','1967-12-07 00:00:00','atmospheric rocket','success',617,'J205','UK-E/HRV 926','','','Test',1836,'',30 from dual union all
select 1544,'1953-A142','1953-03-24 00:00:00','atmospheric rocket','success',836,'1088E','','','','Test',1895,'',10 from dual union all
select 15440,'1967-A143','1967-12-11 00:00:00','military missile','success',1252,'FLA-13','','','','Test',1888,'',6 from dual union all
select 15441,'1967-A144','1967-12-12 04:19:00','military missile','failure',982,'T 22502-16T','8K64U UBP-107','','','OT',566,'',100 from dual union all
select 15442,'1967-A145','1967-12-14 23:00:00','suborbital rocket','success',894,'Rad 1/67','','','','XR Astron',310,'',35 from dual union all
select 15443,'1967-A146','1967-12-27 04:16:00','military missile','failure',982,'T 22502-15T','8K64U UBP-109','','','OT',566,'',10 from dual union all
select 15444,'1967-A147','1967-03-20 00:00:00','suborbital rocket','failure',364,'DA001','FU-163','','','Test',671,'',1 from dual union all
select 15445,'1967-A148','1967-05-08 15:47:00','suborbital rocket','failure',197,'','','','','Aeron',313,'',0 from dual union all
select 15446,'1967-A149','1967-05-09 17:57:00','suborbital rocket','failure',197,'','','','','Aeron',313,'',0 from dual union all
select 15447,'1967-A15','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15448,'1967-A150','1967-05-11 11:47:00','suborbital rocket','failure',197,'','','','','Aeron',313,'',0 from dual union all
select 15449,'1967-A151','1967-02-15 00:00:00','atmospheric rocket','success',852,'Sandia 158-11','Parachute test','','','Test',1490,'',4 from dual union all
select 1545,'1953-A143','1953-03-26 00:00:00','atmospheric rocket','success',836,'325P','Nike CET 25','','','Test',1895,'',10 from dual union all
select 15450,'1967-A152','1967-11-17 00:00:00','atmospheric rocket','success',852,'Sandia 158-12','Parachute test','','','Test',1490,'',4 from dual union all
select 15451,'1967-A153','1967-06-18 00:00:00','suborbital rocket','success',717,'JD-07-16','Qual Test 1-3','','','Test',339,'',20 from dual union all
select 15452,'1967-A154','1967-06-18 00:00:00','suborbital rocket','success',717,'JD-07-17','Qual Test 1-4','','','Test',339,'',20 from dual union all
select 15453,'1967-A155','1967-07-29 00:00:00','suborbital rocket','success',717,'LD-07-01','Qual Test 2-1','','','Test',339,'',12 from dual union all
select 15454,'1967-A156','1967-07-29 00:00:00','suborbital rocket','success',717,'LD-07-03','Qual Test 2-3','','','Test',339,'',20 from dual union all
select 15455,'1967-A157','1967-06-21 00:00:00','military missile','failure',932,'A3TX-08 (A3P-588)','Antelope 6','','ANTELOPE 6','Target',1897,'EAG-154',10 from dual union all
select 15456,'1967-A158','1967-11-01 00:00:00','military missile','failure',1038,'5L','8K99 LKI-2','','','Test',477,'',0 from dual union all
select 15457,'1967-A159','1967-10-27 00:00:00','atmospheric rocket','success',1536,'1','Low impulse test','','','Test',1343,'',0 from dual union all
select 15458,'1967-A16','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15459,'1967-A160','1967-07-01 00:00:00','suborbital rocket','success',275,'','','','','Test',310,'',40 from dual union all
select 1546,'1953-A144','1953-03-26 00:00:00','atmospheric rocket','success',836,'1046E','','','','Test',1895,'',10 from dual union all
select 15460,'1967-A161','1967-10-20 00:00:00','military missile','failure',702,'TP-16','','','','Test',1895,'',0 from dual union all
select 15461,'1967-A162','1967-06-01 00:00:00','military missile','success',702,'Air drop test','','','','Test',1895,'',10 from dual union all
select 15462,'1967-A163','1967-06-01 00:00:00','military missile','failure',702,'Air drop test','','','','Test',1895,'',0 from dual union all
select 15463,'1967-A17','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15464,'1967-A18','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15465,'1967-A19','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15466,'1967-A20','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15467,'1967-A21','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15468,'1967-A22','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15469,'1967-A23','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 1547,'1953-A145','1953-03-26 00:00:00','atmospheric rocket','success',836,'1074E','','','','Test',1895,'',10 from dual union all
select 15470,'1967-A24','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15471,'1967-A25','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15472,'1967-A26','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15473,'1967-A27','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15474,'1967-A28','1967-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15475,'1967-A29','1967-04-05 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15476,'1967-A30','1967-04-06 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15477,'1967-A31','1967-04-10 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15478,'1967-A32','1967-04-11 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15479,'1967-A33','1967-04-14 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 1548,'1953-A146','1953-03-26 00:00:00','atmospheric rocket','success',836,'323P','Nike CET 23','','','Test',1895,'',10 from dual union all
select 15480,'1967-A34','1967-04-17 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15481,'1967-A35','1967-04-19 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15482,'1967-A36','1967-04-20 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15483,'1967-A37','1967-04-23 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15484,'1967-A38','1967-05-02 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15485,'1967-A39','1967-05-03 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15486,'1967-A40','1967-05-06 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15487,'1967-A41','1967-05-11 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15488,'1967-A42','1967-01-01 00:00:00','military missile','success',753,'','Gymnote 4 RSO Test','','','Test',313,'S655',10 from dual;
insert into LAUNCH
select 15489,'1967-A43','1967-01-01 00:00:00','suborbital rocket','success',826,'NAL-16TR','','','','Test',1476,'',45 from dual union all
select 1549,'1953-A147','1953-03-26 00:00:00','atmospheric rocket','success',836,'324P','Nike CET 24','','','Test',1895,'',10 from dual union all
select 15490,'1967-A44','1967-01-01 07:32:59','suborbital rocket','success',725,'','','','','Meteo',1543,'SHIP-SOV',39 from dual union all
select 15491,'1967-A45','1967-01-06 21:53:00','suborbital rocket','success',725,'','','','','Meteo',1538,'SHIP-SOV',45 from dual union all
select 15492,'1967-A46','1967-01-07 08:53:00','suborbital rocket','success',725,'','','','','Meteo',1537,'SHIP-SOV',44 from dual union all
select 15493,'1967-A47','1967-01-11 09:20:00','suborbital rocket','success',725,'','','','','Meteo',1534,'SHIP-SOV',48 from dual union all
select 15494,'1967-A48','1967-01-11 22:31:00','suborbital rocket','failure',848,'CRL AF07.500','Release Ba','','','Aeron',394,'',20 from dual union all
select 15495,'1967-A49','1967-01-17 09:44:00','suborbital rocket','success',725,'','','','','Meteo',1533,'SHIP-SOV',33 from dual union all
select 15496,'1967-A50','1967-01-19 23:22:00','suborbital rocket','failure',848,'CRL AF07.656','Niro TMA E4','','','Aeron',394,'',20 from dual union all
select 15497,'1967-A51','1967-01-22 12:04:59','suborbital rocket','success',725,'','','','','Meteo',1517,'SHIP-SOV',42 from dual union all
select 15498,'1967-A52','1967-01-23 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1518,'SHIP-SOV',43 from dual union all
select 15499,'1967-A53','1967-01-25 00:00:00','military missile','failure',1449,'','8K84 UBP-46 OT','','','OT',508,'',0 from dual union all
select 155,'1944-S140','1944-09-25 00:00:00','military missile','success',1470,'','Maastricht','','','Weapon',1735,'',95 from dual union all
select 1550,'1953-A148','1953-03-26 00:00:00','atmospheric rocket','success',836,'1249A-194','Nike 1249 R&D 193R','','','Test',1895,'',10 from dual union all
select 15500,'1967-A54','1967-02-28 00:00:00','suborbital rocket','success',545,'PEPP','','','','Test',1895,'',40 from dual union all
select 15501,'1967-A55','1967-02-01 01:09:00','suborbital rocket','failure',837,'NASA 14.315UM','UM Pitot 26/Rel','','','Aeronomy',428,'',0 from dual union all
select 15502,'1967-A56','1967-02-07 09:57:59','suborbital rocket','success',725,'','','','','Meteo',1497,'SHIP-SOV',46 from dual union all
select 15503,'1967-A57','1967-02-08 00:00:00','military missile','unknown',255,'A','Target','','BUSY NEEDLE','Target',1689,'',20 from dual union all
select 15504,'1967-A58','1967-02-09 00:00:00','military missile','failure',135,'B045','Ionizing Pellets','ABRES N-4 (USN)','','Reentry Test',614,'',0 from dual union all
select 15505,'1967-A59','1967-02-10 09:53:00','suborbital rocket','success',725,'','','','','Meteo',1496,'SHIP-SOV',48 from dual union all
select 15506,'1967-A60','1967-02-18 09:53:00','suborbital rocket','success',725,'','','','','Meteo',1509,'SHIP-SOV',31 from dual union all
select 15507,'1967-A61','1967-03-11 06:30:00','suborbital rocket','unknown',1070,'PT-210-2','','','','Test',839,'',1 from dual union all
select 15508,'1967-A62','1967-03-15 00:00:00','suborbital spaceplane','failure',1520,'1- A-117','','','','Test',379,'NB-52',14 from dual union all
select 15509,'1967-A63','1967-03-15 11:10:00','suborbital rocket','success',631,'EXAMETNET C18','[Not in SP-176]','','','Aeron',310,'',32 from dual union all
select 1551,'1953-A149','1953-03-27 00:00:00','atmospheric rocket','unknown',576,'188','','','','Test',1895,'',10 from dual union all
select 15510,'1967-A64','1967-03-17 00:00:00','suborbital rocket','failure',1493,'AGI 64','FU-174 Solar corona','','','Solar UV',668,'',32 from dual union all
select 15511,'1967-A65','1967-03-21 00:00:00','suborbital spaceplane','failure',1520,'1- A-118','','','','Test',379,'NB-52',14 from dual union all
select 15512,'1967-A66','1967-03-22 17:52:04','suborbital spaceplane','success',1520,'1-70-119','Stab./Loads/ASAS','','','Test',379,'NB-52 003',40 from dual union all
select 15513,'1967-A67','1967-05-12 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15514,'1967-A68','1967-04-18 00:00:00','military missile','failure',1252,'FLA-9','','','','Test',1888,'',1 from dual union all
select 15515,'1967-A69','1967-04-18 10:25:00','suborbital rocket','success',848,'CRL AF07.625','Niro TMA E8','','','Aeron',339,'',20 from dual union all
select 15516,'1967-A70','1967-04-20 00:00:00','suborbital spaceplane','failure',1520,'1- A-120','','','','Test',379,'NB-52',14 from dual union all
select 15517,'1967-A71','1967-04-20 20:35:00','suborbital rocket','failure',88,'NASA 15.55GT','Tone ranging test','','','Test',1821,'',47 from dual union all
select 15518,'1967-A72','1967-04-26 19:20:17','suborbital spaceplane','failure',1520,'3-58-87','Telem/HT/Guidance','','','Test',379,'NB-52 008',16 from dual union all
select 15519,'1967-A73','1967-05-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 1552,'1953-A15','1953-01-13 00:00:00','atmospheric rocket','success',836,'1249A-164','Nike 1249 R&D 161R','','','Test',1895,'',10 from dual union all
select 15520,'1967-A74','1967-05-01 00:00:00','atmospheric rocket','success',1065,'','','','','Test',1836,'',30 from dual union all
select 15521,'1967-A75','1967-05-04 00:00:00','military missile','failure',135,'D030','SFT-7','ABRES RR-7 (USN)','','Reentry Test',613,'',0 from dual union all
select 15522,'1967-A76','1967-05-05 00:00:00','suborbital spaceplane','failure',1520,'2- A-91','','','','Test',379,'NB-52',14 from dual union all
select 15523,'1967-A77','1967-05-08 19:27:38','suborbital spaceplane','success',1520,'2-51-92','Ramjet flow','','','Test',379,'NB-52 008',29 from dual union all
select 15524,'1967-A78','1967-05-09 17:45:00','suborbital rocket','success',545,'PEPP 05','40-ft Ringsail','','','Test',1895,'',48 from dual union all
select 15525,'1967-A79','1967-05-10 10:04:59','suborbital rocket','success',725,'','','','','Meteo',636,'',38 from dual union all
select 15526,'1967-A80','1967-05-12 00:00:00','suborbital spaceplane','failure',1520,'3- A-88','','','','Test',379,'NB-52',14 from dual union all
select 15527,'1967-A81','1967-05-13 12:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 15528,'1967-A82','1967-05-17 17:45:48','suborbital spaceplane','success',1520,'3-59-89','Telem/HT/Guidance','','','Test',379,'NB-52 003',21 from dual union all
select 15529,'1967-A83','1967-05-20 00:00:00','suborbital rocket','failure',864,'Sandia 152-97','LRL BOX-9','','','XR Astron',855,'',22 from dual union all
select 1553,'1953-A150','1953-03-27 00:00:00','atmospheric rocket','unknown',576,'189','','','','Test',1895,'',10 from dual union all
select 15530,'1967-A84','1967-05-23 00:00:00','military missile','failure',1252,'FLA-10','','','','Test',1888,'',1 from dual union all
select 15531,'1967-A85','1967-05-25 00:00:00','military missile','unknown',255,'A','Target','','BUSY SPOTTER','Target',1690,'',20 from dual union all
select 15532,'1967-A86','1967-05-25 00:00:00','military missile','unknown',255,'A','Target','','GIANT CIGAR','Target',1689,'',20 from dual union all
select 15533,'1967-A87','1967-05-25 00:00:00','suborbital spaceplane','failure',1520,'1- A-122','','','','Test',379,'NB-52',14 from dual union all
select 15534,'1967-A88','1967-06-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 15535,'1967-A89','1967-05-31 13:00:00','suborbital rocket','failure',837,'NASA 14.300UA','Pandora B','','','Meteorites',1895,'',42 from dual union all
select 15536,'1967-A90','1967-06-01 00:00:00','suborbital spaceplane','failure',1520,'1- A-123','','','','Test',379,'NB-52',14 from dual union all
select 15537,'1967-A91','1967-06-14 00:00:00','suborbital spaceplane','failure',1520,'1- A-124','','','','Test',379,'NB-52',14 from dual union all
select 15538,'1967-A92','1967-06-20 15:45:00','suborbital rocket','success',545,'PEPP 04 [F6]','30-ft Cross Para','','','Test',1895,'',45 from dual union all
select 15539,'1967-A93','1967-06-22 21:57:17','suborbital spaceplane','success',1520,'3-60-90','HT/Telem','','','Test',379,'NB-52 008',25 from dual union all
select 1554,'1953-A151','1953-03-31 00:00:00','military missile','success',771,'GM-11045','','','','Test',222,'',12 from dual union all
select 15540,'1967-A94','1967-07-06 00:00:00','military missile','unknown',255,'A','Target','','BUCKBOARD SEAT','Target',1689,'',20 from dual union all
select 15541,'1967-A95','1967-07-11 00:00:00','atmospheric rocket','success',1066,'Ly6','Lynx 6','','','Test',2,'',20 from dual union all
select 15542,'1967-A96','1967-07-15 12:59:00','military missile','failure',982,'T 22503-12','8K64U UBP-98','','','OT',566,'',0 from dual union all
select 15543,'1967-A97','1967-07-18 00:00:00','military missile','success',1252,'FLA-8','','','','Test',1888,'',2 from dual union all
select 15544,'1967-A98','1967-07-20 17:11:00','suborbital spaceplane','success',1520,'3-61-91','HT/Guid/Telem','','','Test',379,'NB-52 008',25 from dual union all
select 15545,'1967-A99','1967-07-27 00:00:00','military missile','failure',1472,'','Dummy test','','','ABM Test',183,'',0 from dual union all
select 15546,'1967-F01','1967-01-31 12:45:01','orbital','failure',1128,'S151C','OV3-5','ATCOS 1','BUSY MASON','Sat',1779,'',430 from dual union all
select 15547,'1967-F02','1967-03-22 14:05:02','orbital','failure',1007,'N 22500-08L','','OGCh No. 08?','','Sat',518,'',140 from dual union all
select 15548,'1967-F03','1967-04-13 02:40:00','orbital','failure',699,'L-4S-3','','L-4S-3','','Sat',834,'',200 from dual union all
select 15549,'1967-F04','1967-04-26 18:00:00','orbital','failure',1424,'3B-5','KH-8','GAMBIT-3 Mission 4305','BUSY TAILOR','Sat',1778,'',130 from dual union all
select 1555,'1953-A152','1953-03-31 00:00:00','atmospheric rocket','success',836,'1249B-1102','Nike 1249 R&D 195R','','','Test',1895,'',10 from dual union all
select 15550,'1967-F05','1967-05-30 02:06:00','orbital','failure',1128,'S152C','ESRO 2A','ESRO 2A','OLD FAD','Sat',1779,'',350 from dual union all
select 15551,'1967-F06','1967-06-20 11:00:00','orbital','failure',1508,'Ya15001- 13','','Zenit-4 No. 32','','Sat',473,'',2 from dual union all
select 15552,'1967-F07','1967-06-26 04:30:00','orbital','failure',680,'N149    -20LM','','Tselina-O GVM','','Sat',457,'',10 from dual union all
select 15553,'1967-F08','1967-07-21 06:00:04','orbital','failure',1508,'Ya15001- 14','','Zenit-4 No. 33','','Sat',562,'',200 from dual union all
select 15554,'1967-F09','1967-09-01 10:30:00','orbital','failure',1508,'Ya15001- 03','','Zenit-2 No. 51','','Sat',473,'',200 from dual union all
select 15555,'1967-F10','1967-09-27 11:00:00','orbital','failure',680,'N149    -15LM','','Tsiklon GVM','','Sat',457,'',0 from dual union all
select 15556,'1967-F11','1967-09-27 22:11:54','orbital','failure',953,'229-01','','L-1 No. 4L','','Sat',587,'',30 from dual union all
select 15557,'1967-F12','1967-11-22 19:07:59','orbital','failure',953,'230-01','','L-1 No. 5L','','Sat',588,'',50 from dual union all
select 15558,'1967-M01','1967-01-01 00:00:00','military missile','unknown',753,'','Caisson 3','','','Test',312,'',50 from dual union all
select 15559,'1967-M02','1967-01-01 00:00:00','military missile','success',753,'','Gymnote 2','','','Test',313,'S655',50 from dual union all
select 1556,'1953-A153','1953-03-31 00:00:00','atmospheric rocket','success',836,'1099E','','','','Test',1895,'',10 from dual union all
select 15560,'1967-M03','1967-01-01 00:00:00','military missile','success',753,'','Gymnote 3','','','Test',313,'S655',50 from dual union all
select 15561,'1967-M04','1967-01-01 00:00:00','military missile','success',749,'','Caisson 1','','','Test',312,'',50 from dual union all
select 15562,'1967-M05','1967-01-11 20:28:00','suborbital rocket','success',278,'IFTV','Marquardt Scramjet','','TREE MOSS','Tech',1661,'',54 from dual union all
select 15563,'1967-M06','1967-03-02 00:00:00','military missile','unknown',1104,'2606','Silo 4','','','Test',301,'',50 from dual union all
select 15564,'1967-M07','1967-04-18 10:25:00','suborbital rocket','success',848,'CRL AF07.624','Niro TMA E7/LangP','','','Aeron',394,'',58 from dual union all
select 15565,'1967-M08','1967-04-19 00:00:00','military missile','success',753,'','Gymnote 1','','','Test',313,'S655',50 from dual union all
select 15566,'1967-M09','1967-04-28 17:23:32','suborbital spaceplane','success',1520,'1-71-121','WTR/HS/E loads','','','Test/Tech',379,'NB-52 003',50 from dual union all
select 15567,'1967-M10','1967-05-11 08:55:00','suborbital rocket','success',839,'NASA 10.161GM','GSFC Ozone 2','','','Aeronomy',1821,'',70 from dual union all
select 15568,'1967-M11','1967-06-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 15569,'1967-M12','1967-06-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 1557,'1953-A154','1953-03-31 00:00:00','atmospheric rocket','success',836,'1249B-1104','Nike 1249 R&D 194R','','','Test',1895,'',10 from dual union all
select 15570,'1967-M13','1967-06-15 18:09:28','suborbital spaceplane','success',1520,'1-72-125','HS/Stab/WTR','','','Test/Tech',379,'NB-52 003',69 from dual union all
select 15571,'1967-M134','1967-01-30 17:03:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 15572,'1967-M135','1967-01-31 16:08:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 15573,'1967-M136','1967-02-27 15:00:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 15574,'1967-M137','1967-02-28 16:30:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 15575,'1967-M138','1967-04-11 00:00:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 15576,'1967-M139','1967-04-12 00:00:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 15577,'1967-M14','1967-06-29 18:27:51','suborbital spaceplane','failure',1520,'1-73-126','WTR/HS/ASAS','','','Test/Tech',379,'NB-52 008',52 from dual union all
select 15578,'1967-M140','1967-05-09 14:21:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual;
insert into LAUNCH
select 15579,'1967-M141','1967-05-09 00:00:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 1558,'1953-A155','1953-04-01 00:00:00','military missile','success',771,'GM-11044','','','','Test',222,'',12 from dual union all
select 15580,'1967-M142','1967-05-12 15:27:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 15581,'1967-M143','1967-05-12 16:27:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 15582,'1967-M144','1967-05-18 11:15:00','suborbital rocket','success',716,'','Ballute','','','Test',293,'',60 from dual union all
select 15583,'1967-M145','1967-04-27 00:00:00','military missile','success',272,'','HIGHBALL MAR test','','','Radar targets',1895,'',60 from dual union all
select 15584,'1967-M146','1967-05-16 00:00:00','suborbital rocket','success',330,'','JTF-8 exercise','','','Atmos',1057,'RB-57C',50 from dual union all
select 15585,'1967-M147','1967-05-16 00:00:00','suborbital rocket','success',330,'','','','','Atmos',1057,'RB-57C',50 from dual union all
select 15586,'1967-M148','1967-05-16 00:00:00','suborbital rocket','success',330,'','','','','Atmos',1057,'RB-57C',50 from dual union all
select 15587,'1967-M149','1967-05-16 00:00:00','suborbital rocket','success',330,'','','','','Atmos',1057,'RB-57C',50 from dual union all
select 15588,'1967-M15','1967-07-25 00:00:00','military missile','unknown',1102,'','S01V-1','','','Test',301,'',50 from dual union all
select 15589,'1967-M150','1967-05-16 00:00:00','suborbital rocket','success',330,'','','','','Atmos',1057,'RB-57C',50 from dual union all
select 1559,'1953-A156','1953-04-02 00:00:00','military missile','unknown',339,'1260RS','','','','Test',1895,'',30 from dual union all
select 15590,'1967-M151','1967-05-16 00:00:00','suborbital rocket','success',330,'','','','','Atmos',1057,'RB-57C',50 from dual union all
select 15591,'1967-M152','1967-05-16 00:00:00','suborbital rocket','success',330,'','','','','Atmos',1057,'RB-57C',50 from dual union all
select 15592,'1967-M153','1967-05-16 00:00:00','suborbital rocket','success',330,'','','','','Atmos',1057,'RB-57C',50 from dual union all
select 15593,'1967-M154','1967-12-01 00:00:00','suborbital rocket','success',361,'','Dardo de Invest. Met.','','','Meteo',310,'',70 from dual union all
select 15594,'1967-M155','1967-12-01 00:00:00','suborbital rocket','success',361,'','Dardo de Invest. Met.','','','Meteo',310,'',70 from dual union all
select 15595,'1967-M156','1967-12-01 00:00:00','suborbital rocket','success',361,'','Dardo de Invest. Met.','','','Meteo',310,'',70 from dual union all
select 15596,'1967-M157','1967-01-19 23:29:00','suborbital rocket','success',88,'','','','','Aeron',394,'',60 from dual union all
select 15597,'1967-M158','1967-01-19 23:34:00','suborbital rocket','success',88,'','','','','Aeron',394,'',60 from dual union all
select 15598,'1967-M16','1967-09-25 00:00:00','suborbital rocket','success',1431,'Sandia 152-61','','','','Test/Aeron?',803,'',80 from dual union all
select 15599,'1967-M17','1967-10-04 17:16:54','suborbital spaceplane','success',1520,'3-63-94','UV Plume/Solar/MM','','','Tech/Meteor/Solar',379,'NB-52 003',76 from dual union all
select 156,'1944-S141','1944-09-25 00:00:00','military missile','success',1470,'','Lille','','','Weapon',1735,'',95 from dual union all
select 1560,'1953-A157','1953-04-02 00:00:00','atmospheric rocket','success',836,'327P','Nike CET 27','','','Test',1895,'',10 from dual union all
select 15600,'1967-M18','1967-11-10 00:00:00','military missile','unknown',1102,'','S01V-2','','','Test',301,'',50 from dual union all
select 15601,'1967-M19','1967-12-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 15602,'1967-M20','1967-12-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 15603,'1967-M21','1967-12-16 14:58:00','suborbital rocket','success',894,'Rad 2/67','','','','XR Astron',310,'',70 from dual union all
select 15604,'1967-M22','1967-01-30 14:30:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 15605,'1967-M23','1967-01-31 15:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 15606,'1967-M24','1967-03-01 15:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 15607,'1967-M25','1967-03-02 16:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 15608,'1967-M26','1967-04-12 15:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 15609,'1967-M27','1967-04-19 15:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 1561,'1953-A158','1953-04-02 00:00:00','atmospheric rocket','success',836,'326P','Nike CET 26','','','Test',1895,'',10 from dual union all
select 15610,'1967-M28','1967-05-10 15:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 15611,'1967-M29','1967-05-10 16:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 15612,'1967-M30','1967-07-06 00:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 15613,'1967-M31','1967-08-16 00:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 15614,'1967-M32','1967-10-27 00:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 15615,'1967-M33','1967-12-18 00:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 15616,'1967-S01','1967-01-01 00:00:00','military missile','success',736,'','','','','Test',313,'',100 from dual union all
select 15617,'1967-S02','1967-01-01 00:00:00','military missile','success',736,'','','','','Test',313,'',100 from dual union all
select 15618,'1967-S03','1967-01-01 00:00:00','military missile','failure',736,'','','','','Test',313,'',100 from dual union all
select 15619,'1967-S04','1967-01-01 00:00:00','military missile','failure',736,'','','','','Test',313,'',100 from dual union all
select 1562,'1953-A159','1953-04-02 00:00:00','atmospheric rocket','success',836,'1249B-1183','Nike 1249 R&D 197R','','','Test',1895,'',10 from dual union all
select 15620,'1967-S05','1967-01-01 00:00:00','military missile','success',736,'','','','','Test',313,'',100 from dual union all
select 15621,'1967-S06','1967-11-01 00:00:00','suborbital rocket','success',706,'02B/1','','','','Test',313,'',115 from dual union all
select 15622,'1967-S07','1967-11-01 00:00:00','suborbital rocket','success',706,'02B/2','','','','Test',313,'',115 from dual union all
select 15623,'1967-S08','1967-11-01 00:00:00','suborbital rocket','success',706,'02B/3','SECT Meteo','','','Meteo',313,'',80 from dual union all
select 15624,'1967-S09','1967-01-01 00:00:00','military missile','success',993,'','','','','Test',1626,'GOLF2',293 from dual union all
select 15625,'1967-S10','1967-10-31 11:07:00','military missile','success',1018,'G 15003-10','8K75 UBP-11','','','OT',584,'',1160 from dual union all
select 15626,'1967-S100','1967-02-01 03:46:00','suborbital rocket','success',837,'NASA 14.317UM','UM Pitot 27/Rel','','','Aeronomy',428,'',139 from dual union all
select 15627,'1967-S101','1967-02-01 03:49:00','suborbital rocket','success',837,'NASA 14.312CM','GCA Release 61 TMA','','','Aeronomy',428,'',193 from dual union all
select 15628,'1967-S102','1967-02-01 04:18:00','suborbital rocket','success',839,'NASA 10.218GM','GSFC Grenade 138 PB','','','Aeronomy',1039,'',126 from dual union all
select 15629,'1967-S103','1967-02-01 05:38:00','suborbital rocket','success',837,'NASA 14.318UM','UM Pitot 28/Rel','','','Aeronomy',428,'',164 from dual union all
select 1563,'1953-A16','1953-01-14 00:00:00','atmospheric rocket','success',968,'','Test','','','Test',1821,'',10 from dual union all
select 15630,'1967-S104','1967-02-01 05:41:00','suborbital rocket','success',837,'NASA 14.313CM','GCA Release 62 TMA','','','Aeronomy',428,'',194 from dual union all
select 15631,'1967-S105','1967-02-01 07:41:00','suborbital rocket','success',839,'NASA 10.219GM','GSFC Grenade 139 PB','','','Aeronomy',1039,'',126 from dual union all
select 15632,'1967-S106','1967-02-01 08:25:59','suborbital rocket','success',837,'NASA 14.316UM','UM Pitot 29/Rel','','','Aeronomy',428,'',160 from dual union all
select 15633,'1967-S107','1967-02-01 08:29:00','suborbital rocket','success',837,'NASA 14.323CM','GCA Release 63 TMA','','','Aeronomy',428,'',193 from dual union all
select 15634,'1967-S108','1967-02-01 09:56:00','suborbital rocket','success',839,'NASA 10.220GM','GSFC Grenade 140 PB','','','Aeronomy',1039,'',126 from dual union all
select 15635,'1967-S109','1967-02-01 11:24:00','suborbital rocket','success',837,'NASA 14.314CM','GCA Release 64 TMA','','','Aeronomy',428,'',138 from dual union all
select 15636,'1967-S11','1967-12-23 09:04:00','military missile','success',1018,'G 15003-01','8K75 UBP-12','','','OT',578,'',1160 from dual union all
select 15637,'1967-S110','1967-02-01 11:58:00','suborbital rocket','success',837,'NASA 14.322UM','UM Pitot 30/Rel','','','Aeronomy',428,'',153 from dual union all
select 15638,'1967-S111','1967-02-01 14:26:00','suborbital rocket','success',839,'NASA 10.221GM','GSFC Grenade 141 PB','','','Aeronomy',1039,'',126 from dual union all
select 15639,'1967-S112','1967-02-03 03:45:00','military missile','success',796,'2087','ERCS 2','494L (SYCAMORE TREE)','SYCAMORE TREE','Comms',1700,'',900 from dual union all
select 1564,'1953-A160','1953-04-02 00:00:00','atmospheric rocket','success',836,'1249B-1181','Nike 1249 R&D 196R','','','Test',1895,'',10 from dual union all
select 15640,'1967-S113','1967-02-03 21:00:00','suborbital rocket','success',740,'','Airglow','','','Solar/Aeron',636,'',153 from dual union all
select 15641,'1967-S114','1967-02-03 17:52:00','suborbital rocket','success',839,'NASA 10.222GM','GSFC Grenade 142 W','','','Aeronomy',1821,'',112 from dual union all
select 15642,'1967-S115','1967-02-04 05:21:00','suborbital rocket','success',215,'AA-2-101','Rec test/Ionos','','','Auroral/Ionos/Meteo',428,'',183 from dual union all
select 15643,'1967-S116','1967-02-04 23:53:31','suborbital rocket','success',288,'CE-22','ESRO C21/2','','','Auroral/CR',408,'',126 from dual union all
select 15644,'1967-S117','1967-02-06 02:10:00','sounding rocket','success',698,'L-3H-3','','','','Ionos/CR/XR Astron',834,'',2150 from dual union all
select 15645,'1967-S118','1967-02-07 00:00:00','military missile','success',795,'893','Mk 11 RV','FOT','PIANO TUNER','OT',1702,'',900 from dual union all
select 15646,'1967-S119','1967-02-09 06:16:00','suborbital rocket','success',624,'NASA 8.41UE','Rice auroral','','','Plasma',428,'',752 from dual union all
select 15647,'1967-S12','1967-01-01 00:00:00','military missile','success',1036,'2','LKI 2','','','Test',636,'',1000 from dual union all
select 15648,'1967-S120','1967-02-13 22:05:00','military missile','success',162,'121F','TVX-22','ABRES TX-22','BUSY BOXER','Reentry Test',1682,'',1400 from dual union all
select 15649,'1967-S121','1967-02-16 07:37:15','suborbital rocket','success',288,'CE-9','ESRO C10/1','','','Ionospheric',408,'',148 from dual union all
select 1565,'1953-A161','1953-04-08 00:00:00','military missile','success',703,'622','','','','Test',222,'',8 from dual union all
select 15650,'1967-S122','1967-02-17 04:18:00','suborbital rocket','success',30,'NASA 04.163UA','JHU Aurora 5','','','Auroral',428,'',168 from dual union all
select 15651,'1967-S123','1967-02-21 22:40:00','military missile','success',795,'700','Mk 11 RV','FOT Ripple','GLORY TRAIL','OT',1698,'',900 from dual union all
select 15652,'1967-S124','1967-02-21 22:40:00','military missile','success',795,'1263','Mk 11 RV','FOT Ripple','BUDDY BOY','OT',1705,'',900 from dual union all
select 15653,'1967-S125','1967-02-22 07:12:25','suborbital rocket','success',288,'CE-10','ESRO C10/2','','','Ionospheric',408,'',155 from dual union all
select 15654,'1967-S126','1967-02-23 00:00:00','military missile','success',135,'D037','SFT-5','ABRES RR-5 (USN)','','Reentry Test',614,'',200 from dual union all
select 15655,'1967-S127','1967-02-24 00:00:00','suborbital rocket','success',1491,'VA81','FU-176 Recovery test','','','Test',668,'',200 from dual union all
select 15656,'1967-S128','1967-02-25 02:05:02','military missile','success',796,'465','Mk 12 RV','R&D','','Test',266,'',900 from dual union all
select 15657,'1967-S129','1967-02-25 02:35:00','sounding rocket','success',129,'NASA 16.05US','','','','Solar RF?',1811,'',2380 from dual union all
select 15658,'1967-S13','1967-01-01 00:00:00','military missile','success',1036,'3','LKI 3','','','Test',636,'',1000 from dual union all
select 15659,'1967-S130','1967-03-01 00:00:00','suborbital rocket','success',846,'STV','','','','Aeron?',1895,'',100 from dual union all
select 1566,'1953-A162','1953-04-08 00:00:00','military missile','failure',1009,'','8A62 LKI I','','','Test',636,'',0 from dual union all
select 15660,'1967-S131','1967-03-06 16:00:00','military missile','success',916,'AO-45','P-117 67 Spring 01','','','Test',204,'',180 from dual union all
select 15661,'1967-S132','1967-02-28 03:47:00','suborbital rocket','success',864,'Sandia 152-100','Aurora','','','Auroral',428,'',167 from dual union all
select 15662,'1967-S133','1967-03-02 13:45:01','military missile','success',932,'A3TX-4 (A3P-531)','Antelope 3','','ANTELOPE 3','Test',251,'',1000 from dual union all
select 15663,'1967-S134','1967-03-02 00:00:00','military missile','success',135,'C048','HAPDEC','ABRES AB-1 (ARPA)','','Reentry Test',613,'',200 from dual union all
select 15664,'1967-S135','1967-03-03 03:30:00','suborbital rocket','success',30,'NASA 04.160GG','UV stellar photom','','','UV Astron',1869,'',157 from dual union all
select 15665,'1967-S136','1967-03-03 04:20:00','suborbital rocket','success',30,'NASA 04.84UG','IR payload','','','IR Astron',1869,'',158 from dual union all
select 15666,'1967-S137','1967-03-03 23:58:00','suborbital rocket','success',837,'Ferdinand 14','Ferdinand 14','','','Ionos/Plasma',29,'',190 from dual union all
select 15667,'1967-S139','1967-03-04 05:03:00','military missile','success',999,'Ya22501-14T','8K67 UBP-9','','','OT',485,'',1000 from dual union all
select 15668,'1967-S14','1967-08-01 00:00:00','military missile','success',1036,'4','LKI 4','','','Test',636,'',1000 from dual;
insert into LAUNCH
select 15669,'1967-S140','1967-03-04 21:05:00','suborbital rocket','success',848,'CRL AF07.582','Sphere','','','Aeron',394,'',196 from dual union all
select 1567,'1953-A163','1953-04-09 00:00:00','atmospheric rocket','success',836,'329P','Nike CET 29','','','Test',1895,'',10 from dual union all
select 15670,'1967-S141','1967-03-06 02:45:01','military missile','unknown',932,'A3P-549','','','','OT',421,'SSBN 658',1000 from dual union all
select 15671,'1967-S142','1967-03-06 03:05:00','suborbital rocket','success',848,'CRL AF07.583','Sphere','','','Aeron',394,'',175 from dual union all
select 15672,'1967-S143','1967-03-06 13:27:00','suborbital rocket','success',837,'NASA 14.161IA','ISRO 10.09 (Release)','TERLS-046','','Aeronomy',1467,'',178 from dual union all
select 15673,'1967-S144','1967-03-07 00:00:00','military missile','success',795,'(64-002)','Mk 11 RV','FOT','OLD SAL','OT',1702,'',900 from dual union all
select 15674,'1967-S145','1967-03-07 10:44:00','suborbital rocket','success',1498,'004','FU-147 Bio (Macaca)','','','Bio',668,'',243 from dual union all
select 15675,'1967-S146','1967-03-07 11:00:00','suborbital rocket','success',655,'K-10-3','FUV Stellar Phot','','','UV Astron,Solar X',839,'',248 from dual union all
select 15676,'1967-S147','1967-03-09 05:07:00','suborbital rocket','success',867,'NASA 18.24GE','Auroral electrons','','','Auroral',428,'',227 from dual union all
select 15677,'1967-S148','1967-03-09 13:27:00','suborbital rocket','success',837,'NASA 14.162IA','ISRO 10.10','TERLS-047','','Aeronomy',1467,'',184 from dual union all
select 15678,'1967-S149','1967-03-12 13:27:00','suborbital rocket','success',837,'NASA 14.163IA-II','ISRO 10.11','TERLS-048','','Ionos',1467,'',168 from dual union all
select 15679,'1967-S150','1967-03-12 17:00:00','suborbital rocket','success',837,'NASA 14.206IA','ISRO 10.12','TERLS-049','','Ionos/Aeronomy',1467,'',155 from dual union all
select 1568,'1953-A164','1953-04-09 00:00:00','atmospheric rocket','success',836,'328P','Nike CET 28','','','Test',1895,'',10 from dual union all
select 15680,'1967-S151','1967-03-13 00:00:00','military missile','success',916,'AO-47','P-119 67 Spring 03','','','OT',204,'',180 from dual union all
select 15681,'1967-S152','1967-03-13 00:00:00','military missile','success',916,'AO-48','P-120 67 Spring 04','','','OT',204,'',180 from dual union all
select 15682,'1967-S153','1967-03-13 10:30:00','suborbital rocket','success',1498,'005','FU-147 Bio (Macaca)','','','Bio',668,'',234 from dual union all
select 15683,'1967-S154','1967-03-13 00:14:00','suborbital rocket','success',837,'NASA 14.267IA','ISRO 10.14','TERLS-050','','Ionos/Aeronomy',1467,'',155 from dual union all
select 15684,'1967-S155','1967-03-14 06:00:00','suborbital rocket','success',1207,'SL406','LUX-36AC7/Cul-7','','','Solar X,UV',1839,'',200 from dual union all
select 15685,'1967-S156','1967-03-14 17:14:00','suborbital rocket','success',35,'NASA 04.102DS','OSO-3 calibration','','','Solar EUV',1821,'',205 from dual union all
select 15686,'1967-S157','1967-03-14 23:46:10','suborbital rocket','success',837,'Ferdinand 15','Ferdinand 15','','','Ionos',29,'',188 from dual union all
select 15687,'1967-S158','1967-03-16 00:00:00','military missile','success',135,'B012','IMPALA 1','ABRES YY-2 (USN)','','Reentry Test',612,'',200 from dual union all
select 15688,'1967-S159','1967-03-16 17:56:00','military missile','success',162,'151F','MBRV No. 601','ABRES MBRV-2','LITTLE CHURCH','Reentry Test',1681,'',1400 from dual union all
select 15689,'1967-S160','1967-03-16 19:05:00','suborbital rocket','success',837,'NASA 14.256II','','','','Ionos',1821,'',141 from dual union all
select 1569,'1953-A165','1953-04-10 00:00:00','atmospheric rocket','success',442,'','F36 Ramjet Model','','','Test',1821,'',10 from dual union all
select 15690,'1967-S161','1967-03-17 00:00:00','military missile','success',135,'D040','ABRES QQ-4 (USAF)','ABRES QQ-4 (USAF)','','Reentry Test',614,'',200 from dual union all
select 15691,'1967-S162','1967-03-17 00:00:00','military missile','success',135,'D051','ABRES SS-5 (USAF)','ABRES SS-5 (USAF)','','Reentry Test',613,'',200 from dual union all
select 15692,'1967-S163','1967-03-17 00:00:00','military missile','success',1420,'B-76','Mk 6 RV','FOT/NTMP KX-58','GIFT HORSE','OT',1658,'',1300 from dual union all
select 15693,'1967-S164','1967-03-17 03:48:00','suborbital rocket','success',30,'NASA 04.194DG','NRL NB3.191/FUIIS','','','UV Astron',1869,'',193 from dual union all
select 15694,'1967-S165','1967-03-18 05:47:00','suborbital rocket','success',624,'NASA 8.47UE','Rice TWINS 1 auroral','','','Plasma',428,'',794 from dual union all
select 15695,'1967-S167','1967-03-22 20:30:00','military missile','success',795,'1253','Mk 11 RV','FOT','GLYCOL JELL','OT',1698,'',900 from dual union all
select 15696,'1967-S168','1967-03-22 16:56:00','suborbital rocket','success',35,'NASA 04.103DS','OSO-3 calibration','','','Solar EUV',1821,'',229 from dual union all
select 15697,'1967-S169','1967-03-24 05:55:00','suborbital rocket','success',1492,'VA86','FU-156 Ionos/Bio','','','Ionos/Plasma/Bio',668,'',366 from dual union all
select 15698,'1967-S17','1967-02-26 00:00:00','military missile','failure',1037,'','8K98 LKI-2-4','','','Test',477,'',1000 from dual union all
select 15699,'1967-S170','1967-03-27 18:35:00','suborbital rocket','success',867,'NASA 18.28UE','UNH neutrons','','','Plasma',330,'',220 from dual union all
select 157,'1944-S142','1944-09-25 00:00:00','military missile','success',1470,'','Liege','','','Weapon',1735,'',95 from dual union all
select 1570,'1953-A166','1953-04-14 00:00:00','atmospheric rocket','success',836,'1249A-199','Nike 1249 R&D 201R','','','Test',1895,'',10 from dual union all
select 15700,'1967-S172','1967-03-29 00:30:00','military missile','success',916,'','P-122 67 Spring 06','','','OT',204,'',180 from dual union all
select 15701,'1967-S173','1967-03-29 00:30:00','military missile','success',916,'','P-123 67 Spring 07','','','OT',204,'',180 from dual union all
select 15702,'1967-S174','1967-03-29 01:00:00','military missile','success',916,'','P-124 67 Spring 08','','','OT',204,'',180 from dual union all
select 15703,'1967-S175','1967-03-28 00:00:00','suborbital rocket','success',288,'C145','CNES FU-177','','','Aeronomy',664,'',163 from dual union all
select 15704,'1967-S176','1967-03-29 06:25:00','suborbital rocket','success',1492,'VA87','FU-156 Ionos/Bio','','','Ionos/Plasma',668,'',306 from dual union all
select 15705,'1967-S177','1967-03-30 00:00:00','military missile','success',135,'C049','HAPDEC','ABRES AB-2 (ARPA)','','Reentry Test',613,'',200 from dual union all
select 15706,'1967-S178','1967-03-30 00:00:00','military missile','success',135,'D046','ABRES SS-7 (USAF)','ABRES SS-7 (USAF)','','Reentry Test',614,'',200 from dual union all
select 15707,'1967-S179','1967-03-30 19:09:00','suborbital rocket','success',867,'NASA 18.12UI','GCA Ion Probe','','','Ionos',1821,'',237 from dual union all
select 15708,'1967-S18','1967-04-19 00:00:00','military missile','success',1037,'Ya23102-27L','8K98 LKI-2-5','','','Test',477,'',1000 from dual union all
select 15709,'1967-S180','1967-04-01 00:00:00','suborbital rocket','success',202,'','','','','Weather',313,'',100 from dual union all
select 1571,'1953-A167','1953-04-14 00:00:00','atmospheric rocket','success',836,'1086E','','','','Test',1895,'',10 from dual union all
select 15710,'1967-S181','1967-04-01 00:00:00','military missile','success',1548,'','','','','Test',636,'',620 from dual union all
select 15711,'1967-S182','1967-03-31 02:48:00','military missile','success',999,'Ya22502-14T','8K67 UBP-10','','','OT',485,'',1000 from dual union all
select 15712,'1967-S183','1967-03-31 00:00:00','military missile','success',135,'E002','FATE I-06','ABRES I-6 (USAF)','','Reentry Test',612,'',200 from dual union all
select 15713,'1967-S184','1967-03-31 00:01:00','suborbital rocket','success',867,'NASA 18.09GE','Ba Release','','','Plasma',1821,'',321 from dual union all
select 15714,'1967-S185','1967-03-31 03:03:00','suborbital rocket','success',837,'NASA 14.283CA','GCA Release 65 Teb','','','Aeronomy',1821,'',176 from dual union all
select 15715,'1967-S186','1967-03-31 06:44:00','military missile','success',1373,'168','AFP-437 CEL','','','Interceptor',772,'',484 from dual union all
select 15716,'1967-S187','1967-03-31 17:19:00','suborbital rocket','success',839,'NASA 10.208GM','GSFC Grenade 143 W','','','Aeronomy',1821,'',115 from dual union all
select 15717,'1967-S188','1967-04-01 03:40:00','suborbital rocket','success',30,'NASA 04.204GG','UV stellar photom','','','UV Astron',1869,'',139 from dual union all
select 15718,'1967-S189','1967-04-03 23:25:01','military missile','unknown',932,'A3P-401','','','','OT',421,'SSBN 657',1000 from dual union all
select 15719,'1967-S19','1967-04-29 00:00:00','military missile','success',1037,'Ya23101-28L','8K98 LKI-2-6','','','Test',477,'',1000 from dual union all
select 1572,'1953-A168','1953-04-14 00:00:00','atmospheric rocket','success',836,'1082E','','','','Test',1895,'',10 from dual union all
select 15720,'1967-S190','1967-04-04 00:00:00','suborbital rocket','success',1492,'VA88','FU-178 PUV-3','','','UV Astron/XR Astron',668,'',196 from dual union all
select 15721,'1967-S191','1967-04-04 00:10:00','suborbital rocket','success',839,'NASA 10.226GM','GSFC Grenade 144 PB','','','Aeronomy',1039,'',120 from dual union all
select 15722,'1967-S192','1967-04-04 00:32:00','suborbital rocket','success',1206,'SL426','UAT-1/UCW-12','','','XR Astron/Ionos',1839,'',220 from dual union all
select 15723,'1967-S193','1967-04-05 00:15:00','suborbital rocket','success',30,'NASA 04.168CS','LMSC Solar/Mod 1','','','Solar X/XR Astron',1869,'',160 from dual union all
select 15724,'1967-S194','1967-04-06 00:00:00','military missile','success',135,'B031','Ionizing Pellets','ABRES N-5 (USN)','','Reentry Test',612,'',200 from dual union all
select 15725,'1967-S195','1967-04-06 00:00:00','military missile','success',135,'D043','SFT-6','ABRES RR-6 (USN)','','Reentry Test',612,'',200 from dual union all
select 15726,'1967-S196','1967-04-07 21:00:00','military missile','failure',795,'1426','Mk 11 RV','FOT','BUCK PASSER','OT',1705,'',10 from dual union all
select 15727,'1967-S197','1967-04-07 08:25:00','suborbital rocket','success',30,'NASA 04.186UG','','','','UV Astron',1869,'',176 from dual union all
select 15728,'1967-S198','1967-04-07 11:19:23','ballistic missile test','success',154,'38D','Mk 11 AX-1 RV/Pod 30','ABRES AX-1','BUSY SUNRISE','Reentry Test/Ionos',1684,'',1800 from dual union all
select 15729,'1967-S199','1967-04-07 19:57:00','suborbital rocket','success',837,'NASA 14.328IE','DLR K-NA-1 Ba','','','Auroral/B',413,'',231 from dual union all
select 1573,'1953-A169','1953-04-14 00:00:00','atmospheric rocket','success',836,'1249A-198','Nike 1249 R&D 200R','','','Test',1895,'',10 from dual union all
select 15730,'1967-S20','1967-05-20 00:00:00','military missile','failure',1037,'','8K98 LKI-2-7','','','Test',477,'',1000 from dual union all
select 15731,'1967-S200','1967-04-08 20:03:00','suborbital rocket','success',837,'NASA 14.329IE','DLR K-NA-2 Ba','','','Auroral/B',413,'',237 from dual union all
select 15732,'1967-S201','1967-04-09 01:00:00','suborbital rocket','success',837,'NASA 14.330IE','DLR K-NA-3 Ba','','','Auroral/B',413,'',242 from dual union all
select 15733,'1967-S202','1967-04-10 10:51:00','suborbital rocket','success',1199,'SL118','LUX XRS2/QUB AGP4','','','XR Astronomy',1839,'',167 from dual union all
select 15734,'1967-S203','1967-04-10 20:14:00','suborbital rocket','success',837,'NASA 14.331IE','DLR K-NA-4 Ba','','','Auroral/B',413,'',230 from dual union all
select 15735,'1967-S204','1967-04-10 21:00:00','suborbital rocket','success',839,'NASA 10.227GM','GSFC Grenade 145 PB','','','Aeronomy',1039,'',120 from dual union all
select 15736,'1967-S205','1967-04-11 17:15:00','suborbital rocket','success',839,'NASA 10.223GM','GSFC Grenade 146 W','','','Aeronomy',1821,'',126 from dual union all
select 15737,'1967-S206','1967-04-11 20:21:00','suborbital rocket','success',837,'NASA 14.332IE','DLR K-NA-5','','','Auroral',413,'',236 from dual union all
select 15738,'1967-S207','1967-04-12 00:00:00','military missile','success',135,'B020','FATE I-03','ABRES I-3 (USAF)','','Reentry Test',614,'',200 from dual union all
select 15739,'1967-S208','1967-04-12 00:00:00','military missile','success',1420,'B-81','Mk 6 RV','FOT','GLAMOR GIRL','OT',1657,'',1300 from dual union all
select 1574,'1953-A17','1953-01-14 00:00:00','military missile','failure',771,'GM-11046','','','','Test',222,'',12 from dual union all
select 15740,'1967-S209','1967-04-12 05:22:00','suborbital rocket','success',624,'NASA 8.39GI','ISIS test/overflight','','','Ionos',428,'',870 from dual union all
select 15741,'1967-S21','1967-06-17 00:00:00','military missile','success',1037,'Ya10300-23L','8K98 LKI-2-8','','','Test',477,'',1000 from dual union all
select 15742,'1967-S210','1967-04-12 12:32:00','suborbital rocket','success',1199,'SL119','LUX XRS3/QUB AGP5','','','XR Astronomy',1839,'',160 from dual union all
select 15743,'1967-S211','1967-04-13 00:01:00','suborbital rocket','success',848,'CRL AF07.168','Mass spectrometer','CRL AA7.168','','Aeron',394,'',144 from dual union all
select 15744,'1967-S212','1967-04-13 00:49:00','suborbital rocket','success',848,'CRL AF07.385','Mass spectrometer','','','Aeron',394,'',148 from dual union all
select 15745,'1967-S213','1967-04-13 00:56:00','suborbital rocket','success',848,'CRL AG07.622','Niro TMA E5','','','Aeron',394,'',167 from dual union all
select 15746,'1967-S214','1967-04-13 00:57:00','suborbital rocket','success',848,'CRL AG07.623','Niro TMA E6','','','Aeron',339,'',169 from dual union all
select 15747,'1967-S215','1967-04-13 07:24:00','suborbital rocket','success',867,'CRL AF08.647','Sphere/TMA','','','Aeron',394,'',175 from dual union all
select 15748,'1967-S216','1967-04-13 07:22:00','suborbital rocket','success',839,'CRL AF6.561','Ion trap','','','Ionos',394,'',116 from dual union all
select 15749,'1967-S217','1967-04-17 00:00:00','military missile','success',796,'2143','ERCS 3','494L','BUSY MISSILE','Comms',1703,'',900 from dual union all
select 1575,'1953-A170','1953-04-16 00:00:00','atmospheric rocket','success',836,'1249A-203','Nike 1249 R&D 202R','','','Test',1895,'',10 from dual union all
select 15750,'1967-S218','1967-04-18 10:20:00','suborbital rocket','success',848,'CRL AF07.384','Mass spectrometer','','','Ionos',394,'',135 from dual union all
select 15751,'1967-S219','1967-04-18 11:21:00','suborbital rocket','success',848,'CRL AF07.386','Mass spectrometer','','','Aeron',394,'',138 from dual union all
select 15752,'1967-S22','1967-07-05 00:00:00','military missile','success',1037,'Ya10300-25L','8K98 LKI-2-9','','','Test',477,'',1000 from dual union all
select 15753,'1967-S220','1967-04-18 11:40:00','suborbital rocket','success',848,'CRL AF07.560','Sphere/Ion trap','','','Aeron',394,'',169 from dual union all
select 15754,'1967-S221','1967-04-18 20:08:00','suborbital rocket','success',839,'NASA 10.228GM','GSFC Grenade 147 PB','','','Aeronomy',1039,'',120 from dual union all
select 15755,'1967-S222','1967-04-19 16:19:10','suborbital rocket','success',290,'CE-24/1','ESRO C24/1','','','Test',408,'',128 from dual union all
select 15756,'1967-S223','1967-04-19 06:14:00','suborbital rocket','success',288,'C87','ISRO 45.01','TERLS-051','','Ionos',1467,'',128 from dual union all
select 15757,'1967-S224','1967-04-20 00:40:00','suborbital rocket','success',867,'CRL AF08.650','Ba','','','Aeron',394,'',260 from dual union all
select 15758,'1967-S225','1967-04-19 23:07:00','military missile','success',1095,'CC-20xx/SV-3','SPARTA SV-3','','','Reentry Test',1846,'',90 from dual;
insert into LAUNCH
select 15759,'1967-S226','1967-04-20 17:32:00','suborbital rocket','failure',839,'NASA 10.224GM','GSFC Grenade 148 W','','','Aeronomy',1821,'',107 from dual union all
select 1576,'1953-A171','1953-04-16 00:00:00','atmospheric rocket','success',836,'1249A-202','Nike 1249 R&D 203R','','','Test',1895,'',10 from dual union all
select 15760,'1967-S227','1967-04-20 22:36:00','suborbital rocket','success',1206,'SL425','UAT-2/UCW-13','','','XR Astron/Ionos',1839,'',218 from dual union all
select 15761,'1967-S228','1967-04-21 00:00:00','military missile','success',795,'1296','Mk 11 RV','FOT','GIPSY CAMP','OT',1698,'',900 from dual union all
select 15762,'1967-S229','1967-04-21 00:00:00','military missile','success',796,'2137','Mk 11A RV','ST OLYMPIC TRIALS 1','OLYMPIC TRIALS 1','OT',1708,'',900 from dual union all
select 15763,'1967-S23','1967-07-24 00:00:00','military missile','failure',1037,'Ya10300-24L','8K98 LKI-2-10','','','Test',477,'',1000 from dual union all
select 15764,'1967-S230','1967-04-24 15:20:00','suborbital rocket','success',30,'NASA 04.117GS','OSO-3 calibration','','','Solar X/EUV',1869,'',181 from dual union all
select 15765,'1967-S231','1967-04-26 04:15:00','suborbital rocket','success',227,'AFF-VB-06','Test','','','Test/Ionos/Plasma',428,'',396 from dual union all
select 15766,'1967-S232','1967-04-27 00:00:00','military missile','success',795,'1193','Mk 11 RV','FOT','OLEO KNIFE','OT',1702,'',900 from dual union all
select 15767,'1967-S233','1967-04-27 18:00:01','military missile','unknown',932,'A3P-510','','','','OT',421,'SSBN 600',1000 from dual union all
select 15768,'1967-S234','1967-04-27 04:10:00','suborbital rocket','success',227,'AFF-VB-07','Test','','','Test/Ionos/Plasma',428,'',398 from dual union all
select 15769,'1967-S235','1967-04-27 04:31:00','suborbital rocket','success',1207,'SL682','RAE Test 37','','','Test',1839,'',203 from dual union all
select 1577,'1953-A172','1953-04-16 00:00:00','atmospheric rocket','success',836,'1084E','','','','Test',1895,'',10 from dual union all
select 15770,'1967-S236','1967-04-28 09:30:00','military missile','success',795,'1019','Mk 11 RV','R&D','BUSY MUMMY','Test',1697,'',900 from dual union all
select 15771,'1967-S237','1967-04-28 01:12:00','suborbital rocket','success',867,'CRL AF08.651','Ba','','','Aeron',394,'',254 from dual union all
select 15772,'1967-S238','1967-04-29 23:39:00','suborbital rocket','success',839,'NASA 10.225GM','GSFC Grenade 149 W','','','Aeronomy',1821,'',123 from dual union all
select 15773,'1967-S239','1967-05-01 00:00:00','military missile','success',7,'6TYa','','','','Test',182,'',100 from dual union all
select 15774,'1967-S24','1967-08-21 00:00:00','military missile','success',1037,'Ya10300-26L','8K98 LKI-2-11','','','Test',477,'',1000 from dual union all
select 15775,'1967-S240','1967-05-24 13:21:00','suborbital rocket','success',846,'STV SR-056','SRE-19','','','Aeron?',1866,'',100 from dual union all
select 15776,'1967-S241','1967-04-30 01:40:00','suborbital rocket','success',839,'NASA 10.229GM','GSFC Grenade 150 PB','','','Aeronomy',1039,'',120 from dual union all
select 15777,'1967-S242','1967-05-03 14:29:00','suborbital rocket','success',288,'Rehbar 15','','','','Aeron',1419,'',150 from dual union all
select 15778,'1967-S243','1967-05-04 20:07:00','suborbital rocket','success',839,'NASA 10.232GM','GSFC Grenade 151 W','','','Aeronomy',1821,'',115 from dual union all
select 15779,'1967-S244','1967-05-05 00:00:00','military missile','success',135,'B032','Ionizing Pellets','ABRES N-7 (USN)','','Reentry Test',612,'',200 from dual union all
select 1578,'1953-A173','1953-04-16 00:00:00','atmospheric rocket','success',836,'333P','Nike CET 33','','','Test',1895,'',10 from dual union all
select 15780,'1967-S245','1967-05-05 00:00:00','military missile','success',135,'B033','IMPALA 2','ABRES YY-5 (USN)','','Reentry Test',614,'',200 from dual union all
select 15781,'1967-S246','1967-05-05 04:00:00','suborbital rocket','success',30,'NASA 04.157GG','Venus UV','','','UV Astron',1869,'',159 from dual union all
select 15782,'1967-S247','1967-05-05 04:26:00','suborbital rocket','success',227,'AAD-VB-10','','','','Aurora/Aeron/Ionos',428,'',361 from dual union all
select 15783,'1967-S248','1967-05-05 07:30:00','suborbital rocket','success',30,'NASA 04.203UG','UV stellar spectra','','','UV Astron',1869,'',176 from dual union all
select 15784,'1967-S249','1967-05-05 14:49:00','suborbital rocket','success',446,'D-33','Shahpar-1','','','Test',1419,'',423 from dual union all
select 15785,'1967-S25','1967-08-25 00:00:00','military missile','failure',1037,'','8K98 LKI-2-12','','','Test',477,'',1000 from dual union all
select 15786,'1967-S250','1967-05-05 17:00:00','suborbital rocket','success',837,'NASA 14.268UI','Auroral','','','Auroral/Test',428,'',136 from dual union all
select 15787,'1967-S251','1967-05-07 14:30:00','suborbital rocket','success',288,'Rehbar 16','','','','Aeron',1419,'',150 from dual union all
select 15788,'1967-S252','1967-05-07 15:26:00','suborbital rocket','success',725,'','','','','Ionos',736,'VOKV',90 from dual union all
select 15789,'1967-S253','1967-05-09 07:20:00','suborbital rocket','success',839,'NASA 10.230GM','GSFC Grenade 152 PB','','','Aeronomy',1039,'',120 from dual union all
select 1579,'1953-A174','1953-04-16 00:00:00','atmospheric rocket','success',836,'332P','Nike CET 32','','','Test',1895,'',10 from dual union all
select 15790,'1967-S254','1967-05-09 21:00:00','suborbital rocket','success',30,'NASA 04.192DS','NRL ND3.199 XUV-H3','','','Solar Opt/EUV',1869,'',182 from dual union all
select 15791,'1967-S255','1967-05-11 14:23:00','military missile','success',796,'2095','Mk 12 RV','Force Mod','BUSY FELLOW','Test',1707,'',900 from dual union all
select 15792,'1967-S256','1967-05-11 00:00:00','suborbital rocket','success',1115,'Sandia 281-4','','AEC SKOL','','Test/Aeron?',855,'',151 from dual union all
select 15793,'1967-S257','1967-05-11 00:00:00','suborbital rocket','success',1115,'Sandia 281-5','Dart','AEC SKOL','','Test/Aeron?',855,'',116 from dual union all
select 15794,'1967-S258','1967-05-11 08:25:00','suborbital rocket','success',839,'NASA 10.233GM','GSFC Grenade 153 W','','','Aeronomy',1821,'',129 from dual union all
select 15795,'1967-S259','1967-05-15 11:40:00','suborbital rocket','success',839,'NASA 10.231GM','GSFC Grenade 154 PB','','','Aeronomy',1039,'',120 from dual union all
select 15796,'1967-S26','1967-01-14 00:00:00','military missile','success',1449,'','8K84 UBP-45 LKI','','','Test',503,'',1000 from dual union all
select 15797,'1967-S260','1967-05-16 08:32:00','suborbital rocket','success',30,'NRL NB3.177','','','','XR Astron',1869,'',148 from dual union all
select 15798,'1967-S261','1967-05-17 00:00:00','military missile','success',795,'774','Mk 11 RV','FOT','GLOSSY COAST','OT',1705,'',900 from dual union all
select 15799,'1967-S262','1967-05-17 20:58:00','suborbital rocket','success',624,'NASA 8.43UA','Ion density/Compos.','','','Aeronomy',1821,'',1006 from dual union all
select 158,'1944-S143','1944-09-25 00:00:00','military missile','success',1470,'','Norwich','','','Weapon',1741,'',95 from dual union all
select 1580,'1953-A175','1953-04-16 00:00:00','atmospheric rocket','success',836,'331P','Nike CET 31','','','Test',1895,'',10 from dual union all
select 15800,'1967-S263','1967-05-18 06:16:00','suborbital rocket','success',864,'Sandia 152-56','LRL BOX-10','','','XR Astron',855,'',172 from dual union all
select 15801,'1967-S264','1967-05-19 00:00:00','military missile','success',795,'667','Mk 11 RV','R&D','BUSY GIANT','Test',1697,'',900 from dual union all
select 15802,'1967-S265','1967-05-19 00:00:00','military missile','success',796,'2122','Mk 11A RV','ST OLYMPIC TRIALS 2','OLYMPIC TRIALS 2','OT',1710,'',900 from dual union all
select 15803,'1967-S266','1967-05-19 22:00:00','military missile','success',162,'119F','Simulated BGRV 2','ABRES BGRV-2','BUSY PIGSKIN','Reentry Test',1680,'',40 from dual union all
select 15804,'1967-S267','1967-05-21 07:00:00','suborbital rocket','success',839,'','Sandia 154-100','','','Aeron',855,'',100 from dual union all
select 15805,'1967-S268','1967-05-22 00:00:00','military missile','success',795,'1188','Mk 11 RV','FOT','GLOSS TWINE','OT',1698,'',900 from dual union all
select 15806,'1967-S269','1967-05-22 00:11:57','suborbital rocket','success',1206,'ISRC-SK-17','ESRO S11/2 ROE5/BXR2','','','UV Astron/XR Astron',1343,'',230 from dual union all
select 15807,'1967-S27','1967-03-02 00:00:00','military missile','success',1449,'','8K84 UBP-47 LKI','','','Test',503,'',1000 from dual union all
select 15808,'1967-S270','1967-05-24 04:29:00','military missile','success',982,'G 22604-03T','8K64U UBP-96','','','OT',565,'',1210 from dual union all
select 15809,'1967-S271','1967-05-25 09:00:00','suborbital rocket','unknown',740,'MP 12','','','','Aeron',636,'',150 from dual union all
select 1581,'1953-A176','1953-04-16 00:00:00','atmospheric rocket','success',836,'330P','Nike CET 30','','','Test',1895,'',10 from dual union all
select 15810,'1967-S272','1967-05-25 23:24:00','suborbital rocket','success',1206,'ISRC-SK-19','ESRO S05/2 ROE6','','','UV Astronomy',1343,'',239 from dual union all
select 15811,'1967-S274','1967-05-31 00:00:00','military missile','success',795,'726','Mk 11 RV','FOT','GLIB TALKER','OT',1702,'',900 from dual union all
select 15812,'1967-S275','1967-06-01 00:00:00','suborbital rocket','success',846,'STV','','','','Aeron?',1895,'',100 from dual union all
select 15813,'1967-S276','1967-06-01 00:00:00','suborbital rocket','success',920,'','Test','','','Test',1447,'',140 from dual union all
select 15814,'1967-S277','1967-06-01 00:00:00','suborbital rocket','success',920,'','Test','','','Test',1447,'',140 from dual union all
select 15815,'1967-S278','1967-05-31 03:12:00','military missile','success',999,'N 22500-10T','8K67 UBP-11','GCh 8F675 LKI','','Test',485,'',1000 from dual union all
select 15816,'1967-S279','1967-06-01 00:00:00','military missile','success',795,'757','Mk 11 RV','ST Cold/Heat Soak 1','BUGGY ROAD','OT',1701,'',900 from dual union all
select 15817,'1967-S28','1967-03-18 00:00:00','military missile','success',1449,'Ya10701-13T','8K84 UBP-48 SLI','','','OT',503,'',1000 from dual union all
select 15818,'1967-S280','1967-06-01 00:00:00','military missile','success',135,'B034','IMPALA 3','ABRES YY-6 (USN)','','Reentry Test',614,'',200 from dual union all
select 15819,'1967-S281','1967-06-01 05:54:00','suborbital rocket','success',837,'','Sandia 154-97','','','Aeron',855,'',165 from dual union all
select 1582,'1953-A177','1953-04-18 00:00:00','suborbital rocket','failure',1490,'N11','','','','Test',666,'',0 from dual union all
select 15820,'1967-S282','1967-06-01 06:45:00','suborbital rocket','success',839,'','Sandia 154-99','','','Aeron',855,'',100 from dual union all
select 15821,'1967-S283','1967-06-01 08:15:00','suborbital rocket','success',837,'','Sandia 154-92','','','Aeron',855,'',160 from dual union all
select 15822,'1967-S284','1967-06-01 10:00:00','suborbital rocket','success',837,'','Sandia 154-95','','','Aeron',855,'',160 from dual union all
select 15823,'1967-S285','1967-06-01 10:30:00','suborbital rocket','success',839,'','Sandia 154-101','','','Aeron',855,'',100 from dual union all
select 15824,'1967-S286','1967-06-01 12:25:00','suborbital rocket','success',837,'','Sandia 154-94','','','Aeron',855,'',160 from dual union all
select 15825,'1967-S287','1967-06-02 00:00:00','military missile','success',916,'','P-125 67 Summer 01','','','OT',204,'',180 from dual union all
select 15826,'1967-S288','1967-06-02 00:00:00','military missile','success',916,'','P-126 67 Summer 02','','','OT',204,'',180 from dual union all
select 15827,'1967-S289','1967-06-02 04:30:00','suborbital rocket','success',30,'NASA 04.210GG','UV stellar spectra','','','UV Astron',1869,'',179 from dual union all
select 15828,'1967-S29','1967-03-23 00:00:00','military missile','success',1449,'Ya10702-01T','8K84 UBP-49 SLI','','','OT',503,'',1000 from dual union all
select 15829,'1967-S290','1967-06-02 09:34:00','suborbital rocket','success',837,'','Sandia 154-93','','','Aeron/Solar',855,'',160 from dual union all
select 1583,'1953-A178','1953-04-20 00:00:00','atmospheric rocket','success',836,'336P','Nike CET 36','','','Test',1895,'',10 from dual union all
select 15830,'1967-S291','1967-06-03 05:47:00','suborbital rocket','success',837,'','Sandia 154-98','','','Aeron',855,'',170 from dual union all
select 15831,'1967-S292','1967-06-06 02:45:00','military missile','success',999,'N 22507-12T','8K67 UBP-12','GCh 8F675 LKI','','Test',514,'',1000 from dual union all
select 15832,'1967-S293','1967-06-06 14:20:00','suborbital rocket','success',30,'NASA 04.222NA','Luster 1967-I','','','Meteorites',1869,'',138 from dual union all
select 15833,'1967-S294','1967-06-06 22:19:00','suborbital rocket','success',867,'NASA 18.29UE','UNH neutron flux','','','Plasma',428,'',369 from dual union all
select 15834,'1967-S295','1967-06-07 13:20:00','suborbital rocket','success',848,'CRL AG07.272','Meteor dust','','','Meteorites',394,'',154 from dual union all
select 15835,'1967-S296','1967-06-08 00:00:00','military missile','success',135,'D053','ABRES SS-8 (USAF)','ABRES SS-8 (USAF)','','Reentry Test',613,'',200 from dual union all
select 15836,'1967-S297','1967-06-09 00:00:00','military missile','success',916,'','P-127 67 Summer 03','','','OT',204,'',180 from dual union all
select 15837,'1967-S298','1967-06-09 04:15:00','suborbital rocket','success',30,'KP3.21','Planetary UV','','','UV Astron',1869,'',150 from dual union all
select 15838,'1967-S299','1967-06-09 10:23:00','military missile','success',168,'122F (KX-101)','RMV-310/SPDS','ABRES RMP-B-1/NTMP KX101','BUCK TYPE','Reentry Test',1682,'',1400 from dual union all
select 15839,'1967-S30','1967-03-25 00:00:00','military missile','success',1449,'Ya22902-23P','8K84 UBP-50 SLI','','','OT',508,'',1000 from dual union all
select 1584,'1953-A179','1953-04-20 00:00:00','atmospheric rocket','success',836,'335P','Nike CET 35','','','Test',1895,'',10 from dual union all
select 15840,'1967-S300','1967-06-13 16:36:00','suborbital rocket','success',725,'','','','','Ionos',738,'VOKV',90 from dual union all
select 15841,'1967-S301','1967-06-13 23:08:00','suborbital rocket','success',837,'NASA 14.297CE','OGO-E AS&E detector','','','Plasma',1821,'',179 from dual union all
select 15842,'1967-S302','1967-06-15 02:55:00','military missile','success',999,'N 22500-11T','8K67 UBP-13','GCh 8F675 LKI','','Test',485,'',1000 from dual union all
select 15843,'1967-S303','1967-06-15 16:38:00','suborbital rocket','unknown',725,'','','','','Ionos',739,'VOKV',90 from dual union all
select 15844,'1967-S304','1967-06-16 10:37:00','suborbital rocket','success',624,'NASA 8.49IE','GRS Test','','','Tech/Plasma',330,'',1100 from dual union all
select 15845,'1967-S305','1967-06-17 08:30:00','suborbital rocket','success',624,'NASA 8.50IE','GRS Test','','','Tech/Plasma',330,'',991 from dual union all
select 15846,'1967-S306','1967-06-17 16:28:00','suborbital rocket','success',725,'','','','','Ionos',740,'VOKV',90 from dual union all
select 15847,'1967-S307','1967-06-17 21:15:00','suborbital rocket','success',725,'','','','','Ionos',741,'VOKV',90 from dual union all
select 15848,'1967-S308','1967-06-21 00:00:00','suborbital rocket','success',740,'M-175','','','','Aeron',404,'',90 from dual;
insert into LAUNCH
select 15849,'1967-S309','1967-06-21 00:00:00','military missile','success',795,'1357','Mk 11 RV','FOT','OHIO FARM','OT',1698,'',900 from dual union all
select 1585,'1953-A18','1953-01-15 00:00:00','atmospheric rocket','success',836,'1249A-176','Nike 1249 R&D 162R','','','Test',1895,'',10 from dual union all
select 15850,'1967-S31','1967-04-21 00:00:00','military missile','success',1449,'Ya22903-23P','8K84 UBP-51 SLI','','','OT',503,'',1000 from dual union all
select 15851,'1967-S310','1967-06-21 19:02:00','suborbital rocket','success',624,'NASA 8.26UI','ISIS overflight','','','Ionos',1821,'',792 from dual union all
select 15852,'1967-S311','1967-06-21 19:49:00','suborbital rocket','success',30,'NASA 04.179UA','UMn mass spectrom.','','','Aeronomy',1869,'',210 from dual union all
select 15853,'1967-S312','1967-06-22 00:00:00','suborbital rocket','success',740,'M-175','','','','Aeron',404,'',90 from dual union all
select 15854,'1967-S313','1967-06-22 00:00:00','military missile','success',916,'','P-128 67 Summer 04','','','OT',204,'',180 from dual union all
select 15855,'1967-S314','1967-06-23 00:00:00','military missile','success',916,'','P-129 67 Summer 05','','','OT',204,'',180 from dual union all
select 15856,'1967-S315','1967-06-23 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 623',1000 from dual union all
select 15857,'1967-S316','1967-06-23 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 623',1000 from dual union all
select 15858,'1967-S317','1967-06-23 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 623',1000 from dual union all
select 15859,'1967-S318','1967-06-23 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 623',1000 from dual union all
select 1586,'1953-A180','1953-04-20 00:00:00','atmospheric rocket','success',836,'337P','Nike CET 37','','','Test',1895,'',10 from dual union all
select 15860,'1967-S319','1967-06-23 00:00:00','military missile','success',1420,'B-70','Mk 6 RV','FOT/NTMP KX-62','BUGGY WHEEL','OT',1657,'',1300 from dual union all
select 15861,'1967-S32','1967-04-26 00:00:00','military missile','success',1449,'Ya10703-01T','8K84 UBP-52 SLI','','','OT',503,'',1000 from dual union all
select 15862,'1967-S320','1967-06-24 08:26:00','suborbital rocket','success',839,'NASA 10.238GM','GSFC Grenade 155 N','','','Aeronomy Fall Equinox',330,'',100 from dual union all
select 15863,'1967-S321','1967-06-24 20:35:00','suborbital rocket','success',839,'NASA 10.237GM','GSFC Grenade 156 N','','','Aeronomy',330,'',100 from dual union all
select 15864,'1967-S322','1967-06-25 08:34:00','suborbital rocket','success',839,'NASA 10.239GM','GSFC Grenade 157 N','','','Aeronomy',330,'',100 from dual union all
select 15865,'1967-S323','1967-06-27 02:35:00','military missile','success',999,'Ya22503-14T','8K67 UBP-14','','','OT',514,'',1000 from dual union all
select 15866,'1967-S324','1967-06-28 00:00:00','suborbital rocket','success',364,'DA002','FU-179','','','Test',313,'',138 from dual union all
select 15867,'1967-S325','1967-06-18 20:32:00','suborbital rocket','success',1440,'AD21.860','AFCRL TB2-1','','','Reentry Test',1821,'',200 from dual union all
select 15868,'1967-S326','1967-06-29 00:00:00','military missile','success',795,'751','Mk 11B RV','R&D','BUSY COUSIN','Test',1697,'',900 from dual union all
select 15869,'1967-S327','1967-07-05 21:33:00','military missile','success',837,'BMTS','SPEEDBALL II 102SN48','','','Target',1895,'',120 from dual union all
select 1587,'1953-A181','1953-04-20 00:00:00','atmospheric rocket','success',836,'334P','Nike CET 34','','','Test',1895,'',10 from dual union all
select 15870,'1967-S328','1967-07-26 16:10:00','military missile','failure',837,'BMTS','SPEEDBALL II 103SN51','','','Target',1895,'',120 from dual union all
select 15871,'1967-S329','1967-07-07 14:53:00','suborbital rocket','success',837,'STV','STV SR-057','','','Aeron?',1895,'',100 from dual union all
select 15872,'1967-S33','1967-06-14 00:00:00','military missile','success',1449,'Ya22904-23P','8K84 UBP-53 SLI','','','OT',503,'',1000 from dual union all
select 15873,'1967-S330','1967-07-01 00:00:00','suborbital rocket','success',920,'','Test','','','Test',1447,'',140 from dual union all
select 15874,'1967-S332','1967-07-04 23:07:00','military missile','success',1095,'CC-2032/SV-4','SPARTA SV-4','','','Reentry Test',1846,'',90 from dual union all
select 15875,'1967-S333','1967-07-04 23:35:00','suborbital rocket','success',217,'ALF-3-19','Mass spectrometer','','','Aeron',428,'',131 from dual union all
select 15876,'1967-S334','1967-07-05 00:00:00','military missile','success',795,'1126','Mk 11 RV','FOT GB40','GLOWING BRIGHT 40','OT',1702,'',900 from dual union all
select 15877,'1967-S335','1967-07-05 02:00:00','suborbital rocket','success',217,'ALF-3-20','Mass spectrometer','','','Aeron',428,'',151 from dual union all
select 15878,'1967-S336','1967-07-05 14:07:00','sounding rocket','success',1068,'R10','Rubis 04','','','RF Astron',307,'',1560 from dual union all
select 15879,'1967-S337','1967-07-05 18:41:00','suborbital rocket','success',30,'CRL AE3.582','Auroral profile','','','Auroral',428,'',200 from dual union all
select 1588,'1953-A182','1953-04-21 00:00:00','military missile','unknown',339,'1259RS','','','','Test',1895,'',30 from dual union all
select 15880,'1967-S338','1967-07-06 00:00:00','military missile','success',154,'65D','Mk 12 RV','ABRES PDV','','Reentry Test',1684,'',1800 from dual union all
select 15881,'1967-S339','1967-07-08 00:00:00','military missile','unknown',929,'A2P','FOT','','','OT',421,'SSBN 620',1000 from dual union all
select 15882,'1967-S34','1967-06-21 00:00:00','military missile','success',1449,'Ya10704-01T','8K84 UBP-54 SLI','','','OT',503,'',1000 from dual union all
select 15883,'1967-S340','1967-07-08 00:00:00','military missile','unknown',929,'A2P','FOT','','','OT',421,'SSBN 620',1000 from dual union all
select 15884,'1967-S341','1967-07-08 04:10:00','suborbital rocket','success',30,'NASA 04.190UG','MIT-4 GC Survey I','','','XR Astron',1869,'',142 from dual union all
select 15885,'1967-S342','1967-07-11 04:25:00','military missile','success',982,'T 22502-06','8K64U UBP-97','','','OT',566,'',1210 from dual union all
select 15886,'1967-S343','1967-07-12 00:00:00','military missile','success',795,'800','Mk 11 RV','FOT GB42','GLOWING BRIGHT 42','OT',1705,'',900 from dual union all
select 15887,'1967-S344','1967-07-12 00:23:00','suborbital rocket','success',217,'ADF-3-24','O2 profile','','','Airglow IR',428,'',95 from dual union all
select 15888,'1967-S345','1967-07-12 11:00:00','suborbital rocket','success',217,'ADF-3-25','O2 profile','','','Airglow IR',428,'',95 from dual union all
select 15889,'1967-S346','1967-07-13 00:00:00','military missile','success',795,'779','Mk 11B RV','ST Cold/Heat Soak 2?','OLD COIN','OT',1701,'',900 from dual union all
select 1589,'1953-A183','1953-04-21 00:00:00','suborbital rocket','success',1490,'N10','','','','Test',666,'',45 from dual union all
select 15890,'1967-S347','1967-07-14 00:00:00','military missile','success',135,'D050','ABRES WW-3 (USAF)','ABRES WW-3 (USAF)','','Reentry Test',613,'',200 from dual union all
select 15891,'1967-S348','1967-07-15 00:00:00','military missile','success',796,'2337 (2095)','Mk 12 RV','R&D','BUSY JOKER','Test',1707,'',900 from dual union all
select 15892,'1967-S349','1967-07-15 11:18:00','military missile','success',795,'755','Mk 11B RV','R&D GIN BABY I','GIN BABY I','Test',1697,'',900 from dual union all
select 15893,'1967-S35','1967-08-03 00:00:00','military missile','success',1449,'Ya22905-23P','8K84 UBP-55','','','OT',503,'',1000 from dual union all
select 15894,'1967-S350','1967-07-20 09:00:00','military missile','success',796,'2370','Mk 11B RV','ST OLYMPIC TRIALS 3','OLYMPIC TRIALS 3','OT',1708,'',900 from dual union all
select 15895,'1967-S351','1967-07-20 14:00:00','military missile','success',999,'Ya22502-15','8K67 UBP-15','SBP Special Test','','Test',485,'',1000 from dual union all
select 15896,'1967-S352','1967-07-20 09:00:00','suborbital rocket','success',30,'NASA 04.212UA','Composition','','','Aeronomy',1869,'',210 from dual union all
select 15897,'1967-S353','1967-07-20 19:24:00','suborbital rocket','success',30,'NASA 04.211UA','Composition','','','Aeronomy',1869,'',209 from dual union all
select 15898,'1967-S354','1967-07-22 00:00:00','military missile','success',168,'114F','RMV-320/RMV-410A','ABRES RMP-B-2/NTMP KX102','','Reentry Test',1682,'',1400 from dual union all
select 15899,'1967-S355','1967-07-23 00:53:00','suborbital rocket','success',837,'NASA 14.337CM','GCA Release 66 TMA','','','Aeronomy',1821,'',204 from dual union all
select 159,'1944-S144','1944-09-25 09:56:00','military missile','success',8,'18815','Ma223','','','Training',661,'',90 from dual union all
select 1590,'1953-A184','1953-04-22 00:00:00','military missile','failure',703,'614','','','','Test',222,'',0 from dual union all
select 15900,'1967-S356','1967-07-24 16:43:00','military missile','success',1095,'CC-2009/SV-5','SPARTA SV-5','','','Reentry Test',1846,'',90 from dual union all
select 15901,'1967-S357','1967-07-25 00:00:00','military missile','success',1017,'E 15002- 05','UBP','','','OT',475,'',1350 from dual union all
select 15902,'1967-S358','1967-07-27 00:00:00','suborbital rocket','success',1431,'Sandia 152-96','9" Ballistic','','','Ionos',1490,'',99 from dual union all
select 15903,'1967-S359','1967-07-29 08:48:00','military missile','success',162,'150F','MBRV No. 5778602','ABRES MBRV-3','BREAD HOOK','Reentry Test',1681,'',1400 from dual union all
select 15904,'1967-S36','1967-08-31 00:00:00','military missile','success',1449,'Ya10704-13T','8K84 UBP-56','','','OT',503,'',1000 from dual union all
select 15905,'1967-S360','1967-07-30 00:30:00','suborbital rocket','success',839,'OB6.03C','','','','Plasma',394,'',100 from dual union all
select 15906,'1967-S361','1967-08-25 07:35:00','suborbital rocket','success',846,'STV SR-058','G-01','','','Aeron?',1895,'',100 from dual union all
select 15907,'1967-S362','1967-07-31 15:18:04','military missile','success',932,'A3P-529','','','','OT',421,'SSBN 659',1000 from dual union all
select 15908,'1967-S363','1967-07-31 00:31:18','suborbital rocket','success',290,'CE-30','ESRO C20/1','','','Aeronomy',408,'',133 from dual union all
select 15909,'1967-S364','1967-07-31 00:58:23','suborbital rocket','success',290,'CE-31','ESRO C32/1','','','Meteorite',408,'',127 from dual union all
select 1591,'1953-A185','1953-04-24 00:00:00','atmospheric rocket','unknown',576,'305','','','','Test',1895,'',10 from dual union all
select 15910,'1967-S365','1967-08-03 07:36:00','suborbital rocket','success',227,'ALF-VB-12','MPE Ba release','','','Magnetospheric',428,'',448 from dual union all
select 15911,'1967-S366','1967-08-03 11:10:01','suborbital rocket','success',837,'NASA 14.97UM','UM Pitot 31','','','Aeronomy',1039,'',155 from dual union all
select 15912,'1967-S367','1967-08-04 03:53:00','test rocket','failure',467,'F-6/1','','','','-',1845,'',180 from dual union all
select 15913,'1967-S368','1967-08-04 04:53:00','suborbital rocket','success',30,'NASA 04.172UG','Stellar UV','','','UV/XR Astron',1869,'',130 from dual union all
select 15914,'1967-S369','1967-08-05 01:59:00','suborbital rocket','success',837,'NASA 14.343GT-UA','Recovery test','','','Test',1895,'',150 from dual union all
select 15915,'1967-S37','1967-09-02 00:00:00','military missile','success',1449,'Ya22906-23P','8K84 UBP-57','','','OT',503,'',1000 from dual union all
select 15916,'1967-S370','1967-08-05 09:56:00','suborbital rocket','success',837,'NASA 14.290UM','UM Pitot 32','','','Aeronomy',1039,'',150 from dual union all
select 15917,'1967-S372','1967-08-06 04:04:00','suborbital rocket','success',227,'ALF-VB-13','MPE Ba release','','','Magnetospheric',428,'',287 from dual union all
select 15918,'1967-S373','1967-08-08 00:00:00','military missile','success',795,'1212','Mk 11 RV','FOT GB45','GLOWING BRIGHT 45','OT',1705,'',900 from dual union all
select 15919,'1967-S374','1967-08-08 03:45:00','military missile','success',982,'T 22505-17','8K64U UBP-99','','','OT',566,'',1210 from dual union all
select 1592,'1953-A186','1953-04-24 00:00:00','atmospheric rocket','unknown',576,'304','','','','Test',1895,'',10 from dual union all
select 15920,'1967-S375','1967-08-08 03:56:00','suborbital rocket','success',1207,'SL305','LUX-39/MSSL SX2-SUV10','','','Solar X,UV',1839,'',182 from dual union all
select 15921,'1967-S376','1967-08-08 12:57:00','suborbital rocket','success',30,'NASA 04.207UA','POGO detector test','','','Ionos',1869,'',289 from dual union all
select 15922,'1967-S377','1967-08-08 21:45:00','suborbital rocket','success',30,'CRL AH3.529','EUV Mon','','','Solar EUV',1869,'',235 from dual union all
select 15923,'1967-S378','1967-08-08 23:56:00','suborbital rocket','success',837,'NASA 14.273UI','SpE ionosphere','','','Ionos',1821,'',141 from dual union all
select 15924,'1967-S379','1967-08-09 02:43:00','suborbital rocket','success',837,'NASA 14.338CM','GCA Release 67 TMA','','','Aeronomy',1821,'',214 from dual union all
select 15925,'1967-S38','1967-09-20 00:00:00','military missile','success',1449,'Ya10744-376','8K84 UBP-58','','','OT',508,'',1000 from dual union all
select 15926,'1967-S380','1967-08-09 07:58:00','suborbital rocket','success',837,'NASA 14.340CM','GCA Release 68 TMA','','','Aeronomy',1821,'',211 from dual union all
select 15927,'1967-S381','1967-08-09 08:47:00','suborbital rocket','success',837,'NASA 14.339CM','GCA Release 69 TMA','','','Aeronomy',1821,'',204 from dual union all
select 15928,'1967-S382','1967-08-09 08:47:00','suborbital rocket','success',837,'NASA 14.342GM','GCA Release 70 TMA','','','Aeronomy',1821,'',158 from dual union all
select 15929,'1967-S383','1967-08-09 09:28:00','suborbital rocket','success',837,'NASA 14.341CM','GCA Release 71 Na','','','Aeronomy',1821,'',199 from dual union all
select 1593,'1953-A187','1953-04-24 00:00:00','military missile','success',771,'GM-11049','','','','Test',222,'',12 from dual union all
select 15930,'1967-S384','1967-08-11 09:00:00','suborbital rocket','success',30,'NASA 04.223NA','Luster 1967-II','','','Meteorites',1869,'',141 from dual union all
select 15931,'1967-S385','1967-08-11 20:25:00','suborbital rocket','success',725,'','','','','Ionos',742,'VOKV',90 from dual union all
select 15932,'1967-S386','1967-08-12 06:30:00','suborbital rocket','success',30,'NRL NC3.187','Ion Composition','','','Aeron/Ionos',1869,'',222 from dual union all
select 15933,'1967-S387','1967-08-15 18:45:00','suborbital rocket','success',30,'CRL AG3.528','EUV Mon','','','Solar EUV',1869,'',243 from dual union all
select 15934,'1967-S388','1967-08-16 06:36:00','suborbital rocket','success',1199,'SL581B','RAE Test 38','','','Test',1839,'',152 from dual union all
select 15935,'1967-S389','1967-08-17 21:18:59','military missile','success',796,'466','Mk 12 RV','R&D','','Test',266,'',900 from dual union all
select 15936,'1967-S39','1967-09-20 00:00:00','military missile','success',1449,'Ya10744-378','8K84 UBP-59','','','OT',504,'',1000 from dual union all
select 15937,'1967-S390','1967-08-17 19:46:00','military missile','success',1095,'CC-2039/SV-6','SPARTA SV-6','','','Reentry Test',1846,'',90 from dual union all
select 15938,'1967-S391','1967-08-18 00:00:00','military missile','success',135,'E003','FATE I-04','ABRES I-4 (USAF)','','Reentry Test',614,'',200 from dual;
insert into LAUNCH
select 15939,'1967-S392','1967-08-18 00:00:00','military missile','success',135,'D055','ABRES WW-4 (USAF)','ABRES WW-4 (USAF)','','Reentry Test',613,'',200 from dual union all
select 1594,'1953-A188','1953-04-27 00:00:00','military missile','unknown',339,'1262 RS','','','','Test',1895,'',30 from dual union all
select 15940,'1967-S393','1967-08-24 10:05:00','suborbital rocket','success',1199,'SL521','OXUV-1 Ly alpha','','','UV Astronomy',1839,'',178 from dual union all
select 15941,'1967-S394','1967-08-25 00:00:00','military missile','success',135,'D048','ABRES QQ-5 (USAF)','ABRES QQ-5 (USAF)','','Reentry Test',612,'',200 from dual union all
select 15942,'1967-S395','1967-08-25 23:43:00','suborbital rocket','success',1181,'NASA 15.21II','Arcas III','','','Ionos',29,'',84 from dual union all
select 15943,'1967-S396','1967-08-26 04:36:00','suborbital rocket','success',30,'NASA 04.187CG','LMSC X-ray','','','XR Astron',1869,'',125 from dual union all
select 15944,'1967-S397','1967-08-26 06:35:00','suborbital rocket','success',848,'CRL AF07.328','Density','','','Aeron',394,'',177 from dual union all
select 15945,'1967-S398','1967-08-26 11:39:00','suborbital rocket','success',839,'NASA 10.241GM','GSFC Grenade 159 N','','','Aeronomy',330,'',92 from dual union all
select 15946,'1967-S399','1967-08-26 23:39:00','suborbital rocket','success',839,'NASA 10.242GM','GSFC Grenade 160 N','','','Aeronomy',330,'',95 from dual union all
select 15947,'1967-S40','1967-09-21 00:00:00','military missile','success',1449,'Ya22901-24P','8K84 UBP-60','','','OT',508,'',1000 from dual union all
select 15948,'1967-S400','1967-08-28 00:30:00','suborbital rocket','success',839,'NASA 10.243GM','GSFC Grenade 161 N','','','Aeronomy',330,'',92 from dual union all
select 15949,'1967-S401','1967-08-29 20:10:00','military missile','success',916,'','P-130 67 Fall 01','','','OT',204,'',180 from dual union all
select 1595,'1953-A189','1953-04-27 00:00:00','atmospheric rocket','unknown',576,'184','','','','Test',1895,'',10 from dual union all
select 15950,'1967-S402','1967-08-29 20:15:00','military missile','success',916,'','P-131 67 Fall 02','','','OT',204,'',180 from dual union all
select 15951,'1967-S403','1967-08-29 20:45:00','military missile','success',916,'','P-132 67 Fall 03','','','OT',204,'',180 from dual union all
select 15952,'1967-S404','1967-08-29 20:45:00','military missile','success',916,'','P-133 67 Fall 04','','','OT',204,'',180 from dual union all
select 15953,'1967-S405','1967-08-29 10:01:00','suborbital rocket','success',1199,'SL522','OXUV-2 Ly alpha','','','UV Astronomy',1839,'',124 from dual union all
select 15954,'1967-S406','1967-08-30 00:00:00','sounding rocket','success',129,'NASA 16.03GR','','','','RF Astron',1811,'',1500 from dual union all
select 15955,'1967-S407','1967-08-30 17:32:00','suborbital rocket','success',837,'','ION 1/67','','','Ionos',310,'',225 from dual union all
select 15956,'1967-S408','1967-09-01 00:00:00','military missile','success',7,'','LKI-1','','','Test',182,'',300 from dual union all
select 15957,'1967-S409','1967-09-15 06:41:00','suborbital rocket','success',846,'STV SR-060','SRG-02','','','Aeron?',1895,'',100 from dual union all
select 15958,'1967-S41','1967-10-02 00:00:00','military missile','success',1449,'Ya10744-379','8K84 UBP-61','','','OT',505,'',1000 from dual union all
select 15959,'1967-S410','1967-09-12 07:00:00','military missile','success',916,'AO-53','P-134 67 Fall 05','','','Test',204,'',180 from dual union all
select 1596,'1953-A19','1953-01-16 00:00:00','atmospheric rocket','unknown',576,'83','','','','Test',1895,'',10 from dual union all
select 15960,'1967-S411','1967-03-15 00:00:00','military missile','success',916,'AO-49','P-121 67 Spring 05','','','Test',204,'',180 from dual union all
select 15961,'1967-S413','1967-08-31 22:06:00','suborbital rocket','success',867,'NASA 18.43GE','Ba clouds','','','Plasma',29,'',321 from dual union all
select 15962,'1967-S414','1967-08-31 22:47:52','suborbital rocket','success',867,'NASA 18.46GE','Double probe','','','Plasma',29,'',228 from dual union all
select 15963,'1967-S415','1967-09-02 17:37:00','suborbital rocket','success',837,'','ION 2/67','','','Ionos',310,'',232 from dual union all
select 15964,'1967-S416','1967-09-02 20:31:00','suborbital rocket','success',867,'NASA 18.44GE','4 Ba clouds','','','Plasma',29,'',321 from dual union all
select 15965,'1967-S417','1967-09-02 20:42:35','suborbital rocket','success',867,'NASA 18.47GE','E fields','','','Plasma',29,'',233 from dual union all
select 15966,'1967-S418','1967-09-04 02:05:00','military missile','success',999,'Ya22504-14T','8K67 UBP-16','','','OT',514,'',1000 from dual union all
select 15967,'1967-S419','1967-09-06 20:35:00','military missile','success',795,'1222','Mk 11 RV','FOT GB46','GLOWING BRIGHT 46','OT',1698,'',900 from dual union all
select 15968,'1967-S42','1967-10-06 00:00:00','military missile','success',1449,'Ya10744-372','8K84 UBP-62','','','OT',504,'',1000 from dual union all
select 15969,'1967-S420','1967-09-06 22:35:00','suborbital rocket','success',894,'AER 1/67','','','','Aeron',310,'',90 from dual union all
select 1597,'1953-A190','1953-04-27 00:00:00','atmospheric rocket','unknown',576,'185','','','','Test',1895,'',10 from dual union all
select 15970,'1967-S421','1967-09-05 06:00:00','suborbital rocket','success',848,'CRL AG07.176','Density','','','Aeron',394,'',101 from dual union all
select 15971,'1967-S422','1967-09-07 06:05:00','suborbital rocket','success',894,'AER 1/67-2','','','','Aeron',310,'',90 from dual union all
select 15972,'1967-S423','1967-09-07 09:50:00','suborbital rocket','success',894,'AER 1/67-3','','','','Aeron',310,'',90 from dual union all
select 15973,'1967-S424','1967-09-07 20:34:00','suborbital rocket','success',837,'NASA 14.308UI','UIUC Arecibo 1','','','Ionos',1483,'',200 from dual union all
select 15974,'1967-S425','1967-09-07 20:34:57','suborbital rocket','success',864,'Sandia 152-88','LRL BOX-8a','','','XR Astron',1490,'',187 from dual union all
select 15975,'1967-S426','1967-09-08 15:45:02','military missile','unknown',932,'A3P-377','','','','OT',421,'SSBN 602',1000 from dual union all
select 15976,'1967-S427','1967-09-08 02:10:00','military missile','success',999,'Ya22502-08T','8K67 UBP-17','','','OT',485,'',1000 from dual union all
select 15977,'1967-S428','1967-09-08 07:36:00','military missile','success',135,'D057','ABRES AI-1 (USAF)','ABRES AI-1 (USAF)','','Reentry Test',614,'',200 from dual union all
select 15978,'1967-S429','1967-09-08 00:07:00','suborbital rocket','success',837,'NASA 14.305UI','UIUC Arecibo 2','','','Ionos',1483,'',200 from dual union all
select 15979,'1967-S43','1967-10-23 00:00:00','military missile','success',1449,'Ya226   01U','8K84 UBP-63','','','OT',508,'',1000 from dual union all
select 1598,'1953-A191','1953-04-28 00:00:00','military missile','success',703,'623','','','','Test',222,'',8 from dual union all
select 15980,'1967-S430','1967-09-08 03:21:00','suborbital rocket','success',30,'NRL NB3.196','Stellar XR','','','XR Astron',1869,'',200 from dual union all
select 15981,'1967-S431','1967-09-11 00:00:00','military missile','success',1420,'B-21','Mk 6 RV','FOT GB44','GLOWING BRIGHT 44','OT',1657,'',1300 from dual union all
select 15982,'1967-S432','1967-09-13 00:26:00','suborbital rocket','success',837,'Speedball SN44','SPEEDBALL II 109','','','Aeron?',1895,'',150 from dual union all
select 15983,'1967-S433','1967-09-12 07:48:00','military missile','success',916,'AO-54 Hit Mexico','P-135 67 Fall 06','','','OT',204,'',180 from dual union all
select 15984,'1967-S434','1967-09-12 00:00:00','military missile','failure',932,'A3TY-1 (A3P-613)','Antelope 7','','ANTELOPE 7','Test',251,'',500 from dual union all
select 15985,'1967-S435','1967-09-12 20:57:00','suborbital rocket','success',867,'NASA 18.45GE','Ba clouds, E field','','','Plasma',29,'',321 from dual union all
select 15986,'1967-S436','1967-09-12 20:59:49','suborbital rocket','failure',867,'NASA 18.48GE','E fields','','','Plasma',29,'',227 from dual union all
select 15987,'1967-S437','1967-09-14 00:00:00','military missile','success',135,'D056','ABRES AD-1 (USAF)','ABRES AD-1 (USAF)','','Reentry Test',612,'',200 from dual union all
select 15988,'1967-S438','1967-09-15 16:17:00','military missile','success',1095,'CC-2012/SV-7','SPARTA SV-7','','','Reentry Test',1846,'',91 from dual union all
select 15989,'1967-S439','1967-09-18 19:10:00','suborbital rocket','success',867,'NASA 18.50GA','UM Thermosphere 7','','','Aeronomy',1821,'',286 from dual union all
select 1599,'1953-A192','1953-04-29 00:00:00','military missile','success',771,'GM-2369','','','','Test',222,'',12 from dual union all
select 15990,'1967-S44','1967-10-29 00:00:00','military missile','success',1449,'Ya10706-01T','8K84 UBP-64','','','OT',508,'',1000 from dual union all
select 15991,'1967-S440','1967-09-19 18:53:00','suborbital rocket','success',837,'Round 110','Speedball II SN45','','','Aeron?',1895,'',150 from dual union all
select 15992,'1967-S441','1967-09-20 17:00:00','suborbital rocket','success',837,'Round 111','Speedball II SN46','','','Aeron?',1895,'',150 from dual union all
select 15993,'1967-S442','1967-09-22 02:28:00','military missile','success',795,'831','Mk 11 RV','FOT GB47','GLOWING BRIGHT 47','OT',1702,'',900 from dual union all
select 15994,'1967-S443','1967-09-21 00:00:00','suborbital rocket','success',865,'Sandia 152-101','LASL Solar','','','Solar',855,'',335 from dual union all
select 15995,'1967-S444','1967-09-21 16:00:00','suborbital rocket','success',846,'STV SR-059','','','','Aeron?',1864,'',150 from dual union all
select 15996,'1967-S445','1967-09-21 07:20:00','military missile','success',999,'Ya22504K','8K67 UBP-18','','','OT',511,'',1000 from dual union all
select 15997,'1967-S446','1967-09-21 00:00:00','suborbital rocket','success',1347,'Sandia 281-7','','','','Aeron?',803,'',383 from dual union all
select 15998,'1967-S447','1967-09-21 07:38:00','suborbital rocket','success',624,'NASA 8.45UI','','','','Ionos',1821,'',768 from dual union all
select 15999,'1967-S448','1967-09-22 20:30:00','suborbital rocket','success',837,'Round 112','Speedball II SN37','','','Aeron?',1895,'',150 from dual union all
select 16,'1943-A09','1943-06-29 00:00:00','military missile','failure',8,'38','','','','Test',675,'',1 from dual union all
select 160,'1944-S145','1944-09-25 11:41:00','military missile','success',8,'18808','Ma225','','','Training',661,'',90 from dual union all
select 1600,'1953-A193','1953-04-30 00:00:00','atmospheric rocket','success',836,'340P','Nike CET 40','','','Test',1895,'',10 from dual union all
select 16000,'1967-S449','1967-09-25 17:58:00','suborbital rocket','success',837,'Round 113','Speedball II SN48','','','Aeron?',1895,'',150 from dual union all
select 16001,'1967-S45','1967-10-29 00:00:00','military missile','success',1449,'Ya226   02U','8K84 UBP-65','','','OT',504,'',1000 from dual union all
select 16002,'1967-S450','1967-09-25 22:00:00','suborbital rocket','success',837,'Round 114','Speedball II SN49','','','Aeron?',1895,'',150 from dual union all
select 16003,'1967-S451','1967-09-27 02:00:00','military missile','success',795,'1151','Mk 11 RV','FOT GB48','GLOWING BRIGHT 48','OT',1705,'',900 from dual union all
select 16004,'1967-S452','1967-09-26 16:30:00','suborbital rocket','success',837,'Round 115','Speedball II SN50','','','Aeron?',1895,'',150 from dual union all
select 16005,'1967-S453','1967-09-26 20:30:00','suborbital rocket','success',837,'Round 116','Speedball II SN51','','','Aeron?',1895,'',150 from dual union all
select 16006,'1967-S454','1967-09-27 00:00:00','suborbital rocket','success',1115,'Sandia 281-6','Dart','','','Test/Aeron?',803,'',104 from dual union all
select 16007,'1967-S455','1967-09-27 12:32:00','suborbital rocket','success',1199,'ESRO S26/1','ESRO S26/1 RSRS-SX1','','','Ionos/Solar X',1343,'',183 from dual union all
select 16008,'1967-S456','1967-09-28 16:35:00','suborbital rocket','success',226,'AFF-VA-119','','','','Test/Ionos',428,'',165 from dual union all
select 16009,'1967-S457','1967-09-29 00:15:00','suborbital rocket','success',864,'Sandia 152-87','LRL BOX-12','','','XR Astron',770,'',168 from dual union all
select 1601,'1953-A194','1953-04-30 00:00:00','atmospheric rocket','success',836,'339P','Nike CET 39','','','Test',1895,'',10 from dual union all
select 16010,'1967-S458','1967-09-29 03:45:00','military missile','success',135,'C051','HAPDEC','ABRES AH-2 (ARPA)','','Reentry Test',613,'',200 from dual union all
select 16011,'1967-S459','1967-09-29 05:45:00','military missile','success',135,'D066','SFT-9','ABRES AC-2 (USN)','','Reentry Test',614,'',200 from dual union all
select 16012,'1967-S46','1967-11-14 00:00:00','military missile','success',1449,'Ya22601-01T','8K84 UBP-66','','','OT',508,'',1000 from dual union all
select 16013,'1967-S460','1967-09-29 07:30:00','military missile','success',135,'C050  [090]','HAPDEC','ABRES AH-1 (ARPA)','','Reentry Test',612,'',200 from dual union all
select 16014,'1967-S461','1967-09-30 16:14:00','suborbital rocket','success',30,'NASA 04.104DS','OSO 3 cal','','','Solar EUV',1821,'',216 from dual union all
select 16015,'1967-S462','1967-10-01 04:10:00','military missile','success',999,'Ya22505-14T','8K67 UBP-19','','','OT',514,'',1000 from dual union all
select 16016,'1967-S463','1967-10-02 12:54:00','military missile','success',982,'T 22502-12T','8K64U UBP-101','','','OT',566,'',1210 from dual union all
select 16017,'1967-S464','1967-10-02 07:22:00','suborbital rocket','success',864,'Sandia 152-106','LRL BOX-11','','','XR Astron',803,'',168 from dual union all
select 16018,'1967-S465','1967-10-02 20:00:00','suborbital rocket','success',839,'CRL AD6.842','Antenna Breakdown','','','Tech',1821,'',108 from dual union all
select 16019,'1967-S466','1967-10-03 00:00:00','suborbital rocket','success',865,'Sandia 152-103','LASL Solar','','','Solar',855,'',309 from dual union all
select 1602,'1953-A195','1953-04-30 00:00:00','atmospheric rocket','success',836,'338P','Nike CET 38','','','Test',1895,'',10 from dual union all
select 16020,'1967-S467','1967-10-03 06:49:00','military missile','success',982,'G 22603-16T','8K64U UBP-102','','','OT',571,'',1210 from dual union all
select 16021,'1967-S468','1967-10-03 00:00:00','suborbital rocket','success',1347,'Sandia 281-8','','','','Aeron?',803,'',423 from dual union all
select 16022,'1967-S469','1967-10-03 18:18:00','suborbital rocket','success',30,'NASA 04.152GS','Solar pointing','','','Solar X/UV',1869,'',184 from dual union all
select 16023,'1967-S47','1967-11-25 00:00:00','military missile','success',1449,'Ya10702-13T','8K84 UBP-67','','','OT',504,'',1000 from dual union all
select 16024,'1967-S470','1967-10-04 00:00:00','suborbital rocket','success',865,'Sandia 152-82','Sampler','','','Aeron?',855,'',301 from dual union all
select 16025,'1967-S471','1967-10-04 00:20:00','suborbital rocket','success',867,'CRL AH08.645','Ba clouds','','','Aeron',1821,'',251 from dual union all
select 16026,'1967-S472','1967-10-04 13:12:30','suborbital rocket','success',446,'D-35','ESRO D34/1','','','Solar X/Aeronomy',29,'',248 from dual union all
select 16027,'1967-S473','1967-10-04 13:23:18','suborbital rocket','success',1199,'ESRO S19/1','ESRO S19/1 MSSL I-26','','','Ionos/Aeron',1343,'',206 from dual;
insert into LAUNCH
select 16028,'1967-S474','1967-10-05 00:18:00','suborbital rocket','success',867,'CRL AH08.646','Ba','','','Aeron',1821,'',249 from dual union all
select 16029,'1967-S475','1967-10-05 04:11:00','suborbital rocket','success',226,'AFF-VA-120','','','','Test/Ionos',428,'',182 from dual union all
select 1603,'1953-A196','1953-04-30 00:00:00','atmospheric rocket','success',836,'1249A-205','Nike 1249 R&D 205R','','','Test',1895,'',10 from dual union all
select 16030,'1967-S476','1967-10-05 14:30:00','suborbital rocket','success',30,'NASA 04.243DS','NRL ND3.192 Cor''gr.','','','Solar EUV',1869,'',173 from dual union all
select 16031,'1967-S477','1967-10-07 10:32:00','suborbital rocket','success',446,'D-30','ESRO D30/1','','','Solar X/Aeronomy',29,'',233 from dual union all
select 16032,'1967-S478','1967-10-07 14:50:00','suborbital rocket','success',1199,'ESRO S26/2','ESRO S26/2 RSRS-SX2','','','Ionos/Solar X',1343,'',164 from dual union all
select 16033,'1967-S479','1967-10-09 04:49:00','military missile','success',982,'T 22503-01','8K64U UBP-103','','','OT',566,'',1210 from dual union all
select 16034,'1967-S48','1967-11-28 00:00:00','military missile','success',1449,'Ya10744-377','8K84 UBP-68','','','OT',508,'',1000 from dual union all
select 16035,'1967-S480','1967-10-09 04:36:00','suborbital rocket','success',1181,'NASA 15.22II','Arcas IV','','','Ionos',29,'',86 from dual union all
select 16036,'1967-S481','1967-10-09 15:30:00','suborbital rocket','success',740,'D-75M','','','','Aeron',1010,'',120 from dual union all
select 16037,'1967-S482','1967-10-10 14:33:00','suborbital rocket','success',1199,'ESRO S19/2','ESRO S19/2 MSSL I-27','','','Ionos/Aeron',1343,'',204 from dual union all
select 16038,'1967-S483','1967-10-10 14:55:00','suborbital rocket','success',740,'D-75M','','','','Aeron',1010,'',180 from dual union all
select 16039,'1967-S484','1967-10-12 04:49:00','suborbital rocket','success',846,'STV SR-061','','','','Aeron?',1865,'',150 from dual union all
select 1604,'1953-A197','1953-04-30 00:00:00','atmospheric rocket','success',836,'1249A-204','Nike 1249 R&D 204R','','','Test',1895,'',10 from dual union all
select 16040,'1967-S485','1967-10-11 00:00:00','military missile','success',135,'B030','IMPALA 4','ABRES YY-3 (USN)','','Reentry Test',613,'',200 from dual union all
select 16041,'1967-S486','1967-10-11 11:30:03','military missile','success',154,'69D','TVX-13','ABRES TVX-13','','Reentry Test',1685,'',1800 from dual union all
select 16042,'1967-S487','1967-10-11 16:22:00','military missile','success',1095,'CC-2030/SV-8','SPARTA SV-8','','','Reentry Test',1846,'',137 from dual union all
select 16043,'1967-S488','1967-10-12 06:14:00','suborbital rocket','success',846,'STV SR-062','','','','Aeron?',1865,'',150 from dual union all
select 16044,'1967-S489','1967-10-12 14:15:00','ballistic missile test','success',679,'U149    -13 (13L)','VKZ No. 11','','','Solar UV',564,'',4400 from dual union all
select 16045,'1967-S49','1967-12-14 00:00:00','military missile','success',1449,'Ya10706-13T','8K84 UBP-69','','','OT',505,'',1000 from dual union all
select 16046,'1967-S490','1967-10-13 07:03:00','military missile','success',982,'T 22503-08','8K64U UBP-104','','','OT',566,'',1210 from dual union all
select 16047,'1967-S491','1967-10-13 04:47:00','suborbital rocket','success',1181,'NASA 15.23II','Arcas V','','','Ionos/Plasma',29,'',87 from dual union all
select 16048,'1967-S492','1967-10-14 00:00:00','military missile','success',162,'118F','MBRV No. 604','ABRES MBRV-4','','Reentry Test',1681,'',1400 from dual union all
select 16049,'1967-S493','1967-10-14 11:20:00','suborbital rocket','success',839,'NASA 10.248GM','GSFC Grenade 162 N','','','Aeronomy',330,'',100 from dual union all
select 1605,'1953-A198','1953-05-01 00:00:00','atmospheric rocket','unknown',576,'186','','','','Test',1895,'',10 from dual union all
select 16050,'1967-S494','1967-10-15 11:15:00','suborbital rocket','success',839,'NASA 10.244GM','GSFC Grenade 163 N','','','Aeronomy',330,'',100 from dual union all
select 16051,'1967-S495','1967-10-15 23:15:00','suborbital rocket','success',839,'NASA 10.245GM','GSFC Grenade 164 N','','','Aeronomy',330,'',100 from dual union all
select 16052,'1967-S496','1967-10-17 16:40:23','suborbital spaceplane','success',1520,'3-64-95','UVPL/Solar/MM','','','Tech/Meteor/Solar',379,'NB-52 008',85 from dual union all
select 16053,'1967-S497','1967-10-19 12:05:00','suborbital rocket','success',30,'NASA 04.239US','Solar/geo Ly-alpha','','','Solar UV',1869,'',182 from dual union all
select 16054,'1967-S498','1967-10-20 10:49:00','suborbital rocket','success',217,'AKF-3-26','','','','Ionos/XR Astron',1315,'',120 from dual union all
select 16055,'1967-S499','1967-10-21 00:00:00','military missile','success',796,'2445','Mk 11B RV','ST OLYMPIC TRIALS 4','OLYMPIC TRIALS 4','OT',1710,'',900 from dual union all
select 16056,'1967-S50','1967-12-15 00:00:00','military missile','success',1449,'Ya10705-13T','8K84 UBP-70','','','OT',508,'',1000 from dual union all
select 16057,'1967-S500','1967-10-22 09:42:00','suborbital rocket','success',217,'AKF-3-27','','','','Ionos/XR Astron',1315,'',156 from dual union all
select 16058,'1967-S501','1967-10-23 16:04:40','suborbital rocket','success',837,'K-NA-7','DLR K-NA-7  EXP-32','','','Aeron/Ionos/Fields',413,'',235 from dual union all
select 16059,'1967-S502','1967-10-24 17:14:00','suborbital rocket','success',848,'CRL AG07.637','Arecibo calibration','','','Ionos',1483,'',180 from dual union all
select 1606,'1953-A199','1953-05-04 00:00:00','atmospheric rocket','unknown',576,'205','','','','Test',1895,'',10 from dual union all
select 16060,'1967-S503','1967-10-24 23:00:00','suborbital rocket','success',848,'CRL AG07.638','Arecibo cal','','','Ionos',1483,'',180 from dual union all
select 16061,'1967-S504','1967-10-26 12:22:00','military missile','success',982,'T 22502-17T','8K64U UBP-105','','','OT',566,'',1210 from dual union all
select 16062,'1967-S505','1967-10-27 15:45:00','suborbital rocket','success',1536,'3','','','','Test',1343,'',145 from dual union all
select 16063,'1967-S506','1967-10-27 10:45:00','suborbital rocket','success',30,'NASA 04.158GG','UV stellar spectra','','','UV Astron',1869,'',160 from dual union all
select 16064,'1967-S507','1967-10-27 17:14:00','suborbital rocket','success',848,'CRL AG07.640','Arecibo cal','','','Ionos',1483,'',180 from dual union all
select 16065,'1967-S508','1967-10-28 09:15:00','suborbital rocket','success',848,'CRL AG07.641','Arecibo cal','','','Ionos',1483,'',179 from dual union all
select 16066,'1967-S51','1967-12-21 00:00:00','military missile','success',1449,'Ya22906-24P','8K84 UBP-71','','','OT',505,'',1000 from dual union all
select 16067,'1967-S510','1967-11-15 15:00:00','military missile','success',916,'AO-55','P-138 67 Fall 09','','','Test',204,'',180 from dual union all
select 16068,'1967-S511','1967-11-15 15:30:00','military missile','success',916,'AO-56','P-139 67 Fall 10','','','Test',204,'',180 from dual union all
select 16069,'1967-S512','1967-11-15 17:45:00','military missile','success',916,'AO-57','P-140 67 Fall 11','','','Test',204,'',180 from dual union all
select 1607,'1953-A20','1953-01-16 00:00:00','atmospheric rocket','unknown',576,'84','','','','Test',1895,'',10 from dual union all
select 16070,'1967-S513','1967-10-31 13:14:00','military missile','success',1095,'CC-2041/SV-9','SPARTA SV-9','','','Reentry Test',1846,'',111 from dual union all
select 16071,'1967-S514','1967-11-01 17:18:07','military missile','unknown',932,'A3TY-3 (A3P-083)','Antelope 8','','ANTELOPE 8','Test',251,'',1000 from dual union all
select 16072,'1967-S515','1967-11-01 00:00:00','military missile','unknown',932,'A3P','FOT','','','OT',421,'SSBN 656',1000 from dual union all
select 16073,'1967-S516','1967-11-01 00:00:00','military missile','unknown',932,'A3P','FOT','','','OT',421,'SSBN 656',1000 from dual union all
select 16074,'1967-S517','1967-11-01 00:00:00','military missile','unknown',932,'A3P','FOT','','','OT',421,'SSBN 656',1000 from dual union all
select 16075,'1967-S518','1967-11-01 04:30:00','suborbital rocket','success',1207,'SL407','LUX-40AC8/Cul-8','','','Solar X,UV',1839,'',209 from dual union all
select 16076,'1967-S519','1967-11-01 11:30:00','suborbital rocket','success',30,'NASA 04.226UG','UV stellar spectra','','','UV Astron',1869,'',176 from dual union all
select 16077,'1967-S52','1967-01-06 16:33:48','military missile','success',932,'A3TX-2 (A3P-496)','Antelope 2','','ANTELOPE 2','Test',251,'',1000 from dual union all
select 16078,'1967-S520','1967-11-01 19:57:00','suborbital rocket','success',740,'','Ion composition','','','Aeron/Ionos',1010,'',180 from dual union all
select 16079,'1967-S521','1967-11-03 00:00:00','military missile','success',795,'820','Mk 11B RV','R&D GIN BABY II','GIN BABY II','Test',1697,'',900 from dual union all
select 1608,'1953-A200','1953-05-04 00:00:00','military missile','success',771,'GM-11053','','','','Test',222,'',12 from dual union all
select 16080,'1967-S522','1967-11-06 00:00:00','military missile','unknown',929,'A2P','FOT','','','OT',421,'SSBN 624',1000 from dual union all
select 16081,'1967-S523','1967-11-06 00:00:00','military missile','unknown',929,'A2P','FOT','','','OT',421,'SSBN 624',1000 from dual union all
select 16082,'1967-S524','1967-11-06 00:00:00','military missile','unknown',929,'A2P','FOT','','','OT',421,'SSBN 624',1000 from dual union all
select 16083,'1967-S525','1967-11-07 00:56:08','military missile','success',796,'467','Mk 12 RV','R&D','','Test',266,'',900 from dual union all
select 16084,'1967-S526','1967-09-07 00:00:00','suborbital rocket','success',288,'C92','','','','Aeron TMA',310,'',150 from dual union all
select 16085,'1967-S527','1967-11-07 13:17:54','military missile','success',154,'94D','Mk 11A-5B AX-2','ABRES AX-2/PDV-3','','Reentry Test',1684,'',1800 from dual union all
select 16086,'1967-S528','1967-11-07 17:45:00','suborbital rocket','success',30,'CRL AH3.530','EUV Mon/RPA','','','Aeron/Solar EUV',1869,'',243 from dual union all
select 16087,'1967-S529','1967-11-08 22:41:00','suborbital rocket','success',850,'MD-67-9.5-3','','','','Aeron?',1895,'',150 from dual union all
select 16088,'1967-S53','1967-01-11 09:35:00','military missile','success',796,'2073  [2113]','Mk 12 RV','R&D/Force Mod/NTMP KX-82','BONUS BOY','Test',1703,'',900 from dual union all
select 16089,'1967-S530','1967-11-10 12:20:00','military missile','success',162,'113F','BGRV S/N 1','ABRES BGRV-3','','Reentry Test',1680,'',40 from dual union all
select 1609,'1953-A201','1953-05-05 00:00:00','atmospheric rocket','unknown',576,'206','','','','Test',1895,'',10 from dual union all
select 16090,'1967-S531','1967-11-14 14:20:00','military missile','success',795,'1215','Mk 11 RV','FOT GB51','GLOWING BRIGHT 51','OT',1702,'',900 from dual union all
select 16091,'1967-S532','1967-11-14 15:22:00','military missile','success',795,'1028','Mk 11 RV','FOT GB52','GLOWING BRIGHT 52','OT',1705,'',900 from dual union all
select 16092,'1967-S533','1967-11-14 05:27:00','suborbital rocket','success',1199,'SL423','RSRS-6','','','Ionospheric',1839,'',178 from dual union all
select 16093,'1967-S534','1967-11-14 10:05:00','suborbital rocket','success',30,'NASA 04.229UG','UV stellar spectra','','','UV Astron',1869,'',172 from dual union all
select 16094,'1967-S535','1967-11-15 18:30:07','suborbital spaceplane','failure',1520,'3-65-97','UVPL/Solar/MM/SA TPS','','','Tech/Meteor/Solar',379,'NB-52 008',81 from dual union all
select 16095,'1967-S536','1967-11-15 21:30:00','suborbital rocket','success',30,'CRL AD3.365','Dayglow','','','Aeron',330,'',224 from dual union all
select 16096,'1967-S537','1967-11-16 00:00:00','military missile','success',135,'D063  [092]','SFT-10','ABRES AC-3 (USN)','','Reentry Test',612,'',200 from dual union all
select 16097,'1967-S538','1967-11-16 18:14:00','suborbital rocket','success',837,'NASA 14.298UI','Electron probes','','','Ionos',1821,'',167 from dual union all
select 16098,'1967-S539','1967-11-17 00:00:00','suborbital rocket','success',1199,'SL424','RSRS-7','','','Ionospheric',1839,'',178 from dual union all
select 16099,'1967-S54','1967-01-11 00:00:00','suborbital rocket','success',1491,'VA84','FU-161 Astronomy','','','XR Astron/UV Astron',668,'',158 from dual union all
select 161,'1944-S146','1944-09-25 16:31:00','military missile','success',8,'18833','Ma226','','','Training',661,'',90 from dual union all
select 1610,'1953-A202','1953-05-05 00:00:00','atmospheric rocket','success',836,'1092E','','','','Test',1895,'',10 from dual union all
select 16100,'1967-S540','1967-11-17 00:00:00','military missile','success',796,'2386','Mk 11B RV','R&D GIN BABY III','GIN BABY III','Test',1700,'',900 from dual union all
select 16101,'1967-S541','1967-11-19 10:08:00','suborbital rocket','success',848,'CRL AG07.314','Recovery test','','','Test',330,'',119 from dual union all
select 16102,'1967-S542','1967-11-20 10:42:00','suborbital rocket','success',30,'NASA 04.228CG','AS&E X-ray','','','XR Astron',1869,'',144 from dual union all
select 16103,'1967-S543','1967-11-21 14:12:00','military missile','success',796,'2071','Mk 11 RV','R&D/MOD','BUSY LOBBY','Test',1703,'',900 from dual union all
select 16104,'1967-S544','1967-11-22 10:07:00','suborbital rocket','success',848,'CRL AG07.319','Recovery test','','','Test',330,'',119 from dual union all
select 16105,'1967-S545','1967-11-23 05:39:00','suborbital rocket','success',867,'NASA 18.34UE','Auroral particles','','','Plasma',428,'',164 from dual union all
select 16106,'1967-S546','1967-11-24 01:37:10','suborbital rocket','failure',290,'CE-36','ESRO C13/P','','','Ionospheric',408,'',160 from dual union all
select 16107,'1967-S547','1967-11-25 00:00:00','military missile','unknown',929,'A2P','FOT','','','OT',421,'SSBN 619',1000 from dual union all
select 16108,'1967-S548','1967-11-25 00:00:00','military missile','unknown',929,'A2P','FOT','','','OT',421,'SSBN 619',1000 from dual union all
select 16109,'1967-S549','1967-11-25 00:00:00','military missile','unknown',929,'A2P','FOT','','','OT',421,'SSBN 619',1000 from dual union all
select 1611,'1953-A203','1953-05-05 00:00:00','atmospheric rocket','success',836,'1249B-1178','Nike 1249 R&D 206R','','','Test',1895,'',10 from dual union all
select 16110,'1967-S55','1967-01-13 10:00:00','suborbital rocket','success',1493,'AGI 63','FU-160 Solar corona','','','Solar UV',668,'',123 from dual union all
select 16111,'1967-S550','1967-11-25 00:00:00','military missile','unknown',929,'A2P','FOT','','','OT',421,'SSBN 619',1000 from dual union all
select 16112,'1967-S551','1967-11-26 19:44:22','suborbital rocket','success',288,'CE-17','ESRO C31/1','','','Ionospheric',408,'',125 from dual union all
select 16113,'1967-S552','1967-11-29 01:06:00','suborbital rocket','success',837,'NASA 14.250IM','Rehbar 14 (Gren.)','','','Aeronomy',1419,'',175 from dual union all
select 16114,'1967-S553','1967-11-29 22:51:00','suborbital rocket','success',848,'CRL AF07.658','Smoke','','','Aeron',394,'',121 from dual union all
select 16115,'1967-S554','1967-11-29 22:51:00','suborbital rocket','success',848,'CRL AF07.661','Smoke','','','Aeron',339,'',122 from dual union all
select 16116,'1967-S555','1967-11-30 00:00:00','military missile','success',135,'B023  [093]','FATE','ABRES I-8 (USAF)','','Reentry Test',614,'',200 from dual union all
select 16117,'1967-S556','1967-12-01 00:00:00','military missile','success',795,'1183','Mk 11 RV','FOT GB50','GLOWING BRIGHT 50','OT',1698,'',900 from dual;
insert into LAUNCH
select 16118,'1967-S557','1967-12-01 06:08:00','suborbital rocket','success',1199,'SL422A','UAT-3','','','XR Astron',1839,'',181 from dual union all
select 16119,'1967-S558','1967-12-03 22:53:00','suborbital rocket','success',848,'CRL AF07.657','Smoke','','','Aeron',394,'',119 from dual union all
select 1612,'1953-A204','1953-05-06 00:00:00','atmospheric rocket','unknown',576,'199','','','','Test',1895,'',10 from dual union all
select 16120,'1967-S559','1967-12-03 22:53:00','suborbital rocket','success',848,'CRL AF07.662','Smoke','','','Aeron',339,'',124 from dual union all
select 16121,'1967-S56','1967-01-16 11:48:00','suborbital rocket','success',848,'CRL AF07.654','IRIS (Niro TMA E2)','','','Aeron',394,'',217 from dual union all
select 16122,'1967-S560','1967-12-04 11:45:00','suborbital rocket','success',30,'KP3.20','UV Tel (Venus)','','','UV Astron',1869,'',167 from dual union all
select 16123,'1967-S561','1967-12-04 21:01:00','test rocket','failure',467,'F-6/2','','','','-',1845,'',200 from dual union all
select 16124,'1967-S562','1967-12-04 22:20:00','suborbital rocket','success',848,'CRL AF07.880','D/E ions','','','Aeron',428,'',148 from dual union all
select 16125,'1967-S563','1967-12-04 22:44:00','suborbital rocket','success',848,'CRL AF07.387','D/E ions','','','Aeron',428,'',122 from dual union all
select 16126,'1967-S564','1967-12-05 03:24:00','suborbital rocket','success',867,'NASA 18.66UA','Auroral profile','','','Aeronomy',428,'',270 from dual union all
select 16127,'1967-S565','1967-12-05 11:30:00','suborbital rocket','success',30,'NASA 04.197UA','JHU UVT F2 (Venus)','','','UV Astron',1869,'',160 from dual union all
select 16128,'1967-S566','1967-12-05 19:11:00','suborbital rocket','success',35,'NASA 04.213NB','Rat centrifuge 1','','','Biology',1821,'',137 from dual union all
select 16129,'1967-S567','1967-12-05 20:34:50','suborbital rocket','success',837,'NASA 14.288IE','DLR K-NA-6','','','Aeron/Plasma',413,'',206 from dual union all
select 1613,'1953-A205','1953-05-07 00:00:00','atmospheric rocket','success',836,'342P','Nike CET 42','','','Test',1895,'',10 from dual union all
select 16130,'1967-S568','1967-12-06 10:45:00','suborbital rocket','success',30,'NASA 04.219GG','UV stellar spectra','','','UV Astron',1869,'',210 from dual union all
select 16131,'1967-S569','1967-12-06 15:41:00','suborbital rocket','success',1431,'Sandia 152-110','LRL BOX-13','','','XR Astron',1488,'',120 from dual union all
select 16132,'1967-S57','1967-01-16 23:34:00','suborbital rocket','success',848,'CRL AF07.503','Release Ba','','','Aeron',394,'',193 from dual union all
select 16133,'1967-S570','1967-12-06 16:14:51','suborbital rocket','success',226,'CRL AF17.750D','Particles','','','Ionos',428,'',137 from dual union all
select 16134,'1967-S571','1967-12-06 18:25:00','suborbital rocket','success',848,'CRL AF07.388','D/E ions','','','Aeron',428,'',132 from dual union all
select 16135,'1967-S572','1967-12-06 21:21:12','suborbital rocket','success',288,'CE-2','ESRO C31/2','','','Ionospheric',408,'',129 from dual union all
select 16136,'1967-S573','1967-12-08 01:00:00','suborbital rocket','success',624,'NASA 8.35UI','Ion-proton transit''n','','','Ionos',428,'',805 from dual union all
select 16137,'1967-S574','1967-12-08 21:03:00','military missile','success',795,'948','Mk 11B RV','R&D GIN BABY IV','GIN BABY IV','Test',1697,'',900 from dual union all
select 16138,'1967-S575','1967-12-09 04:14:00','military missile','success',982,'T 22501-13T','8K64U UBP-106','','','OT',566,'',1210 from dual union all
select 16139,'1967-S576','1967-12-10 22:00:00','suborbital rocket','success',30,'NASA 04.201NT','SPARCS Test','','','Test/Opt Astron',1869,'',160 from dual union all
select 1614,'1953-A206','1953-05-07 00:00:00','atmospheric rocket','success',836,'343P','Nike CET 43','','','Test',1895,'',10 from dual union all
select 16140,'1967-S577','1967-12-11 21:56:00','suborbital rocket','success',846,'STV SR-069','','','','Aeron?',1865,'',150 from dual union all
select 16141,'1967-S578','1967-12-12 21:48:00','suborbital rocket','success',837,'NASA 14.347UA','NO profile','','','Aeronomy',1821,'',94 from dual union all
select 16142,'1967-S579','1967-12-12 21:59:00','suborbital rocket','success',839,'NASA 10.249GM','GSFC Grenade 165 W','','','Aeronomy',1821,'',110 from dual union all
select 16143,'1967-S58','1967-01-17 00:04:00','suborbital rocket','success',848,'CRL AF07.653','JILL (Niro TMA E3)','','','Aeron',394,'',155 from dual union all
select 16144,'1967-S580','1967-12-13 08:43:00','suborbital rocket','success',848,'CRL AG07.627','Niro TMA  E9','','','Aeron',394,'',187 from dual union all
select 16145,'1967-S581','1967-12-13 09:23:00','suborbital rocket','success',848,'CRL AG07.630','Niro TMA  E10/Grenad','','','Aeron',394,'',164 from dual union all
select 16146,'1967-S582','1967-12-13 12:59:00','suborbital rocket','success',30,'CRL AF3.268','Dust collector','','','Meteorites',1869,'',164 from dual union all
select 16147,'1967-S583','1967-12-14 09:37:00','suborbital rocket','success',867,'CRL AH08.665','Release/Grenade','','','Aeron',394,'',106 from dual union all
select 16148,'1967-S584','1967-12-14 09:58:00','suborbital rocket','success',848,'CRL AG07.631','','','','Aeron',394,'',161 from dual union all
select 16149,'1967-S585','1967-12-16 22:58:00','suborbital rocket','success',848,'CRL AG07.663','Smoke','','','Aeron',339,'',127 from dual union all
select 1615,'1953-A207','1953-05-07 00:00:00','atmospheric rocket','success',836,'344P','Nike CET 44','','','Test',1895,'',10 from dual union all
select 16150,'1967-S586','1967-12-16 22:58:00','suborbital rocket','success',848,'CRL AG07.659','Smoke','','','Aeron',394,'',127 from dual union all
select 16151,'1967-S587','1967-12-18 00:00:00','military missile','success',795,'1114','Mk 11 RV','FOT GT01B (GLORY TRIP)','GLORY TRIP 01B','OT',1702,'',900 from dual union all
select 16152,'1967-S588','1967-12-18 13:00:00','suborbital rocket','success',839,'NASA 10.246GM','GSFC Grenade 166 N','','','Aeronomy',330,'',95 from dual union all
select 16153,'1967-S589','1967-12-19 01:00:00','suborbital rocket','success',839,'NASA 10.247GM','GSFC Grenade 167 N','','','Aeronomy',330,'',96 from dual union all
select 16154,'1967-S59','1967-01-17 00:00:00','suborbital rocket','success',1115,'Sandia 281-3','','','','Test/Aeron?',1490,'',188 from dual union all
select 16155,'1967-S590','1967-12-19 13:10:00','suborbital rocket','success',839,'NASA 10.250GM','GSFC Grenade 168 N','','','Aeronomy',330,'',92 from dual union all
select 16156,'1967-S591','1967-12-21 06:38:00','military missile','success',168,'117F','RMV-320/Decoys','ABRES RMP-B-4/NTMP KX106','','Reentry Test',1682,'',1400 from dual union all
select 16157,'1967-S592','1967-12-22 00:35:00','military missile','success',795,'1437','Mk 11 RV','FOT GT02B','GLORY TRIP 02B','OT',1698,'',900 from dual union all
select 16158,'1967-S593','1967-12-22 01:45:00','military missile','success',795,'1288','Mk 11 RV','FOT GT03B','GLORY TRIP 03B','OT',1705,'',900 from dual union all
select 16159,'1967-S594','1967-12-22 04:09:00','military missile','success',982,'T 22502-07T','8K64U UBP-108','','','OT',565,'',1210 from dual union all
select 1616,'1953-A208','1953-05-07 00:00:00','atmospheric rocket','success',836,'1103E','','','','Test',1895,'',10 from dual union all
select 16160,'1967-S595','1967-12-23 00:00:00','military missile','success',796,'2457','Mk 11B RV','ST OLYMPIC TRIALS 5','OLYMPIC TRIALS 5','OT',1710,'',900 from dual union all
select 16161,'1967-S596','1967-12-28 10:55:00','military missile','success',796,'2435','Mk 11B RV','R&D GIN BABY V','GIN BABY V','Test',1700,'',900 from dual union all
select 16162,'1967-S597','1967-11-01 00:00:00','suborbital rocket','success',706,'02B/4','SECT Meteo','','','Meteo',313,'',101 from dual union all
select 16163,'1967-S598','1967-05-14 17:20:00','suborbital rocket','success',725,'','','','','Ionos',737,'VOKV',90 from dual union all
select 16164,'1967-S599','1967-01-26 18:45:00','suborbital rocket','success',837,'ERDA 67-5 Sphere','NICAP RO-67-5','','','Aeronomy',1895,'',104 from dual union all
select 16165,'1967-S60','1967-01-17 00:00:00','suborbital rocket','success',1491,'VA85','FU-145B Astron.','','','Test/UV Astron',668,'',205 from dual union all
select 16166,'1967-S600','1967-12-19 09:00:00','suborbital rocket','success',740,'','VV/RFMS/release','','','Aeron/Ionos',636,'',170 from dual union all
select 16167,'1967-S601','1967-05-16 00:00:00','military missile','success',1018,'G 15002-02','8K75 UBP-P1','','','Test?',468,'',1160 from dual union all
select 16168,'1967-S602','1967-10-01 00:00:00','suborbital rocket','success',1440,'','AFAL 3','','','Reentry Test',1821,'',300 from dual union all
select 16169,'1967-S603','1967-01-01 00:00:00','suborbital rocket','success',1431,'Sandia 276-6','Dart/Parachute','','','Parachute test',1490,'',90 from dual union all
select 1617,'1953-A209','1953-05-07 00:00:00','atmospheric rocket','success',836,'341P','Nike CET 41','','','Test',1895,'',10 from dual union all
select 16170,'1967-S604','1967-01-01 00:00:00','suborbital rocket','success',1431,'Sandia 276-7','Dart/Parachute','','','Parachute test',1490,'',90 from dual union all
select 16171,'1967-S605','1967-01-01 00:00:00','suborbital rocket','success',1431,'Sandia 276-8','Dart/Parachute','','','Parachute test',1490,'',90 from dual union all
select 16172,'1967-S606','1967-01-01 00:00:00','suborbital rocket','success',1431,'Sandia 276-9','Dart/Parachute','','','Parachute test',1490,'',90 from dual union all
select 16173,'1967-S607','1967-02-16 01:17:00','suborbital rocket','success',767,'','B-59 BELFAST','','','Meteo',185,'',123 from dual union all
select 16174,'1967-S608','1967-06-13 03:14:04','suborbital rocket','success',767,'','Y-26 TRAIL 38','','','Meteo',1914,'',136 from dual union all
select 16175,'1967-S609','1967-06-22 02:47:00','suborbital rocket','success',767,'','B-67 CAIRO','','','Meteo',185,'',107 from dual union all
select 16176,'1967-S61','1967-01-17 19:00:00','suborbital rocket','success',30,'CRL AG3.526','EUV Mon','','','Solar EUV',1869,'',243 from dual union all
select 16177,'1967-S610','1967-06-22 23:51:00','suborbital rocket','success',767,'','B-68 DURBAN','','','Meteo',185,'',110 from dual union all
select 16178,'1967-S611','1967-02-16 03:56:00','suborbital rocket','success',767,'','B-61 DUBLIN','','','Meteo',185,'',117 from dual union all
select 16179,'1967-S612','1967-02-17 02:45:00','suborbital rocket','success',767,'','B-63 HOLLYWOOD','','','Meteo',185,'',114 from dual union all
select 1618,'1953-A21','1953-01-16 00:00:00','atmospheric rocket','success',836,'1249B','Nike Eng 6E','','','Test',1895,'',10 from dual union all
select 16180,'1967-S613','1967-02-17 08:17:00','suborbital rocket','success',767,'','B-65 LIMERICK','','','Meteo',185,'',110 from dual union all
select 16181,'1967-S614','1967-08-05 00:00:00','suborbital rocket','success',1506,'SDC VD-01','','','','Test',339,'',140 from dual union all
select 16182,'1967-S615','1967-08-05 00:00:00','suborbital rocket','success',1506,'SDC VD-02','','','','Test',339,'',140 from dual union all
select 16183,'1967-S616','1967-08-05 00:00:00','suborbital rocket','success',1506,'SDC VD-03','','','','Test',339,'',140 from dual union all
select 16184,'1967-S617','1967-08-05 00:00:00','suborbital rocket','success',1506,'SDC VD-04','','','','Test',339,'',140 from dual union all
select 16185,'1967-S618','1967-12-03 00:00:00','suborbital rocket','success',1506,'SDC VD-05','','','','Test',339,'',104 from dual union all
select 16186,'1967-S619','1967-12-03 00:00:00','suborbital rocket','success',1506,'SDC VD-06','','','','Test',339,'',104 from dual union all
select 16187,'1967-S62','1967-01-17 00:59:59','military missile','success',796,'464','Mk 12 RV','R&D','','Test',266,'',900 from dual union all
select 16188,'1967-S620','1967-12-03 00:00:00','suborbital rocket','success',1506,'SDC VD-07','','','','Test',339,'',109 from dual union all
select 16189,'1967-S621','1967-12-03 00:00:00','suborbital rocket','success',1506,'SDC VD-08','','','','Test',339,'',100 from dual union all
select 1619,'1953-A210','1953-05-08 00:00:00','atmospheric rocket','success',1444,'','','','','Test',1821,'',10 from dual union all
select 16190,'1967-S622','1967-12-03 00:00:00','suborbital rocket','success',1506,'SDC VD-09','','','','Test',339,'',100 from dual union all
select 16191,'1967-S623','1967-12-03 00:00:00','suborbital rocket','success',1506,'SDC VD-10','','','','Test',339,'',100 from dual union all
select 16192,'1967-S624','1967-05-19 00:00:00','suborbital rocket','success',894,'BIO-1','Rat payload','','','Bio',310,'',90 from dual union all
select 16193,'1967-S625','1967-01-24 18:00:00','suborbital rocket','success',846,'','STV SR-46','','','Aeron?',1864,'',100 from dual union all
select 16194,'1967-S626','1967-01-24 20:29:00','suborbital rocket','success',846,'','STV SR-47','','','Aeron?',1864,'',100 from dual union all
select 16195,'1967-S627','1967-01-05 16:00:00','suborbital rocket','success',846,'','STV SR-45','','','Aeron?',1864,'',100 from dual union all
select 16196,'1967-S628','1967-03-28 16:33:00','suborbital rocket','success',846,'','STV SR-48','','','Aeron?',1864,'',100 from dual union all
select 16197,'1967-S629','1967-01-01 00:00:00','military missile','failure',356,'601602','R&D','','','Test',809,'',500 from dual union all
select 16198,'1967-S63','1967-01-18 01:34:00','military missile','success',162,'148F','TVX-13','ABRES TVX-13 (21)','BUSY STEPSON','Reentry Test',1681,'',1400 from dual union all
select 16199,'1967-S630','1967-05-26 00:00:00','military missile','success',356,'','R&D','','','Test',809,'',500 from dual union all
select 162,'1944-S147','1944-09-25 18:59:00','military missile','success',8,'18807','Ma224','','','Training',661,'',90 from dual union all
select 1620,'1953-A211','1953-05-12 00:00:00','military missile','success',703,'624','','','','Test',222,'',8 from dual union all
select 16200,'1967-S631','1967-06-10 00:00:00','military missile','success',356,'','R&D','','','Test',809,'',500 from dual union all
select 16201,'1967-S632','1967-01-11 17:22:00','suborbital rocket','success',837,'','SPEEDBALL II 96 SN42','','','Target',1895,'',150 from dual union all
select 16202,'1967-S633','1967-05-02 17:45:00','suborbital rocket','success',837,'','SPEEDBALL II 99 SN45','','','Target',1895,'',150 from dual union all
select 16203,'1967-S634','1967-05-17 15:00:00','suborbital rocket','success',837,'','SPEEDBALL II 100SN46','','','Target',1895,'',150 from dual union all
select 16204,'1967-S635','1967-05-24 17:10:00','suborbital rocket','success',837,'','SPEEDBALL II 101SN47','','','Target',1895,'',150 from dual union all
select 16205,'1967-S636','1967-07-26 18:00:00','military missile','success',837,'BMTS','SPEEDBALL II 104SN50','','','Target',1895,'',150 from dual union all
select 16206,'1967-S637','1967-08-02 20:34:00','military missile','failure',837,'BMTS','SPEEDBALL II 106SN42','','','Target',1895,'',5 from dual union all
select 16207,'1967-S638','1967-08-09 20:06:00','military missile','success',837,'BMTS','SPEEDBALL II 107SN43','','','Target',1895,'',150 from dual;
insert into LAUNCH
select 16208,'1967-S639','1967-08-22 19:37:00','military missile','success',837,'BMTS','SPEEDBALL II 108SN52','','','Target',1895,'',150 from dual union all
select 16209,'1967-S64','1967-01-19 00:00:00','military missile','success',1117,'G5','','','','Reentry Test',669,'',1000 from dual union all
select 1621,'1953-A212','1953-05-12 00:00:00','atmospheric rocket','success',836,'349P','Nike CET 49','','','Test',1895,'',10 from dual union all
select 16210,'1967-S640','1967-09-15 01:30:00','suborbital rocket','success',850,'MD-67-9.5-2','','','','Aeron?',1895,'',150 from dual union all
select 16211,'1967-S641','1967-12-01 00:00:00','suborbital rocket','success',535,'','HAD-WRESAT','','','Aeron',1835,'',100 from dual union all
select 16212,'1967-S642','1967-03-06 16:00:00','military missile','success',916,'AO-46','P-118 67 Spring 02','','','Test',204,'',180 from dual union all
select 16213,'1967-S643','1967-09-06 00:00:00','suborbital rocket','success',288,'C143','','','','Aeron Na',310,'',150 from dual union all
select 16214,'1967-S644','1967-09-07 00:00:00','suborbital rocket','success',288,'C144','','','','Aeron Na',310,'',150 from dual union all
select 16215,'1967-S645','1967-01-01 00:00:00','military missile','success',928,'Mk 2 RB','ISEP 1','Mk 2 RV','','Target',1900,'EAG-154',500 from dual union all
select 16216,'1967-S646','1967-01-01 00:00:00','military missile','success',928,'Mk 2 RB','ISEP 2','Mk 2 RV','','Target',1900,'EAG-154',500 from dual union all
select 16217,'1967-S647','1967-01-01 00:00:00','military missile','success',928,'Mk 2 RB','ISEP 3','Mk 2 RV','','Target',1900,'EAG-154',500 from dual union all
select 16218,'1967-S648','1967-11-29 00:00:00','military missile','success',932,'A3TX-03 (A3P-514)','USN Mk 2 Mod 2 RV x 2','','ANTELOPE 9','Target',1900,'EAG-154',500 from dual union all
select 16219,'1967-S649','1967-12-06 00:00:00','military missile','success',932,'A3TX-05 (A3P-544)','Antelope 10','','ANTELOPE 10','Target',1900,'EAG-154',500 from dual union all
select 1622,'1953-A213','1953-05-12 00:00:00','atmospheric rocket','success',836,'345P','Nike CET 45','','','Test',1895,'',10 from dual union all
select 16220,'1967-S65','1967-01-19 23:22:00','suborbital rocket','success',848,'CRL AF07.655','Release','','','Aeron',394,'',107 from dual union all
select 16221,'1967-S650','1967-05-17 00:00:00','military missile','success',932,'A3TX-06 (A3P-558)','USN Mk 2 Mod 2 RV x 2','','ANTELOPE 4','Target',1897,'EAG-154',500 from dual union all
select 16222,'1967-S651','1967-06-21 00:00:00','military missile','success',932,'A3TX-07 (A3P-573)','Antelope 5','','ANTELOPE 5','Target',1897,'EAG-154',500 from dual union all
select 16223,'1967-S652','1967-09-27 00:00:00','military missile','success',916,'','P-136 67 Fall 07','','','OT',204,'',180 from dual union all
select 16224,'1967-S653','1967-09-27 00:00:00','military missile','success',916,'','P-137 67 Fall 08','','','OT',204,'',180 from dual union all
select 16225,'1967-S654','1967-05-30 00:00:00','suborbital rocket','success',837,'','AEC SKOL','','','Aeron',855,'',150 from dual union all
select 16226,'1967-S655','1967-05-30 00:00:00','suborbital rocket','success',837,'','AEC SKOL','','','Aeron',855,'',150 from dual union all
select 16227,'1967-S656','1967-05-30 00:00:00','suborbital rocket','success',839,'','AEC SKOL','','','Aeron',855,'',100 from dual union all
select 16228,'1967-S657','1967-05-31 00:00:00','suborbital rocket','success',837,'','AEC SKOL','','','Aeron',855,'',150 from dual union all
select 16229,'1967-S658','1967-05-31 00:00:00','suborbital rocket','success',837,'','AEC SKOL','','','Aeron',855,'',150 from dual union all
select 1623,'1953-A214','1953-05-12 00:00:00','atmospheric rocket','success',836,'346P','Nike CET 46','','','Test',1895,'',10 from dual union all
select 16230,'1967-S659','1967-06-01 00:00:00','suborbital rocket','success',837,'','AEC SKOL','','','Aeron',855,'',150 from dual union all
select 16231,'1967-S66','1967-01-21 06:38:00','suborbital rocket','success',215,'AA-2-105','','','','Auroral/Ionos/Plasm',428,'',170 from dual union all
select 16232,'1967-S660','1967-06-01 00:00:00','suborbital rocket','success',837,'','AEC SKOL','','','Aeron',855,'',150 from dual union all
select 16233,'1967-S661','1967-06-02 00:00:00','suborbital rocket','success',837,'','AEC SKOL','','','Aeron',855,'',150 from dual union all
select 16234,'1967-S662','1967-06-02 00:00:00','suborbital rocket','success',837,'','AEC SKOL','','','Aeron',855,'',150 from dual union all
select 16235,'1967-S663','1967-06-02 00:00:00','suborbital rocket','success',839,'','AEC SKOL','','','Aeron',855,'',100 from dual union all
select 16236,'1967-S664','1967-02-21 21:04:00','suborbital rocket','success',837,'','SPEEDBALL II 98 SN44','','','Target',1895,'',150 from dual union all
select 16237,'1967-S665','1967-08-01 19:25:00','military missile','success',837,'BMTS','SPEEDBALL II 105SN49','','','Target',1895,'',150 from dual union all
select 16238,'1967-S666','1967-01-21 00:00:00','military missile','success',982,'G 22603-12T','8K64U UBP-P18','R-16 PL67-1','','OT',477,'',1210 from dual union all
select 16239,'1967-S667','1967-01-28 00:00:00','military missile','success',982,'G 22603-15T','8K64U UBP-P19','R-16 PL67-2','','OT',477,'',1210 from dual union all
select 1624,'1953-A215','1953-05-12 00:00:00','atmospheric rocket','success',836,'347P','Nike CET 47','','','Test',1895,'',10 from dual union all
select 16240,'1967-S668','1967-02-03 00:00:00','military missile','success',982,'G 22603-08T','8K64U UBP-P20','R-16 PL67-3','','OT',477,'',1210 from dual union all
select 16241,'1967-S669','1967-03-11 00:00:00','military missile','success',982,'G 22603-17T','8K64U UBP-P22','R-16 PL67-5','','OT',477,'',1210 from dual union all
select 16242,'1967-S67','1967-01-22 15:44:54','military missile','success',154,'35D','Mk 12R PDV','ABRES Mk 12R PDV','BUSY NIECE','Reentry Test',1684,'',1800 from dual union all
select 16243,'1967-S670','1967-04-19 00:00:00','military missile','success',982,'T 22501-10','8K64U UBP-P24','R-16 PL67-7','','OT',477,'',1210 from dual union all
select 16244,'1967-S671','1967-05-19 00:00:00','military missile','success',982,'T 22502-01T','8K64U UBP-P26','R-16 PL67-9','','OT',477,'',1210 from dual union all
select 16245,'1967-S672','1967-07-22 00:00:00','military missile','success',982,'G 22604-08T','8K64U UBP-P30','R-16 PL67-13','','OT',477,'',1210 from dual union all
select 16246,'1967-S673','1967-08-11 00:00:00','military missile','success',982,'T 22501-09','8K64U UBP-P32','R-16 PL67-15','','OT',477,'',1210 from dual union all
select 16247,'1967-S674','1967-09-02 00:00:00','military missile','failure',982,'','8K64U UBP-P33','R-16 PL67-16','','OT',477,'',0 from dual union all
select 16248,'1967-S675','1967-10-04 00:00:00','military missile','success',982,'T 22503-14T','8K64U UBP-P34','R-16 PL67-17','','OT',477,'',1210 from dual union all
select 16249,'1967-S676','1967-10-11 00:00:00','military missile','success',982,'T 22504-16T','8K64U UBP-P35','R-16 PL67-18','','OT',477,'',1210 from dual union all
select 1625,'1953-A216','1953-05-12 00:00:00','atmospheric rocket','success',836,'348P','Nike CET 48','','','Test',1895,'',10 from dual union all
select 16250,'1967-S677','1967-10-30 00:00:00','military missile','success',982,'T 22502-18T','8K64U UBP-P37','R-16 PL67-20','','OT',477,'',1210 from dual union all
select 16251,'1967-S678','1967-12-07 00:00:00','military missile','success',982,'T 22502-11T','8K64U UBP-P39','R-16 PL67-22','','OT',477,'',1210 from dual union all
select 16252,'1967-S679','1967-12-25 00:00:00','military missile','success',982,'T 22502-14T','8K64U UBP-P40','R-16 PL67-23','','OT',477,'',1210 from dual union all
select 16253,'1967-S68','1967-01-22 18:47:00','suborbital rocket','success',215,'AA-2-102','','','','Auroral/Ionos/Meteo',428,'',129 from dual union all
select 16254,'1967-S680','1967-12-27 00:00:00','military missile','success',982,'T 22504-02','8K64U UBP-P41','R-16 PL67-24','','OT',477,'',1210 from dual union all
select 16255,'1967-S681','1967-02-22 00:00:00','military missile','failure',982,'','8K64U UBP-P21','R-16 PL67-4','','OT',477,'',0 from dual union all
select 16256,'1967-S682','1967-03-31 00:00:00','military missile','success',982,'T 22502-09','8K64U UBP-P23','R-16 PL67-6','','OT',477,'',1210 from dual union all
select 16257,'1967-S683','1967-04-22 00:00:00','military missile','success',982,'G 22603-05T','8K64U UBP-P25','R-16 PL67-8','','OT',477,'',1210 from dual union all
select 16258,'1967-S684','1967-05-26 00:00:00','military missile','success',982,'G 22603-10T','8K64U UBP-P27','R-16 PL67-10','','OT',477,'',1210 from dual union all
select 16259,'1967-S685','1967-06-21 00:00:00','military missile','success',982,'T 22502-02','8K64U UBP-P28','R-16 PL67-11','','OT',477,'',1210 from dual union all
select 1626,'1953-A217','1953-05-14 00:00:00','military missile','unknown',339,'4EBN','EU-4','','','Test',1895,'',30 from dual union all
select 16260,'1967-S686','1967-07-03 00:00:00','military missile','success',982,'T 22502-08','8K64U UBP-P29','R-16 PL67-12','','OT',477,'',1210 from dual union all
select 16261,'1967-S687','1967-07-22 00:00:00','military missile','success',982,'G 22604-06T','8K64U UBP-P31','R-16 PL67-14','','OT',477,'',1210 from dual union all
select 16262,'1967-S688','1967-10-27 00:00:00','military missile','success',982,'T 22501-12T','8K64U UBP-P36','R-16 PL67-19','','OT',477,'',1210 from dual union all
select 16263,'1967-S689','1967-12-01 00:00:00','military missile','success',982,'T 22501-11T','8K64U UBP-P38','R-16 PL67-21','','OT',477,'',1210 from dual union all
select 16264,'1967-S69','1967-01-23 07:00:00','suborbital rocket','success',837,'Sphere 37','MD-67-1 NICAP','RO-67-1/ERDA 67-1','','Aeronomy',1895,'',108 from dual union all
select 16265,'1967-S70','1967-01-23 10:00:00','suborbital rocket','success',837,'Sphere','MD-67-2 NICAP','ERDA 67-2','','Aeronomy',1895,'',108 from dual union all
select 16266,'1967-S71','1967-01-23 13:00:00','suborbital rocket','success',837,'Sphere 35','MD-67-3 NICAP','ERDA 67-3','','Aeronomy',1895,'',82 from dual union all
select 16267,'1967-S72','1967-01-23 19:25:00','suborbital rocket','success',837,'Sphere 36','MD-67-4 NICAP','ERDA 67-4','','Aeronomy',1895,'',90 from dual union all
select 16268,'1967-S73','1967-01-24 00:00:00','military missile','success',795,'828','Mk 11 RV','FOT','TRUCK TRAVEL','OT',1705,'',900 from dual union all
select 16269,'1967-S730','1967-09-08 00:00:00','military missile','failure',1037,'','8K98 LKI-2-13','','','Test',477,'',1000 from dual union all
select 1627,'1953-A218','1953-05-14 00:00:00','military missile','success',703,'613','','','','Test',222,'',8 from dual union all
select 16270,'1967-S731','1967-10-23 00:00:00','military missile','success',1037,'Ya23106-33L','8K98 LKI-2-14','','','Test',477,'',1000 from dual union all
select 16271,'1967-S732','1967-10-29 00:00:00','military missile','success',1037,'Ya23103-30L','8K98 LKI-2-15','','','Test',477,'',1000 from dual union all
select 16272,'1967-S733','1967-11-29 00:00:00','military missile','failure',1037,'Ya23104-31L','8K98 LKI-2-16','','','Test',477,'',1000 from dual union all
select 16273,'1967-S734','1967-11-30 00:00:00','military missile','success',1037,'','8K98 LKI-2-17','','','Test',477,'',1000 from dual union all
select 16274,'1967-S735','1967-07-20 00:00:00','military missile','success',1018,'','8K75 UBP-T1','','','OT',1495,'',1160 from dual union all
select 16275,'1967-S736','1967-10-20 00:00:00','military missile','success',1018,'','8K75 UBP-T2','','','OT',1495,'',1160 from dual union all
select 16276,'1967-S737','1967-03-22 00:00:00','military missile','success',982,'','8K64 42 RD UBP-4','','','OT',1002,'',1210 from dual union all
select 16277,'1967-S738','1967-07-20 00:00:00','military missile','success',982,'','8K64 42 RD UBP-5','','','OT',1002,'',1210 from dual union all
select 16278,'1967-S739','1967-01-01 00:00:00','military missile','success',982,'','8K64 36 RD UBP-2','','','OT',898,'',1210 from dual union all
select 16279,'1967-S74','1967-01-24 08:00:00','suborbital rocket','success',1069,'MT-160-001','Meteo Test','','','Aeron',839,'',86 from dual union all
select 1628,'1953-A219','1953-05-14 00:00:00','atmospheric rocket','success',836,'1249B-1179','Nike 1249 R&D 207R','','','Test',1895,'',10 from dual union all
select 16280,'1967-S740','1967-07-20 00:00:00','military missile','success',1449,'','8K84 36 RD UBP-1','','','OT',898,'',1000 from dual union all
select 16281,'1967-S741','1967-06-01 00:00:00','military missile','success',1449,'','8K84 4 RD UBP-1','','','OT',368,'',1000 from dual union all
select 16282,'1967-S742','1967-10-27 00:00:00','atmospheric rocket','success',1536,'2','Dummy payload','','','Test',1343,'',140 from dual union all
select 16283,'1967-S743','1967-12-13 08:40:00','suborbital rocket','success',1019,'APGC-943','Sphere/TMA','','','Aeron',394,'',200 from dual union all
select 16284,'1967-S744','1967-12-16 00:00:00','suborbital rocket','success',1058,'','','','','Test',310,'',295 from dual union all
select 16285,'1967-S745','1967-02-02 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 16286,'1967-S746','1967-04-01 00:00:00','military missile','success',976,'','8K63 44 RD UBP','','','Test',636,'',402 from dual union all
select 16287,'1967-S747','1967-04-26 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 16288,'1967-S748','1967-06-05 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 16289,'1967-S749','1967-06-13 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 1629,'1953-A22','1953-01-16 00:00:00','atmospheric rocket','success',836,'1249A-173','Nike 1249 R&D 163R','','','Test',1895,'',10 from dual union all
select 16290,'1967-S75','1967-01-24 09:00:00','suborbital rocket','success',867,'MUMP-8 (ETR-1474)','Diurnal thermosphere','','','Aeron/Ionos',293,'',325 from dual union all
select 16291,'1967-S750','1967-12-27 00:00:00','military missile','success',976,'','8K63 43 RD UBP','','','Test',636,'',402 from dual union all
select 16292,'1967-S751','1967-02-07 00:00:00','military missile','success',977,'','8K63U 29 RD UBP','','','Test',636,'',402 from dual union all
select 16293,'1967-S752','1967-04-05 00:00:00','military missile','success',977,'','8K63U 40 RD UBP','','','Test',636,'',402 from dual union all
select 16294,'1967-S753','1967-10-23 00:00:00','military missile','success',976,'','8K63 43 RD UBP','','','Test',636,'',402 from dual union all
select 16295,'1967-S754','1967-10-28 00:00:00','military missile','success',977,'','8K63U 40 RD UBP','','','Test',636,'',402 from dual union all
select 16296,'1967-S755','1967-03-17 00:00:00','military missile','unknown',980,'','8K65U UBP 29 RD','','','Test',636,'',675 from dual union all
select 16297,'1967-S756','1967-05-27 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 16298,'1967-S757','1967-06-20 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual;
insert into LAUNCH
select 16299,'1967-S758','1967-08-15 00:00:00','military missile','unknown',979,'','8K65 UBP 68 RD','','','Test',636,'',675 from dual union all
select 163,'1944-S148','1944-09-25 19:25:00','military missile','success',8,'18800','Ma218','','','Training',661,'',90 from dual union all
select 1630,'1953-A220','1953-05-19 00:00:00','atmospheric rocket','success',836,'1249A-181','Nike 1249 R&D 208R','','','Test',1895,'',10 from dual union all
select 16300,'1967-S759','1967-09-12 00:00:00','military missile','unknown',980,'','8K65U UBP 29 RD','','','Test',636,'',675 from dual union all
select 16301,'1967-S76','1967-01-24 11:51:26','suborbital rocket','success',867,'MUMP-6 (ETR-1828)','','','','Aeron/Ionos',293,'',325 from dual union all
select 16302,'1967-S760','1967-09-29 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 16303,'1967-S761','1967-10-04 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 16304,'1967-S762','1967-08-01 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 16305,'1967-S763','1967-09-27 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 16306,'1967-S77','1967-01-24 15:08:54','suborbital rocket','success',867,'MUMP-3 (ETR-1165)','','','','Aeron/Ionos',293,'',324 from dual union all
select 16307,'1967-S78','1967-01-24 19:33:59','suborbital rocket','success',867,'MUMP-1 (ETR-0381)','','','','Aeron/Ionos',293,'',336 from dual union all
select 16308,'1967-S79','1967-01-24 22:50:00','suborbital rocket','success',867,'MUMP-2 (ETR-0611)','','','','Aeron/Ionos',293,'',320 from dual union all
select 16309,'1967-S80','1967-01-25 03:00:00','suborbital rocket','success',867,'MUMP-7 (ETR-0851)','','','','Aeron/Ionos',293,'',327 from dual union all
select 1631,'1953-A221','1953-05-19 00:00:00','atmospheric rocket','success',836,'1249B-1180','Nike 1249 R&D 209R','','','Test',1895,'',10 from dual union all
select 16310,'1967-S81','1967-04-25 06:30:00','suborbital rocket','success',867,'MUMP-4','','','','Aeron/Ionos',293,'',287 from dual union all
select 16311,'1967-S82','1967-04-25 19:00:00','suborbital rocket','success',867,'MUMP-5','','','','Aeron/Ionos',293,'',334 from dual union all
select 16312,'1967-S83','1967-01-25 23:16:00','suborbital rocket','success',446,'D-24','FU-145B Ions/part.','','','Ionos',361,'',340 from dual union all
select 16313,'1967-S84','1967-01-27 00:00:00','military missile','success',1117,'G6','','','','Reentry Test',669,'',1000 from dual union all
select 16314,'1967-S85','1967-01-27 02:15:00','suborbital rocket','unknown',1069,'IT-160-2','Instrument test','','','Test',839,'',86 from dual union all
select 16315,'1967-S86','1967-01-28 05:16:00','suborbital rocket','success',867,'NASA 18.23GE','Particle fluxes','','','Aeron/Ionos',428,'',285 from dual union all
select 16316,'1967-S87','1967-01-28 12:05:33','suborbital rocket','success',446,'D-25','FU-151 Ions/part.','','','Ionos',361,'',310 from dual union all
select 16317,'1967-S88','1967-01-29 01:21:25','suborbital rocket','success',446,'D-26','FU-151 Ions/part.','','','Ionos',361,'',320 from dual union all
select 16318,'1967-S89','1967-01-29 03:09:20','suborbital rocket','success',446,'D-27','FU-151 Ions/part.','','','Ionos',361,'',340 from dual union all
select 16319,'1967-S90','1967-01-30 21:35:00','military missile','success',795,'1206','Mk 11 RV','LGM-30B FOT-32','BUCCANEER SWORD','OT',1698,'',900 from dual union all
select 1632,'1953-A222','1953-05-21 00:00:00','military missile','unknown',339,'1264R','','','','Test',1895,'',30 from dual union all
select 16320,'1967-S91','1967-01-31 02:05:00','suborbital rocket','success',668,'K-9M-22','','','','Ionos/Solar RF',839,'',327 from dual union all
select 16321,'1967-S92','1967-01-31 18:50:00','suborbital rocket','success',837,'NASA 14.275UI','Electron density','','','Aeron/Ionos',1821,'',195 from dual union all
select 16322,'1967-S93','1967-01-31 19:11:00','suborbital rocket','success',839,'NASA 10.207GM','GSFC Grenade 135 W','','','Aeronomy',1821,'',110 from dual union all
select 16323,'1967-S94','1967-01-31 23:05:00','suborbital rocket','success',740,'','Thermosphere comp. 3','','','Aeron',1010,'',172 from dual union all
select 16324,'1967-S95','1967-01-31 23:17:00','suborbital rocket','success',837,'NASA 14.319UM','UM Pitot 25/Rel','','','Aeronomy',428,'',163 from dual union all
select 16325,'1967-S96','1967-01-31 23:20:00','suborbital rocket','success',837,'NASA 14.310CM','GCA Release 59 TMA','','','Aeronomy',428,'',154 from dual union all
select 16326,'1967-S97','1967-01-31 23:48:00','suborbital rocket','success',839,'NASA 10.216GM','GSFC Grenade 136 PB','','','Aeronomy',1039,'',126 from dual union all
select 16327,'1967-S98','1967-02-01 01:12:00','suborbital rocket','success',837,'NASA 14.311CM','GCA Release 60 TMA','','','Aeronomy',428,'',194 from dual union all
select 16328,'1967-S99','1967-02-01 01:41:00','suborbital rocket','success',839,'NASA 10.217GM','GSFC Grenade 137 PB','','','Aeronomy',1039,'',126 from dual union all
select 16329,'1967-U01','1967-02-14 05:52:00','suborbital rocket','success',1439,'Meteor 10','','Meteor 10','','Reentry Test',1821,'',306 from dual union all
select 1633,'1953-A223','1953-05-22 00:00:00','military missile','unknown',339,'1210R','','','','Test',1895,'',30 from dual union all
select 16330,'1967-U02','1967-03-05 23:05:00','test rocket','success',175,'7002','SV-5D','SV-5D FV-2','GIANT CHIEF','Reentry Test',1775,'',1500 from dual union all
select 16331,'1967-U03','1967-04-20 01:35:01','test rocket','success',175,'7003','SV-5D','SV-5D FV-3','BUSY TOURNAMENT','Reentry Test',1775,'',1500 from dual union all
select 16332,'1967-U04','1967-10-19 17:33:00','test rocket','success',1128,'S159C','RAM C-1','RAM C-1','','Reentry Test',1814,'',217 from dual union all
select 16333,'1967-W01','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16334,'1967-W02','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16335,'1967-W03','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16336,'1967-W04','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16337,'1967-W05','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16338,'1967-W06','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16339,'1967-W07','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1634,'1953-A224','1953-05-25 00:00:00','atmospheric rocket','unknown',576,'353','','','','Test',1895,'',10 from dual union all
select 16340,'1967-W08','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16341,'1967-W09','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16342,'1967-W10','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16343,'1967-W100','1967-04-17 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',79 from dual union all
select 16344,'1967-W101','1967-04-19 10:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',65 from dual union all
select 16345,'1967-W102','1967-04-22 12:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',65 from dual union all
select 16346,'1967-W103','1967-04-24 10:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',68 from dual union all
select 16347,'1967-W104','1967-04-27 05:03:00','suborbital rocket','success',725,'','','','','Meteo',636,'',60 from dual union all
select 16348,'1967-W105','1967-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16349,'1967-W106','1967-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1635,'1953-A225','1953-05-25 00:00:00','atmospheric rocket','unknown',576,'354','','','','Test',1895,'',10 from dual union all
select 16350,'1967-W107','1967-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16351,'1967-W108','1967-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16352,'1967-W109','1967-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16353,'1967-W11','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16354,'1967-W110','1967-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16355,'1967-W111','1967-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16356,'1967-W112','1967-05-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16357,'1967-W113','1967-05-06 10:00:59','suborbital rocket','success',725,'','','','','Meteo',636,'',72 from dual union all
select 16358,'1967-W114','1967-05-13 04:04:59','suborbital rocket','success',725,'','','','','Meteo',636,'',70 from dual union all
select 16359,'1967-W115','1967-05-15 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',79 from dual union all
select 1636,'1953-A226','1953-05-26 00:00:00','atmospheric rocket','unknown',576,'307','','','','Test',1895,'',10 from dual union all
select 16360,'1967-W116','1967-05-15 10:03:59','suborbital rocket','success',725,'','','','','Meteo',636,'',50 from dual union all
select 16361,'1967-W117','1967-05-17 10:06:00','suborbital rocket','success',725,'','','','','Meteo',636,'',62 from dual union all
select 16362,'1967-W118','1967-05-17 16:15:00','suborbital rocket','success',88,'EXAMETNET C20','Arcasonde 2B','','','Aeron',310,'',70 from dual union all
select 16363,'1967-W119','1967-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16364,'1967-W12','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16365,'1967-W120','1967-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16366,'1967-W121','1967-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16367,'1967-W122','1967-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16368,'1967-W123','1967-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16369,'1967-W124','1967-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1637,'1953-A227','1953-05-26 00:00:00','atmospheric rocket','unknown',576,'309','','','','Test',1895,'',10 from dual union all
select 16370,'1967-W125','1967-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16371,'1967-W126','1967-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16372,'1967-W127','1967-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16373,'1967-W128','1967-06-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16374,'1967-W129','1967-06-07 02:00:00','suborbital rocket','success',88,'','Charged particles','','','Ionos',428,'',70 from dual union all
select 16375,'1967-W13','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16376,'1967-W130','1967-06-14 16:40:00','suborbital rocket','success',88,'EXAMETNET C21','Arcasonde 2B','','','Aeron',310,'',72 from dual union all
select 16377,'1967-W131','1967-06-20 10:06:00','suborbital rocket','success',725,'','','','','Meteo',636,'',64 from dual union all
select 16378,'1967-W132','1967-06-21 20:06:00','suborbital rocket','failure',88,'NASA 15.13UP','Chaff test','','','Aeronomy',1821,'',60 from dual union all
select 16379,'1967-W133','1967-06-24 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',78 from dual union all
select 1638,'1953-A228','1953-05-28 00:00:00','atmospheric rocket','success',836,'1249B-1151','Nike 1249 R&D 210R','','','Test',1895,'',10 from dual union all
select 16380,'1967-W134','1967-06-26 09:30:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',63 from dual union all
select 16381,'1967-W135','1967-06-27 11:00:00','suborbital rocket','success',725,'','','','','Meteo',636,'',70 from dual union all
select 16382,'1967-W136','1967-06-28 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',53 from dual union all
select 16383,'1967-W137','1967-06-28 10:03:59','suborbital rocket','success',725,'','','','','Meteo',636,'',79 from dual union all
select 16384,'1967-W138','1967-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16385,'1967-W139','1967-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16386,'1967-W14','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16387,'1967-W140','1967-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16388,'1967-W141','1967-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual;
insert into LAUNCH
select 16389,'1967-W142','1967-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1639,'1953-A229','1953-05-29 00:00:00','military missile','unknown',339,'1266R','','','','Test',1895,'',30 from dual union all
select 16390,'1967-W143','1967-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16391,'1967-W144','1967-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16392,'1967-W145','1967-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16393,'1967-W146','1967-07-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16394,'1967-W147','1967-07-08 00:10:00','suborbital rocket','success',1188,'Su1H','D-region','','','Ionospheric',1447,'',73 from dual union all
select 16395,'1967-W148','1967-07-10 09:02:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',73 from dual union all
select 16396,'1967-W149','1967-07-12 09:24:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',53 from dual union all
select 16397,'1967-W15','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16398,'1967-W150','1967-07-12 09:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',73 from dual union all
select 16399,'1967-W151','1967-07-12 17:02:00','suborbital rocket','success',631,'EXAMETNET C22','[Not in SP-176]','','','Aeron',310,'',70 from dual union all
select 164,'1944-S149','1944-09-26 11:23:00','military missile','success',8,'18802','Ma219','','','Training',661,'',90 from dual union all
select 1640,'1953-A23','1953-01-22 00:00:00','military missile','unknown',339,'42R','','','','Test',1895,'',30 from dual union all
select 16400,'1967-W152','1967-07-15 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',77 from dual union all
select 16401,'1967-W153','1967-07-17 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',76 from dual union all
select 16402,'1967-W154','1967-07-17 09:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',74 from dual union all
select 16403,'1967-W155','1967-07-19 09:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',70 from dual union all
select 16404,'1967-W156','1967-07-28 00:00:00','suborbital rocket','success',88,'','PEPP support','','','Aeron',1895,'',60 from dual union all
select 16405,'1967-W157','1967-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16406,'1967-W158','1967-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16407,'1967-W159','1967-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16408,'1967-W16','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16409,'1967-W160','1967-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1641,'1953-A230','1953-06-02 00:00:00','atmospheric rocket','success',836,'1249B-1153','Nike 1249 R&D 211R','','','Test',1895,'',10 from dual union all
select 16410,'1967-W161','1967-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16411,'1967-W162','1967-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16412,'1967-W163','1967-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16413,'1967-W164','1967-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16414,'1967-W165','1967-08-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16415,'1967-W166','1967-08-02 22:58:00','suborbital rocket','success',88,'NASA 15.16DA','OGO 4 Ozone test','','','Aeronomy',1057,'',52 from dual union all
select 16416,'1967-W167','1967-08-16 14:25:00','suborbital rocket','success',631,'EXAMETNET C23','Chaff','','','Aeron',310,'',68 from dual union all
select 16417,'1967-W168','1967-08-18 10:01:59','suborbital rocket','success',725,'','','','','Meteo',636,'',71 from dual union all
select 16418,'1967-W169','1967-08-21 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',72 from dual union all
select 16419,'1967-W17','1967-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1642,'1953-A231','1953-06-02 00:00:00','atmospheric rocket','success',836,'1249B-1106','Nike 1249 R&D 212R','','','Test',1895,'',10 from dual union all
select 16420,'1967-W170','1967-08-21 19:32:00','suborbital rocket','success',88,'NASA 15.17DA','OGO 4 Ozone test','','','Aeronomy',1057,'',54 from dual union all
select 16421,'1967-W171','1967-08-22 00:00:00','suborbital rocket','success',88,'#76','','','','Aeron',1895,'',68 from dual union all
select 16422,'1967-W172','1967-08-23 10:01:59','suborbital rocket','success',725,'','','','','Meteo',636,'',53 from dual union all
select 16423,'1967-W173','1967-08-30 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',76 from dual union all
select 16424,'1967-W174','1967-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16425,'1967-W175','1967-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16426,'1967-W176','1967-09-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16427,'1967-W177','1967-09-01 10:06:00','suborbital rocket','success',725,'','','','','Meteo',636,'',70 from dual union all
select 16428,'1967-W178','1967-09-13 20:30:00','suborbital rocket','success',631,'EXAMETNET C24','Chaff','','','Aeron',310,'',69 from dual union all
select 16429,'1967-W179','1967-09-14 00:29:00','suborbital rocket','success',88,'NASA 15.35DA','OGO 4','','','Aeronomy',855,'',60 from dual union all
select 1643,'1953-A232','1953-06-04 00:00:00','atmospheric rocket','success',836,'1249B-1192','Nike 1249 R&D 213R','','','Test',1895,'',10 from dual union all
select 16430,'1967-W18','1967-01-31 22:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16431,'1967-W180','1967-09-14 10:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',67 from dual union all
select 16432,'1967-W181','1967-09-17 00:15:00','suborbital rocket','success',88,'NASA 15.36DA','OGO 4','','','Aeronomy',855,'',56 from dual union all
select 16433,'1967-W182','1967-09-18 09:08:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',75 from dual union all
select 16434,'1967-W183','1967-09-18 10:06:00','suborbital rocket','success',725,'','','','','Meteo',636,'',60 from dual union all
select 16435,'1967-W184','1967-09-19 00:00:00','suborbital rocket','success',88,'#91','','','','Aeron',1895,'',64 from dual union all
select 16436,'1967-W185','1967-09-19 00:00:00','suborbital rocket','success',88,'#96','','','','Aeron',1895,'',65 from dual union all
select 16437,'1967-W186','1967-09-20 00:00:00','suborbital rocket','success',88,'NASA 15.37DA','OGO 4','','','Aeronomy',855,'',53 from dual union all
select 16438,'1967-W187','1967-09-20 00:00:00','suborbital rocket','success',88,'#98','','','','Aeron',1895,'',73 from dual union all
select 16439,'1967-W188','1967-09-20 10:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',52 from dual union all
select 1644,'1953-A233','1953-06-04 00:00:00','atmospheric rocket','success',836,'1249B-1098','Nike 1249 R&D 214R','','','Test',1895,'',10 from dual union all
select 16440,'1967-W189','1967-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16441,'1967-W19','1967-01-03 08:00:00','suborbital rocket','success',725,'','','','','Meteo',1542,'SHIP-SOV',64 from dual union all
select 16442,'1967-W190','1967-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16443,'1967-W191','1967-10-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16444,'1967-W192','1967-10-13 23:35:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16445,'1967-W193','1967-10-13 00:00:00','suborbital rocket','success',716,'162','','','','Aeron',1895,'',70 from dual union all
select 16446,'1967-W194','1967-10-13 21:34:00','suborbital rocket','success',88,'NASA 15.38DA','OGO 4','','','Aeronomy',855,'',59 from dual union all
select 16447,'1967-W195','1967-10-18 21:03:00','suborbital rocket','success',631,'EXAMETNET C25','Chaff','','','Aeron',310,'',67 from dual union all
select 16448,'1967-W196','1967-10-19 00:00:00','suborbital rocket','success',716,'173','','','','Aeron',1895,'',66 from dual union all
select 16449,'1967-W197','1967-10-19 21:04:00','suborbital rocket','success',88,'NASA 15.39DA','OGO 4','','','Aeronomy',855,'',58 from dual union all
select 1645,'1953-A234','1953-06-05 00:00:00','atmospheric rocket','success',836,'1188E','','','','Test',1895,'',10 from dual union all
select 16450,'1967-W198','1967-10-20 00:00:00','suborbital rocket','success',716,'175','','','','Aeron',1895,'',73 from dual union all
select 16451,'1967-W199','1967-10-20 09:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',50 from dual union all
select 16452,'1967-W20','1967-01-03 10:25:59','suborbital rocket','success',725,'','','','','Meteo',1499,'SHIP-SOV',60 from dual union all
select 16453,'1967-W200','1967-10-22 20:46:00','suborbital rocket','success',88,'NASA 15.40DA','OGO 4','','','Aeronomy',855,'',59 from dual union all
select 16454,'1967-W201','1967-10-23 09:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',54 from dual union all
select 16455,'1967-W202','1967-10-24 14:00:00','suborbital rocket','success',257,'NASA 15.32GI','Mag North','','','Ionos',1315,'',60 from dual union all
select 16456,'1967-W203','1967-10-24 19:50:00','suborbital rocket','success',257,'NASA 15.33GI','Mag North','','','Ionos',1315,'',60 from dual union all
select 16457,'1967-W204','1967-10-25 20:30:00','suborbital rocket','success',88,'NASA 15.41DA','OGO 4','','','Aeronomy',855,'',59 from dual union all
select 16458,'1967-W205','1967-10-27 10:00:59','suborbital rocket','success',725,'','','','','Meteo',636,'',73 from dual union all
select 16459,'1967-W206','1967-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1646,'1953-A235','1953-06-08 00:00:00','military missile','unknown',339,'5EBN','EU-5','','','Test',1895,'',30 from dual union all
select 16460,'1967-W207','1967-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16461,'1967-W208','1967-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16462,'1967-W209','1967-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16463,'1967-W21','1967-01-04 09:12:59','suborbital rocket','success',725,'','','','','Meteo',1541,'SHIP-SOV',78 from dual union all
select 16464,'1967-W210','1967-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16465,'1967-W211','1967-11-02 00:00:00','suborbital rocket','success',716,'182','','','','Aeron',1895,'',60 from dual union all
select 16466,'1967-W212','1967-11-08 00:00:00','suborbital rocket','success',88,'#119 R-112','','','','Aeron',1895,'',72 from dual union all
select 16467,'1967-W213','1967-11-08 00:00:00','suborbital rocket','success',716,'187 R114','','','','Aeron',1895,'',73 from dual union all
select 16468,'1967-W214','1967-11-08 13:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 16469,'1967-W215','1967-11-09 14:30:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 1647,'1953-A236','1953-06-09 00:00:00','atmospheric rocket','unknown',576,'211','','','','Test',1895,'',10 from dual union all
select 16470,'1967-W216','1967-11-11 09:01:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',78 from dual union all
select 16471,'1967-W217','1967-11-15 00:00:00','suborbital rocket','success',88,'#118 R-113','','','','Aeron',1895,'',70 from dual union all
select 16472,'1967-W218','1967-11-15 00:00:00','suborbital rocket','success',88,'#117 R-112','','','','Aeron',1895,'',68 from dual union all
select 16473,'1967-W219','1967-11-15 15:57:00','suborbital rocket','success',88,'EXAMETNET C26','Arcasonde 2B','','','Aeron',310,'',70 from dual union all
select 16474,'1967-W22','1967-01-05 08:34:00','suborbital rocket','success',725,'','','','','Meteo',1540,'SHIP-SOV',78 from dual union all
select 16475,'1967-W220','1967-11-20 05:15:00','suborbital rocket','success',631,'TERLS-53','ISRO 11.28','TERLS-053','','Aeron',1467,'',53 from dual union all
select 16476,'1967-W221','1967-11-22 10:01:59','suborbital rocket','success',725,'','','','','Meteo',636,'',54 from dual union all
select 16477,'1967-W222','1967-11-22 11:30:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',78 from dual union all
select 16478,'1967-W223','1967-11-24 08:49:59','suborbital rocket','success',725,'','','','','Meteo',636,'',50 from dual;
insert into LAUNCH
select 16479,'1967-W224','1967-11-27 10:30:00','suborbital rocket','success',725,'','','','','Meteo',636,'',54 from dual union all
select 1648,'1953-A237','1953-06-09 00:00:00','atmospheric rocket','success',836,'1187E','','','','Test',1895,'',10 from dual union all
select 16480,'1967-W225','1967-11-28 05:03:59','suborbital rocket','success',725,'','','','','Meteo',636,'',54 from dual union all
select 16481,'1967-W226','1967-11-29 02:10:00','suborbital rocket','unknown',631,'SUPARCO-033/67','','','','Aeron',1419,'',60 from dual union all
select 16482,'1967-W227','1967-11-29 10:07:59','suborbital rocket','success',725,'','','','','Meteo',636,'',54 from dual union all
select 16483,'1967-W228','1967-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16484,'1967-W229','1967-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16485,'1967-W23','1967-01-10 09:04:59','suborbital rocket','success',725,'','','','','Meteo',1535,'SHIP-SOV',75 from dual union all
select 16486,'1967-W230','1967-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16487,'1967-W231','1967-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16488,'1967-W232','1967-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16489,'1967-W233','1967-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1649,'1953-A238','1953-06-10 00:00:00','military missile','failure',255,'623-1','','','','Test',254,'',20 from dual union all
select 16490,'1967-W234','1967-12-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16491,'1967-W235','1967-12-11 09:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',72 from dual union all
select 16492,'1967-W236','1967-12-12 09:13:00','suborbital rocket','unknown',631,'SUPARCO-034/67','','','','Aeron',1419,'',63 from dual union all
select 16493,'1967-W237','1967-12-13 13:55:00','suborbital rocket','success',631,'EXAMETNET C27','Chaff','','','Aeron',310,'',69 from dual union all
select 16494,'1967-W238','1967-12-20 09:00:59','suborbital rocket','success',725,'','','','','Meteo',636,'',75 from dual union all
select 16495,'1967-W239','1967-12-20 09:39:00','suborbital rocket','success',631,'TERLS-55','ISRO 11.29','TERLS-055','','Aeron',1467,'',55 from dual union all
select 16496,'1967-W24','1967-01-18 09:35:59','suborbital rocket','success',725,'','','','','Meteo',636,'',74 from dual union all
select 16497,'1967-W240','1967-01-18 16:04:00','suborbital rocket','success',88,'EXAMETNET W23','Arcasonde IA','','','Meteo',1821,'',53 from dual union all
select 16498,'1967-W241','1967-01-25 16:39:00','suborbital rocket','success',88,'EXAMETNET W24','Arcasonde IA','','','Meteo',1821,'',50 from dual union all
select 16499,'1967-W242','1967-01-31 19:35:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 165,'1944-S15','1944-03-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1650,'1953-A239','1953-06-10 00:00:00','atmospheric rocket','success',365,'','XF3H-1 Model','','','Test',1821,'',10 from dual union all
select 16500,'1967-W243','1967-02-01 18:38:00','suborbital rocket','success',88,'EXAMETNET W25','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16501,'1967-W244','1967-02-03 18:25:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16502,'1967-W245','1967-02-15 16:51:00','suborbital rocket','success',88,'EXAMETNET W27','Arcasonde IA','','','Meteo',1821,'',61 from dual union all
select 16503,'1967-W246','1967-03-08 15:21:00','suborbital rocket','success',88,'EXAMETNET W29','Arcasonde IA','','','Meteo',1821,'',59 from dual union all
select 16504,'1967-W247','1967-03-24 19:13:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16505,'1967-W25','1967-01-18 14:15:00','suborbital rocket','success',631,'EXAMETNET C16','Chaff','','','Aeron',310,'',68 from dual union all
select 16506,'1967-W250','1967-04-06 21:43:00','suborbital rocket','success',88,'EXAMETNET W33','Arcasonde IA','','','Meteo',1821,'',61 from dual union all
select 16507,'1967-W251','1967-04-11 18:25:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16508,'1967-W252','1967-04-14 15:01:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16509,'1967-W253','1967-04-20 18:06:00','suborbital rocket','success',88,'EXAMETNET W35','Arcasonde IA','','','Meteo',1821,'',63 from dual union all
select 1651,'1953-A24','1953-01-22 00:00:00','atmospheric rocket','success',365,'','B-58 Model','','','Test',1821,'',10 from dual union all
select 16510,'1967-W254','1967-04-26 14:51:00','suborbital rocket','success',88,'EXAMETNET W36','Arcasonde IA','','','Meteo',1821,'',58 from dual union all
select 16511,'1967-W255','1967-10-05 14:13:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16512,'1967-W256','1967-10-06 18:34:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16513,'1967-W257','1967-10-12 15:30:00','suborbital rocket','success',88,'EXAMETNET W58','Arcasonde IA','','','Meteo',1821,'',50 from dual union all
select 16514,'1967-W258','1967-10-20 13:50:00','suborbital rocket','success',88,'EXAMETNET W59','Arcasonde IA','','','Meteo',1821,'',53 from dual union all
select 16515,'1967-W259','1967-10-25 14:17:00','suborbital rocket','success',88,'EXAMETNET W60','Arcasonde IA','','','Meteo',1821,'',55 from dual union all
select 16516,'1967-W26','1967-01-19 02:00:00','suborbital rocket','success',759,'MT-135-021','','','','Aeron',839,'',60 from dual union all
select 16517,'1967-W260','1967-10-27 14:55:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16518,'1967-W261','1967-11-03 17:26:00','suborbital rocket','success',88,'EXAMETNET W61','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16519,'1967-W262','1967-11-17 15:42:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 1652,'1953-A240','1953-06-11 00:00:00','atmospheric rocket','success',836,'1249B-1124','Nike 1249 R&D 215R','','','Test',1895,'',10 from dual union all
select 16520,'1967-W263','1967-11-21 15:15:00','suborbital rocket','success',88,'EXAMETNET W63','Arcasonde IA','','','Meteo',1821,'',51 from dual union all
select 16521,'1967-W264','1967-11-29 19:53:00','suborbital rocket','success',88,'EXAMETNET W64','Arcasonde IA','','','Meteo',1821,'',57 from dual union all
select 16522,'1967-W265','1967-12-08 17:59:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16523,'1967-W266','1967-12-13 18:16:00','suborbital rocket','success',88,'EXAMETNET W66','Arcasonde IA','','','Meteo',1821,'',53 from dual union all
select 16524,'1967-W267','1967-12-19 20:41:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16525,'1967-W268','1967-05-03 14:07:00','suborbital rocket','success',88,'EXAMETNET W37','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16526,'1967-W269','1967-05-04 20:33:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16527,'1967-W27','1967-01-21 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1516,'SHIP-SOV',79 from dual union all
select 16528,'1967-W270','1967-05-10 17:58:00','suborbital rocket','success',88,'EXAMETNET W38','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16529,'1967-W271','1967-05-17 14:29:00','suborbital rocket','success',88,'EXAMETNET W39','Arcasonde IA','','','Meteo',1821,'',58 from dual union all
select 1653,'1953-A241','1953-06-11 00:00:00','atmospheric rocket','success',836,'1249A-184','Nike 1249 R&D 216R','','','Test',1895,'',10 from dual union all
select 16530,'1967-W272','1967-05-25 18:49:00','suborbital rocket','success',88,'EXAMETNET W40','Arcasonde IA','','','Meteo',1821,'',54 from dual union all
select 16531,'1967-W273','1967-06-02 18:46:00','suborbital rocket','success',88,'EXAMETNET W41','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16532,'1967-W274','1967-06-07 14:32:00','suborbital rocket','success',88,'EXAMETNET W42','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16533,'1967-W275','1967-06-15 17:42:00','suborbital rocket','success',88,'EXAMETNET W43','Arcasonde IA','','','Meteo',1821,'',53 from dual union all
select 16534,'1967-W276','1967-06-21 14:14:00','suborbital rocket','success',88,'EXAMETNET W44','Arcasonde IA','','','Meteo',1821,'',55 from dual union all
select 16535,'1967-W277','1967-06-28 15:01:00','suborbital rocket','success',88,'EXAMETNET W45','Arcasonde IA','','','Meteo',1821,'',58 from dual union all
select 16536,'1967-W278','1967-07-05 14:42:00','suborbital rocket','success',88,'EXAMETNET W46','Arcasonde IA','','','Meteo',1821,'',65 from dual union all
select 16537,'1967-W279','1967-07-26 14:14:00','suborbital rocket','success',88,'EXAMETNET W48','Arcasonde IA','','','Meteo',1821,'',64 from dual union all
select 16538,'1967-W28','1967-01-21 11:04:59','suborbital rocket','success',725,'','','','','Meteo',1551,'SHIP-SOV',50 from dual union all
select 16539,'1967-W280','1967-07-28 13:59:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 1654,'1953-A242','1953-06-12 00:00:00','atmospheric rocket','unknown',576,'310','','','','Test',1895,'',10 from dual union all
select 16540,'1967-W281','1967-08-16 17:30:00','suborbital rocket','success',88,'EXAMETNET W50','Arcasonde IA','','','Meteo',1821,'',62 from dual union all
select 16541,'1967-W282','1967-08-18 14:11:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16542,'1967-W283','1967-08-25 14:17:00','suborbital rocket','success',88,'EXAMETNET W51','Arcasonde IA','','','Meteo',1821,'',58 from dual union all
select 16543,'1967-W284','1967-08-30 18:18:00','suborbital rocket','success',88,'EXAMETNET W52','Arcasonde IA','','','Meteo',1821,'',57 from dual union all
select 16544,'1967-W285','1967-09-06 14:35:00','suborbital rocket','success',88,'EXAMETNET W53','Arcasonde IA','','','Meteo',1821,'',55 from dual union all
select 16545,'1967-W286','1967-09-08 14:18:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16546,'1967-W287','1967-09-15 13:45:00','suborbital rocket','success',88,'EXAMETNET W54','Arcasonde IA','','','Meteo',1821,'',51 from dual union all
select 16547,'1967-W288','1967-09-20 15:29:00','suborbital rocket','success',88,'EXAMETNET W55','Arcasonde IA','','','Meteo',1821,'',54 from dual union all
select 16548,'1967-W289','1967-09-22 15:30:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16549,'1967-W29','1967-01-22 02:00:00','suborbital rocket','success',759,'MT-135-022','','','','Aeron',839,'',63 from dual union all
select 1655,'1953-A243','1953-06-12 00:00:00','atmospheric rocket','unknown',576,'311','','','','Test',1895,'',10 from dual union all
select 16550,'1967-W290','1967-09-27 14:45:00','suborbital rocket','success',88,'EXAMETNET W56','Arcasonde IA','','','Meteo',1821,'',54 from dual union all
select 16551,'1967-W291','1967-05-05 07:26:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16552,'1967-W292','1967-08-09 00:43:00','suborbital rocket','success',88,'','Arcasonde IA','','','Meteo',1821,'',60 from dual union all
select 16553,'1967-W293','1967-08-09 01:30:00','suborbital rocket','success',88,'EXAMETNET W49','Arcasonde IA','','','Meteo',1821,'',49 from dual union all
select 16554,'1967-W294','1967-01-23 06:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeronomy',1895,'',59 from dual union all
select 16555,'1967-W295','1967-01-23 09:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeronomy',1895,'',60 from dual union all
select 16556,'1967-W296','1967-01-23 12:00:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeronomy',1895,'',63 from dual union all
select 16557,'1967-W297','1967-01-23 18:15:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeronomy',1895,'',59 from dual union all
select 16558,'1967-W298','1967-01-23 18:41:00','suborbital rocket','success',88,'','Arcasonde IA','','','Aeronomy',1895,'',56 from dual union all
select 16559,'1967-W299','1967-06-19 14:30:00','suborbital rocket','success',88,'','PEPP support','','','Meteo',1895,'',50 from dual union all
select 1656,'1953-A244','1953-06-12 00:00:00','military missile','success',771,'GM-11056','','','','Test',222,'',12 from dual union all
select 16560,'1967-W30','1967-01-22 05:00:00','suborbital rocket','success',759,'MT-135-023','','','','Aeron',839,'',62 from dual union all
select 16561,'1967-W300','1967-01-18 15:00:00','suborbital rocket','success',631,'EXAMETNET N17','Chaff','','','Aeron',330,'',67 from dual union all
select 16562,'1967-W301','1967-02-01 15:00:00','suborbital rocket','success',631,'EXAMETNET N18','Chaff','','','Aeron',330,'',65 from dual union all
select 16563,'1967-W302','1967-02-15 15:00:00','suborbital rocket','success',631,'EXAMETNET N19','Chaff','','','Aeron',330,'',64 from dual union all
select 16564,'1967-W303','1967-02-22 15:00:00','suborbital rocket','success',631,'EXAMETNET N20','Chaff','','','Aeron',330,'',60 from dual union all
select 16565,'1967-W304','1967-03-01 15:00:00','suborbital rocket','success',631,'EXAMETNET N21','Chaff','','','Aeron',330,'',65 from dual union all
select 16566,'1967-W305','1967-03-22 15:00:00','suborbital rocket','success',631,'EXAMETNET N22','Chaff','','','Aeron',330,'',60 from dual union all
select 16567,'1967-W306','1967-03-29 16:27:00','suborbital rocket','success',631,'EXAMETNET N23','Chaff','','','Aeron',330,'',67 from dual union all
select 16568,'1967-W307','1967-06-14 15:11:00','suborbital rocket','success',631,'EXAMETNET N24','Chaff','','','Aeron',330,'',61 from dual;
insert into LAUNCH
select 16569,'1967-W308','1967-07-05 15:00:00','suborbital rocket','success',631,'EXAMETNET N25','Chaff','','','Aeron',330,'',66 from dual union all
select 1657,'1953-A245','1953-06-16 00:00:00','atmospheric rocket','success',836,'1249B-1134','Nike 1249 R&D 217R','','','Test',1895,'',10 from dual union all
select 16570,'1967-W309','1967-07-12 16:58:00','suborbital rocket','success',631,'EXAMETNET N26','Chaff','','','Aeron',330,'',66 from dual union all
select 16571,'1967-W31','1967-01-22 08:00:00','suborbital rocket','success',759,'MT-135-024','','','','Aeron',839,'',59 from dual union all
select 16572,'1967-W310','1967-08-02 15:00:00','suborbital rocket','success',631,'EXAMETNET N27','Chaff','','','Aeron',330,'',66 from dual union all
select 16573,'1967-W311','1967-08-16 15:00:00','suborbital rocket','success',631,'EXAMETNET N28','Chaff','','','Aeron',330,'',67 from dual union all
select 16574,'1967-W312','1967-09-13 15:00:00','suborbital rocket','success',631,'EXAMETNET N29','Chaff','','','Aeron',330,'',64 from dual union all
select 16575,'1967-W313','1967-10-25 16:30:00','suborbital rocket','success',631,'EXAMETNET N30','Chaff','','','Aeron',330,'',64 from dual union all
select 16576,'1967-W314','1967-11-15 14:00:00','suborbital rocket','success',631,'EXAMETNET N31','Chaff','','','Aeron',330,'',67 from dual union all
select 16577,'1967-W315','1967-12-13 15:00:00','suborbital rocket','success',631,'EXAMETNET N32','Chaff','','','Aeron',330,'',69 from dual union all
select 16578,'1967-W316','1967-02-09 15:01:00','suborbital rocket','success',631,'EXAMETNET W26','Chaff','','','Aeron',1821,'',70 from dual union all
select 16579,'1967-W317','1967-03-03 16:48:00','suborbital rocket','success',631,'EXAMETNET W28','Chaff','','','Aeron',1821,'',67 from dual union all
select 1658,'1953-A246','1953-06-16 00:00:00','atmospheric rocket','success',836,'1249B-1135','Nike 1249 R&D 218R','','','Test',1895,'',10 from dual union all
select 16580,'1967-W318','1967-03-16 14:29:00','suborbital rocket','success',631,'EXAMETNET W30','WOX-3A','','','Aeron',1821,'',59 from dual union all
select 16581,'1967-W319','1967-03-22 18:45:00','suborbital rocket','success',631,'EXAMETNET W31','Chaff','','','Aeron',1821,'',71 from dual union all
select 16582,'1967-W32','1967-01-23 10:30:00','suborbital rocket','success',725,'','','','','Meteo',1530,'SHIP-SOV',58 from dual union all
select 16583,'1967-W320','1967-03-29 19:52:00','suborbital rocket','success',631,'EXAMETNET W32','Chaff','','','Aeron',1821,'',68 from dual union all
select 16584,'1967-W321','1967-04-12 15:09:00','suborbital rocket','success',88,'EXAMETNET W34','Arcasonde IA','','','Meteo',1821,'',58 from dual union all
select 16585,'1967-W322','1967-10-05 00:07:00','suborbital rocket','success',88,'EXAMETNET W57','Arcasonde IA','','','Meteo',1821,'',54 from dual union all
select 16586,'1967-W323','1967-11-15 17:44:00','suborbital rocket','success',631,'EXAMETNET W62','Chaff','','','Aeron',1821,'',70 from dual union all
select 16587,'1967-W324','1967-12-06 19:45:00','suborbital rocket','success',631,'EXAMETNET W65','Chaff','','','Aeron',1821,'',66 from dual union all
select 16588,'1967-W325','1967-07-20 20:11:00','suborbital rocket','success',88,'EXAMETNET W47','Arcasonde IA','','','Meteo',1821,'',59 from dual union all
select 16589,'1967-W326','1967-06-17 00:00:00','suborbital rocket','success',717,'JD-07-14','Qual Test 1-1','','','Test',339,'',64 from dual union all
select 1659,'1953-A247','1953-06-18 00:00:00','atmospheric rocket','success',365,'','F29 Ramjet','','','Test',1821,'',8 from dual union all
select 16590,'1967-W327','1967-06-18 00:00:00','suborbital rocket','success',717,'JD-07-15','Qual Test 1-2','','','Test',339,'',65 from dual union all
select 16591,'1967-W328','1967-07-29 00:00:00','suborbital rocket','success',717,'LD-07-02','Qual Test 2-2','','','Test',339,'',61 from dual union all
select 16592,'1967-W329','1967-12-17 00:00:00','suborbital rocket','success',717,'LD-12-17-1','Qual Test 3-1','','','Test',339,'',66 from dual union all
select 16593,'1967-W33','1967-01-24 02:00:00','suborbital rocket','success',759,'MT-135-025','','','','Aeron',839,'',59 from dual union all
select 16594,'1967-W330','1967-12-17 00:00:00','suborbital rocket','success',717,'LD-12-17-2','Qual Test 3-2','','','Test',339,'',65 from dual union all
select 16595,'1967-W331','1967-12-17 00:00:00','suborbital rocket','success',717,'LD-12-17-3','Qual Test 3-3','','','Test',339,'',61 from dual union all
select 16596,'1967-W332','1967-12-17 00:00:00','suborbital rocket','success',717,'LD-12-17-4','Qual Test 3-4','','','Test',339,'',66 from dual union all
select 16597,'1967-W333','1967-12-17 00:00:00','suborbital rocket','success',717,'LD-12-17-5','Qual Test 3-5','','','Test',339,'',65 from dual union all
select 16598,'1967-W334','1967-12-17 00:00:00','suborbital rocket','success',717,'LD-12-17-6','Qual Test 3-6','','','Test',339,'',66 from dual union all
select 16599,'1967-W335','1967-12-17 00:00:00','suborbital rocket','success',717,'LD-12-17-7','Qual Test 3-7','','','Test',339,'',68 from dual union all
select 166,'1944-S150','1944-09-26 13:17:00','military missile','success',8,'18805','Ma221','','','Training',661,'',90 from dual union all
select 1660,'1953-A248','1953-06-18 00:00:00','military missile','success',572,'2','RV-A-8','','','Test',1863,'',30 from dual union all
select 16600,'1967-W336','1967-12-17 00:00:00','suborbital rocket','success',717,'LD-12-17-8','Qual Test 3-8','','','Test',339,'',64 from dual union all
select 16601,'1967-W337','1967-05-04 16:29:00','suborbital rocket','success',88,'','Robin','','','Meteo',1415,'',60 from dual union all
select 16602,'1967-W338','1967-05-04 16:29:00','suborbital rocket','success',88,'','Robin','','','Meteo',1727,'',60 from dual union all
select 16603,'1967-W339','1967-05-04 16:29:00','suborbital rocket','success',88,'','Robin','','','Meteo',1057,'',60 from dual union all
select 16604,'1967-W34','1967-01-24 08:06:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 16605,'1967-W340','1967-05-04 04:03:00','suborbital rocket','success',88,'','Robin','','','Meteo',1057,'',60 from dual union all
select 16606,'1967-W341','1967-05-04 04:03:00','suborbital rocket','success',88,'','Robin','','','Meteo',1727,'',60 from dual union all
select 16607,'1967-W35','1967-01-27 11:08:00','suborbital rocket','success',725,'','','','','Meteo',1525,'SHIP-SOV',70 from dual union all
select 16608,'1967-W36','1967-01-28 11:13:00','suborbital rocket','success',725,'','','','','Meteo',1526,'SHIP-SOV',78 from dual union all
select 16609,'1967-W37','1967-01-30 11:13:00','suborbital rocket','success',725,'','','','','Meteo',1528,'SHIP-SOV',78 from dual union all
select 1661,'1953-A249','1953-06-19 00:00:00','military missile','unknown',339,'1268R','','','','Test',1895,'',30 from dual union all
select 16610,'1967-W38','1967-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16611,'1967-W39','1967-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16612,'1967-W40','1967-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16613,'1967-W41','1967-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16614,'1967-W42','1967-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16615,'1967-W43','1967-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16616,'1967-W44','1967-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16617,'1967-W45','1967-02-01 14:30:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16618,'1967-W46','1967-02-08 09:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',74 from dual union all
select 16619,'1967-W47','1967-02-15 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1511,'SHIP-SOV',78 from dual union all
select 1662,'1953-A25','1953-01-22 00:00:00','atmospheric rocket','success',836,'1249A-170','Nike 1249 R&D 166R','','','Test',1895,'',10 from dual union all
select 16620,'1967-W48','1967-02-15 14:01:00','suborbital rocket','success',631,'EXAMETNET C17','Chaff','','','Aeron',310,'',69 from dual union all
select 16621,'1967-W49','1967-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16622,'1967-W50','1967-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16623,'1967-W51','1967-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16624,'1967-W52','1967-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16625,'1967-W53','1967-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16626,'1967-W54','1967-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16627,'1967-W55','1967-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16628,'1967-W56','1967-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16629,'1967-W57','1967-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1663,'1953-A250','1953-06-20 00:00:00','atmospheric rocket','unknown',576,'493','Acc Test 1A','','','Test',1895,'',10 from dual union all
select 16630,'1967-W58','1967-03-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16631,'1967-W59','1967-03-10 00:00:00','suborbital rocket','success',273,'','Robinette','','','Aeron',394,'',76 from dual union all
select 16632,'1967-W60','1967-03-22 11:00:00','suborbital rocket','success',725,'','','','','Meteo',636,'',52 from dual union all
select 16633,'1967-W61','1967-03-29 11:00:00','suborbital rocket','success',725,'','','','','Meteo',636,'',68 from dual union all
select 16634,'1967-W62','1967-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16635,'1967-W63','1967-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16636,'1967-W64','1967-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16637,'1967-W65','1967-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16638,'1967-W66','1967-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16639,'1967-W67','1967-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1664,'1953-A251','1953-06-20 00:00:00','atmospheric rocket','unknown',576,'494','Acc Test 1B','','','Test',1895,'',10 from dual union all
select 16640,'1967-W68','1967-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16641,'1967-W69','1967-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16642,'1967-W70','1967-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16643,'1967-W71','1967-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16644,'1967-W72','1967-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16645,'1967-W73','1967-04-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 16646,'1967-W74','1967-03-31 18:28:00','suborbital rocket','success',759,'MT-135-026','','','','Aeron',1821,'',54 from dual union all
select 16647,'1967-W75','1967-03-31 18:44:00','suborbital rocket','success',88,'','Japan-US','','','Aeron',1821,'',58 from dual union all
select 16648,'1967-W76','1967-04-04 09:10:00','suborbital rocket','success',759,'MT-135-027','','','','Aeron',1821,'',54 from dual union all
select 16649,'1967-W77','1967-04-04 09:27:00','suborbital rocket','success',88,'','Japan-US','','','Aeron',1821,'',63 from dual union all
select 1665,'1953-A252','1953-06-22 00:00:00','atmospheric rocket','unknown',576,'491','Acc Test 2A','','','Test',1895,'',10 from dual union all
select 16650,'1967-W78','1967-04-04 09:55:00','suborbital rocket','success',759,'MT-135-028','','','','Aeron',1821,'',57 from dual union all
select 16651,'1967-W79','1967-04-04 10:02:00','suborbital rocket','success',88,'','Japan-US','','','Aeron',1821,'',63 from dual union all
select 16652,'1967-W80','1967-04-04 11:07:00','suborbital rocket','success',759,'MT-135-029','','','','Aeron',1821,'',54 from dual union all
select 16653,'1967-W81','1967-04-04 11:19:00','suborbital rocket','success',88,'','Japan-US','','','Aeron',1821,'',63 from dual union all
select 16654,'1967-W82','1967-04-04 11:56:00','suborbital rocket','success',759,'MT-135-030','','','','Aeron',1821,'',57 from dual union all
select 16655,'1967-W83','1967-04-04 16:28:00','suborbital rocket','success',759,'MT-135-031','','','','Aeron',1821,'',55 from dual union all
select 16656,'1967-W84','1967-04-04 16:43:00','suborbital rocket','success',88,'','Japan-US','','','Aeron',1821,'',58 from dual union all
select 16657,'1967-W85','1967-04-04 19:02:00','suborbital rocket','success',88,'','Japan-US','','','Aeron',1821,'',60 from dual union all
select 16658,'1967-W86','1967-04-04 22:44:00','suborbital rocket','success',759,'MT-135-032','','','','Aeron',1821,'',56 from dual;
insert into LAUNCH
select 16659,'1967-W87','1967-04-04 22:59:00','suborbital rocket','success',88,'','Japan-US','','','Aeron',1821,'',59 from dual union all
select 1666,'1953-A253','1953-06-22 00:00:00','atmospheric rocket','unknown',576,'492','Acc Test 2B','','','Test',1895,'',10 from dual union all
select 16660,'1967-W88','1967-04-05 00:15:00','suborbital rocket','success',759,'MT-135-033','','','','Aeron',1821,'',56 from dual union all
select 16661,'1967-W89','1967-04-05 00:32:00','suborbital rocket','success',88,'','Japan-US','','','Aeron',1821,'',53 from dual union all
select 16662,'1967-W90','1967-04-05 01:01:00','suborbital rocket','success',759,'MT-135-034','','','','Aeron',1821,'',54 from dual union all
select 16663,'1967-W91','1967-04-05 01:29:00','suborbital rocket','success',88,'','Japan-US','','','Aeron',1821,'',61 from dual union all
select 16664,'1967-W92','1967-04-05 03:33:00','suborbital rocket','success',759,'MT-135-035','','','','Aeron',1821,'',56 from dual union all
select 16665,'1967-W93','1967-04-05 03:58:00','suborbital rocket','success',88,'','Japan-US','','','Aeron',1821,'',60 from dual union all
select 16666,'1967-W94','1967-04-08 09:25:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',75 from dual union all
select 16667,'1967-W95','1967-04-10 10:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',74 from dual union all
select 16668,'1967-W96','1967-04-12 12:56:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',73 from dual union all
select 16669,'1967-W97','1967-04-12 14:46:00','suborbital rocket','success',631,'EXAMETNET C19','Chaff','','','Aeron',310,'',70 from dual union all
select 1667,'1953-A254','1953-06-23 00:00:00','atmospheric rocket','unknown',576,'496','','','','Test',1895,'',10 from dual union all
select 16670,'1967-W98','1967-04-15 11:00:00','suborbital rocket','success',725,'','','','','Meteo',636,'',76 from dual union all
select 16671,'1967-W99','1967-04-15 12:04:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',74 from dual union all
select 16672,'1967-Y01','1967-01-02 09:59:59','suborbital rocket','success',725,'','','','','Meteo',1500,'SHIP-SOV',83 from dual union all
select 16673,'1967-Y02','1967-01-04 09:06:00','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 16674,'1967-Y03','1967-01-04 09:59:59','suborbital rocket','success',725,'','','','','Meteo',1554,'SHIP-SOV',83 from dual union all
select 16675,'1967-Y04','1967-01-06 08:03:00','suborbital rocket','success',725,'','','','','Meteo',1539,'SHIP-SOV',180 from dual union all
select 16676,'1967-Y05','1967-01-06 09:59:59','suborbital rocket','success',725,'','','','','Meteo',1553,'SHIP-SOV',85 from dual union all
select 16677,'1967-Y06','1967-01-08 10:02:59','suborbital rocket','success',725,'','','','','Meteo',1536,'SHIP-SOV',80 from dual union all
select 16678,'1967-Y07','1967-01-10 10:09:59','suborbital rocket','success',725,'','','','','Meteo',1555,'SHIP-SOV',83 from dual union all
select 16679,'1967-Y08','1967-01-14 19:11:59','suborbital rocket','success',725,'','','','','Meteo',1552,'SHIP-SOV',84 from dual union all
select 1668,'1953-A255','1953-06-23 00:00:00','atmospheric rocket','unknown',576,'497','','','','Test',1895,'',10 from dual union all
select 16680,'1967-Y09','1967-01-16 09:59:59','suborbital rocket','success',725,'','','','','Meteo',1512,'SHIP-SOV',80 from dual union all
select 16681,'1967-Y10','1967-01-17 09:59:59','suborbital rocket','success',725,'','','','','Meteo',1513,'SHIP-SOV',83 from dual union all
select 16682,'1967-Y11','1967-01-18 09:22:59','suborbital rocket','success',725,'','','','','Meteo',1532,'SHIP-SOV',80 from dual union all
select 16683,'1967-Y12','1967-01-18 09:59:59','suborbital rocket','success',725,'','','','','Meteo',1514,'SHIP-SOV',83 from dual union all
select 16684,'1967-Y13','1967-01-19 11:34:00','suborbital rocket','success',725,'','','','','Meteo',1531,'SHIP-SOV',80 from dual union all
select 16685,'1967-Y14','1967-01-20 11:04:59','suborbital rocket','success',725,'','','','','Meteo',1515,'SHIP-SOV',83 from dual union all
select 16686,'1967-Y15','1967-01-24 10:48:59','suborbital rocket','success',725,'','','','','Meteo',1529,'SHIP-SOV',80 from dual union all
select 16687,'1967-Y16','1967-01-25 04:00:59','suborbital rocket','success',725,'','','','','Meteo',636,'',85 from dual union all
select 16688,'1967-Y17','1967-01-26 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1524,'SHIP-SOV',80 from dual union all
select 16689,'1967-Y18','1967-01-26 18:45:00','suborbital rocket','unknown',257,'','','','','Aeron',1895,'',104 from dual union all
select 1669,'1953-A256','1953-06-23 00:00:00','atmospheric rocket','success',836,'1249B-1122','Nike 1249 R&D 219R','','','Test',1895,'',10 from dual union all
select 16690,'1967-Y19','1967-02-01 11:33:00','suborbital rocket','success',725,'','','','','Meteo',1527,'SHIP-SOV',80 from dual union all
select 16691,'1967-Y20','1967-02-04 10:55:59','suborbital rocket','success',725,'','','','','Meteo',1523,'SHIP-SOV',80 from dual union all
select 16692,'1967-Y21','1967-02-06 10:26:59','suborbital rocket','success',725,'','','','','Meteo',1498,'SHIP-SOV',80 from dual union all
select 16693,'1967-Y22','1967-02-15 03:10:00','suborbital rocket','success',725,'','','','','Meteo',636,'',82 from dual union all
select 16694,'1967-Y23','1967-02-16 12:12:59','suborbital rocket','success',725,'','','','','Meteo',1510,'SHIP-SOV',80 from dual union all
select 16695,'1967-Y24','1967-02-20 09:53:00','suborbital rocket','success',725,'','','','','Meteo',1508,'SHIP-SOV',80 from dual union all
select 16696,'1967-Y25','1967-03-22 10:19:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',82 from dual union all
select 16697,'1967-Y26','1967-03-29 10:08:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',82 from dual union all
select 16698,'1967-Y27','1967-04-01 00:00:00','suborbital rocket','success',257,'','','','','Aeron',1821,'',90 from dual union all
select 16699,'1967-Y28','1967-04-01 00:00:00','suborbital rocket','success',257,'','','','','Aeron',1821,'',90 from dual union all
select 167,'1944-S151','1944-09-26 13:27:00','military missile','success',8,'18806','Ma222','','','Training',661,'',90 from dual union all
select 1670,'1953-A257','1953-06-23 00:00:00','atmospheric rocket','success',836,'1249B-1213','Nike 1249 R&D 220R','','','Test',1895,'',10 from dual union all
select 16700,'1967-Y29','1967-04-01 00:00:00','suborbital rocket','success',257,'','','','','Aeron',1821,'',90 from dual union all
select 16701,'1967-Y30','1967-04-01 00:00:00','suborbital rocket','success',257,'','','','','Aeron',1821,'',90 from dual union all
select 16702,'1967-Y31','1967-04-01 00:00:00','suborbital rocket','success',257,'','','','','Aeron',1821,'',90 from dual union all
select 16703,'1967-Y32','1967-04-01 00:00:00','suborbital rocket','success',257,'','','','','Aeron',1821,'',90 from dual union all
select 16704,'1967-Y33','1967-04-01 00:00:00','suborbital rocket','success',257,'','','','','Aeron',1821,'',90 from dual union all
select 16705,'1967-Y34','1967-04-01 00:00:00','suborbital rocket','success',257,'','','','','Aeron',1821,'',90 from dual union all
select 16706,'1967-Y35','1967-04-01 00:00:00','suborbital rocket','success',257,'','','','','Aeron',1821,'',90 from dual union all
select 16707,'1967-Y36','1967-04-01 00:00:00','suborbital rocket','success',257,'','','','','Aeron',1821,'',90 from dual union all
select 16708,'1967-Y37','1967-05-07 10:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 16709,'1967-Y38','1967-05-10 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 1671,'1953-A258','1953-06-23 00:00:00','atmospheric rocket','success',836,'1148E','','','','Test',1895,'',10 from dual union all
select 16710,'1967-Y39','1967-05-13 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 16711,'1967-Y40','1967-06-21 10:07:00','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 16712,'1967-Y41','1967-07-12 00:11:00','suborbital rocket','unknown',1188,'Su2H','D-region','','','Ionos',1447,'',81 from dual union all
select 16713,'1967-Y42','1967-07-19 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 16714,'1967-Y43','1967-08-19 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 16715,'1967-Y44','1967-08-21 10:01:59','suborbital rocket','success',725,'','','','','Meteo',636,'',86 from dual union all
select 16716,'1967-Y45','1967-08-25 10:35:59','suborbital rocket','success',725,'','','','','Meteo',636,'',85 from dual union all
select 16717,'1967-Y46','1967-09-04 10:00:59','suborbital rocket','success',725,'','','','','Meteo',636,'',85 from dual union all
select 16718,'1967-Y47','1967-09-06 09:17:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',84 from dual union all
select 16719,'1967-Y48','1967-09-20 09:09:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 1672,'1953-A259','1953-06-25 00:00:00','atmospheric rocket','success',1444,'','F23 Ramjet','','','Test',1821,'',41 from dual union all
select 16720,'1967-Y49','1967-10-16 09:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',84 from dual union all
select 16721,'1967-Y50','1967-10-18 09:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 16722,'1967-Y51','1967-11-15 10:03:59','suborbital rocket','success',725,'','','','','Meteo',636,'',84 from dual union all
select 16723,'1967-Y52','1967-11-17 09:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 16724,'1967-Y53','1967-11-30 05:01:00','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 16725,'1967-Y54','1967-12-08 09:59:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 16726,'1967-Y55','1967-12-16 09:09:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 16727,'1967-Y56','1967-12-20 09:00:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 16728,'1968-001','1968-01-07 06:30:00','deep space','success',179,'AC-15','Surveyor 7','Surveyor SC-7','','Sat',268,'',null from dual union all
select 16729,'1968-002','1968-01-11 16:16:00','orbital','success',1382,'454/D56','Explorer 36','Geos B','','Sat',1723,'',null from dual union all
select 1673,'1953-A26','1953-01-22 00:00:00','atmospheric rocket','success',836,'1249A-171','Nike 1249 R&D 165R','','','Test',1895,'',10 from dual union all
select 16730,'1968-003','1968-01-16 12:00:01','orbital','success',1508,'Ya15002- 05','Kosmos-199','Zenit-2 No. 59','','Sat',473,'',null from dual union all
select 16731,'1968-004','1968-01-17 10:12:00','orbital','success',1394,'498/FTV 2733','OPS 1965','MULTIGROUP 3','','Sat TAG150',1724,'',null from dual union all
select 16732,'1968-005','1968-01-18 19:04:00','orbital','success',1424,'3B-13','KH-8','GAMBIT-3 Mission 4311','','Sat',1778,'',null from dual union all
select 16733,'1968-006','1968-01-19 22:00:00','orbital','success',680,'N149    -21LM','Kosmos-200','Tselina-O','','Sat',457,'',null from dual union all
select 16734,'1968-007','1968-01-22 22:48:08','orbital','success',1121,'SA-204','Apollo 5','Apollo 5','','Sat',271,'',null from dual union all
select 16735,'1968-008','1968-01-24 22:26:00','orbital','success',1395,'516/FTV 1640','CORONA 123/1045','CORONA J-45','','Sat TAG151',1721,'',null from dual union all
select 16736,'1968-009','1968-02-06 08:00:00','orbital','success',1508,'N 15002- 14','Kosmos-201','Zenit-4 No. 40','','Sat',562,'',null from dual union all
select 16737,'1968-010','1968-02-20 10:03:11','orbital','success',678,'','Kosmos-202','DS-U2-V No. 4','','Sat',627,'',null from dual union all
select 16738,'1968-011','1968-02-20 16:00:00','orbital','success',680,'N149    -22LM','Kosmos-203','Sfera','','Sat',457,'',null from dual union all
select 16739,'1968-012','1968-03-02 03:55:01','orbital','success',1125,'S162C','NNS O-18  (N-13)','NNS O-18','','Sat',1779,'',null from dual union all
select 1674,'1953-A260','1953-06-25 00:00:00','atmospheric rocket','unknown',576,'495','','','','Test',1895,'',10 from dual union all
select 16740,'1968-013','1968-03-02 18:29:23','deep space','success',953,'231-01','Zond-4','L-1 No. 6L','','Sat',587,'',null from dual union all
select 16741,'1968-014','1968-03-04 13:06:01','orbital','success',178,'5602A (AA26)','OGO 5','OGO E','','Sat',228,'',null from dual union all
select 16742,'1968-015','1968-03-05 11:20:00','orbital','success',678,'U231-06','Kosmos-204','DS-P1-I No. 3','','Sat',460,'',null from dual union all
select 16743,'1968-016','1968-03-05 12:30:01','orbital','success',1508,'Ya15002- 02','Kosmos-205','Zenit-2 No. 56','','Sat',473,'',null from dual union all
select 16744,'1968-017','1968-03-05 18:28:00','orbital','success',1128,'S160C','Explorer 37','Solar Explorer B','','Sat',1814,'',null from dual union all
select 16745,'1968-018','1968-03-13 19:51:00','orbital','success',1424,'3B-14','KH-8','GAMBIT-3 Mission 4312','','Sat',1778,'',null from dual union all
select 16746,'1968-019','1968-03-14 09:34:01','orbital','success',1510,'G 15000- 03','Kosmos-206','Meteor 11F614 No. 9','','Sat',473,'',null from dual union all
select 16747,'1968-020','1968-03-14 22:00:14','orbital','success',1395,'518/FTV 1638','CORONA 124/1046','CORONA J-48','','Sat TAG152',1721,'',null from dual union all
select 16748,'1968-021','1968-03-16 12:30:00','orbital','success',1508,'Ya15002- 12','Kosmos-207','Zenit-4 No. 38','','Sat',473,'',null from dual;
insert into LAUNCH
select 16749,'1968-022','1968-03-21 09:50:00','orbital','success',1508,'R 15000- 10','Kosmos-208','Zenit-2M','','Sat',480,'',null from dual union all
select 1675,'1953-A261','1953-06-25 00:00:00','atmospheric rocket','unknown',576,'499','','','','Test',1895,'',10 from dual union all
select 16750,'1968-023','1968-03-22 09:30:34','orbital','success',1447,'N 22500-08L','Kosmos-209','US-A reactor','','Sat',590,'',null from dual union all
select 16751,'1968-024','1968-04-03 11:00:00','orbital','success',1508,'Ya15002- 06','Kosmos-210','Zenit-2 No. 60','','Sat',473,'',null from dual union all
select 16752,'1968-025','1968-04-04 12:00:01','orbital','success',1123,'SA-502','Apollo 6','Apollo CM-020','','Sat',921,'LUT2',null from dual union all
select 16753,'1968-026','1968-04-06 09:59:42','orbital','success',162,'107F','OV1-13/14','OV1-13','','Sat',1681,'',null from dual union all
select 16754,'1968-027','1968-04-07 10:09:32','deep space','success',809,'Ya15000- 58  L58','Luna-14','E-6LS No. 113','','Sat',480,'',null from dual union all
select 16755,'1968-028','1968-04-09 11:26:25','orbital','success',678,'Ya231-02','Kosmos-211','DS-P1-Yu No. 13','','Sat',460,'',null from dual union all
select 16756,'1968-029','1968-04-14 10:00:00','orbital','success',1223,'Ya15000- 09','Kosmos-212','Soyuz No. 8','','Sat',562,'',null from dual union all
select 16757,'1968-030','1968-04-15 09:34:18','orbital','success',1223,'U 15000- 06','Kosmos-213','Soyuz No. 7','','Sat',480,'',null from dual union all
select 16758,'1968-031','1968-04-17 17:00:00','orbital','success',1424,'3B-15','KH-8','GAMBIT-3 Mission 4313','','Sat',1778,'',null from dual union all
select 16759,'1968-032','1968-04-18 10:30:01','orbital','success',1508,'V 15001- 12','Kosmos-214','Zenit-4 No. 45','','Sat',473,'',null from dual union all
select 1676,'1953-A262','1953-06-25 00:00:00','atmospheric rocket','success',836,'1249B-1123','490B Booster/221R','','','Test',1895,'',10 from dual union all
select 16760,'1968-033','1968-04-18 22:29:53','orbital','success',678,'','Kosmos-215','DS-U1-A No. 1','','Sat',627,'',null from dual union all
select 16761,'1968-034','1968-04-20 10:30:00','orbital','success',1508,'V 15001- 01','Kosmos-216','Zenit-2 No. 62','','Sat',562,'',null from dual union all
select 16762,'1968-035','1968-04-21 04:20:01','orbital','success',809,'Ya15000- 82  L82','Molniya-1','Molniya-1 11F67 No. 10','','Sat',480,'',null from dual union all
select 16763,'1968-036','1968-04-24 16:00:00','orbital','success',1447,'','Kosmos-217','I2M','','Sat',591,'',null from dual union all
select 16764,'1968-037','1968-04-25 00:45:10','orbital','success',1007,'Ya22500-17L(5MO)','Kosmos-218','OGCh No. 17LU','','Sat',519,'',null from dual union all
select 16765,'1968-038','1968-04-26 04:42:56','orbital','success',678,'','Kosmos-219','DS-U2-D No. 2','','Sat',627,'',null from dual union all
select 16766,'1968-039','1968-05-01 21:31:00','orbital','success',1395,'511/FTV 1643','CORONA 125/1103','CORONA CR-3','','Sat TAG153',1776,'',null from dual union all
select 16767,'1968-040','1968-05-07 13:58:00','orbital','success',680,'N149    -17LM','Kosmos-220','Tsiklon','','Sat',457,'',null from dual union all
select 16768,'1968-041','1968-05-17 02:06:00','orbital','success',1128,'S161C','ESRO 2B','ESRO 2B','','Sat',1779,'',null from dual union all
select 16769,'1968-042','1968-05-23 04:38:12','orbital','success',1364,'277/BII-6','DAPP 5420','DAPP 5420','','Sat',1720,'',null from dual union all
select 1677,'1953-A263','1953-06-25 00:00:00','atmospheric rocket','success',863,'1249B-1217','Nike 222R','','','Test',1895,'',10 from dual union all
select 16770,'1968-043','1968-05-24 07:04:50','orbital','success',678,'','Kosmos-221','DS-P1-Yu No. 14','','Sat',627,'',null from dual union all
select 16771,'1968-044','1968-05-30 20:29:49','orbital','success',678,'Ya231-01','Kosmos-222','DS-P1-Yu No. 12','','Sat',460,'',null from dual union all
select 16772,'1968-045','1968-06-01 10:50:00','orbital','success',1508,'V 15001- 02','Kosmos-223','Zenit-2 No. 63','','Sat',473,'',null from dual union all
select 16773,'1968-046','1968-06-04 06:45:02','orbital','success',1508,'V 15001- 13','Kosmos-224','Zenit-4 No. 46','','Sat',562,'',null from dual union all
select 16774,'1968-047','1968-06-05 17:33:00','orbital','success',1424,'3B-16','KH-8','GAMBIT-3 Mission 4314','','Sat',1778,'',null from dual union all
select 16775,'1968-048','1968-06-11 21:29:54','orbital','success',678,'','Kosmos-225','DS-U1-Ya No. 2','','Sat',627,'',null from dual union all
select 16776,'1968-049','1968-06-12 13:14:59','orbital','success',1510,'N 15000- 58','Kosmos-226','Meteor No. 10','','Sat',473,'',null from dual union all
select 16777,'1968-050','1968-06-13 14:03:50','orbital','success',1425,'3C-16','IDCSP','IDCSP 20','','Sat AMF',275,'',null from dual union all
select 16778,'1968-051','1968-06-18 06:15:06','orbital','success',1508,'V 15001 -11','Kosmos-227','Zenit-4 No. 44','','Sat',562,'',null from dual union all
select 16779,'1968-052','1968-06-20 21:46:05','orbital','success',1395,'517/FTV 1645','CORONA 126/1047','CORONA J-47','','Sat TAG155',1721,'',null from dual union all
select 1678,'1953-A264','1953-06-26 00:00:00','atmospheric rocket','unknown',576,'490','','','','Test',1895,'',10 from dual union all
select 16780,'1968-053','1968-06-21 12:00:03','orbital','success',1508,'Ya15001- 12','Kosmos-228','Zenit-2M','','Sat',480,'',null from dual union all
select 16781,'1968-054','1968-06-26 11:00:01','orbital','success',1508,'Ya15002- 15','Kosmos-229','Zenit-4 No. 41','','Sat',473,'',null from dual union all
select 16782,'1968-055','1968-07-04 17:26:50','orbital','success',1384,'476/D57','Explorer 38','RAE A','','Sat',1723,'',null from dual union all
select 16783,'1968-056','1968-07-05 06:59:50','orbital','success',678,'','Kosmos-230','DS-U3-S No. 2','','Sat',627,'',null from dual union all
select 16784,'1968-057','1968-07-05 15:25:59','orbital','success',809,'Ya15000- 84  L85','Molniya-1','Molniya-1 11F67 No. 13','','Sat',480,'',null from dual union all
select 16785,'1968-058','1968-07-10 19:49:58','orbital','success',1508,'V 15001- 03','Kosmos-231','Zenit-2 No. 64','','Sat',562,'',null from dual union all
select 16786,'1968-059','1968-07-11 19:30:00','orbital','success',162,'75F','OV1-15/16','OV1-15S','','Sat',1681,'',null from dual union all
select 16787,'1968-060','1968-07-16 13:10:00','orbital','success',1508,'V 15001- 14','Kosmos-232','Zenit-4 No. 47','','Sat',473,'',null from dual union all
select 16788,'1968-061','1968-07-18 19:59:50','orbital','success',678,'','Kosmos-233','DS-P1-Yu No. 15','','Sat',460,'',null from dual union all
select 16789,'1968-062','1968-07-30 07:00:00','orbital','success',1508,'Ya15002- 16','Kosmos-234','Zenit-4 No. 42','','Sat',562,'',null from dual union all
select 1679,'1953-A265','1953-06-26 00:00:00','atmospheric rocket','unknown',576,'498','','','','Test',1895,'',10 from dual union all
select 16790,'1968-063','1968-08-06 11:08:00','orbital','success',178,'5501A','AFP-827','CANYON 1','','Sat',228,'',null from dual union all
select 16791,'1968-064','1968-08-06 16:30:00','orbital','success',1424,'3B-17','KH-8','GAMBIT-3 Mission 4315','','Sat',1778,'',null from dual union all
select 16792,'1968-065','1968-08-07 21:36:55','orbital','success',1395,'522/FTV 1644','CORONA 127/1104','CORONA CR-4','','Sat TAG156',1776,'',null from dual union all
select 16793,'1968-066','1968-08-08 20:12:00','orbital','success',1128,'S165C','Explorer 39/40','AD-C','','Sat',1779,'',null from dual union all
select 16794,'1968-067','1968-08-09 07:00:00','orbital','success',1508,'Ya15002- 07','Kosmos-235','Zenit-2 No. 61','','Sat',562,'',null from dual union all
select 16795,'1968-068','1968-08-10 22:33:02','orbital','success',179,'AC-17','ATS 4','ATS D','','Sat',268,'',null from dual union all
select 16796,'1968-069','1968-08-16 11:24:33','orbital','success',1388,'528/D58','ESSA 7','TOS E','','Sat',1723,'',null from dual union all
select 16797,'1968-070','1968-08-27 11:29:00','orbital','success',679,'4000-000-12L','Kosmos-236','Strela-2 No. 4','','Sat',564,'',null from dual union all
select 16798,'1968-071','1968-08-27 12:29:59','orbital','success',1508,'Ya15002- 17','Kosmos-237','Zenit-4 No. 43','','Sat',473,'',null from dual union all
select 16799,'1968-072','1968-08-28 10:00:00','orbital','success',1223,'V 15000- 13','Kosmos-238','Soyuz No. 9','','Sat',562,'',null from dual union all
select 168,'1944-S152','1944-09-26 19:32:00','military missile','success',8,'18804','Ma220','','','Training',661,'',90 from dual union all
select 1680,'1953-A266','1953-06-28 00:00:00','atmospheric rocket','unknown',576,'407','','','','Test',1895,'',10 from dual union all
select 16800,'1968-073','1968-09-05 07:00:01','orbital','success',1508,'V 15000- 15','Kosmos-239','Zenit-4 No. 48','','Sat',562,'',null from dual union all
select 16801,'1968-074','1968-09-10 18:30:00','orbital','success',1424,'3B-18','KH-8','GAMBIT-3 Mission 4316','','Sat',1778,'',null from dual union all
select 16802,'1968-075','1968-09-14 06:50:00','orbital','success',1508,'V 15001- 05','Kosmos-240','Zenit-2 No. 66','','Sat',562,'',null from dual union all
select 16803,'1968-076','1968-09-14 21:42:11','deep space','success',953,'234-01','Zond-5','L-1 No. 9L','','Sat',587,'',null from dual union all
select 16804,'1968-077','1968-09-16 12:30:01','orbital','success',1508,'V 15001- 16','Kosmos-241','Zenit-4 No. 49','','Sat',473,'',null from dual union all
select 16805,'1968-078','1968-09-18 21:32:00','orbital','success',1395,'524/FTV 1647','CORONA 128/1048','CORONA J-49','','Sat TAG157',1721,'',null from dual union all
select 16806,'1968-079','1968-09-20 14:39:59','orbital','success',678,'Ya231-04','Kosmos-242','DS-P1-I No. 4','','Sat',460,'',null from dual union all
select 16807,'1968-080','1968-09-23 07:39:59','orbital','success',1508,'R 15000- 08','Kosmos-243','Zenit-2M','','Sat',480,'',null from dual union all
select 16808,'1968-081','1968-09-26 07:37:01','orbital','success',1425,'3C-5','OV2-5/LES 6','OV2-5','','Sat AMF',275,'',null from dual union all
select 16809,'1968-082','1968-10-02 13:35:01','orbital','success',1007,'V 22500-10T','Kosmos-244','OGCh No. V22500-10 (20?)','','Sat',518,'',null from dual union all
select 1681,'1953-A267','1953-06-28 00:00:00','atmospheric rocket','unknown',576,'408','','','','Test',1895,'',10 from dual union all
select 16810,'1968-083','1968-10-03 12:58:59','orbital','success',678,'V295-02','Kosmos-245','DS-P1-Yu No. 16','','Sat',460,'',null from dual union all
select 16811,'1968-084','1968-10-03 20:49:39','orbital','success',1128,'S167C','ESRO 1A','ESRO 1A','','Sat',1779,'',null from dual union all
select 16812,'1968-085','1968-10-05 00:32:00','orbital','success',809,'V 15000- 86  L86','Molniya-1','Molniya-1 11F67 No. 14','','Sat',480,'',null from dual union all
select 16813,'1968-086','1968-10-05 11:16:00','orbital','success',1395,'521/FTV 2734','OPS 0964','STRAWMAN 1','','Sat TAG158',1722,'',null from dual union all
select 16814,'1968-087','1968-10-07 12:05:46','orbital','success',1508,'V 15002- 11','Kosmos-246','Zenit-4 No. 51','','Sat',473,'',null from dual union all
select 16815,'1968-088','1968-10-11 12:05:01','orbital','success',1508,'V 15001- 04','Kosmos-247','Zenit-2 No. 65','','Sat',473,'',null from dual union all
select 16816,'1968-089','1968-10-11 15:02:45','orbital','success',1121,'SA-205','Apollo 7','Apollo CM-101','','Sat',267,'',null from dual union all
select 16817,'1968-090','1968-10-19 04:20:00','orbital','success',1447,'N 22500-06L','Kosmos-248','I2M','','Sat',590,'',null from dual union all
select 16818,'1968-091','1968-10-20 04:02:00','orbital','success',1447,'N 22500-05L','Kosmos-249','I2P','','Sat',591,'',null from dual union all
select 16819,'1968-092','1968-10-23 04:34:03','orbital','success',1364,'173/BII-8','DAPP 6422','DAPP 6422','','Sat',1720,'',null from dual union all
select 1682,'1953-A268','1953-06-28 00:00:00','atmospheric rocket','unknown',576,'406','','','','Test',1895,'',10 from dual union all
select 16820,'1968-093','1968-10-25 09:00:00','orbital','success',1223,'Ya15000-008','Soyuz-2','Soyuz No. 11','','Sat',480,'',null from dual union all
select 16821,'1968-094','1968-10-26 08:34:18','orbital','success',1223,'Ya15000-010','Soyuz-3','Soyuz No. 10','','Sat',562,'',null from dual union all
select 16822,'1968-095','1968-10-30 22:00:00','orbital','success',680,'V149    -28LM','Kosmos-250','Tselina-O','','Sat',457,'',null from dual union all
select 16823,'1968-096','1968-10-31 09:14:55','orbital','success',1508,'N 15001-010','Kosmos-251','Zenit-4M','','Sat',480,'',null from dual union all
select 16824,'1968-097','1968-11-01 00:27:15','orbital','success',1447,'N 22500-09L','Kosmos-252','I2P','','Sat',591,'',null from dual union all
select 16825,'1968-098','1968-11-03 21:30:00','orbital','success',1395,'515/FTV 1646','CORONA 129/1105','CORONA CR-5','','Sat TAG159',1776,'',null from dual union all
select 16826,'1968-099','1968-11-06 19:10:00','orbital','success',1424,'3B-19','KH-8','GAMBIT-3 Mission 4317','','Sat',1778,'',null from dual union all
select 16827,'1968-100','1968-11-08 09:46:29','deep space','success',1382,'479/D60','Pioneer 9','Pioneer D','','Sat',234,'',null from dual union all
select 16828,'1968-101','1968-11-10 19:11:31','deep space','success',953,'235-01','Zond-6','L-1 No. 12L','','Sat',587,'',null from dual union all
select 16829,'1968-102','1968-11-13 12:00:01','orbital','success',1508,'V 15001-006','Kosmos-253','Zenit-2 No. 67','','Sat',473,'',null from dual union all
select 1683,'1953-A269','1953-06-28 00:00:00','atmospheric rocket','unknown',576,'405','','','','Test',1895,'',10 from dual union all
select 16830,'1968-103','1968-11-16 11:40:00','orbital','success',949,'236-01','Proton-4','N-6 No. 1','','Sat',588,'',null from dual union all
select 16831,'1968-104','1968-11-21 12:10:01','orbital','success',1508,'V 15001-017','Kosmos-254','Zenit-4 No. 50','','Sat',473,'',null from dual union all
select 16832,'1968-105','1968-11-29 12:40:34','orbital','success',1508,'V 15001-007','Kosmos-255','Zenit-2 No. 68','','Sat',473,'',null from dual union all
select 16833,'1968-106','1968-11-30 12:00:01','orbital','success',680,'V149    -24LM','Kosmos-256','Sfera','','Sat',457,'',null from dual union all
select 16834,'1968-107','1968-12-03 14:52:21','orbital','success',678,'V295-03','Kosmos-257','DS-P1-Yu No. 17','','Sat',460,'',null from dual union all
select 16835,'1968-108','1968-12-04 19:23:00','orbital','success',1424,'3B-20','KH-8','GAMBIT-3 Mission 4318','','Sat',1778,'',null from dual union all
select 16836,'1968-109','1968-12-05 18:55:00','orbital','success',1382,'481/D61','HEOS 1','HEOS A1','','Sat',234,'',null from dual union all
select 16837,'1968-110','1968-12-07 08:40:09','orbital','success',179,'AC-16','OAO 2','OAO A2','','Sat',269,'',null from dual union all
select 16838,'1968-111','1968-12-10 08:25:01','orbital','success',1508,'V 15002-001','Kosmos-258','Zenit-2 No. 69','','Sat',562,'',null from dual;
insert into LAUNCH
select 16839,'1968-112','1968-12-12 22:22:00','orbital','success',1395,'527/FTV 1648','CORONA 130/1049','CORONA J-50','','Sat TAG160',1776,'',null from dual union all
select 1684,'1953-A27','1953-01-22 00:00:00','atmospheric rocket','success',836,'1249A-174','Nike 1249 R&D 164R','','','Test',1895,'',10 from dual union all
select 16840,'1968-113','1968-12-14 05:09:54','orbital','success',678,'','Kosmos-259','DS-U2-I No. 3','','Sat',627,'',null from dual union all
select 16841,'1968-114','1968-12-15 17:21:04','orbital','success',1388,'532/D62','ESSA 8','TOS F','','Sat',1723,'',null from dual union all
select 16842,'1968-115','1968-12-16 09:15:03','orbital','success',809,'Ya15000-039  L87','Kosmos-260','Molniya-1Yu 11F67 No. 15','','Sat',480,'',null from dual union all
select 16843,'1968-116','1968-12-19 00:32:00','orbital','success',1386,'536/D63','Intelsat III F-2','INTELSAT III F-2','','Sat',233,'',null from dual union all
select 16844,'1968-117','1968-12-19 23:55:00','orbital','success',678,'N231-13','Kosmos-261','DS-U2-GK No. 1','','Sat',460,'',null from dual union all
select 16845,'1968-118','1968-12-21 12:51:00','deep space','success',1123,'SA-503','Apollo 8','Apollo CM-103','','Sat',921,'LUT1',null from dual union all
select 16846,'1968-119','1968-12-26 09:45:02','orbital','success',678,'','Kosmos-262','DS-U2-GF No. 1','','Sat',627,'',null from dual union all
select 16847,'1968-A01','1968-01-01 00:00:00','suborbital rocket','failure',27,'','','','','Aeron?',1833,'',0 from dual union all
select 16848,'1968-A02','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16849,'1968-A03','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 1685,'1953-A270','1953-06-28 00:00:00','atmospheric rocket','unknown',576,'404','','','','Test',1895,'',10 from dual union all
select 16850,'1968-A04','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16851,'1968-A05','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16852,'1968-A06','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16853,'1968-A07','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16854,'1968-A08','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16855,'1968-A09','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16856,'1968-A10','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16857,'1968-A100','1968-09-20 19:17:00','suborbital rocket','failure',258,'NASA 12.16GT','Launcher test','','','Test',1821,'',43 from dual union all
select 16858,'1968-A101','1968-10-01 00:00:00','military missile','success',545,'DASA 6.1','','','','Aeron?',1895,'',40 from dual union all
select 16859,'1968-A102','1968-10-01 00:00:00','military missile','success',545,'DASA 6.1','','','','Aeron?',1895,'',40 from dual union all
select 1686,'1953-A271','1953-06-28 00:00:00','atmospheric rocket','unknown',576,'403','','','','Test',1895,'',10 from dual union all
select 16860,'1968-A103','1968-10-25 00:00:00','atmospheric rocket','success',1066,'Ob2','Oberon 2','','','Test',1836,'',20 from dual union all
select 16861,'1968-A105','1968-10-07 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 16862,'1968-A106','1968-10-08 00:00:00','military missile','success',1252,'FLA-24','','','','Test',1888,'',6 from dual union all
select 16863,'1968-A107','1968-10-28 02:10:00','military missile','failure',999,'V 22504-15T','8K67 UBP-24','GCh 8F675','','OT',515,'',0 from dual union all
select 16864,'1968-A108','1968-10-28 00:00:00','military missile','success',1252,'FLA-25','','','','Test',1888,'',6 from dual union all
select 16865,'1968-A109','1968-11-09 08:59:00','suborbital rocket','failure',848,'CRL AF07.292','B field/particles','','','Plasma',427,'',16 from dual union all
select 16866,'1968-A11','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16867,'1968-A110','1968-11-13 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 16868,'1968-A111','1968-11-16 15:10:00','military missile','failure',168,'56F','','ABRES RMP-B-10/KX-110','','Reentry Test',1682,'',100 from dual union all
select 16869,'1968-A112','1968-11-21 00:00:00','military missile','failure',1251,'800006','KT-5','','','Test',868,'',2 from dual union all
select 1687,'1953-A272','1953-06-29 00:00:00','atmospheric rocket','unknown',576,'410','','','','Test',1895,'',10 from dual union all
select 16870,'1968-A113','1968-11-27 01:42:00','suborbital rocket','failure',837,'Ferdinand 17','Ferdinand 17','','','Aeron/Ionos/Plasma',29,'',10 from dual union all
select 16871,'1968-A114','1968-12-02 00:00:00','military missile','failure',1252,'FLA-26','','','','Test',1888,'',0 from dual union all
select 16872,'1968-A115','1968-12-12 00:00:00','suborbital spaceplane','failure',1520,'1- A-142','','','','Test/Tech/Aeron',379,'NB-52',14 from dual union all
select 16873,'1968-A116','1968-12-25 05:00:00','suborbital rocket','success',725,'','','','','Meteo',636,'',40 from dual union all
select 16874,'1968-A117','1968-09-26 00:00:00','suborbital rocket','failure',466,'E002','FU-181','','','Test',340,'',0 from dual union all
select 16875,'1968-A118','1968-06-12 00:00:00','atmospheric rocket','success',852,'Sandia 158-13','Parachute test','','','Test',1490,'',4 from dual union all
select 16876,'1968-A119','1968-06-27 16:37:00','suborbital rocket','success',557,'Pacemaker 7','Phenolic ny. ablator','','','Reentry test',1821,'',30 from dual union all
select 16877,'1968-A12','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16878,'1968-A120','1968-11-20 00:00:00','suborbital rocket','failure',1506,'SDC VD-22','ROBIN','','','Aeron',276,'',40 from dual union all
select 16879,'1968-A121','1968-05-21 00:00:00','suborbital rocket','success',1501,'','Kangaroo','','','Test',1057,'',40 from dual union all
select 1688,'1953-A273','1953-06-29 00:00:00','atmospheric rocket','unknown',576,'411','','','','Test',1895,'',10 from dual union all
select 16880,'1968-A122','1968-07-01 00:00:00','suborbital rocket','success',1501,'','Kangaroo','','','Test',1057,'',40 from dual union all
select 16881,'1968-A123','1968-07-01 00:00:00','suborbital rocket','success',1501,'','Kangaroo','','','Test',1057,'',40 from dual union all
select 16882,'1968-A124','1968-07-01 00:00:00','suborbital rocket','success',1501,'','Kangaroo','','','Test',1057,'',40 from dual union all
select 16883,'1968-A125','1968-07-01 00:00:00','suborbital rocket','success',272,'','Kangaroo','','','Test',1057,'',40 from dual union all
select 16884,'1968-A126','1968-10-01 00:00:00','suborbital rocket','success',272,'','Kangaroo','','','Test',1057,'',40 from dual union all
select 16885,'1968-A127','1968-10-01 00:00:00','suborbital rocket','success',272,'','Kangaroo','','','Test',1057,'',50 from dual union all
select 16886,'1968-A128','1968-11-01 00:00:00','suborbital rocket','success',272,'','Kangaroo','','','Test',1057,'',40 from dual union all
select 16887,'1968-A129','1968-11-01 00:00:00','suborbital rocket','success',272,'','Kangaroo','','','Test',1057,'',40 from dual union all
select 16888,'1968-A13','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16889,'1968-A130','1968-11-01 00:00:00','suborbital rocket','success',272,'','Kangaroo','','','Test',1057,'',40 from dual union all
select 1689,'1953-A274','1953-06-29 00:00:00','atmospheric rocket','unknown',576,'412','','','','Test',1895,'',10 from dual union all
select 16890,'1968-A131','1968-11-01 00:00:00','suborbital rocket','success',272,'','Kangaroo','','','Test',1057,'',40 from dual union all
select 16891,'1968-A132','1968-08-30 00:00:00','military missile','success',702,'Recert flight 1','','','','Test',1895,'',10 from dual union all
select 16892,'1968-A14','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16893,'1968-A15','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16894,'1968-A16','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16895,'1968-A17','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16896,'1968-A18','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16897,'1968-A19','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16898,'1968-A20','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16899,'1968-A21','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 169,'1944-S153','1944-09-26 20:13:00','military missile','success',1470,'18770','Ma186','','','Test',661,'',90 from dual union all
select 1690,'1953-A275','1953-06-29 00:00:00','atmospheric rocket','unknown',576,'409','','','','Test',1895,'',10 from dual union all
select 16900,'1968-A22','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16901,'1968-A23','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16902,'1968-A24','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16903,'1968-A25','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16904,'1968-A26','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16905,'1968-A27','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16906,'1968-A28','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16907,'1968-A29','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16908,'1968-A30','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16909,'1968-A31','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 1691,'1953-A276','1953-06-30 00:00:00','atmospheric rocket','success',836,'1249B-1205','Nike 1249 R&D 223R','','','Test',1895,'',10 from dual union all
select 16910,'1968-A32','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16911,'1968-A33','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16912,'1968-A34','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16913,'1968-A35','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16914,'1968-A36','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16915,'1968-A37','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16916,'1968-A38','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16917,'1968-A39','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16918,'1968-A40','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16919,'1968-A41','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 1692,'1953-A277','1953-06-30 00:00:00','atmospheric rocket','success',836,'1249B-1207','Nike 1249 R&D 224R','','','Test',1895,'',10 from dual union all
select 16920,'1968-A42','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16921,'1968-A43','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16922,'1968-A44','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16923,'1968-A45','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16924,'1968-A46','1968-01-01 00:00:00','suborbital rocket','success',776,'','','','','Aeron',933,'',36 from dual union all
select 16925,'1968-A47','1968-01-01 00:00:00','suborbital rocket','success',1032,'4','','','','Test',1467,'',9 from dual union all
select 16926,'1968-A48','1968-01-01 00:00:00','suborbital rocket','success',1032,'5','','','','Test',1467,'',9 from dual union all
select 16927,'1968-A49','1968-01-01 00:00:00','suborbital rocket','success',1032,'6','','','','Test',1467,'',9 from dual union all
select 16928,'1968-A50','1968-03-28 00:00:00','military missile','failure',1038,'8L','8K99 LKI-4','','','Test',477,'',0 from dual;
insert into LAUNCH
select 16929,'1968-A51','1968-08-16 00:00:00','military missile','failure',1038,'11L','8K99 LKI-7','','','Test',477,'',0 from dual union all
select 1693,'1953-A278','1953-07-01 00:00:00','atmospheric rocket','success',836,'1160E','','','','Test',1895,'',10 from dual union all
select 16930,'1968-A52','1968-09-03 00:00:00','military missile','success',1038,'15L','8K99 LKI-8','','','Test',477,'',1000 from dual union all
select 16931,'1968-A53','1968-01-09 10:03:59','suborbital rocket','success',725,'','','','','Meteo',636,'',46 from dual union all
select 16932,'1968-A54','1968-02-01 00:00:00','suborbital rocket','success',1032,'7','','','','Test',1467,'',9 from dual union all
select 16933,'1968-A55','1968-02-01 00:00:00','suborbital rocket','success',1032,'8','','','','Test',1467,'',9 from dual union all
select 16934,'1968-A56','1968-02-01 00:00:00','suborbital rocket','failure',839,'NASA 10.255GM','GSFC Grenade 169 PB','','','Aeronomy',1039,'',0 from dual union all
select 16935,'1968-A57','1968-02-05 00:00:00','military missile','failure',1252,'FLA-16','','','','Test',1888,'',3 from dual union all
select 16936,'1968-A58','1968-02-06 00:00:00','suborbital spaceplane','success',1520,'1- C-127','','','','Test',379,'NB-52',14 from dual union all
select 16937,'1968-A59','1968-02-07 00:00:00','suborbital spaceplane','failure',1520,'1- A-128','','','','Test',379,'NB-52',14 from dual union all
select 16938,'1968-A60','1968-02-12 00:00:00','military missile','failure',1038,'7L','8K99 LKI-3','','','Test',477,'',0 from dual union all
select 16939,'1968-A61','1968-02-14 04:40:00','suborbital rocket','success',725,'','','','','Meteo',636,'',38 from dual union all
select 1694,'1953-A279','1953-07-02 00:00:00','military missile','unknown',339,'1277R','','','','Test',1895,'',30 from dual union all
select 16940,'1968-A62','1968-02-22 05:12:00','suborbital rocket','failure',867,'NASA 18.61GE','Aurora','','','Plasma',428,'',10 from dual union all
select 16941,'1968-A63','1968-02-26 00:00:00','military missile','success',1252,'FLA-17','','','','Test',1888,'',6 from dual union all
select 16942,'1968-A64','1968-02-27 00:00:00','suborbital spaceplane','failure',1520,'1- A-129','','','','Test',379,'NB-52',14 from dual union all
select 16943,'1968-A65','1968-03-01 00:00:00','military missile','success',545,'DASA 6.1','','','','Aeron?',1895,'',40 from dual union all
select 16944,'1968-A66','1968-03-01 19:28:11','suborbital spaceplane','success',1520,'1-74-130','Checkout','','','Test',379,'NB-52 008',31 from dual union all
select 16945,'1968-A67','1968-03-18 00:00:00','suborbital rocket','failure',204,'301','FU-173B','','','Test',307,'',0 from dual union all
select 16946,'1968-A68','1968-03-25 00:00:00','military missile','failure',1252,'FLA-18','','','','Test',1888,'',0 from dual union all
select 16947,'1968-A69','1968-03-28 00:00:00','suborbital spaceplane','failure',1520,'1- A-131','','','','Test',379,'NB-52',14 from dual union all
select 16948,'1968-A70','1968-04-04 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 16949,'1968-A71','1968-04-04 00:00:00','suborbital spaceplane','failure',1520,'1- A-132','','','','Test',379,'NB-52',14 from dual union all
select 1695,'1953-A28','1953-01-23 00:00:00','military missile','failure',255,'621-2','','','','Test',254,'',20 from dual union all
select 16950,'1968-A72','1968-04-17 20:13:00','suborbital rocket','success',258,'NASA 12.10GT','MARC 42A2 test','','','Test',1821,'',8 from dual union all
select 16951,'1968-A73','1968-04-22 00:00:00','military missile','failure',1252,'FLA-19','','','','Test',1888,'',2 from dual union all
select 16952,'1968-A74','1968-04-23 04:47:00','suborbital rocket','failure',215,'AMF-2A-116','Auroral spectrum','','','Auroral/Aeron/Ionos',428,'',21 from dual union all
select 16953,'1968-A75','1968-04-23 05:54:00','suborbital rocket','failure',837,'NASA 14.356UM','UM Pitot 37','','','Aeronomy',428,'',10 from dual union all
select 16954,'1968-A76','1968-05-02 00:00:00','military missile','failure',135,'V116B [101]','RCS','ABRES AE-3 (USN)','','Reentry Test',612,'',0 from dual union all
select 16955,'1968-A77','1968-05-03 07:45:00','suborbital rocket','failure',30,'NASA 04.196UG','CAL XR 1','','','XR Astron',1869,'',3 from dual union all
select 16956,'1968-A78','1968-05-17 06:50:00','military missile','failure',135,'D061  [102]','SFT-11','ABRES AC-4 (USN)','','Reentry Test',614,'',0 from dual union all
select 16957,'1968-A79','1968-05-22 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 16958,'1968-A80','1968-05-23 00:00:00','suborbital spaceplane','failure',1520,'1- A-135','Fixed Alpha Ball/WTR','','','Test/Tech',379,'NB-52',14 from dual union all
select 16959,'1968-A81','1968-06-21 12:28:00','suborbital rocket','failure',1188,'Su3H','','','','Ionospheric',1447,'',0 from dual union all
select 1696,'1953-A280','1953-07-02 00:00:00','military missile','success',771,'GM-11059','','','','Test',222,'',12 from dual union all
select 16960,'1968-A82','1968-06-27 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 16961,'1968-A83','1968-06-27 00:00:00','military missile','success',1252,'FLA-20','','','','Test',1888,'',6 from dual union all
select 16962,'1968-A84','1968-07-15 00:00:00','suborbital spaceplane','failure',1520,'1- A-137','Fluidic Probe/etc','','','Test/Tech',379,'NB-52',14 from dual union all
select 16963,'1968-A85','1968-07-16 00:00:00','military missile','failure',982,'T 22503-05','8K64U UBP-117','','','OT',596,'',0 from dual union all
select 16964,'1968-A86','1968-07-18 00:00:00','atmospheric rocket','success',617,'J206','UK-2H/HRV 927','','','Test',1836,'',30 from dual union all
select 16965,'1968-A87','1968-07-18 00:00:00','military missile','failure',135,'B040  [103]','RCS','ABRES AE-4 (USN)','','Reentry Test',614,'',0 from dual union all
select 16966,'1968-A88','1968-07-29 00:00:00','military missile','success',1252,'FLA-21','','','','Test',1888,'',6 from dual union all
select 16967,'1968-A89','1968-08-14 23:28:00','suborbital rocket','unknown',88,'EXAMETNET C42','Arcasonde 1A','','','Aeron',308,'',48 from dual union all
select 16968,'1968-A90','1968-08-19 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 16969,'1968-A91','1968-08-22 00:00:00','military missile','success',1252,'FLA-22','','','','Test',1888,'',6 from dual union all
select 1697,'1953-A281','1953-07-06 00:00:00','atmospheric rocket','success',836,'1137A','','','','Test',1895,'',10 from dual union all
select 16970,'1968-A92','1968-08-22 22:30:00','suborbital rocket','success',835,'','Launcher test','','','Test',427,'',1 from dual union all
select 16971,'1968-A93','1968-09-09 00:00:00','military missile','success',1252,'FLA-23','','','','Test',1888,'',6 from dual union all
select 16972,'1968-A94','1968-09-13 14:31:00','suborbital rocket','success',32,'NASA 12.14GT','','','','Test',1869,'',5 from dual union all
select 16973,'1968-A95','1968-09-16 00:00:00','suborbital rocket','success',1032,'','Accelerometer','','','Test',1467,'',9 from dual union all
select 16974,'1968-A96','1968-09-16 00:00:00','military missile','failure',1251,'800007','KT-4','','','Test',868,'',2 from dual union all
select 16975,'1968-A97','1968-09-16 16:50:00','suborbital rocket','success',37,'CRL AT3.170','Dummy Sustainer','','','Test',1895,'',12 from dual union all
select 16976,'1968-A98','1968-09-17 07:10:00','suborbital rocket','success',1078,'SA-II A9','','','','Test',1476,'',20 from dual union all
select 16977,'1968-A99','1968-09-18 07:30:00','suborbital rocket','failure',839,'NASA 10.282GM','GSFC Grenade 185 C','','','Aeronomy',428,'',19 from dual union all
select 16978,'1968-E01','1968-07-15 00:00:00','orbital','destroyed before launch',953,'','','','','Sat',588,'',0 from dual union all
select 16979,'1968-F01','1968-02-07 10:43:54','orbital','failure',809,'Ya15000- 57  L57','','E-6LS No. 112','','Sat',480,'',190 from dual union all
select 1698,'1953-A282','1953-07-07 00:00:00','military missile','unknown',339,'1267E','EU-6','','','Test',1895,'',30 from dual union all
select 16980,'1968-F02','1968-03-06 11:02:57','orbital','failure',678,'','','DS-U1-Ya No. 1','','Sat',627,'',255 from dual union all
select 16981,'1968-F03','1968-04-22 23:01:27','orbital','failure',953,'232-01','','L-1 No. 7L','','Sat',588,'',120 from dual union all
select 16982,'1968-F04','1968-05-18 08:23:00','orbital','failure',1395,'520/FTV 6221 (TA9)','Nimbus B','Nimbus B','','Sat TAG154',1723,'',20 from dual union all
select 16983,'1968-F05','1968-06-04 18:45:00','orbital','failure',680,'N149    -23LM','','Sfera No. 12L','','Sat',457,'',1200 from dual union all
select 16984,'1968-F06','1968-06-15 12:25:00','orbital','failure',679,'4000-000-11LP','','Strela-2 No. 3','','Sat',564,'',200 from dual union all
select 16985,'1968-F07','1968-08-16 20:57:44','orbital','failure',148,'7004','P68-1','UVR','','Sat',1775,'',600 from dual union all
select 16986,'1968-F08','1968-09-19 00:09:00','orbital','failure',1386,'529/D59','Intelsat III F-1','INTELSAT III F-1','','Sat',233,'',15 from dual union all
select 16987,'1968-F09','1968-11-29 23:12:02','orbital','failure',467,'F-7','STV 1','STV 1','','Sat',1845,'',230 from dual union all
select 16988,'1968-M01','1968-01-01 00:00:00','suborbital rocket','success',549,'','4-ft Ballute test','','','Test',1895,'',73 from dual union all
select 16989,'1968-M02','1968-01-01 00:00:00','military missile','success',749,'','Caisson 2','','','Test',312,'',50 from dual union all
select 1699,'1953-A283','1953-07-08 00:00:00','military missile','success',703,'626','','','','Test',222,'',8 from dual union all
select 16990,'1968-M03','1968-02-07 16:00:00','suborbital rocket','success',839,'NASA 10.234GM','Ozone/Parachute test','','','Aeronomy',1895,'',67 from dual union all
select 16991,'1968-M04','1968-03-07 22:36:00','suborbital rocket','success',1189,'AMD-S2-5','O2 profile','','','Aeron',428,'',68 from dual union all
select 16992,'1968-M05','1968-04-04 18:02:17','suborbital spaceplane','success',1520,'1-75-133','CO/WTR/SA TPS','','','Test/Tech',379,'NB-52 008',57 from dual union all
select 16993,'1968-M06','1968-04-26 19:51:49','suborbital spaceplane','success',1520,'1-76-134','SA TPS/HS','','','Test/Tech',379,'NB-52 008',63 from dual union all
select 16994,'1968-M07','1968-05-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 16995,'1968-M08','1968-04-30 02:00:00','suborbital rocket','success',1189,'AMD-S2-7','','','','Aeron/CR',428,'',72 from dual union all
select 16996,'1968-M09','1968-04-30 10:00:00','suborbital rocket','success',1189,'AMD-S2-6','','','','Aeron/CR',428,'',58 from dual union all
select 16997,'1968-M10','1968-05-08 11:47:25','suborbital rocket','success',288,'CE-46 (C152)','ESRO C33/1','','','Ionospheric',1343,'',53 from dual union all
select 16998,'1968-M11','1968-05-15 11:24:00','suborbital rocket','success',288,'CE-47 (C153)','ESRO C33/2','','','Ionospheric',1343,'',80 from dual union all
select 16999,'1968-M12','1968-06-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 17,'1943-A10','1943-07-01 00:00:00','military missile','failure',8,'33','','','','Test',675,'',0 from dual union all
select 170,'1944-S154','1944-09-27 00:00:00','military missile','success',1470,'','Ipswich','','','Weapon',1741,'',95 from dual union all
select 1700,'1953-A284','1953-07-09 00:00:00','military missile','success',771,'GM-11057','','','','Test',222,'',12 from dual union all
select 17000,'1968-M13','1968-06-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 17001,'1968-M14','1968-06-12 15:31:01','suborbital spaceplane','success',1520,'1-77-136','WTR/SA TPS/HS/FAB','','','Test/Tech',379,'NB-52 008',67 from dual union all
select 17002,'1968-M15','1968-07-16 22:23:06','suborbital spaceplane','success',1520,'1-78-138','WTR/HS/SB/FAB/FP','','','Test/Tech/Aeron',379,'NB-52 003',67 from dual union all
select 17003,'1968-M16','1968-08-07 00:00:00','military missile','success',1251,'800005','KT-3','','','Test',868,'',50 from dual union all
select 17004,'1968-M17','1968-08-14 16:37:00','suborbital rocket','failure',30,'NRL NE3.183','Solar radiation','','','Solar EUV',1869,'',79 from dual union all
select 17005,'1968-M18','1968-09-10 02:00:00','suborbital rocket','success',1069,'ST-160-F-1','','','','Test',839,'',80 from dual union all
select 17006,'1968-M19','1968-09-10 00:00:00','suborbital rocket','success',1069,'ST-160-F-2','','','','Test',839,'',80 from dual union all
select 17007,'1968-M20','1968-09-12 00:00:00','suborbital rocket','success',1069,'SO-160-1','','','','Test',839,'',80 from dual union all
select 17008,'1968-M21','1968-09-13 18:19:23','suborbital spaceplane','success',1520,'1-80-140','WTR/HS/SB/FAB/FP','','','Test/Tech/Aeron',379,'NB-52 003',77 from dual union all
select 17009,'1968-M22','1968-09-16 15:58:00','suborbital rocket','success',839,'NASA 10.235GM','GSFC Ozone 3','','','Aeronomy',1821,'',69 from dual union all
select 1701,'1953-A285','1953-07-09 00:00:00','atmospheric rocket','success',836,'1204A','','','','Test',1895,'',10 from dual union all
select 17010,'1968-M23','1968-11-22 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 17011,'1968-M24','1968-10-23 15:32:00','suborbital rocket','success',549,'PEPP/SHAPE','40-ft DBG Para','','','Test',1895,'',53 from dual union all
select 17012,'1968-M25','1968-10-24 17:02:47','suborbital spaceplane','success',1520,'1-81-141','WTR/HS/FAB/FP','','','Test/Tech/Aeron',379,'NB-52 003',77 from dual union all
select 17013,'1968-M26','1968-11-01 04:59:00','suborbital rocket','failure',1181,'NASA 15.24II','Arcas VI PCA','','','Ionos/Plasma/Field',29,'',80 from dual union all
select 17014,'1968-M27','1968-12-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 17015,'1968-M28','1968-12-11 00:00:00','suborbital rocket','success',549,'PEPP','SHAPE Parachute test','','','Test',1895,'',53 from dual union all
select 17016,'1968-M29','1968-08-15 02:23:00','suborbital rocket','success',1188,'S101','Met','','','Meteo',1849,'',65 from dual union all
select 17017,'1968-M30','1968-09-04 12:12:00','suborbital rocket','success',1188,'S102','Met','','','Meteo',1849,'',58 from dual;
insert into LAUNCH
select 17018,'1968-M31','1968-09-18 11:33:00','suborbital rocket','success',1188,'S103','Met','','','Meteo',1849,'',64 from dual union all
select 17019,'1968-M32','1968-10-10 00:18:00','suborbital rocket','success',1188,'S104','Met','','','Meteo',1849,'',67 from dual union all
select 1702,'1953-A286','1953-07-09 00:00:00','atmospheric rocket','success',836,'1161A','','','','Test',1895,'',10 from dual union all
select 17020,'1968-M33','1968-11-07 21:36:00','suborbital rocket','success',1188,'S105','Met','','','Meteo',1849,'',63 from dual union all
select 17021,'1968-M34','1968-08-16 01:33:00','suborbital rocket','success',675,'WRE K113','Met','','','Meteo',1849,'',74 from dual union all
select 17022,'1968-M35','1968-09-04 10:42:00','suborbital rocket','success',675,'WRE K115','Met','','','Meteo',1849,'',73 from dual union all
select 17023,'1968-M36','1968-09-18 12:33:00','suborbital rocket','success',675,'WRE K114','Met','','','Meteo',1849,'',76 from dual union all
select 17024,'1968-M37','1968-10-09 00:26:00','suborbital rocket','success',675,'WRE K119','Met','','','Meteo',1849,'',78 from dual union all
select 17025,'1968-M38','1968-10-10 01:33:00','suborbital rocket','success',675,'WRE K117','Met','','','Meteo',1849,'',70 from dual union all
select 17026,'1968-M39','1968-11-19 11:34:00','suborbital rocket','success',675,'WRE K120','Met','','','Meteo',1849,'',73 from dual union all
select 17027,'1968-M40','1968-04-21 00:00:00','suborbital rocket','success',1506,'SDC VD-17','','','','Aeron',339,'',50 from dual union all
select 17028,'1968-M41','1968-11-19 12:03:00','suborbital rocket','success',1188,'S106','Met','','','Meteo',1849,'',61 from dual union all
select 17029,'1968-M42','1968-11-20 17:03:00','suborbital rocket','success',1188,'S107','Met','','','Meteo',1849,'',66 from dual union all
select 1703,'1953-A287','1953-07-09 00:00:00','atmospheric rocket','success',836,'1249B-1203','Radford Booster/225R','','','Test',1895,'',10 from dual union all
select 17030,'1968-M43','1968-12-02 22:23:00','suborbital rocket','success',1188,'S108','Met','','','Meteo',1849,'',62 from dual union all
select 17031,'1968-M44','1968-02-29 00:00:00','suborbital rocket','success',88,'','Ballute/Arcasonde 1A','','','Test',293,'',60 from dual union all
select 17032,'1968-M45','1968-01-17 11:57:00','suborbital rocket','success',631,'INTA H-6801','','','','Aeron',396,'',60 from dual union all
select 17033,'1968-M46','1968-01-24 11:14:00','suborbital rocket','success',631,'INTA H-6802','','','','Aeron',396,'',60 from dual union all
select 17034,'1968-M47','1968-01-31 11:18:00','suborbital rocket','success',631,'INTA H-6803','','','','Aeron',396,'',60 from dual union all
select 17035,'1968-M48','1968-02-14 16:22:00','suborbital rocket','success',631,'INTA H-6804','','','','Aeron',396,'',60 from dual union all
select 17036,'1968-M49','1968-02-22 09:38:00','suborbital rocket','success',631,'INTA H-6805','','','','Aeron',396,'',60 from dual union all
select 17037,'1968-M50','1968-02-28 11:35:00','suborbital rocket','success',631,'INTA H-6806','','','','Aeron',396,'',60 from dual union all
select 17038,'1968-M51','1968-03-13 11:34:00','suborbital rocket','success',631,'INTA H-6807','','','','Aeron',396,'',60 from dual union all
select 17039,'1968-M52','1968-09-04 10:13:00','suborbital rocket','success',631,'INTA H-6808','','','','Aeron',396,'',60 from dual union all
select 1704,'1953-A288','1953-07-09 00:00:00','atmospheric rocket','success',836,'1249B-1219','Nike I Booster/226R','','','Test',1895,'',10 from dual union all
select 17040,'1968-M53','1968-09-11 10:00:00','suborbital rocket','success',631,'INTA H-6809','','','','Aeron',396,'',60 from dual union all
select 17041,'1968-M54','1968-09-13 11:20:00','suborbital rocket','success',631,'INTA H-6810','','','','Aeron',396,'',60 from dual union all
select 17042,'1968-M55','1968-09-18 10:11:00','suborbital rocket','success',631,'INTA H-6811','','','','Aeron',396,'',60 from dual union all
select 17043,'1968-M56','1968-09-25 10:54:00','suborbital rocket','success',631,'INTA H-6812','','','','Aeron',396,'',60 from dual union all
select 17044,'1968-M57','1968-10-09 10:42:00','suborbital rocket','success',631,'INTA H-6813','','','','Aeron',396,'',60 from dual union all
select 17045,'1968-M58','1968-02-07 11:06:00','suborbital rocket','success',1188,'INTA S-6801','','','','Aeron',396,'',75 from dual union all
select 17046,'1968-M59','1968-03-06 11:34:00','suborbital rocket','success',1188,'INTA S-6802','','','','Aeron',396,'',75 from dual union all
select 17047,'1968-M60','1968-11-01 00:00:00','suborbital rocket','success',361,'','Dardo de Invest. Met.','','','Meteo',308,'',70 from dual union all
select 17048,'1968-M61','1968-11-01 00:00:00','suborbital rocket','success',361,'','Dardo de Invest. Met.','','','Meteo',308,'',70 from dual union all
select 17049,'1968-M62','1968-11-01 00:00:00','suborbital rocket','success',361,'','Dardo de Invest. Met.','','','Meteo',308,'',70 from dual union all
select 1705,'1953-A289','1953-07-10 00:00:00','atmospheric rocket','success',836,'1157A','','','','Test',1895,'',10 from dual union all
select 17050,'1968-M63','1968-05-02 00:00:00','military missile','success',1162,'','MINOTAURUS 68','RB250 Training','','OT',984,'',50 from dual union all
select 17051,'1968-M64','1968-06-10 00:00:00','military missile','success',1162,'','','RB150 Training','','OT',984,'',50 from dual union all
select 17052,'1968-M65','1968-03-01 00:00:00','military missile','success',1162,'','','RB650 Training','','OT',984,'',50 from dual union all
select 17053,'1968-M66','1968-03-01 00:00:00','military missile','success',1162,'','','30FA B2 Training','','OT',984,'',50 from dual union all
select 17054,'1968-S01','1968-01-01 00:00:00','military missile','success',736,'','','','','Test',313,'',100 from dual union all
select 17055,'1968-S02','1968-01-01 00:00:00','military missile','success',736,'','','','','Test',313,'',100 from dual union all
select 17056,'1968-S03','1968-01-01 00:00:00','military missile','failure',736,'','','','','Test',313,'',100 from dual union all
select 17057,'1968-S04','1968-01-01 00:00:00','military missile','failure',736,'','','','','Test',311,'',100 from dual union all
select 17058,'1968-S05','1968-03-01 00:00:00','suborbital rocket','success',202,'','','','','Weather',313,'',100 from dual union all
select 17059,'1968-S06','1968-03-01 00:00:00','suborbital rocket','success',202,'','','','','Weather',313,'',100 from dual union all
select 1706,'1953-A29','1953-01-23 00:00:00','atmospheric rocket','success',836,'1249B','Nike Eng 8E','','','Test',1895,'',10 from dual union all
select 17060,'1968-S07','1968-01-01 00:00:00','suborbital rocket','success',205,'','','','','Weather',311,'',90 from dual union all
select 17061,'1968-S08','1968-01-01 00:00:00','military missile','success',750,'A2','A2','','','Test',301,'',500 from dual union all
select 17062,'1968-S09','1968-01-01 00:00:00','military missile','success',750,'B1','B1','','','Test',301,'',500 from dual union all
select 17063,'1968-S10','1968-01-01 00:00:00','military missile','success',750,'B2','B2','','','Test',301,'',500 from dual union all
select 17064,'1968-S100','1968-01-25 00:00:00','military missile','success',796,'2448F','Mk 11B RV','ST OLYMPIC TRIALS 7','OLYMPIC TRIALS 7','OT',1708,'',900 from dual union all
select 17065,'1968-S101','1968-01-26 04:00:00','suborbital rocket','success',30,'NASA 04.205GG','Stellar Astron.','','','UV Astron',1869,'',160 from dual union all
select 17066,'1968-S102','1968-01-30 05:03:00','suborbital rocket','success',215,'AKD-2A-111','','','','Auroral',428,'',161 from dual union all
select 17067,'1968-S103','1968-01-31 10:30:00','military missile','success',168,'94F','RMV-320/Decoys','ABRES RMP-B-5/NTMP KX104','','Reentry Test',1682,'',1400 from dual union all
select 17068,'1968-S104','1968-01-31 17:00:00','military missile','unknown',929,'A2P-91','DEFT','','','OT',421,'SSBN 609',1000 from dual union all
select 17069,'1968-S105','1968-02-01 00:00:00','military missile','success',135,'B037','IMPALA 5','ABRES YY-4 (USN)','','Reentry Test',614,'',200 from dual union all
select 1707,'1953-A290','1953-07-14 00:00:00','atmospheric rocket','unknown',576,'212H','','','','Test',1895,'',10 from dual union all
select 17070,'1968-S106','1968-02-01 05:42:00','suborbital rocket','success',290,'CE-34','ESRO C36/1','','','Aeronomy',408,'',160 from dual union all
select 17071,'1968-S107','1968-02-01 18:53:00','suborbital rocket','success',839,'NASA 10.264GM','GSFC Grenade 170/UM Sph','','','Aeronomy',1821,'',104 from dual union all
select 17072,'1968-S108','1968-02-01 19:00:00','suborbital rocket','success',839,'NASA 10.259GM','GSFC Grenade 171 C','','','Aeronomy',428,'',126 from dual union all
select 17073,'1968-S109','1968-02-01 19:30:00','suborbital rocket','success',839,'NASA 10.260GM','GSFC Grenade 172 C','','','Aeronomy',428,'',127 from dual union all
select 17074,'1968-S11','1968-01-01 00:00:00','military missile','success',751,'V2','V2','','','OT',307,'S655',500 from dual union all
select 17075,'1968-S110','1968-02-01 20:30:00','suborbital rocket','success',839,'NASA 10.261GM','GSFC Grenade 173 C','','','Aeronomy',428,'',137 from dual union all
select 17076,'1968-S111','1968-02-01 21:15:00','suborbital rocket','success',839,'NASA 10.262GM','GSFC Grenade 174 C','','','Aeronomy',428,'',130 from dual union all
select 17077,'1968-S112','1968-02-02 10:17:00','military missile','success',796,'2464','Mk 11B RV','ST OLYMPIC TRIALS 8','OLYMPIC TRIALS 8','OT',1710,'',900 from dual union all
select 17078,'1968-S113','1968-02-02 12:55:00','military missile','failure',795,'1274','Mk 11 RV','ST PACER KITE 1','OLYMPIC TRIALS B-1','OT',1705,'',10 from dual union all
select 17079,'1968-S114','1968-02-02 23:57:00','suborbital rocket','success',837,'Round 119','Speedball II S/N 53','','','Target',1895,'',200 from dual union all
select 1708,'1953-A291','1953-07-14 00:00:00','atmospheric rocket','success',836,'1249B-1211','Nike 227R','','','Test',1895,'',10 from dual union all
select 17080,'1968-S115','1968-02-02 05:30:00','suborbital rocket','success',30,'NASA 04.261GG','','','','XR Astron',1869,'',139 from dual union all
select 17081,'1968-S116','1968-02-02 06:10:00','suborbital rocket','success',30,'NASA 04.220GG','Stellar Astron.','','','UV Astron',1869,'',149 from dual union all
select 17082,'1968-S117','1968-02-02 13:26:00','suborbital rocket','success',837,'NASA 14.259IA','ISRO 10.13','TERLS-064','','Aeron/Ionos',1467,'',191 from dual union all
select 17083,'1968-S118','1968-02-02 13:40:00','suborbital rocket','success',288,'C90','ISRO 15.06','TERLS-065','','Aeron',1467,'',159 from dual union all
select 17084,'1968-S119','1968-02-03 17:12:00','suborbital rocket','success',920,'P5H','Electrons','','','Ionospheric',1447,'',140 from dual union all
select 17085,'1968-S12','1968-01-01 00:00:00','military missile','success',751,'V3','V3','','','OT',307,'S655',500 from dual union all
select 17086,'1968-S120','1968-02-04 05:33:05','suborbital rocket','success',290,'CE-35','ESRO C36/2','','','Aeronomy',408,'',165 from dual union all
select 17087,'1968-S121','1968-02-05 19:00:00','suborbital rocket','success',343,'NASA 12.08GT','','','','Test',1821,'',119 from dual union all
select 17088,'1968-S122','1968-02-05 22:22:00','suborbital rocket','success',839,'NASA 10.263GM','GSFC Grenade 175 C','','','Aeronomy',428,'',127 from dual union all
select 17089,'1968-S123','1968-02-06 05:07:00','suborbital rocket','success',867,'NASA 18.38UE','UNH Aurora/Fields','','','Plasma',428,'',270 from dual union all
select 1709,'1953-A292','1953-07-14 00:00:00','atmospheric rocket','success',836,'1152E','','','','Test',1895,'',10 from dual union all
select 17090,'1968-S124','1968-02-06 07:21:00','suborbital rocket','success',864,'Sandia 152-105','Aurora','','','Auroral',428,'',160 from dual union all
select 17091,'1968-S125','1968-02-07 03:20:21','military missile','success',796,'468','Mk 12 RV','R&D','','Test',266,'',900 from dual union all
select 17092,'1968-S126','1968-02-08 06:01:00','suborbital rocket','success',1210,'SL583A','RAE Test 39','','','Test',1839,'',300 from dual union all
select 17093,'1968-S127','1968-02-09 01:54:00','suborbital rocket','success',864,'Sandia 152-104','Aurora','','','Auroral',428,'',150 from dual union all
select 17094,'1968-S128','1968-02-09 04:41:00','suborbital rocket','success',30,'NASA 04.217UA','JHU Aurora 6','','','Aurora',428,'',155 from dual union all
select 17095,'1968-S129','1968-02-10 11:42:00','military missile','success',796,'2307','Mk 12 RV','R&D/MOD','','Test',1703,'',900 from dual union all
select 17096,'1968-S13','1968-12-20 00:00:00','military missile','success',982,'T 22505-16T','8K64U UBP-124','','','OT',596,'',1210 from dual union all
select 17097,'1968-S131','1968-02-14 00:00:00','suborbital rocket','success',446,'D-32','FU-172 Solar He','','','Solar UV',307,'',440 from dual union all
select 17098,'1968-S132','1968-02-15 16:15:00','military missile','success',932,'A3P-670 (A3E/UK-670)','Antigua','','','OT',421,'UK S22',1000 from dual union all
select 17099,'1968-S133','1968-02-15 15:46:00','suborbital rocket','success',1189,'INTA MP-6801','','','','Ionos',396,'',81 from dual union all
select 171,'1944-S155','1944-09-27 00:00:00','military missile','success',1470,'','Norwich','','','Weapon',1741,'',95 from dual union all
select 1710,'1953-A293','1953-07-15 00:00:00','atmospheric rocket','success',365,'','E25 Model','','','Test',1821,'',10 from dual union all
select 17100,'1968-S134','1968-02-16 10:58:00','suborbital rocket','success',1189,'INTA MP-6802','','','','Ionos',396,'',100 from dual union all
select 17101,'1968-S135','1968-02-17 02:24:00','suborbital rocket','success',30,'KP3.22','Stellar UV','','','UV Astron',1869,'',183 from dual union all
select 17102,'1968-S136','1968-02-17 11:54:00','suborbital rocket','success',1189,'INTA MP-6803','','','','Ionos',396,'',81 from dual union all
select 17103,'1968-S137','1968-02-19 18:55:00','suborbital rocket','success',30,'CRL AH3.531','EUV Mon','','','Solar EUV',1869,'',243 from dual union all
select 17104,'1968-S138','1968-02-19 23:15:00','suborbital rocket','success',30,'NASA 04.169CS','','','','Solar X',1869,'',161 from dual union all
select 17105,'1968-S139','1968-02-21 19:40:00','suborbital rocket','success',288,'C88','ISRO 25.02','TERLS-066','','Aeron',1467,'',136 from dual union all
select 17106,'1968-S14','1968-03-20 00:00:00','military missile','success',982,'E 22505-07T','8K64U UBP-111','','','OT',596,'',1210 from dual union all
select 17107,'1968-S140','1968-02-21 23:17:00','suborbital rocket','success',837,'NASA 14.284CA','GCA Release 72 Teb','','','Aeronomy',1821,'',150 from dual;
insert into LAUNCH
select 17108,'1968-S141','1968-02-22 03:45:00','suborbital rocket','success',837,'NASA 14.276GE','Auroral fluxes','','','Plasma',428,'',200 from dual union all
select 17109,'1968-S142','1968-02-22 05:09:00','suborbital rocket','success',837,'NASA 14.364CM','GCA Ionos-TMA 14','','','Aeronomy',1821,'',156 from dual union all
select 1711,'1953-A294','1953-07-16 00:00:00','military missile','unknown',339,'1278R','','','','Test',1895,'',30 from dual union all
select 17110,'1968-S143','1968-02-22 06:30:00','suborbital rocket','success',837,'NASA 14.365CM','GCA Ionos-TMA 15','','','Aeronomy',1821,'',175 from dual union all
select 17111,'1968-S144','1968-02-22 08:00:00','suborbital rocket','success',837,'NASA 14.366CM','GCA Ionos-TMA 16','','','Aeronomy',1821,'',181 from dual union all
select 17112,'1968-S145','1968-02-22 09:30:00','suborbital rocket','success',837,'NASA 14.367CM','GCA Ionos-TMA 17','','','Aeronomy',1821,'',190 from dual union all
select 17113,'1968-S146','1968-02-22 11:02:00','suborbital rocket','success',837,'NASA 14.208CA','GCA Ionos 14/Rel Na','','','Aeronomy',1821,'',168 from dual union all
select 17114,'1968-S147','1968-02-22 11:25:00','suborbital rocket','success',288,'C89','ISRO 25.01','TERLS-067','','Aeron',1467,'',153 from dual union all
select 17115,'1968-S149','1968-02-23 05:10:00','suborbital rocket','success',867,'NASA 18.35UE','Aurora','','','Plasma',428,'',293 from dual union all
select 17116,'1968-S15','1968-04-22 00:00:00','military missile','success',982,'','8K64U UBP-113','','','OT',596,'',1210 from dual union all
select 17117,'1968-S150','1968-02-23 06:53:00','suborbital rocket','success',867,'NASA 18.67UA','','','','Auroral',428,'',270 from dual union all
select 17118,'1968-S151','1968-02-27 07:35:00','military missile','success',162,'116F','BGRV S/N 2','ABRES BGRV-4','','Reentry Test',1680,'',40 from dual union all
select 17119,'1968-S152','1968-02-27 17:54:00','suborbital rocket','success',837,'STV 070 SRN-02','','','','Aeron?',1895,'',200 from dual union all
select 1712,'1953-A295','1953-07-17 00:00:00','atmospheric rocket','success',836,'1249B-1231','Nike 228R','','','Test',1895,'',10 from dual union all
select 17120,'1968-S153','1968-02-27 01:10:00','suborbital rocket','success',837,'NASA 14.350CA','GCA Release 73','','','Aeronomy',1821,'',154 from dual union all
select 17121,'1968-S154','1968-02-27 01:30:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',170 from dual union all
select 17122,'1968-S155','1968-02-28 15:00:00','military missile','success',1420,'B-88','Mk 6 RV','FOT GT04T','GLORY TRIP 04T','OT',1657,'',1300 from dual union all
select 17123,'1968-S156','1968-02-28 06:01:00','suborbital rocket','success',867,'NASA 18.68UA','','','','Auroral',428,'',295 from dual union all
select 17124,'1968-S157','1968-02-28 14:40:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',168 from dual union all
select 17125,'1968-S158','1968-03-08 00:00:00','military missile','success',917,'','P-142 MartinMar.','','','Test',435,'',180 from dual union all
select 17126,'1968-S159','1968-02-29 03:30:00','military missile','success',999,'V 22501-15T','8K67 UBP-20','','','OT',514,'',1000 from dual union all
select 17127,'1968-S16','1968-05-24 00:00:00','military missile','success',982,'T 22502-19T','8K64U UBP-115','','','OT',596,'',1210 from dual union all
select 17128,'1968-S160','1968-02-29 17:15:00','suborbital rocket','unknown',740,'MR-12','','','','Auroral',1010,'',170 from dual union all
select 17129,'1968-S161','1968-02-29 21:18:00','suborbital rocket','success',920,'P6H','Electrons','','','Ionospheric',1447,'',140 from dual union all
select 1713,'1953-A296','1953-07-17 00:00:00','atmospheric rocket','success',863,'1249B-1239','Nike 229R','','','Test',1895,'',10 from dual union all
select 17130,'1968-S162','1968-03-01 03:30:00','suborbital rocket','success',30,'NASA 04.177UG','Stellar IR','','','IR Astron',1869,'',171 from dual union all
select 17131,'1968-S163','1968-03-02 05:00:00','suborbital rocket','success',624,'NASA 8.48UE','Rice TWINS 2 auroral','','','Auroral',428,'',805 from dual union all
select 17132,'1968-S164','1968-03-02 07:19:00','suborbital rocket','success',867,'NASA 18.37UE','Auroral P/F','','','Auroral/Plasma',428,'',260 from dual union all
select 17133,'1968-S165','1968-03-04 16:15:00','military missile','success',932,'A3P-666 (A3E/UK-666)','Antigua','','','OT',421,'UK S22',1000 from dual union all
select 17134,'1968-S166','1968-03-05 00:00:00','military missile','success',917,'','P-141 MartinMar.','','','OT',435,'',180 from dual union all
select 17135,'1968-S167','1968-03-01 00:00:00','military missile','success',917,'AO-58','P-143 68 Spring 01/MM','','','OT',435,'',180 from dual union all
select 17136,'1968-S168','1968-03-06 10:15:00','military missile','success',161,'74E','RMV-310/Decoys','ABRES RMP-B-6/NTMP KX105','','Reentry Test',1682,'',1600 from dual union all
select 17137,'1968-S169','1968-03-06 16:36:00','suborbital rocket','success',920,'P7H','C/D/E electrons','','','Ionospheric',1447,'',140 from dual union all
select 17138,'1968-S17','1968-12-25 00:00:00','military missile','success',982,'T 22505-15T','8K64U UBP-125','','','OT',596,'',1210 from dual union all
select 17139,'1968-S170','1968-03-06 21:35:09','suborbital rocket','success',290,'CE-37','ESRO C14/P','','','Ionospheric',408,'',155 from dual union all
select 1714,'1953-A297','1953-07-18 22:27:00','suborbital rocket','failure',370,'MUSHRAT 1','SUI 8','','','CR',37,'AGB-5',11 from dual union all
select 17140,'1968-S171','1968-03-08 00:00:00','suborbital rocket','success',364,'DA003','FU-168 A-cone','','','Test',307,'',123 from dual union all
select 17141,'1968-S172','1968-03-11 00:00:00','suborbital rocket','success',625,'','Triple launch test','','','Test',1895,'',80 from dual union all
select 17142,'1968-S173','1968-03-11 00:00:00','suborbital rocket','success',625,'','Triple launch test','','','Test',1895,'',80 from dual union all
select 17143,'1968-S174','1968-03-11 00:00:00','suborbital rocket','success',625,'','Triple launch test','','','Test',1895,'',80 from dual union all
select 17144,'1968-S175','1968-03-12 00:00:00','suborbital rocket','success',205,'','','','','Weather',313,'',90 from dual union all
select 17145,'1968-S176','1968-03-15 03:43:00','suborbital rocket','success',447,'D-251A','FU-171 VLF/Ions','','','Ionos',856,'',400 from dual union all
select 17146,'1968-S177','1968-03-15 07:10:00','suborbital rocket','success',848,'CRL AE07.290','Auroral P/F','','','Auroral/Plasma',428,'',160 from dual union all
select 17147,'1968-S178','1968-03-15 19:04:00','suborbital rocket','success',30,'NASA 04.209CS','AS&E Solar','','','Solar X',1869,'',155 from dual union all
select 17148,'1968-S179','1968-03-15 19:11:00','suborbital rocket','success',867,'NASA 18.10UI','','','','Ionos',1821,'',290 from dual union all
select 17149,'1968-S18','1968-10-07 00:00:00','military missile','success',982,'T 22503-02','8K64U UBP-119','','','OT',596,'',1210 from dual union all
select 1715,'1953-A298','1953-07-19 10:30:00','suborbital rocket','failure',370,'MUSHRAT 2','SUI 9','','','CR',39,'AGB-5',11 from dual union all
select 17150,'1968-S180','1968-03-16 03:15:00','suborbital rocket','success',30,'NASA 04.198GE','Crab X-ray','','','XR Astron',1869,'',153 from dual union all
select 17151,'1968-S181','1968-03-16 17:30:00','suborbital rocket','success',740,'','Release','','','Aeron',1010,'',174 from dual union all
select 17152,'1968-S182','1968-03-17 09:28:00','suborbital rocket','success',867,'NASA 18.49GA','UM Thermosphere 9','','','Aeronomy',1483,'',311 from dual union all
select 17153,'1968-S183','1968-03-17 06:59:01','suborbital rocket','success',837,'NASA 14.344UM','UM Pitot 34','','','Aeronomy',1483,'',148 from dual union all
select 17154,'1968-S184','1968-03-17 07:30:01','suborbital rocket','success',867,'NASA 18.53GA','UM Thermosphere 8','','','Aeronomy',1483,'',295 from dual union all
select 17155,'1968-S185','1968-03-17 18:45:01','suborbital rocket','success',837,'NASA 14.345UM','UM Pitot 35','','','Aeronomy',1483,'',136 from dual union all
select 17156,'1968-S186','1968-03-17 19:18:00','suborbital rocket','success',867,'NASA 18.52GA','UM Thermosphere 10','','','Aeronomy',1483,'',270 from dual union all
select 17157,'1968-S187','1968-03-18 07:00:05','suborbital rocket','success',837,'NASA 14.333UM','UM Pitot 36','','','Aeronomy',1483,'',148 from dual union all
select 17158,'1968-S188','1968-03-18 17:51:00','suborbital rocket','success',867,'NASA 18.36UE','Auroral P/F','','','Plasma',428,'',270 from dual union all
select 17159,'1968-S189','1968-03-18 21:30:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',174 from dual union all
select 1716,'1953-A299','1953-07-19 15:53:00','suborbital rocket','failure',370,'MUSHRAT 3','SUI 10','','','CR',44,'AGB-5',11 from dual union all
select 17160,'1968-S19','1968-12-03 00:00:00','military missile','success',982,'T 22503-18T','8K64U UBP-123','','','OT',596,'',1210 from dual union all
select 17161,'1968-S190','1968-03-18 22:15:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',181 from dual union all
select 17162,'1968-S191','1968-03-19 16:15:00','suborbital rocket','success',30,'NASA 04.202NT','SPARCS Test','','','Test',1869,'',159 from dual union all
select 17163,'1968-S192','1968-03-19 18:20:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',176 from dual union all
select 17164,'1968-S193','1968-03-19 19:05:00','suborbital rocket','success',740,'','MX6407P mass spec','','','Aeron',1010,'',178 from dual union all
select 17165,'1968-S194','1968-03-20 04:57:00','suborbital rocket','success',1206,'SL408','LUX-41AC9/Cul-9','','','Solar X,UV',1839,'',217 from dual union all
select 17166,'1968-S195','1968-03-20 18:23:00','suborbital rocket','success',837,'K-NA-8','MPE Ba EXP-34','','','Aeron/Ionos',413,'',216 from dual union all
select 17167,'1968-S196','1968-03-20 19:55:05','suborbital rocket','success',589,'BAR-1','LRL BAR-1/FL-B','','','Test',1279,'AVM1',195 from dual union all
select 17168,'1968-S197','1968-03-20 20:25:00','suborbital rocket','success',740,'','Release','','','Aeron',1010,'',174 from dual union all
select 17169,'1968-S198','1968-03-20 21:09:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',182 from dual union all
select 1717,'1953-A30','1953-01-23 00:00:00','atmospheric rocket','success',836,'1249B','Nike Eng 7E','','','Test',1895,'',10 from dual union all
select 17170,'1968-S199','1968-03-20 21:36:00','suborbital rocket','success',740,'','Mass spec','','','Aeron',636,'',175 from dual union all
select 17171,'1968-S20','1968-01-05 00:00:00','military missile','success',982,'G 22603-19T','8K64U UBP-110','','','OT',596,'',1210 from dual union all
select 17172,'1968-S200','1968-03-21 00:00:00','military missile','success',135,'B046  [097]','ABRES AN-10 (USAF)','ABRES AN-10 (USAF)','','Reentry Test',614,'',200 from dual union all
select 17173,'1968-S201','1968-03-21 06:01:33','suborbital rocket','success',867,'NASA 18.63UE','Auroral emissions','','','Magnetospheric',428,'',242 from dual union all
select 17174,'1968-S202','1968-03-22 00:57:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',176 from dual union all
select 17175,'1968-S203','1968-03-22 07:24:00','suborbital rocket','success',30,'NASA 04.255GG','','','','UV Astron',1869,'',208 from dual union all
select 17176,'1968-S204','1968-03-23 18:35:00','suborbital rocket','success',837,'K-NA-9','DLR K-NA-9','','','Aeron/Ionos/Fields',413,'',200 from dual union all
select 17177,'1968-S205','1968-03-23 14:05:00','suborbital rocket','success',288,'C155','Rehbar 17','','','Aeron',1419,'',131 from dual union all
select 17178,'1968-S206','1968-03-23 19:27:00','suborbital rocket','success',740,'','Ion composition','','','Aeron/Ionos',1010,'',182 from dual union all
select 17179,'1968-S207','1968-03-24 12:45:00','suborbital rocket','success',839,'NASA 10.270GM','GSFC Grenade 176 N','','','Aeronomy',330,'',120 from dual union all
select 1718,'1953-A300','1953-07-19 21:57:00','suborbital rocket','failure',370,'MUSHRAT 4','SUI 11','','','CR',45,'AGB-5',11 from dual union all
select 17180,'1968-S208','1968-03-25 04:02:00','suborbital rocket','success',740,'','Ion composition','','','Aeron/Ionos',1010,'',180 from dual union all
select 17181,'1968-S209','1968-03-25 05:10:00','suborbital rocket','success',740,'','Composition','','','Aeron/Ionos',1010,'',180 from dual union all
select 17182,'1968-S21','1968-04-03 00:00:00','military missile','success',982,'T 22505-11T','8K64U UBP-112','','','OT',596,'',1210 from dual union all
select 17183,'1968-S210','1968-03-25 06:00:00','suborbital rocket','success',839,'NASA 10.271GM','GSFC Grenade 177 N','','','Aeronomy',330,'',120 from dual union all
select 17184,'1968-S211','1968-03-25 12:58:00','suborbital rocket','success',839,'NASA 10.272GM','GSFC Grenade 178 N','','','Aeronomy',330,'',120 from dual union all
select 17185,'1968-S212','1968-03-25 19:20:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',181 from dual union all
select 17186,'1968-S213','1968-03-25 21:14:00','suborbital rocket','success',920,'P8H','C/D/E electrons','','','Ionospheric',1447,'',135 from dual union all
select 17187,'1968-S214','1968-03-25 22:40:00','suborbital rocket','success',740,'','','','','Aeron',636,'',175 from dual union all
select 17188,'1968-S215','1968-03-26 23:41:21','suborbital rocket','success',1200,'S16/1','ESRO S16/1 MSSL Au1','','','Auroral',416,'',171 from dual union all
select 17189,'1968-S216','1968-03-27 07:55:00','suborbital rocket','success',447,'D-252','FU-171 VLF/Ions','','','Ionos',856,'',400 from dual union all
select 1719,'1953-A301','1953-07-22 00:00:00','atmospheric rocket','unknown',576,'312H','','','','Test',1895,'',10 from dual union all
select 17190,'1968-S217','1968-03-27 04:40:00','suborbital rocket','success',227,'AAF-VB-04','Auroral fluxes','','','Auroral/Ionos/Plasm',428,'',290 from dual union all
select 17191,'1968-S218','1968-03-27 11:50:00','suborbital rocket','success',740,'','','','','Aeron',636,'',179 from dual union all
select 17192,'1968-S219','1968-03-28 00:00:00','ballistic missile test','success',679,'U149    -14 (14L)','VKZ No. 12','','','Solar UV?',564,'',4000 from dual union all
select 17193,'1968-S22','1968-04-26 00:00:00','military missile','success',982,'R 22604-16T','8K64U UBP-114','','','OT',596,'',1210 from dual union all
select 17194,'1968-S220','1968-03-28 00:00:00','military missile','success',135,'B044  [099]','RCS','ABRES AE-2 (USN)','','Reentry Test',612,'',200 from dual union all
select 17195,'1968-S221','1968-03-28 00:00:00','military missile','success',135,'D068  [098]','ABRES AM-2 (USAF)','ABRES AM-2 (USAF)','','Reentry Test',613,'',200 from dual union all
select 17196,'1968-S222','1968-03-28 00:00:00','suborbital rocket','success',364,'DA004','FU-168 B-cone','','','Test',307,'',98 from dual union all
select 17197,'1968-S223','1968-03-28 01:32:00','suborbital rocket','success',837,'NASA 14.375IE-II','ISRO 10.15','TERLS-071','','Ionos',1467,'',211 from dual union all
select 17198,'1968-S224','1968-03-28 16:31:00','suborbital rocket','success',837,'NASA 14.346GR','','','','RF Astron',1821,'',161 from dual;
insert into LAUNCH
select 17199,'1968-S225','1968-03-29 00:00:00','military missile','success',796,'2522','Mk 11B RV','ST OLYMPIC TRIALS 9','OLYMPIC TRIALS 9','OT',1712,'',900 from dual union all
select 172,'1944-S156','1944-09-27 10:15:00','military missile','success',1470,'18771','Ma185','','','Test',661,'',90 from dual union all
select 1720,'1953-A302','1953-07-23 00:00:00','atmospheric rocket','success',836,'1298R','','','','Test',1895,'',10 from dual union all
select 17200,'1968-S226','1968-03-29 07:24:00','military missile','success',135,'V114C [100]','HAPDEC','ABRES AH-5 (ARPA)','','Reentry Test',614,'',200 from dual union all
select 17201,'1968-S227','1968-03-29 06:45:00','suborbital rocket','success',30,'NRL NF3.171','IR Telescope','','','IR Astron',1869,'',220 from dual union all
select 17202,'1968-S228','1968-03-29 08:29:00','suborbital rocket','success',30,'CRL AE3.154','Spectrograph','','','Aeron',1869,'',122 from dual union all
select 17203,'1968-S229','1968-03-29 14:20:00','suborbital rocket','success',446,'D-34','Shahpar-2','','','Test',1419,'',420 from dual union all
select 17204,'1968-S23','1968-07-02 00:00:00','military missile','success',982,'T 22504-03T','8K64U UBP-116','','','OT',596,'',1210 from dual union all
select 17205,'1968-S230','1968-03-30 00:00:00','military missile','success',1251,'800003','KT-1','','','Test',868,'',100 from dual union all
select 17206,'1968-S231','1968-03-30 08:00:00','suborbital rocket','success',740,'','Mass spec/Ne-Te 1','','','Ionos',636,'',170 from dual union all
select 17207,'1968-S232','1968-03-30 13:25:00','suborbital rocket','success',837,'NASA 14.376IE-II','ISRO 10.16','TERLS-072','','Ionos',1467,'',211 from dual union all
select 17208,'1968-S233','1968-03-30 14:30:00','suborbital rocket','success',740,'','','','','Aeron',636,'',176 from dual union all
select 17209,'1968-S234','1968-04-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 1721,'1953-A303','1953-07-23 00:00:00','atmospheric rocket','success',836,'1249B-1277','Nike 230R','','','Test',1895,'',10 from dual union all
select 17210,'1968-S235','1968-04-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 17211,'1968-S236','1968-04-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 17212,'1968-S237','1968-04-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 17213,'1968-S238','1968-04-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 17214,'1968-S239','1968-04-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 17215,'1968-S24','1968-09-28 00:00:00','military missile','success',982,'T 22604-03T','8K64U UBP-118','','','OT',596,'',1210 from dual union all
select 17216,'1968-S240','1968-04-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 17217,'1968-S241','1968-04-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 17218,'1968-S242','1968-04-10 00:00:00','military missile','success',837,'BMTS','','','','Target',435,'',100 from dual union all
select 17219,'1968-S243','1968-04-27 00:00:00','military missile','success',837,'BMTS','','','','Target',1863,'',100 from dual union all
select 1722,'1953-A304','1953-07-23 00:00:00','atmospheric rocket','success',836,'1249B-1209','Nike 231R','','','Test',1895,'',10 from dual union all
select 17220,'1968-S244','1968-04-16 00:00:00','military missile','success',916,'AO-60','P-144 68 Spring 02','','','Test',435,'',180 from dual union all
select 17221,'1968-S245','1968-04-16 00:00:00','military missile','success',916,'AO-61','P-145 68 Spring 03','','','Test',435,'',180 from dual union all
select 17222,'1968-S247','1968-03-31 00:07:00','suborbital rocket','success',837,'NASA 14.378IE','ISRO 10.18','TERLS-073','','Ionos',1467,'',211 from dual union all
select 17223,'1968-S248','1968-03-31 12:15:00','suborbital rocket','success',837,'NASA 14.377IE-II','ISRO 10.17','TERLS-074','','Ionos',1467,'',169 from dual union all
select 17224,'1968-S249','1968-04-01 03:26:00','suborbital rocket','success',447,'D-253','FU-171 VLF/Ions','','','Ionos',856,'',416 from dual union all
select 17225,'1968-S25','1968-10-22 00:00:00','military missile','success',982,'T 22504-14T','8K64U UBP-120','','','OT',596,'',1210 from dual union all
select 17226,'1968-S250','1968-04-02 00:00:00','military missile','success',1420,'B-36','Mk 6 RV','FOT GT10T','GLORY TRIP 10T','OT',1658,'',1300 from dual union all
select 17227,'1968-S251','1968-04-03 00:38:00','suborbital rocket','success',288,'C158','Rehbar 18','','','Aeron',1419,'',119 from dual union all
select 17228,'1968-S252','1968-04-03 13:48:00','suborbital rocket','success',227,'AAF-VB-05','Auroral fluxes','','','Auroral/Ionos/Plasm',428,'',316 from dual union all
select 17229,'1968-S253','1968-04-04 03:59:00','suborbital rocket','success',837,'NASA 14.277GE','Auroral','','','Auroral/Plasma',428,'',129 from dual union all
select 1723,'1953-A305','1953-07-24 00:00:00','atmospheric rocket','success',560,'','F-102 Model','','','Test',1821,'',10 from dual union all
select 17230,'1968-S254','1968-04-04 07:32:00','suborbital rocket','success',867,'NASA 18.60GE','Auroral','','','Auroral/Plasma',428,'',270 from dual union all
select 17231,'1968-S255','1968-04-06 01:15:46','suborbital rocket','failure',837,'K-NA-10','DLR K-NA-10','','','Auroral?',413,'',1 from dual union all
select 17232,'1968-S256','1968-04-06 02:45:00','suborbital rocket','success',219,'AKF-4-10','Calgary CR/ionos','','','Ionos/CR',428,'',862 from dual union all
select 17233,'1968-S257','1968-04-07 05:45:00','suborbital rocket','success',30,'NRL NB3.204','Stellar XR Scan','','','XR Astron',1869,'',158 from dual union all
select 17234,'1968-S258','1968-04-09 00:00:00','military missile','unknown',1102,'','S01V-4','','','Test',301,'',500 from dual union all
select 17235,'1968-S259','1968-04-09 00:00:00','suborbital rocket','success',1493,'AGI 62','FU-184 Sea recovery','','','Test',341,'',114 from dual union all
select 17236,'1968-S26','1968-11-02 00:00:00','military missile','success',982,'G 22604-12T','8K64U UBP-121','','','OT',596,'',1210 from dual union all
select 17237,'1968-S260','1968-04-09 21:26:00','suborbital rocket','success',624,'NASA 8.40UA','EUV Dayglow','','','Aeronomy',1821,'',856 from dual union all
select 17238,'1968-S261','1968-04-10 08:39:00','military missile','success',795,'1411','Mk 11A RV','ST PACER KITE 2','OLYMPIC TRIALS B-2','OT',1698,'',900 from dual union all
select 17239,'1968-S262','1968-04-10 00:00:00','military missile','unknown',932,'A3P','FOT','','','OT',421,'SSBN 630',1000 from dual union all
select 1724,'1953-A306','1953-07-24 16:40:00','suborbital rocket','failure',370,'MUSHRAT 5','SUI 12','','','CR',46,'AGB-5',11 from dual union all
select 17240,'1968-S263','1968-04-10 00:00:00','military missile','unknown',932,'A3P','FOT','','','OT',421,'SSBN 630',1000 from dual union all
select 17241,'1968-S264','1968-04-10 00:00:00','military missile','unknown',932,'A3P','FOT','','','OT',421,'SSBN 630',1000 from dual union all
select 17242,'1968-S265','1968-04-10 00:00:00','military missile','unknown',932,'A3P','FOT','','','OT',421,'SSBN 630',1000 from dual union all
select 17243,'1968-S266','1968-04-16 03:58:00','suborbital rocket','success',215,'AKD-2A-113','Auroral breakup','','','Auroral/Aeron/Ionos',428,'',155 from dual union all
select 17244,'1968-S267','1968-04-18 07:55:00','military missile','success',155,'77E','WT1A-L4/WT1A-S1','ABRES RVTO-1A-1','','Reentry Test',1680,'',1600 from dual union all
select 17245,'1968-S268','1968-04-18 03:30:00','military missile','success',1000,'Ya22500-03L','8K67M LKI-1','GCh 8F675-7000','','Test',511,'',1000 from dual union all
select 17246,'1968-S269','1968-04-18 05:42:00','suborbital rocket','success',215,'AKD-2A-110','Auroral breakup','','','Auroral/Aeron/Ionos',428,'',152 from dual union all
select 17247,'1968-S27','1968-11-12 00:00:00','military missile','success',982,'T 22503-17T','8K64U UBP-122','','','OT',596,'',1210 from dual union all
select 17248,'1968-S270','1968-04-18 06:14:00','suborbital rocket','success',215,'AKD-2A-112','Auroral breakup','','','Auroral/Aeron/Ionos',428,'',155 from dual union all
select 17249,'1968-S271','1968-04-20 17:38:00','sounding rocket','success',129,'NASA 16.04UI','','','','Ionospheric',1811,'',1250 from dual union all
select 1725,'1953-A307','1953-07-27 00:00:00','military missile','failure',255,'623-2','','','','Test',254,'',20 from dual union all
select 17250,'1968-S272','1968-04-22 15:53:00','suborbital rocket','success',837,'NASA 14.241IG','ISRO 40.01','TERLS-076','','XR Astron',1467,'',159 from dual union all
select 17251,'1968-S273','1968-04-24 03:05:00','military missile','success',795,'892','Mk 11 RV','ST Cold/Heat Soak 4?','GLASS POLE','OT',1701,'',900 from dual union all
select 17252,'1968-S274','1968-04-23 04:47:00','suborbital rocket','success',215,'AMF-2A-115','(dup for 116)','','','Auroral/Aeron/Ionos',428,'',337 from dual union all
select 17253,'1968-S275','1968-04-23 04:53:00','suborbital rocket','success',867,'NASA 18.41UE','','','','Auroral',428,'',268 from dual union all
select 17254,'1968-S276','1968-04-23 11:18:00','suborbital rocket','unknown',1189,'INTA MP-6804','','','','Ionos',396,'',100 from dual union all
select 17255,'1968-S277','1968-04-23 23:06:00','suborbital rocket','success',30,'NASA 04.238UA','Airglow','','','Aeronomy',1869,'',185 from dual union all
select 17256,'1968-S278','1968-04-24 10:29:00','suborbital rocket','unknown',1189,'INTA MP-6805','','','','Ionos',396,'',84 from dual union all
select 17257,'1968-S279','1968-04-24 12:37:00','suborbital rocket','unknown',1189,'INTA MP-6806','','','','Ionos',396,'',81 from dual union all
select 17258,'1968-S28','1968-12-04 09:00:00','military missile','success',1018,'G 15003-03','8K75 UBP-13','','','OT',578,'',1160 from dual union all
select 17259,'1968-S280','1968-04-24 15:44:00','suborbital rocket','success',837,'NASA 14.260IG','ISRO 40.02','TERLS-078','','XR Astron',1467,'',159 from dual union all
select 1726,'1953-A308','1953-07-28 00:00:00','atmospheric rocket','success',836,'1249B-1221','Nike 232R','','','Test',1895,'',10 from dual union all
select 17260,'1968-S281','1968-04-25 17:06:00','suborbital rocket','success',837,'Round 121','Speedball II S/N 54','','','Target',1895,'',200 from dual union all
select 17261,'1968-S282','1968-04-25 06:01:00','suborbital rocket','success',867,'NASA 18.33CE','p/e flux','','','Ionos',428,'',120 from dual union all
select 17262,'1968-S283','1968-04-25 06:23:00','suborbital rocket','success',867,'NASA 18.42UE','','','','Auroral',428,'',262 from dual union all
select 17263,'1968-S284','1968-04-26 00:00:00','military missile','unknown',932,'A3P','FOT','','','OT',421,'SSBN 657',1000 from dual union all
select 17264,'1968-S285','1968-04-26 00:00:00','military missile','unknown',932,'A3P','FOT','','','OT',421,'SSBN 657',1000 from dual union all
select 17265,'1968-S286','1968-04-26 00:00:00','military missile','unknown',932,'A3P','FOT','','','OT',421,'SSBN 657',1000 from dual union all
select 17266,'1968-S287','1968-04-26 00:00:00','military missile','unknown',932,'A3P','FOT','','','OT',421,'SSBN 657',1000 from dual union all
select 17267,'1968-S288','1968-04-27 09:41:00','military missile','success',161,'78E','RMV-320/Decoys','ABRES RMP-B-7/NTMP KX107','','Reentry Test',1682,'',1600 from dual union all
select 17268,'1968-S289','1968-04-27 05:19:00','test rocket','success',1150,'S164C','Reentry F','','','Reentry Test',1814,'',175 from dual union all
select 17269,'1968-S29','1968-01-01 00:00:00','military missile','success',1036,'5','','','','Test',636,'',1000 from dual union all
select 1727,'1953-A309','1953-08-03 18:28:00','suborbital rocket','unknown',370,'MUSHRAT 7','SUI 14','','','CR',48,'AGB-5',11 from dual union all
select 17270,'1968-S290','1968-04-27 15:05:00','suborbital rocket','success',30,'NASA 04.244DS','NRL ND3.212','','','Solar UV',1869,'',166 from dual union all
select 17271,'1968-S291','1968-04-28 04:10:00','military missile','success',999,'V 22502-15T','8K67 UBP-21','','','OT',511,'',1000 from dual union all
select 17272,'1968-S292','1968-04-30 00:00:00','military missile','success',916,'AO-63','P-147 68 Spring 05','','','OT',435,'',180 from dual union all
select 17273,'1968-S293','1968-04-29 15:00:00','suborbital rocket','success',30,'NASA 04.245DS','NRL ND3.213','','','Solar UV',1869,'',169 from dual union all
select 17274,'1968-S294','1968-04-29 22:04:00','suborbital rocket','success',1189,'AMD-S2-8','OH profile','','','Aeron/CR',428,'',109 from dual union all
select 17275,'1968-S295','1968-05-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 17276,'1968-S296','1968-05-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 17277,'1968-S297','1968-04-30 00:00:00','military missile','success',796,'2503','Mk 11 RV','DASO GIANT BLADE 1','GIANT BLADE 1','OT',1708,'',900 from dual union all
select 17278,'1968-S298','1968-04-30 07:49:00','suborbital rocket','failure',867,'NASA 18.40UE','','','','Auroral',428,'',109 from dual union all
select 17279,'1968-S299','1968-05-01 20:25:00','suborbital rocket','success',848,'CRL AH07.177','Sphere','','','Aeron',394,'',217 from dual union all
select 1728,'1953-A31','1953-01-23 00:00:00','atmospheric rocket','success',836,'1249A-177','Nike 1249 R&D 167R','','','Test',1895,'',10 from dual union all
select 17280,'1968-S30','1968-01-01 00:00:00','military missile','success',1036,'6','','','','Test',636,'',1000 from dual union all
select 17281,'1968-S300','1968-05-02 05:25:00','suborbital rocket','success',848,'CRL AH07.178','Sphere','','','Aeron',394,'',216 from dual union all
select 17282,'1968-S301','1968-05-02 06:00:00','suborbital rocket','success',848,'CRL AE07.325','Elec backscatter','','',' Ionos',394,'',212 from dual union all
select 17283,'1968-S302','1968-05-02 07:15:00','suborbital rocket','success',848,'CRL AG07.644','Release','','','Aeron',1483,'',218 from dual union all
select 17284,'1968-S303','1968-05-03 14:08:00','military missile','failure',162,'95F','TVX-22/Decoys','ABRES Penaid TVX-22','','Reentry Test',1681,'',1 from dual union all
select 17285,'1968-S304','1968-05-03 05:11:00','suborbital rocket','success',867,'NASA 18.39UE','','','','Auroral',428,'',258 from dual union all
select 17286,'1968-S305','1968-05-03 07:15:00','suborbital rocket','success',30,'NASA 04.227UG','Stellar UV Spectra','','','UV Astron',1869,'',171 from dual union all
select 17287,'1968-S306','1968-05-07 05:10:00','suborbital rocket','success',219,'NASA 19.04II','Isis test','','','Test/Ionospheric',1821,'',820 from dual union all
select 17288,'1968-S307','1968-05-07 07:00:00','suborbital rocket','success',848,'CRL AG07.643','Release','','','Aeron',1483,'',142 from dual;
insert into LAUNCH
select 17289,'1968-S308','1968-05-08 16:25:00','suborbital rocket','success',837,'Round 122','Speedball II S/N 55','','','Aeron?',1895,'',200 from dual union all
select 1729,'1953-A310','1953-08-04 00:00:00','military missile','unknown',339,'1269E','EU-7','','','Test',1895,'',30 from dual union all
select 17290,'1968-S309','1968-05-09 06:15:00','suborbital rocket','success',864,'Sandia 152-114','LRL BOX-14','','','XR Astron',855,'',178 from dual union all
select 17291,'1968-S31','1968-01-01 00:00:00','military missile','success',1036,'7','','','','Test',636,'',1000 from dual union all
select 17292,'1968-S310','1968-05-09 07:30:00','suborbital rocket','success',740,'','','','','Aeron',117,'VISE',162 from dual union all
select 17293,'1968-S311','1968-05-11 01:00:00','suborbital rocket','success',740,'','','','','Aeron/Ionos',118,'VISE',167 from dual union all
select 17294,'1968-S312','1968-05-11 13:00:00','suborbital rocket','success',740,'','','','','Aeron/Ionos',118,'VISE',167 from dual union all
select 17295,'1968-S313','1968-05-12 00:00:00','military missile','success',1000,'V 22500-04L','8K67M LKI-2','','','Test',511,'',1000 from dual union all
select 17296,'1968-S314','1968-05-12 09:06:00','suborbital rocket','success',867,'CRL AH08.670','Release','','','Aeron',1483,'',198 from dual union all
select 17297,'1968-S315','1968-05-12 13:15:00','suborbital rocket','success',740,'','','','','Aeron/Ionos',118,'VISE',171 from dual union all
select 17298,'1968-S316','1968-05-13 09:06:00','suborbital rocket','success',867,'CRL AH08.666','Ba release','','','Aeron',1483,'',198 from dual union all
select 17299,'1968-S317','1968-05-14 00:00:00','suborbital rocket','success',204,'306','FU-193','','','Test',1343,'',130 from dual union all
select 173,'1944-S157','1944-09-27 10:50:00','military missile','success',8,'18837','Ma208','','','Training',661,'',90 from dual union all
select 1730,'1953-A311','1953-08-04 00:00:00','atmospheric rocket','success',365,'','Rascal Model','','','Test',1821,'',10 from dual union all
select 17300,'1968-S318','1968-05-14 07:05:00','military missile','success',1373,'205','AFP-437 CEL','','','Interceptor',772,'',823 from dual union all
select 17301,'1968-S319','1968-05-15 00:38:00','suborbital rocket','success',1301,'TACITE 02','Earth IR','','','Aeron',313,'',175 from dual union all
select 17302,'1968-S32','1968-01-01 00:00:00','military missile','success',1036,'8','','','','Test',636,'',1000 from dual union all
select 17303,'1968-S320','1968-05-15 06:38:00','suborbital rocket','success',864,'Sandia 152-111','LRL BOX-8b','','','XR Astron',846,'',162 from dual union all
select 17304,'1968-S321','1968-05-16 18:33:00','suborbital rocket','success',740,'','','','','Aeron',118,'VISE',168 from dual union all
select 17305,'1968-S322','1968-05-17 03:45:00','suborbital rocket','success',30,'KP3.24','UV Tel (Jupiter)','','','UV Astron',1869,'',159 from dual union all
select 17306,'1968-S323','1968-05-17 04:25:00','suborbital rocket','success',30,'NASA 04.221GG','Stellar UV','','','UV Astron',1869,'',165 from dual union all
select 17307,'1968-S324','1968-05-17 08:00:00','suborbital rocket','success',839,'Sandia 154-106','AFCRL Instr. Sphere','','','Aeron',855,'',110 from dual union all
select 17308,'1968-S325','1968-05-17 09:15:00','suborbital rocket','success',839,'Sandia 154-110','Falling sphere','','','Aeron',855,'',144 from dual union all
select 17309,'1968-S326','1968-05-17 18:25:00','suborbital rocket','success',740,'','','','','Aeron',118,'VISE',159 from dual union all
select 1731,'1953-A312','1953-08-04 00:00:00','military missile','success',771,'GM-11081','','','','Test',222,'',12 from dual union all
select 17310,'1968-S327','1968-05-17 09:00:00','suborbital rocket','success',848,'CRL AH07.571','Sphere','','','Aeron',855,'',202 from dual union all
select 17311,'1968-S328','1968-05-18 00:30:00','suborbital rocket','success',839,'Sandia 154-111','Falling sphere','','','Aeron',855,'',152 from dual union all
select 17312,'1968-S329','1968-05-18 01:33:00','suborbital rocket','success',867,'Sandia 152-112','Ion gages','','','Aeron',855,'',300 from dual union all
select 17313,'1968-S33','1968-01-01 00:00:00','military missile','success',1036,'9','','','','Test',636,'',1000 from dual union all
select 17314,'1968-S330','1968-05-18 18:05:00','suborbital rocket','success',740,'','','','','Aeron',118,'VISE',167 from dual union all
select 17315,'1968-S331','1968-05-19 06:16:00','suborbital rocket','success',864,'Sandia 152-108','LRL ACS-1','','','XR Astron',855,'',147 from dual union all
select 17316,'1968-S332','1968-05-20 15:00:00','suborbital rocket','success',30,'NASA 04.230GS','UV solar imager','','','Solar',1869,'',165 from dual union all
select 17317,'1968-S333','1968-05-20 15:51:20','suborbital rocket','success',290,'CE-38','ESRO C42/1','','','Meteorites',408,'',135 from dual union all
select 17318,'1968-S334','1968-05-21 14:00:00','suborbital rocket','success',740,'','MX6407P','','','Aeron/Ionos',118,'VISE',174 from dual union all
select 17319,'1968-S335','1968-05-21 20:09:00','suborbital rocket','success',740,'','','','','Aeron',118,'VISE',167 from dual union all
select 1732,'1953-A313','1953-08-04 00:00:00','atmospheric rocket','success',836,'1249B-1201','Nike 233R','','','Test',1895,'',10 from dual union all
select 17320,'1968-S336','1968-05-22 10:20:00','military missile','success',795,'1359','Mk 11A RV','ST PACER KITE 3','OLYMPIC TRIALS B-3','OT',1698,'',900 from dual union all
select 17321,'1968-S337','1968-05-22 00:00:00','military missile','success',1251,'800004','KT-2','','','Test',868,'',100 from dual union all
select 17322,'1968-S338','1968-05-22 00:50:00','suborbital rocket','success',740,'','MX6407P','','','Aeron/Ionos',118,'VISE',169 from dual union all
select 17323,'1968-S339','1968-05-22 16:28:44','suborbital rocket','failure',446,'D-31','ESRO D30/2','','','Ionos/Solar X',1343,'',225 from dual union all
select 17324,'1968-S34','1968-01-01 00:00:00','military missile','success',1036,'10','','','','Test',636,'',1000 from dual union all
select 17325,'1968-S340','1968-05-23 05:10:00','military missile','success',795,'1015','Mk 11 RV','ST Cold/Heat Soak 5?','OLD FAITHFUL','OT',1701,'',900 from dual union all
select 17326,'1968-S341','1968-05-23 00:00:00','military missile','success',1000,'V 22500-05L','8K67M LKI-3','','','Test',511,'',1000 from dual union all
select 17327,'1968-S342','1968-05-23 21:00:00','suborbital rocket','success',848,'CRL AH07.572','Sphere','','','Aeron',855,'',201 from dual union all
select 17328,'1968-S343','1968-05-23 19:00:00','suborbital rocket','success',839,'Sandia 154-102','Falling sphere','','','Aeron',855,'',110 from dual union all
select 17329,'1968-S344','1968-05-23 20:00:00','suborbital rocket','success',839,'Sandia 154-103','AFCRL Instr. sphere','','','Aeron',855,'',110 from dual union all
select 1733,'1953-A314','1953-08-06 00:00:00','military missile','unknown',339,'1300R','','','','Test',1895,'',30 from dual union all
select 17330,'1968-S345','1968-05-23 21:00:00','suborbital rocket','success',837,'Sandia 154-104','Pressure probe','','','Aeron',855,'',100 from dual union all
select 17331,'1968-S346','1968-05-24 00:10:00','suborbital rocket','success',839,'Sandia 154-105','Falling sphere','','','Aeron',855,'',110 from dual union all
select 17332,'1968-S347','1968-05-24 00:15:00','suborbital rocket','success',867,'Sandia 154-113','Ion gages','','','Aeron',855,'',300 from dual union all
select 17333,'1968-S348','1968-05-24 02:45:00','suborbital rocket','success',867,'Sandia 152-115','O2 absorption','','','Aeron',855,'',160 from dual union all
select 17334,'1968-S349','1968-05-25 05:19:00','suborbital rocket','success',624,'NASA 8.46UI','VLF radio noise','','','Ionos',428,'',805 from dual union all
select 17335,'1968-S35','1968-01-01 00:00:00','military missile','success',1036,'11','','','','Test',636,'',1000 from dual union all
select 17336,'1968-S350','1968-05-25 06:40:00','suborbital rocket','success',30,'NASA 04.173UG','Stellar UV','','','UV Astron',1869,'',169 from dual union all
select 17337,'1968-S351','1968-05-29 13:15:00','suborbital rocket','success',920,'P17H','Langmuir probe','','','Ionospheric',1447,'',140 from dual union all
select 17338,'1968-S352','1968-05-30 13:32:00','suborbital rocket','success',740,'','','','','Aeron',636,'',172 from dual union all
select 17339,'1968-S353','1968-05-30 21:00:00','suborbital rocket','success',740,'','','','','Auroral',636,'',168 from dual union all
select 1734,'1953-A315','1953-08-06 00:00:00','military missile','unknown',339,'1301R','','','','Test',1895,'',30 from dual union all
select 17340,'1968-S354','1968-05-31 08:49:00','suborbital rocket','success',1199,'SL762','UCL GT-1 Gren/TMA','','','Aeronomy',1839,'',242 from dual union all
select 17341,'1968-S355','1968-05-31 20:39:00','suborbital rocket','success',1199,'SL761','UCL GT-2','','','Aeronomy',1839,'',238 from dual union all
select 17342,'1968-S356','1968-06-01 00:00:00','military missile','success',162,'89F','Mk 12 RV Pre-Damaged','ABRES PDV','','Reentry Test',1681,'',1400 from dual union all
select 17343,'1968-S357','1968-06-01 17:50:00','suborbital rocket','success',920,'P18H','Langmuir probe','','','Ionospheric',1447,'',131 from dual union all
select 17344,'1968-S358','1968-06-02 09:15:00','suborbital rocket','success',837,'SECEDE R1','SECEDE I APPLE','','','Aeron/Fields',1483,'',196 from dual union all
select 17345,'1968-S359','1968-06-04 16:37:00','suborbital rocket','success',837,'NASA 14.363GT','Recovery test','','','Test',1895,'',130 from dual union all
select 17346,'1968-S36','1968-01-01 00:00:00','military missile','success',1036,'12','','','','Test',636,'',1000 from dual union all
select 17347,'1968-S360','1968-06-06 17:40:00','military missile','success',999,'Ya22504-08T','8K67 UBP-22','GCh 8F674 Special Test','','Test',514,'',1000 from dual union all
select 17348,'1968-S361','1968-06-06 13:31:00','suborbital rocket','success',740,'','','','','Aeron/Ionos',636,'',174 from dual union all
select 17349,'1968-S362','1968-06-06 18:00:00','suborbital rocket','success',740,'','','','','Aeron/Plasma',636,'',171 from dual union all
select 1735,'1953-A316','1953-08-06 00:00:00','military missile','success',1214,'N-69A GM-3391','','','','Test',222,'',10 from dual union all
select 17350,'1968-S363','1968-06-07 15:25:45','suborbital rocket','success',290,'CE-44','ESRO C42/2','','','Meteorites',408,'',138 from dual union all
select 17351,'1968-S364','1968-06-08 17:41:00','suborbital rocket','success',30,'NASA 04.263CS','AS&E Solar','','','Solar EUV/X',1869,'',151 from dual union all
select 17352,'1968-S365','1968-06-08 18:38:00','suborbital rocket','success',30,'NASA 04.134DS','NRL NE3.174','','','Solar EUV',1869,'',179 from dual union all
select 17353,'1968-S366','1968-06-08 21:01:16','suborbital rocket','success',290,'CE-32','ESRO C32/2','','','Meteorites',408,'',135 from dual union all
select 17354,'1968-S367','1968-06-10 00:25:00','suborbital rocket','success',837,'NASA 14.222GE','SPICE (p/n)','','','Plasma',428,'',149 from dual union all
select 17355,'1968-S368','1968-06-10 15:49:00','suborbital rocket','failure',226,'CRL AH17.751D','D region protons','','','Ionos',428,'',112 from dual union all
select 17356,'1968-S369','1968-06-10 16:05:00','suborbital rocket','success',837,'NASA 14.223GE','SPICE','','','Plasma',428,'',150 from dual union all
select 17357,'1968-S370','1968-06-10 17:15:00','suborbital rocket','success',837,'NASA 14.335UA','Pandora C','','','Aeronomy',1895,'',152 from dual union all
select 17358,'1968-S371','1968-06-11 16:00:01','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 611',1000 from dual union all
select 17359,'1968-S372','1968-06-11 17:21:02','suborbital rocket','success',837,'NASA 14.355IA','DLR K-NA-13','','','Meteorites',413,'',183 from dual union all
select 1736,'1953-A317','1953-08-06 15:07:00','suborbital rocket','success',370,'MUSHRAT 10','SUI 15','','','CR',49,'AGB-5',68 from dual union all
select 17360,'1968-S373','1968-06-11 23:30:00','suborbital rocket','success',219,'NASA 19.01NE','SAAP','','','Magnetospheric',330,'',790 from dual union all
select 17361,'1968-S374','1968-06-12 00:00:00','military missile','success',1420,'B-82','Mk 6 RV','FOT GT08T','GLORY TRIP 08T','OT',1658,'',1300 from dual union all
select 17362,'1968-S375','1968-06-12 09:08:00','suborbital rocket','success',837,'SECEDE R2','SECEDE I DOGWOOD','','','Aeron/Fields',1483,'',188 from dual union all
select 17363,'1968-S376','1968-06-12 10:30:00','suborbital rocket','success',1197,'SL723','LUX XRS4 M87 Survey','','','XR Astronomy',1839,'',184 from dual union all
select 17364,'1968-S377','1968-06-12 22:59:00','suborbital rocket','success',837,'NASA 14.354IA','DLR K-NA-12','','','Meteorites',413,'',183 from dual union all
select 17365,'1968-S378','1968-06-14 14:55:00','suborbital rocket','success',740,'','','','','Aeron/CR',636,'',169 from dual union all
select 17366,'1968-S379','1968-06-18 18:07:00','military missile','failure',930,'A2E','','','','OT',421,'SSBN 610',10 from dual union all
select 17367,'1968-S380','1968-06-21 00:00:00','military missile','success',916,'AO-64','P-148 68 Summer 01','','','Test',597,'',180 from dual union all
select 17368,'1968-S381','1968-06-21 00:00:00','military missile','success',916,'AO-65','P-149 68 Summer 02','','','Test',597,'',180 from dual union all
select 17369,'1968-S382','1968-06-22 00:00:00','military missile','success',168,'86F','RMV-340C','ABRES RMP-B-8/NTMP KX108','','Reentry Test',1682,'',1400 from dual union all
select 1737,'1953-A318','1953-08-09 19:15:00','suborbital rocket','failure',370,'MUSHRAT 14','NRL Rockoon 3','','','Aeron',42,'AGB-5',38 from dual union all
select 17370,'1968-S383','1968-06-24 22:51:00','suborbital rocket','success',35,'NASA 04.214NB','Rat centrifuge 2','','','Biology',1821,'',144 from dual union all
select 17371,'1968-S384','1968-06-26 04:30:00','suborbital rocket','success',740,'','','','','Aeron',636,'',173 from dual union all
select 17372,'1968-S385','1968-06-26 09:21:00','suborbital rocket','success',740,'','','','','Aeron',636,'',168 from dual union all
select 17373,'1968-S386','1968-06-28 04:37:00','suborbital rocket','success',740,'','','','','Aeron',636,'',174 from dual union all
select 17374,'1968-S387','1968-06-29 10:12:00','military missile','success',162,'32F','WT1A-L1/WT1A-S2','ABRES RVTO-1A-2','','Reentry Test',1680,'',1400 from dual union all
select 17375,'1968-S388','1968-07-19 00:00:00','military missile','success',916,'AO-66','P-150 68 Summer 03','','','Test',597,'',180 from dual union all
select 17376,'1968-S389','1968-07-19 00:00:00','military missile','success',916,'AO-67','P-151 68 Summer 04','','','Test',597,'',180 from dual union all
select 17377,'1968-S390','1968-07-03 00:00:00','suborbital rocket','success',740,'','','','','Aeron/CR',636,'',179 from dual union all
select 17378,'1968-S391','1968-07-03 00:00:00','military missile','success',1102,'','S01V-5','','','Test',301,'',500 from dual;
insert into LAUNCH
select 17379,'1968-S392','1968-07-03 06:20:00','military missile','success',999,'V 22503-15T','8K67 UBP-23','GCh 8F675','','OT',511,'',1000 from dual union all
select 1738,'1953-A319','1953-08-11 00:00:00','atmospheric rocket','success',836,'1249B-1215','Nike 234R','','','Test',1895,'',10 from dual union all
select 17380,'1968-S393','1968-07-08 18:19:00','military missile','success',796,'FTM-2589','Mk 11 RV','DASO GIANT FIST 1','GIANT FIST 1','OT',1699,'',900 from dual union all
select 17381,'1968-S394','1968-07-08 06:05:00','suborbital rocket','success',1207,'SL403','LUX XRS5 M87 Survey','','','XR Astronomy',1839,'',180 from dual union all
select 17382,'1968-S395','1968-07-11 16:00:00','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 610',1000 from dual union all
select 17383,'1968-S396','1968-07-12 09:00:00','military missile','success',795,'1084 (343)','Mk 11A RV','ST OLYMPIC TRIALS B-4','OLYMPIC TRIALS B-4','OT',1698,'',900 from dual union all
select 17384,'1968-S397','1968-07-16 20:36:00','military missile','success',1000,'V 22501-07T','8K67M UBP-1','GCh 8F675','','OT',514,'',1000 from dual union all
select 17385,'1968-S398','1968-07-16 17:40:00','suborbital rocket','success',740,'','Na release','','','Aeron',636,'',146 from dual union all
select 17386,'1968-S399','1968-07-18 01:30:00','suborbital rocket','success',848,'CRL AG07.881','Mass spectrometer','','','Meteorites',394,'',125 from dual union all
select 17387,'1968-S400','1968-07-19 16:00:00','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 611',1000 from dual union all
select 17388,'1968-S401','1968-07-19 17:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',148 from dual union all
select 17389,'1968-S402','1968-07-19 18:33:00','suborbital rocket','failure',848,'CRL AG07.642','Accidental launch','','','Aeron',394,'',1 from dual union all
select 1739,'1953-A32','1953-01-27 00:00:00','atmospheric rocket','success',836,'301P','Nike 301P CET 1','','','Test',1895,'',10 from dual union all
select 17390,'1968-S403','1968-07-22 04:23:00','suborbital rocket','success',848,'CRL AF07.389','Mass spectrometer','','','Aeron',394,'',144 from dual union all
select 17391,'1968-S404','1968-07-23 00:00:00','suborbital rocket','success',448,'D-301','FU-188 Tech','','','Test',307,'',562 from dual union all
select 17392,'1968-S405','1968-07-23 04:18:00','suborbital rocket','success',848,'CRL AG07.628','TMA','','','Aeron',394,'',161 from dual union all
select 17393,'1968-S406','1968-07-23 04:19:00','suborbital rocket','success',848,'CRL AG07.629','TMA','','','Aeron',394,'',202 from dual union all
select 17394,'1968-S407','1968-07-23 04:28:00','suborbital rocket','success',848,'CRL AG07.642A','TMA','','','Aeron',394,'',200 from dual union all
select 17395,'1968-S408','1968-07-23 04:28:00','suborbital rocket','success',848,'CRL AG07.632','TMA','','','Aeron',394,'',150 from dual union all
select 17396,'1968-S409','1968-07-24 00:19:00','suborbital rocket','success',837,'NASA 14.349UA','NO Composition','','','Aeronomy',1821,'',106 from dual union all
select 17397,'1968-S410','1968-07-24 00:46:00','suborbital rocket','success',839,'NASA 10.265GM','GSFC Grenade 181a W','','','Aeronomy',1821,'',126 from dual union all
select 17398,'1968-S411','1968-07-24 01:41:00','suborbital rocket','success',1197,'SL523','UCW-14','','','Ionospheric',1839,'',258 from dual union all
select 17399,'1968-S412','1968-07-24 05:20:00','suborbital rocket','success',839,'NASA 10.253UA','UM Sphere 1/2','','','Aeronomy',1821,'',155 from dual union all
select 174,'1944-S158','1944-10-01 00:00:00','military missile','success',1470,'','Paris','','','Weapon',1739,'',90 from dual union all
select 1740,'1953-A320','1953-08-11 00:00:00','atmospheric rocket','success',836,'1208E','','','','Test',1895,'',10 from dual union all
select 17400,'1968-S413','1968-07-24 09:54:00','suborbital rocket','success',837,'NASA 14.348UA','NO Composition','','','Aeronomy',1821,'',105 from dual union all
select 17401,'1968-S414','1968-07-24 10:06:00','suborbital rocket','success',837,'NASA 14.358UI','UIUC-GCA Ionos','','','Aeronomy',1821,'',206 from dual union all
select 17402,'1968-S415','1968-07-24 10:19:00','suborbital rocket','success',839,'NASA 10.258GM','GSFC Grenade 181b W','','','Aeronomy',1821,'',117 from dual union all
select 17403,'1968-S416','1968-07-24 10:36:00','suborbital rocket','success',837,'NASA 14.359UI','UIUC-GCA Ionos','','','Ionos',1821,'',211 from dual union all
select 17404,'1968-S417','1968-07-24 17:00:00','suborbital rocket','success',837,'NASA 14.360UI','UIUC-GCA Ionos','','','Ionos',1821,'',214 from dual union all
select 17405,'1968-S418','1968-07-24 18:45:00','suborbital rocket','success',839,'NASA 10.254UA','UM Sphere 1/2','','','Aeronomy',1821,'',159 from dual union all
select 17406,'1968-S419','1968-07-24 21:36:00','suborbital rocket','success',837,'NASA 14.361UI','UIUC-GCA Ionos','','','Ionos',1821,'',206 from dual union all
select 17407,'1968-S42','1968-03-22 00:00:00','military missile','success',1037,'Ya23105-32L','8K98 LKI-2-18','','','Test',477,'',1000 from dual union all
select 17408,'1968-S420','1968-07-24 21:55:00','suborbital rocket','success',839,'NASA 10.266GM','GSFC Grenade 182 W','','','Aeronomy',1821,'',84 from dual union all
select 17409,'1968-S421','1968-07-25 00:00:00','suborbital rocket','success',1492,'61M/89','FU-185','','','XR Astron',341,'',185 from dual union all
select 1741,'1953-A321','1953-08-12 00:00:00','atmospheric rocket','unknown',576,'325','','','','Test',1895,'',10 from dual union all
select 17410,'1968-S422','1968-07-25 17:30:00','suborbital rocket','success',740,'','Na release','','','Aeron',636,'',145 from dual union all
select 17411,'1968-S423','1968-07-26 00:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',147 from dual union all
select 17412,'1968-S424','1968-07-26 00:00:00','military missile','unknown',1102,'','S01V-6','','','Test',301,'',500 from dual union all
select 17413,'1968-S425','1968-07-27 03:55:00','suborbital rocket','success',30,'NASA 04.236UG','CAL XR 2','','','XR Astron',1869,'',142 from dual union all
select 17414,'1968-S426','1968-07-27 04:30:00','suborbital rocket','success',30,'NASA 04.225UG','MIT-5 GC Survey II','','','XR Astron',1869,'',135 from dual union all
select 17415,'1968-S427','1968-07-27 17:26:00','suborbital rocket','success',740,'','','','','Aeron',636,'',150 from dual union all
select 17416,'1968-S428','1968-07-30 16:07:00','military missile','success',930,'A2E','','','','OT',421,'SSBN 618',1000 from dual union all
select 17417,'1968-S429','1968-07-30 22:53:30','suborbital rocket','success',290,'CE-31','ESRO C20/2','','','Aeronomy',408,'',135 from dual union all
select 17418,'1968-S43','1968-03-29 00:00:00','military missile','failure',1037,'','8K98 LKI-2-19','','','Test',477,'',1000 from dual union all
select 17419,'1968-S430','1968-08-03 00:00:00','military missile','success',916,'AO-68','P-152 68 Summer 05','','','Test',597,'',180 from dual union all
select 1742,'1953-A322','1953-08-12 00:00:00','atmospheric rocket','unknown',576,'324','','','','Test',1895,'',10 from dual union all
select 17420,'1968-S431','1968-08-05 00:00:00','military missile','success',917,'P1A','P-154 68 Summer 07','','','Test',597,'',180 from dual union all
select 17421,'1968-S432','1968-08-07 00:00:00','military missile','success',917,'P1A','P-155 68 Summer 08','','','Test',597,'',180 from dual union all
select 17422,'1968-S433','1968-08-07 00:00:00','military missile','success',917,'P1A','P-156 68 Summer 09','','','Test',597,'',180 from dual union all
select 17423,'1968-S434','1968-07-31 13:00:00','suborbital rocket','success',740,'','Electron fluxes','','','CR/Plasma',636,'',178 from dual union all
select 17424,'1968-S435','1968-07-31 18:54:00','suborbital rocket','success',740,'','Electron fluxes','','','CR/Plasma',636,'',173 from dual union all
select 17425,'1968-S436','1968-07-31 21:21:00','suborbital rocket','success',867,'NASA 18.31UA','JHU Airglow 9','','','Aeronomy',1821,'',300 from dual union all
select 17426,'1968-S437','1968-08-01 00:00:00','suborbital rocket','success',740,'','Electron fluxes','','','Aeron/Plasma',636,'',168 from dual union all
select 17427,'1968-S438','1968-08-01 02:00:00','suborbital rocket','success',740,'','Electron fluxes','','','Plasma/Fields',636,'',176 from dual union all
select 17428,'1968-S439','1968-08-01 09:10:00','suborbital rocket','success',30,'NASA 04.224NA','Luster NLC','','','Aeronomy/Meteorites',428,'',115 from dual union all
select 17429,'1968-S440','1968-08-01 10:27:00','suborbital rocket','success',920,'P13H','Solar X/La/ne','','','Ionos/Solar X',1447,'',145 from dual union all
select 1743,'1953-A323','1953-08-18 00:00:00','military missile','failure',339,'1297E','EU-8','','','Test',1895,'',1 from dual union all
select 17430,'1968-S441','1968-08-02 06:19:00','suborbital rocket','success',227,'ADD-VB-18','','','','Aeron/Ionos/Fields',428,'',284 from dual union all
select 17431,'1968-S442','1968-08-06 01:55:00','suborbital rocket','success',1197,'SL524','UCW-15','','','Ionospheric',1839,'',234 from dual union all
select 17432,'1968-S443','1968-08-06 18:43:00','suborbital rocket','success',30,'CRL AG3.532','EUV Mon/RPA','','','Ionos/Solar EUV',1869,'',127 from dual union all
select 17433,'1968-S444','1968-08-07 12:05:00','suborbital rocket','success',837,'NASA 14.320UE','Rice Particles','','','Plasma',1821,'',196 from dual union all
select 17434,'1968-S445','1968-08-08 00:00:00','suborbital rocket','success',1288,'','GF-01A test','','','Test',819,'',311 from dual union all
select 17435,'1968-S446','1968-08-08 09:00:00','suborbital rocket','success',848,'CRL AG07.273','Meteor dust','','','Meteorites',330,'',158 from dual union all
select 17436,'1968-S447','1968-08-08 19:10:00','suborbital rocket','success',867,'NASA 18.51GA','UM Thermosphere 11','','','Aeronomy',1821,'',286 from dual union all
select 17437,'1968-S448','1968-08-08 19:35:00','suborbital rocket','success',837,'NASA 14.187UM','UM Pitot 38','','','Aeronomy',1821,'',152 from dual union all
select 17438,'1968-S449','1968-08-09 07:02:00','suborbital rocket','success',867,'NASA 18.56GA','UM Thermosphere 12','','','Aeronomy',1821,'',320 from dual union all
select 17439,'1968-S45','1968-06-27 00:00:00','military missile','success',1037,'Ya23110-37L','8K98 LKI-2-20','','','Test',477,'',1000 from dual union all
select 1744,'1953-A324','1953-08-13 00:00:00','military missile','success',572,'3','RV-A-8','','','Test',1863,'',30 from dual union all
select 17440,'1968-S450','1968-08-09 07:24:00','suborbital rocket','success',837,'NASA 14.357UM','UM Pitot 39','','','Aeronomy',1821,'',146 from dual union all
select 17441,'1968-S451','1968-08-11 09:04:00','suborbital rocket','success',848,'CRL AG07.274','Meteorites','','','Meteorites',330,'',98 from dual union all
select 17442,'1968-S452','1968-08-12 12:14:00','suborbital rocket','success',846,'STV SR-071','','','','Aeron?',1864,'',218 from dual union all
select 17443,'1968-S453','1968-08-12 09:00:00','suborbital rocket','success',848,'CRL AG07.275','Meteorites','','','Meteorites',330,'',153 from dual union all
select 17444,'1968-S454','1968-08-12 08:38:00','suborbital rocket','success',837,'NASA 14.336UA','Pandora D','','','Meteorites',1895,'',155 from dual union all
select 17445,'1968-S455','1968-08-14 00:00:00','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 618',1000 from dual union all
select 17446,'1968-S456','1968-08-14 03:30:00','military missile','success',1000,'U 22505-05','8K67M UBP-2','GCh 8F675','','OT',514,'',1000 from dual union all
select 17447,'1968-S457','1968-08-15 09:00:00','suborbital rocket','failure',848,'CRL AG07.276','Meteorites','','','Meteorites',330,'',95 from dual union all
select 17448,'1968-S458','1968-08-15 07:59:00','suborbital rocket','success',30,'CRL AE3.725','MAP (IR horizon)','','','Aeron IR',428,'',179 from dual union all
select 17449,'1968-S459','1968-08-16 20:30:02','military missile','success',799,'FTM-201','Mk 12 RV x 3','R&D','','Test',266,'',900 from dual union all
select 1745,'1953-A325','1953-08-13 00:00:00','atmospheric rocket','success',836,'1229R','Nike 235R','','','Test',1895,'',10 from dual union all
select 17450,'1968-S46','1968-07-07 00:00:00','military missile','success',1037,'Ya23109-36L','8K98 LKI-2-21','','','Test',477,'',1000 from dual union all
select 17451,'1968-S460','1968-08-16 10:30:00','military missile','success',939,'C3XD-01','','','','Test',246,'',500 from dual union all
select 17452,'1968-S461','1968-08-19 09:17:00','suborbital rocket','success',848,'CRL AG07.636','Niro TMA 4','','','Aeron',428,'',177 from dual union all
select 17453,'1968-S462','1968-08-20 00:00:00','suborbital rocket','success',1288,'','GF-01A test','','','Test',819,'',311 from dual union all
select 17454,'1968-S463','1968-08-20 09:00:00','suborbital rocket','success',867,'CRL AH08.649','NO Jet payload 1','','','Aeron',428,'',194 from dual union all
select 17455,'1968-S464','1968-08-20 10:10:00','suborbital rocket','success',848,'CRL AG07.685','Niro TMA 5/NO','','','Aeron',428,'',175 from dual union all
select 17456,'1968-S465','1968-08-20 19:32:00','suborbital rocket','success',837,'NASA 14.301UA','Pandora E','','','Meteorites',1895,'',154 from dual union all
select 17457,'1968-S466','1968-08-21 13:00:00','military missile','success',1420,'B-53','Mk 6 RV','FOT GT18T','GLORY TRIP 18T','OT',1658,'',1300 from dual union all
select 17458,'1968-S467','1968-08-21 16:04:48','suborbital spaceplane','success',1520,'1-79-139','WTR/HS/SB/FAB/FP','','','Test/Tech/Aeron',379,'NB-52 003',81 from dual union all
select 17459,'1968-S468','1968-08-21 19:40:00','suborbital rocket','success',837,'NASA 14.369GI','','','','Ionos',1821,'',193 from dual union all
select 1746,'1953-A326','1953-08-13 00:00:00','atmospheric rocket','success',836,'1235R','Nike 237R','','','Test',1895,'',10 from dual union all
select 17460,'1968-S469','1968-08-21 19:53:00','suborbital rocket','success',837,'NASA 14.368GI','','','','Ionos',1821,'',189 from dual union all
select 17461,'1968-S47','1968-06-18 00:00:00','military missile','success',1038,'V 22500-10L','8K99 LKI-5','','','Test',477,'',1000 from dual union all
select 17462,'1968-S470','1968-08-21 20:04:00','suborbital rocket','success',837,'NASA 14.370GI','','','','Ionos',1821,'',192 from dual union all
select 17463,'1968-S471','1968-08-21 21:30:00','suborbital rocket','success',867,'NASA 18.30UI','','','','Ionos',1821,'',262 from dual union all
select 17464,'1968-S472','1968-08-23 13:30:00','military missile','success',1002,'V 22500-02L','8K67P LKI-1','RGCh 8F676 LKI (3x8F677)','','Test',511,'',1000 from dual union all
select 17465,'1968-S473','1968-08-23 04:48:00','suborbital rocket','success',1115,'Sandia 152-','LRL AXE-2','','','XR Astron',1490,'',187 from dual union all
select 17466,'1968-S474','1968-08-27 03:34:00','suborbital rocket','success',848,'CRL AH07.667','Niro TMA 6','','','Aeron',428,'',201 from dual union all
select 17467,'1968-S475','1968-08-27 04:44:00','suborbital rocket','success',867,'CRL AH08.668','NO Jet payload 2','','','Aeron',428,'',201 from dual union all
select 17468,'1968-S476','1968-08-24 00:00:00','military missile','success',1037,'Ya23113-40L','8K98 LKI-2-22','','','Test',477,'',1000 from dual;
insert into LAUNCH
select 17469,'1968-S477','1968-08-24 00:00:00','military missile','success',1037,'Ya23114-41L','8K98 LKI-2-23','','','Test',477,'',1000 from dual union all
select 1747,'1953-A327','1953-08-13 00:00:00','atmospheric rocket','success',836,'1237R','Nike 236R','','','Test',1895,'',10 from dual union all
select 17470,'1968-S478','1968-08-24 00:00:00','military missile','success',1037,'Ya23115-42L','8K98 LKI-2-24','','','Test',477,'',1000 from dual union all
select 17471,'1968-S479','1968-08-29 15:15:00','military missile','success',932,'A3T-208','DASO/Antelope','','','OT',421,'SSBN 626',1000 from dual union all
select 17472,'1968-S48','1968-07-02 00:00:00','military missile','success',1038,'V 22500-09L','8K99 LKI-6','','','Test',477,'',0 from dual union all
select 17473,'1968-S480','1968-08-29 16:00:00','military missile','success',932,'A3T-347','DASO/Antelope','','','OT',421,'SSBN 626',1000 from dual union all
select 17474,'1968-S481','1968-08-29 05:38:00','suborbital rocket','success',837,'NASA 14.191','ISRO 20.06','TERLS-081','','Ionos',1467,'',170 from dual union all
select 17475,'1968-S482','1968-08-29 08:45:00','suborbital rocket','success',837,'NASA 14.192','ISRO 20.07','TERLS-082','','Ionos',1467,'',170 from dual union all
select 17476,'1968-S483','1968-08-29 17:30:00','suborbital rocket','success',837,'NASA 14.193','ISRO 20.08','TERLS-083','','Ionos',1467,'',168 from dual union all
select 17477,'1968-S484','1968-08-29 19:43:00','suborbital rocket','success',1207,'SL501','MSSL SX-3/Bir-16','','','Ionos/Solar X',1839,'',210 from dual union all
select 17478,'1968-S485','1968-09-01 18:00:00','military missile','success',796,'1442','Mk 11 RV','ST GIANT BOOST','SHORT ROUND','Test',1707,'',1 from dual union all
select 17479,'1968-S486','1968-09-04 14:05:00','suborbital rocket','success',846,'STV SR-072','','','','Aeron?',1895,'',150 from dual union all
select 1748,'1953-A328','1953-08-17 00:00:00','atmospheric rocket','unknown',576,'68M','','','','Test',1895,'',10 from dual union all
select 17480,'1968-S487','1968-09-04 01:04:00','suborbital rocket','success',848,'CRL A07.913-1','Sphere','','','Aeron',394,'',227 from dual union all
select 17481,'1968-S488','1968-09-04 01:29:00','suborbital rocket','success',848,'CRL A07.913-2','Sphere','','','Aeron',394,'',232 from dual union all
select 17482,'1968-S489','1968-09-04 14:50:00','suborbital rocket','success',30,'NASA 04.130UA','Structure','','','Aeronomy',1869,'',124 from dual union all
select 17483,'1968-S49','1968-10-21 00:00:00','military missile','failure',1038,'14L','8K99 LKI-9','','','Test',477,'',0 from dual union all
select 17484,'1968-S490','1968-09-05 12:00:00','suborbital rocket','success',846,'STV SR-073','','','','Aeron?',1864,'',150 from dual union all
select 17485,'1968-S491','1968-09-05 14:00:00','suborbital rocket','success',846,'STV SR-074','','','','Aeron?',1864,'',150 from dual union all
select 17486,'1968-S492','1968-09-10 16:00:00','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 617',1000 from dual union all
select 17487,'1968-S493','1968-09-11 11:15:00','military missile','success',1002,'V 22500-01L','8K67P LKI-2','RGCh 8F676 LKI','','Test',511,'',1000 from dual union all
select 17488,'1968-S494','1968-09-13 16:58:00','suborbital rocket','success',740,'','','','','Aeron',636,'',155 from dual union all
select 17489,'1968-S495','1968-09-14 01:00:00','suborbital rocket','success',1069,'S-160-1','','','','Aeron',839,'',82 from dual union all
select 1749,'1953-A329','1953-08-18 00:00:00','atmospheric rocket','unknown',576,'213','','','','Test',1895,'',10 from dual union all
select 17490,'1968-S496','1968-09-14 04:00:00','suborbital rocket','success',1069,'S-160-2','','','','Aeron',839,'',82 from dual union all
select 17491,'1968-S497','1968-09-16 17:12:00','suborbital rocket','success',839,'NASA 10.269GM','GSFC Grenade 183 W','','','Aeronomy',1821,'',118 from dual union all
select 17492,'1968-S498','1968-09-17 20:03:00','suborbital rocket','success',839,'NASA 10.257GM','GSFC Grenade 184 PB','','','Aeronomy',1039,'',100 from dual union all
select 17493,'1968-S499','1968-09-19 01:58:00','suborbital rocket','success',694,'LS-C-D','','','','Test',1476,'',10 from dual union all
select 17494,'1968-S50','1968-02-07 00:00:00','military missile','success',1449,'','8K84 UBP-72','','','OT',505,'',1000 from dual union all
select 17495,'1968-S500','1968-09-19 07:01:00','suborbital rocket','success',826,'NAL-16H F1','','','','Test',1476,'',100 from dual union all
select 17496,'1968-S501','1968-09-20 16:30:00','suborbital rocket','success',740,'','','','','Aeron',636,'',155 from dual union all
select 17497,'1968-S502','1968-09-20 19:31:00','suborbital rocket','success',867,'NASA 18.75GE','B field','','','Plasma',29,'',321 from dual union all
select 17498,'1968-S503','1968-09-20 19:32:00','suborbital rocket','success',867,'NASA 18.58IE','Ferdinand 18','','','Plasma',29,'',239 from dual union all
select 17499,'1968-S504','1968-09-21 03:45:00','suborbital rocket','success',30,'NASA 04.174UG','','','','UV Astron',1869,'',173 from dual union all
select 175,'1944-S159','1944-10-01 00:00:00','military missile','success',1470,'','London','','','Weapon',1734,'',90 from dual union all
select 1750,'1953-A33','1953-01-27 00:00:00','atmospheric rocket','success',836,'1249B-1073','Nike 1249 R&D 168R','','','Test',1895,'',10 from dual union all
select 17500,'1968-S505','1968-09-21 19:36:00','suborbital rocket','success',867,'NASA 18.76GE','B field','','','Plasma',29,'',321 from dual union all
select 17501,'1968-S506','1968-09-22 15:30:00','suborbital rocket','success',30,'NASA 04.246DS','','','','Solar UV',1869,'',189 from dual union all
select 17502,'1968-S507','1968-09-23 19:29:00','suborbital rocket','success',867,'NASA 18.77GE','B field','','','Plasma',29,'',321 from dual union all
select 17503,'1968-S508','1968-09-24 16:30:00','suborbital rocket','success',30,'NASA 04.185US','HCO Spectrometer','','','Solar UV',1869,'',180 from dual union all
select 17504,'1968-S509','1968-09-25 11:54:00','military missile','success',168,'99F','RMV-340B/RMV-430C/BSE','ABRES RMP-B-9/NTMP KX109','','Reentry Test',1682,'',1400 from dual union all
select 17505,'1968-S51','1968-02-13 00:00:00','military missile','success',1449,'','8K84 UBP-73','','','OT',508,'',1000 from dual union all
select 17506,'1968-S510','1968-09-25 22:02:00','suborbital rocket','success',740,'','','','','Aeron',636,'',180 from dual union all
select 17507,'1968-S511','1968-09-26 18:00:03','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 617',1000 from dual union all
select 17508,'1968-S513','1968-09-27 00:00:00','military missile','success',162,'84F','WT1A-L2/WT1A-S3/PEM-IV','ABRES RVTO-1A-3','','Reentry Test',1680,'',1400 from dual union all
select 17509,'1968-S514','1968-09-27 23:45:00','suborbital rocket','success',839,'NASA 10.281GM','GSFC Grenade 186 C','','','Aeronomy',428,'',100 from dual union all
select 1751,'1953-A330','1953-08-19 00:00:00','atmospheric rocket','success',365,'','D18 F100 Model','','','Test',1821,'',10 from dual union all
select 17510,'1968-S515','1968-09-30 00:00:00','military missile','success',736,'','','','','Test',311,'',100 from dual union all
select 17511,'1968-S516','1968-09-30 19:30:00','suborbital rocket','success',32,'NASA 04.231GS','GSFC EUV Spec','','','Solar EUV',1869,'',203 from dual union all
select 17512,'1968-S517','1968-10-01 18:52:00','suborbital rocket','success',30,'NASA 04.241UA','UV Dayglow','','','Aeronomy',1869,'',174 from dual union all
select 17513,'1968-S518','1968-10-01 21:44:16','suborbital rocket','success',867,'NASA 18.59IE','Ferdinand 19','','','Plasma',29,'',228 from dual union all
select 17514,'1968-S519','1968-10-02 02:17:00','suborbital rocket','success',740,'','','','','Aeron',636,'',174 from dual union all
select 17515,'1968-S52','1968-03-22 00:00:00','military missile','success',1449,'','8K84 UBP-74','','','OT',508,'',1000 from dual union all
select 17516,'1968-S520','1968-10-03 00:00:00','military missile','success',1037,'15L','8K98 LKI-2-25','','','Test',477,'',1000 from dual union all
select 17517,'1968-S521','1968-10-04 12:30:00','military missile','success',1002,'V 22500-03L','8K67P LKI-3','RGCh 8F676 LKI','','Test',511,'',1000 from dual union all
select 17518,'1968-S522','1968-10-07 21:39:48','suborbital rocket','success',1206,'ESRO S47/1','ESRO S47/1 ROE-7','','','UV Astronomy',1343,'',208 from dual union all
select 17519,'1968-S523','1968-10-08 00:00:00','military missile','unknown',1102,'','S01C-1','','','Test',301,'',500 from dual union all
select 1752,'1953-A331','1953-08-19 00:00:00','atmospheric rocket','success',365,'','F29 Ramjet','','','Test',1821,'',10 from dual union all
select 17520,'1968-S524','1968-10-08 02:26:00','suborbital rocket','success',740,'','','','','Aeron',636,'',125 from dual union all
select 17521,'1968-S525','1968-10-08 05:07:00','suborbital rocket','success',217,'AKF-3-40','','','','XR Astron/Ionos',1315,'',153 from dual union all
select 17522,'1968-S526','1968-10-08 11:42:57','suborbital rocket','success',290,'CE-48','ESRO C35/1','','','XR Astronomy',408,'',140 from dual union all
select 17523,'1968-S527','1968-10-09 03:20:00','suborbital rocket','success',217,'AKF-3-39','','','','Ionos/CR/Solar X',1315,'',120 from dual union all
select 17524,'1968-S528','1968-10-09 03:32:00','suborbital rocket','success',740,'','','','','Aeron',636,'',130 from dual union all
select 17525,'1968-S529','1968-10-09 04:58:00','suborbital rocket','success',1200,'SL791R','RAE Test 40','','','Test',1839,'',106 from dual union all
select 17526,'1968-S53','1968-03-28 00:00:00','military missile','success',1449,'','8K84 UBP-75','','','OT',505,'',1000 from dual union all
select 17527,'1968-S530','1968-10-09 09:06:00','suborbital rocket','success',725,'','Particle flux 3','','','Meteo',1010,'',60 from dual union all
select 17528,'1968-S531','1968-10-10 10:30:00','suborbital rocket','success',30,'NASA 04.265DG','','','','UV Astron',1869,'',175 from dual union all
select 17529,'1968-S532','1968-10-15 00:00:00','suborbital rocket','success',1301,'004','Sun pointing test','','','Test/Solar',311,'',140 from dual union all
select 1753,'1953-A332','1953-08-20 00:00:00','atmospheric rocket','success',1445,'','E10 Hypersonic Test','','','Test',1821,'',20 from dual union all
select 17530,'1968-S533','1968-10-15 01:00:00','suborbital rocket','success',839,'NASA 10.287GM','GSFC Grenade 187 PB','','','Aeronomy',1039,'',129 from dual union all
select 17531,'1968-S534','1968-10-15 03:00:00','suborbital rocket','success',839,'NASA 10.288GM','GSFC Grenade 188 PB','','','Aeronomy',1039,'',179 from dual union all
select 17532,'1968-S535','1968-10-15 16:32:00','suborbital rocket','success',290,'CE-39','ESRO C45/1','','','Aeronomy',408,'',160 from dual union all
select 17533,'1968-S536','1968-10-15 22:12:00','suborbital rocket','success',839,'NASA 10.251GM','GSFC Grenade 189 C','','','Aeronomy',428,'',127 from dual union all
select 17534,'1968-S537','1968-10-16 00:11:00','suborbital rocket','success',867,'NASA 18.69UA','Composition','','','Aeronomy',1821,'',340 from dual union all
select 17535,'1968-S538','1968-10-16 00:12:00','suborbital rocket','success',839,'NASA 10.252GM','GSFC Grenade 190 C','','','Aeronomy',428,'',123 from dual union all
select 17536,'1968-S539','1968-10-16 08:02:00','suborbital rocket','success',740,'','','','','Aeron',636,'',171 from dual union all
select 17537,'1968-S54','1968-03-30 00:00:00','military missile','success',1449,'','8K84 UBP-76','','','OT',508,'',1000 from dual union all
select 17538,'1968-S540','1968-10-16 23:15:00','suborbital rocket','success',30,'NASA 04.248CS','','','','Solar',1869,'',155 from dual union all
select 17539,'1968-S541','1968-10-17 21:30:00','suborbital rocket','success',846,'9.5-68-4-604','','','','Aeron?',1863,'',150 from dual union all
select 1754,'1953-A333','1953-08-20 00:00:00','atmospheric rocket','success',836,'1245R','Nike 238R','','','Test',1895,'',10 from dual union all
select 17540,'1968-S542','1968-10-21 00:00:00','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 608',1000 from dual union all
select 17541,'1968-S543','1968-10-21 00:00:00','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 608',1000 from dual union all
select 17542,'1968-S544','1968-10-21 00:00:00','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 608',1000 from dual union all
select 17543,'1968-S545','1968-10-21 00:00:00','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 608',1000 from dual union all
select 17544,'1968-S546','1968-10-21 00:00:00','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 608',1000 from dual union all
select 17545,'1968-S547','1968-10-22 08:03:00','suborbital rocket','success',740,'','Ion composition','','','Aeron',636,'',168 from dual union all
select 17546,'1968-S548','1968-10-22 10:02:00','suborbital rocket','success',740,'','','','','Aeron',636,'',164 from dual union all
select 17547,'1968-S549','1968-10-22 14:50:00','suborbital rocket','success',846,'9.5-68-5-604','','','','Aeron?',1895,'',150 from dual union all
select 17548,'1968-S55','1968-04-05 00:00:00','military missile','success',1449,'','8K84 UBP-77','','','OT',505,'',1000 from dual union all
select 17549,'1968-S550','1968-10-24 20:30:03','military missile','failure',799,'FTM-202','Mk 12 RV x 3','R&D','','Test',266,'',900 from dual union all
select 1755,'1953-A334','1953-08-20 14:37:00','military missile','failure',1053,'RS-1     XH','','','','Test',272,'',6 from dual union all
select 17550,'1968-S551','1968-10-24 13:03:00','suborbital rocket','success',846,'STV SR-077','','','','Aeron?',1864,'',150 from dual union all
select 17551,'1968-S552','1968-10-24 17:00:00','military missile','success',796,'FTM-2191','Mk 11 RV','DASO GIANT FIST 2A','GIANT FIST 2A','OT',1699,'',900 from dual union all
select 17552,'1968-S553','1968-10-25 10:04:00','suborbital rocket','success',740,'','','','','Aeron',636,'',130 from dual union all
select 17553,'1968-S554','1968-10-26 09:11:00','suborbital rocket','success',37,'NASA 12.13GT-UG','KP3.23','','','UV Astronomy/Test',1869,'',245 from dual union all
select 17554,'1968-S555','1968-10-27 10:03:00','suborbital rocket','unknown',740,'MP-12','','','','Aeron',636,'',168 from dual union all
select 17555,'1968-S556','1968-10-29 11:32:00','suborbital rocket','success',1347,'Sandia 281-10','LRL ACS-3','','','XR Astron',802,'',320 from dual union all
select 17556,'1968-S557','1968-10-30 08:23:00','military missile','success',795,'1136','Mk 11 RV','ST OLYMPIC T. B-5','OLYMPIC TRIALS B-5','OT',1705,'',900 from dual union all
select 17557,'1968-S558','1968-10-30 08:06:00','suborbital rocket','success',740,'','Thermosphere comp 4','','','Aeron',1010,'',171 from dual union all
select 17558,'1968-S559','1968-11-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual;
insert into LAUNCH
select 17559,'1968-S56','1968-06-06 00:00:00','military missile','success',1449,'','8K84 UBP-78','','','OT',542,'',1000 from dual union all
select 1756,'1953-A335','1953-08-24 00:00:00','military missile','success',771,'GM-11083','','','','Test',222,'',12 from dual union all
select 17560,'1968-S560','1968-11-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 17561,'1968-S561','1968-11-01 00:00:00','suborbital rocket','success',1276,'','MTV-I','','','Reentry Test',803,'',200 from dual union all
select 17562,'1968-S562','1968-10-31 20:56:00','suborbital rocket','success',217,'ADF-3-31','PCA Ionos','','','Aeron/Ionos',428,'',140 from dual union all
select 17563,'1968-S563','1968-11-01 03:31:00','suborbital rocket','success',217,'ADF-3-33','PCA Ionos','','','Aeron/Ionos/Plasma',428,'',138 from dual union all
select 17564,'1968-S564','1968-11-01 04:40:00','suborbital rocket','success',290,'CE-50','ESRO C45/2','','','Aeronomy',408,'',167 from dual union all
select 17565,'1968-S565','1968-11-03 01:42:24','suborbital rocket','success',447,'D-254','FU-182 Ions/E field','','','Ionos',29,'',414 from dual union all
select 17566,'1968-S566','1968-11-03 03:19:00','suborbital rocket','success',288,'C104','ISRO 35.01','TERLS-088','','XR Astron',1467,'',159 from dual union all
select 17567,'1968-S567','1968-11-03 18:00:00','suborbital rocket','success',589,'BOX-15','LRL BOX-15','','','XR Astron',1280,'T-AGM-8',227 from dual union all
select 17568,'1968-S568','1968-11-05 16:11:00','military missile','success',916,'AO-70','P-157 68 Winter 01','','','Test',597,'',180 from dual union all
select 17569,'1968-S569','1968-11-05 18:00:00','military missile','success',916,'AO-71','P-158 68 Winter 02','','','Test',597,'',180 from dual union all
select 1757,'1953-A336','1953-08-28 00:00:00','atmospheric rocket','unknown',576,'329','','','','Test',1895,'',10 from dual union all
select 17570,'1968-S57','1968-06-12 00:00:00','military missile','success',1449,'','8K84 UBP-79','','','OT',504,'',1000 from dual union all
select 17571,'1968-S570','1968-11-06 11:22:00','suborbital rocket','success',1189,'INTA MP-6807','','','','Ionos',396,'',92 from dual union all
select 17572,'1968-S571','1968-11-07 03:05:00','suborbital rocket','success',837,'NASA 14.261IG','ISRO 40.03','TERLS-089','','XR Astron',1467,'',150 from dual union all
select 17573,'1968-S572','1968-11-07 06:45:00','suborbital rocket','success',1347,'Sandia 281-11','LRL ACS-2','','','XR Astron',802,'',320 from dual union all
select 17574,'1968-S573','1968-11-07 18:27:00','suborbital rocket','success',837,'NASA 14.379GI','Ion concentration','','','Ionos',1821,'',104 from dual union all
select 17575,'1968-S574','1968-11-08 11:06:00','suborbital rocket','success',1189,'INTA MP-6808','','','','Ionos',396,'',82 from dual union all
select 17576,'1968-S575','1968-11-09 00:00:00','military missile','success',1102,'','S01C-2','','','Test',301,'',500 from dual union all
select 17577,'1968-S576','1968-11-09 08:55:00','suborbital rocket','success',624,'CRL AB19.287','P/F','','','Plasma',428,'',592 from dual union all
select 17578,'1968-S577','1968-11-11 08:00:00','suborbital rocket','success',920,'P15H','C/D/E electrons/SpE','','','Ionospheric',1447,'',142 from dual union all
select 17579,'1968-S578','1968-11-12 11:20:00','suborbital rocket','success',1189,'INTA MP-6809','','','','Ionos',396,'',94 from dual union all
select 1758,'1953-A337','1953-08-28 00:00:00','atmospheric rocket','unknown',576,'330','','','','Test',1895,'',10 from dual union all
select 17580,'1968-S579','1968-11-13 12:10:00','military missile','success',796,'2537','Mk 11 RV','DASO GIANT FIST 4','GIANT FIST 4','OT',1700,'',900 from dual union all
select 17581,'1968-S58','1968-06-24 00:00:00','military missile','success',1449,'','8K84 UBP-80','','','OT',505,'',1000 from dual union all
select 17582,'1968-S580','1968-11-13 11:46:00','suborbital rocket','success',1189,'INTA MP-6810','','','','Ionos',396,'',90 from dual union all
select 17583,'1968-S581','1968-11-13 00:00:00','military missile','success',916,'AO-72','P-159 68 Winter 03','','','OT',597,'',180 from dual union all
select 17584,'1968-S582','1968-11-15 05:30:00','military missile','success',135,'V120B [104]','ABRES AN-11 (USAF)','ABRES AN-11 (USAF)','','Reentry Test',612,'',200 from dual union all
select 17585,'1968-S583','1968-11-15 07:00:00','military missile','success',135,'D062  [105]','SFT-12','ABRES AC-5 (USN)','','Reentry Test',614,'',200 from dual union all
select 17586,'1968-S584','1968-11-15 07:45:00','suborbital rocket','success',30,'NASA 04.268UG','','','','UV Astron',1869,'',174 from dual union all
select 17587,'1968-S585','1968-11-15 11:58:00','suborbital rocket','success',1189,'INTA MP-6811','','','','Ionos',396,'',90 from dual union all
select 17588,'1968-S586','1968-11-16 00:00:00','military missile','success',751,'V1','V1 Vecteur','','','OT',307,'S655',500 from dual union all
select 17589,'1968-S587','1968-11-18 10:51:00','suborbital rocket','success',1189,'INTA MP-6812','','','','Ionos',396,'',95 from dual union all
select 1759,'1953-A338','1953-08-30 14:00:00','suborbital rocket','failure',370,'Eastwind 54-1','SUI 18','','','CR',52,'WYG-279',11 from dual union all
select 17590,'1968-S588','1968-11-19 00:00:00','military missile','success',1420,'B-3','Mk 6 RV','FOT GT26T','GLORY TRIP 26T','OT',1658,'',1300 from dual union all
select 17591,'1968-S589','1968-11-19 01:38:30','suborbital rocket','success',217,'K-BB/III-14','DLR K-BB3-14','','','Ionos/Plasma',413,'',167 from dual union all
select 17592,'1968-S59','1968-07-02 00:00:00','military missile','success',1449,'','8K84 UBP-81','','','OT',505,'',1000 from dual union all
select 17593,'1968-S591','1968-11-19 04:29:00','suborbital rocket','failure',217,'ADF-3-32','','','','Ionos/Plasma',428,'',288 from dual union all
select 17594,'1968-S592','1968-11-19 08:00:31','suborbital rocket','success',226,'CRL AG17.757','PCA D-region','','','Plasma',428,'',129 from dual union all
select 17595,'1968-S593','1968-11-19 18:00:00','suborbital rocket','success',839,'NASA 10.293GM','GSFC Grenade 191 W','','','Aeronomy',1821,'',123 from dual union all
select 17596,'1968-S594','1968-11-19 18:30:00','suborbital rocket','success',837,'NASA 14.371GI','NIR Radiometer','','','Ionospheric',1821,'',200 from dual union all
select 17597,'1968-S595','1968-11-19 20:04:59','suborbital rocket','success',837,'NASA 14.386UM','UM Pitot 40/UM Sphere','','','Aeronomy',1821,'',148 from dual union all
select 17598,'1968-S596','1968-11-20 07:08:00','military missile','success',1373,'252','AFP-437 CEL','','','Interceptor',772,'',1158 from dual union all
select 17599,'1968-S597','1968-11-20 11:24:00','suborbital rocket','success',839,'NASA 10.283GM','GSFC Grenade 192 C','','','Aeronomy',428,'',120 from dual union all
select 176,'1944-S16','1944-03-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1760,'1953-A339','1953-08-30 16:20:00','suborbital rocket','failure',370,'Eastwind 54-2','SUI 19','','','CR',53,'WYG-279',11 from dual union all
select 17600,'1968-S598','1968-11-20 13:24:00','suborbital rocket','success',839,'NASA 10.284GM','GSFC Grenade 193 C','','','Aeronomy',428,'',114 from dual union all
select 17601,'1968-S599','1968-11-20 14:45:00','suborbital rocket','unknown',1189,'INTA MP-6813','','','','Ionos',396,'',100 from dual union all
select 17602,'1968-S60','1968-07-06 00:00:00','military missile','success',1449,'','8K84 UBP-82','','','OT',508,'',1000 from dual union all
select 17603,'1968-S600','1968-11-21 18:28:00','military missile','success',796,'2556','Mk 11 RV','DASO GIANT BLADE 2','GIANT BLADE 2','OT',1711,'',900 from dual union all
select 17604,'1968-S601','1968-11-21 18:30:00','suborbital rocket','success',30,'CRL AT3.533','EUV Mon/RPA','','','Solar EUV',1869,'',244 from dual union all
select 17605,'1968-S602','1968-11-21 19:15:00','suborbital rocket','success',33,'NASA 04.240US','','','','Solar',1869,'',193 from dual union all
select 17606,'1968-S603','1968-11-21 20:45:00','suborbital rocket','success',35,'NASA 04.215NB','Rat centrifuge 3','','','Biology',1821,'',164 from dual union all
select 17607,'1968-S604','1968-11-22 00:31:00','suborbital rocket','success',839,'NASA 10.289GM','GSFC Grenade 194 PB','','','Aeronomy',1039,'',179 from dual union all
select 17608,'1968-S605','1968-11-22 07:55:00','suborbital rocket','success',839,'NASA 10.290GM','GSFC Grenade 195 PB','','','Aeronomy',1039,'',179 from dual union all
select 17609,'1968-S606','1968-11-22 10:43:00','suborbital rocket','success',1189,'INTA MP-6814','','','','Ionos',396,'',99 from dual union all
select 1761,'1953-A34','1953-01-27 00:00:00','atmospheric rocket','success',836,'1249B-1075','Nike 1249 R&D 169R','','','Test',1895,'',10 from dual union all
select 17610,'1968-S607','1968-11-22 11:16:12','suborbital rocket','success',1207,'ESRO S41/1','ESRO S41/1 LUX-44','','','Solar X',1343,'',210 from dual union all
select 17611,'1968-S608','1968-11-24 12:10:00','military missile','success',162,'60F','WT1A-L3/WT1A-S4','ABRES RVTO-1A-4','','Reentry Test',1680,'',1400 from dual union all
select 17612,'1968-S609','1968-11-25 00:41:50','suborbital rocket','success',1204,'ESRO S43/1','ESRO S43/1 Bir-17','','','Auroral',416,'',212 from dual union all
select 17613,'1968-S61','1968-07-09 00:00:00','military missile','success',1449,'','8K84 UBP-83','','','OT',505,'',1000 from dual union all
select 17614,'1968-S610','1968-11-25 11:01:00','suborbital rocket','success',1189,'INTA MP-6815','','','','Ionos',396,'',94 from dual union all
select 17615,'1968-S611','1968-11-26 17:00:00','military missile','success',939,'C3XD-02','','','','Test',246,'',500 from dual union all
select 17616,'1968-S613','1968-11-27 15:02:00','suborbital rocket','success',1189,'INTA MP-6816','','','','Ionos',396,'',86 from dual union all
select 17617,'1968-S614','1968-11-28 18:51:00','suborbital rocket','success',27,'AH1007','','','','Aeron',1833,'',134 from dual union all
select 17618,'1968-S615','1968-11-29 11:21:00','suborbital rocket','success',1189,'INTA MP-6817','','','','Ionos',396,'',87 from dual union all
select 17619,'1968-S616','1968-11-30 14:57:00','suborbital rocket','success',920,'P19H','D-region','','','Ionospheric',1447,'',140 from dual union all
select 1762,'1953-A340','1953-09-01 00:00:00','atmospheric rocket','unknown',576,'200','','','','Test',1895,'',10 from dual union all
select 17620,'1968-S617','1968-12-03 19:30:00','suborbital rocket','success',219,'','(DLR N-BB4)','','','Magnetospheric',330,'',700 from dual union all
select 17621,'1968-S618','1968-12-03 20:38:49','suborbital rocket','success',1206,'ESRO S27/1','ESRO S27/1 ROE-8','','','UV Astronomy',1343,'',198 from dual union all
select 17622,'1968-S619','1968-12-03 21:58:00','suborbital rocket','success',217,'K-BB/III-16','DLR K-BB3-16','','','Ionos/Plasma',413,'',165 from dual union all
select 17623,'1968-S62','1968-07-10 00:00:00','military missile','success',1449,'','8K84 UBP-84','','','OT',504,'',1000 from dual union all
select 17624,'1968-S620','1968-12-04 03:30:00','military missile','success',1000,'V 22503-14T','8K67M UBP-3','GCh 8F675U','','OT',515,'',1000 from dual union all
select 17625,'1968-S621','1968-12-04 07:54:17','suborbital rocket','success',837,'Ferdinand 16','Ferdinand 16','','','Aeron/Ionos/Plasma',29,'',135 from dual union all
select 17626,'1968-S622','1968-12-04 17:51:00','suborbital rocket','failure',219,'','','','','Aeron',330,'',600 from dual union all
select 17627,'1968-S623','1968-12-03 08:10:00','suborbital rocket','success',837,'NASA 14.351UA','Pandora F','','','Aeronomy/Meteorites',1895,'',153 from dual union all
select 17628,'1968-S624','1968-12-04 18:12:00','suborbital rocket','failure',219,'ALF-IV-12','Mass spectrometer','','','Aeron',428,'',100 from dual union all
select 17629,'1968-S625','1968-12-04 19:06:00','suborbital rocket','success',217,'ALF-3-17','BB3/2','','','Aeron',428,'',158 from dual union all
select 1763,'1953-A341','1953-09-01 00:00:00','atmospheric rocket','unknown',576,'201','','','','Test',1895,'',10 from dual union all
select 17630,'1968-S626','1968-12-04 22:26:30','suborbital rocket','success',217,'K-BB/III-15','DLR K-BB3-15','','','Ionos/Plasma',413,'',170 from dual union all
select 17631,'1968-S627','1968-12-05 00:00:00','military missile','success',135,'V137D [107]','ABRES AM-3 (USAF)','ABRES AM-3 (USAF)','','Reentry Test',612,'',200 from dual union all
select 17632,'1968-S628','1968-12-05 00:00:00','military missile','success',135,'D045  [106]','ABRES QQ-6 (USAF)','ABRES QQ-6 (USAF)','','Reentry Test',613,'',200 from dual union all
select 17633,'1968-S629','1968-12-04 05:35:00','suborbital rocket','success',1207,'SL601','Cul UVS 10','','','Solar UV',1839,'',182 from dual union all
select 17634,'1968-S63','1968-07-13 00:00:00','military missile','success',1449,'','8K84 UBP-85','','','OT',508,'',1000 from dual union all
select 17635,'1968-S630','1968-12-05 15:29:00','suborbital rocket','success',920,'P22H','D-region','','','Ionospheric',1447,'',144 from dual union all
select 17636,'1968-S631','1968-12-06 00:00:00','military missile','success',135,'V136D [108]','ABRES AN-3 (USAF)','ABRES AN-3 (USAF)','','Reentry Test',614,'',200 from dual union all
select 17637,'1968-S632','1968-12-07 00:00:00','military missile','success',795,'1375','Mk 11A RV','ST OLYMPIC T. B-6','OLYMPIC TRIALS B-6','OT',1705,'',900 from dual union all
select 17638,'1968-S633','1968-12-07 04:33:00','suborbital rocket','success',1199,'SL725','MSSL I-28/RSRS-8','','','Ionospheric',1839,'',154 from dual union all
select 17639,'1968-S634','1968-12-07 06:11:00','suborbital rocket','success',30,'NASA 04.264CG','','','','XR Astron',1869,'',149 from dual union all
select 1764,'1953-A342','1953-09-03 00:00:00','atmospheric rocket','success',836,'1206A','','','','Test',1895,'',10 from dual union all
select 17640,'1968-S635','1968-12-09 13:17:00','suborbital rocket','success',920,'P9H','Winter abs/Strato','','','Ionospheric',1447,'',118 from dual union all
select 17641,'1968-S636','1968-12-09 21:55:00','suborbital rocket','success',848,'CRL AF07.660','Smoke','','','Aeron',394,'',124 from dual union all
select 17642,'1968-S637','1968-12-09 21:55:00','suborbital rocket','success',848,'CRL AF07.664','Smoke','','','Aeron',394,'',118 from dual union all
select 17643,'1968-S638','1968-12-10 00:00:00','military missile','success',795,'943','Mk 11 RV','ST Cold/Heat Soak 6','OLD FAITHFUL 2','OT',1701,'',900 from dual union all
select 17644,'1968-S639','1968-12-10 17:16:00','suborbital rocket','success',624,'NASA 8.51UA','He ionization','','','Aeronomy',428,'',837 from dual union all
select 17645,'1968-S64','1968-08-17 00:00:00','military missile','success',1449,'','8K84 UBP-86','','','OT',504,'',1000 from dual union all
select 17646,'1968-S640','1968-12-10 18:32:00','suborbital rocket','success',848,'CRL AG07.883','Mass spectrometer','','','Aeron',1821,'',122 from dual union all
select 17647,'1968-S641','1968-12-12 02:45:00','suborbital rocket','success',848,'CRL AH07.887','Mass spectrometer','','','Aeron',1821,'',118 from dual union all
select 17648,'1968-S642','1968-12-12 18:47:00','suborbital rocket','success',27,'AH1005/3','','','','Aeron',1833,'',165 from dual;
insert into LAUNCH
select 17649,'1968-S643','1968-07-17 08:25:00','suborbital rocket','success',839,'NASA 10.294GM','GSFC Grenade 179 W','','','Aeron',1821,'',120 from dual union all
select 1765,'1953-A343','1953-09-03 00:00:00','atmospheric rocket','success',836,'1225R','Nike 241R','','','Test',1895,'',10 from dual union all
select 17650,'1968-S644','1968-07-17 09:08:00','suborbital rocket','success',839,'NASA 10.295GM','GSFC Grenade 180 W','','','Aeron',1821,'',120 from dual union all
select 17651,'1968-S645','1968-12-13 02:10:00','military missile','success',135,'V118B [109]','SFT-13','ABRES AC-6 (USN)','','Reentry Test',613,'',200 from dual union all
select 17652,'1968-S646','1968-12-13 03:11:00','suborbital rocket','success',839,'NASA 10.285GM','GSFC Grenade 196 C','','','Aeronomy',428,'',121 from dual union all
select 17653,'1968-S647','1968-12-13 04:59:00','suborbital rocket','success',839,'NASA 10.291GM','GSFC Grenade 197 PB','','','Aeronomy',1039,'',100 from dual union all
select 17654,'1968-S648','1968-12-13 05:11:00','suborbital rocket','success',839,'NASA 10.286GM','GSFC Grenade 198 C','','','Aeronomy',428,'',124 from dual union all
select 17655,'1968-S649','1968-12-13 06:59:00','suborbital rocket','success',839,'NASA 10.292GM','GSFC Grenade 199 PB','','','Aeronomy',1039,'',93 from dual union all
select 17656,'1968-S65','1968-08-27 00:00:00','military missile','success',1449,'','8K84 UBP-87','','','OT',508,'',1000 from dual union all
select 17657,'1968-S650','1968-12-13 09:45:00','suborbital rocket','success',215,'AAF-2-125','Geminids','','','Ionos/Meteorites',427,'',143 from dual union all
select 17658,'1968-S651','1968-12-14 15:08:00','suborbital rocket','success',837,'NASA 14.352UA','Pandora G','','','Aeronomy/Meteorites',1895,'',159 from dual union all
select 17659,'1968-S652','1968-12-18 17:05:00','military missile','success',1002,'V 22500-04L','8K67P LKI-4','RGCh 8F676 LKI','','Test',511,'',1000 from dual union all
select 1766,'1953-A344','1953-09-03 00:00:00','atmospheric rocket','success',836,'1233R','Nike 239R','','','Test',1895,'',10 from dual union all
select 17660,'1968-S653','1968-12-18 00:00:00','suborbital rocket','success',1492,'61M/83','FU-159 PERSEE','','','UV Astron',341,'',162 from dual union all
select 17661,'1968-S654','1968-12-18 11:38:00','suborbital rocket','success',227,'AKD-VB-21','Calgary auroral','','','Auroral/Ionos',428,'',363 from dual union all
select 17662,'1968-S655','1968-12-20 14:30:00','military missile','success',796,'2418','Mk 11B RV','ST Mk 1A Penaids Test 1?','','Test',1703,'',900 from dual union all
select 17663,'1968-S656','1968-12-21 03:20:00','suborbital rocket','success',30,'NASA 04.178UG','Cornell FIR tel','','','IR Astron',1869,'',161 from dual union all
select 17664,'1968-S657','1968-12-21 00:00:00','military missile','success',1103,'V1','S02V-1','','','Test',301,'',1000 from dual union all
select 17665,'1968-S658','1968-12-22 00:00:00','suborbital rocket','success',1492,'61M/90','FU-159 Crab  X-rays','','','UV Astron,XR Astron',341,'',188 from dual union all
select 17666,'1968-S659','1968-02-07 19:58:03','suborbital rocket','success',1250,'Sparc 813303','UM Sparc Pitot','','','Aeron',1057,'',129 from dual union all
select 17667,'1968-S66','1968-08-30 00:00:00','military missile','success',1449,'','8K84 UBP-88','','','OT',504,'',1000 from dual union all
select 17668,'1968-S660','1968-02-08 19:34:17','suborbital rocket','success',1250,'Sparc 813344','UM Sparc Pitot','','','Aeron',1057,'',131 from dual union all
select 17669,'1968-S661','1968-08-03 00:00:00','military missile','success',916,'AO-69','P-153 68 Summer 06','','','Test',597,'',180 from dual union all
select 1767,'1953-A345','1953-09-03 00:00:00','atmospheric rocket','success',836,'1223R','Nike 240R','','','Test',1895,'',10 from dual union all
select 17670,'1968-S662','1968-11-01 00:00:00','suborbital rocket','success',1440,'','AFAL 4','','','Reentry Test',1821,'',300 from dual union all
select 17671,'1968-S663','1968-11-07 20:47:00','suborbital rocket','success',675,'WRE K118','Met','','','Meteo',1849,'',91 from dual union all
select 17672,'1968-S664','1968-11-20 18:03:00','suborbital rocket','success',675,'WRE K123','Met','','','Meteo',1849,'',80 from dual union all
select 17673,'1968-S665','1968-01-01 00:00:00','suborbital rocket','success',538,'','Explorer 37 cal','','','Aeron',1837,'',80 from dual union all
select 17674,'1968-S666','1968-04-20 00:00:00','suborbital rocket','success',1506,'SDC VD-14','ROBIN','','','Test',339,'',131 from dual union all
select 17675,'1968-S667','1968-04-20 00:00:00','suborbital rocket','success',1506,'SDC VD-16','','','','Aeron',339,'',124 from dual union all
select 17676,'1968-S669','1968-04-21 00:00:00','suborbital rocket','success',1506,'SDC VD-18','','','','Aeron',339,'',100 from dual union all
select 17677,'1968-S67','1968-09-16 00:00:00','military missile','success',1449,'','8K84 UBP-89','','','OT',508,'',1000 from dual union all
select 17678,'1968-S670','1968-04-21 00:00:00','suborbital rocket','success',1506,'SDC VD-19','','','','Aeron',339,'',126 from dual union all
select 17679,'1968-S671','1968-04-21 00:00:00','suborbital rocket','success',1506,'SDC VD-20','','','','Aeron',339,'',131 from dual union all
select 1768,'1953-A346','1953-09-04 00:00:00','military missile','failure',255,'623-3','','','','Test',254,'',20 from dual union all
select 17680,'1968-S672','1968-11-19 00:00:00','suborbital rocket','success',1506,'SDC VD-21','ROBIN','','','Aeron',276,'',142 from dual union all
select 17681,'1968-S673','1968-11-20 00:00:00','suborbital rocket','success',1506,'SDC VD-23','ROBIN','','','Aeron',276,'',129 from dual union all
select 17682,'1968-S674','1968-11-20 00:00:00','suborbital rocket','success',1506,'SDC VD-24','ROBIN','','','Aeron',276,'',129 from dual union all
select 17683,'1968-S675','1968-11-20 00:00:00','suborbital rocket','success',1506,'SDC VD-25','ROBIN','','','Aeron',276,'',128 from dual union all
select 17684,'1968-S676','1968-12-19 00:00:00','suborbital rocket','success',1506,'SDC VD-26','ROBIN','','','Aeron',276,'',121 from dual union all
select 17685,'1968-S677','1968-12-20 00:00:00','suborbital rocket','success',1506,'SDC VD-27','ROBIN','','','Aeron',276,'',139 from dual union all
select 17686,'1968-S678','1968-12-21 00:00:00','suborbital rocket','success',1506,'SDC VD-28','ROBIN','','','Aeron',276,'',137 from dual union all
select 17687,'1968-S679','1968-04-12 18:23:00','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',96 from dual union all
select 17688,'1968-S68','1968-10-12 00:00:00','military missile','success',1449,'','8K84 UBP-90','','','OT',505,'',1000 from dual union all
select 17689,'1968-S680','1968-04-17 20:43:01','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',94 from dual union all
select 1769,'1953-A347','1953-09-04 00:00:00','atmospheric rocket','unknown',576,'331','','','','Test',1895,'',10 from dual union all
select 17690,'1968-S681','1968-09-11 21:15:00','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',92 from dual union all
select 17691,'1968-S682','1968-09-11 23:15:00','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',93 from dual union all
select 17692,'1968-S683','1968-09-12 03:27:00','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',88 from dual union all
select 17693,'1968-S684','1968-12-18 00:00:00','military missile','success',356,'602801','R&D batch 02','','','Test',650,'',500 from dual union all
select 17694,'1968-S685','1968-10-10 12:30:00','suborbital rocket','success',846,'STV SR-076','','','','Aeron?',1895,'',150 from dual union all
select 17695,'1968-S686','1968-01-01 00:00:00','suborbital rocket','success',920,'','Test','','','Test',1447,'',140 from dual union all
select 17696,'1968-S687','1968-10-01 00:00:00','suborbital rocket','failure',1349,'','','','','Aeron?',803,'',150 from dual union all
select 17697,'1968-S688','1968-01-01 00:00:00','suborbital rocket','success',1,'','Yantar''-2','','','Tech/Ionos',636,'',400 from dual union all
select 17698,'1968-S689','1968-05-01 00:00:00','military missile','success',928,'A1SX-1','ISEP 4','Mk 2 RV','','Target',1900,'EAG-154',500 from dual union all
select 17699,'1968-S69','1968-10-19 00:00:00','military missile','success',1449,'','8K84 UBP-91','','','OT',508,'',1000 from dual union all
select 177,'1944-S160','1944-10-13 08:30:00','military missile','success',1470,'','Antwerp','','','Weapon',1728,'',90 from dual union all
select 1770,'1953-A348','1953-09-04 00:00:00','atmospheric rocket','unknown',576,'332','','','','Test',1895,'',10 from dual union all
select 17700,'1968-S690','1968-05-01 00:00:00','military missile','failure',928,'A1SX-2','ISEP 5','Mk 2 RV','','Target',1900,'EAG-154',100 from dual union all
select 17701,'1968-S691','1968-01-01 00:00:00','military missile','success',928,'Mk 2 RB','ISEP 6','Mk 2 RV','','Target',1900,'EAG-154',500 from dual union all
select 17702,'1968-S692','1968-03-16 00:00:00','military missile','success',932,'A3TY-04 (A3P-091)','USN Mk 2 Mod 2 RV x 2','Mk 2 Mod 2 RVs+Decoys','ANTELOPE 11','Target',1900,'EAG-154',500 from dual union all
select 17703,'1968-S693','1968-04-04 00:00:00','military missile','success',932,'A3TY-05 (A3P-102)','USN Mk 2 Mod 2 RV x 2','','ANTELOPE 12','Target',1900,'EAG-154',500 from dual union all
select 17704,'1968-S694','1968-04-11 00:00:00','military missile','success',932,'A3TY-07 (A3P-654)','USN Mk 2 Mod 2 RV x 2','','ANTELOPE 13','Target',1900,'EAG-154',500 from dual union all
select 17705,'1968-S695','1968-04-19 00:00:00','military missile','success',932,'A3TY-06 (A3P-103)','USN Mk 2 Mod 2 RV x 2','','ANTELOPE 14','Target',1900,'EAG-154',500 from dual union all
select 17706,'1968-S70','1968-10-28 00:00:00','military missile','success',1449,'','8K84 UBP-92','','','OT',505,'',1000 from dual union all
select 17707,'1968-S700','1968-04-30 00:00:00','military missile','success',916,'AO-62','P-146 68 Spring 04','','','OT',435,'',180 from dual union all
select 17708,'1968-S701','1968-10-24 00:00:00','suborbital rocket','success',839,'','','','','Aeron?',855,'',100 from dual union all
select 17709,'1968-S702','1968-10-24 00:00:00','suborbital rocket','success',839,'','','','','Aeron?',855,'',100 from dual union all
select 1771,'1953-A349','1953-09-09 00:00:00','atmospheric rocket','success',836,'1224A','','','','Test',1895,'',10 from dual union all
select 17710,'1968-S703','1968-11-08 00:00:00','suborbital rocket','success',867,'Sandia 152-121','Rocket EMP Test 1','','','Plasma/Active',803,'',150 from dual union all
select 17711,'1968-S704','1968-03-20 00:00:00','military missile','failure',982,'T 22505-18T','8K64U UBP-P43','R-16 PL68-02','','OT',477,'',1210 from dual union all
select 17712,'1968-S705','1968-03-27 00:00:00','military missile','success',982,'G 22501-17T','8K64U UBP-P44','R-16 PL68-03','','OT',477,'',1210 from dual union all
select 17713,'1968-S706','1968-04-20 00:00:00','military missile','success',982,'G 22604-16T','8K64U UBP-P46','R-16 PL68-05','','OT',477,'',1210 from dual union all
select 17714,'1968-S707','1968-05-31 00:00:00','military missile','success',982,'T 22502-03T','8K64U UBP-P48','R-16 PL68-07','','OT',477,'',1210 from dual union all
select 17715,'1968-S708','1968-06-20 00:00:00','military missile','success',982,'T 22504-04T','8K64U UBP-P49','R-16 PL68-08','','OT',477,'',1210 from dual union all
select 17716,'1968-S709','1968-06-28 00:00:00','military missile','success',982,'T 22504-19T','8K64U UBP-P50','R-16 PL68-09','','OT',477,'',1210 from dual union all
select 17717,'1968-S71','1968-10-29 00:00:00','military missile','success',1449,'','8K84 UBP-93','','','OT',508,'',1000 from dual union all
select 17718,'1968-S710','1968-07-25 00:00:00','military missile','success',982,'G 22503-11T','8K64U UBP-P51','R-16 PL68-10','','OT',477,'',1210 from dual union all
select 17719,'1968-S711','1968-09-09 00:00:00','military missile','success',982,'T 22504-09','8K64U UBP-P53','R-16 PL68-12','','OT',477,'',1210 from dual union all
select 1772,'1953-A35','1953-01-27 00:00:00','atmospheric rocket','success',836,'302P','Nike 302P CET 2','','','Test',1895,'',10 from dual union all
select 17720,'1968-S712','1968-09-14 00:00:00','military missile','success',982,'T 22505-01','8K64U UBP-P54','R-16 PL68-13','','OT',477,'',1210 from dual union all
select 17721,'1968-S713','1968-09-27 00:00:00','military missile','success',982,'T 22504-05','8K64U UBP-P55','R-16 PL68-14','','OT',477,'',1210 from dual union all
select 17722,'1968-S714','1968-10-16 00:00:00','military missile','success',982,'G 22502-08','8K64U UBP-P56','R-16 PL68-15','','OT',477,'',1210 from dual union all
select 17723,'1968-S715','1968-10-22 00:00:00','military missile','success',982,'T 22503-09','8K64U UBP-P57','R-16 PL68-16','','OT',477,'',1210 from dual union all
select 17724,'1968-S716','1968-10-26 00:00:00','military missile','success',982,'T 22504-01','8K64U UBP-P58','R-16 PL68-17','','OT',477,'',1210 from dual union all
select 17725,'1968-S717','1968-11-01 00:00:00','military missile','success',982,'G 22503-05T','8K64U UBP-P59','R-16 PL68-18','','OT',477,'',1210 from dual union all
select 17726,'1968-S718','1968-11-01 00:00:00','military missile','success',982,'G 22604-10T','8K64U UBP-P60','R-16 PL68-19','','OT',477,'',1210 from dual union all
select 17727,'1968-S719','1968-11-25 00:00:00','military missile','success',982,'T 22504-13T','8K64U UBP-P61','R-16 PL68-20','','OT',477,'',1210 from dual union all
select 17728,'1968-S72','1968-11-21 00:00:00','military missile','success',1449,'','8K84 UBP-94','','','OT',532,'',1000 from dual union all
select 17729,'1968-S722','1968-01-19 00:00:00','military missile','failure',982,'','8K64U UBP-P42','R-16 PL68-01','','OT',477,'',0 from dual union all
select 1773,'1953-A350','1953-09-09 00:00:00','atmospheric rocket','success',836,'1214A','','','','Test',1895,'',10 from dual union all
select 17730,'1968-S723','1968-04-15 00:00:00','military missile','success',982,'G 22604-07T','8K64U UBP-P45','R-16 PL68-04','','OT',477,'',1210 from dual union all
select 17731,'1968-S724','1968-05-25 00:00:00','military missile','success',982,'T 22502-13T','8K64U UBP-P47','R-16 PL68-06','','OT',477,'',1210 from dual union all
select 17732,'1968-S725','1968-08-13 00:00:00','military missile','failure',982,'','8K64U UBP-P52','R-16 PL68-11','','OT',477,'',0 from dual union all
select 17733,'1968-S726','1968-12-20 00:00:00','military missile','success',982,'T 22505-06T','8K64U UBP-P62','R-16 PL68-21','','OT',477,'',1210 from dual union all
select 17734,'1968-S727','1968-10-07 00:00:00','military missile','success',1018,'G 15002-03','8K75 UBP-P1','','','OT',477,'',1160 from dual union all
select 17735,'1968-S728','1968-10-25 00:00:00','military missile','success',1018,'G 15002-06','8K75 UBP-P2','','','OT',477,'',1160 from dual union all
select 17736,'1968-S73','1968-12-04 00:00:00','military missile','success',1449,'','8K84 UBP-95','','','OT',508,'',1000 from dual union all
select 17737,'1968-S730','1968-01-01 00:00:00','military missile','success',1449,'','8K84 54 RD UBP-1','','','OT',1468,'',1000 from dual union all
select 17738,'1968-S731','1968-01-01 00:00:00','military missile','success',1449,'','8K84 54 RD UBP-2','','','OT',1468,'',1000 from dual;
insert into LAUNCH
select 17739,'1968-S732','1968-01-01 00:00:00','military missile','success',1449,'','8K84 54 RD UBP-3','','','OT',1468,'',1000 from dual union all
select 1774,'1953-A351','1953-09-10 00:00:00','atmospheric rocket','unknown',576,'556','','','','Test',1895,'',10 from dual union all
select 17740,'1968-S733','1968-06-07 00:00:00','military missile','success',1018,'','8K75 UBP-T3','','','OT',1495,'',1160 from dual union all
select 17741,'1968-S734','1968-07-10 00:00:00','military missile','success',1449,'','8K84 52-RD UBP','52 RD','','OT',195,'',1000 from dual union all
select 17742,'1968-S735','1968-06-30 00:00:00','military missile','success',982,'','8K64 42 RD UBP-6','','','OT',1002,'',1210 from dual union all
select 17743,'1968-S736','1968-01-01 00:00:00','military missile','success',982,'','8K64 36 RD UBP-2','','','OT',898,'',1210 from dual union all
select 17744,'1968-S737','1968-01-01 00:00:00','military missile','success',982,'','8K64 36 RD UBP-3','','','OT',898,'',1210 from dual union all
select 17745,'1968-S738','1968-01-01 00:00:00','military missile','success',982,'','8K64 36 RD UBP-4','','','OT',898,'',1210 from dual union all
select 17746,'1968-S739','1968-01-01 00:00:00','military missile','success',1449,'','8K84 36 RD UBP-2','','','OT',898,'',1000 from dual union all
select 17747,'1968-S74','1968-12-12 00:00:00','military missile','success',1449,'','8K84 UBP-96','','','OT',508,'',1000 from dual union all
select 17748,'1968-S740','1968-03-01 00:00:00','military missile','success',1449,'','8K84 47 RD UBP-0','','','OT',1911,'',1000 from dual union all
select 17749,'1968-S741','1968-10-28 00:00:00','military missile','success',1037,'V29501-04','8K98 UBP-1','','','Test',477,'',1000 from dual union all
select 1775,'1953-A352','1953-09-10 00:00:00','atmospheric rocket','unknown',576,'557','','','','Test',1895,'',10 from dual union all
select 17750,'1968-S742','1968-11-14 00:00:00','military missile','success',1037,'V29501-03','8K98 UBP-2','','','Test',477,'',1000 from dual union all
select 17751,'1968-S743','1968-11-18 00:00:00','military missile','success',1037,'V29501-08','8K98 UBP-3','','','Test',477,'',1000 from dual union all
select 17752,'1968-S744','1968-12-29 00:00:00','military missile','success',1037,'V29502-02','8K98 UBP-4','','','Test',477,'',1000 from dual union all
select 17753,'1968-S745','1968-01-01 00:00:00','military missile','success',977,'','8K63U UBP','','','Test',840,'',402 from dual union all
select 17754,'1968-S746','1968-01-01 00:00:00','military missile','success',977,'','8K63U UBP','','','Test',840,'',402 from dual union all
select 17755,'1968-S747','1968-04-26 00:00:00','military missile','success',977,'','8K63U UBP 51 RB','','','Test',840,'',402 from dual union all
select 17756,'1968-S748','1968-06-02 00:00:00','military missile','success',976,'','8K63 UBP 51 RB','','','Test',840,'',402 from dual union all
select 17757,'1968-S749','1968-06-04 00:00:00','military missile','success',976,'','8K63 UBP 51 RB','','','Test',840,'',402 from dual union all
select 17758,'1968-S75','1968-12-24 00:00:00','military missile','success',1449,'','8K84 UBP-97','','','OT',505,'',1000 from dual union all
select 17759,'1968-S750','1968-06-06 00:00:00','military missile','success',976,'','8K63 UBP 51 RB','','','Test',840,'',402 from dual union all
select 1776,'1953-A353','1953-09-10 00:00:00','atmospheric rocket','success',836,'1253R','Nike 242R','','','Test',1895,'',10 from dual union all
select 17760,'1968-S751','1968-06-08 00:00:00','military missile','success',976,'','8K63 UBP 51 RB','','','Test',840,'',402 from dual union all
select 17761,'1968-S752','1968-06-05 00:00:00','military missile','success',980,'','8K65U 35 RD UBP-2','','','Test',943,'',675 from dual union all
select 17762,'1968-S753','1968-06-05 00:00:00','military missile','success',980,'','8K65U 35 RD UBP-3','','','Test',943,'',675 from dual union all
select 17763,'1968-S754','1968-08-01 00:00:00','military missile','success',980,'','8K65U 35 RD UBP-4','','','Test',943,'',675 from dual union all
select 17764,'1968-S755','1968-06-07 00:00:00','military missile','success',976,'','8K63 UBP 35 RB','','','Test',1013,'',402 from dual union all
select 17765,'1968-S756','1968-05-16 00:00:00','military missile','success',980,'','8K65U 68 RB UBP-1','','','Test',1420,'',675 from dual union all
select 17766,'1968-S757','1968-02-01 00:00:00','military missile','success',976,'','8K63 32 RD UBP','','','Test',636,'',402 from dual union all
select 17767,'1968-S758','1968-09-28 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 17768,'1968-S759','1968-12-25 00:00:00','military missile','success',976,'','8K63 43 RD UBP','','','Test',636,'',402 from dual union all
select 17769,'1968-S76','1968-01-01 00:00:00','military missile','success',1449,'','8K84 UBP-98','','','OT',596,'',1000 from dual union all
select 1777,'1953-A354','1953-09-11 00:00:00','atmospheric rocket','success',836,'1133A','','','','Test',1895,'',10 from dual union all
select 17770,'1968-S760','1968-05-24 00:00:00','military missile','success',977,'','8K63U 29 RD UBP','','','Test',636,'',402 from dual union all
select 17771,'1968-S761','1968-05-31 00:00:00','military missile','success',977,'','8K63U 29 RD UBP','','','Test',636,'',402 from dual union all
select 17772,'1968-S762','1968-11-14 00:00:00','military missile','success',977,'','8K63U 29 RD UBP','','','Test',636,'',402 from dual union all
select 17773,'1968-S763','1968-05-29 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 17774,'1968-S764','1968-06-20 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 17775,'1968-S765','1968-07-03 00:00:00','military missile','unknown',980,'','8K65U UBP 29 RD','','','Test',636,'',675 from dual union all
select 17776,'1968-S766','1968-07-18 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 17777,'1968-S767','1968-06-07 00:00:00','military missile','success',976,'','8K63 UBP 35 RB','','','Test',1013,'',402 from dual union all
select 17778,'1968-S77','1968-01-08 18:11:00','military missile','unknown',929,'A2P-273','','','','OT',421,'SSBN 608',1000 from dual union all
select 17779,'1968-S78','1968-01-10 09:58:00','military missile','success',796,'2437','Mk 11 RV','ST OLYMPIC TRIALS 6','OLYMPIC TRIALS 6','OT',1711,'',900 from dual union all
select 1778,'1953-A355','1953-09-11 00:00:00','atmospheric rocket','success',836,'1189A','','','','Test',1895,'',10 from dual union all
select 17780,'1968-S79','1968-01-11 00:00:00','military missile','success',135,'B022  [094]','FATE','ABRES I-7 (USAF)','','Reentry Test',613,'',200 from dual union all
select 17781,'1968-S80','1968-01-13 00:00:00','military missile','success',750,'A1','A1','','','Test',301,'',500 from dual union all
select 17782,'1968-S81','1968-01-15 17:43:00','military missile','failure',929,'A2P-138','','','','OT',421,'SSBN 609',10 from dual union all
select 17783,'1968-S83','1968-01-16 00:00:00','military missile','success',795,'(64-011)','Mk 11 RV','ST Cold/Heat Soak 3?','GLOWING SAND','OT',1701,'',900 from dual union all
select 17784,'1968-S84','1968-01-16 19:14:00','suborbital rocket','success',839,'NASA 10.273GI','','','','Ionos',1821,'',137 from dual union all
select 17785,'1968-S85','1968-01-16 19:29:00','suborbital rocket','success',839,'NASA 10.275GI','','','','Ionos',1821,'',141 from dual union all
select 17786,'1968-S86','1968-01-17 23:08:00','suborbital rocket','success',848,'CRL AG07.634','Niro TMA 1','','','Aeron',428,'',193 from dual union all
select 17787,'1968-S87','1968-01-18 00:00:00','military missile','success',135,'B039  [095]','ABRES X-1  (USN)','ABRES X-1 (USN)','','Reentry Test',612,'',200 from dual union all
select 17788,'1968-S88','1968-01-19 02:24:00','suborbital rocket','success',215,'AAF-2-108','Te/ne','','','Aeron/Ionos/Fields',428,'',142 from dual union all
select 17789,'1968-S89','1968-01-19 23:10:00','suborbital rocket','success',848,'CRL AG07.633','Niro TMA 2','','','Aeron',428,'',191 from dual union all
select 1779,'1953-A356','1953-09-14 00:00:00','military missile','unknown',339,'1352R','','','','Test',1895,'',30 from dual union all
select 17790,'1968-S90','1968-01-22 03:04:42','suborbital rocket','success',227,'ADD-VB-11','Hbeta/ne','','','Auroral/Ionos/Plasm',428,'',377 from dual union all
select 17791,'1968-S91','1968-01-22 04:30:00','suborbital rocket','success',867,'NASA 18.62GM','Airglow','','','Aeronomy',1821,'',296 from dual union all
select 17792,'1968-S92','1968-01-22 12:15:00','suborbital rocket','success',867,'CRL AG08.648','NO/TMA','','','Aeron',428,'',169 from dual union all
select 17793,'1968-S93','1968-01-22 13:20:00','suborbital rocket','success',848,'CRL AG07.635','Niro TMA 3/O2','','','Aeron',428,'',188 from dual union all
select 17794,'1968-S94','1968-01-22 17:05:00','suborbital rocket','success',30,'NRL NA3.201','KP3.25  BCS/PC','','','Solar X',1869,'',175 from dual union all
select 17795,'1968-S95','1968-01-23 00:00:00','military missile','unknown',1102,'','S01V-3','','','Test',301,'',500 from dual union all
select 17796,'1968-S96','1968-01-23 05:33:00','suborbital rocket','success',867,'NASA 18.13UE','Ionos currents','','','Aurora/Ionos',428,'',270 from dual union all
select 17797,'1968-S97','1968-01-23 05:58:00','suborbital rocket','success',215,'ADD-2-109/ADD-VB-109','Fields/Ionos/Mmet','','','Auroral/Ionos/Field',428,'',151 from dual union all
select 17798,'1968-S98','1968-01-24 17:15:00','military missile','unknown',929,'A2P-94','DEFT','','','OT',421,'SSBN 608',1000 from dual union all
select 17799,'1968-S99','1968-01-24 11:30:00','suborbital rocket','success',894,'Rad 2/68','','','','CR',308,'',90 from dual union all
select 178,'1944-S161','1944-10-01 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1728,'',90 from dual union all
select 1780,'1953-A357','1953-09-15 00:00:00','atmospheric rocket','unknown',576,'558','','','','Test',1895,'',10 from dual union all
select 17800,'1968-U01','1968-05-20 22:05:02','orbital','success',1007,'Ya22500-18L(6MO)','','OGCh No. 18LU','','Sat',519,'',null from dual union all
select 17801,'1968-U02','1968-05-27 23:15:01','orbital','success',1007,'Ya22501-19L(7MO)','','OGCh No. 19LU','','Sat',518,'',null from dual union all
select 17802,'1968-U03','1968-08-22 15:16:00','test rocket','success',1128,'S168C','RAM C-2','RAM C-2','','Reentry Test',1814,'',227 from dual union all
select 17803,'1968-W01','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17804,'1968-W02','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17805,'1968-W03','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17806,'1968-W04','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17807,'1968-W05','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17808,'1968-W06','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17809,'1968-W07','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1781,'1953-A358','1953-09-15 15:02:00','suborbital rocket','failure',51,'USAF 42','AF Airglow 5','','','Aeronomy SB',640,'',32 from dual union all
select 17810,'1968-W08','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17811,'1968-W09','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17812,'1968-W10','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17813,'1968-W100','1968-05-06 15:30:00','suborbital rocket','success',716,'Cal for Super Loki','HE Loki','','','Aeron',1895,'',72 from dual union all
select 17814,'1968-W101','1968-05-08 09:12:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',68 from dual union all
select 17815,'1968-W102','1968-05-15 09:04:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',70 from dual union all
select 17816,'1968-W103','1968-05-15 19:55:00','suborbital rocket','unknown',631,'EXAMETNET C36','Chaff','','','Aeron',308,'',68 from dual union all
select 17817,'1968-W104','1968-05-16 10:07:59','suborbital rocket','success',725,'','','','','Meteo',636,'',50 from dual union all
select 17818,'1968-W105','1968-05-17 06:30:00','suborbital rocket','success',88,'Sandia','Arcasonde','','','Aeron',855,'',60 from dual union all
select 17819,'1968-W106','1968-05-17 14:30:00','suborbital rocket','success',88,'Sandia','Arcasonde','','','Aeron',855,'',60 from dual union all
select 1782,'1953-A359','1953-09-16 00:00:00','atmospheric rocket','unknown',576,'313','','','','Test',1895,'',10 from dual union all
select 17820,'1968-W107','1968-05-17 23:37:00','suborbital rocket','success',88,'Sandia','Arcasonde','','','Aeron',855,'',60 from dual union all
select 17821,'1968-W108','1968-05-20 14:30:00','suborbital rocket','success',716,'Cal for Super Loki','HE Loki','','','Aeron',1895,'',85 from dual union all
select 17822,'1968-W109','1968-05-20 16:55:00','suborbital rocket','success',716,'Cal for Super Loki','','','','Aeron',1895,'',60 from dual union all
select 17823,'1968-W11','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17824,'1968-W110','1968-05-23 19:30:00','suborbital rocket','success',88,'Sandia','Arcasonde','','','Aeron',855,'',60 from dual union all
select 17825,'1968-W111','1968-05-29 17:32:00','suborbital rocket','unknown',631,'EXAMETNET C37','Chaff','','','Aeron',308,'',71 from dual union all
select 17826,'1968-W112','1968-06-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17827,'1968-W113','1968-06-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17828,'1968-W114','1968-06-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual;
insert into LAUNCH
select 17829,'1968-W115','1968-06-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 1783,'1953-A36','1953-01-28 00:00:00','atmospheric rocket','unknown',576,'86EN','','','','Test',1895,'',10 from dual union all
select 17830,'1968-W116','1968-06-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17831,'1968-W117','1968-06-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17832,'1968-W118','1968-06-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17833,'1968-W119','1968-06-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17834,'1968-W12','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17835,'1968-W120','1968-06-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17836,'1968-W121','1968-06-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17837,'1968-W122','1968-06-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17838,'1968-W123','1968-06-12 03:01:59','suborbital rocket','success',725,'','','','','Meteo',636,'',76 from dual union all
select 17839,'1968-W124','1968-06-12 09:27:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',76 from dual union all
select 1784,'1953-A360','1953-09-16 00:00:00','atmospheric rocket','unknown',576,'323','','','','Test',1895,'',10 from dual union all
select 17840,'1968-W125','1968-06-13 18:20:00','suborbital rocket','unknown',88,'EXAMETNET C38','Arcasonde 1A','','','Aeron',308,'',62 from dual union all
select 17841,'1968-W126','1968-06-19 11:02:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',76 from dual union all
select 17842,'1968-W127','1968-06-26 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',76 from dual union all
select 17843,'1968-W128','1968-06-26 16:50:00','suborbital rocket','unknown',631,'EXAMETNET C39','Chaff','','','Aeron',308,'',69 from dual union all
select 17844,'1968-W129','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17845,'1968-W13','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17846,'1968-W130','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17847,'1968-W131','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17848,'1968-W132','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17849,'1968-W133','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 1785,'1953-A361','1953-09-17 00:00:00','atmospheric rocket','success',836,'1249R','Nike 244R','','','Test',1895,'',10 from dual union all
select 17850,'1968-W134','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17851,'1968-W135','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17852,'1968-W136','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17853,'1968-W137','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17854,'1968-W138','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17855,'1968-W139','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17856,'1968-W14','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17857,'1968-W140','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17858,'1968-W141','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17859,'1968-W142','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 1786,'1953-A362','1953-09-17 00:00:00','atmospheric rocket','success',836,'1255R','Nike 243R','','','Test',1895,'',10 from dual union all
select 17860,'1968-W143','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17861,'1968-W144','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17862,'1968-W145','1968-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17863,'1968-W146','1968-07-02 20:27:59','suborbital rocket','success',725,'','','','','Meteo',636,'',75 from dual union all
select 17864,'1968-W147','1968-07-03 00:59:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',78 from dual union all
select 17865,'1968-W148','1968-07-03 05:03:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',78 from dual union all
select 17866,'1968-W149','1968-07-03 09:10:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',78 from dual union all
select 17867,'1968-W15','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17868,'1968-W150','1968-07-06 10:06:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',50 from dual union all
select 17869,'1968-W151','1968-07-10 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',78 from dual union all
select 1787,'1953-A363','1953-09-22 00:00:00','atmospheric rocket','success',836,'1144E','','','','Test',1895,'',10 from dual union all
select 17870,'1968-W152','1968-07-16 01:45:00','suborbital rocket','unknown',88,'EXAMETNET C40','Arcasonde 1A','','','Aeron',308,'',56 from dual union all
select 17871,'1968-W153','1968-07-17 02:03:59','suborbital rocket','success',725,'','','','','Meteo',636,'',54 from dual union all
select 17872,'1968-W154','1968-07-18 18:13:00','suborbital rocket','success',88,'FC-OR-348','Ozonosonde','','','Aeron',428,'',60 from dual union all
select 17873,'1968-W155','1968-07-19 18:36:00','suborbital rocket','success',88,'FC-OR-348','Ozonosonde','','','Aeron',428,'',60 from dual union all
select 17874,'1968-W156','1968-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17875,'1968-W157','1968-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17876,'1968-W158','1968-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17877,'1968-W159','1968-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17878,'1968-W16','1968-01-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 17879,'1968-W160','1968-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 1788,'1953-A364','1953-09-23 00:00:00','military missile','unknown',339,'1253R','','','','Test',1895,'',30 from dual union all
select 17880,'1968-W161','1968-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17881,'1968-W162','1968-07-31 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',77 from dual union all
select 17882,'1968-W163','1968-07-31 22:28:00','suborbital rocket','unknown',631,'EXAMETNET C41','Chaff','','','Aeron',308,'',65 from dual union all
select 17883,'1968-W164','1968-08-13 18:59:59','suborbital rocket','success',725,'','','','','Meteo',1562,'SHIP-SOV',78 from dual union all
select 17884,'1968-W165','1968-08-16 18:59:59','suborbital rocket','success',725,'','','','','Meteo',1562,'SHIP-SOV',78 from dual union all
select 17885,'1968-W166','1968-08-20 23:00:00','suborbital rocket','success',725,'','','','','Meteo',1322,'SHIP-SOV',78 from dual union all
select 17886,'1968-W167','1968-08-21 09:07:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',74 from dual union all
select 17887,'1968-W168','1968-08-21 17:00:00','suborbital rocket','success',725,'','','','','Meteo',1322,'SHIP-SOV',76 from dual union all
select 17888,'1968-W169','1968-08-23 18:59:59','suborbital rocket','success',725,'','','','','Meteo',1322,'SHIP-SOV',76 from dual union all
select 17889,'1968-W17','1968-01-04 20:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 1789,'1953-A365','1953-09-23 00:00:00','atmospheric rocket','unknown',576,'559','','','','Test',1895,'',10 from dual union all
select 17890,'1968-W170','1968-08-28 22:40:00','suborbital rocket','success',631,'EXAMETNET C43','Chaff','','','Aeron',308,'',68 from dual union all
select 17891,'1968-W171','1968-09-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17892,'1968-W172','1968-09-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17893,'1968-W173','1968-09-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17894,'1968-W174','1968-09-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17895,'1968-W175','1968-09-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17896,'1968-W176','1968-09-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17897,'1968-W177','1968-09-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17898,'1968-W178','1968-09-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17899,'1968-W179','1968-09-02 18:00:00','suborbital rocket','success',725,'','','','','Meteo',1558,'SHIP-SOV',78 from dual union all
select 179,'1944-S162','1944-10-01 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1739,'',90 from dual union all
select 1790,'1953-A366','1953-09-23 00:00:00','atmospheric rocket','unknown',576,'560','','','','Test',1895,'',10 from dual union all
select 17900,'1968-W18','1968-01-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17901,'1968-W180','1968-09-03 18:00:00','suborbital rocket','success',725,'','','','','Meteo',1558,'SHIP-SOV',78 from dual union all
select 17902,'1968-W181','1968-09-04 18:59:59','suborbital rocket','success',725,'','','','','Meteo',1558,'SHIP-SOV',78 from dual union all
select 17903,'1968-W182','1968-09-10 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1568,'SHIP-SOV',54 from dual union all
select 17904,'1968-W183','1968-09-10 23:00:00','suborbital rocket','success',725,'','','','','Meteo',1568,'SHIP-SOV',74 from dual union all
select 17905,'1968-W184','1968-09-15 02:00:00','suborbital rocket','success',759,'MT-135-036','','','','Aeron',839,'',55 from dual union all
select 17906,'1968-W185','1968-09-15 05:18:00','suborbital rocket','success',759,'MT-135-037','','','','Aeron',839,'',56 from dual union all
select 17907,'1968-W186','1968-09-15 15:59:59','suborbital rocket','success',725,'','','','','Meteo',1566,'SHIP-SOV',76 from dual union all
select 17908,'1968-W187','1968-09-16 18:07:00','suborbital rocket','success',88,'NASA 15.60UE','NOTS Ozone','','','Plasma',1821,'',55 from dual union all
select 17909,'1968-W188','1968-09-21 17:00:00','suborbital rocket','success',725,'','','','','Meteo',1564,'SHIP-SOV',76 from dual union all
select 1791,'1953-A367','1953-09-23 00:00:00','military missile','success',771,'GM-11082','','','','Test',222,'',12 from dual union all
select 17910,'1968-W189','1968-09-25 01:40:00','suborbital rocket','success',88,'PSU-NO-1','','','','Aeron',1895,'',70 from dual union all
select 17911,'1968-W19','1968-01-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17912,'1968-W190','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17913,'1968-W191','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17914,'1968-W192','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17915,'1968-W193','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17916,'1968-W194','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17917,'1968-W195','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17918,'1968-W196','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual;
insert into LAUNCH
select 17919,'1968-W197','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 1792,'1953-A368','1953-09-24 00:00:00','military missile','unknown',339,'1352R','','','','Test',1895,'',30 from dual union all
select 17920,'1968-W198','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17921,'1968-W199','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17922,'1968-W20','1968-01-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17923,'1968-W200','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17924,'1968-W201','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17925,'1968-W202','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17926,'1968-W203','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17927,'1968-W204','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17928,'1968-W205','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17929,'1968-W206','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 1793,'1953-A369','1953-09-24 00:00:00','atmospheric rocket','success',836,'1265R','Nike 245R','','','Test',1895,'',10 from dual union all
select 17930,'1968-W207','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17931,'1968-W208','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17932,'1968-W209','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17933,'1968-W21','1968-01-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17934,'1968-W210','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17935,'1968-W211','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17936,'1968-W212','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17937,'1968-W213','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17938,'1968-W214','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17939,'1968-W215','1968-10-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 1794,'1953-A37','1953-01-28 00:00:00','atmospheric rocket','unknown',576,'85EN','','','','Test',1895,'',10 from dual union all
select 17940,'1968-W216','1968-10-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17941,'1968-W217','1968-10-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17942,'1968-W218','1968-10-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17943,'1968-W219','1968-10-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17944,'1968-W22','1968-01-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17945,'1968-W220','1968-10-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17946,'1968-W221','1968-10-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17947,'1968-W222','1968-10-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17948,'1968-W223','1968-10-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17949,'1968-W224','1968-10-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 1795,'1953-A370','1953-09-25 00:00:00','atmospheric rocket','unknown',576,'70','','','','Test',1895,'',10 from dual union all
select 17950,'1968-W225','1968-10-01 00:00:00','suborbital rocket','success',779,'','','','','Aeron',933,'',65 from dual union all
select 17951,'1968-W226','1968-10-02 09:15:59','suborbital rocket','success',725,'','Particle flux 2','','','Meteo',1010,'',60 from dual union all
select 17952,'1968-W227','1968-10-10 19:00:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',60 from dual union all
select 17953,'1968-W228','1968-10-11 20:01:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 17954,'1968-W229','1968-10-16 10:01:59','suborbital rocket','success',725,'','','','','Meteo',636,'',75 from dual union all
select 17955,'1968-W23','1968-01-01 00:00:00','suborbital rocket','success',777,'1','','','','Test',933,'',68 from dual union all
select 17956,'1968-W230','1968-10-30 12:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',75 from dual union all
select 17957,'1968-W231','1968-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17958,'1968-W232','1968-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17959,'1968-W233','1968-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 1796,'1953-A371','1953-09-25 00:00:00','atmospheric rocket','success',836,'1128E','','','','Test',1895,'',10 from dual union all
select 17960,'1968-W234','1968-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17961,'1968-W235','1968-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17962,'1968-W236','1968-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17963,'1968-W237','1968-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17964,'1968-W238','1968-11-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17965,'1968-W239','1968-11-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17966,'1968-W24','1968-01-01 00:00:00','suborbital rocket','success',777,'2','','','','Test',933,'',68 from dual union all
select 17967,'1968-W240','1968-11-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17968,'1968-W241','1968-11-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17969,'1968-W242','1968-11-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 1797,'1953-A372','1953-09-25 00:00:00','atmospheric rocket','success',836,'1258E','','','','Test',1895,'',10 from dual union all
select 17970,'1968-W243','1968-11-04 08:03:00','suborbital rocket','success',725,'','','','','Meteo',636,'',65 from dual union all
select 17971,'1968-W244','1968-11-15 06:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',78 from dual union all
select 17972,'1968-W245','1968-11-20 19:41:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 17973,'1968-W246','1968-11-20 22:45:00','suborbital rocket','success',1284,'','','','','Aeron',276,'',75 from dual union all
select 17974,'1968-W247','1968-11-22 13:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',73 from dual union all
select 17975,'1968-W248','1968-11-27 03:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',76 from dual union all
select 17976,'1968-W249','1968-11-29 08:20:00','suborbital rocket','success',725,'','','','','Meteo',636,'',76 from dual union all
select 17977,'1968-W25','1968-01-03 09:00:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',74 from dual union all
select 17978,'1968-W250','1968-12-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17979,'1968-W251','1968-12-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 1798,'1953-A373','1953-09-25 00:00:00','atmospheric rocket','success',836,'1268E','','','','Test',1895,'',10 from dual union all
select 17980,'1968-W252','1968-12-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17981,'1968-W253','1968-12-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17982,'1968-W254','1968-12-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17983,'1968-W255','1968-12-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17984,'1968-W256','1968-12-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 17985,'1968-W257','1968-12-18 18:02:00','suborbital rocket','unknown',88,'EXAMETNET C50','Arcasonde 1A','','','Aeron',308,'',72 from dual union all
select 17986,'1968-W258','1968-12-18 15:00:00','suborbital rocket','success',88,'EXAMETNET N54','Arcasonde 1A','','','Aeron',330,'',58 from dual union all
select 17987,'1968-W259','1968-12-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17988,'1968-W26','1968-01-13 09:01:59','suborbital rocket','success',725,'','','','','Meteo',636,'',52 from dual union all
select 17989,'1968-W260','1968-12-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 1799,'1953-A374','1953-09-25 00:00:00','atmospheric rocket','success',836,'1263E','','','','Test',1895,'',10 from dual union all
select 17990,'1968-W261','1968-12-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17991,'1968-W262','1968-12-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17992,'1968-W263','1968-12-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 17993,'1968-W264','1968-12-04 09:04:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',76 from dual union all
select 17994,'1968-W265','1968-12-07 01:05:00','suborbital rocket','success',88,'PSU-NO-2','','','','Aeron',1895,'',70 from dual union all
select 17995,'1968-W266','1968-12-13 01:05:00','suborbital rocket','success',88,'PSU-NO-3','','','','Aeron',1895,'',70 from dual union all
select 17996,'1968-W267','1968-12-13 15:15:00','suborbital rocket','success',88,'','','','','Aeron',186,'',76 from dual union all
select 17997,'1968-W268','1968-12-13 17:20:00','suborbital rocket','success',88,'','','','','Aeron',186,'',78 from dual union all
select 17998,'1968-W269','1968-12-14 05:27:00','suborbital rocket','success',88,'','','','','Aeron',186,'',76 from dual union all
select 17999,'1968-W27','1968-01-17 05:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',56 from dual union all
select 18,'1943-A11','1943-07-09 00:00:00','military missile','failure',8,'41','','','','Test',675,'',0 from dual union all
select 180,'1944-S163','1944-10-01 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1737,'',90 from dual union all
select 1800,'1953-A375','1953-09-29 00:00:00','military missile','unknown',339,'1302R','','','','Test',1895,'',30 from dual union all
select 18000,'1968-W270','1968-12-14 07:15:00','suborbital rocket','success',88,'','','','','Aeron',186,'',79 from dual union all
select 18001,'1968-W271','1968-12-14 11:27:00','suborbital rocket','success',88,'','','','','Aeron',186,'',79 from dual union all
select 18002,'1968-W272','1968-12-19 18:00:00','suborbital rocket','success',1284,'','','','','Aeron',276,'',75 from dual union all
select 18003,'1968-W273','1968-12-19 19:10:00','suborbital rocket','success',1284,'','','','','Aeron',276,'',75 from dual union all
select 18004,'1968-W274','1968-12-23 08:30:00','suborbital rocket','success',725,'','','','','Meteo',636,'',76 from dual union all
select 18005,'1968-W275','1968-12-26 23:30:00','suborbital rocket','success',725,'','','','','Meteo',1322,'SHIP-SOV',74 from dual union all
select 18006,'1968-W276','1968-12-30 05:02:00','suborbital rocket','success',725,'','','','','Meteo',1322,'SHIP-SOV',78 from dual union all
select 18007,'1968-W277','1968-12-30 17:53:00','suborbital rocket','success',725,'','','','','Meteo',1322,'SHIP-SOV',74 from dual;
insert into LAUNCH
select 18008,'1968-W278','1968-01-09 00:50:00','suborbital rocket','success',88,'','Ozonesonde','','','Aeron',1895,'',60 from dual union all
select 18009,'1968-W279','1968-01-09 05:00:00','suborbital rocket','success',88,'','Ozonesonde','','','Aeron',1895,'',60 from dual union all
select 1801,'1953-A376','1953-09-29 00:00:00','atmospheric rocket','unknown',576,'359G','','','','Test',1895,'',10 from dual union all
select 18010,'1968-W28','1968-01-17 12:59:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',77 from dual union all
select 18011,'1968-W280','1968-01-09 13:35:00','suborbital rocket','success',88,'','Ozonesonde','','','Aeron',1895,'',60 from dual union all
select 18012,'1968-W281','1968-01-09 17:15:00','suborbital rocket','success',88,'','Ozonesonde','','','Aeron',1895,'',60 from dual union all
select 18013,'1968-W282','1968-01-09 22:15:00','suborbital rocket','success',88,'','Ozonesonde','','','Aeron',1895,'',50 from dual union all
select 18014,'1968-W283','1968-09-11 18:45:00','suborbital rocket','success',716,'STS','Balloon Cal','','','Meteo',1895,'',60 from dual union all
select 18015,'1968-W284','1968-09-11 18:21:00','suborbital rocket','success',88,'Arcasonde','','','','Meteo',1415,'',60 from dual union all
select 18016,'1968-W285','1968-04-05 18:07:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',1307,'',60 from dual union all
select 18017,'1968-W286','1968-07-19 17:32:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',1307,'',60 from dual union all
select 18018,'1968-W287','1968-10-09 18:29:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',1307,'',60 from dual union all
select 18019,'1968-W288','1968-12-18 01:05:00','suborbital rocket','success',88,'PSU-NO-4','Control flight','','','Aeron',1895,'',70 from dual union all
select 1802,'1953-A377','1953-10-01 00:00:00','military missile','unknown',339,'1299E','EU-9','','','Test',1895,'',30 from dual union all
select 18020,'1968-W29','1968-01-17 17:25:00','suborbital rocket','success',88,'EXAMETNET C28','Arcasonde 2B','','','Aeron',310,'',71 from dual union all
select 18021,'1968-W290','1968-12-18 09:12:00','suborbital rocket','success',725,'','Particle flux 7','','','Meteo',1010,'',88 from dual union all
select 18022,'1968-W291','1968-12-25 09:11:00','suborbital rocket','success',725,'','Particle flux 8','','','Meteo',1010,'',88 from dual union all
select 18023,'1968-W292','1968-10-23 18:00:00','suborbital rocket','success',88,'PEPP meteo','','','','Meteo',1895,'',60 from dual union all
select 18024,'1968-W293','1968-09-20 03:40:00','suborbital rocket','unknown',88,'EXAMETNET C44','Arcasonde 1A','','','Aeron',308,'',70 from dual union all
select 18025,'1968-W294','1968-09-25 19:59:00','suborbital rocket','success',631,'EXAMETNET C45','Chaff','','','Aeron',308,'',65 from dual union all
select 18026,'1968-W295','1968-10-02 15:20:00','suborbital rocket','success',631,'EXAMETNET C46','Chaff','','','Aeron',308,'',65 from dual union all
select 18027,'1968-W296','1968-10-10 18:35:00','suborbital rocket','unknown',88,'EXAMETNET C47','Arcasonde 1A','','','Aeron',308,'',62 from dual union all
select 18028,'1968-W297','1968-11-13 19:10:00','suborbital rocket','unknown',88,'EXAMETNET C48','Arcasonde 1A','','','Aeron',308,'',76 from dual union all
select 18029,'1968-W298','1968-11-27 15:50:00','suborbital rocket','unknown',894,'EXAMETNET C49','Chaff','','','Aeron',308,'',56 from dual union all
select 1803,'1953-A378','1953-10-01 00:00:00','atmospheric rocket','unknown',576,'321S','','','','Test',1895,'',10 from dual union all
select 18030,'1968-W299','1968-01-17 15:00:00','suborbital rocket','success',631,'EXAMETNET N33','WOX-1A','','','Aeron',330,'',56 from dual union all
select 18031,'1968-W30','1968-01-23 02:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 18032,'1968-W301','1968-01-31 15:12:00','suborbital rocket','success',631,'EXAMETNET N34','Chaff','','','Aeron',330,'',60 from dual union all
select 18033,'1968-W302','1968-04-17 15:00:00','suborbital rocket','success',631,'EXAMETNET N35','WOX-1A','','','Aeron',330,'',60 from dual union all
select 18034,'1968-W303','1968-05-01 15:00:00','suborbital rocket','success',631,'EXAMETNET N36','Chaff','','','Aeron',330,'',62 from dual union all
select 18035,'1968-W304','1968-05-15 15:00:00','suborbital rocket','success',631,'EXAMETNET N37','WOX-1A','','','Aeron',330,'',52 from dual union all
select 18036,'1968-W305','1968-05-29 15:00:00','suborbital rocket','success',631,'EXAMETNET N38','Chaff','','','Aeron',330,'',63 from dual union all
select 18037,'1968-W306','1968-06-12 15:02:00','suborbital rocket','success',631,'EXAMETNET N39','Chaff','','','Aeron',330,'',65 from dual union all
select 18038,'1968-W307','1968-07-24 16:02:00','suborbital rocket','success',88,'EXAMETNET N40','Arcasonde 1A','','','Aeron',330,'',60 from dual union all
select 18039,'1968-W308','1968-09-18 15:19:00','suborbital rocket','success',88,'EXAMETNET N41','Arcasonde 1A','','','Aeron',330,'',59 from dual union all
select 1804,'1953-A379','1953-10-01 00:00:00','atmospheric rocket','success',836,'1257R','Nike 246R','','','Test',1895,'',10 from dual union all
select 18040,'1968-W309','1968-10-16 15:14:00','suborbital rocket','success',88,'EXAMETNET N52','Arcasonde 1A','','','Aeron',330,'',62 from dual union all
select 18041,'1968-W31','1968-01-24 10:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',51 from dual union all
select 18042,'1968-W310','1968-11-13 15:30:00','suborbital rocket','success',88,'EXAMETNET N53','Arcasonde 1A','','','Aeron',330,'',61 from dual union all
select 18043,'1968-W312','1968-01-05 16:50:00','suborbital rocket','success',88,'EXAMETNET W 67','Arcasonde 1A','','','Meteo',1821,'',53 from dual union all
select 18044,'1968-W313','1968-01-10 18:40:00','suborbital rocket','success',88,'EXAMETNET W 68','Arcasonde 1A','','','Meteo',1821,'',58 from dual union all
select 18045,'1968-W314','1968-01-17 16:12:00','suborbital rocket','success',88,'EXAMETNET W 69','Arcasonde 1A','','','Meteo',1821,'',58 from dual union all
select 18046,'1968-W315','1968-01-26 15:59:00','suborbital rocket','success',88,'EXAMETNET W 70','Arcasonde 1A','','','Meteo',1821,'',55 from dual union all
select 18047,'1968-W316','1968-02-07 16:44:00','suborbital rocket','success',631,'EXAMETNET W 71','Chaff','','','Meteo',1821,'',65 from dual union all
select 18048,'1968-W317','1968-02-14 15:13:00','suborbital rocket','success',88,'EXAMETNET W 72','Arcasonde 1A','','','Meteo',1821,'',62 from dual union all
select 18049,'1968-W318','1968-02-20 21:44:00','suborbital rocket','success',88,'EXAMETNET W 73','Arcasonde 1A','','','Meteo',1821,'',55 from dual union all
select 1805,'1953-A38','1953-01-29 00:00:00','atmospheric rocket','unknown',576,'87EO','','','','Test',1895,'',10 from dual union all
select 18050,'1968-W319','1968-02-28 15:28:00','suborbital rocket','success',88,'EXAMETNET W 74','Arcasonde 1A','','','Meteo',1821,'',53 from dual union all
select 18051,'1968-W32','1968-01-24 12:56:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',74 from dual union all
select 18052,'1968-W320','1968-03-06 15:03:00','suborbital rocket','success',88,'EXAMETNET W 75','Arcasonde 1A','','','Meteo',1821,'',59 from dual union all
select 18053,'1968-W321','1968-03-14 15:00:00','suborbital rocket','success',88,'EXAMETNET W 76','Arcasonde 1A','','','Meteo',1821,'',53 from dual union all
select 18054,'1968-W322','1968-03-21 16:18:00','suborbital rocket','success',88,'EXAMETNET W 77','Arcasonde 1A','','','Meteo',1821,'',56 from dual union all
select 18055,'1968-W323','1968-03-27 18:41:00','suborbital rocket','success',88,'EXAMETNET W 78','Arcasonde 1A','','','Meteo',1821,'',60 from dual union all
select 18056,'1968-W324','1968-04-03 15:59:00','suborbital rocket','success',88,'EXAMETNET W 79','Arcasonde 1A','','','Meteo',1821,'',60 from dual union all
select 18057,'1968-W325','1968-04-10 15:05:00','suborbital rocket','success',88,'EXAMETNET W 80','Arcasonde 1A','','','Meteo',1821,'',60 from dual union all
select 18058,'1968-W326','1968-04-17 18:51:00','suborbital rocket','success',631,'EXAMETNET W 81','WOX-1A','','','Meteo',1821,'',57 from dual union all
select 18059,'1968-W327','1968-04-24 16:12:00','suborbital rocket','success',631,'EXAMETNET W 82','WOX-1A','','','Meteo',1821,'',51 from dual union all
select 1806,'1953-A380','1953-10-01 00:00:00','atmospheric rocket','success',836,'1267R','Nike 247R','','','Test',1895,'',10 from dual union all
select 18060,'1968-W328','1968-05-02 14:59:00','suborbital rocket','success',631,'EXAMETNET W 83','WOX-1A','','','Meteo',1821,'',60 from dual union all
select 18061,'1968-W329','1968-05-08 14:59:00','suborbital rocket','success',88,'EXAMETNET W 84','Arcasonde 1A','','','Meteo',1821,'',60 from dual union all
select 18062,'1968-W33','1968-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 18063,'1968-W330','1968-05-15 17:12:00','suborbital rocket','success',88,'EXAMETNET W 85','Arcasonde 1A','','','Meteo',1821,'',58 from dual union all
select 18064,'1968-W331','1968-05-22 14:20:00','suborbital rocket','success',88,'EXAMETNET W 86','Arcasonde 1A','','','Meteo',1821,'',63 from dual union all
select 18065,'1968-W332','1968-06-05 19:15:00','suborbital rocket','success',88,'EXAMETNET W 87','Arcasonde 1A','','','Meteo',1821,'',60 from dual union all
select 18066,'1968-W333','1968-06-13 14:13:00','suborbital rocket','success',88,'EXAMETNET W 88','Arcasonde 1A','','','Meteo',1821,'',57 from dual union all
select 18067,'1968-W334','1968-06-19 14:32:00','suborbital rocket','success',88,'EXAMETNET W 89','Arcasonde 1A','','','Meteo',1821,'',60 from dual union all
select 18068,'1968-W335','1968-07-02 15:54:00','suborbital rocket','success',88,'EXAMETNET W 90','Arcasonde 1A','','','Meteo',1821,'',54 from dual union all
select 18069,'1968-W336','1968-07-10 14:47:00','suborbital rocket','success',88,'EXAMETNET W 91','Arcasonde 1A','','','Meteo',1821,'',54 from dual union all
select 1807,'1953-A381','1953-10-01 00:00:00','atmospheric rocket','success',836,'1269R','Nike 248R','','','Test',1895,'',10 from dual union all
select 18070,'1968-W337','1968-07-17 14:12:00','suborbital rocket','success',88,'EXAMETNET W 92','Arcasonde 1A','','','Meteo',1821,'',57 from dual union all
select 18071,'1968-W338','1968-07-24 15:06:00','suborbital rocket','success',88,'EXAMETNET W 93','Arcasonde 1A','','','Meteo',1821,'',58 from dual union all
select 18072,'1968-W339','1968-07-31 16:18:00','suborbital rocket','success',88,'EXAMETNET W 94','Arcasonde 1A','','','Meteo',1821,'',60 from dual union all
select 18073,'1968-W34','1968-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 18074,'1968-W340','1968-08-08 15:06:00','suborbital rocket','success',631,'EXAMETNET W 95','WOX-1A','','','Meteo',1821,'',59 from dual union all
select 18075,'1968-W341','1968-08-14 17:30:00','suborbital rocket','success',88,'EXAMETNET W 96','Arcasonde 1A','','','Meteo',1821,'',59 from dual union all
select 18076,'1968-W342','1968-08-21 13:21:00','suborbital rocket','success',88,'EXAMETNET W 97','Arcasonde 1A','','','Meteo',1821,'',60 from dual union all
select 18077,'1968-W343','1968-08-28 14:21:00','suborbital rocket','success',88,'EXAMETNET W 98','Arcasonde 1A','','','Meteo',1821,'',53 from dual union all
select 18078,'1968-W344','1968-09-04 15:44:00','suborbital rocket','success',88,'EXAMETNET W 99','Arcasonde 1A','','','Meteo',1821,'',54 from dual union all
select 18079,'1968-W345','1968-09-11 18:10:00','suborbital rocket','success',88,'EXAMETNET W 100','Arcasonde 1A','','','Meteo',1821,'',61 from dual union all
select 1808,'1953-A382','1953-10-02 00:00:00','atmospheric rocket','unknown',576,'319S','','','','Test',1895,'',10 from dual union all
select 18080,'1968-W346','1968-09-18 14:41:00','suborbital rocket','success',88,'EXAMETNET W 101','Arcasonde 1A','','','Meteo',1821,'',61 from dual union all
select 18081,'1968-W347','1968-09-25 14:24:00','suborbital rocket','success',88,'EXAMETNET W 102','Arcasonde 1A','','','Meteo',1821,'',58 from dual union all
select 18082,'1968-W348','1968-10-02 17:45:00','suborbital rocket','success',88,'EXAMETNET W 103','Arcasonde 1A','','','Meteo',1821,'',61 from dual union all
select 18083,'1968-W349','1968-10-11 15:17:00','suborbital rocket','success',520,'EXAMETNET W 104','Chute','','','Meteo',1790,'',68 from dual union all
select 18084,'1968-W35','1968-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 18085,'1968-W350','1968-10-16 19:20:00','suborbital rocket','success',88,'EXAMETNET W 105','Arcasonde 1A','','','Meteo',1821,'',61 from dual union all
select 18086,'1968-W351','1968-10-23 18:10:00','suborbital rocket','success',88,'EXAMETNET W 106','Arcasonde 1A','','','Meteo',1821,'',69 from dual union all
select 18087,'1968-W352','1968-10-30 16:52:00','suborbital rocket','success',88,'EXAMETNET W 107','Arcasonde 1A','','','Meteo',1821,'',60 from dual union all
select 18088,'1968-W353','1968-12-11 20:59:00','suborbital rocket','success',88,'EXAMETNET W 113','Arcasonde 1A','','','Meteo',1821,'',67 from dual union all
select 18089,'1968-W354','1968-11-13 15:52:00','suborbital rocket','success',716,'EXAMETNET W 109','Datasonde','','','Meteo',1821,'',68 from dual union all
select 1809,'1953-A383','1953-10-05 00:00:00','atmospheric rocket','unknown',576,'2SP','','','','Test',1895,'',10 from dual union all
select 18090,'1968-W355','1968-11-20 15:56:00','suborbital rocket','success',716,'EXAMETNET W 110','Datasonde','','','Meteo',1821,'',43 from dual union all
select 18091,'1968-W356','1968-11-27 16:26:00','suborbital rocket','success',716,'EXAMETNET W 111','Datasonde','','','Meteo',1821,'',63 from dual union all
select 18092,'1968-W357','1968-12-05 15:25:00','suborbital rocket','success',716,'EXAMETNET W 112','Datasonde','','','Meteo',1821,'',61 from dual union all
select 18093,'1968-W358','1968-12-19 19:22:00','suborbital rocket','success',716,'EXAMETNET W 114','Datasonde','','','Meteo',1821,'',57 from dual union all
select 18094,'1968-W359','1968-11-07 15:51:00','suborbital rocket','success',716,'EXAMETNET W 108','Datasonde','','','Meteo',1821,'',64 from dual union all
select 18095,'1968-W36','1968-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 18096,'1968-W37','1968-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 18097,'1968-W38','1968-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 18098,'1968-W39','1968-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual;
insert into LAUNCH
select 18099,'1968-W40','1968-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 181,'1944-S164','1944-09-30 16:05:00','military missile','success',1470,'18774','Ma189','','','Test',661,'',90 from dual union all
select 1810,'1953-A384','1953-10-05 00:00:00','atmospheric rocket','unknown',576,'4SP','','','','Test',1895,'',10 from dual union all
select 18100,'1968-W41','1968-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 18101,'1968-W42','1968-02-01 00:00:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',60 from dual union all
select 18102,'1968-W43','1968-02-05 23:14:00','suborbital rocket','success',88,'','Arcasonde 1A','','','Aeron',428,'',65 from dual union all
select 18103,'1968-W44','1968-02-02 00:00:00','suborbital rocket','success',631,'','','','','Aeron',1467,'',70 from dual union all
select 18104,'1968-W45','1968-02-05 09:40:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',77 from dual union all
select 18105,'1968-W46','1968-02-05 10:00:59','suborbital rocket','success',725,'','','','','Meteo',636,'',64 from dual union all
select 18106,'1968-W47','1968-02-12 10:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',50 from dual union all
select 18107,'1968-W48','1968-02-14 13:33:00','suborbital rocket','success',631,'EXAMETNET C29','Chaff','','','Aeron',310,'',68 from dual union all
select 18108,'1968-W49','1968-02-16 04:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',78 from dual union all
select 18109,'1968-W50','1968-02-28 04:01:59','suborbital rocket','success',725,'','','','','Meteo',636,'',52 from dual union all
select 1811,'1953-A385','1953-10-05 00:00:00','atmospheric rocket','unknown',576,'358G','','','','Test',1895,'',10 from dual union all
select 18110,'1968-W51','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18111,'1968-W52','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18112,'1968-W53','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18113,'1968-W54','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18114,'1968-W55','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18115,'1968-W56','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18116,'1968-W57','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18117,'1968-W58','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18118,'1968-W59','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18119,'1968-W60','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 1812,'1953-A386','1953-10-05 00:00:00','military missile','success',572,'4','RV-A-8','','','Test',1863,'',30 from dual union all
select 18120,'1968-W61','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18121,'1968-W62','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18122,'1968-W63','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18123,'1968-W64','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18124,'1968-W65','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18125,'1968-W66','1968-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18126,'1968-W67','1968-03-04 10:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',55 from dual union all
select 18127,'1968-W68','1968-03-06 00:00:00','suborbital rocket','success',462,'1','','','','Weather',313,'',60 from dual union all
select 18128,'1968-W69','1968-03-13 15:28:00','suborbital rocket','success',88,'EXAMETNET C30','Arcasonde 1A','','','Aeron',310,'',74 from dual union all
select 18129,'1968-W70','1968-03-20 16:00:00','suborbital rocket','success',631,'EXAMETNET C31','Chaff','','','Aeron',310,'',72 from dual union all
select 1813,'1953-A387','1953-10-06 00:00:00','atmospheric rocket','success',836,'1234A','','','','Test',1895,'',10 from dual union all
select 18130,'1968-W71','1968-03-27 15:58:00','suborbital rocket','success',88,'EXAMETNET C32','Arcasonde 1A','','','Aeron',310,'',75 from dual union all
select 18131,'1968-W73','1968-04-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18132,'1968-W74','1968-04-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18133,'1968-W75','1968-04-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18134,'1968-W76','1968-04-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18135,'1968-W77','1968-04-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18136,'1968-W78','1968-04-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18137,'1968-W79','1968-04-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18138,'1968-W80','1968-04-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18139,'1968-W81','1968-04-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 1814,'1953-A388','1953-10-06 00:00:00','atmospheric rocket','success',836,'1236A','','','','Test',1895,'',10 from dual union all
select 18140,'1968-W82','1968-04-03 13:39:00','suborbital rocket','success',631,'EXAMETNET C33','Chaff','','','Aeron',310,'',70 from dual union all
select 18141,'1968-W83','1968-04-04 18:51:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 18142,'1968-W84','1968-04-05 09:01:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',76 from dual union all
select 18143,'1968-W85','1968-04-08 00:00:00','suborbital rocket','failure',1281,'NASA 12.11GM','','','','Aeronomy/Test',1821,'',60 from dual union all
select 18144,'1968-W86','1968-04-10 02:10:00','suborbital rocket','success',88,'EXAMETNET C34','Arcasonde 1A','','','Aeron',310,'',75 from dual union all
select 18145,'1968-W87','1968-04-14 02:03:00','suborbital rocket','unknown',258,'','','','','Ionos',428,'',60 from dual union all
select 18146,'1968-W88','1968-04-14 05:19:00','suborbital rocket','unknown',258,'','','','','Ionos',428,'',60 from dual union all
select 18147,'1968-W89','1968-04-17 16:26:00','suborbital rocket','success',631,'EXAMETNET C35','Chaff','','','Aeron',310,'',72 from dual union all
select 18148,'1968-W90','1968-04-19 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',78 from dual union all
select 18149,'1968-W91','1968-05-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 1815,'1953-A389','1953-10-06 00:00:00','atmospheric rocket','success',836,'1149A','','','','Test',1895,'',10 from dual union all
select 18150,'1968-W92','1968-05-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18151,'1968-W93','1968-05-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18152,'1968-W94','1968-05-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18153,'1968-W95','1968-05-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18154,'1968-W96','1968-05-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18155,'1968-W97','1968-05-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 18156,'1968-W98','1968-05-04 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',76 from dual union all
select 18157,'1968-W99','1968-05-06 14:15:00','suborbital rocket','success',716,'Cal for Super Loki','','','','Aeron',1895,'',55 from dual union all
select 18158,'1968-Y01','1968-01-03 10:03:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 18159,'1968-Y02','1968-01-17 00:27:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 1816,'1953-A39','1953-01-29 00:00:00','atmospheric rocket','success',836,'1249A-179','Nike 1249 R&D 170R','','','Test',1895,'',10 from dual union all
select 18160,'1968-Y03','1968-01-17 00:59:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 18161,'1968-Y04','1968-01-17 08:26:00','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 18162,'1968-Y05','1968-01-17 12:27:59','suborbital rocket','success',725,'','','','','Meteo',636,'',86 from dual union all
select 18163,'1968-Y06','1968-01-17 16:23:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 18164,'1968-Y07','1968-01-20 09:53:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 18165,'1968-Y08','1968-01-22 00:00:00','suborbital rocket','success',88,'','Quark 1 (MRCMcD)','','','Aeron/CR',1895,'',80 from dual union all
select 18166,'1968-Y09','1968-01-31 10:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 18167,'1968-Y10','1968-02-18 18:07:00','suborbital rocket','success',1506,'SDC VD-11','','','','Aeron',339,'',122 from dual union all
select 18168,'1968-Y100','1968-12-05 13:48:00','suborbital rocket','success',1188,'Su12H','','','','Ionospheric',1447,'',92 from dual union all
select 18169,'1968-Y101','1968-12-11 04:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 1817,'1953-A390','1953-10-07 00:00:00','atmospheric rocket','success',365,'','F8U-1 Model','','','Test',1821,'',10 from dual union all
select 18170,'1968-Y102','1968-12-11 09:00:00','suborbital rocket','success',725,'','Particle flux 6','','','Meteo',1010,'',88 from dual union all
select 18171,'1968-Y103','1968-12-14 09:19:00','suborbital rocket','success',88,'','','','','Aeron',186,'',81 from dual union all
select 18172,'1968-Y104','1968-12-14 10:00:00','suborbital rocket','success',88,'','','','','Aeron',186,'',80 from dual union all
select 18173,'1968-Y105','1968-12-20 11:00:00','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 18174,'1968-Y106','1968-12-26 15:59:59','suborbital rocket','success',725,'','','','','Meteo',1322,'SHIP-SOV',80 from dual union all
select 18175,'1968-Y107','1968-12-27 05:11:00','suborbital rocket','success',725,'','','','','Meteo',1322,'SHIP-SOV',80 from dual union all
select 18176,'1968-Y108','1968-12-30 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1322,'SHIP-SOV',80 from dual union all
select 18177,'1968-Y109','1968-12-30 15:49:59','suborbital rocket','success',725,'','','','','Meteo',1322,'SHIP-SOV',80 from dual union all
select 18178,'1968-Y11','1968-02-18 18:57:00','suborbital rocket','success',1506,'SDC VD-12','','','','Aeron',339,'',123 from dual union all
select 18179,'1968-Y110','1968-12-30 23:51:00','suborbital rocket','success',725,'','','','','Meteo',1322,'SHIP-SOV',80 from dual union all
select 1818,'1953-A391','1953-10-07 00:00:00','atmospheric rocket','success',564,'','D18 F8U-1 Model','','','Test',1821,'',10 from dual union all
select 18180,'1968-Y12','1968-02-18 20:07:00','suborbital rocket','success',1506,'SDC VD-13','','','','Aeron',339,'',114 from dual union all
select 18181,'1968-Y13','1968-02-19 09:01:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 18182,'1968-Y14','1968-02-26 09:01:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',84 from dual union all
select 18183,'1968-Y15','1968-02-29 20:49:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 18184,'1968-Y16','1968-03-06 00:00:00','suborbital rocket','success',88,'','Quark 2 (MRCMcD)','','','Aeron/CR',1895,'',80 from dual union all
select 18185,'1968-Y17','1968-03-08 06:35:00','suborbital rocket','success',257,'NASA 15.58UE','ISRO 12.01 BRAHMA-I','TERLS-068','','Plasma',1467,'',83 from dual union all
select 18186,'1968-Y18','1968-03-08 12:40:00','suborbital rocket','success',257,'NASA 15.59UE','ISRO 12.02 BRAHMA-II','TERLS-069','','Plasma',1467,'',94 from dual union all
select 18187,'1968-Y19','1968-03-13 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 18188,'1968-Y20','1968-03-15 03:00:00','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual;
insert into LAUNCH
select 18189,'1968-Y21','1968-03-29 13:53:00','suborbital rocket','success',1188,'Su8H','','','','Ionospheric',1447,'',88 from dual union all
select 1819,'1953-A392','1953-10-08 00:00:00','military missile','unknown',339,'1355R','Type II proto','','','Test',1895,'',30 from dual union all
select 18190,'1968-Y22','1968-04-20 22:10:00','suborbital rocket','success',1506,'SDC VD-15','','','','Aeron',339,'',126 from dual union all
select 18191,'1968-Y23','1968-04-22 00:00:00','suborbital rocket','success',1284,'Test 1','','','','Test',1895,'',128 from dual union all
select 18192,'1968-Y24','1968-05-02 22:42:00','suborbital rocket','success',257,'','325-1','','','Ionos',428,'',95 from dual union all
select 18193,'1968-Y25','1968-05-06 00:00:00','suborbital rocket','success',1284,'Test 3','','','','Test',1895,'',125 from dual union all
select 18194,'1968-Y26','1968-05-06 10:06:00','suborbital rocket','success',725,'','','','','Meteo',636,'',82 from dual union all
select 18195,'1968-Y27','1968-05-06 16:32:00','suborbital rocket','success',1284,'Test 2','','','','Test',1895,'',125 from dual union all
select 18196,'1968-Y28','1968-05-20 00:00:00','suborbital rocket','success',1284,'Test 4','','','','Test',1895,'',125 from dual union all
select 18197,'1968-Y29','1968-05-20 00:00:00','suborbital rocket','success',1284,'Test 5','','','','Test',1895,'',123 from dual union all
select 18198,'1968-Y30','1968-05-22 00:00:00','suborbital rocket','success',88,'','Quark 3 (MRCMcD)','','','Aeron/CR',1895,'',80 from dual union all
select 18199,'1968-Y31','1968-05-23 09:34:59','suborbital rocket','success',725,'','','','','Meteo',636,'',84 from dual union all
select 182,'1944-S165','1944-09-30 20:48:00','military missile','success',1470,'18788','Ma182','','','Test',661,'',90 from dual union all
select 1820,'1953-A393','1953-10-08 00:00:00','atmospheric rocket','success',836,'1275R','Nike 249R','','','Test',1895,'',10 from dual union all
select 18200,'1968-Y32','1968-05-29 03:01:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 18201,'1968-Y33','1968-05-29 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 18202,'1968-Y34','1968-06-05 03:01:59','suborbital rocket','success',725,'','','','','Meteo',636,'',83 from dual union all
select 18203,'1968-Y35','1968-06-10 02:35:00','suborbital rocket','success',257,'','325-2','','','Ionos',428,'',95 from dual union all
select 18204,'1968-Y36','1968-06-19 03:00:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 18205,'1968-Y37','1968-06-22 15:15:00','suborbital rocket','success',1188,'Su4H','','','','Ionospheric',1447,'',101 from dual union all
select 18206,'1968-Y38','1968-06-26 03:03:59','suborbital rocket','success',725,'','','','','Meteo',636,'',82 from dual union all
select 18207,'1968-Y39','1968-07-01 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 18208,'1968-Y40','1968-07-02 13:43:00','suborbital rocket','success',1188,'Su5H','','','','Ionospheric',1447,'',90 from dual union all
select 18209,'1968-Y41','1968-07-03 00:27:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 1821,'1953-A394','1953-10-13 00:00:00','atmospheric rocket','unknown',576,'316S','','','','Test',1895,'',10 from dual union all
select 18210,'1968-Y42','1968-07-03 04:32:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 18211,'1968-Y43','1968-07-03 08:27:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 18212,'1968-Y44','1968-07-03 12:27:59','suborbital rocket','success',725,'','','','','Meteo',636,'',82 from dual union all
select 18213,'1968-Y45','1968-07-03 14:37:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 18214,'1968-Y46','1968-07-03 17:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',85 from dual union all
select 18215,'1968-Y47','1968-07-08 14:55:00','suborbital rocket','success',1188,'Su6H','','','','Ionospheric',1447,'',100 from dual union all
select 18216,'1968-Y48','1968-07-09 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1560,'SHIP-SOV',92 from dual union all
select 18217,'1968-Y49','1968-07-10 03:00:00','suborbital rocket','success',725,'','Particles','','','Aeron/Ionos',636,'',100 from dual union all
select 18218,'1968-Y50','1968-07-10 03:32:00','suborbital rocket','success',725,'','','','','Meteo',636,'',84 from dual union all
select 18219,'1968-Y51','1968-07-21 09:08:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 1822,'1953-A395','1953-10-13 00:00:00','atmospheric rocket','unknown',576,'322S','','','','Test',1895,'',10 from dual union all
select 18220,'1968-Y52','1968-07-25 04:32:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 18221,'1968-Y53','1968-07-28 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 18222,'1968-Y54','1968-08-02 02:18:00','suborbital rocket','success',257,'NASA 15.52GI','D-region','','','Ionos',1315,'',84 from dual union all
select 18223,'1968-Y55','1968-08-02 10:31:00','suborbital rocket','success',257,'NASA 15.34GI','D-region','','','Ionos',1315,'',85 from dual union all
select 18224,'1968-Y56','1968-08-07 13:12:00','suborbital rocket','success',1188,'Su9H','','','','Ionospheric',1447,'',94 from dual union all
select 18225,'1968-Y57','1968-08-09 09:59:59','suborbital rocket','success',725,'','','','','Meteo',1561,'SHIP-SOV',80 from dual union all
select 18226,'1968-Y58','1968-08-12 12:04:00','suborbital rocket','success',258,'AE007','ESRO A50/1','','','Ionospheric',417,'',96 from dual union all
select 18227,'1968-Y59','1968-08-14 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 18228,'1968-Y60','1968-08-14 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1562,'SHIP-SOV',93 from dual union all
select 18229,'1968-Y61','1968-08-14 18:00:00','suborbital rocket','success',725,'','','','','Meteo',1562,'SHIP-SOV',80 from dual union all
select 1823,'1953-A396','1953-10-13 00:00:00','atmospheric rocket','success',836,'1194A','','','','Test',1895,'',10 from dual union all
select 18230,'1968-Y62','1968-08-15 17:00:00','suborbital rocket','success',725,'','','','','Meteo',1562,'SHIP-SOV',80 from dual union all
select 18231,'1968-Y63','1968-08-19 18:59:59','suborbital rocket','success',725,'','','','','Meteo',1563,'SHIP-SOV',80 from dual union all
select 18232,'1968-Y64','1968-08-22 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1322,'SHIP-SOV',90 from dual union all
select 18233,'1968-Y65','1968-08-22 18:59:59','suborbital rocket','success',725,'','','','','Meteo',1322,'SHIP-SOV',80 from dual union all
select 18234,'1968-Y66','1968-08-23 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1322,'SHIP-SOV',92 from dual union all
select 18235,'1968-Y67','1968-08-24 15:59:59','suborbital rocket','success',725,'','','','','Meteo',1322,'SHIP-SOV',80 from dual union all
select 18236,'1968-Y68','1968-08-24 17:00:00','suborbital rocket','success',725,'','','','','Meteo',1322,'SHIP-SOV',80 from dual union all
select 18237,'1968-Y69','1968-08-25 16:40:50','suborbital rocket','success',258,'AE008','ESRO A50/2','','','Ionospheric',417,'',109 from dual union all
select 18238,'1968-Y70','1968-08-28 09:00:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 18239,'1968-Y71','1968-09-01 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1559,'SHIP-SOV',91 from dual union all
select 1824,'1953-A397','1953-10-13 00:00:00','atmospheric rocket','success',836,'1218A','','','','Test',1895,'',10 from dual union all
select 18240,'1968-Y72','1968-09-01 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1559,'SHIP-SOV',90 from dual union all
select 18241,'1968-Y73','1968-09-02 17:00:00','suborbital rocket','success',725,'','','','','Meteo',1558,'SHIP-SOV',82 from dual union all
select 18242,'1968-Y74','1968-09-05 17:00:00','suborbital rocket','success',725,'','','','','Meteo',1569,'SHIP-SOV',80 from dual union all
select 18243,'1968-Y75','1968-09-07 17:00:00','suborbital rocket','success',725,'','','','','Meteo',1557,'SHIP-SOV',80 from dual union all
select 18244,'1968-Y76','1968-09-11 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 18245,'1968-Y77','1968-09-12 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1567,'SHIP-SOV',90 from dual union all
select 18246,'1968-Y78','1968-09-18 09:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 18247,'1968-Y79','1968-09-18 10:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 18248,'1968-Y80','1968-09-18 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1565,'SHIP-SOV',93 from dual union all
select 18249,'1968-Y81','1968-09-21 14:00:00','suborbital rocket','success',725,'','','','','Meteo',1564,'SHIP-SOV',80 from dual union all
select 1825,'1953-A398','1953-10-14 00:00:00','atmospheric rocket','success',836,'1138E','','','','Test',1895,'',10 from dual union all
select 18250,'1968-Y82','1968-09-21 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1564,'SHIP-SOV',94 from dual union all
select 18251,'1968-Y83','1968-09-21 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1564,'SHIP-SOV',96 from dual union all
select 18252,'1968-Y84','1968-09-22 09:14:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',84 from dual union all
select 18253,'1968-Y85','1968-09-25 09:00:00','suborbital rocket','success',725,'','Particle flux 1','','','Meteo',1010,'',80 from dual union all
select 18254,'1968-Y86','1968-09-27 12:02:59','suborbital rocket','success',725,'','','','','Meteo',636,'',82 from dual union all
select 18255,'1968-Y87','1968-10-09 14:03:00','suborbital rocket','success',725,'','','','','Meteo',636,'',80 from dual union all
select 18256,'1968-Y88','1968-10-23 09:08:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',86 from dual union all
select 18257,'1968-Y89','1968-10-24 20:50:00','suborbital rocket','success',258,'NASA 15.43II','RTG SA1/1-K68','','','Ionos/Auroral',417,'',104 from dual union all
select 18258,'1968-Y90','1968-10-30 09:08:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',80 from dual union all
select 18259,'1968-Y91','1968-11-02 16:17:00','suborbital rocket','success',257,'','325-4','','','Ionos',428,'',102 from dual union all
select 1826,'1953-A399','1953-10-14 00:00:00','atmospheric rocket','success',836,'1154E','','','','Test',1895,'',10 from dual union all
select 18260,'1968-Y92','1968-11-02 21:50:00','suborbital rocket','success',257,'','325-5','','','Ionos',428,'',102 from dual union all
select 18261,'1968-Y93','1968-11-08 12:00:00','suborbital rocket','success',1188,'Su10H','','','','Ionospheric',1447,'',90 from dual union all
select 18262,'1968-Y94','1968-11-13 09:14:59','suborbital rocket','success',725,'','Particle flux 4','','','Meteo',1010,'',82 from dual union all
select 18263,'1968-Y95','1968-11-16 11:49:00','suborbital rocket','success',1188,'Su11H','','','','Ionospheric',1447,'',90 from dual union all
select 18264,'1968-Y96','1968-11-20 09:24:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',86 from dual union all
select 18265,'1968-Y97','1968-11-22 12:56:00','suborbital rocket','success',1188,'Su7H','','','','Ionospheric',1447,'',90 from dual union all
select 18266,'1968-Y98','1968-11-27 09:01:59','suborbital rocket','success',725,'','Particle flux 5','','','Meteo',1010,'',80 from dual union all
select 18267,'1968-Y99','1968-12-05 11:36:00','suborbital rocket','success',1188,'Su7aH','','','','Ionospheric',1447,'',111 from dual union all
select 18268,'1969-001','1969-01-05 06:28:08','deep space','success',808,'V 15000-072  72','Venera-5','4V-1 No. 330','','Sat',480,'',null from dual union all
select 18269,'1969-002','1969-01-10 05:51:52','deep space','success',808,'V 15000-073  73','Venera-6','4V-1 No. 331','','Sat',480,'',null from dual union all
select 1827,'1953-A40','1953-01-30 00:00:00','military missile','unknown',339,'1E/1247','EU-1','','','Test',1895,'',30 from dual union all
select 18270,'1969-003','1969-01-12 12:10:00','orbital','success',1508,'Ya15002-002','Kosmos-263','Zenit-2 No. 70','','Sat',473,'',null from dual union all
select 18271,'1969-004','1969-01-14 07:30:00','orbital','success',1223,'Ya15000-012','Soyuz-4','Soyuz No. 12','','Sat',562,'',null from dual union all
select 18272,'1969-005','1969-01-15 07:04:57','orbital','success',1223,'Ya15000-011','Soyuz-5','Soyuz No. 13','','Sat',480,'',null from dual union all
select 18273,'1969-006','1969-01-22 16:48:00','orbital','success',1379,'487/D64','OSO 5','OSO F','','Sat',234,'',null from dual union all
select 18274,'1969-007','1969-01-22 19:10:00','orbital','success',1424,'3B-6','KH-8','GAMBIT-3 Mission 4319','','Sat',1778,'',null from dual union all
select 18275,'1969-008','1969-01-23 09:15:00','orbital','success',1508,'N 15001-011','Kosmos-264','Zenit-4M','','Sat',480,'',null from dual union all
select 18276,'1969-009','1969-01-30 06:46:00','orbital','success',1382,'485/D65','Isis 1','Isis A','','Sat',1723,'',null from dual union all
select 18277,'1969-010','1969-02-05 21:59:54','orbital','success',1395,'519/FTV 1650','CORONA 131/1106','CORONA CR-6','','Sat TAG161',1776,'',null from dual union all
select 18278,'1969-011','1969-02-06 00:39:00','orbital','success',1386,'530/D66','Intelsat III F-3','INTELSAT III F-3','','Sat',233,'',null from dual;
insert into LAUNCH
select 18279,'1969-012','1969-02-07 13:59:00','orbital','success',678,'V295-07','Kosmos-265','DS-P1-Yu No. 21','','Sat',460,'',null from dual union all
select 1828,'1953-A400','1953-10-15 00:00:00','military missile','unknown',339,'1357R','','','','Test',1895,'',30 from dual union all
select 18280,'1969-013','1969-02-09 21:09:00','orbital','success',1425,'3C-17','Tacsat','Tacsat','','Sat UPF-35',275,'',null from dual union all
select 18281,'1969-014','1969-02-25 01:29:02','deep space','success',179,'AC-20','Mariner 6','Mariner 69-3','','Sat',269,'',null from dual union all
select 18282,'1969-015','1969-02-25 10:20:01','orbital','success',1508,'V 15002-003','Kosmos-266','Zenit-2 No. 71','','Sat',473,'',null from dual union all
select 18283,'1969-016','1969-02-26 07:47:01','orbital','success',1382,'483/D67','ESSA 9','TOS G','','Sat',234,'',null from dual union all
select 18284,'1969-017','1969-02-26 08:30:01','orbital','success',1508,'V 15002-014','Kosmos-267','Zenit-4 No. 54','','Sat',562,'',null from dual union all
select 18285,'1969-018','1969-03-03 16:00:00','deep space','success',1123,'SA-504','Apollo 9','Apollo CM-104','','Sat',921,'LUT2',null from dual union all
select 18286,'1969-019','1969-03-04 19:30:00','orbital','success',1424,'3B-7','KH-8','GAMBIT-3 Mission 4320','','Sat',1778,'',null from dual union all
select 18287,'1969-020','1969-03-05 13:04:55','orbital','success',678,'','Kosmos-268','DS-P1-Yu No. 18','','Sat',627,'',null from dual union all
select 18288,'1969-021','1969-03-05 17:25:01','orbital','success',680,'V149    -26LM','Kosmos-269','Tselina-OM','','Sat',457,'',null from dual union all
select 18289,'1969-022','1969-03-06 12:15:01','orbital','success',1508,'V 15002-013','Kosmos-270','Zenit-4 No. 53','','Sat',473,'',null from dual union all
select 1829,'1953-A401','1953-10-15 00:00:00','atmospheric rocket','success',371,'','Sidewinder Test','','','Test',1821,'',10 from dual union all
select 18290,'1969-023','1969-03-15 12:15:00','orbital','success',1508,'V 15002-012','Kosmos-271','Zenit-4 No. 52','','Sat',473,'',null from dual union all
select 18291,'1969-024','1969-03-17 16:40:18','orbital','success',680,'V149    -27LM','Kosmos-272','Sfera','','Sat',457,'',null from dual union all
select 18292,'1969-025','1969-03-18 07:47:35','orbital','success',162,'104F','OV1-17/18/19','OV1-17S','','Sat',1681,'',null from dual union all
select 18293,'1969-026','1969-03-19 21:38:00','orbital','success',1395,'541/FTV 1651','CORONA 132/1050','CORONA J-43','','Sat TAG162',1776,'',null from dual union all
select 18294,'1969-027','1969-03-22 12:15:00','orbital','success',1508,'V 15002-009','Kosmos-273','Zenit-2 No. 77','','Sat',473,'',null from dual union all
select 18295,'1969-028','1969-03-24 10:10:00','orbital','success',1508,'V 15002-018','Kosmos-274','Zenit-4 No. 58','','Sat',562,'',null from dual union all
select 18296,'1969-029','1969-03-26 12:30:00','orbital','success',1510,'R 15000-004','Meteor','Meteor 11F614M No. 12','','Sat',473,'',null from dual union all
select 18297,'1969-030','1969-03-27 22:22:01','deep space','success',179,'AC-19','Mariner 7','Mariner 69-2','','Sat',268,'',null from dual union all
select 18298,'1969-031','1969-03-28 16:00:08','orbital','success',678,'','Kosmos-275','DS-P1-I No. 5','','Sat',460,'',null from dual union all
select 18299,'1969-032','1969-04-04 10:20:01','orbital','success',1508,'V 15002-015','Kosmos-276','Zenit-4 No. 55','','Sat',473,'',null from dual union all
select 183,'1944-S166','1944-09-30 22:43:00','military missile','success',1470,'18794','Ma175','','','Test',661,'',90 from dual union all
select 1830,'1953-A402','1953-10-15 00:00:00','atmospheric rocket','success',836,'1263R','Nike 253R','','','Test',1895,'',10 from dual union all
select 18300,'1969-033','1969-04-04 13:00:04','orbital','success',678,'V295-06','Kosmos-277','DS-P1-Yu No. 20','','Sat',460,'',null from dual union all
select 18301,'1969-034','1969-04-09 13:00:01','orbital','success',1508,'V 15002-010','Kosmos-278','Zenit-2 No. 78','','Sat',473,'',null from dual union all
select 18302,'1969-035','1969-04-11 02:30:01','orbital','success',809,'Ya15000-084  L88','Molniya-1','Molniya-1 11F67 No. 16','','Sat',480,'',null from dual union all
select 18303,'1969-036','1969-04-13 02:30:00','orbital','success',178,'5502A','AFP-827','CANYON 2','','Sat',228,'',null from dual union all
select 18304,'1969-037','1969-04-14 07:54:03','orbital','success',1395,'543/FTV 6222 (TA10)','Nimbus 3','Nimbus B2','','Sat TAG163',1723,'',null from dual union all
select 18305,'1969-038','1969-04-15 08:14:59','orbital','success',1508,'Yu15001-013','Kosmos-279','Zenit-4 No. 61','','Sat',562,'',null from dual union all
select 18306,'1969-039','1969-04-15 17:30:00','orbital','success',1424,'3B-21','KH-8','GAMBIT-3 Mission 4321','','Sat',1778,'',null from dual union all
select 18307,'1969-040','1969-04-23 09:55:01','orbital','success',1508,'R 15000-007','Kosmos-280','Zenit-4M','','Sat',480,'',null from dual union all
select 18308,'1969-041','1969-05-02 01:46:58','orbital','success',1395,'544/FTV 1649','CORONA 133/1051','CORONA J-44','','Sat TAG164',1776,'',null from dual union all
select 18309,'1969-042','1969-05-13 09:15:01','orbital','success',1508,'V 15002-004','Kosmos-281','Zenit-2 No. 72','','Sat',473,'',null from dual union all
select 1831,'1953-A403','1953-10-15 00:00:00','atmospheric rocket','success',836,'1281R','Nike 251R','','','Test',1895,'',10 from dual union all
select 18310,'1969-043','1969-05-18 16:49:00','deep space','success',1123,'SA-505','Apollo 10','Apollo CM-106','','Sat',922,'LUT3',null from dual union all
select 18311,'1969-044','1969-05-20 08:40:02','orbital','success',1508,'V 15002-017','Kosmos-282','Zenit-4 No. 57','','Sat',473,'',null from dual union all
select 18312,'1969-045','1969-05-22 02:00:00','orbital','success',1386,'533/D68','Intelsat III F-4','INTELSAT III F-4','','Sat',233,'',null from dual union all
select 18313,'1969-046','1969-05-23 07:57:01','orbital','success',1425,'3C-15','Vela 5A/5B','ERS 29','','Sat AMF',275,'',null from dual union all
select 18314,'1969-047','1969-05-27 12:59:59','orbital','success',678,'V295-05','Kosmos-283','DS-P1-Yu No. 19','','Sat',460,'',null from dual union all
select 18315,'1969-048','1969-05-29 06:59:59','orbital','success',1508,'Yu15001-014','Kosmos-284','Zenit-4 No. 62','','Sat',562,'',null from dual union all
select 18316,'1969-049','1969-06-03 12:57:27','orbital','success',678,'V295-10','Kosmos-285','DS-P1-Yu No. 24','','Sat',460,'',null from dual union all
select 18317,'1969-050','1969-06-03 16:49:00','orbital','success',1424,'3B-22','KH-8','GAMBIT-3 Mission 4322','','Sat',1778,'',null from dual union all
select 18318,'1969-051','1969-06-05 14:42:45','orbital','success',1396,'526/FTV 6803 (TA11)','OGO 6','OGO F','','Sat TAG165',1723,'',null from dual union all
select 18319,'1969-052','1969-06-15 08:59:01','orbital','success',1508,'Yu15001-012','Kosmos-286','Zenit-4 No. 60','','Sat',473,'',null from dual union all
select 1832,'1953-A404','1953-10-15 00:00:00','atmospheric rocket','success',836,'1279R','Nike 252R','','','Test',1895,'',10 from dual union all
select 18320,'1969-053','1969-06-21 08:47:58','orbital','failure',1382,'482/D69','Explorer 41','IMP G','','Sat',1724,'',null from dual union all
select 18321,'1969-054','1969-06-24 06:50:00','orbital','success',1508,'V 15002-008','Kosmos-287','Zenit-2 No. 76','','Sat',562,'',null from dual union all
select 18322,'1969-055','1969-06-27 06:59:59','orbital','success',1508,'Yu15001-011','Kosmos-288','Zenit-4 No. 59','','Sat',480,'',null from dual union all
select 18323,'1969-056','1969-06-29 03:15:59','orbital','success',1388,'539/D70','Biosatellite 3','Biosatellite D Adapter','','Sat',233,'',null from dual union all
select 18324,'1969-057','1969-07-10 09:00:00','orbital','success',1508,'V 15002-016','Kosmos-289','Zenit-4 No. 56','','Sat',473,'',null from dual union all
select 18325,'1969-058','1969-07-13 02:54:42','deep space','success',953,'242-01','Luna-15','E-8-5 No. 401','','Sat',588,'',null from dual union all
select 18326,'1969-059','1969-07-16 13:32:00','deep space','success',1123,'SA-506','Apollo 11','Apollo CM-107','','Sat',921,'LUT1',null from dual union all
select 18327,'1969-060','1969-07-22 12:30:00','orbital','success',1508,'V 15002-007','Kosmos-290','Zenit-2 No. 75','','Sat',473,'',null from dual union all
select 18328,'1969-061','1969-07-22 12:55:31','orbital','success',809,'V 15000-089  L89','Molniya-1','Molniya-1 11F67 No. 18','','Sat',480,'',null from dual union all
select 18329,'1969-062','1969-07-23 04:39:55','orbital','success',1364,'279/BII-7','DAPP 7421','DAPP 7421','','Sat',1720,'',null from dual union all
select 1833,'1953-A405','1953-10-15 00:00:00','atmospheric rocket','success',836,'1271R','Nike 250R','','','Test',1895,'',10 from dual union all
select 18330,'1969-063','1969-07-24 01:30:58','orbital','success',1396,'545/FTV 1652','CORONA 134/1107','CORONA CR-7','','Sat TAG166',1776,'',null from dual union all
select 18331,'1969-064','1969-07-26 02:06:00','orbital','failure',1386,'547/D71','Intelsat III F-5','INTELSAT III F-5','','Sat',233,'',5391 from dual union all
select 18332,'1969-065','1969-07-31 10:19:00','orbital','success',1395,'523/FTV 2735','OPS 8285','STRAWMAN 2','','Sat TAG167',1722,'',null from dual union all
select 18333,'1969-066','1969-08-06 05:40:03','orbital','success',1446,'V 22500-02L','Kosmos-291','IS-M GVM','','Sat',590,'',null from dual union all
select 18334,'1969-067','1969-08-07 23:48:06','deep space','success',953,'243-01','Zond-7','L-1 No. 11','','Sat',587,'',null from dual union all
select 18335,'1969-068','1969-08-09 07:52:00','orbital','success',1388,'548/D72','OSO 6','OSO G','','Sat',233,'',null from dual union all
select 18336,'1969-069','1969-08-12 11:01:04','orbital','success',179,'AC-18','ATS 5','ATS E','','Sat',268,'',null from dual union all
select 18337,'1969-070','1969-08-13 22:00:02','orbital','success',680,'Yu149   -35LM','Kosmos-292','Tsiklon','','Sat',457,'',null from dual union all
select 18338,'1969-071','1969-08-16 11:59:59','orbital','success',1508,'Yu15002-024','Kosmos-293','Zenit-2M','','Sat',562,'',null from dual union all
select 18339,'1969-072','1969-08-19 13:00:01','orbital','success',1508,'Yu15001-015','Kosmos-294','Zenit-4 No. 63','','Sat',473,'',null from dual union all
select 1834,'1953-A406','1953-10-15 00:00:00','military missile','success',1214,'N-69A GM-3393','','','','Test',222,'',10 from dual union all
select 18340,'1969-073','1969-08-22 14:14:57','orbital','success',678,'Yu295-02','Kosmos-295','DS-P1-Yu No. 29','','Sat',460,'',null from dual union all
select 18341,'1969-074','1969-08-23 16:00:00','orbital','success',1402,'23B-1 (3B-23)','KH-8A','GAMBIT-3 Mission 4323','','Sat',1778,'',null from dual union all
select 18342,'1969-075','1969-08-29 09:05:00','orbital','success',1508,'Yu15001-020','Kosmos-296','Zenit-4 No. 68','','Sat',562,'',null from dual union all
select 18343,'1969-076','1969-09-02 11:00:01','orbital','success',1508,'Yu15001-016','Kosmos-297','Zenit-4 No. 64','','Sat',473,'',null from dual union all
select 18344,'1969-077','1969-09-15 16:00:00','orbital','success',1007,'Yu45021-50T','Kosmos-298','OGCh No. 21?','','Sat',547,'',null from dual union all
select 18345,'1969-078','1969-09-18 08:40:01','orbital','success',1508,'Yu15001-017','Kosmos-299','Zenit-4 No. 65','','Sat',562,'',null from dual union all
select 18346,'1969-079','1969-09-22 21:11:02','orbital','success',1395,'531/FTV 1653','CORONA 135/1052','CORONA J-46','','Sat TAG168',1776,'',null from dual union all
select 18347,'1969-080','1969-09-23 14:07:36','orbital','failure',953,'244-01','Kosmos-300','E-8-5 No. 403','','Sat',588,'',179 from dual union all
select 18348,'1969-081','1969-09-24 12:15:01','orbital','success',1508,'Yu15001-001','Kosmos-301','Zenit-2 No. 79','','Sat',473,'',null from dual union all
select 18349,'1969-082','1969-09-30 13:40:00','orbital','success',1395,'525/FTV 2706','POPPY 6/NRL Comp 7','P-11 No. 4407','','Sat TAG169',1722,'',null from dual union all
select 1835,'1953-A407','1953-10-20 00:00:00','atmospheric rocket','success',836,'1283R','Nike 254R','','','Test',1895,'',10 from dual union all
select 18350,'1969-083','1969-10-01 22:29:00','orbital','success',1128,'S172C','ESRO 1B','ESRO 1B','','Sat',1779,'',null from dual union all
select 18351,'1969-084','1969-10-06 01:45:00','orbital','success',1510,'V 15000-061','Meteor','Meteor 11F614M No. 15','','Sat',473,'',null from dual union all
select 18352,'1969-085','1969-10-11 11:10:00','orbital','success',1223,'V 15000-14','Soyuz-6','Soyuz No. 14','','Sat',562,'',null from dual union all
select 18353,'1969-086','1969-10-12 10:44:42','orbital','success',1223,'Yu15000-19','Soyuz-7','Soyuz No. 15','','Sat',480,'',null from dual union all
select 18354,'1969-087','1969-10-13 10:19:09','orbital','success',1223,'Yu15000-18','Soyuz-8','Soyuz No. 16','','Sat',562,'',null from dual union all
select 18355,'1969-088','1969-10-14 13:19:53','orbital','success',678,'2I','Interkosmos-1','DS-U3-IK No. 1','','Sat',627,'',null from dual union all
select 18356,'1969-089','1969-10-17 11:45:01','orbital','success',1508,'Yu15001-019','Kosmos-302','Zenit-4 No. 67','','Sat',473,'',null from dual union all
select 18357,'1969-090','1969-10-18 10:00:03','orbital','success',678,'Yu295-03','Kosmos-303','DS-P1-Yu No. 28','','Sat',460,'',null from dual union all
select 18358,'1969-091','1969-10-21 12:49:59','orbital','success',680,'Yu226   -11U','Kosmos-304','Tsiklon','','Sat',456,'',null from dual union all
select 18359,'1969-092','1969-10-22 14:09:59','orbital','failure',953,'241-01','Kosmos-305','VA','','Sat',588,'',210 from dual union all
select 1836,'1953-A408','1953-10-20 00:00:00','atmospheric rocket','success',836,'1277R','Nike 255R','','','Test',1895,'',10 from dual union all
select 18360,'1969-093','1969-10-24 09:40:00','orbital','success',1508,'Yu15002-026','Kosmos-306','Zenit-2M','','Sat',480,'',null from dual union all
select 18361,'1969-094','1969-10-24 13:01:58','orbital','success',678,'','Kosmos-307','DS-P1-Yu No. 22','','Sat',627,'',null from dual union all
select 18362,'1969-095','1969-10-24 18:10:00','orbital','success',1402,'23B-2 (3B-24)','KH-8A','GAMBIT-3 Mission 4324','','Sat',1778,'',null from dual union all
select 18363,'1969-096','1969-11-04 11:59:59','orbital','success',678,'V295-09','Kosmos-308','DS-P1-I No. 7','','Sat',460,'',null from dual union all
select 18364,'1969-097','1969-11-08 01:52:00','orbital','success',1128,'S169C','Azur','GRS-A','','Sat',1779,'',null from dual union all
select 18365,'1969-098','1969-11-12 11:30:00','orbital','success',1508,'Yu15001-002','Kosmos-309','Zenit-2 No. 80','','Sat',473,'',null from dual union all
select 18366,'1969-099','1969-11-14 16:22:00','deep space','success',1123,'SA-507','Apollo 12','Apollo CM-108','','Sat',921,'LUT2',null from dual union all
select 18367,'1969-100','1969-11-15 08:30:00','orbital','success',1508,'Yu15002-012','Kosmos-310','Zenit-4 No. 69','','Sat',562,'',null from dual union all
select 18368,'1969-101','1969-11-22 00:37:00','orbital','success',1386,'554/D74','Skynet 1A','Skynet IA','','Sat',233,'',null from dual;
insert into LAUNCH
select 18369,'1969-102','1969-11-24 11:00:04','orbital','success',678,'Yu646-24','Kosmos-311','DS-P1-Yu No. 27','','Sat',460,'',null from dual union all
select 1837,'1953-A409','1953-10-21 00:00:00','military missile','success',572,'5','RV-A-8','','','Test',1863,'',30 from dual union all
select 18370,'1969-103','1969-11-24 16:49:59','orbital','success',680,'Yu149   -41LM','Kosmos-312','Sfera','','Sat',456,'',null from dual union all
select 18371,'1969-104','1969-12-03 13:20:02','orbital','success',1508,'Yu15002-027','Kosmos-313','Zenit-2M','','Sat',475,'',null from dual union all
select 18372,'1969-105','1969-12-04 21:38:00','orbital','success',1396,'549/FTV 1655','CORONA 136/1108','CORONA CR-9','','Sat TAG170',1776,'',null from dual union all
select 18373,'1969-106','1969-12-11 12:58:59','orbital','success',678,'Yu646-25','Kosmos-314','DS-P1-Yu No. 30','','Sat',460,'',null from dual union all
select 18374,'1969-107','1969-12-20 03:26:00','orbital','success',680,'V149    -29LM','Kosmos-315','Tselina-OM','','Sat',456,'',null from dual union all
select 18375,'1969-108','1969-12-23 09:25:00','orbital','success',1446,'V 22500-03L','Kosmos-316','IM IS (I2P?)','','Sat',591,'',null from dual union all
select 18376,'1969-109','1969-12-23 13:50:01','orbital','success',1508,'V 15002-019','Kosmos-317','Zenit-4MK','','Sat',473,'',null from dual union all
select 18377,'1969-110','1969-12-25 09:59:51','orbital','success',678,'','Interkosmos-2','DS-U1-IK No. 1','','Sat',627,'',null from dual union all
select 18378,'1969-A01','1969-07-23 00:00:00','military missile','success',1038,'Yu22500-12L','8K99 LKI-11','','','Test',477,'',1000 from dual union all
select 18379,'1969-A03','1969-01-02 15:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',37 from dual union all
select 1838,'1953-A41','1953-01-30 00:00:00','military missile','success',771,'GM-11050','','','','Test',222,'',12 from dual union all
select 18380,'1969-A04','1969-01-02 18:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',41 from dual union all
select 18381,'1969-A05','1969-01-03 18:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',44 from dual union all
select 18382,'1969-A06','1969-01-05 05:00:00','suborbital rocket','success',761,'MT-135P-1','','','','Aeron',839,'',44 from dual union all
select 18383,'1969-A07','1969-01-09 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 18384,'1969-A08','1969-01-08 20:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',34 from dual union all
select 18385,'1969-A09','1969-01-09 00:00:00','suborbital rocket','failure',1072,'S-300-1','','','','Aeron',839,'',0 from dual union all
select 18386,'1969-A10','1969-01-09 15:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',46 from dual union all
select 18387,'1969-A100','1969-06-12 21:01:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',40 from dual union all
select 18388,'1969-A101','1969-06-17 18:27:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',47 from dual union all
select 18389,'1969-A102','1969-06-17 21:07:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',47 from dual union all
select 1839,'1953-A410','1953-10-21 00:00:00','military missile','success',771,'GM-2368','','','','Test',222,'',12 from dual union all
select 18390,'1969-A103','1969-06-19 15:02:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 18391,'1969-A104','1969-06-20 00:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',47 from dual union all
select 18392,'1969-A105','1969-06-23 00:00:00','military missile','success',1252,'FLA-29','','','','Test',1888,'',6 from dual union all
select 18393,'1969-A106','1969-06-24 21:23:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',45 from dual union all
select 18394,'1969-A107','1969-06-26 21:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',45 from dual union all
select 18395,'1969-A108','1969-06-27 21:35:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',31 from dual union all
select 18396,'1969-A109','1969-06-27 22:58:00','test rocket','failure',207,'R-0','','','','Test',1844,'',4 from dual union all
select 18397,'1969-A11','1969-01-09 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',46 from dual union all
select 18398,'1969-A110','1969-07-01 00:00:00','military missile','success',545,'DASA 6.1','','','','Aeron?',1895,'',40 from dual union all
select 18399,'1969-A111','1969-07-01 00:00:00','military missile','success',545,'DASA 6.1','','','','Aeron?',1895,'',40 from dual union all
select 184,'1944-S167','1944-10-01 09:46:00','military missile','success',1470,'18777','Ma192','','','Test',661,'',90 from dual union all
select 1840,'1953-A411','1953-10-22 00:00:00','atmospheric rocket','unknown',576,'69N','','','','Test',1895,'',10 from dual union all
select 18400,'1969-A112','1969-07-01 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',42 from dual union all
select 18401,'1969-A113','1969-07-01 21:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',45 from dual union all
select 18402,'1969-A114','1969-07-02 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',43 from dual union all
select 18403,'1969-A115','1969-07-02 19:44:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',41 from dual union all
select 18404,'1969-A116','1969-07-02 21:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',42 from dual union all
select 18405,'1969-A117','1969-07-03 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 18406,'1969-A118','1969-07-03 00:00:00','atmospheric rocket','success',1066,'Ly5','Lynx 5','','','Test',2,'',20 from dual union all
select 18407,'1969-A119','1969-07-08 21:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',49 from dual union all
select 18408,'1969-A12','1969-01-10 13:46:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',49 from dual union all
select 18409,'1969-A120','1969-07-09 15:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',26 from dual union all
select 1841,'1953-A412','1953-10-22 00:00:00','military missile','success',771,'GM-11072','','','','Test',222,'',12 from dual union all
select 18410,'1969-A121','1969-07-09 17:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',45 from dual union all
select 18411,'1969-A122','1969-07-10 19:07:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',49 from dual union all
select 18412,'1969-A123','1969-07-11 17:34:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',43 from dual union all
select 18413,'1969-A124','1969-07-12 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 18414,'1969-A125','1969-07-16 00:00:00','atmospheric rocket','success',617,'J208','WRE Ablation','','','Test',1836,'',30 from dual union all
select 18415,'1969-A126','1969-07-16 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',47 from dual union all
select 18416,'1969-A127','1969-07-17 21:11:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',46 from dual union all
select 18417,'1969-A128','1969-07-18 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',48 from dual union all
select 18418,'1969-A129','1969-06-19 09:08:00','suborbital rocket','success',598,'INTA MI-6901','','Booster test','','Test',396,'',1 from dual union all
select 18419,'1969-A13','1969-01-10 14:54:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',47 from dual union all
select 1842,'1953-A413','1953-10-22 00:00:00','atmospheric rocket','success',836,'1289R','Nike 256R','','','Test',1895,'',10 from dual union all
select 18420,'1969-A130','1969-07-22 00:00:00','military missile','success',1252,'FLA-32','Extended Range','','','Test',1888,'',10 from dual union all
select 18421,'1969-A131','1969-07-30 05:30:00','suborbital rocket','success',631,'ISRO 11.34','ISRO 11.34','TERLS-120','','Aeron',1467,'',29 from dual union all
select 18422,'1969-A132','1969-07-30 14:08:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',41 from dual union all
select 18423,'1969-A133','1969-07-30 18:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',37 from dual union all
select 18424,'1969-A134','1969-07-30 19:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',45 from dual union all
select 18425,'1969-A135','1969-08-01 14:37:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',46 from dual union all
select 18426,'1969-A136','1969-08-06 16:29:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',30 from dual union all
select 18427,'1969-A137','1969-08-06 20:15:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',30 from dual union all
select 18428,'1969-A138','1969-08-09 00:00:00','military missile','failure',1038,'V 22500-13L','8K99 LKI-12','','','Test',477,'',0 from dual union all
select 18429,'1969-A139','1969-08-11 17:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',49 from dual union all
select 1843,'1953-A414','1953-10-22 00:00:00','atmospheric rocket','success',836,'1251R','Nike 257R','','','Test',1895,'',10 from dual union all
select 18430,'1969-A14','1969-01-14 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',47 from dual union all
select 18431,'1969-A140','1969-08-13 00:00:00','military missile','failure',1251,'800012','KT-12','','','Test',868,'',0 from dual union all
select 18432,'1969-A141','1969-08-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',49 from dual union all
select 18433,'1969-A142','1969-08-18 18:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',32 from dual union all
select 18434,'1969-A143','1969-08-19 16:18:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',31 from dual union all
select 18435,'1969-A144','1969-08-21 20:01:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',33 from dual union all
select 18436,'1969-A145','1969-08-21 21:35:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',48 from dual union all
select 18437,'1969-A146','1969-08-24 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 18438,'1969-A147','1969-08-26 16:47:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',46 from dual union all
select 18439,'1969-A148','1969-09-03 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 1844,'1953-A415','1953-10-13 00:00:00','military missile','unknown',339,'1351E','EU-10','','','Test',1895,'',30 from dual union all
select 18440,'1969-A149','1969-09-05 00:00:00','suborbital rocket','failure',37,'NRL NE4.234','','','','Solar',1895,'',20 from dual union all
select 18441,'1969-A15','1969-01-15 16:34:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',330,'',43 from dual union all
select 18442,'1969-A150','1969-09-08 10:20:00','suborbital rocket','failure',1072,'S-300-3','','','','Aeron',839,'',10 from dual union all
select 18443,'1969-A151','1969-09-12 21:02:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',47 from dual union all
select 18444,'1969-A152','1969-09-15 00:00:00','military missile','success',1252,'FLA-35','','','','Test',1888,'',10 from dual union all
select 18445,'1969-A153','1969-09-17 17:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',48 from dual union all
select 18446,'1969-A154','1969-09-18 21:49:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',48 from dual union all
select 18447,'1969-A155','1969-09-20 21:26:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 18448,'1969-A156','1969-09-20 06:30:00','suborbital rocket','success',828,'NAL-7 F7','','','','Test',1476,'',20 from dual union all
select 18449,'1969-A157','1969-09-22 21:14:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',44 from dual union all
select 1845,'1953-A416','1953-10-23 00:00:00','atmospheric rocket','unknown',576,'1DJ','Demi John','','','Test',1895,'',1 from dual union all
select 18450,'1969-A158','1969-09-24 19:09:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',45 from dual union all
select 18451,'1969-A159','1969-09-25 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',36 from dual union all
select 18452,'1969-A16','1969-01-15 17:32:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',308,'',41 from dual union all
select 18453,'1969-A160','1969-09-25 21:11:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',49 from dual union all
select 18454,'1969-A161','1969-09-26 14:12:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',33 from dual union all
select 18455,'1969-A162','1969-09-26 14:29:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',40 from dual union all
select 18456,'1969-A163','1969-09-26 15:10:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',31 from dual union all
select 18457,'1969-A164','1969-09-27 05:00:00','suborbital rocket','success',759,'MT-135-043','','','','Aeron',839,'',49 from dual union all
select 18458,'1969-A165','1969-10-01 00:00:00','suborbital rocket','success',486,'F.00','HRV 933','','','Test',1836,'',30 from dual;
insert into LAUNCH
select 18459,'1969-A166','1969-10-01 19:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',46 from dual union all
select 1846,'1953-A417','1953-10-23 00:00:00','atmospheric rocket','failure',835,'','Navaho Model','','','Test',1821,'',1 from dual union all
select 18460,'1969-A167','1969-10-03 13:04:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',42 from dual union all
select 18461,'1969-A168','1969-10-03 20:30:00','suborbital rocket','success',779,'','','','','Aeron',933,'',45 from dual union all
select 18462,'1969-A169','1969-10-03 21:33:00','suborbital rocket','success',779,'','','','','Aeron',933,'',48 from dual union all
select 18463,'1969-A17','1969-01-15 18:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',47 from dual union all
select 18464,'1969-A170','1969-10-03 22:30:00','suborbital rocket','success',779,'','','','','Aeron',933,'',49 from dual union all
select 18465,'1969-A171','1969-10-06 00:00:00','military missile','failure',1252,'FLA-36','','','','Test',1888,'',0 from dual union all
select 18466,'1969-A172','1969-10-09 11:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',46 from dual union all
select 18467,'1969-A173','1969-10-09 14:59:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',43 from dual union all
select 18468,'1969-A174','1969-10-10 12:07:00','military missile','failure',168,'98F','','ABRES RMP-B-13/KX-112?','','Reentry Test',1682,'',10 from dual union all
select 18469,'1969-A175','1969-10-10 21:11:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',49 from dual union all
select 1847,'1953-A418','1953-10-26 00:00:00','atmospheric rocket','success',365,'','Pogo Model','','','Test',1821,'',10 from dual union all
select 18470,'1969-A176','1969-10-14 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',46 from dual union all
select 18471,'1969-A177','1969-10-15 18:23:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',40 from dual union all
select 18472,'1969-A178','1969-10-16 00:00:00','atmospheric rocket','success',617,'J209','UK-2G/HRV 930','','','Test',1836,'',30 from dual union all
select 18473,'1969-A179','1969-10-17 19:18:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',47 from dual union all
select 18474,'1969-A18','1969-01-17 03:05:00','military missile','failure',135,'V145D','ABRES AM-1 (USAF)','ABRES AM-1 (USAF)','','Reentry Test',612,'',0 from dual union all
select 18475,'1969-A180','1969-10-23 00:00:00','military missile','success',1252,'FLA-38','','','','Test',1888,'',10 from dual union all
select 18476,'1969-A181','1969-10-27 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',49 from dual union all
select 18477,'1969-A182','1969-10-29 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',48 from dual union all
select 18478,'1969-A183','1969-10-31 13:44:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',48 from dual union all
select 18479,'1969-A184','1969-10-31 20:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',36 from dual union all
select 1848,'1953-A419','1953-10-27 00:00:00','military missile','unknown',339,'1305E','EU-11','','','Test',1895,'',30 from dual union all
select 18480,'1969-A185','1969-11-03 18:40:00','suborbital rocket','failure',848,'CRL AT07.395','PCA69 C-3 Sphere','','','Aeron',428,'',47 from dual union all
select 18481,'1969-A186','1969-11-04 19:43:00','suborbital rocket','success',88,'','PCA69 H-06','','','Aeron',428,'',69 from dual union all
select 18482,'1969-A187','1969-11-06 00:00:00','military missile','failure',1252,'FLA-33','','','','Test',1888,'',2 from dual union all
select 18483,'1969-A188','1969-11-07 20:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',35 from dual union all
select 18484,'1969-A189','1969-11-13 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 18485,'1969-A19','1969-01-17 00:02:28','suborbital rocket','failure',258,'NASA 15.44II','RTG SA1/2-A69','','','Ionos',29,'',7 from dual union all
select 18486,'1969-A190','1969-11-13 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 18487,'1969-A191','1969-11-18 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 18488,'1969-A192','1969-11-19 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',47 from dual union all
select 18489,'1969-A193','1969-11-20 15:22:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',49 from dual union all
select 1849,'1953-A42','1953-02-03 00:00:00','atmospheric rocket','success',836,'304P','Nike CET 4','','','Test',1895,'',10 from dual union all
select 18490,'1969-A194','1969-11-26 00:00:00','atmospheric rocket','success',1066,'Ob3','Oberon 3','','','Test',1836,'',20 from dual union all
select 18491,'1969-A195','1969-11-28 15:32:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',47 from dual union all
select 18492,'1969-A196','1969-11-28 18:03:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',44 from dual union all
select 18493,'1969-A197','1969-11-28 22:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',45 from dual union all
select 18494,'1969-A198','1969-12-01 00:00:00','military missile','failure',1252,'FLA-34','','','','Test',1888,'',2 from dual union all
select 18495,'1969-A199','1969-12-02 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',42 from dual union all
select 18496,'1969-A20','1969-01-17 14:12:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',32 from dual union all
select 18497,'1969-A200','1969-12-03 15:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',48 from dual union all
select 18498,'1969-A201','1969-12-03 20:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',48 from dual union all
select 18499,'1969-A202','1969-12-04 20:00:02','military missile','failure',799,'FTM-210','Mk 12 RV x 3','R&D','','Test',266,'',0 from dual union all
select 185,'1944-S168','1944-10-01 11:25:00','military missile','success',1470,'18772','Ma187','','','Test',661,'',90 from dual union all
select 1850,'1953-A420','1953-10-27 00:00:00','atmospheric rocket','success',836,'1287R','Nike 258R','','','Test',1895,'',10 from dual union all
select 18500,'1969-A203','1969-12-09 23:12:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',47 from dual union all
select 18501,'1969-A204','1969-12-10 14:00:00','suborbital rocket','success',631,'INTA H-6916','','','','Meteo',396,'',35 from dual union all
select 18502,'1969-A205','1969-12-10 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',46 from dual union all
select 18503,'1969-A206','1969-12-12 22:13:00','suborbital rocket','failure',37,'KP3.26','','','','Solar UV',1869,'',1 from dual union all
select 18504,'1969-A207','1969-12-14 21:00:00','military missile','failure',1018,'U 15002-01','8K75 UBP-15','','','OT',579,'',100 from dual union all
select 18505,'1969-A208','1969-12-17 17:53:00','suborbital rocket','success',1188,'','','','','Meteo',1447,'',36 from dual union all
select 18506,'1969-A209','1969-12-18 20:10:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',45 from dual union all
select 18507,'1969-A21','1969-01-17 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',49 from dual union all
select 18508,'1969-A210','1969-12-22 23:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',39 from dual union all
select 18509,'1969-A211','1969-12-16 00:00:00','suborbital rocket','success',549,'PEPP','40-ft DGB Para','','','Test',1895,'',40 from dual union all
select 1851,'1953-A421','1953-10-28 00:00:00','atmospheric rocket','success',836,'1222A','','','','Test',1895,'',10 from dual union all
select 18510,'1969-A212','1969-07-10 17:30:00','suborbital rocket','success',557,'Pacemaker','Carbon phen. ablator','','','Reentry test',1821,'',20 from dual union all
select 18511,'1969-A213','1969-02-25 00:00:00','suborbital rocket','failure',1506,'SDC VD-29','Ram-Air Sphere','','','Aeron',276,'',40 from dual union all
select 18512,'1969-A214','1969-02-26 00:00:00','suborbital rocket','failure',1506,'SDC VD-30','ROBIN','','','Aeron',276,'',40 from dual union all
select 18513,'1969-A215','1969-05-15 00:00:00','suborbital rocket','failure',1506,'SDC VD-33','ROBIN','','','Aeron',276,'',40 from dual union all
select 18514,'1969-A216','1969-05-16 00:00:00','suborbital rocket','failure',1506,'SDC VD-34','ROBIN','','','Aeron',276,'',40 from dual union all
select 18515,'1969-A217','1969-07-29 00:00:00','military missile','success',1099,'Flight 1','','','','Test',1895,'B-52',10 from dual union all
select 18516,'1969-A218','1969-03-01 00:00:00','military missile','success',702,'Recert flight 5','','','','Test',1895,'',10 from dual union all
select 18517,'1969-A219','1969-03-13 00:00:00','military missile','success',702,'XRL Test 1','','','','Test',1895,'',10 from dual union all
select 18518,'1969-A22','1969-01-17 17:55:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',40 from dual union all
select 18519,'1969-A220','1969-07-11 00:00:00','military missile','failure',702,'XRL Test','','','','Test',1895,'',0 from dual union all
select 1852,'1953-A422','1953-11-01 00:00:00','atmospheric rocket','failure',442,'','B-58 Model','','','Test',1821,'',10 from dual union all
select 18520,'1969-A221','1969-10-24 00:00:00','military missile','success',702,'XRL Test','','','','Test',1895,'',10 from dual union all
select 18521,'1969-A23','1969-01-17 22:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',48 from dual union all
select 18522,'1969-A24','1969-01-20 14:55:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',48 from dual union all
select 18523,'1969-A25','1969-01-20 18:02:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',42 from dual union all
select 18524,'1969-A26','1969-01-21 04:49:00','suborbital rocket','failure',1210,'SL781','UAT-4','','','XR Astron',1839,'',0 from dual union all
select 18525,'1969-A27','1969-01-21 21:02:59','suborbital rocket','success',1061,'','Starute','','','Meteo',617,'',47 from dual union all
select 18526,'1969-A28','1969-01-21 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',45 from dual union all
select 18527,'1969-A29','1969-01-23 18:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',26 from dual union all
select 18528,'1969-A30','1969-01-23 21:30:00','suborbital rocket','success',1061,'','Starute','','','Meteo',617,'',27 from dual union all
select 18529,'1969-A31','1969-01-24 15:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',44 from dual union all
select 1853,'1953-A423','1953-11-02 00:00:00','military missile','success',771,'GM-11097','','','','Test',222,'',12 from dual union all
select 18530,'1969-A32','1969-01-24 18:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',45 from dual union all
select 18531,'1969-A33','1969-01-24 18:42:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',45 from dual union all
select 18532,'1969-A34','1969-01-26 09:47:00','suborbital rocket','failure',867,'NASA 18.11GI','','','','Ionos',1821,'',19 from dual union all
select 18533,'1969-A35','1969-01-27 18:57:59','suborbital rocket','success',1188,'','','','','Meteo',1447,'',39 from dual union all
select 18534,'1969-A36','1969-01-28 21:33:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',49 from dual union all
select 18535,'1969-A37','1969-01-29 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 18536,'1969-A38','1969-01-30 15:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',49 from dual union all
select 18537,'1969-A39','1969-01-31 17:47:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',47 from dual union all
select 18538,'1969-A40','1969-01-31 21:25:00','suborbital rocket','failure',839,'NASA 10.268GM','GSFC Grenade 210 W','','','Aeronomy',1821,'',20 from dual union all
select 18539,'1969-A41','1969-02-04 15:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',39 from dual union all
select 1854,'1953-A424','1953-11-03 00:00:00','atmospheric rocket','unknown',576,'208M','','','','Test',1895,'',10 from dual union all
select 18540,'1969-A42','1969-02-10 00:00:00','military missile','success',1252,'FLA-27','','','','Test',1888,'',6 from dual union all
select 18541,'1969-A43','1969-02-13 13:48:00','suborbital rocket','failure',288,'','Rehbar 20','','','Aeronomy',1419,'',12 from dual union all
select 18542,'1969-A44','1969-02-14 18:35:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',47 from dual union all
select 18543,'1969-A45','1969-02-21 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 18544,'1969-A46','1969-03-12 16:22:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',33 from dual union all
select 18545,'1969-A47','1969-03-12 18:06:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',28 from dual union all
select 18546,'1969-A48','1969-03-13 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',45 from dual union all
select 18547,'1969-A49','1969-03-14 16:13:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',47 from dual union all
select 18548,'1969-A50','1969-03-14 17:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',48 from dual;
insert into LAUNCH
select 18549,'1969-A51','1969-03-15 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 1855,'1953-A425','1953-11-03 00:00:00','atmospheric rocket','unknown',576,'207M','','','','Test',1895,'',10 from dual union all
select 18550,'1969-A52','1969-03-17 00:00:00','military missile','failure',1252,'FLA-30','','','','Test',1888,'',6 from dual union all
select 18551,'1969-A53','1969-03-21 15:12:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',49 from dual union all
select 18552,'1969-A54','1969-03-27 19:05:00','atmospheric rocket','success',835,'Smoke 109','','','','Aeronomy',1821,'',17 from dual union all
select 18553,'1969-A55','1969-03-27 19:47:00','atmospheric rocket','success',835,'Smoke 110','','','','Aeronomy',1821,'',14 from dual union all
select 18554,'1969-A56','1969-04-01 00:00:00','atmospheric rocket','success',617,'J207','','','','Test',1836,'',30 from dual union all
select 18555,'1969-A57','1969-04-07 20:40:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',617,'',49 from dual union all
select 18556,'1969-A58','1969-04-12 19:00:00','suborbital rocket','failure',217,'ADF-3-29','','','','Ionos',428,'',26 from dual union all
select 18557,'1969-A59','1969-04-14 18:15:00','suborbital rocket','failure',290,'C39/1','ESRO C39/1','','','Auroral',408,'',5 from dual union all
select 18558,'1969-A60','1969-04-15 21:17:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',46 from dual union all
select 18559,'1969-A61','1969-04-17 19:52:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 1856,'1953-A426','1953-11-03 00:00:00','atmospheric rocket','success',564,'','D37 Model','','','Test',1821,'',5 from dual union all
select 18560,'1969-A62','1969-04-18 15:10:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',34 from dual union all
select 18561,'1969-A63','1969-04-19 00:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',46 from dual union all
select 18562,'1969-A64','1969-04-21 00:00:00','military missile','success',1252,'FLA-28','','','','Test',1888,'',6 from dual union all
select 18563,'1969-A65','1969-04-21 03:00:00','suborbital rocket','success',1506,'Sphere 4','Met Sphere','','','Aeron',1895,'',43 from dual union all
select 18564,'1969-A66','1969-04-22 21:17:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',49 from dual union all
select 18565,'1969-A67','1969-04-25 16:30:00','suborbital rocket','success',1506,'Sphere 5','Met Sphere','','','Aeron',1895,'',46 from dual union all
select 18566,'1969-A68','1969-04-28 06:07:00','suborbital rocket','success',1506,'Sphere 7','Met Sphere','','','Aeron',1895,'',18 from dual union all
select 18567,'1969-A69','1969-04-29 18:44:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',35 from dual union all
select 18568,'1969-A70','1969-04-29 19:00:00','military missile','failure',916,'AO-78','P-165 69 Spring 06','','','Test',204,'',10 from dual union all
select 18569,'1969-A71','1969-04-29 21:23:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',46 from dual union all
select 1857,'1953-A427','1953-11-03 00:00:00','atmospheric rocket','success',836,'1259R','Nike 262R','','','Test',1895,'',10 from dual union all
select 18570,'1969-A72','1969-04-30 18:20:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 18571,'1969-A73','1969-05-05 00:00:00','atmospheric rocket','failure',617,'J202-2','UK-2Q/HRV 932','','','Test',1836,'',30 from dual union all
select 18572,'1969-A74','1969-05-02 13:58:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',48 from dual union all
select 18573,'1969-A75','1969-05-02 18:32:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',39 from dual union all
select 18574,'1969-A76','1969-05-02 19:21:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',39 from dual union all
select 18575,'1969-A77','1969-05-02 21:37:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',45 from dual union all
select 18576,'1969-A78','1969-05-07 14:57:00','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',39 from dual union all
select 18577,'1969-A79','1969-05-07 17:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1415,'',48 from dual union all
select 18578,'1969-A80','1969-05-12 13:14:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',35,'',41 from dual union all
select 18579,'1969-A81','1969-05-13 17:50:00','atmospheric rocket','success',835,'Smoke 111','','','','Aeronomy',1821,'',18 from dual union all
select 1858,'1953-A428','1953-11-03 00:00:00','atmospheric rocket','success',836,'1241R','Nike 259R','','','Test',1895,'',10 from dual union all
select 18580,'1969-A82','1969-05-13 21:01:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',49 from dual union all
select 18581,'1969-A83','1969-05-14 16:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',34 from dual union all
select 18582,'1969-A84','1969-05-15 14:00:00','suborbital rocket','success',1061,'','','','','Meteo',276,'',43 from dual union all
select 18583,'1969-A85','1969-05-15 20:32:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',43 from dual union all
select 18584,'1969-A86','1969-05-20 20:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',46 from dual union all
select 18585,'1969-A87','1969-05-21 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 18586,'1969-A88','1969-05-21 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',44 from dual union all
select 18587,'1969-A89','1969-05-22 21:01:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',48 from dual union all
select 18588,'1969-A90','1969-05-26 00:00:00','military missile','success',1252,'FLA-31','','','','Test',1888,'',6 from dual union all
select 18589,'1969-A91','1969-05-27 20:26:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',49 from dual union all
select 1859,'1953-A429','1953-11-03 00:00:00','atmospheric rocket','success',836,'1293R','Nike 263R','','','Test',1895,'',10 from dual union all
select 18590,'1969-A92','1969-05-28 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',45 from dual union all
select 18591,'1969-A93','1969-05-29 21:17:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',48 from dual union all
select 18592,'1969-A94','1969-06-18 16:17:00','suborbital rocket','success',549,'PEPP','40-ft Ringsail Para','','','Test',1895,'',40 from dual union all
select 18593,'1969-A95','1969-06-03 21:07:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',49 from dual union all
select 18594,'1969-A96','1969-06-04 00:00:00','military missile','failure',1251,'800011','KT-9','','','Test',868,'',1 from dual union all
select 18595,'1969-A97','1969-06-05 21:06:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',46 from dual union all
select 18596,'1969-A98','1969-06-09 14:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',48 from dual union all
select 18597,'1969-A99','1969-06-12 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',39 from dual union all
select 18598,'1969-F01','1969-01-20 04:14:36','orbital','failure',953,'237-01','','L-1 No. 13L','','Sat',587,'',200 from dual union all
select 18599,'1969-F03','1969-02-01 12:11:00','orbital','failure',1510,'V 15000-060','','Meteor No. 11','','Sat',473,'',150 from dual union all
select 186,'1944-S169','1944-10-01 12:18:00','military missile','success',1470,'18773','Ma188','','','Test',661,'',90 from dual union all
select 1860,'1953-A43','1953-02-05 00:00:00','military missile','unknown',339,'43R','','','','Test',1895,'',30 from dual union all
select 18600,'1969-F04','1969-02-19 06:48:15','orbital','failure',953,'239-01','','E-8 No. 201','','Sat',588,'',15 from dual union all
select 18601,'1969-F05','1969-02-21 09:18:07','orbital','failure',823,'3L','','L-1S No. 3  11F92','','Sat',499,'',30 from dual union all
select 18602,'1969-F06','1969-03-27 10:40:45','orbital','failure',953,'240-01','','2M No. 521','','Sat',587,'',180 from dual union all
select 18603,'1969-F07','1969-04-02 10:33:00','orbital','failure',953,'233-01','','2M No. 522','','Sat',588,'',40 from dual union all
select 18604,'1969-F08','1969-06-14 04:00:47','orbital','failure',953,'238-01','','E-8-5 No. 402','','Sat',588,'',200 from dual union all
select 18605,'1969-F09','1969-07-02 22:55:00','orbital','failure',467,'F-8','STV 2','STV 2 Adapter','','Sat',1845,'',300 from dual union all
select 18606,'1969-F10','1969-07-03 20:18:32','orbital','failure',823,'5L/V15005','','L-1S No. 5','','Sat',499,'',0 from dual union all
select 18607,'1969-F11','1969-07-23 09:00:00','orbital','failure',678,'V295-11','','DS-P1-Yu No. 23','','Sat',460,'',270 from dual union all
select 18608,'1969-F12','1969-08-27 21:59:00','orbital','failure',1385,'540/D73','Pioneer E','TETR C','','Sat',233,'',100 from dual union all
select 18609,'1969-F13','1969-09-22 02:10:00','orbital','failure',699,'L-4S-4','','L-4S-4','','Sat',834,'',400 from dual union all
select 1861,'1953-A430','1953-11-03 00:00:00','atmospheric rocket','success',836,'1273R','Nike 261R','','','Test',1895,'',10 from dual union all
select 18610,'1969-F14','1969-11-28 09:00:00','orbital','failure',953,'245-01','','L-1e No 1','','Sat',587,'',180 from dual union all
select 18611,'1969-F15','1969-12-27 14:20:00','orbital','failure',680,'Yu226   -12U','','Ionosfernaya Stantsiya N','','Sat',456,'',1000 from dual union all
select 18612,'1969-M01','1969-01-19 19:05:00','suborbital rocket','success',839,'NASA 10.314IM','RTG SNC 2B/2 K69/1','','','Aeronomy',413,'',75 from dual union all
select 18613,'1969-M02','1969-01-30 01:31:00','suborbital rocket','success',826,'NAL-16 31D','','','','Test',1476,'',50 from dual union all
select 18614,'1969-M03','1969-02-01 01:40:00','suborbital rocket','success',827,'NAL-25 31','','','','Test',1476,'',50 from dual union all
select 18615,'1969-M04','1969-02-06 00:00:00','military missile','success',1251,'800008','KT-6','','','Test',868,'',50 from dual union all
select 18616,'1969-M05','1969-02-09 13:46:00','suborbital rocket','failure',288,'','Rehbar 19','','','Aeron',1419,'',55 from dual union all
select 18617,'1969-M06','1969-02-25 21:37:10','suborbital rocket','success',1181,'','ESRO A40/3','','','Ionos/Plasma/Field',29,'',88 from dual union all
select 18618,'1969-M08','1969-04-13 07:43:13','suborbital rocket','success',1181,'','ESRO A40/6','','','Ionos/Plasma/Field',29,'',51 from dual union all
select 18619,'1969-M09','1969-04-13 09:40:40','suborbital rocket','success',1181,'','ESRO A40/4','','','Ionos/Plasma/Field',29,'',66 from dual union all
select 1862,'1953-A431','1953-11-03 00:00:00','atmospheric rocket','success',863,'1294B-1261 T48','Nike 260R/Bell motor','','','Test',1895,'',10 from dual union all
select 18620,'1969-M10','1969-04-14 16:11:00','suborbital rocket','success',1181,'','ESRO A40/5','','','Ionos/Plasma/Field',29,'',56 from dual union all
select 18621,'1969-M11','1969-04-14 19:00:04','suborbital rocket','success',1181,'','ESRO A40/1','','','Ionos/Plasma/Field',29,'',58 from dual union all
select 18622,'1969-M12','1969-04-16 14:45:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',65 from dual union all
select 18623,'1969-M13','1969-04-18 14:37:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',52 from dual union all
select 18624,'1969-M14','1969-04-19 00:00:00','suborbital rocket','success',203,'B116','FU-191','','','Test',340,'',70 from dual union all
select 18625,'1969-M15','1969-04-21 14:37:59','suborbital rocket','success',1188,'','','','','Meteo',450,'',62 from dual union all
select 18626,'1969-M16','1969-04-23 14:23:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',66 from dual union all
select 18627,'1969-M17','1969-04-24 14:15:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',64 from dual union all
select 18628,'1969-M18','1969-04-27 19:53:59','suborbital rocket','success',1188,'','','','','Meteo',450,'',63 from dual union all
select 18629,'1969-M19','1969-04-29 14:00:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',62 from dual union all
select 1863,'1953-A432','1953-11-04 00:00:00','military missile','unknown',339,'1303R','','','','Test',1895,'',30 from dual union all
select 18630,'1969-M20','1969-05-01 20:27:59','suborbital rocket','success',1188,'','','','','Meteo',450,'',60 from dual union all
select 18631,'1969-M21','1969-05-03 13:52:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',64 from dual union all
select 18632,'1969-M22','1969-05-03 19:56:59','suborbital rocket','success',1188,'','','','','Meteo',450,'',59 from dual union all
select 18633,'1969-M23','1969-05-04 00:00:00','military missile','success',1162,'','','','','OT?',1862,'',50 from dual union all
select 18634,'1969-M24','1969-05-04 00:00:00','military missile','success',1162,'','','','','OT?',1861,'',50 from dual union all
select 18635,'1969-M26','1969-07-31 00:00:00','military missile','success',1251,'800014','KT-11','','','Test',868,'',50 from dual union all
select 18636,'1969-M27','1969-08-18 00:00:00','suborbital rocket','success',203,'B117','FU-191B','','','Test',340,'',67 from dual union all
select 18637,'1969-M28','1969-08-25 00:00:00','suborbital rocket','success',1069,'IT-160-3','Instrumentation Test','','','Aeron',839,'',78 from dual union all
select 18638,'1969-M29','1969-09-09 06:30:00','suborbital rocket','success',1076,'SB-III F11','','','','Test',1476,'',75 from dual;
insert into LAUNCH
select 18639,'1969-M30','1969-09-23 14:40:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',62 from dual union all
select 1864,'1953-A433','1953-11-04 00:00:00','atmospheric rocket','unknown',576,'2DJ','','','','Test',1895,'',10 from dual union all
select 18640,'1969-M31','1969-09-25 00:00:00','suborbital rocket','success',204,'B307','CRAPEL/ELECTRE','','','Reentry Test',307,'',60 from dual union all
select 18641,'1969-M32','1969-10-17 00:00:00','military missile','success',1251,'800016','KT-13','','','Test',868,'',50 from dual union all
select 18642,'1969-M33','1969-11-25 00:00:00','military missile','success',1251,'800015','KT-14','','','Test',868,'',50 from dual union all
select 18643,'1969-M34','1969-12-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 18644,'1969-M35','1969-12-09 01:41:00','suborbital rocket','unknown',675,'WRE K138','','','','Aeron',1832,'',73 from dual union all
select 18645,'1969-M36','1969-12-20 11:10:00','suborbital rocket','success',598,'INTA I-6901','','','','Test',396,'',73 from dual union all
select 18646,'1969-M37','1969-08-05 00:00:00','suborbital rocket','failure',1506,'SDC VD-35','ROBIN','','','Aeron',276,'',77 from dual union all
select 18647,'1969-M39','1969-10-13 00:00:00','suborbital rocket','success',738,'MMR-06','','','','Aeron',636,'',65 from dual union all
select 18648,'1969-M40','1969-10-14 00:00:00','suborbital rocket','success',738,'MMR-06','','','','Aeron',636,'',65 from dual union all
select 18649,'1969-M44','1969-03-19 10:57:00','suborbital rocket','success',631,'INTA H-6905','','','','Aeron',396,'',60 from dual union all
select 1865,'1953-A434','1953-11-06 00:00:00','military missile','unknown',339,'1358R','','','','Test',1895,'',30 from dual union all
select 18650,'1969-M45','1969-03-28 01:39:00','suborbital rocket','success',631,'INTA H-6906','','','','Aeron',396,'',60 from dual union all
select 18651,'1969-M46','1969-03-29 01:05:00','suborbital rocket','success',631,'INTA H-6907','','','','Aeron',396,'',60 from dual union all
select 18652,'1969-M88','1969-10-01 00:00:00','suborbital rocket','success',229,'RDT&E XM-75','Test 1','','','Test',1895,'',60 from dual union all
select 18653,'1969-M89','1969-03-01 00:00:00','suborbital rocket','success',361,'','Dardo de Invest. Met.','','','Meteo',308,'',70 from dual union all
select 18654,'1969-M90','1969-03-01 00:00:00','suborbital rocket','success',361,'','Dardo de Invest. Met.','','','Meteo',308,'',70 from dual union all
select 18655,'1969-M91','1969-03-01 00:00:00','suborbital rocket','success',361,'','Dardo de Invest. Met.','','','Meteo',308,'',70 from dual union all
select 18656,'1969-M92','1969-08-31 00:00:00','military missile','success',1162,'','','RB350 Training','','OT',984,'',50 from dual union all
select 18657,'1969-S01','1969-01-01 00:00:00','suborbital rocket','success',205,'','','','','Weather',311,'',90 from dual union all
select 18658,'1969-S02','1969-10-11 00:00:00','military missile','success',356,'629903','Env test','','','Test',650,'',500 from dual union all
select 18659,'1969-S03','1969-01-01 00:00:00','military missile','success',751,'V4','V4','','','OT',307,'S655',500 from dual union all
select 1866,'1953-A435','1953-11-10 00:00:00','atmospheric rocket','unknown',576,'3DJ','','','','Test',1895,'',10 from dual union all
select 18660,'1969-S04','1969-01-01 00:00:00','military missile','success',751,'V5','V5','','','OT',307,'S655',500 from dual union all
select 18661,'1969-S05','1969-01-01 00:00:00','military missile','success',751,'V6','V6','','','OT',307,'S655',500 from dual union all
select 18662,'1969-S06','1969-02-28 00:00:00','military missile','success',982,'T 22505-02T','8K64U UBP-127','','','OT',596,'',1210 from dual union all
select 18663,'1969-S07','1969-10-02 00:00:00','military missile','success',982,'','8K64U UBP-128','','','OT',596,'',1210 from dual union all
select 18664,'1969-S08','1969-12-13 00:00:00','military missile','success',982,'T 22501-19T','8K64U UBP-135','','','OT',596,'',1210 from dual union all
select 18665,'1969-S09','1969-10-12 00:00:00','military missile','success',982,'T 22503-10T','8K64U UBP-129','','','OT',596,'',1210 from dual union all
select 18666,'1969-S10','1969-12-01 00:00:00','military missile','success',982,'T 22505-04T','8K64U UBP-134','','','OT',596,'',1210 from dual union all
select 18667,'1969-S100','1969-01-31 18:36:00','suborbital rocket','success',848,'CRL AH07.573','AFCRL Sphere','','','Aeron',1821,'',188 from dual union all
select 18668,'1969-S101','1969-01-31 19:10:00','suborbital rocket','success',837,'NASA 14.389UA','','','','Aeronomy',1821,'',115 from dual union all
select 18669,'1969-S102','1969-01-31 21:12:00','suborbital rocket','success',837,'NASA 14.396GM','','','','Aeronomy',1821,'',116 from dual union all
select 1867,'1953-A436','1953-11-10 00:00:00','atmospheric rocket','unknown',576,'4DJ','','','','Test',1895,'',10 from dual union all
select 18670,'1969-S103','1969-01-31 21:53:00','suborbital rocket','success',848,'CRL AH07.576','AFCRL Sphere','','','Aeron',1821,'',188 from dual union all
select 18671,'1969-S104','1969-02-02 08:25:00','military missile','success',796,'FTM-2208','Mk 11C RV','DASO GIANT FIST 5','GIANT FIST 5','OT',1699,'',1300 from dual union all
select 18672,'1969-S105','1969-02-03 21:05:38','suborbital rocket','success',920,'P24K','Electrons','','','Ionospheric',413,'',157 from dual union all
select 18673,'1969-S106','1969-02-04 15:11:00','suborbital rocket','success',846,'STV 82','','','','Aeron',1865,'',100 from dual union all
select 18674,'1969-S107','1969-02-04 16:00:00','suborbital rocket','success',846,'STV 83','','','','Aeron',1864,'',100 from dual union all
select 18675,'1969-S108','1969-02-04 14:35:00','suborbital rocket','success',30,'NASA 04.272UA','','','','Aeronomy',428,'',186 from dual union all
select 18676,'1969-S109','1969-02-04 22:30:00','suborbital rocket','success',839,'NASA 10.300GM','GSFC Grenade 211 PB','','','Aeronomy',1039,'',100 from dual union all
select 18677,'1969-S11','1969-12-30 00:00:00','military missile','success',982,'T 22504-08T','8K64U UBP-137','','','OT',596,'',1210 from dual union all
select 18678,'1969-S110','1969-02-06 10:30:00','suborbital rocket','success',447,'D-258','FU-180 Mass spec. F1','','','Ionos',307,'',430 from dual union all
select 18679,'1969-S111','1969-02-06 06:31:00','suborbital rocket','failure',694,'LS-C-1','','','','Test',1476,'',20 from dual union all
select 1868,'1953-A437','1953-11-12 00:00:00','atmospheric rocket','unknown',576,'5DJ','','','','Test',1895,'',10 from dual union all
select 18680,'1969-S112','1969-02-06 09:39:00','suborbital rocket','success',839,'NASA 10.303GM','GSFC Grenade 212 C','','','Aeronomy',428,'',129 from dual union all
select 18681,'1969-S113','1969-02-06 14:35:00','suborbital rocket','success',30,'NASA 04.273UA','','','','Aeronomy',428,'',138 from dual union all
select 18682,'1969-S114','1969-02-06 17:09:00','suborbital rocket','success',837,'NASA 14.392UI','','','','Ionos',1821,'',219 from dual union all
select 18683,'1969-S115','1969-02-06 19:29:00','suborbital rocket','success',837,'NASA 14.390UA','','','','Aeronomy',1821,'',117 from dual union all
select 18684,'1969-S116','1969-02-06 21:09:00','suborbital rocket','success',839,'NASA 10.267GM','GSFC Grenade 213 W','','','Aeronomy',1821,'',128 from dual union all
select 18685,'1969-S117','1969-02-06 22:20:00','suborbital rocket','success',447,'D-256','FU-180 Mass spec. F3','','','Ionos',307,'',430 from dual union all
select 18686,'1969-S118','1969-02-06 13:45:00','suborbital rocket','success',447,'D-257','FU-180 Mass spec. F2','','','Ionos',307,'',405 from dual union all
select 18687,'1969-S119','1969-02-07 01:22:00','suborbital rocket','success',447,'D-259','FU-180 Mass spec. F4','','','Ionos',307,'',413 from dual union all
select 18688,'1969-S12','1969-01-22 00:00:00','military missile','success',982,'T 22505-08T','8K64U UBP-126','','','OT',596,'',1210 from dual union all
select 18689,'1969-S120','1969-02-07 06:30:00','suborbital rocket','success',1077,'SC-3','','','','Test',1476,'',85 from dual union all
select 1869,'1953-A438','1953-11-12 00:00:00','atmospheric rocket','success',836,'1136A','','','','Test',1895,'',10 from dual union all
select 18690,'1969-S121','1969-02-08 01:30:00','suborbital rocket','success',1077,'SC-4','','','','Test',1476,'',85 from dual union all
select 18691,'1969-S122','1969-02-08 03:19:00','suborbital rocket','success',30,'NASA 04.308UA','JHU UVT F4 (Venus)','','','UV Astron',1869,'',164 from dual union all
select 18692,'1969-S123','1969-02-08 04:40:00','suborbital rocket','success',837,'NASA 14.269UI','','','','Ionos',428,'',133 from dual union all
select 18693,'1969-S124','1969-02-08 06:30:00','suborbital rocket','success',1077,'SC-1','','','','Test',1476,'',85 from dual union all
select 18694,'1969-S125','1969-02-11 21:09:00','suborbital rocket','success',920,'P23K','Electrons','','','Ionospheric',413,'',157 from dual union all
select 18695,'1969-S126','1969-02-12 03:16:00','suborbital rocket','success',837,'NASA 14.306UI','','','','Ionos',428,'',137 from dual union all
select 18696,'1969-S127','1969-02-12 04:25:00','suborbital rocket','failure',30,'NASA 04.188UA','JHU Aurora 7','','','Aeronomy/Auroral',428,'',160 from dual union all
select 18697,'1969-S128','1969-02-12 06:41:00','suborbital rocket','success',839,'Launcher test','','','','Test',428,'',100 from dual union all
select 18698,'1969-S129','1969-02-12 17:15:00','suborbital rocket','success',30,'NASA 04.135DS','NE3.175','','','Solar UV',1869,'',188 from dual union all
select 18699,'1969-S13','1969-10-13 00:00:00','military missile','success',982,'T 22504-10T','8K64U UBP-130','','','OT',596,'',1210 from dual union all
select 187,'1944-S17','1944-03-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1870,'1953-A439','1953-11-13 00:00:00','military missile','unknown',339,'1359R','','','','Test',1895,'',30 from dual union all
select 18700,'1969-S130','1969-02-13 00:00:00','suborbital rocket','success',655,'PT-420-001','TVC test (1-stage)','','','Test',839,'',16 from dual union all
select 18701,'1969-S131','1969-02-14 03:20:00','military missile','success',135,'B043','Endo Decoy D-5','ABRES AN-2 (USAF)','','Reentry Test',612,'',200 from dual union all
select 18702,'1969-S132','1969-02-13 02:05:00','suborbital rocket','success',668,'K-9M-23','','','','Fields/Solar UV',839,'',310 from dual union all
select 18703,'1969-S133','1969-02-13 23:11:00','suborbital rocket','success',215,'AMF-2-46','','','','Aeron',1821,'',265 from dual union all
select 18704,'1969-S134','1969-02-13 23:11:00','suborbital rocket','success',867,'NASA 18.86CA','GCA Release Na','','','Aeronomy',1821,'',270 from dual union all
select 18705,'1969-S135','1969-02-14 01:00:00','suborbital rocket','success',837,'NASA 14.397CM','GCA Release TMA','','','Aeronomy',1821,'',265 from dual union all
select 18706,'1969-S136','1969-02-14 03:19:00','suborbital rocket','success',837,'NASA 14.399CM','GCA Release TMA','','','Aeronomy',1821,'',212 from dual union all
select 18707,'1969-S137','1969-02-14 05:00:00','suborbital rocket','success',837,'NASA 14.400CM','GCA Release TMA','','','Aeronomy',1821,'',210 from dual union all
select 18708,'1969-S138','1969-02-14 07:00:00','suborbital rocket','success',837,'NASA 14.401CM','GCA Release TMA','','','Aeronomy',1821,'',209 from dual union all
select 18709,'1969-S139','1969-02-14 09:00:00','suborbital rocket','success',837,'NASA 14.398CM','GCA Release TMA','','','Aeronomy',1821,'',206 from dual union all
select 1871,'1953-A44','1953-02-05 00:00:00','military missile','success',771,'GM-2357','','','','Test',222,'',12 from dual union all
select 18710,'1969-S14','1969-10-14 00:00:00','military missile','success',982,'T 22503-06T','8K64U UBP-131','','','OT',596,'',1210 from dual union all
select 18711,'1969-S140','1969-02-14 11:13:00','suborbital rocket','success',837,'NASA 14.402CM','GCA Release Na','','','Aeronomy',1821,'',213 from dual union all
select 18712,'1969-S141','1969-02-14 22:15:40','suborbital rocket','success',920,'P27K','Small scale struct','','','Ionospheric',413,'',149 from dual union all
select 18713,'1969-S142','1969-02-15 21:20:00','suborbital rocket','success',920,'P28K','Small scale struct','','','Ionospheric',413,'',153 from dual union all
select 18714,'1969-S143','1969-02-17 20:02:00','suborbital rocket','success',837,'NASA 14.374UI','','','','Ionos',428,'',135 from dual union all
select 18715,'1969-S144','1969-02-19 16:05:00','military missile','success',939,'C3XD-04','','','','Test',246,'',500 from dual union all
select 18716,'1969-S145','1969-02-19 04:17:00','suborbital rocket','success',215,'AMF-2-117','VUV auroral spectrum','','','Auroral',428,'',124 from dual union all
select 18717,'1969-S146','1969-02-20 10:15:00','military missile','success',795,'1220','Mk 11A RV','ST OLYMPIC T. B-8','OLYMPIC TRIALS B-8','OT',1705,'',1300 from dual union all
select 18718,'1969-S147','1969-02-20 17:41:02','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 616',1000 from dual union all
select 18719,'1969-S148','1969-02-20 11:48:00','suborbital rocket','success',1493,'AGI 61','FU-170 CIRCE','','','Ionos',341,'',107 from dual union all
select 1872,'1953-A440','1953-11-13 00:00:00','atmospheric rocket','success',836,'1210A','','','','Test',1895,'',10 from dual union all
select 18720,'1969-S149','1969-02-21 03:01:00','suborbital rocket','failure',837,'NASA 14.307UI','','','','Ionos',428,'',200 from dual union all
select 18721,'1969-S15','1969-10-30 00:00:00','military missile','success',982,'T 22505-13T','8K64U UBP-132','','','OT',596,'',1210 from dual union all
select 18722,'1969-S150','1969-02-25 13:21:00','suborbital rocket','success',1181,'','ESRO A40/2','','','Ionos/Plasma/Field',29,'',86 from dual union all
select 18723,'1969-S151','1969-02-25 16:33:20','suborbital rocket','success',290,'CE','ESRO C49/1 (R3)','','','Auroral',29,'',136 from dual union all
select 18724,'1969-S152','1969-02-25 23:50:04','suborbital rocket','success',290,'CE','ESRO C49/2','','','Auroral',29,'',128 from dual union all
select 18725,'1969-S153','1969-02-26 10:00:00','suborbital rocket','unknown',289,'ISRO-45.002','ISRO 45.02','TERLS-104','','Ionos',1467,'',145 from dual union all
select 18726,'1969-S154','1969-02-27 16:14:00','suborbital rocket','success',846,'STV 84','','','','Aeron',1864,'',214 from dual union all
select 18727,'1969-S155','1969-02-27 02:05:00','suborbital rocket','success',837,'NASA 14.321UE','','','','Magnetospheric',428,'',166 from dual union all
select 18728,'1969-S156','1969-03-01 00:00:00','suborbital rocket','success',516,'','','','','Test',311,'',130 from dual;
insert into LAUNCH
select 18729,'1969-S157','1969-03-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 1873,'1953-A441','1953-11-13 00:00:00','atmospheric rocket','success',836,'1225A','','','','Test',1895,'',10 from dual union all
select 18730,'1969-S158','1969-03-01 00:00:00','military missile','success',1514,'','','','','Test',989,'',1000 from dual union all
select 18731,'1969-S159','1969-02-28 18:57:00','suborbital rocket','success',218,'NASA 12.04GT','','','','Test',1821,'',215 from dual union all
select 18732,'1969-S16','1969-12-20 00:00:00','military missile','success',982,'T 22504-11T','8K64U UBP-136','','','OT',596,'',1210 from dual union all
select 18733,'1969-S160','1969-03-01 09:00:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',173 from dual union all
select 18734,'1969-S161','1969-03-03 10:00:00','suborbital rocket','success',32,'NASA 04.199GE','XR background','','','XR Astron',1869,'',163 from dual union all
select 18735,'1969-S162','1969-03-04 04:30:00','suborbital rocket','success',846,'SECEDE R3','SECEDE III JUNIPER','','','Aeronomy Ba',1041,'',170 from dual union all
select 18736,'1969-S163','1969-03-05 04:30:00','suborbital rocket','success',846,'SECEDE R4','SECEDE III ELM','','','Aeronomy Ba',1041,'',170 from dual union all
select 18737,'1969-S164','1969-03-05 09:00:00','suborbital rocket','success',725,'','Particle flux 16','','','Plasma',1010,'',80 from dual union all
select 18738,'1969-S165','1969-03-06 00:00:00','suborbital rocket','success',204,'304','FU-173','','','Test',340,'',60 from dual union all
select 18739,'1969-S166','1969-03-06 06:14:00','suborbital rocket','success',219,'AAD-4-16','','','','Ionos/Plasma',428,'',791 from dual union all
select 1874,'1953-A442','1953-11-17 00:00:00','atmospheric rocket','unknown',576,'209M','','','','Test',1895,'',10 from dual union all
select 18740,'1969-S167','1969-03-07 08:20:00','military missile','success',796,'2252','Mk 11C RV','ST SAMSO Stage 3 Test?','SPEC TEST 2','Test',1700,'',1300 from dual union all
select 18741,'1969-S168','1969-03-07 02:27:00','suborbital rocket','success',30,'NASA 04.286UG','CAL XR 3','','','XR Astron',1869,'',160 from dual union all
select 18742,'1969-S169','1969-03-07 13:12:00','suborbital rocket','success',867,'NASA 18.83GE','Ba','','','Magnetospheric',334,'',270 from dual union all
select 18743,'1969-S17','1969-11-15 00:00:00','military missile','success',982,'T 22504-15T','8K64U UBP-133','','','OT',596,'',1210 from dual union all
select 18744,'1969-S170','1969-03-09 22:39:58','suborbital rocket','success',30,'CRL AT3.727','Arcmin probe','','','Solar opt/Aeron',330,'',193 from dual union all
select 18745,'1969-S171','1969-03-08 03:18:00','suborbital rocket','success',867,'NASA 18.84GE','Ba','','','Magnetospheric',334,'',270 from dual union all
select 18746,'1969-S172','1969-03-08 09:00:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',173 from dual union all
select 18747,'1969-S173','1969-03-08 15:35:00','suborbital rocket','success',740,'','Release','','','Aeron',1010,'',173 from dual union all
select 18748,'1969-S174','1969-03-09 03:29:00','suborbital rocket','success',867,'NASA 18.85GE','Ba','','','Magnetospheric',334,'',270 from dual union all
select 18749,'1969-S175','1969-03-09 16:35:00','suborbital rocket','success',740,'','Release','','','Aeron',1010,'',172 from dual union all
select 1875,'1953-A443','1953-11-17 00:00:00','atmospheric rocket','unknown',576,'210M','','','','Test',1895,'',10 from dual union all
select 18750,'1969-S176','1969-03-10 04:40:00','suborbital rocket','success',227,'AKD-VB-20','','','','Auroral/Ionos/Solar',428,'',362 from dual union all
select 18751,'1969-S177','1969-03-10 16:10:00','suborbital rocket','success',740,'','Release','','','Aeron',1010,'',168 from dual union all
select 18752,'1969-S178','1969-03-10 23:00:00','suborbital rocket','success',740,'','Release','','','Aeron',1010,'',175 from dual union all
select 18753,'1969-S179','1969-03-11 03:33:00','suborbital rocket','success',227,'AAF-VB-22','','','','Auroral/Aeron/Ionos',428,'',141 from dual union all
select 18754,'1969-S18','1969-01-06 09:11:00','military missile','success',1018,'G 15003-04','8K75 UBP-14','','','OT',578,'',1160 from dual union all
select 18755,'1969-S180','1969-03-11 06:32:00','suborbital rocket','success',867,'NASA 18.81UA','','','','Aeronomy',428,'',270 from dual union all
select 18756,'1969-S181','1969-03-11 14:08:00','suborbital rocket','success',846,'SECEDE R5','SECEDE III IRONWOOD','','','Aeron/Fields',1041,'',140 from dual union all
select 18757,'1969-S182','1969-03-12 11:20:00','military missile','success',796,'FTM-2642','Mk 11C RV','DASO GIANT FIST 3','GIANT FIST 3','OT',1699,'',1300 from dual union all
select 18758,'1969-S183','1969-03-12 09:00:00','suborbital rocket','success',725,'','Particle flux 17','','','Plasma',1010,'',80 from dual union all
select 18759,'1969-S184','1969-03-13 16:45:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',174 from dual union all
select 1876,'1953-A444','1953-11-17 00:00:00','military missile','success',771,'GM-11100','','','','Test',222,'',12 from dual union all
select 18760,'1969-S185','1969-03-13 18:17:00','suborbital rocket','success',740,'','Release','','','Aeron',1010,'',170 from dual union all
select 18761,'1969-S186','1969-03-13 21:33:00','suborbital rocket','success',725,'','Particle flux 19','','','Plasma',1010,'',80 from dual union all
select 18762,'1969-S187','1969-03-13 22:23:00','suborbital rocket','success',740,'','Release','','','Aeron',1010,'',170 from dual union all
select 18763,'1969-S188','1969-03-14 05:30:00','suborbital rocket','success',30,'NASA 04.290DG','NRL NB3.210','','','XR Astron',1869,'',154 from dual union all
select 18764,'1969-S189','1969-03-14 07:01:00','suborbital rocket','success',30,'NASA 04.251GG-UG','UVT-SDGS 1','','','UV Astronomy',1869,'',177 from dual union all
select 18765,'1969-S19','1969-01-01 00:00:00','military missile','success',1036,'13','','','','Test',636,'',1000 from dual union all
select 18766,'1969-S190','1969-03-14 21:30:00','suborbital rocket','success',725,'','Particle flux 20','','','Plasma',1010,'',80 from dual union all
select 18767,'1969-S191','1969-03-14 22:20:00','suborbital rocket','success',740,'','','','','Aeron/Plasma',1010,'',161 from dual union all
select 18768,'1969-S192','1969-03-15 05:06:00','suborbital rocket','success',547,'SECEDE R6','SECEDE III FIR','','','Aeronomy',1047,'',165 from dual union all
select 18769,'1969-S193','1969-03-15 17:55:00','suborbital rocket','success',837,'K-NA-11','DLR K-NA-11','','','Aeron/Ionos/Fields',413,'',226 from dual union all
select 1877,'1953-A445','1953-11-18 00:00:00','military missile','success',771,'GM-11107','','','','Test',222,'',12 from dual union all
select 18770,'1969-S194','1969-03-15 20:45:15','suborbital rocket','success',1204,'S43/2','ESRO S43/2 Bir-18','','','Auroral',416,'',211 from dual union all
select 18771,'1969-S195','1969-03-15 22:31:02','suborbital rocket','success',290,'C52/1','ESRO C52/1','','','Auroral',408,'',136 from dual union all
select 18772,'1969-S196','1969-03-16 18:05:00','suborbital rocket','success',837,'K-NA-17','DLR K-NA-17','','','Aeron/Ionos',413,'',233 from dual union all
select 18773,'1969-S197','1969-03-17 00:04:50','suborbital rocket','success',920,'P29K','Particles','','','Ionospheric',413,'',151 from dual union all
select 18774,'1969-S198','1969-03-17 12:09:00','suborbital rocket','success',446,'D-29','FU-172 LPSP-EUV1','','','Solar EUV',340,'',410 from dual union all
select 18775,'1969-S199','1969-03-17 12:38:00','suborbital rocket','success',867,'NASA 18.15UE','','','','Magnetospheric',1041,'',216 from dual union all
select 18776,'1969-S20','1969-01-01 00:00:00','military missile','success',1036,'14','','','','Test',636,'',1000 from dual union all
select 18777,'1969-S200','1969-03-17 18:10:00','suborbital rocket','success',837,'K-NA-18','DLR K-NA-18','','','Aeron/Ionos/Fields',413,'',231 from dual union all
select 18778,'1969-S201','1969-03-17 18:23:05','suborbital rocket','success',920,'P25K','E field gradient','','','Ionospheric',413,'',148 from dual union all
select 18779,'1969-S202','1969-03-17 18:35:00','suborbital rocket','success',740,'','','','','Aeron/Plasma',1010,'',169 from dual union all
select 1878,'1953-A446','1953-11-18 00:00:00','military missile','success',771,'GM-11104','','','','Test',222,'',12 from dual union all
select 18780,'1969-S203','1969-03-18 05:17:00','suborbital rocket','success',547,'SECEDE R7','SECEDE III GUM','','','Aeronomy',1047,'',168 from dual union all
select 18781,'1969-S204','1969-03-18 18:16:00','suborbital rocket','success',920,'P11K','E-region E field','','','Ionospheric Ba',413,'',170 from dual union all
select 18782,'1969-S205','1969-03-19 09:00:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',97 from dual union all
select 18783,'1969-S206','1969-03-19 21:30:00','suborbital rocket','success',740,'','','','','Ionos/Plasma',1010,'',181 from dual union all
select 18784,'1969-S207','1969-03-20 23:10:00','military missile','success',939,'C3XD-05','USN Mk 3 RV x 14','','','Test',246,'',500 from dual union all
select 18785,'1969-S208','1969-03-20 05:37:00','suborbital rocket','success',1347,'SECEDE R8','SECEDE III HEMLOCK','','','Aeron/Fields/Ba',1047,'',176 from dual union all
select 18786,'1969-S209','1969-03-20 17:50:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',169 from dual union all
select 18787,'1969-S21','1969-01-01 00:00:00','military missile','success',1036,'15','','','','Test',636,'',1000 from dual union all
select 18788,'1969-S210','1969-03-20 18:30:00','suborbital rocket','success',740,'','Composition','','','Aeron/Plasma',1010,'',171 from dual union all
select 18789,'1969-S211','1969-03-20 20:30:00','suborbital rocket','success',740,'','','','','Aeron/Plasma',1010,'',169 from dual union all
select 1879,'1953-A447','1953-11-19 00:00:00','military missile','unknown',339,'1391R','','','','Test',1895,'',30 from dual union all
select 18790,'1969-S212','1969-03-21 04:53:00','suborbital rocket','success',30,'CRL AT3.756','Auroral Input/Output','','','Auroral',428,'',152 from dual union all
select 18791,'1969-S213','1969-03-21 20:06:00','suborbital rocket','success',740,'','E-region 1','','','Ionos',1010,'',175 from dual union all
select 18792,'1969-S214','1969-03-22 13:50:00','military missile','success',796,'2415','Mk 11B RV','ST Mk 1A Penaids Test 3?','','Test',1707,'',1300 from dual union all
select 18793,'1969-S215','1969-03-22 02:50:00','suborbital rocket','success',740,'','','','','Aeron/Ionos/Plasma',1010,'',184 from dual union all
select 18794,'1969-S216','1969-03-24 00:00:00','military missile','success',795,'791','Mk 11 RV','FOT GT33B','GLORY TRIP 33B','OT',1702,'',1300 from dual union all
select 18795,'1969-S217','1969-03-24 10:01:00','suborbital rocket','success',740,'','','','','Aeron',1010,'',181 from dual union all
select 18796,'1969-S218','1969-03-25 19:00:00','military missile','success',795,'1308','Mk 11 RV','FOT GT34B','GLORY TRIP 34B','OT',1698,'',1300 from dual union all
select 18797,'1969-S219','1969-03-26 20:46:21','military missile','success',799,'FTM-203','Mk 12 RV x3','R&D','','Test',264,'',1300 from dual union all
select 18798,'1969-S22','1969-07-01 00:00:00','military missile','success',1038,'V 22500-16L','8K99 LKI-10','','','Test',477,'',1000 from dual union all
select 18799,'1969-S220','1969-03-26 09:02:00','suborbital rocket','success',725,'','Particle flux 21','','','Plasma',1010,'',80 from dual union all
select 188,'1944-S170','1944-10-02 00:00:00','military missile','success',1470,'','Paris','','','Weapon',1739,'',90 from dual union all
select 1880,'1953-A448','1953-11-19 00:00:00','military missile','success',771,'GM-11098','','','','Test',222,'',12 from dual union all
select 18800,'1969-S221','1969-03-26 14:02:00','suborbital rocket','success',740,'','','','','Aeron/Ionos/Plasma',1010,'',181 from dual union all
select 18801,'1969-S222','1969-03-27 16:00:03','military missile','unknown',932,'A3E-667','','','','OT',421,'UK S23',1000 from dual union all
select 18802,'1969-S223','1969-03-27 00:00:00','military missile','success',135,'V138D','ABRES AJ-1 (ARPA)','ABRES AJ-1 (ARPA)','','Reentry Test',614,'',200 from dual union all
select 18803,'1969-S224','1969-03-27 13:35:00','suborbital rocket','success',740,'','','','','Aeron',636,'',181 from dual union all
select 18804,'1969-S225','1969-03-27 15:30:00','suborbital rocket','success',740,'','','','','Aeron',636,'',181 from dual union all
select 18805,'1969-S226','1969-03-28 00:00:00','military missile','unknown',930,'A2E','FOT','','','OT',421,'SSBN 610',1000 from dual union all
select 18806,'1969-S227','1969-03-28 00:00:00','military missile','unknown',930,'A2E','FOT','','','OT',421,'SSBN 610',1000 from dual union all
select 18807,'1969-S228','1969-03-28 00:00:00','military missile','unknown',930,'A2E','FOT','','','OT',421,'SSBN 610',1000 from dual union all
select 18808,'1969-S229','1969-03-28 00:00:00','military missile','unknown',930,'A2E','FOT','','','OT',421,'SSBN 610',1000 from dual union all
select 18809,'1969-S23','1969-01-01 00:00:00','suborbital rocket','success',1276,'','MTV-I','','','Reentry Test',855,'',200 from dual union all
select 1881,'1953-A449','1953-11-19 00:00:00','military missile','success',771,'GM-2359','','','','Test',222,'',12 from dual union all
select 18810,'1969-S230','1969-03-28 00:25:00','suborbital rocket','success',839,'NASA 10.309GM','INTA NC-6901','GSFC Grenade 214 E','','Aeronomy',396,'',90 from dual union all
select 18811,'1969-S231','1969-03-28 07:01:00','suborbital rocket','success',30,'NRL NB3.205','Stellar X-ray','','','XR Astron',1895,'',161 from dual union all
select 18812,'1969-S232','1969-03-28 07:32:00','suborbital rocket','success',839,'NASA 10.296GM','','GSFC Grenade 215 W','','Aeronomy',1821,'',121 from dual union all
select 18813,'1969-S233','1969-03-29 03:20:00','military missile','success',999,'Yu22501-14T','8K67 UBP-25','','','OT',515,'',1000 from dual union all
select 18814,'1969-S234','1969-03-29 00:05:00','suborbital rocket','success',839,'NASA 10.310GM','INTA NC-6902','GSFC Grenade 216 E','','Aeronomy',396,'',90 from dual union all
select 18815,'1969-S235','1969-03-29 00:41:00','suborbital rocket','success',217,'ADF-3-37','O2 IR profile','','','Auroral',428,'',108 from dual union all
select 18816,'1969-S236','1969-03-29 07:15:00','suborbital rocket','success',839,'NASA 10.308GM','','GSFC Grenade 217 W','','Aeronomy',1821,'',111 from dual union all
select 18817,'1969-S237','1969-03-29 11:58:00','suborbital rocket','success',217,'ADF-3-38','O2 IR profile','','','Auroral',428,'',114 from dual union all
select 18818,'1969-S238','1969-03-29 13:00:00','suborbital rocket','success',740,'','Mass spectrometer','','','Aeron/Ionos',636,'',175 from dual;
insert into LAUNCH
select 18819,'1969-S239','1969-03-29 20:00:00','suborbital rocket','success',920,'P26K','Small scale struct','','','Ionospheric',413,'',150 from dual union all
select 1882,'1953-A45','1953-02-05 00:00:00','atmospheric rocket','success',836,'306P','Nike CET 6','','','Test',1895,'',10 from dual union all
select 18820,'1969-S24','1969-01-21 00:00:00','military missile','success',1449,'','8K84 UBP-99','','','OT',504,'',1000 from dual union all
select 18821,'1969-S240','1969-03-30 03:33:00','suborbital rocket','success',30,'CRL AT3.759','Auroral Input/Output','','','Auroral',428,'',150 from dual union all
select 18822,'1969-S243','1969-04-02 16:30:00','military missile','success',916,'AO-73','P-160 69 Spring 01','','','Test',204,'',180 from dual union all
select 18823,'1969-S244','1969-04-02 17:30:00','military missile','success',916,'AO-74','P-161 69 Spring 02','','','Test',204,'',180 from dual union all
select 18824,'1969-S245','1969-03-31 14:18:00','suborbital rocket','success',289,'','Rehnuma 1','','','Aeron',1419,'',138 from dual union all
select 18825,'1969-S246','1969-04-01 04:38:00','suborbital rocket','success',227,'ADD-VB-23','VUV auroral','','','Auroral',428,'',372 from dual union all
select 18826,'1969-S247','1969-04-01 10:24:00','suborbital rocket','success',1197,'SL724','LUX XRS 6 Survey','','','XR Astronomy',1839,'',192 from dual union all
select 18827,'1969-S248','1969-04-01 19:30:00','suborbital rocket','success',725,'','Particle flux 22','','','Plasma',1010,'',80 from dual union all
select 18828,'1969-S249','1969-04-02 03:20:00','military missile','success',999,'Yu22500-01T','8K67 UBP-26','','','OT',519,'',1000 from dual union all
select 18829,'1969-S25','1969-01-28 00:00:00','military missile','success',1449,'','8K84 UBP-100','','','OT',505,'',1000 from dual union all
select 1883,'1953-A450','1953-11-19 00:00:00','atmospheric rocket','success',836,'1282E','','','','Test',1895,'',10 from dual union all
select 18830,'1969-S250','1969-04-02 08:32:00','suborbital rocket','success',725,'','Particle flux 23','','','Plasma',1010,'',80 from dual union all
select 18831,'1969-S251','1969-04-03 00:35:00','suborbital rocket','success',1211,'SL502','MSSL SX-4/I-30/Bir-19','','','Ionos/Solar X',1839,'',273 from dual union all
select 18832,'1969-S252','1969-04-04 17:00:00','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 616',1000 from dual union all
select 18833,'1969-S253','1969-04-04 19:35:00','suborbital rocket','success',30,'CRL AG3.527','EUV Mon','','','Solar EUV',1869,'',248 from dual union all
select 18834,'1969-S254','1969-04-04 19:50:00','suborbital rocket','success',30,'CRL AT3.534','EUV Mon','','','Solar EUV',1869,'',220 from dual union all
select 18835,'1969-S256','1969-04-08 17:00:00','suborbital rocket','success',30,'NASA 04.282CS','AS&E Solar','','','Solar',1869,'',176 from dual union all
select 18836,'1969-S257','1969-04-09 22:02:01','military missile','failure',939,'C3XD-06','','','','Test',246,'',100 from dual union all
select 18837,'1969-S258','1969-04-09 09:33:00','suborbital rocket','success',740,'','Sun-Atm 69 Manometer','','','Aeron',636,'',170 from dual union all
select 18838,'1969-S259','1969-04-09 20:38:32','suborbital rocket','success',837,'Ferdinand 20','Ferdinand 20','','','Aeron/Ionos/Plasma',29,'',139 from dual union all
select 18839,'1969-S26','1969-01-31 00:00:00','military missile','success',1449,'','8K84 UBP-101','','','OT',532,'',1000 from dual union all
select 1884,'1953-A451','1953-11-19 00:00:00','atmospheric rocket','success',836,'1278E','','','','Test',1895,'',10 from dual union all
select 18840,'1969-S260','1969-04-11 08:55:00','military missile','success',799,'FTM-301','Mk 12 RV x3','R&D','','Test',1697,'',1300 from dual union all
select 18841,'1969-S261','1969-04-12 00:00:00','suborbital rocket','success',466,'E005','FU-181B test flight','','','Test',340,'',406 from dual union all
select 18842,'1969-S262','1969-04-12 23:19:00','suborbital rocket','success',837,'NASA 14.325GE','','','','Magnetospheric',428,'',158 from dual union all
select 18843,'1969-S263','1969-04-13 00:33:10','suborbital rocket','success',447,'D-255','FU-182 Ions/E field','','','Ionos',29,'',399 from dual union all
select 18844,'1969-S264','1969-04-13 14:04:00','suborbital rocket','success',837,'NASA 14.326GE','','','','Magnetospheric',428,'',155 from dual union all
select 18845,'1969-S265','1969-04-14 16:30:00','military missile','unknown',932,'A3E-645','','','','OT',421,'UK S23',1000 from dual union all
select 18846,'1969-S266','1969-04-14 15:36:00','suborbital rocket','success',32,'NASA 04.233GS','','','','Solar',1869,'',187 from dual union all
select 18847,'1969-S267','1969-04-14 23:15:00','suborbital rocket','success',837,'NASA 14.327GE','','','','Magnetospheric',428,'',161 from dual union all
select 18848,'1969-S268','1969-04-15 17:10:00','military missile','success',916,'AO-75','P-162 69 Spring 03','','','Test',204,'',180 from dual union all
select 18849,'1969-S269','1969-04-16 20:45:00','military missile','success',916,'AO-76','P-163 69 Spring 04','','','Test',204,'',180 from dual union all
select 1885,'1953-A452','1953-11-19 00:00:00','atmospheric rocket','success',836,'1262E','','','','Test',1895,'',10 from dual union all
select 18850,'1969-S27','1969-03-21 00:00:00','military missile','success',1449,'','8K84 UBP-102','','','OT',508,'',1000 from dual union all
select 18851,'1969-S270','1969-04-15 11:03:00','suborbital rocket','success',290,'C39/2','ESRO C39/2','','','Auroral',408,'',134 from dual union all
select 18852,'1969-S271','1969-04-16 09:35:00','military missile','failure',796,'2339','Mk 11 RV','OT GT19M','GLORY TRIP 19M','OT',1700,'',1300 from dual union all
select 18853,'1969-S272','1969-04-16 00:00:00','suborbital rocket','success',276,'','','','','Test',308,'',150 from dual union all
select 18854,'1969-S273','1969-04-16 16:20:00','suborbital rocket','success',32,'NASA 04.247DS','','','','Solar',1869,'',188 from dual union all
select 18855,'1969-S274','1969-04-16 21:00:00','suborbital rocket','success',837,'NASA 14.393UI','','','','Ionos',1821,'',210 from dual union all
select 18856,'1969-S275','1969-04-17 05:33:00','suborbital rocket','success',1207,'SL606','Cul UVS 11-XUVS1/MO-8','','','Solar UV,X',1839,'',202 from dual union all
select 18857,'1969-S276','1969-04-17 06:57:00','suborbital rocket','success',867,'NASA 18.80CE','','','','Magnetospheric',428,'',260 from dual union all
select 18858,'1969-S277','1969-04-17 16:45:00','suborbital rocket','success',30,'NASA 04.274DS','','','','Solar',1869,'',182 from dual union all
select 18859,'1969-S278','1969-04-17 21:48:04','suborbital rocket','success',447,'D-260','FU-190 E field/Ba','','','Ionos/Ba',29,'',329 from dual union all
select 1886,'1953-A453','1953-11-19 00:00:00','atmospheric rocket','success',836,'1276E','','','','Test',1895,'',10 from dual union all
select 18860,'1969-S279','1969-04-18 18:51:00','military missile','success',796,'2287','SST M-3','ST 3','SPEC TEST 3','Test',1711,'',1300 from dual union all
select 18861,'1969-S28','1969-04-24 00:00:00','military missile','success',1449,'','8K84 UBP-103','','','OT',508,'',1000 from dual union all
select 18862,'1969-S280','1969-04-18 00:50:00','military missile','success',1002,'V 22500-05L','8K67P LKI-5','RGCh 8F676 LKI','','Test',518,'',1000 from dual union all
select 18863,'1969-S281','1969-04-18 05:47:00','suborbital rocket','success',867,'CRL AT08.288','Mag field','','','Plasma',428,'',296 from dual union all
select 18864,'1969-S282','1969-04-18 05:48:00','suborbital rocket','success',848,'CRL AH07.297','Mag field','','','Plasma',428,'',160 from dual union all
select 18865,'1969-S283','1969-04-18 13:00:00','suborbital rocket','success',740,'','Sun-Atm 69 E-reg 2','','','Ionos',636,'',174 from dual union all
select 18866,'1969-S284','1969-04-18 15:20:00','suborbital rocket','success',740,'','Sun-Atmosphere-69 1','','','Solar EUV/Plasma',636,'',178 from dual union all
select 18867,'1969-S285','1969-04-19 16:32:00','suborbital rocket','success',740,'','Sun-Atmosphere-69 2','','','Solar EUV/Plasma',636,'',174 from dual union all
select 18868,'1969-S286','1969-04-20 13:04:00','suborbital rocket','success',740,'','Sun-Atm 69 E-reg 3','','','Ionos',636,'',175 from dual union all
select 18869,'1969-S287','1969-04-20 15:24:00','suborbital rocket','success',740,'','Sun-Atmosphere-69 3','','','Solar EUV/Plasma',636,'',171 from dual union all
select 1887,'1953-A454','1953-11-19 00:00:00','atmospheric rocket','success',841,'','F40 Hypersonic Test','','','Test',1821,'',10 from dual union all
select 18870,'1969-S288','1969-04-21 13:06:00','suborbital rocket','success',740,'','Sun-Atm 69 Mass spec','','','Aeron/Plasma',636,'',173 from dual union all
select 18871,'1969-S289','1969-04-21 15:26:00','suborbital rocket','success',740,'','Sun-Atmosphere-69 4','','','Solar EUV/Plasma',636,'',176 from dual union all
select 18872,'1969-S29','1969-04-29 00:00:00','military missile','success',1449,'','8K84 UBP-104','','','OT',542,'',1000 from dual union all
select 18873,'1969-S290','1969-04-22 20:15:01','military missile','success',799,'FTM-204','Mk 12 RV x 3','R&D','','Test',264,'',1300 from dual union all
select 18874,'1969-S291','1969-04-22 03:31:00','suborbital rocket','success',1207,'SL604','Cul UVS 12','','','Solar UV',1839,'',181 from dual union all
select 18875,'1969-S292','1969-04-23 23:30:00','military missile','success',795,'1090','Mk 11 RV','FOT GT35B','GLORY TRIP 35B','OT',1702,'',1300 from dual union all
select 18876,'1969-S293','1969-04-23 00:00:00','military missile','success',1103,'V2','S02V-2','','','Test',301,'',1000 from dual union all
select 18877,'1969-S294','1969-04-23 00:00:00','suborbital rocket','success',466,'E003','FU-187 test, A cone','','','Test',340,'',420 from dual union all
select 18878,'1969-S295','1969-04-23 13:10:00','suborbital rocket','success',740,'','Sun-Atm 69','','','Aeron',636,'',169 from dual union all
select 18879,'1969-S296','1969-04-23 15:30:00','suborbital rocket','success',740,'','Sun-Atmosphere-69 5','','','Solar EUV/Plasma',636,'',176 from dual union all
select 1888,'1953-A455','1953-11-20 00:00:00','atmospheric rocket','success',1445,'','E10 Hypersonic Test','','','Test',1821,'',20 from dual union all
select 18880,'1969-S297','1969-04-25 17:00:00','military missile','success',796,'2096','Mk 11 RV','OT GT5F','GLORY TRIP 5F','OT',1708,'',1300 from dual union all
select 18881,'1969-S298','1969-04-25 05:45:00','military missile','success',999,'Yu22500-16L','8K67 UBP-27','','','Test',519,'',1000 from dual union all
select 18882,'1969-S299','1969-04-25 00:00:00','military missile','success',1251,'800010','KT-7','','','Test',868,'',100 from dual union all
select 18883,'1969-S30','1969-05-23 00:00:00','military missile','success',1449,'','8K84 UBP-105','','','OT',508,'',1000 from dual union all
select 18884,'1969-S300','1969-04-26 00:00:00','suborbital rocket','success',448,'D-302','FU-188 Tech','','','Test',340,'',410 from dual union all
select 18885,'1969-S301','1969-04-26 17:00:00','suborbital rocket','success',837,'NASA 14.403IG','ISRO 40.04','TERLS-106','','XR Astron',1467,'',144 from dual union all
select 18886,'1969-S302','1969-04-27 03:22:00','suborbital rocket','success',30,'NASA 04.278UG','MIT-6','','','XR Astron',1869,'',158 from dual union all
select 18887,'1969-S303','1969-04-28 17:04:00','suborbital rocket','success',837,'NASA 14.404IG','ISRO 40.05','TERLS-107','','XR Astron',1467,'',148 from dual union all
select 18888,'1969-S305','1969-04-29 17:55:00','military missile','success',916,'AO-77','P-164 69 Spring 05','','','Test',204,'',180 from dual union all
select 18889,'1969-S306','1969-05-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 1889,'1953-A456','1953-11-20 00:00:00','atmospheric rocket','unknown',576,'355G','','','','Test',1895,'',10 from dual union all
select 18890,'1969-S307','1969-05-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 18891,'1969-S308','1969-05-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 18892,'1969-S309','1969-05-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 18893,'1969-S31','1969-05-31 00:00:00','military missile','success',1449,'','8K84 UBP-106','','','OT',508,'',1000 from dual union all
select 18894,'1969-S310','1969-05-13 15:30:00','military missile','success',916,'AO-79','P-166 69 Spring 07','','','Test',204,'',180 from dual union all
select 18895,'1969-S311','1969-05-13 16:40:00','military missile','success',916,'AO-80','P-167 69 Spring 08','','','Test',204,'',180 from dual union all
select 18896,'1969-S312','1969-05-01 00:00:00','suborbital rocket','success',1116,'Sandia 152-128','Dualhawk-9 Test','','','Test',855,'',528 from dual union all
select 18897,'1969-S313','1969-05-01 20:00:00','suborbital rocket','success',219,'AHF-4-14','Electron content','','','Ionos',428,'',760 from dual union all
select 18898,'1969-S314','1969-05-01 18:26:00','suborbital rocket','success',218,'NASA 12.15GT','','','','Test',1821,'',215 from dual union all
select 18899,'1969-S315','1969-05-02 08:42:00','military missile','success',796,'2505','Mk 11 RV','ST Mk 1A Penaids Test 4?','','Test',1707,'',1300 from dual union all
select 189,'1944-S171','1944-10-02 09:38:00','military missile','success',1470,'18778','Ma193','','','Test',661,'',90 from dual union all
select 1890,'1953-A457','1953-11-20 00:00:00','atmospheric rocket','unknown',576,'356G','','','','Test',1895,'',10 from dual union all
select 18900,'1969-S316','1969-05-07 14:00:00','suborbital rocket','success',219,'AHF-4-15','Electron content','','','Ionos',428,'',820 from dual union all
select 18901,'1969-S317','1969-05-08 00:05:00','suborbital rocket','success',839,'NASA 10.311GM','INTA NC-6903','GSFC Grenade 218 E','','Aeronomy',396,'',91 from dual union all
select 18902,'1969-S318','1969-05-08 03:11:00','suborbital rocket','success',839,'NASA 10.322GM','','GSFC Grenade 219 W','','Aeronomy',1821,'',129 from dual union all
select 18903,'1969-S319','1969-05-10 01:45:00','suborbital rocket','success',839,'NASA 10.312GM','INTA NC-6904','GSFC Grenade 220 E','','Aeronomy',396,'',90 from dual union all
select 18904,'1969-S32','1969-06-07 00:00:00','military missile','success',1449,'','8K84 UBP-107','','','OT',542,'',1000 from dual union all
select 18905,'1969-S320','1969-05-10 04:36:00','suborbital rocket','success',839,'NASA 10.321GM','','GSFC Grenade 221 W','','Aeronomy',1821,'',128 from dual union all
select 18906,'1969-S321','1969-05-12 18:52:00','suborbital rocket','success',837,'NASA 14.380GI','','','','Ionos',1821,'',200 from dual union all
select 18907,'1969-S322','1969-05-12 19:23:00','suborbital rocket','success',837,'NASA 14.362UM','UM Pitot 41','','','Aeronomy',1821,'',203 from dual union all
select 18908,'1969-S323','1969-05-13 01:04:00','suborbital rocket','success',848,'CRL AH07.678','Release','','','Aeron',394,'',192 from dual;
insert into LAUNCH
select 18909,'1969-S324','1969-05-13 01:23:00','suborbital rocket','success',848,'CRL AH07.674','Release QUEENA','','','Aeron',394,'',191 from dual union all
select 1891,'1953-A458','1953-11-20 00:00:00','military missile','success',572,'6','RV-A-8','','','Test',1863,'',30 from dual union all
select 18910,'1969-S325','1969-05-14 02:44:00','suborbital rocket','success',1207,'SL404','Cul XUVS 2/LUX-46','','','Solar UV,X',1839,'',178 from dual union all
select 18911,'1969-S326','1969-05-14 15:10:00','suborbital rocket','success',848,'CRL AG07.686','Niro TMA 7','','','Aeron',855,'',175 from dual union all
select 18912,'1969-S327','1969-05-15 01:05:00','suborbital rocket','success',848,'CRL AH07.679','Release','','','Aeron',394,'',203 from dual union all
select 18913,'1969-S328','1969-05-15 01:28:00','suborbital rocket','success',848,'CRL AH07.675','Release SOPHIA','','','Aeron',394,'',203 from dual union all
select 18914,'1969-S329','1969-05-15 23:10:00','suborbital rocket','success',35,'NASA 04.216NB','Rat centrifuge 4','','','Biology',1821,'',158 from dual union all
select 18915,'1969-S33','1969-06-10 00:00:00','military missile','success',1449,'','8K84 UBP-108','','','OT',536,'',1000 from dual union all
select 18916,'1969-S330','1969-05-16 03:53:00','suborbital rocket','success',624,'CRL AC19.289','B field','','','Plasma',428,'',789 from dual union all
select 18917,'1969-S331','1969-05-16 20:20:00','suborbital rocket','success',32,'NASA 04.232GS','GSFC EUV Spec','','','Solar',1869,'',202 from dual union all
select 18918,'1969-S332','1969-05-17 06:53:00','suborbital rocket','success',864,'Sandia 152-127','LRL BOX-8c','','','XR Astron',855,'',159 from dual union all
select 18919,'1969-S333','1969-05-18 00:00:00','military missile','success',1251,'800009','KT-8','','','Test',868,'',400 from dual union all
select 1892,'1953-A459','1953-11-25 00:00:00','atmospheric rocket','unknown',576,'6DJ','','','','Test',1895,'',10 from dual union all
select 18920,'1969-S334','1969-05-20 00:00:00','military missile','success',796,'(64-15525)','Mk 11 RV','OT GT9F','GLORY TRIP 9F','OT',1712,'',1300 from dual union all
select 18921,'1969-S335','1969-05-20 00:00:00','military missile','success',1420,'B-83','Mk 6 RV','FOT GT39T','','OT',1657,'',1300 from dual union all
select 18922,'1969-S336','1969-05-22 00:55:00','military missile','success',999,'','8K67 UBP-28','','','Test',519,'',1000 from dual union all
select 18923,'1969-S337','1969-05-22 05:46:00','suborbital rocket','success',848,'CRL AH07.687','Niro TMA 8','','','Aeron',855,'',176 from dual union all
select 18924,'1969-S338','1969-05-22 07:38:00','suborbital rocket','success',867,'CRL AH08.669','NO Jet payload 3','','','Aeron',855,'',176 from dual union all
select 18925,'1969-S339','1969-05-22 13:00:00','suborbital rocket','success',867,'CRL AH08.919-1','NO/Photometers','','','Aeron',855,'',156 from dual union all
select 18926,'1969-S34','1969-06-27 00:00:00','military missile','success',1449,'','8K84 UBP-109','','','OT',508,'',1000 from dual union all
select 18927,'1969-S340','1969-05-22 15:03:00','suborbital rocket','success',848,'CRL A07.918-1','Niro TMA 9','','','Aeron',855,'',170 from dual union all
select 18928,'1969-S341','1969-05-24 22:55:02','military missile','success',939,'C3XD-09','USN Mk 3 RV x 3','','','Test',246,'',500 from dual union all
select 18929,'1969-S342','1969-05-24 05:52:44','suborbital rocket','success',1349,'Sandia 152-126','LRL BOX-16 TT-9','','','XR Astron',855,'',398 from dual union all
select 1893,'1953-A46','1953-02-05 00:00:00','atmospheric rocket','success',836,'305P','Nike CET 5','','','Test',1895,'',10 from dual union all
select 18930,'1969-S343','1969-05-27 20:55:13','military missile','success',799,'FTM-205','Mk 12 RV x 3','R&D','','Test',266,'',1300 from dual union all
select 18931,'1969-S344','1969-05-27 00:00:00','suborbital rocket','failure',448,'D-303','FU-192 Plasma','','','Ionos',307,'',100 from dual union all
select 18932,'1969-S345','1969-05-27 02:46:00','suborbital rocket','success',1181,'CRL A30.900-1','PCA69 Q-1 p/l calib.','','','Aeron',428,'',97 from dual union all
select 18933,'1969-S346','1969-05-28 00:00:00','suborbital rocket','success',894,'No. 26','','','','Bio',310,'',90 from dual union all
select 18934,'1969-S347','1969-05-28 20:00:00','military missile','success',796,'2131','Mk 11 RV','OT GT7F','GLORY TRIP 7F','OT',1711,'',1300 from dual union all
select 18935,'1969-S348','1969-05-28 20:48:00','suborbital rocket','success',837,'NASA 14.420UA','Pandora H','','','Aeronomy',1895,'',155 from dual union all
select 18936,'1969-S349','1969-05-29 11:35:00','military missile','success',799,'FTM-302','Mk 12 RV x 2','R&D','','Test',1697,'',1300 from dual union all
select 18937,'1969-S35','1969-07-10 00:00:00','military missile','success',1449,'','8K84 UBP-110','','','OT',542,'',1000 from dual union all
select 18938,'1969-S350','1969-05-29 13:55:00','suborbital rocket','success',837,'STV 85','','','','Aeronomy',1895,'',200 from dual union all
select 18939,'1969-S351','1969-05-30 00:00:00','military missile','success',1103,'V3','S02V-3','','','Test',301,'',1000 from dual union all
select 1894,'1953-A460','1953-11-25 00:00:00','atmospheric rocket','success',836,'1212A','','','','Test',1895,'',10 from dual union all
select 18940,'1969-S352','1969-05-30 08:51:00','suborbital rocket','success',624,'CRL AF19.291','B field','','','Plasma',428,'',825 from dual union all
select 18941,'1969-S354','1969-06-01 00:00:00','suborbital rocket','success',1289,'','FSW test','','','Tech',819,'',100 from dual union all
select 18942,'1969-S355','1969-06-04 16:00:00','suborbital rocket','success',846,'STV 86','','','','Aeron',1864,'',100 from dual union all
select 18943,'1969-S356','1969-06-06 04:55:00','military missile','success',135,'B036','FATE','ABRES I-9 (USAF)','','Reentry Test',613,'',200 from dual union all
select 18944,'1969-S357','1969-06-05 19:27:53','suborbital rocket','success',290,'C35/2','ESRO C35/2','','','XR Astronomy',408,'',130 from dual union all
select 18945,'1969-S358','1969-06-06 08:55:00','military missile','success',135,'V132D','ABRES AJ-2 (ARPA)','ABRES AJ-2 (ARPA)','','Reentry Test',614,'',200 from dual union all
select 18946,'1969-S359','1969-06-09 16:00:00','suborbital rocket','success',837,'NASA 14.421UA','Pandora J','','','Aeronomy',1895,'',104 from dual union all
select 18947,'1969-S36','1969-07-18 00:00:00','military missile','success',1449,'','8K84 UBP-111','','','OT',542,'',1000 from dual union all
select 18948,'1969-S360','1969-06-11 13:15:00','military missile','success',796,'2059','Mk 11 RV','OT GT11F','GLORY TRIP 11F','OT',1710,'',1300 from dual union all
select 18949,'1969-S361','1969-06-11 18:05:00','suborbital rocket','success',740,'','Density/N2','','','Aeron/Plasma',636,'',167 from dual union all
select 1895,'1953-A461','1953-11-25 00:00:00','atmospheric rocket','success',836,'T','','','','Test',1895,'',10 from dual union all
select 18950,'1969-S362','1969-06-13 05:30:00','suborbital rocket','success',30,'KP3.28','','','','UV Astron',1869,'',150 from dual union all
select 18951,'1969-S363','1969-06-13 19:00:00','suborbital rocket','success',32,'NASA 04.242UA','Mariner-71 test','','','Aeronomy',1869,'',172 from dual union all
select 18952,'1969-S364','1969-06-14 21:52:00','suborbital rocket','success',30,'NASA 04.184UG-IG','','','','XR Astronomy',330,'',200 from dual union all
select 18953,'1969-S365','1969-06-15 03:57:01','military missile','success',939,'C3XD-10','USN Mk 3 RV x 10','','','Test',246,'',500 from dual union all
select 18954,'1969-S366','1969-06-15 14:05:00','suborbital rocket','success',219,'CRL A16.010-01','VLF Propagation','','','Ionos',330,'',582 from dual union all
select 18955,'1969-S367','1969-06-18 10:30:00','military missile','success',795,'1285','Mk 11 RV','FOT GT37B','GLORY TRIP 37B','OT',1702,'',1300 from dual union all
select 18956,'1969-S368','1969-06-18 01:56:00','suborbital rocket','success',1440,'CRL AD21.861','AFCRL TB2-2','','','Reentry Test',1821,'',299 from dual union all
select 18957,'1969-S369','1969-06-18 12:50:00','suborbital rocket','success',920,'P34H','Solar X/La/ne','','','Ionospheric/Solar X',1447,'',137 from dual union all
select 18958,'1969-S37','1969-07-28 00:00:00','military missile','success',1449,'','8K84 UBP-112','','','OT',542,'',1000 from dual union all
select 18959,'1969-S370','1969-06-19 15:30:00','suborbital rocket','success',920,'P35H','Solar X/La/ne','','','Ionos/Solar X',1447,'',136 from dual union all
select 1896,'1953-A462','1953-11-25 00:00:00','atmospheric rocket','success',836,'1238A','','','','Test',1895,'',10 from dual union all
select 18960,'1969-S371','1969-06-20 00:00:00','military missile','success',796,'2320','Mk 11B RV','ST Mk 1A Penaids Test 5?','','Test',1707,'',1300 from dual union all
select 18961,'1969-S372','1969-06-20 08:02:00','military missile','success',135,'V119D','RVTO-1B S/N 914','ABRES AR-2 (USAF)','','Reentry Test',613,'',200 from dual union all
select 18962,'1969-S373','1969-06-20 03:15:00','suborbital rocket','success',740,'','','','','Aeron',636,'',166 from dual union all
select 18963,'1969-S374','1969-06-20 05:45:00','suborbital rocket','success',32,'NASA 04.269UG','','','','UV Astronomy',1869,'',181 from dual union all
select 18964,'1969-S375','1969-06-21 00:00:00','military missile','success',1514,'','','','','Test',989,'',1000 from dual union all
select 18965,'1969-S376','1969-06-22 05:28:00','suborbital rocket','success',30,'NASA 04.183UG','','','','XR Astronomy',330,'',165 from dual union all
select 18966,'1969-S377','1969-06-24 00:00:00','military missile','success',1103,'V4','S02V-4','','','Test',301,'',1000 from dual union all
select 18967,'1969-S378','1969-06-24 19:00:00','suborbital rocket','success',30,'CRL AT3.535','EUV Mon','','','Solar EUV',1869,'',216 from dual union all
select 18968,'1969-S379','1969-06-24 19:02:00','suborbital rocket','success',740,'','','','','Aeron',636,'',170 from dual union all
select 18969,'1969-S38','1969-08-18 00:00:00','military missile','success',1449,'','8K84 UBP-113','','','OT',542,'',1000 from dual union all
select 1897,'1953-A463','1953-11-25 00:00:00','atmospheric rocket','success',836,'1286A','','','','Test',1895,'',10 from dual union all
select 18970,'1969-S380','1969-06-25 20:30:00','military missile','failure',799,'FTM-206','Mk 12 RV x 3','R&D','','Test',266,'',1300 from dual union all
select 18971,'1969-S381','1969-06-26 00:00:00','military missile','success',135,'V142D','RVTO-1B S/N 913','ABRES AR-1 (USAF)','','Reentry Test',612,'',200 from dual union all
select 18972,'1969-S382','1969-06-26 18:30:00','suborbital rocket','success',624,'NASA 8.53UI','','','','Ionos',330,'',795 from dual union all
select 18973,'1969-S383','1969-06-27 06:05:00','military missile','success',999,'Yu22502-14T','8K67 UBP-29','GCh 8F674','','OT',518,'',1000 from dual union all
select 18974,'1969-S384','1969-06-28 00:35:00','suborbital rocket','success',837,'NASA 14.387UA','','','','Aeronomy',1821,'',106 from dual union all
select 18975,'1969-S385','1969-07-01 04:36:00','military missile','success',795,'1282','Mk 11 RV','FOT GT38B','GLORY TRIP 38B','OT',1698,'',1300 from dual union all
select 18976,'1969-S386','1969-07-01 00:00:00','suborbital rocket','success',1289,'','FSW test','','','Tech',819,'',100 from dual union all
select 18977,'1969-S387','1969-07-01 12:39:00','suborbital rocket','success',837,'STV 87','','','','Aeronomy',1895,'',200 from dual union all
select 18978,'1969-S388','1969-07-01 18:15:00','suborbital rocket','success',740,'','N2 density','','','Aeron',636,'',141 from dual union all
select 18979,'1969-S389','1969-07-01 22:00:00','suborbital rocket','success',848,'CRL AH07.888','Mass spectrometer','','','Aeron',394,'',137 from dual union all
select 1898,'1953-A464','1953-11-25 00:00:00','atmospheric rocket','success',836,'T','','','','Test',1895,'',10 from dual union all
select 18980,'1969-S39','1969-08-28 00:00:00','military missile','success',1449,'','8K84 UBP-114','','','OT',540,'',1000 from dual union all
select 18981,'1969-S390','1969-07-02 09:00:00','military missile','success',795,'1279','Mk 11A RV','ST OLYMPIC TR. B-9','OLYMPIC TRIALS B-9','OT',1705,'',1300 from dual union all
select 18982,'1969-S391','1969-07-03 03:29:00','suborbital rocket','success',740,'','Density/N2','','','Aeron',636,'',165 from dual union all
select 18983,'1969-S392','1969-07-05 19:00:45','suborbital rocket','success',1199,'S38/1','ESRO S38/1','','','Ionospheric',1343,'',200 from dual union all
select 18984,'1969-S393','1969-07-06 19:50:29','suborbital rocket','success',1206,'S64/1','ESRO S64/1','','','Aeronomy',1343,'',275 from dual union all
select 18985,'1969-S394','1969-07-08 14:05:00','military missile','success',796,'2062','Mk 11 RV','OT GT14F','GLORY TRIP 14F','OT',1711,'',1300 from dual union all
select 18986,'1969-S395','1969-07-08 18:20:00','suborbital rocket','success',740,'','N2 density','','','Aeron',636,'',148 from dual union all
select 18987,'1969-S396','1969-07-09 11:50:00','military missile','success',939,'C3XD-11','','','','Test',246,'',500 from dual union all
select 18988,'1969-S397','1969-07-09 17:55:00','suborbital rocket','success',740,'','','','','Aeron',636,'',150 from dual union all
select 18989,'1969-S398','1969-07-10 17:30:00','suborbital rocket','success',846,'STV 88','','','','Aeron',1864,'',210 from dual union all
select 1899,'1953-A465','1953-11-27 00:00:00','military missile','success',771,'GM-11101','','','','Test',222,'',12 from dual union all
select 18990,'1969-S399','1969-07-10 18:30:00','suborbital rocket','success',846,'Ballistic round','','','','Aeron',1865,'',212 from dual union all
select 18991,'1969-S40','1969-09-27 00:00:00','military missile','success',1449,'','8K84 UBP-115','','','OT',542,'',1000 from dual union all
select 18992,'1969-S400','1969-07-10 08:07:00','suborbital rocket','success',740,'','Ne-Te 4','','','Ionos',636,'',162 from dual union all
select 18993,'1969-S401','1969-07-10 12:31:00','suborbital rocket','success',288,'C81','CNES FU-152','','','Aeronomy',311,'',109 from dual union all
select 18994,'1969-S402','1969-07-10 17:50:00','suborbital rocket','success',288,'C82','CNES FU-152','','','Aeronomy',311,'',114 from dual union all
select 18995,'1969-S403','1969-07-11 17:00:00','military missile','success',1000,'Yu22501-10T','8K67M UBP-4','GCh 8F675','','OT',519,'',1000 from dual union all
select 18996,'1969-S404','1969-07-11 17:55:00','suborbital rocket','success',740,'','','','','Aeron',636,'',148 from dual union all
select 18997,'1969-S405','1969-07-11 19:00:42','suborbital rocket','success',1199,'S38/2','ESRO S38/2','','','Ionospheric',1343,'',203 from dual union all
select 18998,'1969-S406','1969-07-13 19:41:00','suborbital rocket','success',1206,'S64/2','ESRO S64/2','','','Aeronomy',1343,'',280 from dual union all
select 18999,'1969-S407','1969-07-14 23:53:00','suborbital rocket','success',1197,'SL722','SHF-7/UCW-16','','','Ionospheric',1839,'',216 from dual;
insert into LAUNCH
select 19,'1943-A12','1943-07-09 00:00:00','military missile','failure',8,'34','','','','Test',675,'',0 from dual union all
select 190,'1944-S172','1944-10-02 10:19:00','military missile','success',1470,'18776','Ma190','','','Test',661,'',90 from dual union all
select 1900,'1953-A466','1953-12-02 00:00:00','atmospheric rocket','success',365,'','X-2 Model','','','Test',1821,'',10 from dual union all
select 19000,'1969-S408','1969-07-15 14:00:00','military missile','success',796,'2012','Mk 11 RV','OT GT13F','GLORY TRIP 13F','OT',1712,'',1300 from dual union all
select 19001,'1969-S409','1969-07-15 15:03:00','military missile','success',796,'(64-15533)','Mk 11 RV','OT GT12F','GLORY TRIP 12F','OT',1708,'',1300 from dual union all
select 19002,'1969-S41','1969-09-30 00:00:00','military missile','success',1449,'','8K84 UBP-116','','','OT',540,'',1000 from dual union all
select 19003,'1969-S410','1969-07-16 00:00:00','military missile','success',1251,'800013','KT-10','','','Test',868,'',100 from dual union all
select 19004,'1969-S411','1969-07-16 02:50:00','suborbital rocket','success',867,'NASA 18.32UG','','','','XR Astron',1821,'',209 from dual union all
select 19005,'1969-S412','1969-07-16 22:33:00','suborbital rocket','success',1206,'SL729','MSSL I-31/QUB AGP6','','','Ionospheric   UCW-17',1839,'',244 from dual union all
select 19006,'1969-S413','1969-07-23 19:50:00','military missile','success',795,'1140','Mk 11 RV','FOT GT41B','GLORY TRIP 41B','OT',1702,'',1300 from dual union all
select 19007,'1969-S414','1969-07-23 05:30:00','military missile','success',1450,'Yu10761-001','15A20 LKI-1','GCh 15F202 (3x15F204)','','Test',504,'',1000 from dual union all
select 19008,'1969-S415','1969-07-24 16:10:00','military missile','success',932,'A3E','','','','OT',421,'UK S26',1000 from dual union all
select 19009,'1969-S416','1969-07-24 23:55:00','suborbital rocket','success',740,'','','','','Aeron',636,'',145 from dual union all
select 1901,'1953-A467','1953-12-02 00:00:00','military missile','success',771,'GM-11099','','','','Test',222,'',12 from dual union all
select 19010,'1969-S417','1969-07-25 08:55:00','military missile','success',796,'2357','Mk 11B RV','ST Mk 1A Penaids test 6?','','Test',1703,'',1300 from dual union all
select 19011,'1969-S418','1969-07-25 00:00:00','military missile','success',1103,'C1','S02C-1','','','Test',301,'',1000 from dual union all
select 19012,'1969-S419','1969-07-25 01:19:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',134,'ZUB',166 from dual union all
select 19013,'1969-S42','1969-10-23 00:00:00','military missile','success',1449,'','8K84 UBP-117','Pal''ma-3','','OT',536,'',1000 from dual union all
select 19014,'1969-S420','1969-07-25 01:30:00','suborbital rocket','success',740,'MR-12','Thermosphere comp 6','','','Aeron',636,'',172 from dual union all
select 19015,'1969-S421','1969-07-25 02:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',89 from dual union all
select 19016,'1969-S422','1969-07-25 04:10:00','suborbital rocket','success',1197,'SL721','SHF-8/UCW-17','','','Ionospheric',1839,'',213 from dual union all
select 19017,'1969-S423','1969-07-25 22:03:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',170 from dual union all
select 19018,'1969-S424','1969-07-25 23:50:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',145 from dual union all
select 19019,'1969-S425','1969-07-27 21:47:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',134,'ZUB',168 from dual union all
select 1902,'1953-A468','1953-12-03 00:00:00','military missile','unknown',339,'1392R','','','','Test',1895,'',30 from dual union all
select 19020,'1969-S426','1969-07-27 22:37:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',134,'ZUB',164 from dual union all
select 19021,'1969-S427','1969-07-27 23:27:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',134,'ZUB',170 from dual union all
select 19022,'1969-S428','1969-07-28 00:17:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',134,'ZUB',167 from dual union all
select 19023,'1969-S429','1969-07-28 14:15:00','suborbital rocket','success',920,'P40H','SpE fine scale','','','Ionospheric',1447,'',121 from dual union all
select 19024,'1969-S43','1969-10-24 00:00:00','military missile','success',1449,'','8K84 UBP-118','Pal''ma-3','','OT',508,'',1000 from dual union all
select 19025,'1969-S430','1969-07-29 22:57:00','suborbital rocket','success',1206,'SL730','MSSL I-32/QUB AGP7','','','Ionospheric   UCW-18',1839,'',262 from dual union all
select 19026,'1969-S431','1969-07-30 16:55:00','suborbital rocket','success',846,'STV 89','','','','Aeron',1864,'',218 from dual union all
select 19027,'1969-S432','1969-07-30 17:45:00','suborbital rocket','success',846,'Ballistic round','','','','Aeron',1866,'',224 from dual union all
select 19028,'1969-S433','1969-07-30 20:07:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',171 from dual union all
select 19029,'1969-S434','1969-07-31 21:09:18','military missile','success',799,'FTM-207','Mk 12 RV x 3','R&D','','Test',264,'',1300 from dual union all
select 1903,'1953-A469','1953-12-07 00:00:00','atmospheric rocket','success',442,'','E15 Model','','','Test',1821,'',10 from dual union all
select 19030,'1969-S435','1969-08-02 11:37:00','suborbital rocket','success',920,'P12H','SpE fine scale','','','Ionospheric',1447,'',126 from dual union all
select 19031,'1969-S436','1969-08-03 03:25:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',134,'ZUB',164 from dual union all
select 19032,'1969-S437','1969-08-03 04:15:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',134,'ZUB',168 from dual union all
select 19033,'1969-S438','1969-08-03 05:05:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',134,'ZUB',165 from dual union all
select 19034,'1969-S439','1969-08-03 05:51:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',134,'ZUB',169 from dual union all
select 19035,'1969-S44','1969-10-29 00:00:00','military missile','success',1449,'','8K84 UBP-119','','','OT',542,'',1000 from dual union all
select 19036,'1969-S440','1969-08-03 07:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',134,'ZUB',172 from dual union all
select 19037,'1969-S441','1969-08-04 05:00:00','military missile','success',1000,'Yu45023-54T','8K67M UBP-5','GCh 8F675','','OT',519,'',1000 from dual union all
select 19038,'1969-S442','1969-08-06 03:36:00','suborbital rocket','success',850,'PCA69 BRL 1','BRL-1','','','Test',428,'',117 from dual union all
select 19039,'1969-S443','1969-08-07 00:00:00','suborbital rocket','success',1070,'S-210-1','','','','Plasma/Fields/XR Astron',839,'',101 from dual union all
select 1904,'1953-A47','1953-02-06 00:00:00','military missile','success',763,'N-25 GM-974','','','','Test',222,'',10 from dual union all
select 19040,'1969-S444','1969-08-07 03:36:00','suborbital rocket','success',848,'CRL AT07.394','[NOT IN FC]','','','Aeron',428,'',195 from dual union all
select 19041,'1969-S445','1969-08-07 03:36:00','suborbital rocket','success',848,'CRL AT07.179','Sphere','','','Aeron',428,'',196 from dual union all
select 19042,'1969-S446','1969-08-07 04:30:00','suborbital rocket','failure',848,'CRL A07.902-1','AH7.893 Mass spec','','','Aeron',428,'',84 from dual union all
select 19043,'1969-S447','1969-08-07 12:15:00','suborbital rocket','success',668,'K-9M-27','XR sources','','','Ionos/Plasma/XR Astron',839,'',340 from dual union all
select 19044,'1969-S448','1969-08-08 12:00:00','suborbital rocket','success',1070,'S-210-2','','','','XR Astron',839,'',108 from dual union all
select 19045,'1969-S449','1969-08-11 01:30:00','suborbital rocket','success',290,'C51/1','ESRO C51/1','','','Aeronomy',408,'',147 from dual union all
select 19046,'1969-S45','1969-12-04 00:00:00','military missile','success',1449,'','8K84 UBP-120','','','OT',542,'',1000 from dual union all
select 19047,'1969-S450','1969-08-11 15:30:00','military missile','success',932,'A3E','','','','OT',421,'UK S26',1000 from dual union all
select 19048,'1969-S451','1969-08-13 04:23:00','suborbital rocket','success',848,'CRL A07.902-2','Mass spectrometer','','','Aeron',428,'',114 from dual union all
select 19049,'1969-S452','1969-08-13 06:30:00','suborbital rocket','success',837,'NASA 14.422UA','Pandora K (Perseids)','','','Aeronomy/Meteorites',1895,'',143 from dual union all
select 1905,'1953-A470','1953-12-08 00:00:00','atmospheric rocket','success',836,'1256E','','','','Test',1895,'',10 from dual union all
select 19050,'1969-S453','1969-08-14 03:01:00','suborbital rocket','success',290,'C58/1','ESRO C58/1','','','Meteorites',408,'',112 from dual union all
select 19051,'1969-S454','1969-08-15 21:55:00','suborbital rocket','success',30,'CRL A03.903-1','EUV Mon','','','Solar EUV',1869,'',211 from dual union all
select 19052,'1969-S455','1969-08-17 06:00:00','test rocket','success',817,'M-3D-1','','','','Test',835,'',160 from dual union all
select 19053,'1969-S456','1969-08-19 15:30:00','military missile','success',932,'A3E','','','','OT',421,'SSBN 623',1000 from dual union all
select 19054,'1969-S457','1969-08-20 12:05:00','military missile','success',162,'112F','WT1A-S6/WT1A-S5','ABRES RVTO-1A-5','','Reentry Test',1680,'',1400 from dual union all
select 19055,'1969-S458','1969-08-22 04:20:00','military missile','success',796,'2108','Mk 11B RV','OT GT17F','GLORY TRIP 17F','OT',1710,'',1300 from dual union all
select 19056,'1969-S459','1969-08-21 01:57:00','suborbital rocket','success',1207,'SL605','LUX-47 HRCS/PHC','','','Solar X',1839,'',197 from dual union all
select 19057,'1969-S46','1969-08-15 05:00:00','military missile','success',1450,'Yu10761-002','15A20 LKI-2','GCh 15F202','','Test',504,'',1000 from dual union all
select 19058,'1969-S460','1969-08-21 14:09:00','suborbital rocket','success',867,'NASA 18.78GA','Planetary Mass Spec','','','Aeronomy',1821,'',227 from dual union all
select 19059,'1969-S461','1969-08-21 15:09:00','suborbital rocket','success',867,'NASA 18.102GA','UM Thermosphere 13','','','Aeronomy',1821,'',318 from dual union all
select 1906,'1953-A471','1953-12-08 00:00:00','atmospheric rocket','success',836,'1266E','','','','Test',1895,'',10 from dual union all
select 19060,'1969-S463','1969-08-22 00:00:00','suborbital rocket','success',466,'E004','FU-187 test, B cone','','','Test',340,'',242 from dual union all
select 19061,'1969-S464','1969-08-22 00:30:00','military missile','success',939,'C3XD-12','','','','Test',246,'',500 from dual union all
select 19062,'1969-S465','1969-08-22 10:36:00','suborbital rocket','success',837,'NASA 14.423UG','Pandora L (Backgr.)','','','Meteorites',1895,'',158 from dual union all
select 19063,'1969-S466','1969-08-24 00:00:00','suborbital rocket','success',1072,'S-300-2','','','','Test',839,'',160 from dual union all
select 19064,'1969-S467','1969-08-24 08:04:00','suborbital rocket','success',668,'K-9M-26','Radio noise','','','Ionos/Fields',839,'',341 from dual union all
select 19065,'1969-S468','1969-08-27 02:35:00','military missile','success',796,'2065','Mk 11B RV','OT GT15F','GLORY TRIP 15F','OT',1708,'',1300 from dual union all
select 19066,'1969-S469','1969-08-28 00:00:00','military missile','success',795,'1358','Mk 11 RV','FOT GT42B','GLORY TRIP 42B','OT',1698,'',1300 from dual union all
select 19067,'1969-S47','1969-09-12 03:04:00','military missile','success',1450,'Yu10761-003','15A20 LKI-3','GCh 15F211?','','Test',504,'',1000 from dual union all
select 19068,'1969-S470','1969-08-30 00:00:00','suborbital rocket','success',894,'No. 31','','','','XR Astron',310,'',90 from dual union all
select 19069,'1969-S471','1969-08-30 17:30:00','suborbital rocket','success',894,'No. 27','','','','Bio',310,'',15 from dual union all
select 1907,'1953-A472','1953-12-09 00:00:00','atmospheric rocket','unknown',576,'214M','','','','Test',1895,'',10 from dual union all
select 19070,'1969-S472','1969-09-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 19071,'1969-S473','1969-09-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 19072,'1969-S474','1969-09-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 19073,'1969-S475','1969-09-16 00:00:00','military missile','success',917,'AO-83','P-170 69 Fall 03','','','Test',204,'',180 from dual union all
select 19074,'1969-S476','1969-09-16 00:00:00','military missile','success',917,'AO-84','P-171 69 Fall 04','','','Test',204,'',180 from dual union all
select 19075,'1969-S477','1969-09-30 20:17:00','military missile','success',917,'AO-85','P-172 69 Fall 05','','','Test',204,'',180 from dual union all
select 19076,'1969-S478','1969-09-30 23:56:00','military missile','success',917,'AO-86','P-173 69 Fall 06','','','Test',204,'',180 from dual union all
select 19077,'1969-S479','1969-09-03 02:20:00','military missile','success',796,'2145','Mk 11 RV','OT GT20F','GLORY TRIP 20F','OT',1712,'',1300 from dual union all
select 19078,'1969-S48','1969-09-29 04:50:00','military missile','success',1450,'Yu10761-004','15A20 LKI-4','GCh 15202','','Test',504,'',1000 from dual union all
select 19079,'1969-S480','1969-09-03 02:10:00','test rocket','success',701,'L-4T-1','','','','Test',834,'',400 from dual union all
select 1908,'1953-A473','1953-12-09 00:00:00','atmospheric rocket','success',836,'T','','','','Test',1895,'',10 from dual union all
select 19080,'1969-S481','1969-09-06 01:55:00','suborbital rocket','unknown',850,'','Pre-Secede 1','','','Aeron?',1895,'',100 from dual union all
select 19081,'1969-S482','1969-09-05 19:55:00','military missile','success',916,'AO-81','P-168 69 Fall 01','','','Test',204,'',180 from dual union all
select 19082,'1969-S483','1969-09-05 20:30:00','military missile','success',916,'AO-82','P-169 69 Fall 02','','','Test',204,'',180 from dual union all
select 19083,'1969-S484','1969-09-07 01:52:00','suborbital rocket','unknown',850,'','Pre-Secede 2','','','Aeron?',1895,'',100 from dual union all
select 19084,'1969-S485','1969-09-06 11:35:00','suborbital rocket','success',655,'K-10-5','Zodi. light','','','Ionos/Astron',839,'',247 from dual union all
select 19085,'1969-S486','1969-09-06 14:10:00','suborbital rocket','success',219,'CRL A16.010-02','VLF Propagation','','','Ionos',330,'',623 from dual union all
select 19086,'1969-S487','1969-10-08 23:45:00','military missile','success',1002,'Yu51024-54T','8K67P UBP-2','GCh 8F675','','OT',519,'',1000 from dual union all
select 19087,'1969-S488','1969-09-11 01:05:00','military missile','success',795,'1137','Mk 11 RV','FOT GT36B','GLORY TRIP 36B','OT',1702,'',1300 from dual union all
select 19088,'1969-S489','1969-09-10 06:15:00','military missile','success',135,'V147D','RVTO-2B-2','ABRES AS-2 (USAF)','','Reentry Test',613,'',200 from dual;
insert into LAUNCH
select 19089,'1969-S49','1969-10-11 00:00:00','military missile','success',1450,'Yu10761-005','15A20 LKI-5','GCh 15202','','Test',504,'',1000 from dual union all
select 1909,'1953-A474','1953-12-10 00:00:00','military missile','unknown',339,'1393R','','','','Test',1895,'',30 from dual union all
select 19090,'1969-S490','1969-09-10 04:35:00','suborbital rocket','success',30,'NASA 04.312UG','','','','IR Astronomy',1869,'',157 from dual union all
select 19091,'1969-S491','1969-09-10 06:30:00','suborbital rocket','failure',694,'LS-C-2','','','','Test',1476,'',10 from dual union all
select 19092,'1969-S492','1969-09-10 20:46:00','suborbital rocket','success',837,'NASA 14.395UI','','','','Ionos',1821,'',205 from dual union all
select 19093,'1969-S493','1969-09-11 10:21:00','suborbital rocket','success',30,'NASA 04.193US','Solar Studies','','','Solar',1869,'',200 from dual union all
select 19094,'1969-S494','1969-09-11 01:07:00','suborbital rocket','success',215,'AAD-2-122','','','','Aeron/Ionos/Meteorites',428,'',137 from dual union all
select 19095,'1969-S495','1969-09-11 23:44:14','suborbital rocket','success',864,'Sandia 152-109','LRL RCS-3 (Daisy III)','','','Aeron',1488,'',165 from dual union all
select 19096,'1969-S496','1969-09-12 01:02:00','suborbital rocket','success',215,'AAD-2-123','','','','Aeron/Ionos/Meteorites',428,'',138 from dual union all
select 19097,'1969-S497','1969-09-12 04:06:00','suborbital rocket','success',837,'NASA 14.394UI','','','','Ionos',1821,'',189 from dual union all
select 19098,'1969-S498','1969-09-12 04:06:00','suborbital rocket','success',1058,'No 2','','','','Test',310,'',250 from dual union all
select 19099,'1969-S499','1969-09-13 00:00:00','military missile','success',799,'FTM-303','Mk 12 RV x 3','R&D','','Test',1697,'',1300 from dual union all
select 191,'1944-S173','1944-10-02 18:32:00','military missile','success',1470,'18796','Ma203','','','Test',661,'',90 from dual union all
select 1910,'1953-A475','1953-12-10 00:00:00','atmospheric rocket','unknown',576,'3566','','','','Test',1895,'',10 from dual union all
select 19100,'1969-S50','1969-10-29 01:03:00','military missile','success',1450,'Yu10761-006','15A20 LKI-6','GCh 15B20 (3x15F201?)','','Test',505,'',1000 from dual union all
select 19101,'1969-S500','1969-09-14 01:42:00','suborbital rocket','success',850,'','Pre-Secede 3','','','Aeron?',1895,'',100 from dual union all
select 19102,'1969-S501','1969-09-15 01:40:00','suborbital rocket','success',613,'JCR-1','','','','Test',1476,'',140 from dual union all
select 19103,'1969-S502','1969-09-15 13:34:00','suborbital rocket','success',920,'P58H','D-region','','','Ionospheric',1447,'',140 from dual union all
select 19104,'1969-S503','1969-09-16 00:00:00','military missile','success',168,'100F','RMV-340C/RMV-430C/BSE','ABRES RMP-B-12/KX-115','Mk 11 Low Angle RV','Reentry Test',1682,'',1400 from dual union all
select 19105,'1969-S504','1969-09-16 06:40:00','suborbital rocket','success',613,'JCR-2','','','','Test',1476,'',140 from dual union all
select 19106,'1969-S505','1969-09-17 16:37:00','military missile','failure',939,'C3XD-13','','','','Test',247,'',500 from dual union all
select 19107,'1969-S506','1969-09-17 18:45:00','suborbital rocket','success',32,'NASA 04.166DS','','','','Solar',1869,'',218 from dual union all
select 19108,'1969-S507','1969-09-18 16:00:00','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 619',1000 from dual union all
select 19109,'1969-S508','1969-09-19 03:15:00','suborbital rocket','success',219,'NASA 19.05NE','','','','Plasma',330,'',856 from dual union all
select 1911,'1953-A476','1953-12-11 00:00:00','military missile','success',771,'GM-11080','','','','Test',222,'',12 from dual union all
select 19110,'1969-S509','1969-09-20 08:30:00','military missile','success',795,'1276','Mk 11 RV','ST OLYMPIC TR. B-10','OLYMPIC TRIALS B-10','OT',1705,'',1300 from dual union all
select 19111,'1969-S51','1969-11-01 03:42:00','military missile','success',1450,'Yu10761-007','15A20 LKI-7','GCh 15F202','','Test',504,'',1000 from dual union all
select 19112,'1969-S510','1969-09-20 01:55:00','suborbital rocket','success',826,'NAL-16H F2','','','','Test',1476,'',100 from dual union all
select 19113,'1969-S511','1969-09-20 20:00:00','suborbital rocket','failure',446,'D-39','ISRO 26.01','TERLS-129','','Ionos',1467,'',400 from dual union all
select 19114,'1969-S512','1969-09-21 08:05:00','suborbital rocket','success',30,'NRL NB3.202','Stellar Lyman Alpha','','','UV Astron',1869,'',184 from dual union all
select 19115,'1969-S513','1969-09-23 00:00:00','military missile','failure',795,'STF-1','SSTTP 1','SSTTP 1','MR CLEAN','Target',1701,'',100 from dual union all
select 19116,'1969-S514','1969-09-23 16:36:00','suborbital rocket','success',30,'NASA 04.170US','','','','Solar',1869,'',174 from dual union all
select 19117,'1969-S515','1969-09-23 20:10:26','military missile','success',799,'FTM-208','Mk 12 RV x 3','R&D','','Test',264,'',1300 from dual union all
select 19118,'1969-S516','1969-09-24 00:00:00','military missile','success',796,'2129','Mk 11 RV','OT GT21F','GLORY TRIP 21F','OT',1711,'',1300 from dual union all
select 19119,'1969-S517','1969-09-25 01:11:00','suborbital rocket','success',846,'STV 90','','','','Aeron',1866,'',196 from dual union all
select 1912,'1953-A477','1953-12-15 00:00:00','military missile','unknown',339,'12EN','EU-12','','','Test',1895,'',30 from dual union all
select 19120,'1969-S518','1969-09-24 17:45:00','suborbital rocket','success',30,'NASA 04.294US','','','','Solar',1869,'',200 from dual union all
select 19121,'1969-S519','1969-09-25 02:55:00','military missile','success',1002,'','8K67P UBP-1','','','Test',519,'',1000 from dual union all
select 19122,'1969-S520','1969-09-25 00:00:00','military missile','success',135,'V130D','RVTO-2B-1','ABRES AS-1 (USAF)','','Reentry Test',613,'',200 from dual union all
select 19123,'1969-S521','1969-09-26 00:00:00','suborbital rocket','failure',656,'K-10C-2','TVC test','','','Test',839,'',10 from dual union all
select 19124,'1969-S523','1969-10-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 19125,'1969-S524','1969-10-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',135,'ZUB',150 from dual union all
select 19126,'1969-S525','1969-09-30 14:30:00','military missile','unknown',932,'A3E','','','','OT',421,'SSBN 623',1000 from dual union all
select 19127,'1969-S526','1969-09-30 22:40:55','suborbital rocket','success',920,'P46H','Night D/E-region','','','Ionospheric',1447,'',100 from dual union all
select 19128,'1969-S527','1969-10-01 00:00:00','military missile','success',796,'2510','Mk 11 RV','ST','','Test',1707,'',1300 from dual union all
select 19129,'1969-S528','1969-10-01 01:20:00','suborbital rocket','success',920,'P52H','Night D/E-region','','','Ionospheric',1447,'',140 from dual union all
select 1913,'1953-A478','1953-12-15 00:00:00','military missile','success',771,'GM-11105','','','','Test',222,'',12 from dual union all
select 19130,'1969-S529','1969-10-01 04:19:00','suborbital rocket','success',920,'P51H','Night D/E-region','','','Ionospheric',1447,'',140 from dual union all
select 19131,'1969-S53','1969-01-07 00:35:00','suborbital rocket','success',848,'CRL AG07.626','AFGL TMA release','','','Aeron',394,'',154 from dual union all
select 19132,'1969-S530','1969-10-03 01:00:00','military missile','success',795,'1173','Mk 11 RV','FOT GT44B','GLORY TRIP 44B','OT',1698,'',1300 from dual union all
select 19133,'1969-S531','1969-10-03 02:24:00','suborbital rocket','success',32,'NASA 04.279UG','MIT-7 GC Survey III','','','XR Astron',1869,'',158 from dual union all
select 19134,'1969-S532','1969-10-04 05:15:00','suborbital rocket','success',867,'NASA 18.71UG','','','','XR Astron',1821,'',235 from dual union all
select 19135,'1969-S534','1969-10-05 22:28:10','suborbital rocket','success',1204,'S29/1','ESRO S29/1 RSRS-10','','','Auroral',416,'',234 from dual union all
select 19136,'1969-S535','1969-10-10 06:50:00','suborbital rocket','success',920,'P20H','SpE','','','Ionospheric',1447,'',130 from dual union all
select 19137,'1969-S536','1969-10-10 20:01:31','suborbital rocket','success',290,'C52/2','ESRO C52/2','','','Auroral',408,'',135 from dual union all
select 19138,'1969-S537','1969-10-11 02:00:00','suborbital rocket','success',848,'CRL AH07.890','[AH7.893 in FC]','','','Aeron',428,'',118 from dual union all
select 19139,'1969-S538','1969-10-11 11:05:00','suborbital rocket','success',740,'','','','','Ionos',135,'ZUB',158 from dual union all
select 1914,'1953-A479','1953-12-15 00:00:00','military missile','success',771,'GM-11106','','','','Test',222,'',12 from dual union all
select 19140,'1969-S539','1969-10-11 19:36:00','suborbital rocket','success',920,'P38H','E fields','','','Ionospheric',1447,'',140 from dual union all
select 19141,'1969-S54','1969-01-07 23:38:00','suborbital rocket','success',848,'CRL AH07.672','AFGL Ba release','','','Aeron',394,'',215 from dual union all
select 19142,'1969-S540','1969-10-13 22:05:00','military missile','success',796,'2054','Mk 11 RV','OT GT22F','GLORY TRIP 22F','OT',1710,'',1300 from dual union all
select 19143,'1969-S541','1969-10-13 00:00:00','military missile','success',1103,'C2','S02C-2','','','Test',301,'',1000 from dual union all
select 19144,'1969-S542','1969-10-14 04:00:00','suborbital rocket','success',30,'NRL NC3.136','','','','Aeron',1869,'',217 from dual union all
select 19145,'1969-S543','1969-10-14 08:22:00','suborbital rocket','success',740,'','','','','Aeron',636,'',120 from dual union all
select 19146,'1969-S544','1969-10-14 18:43:00','suborbital rocket','success',920,'P14H','D/E regions','','','Ionospheric',1447,'',140 from dual union all
select 19147,'1969-S545','1969-10-15 11:16:00','military missile','success',799,'FTM-305','Mk 12 RV x 3','R&D','','Test',1697,'',1300 from dual union all
select 19148,'1969-S546','1969-10-15 15:30:00','military missile','unknown',932,'A3E','','','','OT',421,'SSBN 624',1000 from dual union all
select 19149,'1969-S547','1969-10-15 19:16:00','suborbital rocket','success',1206,'SL861','UCL GTBa-1 TMA/Ba','','','Aeronomy/Ionos',1839,'',318 from dual union all
select 1915,'1953-A48','1953-02-09 00:00:00','atmospheric rocket','success',836,'1249A-191','Nike 1249 R&D 171R','','','Test',1895,'',10 from dual union all
select 19150,'1969-S548','1969-10-15 19:10:00','military missile','unknown',932,'A3E','','','','OT',421,'SSBN 624',1000 from dual union all
select 19151,'1969-S549','1969-10-15 22:43:00','sounding rocket','success',129,'NASA 16.06GR','RAE-B Test','','','Radio Astron',1811,'',2575 from dual union all
select 19152,'1969-S55','1969-01-08 00:00:00','suborbital rocket','success',1071,'SO-250-B','','','','Test',839,'',100 from dual union all
select 19153,'1969-S550','1969-10-16 10:30:00','suborbital rocket','success',30,'NASA 04.256GG','Zeta Ori study','','','UV Astronomy',1869,'',205 from dual union all
select 19154,'1969-S551','1969-10-17 07:05:00','suborbital rocket','success',920,'P16H','SpE','','','Ionospheric',1447,'',140 from dual union all
select 19155,'1969-S552','1969-10-17 09:45:00','suborbital rocket','success',1206,'SL862','UCL GTBa-2','','','Aeronomy/Ionos',1839,'',318 from dual union all
select 19156,'1969-S553','1969-10-17 22:09:00','suborbital rocket','success',740,'','','','','Aeron',636,'',166 from dual union all
select 19157,'1969-S554','1969-10-17 22:26:40','suborbital rocket','success',1204,'S29/2','ESRO S29/2 RSRS-11','','','Auroral',416,'',230 from dual union all
select 19158,'1969-S555','1969-10-18 11:07:00','suborbital rocket','success',740,'','','','','Ionos',135,'ZUB',165 from dual union all
select 19159,'1969-S556','1969-10-18 17:47:00','suborbital rocket','success',740,'','','','','Ionos',135,'ZUB',160 from dual union all
select 1916,'1953-A480','1953-12-15 00:00:00','military missile','success',771,'GM-11110','','','','Test',222,'',12 from dual union all
select 19160,'1969-S557','1969-10-20 00:00:00','military missile','unknown',929,'A2','OT','','','OT',421,'SSBN 609',1000 from dual union all
select 19161,'1969-S558','1969-10-20 00:00:00','military missile','unknown',929,'A2','OT','','','OT',421,'SSBN 609',1000 from dual union all
select 19162,'1969-S559','1969-10-20 00:00:00','military missile','unknown',929,'A2','OT','','','OT',421,'SSBN 609',1000 from dual union all
select 19163,'1969-S56','1969-01-08 02:10:00','suborbital rocket','success',668,'K-9M-25','Gyro-plasma probe','','','Ionos/CR/Fields',839,'',343 from dual union all
select 19164,'1969-S560','1969-10-20 00:00:00','military missile','unknown',929,'A2','OT','','','OT',421,'SSBN 609',1000 from dual union all
select 19165,'1969-S561','1969-10-20 00:00:00','military missile','unknown',929,'A2','OT','','','OT',421,'SSBN 609',1000 from dual union all
select 19166,'1969-S562','1969-10-20 00:00:00','military missile','unknown',929,'A2','OT','','','OT',421,'SSBN 609',1000 from dual union all
select 19167,'1969-S563','1969-10-20 00:00:00','military missile','unknown',929,'A2','OT','','','OT',421,'SSBN 609',1000 from dual union all
select 19168,'1969-S564','1969-10-20 07:28:00','suborbital rocket','success',1207,'SL586','RAE Test 41','','','Opt/UV Astron',1839,'',223 from dual union all
select 19169,'1969-S565','1969-10-21 00:00:00','military missile','success',795,'1134','Mk 11 RV','FOT GT45B','GLORY TRIP 45B','OT',1702,'',1300 from dual union all
select 1917,'1953-A481','1953-12-15 00:00:00','military missile','success',771,'GM-12418','','','','Test',222,'',12 from dual union all
select 19170,'1969-S566','1969-10-21 12:32:00','suborbital rocket','success',740,'','','','','Aeron',636,'',164 from dual union all
select 19171,'1969-S567','1969-10-22 17:12:00','suborbital rocket','success',846,'STV 91','','','','Aeron',1866,'',196 from dual union all
select 19172,'1969-S568','1969-10-22 06:10:00','suborbital rocket','success',1206,'SL821','MSSL XR1/MSSL SX5','','','XR Astronomy',1839,'',216 from dual union all
select 19173,'1969-S569','1969-10-22 08:31:00','suborbital rocket','success',740,'','','','','Aeron',636,'',116 from dual union all
select 19174,'1969-S57','1969-01-09 07:40:00','suborbital rocket','success',665,'K-8-15','E field antenna','','','Ionos/Fields',839,'',188 from dual union all
select 19175,'1969-S570','1969-10-23 14:06:00','military missile','success',1002,'Yu22502-03T','8K67P UBP-3','GCh 8F675 Pal''ma-3','','OT',518,'',1000 from dual union all
select 19176,'1969-S571','1969-10-23 00:00:00','suborbital rocket','success',276,'','','','','Test',310,'',150 from dual union all
select 19177,'1969-S572','1969-10-23 05:00:00','suborbital rocket','success',30,'CRL A03.910-2','IR airglow profile','','','Auroral IR',428,'',190 from dual union all
select 19178,'1969-S573','1969-10-24 12:10:00','military missile','success',1002,'Ya22504-04T','8K67P UBP-4','GCh 8F675 Pal''ma-3','','OT',515,'',1000 from dual;
insert into LAUNCH
select 19179,'1969-S574','1969-10-24 04:33:00','suborbital rocket','success',740,'','Particles','','','Aeron/Plasma/Solar UV,X',636,'',160 from dual union all
select 1918,'1953-A482','1953-12-15 00:00:00','atmospheric rocket','success',836,'1198A','','','','Test',1895,'',10 from dual union all
select 19180,'1969-S575','1969-10-24 05:44:00','suborbital rocket','success',1199,'S68/1','ESRO S68/1','','','XR Astronomy/Tech',1343,'',184 from dual union all
select 19181,'1969-S576','1969-10-25 03:00:00','military missile','success',1002,'','8K67P UBP-5','','','Test',519,'',1000 from dual union all
select 19182,'1969-S577','1969-10-25 06:00:00','suborbital rocket','success',218,'AKF-3B-50','Soft X-ray survey','','','XR Astron',1315,'',192 from dual union all
select 19183,'1969-S578','1969-10-26 06:40:00','suborbital rocket','success',218,'AAF-3B-51','Soft X-ray survey','','','XR Astron',1315,'',197 from dual union all
select 19184,'1969-S579','1969-10-29 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 601',1000 from dual union all
select 19185,'1969-S58','1969-01-10 02:43:00','suborbital rocket','success',30,'NASA 04.206UA','JHU UVT F3 (Venus)','','','UV Astron',1869,'',166 from dual union all
select 19186,'1969-S580','1969-10-29 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 601',1000 from dual union all
select 19187,'1969-S581','1969-10-29 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 601',1000 from dual union all
select 19188,'1969-S582','1969-10-29 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 601',1000 from dual union all
select 19189,'1969-S583','1969-10-29 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 601',1000 from dual union all
select 1919,'1953-A483','1953-12-16 00:00:00','atmospheric rocket','success',365,'','X-3 Model','','','Test',1821,'',10 from dual union all
select 19190,'1969-S584','1969-10-29 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 601',1000 from dual union all
select 19191,'1969-S585','1969-10-29 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 601',1000 from dual union all
select 19192,'1969-S586','1969-10-30 16:30:00','military missile','unknown',932,'A3E','','','','OT',421,'SSBN 622',1000 from dual union all
select 19193,'1969-S587','1969-10-31 00:00:00','military missile','success',799,'FTM-304','Mk 12 RV x 3','R&D','','Test',1697,'',1300 from dual union all
select 19194,'1969-S588','1969-11-01 10:59:27','test rocket','success',1446,'Yu22500-04L','','GVM US-A','','Test',591,'',200 from dual union all
select 19195,'1969-S589','1969-11-02 20:20:00','suborbital rocket','success',226,'CRL AJ17.602','PCA69 U-2','','','Plasma',428,'',120 from dual union all
select 19196,'1969-S59','1969-01-11 07:30:00','suborbital rocket','success',839,'NASA 10.236GM','GSFC Ozone 4','','','Aeronomy',1039,'',100 from dual union all
select 19197,'1969-S590','1969-11-02 21:10:00','suborbital rocket','success',850,'','PCA69 BRL 4','','','Ionos',428,'',119 from dual union all
select 19198,'1969-S591','1969-11-02 21:11:00','suborbital rocket','success',1181,'CRL A30.900-2','PCA69 Q-1 (D-region)','','','Aeron',428,'',96 from dual union all
select 19199,'1969-S592','1969-11-03 06:05:00','suborbital rocket','success',226,'CRL AJ17.758','PCA69 U-1','','','Plasma',428,'',126 from dual union all
select 192,'1944-S174','1944-10-02 19:08:00','military missile','success',1470,'18795','Ma202','','','Test',661,'',90 from dual union all
select 1920,'1953-A484','1953-12-16 00:00:00','military missile','success',771,'GM-12419','','','','Test',222,'',12 from dual union all
select 19200,'1969-S593','1969-11-03 06:29:00','suborbital rocket','success',848,'CRL A07.902-3','PCA69 N-2','','','Aeron',428,'',117 from dual union all
select 19201,'1969-S594','1969-11-03 06:42:00','suborbital rocket','success',848,'CRL AT07.396','PCA69 C-1','','','Aeron',428,'',210 from dual union all
select 19202,'1969-S595','1969-11-03 07:30:00','suborbital rocket','success',848,'CRL AG07.882','PCA69 N-1','','','Aeron',428,'',138 from dual union all
select 19203,'1969-S596','1969-11-03 10:28:00','suborbital rocket','success',218,'AAF-3B-48','','','','Auroral/CR',1315,'',200 from dual union all
select 19204,'1969-S597','1969-11-03 12:32:00','suborbital rocket','success',848,'CRL AT07.397','PCA69 C-2','','','Aeron',428,'',167 from dual union all
select 19205,'1969-S598','1969-11-03 12:57:00','suborbital rocket','success',850,'','PCA69 BRL 2','','','Ionos/Plasma',428,'',116 from dual union all
select 19206,'1969-S599','1969-11-03 13:30:00','suborbital rocket','success',850,'','PCA69 BRL 3','','','Ionos',428,'',90 from dual union all
select 19207,'1969-S60','1969-01-11 10:15:00','suborbital rocket','success',839,'NASA 10.256GM','GSFC Grenade 200 PB','','','Aeronomy',1039,'',100 from dual union all
select 19208,'1969-S600','1969-11-03 13:52:00','suborbital rocket','success',226,'CRL AJ17.616','PCA69 U-5','','','Plasma',428,'',113 from dual union all
select 19209,'1969-S601','1969-11-03 16:00:00','military missile','failure',939,'C3XD-08','','','','Test',246,'',500 from dual union all
select 1921,'1953-A485','1953-12-16 00:00:00','military missile','success',771,'GM-11103','','','','Test',222,'',12 from dual union all
select 19210,'1969-S602','1969-11-03 17:11:00','suborbital rocket','success',226,'CRL A17.906-1','PCA69 U-4','','','Ionos',428,'',126 from dual union all
select 19211,'1969-S603','1969-11-03 17:30:00','suborbital rocket','success',848,'CRL AH07.886','PCA69 N-3','','','Aeron',428,'',144 from dual union all
select 19212,'1969-S604','1969-11-03 17:47:00','suborbital rocket','success',848,'CRL A07.902-4','PCA69 N-4','','','Aeron',428,'',122 from dual union all
select 19213,'1969-S605','1969-11-03 18:54:00','suborbital rocket','success',850,'','PCA69 BRL 5','','','Ionos',428,'',126 from dual union all
select 19214,'1969-S606','1969-11-03 22:45:00','suborbital rocket','success',226,'CRL AJ17.611','PCA69 U-3','','','Plasma',428,'',82 from dual union all
select 19215,'1969-S607','1969-11-03 23:08:00','suborbital rocket','success',848,'CRL AT07.398','PCA69 C-4 Sphere','','','Aeron',428,'',192 from dual union all
select 19216,'1969-S608','1969-11-04 03:28:00','military missile','success',1002,'U 22502-02','8K67P UBP-6','GCh 8F675/OS-67 No. 1L','','Test',515,'',1000 from dual union all
select 19217,'1969-S609','1969-11-04 19:29:00','suborbital rocket','success',32,'NASA 04.136DS','NRL NE3.215 FDVU-2M','','','Solar EUV',1869,'',200 from dual union all
select 19218,'1969-S61','1969-01-12 01:32:00','suborbital rocket','success',848,'CRL AG07.652','AFGL TMA release','','','Aeron',394,'',157 from dual union all
select 19219,'1969-S610','1969-11-04 20:27:00','suborbital rocket','success',32,'NASA 04.283CS','','','','Solar X',1869,'',181 from dual union all
select 1922,'1953-A486','1953-12-16 00:00:00','atmospheric rocket','success',836,'1216A','','','','Test',1895,'',10 from dual union all
select 19220,'1969-S611','1969-11-04 21:30:00','suborbital rocket','success',850,'','PCA69 BRL 6','','','Ionos',428,'',124 from dual union all
select 19221,'1969-S612','1969-11-04 22:06:00','suborbital rocket','success',1181,'CRL A30.900-3','PCA69 Q-2','','','Aeron',428,'',99 from dual union all
select 19222,'1969-S613','1969-11-04 22:38:00','suborbital rocket','success',850,'','PCA69 BRL 7','','','Ionos',428,'',125 from dual union all
select 19223,'1969-S614','1969-11-04 22:50:00','suborbital rocket','success',848,'CRL AH07.893','PCA69 N-5','','','Aeron',428,'',143 from dual union all
select 19224,'1969-S615','1969-11-04 23:08:00','suborbital rocket','success',226,'CRL AJ17.617','PCA69 U-6','','','Plasma',428,'',126 from dual union all
select 19225,'1969-S616','1969-11-05 08:11:00','suborbital rocket','success',740,'','','','','Ionos',135,'ZUB',160 from dual union all
select 19226,'1969-S617','1969-11-05 11:12:00','suborbital rocket','success',740,'','','','','Ionos',135,'ZUB',162 from dual union all
select 19227,'1969-S618','1969-11-05 11:18:00','suborbital rocket','success',218,'AAF-3B-49','','','','Auroral/CR',1315,'',200 from dual union all
select 19228,'1969-S619','1969-11-05 20:49:00','suborbital rocket','success',848,'CRL AH07.889','PCA69 N','','','Aeron',428,'',122 from dual union all
select 19229,'1969-S62','1969-01-12 05:10:00','suborbital rocket','success',656,'K-10C-1','Stability test','','','Aeron/Solar UV/Test',839,'',229 from dual union all
select 1923,'1953-A487','1953-12-17 00:00:00','military missile','unknown',339,'1356R','','','','Test',1895,'',30 from dual union all
select 19230,'1969-S620','1969-11-07 07:00:00','military missile','success',795,'1225','Mk 11 RV','ST OLYMPIC TR. B-11','OLYMPIC TRIALS B-11','OT',1705,'',1300 from dual union all
select 19231,'1969-S621','1969-11-06 20:00:02','military missile','success',799,'FTM-209','Mk 12 RV x 3','R&D','','Test',266,'',1300 from dual union all
select 19232,'1969-S622','1969-11-07 02:51:00','suborbital rocket','success',848,'CRL AH07.892','PCA69 N-6','','','Aeron',428,'',122 from dual union all
select 19233,'1969-S623','1969-11-08 00:00:00','suborbital rocket','success',1498,'006','FU-189 Star pt test','','','Test/Astron',341,'',204 from dual union all
select 19234,'1969-S624','1969-11-08 07:49:00','suborbital rocket','success',227,'NRL NF9.207','','','','IR Astron',1821,'',189 from dual union all
select 19235,'1969-S625','1969-11-09 16:47:00','suborbital rocket','success',740,'','','','','Ionos',135,'ZUB',162 from dual union all
select 19236,'1969-S626','1969-11-10 21:00:12','suborbital rocket','success',1204,'S46/1','ESRO S46/1 SHF-9','','','Auroral',416,'',214 from dual union all
select 19237,'1969-S627','1969-11-15 17:00:00','military missile','unknown',930,'A2E','','','','OT',421,'SSBN 619',1000 from dual union all
select 19238,'1969-S628','1969-11-18 02:17:00','suborbital rocket','failure',1207,'SL602','MSSL XR2 FSG','','','XR Astronomy',1839,'',208 from dual union all
select 19239,'1969-S629','1969-11-18 13:34:00','suborbital rocket','success',448,'D-304','FU-192 Plasma','','','Ionos',307,'',475 from dual union all
select 1924,'1953-A488','1953-12-17 00:00:00','atmospheric rocket','success',365,'','F11F-1 Model','','','Test',1821,'',10 from dual union all
select 19240,'1969-S63','1969-01-13 23:43:00','suborbital rocket','success',848,'CRL AH07.673','AFGL Ba release','','','Aeron',394,'',220 from dual union all
select 19241,'1969-S630','1969-11-18 21:32:42','suborbital rocket','success',864,'Sandia 152-135','LRL ACS-5','','','Solar X',1490,'',174 from dual union all
select 19242,'1969-S631','1969-11-19 00:00:00','military missile','success',796,'2508','Mk 11 RV','ST','','Test',1707,'',1300 from dual union all
select 19243,'1969-S632','1969-11-20 01:00:00','suborbital rocket','success',1207,'SL801','Cul UVS/XUVS 14','','','Solar UV,X',1839,'',242 from dual union all
select 19244,'1969-S633','1969-11-24 16:37:00','military missile','unknown',932,'A3E','','','','OT',421,'SSBN 622',1000 from dual union all
select 19245,'1969-S634','1969-11-26 05:27:00','military missile','success',795,'2063','Mk 11 RV','FOT GT49B','GLORY TRIP 49B','OT',1698,'',1300 from dual union all
select 19246,'1969-S635','1969-11-26 00:38:37','suborbital rocket','success',839,'Ferdinand 21','Ferdinand 21','','','Aeron/Ionos',29,'',107 from dual union all
select 19247,'1969-S636','1969-11-26 20:01:00','military missile','success',939,'C3XT-17','Tactical config','','','Test',246,'',500 from dual union all
select 19248,'1969-S637','1969-11-26 23:32:00','suborbital rocket','success',1207,'SL603','Cul UVS 15','','','Solar UV',1839,'',180 from dual union all
select 19249,'1969-S638','1969-11-27 05:29:00','suborbital rocket','success',290,'C62/1','ESRO C62/1','','','Auroral',408,'',160 from dual union all
select 1925,'1953-A489','1953-12-17 00:00:00','military missile','success',771,'GM-11102','','','','Test',222,'',12 from dual union all
select 19250,'1969-S639','1969-11-27 06:31:00','suborbital rocket','success',290,'C62/2','ESRO C62/2','','','Auroral',408,'',164 from dual union all
select 19251,'1969-S64','1969-01-14 13:30:00','suborbital rocket','success',846,'STV 75','SR-06','','','Aeron',1864,'',100 from dual union all
select 19252,'1969-S640','1969-11-30 02:58:11','suborbital rocket','success',1250,'','Arcas VII','','','Aeron/Ionos/Plasma',29,'',100 from dual union all
select 19253,'1969-S641','1969-12-02 00:03:00','military missile','success',1002,'','8K67P UBP-7','','','Test',519,'',1000 from dual union all
select 19254,'1969-S642','1969-12-02 16:30:00','military missile','unknown',932,'A3E','','','','OT',421,'SSBN 620',1000 from dual union all
select 19255,'1969-S643','1969-12-04 09:15:00','military missile','success',162,'44F','Mk 11 Mod RV','ABRES RVTO-1A-6','','Reentry Test',1680,'',1400 from dual union all
select 19256,'1969-S644','1969-12-03 18:42:00','suborbital rocket','success',217,'AMF-3-47','','','','Aeron',428,'',120 from dual union all
select 19257,'1969-S645','1969-12-03 23:12:00','suborbital rocket','success',217,'AMF-3-46','','','','Aeron',428,'',117 from dual union all
select 19258,'1969-S646','1969-12-04 06:28:00','suborbital rocket','success',30,'NASA 04.175UG','','','','XR Astronomy',1869,'',200 from dual union all
select 19259,'1969-S647','1969-12-05 22:40:00','military missile','success',795,'1068','Mk 11 RV','FOT GT50B','GLORY TRIP 50B','OT',1702,'',1300 from dual union all
select 1926,'1953-A49','1953-02-11 00:00:00','atmospheric rocket','unknown',576,'88','','','','Test',1895,'',10 from dual union all
select 19260,'1969-S648','1969-12-05 01:31:00','suborbital rocket','success',837,'NASA 14.353UG','','','','XR Astron',1895,'',200 from dual union all
select 19261,'1969-S649','1969-12-05 09:30:00','suborbital rocket','success',848,'CRL AH07.680','Release','','','Aeron',394,'',158 from dual union all
select 19262,'1969-S65','1969-01-14 05:25:00','suborbital rocket','failure',219,'AAF-4-17','','','','Aeron/Plasma',428,'',99 from dual union all
select 19263,'1969-S650','1969-12-05 10:00:00','suborbital rocket','success',848,'CRL AH07.681','Release','','','Aeron',394,'',155 from dual union all
select 19264,'1969-S651','1969-12-05 11:56:00','suborbital rocket','success',848,'CRL AH07.676','Release','','','Aeron',394,'',151 from dual union all
select 19265,'1969-S652','1969-12-07 00:35:00','suborbital rocket','unknown',289,'ISRO-45.003','ISRO 45.03','TERLS-141','','Test/Ionos/XR Astron',1467,'',123 from dual union all
select 19266,'1969-S653','1969-12-09 12:03:00','suborbital rocket','success',1190,'INTA MP-6901','','','','Ionos',396,'',90 from dual union all
select 19267,'1969-S654','1969-12-09 14:49:00','suborbital rocket','success',1190,'INTA MP-6902','','','','Ionos',396,'',96 from dual union all
select 19268,'1969-S655','1969-12-10 00:00:00','military missile','success',1103,'C3','S02C3','','','Test',301,'',1000 from dual;
insert into LAUNCH
select 19269,'1969-S656','1969-12-10 04:31:00','suborbital rocket','success',848,'CRL A07.907-1','Polar disturbance','','','Auroral',428,'',168 from dual union all
select 1927,'1953-A490','1953-12-17 00:00:00','atmospheric rocket','success',836,'1230A','','','','Test',1895,'',10 from dual union all
select 19270,'1969-S657','1969-12-10 08:00:00','suborbital rocket','success',740,'','','','','Aeron/Ionos',636,'',165 from dual union all
select 19271,'1969-S658','1969-12-10 11:54:00','suborbital rocket','success',1190,'INTA MP-6903','','','','Ionos',396,'',90 from dual union all
select 19272,'1969-S659','1969-12-10 12:02:00','suborbital rocket','success',1190,'INTA MP-6904','','','','Ionos',396,'',99 from dual union all
select 19273,'1969-S66','1969-01-14 10:00:00','suborbital rocket','success',655,'K-10-4','','','','IR/XR Astron/Solar UV',839,'',229 from dual union all
select 19274,'1969-S660','1969-12-10 20:59:00','suborbital rocket','success',740,'','','','','Ionos',135,'ZUB',160 from dual union all
select 19275,'1969-S661','1969-12-10 22:13:00','suborbital rocket','success',740,'','','','','Ionos',135,'ZUB',160 from dual union all
select 19276,'1969-S663','1969-12-12 12:13:00','military missile','success',168,'93F','RMV-340B/BSE/LSE','ABRES RMP-B-14/KX-113','','Reentry Test',1682,'',1400 from dual union all
select 19277,'1969-S664','1969-12-13 02:00:00','suborbital rocket','success',839,'','3-sphere ejection','','','Aeron',1895,'',100 from dual union all
select 19278,'1969-S665','1969-12-12 01:24:00','military missile','success',999,'Yu22525-54T','8K67 UBP-30','GCh 8F675','','OT',515,'',1000 from dual union all
select 19279,'1969-S666','1969-12-12 02:30:00','suborbital rocket','success',848,'CRL AH07.682','Release','','','Aeron',394,'',155 from dual union all
select 1928,'1953-A491','1953-12-18 00:00:00','atmospheric rocket','unknown',576,'422N','','','','Test',1895,'',10 from dual union all
select 19280,'1969-S667','1969-12-12 11:50:00','suborbital rocket','success',1189,'INTA MP-6905','','','','Ionos',396,'',100 from dual union all
select 19281,'1969-S668','1969-12-12 12:12:00','suborbital rocket','success',1189,'INTA MP-6906','','','','Ionos',396,'',100 from dual union all
select 19282,'1969-S669','1969-12-13 00:00:00','suborbital rocket','success',30,'NASA 04.252GG-UG','UVT-SDGS 2','','','UV Astronomy',1869,'',200 from dual union all
select 19283,'1969-S67','1969-01-14 20:14:00','suborbital rocket','success',740,'','Thermosphere comp 5','','','Aeron',1010,'',176 from dual union all
select 19284,'1969-S670','1969-12-13 12:00:00','suborbital rocket','success',1189,'INTA MP-6907','','','','Ionos',396,'',100 from dual union all
select 19285,'1969-S671','1969-12-13 22:22:00','suborbital rocket','success',837,'NASA 14.445CM','GCA Release Na','','','Aeronomy',1821,'',200 from dual union all
select 19286,'1969-S672','1969-12-14 00:30:00','suborbital rocket','success',837,'NASA 14.446CM','GCA Release TMA','','','Aeronomy',1821,'',217 from dual union all
select 19287,'1969-S674','1969-12-15 00:00:00','military missile','success',1251,'800017','KT-15','','','Test',868,'',100 from dual union all
select 19288,'1969-S675','1969-12-15 02:27:00','suborbital rocket','success',219,'AAF-4-20','Auroral particles','','','Auroral/Ionos',428,'',774 from dual union all
select 19289,'1969-S676','1969-12-15 12:00:00','suborbital rocket','success',1189,'INTA MP-6908','','','','Ionos',396,'',86 from dual union all
select 1929,'1953-A492','1953-01-15 00:00:00','atmospheric rocket','unknown',490,'1','Warhead Accel Test','','','Test',1895,'',10 from dual union all
select 19290,'1969-S677','1969-12-16 00:00:00','military missile','success',795,'STF-2','SSTTP 2  (Mod Mk 11)','SSTTP 2','MR CLEAN II','Target',1701,'',1300 from dual union all
select 19291,'1969-S678','1969-12-16 08:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',163 from dual union all
select 19292,'1969-S679','1969-12-16 11:56:00','suborbital rocket','success',1189,'INTA MP-6909','','','','Ionos',396,'',96 from dual union all
select 19293,'1969-S68','1969-01-16 11:33:00','military missile','success',168,'70F','RMV-340B/BSE','ABRES RMP-B-11/KX-111','','Reentry Test',1682,'',1400 from dual union all
select 19294,'1969-S680','1969-12-16 17:25:00','military missile','success',939,'C3XT-18','','','','Test',421,'EAG-154',500 from dual union all
select 19295,'1969-S681','1969-12-17 11:50:00','suborbital rocket','success',1189,'INTA MP-6910','','','','Ionos',396,'',98 from dual union all
select 19296,'1969-S682','1969-12-17 18:30:00','suborbital rocket','success',32,'NASA 04.167DS','','','','Solar',1869,'',204 from dual union all
select 19297,'1969-S683','1969-12-18 00:15:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',166 from dual union all
select 19298,'1969-S684','1969-12-18 06:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',162 from dual union all
select 19299,'1969-S685','1969-12-18 11:58:00','suborbital rocket','success',1189,'INTA MP-6911','','','','Ionos',396,'',100 from dual union all
select 193,'1944-S175','1944-10-04 00:00:00','military missile','success',1470,'','London','','','Weapon',1734,'',90 from dual union all
select 1930,'1953-A493','1953-03-25 00:00:00','atmospheric rocket','unknown',491,'308','Warhead Accel Test','','','Test',1895,'',10 from dual union all
select 19300,'1969-S686','1969-12-18 13:28:00','suborbital rocket','success',1189,'INTA MP-6912','','','','Ionos',396,'',95 from dual union all
select 19301,'1969-S687','1969-12-19 01:30:00','military missile','success',999,'','8K67 UBP-31','','','Test',518,'',1000 from dual union all
select 19302,'1969-S688','1969-12-19 08:30:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',636,'',90 from dual union all
select 19303,'1969-S689','1969-12-19 11:15:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',165 from dual union all
select 19304,'1969-S69','1969-01-16 20:28:00','sounding rocket','failure',698,'L-3H-4','','','','Ionos',834,'',100 from dual union all
select 19305,'1969-S690','1969-12-22 00:00:00','suborbital rocket','success',1058,'','','','','Test',310,'',250 from dual union all
select 19306,'1969-S691','1969-12-22 00:00:00','suborbital rocket','success',279,'CR-X1','Dummy St2','Ex.Navidad','','Test',310,'',52 from dual union all
select 19307,'1969-S692','1969-12-23 00:00:00','suborbital rocket','success',276,'No 3','Bio-II','','','Test/Bio',310,'',70 from dual union all
select 19308,'1969-S693','1969-12-24 07:05:00','suborbital rocket','success',740,'MR-12','Particles','','','Ionos/Solar UV,X',636,'',162 from dual union all
select 19309,'1969-S694','1969-07-18 00:00:00','military missile','failure',980,'','8K64U Aldan Target 1','Aldan','','Target',454,'',0 from dual union all
select 1931,'1953-A494','1953-10-16 00:00:00','atmospheric rocket','unknown',491,'3','Warhead Accel Test','','','Test',1895,'',10 from dual union all
select 19310,'1969-S695','1969-02-14 01:07:00','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',100 from dual union all
select 19311,'1969-S696','1969-02-14 03:24:00','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',100 from dual union all
select 19312,'1969-S697','1969-02-14 05:05:00','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',99 from dual union all
select 19313,'1969-S698','1969-02-14 07:05:00','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',97 from dual union all
select 19314,'1969-S699','1969-02-14 09:10:00','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',100 from dual union all
select 19315,'1969-S70','1969-01-17 00:00:00','military missile','success',135,'V140D','ABRES AN-4 (USAF)','ABRES AN-4 (USAF)','','Reentry Test',614,'',200 from dual union all
select 19316,'1969-S700','1969-02-14 11:23:00','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',103 from dual union all
select 19317,'1969-S701','1969-03-14 19:05:00','suborbital rocket','success',1181,'','Recovery test','','','Test',1821,'',122 from dual union all
select 19318,'1969-S702','1969-02-26 00:00:00','suborbital rocket','success',1506,'SDC VD-31','Ram-Air Sphere','','','Aeron',276,'',137 from dual union all
select 19319,'1969-S703','1969-05-14 00:00:00','suborbital rocket','success',1506,'SDC VD-32','ROBIN','','','Aeron',276,'',129 from dual union all
select 1932,'1953-A495','1953-10-29 00:00:00','atmospheric rocket','unknown',491,'4','Warhead Accel Test','','','Test',1895,'',10 from dual union all
select 19320,'1969-S704','1969-09-30 00:00:00','suborbital rocket','success',1506,'SDC VD-36','ROBIN','','','Aeron',276,'',136 from dual union all
select 19321,'1969-S705','1969-09-30 00:00:00','suborbital rocket','success',1506,'SDC VD-37','Double Sphere','','','Aeron',276,'',136 from dual union all
select 19322,'1969-S706','1969-09-30 00:00:00','suborbital rocket','success',1506,'SDC VD-38','Double Sphere','','','Aeron',276,'',133 from dual union all
select 19323,'1969-S707','1969-10-01 00:00:00','suborbital rocket','success',1506,'SDC VD-39','ROBIN','','','Aeron',276,'',136 from dual union all
select 19324,'1969-S708','1969-08-28 18:03:00','suborbital rocket','success',839,'','NRL Cesium','','','Aeron',1821,'',102 from dual union all
select 19325,'1969-S709','1969-08-29 10:00:00','suborbital rocket','success',839,'','NRL Cesium','','','Aeron',1821,'',102 from dual union all
select 19326,'1969-S71','1969-01-17 18:34:00','suborbital rocket','success',839,'NASA 10.313IM','RTG SNC 2A/1 K69/1','','','Aeronomy',413,'',117 from dual union all
select 19327,'1969-S710','1969-06-03 17:58:00','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',102 from dual union all
select 19328,'1969-S711','1969-07-18 17:16:00','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',95 from dual union all
select 19329,'1969-S712','1969-07-23 22:59:00','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',99 from dual union all
select 1933,'1953-A496','1953-10-30 00:00:00','atmospheric rocket','unknown',491,'5','Warhead Accel Test','','','Test',1895,'',10 from dual union all
select 19330,'1969-S713','1969-07-25 10:19:00','suborbital rocket','success',521,'','Cesium package','','','Meteo',1791,'',92 from dual union all
select 19331,'1969-S714','1969-11-16 00:00:00','military missile','success',359,'601902','R&D (Destruct)','','','Test',809,'',500 from dual union all
select 19332,'1969-S715','1969-07-15 00:00:00','test rocket','success',645,'','K63D BOR-1','BOR-1','','Reentry Test',1752,'',100 from dual union all
select 19333,'1969-S716','1969-12-06 06:00:00','test rocket','success',645,'','K63D BOR-2','BOR-2 No. 101','','Reentry Test',1752,'',100 from dual union all
select 19334,'1969-S717','1969-07-25 00:00:00','suborbital rocket','success',538,'WRE HAT 54','','','','Aeron',1837,'',80 from dual union all
select 19335,'1969-S718','1969-07-24 00:00:00','suborbital rocket','success',535,'HAD 309','Nimbus pass','','','Aeron',1835,'',115 from dual union all
select 19336,'1969-S719','1969-12-09 00:00:00','suborbital rocket','success',535,'HAD 310','','','','Aeron',1835,'',115 from dual union all
select 19337,'1969-S72','1969-01-17 21:32:00','suborbital rocket','success',839,'NASA 10.306GM','GSFC Grenade 201 W','','','Aeronomy',1821,'',112 from dual union all
select 19338,'1969-S720','1969-08-05 09:08:00','suborbital rocket','success',27,'AH1021','Grenades','','','Aeron',1833,'',208 from dual union all
select 19339,'1969-S721','1969-10-01 00:00:00','suborbital rocket','success',1,'','Ion 1','','','Tech/Ionos',636,'',400 from dual union all
select 1934,'1953-A497','1953-01-01 00:00:00','atmospheric rocket','unknown',491,'6','Warhead Accel Test','','','Test',1895,'',10 from dual union all
select 19340,'1969-S722','1969-01-01 00:00:00','suborbital rocket','success',1,'','Yantar''-3','','','Tech/Ionos',636,'',400 from dual union all
select 19341,'1969-S723','1969-11-20 20:19:00','suborbital rocket','success',839,'','3-sphere ejection','','','Aeron',1895,'',100 from dual union all
select 19342,'1969-S724','1969-08-06 00:00:00','suborbital rocket','success',30,'','Solar radiation','','','Solar UV?',1895,'',150 from dual union all
select 19343,'1969-S725','1969-05-22 00:00:00','suborbital rocket','success',867,'Sandia 152-125','Rocket EMP Test 2','','','Plasma/Active',855,'',150 from dual union all
select 19344,'1969-S726','1969-01-22 00:00:00','military missile','success',982,'T 22504-07T','8K64U UBP-P63','R-16 PL69-02','','OT',477,'',1210 from dual union all
select 19345,'1969-S727','1969-01-30 00:00:00','military missile','success',982,'G 22501-16T','8K64U UBP-P64','R-16 PL69-03','','OT',477,'',1210 from dual union all
select 19346,'1969-S728','1969-02-28 00:00:00','military missile','success',982,'G 22504-16T','8K64U UBP-P65','R-16 PL69-04','','OT',477,'',1210 from dual union all
select 19347,'1969-S729','1969-01-18 00:00:00','military missile','failure',982,'','8K64U UBP','R-16 PL69-01','','OT',477,'',0 from dual union all
select 19348,'1969-S73','1969-01-19 02:00:00','suborbital rocket','success',839,'NASA 10.299GM','GSFC Grenade 202 PB','','','Aeronomy',1039,'',100 from dual union all
select 19349,'1969-S730','1969-03-27 00:00:00','military missile','success',982,'T 22503-11T','8K64U UBP-P66','R-16 PL69-05','','OT',477,'',1210 from dual union all
select 1935,'1953-A498','1953-03-05 00:00:00','atmospheric rocket','success',836,'1249A-192','Nike 1249 R&D 185R','','','Test',1895,'',10 from dual union all
select 19350,'1969-S731','1969-10-18 00:00:00','military missile','success',982,'T 22505-12T','8K64U UBP-P67','R-16 PL69-06','','OT',477,'',1210 from dual union all
select 19351,'1969-S732','1969-12-16 00:00:00','military missile','success',982,'T 22505-17T','8K64U UBP-P70','R-16 PL69-09','','OT',477,'',1210 from dual union all
select 19352,'1969-S733','1969-10-28 00:00:00','military missile','success',982,'T 22505-14T','8K64U UBP-P68','R-16 PL69-07','','OT',477,'',1210 from dual union all
select 19353,'1969-S734','1969-11-24 00:00:00','military missile','success',982,'T 22505-10T','8K64U UBP-P69','R-16 PL69-08','','OT',477,'',1210 from dual union all
select 19354,'1969-S735','1969-11-01 00:00:00','military missile','success',1018,'G 15002-07','8K75 UBP-P3','','','OT',477,'',1160 from dual union all
select 19355,'1969-S74','1969-01-19 12:00:00','suborbital rocket','success',668,'K-9M-24','Gyro-plasma probe','','','Aeron/Ionos/Fields/',839,'',340 from dual union all
select 19356,'1969-S740','1969-02-07 00:00:00','military missile','success',1037,'Yu29501-06','8K98 UBP-5','RT2','','Test',477,'',1000 from dual union all
select 19357,'1969-S741','1969-07-24 00:00:00','military missile','success',1037,'V 29501-09','8K98 UBP-6','','','Test',477,'',1000 from dual union all
select 19358,'1969-S742','1969-07-25 00:00:00','military missile','success',1037,'V 29502-01','8K98 UBP-7','','','Test',477,'',1000 from dual;
insert into LAUNCH
select 19359,'1969-S743','1969-08-28 00:00:00','military missile','success',1037,'Yu64612-28','8K98 UBP-8','','','Test',477,'',1000 from dual union all
select 1936,'1953-A499','1953-03-06 00:00:00','atmospheric rocket','success',836,'1249A-188','Nike 1249 R&D 187R','','','Test',1895,'',10 from dual union all
select 19360,'1969-S744','1969-09-19 00:00:00','military missile','success',1037,'Yu29501-08','8K98 UBP-9','','','Test',477,'',1000 from dual union all
select 19361,'1969-S745','1969-11-24 00:00:00','military missile','success',1037,'Yu45713-21','8K98 UBP-10','','','Test',477,'',1000 from dual union all
select 19362,'1969-S746','1969-01-01 00:00:00','military missile','success',1449,'','8K84 54 RD UBP-4','','','OT',1468,'',1000 from dual union all
select 19363,'1969-S747','1969-01-01 00:00:00','military missile','success',1449,'','8K84 54 RD UBP-5','','','OT',1468,'',1000 from dual union all
select 19364,'1969-S748','1969-01-01 00:00:00','military missile','success',1449,'','8K84 54 RD UBP-6','','','OT',1468,'',1000 from dual union all
select 19365,'1969-S749','1969-01-01 00:00:00','military missile','success',1449,'','8K84 54 RD UBP-7','','','OT',1468,'',1000 from dual union all
select 19366,'1969-S75','1969-01-20 17:04:00','suborbital rocket','success',839,'NASA 10.301GM','GSFC Grenade 203 C','','','Aeronomy',428,'',97 from dual union all
select 19367,'1969-S750','1969-07-24 00:00:00','military missile','success',1449,'','8K84 10 RD UBP-1','','','OT',897,'',1000 from dual union all
select 19368,'1969-S751','1969-07-27 00:00:00','military missile','success',1449,'','8K84 10 RD UBP-2','','','OT',897,'',1000 from dual union all
select 19369,'1969-S752','1969-06-12 00:00:00','military missile','success',982,'','8K64 42 RD UBP-7','','','OT',1002,'',1210 from dual union all
select 1937,'1953-A50','1953-02-11 00:00:00','atmospheric rocket','unknown',576,'89','','','','Test',1895,'',10 from dual union all
select 19370,'1969-S753','1969-01-01 00:00:00','military missile','success',982,'','8K64 36 RD UBP-5','','','OT',898,'',1210 from dual union all
select 19371,'1969-S754','1969-01-06 23:41:00','suborbital rocket','success',848,'CRL AH07.671','AFGL Ba release','','','Aeron',394,'',200 from dual union all
select 19372,'1969-S755','1969-01-08 00:30:00','suborbital rocket','success',848,'CRL AG07.625','AFGL TMA release','','','Aeron',394,'',150 from dual union all
select 19373,'1969-S756','1969-09-12 00:00:00','military missile','success',980,'T 14902-15','8K64U Aldan Target 2','Aldan','','Target',477,'',675 from dual union all
select 19374,'1969-S757','1969-11-03 00:00:00','military missile','success',980,'T 14902-18TF','8K64U Aldan Target 3','Aldan','','Target',477,'',675 from dual union all
select 19375,'1969-S758','1969-11-20 00:00:00','military missile','success',980,'T 14902-17TF','8K64U Aldan Target 4','Aldan','','Target',477,'',675 from dual union all
select 19376,'1969-S759','1969-12-26 00:00:00','military missile','success',982,'T 22504-12T','8K64U UBP-P71','R-16 PL69-10','','OT',477,'',1210 from dual union all
select 19377,'1969-S76','1969-01-21 09:40:00','military missile','success',795,'1089','Mk 11A RV','ST OLYMPIC T. B-7','OLYMPIC TRIALS B-7','OT',1705,'',1300 from dual union all
select 19378,'1969-S760','1969-01-01 00:00:00','military missile','success',977,'','8K63U UBP','','','Test',840,'',402 from dual union all
select 19379,'1969-S761','1969-01-01 00:00:00','military missile','success',977,'','8K63U UBP','','','Test',840,'',402 from dual union all
select 1938,'1953-A500','1953-04-02 00:00:00','atmospheric rocket','success',836,'1249A-197','Nike 1249 R&D 198R','','','Test',1895,'',10 from dual union all
select 19380,'1969-S762','1969-01-01 00:00:00','military missile','success',977,'','8K63U UBP','','','Test',840,'',402 from dual union all
select 19381,'1969-S763','1969-07-01 00:00:00','military missile','success',980,'','8K65U 35 RD UBP-5','','','Test',943,'',675 from dual union all
select 19382,'1969-S764','1969-07-01 00:00:00','military missile','success',980,'','8K65U 35 RD UBP-6','','','Test',943,'',675 from dual union all
select 19383,'1969-S765','1969-06-12 00:00:00','military missile','success',980,'','8K65U 45 RD UBP','','','Test',1000,'',675 from dual union all
select 19384,'1969-S766','1969-06-01 00:00:00','military missile','success',981,'','8K64 8 RD UBP-3','','','OT',1916,'',1210 from dual union all
select 19385,'1969-S767','1969-02-04 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 19386,'1969-S768','1969-02-12 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 19387,'1969-S769','1969-02-18 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 19388,'1969-S77','1969-01-21 08:03:00','suborbital rocket','success',740,'','Ne-Te 2','','','Ionos',636,'',170 from dual union all
select 19389,'1969-S770','1969-07-24 00:00:00','military missile','success',976,'','8K63 31 RD UBP','Salvo','','Test',636,'',402 from dual union all
select 1939,'1953-A501','1953-02-02 00:00:00','atmospheric rocket','success',1526,'','','','','Test',1895,'B-29',10 from dual union all
select 19390,'1969-S771','1969-07-24 00:00:00','military missile','success',976,'','8K63 31 RD UBP','Salvo','','Test',636,'',402 from dual union all
select 19391,'1969-S772','1969-07-24 00:00:00','military missile','success',976,'','8K63 31 RD UBP','Salvo','','Test',636,'',402 from dual union all
select 19392,'1969-S773','1969-07-24 00:00:00','military missile','success',976,'','8K63 31 RD UBP','Salvo','','Test',636,'',402 from dual union all
select 19393,'1969-S774','1969-07-24 00:00:00','military missile','success',976,'','8K63 31 RD UBP','Salvo','','Test',636,'',402 from dual union all
select 19394,'1969-S775','1969-08-07 00:00:00','military missile','success',976,'','8K63 29 RD UBP','Salvo','','Test',636,'',402 from dual union all
select 19395,'1969-S776','1969-12-21 00:00:00','military missile','success',976,'','8K63 UBP','ShPU','','Test',636,'',402 from dual union all
select 19396,'1969-S777','1969-12-27 00:00:00','military missile','success',976,'','8K63 UBP','ShPU','','Test',636,'',402 from dual union all
select 19397,'1969-S778','1969-01-27 00:00:00','military missile','unknown',980,'','8K65U UBP 29 RD','','','Test',636,'',675 from dual union all
select 19398,'1969-S779','1969-02-18 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 19399,'1969-S78','1969-01-21 16:00:00','military missile','failure',939,'C3XD-03','','','','Test',246,'',2 from dual union all
select 194,'1944-S176','1944-10-05 07:20:00','military missile','success',1470,'18799','Ma204','','','Test',661,'',90 from dual union all
select 1940,'1953-A502','1953-04-08 00:00:00','atmospheric rocket','success',1526,'','','','','Test',1895,'B-29',10 from dual union all
select 19400,'1969-S780','1969-02-18 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 19401,'1969-S781','1969-03-25 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 19402,'1969-S782','1969-04-03 00:00:00','military missile','unknown',979,'','8K65 UBP 32 RD','','','Test',636,'',675 from dual union all
select 19403,'1969-S783','1969-06-21 00:00:00','military missile','unknown',980,'','8K65U UBP 29 RD','','','Test',636,'',675 from dual union all
select 19404,'1969-S784','1969-06-12 00:00:00','military missile','success',980,'','8K65U 68 RB UBP-2','','','Test',1420,'',675 from dual union all
select 19405,'1969-S785','1969-12-01 00:00:00','suborbital rocket','success',846,'','HAVE HORN 1','','','Target',1003,'',100 from dual union all
select 19406,'1969-S786','1969-06-26 21:41:00','suborbital rocket','unknown',547,'','','','','Aeron?',394,'',100 from dual union all
select 19407,'1969-S79','1969-01-22 22:17:00','suborbital rocket','success',839,'NASA 10.304GM','GSFC Grenade 204 C','','','Aeronomy',428,'',94 from dual union all
select 19408,'1969-S80','1969-01-23 00:00:00','military missile','success',135,'B038','FATE Type IIIB','ABRES I-11 (USAF)','','Reentry Test',613,'',200 from dual union all
select 19409,'1969-S81','1969-01-23 12:02:00','suborbital rocket','success',740,'','Ne-Te 3','','','Ionos',636,'',174 from dual union all
select 1941,'1953-A503','1953-09-12 00:00:00','atmospheric rocket','success',1526,'','','','','Test',1895,'B-29',10 from dual union all
select 19410,'1969-S82','1969-01-23 20:00:00','suborbital rocket','success',1199,'SL726','MSSL I-29/RSRS-9','','','Ionospheric',1839,'',149 from dual union all
select 19411,'1969-S83','1969-01-23 23:00:00','suborbital rocket','success',839,'NASA 10.315IM','RTG SNC 2A/3 K69/1','','','Aeronomy',413,'',116 from dual union all
select 19412,'1969-S84','1969-01-24 20:00:00','suborbital rocket','success',839,'NASA 10.305GM','GSFC Grenade 205 W','','','Aeronomy',1821,'',81 from dual union all
select 19413,'1969-S85','1969-01-25 21:00:00','suborbital rocket','success',839,'NASA 10.316IM','RTG SNC 2A/4 K69/1','','','Aeronomy',413,'',118 from dual union all
select 19414,'1969-S86','1969-01-26 05:00:00','suborbital rocket','success',839,'NASA 10.297GM','GSFC Grenade 206 PB','','','Aeronomy',1039,'',91 from dual union all
select 19415,'1969-S87','1969-01-26 09:45:00','suborbital rocket','success',44,'NASA 17.03GE','Electron Accelerator','','','Plasma',1821,'',271 from dual union all
select 19416,'1969-S88','1969-01-28 05:49:00','suborbital rocket','success',867,'NASA 18.25UG','Soft X-ray Counter','','','XR Astron',1821,'',224 from dual union all
select 19417,'1969-S89','1969-01-28 07:16:00','suborbital rocket','success',215,'AEF-2-118','N2,temp profile','','','Aeron',428,'',149 from dual union all
select 19418,'1969-S90','1969-01-29 10:34:00','military missile','success',796,'2504','Mk 11 RV','ST SAMSO Stage 3 Test','SPEC TEST 1','Test',1697,'',1300 from dual union all
select 19419,'1969-S91','1969-01-29 13:52:00','military missile','success',796,'2240','Mk 11B RV','ST Mk 1A Penaids Test 2?','','Test',1703,'',1300 from dual union all
select 1942,'1953-A504','1953-12-08 00:00:00','atmospheric rocket','success',1526,'','','','','Test',1895,'B-29',10 from dual union all
select 19420,'1969-S92','1969-01-30 00:00:00','military missile','success',795,'874','Mk 11 RV','FOT GT32B','GLORY TRIP 32B','OT',1698,'',1300 from dual union all
select 19421,'1969-S93','1969-01-30 17:10:00','suborbital rocket','success',839,'NASA 10.302GM','GSFC Grenade 207 C','','','Aeron',428,'',124 from dual union all
select 19422,'1969-S94','1969-01-31 03:15:00','suborbital rocket','success',32,'NASA 04.266DG','','','','UV Astron',1869,'',179 from dual union all
select 19423,'1969-S95','1969-01-31 07:00:00','suborbital rocket','success',839,'NASA 10.298GM','GSFC Grenade 208 PB','','','Aeronomy',1039,'',100 from dual union all
select 19424,'1969-S96','1969-01-31 08:38:00','suborbital rocket','success',839,'NASA 10.274GM','GSFC Ozone 5','','','Aeronomy',1039,'',100 from dual union all
select 19425,'1969-S97','1969-01-31 17:00:00','suborbital rocket','success',837,'NASA 14.391UI','','','','Ionos',1821,'',227 from dual union all
select 19426,'1969-S98','1969-01-31 17:30:00','suborbital rocket','success',848,'CRL AH07.891','AFCRL Atmosphere','','','Aeron',1821,'',137 from dual union all
select 19427,'1969-S99','1969-01-31 17:57:00','suborbital rocket','success',839,'NASA 10.307GM','GSFC Grenade 209 W','','','Aeronomy',1821,'',121 from dual union all
select 19428,'1969-U01','1969-07-21 17:54:01','lunar return','success',689,'5','Apollo 11','LM 5 Ascent Stage','','Sat',1485,'',null from dual union all
select 19429,'1969-U02','1969-11-20 14:25:56','lunar return','success',689,'6','Apollo 12','LM 6 Ascent Stage','','Sat',1450,'',null from dual union all
select 1943,'1953-A51','1953-02-11 00:00:00','atmospheric rocket','unknown',576,'90','','','','Test',1895,'',10 from dual union all
select 19430,'1969-U03','1969-01-25 11:14:03','orbital','failure',1447,'N 22500-12L','Kosmos','US-A No. 5? reactor','','Sat',590,'',100 from dual union all
select 19431,'1969-W01','1969-01-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 19432,'1969-W02','1969-01-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 19433,'1969-W03','1969-01-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 19434,'1969-W04','1969-01-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 19435,'1969-W05','1969-01-01 00:00:00','suborbital rocket','success',777,'3','','','','Test',933,'',68 from dual union all
select 19436,'1969-W06','1969-01-01 00:00:00','suborbital rocket','success',777,'4','','','','Test',933,'',68 from dual union all
select 19437,'1969-W07','1969-01-01 00:00:00','suborbital rocket','success',777,'5','','','','Test',933,'',68 from dual union all
select 19438,'1969-W08','1969-01-02 15:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',54 from dual union all
select 19439,'1969-W09','1969-01-02 17:46:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 1944,'1953-A52','1953-02-11 00:00:00','atmospheric rocket','unknown',576,'91','','','','Test',1895,'',10 from dual union all
select 19440,'1969-W10','1969-01-03 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',51 from dual union all
select 19441,'1969-W100','1969-01-18 17:48:00','suborbital rocket','success',1188,'','','','','Meteo',1447,'',64 from dual union all
select 19442,'1969-W1000','1969-07-24 16:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',64 from dual union all
select 19443,'1969-W1001','1969-07-24 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 19444,'1969-W1002','1969-07-24 20:24:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',55 from dual union all
select 19445,'1969-W1003','1969-07-24 21:31:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',51 from dual union all
select 19446,'1969-W1004','1969-07-25 09:56:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',63 from dual union all
select 19447,'1969-W1005','1969-07-25 12:32:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',63 from dual union all
select 19448,'1969-W1006','1969-07-25 13:14:59','suborbital rocket','success',1061,'','','','','Meteo',276,'',60 from dual;
insert into LAUNCH
select 19449,'1969-W1007','1969-07-25 14:45:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',176,'',58 from dual union all
select 1945,'1953-A53','1953-02-12 00:00:00','atmospheric rocket','unknown',576,'92','','','','Test',1895,'',10 from dual union all
select 19450,'1969-W1008','1969-07-25 16:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',57 from dual union all
select 19451,'1969-W1009','1969-07-25 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',61 from dual union all
select 19452,'1969-W101','1969-01-20 11:48:59','suborbital rocket','success',631,'INTA H-6901','','','','Meteo',396,'',53 from dual union all
select 19453,'1969-W1010','1969-07-25 17:22:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',59 from dual union all
select 19454,'1969-W1011','1969-07-25 17:33:00','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',65 from dual union all
select 19455,'1969-W1012','1969-07-25 21:14:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',56 from dual union all
select 19456,'1969-W1013','1969-07-28 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 19457,'1969-W1014','1969-07-28 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',63 from dual union all
select 19458,'1969-W1015','1969-07-28 20:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',56 from dual union all
select 19459,'1969-W1016','1969-07-28 20:34:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 1946,'1953-A54','1953-02-12 00:00:00','atmospheric rocket','unknown',576,'93','','','','Test',1895,'',10 from dual union all
select 19460,'1969-W1017','1969-07-28 21:46:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',50 from dual union all
select 19461,'1969-W1018','1969-07-29 18:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',52 from dual union all
select 19462,'1969-W1019','1969-07-29 22:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 19463,'1969-W102','1969-01-20 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',57 from dual union all
select 19464,'1969-W1020','1969-07-30 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',50 from dual union all
select 19465,'1969-W1021','1969-07-30 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 19466,'1969-W1022','1969-07-30 14:45:00','suborbital rocket','unknown',631,'SUPARCO','','','','Aeron',1419,'',64 from dual union all
select 19467,'1969-W1023','1969-07-30 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',60 from dual union all
select 19468,'1969-W1024','1969-07-30 16:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',58 from dual union all
select 19469,'1969-W1025','1969-07-30 16:58:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',61 from dual union all
select 1947,'1953-A55','1953-02-12 00:00:00','military missile','success',703,'607','','','','Test',222,'',8 from dual union all
select 19470,'1969-W1026','1969-07-30 21:03:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',59 from dual union all
select 19471,'1969-W1027','1969-07-31 16:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 19472,'1969-W1028','1969-07-31 21:12:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',51 from dual union all
select 19473,'1969-W1029','1969-07-31 22:27:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 19474,'1969-W103','1969-01-20 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',64 from dual union all
select 19475,'1969-W1030','1969-08-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 19476,'1969-W1031','1969-08-01 15:12:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',66 from dual union all
select 19477,'1969-W1032','1969-08-01 16:00:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1363,'',61 from dual union all
select 19478,'1969-W1033','1969-08-01 18:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',51 from dual union all
select 19479,'1969-W1034','1969-08-01 18:31:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',57 from dual union all
select 1948,'1953-A56','1953-02-12 00:00:00','atmospheric rocket','success',836,'307P','Nike CET 7','','','Test',1895,'',10 from dual union all
select 19480,'1969-W1035','1969-08-01 18:37:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 19481,'1969-W1036','1969-08-01 20:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 19482,'1969-W1037','1969-08-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 19483,'1969-W1038','1969-08-04 14:21:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',35,'',59 from dual union all
select 19484,'1969-W1039','1969-08-04 14:45:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',176,'',62 from dual union all
select 19485,'1969-W104','1969-01-20 15:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 19486,'1969-W1040','1969-08-04 16:23:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',65 from dual union all
select 19487,'1969-W1041','1969-08-04 19:44:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',65 from dual union all
select 19488,'1969-W1042','1969-08-04 21:23:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',50 from dual union all
select 19489,'1969-W1043','1969-08-05 14:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',70 from dual union all
select 1949,'1953-A57','1953-02-12 00:00:00','atmospheric rocket','success',836,'1249A-182','Nike 1249 R&D 172R','','','Test',1895,'',10 from dual union all
select 19490,'1969-W1044','1969-08-05 21:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 19491,'1969-W1045','1969-08-05 21:36:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',52 from dual union all
select 19492,'1969-W1046','1969-08-06 03:59:00','suborbital rocket','success',88,'','Arcas C-H-I','','','Test',428,'',52 from dual union all
select 19493,'1969-W1047','1969-08-06 05:50:00','suborbital rocket','success',631,'ISRO 11.35','ISRO 11.35','TERLS-121','','Aeron',1467,'',64 from dual union all
select 19494,'1969-W1048','1969-08-06 14:00:00','suborbital rocket','success',1061,'','Starute','','','Meteo',276,'',53 from dual union all
select 19495,'1969-W1049','1969-08-06 14:22:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 19496,'1969-W105','1969-01-20 18:41:00','suborbital rocket','success',88,'','Met Para','','','Meteo',428,'',51 from dual union all
select 19497,'1969-W1050','1969-08-06 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',51 from dual union all
select 19498,'1969-W1051','1969-08-06 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 19499,'1969-W1052','1969-08-06 16:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1363,'',52 from dual union all
select 195,'1944-S177','1944-10-05 07:38:00','military missile','success',1470,'18798','Ma205','','','Test',661,'',90 from dual union all
select 1950,'1953-A58','1953-02-13 00:00:00','atmospheric rocket','unknown',576,'96','','','','Test',1895,'',10 from dual union all
select 19500,'1969-W1053','1969-08-06 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 19501,'1969-W1054','1969-08-06 21:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 19502,'1969-W1055','1969-08-07 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 19503,'1969-W1056','1969-08-07 04:46:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 19504,'1969-W1057','1969-08-07 20:32:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',52 from dual union all
select 19505,'1969-W1058','1969-08-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 19506,'1969-W1059','1969-08-08 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',59 from dual union all
select 19507,'1969-W106','1969-01-20 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 19508,'1969-W1060','1969-08-08 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',64 from dual union all
select 19509,'1969-W1061','1969-08-08 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',35,'',59 from dual union all
select 1951,'1953-A59','1953-02-13 00:00:00','atmospheric rocket','unknown',576,'95','','','','Test',1895,'',10 from dual union all
select 19510,'1969-W1062','1969-08-08 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',64 from dual union all
select 19511,'1969-W1063','1969-08-08 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 19512,'1969-W1064','1969-08-08 22:04:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',58 from dual union all
select 19513,'1969-W1065','1969-08-09 01:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 19514,'1969-W1066','1969-08-11 14:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 19515,'1969-W1067','1969-08-11 16:19:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',62 from dual union all
select 19516,'1969-W1068','1969-08-11 18:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',65 from dual union all
select 19517,'1969-W1069','1969-08-11 22:12:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',58 from dual union all
select 19518,'1969-W107','1969-01-20 19:36:59','suborbital rocket','success',88,'','Met Para','','','Meteo',428,'',50 from dual union all
select 19519,'1969-W1070','1969-08-12 21:02:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',51 from dual union all
select 1952,'1953-A60','1953-02-13 00:00:00','atmospheric rocket','unknown',576,'94','','','','Test',1895,'',10 from dual union all
select 19520,'1969-W1071','1969-08-12 22:03:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',57 from dual union all
select 19521,'1969-W1072','1969-08-13 12:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',55 from dual union all
select 19522,'1969-W1073','1969-08-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 19523,'1969-W1074','1969-08-13 14:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',35,'',62 from dual union all
select 19524,'1969-W1075','1969-08-13 15:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1363,'',55 from dual union all
select 19525,'1969-W1076','1969-08-13 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',64 from dual union all
select 19526,'1969-W1077','1969-08-13 18:36:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',61 from dual union all
select 19527,'1969-W1078','1969-08-13 19:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 19528,'1969-W1079','1969-08-13 19:56:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',57 from dual union all
select 19529,'1969-W108','1969-01-20 20:25:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',617,'',53 from dual union all
select 1953,'1953-A61','1953-02-13 00:00:00','atmospheric rocket','success',836,'1249A-183','Nike 1249 R&D 173R','','','Test',1895,'',10 from dual union all
select 19530,'1969-W1080','1969-08-14 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 19531,'1969-W1081','1969-08-14 16:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 19532,'1969-W1082','1969-08-15 12:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',59 from dual union all
select 19533,'1969-W1083','1969-08-15 14:30:00','suborbital rocket','unknown',631,'SUPARCO','','','','Aeron',1419,'',65 from dual union all
select 19534,'1969-W1084','1969-08-15 14:35:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',63 from dual union all
select 19535,'1969-W1085','1969-08-15 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',63 from dual union all
select 19536,'1969-W1086','1969-08-15 16:54:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',63 from dual union all
select 19537,'1969-W1087','1969-08-15 18:03:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',71 from dual union all
select 19538,'1969-W1088','1969-08-15 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual;
insert into LAUNCH
select 19539,'1969-W1089','1969-08-15 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 1954,'1953-A62','1953-02-13 00:00:00','atmospheric rocket','success',836,'1249B','Nike Eng 9E','','','Test',1895,'',10 from dual union all
select 19540,'1969-W109','1969-01-20 20:26:00','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',57 from dual union all
select 19541,'1969-W1090','1969-08-15 21:26:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',52 from dual union all
select 19542,'1969-W1091','1969-08-16 02:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 19543,'1969-W1092','1969-08-18 06:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',60 from dual union all
select 19544,'1969-W1093','1969-08-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 19545,'1969-W1094','1969-08-18 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',65 from dual union all
select 19546,'1969-W1095','1969-08-18 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',64 from dual union all
select 19547,'1969-W1096','1969-08-18 16:49:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',64 from dual union all
select 19548,'1969-W1097','1969-08-18 16:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 19549,'1969-W1098','1969-08-18 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',68 from dual union all
select 1955,'1953-A63','1953-02-13 00:00:00','atmospheric rocket','success',836,'1249A-185','Nike 1249 R&D 174R','','','Test',1895,'',10 from dual union all
select 19550,'1969-W1099','1969-08-18 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 19551,'1969-W11','1969-01-03 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',56 from dual union all
select 19552,'1969-W110','1969-01-20 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 19553,'1969-W1100','1969-08-18 21:42:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',59 from dual union all
select 19554,'1969-W1101','1969-08-19 01:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 19555,'1969-W1102','1969-08-19 14:13:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 19556,'1969-W1103','1969-08-19 17:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 19557,'1969-W1104','1969-08-19 18:19:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',58 from dual union all
select 19558,'1969-W1105','1969-08-20 10:57:00','suborbital rocket','success',631,'ISRO 11.36','ISRO 11.36','TERLS-123','','Aeron',1467,'',64 from dual union all
select 19559,'1969-W1106','1969-08-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 1956,'1953-A64','1953-02-17 00:00:00','military missile','success',771,'GM-11048','','','','Test',222,'',12 from dual union all
select 19560,'1969-W1107','1969-08-20 14:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 19561,'1969-W1108','1969-08-20 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 19562,'1969-W1109','1969-08-20 15:29:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1470,'',60 from dual union all
select 19563,'1969-W111','1969-01-21 16:55:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',55 from dual union all
select 19564,'1969-W1110','1969-08-20 15:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',62 from dual union all
select 19565,'1969-W1111','1969-08-20 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',59 from dual union all
select 19566,'1969-W1112','1969-08-20 17:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',77 from dual union all
select 19567,'1969-W1113','1969-08-20 18:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',54 from dual union all
select 19568,'1969-W1114','1969-08-21 00:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',56 from dual union all
select 19569,'1969-W1115','1969-08-21 01:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 1957,'1953-A65','1953-02-17 00:00:00','atmospheric rocket','success',836,'1249A-166','Nike 1249 R&D 175R','','','Test',1895,'',10 from dual union all
select 19570,'1969-W1116','1969-08-21 15:24:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',54 from dual union all
select 19571,'1969-W1117','1969-08-21 16:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 19572,'1969-W1118','1969-08-22 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 19573,'1969-W1119','1969-08-22 05:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 19574,'1969-W112','1969-01-21 17:00:00','suborbital rocket','success',88,'','','','','Aeron',1363,'',77 from dual union all
select 19575,'1969-W1120','1969-08-22 12:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',65 from dual union all
select 19576,'1969-W1121','1969-08-22 13:58:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 19577,'1969-W1122','1969-08-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 19578,'1969-W1123','1969-08-22 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',66 from dual union all
select 19579,'1969-W1124','1969-08-22 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 1958,'1953-A66','1953-02-17 00:00:00','atmospheric rocket','success',836,'1249B-1100','Nike 1249 R&D 176R','','','Test',1895,'',10 from dual union all
select 19580,'1969-W1125','1969-08-22 18:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 19581,'1969-W1126','1969-08-22 19:04:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',617,'',58 from dual union all
select 19582,'1969-W1127','1969-08-22 20:41:00','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',63 from dual union all
select 19583,'1969-W1128','1969-08-23 00:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',54 from dual union all
select 19584,'1969-W1129','1969-08-23 01:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 19585,'1969-W113','1969-01-21 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',58 from dual union all
select 19586,'1969-W1130','1969-08-25 16:41:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 19587,'1969-W1131','1969-08-25 17:02:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',60 from dual union all
select 19588,'1969-W1132','1969-08-25 17:45:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1363,'',62 from dual union all
select 19589,'1969-W1133','1969-08-25 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',68 from dual union all
select 1959,'1953-A67','1953-02-17 00:00:00','atmospheric rocket','success',836,'1249B-1090','Nike 1249 R&D 177R','','','Test',1895,'',10 from dual union all
select 19590,'1969-W1134','1969-08-25 19:19:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',617,'',57 from dual union all
select 19591,'1969-W1135','1969-08-26 00:27:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',55 from dual union all
select 19592,'1969-W1136','1969-08-26 01:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 19593,'1969-W1137','1969-08-26 14:37:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',62 from dual union all
select 19594,'1969-W1138','1969-08-26 15:19:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',59 from dual union all
select 19595,'1969-W1139','1969-08-26 16:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1363,'',58 from dual union all
select 19596,'1969-W114','1969-01-21 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',50 from dual union all
select 19597,'1969-W1140','1969-08-26 18:11:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',57 from dual union all
select 19598,'1969-W1141','1969-08-27 00:02:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',58 from dual union all
select 19599,'1969-W1142','1969-08-27 03:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 196,'1944-S178','1944-10-05 13:30:00','military missile','success',1470,'18792','Ma199','','','Test',661,'',90 from dual union all
select 1960,'1953-A68','1953-02-17 00:00:00','atmospheric rocket','success',836,'1249A-180','Nike 1249 R&D 178R','','','Test',1895,'',10 from dual union all
select 19600,'1969-W1143','1969-08-27 13:46:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1470,'',60 from dual union all
select 19601,'1969-W1144','1969-08-27 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 19602,'1969-W1145','1969-08-27 14:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 19603,'1969-W1146','1969-08-27 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',63 from dual union all
select 19604,'1969-W1147','1969-08-27 16:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',61 from dual union all
select 19605,'1969-W1148','1969-08-27 17:15:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 19606,'1969-W1149','1969-08-27 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',617,'',58 from dual union all
select 19607,'1969-W115','1969-01-21 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 19608,'1969-W1150','1969-08-27 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',55 from dual union all
select 19609,'1969-W1151','1969-08-27 19:19:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 1961,'1953-A69','1953-02-19 00:00:00','atmospheric rocket','success',836,'1249B','Nike Eng 10E','','','Test',1895,'',10 from dual union all
select 19610,'1969-W1152','1969-08-28 01:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 19611,'1969-W1153','1969-08-28 18:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',64 from dual union all
select 19612,'1969-W1154','1969-08-29 00:08:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',50 from dual union all
select 19613,'1969-W1155','1969-08-29 11:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 19614,'1969-W1156','1969-08-29 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 19615,'1969-W1157','1969-08-29 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',62 from dual union all
select 19616,'1969-W1158','1969-08-29 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',62 from dual union all
select 19617,'1969-W1159','1969-08-29 16:54:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 19618,'1969-W116','1969-01-21 19:01:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 19619,'1969-W1160','1969-08-29 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 1962,'1953-A70','1953-02-20 00:00:00','military missile','unknown',339,'52R','','','','Test',1895,'',30 from dual union all
select 19620,'1969-W1161','1969-08-29 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 19621,'1969-W1162','1969-08-30 00:09:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',55 from dual union all
select 19622,'1969-W1163','1969-09-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 19623,'1969-W1164','1969-09-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 19624,'1969-W1165','1969-09-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 19625,'1969-W1166','1969-09-02 12:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',66 from dual union all
select 19626,'1969-W1167','1969-09-02 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',57 from dual union all
select 19627,'1969-W1168','1969-09-03 00:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',54 from dual union all
select 19628,'1969-W1169','1969-09-03 03:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual;
insert into LAUNCH
select 19629,'1969-W117','1969-01-22 10:56:59','suborbital rocket','success',631,'INTA H-6902','','','','Meteo',396,'',62 from dual union all
select 1963,'1953-A71','1953-02-20 00:00:00','atmospheric rocket','unknown',576,'198','','','','Test',1895,'',10 from dual union all
select 19630,'1969-W1170','1969-09-03 13:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',67 from dual union all
select 19631,'1969-W1171','1969-09-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 19632,'1969-W1172','1969-09-03 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',59 from dual union all
select 19633,'1969-W1173','1969-09-03 16:09:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',55 from dual union all
select 19634,'1969-W1174','1969-09-03 16:49:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 19635,'1969-W1175','1969-09-03 17:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',50 from dual union all
select 19636,'1969-W1176','1969-09-03 21:41:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 19637,'1969-W1177','1969-09-04 02:24:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 19638,'1969-W1178','1969-09-04 14:24:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 19639,'1969-W1179','1969-09-04 16:02:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1057,'',61 from dual union all
select 1964,'1953-A72','1953-02-20 00:00:00','atmospheric rocket','success',836,'308P','Nike CET 8','','','Test',1895,'',10 from dual union all
select 19640,'1969-W118','1969-01-22 11:00:00','suborbital rocket','success',88,'','','','','Aeron',1363,'',76 from dual union all
select 19641,'1969-W1180','1969-09-04 18:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 19642,'1969-W1181','1969-09-05 00:03:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',53 from dual union all
select 19643,'1969-W1182','1969-09-05 14:36:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 19644,'1969-W1183','1969-09-05 14:45:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',55 from dual union all
select 19645,'1969-W1184','1969-09-05 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',59 from dual union all
select 19646,'1969-W1185','1969-09-05 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 19647,'1969-W1186','1969-09-05 17:40:00','suborbital rocket','success',1061,'','Met Sphere,Datasonde','','','Meteo',1363,'',59 from dual union all
select 19648,'1969-W1187','1969-09-05 18:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',60 from dual union all
select 19649,'1969-W1188','1969-09-05 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',617,'',61 from dual union all
select 1965,'1953-A73','1953-02-20 00:00:00','atmospheric rocket','success',836,'309P','Nike CET 9','','','Test',1895,'',10 from dual union all
select 19650,'1969-W1189','1969-09-05 21:24:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',62 from dual union all
select 19651,'1969-W119','1969-01-22 11:10:00','suborbital rocket','unknown',88,'','','','','Aeron',1363,'',74 from dual union all
select 19652,'1969-W1190','1969-09-06 00:06:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',55 from dual union all
select 19653,'1969-W1191','1969-09-06 01:14:59','suborbital rocket','success',1061,'','Starute','','','Meteo',873,'',58 from dual union all
select 19654,'1969-W1192','1969-09-06 02:00:00','suborbital rocket','success',759,'MT-135-041','','','','Aeron',839,'',59 from dual union all
select 19655,'1969-W1193','1969-09-06 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 19656,'1969-W1194','1969-09-07 02:00:00','suborbital rocket','success',761,'MT-135-P -002','','','','Aeron',839,'',53 from dual union all
select 19657,'1969-W1195','1969-09-07 05:00:00','suborbital rocket','success',759,'MT-135-042','','','','Aeron',839,'',57 from dual union all
select 19658,'1969-W1196','1969-09-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 19659,'1969-W1197','1969-09-08 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',64 from dual union all
select 1966,'1953-A74','1953-02-24 00:00:00','military missile','success',703,'612','','','','Test',222,'',8 from dual union all
select 19660,'1969-W1198','1969-09-08 16:14:59','suborbital rocket','success',1061,'','Met Sphere,Datasonde','','','Meteo',1363,'',59 from dual union all
select 19661,'1969-W1199','1969-09-08 18:58:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',56 from dual union all
select 19662,'1969-W12','1969-01-03 15:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',57 from dual union all
select 19663,'1969-W120','1969-01-22 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 19664,'1969-W1200','1969-09-08 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 19665,'1969-W1201','1969-09-08 19:40:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',617,'',53 from dual union all
select 19666,'1969-W1202','1969-09-09 01:33:59','suborbital rocket','success',1061,'','Starute','','','Meteo',873,'',59 from dual union all
select 19667,'1969-W1203','1969-09-09 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',63 from dual union all
select 19668,'1969-W1204','1969-09-09 15:31:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',58 from dual union all
select 19669,'1969-W1205','1969-09-09 16:25:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',62 from dual union all
select 1967,'1953-A75','1953-02-25 00:00:00','atmospheric rocket','unknown',576,'346','','','','Test',1895,'',10 from dual union all
select 19670,'1969-W1206','1969-09-09 17:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 19671,'1969-W1207','1969-09-09 17:20:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1057,'',59 from dual union all
select 19672,'1969-W1208','1969-09-10 06:42:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 19673,'1969-W1209','1969-09-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 19674,'1969-W121','1969-01-22 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',60 from dual union all
select 19675,'1969-W1210','1969-09-10 14:30:00','suborbital rocket','unknown',631,'SUPARCO','','','','Aeron',1419,'',63 from dual union all
select 19676,'1969-W1211','1969-09-10 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',64 from dual union all
select 19677,'1969-W1212','1969-09-10 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 19678,'1969-W1213','1969-09-10 19:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 19679,'1969-W1214','1969-09-10 21:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',51 from dual union all
select 1968,'1953-A76','1953-02-25 00:00:00','atmospheric rocket','unknown',576,'306','','','','Test',1895,'',10 from dual union all
select 19680,'1969-W1215','1969-09-11 01:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 19681,'1969-W1216','1969-09-11 02:15:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 19682,'1969-W1217','1969-09-11 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',61 from dual union all
select 19683,'1969-W1218','1969-09-11 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 19684,'1969-W1219','1969-09-11 21:04:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',52 from dual union all
select 19685,'1969-W122','1969-01-22 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',60 from dual union all
select 19686,'1969-W1220','1969-09-11 22:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',64 from dual union all
select 19687,'1969-W1221','1969-09-12 01:24:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 19688,'1969-W1222','1969-09-12 14:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 19689,'1969-W1223','1969-09-12 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',64 from dual union all
select 1969,'1953-A77','1953-02-25 00:00:00','atmospheric rocket','success',836,'1249A-195','Nike 1249 R&D 179R','','','Test',1895,'',10 from dual union all
select 19690,'1969-W1224','1969-09-12 15:08:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 19691,'1969-W1225','1969-09-12 15:59:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1470,'',61 from dual union all
select 19692,'1969-W1226','1969-09-12 16:14:59','suborbital rocket','success',1061,'','Met Sphere,Datasonde','','','Meteo',1363,'',64 from dual union all
select 19693,'1969-W1227','1969-09-12 17:00:00','suborbital rocket','success',1061,'','Starute,Arcasonde','','','Meteo',1895,'',68 from dual union all
select 19694,'1969-W1228','1969-09-12 18:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',54 from dual union all
select 19695,'1969-W1229','1969-09-12 18:18:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 19696,'1969-W123','1969-01-22 15:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1363,'',57 from dual union all
select 19697,'1969-W1230','1969-09-12 19:09:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',617,'',58 from dual union all
select 19698,'1969-W1231','1969-09-12 22:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',61 from dual union all
select 19699,'1969-W1232','1969-09-13 01:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 197,'1944-S179','1944-10-05 14:53:00','military missile','success',1470,'18797','Ma206','','','Test',661,'',90 from dual union all
select 1970,'1953-A78','1953-02-25 00:00:00','atmospheric rocket','success',836,'1042E','','','','Test',1895,'',10 from dual union all
select 19700,'1969-W1233','1969-09-13 11:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 19701,'1969-W1234','1969-09-13 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 19702,'1969-W1235','1969-09-15 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 19703,'1969-W1236','1969-09-15 12:59:59','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',60 from dual union all
select 19704,'1969-W1237','1969-09-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 19705,'1969-W1238','1969-09-15 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',61 from dual union all
select 19706,'1969-W1239','1969-09-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',58 from dual union all
select 19707,'1969-W124','1969-01-22 17:00:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 19708,'1969-W1240','1969-09-15 16:00:59','suborbital rocket','success',1061,'','Met Sphere,Datasonde','','','Meteo',1363,'',57 from dual union all
select 19709,'1969-W1241','1969-09-15 16:54:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 1971,'1953-A79','1953-02-25 00:00:00','atmospheric rocket','success',836,'1038E','','','','Test',1895,'',10 from dual union all
select 19710,'1969-W1242','1969-09-15 17:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 19711,'1969-W1243','1969-09-15 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',66 from dual union all
select 19712,'1969-W1244','1969-09-15 19:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',66 from dual union all
select 19713,'1969-W1245','1969-09-15 21:33:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',62 from dual union all
select 19714,'1969-W1246','1969-09-16 04:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 19715,'1969-W1247','1969-09-16 12:59:59','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',56 from dual union all
select 19716,'1969-W1248','1969-09-16 14:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',57 from dual union all
select 19717,'1969-W1249','1969-09-16 17:15:00','suborbital rocket','success',88,'','Arcas C-H-10','','','Test',428,'',66 from dual union all
select 19718,'1969-W125','1969-01-22 17:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',57 from dual;
insert into LAUNCH
select 19719,'1969-W1250','1969-09-16 17:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 1972,'1953-A80','1953-02-26 00:00:00','military missile','unknown',339,'51E/1251','EU-2','','','Test',1895,'',30 from dual union all
select 19720,'1969-W1251','1969-09-16 19:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 19721,'1969-W1252','1969-09-16 19:54:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',54 from dual union all
select 19722,'1969-W1253','1969-09-17 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',51 from dual union all
select 19723,'1969-W1254','1969-09-17 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',63 from dual union all
select 19724,'1969-W1255','1969-09-17 16:49:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 19725,'1969-W1256','1969-09-17 16:49:59','suborbital rocket','success',1061,'','Met Sphere,Datasonde','','','Meteo',1363,'',64 from dual union all
select 19726,'1969-W1257','1969-09-17 16:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 19727,'1969-W1258','1969-09-17 17:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',53 from dual union all
select 19728,'1969-W1259','1969-09-17 18:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',62 from dual union all
select 19729,'1969-W126','1969-01-22 19:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 1973,'1953-A81','1953-02-26 00:00:00','atmospheric rocket','success',836,'310P','Nike CET 10','','','Test',1895,'',10 from dual union all
select 19730,'1969-W1260','1969-09-17 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',59 from dual union all
select 19731,'1969-W1261','1969-09-17 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',67 from dual union all
select 19732,'1969-W1262','1969-09-17 20:42:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',51 from dual union all
select 19733,'1969-W1263','1969-09-17 22:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 19734,'1969-W1264','1969-09-18 04:24:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',308,'',59 from dual union all
select 19735,'1969-W1265','1969-09-18 13:44:59','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',56 from dual union all
select 19736,'1969-W1266','1969-09-18 14:30:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',59 from dual union all
select 19737,'1969-W1267','1969-09-18 18:00:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1470,'',68 from dual union all
select 19738,'1969-W1268','1969-09-18 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 19739,'1969-W1269','1969-09-18 22:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',68 from dual union all
select 1974,'1953-A82','1953-02-26 00:00:00','atmospheric rocket','success',836,'311P','Nike CET 11','','','Test',1895,'',10 from dual union all
select 19740,'1969-W127','1969-01-22 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',69 from dual union all
select 19741,'1969-W1270','1969-09-19 14:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 19742,'1969-W1271','1969-09-19 14:45:00','suborbital rocket','success',1061,'','Met Para,Met Sphere','','','Meteo',176,'',61 from dual union all
select 19743,'1969-W1272','1969-09-19 16:53:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 19744,'1969-W1273','1969-09-19 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 19745,'1969-W1274','1969-09-19 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',61 from dual union all
select 19746,'1969-W1275','1969-09-19 21:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',63 from dual union all
select 19747,'1969-W1276','1969-09-20 10:07:00','suborbital rocket','success',725,'','','','','Meteo',1621,'SHIP-SOV',77 from dual union all
select 19748,'1969-W1277','1969-09-20 13:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',66 from dual union all
select 19749,'1969-W1278','1969-09-22 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',60 from dual union all
select 1975,'1953-A83','1953-02-26 00:00:00','atmospheric rocket','success',836,'1021E','','','','Test',1895,'',10 from dual union all
select 19750,'1969-W1279','1969-09-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',70 from dual union all
select 19751,'1969-W128','1969-01-22 22:53:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',52 from dual union all
select 19752,'1969-W1280','1969-09-22 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',63 from dual union all
select 19753,'1969-W1281','1969-09-22 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 19754,'1969-W1282','1969-09-22 17:06:00','suborbital rocket','success',1061,'','Met Sphere,Datasonde','','','Meteo',1363,'',63 from dual union all
select 19755,'1969-W1283','1969-09-22 18:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',65 from dual union all
select 19756,'1969-W1284','1969-09-22 18:34:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 19757,'1969-W1285','1969-09-22 19:11:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',58 from dual union all
select 19758,'1969-W1286','1969-09-22 21:14:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',58 from dual union all
select 19759,'1969-W1287','1969-09-22 22:06:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',54 from dual union all
select 1976,'1953-A84','1953-02-27 00:00:00','atmospheric rocket','unknown',576,'347','','','','Test',1895,'',10 from dual union all
select 19760,'1969-W1288','1969-09-22 22:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',50 from dual union all
select 19761,'1969-W1289','1969-09-23 00:45:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',55 from dual union all
select 19762,'1969-W129','1969-01-22 23:00:00','suborbital rocket','unknown',88,'','','','','Aeron',1363,'',77 from dual union all
select 19763,'1969-W1290','1969-09-23 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 19764,'1969-W1291','1969-09-23 16:30:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',62 from dual union all
select 19765,'1969-W1292','1969-09-23 16:32:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',50 from dual union all
select 19766,'1969-W1293','1969-09-23 21:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',60 from dual union all
select 19767,'1969-W1294','1969-09-23 21:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',53 from dual union all
select 19768,'1969-W1295','1969-09-24 14:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 19769,'1969-W1296','1969-09-24 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',63 from dual union all
select 1977,'1953-A85','1953-02-27 00:00:00','atmospheric rocket','success',836,'1249A-165','Nike 1249 R&D 180R','','','Test',1895,'',10 from dual union all
select 19770,'1969-W1297','1969-09-24 15:02:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',58 from dual union all
select 19771,'1969-W1298','1969-09-24 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 19772,'1969-W1299','1969-09-24 17:03:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 19773,'1969-W13','1969-01-03 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',65 from dual union all
select 19774,'1969-W130','1969-01-23 15:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',54 from dual union all
select 19775,'1969-W1300','1969-09-24 17:48:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',66 from dual union all
select 19776,'1969-W1301','1969-09-24 18:08:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',64 from dual union all
select 19777,'1969-W1302','1969-09-24 21:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 19778,'1969-W1303','1969-09-25 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 19779,'1969-W1304','1969-09-25 15:34:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',54 from dual union all
select 1978,'1953-A86','1953-02-27 00:00:00','atmospheric rocket','success',836,'1249A-162','Nike 1249 R&D 181R','','','Test',1895,'',10 from dual union all
select 19780,'1969-W1305','1969-09-26 04:24:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 19781,'1969-W1306','1969-09-26 16:22:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',56 from dual union all
select 19782,'1969-W1307','1969-09-26 17:44:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',55 from dual union all
select 19783,'1969-W1308','1969-09-26 17:56:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',60 from dual union all
select 19784,'1969-W1309','1969-09-26 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 19785,'1969-W131','1969-01-23 17:55:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',53 from dual union all
select 19786,'1969-W1310','1969-09-26 19:18:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',60 from dual union all
select 19787,'1969-W1311','1969-09-26 19:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 19788,'1969-W1312','1969-09-26 19:46:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',53 from dual union all
select 19789,'1969-W1313','1969-09-26 20:08:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',56 from dual union all
select 1979,'1953-A87','1953-02-27 00:00:00','atmospheric rocket','success',836,'1249A-186','Nike 1249 R&D 182R','','','Test',1895,'',10 from dual union all
select 19790,'1969-W1314','1969-09-26 21:33:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',60 from dual union all
select 19791,'1969-W1315','1969-09-27 02:05:00','suborbital rocket','success',761,'MT-135-P -003','','','','Aeron',839,'',52 from dual union all
select 19792,'1969-W1316','1969-09-27 02:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 19793,'1969-W1317','1969-09-29 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',63 from dual union all
select 19794,'1969-W1318','1969-09-29 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 19795,'1969-W1319','1969-09-29 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 19796,'1969-W132','1969-01-23 19:27:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',58 from dual union all
select 19797,'1969-W1320','1969-09-29 16:51:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 19798,'1969-W1321','1969-09-29 18:02:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',67 from dual union all
select 19799,'1969-W1322','1969-09-29 19:40:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',64 from dual union all
select 198,'1944-S18','1944-03-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 1980,'1953-A88','1953-03-02 00:00:00','atmospheric rocket','unknown',576,'1SP','','','','Test',1895,'',10 from dual union all
select 19800,'1969-W1323','1969-09-29 20:45:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',56 from dual union all
select 19801,'1969-W1324','1969-09-29 21:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 19802,'1969-W1325','1969-10-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 19803,'1969-W1326','1969-09-30 18:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 19804,'1969-W1327','1969-09-30 20:47:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',54 from dual union all
select 19805,'1969-W1328','1969-09-30 22:38:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',54 from dual union all
select 19806,'1969-W1329','1969-10-01 00:26:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 19807,'1969-W133','1969-01-23 23:45:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',61 from dual union all
select 19808,'1969-W1330','1969-10-01 15:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual;
insert into LAUNCH
select 19809,'1969-W1331','1969-10-01 15:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 1981,'1953-A89','1953-03-03 00:00:00','military missile','success',703,'611','','','','Test',222,'',8 from dual union all
select 19810,'1969-W1332','1969-10-01 16:47:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 19811,'1969-W1333','1969-10-01 17:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 19812,'1969-W1334','1969-10-01 21:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 19813,'1969-W1335','1969-10-02 00:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 19814,'1969-W1336','1969-10-02 02:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',65 from dual union all
select 19815,'1969-W1337','1969-10-02 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 19816,'1969-W1338','1969-10-02 18:45:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',76 from dual union all
select 19817,'1969-W1339','1969-10-02 20:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',56 from dual union all
select 19818,'1969-W134','1969-01-24 02:33:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',60 from dual union all
select 19819,'1969-W1340','1969-10-03 02:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',67 from dual union all
select 1982,'1953-A90','1953-03-03 00:00:00','atmospheric rocket','success',836,'1013E','','','','Test',1895,'',10 from dual union all
select 19820,'1969-W1341','1969-10-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 19821,'1969-W1342','1969-10-03 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',59 from dual union all
select 19822,'1969-W1343','1969-10-03 16:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 19823,'1969-W1344','1969-10-03 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 19824,'1969-W1345','1969-10-03 18:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',63 from dual union all
select 19825,'1969-W1346','1969-10-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 19826,'1969-W1347','1969-10-03 21:02:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',50 from dual union all
select 19827,'1969-W1348','1969-10-04 01:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 19828,'1969-W1349','1969-10-06 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 19829,'1969-W135','1969-01-24 10:54:59','suborbital rocket','success',631,'INTA H-6903','','','','Meteo',396,'',58 from dual union all
select 1983,'1953-A91','1953-03-03 00:00:00','atmospheric rocket','success',836,'1249A-169','Nike 1249 R&D 183R','','','Test',1895,'',10 from dual union all
select 19830,'1969-W1350','1969-10-06 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',63 from dual union all
select 19831,'1969-W1351','1969-10-06 16:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 19832,'1969-W1352','1969-10-06 16:10:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 19833,'1969-W1353','1969-10-06 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 19834,'1969-W1354','1969-10-06 21:21:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',56 from dual union all
select 19835,'1969-W1355','1969-10-07 01:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 19836,'1969-W1356','1969-10-07 20:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 19837,'1969-W1357','1969-10-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 19838,'1969-W1358','1969-10-08 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 19839,'1969-W1359','1969-10-08 16:51:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 1984,'1953-A92','1953-03-03 00:00:00','atmospheric rocket','success',836,'1249A-168','Nike 1249 R&D 184R','','','Test',1895,'',10 from dual union all
select 19840,'1969-W136','1969-01-24 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',59 from dual union all
select 19841,'1969-W1360','1969-10-08 17:04:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1057,'',62 from dual union all
select 19842,'1969-W1361','1969-10-08 17:36:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 19843,'1969-W1362','1969-10-08 18:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 19844,'1969-W1363','1969-10-08 18:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',61 from dual union all
select 19845,'1969-W1364','1969-10-08 21:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 19846,'1969-W1365','1969-10-10 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 19847,'1969-W1366','1969-10-10 14:26:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',51 from dual union all
select 19848,'1969-W1367','1969-10-10 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 19849,'1969-W1368','1969-10-10 15:02:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',51 from dual union all
select 1985,'1953-A93','1953-03-04 00:00:00','military missile','success',771,'GM-2358','','','','Test',222,'',12 from dual union all
select 19850,'1969-W1369','1969-10-10 15:08:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',51 from dual union all
select 19851,'1969-W137','1969-01-24 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',60 from dual union all
select 19852,'1969-W1370','1969-10-10 15:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',54 from dual union all
select 19853,'1969-W1371','1969-10-10 15:27:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',56 from dual union all
select 19854,'1969-W1372','1969-10-10 16:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',60 from dual union all
select 19855,'1969-W1373','1969-10-10 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 19856,'1969-W1374','1969-10-10 17:04:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1057,'',65 from dual union all
select 19857,'1969-W1375','1969-10-10 17:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 19858,'1969-W1376','1969-10-10 19:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',62 from dual union all
select 19859,'1969-W1377','1969-10-11 00:53:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 1986,'1953-A94','1953-03-05 00:00:00','atmospheric rocket','success',442,'','Falcon Missile','','','Test',1821,'',10 from dual union all
select 19860,'1969-W1378','1969-10-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 19861,'1969-W1379','1969-10-13 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',55 from dual union all
select 19862,'1969-W138','1969-01-24 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 19863,'1969-W1380','1969-10-13 14:25:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',63 from dual union all
select 19864,'1969-W1381','1969-10-13 14:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',59 from dual union all
select 19865,'1969-W1382','1969-10-13 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 19866,'1969-W1383','1969-10-13 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 19867,'1969-W1384','1969-10-13 21:04:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',59 from dual union all
select 19868,'1969-W1385','1969-10-13 21:38:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',62 from dual union all
select 19869,'1969-W1386','1969-10-13 23:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 1987,'1953-A95','1953-03-05 00:00:00','atmospheric rocket','unknown',576,'344','','','','Test',1895,'',10 from dual union all
select 19870,'1969-W1387','1969-10-14 15:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1057,'',55 from dual union all
select 19871,'1969-W1388','1969-10-14 16:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',64 from dual union all
select 19872,'1969-W1389','1969-10-14 21:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',57 from dual union all
select 19873,'1969-W139','1969-01-24 17:56:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 19874,'1969-W1390','1969-10-15 10:49:59','suborbital rocket','success',631,'INTA H-6913','','','','Meteo',396,'',62 from dual union all
select 19875,'1969-W1391','1969-10-15 12:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',66 from dual union all
select 19876,'1969-W1392','1969-10-15 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 19877,'1969-W1393','1969-10-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 19878,'1969-W1394','1969-10-15 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 19879,'1969-W1395','1969-10-15 16:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 1988,'1953-A96','1953-03-05 00:00:00','atmospheric rocket','success',836,'312P','Nike CET 12','','','Test',1895,'',10 from dual union all
select 19880,'1969-W1396','1969-10-15 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 19881,'1969-W1397','1969-10-15 17:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 19882,'1969-W1398','1969-10-15 17:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',63 from dual union all
select 19883,'1969-W1399','1969-10-15 18:03:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',59 from dual union all
select 19884,'1969-W14','1969-01-03 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',62 from dual union all
select 19885,'1969-W140','1969-01-24 18:00:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1895,'',70 from dual union all
select 19886,'1969-W1400','1969-10-15 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',50 from dual union all
select 19887,'1969-W1401','1969-10-16 03:27:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 19888,'1969-W1402','1969-10-16 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',53 from dual union all
select 19889,'1969-W1403','1969-10-16 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',64 from dual union all
select 1989,'1953-A97','1953-03-05 00:00:00','atmospheric rocket','success',836,'313P','Nike CET 13','','','Test',1895,'',10 from dual union all
select 19890,'1969-W1404','1969-10-16 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',69 from dual union all
select 19891,'1969-W1405','1969-10-16 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 19892,'1969-W1406','1969-10-16 17:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',69 from dual union all
select 19893,'1969-W1407','1969-10-16 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 19894,'1969-W1408','1969-10-17 14:13:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 19895,'1969-W1409','1969-10-17 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 19896,'1969-W141','1969-01-24 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',69 from dual union all
select 19897,'1969-W1410','1969-10-17 15:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',54 from dual union all
select 19898,'1969-W1411','1969-10-17 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',60 from dual union all
select 19899,'1969-W1412','1969-10-17 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',64 from dual;
insert into LAUNCH
select 199,'1944-S180','1944-10-05 15:30:00','military missile','success',1470,'18803','Ma207','','','Test',661,'',90 from dual union all
select 1990,'1953-A98','1953-03-05 00:00:00','atmospheric rocket','success',836,'1249A-175','Nike 1249 R&D 186R','','','Test',1895,'',10 from dual union all
select 19900,'1969-W1413','1969-10-17 18:12:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 19901,'1969-W1414','1969-10-17 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',59 from dual union all
select 19902,'1969-W1415','1969-10-17 19:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 19903,'1969-W1416','1969-10-17 19:42:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',55 from dual union all
select 19904,'1969-W1417','1969-10-17 22:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 19905,'1969-W1418','1969-10-18 01:54:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 19906,'1969-W1419','1969-10-20 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 19907,'1969-W142','1969-01-24 20:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 19908,'1969-W1420','1969-10-20 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 19909,'1969-W1421','1969-10-20 14:49:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',50 from dual union all
select 1991,'1953-A99','1953-03-05 00:00:00','atmospheric rocket','success',836,'1050E','','','','Test',1895,'',10 from dual union all
select 19910,'1969-W1422','1969-10-20 21:42:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',64 from dual union all
select 19911,'1969-W1423','1969-10-21 03:22:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 19912,'1969-W1424','1969-10-21 15:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',55 from dual union all
select 19913,'1969-W1425','1969-10-21 21:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 19914,'1969-W1426','1969-10-22 03:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 19915,'1969-W1427','1969-10-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 19916,'1969-W1428','1969-10-22 14:01:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',59 from dual union all
select 19917,'1969-W1429','1969-10-22 14:10:00','suborbital rocket','unknown',631,'SUPARCO-047/69','','','','Aeron',1419,'',63 from dual union all
select 19918,'1969-W143','1969-01-24 20:37:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',55 from dual union all
select 19919,'1969-W1430','1969-10-22 14:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',61 from dual union all
select 1992,'1953-M01','1953-02-11 00:00:00','military missile','success',1046,'1','Hermes A2 engine','','','Test',293,'',50 from dual union all
select 19920,'1969-W1431','1969-10-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 19921,'1969-W1432','1969-10-22 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',57 from dual union all
select 19922,'1969-W1433','1969-10-22 15:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 19923,'1969-W1434','1969-10-22 17:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 19924,'1969-W1435','1969-10-22 19:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',67 from dual union all
select 19925,'1969-W1436','1969-10-22 22:17:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',52 from dual union all
select 19926,'1969-W1437','1969-10-23 02:45:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 19927,'1969-W1438','1969-10-23 14:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',62 from dual union all
select 19928,'1969-W1439','1969-10-23 16:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 19929,'1969-W144','1969-01-27 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',56 from dual union all
select 1993,'1953-M02','1953-03-04 00:00:00','military missile','failure',1046,'2','[Impact Vero Beach]','','','Test',293,'',20 from dual union all
select 19930,'1969-W1440','1969-10-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 19931,'1969-W1441','1969-10-24 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',55 from dual union all
select 19932,'1969-W1442','1969-10-24 15:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 19933,'1969-W1443','1969-10-24 18:00:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 19934,'1969-W1444','1969-10-27 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',64 from dual union all
select 19935,'1969-W1445','1969-10-27 14:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 19936,'1969-W1446','1969-10-27 15:31:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',59 from dual union all
select 19937,'1969-W1447','1969-10-27 15:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 19938,'1969-W1448','1969-10-27 16:22:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',59 from dual union all
select 19939,'1969-W1449','1969-10-27 17:20:00','suborbital rocket','success',1061,'','Met Sphere,Datasonde','','','Meteo',1363,'',58 from dual union all
select 1994,'1953-M03','1953-03-25 00:00:00','military missile','success',1046,'3','','','','Test',293,'',50 from dual union all
select 19940,'1969-W145','1969-01-27 14:57:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 19941,'1969-W1450','1969-10-27 17:51:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 19942,'1969-W1451','1969-10-27 18:35:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',63 from dual union all
select 19943,'1969-W1452','1969-10-27 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 19944,'1969-W1453','1969-10-27 19:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 19945,'1969-W1454','1969-10-28 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',66 from dual union all
select 19946,'1969-W1455','1969-10-28 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 19947,'1969-W1456','1969-10-28 22:04:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',53 from dual union all
select 19948,'1969-W1457','1969-10-29 14:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 19949,'1969-W1458','1969-10-29 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 1995,'1953-M04','1953-03-25 00:00:00','military missile','success',1046,'4','','','','Test',293,'',50 from dual union all
select 19950,'1969-W1459','1969-10-29 16:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',69 from dual union all
select 19951,'1969-W146','1969-01-27 15:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',51 from dual union all
select 19952,'1969-W1460','1969-10-29 17:49:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 19953,'1969-W1461','1969-10-29 18:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 19954,'1969-W1462','1969-10-29 19:20:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',51 from dual union all
select 19955,'1969-W1463','1969-10-29 19:47:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',66 from dual union all
select 19956,'1969-W1464','1969-10-30 02:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',65 from dual union all
select 19957,'1969-W1465','1969-10-30 17:59:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',53 from dual union all
select 19958,'1969-W1466','1969-10-30 22:01:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',53 from dual union all
select 19959,'1969-W1467','1969-10-31 11:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',66 from dual union all
select 1996,'1953-M05','1953-08-05 21:54:00','suborbital rocket','success',370,'MUSHRAT 9','NRL Rockoon 1','','','Aeron',40,'AGB-5',80 from dual union all
select 19960,'1969-W1468','1969-10-31 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',50 from dual union all
select 19961,'1969-W1469','1969-10-31 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 19962,'1969-W147','1969-01-27 17:48:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',60 from dual union all
select 19963,'1969-W1470','1969-10-31 15:08:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',63 from dual union all
select 19964,'1969-W1471','1969-10-31 15:34:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',60 from dual union all
select 19965,'1969-W1472','1969-10-31 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 19966,'1969-W1473','1969-10-31 16:07:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 19967,'1969-W1474','1969-10-31 16:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 19968,'1969-W1475','1969-10-31 20:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',63 from dual union all
select 19969,'1969-W1476','1969-10-31 22:08:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 1997,'1953-M06','1953-08-08 15:09:00','suborbital rocket','success',370,'MUSHRAT 12','NRL Rockoon 2','','','Aeron',41,'AGB-5',80 from dual union all
select 19970,'1969-W1477','1969-11-02 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',65 from dual union all
select 19971,'1969-W1478','1969-11-03 04:54:59','suborbital rocket','success',631,'Starute,Datasonde','PCA69 Met','','','Meteo',428,'',63 from dual union all
select 19972,'1969-W1479','1969-11-03 05:35:00','suborbital rocket','success',88,'','PCA69 H-01','','','Aeron',428,'',66 from dual union all
select 19973,'1969-W148','1969-01-27 19:03:59','suborbital rocket','success',1061,'','','','','Meteo',1895,'',61 from dual union all
select 19974,'1969-W1480','1969-11-03 07:00:00','suborbital rocket','success',88,'','PCA69 H-02','','','Aeron',428,'',67 from dual union all
select 19975,'1969-W1481','1969-11-03 12:28:00','suborbital rocket','success',88,'','PCA69 H-03','','','Aeron',428,'',64 from dual union all
select 19976,'1969-W1482','1969-11-03 12:59:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',56 from dual union all
select 19977,'1969-W1483','1969-11-03 13:10:59','suborbital rocket','success',631,'Starute,Datasonde','PCA69 Met','','','Meteo',428,'',64 from dual union all
select 19978,'1969-W1484','1969-11-03 14:05:00','suborbital rocket','success',88,'','PCA69 H-04','','','Aeron',428,'',65 from dual union all
select 19979,'1969-W1485','1969-11-03 16:29:00','suborbital rocket','success',88,'','PCA69 H-05','','','Aeron',428,'',66 from dual union all
select 1998,'1953-M07','1953-08-11 17:09:00','suborbital rocket','success',370,'MUSHRAT 15','NRL Rockoon 4','','','Aeron',43,'AGB-5',80 from dual union all
select 19980,'1969-W1486','1969-11-03 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 19981,'1969-W1487','1969-11-03 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 19982,'1969-W1488','1969-11-03 18:08:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',61 from dual union all
select 19983,'1969-W1489','1969-11-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 19984,'1969-W149','1969-01-27 20:01:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',59 from dual union all
select 19985,'1969-W1490','1969-11-03 19:01:59','suborbital rocket','success',631,'Starute,Datasonde','PCA69 Met','','','Meteo',428,'',63 from dual union all
select 19986,'1969-W1491','1969-11-03 21:30:00','suborbital rocket','success',631,'Starute,Datasonde','PCA69 Met','','','Meteo',428,'',62 from dual union all
select 19987,'1969-W1492','1969-11-04 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 19988,'1969-W1493','1969-11-04 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',52 from dual union all
select 19989,'1969-W1494','1969-11-04 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual;
insert into LAUNCH
select 1999,'1953-M08','1953-09-04 03:59:00','suborbital rocket','success',370,'Eastwind 54-7','NRL Rockoon 5','','','Aeron',58,'WYG-279',70 from dual union all
select 19990,'1969-W1495','1969-11-04 20:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 19991,'1969-W1496','1969-11-04 22:20:00','suborbital rocket','success',88,'','PCA69 H-07','','','Aeron',428,'',70 from dual union all
select 19992,'1969-W1497','1969-11-04 23:25:00','suborbital rocket','success',88,'','PCA69 H-08','','','Aeron',428,'',68 from dual union all
select 19993,'1969-W1498','1969-11-04 23:57:00','suborbital rocket','success',88,'','PCA69 H-09','','','Aeron',428,'',70 from dual union all
select 19994,'1969-W1499','1969-11-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 19995,'1969-W15','1969-01-03 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',64 from dual union all
select 19996,'1969-W150','1969-01-27 20:55:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 19997,'1969-W1500','1969-11-05 15:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',51 from dual union all
select 19998,'1969-W1501','1969-11-05 15:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 19999,'1969-W1502','1969-11-05 16:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 2,'1942-A02','1942-08-16 11:15:00','military missile','failure',8,'3','','','','Test',675,'',11 from dual union all
select 20,'1943-A13','1943-11-05 00:00:00','military missile','failure',8,'','Heidelager Test','','','Test',662,'',1 from dual union all
select 200,'1944-S181','1944-10-05 18:31:00','military missile','success',1470,'18791','Ma200','','','Test',661,'',90 from dual union all
select 2000,'1953-M09','1953-09-04 15:51:00','suborbital rocket','success',370,'Eastwind 54-8','NRL Rockoon 6','','','Aeron',59,'WYG-279',70 from dual union all
select 20000,'1969-W1503','1969-11-05 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',61 from dual union all
select 20001,'1969-W1504','1969-11-05 18:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',62 from dual union all
select 20002,'1969-W1505','1969-11-05 19:27:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',53 from dual union all
select 20003,'1969-W1506','1969-11-05 21:17:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',62 from dual union all
select 20004,'1969-W1507','1969-11-05 21:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',59 from dual union all
select 20005,'1969-W1508','1969-11-06 01:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 20006,'1969-W1509','1969-11-06 19:30:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1057,'',61 from dual union all
select 20007,'1969-W151','1969-01-28 06:49:59','suborbital rocket','success',631,'','Starute','','','Meteo',428,'',61 from dual union all
select 20008,'1969-W1510','1969-11-07 03:09:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 20009,'1969-W1511','1969-11-07 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 2001,'1953-M10','1953-09-29 20:50:00','suborbital rocket','success',51,'SC 31','Sphere','','','Aeronomy',1869,'',76 from dual union all
select 20010,'1969-W1512','1969-11-07 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 20011,'1969-W1513','1969-11-07 16:12:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 20012,'1969-W1514','1969-11-07 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 20013,'1969-W1515','1969-11-07 18:09:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',58 from dual union all
select 20014,'1969-W1516','1969-11-07 19:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',65 from dual union all
select 20015,'1969-W1517','1969-11-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',54 from dual union all
select 20016,'1969-W1518','1969-11-10 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 20017,'1969-W1519','1969-11-10 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',50 from dual union all
select 20018,'1969-W152','1969-01-28 08:37:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',55 from dual union all
select 20019,'1969-W1520','1969-11-10 21:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 2002,'1953-S01','1953-02-10 21:09:00','suborbital rocket','success',54,'NRL 12','Mass spectrometer','','','Aeronomy/CR',1869,'',137 from dual union all
select 20020,'1969-W1521','1969-11-12 11:20:00','suborbital rocket','success',631,'INTA H-6914','','','','Meteo',396,'',62 from dual union all
select 20021,'1969-W1522','1969-11-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 20022,'1969-W1523','1969-11-12 15:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 20023,'1969-W1524','1969-11-12 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',58 from dual union all
select 20024,'1969-W1525','1969-11-12 19:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 20025,'1969-W1526','1969-11-12 19:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',54 from dual union all
select 20026,'1969-W1527','1969-11-12 19:53:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',60 from dual union all
select 20027,'1969-W1528','1969-11-13 02:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',65 from dual union all
select 20028,'1969-W1529','1969-11-13 04:49:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',54 from dual union all
select 20029,'1969-W153','1969-01-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 2003,'1953-S02','1953-02-12 07:09:00','suborbital rocket','success',54,'NRL 13','Mass spectrometer','','','Aeronomy/CR',1869,'',137 from dual union all
select 20030,'1969-W1530','1969-11-13 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 20031,'1969-W1531','1969-11-13 15:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',52 from dual union all
select 20032,'1969-W1532','1969-11-13 20:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 20033,'1969-W1533','1969-11-14 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 20034,'1969-W1534','1969-11-14 18:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',54 from dual union all
select 20035,'1969-W1535','1969-11-14 19:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',60 from dual union all
select 20036,'1969-W1536','1969-11-14 20:02:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',54 from dual union all
select 20037,'1969-W1537','1969-11-15 04:23:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 20038,'1969-W1538','1969-11-17 14:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 20039,'1969-W1539','1969-11-17 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 2004,'1953-S03','1953-02-18 06:50:00','suborbital rocket','success',60,'SC 27','Grenades','','','Aeronomy',1869,'',109 from dual union all
select 20040,'1969-W154','1969-01-28 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 20041,'1969-W1540','1969-11-17 15:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',59 from dual union all
select 20042,'1969-W1541','1969-11-17 16:30:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',52 from dual union all
select 20043,'1969-W1542','1969-11-17 17:15:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1363,'',68 from dual union all
select 20044,'1969-W1543','1969-11-17 17:47:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 20045,'1969-W1544','1969-11-17 18:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 20046,'1969-W1545','1969-11-17 18:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',59 from dual union all
select 20047,'1969-W1546','1969-11-17 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 20048,'1969-W1547','1969-11-17 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 20049,'1969-W1548','1969-11-18 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 2005,'1953-S04','1953-02-18 17:42:00','suborbital rocket','success',51,'USAF 34','Heating/beacon test','','','Test',640,'',117 from dual union all
select 20050,'1969-W1549','1969-11-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 20051,'1969-W155','1969-01-29 10:54:59','suborbital rocket','success',631,'INTA H-6904','','','','Meteo',396,'',50 from dual union all
select 20052,'1969-W1550','1969-11-18 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 20053,'1969-W1551','1969-11-18 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 20054,'1969-W1552','1969-11-18 17:51:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 20055,'1969-W1553','1969-11-19 14:35:00','suborbital rocket','unknown',631,'SUPARCO-048/69','','','','Aeron',1419,'',64 from dual union all
select 20056,'1969-W1554','1969-11-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 20057,'1969-W1555','1969-11-19 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 20058,'1969-W1556','1969-11-19 16:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1363,'',56 from dual union all
select 20059,'1969-W1557','1969-11-19 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 2006,'1953-S05','1953-03-01 00:00:00','military missile','success',970,'No 1020','','','','OT',636,'',100 from dual union all
select 20060,'1969-W1558','1969-11-19 17:47:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 20061,'1969-W1559','1969-11-19 17:48:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 20062,'1969-W156','1969-01-29 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',57 from dual union all
select 20063,'1969-W1560','1969-11-19 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',65 from dual union all
select 20064,'1969-W1561','1969-11-19 22:17:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',60 from dual union all
select 20065,'1969-W1562','1969-11-19 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',53 from dual union all
select 20066,'1969-W1563','1969-11-20 01:31:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',64 from dual union all
select 20067,'1969-W1564','1969-11-20 13:50:00','suborbital rocket','unknown',631,'SUPARCO-049/69','','','','Aeron',1419,'',64 from dual union all
select 20068,'1969-W1565','1969-11-20 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',54 from dual union all
select 20069,'1969-W1566','1969-11-20 15:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',52 from dual union all
select 2007,'1953-S06','1953-03-05 00:00:00','military missile','success',970,'No 319','','','','OT',636,'',100 from dual union all
select 20070,'1969-W1567','1969-11-20 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 20071,'1969-W1568','1969-11-20 21:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 20072,'1969-W1569','1969-11-20 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',55 from dual union all
select 20073,'1969-W157','1969-01-29 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 20074,'1969-W1570','1969-11-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 20075,'1969-W1571','1969-11-21 15:02:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 20076,'1969-W1572','1969-11-21 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 20077,'1969-W1573','1969-11-21 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual;
insert into LAUNCH
select 20078,'1969-W1574','1969-11-21 22:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 20079,'1969-W1575','1969-11-22 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 2008,'1953-S07','1953-03-19 00:00:00','military missile','success',970,'No 820','','','','OT',636,'',100 from dual union all
select 20080,'1969-W1576','1969-11-24 17:47:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 20081,'1969-W1577','1969-11-24 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 20082,'1969-W1578','1969-11-24 22:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 20083,'1969-W1579','1969-11-25 17:49:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 20084,'1969-W158','1969-01-29 15:15:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1895,'',68 from dual union all
select 20085,'1969-W1580','1969-11-25 18:22:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',62 from dual union all
select 20086,'1969-W1581','1969-11-25 20:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 20087,'1969-W1582','1969-11-25 20:14:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',58 from dual union all
select 20088,'1969-W1583','1969-11-25 21:38:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 20089,'1969-W1584','1969-11-25 22:31:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',57 from dual union all
select 2009,'1953-S08','1953-04-19 00:00:00','military missile','success',1009,'','8A62 LKI I','','','Test',636,'',300 from dual union all
select 20090,'1969-W1585','1969-11-26 06:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 20091,'1969-W1586','1969-11-26 11:03:59','suborbital rocket','success',631,'INTA H-6915','','','','Meteo',396,'',65 from dual union all
select 20092,'1969-W1587','1969-11-26 15:48:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 20093,'1969-W1588','1969-11-26 15:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',61 from dual union all
select 20094,'1969-W1589','1969-11-26 17:44:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 20095,'1969-W159','1969-01-29 15:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1363,'',64 from dual union all
select 20096,'1969-W1590','1969-11-26 17:53:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 20097,'1969-W1591','1969-11-26 19:09:59','suborbital rocket','success',1188,'','','','','Meteo',1447,'',50 from dual union all
select 20098,'1969-W1592','1969-11-26 20:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 20099,'1969-W1593','1969-11-26 21:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 201,'1944-S182','1944-10-05 20:19:00','military missile','success',1470,'18790','Ma198','','','Test',661,'',90 from dual union all
select 2010,'1953-S09','1953-04-02 00:00:00','military missile','success',1009,'','8A62 LKI I','','','Test',636,'',300 from dual union all
select 20100,'1969-W1594','1969-11-26 22:02:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',62 from dual union all
select 20101,'1969-W1595','1969-11-27 02:13:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 20102,'1969-W1596','1969-11-27 17:43:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 20103,'1969-W1597','1969-11-28 18:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',62 from dual union all
select 20104,'1969-W1598','1969-11-29 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 20105,'1969-W1599','1969-11-29 17:43:00','suborbital rocket','success',1188,'','','','','Meteo',1447,'',60 from dual union all
select 20106,'1969-W16','1969-01-03 20:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',60 from dual union all
select 20107,'1969-W160','1969-01-29 17:55:00','suborbital rocket','success',631,'','Starute','','','Meteo',428,'',54 from dual union all
select 20108,'1969-W1600','1969-12-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 20109,'1969-W1601','1969-12-01 14:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',57 from dual union all
select 2011,'1953-S10','1953-04-14 15:47:00','suborbital rocket','success',51,'USAF 35','Heating test','','','Test',640,'',122 from dual union all
select 20110,'1969-W1602','1969-12-01 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 20111,'1969-W1603','1969-12-01 15:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',59 from dual union all
select 20112,'1969-W1604','1969-12-01 15:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 20113,'1969-W1605','1969-12-01 16:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',54 from dual union all
select 20114,'1969-W1606','1969-12-01 20:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 20115,'1969-W1607','1969-12-01 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 20116,'1969-W1608','1969-12-02 17:47:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 20117,'1969-W1609','1969-12-02 18:30:00','suborbital rocket','success',1188,'','','','','Meteo',1447,'',60 from dual union all
select 20118,'1969-W161','1969-01-29 18:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',61 from dual union all
select 20119,'1969-W1610','1969-12-02 22:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',57 from dual union all
select 2012,'1953-S11','1953-04-18 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 20120,'1969-W1611','1969-12-02 22:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 20121,'1969-W1612','1969-12-03 09:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 20122,'1969-W1613','1969-12-03 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 20123,'1969-W1614','1969-12-03 14:33:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 20124,'1969-W1615','1969-12-03 15:18:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 20125,'1969-W1616','1969-12-03 16:07:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1057,'',57 from dual union all
select 20126,'1969-W1617','1969-12-03 20:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',58 from dual union all
select 20127,'1969-W1618','1969-12-04 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',65 from dual union all
select 20128,'1969-W1619','1969-12-04 16:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 20129,'1969-W162','1969-01-29 18:37:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',60 from dual union all
select 2013,'1953-S12','1953-04-24 10:19:00','suborbital rocket','success',60,'SC 28','Grenade','','','Aeronomy',1869,'',108 from dual union all
select 20130,'1969-W1620','1969-12-04 17:13:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 20131,'1969-W1621','1969-12-04 17:19:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 20132,'1969-W1622','1969-12-04 18:17:00','suborbital rocket','success',1188,'','','','','Meteo',1447,'',54 from dual union all
select 20133,'1969-W1623','1969-12-04 21:22:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',60 from dual union all
select 20134,'1969-W1624','1969-12-05 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 20135,'1969-W1625','1969-12-05 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 20136,'1969-W1626','1969-12-05 15:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',50 from dual union all
select 20137,'1969-W1627','1969-12-05 18:00:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 20138,'1969-W1628','1969-12-05 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 20139,'1969-W1629','1969-12-05 19:30:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',60 from dual union all
select 2014,'1953-S13','1953-04-23 19:33:00','suborbital rocket','success',60,'SC 30','Sphere','','','Aeronomy',1869,'',124 from dual union all
select 20140,'1969-W163','1969-01-29 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',53 from dual union all
select 20141,'1969-W1630','1969-12-05 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 20142,'1969-W1631','1969-12-06 00:00:00','suborbital rocket','unknown',88,'','','','','Aeron',1895,'',60 from dual union all
select 20143,'1969-W1632','1969-12-06 02:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 20144,'1969-W1633','1969-12-08 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',58 from dual union all
select 20145,'1969-W1634','1969-12-08 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 20146,'1969-W1635','1969-12-08 16:54:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 20147,'1969-W1636','1969-12-08 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 20148,'1969-W1637','1969-12-08 17:54:00','suborbital rocket','success',1188,'','','','','Meteo',1447,'',60 from dual union all
select 20149,'1969-W1638','1969-12-08 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 2015,'1953-S14','1953-04-28 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 20150,'1969-W1639','1969-12-08 22:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',64 from dual union all
select 20151,'1969-W164','1969-01-29 19:31:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',65 from dual union all
select 20152,'1969-W1640','1969-12-09 01:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 20153,'1969-W1641','1969-12-09 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 20154,'1969-W1642','1969-12-09 19:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1057,'',55 from dual union all
select 20155,'1969-W1643','1969-12-10 00:45:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 20156,'1969-W1644','1969-12-10 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',63 from dual union all
select 20157,'1969-W1645','1969-12-10 15:13:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 20158,'1969-W1646','1969-12-10 16:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',64 from dual union all
select 20159,'1969-W1647','1969-12-10 17:18:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1057,'',60 from dual union all
select 2016,'1953-S15','1953-04-24 00:00:00','military missile','failure',1009,'','8A62 LKI I','','','Test',636,'',300 from dual union all
select 20160,'1969-W1648','1969-12-10 18:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',55 from dual union all
select 20161,'1969-W1649','1969-12-10 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 20162,'1969-W165','1969-01-29 19:44:59','suborbital rocket','success',1061,'','Starute','','','Meteo',617,'',52 from dual union all
select 20163,'1969-W1650','1969-12-10 23:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 20164,'1969-W1651','1969-12-11 01:03:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 20165,'1969-W1652','1969-12-11 01:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 20166,'1969-W1653','1969-12-11 15:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 20167,'1969-W1654','1969-12-11 18:00:00','suborbital rocket','success',1188,'','','','','Meteo',1447,'',61 from dual;
insert into LAUNCH
select 20168,'1969-W1655','1969-12-11 21:45:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',55 from dual union all
select 20169,'1969-W1656','1969-12-12 00:00:00','suborbital rocket','unknown',88,'','','','','Aeron',1895,'',60 from dual union all
select 2017,'1953-S16','1953-05-13 00:00:00','military missile','success',1009,'','8A62 LKI I','','','Test',636,'',300 from dual union all
select 20170,'1969-W1657','1969-12-12 02:33:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 20171,'1969-W1658','1969-12-12 13:04:00','suborbital rocket','success',1188,'Su24H','','','','Ionospheric',1447,'',79 from dual union all
select 20172,'1969-W1659','1969-12-12 14:12:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',61 from dual union all
select 20173,'1969-W166','1969-01-29 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',60 from dual union all
select 20174,'1969-W1660','1969-12-12 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 20175,'1969-W1661','1969-12-12 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',58 from dual union all
select 20176,'1969-W1662','1969-12-12 14:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 20177,'1969-W1663','1969-12-12 18:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',54 from dual union all
select 20178,'1969-W1664','1969-12-12 18:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 20179,'1969-W1665','1969-12-12 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 2018,'1953-S17','1953-05-11 00:00:00','military missile','success',970,'No 220','','','','OT',636,'',100 from dual union all
select 20180,'1969-W1666','1969-12-13 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 20181,'1969-W1667','1969-12-13 02:02:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 20182,'1969-W1668','1969-12-13 03:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',57 from dual union all
select 20183,'1969-W1669','1969-12-15 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',62 from dual union all
select 20184,'1969-W167','1969-01-29 20:12:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',58 from dual union all
select 20185,'1969-W1670','1969-12-15 15:27:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',61 from dual union all
select 20186,'1969-W1671','1969-12-15 16:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 20187,'1969-W1672','1969-12-15 16:29:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',64 from dual union all
select 20188,'1969-W1673','1969-12-15 17:45:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1363,'',56 from dual union all
select 20189,'1969-W1674','1969-12-15 17:53:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 2019,'1953-S18','1953-05-20 14:04:00','suborbital rocket','success',51,'USAF 36','AF Airglow 3','','','Aeronomy SB',640,'',114 from dual union all
select 20190,'1969-W1675','1969-12-15 18:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',51 from dual union all
select 20191,'1969-W1676','1969-12-15 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 20192,'1969-W1677','1969-12-15 18:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 20193,'1969-W1678','1969-12-15 21:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 20194,'1969-W1679','1969-12-16 01:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 20195,'1969-W168','1969-01-29 20:44:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',64 from dual union all
select 20196,'1969-W1680','1969-12-16 17:26:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 20197,'1969-W1681','1969-12-16 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 20198,'1969-W1682','1969-12-16 17:48:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 20199,'1969-W1683','1969-12-16 20:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 202,'1944-S183','1944-10-06 00:00:00','military missile','success',1470,'','Paris','','','Weapon',1739,'',90 from dual union all
select 2020,'1953-S19','1953-05-21 15:47:00','suborbital rocket','success',51,'USAF 37','AF Airglow 4','','','Aeronomy SB',640,'',114 from dual union all
select 20200,'1969-W1684','1969-12-16 21:11:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',50 from dual union all
select 20201,'1969-W1685','1969-12-17 13:10:00','suborbital rocket','unknown',631,'SUPARCO-050/69','','','','Aeron',1419,'',63 from dual union all
select 20202,'1969-W1686','1969-12-17 14:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 20203,'1969-W1687','1969-12-17 15:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 20204,'1969-W1688','1969-12-17 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',63 from dual union all
select 20205,'1969-W1689','1969-12-17 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',55 from dual union all
select 20206,'1969-W169','1969-01-29 20:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',66 from dual union all
select 20207,'1969-W1690','1969-12-17 17:38:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',52 from dual union all
select 20208,'1969-W1691','1969-12-17 17:56:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 20209,'1969-W1692','1969-12-17 19:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 2021,'1953-S20','1953-05-23 00:00:00','military missile','success',1009,'','8A62 LKI I','','','Test',636,'',300 from dual union all
select 20210,'1969-W1693','1969-12-17 22:17:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',58 from dual union all
select 20211,'1969-W1694','1969-12-18 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 20212,'1969-W1695','1969-12-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 20213,'1969-W1696','1969-12-18 16:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',53 from dual union all
select 20214,'1969-W1697','1969-12-18 17:47:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 20215,'1969-W1698','1969-12-18 21:04:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',55 from dual union all
select 20216,'1969-W1699','1969-12-19 00:00:00','suborbital rocket','unknown',88,'','','','','Aeron',1895,'',60 from dual union all
select 20217,'1969-W17','1969-01-03 23:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 20218,'1969-W170','1969-01-29 20:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',68 from dual union all
select 20219,'1969-W1700','1969-12-19 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',59 from dual union all
select 2022,'1953-S21','1953-06-03 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 20220,'1969-W1701','1969-12-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 20221,'1969-W1702','1969-12-19 15:46:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',58 from dual union all
select 20222,'1969-W1703','1969-12-19 17:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 20223,'1969-W1704','1969-12-19 17:35:00','suborbital rocket','success',1188,'','','','','Meteo',1447,'',62 from dual union all
select 20224,'1969-W1705','1969-12-19 17:47:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 20225,'1969-W1706','1969-12-19 18:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',55 from dual union all
select 20226,'1969-W1707','1969-12-19 20:15:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',56 from dual union all
select 20227,'1969-W1708','1969-12-20 02:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 20228,'1969-W1709','1969-12-22 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',64 from dual union all
select 20229,'1969-W171','1969-01-30 03:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',308,'',65 from dual union all
select 2023,'1953-S22','1953-06-26 19:10:00','suborbital rocket','success',51,'USAF 38','AF/Utah Ionos 3','','','Ionos',640,'',135 from dual union all
select 20230,'1969-W1710','1969-12-22 17:48:00','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',62 from dual union all
select 20231,'1969-W1711','1969-12-22 18:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 20232,'1969-W1712','1969-12-22 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',50 from dual union all
select 20233,'1969-W1713','1969-12-22 21:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 20234,'1969-W1714','1969-12-23 01:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 20235,'1969-W1715','1969-12-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 20236,'1969-W1716','1969-12-23 17:20:00','suborbital rocket','success',1188,'','','','','Meteo',1447,'',61 from dual union all
select 20237,'1969-W1717','1969-12-23 21:24:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',54 from dual union all
select 20238,'1969-W1718','1969-12-24 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',62 from dual union all
select 20239,'1969-W1719','1969-12-24 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 2024,'1953-S23','1953-07-01 17:52:00','suborbital rocket','success',51,'USAF 39','AF/Utah Ionos 4','','','Ionos',640,'',138 from dual union all
select 20240,'1969-W172','1969-01-30 20:06:00','suborbital rocket','success',631,'','Starute','','','Meteo',428,'',54 from dual union all
select 20241,'1969-W1720','1969-12-24 16:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 20242,'1969-W1721','1969-12-24 17:48:59','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',55 from dual union all
select 20243,'1969-W1722','1969-12-24 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 20244,'1969-W1723','1969-12-24 21:18:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',60 from dual union all
select 20245,'1969-W1724','1969-12-25 01:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 20246,'1969-W1725','1969-12-26 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 20247,'1969-W1726','1969-12-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 20248,'1969-W1727','1969-12-27 17:36:00','suborbital rocket','success',1188,'','','','','Meteo',1447,'',55 from dual union all
select 20249,'1969-W1728','1969-12-29 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',67 from dual union all
select 2025,'1953-S24','1953-07-06 00:00:00','military missile','success',970,'No 420','','','','OT',636,'',100 from dual union all
select 20250,'1969-W1729','1969-12-29 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 20251,'1969-W173','1969-01-30 20:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',60 from dual union all
select 20252,'1969-W1730','1969-12-29 18:46:59','suborbital rocket','success',1188,'','','','','Meteo',1447,'',61 from dual union all
select 20253,'1969-W1731','1969-12-29 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',64 from dual union all
select 20254,'1969-W1732','1969-12-29 22:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 20255,'1969-W1733','1969-12-30 01:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 20256,'1969-W1734','1969-12-30 17:45:00','suborbital rocket','success',1188,'','','','','Meteo',1447,'',63 from dual union all
select 20257,'1969-W1735','1969-12-30 19:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',65 from dual;
insert into LAUNCH
select 20258,'1969-W1736','1969-12-30 23:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 20259,'1969-W1737','1969-12-31 06:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',58 from dual union all
select 2026,'1953-S25','1953-07-14 15:30:00','suborbital rocket','success',51,'USAF 40','Solar flux','','','Solar UV',640,'',103 from dual union all
select 20260,'1969-W1738','1969-12-31 13:15:00','suborbital rocket','unknown',631,'SUPARCO-051/69','','','','Aeron',1419,'',75 from dual union all
select 20261,'1969-W1739','1969-12-31 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',66 from dual union all
select 20262,'1969-W174','1969-01-30 20:51:00','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',55 from dual union all
select 20263,'1969-W1740','1969-12-31 16:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 20264,'1969-W1741','1969-12-31 18:41:00','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',61 from dual union all
select 20265,'1969-W1742','1969-12-31 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 20266,'1969-W1743','1969-12-31 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 20267,'1969-W1744','1969-12-31 20:48:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 20268,'1969-W1745','1969-12-31 21:46:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',55 from dual union all
select 20269,'1969-W1746','1969-03-13 18:17:00','suborbital rocket','success',725,'','Particle flux 18','','','Plasma',1010,'',80 from dual union all
select 2027,'1953-S26','1953-07-23 09:47:00','suborbital rocket','success',51,'USAF 41','Airglow spectrograph','','','Aeronomy',640,'',95 from dual union all
select 20270,'1969-W1747','1969-06-18 19:00:00','suborbital rocket','success',88,'','PEPP support','','','Weather',1895,'',60 from dual union all
select 20271,'1969-W175','1969-01-30 22:04:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',61 from dual union all
select 20272,'1969-W176','1969-02-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 20273,'1969-W177','1969-02-01 00:00:00','suborbital rocket','success',631,'','','','','Aeron',428,'',60 from dual union all
select 20274,'1969-W178','1969-01-31 03:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',64 from dual union all
select 20275,'1969-W179','1969-01-31 14:35:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',54 from dual union all
select 20276,'1969-W18','1969-01-04 02:00:00','suborbital rocket','success',759,'MT-135-038','','','','Aeron',839,'',57 from dual union all
select 20277,'1969-W180','1969-01-31 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 20278,'1969-W181','1969-01-31 15:18:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 20279,'1969-W182','1969-01-31 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',55 from dual union all
select 2028,'1953-S27','1953-07-28 09:41:00','suborbital rocket','success',370,'MUSHRAT 6','SUI 13','','','CR',47,'AGB-5',90 from dual union all
select 20280,'1969-W183','1969-01-31 18:21:00','suborbital rocket','success',88,'NASA 15.61DM','','','','Plasma',1821,'',54 from dual union all
select 20281,'1969-W184','1969-01-31 18:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',60 from dual union all
select 20282,'1969-W185','1969-01-31 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',68 from dual union all
select 20283,'1969-W186','1969-01-31 20:47:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',66 from dual union all
select 20284,'1969-W187','1969-01-31 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 20285,'1969-W188','1969-02-01 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',69 from dual union all
select 20286,'1969-W189','1969-02-02 20:17:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',69 from dual union all
select 20287,'1969-W19','1969-01-04 05:00:00','suborbital rocket','success',759,'MT-135-039','','','','Aeron',839,'',59 from dual union all
select 20288,'1969-W190','1969-02-03 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 20289,'1969-W191','1969-02-03 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',64 from dual union all
select 2029,'1953-S28','1953-08-06 18:40:00','suborbital rocket','success',370,'MUSHRAT 11','SUI 16','','','CR',50,'AGB-5',96 from dual union all
select 20290,'1969-W192','1969-02-03 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',61 from dual union all
select 20291,'1969-W193','1969-02-03 17:15:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',58 from dual union all
select 20292,'1969-W194','1969-02-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 20293,'1969-W195','1969-02-03 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',60 from dual union all
select 20294,'1969-W196','1969-02-03 20:34:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 20295,'1969-W197','1969-02-03 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 20296,'1969-W198','1969-02-03 22:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',54 from dual union all
select 20297,'1969-W199','1969-02-04 18:55:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',58 from dual union all
select 20298,'1969-W20','1969-01-05 02:00:00','suborbital rocket','success',759,'MT-135-40','','','','Aeron',839,'',59 from dual union all
select 20299,'1969-W200','1969-02-04 19:01:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',60 from dual union all
select 203,'1944-S184','1944-10-06 11:04:00','military missile','success',1470,'19041','WH3','','','Test',661,'',90 from dual union all
select 2030,'1953-S29','1953-08-09 05:54:00','suborbital rocket','success',370,'MUSHRAT 13','SUI 17','','','CR',38,'AGB-5',100 from dual union all
select 20300,'1969-W201','1969-02-04 19:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',51 from dual union all
select 20301,'1969-W202','1969-02-04 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',71 from dual union all
select 20302,'1969-W203','1969-02-04 20:45:00','suborbital rocket','success',1061,'','','','','Meteo',1057,'',61 from dual union all
select 20303,'1969-W204','1969-02-04 22:52:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',56 from dual union all
select 20304,'1969-W205','1969-02-05 13:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 20305,'1969-W206','1969-02-05 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',63 from dual union all
select 20306,'1969-W207','1969-02-05 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',58 from dual union all
select 20307,'1969-W208','1969-02-05 17:51:00','suborbital rocket','success',1061,'','','','','Meteo',1307,'',65 from dual union all
select 20308,'1969-W209','1969-02-05 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 20309,'1969-W21','1969-01-05 16:41:59','suborbital rocket','success',1188,'','','','','Meteo',1447,'',54 from dual union all
select 2031,'1953-S30','1953-08-30 20:46:00','suborbital rocket','success',370,'Eastwind 54-3','SUI 20','','','CR',54,'WYG-279',100 from dual union all
select 20310,'1969-W210','1969-02-05 19:04:59','suborbital rocket','success',1061,'','','','','Meteo',1057,'',56 from dual union all
select 20311,'1969-W211','1969-02-05 20:15:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',67 from dual union all
select 20312,'1969-W212','1969-02-05 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 20313,'1969-W213','1969-02-05 21:41:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',58 from dual union all
select 20314,'1969-W214','1969-02-05 22:19:59','suborbital rocket','success',1061,'','','','','Meteo',1057,'',65 from dual union all
select 20315,'1969-W215','1969-02-06 10:23:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 20316,'1969-W216','1969-02-06 14:38:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',57 from dual union all
select 20317,'1969-W217','1969-02-06 18:00:00','suborbital rocket','success',88,'NASA 15.62DM','','','','Plasma',1821,'',56 from dual union all
select 20318,'1969-W218','1969-02-06 18:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',58 from dual union all
select 20319,'1969-W219','1969-02-06 19:53:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',53 from dual union all
select 2032,'1953-S31','1953-09-01 05:05:00','suborbital rocket','success',60,'SC 32','Grenades','','','Aeronomy',1869,'',108 from dual union all
select 20320,'1969-W22','1969-01-06 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 20321,'1969-W220','1969-02-06 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',58 from dual union all
select 20322,'1969-W221','1969-02-06 21:56:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',64 from dual union all
select 20323,'1969-W222','1969-02-07 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',66 from dual union all
select 20324,'1969-W223','1969-02-07 14:47:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',63 from dual union all
select 20325,'1969-W224','1969-02-07 14:55:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',63 from dual union all
select 20326,'1969-W225','1969-02-07 16:10:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',56 from dual union all
select 20327,'1969-W226','1969-02-07 20:13:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',67 from dual union all
select 20328,'1969-W227','1969-02-07 21:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 20329,'1969-W228','1969-02-07 21:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',67 from dual union all
select 2033,'1953-S32','1953-09-03 09:50:00','suborbital rocket','success',370,'Eastwind 54-4','SUI 21','','','CR',55,'WYG-279',90 from dual union all
select 20330,'1969-W229','1969-02-07 23:15:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',61 from dual union all
select 20331,'1969-W23','1969-01-06 15:00:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',56 from dual union all
select 20332,'1969-W230','1969-02-08 05:46:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 20333,'1969-W231','1969-02-10 14:08:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',62 from dual union all
select 20334,'1969-W232','1969-02-10 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',64 from dual union all
select 20335,'1969-W233','1969-02-10 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',56 from dual union all
select 20336,'1969-W234','1969-02-10 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',68 from dual union all
select 20337,'1969-W235','1969-02-10 19:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 20338,'1969-W236','1969-02-10 20:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 20339,'1969-W237','1969-02-10 20:07:00','suborbital rocket','success',1061,'','','','','Meteo',1057,'',57 from dual union all
select 2034,'1953-S33','1953-09-03 11:51:00','suborbital rocket','success',370,'Eastwind 54-5','SUI 22','','','CR',56,'WYG-279',100 from dual union all
select 20340,'1969-W238','1969-02-10 20:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',56 from dual union all
select 20341,'1969-W239','1969-02-11 17:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',58 from dual union all
select 20342,'1969-W24','1969-01-06 15:22:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',62 from dual union all
select 20343,'1969-W240','1969-02-11 18:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',54 from dual union all
select 20344,'1969-W241','1969-02-11 19:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',57 from dual union all
select 20345,'1969-W242','1969-02-12 00:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',54 from dual union all
select 20346,'1969-W243','1969-02-12 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 20347,'1969-W244','1969-02-12 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',64 from dual;
insert into LAUNCH
select 20348,'1969-W245','1969-02-12 15:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',330,'',67 from dual union all
select 20349,'1969-W246','1969-02-12 16:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',63 from dual union all
select 2035,'1953-S34','1953-09-03 14:05:00','suborbital rocket','success',370,'Eastwind 54-6','SUI 23','','','CR',57,'WYG-279',100 from dual union all
select 20350,'1969-W247','1969-02-12 16:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',54 from dual union all
select 20351,'1969-W248','1969-02-12 18:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',64 from dual union all
select 20352,'1969-W249','1969-02-12 18:38:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',58 from dual union all
select 20353,'1969-W25','1969-01-06 16:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',66 from dual union all
select 20354,'1969-W250','1969-02-12 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',62 from dual union all
select 20355,'1969-W251','1969-02-12 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',55 from dual union all
select 20356,'1969-W252','1969-02-12 21:42:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',57 from dual union all
select 20357,'1969-W253','1969-02-13 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',60 from dual union all
select 20358,'1969-W254','1969-02-13 20:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',57 from dual union all
select 20359,'1969-W255','1969-02-13 20:36:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',59 from dual union all
select 2036,'1953-S35','1953-09-05 05:36:00','suborbital rocket','success',59,'SC 33','Grenades','','','Aeronomy',1869,'',114 from dual union all
select 20360,'1969-W256','1969-02-13 20:59:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',53 from dual union all
select 20361,'1969-W257','1969-02-13 21:21:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',50 from dual union all
select 20362,'1969-W258','1969-02-14 03:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 20363,'1969-W259','1969-02-14 03:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',66 from dual union all
select 20364,'1969-W26','1969-01-06 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',61 from dual union all
select 20365,'1969-W260','1969-02-14 04:09:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',61 from dual union all
select 20366,'1969-W261','1969-02-14 15:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',62 from dual union all
select 20367,'1969-W262','1969-02-14 16:32:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',57 from dual union all
select 20368,'1969-W263','1969-02-14 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',56 from dual union all
select 20369,'1969-W264','1969-02-14 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',64 from dual union all
select 2037,'1953-S36','1953-12-09 00:00:00','military missile','success',992,'530-50-101','NII-88 Production','','','OT',636,'',100 from dual union all
select 20370,'1969-W265','1969-02-14 18:37:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',59 from dual union all
select 20371,'1969-W266','1969-02-14 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 20372,'1969-W267','1969-02-14 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',65 from dual union all
select 20373,'1969-W268','1969-02-14 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 20374,'1969-W269','1969-02-14 22:55:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',57 from dual union all
select 20375,'1969-W27','1969-01-06 20:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',64 from dual union all
select 20376,'1969-W270','1969-02-17 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',63 from dual union all
select 20377,'1969-W271','1969-02-17 15:38:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 20378,'1969-W272','1969-02-17 16:02:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',64 from dual union all
select 20379,'1969-W273','1969-02-17 17:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 2038,'1953-S37','1953-12-21 00:00:00','military missile','success',992,'531-10-320','NII-88 Production','','','OT',636,'',100 from dual union all
select 20380,'1969-W274','1969-02-17 18:56:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',64 from dual union all
select 20381,'1969-W275','1969-02-17 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 20382,'1969-W276','1969-02-17 20:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 20383,'1969-W277','1969-02-17 20:55:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',61 from dual union all
select 20384,'1969-W278','1969-02-18 17:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',61 from dual union all
select 20385,'1969-W279','1969-02-18 21:24:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',61 from dual union all
select 20386,'1969-W28','1969-01-07 16:19:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',58 from dual union all
select 20387,'1969-W280','1969-02-19 04:56:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',53 from dual union all
select 20388,'1969-W281','1969-02-19 13:34:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',62 from dual union all
select 20389,'1969-W282','1969-02-19 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',68 from dual union all
select 2039,'1953-S40','1953-10-07 17:00:00','suborbital rocket','success',51,'USAF 43','Solar spectrum','','','Solar',640,'',100 from dual union all
select 20390,'1969-W283','1969-02-19 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 20391,'1969-W284','1969-02-19 16:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',57 from dual union all
select 20392,'1969-W285','1969-02-19 16:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',62 from dual union all
select 20393,'1969-W286','1969-02-19 18:19:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',55 from dual union all
select 20394,'1969-W287','1969-02-19 18:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 20395,'1969-W288','1969-02-19 18:38:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',63 from dual union all
select 20396,'1969-W289','1969-02-20 00:14:59','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',64 from dual union all
select 20397,'1969-W29','1969-01-07 17:55:00','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',65 from dual union all
select 20398,'1969-W290','1969-02-20 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',63 from dual union all
select 20399,'1969-W291','1969-02-20 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 204,'1944-S185','1944-10-06 13:54:00','military missile','success',1470,'19048','WH4','','','Test',661,'',90 from dual union all
select 2040,'1953-S41','1953-08-29 00:00:00','military missile','success',992,'530-80-118','8Zh38 SP-1','NII-88 Production','','OT',636,'',100 from dual union all
select 20400,'1969-W292','1969-02-20 19:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',61 from dual union all
select 20401,'1969-W293','1969-02-20 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 20402,'1969-W294','1969-02-20 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',72 from dual union all
select 20403,'1969-W295','1969-02-20 21:37:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',58 from dual union all
select 20404,'1969-W296','1969-02-21 17:49:59','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',57 from dual union all
select 20405,'1969-W297','1969-02-21 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',63 from dual union all
select 20406,'1969-W298','1969-02-24 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',64 from dual union all
select 20407,'1969-W299','1969-02-24 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',59 from dual union all
select 20408,'1969-W30','1969-01-07 18:48:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',62 from dual union all
select 20409,'1969-W300','1969-02-24 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',61 from dual union all
select 2041,'1953-S42','1953-10-16 00:00:00','military missile','success',970,'No 820','','','','OT',636,'',100 from dual union all
select 20410,'1969-W301','1969-02-24 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 20411,'1969-W302','1969-02-24 20:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',72 from dual union all
select 20412,'1969-W303','1969-02-24 20:47:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 20413,'1969-W304','1969-02-25 15:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',57 from dual union all
select 20414,'1969-W305','1969-02-25 21:29:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',52 from dual union all
select 20415,'1969-W306','1969-02-26 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',61 from dual union all
select 20416,'1969-W307','1969-02-26 17:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',59 from dual union all
select 20417,'1969-W308','1969-02-26 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',64 from dual union all
select 20418,'1969-W309','1969-02-26 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',66 from dual union all
select 20419,'1969-W31','1969-01-07 18:48:59','suborbital rocket','success',1188,'','','','','Meteo',1447,'',54 from dual union all
select 2042,'1953-S43','1953-10-17 00:00:00','military missile','success',970,'No 520','','','','OT',636,'',100 from dual union all
select 20420,'1969-W310','1969-02-27 03:18:00','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',53 from dual union all
select 20421,'1969-W311','1969-02-27 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',60 from dual union all
select 20422,'1969-W312','1969-02-27 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 20423,'1969-W313','1969-02-27 21:45:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',62 from dual union all
select 20424,'1969-W314','1969-02-28 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',66 from dual union all
select 20425,'1969-W315','1969-02-28 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 20426,'1969-W316','1969-02-28 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',65 from dual union all
select 20427,'1969-W317','1969-02-28 18:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',61 from dual union all
select 20428,'1969-W318','1969-02-28 19:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 20429,'1969-W319','1969-02-28 19:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',60 from dual union all
select 2043,'1953-S44','1953-10-19 00:00:00','military missile','success',970,'No 320','','','','OT',636,'',100 from dual union all
select 20430,'1969-W32','1969-01-08 14:29:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',62 from dual union all
select 20431,'1969-W320','1969-02-28 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',67 from dual union all
select 20432,'1969-W321','1969-02-28 20:14:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 20433,'1969-W322','1969-02-28 21:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',60 from dual union all
select 20434,'1969-W323','1969-03-01 05:48:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',53 from dual union all
select 20435,'1969-W324','1969-03-01 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',60 from dual union all
select 20436,'1969-W325','1969-03-02 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',58 from dual union all
select 20437,'1969-W326','1969-03-03 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',59 from dual;
insert into LAUNCH
select 20438,'1969-W327','1969-03-03 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',62 from dual union all
select 20439,'1969-W328','1969-03-03 16:22:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 2044,'1953-S45','1953-10-20 00:00:00','military missile','success',970,'No 620','','','','OT',636,'',100 from dual union all
select 20440,'1969-W329','1969-03-03 17:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',56 from dual union all
select 20441,'1969-W33','1969-01-08 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',59 from dual union all
select 20442,'1969-W330','1969-03-03 18:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',60 from dual union all
select 20443,'1969-W331','1969-03-03 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',55 from dual union all
select 20444,'1969-W332','1969-03-03 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',61 from dual union all
select 20445,'1969-W333','1969-03-03 20:42:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 20446,'1969-W334','1969-03-04 04:37:59','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',60 from dual union all
select 20447,'1969-W335','1969-03-04 21:10:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',58 from dual union all
select 20448,'1969-W336','1969-03-04 21:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 20449,'1969-W337','1969-03-05 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',67 from dual union all
select 2045,'1953-S46','1953-10-26 00:00:00','military missile','success',970,'No 720','','','','OT',636,'',100 from dual union all
select 20450,'1969-W338','1969-03-05 15:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',60 from dual union all
select 20451,'1969-W339','1969-03-05 16:40:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',60 from dual union all
select 20452,'1969-W34','1969-01-08 15:31:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',59 from dual union all
select 20453,'1969-W340','1969-03-05 17:25:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',64 from dual union all
select 20454,'1969-W341','1969-03-05 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',67 from dual union all
select 20455,'1969-W342','1969-03-05 19:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 20456,'1969-W343','1969-03-05 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',63 from dual union all
select 20457,'1969-W344','1969-03-05 20:38:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',57 from dual union all
select 20458,'1969-W345','1969-03-05 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',56 from dual union all
select 20459,'1969-W346','1969-03-06 02:48:00','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',53 from dual union all
select 2046,'1953-S47','1953-10-27 00:00:00','military missile','success',970,'No 1020','','','','OT',636,'',100 from dual union all
select 20460,'1969-W347','1969-03-06 19:49:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',54 from dual union all
select 20461,'1969-W348','1969-03-06 21:04:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',65 from dual union all
select 20462,'1969-W349','1969-03-07 01:44:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',57 from dual union all
select 20463,'1969-W35','1969-01-08 16:07:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1363,'',58 from dual union all
select 20464,'1969-W350','1969-03-07 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 20465,'1969-W351','1969-03-07 15:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 20466,'1969-W352','1969-03-07 17:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',57 from dual union all
select 20467,'1969-W353','1969-03-07 17:31:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',67 from dual union all
select 20468,'1969-W354','1969-03-07 17:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 20469,'1969-W355','1969-03-07 18:26:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',60 from dual union all
select 2047,'1953-S48','1953-10-28 00:00:00','military missile','success',970,'No 920','','','','OT',636,'',100 from dual union all
select 20470,'1969-W356','1969-03-07 19:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',59 from dual union all
select 20471,'1969-W357','1969-03-07 20:02:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',64 from dual union all
select 20472,'1969-W358','1969-03-07 20:56:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 20473,'1969-W359','1969-03-10 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',64 from dual union all
select 20474,'1969-W36','1969-01-08 17:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',64 from dual union all
select 20475,'1969-W360','1969-03-10 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',60 from dual union all
select 20476,'1969-W361','1969-03-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 20477,'1969-W362','1969-03-10 16:09:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',61 from dual union all
select 20478,'1969-W363','1969-03-10 17:11:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',62 from dual union all
select 20479,'1969-W364','1969-03-10 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',55 from dual union all
select 2048,'1953-S49','1953-10-28 00:00:00','military missile','success',970,'No 1220','','','','OT',636,'',100 from dual union all
select 20480,'1969-W365','1969-03-10 20:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',62 from dual union all
select 20481,'1969-W366','1969-03-10 20:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',58 from dual union all
select 20482,'1969-W367','1969-03-10 20:55:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 20483,'1969-W368','1969-03-10 21:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 20484,'1969-W369','1969-03-11 19:30:00','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',61 from dual union all
select 20485,'1969-W37','1969-01-08 18:07:00','suborbital rocket','success',88,'','Met Para','','','Meteo',428,'',54 from dual union all
select 20486,'1969-W370','1969-03-11 20:24:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',59 from dual union all
select 20487,'1969-W371','1969-03-11 21:29:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',52 from dual union all
select 20488,'1969-W372','1969-03-12 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 20489,'1969-W373','1969-03-12 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',52 from dual union all
select 2049,'1953-S50','1953-10-30 00:00:00','military missile','success',1009,'','8A62 LKI II','','','Test',636,'',300 from dual union all
select 20490,'1969-W374','1969-03-12 15:42:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',54 from dual union all
select 20491,'1969-W375','1969-03-12 15:56:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',330,'',58 from dual union all
select 20492,'1969-W376','1969-03-12 16:52:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',58 from dual union all
select 20493,'1969-W377','1969-03-12 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',60 from dual union all
select 20494,'1969-W378','1969-03-12 18:18:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',308,'',60 from dual union all
select 20495,'1969-W379','1969-03-12 19:12:59','suborbital rocket','success',1061,'','','','','Meteo',1895,'',54 from dual union all
select 20496,'1969-W38','1969-01-08 18:33:00','suborbital rocket','success',1188,'','','','','Meteo',1447,'',60 from dual union all
select 20497,'1969-W380','1969-03-12 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',59 from dual union all
select 20498,'1969-W381','1969-03-12 20:46:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',60 from dual union all
select 20499,'1969-W382','1969-03-12 20:56:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',59 from dual union all
select 205,'1944-S186','1944-10-09 11:04:00','military missile','success',1470,'19045','WH9','','','Test',661,'',90 from dual union all
select 2050,'1953-S51','1953-11-03 00:00:00','military missile','success',1009,'','8A62 LKI II','','','Test',636,'',300 from dual union all
select 20500,'1969-W383','1969-03-12 22:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 20501,'1969-W384','1969-03-13 04:30:00','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',63 from dual union all
select 20502,'1969-W385','1969-03-13 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 20503,'1969-W386','1969-03-13 19:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',58 from dual union all
select 20504,'1969-W387','1969-03-13 20:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 20505,'1969-W388','1969-03-13 21:11:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',64 from dual union all
select 20506,'1969-W389','1969-03-14 02:03:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 20507,'1969-W39','1969-01-09 17:10:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',56 from dual union all
select 20508,'1969-W390','1969-03-14 11:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 20509,'1969-W391','1969-03-14 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 2051,'1953-S52','1953-11-17 00:00:00','military missile','success',1009,'','8A62 LKI II','','','Test',636,'',300 from dual union all
select 20510,'1969-W392','1969-03-14 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',62 from dual union all
select 20511,'1969-W393','1969-03-14 14:48:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',63 from dual union all
select 20512,'1969-W394','1969-03-14 18:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',59 from dual union all
select 20513,'1969-W395','1969-03-14 19:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',50 from dual union all
select 20514,'1969-W396','1969-03-14 20:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',53 from dual union all
select 20515,'1969-W397','1969-03-14 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',62 from dual union all
select 20516,'1969-W398','1969-03-14 20:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 20517,'1969-W399','1969-03-15 02:03:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 20518,'1969-W40','1969-01-09 17:47:00','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',65 from dual union all
select 20519,'1969-W400','1969-03-17 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 2052,'1953-S53','1953-11-21 00:00:00','military missile','success',1009,'','8A62 LKI II','','','Test',636,'',300 from dual union all
select 20520,'1969-W401','1969-03-17 14:51:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 20521,'1969-W402','1969-03-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 20522,'1969-W403','1969-03-17 15:55:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',59 from dual union all
select 20523,'1969-W404','1969-03-17 18:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',51 from dual union all
select 20524,'1969-W405','1969-03-17 18:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',54 from dual union all
select 20525,'1969-W406','1969-03-17 18:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 20526,'1969-W407','1969-03-17 20:18:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',60 from dual union all
select 20527,'1969-W408','1969-03-17 20:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual;
insert into LAUNCH
select 20528,'1969-W409','1969-03-18 17:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',59 from dual union all
select 20529,'1969-W41','1969-01-09 18:59:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',61 from dual union all
select 2053,'1953-S54','1953-11-26 00:00:00','military missile','success',1009,'','8A62 LKI II','','','Test',636,'',300 from dual union all
select 20530,'1969-W410','1969-03-18 18:35:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1057,'',58 from dual union all
select 20531,'1969-W411','1969-03-18 21:45:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',57 from dual union all
select 20532,'1969-W412','1969-03-19 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 20533,'1969-W413','1969-03-19 15:08:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',67 from dual union all
select 20534,'1969-W414','1969-03-19 16:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',62 from dual union all
select 20535,'1969-W415','1969-03-19 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 20536,'1969-W416','1969-03-19 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 20537,'1969-W417','1969-03-19 18:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',66 from dual union all
select 20538,'1969-W418','1969-03-19 18:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 20539,'1969-W419','1969-03-19 18:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',62 from dual union all
select 2054,'1953-S59','1953-11-02 18:32:00','suborbital rocket','success',51,'USAF 44','AF/Utah Ionos 5','','','Ionos',640,'',120 from dual union all
select 20540,'1969-W42','1969-01-09 19:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 20541,'1969-W420','1969-03-19 18:37:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',56 from dual union all
select 20542,'1969-W421','1969-03-19 19:00:00','suborbital rocket','success',1506,'Sphere 3','Met Sphere','','','Aeron',1895,'',51 from dual union all
select 20543,'1969-W422','1969-03-19 20:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',58 from dual union all
select 20544,'1969-W423','1969-03-19 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',56 from dual union all
select 20545,'1969-W424','1969-03-20 15:25:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',59 from dual union all
select 20546,'1969-W425','1969-03-20 20:42:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1057,'',61 from dual union all
select 20547,'1969-W426','1969-03-20 21:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',62 from dual union all
select 20548,'1969-W427','1969-03-20 23:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',59 from dual union all
select 20549,'1969-W428','1969-03-21 03:58:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 2055,'1953-S60','1953-11-03 18:15:00','suborbital rocket','success',51,'USAF 45','AF/Utah Ionos 6','','','Ionos',640,'',121 from dual union all
select 20550,'1969-W429','1969-03-21 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',63 from dual union all
select 20551,'1969-W43','1969-01-10 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',57 from dual union all
select 20552,'1969-W430','1969-03-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 20553,'1969-W431','1969-03-21 15:09:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',58 from dual union all
select 20554,'1969-W432','1969-03-21 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 20555,'1969-W433','1969-03-21 20:46:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 20556,'1969-W434','1969-03-24 10:10:00','suborbital rocket','success',725,'','Particle study 1','','','Aeron/Particles',636,'',75 from dual union all
select 20557,'1969-W435','1969-03-24 14:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 20558,'1969-W436','1969-03-24 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',60 from dual union all
select 20559,'1969-W437','1969-03-24 15:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',64 from dual union all
select 2056,'1953-S61','1953-11-12 00:00:00','military missile','success',970,'No 1120','','','','OT',636,'',100 from dual union all
select 20560,'1969-W438','1969-03-24 15:08:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',62 from dual union all
select 20561,'1969-W439','1969-03-24 18:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',60 from dual union all
select 20562,'1969-W44','1969-01-10 15:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',56 from dual union all
select 20563,'1969-W440','1969-03-24 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',60 from dual union all
select 20564,'1969-W441','1969-03-24 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',63 from dual union all
select 20565,'1969-W442','1969-03-25 03:22:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 20566,'1969-W443','1969-03-25 21:22:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',64 from dual union all
select 20567,'1969-W444','1969-03-25 21:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 20568,'1969-W445','1969-03-26 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',58 from dual union all
select 20569,'1969-W446','1969-03-26 15:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',54 from dual union all
select 2057,'1953-S63','1953-11-15 00:00:00','military missile','success',970,'No 1320','','','','OT',636,'',100 from dual union all
select 20570,'1969-W447','1969-03-26 17:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',55 from dual union all
select 20571,'1969-W448','1969-03-26 17:59:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',58 from dual union all
select 20572,'1969-W449','1969-03-26 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',62 from dual union all
select 20573,'1969-W45','1969-01-10 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',68 from dual union all
select 20574,'1969-W450','1969-03-26 20:04:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',73 from dual union all
select 20575,'1969-W451','1969-03-26 21:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 20576,'1969-W452','1969-03-26 22:51:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',63 from dual union all
select 20577,'1969-W453','1969-03-27 01:31:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 20578,'1969-W454','1969-03-27 03:32:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',63 from dual union all
select 20579,'1969-W455','1969-03-27 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 2058,'1953-S64','1953-11-19 22:40:00','suborbital rocket','success',54,'NRL 14','Oxygen content','','','Solar X/UV/Aeron',1869,'',112 from dual union all
select 20580,'1969-W456','1969-03-27 21:22:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',56 from dual union all
select 20581,'1969-W457','1969-03-28 00:21:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 20582,'1969-W458','1969-03-28 04:36:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',66 from dual union all
select 20583,'1969-W459','1969-03-28 05:37:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 20584,'1969-W46','1969-01-10 17:15:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',62 from dual union all
select 20585,'1969-W460','1969-03-28 08:24:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',59 from dual union all
select 20586,'1969-W461','1969-03-28 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',63 from dual union all
select 20587,'1969-W462','1969-03-28 14:47:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',64 from dual union all
select 20588,'1969-W463','1969-03-28 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 20589,'1969-W464','1969-03-28 18:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 2059,'1953-S65','1953-11-24 00:00:00','military missile','success',970,'No 1520','','','','OT',636,'',100 from dual union all
select 20590,'1969-W465','1969-03-28 18:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',62 from dual union all
select 20591,'1969-W466','1969-03-28 18:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',68 from dual union all
select 20592,'1969-W467','1969-03-28 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',54 from dual union all
select 20593,'1969-W468','1969-03-28 20:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',617,'',60 from dual union all
select 20594,'1969-W469','1969-03-28 21:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',60 from dual union all
select 20595,'1969-W47','1969-01-10 17:55:00','suborbital rocket','success',1188,'','','','','Meteo',1447,'',53 from dual union all
select 20596,'1969-W470','1969-03-29 01:11:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 20597,'1969-W471','1969-03-29 04:42:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 20598,'1969-W472','1969-04-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 20599,'1969-W473','1969-04-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 206,'1944-S187','1944-10-09 13:07:00','military missile','success',1470,'19047','WH1','','','Test',661,'',90 from dual union all
select 2060,'1953-S66','1953-11-25 15:46:00','suborbital rocket','success',54,'NRL 15','Oxygen content','','','Solar X/UV/Aeron',1869,'',95 from dual union all
select 20600,'1969-W474','1969-03-31 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',62 from dual union all
select 20601,'1969-W475','1969-04-01 00:59:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 20602,'1969-W476','1969-04-01 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',70 from dual union all
select 20603,'1969-W477','1969-04-01 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',53 from dual union all
select 20604,'1969-W478','1969-04-02 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',59 from dual union all
select 20605,'1969-W479','1969-04-02 15:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',54 from dual union all
select 20606,'1969-W48','1969-01-10 18:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 20607,'1969-W480','1969-04-02 15:53:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',72 from dual union all
select 20608,'1969-W481','1969-04-02 15:58:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',60 from dual union all
select 20609,'1969-W482','1969-04-02 16:26:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',52 from dual union all
select 2061,'1953-S67','1953-12-05 00:00:00','military missile','success',1009,'','8A62 LKI II','','','Test',636,'',300 from dual union all
select 20610,'1969-W483','1969-04-02 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',73 from dual union all
select 20611,'1969-W484','1969-04-02 19:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',62 from dual union all
select 20612,'1969-W485','1969-04-02 20:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 20613,'1969-W486','1969-04-02 20:43:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 20614,'1969-W487','1969-04-03 13:58:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',69 from dual union all
select 20615,'1969-W488','1969-04-03 18:13:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',65 from dual union all
select 20616,'1969-W489','1969-04-03 21:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1057,'',63 from dual union all
select 20617,'1969-W49','1969-01-10 18:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',54 from dual;
insert into LAUNCH
select 20618,'1969-W490','1969-04-04 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',64 from dual union all
select 20619,'1969-W491','1969-04-04 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',62 from dual union all
select 2062,'1953-S68','1953-12-01 15:30:00','suborbital rocket','success',54,'NRL 16','Oxygen content','','','Solar X/UV/Aeron',1869,'',129 from dual union all
select 20620,'1969-W492','1969-04-04 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 20621,'1969-W493','1969-04-04 16:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 20622,'1969-W494','1969-04-04 17:02:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 20623,'1969-W495','1969-04-04 17:15:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',57 from dual union all
select 20624,'1969-W496','1969-04-04 20:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',62 from dual union all
select 20625,'1969-W497','1969-04-07 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',50 from dual union all
select 20626,'1969-W498','1969-04-07 15:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',65 from dual union all
select 20627,'1969-W499','1969-04-07 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 20628,'1969-W50','1969-01-10 18:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',59 from dual union all
select 20629,'1969-W500','1969-04-07 16:04:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',35,'',58 from dual union all
select 2063,'1953-S69','1953-12-09 00:00:00','military missile','success',1009,'','8A62 LKI II','','','Test',636,'',300 from dual union all
select 20630,'1969-W501','1969-04-07 16:11:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',64 from dual union all
select 20631,'1969-W502','1969-04-07 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',55 from dual union all
select 20632,'1969-W503','1969-04-07 20:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 20633,'1969-W504','1969-04-07 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 20634,'1969-W505','1969-04-08 13:51:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',54 from dual union all
select 20635,'1969-W506','1969-04-08 18:27:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',65 from dual union all
select 20636,'1969-W507','1969-04-08 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1057,'',64 from dual union all
select 20637,'1969-W508','1969-04-09 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',64 from dual union all
select 20638,'1969-W509','1969-04-09 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',63 from dual union all
select 20639,'1969-W51','1969-01-10 18:56:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',63 from dual union all
select 2064,'1953-S70','1953-10-24 00:00:00','military missile','success',992,'530-90-220','NII-88 Production','','','OT',636,'',100 from dual union all
select 20640,'1969-W510','1969-04-09 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 20641,'1969-W511','1969-04-09 15:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',59 from dual union all
select 20642,'1969-W512','1969-04-09 16:34:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1363,'',60 from dual union all
select 20643,'1969-W513','1969-04-09 17:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 20644,'1969-W514','1969-04-09 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 20645,'1969-W515','1969-04-09 18:31:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',54 from dual union all
select 20646,'1969-W516','1969-04-09 20:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',61 from dual union all
select 20647,'1969-W517','1969-04-09 20:48:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',57 from dual union all
select 20648,'1969-W518','1969-04-09 23:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 20649,'1969-W519','1969-04-10 02:40:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 2065,'1953-S71','1953-05-09 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 20650,'1969-W52','1969-01-10 19:36:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',61 from dual union all
select 20651,'1969-W520','1969-04-10 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',53 from dual union all
select 20652,'1969-W521','1969-04-10 16:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 20653,'1969-W522','1969-04-10 17:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',62 from dual union all
select 20654,'1969-W523','1969-04-11 09:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 20655,'1969-W524','1969-04-11 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',68 from dual union all
select 20656,'1969-W525','1969-04-11 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',51 from dual union all
select 20657,'1969-W526','1969-04-11 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',66 from dual union all
select 20658,'1969-W527','1969-04-11 16:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',52 from dual union all
select 20659,'1969-W528','1969-04-11 17:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',61 from dual union all
select 2066,'1953-S72','1953-05-17 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 20660,'1969-W529','1969-04-11 18:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 20661,'1969-W53','1969-01-10 20:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',59 from dual union all
select 20662,'1969-W530','1969-04-11 18:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',65 from dual union all
select 20663,'1969-W531','1969-04-11 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',63 from dual union all
select 20664,'1969-W532','1969-04-11 20:35:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 20665,'1969-W533','1969-04-12 03:15:59','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',57 from dual union all
select 20666,'1969-W534','1969-04-13 15:23:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 20667,'1969-W535','1969-04-14 14:49:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',63 from dual union all
select 20668,'1969-W536','1969-04-14 16:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 20669,'1969-W537','1969-04-14 17:19:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',62 from dual union all
select 2067,'1953-S73','1953-05-22 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 20670,'1969-W538','1969-04-14 18:00:00','suborbital rocket','success',1061,'','Starute','','','Meteo',276,'',59 from dual union all
select 20671,'1969-W539','1969-04-14 18:36:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',70 from dual union all
select 20672,'1969-W54','1969-01-10 20:11:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',63 from dual union all
select 20673,'1969-W540','1969-04-14 19:44:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',58 from dual union all
select 20674,'1969-W541','1969-04-14 20:01:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',58 from dual union all
select 20675,'1969-W542','1969-04-14 22:07:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 20676,'1969-W543','1969-04-15 03:11:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 20677,'1969-W544','1969-04-15 14:46:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',70 from dual union all
select 20678,'1969-W545','1969-04-15 16:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',66 from dual union all
select 20679,'1969-W546','1969-04-15 21:57:59','suborbital rocket','success',1061,'','','','','Meteo',1057,'',64 from dual union all
select 2068,'1953-S74','1953-05-26 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 20680,'1969-W547','1969-04-16 11:13:00','suborbital rocket','success',631,'INTA H-6908','','','','Meteo',396,'',61 from dual union all
select 20681,'1969-W548','1969-04-16 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 20682,'1969-W549','1969-04-16 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',65 from dual union all
select 20683,'1969-W55','1969-01-10 20:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',65 from dual union all
select 20684,'1969-W550','1969-04-16 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 20685,'1969-W551','1969-04-16 15:03:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',59 from dual union all
select 20686,'1969-W552','1969-04-16 16:21:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',56 from dual union all
select 20687,'1969-W553','1969-04-16 16:22:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',65 from dual union all
select 20688,'1969-W554','1969-04-16 16:56:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',60 from dual union all
select 20689,'1969-W555','1969-04-16 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',62 from dual union all
select 2069,'1953-S75','1953-05-27 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 20690,'1969-W556','1969-04-16 19:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 20691,'1969-W557','1969-04-16 21:14:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',65 from dual union all
select 20692,'1969-W558','1969-04-16 22:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 20693,'1969-W559','1969-04-16 22:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',308,'',60 from dual union all
select 20694,'1969-W56','1969-01-10 20:41:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',57 from dual union all
select 20695,'1969-W560','1969-04-17 16:32:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',59 from dual union all
select 20696,'1969-W561','1969-04-17 17:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 20697,'1969-W562','1969-04-17 20:09:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',53 from dual union all
select 20698,'1969-W563','1969-04-18 13:58:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',35,'',62 from dual union all
select 20699,'1969-W564','1969-04-18 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',63 from dual union all
select 207,'1944-S188','1944-10-09 13:59:00','military missile','success',1470,'19039','WH2','','','Test',661,'',90 from dual union all
select 2070,'1953-S76','1953-05-28 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 20700,'1969-W565','1969-04-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 20701,'1969-W566','1969-04-18 16:54:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',65 from dual union all
select 20702,'1969-W567','1969-04-18 18:11:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',58 from dual union all
select 20703,'1969-W568','1969-04-18 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',56 from dual union all
select 20704,'1969-W569','1969-04-18 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',67 from dual union all
select 20705,'1969-W57','1969-01-10 21:44:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 20706,'1969-W570','1969-04-19 04:09:00','suborbital rocket','failure',258,'AMM-BA-3','O/OH profile','','','Aeron',428,'',55 from dual union all
select 20707,'1969-W571','1969-04-21 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',64 from dual;
insert into LAUNCH
select 20708,'1969-W572','1969-04-21 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',35,'',61 from dual union all
select 20709,'1969-W573','1969-04-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 2071,'1953-S77','1953-05-30 00:00:00','military missile','failure',971,'','','','','Test',636,'',150 from dual union all
select 20710,'1969-W574','1969-04-21 18:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',58 from dual union all
select 20711,'1969-W575','1969-04-21 19:32:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 20712,'1969-W576','1969-04-21 19:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 20713,'1969-W577','1969-04-21 20:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',57 from dual union all
select 20714,'1969-W578','1969-04-21 20:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 20715,'1969-W579','1969-04-21 20:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',54 from dual union all
select 20716,'1969-W58','1969-01-13 14:52:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',60 from dual union all
select 20717,'1969-W580','1969-04-21 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',54 from dual union all
select 20718,'1969-W581','1969-04-21 21:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',53 from dual union all
select 20719,'1969-W582','1969-04-22 02:38:59','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',62 from dual union all
select 2072,'1954-A01','1954-01-01 00:00:00','atmospheric rocket','success',564,'','E14 Model','','','Test',1821,'',5 from dual union all
select 20720,'1969-W583','1969-04-22 22:09:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',62 from dual union all
select 20721,'1969-W584','1969-04-23 01:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 20722,'1969-W585','1969-04-23 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',67 from dual union all
select 20723,'1969-W586','1969-04-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',53 from dual union all
select 20724,'1969-W587','1969-04-23 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 20725,'1969-W588','1969-04-23 16:09:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',64 from dual union all
select 20726,'1969-W589','1969-04-23 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 20727,'1969-W59','1969-01-13 15:11:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',54 from dual union all
select 20728,'1969-W590','1969-04-23 17:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 20729,'1969-W591','1969-04-23 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 2073,'1954-A02','1954-01-01 00:00:00','suborbital rocket','success',840,'','','','','Test',1821,'',10 from dual union all
select 20730,'1969-W592','1969-04-23 18:34:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',62 from dual union all
select 20731,'1969-W593','1969-04-23 21:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 20732,'1969-W594','1969-04-24 02:22:00','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',63 from dual union all
select 20733,'1969-W595','1969-04-24 21:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 20734,'1969-W596','1969-04-25 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',62 from dual union all
select 20735,'1969-W597','1969-04-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 20736,'1969-W598','1969-04-25 15:34:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',35,'',61 from dual union all
select 20737,'1969-W599','1969-04-25 17:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 20738,'1969-W60','1969-01-13 16:40:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',53 from dual union all
select 20739,'1969-W600','1969-04-25 17:04:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',65 from dual union all
select 2074,'1954-A03','1954-01-04 00:00:00','atmospheric rocket','success',365,'','F-104 Model','','','Test',1821,'',10 from dual union all
select 20740,'1969-W601','1969-04-25 17:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',66 from dual union all
select 20741,'1969-W602','1969-04-25 23:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 20742,'1969-W603','1969-04-26 02:31:00','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',65 from dual union all
select 20743,'1969-W604','1969-04-27 03:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',57 from dual union all
select 20744,'1969-W605','1969-04-27 03:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',57 from dual union all
select 20745,'1969-W606','1969-04-27 04:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',58 from dual union all
select 20746,'1969-W607','1969-04-27 05:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',59 from dual union all
select 20747,'1969-W608','1969-04-27 05:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 20748,'1969-W609','1969-04-27 05:53:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',56 from dual union all
select 20749,'1969-W61','1969-01-13 17:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 2075,'1954-A04','1954-01-06 00:00:00','atmospheric rocket','success',836,'1288T','','','','Test',1895,'',10 from dual union all
select 20750,'1969-W610','1969-04-27 06:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 20751,'1969-W611','1969-04-28 14:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 20752,'1969-W612','1969-04-28 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',66 from dual union all
select 20753,'1969-W613','1969-04-28 18:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',67 from dual union all
select 20754,'1969-W614','1969-04-28 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',65 from dual union all
select 20755,'1969-W615','1969-04-28 21:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',61 from dual union all
select 20756,'1969-W616','1969-04-28 21:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 20757,'1969-W617','1969-04-29 15:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',62 from dual union all
select 20758,'1969-W618','1969-04-29 16:03:59','suborbital rocket','success',1061,'','','','','Meteo',1057,'',57 from dual union all
select 20759,'1969-W619','1969-04-29 17:07:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',58 from dual union all
select 2076,'1954-A05','1954-01-07 00:00:00','atmospheric rocket','success',365,'','F8U-1 Model','','','Test',1821,'',10 from dual union all
select 20760,'1969-W62','1969-01-13 18:01:59','suborbital rocket','success',1188,'','','','','Meteo',1447,'',61 from dual union all
select 20761,'1969-W620','1969-04-29 17:58:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',61 from dual union all
select 20762,'1969-W621','1969-04-29 18:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',60 from dual union all
select 20763,'1969-W622','1969-04-30 10:44:59','suborbital rocket','success',631,'INTA H-6909','','','','Meteo',396,'',62 from dual union all
select 20764,'1969-W623','1969-04-30 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 20765,'1969-W624','1969-04-30 14:45:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',56 from dual union all
select 20766,'1969-W625','1969-04-30 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',67 from dual union all
select 20767,'1969-W626','1969-04-30 16:09:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',60 from dual union all
select 20768,'1969-W627','1969-04-30 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',35,'',55 from dual union all
select 20769,'1969-W628','1969-04-30 18:01:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',64 from dual union all
select 2077,'1954-A06','1954-01-07 00:00:00','military missile','success',771,'GM-12427','','','','Test',222,'',12 from dual union all
select 20770,'1969-W629','1969-04-30 19:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 20771,'1969-W63','1969-01-13 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 20772,'1969-W630','1969-04-30 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',57 from dual union all
select 20773,'1969-W631','1969-05-01 14:57:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',62 from dual union all
select 20774,'1969-W632','1969-05-01 17:00:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',68 from dual union all
select 20775,'1969-W633','1969-05-01 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 20776,'1969-W634','1969-05-01 21:32:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',50 from dual union all
select 20777,'1969-W635','1969-05-01 23:43:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',308,'',58 from dual union all
select 20778,'1969-W636','1969-05-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 20779,'1969-W637','1969-05-02 14:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',60 from dual union all
select 2078,'1954-A07','1954-01-07 00:00:00','atmospheric rocket','success',836,'1197A','','','','Test',1895,'',10 from dual union all
select 20780,'1969-W638','1969-05-02 14:57:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',176,'',60 from dual union all
select 20781,'1969-W639','1969-05-02 15:22:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',64 from dual union all
select 20782,'1969-W64','1969-01-13 19:09:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',58 from dual union all
select 20783,'1969-W640','1969-05-02 16:18:59','suborbital rocket','success',88,'','Met Para,Datasonde','','','Meteo',428,'',68 from dual union all
select 20784,'1969-W641','1969-05-02 18:06:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',57 from dual union all
select 20785,'1969-W642','1969-05-02 18:22:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',64 from dual union all
select 20786,'1969-W643','1969-05-02 18:24:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',60 from dual union all
select 20787,'1969-W644','1969-05-02 18:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 20788,'1969-W645','1969-05-02 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',58 from dual union all
select 20789,'1969-W646','1969-05-02 19:04:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',617,'',58 from dual union all
select 2079,'1954-A08','1954-01-08 00:00:00','military missile','success',771,'GM-12421','','','','Test',222,'',12 from dual union all
select 20790,'1969-W647','1969-05-02 19:45:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',57 from dual union all
select 20791,'1969-W648','1969-05-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 20792,'1969-W649','1969-05-05 14:33:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',35,'',64 from dual union all
select 20793,'1969-W65','1969-01-13 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',58 from dual union all
select 20794,'1969-W650','1969-05-05 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',63 from dual union all
select 20795,'1969-W651','1969-05-05 17:35:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',62 from dual union all
select 20796,'1969-W652','1969-05-05 18:36:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',70 from dual union all
select 20797,'1969-W653','1969-05-05 19:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',60 from dual union all
select 20798,'1969-W654','1969-05-05 20:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',64 from dual;
insert into LAUNCH
select 20799,'1969-W655','1969-05-05 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',59 from dual union all
select 208,'1944-S189','1944-10-09 15:27:00','military missile','success',1470,'19043','WH10','','','Test',661,'',90 from dual union all
select 2080,'1954-A09','1954-01-08 00:00:00','military missile','success',771,'GM-12420','','','','Test',222,'',12 from dual union all
select 20800,'1969-W656','1969-05-06 14:56:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',53 from dual union all
select 20801,'1969-W657','1969-05-06 18:04:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',56 from dual union all
select 20802,'1969-W658','1969-05-06 21:20:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',50 from dual union all
select 20803,'1969-W659','1969-05-07 14:46:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 20804,'1969-W66','1969-01-14 15:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',54 from dual union all
select 20805,'1969-W660','1969-05-07 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',60 from dual union all
select 20806,'1969-W661','1969-05-07 15:38:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',66 from dual union all
select 20807,'1969-W662','1969-05-07 16:27:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',63 from dual union all
select 20808,'1969-W663','1969-05-07 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',60 from dual union all
select 20809,'1969-W664','1969-05-07 17:26:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 2081,'1954-A10','1954-01-08 00:00:00','military missile','success',771,'GM-11109','','','','Test',222,'',12 from dual union all
select 20810,'1969-W665','1969-05-07 18:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',276,'',54 from dual union all
select 20811,'1969-W666','1969-05-07 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 20812,'1969-W667','1969-05-07 18:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',61 from dual union all
select 20813,'1969-W668','1969-05-07 21:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',54 from dual union all
select 20814,'1969-W669','1969-05-08 01:02:59','suborbital rocket','success',631,'INTA H-6910','','','','Meteo',396,'',60 from dual union all
select 20815,'1969-W67','1969-01-14 15:04:59','suborbital rocket','success',1061,'','','','','Meteo',1895,'',68 from dual union all
select 20816,'1969-W670','1969-05-08 14:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',59 from dual union all
select 20817,'1969-W671','1969-05-08 18:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',61 from dual union all
select 20818,'1969-W672','1969-05-08 19:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',67 from dual union all
select 20819,'1969-W673','1969-05-08 21:13:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',50 from dual union all
select 2082,'1954-A100','1954-03-05 00:00:00','military missile','success',771,'GM-12437','','','','Test',222,'',12 from dual union all
select 20820,'1969-W674','1969-05-09 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',58 from dual union all
select 20821,'1969-W675','1969-05-09 16:49:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',57 from dual union all
select 20822,'1969-W676','1969-05-09 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',63 from dual union all
select 20823,'1969-W677','1969-05-09 19:00:00','suborbital rocket','success',716,'','Rocketsonde','','','Aeron',1895,'',70 from dual union all
select 20824,'1969-W678','1969-05-09 19:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',53 from dual union all
select 20825,'1969-W679','1969-05-09 21:35:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',56 from dual union all
select 20826,'1969-W68','1969-01-14 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',57 from dual union all
select 20827,'1969-W680','1969-05-10 02:55:00','suborbital rocket','success',631,'INTA H-6911','','','','Meteo',396,'',60 from dual union all
select 20828,'1969-W681','1969-05-10 05:13:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',63 from dual union all
select 20829,'1969-W682','1969-05-11 01:15:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 2083,'1954-A101','1954-03-05 00:00:00','atmospheric rocket','success',836,'1313R','Nike 267R','','','Test',1895,'',10 from dual union all
select 20830,'1969-W683','1969-05-11 03:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',58 from dual union all
select 20831,'1969-W684','1969-05-11 03:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 20832,'1969-W685','1969-05-11 04:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',67 from dual union all
select 20833,'1969-W686','1969-05-11 04:32:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 20834,'1969-W687','1969-05-11 05:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',59 from dual union all
select 20835,'1969-W688','1969-05-11 05:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 20836,'1969-W689','1969-05-11 06:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',68 from dual union all
select 20837,'1969-W69','1969-01-14 16:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',73 from dual union all
select 20838,'1969-W690','1969-05-11 06:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 20839,'1969-W691','1969-05-11 06:33:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 2084,'1954-A102','1954-03-05 00:00:00','atmospheric rocket','success',836,'1274A','','','','Test',1895,'',10 from dual union all
select 20840,'1969-W692','1969-05-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 20841,'1969-W693','1969-05-12 16:45:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',60 from dual union all
select 20842,'1969-W694','1969-05-12 17:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',61 from dual union all
select 20843,'1969-W695','1969-05-12 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',64 from dual union all
select 20844,'1969-W696','1969-05-12 19:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',64 from dual union all
select 20845,'1969-W697','1969-05-12 19:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',70 from dual union all
select 20846,'1969-W698','1969-05-12 21:33:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',57 from dual union all
select 20847,'1969-W699','1969-05-13 15:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 20848,'1969-W70','1969-01-14 17:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',58 from dual union all
select 20849,'1969-W700','1969-05-13 15:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 2085,'1954-A103','1954-03-05 00:00:00','atmospheric rocket','success',836,'1649E','','','','Test',1895,'',10 from dual union all
select 20850,'1969-W701','1969-05-13 16:02:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',57 from dual union all
select 20851,'1969-W702','1969-05-13 17:12:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',59 from dual union all
select 20852,'1969-W703','1969-05-13 17:15:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',61 from dual union all
select 20853,'1969-W704','1969-05-14 10:46:59','suborbital rocket','success',631,'INTA H-6912','','','','Meteo',396,'',56 from dual union all
select 20854,'1969-W705','1969-05-14 11:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 20855,'1969-W706','1969-05-14 14:00:00','suborbital rocket','success',1061,'','Starute','','','Meteo',276,'',61 from dual union all
select 20856,'1969-W707','1969-05-14 14:12:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',59 from dual union all
select 20857,'1969-W708','1969-05-14 16:27:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',66 from dual union all
select 20858,'1969-W709','1969-05-14 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',73 from dual union all
select 20859,'1969-W71','1969-01-14 20:10:00','suborbital rocket','success',716,'','Rocketsonde','','','Aeron',1895,'',70 from dual union all
select 2086,'1954-A104','1954-03-05 00:00:00','atmospheric rocket','success',836,'1535E','','','','Test',1895,'',10 from dual union all
select 20860,'1969-W710','1969-05-14 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',71 from dual union all
select 20861,'1969-W711','1969-05-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 20862,'1969-W712','1969-05-15 15:13:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',51 from dual union all
select 20863,'1969-W713','1969-05-15 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',56 from dual union all
select 20864,'1969-W714','1969-05-15 22:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',308,'',58 from dual union all
select 20865,'1969-W715','1969-05-15 22:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 20866,'1969-W716','1969-05-16 14:45:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',176,'',55 from dual union all
select 20867,'1969-W717','1969-05-16 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 20868,'1969-W718','1969-05-16 16:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',63 from dual union all
select 20869,'1969-W719','1969-05-16 18:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',73 from dual union all
select 2087,'1954-A106','1954-03-05 00:00:00','atmospheric rocket','success',836,'1366T','','','','Test',1895,'',10 from dual union all
select 20870,'1969-W72','1969-01-15 14:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',54 from dual union all
select 20871,'1969-W720','1969-05-16 18:11:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 20872,'1969-W721','1969-05-16 19:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 20873,'1969-W722','1969-05-16 20:41:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',60 from dual union all
select 20874,'1969-W723','1969-05-16 21:34:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',62 from dual union all
select 20875,'1969-W724','1969-05-17 13:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 20876,'1969-W725','1969-05-18 19:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 20877,'1969-W726','1969-05-19 04:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1592,'SHIP-SOV',76 from dual union all
select 20878,'1969-W727','1969-05-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',53 from dual union all
select 20879,'1969-W728','1969-05-19 14:45:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',176,'',65 from dual union all
select 2088,'1954-A107','1954-03-09 00:00:00','military missile','unknown',339,'1293R','','','','Test',1895,'',30 from dual union all
select 20880,'1969-W729','1969-05-19 15:31:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',54 from dual union all
select 20881,'1969-W73','1969-01-15 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',65 from dual union all
select 20882,'1969-W730','1969-05-19 17:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 20883,'1969-W731','1969-05-19 22:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 20884,'1969-W732','1969-05-20 18:13:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',68 from dual union all
select 20885,'1969-W733','1969-05-20 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 20886,'1969-W734','1969-05-21 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',64 from dual union all
select 20887,'1969-W735','1969-05-21 15:04:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',61 from dual union all
select 20888,'1969-W736','1969-05-21 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual;
insert into LAUNCH
select 20889,'1969-W737','1969-05-21 16:34:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1363,'',59 from dual union all
select 2089,'1954-A108','1954-03-10 00:00:00','atmospheric rocket','unknown',576,'220','','','','Test',1895,'',10 from dual union all
select 20890,'1969-W738','1969-05-21 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',68 from dual union all
select 20891,'1969-W739','1969-05-21 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',69 from dual union all
select 20892,'1969-W74','1969-01-15 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',55 from dual union all
select 20893,'1969-W740','1969-05-21 20:32:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',57 from dual union all
select 20894,'1969-W741','1969-05-21 21:57:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 20895,'1969-W742','1969-05-22 01:04:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',70 from dual union all
select 20896,'1969-W743','1969-05-22 17:56:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',53 from dual union all
select 20897,'1969-W744','1969-05-22 18:06:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',56 from dual union all
select 20898,'1969-W745','1969-05-22 18:37:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',56 from dual union all
select 20899,'1969-W746','1969-05-22 18:57:59','suborbital rocket','success',1061,'','Met Para,Exp. Sonde','','','Meteo',1821,'',59 from dual union all
select 209,'1944-S19','1944-03-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 2090,'1954-A109','1954-03-12 00:00:00','atmospheric rocket','success',836,'1294T','','','','Test',1895,'',10 from dual union all
select 20900,'1969-W747','1969-05-23 14:21:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',52 from dual union all
select 20901,'1969-W748','1969-05-23 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 20902,'1969-W749','1969-05-23 14:45:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',176,'',60 from dual union all
select 20903,'1969-W75','1969-01-15 16:02:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',69 from dual union all
select 20904,'1969-W750','1969-05-23 15:46:59','suborbital rocket','success',1061,'','Met Para,Exp. Sonde','','','Meteo',1821,'',54 from dual union all
select 20905,'1969-W751','1969-05-23 17:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',64 from dual union all
select 20906,'1969-W752','1969-05-23 17:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',62 from dual union all
select 20907,'1969-W753','1969-05-23 18:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',74 from dual union all
select 20908,'1969-W754','1969-05-23 18:56:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 20909,'1969-W755','1969-05-23 20:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',65 from dual union all
select 2091,'1954-A11','1954-01-11 00:00:00','atmospheric rocket','success',836,'1298T','','','','Test',1895,'',10 from dual union all
select 20910,'1969-W756','1969-05-23 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',64 from dual union all
select 20911,'1969-W757','1969-05-24 23:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 20912,'1969-W758','1969-05-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 20913,'1969-W759','1969-05-26 14:46:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 20914,'1969-W76','1969-01-15 16:38:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 20915,'1969-W760','1969-05-26 17:03:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',55 from dual union all
select 20916,'1969-W761','1969-05-26 18:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',66 from dual union all
select 20917,'1969-W762','1969-05-26 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',62 from dual union all
select 20918,'1969-W763','1969-05-26 21:58:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',57 from dual union all
select 20919,'1969-W764','1969-05-27 00:53:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',69 from dual union all
select 2092,'1954-A110','1954-03-14 00:00:00','atmospheric rocket','success',836,'1131T','','','','Test',1895,'',10 from dual union all
select 20920,'1969-W765','1969-05-27 18:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',53 from dual union all
select 20921,'1969-W766','1969-05-28 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',64 from dual union all
select 20922,'1969-W767','1969-05-28 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 20923,'1969-W768','1969-05-28 16:41:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1363,'',59 from dual union all
select 20924,'1969-W769','1969-05-28 16:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',70 from dual union all
select 20925,'1969-W77','1969-01-15 16:47:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',63 from dual union all
select 20926,'1969-W770','1969-05-28 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',60 from dual union all
select 20927,'1969-W771','1969-05-28 18:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 20928,'1969-W772','1969-05-28 18:22:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',71 from dual union all
select 20929,'1969-W773','1969-05-28 19:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 2093,'1954-A111','1954-03-17 00:00:00','atmospheric rocket','success',836,'1285R  Bell motor','Nike 268R','','','Test',1895,'',10 from dual union all
select 20930,'1969-W774','1969-05-28 21:12:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',57 from dual union all
select 20931,'1969-W775','1969-05-29 14:59:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 20932,'1969-W776','1969-05-29 15:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',57 from dual union all
select 20933,'1969-W777','1969-05-29 15:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',62 from dual union all
select 20934,'1969-W778','1969-05-29 16:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',60 from dual union all
select 20935,'1969-W779','1969-05-29 18:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',68 from dual union all
select 20936,'1969-W78','1969-01-15 18:47:59','suborbital rocket','success',1188,'','','','','Meteo',1447,'',56 from dual union all
select 20937,'1969-W780','1969-05-29 18:55:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',308,'',64 from dual union all
select 20938,'1969-W781','1969-05-30 15:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',58 from dual union all
select 20939,'1969-W782','1969-05-30 18:15:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',68 from dual union all
select 2094,'1954-A112','1954-03-18 00:00:00','military missile','unknown',339,'1246T-1','','','','Test',1890,'',30 from dual union all
select 20940,'1969-W783','1969-05-30 19:56:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',68 from dual union all
select 20941,'1969-W784','1969-06-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 20942,'1969-W785','1969-06-02 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',58 from dual union all
select 20943,'1969-W786','1969-06-02 16:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',66 from dual union all
select 20944,'1969-W787','1969-06-02 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',61 from dual union all
select 20945,'1969-W788','1969-06-02 20:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',61 from dual union all
select 20946,'1969-W789','1969-06-02 21:10:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',54 from dual union all
select 20947,'1969-W79','1969-01-15 19:58:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 20948,'1969-W790','1969-06-04 12:26:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',62 from dual union all
select 20949,'1969-W791','1969-06-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 2095,'1954-A113','1954-03-18 00:00:00','atmospheric rocket','unknown',576,'202','','','','Test',1895,'',10 from dual union all
select 20950,'1969-W792','1969-06-04 14:49:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',60 from dual union all
select 20951,'1969-W793','1969-06-04 15:59:59','suborbital rocket','success',725,'','','','','Meteo',1608,'SHIP-SOV',72 from dual union all
select 20952,'1969-W794','1969-06-04 16:44:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',66 from dual union all
select 20953,'1969-W795','1969-06-04 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',66 from dual union all
select 20954,'1969-W796','1969-06-04 18:24:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',58 from dual union all
select 20955,'1969-W797','1969-06-04 22:24:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',55 from dual union all
select 20956,'1969-W798','1969-06-05 17:34:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',54 from dual union all
select 20957,'1969-W799','1969-06-06 05:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',67 from dual union all
select 20958,'1969-W80','1969-01-15 20:00:00','suborbital rocket','success',1061,'','Starute','','','Meteo',617,'',55 from dual union all
select 20959,'1969-W800','1969-06-06 06:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 2096,'1954-A114','1954-03-18 00:00:00','atmospheric rocket','success',836,'1610E','','','','Test',1895,'',10 from dual union all
select 20960,'1969-W801','1969-06-06 14:41:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 20961,'1969-W802','1969-06-06 16:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 20962,'1969-W803','1969-06-06 17:13:00','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',62 from dual union all
select 20963,'1969-W804','1969-06-06 17:35:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',63 from dual union all
select 20964,'1969-W805','1969-06-06 20:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',60 from dual union all
select 20965,'1969-W806','1969-06-06 21:49:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',62 from dual union all
select 20966,'1969-W807','1969-06-06 21:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',62 from dual union all
select 20967,'1969-W808','1969-06-06 22:49:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',75 from dual union all
select 20968,'1969-W809','1969-06-09 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',54 from dual union all
select 20969,'1969-W81','1969-01-15 20:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',56 from dual union all
select 2097,'1954-A115','1954-03-18 00:00:00','atmospheric rocket','success',836,'1785','','','','Test',1895,'',10 from dual union all
select 20970,'1969-W810','1969-06-09 17:45:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 20971,'1969-W811','1969-06-09 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',60 from dual union all
select 20972,'1969-W812','1969-06-10 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',66 from dual union all
select 20973,'1969-W813','1969-06-10 16:14:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',58 from dual union all
select 20974,'1969-W814','1969-06-10 18:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 20975,'1969-W815','1969-06-10 19:56:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',65 from dual union all
select 20976,'1969-W816','1969-06-11 05:20:00','suborbital rocket','success',631,'TERLS-110','ISRO 11.30','TERLS-110','','Aeron',1467,'',65 from dual union all
select 20977,'1969-W817','1969-06-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 20978,'1969-W818','1969-06-11 14:27:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual;
insert into LAUNCH
select 20979,'1969-W819','1969-06-11 14:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',66 from dual union all
select 2098,'1954-A116','1954-03-19 00:00:00','atmospheric rocket','success',365,'','XF2Y-1 Model','','','Test',1821,'',10 from dual union all
select 20980,'1969-W82','1969-01-15 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 20981,'1969-W820','1969-06-11 15:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 20982,'1969-W821','1969-06-11 16:12:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',54 from dual union all
select 20983,'1969-W822','1969-06-11 16:38:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',69 from dual union all
select 20984,'1969-W823','1969-06-11 16:57:59','suborbital rocket','success',631,'','','','','Meteo',428,'',64 from dual union all
select 20985,'1969-W824','1969-06-11 17:43:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',62 from dual union all
select 20986,'1969-W825','1969-06-11 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',56 from dual union all
select 20987,'1969-W826','1969-06-12 18:32:00','suborbital rocket','success',88,'NASA 12.12GM','','','','Aeronomy',1821,'',73 from dual union all
select 20988,'1969-W827','1969-06-12 21:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',72 from dual union all
select 20989,'1969-W828','1969-06-13 14:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 2099,'1954-A117','1954-03-19 00:00:00','atmospheric rocket','unknown',576,'500','','','','Test',1895,'',10 from dual union all
select 20990,'1969-W829','1969-06-13 14:57:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1470,'',58 from dual union all
select 20991,'1969-W83','1969-01-16 15:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',54 from dual union all
select 20992,'1969-W830','1969-06-13 17:02:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',61 from dual union all
select 20993,'1969-W831','1969-06-13 18:34:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',51 from dual union all
select 20994,'1969-W832','1969-06-13 19:44:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',67 from dual union all
select 20995,'1969-W833','1969-06-13 21:33:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',50 from dual union all
select 20996,'1969-W834','1969-06-15 04:18:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',276,'',61 from dual union all
select 20997,'1969-W835','1969-06-15 17:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',61 from dual union all
select 20998,'1969-W836','1969-06-16 17:14:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',56 from dual union all
select 20999,'1969-W837','1969-06-16 17:54:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',52 from dual union all
select 21,'1943-A14','1943-11-25 01:00:00','military missile','failure',8,'','','','','Test',662,'',20 from dual union all
select 210,'1944-S190','1944-10-09 16:42:00','military missile','success',1470,'19042','WH6','','','Test',661,'',90 from dual union all
select 2100,'1954-A118','1954-03-19 00:00:00','atmospheric rocket','success',836,'1374T','','','','Test',1895,'',10 from dual union all
select 21000,'1969-W838','1969-06-17 15:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',61 from dual union all
select 21001,'1969-W839','1969-06-18 11:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 21002,'1969-W84','1969-01-16 20:36:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 21003,'1969-W840','1969-06-18 14:18:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 21004,'1969-W841','1969-06-18 15:11:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 21005,'1969-W842','1969-06-18 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',63 from dual union all
select 21006,'1969-W843','1969-06-18 16:48:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',64 from dual union all
select 21007,'1969-W844','1969-06-18 16:56:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',67 from dual union all
select 21008,'1969-W845','1969-06-18 17:37:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',65 from dual union all
select 21009,'1969-W846','1969-06-18 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',62 from dual union all
select 2101,'1954-A119','1954-03-20 00:00:00','atmospheric rocket','success',836,'1300T','','','','Test',1895,'',10 from dual union all
select 21010,'1969-W847','1969-06-18 19:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',64 from dual union all
select 21011,'1969-W848','1969-06-18 21:19:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',60 from dual union all
select 21012,'1969-W849','1969-06-18 22:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',308,'',57 from dual union all
select 21013,'1969-W85','1969-01-16 23:45:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',59 from dual union all
select 21014,'1969-W850','1969-06-19 15:31:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',69 from dual union all
select 21015,'1969-W851','1969-06-19 17:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',50 from dual union all
select 21016,'1969-W852','1969-06-20 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 21017,'1969-W853','1969-06-20 01:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 21018,'1969-W854','1969-06-20 08:37:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 21019,'1969-W855','1969-06-20 14:30:00','suborbital rocket','unknown',631,'SUPARCO','','','','Aeron',1419,'',59 from dual union all
select 2102,'1954-A12','1954-01-11 00:00:00','atmospheric rocket','success',836,'1200A','','','','Test',1895,'',10 from dual union all
select 21020,'1969-W856','1969-06-20 15:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 21021,'1969-W857','1969-06-20 15:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 21022,'1969-W858','1969-06-20 16:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',63 from dual union all
select 21023,'1969-W859','1969-06-20 16:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 21024,'1969-W86','1969-01-17 00:40:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',60 from dual union all
select 21025,'1969-W860','1969-06-20 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 21026,'1969-W861','1969-06-20 17:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 21027,'1969-W862','1969-06-20 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 21028,'1969-W863','1969-06-20 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 21029,'1969-W864','1969-06-20 18:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 2103,'1954-A120','1954-03-22 00:00:00','atmospheric rocket','success',836,'2245A','','','','Test',1895,'',10 from dual union all
select 21030,'1969-W865','1969-06-20 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',69 from dual union all
select 21031,'1969-W866','1969-06-20 19:14:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',56 from dual union all
select 21032,'1969-W867','1969-06-20 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 21033,'1969-W868','1969-06-20 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 21034,'1969-W869','1969-06-20 21:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',54 from dual union all
select 21035,'1969-W87','1969-01-17 03:34:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 21036,'1969-W870','1969-06-20 21:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',57 from dual union all
select 21037,'1969-W871','1969-06-23 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 21038,'1969-W872','1969-06-23 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',60 from dual union all
select 21039,'1969-W873','1969-06-23 15:17:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',55 from dual union all
select 2104,'1954-A121','1954-03-23 00:00:00','military missile','success',771,'GM-12445','','','','Test',222,'',12 from dual union all
select 21040,'1969-W874','1969-06-23 15:27:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',66 from dual union all
select 21041,'1969-W875','1969-06-23 16:02:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1363,'',58 from dual union all
select 21042,'1969-W876','1969-06-23 16:48:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',58 from dual union all
select 21043,'1969-W877','1969-06-23 17:30:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',63 from dual union all
select 21044,'1969-W878','1969-06-23 18:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',61 from dual union all
select 21045,'1969-W879','1969-06-23 21:03:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',53 from dual union all
select 21046,'1969-W88','1969-01-17 06:34:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',62 from dual union all
select 21047,'1969-W880','1969-06-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 21048,'1969-W881','1969-06-24 14:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',58 from dual union all
select 21049,'1969-W882','1969-06-24 14:59:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',63 from dual union all
select 2105,'1954-A122','1954-03-23 00:00:00','military missile','success',771,'GM-12444','','','','Test',222,'',12 from dual union all
select 21050,'1969-W883','1969-06-24 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 21051,'1969-W884','1969-06-24 18:13:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',66 from dual union all
select 21052,'1969-W885','1969-06-25 06:12:00','suborbital rocket','success',631,'TERLS-115','ISRO 11.31','TERLS-115','','Aeron',1467,'',64 from dual union all
select 21053,'1969-W886','1969-06-25 14:31:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',66 from dual union all
select 21054,'1969-W887','1969-06-25 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 21055,'1969-W888','1969-06-25 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 21056,'1969-W889','1969-06-25 14:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 21057,'1969-W89','1969-01-17 06:55:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 21058,'1969-W890','1969-06-25 15:00:00','suborbital rocket','success',1061,'','Starute','','','Meteo',276,'',59 from dual union all
select 21059,'1969-W891','1969-06-25 15:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',60 from dual union all
select 2106,'1954-A123','1954-03-24 00:00:00','military missile','unknown',339,'2T','','','','Test',1890,'',30 from dual union all
select 21060,'1969-W892','1969-06-25 16:02:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1363,'',62 from dual union all
select 21061,'1969-W893','1969-06-25 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',64 from dual union all
select 21062,'1969-W894','1969-06-25 20:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',75 from dual union all
select 21063,'1969-W895','1969-06-25 21:27:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',54 from dual union all
select 21064,'1969-W896','1969-06-25 21:57:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',62 from dual union all
select 21065,'1969-W897','1969-06-26 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',68 from dual union all
select 21066,'1969-W898','1969-06-26 15:12:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',61 from dual union all
select 21067,'1969-W899','1969-06-26 21:46:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',64 from dual;
insert into LAUNCH
select 21068,'1969-W90','1969-01-17 07:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',62 from dual union all
select 21069,'1969-W900','1969-06-27 14:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 2107,'1954-A124','1954-03-24 00:00:00','atmospheric rocket','success',442,'','Regulus II Model','','','Test',1821,'',10 from dual union all
select 21070,'1969-W901','1969-06-27 16:54:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',66 from dual union all
select 21071,'1969-W902','1969-06-27 18:00:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',72 from dual union all
select 21072,'1969-W903','1969-06-27 18:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',69 from dual union all
select 21073,'1969-W904','1969-06-27 18:51:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',63 from dual union all
select 21074,'1969-W905','1969-06-27 20:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',63 from dual union all
select 21075,'1969-W906','1969-06-27 21:22:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',54 from dual union all
select 21076,'1969-W907','1969-06-27 22:55:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',61 from dual union all
select 21077,'1969-W908','1969-06-30 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 21078,'1969-W909','1969-06-30 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',69 from dual union all
select 21079,'1969-W91','1969-01-17 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 2108,'1954-A125','1954-03-26 00:00:00','military missile','unknown',339,'3T','','','','Test',1895,'',30 from dual union all
select 21080,'1969-W910','1969-06-30 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',62 from dual union all
select 21081,'1969-W911','1969-06-30 14:51:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',56 from dual union all
select 21082,'1969-W912','1969-06-30 15:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 21083,'1969-W913','1969-06-30 21:01:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',57 from dual union all
select 21084,'1969-W914','1969-07-01 05:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 21085,'1969-W915','1969-07-01 18:31:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 21086,'1969-W916','1969-07-02 10:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 21087,'1969-W917','1969-07-02 14:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 21088,'1969-W918','1969-07-02 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',60 from dual union all
select 21089,'1969-W919','1969-07-02 15:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 2109,'1954-A126','1954-03-26 00:00:00','atmospheric rocket','success',968,'','Drag Model','','','Test',1821,'',10 from dual union all
select 21090,'1969-W92','1969-01-17 16:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 21091,'1969-W920','1969-07-02 15:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',69 from dual union all
select 21092,'1969-W921','1969-07-02 16:09:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',63 from dual union all
select 21093,'1969-W922','1969-07-02 16:45:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',61 from dual union all
select 21094,'1969-W923','1969-07-02 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 21095,'1969-W924','1969-07-02 18:00:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1307,'',62 from dual union all
select 21096,'1969-W925','1969-07-02 20:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 21097,'1969-W926','1969-07-02 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',54 from dual union all
select 21098,'1969-W927','1969-07-03 15:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',53 from dual union all
select 21099,'1969-W928','1969-07-03 18:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',308,'',58 from dual union all
select 211,'1944-S191','1944-10-10 09:55:00','military missile','success',1470,'19046','WH7','','','Test',661,'',90 from dual union all
select 2110,'1954-A127','1954-03-26 00:00:00','atmospheric rocket','unknown',576,'166','','','','Test',1895,'',10 from dual union all
select 21100,'1969-W929','1969-07-03 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',51 from dual union all
select 21101,'1969-W93','1969-01-17 16:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',64 from dual union all
select 21102,'1969-W930','1969-07-03 19:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',62 from dual union all
select 21103,'1969-W931','1969-07-03 21:24:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',50 from dual union all
select 21104,'1969-W932','1969-07-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 21105,'1969-W933','1969-07-07 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',63 from dual union all
select 21106,'1969-W934','1969-07-07 14:49:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',35,'',58 from dual union all
select 21107,'1969-W935','1969-07-07 16:15:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1363,'',56 from dual union all
select 21108,'1969-W936','1969-07-07 17:13:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 21109,'1969-W937','1969-07-08 18:33:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 2111,'1954-A128','1954-03-26 00:00:00','atmospheric rocket','unknown',576,'490','','','','Test',1895,'',10 from dual union all
select 21110,'1969-W938','1969-07-09 14:19:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 21111,'1969-W939','1969-07-09 15:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 21112,'1969-W94','1969-01-17 17:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 21113,'1969-W940','1969-07-09 15:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',60 from dual union all
select 21114,'1969-W941','1969-07-09 16:48:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',56 from dual union all
select 21115,'1969-W942','1969-07-09 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 21116,'1969-W943','1969-07-09 18:18:00','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',67 from dual union all
select 21117,'1969-W944','1969-07-09 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',56 from dual union all
select 21118,'1969-W945','1969-07-10 19:41:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',66 from dual union all
select 21119,'1969-W946','1969-07-11 01:19:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',60 from dual union all
select 2112,'1954-A129','1954-03-26 00:00:00','military missile','success',771,'GM-12446','','','','Test',222,'',12 from dual union all
select 21120,'1969-W947','1969-07-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 21121,'1969-W948','1969-07-11 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',61 from dual union all
select 21122,'1969-W949','1969-07-11 15:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',35,'',62 from dual union all
select 21123,'1969-W95','1969-01-17 18:35:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',63 from dual union all
select 21124,'1969-W950','1969-07-11 16:47:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',63 from dual union all
select 21125,'1969-W951','1969-07-11 19:14:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',58 from dual union all
select 21126,'1969-W952','1969-07-11 23:22:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 21127,'1969-W953','1969-07-14 14:31:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 21128,'1969-W954','1969-07-14 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',63 from dual union all
select 21129,'1969-W955','1969-07-14 15:02:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',55 from dual union all
select 2113,'1954-A13','1954-01-12 00:00:00','military missile','unknown',339,'13EN','EU-13','','','Test',1895,'',30 from dual union all
select 21130,'1969-W956','1969-07-14 15:45:00','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',62 from dual union all
select 21131,'1969-W957','1969-07-14 18:45:00','suborbital rocket','success',1061,'','Met Sphere,Datasonde','','','Meteo',1895,'',71 from dual union all
select 21132,'1969-W958','1969-07-14 19:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',52 from dual union all
select 21133,'1969-W959','1969-07-14 19:07:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',56 from dual union all
select 21134,'1969-W96','1969-01-17 19:33:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 21135,'1969-W960','1969-07-14 19:07:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',56 from dual union all
select 21136,'1969-W961','1969-07-15 14:00:00','suborbital rocket','success',1061,'','Starute','','','Meteo',276,'',62 from dual union all
select 21137,'1969-W962','1969-07-15 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',59 from dual union all
select 21138,'1969-W963','1969-07-15 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 21139,'1969-W964','1969-07-15 21:04:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',50 from dual union all
select 2114,'1954-A130','1954-03-26 00:00:00','atmospheric rocket','success',836,'2358E','','','','Test',1895,'',10 from dual union all
select 21140,'1969-W965','1969-07-16 05:30:00','suborbital rocket','success',631,'TERLS-116','ISRO 11.32','TERLS-116','','Aeron',1467,'',64 from dual union all
select 21141,'1969-W966','1969-07-16 14:34:00','suborbital rocket','unknown',631,'SUPARCO','','','','Aeron',1419,'',65 from dual union all
select 21142,'1969-W967','1969-07-16 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 21143,'1969-W968','1969-07-16 15:11:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 21144,'1969-W969','1969-07-16 15:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 21145,'1969-W97','1969-01-17 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 21146,'1969-W970','1969-07-16 15:59:59','suborbital rocket','success',1061,'','Met Sphere,Datasonde','','','Meteo',1895,'',63 from dual union all
select 21147,'1969-W971','1969-07-16 16:45:59','suborbital rocket','success',631,'','Met Para,Datasonde','','','Meteo',428,'',67 from dual union all
select 21148,'1969-W972','1969-07-16 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 21149,'1969-W973','1969-07-16 18:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',50 from dual union all
select 2115,'1954-A131','1954-03-26 00:00:00','atmospheric rocket','success',836,'1372T','','','','Test',1895,'',10 from dual union all
select 21150,'1969-W974','1969-07-16 21:56:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',58 from dual union all
select 21151,'1969-W975','1969-07-17 14:43:00','suborbital rocket','success',1061,'','Starute','','','Meteo',276,'',59 from dual union all
select 21152,'1969-W976','1969-07-17 17:40:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',54 from dual union all
select 21153,'1969-W977','1969-07-17 18:00:00','suborbital rocket','success',1061,'','Met Sphere,Datasonde','','','Meteo',1895,'',55 from dual union all
select 21154,'1969-W978','1969-07-17 19:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',68 from dual union all
select 21155,'1969-W979','1969-07-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 21156,'1969-W98','1969-01-17 20:58:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',54 from dual union all
select 21157,'1969-W980','1969-07-18 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',64 from dual;
insert into LAUNCH
select 21158,'1969-W981','1969-07-18 15:51:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 21159,'1969-W982','1969-07-18 15:59:59','suborbital rocket','success',1061,'','Met Sphere,Datasonde','','','Meteo',1895,'',73 from dual union all
select 2116,'1954-A132','1954-03-26 00:00:00','atmospheric rocket','success',836,'1336E','','','','Test',1895,'',10 from dual union all
select 21160,'1969-W983','1969-07-18 17:00:00','suborbital rocket','success',1061,'','Met Sphere,Datasonde','','','Meteo',1895,'',62 from dual union all
select 21161,'1969-W984','1969-07-18 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 21162,'1969-W985','1969-07-18 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',855,'',50 from dual union all
select 21163,'1969-W986','1969-07-22 19:14:59','suborbital rocket','success',1061,'','Met Sphere,Datasonde','','','Meteo',1895,'',69 from dual union all
select 21164,'1969-W987','1969-07-22 21:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',52 from dual union all
select 21165,'1969-W988','1969-07-22 21:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 21166,'1969-W989','1969-07-23 05:50:00','suborbital rocket','success',631,'TERLS-117','ISRO 11.33','TERLS-117','','Aeron',1467,'',64 from dual union all
select 21167,'1969-W99','1969-01-17 22:41:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 21168,'1969-W990','1969-07-23 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 21169,'1969-W991','1969-07-23 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',60 from dual union all
select 2117,'1954-A133','1954-03-29 00:00:00','atmospheric rocket','unknown',576,'443','','','','Test',1895,'',10 from dual union all
select 21170,'1969-W992','1969-07-23 16:02:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',60 from dual union all
select 21171,'1969-W993','1969-07-23 17:42:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 21172,'1969-W994','1969-07-23 18:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',53 from dual union all
select 21173,'1969-W995','1969-07-23 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1057,'',59 from dual union all
select 21174,'1969-W996','1969-07-23 20:26:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',56 from dual union all
select 21175,'1969-W997','1969-07-23 21:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',67 from dual union all
select 21176,'1969-W998','1969-07-23 21:43:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 21177,'1969-W999','1969-07-24 14:30:00','suborbital rocket','success',1061,'','Met Sphere,Datasonde','','','Meteo',1895,'',70 from dual union all
select 21178,'1969-Y01','1969-01-01 00:00:00','suborbital rocket','success',1287,'','DFH test','','','Tech',819,'',81 from dual union all
select 21179,'1969-Y02','1969-01-02 03:50:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',89 from dual union all
select 2118,'1954-A134','1954-03-30 00:00:00','military missile','success',771,'GM-12436','','','','Test',222,'',12 from dual union all
select 21180,'1969-Y03','1969-01-02 09:42:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',88 from dual union all
select 21181,'1969-Y04','1969-01-02 16:40:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',93 from dual union all
select 21182,'1969-Y05','1969-01-02 16:45:00','suborbital rocket','success',725,'','','','','Aeron',1578,'SHIP-SOV',91 from dual union all
select 21183,'1969-Y06','1969-01-02 22:05:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',92 from dual union all
select 21184,'1969-Y07','1969-01-03 00:00:00','suborbital rocket','success',725,'','','','','Aeron',1578,'SHIP-SOV',91 from dual union all
select 21185,'1969-Y08','1969-01-07 03:00:00','suborbital rocket','success',725,'','','','','Aeron',1578,'SHIP-SOV',94 from dual union all
select 21186,'1969-Y09','1969-01-07 03:52:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',90 from dual union all
select 21187,'1969-Y10','1969-01-07 09:50:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',91 from dual union all
select 21188,'1969-Y100','1969-02-19 13:48:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',84 from dual union all
select 21189,'1969-Y101','1969-02-19 19:30:00','suborbital rocket','success',1284,'','Met Sphere','','','Aeron',1895,'',120 from dual union all
select 2119,'1954-A135','1954-03-30 00:00:00','military missile','success',771,'GM-12432','','','','Test',222,'',12 from dual union all
select 21190,'1969-Y102','1969-02-22 09:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',94 from dual union all
select 21191,'1969-Y103','1969-03-03 18:23:59','suborbital rocket','success',1061,'','MK-1','','','Meteo',276,'',100 from dual union all
select 21192,'1969-Y104','1969-03-13 15:55:00','suborbital rocket','success',1061,'','MK-1','','','Meteo',276,'',81 from dual union all
select 21193,'1969-Y105','1969-03-29 12:30:00','suborbital rocket','unknown',725,'','','','','Plasma',636,'',85 from dual union all
select 21194,'1969-Y106','1969-03-29 13:34:59','suborbital rocket','success',725,'','Particle study 2','','','Aeron/Particles',636,'',83 from dual union all
select 21195,'1969-Y107','1969-03-29 15:30:00','suborbital rocket','success',725,'','Particle study 3','','','Aeron/Particles',636,'',83 from dual union all
select 21196,'1969-Y108','1969-04-09 10:03:00','suborbital rocket','success',725,'Sun-Atm 69','Particle study 4','','','Aeron/Particles',636,'',89 from dual union all
select 21197,'1969-Y109','1969-04-14 02:03:00','suborbital rocket','success',258,'','325-4','','','Ionos',428,'',101 from dual union all
select 21198,'1969-Y11','1969-01-07 10:00:00','suborbital rocket','success',725,'','','','','Aeron',1578,'SHIP-SOV',90 from dual union all
select 21199,'1969-Y110','1969-04-14 05:19:00','suborbital rocket','success',258,'','325-5','','','Ionos',428,'',99 from dual union all
select 212,'1944-S192','1944-10-10 11:42:00','military missile','success',1470,'19040','WH5','','','Test',661,'',90 from dual union all
select 2120,'1954-A136','1954-03-30 00:00:00','atmospheric rocket','success',836,'1295R  AGC motor','Nike 269R','','','Test',1895,'',10 from dual union all
select 21200,'1969-Y111','1969-04-14 21:08:00','suborbital rocket','success',258,'','325-6','','','Ionos',428,'',110 from dual union all
select 21201,'1969-Y112','1969-04-15 18:49:00','suborbital rocket','success',725,'M-100','Particle flux 24','','','Plasma',1010,'',94 from dual union all
select 21202,'1969-Y113','1969-04-16 08:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',90 from dual union all
select 21203,'1969-Y114','1969-04-16 10:03:00','suborbital rocket','unknown',725,'Sun-Atm 69','','','','Aeron',636,'',91 from dual union all
select 21204,'1969-Y115','1969-04-18 13:30:00','suborbital rocket','success',725,'Sun-Atm 69','Particle study 5','','','Aeron/Particles',636,'',87 from dual union all
select 21205,'1969-Y116','1969-04-18 15:50:00','suborbital rocket','success',725,'Sun-Atm 69','Particle study 6','','','Aeron/Particles',636,'',88 from dual union all
select 21206,'1969-Y117','1969-04-19 00:10:00','suborbital rocket','success',258,'AMM-BA-1','O/OH profile','','','Aeron',428,'',104 from dual union all
select 21207,'1969-Y118','1969-04-19 01:17:00','suborbital rocket','success',258,'AMM-BA-2','O/OH profile','','','Aeron',428,'',103 from dual union all
select 21208,'1969-Y119','1969-04-19 13:32:00','suborbital rocket','success',725,'Sun-Atm 69','Particle study 7','','','Aeron/Particles',636,'',90 from dual union all
select 21209,'1969-Y12','1969-01-07 10:01:00','suborbital rocket','success',725,'','','','','Aeron',636,'',94 from dual union all
select 2121,'1954-A137','1954-03-31 00:00:00','atmospheric rocket','success',836,'1623E','','','','Test',1895,'',10 from dual union all
select 21210,'1969-Y120','1969-04-19 15:29:00','suborbital rocket','success',725,'Sun-Atm 69','Particle study 8','','','Aeron/Particles',636,'',81 from dual union all
select 21211,'1969-Y121','1969-04-21 04:15:00','suborbital rocket','success',1284,'','Met Sphere','','','Aeron',1895,'',120 from dual union all
select 21212,'1969-Y122','1969-04-20 12:34:00','suborbital rocket','unknown',725,'Sun-Atm 69','','','','Plasma',636,'',85 from dual union all
select 21213,'1969-Y123','1969-04-20 15:54:00','suborbital rocket','success',725,'Sun-Atm 69','Particle study 9','','','Aeron/Particles',636,'',90 from dual union all
select 21214,'1969-Y124','1969-04-21 13:36:00','suborbital rocket','success',725,'Sun-Atm 69','Particle study 10','','','Aeron/Particles',636,'',89 from dual union all
select 21215,'1969-Y125','1969-04-21 15:56:00','suborbital rocket','success',725,'Sun-Atm 69','Particle study 11','','','Aeron/Particles',636,'',87 from dual union all
select 21216,'1969-Y126','1969-04-23 08:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',97 from dual union all
select 21217,'1969-Y127','1969-04-23 12:10:00','suborbital rocket','success',725,'Sun-Atm 69','Particle study 12','','','Aeron/Particles',636,'',88 from dual union all
select 21218,'1969-Y128','1969-04-23 16:00:00','suborbital rocket','success',725,'Sun-Atm 69','Particle study 13','','','Aeron/Particles',636,'',88 from dual union all
select 21219,'1969-Y129','1969-04-24 00:45:00','suborbital rocket','success',725,'M-100','Particle flux 25','','','Plasma',1010,'',94 from dual union all
select 2122,'1954-A138','1954-03-31 00:00:00','atmospheric rocket','success',836,'1708E','','','','Test',1895,'',10 from dual union all
select 21220,'1969-Y13','1969-01-07 17:37:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',83 from dual union all
select 21221,'1969-Y130','1969-04-28 04:19:00','suborbital rocket','success',1506,'Sphere 6','Met Sphere','','','Aeron',1895,'',146 from dual union all
select 21222,'1969-Y131','1969-04-30 04:01:00','suborbital rocket','success',725,'M-100','Particle study 14','','','Aeron/Particles',636,'',88 from dual union all
select 21223,'1969-Y132','1969-04-30 08:00:00','suborbital rocket','success',725,'M-100','Particle flux 26','','','Plasma',1010,'',94 from dual union all
select 21224,'1969-Y133','1969-05-07 03:03:00','suborbital rocket','success',725,'M-100','Particle study 15','','','Aeron/Particles',636,'',89 from dual union all
select 21225,'1969-Y134','1969-05-09 17:00:00','suborbital rocket','success',1506,'Sphere 8','Met Sphere','','','Aeron',1895,'',94 from dual union all
select 21226,'1969-Y135','1969-05-11 02:00:00','suborbital rocket','success',1506,'Sphere 9','Met Sphere','','','Aeron',1895,'',145 from dual union all
select 21227,'1969-Y136','1969-05-11 07:00:00','suborbital rocket','success',1284,'','Met Sphere','','','Aeron',1895,'',126 from dual union all
select 21228,'1969-Y137','1969-05-14 08:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',92 from dual union all
select 21229,'1969-Y138','1969-05-14 16:08:59','suborbital rocket','success',1061,'','MK-1','','','Meteo',276,'',92 from dual union all
select 2123,'1954-A139','1954-04-01 00:00:00','atmospheric rocket','unknown',576,'86A','','','','Test',1895,'',10 from dual union all
select 21230,'1969-Y139','1969-05-18 20:30:00','suborbital rocket','success',1061,'','MK-1','','','Meteo',276,'',90 from dual union all
select 21231,'1969-Y14','1969-01-07 21:47:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',90 from dual union all
select 21232,'1969-Y140','1969-05-19 16:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1593,'SHIP-SOV',80 from dual union all
select 21233,'1969-Y141','1969-05-19 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1594,'SHIP-SOV',86 from dual union all
select 21234,'1969-Y142','1969-05-20 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1595,'SHIP-SOV',91 from dual union all
select 21235,'1969-Y143','1969-05-21 08:03:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',92 from dual union all
select 21236,'1969-Y144','1969-05-21 11:03:00','suborbital rocket','success',725,'M-100','Particle study 16','','','Aeron/Particles',636,'',86 from dual union all
select 21237,'1969-Y145','1969-05-21 16:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1596,'SHIP-SOV',83 from dual union all
select 21238,'1969-Y146','1969-05-21 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1597,'SHIP-SOV',82 from dual union all
select 21239,'1969-Y147','1969-05-22 04:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1597,'SHIP-SOV',88 from dual union all
select 2124,'1954-A14','1954-01-12 00:00:00','atmospheric rocket','unknown',576,'500C','','','','Test',1895,'',10 from dual union all
select 21240,'1969-Y148','1969-05-22 10:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1598,'SHIP-SOV',88 from dual union all
select 21241,'1969-Y149','1969-05-23 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1599,'SHIP-SOV',89 from dual union all
select 21242,'1969-Y15','1969-01-08 09:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',94 from dual union all
select 21243,'1969-Y150','1969-05-24 11:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1600,'SHIP-SOV',89 from dual union all
select 21244,'1969-Y151','1969-05-24 16:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1601,'SHIP-SOV',92 from dual union all
select 21245,'1969-Y152','1969-05-25 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 21246,'1969-Y153','1969-05-25 04:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1602,'SHIP-SOV',92 from dual union all
select 21247,'1969-Y154','1969-05-25 23:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1591,'SHIP-SOV',92 from dual;
insert into LAUNCH
select 21248,'1969-Y155','1969-05-26 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1603,'SHIP-SOV',92 from dual union all
select 21249,'1969-Y156','1969-05-27 16:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1604,'SHIP-SOV',88 from dual union all
select 2125,'1954-A140','1954-04-01 00:00:00','atmospheric rocket','unknown',576,'161A','','','','Test',1895,'',10 from dual union all
select 21250,'1969-Y157','1969-05-28 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 21251,'1969-Y158','1969-05-28 08:35:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',91 from dual union all
select 21252,'1969-Y159','1969-05-28 11:26:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',87 from dual union all
select 21253,'1969-Y16','1969-01-10 00:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',80 from dual union all
select 21254,'1969-Y160','1969-05-30 03:02:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',86 from dual union all
select 21255,'1969-Y161','1969-05-31 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 21256,'1969-Y162','1969-06-02 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1605,'SHIP-SOV',91 from dual union all
select 21257,'1969-Y163','1969-06-03 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1606,'SHIP-SOV',91 from dual union all
select 21258,'1969-Y164','1969-06-04 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 21259,'1969-Y165','1969-06-04 04:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1607,'SHIP-SOV',90 from dual union all
select 2126,'1954-A141','1954-04-02 00:00:00','atmospheric rocket','success',836,'1139T','','','','Test',1895,'',10 from dual union all
select 21260,'1969-Y166','1969-06-04 04:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',91 from dual union all
select 21261,'1969-Y167','1969-06-04 08:11:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',93 from dual union all
select 21262,'1969-Y168','1969-06-06 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1609,'SHIP-SOV',92 from dual union all
select 21263,'1969-Y169','1969-06-07 04:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1609,'SHIP-SOV',89 from dual union all
select 21264,'1969-Y17','1969-01-11 03:51:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',91 from dual union all
select 21265,'1969-Y170','1969-06-07 10:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1610,'SHIP-SOV',93 from dual union all
select 21266,'1969-Y171','1969-06-07 16:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1611,'SHIP-SOV',92 from dual union all
select 21267,'1969-Y172','1969-06-08 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1612,'SHIP-SOV',89 from dual union all
select 21268,'1969-Y173','1969-06-09 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1613,'SHIP-SOV',87 from dual union all
select 21269,'1969-Y174','1969-06-10 04:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1614,'SHIP-SOV',86 from dual union all
select 2127,'1954-A142','1954-04-02 00:00:00','atmospheric rocket','success',836,'1489E','','','','Test',1895,'',10 from dual union all
select 21270,'1969-Y175','1969-06-10 22:14:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',87 from dual union all
select 21271,'1969-Y176','1969-06-11 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 21272,'1969-Y177','1969-06-11 08:05:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',90 from dual union all
select 21273,'1969-Y178','1969-06-11 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1615,'SHIP-SOV',85 from dual union all
select 21274,'1969-Y179','1969-06-12 21:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',84 from dual union all
select 21275,'1969-Y18','1969-01-11 09:45:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',94 from dual union all
select 21276,'1969-Y180','1969-06-12 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1618,'SHIP-SOV',90 from dual union all
select 21277,'1969-Y181','1969-06-13 04:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1616,'SHIP-SOV',91 from dual union all
select 21278,'1969-Y182','1969-06-13 16:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1617,'SHIP-SOV',86 from dual union all
select 21279,'1969-Y183','1969-06-18 08:37:00','suborbital rocket','success',725,'M-100','Particle flux 27','','','Plasma',1010,'',91 from dual union all
select 2128,'1954-A143','1954-04-02 00:00:00','atmospheric rocket','success',836,'1734E','','','','Test',1895,'',10 from dual union all
select 21280,'1969-Y184','1969-06-25 08:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',92 from dual union all
select 21281,'1969-Y185','1969-06-25 10:01:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',90 from dual union all
select 21282,'1969-Y186','1969-06-27 00:00:00','suborbital rocket','success',1284,'','','','','Aeron',276,'',91 from dual union all
select 21283,'1969-Y187','1969-07-02 09:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',94 from dual union all
select 21284,'1969-Y188','1969-07-02 10:01:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',87 from dual union all
select 21285,'1969-Y189','1969-07-02 15:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',82 from dual union all
select 21286,'1969-Y19','1969-01-11 10:00:00','suborbital rocket','success',725,'','','','','Aeron',1578,'SHIP-SOV',92 from dual union all
select 21287,'1969-Y190','1969-07-09 09:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',90 from dual union all
select 21288,'1969-Y191','1969-07-14 13:09:59','suborbital rocket','success',1506,'','Met Sphere','','','Meteo',276,'',91 from dual union all
select 21289,'1969-Y192','1969-07-16 09:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',92 from dual union all
select 2129,'1954-A144','1954-04-06 00:00:00','atmospheric rocket','unknown',576,'452M','','','','Test',1895,'',10 from dual union all
select 21290,'1969-Y193','1969-07-16 10:04:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',86 from dual union all
select 21291,'1969-Y194','1969-07-16 16:26:59','suborbital rocket','success',1506,'','Met Sphere','','','Meteo',276,'',91 from dual union all
select 21292,'1969-Y195','1969-07-23 09:03:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',90 from dual union all
select 21293,'1969-Y196','1969-07-23 10:01:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',88 from dual union all
select 21294,'1969-Y197','1969-07-23 21:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',90 from dual union all
select 21295,'1969-Y198','1969-07-24 21:02:00','suborbital rocket','unknown',725,'','','','','Plasma',636,'',85 from dual union all
select 21296,'1969-Y199','1969-07-25 02:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',134,'ZUB',94 from dual union all
select 21297,'1969-Y20','1969-01-11 15:56:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',93 from dual union all
select 21298,'1969-Y200','1969-07-28 17:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',81 from dual union all
select 21299,'1969-Y201','1969-07-30 02:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',134,'ZUB',84 from dual union all
select 213,'1944-S193','1944-10-10 12:31:00','military missile','success',1470,'19044','WH8','','','Test',661,'',90 from dual union all
select 2130,'1954-A145','1954-04-06 00:00:00','atmospheric rocket','unknown',576,'454M','','','','Test',1895,'',10 from dual union all
select 21300,'1969-Y202','1969-07-30 03:01:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',91 from dual union all
select 21301,'1969-Y203','1969-07-30 09:00:00','suborbital rocket','unknown',725,'MR-12','','','','Aeron',1010,'',90 from dual union all
select 21302,'1969-Y204','1969-08-06 02:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',134,'ZUB',94 from dual union all
select 21303,'1969-Y205','1969-08-06 09:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',89 from dual union all
select 21304,'1969-Y206','1969-08-06 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',81 from dual union all
select 21305,'1969-Y207','1969-08-13 02:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',134,'ZUB',91 from dual union all
select 21306,'1969-Y208','1969-08-20 02:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',134,'ZUB',88 from dual union all
select 21307,'1969-Y209','1969-08-20 09:00:00','suborbital rocket','unknown',725,'MP-12','','','','Aeron',1010,'',90 from dual union all
select 21308,'1969-Y21','1969-01-11 16:30:00','suborbital rocket','success',725,'','','','','Aeron',1578,'SHIP-SOV',92 from dual union all
select 21309,'1969-Y210','1969-08-27 02:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',134,'ZUB',90 from dual union all
select 2131,'1954-A146','1954-04-06 00:00:00','atmospheric rocket','success',836,'1316A','','','','Test',1895,'',10 from dual union all
select 21310,'1969-Y211','1969-08-27 10:37:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',90 from dual union all
select 21311,'1969-Y212','1969-09-03 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 21312,'1969-Y213','1969-09-03 09:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',93 from dual union all
select 21313,'1969-Y214','1969-09-03 10:03:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',93 from dual union all
select 21314,'1969-Y215','1969-09-10 02:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1622,'ZUB',91 from dual union all
select 21315,'1969-Y216','1969-09-10 09:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',93 from dual union all
select 21316,'1969-Y217','1969-09-16 19:50:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1621,'SHIP-SOV',88 from dual union all
select 21317,'1969-Y218','1969-09-16 21:43:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1621,'SHIP-SOV',90 from dual union all
select 21318,'1969-Y219','1969-09-17 02:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1622,'SHIP-SOV',89 from dual union all
select 21319,'1969-Y22','1969-01-11 21:45:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',91 from dual union all
select 2132,'1954-A147','1954-04-07 00:00:00','atmospheric rocket','success',836,'1559E','','','','Test',1895,'',10 from dual union all
select 21320,'1969-Y220','1969-09-17 09:10:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',98 from dual union all
select 21321,'1969-Y221','1969-09-17 10:02:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',90 from dual union all
select 21322,'1969-Y222','1969-09-17 19:50:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1621,'SHIP-SOV',90 from dual union all
select 21323,'1969-Y223','1969-09-17 21:40:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1621,'SHIP-SOV',87 from dual union all
select 21324,'1969-Y224','1969-09-18 11:45:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1621,'SHIP-SOV',89 from dual union all
select 21325,'1969-Y225','1969-09-18 21:59:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1621,'SHIP-SOV',87 from dual union all
select 21326,'1969-Y226','1969-09-18 23:08:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1621,'SHIP-SOV',89 from dual union all
select 21327,'1969-Y227','1969-09-19 20:10:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1622,'SHIP-SOV',88 from dual union all
select 21328,'1969-Y228','1969-09-19 22:32:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1621,'SHIP-SOV',88 from dual union all
select 21329,'1969-Y229','1969-09-20 20:08:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1621,'SHIP-SOV',90 from dual union all
select 2133,'1954-A148','1954-04-07 00:00:00','atmospheric rocket','success',836,'1477E','','','','Test',1895,'',10 from dual union all
select 21330,'1969-Y23','1969-01-11 22:00:00','suborbital rocket','success',725,'MR-12','','','','Aeron',1578,'SHIP-SOV',89 from dual union all
select 21331,'1969-Y230','1969-09-24 02:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1622,'SHIP-SOV',95 from dual union all
select 21332,'1969-Y231','1969-09-24 04:08:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',89 from dual union all
select 21333,'1969-Y232','1969-09-24 09:20:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',87 from dual union all
select 21334,'1969-Y233','1969-09-25 23:15:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1322,'SHIP-SOV',90 from dual union all
select 21335,'1969-Y234','1969-09-26 20:05:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1322,'SHIP-SOV',89 from dual union all
select 21336,'1969-Y235','1969-09-26 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1322,'SHIP-SOV',91 from dual union all
select 21337,'1969-Y236','1969-09-27 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1322,'SHIP-SOV',88 from dual;
insert into LAUNCH
select 21338,'1969-Y237','1969-09-28 20:03:00','suborbital rocket','unknown',725,'','','','','Aeron',1322,'SHIP-SOV',89 from dual union all
select 21339,'1969-Y238','1969-09-28 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1322,'SHIP-SOV',90 from dual union all
select 2134,'1954-A149','1954-04-07 00:00:00','atmospheric rocket','success',836,'1319R  Bell motor','Nike 270R','','','Test',1895,'',10 from dual union all
select 21340,'1969-Y239','1969-09-29 21:23:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1322,'SHIP-SOV',88 from dual union all
select 21341,'1969-Y24','1969-01-13 09:00:00','suborbital rocket','success',725,'MR-12','','','','Aeron',1010,'',88 from dual union all
select 21342,'1969-Y240','1969-09-29 22:31:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1322,'SHIP-SOV',89 from dual union all
select 21343,'1969-Y241','1969-10-01 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 21344,'1969-Y242','1969-10-01 16:18:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',100 from dual union all
select 21345,'1969-Y243','1969-10-02 14:01:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 21346,'1969-Y244','1969-10-03 08:41:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 21347,'1969-Y245','1969-10-08 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 21348,'1969-Y246','1969-10-08 04:05:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 21349,'1969-Y247','1969-10-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 2135,'1954-A15','1954-01-12 00:00:00','atmospheric rocket','success',836,'1202E','','','','Test',1895,'',10 from dual union all
select 21350,'1969-Y248','1969-10-10 01:19:00','suborbital rocket','success',258,'NASA 15.53GI','','','','Plasma',1315,'',110 from dual union all
select 21351,'1969-Y249','1969-10-11 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1620,'SHIP-SOV',87 from dual union all
select 21352,'1969-Y25','1969-01-14 19:05:00','suborbital rocket','success',1506,'Sphere 1','','','','Aeron',1895,'',147 from dual union all
select 21353,'1969-Y250','1969-10-11 21:54:00','suborbital rocket','unknown',725,'','','','','Aeron',1622,'SHIP-SOV',88 from dual union all
select 21354,'1969-Y251','1969-10-12 19:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1620,'SHIP-SOV',88 from dual union all
select 21355,'1969-Y252','1969-10-12 21:44:00','suborbital rocket','unknown',725,'','','','','Aeron',1620,'SHIP-SOV',87 from dual union all
select 21356,'1969-Y253','1969-10-13 19:55:00','suborbital rocket','unknown',725,'','','','','Aeron',1620,'SHIP-SOV',87 from dual union all
select 21357,'1969-Y254','1969-10-13 21:55:00','suborbital rocket','unknown',725,'','','','','Aeron',1620,'SHIP-SOV',87 from dual union all
select 21358,'1969-Y255','1969-10-14 21:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1620,'SHIP-SOV',87 from dual union all
select 21359,'1969-Y256','1969-10-15 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 2136,'1954-A150','1954-04-07 00:00:00','atmospheric rocket','success',836,'1430E','','','','Test',1895,'',10 from dual union all
select 21360,'1969-Y257','1969-10-15 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 21361,'1969-Y258','1969-10-15 19:57:00','suborbital rocket','unknown',725,'','','','','Aeron',1620,'SHIP-SOV',89 from dual union all
select 21362,'1969-Y259','1969-10-15 21:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1620,'SHIP-SOV',93 from dual union all
select 21363,'1969-Y26','1969-01-15 08:08:00','suborbital rocket','success',725,'','','','','Aeron',636,'',90 from dual union all
select 21364,'1969-Y260','1969-10-16 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 21365,'1969-Y261','1969-10-16 10:02:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 21366,'1969-Y262','1969-10-16 15:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',82 from dual union all
select 21367,'1969-Y263','1969-10-18 21:36:00','suborbital rocket','unknown',725,'','','','','Aeron',1619,'SHIP-SOV',88 from dual union all
select 21368,'1969-Y264','1969-10-18 22:36:00','suborbital rocket','unknown',725,'','','','','Aeron',1619,'SHIP-SOV',87 from dual union all
select 21369,'1969-Y265','1969-10-19 19:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1619,'SHIP-SOV',91 from dual union all
select 2137,'1954-A151','1954-04-08 00:00:00','atmospheric rocket','unknown',576,'296M','','','','Test',1895,'',10 from dual union all
select 21370,'1969-Y266','1969-10-19 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1619,'SHIP-SOV',83 from dual union all
select 21371,'1969-Y267','1969-10-20 19:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1619,'SHIP-SOV',95 from dual union all
select 21372,'1969-Y268','1969-10-20 22:17:00','suborbital rocket','unknown',725,'','','','','Aeron',1619,'SHIP-SOV',94 from dual union all
select 21373,'1969-Y269','1969-10-21 09:49:00','suborbital rocket','unknown',725,'','','','','Aeron',1619,'SHIP-SOV',89 from dual union all
select 21374,'1969-Y27','1969-01-15 09:28:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',90 from dual union all
select 21375,'1969-Y270','1969-10-21 18:54:00','suborbital rocket','unknown',725,'','','','','Aeron',1619,'SHIP-SOV',88 from dual union all
select 21376,'1969-Y271','1969-10-21 20:58:00','suborbital rocket','unknown',725,'','','','','Aeron',1619,'SHIP-SOV',88 from dual union all
select 21377,'1969-Y272','1969-10-22 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',96 from dual union all
select 21378,'1969-Y273','1969-10-22 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 21379,'1969-Y274','1969-10-22 19:35:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',81 from dual union all
select 2138,'1954-A152','1954-04-08 00:00:00','atmospheric rocket','unknown',576,'470M','','','','Test',1895,'',10 from dual union all
select 21380,'1969-Y275','1969-10-29 00:00:00','suborbital rocket','success',257,'NASA 15.42UI','','','','Ionos',1821,'',110 from dual union all
select 21381,'1969-Y276','1969-10-29 02:07:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 21382,'1969-Y277','1969-10-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 21383,'1969-Y278','1969-11-05 02:07:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 21384,'1969-Y279','1969-11-05 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 21385,'1969-Y28','1969-01-15 19:30:00','suborbital rocket','success',1506,'Sphere 2','Met Sphere','','','Aeron',1895,'',147 from dual union all
select 21386,'1969-Y280','1969-11-12 02:19:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 21387,'1969-Y281','1969-11-12 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 21388,'1969-Y282','1969-11-12 15:20:00','suborbital rocket','success',1061,'','MK-1','','','Meteo',276,'',89 from dual union all
select 21389,'1969-Y283','1969-11-13 21:31:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',93 from dual union all
select 2139,'1954-A153','1954-04-08 00:00:00','atmospheric rocket','success',836,'1369T','','','','Test',1895,'',10 from dual union all
select 21390,'1969-Y284','1969-11-19 01:55:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 21391,'1969-Y285','1969-11-19 09:03:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 21392,'1969-Y286','1969-11-26 01:55:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 21393,'1969-Y287','1969-11-26 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 21394,'1969-Y288','1969-11-27 10:03:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 21395,'1969-Y289','1969-12-10 01:55:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 21396,'1969-Y29','1969-01-16 09:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',91 from dual union all
select 21397,'1969-Y290','1969-12-11 10:01:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 21398,'1969-Y291','1969-12-16 14:43:00','suborbital rocket','success',1188,'Su25H','','','','Ionospheric',1447,'',89 from dual union all
select 21399,'1969-Y292','1969-12-17 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 214,'1944-S194','1944-10-10 14:38:00','military missile','success',1470,'18810','Ma229','','','Test',661,'',90 from dual union all
select 2140,'1954-A154','1954-04-09 00:00:00','atmospheric rocket','unknown',576,'501','','','','Test',1895,'',10 from dual union all
select 21400,'1969-Y293','1969-12-17 09:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 21401,'1969-Y294','1969-12-17 15:14:59','suborbital rocket','success',1061,'','MK-1','','','Meteo',276,'',91 from dual union all
select 21402,'1969-Y295','1969-12-18 04:07:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 21403,'1969-Y296','1969-12-24 04:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 21404,'1969-Y297','1969-12-24 04:52:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 21405,'1969-Y298','1969-12-29 17:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 21406,'1969-Y299','1969-12-31 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',96 from dual union all
select 21407,'1969-Y30','1969-01-17 09:05:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',91 from dual union all
select 21408,'1969-Y300','1969-12-31 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 21409,'1969-Y31','1969-01-18 09:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',92 from dual union all
select 2141,'1954-A155','1954-04-09 00:00:00','atmospheric rocket','unknown',576,'511','','','','Test',1895,'',10 from dual union all
select 21410,'1969-Y32','1969-01-18 19:23:30','suborbital rocket','success',258,'NASA 15.45II','RTG SA1/3-A69','','','Ionos',29,'',99 from dual union all
select 21411,'1969-Y33','1969-01-19 09:00:00','suborbital rocket','success',725,'','Particle flux 9','','','Aeron',1010,'',91 from dual union all
select 21412,'1969-Y34','1969-01-20 19:00:00','suborbital rocket','success',88,'','','','','Aeron',1363,'',81 from dual union all
select 21413,'1969-Y35','1969-01-22 05:15:00','suborbital rocket','success',88,'','','','','Aeron',1363,'',84 from dual union all
select 21414,'1969-Y36','1969-01-22 09:03:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',93 from dual union all
select 21415,'1969-Y37','1969-01-22 17:30:00','suborbital rocket','success',88,'','','','','Aeron',1363,'',94 from dual union all
select 21416,'1969-Y38','1969-01-23 03:50:00','suborbital rocket','success',725,'','','','','Aeron',1578,'SHIP-SOV',93 from dual union all
select 21417,'1969-Y39','1969-01-23 04:00:00','suborbital rocket','success',725,'','','','','Aeron',1590,'SHIP-SOV',93 from dual union all
select 21418,'1969-Y40','1969-01-23 05:03:00','suborbital rocket','unknown',88,'','','','','Aeron',1363,'',81 from dual union all
select 21419,'1969-Y41','1969-01-23 09:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',88 from dual union all
select 2142,'1954-A156','1954-04-09 00:00:00','military missile','success',771,'GM-12468','','','','Test',222,'',12 from dual union all
select 21420,'1969-Y42','1969-01-23 09:48:00','suborbital rocket','success',725,'','','','','Aeron',1590,'SHIP-SOV',93 from dual union all
select 21421,'1969-Y43','1969-01-23 10:00:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',89 from dual union all
select 21422,'1969-Y44','1969-01-23 16:20:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',92 from dual union all
select 21423,'1969-Y45','1969-01-23 17:00:00','suborbital rocket','unknown',88,'','','','','Aeron',1363,'',83 from dual union all
select 21424,'1969-Y46','1969-01-23 21:49:00','suborbital rocket','success',725,'','','','','Aeron',1578,'SHIP-SOV',94 from dual union all
select 21425,'1969-Y47','1969-01-24 09:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',94 from dual union all
select 21426,'1969-Y48','1969-01-24 15:58:00','suborbital rocket','success',725,'','','','','Aeron',1578,'SHIP-SOV',87 from dual union all
select 21427,'1969-Y49','1969-01-24 17:01:00','suborbital rocket','unknown',88,'','','','','Aeron',1363,'',80 from dual;
insert into LAUNCH
select 21428,'1969-Y50','1969-01-25 03:00:00','suborbital rocket','success',1284,'','Met Sphere','','','Aeron',1895,'',126 from dual union all
select 21429,'1969-Y51','1969-01-25 09:00:00','suborbital rocket','success',725,'','Particle flux 10','','','Aeron',1010,'',96 from dual union all
select 2143,'1954-A157','1954-04-09 00:00:00','military missile','success',771,'GM-12429','','','','Test',222,'',12 from dual union all
select 21430,'1969-Y52','1969-01-26 09:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',92 from dual union all
select 21431,'1969-Y53','1969-01-27 03:53:00','suborbital rocket','success',725,'','','','','Aeron',1578,'SHIP-SOV',91 from dual union all
select 21432,'1969-Y54','1969-01-27 04:00:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',92 from dual union all
select 21433,'1969-Y55','1969-01-27 09:43:00','suborbital rocket','success',725,'','Particle flux 11','','','Plasma',1010,'',90 from dual union all
select 21434,'1969-Y56','1969-01-27 09:45:00','suborbital rocket','success',725,'','','','','Aeron',1578,'SHIP-SOV',92 from dual union all
select 21435,'1969-Y57','1969-01-27 15:45:00','suborbital rocket','success',725,'','','','','Aeron',1578,'SHIP-SOV',96 from dual union all
select 21436,'1969-Y58','1969-01-27 16:00:00','suborbital rocket','unknown',88,'','','','','Aeron',1363,'',83 from dual union all
select 21437,'1969-Y59','1969-01-27 16:40:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',96 from dual union all
select 21438,'1969-Y60','1969-01-27 19:53:00','suborbital rocket','success',258,'NASA 15.47GT','Res Bay Eval','','','Test',1821,'',102 from dual union all
select 21439,'1969-Y61','1969-01-27 21:45:00','suborbital rocket','success',725,'','','','','Aeron',1578,'SHIP-SOV',96 from dual union all
select 2144,'1954-A158','1954-04-09 00:00:00','military missile','success',771,'GM-12436','','','','Test',222,'',12 from dual union all
select 21440,'1969-Y62','1969-01-27 22:00:00','suborbital rocket','success',1284,'','Met Sphere','','','Aeron',1895,'',127 from dual union all
select 21441,'1969-Y63','1969-01-27 22:00:00','suborbital rocket','success',725,'','','','','Aeron',1322,'SHIP-SOV',96 from dual union all
select 21442,'1969-Y64','1969-01-28 09:07:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',94 from dual union all
select 21443,'1969-Y65','1969-01-28 18:52:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1582,'SHIP-SOV',92 from dual union all
select 21444,'1969-Y66','1969-01-29 09:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',87 from dual union all
select 21445,'1969-Y67','1969-01-29 10:02:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',92 from dual union all
select 21446,'1969-Y68','1969-01-30 09:02:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',90 from dual union all
select 21447,'1969-Y69','1969-01-30 10:03:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1581,'SHIP-SOV',92 from dual union all
select 21448,'1969-Y70','1969-01-30 20:04:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1589,'SHIP-SOV',91 from dual union all
select 21449,'1969-Y71','1969-01-31 09:04:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',87 from dual union all
select 2145,'1954-A159','1954-04-09 00:00:00','military missile','success',771,'GM-12426','','','','Test',222,'',12 from dual union all
select 21450,'1969-Y72','1969-01-31 19:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1580,'SHIP-SOV',91 from dual union all
select 21451,'1969-Y73','1969-02-01 19:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1588,'SHIP-SOV',87 from dual union all
select 21452,'1969-Y74','1969-02-02 09:00:00','suborbital rocket','success',725,'M-100','Particle flux 12','','','Plasma',1010,'',87 from dual union all
select 21453,'1969-Y75','1969-02-02 18:05:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1577,'SHIP-SOV',91 from dual union all
select 21454,'1969-Y76','1969-02-02 19:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1587,'SHIP-SOV',84 from dual union all
select 21455,'1969-Y77','1969-02-03 18:06:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1576,'SHIP-SOV',90 from dual union all
select 21456,'1969-Y78','1969-02-03 20:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1579,'SHIP-SOV',90 from dual union all
select 21457,'1969-Y79','1969-02-04 18:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1575,'SHIP-SOV',96 from dual union all
select 21458,'1969-Y80','1969-02-04 20:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1586,'SHIP-SOV',98 from dual union all
select 21459,'1969-Y81','1969-02-05 08:02:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',85 from dual union all
select 2146,'1954-A16','1954-01-13 00:00:00','atmospheric rocket','success',836,'1186A','','','','Test',1895,'',10 from dual union all
select 21460,'1969-Y82','1969-02-05 09:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',90 from dual union all
select 21461,'1969-Y83','1969-02-05 18:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1574,'SHIP-SOV',89 from dual union all
select 21462,'1969-Y84','1969-02-05 20:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1584,'SHIP-SOV',90 from dual union all
select 21463,'1969-Y85','1969-02-06 09:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',87 from dual union all
select 21464,'1969-Y86','1969-02-06 17:45:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1590,'SHIP-SOV',90 from dual union all
select 21465,'1969-Y87','1969-02-06 18:25:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1573,'SHIP-SOV',94 from dual union all
select 21466,'1969-Y88','1969-02-06 20:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1585,'SHIP-SOV',95 from dual union all
select 21467,'1969-Y89','1969-02-07 09:00:00','suborbital rocket','success',725,'M-100','Particle flux 13','','','Plasma',1010,'',89 from dual union all
select 21468,'1969-Y90','1969-02-07 17:20:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1571,'SHIP-SOV',86 from dual union all
select 21469,'1969-Y91','1969-02-09 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1590,'SHIP-SOV',91 from dual union all
select 2147,'1954-A160','1954-04-09 00:00:00','military missile','success',771,'GM-12452','','','','Test',222,'',12 from dual union all
select 21470,'1969-Y92','1969-02-10 09:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',89 from dual union all
select 21471,'1969-Y93','1969-02-10 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1572,'SHIP-SOV',93 from dual union all
select 21472,'1969-Y94','1969-02-11 22:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1583,'SHIP-SOV',93 from dual union all
select 21473,'1969-Y95','1969-02-12 09:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',90 from dual union all
select 21474,'1969-Y96','1969-02-12 13:03:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',636,'',81 from dual union all
select 21475,'1969-Y97','1969-02-14 09:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',90 from dual union all
select 21476,'1969-Y98','1969-02-17 09:00:00','suborbital rocket','unknown',725,'M-100','','','','Aeron',1010,'',85 from dual union all
select 21477,'1969-Y99','1969-02-19 09:28:00','suborbital rocket','success',725,'M-100','Particle flux 14','','','Plasma',1010,'',90 from dual union all
select 21478,'1970-001','1970-01-09 09:20:00','orbital','success',1508,'Yu15002-025','Kosmos-318','Zenit-2M','','Sat',562,'',null from dual union all
select 21479,'1970-002','1970-01-14 18:43:00','orbital','success',1402,'23B-3 (3B-25)','KH-8A','GAMBIT-3 Mission 4325','','Sat',1778,'',null from dual union all
select 2148,'1954-A161','1954-04-10 00:00:00','atmospheric rocket','success',836,'1270T','','','','Test',1895,'',10 from dual union all
select 21480,'1970-003','1970-01-15 00:16:03','orbital','success',1386,'557/D75','Intelsat III F-6','INTELSAT III F-6','','Sat',233,'',null from dual union all
select 21481,'1970-004','1970-01-15 13:39:59','orbital','success',678,'V295-12','Kosmos-319','DS-P1-Yu No. 25','','Sat',460,'',null from dual union all
select 21482,'1970-005','1970-01-16 10:59:58','orbital','success',678,'','Kosmos-320','DS-MO No. 3','','Sat',627,'',null from dual union all
select 21483,'1970-006','1970-01-20 20:20:00','orbital','success',678,'N231-06','Kosmos-321','DS-U2-MG No. 1','','Sat',460,'',null from dual union all
select 21484,'1970-007','1970-01-21 12:00:00','orbital','success',1508,'Yu15002-011','Kosmos-322','Zenit-4 No. 70','','Sat',473,'',null from dual union all
select 21485,'1970-008','1970-01-23 11:31:02','orbital','success',1389,'542/D76','ITOS 1','Tiros M','','Sat',1724,'',null from dual union all
select 21486,'1970-009','1970-02-04 02:59:50','orbital','success',1395,'534/FTV 6111 (TA12)','SERT 2','SERT 2','','Sat TAG171',1723,'',null from dual union all
select 21487,'1970-010','1970-02-10 12:00:01','orbital','success',1508,'Yu15002-013','Kosmos-323','Zenit-4 No. 71','','Sat',473,'',null from dual union all
select 21488,'1970-011','1970-02-11 04:25:00','orbital','success',699,'L-4S-5','','L-4S-5','','Sat',834,'',5115 from dual union all
select 21489,'1970-012','1970-02-11 08:40:47','orbital','success',1364,'287/BII-9','DAPP 1524','DAPP Block 5A F-1','','Sat',1720,'',null from dual union all
select 2149,'1954-A162','1954-04-10 00:00:00','atmospheric rocket','success',836,'1378T','','','','Test',1895,'',10 from dual union all
select 21490,'1970-013','1970-02-19 18:57:15','orbital','success',804,'V 15000-087','Molniya-1','Molniya-1 11F67 No. 17','','Sat',475,'',null from dual union all
select 21491,'1970-014','1970-02-27 17:24:55','orbital','success',678,'Yu646-27','Kosmos-324','DS-P1-Yu No. 32','','Sat',460,'',null from dual union all
select 21492,'1970-015','1970-03-04 12:14:42','orbital','success',1508,'V 15002-005','Kosmos-325','Zenit-2 No. 73','','Sat',475,'',null from dual union all
select 21493,'1970-016','1970-03-04 22:15:00','orbital','success',1396,'551/FTV 1657','CORONA 137/1109','CORONA CR-10','','Sat TAG172',1776,'',null from dual union all
select 21494,'1970-017','1970-03-10 12:20:00','orbital','success',432,'No. 1','Mika/Wika','DIAL Wissenschaftliche K','','Sat',343,'',null from dual union all
select 21495,'1970-018','1970-03-13 08:00:02','orbital','success',1508,'V 15002-006','Kosmos-326','Zenit-2 No. 74','','Sat',475,'',null from dual union all
select 21496,'1970-019','1970-03-17 11:10:01','orbital','success',1510,'R 15000-005','Meteor','Meteor 11F614M No. 14','','Sat',473,'',null from dual union all
select 21497,'1970-020','1970-03-18 14:39:56','orbital','success',678,'Yu646-28','Kosmos-327','DS-P1-I No. 8','','Sat',460,'',null from dual union all
select 21498,'1970-021','1970-03-20 23:52:00','orbital','success',1386,'558/D77','NATO 2A','NATO IIA','','Sat',233,'',null from dual union all
select 21499,'1970-022','1970-03-27 11:45:00','orbital','success',1508,'V 15002-020','Kosmos-328','Zenit-4MK','','Sat',473,'',null from dual union all
select 215,'1944-S195','1944-10-10 16:43:00','military missile','success',1470,'18813','Ma228','','','Test',661,'',90 from dual union all
select 2150,'1954-A163','1954-04-10 09:00:00','suborbital rocket','failure',54,'NRL 19','Mass spectrometer','','','Aeronomy',1869,'',5 from dual union all
select 21500,'1970-023','1970-04-03 08:30:00','orbital','success',1508,'Yu15002-032','Kosmos-329','Zenit-2M','','Sat',475,'',null from dual union all
select 21501,'1970-024','1970-04-07 11:10:01','orbital','success',680,'V149    -34LM','Kosmos-330','Tselina-OM','','Sat',457,'',null from dual union all
select 21502,'1970-025','1970-04-08 08:17:56','orbital','success',1395,'553/FTV 6231 (TA13)','Nimbus 4','Nimbus D','','Sat TAG173',1723,'',null from dual union all
select 21503,'1970-026','1970-04-08 10:15:20','orbital','success',1508,'Yu15001-018','Kosmos-331','Zenit-4 No. 66','','Sat',562,'',null from dual union all
select 21504,'1970-027','1970-04-08 10:50:01','orbital','success',1425,'3C-18','Vela 6A/6B','Vela 6A','','Sat UPF',274,'',null from dual union all
select 21505,'1970-028','1970-04-11 17:00:00','orbital','success',680,'V149    -36LM','Kosmos-332','Tsiklon','','Sat',457,'',null from dual union all
select 21506,'1970-029','1970-04-11 19:13:03','deep space','success',1123,'SA-508','Apollo 13','Apollo CM-109','','Sat',921,'LUT3',null from dual union all
select 21507,'1970-030','1970-04-15 09:00:01','orbital','success',1508,'Yu15003-044','Kosmos-333','Zenit-4M','','Sat',473,'',null from dual union all
select 21508,'1970-031','1970-04-15 15:52:00','orbital','success',1402,'23B-4 (3B-26)','KH-8A','GAMBIT-3 Mission 4326','','Sat',1778,'',null from dual union all
select 21509,'1970-032','1970-04-23 00:46:12','orbital','success',1386,'559/D78','Intelsat III F-7','INTELSAT III F-7','','Sat',233,'',null from dual union all
select 2151,'1954-A164','1954-04-11 00:00:00','atmospheric rocket','success',836,'1380T','','','','Test',1895,'',10 from dual union all
select 21510,'1970-033','1970-04-23 13:20:00','orbital','success',678,'Yu646-26','Kosmos-334','DS-P1-Yu No. 31','','Sat',460,'',null from dual union all
select 21511,'1970-034','1970-04-24 13:35:45','orbital','success',293,'601904','DFH-1','Dong Fang Hong','','Sat',809,'',2390 from dual union all
select 21512,'1970-035','1970-04-24 22:24:49','orbital','success',678,'','Kosmos-335','DS-U1-R No. 1','','Sat',627,'',null from dual union all
select 21513,'1970-036','1970-04-25 17:09:16','orbital','success',680,'V149    -33LM','Kosmos-336/343','Strela-1M','','Sat',457,'',null from dual union all
select 21514,'1970-037','1970-04-28 10:50:00','orbital','success',1510,'Yu15000-062','Meteor','Meteor 11F614M No. 13','','Sat',473,'',null from dual union all
select 21515,'1970-038','1970-05-12 10:10:01','orbital','success',1508,'Yu15001-003','Kosmos-344','Zenit-2 No. 81','','Sat',473,'',null from dual union all
select 21516,'1970-039','1970-05-20 09:20:01','orbital','success',1508,'Yu15002-019','Kosmos-345','Zenit-4 No. 77','','Sat',562,'',null from dual union all
select 21517,'1970-040','1970-05-20 21:35:10','orbital','success',1396,'555/FTV 1656','CORONA 138/1110','CORONA CR-11','','Sat TAG174',1776,'',null from dual;
insert into LAUNCH
select 21518,'1970-041','1970-06-01 19:00:00','orbital','success',1223,'Yu15000-021S','Soyuz-9','Soyuz No. 17','','Sat',562,'',null from dual union all
select 21519,'1970-042','1970-06-10 09:30:01','orbital','success',1508,'Yu15002-014','Kosmos-346','Zenit-4 No. 72','','Sat',562,'',null from dual union all
select 2152,'1954-A165','1954-04-12 00:00:00','atmospheric rocket','success',836,'1381T','','','','Test',1895,'',10 from dual union all
select 21520,'1970-043','1970-06-12 09:30:02','orbital','success',678,'','Kosmos-347','DS-P1-Yu No. 35','','Sat',627,'',null from dual union all
select 21521,'1970-044','1970-06-13 04:59:58','orbital','success',678,'N231-10','Kosmos-348','DS-U2-GK No. 2','','Sat',460,'',null from dual union all
select 21522,'1970-045','1970-06-17 12:59:57','orbital','success',1508,'Yu15002-015','Kosmos-349','Zenit-4 No. 73','','Sat',475,'',null from dual union all
select 21523,'1970-046','1970-06-19 11:37:00','orbital','success',178,'5201A','AFP-720','RHYOLITE 1','','Sat',228,'',null from dual union all
select 21524,'1970-047','1970-06-23 14:15:00','orbital','success',1510,'Yu15000-063','Meteor','Meteor-M No. 17','','Sat',473,'',null from dual union all
select 21525,'1970-048','1970-06-25 14:50:00','orbital','success',1402,'23B-5 (3B-27)','KH-8A','GAMBIT-3 Mission 4327','','Sat',1778,'',null from dual union all
select 21526,'1970-049','1970-06-26 03:23:01','orbital','success',804,'Yu15000-089','Molniya-1','Molniya-1 11F67 No. 21','','Sat',475,'',null from dual union all
select 21527,'1970-050','1970-06-26 12:00:01','orbital','success',1508,'Kh76001-036','Kosmos-350','Zenit-2M','','Sat',562,'',null from dual union all
select 21528,'1970-051','1970-06-27 07:39:55','orbital','success',678,'Yu457-34','Kosmos-351','DS-P1-Yu No. 38','','Sat',460,'',null from dual union all
select 21529,'1970-052','1970-07-07 10:30:01','orbital','success',1508,'Yu15002-016','Kosmos-352','Zenit-4 No. 74','','Sat',562,'',null from dual union all
select 2153,'1954-A166','1954-04-13 00:00:00','military missile','success',771,'GM-12453','','','','Test',222,'',12 from dual union all
select 21530,'1970-053','1970-07-09 13:35:01','orbital','success',1508,'Yu15002-028','Kosmos-353','Zenit-2M','','Sat',473,'',null from dual union all
select 21531,'1970-054','1970-07-23 01:25:00','orbital','success',1396,'556/FTV 1654','CORONA 139/1111','CORONA CR-12','','Sat TAG175',1776,'',null from dual union all
select 21532,'1970-055','1970-07-23 23:23:00','orbital','success',1386,'563/D79','Intelsat III F-8','INTELSAT III F-8','','Sat',233,'',null from dual union all
select 21533,'1970-056','1970-07-28 22:00:28','orbital','success',1007,'Yu45021-49T','Kosmos-354','OGCh No. Yu22500-11 (22)','','Sat',547,'',null from dual union all
select 21534,'1970-057','1970-08-07 02:59:53','orbital','success',678,'','Interkosmos-3','DS-U2-IK No. 1','','Sat',627,'',null from dual union all
select 21535,'1970-058','1970-08-07 09:30:00','orbital','success',1508,'Yu15002-018','Kosmos-355','Zenit-4 No. 76','','Sat',475,'',null from dual union all
select 21536,'1970-059','1970-08-10 19:59:55','orbital','success',678,'N231-05','Kosmos-356','DS-U2-MG No. 2','','Sat',460,'',null from dual union all
select 21537,'1970-060','1970-08-17 05:38:22','deep space','success',806,'Kh15000-062','Venera-7','4V-1 No. 630','','Sat',562,'',null from dual union all
select 21538,'1970-061','1970-08-18 14:45:00','orbital','success',1402,'23B-6 (3B-28)','KH-8A','GAMBIT-3 Mission 4328','','Sat',1778,'',null from dual union all
select 21539,'1970-062','1970-08-19 12:11:00','orbital','success',1386,'561/D80','Skynet 1B','Skynet IB','','Sat',233,'',null from dual union all
select 2154,'1954-A167','1954-04-13 00:00:00','military missile','success',771,'GM-12439','','','','Test',222,'',12 from dual union all
select 21540,'1970-063','1970-08-19 14:59:53','orbital','success',678,'Yu457-36','Kosmos-357','DS-P1-Yu No. 40','','Sat',460,'',null from dual union all
select 21541,'1970-064','1970-08-20 14:30:01','orbital','success',680,'Yu47121-13','Kosmos-358','Lira','','Sat',456,'',null from dual union all
select 21542,'1970-065','1970-08-22 05:06:09','orbital','failure',806,'Kh15000-061','Kosmos-359','4V-1 No. 631','','Sat',562,'',836 from dual union all
select 21543,'1970-066','1970-08-26 10:01:00','orbital','success',1395,'535/FTV 2736','OPS 8329','STRAWMAN 3','','Sat TAG176',1722,'',null from dual union all
select 21544,'1970-067','1970-08-27 13:23:34','orbital','success',1125,'S176C','NNS O-19 (N-14)','NNS O-19','','Sat',1779,'',null from dual union all
select 21545,'1970-068','1970-08-29 08:30:00','orbital','success',1508,'Kh76002-045','Kosmos-360','Zenit-4M','','Sat',562,'',null from dual union all
select 21546,'1970-069','1970-09-01 01:00:00','orbital','success',178,'5203A','AFP-827','CANYON 3','','Sat',228,'',null from dual union all
select 21547,'1970-070','1970-09-03 08:39:45','orbital','success',1364,'288/BII-13','DAPP 2525','DAPP Block 5A F-2','','Sat',1720,'',null from dual union all
select 21548,'1970-071','1970-09-08 10:30:00','orbital','success',1508,'Kh76002-046','Kosmos-361','Zenit-4M','','Sat',473,'',null from dual union all
select 21549,'1970-072','1970-09-12 13:25:53','deep space','success',953,'248-01','Luna-16','E-8-5 No. 406','','Sat',587,'',null from dual union all
select 2155,'1954-A168','1954-04-13 00:00:00','atmospheric rocket','success',836,'2294A','','','','Test',1895,'',10 from dual union all
select 21550,'1970-073','1970-09-16 11:59:55','orbital','success',678,'Yu457-37','Kosmos-362','DS-P1-I No. 9','','Sat',460,'',null from dual union all
select 21551,'1970-074','1970-09-17 08:10:01','orbital','success',1508,'Kh76001-034','Kosmos-363','Zenit-2M','','Sat',562,'',null from dual union all
select 21552,'1970-075','1970-09-22 13:00:01','orbital','success',1508,'Yu15000-003','Kosmos-364','Zenit-4MK','','Sat',473,'',null from dual union all
select 21553,'1970-076','1970-09-25 14:05:00','orbital','success',1008,'4502741-260T','Kosmos-365','OGCh-K No. Xh22511-65','','Sat',547,'',null from dual union all
select 21554,'1970-077','1970-09-29 08:14:01','orbital','success',804,'Yu15000-090','Molniya-1','Molniya-1 11F67 No. 19','','Sat',475,'',null from dual union all
select 21555,'1970-078','1970-10-01 08:20:03','orbital','success',1508,'Yu15002-029','Kosmos-366','Zenit-2M','','Sat',480,'',null from dual union all
select 21556,'1970-079','1970-10-03 10:26:45','orbital','success',1446,'Yu22500-07L','Kosmos-367','US-A reactor','','Sat',590,'',null from dual union all
select 21557,'1970-080','1970-10-08 12:39:02','orbital','success',1508,'7705711-121','Kosmos-368','Zenit-2M (Bion)','','Sat',562,'',null from dual union all
select 21558,'1970-081','1970-10-08 15:10:03','orbital','success',678,'64657-132','Kosmos-369','DS-P1-Yu No. 42','','Sat',460,'',null from dual union all
select 21559,'1970-082','1970-10-09 11:04:58','orbital','success',1508,'Kh76002-049','Kosmos-370','Zenit-4M','','Sat',480,'',null from dual union all
select 2156,'1954-A169','1954-04-14 00:00:00','military missile','unknown',339,'1401RJ','','','','Test',1895,'',30 from dual union all
select 21560,'1970-083','1970-10-12 13:57:00','orbital','success',680,'47117-103','Kosmos-371','Tsiklon','','Sat',456,'',null from dual union all
select 21561,'1970-084','1970-10-14 11:29:58','orbital','success',678,'','Interkosmos-4','DS-U3-IK No. 2','','Sat',627,'',null from dual union all
select 21562,'1970-085','1970-10-15 11:22:02','orbital','success',1510,'Yu15000-065','Meteor','Meteor-M No. 16','','Sat',473,'',null from dual union all
select 21563,'1970-086','1970-10-16 14:59:59','orbital','success',680,'53717-104','Kosmos-372','Strela-2M','','Sat',456,'',null from dual union all
select 21564,'1970-087','1970-10-20 05:38:00','orbital','success',1446,'Yu22500-06L','Kosmos-373','IS-M','','Sat',590,'',null from dual union all
select 21565,'1970-088','1970-10-20 19:55:39','deep space','success',953,'250-01','Zond-8','L-1 No. 14','','Sat',587,'',null from dual union all
select 21566,'1970-089','1970-10-23 04:42:56','orbital','success',1446,'Yu22500-05L','Kosmos-374','I2P','','Sat',591,'',null from dual union all
select 21567,'1970-090','1970-10-23 17:40:00','orbital','success',1402,'23B-7 (3B-29)','KH-8A','GAMBIT-3 Mission 4329','','Sat',1778,'',null from dual union all
select 21568,'1970-091','1970-10-30 02:36:51','orbital','success',1446,'Yu22500-12L','Kosmos-375','I2P','','Sat',591,'',null from dual union all
select 21569,'1970-092','1970-10-30 13:20:00','orbital','success',1508,'Kh76002-047','Kosmos-376','Zenit-4M','','Sat',475,'',null from dual union all
select 2157,'1954-A17','1954-01-15 00:00:00','military missile','unknown',339,'1394R','','','','Test',1895,'',30 from dual union all
select 21570,'1970-093','1970-11-06 10:35:57','orbital','success',1425,'23C-1 (3C-19)','DSP 1','DSP 1','','Sat UPF',274,'',null from dual union all
select 21571,'1970-094','1970-11-09 06:00:00','orbital','success',1128,'S174C','OFO','OFO','','Sat',1814,'',null from dual union all
select 21572,'1970-095','1970-11-10 14:44:01','deep space','success',953,'251-01','Luna-17','E-8 No. 203','','Sat',587,'',null from dual union all
select 21573,'1970-096','1970-11-11 09:20:01','orbital','success',1508,'Kh76001-035','Kosmos-377','Zenit-2M','','Sat',562,'',null from dual union all
select 21574,'1970-097','1970-11-17 18:20:01','orbital','success',680,'65017-107','Kosmos-378','DS-U2-IP No. 1','','Sat',457,'',null from dual union all
select 21575,'1970-098','1970-11-18 21:28:00','orbital','success',1396,'552/FTV 1658','CORONA 140/1112','CORONA QR-2R','','Sat TAG177',1776,'',null from dual union all
select 21576,'1970-099','1970-11-24 05:15:01','orbital','success',1224,'Yu15000-016','Kosmos-379','T2K No. 1','','Sat',562,'',null from dual union all
select 21577,'1970-100','1970-11-24 10:59:56','orbital','success',678,'64657-131','Kosmos-380','DS-P1-Yu No. 26','','Sat',460,'',null from dual union all
select 21578,'1970-101','1970-11-27 15:47:17','orbital','success',809,'Kh15000-095','Molniya-1','Molniya-1 11F67 No. 23','','Sat',475,'',null from dual union all
select 21579,'1970-102','1970-12-02 04:00:02','orbital','success',680,'47117-105','Kosmos-381','Ionosfernaya Stantsiya N','','Sat',457,'',null from dual union all
select 2158,'1954-A170','1954-04-14 00:00:00','atmospheric rocket','success',836,'1299R  Bell motor','Nike 271R','','','Test',1895,'',10 from dual union all
select 21580,'1970-103','1970-12-02 17:00:00','orbital','success',953,'252-01','Kosmos-382','L-1e No. 2K','','Sat',587,'',null from dual union all
select 21581,'1970-104','1970-12-03 13:55:19','orbital','success',1508,'Kh15000-004','Kosmos-383','Zenit-4MK','','Sat',475,'',null from dual union all
select 21582,'1970-105','1970-12-10 11:10:00','orbital','success',1508,'Yu15002-031','Kosmos-384','Zenit-2M','','Sat',473,'',null from dual union all
select 21583,'1970-106','1970-12-11 11:35:00','orbital','success',1389,'546/D81','NOAA 1','ITOS A','','Sat',1724,'',null from dual union all
select 21584,'1970-107','1970-12-12 10:53:50','orbital','success',1128,'S175C','Explorer 42','SAS A','','Sat',1412,'',null from dual union all
select 21585,'1970-108','1970-12-12 13:00:00','orbital','success',680,'V149-39LM','Kosmos-385','Tsiklon','','Sat',457,'',null from dual union all
select 21586,'1970-109','1970-12-12 13:04:44','orbital','success',432,'No. 2','Peole','Peole','','Sat',343,'',null from dual union all
select 21587,'1970-110','1970-12-15 10:00:01','orbital','success',1508,'Kh76002-051','Kosmos-386','Zenit-4M','','Sat',562,'',null from dual union all
select 21588,'1970-111','1970-12-16 04:29:59','orbital','success',680,'V149-31LM','Kosmos-387','Tselina-OM','','Sat',457,'',null from dual union all
select 21589,'1970-112','1970-12-18 09:39:13','orbital','success',678,'Yu457-29','Kosmos-388','DS-P1-Yu No. 43','','Sat',460,'',null from dual union all
select 2159,'1954-A171','1954-04-14 00:00:00','atmospheric rocket','success',836,'1315R','Nike 272R','','','Test',1895,'',10 from dual union all
select 21590,'1970-113','1970-12-18 16:15:00','orbital','success',1510,'R 15000-006','Kosmos-389','Tselina-D No. Yu2250-01','','Sat',473,'',null from dual union all
select 21591,'1970-114','1970-12-25 03:50:02','orbital','success',809,'V 15000-078 94','Molniya-1','Molniya-1 11F67 No. 22','','Sat',480,'',null from dual union all
select 21592,'1970-A01','1970-01-01 00:00:00','suborbital rocket','failure',27,'','','','','Aeron?',1833,'',0 from dual union all
select 21593,'1970-A02','1970-01-02 18:12:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',46 from dual union all
select 21594,'1970-A03','1970-01-05 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',41 from dual union all
select 21595,'1970-A04','1970-01-06 21:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',44 from dual union all
select 21596,'1970-A05','1970-01-07 22:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',42 from dual union all
select 21597,'1970-A06','1970-01-09 19:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',46 from dual union all
select 21598,'1970-A07','1970-01-13 22:36:00','suborbital rocket','failure',837,'NASA 14.459CM','GCA Release Na','','','Aeronomy',1821,'',24 from dual union all
select 21599,'1970-A08','1970-01-14 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',26 from dual union all
select 216,'1944-S196','1944-10-10 20:01:00','military missile','success',1470,'18809','Ma230','','','Test',661,'',90 from dual union all
select 2160,'1954-A172','1954-04-14 00:00:00','atmospheric rocket','success',836,'1317R','Nike 273R','','','Test',1895,'',10 from dual union all
select 21600,'1970-A09','1970-01-15 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 21601,'1970-A10','1970-01-16 11:06:00','suborbital rocket','success',264,'NASA 20.01GM','','','','Aeronomy',1821,'',41 from dual union all
select 21602,'1970-A100','1970-09-11 13:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',47 from dual union all
select 21603,'1970-A101','1970-09-19 15:31:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',48 from dual union all
select 21604,'1970-A102','1970-09-23 13:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',46 from dual union all
select 21605,'1970-A103','1970-09-26 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 21606,'1970-A104','1970-09-26 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 21607,'1970-A105','1970-09-28 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',45 from dual;
insert into LAUNCH
select 21608,'1970-A106','1970-09-29 17:37:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',48 from dual union all
select 21609,'1970-A107','1970-10-05 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',48 from dual union all
select 2161,'1954-A173','1954-04-15 00:00:00','military missile','unknown',339,'1296RJ-S','','','','Test',1895,'',30 from dual union all
select 21610,'1970-A108','1970-10-06 03:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',46 from dual union all
select 21611,'1970-A109','1970-10-07 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',43 from dual union all
select 21612,'1970-A11','1970-01-19 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',48 from dual union all
select 21613,'1970-A110','1970-10-12 13:35:00','military missile','failure',999,'4925761-026','8K67 UBP-39','Pal''ma-4','','OT',518,'',50 from dual union all
select 21614,'1970-A111','1970-10-14 04:15:00','military missile','failure',916,'AO-96','P-193 70 Fall 08','','','Test',204,'',1 from dual union all
select 21615,'1970-A112','1970-10-21 12:09:00','suborbital rocket','success',631,'INTA H-7023','','','','Meteo',396,'',42 from dual union all
select 21616,'1970-A113','1970-10-23 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',46 from dual union all
select 21617,'1970-A114','1970-10-28 00:00:00','military missile','failure',1252,'','SAFEGUARD M1-9','','','Interceptor',868,'',0 from dual union all
select 21618,'1970-A115','1970-11-02 15:34:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',48 from dual union all
select 21619,'1970-A116','1970-11-04 20:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',46 from dual union all
select 2162,'1954-A174','1954-04-15 00:00:00','military missile','success',771,'GM-12428','','','','Test',222,'',12 from dual union all
select 21620,'1970-A117','1970-11-05 05:25:00','military missile','failure',916,'','P-196 70 Fall 11','','','OT',204,'',1 from dual union all
select 21621,'1970-A118','1970-11-05 22:00:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',49 from dual union all
select 21622,'1970-A119','1970-11-06 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',48 from dual union all
select 21623,'1970-A12','1970-01-19 16:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',46 from dual union all
select 21624,'1970-A120','1970-11-10 21:11:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',49 from dual union all
select 21625,'1970-A121','1970-11-12 21:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',47 from dual union all
select 21626,'1970-A122','1970-11-19 15:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',49 from dual union all
select 21627,'1970-A123','1970-11-24 21:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',45 from dual union all
select 21628,'1970-A124','1970-12-02 13:03:00','suborbital rocket','failure',920,'P81H','D-region','','','Ionospheric',1447,'',38 from dual union all
select 21629,'1970-A125','1970-12-04 00:00:00','military missile','success',1252,'','SAFEGUARD M1-9A','','','Interceptor',868,'',10 from dual union all
select 2163,'1954-A175','1954-04-15 00:00:00','atmospheric rocket','success',836,'1629E','','','','Test',1895,'',10 from dual union all
select 21630,'1970-A126','1970-12-09 18:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',42 from dual union all
select 21631,'1970-A127','1970-12-11 15:45:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1821,'',47 from dual union all
select 21632,'1970-A128','1970-12-11 19:41:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1821,'',41 from dual union all
select 21633,'1970-A129','1970-12-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',43 from dual union all
select 21634,'1970-A13','1970-01-21 03:01:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',23 from dual union all
select 21635,'1970-A130','1970-12-15 19:40:00','suborbital rocket','success',230,'','MDSS-001','','','Test',428,'',30 from dual union all
select 21636,'1970-A131','1970-12-16 15:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',47 from dual union all
select 21637,'1970-A132','1970-12-16 15:44:00','suborbital rocket','success',230,'','MDSS-002','','','Test',428,'',30 from dual union all
select 21638,'1970-A133','1970-12-16 17:15:00','suborbital rocket','success',230,'','MDSS-003','','','Test',428,'',30 from dual union all
select 21639,'1970-A134','1970-12-16 18:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',45 from dual union all
select 2164,'1954-A176','1954-04-15 00:00:00','atmospheric rocket','success',836,'1663E','','','','Test',1895,'',10 from dual union all
select 21640,'1970-A135','1970-12-16 20:26:00','suborbital rocket','success',230,'','MDSS-004','','','Test',428,'',30 from dual union all
select 21641,'1970-A136','1970-12-16 21:00:00','suborbital rocket','success',1061,'','','','','Meteo',617,'',46 from dual union all
select 21642,'1970-A137','1970-12-18 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',49 from dual union all
select 21643,'1970-A138','1970-12-19 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',45 from dual union all
select 21644,'1970-A139','1970-12-22 19:14:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',46 from dual union all
select 21645,'1970-A14','1970-01-22 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',47 from dual union all
select 21646,'1970-A140','1970-12-23 00:00:00','military missile','success',1252,'FLA-49','Safeguard M1-12','','','Interceptor',868,'',10 from dual union all
select 21647,'1970-A141','1970-12-28 02:33:00','military missile','failure',1450,'4764732-1P','15A20 K','GCh 15B20','','OT',505,'',500 from dual union all
select 21648,'1970-A142','1970-07-15 02:00:00','suborbital rocket','failure',761,'MT135P-R1','Range Test','','','Aeron',1332,'',44 from dual union all
select 21649,'1970-A143','1970-08-19 02:10:00','suborbital rocket','success',761,'MT135P-R3','','','','Aeron',1332,'',47 from dual union all
select 2165,'1954-A177','1954-04-15 00:00:00','atmospheric rocket','success',836,'1398E','','','','Test',1895,'',10 from dual union all
select 21650,'1970-A144','1970-09-02 02:20:00','suborbital rocket','success',761,'MT135P-R4','','','','Aeron',1332,'',47 from dual union all
select 21651,'1970-A145','1970-10-01 02:05:00','suborbital rocket','success',761,'MT135P-R5','','','','Aeron',1332,'',49 from dual union all
select 21652,'1970-A146','1970-10-14 02:00:00','suborbital rocket','success',761,'MT135P-R7','','','','Aeron',1332,'',47 from dual union all
select 21653,'1970-A147','1970-11-25 02:00:00','suborbital rocket','success',761,'MT135P-R9','','','','Aeron',1332,'',48 from dual union all
select 21654,'1970-A148','1970-12-10 02:00:00','suborbital rocket','success',761,'MT135P-R10','','','','Aeron',1332,'',49 from dual union all
select 21655,'1970-A149','1970-01-01 00:00:00','suborbital rocket','success',1022,'','First Flight','','','Test',1467,'',11 from dual union all
select 21656,'1970-A15','1970-01-23 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',49 from dual union all
select 21657,'1970-A150','1970-01-01 00:00:00','suborbital rocket','success',773,'','First Flight','','','Test',1467,'',40 from dual union all
select 21658,'1970-A151','1970-12-19 00:00:00','suborbital rocket','success',279,'CR-X2','Dummy St2','','','Test',310,'',45 from dual union all
select 21659,'1970-A152','1970-08-04 15:40:00','suborbital rocket','success',88,'','Biospace','','','Bio',1821,'',40 from dual union all
select 2166,'1954-A178','1954-04-15 00:00:00','atmospheric rocket','success',836,'1383T','','','','Test',1895,'',10 from dual union all
select 21660,'1970-A153','1970-08-18 00:00:00','suborbital rocket','success',88,'','Biospace (rat)','','','Bio',1821,'',40 from dual union all
select 21661,'1970-A154','1970-08-18 00:00:00','suborbital rocket','success',88,'','Biospace (rat)','','','Bio',1821,'',40 from dual union all
select 21662,'1970-A155','1970-08-18 00:00:00','suborbital rocket','success',88,'','Biospace (chicken)','','','Bio',1821,'',40 from dual union all
select 21663,'1970-A156','1970-08-01 00:00:00','suborbital rocket','success',88,'','Biospace','','','Bio',1821,'',40 from dual union all
select 21664,'1970-A157','1970-02-10 00:00:00','military missile','success',843,'','ASF','','','Test',443,'',10 from dual union all
select 21665,'1970-A158','1970-10-01 00:00:00','military missile','success',849,'1','Japan test','','','Test',433,'',20 from dual union all
select 21666,'1970-A159','1970-10-01 00:00:00','military missile','success',849,'2','Japan test','','','Test',433,'',20 from dual union all
select 21667,'1970-A16','1970-01-27 16:54:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',44 from dual union all
select 21668,'1970-A160','1970-10-01 00:00:00','military missile','success',849,'3','Japan test','','','Test',433,'',20 from dual union all
select 21669,'1970-A161','1970-10-01 00:00:00','military missile','success',849,'4','Japan test','','','Test',433,'',20 from dual union all
select 2167,'1954-A179','1954-04-16 00:00:00','atmospheric rocket','success',836,'1367T','','','','Test',1895,'',10 from dual union all
select 21670,'1970-A162','1970-11-01 00:00:00','military missile','success',849,'5','Japan test','','','Test',433,'',20 from dual union all
select 21671,'1970-A163','1970-11-01 00:00:00','military missile','success',849,'6','Japan test','','','Test',433,'',20 from dual union all
select 21672,'1970-A164','1970-11-01 00:00:00','military missile','success',849,'7','Japan test','','','Test',433,'',20 from dual union all
select 21673,'1970-A165','1970-11-01 00:00:00','military missile','success',849,'8','Japan test','','','Test',433,'',20 from dual union all
select 21674,'1970-A166','1970-11-01 00:00:00','military missile','success',849,'9','Japan test','','','Test',433,'',20 from dual union all
select 21675,'1970-A167','1970-10-21 09:39:00','suborbital rocket','failure',675,'WRE K149','','','','Aeron',1832,'',20 from dual union all
select 21676,'1970-A168','1970-11-06 03:36:00','suborbital rocket','failure',675,'WRE K150','','','','Aeron',1832,'',20 from dual union all
select 21677,'1970-A169','1970-05-01 00:00:00','suborbital rocket','failure',1346,'Sandia','Flight test 1','','','Test',1490,'',3 from dual union all
select 21678,'1970-A17','1970-01-28 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',48 from dual union all
select 21679,'1970-A170','1970-07-01 00:00:00','suborbital rocket','success',1346,'Sandia','Flight test 2','','','Test',1490,'',14 from dual union all
select 2168,'1954-A18','1954-01-15 00:00:00','atmospheric rocket','unknown',576,'5SP','','','','Test',1895,'',10 from dual union all
select 21680,'1970-A171','1970-10-07 04:00:00','suborbital rocket','failure',839,'','Chemical Trail','','','Aeron',1895,'',1 from dual union all
select 21681,'1970-A172','1970-01-01 00:00:00','military missile','success',1429,'','','','','Test',636,'',20 from dual union all
select 21682,'1970-A173','1970-10-01 00:00:00','atmospheric rocket','success',1346,'Sandia','SAMS Vehicle 0','','','Tech',855,'',3 from dual union all
select 21683,'1970-A174','1970-08-29 00:00:00','military missile','failure',1044,'47827-431-11L','8K98P LKI-8','','','Test',477,'',0 from dual union all
select 21684,'1970-A175','1970-10-30 00:00:00','military missile','failure',1044,'64927-433-13L','8K98P LKI-10','','','Test',477,'',0 from dual union all
select 21685,'1970-A176','1970-12-03 00:00:00','military missile','failure',1037,'','8K98 UBP-15','','','Test',477,'',0 from dual union all
select 21686,'1970-A177','1970-03-06 00:00:00','military missile','success',702,'XRL Test','','','','Test',1895,'',10 from dual union all
select 21687,'1970-A18','1970-01-30 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',45 from dual union all
select 21688,'1970-A19','1970-02-04 22:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',42 from dual union all
select 21689,'1970-A20','1970-02-05 00:00:00','atmospheric rocket','success',1066,'Ly8','Lynx 8','','','Test',2,'',20 from dual union all
select 2169,'1954-A180','1954-04-17 00:00:00','atmospheric rocket','success',836,'1319R','','','','Test',1895,'',10 from dual union all
select 21690,'1970-A21','1970-02-06 14:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',23 from dual union all
select 21691,'1970-A22','1970-02-06 16:45:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',45 from dual union all
select 21692,'1970-A23','1970-02-11 21:03:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',48 from dual union all
select 21693,'1970-A24','1970-03-03 20:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',46 from dual union all
select 21694,'1970-A25','1970-03-07 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 21695,'1970-A26','1970-03-07 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 21696,'1970-A27','1970-03-09 14:48:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',46 from dual union all
select 21697,'1970-A28','1970-03-09 22:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',46 from dual union all
select 21698,'1970-A29','1970-03-11 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',44 from dual;
insert into LAUNCH
select 21699,'1970-A30','1970-03-13 20:30:02','military missile','failure',799,'FTM-212','Mk 12 RV x 3','R&D','','Test',266,'',0 from dual union all
select 217,'1944-S197','1944-10-11 09:52:00','military missile','success',1470,'18811','Ma231','','','Test',661,'',90 from dual union all
select 2170,'1954-A181','1954-04-20 00:00:00','atmospheric rocket','success',836,'2263A','','','','Test',1895,'',10 from dual union all
select 21700,'1970-A31','1970-03-14 18:00:15','military missile','success',796,'Mobile Feasibility','Ballast RV','R&D Sheltered Base','','Test',263,'',1 from dual union all
select 21701,'1970-A32','1970-03-16 02:29:00','suborbital rocket','failure',675,'WRE K142','','','','Aeron',1832,'',38 from dual union all
select 21702,'1970-A33','1970-03-16 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',49 from dual union all
select 21703,'1970-A34','1970-03-19 00:00:00','military missile','failure',1252,'FLA-39','','','','Test',1888,'',2 from dual union all
select 21704,'1970-A35','1970-03-17 21:02:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',47 from dual union all
select 21705,'1970-A36','1970-03-20 18:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',42 from dual union all
select 21706,'1970-A37','1970-03-22 02:08:00','suborbital rocket','failure',1206,'SN5/1','DLR S-CR-21','','','Aeron',1343,'',0 from dual union all
select 21707,'1970-A38','1970-03-23 19:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',47 from dual union all
select 21708,'1970-A39','1970-03-24 02:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',45 from dual union all
select 21709,'1970-A40','1970-03-24 15:31:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',49 from dual union all
select 2171,'1954-A182','1954-04-20 00:00:00','atmospheric rocket','success',836,'1375T','','','','Test',1895,'',10 from dual union all
select 21710,'1970-A41','1970-03-25 15:51:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',49 from dual union all
select 21711,'1970-A42','1970-03-25 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',47 from dual union all
select 21712,'1970-A43','1970-03-25 20:03:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',44 from dual union all
select 21713,'1970-A44','1970-03-30 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',47 from dual union all
select 21714,'1970-A45','1970-03-30 14:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',48 from dual union all
select 21715,'1970-A46','1970-03-30 20:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',45 from dual union all
select 21716,'1970-A47','1970-04-01 00:00:00','atmospheric rocket','success',617,'J210','HRV 931/WRE Delta','','','Test',1836,'',30 from dual union all
select 21717,'1970-A48','1970-04-01 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',48 from dual union all
select 21718,'1970-A49','1970-04-03 22:09:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',47 from dual union all
select 21719,'1970-A50','1970-04-04 01:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',48 from dual union all
select 2172,'1954-A183','1954-04-21 00:00:00','atmospheric rocket','success',442,'','D35 Model','','','Test',1821,'',10 from dual union all
select 21720,'1970-A51','1970-04-08 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',49 from dual union all
select 21721,'1970-A52','1970-04-08 23:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',46 from dual union all
select 21722,'1970-A53','1970-04-09 16:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',49 from dual union all
select 21723,'1970-A54','1970-04-16 22:01:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',49 from dual union all
select 21724,'1970-A55','1970-04-18 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 21725,'1970-A56','1970-04-20 14:13:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',41 from dual union all
select 21726,'1970-A57','1970-04-22 18:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',43 from dual union all
select 21727,'1970-A58','1970-04-28 00:00:00','military missile','failure',917,'AO-89','P-176 70 Spring 03','','','Test',204,'',1 from dual union all
select 21728,'1970-A59','1970-04-29 11:37:59','suborbital rocket','success',631,'INTA H-7008','','','','Meteo',396,'',48 from dual union all
select 21729,'1970-A60','1970-04-29 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',47 from dual union all
select 2173,'1954-A184','1954-04-21 00:00:00','atmospheric rocket','unknown',576,'476M','','','','Test',1895,'',10 from dual union all
select 21730,'1970-A61','1970-05-04 16:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',46 from dual union all
select 21731,'1970-A62','1970-05-06 09:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',45 from dual union all
select 21732,'1970-A63','1970-05-21 20:10:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',42 from dual union all
select 21733,'1970-A64','1970-05-22 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',45 from dual union all
select 21734,'1970-A65','1970-05-22 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',46 from dual union all
select 21735,'1970-A66','1970-05-25 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',46 from dual union all
select 21736,'1970-A67','1970-05-28 00:00:00','military missile','success',1252,'FLA-40','','','','Test',1888,'',10 from dual union all
select 21737,'1970-A68','1970-06-02 15:14:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',49 from dual union all
select 21738,'1970-A69','1970-06-04 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',45 from dual union all
select 21739,'1970-A70','1970-06-08 00:00:00','military missile','success',1252,'FLA-44','','','','Test',1888,'',10 from dual union all
select 2174,'1954-A185','1954-04-21 00:00:00','atmospheric rocket','unknown',576,'479M','','','','Test',1895,'',10 from dual union all
select 21740,'1970-A71','1970-06-10 18:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',49 from dual union all
select 21741,'1970-A72','1970-06-11 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',49 from dual union all
select 21742,'1970-A73','1970-06-20 17:15:00','suborbital rocket','success',549,'PEPP/SHAPE','40-ft Reefed DGB','','','Parachute test',1895,'',44 from dual union all
select 21743,'1970-A74','1970-06-22 14:04:00','suborbital rocket','failure',1204,'S68/2','ESRO S68/2','','','XR Astronomy',1343,'',3 from dual union all
select 21744,'1970-A75','1970-06-23 16:49:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',49 from dual union all
select 21745,'1970-A76','1970-06-24 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',44 from dual union all
select 21746,'1970-A77','1970-06-27 00:22:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',45 from dual union all
select 21747,'1970-A78','1970-06-29 00:00:00','military missile','success',1252,'FLA-43','','','','Test',1888,'',10 from dual union all
select 21748,'1970-A79','1970-06-29 22:08:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',49 from dual union all
select 21749,'1970-A80','1970-06-30 16:13:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',49 from dual union all
select 2175,'1954-A186','1954-04-22 00:00:00','atmospheric rocket','success',442,'','Regulus II Model','','','Test',1821,'',10 from dual union all
select 21750,'1970-A81','1970-06-30 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',48 from dual union all
select 21751,'1970-A82','1970-07-01 18:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',39 from dual union all
select 21752,'1970-A83','1970-07-01 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',46 from dual union all
select 21753,'1970-A84','1970-07-02 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 21754,'1970-A85','1970-07-06 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',49 from dual union all
select 21755,'1970-A86','1970-07-06 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',45 from dual union all
select 21756,'1970-A87','1970-07-14 21:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',49 from dual union all
select 21757,'1970-A88','1970-07-22 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',45 from dual union all
select 21758,'1970-A89','1970-07-24 13:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',47 from dual union all
select 21759,'1970-A90','1970-07-24 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',48 from dual union all
select 2176,'1954-A187','1954-04-22 00:00:00','atmospheric rocket','success',836,'1395T','','','','Test',1895,'',10 from dual union all
select 21760,'1970-A91','1970-07-29 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 21761,'1970-A92','1970-08-12 00:00:00','military missile','success',1252,'FLA-45','','','','Test',1888,'',10 from dual union all
select 21762,'1970-A93','1970-08-14 16:19:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',44 from dual union all
select 21763,'1970-A94','1970-08-20 15:58:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',45 from dual union all
select 21764,'1970-A95','1970-08-26 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',41 from dual union all
select 21765,'1970-A96','1970-08-28 14:46:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',47 from dual union all
select 21766,'1970-A97','1970-09-02 14:56:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',47 from dual union all
select 21767,'1970-A98','1970-09-02 17:23:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',47 from dual union all
select 21768,'1970-A99','1970-09-07 06:30:00','suborbital rocket','success',828,'NAL-7 BS9','','','','Test',1476,'',20 from dual union all
select 21769,'1970-F01','1970-01-30 15:40:01','orbital','failure',678,'V295-13','','DS-P1-I No. 6','','Sat',460,'',100 from dual union all
select 2177,'1954-A188','1954-04-22 00:00:00','atmospheric rocket','success',836,'1638E','','','','Test',1895,'',10 from dual union all
select 21770,'1970-F02','1970-02-06 04:16:06','orbital','failure',953,'247-01','','E-8-5 No. 405','','Sat',587,'',50 from dual union all
select 21771,'1970-F03','1970-05-22 12:39:55','orbital','failure',678,'Yu457-32','','DS-P1-Yu No. 36','','Sat',460,'',50 from dual union all
select 21772,'1970-F05','1970-06-27 16:39:59','orbital','failure',680,'47117-102','','Strela-2M','','Sat',456,'',40 from dual union all
select 21773,'1970-F06','1970-07-21 12:30:01','orbital','failure',1508,'Yu15002-017','','Zenit-4 No. 75','','Sat',475,'',50 from dual union all
select 21774,'1970-F07','1970-09-02 00:34:05','orbital','failure',208,'R-2','X-2','X-2','','Sat',1844,'',550 from dual union all
select 21775,'1970-F08','1970-09-25 05:00:00','orbital','failure',821,'M-4S-1','MS-F1','MS-F1','','Sat',835,'',712 from dual union all
select 21776,'1970-F10','1970-12-22 21:30:00','orbital','failure',680,'47117-106','','DS-P1-M No. 1','','Sat',457,'',0 from dual union all
select 21777,'1970-M01','1970-01-19 13:16:00','suborbital rocket','success',1189,'INTA MP-7005','','','','Ionos',396,'',67 from dual union all
select 21778,'1970-M02','1970-01-23 05:00:00','suborbital rocket','success',1069,'MT-160-2','Meteo Test','','','Aeron',839,'',75 from dual union all
select 21779,'1970-M03','1970-01-29 17:21:00','suborbital rocket','success',839,'NASA 10.339GM','GSFC Grenade 230 PB','','','Aeronomy',1039,'',74 from dual union all
select 2178,'1954-A189','1954-04-23 00:00:00','military missile','success',771,'GM-12470','','','','Test',222,'',12 from dual union all
select 21780,'1970-M04','1970-02-03 00:00:00','suborbital rocket','success',203,'B139','FU-191C','','','Test',340,'',64 from dual union all
select 21781,'1970-M05','1970-03-07 16:00:00','suborbital rocket','success',839,'NASA 10.345GM','Ozone/H2O','','','Eclipse Aeron',1818,'',67 from dual union all
select 21782,'1970-M06','1970-03-07 18:10:00','suborbital rocket','success',848,'CRL A07.016-1','Sphere','','','Eclipse Aeron',339,'',69 from dual union all
select 21783,'1970-M07','1970-03-07 18:37:30','suborbital rocket','success',848,'CRL A07.902-9','Mass spectrometer','','','Eclipse Aeron',1812,'',76 from dual union all
select 21784,'1970-M08','1970-03-07 18:40:30','suborbital rocket','success',839,'NASA 10.346GM','GSFC Ozone','','','Eclipse Aeronomy',1818,'',68 from dual union all
select 21785,'1970-M09','1970-03-10 14:36:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',50 from dual union all
select 21786,'1970-M10','1970-03-12 03:48:00','suborbital rocket','success',675,'WRE K141','','','','Aeron',1832,'',80 from dual union all
select 21787,'1970-M11','1970-03-12 14:00:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',50 from dual union all
select 21788,'1970-M12','1970-03-13 14:36:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',50 from dual;
insert into LAUNCH
select 21789,'1970-M13','1970-03-14 14:20:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',50 from dual union all
select 2179,'1954-A19','1954-01-15 00:00:00','atmospheric rocket','unknown',576,'3SP','','','','Test',1895,'',10 from dual union all
select 21790,'1970-M14','1970-03-16 14:12:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',50 from dual union all
select 21791,'1970-M15','1970-03-18 14:16:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',50 from dual union all
select 21792,'1970-M16','1970-03-20 14:30:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',50 from dual union all
select 21793,'1970-M17','1970-03-20 23:50:00','suborbital rocket','success',839,'NASA 10.277NA','','','','Aeronomy',1895,'',56 from dual union all
select 21794,'1970-M18','1970-03-22 15:41:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',50 from dual union all
select 21795,'1970-M19','1970-03-23 14:30:00','suborbital rocket','success',1188,'','','','','Meteo',450,'',50 from dual union all
select 21796,'1970-M20','1970-04-12 19:14:00','suborbital rocket','success',848,'CRL A07.913-3','Sphere','','','Aeron',394,'',79 from dual union all
select 21797,'1970-M21','1970-04-22 08:11:00','suborbital rocket','failure',215,'AEF-2-121','','','','Aeron/Ionos',428,'',75 from dual union all
select 21798,'1970-M22','1970-06-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 21799,'1970-M23','1970-06-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 218,'1944-S198','1944-10-11 13:47:00','military missile','success',1470,'18816','Ma234','','','Test',661,'',90 from dual union all
select 2180,'1954-A190','1954-04-23 00:00:00','military missile','success',771,'GM-12469','','','','Test',222,'',12 from dual union all
select 21800,'1970-M24','1970-08-21 01:17:00','suborbital rocket','failure',219,'CRL A16.010-03','Propagation','','','Ionos',428,'',56 from dual union all
select 21801,'1970-M25','1970-10-07 09:30:00','suborbital rocket','unknown',778,'','','','','Aeron',933,'',75 from dual union all
select 21802,'1970-M26','1970-11-06 00:00:00','suborbital rocket','success',203,'B140','FU-191D Range test','','','Test',340,'',58 from dual union all
select 21803,'1970-M27','1970-11-24 02:18:00','suborbital rocket','success',263,'','','','','Aeron Ba',1047,'',50 from dual union all
select 21804,'1970-M28','1970-12-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 21805,'1970-M29','1970-12-02 02:00:00','suborbital rocket','success',263,'','','','','Aeron Ba',1047,'',50 from dual union all
select 21806,'1970-M30','1970-07-22 10:47:00','suborbital rocket','success',631,'INTA H-7016','','','','Meteo',396,'',58 from dual union all
select 21807,'1970-M31','1970-03-26 00:00:00','suborbital rocket','success',273,'','','','','Aeron',293,'',75 from dual union all
select 21808,'1970-M32','1970-03-30 00:00:00','suborbital rocket','success',273,'','','','','Aeron',293,'',75 from dual union all
select 21809,'1970-M33','1970-04-01 00:00:00','suborbital rocket','success',273,'','','','','Aeron',293,'',75 from dual union all
select 2181,'1954-A191','1954-04-23 00:00:00','military missile','success',771,'GM-12462','','','','Test',222,'',12 from dual union all
select 21810,'1970-M34','1970-04-02 00:00:00','suborbital rocket','success',273,'','','','','Aeron',293,'',75 from dual union all
select 21811,'1970-M35','1970-04-03 00:00:00','suborbital rocket','success',273,'','','','','Aeron',293,'',75 from dual union all
select 21812,'1970-M36','1970-04-13 00:00:00','suborbital rocket','success',273,'','','','','Aeron',293,'',75 from dual union all
select 21813,'1970-M37','1970-04-14 00:00:00','suborbital rocket','success',273,'','','','','Aeron',293,'',75 from dual union all
select 21814,'1970-M38','1970-09-01 00:00:00','military missile','success',1162,'','MINOTAURUS 70','RakArtLBttr Training','','OT',984,'',50 from dual union all
select 21815,'1970-M44','1974-03-01 00:00:00','military missile','success',1162,'','MINOTAURUS 74','RakArtLBttr Training','','OT',984,'',50 from dual union all
select 21816,'1970-S01','1970-01-01 00:00:00','military missile','success',751,'V7','V7','','','OT',307,'S655',500 from dual union all
select 21817,'1970-S02','1970-01-01 00:00:00','military missile','success',751,'C2','C2','','','OT',307,'S655',500 from dual union all
select 21818,'1970-S03','1970-01-01 00:00:00','military missile','success',751,'C3','C3','','','OT',307,'S655',500 from dual union all
select 21819,'1970-S04','1970-01-01 00:00:00','military missile','success',751,'C4','C4','','','OT',307,'S655',500 from dual union all
select 2182,'1954-A192','1954-04-23 00:00:00','military missile','success',771,'GM-12479','','','','Test',222,'',12 from dual union all
select 21820,'1970-S05','1970-09-26 00:00:00','military missile','success',982,'G 22501-23T','8K64U UBP-147','','','OT',596,'',1210 from dual union all
select 21821,'1970-S06','1970-02-07 00:00:00','military missile','success',982,'G 22503-19T','8K64U UBP-139','','','OT',596,'',1210 from dual union all
select 21822,'1970-S07','1970-03-24 00:00:00','military missile','success',982,'G 22501-04T','8K64U UBP-142','','','OT',596,'',1210 from dual union all
select 21823,'1970-S10','1970-06-03 00:00:00','military missile','success',982,'G 22501-05T','8K64U UBP-144','','','OT',596,'',1210 from dual union all
select 21824,'1970-S100','1970-02-04 05:26:00','suborbital rocket','success',290,'C57/1','ESRO C57/1','','','Aeronomy',408,'',172 from dual union all
select 21825,'1970-S101','1970-02-04 06:19:00','suborbital rocket','success',867,'NASA 18.100UA','','','','Auroral/Aeronomy',428,'',310 from dual union all
select 21826,'1970-S102','1970-02-04 20:38:00','military missile','failure',939,'C3XT-20','','','','Test',421,'EAG-154',100 from dual union all
select 21827,'1970-S103','1970-02-05 15:51:00','suborbital rocket','success',37,'NASA 13.13GT','','','','Test',1821,'',200 from dual union all
select 21828,'1970-S104','1970-02-05 02:21:00','suborbital rocket','success',867,'NASA 18.101UA','','','','Aeronomy',428,'',340 from dual union all
select 21829,'1970-S105','1970-02-06 16:30:00','military missile','unknown',932,'A3E','','','','OT',421,'SSBN 626',1000 from dual union all
select 2183,'1954-A193','1954-04-23 00:00:00','atmospheric rocket','success',836,'1401T','','','','Test',1895,'',10 from dual union all
select 21830,'1970-S106','1970-02-06 19:42:00','suborbital rocket','success',1205,'ESRO S44/1','ESRO S44/1','','','UV Astronomy',1343,'',211 from dual union all
select 21831,'1970-S107','1970-02-07 03:00:00','suborbital rocket','success',30,'NASA 04.262GG','','','','XR Astronomy',1869,'',200 from dual union all
select 21832,'1970-S108','1970-02-07 05:31:00','suborbital rocket','success',33,'CRL AT3.726','MAP-I/LO STAR I','','','IR Astron',1860,'',142 from dual union all
select 21833,'1970-S109','1970-02-08 12:22:00','military missile','success',168,'96F','RMV-340A/BSE/RMV-420D','ABRES RMP-B-15/KX-114','','Reentry Test',1682,'',1400 from dual union all
select 21834,'1970-S110','1970-02-09 04:00:00','military missile','success',1000,'N 22500-08T','8K67M UBP-10','GCh 8F675','','Test',518,'',1000 from dual union all
select 21835,'1970-S111','1970-02-10 05:25:55','suborbital rocket','success',867,'NASA 18.16UE','U Alaska M/DTR','','','Plasma',428,'',270 from dual union all
select 21836,'1970-S112','1970-02-10 05:27:00','suborbital rocket','success',837,'NASA 14.442UI','','','','Auroral/Ionos',428,'',200 from dual union all
select 21837,'1970-S113','1970-02-10 12:30:00','suborbital rocket','success',1069,'S-160JA-1','JARE 11','','','Auroral',1458,'',87 from dual union all
select 21838,'1970-S114','1970-02-10 14:10:00','suborbital rocket','failure',1207,'ESRO S72','UCL UVS-1','','','UV Astronomy',1839,'',244 from dual union all
select 21839,'1970-S115','1970-02-12 03:14:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',165 from dual union all
select 2184,'1954-A194','1954-04-24 00:00:00','atmospheric rocket','success',836,'1384T','','','','Test',1895,'',10 from dual union all
select 21840,'1970-S116','1970-02-12 22:40:00','suborbital rocket','unknown',740,'MR-12','Polar Atm/pressure','','','Aeron/Ionos',1010,'',171 from dual union all
select 21841,'1970-S117','1970-02-14 03:26:19','suborbital rocket','success',226,'A-BBVA-24   (X2)','DLR A-BBV-24 Monica','','','Auroral',29,'',210 from dual union all
select 21842,'1970-S118','1970-02-14 05:01:50','suborbital rocket','success',226,'A-BBVA-25   (X3)','DLR A-BBV-25 Evelyn','','','Auroral',29,'',223 from dual union all
select 21843,'1970-S119','1970-02-14 05:30:00','suborbital rocket','success',37,'NASA 13.01UG','CAL XR 4','','','XR Astron',1895,'',231 from dual union all
select 21844,'1970-S12','1970-06-30 00:00:00','military missile','success',982,'G 22501-09T','8K64U UBP-145','','','OT',596,'',1210 from dual union all
select 21845,'1970-S120','1970-02-14 16:15:40','suborbital rocket','success',1204,'ESRO S66/2','ESRO S66/2','','','Ionospheric',416,'',210 from dual union all
select 21846,'1970-S121','1970-02-16 12:13:00','suborbital rocket','success',1189,'INTA MP-7007','','','','Ionos',396,'',96 from dual union all
select 21847,'1970-S122','1970-02-17 12:02:00','suborbital rocket','success',1189,'INTA MP-7008','','','','Ionos',396,'',94 from dual union all
select 21848,'1970-S123','1970-02-17 12:10:00','suborbital rocket','success',1069,'S-160JA-2','JARE 11','','','Auroral',1458,'',88 from dual union all
select 21849,'1970-S124','1970-02-18 04:30:00','military missile','success',1002,'N 22500-05T','8K67P UBP-8','GCh 8F675','','Test',519,'',1000 from dual union all
select 2185,'1954-A195','1954-04-26 00:00:00','atmospheric rocket','success',836,'2220A','','','','Test',1895,'',10 from dual union all
select 21850,'1970-S125','1970-02-18 07:34:00','suborbital rocket','success',624,'NASA 8.55UE','Auroral substorm','','','Plasma',428,'',494 from dual union all
select 21851,'1970-S126','1970-02-18 11:53:00','suborbital rocket','success',1189,'INTA MP-7009','','','','Ionos',396,'',95 from dual union all
select 21852,'1970-S127','1970-02-19 03:25:00','military missile','success',1000,'','8K67M UBP-11','','','Test',515,'',1000 from dual union all
select 21853,'1970-S128','1970-02-19 05:29:00','suborbital rocket','success',837,'NASA 14.381GE','Auroral particles','','','Plasma',428,'',200 from dual union all
select 21854,'1970-S129','1970-02-19 12:00:00','suborbital rocket','success',1189,'INTA MP-7010','','','','Ionos',396,'',95 from dual union all
select 21855,'1970-S13','1970-09-07 00:00:00','military missile','success',982,'G 22502-03T','8K64U UBP-146','','','OT',596,'',1210 from dual union all
select 21856,'1970-S130','1970-02-20 04:30:00','military missile','success',999,'Kh22511-64T','8K67 UBP-32','GCh 8F675','','OT',518,'',1000 from dual union all
select 21857,'1970-S131','1970-02-20 12:00:00','suborbital rocket','success',1189,'INTA MP-7011','','','','Ionos',396,'',96 from dual union all
select 21858,'1970-S132','1970-02-21 12:00:00','suborbital rocket','success',1189,'INTA MP-7012','','','','Ionos',396,'',94 from dual union all
select 21859,'1970-S133','1970-02-21 11:35:00','suborbital rocket','success',920,'P53H','UK-4 inst prototype','','','Ionospheric',1447,'',148 from dual union all
select 2186,'1954-A196','1954-04-26 00:00:00','military missile','success',1214,'N-69A GM-11111','','','','Test',238,'',10 from dual union all
select 21860,'1970-S134','1970-02-23 12:07:00','suborbital rocket','success',1189,'INTA MP-7013','','','','Ionos',396,'',90 from dual union all
select 21861,'1970-S135','1970-02-23 13:20:00','suborbital rocket','success',740,'MR-12','Wind trail','','','Aeron',1010,'',163 from dual union all
select 21862,'1970-S136','1970-02-23 16:49:40','suborbital rocket','success',290,'C37/1','ESRO C37/1 MPE Ba','','','Ionospheric Ba',408,'',210 from dual union all
select 21863,'1970-S137','1970-02-24 00:00:00','suborbital rocket','success',1189,'INTA SM-7001','','Tidal Mesosphere','','Ionos',396,'',100 from dual union all
select 21864,'1970-S138','1970-02-24 00:16:00','suborbital rocket','success',1189,'INTA SM-7002','','','','Ionos',396,'',100 from dual union all
select 21865,'1970-S139','1970-02-24 04:01:00','suborbital rocket','success',1189,'INTA SM-7003','','','','Ionos',396,'',100 from dual union all
select 21866,'1970-S140','1970-02-24 07:19:00','suborbital rocket','success',219,'AAD-4-23','VLF/Particles','','','Ionos/Plasma',428,'',796 from dual union all
select 21867,'1970-S141','1970-02-24 08:00:00','suborbital rocket','success',1189,'INTA SM-7004','','','','Ionos',396,'',100 from dual union all
select 21868,'1970-S142','1970-02-24 12:00:00','suborbital rocket','success',1189,'INTA SM-7005','','','','Ionos',396,'',97 from dual union all
select 21869,'1970-S143','1970-02-24 19:09:59','suborbital rocket','success',1204,'S70/1','ESRO S70/1','','','Ionospheric',416,'',186 from dual union all
select 2187,'1954-A197','1954-04-27 00:00:00','military missile','unknown',339,'4T','','','','Test',1895,'',30 from dual union all
select 21870,'1970-S144','1970-02-24 20:00:00','suborbital rocket','success',1189,'INTA SM-7007','Chaff','','','Ionos',396,'',102 from dual union all
select 21871,'1970-S145','1970-02-26 01:25:00','military missile','success',795,'855','Mk 11 RV','FOT GT53B','GLORY TRIP 53B','OT',1702,'',1300 from dual union all
select 21872,'1970-S146','1970-02-25 00:00:00','suborbital rocket','success',1189,'INTA SM-7008','','','','Ionos',396,'',92 from dual union all
select 21873,'1970-S147','1970-02-25 01:05:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',164 from dual union all
select 21874,'1970-S148','1970-02-25 04:00:00','suborbital rocket','success',1189,'INTA SM-7009','','','','Ionos',396,'',98 from dual union all
select 21875,'1970-S149','1970-02-25 08:00:00','suborbital rocket','success',1189,'INTA SM-7010','','','','Ionos',396,'',100 from dual union all
select 21876,'1970-S15','1970-01-01 00:00:00','military missile','success',1514,'','','','','Test',989,'',1000 from dual union all
select 21877,'1970-S150','1970-02-25 08:20:00','suborbital rocket','success',1189,'INTA SM-7011','','','','Ionos',396,'',95 from dual union all
select 21878,'1970-S151','1970-02-25 12:01:00','suborbital rocket','success',1189,'INTA SM-7012','','','','Ionos',396,'',96 from dual;
insert into LAUNCH
select 21879,'1970-S152','1970-02-25 16:00:00','suborbital rocket','success',1189,'INTA SM-7013','','','','Ionos',396,'',91 from dual union all
select 2188,'1954-A198','1954-04-27 00:00:00','atmospheric rocket','unknown',576,'503M','','','','Test',1895,'',10 from dual union all
select 21880,'1970-S153','1970-02-25 16:33:03','suborbital rocket','success',1204,'S61/1','ESRO S61/1','','','Aeronomy',416,'',220 from dual union all
select 21881,'1970-S154','1970-02-25 20:00:00','suborbital rocket','success',1189,'INTA SM-7014','Chaff','','','Ionos',396,'',101 from dual union all
select 21882,'1970-S155','1970-02-26 00:00:00','suborbital rocket','success',1189,'INTA SM-7015','','','','Ionos',396,'',100 from dual union all
select 21883,'1970-S156','1970-02-26 00:11:00','suborbital rocket','success',1189,'INTA SM-7016','','','','Ionos',396,'',95 from dual union all
select 21884,'1970-S157','1970-02-26 04:00:00','suborbital rocket','success',1189,'INTA SM-7017','','','','Ionos',396,'',100 from dual union all
select 21885,'1970-S158','1970-02-26 04:47:00','suborbital rocket','success',1204,'S61/2','ESRO S61/2','','','Aeronomy',416,'',225 from dual union all
select 21886,'1970-S159','1970-02-26 08:00:00','suborbital rocket','success',1189,'INTA SM-7018','','','','Ionos',396,'',96 from dual union all
select 21887,'1970-S16','1970-02-01 00:00:00','military missile','success',1036,'19','','','','Test',636,'',1000 from dual union all
select 21888,'1970-S160','1970-02-26 13:30:00','suborbital rocket','success',740,'MR-12','Wind trail','','','Aeron',1010,'',150 from dual union all
select 21889,'1970-S161','1970-02-26 14:15:00','suborbital rocket','success',740,'MR-12','Polar Atm/pressure','','','Aeron/Ionos',1010,'',181 from dual union all
select 2189,'1954-A199','1954-04-27 00:00:00','atmospheric rocket','unknown',576,'472M','','','','Test',1895,'',10 from dual union all
select 21890,'1970-S162','1970-02-26 20:00:00','suborbital rocket','success',1189,'INTA SM-7021','','','','Ionos',396,'',100 from dual union all
select 21891,'1970-S163','1970-02-27 00:00:00','suborbital rocket','success',1189,'INTA SM-7022','','','','Ionos',396,'',103 from dual union all
select 21892,'1970-S164','1970-02-27 04:00:00','suborbital rocket','success',1189,'INTA SM-7023','','','','Ionos',396,'',96 from dual union all
select 21893,'1970-S165','1970-02-27 08:00:00','suborbital rocket','success',1189,'INTA SM-7024','','','','Ionos',396,'',94 from dual union all
select 21894,'1970-S166','1970-02-27 12:00:00','suborbital rocket','success',1189,'INTA SM-7025','','','','Ionos',396,'',97 from dual union all
select 21895,'1970-S167','1970-02-27 16:00:00','suborbital rocket','success',1189,'INTA SM-7026','','','','Ionos',396,'',95 from dual union all
select 21896,'1970-S168','1970-02-27 20:10:00','suborbital rocket','success',1189,'INTA SM-7027','','','','Ionos',396,'',100 from dual union all
select 21897,'1970-S169','1970-02-27 20:55:00','suborbital rocket','success',1189,'INTA SM-7029','','','','Ionos',396,'',104 from dual union all
select 21898,'1970-S17','1970-01-16 00:00:00','military missile','success',1449,'','8K84 UBP-121','','','OT',504,'',1000 from dual union all
select 21899,'1970-S170','1970-03-01 00:00:00','suborbital rocket','success',742,'M-250','','','','Test',404,'',250 from dual union all
select 219,'1944-S199','1944-10-11 16:34:00','military missile','success',1470,'18814','Ma233','','','Test',661,'',90 from dual union all
select 2190,'1954-A20','1954-01-15 00:00:00','military missile','success',572,'7','RV-A-8','','','Test',1863,'',30 from dual union all
select 21900,'1970-S171','1970-03-01 00:00:00','suborbital rocket','success',742,'M-250','','','','Test',404,'',250 from dual union all
select 21901,'1970-S172','1970-03-01 00:00:00','military missile','success',1036,'20','','','','Test',636,'',1000 from dual union all
select 21902,'1970-S173','1970-02-28 00:00:00','suborbital rocket','success',1189,'INTA SM-7030','','','','Ionos',396,'',103 from dual union all
select 21903,'1970-S174','1970-02-28 03:15:00','suborbital rocket','success',837,'NASA 14.382GE','Auroral particles','','','Plasma',428,'',200 from dual union all
select 21904,'1970-S175','1970-02-28 04:00:00','suborbital rocket','success',1189,'INTA SM-7031','','','','Ionos',396,'',96 from dual union all
select 21905,'1970-S176','1970-02-28 08:00:00','suborbital rocket','success',1189,'INTA SM-7032','Tidal Mesosphere','','','Ionos',396,'',99 from dual union all
select 21906,'1970-S177','1970-02-28 12:00:00','suborbital rocket','success',1189,'INTA MP-7014','','','','Ionos',396,'',98 from dual union all
select 21907,'1970-S178','1970-02-28 13:51:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',166 from dual union all
select 21908,'1970-S179','1970-02-28 14:36:00','suborbital rocket','success',740,'MR-12','Polar Atm/pressure','','','Aeron/Ionos',1010,'',182 from dual union all
select 21909,'1970-S18','1970-02-21 00:00:00','military missile','success',1449,'','8K84 UBP-122','','','OT',542,'',1000 from dual union all
select 2191,'1954-A200','1954-04-27 00:00:00','military missile','success',771,'GM-12440','','','','Test',222,'',12 from dual union all
select 21910,'1970-S180','1970-03-01 04:00:00','suborbital rocket','success',30,'NRL NB3.237','','','','XR Astron',1869,'',210 from dual union all
select 21911,'1970-S181','1970-03-02 00:25:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',150 from dual union all
select 21912,'1970-S182','1970-03-02 01:40:00','suborbital rocket','unknown',740,'MR-12','Polar Atm/pressure','','','Aeron/Ionos',1010,'',179 from dual union all
select 21913,'1970-S183','1970-03-02 04:30:00','suborbital rocket','success',867,'NASA 18.87GE','Dual Ba 1A','','','Plasma Ba',188,'',270 from dual union all
select 21914,'1970-S184','1970-03-02 04:30:00','suborbital rocket','success',867,'NASA 18.90GE','Dual Ba 1B','','','Plasma Ba',1041,'',270 from dual union all
select 21915,'1970-S185','1970-03-02 14:37:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',150 from dual union all
select 21916,'1970-S186','1970-03-03 14:53:00','suborbital rocket','success',867,'NASA 18.108GE','Dual Ba 2B','','','Plasma',1041,'',270 from dual union all
select 21917,'1970-S187','1970-03-03 21:15:00','test rocket','success',207,'R-1','','','','Test',1844,'',550 from dual union all
select 21918,'1970-S188','1970-03-05 03:05:00','military missile','success',795,'782','Mk 11 RV','FOT GT54B','GLORY TRIP 54B','OT',1705,'',1300 from dual union all
select 21919,'1970-S189','1970-03-04 14:30:00','suborbital rocket','success',867,'NASA 18.88GE','Dual Ba 2A','','','Plasma',188,'',270 from dual union all
select 2192,'1954-A201','1954-04-27 00:00:00','atmospheric rocket','success',836,'2256T','','','','Test',1895,'',10 from dual union all
select 21920,'1970-S19','1970-04-18 00:00:00','military missile','success',1449,'','8K84 UBP-123','','','OT',505,'',1000 from dual union all
select 21921,'1970-S190','1970-03-04 14:41:00','suborbital rocket','success',867,'NASA 18.109GE','Dual Ba 3B','','','Plasma',1041,'',270 from dual union all
select 21922,'1970-S191','1970-03-05 12:13:00','suborbital rocket','success',1189,'INTA MP-7015','Horiz Sep. Chaff','','','Ionos',396,'',91 from dual union all
select 21923,'1970-S192','1970-03-05 12:17:00','suborbital rocket','success',1189,'INTA MP-7016','Horiz Sep. Chaff','','','Ionos',396,'',94 from dual union all
select 21924,'1970-S193','1970-03-05 14:10:00','suborbital rocket','success',867,'NASA 18.89GE','Dual Ba 3A','','','Plasma',188,'',270 from dual union all
select 21925,'1970-S194','1970-03-05 23:50:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',162 from dual union all
select 21926,'1970-S195','1970-03-06 09:27:00','suborbital rocket','success',839,'NASA 10.344GM','Ozone/H2O','','','Aeronomy',1821,'',100 from dual union all
select 21927,'1970-S196','1970-03-06 18:24:00','suborbital rocket','success',837,'NASA 14.426UM','UM Pitot 43','','','Aeronomy',1818,'',162 from dual union all
select 21928,'1970-S197','1970-03-06 02:12:50','suborbital rocket','success',226,'A-BBVA-27','DLR A-BBV-27 Chr''ine','','','Aeron/Ionos/Plasma',29,'',217 from dual union all
select 21929,'1970-S198','1970-03-06 02:31:00','suborbital rocket','success',675,'WRE K140','','','','Aeron',1832,'',93 from dual union all
select 2193,'1954-A202','1954-04-27 00:00:00','atmospheric rocket','success',836,'2320A','','','','Test',1895,'',10 from dual union all
select 21930,'1970-S199','1970-03-06 18:36:30','suborbital rocket','success',837,'NASA 14.456CI','Ionospheric/SX','','','Ionos/Solar X',1821,'',200 from dual union all
select 21931,'1970-S20','1970-06-11 00:00:00','military missile','success',1449,'','8K84 UBP-124','','','OT',536,'',1000 from dual union all
select 21932,'1970-S200','1970-03-06 23:35:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',150 from dual union all
select 21933,'1970-S201','1970-03-07 18:41:00','suborbital rocket','success',837,'NASA 14.429UM','UM Pitot 46','','','Eclipse Aeron',1821,'',150 from dual union all
select 21934,'1970-S202','1970-03-07 14:50:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',161 from dual union all
select 21935,'1970-S203','1970-03-07 15:45:00','suborbital rocket','success',837,'NASA 14.435UI','Ionospheric','','','Eclipse Ionos',1801,'',188 from dual union all
select 21936,'1970-S204','1970-03-07 16:51:00','suborbital rocket','success',848,'CRL A07.015-1','Sphere','','','Eclipse Aeron',339,'',105 from dual union all
select 21937,'1970-S205','1970-03-07 17:40:00','suborbital rocket','success',217,'AAF-3A-42','','','','Eclipse/Solar UV/X',406,'',152 from dual union all
select 21938,'1970-S206','1970-03-07 17:59:00','suborbital rocket','success',837,'NASA 14.427UM','UM Pitot 44','','','Eclipse Aeron',1818,'',171 from dual union all
select 21939,'1970-S207','1970-03-07 18:00:00','suborbital rocket','success',867,'NASA 18.104GA','UM Thermosphere 14','','','Eclipse Aeron',1816,'',289 from dual union all
select 2194,'1954-A203','1954-04-27 00:00:00','atmospheric rocket','success',836,'1404E','','','','Test',1895,'',10 from dual union all
select 21940,'1970-S208','1970-03-07 18:09:00','suborbital rocket','success',848,'CRL A07.024-1','O2-OH/Soft X','','','Eclipse Aeron,Solar X',339,'',97 from dual union all
select 21941,'1970-S209','1970-03-07 18:26:00','suborbital rocket','success',837,'NASA 14.428UM','UM Pitot 45','','','Eclipse Aeron',1818,'',176 from dual union all
select 21942,'1970-S21','1970-06-26 00:00:00','military missile','success',1449,'','8K84 UBP-125M','','','OT',542,'',1000 from dual union all
select 21943,'1970-S210','1970-03-07 18:27:00','suborbital rocket','success',867,'NASA 18.105GA','UM Thermosphere 15','','','Eclipse Aeron',1816,'',290 from dual union all
select 21944,'1970-S211','1970-03-07 18:35:55','suborbital rocket','success',37,'NASA 13.03DS','NRL NE4.238','','','Eclipse Solar UV',1797,'',165 from dual union all
select 21945,'1970-S212','1970-03-07 18:36:30','suborbital rocket','success',837,'NASA 14.457CI','','','','Eclipse Aeron',1812,'',200 from dual union all
select 21946,'1970-S213','1970-03-07 18:36:15','suborbital rocket','success',30,'NASA 04.317US','FUV Eclipse Spectrum','','','Eclipse Solar UV',1797,'',142 from dual union all
select 21947,'1970-S214','1970-03-07 18:37:25','suborbital rocket','success',867,'NASA 18.106GA','','','','Eclipse Aeron',1816,'',290 from dual union all
select 21948,'1970-S215','1970-03-07 18:37:10','suborbital rocket','success',837,'NASA 14.436UI','','','','Eclipse Ionos',1801,'',178 from dual union all
select 21949,'1970-S216','1970-03-07 18:38:00','suborbital rocket','success',837,'NASA 14.437UI','','','','Eclipse Ionos',1801,'',175 from dual union all
select 2195,'1954-A204','1954-04-27 00:00:00','atmospheric rocket','success',836,'1452E','','','','Test',1895,'',10 from dual union all
select 21950,'1970-S217','1970-03-07 18:39:30','suborbital rocket','success',848,'CRL A07.902-5','Mass spectrometer','','','Eclipse Aeron',1812,'',113 from dual union all
select 21951,'1970-S218','1970-03-07 18:40:40','suborbital rocket','success',837,'NASA 14.438UI','','','','Eclipse Ionos',1801,'',192 from dual union all
select 21952,'1970-S219','1970-03-07 18:46:00','suborbital rocket','success',848,'CRL A07.902-10','NACS Mass spec','','','Eclipse Aeron',1812,'',122 from dual union all
select 21953,'1970-S22','1970-12-04 00:00:00','military missile','success',1449,'','8K84 UBP-133M','','','OT',536,'',1000 from dual union all
select 21954,'1970-S220','1970-03-07 18:47:00','suborbital rocket','success',624,'NASA 8.57CE','','','','Eclipse Plasma',1821,'',829 from dual union all
select 21955,'1970-S221','1970-03-07 18:52:00','suborbital rocket','success',30,'NASA 04.284CS','','','','Solar Eclipse',1869,'',200 from dual union all
select 21956,'1970-S222','1970-03-07 18:55:00','suborbital rocket','success',217,'AAF-3A-43','','','','Eclipse/Solar UV/X',406,'',156 from dual union all
select 21957,'1970-S223','1970-03-07 18:57:00','suborbital rocket','success',217,'AAF-3A-44','','','','Eclipse/Solar UV/X',406,'',156 from dual union all
select 21958,'1970-S224','1970-03-07 19:00:00','suborbital rocket','success',30,'NASA 04.137DS','NRL NE3.216','','','Solar Eclipse',1869,'',187 from dual union all
select 21959,'1970-S225','1970-03-07 19:03:00','suborbital rocket','success',217,'AAF-3A-45','','','','Eclipse/Solar UV/X',406,'',156 from dual union all
select 2196,'1954-A205','1954-04-27 00:00:00','atmospheric rocket','success',836,'1552E','','','','Test',1895,'',10 from dual union all
select 21960,'1970-S226','1970-03-07 19:25:00','suborbital rocket','success',848,'CRL A07.016-2','Sphere','','','Eclipse Aeron',339,'',152 from dual union all
select 21961,'1970-S227','1970-03-07 19:30:00','suborbital rocket','success',35,'NASA 04.275DS','NRL NE3.235','','','Solar Eclipse',1869,'',163 from dual union all
select 21962,'1970-S228','1970-03-07 19:44:00','suborbital rocket','success',848,'CRL A07.024-2','Composition/Soft X','','','Eclipse Aeron/Solar X',339,'',99 from dual union all
select 21963,'1970-S229','1970-03-08 00:00:00','suborbital rocket','success',839,'NASA 10.331GM','GSFC Grenade 233 W','','','Aeronomy',1818,'',100 from dual union all
select 21964,'1970-S23','1970-12-10 00:00:00','military missile','success',1449,'','8K84 UBP-134M','','','OT',542,'',1000 from dual union all
select 21965,'1970-S230','1970-03-08 17:25:00','suborbital rocket','success',837,'NASA 14.430UM','UM Pitot 47','','','Aeronomy',1821,'',167 from dual union all
select 21966,'1970-S231','1970-03-08 00:17:36','suborbital rocket','success',867,'NASA 18.114IE','Ferdinand 24','','','Auroral/Ionos/Plas',29,'',215 from dual union all
select 21967,'1970-S232','1970-03-08 02:03:20','suborbital rocket','success',226,'','DLR A-BBV-26 Inger','','','Aeron/Ionos/Plasma',29,'',161 from dual union all
select 21968,'1970-S233','1970-03-09 03:34:00','military missile','success',1002,'N 22500-09T','8K67P UBP-8','GCh 8F675','','Test',519,'',1000 from dual;
insert into LAUNCH
select 21969,'1970-S234','1970-03-09 03:14:00','suborbital rocket','success',867,'CRL AT08.298','B field/particles','','','Auroral',428,'',298 from dual union all
select 2197,'1954-A206','1954-04-27 00:00:00','atmospheric rocket','success',836,'1724E','','','','Test',1895,'',10 from dual union all
select 21970,'1970-S235','1970-03-09 03:21:00','suborbital rocket','success',848,'CRL A07.907-2','E field','','','Plasma',428,'',158 from dual union all
select 21971,'1970-S236','1970-03-09 20:35:00','suborbital rocket','success',867,'NASA 18.99GI','ISRO 17.03','TERLS-158 UCB EUV Phot','','Ionos/EUV Astron',1467,'',264 from dual union all
select 21972,'1970-S237','1970-03-09 14:30:00','suborbital rocket','success',867,'NASA 18.97GI','ISRO 17.01','TERLS-156','','Ionos',1467,'',298 from dual union all
select 21973,'1970-S238','1970-03-09 19:38:00','suborbital rocket','success',867,'NASA 18.98GI','ISRO 17.02','TERLS-157','','Ionos',1467,'',296 from dual union all
select 21974,'1970-S239','1970-03-10 00:00:00','military missile','success',795,'1180','Mk 11 RV','FOT GT62B','GLORY TRIP 62B','OT',1698,'',1300 from dual union all
select 21975,'1970-S24','1970-10-04 00:00:00','military missile','success',1449,'','8K84 UBP-128M','','','OT',536,'',1000 from dual union all
select 21976,'1970-S240','1970-03-10 03:33:00','suborbital rocket','success',675,'WRE K139','','','','Aeron',1832,'',88 from dual union all
select 21977,'1970-S241','1970-03-11 23:00:00','military missile','success',796,'2424','Mk 11 RV','OT GT24F','GLORY TRIP 24F','OT',1708,'',1300 from dual union all
select 21978,'1970-S242','1970-03-11 22:01:00','suborbital rocket','success',1206,'SL921','MSSL XR3 RMC','','','XR Astronomy',1839,'',247 from dual union all
select 21979,'1970-S243','1970-03-12 21:22:00','suborbital rocket','unknown',846,'STV 92','','','','Aeron?',1866,'',222 from dual union all
select 2198,'1954-A207','1954-04-28 00:00:00','atmospheric rocket','success',836,'1358A','','','','Test',1895,'',10 from dual union all
select 21980,'1970-S244','1970-03-13 11:25:00','military missile','success',168,'28F','RMV-340B/RMV-410B/420E','ABRES RMP-B-16/KX-116','','Reentry Test',1682,'',1400 from dual union all
select 21981,'1970-S245','1970-03-13 06:18:00','suborbital rocket','success',30,'CRL A03.006-1','Electron beam','','','Aeron/Ionos',428,'',205 from dual union all
select 21982,'1970-S246','1970-03-14 03:34:00','suborbital rocket','success',30,'NASA 04.328DG','UV','','','UV Astronomy',1869,'',164 from dual union all
select 21983,'1970-S247','1970-03-15 13:35:00','suborbital rocket','success',219,'','DLR A-BB4-28 Eva','','','Ionos/Plasma',29,'',650 from dual union all
select 21984,'1970-S248','1970-03-18 02:45:00','suborbital rocket','success',219,'','DLR A-BB4-29 Gerda','','','Ionos/Plasma',29,'',665 from dual union all
select 21985,'1970-S249','1970-03-18 22:30:00','suborbital rocket','success',1207,'SL901','LUX XRS7 BCS','','','XR Astronomy',1839,'',208 from dual union all
select 21986,'1970-S25','1970-10-23 00:00:00','military missile','success',1449,'','8K84 UBP-129M','','','OT',542,'',1000 from dual union all
select 21987,'1970-S250','1970-03-19 02:30:50','suborbital rocket','success',1189,'K70/01','RTG SSK3/1','','','Aeron/Ionos',417,'',108 from dual union all
select 21988,'1970-S251','1970-03-19 03:19:00','suborbital rocket','success',837,'NASA 14.425GI','ISRO 10.33/17.04','TERLS-170','','Ionos',1467,'',200 from dual union all
select 21989,'1970-S252','1970-03-19 03:23:30','suborbital rocket','success',1189,'K70/02','RTG SSK3/2','','','Aeron/Ionos',417,'',108 from dual union all
select 2199,'1954-A208','1954-04-28 00:00:00','atmospheric rocket','success',836,'2163R','Nike 276R','','','Test',1895,'',10 from dual union all
select 21990,'1970-S253','1970-03-19 03:56:40','suborbital rocket','success',1189,'K70/03','RTG SSK3/3 (K70/03)','','','Aeron/Ionos',417,'',101 from dual union all
select 21991,'1970-S254','1970-03-19 04:28:50','suborbital rocket','success',1189,'K70/04','RTG SSK3/4 (K70/04)','','','Aeron/Ionos',417,'',105 from dual union all
select 21992,'1970-S255','1970-03-19 05:09:00','suborbital rocket','success',837,'NASA 14.424GI','ISRO 10.34/17.05','TERLS-171','','Ionos',1467,'',97 from dual union all
select 21993,'1970-S256','1970-03-19 05:18:00','suborbital rocket','success',1189,'K70/05','RTG SSK3/5 (K70/05)','','','Aeron/Ionos',417,'',105 from dual union all
select 21994,'1970-S257','1970-03-19 10:01:00','suborbital rocket','success',837,'NASA 14.372GI','ISRO 10.35/17.06','TERLS-172','','Ionos',1467,'',196 from dual union all
select 21995,'1970-S258','1970-03-19 15:18:00','suborbital rocket','success',1207,'SL401','ROE-9 UV Schmidt OP','','','UV Astronomy',1839,'',181 from dual union all
select 21996,'1970-S259','1970-03-23 00:00:00','military missile','success',795,'1094','Mk 11 RV','FOT GT63B','GLORY TRIP 63B','OT',1702,'',1300 from dual union all
select 21997,'1970-S26','1970-10-30 00:00:00','military missile','success',1449,'','8K84 UBP-130M','','','OT',536,'',1000 from dual union all
select 21998,'1970-S260','1970-03-23 22:58:24','suborbital rocket','success',839,'Ferdinand 22','Ferdinand 22','','','Aeron/Ionos',29,'',103 from dual union all
select 21999,'1970-S261','1970-03-24 19:17:00','military missile','failure',939,'C3XT-22','','','','Test',421,'EAG-154',500 from dual union all
select 22,'1943-A15','1943-12-05 01:00:00','military missile','success',8,'','','','','Test',662,'',20 from dual union all
select 220,'1944-S20','1944-03-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 2200,'1954-A209','1954-04-28 00:00:00','atmospheric rocket','success',836,'1321R','Nike 274R','','','Test',1895,'',10 from dual union all
select 22000,'1970-S262','1970-03-24 22:00:00','suborbital rocket','success',1207,'SL802','LUX XRS8 MC','','','XR Astronomy',1839,'',179 from dual union all
select 22001,'1970-S263','1970-03-25 07:01:00','suborbital rocket','success',848,'CRL A07.907-3','E field','','','Plasma',428,'',169 from dual union all
select 22002,'1970-S264','1970-03-26 09:00:00','military missile','success',796,'2656','Mk 11C RV','OT GT25M','GLORY TRIP 25M','OT',1700,'',1300 from dual union all
select 22003,'1970-S265','1970-03-26 03:17:00','suborbital rocket','success',848,'CRL A07.907-4','E field','','','Plasma',428,'',101 from dual union all
select 22004,'1970-S266','1970-03-26 04:44:00','suborbital rocket','success',31,'NASA 04.309UA','JHU Aurora 8','','','Aeron/Auroral',428,'',163 from dual union all
select 22005,'1970-S267','1970-03-26 04:45:00','suborbital rocket','success',867,'NASA 18.82UA','Airglow','','','Aeronomy',428,'',236 from dual union all
select 22006,'1970-S268','1970-03-27 05:25:00','suborbital rocket','success',837,'NASA 14.405IE/II','ISRO 10.36','TERLS-175','','Plasma/Ionos',1467,'',195 from dual union all
select 22007,'1970-S269','1970-03-27 15:50:00','suborbital rocket','success',839,'Rehbar 21','','','','Aeron',1419,'',100 from dual union all
select 22008,'1970-S27','1970-10-31 00:00:00','military missile','success',1449,'','8K84 UBP-131M','','','OT',542,'',1000 from dual union all
select 22009,'1970-S270','1970-03-28 07:37:22','military missile','success',1373,'152','AFP-437','','','Interceptor',772,'',1074 from dual union all
select 2201,'1954-A21','1954-01-15 00:00:00','atmospheric rocket','success',836,'1596E','','','','Test',1895,'',10 from dual union all
select 22010,'1970-S271','1970-03-28 15:45:00','suborbital rocket','success',839,'Rehbar 22','','','','Aeron',1419,'',141 from dual union all
select 22011,'1970-S272','1970-03-28 22:30:00','military missile','success',1002,'','8K67P UBP-9','','','Test',518,'',1000 from dual union all
select 22012,'1970-S273','1970-03-30 01:30:00','military missile','success',1002,'','8K67P UBP-10','','','Test',519,'',1000 from dual union all
select 22013,'1970-S274','1970-04-01 00:00:00','suborbital rocket','unknown',13,'','','','','Test?',1895,'F-4D',100 from dual union all
select 22014,'1970-S275','1970-03-31 00:00:00','military missile','success',796,'2594','Mk 11 RV','OT GT23F','GLORY TRIP 23F','OT',1712,'',1300 from dual union all
select 22015,'1970-S276','1970-06-25 00:00:00','military missile','success',916,'','P-181 70 Spring 08','','','OT',204,'',180 from dual union all
select 22016,'1970-S277','1970-04-02 00:00:00','military missile','success',799,'FTM-309','Mk 12 RV x 3','R&D','','Test',1703,'',1300 from dual union all
select 22017,'1970-S278','1970-04-02 01:46:00','suborbital rocket','success',1204,'S66/1','ESRO S66/1','','','Ionospheric',416,'',198 from dual union all
select 22018,'1970-S279','1970-04-03 01:22:00','suborbital rocket','success',290,'C57/2','ESRO C57/2','','','Aeronomy',408,'',173 from dual union all
select 22019,'1970-S28','1970-11-28 00:00:00','military missile','success',1449,'','8K84 UBP-132M','','','OT',542,'',1000 from dual union all
select 2202,'1954-A210','1954-04-28 00:00:00','atmospheric rocket','success',836,'1605E','','','','Test',1895,'',10 from dual union all
select 22020,'1970-S280','1970-04-03 04:59:00','suborbital rocket','success',624,'NASA 8.56UE','Auroral substorm','','','Plasma',428,'',557 from dual union all
select 22021,'1970-S281','1970-04-03 09:03:00','suborbital rocket','success',846,'GI-SF6','Na hexaflouride','','','Auroral Release',1047,'',150 from dual union all
select 22022,'1970-S282','1970-04-03 20:30:07','military missile','success',799,'FTM-213','Mk 12 RV x 3','R&D','','Test',266,'',1300 from dual union all
select 22023,'1970-S283','1970-04-04 17:28:01','suborbital rocket','success',228,'CRL A18.903-3','EUV','','','Solar EUV',394,'',354 from dual union all
select 22024,'1970-S284','1970-04-04 19:12:00','suborbital rocket','success',1204,'S28/1','ESRO S28/1 QUB Na-12','','','Aeronomy',416,'',203 from dual union all
select 22025,'1970-S285','1970-04-07 03:20:00','suborbital rocket','success',1207,'SL803','Cul UVS 16','','','Solar UV',1839,'',204 from dual union all
select 22026,'1970-S286','1970-04-10 16:30:00','military missile','failure',932,'A3E','','','','OT',421,'SSBN 625',10 from dual union all
select 22027,'1970-S287','1970-04-11 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 641',1000 from dual union all
select 22028,'1970-S288','1970-04-11 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 641',1000 from dual union all
select 22029,'1970-S289','1970-04-11 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 641',1000 from dual union all
select 2203,'1954-A211','1954-04-28 00:00:00','atmospheric rocket','success',836,'1323R  Bell motor','Nike 275R','','','Test',1895,'',10 from dual union all
select 22030,'1970-S29','1970-07-27 05:37:00','military missile','success',1450,'6305761-022','15A20 LKI-21','GCh 15F202','','Test',532,'',1000 from dual union all
select 22031,'1970-S290','1970-04-11 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 641',1000 from dual union all
select 22032,'1970-S291','1970-04-11 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 641',1000 from dual union all
select 22033,'1970-S292','1970-04-11 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 641',1000 from dual union all
select 22034,'1970-S293','1970-04-11 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 641',1000 from dual union all
select 22035,'1970-S294','1970-04-11 04:05:00','military missile','success',1002,'Yu22500-02L','8K67P UBP-11','GCh 8F675','','Test',519,'',1000 from dual union all
select 22036,'1970-S295','1970-04-11 03:45:00','suborbital rocket','success',867,'NASA 18.91UE','','','','Plasma/Auroral',428,'',266 from dual union all
select 22037,'1970-S296','1970-04-13 03:35:00','military missile','success',1002,'','8K67P UBP-12','','','Test',515,'',1000 from dual union all
select 22038,'1970-S297','1970-04-13 12:34:00','suborbital rocket','success',30,'NASA 04.321UA','','','','Aeronomy',1869,'',200 from dual union all
select 22039,'1970-S298','1970-04-14 00:00:00','military missile','failure',1251,'800018','SAFEGUARD M1-1','','','Interceptor',868,'',40 from dual union all
select 2204,'1954-A212','1954-04-28 00:00:00','atmospheric rocket','success',836,'1468T','','','','Test',1895,'',10 from dual union all
select 22040,'1970-S299','1970-04-14 15:30:00','suborbital rocket','success',44,'NASA 17.05GT-GG','','','','XR Astron/Test',1895,'',88 from dual union all
select 22041,'1970-S30','1970-07-28 05:02:00','military missile','success',1450,'Kh10761-020','15A20 LKI-22','GCh 15F202','','Test',505,'',1000 from dual union all
select 22042,'1970-S300','1970-04-15 03:35:00','military missile','success',1002,'Kh22500-07L','8K67P UBP-13','GCh 8F675','','Test',518,'',1000 from dual union all
select 22043,'1970-S301','1970-04-16 20:00:00','military missile','success',916,'AO-87','P-174 70 Spring 01','','','Test',204,'',180 from dual union all
select 22044,'1970-S302','1970-04-16 20:37:00','military missile','success',916,'AO-88','P-175 70 Spring 02','','','Test',204,'',180 from dual union all
select 22045,'1970-S303','1970-04-16 06:38:00','suborbital rocket','unknown',331,'WRE Cockatoo 101','Sphere','','','Aeron',1835,'',100 from dual union all
select 22046,'1970-S304','1970-04-16 06:40:00','suborbital rocket','success',837,'NASA 14.443UI','','','','Ionos',428,'',136 from dual union all
select 22047,'1970-S305','1970-04-16 07:06:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',170 from dual union all
select 22048,'1970-S306','1970-04-16 07:30:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',175 from dual union all
select 22049,'1970-S307','1970-04-16 09:07:50','suborbital rocket','success',1206,'SL728','UAT-5/MO-11','','','XR Astronomy/Aeron',1839,'',210 from dual union all
select 2205,'1954-A213','1954-04-28 00:00:00','atmospheric rocket','success',836,'1676E','','','','Test',1895,'',10 from dual union all
select 22050,'1970-S308','1970-04-16 10:02:00','suborbital rocket','unknown',331,'WRE Cockatoo 102','Sphere','','','Aeron',1835,'',100 from dual union all
select 22051,'1970-S309','1970-04-16 11:02:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',170 from dual union all
select 22052,'1970-S31','1970-01-06 03:05:00','military missile','success',1450,'Yu10761-009','15A20 LKI-8','GCh 15B20','','Test',504,'',1000 from dual union all
select 22053,'1970-S310','1970-04-20 00:00:00','military missile','success',1103,'C4','S02C4','','','Test',301,'',1000 from dual union all
select 22054,'1970-S311','1970-04-20 03:35:00','military missile','success',999,'4502742-264T','8K67 UBP-33','GCh 8F675','','OT',519,'',1000 from dual union all
select 22055,'1970-S312','1970-04-22 01:01:00','military missile','success',796,'2474','Mk 11 RV','OT GT48F','GLORY TRIP 48F','OT',1710,'',1300 from dual union all
select 22056,'1970-S313','1970-04-21 00:00:00','military missile','unknown',929,'A2','OT','','','OT',421,'SSBN 608',1000 from dual union all
select 22057,'1970-S314','1970-04-21 00:00:00','military missile','unknown',929,'A2','OT','','','OT',421,'SSBN 608',1000 from dual;
insert into LAUNCH
select 22058,'1970-S315','1970-04-21 00:00:00','military missile','unknown',929,'A2','OT','','','OT',421,'SSBN 608',1000 from dual union all
select 22059,'1970-S316','1970-04-21 00:00:00','military missile','unknown',929,'A2','OT','','','OT',421,'SSBN 608',1000 from dual union all
select 2206,'1954-A214','1954-04-29 00:00:00','atmospheric rocket','success',1441,'','Test','','','Test',1821,'',10 from dual union all
select 22060,'1970-S317','1970-04-21 00:00:00','military missile','unknown',929,'A2','OT','','','OT',421,'SSBN 608',1000 from dual union all
select 22061,'1970-S318','1970-04-21 00:00:00','military missile','unknown',929,'A2','OT','','','OT',421,'SSBN 608',1000 from dual union all
select 22062,'1970-S319','1970-04-21 00:00:00','military missile','unknown',929,'A2','OT','','','OT',421,'SSBN 608',1000 from dual union all
select 22063,'1970-S32','1970-01-19 02:30:00','military missile','success',1450,'Yu10761-008','15A20 LKI-9','GCh 15B20','','Test',505,'',1000 from dual union all
select 22064,'1970-S320','1970-04-21 06:01:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',636,'',175 from dual union all
select 22065,'1970-S321','1970-04-21 20:42:00','suborbital rocket','success',920,'P36H','Particles','','','Ionospheric',1447,'',139 from dual union all
select 22066,'1970-S322','1970-04-22 00:00:00','suborbital rocket','unknown',740,'','','','','Ionos',636,'',170 from dual union all
select 22067,'1970-S323','1970-04-22 13:45:00','military missile','success',799,'FTM-308','Mk 12 RV x 3','R&D','','Test',1697,'',1300 from dual union all
select 22068,'1970-S324','1970-04-24 07:44:00','military missile','success',135,'V141D','ABRES CBD (USAF)','ABRES CBD (USAF)','','Reentry Test',612,'',200 from dual union all
select 22069,'1970-S325','1970-04-23 07:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',175 from dual union all
select 2207,'1954-A215','1954-04-29 00:00:00','atmospheric rocket','unknown',576,'477M','','','','Test',1895,'',10 from dual union all
select 22070,'1970-S326','1970-04-23 07:32:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',170 from dual union all
select 22071,'1970-S327','1970-04-23 11:01:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',170 from dual union all
select 22072,'1970-S328','1970-04-25 00:00:00','military missile','success',1373,'225','SDP  (DSDD)','','','Test',773,'',500 from dual union all
select 22073,'1970-S329','1970-04-24 00:52:00','suborbital rocket','success',227,'AMD-VB-25','','','','Auroral/Ionos',428,'',288 from dual union all
select 22074,'1970-S33','1970-02-18 02:07:00','military missile','success',1450,'Yu10761-010','15A20 LKI-10','GCh 15B20','','Test',505,'',1000 from dual union all
select 22075,'1970-S330','1970-04-24 07:30:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',636,'',175 from dual union all
select 22076,'1970-S331','1970-04-24 08:53:00','suborbital rocket','success',30,'KP3.30','CAL XR 5 [Ae170]','','','XR Astron',1869,'',274 from dual union all
select 22077,'1970-S332','1970-04-25 00:00:00','military missile','success',796,'2588','Mk 11 RV','OT GT57F (DSDD Targ)','GLORY TRIP 57F','OT',1708,'',1300 from dual union all
select 22078,'1970-S333','1970-04-26 13:05:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',175 from dual union all
select 22079,'1970-S334','1970-04-27 03:50:00','military missile','success',1002,'Kh22500-08L','8K67P UBP-14','GCh 8F675','','Test',515,'',1000 from dual union all
select 2208,'1954-A216','1954-04-29 00:00:00','atmospheric rocket','unknown',576,'475M','','','','Test',1895,'',10 from dual union all
select 22080,'1970-S335','1970-04-27 00:00:00','suborbital rocket','success',1014,'','R-5V Geophysical','RAO','','Solar?',636,'',500 from dual union all
select 22081,'1970-S336','1970-04-28 09:00:00','suborbital rocket','success',30,'NASA 04.326NA','Luster','','','Aeronomy/Dust',428,'',127 from dual union all
select 22082,'1970-S337','1970-04-29 16:02:06','military missile','success',799,'FTM-214','Mk 12 RV x 3','R&D','','Test',266,'',1300 from dual union all
select 22083,'1970-S338','1970-05-05 17:00:00','suborbital rocket','unknown',846,'STV-93','','','','Aeron?',1867,'',214 from dual union all
select 22084,'1970-S339','1970-05-05 14:00:00','suborbital rocket','unknown',846,'STV-94','','','','Aeron?',1866,'',196 from dual union all
select 22085,'1970-S34','1970-02-19 02:09:00','military missile','success',1450,'Yu10761-011','15A20 LKI-11','GCh 15F202','','Test',505,'',1000 from dual union all
select 22086,'1970-S340','1970-05-01 15:53:00','suborbital rocket','success',30,'NASA 04.327NA','Luster','','','Aeronomy/Dust',428,'',128 from dual union all
select 22087,'1970-S341','1970-05-03 12:45:00','suborbital rocket','success',894,'','','','','CR/Fields',308,'',90 from dual union all
select 22088,'1970-S342','1970-05-05 03:50:00','military missile','success',795,'1330','Mk 11A RV','FOT GT64B','GLORY TRIP 64B','OT',1702,'',1300 from dual union all
select 22089,'1970-S343','1970-05-05 06:26:00','suborbital rocket','success',864,'Sandia 152-136','LRL ACS-1B [or 0822]','','','XR Astron',846,'',144 from dual union all
select 2209,'1954-A217','1954-04-29 00:00:00','atmospheric rocket','success',854,'','F40 Hypersonic Test','','','Test',1821,'',13 from dual union all
select 22090,'1970-S344','1970-05-06 22:26:00','suborbital rocket','success',446,'D-37','Electron density','','','Test/Plasma/Fields/',308,'',430 from dual union all
select 22091,'1970-S345','1970-05-08 01:05:00','suborbital rocket','success',848,'CRL AH07.677','Release','','','Aeron',394,'',159 from dual union all
select 22092,'1970-S346','1970-05-08 02:35:00','suborbital rocket','success',848,'CRL A07.901-2','Release','','','Aeron',394,'',153 from dual union all
select 22093,'1970-S347','1970-05-08 07:49:00','military missile','success',799,'FTM-310','Mk 12 RV x 3','R&D','','Test',1697,'',1300 from dual union all
select 22094,'1970-S348','1970-05-08 10:15:00','suborbital rocket','success',37,'NASA 13.06UG','MIT-8','','','XR Astron',1895,'',146 from dual union all
select 22095,'1970-S349','1970-05-11 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 657',1000 from dual union all
select 22096,'1970-S35','1970-04-14 04:10:00','military missile','success',1450,'Yu10761-012','15A20 LKI-12','GCh 15F202','','Test',505,'',1000 from dual union all
select 22097,'1970-S350','1970-05-11 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 657',1000 from dual union all
select 22098,'1970-S351','1970-05-11 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 657',1000 from dual union all
select 22099,'1970-S352','1970-05-11 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 657',1000 from dual union all
select 221,'1944-S200','1944-10-11 17:53:00','military missile','success',1470,'18818','Ma236','','','Test',661,'',90 from dual union all
select 2210,'1954-A218','1954-04-29 00:00:00','atmospheric rocket','success',836,'2288T','','','','Test',1895,'',10 from dual union all
select 22100,'1970-S353','1970-05-11 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 657',1000 from dual union all
select 22101,'1970-S354','1970-05-11 00:00:00','military missile','unknown',932,'A3','OT','','','OT',421,'SSBN 657',1000 from dual union all
select 22102,'1970-S355','1970-05-11 19:42:00','suborbital rocket','success',30,'NASA 04.322UA','','','','Aeronomy',428,'',188 from dual union all
select 22103,'1970-S356','1970-05-13 08:03:00','suborbital rocket','success',1347,'Sandia 281-14','LRL ACS-4','','','XR Astron',855,'',306 from dual union all
select 22104,'1970-S357','1970-05-14 20:00:00','military missile','success',939,'C3XT-21','','','','Test',246,'',500 from dual union all
select 22105,'1970-S358','1970-05-16 02:30:00','suborbital rocket','success',1014,'','R-5V Geophysical','RAO K-2 No. 1','','Solar X',636,'',500 from dual union all
select 22106,'1970-S359','1970-05-16 05:45:00','suborbital rocket','success',1347,'SECEDE R9','ROADRUNNER','','','Aeron/Fields',855,'',208 from dual union all
select 22107,'1970-S36','1970-04-17 04:05:00','military missile','success',1450,'Yu10761-013','15A20 LKI-13','GCh 15F202','','Test',504,'',1000 from dual union all
select 22108,'1970-S360','1970-05-21 18:16:00','military missile','success',796,'2486','Mk 11 RV','OT GT55F','GLORY TRIP 55F','OT',1711,'',1300 from dual union all
select 22109,'1970-S361','1970-05-21 01:08:00','suborbital rocket','success',848,'CRL A07.901-4','Release','','','Aeron',394,'',130 from dual union all
select 2211,'1954-A219','1954-04-29 00:00:00','atmospheric rocket','success',836,'2264T','','','','Test',1895,'',10 from dual union all
select 22110,'1970-S362','1970-05-21 02:56:00','suborbital rocket','success',675,'WRE K143','','','','Aeron',1832,'',85 from dual union all
select 22111,'1970-S363','1970-05-21 10:15:00','suborbital rocket','success',331,'WRE Cockatoo 104','Lunar UV Ozone','','','Aeron',1835,'',96 from dual union all
select 22112,'1970-S364','1970-05-21 10:35:00','suborbital rocket','success',848,'CRL A07.901-3','Release','','','Aeron',394,'',153 from dual union all
select 22113,'1970-S365','1970-05-21 11:06:00','suborbital rocket','success',848,'CRL A07.901-1','Release','','','Aeron',394,'',163 from dual union all
select 22114,'1970-S366','1970-05-22 18:00:00','military missile','unknown',932,'A3E','','','','OT',421,'SSBN 625',1000 from dual union all
select 22115,'1970-S367','1970-05-22 11:12:00','suborbital rocket','success',848,'CRL A07.901-5','Release','','','Aeron Ba',394,'',142 from dual union all
select 22116,'1970-S368','1970-05-25 08:00:00','suborbital rocket','success',30,'NRL NB3.211','','','','XR Astron?',1869,'',200 from dual union all
select 22117,'1970-S369','1970-05-26 12:07:00','suborbital rocket','success',30,'NASA 04.316NG','','','','UV Astronomy',1846,'',200 from dual union all
select 22118,'1970-S37','1970-05-16 04:25:00','military missile','success',1450,'Yu10761-014','15A20 LKI-14','GCh 15B20','','Test',505,'',1000 from dual union all
select 22119,'1970-S370','1970-05-26 03:38:00','suborbital rocket','success',675,'WRE K144','','','','Aeron',1832,'',87 from dual union all
select 2212,'1954-A22','1954-01-15 00:00:00','atmospheric rocket','success',836,'1435E','','','','Test',1895,'',10 from dual union all
select 22120,'1970-S371','1970-05-26 05:50:00','suborbital rocket','success',1347,'SECEDE R10','SAPSUCKER','','','Aeron/Fields',855,'',204 from dual union all
select 22121,'1970-S372','1970-05-26 07:00:00','suborbital rocket','unknown',331,'WRE Cockatoo 103','Sphere','','','Aeron',1835,'',100 from dual union all
select 22122,'1970-S373','1970-05-27 08:40:00','military missile','success',795,'1404','Mk 11A RV','FOT GT65B','GLORY TRIP 65B','OT',1698,'',1300 from dual union all
select 22123,'1970-S374','1970-05-27 15:59:42','military missile','success',799,'FTM-215','Mk 12 RV x 3','R&D','','Test',266,'',1300 from dual union all
select 22124,'1970-S375','1970-05-29 13:45:00','suborbital rocket','success',30,'NASA 04.301UG','','','','XR Astron',1846,'',200 from dual union all
select 22125,'1970-S376','1970-05-30 08:38:00','military missile','success',168,'91F','RMV-340C/RMV-430C/420F','ABRES RMP-B-17/KX-117','','Reentry Test',1682,'',1400 from dual union all
select 22126,'1970-S378','1970-06-03 01:10:00','military missile','success',916,'AO-90','P-178 70 Spring 04','','','Test',204,'',180 from dual union all
select 22127,'1970-S379','1970-06-04 20:00:00','military missile','success',916,'AO-91','P-179 70 Spring 05','','','Test',204,'',180 from dual union all
select 22128,'1970-S38','1970-05-27 04:31:00','military missile','success',1450,'Kh10761-017','15A20 LKI-15','GCh 15B20','','Test',505,'',1000 from dual union all
select 22129,'1970-S380','1970-06-05 00:00:00','military missile','success',916,'','P-180 70 Spring 06','','','Test',204,'',180 from dual union all
select 2213,'1954-A220','1954-04-30 00:00:00','atmospheric rocket','unknown',576,'212','','','','Test',1895,'',10 from dual union all
select 22130,'1970-S381','1970-06-25 00:00:00','military missile','success',916,'','P-181 70 Spring 07','','','OT',204,'',180 from dual union all
select 22131,'1970-S382','1970-06-03 00:00:00','suborbital rocket','success',1276,'','MTVI-2','','','Reentry Test',855,'',200 from dual union all
select 22132,'1970-S383','1970-06-01 15:30:00','military missile','unknown',932,'A3E-671','','','','OT',421,'UK S27',1000 from dual union all
select 22133,'1970-S384','1970-06-02 18:00:00','suborbital rocket','success',30,'NASA 04.302UG','','','','XR Astron',1846,'',200 from dual union all
select 22134,'1970-S385','1970-06-02 07:00:00','suborbital rocket','success',30,'NASA 04.311UA','JHU UVT F5 (Jupiter)','','','UV Astron',1869,'',200 from dual union all
select 22135,'1970-S386','1970-06-02 04:47:00','suborbital rocket','success',30,'NASA 04.271UG','','','','UV Astronomy',1869,'',165 from dual union all
select 22136,'1970-S387','1970-06-06 05:50:00','suborbital rocket','success',1347,'SECEDE R11','TITMOUSE','','','Aeron/Fields',855,'',210 from dual union all
select 22137,'1970-S388','1970-06-07 07:25:50','suborbital rocket','success',290,'C48/1','ESRO C48/1','','','Solar UV',408,'',162 from dual union all
select 22138,'1970-S389','1970-06-08 00:00:00','suborbital rocket','success',132,'Test 1','','','','Test',1895,'',98 from dual union all
select 22139,'1970-S39','1970-06-05 06:15:00','military missile','success',1450,'Kh10761-016','15A20 LKI-16','GCh 15F202 Min-range','','Test',505,'',1000 from dual union all
select 2214,'1954-A221','1954-04-30 00:00:00','atmospheric rocket','unknown',576,'277','','','','Test',1895,'',10 from dual union all
select 22140,'1970-S390','1970-06-08 00:00:00','suborbital rocket','success',132,'Test 2','','','','Test',1895,'',98 from dual union all
select 22141,'1970-S391','1970-06-08 23:00:00','military missile','success',795,'1242','Mk 11 RV','FOT GT72B','GLORY TRIP 72B','OT',1702,'',1300 from dual union all
select 22142,'1970-S392','1970-06-09 12:23:00','military missile','success',162,'92F','WT1A-S7/WT1A-L7','ABRES RVTO-1A-7','','Reentry Test',1680,'',1400 from dual union all
select 22143,'1970-S393','1970-06-11 00:00:00','military missile','success',751,'C1','C1 Charge (RV)','','','OT',307,'S655',500 from dual union all
select 22144,'1970-S394','1970-06-15 15:55:00','suborbital rocket','success',30,'KP3.31','','','','Solar UV/Aeron',1869,'',150 from dual union all
select 22145,'1970-S395','1970-06-17 08:00:00','military missile','success',799,'FTM-312','Mk 12 RV x 3','R&D','','Test',1697,'',1300 from dual union all
select 22146,'1970-S396','1970-06-17 15:00:01','military missile','success',939,'C3XT-23','','','','Test',246,'',500 from dual union all
select 22147,'1970-S397','1970-06-18 15:30:00','military missile','unknown',932,'A3E-468','','','','OT',421,'UK S27',1000 from dual;
insert into LAUNCH
select 22148,'1970-S398','1970-06-19 13:15:00','military missile','success',796,'2478','Mk 11C RV','OT GT28M','GLORY TRIP 28M','OT',1700,'',1300 from dual union all
select 22149,'1970-S399','1970-06-20 00:00:00','military missile','success',796,'2338','Mk 11C RV','OT GT27M','GLORY TRIP 27M','OT',1699,'',1300 from dual union all
select 2215,'1954-A222','1954-04-30 00:00:00','military missile','success',771,'GM-12477','','','','Test',222,'',12 from dual union all
select 22150,'1970-S40','1970-06-22 03:25:00','military missile','success',1450,'Kh10761-018','15A20 LKI-17','GCh 15B20 Min-range','','Test',504,'',1000 from dual union all
select 22151,'1970-S400','1970-06-22 14:45:00','suborbital rocket','success',30,'NASA 04.323US','','','','Solar',1869,'',200 from dual union all
select 22152,'1970-S401','1970-06-22 16:00:00','suborbital rocket','success',839,'NASA 10.332GM','GSFC Grenade 234 W','','','Aeronomy',1821,'',132 from dual union all
select 22153,'1970-S402','1970-06-24 04:33:00','military missile','success',799,'FTM-313','Mk 12 RV x 3','R&D','','Test',1703,'',1300 from dual union all
select 22154,'1970-S403','1970-06-24 00:00:00','military missile','success',1251,'800019','SAFEGUARD M1-1A','','','Interceptor',868,'',100 from dual union all
select 22155,'1970-S404','1970-06-25 23:10:00','military missile','success',795,'1413','Mk 11A RV','FOT GT73B','GLORY TRIP 73B','OT',1698,'',1300 from dual union all
select 22156,'1970-S405','1970-06-25 15:37:00','suborbital rocket','success',839,'NASA 10.342GM','GSFC Grenade 235 W','','','Aeronomy',1821,'',84 from dual union all
select 22157,'1970-S406','1970-06-26 00:00:00','military missile','success',796,'2559','Mk 11 RV','OT GT59F','GLORY TRIP 59F','OT',1708,'',1300 from dual union all
select 22158,'1970-S407','1970-06-26 18:20:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',145 from dual union all
select 22159,'1970-S408','1970-06-27 04:55:00','suborbital rocket','success',37,'NASA 13.12CG','ASE 1D-XRT Cyg Loop','','','XR Astron',1895,'',175 from dual union all
select 2216,'1954-A223','1954-04-30 00:00:00','military missile','success',771,'GM-12476','','','','Test',222,'',12 from dual union all
select 22160,'1970-S409','1970-06-27 08:45:00','suborbital rocket','success',837,'NASA 14.413UG','','','','XR Astron',1895,'',200 from dual union all
select 22161,'1970-S41','1970-07-03 03:10:00','military missile','success',1450,'Kh10761-019','15A20 LKI-18','GCh 15F202 Min-range','','Test',504,'',1000 from dual union all
select 22162,'1970-S410','1970-06-27 17:27:00','suborbital rocket','success',920,'P39H','SpE fine scale','','','Ionospheric',1447,'',126 from dual union all
select 22163,'1970-S411','1970-06-30 01:39:59','military missile','failure',939,'C3XT-24','NF no sep','','','Test',246,'',500 from dual union all
select 22164,'1970-S412','1970-06-29 04:13:00','military missile','success',999,'5102743-264T','8K67 UBP-34','GCh 8F674','','OT',519,'',1000 from dual union all
select 22165,'1970-S413','1970-07-01 17:02:00','suborbital rocket','success',740,'MR-12','Na-Li/Te','','','Aeron/Ionos',636,'',149 from dual union all
select 22166,'1970-S414','1970-07-02 06:01:00','suborbital rocket','success',34,'NASA 04.299UG','','OAO Cal','','UV Astronomy',1869,'',168 from dual union all
select 22167,'1970-S415','1970-07-03 15:47:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',167 from dual union all
select 22168,'1970-S416','1970-07-03 19:51:00','suborbital rocket','success',1200,'S63/2','ESRO S63/2','','','Ionospheric',1343,'',150 from dual union all
select 22169,'1970-S417','1970-07-06 16:44:00','suborbital rocket','success',740,'MR-12','Na-Li/Te','','','Ionos',636,'',149 from dual union all
select 2217,'1954-A224','1954-04-30 00:00:00','military missile','failure',771,'GM-12482','','','','Test',222,'',12 from dual union all
select 22170,'1970-S418','1970-07-06 22:40:00','suborbital rocket','success',740,'MR-12','Na-LI/Te','','','Aeron',636,'',171 from dual union all
select 22171,'1970-S419','1970-07-06 23:43:00','suborbital rocket','success',740,'MR-12','TMA','','','Aeron',636,'',150 from dual union all
select 22172,'1970-S42','1970-07-08 04:02:00','military missile','success',1450,'Yu10761-015','15A20 LKI-19','GCh 15F202','','Test',532,'',1000 from dual union all
select 22173,'1970-S420','1970-07-07 01:48:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',169 from dual union all
select 22174,'1970-S421','1970-07-08 19:45:00','suborbital rocket','success',1200,'S63/1','ESRO S63/1','','','Ionospheric',1343,'',149 from dual union all
select 22175,'1970-S422','1970-07-08 23:48:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',140 from dual union all
select 22176,'1970-S423','1970-07-09 00:00:00','military missile','success',796,'','Mk 11 RV','OT GT66F','GLORY TRIP 66F','OT',1710,'',1300 from dual union all
select 22177,'1970-S424','1970-07-09 17:52:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',140 from dual union all
select 22178,'1970-S425','1970-07-10 00:30:00','suborbital rocket','success',1197,'SL727','UAT-6/MSSL I-33','','','XR Astronomy/Ionos RSRS',1839,'',203 from dual union all
select 22179,'1970-S426','1970-07-10 04:45:00','suborbital rocket','unknown',778,'','','','','Aeron',933,'',85 from dual union all
select 2218,'1954-A225','1954-04-30 00:00:00','military missile','failure',771,'GM-12480','','','','Test',222,'',12 from dual union all
select 22180,'1970-S427','1970-07-11 08:40:00','military missile','failure',135,'V123D','RVTO-2B-5','ABRES AS-5 (USAF)','','Reentry Test',612,'',200 from dual union all
select 22181,'1970-S428','1970-07-14 00:00:00','military missile','success',795,'','Mk 11 RV','FOT GT71B-2','GLORY TRIP 71B-2','OT',1705,'',1300 from dual union all
select 22182,'1970-S429','1970-07-14 02:47:00','suborbital rocket','success',675,'WRE K146','','','','Aeron',1832,'',85 from dual union all
select 22183,'1970-S43','1970-07-24 04:22:00','military missile','success',1450,'5115761-021','15A20 LKI-20','GCh 15B20','','Test',504,'',1000 from dual union all
select 22184,'1970-S430','1970-07-14 04:52:00','suborbital rocket','success',1210,'SL971','MSSL XR4 RMC','','','XR Astronomy',1839,'',223 from dual union all
select 22185,'1970-S431','1970-07-14 07:04:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',119,'VISE',171 from dual union all
select 22186,'1970-S432','1970-07-14 15:35:00','military missile','success',917,'DASO','P-182 70 Spring 09','','','OT',204,'',180 from dual union all
select 22187,'1970-S433','1970-07-14 16:00:00','military missile','success',917,'DASO','P-183 70 Spring 10','','','OT',204,'',180 from dual union all
select 22188,'1970-S434','1970-07-15 14:29:00','suborbital rocket','success',1207,'SL811','Cul FUV-1','','','UV Astronomy',1839,'',164 from dual union all
select 22189,'1970-S435','1970-07-16 16:00:00','suborbital rocket','success',839,'NASA 10.320UI','','','','Ionos',1821,'',143 from dual union all
select 2219,'1954-A226','1954-04-30 00:00:00','atmospheric rocket','success',836,'1397T','','','','Test',1895,'',10 from dual union all
select 22190,'1970-S436','1970-07-18 00:00:00','military missile','success',796,'','Mk 11 RV','OT GT30M','GLORY TRIP 30M','OT',1699,'',1300 from dual union all
select 22191,'1970-S437','1970-07-19 00:00:00','suborbital rocket','success',778,'8','','','','Test',933,'',90 from dual union all
select 22192,'1970-S438','1970-07-23 00:00:00','military missile','success',795,'','SSTTP','SSTTP M1-3','SSTTP M1-3','Target',1701,'',1300 from dual union all
select 22193,'1970-S439','1970-07-23 12:25:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',119,'VISE',167 from dual union all
select 22194,'1970-S44','1970-10-21 04:13:00','military missile','success',1450,'','15A20 LKI-26','GCh 15B20','','OT',505,'',1000 from dual union all
select 22195,'1970-S440','1970-07-23 13:55:00','suborbital rocket','success',1200,'ESRO S56A/2','LUX-50/RSRS SUV-2','','','Ionos/Solar UV',1343,'',137 from dual union all
select 22196,'1970-S441','1970-07-24 03:26:00','suborbital rocket','success',1200,'ESRO S65/1','RSRS SUV-3','','','Ionospheric',1343,'',125 from dual union all
select 22197,'1970-S442','1970-07-24 16:40:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',119,'VISE',180 from dual union all
select 22198,'1970-S443','1970-07-25 12:39:00','military missile','success',999,'4502742-259T','8K67 UBP-35','GCh 8F675','','OT',519,'',1000 from dual union all
select 22199,'1970-S444','1970-07-25 14:03:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',119,'VISE',150 from dual union all
select 222,'1944-S201','1944-10-14 03:24:00','military missile','success',1470,'18812','Ma232','','','Test',661,'',90 from dual union all
select 2220,'1954-A227','1954-05-03 00:00:00','military missile','unknown',339,'5T','','','','Test',1895,'',30 from dual union all
select 22200,'1970-S445','1970-07-26 13:01:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',119,'VISE',158 from dual union all
select 22201,'1970-S446','1970-07-26 17:20:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',119,'VISE',158 from dual union all
select 22202,'1970-S447','1970-07-26 19:49:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',119,'VISE',180 from dual union all
select 22203,'1970-S448','1970-12-18 00:00:00','military missile','success',1449,'','8K84 UBP-135M','','','OT',536,'',1000 from dual union all
select 22204,'1970-S449','1970-07-27 04:16:00','suborbital rocket','success',1200,'ESRO S65/2','RSRS SUV-4','','','Ionospheric',1343,'',128 from dual union all
select 22205,'1970-S45','1970-12-23 02:37:00','military missile','success',1450,'4925751-147T','15A20 K (Kontrol''niy)','GCh 15F202','','OT',504,'',1000 from dual union all
select 22206,'1970-S450','1970-07-27 04:37:00','suborbital rocket','success',30,'NASA 04.259GG','','','','UV Astronomy?',1869,'',200 from dual union all
select 22207,'1970-S451','1970-07-27 06:35:00','suborbital rocket','success',37,'CRL A04.004-1','MAP-II/LO STAR II','','','IR Astron',1895,'',125 from dual union all
select 22208,'1970-S452','1970-07-28 23:17:00','military missile','success',799,'FTM-5008','Mk 12 RV x 3','R&D','','Test',1697,'',1300 from dual union all
select 22209,'1970-S453','1970-12-22 00:00:00','military missile','success',1449,'','8K84 UBP-136M','','','OT',542,'',1000 from dual union all
select 2221,'1954-A228','1954-05-03 00:00:00','atmospheric rocket','success',836,'1774E','','','','Test',1895,'',10 from dual union all
select 22210,'1970-S454','1970-07-30 08:21:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',119,'VISE',160 from dual union all
select 22211,'1970-S455','1970-07-30 12:46:00','suborbital rocket','success',867,'NASA 18.72UE','','','','Plasma',1821,'',270 from dual union all
select 22212,'1970-S456','1970-07-30 14:26:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',119,'VISE',159 from dual union all
select 22213,'1970-S457','1970-08-10 17:19:00','suborbital rocket','success',846,'STV 95','','','','Reentry test',1866,'',209 from dual union all
select 22214,'1970-S458','1970-09-24 00:00:00','military missile','success',1449,'','8K84 UBP-126M','','','OT',542,'',1000 from dual union all
select 22215,'1970-S459','1970-09-30 00:00:00','military missile','success',1449,'','8K84 UBP-127M','','','OT',526,'',1000 from dual union all
select 22216,'1970-S46','1970-08-21 04:10:00','military missile','success',1450,'4925761-023','15A20 LKI-23','GCh 15B20','','Test',504,'',1000 from dual union all
select 22217,'1970-S460','1970-08-03 00:00:00','military missile','success',796,'','Mk 11 RV','OT GT61F','GLORY TRIP 61F','OT',1711,'',1300 from dual union all
select 22218,'1970-S461','1970-08-03 15:00:00','military missile','success',939,'C3P-1','PEM-1','','','OT',421,'SSBN 627',500 from dual union all
select 22219,'1970-S462','1970-08-03 16:09:00','suborbital rocket','success',837,'NASA 14.466UM','UM Pitot 48','','','Aeronomy',1821,'',170 from dual union all
select 2222,'1954-A229','1954-05-03 00:00:00','atmospheric rocket','success',836,'1583E','','','','Test',1895,'',10 from dual union all
select 22220,'1970-S463','1970-08-04 00:00:00','military missile','success',796,'','ERCS','ERCS GT16L','GLORY TRIP 16L','OT',1700,'',1300 from dual union all
select 22221,'1970-S464','1970-08-04 04:31:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',119,'VISE',165 from dual union all
select 22222,'1970-S465','1970-08-04 12:05:23','suborbital rocket','success',1250,'','Arcas VIII','','','Ionos',29,'',85 from dual union all
select 22223,'1970-S466','1970-08-04 17:18:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',119,'VISE',158 from dual union all
select 22224,'1970-S467','1970-08-04 21:01:00','military missile','success',917,'','P-184 70 Spring 11','','','OT',204,'',180 from dual union all
select 22225,'1970-S468','1970-08-04 21:25:00','military missile','success',917,'','P-185 70 Spring 12','','','OT',204,'',180 from dual union all
select 22226,'1970-S469','1970-08-05 08:20:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',119,'VISE',149 from dual union all
select 22227,'1970-S47','1970-08-22 02:15:00','military missile','success',1450,'5115761-024','15A20 LKI-24','GCh 15B20','','Test',505,'',1000 from dual union all
select 22228,'1970-S470','1970-08-05 16:30:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',119,'VISE',149 from dual union all
select 22229,'1970-S471','1970-08-07 06:30:00','military missile','success',999,'6702744-264T','8K67 UBP-36','GCh 8F674','','OT',519,'',1000 from dual union all
select 2223,'1954-A23','1954-01-15 00:00:00','atmospheric rocket','success',836,'1587E','','','','Test',1895,'',10 from dual union all
select 22230,'1970-S472','1970-08-08 00:48:10','suborbital rocket','success',837,'NASA 14.454UA','RTG SNA5/1 Pandora M','','','Aeronomy',413,'',128 from dual union all
select 22231,'1970-S473','1970-08-08 00:12:00','suborbital rocket','success',837,'NASA 14.462DA','RTG SNA6/1 NLC','','','Aeronomy',413,'',99 from dual union all
select 22232,'1970-S474','1970-08-08 02:54:00','suborbital rocket','success',837,'NASA 14.455UA','RTG SNA5/2 Pandora N','','','Aeronomy',413,'',133 from dual union all
select 22233,'1970-S475','1970-08-08 01:32:00','suborbital rocket','success',837,'NASA 14.463DA','RTG SNA6/2 NLC','','','Aeronomy',413,'',95 from dual union all
select 22234,'1970-S476','1970-08-09 23:39:40','suborbital rocket','success',290,'C58/2','ESRO C58/2','','','Ionos/Meteorites',408,'',114 from dual union all
select 22235,'1970-S477','1970-08-10 00:50:00','suborbital rocket','success',290,'C51/2','ESRO C51/2','','','Aeronomy',408,'',154 from dual union all
select 22236,'1970-S478','1970-08-10 06:12:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',119,'VISE',158 from dual union all
select 22237,'1970-S479','1970-08-10 16:37:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',119,'VISE',150 from dual;
insert into LAUNCH
select 22238,'1970-S48','1970-10-15 03:32:00','military missile','success',1450,'','15A20 LKI-25','GCh 15B20','','OT',504,'',1000 from dual union all
select 22239,'1970-S480','1970-08-11 23:00:00','military missile','success',796,'','Mk 11 RV','OT GT06F','GLORY TRIP 06F','OT',1712,'',1300 from dual union all
select 2224,'1954-A230','1954-05-04 00:00:00','military missile','unknown',339,'1326E','','','','Test',1895,'',30 from dual union all
select 22240,'1970-S481','1970-08-12 18:26:00','suborbital rocket','success',30,'CRL A03.903-2','EUV','','','Solar UV',1869,'',222 from dual union all
select 22241,'1970-S482','1970-08-12 18:52:00','suborbital rocket','success',37,'CRL A04.002-1','EUV','','','Solar EUV',1895,'',246 from dual union all
select 22242,'1970-S483','1970-08-13 16:31:00','suborbital rocket','success',44,'NASA 17.06UE','ECHO I','','','Plasma',1821,'',350 from dual union all
select 22243,'1970-S484','1970-08-13 19:02:00','suborbital rocket','success',30,'NASA 04.200DS','NRL NE3.247  (S082)','','','Solar EUV',1869,'',186 from dual union all
select 22244,'1970-S485','1970-08-14 08:35:00','military missile','success',796,'','Mk 11 RV','OT GT31M','GLORY TRIP 31M','OT',1699,'',1300 from dual union all
select 22245,'1970-S486','1970-08-16 03:56:00','suborbital rocket','success',227,'AAD-VB-31','E fields ,electrons','','','Ionos',428,'',260 from dual union all
select 22246,'1970-S487','1970-08-17 16:16:00','military missile','success',939,'C3P-2','PEM-2','','','OT',421,'SSBN 627',500 from dual union all
select 22247,'1970-S488','1970-08-18 03:45:00','test rocket','success',949,'246-01','82-EV','GVM 7K-L1E','','Test',587,'',200 from dual union all
select 22248,'1970-S489','1970-08-19 11:03:00','suborbital rocket','success',290,'C48/2','ESRO C48/2','','','Solar UV',408,'',163 from dual union all
select 22249,'1970-S49','1970-01-02 06:42:00','suborbital rocket','success',837,'NASA 14.240II','ISRO 30.01','TERLS-142','','Ionos/RF Astron',1467,'',125 from dual union all
select 2225,'1954-A231','1954-05-04 00:00:00','atmospheric rocket','success',836,'1400T','','','','Test',1895,'',10 from dual union all
select 22250,'1970-S490','1970-08-21 06:25:00','military missile','success',795,'','Mk 11 RV','FOT GT74B','GLORY TRIP 74B','OT',1705,'',1300 from dual union all
select 22251,'1970-S491','1970-08-22 04:58:00','suborbital rocket','success',37,'NASA 13.17DG','','','','UV Astron?',1895,'',200 from dual union all
select 22252,'1970-S492','1970-08-26 08:05:00','military missile','success',796,'','Mk 11 RV','OT GT43M','GLORY TRIP 43M','OT',1700,'',1300 from dual union all
select 22253,'1970-S493','1970-08-27 06:00:00','military missile','success',799,'','Mk 12 RV x 3','DASO OLD FOX 01M','OLD FOX 01M','OT',1703,'',1300 from dual union all
select 22254,'1970-S494','1970-08-29 03:15:00','military missile','success',795,'','SSTTP','SSTTP M1-4','','Target',1701,'',1300 from dual union all
select 22255,'1970-S495','1970-08-29 04:00:00','suborbital rocket','success',846,'STV 96','','','','Reentry test',1865,'',176 from dual union all
select 22256,'1970-S496','1970-08-28 00:00:00','military missile','success',1251,'','SAFEGUARD M1-4','','','Interceptor',868,'',100 from dual union all
select 22257,'1970-S497','1970-08-28 04:43:00','suborbital rocket','success',837,'NASA 14.414UG','','','','XR Astron',1895,'',200 from dual union all
select 22258,'1970-S498','1970-08-31 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 618',1000 from dual union all
select 22259,'1970-S499','1970-08-31 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 618',1000 from dual union all
select 2226,'1954-A232','1954-05-05 00:00:00','atmospheric rocket','unknown',576,'211M','','','','Test',1895,'',10 from dual union all
select 22260,'1970-S50','1970-01-05 00:00:00','suborbital rocket','success',839,'NASA 10.329GM','GSFC Grenade 222 W','','','Aeronomy',1821,'',100 from dual union all
select 22261,'1970-S500','1970-08-31 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 618',1000 from dual union all
select 22262,'1970-S501','1970-09-01 20:10:00','suborbital rocket','success',624,'NASA 8.52UA','','','','Aeronomy',1821,'',800 from dual union all
select 22263,'1970-S502','1970-09-03 02:05:00','military missile','success',795,'','Mk 11 RV','FOT GT75B','GLORY TRIP 75B','OT',1698,'',1300 from dual union all
select 22264,'1970-S503','1970-09-02 10:14:00','suborbital rocket','success',665,'K-8-16','Na/ne','','','Ionos/Release',839,'',174 from dual union all
select 22265,'1970-S504','1970-09-08 05:07:00','suborbital rocket','success',920,'P30H','D/E regions','','','Ionospheric',1447,'',148 from dual union all
select 22266,'1970-S505','1970-09-08 16:00:00','military missile','success',939,'C3P-3','PEM-3','','','OT',421,'SSBN 629',500 from dual union all
select 22267,'1970-S506','1970-09-08 19:01:00','suborbital rocket','success',920,'P32H','D/E regions','','','Ionospheric',1447,'',149 from dual union all
select 22268,'1970-S507','1970-09-09 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 655',1000 from dual union all
select 22269,'1970-S508','1970-09-09 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 655',1000 from dual union all
select 2227,'1954-A233','1954-05-05 00:00:00','atmospheric rocket','unknown',576,'207M','','','','Test',1895,'',10 from dual union all
select 22270,'1970-S509','1970-09-09 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 655',1000 from dual union all
select 22271,'1970-S51','1970-01-05 21:30:00','suborbital rocket','success',839,'NASA 10.325GM','GSFC Grenade 223 C','','','Aeronomy',428,'',95 from dual union all
select 22272,'1970-S510','1970-09-09 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 655',1000 from dual union all
select 22273,'1970-S511','1970-09-09 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 655',1000 from dual union all
select 22274,'1970-S512','1970-09-09 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 655',1000 from dual union all
select 22275,'1970-S513','1970-09-09 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 655',1000 from dual union all
select 22276,'1970-S514','1970-09-09 06:40:00','suborbital rocket','success',694,'LS-C-4','','','','Test',1476,'',45 from dual union all
select 22277,'1970-S515','1970-09-11 05:59:00','suborbital rocket','success',920,'P31H','D/E regions','','','Ionospheric',1447,'',140 from dual union all
select 22278,'1970-S516','1970-09-12 11:30:00','military missile','success',999,'N 22503-05','8K67 UBP-37','GCh 8F675 LKI OS-67','','Test',515,'',1000 from dual union all
select 22279,'1970-S517','1970-09-13 00:00:00','suborbital rocket','success',656,'K-10C-3','TVC test','','','Test',839,'',160 from dual union all
select 2228,'1954-A234','1954-05-05 00:00:00','atmospheric rocket','success',836,'2199R  Bell motor','Nike 280R','','','Test',1895,'',10 from dual union all
select 22280,'1970-S518','1970-09-14 23:30:00','military missile','success',796,'','Mk 11 RV','OT GT67F','GLORY TRIP 67F','OT',1711,'',1300 from dual union all
select 22281,'1970-S519','1970-09-15 00:00:00','military missile','success',916,'','P-186 70 Fall 01','','','Test',204,'',180 from dual union all
select 22282,'1970-S52','1970-01-10 00:00:00','suborbital rocket','success',839,'NASA 10.330GM','GSFC Grenade 224 W','','','Aeronomy',1821,'',103 from dual union all
select 22283,'1970-S520','1970-09-16 19:30:00','military missile','success',799,'STM-1','Mk 12 RV x 3','R&D','','Test',266,'',1300 from dual union all
select 22284,'1970-S521','1970-09-17 18:00:00','military missile','success',796,'','Mk 11 RV','OT GT68F','GLORY TRIP 68F','OT',1712,'',1300 from dual union all
select 22285,'1970-S522','1970-09-17 15:42:00','suborbital rocket','success',839,'NASA 10.324GM','GSFC Grenade 236 W','','','Aeronomy',1821,'',127 from dual union all
select 22286,'1970-S523','1970-09-17 15:58:00','suborbital rocket','success',837,'NASA 14.385UM','UM Pitot 50','','','Aeronomy',1821,'',156 from dual union all
select 22287,'1970-S524','1970-09-17 20:00:00','military missile','success',916,'','P-187 70 Fall 02','','','Test',204,'',180 from dual union all
select 22288,'1970-S525','1970-09-18 11:39:00','suborbital rocket','success',920,'P10H','Winter anom/Strato','','','Ionospheric',1447,'',144 from dual union all
select 22289,'1970-S526','1970-09-19 11:30:00','sounding rocket','success',698,'L-3H-5','','','','XR Astron/UV Astron',834,'',2017 from dual union all
select 2229,'1954-A235','1954-05-05 00:00:00','atmospheric rocket','success',836,'1325R','Nike 279R','','','Test',1895,'',10 from dual union all
select 22290,'1970-S527','1970-09-21 15:00:00','military missile','success',939,'C3P-4','PEM-4','','','OT',421,'SSBN 629',500 from dual union all
select 22291,'1970-S528','1970-09-21 16:14:00','suborbital rocket','success',839,'NASA 10.327UM','UM Pitot 51','','','Aeronomy',1821,'',145 from dual union all
select 22292,'1970-S529','1970-09-22 04:00:00','suborbital rocket','failure',837,'NASA 14.464UA','','','','Aeronomy',1895,'',10 from dual union all
select 22293,'1970-S53','1970-01-10 00:00:00','suborbital rocket','success',839,'NASA 10.337GM','GSFC Grenade 225 PB','','','Aeronomy',1039,'',103 from dual union all
select 22294,'1970-S530','1970-09-22 04:30:00','suborbital rocket','success',37,'NASA 13.07GG','','','','XR Astron',1895,'',146 from dual union all
select 22295,'1970-S531','1970-09-24 00:00:00','military missile','success',1103,'C5','S02C5','','','Test',301,'',1000 from dual union all
select 22296,'1970-S532','1970-09-24 12:54:01','ballistic missile test','success',1373,'271','HAP/SXRE','','','XR Astron/Test',773,'',500 from dual union all
select 22297,'1970-S533','1970-09-25 00:00:00','military missile','success',795,'','Mk 11 RV','OT GT76B','GLORY TRIP 76B','OT',1702,'',1300 from dual union all
select 22298,'1970-S534','1970-09-26 07:20:00','military missile','success',799,'','Mk 12 RV x 3','DASO OLD FOX 02M','OLD FOX 02M','OT',1697,'',1300 from dual union all
select 22299,'1970-S535','1970-09-25 03:10:00','suborbital rocket','success',219,'NASA 19.02NE','','','','Plasma',330,'',600 from dual union all
select 223,'1944-S202','1944-10-14 04:12:00','military missile','success',1470,'18819','Ma237','','','Test',661,'',90 from dual union all
select 2230,'1954-A236','1954-05-05 00:00:00','atmospheric rocket','success',836,'1297R','Nike 277R','','','Test',1895,'',10 from dual union all
select 22300,'1970-S536','1970-09-27 06:42:00','suborbital rocket','success',668,'K-9M-32','B field/Te probe','','','Ionos',839,'',346 from dual union all
select 22301,'1970-S537','1970-09-28 01:20:00','suborbital rocket','success',1070,'S-210-5','','','','Ionos',839,'',108 from dual union all
select 22302,'1970-S538','1970-09-28 09:01:00','suborbital rocket','success',37,'NASA 13.26CG','','','','XR Astron',1895,'',200 from dual union all
select 22303,'1970-S539','1970-09-29 03:00:00','suborbital rocket','success',219,'NASA 19.03NE','','','','Plasma',330,'',600 from dual union all
select 22304,'1970-S54','1970-01-12 11:33:00','suborbital rocket','success',1189,'INTA MP-7001','','','','Ionos',396,'',88 from dual union all
select 22305,'1970-S540','1970-09-29 19:15:00','military missile','success',916,'','P-188 70 Fall 03','','','OT',204,'',180 from dual union all
select 22306,'1970-S541','1970-09-29 19:36:00','military missile','success',916,'','P-189 70 Fall 04','','','OT',204,'',180 from dual union all
select 22307,'1970-S542','1970-09-29 21:10:00','military missile','success',916,'','P-190 70 Fall 05','','','OT',204,'',180 from dual union all
select 22308,'1970-S543','1970-09-29 21:34:00','military missile','success',916,'OffR 117','P-191 70 Fall 06','','','OT WS-117',204,'',180 from dual union all
select 22309,'1970-S544','1970-10-01 00:00:00','suborbital rocket','success',1116,'Sandia 152-134','Dualhawk-9 Test','','','Test',855,'',515 from dual union all
select 2231,'1954-A237','1954-05-05 00:00:00','atmospheric rocket','success',836,'187R','Nike 278R','','','Test',1895,'',10 from dual union all
select 22310,'1970-S545','1970-09-30 20:06:29','test rocket','success',1128,'S171C','RAM C-3','','','Reentry Test',1814,'',269 from dual union all
select 22311,'1970-S546','1970-10-01 13:46:12','suborbital rocket','success',837,'N-NA-30','DLR N-NA-30','','','Ionos/Plasma?',330,'',200 from dual union all
select 22312,'1970-S547','1970-10-03 00:00:00','military missile','success',795,'','SSTTP','SSTTP M1-27','','Target',1701,'',1300 from dual union all
select 22313,'1970-S548','1970-10-03 00:00:00','military missile','success',796,'','Mk 11 RV','OT GT46M','GLORY TRIP 46M','OT',1699,'',1300 from dual union all
select 22314,'1970-S549','1970-10-03 00:00:00','military missile','success',1251,'','SAFEGUARD M1-27','','','Interceptor',868,'',100 from dual union all
select 22315,'1970-S55','1970-01-13 05:32:00','suborbital rocket','success',864,'Sandia 152-137','LRL ROX-3','','','XR Astron',1490,'',180 from dual union all
select 22316,'1970-S550','1970-10-03 04:20:00','suborbital rocket','success',1014,'','R-5V Geophysical','RAO K-2 No. 2','','Solar X',636,'',500 from dual union all
select 22317,'1970-S551','1970-10-05 23:00:00','military missile','success',795,'','Mk 11 RV','FOT GT77B','GLORY TRIP 77B','OT',1705,'',1300 from dual union all
select 22318,'1970-S552','1970-10-05 09:05:00','suborbital rocket','success',624,'NASA 8.58IE','MPE Ba','','','Plasma',1821,'',900 from dual union all
select 22319,'1970-S553','1970-10-06 10:50:00','suborbital rocket','success',448,'D-305','EIDI-1','','','Ionos',307,'',480 from dual union all
select 2232,'1954-A238','1954-05-05 00:00:00','atmospheric rocket','success',836,'1656E','','','','Test',1895,'',10 from dual union all
select 22320,'1970-S554','1970-10-06 13:36:25','suborbital rocket','success',837,'N-NA-31','DLR N-NA-31','','','Ionos/Plasma?',330,'',200 from dual union all
select 22321,'1970-S555','1970-10-06 15:15:00','military missile','success',939,'C3P-5','PEM-5','','','OT',421,'SSBN 629',500 from dual union all
select 22322,'1970-S556','1970-10-07 11:15:00','military missile','success',796,'','Mk 11 RV','OT GT47M','GLORY TRIP 47M','OT',1700,'',1300 from dual union all
select 22323,'1970-S557','1970-10-07 09:48:00','suborbital rocket','success',867,'NASA 18.54NE','Ba release test','','','Plasma',1821,'',270 from dual union all
select 22324,'1970-S558','1970-10-07 10:55:00','suborbital rocket','unknown',778,'','','','','Aeron',933,'',89 from dual union all
select 22325,'1970-S559','1970-10-08 13:30:00','military missile','success',999,'6305761-025','8K67 UBP-38','Pal''ma-4','','OT',515,'',1000 from dual union all
select 22326,'1970-S56','1970-01-13 22:23:00','suborbital rocket','success',839,'NASA 10.326GM','GSFC Grenade 226 C','','','Aeronomy',428,'',126 from dual union all
select 22327,'1970-S560','1970-10-08 08:20:00','suborbital rocket','success',1210,'SL972','LUX XRS9','','','XR Astronomy',1839,'',232 from dual;
insert into LAUNCH
select 22328,'1970-S561','1970-10-08 13:29:19','suborbital rocket','success',837,'N-NA-32','DLR N-NA-32','','','Ionos/Plasma?',330,'',200 from dual union all
select 22329,'1970-S562','1970-10-08 14:42:38','suborbital rocket','success',837,'N-NA-33','DLR N-NA-33','','','Ionos/Plasma?',330,'',200 from dual union all
select 2233,'1954-A239','1954-05-05 00:00:00','atmospheric rocket','success',836,'1503E','','','','Test',1895,'',10 from dual union all
select 22330,'1970-S563','1970-10-09 17:57:00','suborbital rocket','success',277,'','SPED II','','','Reentry Test',1816,'',85 from dual union all
select 22331,'1970-S564','1970-10-12 14:58:00','suborbital rocket','success',837,'N-NA-34','DLR N-NA-34','','','Ionos/Plasma?',330,'',200 from dual union all
select 22332,'1970-S565','1970-10-13 00:08:07','suborbital rocket','success',1206,'SN5/2','DLR S-CR-22','','','Aeron',1343,'',288 from dual union all
select 22333,'1970-S566','1970-10-14 03:05:00','military missile','success',916,'AO-95','P-192 70 Fall 07','','','Test',204,'',180 from dual union all
select 22334,'1970-S567','1970-10-14 08:45:00','suborbital rocket','success',1206,'SL1021','MSSL XR5 RMC','','','XR Astronomy',1839,'',270 from dual union all
select 22335,'1970-S568','1970-10-14 16:41:00','suborbital rocket','success',839,'NASA 10.276AM','','','','Aeronomy',1821,'',116 from dual union all
select 22336,'1970-S569','1970-10-16 00:30:00','suborbital rocket','failure',1207,'ESRO S53','ESRO S53','','','Solar X',1839,'',209 from dual union all
select 22337,'1970-S57','1970-01-13 23:50:50','suborbital rocket','success',837,'NASA 14.384UM','UM Pitot 42','','','Aeronomy',428,'',171 from dual union all
select 22338,'1970-S570','1970-10-19 00:00:00','suborbital rocket','failure',1207,'ESRO S58','Cul UVS 17','','','Solar UV?',1839,'',200 from dual union all
select 22339,'1970-S571','1970-10-20 13:05:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',636,'',150 from dual union all
select 2234,'1954-A24','1954-01-18 00:00:00','atmospheric rocket','success',836,'1723E','','','','Test',1895,'',10 from dual union all
select 22340,'1970-S572','1970-10-21 09:39:00','suborbital rocket','success',331,'WRE Cockatoo 106','Sphere','','','Aeron',1835,'',100 from dual union all
select 22341,'1970-S573','1970-10-21 13:25:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',636,'',176 from dual union all
select 22342,'1970-S574','1970-10-21 14:30:00','suborbital rocket','unknown',740,'','','','','Plasma',636,'',150 from dual union all
select 22343,'1970-S575','1970-10-21 15:03:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos/Plasma',636,'',168 from dual union all
select 22344,'1970-S576','1970-10-22 22:35:00','military missile','success',796,'','Mk 11 RV','OT GT69F (Salvo)','GLORY TRIP 69F','OT',1711,'',1300 from dual union all
select 22345,'1970-S577','1970-10-22 22:35:00','military missile','success',796,'','Mk 11 RV','OT GT70F (Salvo)','GLORY TRIP 70F','OT',1712,'',1300 from dual union all
select 22346,'1970-S578','1970-10-22 08:23:00','suborbital rocket','success',448,'D-306','EIDI-2','','','Ionos',307,'',480 from dual union all
select 22347,'1970-S579','1970-10-22 14:24:26','suborbital rocket','success',837,'N-NA-35','DLR N-NA-35','','','Ionos/Plasma?',330,'',200 from dual union all
select 22348,'1970-S58','1970-01-14 13:47:00','suborbital rocket','success',837,'NASA 14.452GM','GSFC Grenade NA3','','','Aeronomy',1821,'',200 from dual union all
select 22349,'1970-S580','1970-10-23 02:10:00','military missile','success',1002,'','8K67P UBP-15','','','Test',519,'',1000 from dual union all
select 2235,'1954-A240','1954-05-05 00:00:00','atmospheric rocket','success',836,'1338T','','','','Test',1895,'',10 from dual union all
select 22350,'1970-S581','1970-10-23 22:00:00','military missile','success',939,'C3E-5','USN Mk 3 RV x 10','DASO-1','DASO-1','OT',421,'SSBN 627',500 from dual union all
select 22351,'1970-S582','1970-10-26 00:00:00','military missile','success',795,'','Mk 11 RV','FOT GT78B','GLORY TRIP 78B','OT',1698,'',1300 from dual union all
select 22352,'1970-S583','1970-10-27 20:34:22','suborbital rocket','success',1204,'ESRO S67/1','ESRO S67/1 RSRS-15','','','Auroral',416,'',229 from dual union all
select 22353,'1970-S584','1970-10-28 16:23:00','suborbital rocket','success',30,'NASA 04.131UA','Structure','','','Aeronomy',1869,'',200 from dual union all
select 22354,'1970-S585','1970-10-29 16:25:00','military missile','success',999,'','8K67 UBP-40','','','OT',515,'',1000 from dual union all
select 22355,'1970-S586','1970-10-29 21:39:30','suborbital rocket','success',1204,'ESRO S67/2','ESRO S67/2 RSRS-16','','','Auroral',416,'',234 from dual union all
select 22356,'1970-S587','1970-08-07 00:00:00','military missile','success',1044,'Kh64923-9L','8K98P LKI-7','','','Test',477,'',1000 from dual union all
select 22357,'1970-S588','1970-10-12 00:00:00','military missile','success',1044,'52627-432-12L','8K98P LKI-9','','','Test',477,'',1000 from dual union all
select 22358,'1970-S589','1970-10-31 10:32:00','suborbital rocket','success',37,'NASA 13.41GG-UG','UVT-SDGS 3','','','UV Astron',1895,'',200 from dual union all
select 22359,'1970-S59','1970-01-14 17:55:00','suborbital rocket','success',839,'NASA 10.323GM','GSFC Grenade 228 W','','','Aeronomy',1821,'',100 from dual union all
select 2236,'1954-A241','1954-05-05 00:00:00','atmospheric rocket','success',836,'1412T','','','','Test',1895,'',10 from dual union all
select 22360,'1970-S590','1970-11-02 23:05:00','military missile','success',796,'','Mk 11 RV','OT GT51M','GLORY TRIP 51M','OT',1699,'',1300 from dual union all
select 22361,'1970-S591','1970-11-04 09:05:00','military missile','success',799,'STM-1W','Mk 12 RV x 3','R&D STM-1W','STM-1W','Test',1707,'',1300 from dual union all
select 22362,'1970-S592','1970-11-06 02:00:00','military missile','success',796,'','Mk 11 RV','OT GT52M','GLORY TRIP 52M','OT',1700,'',1300 from dual union all
select 22363,'1970-S593','1970-11-05 02:20:00','military missile','success',999,'','8K67 UBP-41','','','Test',519,'',1000 from dual union all
select 22364,'1970-S594','1970-11-05 03:35:00','military missile','success',916,'','P-194 70 Fall 09','','','OT',204,'',180 from dual union all
select 22365,'1970-S595','1970-11-05 04:05:00','military missile','success',916,'','P-195 70 Fall 10','','','OT',204,'',180 from dual union all
select 22366,'1970-S596','1970-11-05 07:35:00','military missile','success',916,'','P-197 70 Fall 12','','','OT',204,'',180 from dual union all
select 22367,'1970-S597','1970-11-05 19:10:00','suborbital rocket','success',331,'WRE Cockatoo 108','Sphere FC11/2','','','Aeron',1835,'',100 from dual union all
select 22368,'1970-S598','1970-11-08 07:50:00','military missile','success',795,'','Mk 11 RV','FOT GT79B','GLORY TRIP 79B','OT',1705,'',1300 from dual union all
select 22369,'1970-S599','1970-11-10 00:00:00','military missile','success',1103,'C6','S02C6','','','Test',301,'',1000 from dual union all
select 2237,'1954-A242','1954-05-05 17:28:00','military missile','failure',1053,'RS-3     XC','','','','Test',272,'',0 from dual union all
select 22370,'1970-S60','1970-01-14 17:23:00','suborbital rocket','success',837,'NASA 14.453GM','GSFC Grenade NA4','','','Aeronomy',1821,'',200 from dual union all
select 22371,'1970-S600','1970-11-10 11:09:00','suborbital rocket','success',867,'NASA 18.92UE','','','','Plasma',1047,'',270 from dual union all
select 22372,'1970-S601','1970-11-11 04:55:00','suborbital rocket','success',1207,'SL905','MSSL XR6','','','XR Astronomy',1839,'',183 from dual union all
select 22373,'1970-S602','1970-11-12 11:00:00','suborbital rocket','success',331,'WRE Cockatoo 105','Lunar UV Ozone','','','Aeron',1835,'',104 from dual union all
select 22374,'1970-S603','1970-11-12 12:30:00','suborbital rocket','success',920,'P21H','Winter anom/Strato','','','Ionospheric',1447,'',85 from dual union all
select 22375,'1970-S604','1970-11-14 00:00:00','military missile','success',799,'','Mk 12 RV x 3','DASO OLD FOX 03M','OLD FOX 03M','OT',1703,'',1300 from dual union all
select 22376,'1970-S605','1970-11-13 21:27:00','suborbital rocket','success',1189,'AMM-S2-9','D-region O profile','','','Aeron',428,'',98 from dual union all
select 22377,'1970-S606','1970-11-13 22:19:41','suborbital rocket','success',867,'NASA 18.115IE','Ferdinand 23','','','Auroral',29,'',214 from dual union all
select 22378,'1970-S607','1970-11-16 20:00:00','suborbital rocket','success',920,'P37H','Particles','','','Ionospheric',1447,'',140 from dual union all
select 22379,'1970-S608','1970-11-18 12:30:00','suborbital rocket','success',867,'NASA 18.93UE','','','','Plasma',1047,'',270 from dual union all
select 2238,'1954-A243','1954-05-07 00:00:00','atmospheric rocket','success',836,'1386T','','','','Test',1895,'',10 from dual union all
select 22380,'1970-S609','1970-11-19 05:15:00','suborbital rocket','success',920,'P47H','LP/VLF 1','','','Ionospheric',1447,'',129 from dual union all
select 22381,'1970-S61','1970-01-14 01:10:00','suborbital rocket','success',839,'NASA 10.343GM','GCA Release TMA','','','Aeronomy',1821,'',160 from dual union all
select 22382,'1970-S610','1970-11-19 22:00:00','suborbital rocket','success',1207,'SL904','LUX XRS10','','','XR Astronomy',1839,'',209 from dual union all
select 22383,'1970-S611','1970-11-20 19:00:00','suborbital rocket','failure',837,'NASA 14.458UA','','','','Aeronomy',1895,'',10 from dual union all
select 22384,'1970-S612','1970-11-20 23:19:00','suborbital rocket','success',848,'CRL A07.901-6','ALADDIN Release','','','Aeron',394,'',150 from dual union all
select 22385,'1970-S613','1970-11-20 23:19:00','suborbital rocket','success',848,'CRL A07.917-1','ALADDIN Release','','','Aeron',394,'',170 from dual union all
select 22386,'1970-S614','1970-11-20 23:23:00','suborbital rocket','success',848,'CRL AT07.895','ALADDIN NACS Mass sp','','','Aeron',394,'',121 from dual union all
select 22387,'1970-S615','1970-11-20 23:23:00','suborbital rocket','success',848,'CRL AT07.896','ALADDIN Mass sp','','','Ionos',394,'',130 from dual union all
select 22388,'1970-S616','1970-11-20 23:26:00','suborbital rocket','success',848,'CRL A07.917-3','ALADDIN Release','','','Aeron',394,'',166 from dual union all
select 22389,'1970-S617','1970-11-20 23:26:00','suborbital rocket','success',848,'CRL A07.917-2','ALADDIN Release','','','Aeron',394,'',158 from dual union all
select 2239,'1954-A244','1954-05-07 00:00:00','atmospheric rocket','success',836,'2101E','','','','Test',1895,'',10 from dual union all
select 22390,'1970-S618','1970-11-20 23:29:00','suborbital rocket','success',837,'NASA 14.460UM','UM Pitot 52','','','Aeronomy',394,'',181 from dual union all
select 22391,'1970-S619','1970-11-24 23:56:00','suborbital rocket','success',1440,'CRL AD21.862','AFCRL TB2-3','','','Reentry Test',1821,'',321 from dual union all
select 22392,'1970-S62','1970-01-14 01:35:00','suborbital rocket','success',839,'NASA 10.333GM','GSFC Grenade 227 C','','','Aeronomy',428,'',113 from dual union all
select 22393,'1970-S620','1970-11-23 20:30:00','military missile','success',916,'','P-198 70 Fall 13','','','OT',204,'',180 from dual union all
select 22394,'1970-S621','1970-11-23 23:10:00','military missile','success',916,'','P-199 70 Fall 14','','','OT',204,'',180 from dual union all
select 22395,'1970-S622','1970-11-23 23:45:00','military missile','success',916,'','P-200 70 Fall 15','','','OT',204,'',180 from dual union all
select 22396,'1970-S623','1970-11-24 00:20:00','military missile','success',916,'','P-201 70 Fall 16','','','OT OR-127/WS-139',204,'',180 from dual union all
select 22397,'1970-S624','1970-11-24 20:45:00','suborbital rocket','success',30,'NASA 04.285CS','','','','Solar',1869,'',200 from dual union all
select 22398,'1970-S625','1970-11-24 22:13:00','suborbital rocket','success',1207,'SL804','LUX BCS 3','','','Solar X',1839,'',182 from dual union all
select 22399,'1970-S626','1970-11-25 03:17:00','ballistic missile test','success',249,'NRL NB22.208','UV Telescope','','','UV Astron',1821,'',1480 from dual union all
select 224,'1944-S203','1944-10-14 05:11:00','military missile','success',1470,'18817','Ma235','','','Test',661,'',90 from dual union all
select 2240,'1954-A245','1954-05-07 00:00:00','atmospheric rocket','success',836,'1541E','','','','Test',1895,'',10 from dual union all
select 22400,'1970-S627','1970-11-27 20:15:00','suborbital rocket','success',740,'MR-12','','','','Aeron',1010,'',160 from dual union all
select 22401,'1970-S628','1970-11-27 22:17:00','military missile','success',939,'C3E-10','DASO-2','','','OT',421,'SSBN 629',500 from dual union all
select 22402,'1970-S629','1970-11-28 05:30:00','suborbital rocket','success',1014,'','Vertikal''-1','VZAF','','Aeron/Ionos/Solar UV/X',636,'',468 from dual union all
select 22403,'1970-S63','1970-01-14 03:00:00','suborbital rocket','success',837,'NASA 14.447CM','GCA Release TMA','','','Aeronomy',1821,'',197 from dual union all
select 22404,'1970-S631','1970-12-02 07:30:00','suborbital rocket','success',30,'NASA 04.267UG','','','','UV Astronomy',1869,'',200 from dual union all
select 22405,'1970-S632','1970-12-02 08:32:00','suborbital rocket','success',37,'CRL A04.004-3','IPRCE-1/HI STAR Test','','','IR Astron',1895,'',190 from dual union all
select 22406,'1970-S633','1970-12-02 18:02:00','military missile','success',799,'STM-3','Mk 12 RV x 3','R&D','','Test',266,'',1300 from dual union all
select 22407,'1970-S634','1970-12-06 11:13:00','suborbital rocket','success',1206,'ESRO S69','LUX BCS 4','','','Solar X',1343,'',187 from dual union all
select 22408,'1970-S635','1970-12-07 09:50:00','military missile','success',796,'','Mk 11C RV','OT GT56M','GLORY TRIP 56M','OT',1699,'',1300 from dual union all
select 22409,'1970-S636','1970-12-07 21:34:00','suborbital rocket','success',219,'AKF-4A-19','Twilight airglow','','','Aeron/XR Astron?',330,'',770 from dual union all
select 2241,'1954-A246','1954-05-07 00:00:00','atmospheric rocket','success',836,'1354T','','','','Test',1895,'',10 from dual union all
select 22410,'1970-S637','1970-12-09 12:30:00','suborbital rocket','success',1189,'INTA EMP-7001','','','','Ionos',396,'',87 from dual union all
select 22411,'1970-S638','1970-12-10 16:27:00','suborbital rocket','success',867,'DLR W-NT-36','Biosonde 1','','','Bio',1821,'',257 from dual union all
select 22412,'1970-S639','1970-11-11 17:53:00','military missile','success',999,'4502745-253T','8K67 UBP-42','GCh 8F675 Glavkom RVSN','','OT',518,'',1000 from dual union all
select 22413,'1970-S64','1970-01-14 04:05:30','suborbital rocket','success',867,'NASA 18.64UE','UMd Auroral','','','Auroral',428,'',259 from dual union all
select 22414,'1970-S640','1970-12-11 15:37:00','suborbital rocket','success',1189,'INTA EMP-7002','','','','Ionos',396,'',80 from dual union all
select 22415,'1970-S641','1970-12-12 02:50:00','military missile','success',999,'N 22506-01','8K67 UBP-43','GCh 8F675 Glavkom RVSN','','OT',515,'',1000 from dual union all
select 22416,'1970-S642','1970-12-12 11:40:00','suborbital rocket','success',1189,'INTA EMP-7003','','','','Ionos',396,'',80 from dual union all
select 22417,'1970-S643','1970-12-14 10:02:00','suborbital rocket','success',227,'AAD-VB-30','Geminids','','','Ionos/Meteorites',428,'',340 from dual;
insert into LAUNCH
select 22418,'1970-S644','1970-12-14 15:30:38','military missile','success',799,'STM-2','Mk 12 RV x 3','R&D','','Test',266,'',1300 from dual union all
select 22419,'1970-S645','1970-12-15 03:08:00','suborbital rocket','success',837,'NASA 14.470CM','GCA Release TMA','','','Aeronomy',1821,'',208 from dual union all
select 2242,'1954-A247','1954-05-08 00:00:00','atmospheric rocket','success',836,'1437T','','','','Test',1895,'',10 from dual union all
select 22420,'1970-S646','1970-12-15 05:18:00','suborbital rocket','success',837,'NASA 14.471CM','GCA Release TMA','','','Aeronomy',1821,'',214 from dual union all
select 22421,'1970-S647','1970-12-15 07:00:00','suborbital rocket','success',837,'NASA 14.472CM','GCA Release TMA','','','Aeronomy',1821,'',213 from dual union all
select 22422,'1970-S648','1970-12-15 09:04:00','suborbital rocket','success',837,'NASA 14.473CM','GCA Release TMA','','','Aeronomy',1821,'',195 from dual union all
select 22423,'1970-S649','1970-12-15 11:27:00','suborbital rocket','success',837,'NASA 14.474CM','GCA Release Na-Li','','','Aeronomy',1821,'',215 from dual union all
select 22424,'1970-S65','1970-01-14 04:41:00','suborbital rocket','success',227,'AAF-VB-29','Auroral low elev.','','','Ionos/Plasma/Meteor',428,'',128 from dual union all
select 22425,'1970-S650','1970-12-16 09:03:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',165 from dual union all
select 22426,'1970-S651','1970-12-16 16:39:00','suborbital rocket','success',867,'DLR W-NT-37','Biosonde 2','','','Bio',1821,'',246 from dual union all
select 22427,'1970-S652','1970-12-18 01:00:00','military missile','success',796,'','Mk 11 RV','OT GT58M','GLORY TRIP 58M','OT',1700,'',1300 from dual union all
select 22428,'1970-S653','1970-12-17 19:23:00','suborbital rocket','success',837,'NASA 14.418UA','','','','Aeronomy',1895,'',200 from dual union all
select 22429,'1970-S654','1970-12-18 05:30:00','suborbital rocket','success',740,'MR-12','','','','Ionos/Aeron',636,'',160 from dual union all
select 2243,'1954-A248','1954-05-08 00:00:00','atmospheric rocket','success',836,'1333T','','','','Test',1895,'',10 from dual union all
select 22430,'1970-S655','1970-12-19 03:10:02','suborbital rocket','success',37,'NASA 13.11UG','','','','XR Astron',1895,'',160 from dual union all
select 22431,'1970-S656','1970-12-22 08:38:00','military missile','success',168,'105F','WT2A-L21/WT2A-S21','ABRES RVTO-2A-1','','Reentry Test',1682,'',1400 from dual union all
select 22432,'1970-S657','1970-12-12 15:15:00','suborbital rocket','success',598,'INTA I-7001','','','','Test',396,'',132 from dual union all
select 22433,'1970-S658','1970-12-22 10:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',170 from dual union all
select 22434,'1970-S659','1970-12-22 18:25:00','suborbital rocket','success',740,'MR-12','Mass spectrometer','','','Aeron',1010,'',185 from dual union all
select 22435,'1970-S66','1970-01-14 05:00:00','suborbital rocket','success',837,'NASA 14.448CM','GCA Release TMA','','','Aeronomy',1821,'',204 from dual union all
select 22436,'1970-S660','1970-12-22 23:30:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',165 from dual union all
select 22437,'1970-S661','1970-12-24 01:55:00','military missile','success',795,'','SSTTP','SSTTP M1-12','','Target',1701,'',1300 from dual union all
select 22438,'1970-S662','1970-12-28 14:00:00','suborbital rocket','success',740,'MR-12','Mass spectrometer','','','Aeron',1010,'',150 from dual union all
select 22439,'1970-S663','1970-12-29 20:08:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',160 from dual union all
select 2244,'1954-A249','1954-05-09 00:00:00','atmospheric rocket','success',836,'1342T','','','','Test',1895,'',10 from dual union all
select 22440,'1970-S664','1970-08-21 15:39:00','suborbital rocket','success',837,'NASA 14.431UM','UM Pitot 49','','','Aeronomy',1821,'',197 from dual union all
select 22441,'1970-S665','1970-08-09 23:40:10','suborbital rocket','success',920,'','RTG SP4  (NLC)','','','Ionospheric',413,'',147 from dual union all
select 22442,'1970-S666','1970-05-15 11:14:00','suborbital rocket','success',1347,'','FIR Radiometer','','','IR Astron',855,'',300 from dual union all
select 22443,'1970-S667','1970-11-23 00:00:00','military missile','failure',359,'601906','Long range test','','','Test',804,'',500 from dual union all
select 22444,'1970-S668','1970-07-31 02:00:00','test rocket','success',645,'','K63D BOR-2','BOR-2 No. 102','','Reentry Test',1752,'',100 from dual union all
select 22445,'1970-S669','1970-04-09 00:00:00','suborbital rocket','success',839,'','3-sphere ejection','','','Aeron',1895,'',100 from dual union all
select 22446,'1970-S67','1970-01-14 07:17:00','suborbital rocket','success',837,'NASA 14.449CM','GCA Release TMA','','','Aeronomy',1821,'',204 from dual union all
select 22447,'1970-S670','1970-07-23 17:30:00','suborbital rocket','success',839,'3-Sphere Ejection','','','','Aeron',1895,'',100 from dual union all
select 22448,'1970-S671','1970-04-09 18:16:00','suborbital rocket','success',850,'','ECM Round','','','Aeron',1895,'',100 from dual union all
select 22449,'1970-S672','1970-11-05 18:59:00','suborbital rocket','success',27,'AH1016','Grenades','','','Aeron',1833,'',148 from dual union all
select 2245,'1954-A25','1954-01-18 00:00:00','atmospheric rocket','success',836,'1617E','','','','Test',1895,'',10 from dual union all
select 22450,'1970-S673','1970-05-26 14:53:00','suborbital rocket','success',675,'WRE K145','','','','Aeron',1832,'',43 from dual union all
select 22451,'1970-S674','1970-08-31 09:03:00','suborbital rocket','success',675,'WRE K147','','','','Aeron',1832,'',85 from dual union all
select 22452,'1970-S675','1970-10-21 09:18:00','suborbital rocket','success',675,'WRE K148','','','','Aeron',1832,'',85 from dual union all
select 22453,'1970-S676','1970-11-05 16:33:00','suborbital rocket','success',675,'WRE K151','','','','Aeron',1832,'',85 from dual union all
select 22454,'1970-S677','1970-08-01 00:00:00','suborbital rocket','success',1,'','Ion 2','','','Tech/Ionos',636,'',400 from dual union all
select 22455,'1970-S678','1970-10-07 02:00:00','suborbital rocket','success',839,'','Chemical Trail','','','Aeron',1895,'',100 from dual union all
select 22456,'1970-S679','1970-03-09 19:46:00','suborbital rocket','success',839,'3-Sphere Ejection','','','','Aeron',1895,'',100 from dual union all
select 22457,'1970-S68','1970-01-14 09:15:00','suborbital rocket','success',837,'NASA 14.450CM','GCA Release TMA','','','Aeronomy',1821,'',201 from dual union all
select 22458,'1970-S680','1970-10-06 02:00:00','suborbital rocket','success',839,'','Chemical Trail','','','Aeron',1895,'',100 from dual union all
select 22459,'1970-S681','1970-10-06 04:00:00','suborbital rocket','success',839,'','Chemical Trail','','','Aeron',1895,'',100 from dual union all
select 2246,'1954-A250','1954-05-10 00:00:00','atmospheric rocket','success',836,'1398E','','','','Test',1895,'',10 from dual union all
select 22460,'1970-S682','1970-11-17 22:20:00','suborbital rocket','unknown',837,'','','','','Aeron?',394,'',150 from dual union all
select 22461,'1970-S683','1970-05-26 09:10:00','suborbital rocket','success',1347,'Sandia 281-16','','','','Ionos',855,'',181 from dual union all
select 22462,'1970-S684','1970-05-26 10:30:00','suborbital rocket','success',839,'Sandia 154-200','Falling Sphere','','','Aeron',855,'',140 from dual union all
select 22463,'1970-S685','1970-05-27 08:45:00','suborbital rocket','success',867,'Sandia 152-119','','','','Aeron',855,'',301 from dual union all
select 22464,'1970-S686','1970-06-06 08:53:00','suborbital rocket','success',839,'Sandia 154-201','Falling Sphere','','','Aeron',855,'',140 from dual union all
select 22465,'1970-S687','1970-06-07 04:53:00','suborbital rocket','success',867,'Sandia 152-118','','','','Aeron',855,'',300 from dual union all
select 22466,'1970-S688','1970-10-21 00:00:00','suborbital rocket','unknown',1282,'','Diagnostic Test','','','Test',1415,'',300 from dual union all
select 22467,'1970-S689','1970-05-26 08:00:00','suborbital rocket','success',1268,'Sandia 428-12','BIRDSEED I Duck','Plasma gun','','Tech',855,'',210 from dual union all
select 22468,'1970-S69','1970-01-14 11:35:00','suborbital rocket','success',837,'NASA 14.451CM','GCA Release Na','','','Aeronomy',1821,'',207 from dual union all
select 22469,'1970-S690','1970-06-06 08:00:00','suborbital rocket','success',1268,'Sandia 428-','BIRDSEED I Egret','Plasma gun','','Tech',855,'',210 from dual union all
select 2247,'1954-A251','1954-05-11 00:00:00','military missile','unknown',339,'1295R','','','','Test',1895,'',30 from dual union all
select 22470,'1970-S691','1970-06-10 00:00:00','military missile','success',982,'R 22601-01T','8K64U UBP-P72','R-16 PL70-11','','OT',477,'',1210 from dual union all
select 22471,'1970-S692','1970-06-16 00:00:00','military missile','success',982,'R 22602-08','8K64U UBP-P73','R-16 PL70-02','','OT',477,'',1210 from dual union all
select 22472,'1970-S693','1970-07-07 00:00:00','military missile','success',982,'G 22501-08T','8K64U UBP-P75','R-16 PL70-03','','OT',477,'',1210 from dual union all
select 22473,'1970-S694','1970-07-31 00:00:00','military missile','success',982,'G 22501-24T','8K64U UBP-P76','R-16 PL70-04','','OT',477,'',1210 from dual union all
select 22474,'1970-S695','1970-08-14 00:00:00','military missile','success',982,'G 22501-22T','8K64U UBP-P77','R-16 PL70-05','','OT',477,'',1210 from dual union all
select 22475,'1970-S696','1970-08-25 00:00:00','military missile','success',982,'G 22502-10T','8K64U UBP-P78','R-16 PL70-06','','OT',477,'',1210 from dual union all
select 22476,'1970-S697','1970-09-15 00:00:00','military missile','success',982,'G 22501-11T','8K64U UBP-P79','R-16 PL70-07','','OT',477,'',1210 from dual union all
select 22477,'1970-S70','1970-01-16 00:00:00','military missile','success',1044,'Yu64913-3L','8K98P LKI-1','','','Test',464,'',1000 from dual union all
select 22478,'1970-S701','1970-06-25 00:00:00','military missile','success',982,'R 22604-18T','8K64U UBP-P74','R-16 PL70-11','','OT',477,'',1210 from dual union all
select 22479,'1970-S706','1970-07-08 00:00:00','military missile','success',1044,'Kh26522-8L','8K98P LKI-6','','','Test',477,'',1000 from dual union all
select 2248,'1954-A252','1954-05-11 00:00:00','atmospheric rocket','unknown',576,'220M','','','','Test',1895,'',10 from dual union all
select 22480,'1970-S707','1970-03-09 00:00:00','military missile','success',1037,'Kh29521-32','8K98 UBP-11','','','Test',477,'',1000 from dual union all
select 22481,'1970-S708','1970-02-17 00:00:00','military missile','success',1044,'Yu52614-4L','8K98P LKI-2','','','Test',477,'',1000 from dual union all
select 22482,'1970-S709','1970-05-18 00:00:00','military missile','success',1044,'Yu64916-6L','8K98P LKI-4','','','Test',477,'',1000 from dual union all
select 22483,'1970-S71','1970-01-16 05:40:00','suborbital rocket','success',867,'NASA 18.79GM','Auroral EUV','','','Auroral/Aeronomy',428,'',210 from dual union all
select 22484,'1970-S710','1970-06-08 00:00:00','military missile','success',1044,'Kh52621-7L','8K98P LKI-5','','','Test',477,'',1000 from dual union all
select 22485,'1970-S712','1970-04-10 00:00:00','military missile','success',1044,'Yu26515-5L','8K98P LKI-3','','','Test',477,'',1000 from dual union all
select 22486,'1970-S713','1970-07-29 00:00:00','military missile','success',1037,'V 29501-01','8K98 UBP-12','','','Test',477,'',1000 from dual union all
select 22487,'1970-S714','1970-08-18 00:00:00','military missile','success',1037,'64647-221-31','8K98 UBP-13','','','Test',477,'',1000 from dual union all
select 22488,'1970-S715','1970-09-04 00:00:00','military missile','success',1037,'Yu45713-28','8K98 UBP-14','','','Test',477,'',1000 from dual union all
select 22489,'1970-S716','1970-06-21 00:00:00','military missile','success',1449,'','8K84 54 RD UBP-8','','','OT',1468,'',1000 from dual union all
select 2249,'1954-A253','1954-05-11 00:00:00','atmospheric rocket','unknown',576,'268M','','','','Test',1895,'',10 from dual union all
select 22490,'1970-S717','1970-11-11 00:00:00','military missile','success',1449,'','8K84 54 RD UBP-9','','','OT',1468,'',1000 from dual union all
select 22491,'1970-S718','1970-11-11 00:00:00','military missile','success',1449,'','8K84 54 RD UBP-10','','','OT',1468,'',1000 from dual union all
select 22492,'1970-S719','1970-11-11 00:00:00','military missile','success',1449,'','8K84 10 RD UBP-3','','','OT',897,'',1000 from dual union all
select 22493,'1970-S72','1970-01-16 12:00:00','suborbital rocket','success',1189,'INTA MP-7003','','','','Ionos',396,'',97 from dual union all
select 22494,'1970-S720','1970-11-11 00:00:00','military missile','success',1449,'','8K84 UBP 52 RD','','','OT',195,'',1000 from dual union all
select 22495,'1970-S721','1970-11-12 00:00:00','military missile','success',1449,'','8K84 36 RD UBP-3','','','OT',898,'',1000 from dual union all
select 22496,'1970-S722','1970-10-28 00:00:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 22497,'1970-S723','1970-11-11 17:53:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 22498,'1970-S724','1970-05-26 13:59:00','suborbital rocket','success',27,'FC9/1','Grenades/Li','','','Aeron',1833,'',175 from dual union all
select 22499,'1970-S725','1970-05-27 08:37:00','suborbital rocket','success',27,'FC11/3','Grenades/Li','','','Aeron',1833,'',175 from dual union all
select 225,'1944-S204','1944-10-16 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1728,'',90 from dual union all
select 2250,'1954-A254','1954-05-11 00:00:00','military missile','success',573,'1','XSSM-A-16','','','Test',1863,'',32 from dual union all
select 22500,'1970-S726','1970-11-16 23:19:00','suborbital rocket','success',848,'','ALADDIN test','','','Aeron',394,'',150 from dual union all
select 22501,'1970-S727','1970-01-24 00:00:00','military missile','success',982,'G 22501-01T','8K64U UBP-138','','','OT',596,'',1210 from dual union all
select 22502,'1970-S728','1970-02-28 00:00:00','military missile','success',982,'G 22501-02T','8K64U UBP-140','','','OT',596,'',1210 from dual union all
select 22503,'1970-S729','1970-03-12 00:00:00','military missile','success',982,'G 22501-21T','8K64U UBP-141','','','OT',596,'',1210 from dual union all
select 22504,'1970-S73','1970-01-17 03:30:00','military missile','success',1000,'N 22500-06T','8K67M UBP-6','GCh 8F675','','Test',518,'',1000 from dual union all
select 22505,'1970-S730','1970-05-29 00:00:00','military missile','success',980,'T 14902-06T','8K64U Aldan Target 5','Aldan','','Target',477,'',675 from dual union all
select 22506,'1970-S731','1970-06-06 00:00:00','military missile','success',980,'T 14901-01T','8K64U Aldan Target 6','Aldan','','Target',477,'',675 from dual union all
select 22507,'1970-S732','1970-11-26 00:00:00','military missile','success',1044,'47827-434-14L','8K98P LKI-11','','','Test',477,'',1000 from dual;
insert into LAUNCH
select 22508,'1970-S733','1970-11-28 00:00:00','military missile','success',1044,'52627-435-15L','8K98P LKI-12','','','Test',477,'',1000 from dual union all
select 22509,'1970-S734','1970-03-27 00:00:00','military missile','success',982,'T 22503-07T','8K64U UBP-143','','','OT',596,'',1210 from dual union all
select 2251,'1954-A255','1954-05-12 00:00:00','atmospheric rocket','unknown',576,'585','','','','Test',1895,'',10 from dual union all
select 22510,'1970-S735','1970-05-28 00:00:00','military missile','success',982,'','8K64U UBP 52-RD','52 RD','','OT',195,'',1210 from dual union all
select 22511,'1970-S736','1970-08-01 00:00:00','military missile','success',982,'','8K64U UBP 52-RD','52 RD','','OT',195,'',1210 from dual union all
select 22512,'1970-S737','1970-01-01 00:00:00','military missile','success',977,'','8K63U UBP','','','Test',840,'',402 from dual union all
select 22513,'1970-S738','1970-01-01 00:00:00','military missile','success',977,'','8K63U UBP','','','Test',840,'',402 from dual union all
select 22514,'1970-S739','1970-01-01 00:00:00','military missile','success',977,'','8K63U UBP','','','Test',840,'',402 from dual union all
select 22515,'1970-S74','1970-01-17 01:20:00','suborbital rocket','success',1070,'S-210-3','','','','XR Astron',839,'',119 from dual union all
select 22516,'1970-S740','1970-03-01 00:00:00','military missile','success',976,'','8K63 44 RD UBP','','','Test',636,'',402 from dual union all
select 22517,'1970-S741','1970-03-17 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 22518,'1970-S742','1970-10-21 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 22519,'1970-S743','1970-10-21 00:00:00','military missile','success',976,'','8K63 43 RD UBP','','','Test',636,'',402 from dual union all
select 2252,'1954-A256','1954-05-12 00:00:00','atmospheric rocket','unknown',576,'586','','','','Test',1895,'',10 from dual union all
select 22520,'1970-S744','1970-10-22 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 22521,'1970-S745','1970-02-20 00:00:00','military missile','unknown',979,'','8K65 UBP 32 RD','','','Test',636,'',675 from dual union all
select 22522,'1970-S746','1970-06-20 00:00:00','military missile','unknown',979,'','8K65 UBP 32 RD','','','Test',636,'',675 from dual union all
select 22523,'1970-S747','1970-10-13 00:00:00','military missile','unknown',980,'','8K65U UBP 68 RD','','','Test',636,'',675 from dual union all
select 22524,'1970-S748','1970-01-01 00:00:00','suborbital rocket','success',846,'','HAVE HORN 2','','','Target',1003,'',100 from dual union all
select 22525,'1970-S749','1970-05-27 00:00:00','suborbital rocket','success',1276,'','MTVI-1','','','Reentry Test',855,'',200 from dual union all
select 22526,'1970-S75','1970-01-17 03:03:20','suborbital rocket','success',867,'NASA 18.65UE','','','','Auroral',428,'',252 from dual union all
select 22527,'1970-S76','1970-01-17 11:45:00','suborbital rocket','success',1189,'INTA MP-7004','','','','Ionos',396,'',89 from dual union all
select 22528,'1970-S77','1970-01-21 03:05:00','military missile','success',1000,'','8K67M UBP-7','','','Test',515,'',1000 from dual union all
select 22529,'1970-S78','1970-01-21 02:10:00','sounding rocket','success',698,'L-3H-6','Te probe','','','Ionos/Plasma',834,'',1848 from dual union all
select 2253,'1954-A257','1954-05-12 00:00:00','atmospheric rocket','success',836,'1484E','','','','Test',1895,'',10 from dual union all
select 22530,'1970-S79','1970-01-21 11:35:00','suborbital rocket','success',1189,'INTA MP-7006','','','','Ionos',396,'',100 from dual union all
select 22531,'1970-S80','1970-01-23 18:00:00','military missile','success',939,'C3XT-19','','','','Test',246,'',500 from dual union all
select 22532,'1970-S81','1970-01-24 14:27:00','suborbital rocket','success',1200,'ESRO S56A/1','RSRS SUV-1','','','Ionos/Solar UV',1343,'',100 from dual union all
select 22533,'1970-S82','1970-01-24 23:00:00','suborbital rocket','success',30,'NRL NF3.193','Solar IR','','','Solar IR',1869,'',207 from dual union all
select 22534,'1970-S83','1970-01-25 02:20:00','suborbital rocket','success',32,'NASA 04.270UG','','','','UV Astronomy',1869,'',169 from dual union all
select 22535,'1970-S84','1970-01-25 05:00:00','suborbital rocket','success',668,'K-9M-28','Te probe','','','Ionos/Aeron',839,'',369 from dual union all
select 22536,'1970-S85','1970-01-26 22:30:00','suborbital rocket','success',624,'N-J-19','DLR N-J-19','','','Ionos?',330,'',800 from dual union all
select 22537,'1970-S86','1970-01-27 10:20:00','suborbital rocket','success',668,'K-9M-29','Te probe','','','Ionos',839,'',357 from dual union all
select 22538,'1970-S87','1970-01-29 17:19:00','suborbital rocket','success',839,'NASA 10.338GM','GSFC Grenade 229 PB','','','Aeronomy',1039,'',91 from dual union all
select 22539,'1970-S88','1970-01-10 00:00:00','military missile','success',293,'601901','R&D (Minfeng target)','','','Test',809,'',500 from dual union all
select 2254,'1954-A258','1954-05-12 00:00:00','atmospheric rocket','success',836,'2177R','Nike 283R','','','Test',1895,'',10 from dual union all
select 22540,'1970-S89','1970-01-30 02:05:00','military missile','success',1000,'','8K67M UBP-8','','','Test',515,'',1000 from dual union all
select 22541,'1970-S90','1970-01-30 03:05:00','military missile','success',1000,'N 22500-07T','8K67M UBP-9','GCh 8F675','','Test',518,'',1000 from dual union all
select 22542,'1970-S91','1970-01-30 22:30:00','suborbital rocket','success',624,'N-J-20','DLR N-J-20','','','Ionos?',330,'',800 from dual union all
select 22543,'1970-S92','1970-02-01 01:00:00','suborbital rocket','success',613,'JCR-3','','','','Test',1476,'',140 from dual union all
select 22544,'1970-S93','1970-02-02 23:07:31','suborbital rocket','success',226,'A-BBVA-23   (X1)','DLR A-BBV-23 Synove','','','Auroral',29,'',202 from dual union all
select 22545,'1970-S94','1970-02-03 13:44:00','suborbital rocket','success',839,'NASA 10.340GM','GSFC Grenade 231 PB','','','Aeronomy',1039,'',91 from dual union all
select 22546,'1970-S95','1970-02-03 13:55:00','suborbital rocket','success',839,'NASA 10.341GM','GSFC Grenade 232 PB','','','Aeronomy',1039,'',87 from dual union all
select 22547,'1970-S96','1970-02-03 06:03:00','suborbital rocket','success',694,'LS-C-3','','','','Test',1476,'',65 from dual union all
select 22548,'1970-S97','1970-02-03 22:24:33','suborbital rocket','success',867,'NASA 18.113IE','Ferdinand 25/Polar 1','','','Plasma',29,'',283 from dual union all
select 22549,'1970-S98','1970-09-04 00:00:00','suborbital rocket','failure',613,'JCR-4','','','','Test',1476,'',19 from dual union all
select 2255,'1954-A259','1954-05-12 00:00:00','atmospheric rocket','success',836,'2249R','Nike 284R','','','Test',1895,'',10 from dual union all
select 22550,'1970-S99','1970-02-04 04:41:00','suborbital rocket','success',837,'NASA 14.441UI','','','','Auroral/Ionos',428,'',141 from dual union all
select 22551,'1970-U01','1970-09-21 07:43:00','lunar return','success',450,'No. 406','Luna-16','VA','','Sat',429,'',null from dual union all
select 22552,'1970-U02','1970-06-12 01:10:32','orbital','failure',467,'F-9','STV 3','STV 3','','Sat',1845,'',2000 from dual union all
select 22553,'1970-U03','1970-11-30 22:40:05','orbital','failure',179,'AC-21','OAO B','OAO B','','Sat',269,'',600 from dual union all
select 22554,'1970-W01','1970-04-29 13:58:00','suborbital rocket','failure',88,'PCA','AFCRL Arcas 70-4','','','Ionos',1470,'',60 from dual union all
select 22555,'1970-W02','1970-03-29 10:34:12','suborbital rocket','success',88,'PCA','AFCRL Arcas 70-','','','Ionos',1470,'',70 from dual union all
select 22556,'1970-W03','1970-04-20 10:01:00','suborbital rocket','success',88,'PCA','AFCRL Arcas 70-5','','','Ionos',1470,'',60 from dual union all
select 22557,'1970-W04','1970-04-29 10:36:00','suborbital rocket','success',88,'PCA','AFCRL Arcas 70-3','','','Ionos',1470,'',60 from dual union all
select 22558,'1970-W05','1970-04-29 12:34:00','suborbital rocket','success',88,'PCA','AFCRL Arcas 70-6','','','Ionos',1470,'',60 from dual union all
select 22559,'1970-W06','1970-01-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 2256,'1954-A26','1954-01-19 00:00:00','atmospheric rocket','unknown',576,'401M','','','','Test',1895,'',10 from dual union all
select 22560,'1970-W07','1970-01-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 22561,'1970-W08','1970-01-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 22562,'1970-W09','1970-01-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 22563,'1970-W10','1970-01-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 22564,'1970-W100','1970-01-16 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 22565,'1970-W1000','1970-07-09 02:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 22566,'1970-W1001','1970-07-09 15:45:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',53 from dual union all
select 22567,'1970-W1002','1970-07-09 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',52 from dual union all
select 22568,'1970-W1003','1970-07-10 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 22569,'1970-W1004','1970-07-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 2257,'1954-A260','1954-05-12 00:00:00','atmospheric rocket','success',836,'1334T','','','','Test',1895,'',10 from dual union all
select 22570,'1970-W1005','1970-07-10 16:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',63 from dual union all
select 22571,'1970-W1006','1970-07-10 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 22572,'1970-W1007','1970-07-10 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',57 from dual union all
select 22573,'1970-W1008','1970-07-10 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',53 from dual union all
select 22574,'1970-W1009','1970-07-11 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 22575,'1970-W101','1970-01-16 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 22576,'1970-W1010','1970-07-13 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 22577,'1970-W1011','1970-07-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 22578,'1970-W1012','1970-07-13 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 22579,'1970-W1013','1970-07-13 16:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 2258,'1954-A261','1954-05-12 00:00:00','atmospheric rocket','success',836,'1345R','Nike 285R','','','Test',1895,'',10 from dual union all
select 22580,'1970-W1014','1970-07-13 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 22581,'1970-W1015','1970-07-13 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 22582,'1970-W1016','1970-07-13 17:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 22583,'1970-W1017','1970-07-13 17:07:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',61 from dual union all
select 22584,'1970-W1018','1970-07-13 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 22585,'1970-W1019','1970-07-13 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 22586,'1970-W102','1970-01-16 15:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 22587,'1970-W1020','1970-07-14 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 22588,'1970-W1021','1970-07-14 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',69 from dual union all
select 22589,'1970-W1022','1970-07-14 16:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',68 from dual union all
select 2259,'1954-A262','1954-05-12 00:00:00','atmospheric rocket','success',836,'2207R','Nike 281R','','','Test',1895,'',10 from dual union all
select 22590,'1970-W1023','1970-07-14 22:34:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',51 from dual union all
select 22591,'1970-W1024','1970-07-15 02:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',57 from dual union all
select 22592,'1970-W1025','1970-07-15 13:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 22593,'1970-W1026','1970-07-15 14:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',65 from dual union all
select 22594,'1970-W1027','1970-07-15 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 22595,'1970-W1028','1970-07-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 22596,'1970-W1029','1970-07-15 15:05:00','suborbital rocket','unknown',631,'SUPARCO-063/70','','','','Aeron',1419,'',63 from dual union all
select 22597,'1970-W103','1970-01-16 16:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 22598,'1970-W1030','1970-07-15 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual;
insert into LAUNCH
select 22599,'1970-W1031','1970-07-15 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 226,'1944-S205','1944-10-20 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1742,'',95 from dual union all
select 2260,'1954-A263','1954-05-12 00:00:00','atmospheric rocket','success',836,'1700E','','','','Test',1895,'',10 from dual union all
select 22600,'1970-W1032','1970-07-15 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',57 from dual union all
select 22601,'1970-W1033','1970-07-15 22:12:59','suborbital rocket','success',1188,'INTA S-7006','','','','Meteo',396,'',56 from dual union all
select 22602,'1970-W1034','1970-07-16 01:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 22603,'1970-W1035','1970-07-16 16:25:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',51 from dual union all
select 22604,'1970-W1036','1970-07-16 17:19:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 22605,'1970-W1037','1970-07-16 17:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 22606,'1970-W1038','1970-07-16 21:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',50 from dual union all
select 22607,'1970-W1039','1970-07-17 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 22608,'1970-W104','1970-01-16 16:52:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',50 from dual union all
select 22609,'1970-W1040','1970-07-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 2261,'1954-A264','1954-05-12 00:00:00','atmospheric rocket','success',836,'1643E','','','','Test',1895,'',10 from dual union all
select 22610,'1970-W1041','1970-07-17 15:24:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',64 from dual union all
select 22611,'1970-W1042','1970-07-17 17:12:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',65 from dual union all
select 22612,'1970-W1043','1970-07-17 17:51:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',63 from dual union all
select 22613,'1970-W1044','1970-07-17 17:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 22614,'1970-W1045','1970-07-17 18:03:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',60 from dual union all
select 22615,'1970-W1046','1970-07-17 18:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 22616,'1970-W1047','1970-07-17 21:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 22617,'1970-W1048','1970-07-18 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 22618,'1970-W1049','1970-07-18 11:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 22619,'1970-W105','1970-01-16 17:25:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1363,'',61 from dual union all
select 2262,'1954-A265','1954-05-12 00:00:00','atmospheric rocket','success',836,'1467E','','','','Test',1895,'',10 from dual union all
select 22620,'1970-W1050','1970-07-20 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 22621,'1970-W1051','1970-07-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 22622,'1970-W1052','1970-07-20 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',52 from dual union all
select 22623,'1970-W1053','1970-07-20 15:12:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',64 from dual union all
select 22624,'1970-W1054','1970-07-20 15:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 22625,'1970-W1055','1970-07-20 16:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',64 from dual union all
select 22626,'1970-W1056','1970-07-20 16:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 22627,'1970-W1057','1970-07-20 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',70 from dual union all
select 22628,'1970-W1058','1970-07-20 18:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',63 from dual union all
select 22629,'1970-W1059','1970-07-20 18:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 2263,'1954-A266','1954-05-12 00:00:00','atmospheric rocket','success',836,'1305R','Nike 282R','','','Test',1895,'',10 from dual union all
select 22630,'1970-W106','1970-01-16 18:11:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',56 from dual union all
select 22631,'1970-W1060','1970-07-20 21:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 22632,'1970-W1061','1970-07-21 01:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 22633,'1970-W1062','1970-07-21 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 22634,'1970-W1063','1970-07-21 14:59:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 22635,'1970-W1064','1970-07-21 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',54 from dual union all
select 22636,'1970-W1065','1970-07-21 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',58 from dual union all
select 22637,'1970-W1066','1970-07-21 21:35:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',57 from dual union all
select 22638,'1970-W1067','1970-07-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 22639,'1970-W1068','1970-07-22 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 2264,'1954-A267','1954-05-13 00:00:00','atmospheric rocket','unknown',576,'580','','','','Test',1895,'',10 from dual union all
select 22640,'1970-W1069','1970-07-22 16:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 22641,'1970-W107','1970-01-16 19:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',55 from dual union all
select 22642,'1970-W1070','1970-07-22 17:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',52 from dual union all
select 22643,'1970-W1071','1970-07-22 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 22644,'1970-W1072','1970-07-22 17:55:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',64 from dual union all
select 22645,'1970-W1073','1970-07-22 18:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',77 from dual union all
select 22646,'1970-W1074','1970-07-22 19:10:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 22647,'1970-W1075','1970-07-22 19:44:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 22648,'1970-W1076','1970-07-22 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',52 from dual union all
select 22649,'1970-W1077','1970-07-23 02:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 2265,'1954-A268','1954-05-13 00:00:00','atmospheric rocket','unknown',576,'584','','','','Test',1895,'',10 from dual union all
select 22650,'1970-W1078','1970-07-23 15:32:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',79 from dual union all
select 22651,'1970-W1079','1970-07-23 15:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 22652,'1970-W108','1970-01-16 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',57 from dual union all
select 22653,'1970-W1080','1970-07-23 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 22654,'1970-W1081','1970-07-23 17:01:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 22655,'1970-W1082','1970-07-23 18:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',53 from dual union all
select 22656,'1970-W1083','1970-07-23 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 22657,'1970-W1084','1970-07-23 21:31:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',53 from dual union all
select 22658,'1970-W1085','1970-07-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 22659,'1970-W1086','1970-07-24 14:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 2266,'1954-A269','1954-05-13 00:00:00','atmospheric rocket','success',836,'1350T','','','','Test',1895,'',10 from dual union all
select 22660,'1970-W1087','1970-07-24 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',60 from dual union all
select 22661,'1970-W1088','1970-07-24 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 22662,'1970-W1089','1970-07-24 17:45:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',63 from dual union all
select 22663,'1970-W109','1970-01-16 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 22664,'1970-W1090','1970-07-24 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 22665,'1970-W1091','1970-07-24 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 22666,'1970-W1092','1970-07-25 01:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 22667,'1970-W1093','1970-07-27 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 22668,'1970-W1094','1970-07-27 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 22669,'1970-W1095','1970-07-27 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',67 from dual union all
select 2267,'1954-A27','1954-01-20 00:00:00','atmospheric rocket','unknown',576,'216M','','','','Test',1895,'',10 from dual union all
select 22670,'1970-W1096','1970-07-27 16:42:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',66 from dual union all
select 22671,'1970-W1097','1970-07-27 17:11:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 22672,'1970-W1098','1970-07-27 19:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 22673,'1970-W1099','1970-07-27 21:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 22674,'1970-W11','1970-01-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 22675,'1970-W110','1970-01-16 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 22676,'1970-W1100','1970-07-28 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 22677,'1970-W1101','1970-07-28 14:45:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',61 from dual union all
select 22678,'1970-W1102','1970-07-28 16:02:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',58 from dual union all
select 22679,'1970-W1103','1970-07-28 16:42:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',61 from dual union all
select 2268,'1954-A270','1954-05-14 00:00:00','atmospheric rocket','unknown',576,'577','','','','Test',1895,'',10 from dual union all
select 22680,'1970-W1104','1970-07-28 21:38:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',52 from dual union all
select 22681,'1970-W1105','1970-07-29 02:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 22682,'1970-W1106','1970-07-29 10:54:59','suborbital rocket','success',631,'INTA H-7017','','','','Meteo',396,'',58 from dual union all
select 22683,'1970-W1107','1970-07-29 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 22684,'1970-W1108','1970-07-29 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 22685,'1970-W1109','1970-07-29 14:20:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',66 from dual union all
select 22686,'1970-W111','1970-01-17 00:58:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 22687,'1970-W1110','1970-07-29 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 22688,'1970-W1111','1970-07-29 15:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual;
insert into LAUNCH
select 22689,'1970-W1112','1970-07-29 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 2269,'1954-A271','1954-05-14 00:00:00','atmospheric rocket','unknown',576,'581','','','','Test',1895,'',10 from dual union all
select 22690,'1970-W1113','1970-07-29 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 22691,'1970-W1114','1970-07-29 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 22692,'1970-W1115','1970-07-29 21:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 22693,'1970-W1116','1970-07-30 01:24:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 22694,'1970-W1117','1970-07-30 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 22695,'1970-W1118','1970-07-30 15:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 22696,'1970-W1119','1970-07-30 16:19:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',52 from dual union all
select 22697,'1970-W112','1970-01-17 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 22698,'1970-W1120','1970-07-30 17:07:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',58 from dual union all
select 22699,'1970-W1121','1970-08-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 227,'1944-S206','1944-10-20 00:00:00','military missile','success',1470,'','London/Croydon','','','Weapon',1734,'',95 from dual union all
select 2270,'1954-A272','1954-05-14 00:00:00','military missile','success',771,'GM-12481','','','','Test',222,'',12 from dual union all
select 22700,'1970-W1122','1970-07-31 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 22701,'1970-W1123','1970-07-31 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 22702,'1970-W1124','1970-07-31 15:38:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',65 from dual union all
select 22703,'1970-W1125','1970-07-31 16:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',66 from dual union all
select 22704,'1970-W1126','1970-07-31 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',61 from dual union all
select 22705,'1970-W1127','1970-07-31 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 22706,'1970-W1128','1970-07-31 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',65 from dual union all
select 22707,'1970-W1129','1970-07-31 21:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 22708,'1970-W113','1970-01-17 14:46:00','suborbital rocket','success',1188,'UCL 06','','','','Aeron',417,'',72 from dual union all
select 22709,'1970-W1130','1970-08-01 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 2271,'1954-A273','1954-05-14 00:00:00','military missile','success',771,'GM-12478','','','','Test',222,'',12 from dual union all
select 22710,'1970-W1131','1970-08-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 22711,'1970-W1132','1970-08-03 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',65 from dual union all
select 22712,'1970-W1133','1970-08-03 15:56:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',51 from dual union all
select 22713,'1970-W1134','1970-08-03 15:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 22714,'1970-W1135','1970-08-03 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 22715,'1970-W1136','1970-08-03 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',62 from dual union all
select 22716,'1970-W1137','1970-08-03 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 22717,'1970-W1138','1970-08-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',63 from dual union all
select 22718,'1970-W1139','1970-08-03 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 22719,'1970-W114','1970-01-19 11:30:00','suborbital rocket','success',1188,'INTA S-7001','','','','Meteo',396,'',59 from dual union all
select 2272,'1954-A274','1954-05-14 00:00:00','military missile','success',771,'GM-12487','','','','Test',222,'',12 from dual union all
select 22720,'1970-W1140','1970-08-04 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 22721,'1970-W1141','1970-08-04 14:15:00','suborbital rocket','success',1061,'','','','','Meteo',1895,'',66 from dual union all
select 22722,'1970-W1142','1970-08-04 19:47:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 22723,'1970-W1143','1970-08-04 21:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',50 from dual union all
select 22724,'1970-W1144','1970-08-05 02:00:00','suborbital rocket','success',761,'MT135P-R2','Range Test','','','Aeron',1332,'',50 from dual union all
select 22725,'1970-W1145','1970-08-05 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 22726,'1970-W1146','1970-08-05 13:37:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1821,'',68 from dual union all
select 22727,'1970-W1147','1970-08-05 13:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 22728,'1970-W1148','1970-08-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 22729,'1970-W1149','1970-08-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 2273,'1954-A275','1954-05-14 00:00:00','atmospheric rocket','success',836,'1405T','','','','Test',1895,'',10 from dual union all
select 22730,'1970-W115','1970-01-19 15:01:00','suborbital rocket','success',1188,'UCL 07','','','','Aeron',417,'',70 from dual union all
select 22731,'1970-W1150','1970-08-05 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 22732,'1970-W1151','1970-08-05 18:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 22733,'1970-W1152','1970-08-05 18:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 22734,'1970-W1153','1970-08-05 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 22735,'1970-W1154','1970-08-05 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 22736,'1970-W1155','1970-08-06 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 22737,'1970-W1156','1970-08-06 05:37:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',54 from dual union all
select 22738,'1970-W1157','1970-08-06 15:22:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',61 from dual union all
select 22739,'1970-W1158','1970-08-06 15:44:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 2274,'1954-A276','1954-05-14 00:00:00','atmospheric rocket','success',836,'1692E','','','','Test',1895,'',10 from dual union all
select 22740,'1970-W1159','1970-08-06 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 22741,'1970-W116','1970-01-19 16:04:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1363,'',56 from dual union all
select 22742,'1970-W1160','1970-08-06 21:07:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',54 from dual union all
select 22743,'1970-W1161','1970-08-07 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 22744,'1970-W1162','1970-08-07 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 22745,'1970-W1163','1970-08-07 14:14:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 22746,'1970-W1164','1970-08-07 17:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',71 from dual union all
select 22747,'1970-W1165','1970-08-07 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 22748,'1970-W1166','1970-08-07 18:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 22749,'1970-W1167','1970-08-07 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',62 from dual union all
select 2275,'1954-A277','1954-05-14 00:00:00','atmospheric rocket','success',836,'1511E','','','','Test',1895,'',10 from dual union all
select 22750,'1970-W1168','1970-08-08 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 22751,'1970-W1169','1970-08-08 04:55:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 22752,'1970-W117','1970-01-19 17:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',62 from dual union all
select 22753,'1970-W1170','1970-08-10 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 22754,'1970-W1171','1970-08-10 14:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',52 from dual union all
select 22755,'1970-W1172','1970-08-10 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 22756,'1970-W1173','1970-08-10 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 22757,'1970-W1174','1970-08-10 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 22758,'1970-W1175','1970-08-10 19:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 22759,'1970-W1176','1970-08-11 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 2276,'1954-A278','1954-05-17 00:00:00','atmospheric rocket','unknown',576,'576M','','','','Test',1895,'',10 from dual union all
select 22760,'1970-W1177','1970-08-11 04:37:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 22761,'1970-W1178','1970-08-11 21:41:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',54 from dual union all
select 22762,'1970-W1179','1970-08-12 13:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 22763,'1970-W118','1970-01-19 18:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',58 from dual union all
select 22764,'1970-W1180','1970-08-12 14:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',59 from dual union all
select 22765,'1970-W1181','1970-08-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 22766,'1970-W1182','1970-08-12 14:26:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',52 from dual union all
select 22767,'1970-W1183','1970-08-12 14:34:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',62 from dual union all
select 22768,'1970-W1184','1970-08-12 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 22769,'1970-W1185','1970-08-12 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 2277,'1954-A279','1954-05-17 00:00:00','atmospheric rocket','unknown',576,'578','','','','Test',1895,'',10 from dual union all
select 22770,'1970-W1186','1970-08-12 15:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 22771,'1970-W1187','1970-08-12 15:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 22772,'1970-W1188','1970-08-12 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 22773,'1970-W1189','1970-08-12 16:54:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',52 from dual union all
select 22774,'1970-W119','1970-01-19 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 22775,'1970-W1190','1970-08-12 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 22776,'1970-W1191','1970-08-12 18:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 22777,'1970-W1192','1970-08-12 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 22778,'1970-W1193','1970-08-12 23:44:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual;
insert into LAUNCH
select 22779,'1970-W1194','1970-08-13 01:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 2278,'1954-A28','1954-01-21 00:00:00','military missile','unknown',339,'1395R','','','','Test',1895,'',30 from dual union all
select 22780,'1970-W1195','1970-08-13 14:48:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',60 from dual union all
select 22781,'1970-W1196','1970-08-13 15:31:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',62 from dual union all
select 22782,'1970-W1197','1970-08-13 21:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',53 from dual union all
select 22783,'1970-W1198','1970-08-14 10:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 22784,'1970-W1199','1970-08-14 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 22785,'1970-W12','1970-01-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 22786,'1970-W120','1970-01-19 18:59:59','suborbital rocket','success',1061,'','Starute,STS','','','Meteo',1895,'',63 from dual union all
select 22787,'1970-W1200','1970-08-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 22788,'1970-W1201','1970-08-14 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 22789,'1970-W1202','1970-08-14 14:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 2279,'1954-A280','1954-05-17 00:00:00','atmospheric rocket','success',836,'1716E','','','','Test',1895,'',10 from dual union all
select 22790,'1970-W1203','1970-08-14 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 22791,'1970-W1204','1970-08-14 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 22792,'1970-W1205','1970-08-14 18:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',57 from dual union all
select 22793,'1970-W1206','1970-08-14 20:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',58 from dual union all
select 22794,'1970-W1207','1970-08-14 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 22795,'1970-W1208','1970-08-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 22796,'1970-W1209','1970-08-17 15:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',64 from dual union all
select 22797,'1970-W121','1970-01-19 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',57 from dual union all
select 22798,'1970-W1210','1970-08-17 15:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 22799,'1970-W1211','1970-08-17 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',59 from dual union all
select 228,'1944-S207','1944-10-28 10:48:00','military missile','success',1470,'19487','Ma239','','','Test',661,'',90 from dual union all
select 2280,'1954-A281','1954-05-18 00:00:00','military missile','unknown',339,'1403R','','','','Test',1895,'',30 from dual union all
select 22800,'1970-W1212','1970-08-17 17:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 22801,'1970-W1213','1970-08-17 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',57 from dual union all
select 22802,'1970-W1214','1970-08-17 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 22803,'1970-W1215','1970-08-17 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',56 from dual union all
select 22804,'1970-W1216','1970-08-17 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 22805,'1970-W1217','1970-08-18 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',53 from dual union all
select 22806,'1970-W1218','1970-08-18 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 22807,'1970-W1219','1970-08-18 16:13:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 22808,'1970-W122','1970-01-19 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 22809,'1970-W1220','1970-08-18 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 2281,'1954-A282','1954-05-18 00:00:00','military missile','success',771,'GM-12441','','','','Test',222,'',12 from dual union all
select 22810,'1970-W1221','1970-08-18 21:17:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',56 from dual union all
select 22811,'1970-W1222','1970-08-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 22812,'1970-W1223','1970-08-19 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 22813,'1970-W1224','1970-08-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 22814,'1970-W1225','1970-08-19 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',73 from dual union all
select 22815,'1970-W1226','1970-08-19 16:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 22816,'1970-W1227','1970-08-19 17:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',50 from dual union all
select 22817,'1970-W1228','1970-08-19 17:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 22818,'1970-W1229','1970-08-19 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 22819,'1970-W123','1970-01-20 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 2282,'1954-A283','1954-05-18 00:00:00','atmospheric rocket','success',836,'1455T','','','','Test',1895,'',10 from dual union all
select 22820,'1970-W1230','1970-08-19 18:36:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',59 from dual union all
select 22821,'1970-W1231','1970-08-19 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 22822,'1970-W1232','1970-08-19 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',66 from dual union all
select 22823,'1970-W1233','1970-08-19 21:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 22824,'1970-W1234','1970-08-19 22:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 22825,'1970-W1235','1970-08-20 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 22826,'1970-W1236','1970-08-20 02:00:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 22827,'1970-W1237','1970-08-20 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 22828,'1970-W1238','1970-08-20 15:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 22829,'1970-W1239','1970-08-20 16:23:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',61 from dual union all
select 2283,'1954-A284','1954-05-19 00:00:00','military missile','success',771,'GM-12473','','','','Test',222,'',12 from dual union all
select 22830,'1970-W124','1970-01-20 03:25:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 22831,'1970-W1240','1970-08-20 21:36:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',54 from dual union all
select 22832,'1970-W1241','1970-08-21 13:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 22833,'1970-W1242','1970-08-21 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 22834,'1970-W1243','1970-08-21 20:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',60 from dual union all
select 22835,'1970-W1244','1970-08-21 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 22836,'1970-W1245','1970-08-22 00:15:00','suborbital rocket','success',779,'','','','','Aeron',933,'',57 from dual union all
select 22837,'1970-W1246','1970-08-22 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',57 from dual union all
select 22838,'1970-W1247','1970-08-24 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 22839,'1970-W1248','1970-08-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 2284,'1954-A285','1954-05-19 00:00:00','military missile','success',771,'GM-12484','','','','Test',222,'',12 from dual union all
select 22840,'1970-W1249','1970-08-24 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',62 from dual union all
select 22841,'1970-W125','1970-01-20 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 22842,'1970-W1250','1970-08-24 16:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',51 from dual union all
select 22843,'1970-W1251','1970-08-24 16:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 22844,'1970-W1252','1970-08-24 16:48:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 22845,'1970-W1253','1970-08-24 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',53 from dual union all
select 22846,'1970-W1254','1970-08-24 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',60 from dual union all
select 22847,'1970-W1255','1970-08-24 21:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 22848,'1970-W1256','1970-08-25 21:14:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',54 from dual union all
select 22849,'1970-W1257','1970-08-26 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 2285,'1954-A286','1954-05-19 00:00:00','atmospheric rocket','success',836,'1419R','Nike 286R','','','Test',1895,'',10 from dual union all
select 22850,'1970-W1258','1970-08-26 14:52:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 22851,'1970-W1259','1970-08-26 14:55:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 22852,'1970-W126','1970-01-20 17:25:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 22853,'1970-W1260','1970-08-26 15:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',66 from dual union all
select 22854,'1970-W1261','1970-08-26 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 22855,'1970-W1262','1970-08-26 23:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 22856,'1970-W1263','1970-08-27 00:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',57 from dual union all
select 22857,'1970-W1264','1970-08-27 01:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 22858,'1970-W1265','1970-08-27 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 22859,'1970-W1266','1970-08-27 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 2286,'1954-A287','1954-05-19 00:00:00','atmospheric rocket','success',836,'1394T','','','','Test',1895,'',10 from dual union all
select 22860,'1970-W1267','1970-08-27 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',64 from dual union all
select 22861,'1970-W1268','1970-08-28 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 22862,'1970-W1269','1970-08-28 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 22863,'1970-W127','1970-01-20 22:02:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',55 from dual union all
select 22864,'1970-W1270','1970-08-28 14:01:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 22865,'1970-W1271','1970-08-28 14:49:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 22866,'1970-W1272','1970-08-28 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',60 from dual union all
select 22867,'1970-W1273','1970-08-28 18:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',60 from dual union all
select 22868,'1970-W1274','1970-08-28 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',64 from dual;
insert into LAUNCH
select 22869,'1970-W1275','1970-08-28 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 2287,'1954-A288','1954-05-19 00:00:00','atmospheric rocket','success',836,'1377E','','','','Test',1895,'',10 from dual union all
select 22870,'1970-W1276','1970-08-28 21:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 22871,'1970-W1277','1970-08-31 14:43:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 22872,'1970-W1278','1970-08-31 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',61 from dual union all
select 22873,'1970-W1279','1970-08-31 16:20:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 22874,'1970-W128','1970-01-21 03:01:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 22875,'1970-W1280','1970-08-31 16:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 22876,'1970-W1281','1970-08-31 16:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 22877,'1970-W1282','1970-08-31 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 22878,'1970-W1283','1970-08-31 19:44:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',61 from dual union all
select 22879,'1970-W1284','1970-08-31 21:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 2288,'1954-A289','1954-05-20 00:00:00','military missile','success',771,'GM-12458','','','','Test',222,'',12 from dual union all
select 22880,'1970-W1285','1970-09-01 15:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',64 from dual union all
select 22881,'1970-W1286','1970-09-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 22882,'1970-W1287','1970-09-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 22883,'1970-W1288','1970-09-02 14:22:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',65 from dual union all
select 22884,'1970-W1289','1970-09-02 14:47:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 22885,'1970-W129','1970-01-21 12:24:59','suborbital rocket','success',631,'INTA H-7003','','','','Meteo',396,'',66 from dual union all
select 22886,'1970-W1290','1970-09-02 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 22887,'1970-W1291','1970-09-02 19:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',56 from dual union all
select 22888,'1970-W1292','1970-09-02 20:15:00','suborbital rocket','success',779,'','','','','Aeron',933,'',63 from dual union all
select 22889,'1970-W1293','1970-09-02 22:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 2289,'1954-A29','1954-01-21 00:00:00','atmospheric rocket','success',564,'','Turbojet study','','','Test',1821,'',5 from dual union all
select 22890,'1970-W1294','1970-09-02 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',67 from dual union all
select 22891,'1970-W1295','1970-09-03 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 22892,'1970-W1296','1970-09-03 17:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',52 from dual union all
select 22893,'1970-W1297','1970-09-03 22:51:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',52 from dual union all
select 22894,'1970-W1298','1970-09-04 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 22895,'1970-W1299','1970-09-04 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 22896,'1970-W13','1970-01-01 00:00:00','suborbital rocket','success',777,'6','','','','Test',933,'',68 from dual union all
select 22897,'1970-W130','1970-01-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 22898,'1970-W1300','1970-09-04 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 22899,'1970-W1301','1970-09-04 15:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 229,'1944-S208','1944-10-29 13:53:00','military missile','success',1470,'19486','Ma242','','','Test',661,'',90 from dual union all
select 2290,'1954-A290','1954-05-20 00:00:00','atmospheric rocket','success',836,'1390T','','','','Test',1895,'',10 from dual union all
select 22900,'1970-W1302','1970-09-04 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',52 from dual union all
select 22901,'1970-W1303','1970-09-04 21:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 22902,'1970-W1304','1970-09-05 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 22903,'1970-W1305','1970-09-07 20:15:00','suborbital rocket','success',779,'','','','','Aeron',933,'',62 from dual union all
select 22904,'1970-W1306','1970-09-07 23:35:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',276,'',64 from dual union all
select 22905,'1970-W1307','1970-09-08 15:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 22906,'1970-W1308','1970-09-08 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',59 from dual union all
select 22907,'1970-W1309','1970-09-08 17:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 22908,'1970-W131','1970-01-21 15:00:00','suborbital rocket','success',1188,'UCL 08','','','','Aeron',417,'',71 from dual union all
select 22909,'1970-W1310','1970-09-08 19:10:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',62 from dual union all
select 2291,'1954-A291','1954-05-20 00:00:00','atmospheric rocket','success',836,'1415','','','','Test',1895,'',10 from dual union all
select 22910,'1970-W1311','1970-09-08 21:41:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',54 from dual union all
select 22911,'1970-W1312','1970-09-09 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',66 from dual union all
select 22912,'1970-W1313','1970-09-09 11:00:00','suborbital rocket','success',631,'INTA H-7018','','','','Meteo',396,'',58 from dual union all
select 22913,'1970-W1314','1970-09-09 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 22914,'1970-W1315','1970-09-09 13:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 22915,'1970-W1316','1970-09-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 22916,'1970-W1317','1970-09-09 14:45:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 22917,'1970-W1318','1970-09-09 15:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',78 from dual union all
select 22918,'1970-W1319','1970-09-09 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 22919,'1970-W132','1970-01-21 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 2292,'1954-A292','1954-05-21 00:00:00','atmospheric rocket','unknown',576,'205','','','','Test',1895,'',10 from dual union all
select 22920,'1970-W1320','1970-09-09 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 22921,'1970-W1321','1970-09-09 18:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 22922,'1970-W1322','1970-09-09 20:15:00','suborbital rocket','success',779,'','','','','Aeron',933,'',61 from dual union all
select 22923,'1970-W1323','1970-09-09 21:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',51 from dual union all
select 22924,'1970-W1324','1970-09-09 23:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 22925,'1970-W1325','1970-09-10 01:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 22926,'1970-W1326','1970-09-10 15:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 22927,'1970-W1327','1970-09-10 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',63 from dual union all
select 22928,'1970-W1328','1970-09-10 21:47:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',65 from dual union all
select 22929,'1970-W1329','1970-09-11 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 2293,'1954-A293','1954-05-24 00:00:00','atmospheric rocket','unknown',576,'213','','','','Test',1895,'',10 from dual union all
select 22930,'1970-W133','1970-01-21 18:00:00','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',1307,'',62 from dual union all
select 22931,'1970-W1330','1970-09-11 13:57:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 22932,'1970-W1331','1970-09-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 22933,'1970-W1332','1970-09-11 15:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',59 from dual union all
select 22934,'1970-W1333','1970-09-11 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 22935,'1970-W1334','1970-09-11 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',617,'',59 from dual union all
select 22936,'1970-W1335','1970-09-11 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 22937,'1970-W1336','1970-09-12 01:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 22938,'1970-W1337','1970-09-12 18:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 22939,'1970-W1338','1970-09-14 13:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 2294,'1954-A294','1954-05-24 00:00:00','atmospheric rocket','unknown',576,'436','','','','Test',1895,'',10 from dual union all
select 22940,'1970-W1339','1970-09-14 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 22941,'1970-W134','1970-01-21 18:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',52 from dual union all
select 22942,'1970-W1340','1970-09-14 14:35:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 22943,'1970-W1341','1970-09-14 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 22944,'1970-W1342','1970-09-14 15:44:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 22945,'1970-W1343','1970-09-14 15:57:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 22946,'1970-W1344','1970-09-14 16:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 22947,'1970-W1345','1970-09-14 16:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 22948,'1970-W1346','1970-09-14 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 22949,'1970-W1347','1970-09-14 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 2295,'1954-A295','1954-05-24 00:00:00','atmospheric rocket','success',836,'1389T','','','','Test',1895,'',10 from dual union all
select 22950,'1970-W1348','1970-09-14 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 22951,'1970-W1349','1970-09-14 23:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',61 from dual union all
select 22952,'1970-W135','1970-01-21 20:40:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',58 from dual union all
select 22953,'1970-W1350','1970-09-15 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 22954,'1970-W1351','1970-09-15 14:24:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',61 from dual union all
select 22955,'1970-W1352','1970-09-15 14:56:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 22956,'1970-W1353','1970-09-15 15:32:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',59 from dual union all
select 22957,'1970-W1354','1970-09-15 18:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',53 from dual union all
select 22958,'1970-W1355','1970-09-15 21:22:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',61 from dual;
insert into LAUNCH
select 22959,'1970-W1356','1970-09-15 22:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',55 from dual union all
select 2296,'1954-A296','1954-05-25 00:00:00','military missile','unknown',339,'1314E','','','','Test',1895,'',30 from dual union all
select 22960,'1970-W1357','1970-09-16 11:00:00','suborbital rocket','success',631,'INTA H-7019','','','','Meteo',396,'',63 from dual union all
select 22961,'1970-W1358','1970-09-16 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 22962,'1970-W1359','1970-09-16 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',53 from dual union all
select 22963,'1970-W136','1970-01-21 20:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 22964,'1970-W1360','1970-09-16 15:13:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 22965,'1970-W1361','1970-09-16 15:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 22966,'1970-W1362','1970-09-16 16:36:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',50 from dual union all
select 22967,'1970-W1363','1970-09-16 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 22968,'1970-W1364','1970-09-16 16:56:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 22969,'1970-W1365','1970-09-16 17:21:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',67 from dual union all
select 2297,'1954-A297','1954-05-25 00:00:00','atmospheric rocket','success',836,'1414TT','','','','Test',1895,'',10 from dual union all
select 22970,'1970-W1366','1970-09-16 18:22:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',60 from dual union all
select 22971,'1970-W1367','1970-09-16 18:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 22972,'1970-W1368','1970-09-16 18:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1415,'',55 from dual union all
select 22973,'1970-W1369','1970-09-16 19:04:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',54 from dual union all
select 22974,'1970-W137','1970-01-21 21:30:00','suborbital rocket','success',1061,'','Density gauge','','','Meteo',617,'',50 from dual union all
select 22975,'1970-W1370','1970-09-16 19:14:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',66 from dual union all
select 22976,'1970-W1371','1970-09-16 19:14:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',61 from dual union all
select 22977,'1970-W1372','1970-09-16 22:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 22978,'1970-W1373','1970-09-17 00:00:00','suborbital rocket','success',88,'','Nimbus cal','','','Aeron',1821,'',60 from dual union all
select 22979,'1970-W1374','1970-09-17 00:00:00','suborbital rocket','success',88,'','Nimbus cal','','','Aeron',1821,'',60 from dual union all
select 2298,'1954-A298','1954-05-26 00:00:00','atmospheric rocket','success',863,'1249A-193','Nike 287R T48E2','','','Test',1895,'',10 from dual union all
select 22980,'1970-W1375','1970-09-17 02:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 22981,'1970-W1376','1970-09-17 15:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 22982,'1970-W1377','1970-09-17 16:29:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1821,'',57 from dual union all
select 22983,'1970-W1378','1970-09-17 16:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 22984,'1970-W1379','1970-09-17 16:57:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',59 from dual union all
select 22985,'1970-W138','1970-01-21 22:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 22986,'1970-W1380','1970-09-17 20:20:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 22987,'1970-W1381','1970-09-17 20:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',55 from dual union all
select 22988,'1970-W1382','1970-09-17 21:06:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',65 from dual union all
select 22989,'1970-W1383','1970-09-18 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 2299,'1954-A299','1954-05-26 00:00:00','atmospheric rocket','success',863,'1249A-190','Nike 289R T48E2','','','Test',1895,'',10 from dual union all
select 22990,'1970-W1384','1970-09-18 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 22991,'1970-W1385','1970-09-18 14:45:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',59 from dual union all
select 22992,'1970-W1386','1970-09-18 15:38:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 22993,'1970-W1387','1970-09-18 19:44:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',57 from dual union all
select 22994,'1970-W1388','1970-09-18 21:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 22995,'1970-W1389','1970-09-18 21:26:59','suborbital rocket','success',631,'','Met Para,Arcasonde','','','Meteo',428,'',64 from dual union all
select 22996,'1970-W139','1970-01-21 22:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 22997,'1970-W1390','1970-09-19 15:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 22998,'1970-W1391','1970-09-19 16:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',52 from dual union all
select 22999,'1970-W1392','1970-09-21 00:00:00','suborbital rocket','success',88,'','Nimbus cal','','','Aeron',1821,'',60 from dual union all
select 23,'1943-S01','1943-03-18 00:00:00','military missile','failure',8,'18','','','','Test',675,'',95 from dual union all
select 230,'1944-S209','1944-10-30 09:51:00','military missile','success',1470,'19484','Ma238','','','Test',661,'',90 from dual union all
select 2300,'1954-A30','1954-01-21 00:00:00','military missile','success',771,'GM-12422','','','','Test',222,'',12 from dual union all
select 23000,'1970-W1393','1970-09-21 14:33:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 23001,'1970-W1394','1970-09-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 23002,'1970-W1395','1970-09-21 15:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 23003,'1970-W1396','1970-09-21 15:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 23004,'1970-W1397','1970-09-21 16:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 23005,'1970-W1398','1970-09-21 16:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 23006,'1970-W1399','1970-09-21 16:52:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 23007,'1970-W14','1970-01-01 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',57 from dual union all
select 23008,'1970-W140','1970-01-22 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 23009,'1970-W1400','1970-09-21 17:33:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1821,'',64 from dual union all
select 2301,'1954-A300','1954-05-26 00:00:00','atmospheric rocket','success',836,'1301R Bell motor','Nike 288R','','','Test',1895,'',10 from dual union all
select 23010,'1970-W1401','1970-09-21 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 23011,'1970-W1402','1970-09-21 19:23:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',61 from dual union all
select 23012,'1970-W1403','1970-09-21 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 23013,'1970-W1404','1970-09-21 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 23014,'1970-W1405','1970-09-22 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 23015,'1970-W1406','1970-09-22 19:53:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',53 from dual union all
select 23016,'1970-W1407','1970-09-22 21:52:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',62 from dual union all
select 23017,'1970-W1408','1970-09-23 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 23018,'1970-W1409','1970-09-23 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 23019,'1970-W141','1970-01-22 02:57:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 2302,'1954-A301','1954-05-27 00:00:00','military missile','unknown',339,'1404R','','','','Test',1895,'',30 from dual union all
select 23020,'1970-W1410','1970-09-23 14:23:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',67 from dual union all
select 23021,'1970-W1411','1970-09-23 14:37:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 23022,'1970-W1412','1970-09-23 16:41:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',51 from dual union all
select 23023,'1970-W1413','1970-09-23 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 23024,'1970-W1414','1970-09-23 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',51 from dual union all
select 23025,'1970-W1415','1970-09-23 19:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 23026,'1970-W1416','1970-09-23 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',55 from dual union all
select 23027,'1970-W1417','1970-09-23 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 23028,'1970-W1418','1970-09-24 01:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 23029,'1970-W1419','1970-09-24 21:22:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',63 from dual union all
select 2303,'1954-A302','1954-05-27 00:00:00','atmospheric rocket','success',836,'1393T','','','','Test',1895,'',10 from dual union all
select 23030,'1970-W142','1970-01-22 05:00:00','suborbital rocket','success',761,'MT-135-P -004','','','','Aeron',839,'',53 from dual union all
select 23031,'1970-W1420','1970-09-25 11:00:00','suborbital rocket','success',631,'INTA H-7020','','','','Meteo',396,'',64 from dual union all
select 23032,'1970-W1421','1970-09-25 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 23033,'1970-W1422','1970-09-25 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 23034,'1970-W1423','1970-09-25 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 23035,'1970-W1424','1970-09-25 19:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 23036,'1970-W1425','1970-09-25 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 23037,'1970-W1426','1970-09-26 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 23038,'1970-W1427','1970-09-28 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 23039,'1970-W1428','1970-09-28 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 2304,'1954-A303','1954-05-28 00:00:00','atmospheric rocket','success',836,'1475T','','','','Test',1895,'',10 from dual union all
select 23040,'1970-W1429','1970-09-28 14:22:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 23041,'1970-W143','1970-01-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 23042,'1970-W1430','1970-09-28 15:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 23043,'1970-W1431','1970-09-28 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 23044,'1970-W1432','1970-09-28 18:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',61 from dual union all
select 23045,'1970-W1433','1970-09-29 01:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 23046,'1970-W1434','1970-09-29 18:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 23047,'1970-W1435','1970-09-29 21:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual;
insert into LAUNCH
select 23048,'1970-W1436','1970-09-29 21:52:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',64 from dual union all
select 23049,'1970-W1437','1970-09-30 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',63 from dual union all
select 2305,'1954-A304','1954-05-28 00:00:00','atmospheric rocket','success',836,'1335T','','','','Test',1895,'',10 from dual union all
select 23050,'1970-W1438','1970-09-30 11:00:00','suborbital rocket','success',631,'INTA H-7021','','','','Meteo',396,'',57 from dual union all
select 23051,'1970-W1439','1970-09-30 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 23052,'1970-W144','1970-01-22 17:46:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',55 from dual union all
select 23053,'1970-W1440','1970-09-30 14:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 23054,'1970-W1441','1970-09-30 14:42:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 23055,'1970-W1442','1970-09-30 18:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',53 from dual union all
select 23056,'1970-W1443','1970-09-30 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',65 from dual union all
select 23057,'1970-W1444','1970-10-01 02:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 23058,'1970-W1445','1970-10-01 22:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',194,'',56 from dual union all
select 23059,'1970-W1446','1970-10-02 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 2306,'1954-A305','1954-05-28 00:00:00','atmospheric rocket','success',836,'1408T','','','','Test',1895,'',10 from dual union all
select 23060,'1970-W1447','1970-10-02 13:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',52 from dual union all
select 23061,'1970-W1448','1970-10-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 23062,'1970-W1449','1970-10-02 14:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 23063,'1970-W145','1970-01-23 02:20:00','suborbital rocket','success',761,'MT-135-P -005','','','','Aeron',839,'',53 from dual union all
select 23064,'1970-W1450','1970-10-02 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 23065,'1970-W1451','1970-10-02 17:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',62 from dual union all
select 23066,'1970-W1452','1970-10-03 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 23067,'1970-W1453','1970-10-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 23068,'1970-W1454','1970-10-05 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',55 from dual union all
select 23069,'1970-W1455','1970-10-05 14:47:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 2307,'1954-A306','1954-06-02 00:00:00','atmospheric rocket','unknown',576,'582','','','','Test',1895,'',10 from dual union all
select 23070,'1970-W1456','1970-10-05 15:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 23071,'1970-W1457','1970-10-05 18:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 23072,'1970-W1458','1970-10-05 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 23073,'1970-W1459','1970-10-05 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 23074,'1970-W146','1970-01-23 15:06:00','suborbital rocket','success',1188,'UCL 09','','','','Aeron',417,'',71 from dual union all
select 23075,'1970-W1460','1970-10-06 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 23076,'1970-W1461','1970-10-06 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 23077,'1970-W1462','1970-10-06 04:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',77 from dual union all
select 23078,'1970-W1463','1970-10-06 14:56:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',50 from dual union all
select 23079,'1970-W1464','1970-10-06 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 2308,'1954-A307','1954-06-02 00:00:00','atmospheric rocket','unknown',576,'575','','','','Test',1895,'',10 from dual union all
select 23080,'1970-W1465','1970-10-06 18:40:00','suborbital rocket','unknown',1284,'AFCRL 70-1','','','','Test',276,'',73 from dual union all
select 23081,'1970-W1466','1970-10-06 21:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',64 from dual union all
select 23082,'1970-W1467','1970-10-07 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 23083,'1970-W1468','1970-10-07 11:00:00','suborbital rocket','success',631,'INTA H-7022','','','','Meteo',396,'',50 from dual union all
select 23084,'1970-W1469','1970-10-07 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 23085,'1970-W147','1970-01-23 15:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 23086,'1970-W1470','1970-10-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 23087,'1970-W1471','1970-10-07 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 23088,'1970-W1472','1970-10-07 15:06:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',66 from dual union all
select 23089,'1970-W1473','1970-10-07 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 2309,'1954-A308','1954-06-02 00:00:00','atmospheric rocket','success',836,'1346T','','','','Test',1895,'',10 from dual union all
select 23090,'1970-W1474','1970-10-07 16:00:00','suborbital rocket','unknown',1284,'AFCRL 70-2','','','','Test',276,'',72 from dual union all
select 23091,'1970-W1475','1970-10-07 16:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',54 from dual union all
select 23092,'1970-W1476','1970-10-07 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 23093,'1970-W1477','1970-10-07 18:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 23094,'1970-W1478','1970-10-08 02:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 23095,'1970-W1479','1970-10-08 13:00:00','suborbital rocket','unknown',1284,'AFCRL 70-3','','','','Test',276,'',78 from dual union all
select 23096,'1970-W148','1970-01-23 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',59 from dual union all
select 23097,'1970-W1480','1970-10-08 15:15:00','suborbital rocket','unknown',1284,'AFCRL 70-4','','','','Test',276,'',74 from dual union all
select 23098,'1970-W1481','1970-10-08 16:23:00','suborbital rocket','unknown',1284,'AFCRL 70-5','','','','Test',276,'',75 from dual union all
select 23099,'1970-W1482','1970-10-08 17:24:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 231,'1944-S21','1944-04-01 00:00:00','military missile','success',8,'','(30 in Apr)','','','Test',662,'',90 from dual union all
select 2310,'1954-A309','1954-06-02 00:00:00','atmospheric rocket','success',836,'1249B-1423','Nike 291R','','','Test',1895,'',10 from dual union all
select 23100,'1970-W1483','1970-10-08 17:55:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 23101,'1970-W1484','1970-10-08 18:59:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',64 from dual union all
select 23102,'1970-W1485','1970-10-08 22:03:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',64 from dual union all
select 23103,'1970-W1486','1970-10-09 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 23104,'1970-W1487','1970-10-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 23105,'1970-W1488','1970-10-09 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 23106,'1970-W1489','1970-10-09 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 23107,'1970-W149','1970-01-23 18:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 23108,'1970-W1490','1970-10-09 15:11:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 23109,'1970-W1491','1970-10-09 16:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 2311,'1954-A31','1954-01-22 00:00:00','military missile','failure',339,'1312E','EU-14','','','Test',1895,'',2 from dual union all
select 23110,'1970-W1492','1970-10-09 17:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 23111,'1970-W1493','1970-10-09 18:24:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',63 from dual union all
select 23112,'1970-W1494','1970-10-09 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',57 from dual union all
select 23113,'1970-W1495','1970-10-09 20:52:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 23114,'1970-W1496','1970-10-09 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 23115,'1970-W1497','1970-10-10 02:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 23116,'1970-W1498','1970-10-10 18:00:00','suborbital rocket','success',88,'NASA 15.80GM','','','','Aeronomy',1307,'',68 from dual union all
select 23117,'1970-W1499','1970-10-12 15:09:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',64 from dual union all
select 23118,'1970-W15','1970-01-01 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',52 from dual union all
select 23119,'1970-W150','1970-01-23 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 2312,'1954-A310','1954-06-02 00:00:00','atmospheric rocket','success',836,'1249B-1913','Nike 293R','','','Test',1895,'',10 from dual union all
select 23120,'1970-W1500','1970-10-12 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 23121,'1970-W1501','1970-10-13 01:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 23122,'1970-W1502','1970-10-13 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 23123,'1970-W1503','1970-10-13 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',56 from dual union all
select 23124,'1970-W1504','1970-10-13 21:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',62 from dual union all
select 23125,'1970-W1505','1970-10-14 01:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',67 from dual union all
select 23126,'1970-W1506','1970-10-14 02:25:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',69 from dual union all
select 23127,'1970-W1507','1970-10-14 03:36:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',59 from dual union all
select 23128,'1970-W1508','1970-10-14 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 23129,'1970-W1509','1970-10-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 2313,'1954-A311','1954-06-02 00:00:00','atmospheric rocket','success',836,'1465T','','','','Test',1895,'',10 from dual union all
select 23130,'1970-W151','1970-01-23 19:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 23131,'1970-W1510','1970-10-14 15:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',65 from dual union all
select 23132,'1970-W1511','1970-10-14 15:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',61 from dual union all
select 23133,'1970-W1512','1970-10-14 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 23134,'1970-W1513','1970-10-14 15:00:59','suborbital rocket','success',631,'[Not in FC]','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 23135,'1970-W1514','1970-10-14 17:17:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',62 from dual union all
select 23136,'1970-W1515','1970-10-14 17:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 23137,'1970-W1516','1970-10-14 18:09:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual;
insert into LAUNCH
select 23138,'1970-W1517','1970-10-14 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 23139,'1970-W1518','1970-10-14 19:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 2314,'1954-A312','1954-06-02 00:00:00','atmospheric rocket','success',863,'1249B-1409','Nike 290R T48E2','','','Test',1895,'',10 from dual union all
select 23140,'1970-W1519','1970-10-14 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',53 from dual union all
select 23141,'1970-W152','1970-01-23 20:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 23142,'1970-W1520','1970-10-14 20:15:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',61 from dual union all
select 23143,'1970-W1521','1970-10-14 22:15:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',59 from dual union all
select 23144,'1970-W1522','1970-10-14 23:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 23145,'1970-W1523','1970-10-15 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',57 from dual union all
select 23146,'1970-W1524','1970-10-15 15:14:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',60 from dual union all
select 23147,'1970-W1525','1970-10-15 15:27:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',50 from dual union all
select 23148,'1970-W1526','1970-10-16 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 23149,'1970-W1527','1970-10-16 13:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',51 from dual union all
select 2315,'1954-A313','1954-06-02 00:00:00','atmospheric rocket','success',863,'1249B-1303','Nike 292R T48E2','','','Test',1895,'',10 from dual union all
select 23150,'1970-W1528','1970-10-16 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 23151,'1970-W1529','1970-10-16 14:45:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 23152,'1970-W153','1970-01-23 20:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',50 from dual union all
select 23153,'1970-W1530','1970-10-16 14:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 23154,'1970-W1531','1970-10-16 15:17:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',55 from dual union all
select 23155,'1970-W1532','1970-10-16 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 23156,'1970-W1533','1970-10-16 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 23157,'1970-W1534','1970-10-16 18:02:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',54 from dual union all
select 23158,'1970-W1535','1970-10-16 19:34:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',56 from dual union all
select 23159,'1970-W1536','1970-10-16 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 2316,'1954-A315','1954-06-03 00:00:00','atmospheric rocket','success',365,'','Inlet model','','','Test',1821,'',10 from dual union all
select 23160,'1970-W1537','1970-10-17 01:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 23161,'1970-W1538','1970-10-17 18:00:00','suborbital rocket','success',88,'NASA 15.81GM','','','','Aeronomy',1307,'',62 from dual union all
select 23162,'1970-W1539','1970-10-19 14:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 23163,'1970-W154','1970-01-23 21:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 23164,'1970-W1540','1970-10-19 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 23165,'1970-W1541','1970-10-19 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 23166,'1970-W1542','1970-10-19 16:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 23167,'1970-W1543','1970-10-19 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 23168,'1970-W1544','1970-10-19 20:15:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',57 from dual union all
select 23169,'1970-W1545','1970-10-19 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 2317,'1954-A316','1954-06-03 00:00:00','military missile','failure',771,'GM-12472','','','','Test',222,'',12 from dual union all
select 23170,'1970-W1546','1970-10-20 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 23171,'1970-W1547','1970-10-20 15:56:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',51 from dual union all
select 23172,'1970-W1548','1970-10-21 13:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 23173,'1970-W1549','1970-10-21 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',53 from dual union all
select 23174,'1970-W155','1970-01-26 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 23175,'1970-W1550','1970-10-21 14:32:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',65 from dual union all
select 23176,'1970-W1551','1970-10-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 23177,'1970-W1552','1970-10-21 15:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 23178,'1970-W1553','1970-10-21 15:45:00','suborbital rocket','success',631,'','WOX-1A RSo/Para','','','Aeron',1419,'',65 from dual union all
select 23179,'1970-W1554','1970-10-21 18:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',51 from dual union all
select 2318,'1954-A317','1954-06-03 00:00:00','military missile','success',1214,'N-69A GM-3394-1','','','','Test',222,'',10 from dual union all
select 23180,'1970-W1555','1970-10-21 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',58 from dual union all
select 23181,'1970-W1556','1970-10-21 19:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 23182,'1970-W1557','1970-10-21 21:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 23183,'1970-W1558','1970-10-22 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 23184,'1970-W1559','1970-10-22 15:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',67 from dual union all
select 23185,'1970-W156','1970-01-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 23186,'1970-W1560','1970-10-22 16:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 23187,'1970-W1561','1970-10-22 21:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',50 from dual union all
select 23188,'1970-W1562','1970-10-23 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 23189,'1970-W1563','1970-10-23 14:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 2319,'1954-A318','1954-06-04 00:00:00','atmospheric rocket','success',365,'','Snark Model','','','Test',1821,'',10 from dual union all
select 23190,'1970-W1564','1970-10-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',54 from dual union all
select 23191,'1970-W1565','1970-10-23 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 23192,'1970-W1566','1970-10-23 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',50 from dual union all
select 23193,'1970-W1567','1970-10-23 20:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 23194,'1970-W1568','1970-10-23 23:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 23195,'1970-W1569','1970-10-24 01:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 23196,'1970-W157','1970-01-26 15:18:00','suborbital rocket','success',1188,'UCL 10','','','','Aeron',417,'',73 from dual union all
select 23197,'1970-W1570','1970-10-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 23198,'1970-W1571','1970-10-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 23199,'1970-W1572','1970-10-26 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 232,'1944-S210','1944-10-30 15:41:00','military missile','success',1470,'19498','Ma243','','','Test',661,'',90 from dual union all
select 2320,'1954-A319','1954-06-04 00:00:00','atmospheric rocket','unknown',576,'575','','','','Test',1895,'',10 from dual union all
select 23200,'1970-W1573','1970-10-26 17:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 23201,'1970-W1574','1970-10-26 17:07:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',54 from dual union all
select 23202,'1970-W1575','1970-10-26 18:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',63 from dual union all
select 23203,'1970-W1576','1970-10-26 18:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 23204,'1970-W1577','1970-10-26 19:49:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',54 from dual union all
select 23205,'1970-W1578','1970-10-26 20:01:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',63 from dual union all
select 23206,'1970-W1579','1970-10-26 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 23207,'1970-W158','1970-01-26 18:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 23208,'1970-W1580','1970-10-27 02:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 23209,'1970-W1581','1970-10-27 22:14:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',50 from dual union all
select 2321,'1954-A32','1954-01-25 00:00:00','atmospheric rocket','success',836,'1280T','','','','Test',1895,'',10 from dual union all
select 23210,'1970-W1582','1970-10-28 11:26:59','suborbital rocket','success',631,'INTA H-7024','','','','Meteo',396,'',61 from dual union all
select 23211,'1970-W1583','1970-10-28 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',61 from dual union all
select 23212,'1970-W1584','1970-10-28 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 23213,'1970-W1585','1970-10-28 14:13:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',66 from dual union all
select 23214,'1970-W1586','1970-10-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 23215,'1970-W1587','1970-10-28 15:40:00','suborbital rocket','success',631,'[Not in FC]','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 23216,'1970-W1588','1970-10-28 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 23217,'1970-W1589','1970-10-28 17:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',55 from dual union all
select 23218,'1970-W159','1970-01-26 20:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 23219,'1970-W1590','1970-10-28 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 2322,'1954-A320','1954-06-04 00:00:00','military missile','success',771,'GM-12454','','','','Test',222,'',12 from dual union all
select 23220,'1970-W1591','1970-10-28 18:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 23221,'1970-W1592','1970-10-28 19:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',63 from dual union all
select 23222,'1970-W1593','1970-10-28 20:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',58 from dual union all
select 23223,'1970-W1594','1970-10-28 21:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',52 from dual union all
select 23224,'1970-W1595','1970-10-28 22:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 23225,'1970-W1596','1970-10-29 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 23226,'1970-W1597','1970-10-29 16:09:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',61 from dual union all
select 23227,'1970-W1598','1970-10-29 16:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual;
insert into LAUNCH
select 23228,'1970-W1599','1970-10-29 17:10:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',62 from dual union all
select 23229,'1970-W16','1970-01-02 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',55 from dual union all
select 2323,'1954-A321','1954-06-04 00:00:00','military missile','failure',771,'GM-12471','','','','Test',222,'',12 from dual union all
select 23230,'1970-W160','1970-01-26 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 23231,'1970-W1600','1970-10-29 17:54:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',61 from dual union all
select 23232,'1970-W1601','1970-10-29 19:24:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',62 from dual union all
select 23233,'1970-W1602','1970-10-30 13:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',61 from dual union all
select 23234,'1970-W1603','1970-10-30 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 23235,'1970-W1604','1970-10-30 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 23236,'1970-W1605','1970-10-30 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',52 from dual union all
select 23237,'1970-W1606','1970-10-30 15:37:59','suborbital rocket','success',631,'[Not in FC]','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 23238,'1970-W1607','1970-10-30 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',50 from dual union all
select 23239,'1970-W1608','1970-10-30 19:52:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',58 from dual union all
select 2324,'1954-A322','1954-06-04 00:00:00','atmospheric rocket','success',836,'1344T','','','','Test',1895,'',10 from dual union all
select 23240,'1970-W1609','1970-10-30 20:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 23241,'1970-W161','1970-01-27 03:46:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 23242,'1970-W1610','1970-10-30 20:26:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',59 from dual union all
select 23243,'1970-W1611','1970-10-30 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 23244,'1970-W1612','1970-11-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 23245,'1970-W1613','1970-10-31 02:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 23246,'1970-W1614','1970-10-31 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',54 from dual union all
select 23247,'1970-W1615','1970-11-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',55 from dual union all
select 23248,'1970-W1616','1970-11-02 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 23249,'1970-W1617','1970-11-02 15:51:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 2325,'1954-A323','1954-06-07 00:00:00','atmospheric rocket','unknown',576,'180M','','','','Test',1895,'',10 from dual union all
select 23250,'1970-W1618','1970-11-02 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 23251,'1970-W1619','1970-11-02 16:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',72 from dual union all
select 23252,'1970-W162','1970-01-27 15:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 23253,'1970-W1620','1970-11-02 17:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 23254,'1970-W1621','1970-11-02 19:41:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',59 from dual union all
select 23255,'1970-W1622','1970-11-02 20:00:00','suborbital rocket','success',1061,'','','','','Meteo',617,'',63 from dual union all
select 23256,'1970-W1623','1970-11-02 20:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 23257,'1970-W1624','1970-11-03 15:43:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 23258,'1970-W1625','1970-11-03 18:59:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',61 from dual union all
select 23259,'1970-W1626','1970-11-03 21:29:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',51 from dual union all
select 2326,'1954-A324','1954-06-07 00:00:00','atmospheric rocket','unknown',576,'206M','','','','Test',1895,'',10 from dual union all
select 23260,'1970-W1627','1970-11-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 23261,'1970-W1628','1970-11-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 23262,'1970-W1629','1970-11-04 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 23263,'1970-W163','1970-01-28 11:18:00','suborbital rocket','success',631,'INTA H-7004','','','','Meteo',396,'',63 from dual union all
select 23264,'1970-W1630','1970-11-04 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 23265,'1970-W1631','1970-11-04 15:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',53 from dual union all
select 23266,'1970-W1632','1970-11-04 15:43:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 23267,'1970-W1633','1970-11-04 17:32:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',62 from dual union all
select 23268,'1970-W1634','1970-11-04 21:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',58 from dual union all
select 23269,'1970-W1635','1970-11-04 22:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 2327,'1954-A325','1954-06-07 00:00:00','atmospheric rocket','success',836,'2266A','','','','Test',1895,'',10 from dual union all
select 23270,'1970-W1636','1970-11-05 01:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 23271,'1970-W1637','1970-11-05 02:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 23272,'1970-W1638','1970-11-05 05:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',59 from dual union all
select 23273,'1970-W1639','1970-11-05 18:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',66 from dual union all
select 23274,'1970-W164','1970-01-28 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',61 from dual union all
select 23275,'1970-W1640','1970-11-06 12:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 23276,'1970-W1641','1970-11-06 14:00:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1363,'',55 from dual union all
select 23277,'1970-W1642','1970-11-06 15:20:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',59 from dual union all
select 23278,'1970-W1643','1970-11-06 15:37:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 23279,'1970-W1644','1970-11-06 16:45:00','suborbital rocket','success',88,'NASA 15.82GM','','','','Aeronomy',1363,'',60 from dual union all
select 2328,'1954-A326','1954-06-07 00:00:00','atmospheric rocket','success',836,'1356A','','','','Test',1895,'',10 from dual union all
select 23280,'1970-W1645','1970-11-06 20:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',62 from dual union all
select 23281,'1970-W1646','1970-11-06 21:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',74 from dual union all
select 23282,'1970-W1647','1970-11-06 22:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 23283,'1970-W1648','1970-11-07 02:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',57 from dual union all
select 23284,'1970-W1649','1970-11-09 13:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 23285,'1970-W165','1970-01-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 23286,'1970-W1650','1970-11-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 23287,'1970-W1651','1970-11-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 23288,'1970-W1652','1970-11-09 15:41:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 23289,'1970-W1653','1970-11-09 17:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 2329,'1954-A327','1954-06-07 00:00:00','atmospheric rocket','success',836,'2166A','','','','Test',1895,'',10 from dual union all
select 23290,'1970-W1654','1970-11-09 18:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 23291,'1970-W1655','1970-11-09 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 23292,'1970-W1656','1970-11-09 20:15:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',54 from dual union all
select 23293,'1970-W1657','1970-11-09 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 23294,'1970-W1658','1970-11-10 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 23295,'1970-W1659','1970-11-10 17:09:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',50 from dual union all
select 23296,'1970-W166','1970-01-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 23297,'1970-W1660','1970-11-10 17:42:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 23298,'1970-W1661','1970-11-10 18:55:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1363,'',53 from dual union all
select 23299,'1970-W1662','1970-11-11 00:59:59','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1363,'',57 from dual union all
select 233,'1944-S211','1944-11-01 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1728,'',90 from dual union all
select 2330,'1954-A328','1954-06-07 00:00:00','atmospheric rocket','success',836,'1340A','','','','Test',1895,'',10 from dual union all
select 23300,'1970-W1663','1970-11-12 14:40:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',57 from dual union all
select 23301,'1970-W1664','1970-11-12 15:36:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 23302,'1970-W1665','1970-11-12 15:58:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',55 from dual union all
select 23303,'1970-W1666','1970-11-12 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',52 from dual union all
select 23304,'1970-W1667','1970-11-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 23305,'1970-W1668','1970-11-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 23306,'1970-W1669','1970-11-13 16:45:00','suborbital rocket','success',88,'NASA 15.83GM','','','','Aeronomy',1363,'',60 from dual union all
select 23307,'1970-W167','1970-01-28 15:18:00','suborbital rocket','success',1188,'UCL 11','','','','Aeron',417,'',75 from dual union all
select 23308,'1970-W1670','1970-11-13 17:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',71 from dual union all
select 23309,'1970-W1671','1970-11-13 18:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 2331,'1954-A329','1954-06-08 00:00:00','military missile','unknown',339,'1316E','','','','Test',1895,'',30 from dual union all
select 23310,'1970-W1672','1970-11-13 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 23311,'1970-W1673','1970-11-13 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',50 from dual union all
select 23312,'1970-W1674','1970-11-13 20:27:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 23313,'1970-W1675','1970-11-16 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 23314,'1970-W1676','1970-11-16 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',53 from dual union all
select 23315,'1970-W1677','1970-11-16 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 23316,'1970-W1678','1970-11-16 15:37:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 23317,'1970-W1679','1970-11-16 15:45:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',64 from dual;
insert into LAUNCH
select 23318,'1970-W168','1970-01-28 18:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',60 from dual union all
select 23319,'1970-W1680','1970-11-16 16:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 2332,'1954-A33','1954-01-27 00:00:00','atmospheric rocket','success',442,'','MX-1712 Model','','','Test',1821,'',10 from dual union all
select 23320,'1970-W1681','1970-11-16 19:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 23321,'1970-W1682','1970-11-16 19:44:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',60 from dual union all
select 23322,'1970-W1683','1970-11-16 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 23323,'1970-W1684','1970-11-17 15:37:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 23324,'1970-W1685','1970-11-17 18:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1821,'',70 from dual union all
select 23325,'1970-W1686','1970-11-17 20:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1821,'',72 from dual union all
select 23326,'1970-W1687','1970-11-18 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',52 from dual union all
select 23327,'1970-W1688','1970-11-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 23328,'1970-W1689','1970-11-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 23329,'1970-W169','1970-01-28 19:42:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 2333,'1954-A330','1954-06-08 00:00:00','military missile','unknown',339,'1402R','','','','Test',1895,'',30 from dual union all
select 23330,'1970-W1690','1970-11-18 15:41:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 23331,'1970-W1691','1970-11-18 15:56:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 23332,'1970-W1692','1970-11-18 18:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 23333,'1970-W1693','1970-11-18 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 23334,'1970-W1694','1970-11-18 20:45:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',51 from dual union all
select 23335,'1970-W1695','1970-11-18 22:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 23336,'1970-W1696','1970-11-18 23:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',56 from dual union all
select 23337,'1970-W1697','1970-11-19 02:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 23338,'1970-W1698','1970-11-19 15:42:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 23339,'1970-W1699','1970-11-19 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',65 from dual union all
select 2334,'1954-A331','1954-06-08 00:00:00','atmospheric rocket','success',836,'1332T','','','','Test',1895,'',10 from dual union all
select 23340,'1970-W17','1970-01-02 15:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 23341,'1970-W170','1970-01-28 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 23342,'1970-W1700','1970-11-19 17:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 23343,'1970-W1701','1970-11-19 17:47:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',54 from dual union all
select 23344,'1970-W1702','1970-11-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 23345,'1970-W1703','1970-11-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 23346,'1970-W1704','1970-11-20 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',66 from dual union all
select 23347,'1970-W1705','1970-11-20 15:31:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',71 from dual union all
select 23348,'1970-W1706','1970-11-20 16:18:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 23349,'1970-W1707','1970-11-20 16:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 2335,'1954-A332','1954-06-09 00:00:00','military missile','success',771,'GM-12431','','','','Test',222,'',12 from dual union all
select 23350,'1970-W1708','1970-11-20 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 23351,'1970-W1709','1970-11-21 01:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 23352,'1970-W171','1970-01-28 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 23353,'1970-W1710','1970-11-21 20:45:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',63 from dual union all
select 23354,'1970-W1711','1970-11-23 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 23355,'1970-W1712','1970-11-23 14:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 23356,'1970-W1713','1970-11-23 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 23357,'1970-W1714','1970-11-23 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',64 from dual union all
select 23358,'1970-W1715','1970-11-23 16:14:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',73 from dual union all
select 23359,'1970-W1716','1970-11-23 17:52:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 2336,'1954-A333','1954-06-09 00:00:00','atmospheric rocket','success',836,'1387T','','','','Test',1895,'',10 from dual union all
select 23360,'1970-W1717','1970-11-23 19:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',61 from dual union all
select 23361,'1970-W1718','1970-11-23 19:42:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 23362,'1970-W1719','1970-11-23 20:10:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',57 from dual union all
select 23363,'1970-W172','1970-01-28 20:21:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 23364,'1970-W1720','1970-11-23 23:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 23365,'1970-W1721','1970-11-24 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 23366,'1970-W1722','1970-11-24 15:41:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 23367,'1970-W1723','1970-11-24 18:49:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',68 from dual union all
select 23368,'1970-W1724','1970-11-24 21:01:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',54 from dual union all
select 23369,'1970-W1725','1970-11-25 13:15:00','suborbital rocket','success',631,'','WOX-1A Rso/Para','','','Aeron',1419,'',70 from dual union all
select 2337,'1954-A334','1954-06-09 00:00:00','atmospheric rocket','success',836,'1348E','','','','Test',1895,'',10 from dual union all
select 23370,'1970-W1726','1970-11-25 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 23371,'1970-W1727','1970-11-25 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 23372,'1970-W1728','1970-11-25 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',59 from dual union all
select 23373,'1970-W1729','1970-11-25 15:11:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',59 from dual union all
select 23374,'1970-W173','1970-01-28 22:10:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',53 from dual union all
select 23375,'1970-W1730','1970-11-25 15:56:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',61 from dual union all
select 23376,'1970-W1731','1970-11-25 15:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',54 from dual union all
select 23377,'1970-W1732','1970-11-25 16:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 23378,'1970-W1733','1970-11-25 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 23379,'1970-W1734','1970-11-25 17:45:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 2338,'1954-A335','1954-06-09 00:00:00','atmospheric rocket','success',836,'1249B-2291','Nike 294R','','','Test',1895,'',10 from dual union all
select 23380,'1970-W1735','1970-11-25 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 23381,'1970-W1736','1970-11-25 23:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 23382,'1970-W1737','1970-11-26 00:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',54 from dual union all
select 23383,'1970-W1738','1970-11-27 14:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 23384,'1970-W1739','1970-11-27 15:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',64 from dual union all
select 23385,'1970-W174','1970-01-29 02:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 23386,'1970-W1740','1970-11-27 15:59:59','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',59 from dual union all
select 23387,'1970-W1741','1970-11-27 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 23388,'1970-W1742','1970-11-27 23:04:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',59 from dual union all
select 23389,'1970-W1743','1970-11-27 23:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 2339,'1954-A336','1954-06-09 00:00:00','atmospheric rocket','success',836,'1421R','Nike 295R','','','Test',1895,'',10 from dual union all
select 23390,'1970-W1744','1970-12-01 00:00:00','suborbital rocket','unknown',716,'','','','','Meteo',428,'',64 from dual union all
select 23391,'1970-W1745','1970-12-01 00:00:00','suborbital rocket','unknown',716,'','','','','Meteo',428,'',64 from dual union all
select 23392,'1970-W1746','1970-12-01 00:00:00','suborbital rocket','unknown',716,'','','','','Meteo',428,'',64 from dual union all
select 23393,'1970-W1747','1970-11-30 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 23394,'1970-W1748','1970-11-30 15:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',64 from dual union all
select 23395,'1970-W1749','1970-11-30 15:44:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 23396,'1970-W175','1970-01-29 03:17:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 23397,'1970-W1750','1970-11-30 18:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 23398,'1970-W1751','1970-11-30 18:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 23399,'1970-W1752','1970-11-30 18:46:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1821,'',56 from dual union all
select 234,'1944-S212','1944-11-01 00:00:00','military missile','success',1470,'','Antwerp (60/day)','','','Weapon',1728,'',90 from dual union all
select 2340,'1954-A337','1954-06-09 00:00:00','atmospheric rocket','success',836,'1243R','Nike 296R','','','Test',1895,'',10 from dual union all
select 23400,'1970-W1753','1970-11-30 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 23401,'1970-W1754','1970-11-30 20:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 23402,'1970-W1755','1970-11-30 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 23403,'1970-W1756','1970-11-30 21:10:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',57 from dual union all
select 23404,'1970-W1757','1970-12-01 01:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 23405,'1970-W1758','1970-12-01 15:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 23406,'1970-W1759','1970-12-01 16:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 23407,'1970-W176','1970-01-29 21:56:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual;
insert into LAUNCH
select 23408,'1970-W1760','1970-12-02 12:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 23409,'1970-W1761','1970-12-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 2341,'1954-A338','1954-06-10 00:00:00','atmospheric rocket','unknown',576,'262','','','','Test',1895,'',10 from dual union all
select 23410,'1970-W1762','1970-12-02 14:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 23411,'1970-W1763','1970-12-02 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 23412,'1970-W1764','1970-12-02 18:00:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 23413,'1970-W1765','1970-12-02 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 23414,'1970-W1766','1970-12-02 21:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',56 from dual union all
select 23415,'1970-W1767','1970-12-03 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 23416,'1970-W1768','1970-12-03 14:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',52 from dual union all
select 23417,'1970-W1769','1970-12-03 15:45:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 23418,'1970-W177','1970-01-29 22:37:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',56 from dual union all
select 23419,'1970-W1770','1970-12-03 17:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',70 from dual union all
select 2342,'1954-A339','1954-06-10 00:00:00','atmospheric rocket','unknown',576,'271','','','','Test',1895,'',10 from dual union all
select 23420,'1970-W1771','1970-12-03 19:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 23421,'1970-W1772','1970-12-03 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 23422,'1970-W1773','1970-12-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 23423,'1970-W1774','1970-12-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 23424,'1970-W1775','1970-12-04 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 23425,'1970-W1776','1970-12-04 15:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 23426,'1970-W1777','1970-12-04 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 23427,'1970-W1778','1970-12-04 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 23428,'1970-W1779','1970-12-04 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 23429,'1970-W178','1970-01-30 15:27:00','suborbital rocket','success',1188,'UCL 12','','','','Aeron',417,'',78 from dual union all
select 2343,'1954-A34','1954-01-28 00:00:00','military missile','unknown',339,'1360R','','','','Test',1895,'',30 from dual union all
select 23430,'1970-W1780','1970-12-04 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',57 from dual union all
select 23431,'1970-W1781','1970-12-04 23:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',60 from dual union all
select 23432,'1970-W1782','1970-12-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 23433,'1970-W1783','1970-12-07 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',64 from dual union all
select 23434,'1970-W1784','1970-12-07 16:43:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 23435,'1970-W1785','1970-12-07 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 23436,'1970-W1786','1970-12-07 21:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 23437,'1970-W1787','1970-12-08 01:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 23438,'1970-W1788','1970-12-08 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 23439,'1970-W1789','1970-12-08 15:51:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 2344,'1954-A340','1954-06-10 00:00:00','military missile','success',771,'GM-12460','','','','Test',222,'',12 from dual union all
select 23440,'1970-W179','1970-01-30 17:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 23441,'1970-W1790','1970-12-08 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 23442,'1970-W1791','1970-12-08 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 23443,'1970-W1792','1970-12-09 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',61 from dual union all
select 23444,'1970-W1793','1970-12-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 23445,'1970-W1794','1970-12-09 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 23446,'1970-W1795','1970-12-09 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 23447,'1970-W1796','1970-12-09 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 23448,'1970-W1797','1970-12-09 16:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 23449,'1970-W1798','1970-12-09 18:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 2345,'1954-A341','1954-06-10 00:00:00','military missile','success',771,'GM-12489','','','','Test',222,'',12 from dual union all
select 23450,'1970-W1799','1970-12-09 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 23451,'1970-W18','1970-01-02 16:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',55 from dual union all
select 23452,'1970-W180','1970-01-30 18:14:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',63 from dual union all
select 23453,'1970-W1800','1970-12-09 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 23454,'1970-W1801','1970-12-10 15:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 23455,'1970-W1802','1970-12-10 20:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 23456,'1970-W1803','1970-12-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 23457,'1970-W1804','1970-12-11 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 23458,'1970-W1805','1970-12-11 16:01:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',56 from dual union all
select 23459,'1970-W1806','1970-12-11 16:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 2346,'1954-A342','1954-06-10 00:00:00','atmospheric rocket','success',836,'1434A','','','','Test',1895,'',10 from dual union all
select 23460,'1970-W1807','1970-12-11 17:15:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',56 from dual union all
select 23461,'1970-W1808','1970-12-11 17:23:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',72 from dual union all
select 23462,'1970-W1809','1970-12-11 17:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',58 from dual union all
select 23463,'1970-W181','1970-01-30 19:30:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',64 from dual union all
select 23464,'1970-W1810','1970-12-11 17:57:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 23465,'1970-W1811','1970-12-11 18:09:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',59 from dual union all
select 23466,'1970-W1812','1970-12-11 18:35:59','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1821,'',61 from dual union all
select 23467,'1970-W1813','1970-12-11 19:04:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',70 from dual union all
select 23468,'1970-W1814','1970-12-11 20:06:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1821,'',60 from dual union all
select 23469,'1970-W1815','1970-12-12 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 2347,'1954-A343','1954-06-11 00:00:00','atmospheric rocket','success',836,'1808E','','','','Test',1895,'',10 from dual union all
select 23470,'1970-W1816','1970-12-14 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 23471,'1970-W1817','1970-12-14 17:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 23472,'1970-W1818','1970-12-14 18:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 23473,'1970-W1819','1970-12-14 18:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 23474,'1970-W182','1970-01-30 20:37:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',65 from dual union all
select 23475,'1970-W1820','1970-12-14 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 23476,'1970-W1821','1970-12-14 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 23477,'1970-W1822','1970-12-14 23:15:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',58 from dual union all
select 23478,'1970-W1823','1970-12-15 02:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 23479,'1970-W1824','1970-12-15 16:26:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',54 from dual union all
select 2348,'1954-A344','1954-06-11 00:00:00','atmospheric rocket','success',836,'1519E','','','','Test',1895,'',10 from dual union all
select 23480,'1970-W1825','1970-12-15 18:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 23481,'1970-W1826','1970-12-16 12:00:00','suborbital rocket','success',631,'INTA H-7025','','','','Meteo',396,'',63 from dual union all
select 23482,'1970-W1827','1970-12-16 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 23483,'1970-W1828','1970-12-16 14:51:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',69 from dual union all
select 23484,'1970-W1829','1970-12-16 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 23485,'1970-W183','1970-01-30 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 23486,'1970-W1830','1970-12-16 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 23487,'1970-W1831','1970-12-16 17:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',54 from dual union all
select 23488,'1970-W1832','1970-12-16 17:35:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',70 from dual union all
select 23489,'1970-W1833','1970-12-16 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 2349,'1954-A345','1954-06-14 00:00:00','atmospheric rocket','unknown',576,'279M','','','','Test',1895,'',10 from dual union all
select 23490,'1970-W1834','1970-12-16 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',71 from dual union all
select 23491,'1970-W1835','1970-12-16 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 23492,'1970-W1836','1970-12-16 23:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 23493,'1970-W1837','1970-12-17 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 23494,'1970-W1838','1970-12-17 15:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 23495,'1970-W1839','1970-12-17 15:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',51 from dual union all
select 23496,'1970-W184','1970-02-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 23497,'1970-W1840','1970-12-17 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 23498,'1970-W1841','1970-12-17 15:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',76 from dual;
insert into LAUNCH
select 23499,'1970-W1842','1970-12-17 17:15:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',65 from dual union all
select 235,'1944-S213','1944-11-01 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1728,'',90 from dual union all
select 2350,'1954-A346','1954-06-14 00:00:00','atmospheric rocket','unknown',576,'203M','','','','Test',1895,'',10 from dual union all
select 23500,'1970-W1843','1970-12-17 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 23501,'1970-W1844','1970-12-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 23502,'1970-W1845','1970-12-18 14:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 23503,'1970-W1846','1970-12-18 15:45:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 23504,'1970-W1847','1970-12-18 16:34:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',308,'',54 from dual union all
select 23505,'1970-W1848','1970-12-18 17:15:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',57 from dual union all
select 23506,'1970-W1849','1970-12-18 19:09:59','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',58 from dual union all
select 23507,'1970-W185','1970-02-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 23508,'1970-W1850','1970-12-18 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 23509,'1970-W1851','1970-12-19 01:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 2351,'1954-A347','1954-06-14 00:00:00','atmospheric rocket','success',836,'1388T','','','','Test',1895,'',10 from dual union all
select 23510,'1970-W1852','1970-12-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 23511,'1970-W1853','1970-12-21 15:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',57 from dual union all
select 23512,'1970-W1854','1970-12-21 15:52:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 23513,'1970-W1855','1970-12-21 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 23514,'1970-W1856','1970-12-21 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 23515,'1970-W1857','1970-12-22 01:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 23516,'1970-W1858','1970-12-22 12:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 23517,'1970-W1859','1970-12-22 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 23518,'1970-W186','1970-02-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 23519,'1970-W1860','1970-12-22 15:48:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 2352,'1954-A348','1954-06-15 00:00:00','atmospheric rocket','success',836,'1424E','','','','Test',1895,'',10 from dual union all
select 23520,'1970-W1861','1970-12-22 15:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 23521,'1970-W1862','1970-12-22 21:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 23522,'1970-W1863','1970-12-23 00:00:00','suborbital rocket','success',1061,'','','','','Meteo',617,'',61 from dual union all
select 23523,'1970-W1864','1970-12-23 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 23524,'1970-W1865','1970-12-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',54 from dual union all
select 23525,'1970-W1866','1970-12-23 15:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 23526,'1970-W1867','1970-12-23 17:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 23527,'1970-W1868','1970-12-23 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 23528,'1970-W1869','1970-12-23 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 23529,'1970-W187','1970-02-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 2353,'1954-A349','1954-06-16 00:00:00','atmospheric rocket','success',836,'1420E','','','','Test',1895,'',10 from dual union all
select 23530,'1970-W1870','1970-12-23 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',617,'',64 from dual union all
select 23531,'1970-W1871','1970-12-23 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 23532,'1970-W1872','1970-12-23 20:34:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 23533,'1970-W1873','1970-12-23 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 23534,'1970-W1874','1970-12-24 01:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 23535,'1970-W1875','1970-12-24 15:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 23536,'1970-W1876','1970-12-26 18:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',64 from dual union all
select 23537,'1970-W1877','1970-12-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 23538,'1970-W1878','1970-12-28 15:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 23539,'1970-W1879','1970-12-28 18:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 2354,'1954-A35','1954-01-28 00:00:00','atmospheric rocket','unknown',576,'400M','','','','Test',1895,'',10 from dual union all
select 23540,'1970-W188','1970-01-31 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 23541,'1970-W1880','1970-12-28 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 23542,'1970-W1881','1970-12-28 23:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',61 from dual union all
select 23543,'1970-W1882','1970-12-29 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 23544,'1970-W1883','1970-12-29 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 23545,'1970-W1884','1970-12-29 15:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 23546,'1970-W1885','1970-12-29 19:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 23547,'1970-W1886','1970-12-29 21:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 23548,'1970-W1887','1970-12-30 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 23549,'1970-W1888','1970-12-30 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 2355,'1954-A350','1954-06-16 00:00:00','atmospheric rocket','success',836,'2341R','Nike 297R','','','Test',1895,'',10 from dual union all
select 23550,'1970-W1889','1970-12-30 15:40:00','suborbital rocket','success',631,'','WOX-1A Rso/Para','','','Aeron',1419,'',75 from dual union all
select 23551,'1970-W189','1970-02-02 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 23552,'1970-W1890','1970-12-30 16:27:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 23553,'1970-W1891','1970-12-30 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 23554,'1970-W1892','1970-12-30 21:14:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',56 from dual union all
select 23555,'1970-W1893','1970-12-30 21:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 23556,'1970-W1894','1970-12-31 01:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 23557,'1970-W1895','1970-12-31 15:53:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 23558,'1970-W1896','1970-12-31 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',62 from dual union all
select 23559,'1970-W1897','1970-12-31 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 2356,'1954-A351','1954-06-16 00:00:00','atmospheric rocket','success',836,'2311R','Nike 299R','','','Test',1895,'',10 from dual union all
select 23560,'1970-W1898','1970-10-28 02:00:00','suborbital rocket','success',761,'MT135P-R8','','','','Aeron',1332,'',50 from dual union all
select 23561,'1970-W1899','1970-11-12 02:00:00','suborbital rocket','success',761,'MT135P-R6','','','','Aeron',1332,'',50 from dual union all
select 23562,'1970-W19','1970-01-02 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',64 from dual union all
select 23563,'1970-W190','1970-02-02 22:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 23564,'1970-W1900','1970-12-23 02:20:00','suborbital rocket','success',761,'MT135P-R11','','','','Aeron',1332,'',54 from dual union all
select 23565,'1970-W1901','1970-09-01 00:00:00','suborbital rocket','success',1188,'','Gan Diurnal 1','','','Meteo',450,'',60 from dual union all
select 23566,'1970-W1902','1970-09-01 00:00:00','suborbital rocket','success',1188,'','Gan Diurnal 2','','','Meteo',450,'',60 from dual union all
select 23567,'1970-W1903','1970-09-01 00:00:00','suborbital rocket','success',1188,'','Gan Diurnal 3','','','Meteo',450,'',60 from dual union all
select 23568,'1970-W1904','1970-09-01 00:00:00','suborbital rocket','success',1188,'','Gan Diurnal 4','','','Meteo',450,'',60 from dual union all
select 23569,'1970-W1905','1970-09-01 00:00:00','suborbital rocket','success',1188,'','Gan Diurnal 5','','','Meteo',450,'',60 from dual union all
select 2357,'1954-A352','1954-06-16 00:00:00','atmospheric rocket','success',836,'1385E','','','','Test',1895,'',10 from dual union all
select 23570,'1970-W1906','1970-09-01 00:00:00','suborbital rocket','success',1188,'','Gan Diurnal 6','','','Meteo',450,'',60 from dual union all
select 23571,'1970-W1907','1970-09-01 00:00:00','suborbital rocket','success',1188,'','Gan Diurnal 7','','','Meteo',450,'',60 from dual union all
select 23572,'1970-W1908','1970-09-01 00:00:00','suborbital rocket','success',1188,'','Gan Diurnal 8','','','Meteo',450,'',60 from dual union all
select 23573,'1970-W1909','1970-09-01 00:00:00','suborbital rocket','success',1188,'','Gan Diurnal 9','','','Meteo',450,'',60 from dual union all
select 23574,'1970-W191','1970-02-02 22:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 23575,'1970-W1910','1970-09-01 00:00:00','suborbital rocket','success',1188,'','Gan Diurnal 10','','','Meteo',450,'',60 from dual union all
select 23576,'1970-W1911','1970-09-01 00:00:00','suborbital rocket','success',1188,'','Gan Diurnal 11','','','Meteo',450,'',60 from dual union all
select 23577,'1970-W1912','1970-09-01 00:00:00','suborbital rocket','success',1188,'','Gan Diurnal 12','','','Meteo',450,'',60 from dual union all
select 23578,'1970-W192','1970-02-03 01:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 23579,'1970-W193','1970-02-03 17:45:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',58 from dual union all
select 2358,'1954-A353','1954-06-16 00:00:00','atmospheric rocket','success',836,'2363RR','Nike 298R','','','Test',1895,'',10 from dual union all
select 23580,'1970-W194','1970-02-03 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 23581,'1970-W195','1970-02-03 20:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 23582,'1970-W196','1970-02-03 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',52 from dual union all
select 23583,'1970-W197','1970-02-04 02:58:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 23584,'1970-W198','1970-02-04 14:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 23585,'1970-W199','1970-02-04 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',55 from dual union all
select 23586,'1970-W20','1970-01-02 20:15:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 23587,'1970-W200','1970-02-04 18:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 23588,'1970-W201','1970-02-04 18:37:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',56 from dual;
insert into LAUNCH
select 23589,'1970-W202','1970-02-04 20:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 2359,'1954-A354','1954-06-16 00:00:00','atmospheric rocket','success',836,'2481T','','','','Test',1895,'',10 from dual union all
select 23590,'1970-W203','1970-02-04 21:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',61 from dual union all
select 23591,'1970-W204','1970-02-04 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',54 from dual union all
select 23592,'1970-W205','1970-02-05 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 23593,'1970-W206','1970-02-05 01:31:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 23594,'1970-W207','1970-02-05 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 23595,'1970-W208','1970-02-05 18:01:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',59 from dual union all
select 23596,'1970-W209','1970-02-05 18:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 23597,'1970-W21','1970-01-02 21:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',61 from dual union all
select 23598,'1970-W210','1970-02-05 21:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 23599,'1970-W211','1970-02-06 14:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',61 from dual union all
select 236,'1944-S214','1944-11-01 00:00:00','military missile','success',1470,'','','','','Test',661,'',90 from dual union all
select 2360,'1954-A355','1954-06-16 00:00:00','atmospheric rocket','success',836,'1324E','','','','Test',1895,'',10 from dual union all
select 23600,'1970-W212','1970-02-06 18:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',60 from dual union all
select 23601,'1970-W213','1970-02-06 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 23602,'1970-W214','1970-02-06 21:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 23603,'1970-W215','1970-02-07 02:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 23604,'1970-W216','1970-02-07 03:29:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 23605,'1970-W217','1970-02-08 13:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 23606,'1970-W218','1970-02-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 23607,'1970-W219','1970-02-09 15:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 23608,'1970-W22','1970-01-03 01:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 23609,'1970-W220','1970-02-09 15:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 2361,'1954-A356','1954-06-16 00:00:00','atmospheric rocket','success',836,'1347T','','','','Test',1895,'',10 from dual union all
select 23610,'1970-W221','1970-02-09 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',70 from dual union all
select 23611,'1970-W222','1970-02-09 17:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 23612,'1970-W223','1970-02-09 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 23613,'1970-W224','1970-02-09 19:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 23614,'1970-W225','1970-02-09 19:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',60 from dual union all
select 23615,'1970-W226','1970-02-09 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 23616,'1970-W227','1970-02-09 20:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',52 from dual union all
select 23617,'1970-W228','1970-02-09 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 23618,'1970-W229','1970-02-09 20:34:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',70 from dual union all
select 23619,'1970-W23','1970-01-03 06:00:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 2362,'1954-A357','1954-06-17 00:00:00','military missile','unknown',339,'1405RS','High Alt','','','Test',1895,'',30 from dual union all
select 23620,'1970-W230','1970-02-09 21:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 23621,'1970-W231','1970-02-09 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',56 from dual union all
select 23622,'1970-W232','1970-02-10 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 23623,'1970-W233','1970-02-10 06:14:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',55 from dual union all
select 23624,'1970-W234','1970-02-10 15:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 23625,'1970-W235','1970-02-10 17:25:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',54 from dual union all
select 23626,'1970-W236','1970-02-10 19:15:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',51 from dual union all
select 23627,'1970-W237','1970-02-10 20:57:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 23628,'1970-W238','1970-02-11 11:00:00','suborbital rocket','success',631,'INTA H-7005','','','','Meteo',396,'',63 from dual union all
select 23629,'1970-W239','1970-02-11 13:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 2363,'1954-A358','1954-06-17 00:00:00','atmospheric rocket','unknown',576,'246C','','','','Test',1895,'',10 from dual union all
select 23630,'1970-W24','1970-01-03 06:33:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 23631,'1970-W240','1970-02-11 13:30:00','suborbital rocket','unknown',631,'SUPARCO-055/70','','','','Aeron',1419,'',63 from dual union all
select 23632,'1970-W241','1970-02-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 23633,'1970-W242','1970-02-11 15:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 23634,'1970-W243','1970-02-11 18:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',61 from dual union all
select 23635,'1970-W244','1970-02-11 19:14:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',56 from dual union all
select 23636,'1970-W245','1970-02-11 22:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 23637,'1970-W246','1970-02-12 01:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 23638,'1970-W247','1970-02-12 04:17:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 23639,'1970-W248','1970-02-12 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 2364,'1954-A359','1954-06-17 00:00:00','atmospheric rocket','unknown',576,'274CC','','','','Test',1895,'',10 from dual union all
select 23640,'1970-W249','1970-02-12 16:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 23641,'1970-W25','1970-01-03 18:00:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',54 from dual union all
select 23642,'1970-W250','1970-02-12 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',50 from dual union all
select 23643,'1970-W251','1970-02-12 23:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 23644,'1970-W252','1970-02-13 00:19:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 23645,'1970-W253','1970-02-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 23646,'1970-W254','1970-02-13 15:08:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',59 from dual union all
select 23647,'1970-W255','1970-02-13 19:23:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',61 from dual union all
select 23648,'1970-W256','1970-02-13 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',58 from dual union all
select 23649,'1970-W257','1970-02-13 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 2365,'1954-A36','1954-01-29 00:00:00','military missile','success',771,'GM-12423','','','','Test',222,'',12 from dual union all
select 23650,'1970-W258','1970-02-14 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 23651,'1970-W259','1970-02-16 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 23652,'1970-W26','1970-01-03 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 23653,'1970-W260','1970-02-16 15:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 23654,'1970-W261','1970-02-16 16:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 23655,'1970-W262','1970-02-16 18:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',66 from dual union all
select 23656,'1970-W263','1970-02-16 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 23657,'1970-W264','1970-02-16 20:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 23658,'1970-W265','1970-02-16 21:31:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',63 from dual union all
select 23659,'1970-W266','1970-02-17 00:26:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 2366,'1954-A360','1954-06-17 00:00:00','atmospheric rocket','success',836,'1402T','','','','Test',1895,'',10 from dual union all
select 23660,'1970-W267','1970-02-17 01:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 23661,'1970-W268','1970-02-17 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 23662,'1970-W269','1970-02-17 21:03:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',56 from dual union all
select 23663,'1970-W27','1970-01-03 20:01:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',55 from dual union all
select 23664,'1970-W270','1970-02-17 22:11:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',54 from dual union all
select 23665,'1970-W271','1970-02-18 02:31:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 23666,'1970-W272','1970-02-18 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',52 from dual union all
select 23667,'1970-W273','1970-02-18 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 23668,'1970-W274','1970-02-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 23669,'1970-W275','1970-02-18 18:11:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',59 from dual union all
select 2367,'1954-A361','1954-06-18 00:00:00','military missile','success',771,'GM-52-1824','','','','Test',222,'',12 from dual union all
select 23670,'1970-W276','1970-02-18 18:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 23671,'1970-W277','1970-02-18 20:43:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',61 from dual union all
select 23672,'1970-W278','1970-02-18 20:48:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',61 from dual union all
select 23673,'1970-W279','1970-02-18 21:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',51 from dual union all
select 23674,'1970-W28','1970-01-04 06:02:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 23675,'1970-W280','1970-02-19 00:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 23676,'1970-W281','1970-02-19 01:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 23677,'1970-W282','1970-02-19 02:02:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 23678,'1970-W283','1970-02-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual;
insert into LAUNCH
select 23679,'1970-W284','1970-02-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 2368,'1954-A362','1954-06-21 00:00:00','atmospheric rocket','unknown',576,'270M','','','','Test',1895,'',10 from dual union all
select 23680,'1970-W285','1970-02-20 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 23681,'1970-W286','1970-02-20 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 23682,'1970-W287','1970-02-20 23:59:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 23683,'1970-W288','1970-02-21 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 23684,'1970-W289','1970-02-24 07:20:00','suborbital rocket','success',88,'NASA 15.72UI','','','','Ionos',428,'',60 from dual union all
select 23685,'1970-W29','1970-01-04 18:02:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 23686,'1970-W290','1970-02-24 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 23687,'1970-W291','1970-02-24 15:11:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',56 from dual union all
select 23688,'1970-W292','1970-02-24 17:37:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 23689,'1970-W293','1970-02-24 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 2369,'1954-A363','1954-06-21 00:00:00','atmospheric rocket','success',836,'2424A','','','','Test',1895,'',10 from dual union all
select 23690,'1970-W294','1970-02-24 21:43:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',50 from dual union all
select 23691,'1970-W295','1970-02-25 05:00:00','suborbital rocket','success',631,'INTA H-7006','','','','Meteo',396,'',59 from dual union all
select 23692,'1970-W296','1970-02-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 23693,'1970-W297','1970-02-25 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 23694,'1970-W298','1970-02-25 15:36:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 23695,'1970-W299','1970-02-25 17:03:59','suborbital rocket','success',631,'INTA H-7007','','','','Meteo',396,'',61 from dual union all
select 23696,'1970-W30','1970-01-04 18:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',60 from dual union all
select 23697,'1970-W300','1970-02-25 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 23698,'1970-W301','1970-02-25 18:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 23699,'1970-W302','1970-02-25 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',56 from dual union all
select 237,'1944-S215','1944-11-01 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1737,'',90 from dual union all
select 2370,'1954-A364','1954-06-22 00:00:00','military missile','unknown',339,'6T','','','','Test',1895,'',30 from dual union all
select 23700,'1970-W303','1970-02-25 20:15:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',64 from dual union all
select 23701,'1970-W304','1970-02-25 20:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 23702,'1970-W305','1970-02-25 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 23703,'1970-W306','1970-02-26 02:14:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 23704,'1970-W307','1970-02-26 02:37:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 23705,'1970-W308','1970-02-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 23706,'1970-W309','1970-02-26 17:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 23707,'1970-W31','1970-01-04 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 23708,'1970-W310','1970-02-26 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 23709,'1970-W311','1970-02-26 21:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',54 from dual union all
select 2371,'1954-A365','1954-06-22 00:00:00','atmospheric rocket','success',836,'2477T','','','','Test',1895,'',10 from dual union all
select 23710,'1970-W312','1970-02-27 03:00:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 23711,'1970-W313','1970-02-27 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 23712,'1970-W314','1970-02-27 14:37:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',59 from dual union all
select 23713,'1970-W315','1970-02-27 14:42:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',65 from dual union all
select 23714,'1970-W316','1970-02-27 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',62 from dual union all
select 23715,'1970-W317','1970-02-27 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 23716,'1970-W318','1970-02-27 18:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',70 from dual union all
select 23717,'1970-W319','1970-02-27 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',54 from dual union all
select 23718,'1970-W32','1970-01-04 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',54 from dual union all
select 23719,'1970-W320','1970-02-27 21:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 2372,'1954-A366','1954-06-23 00:00:00','atmospheric rocket','unknown',576,'278M','','','','Test',1895,'',10 from dual union all
select 23720,'1970-W321','1970-03-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 23721,'1970-W322','1970-03-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 23722,'1970-W323','1970-02-28 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 23723,'1970-W324','1970-02-28 04:09:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 23724,'1970-W325','1970-03-01 17:52:00','suborbital rocket','success',88,'NASA 15.73UI','','','','Ionos',428,'',60 from dual union all
select 23725,'1970-W326','1970-03-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 23726,'1970-W327','1970-03-02 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 23727,'1970-W328','1970-03-02 16:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1470,'',62 from dual union all
select 23728,'1970-W329','1970-03-02 18:22:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',67 from dual union all
select 23729,'1970-W33','1970-01-05 06:00:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 2373,'1954-A367','1954-06-23 00:00:00','atmospheric rocket','unknown',576,'398M','','','','Test',1895,'',10 from dual union all
select 23730,'1970-W330','1970-03-02 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 23731,'1970-W331','1970-03-02 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',61 from dual union all
select 23732,'1970-W332','1970-03-03 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 23733,'1970-W333','1970-03-03 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',51 from dual union all
select 23734,'1970-W334','1970-03-03 20:36:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',52 from dual union all
select 23735,'1970-W335','1970-03-03 21:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 23736,'1970-W336','1970-03-04 00:49:59','suborbital rocket','success',1188,'INTA S-7002','','','','Meteo',396,'',66 from dual union all
select 23737,'1970-W337','1970-03-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 23738,'1970-W338','1970-03-04 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 23739,'1970-W339','1970-03-04 15:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 2374,'1954-A368','1954-06-23 00:00:00','atmospheric rocket','success',836,'2415R','Nike 351R','','','Test',1895,'',10 from dual union all
select 23740,'1970-W34','1970-01-05 14:23:00','suborbital rocket','success',1188,'UCL 01','','','','Aeron',417,'',77 from dual union all
select 23741,'1970-W340','1970-03-04 16:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 23742,'1970-W341','1970-03-04 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',56 from dual union all
select 23743,'1970-W342','1970-03-04 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 23744,'1970-W343','1970-03-04 20:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',54 from dual union all
select 23745,'1970-W344','1970-03-04 22:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 23746,'1970-W345','1970-03-05 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',57 from dual union all
select 23747,'1970-W346','1970-03-05 01:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 23748,'1970-W347','1970-03-05 21:48:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',50 from dual union all
select 23749,'1970-W348','1970-03-05 22:18:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',53 from dual union all
select 2375,'1954-A369','1954-06-23 00:00:00','atmospheric rocket','success',836,'1438A','','','','Test',1895,'',10 from dual union all
select 23750,'1970-W349','1970-03-06 18:39:00','suborbital rocket','success',88,'NASA 15.66UE','','','','Plasma',1801,'',60 from dual union all
select 23751,'1970-W35','1970-01-05 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',64 from dual union all
select 23752,'1970-W350','1970-03-06 13:45:00','suborbital rocket','success',1188,'ISRO 16.02','ISRO 16.02','TERLS-154','','Aeron',1467,'',61 from dual union all
select 23753,'1970-W351','1970-03-06 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 23754,'1970-W352','1970-03-06 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',55 from dual union all
select 23755,'1970-W353','1970-03-06 15:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 23756,'1970-W354','1970-03-06 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 23757,'1970-W355','1970-03-07 01:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 23758,'1970-W356','1970-03-07 14:30:00','suborbital rocket','success',88,'NASA 15.68NM','','','','Aeronomy',1801,'',60 from dual union all
select 23759,'1970-W357','1970-03-07 16:00:00','suborbital rocket','success',88,'NASA 15.69NM','','','','Aeronomy',1801,'',60 from dual union all
select 2376,'1954-A37','1954-01-29 00:00:00','atmospheric rocket','success',836,'1633A','','','','Test',1895,'',10 from dual union all
select 23760,'1970-W359','1970-03-07 17:45:00','suborbital rocket','success',88,'NASA 15.70NM','','','','Aeronomy',1801,'',67 from dual union all
select 23761,'1970-W36','1970-01-05 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1470,'',55 from dual union all
select 23762,'1970-W360','1970-03-07 18:57:00','suborbital rocket','success',88,'NASA 15.71NM','','','','Aeronomy',1801,'',64 from dual union all
select 23763,'1970-W361','1970-03-07 19:25:00','suborbital rocket','success',88,'NASA 15.67NM','','','','Aeronomy',1801,'',66 from dual union all
select 23764,'1970-W362','1970-03-07 20:00:00','suborbital rocket','success',88,'NASA 15.76NM','','','','Aeronomy',1801,'',60 from dual union all
select 23765,'1970-W363','1970-03-08 17:45:00','suborbital rocket','success',88,'NASA 15.78NM','','','','Aeronomy',1801,'',60 from dual union all
select 23766,'1970-W364','1970-03-08 18:06:00','suborbital rocket','success',88,'NASA 15.77NM','','','','Aeronomy',1801,'',60 from dual union all
select 23767,'1970-W365','1970-03-08 13:45:00','suborbital rocket','success',1188,'ISRO-16.003','ISRO 16.03','TERLS-155','','Aeron',1467,'',58 from dual union all
select 23768,'1970-W366','1970-03-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual;
insert into LAUNCH
select 23769,'1970-W367','1970-03-09 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 2377,'1954-A370','1954-06-23 00:00:00','atmospheric rocket','success',836,'2299R','Nike 350R','','','Test',1895,'',10 from dual union all
select 23770,'1970-W368','1970-03-09 15:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 23771,'1970-W369','1970-03-09 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 23772,'1970-W37','1970-01-05 15:34:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 23773,'1970-W370','1970-03-09 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 23774,'1970-W371','1970-03-09 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 23775,'1970-W372','1970-03-10 01:07:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 23776,'1970-W373','1970-03-10 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 23777,'1970-W374','1970-03-10 14:15:00','suborbital rocket','success',1188,'ISRO-16.004','ISRO 16.04','TERLS-159','','Aeron',1467,'',60 from dual union all
select 23778,'1970-W375','1970-03-10 15:26:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 23779,'1970-W376','1970-03-10 21:15:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',56 from dual union all
select 2378,'1954-A371','1954-06-23 00:00:00','atmospheric rocket','success',836,'1495E','','','','Test',1895,'',10 from dual union all
select 23780,'1970-W377','1970-03-11 14:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 23781,'1970-W378','1970-03-11 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',62 from dual union all
select 23782,'1970-W379','1970-03-11 15:15:00','suborbital rocket','unknown',631,'SUPARCO-056/70','','','','Aeron',1419,'',75 from dual union all
select 23783,'1970-W38','1970-01-05 16:08:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 23784,'1970-W380','1970-03-11 15:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 23785,'1970-W381','1970-03-11 16:10:00','suborbital rocket','unknown',631,'SUPARCO-057/70','','','','Aeron',1419,'',64 from dual union all
select 23786,'1970-W382','1970-03-11 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 23787,'1970-W383','1970-03-11 18:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',65 from dual union all
select 23788,'1970-W384','1970-03-11 19:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 23789,'1970-W385','1970-03-11 23:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',56 from dual union all
select 2379,'1954-A372','1954-06-23 00:00:00','atmospheric rocket','success',836,'2387R','Nike 300R','','','Test',1895,'',10 from dual union all
select 23790,'1970-W386','1970-03-12 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 23791,'1970-W387','1970-03-12 06:43:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 23792,'1970-W388','1970-03-12 13:45:00','suborbital rocket','success',1188,'ISRO-16.005','ISRO 16.05','TERLS-160','','Aeron',1467,'',66 from dual union all
select 23793,'1970-W389','1970-03-12 17:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',50 from dual union all
select 23794,'1970-W39','1970-01-05 17:48:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 23795,'1970-W390','1970-03-13 12:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 23796,'1970-W391','1970-03-13 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',63 from dual union all
select 23797,'1970-W392','1970-03-13 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',52 from dual union all
select 23798,'1970-W393','1970-03-13 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 23799,'1970-W394','1970-03-13 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 238,'1944-S216','1944-11-01 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1735,'',90 from dual union all
select 2380,'1954-A373','1954-06-23 00:00:00','atmospheric rocket','success',836,'1337E','','','','Test',1895,'',10 from dual union all
select 23800,'1970-W395','1970-03-13 20:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 23801,'1970-W396','1970-03-13 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 23802,'1970-W397','1970-03-13 21:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 23803,'1970-W398','1970-03-14 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',57 from dual union all
select 23804,'1970-W399','1970-03-14 02:29:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 23805,'1970-W40','1970-01-05 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 23806,'1970-W400','1970-03-14 13:05:00','suborbital rocket','success',1188,'ISRO-16.006','ISRO 16.06','TERLS-161','','Aeron',1467,'',63 from dual union all
select 23807,'1970-W401','1970-03-16 13:45:00','suborbital rocket','success',1188,'ISRO-16.007','ISRO 16.07','TERLS-162','','Aeron',1467,'',62 from dual union all
select 23808,'1970-W402','1970-03-16 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',57 from dual union all
select 23809,'1970-W403','1970-03-16 15:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 2381,'1954-A374','1954-06-24 00:00:00','military missile','success',771,'GM-52-1826','','','','Test',222,'',12 from dual union all
select 23810,'1970-W404','1970-03-16 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 23811,'1970-W405','1970-03-16 21:03:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',52 from dual union all
select 23812,'1970-W406','1970-03-16 21:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 23813,'1970-W407','1970-03-16 23:50:00','suborbital rocket','success',1188,'ISRO-16.008','ISRO 16.08','TERLS-163','','Aeron',1467,'',62 from dual union all
select 23814,'1970-W408','1970-03-17 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 23815,'1970-W409','1970-03-17 02:25:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 23816,'1970-W41','1970-01-05 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 23817,'1970-W410','1970-03-17 18:33:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',53 from dual union all
select 23818,'1970-W411','1970-03-18 13:45:00','suborbital rocket','success',1188,'ISRO-16.009','ISRO 16.09','TERLS-169','','Aeron',1467,'',62 from dual union all
select 23819,'1970-W412','1970-03-18 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',57 from dual union all
select 2382,'1954-A375','1954-06-24 00:00:00','atmospheric rocket','success',836,'1448E','','','','Test',1895,'',10 from dual union all
select 23820,'1970-W413','1970-03-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',55 from dual union all
select 23821,'1970-W414','1970-03-18 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 23822,'1970-W415','1970-03-18 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',58 from dual union all
select 23823,'1970-W416','1970-03-18 18:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',66 from dual union all
select 23824,'1970-W417','1970-03-19 01:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 23825,'1970-W418','1970-03-19 02:38:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 23826,'1970-W419','1970-03-19 15:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 23827,'1970-W42','1970-01-05 21:42:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 23828,'1970-W420','1970-03-19 17:07:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 23829,'1970-W421','1970-03-19 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',50 from dual union all
select 2383,'1954-A376','1954-06-25 00:00:00','atmospheric rocket','success',836,'2379T','','','','Test',1895,'',10 from dual union all
select 23830,'1970-W422','1970-03-20 13:45:00','suborbital rocket','success',1188,'ISRO-16.010','ISRO 16.10','TERLS-173','','Aeron',1467,'',68 from dual union all
select 23831,'1970-W423','1970-03-20 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',58 from dual union all
select 23832,'1970-W424','1970-03-20 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 23833,'1970-W425','1970-03-20 17:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 23834,'1970-W426','1970-03-20 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 23835,'1970-W427','1970-03-22 13:35:00','suborbital rocket','success',1188,'ISRO-16.011','ISRO 16.11','TERLS-174','','Aeron',1467,'',64 from dual union all
select 23836,'1970-W428','1970-03-23 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',56 from dual union all
select 23837,'1970-W429','1970-03-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 23838,'1970-W43','1970-01-05 22:00:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 23839,'1970-W430','1970-03-23 15:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 2384,'1954-A377','1954-06-25 00:00:00','atmospheric rocket','success',836,'1403T','','','','Test',1895,'',10 from dual union all
select 23840,'1970-W431','1970-03-23 16:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 23841,'1970-W432','1970-03-23 20:45:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',60 from dual union all
select 23842,'1970-W433','1970-03-23 21:34:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',56 from dual union all
select 23843,'1970-W434','1970-03-23 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',54 from dual union all
select 23844,'1970-W435','1970-03-23 22:24:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',61 from dual union all
select 23845,'1970-W436','1970-03-24 01:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 23846,'1970-W437','1970-03-24 13:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 23847,'1970-W438','1970-03-24 18:40:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',53 from dual union all
select 23848,'1970-W439','1970-03-24 19:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',51 from dual union all
select 23849,'1970-W44','1970-01-06 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',65 from dual union all
select 2385,'1954-A378','1954-06-25 00:00:00','atmospheric rocket','success',836,'1351T','','','','Test',1895,'',10 from dual union all
select 23850,'1970-W440','1970-03-24 20:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 23851,'1970-W441','1970-03-25 13:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 23852,'1970-W442','1970-03-25 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',61 from dual union all
select 23853,'1970-W443','1970-03-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 23854,'1970-W444','1970-03-25 16:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 23855,'1970-W445','1970-03-25 16:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 23856,'1970-W446','1970-03-25 16:52:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',64 from dual union all
select 23857,'1970-W447','1970-03-25 19:01:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',60 from dual union all
select 23858,'1970-W448','1970-03-25 19:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual;
insert into LAUNCH
select 23859,'1970-W449','1970-03-25 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 2386,'1954-A379','1954-06-28 00:00:00','atmospheric rocket','success',442,'','MX-1712 Model','','','Test',1821,'',10 from dual union all
select 23860,'1970-W45','1970-01-06 06:00:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 23861,'1970-W450','1970-03-26 01:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 23862,'1970-W451','1970-03-26 09:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 23863,'1970-W452','1970-03-26 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',53 from dual union all
select 23864,'1970-W453','1970-03-27 07:00:00','suborbital rocket','success',257,'NASA 15.54GI','ISRO-12.03','TERLS-176','','Ionos',1467,'',68 from dual union all
select 23865,'1970-W454','1970-03-27 14:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',52 from dual union all
select 23866,'1970-W455','1970-03-27 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 23867,'1970-W456','1970-03-27 16:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 23868,'1970-W457','1970-03-27 17:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 23869,'1970-W458','1970-03-27 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 2387,'1954-A38','1954-01-29 00:00:00','atmospheric rocket','success',444,'','F-102 Model','','','Test',1821,'',10 from dual union all
select 23870,'1970-W459','1970-03-28 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 23871,'1970-W46','1970-01-06 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 23872,'1970-W460','1970-03-28 16:40:00','suborbital rocket','unknown',631,'SUPARCO-058/70','','','','Aeron',1419,'',63 from dual union all
select 23873,'1970-W461','1970-03-30 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',52 from dual union all
select 23874,'1970-W462','1970-03-30 16:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',64 from dual union all
select 23875,'1970-W463','1970-03-30 17:49:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1363,'',66 from dual union all
select 23876,'1970-W464','1970-03-30 21:11:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',54 from dual union all
select 23877,'1970-W465','1970-03-31 01:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 23878,'1970-W466','1970-03-31 17:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 23879,'1970-W467','1970-03-31 19:10:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',54 from dual union all
select 2388,'1954-A380','1954-06-28 00:00:00','atmospheric rocket','success',836,'1391E','','','','Test',1895,'',10 from dual union all
select 23880,'1970-W468','1970-03-31 21:27:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',56 from dual union all
select 23881,'1970-W469','1970-04-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 23882,'1970-W47','1970-01-06 16:11:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',50 from dual union all
select 23883,'1970-W470','1970-04-01 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 23884,'1970-W471','1970-04-01 15:31:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 23885,'1970-W472','1970-04-01 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 23886,'1970-W473','1970-04-01 16:40:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',60 from dual union all
select 23887,'1970-W474','1970-04-01 16:52:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',69 from dual union all
select 23888,'1970-W475','1970-04-01 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 23889,'1970-W476','1970-04-01 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',54 from dual union all
select 2389,'1954-A381','1954-06-29 00:00:00','military missile','unknown',339,'T-7','','','','Test',1895,'',30 from dual union all
select 23890,'1970-W477','1970-04-01 21:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 23891,'1970-W478','1970-04-02 01:22:59','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',61 from dual union all
select 23892,'1970-W479','1970-04-02 01:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 23893,'1970-W48','1970-01-06 23:06:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 23894,'1970-W480','1970-04-02 19:51:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',53 from dual union all
select 23895,'1970-W481','1970-04-03 05:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 23896,'1970-W482','1970-04-03 14:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 23897,'1970-W483','1970-04-03 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 23898,'1970-W484','1970-04-03 17:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',64 from dual union all
select 23899,'1970-W485','1970-04-03 18:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',59 from dual union all
select 239,'1944-S217','1944-10-31 10:57:00','military missile','success',1470,'19488','Ma240','','','Test',661,'',90 from dual union all
select 2390,'1954-A382','1954-06-29 00:00:00','military missile','unknown',339,'1310','','','','Test',1895,'',30 from dual union all
select 23900,'1970-W486','1970-04-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 23901,'1970-W487','1970-04-03 21:33:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',61 from dual union all
select 23902,'1970-W488','1970-04-04 02:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',50 from dual union all
select 23903,'1970-W489','1970-04-06 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',52 from dual union all
select 23904,'1970-W49','1970-01-07 04:17:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 23905,'1970-W490','1970-04-06 14:59:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',62 from dual union all
select 23906,'1970-W491','1970-04-06 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 23907,'1970-W492','1970-04-06 16:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 23908,'1970-W493','1970-04-06 17:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 23909,'1970-W494','1970-04-06 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 2391,'1954-A383','1954-06-29 00:00:00','military missile','success',771,'GM-12442','','','','Test',222,'',12 from dual union all
select 23910,'1970-W495','1970-04-06 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',53 from dual union all
select 23911,'1970-W496','1970-04-06 21:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',57 from dual union all
select 23912,'1970-W497','1970-04-07 12:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 23913,'1970-W498','1970-04-07 14:27:59','suborbital rocket','success',1061,'','Starute','','','Meteo',276,'',67 from dual union all
select 23914,'1970-W499','1970-04-07 15:25:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 23915,'1970-W50','1970-01-07 14:26:18','suborbital rocket','success',1188,'UCL 02','','','','Aeron',417,'',75 from dual union all
select 23916,'1970-W500','1970-04-07 18:00:00','suborbital rocket','success',1061,'','Starute','','','Meteo',276,'',66 from dual union all
select 23917,'1970-W501','1970-04-07 20:26:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',55 from dual union all
select 23918,'1970-W502','1970-04-08 14:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 23919,'1970-W503','1970-04-08 15:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 2392,'1954-A384','1954-06-29 00:00:00','atmospheric rocket','success',836,'2376A','','','','Test',1895,'',10 from dual union all
select 23920,'1970-W504','1970-04-08 17:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 23921,'1970-W505','1970-04-08 17:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 23922,'1970-W506','1970-04-08 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 23923,'1970-W507','1970-04-08 18:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',61 from dual union all
select 23924,'1970-W508','1970-04-08 20:12:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 23925,'1970-W509','1970-04-09 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 23926,'1970-W51','1970-01-07 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',176,'',57 from dual union all
select 23927,'1970-W510','1970-04-09 15:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 23928,'1970-W511','1970-04-09 15:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 23929,'1970-W512','1970-04-09 17:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 2393,'1954-A385','1954-06-29 00:00:00','atmospheric rocket','success',836,'1353','','','','Test',1895,'',10 from dual union all
select 23930,'1970-W513','1970-04-09 18:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 23931,'1970-W514','1970-04-09 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',54 from dual union all
select 23932,'1970-W515','1970-04-09 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 23933,'1970-W516','1970-04-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 23934,'1970-W517','1970-04-10 17:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 23935,'1970-W518','1970-04-10 17:48:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 23936,'1970-W519','1970-04-10 18:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 23937,'1970-W52','1970-01-07 16:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 23938,'1970-W520','1970-04-10 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 23939,'1970-W521','1970-04-10 20:20:00','suborbital rocket','unknown',1284,'','','','','Aeron',276,'',75 from dual union all
select 2394,'1954-A386','1954-06-30 00:00:00','atmospheric rocket','unknown',576,'432M','','','','Test',1895,'',10 from dual union all
select 23940,'1970-W522','1970-04-10 21:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',57 from dual union all
select 23941,'1970-W523','1970-04-10 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 23942,'1970-W524','1970-04-11 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',56 from dual union all
select 23943,'1970-W525','1970-04-11 20:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 23944,'1970-W526','1970-04-11 21:43:00','suborbital rocket','unknown',1284,'','','','','Aeron',276,'',75 from dual union all
select 23945,'1970-W527','1970-04-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 23946,'1970-W528','1970-04-13 14:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',62 from dual union all
select 23947,'1970-W529','1970-04-13 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 23948,'1970-W53','1970-01-07 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',54 from dual;
insert into LAUNCH
select 23949,'1970-W530','1970-04-13 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 2395,'1954-A387','1954-06-30 00:00:00','atmospheric rocket','unknown',576,'437M','','','','Test',1895,'',10 from dual union all
select 23950,'1970-W531','1970-04-13 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 23951,'1970-W532','1970-04-13 19:32:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 23952,'1970-W533','1970-04-13 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 23953,'1970-W534','1970-04-13 21:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',53 from dual union all
select 23954,'1970-W535','1970-04-14 12:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 23955,'1970-W536','1970-04-14 16:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',52 from dual union all
select 23956,'1970-W537','1970-04-14 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 23957,'1970-W538','1970-04-14 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',54 from dual union all
select 23958,'1970-W539','1970-04-15 13:24:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 23959,'1970-W54','1970-01-07 18:48:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',60 from dual union all
select 2396,'1954-A388','1954-06-30 00:00:00','military missile','success',771,'GM-12435','','','','Test',222,'',12 from dual union all
select 23960,'1970-W540','1970-04-15 13:45:00','suborbital rocket','unknown',631,'SUPARCO-059/70','','','','Aeron',1419,'',75 from dual union all
select 23961,'1970-W541','1970-04-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 23962,'1970-W542','1970-04-15 14:45:00','suborbital rocket','unknown',631,'SUPARCO-060/70','','','','Aeron',1419,'',66 from dual union all
select 23963,'1970-W543','1970-04-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 23964,'1970-W544','1970-04-15 17:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 23965,'1970-W545','1970-04-15 18:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 23966,'1970-W546','1970-04-15 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 23967,'1970-W547','1970-04-15 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 23968,'1970-W548','1970-04-15 20:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 23969,'1970-W549','1970-04-15 20:53:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',60 from dual union all
select 2397,'1954-A389','1954-06-30 00:00:00','atmospheric rocket','success',836,'2478T','','','','Test',1895,'',10 from dual union all
select 23970,'1970-W55','1970-01-07 20:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',68 from dual union all
select 23971,'1970-W550','1970-04-15 23:48:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 23972,'1970-W551','1970-04-16 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 23973,'1970-W552','1970-04-16 06:39:00','suborbital rocket','success',88,'NASA 15.74UI','','','','Ionos',428,'',40 from dual union all
select 23974,'1970-W553','1970-04-16 16:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 23975,'1970-W554','1970-04-16 17:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 23976,'1970-W555','1970-04-16 17:13:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',62 from dual union all
select 23977,'1970-W556','1970-04-16 19:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',61 from dual union all
select 23978,'1970-W557','1970-04-16 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 23979,'1970-W558','1970-04-17 14:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',55 from dual union all
select 2398,'1954-A39','1954-02-01 00:00:00','atmospheric rocket','unknown',576,'402ST','','','','Test',1895,'',10 from dual union all
select 23980,'1970-W559','1970-04-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',55 from dual union all
select 23981,'1970-W56','1970-01-07 20:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',64 from dual union all
select 23982,'1970-W560','1970-04-17 17:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 23983,'1970-W561','1970-04-17 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 23984,'1970-W562','1970-04-17 18:45:59','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',63 from dual union all
select 23985,'1970-W563','1970-04-17 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',51 from dual union all
select 23986,'1970-W564','1970-04-17 19:14:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',65 from dual union all
select 23987,'1970-W565','1970-04-17 19:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 23988,'1970-W566','1970-04-17 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',54 from dual union all
select 23989,'1970-W567','1970-04-17 20:15:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',64 from dual union all
select 2399,'1954-A390','1954-06-30 00:00:00','atmospheric rocket','success',836,'2395R','Nike 353R','','','Test',1895,'',10 from dual union all
select 23990,'1970-W568','1970-04-17 21:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 23991,'1970-W569','1970-04-20 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 23992,'1970-W57','1970-01-07 21:33:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',57 from dual union all
select 23993,'1970-W570','1970-04-20 17:00:00','suborbital rocket','unknown',273,'','','','','Aeron',293,'',75 from dual union all
select 23994,'1970-W571','1970-04-20 18:02:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',56 from dual union all
select 23995,'1970-W572','1970-04-20 18:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 23996,'1970-W573','1970-04-20 19:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 23997,'1970-W574','1970-04-21 18:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',57 from dual union all
select 23998,'1970-W575','1970-04-21 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 23999,'1970-W576','1970-04-22 09:20:00','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',60 from dual union all
select 24,'1943-S02','1943-04-14 00:00:00','military missile','success',8,'20','','','','Test',675,'',95 from dual union all
select 240,'1944-S218','1944-10-31 12:05:00','military missile','success',1470,'19491','Ma245','','','Test',661,'',90 from dual union all
select 2400,'1954-A391','1954-06-30 00:00:00','atmospheric rocket','success',836,'2405R','Nike 352R','','','Test',1895,'',10 from dual union all
select 24000,'1970-W577','1970-04-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 24001,'1970-W578','1970-04-22 14:58:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',52 from dual union all
select 24002,'1970-W579','1970-04-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 24003,'1970-W58','1970-01-07 23:03:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 24004,'1970-W580','1970-04-22 17:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 24005,'1970-W581','1970-04-22 18:04:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',58 from dual union all
select 24006,'1970-W582','1970-04-22 18:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 24007,'1970-W583','1970-04-22 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 24008,'1970-W584','1970-04-22 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',65 from dual union all
select 24009,'1970-W585','1970-04-22 22:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',60 from dual union all
select 2401,'1954-A392','1954-07-01 00:00:00','atmospheric rocket','success',836,'1399T','','','','Test',1895,'',10 from dual union all
select 24010,'1970-W586','1970-04-23 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 24011,'1970-W587','1970-04-23 19:13:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',54 from dual union all
select 24012,'1970-W588','1970-04-23 20:34:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',68 from dual union all
select 24013,'1970-W589','1970-04-23 21:22:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',56 from dual union all
select 24014,'1970-W59','1970-01-08 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 24015,'1970-W590','1970-04-24 03:34:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',63 from dual union all
select 24016,'1970-W591','1970-04-24 08:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',56 from dual union all
select 24017,'1970-W592','1970-04-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 24018,'1970-W593','1970-04-24 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 24019,'1970-W594','1970-04-24 15:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 2402,'1954-A393','1954-07-02 00:00:00','military missile','unknown',339,'T-8','','','','Test',1895,'',30 from dual union all
select 24020,'1970-W595','1970-04-24 18:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 24021,'1970-W596','1970-04-24 18:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 24022,'1970-W597','1970-04-24 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 24023,'1970-W598','1970-04-24 20:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 24024,'1970-W599','1970-04-24 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 24025,'1970-W60','1970-01-08 18:37:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',56 from dual union all
select 24026,'1970-W600','1970-04-24 22:00:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',54 from dual union all
select 24027,'1970-W601','1970-04-27 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 24028,'1970-W602','1970-04-27 14:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 24029,'1970-W603','1970-04-27 15:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 2403,'1954-A394','1954-07-02 00:00:00','atmospheric rocket','success',836,'2755T','','','','Test',1895,'',10 from dual union all
select 24030,'1970-W604','1970-04-27 17:48:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 24031,'1970-W605','1970-04-27 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 24032,'1970-W606','1970-04-27 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',58 from dual union all
select 24033,'1970-W607','1970-04-27 22:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 24034,'1970-W608','1970-04-28 15:25:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',57 from dual union all
select 24035,'1970-W609','1970-04-28 17:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',56 from dual union all
select 24036,'1970-W61','1970-01-08 20:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',78 from dual union all
select 24037,'1970-W610','1970-04-28 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual;
insert into LAUNCH
select 24038,'1970-W611','1970-04-28 22:06:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',52 from dual union all
select 24039,'1970-W612','1970-04-29 12:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 2404,'1954-A395','1954-07-07 00:00:00','military missile','success',771,'GM-11067','','','','Test',222,'',12 from dual union all
select 24040,'1970-W613','1970-04-29 15:07:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',64 from dual union all
select 24041,'1970-W614','1970-04-29 16:56:59','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',428,'',61 from dual union all
select 24042,'1970-W615','1970-04-29 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 24043,'1970-W616','1970-04-29 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 24044,'1970-W617','1970-04-29 17:48:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 24045,'1970-W618','1970-04-29 17:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 24046,'1970-W619','1970-04-29 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 24047,'1970-W62','1970-01-09 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 24048,'1970-W620','1970-04-29 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',65 from dual union all
select 24049,'1970-W621','1970-04-29 23:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',60 from dual union all
select 2405,'1954-A396','1954-07-07 00:00:00','atmospheric rocket','success',836,'2752T','','','','Test',1895,'',10 from dual union all
select 24050,'1970-W622','1970-05-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 24051,'1970-W623','1970-05-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 24052,'1970-W624','1970-05-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 24053,'1970-W625','1970-05-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 24054,'1970-W626','1970-05-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 24055,'1970-W627','1970-04-30 14:17:00','suborbital rocket','success',1061,'','Starute','','','Meteo',276,'',73 from dual union all
select 24056,'1970-W628','1970-04-30 17:15:00','suborbital rocket','success',1061,'','Starute','','','Meteo',276,'',71 from dual union all
select 24057,'1970-W629','1970-04-30 17:25:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',51 from dual union all
select 24058,'1970-W63','1970-01-09 14:44:00','suborbital rocket','success',1188,'UCL 03','','','','Aeron',417,'',74 from dual union all
select 24059,'1970-W630','1970-04-30 21:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',67 from dual union all
select 2406,'1954-A397','1954-07-07 00:00:00','atmospheric rocket','success',836,'1349R','Nike 354R','','','Test',1895,'',10 from dual union all
select 24060,'1970-W631','1970-05-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 24061,'1970-W632','1970-05-01 14:21:00','suborbital rocket','success',1061,'','Starute,Exp. Sonde','','','Meteo',276,'',70 from dual union all
select 24062,'1970-W633','1970-05-01 15:38:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',55 from dual union all
select 24063,'1970-W634','1970-05-01 16:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 24064,'1970-W635','1970-05-01 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 24065,'1970-W636','1970-05-01 18:26:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 24066,'1970-W637','1970-05-01 19:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',64 from dual union all
select 24067,'1970-W638','1970-05-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 24068,'1970-W639','1970-05-04 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 24069,'1970-W64','1970-01-09 17:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 2407,'1954-A398','1954-07-07 00:00:00','atmospheric rocket','success',836,'2437R','Nike 355R','','','Test',1895,'',10 from dual union all
select 24070,'1970-W640','1970-05-04 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',57 from dual union all
select 24071,'1970-W641','1970-05-04 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 24072,'1970-W642','1970-05-04 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 24073,'1970-W643','1970-05-04 20:57:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1057,'',60 from dual union all
select 24074,'1970-W644','1970-05-05 15:41:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',54 from dual union all
select 24075,'1970-W645','1970-05-05 16:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',59 from dual union all
select 24076,'1970-W646','1970-05-05 17:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 24077,'1970-W647','1970-05-05 22:10:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',57 from dual union all
select 24078,'1970-W648','1970-05-05 23:15:00','suborbital rocket','success',1188,'INTA S-7003','','','','Meteo',396,'',57 from dual union all
select 24079,'1970-W649','1970-05-06 10:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 2408,'1954-A399','1954-07-09 00:00:00','atmospheric rocket','success',836,'2754T','','','','Test',1895,'',10 from dual union all
select 24080,'1970-W65','1970-01-09 17:15:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',60 from dual union all
select 24081,'1970-W650','1970-05-06 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 24082,'1970-W651','1970-05-06 14:36:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',65 from dual union all
select 24083,'1970-W652','1970-05-06 15:53:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 24084,'1970-W653','1970-05-06 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 24085,'1970-W654','1970-05-06 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 24086,'1970-W655','1970-05-06 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 24087,'1970-W656','1970-05-06 18:45:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',54 from dual union all
select 24088,'1970-W657','1970-05-06 18:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 24089,'1970-W658','1970-05-06 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 2409,'1954-A40','1954-02-02 00:00:00','atmospheric rocket','unknown',576,'202H','','','','Test',1895,'',10 from dual union all
select 24090,'1970-W659','1970-05-07 01:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 24091,'1970-W66','1970-01-09 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',55 from dual union all
select 24092,'1970-W660','1970-05-07 22:13:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',63 from dual union all
select 24093,'1970-W661','1970-05-08 13:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 24094,'1970-W662','1970-05-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 24095,'1970-W663','1970-05-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 24096,'1970-W664','1970-05-08 14:11:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 24097,'1970-W665','1970-05-08 18:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 24098,'1970-W666','1970-05-08 18:56:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 24099,'1970-W667','1970-05-08 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 241,'1944-S219','1944-10-31 17:31:00','military missile','success',1470,'19489','Ma244','','','Test',661,'',90 from dual union all
select 2410,'1954-A400','1954-07-09 00:00:00','atmospheric rocket','success',836,'1793E','','','','Test',1895,'',10 from dual union all
select 24100,'1970-W668','1970-05-09 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 24101,'1970-W669','1970-05-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 24102,'1970-W67','1970-01-09 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 24103,'1970-W670','1970-05-11 15:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 24104,'1970-W671','1970-05-11 16:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 24105,'1970-W672','1970-05-11 16:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 24106,'1970-W673','1970-05-11 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 24107,'1970-W674','1970-05-11 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 24108,'1970-W675','1970-05-11 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 24109,'1970-W676','1970-05-11 22:43:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 2411,'1954-A401','1954-07-09 00:00:00','atmospheric rocket','success',836,'1331E','','','','Test',1895,'',10 from dual union all
select 24110,'1970-W677','1970-05-12 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',57 from dual union all
select 24111,'1970-W678','1970-05-12 18:25:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',55 from dual union all
select 24112,'1970-W679','1970-05-12 22:43:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',52 from dual union all
select 24113,'1970-W68','1970-01-09 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 24114,'1970-W680','1970-05-13 10:44:59','suborbital rocket','success',631,'INTA H-7009','','','','Meteo',396,'',61 from dual union all
select 24115,'1970-W681','1970-05-13 13:37:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 24116,'1970-W682','1970-05-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 24117,'1970-W683','1970-05-13 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',52 from dual union all
select 24118,'1970-W684','1970-05-13 16:07:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',55 from dual union all
select 24119,'1970-W685','1970-05-13 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 2412,'1954-A402','1954-07-09 00:00:00','atmospheric rocket','success',836,'1407E','','','','Test',1895,'',10 from dual union all
select 24120,'1970-W686','1970-05-13 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 24121,'1970-W687','1970-05-13 17:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 24122,'1970-W688','1970-05-13 17:52:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',54 from dual union all
select 24123,'1970-W689','1970-05-13 17:57:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',64 from dual union all
select 24124,'1970-W69','1970-01-10 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 24125,'1970-W690','1970-05-13 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 24126,'1970-W691','1970-05-13 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',58 from dual union all
select 24127,'1970-W692','1970-05-13 19:18:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',70 from dual;
insert into LAUNCH
select 24128,'1970-W693','1970-05-13 22:27:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',57 from dual union all
select 24129,'1970-W694','1970-05-14 19:23:00','suborbital rocket','success',88,'NASA 15.57UE','','','','Plasma',1727,'',60 from dual union all
select 2413,'1954-A403','1954-07-09 00:00:00','atmospheric rocket','success',836,'1461E','','','','Test',1895,'',10 from dual union all
select 24130,'1970-W695','1970-05-14 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 24131,'1970-W696','1970-05-14 17:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',58 from dual union all
select 24132,'1970-W697','1970-05-14 20:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 24133,'1970-W698','1970-05-14 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',51 from dual union all
select 24134,'1970-W699','1970-05-14 22:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 24135,'1970-W70','1970-01-10 02:26:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 24136,'1970-W700','1970-05-15 13:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 24137,'1970-W701','1970-05-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 24138,'1970-W702','1970-05-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 24139,'1970-W703','1970-05-15 15:03:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 2414,'1954-A404','1954-07-09 00:00:00','atmospheric rocket','success',836,'1359E','','','','Test',1895,'',10 from dual union all
select 24140,'1970-W704','1970-05-15 15:19:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',66 from dual union all
select 24141,'1970-W705','1970-05-15 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 24142,'1970-W706','1970-05-15 17:06:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 24143,'1970-W707','1970-05-15 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 24144,'1970-W708','1970-05-15 19:00:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1057,'',59 from dual union all
select 24145,'1970-W709','1970-05-15 21:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',62 from dual union all
select 24146,'1970-W71','1970-01-10 11:40:00','suborbital rocket','success',631,'INTA H-7001','','','','Meteo',396,'',65 from dual union all
select 24147,'1970-W710','1970-05-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 24148,'1970-W711','1970-05-18 14:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 24149,'1970-W712','1970-05-18 14:38:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 2415,'1954-A405','1954-07-09 00:00:00','atmospheric rocket','success',836,'1464E','','','','Test',1895,'',10 from dual union all
select 24150,'1970-W713','1970-05-18 15:51:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1057,'',58 from dual union all
select 24151,'1970-W714','1970-05-18 16:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 24152,'1970-W715','1970-05-18 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 24153,'1970-W716','1970-05-18 18:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',62 from dual union all
select 24154,'1970-W717','1970-05-18 21:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',56 from dual union all
select 24155,'1970-W718','1970-05-19 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 24156,'1970-W719','1970-05-19 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',63 from dual union all
select 24157,'1970-W72','1970-01-12 14:36:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 24158,'1970-W720','1970-05-19 17:37:00','suborbital rocket','success',1061,'','Starute,Exp. Sonde','','','Meteo',276,'',64 from dual union all
select 24159,'1970-W721','1970-05-19 18:56:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',55 from dual union all
select 2416,'1954-A406','1954-07-09 00:00:00','atmospheric rocket','success',836,'1527E','','','','Test',1895,'',10 from dual union all
select 24160,'1970-W722','1970-05-19 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',53 from dual union all
select 24161,'1970-W723','1970-05-20 11:01:00','suborbital rocket','success',631,'INTA H-7010','','','','Meteo',396,'',66 from dual union all
select 24162,'1970-W724','1970-05-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 24163,'1970-W725','1970-05-20 14:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 24164,'1970-W726','1970-05-20 14:07:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',52 from dual union all
select 24165,'1970-W727','1970-05-20 14:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 24166,'1970-W728','1970-05-20 14:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 24167,'1970-W729','1970-05-20 14:12:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',53 from dual union all
select 24168,'1970-W73','1970-01-12 14:39:00','suborbital rocket','success',1188,'UCL 04','','','','Aeron',417,'',68 from dual union all
select 24169,'1970-W730','1970-05-20 14:22:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',64 from dual union all
select 2417,'1954-A407','1954-07-09 00:00:00','atmospheric rocket','success',836,'1740E','','','','Test',1895,'',10 from dual union all
select 24170,'1970-W731','1970-05-20 16:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 24171,'1970-W732','1970-05-20 16:56:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 24172,'1970-W733','1970-05-20 17:00:00','suborbital rocket','success',1061,'','Starute,Exp. Sonde','','','Meteo',276,'',70 from dual union all
select 24173,'1970-W734','1970-05-20 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 24174,'1970-W735','1970-05-20 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',60 from dual union all
select 24175,'1970-W736','1970-05-20 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 24176,'1970-W737','1970-05-20 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 24177,'1970-W738','1970-05-21 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 24178,'1970-W739','1970-05-21 14:36:00','suborbital rocket','success',1061,'','Starute,Exp. Sonde','','','Meteo',276,'',70 from dual union all
select 24179,'1970-W74','1970-01-12 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 2418,'1954-A408','1954-07-13 00:00:00','military missile','unknown',339,'T-9','','','','Test',1895,'',30 from dual union all
select 24180,'1970-W740','1970-05-21 17:12:00','suborbital rocket','success',1061,'','Starute,Exp. Sonde','','','Meteo',276,'',73 from dual union all
select 24181,'1970-W741','1970-05-21 21:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',54 from dual union all
select 24182,'1970-W742','1970-05-22 13:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',65 from dual union all
select 24183,'1970-W743','1970-05-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',55 from dual union all
select 24184,'1970-W744','1970-05-22 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 24185,'1970-W745','1970-05-22 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 24186,'1970-W746','1970-05-22 17:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1057,'',59 from dual union all
select 24187,'1970-W747','1970-05-22 18:00:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 24188,'1970-W748','1970-05-22 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 24189,'1970-W749','1970-05-22 20:30:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',71 from dual union all
select 2419,'1954-A409','1954-07-13 00:00:00','atmospheric rocket','unknown',576,'437M','','','','Test',1895,'',10 from dual union all
select 24190,'1970-W75','1970-01-12 16:14:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1363,'',61 from dual union all
select 24191,'1970-W750','1970-05-22 22:01:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',60 from dual union all
select 24192,'1970-W751','1970-05-25 06:00:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',50 from dual union all
select 24193,'1970-W752','1970-05-25 07:00:00','suborbital rocket','success',88,'ARCAS','','','','Aeron',1895,'',60 from dual union all
select 24194,'1970-W753','1970-05-25 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 24195,'1970-W754','1970-05-25 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',52 from dual union all
select 24196,'1970-W755','1970-05-25 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 24197,'1970-W756','1970-05-25 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 24198,'1970-W757','1970-05-25 17:31:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 24199,'1970-W758','1970-05-25 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 242,'1944-S22','1944-03-31 22:50:00','military missile','failure',8,'','','','','Test',662,'',90 from dual union all
select 2420,'1954-A41','1954-02-02 00:00:00','military missile','success',1214,'N-69A GM-3395','','','','Test',222,'',10 from dual union all
select 24200,'1970-W759','1970-05-25 18:12:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',61 from dual union all
select 24201,'1970-W76','1970-01-12 19:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',57 from dual union all
select 24202,'1970-W760','1970-05-25 19:14:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',60 from dual union all
select 24203,'1970-W761','1970-05-25 21:03:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',57 from dual union all
select 24204,'1970-W762','1970-05-25 21:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 24205,'1970-W763','1970-05-25 21:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',56 from dual union all
select 24206,'1970-W764','1970-05-25 22:04:00','suborbital rocket','success',88,'ARCAS','','','','Aeron',1895,'',60 from dual union all
select 24207,'1970-W765','1970-05-25 23:45:00','suborbital rocket','success',88,'ARCAS','','','','Aeron',1895,'',60 from dual union all
select 24208,'1970-W766','1970-05-26 01:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 24209,'1970-W767','1970-05-26 01:40:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',57 from dual union all
select 2421,'1954-A410','1954-07-13 00:00:00','atmospheric rocket','success',836,'2765T','','','','Test',1895,'',10 from dual union all
select 24210,'1970-W768','1970-05-26 02:40:00','suborbital rocket','success',88,'ARCAS','','','','Aeron',1895,'',60 from dual union all
select 24211,'1970-W769','1970-05-26 03:40:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',57 from dual union all
select 24212,'1970-W77','1970-01-12 21:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 24213,'1970-W770','1970-05-26 04:40:00','suborbital rocket','success',88,'ARCAS','','','','Aeron',1895,'',60 from dual union all
select 24214,'1970-W771','1970-05-26 16:14:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',53 from dual union all
select 24215,'1970-W772','1970-05-26 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',51 from dual union all
select 24216,'1970-W773','1970-05-27 09:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 24217,'1970-W774','1970-05-27 11:07:00','suborbital rocket','success',631,'INTA H-7011','','','','Meteo',396,'',65 from dual;
insert into LAUNCH
select 24218,'1970-W775','1970-05-27 12:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 24219,'1970-W776','1970-05-27 14:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 2422,'1954-A411','1954-07-14 00:00:00','atmospheric rocket','success',836,'2476T','','','','Test',1895,'',10 from dual union all
select 24220,'1970-W777','1970-05-27 14:27:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',62 from dual union all
select 24221,'1970-W778','1970-05-27 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 24222,'1970-W779','1970-05-27 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 24223,'1970-W78','1970-01-12 22:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 24224,'1970-W780','1970-05-27 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 24225,'1970-W781','1970-05-27 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 24226,'1970-W782','1970-05-27 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',54 from dual union all
select 24227,'1970-W783','1970-05-28 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 24228,'1970-W784','1970-05-28 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',50 from dual union all
select 24229,'1970-W785','1970-05-28 14:55:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',54 from dual union all
select 2423,'1954-A412','1954-07-15 00:00:00','atmospheric rocket','unknown',576,'434','','','','Test',1895,'',10 from dual union all
select 24230,'1970-W786','1970-05-28 15:01:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',53 from dual union all
select 24231,'1970-W787','1970-05-28 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',57 from dual union all
select 24232,'1970-W788','1970-05-29 18:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 24233,'1970-W789','1970-06-01 13:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',62 from dual union all
select 24234,'1970-W79','1970-01-13 01:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 24235,'1970-W790','1970-06-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 24236,'1970-W791','1970-06-01 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 24237,'1970-W792','1970-06-01 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 24238,'1970-W793','1970-06-01 17:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',54 from dual union all
select 24239,'1970-W794','1970-06-01 17:14:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',53 from dual union all
select 2424,'1954-A413','1954-07-16 00:00:00','atmospheric rocket','success',836,'2753T','','','','Test',1895,'',10 from dual union all
select 24240,'1970-W795','1970-06-01 17:42:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',56 from dual union all
select 24241,'1970-W796','1970-06-01 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 24242,'1970-W797','1970-06-01 22:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 24243,'1970-W798','1970-06-02 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 24244,'1970-W799','1970-06-02 19:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',62 from dual union all
select 24245,'1970-W80','1970-01-13 17:22:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',53 from dual union all
select 24246,'1970-W800','1970-06-02 22:04:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',50 from dual union all
select 24247,'1970-W801','1970-06-03 02:55:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',61 from dual union all
select 24248,'1970-W802','1970-06-03 11:03:00','suborbital rocket','success',1188,'INTA S-7004','','','','Meteo',396,'',63 from dual union all
select 24249,'1970-W803','1970-06-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 2425,'1954-A414','1954-07-16 21:58:00','suborbital rocket','failure',370,'','SUI 25','','','Ionos/Aeron?',61,'WYG-279',11 from dual union all
select 24250,'1970-W804','1970-06-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 24251,'1970-W805','1970-06-03 14:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 24252,'1970-W806','1970-06-03 14:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',65 from dual union all
select 24253,'1970-W807','1970-06-03 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 24254,'1970-W808','1970-06-03 16:52:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 24255,'1970-W809','1970-06-03 17:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 24256,'1970-W81','1970-01-14 01:55:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 24257,'1970-W810','1970-06-03 17:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',53 from dual union all
select 24258,'1970-W811','1970-06-03 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 24259,'1970-W812','1970-06-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',65 from dual union all
select 2426,'1954-A415','1954-07-19 00:00:00','atmospheric rocket','success',836,'1754E','','','','Test',1895,'',10 from dual union all
select 24260,'1970-W813','1970-06-03 20:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 24261,'1970-W814','1970-06-03 20:58:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',52 from dual union all
select 24262,'1970-W815','1970-06-03 22:24:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 24263,'1970-W816','1970-06-04 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 24264,'1970-W817','1970-06-05 02:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 24265,'1970-W818','1970-06-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 24266,'1970-W819','1970-06-05 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 24267,'1970-W82','1970-01-14 11:35:00','suborbital rocket','success',631,'INTA H-7002','','','','Meteo',396,'',64 from dual union all
select 24268,'1970-W820','1970-06-05 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 24269,'1970-W821','1970-06-05 14:44:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 2427,'1954-A416','1954-07-19 00:00:00','atmospheric rocket','success',836,'1440E','','','','Test',1895,'',10 from dual union all
select 24270,'1970-W822','1970-06-05 15:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 24271,'1970-W823','1970-06-05 15:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',53 from dual union all
select 24272,'1970-W824','1970-06-05 15:59:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',57 from dual union all
select 24273,'1970-W825','1970-06-05 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 24274,'1970-W826','1970-06-05 20:40:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',61 from dual union all
select 24275,'1970-W827','1970-06-05 21:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 24276,'1970-W828','1970-06-08 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 24277,'1970-W829','1970-06-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 24278,'1970-W83','1970-01-14 13:00:00','suborbital rocket','unknown',631,'SUPARCO-052/69','','','','Aeron',1419,'',75 from dual union all
select 24279,'1970-W830','1970-06-08 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 2428,'1954-A417','1954-07-19 16:00:00','suborbital rocket','success',370,'','NRL Rockoon 7','','','Aeron',66,'WYG-279',88 from dual union all
select 24280,'1970-W831','1970-06-08 15:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 24281,'1970-W832','1970-06-08 16:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 24282,'1970-W833','1970-06-08 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',64 from dual union all
select 24283,'1970-W834','1970-06-08 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',62 from dual union all
select 24284,'1970-W835','1970-06-08 22:17:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 24285,'1970-W836','1970-06-09 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 24286,'1970-W837','1970-06-09 13:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 24287,'1970-W838','1970-06-09 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 24288,'1970-W839','1970-06-09 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 24289,'1970-W84','1970-01-14 14:00:00','suborbital rocket','unknown',631,'SUPARCO-053/69','','','','Aeron',1419,'',75 from dual union all
select 2429,'1954-A418','1954-07-19 20:30:00','suborbital rocket','success',370,'','SUI 26','','','Aeron',72,'WYG-279',43 from dual union all
select 24290,'1970-W840','1970-06-09 19:11:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',60 from dual union all
select 24291,'1970-W841','1970-06-09 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',52 from dual union all
select 24292,'1970-W842','1970-06-10 11:00:00','suborbital rocket','success',631,'INTA H-7012','','','','Meteo',396,'',60 from dual union all
select 24293,'1970-W843','1970-06-10 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 24294,'1970-W844','1970-06-10 13:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 24295,'1970-W845','1970-06-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 24296,'1970-W846','1970-06-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 24297,'1970-W847','1970-06-10 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 24298,'1970-W848','1970-06-10 16:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 24299,'1970-W849','1970-06-10 18:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 243,'1944-S220','1944-11-01 10:27:00','military missile','success',1470,'19504','Ma246','','','Test',661,'',90 from dual union all
select 2430,'1954-A419','1954-07-20 00:00:00','atmospheric rocket','unknown',576,'438','','','','Test',1895,'',10 from dual union all
select 24300,'1970-W85','1970-01-14 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',67 from dual union all
select 24301,'1970-W850','1970-06-10 19:30:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',50 from dual union all
select 24302,'1970-W851','1970-06-10 22:25:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',59 from dual union all
select 24303,'1970-W852','1970-06-11 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 24304,'1970-W853','1970-06-11 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 24305,'1970-W854','1970-06-11 15:45:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 24306,'1970-W855','1970-06-11 16:48:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 24307,'1970-W856','1970-06-11 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual;
insert into LAUNCH
select 24308,'1970-W857','1970-06-11 22:09:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',54 from dual union all
select 24309,'1970-W858','1970-06-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 2431,'1954-A42','1954-02-03 00:00:00','atmospheric rocket','success',835,'','Navaho Model','','','Test',1821,'',10 from dual union all
select 24310,'1970-W859','1970-06-12 16:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 24311,'1970-W86','1970-01-14 15:15:00','suborbital rocket','unknown',631,'SUPARCO-054/69','','','','Aeron',1419,'',56 from dual union all
select 24312,'1970-W860','1970-06-12 16:40:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',61 from dual union all
select 24313,'1970-W861','1970-06-12 17:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 24314,'1970-W862','1970-06-12 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',58 from dual union all
select 24315,'1970-W863','1970-06-13 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 24316,'1970-W864','1970-06-15 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 24317,'1970-W865','1970-06-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 24318,'1970-W866','1970-06-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 24319,'1970-W867','1970-06-15 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 2432,'1954-A420','1954-07-20 00:00:00','atmospheric rocket','unknown',576,'413','','','','Test',1895,'',10 from dual union all
select 24320,'1970-W868','1970-06-15 16:13:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 24321,'1970-W869','1970-06-15 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',59 from dual union all
select 24322,'1970-W87','1970-01-14 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',51 from dual union all
select 24323,'1970-W870','1970-06-15 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',64 from dual union all
select 24324,'1970-W871','1970-06-15 22:22:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 24325,'1970-W872','1970-06-16 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 24326,'1970-W873','1970-06-16 15:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',71 from dual union all
select 24327,'1970-W874','1970-06-16 17:13:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',65 from dual union all
select 24328,'1970-W875','1970-06-16 17:40:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',68 from dual union all
select 24329,'1970-W876','1970-06-16 19:30:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 2433,'1954-A421','1954-07-20 00:00:00','military missile','success',573,'2/A-16','XSSM-A-16','','','Test',1863,'',35 from dual union all
select 24330,'1970-W877','1970-06-16 21:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 24331,'1970-W878','1970-06-16 22:21:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',67 from dual union all
select 24332,'1970-W879','1970-06-17 11:15:00','suborbital rocket','success',631,'INTA H-7013','','','','Meteo',396,'',62 from dual union all
select 24333,'1970-W88','1970-01-14 16:54:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',63 from dual union all
select 24334,'1970-W880','1970-06-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 24335,'1970-W881','1970-06-17 14:50:00','suborbital rocket','unknown',631,'SUPARCO-062/70','','','','Aeron',1419,'',64 from dual union all
select 24336,'1970-W882','1970-06-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 24337,'1970-W883','1970-06-17 15:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 24338,'1970-W884','1970-06-17 15:37:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',68 from dual union all
select 24339,'1970-W885','1970-06-17 16:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 2434,'1954-A422','1954-07-20 00:00:00','military missile','success',771,'GM-12459','','','','Test',222,'',12 from dual union all
select 24340,'1970-W886','1970-06-17 16:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',54 from dual union all
select 24341,'1970-W887','1970-06-17 16:51:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 24342,'1970-W888','1970-06-17 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',61 from dual union all
select 24343,'1970-W889','1970-06-17 17:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',54 from dual union all
select 24344,'1970-W89','1970-01-14 17:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 24345,'1970-W890','1970-06-17 17:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 24346,'1970-W891','1970-06-17 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 24347,'1970-W892','1970-06-17 18:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 24348,'1970-W893','1970-06-17 18:46:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',64 from dual union all
select 24349,'1970-W894','1970-06-17 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 2435,'1954-A423','1954-07-20 00:00:00','atmospheric rocket','success',836,'1406E','','','','Test',1895,'',10 from dual union all
select 24350,'1970-W895','1970-06-17 19:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',72 from dual union all
select 24351,'1970-W896','1970-06-18 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 24352,'1970-W897','1970-06-18 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',73 from dual union all
select 24353,'1970-W898','1970-06-18 18:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',72 from dual union all
select 24354,'1970-W899','1970-06-18 19:08:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 24355,'1970-W90','1970-01-14 18:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',76 from dual union all
select 24356,'1970-W900','1970-06-18 19:25:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',52 from dual union all
select 24357,'1970-W901','1970-06-18 20:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 24358,'1970-W902','1970-06-18 21:26:00','suborbital rocket','success',88,'NASA 15.79GM','','','','Aeronomy',1727,'',60 from dual union all
select 24359,'1970-W903','1970-06-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 2436,'1954-A424','1954-07-20 00:00:00','atmospheric rocket','success',836,'1428E','','','','Test',1895,'',10 from dual union all
select 24360,'1970-W904','1970-06-19 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 24361,'1970-W905','1970-06-19 14:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 24362,'1970-W906','1970-06-19 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',64 from dual union all
select 24363,'1970-W907','1970-06-19 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',52 from dual union all
select 24364,'1970-W908','1970-06-19 17:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 24365,'1970-W909','1970-06-19 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',51 from dual union all
select 24366,'1970-W91','1970-01-14 18:12:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',58 from dual union all
select 24367,'1970-W910','1970-06-19 22:24:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',56 from dual union all
select 24368,'1970-W911','1970-06-20 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 24369,'1970-W912','1970-06-20 11:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 2437,'1954-A425','1954-07-21 00:00:00','atmospheric rocket','success',836,'2767T','','','','Test',1895,'',10 from dual union all
select 24370,'1970-W913','1970-06-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 24371,'1970-W914','1970-06-22 14:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 24372,'1970-W915','1970-06-22 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',61 from dual union all
select 24373,'1970-W916','1970-06-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 24374,'1970-W917','1970-06-22 15:54:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 24375,'1970-W918','1970-06-22 16:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',51 from dual union all
select 24376,'1970-W919','1970-06-22 16:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',52 from dual union all
select 24377,'1970-W92','1970-01-14 18:26:59','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 24378,'1970-W920','1970-06-22 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',58 from dual union all
select 24379,'1970-W921','1970-06-22 22:07:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',53 from dual union all
select 2438,'1954-A426','1954-07-21 00:00:00','atmospheric rocket','success',836,'2751T','','','','Test',1895,'',10 from dual union all
select 24380,'1970-W922','1970-06-23 01:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 24381,'1970-W923','1970-06-23 14:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',57 from dual union all
select 24382,'1970-W924','1970-06-23 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 24383,'1970-W925','1970-06-23 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',57 from dual union all
select 24384,'1970-W926','1970-06-23 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',55 from dual union all
select 24385,'1970-W927','1970-06-24 10:55:59','suborbital rocket','success',631,'INTA H-7014','','','','Meteo',396,'',63 from dual union all
select 24386,'1970-W928','1970-06-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',54 from dual union all
select 24387,'1970-W929','1970-06-24 14:20:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',60 from dual union all
select 24388,'1970-W93','1970-01-14 18:30:00','suborbital rocket','success',1061,'','Met Sphere,STS','','','Meteo',1895,'',62 from dual union all
select 24389,'1970-W930','1970-06-24 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 2439,'1954-A427','1954-07-21 00:00:00','military missile','success',1214,'N-69A GM-3392-1','','','','Test',222,'',10 from dual union all
select 24390,'1970-W931','1970-06-24 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 24391,'1970-W932','1970-06-24 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',55 from dual union all
select 24392,'1970-W933','1970-06-24 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',75 from dual union all
select 24393,'1970-W934','1970-06-24 18:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',62 from dual union all
select 24394,'1970-W935','1970-06-24 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',65 from dual union all
select 24395,'1970-W936','1970-06-24 20:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 24396,'1970-W937','1970-06-24 20:54:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',67 from dual union all
select 24397,'1970-W938','1970-06-24 22:02:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',52 from dual union all
select 24398,'1970-W939','1970-06-25 15:26:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',52 from dual;
insert into LAUNCH
select 24399,'1970-W94','1970-01-15 00:22:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',55 from dual union all
select 244,'1944-S221','1944-11-01 16:20:00','military missile','success',1470,'19490','Ma247','','','Test',661,'',90 from dual union all
select 2440,'1954-A428','1954-07-21 20:49:00','suborbital rocket','failure',370,'','SUI 29','','','Ionos/Aeron?',75,'WYG-279',40 from dual union all
select 24400,'1970-W940','1970-06-25 15:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 24401,'1970-W941','1970-06-25 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 24402,'1970-W942','1970-06-25 21:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 24403,'1970-W943','1970-06-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',54 from dual union all
select 24404,'1970-W944','1970-06-26 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',66 from dual union all
select 24405,'1970-W945','1970-06-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 24406,'1970-W946','1970-06-26 18:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',617,'',57 from dual union all
select 24407,'1970-W947','1970-06-26 18:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',52 from dual union all
select 24408,'1970-W948','1970-06-26 19:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 24409,'1970-W949','1970-06-26 19:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',62 from dual union all
select 2441,'1954-A429','1954-07-22 00:00:00','military missile','unknown',339,'1328E','','','','Test',1895,'',30 from dual union all
select 24410,'1970-W95','1970-01-15 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 24411,'1970-W950','1970-06-29 13:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',50 from dual union all
select 24412,'1970-W951','1970-06-29 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 24413,'1970-W952','1970-06-29 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',68 from dual union all
select 24414,'1970-W953','1970-06-29 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 24415,'1970-W954','1970-06-29 15:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 24416,'1970-W955','1970-06-29 16:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 24417,'1970-W956','1970-06-29 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',62 from dual union all
select 24418,'1970-W957','1970-06-29 23:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 24419,'1970-W958','1970-06-30 01:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 2442,'1954-A43','1954-02-04 00:00:00','military missile','success',771,'GM-12425','','','','Test',222,'',12 from dual union all
select 24420,'1970-W959','1970-06-30 03:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 24421,'1970-W96','1970-01-15 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',72 from dual union all
select 24422,'1970-W960','1970-06-30 14:56:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',56 from dual union all
select 24423,'1970-W961','1970-07-01 12:00:00','suborbital rocket','success',631,'INTA H-7015','','','','Meteo',396,'',62 from dual union all
select 24424,'1970-W962','1970-07-01 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 24425,'1970-W963','1970-07-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 24426,'1970-W964','1970-07-01 14:25:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',63 from dual union all
select 24427,'1970-W965','1970-07-01 14:47:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',50 from dual union all
select 24428,'1970-W966','1970-07-01 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 24429,'1970-W967','1970-07-01 15:04:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',65 from dual union all
select 2443,'1954-A430','1954-07-22 00:00:00','military missile','success',771,'GM-11070','','','','Test',222,'',12 from dual union all
select 24430,'1970-W968','1970-07-01 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 24431,'1970-W969','1970-07-01 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 24432,'1970-W97','1970-01-15 19:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 24433,'1970-W970','1970-07-01 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',56 from dual union all
select 24434,'1970-W971','1970-07-01 20:12:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',59 from dual union all
select 24435,'1970-W972','1970-07-01 21:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 24436,'1970-W973','1970-07-02 00:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',57 from dual union all
select 24437,'1970-W974','1970-07-02 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 24438,'1970-W975','1970-07-02 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1363,'',65 from dual union all
select 24439,'1970-W976','1970-07-02 15:47:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',61 from dual union all
select 2444,'1954-A431','1954-07-22 00:00:00','atmospheric rocket','success',836,'1426E','','','','Test',1895,'',10 from dual union all
select 24440,'1970-W977','1970-07-02 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',67 from dual union all
select 24441,'1970-W978','1970-07-02 22:00:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',52 from dual union all
select 24442,'1970-W979','1970-07-04 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',54 from dual union all
select 24443,'1970-W98','1970-01-15 21:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',617,'',63 from dual union all
select 24444,'1970-W980','1970-07-06 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 24445,'1970-W981','1970-07-06 14:21:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 24446,'1970-W982','1970-07-06 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 24447,'1970-W983','1970-07-06 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 24448,'1970-W984','1970-07-06 16:30:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',65 from dual union all
select 24449,'1970-W985','1970-07-06 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',59 from dual union all
select 2445,'1954-A432','1954-07-22 00:00:00','atmospheric rocket','success',836,'2779T','','','','Test',1895,'',10 from dual union all
select 24450,'1970-W986','1970-07-06 21:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',54 from dual union all
select 24451,'1970-W987','1970-07-07 15:33:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',50 from dual union all
select 24452,'1970-W988','1970-07-07 23:02:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',50 from dual union all
select 24453,'1970-W989','1970-07-08 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 24454,'1970-W99','1970-01-16 16:53:00','suborbital rocket','success',88,'NASA 15.56UE','','','','Plasma',1821,'',60 from dual union all
select 24455,'1970-W990','1970-07-08 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 24456,'1970-W991','1970-07-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 24457,'1970-W992','1970-07-08 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 24458,'1970-W993','1970-07-08 15:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 24459,'1970-W994','1970-07-08 16:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 2446,'1954-A433','1954-07-22 00:00:00','atmospheric rocket','success',836,'1410A','','','','Test',1895,'',10 from dual union all
select 24460,'1970-W995','1970-07-08 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 24461,'1970-W996','1970-07-08 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 24462,'1970-W997','1970-07-08 18:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 24463,'1970-W998','1970-07-08 21:42:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',855,'',58 from dual union all
select 24464,'1970-W999','1970-07-08 21:59:59','suborbital rocket','success',1188,'INTA S-7005','','','','Meteo',396,'',60 from dual union all
select 24465,'1970-Y01','1970-01-06 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',91 from dual union all
select 24466,'1970-Y02','1970-01-07 00:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',93 from dual union all
select 24467,'1970-Y03','1970-01-07 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 24468,'1970-Y04','1970-01-07 20:25:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',89 from dual union all
select 24469,'1970-Y05','1970-01-07 22:18:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',87 from dual union all
select 2447,'1954-A434','1954-07-23 00:00:00','atmospheric rocket','unknown',576,'462','','','','Test',1895,'',10 from dual union all
select 24470,'1970-Y06','1970-01-08 18:20:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',81 from dual union all
select 24471,'1970-Y07','1970-01-08 20:34:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',86 from dual union all
select 24472,'1970-Y08','1970-01-08 22:39:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',85 from dual union all
select 24473,'1970-Y09','1970-01-09 14:17:00','suborbital rocket','success',1188,'Su26H','','','','Ionospheric',1447,'',82 from dual union all
select 24474,'1970-Y10','1970-01-09 20:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',92 from dual union all
select 24475,'1970-Y100','1970-03-21 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',96 from dual union all
select 24476,'1970-Y101','1970-03-24 19:23:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',94 from dual union all
select 24477,'1970-Y102','1970-03-24 21:32:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',93 from dual union all
select 24478,'1970-Y103','1970-03-25 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 24479,'1970-Y104','1970-03-25 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',96 from dual union all
select 2448,'1954-A435','1954-07-23 19:37:00','suborbital rocket','failure',370,'','SUI 32','','','Ionos/Aeron?',77,'WYG-279',23 from dual union all
select 24480,'1970-Y105','1970-03-25 19:47:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',85 from dual union all
select 24481,'1970-Y106','1970-03-25 20:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',91 from dual union all
select 24482,'1970-Y107','1970-03-25 20:59:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',94 from dual union all
select 24483,'1970-Y108','1970-03-25 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 24484,'1970-Y109','1970-03-26 20:32:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',91 from dual union all
select 24485,'1970-Y11','1970-01-09 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',87 from dual union all
select 24486,'1970-Y110','1970-03-26 21:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',90 from dual union all
select 24487,'1970-Y111','1970-03-27 12:42:00','suborbital rocket','success',258,'NASA 15.63UE','ISRO-12.04','TERLS-177','','Plasma',1467,'',93 from dual union all
select 24488,'1970-Y112','1970-03-27 13:06:00','suborbital rocket','success',258,'NASA 15.64UE','ISRO-12.05','TERLS-178','','Plasma',1467,'',103 from dual;
insert into LAUNCH
select 24489,'1970-Y113','1970-03-27 13:30:00','suborbital rocket','success',258,'NASA 15.65UE','ISRO-12.06','TERLS-179','','Plasma',1467,'',97 from dual union all
select 2449,'1954-A436','1954-07-26 00:00:00','atmospheric rocket','success',836,'1352E','','','','Test',1895,'',10 from dual union all
select 24490,'1970-Y114','1970-03-28 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 24491,'1970-Y115','1970-03-29 19:53:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',89 from dual union all
select 24492,'1970-Y116','1970-03-29 21:37:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',92 from dual union all
select 24493,'1970-Y117','1970-03-30 19:56:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',93 from dual union all
select 24494,'1970-Y118','1970-04-01 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 24495,'1970-Y119','1970-04-01 05:01:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 24496,'1970-Y12','1970-01-10 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',93 from dual union all
select 24497,'1970-Y120','1970-04-01 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 24498,'1970-Y121','1970-04-04 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 24499,'1970-Y122','1970-04-06 19:54:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',95 from dual union all
select 245,'1944-S222','1944-11-02 09:14:00','military missile','success',1470,'19492','Ma249','','','Test',661,'',90 from dual union all
select 2450,'1954-A437','1954-07-26 00:00:00','atmospheric rocket','success',836,'1818E','','','','Test',1895,'',10 from dual union all
select 24500,'1970-Y123','1970-04-06 21:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',88 from dual union all
select 24501,'1970-Y124','1970-04-07 19:31:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',91 from dual union all
select 24502,'1970-Y125','1970-04-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 24503,'1970-Y126','1970-04-08 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',82 from dual union all
select 24504,'1970-Y127','1970-04-08 19:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',92 from dual union all
select 24505,'1970-Y128','1970-04-08 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',92 from dual union all
select 24506,'1970-Y129','1970-04-09 15:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',81 from dual union all
select 24507,'1970-Y13','1970-01-11 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',87 from dual union all
select 24508,'1970-Y130','1970-04-09 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',99 from dual union all
select 24509,'1970-Y131','1970-04-09 16:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',80 from dual union all
select 2451,'1954-A438','1954-07-26 00:00:00','atmospheric rocket','success',836,'1446E','','','','Test',1895,'',10 from dual union all
select 24510,'1970-Y132','1970-04-09 19:49:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',94 from dual union all
select 24511,'1970-Y133','1970-04-09 21:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',95 from dual union all
select 24512,'1970-Y134','1970-04-10 19:31:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',94 from dual union all
select 24513,'1970-Y135','1970-04-11 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 24514,'1970-Y136','1970-04-11 13:06:00','suborbital rocket','success',1188,'Su28H','','','','Ionospheric',1447,'',90 from dual union all
select 24515,'1970-Y137','1970-04-11 16:06:00','suborbital rocket','success',1188,'Su27H','','','','Ionospheric',1447,'',90 from dual union all
select 24516,'1970-Y138','1970-04-13 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',93 from dual union all
select 24517,'1970-Y139','1970-04-14 09:15:00','suborbital rocket','success',1188,'Su16H','','','','Ionospheric',1447,'',92 from dual union all
select 24518,'1970-Y14','1970-01-12 13:53:00','suborbital rocket','success',1188,'Su22H','','','','Aeronomy',1447,'',85 from dual union all
select 24519,'1970-Y140','1970-04-14 12:33:00','suborbital rocket','success',1188,'Su17H','','','','Ionospheric',1447,'',92 from dual union all
select 2452,'1954-A439','1954-07-26 00:00:00','atmospheric rocket','success',836,'1411E','','','','Test',1895,'',10 from dual union all
select 24520,'1970-Y141','1970-04-14 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',87 from dual union all
select 24521,'1970-Y142','1970-04-15 02:05:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',96 from dual union all
select 24522,'1970-Y143','1970-04-15 09:11:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 24523,'1970-Y144','1970-04-15 10:05:00','suborbital rocket','success',1189,'Su15T/C','ISRO-16.13','TERLS-182','','Ionospheric',1467,'',93 from dual union all
select 24524,'1970-Y145','1970-04-15 19:25:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',92 from dual union all
select 24525,'1970-Y146','1970-04-15 20:03:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',96 from dual union all
select 24526,'1970-Y147','1970-04-15 21:26:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',95 from dual union all
select 24527,'1970-Y148','1970-04-16 01:50:00','suborbital rocket','success',1189,'Su14T/C','ISRO-16.12','TERLS-183','','Ionospheric',1467,'',91 from dual union all
select 24528,'1970-Y149','1970-04-16 08:02:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 24529,'1970-Y15','1970-01-14 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 2453,'1954-A44','1954-02-06 00:00:00','atmospheric rocket','success',836,'1264T','','','','Test',1895,'',10 from dual union all
select 24530,'1970-Y150','1970-04-16 19:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',96 from dual union all
select 24531,'1970-Y151','1970-04-16 21:28:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',95 from dual union all
select 24532,'1970-Y152','1970-04-17 13:06:00','suborbital rocket','success',1188,'Su18H','','','','Ionospheric',1447,'',95 from dual union all
select 24533,'1970-Y153','1970-04-17 19:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',92 from dual union all
select 24534,'1970-Y154','1970-04-17 21:27:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',93 from dual union all
select 24535,'1970-Y155','1970-04-18 19:17:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',94 from dual union all
select 24536,'1970-Y156','1970-04-18 21:33:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',94 from dual union all
select 24537,'1970-Y157','1970-04-22 10:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 24538,'1970-Y158','1970-04-27 05:01:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 24539,'1970-Y159','1970-04-29 02:05:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 2454,'1954-A440','1954-07-26 00:00:00','atmospheric rocket','success',836,'1417E','','','','Test',1895,'',10 from dual union all
select 24540,'1970-Y16','1970-01-14 14:40:00','suborbital rocket','unknown',1188,'UCL 05','','','','Aeron',417,'',78 from dual union all
select 24541,'1970-Y160','1970-04-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 24542,'1970-Y161','1970-05-06 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',95 from dual union all
select 24543,'1970-Y162','1970-05-06 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 24544,'1970-Y163','1970-05-13 01:50:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 24545,'1970-Y164','1970-05-13 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 24546,'1970-Y165','1970-05-14 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 24547,'1970-Y166','1970-05-14 10:32:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 24548,'1970-Y167','1970-05-20 01:50:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',96 from dual union all
select 24549,'1970-Y168','1970-05-20 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 2455,'1954-A441','1954-07-26 00:00:00','atmospheric rocket','success',836,'1457E','','','','Test',1895,'',10 from dual union all
select 24550,'1970-Y169','1970-05-21 07:31:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 24551,'1970-Y17','1970-01-14 04:17:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',94 from dual union all
select 24552,'1970-Y170','1970-05-21 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 24553,'1970-Y171','1970-05-27 02:05:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',97 from dual union all
select 24554,'1970-Y172','1970-05-27 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 24555,'1970-Y173','1970-06-04 10:01:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',95 from dual union all
select 24556,'1970-Y174','1970-06-04 15:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',81 from dual union all
select 24557,'1970-Y175','1970-06-10 02:10:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',95 from dual union all
select 24558,'1970-Y176','1970-06-10 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 24559,'1970-Y177','1970-06-12 03:59:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 2456,'1954-A442','1954-07-26 00:29:00','suborbital rocket','failure',370,'','NRL Rockoon 11','','','Ionos/Aeron?',70,'WYG-279',10 from dual union all
select 24560,'1970-Y178','1970-06-17 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 24561,'1970-Y179','1970-06-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 24562,'1970-Y18','1970-01-14 09:17:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 24563,'1970-Y180','1970-06-17 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 24564,'1970-Y181','1970-06-18 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',91 from dual union all
select 24565,'1970-Y182','1970-06-20 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',90 from dual union all
select 24566,'1970-Y183','1970-06-21 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',91 from dual union all
select 24567,'1970-Y184','1970-06-24 09:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 24568,'1970-Y185','1970-06-24 15:09:00','suborbital rocket','success',1188,'Su23H','','','','Ionospheric',1447,'',90 from dual union all
select 24569,'1970-Y186','1970-07-01 02:05:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',95 from dual union all
select 2457,'1954-A443','1954-07-27 00:00:00','military missile','unknown',339,'1276R','','','','Test',1895,'',30 from dual union all
select 24570,'1970-Y187','1970-07-01 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 24571,'1970-Y188','1970-07-01 15:06:00','suborbital rocket','success',1188,'Su29H','','','','Ionospheric',1447,'',90 from dual union all
select 24572,'1970-Y189','1970-07-01 20:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',89 from dual union all
select 24573,'1970-Y19','1970-01-14 10:35:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 24574,'1970-Y190','1970-07-01 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',92 from dual union all
select 24575,'1970-Y191','1970-07-05 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',88 from dual union all
select 24576,'1970-Y192','1970-07-06 18:24:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 24577,'1970-Y193','1970-07-06 20:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',94 from dual union all
select 24578,'1970-Y194','1970-07-07 00:05:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual;
insert into LAUNCH
select 24579,'1970-Y195','1970-07-08 00:05:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 2458,'1954-A444','1954-07-27 00:00:00','atmospheric rocket','failure',365,'','AGARD Model 2','','','Test',1821,'',10 from dual union all
select 24580,'1970-Y196','1970-07-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 24581,'1970-Y197','1970-07-08 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 24582,'1970-Y198','1970-07-08 19:56:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',85 from dual union all
select 24583,'1970-Y199','1970-07-10 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',85 from dual union all
select 24584,'1970-Y20','1970-01-14 15:59:59','suborbital rocket','success',1061,'','MK-1','','','Meteo',276,'',90 from dual union all
select 24585,'1970-Y200','1970-07-13 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',86 from dual union all
select 24586,'1970-Y201','1970-07-13 22:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',92 from dual union all
select 24587,'1970-Y202','1970-07-15 07:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 24588,'1970-Y203','1970-07-15 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 24589,'1970-Y204','1970-07-15 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',95 from dual union all
select 2459,'1954-A445','1954-07-27 00:00:00','atmospheric rocket','success',836,'1450A','','','','Test',1895,'',10 from dual union all
select 24590,'1970-Y205','1970-07-15 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',93 from dual union all
select 24591,'1970-Y206','1970-07-17 20:24:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',85 from dual union all
select 24592,'1970-Y207','1970-07-17 22:22:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',86 from dual union all
select 24593,'1970-Y208','1970-07-18 20:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',90 from dual union all
select 24594,'1970-Y209','1970-07-18 22:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',89 from dual union all
select 24595,'1970-Y21','1970-01-17 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',88 from dual union all
select 24596,'1970-Y210','1970-07-20 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',91 from dual union all
select 24597,'1970-Y211','1970-07-20 21:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',91 from dual union all
select 24598,'1970-Y212','1970-07-21 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1625,'',92 from dual union all
select 24599,'1970-Y213','1970-07-22 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 246,'1944-S223','1944-11-02 10:54:00','military missile','success',1470,'19493','Ma248','','','Test',661,'',90 from dual union all
select 2460,'1954-A446','1954-07-27 00:00:00','atmospheric rocket','success',836,'1341E','','','','Test',1895,'',10 from dual union all
select 24600,'1970-Y214','1970-07-22 13:35:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',95 from dual union all
select 24601,'1970-Y215','1970-07-22 20:54:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 24602,'1970-Y216','1970-07-29 01:55:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 24603,'1970-Y217','1970-07-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 24604,'1970-Y218','1970-08-05 02:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 24605,'1970-Y219','1970-08-05 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 24606,'1970-Y22','1970-01-18 20:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',81 from dual union all
select 24607,'1970-Y220','1970-08-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 24608,'1970-Y221','1970-08-12 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 24609,'1970-Y222','1970-08-12 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 2461,'1954-A447','1954-07-27 00:00:00','atmospheric rocket','success',836,'1827E','','','','Test',1895,'',10 from dual union all
select 24610,'1970-Y223','1970-08-13 06:02:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 24611,'1970-Y224','1970-08-19 01:50:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 24612,'1970-Y225','1970-08-19 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 24613,'1970-Y226','1970-08-20 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 24614,'1970-Y227','1970-08-22 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 24615,'1970-Y228','1970-08-26 01:55:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',95 from dual union all
select 24616,'1970-Y229','1970-08-26 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 24617,'1970-Y23','1970-01-18 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',86 from dual union all
select 24618,'1970-Y230','1970-08-27 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 24619,'1970-Y231','1970-08-27 10:02:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 2462,'1954-A448','1954-07-28 00:00:00','atmospheric rocket','unknown',576,'409M','','','','Test',1895,'',10 from dual union all
select 24620,'1970-Y232','1970-09-02 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 24621,'1970-Y233','1970-09-02 13:35:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 24622,'1970-Y234','1970-09-02 15:08:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',104 from dual union all
select 24623,'1970-Y235','1970-09-02 21:04:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 24624,'1970-Y236','1970-09-03 05:01:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 24625,'1970-Y237','1970-09-09 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 24626,'1970-Y238','1970-09-09 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 24627,'1970-Y239','1970-09-09 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 24628,'1970-Y24','1970-01-19 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',90 from dual union all
select 24629,'1970-Y240','1970-09-10 10:02:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 2463,'1954-A449','1954-07-28 00:00:00','atmospheric rocket','unknown',576,'214M','','','','Test',1895,'',10 from dual union all
select 24630,'1970-Y241','1970-09-15 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 24631,'1970-Y242','1970-09-16 02:05:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',98 from dual union all
select 24632,'1970-Y243','1970-09-16 17:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 24633,'1970-Y244','1970-09-23 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 24634,'1970-Y245','1970-09-23 15:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',104 from dual union all
select 24635,'1970-Y246','1970-09-23 18:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 24636,'1970-Y247','1970-09-30 01:55:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',97 from dual union all
select 24637,'1970-Y248','1970-09-30 09:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 24638,'1970-Y249','1970-09-30 15:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',85 from dual union all
select 24639,'1970-Y25','1970-01-19 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',84 from dual union all
select 2464,'1954-A45','1954-02-08 00:00:00','atmospheric rocket','success',836,'1307R','Nike 264R','','','Test',1895,'',10 from dual union all
select 24640,'1970-Y250','1970-09-30 19:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 24641,'1970-Y251','1970-10-07 01:50:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 24642,'1970-Y252','1970-10-07 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',96 from dual union all
select 24643,'1970-Y253','1970-10-07 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',91 from dual union all
select 24644,'1970-Y254','1970-10-07 15:06:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',93 from dual union all
select 24645,'1970-Y255','1970-10-07 15:33:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 24646,'1970-Y256','1970-10-07 22:19:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 24647,'1970-Y257','1970-10-14 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',96 from dual union all
select 24648,'1970-Y258','1970-10-14 03:20:00','suborbital rocket','unknown',88,'','','','','Aeron',1895,'',85 from dual union all
select 24649,'1970-Y259','1970-10-14 04:03:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 2465,'1954-A450','1954-07-28 00:00:00','atmospheric rocket','success',836,'1447E','','','','Test',1895,'',10 from dual union all
select 24650,'1970-Y26','1970-01-20 19:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',87 from dual union all
select 24651,'1970-Y260','1970-10-14 09:55:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 24652,'1970-Y261','1970-10-21 02:15:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',97 from dual union all
select 24653,'1970-Y262','1970-10-21 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 24654,'1970-Y263','1970-10-21 09:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 24655,'1970-Y264','1970-10-21 15:43:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 24656,'1970-Y265','1970-10-21 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 24657,'1970-Y266','1970-10-28 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',96 from dual union all
select 24658,'1970-Y267','1970-10-28 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',95 from dual union all
select 24659,'1970-Y268','1970-10-28 10:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',91 from dual union all
select 2466,'1954-A451','1954-07-28 00:00:00','atmospheric rocket','success',836,'2482A','','','','Test',1895,'',10 from dual union all
select 24660,'1970-Y269','1970-10-28 16:14:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',94 from dual union all
select 24661,'1970-Y27','1970-01-20 20:49:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',87 from dual union all
select 24662,'1970-Y270','1970-10-28 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',92 from dual union all
select 24663,'1970-Y271','1970-11-01 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',96 from dual union all
select 24664,'1970-Y272','1970-11-04 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 24665,'1970-Y273','1970-11-04 04:01:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 24666,'1970-Y274','1970-11-04 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 24667,'1970-Y275','1970-11-04 12:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',93 from dual union all
select 24668,'1970-Y276','1970-11-04 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',91 from dual;
insert into LAUNCH
select 24669,'1970-Y277','1970-11-06 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',94 from dual union all
select 2467,'1954-A452','1954-07-28 00:00:00','atmospheric rocket','success',836,'1420E','','','','Test',1895,'',10 from dual union all
select 24670,'1970-Y278','1970-11-07 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',96 from dual union all
select 24671,'1970-Y279','1970-11-11 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',95 from dual union all
select 24672,'1970-Y28','1970-01-20 22:37:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',86 from dual union all
select 24673,'1970-Y280','1970-11-12 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 24674,'1970-Y281','1970-11-13 10:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',94 from dual union all
select 24675,'1970-Y282','1970-11-13 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',91 from dual union all
select 24676,'1970-Y283','1970-11-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',93 from dual union all
select 24677,'1970-Y284','1970-11-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 24678,'1970-Y285','1970-11-18 03:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 24679,'1970-Y286','1970-11-18 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 2468,'1954-A453','1954-07-28 00:00:00','atmospheric rocket','success',836,'2770T','','','','Test',1895,'',10 from dual union all
select 24680,'1970-Y287','1970-11-18 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 24681,'1970-Y288','1970-11-18 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',91 from dual union all
select 24682,'1970-Y289','1970-11-18 22:09:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 24683,'1970-Y29','1970-01-21 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 24684,'1970-Y290','1970-11-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',91 from dual union all
select 24685,'1970-Y291','1970-11-20 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',91 from dual union all
select 24686,'1970-Y292','1970-11-20 04:01:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 24687,'1970-Y293','1970-11-20 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',87 from dual union all
select 24688,'1970-Y294','1970-11-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',91 from dual union all
select 24689,'1970-Y295','1970-11-22 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',94 from dual union all
select 2469,'1954-A454','1954-07-28 00:00:00','atmospheric rocket','success',836,'1249B-2439','Nike 356R','','','Test',1895,'',10 from dual union all
select 24690,'1970-Y296','1970-11-23 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',96 from dual union all
select 24691,'1970-Y297','1970-11-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',88 from dual union all
select 24692,'1970-Y298','1970-11-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 24693,'1970-Y299','1970-11-24 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',95 from dual union all
select 24694,'1970-Y30','1970-01-21 17:45:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',82 from dual union all
select 24695,'1970-Y300','1970-11-25 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',93 from dual union all
select 24696,'1970-Y301','1970-11-26 08:01:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 24697,'1970-Y302','1970-11-26 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',93 from dual union all
select 24698,'1970-Y303','1970-11-27 05:25:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 24699,'1970-Y304','1970-11-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',87 from dual union all
select 247,'1944-S224','1944-11-02 23:50:00','military missile','success',1470,'19494','Ma251','','','Test',661,'',90 from dual union all
select 2470,'1954-A455','1954-07-28 00:00:00','atmospheric rocket','success',836,'1432E','','','','Test',1895,'',10 from dual union all
select 24700,'1970-Y305','1970-11-30 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',92 from dual union all
select 24701,'1970-Y306','1970-12-02 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',96 from dual union all
select 24702,'1970-Y307','1970-12-02 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',93 from dual union all
select 24703,'1970-Y308','1970-12-02 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',93 from dual union all
select 24704,'1970-Y309','1970-12-02 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',92 from dual union all
select 24705,'1970-Y31','1970-01-21 21:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',88 from dual union all
select 24706,'1970-Y310','1970-12-03 07:03:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 24707,'1970-Y311','1970-12-03 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',87 from dual union all
select 24708,'1970-Y312','1970-12-03 18:33:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 24709,'1970-Y313','1970-12-05 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',93 from dual union all
select 2471,'1954-A456','1954-07-28 00:00:00','atmospheric rocket','success',836,'2567R','Nike 357R','','','Test',1895,'',10 from dual union all
select 24710,'1970-Y314','1970-12-06 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',92 from dual union all
select 24711,'1970-Y315','1970-12-07 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',91 from dual union all
select 24712,'1970-Y316','1970-12-08 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 24713,'1970-Y317','1970-12-09 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',95 from dual union all
select 24714,'1970-Y318','1970-12-09 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 24715,'1970-Y319','1970-12-09 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 24716,'1970-Y32','1970-01-25 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 24717,'1970-Y320','1970-12-09 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',91 from dual union all
select 24718,'1970-Y321','1970-12-09 17:45:00','suborbital rocket','unknown',725,'','ISRO-8.001','TERLS-258','','Aeron',1467,'',91 from dual union all
select 24719,'1970-Y322','1970-12-11 03:41:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 2472,'1954-A457','1954-07-29 00:00:00','military missile','unknown',339,'1406R','','','','Test',1895,'',30 from dual union all
select 24720,'1970-Y323','1970-12-11 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 24721,'1970-Y324','1970-12-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',88 from dual union all
select 24722,'1970-Y325','1970-12-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',91 from dual union all
select 24723,'1970-Y326','1970-12-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',85 from dual union all
select 24724,'1970-Y327','1970-12-16 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 24725,'1970-Y328','1970-12-16 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 24726,'1970-Y329','1970-12-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',94 from dual union all
select 24727,'1970-Y33','1970-01-28 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 24728,'1970-Y330','1970-12-16 17:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',83 from dual union all
select 24729,'1970-Y331','1970-12-16 17:36:00','suborbital rocket','unknown',725,'','ISRO-8.002','TERLS-270','','Aeron',1467,'',93 from dual union all
select 2473,'1954-A458','1954-08-02 00:00:00','atmospheric rocket','success',836,'2773T','','','','Test',1895,'',10 from dual union all
select 24730,'1970-Y332','1970-12-17 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',92 from dual union all
select 24731,'1970-Y333','1970-12-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',92 from dual union all
select 24732,'1970-Y334','1970-12-22 03:32:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 24733,'1970-Y335','1970-12-22 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',92 from dual union all
select 24734,'1970-Y336','1970-12-23 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 24735,'1970-Y337','1970-12-23 09:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 24736,'1970-Y338','1970-12-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',91 from dual union all
select 24737,'1970-Y339','1970-12-23 17:06:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 24738,'1970-Y34','1970-01-28 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 24739,'1970-Y340','1970-12-23 17:30:00','suborbital rocket','unknown',725,'','ISRO-8.003','TERLS-271','','Aeron',1467,'',93 from dual union all
select 2474,'1954-A459','1954-08-03 00:00:00','atmospheric rocket','success',836,'2758T','','','','Test',1895,'',10 from dual union all
select 24740,'1970-Y341','1970-12-24 11:03:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 24741,'1970-Y342','1970-12-26 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',94 from dual union all
select 24742,'1970-Y343','1970-12-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',88 from dual union all
select 24743,'1970-Y344','1970-12-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',89 from dual union all
select 24744,'1970-Y345','1970-12-28 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',94 from dual union all
select 24745,'1970-Y346','1970-12-29 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',92 from dual union all
select 24746,'1970-Y347','1970-12-30 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 24747,'1970-Y348','1970-12-30 04:31:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 24748,'1970-Y349','1970-12-30 08:55:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 24749,'1970-Y35','1970-01-28 10:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 2475,'1954-A46','1954-02-09 00:00:00','atmospheric rocket','success',365,'','F11F-1 Model','','','Test',1821,'',10 from dual union all
select 24750,'1970-Y350','1970-12-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',94 from dual union all
select 24751,'1970-Y351','1970-12-30 17:35:00','suborbital rocket','unknown',725,'','ISRO-8.004','TERLS-272','','Aeron',1467,'',94 from dual union all
select 24752,'1970-Y352','1970-12-31 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',91 from dual union all
select 24753,'1970-Y353','1970-02-26 21:00:00','suborbital rocket','success',725,'','Atomic O','','','Aeron',744,'SHOK',90 from dual union all
select 24754,'1970-Y36','1970-01-28 19:14:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',81 from dual union all
select 24755,'1970-Y37','1970-01-30 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 24756,'1970-Y38','1970-02-01 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 24757,'1970-Y39','1970-02-04 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 24758,'1970-Y40','1970-02-04 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual;
insert into LAUNCH
select 24759,'1970-Y41','1970-02-04 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 2476,'1954-A460','1954-08-04 00:00:00','atmospheric rocket','success',836,'2455R','Nike 360R','','','Test',1895,'',10 from dual union all
select 24760,'1970-Y42','1970-02-04 20:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',87 from dual union all
select 24761,'1970-Y43','1970-02-04 22:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',87 from dual union all
select 24762,'1970-Y44','1970-02-05 10:05:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 24763,'1970-Y45','1970-02-05 20:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',83 from dual union all
select 24764,'1970-Y46','1970-02-06 01:23:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',86 from dual union all
select 24765,'1970-Y47','1970-02-06 09:19:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 24766,'1970-Y48','1970-02-06 20:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',87 from dual union all
select 24767,'1970-Y49','1970-02-06 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',86 from dual union all
select 24768,'1970-Y50','1970-02-07 20:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',89 from dual union all
select 24769,'1970-Y51','1970-02-07 22:32:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',91 from dual union all
select 2477,'1954-A461','1954-08-04 00:00:00','atmospheric rocket','success',836,'2445R','Nike 358R','','','Test',1895,'',10 from dual union all
select 24770,'1970-Y52','1970-02-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 24771,'1970-Y53','1970-02-11 02:10:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 24772,'1970-Y54','1970-02-11 04:41:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 24773,'1970-Y55','1970-02-11 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 24774,'1970-Y56','1970-02-11 10:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 24775,'1970-Y57','1970-02-11 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',88 from dual union all
select 24776,'1970-Y58','1970-02-12 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',90 from dual union all
select 24777,'1970-Y59','1970-02-13 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',90 from dual union all
select 24778,'1970-Y60','1970-02-13 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',93 from dual union all
select 24779,'1970-Y61','1970-02-14 09:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 2478,'1954-A462','1954-08-04 00:00:00','atmospheric rocket','success',836,'2449R','Nike 359R','','','Test',1895,'',10 from dual union all
select 24780,'1970-Y62','1970-02-14 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',90 from dual union all
select 24781,'1970-Y63','1970-02-14 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',90 from dual union all
select 24782,'1970-Y64','1970-02-16 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 24783,'1970-Y65','1970-02-17 20:43:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',83 from dual union all
select 24784,'1970-Y66','1970-02-17 22:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',90 from dual union all
select 24785,'1970-Y67','1970-02-18 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 24786,'1970-Y68','1970-02-18 04:18:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 24787,'1970-Y69','1970-02-18 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 24788,'1970-Y70','1970-02-18 09:35:00','suborbital rocket','success',1188,'Su13T/C','ISRO 16.01','TERLS-153','','Ionospheric',1467,'',92 from dual union all
select 24789,'1970-Y71','1970-02-18 18:14:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',83 from dual union all
select 2479,'1954-A463','1954-08-05 00:00:00','military missile','failure',255,'623-7','','','','Test',253,'',20 from dual union all
select 24790,'1970-Y72','1970-02-18 20:17:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',87 from dual union all
select 24791,'1970-Y73','1970-02-18 22:18:00','suborbital rocket','unknown',725,'','','','','Aeron',1623,'',88 from dual union all
select 24792,'1970-Y74','1970-02-21 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 24793,'1970-Y75','1970-02-25 02:10:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 24794,'1970-Y76','1970-02-25 02:10:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 24795,'1970-Y77','1970-02-25 12:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 24796,'1970-Y78','1970-02-26 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',94 from dual union all
select 24797,'1970-Y79','1970-02-28 15:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 24798,'1970-Y80','1970-03-04 03:15:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 24799,'1970-Y81','1970-03-04 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 248,'1944-S225','1944-11-03 06:52:00','military missile','success',1470,'19496','Ma250','','','Test',661,'',90 from dual union all
select 2480,'1954-A464','1954-08-06 00:00:00','atmospheric rocket','success',836,'2822T','','','','Test',1895,'',10 from dual union all
select 24800,'1970-Y82','1970-03-05 10:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 24801,'1970-Y83','1970-03-07 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 24802,'1970-Y84','1970-03-08 20:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',85 from dual union all
select 24803,'1970-Y85','1970-03-08 22:11:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',85 from dual union all
select 24804,'1970-Y86','1970-03-10 20:34:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',96 from dual union all
select 24805,'1970-Y87','1970-03-10 22:42:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',94 from dual union all
select 24806,'1970-Y88','1970-03-11 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',95 from dual union all
select 24807,'1970-Y89','1970-03-11 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 24808,'1970-Y90','1970-03-11 19:54:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',94 from dual union all
select 24809,'1970-Y91','1970-03-11 21:44:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',96 from dual union all
select 2481,'1954-A465','1954-08-06 00:00:00','atmospheric rocket','success',836,'2756T','','','','Test',1895,'',10 from dual union all
select 24810,'1970-Y92','1970-03-12 10:01:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',94 from dual union all
select 24811,'1970-Y93','1970-03-12 20:25:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',94 from dual union all
select 24812,'1970-Y94','1970-03-12 21:54:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',91 from dual union all
select 24813,'1970-Y95','1970-03-13 20:13:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',92 from dual union all
select 24814,'1970-Y96','1970-03-13 21:34:00','suborbital rocket','unknown',725,'','','','','Aeron',1624,'',90 from dual union all
select 24815,'1970-Y97','1970-03-18 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 24816,'1970-Y98','1970-03-18 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 24817,'1970-Y99','1970-03-19 04:05:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 24818,'1971-000','1971-05-05 07:43:01','orbital','success',1019,'23C-2 (3C-20)','Duplicate of 1971-039','See SSN 5310','','Sat',274,'',null from dual union all
select 24819,'1971-001','1971-01-12 09:30:01','orbital','success',1508,'7705712-216','Kosmos-390','Zenit-4M','','Sat',562,'',null from dual union all
select 2482,'1954-A466','1954-08-10 00:00:00','atmospheric rocket','unknown',576,'464S','','','','Test',1895,'',10 from dual union all
select 24820,'1971-002','1971-01-14 12:00:00','orbital','success',678,'Yu295-01','Kosmos-391','DS-P1-I No. 11','','Sat',460,'',null from dual union all
select 24821,'1971-003','1971-01-20 11:24:00','orbital','success',1510,'Kh15000-005','Meteor','Meteor-M No. 19','','Sat',473,'',null from dual union all
select 24822,'1971-004','1971-01-21 08:40:01','orbital','success',1508,'Kh76001-038','Kosmos-392','Zenit-2M','','Sat',562,'',null from dual union all
select 24823,'1971-005','1971-01-21 18:20:00','orbital','success',1402,'23B-8 (3B-30)','KH-8A','GAMBIT-3 Mission 4330','','Sat',1778,'',null from dual union all
select 24824,'1971-006','1971-01-26 00:36:03','orbital','success',179,'AC-25','Intelsat 4 F-2','INTELSAT IV F2','','Sat',268,'',null from dual union all
select 24825,'1971-007','1971-01-26 12:44:33','orbital','success',678,'Yu457-31','Kosmos-393','DS-P1-Yu No. 34','','Sat',460,'',null from dual union all
select 24826,'1971-008','1971-01-31 21:03:00','deep space','success',1123,'SA-509','Apollo 14','Apollo CM-110','','Sat',921,'LUT2',null from dual union all
select 24827,'1971-009','1971-02-03 01:41:40','orbital','success',1386,'560/D82','NATO 2B','NATO IIB','','Sat',233,'',null from dual union all
select 24828,'1971-010','1971-02-09 18:48:48','orbital','success',680,'65027-119','Kosmos-394','DS-P1-M No. 2','','Sat',456,'',null from dual union all
select 24829,'1971-011','1971-02-16 04:00:00','orbital','success',821,'M-4S-2','Tansei','MS-T1','','Sat',835,'',null from dual union all
select 2483,'1954-A467','1954-08-10 00:00:00','atmospheric rocket','unknown',576,'433C','','','','Test',1895,'',10 from dual union all
select 24830,'1971-012','1971-02-17 03:52:05','orbital','success',1364,'249/BII-11','DAPP 3526','DAPP Block 5A F-3','','Sat',1720,'',null from dual union all
select 24831,'1971-013','1971-02-17 21:09:59','orbital','success',680,'53727-117','Kosmos-395','Tselina-OM','','Sat',456,'',null from dual union all
select 24832,'1971-014','1971-02-18 13:59:59','orbital','success',1508,'Kh76002-048','Kosmos-396','Zenit-4M','','Sat',474,'',null from dual union all
select 24833,'1971-015','1971-02-25 11:11:10','orbital','success',1446,'Yu22500-13L','Kosmos-397','I2P','','Sat',591,'',null from dual union all
select 24834,'1971-016','1971-02-26 05:06:21','orbital','success',1224,'Yu15000-015','Kosmos-398','T2K No. 2','','Sat',562,'',null from dual union all
select 24835,'1971-017','1971-03-03 09:30:01','orbital','success',1508,'7705712-220','Kosmos-399','Zenit-4M','','Sat',562,'',null from dual union all
select 24836,'1971-018','1971-03-03 12:02:00','orbital','success',293,'','SJ-1','shijian','','Sat',809,'',1800 from dual union all
select 24837,'1971-019','1971-03-13 16:15:00','orbital','success',1387,'562/D83','Explorer 43','IMP I','','Sat',233,'',null from dual union all
select 24838,'1971-020','1971-03-18 21:45:00','orbital','success',680,'65027-118','Kosmos-400','DS-P1-M No. 3','','Sat',456,'',null from dual union all
select 24839,'1971-021','1971-03-21 03:45:00','orbital','success',1404,'33B-1 (3B-36)','OPS 4788','Jumpseat 1','','Sat',1778,'',null from dual union all
select 2484,'1954-A468','1954-08-10 00:00:00','atmospheric rocket','unknown',576,'439C','','','','Test',1895,'',10 from dual union all
select 24840,'1971-022','1971-03-24 21:06:00','orbital','success',1396,'538/FTV 1660','CORONA 142/1114','CORONA CR-14','','Sat TAG179',1776,'',null from dual union all
select 24841,'1971-023','1971-03-27 10:59:59','orbital','success',1508,'7705712-219','Kosmos-401','Zenit-4M','','Sat',475,'',null from dual union all
select 24842,'1971-024','1971-04-01 02:57:07','orbital','success',1382,'491/D84','Isis 2','Isis B','','Sat',1723,'',null from dual union all
select 24843,'1971-025','1971-04-01 11:29:59','orbital','success',1446,'Yu22500-08L','Kosmos-402','US-A reactor','','Sat',590,'',null from dual union all
select 24844,'1971-026','1971-04-02 08:20:00','orbital','success',1508,'Kh76001-039','Kosmos-403','Zenit-2M','','Sat',474,'',null from dual union all
select 24845,'1971-027','1971-04-04 14:27:43','orbital','success',1446,'Yu22500-14L','Kosmos-404','I2P','','Sat',591,'',null from dual union all
select 24846,'1971-028','1971-04-07 07:10:01','orbital','success',1510,'Yu15000-064','Kosmos-405','Tselina-D','','Sat',475,'',null from dual union all
select 24847,'1971-029','1971-04-14 08:00:00','orbital','success',1508,'Kh76002-050','Kosmos-406','Zenit-4M','','Sat',474,'',null from dual union all
select 24848,'1971-030','1971-04-15 09:19:37','orbital','success',432,'No. 3','D-2A','D-2A','','Sat',343,'',null from dual;
insert into LAUNCH
select 24849,'1971-031','1971-04-17 11:44:58','orbital','success',1510,'Ya15000-059','Meteor','Meteor-M No. 21 (71)','','Sat',475,'',null from dual union all
select 2485,'1954-A469','1954-08-11 00:00:00','atmospheric rocket','success',365,'','F11F-1 Model','','','Test',1821,'',10 from dual union all
select 24850,'1971-032','1971-04-19 01:40:00','orbital','success',949,'254-01','Salyut','DOS-1 17K No. 121','','Sat',588,'',null from dual union all
select 24851,'1971-033','1971-04-22 15:30:00','orbital','success',1402,'23B-9 (3B-31)','KH-8A','GAMBIT-3 Mission 4331','','Sat',1778,'',null from dual union all
select 24852,'1971-034','1971-04-22 23:54:06','orbital','success',1223,'Kh15000-025','Soyuz-10','Soyuz 7K-T No. 31','','Sat',480,'',null from dual union all
select 24853,'1971-035','1971-04-23 11:30:00','orbital','success',680,'53727-116','Kosmos-407','Strela-2M','','Sat',456,'',null from dual union all
select 24854,'1971-036','1971-04-24 07:32:29','orbital','success',1128,'S173C','San Marco C','San Marco C','','Sat',1412,'',null from dual union all
select 24855,'1971-037','1971-04-24 11:15:02','orbital','success',678,'Yu457-33','Kosmos-408','DS-P1-Yu No. 37','','Sat',460,'',null from dual union all
select 24856,'1971-038','1971-04-28 14:35:00','orbital','success',680,'53727-115','Kosmos-409','Sfera','','Sat',456,'',null from dual union all
select 24857,'1971-039','1971-05-05 07:43:01','orbital','success',1425,'23C-2 (3C-20)','DSP 2','DSP 3','','Sat UPF',274,'',null from dual union all
select 24858,'1971-040','1971-05-06 06:20:01','orbital','success',1508,'7705711-122','Kosmos-410','Zenit-2M','','Sat',562,'',null from dual union all
select 24859,'1971-041','1971-05-07 14:20:00','orbital','success',680,'53727-121','Kosmos-411/418','Strela-1M','','Sat',456,'',null from dual union all
select 2486,'1954-A47','1954-02-09 00:00:00','atmospheric rocket','unknown',576,'417ST','','','','Test',1895,'',10 from dual union all
select 24860,'1971-042','1971-05-10 16:58:42','orbital','failure',953,'253-01','Kosmos-419','3MS No. 170','','Sat',587,'',191 from dual union all
select 24861,'1971-043','1971-05-18 08:00:00','orbital','success',1508,'7705712-218','Kosmos-420','Zenit-4M','','Sat',562,'',null from dual union all
select 24862,'1971-044','1971-05-19 10:20:00','orbital','success',678,'45747-138','Kosmos-421','DS-P1-Yu No. 48','','Sat',460,'',null from dual union all
select 24863,'1971-045','1971-05-19 16:22:44','deep space','success',953,'255-01','Mars-2','4M No. 171','','Sat',588,'',null from dual union all
select 24864,'1971-046','1971-05-22 00:51:31','orbital','success',680,'53727-120','Kosmos-422','Tsiklon','','Sat',456,'',null from dual union all
select 24865,'1971-047','1971-05-27 11:59:55','orbital','success',678,'64657-133','Kosmos-423','DS-P1-Yu No. 47','','Sat',460,'',null from dual union all
select 24866,'1971-048','1971-05-28 10:30:00','orbital','success',1508,'Kh15002-052','Kosmos-424','Zenit-4M','','Sat',475,'',null from dual union all
select 24867,'1971-049','1971-05-28 15:26:30','deep space','success',953,'249-01','Mars-3','4M No. 172','','Sat',587,'',null from dual union all
select 24868,'1971-050','1971-05-29 03:49:58','orbital','success',680,'V149-30LM','Kosmos-425','Tselina-OM','','Sat',456,'',null from dual union all
select 24869,'1971-051','1971-05-30 22:23:04','deep space','success',179,'AC-23','Mariner 9','Mariner 71J','','Sat',269,'',null from dual union all
select 2487,'1954-A470','1954-08-11 00:00:00','atmospheric rocket','success',836,'2775T','','','','Test',1895,'',10 from dual union all
select 24870,'1971-052','1971-06-04 18:10:00','orbital','success',680,'65014-101','Kosmos-426','DS-U2-K No. 1','','Sat',457,'',null from dual union all
select 24871,'1971-053','1971-06-06 04:55:09','orbital','success',1223,'Kh15000-024','Soyuz-11','Soyuz 7K-T No. 32','','Sat',480,'',null from dual union all
select 24872,'1971-054','1971-06-08 14:00:05','orbital','success',1364,'210/BII-12','P70-1','P70-1','','Sat',1720,'',null from dual union all
select 24873,'1971-055','1971-06-11 10:00:00','orbital','success',1508,'7801424-715','Kosmos-427','Zenit-4MK','','Sat',475,'',null from dual union all
select 24874,'1971-056','1971-06-15 18:41:00','orbital','success',1426,'23D-1','HEXAGON SV1','HEXAGON SV-1','','Sat',1777,'',null from dual union all
select 24875,'1971-057','1971-06-24 07:59:59','orbital','success',1508,'7705711-123','Kosmos-428','Zenit-2M','','Sat',562,'',null from dual union all
select 24876,'1971-058','1971-07-08 22:58:00','orbital','success',1128,'S177C','Explorer 44','Solar Explorer C','','Sat',1814,'',null from dual union all
select 24877,'1971-059','1971-07-16 01:41:36','orbital','success',1510,'7705711-308','Meteor','Meteor-M No. 20','','Sat',475,'',null from dual union all
select 24878,'1971-060','1971-07-16 10:50:00','orbital','success',1396,'550/FTV 2737','OPS 8373','STRAWMAN 4','','Sat TAG180',1722,'',null from dual union all
select 24879,'1971-061','1971-07-20 10:00:00','orbital','success',1508,'7705712-221','Kosmos-429','Zenit-4M','','Sat',562,'',null from dual union all
select 2488,'1954-A471','1954-08-11 00:00:00','atmospheric rocket','success',836,'2540A','','','','Test',1895,'',10 from dual union all
select 24880,'1971-062','1971-07-23 11:00:09','orbital','success',1508,'7705712-215','Kosmos-430','Zenit-4M','','Sat',474,'',null from dual union all
select 24881,'1971-063','1971-07-26 13:34:00','deep space','success',1123,'SA-510','Apollo 15','Apollo CM-112','','Sat',921,'LUT3',null from dual union all
select 24882,'1971-064','1971-07-28 03:29:01','orbital','success',809,'Kh15000-096','Molniya-1','Molniya-1 11F67 No. 20','','Sat',475,'',null from dual union all
select 24883,'1971-065','1971-07-30 08:29:54','orbital','success',1508,'7801421-141','Kosmos-431','Zenit-2M','','Sat',562,'',null from dual union all
select 24884,'1971-066','1971-08-05 10:00:00','orbital','success',1508,'7801422-242','Kosmos-432','Zenit-4M','','Sat',562,'',null from dual union all
select 24885,'1971-067','1971-08-07 00:11:00','orbital','success',162,'76F','OV1-20/21','OAR-901','','Sat',1687,'',null from dual union all
select 24886,'1971-068','1971-08-08 23:45:00','orbital','success',1008,'V 22501-07','Kosmos-433','OGCh-K No. 5102740-251','','Sat',547,'',null from dual union all
select 24887,'1971-069','1971-08-12 05:30:00','orbital','success',1224,'Yu15000-017','Kosmos-434','T2K No. 3','','Sat',562,'',null from dual union all
select 24888,'1971-070','1971-08-12 15:30:00','orbital','success',1403,'24B-1 (3B-32)','KH-8A','GAMBIT-3 Mission 4332','','Sat',1778,'',null from dual union all
select 24889,'1971-071','1971-08-16 18:39:00','orbital','success',1129,'S180C','Eole','Eole','','Sat',1814,'',null from dual union all
select 2489,'1954-A472','1954-08-11 00:00:00','atmospheric rocket','success',836,'2527R','Nike 361R','','','Test',1895,'',10 from dual union all
select 24890,'1971-072','1971-08-27 10:54:56','orbital','success',678,'64657-134','Kosmos-435','DS-P1-Yu No. 41','','Sat',460,'',null from dual union all
select 24891,'1971-073','1971-09-02 13:40:40','deep space','success',953,'256-01','Luna-18','E-8-5 No. 407','','Sat',588,'',null from dual union all
select 24892,'1971-074','1971-09-07 01:15:01','orbital','success',680,'Yu149-42','Kosmos-436','Tselina-OM','','Sat',457,'',null from dual union all
select 24893,'1971-075','1971-09-10 03:37:59','orbital','success',680,'65014-106','Kosmos-437','Tselina-OM','','Sat',457,'',null from dual union all
select 24894,'1971-076','1971-09-10 21:32:56','orbital','success',1396,'567/FTV 1662','CORONA 143/1115','CORONA CR-15','','Sat TAG181',1776,'',null from dual union all
select 24895,'1971-077','1971-09-14 13:00:29','orbital','success',1508,'7702424-716','Kosmos-438','Zenit-4MK','','Sat',473,'',null from dual union all
select 24896,'1971-078','1971-09-21 12:00:00','orbital','success',1508,'7705711-124','Kosmos-439','Zenit-2M','','Sat',474,'',null from dual union all
select 24897,'1971-079','1971-09-24 10:30:00','orbital','success',678,'64634-148','Kosmos-440','DS-P1-I No. 10','','Sat',460,'',null from dual union all
select 24898,'1971-080','1971-09-28 04:00:00','orbital','success',821,'M-4S-3','Shinsei','MS-F2','','Sat',835,'',null from dual union all
select 24899,'1971-081','1971-09-28 07:40:00','orbital','success',1508,'7705712-224 ','Kosmos-441','Zenit-4M','','Sat',562,'',null from dual union all
select 249,'1944-S226','1944-11-04 16:14:00','military missile','success',1470,'19495','Ma253','','','Test',661,'',90 from dual union all
select 2490,'1954-A473','1954-08-11 00:00:00','atmospheric rocket','success',836,'2776T','','','','Test',1895,'',10 from dual union all
select 24900,'1971-082','1971-09-28 10:00:22','deep space','success',953,'257-01','Luna-19','E-8LS No. 202','','Sat',588,'',null from dual union all
select 24901,'1971-083','1971-09-29 09:45:00','orbital','failure',1388,'565/D85','OSO 7','OSO H','','Sat',233,'',null from dual union all
select 24902,'1971-084','1971-09-29 11:30:01','orbital','success',1508,'7705712-222','Kosmos-442','Zenit-4M','','Sat',474,'',null from dual union all
select 24903,'1971-085','1971-10-07 12:30:00','orbital','success',1508,'7801421-135','Kosmos-443','Zenit-2M','','Sat',474,'',null from dual union all
select 24904,'1971-086','1971-10-13 13:41:00','orbital','success',680,'47114-104','Kosmos-444/451','Strela-1M','','Sat',457,'',null from dual union all
select 24905,'1971-087','1971-10-14 07:51:17','orbital','success',1365,'159/BIIA-10','DMSP 4527','DMSP Block 5B F-1 (SV-2)','','Sat',1720,'',null from dual union all
select 24906,'1971-088','1971-10-14 09:00:01','orbital','success',1508,'7705712-223','Kosmos-452','Zenit-4M','','Sat',562,'',null from dual union all
select 24907,'1971-089','1971-10-17 13:36:00','orbital','success',1395,'570/FTV ','P71-2','ASTEX P71-2','','Sat TAG182',1722,'',null from dual union all
select 24908,'1971-090','1971-10-19 12:40:01','orbital','success',678,'64634-145','Kosmos-453','DS-P1-Yu No. 44','','Sat',460,'',null from dual union all
select 24909,'1971-091','1971-10-21 11:32:00','orbital','failure',1389,'572/D86','ITOS B','ITOS B','','Sat',1723,'',1467 from dual union all
select 2491,'1954-A474','1954-08-12 00:00:00','atmospheric rocket','unknown',576,'486M','','','','Test',1895,'',10 from dual union all
select 24910,'1971-092','1971-10-23 17:10:00','orbital','success',1403,'24B-2 (3B-33)','KH-8A','GAMBIT-3 Mission 4333','','Sat',1778,'',null from dual union all
select 24911,'1971-093','1971-10-28 04:09:29','orbital','success',208,'R-3','X-3','X-3','','Sat',1844,'',1582 from dual union all
select 24912,'1971-094','1971-11-02 14:25:02','orbital','success',1508,'7705712-217','Kosmos-454','Zenit-4M','','Sat',473,'',null from dual union all
select 24913,'1971-095','1971-11-03 03:09:06','orbital','success',1425,'23C-3 (3C-21)','DSCS II 1/2','DSCS II A-1','','Sat UPF',274,'',null from dual union all
select 24914,'1971-096','1971-11-15 05:52:00','orbital','success',1128,'S163CR','Explorer 45','SSS A','','Sat',1412,'',null from dual union all
select 24915,'1971-097','1971-11-17 11:09:48','orbital','success',678,'29547-144','Kosmos-455','DS-P1-Yu No. 54','','Sat',460,'',null from dual union all
select 24916,'1971-098','1971-11-19 12:00:01','orbital','success',1508,'7705712-214','Kosmos-456','Zenit-4M','','Sat',474,'',null from dual union all
select 24917,'1971-099','1971-11-20 18:00:01','orbital','success',680,'47114-105','Kosmos-457','Sfera','','Sat',457,'',null from dual union all
select 24918,'1971-100','1971-11-24 09:30:02','orbital','success',809,'Yu15000-092','Molniya-2','Molniya-2','','Sat',475,'',null from dual union all
select 24919,'1971-101','1971-11-29 10:09:56','orbital','success',678,'29524-151','Kosmos-458','DS-P1-Yu No. 53','','Sat',460,'',null from dual union all
select 2492,'1954-A475','1954-08-12 00:00:00','atmospheric rocket','unknown',576,'487M','','','','Test',1895,'',10 from dual union all
select 24920,'1971-102','1971-11-29 17:30:00','orbital','success',680,'65027-110','Kosmos-459','DS-P1-M No. 5','','Sat',456,'',null from dual union all
select 24921,'1971-103','1971-11-30 16:39:01','orbital','success',680,'65017-101','Kosmos-460','Tselina-OM','','Sat',457,'',null from dual union all
select 24922,'1971-104','1971-12-02 08:25:14','orbital','success',678,'','Interkosmos-5','DS-U2-IK No. 2','','Sat',627,'',null from dual union all
select 24923,'1971-105','1971-12-02 17:30:01','orbital','success',680,'47117-109','Kosmos-461','DS-U2-MT No. 1','','Sat',456,'',null from dual union all
select 24924,'1971-106','1971-12-03 13:19:22','orbital','success',1446,'Yu22500-09L','Kosmos-462','I2P','','Sat',591,'',null from dual union all
select 24925,'1971-107','1971-12-06 09:50:01','orbital','success',1508,'Kh76002-053','Kosmos-463','Zenit-4M','','Sat',562,'',null from dual union all
select 24926,'1971-108','1971-12-10 11:00:00','orbital','success',1508,'7801422-238','Kosmos-464','Zenit-4M','','Sat',474,'',null from dual union all
select 24927,'1971-109','1971-12-11 20:47:01','orbital','success',1129,'S183C','Ariel 4','UK 4','','Sat',1779,'',null from dual union all
select 24928,'1971-110','1971-12-14 12:13:00','orbital','success',1395,'568/FTV 2707','POPPY 7','POPPY 7 SV1','','Sat TAG183',1722,'',null from dual union all
select 24929,'1971-111','1971-12-15 04:31:21','orbital','success',680,'65017-108','Kosmos-465','Tsiklon','','Sat',457,'',null from dual union all
select 2493,'1954-A476','1954-08-13 00:00:00','atmospheric rocket','success',836,'1156E','','','','Test',1895,'',10 from dual union all
select 24930,'1971-112','1971-12-16 09:39:59','orbital','success',1508,'7801422-236','Kosmos-466','Zenit-4M','','Sat',562,'',null from dual union all
select 24931,'1971-113','1971-12-17 10:39:58','orbital','success',678,'45747-140','Kosmos-467','DS-P1-Yu No. 45','','Sat',460,'',null from dual union all
select 24932,'1971-114','1971-12-17 13:00:01','orbital','success',680,'65014-102','Kosmos-468','Strela-2M','','Sat',457,'',null from dual union all
select 24933,'1971-115','1971-12-19 22:50:12','orbital','success',809,'7705711-497','Molniya-1','Molniya-1 11F67 No. 25','','Sat',475,'',null from dual union all
select 24934,'1971-116','1971-12-20 01:10:04','orbital','success',179,'AC-26','Intelsat 4 F-3','INTELSAT IV F3','','Sat',268,'',null from dual union all
select 24935,'1971-117','1971-12-25 11:29:58','orbital','success',1446,'Yu22500-18L','Kosmos-469','US-A reactor','','Sat',590,'',null from dual union all
select 24936,'1971-118','1971-12-27 14:04:51','orbital','success',1225,'Yu15000-001','Kosmos-470','Zenit-4MT','','Sat',473,'',null from dual union all
select 24937,'1971-119','1971-12-27 19:00:00','orbital','success',680,'65014-103','Aureole 1','DS-U2-GKA No. 1','','Sat',457,'',null from dual union all
select 24938,'1971-120','1971-12-29 10:50:01','orbital','success',1510,'7705711-307','Meteor','Meteor-MV No. 18','','Sat',473,'',null from dual;
insert into LAUNCH
select 24939,'1971-A01','1971-01-06 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',47 from dual union all
select 2494,'1954-A477','1954-08-17 00:00:00','atmospheric rocket','success',836,'2564T','','','','Test',1895,'',10 from dual union all
select 24940,'1971-A02','1971-01-07 00:27:00','suborbital rocket','failure',289,'ISRO 5.12','ISRO 5.12 Recovery','TERLS-276','','Test/Aeron',1467,'',1 from dual union all
select 24941,'1971-A03','1971-01-07 01:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',48 from dual union all
select 24942,'1971-A04','1971-01-10 15:24:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',43 from dual union all
select 24943,'1971-A05','1971-01-11 14:35:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',40 from dual union all
select 24944,'1971-A06','1971-01-11 17:00:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',46 from dual union all
select 24945,'1971-A07','1971-01-13 19:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',49 from dual union all
select 24946,'1971-A08','1971-01-14 11:38:00','suborbital rocket','success',1210,'LT.1','DLR AB-GR-40','','','Test',2,'',7 from dual union all
select 24947,'1971-A09','1971-01-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',48 from dual union all
select 24948,'1971-A10','1971-01-17 16:00:00','suborbital rocket','success',1189,'Sk83/3','ESRO Sk83/3','','','Aeronomy',417,'',35 from dual union all
select 24949,'1971-A100','1971-11-18 16:03:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',36 from dual union all
select 2495,'1954-A478','1954-08-18 00:00:00','atmospheric rocket','success',836,'2447R','Nike 363R','','','Test',1895,'',10 from dual union all
select 24950,'1971-A101','1971-12-01 14:37:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',46 from dual union all
select 24951,'1971-A102','1971-12-01 15:26:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',49 from dual union all
select 24952,'1971-A103','1971-12-01 18:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',48 from dual union all
select 24953,'1971-A104','1971-12-03 15:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',45 from dual union all
select 24954,'1971-A105','1971-12-04 00:00:00','atmospheric rocket','success',617,'J-2','Chevaline','','','Test',1836,'',30 from dual union all
select 24955,'1971-A106','1971-12-03 07:00:00','military missile','success',1004,'67014-03','15A14 BI-2-3','','','Test',573,'',1 from dual union all
select 24956,'1971-A107','1971-12-07 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',45 from dual union all
select 24957,'1971-A108','1971-12-10 00:00:00','military missile','failure',744,'51014-01','15A15 BI-2-1','Silo eject series 2','','Test',573,'',0 from dual union all
select 24958,'1971-A109','1971-12-13 15:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',48 from dual union all
select 24959,'1971-A11','1971-01-23 00:14:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',46 from dual union all
select 2496,'1954-A479','1954-08-18 00:00:00','atmospheric rocket','success',836,'2825T','','','','Test',1895,'',10 from dual union all
select 24960,'1971-A110','1971-12-16 19:35:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',44 from dual union all
select 24961,'1971-A111','1971-12-28 20:37:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',48 from dual union all
select 24962,'1971-A112','1971-12-29 07:20:00','military missile','success',1004,'45014-01','15A14 BI-3-1','','','Test',573,'',10 from dual union all
select 24963,'1971-A113','1971-01-13 02:00:00','suborbital rocket','success',761,'MT135P-R13','','','','Aeron',1332,'',43 from dual union all
select 24964,'1971-A114','1971-01-27 02:00:00','suborbital rocket','success',761,'MT135P-R15','','','','Aeron',1332,'',48 from dual union all
select 24965,'1971-A115','1971-03-04 02:00:00','suborbital rocket','failure',761,'MT135P-R19','','','','Aeron',1332,'',0 from dual union all
select 24966,'1971-A116','1971-07-30 00:00:00','suborbital rocket','failure',1535,'Zenit-Cuckoo','ESRO Z82/1','','','Aeron/Meteorites',1343,'',26 from dual union all
select 24967,'1971-A117','1971-01-01 00:00:00','military missile','success',843,'','ASF (Last Fairbanks)','','','Test',443,'',10 from dual union all
select 24968,'1971-A118','1971-09-23 00:00:00','suborbital rocket','failure',74,'','','','','Test',767,'',3 from dual union all
select 24969,'1971-A12','1971-01-27 15:25:40','suborbital rocket','success',1189,'Sk83/8','ESRO Sk83/8','','','Aeronomy',417,'',47 from dual union all
select 2497,'1954-A48','1954-02-09 00:00:00','atmospheric rocket','unknown',576,'416ST','','','','Test',1895,'',10 from dual union all
select 24970,'1971-A13','1971-01-28 00:00:00','suborbital rocket','success',1188,'M--333H','','','','Aeron',1447,'',47 from dual union all
select 24971,'1971-A15','1971-02-08 20:48:17','suborbital rocket','success',1189,'Sk83/12','ESRO Sk83/12','','','Aeronomy',417,'',48 from dual union all
select 24972,'1971-A16','1971-02-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',48 from dual union all
select 24973,'1971-A169','1971-03-02 18:10:00','suborbital rocket','failure',675,'WRE K153','','','','Aeron',1832,'',0 from dual union all
select 24974,'1971-A17','1971-02-12 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',46 from dual union all
select 24975,'1971-A170','1971-03-15 00:00:00','atmospheric rocket','success',1346,'Sandia','SAMS Vehicle 1','','','Tech',1821,'',3 from dual union all
select 24976,'1971-A171','1971-03-26 00:00:00','atmospheric rocket','success',1346,'Sandia','SAMS Vehicle 2','','','Tech',1821,'',3 from dual union all
select 24977,'1971-A172','1971-12-22 07:05:00','military missile','success',744,'51014-02','15A15 BI-2-2','','','Test',573,'',1 from dual union all
select 24978,'1971-A173','1971-03-17 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 4','','','Meteo',1849,'',30 from dual union all
select 24979,'1971-A174','1971-03-24 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 5','','','Meteo',1849,'',30 from dual union all
select 2498,'1954-A480','1954-08-18 00:00:00','atmospheric rocket','success',836,'2469R','Nike 362R','','','Test',1895,'',10 from dual union all
select 24980,'1971-A175','1971-06-21 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 9','','','Meteo',1849,'',30 from dual union all
select 24981,'1971-A176','1971-07-14 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 10','','','Meteo',1849,'',30 from dual union all
select 24982,'1971-A177','1971-09-22 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 13','','','Meteo',1849,'',30 from dual union all
select 24983,'1971-A178','1971-10-22 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 14','','','Meteo',1849,'',30 from dual union all
select 24984,'1971-A179','1971-08-26 00:00:00','military missile','failure',1037,'','8K98 UBP-17','','','Test',477,'',0 from dual union all
select 24985,'1971-A18','1971-02-16 00:00:00','military missile','success',1472,'','Test','','','ABM Test',183,'',25 from dual union all
select 24986,'1971-A180','1971-01-30 00:00:00','military missile','success',1004,'','15A14 BI-1 K-1','Silo ejection test','','Test',1037,'',1 from dual union all
select 24987,'1971-A181','1971-05-01 00:00:00','military missile','success',1004,'','15A14 BI-1 D-1','Silo ejection test','','Test',1037,'',1 from dual union all
select 24988,'1971-A182','1971-08-28 00:00:00','military missile','success',1004,'','15A14 BI-1 D-5','Silo ejection test','','Test',1037,'',1 from dual union all
select 24989,'1971-A183','1971-09-11 00:00:00','military missile','success',1004,'','15A14 BI-1 K-15','Silo ejection test','','Test',1037,'',1 from dual union all
select 2499,'1954-A481','1954-08-19 00:00:00','atmospheric rocket','unknown',576,'447','','','','Test',1895,'',10 from dual union all
select 24990,'1971-A184','1971-05-07 00:00:00','military missile','success',744,'','15A15 BI-1-1','Silo ejection test','','Test',1037,'',1 from dual union all
select 24991,'1971-A185','1971-01-01 00:00:00','military missile','success',744,'','15A15 BI-1-2','Silo ejection test','','Test',1037,'',1 from dual union all
select 24992,'1971-A186','1971-01-01 00:00:00','military missile','success',744,'','15A15 BI-1-3','Silo ejection test','','Test',1037,'',1 from dual union all
select 24993,'1971-A187','1971-01-01 00:00:00','military missile','success',744,'','15A15 BI-1-4','Silo ejection test','','Test',1037,'',1 from dual union all
select 24994,'1971-A188','1971-01-01 00:00:00','military missile','success',744,'','15A15 BI-1-5','Silo ejection test','','Test',1037,'',1 from dual union all
select 24995,'1971-A189','1971-01-01 00:00:00','military missile','success',744,'','15A15 BI-1-6','Silo ejection test','','Test',1037,'',1 from dual union all
select 24996,'1971-A19','1971-02-16 15:18:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',43 from dual union all
select 24997,'1971-A190','1971-01-01 00:00:00','military missile','success',744,'','15A15 BI-1-7','Silo ejection test','','Test',1037,'',1 from dual union all
select 24998,'1971-A191','1971-01-01 00:00:00','military missile','success',744,'','15A15 BI-1-8','Silo ejection test','','Test',1037,'',1 from dual union all
select 24999,'1971-A192','1971-01-01 00:00:00','military missile','success',744,'','15A15 BI-1-9','Silo ejection test','','Test',1037,'',1 from dual union all
select 25,'1943-S03','1943-04-22 14:25:00','military missile','success',8,'21','','','','Test',675,'',95 from dual union all
select 250,'1944-S227','1944-11-04 17:04:00','military missile','success',1470,'19497','Ma252','','','Test',661,'',90 from dual union all
select 2500,'1954-A482','1954-08-23 00:00:00','atmospheric rocket','success',836,'2827T','','','','Test',1895,'',10 from dual union all
select 25000,'1971-A193','1971-01-01 00:00:00','military missile','success',744,'','15A15 BI-1-10','Silo ejection test','','Test',1037,'',1 from dual union all
select 25001,'1971-A194','1971-01-01 00:00:00','military missile','success',744,'','15A15 BI-1-11','Silo ejection test','','Test',1037,'',1 from dual union all
select 25002,'1971-A195','1971-12-29 00:00:00','military missile','success',744,'','15A15 BI-1-12','Silo ejection test','','Test',1037,'',1 from dual union all
select 25003,'1971-A196','1971-01-01 00:00:00','military missile','success',1004,'','15A14 BI-1 K-2','Silo ejection test','','Test',1037,'',1 from dual union all
select 25004,'1971-A197','1971-01-01 00:00:00','military missile','success',1004,'','15A14 BI-1 K-3','Silo ejection test','','Test',1037,'',1 from dual union all
select 25005,'1971-A198','1971-01-01 00:00:00','military missile','success',1004,'','15A14 BI-1 K-4','Silo ejection test','','Test',1037,'',1 from dual union all
select 25006,'1971-A199','1971-01-01 00:00:00','military missile','success',1004,'','15A14 BI-1 K-5','Silo ejection test','','Test',1037,'',1 from dual union all
select 25007,'1971-A20','1971-02-16 16:18:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',26 from dual union all
select 25008,'1971-A200','1971-01-01 00:00:00','military missile','success',1004,'','15A14 BI-1 K-6','Silo ejection test','','Test',1037,'',1 from dual union all
select 25009,'1971-A201','1971-01-01 00:00:00','military missile','success',1004,'','15A14 BI-1 K-7','Silo ejection test','','Test',1037,'',1 from dual union all
select 2501,'1954-A483','1954-08-24 00:00:00','atmospheric rocket','success',836,'2789T','','','','Test',1895,'',10 from dual union all
select 25010,'1971-A202','1971-01-01 00:00:00','military missile','success',1004,'','15A14 BI-1 K-8','Silo ejection test','','Test',1037,'',1 from dual union all
select 25011,'1971-A203','1971-01-01 00:00:00','military missile','success',1004,'','15A14 BI-1 K-9','Silo ejection test','','Test',1037,'',1 from dual union all
select 25012,'1971-A204','1971-01-01 00:00:00','military missile','success',1004,'','15A14 BI-1 K-10','Silo ejection test','','Test',1037,'',1 from dual union all
select 25013,'1971-A205','1971-01-01 00:00:00','military missile','success',1004,'','15A14 BI-1 K-11','Silo ejection test','','Test',1037,'',1 from dual union all
select 25014,'1971-A206','1971-01-01 00:00:00','military missile','success',1004,'','15A14 BI-1 K-12','Silo ejection test','','Test',1037,'',1 from dual union all
select 25015,'1971-A207','1971-01-01 00:00:00','military missile','success',1004,'','15A14 BI-1 K-13','Silo ejection test','','Test',1037,'',1 from dual union all
select 25016,'1971-A208','1971-01-01 00:00:00','military missile','success',1004,'','15A14 BI-1 K-14','Silo ejection test','','Test',1037,'',1 from dual union all
select 25017,'1971-A209','1971-05-01 00:00:00','military missile','success',1004,'','15A14 BI-1 D-2','Silo ejection test','','Test',1037,'',1 from dual union all
select 25018,'1971-A21','1971-02-16 20:30:00','suborbital rocket','success',1061,'','Starute,Arcasonde','','','Meteo',1895,'',46 from dual union all
select 25019,'1971-A210','1971-06-01 00:00:00','military missile','success',1004,'','15A14 BI-1 D-3','Silo ejection test','','Test',1037,'',1 from dual union all
select 2502,'1954-A484','1954-08-25 00:00:00','atmospheric rocket','success',863,'1249B-2517','Nike 364R T48E2','','','Test',1895,'',10 from dual union all
select 25020,'1971-A211','1971-07-01 00:00:00','military missile','success',1004,'','15A14 BI-1 D-4','Silo ejection test','','Test',1037,'',1 from dual union all
select 25021,'1971-A212','1971-08-13 00:00:00','military missile','failure',702,'EST Test 1','','','','Test',1895,'',0 from dual union all
select 25022,'1971-A213','1971-11-30 00:00:00','military missile','failure',702,'EST Test','','','','Test',1895,'',0 from dual union all
select 25023,'1971-A22','1971-02-17 14:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',49 from dual union all
select 25024,'1971-A23','1971-02-23 00:00:00','military missile','success',1252,'FLA-46','Safeguard M1-28','','','Interceptor',868,'',10 from dual union all
select 25025,'1971-A24','1971-02-24 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',48 from dual union all
select 25026,'1971-A25','1971-02-27 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 25027,'1971-A26','1971-02-27 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual;
insert into LAUNCH
select 25028,'1971-A27','1971-03-03 19:44:59','suborbital rocket','success',1061,'','','','','Meteo',1895,'',29 from dual union all
select 25029,'1971-A28','1971-03-08 11:38:00','suborbital rocket','failure',1349,'','Caribou V','','','Auroral UV',1047,'',0 from dual union all
select 2503,'1954-A485','1954-08-25 00:00:00','atmospheric rocket','success',836,'2510E','','','','Test',1895,'',10 from dual union all
select 25030,'1971-A29','1971-03-10 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',44 from dual union all
select 25031,'1971-A30','1971-03-12 21:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',49 from dual union all
select 25032,'1971-A31','1971-03-17 00:00:00','military missile','success',1252,'FLA-50','SAFEGUARD M1-13','','','Interceptor',868,'',10 from dual union all
select 25033,'1971-A32','1971-03-17 00:00:00','military missile','success',1252,'FLA-51','SAFEGUARD M1-13','','','Interceptor',868,'',10 from dual union all
select 25034,'1971-A33','1971-03-22 14:01:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',49 from dual union all
select 25035,'1971-A34','1971-04-08 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',46 from dual union all
select 25036,'1971-A35','1971-04-14 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 25037,'1971-A36','1971-04-14 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 25038,'1971-A37','1971-04-26 15:35:00','suborbital rocket','success',230,'','F-EXM-006','','','Test',428,'',29 from dual union all
select 25039,'1971-A38','1971-04-26 15:36:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',49 from dual union all
select 2504,'1954-A486','1954-08-25 00:00:00','atmospheric rocket','success',836,'2759T','','','','Test',1895,'',10 from dual union all
select 25040,'1971-A39','1971-04-26 17:02:00','suborbital rocket','success',230,'','F-EXM-005','','','Test',428,'',29 from dual union all
select 25041,'1971-A40','1971-04-26 18:06:00','suborbital rocket','success',230,'','F-EXM-008','','','Test',428,'',37 from dual union all
select 25042,'1971-A41','1971-04-26 19:33:00','suborbital rocket','success',230,'','F-EXM-009','','','Test',428,'',36 from dual union all
select 25043,'1971-A42','1971-04-26 20:44:00','suborbital rocket','success',230,'','F-EXM-007','','','Test',428,'',32 from dual union all
select 25044,'1971-A43','1971-04-30 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',49 from dual union all
select 25045,'1971-A44','1971-05-05 14:43:00','suborbital rocket','success',230,'','F-EXM-010','','','Test',428,'',38 from dual union all
select 25046,'1971-A45','1971-05-05 15:46:00','suborbital rocket','success',230,'','F-EXM-011','','','Test',428,'',36 from dual union all
select 25047,'1971-A46','1971-05-05 16:45:00','suborbital rocket','success',230,'','F-EXM-012','','','Test',428,'',38 from dual union all
select 25048,'1971-A47','1971-05-05 17:17:00','suborbital rocket','success',230,'','F-EXM-013','','','Test',428,'',34 from dual union all
select 25049,'1971-A48','1971-05-05 18:25:00','suborbital rocket','success',230,'','F-EXM-014','','','Test',428,'',35 from dual union all
select 2505,'1954-A487','1954-08-25 00:00:00','atmospheric rocket','success',836,'2350E','','','','Test',1895,'',10 from dual union all
select 25050,'1971-A49','1971-05-05 18:56:00','suborbital rocket','success',230,'','F-EXM-015','','','Test',428,'',35 from dual union all
select 25051,'1971-A50','1971-05-05 19:24:00','suborbital rocket','success',230,'','F-EXM-016','','','Test',428,'',35 from dual union all
select 25052,'1971-A51','1971-05-07 00:00:00','military missile','success',1252,'FLA-42','Safeguard M1-8','','','Interceptor',868,'',10 from dual union all
select 25053,'1971-A52','1971-05-07 16:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',45 from dual union all
select 25054,'1971-A53','1971-05-28 15:33:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',39 from dual union all
select 25055,'1971-A54','1971-06-03 00:00:00','military missile','failure',1449,'8K84','8K84 UBP-138','','','OT',536,'',0 from dual union all
select 25056,'1971-A55','1971-06-11 20:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',49 from dual union all
select 25057,'1971-A56','1971-06-16 16:14:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',48 from dual union all
select 25058,'1971-A57','1971-06-22 00:00:00','atmospheric rocket','success',1066,'Ori23','','','','Test',2,'',20 from dual union all
select 25059,'1971-A58','1971-06-30 01:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 2506,'1954-A488','1954-08-25 00:00:00','atmospheric rocket','success',863,'1249B-2521','Nike 365R T48E2','','','Test',1895,'',10 from dual union all
select 25060,'1971-A59','1971-06-30 01:30:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 25061,'1971-A60','1971-07-01 00:00:00','atmospheric rocket','success',1066,'F1','FFARV','','','Test',1836,'',4 from dual union all
select 25062,'1971-A61','1971-07-03 00:00:00','atmospheric rocket','success',1066,'Ly7','Lynx 7','','','Test',2,'',20 from dual union all
select 25063,'1971-A62','1971-07-07 11:37:59','suborbital rocket','success',716,'INTA HT-7110','','','','Meteo',396,'',40 from dual union all
select 25064,'1971-A63','1971-07-28 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',47 from dual union all
select 25065,'1971-A64','1971-08-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',10 from dual union all
select 25066,'1971-A65','1971-08-02 18:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',49 from dual union all
select 25067,'1971-A66','1971-08-04 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',43 from dual union all
select 25068,'1971-A67','1971-08-09 15:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',44 from dual union all
select 25069,'1971-A68','1971-08-09 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',47 from dual union all
select 2507,'1954-A489','1954-08-25 00:00:00','atmospheric rocket','success',836,'2805T','','','','Test',1895,'',10 from dual union all
select 25070,'1971-A69','1971-08-18 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',49 from dual union all
select 25071,'1971-A70','1971-08-25 05:40:00','suborbital rocket','failure',668,'K-9M-36','','','','Ionos',839,'',0 from dual union all
select 25072,'1971-A71','1971-08-30 15:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',46 from dual union all
select 25073,'1971-A72','1971-08-30 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',47 from dual union all
select 25074,'1971-A73','1971-09-01 00:00:00','suborbital rocket','failure',631,'','','','','Meteo',428,'',10 from dual union all
select 25075,'1971-A74','1971-09-07 18:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',615,'',41 from dual union all
select 25076,'1971-A75','1971-09-08 19:24:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',43 from dual union all
select 25077,'1971-A76','1971-09-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',49 from dual union all
select 25078,'1971-A77','1971-09-13 21:14:00','suborbital rocket','failure',920,'P74H','Night D/E regions','','','Ionospheric',1447,'',18 from dual union all
select 25079,'1971-A78','1971-09-17 02:50:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 2508,'1954-A49','1954-02-10 00:00:00','atmospheric rocket','unknown',576,'218M','','','','Test',1895,'',10 from dual union all
select 25080,'1971-A79','1971-09-17 03:45:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 25081,'1971-A80','1971-09-23 19:50:00','military missile','failure',917,'Imp.','P-214 71 Fall 01','','','OT',433,'',1 from dual union all
select 25082,'1971-A81','1971-09-22 12:14:59','suborbital rocket','success',716,'INTA HT-7114','','','','Meteo',396,'',48 from dual union all
select 25083,'1971-A82','1971-09-23 00:00:00','military missile','failure',1251,'800032','SAFEGUARD M1-14A','','','Interceptor',868,'',10 from dual union all
select 25084,'1971-A83','1971-10-09 00:00:00','suborbital rocket','success',1022,'','Range Test','','','Test',1361,'',11 from dual union all
select 25085,'1971-A84','1971-09-30 16:02:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',46 from dual union all
select 25086,'1971-A85','1971-10-07 00:00:00','military missile','success',1252,'FLA-48','SAFEGUARD M2-2','','','Interceptor',868,'',5 from dual union all
select 25087,'1971-A86','1971-10-07 18:32:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',49 from dual union all
select 25088,'1971-A87','1971-10-19 15:00:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',43 from dual union all
select 25089,'1971-A88','1971-10-20 00:00:00','atmospheric rocket','success',617,'J-1','Chevaline','','','Test',1836,'',30 from dual union all
select 2509,'1954-A490','1954-08-26 00:00:00','atmospheric rocket','unknown',576,'550','','','','Test',1895,'',10 from dual union all
select 25090,'1971-A89','1971-10-22 06:25:00','military missile','success',1004,'45014-01','15A14 BI-2-1','Silo ejection series II','','Test',573,'',1 from dual union all
select 25091,'1971-A90','1971-10-29 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',48 from dual union all
select 25092,'1971-A91','1971-11-17 00:00:00','military missile','success',1466,'','External burning','','','Test',1895,'',20 from dual union all
select 25093,'1971-A92','1971-11-01 00:00:00','suborbital rocket','failure',631,'','','','','Meteo',428,'',10 from dual union all
select 25094,'1971-A93','1971-11-03 04:30:00','military missile','success',1004,'51014-02','15A14 BI-2-2','','','Test',573,'',1 from dual union all
select 25095,'1971-A94','1971-11-10 00:00:00','military missile','success',1252,'FLA-57','SAFEGUARD M2-5','','','Interceptor',868,'',5 from dual union all
select 25096,'1971-A95','1971-11-12 00:00:00','military missile','success',1252,'FLA-56','SAFEGUARD M2-8','','','Interceptor',868,'',5 from dual union all
select 25097,'1971-A96','1971-11-11 23:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',39 from dual union all
select 25098,'1971-A97','1971-11-16 00:04:00','suborbital rocket','failure',1205,'ESRO S44/2','ESRO S44/2','','','UV Astronomy',1343,'',0 from dual union all
select 25099,'1971-A98','1971-11-17 14:41:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',44 from dual union all
select 251,'1944-S228','1944-11-05 09:06:00','military missile','success',1470,'19502','Ma257','','','Test',661,'',90 from dual union all
select 2510,'1954-A491','1954-08-26 00:00:00','military missile','failure',573,'3','XSSM-A-16','','','Test',1863,'',3 from dual union all
select 25100,'1971-A99','1971-11-17 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',46 from dual union all
select 25101,'1971-F01','1971-02-17 20:04:30','orbital','failure',1396,'537/FTV 1659','CORONA 141/1113','CORONA CR-13','','Sat TAG178',1776,'',2 from dual union all
select 25102,'1971-F02','1971-03-05 08:15:02','orbital','failure',678,'','','DS-P1-Yu No. 39','','Sat',627,'',90 from dual union all
select 25103,'1971-F03','1971-03-05 01:02:03','orbital','failure',1508,'Kh76001-037','','Zenit-2M','','Sat',475,'',20 from dual union all
select 25104,'1971-F04','1971-05-09 01:11:02','orbital','failure',179,'AC-24','Mariner 8','Mariner 71H','','Sat',268,'',160 from dual union all
select 25105,'1971-F05','1971-06-25 10:30:01','orbital','failure',1508,'7801422-235','','Zenit-4M','','Sat',475,'',20 from dual union all
select 25106,'1971-F06','1971-06-26 23:15:08','orbital','failure',823,'6L/Kh15006','','LK','','Sat',498,'',2 from dual union all
select 25107,'1971-F07','1971-07-22 13:45:00','orbital','failure',680,'V149-32LM','','Tselina-OM','','Sat',457,'',540 from dual union all
select 25108,'1971-F08','1971-08-03 11:00:00','orbital','failure',678,'45747-139','','DS-P1-Yu No. 33','','Sat',460,'',270 from dual union all
select 25109,'1971-F09','1971-08-19 06:30:00','orbital','failure',1508,'Kh76001-040','','Zenit-4M','','Sat',562,'',40 from dual union all
select 2511,'1954-A492','1954-08-26 00:00:00','atmospheric rocket','success',836,'2833T','','','','Test',1895,'',10 from dual union all
select 25110,'1971-F10','1971-11-05 13:00:00','orbital','failure',468,'F-11','STV 4/PAS','STV 4','','Sat',342,'',27 from dual union all
select 25111,'1971-F12','1971-12-03 13:00:01','orbital','failure',1508,'7801421-137','','Zenit-2M','','Sat',475,'',0 from dual union all
select 25112,'1971-F13','1971-12-04 22:33:00','orbital','failure',178,'5503A','AFP-827','CANYON 4','','Sat',228,'',40 from dual union all
select 25113,'1971-F14','1971-12-05 16:20:00','orbital','failure',432,'No. 4','D-2A','D-2A Polaire','','Sat',343,'',300 from dual union all
select 25114,'1971-M01','1971-02-03 01:35:00','suborbital rocket','success',1076,'SB-III A12','','','','Test',1476,'',75 from dual union all
select 25115,'1971-M02','1971-02-03 06:30:00','suborbital rocket','success',1076,'SB-III A13','','','','Test',1476,'',75 from dual union all
select 25116,'1971-M03','1971-03-15 06:01:00','suborbital rocket','success',839,'NASA 10.375GM','GSFC Ozone','','','Aeronomy',340,'',73 from dual union all
select 25117,'1971-M04','1971-03-16 13:16:00','suborbital rocket','success',839,'NASA 10.376GM','GSFC Ozone','','','Aeronomy',340,'',76 from dual;
insert into LAUNCH
select 25118,'1971-M05','1971-03-22 22:50:00','suborbital rocket','success',1189,'INTA AB-7101','','','','Aeron',396,'',65 from dual union all
select 25119,'1971-M06','1971-03-23 22:15:00','suborbital rocket','success',1189,'INTA AB-7102','','','','Aeron',396,'',72 from dual union all
select 2512,'1954-A493','1954-08-27 00:00:00','atmospheric rocket','success',836,'2699T','','','','Test',1895,'',10 from dual union all
select 25120,'1971-M07','1971-03-23 23:21:00','suborbital rocket','success',1189,'INTA AB-7103','','','','Aeron',396,'',72 from dual union all
select 25121,'1971-M08','1971-03-24 19:42:00','suborbital rocket','success',1189,'INTA AB-7104','','','','Aeron',396,'',74 from dual union all
select 25122,'1971-M09','1971-03-24 20:20:00','suborbital rocket','success',1189,'INTA AB-7105','','','','Aeron',396,'',74 from dual union all
select 25123,'1971-M10','1971-05-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 25124,'1971-M11','1971-08-27 09:00:00','suborbital rocket','success',263,'','','','','Particles',1047,'',50 from dual union all
select 25125,'1971-M12','1971-09-06 01:35:00','suborbital rocket','success',1076,'SB-III A14','','','','Test',1476,'',75 from dual union all
select 25126,'1971-M13','1971-09-11 01:45:00','suborbital rocket','success',1076,'SB-III A15','','','','Test',1476,'',75 from dual union all
select 25127,'1971-M14','1971-09-29 00:00:00','suborbital rocket','success',1069,'S-160-3','Para. Test','','','Test',839,'',80 from dual union all
select 25128,'1971-M15','1971-11-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 25129,'1971-M16','1971-12-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 2513,'1954-A494','1954-08-31 00:00:00','atmospheric rocket','success',836,'2762T','','','','Test',1895,'',10 from dual union all
select 25130,'1971-M17','1971-03-12 22:42:20','suborbital rocket','failure',920,'P92K','Auroral electrons','','','Auroral',413,'',60 from dual union all
select 25131,'1971-M18','1971-01-16 11:45:00','suborbital rocket','success',631,'INTA H-7102','','','','Meteo',396,'',61 from dual union all
select 25132,'1971-M19','1971-02-24 11:30:00','suborbital rocket','success',631,'INTA H-7105','','','','Meteo',396,'',65 from dual union all
select 25133,'1971-M20','1971-03-10 11:35:00','suborbital rocket','success',631,'INTA H-7108','','','','Meteo',396,'',65 from dual union all
select 25134,'1971-M21','1971-04-14 13:40:00','suborbital rocket','success',716,'INTA HT-7102','','','','Meteo',396,'',60 from dual union all
select 25135,'1971-M22','1971-05-12 11:30:00','suborbital rocket','success',716,'INTA HT-7105','','','','Meteo',396,'',60 from dual union all
select 25136,'1971-M23','1971-05-26 11:34:00','suborbital rocket','success',716,'INTA HT-7107','','','','Meteo',396,'',60 from dual union all
select 25137,'1971-M24','1971-06-30 11:34:00','suborbital rocket','success',716,'INTA HT-7109','','','','Meteo',396,'',60 from dual union all
select 25138,'1971-M25','1971-07-21 11:35:00','suborbital rocket','success',716,'INTA HT-7111','','','','Meteo',396,'',60 from dual union all
select 25139,'1971-M26','1971-10-06 11:50:00','suborbital rocket','success',716,'INTA HT-7118','','','','Meteo',396,'',60 from dual union all
select 2514,'1954-A495','1954-09-01 00:00:00','atmospheric rocket','unknown',576,'384M','','','','Test',1895,'',10 from dual union all
select 25140,'1971-M27','1971-12-15 22:19:00','suborbital rocket','success',716,'INTA HT-7121','','','','Meteo',396,'',60 from dual union all
select 25141,'1971-M28','1971-11-05 00:00:00','suborbital rocket','success',276,'','','','','Test',310,'',60 from dual union all
select 25142,'1971-M29','1971-02-11 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 1A','','','Aeron',1849,'',60 from dual union all
select 25143,'1971-M30','1971-02-11 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 1B','','','Aeron',1849,'',60 from dual union all
select 25144,'1971-M31','1971-06-02 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 6','','','Meteo',1849,'',30 from dual union all
select 25145,'1971-M32','1971-06-02 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 7','','','Meteo',1849,'',30 from dual union all
select 25146,'1971-M33','1971-06-21 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 8','','','Meteo',1849,'',60 from dual union all
select 25147,'1971-M34','1971-07-27 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 11','','','Meteo',1849,'',60 from dual union all
select 25148,'1971-M35','1971-08-03 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 12','','','Meteo',1849,'',60 from dual union all
select 25149,'1971-M36','1971-11-05 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 15','','','Meteo',1849,'',60 from dual union all
select 2515,'1954-A496','1954-09-01 00:00:00','atmospheric rocket','success',863,'1249B-2549','Nike 367R T48E2','','','Test',1895,'',10 from dual union all
select 25150,'1971-M37','1971-11-09 00:50:00','suborbital rocket','success',716,'HASP','WRE HASP 16','','','Meteo',1849,'',60 from dual union all
select 25151,'1971-M38','1971-11-11 01:00:00','suborbital rocket','success',716,'HASP','WRE HASP 17','','','Meteo',1849,'',60 from dual union all
select 25152,'1971-M39','1971-11-18 00:00:00','suborbital rocket','success',716,'HASP','WRE HASP 18','','','Meteo',1849,'',60 from dual union all
select 25153,'1971-S01','1971-01-01 00:00:00','military missile','success',6,'','LKI','','','Test',183,'',300 from dual union all
select 25154,'1971-S02','1971-01-01 00:00:00','military missile','success',6,'','LKI','','','Test',183,'',300 from dual union all
select 25155,'1971-S03','1971-01-01 00:00:00','military missile','success',6,'','LKI','','','Test',183,'',300 from dual union all
select 25156,'1971-S04','1971-01-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 25157,'1971-S05','1971-01-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 25158,'1971-S06','1971-01-01 00:00:00','military missile','success',751,'C5','C5','','','OT',307,'S655',500 from dual union all
select 25159,'1971-S07','1971-01-01 00:00:00','military missile','success',751,'C6','C6','','','OT',307,'S655',500 from dual union all
select 2516,'1954-A497','1954-09-01 00:00:00','atmospheric rocket','success',836,'2725R','Nike 368R','','','Test',1895,'',10 from dual union all
select 25160,'1971-S08','1971-01-01 00:00:00','military missile','success',751,'C7','C7','','','OT',307,'S655',500 from dual union all
select 25161,'1971-S09','1971-01-01 00:00:00','military missile','success',751,'C8','C8','','','OT',307,'S655',500 from dual union all
select 25162,'1971-S10','1971-01-01 00:00:00','military missile','success',752,'','','','','OT',307,'S655',500 from dual union all
select 25163,'1971-S100','1971-01-28 15:55:00','suborbital rocket','success',867,'DLR A-NT-38','MPE Ba/Erika','','','Ionos/Fields',29,'',276 from dual union all
select 25164,'1971-S101','1971-01-28 18:33:46','suborbital rocket','success',1204,'S46/2','ESRO S46/2','','','Ionospheric',416,'',218 from dual union all
select 25165,'1971-S102','1971-01-28 22:57:00','suborbital rocket','success',1349,'SECEDE','SECEDE II Probe F','','','Aeron/Fields',394,'',300 from dual union all
select 25166,'1971-S103','1971-01-29 07:55:00','suborbital rocket','success',1207,'SL1001','MSSL XR7','','','XR Astronomy',1839,'',203 from dual union all
select 25167,'1971-S104','1971-01-29 17:03:00','suborbital rocket','success',837,'NASA 14.167GI','','','','Ionos',858,'',132 from dual union all
select 25168,'1971-S105','1971-01-29 23:52:00','suborbital rocket','success',1268,'SECEDE R16','SECEDE II OLIVE','','','Aeron',394,'',190 from dual union all
select 25169,'1971-S106','1971-01-31 13:55:00','suborbital rocket','success',837,'NASA 14.373GI','','','','Ionos',858,'',185 from dual union all
select 2517,'1954-A498','1954-09-01 00:00:00','atmospheric rocket','success',863,'1249B-2529','Nike 366R T48E2','','','Test',1895,'',10 from dual union all
select 25170,'1971-S107','1971-02-01 01:30:00','suborbital rocket','success',613,'JCR-5','','','','Test',1476,'',150 from dual union all
select 25171,'1971-S108','1971-02-01 22:57:00','suborbital rocket','success',1349,'SECEDE R18','SECEDE II SPRUCE II','(Probe C)','','Ionos/Fields',394,'',260 from dual union all
select 25172,'1971-S109','1971-02-02 15:45:00','military missile','success',1451,'52447-322T1','8K84M LKI-1','GCh 15F842P1','','Test',526,'',1000 from dual union all
select 25173,'1971-S11','1971-05-29 00:00:00','military missile','success',982,'G 22504-13T','8K64U UBP-152','','','OT',596,'',1210 from dual union all
select 25174,'1971-S110','1971-02-02 16:35:00','suborbital rocket','success',1349,'SECEDE R19','SECEDE II QUINCE II','(Probe A)','','Aeron/Ionos/Fields',394,'',300 from dual union all
select 25175,'1971-S111','1971-02-02 16:36:00','suborbital rocket','success',1349,'SECEDE R20','SECEDE II QUINCE II','(Probe E)','','Aeron/Fields',394,'',300 from dual union all
select 25176,'1971-S112','1971-02-03 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT101F','GLORY TRIP 101F','OT',1710,'',1300 from dual union all
select 25177,'1971-S113','1971-02-05 05:20:00','military missile','success',796,'','Mk 11 RV','OT GT60M','GLORY TRIP 60M','OT',1699,'',1300 from dual union all
select 25178,'1971-S114','1971-02-04 22:45:00','military missile','success',939,'C3E-19','DASO-3','','','OT',421,'SSBN 632',500 from dual union all
select 25179,'1971-S115','1971-02-05 22:46:00','suborbital rocket','success',220,'AKF-4B-24','Impedance dipole','','','Auroral/Ionos/Meteo',428,'',784 from dual union all
select 2518,'1954-A499','1954-09-02 00:00:00','atmospheric rocket','success',442,'','D4 Model','','','Test',1821,'',10 from dual union all
select 25180,'1971-S116','1971-02-06 18:30:00','suborbital rocket','success',1206,'S80/1','ESRO S80/1','','','Ionospheric',1343,'',222 from dual union all
select 25181,'1971-S117','1971-02-06 23:22:00','suborbital rocket','success',1206,'S80/2','ESRO S80/2','','','Ionospheric',1343,'',226 from dual union all
select 25182,'1971-S118','1971-02-07 03:45:00','suborbital rocket','success',1206,'S80/3','ESRO S80/3','','','Ionospheric',1343,'',214 from dual union all
select 25183,'1971-S119','1971-02-08 00:00:00','military missile','success',795,'','Mk 11 RV','FOT GT91B','GLORY TRIP 91B','OT',1702,'',1300 from dual union all
select 25184,'1971-S12','1971-09-25 00:00:00','military missile','success',982,'G 22504-14T','8K64U UBP-156','','','OT',596,'',1210 from dual union all
select 25185,'1971-S120','1971-02-08 13:56:00','suborbital rocket','success',37,'NASA 13.51UA','','','','Aeronomy',1895,'',265 from dual union all
select 25186,'1971-S121','1971-02-12 00:00:00','military missile','success',1451,'8K84M','8K84M LKI-2','','','Test',526,'',1000 from dual union all
select 25187,'1971-S122','1971-02-14 08:17:00','suborbital rocket','success',867,'NASA 18.110UE','','','','Plasma',1047,'',210 from dual union all
select 25188,'1971-S123','1971-02-17 07:25:00','military missile','success',799,'','Mk 12 RV x 3','DASO OLD FOX 05M','OLD FOX 05M','OT',1697,'',1300 from dual union all
select 25189,'1971-S124','1971-02-17 06:47:00','military missile','success',999,'4501421-180T','8K67 UBP-44','GCh 8F675','','OT',518,'',1000 from dual union all
select 2519,'1954-A50','1954-02-10 00:00:00','atmospheric rocket','unknown',576,'217M','','','','Test',1895,'',10 from dual union all
select 25190,'1971-S125','1971-02-20 03:23:05','suborbital rocket','success',219,'AKF-4A-18','Auroral emissions','','','Auroral/Ionos/Plasm',428,'',785 from dual union all
select 25191,'1971-S126','1971-02-22 00:43:00','suborbital rocket','success',44,'NASA 17.09UG','CAL XR 6 Polarimeter','','','XR Astron',1821,'',196 from dual union all
select 25192,'1971-S127','1971-02-22 11:40:00','suborbital rocket','failure',1207,'ESRO S85','MSSL UVS-2','','','UV Astronomy',1839,'',222 from dual union all
select 25193,'1971-S128','1971-02-23 00:00:00','military missile','success',795,'','SSTTP','SSTTP M1-28','','Target',1701,'',1300 from dual union all
select 25194,'1971-S129','1971-02-23 18:09:00','suborbital rocket','success',1400,'','ELECTRE','','','Reentry Test/Aeron',307,'',130 from dual union all
select 25195,'1971-S13','1971-06-24 00:00:00','military missile','success',982,'G 22502-06T','8K64U UBP-153','Svinets (Salyut target)','','OT',596,'',1210 from dual union all
select 25196,'1971-S130','1971-02-25 02:00:00','military missile','success',796,'','Mk 11 RV','FOT GT102F','GLORY TRIP 102F','OT',1711,'',1300 from dual union all
select 25197,'1971-S131','1971-02-24 14:40:00','suborbital rocket','success',30,'NASA 04.249NA','Luster (Encke)','','','Aeronomy/Dust',1869,'',200 from dual union all
select 25198,'1971-S132','1971-02-25 01:13:00','suborbital rocket','success',227,'NRL NF9.','Airglow/FIR BG','','','IR Astron/Aeron',1821,'',275 from dual union all
select 25199,'1971-S133','1971-02-26 10:18:00','suborbital rocket','failure',1207,'ESRO S54','MSSL SX6','','','Solar X',1343,'',250 from dual union all
select 252,'1944-S229','1944-11-05 13:33:00','military missile','success',1470,'19500','Ma256','','','Test',661,'',90 from dual union all
select 2520,'1954-A500','1954-09-02 00:00:00','atmospheric rocket','success',836,'2691T','','','','Test',1895,'',10 from dual union all
select 25200,'1971-S134','1971-02-27 00:00:00','military missile','success',796,'','Mk 11 RV','OT GT80M','GLORY TRIP 80M','OT',1699,'',1300 from dual union all
select 25201,'1971-S135','1971-03-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 25202,'1971-S136','1971-03-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 25203,'1971-S137','1971-02-28 07:53:00','suborbital rocket','success',217,'AND-3-41','Microwave background','','','Submm Astron',428,'',152 from dual union all
select 25204,'1971-S138','1971-03-01 09:15:00','suborbital rocket','success',37,'NASA 13.22UG','','','','UV Astron?',1895,'',200 from dual union all
select 25205,'1971-S139','1971-03-01 11:40:00','suborbital rocket','success',30,'KP3.32','Venus LyAlpha','','','UV Astron',1869,'',160 from dual union all
select 25206,'1971-S14','1971-06-28 00:00:00','military missile','success',982,'G 22503-19T','8K64U UBP-154','Svinets','','OT',596,'',1210 from dual union all
select 25207,'1971-S140','1971-03-02 00:00:00','military missile','success',795,'','Mk 11 RV','FOT GT92B-3','GLORY TRIP 92B-3','OT',1698,'',1300 from dual;
insert into LAUNCH
select 25208,'1971-S141','1971-03-02 10:08:00','suborbital rocket','success',331,'WRE Cockatoo 110','Sphere/MPE Ba','','','Aeron',1835,'',218 from dual union all
select 25209,'1971-S142','1971-03-02 10:33:00','suborbital rocket','unknown',740,'MR-12','MX-6407 Mass spec','','','Aeron/Ionos',636,'',150 from dual union all
select 2521,'1954-A501','1954-09-02 00:00:00','atmospheric rocket','success',836,'2513R','Nike 369R','','','Test',1895,'',10 from dual union all
select 25210,'1971-S143','1971-03-02 10:08:00','suborbital rocket','success',1206,'SL922','UCL-MI1 Ionos Curr.','','','Ionospheric',1839,'',223 from dual union all
select 25211,'1971-S144','1971-03-02 12:10:00','suborbital rocket','success',27,'WRE AH1023 (AH4)','','','','Aeron',1833,'',140 from dual union all
select 25212,'1971-S145','1971-03-02 19:54:00','suborbital rocket','success',27,'WRE H1024 (AH5/16)','','','','Aeron',1833,'',186 from dual union all
select 25213,'1971-S146','1971-03-03 06:52:00','suborbital rocket','success',227,'AAF-VB-32','Electrons/ions','','','Auroral',428,'',275 from dual union all
select 25214,'1971-S147','1971-03-05 14:00:00','suborbital rocket','success',839,'NASA 12.19GM','Range Test','','','Aeronomy',342,'',100 from dual union all
select 25215,'1971-S148','1971-03-10 17:38:00','suborbital rocket','success',839,'NASA 10.382GM','GSFC Grenade 243 W','','','Aeronomy',1821,'',113 from dual union all
select 25216,'1971-S149','1971-03-10 17:59:00','suborbital rocket','success',837,'NASA 14.478UM','UM Pitot 53','','','Aeronomy',1821,'',179 from dual union all
select 25217,'1971-S15','1971-10-23 00:00:00','military missile','success',982,'R 22601-10T','8K64U UBP-158','','','OT',596,'',1210 from dual union all
select 25218,'1971-S150','1971-03-10 18:26:00','suborbital rocket','success',837,'NASA 14.479UM','UM Pitot 54','','','Aeronomy',1821,'',181 from dual union all
select 25219,'1971-S151','1971-03-10 18:41:00','suborbital rocket','success',837,'NASA 14.480UM','UM Pitot 55','','','Aeronomy',1821,'',171 from dual union all
select 2522,'1954-A502','1954-09-08 00:00:00','atmospheric rocket','unknown',576,'435M','','','','Test',1895,'',10 from dual union all
select 25220,'1971-S152','1971-03-10 19:11:00','suborbital rocket','success',839,'NASA 10.383GM','GSFC Grenade 244 W','','','Aeronomy',1821,'',113 from dual union all
select 25221,'1971-S153','1971-03-11 00:00:00','military missile','success',135,'B035','RVTO-2B-3','ABRES AS-3 (USAF)','','Reentry Test',614,'',200 from dual union all
select 25222,'1971-S154','1971-03-11 08:56:00','suborbital rocket','success',1207,'ESRO S55','LUX BCS 5','','','XR Astronomy/Solar X',1343,'',159 from dual union all
select 25223,'1971-S155','1971-03-11 10:37:00','suborbital rocket','success',867,'Caribou V','','','','Auroral UV',1047,'',270 from dual union all
select 25224,'1971-S156','1971-03-12 04:13:00','suborbital rocket','success',848,'CRL A07.015-3','Sphere','','','Aeron',1895,'',280 from dual union all
select 25225,'1971-S157','1971-03-12 06:55:00','suborbital rocket','success',848,'CRL A07.913-3','Sphere','','','Aeron',1895,'',283 from dual union all
select 25226,'1971-S158','1971-03-02 21:14:20','suborbital rocket','success',1204,'ESRO S16/2','ESRO S16/2','','','Ionospheric',416,'',143 from dual union all
select 25227,'1971-S159','1971-03-12 21:49:50','suborbital rocket','success',920,'P69K','Auroral electrons','','','Auroral',413,'',153 from dual union all
select 25228,'1971-S16','1971-12-24 00:00:00','military missile','success',982,'G 22602-07T','8K64U UBP-160','','','OT',596,'',1210 from dual union all
select 25229,'1971-S160','1971-03-13 00:50:00','suborbital rocket','success',837,'NASA 14.388UA','','','','Aeronomy',1895,'',200 from dual union all
select 2523,'1954-A503','1954-09-08 00:00:00','atmospheric rocket','success',836,'2757R','Nike 371R','','','Test',1895,'',10 from dual union all
select 25230,'1971-S161','1971-03-13 16:15:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',150 from dual union all
select 25231,'1971-S162','1971-03-13 22:55:00','suborbital rocket','success',740,'MR-12','Wind trail','','','Aeron',1010,'',150 from dual union all
select 25232,'1971-S163','1971-03-14 16:25:00','suborbital rocket','success',740,'MR-12','ne/Te 71-1','','','Ionos',1010,'',158 from dual union all
select 25233,'1971-S164','1971-03-15 17:19:00','suborbital rocket','success',867,'NASA 18.126GE','','','','Plasma',448,'',315 from dual union all
select 25234,'1971-S165','1971-03-15 00:00:00','suborbital rocket','success',867,'NASA 18.122GE','','','','Plasma',333,'',270 from dual union all
select 25235,'1971-S166','1971-03-15 02:35:00','military missile','success',1450,'4764711-543P','15A20 Kontrol''niy','GCh 15B20','','Test',505,'',1000 from dual union all
select 25236,'1971-S167','1971-03-15 17:55:16','suborbital rocket','success',1204,'ESRO S87/1','ESRO S87/1','','','Ionospheric',416,'',188 from dual union all
select 25237,'1971-S168','1971-03-15 18:17:00','suborbital rocket','success',740,'MR-12','Mass spectrometer','','','Aeron',1010,'',150 from dual union all
select 25238,'1971-S169','1971-03-15 23:00:00','military missile','success',939,'C3E-39','DASO-4','','','OT',421,'SSBN 632',500 from dual union all
select 25239,'1971-S17','1971-12-04 00:00:00','military missile','success',982,'G 22602-11T','8K64U UBP-159','','','OT',596,'',1210 from dual union all
select 2524,'1954-A504','1954-09-08 00:00:00','atmospheric rocket','success',836,'2695T','','','','Test',1895,'',10 from dual union all
select 25240,'1971-S170','1971-03-16 09:05:00','suborbital rocket','success',867,'NASA 18.14UE','','','','Plasma',1047,'',270 from dual union all
select 25241,'1971-S171','1971-03-16 19:00:00','suborbital rocket','success',37,'CRL A04.002-2','EUV','','','Solar EUV',1895,'',270 from dual union all
select 25242,'1971-S172','1971-03-17 00:00:00','military missile','success',795,'','SSTTP','SSTTP M1-13','','Target',1701,'',1300 from dual union all
select 25243,'1971-S173','1971-03-17 10:57:00','suborbital rocket','success',867,'NASA 18.95UA','','','','Aeronomy',1047,'',270 from dual union all
select 25244,'1971-S174','1971-03-17 16:55:00','suborbital rocket','success',740,'MR-12','Wind trail','','','Aeron',1010,'',150 from dual union all
select 25245,'1971-S175','1971-03-18 17:00:00','suborbital rocket','success',867,'NASA 18.127GE','','','','Plasma',448,'',318 from dual union all
select 25246,'1971-S176','1971-03-18 17:10:00','suborbital rocket','success',740,'MR-12','Wind trail','','','Aeron',1010,'',150 from dual union all
select 25247,'1971-S177','1971-03-18 19:39:00','suborbital rocket','success',839,'NASA 10.380GM','GSFC Grenade 245 CSG','','','Aeronomy',340,'',125 from dual union all
select 25248,'1971-S178','1971-03-19 00:00:00','suborbital rocket','success',867,'NASA 18.129GE','','','','Plasma',448,'',250 from dual union all
select 25249,'1971-S179','1971-03-19 00:00:00','suborbital rocket','success',867,'NASA 18.123GE','','','','Plasma',448,'',270 from dual union all
select 2525,'1954-A505','1954-09-08 00:00:00','atmospheric rocket','success',836,'2456A','','','','Test',1895,'',10 from dual union all
select 25250,'1971-S18','1971-02-03 00:00:00','military missile','success',982,'G 22502-19T','8K64U UBP-148','','','OT',596,'',1210 from dual union all
select 25251,'1971-S180','1971-03-19 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 659',1000 from dual union all
select 25252,'1971-S181','1971-03-19 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 659',1000 from dual union all
select 25253,'1971-S182','1971-03-19 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 659',1000 from dual union all
select 25254,'1971-S183','1971-03-19 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 659',1000 from dual union all
select 25255,'1971-S184','1971-03-19 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 659',1000 from dual union all
select 25256,'1971-S185','1971-03-19 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 659',1000 from dual union all
select 25257,'1971-S186','1971-03-19 12:05:00','suborbital rocket','success',867,'NASA 18.94UA','','','','Aeronomy',1047,'',270 from dual union all
select 25258,'1971-S187','1971-03-19 12:38:00','suborbital rocket','success',867,'','','','','Ionos/Auroral',1047,'',270 from dual union all
select 25259,'1971-S188','1971-03-20 00:00:00','suborbital rocket','success',867,'NASA 18.124GE','','','','Plasma',448,'',270 from dual union all
select 2526,'1954-A506','1954-09-08 00:00:00','atmospheric rocket','success',836,'2563R','Nike 372R','','','Test',1895,'',10 from dual union all
select 25260,'1971-S189','1971-03-20 05:30:00','suborbital rocket','failure',37,'NASA 13.21UG','','','','XR Astron',1895,'',200 from dual union all
select 25261,'1971-S19','1971-03-19 00:00:00','military missile','success',982,'G 22503-18T','8K64U UBP-149','','','OT',596,'',1210 from dual union all
select 25262,'1971-S190','1971-03-20 05:42:41','suborbital rocket','success',867,'CRL A08.019-1','','','','Auroral',428,'',336 from dual union all
select 25263,'1971-S191','1971-03-20 05:44:01','suborbital rocket','success',848,'CRL A07.019-2','','','','Auroral',428,'',164 from dual union all
select 25264,'1971-S192','1971-03-20 07:53:00','suborbital rocket','success',215,'AHF-2-115','Auroral VUV','','','Test/Auroral/Ionos',428,'',151 from dual union all
select 25265,'1971-S193','1971-03-20 08:15:00','suborbital rocket','success',867,'NASA 18.17UE','','','','Plasma',1047,'',270 from dual union all
select 25266,'1971-S194','1971-03-21 00:00:00','suborbital rocket','success',867,'NASA 18.125GE','','','','Plasma',448,'',270 from dual union all
select 25267,'1971-S195','1971-03-22 16:40:00','suborbital rocket','success',1281,'PCA imped probe','AFCRL Arcas 70-1E','','','Ionos',1470,'',112 from dual union all
select 25268,'1971-S196','1971-03-22 16:50:00','suborbital rocket','success',867,'NASA 18.128GE','','','','Plasma',448,'',265 from dual union all
select 25269,'1971-S197','1971-03-24 07:25:00','military missile','success',799,'','Mk 12 RV x 3','OT GT01GM','GLORY TRIP 01GM','OT',1703,'',1300 from dual union all
select 2527,'1954-A507','1954-09-08 00:00:00','atmospheric rocket','success',836,'2512A','','','','Test',1895,'',10 from dual union all
select 25270,'1971-S198','1971-03-24 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 643',1000 from dual union all
select 25271,'1971-S199','1971-03-24 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 643',1000 from dual union all
select 25272,'1971-S20','1971-03-27 00:00:00','military missile','success',982,'G 22502-04T','8K64U UBP-150','','','OT',596,'',1210 from dual union all
select 25273,'1971-S200','1971-03-24 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 643',1000 from dual union all
select 25274,'1971-S201','1971-03-24 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 643',1000 from dual union all
select 25275,'1971-S202','1971-03-24 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 643',1000 from dual union all
select 25276,'1971-S203','1971-03-24 00:00:00','military missile','unknown',932,'A3','FOT','','','OT',421,'SSBN 643',1000 from dual union all
select 25277,'1971-S204','1971-03-24 05:26:00','suborbital rocket','success',228,'CRL A18.006-3','Airglow/Auroral IR','','','Aeron',428,'',366 from dual union all
select 25278,'1971-S205','1971-03-24 18:32:00','suborbital rocket','success',1204,'ESRO S87/2','ESRO S87/2','','','Ionospheric',416,'',190 from dual union all
select 25279,'1971-S206','1971-03-25 13:50:00','suborbital rocket','failure',1199,'ESRO S56B/1','LUX-55/RSRS','','','Ionos/Solar UV,X',1343,'',130 from dual union all
select 2528,'1954-A508','1954-09-08 00:00:00','atmospheric rocket','success',836,'2625R','Nike 370R','','','Test',1895,'',10 from dual union all
select 25280,'1971-S207','1971-03-26 15:06:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',150 from dual union all
select 25281,'1971-S208','1971-03-27 02:44:00','suborbital rocket','success',30,'NRL NB3.245','','','','XR Astron?',1869,'',200 from dual union all
select 25282,'1971-S209','1971-03-27 12:26:00','suborbital rocket','success',867,'','','','','Ionos/Auroral',1047,'',270 from dual union all
select 25283,'1971-S21','1971-05-25 00:00:00','military missile','success',982,'G 22501-06T','8K64U UBP-151','Svinets ("Lead")','','OT',596,'',1210 from dual union all
select 25284,'1971-S210','1971-03-28 19:46:00','suborbital rocket','success',740,'MR-12','ne/Te 71-2','','','Ionos',1010,'',183 from dual union all
select 25285,'1971-S211','1971-03-29 08:45:00','suborbital rocket','success',740,'MR-12','ne/Te 71-3','','','Ionos',1010,'',183 from dual union all
select 25286,'1971-S212','1971-03-29 20:10:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',150 from dual union all
select 25287,'1971-S213','1971-03-29 23:05:00','military missile','success',939,'C3E-12','DASO-5','','','OT',421,'SSBN 631',500 from dual union all
select 25288,'1971-S214','1971-03-30 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 611',1000 from dual union all
select 25289,'1971-S215','1971-03-30 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 611',1000 from dual union all
select 2529,'1954-A509','1954-09-08 00:00:00','atmospheric rocket','success',836,'2617R','Nike 373R','','','Test',1895,'',10 from dual union all
select 25290,'1971-S216','1971-03-30 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 611',1000 from dual union all
select 25291,'1971-S217','1971-03-30 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 611',1000 from dual union all
select 25292,'1971-S218','1971-03-30 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 611',1000 from dual union all
select 25293,'1971-S219','1971-03-30 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 611',1000 from dual union all
select 25294,'1971-S22','1971-08-06 00:00:00','military missile','success',982,'G 22501-12T','8K64U UBP-155','','','OT',596,'',1210 from dual union all
select 25295,'1971-S220','1971-03-30 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 611',1000 from dual union all
select 25296,'1971-S221','1971-04-01 08:40:00','military missile','success',796,'','Mk 11 RV','OT GT82M','GLORY TRIP 82M','OT',1700,'',1300 from dual union all
select 25297,'1971-S222','1971-03-31 07:51:00','suborbital rocket','success',867,'NASA 18.130CE','','','','Auroral',428,'',298 from dual union all
select 25298,'1971-S223','1971-04-01 22:40:00','suborbital rocket','success',740,'MR-12','ne/Te 71-4','','','Ionos',1010,'',186 from dual;
insert into LAUNCH
select 25299,'1971-S225','1971-04-03 09:50:00','suborbital rocket','success',37,'CRL A04.004-2','HI STAR 1','','','IR Astron',1873,'',158 from dual union all
select 253,'1944-S23','1944-04-14 00:00:00','military missile','success',8,'17.063','','','','Test',675,'',90 from dual union all
select 2530,'1954-A51','1954-02-10 00:00:00','atmospheric rocket','success',836,'1309R','Nike 265R','','','Test',1895,'',10 from dual union all
select 25300,'1971-S226','1971-04-03 17:00:00','military missile','success',136,'H001JD','ROCS','ABRES ROCS','','Reentry Test',615,'',200 from dual union all
select 25301,'1971-S227','1971-04-05 08:43:00','military missile','success',162,'85F','LAR-1/LAR-3','ABRES LAR-1','','Reentry Test',1680,'',1400 from dual union all
select 25302,'1971-S228','1971-04-05 23:53:50','suborbital rocket','success',920,'P68K','Small-scale struc.','','','Ionospheric',413,'',134 from dual union all
select 25303,'1971-S229','1971-04-06 18:00:00','suborbital rocket','success',1281,'PCA imped. probe','AFCRL Arcas 70-2E','','','Ionos',1470,'',111 from dual union all
select 25304,'1971-S23','1971-01-01 00:00:00','military missile','success',1514,'','','','','Test',989,'',1000 from dual union all
select 25305,'1971-S230','1971-04-07 04:30:00','suborbital rocket','success',1281,'PCA imped. probe','AFCRL Arcas 70-3E','','','Ionos',1470,'',122 from dual union all
select 25306,'1971-S231','1971-04-08 10:40:00','military missile','success',799,'STM-2W','Mk 12 RV x 3','R&D','','Test',1707,'',1300 from dual union all
select 25307,'1971-S232','1971-04-14 22:27:10','suborbital rocket','success',920,'P61K','Particles','','','Ionospheric',413,'',154 from dual union all
select 25308,'1971-S233','1971-04-15 00:00:00','military missile','success',929,'A2S-01','Safeguard M1-7','','','Target',1061,'EAG-154',1000 from dual union all
select 25309,'1971-S234','1971-04-15 00:30:30','suborbital rocket','success',920,'P66K','Winds/E field','','','Auroral',413,'',131 from dual union all
select 2531,'1954-A510','1954-09-09 00:00:00','military missile','unknown',339,'1407R','','','','Test',1895,'',30 from dual union all
select 25310,'1971-S235','1971-04-16 05:25:10','suborbital rocket','success',920,'P62K','LP/VLF 2','','','Ionospheric',413,'',138 from dual union all
select 25311,'1971-S236','1971-04-19 06:30:00','military missile','success',135,'V127D','SAM/AST-01','ABRES SAMAST','','Reentry Test',614,'',200 from dual union all
select 25312,'1971-S237','1971-04-19 13:25:00','suborbital rocket','success',837,'NASA 14.406IA','ISRO 10.41','TERLS-303','','Aeronomy',1467,'',198 from dual union all
select 25313,'1971-S238','1971-04-19 23:45:00','military missile','success',939,'C3E-44','DASO-6','','','OT',421,'SSBN 630',500 from dual union all
select 25314,'1971-S239','1971-04-21 00:00:00','military missile','success',1251,'','SAFEGUARD M1-7A','','','Interceptor',868,'',100 from dual union all
select 25315,'1971-S24','1971-09-25 14:00:00','suborbital rocket','success',1281,'PCA','AFCRL Arcas 71-1','','','Ionos',1470,'',100 from dual union all
select 25316,'1971-S240','1971-04-21 18:59:20','suborbital rocket','success',920,'P64K','E field','','','Ionospheric',413,'',143 from dual union all
select 25317,'1971-S241','1971-04-21 21:42:50','suborbital rocket','success',920,'P67K','Winds/E field','','','Auroral',413,'',128 from dual union all
select 25318,'1971-S242','1971-04-21 23:03:30','suborbital rocket','success',920,'P63K','E field','','','Ionospheric',413,'',165 from dual union all
select 25319,'1971-S243','1971-04-22 13:00:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',636,'',150 from dual union all
select 2532,'1954-A511','1954-09-10 00:00:00','military missile','unknown',339,'1445E','','','','Test',1895,'',30 from dual union all
select 25320,'1971-S244','1971-04-22 14:15:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',636,'',150 from dual union all
select 25321,'1971-S245','1971-04-23 09:25:00','military missile','success',799,'','Mk 12 RV x 3','DASO OLD FOX 06F','OLD FOX 06F','OT',1712,'',1300 from dual union all
select 25322,'1971-S246','1971-04-23 01:18:00','suborbital rocket','success',837,'NASA 14.465UA','','','','Aeronomy',1895,'',200 from dual union all
select 25323,'1971-S247','1971-04-23 01:47:35','suborbital rocket','success',837,'NASA 14.467UA','MPIK Dust','','','Meteorites',1895,'',124 from dual union all
select 25324,'1971-S248','1971-04-24 23:35:55','suborbital rocket','success',920,'P54K','E field','','','Ionospheric',413,'',162 from dual union all
select 25325,'1971-S249','1971-04-25 06:00:00','suborbital rocket','success',837,'NASA 14.407II/IE','ISRO 10.40/20.11','TERLS-304','','Ionos/Plasma',1467,'',160 from dual union all
select 25326,'1971-S25','1971-10-24 14:30:00','suborbital rocket','success',1281,'PCA','AFCRL Arcas 71-5','','','Ionos',1470,'',100 from dual union all
select 25327,'1971-S250','1971-04-25 13:25:00','suborbital rocket','failure',837,'NASA 14.408IA','ISRO 10.39','TERLS-305','','Aeronomy',1467,'',1 from dual union all
select 25328,'1971-S251','1971-04-26 00:00:00','military missile','success',795,'','Mk 11 RV','FOT GT93B','GLORY TRIP 93B','OT',1702,'',1300 from dual union all
select 25329,'1971-S252','1971-04-26 00:00:00','suborbital rocket','success',839,'NASA 10.387II','','','','Ionos',396,'',100 from dual union all
select 2533,'1954-A512','1954-09-10 00:00:00','military missile','success',771,'GM-11074','','','','Test',222,'',12 from dual union all
select 25330,'1971-S253','1971-04-26 00:00:00','suborbital rocket','success',839,'NASA 10.388II','','','','Ionos',396,'',100 from dual union all
select 25331,'1971-S254','1971-04-26 19:34:00','suborbital rocket','success',839,'NASA 10.386II','','','','Ionos',396,'',100 from dual union all
select 25332,'1971-S255','1971-04-26 19:34:00','suborbital rocket','success',288,'C136','INTA C-7101','CNES FU-207A','','Aeronomy',396,'',129 from dual union all
select 25333,'1971-S256','1971-04-26 22:15:00','suborbital rocket','success',839,'NASA 10.319II','INTA NC-7101','','','Ionos',396,'',154 from dual union all
select 25334,'1971-S257','1971-04-27 18:30:00','military missile','success',917,'','P-202 71 Spring 01','','','OT',615,'',180 from dual union all
select 25335,'1971-S258','1971-04-27 19:00:00','military missile','success',917,'','P-203 71 Spring 02','','','OT',615,'',180 from dual union all
select 25336,'1971-S259','1971-04-27 20:00:00','military missile','success',917,'','P-204 71 Spring 03','','','OT',615,'',180 from dual union all
select 25337,'1971-S26','1971-04-06 17:19:00','suborbital rocket','success',1281,'PCA scintillator','AFCRL Arcas 70-4P','','','Ionos',1470,'',89 from dual union all
select 25338,'1971-S260','1971-04-27 20:30:00','military missile','success',917,'','P-205 71 Spring 04','','','OT',615,'',180 from dual union all
select 25339,'1971-S261','1971-04-29 16:32:00','suborbital rocket','success',37,'NASA 13.14CS','','','','Solar X',1895,'',213 from dual union all
select 2534,'1954-A513','1954-09-10 00:00:00','atmospheric rocket','success',836,'2458A','','','','Test',1895,'',10 from dual union all
select 25340,'1971-S262','1971-04-30 06:08:00','suborbital rocket','success',837,'NASA 14.411II','ISRO 10.42/20.12','TERLS-306','','Ionos',1467,'',171 from dual union all
select 25341,'1971-S263','1971-04-30 10:00:00','suborbital rocket','success',1069,'S-160JA-3','','','','Auroral',1458,'',83 from dual union all
select 25342,'1971-S264','1971-05-01 00:03:00','military missile','success',939,'C3E-70','DASO-7','','','OT',421,'SSBN 628',500 from dual union all
select 25343,'1971-S265','1971-05-01 09:00:00','suborbital rocket','success',30,'NASA 04.332GG','','','','UV Astronomy',1869,'',200 from dual union all
select 25344,'1971-S266','1971-05-01 09:35:00','suborbital rocket','success',37,'NASA 13.36UG','MIT-9','','','XR Astron',1895,'',200 from dual union all
select 25345,'1971-S267','1971-05-01 23:36:30','suborbital rocket','success',920,'P65K','E field','','','Ionospheric',413,'',146 from dual union all
select 25346,'1971-S268','1971-05-06 01:16:00','suborbital rocket','success',848,'CRL A07.902-7','Mass spectrometer','','','Aeron',394,'',120 from dual union all
select 25347,'1971-S269','1971-05-07 00:00:00','military missile','success',929,'A2S-03','Safeguard M1-8','','','Target',1061,'EAG-154',1000 from dual union all
select 25348,'1971-S27','1971-05-20 00:00:00','military missile','success',1449,'8K84','8K84 UBP-137','','','Test',536,'',1000 from dual union all
select 25349,'1971-S270','1971-05-08 16:15:00','suborbital rocket','success',848,'CRL A07.902-8','Mass spectrometer','','','Aeron',394,'',135 from dual union all
select 2535,'1954-A514','1954-09-10 00:00:00','atmospheric rocket','success',836,'2807T','','','','Test',1895,'',10 from dual union all
select 25350,'1971-S271','1971-05-10 12:03:00','suborbital rocket','failure',1211,'SL812','LUX XRS11 Survey','','','XR Astronomy',1839,'',219 from dual union all
select 25351,'1971-S272','1971-05-11 22:30:00','military missile','success',939,'C3E-2','DASO-8','','','OT',421,'SSBN 631',500 from dual union all
select 25352,'1971-S273','1971-05-11 23:47:00','military missile','success',939,'C3E-9','DASO-9','','','OT',421,'SSBN 631',500 from dual union all
select 25353,'1971-S274','1971-05-12 00:00:00','military missile','unknown',917,'','P-206 71 Spring 05','','','OT',615,'',180 from dual union all
select 25354,'1971-S275','1971-05-12 00:00:00','military missile','unknown',917,'','P-207 71 Spring 06','','','OT',615,'',180 from dual union all
select 25355,'1971-S276','1971-05-12 00:00:00','military missile','unknown',917,'','P-208 71 Spring 07','','','OT',615,'',180 from dual union all
select 25356,'1971-S277','1971-05-12 00:00:00','military missile','unknown',917,'','P-209 71 Spring 08','','','OT',615,'',180 from dual union all
select 25357,'1971-S278','1971-05-12 04:45:00','suborbital rocket','success',839,'NASA 10.280NA','','','','Aeronomy',1895,'',100 from dual union all
select 25358,'1971-S279','1971-05-13 01:10:00','suborbital rocket','success',839,'NASA 10.278NA','','','','Aeronomy',1895,'',100 from dual union all
select 25359,'1971-S28','1971-06-16 00:00:00','military missile','success',1449,'8K84','8K84 UBP-139','','','Test',503,'',1000 from dual union all
select 2536,'1954-A515','1954-09-10 00:00:00','atmospheric rocket','success',836,'2595A','','','','Test',1895,'',10 from dual union all
select 25360,'1971-S280','1971-05-14 00:00:00','suborbital rocket','success',1200,'S56B/2','LUX-57/RSRS','','','Ionos/Solar UV,X',1343,'',190 from dual union all
select 25361,'1971-S281','1971-05-14 00:00:00','military missile','success',135,'V146D','TRR-01','ABRES TRR01','','Reentry Test',614,'',200 from dual union all
select 25362,'1971-S282','1971-05-17 00:00:00','suborbital rocket','success',466,'E006','FU-203 recovery test','','','Test',340,'',196 from dual union all
select 25363,'1971-S283','1971-05-17 08:02:00','suborbital rocket','unknown',848,'','KAY','','','Aeron',394,'',152 from dual union all
select 25364,'1971-S284','1971-05-17 08:02:00','suborbital rocket','unknown',848,'','LYNN','','','Aeron',394,'',117 from dual union all
select 25365,'1971-S285','1971-05-19 06:20:00','military missile','success',796,'','Mk 11 RV','FOT GT105F','GLORY TRIP 105F','OT',1710,'',1300 from dual union all
select 25366,'1971-S286','1971-05-18 00:24:00','suborbital rocket','success',848,'CRL A07.913-5','Sphere','','','Aeron',394,'',145 from dual union all
select 25367,'1971-S287','1971-05-18 01:05:00','suborbital rocket','success',848,'CRL A07.917-5','Release','','','Aeron',394,'',180 from dual union all
select 25368,'1971-S288','1971-05-18 01:08:00','suborbital rocket','success',848,'CRL A07.918-2','Release','','','Aeron',394,'',150 from dual union all
select 25369,'1971-S289','1971-05-18 02:21:00','suborbital rocket','success',848,'CRL A07.015-4','Sphere','','','Aeron',394,'',156 from dual union all
select 2537,'1954-A516','1954-09-13 00:00:00','atmospheric rocket','success',836,'2831T','','','','Test',1895,'',10 from dual union all
select 25370,'1971-S29','1971-08-23 00:00:00','military missile','success',1449,'8K84','8K84 UBP-140','','','Test',526,'',1000 from dual union all
select 25371,'1971-S290','1971-05-18 02:41:00','suborbital rocket','success',867,'CRL A08.919-2','Release','','','Aeron',394,'',203 from dual union all
select 25372,'1971-S291','1971-05-18 03:05:00','suborbital rocket','success',848,'CRL A07.918-4','Release','','','Aeron',394,'',161 from dual union all
select 25373,'1971-S292','1971-05-18 08:39:00','suborbital rocket','success',331,'WRE C112','Sphere','','','Aeron',1835,'',100 from dual union all
select 25374,'1971-S293','1971-05-18 10:09:00','suborbital rocket','success',848,'CRL A07.917-6','Release','','','Aeron',394,'',153 from dual union all
select 25375,'1971-S294','1971-05-19 01:14:00','suborbital rocket','success',848,'CRL A07.918-3','NO Release','','','Aeron',394,'',145 from dual union all
select 25376,'1971-S295','1971-05-18 02:06:00','suborbital rocket','success',848,'CRL A07.921-1','Sphere','','','Aeron',394,'',148 from dual union all
select 25377,'1971-S296','1971-05-19 07:30:00','suborbital rocket','success',30,'NASA 04.303NG','','','','UV Astronomy',1869,'',200 from dual union all
select 25378,'1971-S297','1971-05-21 00:00:00','military missile','success',796,'','Mk 11 RV','OT GT83M','GLORY TRIP 83M','OT',1700,'',1300 from dual union all
select 25379,'1971-S298','1971-05-21 01:15:00','suborbital rocket','success',848,'CRL A07.917-4','Release','','','Aeron',394,'',145 from dual union all
select 2538,'1954-A517','1954-09-13 00:00:00','atmospheric rocket','success',836,'2808T','','','','Test',1895,'',10 from dual union all
select 25380,'1971-S299','1971-05-21 22:35:00','suborbital rocket','success',1189,'INTA MP-7101','SA-7101','','','Ionos',396,'',81 from dual union all
select 25381,'1971-S30','1971-09-29 00:00:00','military missile','success',1451,'8K84M','8K84M LKI-10','','','Test',526,'',1000 from dual union all
select 25382,'1971-S300','1971-05-24 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT103F','GLORY TRIP 103F','OT',1708,'',1300 from dual union all
select 25383,'1971-S301','1971-05-25 09:37:00','suborbital rocket','success',864,'Sandia 152-171','LRL ACS-1D','','','XR Astron',844,'',141 from dual union all
select 25384,'1971-S302','1971-05-27 06:56:00','military missile','success',799,'','Mk 12 RV x 3','OT GT02GM','GLORY TRIP 02GM','OT',1703,'',1300 from dual union all
select 25385,'1971-S303','1971-05-26 09:47:00','suborbital rocket','success',1347,'Sandia 281-19','LRL ACS-4B','','','XR Astron',855,'',300 from dual union all
select 25386,'1971-S304','1971-05-26 10:10:00','suborbital rocket','success',867,'','LEINAX I','','','XR Astron',855,'',209 from dual union all
select 25387,'1971-S305','1971-05-26 20:00:00','suborbital rocket','success',288,'C141','INTA C-7102','','','Aeronomy/Ionos',396,'',128 from dual union all
select 25388,'1971-S306','1971-05-26 23:31:00','suborbital rocket','success',839,'INTA NC-7102','','','','Ionos',396,'',145 from dual;
insert into LAUNCH
select 25389,'1971-S307','1971-05-28 06:20:00','military missile','success',799,'','Mk 12 RV x 3','OT GT03GM','GLORY TRIP 03GM','OT',1697,'',1300 from dual union all
select 2539,'1954-A518','1954-09-13 00:00:00','atmospheric rocket','success',836,'2562E','','','','Test',1895,'',10 from dual union all
select 25390,'1971-S308','1971-05-27 01:41:00','suborbital rocket','success',839,'INTA NC-7103','','','','Ionos',396,'',152 from dual union all
select 25391,'1971-S309','1971-05-27 04:36:00','suborbital rocket','success',288,'C137','INTA C-7103','','','Aeronomy',396,'',127 from dual union all
select 25392,'1971-S31','1971-09-30 00:00:00','military missile','success',1451,'8K84M','8K84M LKI-11','','','Test',528,'',1000 from dual union all
select 25393,'1971-S310','1971-05-27 20:00:00','suborbital rocket','success',288,'C142','INTA C-7104','','','Aeronomy',396,'',103 from dual union all
select 25394,'1971-S311','1971-05-28 02:52:00','suborbital rocket','success',839,'INTA NC-7104','','','','Ionos',396,'',160 from dual union all
select 25395,'1971-S312','1971-05-28 02:56:00','suborbital rocket','success',1189,'INTA MP-7102','SC-7101','','','Ionos',396,'',96 from dual union all
select 25396,'1971-S313','1971-05-27 16:20:00','military missile','unknown',917,'','P-210 71 Spring 09','','','OT',615,'',180 from dual union all
select 25397,'1971-S314','1971-05-28 04:31:00','suborbital rocket','success',1189,'INTA MP-7104','SC-7102','','','Ionos',396,'',103 from dual union all
select 25398,'1971-S315','1971-05-27 16:50:00','military missile','unknown',917,'','P-211 71 Spring 10','','','OT',615,'',180 from dual union all
select 25399,'1971-S316','1971-05-27 18:55:00','military missile','unknown',917,'','P-212 71 Spring 11','','','OT',615,'',180 from dual union all
select 254,'1944-S230','1944-11-05 15:43:00','military missile','success',1470,'19501','Ma254','','','Test',661,'',90 from dual union all
select 2540,'1954-A519','1954-09-13 00:00:00','atmospheric rocket','success',836,'2446E','','','','Test',1895,'',10 from dual union all
select 25400,'1971-S317','1971-05-28 09:40:00','suborbital rocket','success',33,'KP3.37','','','','UV Astron',1869,'',179 from dual union all
select 25401,'1971-S318','1971-05-29 10:00:00','military missile','success',752,'','','','','OT',307,'S611',500 from dual union all
select 25402,'1971-S319','1971-05-29 10:48:00','suborbital rocket','success',1347,'','','','','IR Astron',855,'',340 from dual union all
select 25403,'1971-S32','1971-09-27 03:55:00','military missile','success',1456,'4924461-053','15A20U LKI-1','GCh 15B20','','Test',508,'',1000 from dual union all
select 25404,'1971-S320','1971-06-03 00:00:00','military missile','success',135,'B041','RVTO-2B-4','ABRES AS-4','','Reentry Test',612,'',200 from dual union all
select 25405,'1971-S321','1971-06-04 00:00:00','military missile','success',796,'','Mk 11 RV','OT GT81M','GLORY TRIP 81M','OT',1699,'',1300 from dual union all
select 25406,'1971-S322','1971-06-07 16:53:00','military missile','success',939,'C3E-4','DASO-10','','','OT',421,'SSBN 630',500 from dual union all
select 25407,'1971-S323','1971-06-07 19:35:00','suborbital rocket','success',219,'CRL A16.108-1','VLF Propagation','','','Ionos VLF',428,'',800 from dual union all
select 25408,'1971-S324','1971-06-08 19:46:00','suborbital rocket','success',1492,'61M/93','FU-194 GESAIR','','','Bio/Geophys',341,'',210 from dual union all
select 25409,'1971-S325','1971-06-08 20:31:00','military missile','unknown',917,'','P-213 71 Spring 12','','','OT',615,'',180 from dual union all
select 2541,'1954-A52','1954-02-11 00:00:00','military missile','unknown',339,'1284R','','','','Test',1895,'',30 from dual union all
select 25410,'1971-S326','1971-06-10 23:10:00','suborbital rocket','success',44,'NASA 17.11UA','JHU Airglow 11/S169','','','Aeronomy/Test',1821,'',328 from dual union all
select 25411,'1971-S327','1971-06-11 14:55:00','military missile','success',799,'STM-6W','Mk 12 RV x 3','R&D','','Test',1707,'',1300 from dual union all
select 25412,'1971-S328','1971-06-11 05:25:00','military missile','success',135,'V139D','','','','Reentry Test',614,'',200 from dual union all
select 25413,'1971-S329','1971-06-12 19:44:00','suborbital rocket','success',1492,'61M/94','FU-194 GESAIR','','','Bio/Geophys',341,'',210 from dual union all
select 25414,'1971-S33','1971-10-10 00:00:00','military missile','success',1456,'15A20U','15A20U LKI-2','','','Test',508,'',1000 from dual union all
select 25415,'1971-S330','1971-06-14 17:17:00','suborbital rocket','success',30,'NASA 04.324US','','','','Solar',1869,'',200 from dual union all
select 25416,'1971-S331','1971-12-25 00:00:00','military missile','success',1456,'','15A20U LKI-10','','','Test',506,'',1000 from dual union all
select 25417,'1971-S332','1971-06-18 00:00:00','military missile','success',795,'','Mk 11 RV','FOT GT94B','GLORY TRIP 94B','OT',1702,'',1300 from dual union all
select 25418,'1971-S333','1971-06-20 22:45:00','military missile','success',1420,'B-12','Mk 6 SSTTP T1','SAFSCOM M1-17','','Target',1658,'',1300 from dual union all
select 25419,'1971-S334','1971-06-20 19:31:00','test rocket','success',1128,'S144CR','PAET','','','Reentry Test',1814,'',377 from dual union all
select 2542,'1954-A520','1954-09-13 00:00:00','atmospheric rocket','success',836,'2793T','','','','Test',1895,'',10 from dual union all
select 25420,'1971-S335','1971-06-22 19:20:00','suborbital rocket','success',30,'NASA 04.237NA','','','','Aeronomy',1869,'',200 from dual union all
select 25421,'1971-S336','1971-06-23 00:00:00','military missile','success',796,'','Mk 11 RV','OT GT84M','GLORY TRIP 84M','OT',1700,'',1300 from dual union all
select 25422,'1971-S337','1971-06-24 01:05:00','suborbital rocket','success',1070,'S-210JA-4','','','','Auroral',1458,'',130 from dual union all
select 25423,'1971-S338','1971-06-24 05:50:00','suborbital rocket','success',44,'NASA 17.08CG','LPARL XRT','','','XR Astron',1895,'',300 from dual union all
select 25424,'1971-S339','1971-06-24 15:00:00','suborbital rocket','success',839,'NASA 10.367GM','GSFC Grenade 246 PB','','','Aeronomy',1039,'',111 from dual union all
select 25425,'1971-S34','1971-10-21 00:00:00','military missile','success',1456,'15A20U','15A20U LKI-3','','','Test',508,'',1000 from dual union all
select 25426,'1971-S340','1971-06-24 15:10:00','suborbital rocket','success',839,'NASA 10.371GM','GSFC Grenade 247 W','','','Aeronomy',1821,'',114 from dual union all
select 25427,'1971-S341','1971-06-25 08:44:00','suborbital rocket','success',624,'NASA 8.59AI','NOAA Ionos Sounder','','','Ionos',1821,'',660 from dual union all
select 25428,'1971-S342','1971-06-25 11:30:00','suborbital rocket','success',920,'P48H','D/E regions','','','Ionospheric',1447,'',140 from dual union all
select 25429,'1971-S343','1971-06-27 03:18:00','military missile','success',795,'','SSTTP','SSTTP M1-14','','Target',1701,'',1300 from dual union all
select 2543,'1954-A521','1954-09-14 00:00:00','atmospheric rocket','success',836,'2803T','','','','Test',1895,'',10 from dual union all
select 25430,'1971-S344','1971-06-26 00:00:00','military missile','success',752,'','','','','OT',307,'S611',500 from dual union all
select 25431,'1971-S345','1971-06-26 20:30:00','military missile','success',1000,'','8K67M UBP-12','','','OT',515,'',1000 from dual union all
select 25432,'1971-S346','1971-06-26 19:44:00','suborbital rocket','success',920,'P120H','Solar La/X/ne','','','Ionos/Solar EUV',1447,'',143 from dual union all
select 25433,'1971-S347','1971-06-27 15:07:00','suborbital rocket','success',839,'NASA 10.334GM','GSFC Grenade 248 C','','','Aeronomy',428,'',124 from dual union all
select 25434,'1971-S348','1971-06-28 12:25:00','suborbital rocket','success',920,'P109H','Solar La/X/ne','','','Ionos/Solar EUV',1447,'',143 from dual union all
select 25435,'1971-S349','1971-06-29 09:10:00','suborbital rocket','success',37,'CRL A04.004-4','HI STAR 2','','','IR Astron',1873,'',156 from dual union all
select 25436,'1971-S35','1971-11-24 00:00:00','military missile','success',1456,'','15A20U LKI-4','','','Test',508,'',1000 from dual union all
select 25437,'1971-S350','1971-06-29 11:12:00','military missile','success',168,'103F','WT2A-L21/WT2A-S21','ABRES RVTO-2A-3','','Reentry Test',1682,'',1400 from dual union all
select 25438,'1971-S351','1971-06-29 11:12:00','suborbital rocket','success',37,'CRL A04.116-1','CHASER 1','','','ABM Tech IR',1768,'',161 from dual union all
select 25439,'1971-S352','1971-06-30 17:10:00','suborbital rocket','success',30,'NASA 04.234GS','','','','Solar',1869,'',200 from dual union all
select 2544,'1954-A522','1954-09-15 00:00:00','atmospheric rocket','unknown',576,'546M','','','','Test',1895,'',10 from dual union all
select 25440,'1971-S353','1971-07-02 10:55:00','suborbital rocket','success',1207,'S73','ESRO S73','','','Solar X/Opt Astron',1343,'',224 from dual union all
select 25441,'1971-S354','1971-07-02 14:37:00','suborbital rocket','success',839,'NASA 10.328GM','GSFC Grenade 249 C','','','Aeronomy',428,'',122 from dual union all
select 25442,'1971-S355','1971-07-02 14:42:00','suborbital rocket','success',839,'NASA 10.335GM','GSFC Grenade 250 PB','','','Aeronomy',1039,'',100 from dual union all
select 25443,'1971-S356','1971-07-02 16:30:00','military missile','success',939,'','DASO-11','','','OT',421,'SSBN 633',500 from dual union all
select 25444,'1971-S357','1971-07-02 19:59:00','suborbital rocket','success',839,'NASA 10.368GM','GSFC Grenade 251 PB','','','Aeronomy',1039,'',95 from dual union all
select 25445,'1971-S358','1971-07-02 20:05:00','suborbital rocket','success',839,'NASA 10.336GM','GSFC Grenade 252 C','','','Aeronomy',428,'',122 from dual union all
select 25446,'1971-S359','1971-07-08 00:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT04GM','GLORY TRIP 04GM','OT',1697,'',1300 from dual union all
select 25447,'1971-S36','1971-12-02 00:00:00','military missile','success',1456,'','15A20U LKI-5','','','OT',508,'',1000 from dual union all
select 25448,'1971-S360','1971-07-09 19:43:00','suborbital rocket','success',39,'NASA 12.22GT','','','','Test',1895,'',191 from dual union all
select 25449,'1971-S361','1971-07-12 11:00:00','military missile','failure',795,'','Mk 11 RV','FOT GT95B','GLORY TRIP 95B','OT',1705,'',0 from dual union all
select 2545,'1954-A523','1954-09-15 00:00:00','atmospheric rocket','success',836,'2636A','','','','Test',1895,'',10 from dual union all
select 25450,'1971-S362','1971-07-12 15:06:00','suborbital rocket','success',839,'NASA 10.370GM','GSFC Grenade 253 C','','','Aeronomy',428,'',125 from dual union all
select 25451,'1971-S363','1971-07-12 15:18:00','suborbital rocket','success',839,'NASA 10.381GM','GSFC Grenade 254 PB','','','Aeronomy',1039,'',109 from dual union all
select 25452,'1971-S364','1971-07-12 16:06:00','suborbital rocket','success',839,'NASA 10.374GM','GSFC Grenade 255 W','','','Aeronomy',1821,'',136 from dual union all
select 25453,'1971-S365','1971-07-13 02:50:00','suborbital rocket','success',867,'NASA 18.117UI','','','','Ionos',1821,'',270 from dual union all
select 25454,'1971-S366','1971-07-13 18:30:00','suborbital rocket','success',867,'NASA 18.118UI','','','','Ionos',1821,'',270 from dual union all
select 25455,'1971-S367','1971-07-14 21:00:00','suborbital rocket','success',920,'P79H','Elec. analyser','','','Ionospheric',1447,'',143 from dual union all
select 25456,'1971-S368','1971-07-14 22:04:00','suborbital rocket','success',920,'P93H','Elec. analyser','','','Ionospheric',1447,'',142 from dual union all
select 25457,'1971-S369','1971-07-17 04:53:00','suborbital rocket','success',38,'KP3.39','IPRCE-2','','','IR Astron',1869,'',190 from dual union all
select 25458,'1971-S37','1971-12-10 01:45:00','military missile','success',1456,'','15A20U LKI-6','','','OT',508,'',1000 from dual union all
select 25459,'1971-S370','1971-07-17 23:30:00','military missile','success',939,'','DASO-12','','','OT',421,'SSBN 628',500 from dual union all
select 2546,'1954-A524','1954-09-15 00:00:00','atmospheric rocket','success',863,'1249B-2795','Nike 375R T48E2','','','Test',1895,'',10 from dual union all
select 25460,'1971-S371','1971-07-19 08:43:00','suborbital rocket','success',331,'WRE C1016','Sphere','','','Aeron',1835,'',100 from dual union all
select 25461,'1971-S372','1971-07-21 16:00:00','suborbital rocket','failure',228,'NASA 21.04GT','','','','Test',1821,'',174 from dual union all
select 25462,'1971-S373','1971-07-21 21:52:00','suborbital rocket','success',1070,'S-210JA-3','','','','Auroral',1458,'',131 from dual union all
select 25463,'1971-S374','1971-07-27 13:18:00','suborbital rocket','success',37,'NASA 13.52UA','','','','Aeronomy',1895,'',200 from dual union all
select 25464,'1971-S376','1971-08-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 25465,'1971-S377','1971-07-31 00:21:00','suborbital rocket','success',291,'C59/1','ESRO C59/1','','','Aeronomy',408,'',120 from dual union all
select 25466,'1971-S378','1971-07-31 00:44:54','suborbital rocket','success',837,'NASA 14.468UA','RTG NNA1 (Pandora P)','','','Aeronomy',417,'',114 from dual union all
select 25467,'1971-S379','1971-07-31 23:34:00','suborbital rocket','success',837,'NASA 14.469UA','RTG NNA2 (Pandora Q)','','','Aeronomy',417,'',106 from dual union all
select 25468,'1971-S38','1971-10-20 05:21:00','military missile','success',1451,'64147-332T1','8K84M LKI-12','GCh 15F842','','Test',526,'',1000 from dual union all
select 25469,'1971-S380','1971-08-01 00:09:00','suborbital rocket','success',291,'C59/2','ESRO C59/2','','','Aeronomy',408,'',119 from dual union all
select 2547,'1954-A525','1954-09-15 00:00:00','atmospheric rocket','success',863,'1249B-2515','Nike 376R T48E2','','','Test',1895,'',10 from dual union all
select 25470,'1971-S381','1971-08-03 00:00:00','military missile','success',795,'','Mk 11 RV','FOT GT96B','GLORY TRIP 96B','OT',1702,'',1300 from dual union all
select 25471,'1971-S382','1971-08-04 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT104F-1','GLORY TRIP 104F-1','OT',1711,'',1300 from dual union all
select 25472,'1971-S383','1971-08-05 02:15:00','suborbital rocket','success',1211,'SL902','Cul UVS 18','','','Solar UV',1839,'',272 from dual union all
select 25473,'1971-S384','1971-08-06 12:50:00','military missile','success',796,'','Mk 11 RV','OT GT85M','GLORY TRIP 85M','OT',1699,'',1300 from dual union all
select 25474,'1971-S385','1971-08-08 12:11:46','suborbital rocket','success',839,'Ferdinand 27','Ferdinand 27','','','Test/Ionos/Plasma',29,'',99 from dual union all
select 25475,'1971-S386','1971-08-10 02:24:00','suborbital rocket','success',1070,'S-210JA-1','','','','Auroral',1458,'',139 from dual union all
select 25476,'1971-S387','1971-08-10 04:40:00','suborbital rocket','success',37,'NASA 13.18DG','NRL NB4.243','','','UV Astron?',1895,'',200 from dual union all
select 25477,'1971-S388','1971-08-10 05:30:00','suborbital rocket','success',38,'NASA 13.08GG','','','','XR Astron',1895,'',200 from dual union all
select 25478,'1971-S389','1971-08-12 21:14:00','suborbital rocket','success',37,'NASA 13.62DS','NRL NE4.251','','','Solar',1895,'',200 from dual;
insert into LAUNCH
select 25479,'1971-S39','1971-01-19 03:40:00','military missile','success',1450,'5244732-2P','15A20 K','GCh 15B20','','Test',504,'',1000 from dual union all
select 2548,'1954-A526','1954-09-15 00:00:00','atmospheric rocket','success',863,'1249B-2533','Nike 374R T48E2','','','Test',1895,'',10 from dual union all
select 25480,'1971-S390','1971-08-13 00:00:00','military missile','success',796,'','Mk 11 RV','OT GT87M','GLORY TRIP 87M','OT',1700,'',1300 from dual union all
select 25481,'1971-S391','1971-08-13 16:00:00','military missile','success',136,'H002','ADE-2','ABRES ADE-2','','Reentry Test',615,'',200 from dual union all
select 25482,'1971-S392','1971-08-16 16:32:00','military missile','success',939,'','DASO-13','','','OT',421,'SSBN 633',500 from dual union all
select 25483,'1971-S393','1971-08-18 12:00:00','suborbital rocket','success',668,'K-9M-31','Sco X-1','','','XR Astron',839,'',330 from dual union all
select 25484,'1971-S394','1971-08-19 20:11:00','suborbital rocket','success',837,'NASA 14.491GM','','','','Aeronomy',1821,'',171 from dual union all
select 25485,'1971-S395','1971-08-20 00:00:00','suborbital rocket','failure',700,'L-4SC-1','TVC test','','','Test',834,'',100 from dual union all
select 25486,'1971-S396','1971-08-20 14:30:00','military missile','success',135,'V131D','HAVE SINEW 1','','','Reentry Test',612,'',200 from dual union all
select 25487,'1971-S397','1971-08-20 03:20:00','suborbital rocket','success',1014,'','Vertikal''-2','VZAF','','Solar UV/Solar X',636,'',463 from dual union all
select 25488,'1971-S398','1971-08-20 07:30:00','suborbital rocket','success',839,'NASA 10.390GM','','','','Aeronomy',1821,'',132 from dual union all
select 25489,'1971-S399','1971-08-20 07:45:00','suborbital rocket','success',837,'NASA 14.492GM','','','','Aeronomy',1821,'',180 from dual union all
select 2549,'1954-A527','1954-09-15 00:00:00','atmospheric rocket','success',836,'2823T','','','','Test',1895,'',10 from dual union all
select 25490,'1971-S40','1971-01-23 02:15:00','military missile','success',1450,'6305761-028','15A20 LKI-27','GCh 15B20','','Test',505,'',1000 from dual union all
select 25491,'1971-S400','1971-08-20 09:19:00','suborbital rocket','success',837,'NASA 14.475UI','','','','Ionos',1821,'',193 from dual union all
select 25492,'1971-S401','1971-08-20 09:44:00','suborbital rocket','success',837,'NASA 14.476UI','','','','Ionos',1821,'',198 from dual union all
select 25493,'1971-S402','1971-08-20 10:14:00','suborbital rocket','success',837,'NASA 14.477UI','','','','Ionos',1821,'',200 from dual union all
select 25494,'1971-S403','1971-08-20 12:10:00','suborbital rocket','success',655,'K-10-7','','','','Opt Astron',839,'',274 from dual union all
select 25495,'1971-S404','1971-08-20 19:56:00','suborbital rocket','success',839,'NASA 10.389GM','','','','Aeronomy',1821,'',125 from dual union all
select 25496,'1971-S405','1971-08-22 02:34:00','suborbital rocket','success',837,'','','','','Ionos',1438,'',223 from dual union all
select 25497,'1971-S406','1971-08-23 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 618',1000 from dual union all
select 25498,'1971-S407','1971-08-23 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 618',1000 from dual union all
select 25499,'1971-S408','1971-08-23 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 618',1000 from dual union all
select 255,'1944-S231','1944-11-05 19:06:00','military missile','success',1470,'19499','Ma255','','','Test',661,'',90 from dual union all
select 2550,'1954-A528','1954-09-20 00:00:00','atmospheric rocket','success',836,'2802T','','','','Test',1895,'',10 from dual union all
select 25500,'1971-S409','1971-08-24 02:24:00','suborbital rocket','success',837,'','','','','Ionos',1438,'',223 from dual union all
select 25501,'1971-S41','1971-03-11 04:10:00','military missile','success',1450,'5115761-027','15A20 LKI-28','GCh 15B20','','Test',504,'',1000 from dual union all
select 25502,'1971-S410','1971-08-24 09:05:00','suborbital rocket','success',331,'WRE C1017','Sphere','','','Aeron',1835,'',100 from dual union all
select 25503,'1971-S411','1971-08-26 10:35:00','suborbital rocket','success',668,'K-9M-33','Cs/Ba Release','','','Ionos',839,'',340 from dual union all
select 25504,'1971-S412','1971-08-27 00:00:00','military missile','success',1251,'800031','SAFEGUARD M2-1','','','Interceptor',868,'',123 from dual union all
select 25505,'1971-S413','1971-08-28 02:22:00','military missile','success',1420,'B-100','Mk 6 SSTTP T2','SAFSCOM M2-1','','Target',1658,'',1300 from dual union all
select 25506,'1971-S414','1971-09-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 25507,'1971-S415','1971-09-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 25508,'1971-S416','1971-09-30 19:50:00','military missile','failure',917,'Imp.','P-215 71 Fall 02','','','OT',433,'',10 from dual union all
select 25509,'1971-S417','1971-09-16 00:00:00','military missile','success',1451,'8K84M','8K84M LKI-9','','','OT',528,'',1000 from dual union all
select 2551,'1954-A529','1954-09-21 00:00:00','military missile','unknown',339,'1408RO','','','','Test',1895,'',30 from dual union all
select 25510,'1971-S418','1971-09-01 11:27:00','military missile','success',162,'74F','LAR-2/LAR-4','ABRES LAR-2','','Reentry Test',1686,'',1400 from dual union all
select 25511,'1971-S419','1971-09-01 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 616',1000 from dual union all
select 25512,'1971-S42','1971-03-24 00:00:00','military missile','success',1451,'8K84M','8K84M LKI-3','','','Test',526,'',1000 from dual union all
select 25513,'1971-S420','1971-09-01 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 616',1000 from dual union all
select 25514,'1971-S421','1971-09-01 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 616',1000 from dual union all
select 25515,'1971-S422','1971-09-01 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 616',1000 from dual union all
select 25516,'1971-S423','1971-09-01 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 616',1000 from dual union all
select 25517,'1971-S424','1971-09-01 02:10:00','suborbital rocket','success',655,'K-10-6','Solar UV','','','Solar UV',839,'',228 from dual union all
select 25518,'1971-S425','1971-09-01 09:45:00','suborbital rocket','success',740,'','Magnetic','','','Ionos',636,'',177 from dual union all
select 25519,'1971-S426','1971-09-01 12:45:00','suborbital rocket','success',740,'','Magnetic','','','Ionos',636,'',177 from dual union all
select 2552,'1954-A53','1954-02-11 00:00:00','atmospheric rocket','unknown',576,'420ST','','','','Test',1895,'',10 from dual union all
select 25520,'1971-S427','1971-09-02 00:00:00','military missile','success',796,'','Mk 11 RV','OT GT29M','GLORY TRIP 29M','OT',1699,'',1300 from dual union all
select 25521,'1971-S428','1971-09-02 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 616',1000 from dual union all
select 25522,'1971-S429','1971-09-02 00:00:00','military missile','unknown',929,'A2','FOT','','','OT',421,'SSBN 616',1000 from dual union all
select 25523,'1971-S43','1971-04-17 00:00:00','military missile','success',1451,'8K84M','8K84M LKI-4','','','Test',526,'',1000 from dual union all
select 25524,'1971-S430','1971-09-02 07:58:00','suborbital rocket','success',837,'NASA 14.434GE','PCA particles','','','Plasma',428,'',160 from dual union all
select 25525,'1971-S431','1971-09-03 00:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT05GM','GLORY TRIP 05GM','OT',1697,'',1300 from dual union all
select 25526,'1971-S432','1971-09-03 12:00:00','sounding rocket','success',698,'L-3H-7','','','','Aeron/Ionos/XR Ast',834,'',1718 from dual union all
select 25527,'1971-S433','1971-09-04 13:52:00','suborbital rocket','success',218,'NASA 22.01UE','','','','Plasma',1315,'',150 from dual union all
select 25528,'1971-S434','1971-09-05 13:44:00','suborbital rocket','success',218,'NASA 22.02UE','','','','Plasma',1315,'',150 from dual union all
select 25529,'1971-S435','1971-09-07 23:57:00','suborbital rocket','success',837,'NASA 14.493CA','GCA Release Ba-Li','','','Aeronomy',1821,'',176 from dual union all
select 2553,'1954-A530','1954-09-21 00:00:00','military missile','success',573,'4','XSSM-A-16','','','Test',1863,'',36 from dual union all
select 25530,'1971-S436','1971-09-08 00:00:00','military missile','success',795,'','Mk 11 RV','FOT GT97B-1','GLORY TRIP 97B-1','OT',1702,'',1300 from dual union all
select 25531,'1971-S437','1971-09-09 02:30:00','military missile','success',1000,'Yu45023-41','8K67M UBP-13','GCh 8F675','','OT',515,'',1000 from dual union all
select 25532,'1971-S438','1971-09-10 00:00:00','military missile','success',436,'','','','','Test',810,'',1000 from dual union all
select 25533,'1971-S439','1971-09-10 11:00:00','military missile','success',796,'','Mk 11 RV','OT GT89M','GLORY TRIP 89M','OT',1700,'',1300 from dual union all
select 25534,'1971-S44','1971-05-21 00:00:00','military missile','success',1451,'8K84M','8K84M LKI-5','','','Test',526,'',1000 from dual union all
select 25535,'1971-S440','1971-09-10 06:47:00','suborbital rocket','success',694,'LS-C-5','','','','Test',1476,'',53 from dual union all
select 25536,'1971-S441','1971-09-13 00:00:00','suborbital rocket','success',288,'C133','CNES FU-199','','','Aeronomy',340,'',135 from dual union all
select 25537,'1971-S442','1971-09-13 21:50:00','suborbital rocket','success',1070,'S-210JA-5','','','','Auroral',1458,'',115 from dual union all
select 25538,'1971-S443','1971-09-14 04:20:00','suborbital rocket','success',364,'DA005','PHARE (FU-196)','','','Aeronomy',340,'',103 from dual union all
select 25539,'1971-S444','1971-09-14 19:10:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',165 from dual union all
select 2554,'1954-A531','1954-09-21 00:00:00','military missile','success',1214,'N-69B GM-11113','','','','Test',222,'',10 from dual union all
select 25540,'1971-S445','1971-09-14 22:40:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',165 from dual union all
select 25541,'1971-S446','1971-09-15 17:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',165 from dual union all
select 25542,'1971-S447','1971-09-15 19:18:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',165 from dual union all
select 25543,'1971-S448','1971-09-17 05:00:00','military missile','success',135,'V126D','SAM/AST 01A','ABRES SAMAST','','Reentry Test',614,'',200 from dual union all
select 25544,'1971-S449','1971-09-16 09:56:00','suborbital rocket','success',837,'NASA 14.494CA','GCA Release Ba-Li','','','Aeronomy',1821,'',175 from dual union all
select 25545,'1971-S45','1971-05-29 04:18:00','military missile','success',1451,'52447-322T','8K84M LKI-6','GCh 15F842P3','','Test',526,'',1000 from dual union all
select 25546,'1971-S450','1971-09-17 06:40:00','suborbital rocket','success',613,'JCR-6','','','','Test',1476,'',133 from dual union all
select 25547,'1971-S451','1971-09-19 21:56:00','suborbital rocket','success',839,'NASA 10.348GM','GSFC Grenade 256 CSG','','','Aeronomy',340,'',125 from dual union all
select 25548,'1971-S452','1971-09-19 22:00:00','suborbital rocket','success',288,'C132','CNES FU-199','','','Aeronomy',340,'',128 from dual union all
select 25549,'1971-S453','1971-09-19 22:00:00','suborbital rocket','success',288,'C146','CNES FU-199','','','Aeronomy',340,'',128 from dual union all
select 2555,'1954-A532','1954-09-22 00:00:00','atmospheric rocket','unknown',576,'551M','','','','Test',1895,'',10 from dual union all
select 25550,'1971-S454','1971-09-20 03:30:00','suborbital rocket','success',839,'NASA 10.349GM','GSFC Grenade 257 CSG','','','Aeronomy',340,'',122 from dual union all
select 25551,'1971-S455','1971-09-20 06:15:00','suborbital rocket','success',839,'NASA 10.354GM','GSFC Grenade 258 CSG','','','Aeronomy',340,'',115 from dual union all
select 25552,'1971-S456','1971-09-20 09:18:00','suborbital rocket','success',331,'WRE C1018','Sphere','','','Aeron',1835,'',100 from dual union all
select 25553,'1971-S457','1971-09-20 09:30:00','suborbital rocket','success',839,'NASA 10.352GM','GSFC Grenade 259 CSG','','','Aeronomy',340,'',117 from dual union all
select 25554,'1971-S458','1971-09-20 14:45:00','suborbital rocket','success',839,'NASA 10.350GM','GSFC Grenade 260 CSG','','','Aeronomy',340,'',118 from dual union all
select 25555,'1971-S459','1971-09-20 19:00:00','suborbital rocket','success',839,'NASA 10.353GM','GSFC Grenade 261 CSG','','','Aeronomy',340,'',116 from dual union all
select 25556,'1971-S46','1971-09-02 00:00:00','military missile','success',1451,'8K84M','8K84M LKI-7','','','Test',528,'',1000 from dual union all
select 25557,'1971-S460','1971-09-21 00:05:00','suborbital rocket','success',839,'NASA 10.355GM','GSFC Grenade 262 CSG','','','Aeronomy',340,'',117 from dual union all
select 25558,'1971-S461','1971-09-21 04:50:00','suborbital rocket','success',839,'NASA 10.356GM','GSFC Grenade 263 CSG','','','Aeronomy',340,'',114 from dual union all
select 25559,'1971-S462','1971-09-21 09:14:00','suborbital rocket','success',839,'NASA 10.357GM','GSFC Grenade 264 CSG','','','Aeronomy',340,'',118 from dual union all
select 2556,'1954-A533','1954-09-22 00:00:00','atmospheric rocket','success',863,'1249B-2685','Nike 378R T48E2','','','Test',1895,'',10 from dual union all
select 25560,'1971-S463','1971-09-21 08:30:00','suborbital rocket','success',848,'CRL A07.017-1','Density','','','Aeron',1895,'',167 from dual union all
select 25561,'1971-S464','1971-09-21 14:00:00','suborbital rocket','success',839,'NASA 10.358GM','GSFC Grenade 265 CSG','','','Aeronomy',340,'',118 from dual union all
select 25562,'1971-S465','1971-09-21 18:40:00','suborbital rocket','success',839,'NASA 10.359GM','GSFC Grenade 266 CSG','','','Aeronomy',340,'',116 from dual union all
select 25563,'1971-S466','1971-09-21 23:40:00','suborbital rocket','success',839,'NASA 10.360GM','GSFC Grenade 267 CSG','','','Aeronomy',340,'',116 from dual union all
select 25564,'1971-S467','1971-09-22 03:50:00','suborbital rocket','success',839,'NASA 10.351GM','GSFC Grenade 268 CSG','','','Aeronomy',340,'',118 from dual union all
select 25565,'1971-S468','1971-09-22 12:41:00','suborbital rocket','success',920,'P121H','Solar La/X/ne','','','Iono-solar',1447,'',138 from dual union all
select 25566,'1971-S469','1971-09-24 00:00:00','suborbital rocket','success',1014,'','R-5V Geophysical','','','Ionos?',636,'',500 from dual union all
select 25567,'1971-S47','1971-09-07 00:00:00','military missile','success',1451,'8K84M','8K84M LKI-8','','','Test',526,'',1000 from dual union all
select 25568,'1971-S470','1971-09-24 09:45:00','suborbital rocket','success',740,'MR-12','Sun-Atm 71','','','Ionos',636,'',165 from dual;
insert into LAUNCH
select 25569,'1971-S471','1971-09-24 11:15:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm 71','','','Ionos',636,'',165 from dual union all
select 2557,'1954-A534','1954-09-22 00:00:00','atmospheric rocket','success',836,'2797R','Nike 380R','','','Test',1895,'',10 from dual union all
select 25570,'1971-S472','1971-09-24 12:45:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm 71','','','Ionos',636,'',165 from dual union all
select 25571,'1971-S473','1971-09-24 17:26:00','suborbital rocket','success',920,'P122H','Solar La/X/ne','','','Iono-solar',1447,'',125 from dual union all
select 25572,'1971-S474','1971-09-24 21:08:00','suborbital rocket','success',1070,'S-210JA-2','','','','Auroral',1458,'',138 from dual union all
select 25573,'1971-S475','1971-09-25 02:53:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm 71','','','Plasma',636,'',165 from dual union all
select 25574,'1971-S476','1971-09-25 13:17:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm 71','','','Ionos',636,'',165 from dual union all
select 25575,'1971-S477','1971-09-27 02:30:00','suborbital rocket','success',1211,'SL1002','LUX XRS12','','','XR Astronomy',1839,'',224 from dual union all
select 25576,'1971-S478','1971-09-27 08:30:00','suborbital rocket','success',848,'CRL A07.914-1','Density','','','Aeron',1895,'',170 from dual union all
select 25577,'1971-S479','1971-09-27 13:17:00','suborbital rocket','unknown',740,'','Sun-Atm 71','','','Ionos',636,'',150 from dual union all
select 25578,'1971-S48','1971-12-17 00:00:00','military missile','success',1456,'','15A20U LKI-8','','','Test',506,'',1000 from dual union all
select 25579,'1971-S480','1971-09-28 11:48:00','suborbital rocket','success',920,'P49H','Winter anom/Strato','','','Ionospheric',1447,'',142 from dual union all
select 2558,'1954-A535','1954-09-22 00:00:00','atmospheric rocket','success',836,'2715R','Nike 381R','','','Test',1895,'',10 from dual union all
select 25580,'1971-S481','1971-09-29 16:00:00','suborbital rocket','success',1206,'ESRO S47/2','ROE-10','','','UV Astronomy',1839,'',210 from dual union all
select 25581,'1971-S482','1971-10-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 25582,'1971-S483','1971-10-22 00:00:00','military missile','success',917,'','P-216 71 Fall 03','','','OT',433,'',180 from dual union all
select 25583,'1971-S484','1971-10-27 00:00:00','military missile','success',917,'','P-217 71 Fall 04','','','OT',433,'',180 from dual union all
select 25584,'1971-S485','1971-10-01 09:45:00','suborbital rocket','success',740,'MR-12','Sun-Atm 71 MX-6407P','','','Ionos',636,'',150 from dual union all
select 25585,'1971-S486','1971-10-01 11:13:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm 71','','','Aeron',636,'',150 from dual union all
select 25586,'1971-S487','1971-10-01 12:43:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm 71','','','Ionos',636,'',150 from dual union all
select 25587,'1971-S488','1971-10-01 13:17:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm 71','','','Ionos',636,'',171 from dual union all
select 25588,'1971-S489','1971-10-02 03:10:00','military missile','success',1000,'','8K67M UBP-14','','','OT',518,'',1000 from dual union all
select 25589,'1971-S49','1971-12-22 00:00:00','military missile','success',1456,'','15A20U LKI-9','','','Test',508,'',1000 from dual union all
select 2559,'1954-A536','1954-09-22 00:00:00','atmospheric rocket','success',863,'1249B-2815','Nike 379R T48E2','','','Test',1895,'',10 from dual union all
select 25590,'1971-S490','1971-10-02 03:06:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm 71','','','Ionos',636,'',150 from dual union all
select 25591,'1971-S491','1971-10-05 18:30:00','suborbital rocket','success',867,'CRL A08.103-2','Photometers','','','Aeron',1821,'',272 from dual union all
select 25592,'1971-S492','1971-10-06 00:00:00','military missile','success',796,'','Mk 11 RV','OT GT86F','GLORY TRIP 86F','OT',1711,'',1300 from dual union all
select 25593,'1971-S493','1971-10-06 00:32:00','suborbital rocket','success',848,'CRL A07.913-6A','Density','','','Aeron',1821,'',189 from dual union all
select 25594,'1971-S494','1971-10-06 00:33:00','suborbital rocket','success',848,'CRL A07.101-5','NACS mass spec','','','Aeron',1821,'',117 from dual union all
select 25595,'1971-S495','1971-10-06 00:42:00','suborbital rocket','success',848,'CRL A07.102-4','Sphere','','','Aeron',1821,'',232 from dual union all
select 25596,'1971-S496','1971-10-06 01:03:00','suborbital rocket','success',848,'CRL A07.913-6','Sphere','','','Aeron',1821,'',222 from dual union all
select 25597,'1971-S497','1971-10-06 02:45:00','suborbital rocket','success',848,'CRL A07.101-6','Neutral Atmosphere','','','Aeron',1821,'',119 from dual union all
select 25598,'1971-S498','1971-10-06 09:54:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm 71','','','Ionos',636,'',150 from dual union all
select 25599,'1971-S499','1971-10-06 12:00:00','suborbital rocket','success',848,'CRL A07.902-6','Mass Spectrometer','','','Aeron',1821,'',120 from dual union all
select 256,'1944-S232','1944-11-06 12:36:00','military missile','success',1470,'19508','Ma264','','','Test',661,'',90 from dual union all
select 2560,'1954-A537','1954-09-23 00:00:00','military missile','unknown',339,'T 10','','','','Test',1895,'',30 from dual union all
select 25600,'1971-S50','1971-01-07 20:01:00','suborbital rocket','success',839,'NASA 10.365GM','GSFC Grenade 237 PB','','','Aeronomy',1039,'',122 from dual union all
select 25601,'1971-S500','1971-10-06 12:11:00','suborbital rocket','success',848,'CRL A07.101-3','Mass Spectrometer','','','Aeron',1821,'',118 from dual union all
select 25602,'1971-S501','1971-10-06 12:33:00','suborbital rocket','success',740,'MR-12','Sun-Atm 71 Magnetic','','','Ionos',636,'',173 from dual union all
select 25603,'1971-S502','1971-10-06 12:44:00','suborbital rocket','success',848,'CRL A07.101-4','Mass Spectrometer','','','Aeron',1821,'',119 from dual union all
select 25604,'1971-S503','1971-10-06 13:05:00','suborbital rocket','success',848,'CRL A07.101-2','Mass Spectrometer','','','Aeron',1821,'',117 from dual union all
select 25605,'1971-S504','1971-10-06 14:08:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm 71','','','Aeron',636,'',150 from dual union all
select 25606,'1971-S505','1971-10-08 01:56:00','military missile','success',795,'','SSTTP','SSTTP M2-2','','Target',1701,'',1300 from dual union all
select 25607,'1971-S506','1971-10-07 03:36:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm 71','','','Ionos',636,'',150 from dual union all
select 25608,'1971-S507','1971-10-07 09:30:00','suborbital rocket','success',740,'MR-12','Sun-Atm 71 B/MX-6407','','','Ionos',636,'',185 from dual union all
select 25609,'1971-S508','1971-10-07 09:40:00','suborbital rocket','success',1210,'SL973','MSSL XR8 RMC','','','XR Astronomy',1839,'',233 from dual union all
select 2561,'1954-A538','1954-09-23 00:00:00','military missile','unknown',339,'T 11','','','','Test',1895,'',30 from dual union all
select 25610,'1971-S509','1971-10-07 11:01:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm 71','','','Aeron',636,'',150 from dual union all
select 25611,'1971-S51','1971-01-07 20:15:00','suborbital rocket','success',839,'NASA 10.347GM','GSFC Grenade 238 C','','','Aeronomy',428,'',123 from dual union all
select 25612,'1971-S510','1971-10-07 11:01:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm 71','','','Aeron',636,'',150 from dual union all
select 25613,'1971-S511','1971-10-07 13:06:00','suborbital rocket','unknown',740,'','Sun-Atm 71','','','Plasma',636,'',150 from dual union all
select 25614,'1971-S512','1971-10-08 08:10:00','suborbital rocket','success',37,'NRL NC4.253','FUV Stellar Phot.','','','UV Astron',1895,'',200 from dual union all
select 25615,'1971-S513','1971-10-08 11:00:00','suborbital rocket','success',740,'MR-12','Sun-Atm 71 MassSp','','','Aeron',636,'',150 from dual union all
select 25616,'1971-S514','1971-10-08 12:30:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm  71','','','Aeron',636,'',150 from dual union all
select 25617,'1971-S515','1971-10-08 14:05:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm  71','','','Ionos',636,'',150 from dual union all
select 25618,'1971-S517','1971-10-09 00:00:00','suborbital rocket','success',1014,'','R-5V Geophysical','','','Ionos?',636,'',500 from dual union all
select 25619,'1971-S518','1971-10-09 11:00:00','suborbital rocket','unknown',740,'MR-12','Sun-Atm  71','','','Aeron',636,'',150 from dual union all
select 2562,'1954-A539','1954-09-23 00:00:00','atmospheric rocket','unknown',576,'390M','','','','Test',1895,'',10 from dual union all
select 25620,'1971-S519','1971-10-09 12:25:00','suborbital rocket','success',740,'MR-12','Sun-Atm  71 Magnetic','','','Ionos',636,'',150 from dual union all
select 25621,'1971-S52','1971-01-07 20:30:00','suborbital rocket','success',839,'NASA 10.372GM','GSFC Grenade 239 W','','','Aeronomy',1821,'',131 from dual union all
select 25622,'1971-S520','1971-10-12 00:00:00','suborbital rocket','success',1206,'ESRO S81','ESRO S81','','','XR Astronomy',1839,'',210 from dual union all
select 25623,'1971-S521','1971-10-13 19:16:00','suborbital rocket','success',839,'NASA 10.378GI','','','','Ionos',1895,'',81 from dual union all
select 25624,'1971-S522','1971-10-13 19:28:00','suborbital rocket','success',839,'NASA 10.379GI','','','','Ionos',1895,'',81 from dual union all
select 25625,'1971-S523','1971-10-14 21:13:00','military missile','success',939,'','DASO-14','','','OT',421,'SSBN 635',500 from dual union all
select 25626,'1971-S524','1971-10-15 09:05:00','military missile','success',799,'','Mk 12 RV x 3','OT GT06GM','GLORY TRIP 06GM','OT',1697,'',1300 from dual union all
select 25627,'1971-S525','1971-10-16 00:00:00','suborbital rocket','success',466,'E007 (FU-202)','ATLAS/CM16/Gemini','','','UV Astron',340,'',205 from dual union all
select 25628,'1971-S526','1971-10-16 02:31:00','suborbital rocket','success',30,'CRL A03.910-1','IR Airglow','','','Aeron',1869,'',215 from dual union all
select 25629,'1971-S527','1971-10-18 00:00:00','military missile','success',795,'','Mk 11 RV','FOT GT98B','GLORY TRIP 98B','OT',1705,'',1300 from dual union all
select 2563,'1954-A54','1954-02-12 00:00:00','atmospheric rocket','unknown',576,'378-O','','','','Test',1895,'',10 from dual union all
select 25630,'1971-S528','1971-10-18 00:00:00','suborbital rocket','success',1116,'BUBIA','','','','Aeron/Fields',855,'',470 from dual union all
select 25631,'1971-S529','1971-10-18 09:38:00','suborbital rocket','success',331,'WRE C1019','Sphere','','','Aeron',1835,'',100 from dual union all
select 25632,'1971-S53','1971-01-08 21:41:00','suborbital rocket','success',848,'CRL A07.016-3','Sphere','','','Aeron',1895,'',278 from dual union all
select 25633,'1971-S530','1971-10-18 14:00:00','suborbital rocket','success',288,'C154','ISRO 5.13','','','Recovery Test',340,'',150 from dual union all
select 25634,'1971-S531','1971-10-19 00:00:00','suborbital rocket','success',1116,'BUBIA','','','','Aeron/Fields',855,'',467 from dual union all
select 25635,'1971-S532','1971-10-19 17:30:00','suborbital rocket','success',38,'NASA 13.113GT','MSFC RR-2','','','Microgravity',1895,'',200 from dual union all
select 25636,'1971-S533','1971-10-20 16:16:00','military missile','success',799,'STM-3W','Mk 12 RV x 3','R&D','','Test',1703,'',1300 from dual union all
select 25637,'1971-S534','1971-10-20 04:06:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',636,'',150 from dual union all
select 25638,'1971-S535','1971-10-20 09:40:00','suborbital rocket','success',1207,'ESRO S89','LUX BCS 6','','','Solar X',1343,'',210 from dual union all
select 25639,'1971-S536','1971-10-22 01:40:00','military missile','success',796,'','ERCS','ERCS GT40L','GLORY TRIP 40L','Comms',1700,'',1300 from dual union all
select 2564,'1954-A540','1954-09-23 00:00:00','atmospheric rocket','unknown',576,'497M','','','','Test',1895,'',10 from dual union all
select 25640,'1971-S537','1971-10-23 03:45:00','suborbital rocket','success',38,'NASA 13.63UG','','','','XR Astron',1895,'',181 from dual union all
select 25641,'1971-S538','1971-10-23 12:25:00','suborbital rocket','success',1347,'Sandia 281-23','LRL ACS-4C','','','XR Astron',847,'',303 from dual union all
select 25642,'1971-S539','1971-10-24 04:30:00','suborbital rocket','success',1349,'SECEDE I R21','BARBIZON CANUTO-A','','','Aeron/Fields',855,'',202 from dual union all
select 25643,'1971-S54','1971-01-08 22:11:00','military missile','success',135,'V144D','ABRES CCD-1','ABRES CCD-1','','Reentry Test',614,'',200 from dual union all
select 25644,'1971-S540','1971-10-24 04:34:00','suborbital rocket','success',1349,'SECEDE I R22','BARBIZON CANUTO-B','','','Aeron/Fields',855,'',202 from dual union all
select 25645,'1971-S541','1971-10-24 11:35:00','suborbital rocket','success',1210,'SL974','MSSL XR9','','','XR Astronomy',1839,'',245 from dual union all
select 25646,'1971-S542','1971-10-27 10:04:00','suborbital rocket','failure',37,'NASA 13.56GG','','','','UV Astron?',1895,'',200 from dual union all
select 25647,'1971-S543','1971-10-29 10:09:00','suborbital rocket','success',37,'CRL A04.004-5','HI STAR 3','','','IR Astron',1873,'',158 from dual union all
select 25648,'1971-S544','1971-11-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',137,'ZUB',150 from dual union all
select 25649,'1971-S545','1971-11-09 00:00:00','military missile','success',917,'','P-218 71 Fall 05','','','OT',433,'',180 from dual union all
select 2565,'1954-A541','1954-09-23 00:00:00','atmospheric rocket','success',836,'2826T','','','','Test',1895,'',10 from dual union all
select 25650,'1971-S546','1971-11-09 00:00:00','military missile','success',917,'','P-219 71 Fall 06','','','OT WS-157?',433,'',180 from dual union all
select 25651,'1971-S547','1971-11-03 10:10:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',150 from dual union all
select 25652,'1971-S548','1971-11-03 12:30:00','suborbital rocket','success',37,'CRL A04.012-1','OH/Radiance','','','Aeron',1895,'',251 from dual union all
select 25653,'1971-S549','1971-11-04 16:00:00','military missile','success',939,'','DASO-15 Surface','','','OT',421,'SSBN 636',500 from dual union all
select 25654,'1971-S55','1971-01-08 22:31:00','suborbital rocket','success',848,'CRL A07.015-2','Sphere','','','Aeron',1895,'',276 from dual union all
select 25655,'1971-S550','1971-11-10 00:00:00','military missile','success',929,'A2S-02','Safeguard M2-5','','','Target',1061,'EAG-154',1000 from dual union all
select 25656,'1971-S551','1971-11-07 09:18:00','suborbital rocket','unknown',740,'M-180','','','','Aeron',1010,'',192 from dual union all
select 25657,'1971-S552','1971-11-08 04:28:00','suborbital rocket','success',1349,'SECEDE I R23','BARBIZON DARDABASI-A','','','Aeron/Fields',855,'',198 from dual union all
select 25658,'1971-S553','1971-11-08 04:32:00','suborbital rocket','success',1349,'SECEDE I R24','BARBIZON DARDABASI-B','','','Aeron/Fields',855,'',198 from dual;
insert into LAUNCH
select 25659,'1971-S554','1971-11-08 09:54:00','suborbital rocket','success',331,'WRE C1022','Sphere','','','Aeron',1835,'',100 from dual union all
select 2566,'1954-A542','1954-09-23 00:00:00','atmospheric rocket','success',835,'','F-101A Model','','','Test',1821,'',10 from dual union all
select 25660,'1971-S555','1971-11-09 19:00:00','suborbital rocket','success',30,'CRL A03.002-4','XUV','','','Solar UV/Solar EUV',1869,'',215 from dual union all
select 25661,'1971-S556','1971-11-09 19:00:00','suborbital rocket','success',30,'CRL A03.002-3','XUV','','','Solar UV/Solar EUV',1869,'',219 from dual union all
select 25662,'1971-S557','1971-11-11 06:00:00','suborbital rocket','success',37,'NRL NB4.244','','','','XR Astron?',1895,'',270 from dual union all
select 25663,'1971-S558','1971-11-12 00:00:00','military missile','success',929,'A2S-04','Safeguard M2-8','','','Target',1061,'EAG-154',1000 from dual union all
select 25664,'1971-S559','1971-11-16 15:45:00','suborbital rocket','success',30,'KP3.36','','','','Aeron',1869,'',200 from dual union all
select 25665,'1971-S56','1971-01-11 00:00:00','military missile','success',795,'','SSTTP','SSTTP M1-30','','Target',1701,'',1300 from dual union all
select 25666,'1971-S560','1971-11-16 23:18:00','suborbital rocket','success',1468,'CRL A09.106-5','Release','','','Aeron/Test',394,'',167 from dual union all
select 25667,'1971-S561','1971-11-18 07:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT07GM','GLORY TRIP 07GM','OT',1699,'',1300 from dual union all
select 25668,'1971-S562','1971-11-17 05:33:54','suborbital rocket','success',867,'NASA 18.103GA','','','','Aeronomy',1412,'',200 from dual union all
select 25669,'1971-S563','1971-11-18 01:01:00','suborbital rocket','success',920,'P123S/C','Li','','','Aeronomy',1419,'',120 from dual union all
select 2567,'1954-A543','1954-09-24 00:00:00','atmospheric rocket','success',835,'','F23 Ramjet','','','Test',1821,'',10 from dual union all
select 25670,'1971-S564','1971-11-18 07:04:52','suborbital rocket','success',588,'','LRL BOX-17','','','Test/XR Astron',1282,'AVM1',267 from dual union all
select 25671,'1971-S565','1971-11-20 07:21:00','suborbital rocket','success',37,'NASA 13.57GG','','','','UV Astron?',1895,'',200 from dual union all
select 25672,'1971-S566','1971-11-20 08:30:00','suborbital rocket','success',37,'NASA 13.58GG','','','','UV Astron?',1895,'',200 from dual union all
select 25673,'1971-S567','1971-11-23 06:55:00','military missile','success',799,'','Mk 12 RV x 3','OT GT08GM','GLORY TRIP 08GM','OT',1697,'',1300 from dual union all
select 25674,'1971-S568','1971-11-22 20:07:40','suborbital rocket','success',290,'CE-67','ESRO C78/2','','','Auroral',408,'',138 from dual union all
select 25675,'1971-S569','1971-11-24 06:15:00','military missile','success',799,'(Chaser targ)','Mk 12 RV x 3','OT GT09GM','GLORY TRIP 09GM','OT',1700,'',1300 from dual union all
select 25676,'1971-S57','1971-01-11 00:00:00','military missile','success',1251,'800021','SAFEGUARD M1-30','','','Interceptor',868,'',100 from dual union all
select 25677,'1971-S570','1971-11-24 06:15:00','suborbital rocket','success',37,'CRL A04.116-3','CHASER 2','','','ABM Tech IR',1768,'',145 from dual union all
select 25678,'1971-S571','1971-11-24 12:43:00','suborbital rocket','unknown',740,'MR-12','','','','Test',636,'',150 from dual union all
select 25679,'1971-S572','1971-11-24 21:15:00','military missile','success',939,'','DASO-16','','','OT',421,'SSBN 635',500 from dual union all
select 2568,'1954-A544','1954-09-27 00:00:00','atmospheric rocket','success',836,'3644A','','','','Test',1895,'',10 from dual union all
select 25680,'1971-S573','1971-11-25 01:11:00','suborbital rocket','success',920,'P124S/C','Li','','','Aeronomy',1419,'',120 from dual union all
select 25681,'1971-S574','1971-11-25 17:13:05','suborbital rocket','success',1204,'ESRO S70/2','ESRO S70/2','','','Ionospheric/VLF',416,'',172 from dual union all
select 25682,'1971-S575','1971-11-30 05:59:00','suborbital rocket','success',1207,'SL1101','LUX BCS 7','','','Solar X',1839,'',193 from dual union all
select 25683,'1971-S576','1971-12-01 12:50:00','suborbital rocket','unknown',740,'MR-12','','','','Test',636,'',150 from dual union all
select 25684,'1971-S577','1971-12-01 13:34:00','suborbital rocket','success',920,'P80H','D-region','','','Ionospheric',1447,'',146 from dual union all
select 25685,'1971-S578','1971-12-03 03:45:00','military missile','success',795,'','Mk 11 RV','FOT GT100B','GLORY TRIP 100B','OT',1705,'',1300 from dual union all
select 25686,'1971-S579','1971-12-02 01:15:00','military missile','success',1000,'U 22505-07T','8K67M UBP-15','GCh 8F674 LKI OS-67','','Test',515,'',1000 from dual union all
select 25687,'1971-S58','1971-01-11 00:00:00','military missile','success',1251,'800024','SAFEGUARD M1-30','','','Interceptor',868,'',100 from dual union all
select 25688,'1971-S580','1971-12-03 12:00:00','suborbital rocket','success',1070,'S-210JA-6','','','','Auroral',1458,'',131 from dual union all
select 25689,'1971-S581','1971-12-03 12:58:00','suborbital rocket','success',920,'P82H','D-region','','','Ionospheric',1447,'',140 from dual union all
select 2569,'1954-A545','1954-09-28 00:00:00','military missile','unknown',339,'T 12','','','','Test',1895,'',30 from dual union all
select 25690,'1971-S582','1971-12-06 03:00:00','suborbital rocket','failure',837,'NASA 14.495UM','UM Pitot','','','Aeronomy',1039,'',100 from dual union all
select 25691,'1971-S583','1971-12-06 03:02:00','suborbital rocket','success',839,'NASA 10.384GM','GSFC Grenade 269 PB','','','Aeronomy',1039,'',100 from dual union all
select 25692,'1971-S584','1971-12-06 04:30:00','suborbital rocket','success',837,'NASA 14.496UM','UM Pitot','','','Aeronomy',1039,'',200 from dual union all
select 25693,'1971-S585','1971-12-06 04:42:00','suborbital rocket','success',837,'NASA 14.497UM','UM Pitot','','','Aeronomy',1039,'',200 from dual union all
select 25694,'1971-S586','1971-12-06 07:52:00','suborbital rocket','success',837,'NASA 14.498UM','UM Pitot','','','Aeronomy',1039,'',200 from dual union all
select 25695,'1971-S587','1971-12-06 08:02:00','suborbital rocket','success',839,'NASA 10.385GM','GSFC Grenade 270 PB','','','Aeronomy',1039,'',100 from dual union all
select 25696,'1971-S588','1971-12-06 10:41:00','suborbital rocket','success',839,'NASA 10.392GM','Langmuir Probe','','','Aeronomy',1039,'',100 from dual union all
select 25697,'1971-S589','1971-12-06 11:30:00','suborbital rocket','success',839,'NASA 10.394GM','Langmuir Probe','','','Aeronomy',1039,'',100 from dual union all
select 25698,'1971-S59','1971-01-12 07:50:00','suborbital rocket','success',839,'NASA 10.366GM','GSFC Grenade 240 PB','','','Aeronomy',1039,'',116 from dual union all
select 25699,'1971-S590','1971-12-06 13:30:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',636,'',150 from dual union all
select 257,'1944-S233','1944-11-07 10:28:00','military missile','success',1470,'19511','Ma265','','','Test',661,'',90 from dual union all
select 2570,'1954-A546','1954-09-28 00:00:00','military missile','unknown',339,'1452EU','','','','Test',1895,'',30 from dual union all
select 25700,'1971-S591','1971-12-06 14:03:00','suborbital rocket','success',920,'P83H','D-region','','','Ionospheric',1447,'',145 from dual union all
select 25701,'1971-S592','1971-12-06 15:30:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',636,'',150 from dual union all
select 25702,'1971-S593','1971-12-06 16:30:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',636,'',150 from dual union all
select 25703,'1971-S594','1971-12-06 17:47:00','military missile','success',939,'','DASO-17','','','OT',421,'SSBN 634',500 from dual union all
select 25704,'1971-S595','1971-12-07 13:10:00','suborbital rocket','success',920,'P84H','D-region','','','Ionospheric',1447,'',140 from dual union all
select 25705,'1971-S596','1971-12-08 00:00:00','military missile','success',932,'A3S-01','Safeguard M2-3','','','Target',1061,'EAG-154',1000 from dual union all
select 25706,'1971-S597','1971-12-08 00:00:00','military missile','success',1251,'800027','SAFEGUARD M2-3','','','Interceptor',868,'',148 from dual union all
select 25707,'1971-S598','1971-12-10 01:00:00','military missile','success',795,'','Mk 11 RV','FOT GT99B-1','GLORY TRIP 99B-1','OT',1698,'',1300 from dual union all
select 25708,'1971-S599','1971-12-10 04:30:00','suborbital rocket','success',1207,'SL903','MSSL SX7 BCS','','','Solar X',1839,'',198 from dual union all
select 25709,'1971-S60','1971-01-12 20:53:00','suborbital rocket','success',839,'NASA 10.373GM','GSFC Grenade 241 W','','','Aeronomy',1821,'',133 from dual union all
select 2571,'1954-A547','1954-09-29 00:00:00','military missile','unknown',339,'T-14','','','','Test',1895,'',30 from dual union all
select 25710,'1971-S600','1971-12-10 12:22:00','suborbital rocket','success',920,'P119H','Electron temp','','','Ionospheric',1447,'',140 from dual union all
select 25711,'1971-S601','1971-12-10 14:46:00','suborbital rocket','success',848,'CRL A07.102-3','7-in Density Sphere','','','Aeron',428,'',253 from dual union all
select 25712,'1971-S602','1971-12-10 16:24:00','suborbital rocket','success',867,'CRL A08.102-2','7-in Density Sphere','','','Aeron',428,'',292 from dual union all
select 25713,'1971-S603','1971-12-10 18:00:00','suborbital rocket','success',848,'CRL A07.216-1','7-in Density Sphere','','','Aeron',428,'',248 from dual union all
select 25714,'1971-S604','1971-12-11 16:30:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',1010,'',180 from dual union all
select 25715,'1971-S605','1971-12-14 12:11:16','suborbital rocket','success',1204,'ESRO S82','ESRO S82/2','','','Meteorites',1343,'',151 from dual union all
select 25716,'1971-S606','1971-12-14 15:41:00','suborbital rocket','unknown',740,'MR-12','at CSG','','','Test',137,'ZUB',151 from dual union all
select 25717,'1971-S607','1971-12-15 00:00:00','military missile','success',799,'STM-4W','Mk 12 RV x 3','R&D','','Test',1703,'',1300 from dual union all
select 25718,'1971-S608','1971-12-15 00:00:00','military missile','success',1514,'R-29','','','','Test',190,'K-145',1000 from dual union all
select 25719,'1971-S609','1971-12-15 10:32:00','suborbital rocket','success',331,'WRE C1025','Sphere','','','Aeron',1835,'',100 from dual union all
select 2572,'1954-A548','1954-09-29 00:00:00','military missile','unknown',339,'T-13','','','','Test',1895,'',30 from dual union all
select 25720,'1971-S61','1971-01-13 02:30:00','suborbital rocket','success',839,'NASA 10.279NA','','','','Aeronomy',1895,'',100 from dual union all
select 25721,'1971-S610','1971-12-16 00:00:00','military missile','success',1103,'S-1E-2601','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 25722,'1971-S611','1971-12-16 05:59:00','suborbital rocket','unknown',740,'MR-12','at CSG','','','Aeron',137,'ZUB',170 from dual union all
select 25723,'1971-S612','1971-12-16 19:57:00','suborbital rocket','success',1492,'61M/92','FU-208 CISASPE','','','Ionos',341,'',227 from dual union all
select 25724,'1971-S613','1971-12-17 09:45:00','military missile','success',799,'','Mk 12 RV x 3','OT GT10GM','GLORY TRIP 10GM','OT',1699,'',1300 from dual union all
select 25725,'1971-S614','1971-12-17 21:45:00','military missile','success',939,'','DASO-18','','','OT',421,'SSBN 636',500 from dual union all
select 25726,'1971-S615','1971-12-19 23:33:00','suborbital rocket','success',894,'Campus','','','','Test',310,'',106 from dual union all
select 25727,'1971-S616','1971-12-26 04:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',137,'ZUB',179 from dual union all
select 25728,'1971-S617','1971-12-26 12:30:00','suborbital rocket','success',740,'MR-12 1054LT','MX-6407P Mass spec','','','Aeron',136,'ZUB',150 from dual union all
select 25729,'1971-S618','1971-12-26 14:30:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',137,'ZUB',175 from dual union all
select 2573,'1954-A549','1954-09-29 00:00:00','atmospheric rocket','unknown',576,'549M','','','','Test',1895,'',10 from dual union all
select 25730,'1971-S619','1971-12-26 17:46:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',1010,'',150 from dual union all
select 25731,'1971-S62','1971-01-13 16:00:00','suborbital rocket','success',839,'NASA 10.369GM','GSFC Grenade 242 C','','','Aeronomy',428,'',122 from dual union all
select 25732,'1971-S620','1971-12-27 02:00:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',137,'ZUB',176 from dual union all
select 25733,'1971-S621','1971-12-31 19:49:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',137,'ZUB',177 from dual union all
select 25734,'1971-S622','1971-12-31 20:49:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',137,'ZUB',158 from dual union all
select 25735,'1971-S623','1971-12-31 21:57:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',137,'ZUB',179 from dual union all
select 25736,'1971-S624','1971-12-31 23:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',137,'ZUB',183 from dual union all
select 25737,'1971-S625','1971-05-28 04:25:00','suborbital rocket','success',1189,'INTA MP-7103','SA-7102','','','Ionos',396,'',82 from dual union all
select 25738,'1971-S626','1971-10-15 19:28:00','suborbital rocket','success',740,'_','Sun-Atm  71 TMA rel','','','Aeron',636,'',150 from dual union all
select 25739,'1971-S627','1971-04-06 04:30:00','suborbital rocket','success',1281,'PCA','AFCRL Arcas 70-3','','','Ionos',1470,'',100 from dual union all
select 2574,'1954-A55','1954-02-12 00:00:00','atmospheric rocket','unknown',576,'379-O','','','','Test',1895,'',10 from dual union all
select 25740,'1971-S628','1971-06-09 12:05:00','suborbital rocket','success',1281,'Mesospheric probe','AFCRL Arcas 71-2','','','Aeron',1895,'',106 from dual union all
select 25741,'1971-S629','1971-07-28 12:15:00','suborbital rocket','success',1250,'','AFCRL Arcas 71-3','','','Aeron',1895,'',130 from dual union all
select 25742,'1971-S63','1971-01-13 20:10:00','suborbital rocket','success',215,'ADD-2-124','O2 profile','','','Aeronomy',428,'',146 from dual union all
select 25743,'1971-S630','1971-07-28 12:49:59','suborbital rocket','success',1250,'','AFCRL Arcas 71-4','','','Aeron',1895,'',137 from dual union all
select 25744,'1971-S631','1971-11-04 05:02:00','suborbital rocket','success',88,'SID','AFCRL Arcas 71-6','','','Ionos',1821,'',100 from dual union all
select 25745,'1971-S632','1971-05-22 05:30:00','suborbital rocket','success',1347,'','SHIRX','','','XR Astron',855,'',300 from dual union all
select 25746,'1971-S633','1971-02-01 22:55:00','suborbital rocket','success',1349,'SECEDE R17/R18','SECEDE II SPRUCE II','(Probe B)','','Aeron/Fields',394,'',184 from dual union all
select 25747,'1971-S634','1971-05-26 15:56:10','suborbital rocket','success',864,'Sandia 152-','NO density','','','Aeron',855,'',135 from dual union all
select 25748,'1971-S635','1971-11-01 00:00:00','military missile','success',353,'','','','','Test',811,'',200 from dual;
insert into LAUNCH
select 25749,'1971-S636','1971-04-22 04:22:00','test rocket','success',645,'','K63D BOR-2','BOR-2 No. 103','','Reentry Test',1752,'',100 from dual union all
select 2575,'1954-A550','1954-09-29 00:00:00','military missile','success',771,'GM-12443','','','','Test',222,'',12 from dual union all
select 25750,'1971-S637','1971-09-20 20:56:00','suborbital rocket','success',27,'WRE AH1013','Li release','','','Aeron',1833,'',166 from dual union all
select 25751,'1971-S638','1971-02-02 02:32:00','suborbital rocket','success',675,'WRE K152','','','','Aeron',1832,'',85 from dual union all
select 25752,'1971-S639','1971-05-18 02:37:00','suborbital rocket','success',675,'WRE K154','','','','Aeron',1832,'',85 from dual union all
select 25753,'1971-S64','1971-01-15 20:30:00','suborbital rocket','success',867,'MUMP 10','','','','Aeron',1821,'',289 from dual union all
select 25754,'1971-S640','1971-05-18 08:39:00','suborbital rocket','success',675,'WRE K155','','','','Aeron',1832,'',85 from dual union all
select 25755,'1971-S641','1971-06-22 02:45:00','suborbital rocket','success',675,'WRE K156','','','','Aeron',1832,'',85 from dual union all
select 25756,'1971-S642','1971-07-19 08:37:00','suborbital rocket','success',675,'WRE K1057','','','','Aeron',1832,'',85 from dual union all
select 25757,'1971-S643','1971-07-20 02:48:00','suborbital rocket','success',675,'WRE K1058','','','','Aeron',1832,'',85 from dual union all
select 25758,'1971-S644','1971-09-21 01:30:00','suborbital rocket','success',675,'WRE K1059','','','','Aeron',1832,'',85 from dual union all
select 25759,'1971-S645','1971-10-18 09:39:00','suborbital rocket','success',675,'WRE K1060','','','','Aeron',1832,'',85 from dual union all
select 2576,'1954-A551','1954-09-29 00:00:00','atmospheric rocket','success',863,'1249B-2847','Nike 382R T48E2','','','Test',1895,'',10 from dual union all
select 25760,'1971-S646','1971-11-26 02:30:00','suborbital rocket','success',675,'WRE K1061','','','','Aeron',1832,'',85 from dual union all
select 25761,'1971-S647','1971-11-24 09:36:00','suborbital rocket','success',538,'WRE HAT 637','','','','Aeron',1837,'',81 from dual union all
select 25762,'1971-S648','1971-01-01 00:00:00','suborbital rocket','success',1,'','Yantar''-4','','','Tech/Ionos',636,'',400 from dual union all
select 25763,'1971-S649','1971-11-15 00:00:00','military missile','success',359,'','Long range test','','','Test',804,'',500 from dual union all
select 25764,'1971-S65','1971-01-16 00:55:00','suborbital rocket','success',867,'MUMP-9','MUMP 9','','','Aeron',1821,'',297 from dual union all
select 25765,'1971-S650','1971-03-25 16:47:00','suborbital rocket','success',839,'Sphere Ejection','','','','Aeron',1895,'',100 from dual union all
select 25766,'1971-S651','1971-07-21 17:45:00','suborbital rocket','success',839,'3-Sphere Ejection 6','','','','Aeron',1895,'',100 from dual union all
select 25767,'1971-S652','1971-09-07 20:00:00','suborbital rocket','success',839,'3-Sphere Ejection 7','','','','Aeron',1895,'',100 from dual union all
select 25768,'1971-S653','1971-09-22 16:30:00','suborbital rocket','success',839,'3-Sphere Ejection 8','','','','Aeron',1895,'',100 from dual union all
select 25769,'1971-S654','1971-05-20 18:00:00','suborbital rocket','success',846,'BE-01 IEO','','','','Test?',1895,'',150 from dual union all
select 2577,'1954-A552','1954-09-29 00:00:00','atmospheric rocket','success',863,'1249B-2873','Nike 383R T48E2','','','Test',1895,'',10 from dual union all
select 25770,'1971-S655','1971-11-01 00:00:00','suborbital rocket','success',1268,'Sandia 428-025','BIRDSEED II BUITRE','Plasma gun','','Tech',855,'',210 from dual union all
select 25771,'1971-S656','1971-10-30 16:16:00','suborbital rocket','success',867,'Sandia 152-133','Rocket EMP Test 3','','','Plasma/Active',855,'',272 from dual union all
select 25772,'1971-S657','1971-02-02 16:32:31','suborbital rocket','success',1349,'SECEDE','SECEDE II Early Beacon','','','Ionos',394,'',259 from dual union all
select 25773,'1971-S658','1971-01-20 23:45:31','suborbital rocket','success',1349,'SECEDE','SECEDE II Early Beacon','','','Ionos',394,'',270 from dual union all
select 25774,'1971-S659','1971-01-26 23:49:51','suborbital rocket','success',1349,'SECEDE','SECEDE II Early Beacon','','','Ionos',394,'',260 from dual union all
select 25775,'1971-S66','1971-01-16 02:00:00','suborbital rocket','success',1070,'S-210-4','Te probe','','','Ionos',839,'',115 from dual union all
select 25776,'1971-S660','1971-01-29 23:52:17','suborbital rocket','success',1349,'SECEDE','SECEDE II Early Beacon','','','Ionos',394,'',263 from dual union all
select 25777,'1971-S661','1971-01-30 00:24:00','suborbital rocket','success',1349,'SECEDE','SECEDE II Late Beacon','','','Ionos',394,'',260 from dual union all
select 25778,'1971-S662','1971-02-01 00:01:24','suborbital rocket','success',1349,'SECEDE','SECEDE II Late Beacon','','','Ionos',394,'',260 from dual union all
select 25779,'1971-S663','1971-06-17 00:00:00','military missile','success',982,'G 22501-07T','8K64U UBP-P80','R-16 PL71-02','','OT',477,'',1210 from dual union all
select 2578,'1954-A553','1954-09-29 00:00:00','atmospheric rocket','success',836,'2809T','','','','Test',1895,'',10 from dual union all
select 25780,'1971-S664','1971-06-29 00:00:00','military missile','success',982,'G 22501-10T','8K64U UBP-P81','R-16 PL71-03','','OT',477,'',1210 from dual union all
select 25781,'1971-S665','1971-07-15 00:00:00','military missile','success',982,'T 22505-05T','8K64U UBP-P82','R-16 PL71-04','','OT',477,'',1210 from dual union all
select 25782,'1971-S666','1971-08-16 00:00:00','military missile','success',982,'G 22501-15T','8K64U UBP-P84','R-16 PL71-05','','OT',477,'',1210 from dual union all
select 25783,'1971-S667','1971-05-31 00:00:00','military missile','success',1018,'G 15001-01','8K75 UBP-P4','','','OT',477,'',1160 from dual union all
select 25784,'1971-S669','1971-08-25 00:00:00','military missile','success',982,'U 22502-10T','8K64U UBP-P85','R-16 PL71-06','','OT',477,'',1210 from dual union all
select 25785,'1971-S67','1971-01-16 07:15:00','suborbital rocket','success',668,'K-9M-30','Te probe','','','Ionos',839,'',357 from dual union all
select 25786,'1971-S670','1971-07-26 00:00:00','military missile','success',982,'G 22502-13T','8K64U UBP-P83','R-16 PL71-01','','OT',477,'',1210 from dual union all
select 25787,'1971-S671','1971-03-27 00:00:00','military missile','success',1044,'Kh52624-10L','8K98P LKI-13','','','Test',477,'',1000 from dual union all
select 25788,'1971-S672','1971-12-18 00:00:00','military missile','success',1044,'64927-436-16L','8K98P LKI-14','','','Test',477,'',1000 from dual union all
select 25789,'1971-S673','1971-04-27 00:00:00','military missile','success',1037,'64647-221-38','8K98 UBP-16','','','Test',477,'',1000 from dual union all
select 2579,'1954-A554','1954-09-29 00:00:00','atmospheric rocket','success',836,'2787R','Nike 384R','','','Test',1895,'',10 from dual union all
select 25790,'1971-S674','1971-01-01 00:00:00','military missile','success',1449,'','8K84 54 RD UBP-11','','','OT',1468,'',1000 from dual union all
select 25791,'1971-S675','1971-01-01 00:00:00','military missile','success',1449,'','8K84 54 RD UBP-12','','','OT',1468,'',1000 from dual union all
select 25792,'1971-S676','1971-06-16 00:00:00','military missile','success',1449,'','8K84 UBP 52 RD','','','OT',195,'',1000 from dual union all
select 25793,'1971-S677','1971-09-20 00:00:00','military missile','success',999,'','8K67 62 RD UBP-1','','','OT',1646,'',1000 from dual union all
select 25794,'1971-S678','1971-07-17 03:30:00','suborbital rocket','success',37,'','Unknown','','','Aeron?',1895,'',200 from dual union all
select 25795,'1971-S679','1971-05-15 21:02:00','suborbital rocket','unknown',848,'','TMA release','[dup of KAY?]','','Aeron',394,'',150 from dual union all
select 25796,'1971-S68','1971-01-16 23:32:00','suborbital rocket','success',1349,'SECEDE R12','SECEDE II NUTMEG I','','','Aeron',394,'',144 from dual union all
select 25797,'1971-S680','1971-09-21 06:00:00','suborbital rocket','success',1025,'','','','','Test',1467,'',100 from dual union all
select 25798,'1971-S681','1971-06-24 00:00:00','military missile','success',1323,'','9M76B','Svinets','','Test',596,'',100 from dual union all
select 25799,'1971-S682','1971-06-25 00:00:00','military missile','success',1323,'','9M76B','Svinets','','Test',596,'',100 from dual union all
select 258,'1944-S234','1944-11-07 12:35:00','military missile','success',1470,'19514','Ma266','','','Test',661,'',90 from dual union all
select 2580,'1954-A555','1954-09-30 00:00:00','military missile','unknown',339,'T-15','','','','Test',1895,'',30 from dual union all
select 25800,'1971-S683','1971-10-08 00:00:00','military missile','success',982,'R 22601-05T','8K64U UBP-157','','','OT',596,'',1210 from dual union all
select 25801,'1971-S684','1971-12-10 19:40:00','military missile','success',1456,'','15A20U LKI-7','','','OT',506,'',1000 from dual union all
select 25802,'1971-S685','1971-01-01 00:00:00','military missile','success',977,'','8K63U UBP','','','Test',840,'',402 from dual union all
select 25803,'1971-S686','1971-01-01 00:00:00','military missile','success',977,'','8K63U UBP','','','Test',840,'',402 from dual union all
select 25804,'1971-S687','1971-06-01 00:00:00','military missile','success',976,'','8K63 51 RB UBP','','','Test',840,'',402 from dual union all
select 25805,'1971-S688','1971-07-24 00:00:00','military missile','success',977,'','8K63U 40 RD UBP','','','Test',840,'',402 from dual union all
select 25806,'1971-S689','1971-10-08 00:00:00','military missile','success',977,'','8K63U 51 RB UBP','','','Test',840,'',402 from dual union all
select 25807,'1971-S69','1971-01-17 14:48:10','suborbital rocket','success',290,'C37/2','ESRO C37/2 MPE Ba','','','Ionospheric Ba',408,'',215 from dual union all
select 25808,'1971-S690','1971-06-01 00:00:00','military missile','success',980,'','8K65U 35 RD UBP-7','Yug','','Test',943,'',675 from dual union all
select 25809,'1971-S691','1971-06-01 00:00:00','military missile','success',980,'','8K65U 35 RD UBP-8','Yug','','Test',943,'',675 from dual union all
select 2581,'1954-A556','1954-10-01 00:00:00','atmospheric rocket','success',836,'1363T','','','','Test',1895,'',10 from dual union all
select 25810,'1971-S692','1971-06-01 00:00:00','military missile','success',980,'','8K65U 35 RD UBP-9','Yug','','Test',943,'',675 from dual union all
select 25811,'1971-S693','1971-04-14 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 25812,'1971-S694','1971-04-14 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 25813,'1971-S695','1971-04-21 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 25814,'1971-S696','1971-09-15 00:00:00','military missile','success',977,'','8K63U 29 RD UBP','','','Test',636,'',402 from dual union all
select 25815,'1971-S697','1971-05-28 00:00:00','military missile','unknown',980,'','8K65U UBP 29 RD','','','Test',636,'',675 from dual union all
select 25816,'1971-S698','1971-07-01 00:00:00','military missile','unknown',979,'','8K65 UBP 32 RD','','','Test',636,'',675 from dual union all
select 25817,'1971-S699','1971-08-15 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 25818,'1971-S70','1971-01-17 15:33:00','suborbital rocket','success',867,'DLR A-NT-39','MPE Ba/Brigitte','','','Ionos/Fields',29,'',328 from dual union all
select 25819,'1971-S700','1971-09-23 00:00:00','military missile','unknown',979,'','8K65 UBP 68 RD','','','Test',636,'',675 from dual union all
select 2582,'1954-A557','1954-10-04 00:00:00','military missile','unknown',339,'1409RO','','','','Test',1895,'',30 from dual union all
select 25820,'1971-S701','1971-10-12 00:00:00','military missile','unknown',980,'','8K65U UBP 29 RD','','','Test',636,'',675 from dual union all
select 25821,'1971-S702','1971-01-17 00:40:00','suborbital rocket','success',1116,'','','','','Test',394,'',400 from dual union all
select 25822,'1971-S71','1971-01-19 19:43:00','suborbital rocket','success',132,'Robin','AFCRL 71-1','','','Aeron',1821,'',146 from dual union all
select 25823,'1971-S72','1971-01-20 23:45:00','suborbital rocket','success',1349,'SECEDE R13','SECEDE II PLUM','','','Aeron',394,'',182 from dual union all
select 25824,'1971-S73','1971-01-21 02:32:00','suborbital rocket','success',227,'AKD-VB-27','Auroral','','','Auroral/Ionos/Plasm',428,'',258 from dual union all
select 25825,'1971-S74','1971-01-21 18:18:00','suborbital rocket','success',132,'Robin','AFCRL 71-2','','','Aeron',1821,'',130 from dual union all
select 25826,'1971-S75','1971-01-21 19:02:00','suborbital rocket','success',132,'Robin','AFCRL 71-3','','','Aeron',1821,'',129 from dual union all
select 25827,'1971-S76','1971-01-21 19:38:00','suborbital rocket','success',132,'Robin','AFCRL 71-4','','','Aeron',1821,'',132 from dual union all
select 25828,'1971-S77','1971-01-21 20:19:00','suborbital rocket','success',132,'Robin','AFCRL 71-5','','','Aeron',1821,'',132 from dual union all
select 25829,'1971-S78','1971-01-21 21:02:00','suborbital rocket','success',132,'Robin','AFCRL 71-6','','','Aeron',1821,'',127 from dual union all
select 2583,'1954-A558','1954-10-04 00:00:00','atmospheric rocket','success',836,'3305T','','','','Test',1895,'',10 from dual union all
select 25830,'1971-S79','1971-01-22 20:22:42','suborbital rocket','success',867,'NASA 18.116IE','Ferdinand 26/Polar 2','','','Plasma',29,'',253 from dual union all
select 25831,'1971-S80','1971-01-23 02:30:00','suborbital rocket','success',30,'NASA 04.260GG','','','','UV Astronomy',1869,'',200 from dual union all
select 25832,'1971-S81','1971-01-23 04:44:00','suborbital rocket','success',227,'ADD-VB-24','Auroral VUV','','','Auroral/Ionos/Plasm',428,'',375 from dual union all
select 25833,'1971-S82','1971-01-23 13:20:00','suborbital rocket','success',668,'K-9M-35','','','','Ionos',839,'',338 from dual union all
select 25834,'1971-S83','1971-01-23 20:08:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',1010,'',150 from dual union all
select 25835,'1971-S84','1971-01-24 02:00:00','suborbital rocket','success',668,'K-9M-34','Sco X-1/Te probe','','','XR Astron/Solar UV/Ionos',839,'',328 from dual union all
select 25836,'1971-S85','1971-01-25 08:19:00','suborbital rocket','success',837,'NASA 14.324GE','PCA particle spectra','','','Plasma',428,'',160 from dual union all
select 25837,'1971-S86','1971-01-25 11:30:00','suborbital rocket','success',30,'NASA 04.318UA','JHU UVT F6 (Venus)','','','UV Astron',1869,'',160 from dual union all
select 25838,'1971-S87','1971-01-25 12:38:00','suborbital rocket','success',290,'C60/1','ESRO C60/1','','','CR/particles',408,'',175 from dual;
insert into LAUNCH
select 25839,'1971-S88','1971-01-25 14:26:00','suborbital rocket','success',37,'NASA 13.46UA','JHU Airglow 10','','','Aeronomy',1895,'',245 from dual union all
select 2584,'1954-A559','1954-10-06 00:00:00','atmospheric rocket','unknown',576,'407M','','','','Test',1895,'',10 from dual union all
select 25840,'1971-S89','1971-01-25 15:12:00','suborbital rocket','success',837,'NASA 14.432GE','PCA particle spectra','','','Plasma',428,'',160 from dual union all
select 25841,'1971-S90','1971-01-26 10:50:00','suborbital rocket','success',290,'C60/2','ESRO C60/2','','','CR/particles',408,'',167 from dual union all
select 25842,'1971-S91','1971-01-26 13:22:15','suborbital rocket','success',920,'ESRO P40/1','ESRO P40/1','','','Ionospheric',413,'',145 from dual union all
select 25843,'1971-S92','1971-01-26 17:23:00','suborbital rocket','success',227,'AMD-VB-26','Airglow','','','Ionos/Solar',428,'',282 from dual union all
select 25844,'1971-S93','1971-01-26 22:58:00','suborbital rocket','success',1349,'SECEDE','Cert.Round (Probe C)','','','Test',394,'',300 from dual union all
select 25845,'1971-S94','1971-01-26 23:50:00','suborbital rocket','success',1349,'SECEDE R14/R15','SECEDE II REDWOOD','','','Aeron',394,'',252 from dual union all
select 25846,'1971-S95','1971-01-27 09:45:00','military missile','success',799,'','Mk 12 RV x 3','DASO OLD FOX 04M','OLD FOX 04M','OT',1703,'',1300 from dual union all
select 25847,'1971-S96','1971-01-27 23:02:45','suborbital rocket','success',290,'C78/1','ESRO C78/1','','','Auroral',408,'',140 from dual union all
select 25848,'1971-S97','1971-01-29 02:10:00','military missile','success',795,'','Mk 5 RV','FOT GT90B','GLORY TRIP 90B','OT',1705,'',1300 from dual union all
select 25849,'1971-S98','1971-01-28 05:10:00','suborbital rocket','success',837,'NASA 14.409II','ISRO 10.37','TERLS-280','','Ionos',1467,'',163 from dual union all
select 2585,'1954-A56','1954-02-15 00:00:00','atmospheric rocket','unknown',576,'430ST','','','','Test',1895,'',10 from dual union all
select 25850,'1971-S99','1971-01-28 05:40:00','suborbital rocket','success',837,'NASA 14.410II','ISRO 10.38','TERLS-281','','Ionos',1467,'',189 from dual union all
select 25851,'1971-U01','1971-02-06 18:48:00','lunar return','success',689,'8','Apollo 14','LM 8 Ascent Stage','','Sat',431,'',null from dual union all
select 25852,'1971-U02','1971-08-02 17:11:23','lunar return','success',689,'10','Apollo 15','LM 10 Ascent Stage','','Sat',638,'',null from dual union all
select 25853,'1971-U03','1971-09-20 23:31:00','ballistic missile test','success',1128,'S166C','BIC','Barium Ion Cloud','','Plasma Ba',1814,'',31479 from dual union all
select 25854,'1971-W01','1971-01-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 25855,'1971-W02','1971-01-03 20:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',50 from dual union all
select 25856,'1971-W03','1971-01-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 25857,'1971-W04','1971-01-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 25858,'1971-W05','1971-01-04 15:56:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 25859,'1971-W06','1971-01-04 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 2586,'1954-A560','1954-10-06 00:00:00','atmospheric rocket','success',836,'2791T','','','','Test',1895,'',10 from dual union all
select 25860,'1971-W07','1971-01-05 01:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',57 from dual union all
select 25861,'1971-W08','1971-01-05 19:03:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 25862,'1971-W09','1971-01-05 20:00:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',70 from dual union all
select 25863,'1971-W10','1971-01-06 14:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 25864,'1971-W100','1971-01-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',51 from dual union all
select 25865,'1971-W1000','1971-07-27 01:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',53 from dual union all
select 25866,'1971-W1001','1971-07-27 15:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 25867,'1971-W1002','1971-07-27 17:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',54 from dual union all
select 25868,'1971-W1003','1971-07-28 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 25869,'1971-W1005','1971-07-28 14:05:00','suborbital rocket','success',88,'WSMR/PSU','Conductivity','','','Aeron',1895,'',76 from dual union all
select 2587,'1954-A561','1954-10-07 00:00:00','atmospheric rocket','success',836,'2760T','','','','Test',1895,'',10 from dual union all
select 25870,'1971-W1006','1971-07-28 14:48:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 25871,'1971-W1007','1971-07-28 14:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 25872,'1971-W1008','1971-07-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 25873,'1971-W1009','1971-07-28 15:00:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 25874,'1971-W101','1971-01-20 14:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 25875,'1971-W1010','1971-07-28 15:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',66 from dual union all
select 25876,'1971-W1011','1971-07-28 15:59:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',61 from dual union all
select 25877,'1971-W1012','1971-07-28 16:14:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',69 from dual union all
select 25878,'1971-W1013','1971-07-28 16:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 25879,'1971-W1014','1971-07-28 17:04:59','suborbital rocket','success',716,'[MC-71-3]','Starute,Datasonde','','','Meteo',308,'',55 from dual union all
select 2588,'1954-A562','1954-10-08 00:00:00','military missile','unknown',339,'1410RJ','','','','Test',1895,'',30 from dual union all
select 25880,'1971-W1015','1971-07-28 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 25881,'1971-W1016','1971-07-29 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 25882,'1971-W1017','1971-07-29 14:20:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',63 from dual union all
select 25883,'1971-W1018','1971-07-29 17:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 25884,'1971-W1019','1971-07-29 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 25885,'1971-W102','1971-01-20 15:04:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',69 from dual union all
select 25886,'1971-W1020','1971-07-30 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 25887,'1971-W1021','1971-07-30 14:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 25888,'1971-W1022','1971-07-30 14:26:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',50 from dual union all
select 25889,'1971-W1023','1971-07-30 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 2589,'1954-A563','1954-10-08 00:00:00','atmospheric rocket','success',836,'3311T','','','','Test',1895,'',10 from dual union all
select 25890,'1971-W1024','1971-07-30 17:12:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',60 from dual union all
select 25891,'1971-W1025','1971-07-30 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 25892,'1971-W1026','1971-07-30 18:10:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',50 from dual union all
select 25893,'1971-W1027','1971-07-30 19:03:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',61 from dual union all
select 25894,'1971-W1028','1971-07-30 21:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 25895,'1971-W1029','1971-08-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 25896,'1971-W103','1971-01-20 16:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 25897,'1971-W1030','1971-08-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 25898,'1971-W1031','1971-07-31 02:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',57 from dual union all
select 25899,'1971-W1032','1971-08-02 13:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 259,'1944-S235','1944-11-07 16:02:00','military missile','success',1470,'19506','Ma260','','','Test',661,'',90 from dual union all
select 2590,'1954-A564','1954-10-08 00:00:00','atmospheric rocket','success',836,'3306T','','','','Test',1895,'',10 from dual union all
select 25900,'1971-W1033','1971-08-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 25901,'1971-W1034','1971-08-02 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 25902,'1971-W1035','1971-08-02 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 25903,'1971-W1036','1971-08-02 17:59:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',59 from dual union all
select 25904,'1971-W1037','1971-08-02 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 25905,'1971-W1038','1971-08-02 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',63 from dual union all
select 25906,'1971-W1039','1971-08-02 20:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 25907,'1971-W104','1971-01-20 17:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 25908,'1971-W1040','1971-08-02 21:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 25909,'1971-W1041','1971-08-03 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 2591,'1954-A565','1954-10-11 00:00:00','atmospheric rocket','unknown',576,'394ES','','','','Test',1895,'',10 from dual union all
select 25910,'1971-W1042','1971-08-03 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 25911,'1971-W1043','1971-08-03 19:16:00','suborbital rocket','success',88,'NASA 15.46UI','','','','Ionos',1821,'',60 from dual union all
select 25912,'1971-W1044','1971-08-03 20:10:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 25913,'1971-W1045','1971-08-04 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 25914,'1971-W1046','1971-08-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 25915,'1971-W1047','1971-08-04 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',55 from dual union all
select 25916,'1971-W1048','1971-08-04 15:08:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 25917,'1971-W1049','1971-08-04 15:23:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 25918,'1971-W105','1971-01-20 18:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 25919,'1971-W1050','1971-08-04 15:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 2592,'1954-A566','1954-10-11 00:00:00','atmospheric rocket','unknown',576,'395ES','','','','Test',1895,'',10 from dual union all
select 25920,'1971-W1051','1971-08-04 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 25921,'1971-W1052','1971-08-04 16:10:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 25922,'1971-W1053','1971-08-04 17:51:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',62 from dual union all
select 25923,'1971-W1054','1971-08-04 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',51 from dual union all
select 25924,'1971-W1055','1971-08-04 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 25925,'1971-W1056','1971-08-05 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 25926,'1971-W1057','1971-08-05 16:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 25927,'1971-W1058','1971-08-05 17:15:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',69 from dual union all
select 25928,'1971-W1059','1971-08-05 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual;
insert into LAUNCH
select 25929,'1971-W106','1971-01-20 18:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 2593,'1954-A567','1954-10-12 00:00:00','military missile','success',1214,'N-69B GM-11114','','','','Test',222,'',10 from dual union all
select 25930,'1971-W1060','1971-08-06 13:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 25931,'1971-W1061','1971-08-06 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 25932,'1971-W1062','1971-08-06 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 25933,'1971-W1063','1971-08-06 14:46:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 25934,'1971-W1064','1971-08-06 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 25935,'1971-W1065','1971-08-06 15:04:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',64 from dual union all
select 25936,'1971-W1066','1971-08-06 15:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 25937,'1971-W1067','1971-08-06 16:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 25938,'1971-W1068','1971-08-06 17:52:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',67 from dual union all
select 25939,'1971-W1069','1971-08-06 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 2594,'1954-A568','1954-10-13 00:00:00','military missile','unknown',339,'1554R','','','','Test',1895,'',30 from dual union all
select 25940,'1971-W107','1971-01-20 18:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',68 from dual union all
select 25941,'1971-W1070','1971-08-07 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 25942,'1971-W1071','1971-08-07 19:49:00','suborbital rocket','success',88,'NASA 15.87GM','','','','Aeronomy',617,'',60 from dual union all
select 25943,'1971-W1072','1971-08-07 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 25944,'1971-W1073','1971-08-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 25945,'1971-W1074','1971-08-09 14:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 25946,'1971-W1075','1971-08-09 16:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 25947,'1971-W1076','1971-08-09 19:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',60 from dual union all
select 25948,'1971-W1077','1971-08-09 20:12:00','suborbital rocket','success',88,'NASA 15.88GM','','','','Aeronomy',617,'',60 from dual union all
select 25949,'1971-W1078','1971-08-09 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 2595,'1954-A569','1954-10-13 00:00:00','atmospheric rocket','unknown',576,'280AM','','','','Test',1895,'',10 from dual union all
select 25950,'1971-W1079','1971-08-10 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 25951,'1971-W108','1971-01-20 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 25952,'1971-W1080','1971-08-10 20:15:00','suborbital rocket','success',258,'NASA 15.14GI','','','','Ionos',1895,'',70 from dual union all
select 25953,'1971-W1081','1971-08-10 23:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 25954,'1971-W1082','1971-08-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 25955,'1971-W1083','1971-08-11 14:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 25956,'1971-W1084','1971-08-11 15:17:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 25957,'1971-W1085','1971-08-11 15:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',69 from dual union all
select 25958,'1971-W1086','1971-08-11 15:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 25959,'1971-W1087','1971-08-11 16:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 2596,'1954-A57','1954-02-15 00:00:00','atmospheric rocket','unknown',576,'431ST','','','','Test',1895,'',10 from dual union all
select 25960,'1971-W1088','1971-08-11 16:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',57 from dual union all
select 25961,'1971-W1089','1971-08-11 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',62 from dual union all
select 25962,'1971-W109','1971-01-20 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 25963,'1971-W1090','1971-08-11 22:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 25964,'1971-W1091','1971-08-12 01:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 25965,'1971-W1092','1971-08-12 14:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 25966,'1971-W1093','1971-08-12 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 25967,'1971-W1094','1971-08-12 21:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 25968,'1971-W1095','1971-08-13 12:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',58 from dual union all
select 25969,'1971-W1096','1971-08-13 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 2597,'1954-A570','1954-10-13 00:00:00','atmospheric rocket','success',836,'2783R','Nike 386R','','','Test',1895,'',10 from dual union all
select 25970,'1971-W1097','1971-08-13 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 25971,'1971-W1098','1971-08-13 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 25972,'1971-W1099','1971-08-13 14:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 25973,'1971-W11','1971-01-06 14:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',55 from dual union all
select 25974,'1971-W110','1971-01-21 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 25975,'1971-W1100','1971-08-13 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 25976,'1971-W1101','1971-08-13 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 25977,'1971-W1102','1971-08-13 16:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 25978,'1971-W1103','1971-08-13 16:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',62 from dual union all
select 25979,'1971-W1104','1971-08-13 16:45:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',62 from dual union all
select 2598,'1954-A571','1954-10-13 00:00:00','atmospheric rocket','success',836,'3101R','Nike 385R','','','Test',1895,'',10 from dual union all
select 25980,'1971-W1105','1971-08-13 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 25981,'1971-W1106','1971-08-14 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 25982,'1971-W1107','1971-08-14 19:54:00','suborbital rocket','success',88,'NASA 15.90GM','','','','Aeronomy',617,'',60 from dual union all
select 25983,'1971-W1108','1971-08-16 14:45:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',61 from dual union all
select 25984,'1971-W1109','1971-08-16 15:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',59 from dual union all
select 25985,'1971-W111','1971-01-21 03:33:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 25986,'1971-W1110','1971-08-16 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 25987,'1971-W1111','1971-08-16 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 25988,'1971-W1112','1971-08-16 16:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 25989,'1971-W1113','1971-08-16 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',59 from dual union all
select 2599,'1954-A572','1954-10-13 00:00:00','atmospheric rocket','success',836,'2834T','','','','Test',1895,'',10 from dual union all
select 25990,'1971-W1114','1971-08-16 17:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 25991,'1971-W1115','1971-08-16 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 25992,'1971-W1116','1971-08-16 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',58 from dual union all
select 25993,'1971-W1117','1971-08-17 01:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 25994,'1971-W1118','1971-08-17 16:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 25995,'1971-W1119','1971-08-17 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 25996,'1971-W112','1971-01-21 15:07:26','suborbital rocket','success',1189,'Sk83/5','ESRO Sk83/5','','','Aeronomy',417,'',69 from dual union all
select 25997,'1971-W1120','1971-08-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',51 from dual union all
select 25998,'1971-W1121','1971-08-18 14:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 25999,'1971-W1122','1971-08-18 14:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 26,'1943-S04','1943-05-14 00:00:00','military missile','success',8,'22','','','','Test',675,'',95 from dual union all
select 260,'1944-S236','1944-11-07 22:35:00','military missile','success',1470,'19507','Ma258','','','Test',661,'',90 from dual union all
select 2600,'1954-A573','1954-10-14 00:00:00','atmospheric rocket','unknown',576,'138ES','','','','Test',1895,'',10 from dual union all
select 26000,'1971-W1123','1971-08-18 15:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',61 from dual union all
select 26001,'1971-W1124','1971-08-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 26002,'1971-W1125','1971-08-18 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 26003,'1971-W1126','1971-08-18 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 26004,'1971-W1127','1971-08-18 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 26005,'1971-W1128','1971-08-18 22:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 26006,'1971-W1129','1971-08-19 01:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 26007,'1971-W113','1971-01-21 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 26008,'1971-W1130','1971-08-19 15:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26009,'1971-W1131','1971-08-19 21:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 2601,'1954-A574','1954-10-14 00:00:00','atmospheric rocket','unknown',576,'122ES','','','','Test',1895,'',10 from dual union all
select 26010,'1971-W1132','1971-08-20 11:02:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',62 from dual union all
select 26011,'1971-W1133','1971-08-20 11:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 26012,'1971-W1134','1971-08-20 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 26013,'1971-W1135','1971-08-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 26014,'1971-W1136','1971-08-20 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',57 from dual union all
select 26015,'1971-W1137','1971-08-20 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 26016,'1971-W1138','1971-08-20 15:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',65 from dual union all
select 26017,'1971-W1139','1971-08-20 15:33:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',66 from dual;
insert into LAUNCH
select 26018,'1971-W114','1971-01-21 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26019,'1971-W1140','1971-08-20 16:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',71 from dual union all
select 2602,'1954-A575','1954-10-14 00:00:00','atmospheric rocket','success',836,'2798T','','','','Test',1895,'',10 from dual union all
select 26020,'1971-W1141','1971-08-20 16:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26021,'1971-W1142','1971-08-20 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',65 from dual union all
select 26022,'1971-W1143','1971-08-20 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26023,'1971-W1144','1971-08-21 01:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 26024,'1971-W1145','1971-08-23 13:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 26025,'1971-W1146','1971-08-23 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 26026,'1971-W1147','1971-08-23 14:47:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 26027,'1971-W1148','1971-08-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 26028,'1971-W1149','1971-08-23 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 26029,'1971-W115','1971-01-21 19:01:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1821,'',65 from dual union all
select 2603,'1954-A576','1954-10-14 00:00:00','atmospheric rocket','success',836,'3323T','','','','Test',1895,'',10 from dual union all
select 26030,'1971-W1150','1971-08-23 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',54 from dual union all
select 26031,'1971-W1151','1971-08-23 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26032,'1971-W1152','1971-08-23 19:12:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',57 from dual union all
select 26033,'1971-W1153','1971-08-23 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 26034,'1971-W1154','1971-08-23 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 26035,'1971-W1155','1971-08-24 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 26036,'1971-W1156','1971-08-24 17:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26037,'1971-W1157','1971-08-24 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 26038,'1971-W1158','1971-08-25 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 26039,'1971-W1159','1971-08-25 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 2604,'1954-A577','1954-10-14 20:24:00','atmospheric rocket','success',855,'','Stability Test','','','Test',1821,'',10 from dual union all
select 26040,'1971-W116','1971-01-21 19:37:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1821,'',64 from dual union all
select 26041,'1971-W1160','1971-08-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 26042,'1971-W1161','1971-08-25 15:07:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 26043,'1971-W1162','1971-08-25 16:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 26044,'1971-W1163','1971-08-25 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 26045,'1971-W1164','1971-08-25 19:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 26046,'1971-W1165','1971-08-25 19:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26047,'1971-W1166','1971-08-25 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',58 from dual union all
select 26048,'1971-W1167','1971-08-25 21:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 26049,'1971-W1168','1971-08-26 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 2605,'1954-A578','1954-10-17 00:00:00','suborbital rocket','failure',1490,'NA13','','','','Test/Ionos',666,'',39 from dual union all
select 26050,'1971-W1169','1971-08-26 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 26051,'1971-W117','1971-01-21 20:48:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1821,'',66 from dual union all
select 26052,'1971-W1170','1971-08-27 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 26053,'1971-W1171','1971-08-27 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 26054,'1971-W1172','1971-08-27 14:56:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 26055,'1971-W1173','1971-08-27 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 26056,'1971-W1174','1971-08-27 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',59 from dual union all
select 26057,'1971-W1175','1971-08-27 17:26:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',66 from dual union all
select 26058,'1971-W1176','1971-08-27 17:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26059,'1971-W1177','1971-08-27 19:10:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',64 from dual union all
select 2606,'1954-A579','1954-10-18 00:00:00','atmospheric rocket','success',836,'3326T','','','','Test',1895,'',10 from dual union all
select 26060,'1971-W1178','1971-08-27 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 26061,'1971-W1179','1971-08-27 22:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 26062,'1971-W118','1971-01-21 22:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26063,'1971-W1180','1971-08-30 13:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 26064,'1971-W1181','1971-08-30 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 26065,'1971-W1182','1971-08-30 15:04:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26066,'1971-W1183','1971-08-30 15:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26067,'1971-W1184','1971-08-30 16:34:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 26068,'1971-W1185','1971-08-30 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',54 from dual union all
select 26069,'1971-W1186','1971-08-30 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 2607,'1954-A58','1954-02-16 00:00:00','atmospheric rocket','unknown',576,'432ST','','','','Test',1895,'',10 from dual union all
select 26070,'1971-W1187','1971-09-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 26071,'1971-W1188','1971-08-31 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 26072,'1971-W1189','1971-08-31 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 26073,'1971-W119','1971-01-22 00:00:00','suborbital rocket','success',1188,'M--327H','','','','Aeron',1447,'',60 from dual union all
select 26074,'1971-W1190','1971-08-31 15:27:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 26075,'1971-W1191','1971-08-31 19:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26076,'1971-W1192','1971-08-31 22:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26077,'1971-W1193','1971-09-01 06:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 26078,'1971-W1194','1971-09-01 12:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 26079,'1971-W1195','1971-09-01 12:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 2608,'1954-A580','1954-10-19 00:00:00','military missile','success',573,'5','XSSM-A-16','','','Test',1863,'',36 from dual union all
select 26080,'1971-W1196','1971-09-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 26081,'1971-W1197','1971-09-01 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 26082,'1971-W1198','1971-09-01 14:45:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',65 from dual union all
select 26083,'1971-W1199','1971-09-01 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 26084,'1971-W12','1971-01-06 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 26085,'1971-W120','1971-01-22 00:00:00','suborbital rocket','success',1188,'M--324A','','','','Aeron',2,'',60 from dual union all
select 26086,'1971-W1200','1971-09-01 14:58:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 26087,'1971-W1201','1971-09-01 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26088,'1971-W1202','1971-09-01 15:20:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',71 from dual union all
select 26089,'1971-W1203','1971-09-01 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 2609,'1954-A581','1954-10-20 00:00:00','atmospheric rocket','unknown',576,'548M','','','','Test',1895,'',10 from dual union all
select 26090,'1971-W1204','1971-09-01 16:09:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',65 from dual union all
select 26091,'1971-W1205','1971-09-01 16:09:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',70 from dual union all
select 26092,'1971-W1206','1971-09-01 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 26093,'1971-W1207','1971-09-01 22:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 26094,'1971-W1208','1971-09-02 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 26095,'1971-W1209','1971-09-02 16:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 26096,'1971-W121','1971-01-22 03:10:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 26097,'1971-W1210','1971-09-02 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 26098,'1971-W1211','1971-09-03 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 26099,'1971-W1212','1971-09-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 261,'1944-S237','1944-11-08 12:02:00','military missile','success',1470,'19509','Ma259','','','Test',661,'',90 from dual union all
select 2610,'1954-A582','1954-10-20 00:00:00','military missile','success',771,'GM-12467','','','','Test',222,'',12 from dual union all
select 26100,'1971-W1213','1971-09-03 14:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',69 from dual union all
select 26101,'1971-W1214','1971-09-03 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',55 from dual union all
select 26102,'1971-W1215','1971-09-03 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 26103,'1971-W1216','1971-09-04 01:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 26104,'1971-W1217','1971-09-07 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',54 from dual union all
select 26105,'1971-W1218','1971-09-07 18:56:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 26106,'1971-W1219','1971-09-07 18:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',64 from dual union all
select 26107,'1971-W122','1971-01-22 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual;
insert into LAUNCH
select 26108,'1971-W1220','1971-09-07 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26109,'1971-W1221','1971-09-07 22:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 2611,'1954-A583','1954-10-20 00:00:00','military missile','success',771,'GM-12455','','','','Test',222,'',12 from dual union all
select 26110,'1971-W1222','1971-09-08 11:35:00','suborbital rocket','success',716,'INTA HT-7112','','','','Meteo',396,'',54 from dual union all
select 26111,'1971-W1223','1971-09-08 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 26112,'1971-W1224','1971-09-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 26113,'1971-W1225','1971-09-08 14:02:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 26114,'1971-W1226','1971-09-08 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26115,'1971-W1227','1971-09-08 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',55 from dual union all
select 26116,'1971-W1228','1971-09-08 17:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 26117,'1971-W1229','1971-09-08 19:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 26118,'1971-W123','1971-01-22 18:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 26119,'1971-W1230','1971-09-08 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 2612,'1954-A584','1954-10-20 00:00:00','military missile','success',771,'GM-12456','','','','Test',222,'',12 from dual union all
select 26120,'1971-W1231','1971-09-09 02:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 26121,'1971-W1232','1971-09-09 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26122,'1971-W1233','1971-09-09 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26123,'1971-W1234','1971-09-10 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',62 from dual union all
select 26124,'1971-W1235','1971-09-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 26125,'1971-W1236','1971-09-10 14:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 26126,'1971-W1237','1971-09-10 14:35:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 26127,'1971-W1238','1971-09-10 14:42:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 26128,'1971-W1239','1971-09-10 14:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26129,'1971-W124','1971-01-22 20:00:00','suborbital rocket','unknown',88,'WSMR/PSU','Conductivity','','','Ionos',1895,'',76 from dual union all
select 2613,'1954-A585','1954-10-20 00:00:00','atmospheric rocket','success',836,'2987R','Nike 387R','','','Test',1895,'',10 from dual union all
select 26130,'1971-W1240','1971-09-10 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',53 from dual union all
select 26131,'1971-W1241','1971-09-10 21:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26132,'1971-W1242','1971-09-11 01:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 26133,'1971-W1243','1971-09-13 13:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 26134,'1971-W1244','1971-09-13 14:37:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 26135,'1971-W1245','1971-09-13 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',51 from dual union all
select 26136,'1971-W1246','1971-09-13 19:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 26137,'1971-W1247','1971-09-13 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 26138,'1971-W1248','1971-09-14 02:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 26139,'1971-W1249','1971-09-14 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 2614,'1954-A586','1954-10-20 00:00:00','atmospheric rocket','success',836,'3337T','','','','Test',1895,'',10 from dual union all
select 26140,'1971-W125','1971-01-22 23:04:00','suborbital rocket','unknown',88,'','','','','Aeron',1895,'',52 from dual union all
select 26141,'1971-W1250','1971-09-15 11:55:00','suborbital rocket','success',716,'INTA HT-7113','','','','Meteo',396,'',54 from dual union all
select 26142,'1971-W1251','1971-09-15 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 26143,'1971-W1252','1971-09-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 26144,'1971-W1253','1971-09-15 14:40:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 26145,'1971-W1254','1971-09-15 14:47:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',64 from dual union all
select 26146,'1971-W1255','1971-09-15 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26147,'1971-W1256','1971-09-15 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 26148,'1971-W1257','1971-09-15 17:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 26149,'1971-W1258','1971-09-15 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 2615,'1954-A587','1954-10-20 00:00:00','atmospheric rocket','success',836,'2443R','Nike 388R','','','Test',1895,'',10 from dual union all
select 26150,'1971-W1259','1971-09-16 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 26151,'1971-W126','1971-01-23 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 26152,'1971-W1260','1971-09-16 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26153,'1971-W1261','1971-09-17 00:33:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',62 from dual union all
select 26154,'1971-W1262','1971-09-17 05:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 26155,'1971-W1263','1971-09-17 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 26156,'1971-W1264','1971-09-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 26157,'1971-W1265','1971-09-17 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 26158,'1971-W1266','1971-09-17 14:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',60 from dual union all
select 26159,'1971-W1267','1971-09-17 14:44:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 2616,'1954-A588','1954-10-22 00:00:00','military missile','unknown',339,'T-16','','','','Test',1895,'',30 from dual union all
select 26160,'1971-W1268','1971-09-17 15:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 26161,'1971-W1269','1971-09-17 16:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26162,'1971-W127','1971-01-23 00:00:00','suborbital rocket','success',1188,'M--329H','','','','Aeron',1447,'',55 from dual union all
select 26163,'1971-W1270','1971-09-17 17:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 26164,'1971-W1271','1971-09-17 18:30:00','suborbital rocket','success',88,'','Arcas Starute 1/STS','','','Meteo',1895,'',51 from dual union all
select 26165,'1971-W1272','1971-09-18 01:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 26166,'1971-W1273','1971-09-20 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 26167,'1971-W1274','1971-09-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 26168,'1971-W1275','1971-09-20 14:49:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26169,'1971-W1276','1971-09-20 15:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 2617,'1954-A589','1954-10-22 00:00:00','atmospheric rocket','failure',442,'','B-58 Model','','','Test',1821,'',10 from dual union all
select 26170,'1971-W1277','1971-09-20 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 26171,'1971-W1278','1971-09-20 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 26172,'1971-W1279','1971-09-20 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26173,'1971-W128','1971-01-23 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 26174,'1971-W1280','1971-09-21 01:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 26175,'1971-W1281','1971-09-21 14:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26176,'1971-W1282','1971-09-21 15:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 26177,'1971-W1283','1971-09-21 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26178,'1971-W1284','1971-09-22 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 26179,'1971-W1285','1971-09-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 2618,'1954-A59','1954-02-16 00:00:00','atmospheric rocket','unknown',576,'418ST','','','','Test',1895,'',10 from dual union all
select 26180,'1971-W1286','1971-09-22 14:38:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 26181,'1971-W1287','1971-09-22 15:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 26182,'1971-W1288','1971-09-22 16:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26183,'1971-W1289','1971-09-22 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 26184,'1971-W129','1971-01-23 02:19:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 26185,'1971-W1290','1971-09-22 17:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 26186,'1971-W1291','1971-09-22 18:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',59 from dual union all
select 26187,'1971-W1292','1971-09-22 19:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 26188,'1971-W1293','1971-09-22 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 26189,'1971-W1294','1971-09-23 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 2619,'1954-A590','1954-10-25 00:00:00','military missile','success',255,'623-8','','','','Test',253,'',20 from dual union all
select 26190,'1971-W1295','1971-09-23 14:45:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',50 from dual union all
select 26191,'1971-W1296','1971-09-23 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 26192,'1971-W1297','1971-09-23 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 26193,'1971-W1298','1971-09-24 12:00:00','suborbital rocket','success',716,'INTA HT-7115','','','','Meteo',396,'',52 from dual union all
select 26194,'1971-W1299','1971-09-24 12:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 26195,'1971-W13','1971-01-06 19:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 26196,'1971-W130','1971-01-23 15:53:30','suborbital rocket','success',1189,'Sk83/6','ESRO Sk83/6','','','Aeronomy',417,'',67 from dual union all
select 26197,'1971-W1300','1971-09-24 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 26198,'1971-W1301','1971-09-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual;
insert into LAUNCH
select 26199,'1971-W1302','1971-09-24 14:38:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 262,'1944-S238','1944-11-09 15:50:00','military missile','success',1470,'19520','Ma285','','','Test',661,'',90 from dual union all
select 2620,'1954-A591','1954-10-25 00:00:00','atmospheric rocket','unknown',576,'813M','','','','Test',1895,'',10 from dual union all
select 26200,'1971-W1303','1971-09-24 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',53 from dual union all
select 26201,'1971-W1304','1971-09-24 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 26202,'1971-W1305','1971-09-25 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 26203,'1971-W1306','1971-09-25 16:40:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',61 from dual union all
select 26204,'1971-W1307','1971-09-27 11:49:59','suborbital rocket','success',716,'INTA HT-7116','','','','Meteo',396,'',51 from dual union all
select 26205,'1971-W1308','1971-09-27 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 26206,'1971-W1309','1971-09-27 16:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26207,'1971-W131','1971-01-25 00:00:00','suborbital rocket','success',1188,'M--331H','','','','Aeron',1447,'',60 from dual union all
select 26208,'1971-W1310','1971-09-27 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 26209,'1971-W1311','1971-09-27 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 2621,'1954-A592','1954-10-25 00:00:00','atmospheric rocket','success',836,'3282T','','','','Test',1895,'',10 from dual union all
select 26210,'1971-W1312','1971-09-27 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26211,'1971-W1313','1971-09-28 02:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 26212,'1971-W1314','1971-09-28 16:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',51 from dual union all
select 26213,'1971-W1315','1971-09-28 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26214,'1971-W1316','1971-09-29 11:45:00','suborbital rocket','success',716,'INTA HT-7117','','','','Meteo',396,'',52 from dual union all
select 26215,'1971-W1317','1971-09-29 15:44:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 26216,'1971-W1318','1971-09-29 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',52 from dual union all
select 26217,'1971-W1319','1971-09-29 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 26218,'1971-W132','1971-01-25 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 26219,'1971-W1320','1971-09-29 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 2622,'1954-A593','1954-10-27 00:00:00','atmospheric rocket','unknown',576,'559M','','','','Test',1895,'',10 from dual union all
select 26220,'1971-W1321','1971-09-29 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26221,'1971-W1322','1971-09-30 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 26222,'1971-W1323','1971-09-30 20:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 26223,'1971-W1324','1971-09-30 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26224,'1971-W1325','1971-10-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 26225,'1971-W1326','1971-10-01 14:41:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 26226,'1971-W1327','1971-10-01 15:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26227,'1971-W1328','1971-10-01 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 26228,'1971-W1329','1971-10-01 17:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 26229,'1971-W133','1971-01-25 15:10:00','suborbital rocket','success',1189,'Sk83/7','ESRO Sk83/7','','','Aeronomy',417,'',68 from dual union all
select 2623,'1954-A594','1954-10-27 00:00:00','atmospheric rocket','success',836,'2909R','Nike 391R','','','Test',1895,'',10 from dual union all
select 26230,'1971-W1330','1971-10-01 22:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26231,'1971-W1331','1971-10-04 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 26232,'1971-W1332','1971-10-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 26233,'1971-W1333','1971-10-04 14:38:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 26234,'1971-W1334','1971-10-04 15:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26235,'1971-W1335','1971-10-04 17:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 26236,'1971-W1336','1971-10-04 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 26237,'1971-W1337','1971-10-04 19:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 26238,'1971-W1338','1971-10-04 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26239,'1971-W1339','1971-10-05 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 2624,'1954-A595','1954-10-27 00:00:00','atmospheric rocket','success',836,'2557R','Nike 389R','','','Test',1895,'',10 from dual union all
select 26240,'1971-W134','1971-01-25 15:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',69 from dual union all
select 26241,'1971-W1340','1971-10-05 22:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 26242,'1971-W1341','1971-10-06 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 26243,'1971-W1342','1971-10-06 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 26244,'1971-W1343','1971-10-06 14:23:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 26245,'1971-W1344','1971-10-06 17:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 26246,'1971-W1345','1971-10-06 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 26247,'1971-W1346','1971-10-06 20:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 26248,'1971-W1347','1971-10-06 21:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26249,'1971-W1348','1971-10-07 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 2625,'1954-A596','1954-10-27 00:00:00','atmospheric rocket','success',836,'2799R','Nike 390R','','','Test',1895,'',10 from dual union all
select 26250,'1971-W1349','1971-10-07 14:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',308,'',64 from dual union all
select 26251,'1971-W135','1971-01-25 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 26252,'1971-W1350','1971-10-07 15:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 26253,'1971-W1351','1971-10-07 15:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 26254,'1971-W1352','1971-10-07 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 26255,'1971-W1353','1971-10-07 17:19:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',62 from dual union all
select 26256,'1971-W1354','1971-10-07 18:07:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',67 from dual union all
select 26257,'1971-W1355','1971-10-07 18:18:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',66 from dual union all
select 26258,'1971-W1356','1971-10-07 19:07:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',67 from dual union all
select 26259,'1971-W1357','1971-10-07 19:34:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',67 from dual union all
select 2626,'1954-A597','1954-10-28 00:00:00','military missile','unknown',339,'1552RO','Type II ORD-437','','','Test',1895,'',40 from dual union all
select 26260,'1971-W1358','1971-10-07 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 26261,'1971-W1359','1971-10-08 13:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 26262,'1971-W136','1971-01-25 17:15:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',58 from dual union all
select 26263,'1971-W1360','1971-10-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 26264,'1971-W1361','1971-10-08 15:00:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',52 from dual union all
select 26265,'1971-W1362','1971-10-08 15:17:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26266,'1971-W1363','1971-10-08 20:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26267,'1971-W1364','1971-10-09 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',54 from dual union all
select 26268,'1971-W1365','1971-10-12 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 26269,'1971-W1366','1971-10-12 19:10:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 2627,'1954-A598','1954-10-28 00:00:00','atmospheric rocket','unknown',576,'814M','','','','Test',1895,'',10 from dual union all
select 26270,'1971-W1367','1971-10-12 20:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26271,'1971-W1368','1971-10-12 22:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26272,'1971-W1369','1971-10-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 26273,'1971-W137','1971-01-25 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 26274,'1971-W1370','1971-10-13 14:37:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26275,'1971-W1371','1971-10-13 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 26276,'1971-W1372','1971-10-13 17:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 26277,'1971-W1373','1971-10-13 17:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 26278,'1971-W1374','1971-10-13 17:50:00','suborbital rocket','success',258,'','','','','Ionos',1895,'',69 from dual union all
select 26279,'1971-W1375','1971-10-13 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 2628,'1954-A599','1954-10-28 00:00:00','atmospheric rocket','success',836,'2821T','','','','Test',1895,'',10 from dual union all
select 26280,'1971-W1376','1971-10-13 18:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 26281,'1971-W1377','1971-10-13 19:59:00','suborbital rocket','success',258,'NASA 15.15GI','','','','Ionos',1895,'',63 from dual union all
select 26282,'1971-W1378','1971-10-13 20:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26283,'1971-W1379','1971-10-13 20:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 26284,'1971-W138','1971-01-25 19:11:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 26285,'1971-W1380','1971-10-14 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 26286,'1971-W1381','1971-10-14 14:30:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',308,'',57 from dual union all
select 26287,'1971-W1382','1971-10-14 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 26288,'1971-W1383','1971-10-14 22:37:59','suborbital rocket','success',1061,'','Starute','','','Meteo',276,'',64 from dual;
insert into LAUNCH
select 26289,'1971-W1384','1971-10-15 00:04:59','suborbital rocket','success',1061,'','Starute','','','Meteo',276,'',65 from dual union all
select 2629,'1954-A60','1954-02-17 00:00:00','atmospheric rocket','unknown',576,'219M','','','','Test',1895,'',10 from dual union all
select 26290,'1971-W1385','1971-10-15 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',64 from dual union all
select 26291,'1971-W1386','1971-10-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 26292,'1971-W1387','1971-10-15 14:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 26293,'1971-W1388','1971-10-15 14:47:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26294,'1971-W1389','1971-10-15 18:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',64 from dual union all
select 26295,'1971-W139','1971-01-25 19:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 26296,'1971-W1390','1971-10-16 02:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 26297,'1971-W1391','1971-10-18 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 26298,'1971-W1392','1971-10-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 26299,'1971-W1393','1971-10-18 14:42:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 263,'1944-S239','1944-11-10 09:31:00','military missile','success',1470,'19516','Ma280','','','Test',661,'',90 from dual union all
select 2630,'1954-A600','1954-11-01 00:00:00','military missile','unknown',339,'T-17','','','','Test',1895,'',40 from dual union all
select 26300,'1971-W1394','1971-10-18 18:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 26301,'1971-W1395','1971-10-18 21:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26302,'1971-W1396','1971-10-19 01:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 26303,'1971-W1397','1971-10-19 15:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26304,'1971-W1398','1971-10-19 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 26305,'1971-W1399','1971-10-20 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 26306,'1971-W14','1971-01-06 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 26307,'1971-W140','1971-01-25 20:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',53 from dual union all
select 26308,'1971-W1400','1971-10-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 26309,'1971-W1401','1971-10-20 14:38:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 2631,'1954-A601','1954-11-01 00:00:00','atmospheric rocket','success',836,'3316T','','','','Test',1895,'',10 from dual union all
select 26310,'1971-W1402','1971-10-20 15:02:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',308,'',63 from dual union all
select 26311,'1971-W1403','1971-10-20 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 26312,'1971-W1404','1971-10-20 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26313,'1971-W1405','1971-10-20 20:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',57 from dual union all
select 26314,'1971-W1406','1971-10-20 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 26315,'1971-W1407','1971-10-21 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 26316,'1971-W1408','1971-10-21 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26317,'1971-W1409','1971-10-21 21:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 26318,'1971-W141','1971-01-25 22:51:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26319,'1971-W1410','1971-10-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 2632,'1954-A602','1954-11-02 00:00:00','atmospheric rocket','unknown',576,'450M','','','','Test',1895,'',10 from dual union all
select 26320,'1971-W1411','1971-10-22 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 26321,'1971-W1412','1971-10-22 16:49:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 26322,'1971-W1413','1971-10-22 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 26323,'1971-W1414','1971-10-22 17:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26324,'1971-W1415','1971-10-22 21:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 26325,'1971-W1416','1971-10-22 22:03:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',55 from dual union all
select 26326,'1971-W1417','1971-10-23 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 26327,'1971-W1418','1971-10-25 14:35:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26328,'1971-W1419','1971-10-26 16:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26329,'1971-W142','1971-01-26 00:00:00','suborbital rocket','success',1188,'M--326A','','','','Aeron',2,'',61 from dual union all
select 2633,'1954-A603','1954-11-02 00:00:00','atmospheric rocket','unknown',576,'416M','','','','Test',1895,'',10 from dual union all
select 26330,'1971-W1420','1971-10-26 19:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 26331,'1971-W1421','1971-10-26 19:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 26332,'1971-W1422','1971-10-26 21:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26333,'1971-W1423','1971-10-27 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',54 from dual union all
select 26334,'1971-W1424','1971-10-27 13:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 26335,'1971-W1425','1971-10-27 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 26336,'1971-W1426','1971-10-27 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 26337,'1971-W1427','1971-10-27 14:40:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 26338,'1971-W1428','1971-10-27 14:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',308,'',61 from dual union all
select 26339,'1971-W1429','1971-10-27 16:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 2634,'1954-A604','1954-11-03 00:00:00','atmospheric rocket','unknown',576,'455M','','','','Test',1895,'',10 from dual union all
select 26340,'1971-W143','1971-01-26 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 26341,'1971-W1430','1971-10-27 16:29:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 26342,'1971-W1431','1971-10-27 16:51:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 26343,'1971-W1432','1971-10-27 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 26344,'1971-W1433','1971-10-27 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26345,'1971-W1434','1971-10-28 01:42:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 26346,'1971-W1435','1971-10-28 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26347,'1971-W1436','1971-10-28 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26348,'1971-W1437','1971-10-29 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 26349,'1971-W1438','1971-10-29 14:43:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 2635,'1954-A605','1954-11-03 00:00:00','atmospheric rocket','success',836,'2785R','Nike 392R','','','Test',1895,'',10 from dual union all
select 26350,'1971-W1439','1971-10-29 17:02:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',59 from dual union all
select 26351,'1971-W144','1971-01-26 17:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26352,'1971-W1440','1971-10-29 17:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 26353,'1971-W1441','1971-10-29 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 26354,'1971-W1442','1971-10-29 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26355,'1971-W1443','1971-11-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 26356,'1971-W1444','1971-11-01 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 26357,'1971-W1445','1971-11-01 15:41:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26358,'1971-W1446','1971-11-01 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 26359,'1971-W1447','1971-11-01 18:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 2636,'1954-A606','1954-11-04 00:00:00','atmospheric rocket','unknown',576,'742','','','','Test',1895,'',10 from dual union all
select 26360,'1971-W1448','1971-11-01 21:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 26361,'1971-W145','1971-01-26 19:18:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 26362,'1971-W1450','1971-11-02 20:41:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 26363,'1971-W1451','1971-11-02 21:00:00','suborbital rocket','success',88,'','Arcas Starute 2/STS','','','Test/Aeron',1895,'',66 from dual union all
select 26364,'1971-W1452','1971-11-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 26365,'1971-W1453','1971-11-03 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 26366,'1971-W1454','1971-11-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',73 from dual union all
select 26367,'1971-W1455','1971-11-03 15:46:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 26368,'1971-W1456','1971-11-03 17:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 26369,'1971-W1457','1971-11-03 18:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',60 from dual union all
select 2637,'1954-A607','1954-11-04 00:00:00','atmospheric rocket','unknown',576,'653','','','','Test',1895,'',10 from dual union all
select 26370,'1971-W1458','1971-11-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 26371,'1971-W1459','1971-11-03 22:04:59','suborbital rocket','success',716,'INTA HT-7119','','','','Meteo',396,'',56 from dual union all
select 26372,'1971-W146','1971-01-26 20:15:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',56 from dual union all
select 26373,'1971-W1460','1971-11-04 13:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',308,'',62 from dual union all
select 26374,'1971-W1461','1971-11-04 15:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 26375,'1971-W1462','1971-11-04 15:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 26376,'1971-W1463','1971-11-04 15:45:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 26377,'1971-W1464','1971-11-04 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 26378,'1971-W1465','1971-11-04 18:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual;
insert into LAUNCH
select 26379,'1971-W1466','1971-11-05 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 2638,'1954-A608','1954-11-04 00:00:00','atmospheric rocket','success',836,'2835T','','','','Test',1895,'',10 from dual union all
select 26380,'1971-W1467','1971-11-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 26381,'1971-W1468','1971-11-05 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 26382,'1971-W1469','1971-11-05 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 26383,'1971-W147','1971-01-26 21:30:00','suborbital rocket','unknown',88,'WSMR/PSU','Conductivity','','','Ionos',1895,'',76 from dual union all
select 26384,'1971-W1470','1971-11-05 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 26385,'1971-W1471','1971-11-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 26386,'1971-W1472','1971-11-08 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 26387,'1971-W1473','1971-11-08 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 26388,'1971-W1474','1971-11-08 15:40:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 26389,'1971-W1475','1971-11-08 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 2639,'1954-A609','1954-11-04 00:00:00','atmospheric rocket','success',836,'3255T','','','','Test',1895,'',10 from dual union all
select 26390,'1971-W1476','1971-11-08 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 26391,'1971-W1477','1971-11-08 19:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26392,'1971-W1478','1971-11-08 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 26393,'1971-W1479','1971-11-08 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 26394,'1971-W148','1971-01-26 22:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',60 from dual union all
select 26395,'1971-W1480','1971-11-09 13:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 26396,'1971-W1481','1971-11-09 16:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',59 from dual union all
select 26397,'1971-W1482','1971-11-09 20:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26398,'1971-W1483','1971-11-10 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 26399,'1971-W1484','1971-11-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 264,'1944-S24','1944-05-01 00:00:00','military missile','success',8,'','(30 in May)','','','Test',662,'',90 from dual union all
select 2640,'1954-A61','1954-02-17 00:00:00','atmospheric rocket','unknown',576,'220M','','','','Test',1895,'',10 from dual union all
select 26400,'1971-W1485','1971-11-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 26401,'1971-W1486','1971-11-10 15:44:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 26402,'1971-W1487','1971-11-10 17:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 26403,'1971-W1488','1971-11-10 18:14:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',63 from dual union all
select 26404,'1971-W1489','1971-11-10 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 26405,'1971-W149','1971-01-27 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 26406,'1971-W1490','1971-11-10 22:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 26407,'1971-W1491','1971-11-11 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 26408,'1971-W1492','1971-11-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 26409,'1971-W1493','1971-11-12 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 2641,'1954-A610','1954-11-05 00:00:00','atmospheric rocket','unknown',576,'735','','','','Test',1895,'',10 from dual union all
select 26410,'1971-W1494','1971-11-12 15:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',67 from dual union all
select 26411,'1971-W1495','1971-11-12 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',57 from dual union all
select 26412,'1971-W1496','1971-11-12 17:48:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',54 from dual union all
select 26413,'1971-W1497','1971-11-12 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 26414,'1971-W1498','1971-11-12 18:59:59','suborbital rocket','success',1061,'','Density gauge','','','Meteo',1895,'',59 from dual union all
select 26415,'1971-W1499','1971-11-15 14:40:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',65 from dual union all
select 26416,'1971-W15','1971-01-06 20:18:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 26417,'1971-W150','1971-01-27 14:22:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',63 from dual union all
select 26418,'1971-W1500','1971-11-15 15:45:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 26419,'1971-W1501','1971-11-15 16:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',63 from dual union all
select 2642,'1954-A611','1954-11-05 00:00:00','atmospheric rocket','success',836,'2608A','','','','Test',1895,'',10 from dual union all
select 26420,'1971-W1502','1971-11-15 18:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',53 from dual union all
select 26421,'1971-W1503','1971-11-15 18:00:59','suborbital rocket','success',1061,'','Density gauge','','','Meteo',1307,'',62 from dual union all
select 26422,'1971-W1504','1971-11-15 18:40:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',55 from dual union all
select 26423,'1971-W1505','1971-11-15 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 26424,'1971-W1506','1971-11-15 20:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 26425,'1971-W1507','1971-11-15 22:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26426,'1971-W1508','1971-11-16 15:46:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 26427,'1971-W1509','1971-11-16 16:44:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',53 from dual union all
select 26428,'1971-W151','1971-01-27 15:34:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',52 from dual union all
select 26429,'1971-W1510','1971-11-16 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 2643,'1954-A612','1954-11-08 00:00:00','atmospheric rocket','success',836,'2766A','','','','Test',1895,'',10 from dual union all
select 26430,'1971-W1511','1971-11-16 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26431,'1971-W1512','1971-11-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 26432,'1971-W1513','1971-11-17 14:30:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',308,'',63 from dual union all
select 26433,'1971-W1514','1971-11-17 15:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',62 from dual union all
select 26434,'1971-W1515','1971-11-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 26435,'1971-W1516','1971-11-17 15:53:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 26436,'1971-W1517','1971-11-17 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 26437,'1971-W1518','1971-11-17 20:23:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 26438,'1971-W1519','1971-11-17 20:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26439,'1971-W152','1971-01-27 16:22:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 2644,'1954-A613','1954-11-08 00:00:00','atmospheric rocket','success',836,'2586E','','','','Test',1895,'',10 from dual union all
select 26440,'1971-W1520','1971-11-17 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 26441,'1971-W1521','1971-11-18 15:45:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 26442,'1971-W1522','1971-11-18 17:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',67 from dual union all
select 26443,'1971-W1523','1971-11-18 17:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26444,'1971-W1524','1971-11-18 18:04:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',63 from dual union all
select 26445,'1971-W1525','1971-11-18 20:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26446,'1971-W1526','1971-11-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 26447,'1971-W1527','1971-11-19 14:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',70 from dual union all
select 26448,'1971-W1528','1971-11-19 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',51 from dual union all
select 26449,'1971-W1529','1971-11-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 2645,'1954-A614','1954-11-08 00:00:00','atmospheric rocket','success',836,'2708E','','','','Test',1895,'',10 from dual union all
select 26450,'1971-W153','1971-01-27 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 26451,'1971-W1530','1971-11-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',53 from dual union all
select 26452,'1971-W1531','1971-11-19 15:46:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26453,'1971-W1532','1971-11-19 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 26454,'1971-W1533','1971-11-19 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',50 from dual union all
select 26455,'1971-W1534','1971-11-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 26456,'1971-W1535','1971-11-22 15:56:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 26457,'1971-W1536','1971-11-22 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 26458,'1971-W1537','1971-11-22 16:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26459,'1971-W1538','1971-11-22 19:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 2646,'1954-A615','1954-11-08 00:00:00','atmospheric rocket','success',836,'2813A','','','','Test',1895,'',10 from dual union all
select 26460,'1971-W1539','1971-11-22 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 26461,'1971-W154','1971-01-27 18:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',50 from dual union all
select 26462,'1971-W1540','1971-11-22 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 26463,'1971-W1541','1971-11-23 15:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26464,'1971-W1542','1971-11-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 26465,'1971-W1543','1971-11-24 15:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',63 from dual union all
select 26466,'1971-W1544','1971-11-24 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 26467,'1971-W1545','1971-11-24 15:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 26468,'1971-W1546','1971-11-24 15:53:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual;
insert into LAUNCH
select 26469,'1971-W1547','1971-11-24 16:32:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 2647,'1954-A616','1954-11-09 00:00:00','atmospheric rocket','unknown',576,'471M','','','','Test',1895,'',10 from dual union all
select 26470,'1971-W1548','1971-11-24 17:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26471,'1971-W1549','1971-11-24 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',55 from dual union all
select 26472,'1971-W155','1971-01-27 18:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 26473,'1971-W1550','1971-11-24 20:04:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',56 from dual union all
select 26474,'1971-W1551','1971-11-24 20:55:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',51 from dual union all
select 26475,'1971-W1552','1971-11-24 23:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 26476,'1971-W1553','1971-11-25 16:02:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 26477,'1971-W1554','1971-11-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 26478,'1971-W1555','1971-11-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 26479,'1971-W1556','1971-11-26 14:32:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',64 from dual union all
select 2648,'1954-A617','1954-11-09 00:00:00','atmospheric rocket','unknown',576,'473M','','','','Test',1895,'',10 from dual union all
select 26480,'1971-W1557','1971-11-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 26481,'1971-W1558','1971-11-26 15:49:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26482,'1971-W1559','1971-11-26 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 26483,'1971-W156','1971-01-27 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 26484,'1971-W1560','1971-11-26 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',51 from dual union all
select 26485,'1971-W1561','1971-11-29 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 26486,'1971-W1562','1971-11-29 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 26487,'1971-W1563','1971-11-29 15:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 26488,'1971-W1564','1971-11-29 15:53:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 26489,'1971-W1565','1971-11-29 17:18:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1307,'',63 from dual union all
select 2649,'1954-A618','1954-11-09 00:00:00','atmospheric rocket','success',836,'1470E','','','','Test',1895,'',10 from dual union all
select 26490,'1971-W1566','1971-11-29 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 26491,'1971-W1567','1971-11-29 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 26492,'1971-W1568','1971-11-29 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 26493,'1971-W1569','1971-11-29 23:13:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26494,'1971-W157','1971-01-27 20:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26495,'1971-W1570','1971-11-30 15:45:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 26496,'1971-W1571','1971-11-30 16:30:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',74 from dual union all
select 26497,'1971-W1572','1971-11-30 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 26498,'1971-W1573','1971-11-30 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 26499,'1971-W1574','1971-11-30 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 265,'1944-S240','1944-11-10 12:06:00','military missile','success',1470,'19518','Ma282','','','Test',661,'',90 from dual union all
select 2650,'1954-A619','1954-11-09 00:00:00','atmospheric rocket','success',836,'1670E','','','','Test',1895,'',10 from dual union all
select 26500,'1971-W1575','1971-12-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 26501,'1971-W1576','1971-12-01 14:22:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',66 from dual union all
select 26502,'1971-W1577','1971-12-01 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 26503,'1971-W1578','1971-12-01 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 26504,'1971-W1579','1971-12-01 15:47:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 26505,'1971-W158','1971-01-27 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 26506,'1971-W1580','1971-12-01 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 26507,'1971-W1581','1971-12-01 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',70 from dual union all
select 26508,'1971-W1582','1971-12-01 21:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 26509,'1971-W1583','1971-12-01 21:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 2651,'1954-A62','1954-02-17 00:00:00','atmospheric rocket','success',836,'1585A','','','','Test',1895,'',10 from dual union all
select 26510,'1971-W1584','1971-12-02 15:46:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 26511,'1971-W1585','1971-12-02 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 26512,'1971-W1586','1971-12-02 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26513,'1971-W1587','1971-12-03 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 26514,'1971-W1588','1971-12-03 15:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 26515,'1971-W1589','1971-12-03 15:55:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 26516,'1971-W159','1971-01-28 00:00:00','suborbital rocket','success',1188,'M--328A','','','','Aeron',2,'',57 from dual union all
select 26517,'1971-W1590','1971-12-03 17:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 26518,'1971-W1591','1971-12-03 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 26519,'1971-W1592','1971-12-03 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 2652,'1954-A620','1954-11-09 00:00:00','atmospheric rocket','success',836,'3331T','','','','Test',1895,'',10 from dual union all
select 26520,'1971-W1593','1971-12-06 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 26521,'1971-W1594','1971-12-06 15:49:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 26522,'1971-W1595','1971-12-06 16:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 26523,'1971-W1596','1971-12-06 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 26524,'1971-W1597','1971-12-06 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 26525,'1971-W1598','1971-12-07 17:56:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 26526,'1971-W1599','1971-12-07 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 26527,'1971-W16','1971-01-07 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26528,'1971-W160','1971-01-28 01:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 26529,'1971-W1600','1971-12-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 2653,'1954-A621','1954-11-10 00:00:00','atmospheric rocket','unknown',576,'547M','','','','Test',1895,'',10 from dual union all
select 26530,'1971-W1601','1971-12-08 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',50 from dual union all
select 26531,'1971-W1602','1971-12-08 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 26532,'1971-W1603','1971-12-08 15:54:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 26533,'1971-W1604','1971-12-08 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',57 from dual union all
select 26534,'1971-W1605','1971-12-08 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 26535,'1971-W1606','1971-12-08 20:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 26536,'1971-W1607','1971-12-08 21:32:00','suborbital rocket','success',716,'INTA HT-7120','','','','Meteo',396,'',56 from dual union all
select 26537,'1971-W1608','1971-12-08 21:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26538,'1971-W1609','1971-12-09 15:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',78 from dual union all
select 26539,'1971-W161','1971-01-28 18:24:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',61 from dual union all
select 2654,'1954-A622','1954-11-10 00:00:00','atmospheric rocket','success',836,'2817R','Nike 393R','','','Test',1895,'',10 from dual union all
select 26540,'1971-W1610','1971-12-09 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',55 from dual union all
select 26541,'1971-W1611','1971-12-09 17:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26542,'1971-W1612','1971-12-09 19:29:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 26543,'1971-W1613','1971-12-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 26544,'1971-W1614','1971-12-10 14:04:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26545,'1971-W1615','1971-12-10 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 26546,'1971-W1616','1971-12-10 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',54 from dual union all
select 26547,'1971-W1617','1971-12-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 26548,'1971-W1618','1971-12-10 15:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 26549,'1971-W1619','1971-12-10 15:38:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 2655,'1954-A623','1954-11-10 00:00:00','atmospheric rocket','success',836,'2535R','Nike 396R','','','Test',1895,'',10 from dual union all
select 26550,'1971-W162','1971-01-28 19:04:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',60 from dual union all
select 26551,'1971-W1620','1971-12-10 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 26552,'1971-W1621','1971-12-10 16:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26553,'1971-W1622','1971-12-10 16:49:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 26554,'1971-W1623','1971-12-10 18:23:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 26555,'1971-W1624','1971-12-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 26556,'1971-W1625','1971-12-13 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 26557,'1971-W1626','1971-12-13 15:58:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 26558,'1971-W1627','1971-12-13 16:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual;
insert into LAUNCH
select 26559,'1971-W1628','1971-12-13 17:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 2656,'1954-A624','1954-11-10 00:00:00','atmospheric rocket','success',836,'2801T','','','','Test',1895,'',10 from dual union all
select 26560,'1971-W1629','1971-12-13 18:35:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',58 from dual union all
select 26561,'1971-W163','1971-01-28 20:13:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',60 from dual union all
select 26562,'1971-W1630','1971-12-13 19:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 26563,'1971-W1631','1971-12-14 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 26564,'1971-W1632','1971-12-14 22:00:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',63 from dual union all
select 26565,'1971-W1633','1971-12-14 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 26566,'1971-W1634','1971-12-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 26567,'1971-W1635','1971-12-15 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 26568,'1971-W1636','1971-12-15 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 26569,'1971-W1637','1971-12-15 15:47:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 2657,'1954-A625','1954-11-10 00:00:00','atmospheric rocket','success',836,'2750A','','','','Test',1895,'',10 from dual union all
select 26570,'1971-W1638','1971-12-15 16:31:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',60 from dual union all
select 26571,'1971-W1639','1971-12-15 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',57 from dual union all
select 26572,'1971-W164','1971-01-28 20:26:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',58 from dual union all
select 26573,'1971-W1640','1971-12-15 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 26574,'1971-W1641','1971-12-15 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26575,'1971-W1642','1971-12-16 00:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',308,'',59 from dual union all
select 26576,'1971-W1643','1971-12-16 14:41:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',78 from dual union all
select 26577,'1971-W1644','1971-12-16 15:45:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 26578,'1971-W1645','1971-12-16 16:08:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 26579,'1971-W1646','1971-12-16 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 2658,'1954-A626','1954-11-10 00:00:00','atmospheric rocket','success',836,'2969R','Nike 395R','','','Test',1895,'',10 from dual union all
select 26580,'1971-W1647','1971-12-16 22:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',64 from dual union all
select 26581,'1971-W1648','1971-12-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 26582,'1971-W1649','1971-12-17 15:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 26583,'1971-W165','1971-01-28 20:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26584,'1971-W1650','1971-12-17 15:40:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26585,'1971-W1651','1971-12-17 16:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26586,'1971-W1652','1971-12-17 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 26587,'1971-W1653','1971-12-17 23:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 26588,'1971-W1654','1971-12-20 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',52 from dual union all
select 26589,'1971-W1655','1971-12-20 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 2659,'1954-A627','1954-11-10 00:00:00','atmospheric rocket','success',836,'2811R','Nike 394R','','','Test',1895,'',10 from dual union all
select 26590,'1971-W1656','1971-12-20 16:03:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 26591,'1971-W1657','1971-12-20 16:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 26592,'1971-W1658','1971-12-20 16:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26593,'1971-W1659','1971-12-20 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 26594,'1971-W166','1971-01-28 21:14:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',68 from dual union all
select 26595,'1971-W1660','1971-12-21 15:57:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 26596,'1971-W1661','1971-12-21 16:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 26597,'1971-W1662','1971-12-21 20:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 26598,'1971-W1663','1971-12-22 12:04:59','suborbital rocket','success',716,'INTA HT-7122','','','','Meteo',396,'',52 from dual union all
select 26599,'1971-W1664','1971-12-22 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 266,'1944-S241','1944-11-12 10:38:00','military missile','success',1470,'19528','Ma278','','','Test',661,'',90 from dual union all
select 2660,'1954-A628','1954-11-11 00:00:00','atmospheric rocket','success',836,'3320T','','','','Test',1895,'',10 from dual union all
select 26600,'1971-W1665','1971-12-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 26601,'1971-W1666','1971-12-22 15:45:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 26602,'1971-W1667','1971-12-22 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 26603,'1971-W1668','1971-12-22 20:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26604,'1971-W1669','1971-12-22 22:18:59','suborbital rocket','success',716,'INTA HT-7123','','','','Meteo',396,'',56 from dual union all
select 26605,'1971-W167','1971-01-29 14:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 26606,'1971-W1670','1971-12-23 15:45:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 26607,'1971-W1671','1971-12-23 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 26608,'1971-W1672','1971-12-24 15:47:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 26609,'1971-W1673','1971-12-27 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 2661,'1954-A629','1954-11-12 00:00:00','atmospheric rocket','success',836,'2777T','','','','Test',1895,'',10 from dual union all
select 26610,'1971-W1674','1971-12-27 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26611,'1971-W1675','1971-12-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 26612,'1971-W1676','1971-12-28 16:36:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 26613,'1971-W1677','1971-12-28 19:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 26614,'1971-W1678','1971-12-28 22:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26615,'1971-W1679','1971-12-29 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 26616,'1971-W168','1971-01-29 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 26617,'1971-W1680','1971-12-29 15:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 26618,'1971-W1681','1971-12-29 15:41:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 26619,'1971-W1682','1971-12-29 18:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 2662,'1954-A63','1954-02-17 00:00:00','atmospheric rocket','success',836,'1232A','','','','Test',1895,'',10 from dual union all
select 26620,'1971-W1683','1971-12-29 20:55:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 26621,'1971-W1684','1971-12-29 21:37:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',73 from dual union all
select 26622,'1971-W1685','1971-12-29 22:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26623,'1971-W1686','1971-12-30 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 26624,'1971-W1687','1971-12-30 15:33:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 26625,'1971-W1688','1971-12-30 17:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26626,'1971-W1689','1971-12-30 20:47:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 26627,'1971-W169','1971-01-29 15:37:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',63 from dual union all
select 26628,'1971-W1690','1971-12-30 21:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 26629,'1971-W1691','1971-12-31 15:41:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 2663,'1954-A630','1954-11-12 00:00:00','military missile','success',1214,'N-69B GM-11116','','','','Test',222,'',10 from dual union all
select 26630,'1971-W1692','1971-01-06 02:01:00','suborbital rocket','success',761,'MT135P-R12','','','','Aeron',1332,'',51 from dual union all
select 26631,'1971-W1693','1971-01-20 02:00:00','suborbital rocket','success',761,'MT135P-R14','','','','Aeron',1332,'',50 from dual union all
select 26632,'1971-W1694','1971-02-03 02:00:00','suborbital rocket','success',761,'MT135P-R16','','','','Aeron',1332,'',56 from dual union all
select 26633,'1971-W1695','1971-02-11 02:00:00','suborbital rocket','success',761,'MT135P-R17','','','','Aeron',1332,'',51 from dual union all
select 26634,'1971-W1696','1971-02-18 02:00:00','suborbital rocket','success',761,'MT135P-R18','','','','Aeron',1332,'',53 from dual union all
select 26635,'1971-W1697','1971-03-11 02:00:00','suborbital rocket','success',761,'MT135P-R20','','','','Aeron',1332,'',56 from dual union all
select 26636,'1971-W1698','1971-03-24 02:00:00','suborbital rocket','success',761,'MT135P-R21','','','','Aeron',1332,'',54 from dual union all
select 26637,'1971-W1699','1971-04-07 02:00:00','suborbital rocket','success',761,'MT135P-R22','','','','Aeron',1332,'',54 from dual union all
select 26638,'1971-W17','1971-01-07 19:07:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 26639,'1971-W170','1971-01-29 15:51:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',67 from dual union all
select 2664,'1954-A631','1954-11-15 00:00:00','atmospheric rocket','unknown',576,'795','','','','Test',1895,'',10 from dual union all
select 26640,'1971-W1700','1971-04-21 02:00:00','suborbital rocket','success',761,'MT135P-R23','','','','Aeron',1332,'',54 from dual union all
select 26641,'1971-W1701','1971-05-06 02:00:00','suborbital rocket','success',761,'MT135P-R24','','','','Aeron',1332,'',55 from dual union all
select 26642,'1971-W1702','1971-05-12 02:00:00','suborbital rocket','success',761,'MT135P-R25','','','','Aeron',1332,'',49 from dual union all
select 26643,'1971-W171','1971-01-29 15:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 26644,'1971-W172','1971-01-29 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26645,'1971-W173','1971-01-29 17:21:00','suborbital rocket','success',1189,'Sk83/9','ESRO Sk83/9','','','Aeronomy',417,'',71 from dual union all
select 26646,'1971-W174','1971-01-29 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',54 from dual union all
select 26647,'1971-W175','1971-01-30 01:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 26648,'1971-W176','1971-01-30 14:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual;
insert into LAUNCH
select 26649,'1971-W177','1971-01-31 16:45:00','suborbital rocket','success',1189,'Sk83/10','ESRO Sk83/10','','','Aeronomy',417,'',70 from dual union all
select 2665,'1954-A632','1954-11-15 00:00:00','atmospheric rocket','unknown',576,'791','','','','Test',1895,'',10 from dual union all
select 26650,'1971-W178','1971-01-31 23:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 26651,'1971-W179','1971-02-01 00:00:00','suborbital rocket','success',1188,'M--335H','','','','Aeron',1447,'',58 from dual union all
select 26652,'1971-W18','1971-01-07 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 26653,'1971-W180','1971-02-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 26654,'1971-W181','1971-02-01 16:36:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 26655,'1971-W182','1971-02-01 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 26656,'1971-W183','1971-02-01 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 26657,'1971-W184','1971-02-01 17:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26658,'1971-W185','1971-02-01 19:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 26659,'1971-W186','1971-02-01 19:14:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',67 from dual union all
select 2666,'1954-A633','1954-11-16 00:00:00','military missile','success',573,'6','XSSM-A-16','','','Test',1863,'',34 from dual union all
select 26660,'1971-W187','1971-02-01 19:15:00','suborbital rocket','unknown',88,'WSMR/PSU','Conductivity','','','Ionos',1895,'',76 from dual union all
select 26661,'1971-W188','1971-02-01 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',53 from dual union all
select 26662,'1971-W189','1971-02-01 21:49:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',70 from dual union all
select 26663,'1971-W19','1971-01-07 21:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 26664,'1971-W190','1971-02-02 01:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 26665,'1971-W191','1971-02-02 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',52 from dual union all
select 26666,'1971-W192','1971-02-02 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',57 from dual union all
select 26667,'1971-W193','1971-02-02 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 26668,'1971-W194','1971-02-03 00:00:00','suborbital rocket','success',1188,'M--336H','','','','Aeron',1447,'',58 from dual union all
select 26669,'1971-W195','1971-02-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 2667,'1954-A634','1954-11-16 00:00:00','atmospheric rocket','success',836,'2786A','','','','Test',1895,'',10 from dual union all
select 26670,'1971-W196','1971-02-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 26671,'1971-W197','1971-02-03 14:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 26672,'1971-W198','1971-02-03 15:48:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 26673,'1971-W199','1971-02-03 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',75 from dual union all
select 26674,'1971-W20','1971-01-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 26675,'1971-W200','1971-02-03 17:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26676,'1971-W201','1971-02-03 18:12:00','suborbital rocket','success',1189,'Sk83/11','ESRO Sk83/11','','','Aeronomy',417,'',68 from dual union all
select 26677,'1971-W202','1971-02-03 18:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 26678,'1971-W203','1971-02-03 19:43:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',61 from dual union all
select 26679,'1971-W204','1971-02-03 20:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 2668,'1954-A635','1954-11-17 00:00:00','atmospheric rocket','unknown',576,'442','','','','Test',1895,'',10 from dual union all
select 26680,'1971-W205','1971-02-03 21:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',64 from dual union all
select 26681,'1971-W206','1971-02-04 00:00:00','suborbital rocket','success',1188,'M--330A','','','','Aeron',2,'',62 from dual union all
select 26682,'1971-W207','1971-02-04 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 26683,'1971-W208','1971-02-04 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',70 from dual union all
select 26684,'1971-W209','1971-02-04 18:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26685,'1971-W21','1971-01-08 15:02:00','suborbital rocket','success',1189,'Sk83/1','ESRO Sk83/1','','','Aeronomy',417,'',67 from dual union all
select 26686,'1971-W210','1971-02-04 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 26687,'1971-W211','1971-02-04 20:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',70 from dual union all
select 26688,'1971-W212','1971-02-04 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 26689,'1971-W213','1971-02-05 00:00:00','suborbital rocket','success',1188,'M--337H','','','','Aeron',1447,'',58 from dual union all
select 2669,'1954-A636','1954-11-17 00:00:00','atmospheric rocket','success',836,'2939R','Nike 400R','','','Test',1895,'',10 from dual union all
select 26690,'1971-W214','1971-02-05 00:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 26691,'1971-W215','1971-02-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 26692,'1971-W216','1971-02-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',55 from dual union all
select 26693,'1971-W217','1971-02-05 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 26694,'1971-W218','1971-02-05 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 26695,'1971-W219','1971-02-05 20:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26696,'1971-W22','1971-01-08 15:04:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',55 from dual union all
select 26697,'1971-W220','1971-02-05 21:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',50 from dual union all
select 26698,'1971-W221','1971-02-05 23:26:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26699,'1971-W222','1971-02-08 00:00:00','suborbital rocket','success',1188,'M--332A','','','','Aeron',2,'',62 from dual union all
select 267,'1944-S242','1944-11-12 14:28:00','military missile','success',1470,'19510','Ma263','','','Test',661,'',90 from dual union all
select 2670,'1954-A637','1954-11-17 00:00:00','atmospheric rocket','success',836,'3001R','Nike 399R','','','Test',1895,'',10 from dual union all
select 26700,'1971-W223','1971-02-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 26701,'1971-W224','1971-02-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 26702,'1971-W225','1971-02-08 16:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26703,'1971-W226','1971-02-08 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',57 from dual union all
select 26704,'1971-W227','1971-02-08 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 26705,'1971-W228','1971-02-08 20:13:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 26706,'1971-W229','1971-02-08 21:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',53 from dual union all
select 26707,'1971-W23','1971-01-08 16:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 26708,'1971-W230','1971-02-09 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 26709,'1971-W231','1971-02-09 17:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 2671,'1954-A638','1954-11-17 00:00:00','atmospheric rocket','success',836,'3492T','','','','Test',1895,'',10 from dual union all
select 26710,'1971-W232','1971-02-09 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26711,'1971-W233','1971-02-09 18:00:00','suborbital rocket','success',1189,'Sk83/13','ESRO Sk83/13','','','Aeronomy',417,'',67 from dual union all
select 26712,'1971-W234','1971-02-09 19:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 26713,'1971-W235','1971-02-09 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 26714,'1971-W236','1971-02-10 00:00:00','suborbital rocket','success',1188,'M--338H','','','','Aeron',1447,'',60 from dual union all
select 26715,'1971-W237','1971-02-10 00:00:00','suborbital rocket','success',1188,'M--334A','','','','Aeron',2,'',62 from dual union all
select 26716,'1971-W238','1971-02-10 14:11:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 26717,'1971-W239','1971-02-10 14:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',63 from dual union all
select 26718,'1971-W24','1971-01-08 17:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26719,'1971-W240','1971-02-10 15:15:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 2672,'1954-A639','1954-11-17 00:00:00','atmospheric rocket','success',836,'1327R','Nike 397R','','','Test',1895,'',10 from dual union all
select 26720,'1971-W241','1971-02-10 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',60 from dual union all
select 26721,'1971-W242','1971-02-10 16:12:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 26722,'1971-W243','1971-02-10 16:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26723,'1971-W244','1971-02-10 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',53 from dual union all
select 26724,'1971-W245','1971-02-10 18:01:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 26725,'1971-W246','1971-02-10 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 26726,'1971-W247','1971-02-10 22:31:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',61 from dual union all
select 26727,'1971-W248','1971-02-10 23:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',64 from dual union all
select 26728,'1971-W249','1971-02-11 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 26729,'1971-W25','1971-01-08 17:45:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',62 from dual union all
select 2673,'1954-A64','1954-02-18 00:00:00','military missile','unknown',339,'1294RD','','','','Test',1895,'',30 from dual union all
select 26730,'1971-W250','1971-02-11 15:55:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 26731,'1971-W251','1971-02-11 17:50:00','suborbital rocket','success',1189,'Sk83/14','ESRO Sk83/14','','','Aeronomy',417,'',65 from dual union all
select 26732,'1971-W252','1971-02-11 18:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 26733,'1971-W253','1971-02-11 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 26734,'1971-W254','1971-02-11 21:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 26735,'1971-W255','1971-02-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 26736,'1971-W256','1971-02-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 26737,'1971-W257','1971-02-12 16:24:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26738,'1971-W258','1971-02-12 17:30:00','suborbital rocket','success',1189,'Sk83/15','ESRO Sk83/15','','','Aeronomy',417,'',67 from dual;
insert into LAUNCH
select 26739,'1971-W259','1971-02-12 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 2674,'1954-A640','1954-11-17 00:00:00','atmospheric rocket','success',836,'2995R','Nike 398R','','','Test',1895,'',10 from dual union all
select 26740,'1971-W26','1971-01-08 18:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',51 from dual union all
select 26741,'1971-W260','1971-02-12 21:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 26742,'1971-W261','1971-02-13 02:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 26743,'1971-W262','1971-02-15 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 26744,'1971-W263','1971-02-16 00:19:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26745,'1971-W264','1971-02-16 16:47:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',52 from dual union all
select 26746,'1971-W265','1971-02-16 21:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26747,'1971-W266','1971-02-17 00:41:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 26748,'1971-W267','1971-02-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 26749,'1971-W268','1971-02-17 14:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 2675,'1954-A641','1954-11-17 18:12:00','military missile','failure',1053,'RS-6     XV','','','','Test',272,'',0 from dual union all
select 26750,'1971-W269','1971-02-17 15:00:00','suborbital rocket','success',631,'INTA H-7104','','','','Meteo',396,'',65 from dual union all
select 26751,'1971-W27','1971-01-08 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 26752,'1971-W270','1971-02-17 15:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 26753,'1971-W271','1971-02-17 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 26754,'1971-W272','1971-02-17 18:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26755,'1971-W273','1971-02-17 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 26756,'1971-W274','1971-02-17 19:44:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',65 from dual union all
select 26757,'1971-W275','1971-02-17 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 26758,'1971-W276','1971-02-18 02:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 26759,'1971-W277','1971-02-18 19:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 2676,'1954-A642','1954-11-18 00:00:00','military missile','unknown',339,'1551R','','','','Test',1895,'',40 from dual union all
select 26760,'1971-W278','1971-02-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 26761,'1971-W279','1971-02-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 26762,'1971-W28','1971-01-08 19:30:00','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',1895,'',56 from dual union all
select 26763,'1971-W280','1971-02-19 15:03:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 26764,'1971-W281','1971-02-19 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 26765,'1971-W282','1971-02-19 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 26766,'1971-W283','1971-02-19 21:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 26767,'1971-W284','1971-02-20 02:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',57 from dual union all
select 26768,'1971-W285','1971-02-20 03:23:00','suborbital rocket','success',257,'NASA 15.75UI','Auroral particles','','','Ionos',428,'',59 from dual union all
select 26769,'1971-W286','1971-02-20 04:46:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 2677,'1954-A643','1954-11-19 00:00:00','atmospheric rocket','unknown',576,'790','','','','Test',1895,'',10 from dual union all
select 26770,'1971-W287','1971-02-22 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',51 from dual union all
select 26771,'1971-W288','1971-02-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 26772,'1971-W289','1971-02-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 26773,'1971-W29','1971-01-08 19:38:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 26774,'1971-W290','1971-02-22 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26775,'1971-W291','1971-02-22 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 26776,'1971-W292','1971-02-22 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 26777,'1971-W293','1971-02-23 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 26778,'1971-W294','1971-02-23 20:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',66 from dual union all
select 26779,'1971-W295','1971-02-23 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 2678,'1954-A644','1954-11-19 00:00:00','atmospheric rocket','unknown',576,'789','','','','Test',1895,'',10 from dual union all
select 26780,'1971-W296','1971-02-23 21:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26781,'1971-W297','1971-02-24 12:22:59','suborbital rocket','success',631,'INTA H-7106','','','','Meteo',396,'',67 from dual union all
select 26782,'1971-W298','1971-02-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',54 from dual union all
select 26783,'1971-W299','1971-02-24 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 26784,'1971-W30','1971-01-08 22:46:59','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',1895,'',57 from dual union all
select 26785,'1971-W300','1971-02-24 14:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 26786,'1971-W301','1971-02-24 15:27:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',65 from dual union all
select 26787,'1971-W302','1971-02-24 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',71 from dual union all
select 26788,'1971-W303','1971-02-24 20:24:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 26789,'1971-W304','1971-02-24 21:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 2679,'1954-A645','1954-11-19 00:00:00','atmospheric rocket','success',836,'3249T','','','','Test',1895,'',10 from dual union all
select 26790,'1971-W305','1971-02-24 21:58:00','suborbital rocket','success',88,'NASA 15.84GM','','','','Aeronomy',855,'',57 from dual union all
select 26791,'1971-W306','1971-02-25 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26792,'1971-W307','1971-02-25 21:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26793,'1971-W308','1971-02-26 00:14:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',62 from dual union all
select 26794,'1971-W309','1971-02-26 01:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 26795,'1971-W31','1971-01-09 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 26796,'1971-W310','1971-02-26 13:31:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 26797,'1971-W311','1971-02-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 26798,'1971-W312','1971-02-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 26799,'1971-W313','1971-02-26 17:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 268,'1944-S243','1944-11-13 09:52:00','military missile','success',1470,'19512','Ma262','','','Test',661,'',90 from dual union all
select 2680,'1954-A646','1954-11-19 00:00:00','atmospheric rocket','success',836,'3329T','','','','Test',1895,'',10 from dual union all
select 26800,'1971-W314','1971-02-26 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 26801,'1971-W315','1971-02-26 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 26802,'1971-W316','1971-02-27 01:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 26803,'1971-W317','1971-03-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 26804,'1971-W318','1971-03-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 26805,'1971-W319','1971-03-01 17:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 26806,'1971-W32','1971-01-09 14:56:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',63 from dual union all
select 26807,'1971-W320','1971-03-01 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',57 from dual union all
select 26808,'1971-W321','1971-03-01 20:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26809,'1971-W322','1971-03-01 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 2681,'1954-A647','1954-11-19 00:00:00','atmospheric rocket','success',836,'3310T','','','','Test',1895,'',10 from dual union all
select 26810,'1971-W323','1971-03-02 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 26811,'1971-W324','1971-03-02 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',70 from dual union all
select 26812,'1971-W325','1971-03-02 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 26813,'1971-W326','1971-03-03 06:52:00','suborbital rocket','success',257,'NASA 15.89UI','Auroral particles','','','Ionos',428,'',65 from dual union all
select 26814,'1971-W327','1971-03-03 11:31:00','suborbital rocket','success',631,'INTA H-7107','','','','Meteo',396,'',66 from dual union all
select 26815,'1971-W328','1971-03-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 26816,'1971-W329','1971-03-03 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',73 from dual union all
select 26817,'1971-W33','1971-01-10 02:40:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',69 from dual union all
select 26818,'1971-W330','1971-03-03 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 26819,'1971-W331','1971-03-03 18:07:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',52 from dual union all
select 2682,'1954-A648','1954-11-19 00:00:00','atmospheric rocket','success',836,'2800E','','','','Test',1895,'',10 from dual union all
select 26820,'1971-W332','1971-03-03 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26821,'1971-W333','1971-03-03 21:58:00','suborbital rocket','success',88,'NASA 15.85GM','Nimbus cal','','','Aeronomy',855,'',59 from dual union all
select 26822,'1971-W334','1971-03-04 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 26823,'1971-W335','1971-03-04 15:42:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 26824,'1971-W336','1971-03-04 17:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',56 from dual union all
select 26825,'1971-W337','1971-03-04 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26826,'1971-W338','1971-03-04 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26827,'1971-W339','1971-03-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 26828,'1971-W34','1971-01-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual;
insert into LAUNCH
select 26829,'1971-W340','1971-03-05 15:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 2683,'1954-A649','1954-11-22 00:00:00','atmospheric rocket','unknown',576,'861','','','','Test',1895,'',10 from dual union all
select 26830,'1971-W341','1971-03-05 15:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 26831,'1971-W342','1971-03-05 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 26832,'1971-W343','1971-03-05 18:25:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 26833,'1971-W344','1971-03-05 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 26834,'1971-W345','1971-03-05 19:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26835,'1971-W346','1971-03-05 22:07:00','suborbital rocket','success',88,'NASA 15.86GM','Nimbus cal','','','Aeronomy',855,'',58 from dual union all
select 26836,'1971-W347','1971-03-05 22:09:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',53 from dual union all
select 26837,'1971-W348','1971-03-06 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 26838,'1971-W349','1971-03-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 26839,'1971-W35','1971-01-11 14:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',54 from dual union all
select 2684,'1954-A65','1954-02-18 00:00:00','atmospheric rocket','unknown',576,'203H','','','','Test',1895,'',10 from dual union all
select 26840,'1971-W350','1971-03-08 17:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26841,'1971-W351','1971-03-08 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',56 from dual union all
select 26842,'1971-W352','1971-03-08 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 26843,'1971-W353','1971-03-08 19:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 26844,'1971-W354','1971-03-08 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 26845,'1971-W355','1971-03-09 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 26846,'1971-W356','1971-03-09 02:44:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26847,'1971-W357','1971-03-09 05:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 26848,'1971-W358','1971-03-09 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 26849,'1971-W359','1971-03-10 02:06:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 2685,'1954-A650','1954-11-22 00:00:00','atmospheric rocket','unknown',576,'862','','','','Test',1895,'',10 from dual union all
select 26850,'1971-W36','1971-01-11 17:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26851,'1971-W360','1971-03-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 26852,'1971-W361','1971-03-10 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 26853,'1971-W362','1971-03-10 18:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 26854,'1971-W363','1971-03-11 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 26855,'1971-W364','1971-03-11 07:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 26856,'1971-W365','1971-03-11 17:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26857,'1971-W366','1971-03-12 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',61 from dual union all
select 26858,'1971-W367','1971-03-12 02:30:00','suborbital rocket','success',88,'','Met Para,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 26859,'1971-W368','1971-03-12 07:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 2686,'1954-A651','1954-11-22 00:00:00','atmospheric rocket','success',836,'3270T','','','','Test',1895,'',10 from dual union all
select 26860,'1971-W369','1971-03-12 13:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 26861,'1971-W37','1971-01-11 18:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 26862,'1971-W370','1971-03-12 15:54:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 26863,'1971-W371','1971-03-12 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 26864,'1971-W372','1971-03-13 01:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 26865,'1971-W373','1971-03-15 16:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 26866,'1971-W374','1971-03-15 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 26867,'1971-W375','1971-03-15 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 26868,'1971-W376','1971-03-15 20:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 26869,'1971-W377','1971-03-15 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 2687,'1954-A652','1954-11-23 00:00:00','atmospheric rocket','unknown',576,'456M','','','','Test',1895,'',10 from dual union all
select 26870,'1971-W378','1971-03-16 01:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 26871,'1971-W379','1971-03-16 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 26872,'1971-W38','1971-01-11 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 26873,'1971-W380','1971-03-16 18:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26874,'1971-W381','1971-03-16 21:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 26875,'1971-W382','1971-03-17 11:30:00','suborbital rocket','success',631,'INTA H-7109','','','','Meteo',396,'',66 from dual union all
select 26876,'1971-W383','1971-03-17 15:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',59 from dual union all
select 26877,'1971-W384','1971-03-17 16:23:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',63 from dual union all
select 26878,'1971-W385','1971-03-17 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 26879,'1971-W386','1971-03-17 19:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 2688,'1954-A653','1954-11-23 00:00:00','atmospheric rocket','unknown',576,'509M','','','','Test',1895,'',10 from dual union all
select 26880,'1971-W387','1971-03-17 22:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 26881,'1971-W388','1971-03-18 01:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 26882,'1971-W389','1971-03-18 13:34:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',65 from dual union all
select 26883,'1971-W39','1971-01-11 20:26:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 26884,'1971-W390','1971-03-18 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 26885,'1971-W391','1971-03-19 14:13:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 26886,'1971-W392','1971-03-19 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 26887,'1971-W393','1971-03-19 17:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26888,'1971-W394','1971-03-19 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 26889,'1971-W395','1971-03-20 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',57 from dual union all
select 2689,'1954-A654','1954-11-23 00:00:00','atmospheric rocket','success',836,'2816E','','','','Test',1895,'',10 from dual union all
select 26890,'1971-W396','1971-03-20 05:43:00','suborbital rocket','unknown',88,'[= Nike]','','','','Plasma/Fields',428,'',60 from dual union all
select 26891,'1971-W397','1971-03-20 05:46:00','suborbital rocket','unknown',88,'[= Nike]','','','','Plasma/Fields',428,'',60 from dual union all
select 26892,'1971-W398','1971-03-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 26893,'1971-W399','1971-03-22 15:19:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',65 from dual union all
select 26894,'1971-W40','1971-01-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 26895,'1971-W400','1971-03-22 18:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26896,'1971-W401','1971-03-22 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 26897,'1971-W402','1971-03-22 19:51:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 26898,'1971-W403','1971-03-23 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 26899,'1971-W404','1971-03-23 02:51:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 269,'1944-S244','1944-11-13 12:10:00','military missile','success',1470,'19513','Ma267','','','Test',661,'',90 from dual union all
select 2690,'1954-A655','1954-11-23 00:00:00','atmospheric rocket','success',836,'3536T','','','','Test',1895,'',10 from dual union all
select 26900,'1971-W405','1971-03-24 03:00:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 26901,'1971-W406','1971-03-24 12:00:00','suborbital rocket','success',631,'INTA H-7110','','','','Meteo',396,'',63 from dual union all
select 26902,'1971-W407','1971-03-24 14:53:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',62 from dual union all
select 26903,'1971-W408','1971-03-24 14:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',55 from dual union all
select 26904,'1971-W409','1971-03-24 16:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',61 from dual union all
select 26905,'1971-W41','1971-01-12 16:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 26906,'1971-W410','1971-03-24 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 26907,'1971-W411','1971-03-24 20:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',58 from dual union all
select 26908,'1971-W412','1971-03-24 20:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26909,'1971-W413','1971-03-24 21:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 2691,'1954-A656','1954-11-23 00:00:00','atmospheric rocket','success',836,'1242E','','','','Test',1895,'',10 from dual union all
select 26910,'1971-W414','1971-03-25 02:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 26911,'1971-W415','1971-03-25 18:00:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26912,'1971-W416','1971-03-25 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',54 from dual union all
select 26913,'1971-W417','1971-03-25 19:14:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',63 from dual union all
select 26914,'1971-W418','1971-03-25 19:15:00','suborbital rocket','unknown',88,'','','','','Aeron',1895,'',68 from dual union all
select 26915,'1971-W419','1971-03-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 26916,'1971-W42','1971-01-12 18:00:00','suborbital rocket','unknown',88,'','','','','Aeron',1895,'',60 from dual union all
select 26917,'1971-W420','1971-03-26 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 26918,'1971-W421','1971-03-26 18:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual;
insert into LAUNCH
select 26919,'1971-W422','1971-03-26 19:27:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 2692,'1954-A657','1954-11-24 00:00:00','military missile','success',255,'623-9','','','','Test',253,'',20 from dual union all
select 26920,'1971-W423','1971-03-26 20:30:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',58 from dual union all
select 26921,'1971-W424','1971-03-26 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 26922,'1971-W425','1971-03-27 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 26923,'1971-W426','1971-03-29 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 26924,'1971-W427','1971-03-29 17:57:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 26925,'1971-W428','1971-03-29 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 26926,'1971-W429','1971-03-29 21:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26927,'1971-W43','1971-01-12 18:18:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',59 from dual union all
select 26928,'1971-W430','1971-03-29 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 26929,'1971-W431','1971-03-30 01:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 2693,'1954-A658','1954-11-24 00:00:00','atmospheric rocket','unknown',576,'460M','','','','Test',1895,'',10 from dual union all
select 26930,'1971-W432','1971-03-30 18:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 26931,'1971-W433','1971-03-30 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26932,'1971-W434','1971-03-31 13:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',50 from dual union all
select 26933,'1971-W435','1971-03-31 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 26934,'1971-W436','1971-03-31 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 26935,'1971-W437','1971-03-31 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 26936,'1971-W438','1971-03-31 18:03:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 26937,'1971-W439','1971-03-31 18:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 26938,'1971-W44','1971-01-12 19:14:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',60 from dual union all
select 26939,'1971-W440','1971-03-31 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 2694,'1954-A659','1954-11-24 00:00:00','atmospheric rocket','success',836,'2999R','Nike 403R','','','Test',1895,'',10 from dual union all
select 26940,'1971-W441','1971-03-31 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 26941,'1971-W442','1971-04-01 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 26942,'1971-W443','1971-04-01 18:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 26943,'1971-W444','1971-04-01 21:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 26944,'1971-W445','1971-04-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 26945,'1971-W446','1971-04-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 26946,'1971-W447','1971-04-02 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 26947,'1971-W448','1971-04-02 18:03:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 26948,'1971-W449','1971-04-02 18:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 26949,'1971-W45','1971-01-12 20:52:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 2695,'1954-A66','1954-02-18 00:00:00','military missile','success',1214,'N-69A GM-3396','','','','Test',238,'',10 from dual union all
select 26950,'1971-W450','1971-04-02 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',64 from dual union all
select 26951,'1971-W451','1971-04-02 21:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 26952,'1971-W452','1971-04-02 23:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',54 from dual union all
select 26953,'1971-W453','1971-04-03 01:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 26954,'1971-W454','1971-04-03 12:45:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',63 from dual union all
select 26955,'1971-W455','1971-04-03 17:31:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',69 from dual union all
select 26956,'1971-W456','1971-04-03 17:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',53 from dual union all
select 26957,'1971-W457','1971-04-05 05:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 26958,'1971-W458','1971-04-05 09:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 26959,'1971-W459','1971-04-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 2696,'1954-A660','1954-11-24 00:00:00','atmospheric rocket','success',836,'2819R','Nike 404R','','','Test',1895,'',10 from dual union all
select 26960,'1971-W46','1971-01-12 21:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 26961,'1971-W460','1971-04-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 26962,'1971-W461','1971-04-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 26963,'1971-W462','1971-04-05 15:46:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 26964,'1971-W463','1971-04-05 17:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 26965,'1971-W464','1971-04-05 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 26966,'1971-W465','1971-04-05 19:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',52 from dual union all
select 26967,'1971-W466','1971-04-05 19:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 26968,'1971-W467','1971-04-05 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',57 from dual union all
select 26969,'1971-W468','1971-04-05 20:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 2697,'1954-A661','1954-11-24 00:00:00','atmospheric rocket','success',836,'3095R','Nike 401R','','','Test',1895,'',10 from dual union all
select 26970,'1971-W469','1971-04-06 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 26971,'1971-W47','1971-01-13 03:04:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',50 from dual union all
select 26972,'1971-W470','1971-04-06 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 26973,'1971-W471','1971-04-07 12:07:00','suborbital rocket','success',716,'INTA HT-7101','','','','Meteo',396,'',57 from dual union all
select 26974,'1971-W472','1971-04-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 26975,'1971-W473','1971-04-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 26976,'1971-W474','1971-04-07 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 26977,'1971-W475','1971-04-07 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 26978,'1971-W476','1971-04-07 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 26979,'1971-W477','1971-04-07 16:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 2698,'1954-A662','1954-11-24 00:00:00','atmospheric rocket','success',836,'3019R','Nike 402R','','','Test',1895,'',10 from dual union all
select 26980,'1971-W478','1971-04-07 17:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',71 from dual union all
select 26981,'1971-W479','1971-04-07 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 26982,'1971-W48','1971-01-13 04:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 26983,'1971-W480','1971-04-07 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 26984,'1971-W481','1971-04-07 20:15:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',70 from dual union all
select 26985,'1971-W482','1971-04-07 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 26986,'1971-W483','1971-04-08 03:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 26987,'1971-W484','1971-04-08 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 26988,'1971-W485','1971-04-08 17:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 26989,'1971-W486','1971-04-08 23:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 2699,'1954-A663','1954-11-29 00:00:00','atmospheric rocket','unknown',576,'484ES','','','','Test',1895,'',10 from dual union all
select 26990,'1971-W487','1971-04-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 26991,'1971-W488','1971-04-09 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 26992,'1971-W489','1971-04-09 15:11:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',59 from dual union all
select 26993,'1971-W49','1971-01-13 12:41:00','suborbital rocket','success',631,'INTA H-7101','','','','Meteo',396,'',62 from dual union all
select 26994,'1971-W490','1971-04-09 16:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 26995,'1971-W491','1971-04-09 16:35:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',53 from dual union all
select 26996,'1971-W492','1971-04-09 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 26997,'1971-W493','1971-04-09 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',57 from dual union all
select 26998,'1971-W494','1971-04-09 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 26999,'1971-W495','1971-04-10 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 27,'1943-S05','1943-05-26 11:00:00','military missile','success',8,'26','','','','Test',675,'',95 from dual union all
select 270,'1944-S245','1944-11-14 08:59:00','military missile','success',1470,'19525','Ma283','','','Test',661,'',90 from dual union all
select 2700,'1954-A664','1954-11-29 00:00:00','atmospheric rocket','unknown',576,'449ES','','','','Test',1895,'',10 from dual union all
select 27000,'1971-W496','1971-04-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 27001,'1971-W497','1971-04-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 27002,'1971-W498','1971-04-12 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 27003,'1971-W499','1971-04-12 17:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 27004,'1971-W50','1971-01-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 27005,'1971-W500','1971-04-12 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 27006,'1971-W501','1971-04-12 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',58 from dual union all
select 27007,'1971-W502','1971-04-12 20:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual;
insert into LAUNCH
select 27008,'1971-W503','1971-04-12 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 27009,'1971-W504','1971-04-13 02:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',57 from dual union all
select 2701,'1954-A665','1954-12-01 00:00:00','atmospheric rocket','unknown',576,'494M','','','','Test',1895,'',10 from dual union all
select 27010,'1971-W505','1971-04-13 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 27011,'1971-W506','1971-04-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',55 from dual union all
select 27012,'1971-W507','1971-04-14 15:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 27013,'1971-W508','1971-04-14 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 27014,'1971-W509','1971-04-14 16:13:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 27015,'1971-W51','1971-01-13 14:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 27016,'1971-W510','1971-04-14 16:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 27017,'1971-W511','1971-04-14 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',73 from dual union all
select 27018,'1971-W512','1971-04-14 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',57 from dual union all
select 27019,'1971-W513','1971-04-14 21:13:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',54 from dual union all
select 2702,'1954-A666','1954-12-01 00:00:00','atmospheric rocket','success',836,'3075R','Nike 405R','','','Test',1895,'',10 from dual union all
select 27020,'1971-W514','1971-04-14 21:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 27021,'1971-W515','1971-04-15 02:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 27022,'1971-W516','1971-04-15 20:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 27023,'1971-W517','1971-04-15 23:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 27024,'1971-W518','1971-04-16 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 27025,'1971-W519','1971-04-16 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 27026,'1971-W52','1971-01-13 15:24:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',64 from dual union all
select 27027,'1971-W520','1971-04-16 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 27028,'1971-W521','1971-04-16 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 27029,'1971-W522','1971-04-16 15:45:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 2703,'1954-A667','1954-12-01 00:00:00','atmospheric rocket','success',836,'2525R','Nike 408R','','','Test',1895,'',10 from dual union all
select 27030,'1971-W523','1971-04-16 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',73 from dual union all
select 27031,'1971-W524','1971-04-16 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 27032,'1971-W525','1971-04-16 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 27033,'1971-W526','1971-04-17 00:34:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',59 from dual union all
select 27034,'1971-W527','1971-04-17 06:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 27035,'1971-W528','1971-04-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 27036,'1971-W529','1971-04-19 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 27037,'1971-W53','1971-01-13 15:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',51 from dual union all
select 27038,'1971-W530','1971-04-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 27039,'1971-W531','1971-04-19 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 2704,'1954-A668','1954-12-01 00:00:00','atmospheric rocket','success',836,'3089R','Nike 406R','','','Test',1895,'',10 from dual union all
select 27040,'1971-W532','1971-04-19 17:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',72 from dual union all
select 27041,'1971-W533','1971-04-19 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',56 from dual union all
select 27042,'1971-W534','1971-04-19 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 27043,'1971-W535','1971-04-19 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 27044,'1971-W536','1971-04-19 20:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 27045,'1971-W537','1971-04-19 21:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 27046,'1971-W538','1971-04-19 22:23:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',855,'',63 from dual union all
select 27047,'1971-W539','1971-04-20 01:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 27048,'1971-W54','1971-01-13 16:19:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 27049,'1971-W540','1971-04-20 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 2705,'1954-A669','1954-12-01 00:00:00','atmospheric rocket','success',836,'2794A','','','','Test',1895,'',10 from dual union all
select 27050,'1971-W541','1971-04-20 18:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 27051,'1971-W542','1971-04-20 22:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 27052,'1971-W543','1971-04-21 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 27053,'1971-W544','1971-04-21 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 27054,'1971-W545','1971-04-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 27055,'1971-W546','1971-04-21 15:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 27056,'1971-W547','1971-04-21 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',65 from dual union all
select 27057,'1971-W548','1971-04-21 17:57:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 27058,'1971-W549','1971-04-21 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',51 from dual union all
select 27059,'1971-W55','1971-01-13 18:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',60 from dual union all
select 2706,'1954-A67','1954-02-19 00:00:00','atmospheric rocket','unknown',576,'434ST','','','','Test',1895,'',10 from dual union all
select 27060,'1971-W550','1971-04-21 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 27061,'1971-W551','1971-04-21 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 27062,'1971-W552','1971-04-21 18:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 27063,'1971-W553','1971-04-21 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 27064,'1971-W554','1971-04-21 22:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 27065,'1971-W555','1971-04-22 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 27066,'1971-W556','1971-04-22 15:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 27067,'1971-W557','1971-04-23 09:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 27068,'1971-W558','1971-04-23 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 27069,'1971-W559','1971-04-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 2707,'1954-A670','1954-12-02 00:00:00','atmospheric rocket','success',836,'2774A','','','','Test',1895,'',10 from dual union all
select 27070,'1971-W56','1971-01-13 18:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',52 from dual union all
select 27071,'1971-W560','1971-04-23 15:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',56 from dual union all
select 27072,'1971-W561','1971-04-23 18:09:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 27073,'1971-W562','1971-04-23 18:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 27074,'1971-W563','1971-04-23 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',51 from dual union all
select 27075,'1971-W564','1971-04-23 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 27076,'1971-W565','1971-04-24 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 27077,'1971-W566','1971-04-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 27078,'1971-W567','1971-04-26 15:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 27079,'1971-W568','1971-04-26 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 2708,'1954-A671','1954-12-02 00:00:00','atmospheric rocket','success',836,'3321T','','','','Test',1895,'',10 from dual union all
select 27080,'1971-W569','1971-04-26 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 27081,'1971-W57','1971-01-13 19:36:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 27082,'1971-W570','1971-04-26 17:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 27083,'1971-W571','1971-04-26 20:59:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 27084,'1971-W572','1971-04-26 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 27085,'1971-W573','1971-04-27 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 27086,'1971-W574','1971-04-27 14:15:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',56 from dual union all
select 27087,'1971-W575','1971-04-27 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 27088,'1971-W576','1971-04-27 14:24:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',63 from dual union all
select 27089,'1971-W577','1971-04-27 15:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 2709,'1954-A672','1954-12-03 00:00:00','atmospheric rocket','success',836,'3314T','','','','Test',1895,'',10 from dual union all
select 27090,'1971-W578','1971-04-27 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 27091,'1971-W579','1971-04-27 16:33:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',53 from dual union all
select 27092,'1971-W58','1971-01-13 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 27093,'1971-W580','1971-04-27 16:44:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',64 from dual union all
select 27094,'1971-W581','1971-04-27 18:40:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',60 from dual union all
select 27095,'1971-W582','1971-04-27 19:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 27096,'1971-W583','1971-04-27 19:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 27097,'1971-W584','1971-04-27 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 27098,'1971-W585','1971-04-27 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual;
insert into LAUNCH
select 27099,'1971-W586','1971-04-28 11:35:00','suborbital rocket','success',716,'INTA HT-7103','','','','Meteo',396,'',58 from dual union all
select 271,'1944-S246','1944-11-14 11:09:00','military missile','success',1470,'19517','Ma279','','','Test',661,'',90 from dual union all
select 2710,'1954-A673','1954-12-06 00:00:00','atmospheric rocket','unknown',576,'792PO','','','','Test',1895,'',10 from dual union all
select 27100,'1971-W587','1971-04-28 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 27101,'1971-W588','1971-04-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',52 from dual union all
select 27102,'1971-W589','1971-04-28 15:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 27103,'1971-W59','1971-01-14 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 27104,'1971-W590','1971-04-28 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 27105,'1971-W591','1971-04-28 15:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 27106,'1971-W592','1971-04-28 17:17:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 27107,'1971-W593','1971-04-28 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',56 from dual union all
select 27108,'1971-W594','1971-04-28 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 27109,'1971-W595','1971-04-29 01:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 2711,'1954-A674','1954-12-06 00:00:00','atmospheric rocket','unknown',576,'784PO','','','','Test',1895,'',10 from dual union all
select 27110,'1971-W596','1971-04-29 14:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 27111,'1971-W597','1971-04-29 17:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 27112,'1971-W598','1971-04-29 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 27113,'1971-W599','1971-05-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 27114,'1971-W60','1971-01-14 02:52:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',63 from dual union all
select 27115,'1971-W600','1971-04-30 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 27116,'1971-W601','1971-04-30 14:45:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 27117,'1971-W602','1971-04-30 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 27118,'1971-W603','1971-04-30 15:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 27119,'1971-W604','1971-04-30 18:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 2712,'1954-A675','1954-12-06 00:00:00','atmospheric rocket','success',836,'2574E','','','','Test',1895,'',10 from dual union all
select 27120,'1971-W605','1971-04-30 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 27121,'1971-W606','1971-05-01 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 27122,'1971-W607','1971-05-01 03:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 27123,'1971-W608','1971-05-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 27124,'1971-W609','1971-05-03 14:59:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 27125,'1971-W61','1971-01-14 15:41:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 27126,'1971-W610','1971-05-03 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 27127,'1971-W611','1971-05-03 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 27128,'1971-W612','1971-05-03 18:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 27129,'1971-W613','1971-05-03 18:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 2713,'1954-A676','1954-12-06 00:00:00','atmospheric rocket','success',836,'1240E','','','','Test',1895,'',10 from dual union all
select 27130,'1971-W614','1971-05-03 20:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',57 from dual union all
select 27131,'1971-W615','1971-05-03 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',53 from dual union all
select 27132,'1971-W616','1971-05-04 02:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 27133,'1971-W617','1971-05-04 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 27134,'1971-W618','1971-05-04 20:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 27135,'1971-W619','1971-05-05 11:31:00','suborbital rocket','success',716,'INTA HT-7104','','','','Meteo',396,'',57 from dual union all
select 27136,'1971-W62','1971-01-14 16:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',60 from dual union all
select 27137,'1971-W620','1971-05-05 14:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 27138,'1971-W621','1971-05-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 27139,'1971-W622','1971-05-05 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 2714,'1954-A677','1954-12-06 00:00:00','atmospheric rocket','success',836,'2820A','','','','Test',1895,'',10 from dual union all
select 27140,'1971-W623','1971-05-05 17:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 27141,'1971-W624','1971-05-05 18:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 27142,'1971-W625','1971-05-05 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 27143,'1971-W626','1971-05-05 21:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 27144,'1971-W627','1971-05-06 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 27145,'1971-W628','1971-05-06 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 27146,'1971-W629','1971-05-06 18:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 27147,'1971-W63','1971-01-14 17:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 27148,'1971-W630','1971-05-06 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 27149,'1971-W631','1971-05-07 00:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',60 from dual union all
select 2715,'1954-A678','1954-12-06 00:00:00','atmospheric rocket','success',836,'3518T','','','','Test',1895,'',10 from dual union all
select 27150,'1971-W632','1971-05-07 11:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 27151,'1971-W633','1971-05-07 14:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 27152,'1971-W634','1971-05-07 14:55:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 27153,'1971-W635','1971-05-07 17:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 27154,'1971-W636','1971-05-07 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 27155,'1971-W637','1971-05-07 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 27156,'1971-W638','1971-05-08 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 27157,'1971-W639','1971-05-10 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 27158,'1971-W64','1971-01-14 17:49:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',55 from dual union all
select 27159,'1971-W640','1971-05-10 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 2716,'1954-A679','1954-12-07 00:00:00','atmospheric rocket','success',836,'3317T','','','','Test',1895,'',10 from dual union all
select 27160,'1971-W641','1971-05-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 27161,'1971-W642','1971-05-10 14:56:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 27162,'1971-W643','1971-05-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 27163,'1971-W644','1971-05-10 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 27164,'1971-W645','1971-05-10 17:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',55 from dual union all
select 27165,'1971-W646','1971-05-10 18:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 27166,'1971-W647','1971-05-10 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 27167,'1971-W648','1971-05-10 23:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',53 from dual union all
select 27168,'1971-W649','1971-05-11 20:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 27169,'1971-W65','1971-01-14 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 2717,'1954-A68','1954-02-19 00:00:00','atmospheric rocket','unknown',576,'414ST','','','','Test',1895,'',10 from dual union all
select 27170,'1971-W650','1971-05-11 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',54 from dual union all
select 27171,'1971-W651','1971-05-12 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 27172,'1971-W652','1971-05-12 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 27173,'1971-W653','1971-05-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 27174,'1971-W654','1971-05-12 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 27175,'1971-W655','1971-05-12 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 27176,'1971-W656','1971-05-12 14:57:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 27177,'1971-W657','1971-05-12 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 27178,'1971-W658','1971-05-12 16:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 27179,'1971-W659','1971-05-12 16:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',71 from dual union all
select 2718,'1954-A680','1954-12-08 00:00:00','atmospheric rocket','unknown',576,'117AEM','','','','Test',1895,'',10 from dual union all
select 27180,'1971-W66','1971-01-14 22:17:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',50 from dual union all
select 27181,'1971-W660','1971-05-12 17:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 27182,'1971-W661','1971-05-12 18:11:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',68 from dual union all
select 27183,'1971-W662','1971-05-12 19:21:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 27184,'1971-W663','1971-05-12 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',62 from dual union all
select 27185,'1971-W664','1971-05-13 00:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 27186,'1971-W665','1971-05-13 01:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 27187,'1971-W666','1971-05-13 16:42:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 27188,'1971-W667','1971-05-13 17:40:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',60 from dual;
insert into LAUNCH
select 27189,'1971-W668','1971-05-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 2719,'1954-A681','1954-12-08 00:00:00','atmospheric rocket','success',836,'3315T','','','','Test',1895,'',10 from dual union all
select 27190,'1971-W669','1971-05-14 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 27191,'1971-W67','1971-01-14 22:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',51 from dual union all
select 27192,'1971-W670','1971-05-14 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 27193,'1971-W671','1971-05-14 15:43:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 27194,'1971-W672','1971-05-14 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 27195,'1971-W673','1971-05-14 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',62 from dual union all
select 27196,'1971-W674','1971-05-14 20:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 27197,'1971-W675','1971-05-14 22:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',53 from dual union all
select 27198,'1971-W676','1971-05-15 00:45:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',65 from dual union all
select 27199,'1971-W677','1971-05-15 01:42:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 272,'1944-S247','1944-11-15 10:35:00','military missile','success',1470,'19549','Ma288','','','Test',661,'',90 from dual union all
select 2720,'1954-A682','1954-12-08 00:00:00','atmospheric rocket','success',836,'2959R','Nike 409R','','','Test',1895,'',10 from dual union all
select 27200,'1971-W678','1971-05-15 04:41:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',61 from dual union all
select 27201,'1971-W679','1971-05-16 17:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',54 from dual union all
select 27202,'1971-W68','1971-01-15 03:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 27203,'1971-W680','1971-05-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 27204,'1971-W681','1971-05-17 14:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 27205,'1971-W682','1971-05-17 14:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 27206,'1971-W683','1971-05-17 14:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 27207,'1971-W684','1971-05-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 27208,'1971-W685','1971-05-17 15:49:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',61 from dual union all
select 27209,'1971-W686','1971-05-17 16:54:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 2721,'1954-A683','1954-12-08 00:00:00','atmospheric rocket','success',836,'3099R','Nike 410R','','','Test',1895,'',10 from dual union all
select 27210,'1971-W687','1971-05-17 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',51 from dual union all
select 27211,'1971-W688','1971-05-17 18:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 27212,'1971-W689','1971-05-17 20:48:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',78 from dual union all
select 27213,'1971-W69','1971-01-15 04:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 27214,'1971-W690','1971-05-17 22:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 27215,'1971-W691','1971-05-17 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',64 from dual union all
select 27216,'1971-W692','1971-05-18 14:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 27217,'1971-W693','1971-05-18 21:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 27218,'1971-W694','1971-05-19 11:15:00','suborbital rocket','success',716,'INTA HT-7106','','','','Meteo',396,'',59 from dual union all
select 27219,'1971-W695','1971-05-19 13:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 2722,'1954-A684','1954-12-09 00:00:00','atmospheric rocket','success',836,'2772T','','','','Test',1895,'',10 from dual union all
select 27220,'1971-W696','1971-05-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 27221,'1971-W697','1971-05-19 14:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 27222,'1971-W698','1971-05-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 27223,'1971-W699','1971-05-19 15:09:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 27224,'1971-W70','1971-01-15 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 27225,'1971-W700','1971-05-19 16:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',55 from dual union all
select 27226,'1971-W701','1971-05-19 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 27227,'1971-W702','1971-05-19 18:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',58 from dual union all
select 27228,'1971-W703','1971-05-19 18:04:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',63 from dual union all
select 27229,'1971-W704','1971-05-19 20:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',58 from dual union all
select 2723,'1954-A685','1954-12-10 00:00:00','atmospheric rocket','success',836,'3333T','','','','Test',1895,'',10 from dual union all
select 27230,'1971-W705','1971-05-19 20:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 27231,'1971-W706','1971-05-19 21:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 27232,'1971-W707','1971-05-20 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 27233,'1971-W708','1971-05-20 15:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 27234,'1971-W709','1971-05-21 13:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 27235,'1971-W71','1971-01-15 15:25:00','suborbital rocket','success',1189,'Sk83/2','ESRO Sk83/2','','','Aeronomy',417,'',69 from dual union all
select 27236,'1971-W710','1971-05-21 13:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 27237,'1971-W711','1971-05-21 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 27238,'1971-W712','1971-05-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 27239,'1971-W713','1971-05-21 17:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',66 from dual union all
select 2724,'1954-A686','1954-12-10 00:00:00','atmospheric rocket','success',836,'3324T','','','','Test',1895,'',10 from dual union all
select 27240,'1971-W714','1971-05-21 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 27241,'1971-W715','1971-05-21 19:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 27242,'1971-W716','1971-05-21 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',53 from dual union all
select 27243,'1971-W717','1971-05-21 21:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 27244,'1971-W718','1971-05-22 01:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 27245,'1971-W719','1971-05-22 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 27246,'1971-W72','1971-01-15 16:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',52 from dual union all
select 27247,'1971-W720','1971-05-23 17:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 27248,'1971-W721','1971-05-24 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 27249,'1971-W722','1971-05-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 2725,'1954-A687','1954-12-10 00:00:00','atmospheric rocket','success',836,'2810E','','','','Test',1895,'',10 from dual union all
select 27250,'1971-W723','1971-05-24 14:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 27251,'1971-W724','1971-05-24 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 27252,'1971-W725','1971-05-24 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 27253,'1971-W726','1971-05-24 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 27254,'1971-W727','1971-05-24 22:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',56 from dual union all
select 27255,'1971-W728','1971-05-25 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 27256,'1971-W729','1971-05-25 19:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 27257,'1971-W73','1971-01-15 16:01:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 27258,'1971-W730','1971-05-25 21:19:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',855,'',54 from dual union all
select 27259,'1971-W731','1971-05-26 13:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',60 from dual union all
select 2726,'1954-A688','1954-12-10 00:00:00','atmospheric rocket','success',836,'2818E','','','','Test',1895,'',10 from dual union all
select 27260,'1971-W732','1971-05-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 27261,'1971-W733','1971-05-26 14:46:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 27262,'1971-W734','1971-05-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 27263,'1971-W735','1971-05-26 15:14:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',65 from dual union all
select 27264,'1971-W736','1971-05-26 16:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 27265,'1971-W737','1971-05-26 17:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 27266,'1971-W738','1971-05-26 18:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',55 from dual union all
select 27267,'1971-W739','1971-05-26 19:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 27268,'1971-W74','1971-01-15 16:24:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',72 from dual union all
select 27269,'1971-W740','1971-05-26 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',56 from dual union all
select 2727,'1954-A689','1954-12-10 00:00:00','military missile','success',1214,'N-69B GM-11115','','','','Test',222,'',10 from dual union all
select 27270,'1971-W741','1971-05-26 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 27271,'1971-W742','1971-05-26 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',53 from dual union all
select 27272,'1971-W743','1971-05-27 02:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 27273,'1971-W744','1971-05-27 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 27274,'1971-W745','1971-05-27 14:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 27275,'1971-W746','1971-05-27 19:10:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',59 from dual union all
select 27276,'1971-W747','1971-05-27 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 27277,'1971-W748','1971-05-28 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 27278,'1971-W749','1971-05-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',55 from dual;
insert into LAUNCH
select 27279,'1971-W75','1971-01-15 20:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 2728,'1954-A69','1954-02-19 00:00:00','military missile','success',771,'GM-12424','','','','Test',222,'',12 from dual union all
select 27280,'1971-W750','1971-05-28 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 27281,'1971-W751','1971-05-28 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 27282,'1971-W752','1971-05-28 17:09:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 27283,'1971-W753','1971-05-28 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',66 from dual union all
select 27284,'1971-W754','1971-05-28 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 27285,'1971-W755','1971-05-29 03:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 27286,'1971-W756','1971-06-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',64 from dual union all
select 27287,'1971-W757','1971-05-31 18:36:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 27288,'1971-W758','1971-06-01 14:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 27289,'1971-W759','1971-06-01 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 2729,'1954-A690','1954-12-11 00:00:00','atmospheric rocket','success',836,'1498T','','','','Test',1895,'',10 from dual union all
select 27290,'1971-W76','1971-01-15 21:22:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 27291,'1971-W760','1971-06-01 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 27292,'1971-W761','1971-06-01 16:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',68 from dual union all
select 27293,'1971-W762','1971-06-01 17:20:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',62 from dual union all
select 27294,'1971-W763','1971-06-01 17:56:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',66 from dual union all
select 27295,'1971-W764','1971-06-01 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 27296,'1971-W765','1971-06-02 13:34:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',56 from dual union all
select 27297,'1971-W766','1971-06-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 27298,'1971-W767','1971-06-02 15:09:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 27299,'1971-W768','1971-06-02 15:14:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',63 from dual union all
select 273,'1944-S248','1944-11-15 12:02:00','military missile','success',1470,'19548','Ma289','','','Test',661,'',90 from dual union all
select 2730,'1954-A691','1954-12-12 00:00:00','atmospheric rocket','success',836,'1505T','','','','Test',1895,'',10 from dual union all
select 27300,'1971-W769','1971-06-02 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 27301,'1971-W77','1971-01-16 01:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 27302,'1971-W770','1971-06-02 17:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 27303,'1971-W771','1971-06-02 17:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',60 from dual union all
select 27304,'1971-W772','1971-06-02 18:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 27305,'1971-W773','1971-06-02 21:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 27306,'1971-W774','1971-06-03 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 27307,'1971-W775','1971-06-03 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 27308,'1971-W776','1971-06-03 21:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 27309,'1971-W777','1971-06-04 01:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',56 from dual union all
select 2731,'1954-A692','1954-12-12 00:00:00','atmospheric rocket','success',836,'3272T','','','','Test',1895,'',10 from dual union all
select 27310,'1971-W778','1971-06-04 06:12:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',61 from dual union all
select 27311,'1971-W779','1971-06-04 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 27312,'1971-W78','1971-01-16 01:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 27313,'1971-W780','1971-06-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 27314,'1971-W781','1971-06-04 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 27315,'1971-W782','1971-06-04 20:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',63 from dual union all
select 27316,'1971-W783','1971-06-04 21:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',64 from dual union all
select 27317,'1971-W784','1971-06-04 23:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 27318,'1971-W785','1971-06-07 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 27319,'1971-W786','1971-06-07 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 2732,'1954-A693','1954-12-13 00:00:00','military missile','unknown',339,'1554R','','','','Test',1895,'',40 from dual union all
select 27320,'1971-W787','1971-06-07 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 27321,'1971-W788','1971-06-07 17:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 27322,'1971-W789','1971-06-07 18:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 27323,'1971-W79','1971-01-16 15:04:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',57 from dual union all
select 27324,'1971-W790','1971-06-07 21:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 27325,'1971-W791','1971-06-07 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',63 from dual union all
select 27326,'1971-W792','1971-06-08 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 27327,'1971-W793','1971-06-08 15:01:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',50 from dual union all
select 27328,'1971-W794','1971-06-08 17:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',61 from dual union all
select 27329,'1971-W795','1971-06-08 21:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 2733,'1954-A694','1954-12-13 00:00:00','atmospheric rocket','unknown',576,'545ES','','','','Test',1895,'',10 from dual union all
select 27330,'1971-W796','1971-06-09 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 27331,'1971-W797','1971-06-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 27332,'1971-W798','1971-06-09 14:20:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',59 from dual union all
select 27333,'1971-W799','1971-06-09 14:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 27334,'1971-W80','1971-01-17 15:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',62 from dual union all
select 27335,'1971-W800','1971-06-09 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 27336,'1971-W801','1971-06-09 15:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 27337,'1971-W802','1971-06-09 15:09:00','suborbital rocket','success',88,'WSMR/PSU','Conductivity','','','Ionos',1895,'',60 from dual union all
select 27338,'1971-W803','1971-06-09 15:13:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 27339,'1971-W804','1971-06-09 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 2734,'1954-A695','1954-12-13 00:00:00','atmospheric rocket','unknown',576,'556ES','','','','Test',1895,'',10 from dual union all
select 27340,'1971-W805','1971-06-09 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 27341,'1971-W806','1971-06-09 19:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 27342,'1971-W807','1971-06-09 20:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 27343,'1971-W808','1971-06-09 22:11:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',75 from dual union all
select 27344,'1971-W809','1971-06-09 22:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 27345,'1971-W81','1971-01-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 27346,'1971-W810','1971-06-10 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 27347,'1971-W811','1971-06-10 15:14:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',50 from dual union all
select 27348,'1971-W812','1971-06-11 02:46:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',615,'',64 from dual union all
select 27349,'1971-W813','1971-06-11 13:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 2735,'1954-A696','1954-12-13 00:00:00','atmospheric rocket','success',836,'1513T','','','','Test',1895,'',10 from dual union all
select 27350,'1971-W814','1971-06-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 27351,'1971-W815','1971-06-11 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 27352,'1971-W816','1971-06-11 17:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 27353,'1971-W817','1971-06-11 17:37:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 27354,'1971-W818','1971-06-11 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 27355,'1971-W819','1971-06-11 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',62 from dual union all
select 27356,'1971-W82','1971-01-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 27357,'1971-W820','1971-06-11 20:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 27358,'1971-W821','1971-06-11 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 27359,'1971-W822','1971-06-12 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 2736,'1954-A697','1954-12-13 00:00:00','atmospheric rocket','success',836,'1497T','','','','Test',1895,'',10 from dual union all
select 27360,'1971-W823','1971-06-14 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 27361,'1971-W824','1971-06-14 17:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',60 from dual union all
select 27362,'1971-W825','1971-06-14 17:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 27363,'1971-W826','1971-06-14 17:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 27364,'1971-W827','1971-06-14 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',65 from dual union all
select 27365,'1971-W828','1971-06-15 02:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 27366,'1971-W829','1971-06-15 15:25:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',50 from dual union all
select 27367,'1971-W83','1971-01-18 17:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 27368,'1971-W830','1971-06-15 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual;
insert into LAUNCH
select 27369,'1971-W831','1971-06-16 11:42:00','suborbital rocket','success',716,'INTA HT-7108','','','','Meteo',396,'',57 from dual union all
select 2737,'1954-A698','1954-12-14 00:00:00','atmospheric rocket','unknown',576,'M-62A-RM','','','','Test',1895,'',10 from dual union all
select 27370,'1971-W832','1971-06-16 14:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 27371,'1971-W833','1971-06-16 15:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 27372,'1971-W834','1971-06-16 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 27373,'1971-W835','1971-06-16 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 27374,'1971-W836','1971-06-16 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 27375,'1971-W837','1971-06-17 02:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 27376,'1971-W838','1971-06-17 17:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',51 from dual union all
select 27377,'1971-W839','1971-06-18 02:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 27378,'1971-W84','1971-01-18 18:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',52 from dual union all
select 27379,'1971-W840','1971-06-18 13:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 2738,'1954-A699','1954-12-14 00:00:00','atmospheric rocket','unknown',576,'M-64A-RM','','','','Test',1895,'',10 from dual union all
select 27380,'1971-W841','1971-06-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 27381,'1971-W842','1971-06-20 15:27:00','suborbital rocket','success',88,'','PAET Arcasonde','','','Aeron',194,'',55 from dual union all
select 27382,'1971-W843','1971-06-20 21:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',61 from dual union all
select 27383,'1971-W844','1971-06-20 22:59:00','suborbital rocket','success',88,'','PAET Arcasonde','','','Aeron',194,'',55 from dual union all
select 27384,'1971-W845','1971-06-21 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 27385,'1971-W846','1971-06-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 27386,'1971-W847','1971-06-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 27387,'1971-W848','1971-06-21 16:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',59 from dual union all
select 27388,'1971-W849','1971-06-21 17:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',61 from dual union all
select 27389,'1971-W85','1971-01-18 18:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 2739,'1954-A70','1954-02-20 00:00:00','suborbital rocket','failure',1490,'NA15','','','','Test',666,'',29 from dual union all
select 27390,'1971-W850','1971-06-21 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 27391,'1971-W851','1971-06-22 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 27392,'1971-W852','1971-06-22 15:57:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',51 from dual union all
select 27393,'1971-W853','1971-06-23 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 27394,'1971-W854','1971-06-23 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 27395,'1971-W855','1971-06-23 16:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 27396,'1971-W856','1971-06-23 17:22:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 27397,'1971-W857','1971-06-23 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 27398,'1971-W858','1971-06-23 19:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',50 from dual union all
select 27399,'1971-W859','1971-06-24 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 274,'1944-S249','1944-11-18 19:00:00','military missile','success',1470,'19544','Ma290','','','Test',661,'',90 from dual union all
select 2740,'1954-A700','1954-12-14 00:00:00','atmospheric rocket','success',836,'1514T','','','','Test',1895,'',10 from dual union all
select 27400,'1971-W86','1971-01-18 19:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 27401,'1971-W860','1971-06-24 01:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 27402,'1971-W861','1971-06-24 15:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',63 from dual union all
select 27403,'1971-W862','1971-06-24 15:56:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 27404,'1971-W863','1971-06-24 17:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 27405,'1971-W864','1971-06-24 17:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 27406,'1971-W865','1971-06-24 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 27407,'1971-W866','1971-06-24 22:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 27408,'1971-W867','1971-06-25 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 27409,'1971-W868','1971-06-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 2741,'1954-A701','1954-12-14 00:00:00','atmospheric rocket','success',836,'1516T','','','','Test',1895,'',10 from dual union all
select 27410,'1971-W869','1971-06-25 17:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',62 from dual union all
select 27411,'1971-W87','1971-01-18 19:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 27412,'1971-W870','1971-06-25 18:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',53 from dual union all
select 27413,'1971-W871','1971-06-25 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 27414,'1971-W872','1971-06-26 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 27415,'1971-W873','1971-06-28 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 27416,'1971-W874','1971-06-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 27417,'1971-W875','1971-06-28 15:45:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',60 from dual union all
select 27418,'1971-W876','1971-06-28 19:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 27419,'1971-W877','1971-06-28 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 2742,'1954-A702','1954-12-15 00:00:00','military missile','unknown',339,'U-1 1462','','','','Test',1895,'',40 from dual union all
select 27420,'1971-W878','1971-06-29 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 27421,'1971-W879','1971-06-29 06:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 27422,'1971-W88','1971-01-19 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 27423,'1971-W880','1971-06-29 15:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 27424,'1971-W881','1971-06-29 17:45:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',52 from dual union all
select 27425,'1971-W882','1971-06-29 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 27426,'1971-W883','1971-06-30 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 27427,'1971-W884','1971-06-30 14:48:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 27428,'1971-W885','1971-06-30 15:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',52 from dual union all
select 27429,'1971-W886','1971-06-30 15:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 2743,'1954-A703','1954-12-15 00:00:00','atmospheric rocket','success',836,'1249B-3897','Nike 411R','','','Test',1895,'',10 from dual union all
select 27430,'1971-W887','1971-06-30 15:59:59','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',68 from dual union all
select 27431,'1971-W888','1971-06-30 16:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',55 from dual union all
select 27432,'1971-W889','1971-06-30 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 27433,'1971-W89','1971-01-19 01:07:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 27434,'1971-W890','1971-06-30 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 27435,'1971-W891','1971-06-30 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 27436,'1971-W892','1971-06-30 17:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',54 from dual union all
select 27437,'1971-W893','1971-06-30 21:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 27438,'1971-W894','1971-06-30 22:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 27439,'1971-W895','1971-07-01 16:04:59','suborbital rocket','success',716,'[MC-71-1]','Starute,Datasonde','','','Meteo',308,'',55 from dual union all
select 2744,'1954-A704','1954-12-15 00:00:00','atmospheric rocket','success',836,'3031R','Nike 412R','','','Test',1895,'',10 from dual union all
select 27440,'1971-W896','1971-07-01 21:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 27441,'1971-W897','1971-07-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 27442,'1971-W898','1971-07-02 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 27443,'1971-W899','1971-07-02 15:45:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 27444,'1971-W90','1971-01-19 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 27445,'1971-W900','1971-07-02 17:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',61 from dual union all
select 27446,'1971-W901','1971-07-02 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 27447,'1971-W902','1971-07-02 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 27448,'1971-W903','1971-07-02 20:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 27449,'1971-W904','1971-07-02 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 2745,'1954-A705','1954-12-15 00:00:00','atmospheric rocket','success',836,'2792E','','','','Test',1895,'',10 from dual union all
select 27450,'1971-W905','1971-07-06 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 27451,'1971-W906','1971-07-06 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 27452,'1971-W907','1971-07-07 13:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 27453,'1971-W908','1971-07-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',68 from dual union all
select 27454,'1971-W909','1971-07-07 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 27455,'1971-W91','1971-01-19 04:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 27456,'1971-W910','1971-07-07 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 27457,'1971-W911','1971-07-07 15:20:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 27458,'1971-W912','1971-07-07 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual;
insert into LAUNCH
select 27459,'1971-W913','1971-07-07 16:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 2746,'1954-A706','1954-12-15 00:00:00','atmospheric rocket','success',836,'2804E','','','','Test',1895,'',10 from dual union all
select 27460,'1971-W914','1971-07-07 19:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',53 from dual union all
select 27461,'1971-W915','1971-07-08 17:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 27462,'1971-W916','1971-07-08 21:01:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',75 from dual union all
select 27463,'1971-W917','1971-07-08 21:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 27464,'1971-W918','1971-07-09 13:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 27465,'1971-W919','1971-07-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 27466,'1971-W92','1971-01-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 27467,'1971-W920','1971-07-09 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 27468,'1971-W921','1971-07-09 16:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',62 from dual union all
select 27469,'1971-W922','1971-07-09 19:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 2747,'1954-A707','1954-12-16 00:00:00','atmospheric rocket','success',442,'','Sidewinder','','','Test',1821,'',10 from dual union all
select 27470,'1971-W923','1971-07-09 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 27471,'1971-W924','1971-07-12 13:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 27472,'1971-W925','1971-07-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 27473,'1971-W926','1971-07-12 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',55 from dual union all
select 27474,'1971-W927','1971-07-12 15:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',54 from dual union all
select 27475,'1971-W928','1971-07-12 15:34:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 27476,'1971-W929','1971-07-12 16:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',59 from dual union all
select 27477,'1971-W93','1971-01-19 15:20:00','suborbital rocket','success',1188,'Sk83/4','ESRO Sk83/4','','','Aeronomy',417,'',71 from dual union all
select 27478,'1971-W930','1971-07-12 16:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 27479,'1971-W931','1971-07-12 17:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 2748,'1954-A708','1954-12-16 00:00:00','military missile','success',771,'GM-12430','','','','Test',222,'',12 from dual union all
select 27480,'1971-W932','1971-07-12 21:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 27481,'1971-W933','1971-07-13 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 27482,'1971-W934','1971-07-14 13:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 27483,'1971-W935','1971-07-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 27484,'1971-W936','1971-07-14 14:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 27485,'1971-W937','1971-07-14 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 27486,'1971-W938','1971-07-14 15:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 27487,'1971-W939','1971-07-14 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 27488,'1971-W94','1971-01-19 17:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 27489,'1971-W940','1971-07-14 17:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 2749,'1954-A709','1954-12-17 00:00:00','military missile','unknown',339,'1555R','','','','Test',1895,'',40 from dual union all
select 27490,'1971-W941','1971-07-14 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 27491,'1971-W942','1971-07-15 01:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 27492,'1971-W943','1971-07-15 16:14:59','suborbital rocket','success',716,'[MC-71-2]','Starute,Datasonde','','','Meteo',308,'',61 from dual union all
select 27493,'1971-W944','1971-07-15 17:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',51 from dual union all
select 27494,'1971-W945','1971-07-15 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 27495,'1971-W946','1971-07-16 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 27496,'1971-W947','1971-07-16 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 27497,'1971-W948','1971-07-16 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 27498,'1971-W949','1971-07-16 17:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 27499,'1971-W95','1971-01-19 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 275,'1944-S25','1944-05-20 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 2750,'1954-A71','1954-02-23 00:00:00','atmospheric rocket','unknown',576,'423ST','','','','Test',1895,'',10 from dual union all
select 27500,'1971-W950','1971-07-16 19:30:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',59 from dual union all
select 27501,'1971-W951','1971-07-16 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 27502,'1971-W952','1971-07-17 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 27503,'1971-W953','1971-07-18 01:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 27504,'1971-W954','1971-07-18 02:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 27505,'1971-W955','1971-07-19 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 27506,'1971-W956','1971-07-19 14:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 27507,'1971-W957','1971-07-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 27508,'1971-W958','1971-07-19 15:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',59 from dual union all
select 27509,'1971-W959','1971-07-19 17:04:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',64 from dual union all
select 2751,'1954-A710','1954-12-20 00:00:00','atmospheric rocket','success',471,'','D37 Model','','','Test',1821,'',5 from dual union all
select 27510,'1971-W96','1971-01-19 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 27511,'1971-W960','1971-07-19 17:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 27512,'1971-W961','1971-07-19 19:35:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 27513,'1971-W962','1971-07-19 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 27514,'1971-W963','1971-07-20 02:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 27515,'1971-W964','1971-07-20 21:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 27516,'1971-W965','1971-07-20 21:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 27517,'1971-W966','1971-07-21 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',54 from dual union all
select 27518,'1971-W967','1971-07-21 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 27519,'1971-W968','1971-07-21 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 2752,'1954-A711','1954-04-20 00:00:00','suborbital rocket','success',925,'','POGO 1','','','Target',1895,'',30 from dual union all
select 27520,'1971-W969','1971-07-21 14:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 27521,'1971-W97','1971-01-20 00:00:00','suborbital rocket','success',1188,'M--325H','','','','Aeron',1447,'',60 from dual union all
select 27522,'1971-W970','1971-07-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 27523,'1971-W971','1971-07-21 15:23:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 27524,'1971-W972','1971-07-21 16:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 27525,'1971-W973','1971-07-21 16:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 27526,'1971-W974','1971-07-21 17:15:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',56 from dual union all
select 27527,'1971-W975','1971-07-21 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 27528,'1971-W976','1971-07-21 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 27529,'1971-W977','1971-07-22 01:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 2753,'1954-A712','1954-04-09 00:00:00','military missile','failure',1080,'F02','','','','Test',671,'',0 from dual union all
select 27530,'1971-W978','1971-07-22 18:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 27531,'1971-W979','1971-07-22 21:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 27532,'1971-W98','1971-01-20 00:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 27533,'1971-W980','1971-07-23 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 27534,'1971-W981','1971-07-23 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 27535,'1971-W982','1971-07-23 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',54 from dual union all
select 27536,'1971-W983','1971-07-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 27537,'1971-W984','1971-07-23 15:59:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',62 from dual union all
select 27538,'1971-W985','1971-07-23 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 27539,'1971-W986','1971-07-23 19:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',58 from dual union all
select 2754,'1954-A713','1954-08-04 00:00:00','atmospheric rocket','unknown',491,'7','Papa John 553-1','','','Test',1895,'',10 from dual union all
select 27540,'1971-W987','1971-07-23 21:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 27541,'1971-W988','1971-07-23 22:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 27542,'1971-W989','1971-07-24 12:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 27543,'1971-W99','1971-01-20 12:34:59','suborbital rocket','success',631,'INTA H-7103','','','','Meteo',396,'',61 from dual union all
select 27544,'1971-W990','1971-07-25 12:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 27545,'1971-W991','1971-07-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 27546,'1971-W992','1971-07-26 14:15:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',62 from dual union all
select 27547,'1971-W993','1971-07-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 27548,'1971-W994','1971-07-26 15:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual;
insert into LAUNCH
select 27549,'1971-W995','1971-07-26 16:35:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',66 from dual union all
select 2755,'1954-A714','1954-09-29 00:00:00','atmospheric rocket','unknown',491,'8','Papa John 553-2','','','Test',1895,'',10 from dual union all
select 27550,'1971-W996','1971-07-26 17:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',62 from dual union all
select 27551,'1971-W997','1971-07-26 17:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 27552,'1971-W998','1971-07-26 19:08:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 27553,'1971-W999','1971-07-26 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 27554,'1971-Y01','1971-01-01 20:03:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 27555,'1971-Y02','1971-01-06 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 27556,'1971-Y03','1971-01-06 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',97 from dual union all
select 27557,'1971-Y04','1971-01-06 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',94 from dual union all
select 27558,'1971-Y05','1971-01-06 17:30:00','suborbital rocket','unknown',725,'','ISRO-8.005','TERLS-275','','Aeron',1467,'',89 from dual union all
select 27559,'1971-Y06','1971-01-07 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 2756,'1954-A715','1954-11-15 00:00:00','atmospheric rocket','unknown',491,'9','Papa John 553-3','','','Test',1895,'',10 from dual union all
select 27560,'1971-Y07','1971-01-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',85 from dual union all
select 27561,'1971-Y08','1971-01-08 16:10:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 27562,'1971-Y09','1971-01-11 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 27563,'1971-Y10','1971-01-11 22:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 27564,'1971-Y100','1971-03-17 22:18:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 27565,'1971-Y101','1971-03-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 27566,'1971-Y102','1971-03-24 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',95 from dual union all
select 27567,'1971-Y103','1971-03-24 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 27568,'1971-Y104','1971-03-24 16:50:00','suborbital rocket','unknown',725,'','ISRO-8.017','TERLS-299','','Aeron',1467,'',89 from dual union all
select 27569,'1971-Y105','1971-03-24 21:11:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 2757,'1954-A716','1954-01-01 00:00:00','atmospheric rocket','unknown',491,'10','Warhead Accel Test','','','Test',1895,'',10 from dual union all
select 27570,'1971-Y106','1971-03-26 16:06:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',90 from dual union all
select 27571,'1971-Y107','1971-03-27 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 27572,'1971-Y108','1971-03-31 02:36:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 27573,'1971-Y109','1971-03-31 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 27574,'1971-Y11','1971-01-12 05:55:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 27575,'1971-Y110','1971-03-31 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 27576,'1971-Y111','1971-03-31 10:04:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',103 from dual union all
select 27577,'1971-Y112','1971-03-31 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',91 from dual union all
select 27578,'1971-Y113','1971-03-31 16:30:00','suborbital rocket','unknown',725,'','ISRO-8.018','TERLS-300','','Aeron',1467,'',89 from dual union all
select 27579,'1971-Y114','1971-04-01 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 2758,'1954-A717','1954-09-15 00:00:00','atmospheric rocket','success',836,'1249B-2531','Nike 377R','','','Test',1895,'',10 from dual union all
select 27580,'1971-Y115','1971-04-01 22:56:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 27581,'1971-Y116','1971-04-05 13:27:00','suborbital rocket','success',725,'','ISRO-8.019','TERLS-301','','Aeron',1467,'',88 from dual union all
select 27582,'1971-Y117','1971-04-05 14:02:00','suborbital rocket','unknown',725,'','ISRO-8.020','TERLS-302','','Aeron',1467,'',90 from dual union all
select 27583,'1971-Y118','1971-04-07 02:09:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 27584,'1971-Y119','1971-04-07 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 27585,'1971-Y12','1971-01-12 11:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 27586,'1971-Y120','1971-04-07 09:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 27587,'1971-Y121','1971-04-07 10:11:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',103 from dual union all
select 27588,'1971-Y122','1971-04-08 22:09:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',80 from dual union all
select 27589,'1971-Y123','1971-04-10 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 2759,'1954-A718','1954-12-01 00:00:00','atmospheric rocket','success',836,'3109R','Nike 407R','','','Test',1895,'',10 from dual union all
select 27590,'1971-Y124','1971-04-14 02:35:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 27591,'1971-Y125','1971-04-14 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 27592,'1971-Y126','1971-04-16 20:06:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 27593,'1971-Y127','1971-04-21 02:26:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 27594,'1971-Y128','1971-04-21 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 27595,'1971-Y129','1971-04-21 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 27596,'1971-Y13','1971-01-12 16:57:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 27597,'1971-Y130','1971-04-22 23:19:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 27598,'1971-Y131','1971-04-23 18:07:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',89 from dual union all
select 27599,'1971-Y132','1971-04-28 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 276,'1944-S250','1944-11-19 10:14:00','military missile','success',1470,'19547','Ma292','','','Test',661,'',90 from dual union all
select 2760,'1954-A719','1954-04-27 00:00:00','suborbital rocket','success',925,'','POGO','','','Target',1895,'',30 from dual union all
select 27600,'1971-Y133','1971-04-28 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 27601,'1971-Y134','1971-04-28 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',92 from dual union all
select 27602,'1971-Y135','1971-04-28 16:03:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 27603,'1971-Y136','1971-04-28 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 27604,'1971-Y137','1971-05-03 09:48:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 27605,'1971-Y138','1971-05-05 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 27606,'1971-Y139','1971-05-05 09:03:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 27607,'1971-Y14','1971-01-12 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 27608,'1971-Y140','1971-05-05 10:09:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',90 from dual union all
select 27609,'1971-Y141','1971-05-06 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 2761,'1954-A72','1954-02-23 00:00:00','atmospheric rocket','unknown',576,'429ST','','','','Test',1895,'',10 from dual union all
select 27610,'1971-Y142','1971-05-08 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 27611,'1971-Y143','1971-05-12 01:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 27612,'1971-Y144','1971-05-12 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 27613,'1971-Y145','1971-05-12 10:24:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',91 from dual union all
select 27614,'1971-Y146','1971-05-12 15:07:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 27615,'1971-Y147','1971-05-14 22:01:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 27616,'1971-Y148','1971-05-19 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 27617,'1971-Y149','1971-05-19 09:14:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 27618,'1971-Y15','1971-01-13 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 27619,'1971-Y150','1971-05-19 10:04:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',94 from dual union all
select 2762,'1954-A720','1954-05-28 00:00:00','suborbital rocket','success',925,'','POGO','','','Target',1895,'',30 from dual union all
select 27620,'1971-Y151','1971-05-19 14:35:00','suborbital rocket','unknown',725,'','ISRO-8.21','TERLS-309','','Aeron',1467,'',90 from dual union all
select 27621,'1971-Y152','1971-05-19 14:45:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',96 from dual union all
select 27622,'1971-Y153','1971-05-19 15:49:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',94 from dual union all
select 27623,'1971-Y154','1971-05-19 20:32:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 27624,'1971-Y155','1971-05-19 21:33:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 27625,'1971-Y156','1971-05-20 20:22:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',93 from dual union all
select 27626,'1971-Y157','1971-05-20 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',88 from dual union all
select 27627,'1971-Y158','1971-05-22 17:15:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 27628,'1971-Y159','1971-05-22 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',88 from dual union all
select 27629,'1971-Y16','1971-01-13 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 2763,'1954-A721','1954-07-27 00:00:00','suborbital rocket','success',925,'','POGO','','','Target',1895,'',30 from dual union all
select 27630,'1971-Y160','1971-05-24 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',92 from dual union all
select 27631,'1971-Y161','1971-05-26 09:31:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 27632,'1971-Y162','1971-05-26 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',96 from dual union all
select 27633,'1971-Y163','1971-05-26 14:40:00','suborbital rocket','unknown',725,'','ISRO-8.22','TERLS-310','','Aeron',1467,'',92 from dual union all
select 27634,'1971-Y164','1971-05-26 15:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',91 from dual union all
select 27635,'1971-Y165','1971-05-27 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',85 from dual union all
select 27636,'1971-Y166','1971-05-28 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 27637,'1971-Y167','1971-05-29 17:34:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',92 from dual union all
select 27638,'1971-Y168','1971-06-01 14:27:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual;
insert into LAUNCH
select 27639,'1971-Y169','1971-06-02 09:56:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 2764,'1954-A722','1954-08-13 00:00:00','suborbital rocket','success',925,'','POGO','','','Target',1895,'',30 from dual union all
select 27640,'1971-Y17','1971-01-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 27641,'1971-Y170','1971-06-02 14:40:00','suborbital rocket','unknown',725,'','ISRO-8.23','TERLS-311','','Aeron',1467,'',89 from dual union all
select 27642,'1971-Y171','1971-06-02 15:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',90 from dual union all
select 27643,'1971-Y172','1971-06-04 23:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 27644,'1971-Y173','1971-06-06 19:57:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',87 from dual union all
select 27645,'1971-Y174','1971-06-09 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 27646,'1971-Y175','1971-06-09 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',92 from dual union all
select 27647,'1971-Y176','1971-06-09 14:40:00','suborbital rocket','unknown',725,'','ISRO-8.24','TERLS-312','','Aeron',1467,'',85 from dual union all
select 27648,'1971-Y177','1971-06-09 15:37:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',100 from dual union all
select 27649,'1971-Y178','1971-06-09 19:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',87 from dual union all
select 2765,'1954-A723','1954-09-07 00:00:00','suborbital rocket','success',925,'','POGO','','','Target',1895,'',30 from dual union all
select 27650,'1971-Y179','1971-06-10 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',87 from dual union all
select 27651,'1971-Y18','1971-01-13 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',104 from dual union all
select 27652,'1971-Y180','1971-06-12 18:59:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',87 from dual union all
select 27653,'1971-Y181','1971-06-13 18:59:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',82 from dual union all
select 27654,'1971-Y182','1971-06-14 18:53:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',87 from dual union all
select 27655,'1971-Y183','1971-06-16 09:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 27656,'1971-Y184','1971-06-16 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.25','TERLS-318','','Aeron',1467,'',91 from dual union all
select 27657,'1971-Y185','1971-06-16 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 27658,'1971-Y186','1971-06-16 18:56:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',90 from dual union all
select 27659,'1971-Y187','1971-06-20 17:27:00','suborbital rocket','success',1506,'','PAET Sphere','','','Aeron',194,'',85 from dual union all
select 2766,'1954-A724','1954-09-22 00:00:00','suborbital rocket','success',925,'','POGO','','','Target',1895,'',30 from dual union all
select 27660,'1971-Y188','1971-06-21 00:00:00','suborbital rocket','success',1506,'','PAET Sphere','','','Aeron',194,'',85 from dual union all
select 27661,'1971-Y189','1971-06-23 14:45:00','suborbital rocket','unknown',725,'','ISRO-8.26','TERLS-322','','Aeron',1467,'',91 from dual union all
select 27662,'1971-Y19','1971-01-13 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',92 from dual union all
select 27663,'1971-Y190','1971-06-23 19:20:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 27664,'1971-Y191','1971-06-24 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',89 from dual union all
select 27665,'1971-Y192','1971-06-24 22:06:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 27666,'1971-Y193','1971-06-26 17:07:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',89 from dual union all
select 27667,'1971-Y194','1971-06-28 20:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',88 from dual union all
select 27668,'1971-Y195','1971-06-29 10:43:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',92 from dual union all
select 27669,'1971-Y196','1971-06-30 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 2767,'1954-A725','1954-11-18 00:00:00','atmospheric rocket','success',762,'1','HTV','','','Heat transfer',1895,'',10 from dual union all
select 27670,'1971-Y197','1971-06-30 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 27671,'1971-Y198','1971-06-30 16:06:00','suborbital rocket','failure',725,'','ISRO-8.27','TERLS-323','','Aeron',1467,'',88 from dual union all
select 27672,'1971-Y199','1971-06-30 20:03:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',86 from dual union all
select 27673,'1971-Y20','1971-01-13 17:35:00','suborbital rocket','unknown',725,'','ISRO-8.006','TERLS-277','','Aeron',1467,'',94 from dual union all
select 27674,'1971-Y200','1971-06-30 21:35:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 27675,'1971-Y201','1971-07-01 00:00:00','suborbital rocket','success',1188,'SK39H  [spurious]','Ionos/Neutral atm.','','','Aeron?',1447,'',89 from dual union all
select 27676,'1971-Y202','1971-07-01 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',90 from dual union all
select 27677,'1971-Y203','1971-07-02 20:03:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 27678,'1971-Y204','1971-07-02 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',91 from dual union all
select 27679,'1971-Y205','1971-07-03 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',89 from dual union all
select 2768,'1954-A726','1954-11-23 00:00:00','atmospheric rocket','success',762,'2','HTV','','','Heat transfer',1895,'',10 from dual union all
select 27680,'1971-Y206','1971-07-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 27681,'1971-Y207','1971-07-06 20:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 27682,'1971-Y208','1971-07-07 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 27683,'1971-Y209','1971-07-07 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',87 from dual union all
select 27684,'1971-Y21','1971-01-14 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 27685,'1971-Y210','1971-07-07 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.28','TERLS-324','','Aeron',1467,'',94 from dual union all
select 27686,'1971-Y211','1971-07-07 15:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',82 from dual union all
select 27687,'1971-Y212','1971-07-08 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',91 from dual union all
select 27688,'1971-Y213','1971-07-09 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1522,'',90 from dual union all
select 27689,'1971-Y214','1971-07-14 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 2769,'1954-A727','1954-12-14 00:00:00','atmospheric rocket','success',762,'3','HTV','','','Heat transfer',1895,'',10 from dual union all
select 27690,'1971-Y215','1971-07-14 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 27691,'1971-Y216','1971-07-14 10:44:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',102 from dual union all
select 27692,'1971-Y217','1971-07-14 12:50:00','suborbital rocket','success',1188,'SK36H','','','','Ionos',1447,'',100 from dual union all
select 27693,'1971-Y218','1971-07-14 14:34:00','suborbital rocket','unknown',725,'','ISRO-8.29','TERLS-325','','Aeron',1467,'',89 from dual union all
select 27694,'1971-Y219','1971-07-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 27695,'1971-Y22','1971-01-14 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',87 from dual union all
select 27696,'1971-Y220','1971-07-14 17:35:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 27697,'1971-Y221','1971-07-15 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 27698,'1971-Y222','1971-07-21 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 27699,'1971-Y223','1971-07-21 10:14:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',95 from dual union all
select 277,'1944-S251','1944-11-19 11:47:00','military missile','success',1470,'19545','Ma291','','','Test',661,'',90 from dual union all
select 2770,'1954-A728','1954-12-17 00:00:00','atmospheric rocket','success',762,'4','HTV','','','Heat transfer',1895,'',10 from dual union all
select 27700,'1971-Y224','1971-07-21 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.30','TERLS-326','','Aeron',1467,'',92 from dual union all
select 27701,'1971-Y225','1971-07-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 27702,'1971-Y226','1971-07-23 15:11:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 27703,'1971-Y227','1971-07-28 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 27704,'1971-Y228','1971-07-28 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',91 from dual union all
select 27705,'1971-Y229','1971-07-28 14:37:00','suborbital rocket','unknown',725,'','ISRO-8.31','TERLS-327','','Aeron',1467,'',91 from dual union all
select 27706,'1971-Y23','1971-01-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',85 from dual union all
select 27707,'1971-Y230','1971-07-28 15:34:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',89 from dual union all
select 27708,'1971-Y231','1971-07-30 12:34:00','suborbital rocket','success',1188,'SK37H','','','','Ionos',1447,'',100 from dual union all
select 27709,'1971-Y232','1971-08-04 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 2771,'1954-A729','1954-09-29 00:00:00','atmospheric rocket','success',564,'','','','','Test',1895,'',5 from dual union all
select 27710,'1971-Y233','1971-08-04 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 27711,'1971-Y234','1971-08-04 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',92 from dual union all
select 27712,'1971-Y235','1971-08-04 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.32','TERLS-337','','Aeron',1467,'',91 from dual union all
select 27713,'1971-Y236','1971-08-04 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 27714,'1971-Y237','1971-08-04 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',91 from dual union all
select 27715,'1971-Y238','1971-08-04 15:23:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 27716,'1971-Y239','1971-08-04 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 27717,'1971-Y24','1971-01-16 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',88 from dual union all
select 27718,'1971-Y240','1971-08-07 21:06:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 27719,'1971-Y241','1971-08-11 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 2772,'1954-A73','1954-02-24 00:00:00','atmospheric rocket','success',365,'','F29 Ramjet','','','Test',1821,'',10 from dual union all
select 27720,'1971-Y242','1971-08-11 10:34:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',90 from dual union all
select 27721,'1971-Y243','1971-08-11 14:33:00','suborbital rocket','unknown',725,'','ISRO-8.33','TERLS-338','','Aeron',1467,'',87 from dual union all
select 27722,'1971-Y244','1971-08-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 27723,'1971-Y245','1971-08-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',90 from dual union all
select 27724,'1971-Y246','1971-08-11 15:01:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 27725,'1971-Y247','1971-08-11 22:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 27726,'1971-Y248','1971-08-12 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 27727,'1971-Y249','1971-08-18 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 27728,'1971-Y25','1971-01-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',89 from dual;
insert into LAUNCH
select 27729,'1971-Y250','1971-08-18 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 2773,'1954-A730','1954-09-29 00:00:00','atmospheric rocket','success',564,'','','','','Test',1895,'',5 from dual union all
select 27730,'1971-Y251','1971-08-18 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.34','TERLS-343','','Aeron',1467,'',92 from dual union all
select 27731,'1971-Y252','1971-08-18 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 27732,'1971-Y253','1971-08-18 21:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 27733,'1971-Y254','1971-08-19 18:32:00','suborbital rocket','success',1506,'','','','','Aeron',1821,'',135 from dual union all
select 27734,'1971-Y255','1971-08-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 27735,'1971-Y256','1971-08-25 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 27736,'1971-Y257','1971-08-25 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 27737,'1971-Y258','1971-08-25 11:15:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',90 from dual union all
select 27738,'1971-Y259','1971-08-25 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.35','TERLS-344','','Aeron',1467,'',90 from dual union all
select 27739,'1971-Y26','1971-01-18 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 2774,'1954-A731','1954-01-26 00:00:00','atmospheric rocket','success',1526,'MX-883','','','','Test',1895,'B-29',10 from dual union all
select 27740,'1971-Y260','1971-08-25 18:02:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 27741,'1971-Y261','1971-08-31 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 27742,'1971-Y262','1971-08-31 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 27743,'1971-Y263','1971-09-01 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 27744,'1971-Y264','1971-09-01 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 27745,'1971-Y265','1971-09-01 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',94 from dual union all
select 27746,'1971-Y266','1971-09-01 14:30:00','suborbital rocket','unknown',725,'TERLS-345','ISRO-8.36','TERLS-345','','Aeron',1467,'',90 from dual union all
select 27747,'1971-Y267','1971-09-01 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 27748,'1971-Y268','1971-09-03 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 27749,'1971-Y269','1971-09-03 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 2775,'1954-A732','1954-02-24 00:00:00','atmospheric rocket','success',1526,'MX-883','','','','Test',1895,'B-29',10 from dual union all
select 27750,'1971-Y27','1971-01-18 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 27751,'1971-Y270','1971-09-08 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',92 from dual union all
select 27752,'1971-Y271','1971-09-08 12:06:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 27753,'1971-Y272','1971-09-08 14:30:00','suborbital rocket','unknown',725,'TERLS-346','ISRO-8.37','TERLS-346','','Aeron',1467,'',90 from dual union all
select 27754,'1971-Y273','1971-09-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 27755,'1971-Y274','1971-09-14 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 27756,'1971-Y275','1971-09-15 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 27757,'1971-Y276','1971-09-15 13:22:00','suborbital rocket','success',1188,'SK38H','','','','Ionos',1447,'',82 from dual union all
select 27758,'1971-Y277','1971-09-15 14:05:00','suborbital rocket','unknown',725,'TERLS-347','ISRO-8.38','TERLS-347','','Aeron',1467,'',91 from dual union all
select 27759,'1971-Y278','1971-09-15 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 2776,'1954-A733','1954-06-08 00:00:00','atmospheric rocket','success',1526,'MX-883','','','','Test',1895,'B-29',10 from dual union all
select 27760,'1971-Y279','1971-09-15 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 27761,'1971-Y28','1971-01-19 00:00:00','suborbital rocket','success',1188,'M--320A','','','','Aeron',2,'',100 from dual union all
select 27762,'1971-Y280','1971-09-15 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 27763,'1971-Y281','1971-09-22 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 27764,'1971-Y282','1971-09-22 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',92 from dual union all
select 27765,'1971-Y283','1971-09-22 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.39','TERLS-349','','Aeron',1467,'',90 from dual union all
select 27766,'1971-Y284','1971-09-22 15:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',85 from dual union all
select 27767,'1971-Y285','1971-09-22 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 27768,'1971-Y286','1971-09-24 14:40:00','suborbital rocket','unknown',725,'','Sun-Atm 71','','','Aeron',636,'',87 from dual union all
select 27769,'1971-Y287','1971-09-29 03:00:00','suborbital rocket','unknown',725,'','Sun-Atm 71','','','Aeron',636,'',86 from dual union all
select 2777,'1954-A734','1954-09-23 00:00:00','atmospheric rocket','success',1526,'MX-883','','','','Test',1895,'B-29',10 from dual union all
select 27770,'1971-Y288','1971-09-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 27771,'1971-Y289','1971-09-29 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.40','TERLS-350','','Aeron',1467,'',88 from dual union all
select 27772,'1971-Y29','1971-01-20 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 27773,'1971-Y290','1971-10-01 16:00:00','suborbital rocket','unknown',725,'','Sun-Atm 71','','','Aeron',636,'',88 from dual union all
select 27774,'1971-Y291','1971-10-02 03:30:00','suborbital rocket','unknown',725,'','Sun-Atm 71','','','Aeron',636,'',87 from dual union all
select 27775,'1971-Y292','1971-10-03 03:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 27776,'1971-Y293','1971-10-03 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 27777,'1971-Y294','1971-10-04 03:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 27778,'1971-Y295','1971-10-04 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 27779,'1971-Y296','1971-10-05 03:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 2778,'1954-A735','1954-10-21 00:00:00','atmospheric rocket','success',1526,'MX-883','','','','Test',1895,'B-29',10 from dual union all
select 27780,'1971-Y297','1971-10-05 14:55:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 27781,'1971-Y298','1971-10-06 02:00:00','suborbital rocket','unknown',725,'','Sun-Atm 71','','','Aeron',636,'',91 from dual union all
select 27782,'1971-Y299','1971-10-06 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 27783,'1971-Y30','1971-01-20 04:30:00','suborbital rocket','unknown',725,'','ISRO-8.007','TERLS-278','','Aeron',1467,'',91 from dual union all
select 27784,'1971-Y300','1971-10-06 11:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',85 from dual union all
select 27785,'1971-Y301','1971-10-06 14:30:00','suborbital rocket','unknown',725,'TERLS-363','ISRO-8.41','TERLS-363','','Aeron',1467,'',90 from dual union all
select 27786,'1971-Y302','1971-10-06 14:45:00','suborbital rocket','unknown',725,'','Sun-Atm 71','','','Aeron',636,'',89 from dual union all
select 27787,'1971-Y303','1971-10-06 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',96 from dual union all
select 27788,'1971-Y304','1971-10-07 03:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 27789,'1971-Y305','1971-10-07 05:35:00','suborbital rocket','unknown',725,'','Sun-Atm 71','','','Aeron',636,'',88 from dual union all
select 2779,'1954-A736','1954-11-24 00:00:00','atmospheric rocket','success',1526,'MX-883','','','','Test',1895,'B-29',10 from dual union all
select 27790,'1971-Y306','1971-10-07 14:40:00','suborbital rocket','unknown',725,'','Sun-Atm 71','','','Aeron',636,'',86 from dual union all
select 27791,'1971-Y307','1971-10-08 03:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',96 from dual union all
select 27792,'1971-Y308','1971-10-08 03:40:00','suborbital rocket','unknown',725,'','Sun-Atm 71','','','Aeron',636,'',89 from dual union all
select 27793,'1971-Y309','1971-10-08 15:00:00','suborbital rocket','unknown',725,'','Sun-Atm 71','','','Aeron',636,'',88 from dual union all
select 27794,'1971-Y31','1971-01-20 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 27795,'1971-Y310','1971-10-09 14:00:00','suborbital rocket','unknown',725,'','Sun-Atm 71','','','Aeron',636,'',88 from dual union all
select 27796,'1971-Y311','1971-10-13 02:00:00','suborbital rocket','unknown',725,'','Sun-Atm 71','','','Aeron',636,'',87 from dual union all
select 27797,'1971-Y312','1971-10-13 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 27798,'1971-Y313','1971-10-13 14:31:00','suborbital rocket','unknown',725,'','ISRO-8.42','TERLS-364','','Aeron',1467,'',87 from dual union all
select 27799,'1971-Y314','1971-10-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 278,'1944-S252','1944-11-20 09:46:00','military missile','success',1470,'19533','Ma293','','','Test',661,'',90 from dual union all
select 2780,'1954-A737','1954-12-14 00:00:00','atmospheric rocket','success',1526,'MX-883','','','','Test',1895,'B-29',10 from dual union all
select 27800,'1971-Y315','1971-10-14 11:55:00','suborbital rocket','success',1188,'SK41H','','','','Ionos',1447,'',91 from dual union all
select 27801,'1971-Y316','1971-10-20 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 27802,'1971-Y317','1971-10-20 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 27803,'1971-Y318','1971-10-20 10:14:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',85 from dual union all
select 27804,'1971-Y319','1971-10-20 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.43','TERLS-371','','Aeron',1467,'',80 from dual union all
select 27805,'1971-Y32','1971-01-20 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',92 from dual union all
select 27806,'1971-Y320','1971-10-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 27807,'1971-Y321','1971-10-27 01:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 27808,'1971-Y322','1971-10-27 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 27809,'1971-Y323','1971-10-27 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',97 from dual union all
select 2781,'1954-A738','1954-12-21 00:00:00','atmospheric rocket','success',1526,'MX-883','','','','Test',1895,'B-29',10 from dual union all
select 27810,'1971-Y324','1971-10-27 16:20:00','suborbital rocket','unknown',725,'','ISRO-8.44','TERLS-372','','Aeron',1467,'',89 from dual union all
select 27811,'1971-Y325','1971-10-28 08:26:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 27812,'1971-Y326','1971-11-03 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 27813,'1971-Y327','1971-11-03 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 27814,'1971-Y328','1971-11-03 10:54:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',85 from dual union all
select 27815,'1971-Y329','1971-11-03 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.45','TERLS-376','','Aeron',1467,'',90 from dual union all
select 27816,'1971-Y33','1971-01-20 16:32:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',92 from dual union all
select 27817,'1971-Y330','1971-11-03 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',85 from dual union all
select 27818,'1971-Y331','1971-11-09 16:09:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual;
insert into LAUNCH
select 27819,'1971-Y332','1971-11-10 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 2782,'1954-A739','1954-12-25 00:00:00','military missile','failure',992,'','Generator-2','','','Test',636,'',30 from dual union all
select 27820,'1971-Y333','1971-11-10 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.46','TERLS-383','','Aeron',1467,'',87 from dual union all
select 27821,'1971-Y334','1971-11-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 27822,'1971-Y335','1971-11-16 14:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',95 from dual union all
select 27823,'1971-Y336','1971-11-17 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 27824,'1971-Y337','1971-11-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 27825,'1971-Y338','1971-11-17 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',91 from dual union all
select 27826,'1971-Y339','1971-11-17 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.47','TERLS-384','','Aeron',1467,'',90 from dual union all
select 27827,'1971-Y34','1971-01-20 17:30:00','suborbital rocket','unknown',725,'','ISRO-8.008','TERLS-279','','Aeron',1467,'',90 from dual union all
select 27828,'1971-Y340','1971-11-17 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',92 from dual union all
select 27829,'1971-Y341','1971-11-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 2783,'1954-A74','1954-02-24 00:00:00','atmospheric rocket','unknown',576,'424ST','','','','Test',1895,'',10 from dual union all
select 27830,'1971-Y342','1971-11-18 16:54:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',84 from dual union all
select 27831,'1971-Y343','1971-11-19 11:53:00','suborbital rocket','success',1188,'SK42H','','','','Ionos',1447,'',92 from dual union all
select 27832,'1971-Y344','1971-11-24 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',95 from dual union all
select 27833,'1971-Y345','1971-11-24 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.48','TERLS-385','','Aeron',1467,'',90 from dual union all
select 27834,'1971-Y346','1971-11-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 27835,'1971-Y347','1971-11-25 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 27836,'1971-Y348','1971-11-30 16:42:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 27837,'1971-Y349','1971-12-01 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 27838,'1971-Y35','1971-01-21 00:00:00','suborbital rocket','success',1188,'M--322A','','','','Aeron',2,'',100 from dual union all
select 27839,'1971-Y350','1971-12-01 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 2784,'1954-A75','1954-02-24 00:00:00','atmospheric rocket','unknown',576,'421ST','','','','Test',1895,'',10 from dual union all
select 27840,'1971-Y351','1971-12-01 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',85 from dual union all
select 27841,'1971-Y352','1971-12-01 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.49','TERLS-386','','Aeron',1467,'',90 from dual union all
select 27842,'1971-Y353','1971-12-01 17:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',85 from dual union all
select 27843,'1971-Y354','1971-12-01 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 27844,'1971-Y355','1971-12-08 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 27845,'1971-Y356','1971-12-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 27846,'1971-Y357','1971-12-08 14:30:00','suborbital rocket','unknown',725,'TERLS-390','ISRO-8.50','TERLS-390','','Aeron',1467,'',87 from dual union all
select 27847,'1971-Y358','1971-12-08 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 27848,'1971-Y359','1971-12-14 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 27849,'1971-Y36','1971-01-21 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 2785,'1954-A76','1954-02-24 00:00:00','atmospheric rocket','unknown',576,'433ST','','','','Test',1895,'',10 from dual union all
select 27850,'1971-Y360','1971-12-15 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 27851,'1971-Y361','1971-12-15 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 27852,'1971-Y362','1971-12-15 11:04:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',92 from dual union all
select 27853,'1971-Y363','1971-12-15 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 27854,'1971-Y364','1971-12-15 15:00:00','suborbital rocket','unknown',725,'TERLS-391','ISRO-8.51','TERLS-391','','Ionos',1467,'',87 from dual union all
select 27855,'1971-Y365','1971-12-15 16:33:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',84 from dual union all
select 27856,'1971-Y366','1971-12-15 17:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',85 from dual union all
select 27857,'1971-Y367','1971-12-15 18:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',86 from dual union all
select 27858,'1971-Y368','1971-12-21 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 27859,'1971-Y369','1971-12-22 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 2786,'1954-A77','1954-02-25 00:00:00','atmospheric rocket','unknown',576,'413ST','','','','Test',1895,'',10 from dual union all
select 27860,'1971-Y37','1971-01-22 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 27861,'1971-Y370','1971-12-22 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 27862,'1971-Y371','1971-12-22 15:00:00','suborbital rocket','unknown',725,'TERLS-396','ISRO-8.52','TERLS-396','','Aeron',1467,'',86 from dual union all
select 27863,'1971-Y372','1971-12-22 15:00:00','suborbital rocket','unknown',725,'','','Spurious','','Aeron',1467,'',90 from dual union all
select 27864,'1971-Y373','1971-12-28 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 27865,'1971-Y374','1971-12-29 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 27866,'1971-Y375','1971-12-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 27867,'1971-Y376','1971-12-29 14:30:00','suborbital rocket','unknown',725,'TERLS-397','ISRO-8.53','TERLS-397','','Aeron',1467,'',92 from dual union all
select 27868,'1971-Y377','1971-12-29 15:00:00','suborbital rocket','unknown',725,'','','Spurious','','Aeron',1467,'',87 from dual union all
select 27869,'1971-Y378','1971-12-29 16:41:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',85 from dual union all
select 2787,'1954-A78','1954-02-25 00:00:00','atmospheric rocket','unknown',576,'428ST','','','','Test',1895,'',10 from dual union all
select 27870,'1971-Y379','1971-12-29 19:04:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',82 from dual union all
select 27871,'1971-Y38','1971-01-22 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',93 from dual union all
select 27872,'1971-Y39','1971-01-23 00:27:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 27873,'1971-Y40','1971-01-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',92 from dual union all
select 27874,'1971-Y41','1971-01-25 09:12:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 27875,'1971-Y42','1971-01-27 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 27876,'1971-Y43','1971-01-27 09:12:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 27877,'1971-Y44','1971-01-27 10:04:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',96 from dual union all
select 27878,'1971-Y45','1971-01-29 09:02:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 27879,'1971-Y46','1971-01-29 19:48:00','suborbital rocket','success',258,'NASA 15.50UE','','','','Plasma',1821,'',122 from dual union all
select 2788,'1954-A79','1954-02-25 00:00:00','atmospheric rocket','unknown',576,'415ST','','','','Test',1895,'',10 from dual union all
select 27880,'1971-Y47','1971-01-30 05:22:00','suborbital rocket','unknown',725,'','ISRO-8.009','TERLS-283','','Aeron',1467,'',89 from dual union all
select 27881,'1971-Y48','1971-02-01 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',82 from dual union all
select 27882,'1971-Y49','1971-02-01 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 27883,'1971-Y50','1971-02-03 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 27884,'1971-Y51','1971-02-03 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',89 from dual union all
select 27885,'1971-Y52','1971-02-03 11:20:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',92 from dual union all
select 27886,'1971-Y53','1971-02-03 16:58:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 27887,'1971-Y54','1971-02-03 19:05:00','suborbital rocket','unknown',725,'','ISRO-8.010','TERLS-284','','Aeron',1467,'',91 from dual union all
select 27888,'1971-Y55','1971-02-03 21:25:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 27889,'1971-Y56','1971-02-04 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 2789,'1954-A80','1954-02-25 00:00:00','military missile','success',771,'GM-12447','','','','Test',222,'',12 from dual union all
select 27890,'1971-Y57','1971-02-05 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',89 from dual union all
select 27891,'1971-Y58','1971-02-05 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 27892,'1971-Y59','1971-02-08 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',91 from dual union all
select 27893,'1971-Y60','1971-02-08 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 27894,'1971-Y61','1971-02-09 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',85 from dual union all
select 27895,'1971-Y62','1971-02-10 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 27896,'1971-Y63','1971-02-10 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',104 from dual union all
select 27897,'1971-Y64','1971-02-10 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',87 from dual union all
select 27898,'1971-Y65','1971-02-10 18:00:00','suborbital rocket','unknown',725,'','ISRO-8.011','TERLS-285','','Aeron',1467,'',91 from dual union all
select 27899,'1971-Y66','1971-02-12 09:08:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 279,'1944-S253','1944-11-20 14:13:00','military missile','success',1470,'19550','Ma295','','','Test',661,'',90 from dual union all
select 2790,'1954-A81','1954-02-25 00:00:00','atmospheric rocket','success',836,'1318A','','','','Test',1895,'',10 from dual union all
select 27900,'1971-Y67','1971-02-13 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',90 from dual union all
select 27901,'1971-Y68','1971-02-15 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 27902,'1971-Y69','1971-02-17 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 27903,'1971-Y70','1971-02-17 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 27904,'1971-Y71','1971-02-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1521,'',87 from dual union all
select 27905,'1971-Y72','1971-02-17 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 27906,'1971-Y73','1971-02-17 11:15:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',102 from dual union all
select 27907,'1971-Y74','1971-02-17 17:30:00','suborbital rocket','unknown',725,'','ISRO-8.012','TERLS-286','','Aeron',1467,'',84 from dual union all
select 27908,'1971-Y75','1971-02-17 21:20:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual;
insert into LAUNCH
select 27909,'1971-Y76','1971-02-22 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 2791,'1954-A82','1954-02-26 00:00:00','atmospheric rocket','unknown',576,'427ST','','','','Test',1895,'',10 from dual union all
select 27910,'1971-Y77','1971-02-24 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 27911,'1971-Y78','1971-02-24 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 27912,'1971-Y79','1971-02-24 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 27913,'1971-Y80','1971-02-24 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',103 from dual union all
select 27914,'1971-Y81','1971-02-24 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 27915,'1971-Y82','1971-02-24 17:30:00','suborbital rocket','unknown',725,'','ISRO-8.013','TERLS-289','','Aeron',1467,'',91 from dual union all
select 27916,'1971-Y83','1971-02-25 21:15:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 27917,'1971-Y84','1971-02-26 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 27918,'1971-Y85','1971-03-03 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 27919,'1971-Y86','1971-03-03 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 2792,'1954-A83','1954-02-26 00:00:00','atmospheric rocket','unknown',576,'425ST','','','','Test',1895,'',10 from dual union all
select 27920,'1971-Y87','1971-03-03 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 27921,'1971-Y88','1971-03-03 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',94 from dual union all
select 27922,'1971-Y89','1971-03-03 18:00:00','suborbital rocket','unknown',725,'','ISRO-8.014','TERLS-290','','Aeron',1467,'',90 from dual union all
select 27923,'1971-Y90','1971-03-04 18:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 27924,'1971-Y91','1971-03-10 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 27925,'1971-Y92','1971-03-10 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 27926,'1971-Y93','1971-03-10 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 27927,'1971-Y94','1971-03-10 17:30:00','suborbital rocket','unknown',725,'','ISRO-8.015','TERLS-291','','Aeron',1467,'',90 from dual union all
select 27928,'1971-Y95','1971-03-12 21:12:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 27929,'1971-Y96','1971-03-17 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 2793,'1954-A84','1954-02-26 00:00:00','atmospheric rocket','success',836,'1260T','','','','Test',1895,'',10 from dual union all
select 27930,'1971-Y97','1971-03-17 09:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 27931,'1971-Y98','1971-03-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 27932,'1971-Y99','1971-03-17 16:30:00','suborbital rocket','unknown',725,'','ISRO-8.016','TERLS-292','','Aeron',1467,'',92 from dual union all
select 27933,'1972-001','1972-01-12 09:59:59','orbital','success',1508,'7702423-247','Kosmos-471','Zenit-4M','','Sat',562,'',null from dual union all
select 27934,'1972-002','1972-01-20 18:36:00','orbital','success',1426,'23D-2','HEXAGON SV2','HEXAGON SV-2','','Sat',1777,'',null from dual union all
select 27935,'1972-003','1972-01-23 00:12:04','orbital','success',179,'AC-28','Intelsat 4 F-4','INTELSAT IV F4','','Sat',269,'',null from dual union all
select 27936,'1972-004','1972-01-25 11:15:01','orbital','success',678,'29547-143','Kosmos-472','DS-P1-Yu No. 52','','Sat',460,'',null from dual union all
select 27937,'1972-005','1972-01-31 17:20:02','orbital','success',1385,'564/D87','HEOS 2','HEOS A2','','Sat',1723,'',null from dual union all
select 27938,'1972-006','1972-02-03 08:40:00','orbital','success',1508,'7801421-136','Kosmos-473','Zenit-2M','','Sat',562,'',null from dual union all
select 27939,'1972-007','1972-02-14 03:27:59','deep space','success',953,'258-01','Luna-20','VA','','Sat',588,'',null from dual union all
select 2794,'1954-A85','1954-03-01 00:00:00','atmospheric rocket','success',365,'','E17 Drag Model','','','Test',1821,'',10 from dual union all
select 27940,'1972-008','1972-02-16 09:30:00','orbital','success',1508,'7801422-240','Kosmos-474','Zenit-4M','','Sat',562,'',null from dual union all
select 27941,'1972-009','1972-02-25 07:52:29','orbital','success',680,'53727-113','Kosmos-475','Tsiklon','','Sat',457,'',null from dual union all
select 27942,'1972-010','1972-03-01 09:39:28','orbital','success',1425,'23C-4 (3C-22)','DSP 3','DSP 4','','Sat UPF',274,'',null from dual union all
select 27943,'1972-011','1972-03-01 11:15:00','orbital','success',1510,'S 15000-009','Kosmos-476','Tselina-D','','Sat',475,'',null from dual union all
select 27944,'1972-012','1972-03-03 01:49:04','deep space','success',179,'AC-27','Pioneer 10','Pioneer F','','Sat',268,'',null from dual union all
select 27945,'1972-013','1972-03-04 10:00:00','orbital','success',1508,'7801421-138','Kosmos-477','Zenit-2M','','Sat',473,'',null from dual union all
select 27946,'1972-014','1972-03-12 01:55:08','orbital','success',1388,'573/D88','TD-1A','TD-1A','','Sat',1723,'',null from dual union all
select 27947,'1972-015','1972-03-15 13:00:01','orbital','success',1508,'7702423-252','Kosmos-478','Zenit-4M','','Sat',474,'',null from dual union all
select 27948,'1972-016','1972-03-17 17:00:00','orbital','success',1403,'24B-3 (3B-34)','KH-8A','GAMBIT-3 Mission 4334','','Sat',1778,'',null from dual union all
select 27949,'1972-017','1972-03-22 20:30:59','orbital','success',680,'47127-122','Kosmos-479','Tselina-OM','','Sat',457,'',null from dual union all
select 2795,'1954-A86','1954-03-01 00:00:00','atmospheric rocket','failure',835,'','Navaho Model','','','Test',1821,'',1 from dual union all
select 27950,'1972-018','1972-03-24 08:46:12','orbital','success',1365,'153/BIIA-15','DMSP 5528','DMSP Block 5B F-2 (SV-1)','','Sat',1720,'',null from dual union all
select 27951,'1972-019','1972-03-25 02:20:00','orbital','success',680,'65027-123','Kosmos-480','Sfera','','Sat',457,'',null from dual union all
select 27952,'1972-020','1972-03-25 10:39:59','orbital','success',678,'64657-134','Kosmos-481','DS-P1-Yu No. 46','','Sat',460,'',null from dual union all
select 27953,'1972-021','1972-03-27 04:15:01','deep space','success',806,'S 15000-063','Venera-8','4V-1 No. 670','','Sat',562,'',null from dual union all
select 27954,'1972-022','1972-03-30 14:05:01','orbital','success',1510,'7702421-334','Meteor','Meteor-MV No. 23','','Sat',473,'',null from dual union all
select 27955,'1972-023','1972-03-31 04:02:33','orbital','failure',806,'7801421-513','Kosmos-482','4V-1 No. 671','','Sat',562,'',9732 from dual union all
select 27956,'1972-024','1972-04-03 10:15:00','orbital','success',1508,'7801422-245','Kosmos-483','Zenit-4M','','Sat',473,'',null from dual union all
select 27957,'1972-025','1972-04-04 20:38:30','orbital','success',809,'Yu15000-093','Molniya-1','Molniya-1 11F67 No. 27','','Sat',475,'',null from dual union all
select 27958,'1972-026','1972-04-06 08:00:00','orbital','success',1508,'7801421-139','Kosmos-484','Zenit-2M','','Sat',474,'',null from dual union all
select 27959,'1972-027','1972-04-07 10:00:00','orbital','success',1508,'Kh15000-013','Interkosmos-6','Energiya No. 1','','Sat',562,'',null from dual union all
select 2796,'1954-A87','1954-03-01 00:00:00','atmospheric rocket','unknown',576,'419ST','','','','Test',1895,'',10 from dual union all
select 27960,'1972-028','1972-04-11 11:04:58','orbital','success',678,'45734-142','Kosmos-485','DS-P1-Yu No. 58','','Sat',460,'',null from dual union all
select 27961,'1972-029','1972-04-14 00:54:36','orbital','success',807,'Kh15000-079','Prognoz','SO No. 501','','Sat',562,'',null from dual union all
select 27962,'1972-030','1972-04-14 08:00:00','orbital','success',1508,'7702423-253','Kosmos-486','Zenit-4M','','Sat',474,'',null from dual union all
select 27963,'1972-031','1972-04-16 17:54:00','deep space','success',1123,'SA-511','Apollo 16','Apollo CM-113','','Sat',921,'LUT3',null from dual union all
select 27964,'1972-032','1972-04-19 21:43:59','orbital','success',1396,'569/FTV 1661','CORONA 144/1116','CORONA CR-16','','Sat TAG184',1776,'',null from dual union all
select 27965,'1972-033','1972-04-21 11:59:59','orbital','success',678,'45747-137','Kosmos-487','DS-P1-Yu No. 57','','Sat',460,'',null from dual union all
select 27966,'1972-034','1972-05-05 11:20:00','orbital','success',1508,'7702424-717','Kosmos-488','Zenit-4MK','','Sat',475,'',null from dual union all
select 27967,'1972-035','1972-05-06 11:24:51','orbital','success',680,'65024-110','Kosmos-489','Tsiklon','','Sat',456,'',null from dual union all
select 27968,'1972-036','1972-05-17 10:19:59','orbital','success',1508,'7701421-140','Kosmos-490','Zenit-2M','','Sat',474,'',null from dual union all
select 27969,'1972-037','1972-05-19 14:30:03','orbital','success',809,'Yu15000-091','Molniya-2','Molniya-2','','Sat',475,'',null from dual union all
select 2797,'1954-A88','1954-03-01 00:00:00','atmospheric rocket','unknown',576,'426ST','','','','Test',1895,'',10 from dual union all
select 27970,'1972-038','1972-05-25 06:35:00','orbital','success',1508,'7801422-244','Kosmos-491','Zenit-4M','','Sat',562,'',null from dual union all
select 27971,'1972-039','1972-05-25 18:41:00','orbital','success',1396,'571/FTV 1663','CORONA 145/1117','CORONA CR-8R','','Sat TAG185',1776,'',null from dual union all
select 27972,'1972-040','1972-06-09 06:59:59','orbital','success',1508,'7702423-250','Kosmos-492','Zenit-4M','','Sat',562,'',null from dual union all
select 27973,'1972-041','1972-06-13 21:53:04','orbital','success',179,'AC-29','Intelsat 4 F-5','INTELSAT IV F5','','Sat',269,'',null from dual union all
select 27974,'1972-042','1972-06-21 06:25:00','orbital','success',1508,'7801421-142','Kosmos-493','Zenit-2M','','Sat',562,'',null from dual union all
select 27975,'1972-043','1972-06-23 09:24:00','orbital','success',680,'47127-111','Kosmos-494','Strela-2M','','Sat',457,'',null from dual union all
select 27976,'1972-044','1972-06-23 11:19:57','orbital','success',1508,'7801422-239','Kosmos-495','Zenit-4M','','Sat',474,'',null from dual union all
select 27977,'1972-045','1972-06-26 14:53:00','orbital','success',1223,'Yu15000-020','Kosmos-496','Soyuz 7K-T No. 33A','','Sat',480,'',null from dual union all
select 27978,'1972-046','1972-06-29 03:47:35','orbital','success',807,'M 15000-108','Prognoz-2','SO-M No. 503','','Sat',562,'',null from dual union all
select 27979,'1972-047','1972-06-30 05:58:41','orbital','success',678,'','Interkosmos-7','DS-U3-IK No. 3','','Sat',627,'',null from dual union all
select 2798,'1954-A89','1954-03-02 00:00:00','atmospheric rocket','success',564,'','D18 F8U-1 Model','','','Test',1821,'',10 from dual union all
select 27980,'1972-048','1972-06-30 09:19:49','orbital','success',678,'29524-153','Kosmos-497','DS-P1-I No. 12','','Sat',460,'',null from dual union all
select 27981,'1972-049','1972-06-30 18:52:01','orbital','success',1510,'7801421-333','Meteor','Meteor-MV No. 26','','Sat',473,'',null from dual union all
select 27982,'1972-050','1972-07-05 09:29:58','orbital','success',678,'29524-152','Kosmos-498','DS-P1-Yu No. 56','','Sat',460,'',null from dual union all
select 27983,'1972-051','1972-07-06 10:40:00','orbital','success',1508,'7702423-246','Kosmos-499','Zenit-4M','','Sat',562,'',null from dual union all
select 27984,'1972-052','1972-07-07 17:46:00','orbital','success',1426,'23D-5','HEXAGON SV3','HEXAGON SV-3','','Sat',1777,'',null from dual union all
select 27985,'1972-053','1972-07-10 16:15:00','orbital','success',680,'65014-107','Kosmos-500','Tselina-OM','','Sat',457,'',null from dual union all
select 27986,'1972-054','1972-07-12 05:59:57','orbital','success',678,'','Kosmos-501','DS-P1-Yu No. 50','','Sat',627,'',null from dual union all
select 27987,'1972-055','1972-07-13 14:30:25','orbital','success',1225,'Kh15000-002','Kosmos-502','Zenit-4MT','','Sat',475,'',null from dual union all
select 27988,'1972-056','1972-07-19 13:45:00','orbital','success',1508,'78033  -267','Kosmos-503','Zenit-4M','','Sat',474,'',null from dual union all
select 27989,'1972-057','1972-07-20 18:10:00','orbital','success',680,'47127-112','Kosmos-504/511','Strela-1M','','Sat',457,'',null from dual union all
select 2799,'1954-A90','1954-03-03 00:00:00','atmospheric rocket','unknown',576,'381-O','','','','Test',1895,'',10 from dual union all
select 27990,'1972-058','1972-07-23 18:06:06','orbital','success',374,'574/D89','ERTS 1','ERTS A','','Sat',1724,'',null from dual union all
select 27991,'1972-059','1972-07-28 10:19:00','orbital','success',1508,'7702421-143','Kosmos-512','Zenit-2M','','Sat',475,'',null from dual union all
select 27992,'1972-060','1972-08-02 08:15:00','orbital','success',1508,'78033  -266','Kosmos-513','Zenit-4M','','Sat',562,'',null from dual union all
select 27993,'1972-061','1972-08-13 15:10:00','orbital','success',1131,'S184C','Explorer 46','Meteoroid Technology Sat','','Sat',1814,'',null from dual union all
select 27994,'1972-062','1972-08-16 13:40:01','orbital','success',680,'Yu149-38','Kosmos-514','Tsiklon GVM','','Sat',457,'',null from dual union all
select 27995,'1972-063','1972-08-18 10:00:01','orbital','success',1508,'7702424-718','Kosmos-515','Zenit-4MK','','Sat',475,'',null from dual union all
select 27996,'1972-064','1972-08-19 02:40:00','orbital','success',821,'M-4S-4','Denpa','REXS','','Sat',835,'',null from dual union all
select 27997,'1972-065','1972-08-21 10:28:02','orbital','success',179,'AC-22','OAO 3','OAO-C','','Sat',269,'',null from dual union all
select 27998,'1972-066','1972-08-21 10:36:20','orbital','success',1446,'45014-19L','Kosmos-516','US-A reactor','','Sat',590,'',null from dual union all
select 27999,'1972-067','1972-08-30 08:19:59','orbital','success',1508,'7702421-144','Kosmos-517','Zenit-2M','','Sat',562,'',null from dual;
insert into LAUNCH
select 28,'1943-S06','1943-05-27 00:00:00','military missile','success',8,'24','','','','Test',675,'',95 from dual union all
select 280,'1944-S254','1944-11-20 15:48:00','military missile','success',1470,'19552','Ma294','','','Test',661,'',90 from dual union all
select 2800,'1954-A91','1954-03-03 00:00:00','atmospheric rocket','unknown',576,'380-O','','','','Test',1895,'',10 from dual union all
select 28000,'1972-068','1972-09-01 17:36:00','orbital','success',1403,'24B-5 (3B-39)','KH-8A','GAMBIT-3 Mission 4336','','Sat',1778,'',null from dual union all
select 28001,'1972-069','1972-09-02 17:50:30','orbital','success',1129,'S182C','Triad','Triad OI-1X','','Sat',1779,'',null from dual union all
select 28002,'1972-070','1972-09-15 09:40:00','orbital','success',1508,'78033-  156','Kosmos-518','Zenit-2M','','Sat',475,'',null from dual union all
select 28003,'1972-071','1972-09-16 08:20:00','orbital','success',1508,'78033  -268','Kosmos-519','Zenit-4M','','Sat',562,'',null from dual union all
select 28004,'1972-072','1972-09-19 19:19:03','orbital','success',809,'S 15000-100','Kosmos-520','Oko','','Sat',473,'',null from dual union all
select 28005,'1972-073','1972-09-23 01:20:00','orbital','success',376,'579/D90','Explorer 47','IMP H','','Sat',234,'',null from dual union all
select 28006,'1972-074','1972-09-29 20:18:59','orbital','success',680,'Yu47121-11','Kosmos-521','DS-P1-M No. 4','','Sat',457,'',null from dual union all
select 28007,'1972-075','1972-09-30 20:19:01','orbital','success',809,'S 15000-106','Molniya-2','Molniya-2','','Sat',473,'',null from dual union all
select 28008,'1972-076','1972-10-02 20:10:00','orbital','success',148,'102F','P72-1','Radcat','','Sat',1686,'',null from dual union all
select 28009,'1972-077','1972-10-04 12:00:02','orbital','success',1508,'78033-  272','Kosmos-522','Zenit-4M','','Sat',473,'',null from dual union all
select 2801,'1954-A92','1954-03-03 00:00:00','atmospheric rocket','success',836,'1311R','Nike 266R','','','Test',1895,'',10 from dual union all
select 28010,'1972-078','1972-10-05 11:30:00','orbital','success',678,'64657-135','Kosmos-523','DS-P1-Yu No. 63','','Sat',460,'',null from dual union all
select 28011,'1972-079','1972-10-10 18:03:00','orbital','success',1426,'23D-3','HEXAGON SV4','HEXAGON SV-4','','Sat',1777,'',null from dual union all
select 28012,'1972-080','1972-10-11 13:19:58','orbital','success',678,'64634-147','Kosmos-524','DS-P1-Yu No. 49','','Sat',460,'',null from dual union all
select 28013,'1972-081','1972-10-14 06:16:00','orbital','success',809,'7705711-498','Molniya-1','Molniya-1 11F67 No. 26','','Sat',473,'',null from dual union all
select 28014,'1972-082','1972-10-15 17:19:19','orbital','success',373,'575/D91','NOAA 2','ITOS D','','Sat',1724,'',null from dual union all
select 28015,'1972-083','1972-10-18 11:59:59','orbital','success',1508,'76043-  161','Kosmos-525','Zenit-2M','','Sat',475,'',null from dual union all
select 28016,'1972-084','1972-10-25 10:39:57','orbital','success',678,'64657-136','Kosmos-526','DS-P1-Yu No. 61','','Sat',460,'',null from dual union all
select 28017,'1972-085','1972-10-26 22:05:01','orbital','success',1510,'Kh15000-006','Meteor','Meteor-M  No. 25','','Sat',475,'',null from dual union all
select 28018,'1972-086','1972-10-31 13:29:59','orbital','success',1508,'78033-  729','Kosmos-527','Zenit-4MK','','Sat',475,'',null from dual union all
select 28019,'1972-087','1972-11-01 02:08:00','orbital','success',680,'Yu149-46','Kosmos-528/535','Strela-1M','','Sat',457,'',null from dual union all
select 2802,'1954-A93','1954-03-04 00:00:00','atmospheric rocket','unknown',576,'292-T','','','','Test',1895,'',10 from dual union all
select 28020,'1972-088','1972-11-03 01:34:00','orbital','success',680,'Yu47121-1U','Kosmos-536','Tselina-OM','','Sat',456,'',null from dual union all
select 28021,'1972-089','1972-11-09 03:23:46','orbital','success',1365,'294/BIIA-16','DMSP 6530','DMSP Block 5B F-3','','Sat',1720,'',null from dual union all
select 28022,'1972-090','1972-11-10 01:14:03','orbital','success',380,'580/D92','Anik A1','Anik A1','','Sat',234,'',null from dual union all
select 28023,'1972-091','1972-11-15 22:13:46','orbital','success',1131,'S170CR','Explorer 48','SAS B','','Sat',1412,'',null from dual union all
select 28024,'1972-092','1972-11-22 00:17:01','orbital','success',1131,'S185C','ESRO 4','ESRO 4','','Sat',1779,'',null from dual union all
select 28025,'1972-093','1972-11-25 09:10:01','orbital','success',1508,'78033  -157','Kosmos-537','Zenit-2M','','Sat',562,'',null from dual union all
select 28026,'1972-094','1972-11-30 21:49:57','orbital','success',678,'29524-155','Interkosmos-8','DS-U1-IK No. 2','','Sat',460,'',null from dual union all
select 28027,'1972-095','1972-12-02 04:39:57','orbital','success',809,'7801421-512','Molniya-1','Molniya-1 11F67 No. 28','','Sat',480,'',null from dual union all
select 28028,'1972-096','1972-12-07 05:33:00','deep space','success',1123,'SA-512','Apollo 17','Apollo CM-114','','Sat',921,'LUT3',null from dual union all
select 28029,'1972-097','1972-12-11 07:56:00','orbital','success',374,'577/D93','Nimbus 5','Nimbus E','','Sat',1724,'',null from dual union all
select 2803,'1954-A94','1954-03-04 00:00:00','atmospheric rocket','unknown',576,'293-T','','','','Test',1895,'',10 from dual union all
select 28030,'1972-098','1972-12-12 06:51:01','orbital','success',809,'76043-  537','Molniya-2','Molniya-2','','Sat',473,'',null from dual union all
select 28031,'1972-099','1972-12-14 13:40:02','orbital','success',1508,'76043-  274','Kosmos-538','Zenit-4M','','Sat',475,'',null from dual union all
select 28032,'1972-100','1972-12-16 11:24:56','orbital','success',1131,'S181C','Aeros','GRS-B','','Sat',1779,'',null from dual union all
select 28033,'1972-101','1972-12-20 22:20:00','orbital','success',178,'5204A','AFP-827','CANYON 5','','Sat',228,'',null from dual union all
select 28034,'1972-102','1972-12-21 02:05:00','orbital','success',680,'Yu149-43','Kosmos-539','Sfera','','Sat',457,'',null from dual union all
select 28035,'1972-103','1972-12-21 17:45:00','orbital','success',1403,'24B-6 (3B-40)','KH-8A','GAMBIT-3 Mission 4337','','Sat',1778,'',null from dual union all
select 28036,'1972-104','1972-12-25 23:05:01','orbital','success',680,'53724-113','Kosmos-540','Strela-2M','','Sat',457,'',null from dual union all
select 28037,'1972-105','1972-12-27 10:30:04','orbital','success',1225,'S 15000-003','Kosmos-541','Zenit-4MT','','Sat',473,'',null from dual union all
select 28038,'1972-106','1972-12-28 11:00:00','orbital','success',1510,'S 15000-012','Kosmos-542','Tselina-D','','Sat',475,'',null from dual union all
select 28039,'1972-A01','1972-01-01 00:00:00','suborbital rocket','failure',27,'','','','','Aeron',1833,'',0 from dual union all
select 2804,'1954-A95','1954-03-04 00:00:00','atmospheric rocket','unknown',576,'291-T','','','','Test',1895,'',10 from dual union all
select 28040,'1972-A02','1972-06-23 00:00:00','military missile','failure',1452,'','15A30 BI-2','','','Test',505,'',0 from dual union all
select 28041,'1972-A03','1972-01-05 13:14:00','suborbital rocket','failure',1188,'SK30T/C','ISRO-16.14','TERLS-399','','Ionospheric Ba',1467,'',2 from dual union all
select 28042,'1972-A04','1972-01-06 00:00:00','military missile','success',1252,'FLA-61 Sim targ','SAFEGUARD M2-4','','','Interceptor',868,'',5 from dual union all
select 28043,'1972-A05','1972-01-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',49 from dual union all
select 28044,'1972-A06','1972-01-18 00:37:00','suborbital rocket','success',1188,'SK31T/C','ISRO-16.15','TERLS-405','','Ionospheric Ba',1467,'',40 from dual union all
select 28045,'1972-A07','1972-01-18 05:35:00','suborbital rocket','failure',920,'P88T/C','ISRO 18.05','TERLS-406','','Ionospheric',1467,'',23 from dual union all
select 28046,'1972-A08','1972-01-19 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 28047,'1972-A09','1972-01-26 16:13:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',42 from dual union all
select 28048,'1972-A10','1972-01-27 15:39:00','suborbital rocket','success',230,'','F-EXM-064','','','Test',428,'',38 from dual union all
select 28049,'1972-A100','1972-02-09 00:00:00','military missile','success',1452,'','15A30 BI-1','Silo eject test','','Test',505,'',1 from dual union all
select 2805,'1954-A96','1954-03-04 00:00:00','atmospheric rocket','unknown',576,'294-T','','','','Test',1895,'',10 from dual union all
select 28050,'1972-A101','1972-09-20 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',23 from dual union all
select 28051,'1972-A102','1972-09-20 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',49 from dual union all
select 28052,'1972-A103','1972-10-04 17:25:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',45 from dual union all
select 28053,'1972-A104','1972-10-10 14:00:00','suborbital rocket','failure',493,'1','DLR C-FFK-45','','','Test',307,'',0 from dual union all
select 28054,'1972-A105','1972-10-11 00:00:00','military missile','failure',1252,'FLA-52','SAFEGUARD M2-14','','','Interceptor',868,'',1 from dual union all
select 28055,'1972-A106','1972-10-11 13:13:00','suborbital rocket','failure',1116,'Sandia 152-192','Ba FAISON','','','Aeron/Fields',1047,'',0 from dual union all
select 28056,'1972-A107','1972-10-13 03:39:00','suborbital rocket','failure',837,'NASA 14.506CA','GCA Release TMA','','','Aeronomy',1047,'',10 from dual union all
select 28057,'1972-A108','1972-10-19 03:37:00','military missile','failure',1004,'45023-301','15A14 BI-4-1','GCh 15F141','','Test',489,'',10 from dual union all
select 28058,'1972-A109','1972-10-24 15:00:00','suborbital rocket','failure',493,'2','DLR C-FFK-46','','','Test',307,'',0 from dual union all
select 28059,'1972-A11','1972-01-27 18:32:00','suborbital rocket','success',230,'','F-EXM-065','','','Test',428,'',34 from dual union all
select 2806,'1954-A97','1954-03-05 00:00:00','atmospheric rocket','unknown',576,'295-T','','','','Test',1895,'',10 from dual union all
select 28060,'1972-A110','1972-11-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',47 from dual union all
select 28061,'1972-A111','1972-11-25 03:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',49 from dual union all
select 28062,'1972-A112','1972-11-29 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 28063,'1972-A113','1972-11-29 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',43 from dual union all
select 28064,'1972-A114','1972-12-01 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 28065,'1972-A115','1972-12-05 00:00:00','military missile','success',1252,'FLA-58','SAFEGUARD M2-13','','','Interceptor',866,'',10 from dual union all
select 28066,'1972-A116','1972-12-06 06:35:00','suborbital rocket','failure',132,'CRL A30.205-1','Auroral Quiet IR','','','Ionos/Aeron',428,'',27 from dual union all
select 28067,'1972-A117','1972-12-08 00:00:00','military missile','success',1252,'FLA-76','Safeguard M2-21','','','Interceptor',868,'',10 from dual union all
select 28068,'1972-A118','1972-12-18 22:55:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 28069,'1972-A119','1972-12-21 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',44 from dual union all
select 2807,'1954-A98','1954-03-05 00:00:00','atmospheric rocket','unknown',576,'296-T','','','','Test',1895,'',10 from dual union all
select 28070,'1972-A12','1972-01-27 19:36:00','suborbital rocket','success',230,'','F-EXM-056','','','Test',428,'',34 from dual union all
select 28071,'1972-A121','1972-12-28 00:18:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 28072,'1972-A122','1972-12-29 23:44:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 28073,'1972-A123','1972-12-29 07:30:00','military missile','failure',1004,'67023-302','15A14 BI-4-2','GCh 15F141','','Test',489,'',100 from dual union all
select 28074,'1972-A124','1972-02-03 20:17:00','atmospheric rocket','success',1346,'Sandia','SAMS Vehicle 3','Nose rain erosion','','Tech',1794,'',3 from dual union all
select 28075,'1972-A125','1972-02-17 15:12:00','atmospheric rocket','success',1346,'Sandia','SAMS Vehicle 4','','','Tech',1795,'',3 from dual union all
select 28076,'1972-A126','1972-03-17 21:20:38','atmospheric rocket','success',1346,'Sandia','SAMS Vehicle 7','Nose rain erosion','','Test',1794,'',3 from dual union all
select 28077,'1972-A127','1972-03-22 15:48:00','atmospheric rocket','success',1346,'Sandia','SAMS Vehicle 5','','','Tech',1795,'',3 from dual union all
select 28078,'1972-A128','1972-08-16 15:02:00','atmospheric rocket','success',1346,'','Nose rain erosion','','','Test',1821,'',10 from dual union all
select 28079,'1972-A129','1972-05-01 00:00:00','military missile','failure',1514,'R-29','','','','Test',190,'K-145',0 from dual union all
select 2808,'1954-A99','1954-03-05 00:00:00','military missile','success',771,'GM-2360','','','','Test',222,'',12 from dual union all
select 28080,'1972-A13','1972-01-27 21:48:00','suborbital rocket','success',230,'','F-EXM-052','','','Test',428,'',40 from dual union all
select 28081,'1972-A130','1972-10-02 00:00:00','suborbital rocket','success',1502,'Stable Viper 1-7','Robin Dart','','','Test',1821,'',12 from dual union all
select 28082,'1972-A131','1972-05-18 02:30:00','suborbital rocket','failure',675,'WRE K1065','','','','Aeron',1832,'',20 from dual union all
select 28083,'1972-A132','1972-02-17 14:56:00','atmospheric rocket','success',1346,'Sandia','SAMS Vehicle 6','','','Tech',1794,'',3 from dual union all
select 28084,'1972-A133','1972-02-23 01:00:00','suborbital rocket','success',716,'HASP','WRE HASP 66','','','Meteo',1849,'',30 from dual union all
select 28085,'1972-A134','1972-08-22 01:00:00','suborbital rocket','success',716,'HASP','WRE HASP 88','','','Meteo',1849,'',30 from dual union all
select 28086,'1972-A135','1972-07-11 17:37:00','atmospheric rocket','failure',199,'AV-1','NASA BLDT AV-1','Supersonic','','Test',1895,'',44 from dual union all
select 28087,'1972-A136','1972-07-26 16:55:00','atmospheric rocket','success',199,'AV-2','NASA BLDT AV-2','Transonic','','Test',1895,'',41 from dual union all
select 28088,'1972-A137','1972-08-13 16:24:30','atmospheric rocket','success',199,'AV-4','NASA BLDT AV-4','Supersonic','','Test',1895,'',45 from dual;
insert into LAUNCH
select 28089,'1972-A138','1972-08-19 17:45:00','atmospheric rocket','success',199,'AV-3','NASA BLDT AV-3','Subsonic','','Test',1895,'',28 from dual union all
select 2809,'1954-M01','1954-07-21 09:03:00','suborbital rocket','success',370,'','SUI 27','','','Ionos/Aeron?',73,'WYG-279',60 from dual union all
select 28090,'1972-A14','1972-01-28 20:09:00','suborbital rocket','failure',837,'NASA 14.419UA','','','','Aeronomy',1895,'',4 from dual union all
select 28091,'1972-A15','1972-02-02 15:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',49 from dual union all
select 28092,'1972-A16','1972-02-18 12:38:00','suborbital rocket','success',920,'P117H','E-region multiprobe','','','Ionospheric',1447,'',25 from dual union all
select 28093,'1972-A17','1972-02-21 11:50:00','military missile','success',744,'67014-03','15A15 BI-2-3','','','Test',573,'',1 from dual union all
select 28094,'1972-A18','1972-02-23 17:55:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',46 from dual union all
select 28095,'1972-A19','1972-03-01 00:00:00','military missile','success',1466,'','','','','Test',1895,'',20 from dual union all
select 28096,'1972-A20','1972-02-29 18:45:00','atmospheric rocket','success',835,'','Propellant dynamics','','','Test',1821,'',11 from dual union all
select 28097,'1972-A21','1972-03-06 05:15:00','military missile','success',1004,'51014-02','15A14 BI-3-2','','','Test',573,'',10 from dual union all
select 28098,'1972-A22','1972-03-06 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',45 from dual union all
select 28099,'1972-A23','1972-03-16 00:00:00','military missile','success',1252,'FLA-63','Safeguard M2-7','','','Interceptor',866,'',5 from dual union all
select 281,'1944-S255','1944-11-21 08:48:00','military missile','success',1470,'19551','Ma296','','','Test',661,'',90 from dual union all
select 2810,'1954-M02','1954-10-14 22:12:00','suborbital rocket','success',857,'','F40 Heat Transfer','','','Test',1821,'',50 from dual union all
select 28100,'1972-A24','1972-03-17 12:14:59','suborbital rocket','success',1061,'','Starute','','','Meteo',615,'',48 from dual union all
select 28101,'1972-A25','1972-03-21 13:04:00','military missile','success',744,'45023-304','15A15 BI-2-4','','','Test',573,'',1 from dual union all
select 28102,'1972-A26','1972-03-22 01:21:38','suborbital rocket','failure',839,'Ferdinand 28','Ferdinand 28','','','Test/Aeron/Ionos',29,'',24 from dual union all
select 28103,'1972-A27','1972-03-28 12:12:00','suborbital rocket','failure',905,'','','','','Auroral',1047,'',0 from dual union all
select 28104,'1972-A28','1972-03-29 17:45:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',41 from dual union all
select 28105,'1972-A29','1972-03-29 20:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',45 from dual union all
select 28106,'1972-A30','1972-04-12 16:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',47 from dual union all
select 28107,'1972-A31','1972-04-14 21:39:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 28108,'1972-A32','1972-04-19 13:35:59','suborbital rocket','success',716,'INTA HD-7209','','','','Meteo',396,'',48 from dual union all
select 28109,'1972-A33','1972-04-19 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',48 from dual union all
select 2811,'1954-S01','1954-01-27 15:20:00','military missile','success',1053,'RS-2     XU','','','','Test',272,'',90 from dual union all
select 28110,'1972-A34','1972-04-19 21:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',46 from dual union all
select 28111,'1972-A35','1972-04-21 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 28112,'1972-A36','1972-04-21 20:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',48 from dual union all
select 28113,'1972-A37','1972-04-24 14:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',40 from dual union all
select 28114,'1972-A38','1972-04-24 22:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',45 from dual union all
select 28115,'1972-A39','1972-04-26 13:00:59','suborbital rocket','success',716,'INTA HD-7210','','','','Meteo',396,'',39 from dual union all
select 28116,'1972-A40','1972-04-29 07:10:00','military missile','success',1004,'67014-03','15A14 BI-3-3','','','Test',573,'',10 from dual union all
select 28117,'1972-A41','1972-05-01 00:00:00','military missile','success',1466,'','','','','Test',1895,'',20 from dual union all
select 28118,'1972-A42','1972-05-02 16:56:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',49 from dual union all
select 28119,'1972-A43','1972-05-03 14:35:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',49 from dual union all
select 2812,'1954-S02','1954-02-02 18:35:00','suborbital rocket','success',54,'NRL 20','Sunfollower UVS','','','Solar UV',1869,'',101 from dual union all
select 28120,'1972-A44','1972-05-03 15:48:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',47 from dual union all
select 28121,'1972-A45','1972-05-05 00:00:00','military missile','success',1252,'FLA-64','Safeguard M2-11','','','Interceptor',868,'',5 from dual union all
select 28122,'1972-A46','1972-05-05 18:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',47 from dual union all
select 28123,'1972-A47','1972-05-05 19:09:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',46 from dual union all
select 28124,'1972-A48','1972-05-06 07:10:00','military missile','success',744,'45023-301','15A15 BI-3-1','','','Test',573,'',10 from dual union all
select 28125,'1972-A49','1972-05-10 16:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',48 from dual union all
select 28126,'1972-A50','1972-05-11 14:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',45 from dual union all
select 28127,'1972-A51','1972-05-11 19:25:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',48 from dual union all
select 28128,'1972-A52','1972-05-12 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',45 from dual union all
select 28129,'1972-A53','1972-05-15 14:45:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',49 from dual union all
select 2813,'1954-S03','1954-02-21 00:00:00','suborbital rocket','success',1490,'NA14','','','','Test',666,'',135 from dual union all
select 28130,'1972-A54','1972-05-15 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',47 from dual union all
select 28131,'1972-A55','1972-05-17 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',46 from dual union all
select 28132,'1972-A56','1972-05-18 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',47 from dual union all
select 28133,'1972-A57','1972-05-19 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',49 from dual union all
select 28134,'1972-A58','1972-05-24 11:30:00','military missile','success',744,'45023-302','15A15 BI-3-2','','','Test',573,'',10 from dual union all
select 28135,'1972-A59','1972-05-24 00:00:00','military missile','success',1252,'FLA-59','Safeguard M2-10','','','Interceptor',866,'',5 from dual union all
select 28136,'1972-A60','1972-05-24 18:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',46 from dual union all
select 28137,'1972-A61','1972-05-31 15:49:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',49 from dual union all
select 28138,'1972-A62','1972-06-06 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',49 from dual union all
select 28139,'1972-A63','1972-06-08 14:30:00','military missile','success',1004,'45023-304','15A14 BI-3-4','','','Test',573,'',10 from dual union all
select 2814,'1954-S04','1954-03-11 00:00:00','military missile','success',970,'No 1820','','','','OT',636,'',100 from dual union all
select 28140,'1972-A64','1972-06-09 00:00:00','military missile','success',1251,'','Isolated Rail Test','','','Test',868,'',1 from dual union all
select 28141,'1972-A65','1972-06-15 11:20:00','military missile','success',744,'51023-303','15A15 BI-3-3','','','Test',573,'',10 from dual union all
select 28142,'1972-A66','1972-06-20 16:45:00','suborbital rocket','failure',37,'CRL A04.116-3','CHASER','','','ABM Tech IR',1768,'',8 from dual union all
select 28143,'1972-A67','1972-06-22 00:00:00','atmospheric rocket','success',1065,'Ba5','Badger 5','','','Test',2,'',20 from dual union all
select 28144,'1972-A68','1972-06-22 14:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',31 from dual union all
select 28145,'1972-A69','1972-06-23 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',43 from dual union all
select 28146,'1972-A70','1972-06-28 17:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',49 from dual union all
select 28147,'1972-A71','1972-07-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',30 from dual union all
select 28148,'1972-A72','1972-07-12 22:30:00','suborbital rocket','failure',229,'','MDSS FP-022','','','Test',428,'',14 from dual union all
select 28149,'1972-A73','1972-07-13 00:13:00','suborbital rocket','failure',229,'','MDSS FP-023','','','Test',428,'',7 from dual union all
select 2815,'1954-S05','1954-03-16 00:00:00','military missile','success',970,'No 1916','','','','OT',636,'',100 from dual union all
select 28150,'1972-A74','1972-07-13 18:54:00','suborbital rocket','failure',229,'','MDSS FP-028','','','Test',428,'',16 from dual union all
select 28151,'1972-A75','1972-07-14 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',46 from dual union all
select 28152,'1972-A76','1972-07-15 00:00:00','military missile','success',1252,'FLA-53','Safeguard M2-15','','','Interceptor',868,'',26 from dual union all
select 28153,'1972-A77','1972-07-15 17:12:00','suborbital rocket','failure',227,'CRL A18.903-4','RM-52 EUV','','','Aeron/Solar UV',394,'',7 from dual union all
select 28154,'1972-A78','1972-07-19 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',38 from dual union all
select 28155,'1972-A79','1972-07-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',29 from dual union all
select 28156,'1972-A80','1972-07-20 00:00:00','suborbital rocket','success',1210,'SL1183','RAE Test 43 (Dummy)','','','Test',2,'',5 from dual union all
select 28157,'1972-A81','1972-07-27 00:00:00','military missile','unknown',255,'A','Target','','','Target',1689,'',20 from dual union all
select 28158,'1972-A82','1972-07-27 00:00:00','military missile','unknown',255,'A','Target','','','Target',1690,'',20 from dual union all
select 28159,'1972-A83','1972-08-01 00:00:00','military missile','success',1466,'','Jet Interaction','','','Test',1895,'',20 from dual union all
select 2816,'1954-S06','1954-03-16 00:00:00','military missile','success',970,'No 1920','','','','OT',636,'',100 from dual union all
select 28160,'1972-A84','1972-08-01 00:00:00','military missile','success',1466,'','','','','Test',1895,'',20 from dual union all
select 28161,'1972-A85','1972-08-05 05:26:00','suborbital rocket','failure',837,'NASA 14.490GM','Solar cosmic rays','','','Plasma',428,'',10 from dual union all
select 28162,'1972-A86','1972-08-14 18:43:00','suborbital rocket','unknown',716,'T 1-6097','','','','Aeron',1821,'',35 from dual union all
select 28163,'1972-A87','1972-08-16 13:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',41 from dual union all
select 28164,'1972-A88','1972-08-16 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',27 from dual union all
select 28165,'1972-A89','1972-08-17 00:00:00','military missile','failure',1252,'FLA-70','SAFEGUARD M2-42','','','Interceptor',868,'',0 from dual union all
select 28166,'1972-A90','1972-08-18 00:00:00','military missile','success',1004,'67023-305','15A14 BI-3-5','','','Test',573,'',10 from dual union all
select 28167,'1972-A91','1972-08-24 00:00:00','military missile','success',1252,'FLA-66','SAFEGUARD M2-42A','','','Interceptor',868,'',10 from dual union all
select 28168,'1972-A92','1972-08-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',48 from dual union all
select 28169,'1972-A93','1972-08-30 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',24 from dual union all
select 2817,'1954-S07','1954-03-20 00:00:00','military missile','success',970,'No 1918','','','','OT',636,'',100 from dual union all
select 28170,'1972-A94','1972-09-01 00:00:00','suborbital rocket','failure',716,'','','','','Meteo',428,'',10 from dual union all
select 28171,'1972-A95','1972-09-06 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',23 from dual union all
select 28172,'1972-A96','1972-09-06 16:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',48 from dual union all
select 28173,'1972-A97','1972-09-11 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',48 from dual union all
select 28174,'1972-A98','1972-09-13 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',48 from dual union all
select 28175,'1972-A99','1972-09-13 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',0 from dual union all
select 28176,'1972-F01','1972-02-16 09:59:00','orbital','failure',1404,'33B-2 (3B-37)','OPS 1844','Jumpseat 2','','Sat',1778,'',400 from dual union all
select 28177,'1972-F02','1972-04-25 11:29:59','orbital','failure',678,'64634-146','','DS-P1-Yu No. 51','','Sat',460,'',270 from dual union all
select 28178,'1972-F03','1972-05-20 15:30:00','orbital','failure',1403,'24B-4 (3B-35)','KH-8A','GAMBIT-3 Mission 4335','','Sat',1778,'',197 from dual;
insert into LAUNCH
select 28179,'1972-F04','1972-07-29 03:20:57','orbital','failure',949,'260-01','','DOS 2 (17K No. 122)','','Sat',587,'',90 from dual union all
select 2818,'1954-S08','1954-04-09 21:12:00','suborbital rocket','success',54,'NRL 18','Mass spectrometer','','','Aeronomy',1869,'',143 from dual union all
select 28180,'1972-F05','1972-09-02 10:49:59','orbital','failure',1508,'78033-  271','','Zenit-4M','','Sat',475,'',50 from dual union all
select 28181,'1972-F06','1972-10-17 19:59:01','orbital','failure',680,'Yu149-40','','Strela-2M','','Sat',456,'',800 from dual union all
select 28182,'1972-F07','1972-11-23 06:11:55','orbital','failure',823,'7L/Kh15007','','Mockup LK','','Sat',498,'',40 from dual union all
select 28183,'1972-M01','1972-01-31 13:22:00','suborbital rocket','success',1189,'SK32T/C','ISRO-16.16','TERLS-409','','Ionospheric Ba',1467,'',79 from dual union all
select 28184,'1972-M02','1972-03-05 14:35:00','suborbital rocket','failure',1116,'DLR T-DH-43','ISRO 21.01','TERLS-420 Dual Hawk','','Ionos',1467,'',52 from dual union all
select 28185,'1972-M03','1972-03-11 14:38:00','suborbital rocket','failure',1116,'DLT T-DH-44','ISRO 21.02','TERLS-424 Dual Hawk','','Ionos',1467,'',52 from dual union all
select 28186,'1972-M04','1972-03-29 20:24:00','suborbital rocket','success',263,'','','','','Particles',1047,'',50 from dual union all
select 28187,'1972-M05','1972-06-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 28188,'1972-M06','1972-06-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 28189,'1972-M07','1972-06-23 00:00:00','military missile','success',1251,'800026','SAFEGUARD M2-19','','','Interceptor',868,'',70 from dual union all
select 2819,'1954-S09','1954-04-20 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 28190,'1972-M08','1972-11-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 28191,'1972-M09','1972-11-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 28192,'1972-M10','1972-11-03 00:00:00','military missile','success',1251,'800036','SAFEGUARD M2-40','','','Interceptor',868,'',50 from dual union all
select 28193,'1972-M11','1972-11-05 23:45:00','suborbital rocket','success',867,'CRL A08.105-10','Release','','','Aeron',394,'',62 from dual union all
select 28194,'1972-M12','1972-12-09 01:30:00','suborbital rocket','success',132,'CRL A30.205-2','Auroral Quiet IR','','','Ionos/Aeron',428,'',70 from dual union all
select 28195,'1972-M13','1972-11-15 20:38:00','suborbital rocket','success',1506,'T 1-6150','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 28196,'1972-M14','1972-11-17 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-15','','','Meteo',350,'',60 from dual union all
select 28197,'1972-M15','1972-12-18 19:00:00','suborbital rocket','success',1506,'T 1-6261','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 28198,'1972-M16','1972-10-03 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-13','','','Meteo',350,'',75 from dual union all
select 28199,'1972-M17','1972-07-19 16:04:00','suborbital rocket','success',1506,'T 1-5798','Starute,Datasonde','T1-5798','','Meteo',1821,'',60 from dual union all
select 282,'1944-S256','1944-11-21 10:00:00','military missile','success',1470,'19546','Ma297','','','Test',661,'',90 from dual union all
select 2820,'1954-S10','1954-04-23 00:00:00','military missile','success',970,'No 1720','','','','OT',636,'',100 from dual union all
select 28200,'1972-M18','1972-01-12 22:18:00','suborbital rocket','success',716,'INTA HD-7201','','','','Meteo',396,'',60 from dual union all
select 28201,'1972-M19','1972-01-22 01:53:00','suborbital rocket','success',716,'INTA HD-7203','','','','Meteo',396,'',60 from dual union all
select 28202,'1972-M20','1972-05-24 12:06:00','suborbital rocket','success',716,'INTA HT-7201','','','','Meteo',396,'',60 from dual union all
select 28203,'1972-M21','1972-09-15 12:35:00','suborbital rocket','success',716,'INTA HW-7213','','','','Meteo',396,'',60 from dual union all
select 28204,'1972-M22','1972-09-20 12:04:00','suborbital rocket','success',716,'INTA HW-7214','','','','Meteo',396,'',60 from dual union all
select 28205,'1972-M23','1972-09-27 10:06:00','suborbital rocket','success',716,'INTA HW-7215','','','','Meteo',396,'',60 from dual union all
select 28206,'1972-M24','1972-07-11 15:23:00','suborbital rocket','success',88,'T1-5999','','','','Aeron',1821,'',60 from dual union all
select 28207,'1972-M25','1972-03-20 20:32:00','suborbital rocket','success',761,'','','','','Aeron',1821,'',50 from dual union all
select 28208,'1972-M26','1972-03-16 15:16:00','suborbital rocket','success',761,'','','','','Aeron',1821,'',50 from dual union all
select 28209,'1972-S01','1972-01-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',137,'ZUB',150 from dual union all
select 2821,'1954-S100','1954-10-01 00:00:00','military missile','success',992,'K1-10','','','','OT',636,'',150 from dual union all
select 28210,'1972-S02','1972-01-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 28211,'1972-S03','1972-01-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 28212,'1972-S04','1972-01-01 00:00:00','military missile','success',752,'','','','','OT',307,'S612',500 from dual union all
select 28213,'1972-S05','1972-02-19 00:00:00','military missile','success',982,'G 22502-15T','8K64U UBP-163','','','OT',596,'',1210 from dual union all
select 28214,'1972-S06','1972-04-11 00:00:00','military missile','success',982,'G 22504-06T','8K64U UBP-165','','','OT',596,'',1210 from dual union all
select 28215,'1972-S07','1972-08-18 00:00:00','military missile','success',982,'G 22503-14T','8K64U UBP-167','','','OT',596,'',1210 from dual union all
select 28216,'1972-S08','1972-01-20 00:00:00','military missile','success',982,'G 22502-16T','8K64U UBP-161','','','OT',596,'',1210 from dual union all
select 28217,'1972-S09','1972-02-09 00:00:00','military missile','success',982,'G 22503-04T','8K64U UBP-162','','','OT',596,'',1210 from dual union all
select 28218,'1972-S10','1972-03-04 00:00:00','military missile','success',982,'G 22502-14T','8K64U UBP-164','','','OT',596,'',1210 from dual union all
select 28219,'1972-S100','1972-03-06 16:30:00','military missile','success',931,'A3T','DASO','','','OT',421,'UK S22',1000 from dual union all
select 2822,'1954-S101','1954-10-05 00:00:00','military missile','success',992,'K2-20','','','','OT',636,'',150 from dual union all
select 28220,'1972-S101','1972-03-06 12:14:00','suborbital rocket','success',132,'CRL A30.205-3','IR emission','','','Aeron',1047,'',86 from dual union all
select 28221,'1972-S102','1972-03-07 05:16:00','suborbital rocket','success',867,'NASA 18.141GE','','','','Plasma',1047,'',270 from dual union all
select 28222,'1972-S103','1972-03-07 06:52:00','suborbital rocket','success',1116,'Sandia 152-188','Ba OOSIK','','','Fields',1047,'',500 from dual union all
select 28223,'1972-S104','1972-03-07 14:26:00','suborbital rocket','success',867,'NASA 18.142GE','','','','Plasma',1047,'',270 from dual union all
select 28224,'1972-S105','1972-03-08 10:32:53','suborbital rocket','success',228,'N-BB/V-41','DLR N-BBV-41','','','Aeron/Ionos',330,'',223 from dual union all
select 28225,'1972-S106','1972-03-08 22:10:00','suborbital rocket','success',740,'MR-12','Polar thermosphere','','','Aeron/CR',1010,'',169 from dual union all
select 28226,'1972-S107','1972-03-09 10:52:00','suborbital rocket','success',132,'CRL A30.205-4','IR emission','','','Aeron',1047,'',90 from dual union all
select 28227,'1972-S108','1972-03-09 12:00:00','suborbital rocket','success',867,'NASA 18.73UE','','','','Plasma',1821,'',243 from dual union all
select 28228,'1972-S109','1972-03-09 14:08:00','suborbital rocket','success',867,'NASA 18.143GE','','','','Plasma',1047,'',270 from dual union all
select 28229,'1972-S11','1972-04-21 00:00:00','military missile','success',982,'G 22504-01T','8K64U UBP-166','','','OT',596,'',1210 from dual union all
select 2823,'1954-S102','1954-10-16 00:00:00','military missile','success',992,'K2-10','','','','OT',636,'',170 from dual union all
select 28230,'1972-S110','1972-03-13 16:00:00','suborbital rocket','success',837,'ISRC-PO-4','','','','Aeron',1412,'',185 from dual union all
select 28231,'1972-S111','1972-03-14 00:00:00','military missile','success',1322,'44654-732-2L','15Zh42 LKI-1','Mobile PU','','Test',461,'',1000 from dual union all
select 28232,'1972-S112','1972-03-14 15:58:00','suborbital rocket','success',837,'ISRC-PO-5','','','','Aeron',1412,'',185 from dual union all
select 28233,'1972-S113','1972-03-15 16:00:00','suborbital rocket','success',837,'ISRC-PO-6','','','','Aeron',1412,'',185 from dual union all
select 28234,'1972-S114','1972-03-16 00:00:00','military missile','success',795,'','SSTTP','SSTTP M2-7','','Target',1701,'',1300 from dual union all
select 28235,'1972-S115','1972-03-16 02:30:00','suborbital rocket','success',1210,'SL1023','UCL-MI2 Midday Sq','','','Ionospheric',1839,'',260 from dual union all
select 28236,'1972-S116','1972-03-16 09:54:00','suborbital rocket','success',1347,'Sandia 281-24','CARIBOU VI','','','Auroral?',1047,'',300 from dual union all
select 28237,'1972-S117','1972-03-16 10:16:00','suborbital rocket','success',226,'CRL A17.110-3','ICECAP 72','','','Plasma,IR Aeron',1047,'',145 from dual union all
select 28238,'1972-S118','1972-03-16 15:43:00','suborbital rocket','success',837,'ISRC-PO-7','','','','Aeron',1412,'',185 from dual union all
select 28239,'1972-S119','1972-03-17 05:10:00','suborbital rocket','success',1211,'SL1009','Cul UVS 19','','','Solar UV',1839,'',237 from dual union all
select 2824,'1954-S103','1954-10-30 00:00:00','military missile','success',970,'D21-14','UBP','','','Test',636,'',100 from dual union all
select 28240,'1972-S12','1972-12-29 01:00:00','military missile','success',744,'51049-202-2L','15A15 LKI-2','GCh 15F161','','Test',542,'',1000 from dual union all
select 28241,'1972-S120','1972-03-17 07:47:35','suborbital rocket','success',30,'NASA 04.320UA','JHU Aurora 9','','','Auroral UV',428,'',162 from dual union all
select 28242,'1972-S121','1972-03-17 09:39:00','suborbital rocket','success',1347,'Sandia 281-25','','','','Auroral?',1047,'',300 from dual union all
select 28243,'1972-S122','1972-03-17 15:31:00','military missile','success',136,'H003','SAMAST-1B','ABRES SAMAST 1B','','Reentry Test',615,'',281 from dual union all
select 28244,'1972-S123','1972-03-17 16:56:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Plasma',1010,'',168 from dual union all
select 28245,'1972-S124','1972-03-17 17:04:00','suborbital rocket','success',740,'MR-12','Ionos','','','Aeron/Plasma',1010,'',189 from dual union all
select 28246,'1972-S125','1972-03-17 19:01:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Plasma',1010,'',173 from dual union all
select 28247,'1972-S126','1972-03-17 20:03:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',176 from dual union all
select 28248,'1972-S127','1972-03-18 18:19:00','suborbital rocket','success',1400,'','ELECTRE','','','Reentry Test',307,'',159 from dual union all
select 28249,'1972-S128','1972-03-18 09:03:00','suborbital rocket','success',1467,'','','','','Auroral/Airglow',1047,'',200 from dual union all
select 2825,'1954-S104','1954-11-27 00:00:00','military missile','success',992,'K3-10','','','','OT',636,'',170 from dual union all
select 28250,'1972-S129','1972-03-20 09:45:00','suborbital rocket','unknown',740,'M-180 (Mar 21)','','','','Aeron',1010,'',192 from dual union all
select 28251,'1972-S13','1972-01-14 00:00:00','military missile','success',1044,'47827-437-17L','8K98P LKI-15','','','Test',477,'',1000 from dual union all
select 28252,'1972-S130','1972-03-22 03:00:00','military missile','success',796,'','ERCS','ERCS GT200L','GLORY TRIP 200L','Comms',1700,'',1300 from dual union all
select 28253,'1972-S131','1972-03-21 08:37:58','suborbital rocket','success',217,'AMF-3A-52','Oxygen recomb','','','Aeron',428,'',119 from dual union all
select 28254,'1972-S132','1972-03-21 13:35:00','suborbital rocket','success',30,'KP3.38','CAL XR 7','','','XR Astron',1869,'',200 from dual union all
select 28255,'1972-S133','1972-03-22 08:22:00','suborbital rocket','success',837,'ISRC-PO-9','','','','Ionos',1412,'',200 from dual union all
select 28256,'1972-S134','1972-03-22 09:33:00','suborbital rocket','success',331,'WRE C1026','Sphere','','','Aeron',1835,'',100 from dual union all
select 28257,'1972-S135','1972-03-24 09:44:00','suborbital rocket','success',227,'AED-VB-28','Nightglow','','','Aeron/Ionos',428,'',193 from dual union all
select 28258,'1972-S136','1972-03-24 23:31:00','suborbital rocket','success',1204,'ESRO S75/1','ESRO S75/1','','','Ionospheric',1343,'',136 from dual union all
select 28259,'1972-S137','1972-03-25 10:15:00','suborbital rocket','success',850,'NRL (Unknown)','NRL Cs Release 1','','','Aeron',1821,'',100 from dual union all
select 2826,'1954-S105','1954-12-01 00:00:00','military missile','success',992,'','Generator-2','','','Test',636,'',170 from dual union all
select 28260,'1972-S138','1972-03-26 23:30:00','suborbital rocket','success',1211,'SL1081','RAE Test 42 (ER1)','','','Test/Earth imaging',1839,'',279 from dual union all
select 28261,'1972-S139','1972-03-27 10:08:00','suborbital rocket','success',850,'NRL (Unknown)','NRL Cs Release 2','','','Aeron',1821,'',100 from dual union all
select 28262,'1972-S14','1972-04-14 02:30:00','suborbital rocket','success',1281,'PCA','AFCRL Arcas 71-A2','','','Ionos',1470,'',100 from dual union all
select 28263,'1972-S140','1972-03-29 10:07:00','suborbital rocket','success',850,'NRL (Unknown)','NRL Cs Release 3','','','Aeron',1821,'',100 from dual union all
select 28264,'1972-S141','1972-03-30 00:00:00','military missile','success',1251,'800033','SAFEGUARD M2-9','','','Interceptor',868,'',130 from dual union all
select 28265,'1972-S142','1972-03-30 01:25:00','suborbital rocket','success',1208,'ESRO S75/2','ESRO S75/2','','','Ionospheric',1343,'',133 from dual union all
select 28266,'1972-S143','1972-04-18 00:00:00','military missile','failure',1322,'1L','15Zh42 LKI-2','','','Test',477,'',0 from dual union all
select 28267,'1972-S144','1972-03-31 03:10:00','suborbital rocket','success',30,'NASA 04.329UA','UM Aeronomy','','','Aeronomy',428,'',181 from dual union all
select 28268,'1972-S145','1972-04-01 03:47:00','suborbital rocket','success',30,'NRL NB3.254','','','','XR Astron?',1869,'',200 from dual;
insert into LAUNCH
select 28269,'1972-S146','1972-04-01 05:05:00','suborbital rocket','success',37,'NASA 13.027CG','','','','XR Astron',1895,'',193 from dual union all
select 2827,'1954-S106','1954-12-06 00:00:00','military missile','success',992,'','Generator-2','','','Test',636,'',170 from dual union all
select 28270,'1972-S147','1972-04-03 10:04:00','suborbital rocket','success',850,'NRL (Unknown)','NRL Cs Release 4','','','Aeron',1821,'',100 from dual union all
select 28271,'1972-S148','1972-04-03 10:06:00','suborbital rocket','success',850,'NRL (Unknown)','NRL Cs Release 5','','','Aeron',1821,'',100 from dual union all
select 28272,'1972-S149','1972-04-05 05:46:00','suborbital rocket','success',228,'CRL A18.109-1','E Field/Particles','','','Plasma/Auroral',428,'',247 from dual union all
select 28273,'1972-S15','1972-04-15 05:16:00','suborbital rocket','success',88,'PCA','AFCRL Arcas 70-6','','','Ionos',1470,'',100 from dual union all
select 28274,'1972-S150','1972-04-05 08:31:00','suborbital rocket','success',740,'MR-12','Polar thermosphere','','','CR',1010,'',150 from dual union all
select 28275,'1972-S151','1972-04-05 23:45:00','suborbital rocket','success',848,'CRL A07.921-3','Release','','','Aeron',394,'',160 from dual union all
select 28276,'1972-S152','1972-04-07 07:00:00','suborbital rocket','success',837,'NASA 14.412IP','ISRO 10.43','TERLS-430','','Ionos',1467,'',147 from dual union all
select 28277,'1972-S153','1972-04-07 14:20:00','suborbital rocket','success',288,'Rehbar 23','','','','Aeron/Ionos',1419,'',150 from dual union all
select 28278,'1972-S154','1972-04-08 14:18:00','suborbital rocket','success',288,'Rehbar 24','','','','Aeron/Ionos',1419,'',150 from dual union all
select 28279,'1972-S155','1972-04-11 00:38:00','suborbital rocket','success',848,'CRL A07.106-4','','Release','','Aeron',394,'',156 from dual union all
select 2828,'1954-S107','1954-12-09 00:00:00','military missile','success',992,'K3-20','','','','OT',636,'',170 from dual union all
select 28280,'1972-S156','1972-04-12 17:25:00','military missile','success',1001,'45014-04L','8K67MA LKI-1','GCh 15F141-500','','Test',518,'',1000 from dual union all
select 28281,'1972-S157','1972-04-13 00:43:00','suborbital rocket','success',848,'CRL A07.921-5','ALADDIN II Release','','','Aeron',394,'',177 from dual union all
select 28282,'1972-S158','1972-04-13 00:00:00','military missile','success',796,'','Mk 11 RV','OT GT88M','GLORY TRIP 88M','OT',1700,'',1300 from dual union all
select 28283,'1972-S159','1972-04-13 02:00:00','suborbital rocket','success',848,'CRL A07.107-1','ALADDIN II NACS MSp.','','','Aeron',394,'',120 from dual union all
select 28284,'1972-S16','1972-12-07 00:00:00','suborbital rocket','success',466,'E009','FU-205 JANUS','','','UV Astron',340,'',213 from dual union all
select 28285,'1972-S160','1972-04-13 02:16:00','suborbital rocket','success',848,'CRL AT07.894','ALADDIN II NACS MSp.','','','Aeron',394,'',128 from dual union all
select 28286,'1972-S161','1972-04-13 02:30:00','suborbital rocket','success',867,'CRL A08.013-1','ALADDIN II Release','','','Aeron',394,'',214 from dual union all
select 28287,'1972-S162','1972-04-13 02:45:00','suborbital rocket','success',848,'CRL A07.217-2','ALADDIN II Sphere','','','Aeron',394,'',190 from dual union all
select 28288,'1972-S163','1972-04-13 05:00:00','suborbital rocket','success',848,'CRL A07.215-1','ALADDIN II Sphere','','','Aeron',394,'',219 from dual union all
select 28289,'1972-S164','1972-04-13 05:15:00','suborbital rocket','success',848,'CRL A07.921-4','ALADDIN II Release','','','Aeron',394,'',164 from dual union all
select 2829,'1954-S108','1954-12-23 00:00:00','military missile','success',992,'','Generator-2','','','Test',636,'',170 from dual union all
select 28290,'1972-S165','1972-04-13 08:50:00','suborbital rocket','success',848,'CRL A07.921-2','ALADDIN II TMA/Gren','','','Aeron',394,'',160 from dual union all
select 28291,'1972-S166','1972-04-13 10:30:00','suborbital rocket','success',848,'CRL A07.921-6','ALADDIN II TMA','','','Aeron',394,'',175 from dual union all
select 28292,'1972-S167','1972-04-14 12:18:00','suborbital rocket','success',1189,'INTA MSC-7201','','','','Ionos',396,'',101 from dual union all
select 28293,'1972-S168','1972-04-15 00:45:00','suborbital rocket','success',848,'CRL A07.106-2','','Release','','Aeron',394,'',148 from dual union all
select 28294,'1972-S169','1972-04-15 01:04:00','suborbital rocket','success',906,'CRL A10.005-1','Release','','','Aeron/Test',394,'',229 from dual union all
select 28295,'1972-S17','1972-11-18 03:10:00','military missile','success',1450,'49257-51-148T','15A20 UBP-1 K','GCh 15B20','','Test',504,'',1000 from dual union all
select 28296,'1972-S170','1972-04-15 06:22:00','suborbital rocket','success',37,'CRL A04.004-7','HI STAR 5','','','IR Astron',1873,'',177 from dual union all
select 28297,'1972-S171','1972-04-16 23:42:00','suborbital rocket','success',1069,'S-160JA-4','','','','Ionos',1458,'',86 from dual union all
select 28298,'1972-S172','1972-04-18 16:00:00','military missile','success',916,'Imp. / MICOM','P-220 72 Spring 01','','','OT/R&D',615,'',180 from dual union all
select 28299,'1972-S173','1972-04-18 18:40:00','military missile','success',917,'Imp. / MICOM','P-221 72 Spring 02','','','OT/R&D',615,'',180 from dual union all
select 283,'1944-S257','1944-11-21 14:26:00','military missile','success',1470,'19829','Ma301','','','Test',661,'',90 from dual union all
select 2830,'1954-S11','1954-04-24 00:00:00','military missile','success',970,'No 1910','','','','OT',636,'',100 from dual union all
select 28300,'1972-S174','1972-04-19 10:01:00','suborbital rocket','success',331,'WRE C1027','Sphere','','','Aeron',1835,'',100 from dual union all
select 28301,'1972-S175','1972-04-19 12:22:00','suborbital rocket','success',1189,'INTA MSC-7202','','','','Ionos',396,'',90 from dual union all
select 28302,'1972-S176','1972-04-21 11:32:00','suborbital rocket','success',1189,'INTA MSCL-7201','','','','Ionos',396,'',94 from dual union all
select 28303,'1972-S177','1972-04-21 15:13:00','suborbital rocket','success',30,'NASA 04.208UA','','','','Aeronomy',1869,'',200 from dual union all
select 28304,'1972-S178','1972-04-24 16:00:00','military missile','success',931,'A3T','DASO','','','OT',421,'SSBN 599',1000 from dual union all
select 28305,'1972-S179','1972-04-24 20:57:00','suborbital rocket','success',37,'NASA 13.15CS','','','','Solar X',1895,'',203 from dual union all
select 28306,'1972-S18','1972-11-28 12:05:00','military missile','success',1450,'51173-21-179T','15A20 UBP-2 K','GCh 15F202','','Test',504,'',1000 from dual union all
select 28307,'1972-S180','1972-04-28 00:10:00','suborbital rocket','success',447,'SUP','Pak Shahpar 1','','','Test/Ionos',1419,'',400 from dual union all
select 28308,'1972-S181','1972-04-28 11:19:00','suborbital rocket','success',1189,'INTA MSCL-7203','','','','Ionos',396,'',95 from dual union all
select 28309,'1972-S182','1972-05-17 15:20:00','military missile','success',917,'Imp. / MICOM','P-222 72 Spring 03','','','OT/R&D',615,'',180 from dual union all
select 2831,'1954-S12','1954-04-26 00:00:00','military missile','success',970,'No 1912','','','','OT',636,'',100 from dual union all
select 28310,'1972-S183','1972-05-17 16:00:00','military missile','success',917,'Imp. / MICOM','P-223 72 Spring 04','','','OT/R&D',615,'',180 from dual union all
select 28311,'1972-S184','1972-05-17 16:30:00','military missile','success',917,'','P-224 72 Spring 05','','','OT',615,'',180 from dual union all
select 28312,'1972-S185','1972-05-17 17:05:00','military missile','success',917,'','P-225 72 Spring 06','','','OT',615,'',180 from dual union all
select 28313,'1972-S186','1972-05-01 08:23:00','suborbital rocket','success',867,'CRL A08.113-1','Auroral B','','','Plasma',428,'',231 from dual union all
select 28314,'1972-S187','1972-05-01 08:24:00','suborbital rocket','success',867,'CRL A08.112-1','Auroral B','','','Plasma',428,'',213 from dual union all
select 28315,'1972-S188','1972-05-03 11:30:00','suborbital rocket','unknown',1189,'INTA MSC-7203','','','','Ionos',396,'',100 from dual union all
select 28316,'1972-S189','1972-05-05 00:00:00','military missile','success',795,'','SSTTP','SSTTP M2-11','','Target',1701,'',1300 from dual union all
select 28317,'1972-S19','1972-12-08 02:15:00','military missile','success',1450,'47653-41-273L','15A20 UBP-3 ZP','GCh 15B20','','Test',504,'',1000 from dual union all
select 28318,'1972-S190','1972-05-05 11:15:00','suborbital rocket','success',1189,'INTA MSCL-7204','','','','Ionos',396,'',96 from dual union all
select 28319,'1972-S191','1972-05-10 11:22:00','suborbital rocket','success',1189,'INTA MSCL-7205','','','','Ionos',396,'',93 from dual union all
select 2832,'1954-S13','1954-04-29 00:00:00','military missile','success',970,'No 2120','','','','OT',636,'',100 from dual union all
select 28320,'1972-S192','1972-05-12 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT106M','GLORY TRIP 106M','OT',1700,'',1300 from dual union all
select 28321,'1972-S193','1972-05-12 10:30:00','suborbital rocket','success',38,'NASA 13.19DG','NRL NB4.255','FUV Schmidt','','UV Astron',1895,'',200 from dual union all
select 28322,'1972-S194','1972-05-13 23:13:00','suborbital rocket','success',1070,'S-210JA-9','','','','Auroral/Ionos/Field',1458,'',129 from dual union all
select 28323,'1972-S195','1972-05-15 23:02:00','suborbital rocket','success',1070,'S-210JA-10','','','','Auroral/Ionos/Field',1458,'',115 from dual union all
select 28324,'1972-S196','1972-05-17 11:12:00','suborbital rocket','success',1347,'','','','','IR Astron',855,'',325 from dual union all
select 28325,'1972-S197','1972-05-17 07:00:00','suborbital rocket','success',220,'APF-4B-26','X-ray survey','','','XR Astron',428,'',660 from dual union all
select 28326,'1972-S198','1972-05-17 07:40:00','suborbital rocket','success',839,'NASA 10.397GM','GSFC Grenade 271 PB','','','Aeronomy',1039,'',110 from dual union all
select 28327,'1972-S199','1972-05-17 11:15:00','suborbital rocket','success',1189,'INTA MSC-7204','','','','Ionos',396,'',93 from dual union all
select 28328,'1972-S20','1972-12-22 02:02:00','military missile','success',1450,'51173-21-180T','15A20 UBP-4 K','GCh 15B20','','Test',504,'',1000 from dual union all
select 28329,'1972-S200','1972-05-17 11:30:00','suborbital rocket','success',839,'NASA 10.395GM','GSFC Ozone','','','Aeronomy',1039,'',85 from dual union all
select 2833,'1954-S14','1954-04-22 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 28330,'1972-S201','1972-05-17 13:00:00','suborbital rocket','success',839,'NASA 10.393GM','UM Oxygen','','','Aeronomy',1039,'',127 from dual union all
select 28331,'1972-S202','1972-05-17 16:45:00','suborbital rocket','success',839,'NASA 10.398GM','GSFC Grenade 272 PB','','','Aeronomy',1039,'',92 from dual union all
select 28332,'1972-S203','1972-05-18 08:37:00','suborbital rocket','success',331,'WRE C1030','Sphere','','','Aeron',1835,'',100 from dual union all
select 28333,'1972-S204','1972-05-19 08:35:00','suborbital rocket','success',37,'NASA 13.37UG','MIT-10','','','XR Astron',1895,'',200 from dual union all
select 28334,'1972-S205','1972-05-19 10:00:00','suborbital rocket','success',38,'NASA 13.09GG','','','','XR Astron',1895,'',200 from dual union all
select 28335,'1972-S206','1972-05-20 09:24:30','suborbital rocket','success',1349,'Sandia 281-','LRL ACS-7A (TT-9)','','','XR Astron',855,'',300 from dual union all
select 28336,'1972-S207','1972-05-22 12:00:00','suborbital rocket','success',867,'LEINAX II','','','','XR Astron',855,'',270 from dual union all
select 28337,'1972-S208','1972-05-24 23:45:00','military missile','success',1420,'B-46','Mk 6 SSTTP T3','SAFSCOM M2-10','','Target',1658,'',1300 from dual union all
select 28338,'1972-S209','1972-05-24 11:08:00','suborbital rocket','success',1189,'INTA MSC-7205','','','','Ionos',396,'',89 from dual union all
select 28339,'1972-S21','1972-03-17 00:00:00','military missile','success',1456,'','15A20U LKI-11','','','Test',506,'',1000 from dual union all
select 2834,'1954-S15','1954-02-28 00:00:00','military missile','unknown',992,'531-20-420','NII-88 Production','','','Test',636,'',150 from dual union all
select 28340,'1972-S210','1972-05-26 04:20:00','military missile','success',1001,'','8K67MA LKI-2','GCh 15F141','','Test',518,'',1000 from dual union all
select 28341,'1972-S211','1972-05-29 10:34:00','suborbital rocket','success',331,'WRE C1014','Lunar UV','','','Fields/UV Astron',1835,'',120 from dual union all
select 28342,'1972-S212','1972-05-31 14:20:00','military missile','success',799,'PVM-1/FTM-5255','Mk 12 RV x 3','R&D','','Test',1707,'',1300 from dual union all
select 28343,'1972-S213','1972-06-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 28344,'1972-S214','1972-06-02 06:10:00','military missile','success',917,'','P-226 72 Spring 07','','','OT',615,'',180 from dual union all
select 28345,'1972-S215','1972-06-02 06:45:00','military missile','success',917,'','P-227 72 Spring 08','','','OT',615,'',180 from dual union all
select 28346,'1972-S216','1972-06-03 00:00:00','military missile','success',917,'','P-228 72 Spring 09','','','OT',615,'',180 from dual union all
select 28347,'1972-S217','1972-06-03 03:55:00','military missile','failure',917,'','P-229 72 Spring 10','','','OT',615,'',10 from dual union all
select 28348,'1972-S218','1972-06-07 17:15:00','military missile','success',917,'','P-230 72 Spring 11','','','OT',615,'',180 from dual union all
select 28349,'1972-S219','1972-06-08 00:00:00','military missile','success',917,'','P-231 72 Spring 12','','','OT',615,'',180 from dual union all
select 2835,'1954-S16','1954-03-10 00:00:00','military missile','unknown',992,'531-20-520','NII-88 Production','','','Test',636,'',150 from dual union all
select 28350,'1972-S22','1972-03-18 00:00:00','military missile','success',1456,'','15A20U K ZP-1','','','Test',508,'',1000 from dual union all
select 28351,'1972-S220','1972-06-13 14:00:00','military missile','success',917,'AO-108','P-232 72 Spring 13','','','OT',615,'',180 from dual union all
select 28352,'1972-S221','1972-06-13 15:30:00','military missile','success',917,'AO-109','P-233 72 Spring 14','','','OT',615,'',180 from dual union all
select 28353,'1972-S222','1972-06-29 00:00:00','military missile','success',917,'','P-236 72 Spring 17','','','OT',615,'',180 from dual union all
select 28354,'1972-S223','1972-06-29 00:00:00','military missile','success',917,'','P-237 72 Spring 18','','','OT',615,'',180 from dual union all
select 28355,'1972-S224','1972-06-29 00:00:00','military missile','success',917,'','P-238 72 Spring 19','','','OT',615,'',180 from dual union all
select 28356,'1972-S225','1972-10-19 00:00:00','military missile','success',917,'AO-118','P-246 72 Fall 07','','','OT',615,'',180 from dual union all
select 28357,'1972-S226','1972-06-02 00:00:00','military missile','failure',1322,'3L','15Zh42 LKI-3','','','Test',477,'',0 from dual union all
select 28358,'1972-S227','1972-05-31 06:19:11','suborbital rocket','success',837,'NASA 14.485UI','Ferdinand 34','','','Ionos',29,'',154 from dual;
insert into LAUNCH
select 28359,'1972-S228','1972-05-31 06:53:20','suborbital rocket','success',839,'NASA 10.391UI','Ferdinand 33','','','Ionos',29,'',110 from dual union all
select 2836,'1954-S17','1954-05-26 00:00:00','military missile','success',992,'Prod. missile','NII-88 Prod GIM-2','','','Test',636,'',150 from dual union all
select 28360,'1972-S229','1972-05-31 11:15:00','suborbital rocket','success',1189,'INTA MSC-7206','','','','Ionos',396,'',94 from dual union all
select 28361,'1972-S23','1972-03-27 00:00:00','military missile','success',1456,'','15A20U K ZP-2','','','Test',508,'',1000 from dual union all
select 28362,'1972-S230','1972-05-31 22:25:00','suborbital rocket','success',228,'NASA 21.01US','','','','Solar',1873,'',207 from dual union all
select 28363,'1972-S231','1972-06-02 06:16:00','suborbital rocket','success',864,'Sandia 152-185','LRL ACS-1C','','','XR Astron',1488,'',175 from dual union all
select 28364,'1972-S232','1972-06-06 00:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT15GM','GLORY TRIP 15GM','OT',1699,'',1300 from dual union all
select 28365,'1972-S233','1972-06-07 11:23:00','suborbital rocket','success',1189,'INTA MSC-7207','','','','Ionos',396,'',95 from dual union all
select 28366,'1972-S234','1972-06-09 05:25:00','suborbital rocket','success',228,'NASA 21.08UG','','UCB 0.15m UVT/EUV','','UV Astron/EUV Astron',1873,'',255 from dual union all
select 28367,'1972-S235','1972-06-09 05:45:00','suborbital rocket','success',38,'NASA 13.59GG','','','','UV Astron?',1895,'',200 from dual union all
select 28368,'1972-S236','1972-06-10 04:30:00','suborbital rocket','success',837,'NASA 14.415UG','','','','XR Astron',1895,'',164 from dual union all
select 28369,'1972-S237','1972-06-11 20:35:00','military missile','success',799,'','Mk 12 RV x 3','OT GT14GM','GLORY TRIP 14GM','OT',1697,'',1300 from dual union all
select 2837,'1954-S18','1954-05-31 00:00:00','military missile','success',992,'Prod. missile','ZAV586 Prod GIM-2','','','Test',636,'',150 from dual union all
select 28370,'1972-S238','1972-06-13 17:45:00','military missile','success',796,'','Mk 11 RV/OBLSS','OBL','GIANT PATRIOT 1','Test',1702,'',1300 from dual union all
select 28371,'1972-S239','1972-06-13 20:00:00','suborbital rocket','success',37,'NASA 13.66US','','','','Solar X',1895,'',200 from dual union all
select 28372,'1972-S24','1972-06-16 00:00:00','military missile','success',1456,'','15A20U LKI-12','','','Test',506,'',1000 from dual union all
select 28373,'1972-S240','1972-06-14 11:25:00','suborbital rocket','success',1189,'INTA MSC-7208','','','','Ionos',396,'',90 from dual union all
select 28374,'1972-S241','1972-06-15 08:31:00','suborbital rocket','success',331,'WRE C1033','Sphere','','','Aeron',1835,'',100 from dual union all
select 28375,'1972-S242','1972-06-16 23:15:00','military missile','success',939,'C3E','DASO-20','','','OT',421,'SSBN 640',500 from dual union all
select 28376,'1972-S243','1972-06-18 00:00:00','military missile','success',799,'STM-5W','Mk 12 RV x 3','R&D','','Test',1703,'',1300 from dual union all
select 28377,'1972-S244','1972-06-20 16:45:00','military missile','success',796,'','Mk 11 RV','FOT GT107M','GLORY TRIP 107M','OT',1700,'',1300 from dual union all
select 28378,'1972-S245','1972-06-21 11:25:00','suborbital rocket','success',1189,'INTA MSC-7209','','','','Ionos',396,'',90 from dual union all
select 28379,'1972-S246','1972-06-21 20:01:00','suborbital rocket','success',30,'NASA 04.304NA','','','','Aeronomy',1869,'',200 from dual union all
select 2838,'1954-S19','1954-06-02 00:00:00','military missile','success',992,'Prod. missile','ZAV586 Prod GIM-2','','','Test',636,'',150 from dual union all
select 28380,'1972-S247','1972-06-22 14:20:00','military missile','success',917,'AO-110','P-234 72 Spring 15','','','OT',615,'',180 from dual union all
select 28381,'1972-S248','1972-06-22 15:00:00','military missile','success',917,'AO-111','P-235 72 Spring 16','','','OT',615,'',180 from dual union all
select 28382,'1972-S249','1972-06-25 14:47:00','suborbital rocket','success',1207,'ESRO S91','MSSL XR10','','','XR Astronomy',1343,'',213 from dual union all
select 28383,'1972-S25','1972-07-13 00:00:00','military missile','success',1456,'','15A20U K ZP-3','','','Test',506,'',1000 from dual union all
select 28384,'1972-S250','1972-06-27 20:00:00','suborbital rocket','success',37,'NASA 13.05US','CAL Solar 1','','','Solar',1895,'',200 from dual union all
select 28385,'1972-S251','1972-07-03 08:10:00','suborbital rocket','success',837,'NASA 14.483GI-II','INTA NA-7201','','','Ionos',396,'',122 from dual union all
select 28386,'1972-S252','1972-07-05 00:00:00','suborbital rocket','success',1211,'SL1191','SUPER ANTELOPE S1','','','Test',1839,'',150 from dual union all
select 28387,'1972-S253','1972-07-05 11:17:00','suborbital rocket','success',1189,'INTA MSC-7211','','','','Ionos',396,'',87 from dual union all
select 28388,'1972-S254','1972-07-05 23:15:00','military missile','success',939,'','DASO-21','','','OT',421,'SSBN 641',500 from dual union all
select 28389,'1972-S255','1972-07-06 08:10:00','suborbital rocket','success',839,'NASA 10.408GI','INTA NC-7201','','','Ionos',396,'',85 from dual union all
select 2839,'1954-S20','1954-05-17 00:00:00','military missile','unknown',992,'Prod. missile','NII-88 Production','','','Test',636,'',150 from dual union all
select 28390,'1972-S256','1972-07-10 11:00:00','suborbital rocket','success',217,'ADD-3A-55','','','','Aeron/Eclipse',428,'',106 from dual union all
select 28391,'1972-S257','1972-07-10 18:51:00','suborbital rocket','success',1347,'Sandia 281-28','','','','Solar Eclipse',1047,'',300 from dual union all
select 28392,'1972-S258','1972-07-10 19:19:00','suborbital rocket','success',217,'ADD-3A-56','','','','Aeron/Eclipse',428,'',115 from dual union all
select 28393,'1972-S259','1972-07-10 19:50:00','suborbital rocket','success',217,'ADD-3A-57','','','','Aeron/Eclipse',428,'',166 from dual union all
select 28394,'1972-S26','1972-10-13 00:00:00','military missile','success',1456,'','15A20U K ZP-4','','','Test',506,'',1000 from dual union all
select 28395,'1972-S260','1972-07-10 20:26:00','suborbital rocket','success',217,'ADD-3A-58','','','','Aeron/Eclipse',428,'',103 from dual union all
select 28396,'1972-S261','1972-07-10 20:29:00','suborbital rocket','success',217,'ADD-3A-59','','','','Solar UV/X',406,'',143 from dual union all
select 28397,'1972-S262','1972-07-10 21:32:00','suborbital rocket','success',217,'ADD-3A-60','','','','Solar UV/X',406,'',148 from dual union all
select 28398,'1972-S263','1972-07-10 21:39:00','suborbital rocket','success',217,'ADD-3A-61','','','','Solar UV/X',406,'',144 from dual union all
select 28399,'1972-S264','1972-07-10 21:41:00','suborbital rocket','success',217,'ADD-3A-62','','','','Solar UV/X',406,'',152 from dual union all
select 284,'1944-S258','1944-11-22 11:35:00','military missile','success',1470,'19843','Ma308','','','Test',661,'',90 from dual union all
select 2840,'1954-S21','1954-05-12 00:00:00','military missile','unknown',992,'Prod. missile','NII-88 Production','','','Test',636,'',150 from dual union all
select 28400,'1972-S265','1972-07-10 22:01:00','suborbital rocket','success',37,'NASA 13.80DS','NRL NE4.246','','','Solar',1895,'',200 from dual union all
select 28401,'1972-S266','1972-07-10 22:05:00','suborbital rocket','success',38,'NASA 13.28CS','','','','Solar',1895,'',200 from dual union all
select 28402,'1972-S267','1972-07-12 11:35:00','suborbital rocket','success',1189,'INTA MSC-7212','','','','Ionos',396,'',90 from dual union all
select 28403,'1972-S268','1972-07-14 03:05:00','military missile','success',1001,'','8K67MA LKI-3','GCh 15F141','','Test',515,'',1000 from dual union all
select 28404,'1972-S269','1972-07-14 06:01:00','suborbital rocket','success',37,'NASA 13.04UG','','','','UV Astron',1895,'',200 from dual union all
select 28405,'1972-S27','1972-10-18 00:00:00','military missile','success',1456,'','15A20U K ZP-5','','','Test',508,'',1000 from dual union all
select 28406,'1972-S270','1972-07-15 00:00:00','military missile','success',795,'','SSTTP','SSTTP M2-15','','Target',1701,'',1300 from dual union all
select 28407,'1972-S271','1972-07-18 08:20:00','suborbital rocket','success',38,'KP3.40','IPRCE-3','','','IR Astron',1869,'',222 from dual union all
select 28408,'1972-S272','1972-07-26 05:10:00','military missile','success',796,'Destruct test','OBLSS package','OBL','GIANT PATRIOT 2','Test',1702,'',21 from dual union all
select 28409,'1972-S273','1972-07-27 18:10:00','suborbital rocket','success',37,'NASA 13.31US','','','','Solar',1895,'',200 from dual union all
select 2841,'1954-S22','1954-05-14 00:00:00','military missile','unknown',992,'Prod. missile','NII-88 Production','','','Test',636,'',150 from dual union all
select 28410,'1972-S274','1972-07-28 18:59:00','suborbital rocket','success',1440,'CRL A21.011-1','AFCRL TB2-4','','','Reentry Test',1821,'',314 from dual union all
select 28411,'1972-S275','1972-08-01 00:00:00','military missile','success',1514,'','','','','Test',190,'PLBR',1000 from dual union all
select 28412,'1972-S276','1972-08-02 00:00:00','military missile','success',799,'STM-7W','Mk 12 RV x 3','R&D','','Test',1707,'',1300 from dual union all
select 28413,'1972-S277','1972-08-03 00:00:00','military missile','success',1251,'800029','SAFEGUARD M2-34','','','Interceptor',868,'',100 from dual union all
select 28414,'1972-S278','1972-08-04 19:16:00','suborbital rocket','success',837,'NASA 14.489GE','Solar cosmic rays','','','Plasma',428,'',159 from dual union all
select 28415,'1972-S279','1972-08-05 05:52:00','suborbital rocket','success',37,'NASA 13.30CG','','','','XR Astron',1895,'',200 from dual union all
select 28416,'1972-S28','1972-12-12 00:00:00','military missile','success',1456,'','15A20U K ZP-6','','','Test',506,'',1000 from dual union all
select 28417,'1972-S280','1972-08-07 01:45:00','suborbital rocket','success',1070,'S-210JA-11','','','','Auroral/Ionos/Field',1458,'',126 from dual union all
select 28418,'1972-S281','1972-08-10 00:32:00','test rocket','success',492,'701-02','Shiyan Peizhong','','','Test',810,'',200 from dual union all
select 28419,'1972-S282','1972-08-10 05:45:00','suborbital rocket','success',37,'NASA 13.38UG','MIT LATP 1','','','XR Astron',1895,'',200 from dual union all
select 2842,'1954-S23','1954-05-15 00:00:00','military missile','unknown',992,'Prod. missile','NII-88 Production','','','Test',636,'',150 from dual union all
select 28420,'1972-S283','1972-08-10 06:59:00','suborbital rocket','success',37,'NASA 13.23UG','','','','UV Astron?',1895,'',200 from dual union all
select 28421,'1972-S284','1972-08-11 01:01:00','suborbital rocket','success',1070,'S-210JA-8','','','','Ionos/Fields',1458,'',127 from dual union all
select 28422,'1972-S285','1972-08-12 09:28:00','military missile','success',136,'H004','RVTO-3B-2','ABRES RVTO-3B-2','','Reentry Test',615,'',200 from dual union all
select 28423,'1972-S286','1972-08-12 02:15:00','suborbital rocket','success',289,'ISRO 5.14','ISRO 5.14','TERLS-463','','Ionos',1467,'',150 from dual union all
select 28424,'1972-S287','1972-08-17 10:02:00','suborbital rocket','success',289,'ISRO 5.16','ISRO 5.16  CO5.16','TERLS-465','','Ionos',1467,'',151 from dual union all
select 28425,'1972-S288','1972-08-18 00:00:00','military missile','success',939,'','FOT-1','','','OT',421,'SSBN 631',500 from dual union all
select 28426,'1972-S289','1972-08-18 00:00:00','military missile','success',939,'','FOT-1','','','OT',421,'SSBN 631',500 from dual union all
select 28427,'1972-S29','1972-12-28 00:00:00','military missile','success',1456,'','15A20U K ZP-7','','','Test',508,'',1000 from dual union all
select 28428,'1972-S290','1972-08-18 00:00:00','military missile','success',939,'','FOT-1','','','OT',421,'SSBN 631',500 from dual union all
select 28429,'1972-S291','1972-08-18 00:00:00','military missile','success',939,'','FOT-1','','','OT',421,'SSBN 631',500 from dual union all
select 2843,'1954-S24','1954-05-03 00:00:00','military missile','success',970,'No 1620','','','','OT',636,'',100 from dual union all
select 28430,'1972-S292','1972-08-18 10:21:00','suborbital rocket','success',37,'CRL A04.004-8','HI STAR 6','','','IR Astron',1873,'',171 from dual union all
select 28431,'1972-S293','1972-08-23 21:00:00','suborbital rocket','success',37,'CRL A04.104-1','RM-57','','','Solar XUV',1895,'',268 from dual union all
select 28432,'1972-S294','1972-08-24 04:15:00','military missile','success',1001,'','8K67MA LKI-4','GCh 15F141','','Test',518,'',1000 from dual union all
select 28433,'1972-S295','1972-08-29 11:12:00','suborbital rocket','success',920,'P118H','E-region multiprobe','','','Ionospheric',1447,'',140 from dual union all
select 28434,'1972-S296','1972-08-29 13:12:00','suborbital rocket','success',920,'P130H','Solar La/X','','','Solar UV,X',1447,'',135 from dual union all
select 28435,'1972-S297','1972-08-30 22:02:00','suborbital rocket','success',920,'P75H','Scattered La','','','Ionospheric/Aeron',1447,'',118 from dual union all
select 28436,'1972-S298','1972-09-01 00:00:00','military missile','success',1514,'','','','','Test',190,'PLBR',1000 from dual union all
select 28437,'1972-S299','1972-09-28 00:00:00','military missile','success',1322,'35444-30-113-5L','15Zh42 LKI-5','','','Test',477,'',1000 from dual union all
select 28438,'1972-S30','1972-01-01 07:05:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',137,'ZUB',180 from dual union all
select 28439,'1972-S300','1972-08-31 00:02:00','suborbital rocket','success',920,'P108H','Scattered La','','','Ionospheric',1447,'',146 from dual union all
select 2844,'1954-S25','1954-05-04 00:00:00','military missile','success',970,'No 2020','','','','OT',636,'',100 from dual union all
select 28440,'1972-S301','1972-08-31 02:16:00','suborbital rocket','success',920,'P76H','Scattered La','','','Ionospheric',1447,'',126 from dual union all
select 28441,'1972-S302','1972-09-01 04:10:00','suborbital rocket','success',38,'NASA 13.47UL','JHU UVT F7','','','UV Astron',1895,'',159 from dual union all
select 28442,'1972-S303','1972-09-12 05:31:00','suborbital rocket','success',624,'NASA 8.62GE','Auroral substorm','','','Auroral',428,'',718 from dual union all
select 28443,'1972-S304','1972-09-12 11:10:00','suborbital rocket','success',655,'K-10-8','Fiberglass Casing','','','IR Astron/XR Astron',839,'',259 from dual union all
select 28444,'1972-S305','1972-09-15 08:02:00','military missile','success',744,'45023-301','15A15 BI-4-1','GCh 15F152 / Live St 2','','Test',530,'',1000 from dual union all
select 28445,'1972-S306','1972-09-15 03:10:00','military missile','success',1001,'','8K67MA LKI-5','GCh 15F141','','Test',515,'',1000 from dual union all
select 28446,'1972-S307','1972-09-16 04:00:00','suborbital rocket','success',37,'NASA 13.39UG','MIT LATP 2','','','XR Astron',1895,'',200 from dual union all
select 28447,'1972-S308','1972-09-18 00:00:00','military missile','success',917,'AO-112','P-240 72 Fall 01','','','OT',615,'',180 from dual union all
select 28448,'1972-S309','1972-09-18 00:00:00','military missile','success',917,'AO-113','P-241 72 Fall 02','','','OT',615,'',180 from dual;
insert into LAUNCH
select 28449,'1972-S31','1972-01-01 08:02:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',180 from dual union all
select 2845,'1954-S26','1954-05-04 00:00:00','military missile','success',970,'No 1420','','','','OT',636,'',100 from dual union all
select 28450,'1972-S310','1972-09-19 00:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT16GB','GLORY TRIP 16GM','OT',1708,'',1300 from dual union all
select 28451,'1972-S311','1972-09-20 05:00:00','suborbital rocket','success',668,'K-9M-40','','','','Ionos',839,'',326 from dual union all
select 28452,'1972-S312','1972-09-24 14:05:00','suborbital rocket','success',226,'CRL A17.110-2','Auroral D/E region','','','Ionos',428,'',127 from dual union all
select 28453,'1972-S313','1972-09-24 21:50:00','suborbital rocket','success',1208,'ESRO S77/2','ESRO S77/2','','','Auroral',416,'',227 from dual union all
select 28454,'1972-S314','1972-09-25 23:30:00','military missile','success',796,'','Mk 11 RV','FOT GT108M','GLORY TRIP 108M','OT',1700,'',1300 from dual union all
select 28455,'1972-S315','1972-09-25 02:40:00','suborbital rocket','success',228,'NASA 21.05UE','ECHO II','','','Plasma',428,'',256 from dual union all
select 28456,'1972-S316','1972-09-25 06:05:00','suborbital rocket','success',694,'LS-C-6','','','','Test',1476,'',40 from dual union all
select 28457,'1972-S317','1972-09-25 14:30:00','military missile','success',917,'AO-114','P-242 72 Fall 03','','','OT',615,'',180 from dual union all
select 28458,'1972-S318','1972-09-25 15:30:00','military missile','success',917,'AO-115','P-243 72 Fall 04','','','OT',615,'',180 from dual union all
select 28459,'1972-S319','1972-09-25 19:24:00','suborbital rocket','success',226,'CRL A17.110-1','Auroral D/E region','','','Ionos',428,'',126 from dual union all
select 2846,'1954-S27','1954-05-07 00:00:00','military missile','success',970,'No 220','','','','OT',636,'',100 from dual union all
select 28460,'1972-S32','1972-01-04 13:14:00','suborbital rocket','success',920,'P78T/C','ISRO 18.01','TERLS-398','','Ionospheric',1467,'',167 from dual union all
select 28461,'1972-S320','1972-09-25 22:45:00','military missile','success',939,'','DASO-22','','','OT',421,'SSBN 643',500 from dual union all
select 28462,'1972-S321','1972-09-26 18:13:00','suborbital rocket','success',133,'NASA 12.20GT','','','','Test',1797,'',199 from dual union all
select 28463,'1972-S322','1972-09-28 00:00:00','military missile','success',939,'','FOT-2','','','OT',421,'SSBN 632',500 from dual union all
select 28464,'1972-S323','1972-09-28 00:00:00','military missile','success',939,'','FOT-2','','','OT',421,'SSBN 632',500 from dual union all
select 28465,'1972-S324','1972-09-28 00:00:00','military missile','success',939,'','FOT-2','','','OT',421,'SSBN 632',500 from dual union all
select 28466,'1972-S325','1972-09-28 00:00:00','military missile','success',939,'','FOT-2','','','OT',421,'SSBN 632',500 from dual union all
select 28467,'1972-S326','1972-10-19 00:00:00','military missile','success',917,'AO-119','P-247 72 Fall 08','','','OT WS-185?',615,'',180 from dual union all
select 28468,'1972-S327','1972-09-01 00:00:00','military missile','success',1550,'Mod 2, MIRV','','','','Test',1626,'PLBR',620 from dual union all
select 28469,'1972-S328','1972-10-01 00:00:00','military missile','success',1514,'','','','','Test',190,'PLBR',1000 from dual union all
select 2847,'1954-S28','1954-05-07 17:00:00','suborbital rocket','success',1500,'10','Viking 10','','','Aeron/Ionos',1889,'',219 from dual union all
select 28470,'1972-S329','1972-10-03 08:30:00','military missile','success',796,'','Mk 11 RV','FOT GT109M','GLORY TRIP 109M','OT',1699,'',1300 from dual union all
select 28471,'1972-S33','1972-01-05 11:41:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',1010,'',186 from dual union all
select 28472,'1972-S330','1972-10-05 15:00:00','military missile','success',931,'A3T','DASO','','','OT',421,'SSBN 601',1000 from dual union all
select 28473,'1972-S331','1972-10-05 19:24:07','suborbital rocket','success',920,'P127A','Particles/substorm','','','Auroral',29,'',117 from dual union all
select 28474,'1972-S332','1972-10-07 22:47:55','suborbital rocket','success',920,'P42A','Particles/substorm','','','Auroral',29,'',140 from dual union all
select 28475,'1972-S333','1972-10-08 16:10:00','suborbital rocket','success',837,'NASA 14.499UA','SSC NNA 3','','','Meteorites',417,'',109 from dual union all
select 28476,'1972-S334','1972-10-09 12:51:35','suborbital rocket','success',1116,'Sandia 152-191','Ba CHACHALACA','','','Aeron/Fields',1047,'',500 from dual union all
select 28477,'1972-S335','1972-10-09 15:30:00','suborbital rocket','success',837,'NASA 14.501UA','SSC NNA 4','','','Meteorites',417,'',109 from dual union all
select 28478,'1972-S336','1972-10-10 21:21:51','suborbital rocket','success',920,'P55A','LP/VLF 3','','','Auroral',29,'',140 from dual union all
select 28479,'1972-S337','1972-10-11 00:00:00','military missile','success',917,'AO-116','P-244 72 Fall 05','','','OT',615,'',180 from dual union all
select 2848,'1954-S29','1954-05-11 15:00:00','suborbital rocket','success',51,'USAF 46','Beacon test','','','Test',640,'',98 from dual union all
select 28480,'1972-S338','1972-10-11 00:00:00','military missile','success',917,'AO-117','P-245 72 Fall 06','','','OT',615,'',180 from dual union all
select 28481,'1972-S339','1972-10-11 11:30:00','military missile','success',1420,'B-78','Mk 6 SSTTP T4','SAFSCOM M2-14/ChaserT','','Target',1658,'',1300 from dual union all
select 28482,'1972-S34','1972-01-06 13:14:00','suborbital rocket','success',920,'P95T/C','ISRO 18.02','TERLS-401','','Ionospheric',1467,'',126 from dual union all
select 28483,'1972-S340','1972-10-11 03:00:00','suborbital rocket','success',226,'NRL NF9.250','Auroral IR','','','Auroral',428,'',144 from dual union all
select 28484,'1972-S341','1972-10-11 11:31:00','suborbital rocket','success',37,'CRL A04.116-4','CHASER','','','ABM Tech IR',1768,'',151 from dual union all
select 28485,'1972-S342','1972-10-12 04:14:00','suborbital rocket','success',1189,'SK19A','','','','Ionospheric Ba',29,'',100 from dual union all
select 28486,'1972-S343','1972-10-12 15:33:30','suborbital rocket','success',837,'NASA 14.502UA','SSC NNA 5','','','Meteorites',417,'',109 from dual union all
select 28487,'1972-S344','1972-10-13 00:28:15','suborbital rocket','success',1204,'ESRO S105','ESRO S105','','','Auroral',416,'',207 from dual union all
select 28488,'1972-S345','1972-08-13 06:59:00','suborbital rocket','success',837,'ISRO 10.44','ISRO 10.44','TERLS-489','','Ionos',1467,'',148 from dual union all
select 28489,'1972-S346','1972-10-13 15:11:00','suborbital rocket','success',837,'NASA 14.504CA','GCA Release Na-Li','','','Aeronomy',1047,'',160 from dual union all
select 2849,'1954-S30','1954-05-13 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 28490,'1972-S347','1972-10-13 16:39:00','suborbital rocket','success',837,'NASA 14.505CA','GCA Release Na-Li','','','Aeronomy',1047,'',160 from dual union all
select 28491,'1972-S348','1972-10-13 18:57:00','suborbital rocket','success',289,'ISRO 5.17','CNES FU-198','TERLS-488','','Fields',1467,'',120 from dual union all
select 28492,'1972-S349','1972-10-15 15:00:00','suborbital rocket','success',1277,'NASA 12.18NE','RCTV/Art. Aurora','','','Test/Electron gun',855,'',500 from dual union all
select 28493,'1972-S35','1972-01-07 13:11:00','suborbital rocket','success',920,'P77T/C','ISRO 18.03','TERLS-402','','Ionospheric',1467,'',166 from dual union all
select 28494,'1972-S350','1972-10-16 00:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT17GB','GLORY TRIP 17GB','OT',1711,'',1300 from dual union all
select 28495,'1972-S351','1972-10-17 18:39:00','suborbital rocket','success',1204,'SL1022','QUB AGP8/SHF-10','','','Ionospheric',1839,'',142 from dual union all
select 28496,'1972-S352','1972-10-18 12:09:00','suborbital rocket','success',1189,'INTA MSCLO-7201','','','','Ionos',396,'',91 from dual union all
select 28497,'1972-S353','1972-10-20 09:05:00','military missile','success',796,'','Mk 11 RV','FOT GT110M','GLORY TRIP 110M','OT',1697,'',1300 from dual union all
select 28498,'1972-S354','1972-10-20 12:07:00','suborbital rocket','success',1189,'INTA MSCLO-7202','','','','Ionos',396,'',94 from dual union all
select 28499,'1972-S355','1972-10-20 12:17:30','suborbital rocket','success',1347,'Sandia 281-29','LRL ACS-4D','','','XR Astron',846,'',310 from dual union all
select 285,'1944-S259','1944-11-22 15:59:00','military missile','success',1470,'19830','Ma300','','','Test',661,'',90 from dual union all
select 2850,'1954-S31','1954-05-21 00:00:00','military missile','success',970,'No 120','','','','OT',636,'',100 from dual union all
select 28500,'1972-S356','1972-10-21 12:34:51','suborbital rocket','success',1349,'Sandia 152-197','LRL ACS-7B','','','XR Astron',845,'',300 from dual union all
select 28501,'1972-S357','1972-10-21 13:16:13','suborbital rocket','success',867,'NASA 18.151UE','','','','Plasma',1047,'',270 from dual union all
select 28502,'1972-S358','1972-10-21 20:00:50','suborbital rocket','success',920,'P126A','Particles/substorm','','','Auroral',29,'',117 from dual union all
select 28503,'1972-S359','1972-10-23 11:51:00','suborbital rocket','success',1211,'SL402','MSSL XR11/MO LunarUV','','','XR Astron/UV Astron',1839,'',210 from dual union all
select 28504,'1972-S36','1972-01-12 21:16:35','suborbital rocket','success',837,'NASA 14.383AE','','','','Plasma',29,'',197 from dual union all
select 28505,'1972-S360','1972-10-23 16:14:57','suborbital rocket','success',1189,'SK20A','','','','Ionospheric Ba',29,'',100 from dual union all
select 28506,'1972-S361','1972-10-23 19:05:21','suborbital rocket','success',920,'P125A','Elec. analyser','','','Auroral',29,'',140 from dual union all
select 28507,'1972-S362','1972-10-24 17:20:00','suborbital rocket','success',227,'NASA 21.20GT','CALROC Test','','','Test',1873,'',282 from dual union all
select 28508,'1972-S363','1972-10-25 12:08:00','suborbital rocket','success',1189,'INTA MSCLO-7203','','','','Ionos',396,'',93 from dual union all
select 28509,'1972-S364','1972-10-25 13:22:00','suborbital rocket','success',740,'','','','','Aeron?',636,'',160 from dual union all
select 2851,'1954-S32','1954-05-24 17:00:00','suborbital rocket','success',1500,'11','Viking 11','','','Reentry test',1889,'',254 from dual union all
select 28510,'1972-S365','1972-10-27 00:00:00','military missile','success',136,'H007','ABC-2','ABRES ABC-2','','Reentry Test',615,'',200 from dual union all
select 28511,'1972-S366','1972-10-26 00:00:00','military missile','success',1251,'800037','SAFEGUARD M2-12','','','Interceptor',868,'',100 from dual union all
select 28512,'1972-S367','1972-10-26 04:30:00','suborbital rocket','success',1207,'SL1003','MSSL SX8 BCS/RMC','','','Solar X',1839,'',195 from dual union all
select 28513,'1972-S368','1972-10-26 11:00:00','suborbital rocket','success',740,'','','','','Aeron?',636,'',170 from dual union all
select 28514,'1972-S369','1972-10-28 03:23:00','military missile','success',795,'','HK-1','Army STP HK-1','','Reentry Test?',1701,'',1300 from dual union all
select 28515,'1972-S37','1972-01-13 20:04:35','suborbital rocket','success',867,'NASA 18.135IE','Ferdinand 29/Electra','','','Plasma',29,'',192 from dual union all
select 28516,'1972-S370','1972-10-27 23:02:00','suborbital rocket','success',740,'','','','','Aeron?',636,'',170 from dual union all
select 28517,'1972-S371','1972-10-29 15:48:00','suborbital rocket','success',1189,'SK21A','','','','Ionospheric Ba',29,'',100 from dual union all
select 28518,'1972-S372','1972-10-30 17:45:00','suborbital rocket','success',37,'NASA 13.67US','','','','Solar X',1895,'',200 from dual union all
select 28519,'1972-S373','1972-10-30 22:00:00','suborbital rocket','success',740,'','N2 concentration','','','Aeron',636,'',165 from dual union all
select 2852,'1954-S33','1954-05-26 14:24:00','suborbital rocket','success',5,'Geophysical rocket','[Same as R-2]','','','Ionos',636,'',106 from dual union all
select 28520,'1972-S374','1972-10-31 10:30:00','suborbital rocket','success',740,'','','','','Aeron?',636,'',120 from dual union all
select 28521,'1972-S375','1972-10-31 23:32:00','suborbital rocket','success',848,'CRL A07.106-1','HAVE GENIE II','Release','','Aeron',394,'',170 from dual union all
select 28522,'1972-S376','1972-10-31 23:51:00','suborbital rocket','success',848,'CRL A07.105-6','HAVE GENIE II','Release','','Aeron',394,'',192 from dual union all
select 28523,'1972-S377','1972-11-01 01:04:00','suborbital rocket','success',848,'CRL A07.105-5','HAVE GENIE II','Release','','Aeron',394,'',174 from dual union all
select 28524,'1972-S378','1972-11-01 01:30:00','suborbital rocket','success',848,'CRL A07.105-4','HAVE GENIE II','Release','','Aeron',394,'',203 from dual union all
select 28525,'1972-S379','1972-11-01 02:40:00','suborbital rocket','success',740,'','','','','Aeron?',636,'',160 from dual union all
select 28526,'1972-S38','1972-01-14 18:13:00','suborbital rocket','success',920,'P89T/C','ISRO 18.04','TERLS-404','','Ionospheric',1467,'',128 from dual union all
select 28527,'1972-S380','1972-11-01 05:03:00','suborbital rocket','success',837,'NASA 14.439UI','','','','Ionos',1821,'',195 from dual union all
select 28528,'1972-S381','1972-11-01 05:30:00','suborbital rocket','success',37,'NRL NB4.257','NRL NB4.257','','','XR Astron',1895,'',205 from dual union all
select 28529,'1972-S382','1972-11-01 09:33:00','suborbital rocket','success',848,'CRL A07.216-2','HIBAL Sphere','','','Aeron',394,'',220 from dual union all
select 2853,'1954-S34','1954-05-26 00:00:00','military missile','success',992,'Prod. missile','NII-88 Prod GIM-2','','','Test',636,'',150 from dual union all
select 28530,'1972-S383','1972-11-02 00:00:00','military missile','success',939,'','FOT-3','','','OT',421,'SSBN 630',500 from dual union all
select 28531,'1972-S384','1972-11-02 00:00:00','military missile','success',939,'','FOT-3','','','OT',421,'SSBN 630',500 from dual union all
select 28532,'1972-S385','1972-11-02 00:00:00','military missile','success',939,'','FOT-3','','','OT',421,'SSBN 630',500 from dual union all
select 28533,'1972-S386','1972-11-02 00:00:00','military missile','success',939,'','FOT-3','','','OT',421,'SSBN 630',500 from dual union all
select 28534,'1972-S387','1972-11-02 04:27:00','suborbital rocket','success',228,'NASA 21.02UI','','','','Ionos',428,'',237 from dual union all
select 28535,'1972-S388','1972-11-02 06:10:00','suborbital rocket','success',1211,'ESRO S95','ESRO S95','','','XR Astron/Solar X',1839,'',211 from dual union all
select 28536,'1972-S389','1972-11-02 23:28:00','suborbital rocket','success',1058,'EGANI/72','MPE Eu/Ba  EXP 63','','','Aeron/Ionos',310,'',240 from dual union all
select 28537,'1972-S39','1972-01-15 05:37:00','suborbital rocket','success',218,'AAF-3B-53','','','','Auroral',598,'',190 from dual union all
select 28538,'1972-S390','1972-11-03 10:10:00','military missile','success',744,'67023-302','15A15 BI-4-2','GCh 8F677','','Test',530,'',1000 from dual;
insert into LAUNCH
select 28539,'1972-S391','1972-11-03 11:45:48','suborbital rocket','success',920,'P129A','LP/VLF 4 Pol chorus','','','Auroral',29,'',140 from dual union all
select 2854,'1954-S35','1954-06-02 16:10:00','suborbital rocket','success',51,'USAF 47','Solar UV spectrum','','','Test',640,'',93 from dual union all
select 28540,'1972-S392','1972-11-04 02:15:00','suborbital rocket','success',837,'NASA 14.487UA','','','','Aeronomy',1895,'',123 from dual union all
select 28541,'1972-S393','1972-11-04 23:27:00','suborbital rocket','success',1058,'EGANI/72','MPE Eu/Ba  EXP 64','','','Aeron/Ionos',310,'',252 from dual union all
select 28542,'1972-S394','1972-11-06 00:04:00','suborbital rocket','success',867,'CRL A08.215-2','Sphere','','','Aeron',394,'',212 from dual union all
select 28543,'1972-S395','1972-11-08 08:30:00','suborbital rocket','success',37,'NASA 13.86GG','','','','UV Astron',1895,'',200 from dual union all
select 28544,'1972-S396','1972-11-08 22:56:24','suborbital rocket','success',920,'P41A','Particles/substorm','','','Auroral',29,'',140 from dual union all
select 28545,'1972-S397','1972-11-11 08:26:00','suborbital rocket','success',1058,'EGANI/72','MPE Eu/Ba  EXP 65','','','Ionos',310,'',260 from dual union all
select 28546,'1972-S398','1972-11-14 18:48:00','suborbital rocket','success',37,'NASA 13.02GS','','','','Solar',1895,'',200 from dual union all
select 28547,'1972-S40','1972-01-15 06:01:00','suborbital rocket','success',227,'AAF-VB-33','[Not in FC]','','','Aeron/Plasma',428,'',300 from dual union all
select 28548,'1972-S400','1972-11-20 04:54:00','suborbital rocket','success',1206,'ESRO S94','ESRO S94','','','XR Astronomy',1343,'',210 from dual union all
select 28549,'1972-S401','1972-11-20 17:07:00','suborbital rocket','success',41,'NASA 26.13GT','','','','Test',1895,'',250 from dual union all
select 2855,'1954-S36','1954-06-11 00:00:00','military missile','success',970,'No 12-1953','','','','OT',636,'',100 from dual union all
select 28550,'1972-S402','1972-11-23 03:00:00','suborbital rocket','success',740,'','','','','Aeron?',636,'',180 from dual union all
select 28551,'1972-S403','1972-11-25 02:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT18GB','GLORY TRIP 18GB','OT',1708,'',1300 from dual union all
select 28552,'1972-S404','1972-11-25 00:00:00','military missile','success',1514,'','','','','Test',190,'PLBR',1000 from dual union all
select 28553,'1972-S405','1972-11-28 00:00:00','suborbital rocket','success',837,'','','','','Ionos/Aeron?',1412,'',200 from dual union all
select 28554,'1972-S406','1972-11-28 00:00:00','military missile','success',1514,'','','','','Test',190,'PLBR',1000 from dual union all
select 28555,'1972-S407','1972-11-28 00:00:00','military missile','success',1514,'','','','','Test',190,'PLBR',1000 from dual union all
select 28556,'1972-S408','1972-12-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 28557,'1972-S409','1972-12-01 01:23:00','suborbital rocket','success',218,'AND-3B-63','Microwave background','','','Submm Astron',428,'',185 from dual union all
select 28558,'1972-S41','1972-01-15 06:04:00','suborbital rocket','success',220,'AAF-4B-25','[Not in FC]','','','Ionos',428,'',800 from dual union all
select 28559,'1972-S410','1972-12-01 05:04:00','suborbital rocket','success',740,'','','','','Aeron?',636,'',165 from dual union all
select 2856,'1954-S37','1954-06-12 00:00:00','military missile','success',970,'No 720-1952','','','','OT',636,'',100 from dual union all
select 28560,'1972-S411','1972-12-01 09:02:00','suborbital rocket','success',740,'','','','','Aeron?',636,'',180 from dual union all
select 28561,'1972-S412','1972-12-01 12:39:00','suborbital rocket','success',740,'','','','','Ionos',636,'',160 from dual union all
select 28562,'1972-S413','1972-12-05 02:35:00','military missile','success',799,'First phase 2 OT','Mk 12 RV x 3','FOT GT41GM','GLORY TRIP 41GM','OT',1697,'',1300 from dual union all
select 28563,'1972-S414','1972-12-05 04:20:00','suborbital rocket','success',37,'CRL A04.004-9','HI STAR 7','','','IR Astron',1873,'',179 from dual union all
select 28564,'1972-S415','1972-12-05 07:00:00','suborbital rocket','success',228,'CRL A18.006-5','Auroral IR','','','Aeron',428,'',194 from dual union all
select 28565,'1972-S416','1972-12-05 17:30:00','suborbital rocket','success',837,'NASA 14.509UI','','','','Ionos',1821,'',185 from dual union all
select 28566,'1972-S417','1972-12-07 05:03:00','suborbital rocket','success',1207,'SL1202','LUX XRS13','','','XR Astronomy',1839,'',248 from dual union all
select 28567,'1972-S418','1972-12-07 05:45:00','suborbital rocket','success',1468,'CRL A09.107-2','Auroral composition','','','Aeron',428,'',171 from dual union all
select 28568,'1972-S419','1972-12-07 07:07:00','suborbital rocket','success',848,'CRL A07.307-3','7-in Density Sphere','','','Aeron',428,'',210 from dual union all
select 28569,'1972-S42','1972-01-15 13:37:00','suborbital rocket','success',920,'P85H','Solar La/X/ne','','','Ionos/Solar X',1447,'',147 from dual union all
select 2857,'1954-S38','1954-06-14 00:00:00','military missile','success',970,'No 14-1951','','','','OT',636,'',100 from dual union all
select 28570,'1972-S420','1972-12-08 00:00:00','military missile','success',795,'','SSTTP','SSTTP M2-21','','Target',1701,'',1300 from dual union all
select 28571,'1972-S422','1972-12-08 06:26:00','suborbital rocket','success',37,'NASA 13.083UG','Wisc. Aerobee-III','','','XR Astron',1895,'',200 from dual union all
select 28572,'1972-S423','1972-12-11 00:00:00','suborbital rocket','success',1207,'SL1005','QUB AGP9/SHF-11','','','Ionos/Aeron',1839,'',196 from dual union all
select 28573,'1972-S424','1972-12-11 00:00:00','military missile','success',939,'','FOT-4','','','OT',421,'SSBN 629',500 from dual union all
select 28574,'1972-S425','1972-12-11 00:00:00','military missile','success',939,'','FOT-4','','','OT',421,'SSBN 629',500 from dual union all
select 28575,'1972-S426','1972-12-11 00:00:00','military missile','success',939,'','FOT-4','','','OT',421,'SSBN 629',500 from dual union all
select 28576,'1972-S427','1972-12-11 00:00:00','military missile','success',939,'','FOT-4','','','OT',421,'SSBN 629',500 from dual union all
select 28577,'1972-S428','1972-12-11 01:30:00','suborbital rocket','success',848,'CRL A07.107-3','Auroral composition','','','Aeron',428,'',136 from dual union all
select 28578,'1972-S429','1972-12-11 01:39:00','suborbital rocket','success',848,'CRL A07.301-1','7-in Density Sphere','','','Aeron',428,'',255 from dual union all
select 28579,'1972-S43','1972-01-18 02:07:00','suborbital rocket','success',37,'CRL A04.004-6','HI STAR 4','','','IR Astron',1873,'',161 from dual union all
select 2858,'1954-S40','1954-06-26 13:24:00','suborbital rocket','success',987,'1','1RD','','','Test/Bio/Ionos/Aeron',636,'',106 from dual union all
select 28580,'1972-S430','1972-12-11 21:38:00','suborbital rocket','success',38,'NASA 13.96UA','JHU Airglow 12','','','Aeronomy',1895,'',214 from dual union all
select 28581,'1972-S431','1972-12-12 00:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT19GB','GLORY TRIP 19GB','OT',1711,'',1300 from dual union all
select 28582,'1972-S432','1972-12-12 00:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT20GB','GLORY TRIP 20GB','OT',1712,'',1300 from dual union all
select 28583,'1972-S433','1972-12-13 05:18:00','suborbital rocket','success',446,'D-38','FU-206 Protons/elec','','','Ionos/geocorona',310,'',380 from dual union all
select 28584,'1972-S434','1972-12-13 14:19:59','suborbital rocket','success',1535,'Zenit-Cuckoo','DLR A-ZC-48 Astrik','','','Ionos',29,'',144 from dual union all
select 28585,'1972-S435','1972-12-13 21:03:00','suborbital rocket','success',37,'NASA 13.94US','','','','Solar',1895,'',200 from dual union all
select 28586,'1972-S436','1972-12-13 21:23:00','suborbital rocket','success',1070,'S-210JA-7','','','','Auroral/Ionos/Field',1458,'',126 from dual union all
select 28587,'1972-S437','1972-12-14 00:00:00','military missile','success',1514,'','','','','Test',190,'PLBR',1000 from dual union all
select 28588,'1972-S438','1972-12-14 00:00:00','military missile','success',1514,'','','','','Test',190,'PLBR',1000 from dual union all
select 28589,'1972-S439','1972-12-14 00:00:00','military missile','success',1514,'','','','','Test',190,'PLBR',1000 from dual union all
select 2859,'1954-S41','1954-07-02 00:00:00','suborbital rocket','success',987,'2','1RD','','','Test/Bio/Ionos/Aeron',636,'',100 from dual union all
select 28590,'1972-S44','1972-01-20 00:00:00','military missile','success',1251,'800030','SAFEGUARD M2-6','','','Interceptor',868,'',88 from dual union all
select 28591,'1972-S440','1972-12-14 00:00:00','military missile','success',1514,'','','','','Test',190,'PLBR',1000 from dual union all
select 28592,'1972-S441','1972-12-15 21:25:00','military missile','success',939,'','DASO-23','','','OT',421,'SSBN 642',500 from dual union all
select 28593,'1972-S442','1972-12-18 21:15:00','military missile','success',939,'','DASO-24','','','OT',421,'SSBN 644',500 from dual union all
select 28594,'1972-S443','1972-12-19 00:00:00','military missile','success',795,'','HK-3','Army STP HK-3','','Reentry Test?',1701,'',1300 from dual union all
select 28595,'1972-S444','1972-12-22 04:55:00','military missile','success',796,'','Mk 11 RV','FOT GT111M','GLORY TRIP 111GM','OT',1702,'',1300 from dual union all
select 28596,'1972-S445','1972-12-22 05:00:00','suborbital rocket','success',740,'','','','','Aeron?',636,'',160 from dual union all
select 28597,'1972-S446','1972-12-26 08:00:00','military missile','success',744,'45053-301-1L','15A15 LKI-1 (GSLI)','GCh 15F152 / 8F677?','','Test',530,'',1000 from dual union all
select 28598,'1972-S447','1972-06-29 00:00:00','military missile','success',917,'','P-239 72 Spring 20','','','OT',615,'',180 from dual union all
select 28599,'1972-S448','1972-04-12 01:05:00','suborbital rocket','success',1284,'','ALADDIN II sphere','','','Aeron',394,'',115 from dual union all
select 286,'1944-S26','1944-05-26 11:00:00','military missile','success',8,'','','','','Test',675,'',90 from dual union all
select 2860,'1954-S42','1954-07-07 00:00:00','suborbital rocket','success',987,'3','1RD','','','Test/Bio/Ionos/Aeron',636,'',100 from dual union all
select 28600,'1972-S449','1972-03-04 00:03:00','suborbital rocket','success',740,'MR-12','','','','Aeron/Plasma',1010,'',170 from dual union all
select 28601,'1972-S45','1972-01-21 02:24:00','suborbital rocket','success',1070,'S-210-6','','','','Ionos',839,'',112 from dual union all
select 28602,'1972-S450','1972-04-15 08:38:00','suborbital rocket','success',740,'','Composition','','','Aeron',1010,'',150 from dual union all
select 28603,'1972-S451','1972-03-04 23:52:00','suborbital rocket','success',740,'','Composition','','','Aeron',1010,'',150 from dual union all
select 28604,'1972-S452','1972-01-01 00:00:00','military missile','success',1514,'R-29','','','','Test',190,'K-145',1000 from dual union all
select 28605,'1972-S453','1972-03-01 00:00:00','military missile','success',1514,'R-29','','','','Test',190,'K-145',1000 from dual union all
select 28606,'1972-S454','1972-04-01 00:00:00','military missile','success',1514,'R-29','','','','Test',190,'K-145',1000 from dual union all
select 28607,'1972-S455','1972-09-19 00:00:00','suborbital rocket','success',1257,'D1-6160 SSL 1-1','Robin Dart E4-1','','','Test',1821,'',99 from dual union all
select 28608,'1972-S456','1972-09-19 00:00:00','suborbital rocket','success',1257,'D1-6161 SSL 1-2','Robin Dart E4-1','','','Test',1821,'',99 from dual union all
select 28609,'1972-S457','1972-09-19 00:00:00','suborbital rocket','success',1257,'D1-6162 SSL 1-3','Robin Dart E4-1','','','Test',1821,'',99 from dual union all
select 2861,'1954-S43','1954-07-14 13:55:00','suborbital rocket','success',51,'USAF 48','UM wind vanes','','','Aeronomy',640,'',92 from dual union all
select 28610,'1972-S458','1972-10-02 00:00:00','suborbital rocket','success',1257,'D1-6163 SSL 1-8','Robin Dart E4-1A','','','Test',1821,'',97 from dual union all
select 28611,'1972-S459','1972-09-27 00:00:00','suborbital rocket','success',1502,'Stable Viper 1-4','Robin Dart','','','Test',1821,'',116 from dual union all
select 28612,'1972-S46','1972-01-21 19:02:20','suborbital rocket','success',291,'C76/1','ESRO C76/1','','','Particles/Auroral',408,'',130 from dual union all
select 28613,'1972-S460','1972-09-27 00:00:00','suborbital rocket','success',1502,'Stable Viper 1-5','Robin Dart','','','Test',1821,'',121 from dual union all
select 28614,'1972-S461','1972-10-02 00:00:00','suborbital rocket','success',1502,'Stable Viper 1-6','Robin Dart','','','Test',1821,'',120 from dual union all
select 28615,'1972-S462','1972-02-08 05:59:00','test rocket','success',645,'','K63D BOR-2','BOR-2 No. 104','','Reentry Test',1752,'',100 from dual union all
select 28616,'1972-S463','1972-11-22 22:55:00','suborbital rocket','success',331,'WRE C1029','O2 density','','','Aeron',1835,'',131 from dual union all
select 28617,'1972-S464','1972-07-20 00:00:00','suborbital rocket','success',331,'WRE C1020','O2 density','','','Aeron',1835,'',130 from dual union all
select 28618,'1972-S465','1972-02-23 03:34:00','suborbital rocket','success',675,'WRE K1062','','','','Aeron',1832,'',85 from dual union all
select 28619,'1972-S466','1972-03-22 03:00:00','suborbital rocket','success',675,'WRE K1063','','','','Aeron',1832,'',85 from dual union all
select 2862,'1954-S44','1954-07-16 12:13:00','suborbital rocket','success',370,'','SUI 24','','','Ionos/Aeron?',61,'WYG-279',90 from dual union all
select 28620,'1972-S467','1972-04-19 02:45:00','suborbital rocket','success',675,'WRE K1064','','','','Aeron',1832,'',85 from dual union all
select 28621,'1972-S468','1972-08-23 02:30:00','suborbital rocket','success',675,'WRE K1066','','','','Aeron',1832,'',85 from dual union all
select 28622,'1972-S469','1972-09-13 04:00:00','suborbital rocket','success',675,'WRE K1067','','','','Aeron',1832,'',85 from dual union all
select 28623,'1972-S47','1972-01-21 21:45:00','military missile','success',939,'','DASO-19','','','OT',421,'SSBN 634',500 from dual union all
select 28624,'1972-S470','1972-11-15 02:50:00','suborbital rocket','success',675,'WRE K1069','','','','Aeron',1832,'',85 from dual union all
select 28625,'1972-S471','1972-12-13 03:06:00','suborbital rocket','success',675,'WRE K1070','','','','Aeron',1832,'',85 from dual union all
select 28626,'1972-S472','1972-07-19 08:15:00','suborbital rocket','success',538,'WRE HAT 638','','','','Aeron',1837,'',80 from dual union all
select 28627,'1972-S473','1972-10-11 09:03:00','suborbital rocket','success',538,'WRE HAT 639','','','','Aeron',1837,'',80 from dual union all
select 28628,'1972-S474','1972-11-11 00:00:00','suborbital rocket','success',466,'E008','FU-201 LPSP-EUV2','','','Solar EUV',340,'',300 from dual;
insert into LAUNCH
select 28629,'1972-S475','1972-03-15 19:26:00','suborbital rocket','success',1281,'WMO','','','','Aeron',1821,'',80 from dual union all
select 2863,'1954-S45','1954-07-20 02:55:00','suborbital rocket','success',370,'','NRL Rockoon 8','','','Ionos/Aeron?',67,'WYG-279',90 from dual union all
select 28630,'1972-S476','1972-03-16 00:39:00','suborbital rocket','success',1281,'WMO','','','','Aeron',1821,'',80 from dual union all
select 28631,'1972-S477','1972-03-20 18:27:00','suborbital rocket','success',1281,'WMO','','','','Aeron',1821,'',80 from dual union all
select 28632,'1972-S478','1972-03-20 20:45:00','suborbital rocket','success',1281,'WMO','','','','Aeron',1821,'',80 from dual union all
select 28633,'1972-S479','1972-03-21 01:46:00','suborbital rocket','success',1281,'WMO','','','','Aeron',1821,'',80 from dual union all
select 28634,'1972-S48','1972-01-22 00:58:30','suborbital rocket','success',291,'C76/2','ESRO C76/2','','','Particles/Auroral',408,'',131 from dual union all
select 28635,'1972-S480','1972-01-15 00:00:00','military missile','success',982,'G 22504-17T','8K64U UBP-P86','R-16 PL72-09','','OT',477,'',1210 from dual union all
select 28636,'1972-S481','1972-04-18 00:00:00','military missile','success',982,'G 22503-12T','8K64U UBP-P91','R-16 PL72-10','','OT',477,'',1210 from dual union all
select 28637,'1972-S482','1972-08-17 00:00:00','military missile','success',982,'G 22502-02T','8K64U UBP-P97','R-16 PL72-11','','OT',477,'',1210 from dual union all
select 28638,'1972-S483','1972-01-28 00:00:00','military missile','success',982,'G 22503-03T','8K64U UBP-P87','R-16 PL72-01','','OT',477,'',1210 from dual union all
select 28639,'1972-S484','1972-01-29 00:00:00','military missile','success',982,'G 22503-06T','8K64U UBP-P88','R-16 PL72-02','','OT',477,'',1210 from dual union all
select 2864,'1954-S46','1954-07-21 12:45:00','suborbital rocket','success',370,'','SUI 28','','','Ionos/Aeron?',74,'WYG-279',90 from dual union all
select 28640,'1972-S485','1972-03-17 00:00:00','military missile','success',982,'G 22502-11T','8K64U UBP-P89','R-16 PL72-03','','OT',477,'',1210 from dual union all
select 28641,'1972-S486','1972-03-30 00:00:00','military missile','success',982,'G 22502-17T','8K64U UBP-P90','R-16 PL72-04','','OT',477,'',1210 from dual union all
select 28642,'1972-S487','1972-06-02 00:00:00','military missile','success',982,'G 22504-02T','8K64U UBP-P92','R-16 PL72-05','','OT',477,'',1210 from dual union all
select 28643,'1972-S488','1972-06-16 00:00:00','military missile','success',982,'G 22504-03T','8K64U UBP-P93','R-16 PL72-06','','OT',477,'',1210 from dual union all
select 28644,'1972-S489','1972-06-28 00:00:00','military missile','success',982,'G 22504-07T','8K64U UBP-P94','R-16 PL72-07','','OT',477,'',1210 from dual union all
select 28645,'1972-S49','1972-01-22 12:44:00','suborbital rocket','success',920,'P107H','Solar La/X/ne','','','Ionos/Solar X',1447,'',142 from dual union all
select 28646,'1972-S490','1972-07-20 00:00:00','military missile','success',982,'G 22501-03T','8K64U UBP-P95','R-16 PL72-08','','OT',477,'',1210 from dual union all
select 28647,'1972-S491','1972-11-14 00:00:00','military missile','success',1037,'V 29501-06','8K98 UBP-18','','','Test',477,'',1000 from dual union all
select 28648,'1972-S496','1972-08-03 00:00:00','military missile','failure',1322,'4L','15Zh42 LKI-4','','','Test',477,'',0 from dual union all
select 28649,'1972-S497','1972-01-01 00:00:00','military missile','success',1449,'','8K84 54 RD UBP-13','','','OT',1468,'',1000 from dual union all
select 2865,'1954-S47','1954-07-22 19:15:00','suborbital rocket','success',365,'','','','','GR Astron',447,'',86 from dual union all
select 28650,'1972-S498','1972-01-01 00:00:00','military missile','success',1449,'','8K84 54 RD UBP-14','','','OT',1468,'',1000 from dual union all
select 28651,'1972-S499','1972-03-14 00:00:00','military missile','success',1018,'','8K75 UBP Omsk-4','','','OT',1012,'',1160 from dual union all
select 28652,'1972-S50','1972-01-23 11:22:00','suborbital rocket','success',668,'K-9M-37','','','','Ionos',839,'',340 from dual union all
select 28653,'1972-S51','1972-01-26 11:25:00','military missile','success',799,'','Mk 12 RV x 3','OT GT11GM','GLORY TRIP 11GM','OT',1697,'',1300 from dual union all
select 28654,'1972-S52','1972-01-27 18:16:00','suborbital rocket','success',228,'NASA 21.06GT','MSFC RR-1','','','Microgravity',1821,'',232 from dual union all
select 28655,'1972-S53','1972-01-28 21:08:00','suborbital rocket','success',132,'CRL A30.116-1','','','','Plasma',1895,'',117 from dual union all
select 28656,'1972-S54','1972-01-29 00:00:00','suborbital rocket','success',1070,'DT-210-1','Structural Dynamics','','','Test',839,'',100 from dual union all
select 28657,'1972-S55','1972-01-29 05:00:00','suborbital rocket','success',132,'CRL A30.116-2','','','','Plasma',1895,'',122 from dual union all
select 28658,'1972-S56','1972-02-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 28659,'1972-S57','1972-01-31 17:30:00','suborbital rocket','success',837,'NASA 14.440UI','','','','Ionos',1821,'',200 from dual union all
select 2866,'1954-S48','1954-07-23 14:46:00','suborbital rocket','success',370,'','SUI 30','','','Ionos/Aeron?',76,'WYG-279',90 from dual union all
select 28660,'1972-S58','1972-01-31 17:45:00','suborbital rocket','success',848,'CRL A07.217-1','Sphere','','','Aeron',1821,'',220 from dual union all
select 28661,'1972-S59','1972-01-31 21:50:00','suborbital rocket','success',848,'CRL AT07.897','Mass spectrometer','','','Aeron',1821,'',117 from dual union all
select 28662,'1972-S60','1972-02-02 05:30:00','suborbital rocket','success',920,'P110T/C','ISRO 18.06','TERLS-410','','Ionospheric',1467,'',134 from dual union all
select 28663,'1972-S61','1972-02-02 11:25:00','suborbital rocket','success',867,'NASA 18.111UE','','','','Plasma',1047,'',173 from dual union all
select 28664,'1972-S62','1972-02-03 10:27:00','suborbital rocket','success',331,'WRE C1023','Sphere','','','Aeron',1835,'',100 from dual union all
select 28665,'1972-S63','1972-02-03 13:55:00','suborbital rocket','success',228,'N-BB/V-42','DLR N-BBV-42','','','Auroral/Ionos?',330,'',250 from dual union all
select 28666,'1972-S64','1972-02-04 00:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT12GM','GLORY TRIP 12GM','OT',1700,'',1300 from dual union all
select 28667,'1972-S65','1972-02-04 16:15:00','military missile','success',931,'A3T','DASO','','','OT',421,'SSBN 598',1000 from dual union all
select 28668,'1972-S66','1972-02-05 06:00:00','suborbital rocket','success',920,'P59T/C','ISRO 18.07','TERLS-412','','Ionospheric',1467,'',139 from dual union all
select 28669,'1972-S67','1972-02-06 06:35:00','suborbital rocket','success',613,'JCR-7','','','','Test',1476,'',145 from dual union all
select 2867,'1954-S49','1954-07-23 17:09:00','suborbital rocket','success',370,'','NRL Rockoon 9','','','Ionos/Aeron?',68,'WYG-279',90 from dual union all
select 28670,'1972-S68','1972-02-06 17:15:00','suborbital rocket','success',848,'CRL A07.001-1','Mass spectrometer','','','Aeron',1821,'',95 from dual union all
select 28671,'1972-S69','1972-02-07 23:33:34','suborbital rocket','success',867,'NASA 18.131IA','RTG SNT 7/1 (S7-1)','','','Aeronomy',414,'',180 from dual union all
select 28672,'1972-S70','1972-02-08 09:45:00','suborbital rocket','success',217,'ADD-3A-54','IR airglow profile','','','Aeron/Ionos/IR Astr',428,'',121 from dual union all
select 28673,'1972-S71','1972-02-09 00:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT13GM','GLORY TRIP 13GM','OT',1699,'',1300 from dual union all
select 28674,'1972-S72','1972-02-10 17:02:00','suborbital rocket','success',624,'NASA 8.60AE','Composition','','','Plasma',1821,'',851 from dual union all
select 28675,'1972-S73','1972-02-11 03:20:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/Plasma',1010,'',177 from dual union all
select 28676,'1972-S74','1972-02-11 12:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Plasma',1010,'',173 from dual union all
select 28677,'1972-S75','1972-02-11 12:00:00','suborbital rocket','success',1070,'S-210JA-12','Mass spec.','','','Ionos',1458,'',108 from dual union all
select 28678,'1972-S754','1972-03-23 00:00:00','military missile','success',982,'','8K64U 36 RD UBP-6','','','OT',898,'',1210 from dual union all
select 28679,'1972-S755','1972-03-23 00:00:00','military missile','success',982,'','8K64U 36 RD UBP-7','','','OT',898,'',1210 from dual union all
select 2868,'1954-S50','1954-07-23 17:54:00','suborbital rocket','success',370,'','SUI 31','','','Ionos/Aeron?',78,'WYG-279',90 from dual union all
select 28680,'1972-S756','1972-06-29 00:00:00','military missile','success',981,'','8K64 8 RD UBP-4','Vostok-72','','OT',1916,'',1210 from dual union all
select 28681,'1972-S757','1972-04-26 11:23:00','suborbital rocket','success',1189,'INTA MSCL-7202','','','','Ionos',396,'',94 from dual union all
select 28682,'1972-S758','1972-06-28 11:15:00','suborbital rocket','success',1189,'INTA MSC-7210','','','','Ionos',396,'',90 from dual union all
select 28683,'1972-S759','1972-10-17 19:51:00','suborbital rocket','success',331,'WRE C1047','Li release','','','Aeron',1835,'',115 from dual union all
select 28684,'1972-S76','1972-02-13 12:55:00','suborbital rocket','success',920,'P60T/C','ISRO 18.08','TERLS-414','','Ionospheric',1467,'',137 from dual union all
select 28685,'1972-S760','1972-07-31 00:00:00','military missile','success',982,'G 22503-17T','8K64U UBP-P96','R-16 PL72-08','','OT',477,'',1210 from dual union all
select 28686,'1972-S761','1972-08-31 00:00:00','military missile','success',982,'R 22604-01T','8K64U UBP-P98','R-16 PL72-11','','OT',477,'',1210 from dual union all
select 28687,'1972-S762','1972-12-02 00:00:00','military missile','success',982,'R 22604-10T','8K64U UBP-P99','R-16 PL72-11','','OT',477,'',1210 from dual union all
select 28688,'1972-S763','1972-02-15 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 28689,'1972-S764','1972-05-01 00:00:00','military missile','unknown',980,'','8K65U UBP 35 RD','','','Test',636,'',675 from dual union all
select 2869,'1954-S51','1954-07-24 08:57:00','suborbital rocket','success',370,'','SUI 33','','','Ionos/Aeron?',79,'WYG-279',90 from dual union all
select 28690,'1972-S765','1972-05-19 00:00:00','military missile','unknown',980,'','8K65U UBP 29 RD','','','Test',636,'',675 from dual union all
select 28691,'1972-S766','1972-05-26 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 28692,'1972-S767','1972-07-08 00:00:00','military missile','unknown',979,'','8K65 UBP 32 RD','','','Test',636,'',675 from dual union all
select 28693,'1972-S77','1972-02-13 23:20:11','suborbital rocket','success',867,'NASA 18.137IE','Ferdinand 31/Proton 1','','','Plasma',29,'',222 from dual union all
select 28694,'1972-S78','1972-02-14 00:00:00','suborbital rocket','success',227,'','[Not in FC]','','','Auroral/Ionos?',428,'',309 from dual union all
select 28695,'1972-S79','1972-02-15 03:48:00','suborbital rocket','success',215,'ADD-2-114','Auroral VUV','','','Auroral/Ionos',428,'',133 from dual union all
select 28696,'1972-S80','1972-02-15 04:21:00','suborbital rocket','success',837,'NASA 14.486UA','Auroral NO','','','Aeronomy',428,'',141 from dual union all
select 28697,'1972-S81','1972-02-17 08:00:00','suborbital rocket','success',1070,'S-210-7','','','','Ionos',839,'',118 from dual union all
select 28698,'1972-S82','1972-02-17 13:48:00','suborbital rocket','success',920,'P111H','Faraday rotation','','','Ionospheric',1447,'',134 from dual union all
select 28699,'1972-S83','1972-02-18 02:33:00','suborbital rocket','success',740,'MR-12','','','','Aeron/Ionos/Plasma',1010,'',168 from dual union all
select 287,'1944-S260','1944-11-24 14:17:00','military missile','success',1470,'19833','Ma303','','','Test',661,'',90 from dual union all
select 2870,'1954-S52','1954-07-24 13:16:00','suborbital rocket','success',370,'','SUI 34','','','Ionos/Aeron?',62,'WYG-279',90 from dual union all
select 28700,'1972-S84','1972-02-18 09:28:00','suborbital rocket','success',668,'K-9M-39','Cs Release','','','Ionos',839,'',310 from dual union all
select 28701,'1972-S85','1972-02-19 05:12:00','suborbital rocket','success',37,'NASA 13.082UG','','','','XR Astron',1895,'',200 from dual union all
select 28702,'1972-S86','1972-02-21 02:00:00','suborbital rocket','success',1069,'S-160-4','','','','Fields',839,'',81 from dual union all
select 28703,'1972-S87','1972-02-21 12:55:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',1010,'',165 from dual union all
select 28704,'1972-S88','1972-02-21 13:35:00','suborbital rocket','success',740,'MR-12','Ionos','','','Aeron/Plasma',1010,'',187 from dual union all
select 28705,'1972-S89','1972-02-22 06:00:00','suborbital rocket','success',668,'K-9M-38','','','','Aeron/Ionos',839,'',352 from dual union all
select 28706,'1972-S90','1972-02-23 10:06:00','suborbital rocket','success',331,'WRE C1024','Sphere','','','Aeron',1835,'',100 from dual union all
select 28707,'1972-S91','1972-02-25 07:23:00','suborbital rocket','success',867,'NASA 18.112UE','','','','Plasma',1047,'',270 from dual union all
select 28708,'1972-S92','1972-02-28 05:54:00','suborbital rocket','success',1207,'ESRO S84','ESRO S84','','','Solar UV',1343,'',242 from dual union all
select 28709,'1972-S93','1972-02-28 09:47:03','suborbital rocket','success',1207,'ESRO S90','ESRO S90','','','Solar UV',1343,'',251 from dual union all
select 2871,'1954-S53','1954-07-25 06:51:00','suborbital rocket','success',370,'','SUI 35','','','Ionos/Aeron?',63,'WYG-279',90 from dual union all
select 28710,'1972-S94','1972-03-02 01:17:00','suborbital rocket','unknown',740,'','','','','Plasma',1010,'',170 from dual union all
select 28711,'1972-S95','1972-03-02 18:27:00','suborbital rocket','success',30,'NRL NA3.218','','','','Solar X',1869,'',200 from dual union all
select 28712,'1972-S96','1972-03-02 22:26:50','suborbital rocket','success',1208,'ESRO S77/1','ESRO S77/1','','','Auroral',416,'',237 from dual union all
select 28713,'1972-S97','1972-03-04 11:52:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Plasma',1010,'',174 from dual union all
select 28714,'1972-S98','1972-03-04 13:22:00','suborbital rocket','success',740,'MR-12','Polar thermosphere','','','Aeron/Ionos/CR',1010,'',184 from dual union all
select 28715,'1972-S99','1972-03-04 18:48:00','suborbital rocket','success',740,'MR-12','Polar thermosphere','','','Aeron/Ionos/CR',1010,'',173 from dual union all
select 28716,'1972-U01','1972-02-22 22:58:00','lunar return','success',450,'No. 408','Luna-20','VA','','Sat',166,'',null from dual union all
select 28717,'1972-U02','1972-04-24 01:25:48','lunar return','success',689,'11','Apollo 16','LM 11 Ascent Stage','','Sat',364,'',null from dual union all
select 28718,'1972-U03','1972-12-14 22:54:37','lunar return','success',689,'12','Apollo 17','LM 12 Ascent Stage','','Sat',934,'',null from dual;
insert into LAUNCH
select 28719,'1972-W01','1972-08-04 17:00:00','suborbital rocket','success',1281,'PCA','AFCRL Arcas 71-5P','','','Ionos',1470,'',60 from dual union all
select 2872,'1954-S54','1954-07-25 12:36:00','suborbital rocket','success',370,'','SUI 36','','','Ionos/Aeron?',64,'WYG-279',90 from dual union all
select 28720,'1972-W02','1972-08-05 21:02:00','suborbital rocket','success',88,'PCA','AFCRL Arcas 71-A3','','','Ionos',1470,'',60 from dual union all
select 28721,'1972-W03','1972-08-05 22:24:00','suborbital rocket','success',88,'PCA','AFCRL Arcas 71-6P','','','Ionos',1470,'',60 from dual union all
select 28722,'1972-W04','1972-08-08 22:00:00','suborbital rocket','success',88,'PCA','AFCRL Arcas 71-A1','','','Ionos',1470,'',60 from dual union all
select 28723,'1972-W05','1972-01-03 14:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',52 from dual union all
select 28724,'1972-W06','1972-01-03 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 28725,'1972-W07','1972-01-03 17:24:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 28726,'1972-W08','1972-01-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 28727,'1972-W09','1972-01-03 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',68 from dual union all
select 28728,'1972-W10','1972-01-03 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 28729,'1972-W100','1972-01-19 22:44:59','suborbital rocket','success',716,'INTA HW-7201','','','','Meteo',396,'',58 from dual union all
select 2873,'1954-S55','1954-07-25 15:30:00','suborbital rocket','success',370,'','SUI 37','','','Ionos/Aeron?',65,'WYG-279',90 from dual union all
select 28730,'1972-W1000','1972-08-04 14:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 28731,'1972-W1001','1972-08-04 14:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 28732,'1972-W1002','1972-08-04 15:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 28733,'1972-W1003','1972-08-04 19:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 28734,'1972-W1004','1972-08-04 22:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 28735,'1972-W1005','1972-08-04 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 28736,'1972-W1006','1972-08-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 28737,'1972-W1007','1972-08-07 14:55:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',61 from dual union all
select 28738,'1972-W1008','1972-08-07 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 28739,'1972-W1009','1972-08-07 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 2874,'1954-S56','1954-07-25 18:45:00','suborbital rocket','success',370,'','NRL Rockoon 10','','','Aeron',69,'WYG-279',85 from dual union all
select 28740,'1972-W101','1972-01-20 01:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 28741,'1972-W1010','1972-08-07 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 28742,'1972-W1011','1972-08-07 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 28743,'1972-W1012','1972-08-08 02:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',59 from dual union all
select 28744,'1972-W1013','1972-08-08 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 28745,'1972-W1014','1972-08-08 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 28746,'1972-W1015','1972-08-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 28747,'1972-W1016','1972-08-09 14:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 28748,'1972-W1017','1972-08-09 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 28749,'1972-W1018','1972-08-09 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 2875,'1954-S58','1954-07-26 11:02:00','suborbital rocket','success',370,'','NRL Rockoon 12','','','Ionos/Aeron?',71,'WYG-279',90 from dual union all
select 28750,'1972-W1019','1972-08-09 18:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 28751,'1972-W102','1972-01-20 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',64 from dual union all
select 28752,'1972-W1020','1972-08-09 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 28753,'1972-W1021','1972-08-09 20:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 28754,'1972-W1022','1972-08-09 23:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 28755,'1972-W1023','1972-08-11 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 28756,'1972-W1024','1972-08-11 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 28757,'1972-W1025','1972-08-11 14:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 28758,'1972-W1026','1972-08-11 17:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 28759,'1972-W1027','1972-08-11 17:31:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',58 from dual union all
select 2876,'1954-S59','1954-08-17 00:00:00','military missile','success',1009,'K3-2','8A62 LKI III','','','Test',636,'',300 from dual union all
select 28760,'1972-W1028','1972-08-11 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 28761,'1972-W1029','1972-08-11 19:38:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 28762,'1972-W103','1972-01-20 15:42:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 28763,'1972-W1030','1972-08-12 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 28764,'1972-W1031','1972-08-12 04:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',58 from dual union all
select 28765,'1972-W1032','1972-08-12 09:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 28766,'1972-W1033','1972-08-12 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 28767,'1972-W1034','1972-08-13 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 28768,'1972-W1035','1972-08-13 04:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',52 from dual union all
select 28769,'1972-W1036','1972-08-13 17:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 2877,'1954-S60','1954-08-19 00:00:00','military missile','success',1009,'K3-3','8A62 LKI III','','','Test',636,'',300 from dual union all
select 28770,'1972-W1037','1972-08-14 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 28771,'1972-W1038','1972-08-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 28772,'1972-W1039','1972-08-14 15:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 28773,'1972-W104','1972-01-20 21:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 28774,'1972-W1040','1972-08-14 18:01:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 28775,'1972-W1041','1972-08-14 18:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',56 from dual union all
select 28776,'1972-W1042','1972-08-14 19:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 28777,'1972-W1043','1972-08-14 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 28778,'1972-W1044','1972-08-14 21:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 28779,'1972-W1045','1972-08-16 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 2878,'1954-S61','1954-08-02 00:00:00','military missile','success',970,'No 320','','','','OT',636,'',100 from dual union all
select 28780,'1972-W1046','1972-08-16 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 28781,'1972-W1047','1972-08-16 14:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 28782,'1972-W1048','1972-08-16 14:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 28783,'1972-W1049','1972-08-16 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 28784,'1972-W105','1972-01-21 01:00:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',53 from dual union all
select 28785,'1972-W1050','1972-08-16 16:20:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',308,'',62 from dual union all
select 28786,'1972-W1051','1972-08-16 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 28787,'1972-W1052','1972-08-16 18:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 28788,'1972-W1053','1972-08-16 18:35:59','suborbital rocket','success',1506,'T 1-6067','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 28789,'1972-W1054','1972-08-16 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 2879,'1954-S62','1954-08-11 17:25:00','suborbital rocket','success',51,'USAF 49','AF/Utah D-layer','','','Ionos',640,'',92 from dual union all
select 28790,'1972-W1055','1972-08-17 15:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 28791,'1972-W1056','1972-08-17 15:31:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',57 from dual union all
select 28792,'1972-W1057','1972-08-18 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 28793,'1972-W1058','1972-08-18 14:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 28794,'1972-W1059','1972-08-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 28795,'1972-W106','1972-01-21 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 28796,'1972-W1060','1972-08-18 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',61 from dual union all
select 28797,'1972-W1061','1972-08-18 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 28798,'1972-W1062','1972-08-18 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 28799,'1972-W1063','1972-08-18 19:04:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',55 from dual union all
select 288,'1944-S261','1944-11-24 16:57:00','military missile','success',1470,'19844','Ma271','','','Test',661,'',90 from dual union all
select 2880,'1954-S63','1954-08-12 00:00:00','military missile','success',1009,'K3-1','8A62 LKI III','','','Test',636,'',300 from dual union all
select 28800,'1972-W1064','1972-08-18 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 28801,'1972-W1065','1972-08-21 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 28802,'1972-W1066','1972-08-21 14:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 28803,'1972-W1067','1972-08-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 28804,'1972-W1068','1972-08-21 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',50 from dual union all
select 28805,'1972-W1069','1972-08-21 19:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 28806,'1972-W107','1972-01-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 28807,'1972-W1070','1972-08-21 21:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 28808,'1972-W1071','1972-08-23 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual;
insert into LAUNCH
select 28809,'1972-W1072','1972-08-23 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 2881,'1954-S64','1954-08-18 14:04:00','military missile','success',1053,'RS-4     XT','','','','Test',272,'',90 from dual union all
select 28810,'1972-W1073','1972-08-23 15:18:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 28811,'1972-W1074','1972-08-23 15:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 28812,'1972-W1075','1972-08-23 15:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 28813,'1972-W1076','1972-08-23 17:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 28814,'1972-W1077','1972-08-23 18:23:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',59 from dual union all
select 28815,'1972-W1078','1972-08-23 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',68 from dual union all
select 28816,'1972-W1079','1972-08-23 22:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 28817,'1972-W108','1972-01-21 15:42:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 28818,'1972-W1080','1972-08-25 14:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 28819,'1972-W1081','1972-08-25 14:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 2882,'1954-S65','1954-08-27 00:00:00','military missile','success',970,'No 520','','','','OT',636,'',100 from dual union all
select 28820,'1972-W1082','1972-08-25 14:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 28821,'1972-W1083','1972-08-25 16:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 28822,'1972-W1084','1972-08-25 16:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 28823,'1972-W1085','1972-08-25 17:40:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',60 from dual union all
select 28824,'1972-W1086','1972-08-25 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 28825,'1972-W1087','1972-08-25 23:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 28826,'1972-W1088','1972-08-28 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 28827,'1972-W1089','1972-08-28 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 28828,'1972-W109','1972-01-21 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',50 from dual union all
select 28829,'1972-W1090','1972-08-28 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 2883,'1954-S66','1954-08-27 00:00:00','military missile','success',970,'No 420','','','','OT',636,'',100 from dual union all
select 28830,'1972-W1091','1972-08-28 14:48:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 28831,'1972-W1092','1972-08-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 28832,'1972-W1093','1972-08-28 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 28833,'1972-W1094','1972-08-28 17:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',62 from dual union all
select 28834,'1972-W1095','1972-08-28 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 28835,'1972-W1096','1972-08-28 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 28836,'1972-W1097','1972-08-28 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 28837,'1972-W1098','1972-08-30 01:30:00','suborbital rocket','success',761,'MT135P T3','','','','Aeron?',1476,'',50 from dual union all
select 28838,'1972-W1099','1972-08-30 06:15:00','suborbital rocket','success',761,'MT135P T4','','','','Aeron?',1476,'',52 from dual union all
select 28839,'1972-W11','1972-01-04 15:42:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 2884,'1954-S67','1954-08-24 00:00:00','military missile','success',1009,'K3-4','8A62 LKI III','','','Test',636,'',300 from dual union all
select 28840,'1972-W110','1972-01-21 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 28841,'1972-W1100','1972-08-30 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 28842,'1972-W1101','1972-08-30 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 28843,'1972-W1102','1972-08-30 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 28844,'1972-W1103','1972-08-30 14:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',63 from dual union all
select 28845,'1972-W1104','1972-08-30 14:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 28846,'1972-W1105','1972-08-30 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 28847,'1972-W1106','1972-08-30 15:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',308,'',65 from dual union all
select 28848,'1972-W1107','1972-08-30 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 28849,'1972-W1108','1972-08-30 18:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 2885,'1954-S68','1954-08-25 00:00:00','military missile','success',1009,'K3-5','8A62 LKI III','','','Test',636,'',300 from dual union all
select 28850,'1972-W1109','1972-08-30 18:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 28851,'1972-W111','1972-01-21 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',55 from dual union all
select 28852,'1972-W1110','1972-09-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 28853,'1972-W1111','1972-08-31 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 28854,'1972-W1112','1972-09-01 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 28855,'1972-W1113','1972-09-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 28856,'1972-W1114','1972-09-01 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 28857,'1972-W1115','1972-09-01 14:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 28858,'1972-W1116','1972-09-01 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 28859,'1972-W1117','1972-09-01 23:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 2886,'1954-S69','1954-09-05 00:00:00','military missile','success',1009,'K3-11','8A62 LKI III','','','Test',636,'',300 from dual union all
select 28860,'1972-W1118','1972-09-05 17:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',66 from dual union all
select 28861,'1972-W1119','1972-09-05 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 28862,'1972-W112','1972-01-21 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',53 from dual union all
select 28863,'1972-W1120','1972-09-06 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 28864,'1972-W1121','1972-09-06 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 28865,'1972-W1122','1972-09-06 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 28866,'1972-W1123','1972-09-06 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 28867,'1972-W1124','1972-09-06 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',75 from dual union all
select 28868,'1972-W1125','1972-09-06 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',54 from dual union all
select 28869,'1972-W1126','1972-09-06 21:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 2887,'1954-S70','1954-09-17 14:31:00','suborbital rocket','success',51,'USAF 50','Solar flux','','','Solar UV',640,'',94 from dual union all
select 28870,'1972-W1127','1972-09-07 00:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 28871,'1972-W1128','1972-09-07 02:33:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 28872,'1972-W1129','1972-09-07 17:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',77 from dual union all
select 28873,'1972-W113','1972-01-21 19:30:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',61 from dual union all
select 28874,'1972-W1130','1972-09-08 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 28875,'1972-W1131','1972-09-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 28876,'1972-W1132','1972-09-08 14:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',53 from dual union all
select 28877,'1972-W1133','1972-09-08 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 28878,'1972-W1134','1972-09-08 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 28879,'1972-W1135','1972-09-08 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 2888,'1954-S71','1954-09-26 00:00:00','military missile','success',974,'','','','','Test',633,'',150 from dual union all
select 28880,'1972-W1136','1972-09-09 17:20:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',65 from dual union all
select 28881,'1972-W1137','1972-09-10 17:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',62 from dual union all
select 28882,'1972-W1138','1972-09-11 13:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 28883,'1972-W1139','1972-09-11 13:58:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 28884,'1972-W114','1972-01-21 19:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',53 from dual union all
select 28885,'1972-W1140','1972-09-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 28886,'1972-W1141','1972-09-11 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 28887,'1972-W1142','1972-09-11 18:56:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',53 from dual union all
select 28888,'1972-W1143','1972-09-12 01:48:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 28889,'1972-W1144','1972-09-12 20:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',65 from dual union all
select 2889,'1954-S72','1954-10-01 00:00:00','military missile','success',974,'','','','','Test',636,'',150 from dual union all
select 28890,'1972-W1145','1972-09-13 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',55 from dual union all
select 28891,'1972-W1146','1972-09-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 28892,'1972-W1147','1972-09-13 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',58 from dual union all
select 28893,'1972-W1148','1972-09-13 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 28894,'1972-W1149','1972-09-13 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 28895,'1972-W115','1972-01-21 20:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',54 from dual union all
select 28896,'1972-W1150','1972-09-13 16:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 28897,'1972-W1151','1972-09-13 17:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 28898,'1972-W1152','1972-09-13 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 28899,'1972-W1153','1972-09-13 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual;
insert into LAUNCH
select 289,'1944-S262','1944-11-26 13:39:00','military missile','success',1470,'19836','Ma305','','','Test',661,'',90 from dual union all
select 2890,'1954-S73','1954-09-08 00:00:00','military missile','success',1009,'K3-7','8A62 LKI III','','','Test',636,'',300 from dual union all
select 28900,'1972-W1154','1972-09-14 02:03:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 28901,'1972-W1155','1972-09-14 17:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',64 from dual union all
select 28902,'1972-W1156','1972-09-15 13:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 28903,'1972-W1157','1972-09-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',53 from dual union all
select 28904,'1972-W1158','1972-09-15 14:40:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',53 from dual union all
select 28905,'1972-W1159','1972-09-15 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 28906,'1972-W116','1972-01-21 20:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 28907,'1972-W1160','1972-09-15 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 28908,'1972-W1161','1972-09-16 01:58:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 28909,'1972-W1162','1972-09-16 17:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',52 from dual union all
select 2891,'1954-S74','1954-10-09 00:00:00','military missile','success',1009,'K3-9','8A62 LKI III','','','Test',636,'',300 from dual union all
select 28910,'1972-W1163','1972-09-17 17:31:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',51 from dual union all
select 28911,'1972-W1164','1972-09-18 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 28912,'1972-W1165','1972-09-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 28913,'1972-W1166','1972-09-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 28914,'1972-W1167','1972-09-18 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 28915,'1972-W1168','1972-09-18 17:45:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',63 from dual union all
select 28916,'1972-W1169','1972-09-18 18:19:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',52 from dual union all
select 28917,'1972-W117','1972-01-21 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 28918,'1972-W1170','1972-09-18 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 28919,'1972-W1171','1972-09-18 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 2892,'1954-S75','1954-10-19 00:00:00','military missile','success',1009,'K3-6','8A62 LKI III','','','Test',636,'',300 from dual union all
select 28920,'1972-W1172','1972-09-19 08:02:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 28921,'1972-W1173','1972-09-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 28922,'1972-W1174','1972-09-20 15:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 28923,'1972-W1175','1972-09-20 16:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 28924,'1972-W1176','1972-09-20 17:04:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',308,'',57 from dual union all
select 28925,'1972-W1177','1972-09-20 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 28926,'1972-W1178','1972-09-20 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 28927,'1972-W1179','1972-09-20 21:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 28928,'1972-W118','1972-01-21 23:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual union all
select 28929,'1972-W1180','1972-09-20 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 2893,'1954-S76','1954-10-05 18:15:00','suborbital rocket','success',56,'RTV-N-10b','','','','Remote Sensing',1869,'',158 from dual union all
select 28930,'1972-W1181','1972-09-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 28931,'1972-W1182','1972-09-22 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',75 from dual union all
select 28932,'1972-W1183','1972-09-22 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 28933,'1972-W1184','1972-09-22 23:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 28934,'1972-W1185','1972-09-23 03:12:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 28935,'1972-W1186','1972-09-25 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 28936,'1972-W1187','1972-09-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 28937,'1972-W1188','1972-09-25 15:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 28938,'1972-W1189','1972-09-25 15:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 28939,'1972-W119','1972-01-24 10:47:59','suborbital rocket','success',716,'INTA HD-7204','','','','Meteo',396,'',55 from dual union all
select 2894,'1954-S77','1954-10-14 21:20:00','suborbital rocket','success',858,'','Hypersonic Test','','','Reentry Test',1821,'',352 from dual union all
select 28940,'1972-W1190','1972-09-25 16:07:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 28941,'1972-W1191','1972-09-25 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 28942,'1972-W1192','1972-09-25 18:08:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 28943,'1972-W1193','1972-09-25 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 28944,'1972-W1194','1972-09-26 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 28945,'1972-W1195','1972-09-26 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 28946,'1972-W1196','1972-09-26 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 28947,'1972-W1197','1972-09-27 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 28948,'1972-W1198','1972-09-27 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 28949,'1972-W1199','1972-09-27 14:00:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',308,'',61 from dual union all
select 2895,'1954-S78','1954-10-20 00:00:00','military missile','success',974,'','','','','Test',636,'',150 from dual union all
select 28950,'1972-W12','1972-01-04 20:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 28951,'1972-W120','1972-01-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 28952,'1972-W1200','1972-09-27 14:25:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 28953,'1972-W1201','1972-09-27 14:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',58 from dual union all
select 28954,'1972-W1202','1972-09-27 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 28955,'1972-W1203','1972-09-27 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 28956,'1972-W1204','1972-09-27 16:57:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 28957,'1972-W1205','1972-09-27 17:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 28958,'1972-W1206','1972-09-27 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 28959,'1972-W1207','1972-09-29 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 2896,'1954-S79','1954-10-29 00:00:00','suborbital rocket','success',1490,'NA12','','','','Test',666,'',90 from dual union all
select 28960,'1972-W1208','1972-09-29 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 28961,'1972-W1209','1972-09-29 14:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 28962,'1972-W121','1972-01-24 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 28963,'1972-W1210','1972-09-29 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 28964,'1972-W1211','1972-09-29 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 28965,'1972-W1212','1972-09-29 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 28966,'1972-W1213','1972-09-29 21:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 28967,'1972-W1214','1972-10-02 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 28968,'1972-W1215','1972-10-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 28969,'1972-W1216','1972-10-02 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 2897,'1954-S80','1954-12-30 00:00:00','military missile','success',1009,'K3-10','8A62 ZI-1','','','Test',636,'',300 from dual union all
select 28970,'1972-W1217','1972-10-02 14:52:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 28971,'1972-W1218','1972-10-02 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 28972,'1972-W1219','1972-10-02 15:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 28973,'1972-W122','1972-01-24 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 28974,'1972-W1220','1972-10-02 15:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',54 from dual union all
select 28975,'1972-W1221','1972-10-02 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 28976,'1972-W1222','1972-10-02 20:09:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',59 from dual union all
select 28977,'1972-W1223','1972-10-02 21:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 28978,'1972-W1224','1972-10-03 15:00:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',65 from dual union all
select 28979,'1972-W1225','1972-10-03 15:59:59','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',65 from dual union all
select 2898,'1954-S83','1954-12-31 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 28980,'1972-W1226','1972-10-04 14:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 28981,'1972-W1227','1972-10-04 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',61 from dual union all
select 28982,'1972-W1228','1972-10-04 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 28983,'1972-W1229','1972-10-04 14:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 28984,'1972-W123','1972-01-24 15:41:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 28985,'1972-W1230','1972-10-04 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 28986,'1972-W1231','1972-10-04 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 28987,'1972-W1232','1972-10-04 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 28988,'1972-W1233','1972-10-04 17:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 28989,'1972-W1234','1972-10-04 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual;
insert into LAUNCH
select 2899,'1954-S87','1954-11-30 00:00:00','military missile','success',970,'No 1915 (1953)','','','','OT',636,'',100 from dual union all
select 28990,'1972-W1235','1972-10-05 03:23:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 28991,'1972-W1236','1972-10-06 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 28992,'1972-W1237','1972-10-06 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 28993,'1972-W1238','1972-10-06 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 28994,'1972-W1239','1972-10-06 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 28995,'1972-W124','1972-01-24 16:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 28996,'1972-W1240','1972-10-06 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 28997,'1972-W1241','1972-10-06 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 28998,'1972-W1242','1972-10-06 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 28999,'1972-W1243','1972-10-07 02:06:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 29,'1943-S07','1943-06-01 00:00:00','military missile','success',8,'23','','','','Test',675,'',95 from dual union all
select 290,'1944-S263','1944-11-27 10:50:00','military missile','success',1470,'19831','Ma276','','','Test',661,'',90 from dual union all
select 2900,'1954-S88','1954-12-01 00:00:00','military missile','success',970,'No 620','','','','OT',636,'',100 from dual union all
select 29000,'1972-W1244','1972-10-09 13:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',52 from dual union all
select 29001,'1972-W1245','1972-10-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29002,'1972-W1246','1972-10-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 29003,'1972-W1247','1972-10-10 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 29004,'1972-W1249','1972-10-11 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 29005,'1972-W125','1972-01-24 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 29006,'1972-W1250','1972-10-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29007,'1972-W1251','1972-10-11 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 29008,'1972-W1252','1972-10-11 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 29009,'1972-W1253','1972-10-11 15:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',71 from dual union all
select 2901,'1954-S89','1954-04-24 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 29010,'1972-W1254','1972-10-11 15:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 29011,'1972-W1255','1972-10-11 15:59:59','suborbital rocket','success',88,'','Arcas Starute 4/STS','','','Test/Aeron',1895,'',76 from dual union all
select 29012,'1972-W1256','1972-10-11 16:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 29013,'1972-W1257','1972-10-11 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 29014,'1972-W1258','1972-10-11 19:04:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 29015,'1972-W1259','1972-10-11 21:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 29016,'1972-W126','1972-01-24 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29017,'1972-W1260','1972-10-11 21:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 29018,'1972-W1261','1972-10-13 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 29019,'1972-W1262','1972-10-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 2902,'1954-S90','1954-04-27 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 29020,'1972-W1263','1972-10-13 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',62 from dual union all
select 29021,'1972-W1264','1972-10-13 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 29022,'1972-W1265','1972-10-13 16:45:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 29023,'1972-W1266','1972-10-13 16:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29024,'1972-W1267','1972-10-13 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 29025,'1972-W1268','1972-10-16 14:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29026,'1972-W1269','1972-10-16 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 29027,'1972-W127','1972-01-25 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',56 from dual union all
select 29028,'1972-W1270','1972-10-16 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 29029,'1972-W1271','1972-10-16 20:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 2903,'1954-S91','1954-05-04 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 29030,'1972-W1272','1972-10-17 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29031,'1972-W1273','1972-10-17 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29032,'1972-W1274','1972-10-17 23:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 29033,'1972-W1275','1972-10-18 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 29034,'1972-W1276','1972-10-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29035,'1972-W1277','1972-10-18 14:52:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29036,'1972-W1278','1972-10-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 29037,'1972-W1279','1972-10-18 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 29038,'1972-W128','1972-01-25 01:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 29039,'1972-W1280','1972-10-18 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 2904,'1954-S92','1954-05-05 00:00:00','military missile','failure',971,'','','','','Test',636,'',150 from dual union all
select 29040,'1972-W1281','1972-10-18 18:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 29041,'1972-W1282','1972-10-18 19:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29042,'1972-W1283','1972-10-18 22:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29043,'1972-W1284','1972-10-18 23:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29044,'1972-W1285','1972-10-19 14:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 29045,'1972-W1286','1972-10-19 17:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',60 from dual union all
select 29046,'1972-W1287','1972-10-20 13:34:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',65 from dual union all
select 29047,'1972-W1288','1972-10-20 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 29048,'1972-W1289','1972-10-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29049,'1972-W129','1972-01-25 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 2905,'1954-S93','1954-05-06 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 29050,'1972-W1290','1972-10-20 14:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 29051,'1972-W1291','1972-10-20 14:52:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 29052,'1972-W1292','1972-10-20 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 29053,'1972-W1293','1972-10-20 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 29054,'1972-W1294','1972-10-20 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1895,'',53 from dual union all
select 29055,'1972-W1295','1972-10-20 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 29056,'1972-W1296','1972-10-23 15:01:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29057,'1972-W1297','1972-10-24 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 29058,'1972-W1298','1972-10-24 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 29059,'1972-W1299','1972-10-25 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 2906,'1954-S94','1954-05-08 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 29060,'1972-W13','1972-01-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 29061,'1972-W130','1972-01-25 18:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',57 from dual union all
select 29062,'1972-W1300','1972-10-25 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',57 from dual union all
select 29063,'1972-W1301','1972-10-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 29064,'1972-W1302','1972-10-25 15:17:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29065,'1972-W1303','1972-10-25 15:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 29066,'1972-W1304','1972-10-25 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29067,'1972-W1305','1972-10-25 18:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 29068,'1972-W1306','1972-10-25 21:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 29069,'1972-W1307','1972-10-25 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 2907,'1954-S95','1954-05-12 00:00:00','military missile','success',971,'','','','','Test',636,'',150 from dual union all
select 29070,'1972-W1308','1972-10-25 22:10:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29071,'1972-W1309','1972-10-25 23:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 29072,'1972-W131','1972-01-25 22:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 29073,'1972-W1310','1972-10-26 14:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',63 from dual union all
select 29074,'1972-W1311','1972-10-26 19:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29075,'1972-W1312','1972-10-26 22:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29076,'1972-W1313','1972-10-26 23:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 29077,'1972-W1314','1972-10-27 00:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',66 from dual union all
select 29078,'1972-W1315','1972-10-27 03:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual;
insert into LAUNCH
select 29079,'1972-W1316','1972-10-27 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 2908,'1954-S96','1954-06-08 00:00:00','military missile','success',992,'Prod. missile','ZAV586 Prod GIM-2','','','Test',636,'',150 from dual union all
select 29080,'1972-W1317','1972-10-27 14:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29081,'1972-W1318','1972-10-27 14:56:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 29082,'1972-W1319','1972-10-27 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',55 from dual union all
select 29083,'1972-W132','1972-01-26 11:34:00','suborbital rocket','success',716,'INTA HW-7202','','','','Meteo',396,'',61 from dual union all
select 29084,'1972-W1320','1972-10-27 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 29085,'1972-W1321','1972-10-27 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 29086,'1972-W1322','1972-10-27 23:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 29087,'1972-W1323','1972-10-30 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 29088,'1972-W1324','1972-10-30 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 29089,'1972-W1325','1972-10-30 16:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 2909,'1954-S97','1954-06-09 00:00:00','military missile','success',992,'Prod. missile','ZAV586 Prod GIM-2','','','Test',636,'',150 from dual union all
select 29090,'1972-W1326','1972-10-30 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',70 from dual union all
select 29091,'1972-W1327','1972-10-30 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29092,'1972-W1328','1972-10-30 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29093,'1972-W1329','1972-10-30 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 29094,'1972-W133','1972-01-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 29095,'1972-W1330','1972-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 29096,'1972-W1331','1972-10-31 03:47:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 29097,'1972-W1332','1972-10-31 19:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29098,'1972-W1333','1972-10-31 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29099,'1972-W1334','1972-10-31 23:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 291,'1944-S264','1944-11-28 08:59:00','military missile','success',1470,'19827','Ma299','','','Test',661,'',90 from dual union all
select 2910,'1954-S98','1954-07-22 00:00:00','military missile','success',992,'S0-6','UGCh','','','R&D',636,'',150 from dual union all
select 29100,'1972-W1335','1972-11-01 06:57:00','suborbital rocket','success',1281,'NASA 15.94UI','','','','Ionos',1821,'',79 from dual union all
select 29101,'1972-W1336','1972-11-01 07:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 29102,'1972-W1337','1972-11-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29103,'1972-W1338','1972-11-01 14:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 29104,'1972-W1339','1972-11-01 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 29105,'1972-W134','1972-01-26 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 29106,'1972-W1340','1972-11-01 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 29107,'1972-W1341','1972-11-01 18:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29108,'1972-W1342','1972-11-01 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29109,'1972-W1343','1972-11-01 23:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 2911,'1954-S99','1954-09-30 00:00:00','military missile','success',992,'K1-20','','','','OT',636,'',150 from dual union all
select 29110,'1972-W1344','1972-11-01 23:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 29111,'1972-W1345','1972-11-01 23:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 29112,'1972-W1346','1972-11-02 04:26:00','suborbital rocket','success',257,'NASA 15.96UI','','','','Ionos',428,'',57 from dual union all
select 29113,'1972-W1347','1972-11-02 05:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 29114,'1972-W1348','1972-11-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 29115,'1972-W1349','1972-11-03 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 29116,'1972-W135','1972-01-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 29117,'1972-W1350','1972-11-03 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 29118,'1972-W1351','1972-11-03 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 29119,'1972-W1352','1972-11-03 15:55:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 2912,'1955-A01','1955-01-01 00:00:00','atmospheric rocket','success',365,'','XF3H-1 Model','','','Test',1821,'',10 from dual union all
select 29120,'1972-W1353','1972-11-03 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',60 from dual union all
select 29121,'1972-W1354','1972-11-03 18:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29122,'1972-W1355','1972-11-03 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29123,'1972-W1356','1972-11-03 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 29124,'1972-W1357','1972-11-06 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 29125,'1972-W1358','1972-11-06 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 29126,'1972-W1359','1972-11-06 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 29127,'1972-W136','1972-01-26 15:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 29128,'1972-W1360','1972-11-06 15:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 29129,'1972-W1361','1972-11-06 16:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',57 from dual union all
select 2913,'1955-A02','1955-01-01 00:00:00','atmospheric rocket','success',365,'','XF3H-1 Model','','','Test',1821,'',10 from dual union all
select 29130,'1972-W1362','1972-11-06 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 29131,'1972-W1363','1972-11-06 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 29132,'1972-W1364','1972-11-06 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 29133,'1972-W1365','1972-11-07 15:45:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 29134,'1972-W1367','1972-11-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29135,'1972-W1368','1972-11-08 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 29136,'1972-W1369','1972-11-08 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 29137,'1972-W137','1972-01-26 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 29138,'1972-W1370','1972-11-08 15:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 29139,'1972-W1371','1972-11-08 16:14:59','suborbital rocket','success',88,'','Arcas Starute 5/STS','','','Meteo',1895,'',76 from dual union all
select 2914,'1955-A03','1955-01-05 00:00:00','atmospheric rocket','success',836,'2814A','','','','Test',1895,'',10 from dual union all
select 29140,'1972-W1372','1972-11-08 16:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 29141,'1972-W1373','1972-11-08 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 29142,'1972-W1374','1972-11-08 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 29143,'1972-W1375','1972-11-08 19:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 29144,'1972-W1376','1972-11-08 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 29145,'1972-W1377','1972-11-08 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 29146,'1972-W1378','1972-11-08 23:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 29147,'1972-W1379','1972-11-08 23:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 29148,'1972-W138','1972-01-26 17:58:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 29149,'1972-W1380','1972-11-09 04:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 2915,'1955-A04','1955-01-05 00:00:00','atmospheric rocket','success',835,'','F23 Ramjet','','','Test',1821,'',10 from dual union all
select 29150,'1972-W1381','1972-11-09 15:45:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 29151,'1972-W1382','1972-11-09 19:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 29152,'1972-W1383','1972-11-09 21:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29153,'1972-W1384','1972-11-09 23:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 29154,'1972-W1386','1972-11-10 13:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 29155,'1972-W1387','1972-11-10 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',54 from dual union all
select 29156,'1972-W1388','1972-11-10 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 29157,'1972-W1389','1972-11-10 15:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29158,'1972-W139','1972-01-26 21:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 29159,'1972-W1390','1972-11-10 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 2916,'1955-A05','1955-01-06 00:00:00','atmospheric rocket','success',836,'3090R','','','','Test',1895,'',10 from dual union all
select 29160,'1972-W1391','1972-11-10 18:00:00','suborbital rocket','success',88,'','Arcas Starute 6/STS','','','Meteo',1895,'',78 from dual union all
select 29161,'1972-W1392','1972-11-10 18:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 29162,'1972-W1393','1972-11-10 19:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29163,'1972-W1394','1972-11-10 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29164,'1972-W1395','1972-11-10 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 29165,'1972-W1396','1972-11-10 23:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29166,'1972-W1397','1972-11-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 29167,'1972-W1398','1972-11-13 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',61 from dual union all
select 29168,'1972-W1399','1972-11-13 18:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual;
insert into LAUNCH
select 29169,'1972-W14','1972-01-05 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 2917,'1955-A06','1955-01-06 00:00:00','atmospheric rocket','success',836,'3091R','Nike 413R','','','Test',1895,'',10 from dual union all
select 29170,'1972-W140','1972-01-26 22:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 29171,'1972-W1400','1972-11-13 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29172,'1972-W1401','1972-11-13 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 29173,'1972-W1402','1972-11-14 15:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29174,'1972-W1403','1972-11-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 29175,'1972-W1404','1972-11-15 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 29176,'1972-W1405','1972-11-15 15:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 29177,'1972-W1406','1972-11-15 16:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 29178,'1972-W1407','1972-11-15 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 29179,'1972-W1408','1972-11-15 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 2918,'1955-A07','1955-01-06 00:00:00','atmospheric rocket','success',836,'2608A','','','','Test',1895,'',10 from dual union all
select 29180,'1972-W1409','1972-11-15 21:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29181,'1972-W141','1972-01-27 01:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',54 from dual union all
select 29182,'1972-W1410','1972-11-15 21:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',50 from dual union all
select 29183,'1972-W1411','1972-11-15 22:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 29184,'1972-W1412','1972-11-16 15:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 29185,'1972-W1413','1972-11-16 18:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29186,'1972-W1414','1972-11-16 21:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29187,'1972-W1415','1972-11-16 22:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29188,'1972-W1416','1972-11-16 23:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',61 from dual union all
select 29189,'1972-W1417','1972-11-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 2919,'1955-A08','1955-01-06 00:00:00','atmospheric rocket','success',836,'2796A','','','','Test',1895,'',10 from dual union all
select 29190,'1972-W1418','1972-11-17 14:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',72 from dual union all
select 29191,'1972-W1419','1972-11-17 15:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29192,'1972-W142','1972-01-27 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',54 from dual union all
select 29193,'1972-W1420','1972-11-17 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 29194,'1972-W1421','1972-11-17 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 29195,'1972-W1422','1972-11-17 19:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29196,'1972-W1423','1972-11-17 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 29197,'1972-W1424','1972-11-17 21:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29198,'1972-W1425','1972-11-17 22:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 29199,'1972-W1426','1972-11-17 22:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 292,'1944-S265','1944-11-29 10:32:00','military missile','success',1470,'19841','Ma316','','','Test',661,'',90 from dual union all
select 2920,'1955-A09','1955-01-06 00:00:00','atmospheric rocket','success',836,'3090R','Nike 415R','','','Test',1895,'',10 from dual union all
select 29200,'1972-W1427','1972-11-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 29201,'1972-W1428','1972-11-20 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29202,'1972-W1429','1972-11-20 18:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29203,'1972-W143','1972-01-27 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 29204,'1972-W1430','1972-11-20 18:43:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',65 from dual union all
select 29205,'1972-W1431','1972-11-20 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 29206,'1972-W1432','1972-11-20 20:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29207,'1972-W1433','1972-11-20 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 29208,'1972-W1434','1972-11-20 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 29209,'1972-W1435','1972-11-21 15:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 2921,'1955-A10','1955-01-06 00:00:00','atmospheric rocket','success',836,'3088R','Nike 414R','','','Test',1895,'',10 from dual union all
select 29210,'1972-W1436','1972-11-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 29211,'1972-W1437','1972-11-22 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 29212,'1972-W1438','1972-11-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 29213,'1972-W1439','1972-11-22 15:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 29214,'1972-W144','1972-01-27 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 29215,'1972-W1440','1972-11-22 15:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 29216,'1972-W1441','1972-11-22 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',73 from dual union all
select 29217,'1972-W1442','1972-11-22 17:00:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',67 from dual union all
select 29218,'1972-W1443','1972-11-22 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29219,'1972-W1444','1972-11-22 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 2922,'1955-A100','1955-02-15 00:00:00','atmospheric rocket','success',836,'1722T','','','','Test',1895,'',10 from dual union all
select 29220,'1972-W1445','1972-11-22 21:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 29221,'1972-W1446','1972-11-23 15:53:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 29222,'1972-W1447','1972-11-24 15:51:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29223,'1972-W1448','1972-11-24 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29224,'1972-W1449','1972-11-27 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29225,'1972-W145','1972-01-28 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 29226,'1972-W1450','1972-11-27 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',70 from dual union all
select 29227,'1972-W1451','1972-11-27 16:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 29228,'1972-W1452','1972-11-27 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29229,'1972-W1453','1972-11-27 19:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 2923,'1955-A101','1955-02-16 00:00:00','military missile','success',771,'GM-12450','','','','Test',222,'',12 from dual union all
select 29230,'1972-W1454','1972-11-27 20:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 29231,'1972-W1455','1972-11-27 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',54 from dual union all
select 29232,'1972-W1456','1972-11-28 03:04:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29233,'1972-W1457','1972-11-29 01:18:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',53 from dual union all
select 29234,'1972-W1458','1972-11-29 12:27:59','suborbital rocket','success',716,'INTA HW-7216','','','','Meteo',396,'',53 from dual union all
select 29235,'1972-W1459','1972-11-29 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 29236,'1972-W146','1972-01-28 14:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 29237,'1972-W1460','1972-11-29 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 29238,'1972-W1461','1972-11-29 15:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29239,'1972-W1462','1972-11-29 16:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',56 from dual union all
select 2924,'1955-A102','1955-02-16 00:00:00','atmospheric rocket','success',836,'3078R','Nike 421R','','','Test',1895,'',10 from dual union all
select 29240,'1972-W1463','1972-11-29 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 29241,'1972-W1464','1972-11-29 18:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',276,'',70 from dual union all
select 29242,'1972-W1465','1972-11-29 21:04:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 29243,'1972-W1466','1972-12-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 29244,'1972-W1467','1972-12-01 14:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 29245,'1972-W1468','1972-12-01 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',56 from dual union all
select 29246,'1972-W1469','1972-12-01 18:00:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',63 from dual union all
select 29247,'1972-W147','1972-01-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',53 from dual union all
select 29248,'1972-W1470','1972-12-01 18:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29249,'1972-W1471','1972-12-01 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 2925,'1955-A103','1955-02-16 00:00:00','atmospheric rocket','success',836,'4249E','','','','Test',1895,'',10 from dual union all
select 29250,'1972-W1472','1972-12-01 20:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29251,'1972-W1473','1972-12-02 02:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29252,'1972-W1474','1972-12-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 29253,'1972-W1475','1972-12-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 29254,'1972-W1476','1972-12-04 15:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 29255,'1972-W1477','1972-12-04 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 29256,'1972-W1478','1972-12-04 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 29257,'1972-W1479','1972-12-04 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 29258,'1972-W148','1972-01-28 15:44:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual;
insert into LAUNCH
select 29259,'1972-W1480','1972-12-05 00:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 2926,'1955-A104','1955-02-16 00:00:00','atmospheric rocket','success',836,'3105R','Nike 424R','','','Test',1895,'',10 from dual union all
select 29260,'1972-W1481','1972-12-05 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 29261,'1972-W1482','1972-12-05 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29262,'1972-W1483','1972-12-05 17:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29263,'1972-W1484','1972-12-05 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 29264,'1972-W1485','1972-12-06 02:00:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29265,'1972-W1486','1972-12-06 12:17:00','suborbital rocket','success',716,'INTA HW-7217','','','','Meteo',396,'',57 from dual union all
select 29266,'1972-W1487','1972-12-06 16:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 29267,'1972-W1488','1972-12-06 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 29268,'1972-W1489','1972-12-06 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 29269,'1972-W149','1972-01-28 15:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 2927,'1955-A105','1955-02-16 00:00:00','atmospheric rocket','success',836,'4209R','Nike 422R','','','Test',1895,'',10 from dual union all
select 29270,'1972-W1490','1972-12-06 19:44:59','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',67 from dual union all
select 29271,'1972-W1491','1972-12-06 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 29272,'1972-W1492','1972-12-06 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 29273,'1972-W1493','1972-12-07 02:55:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 29274,'1972-W1494','1972-12-07 07:37:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 29275,'1972-W1495','1972-12-07 08:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 29276,'1972-W1496','1972-12-07 08:34:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29277,'1972-W1497','1972-12-07 19:08:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29278,'1972-W1498','1972-12-08 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',61 from dual union all
select 29279,'1972-W1499','1972-12-08 02:51:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 2928,'1955-A106','1955-02-16 00:00:00','atmospheric rocket','success',836,'2950R','Nike 420R','','','Test',1895,'',10 from dual union all
select 29280,'1972-W15','1972-01-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 29281,'1972-W150','1972-01-28 18:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 29282,'1972-W1500','1972-12-08 06:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',56 from dual union all
select 29283,'1972-W1501','1972-12-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 29284,'1972-W1502','1972-12-08 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual union all
select 29285,'1972-W1503','1972-12-08 15:00:00','suborbital rocket','success',1061,'','Chaff,AN/DMQ-9-','','','Meteo',1470,'',61 from dual union all
select 29286,'1972-W1504','1972-12-08 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 29287,'1972-W1505','1972-12-09 02:29:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29288,'1972-W1506','1972-12-11 00:57:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 29289,'1972-W1507','1972-12-11 02:10:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 2929,'1955-A107','1955-02-16 00:00:00','atmospheric rocket','success',836,'4197R','Nike 423R','','','Test',1895,'',10 from dual union all
select 29290,'1972-W1508','1972-12-11 02:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29291,'1972-W1509','1972-12-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 29292,'1972-W151','1972-01-28 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',58 from dual union all
select 29293,'1972-W1510','1972-12-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 29294,'1972-W1511','1972-12-11 14:25:00','suborbital rocket','success',1061,'','Chaff,AN/DMQ-9-','','','Meteo',1470,'',66 from dual union all
select 29295,'1972-W1512','1972-12-11 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 29296,'1972-W1513','1972-12-11 17:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 29297,'1972-W1514','1972-12-11 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 29298,'1972-W1515','1972-12-11 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29299,'1972-W1516','1972-12-11 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 293,'1944-S266','1944-11-29 18:40:00','military missile','success',1470,'19825','Ma277','','','Test',661,'',90 from dual union all
select 2930,'1955-A108','1955-02-16 00:00:00','atmospheric rocket','success',836,'1539T','','','','Test',1895,'',10 from dual union all
select 29300,'1972-W1517','1972-12-12 02:30:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29301,'1972-W1518','1972-12-12 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 29302,'1972-W1519','1972-12-13 00:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 29303,'1972-W152','1972-01-29 01:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 29304,'1972-W1520','1972-12-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 29305,'1972-W1521','1972-12-13 14:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',55 from dual union all
select 29306,'1972-W1522','1972-12-13 15:47:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 29307,'1972-W1523','1972-12-13 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 29308,'1972-W1524','1972-12-13 18:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29309,'1972-W1525','1972-12-13 19:32:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',59 from dual union all
select 2931,'1955-A109','1955-02-16 00:00:00','atmospheric rocket','success',836,'1504T','','','','Test',1895,'',10 from dual union all
select 29310,'1972-W1526','1972-12-13 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 29311,'1972-W1527','1972-12-14 15:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',55 from dual union all
select 29312,'1972-W1528','1972-12-14 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',55 from dual union all
select 29313,'1972-W1529','1972-12-14 20:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 29314,'1972-W153','1972-01-31 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',52 from dual union all
select 29315,'1972-W1530','1972-12-15 15:47:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 29316,'1972-W1531','1972-12-15 15:59:59','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',65 from dual union all
select 29317,'1972-W1532','1972-12-15 16:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29318,'1972-W1533','1972-12-15 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 29319,'1972-W1534','1972-12-15 22:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 2932,'1955-A11','1955-01-11 00:00:00','military missile','unknown',339,'1556R','','','','Test',1895,'',40 from dual union all
select 29320,'1972-W1535','1972-12-15 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 29321,'1972-W1536','1972-12-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 29322,'1972-W1537','1972-12-18 14:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 29323,'1972-W1538','1972-12-18 15:59:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 29324,'1972-W1539','1972-12-18 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29325,'1972-W154','1972-01-31 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 29326,'1972-W1540','1972-12-18 17:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 29327,'1972-W1541','1972-12-18 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29328,'1972-W1542','1972-12-18 22:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 29329,'1972-W1543','1972-12-19 09:59:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',61 from dual union all
select 2933,'1955-A110','1955-02-16 00:00:00','atmospheric rocket','success',836,'4231E','','','','Test',1895,'',10 from dual union all
select 29330,'1972-W1544','1972-12-19 15:17:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',56 from dual union all
select 29331,'1972-W1545','1972-12-19 15:53:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 29332,'1972-W1546','1972-12-19 16:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 29333,'1972-W1547','1972-12-19 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29334,'1972-W1548','1972-12-19 20:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29335,'1972-W1549','1972-12-20 01:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 29336,'1972-W155','1972-01-31 15:53:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 29337,'1972-W1550','1972-12-20 14:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',76 from dual union all
select 29338,'1972-W1551','1972-12-20 14:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29339,'1972-W1552','1972-12-20 16:19:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 2934,'1955-A111','1955-02-17 00:00:00','military missile','unknown',339,'1564R','','','','Test',1895,'',40 from dual union all
select 29340,'1972-W1553','1972-12-20 16:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 29341,'1972-W1554','1972-12-20 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 29342,'1972-W1555','1972-12-20 19:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29343,'1972-W1556','1972-12-21 00:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 29344,'1972-W1557','1972-12-21 15:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29345,'1972-W1558','1972-12-21 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 29346,'1972-W1559','1972-12-22 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29347,'1972-W156','1972-01-31 16:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',53 from dual union all
select 29348,'1972-W1560','1972-12-22 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual;
insert into LAUNCH
select 29349,'1972-W1561','1972-12-22 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',51 from dual union all
select 2935,'1955-A112','1955-02-18 00:00:00','atmospheric rocket','success',836,'1752T','','','','Test',1895,'',10 from dual union all
select 29350,'1972-W1562','1972-12-26 18:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 29351,'1972-W1563','1972-12-26 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 29352,'1972-W1564','1972-12-27 15:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 29353,'1972-W1565','1972-12-27 16:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',72 from dual union all
select 29354,'1972-W1566','1972-12-27 18:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29355,'1972-W1567','1972-12-27 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29356,'1972-W1568','1972-12-27 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 29357,'1972-W1569','1972-12-27 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 29358,'1972-W157','1972-01-31 19:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 29359,'1972-W1570','1972-12-28 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 2936,'1955-A113','1955-02-19 00:00:00','atmospheric rocket','success',836,'1713T','','','','Test',1895,'',10 from dual union all
select 29360,'1972-W1571','1972-12-29 14:24:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',71 from dual union all
select 29361,'1972-W1572','1972-12-29 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 29362,'1972-W1573','1972-12-29 18:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 29363,'1972-W1574','1972-12-29 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 29364,'1972-W1575','1972-12-29 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 29365,'1972-W1576','1972-07-01 00:00:00','suborbital rocket','success',1188,'','','','','Meteor',450,'',70 from dual union all
select 29366,'1972-W158','1972-01-31 19:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29367,'1972-W159','1972-01-31 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 29368,'1972-W16','1972-01-05 15:45:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 29369,'1972-W160','1972-01-31 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',64 from dual union all
select 2937,'1955-A114','1955-02-20 00:00:00','atmospheric rocket','success',836,'1923T','','','','Test',1895,'',10 from dual union all
select 29370,'1972-W161','1972-01-31 22:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',51 from dual union all
select 29371,'1972-W162','1972-02-01 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 29372,'1972-W163','1972-02-01 15:47:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 29373,'1972-W164','1972-02-01 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 29374,'1972-W165','1972-02-01 18:00:00','suborbital rocket','unknown',88,'','Ozone','','','Aeron',1895,'',54 from dual union all
select 29375,'1972-W166','1972-02-01 18:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',69 from dual union all
select 29376,'1972-W167','1972-02-01 18:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 29377,'1972-W168','1972-02-01 19:00:00','suborbital rocket','unknown',88,'','','','','Ionos',1895,'',65 from dual union all
select 29378,'1972-W169','1972-02-01 19:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29379,'1972-W17','1972-01-05 17:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',60 from dual union all
select 2938,'1955-A115','1955-02-21 00:00:00','military missile','success',771,'GM-12486','','','','Test',222,'',12 from dual union all
select 29380,'1972-W170','1972-02-02 06:56:00','suborbital rocket','success',761,'MT135P T1','','','','Aeron?',1476,'',50 from dual union all
select 29381,'1972-W171','1972-02-02 11:34:00','suborbital rocket','success',716,'INTA HW-7203','','','','Meteo',396,'',58 from dual union all
select 29382,'1972-W172','1972-02-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29383,'1972-W173','1972-02-02 14:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 29384,'1972-W174','1972-02-02 15:43:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 29385,'1972-W175','1972-02-02 16:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 29386,'1972-W176','1972-02-02 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 29387,'1972-W177','1972-02-02 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 29388,'1972-W178','1972-02-03 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 29389,'1972-W179','1972-02-03 15:45:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 2939,'1955-A116','1955-02-22 00:00:00','atmospheric rocket','success',836,'1696T','','','','Test',1895,'',10 from dual union all
select 29390,'1972-W18','1972-01-05 19:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29391,'1972-W180','1972-02-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',57 from dual union all
select 29392,'1972-W181','1972-02-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 29393,'1972-W182','1972-02-04 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 29394,'1972-W183','1972-02-04 15:48:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 29395,'1972-W184','1972-02-04 16:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 29396,'1972-W185','1972-02-04 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29397,'1972-W186','1972-02-04 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 29398,'1972-W187','1972-02-04 19:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 29399,'1972-W188','1972-02-05 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',63 from dual union all
select 294,'1944-S267','1944-12-01 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1735,'',90 from dual union all
select 2940,'1955-A117','1955-02-22 00:00:00','atmospheric rocket','success',836,'1715T','','','','Test',1895,'',10 from dual union all
select 29400,'1972-W189','1972-02-05 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 29401,'1972-W19','1972-01-05 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 29402,'1972-W190','1972-02-07 06:45:00','suborbital rocket','success',761,'MT135P T2','','','','Aeron?',1476,'',50 from dual union all
select 29403,'1972-W191','1972-02-07 12:01:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29404,'1972-W192','1972-02-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 29405,'1972-W193','1972-02-07 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 29406,'1972-W194','1972-02-07 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 29407,'1972-W195','1972-02-07 15:59:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',74 from dual union all
select 29408,'1972-W196','1972-02-07 16:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 29409,'1972-W197','1972-02-07 19:10:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',60 from dual union all
select 2941,'1955-A118','1955-02-22 00:00:00','atmospheric rocket','success',836,'1737T','','','','Test',1895,'',10 from dual union all
select 29410,'1972-W198','1972-02-07 20:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 29411,'1972-W199','1972-02-07 23:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 29412,'1972-W20','1972-01-05 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',59 from dual union all
select 29413,'1972-W200','1972-02-08 07:10:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 29414,'1972-W201','1972-02-08 18:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29415,'1972-W202','1972-02-08 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29416,'1972-W203','1972-02-08 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 29417,'1972-W204','1972-02-09 11:33:00','suborbital rocket','success',716,'INTA HW-7204','','','','Meteo',396,'',57 from dual union all
select 29418,'1972-W205','1972-02-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',55 from dual union all
select 29419,'1972-W206','1972-02-09 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 2942,'1955-A119','1955-02-23 00:00:00','atmospheric rocket','success',836,'1963T','','','','Test',1895,'',10 from dual union all
select 29420,'1972-W207','1972-02-09 14:49:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',66 from dual union all
select 29421,'1972-W208','1972-02-09 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 29422,'1972-W209','1972-02-09 18:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',57 from dual union all
select 29423,'1972-W21','1972-01-05 21:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',51 from dual union all
select 29424,'1972-W210','1972-02-09 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 29425,'1972-W211','1972-02-09 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 29426,'1972-W212','1972-02-09 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 29427,'1972-W213','1972-02-09 21:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29428,'1972-W214','1972-02-09 22:35:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',75 from dual union all
select 29429,'1972-W215','1972-02-09 23:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 2943,'1955-A12','1955-01-11 00:00:00','atmospheric rocket','success',442,'','Falcon Missile','','','Test',1821,'',10 from dual union all
select 29430,'1972-W216','1972-02-09 23:23:00','suborbital rocket','success',716,'INTA HW-7205','','','','Meteo',396,'',51 from dual union all
select 29431,'1972-W217','1972-02-10 01:17:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 29432,'1972-W218','1972-02-10 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 29433,'1972-W219','1972-02-10 15:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29434,'1972-W22','1972-01-05 23:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 29435,'1972-W220','1972-02-10 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 29436,'1972-W221','1972-02-11 04:02:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29437,'1972-W222','1972-02-11 12:52:00','suborbital rocket','success',716,'INTA HW-7206','','','','Meteo',396,'',57 from dual union all
select 29438,'1972-W223','1972-02-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual;
insert into LAUNCH
select 29439,'1972-W224','1972-02-11 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 2944,'1955-A120','1955-02-23 00:00:00','atmospheric rocket','success',836,'1709T','','','','Test',1895,'',10 from dual union all
select 29440,'1972-W225','1972-02-11 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 29441,'1972-W226','1972-02-11 16:44:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',51 from dual union all
select 29442,'1972-W227','1972-02-11 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 29443,'1972-W228','1972-02-11 17:48:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',70 from dual union all
select 29444,'1972-W229','1972-02-11 18:00:00','suborbital rocket','unknown',88,'','Ozone','','','Aeron',1895,'',56 from dual union all
select 29445,'1972-W23','1972-01-06 02:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 29446,'1972-W230','1972-02-11 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 29447,'1972-W231','1972-02-11 19:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 29448,'1972-W232','1972-02-11 21:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 29449,'1972-W233','1972-02-12 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 2945,'1955-A121','1955-02-23 00:00:00','atmospheric rocket','success',836,'2457R','Nike 428R','','','Test',1895,'',10 from dual union all
select 29450,'1972-W234','1972-02-12 06:48:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 29451,'1972-W235','1972-02-14 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',55 from dual union all
select 29452,'1972-W236','1972-02-14 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 29453,'1972-W237','1972-02-14 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 29454,'1972-W238','1972-02-14 15:04:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',62 from dual union all
select 29455,'1972-W239','1972-02-14 15:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 29456,'1972-W24','1972-01-06 17:51:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 29457,'1972-W240','1972-02-14 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 29458,'1972-W241','1972-02-14 21:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 29459,'1972-W242','1972-02-14 22:33:59','suborbital rocket','success',716,'INTA HW-7207','','','','Meteo',396,'',58 from dual union all
select 2946,'1955-A122','1955-02-23 00:00:00','atmospheric rocket','success',836,'4198E','','','','Test',1895,'',10 from dual union all
select 29460,'1972-W243','1972-02-15 05:55:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 29461,'1972-W244','1972-02-15 16:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29462,'1972-W245','1972-02-15 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',52 from dual union all
select 29463,'1972-W246','1972-02-15 19:01:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 29464,'1972-W247','1972-02-16 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 29465,'1972-W248','1972-02-16 11:30:00','suborbital rocket','success',716,'INTA HW-7208','','','','Meteo',396,'',58 from dual union all
select 29466,'1972-W249','1972-02-16 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 29467,'1972-W25','1972-01-06 18:37:00','suborbital rocket','success',1281,'NASA 15.92UI','','','','Ionos',1821,'',67 from dual union all
select 29468,'1972-W250','1972-02-16 14:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',64 from dual union all
select 29469,'1972-W251','1972-02-16 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 2947,'1955-A123','1955-02-23 00:00:00','atmospheric rocket','success',836,'2997R','Nike 427R','','','Test',1895,'',10 from dual union all
select 29470,'1972-W252','1972-02-16 14:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',62 from dual union all
select 29471,'1972-W253','1972-02-16 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',61 from dual union all
select 29472,'1972-W254','1972-02-16 15:45:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 29473,'1972-W255','1972-02-16 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29474,'1972-W256','1972-02-16 19:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29475,'1972-W257','1972-02-16 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 29476,'1972-W258','1972-02-17 16:06:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 29477,'1972-W259','1972-02-17 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 29478,'1972-W26','1972-01-06 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 29479,'1972-W260','1972-02-17 22:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 2948,'1955-A124','1955-02-23 00:00:00','atmospheric rocket','success',836,'2980R','Nike 426R','','','Test',1895,'',10 from dual union all
select 29480,'1972-W261','1972-02-18 11:37:59','suborbital rocket','success',716,'INTA HW-7209','','','','Meteo',396,'',58 from dual union all
select 29481,'1972-W262','1972-02-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 29482,'1972-W263','1972-02-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 29483,'1972-W264','1972-02-18 15:24:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',51 from dual union all
select 29484,'1972-W265','1972-02-18 15:45:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 29485,'1972-W266','1972-02-18 15:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 29486,'1972-W267','1972-02-18 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',62 from dual union all
select 29487,'1972-W268','1972-02-19 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 29488,'1972-W269','1972-02-21 14:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',62 from dual union all
select 29489,'1972-W27','1972-01-06 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',50 from dual union all
select 2949,'1955-A125','1955-02-23 00:00:00','atmospheric rocket','success',836,'1776T','','','','Test',1895,'',10 from dual union all
select 29490,'1972-W270','1972-02-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 29491,'1972-W271','1972-02-22 15:52:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 29492,'1972-W272','1972-02-22 17:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 29493,'1972-W273','1972-02-22 20:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 29494,'1972-W274','1972-02-23 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',61 from dual union all
select 29495,'1972-W275','1972-02-23 11:40:00','suborbital rocket','success',716,'INTA HW-7210','','','','Meteo',396,'',63 from dual union all
select 29496,'1972-W276','1972-02-23 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 29497,'1972-W277','1972-02-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 29498,'1972-W278','1972-02-23 15:47:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29499,'1972-W279','1972-02-23 17:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 295,'1944-S268','1944-12-01 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1728,'',90 from dual union all
select 2950,'1955-A126','1955-02-23 00:00:00','atmospheric rocket','success',836,'4219R','Nike 425R','','','Test',1895,'',10 from dual union all
select 29500,'1972-W28','1972-01-06 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 29501,'1972-W280','1972-02-23 18:56:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 29502,'1972-W281','1972-02-23 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 29503,'1972-W282','1972-02-23 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 29504,'1972-W283','1972-02-24 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 29505,'1972-W284','1972-02-24 15:36:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',59 from dual union all
select 29506,'1972-W285','1972-02-24 15:51:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29507,'1972-W286','1972-02-24 15:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29508,'1972-W287','1972-02-24 18:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',56 from dual union all
select 29509,'1972-W288','1972-02-24 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 2951,'1955-A127','1955-02-23 00:00:00','atmospheric rocket','success',836,'4223E','','','','Test',1895,'',10 from dual union all
select 29510,'1972-W289','1972-02-24 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 29511,'1972-W29','1972-01-06 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 29512,'1972-W290','1972-02-25 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 29513,'1972-W291','1972-02-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 29514,'1972-W292','1972-02-25 15:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',70 from dual union all
select 29515,'1972-W293','1972-02-25 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 29516,'1972-W294','1972-02-25 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',51 from dual union all
select 29517,'1972-W295','1972-02-25 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 29518,'1972-W296','1972-02-25 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 29519,'1972-W297','1972-02-26 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 2952,'1955-A128','1955-02-24 00:00:00','military missile','success',255,'623-21','','','','Test',253,'',22 from dual union all
select 29520,'1972-W298','1972-02-28 14:47:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',61 from dual union all
select 29521,'1972-W299','1972-02-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 29522,'1972-W30','1972-01-06 23:24:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 29523,'1972-W300','1972-02-28 15:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 29524,'1972-W301','1972-02-28 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 29525,'1972-W302','1972-02-28 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 29526,'1972-W303','1972-02-28 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 29527,'1972-W304','1972-02-29 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 29528,'1972-W305','1972-03-01 14:35:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',67 from dual;
insert into LAUNCH
select 29529,'1972-W306','1972-03-01 14:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',58 from dual union all
select 2953,'1955-A129','1955-02-24 00:00:00','atmospheric rocket','unknown',576,'835-PO','','','','Test',1895,'',10 from dual union all
select 29530,'1972-W307','1972-03-01 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 29531,'1972-W308','1972-03-01 15:45:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 29532,'1972-W309','1972-03-01 18:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',54 from dual union all
select 29533,'1972-W31','1972-01-07 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 29534,'1972-W310','1972-03-01 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 29535,'1972-W311','1972-03-01 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',65 from dual union all
select 29536,'1972-W312','1972-03-02 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 29537,'1972-W313','1972-03-02 15:51:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29538,'1972-W314','1972-03-02 21:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 29539,'1972-W315','1972-03-03 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 2954,'1955-A13','1955-01-12 00:00:00','atmospheric rocket','success',835,'','Wing drag model','','','Test',1821,'',10 from dual union all
select 29540,'1972-W316','1972-03-03 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 29541,'1972-W317','1972-03-03 15:30:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',66 from dual union all
select 29542,'1972-W318','1972-03-03 16:00:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29543,'1972-W319','1972-03-03 16:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 29544,'1972-W32','1972-01-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',55 from dual union all
select 29545,'1972-W320','1972-03-03 18:14:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',51 from dual union all
select 29546,'1972-W321','1972-03-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 29547,'1972-W322','1972-03-03 19:43:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29548,'1972-W323','1972-03-03 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 29549,'1972-W325','1972-03-06 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 2955,'1955-A130','1955-02-24 00:00:00','atmospheric rocket','unknown',576,'915-RS','','','','Test',1895,'',10 from dual union all
select 29550,'1972-W326','1972-03-06 16:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 29551,'1972-W327','1972-03-06 18:00:00','suborbital rocket','success',88,'','Arcas Starute 3/STS','','','Test/Aeron',1895,'',74 from dual union all
select 29552,'1972-W328','1972-03-06 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 29553,'1972-W329','1972-03-06 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 29554,'1972-W33','1972-01-07 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 29555,'1972-W330','1972-03-06 19:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 29556,'1972-W331','1972-03-06 20:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 29557,'1972-W332','1972-03-06 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 29558,'1972-W333','1972-03-07 14:15:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',62 from dual union all
select 29559,'1972-W334','1972-03-08 04:18:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 2956,'1955-A131','1955-02-24 00:00:00','atmospheric rocket','unknown',576,'839-PO','','','','Test',1895,'',10 from dual union all
select 29560,'1972-W335','1972-03-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29561,'1972-W336','1972-03-08 15:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',62 from dual union all
select 29562,'1972-W337','1972-03-08 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 29563,'1972-W338','1972-03-08 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 29564,'1972-W339','1972-03-08 18:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',62 from dual union all
select 29565,'1972-W34','1972-01-07 15:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 29566,'1972-W340','1972-03-08 18:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 29567,'1972-W341','1972-03-08 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 29568,'1972-W342','1972-03-10 02:14:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29569,'1972-W343','1972-03-10 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 2957,'1955-A132','1955-02-24 00:00:00','atmospheric rocket','unknown',576,'911-RS','','','','Test',1895,'',10 from dual union all
select 29570,'1972-W344','1972-03-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29571,'1972-W345','1972-03-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 29572,'1972-W346','1972-03-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 29573,'1972-W347','1972-03-10 15:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 29574,'1972-W348','1972-03-10 16:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',55 from dual union all
select 29575,'1972-W349','1972-03-10 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 29576,'1972-W35','1972-01-07 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 29577,'1972-W350','1972-03-10 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 29578,'1972-W351','1972-03-11 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 29579,'1972-W352','1972-03-11 01:45:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 2958,'1955-A133','1955-02-24 00:00:00','military missile','success',771,'GM-12449','','','','Test',222,'',12 from dual union all
select 29580,'1972-W353','1972-03-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29581,'1972-W354','1972-03-13 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 29582,'1972-W355','1972-03-13 15:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 29583,'1972-W356','1972-03-13 17:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 29584,'1972-W357','1972-03-13 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 29585,'1972-W358','1972-03-15 11:57:00','suborbital rocket','success',716,'INTA HD-7205','','','','Meteo',396,'',60 from dual union all
select 29586,'1972-W359','1972-03-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 29587,'1972-W36','1972-01-07 16:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 29588,'1972-W360','1972-03-15 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 29589,'1972-W361','1972-03-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 2959,'1955-A134','1955-02-25 00:00:00','atmospheric rocket','success',836,'1910T','','','','Test',1895,'',10 from dual union all
select 29590,'1972-W362','1972-03-15 15:33:00','suborbital rocket','success',716,'T1-5939','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29591,'1972-W363','1972-03-15 15:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',50 from dual union all
select 29592,'1972-W364','1972-03-15 16:21:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',52 from dual union all
select 29593,'1972-W365','1972-03-15 17:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',56 from dual union all
select 29594,'1972-W366','1972-03-15 18:08:00','suborbital rocket','success',1281,'','Starute,Datasonde','','','Aeron',1821,'',63 from dual union all
select 29595,'1972-W367','1972-03-15 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',57 from dual union all
select 29596,'1972-W368','1972-03-15 19:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29597,'1972-W369','1972-03-15 20:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 29598,'1972-W37','1972-01-07 17:41:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 29599,'1972-W370','1972-03-15 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 296,'1944-S269','1944-12-01 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1728,'',90 from dual union all
select 2960,'1955-A135','1955-02-25 00:00:00','atmospheric rocket','success',836,'1931T','','','','Test',1895,'',10 from dual union all
select 29600,'1972-W371','1972-03-16 00:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 29601,'1972-W372','1972-03-16 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',57 from dual union all
select 29602,'1972-W373','1972-03-16 02:03:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 29603,'1972-W374','1972-03-16 02:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 29604,'1972-W375','1972-03-16 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 29605,'1972-W376','1972-03-16 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29606,'1972-W377','1972-03-16 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',55 from dual union all
select 29607,'1972-W378','1972-03-16 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 29608,'1972-W379','1972-03-16 18:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',61 from dual union all
select 29609,'1972-W38','1972-01-07 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 2961,'1955-A136','1955-02-28 00:00:00','atmospheric rocket','success',836,'3313T','','','','Test',1895,'',10 from dual union all
select 29610,'1972-W380','1972-03-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29611,'1972-W381','1972-03-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 29612,'1972-W382','1972-03-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 29613,'1972-W383','1972-03-17 15:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29614,'1972-W384','1972-03-17 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',62 from dual union all
select 29615,'1972-W385','1972-03-17 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 29616,'1972-W386','1972-03-17 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 29617,'1972-W387','1972-03-17 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 29618,'1972-W388','1972-03-17 20:26:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual;
insert into LAUNCH
select 29619,'1972-W389','1972-03-18 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',53 from dual union all
select 2962,'1955-A137','1955-03-01 00:00:00','atmospheric rocket','success',836,'1691T','','','','Test',1895,'',10 from dual union all
select 29620,'1972-W39','1972-01-07 20:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 29621,'1972-W390','1972-03-20 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 29622,'1972-W391','1972-03-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 29623,'1972-W392','1972-03-20 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 29624,'1972-W393','1972-03-20 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 29625,'1972-W394','1972-03-20 17:24:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 29626,'1972-W395','1972-03-20 17:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29627,'1972-W396','1972-03-20 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 29628,'1972-W397','1972-03-20 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29629,'1972-W398','1972-03-20 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',54 from dual union all
select 2963,'1955-A138','1955-03-01 00:00:00','atmospheric rocket','success',836,'1507T','','','','Test',1895,'',10 from dual union all
select 29630,'1972-W399','1972-03-20 20:00:00','suborbital rocket','success',716,'T1-5549','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 29631,'1972-W40','1972-01-07 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 29632,'1972-W400','1972-03-21 01:32:59','suborbital rocket','success',716,'T1-5950','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 29633,'1972-W401','1972-03-22 07:32:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 29634,'1972-W402','1972-03-22 11:37:59','suborbital rocket','success',716,'INTA HD-7206','','','','Meteo',396,'',59 from dual union all
select 29635,'1972-W403','1972-03-22 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',50 from dual union all
select 29636,'1972-W404','1972-03-22 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 29637,'1972-W405','1972-03-22 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 29638,'1972-W406','1972-03-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 29639,'1972-W407','1972-03-22 16:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 2964,'1955-A139','1955-03-01 00:00:00','atmospheric rocket','success',836,'1956T','','','','Test',1895,'',10 from dual union all
select 29640,'1972-W408','1972-03-22 17:10:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',61 from dual union all
select 29641,'1972-W409','1972-03-22 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 29642,'1972-W41','1972-01-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 29643,'1972-W410','1972-03-22 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',57 from dual union all
select 29644,'1972-W411','1972-03-23 01:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 29645,'1972-W412','1972-03-23 12:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',53 from dual union all
select 29646,'1972-W413','1972-03-24 02:24:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29647,'1972-W414','1972-03-24 12:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29648,'1972-W415','1972-03-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29649,'1972-W416','1972-03-24 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 2965,'1955-A14','1955-01-13 00:00:00','military missile','unknown',844,'B1 1810-1','Test','','','Test',1895,'',10 from dual union all
select 29650,'1972-W417','1972-03-24 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 29651,'1972-W418','1972-03-24 18:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29652,'1972-W419','1972-03-24 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 29653,'1972-W42','1972-01-10 15:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 29654,'1972-W420','1972-03-24 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 29655,'1972-W421','1972-03-25 01:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 29656,'1972-W422','1972-03-25 03:19:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 29657,'1972-W423','1972-03-27 12:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29658,'1972-W424','1972-03-27 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 29659,'1972-W425','1972-03-27 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',51 from dual union all
select 2966,'1955-A140','1955-03-01 00:00:00','atmospheric rocket','success',836,'1844T','','','','Test',1895,'',10 from dual union all
select 29660,'1972-W426','1972-03-27 16:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 29661,'1972-W427','1972-03-27 18:45:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',51 from dual union all
select 29662,'1972-W428','1972-03-27 19:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 29663,'1972-W429','1972-03-27 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',58 from dual union all
select 29664,'1972-W43','1972-01-10 16:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 29665,'1972-W430','1972-03-28 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 29666,'1972-W431','1972-03-28 08:32:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 29667,'1972-W432','1972-03-29 12:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 29668,'1972-W433','1972-03-29 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 29669,'1972-W434','1972-03-29 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 2967,'1955-A141','1955-03-02 00:00:00','atmospheric rocket','unknown',576,'570EM','','','','Test',1895,'',10 from dual union all
select 29670,'1972-W435','1972-03-29 17:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 29671,'1972-W436','1972-03-29 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 29672,'1972-W437','1972-03-29 20:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29673,'1972-W438','1972-03-29 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 29674,'1972-W439','1972-03-30 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 29675,'1972-W44','1972-01-10 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 29676,'1972-W440','1972-03-30 09:24:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 29677,'1972-W441','1972-03-30 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 29678,'1972-W442','1972-03-30 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',57 from dual union all
select 29679,'1972-W443','1972-03-30 19:40:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',56 from dual union all
select 2968,'1955-A142','1955-03-02 00:00:00','military missile','success',771,'GM-12461','','','','Test',222,'',12 from dual union all
select 29680,'1972-W444','1972-03-31 09:33:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 29681,'1972-W445','1972-03-31 11:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29682,'1972-W446','1972-03-31 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 29683,'1972-W447','1972-03-31 14:45:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',58 from dual union all
select 29684,'1972-W448','1972-03-31 19:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 29685,'1972-W449','1972-03-31 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 29686,'1972-W45','1972-01-10 21:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 29687,'1972-W450','1972-04-01 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 29688,'1972-W451','1972-04-03 11:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29689,'1972-W452','1972-04-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 2969,'1955-A143','1955-03-02 00:00:00','military missile','success',771,'GM-12448','','','','Test',222,'',12 from dual union all
select 29690,'1972-W453','1972-04-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 29691,'1972-W454','1972-04-03 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 29692,'1972-W455','1972-04-03 15:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29693,'1972-W456','1972-04-03 16:01:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',56 from dual union all
select 29694,'1972-W457','1972-04-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 29695,'1972-W458','1972-04-03 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',57 from dual union all
select 29696,'1972-W459','1972-04-04 01:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 29697,'1972-W46','1972-01-11 14:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',79 from dual union all
select 29698,'1972-W460','1972-04-04 19:12:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',50 from dual union all
select 29699,'1972-W461','1972-04-05 11:55:00','suborbital rocket','success',716,'INTA HD-7207','','','','Meteo',396,'',60 from dual union all
select 297,'1944-S27','1944-06-01 00:00:00','military missile','success',8,'','','','','Test',662,'',90 from dual union all
select 2970,'1955-A144','1955-03-02 00:00:00','atmospheric rocket','success',836,'1955T','','','','Test',1895,'',10 from dual union all
select 29700,'1972-W462','1972-04-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 29701,'1972-W463','1972-04-05 14:31:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',64 from dual union all
select 29702,'1972-W464','1972-04-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 29703,'1972-W465','1972-04-05 15:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 29704,'1972-W466','1972-04-05 15:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',51 from dual union all
select 29705,'1972-W467','1972-04-05 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',55 from dual union all
select 29706,'1972-W468','1972-04-05 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 29707,'1972-W469','1972-04-06 01:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 29708,'1972-W47','1972-01-11 16:10:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual;
insert into LAUNCH
select 29709,'1972-W470','1972-04-06 09:32:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 2971,'1955-A145','1955-03-02 22:09:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 29710,'1972-W471','1972-04-06 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 29711,'1972-W472','1972-04-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 29712,'1972-W473','1972-04-07 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 29713,'1972-W474','1972-04-07 14:45:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',59 from dual union all
select 29714,'1972-W475','1972-04-07 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 29715,'1972-W476','1972-04-07 15:17:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 29716,'1972-W477','1972-04-07 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',55 from dual union all
select 29717,'1972-W478','1972-04-08 01:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 29718,'1972-W479','1972-04-08 09:13:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 29719,'1972-W48','1972-01-11 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 2972,'1955-A146','1955-03-03 00:00:00','military missile','success',255,'623-11','','','','Test',253,'',20 from dual union all
select 29720,'1972-W480','1972-04-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 29721,'1972-W481','1972-04-10 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 29722,'1972-W482','1972-04-10 15:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',61 from dual union all
select 29723,'1972-W483','1972-04-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 29724,'1972-W484','1972-04-10 16:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 29725,'1972-W485','1972-04-10 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',51 from dual union all
select 29726,'1972-W486','1972-04-10 20:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 29727,'1972-W487','1972-04-10 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',51 from dual union all
select 29728,'1972-W488','1972-04-11 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 29729,'1972-W489','1972-04-11 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',55 from dual union all
select 2973,'1955-A147','1955-03-03 00:00:00','atmospheric rocket','success',836,'1751T','','','','Test',1895,'',10 from dual union all
select 29730,'1972-W49','1972-01-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 29731,'1972-W490','1972-04-12 03:45:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 29732,'1972-W491','1972-04-12 11:37:00','suborbital rocket','success',716,'INTA HD-7208','','','','Meteo',396,'',61 from dual union all
select 29733,'1972-W492','1972-04-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29734,'1972-W493','1972-04-12 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 29735,'1972-W494','1972-04-12 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 29736,'1972-W495','1972-04-12 15:10:00','suborbital rocket','success',1506,'T 1-5765','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 29737,'1972-W496','1972-04-12 16:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 29738,'1972-W497','1972-04-12 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 29739,'1972-W498','1972-04-13 01:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 2974,'1955-A148','1955-03-03 00:00:00','atmospheric rocket','success',836,'1711T','','','','Test',1895,'',10 from dual union all
select 29740,'1972-W499','1972-04-13 05:12:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29741,'1972-W50','1972-01-12 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 29742,'1972-W500','1972-04-14 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 29743,'1972-W501','1972-04-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 29744,'1972-W502','1972-04-14 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 29745,'1972-W503','1972-04-14 15:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 29746,'1972-W504','1972-04-14 16:04:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',54 from dual union all
select 29747,'1972-W505','1972-04-14 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 29748,'1972-W506','1972-04-14 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',63 from dual union all
select 29749,'1972-W507','1972-04-15 03:56:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 2975,'1955-A149','1955-03-03 00:00:00','atmospheric rocket','success',836,'1726T','','','','Test',1895,'',10 from dual union all
select 29750,'1972-W508','1972-04-15 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 29751,'1972-W509','1972-04-16 19:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 29752,'1972-W51','1972-01-12 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 29753,'1972-W510','1972-04-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 29754,'1972-W511','1972-04-17 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 29755,'1972-W512','1972-04-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 29756,'1972-W513','1972-04-17 17:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',54 from dual union all
select 29757,'1972-W514','1972-04-17 18:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29758,'1972-W515','1972-04-17 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',53 from dual union all
select 29759,'1972-W516','1972-04-17 20:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 2976,'1955-A150','1955-03-03 00:00:00','atmospheric rocket','success',836,'1613T','','','','Test',1895,'',10 from dual union all
select 29760,'1972-W517','1972-04-17 21:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 29761,'1972-W518','1972-04-18 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 29762,'1972-W519','1972-04-18 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',52 from dual union all
select 29763,'1972-W52','1972-01-12 15:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 29764,'1972-W520','1972-04-18 19:08:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 29765,'1972-W521','1972-04-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 29766,'1972-W522','1972-04-19 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 29767,'1972-W523','1972-04-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 29768,'1972-W524','1972-04-19 16:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 29769,'1972-W525','1972-04-19 16:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 2977,'1955-A151','1955-03-04 00:00:00','atmospheric rocket','success',836,'2763T','','','','Test',1895,'',10 from dual union all
select 29770,'1972-W526','1972-04-19 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 29771,'1972-W527','1972-04-19 19:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 29772,'1972-W528','1972-04-20 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 29773,'1972-W529','1972-04-20 04:09:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29774,'1972-W53','1972-01-12 17:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 29775,'1972-W530','1972-04-20 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 29776,'1972-W531','1972-04-21 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29777,'1972-W532','1972-04-21 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 29778,'1972-W533','1972-04-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 29779,'1972-W534','1972-04-21 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 2978,'1955-A152','1955-03-04 00:00:00','atmospheric rocket','success',836,'1517T','','','','Test',1895,'',10 from dual union all
select 29780,'1972-W535','1972-04-21 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 29781,'1972-W536','1972-04-22 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 29782,'1972-W537','1972-04-22 03:17:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 29783,'1972-W538','1972-04-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 29784,'1972-W539','1972-04-24 15:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',63 from dual union all
select 29785,'1972-W54','1972-01-12 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',61 from dual union all
select 29786,'1972-W540','1972-04-24 16:04:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',59 from dual union all
select 29787,'1972-W541','1972-04-24 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 29788,'1972-W542','1972-04-24 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 29789,'1972-W543','1972-04-24 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',51 from dual union all
select 2979,'1955-A153','1955-03-04 00:00:00','atmospheric rocket','success',836,'1904T','','','','Test',1895,'',10 from dual union all
select 29790,'1972-W544','1972-04-24 20:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',63 from dual union all
select 29791,'1972-W545','1972-04-24 20:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 29792,'1972-W546','1972-04-25 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 29793,'1972-W547','1972-04-25 03:59:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 29794,'1972-W548','1972-04-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 29795,'1972-W549','1972-04-25 16:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29796,'1972-W55','1972-01-12 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 29797,'1972-W550','1972-04-25 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 29798,'1972-W551','1972-04-26 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 29799,'1972-W552','1972-04-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual;
insert into LAUNCH
select 298,'1944-S270','1944-12-01 00:00:00','military missile','success',1470,'','','','','Weapon',1734,'',90 from dual union all
select 2980,'1955-A154','1955-03-04 00:00:00','atmospheric rocket','success',836,'3309T','','','','Test',1895,'',10 from dual union all
select 29800,'1972-W553','1972-04-26 15:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 29801,'1972-W554','1972-04-26 16:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 29802,'1972-W555','1972-04-26 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',57 from dual union all
select 29803,'1972-W556','1972-04-26 22:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 29804,'1972-W557','1972-04-27 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 29805,'1972-W558','1972-04-27 01:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',62 from dual union all
select 29806,'1972-W559','1972-04-27 04:24:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 29807,'1972-W56','1972-01-12 20:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 29808,'1972-W560','1972-04-27 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 29809,'1972-W561','1972-04-27 19:30:00','suborbital rocket','success',1061,'','Density gauge','','','Meteo',873,'',55 from dual union all
select 2981,'1955-A155','1955-03-04 00:00:00','atmospheric rocket','success',836,'4211E','','','','Test',1895,'',10 from dual union all
select 29810,'1972-W562','1972-04-28 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 29811,'1972-W563','1972-04-28 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 29812,'1972-W564','1972-04-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 29813,'1972-W565','1972-04-28 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',51 from dual union all
select 29814,'1972-W566','1972-04-28 15:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 29815,'1972-W567','1972-04-28 18:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 29816,'1972-W568','1972-04-28 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',61 from dual union all
select 29817,'1972-W569','1972-04-28 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 29818,'1972-W57','1972-01-12 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',75 from dual union all
select 29819,'1972-W570','1972-04-28 21:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 2982,'1955-A156','1955-03-04 00:00:00','atmospheric rocket','success',836,'1644T','','','','Test',1895,'',10 from dual union all
select 29820,'1972-W571','1972-04-29 01:10:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 29821,'1972-W572','1972-05-01 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 29822,'1972-W573','1972-05-01 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',52 from dual union all
select 29823,'1972-W574','1972-05-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29824,'1972-W575','1972-05-01 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 29825,'1972-W576','1972-05-01 15:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 29826,'1972-W577','1972-05-01 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 29827,'1972-W578','1972-05-01 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',55 from dual union all
select 29828,'1972-W579','1972-05-01 21:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 29829,'1972-W58','1972-01-12 21:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 2983,'1955-A157','1955-03-05 00:00:00','atmospheric rocket','success',836,'1764T','','','','Test',1895,'',10 from dual union all
select 29830,'1972-W580','1972-05-01 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',53 from dual union all
select 29831,'1972-W581','1972-05-02 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 29832,'1972-W582','1972-05-03 13:00:59','suborbital rocket','success',716,'INTA HD-7211','','','','Meteo',396,'',59 from dual union all
select 29833,'1972-W583','1972-05-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 29834,'1972-W584','1972-05-03 14:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',59 from dual union all
select 29835,'1972-W585','1972-05-03 14:44:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 29836,'1972-W586','1972-05-03 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 29837,'1972-W587','1972-05-03 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 29838,'1972-W588','1972-05-03 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 29839,'1972-W589','1972-05-03 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 2984,'1955-A158','1955-03-08 00:00:00','atmospheric rocket','success',836,'1976T','','','','Test',1895,'',10 from dual union all
select 29840,'1972-W59','1972-01-12 23:00:00','suborbital rocket','success',716,'INTA HD-7202','','','','Meteo',396,'',60 from dual union all
select 29841,'1972-W590','1972-05-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',54 from dual union all
select 29842,'1972-W591','1972-05-03 19:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 29843,'1972-W592','1972-05-03 22:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 29844,'1972-W593','1972-05-04 01:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 29845,'1972-W594','1972-05-04 15:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 29846,'1972-W595','1972-05-04 17:00:00','suborbital rocket','success',88,'NASA 15.91GM','','','','Aeronomy',1821,'',60 from dual union all
select 29847,'1972-W596','1972-05-10 13:08:00','suborbital rocket','success',716,'INTA HW-7211','','','','Meteo',396,'',56 from dual union all
select 29848,'1972-W597','1972-05-05 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 29849,'1972-W598','1972-05-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 2985,'1955-A159','1955-03-08 00:00:00','atmospheric rocket','success',836,'1899T','','','','Test',1895,'',10 from dual union all
select 29850,'1972-W599','1972-05-05 14:48:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',57 from dual union all
select 29851,'1972-W60','1972-01-13 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 29852,'1972-W600','1972-05-05 14:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29853,'1972-W601','1972-05-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 29854,'1972-W602','1972-05-05 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 29855,'1972-W603','1972-05-05 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29856,'1972-W604','1972-05-05 19:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 29857,'1972-W605','1972-05-05 19:43:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 29858,'1972-W606','1972-05-05 19:55:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29859,'1972-W607','1972-05-05 21:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 2986,'1955-A16','1955-01-13 00:00:00','atmospheric rocket','success',836,'2771T','','','','Test',1895,'',10 from dual union all
select 29860,'1972-W608','1972-05-06 01:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 29861,'1972-W609','1972-05-08 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 29862,'1972-W61','1972-01-13 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 29863,'1972-W610','1972-05-08 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 29864,'1972-W611','1972-05-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 29865,'1972-W612','1972-05-08 14:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 29866,'1972-W613','1972-05-08 14:35:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',57 from dual union all
select 29867,'1972-W614','1972-05-08 14:52:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 29868,'1972-W615','1972-05-08 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 29869,'1972-W616','1972-05-08 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 2987,'1955-A160','1955-03-08 00:00:00','atmospheric rocket','success',836,'1741T','','','','Test',1895,'',10 from dual union all
select 29870,'1972-W617','1972-05-08 19:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29871,'1972-W618','1972-05-08 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',50 from dual union all
select 29872,'1972-W619','1972-05-08 20:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 29873,'1972-W62','1972-01-13 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 29874,'1972-W620','1972-05-09 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 29875,'1972-W621','1972-05-09 14:03:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',50 from dual union all
select 29876,'1972-W622','1972-05-09 14:54:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29877,'1972-W623','1972-05-10 13:54:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',276,'',52 from dual union all
select 29878,'1972-W624','1972-05-10 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',54 from dual union all
select 29879,'1972-W625','1972-05-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 2988,'1955-A161','1955-03-09 00:00:00','atmospheric rocket','unknown',576,'M-102-EM','','','','Test',1895,'',10 from dual union all
select 29880,'1972-W626','1972-05-10 14:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 29881,'1972-W627','1972-05-10 14:45:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 29882,'1972-W628','1972-05-10 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29883,'1972-W629','1972-05-10 18:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29884,'1972-W63','1972-01-13 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 29885,'1972-W630','1972-05-10 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 29886,'1972-W631','1972-05-10 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',64 from dual union all
select 29887,'1972-W632','1972-05-11 01:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 29888,'1972-W633','1972-05-11 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 29889,'1972-W634','1972-05-11 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual;
insert into LAUNCH
select 2989,'1955-A162','1955-03-09 00:00:00','military missile','success',771,'GM-52-1889','','','','Test',222,'',12 from dual union all
select 29890,'1972-W635','1972-05-12 13:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 29891,'1972-W636','1972-05-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29892,'1972-W637','1972-05-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 29893,'1972-W638','1972-05-12 14:42:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 29894,'1972-W639','1972-05-12 14:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 29895,'1972-W64','1972-01-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 29896,'1972-W640','1972-05-12 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 29897,'1972-W641','1972-05-12 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 29898,'1972-W642','1972-05-12 21:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 29899,'1972-W643','1972-05-13 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 299,'1944-S271','1944-12-01 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1728,'',90 from dual union all
select 2990,'1955-A163','1955-03-09 00:00:00','atmospheric rocket','success',836,'3097R','Nike 429R','','','Test',1895,'',10 from dual union all
select 29900,'1972-W644','1972-05-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 29901,'1972-W645','1972-05-15 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 29902,'1972-W646','1972-05-15 14:40:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 29903,'1972-W647','1972-05-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 29904,'1972-W648','1972-05-15 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 29905,'1972-W649','1972-05-15 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29906,'1972-W65','1972-01-14 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 29907,'1972-W650','1972-05-15 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 29908,'1972-W651','1972-05-15 21:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 29909,'1972-W652','1972-05-16 14:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',51 from dual union all
select 2991,'1955-A164','1955-03-09 00:00:00','atmospheric rocket','success',836,'1702T','','','','Test',1895,'',10 from dual union all
select 29910,'1972-W653','1972-05-16 15:11:00','suborbital rocket','success',88,'NASA 15.99GM','','','','Aeronomy',35,'',52 from dual union all
select 29911,'1972-W654','1972-05-17 12:51:00','suborbital rocket','success',716,'INTA HW-7212','','','','Meteo',396,'',58 from dual union all
select 29912,'1972-W655','1972-05-17 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 29913,'1972-W656','1972-05-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 29914,'1972-W657','1972-05-17 14:15:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',64 from dual union all
select 29915,'1972-W658','1972-05-17 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',54 from dual union all
select 29916,'1972-W659','1972-05-17 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',76 from dual union all
select 29917,'1972-W66','1972-01-14 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 29918,'1972-W660','1972-05-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 29919,'1972-W661','1972-05-17 16:54:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 2992,'1955-A165','1955-03-09 00:00:00','atmospheric rocket','success',836,'1727T','','','','Test',1895,'',10 from dual union all
select 29920,'1972-W662','1972-05-17 17:07:00','suborbital rocket','success',1506,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 29921,'1972-W663','1972-05-17 17:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29922,'1972-W664','1972-05-17 18:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 29923,'1972-W665','1972-05-17 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 29924,'1972-W666','1972-05-17 22:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 29925,'1972-W667','1972-05-17 23:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',60 from dual union all
select 29926,'1972-W668','1972-05-18 14:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',62 from dual union all
select 29927,'1972-W669','1972-05-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 29928,'1972-W67','1972-01-14 15:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',54 from dual union all
select 29929,'1972-W670','1972-05-18 18:10:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 2993,'1955-A166','1955-03-09 00:00:00','atmospheric rocket','success',836,'1886T','','','','Test',1895,'',10 from dual union all
select 29930,'1972-W671','1972-05-19 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',50 from dual union all
select 29931,'1972-W672','1972-05-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',55 from dual union all
select 29932,'1972-W673','1972-05-19 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 29933,'1972-W674','1972-05-19 14:51:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 29934,'1972-W675','1972-05-19 15:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 29935,'1972-W676','1972-05-19 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 29936,'1972-W677','1972-05-19 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 29937,'1972-W678','1972-05-22 14:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29938,'1972-W679','1972-05-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 29939,'1972-W68','1972-01-14 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 2994,'1955-A167','1955-03-09 00:00:00','atmospheric rocket','success',836,'3014R','Nike 430R','','','Test',1895,'',10 from dual union all
select 29940,'1972-W680','1972-05-22 15:34:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',61 from dual union all
select 29941,'1972-W681','1972-05-22 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 29942,'1972-W682','1972-05-22 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',59 from dual union all
select 29943,'1972-W683','1972-05-22 21:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 29944,'1972-W684','1972-05-23 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 29945,'1972-W685','1972-05-23 15:02:00','suborbital rocket','success',88,'NASA 15.100GM','','','','Aeronomy',35,'',56 from dual union all
select 29946,'1972-W686','1972-05-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 29947,'1972-W687','1972-05-24 14:35:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 29948,'1972-W688','1972-05-24 14:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 29949,'1972-W689','1972-05-24 14:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 2995,'1955-A168','1955-03-09 00:00:00','atmospheric rocket','success',836,'1927T','','','','Test',1895,'',10 from dual union all
select 29950,'1972-W69','1972-01-14 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 29951,'1972-W690','1972-05-24 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',58 from dual union all
select 29952,'1972-W691','1972-05-24 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 29953,'1972-W692','1972-05-24 17:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29954,'1972-W693','1972-05-24 19:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',54 from dual union all
select 29955,'1972-W694','1972-05-25 15:52:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 29956,'1972-W695','1972-05-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 29957,'1972-W696','1972-05-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 29958,'1972-W697','1972-05-26 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 29959,'1972-W698','1972-05-26 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 2996,'1955-A169','1955-03-09 00:00:00','atmospheric rocket','success',836,'4351E','','','','Test',1895,'',10 from dual union all
select 29960,'1972-W699','1972-05-26 14:43:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 29961,'1972-W70','1972-01-14 22:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 29962,'1972-W700','1972-05-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 29963,'1972-W701','1972-05-26 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 29964,'1972-W702','1972-05-26 18:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 29965,'1972-W703','1972-05-26 18:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 29966,'1972-W704','1972-05-26 20:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 29967,'1972-W705','1972-05-29 16:47:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 29968,'1972-W706','1972-05-30 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 29969,'1972-W707','1972-05-30 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 2997,'1955-A17','1955-01-13 00:00:00','military missile','success',1214,'N-69B GM-13097','','','','Test',224,'',10 from dual union all
select 29970,'1972-W708','1972-05-30 18:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',51 from dual union all
select 29971,'1972-W709','1972-05-30 21:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 29972,'1972-W71','1972-01-15 01:01:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 29973,'1972-W710','1972-06-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',60 from dual union all
select 29974,'1972-W711','1972-06-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',60 from dual union all
select 29975,'1972-W712','1972-06-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',60 from dual union all
select 29976,'1972-W713','1972-06-01 00:00:00','suborbital rocket','unknown',631,'','','','','Meteo',428,'',60 from dual union all
select 29977,'1972-W714','1972-05-31 10:14:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',72 from dual union all
select 29978,'1972-W715','1972-05-31 13:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 29979,'1972-W716','1972-05-31 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual;
insert into LAUNCH
select 2998,'1955-A170','1955-03-10 00:00:00','atmospheric rocket','success',836,'1975T','','','','Test',1895,'',10 from dual union all
select 29980,'1972-W717','1972-05-31 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 29981,'1972-W718','1972-05-31 14:14:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 29982,'1972-W719','1972-05-31 14:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 29983,'1972-W72','1972-01-15 23:59:00','suborbital rocket','success',257,'NASA 15.97UI','ATT-BA-4','','','Auroral',428,'',53 from dual union all
select 29984,'1972-W720','1972-05-31 14:42:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 29985,'1972-W721','1972-05-31 14:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',60 from dual union all
select 29986,'1972-W722','1972-05-31 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 29987,'1972-W723','1972-05-31 16:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 29988,'1972-W724','1972-05-31 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 29989,'1972-W725','1972-05-31 18:20:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',54 from dual union all
select 2999,'1955-A171','1955-03-10 00:00:00','atmospheric rocket','success',836,'1753T','','','','Test',1895,'',10 from dual union all
select 29990,'1972-W726','1972-05-31 21:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 29991,'1972-W727','1972-06-01 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 29992,'1972-W728','1972-06-02 03:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 29993,'1972-W729','1972-06-02 08:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 29994,'1972-W73','1972-01-16 00:00:00','suborbital rocket','success',257,'NASA 15.98UI','','','','Ionos',428,'',60 from dual union all
select 29995,'1972-W730','1972-06-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 29996,'1972-W731','1972-06-02 14:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 29997,'1972-W732','1972-06-02 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 29998,'1972-W733','1972-06-02 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 29999,'1972-W734','1972-06-02 18:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 3,'1942-A03','1942-10-21 00:00:00','military missile','success',8,'5','','','','Test',675,'',40 from dual union all
select 30,'1943-S08','1943-06-11 00:00:00','military missile','success',8,'29','','','','Test',675,'',95 from dual union all
select 300,'1944-S272','1944-11-30 11:19:00','military missile','success',1470,'19846','Ma310','','','Test',661,'',90 from dual union all
select 3000,'1955-A172','1955-03-10 00:00:00','atmospheric rocket','success',836,'1949T','','','','Test',1895,'',10 from dual union all
select 30000,'1972-W735','1972-06-02 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 30001,'1972-W736','1972-06-02 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 30002,'1972-W737','1972-06-03 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 30003,'1972-W738','1972-06-03 04:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 30004,'1972-W739','1972-06-05 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 30005,'1972-W74','1972-01-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 30006,'1972-W740','1972-06-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 30007,'1972-W741','1972-06-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 30008,'1972-W742','1972-06-05 15:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 30009,'1972-W743','1972-06-05 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 3001,'1955-A173','1955-03-10 00:00:00','atmospheric rocket','success',835,'','F-101A Model','','','Test',1821,'',10 from dual union all
select 30010,'1972-W744','1972-06-05 19:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 30011,'1972-W745','1972-06-05 23:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 30012,'1972-W746','1972-06-07 13:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 30013,'1972-W747','1972-06-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 30014,'1972-W748','1972-06-07 14:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 30015,'1972-W749','1972-06-07 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',52 from dual union all
select 30016,'1972-W75','1972-01-17 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 30017,'1972-W750','1972-06-07 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 30018,'1972-W751','1972-06-07 17:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 30019,'1972-W752','1972-06-07 17:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 3002,'1955-A174','1955-03-10 19:42:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 30020,'1972-W753','1972-06-07 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 30021,'1972-W754','1972-06-07 23:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',54 from dual union all
select 30022,'1972-W755','1972-06-08 22:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 30023,'1972-W756','1972-06-09 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',51 from dual union all
select 30024,'1972-W757','1972-06-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 30025,'1972-W758','1972-06-09 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 30026,'1972-W759','1972-06-09 14:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 30027,'1972-W76','1972-01-17 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 30028,'1972-W760','1972-06-09 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 30029,'1972-W761','1972-06-09 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 3003,'1955-A175','1955-03-10 21:46:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 30030,'1972-W762','1972-06-09 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 30031,'1972-W763','1972-06-09 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 30032,'1972-W764','1972-06-09 23:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 30033,'1972-W765','1972-06-12 09:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 30034,'1972-W766','1972-06-12 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 30035,'1972-W767','1972-06-12 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 30036,'1972-W768','1972-06-12 15:59:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',51 from dual union all
select 30037,'1972-W769','1972-06-12 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 30038,'1972-W77','1972-01-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 30039,'1972-W770','1972-06-12 18:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 3004,'1955-A176','1955-03-11 00:00:00','atmospheric rocket','success',836,'4199E','','','','Test',1895,'',10 from dual union all
select 30040,'1972-W771','1972-06-12 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',54 from dual union all
select 30041,'1972-W772','1972-06-12 21:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 30042,'1972-W773','1972-06-12 22:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 30043,'1972-W774','1972-06-13 13:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 30044,'1972-W775','1972-06-13 15:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 30045,'1972-W776','1972-06-14 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 30046,'1972-W777','1972-06-14 13:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',60 from dual union all
select 30047,'1972-W778','1972-06-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 30048,'1972-W779','1972-06-14 14:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',64 from dual union all
select 30049,'1972-W78','1972-01-17 15:57:59','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 3005,'1955-A177','1955-03-11 00:00:00','military missile','unknown',844,'B3 1810-3','Test','','','Test',1895,'',10 from dual union all
select 30050,'1972-W780','1972-06-14 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 30051,'1972-W781','1972-06-14 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 30052,'1972-W782','1972-06-14 21:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 30053,'1972-W783','1972-06-14 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 30054,'1972-W784','1972-06-14 23:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',54 from dual union all
select 30055,'1972-W785','1972-06-15 14:38:59','suborbital rocket','success',1506,'T 1-5795','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 30056,'1972-W786','1972-06-15 15:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',63 from dual union all
select 30057,'1972-W787','1972-06-15 18:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 30058,'1972-W788','1972-06-15 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 30059,'1972-W789','1972-06-16 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 3006,'1955-A178','1955-03-11 00:00:00','atmospheric rocket','success',836,'3529T','','','','Test',1895,'',10 from dual union all
select 30060,'1972-W79','1972-01-17 16:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',60 from dual union all
select 30061,'1972-W790','1972-06-16 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 30062,'1972-W791','1972-06-16 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 30063,'1972-W792','1972-06-16 15:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 30064,'1972-W793','1972-06-16 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 30065,'1972-W794','1972-06-16 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 30066,'1972-W795','1972-06-16 22:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 30067,'1972-W796','1972-06-16 23:48:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual;
insert into LAUNCH
select 30068,'1972-W797','1972-06-17 00:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 30069,'1972-W798','1972-06-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 3007,'1955-A179','1955-03-11 00:00:00','atmospheric rocket','success',836,'1984T','','','','Test',1895,'',10 from dual union all
select 30070,'1972-W799','1972-06-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 30071,'1972-W80','1972-01-17 17:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 30072,'1972-W800','1972-06-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 30073,'1972-W801','1972-06-19 15:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 30074,'1972-W802','1972-06-19 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 30075,'1972-W803','1972-06-19 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 30076,'1972-W804','1972-06-19 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',54 from dual union all
select 30077,'1972-W805','1972-06-19 20:03:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',51 from dual union all
select 30078,'1972-W806','1972-06-19 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 30079,'1972-W807','1972-06-19 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',52 from dual union all
select 3008,'1955-A18','1955-01-14 00:00:00','military missile','success',771,'GM-12434','','','','Test',222,'',12 from dual union all
select 30080,'1972-W808','1972-06-20 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 30081,'1972-W809','1972-06-20 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 30082,'1972-W81','1972-01-17 19:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',51 from dual union all
select 30083,'1972-W810','1972-06-20 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 30084,'1972-W811','1972-06-21 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 30085,'1972-W812','1972-06-21 15:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 30086,'1972-W813','1972-06-21 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 30087,'1972-W814','1972-06-21 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 30088,'1972-W815','1972-06-21 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 30089,'1972-W816','1972-06-21 22:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 3009,'1955-A180','1955-03-11 00:00:00','atmospheric rocket','success',836,'1787T','','','','Test',1895,'',10 from dual union all
select 30090,'1972-W817','1972-06-21 23:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',57 from dual union all
select 30091,'1972-W818','1972-06-22 13:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 30092,'1972-W819','1972-06-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 30093,'1972-W82','1972-01-17 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 30094,'1972-W820','1972-06-22 15:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 30095,'1972-W821','1972-06-22 15:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 30096,'1972-W822','1972-06-22 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 30097,'1972-W823','1972-06-23 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 30098,'1972-W824','1972-06-23 14:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 30099,'1972-W825','1972-06-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 301,'1944-S273','1944-11-30 11:43:00','military missile','success',1470,'19849','Ma312','','','Test',661,'',90 from dual union all
select 3010,'1955-A181','1955-03-11 00:00:00','atmospheric rocket','success',836,'1766T','','','','Test',1895,'',10 from dual union all
select 30100,'1972-W826','1972-06-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 30101,'1972-W827','1972-06-23 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 30102,'1972-W828','1972-06-23 22:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 30103,'1972-W829','1972-06-23 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 30104,'1972-W83','1972-01-17 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 30105,'1972-W830','1972-06-24 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 30106,'1972-W831','1972-06-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 30107,'1972-W832','1972-06-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 30108,'1972-W833','1972-06-26 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 30109,'1972-W834','1972-06-26 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 3011,'1955-A182','1955-03-14 00:00:00','atmospheric rocket','unknown',576,'M-930-RM','','','','Test',1895,'',10 from dual union all
select 30110,'1972-W835','1972-06-26 23:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 30111,'1972-W836','1972-06-27 00:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',53 from dual union all
select 30112,'1972-W837','1972-06-28 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 30113,'1972-W838','1972-06-28 14:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 30114,'1972-W839','1972-06-28 15:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 30115,'1972-W84','1972-01-18 15:51:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 30116,'1972-W840','1972-06-28 15:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',61 from dual union all
select 30117,'1972-W841','1972-06-28 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 30118,'1972-W842','1972-06-28 18:25:00','suborbital rocket','success',88,'L1-5480','Stokes Flow Para','','','Test',1821,'',63 from dual union all
select 30119,'1972-W843','1972-06-28 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 3012,'1955-A183','1955-03-14 00:00:00','atmospheric rocket','unknown',576,'M-919-RM','','','','Test',1895,'',10 from dual union all
select 30120,'1972-W844','1972-06-28 23:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 30121,'1972-W845','1972-06-29 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 30122,'1972-W846','1972-06-29 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 30123,'1972-W847','1972-06-29 17:08:00','suborbital rocket','success',88,'L1-5481','Stokes Flow Para','','','Test',1821,'',69 from dual union all
select 30124,'1972-W848','1972-06-29 18:41:00','suborbital rocket','success',88,'L1-5482','Stokes Flow Para','','','Test',1821,'',74 from dual union all
select 30125,'1972-W849','1972-06-30 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 30126,'1972-W85','1972-01-18 16:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 30127,'1972-W850','1972-06-30 14:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 30128,'1972-W851','1972-06-30 15:08:00','suborbital rocket','success',88,'L1-5483','Stokes Flow Para','','','Test',1821,'',70 from dual union all
select 30129,'1972-W852','1972-06-30 19:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 3013,'1955-A184','1955-03-14 00:00:00','atmospheric rocket','success',836,'1962T','','','','Test',1895,'',10 from dual union all
select 30130,'1972-W853','1972-06-30 20:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 30131,'1972-W854','1972-06-30 20:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 30132,'1972-W855','1972-06-30 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 30133,'1972-W856','1972-07-03 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 30134,'1972-W857','1972-07-03 13:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 30135,'1972-W858','1972-07-03 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 30136,'1972-W859','1972-07-03 18:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 30137,'1972-W86','1972-01-18 17:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 30138,'1972-W860','1972-07-05 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 30139,'1972-W861','1972-07-05 14:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',65 from dual union all
select 3014,'1955-A185','1955-03-15 00:00:00','military missile','unknown',339,'1565R','','','','Test',1895,'',40 from dual union all
select 30140,'1972-W862','1972-07-05 14:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 30141,'1972-W863','1972-07-05 14:57:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 30142,'1972-W864','1972-07-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 30143,'1972-W865','1972-07-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 30144,'1972-W866','1972-07-05 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 30145,'1972-W867','1972-07-05 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 30146,'1972-W868','1972-07-06 00:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 30147,'1972-W869','1972-07-06 14:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 30148,'1972-W87','1972-01-18 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',617,'',58 from dual union all
select 30149,'1972-W870','1972-07-06 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 3015,'1955-A186','1955-03-15 00:00:00','atmospheric rocket','success',836,'1851T','','','','Test',1895,'',10 from dual union all
select 30150,'1972-W871','1972-07-07 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',52 from dual union all
select 30151,'1972-W872','1972-07-07 13:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 30152,'1972-W873','1972-07-07 14:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 30153,'1972-W874','1972-07-07 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 30154,'1972-W875','1972-07-07 15:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 30155,'1972-W876','1972-07-07 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 30156,'1972-W877','1972-07-07 16:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 30157,'1972-W878','1972-07-07 17:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual;
insert into LAUNCH
select 30158,'1972-W879','1972-07-07 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 30159,'1972-W88','1972-01-18 20:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 3016,'1955-A187','1955-03-16 00:00:00','atmospheric rocket','unknown',576,'172-A-EM','','','','Test',1895,'',10 from dual union all
select 30160,'1972-W880','1972-07-07 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 30161,'1972-W881','1972-07-08 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 30162,'1972-W882','1972-07-10 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 30163,'1972-W883','1972-07-10 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 30164,'1972-W884','1972-07-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 30165,'1972-W885','1972-07-10 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 30166,'1972-W886','1972-07-10 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 30167,'1972-W887','1972-07-10 19:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 30168,'1972-W888','1972-07-10 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 30169,'1972-W889','1972-07-11 02:20:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',62 from dual union all
select 3017,'1955-A188','1955-03-16 00:00:00','atmospheric rocket','success',836,'1248E','','','','Test',1895,'',10 from dual union all
select 30170,'1972-W89','1972-01-19 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 30171,'1972-W890','1972-07-11 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 30172,'1972-W891','1972-07-12 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 30173,'1972-W892','1972-07-12 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 30174,'1972-W893','1972-07-12 15:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 30175,'1972-W894','1972-07-12 15:22:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 30176,'1972-W895','1972-07-12 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 30177,'1972-W896','1972-07-12 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 30178,'1972-W897','1972-07-12 19:07:00','suborbital rocket','success',229,'RDT&E','MDSS FP-019','','','Test',428,'',71 from dual union all
select 30179,'1972-W898','1972-07-12 20:52:00','suborbital rocket','success',229,'','MDSS FP-020','','','Test',428,'',72 from dual union all
select 3018,'1955-A189','1955-03-16 00:00:00','atmospheric rocket','success',836,'1948T','','','','Test',1895,'',10 from dual union all
select 30180,'1972-W899','1972-07-12 22:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 30181,'1972-W90','1972-01-19 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 30182,'1972-W900','1972-07-12 23:35:00','suborbital rocket','success',229,'','MDSS FP-021','','','Test',428,'',67 from dual union all
select 30183,'1972-W901','1972-07-13 00:46:00','suborbital rocket','success',229,'','MDSS FP-025','','','Test',428,'',77 from dual union all
select 30184,'1972-W902','1972-07-13 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 30185,'1972-W903','1972-07-13 19:52:00','suborbital rocket','success',229,'','MDSS FP-029','','','Test',428,'',73 from dual union all
select 30186,'1972-W904','1972-07-13 21:34:00','suborbital rocket','success',229,'','MDSS FP-027','','','Test',428,'',76 from dual union all
select 30187,'1972-W905','1972-07-13 23:30:00','suborbital rocket','success',229,'','MDSS FP-026','','','Test',428,'',74 from dual union all
select 30188,'1972-W906','1972-07-14 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 30189,'1972-W907','1972-07-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 3019,'1955-A19','1955-01-17 00:00:00','atmospheric rocket','success',836,'3307T','','','','Test',1895,'',10 from dual union all
select 30190,'1972-W908','1972-07-14 14:24:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 30191,'1972-W909','1972-07-14 14:53:00','suborbital rocket','success',229,'','MDSS FP-024','','','Test',428,'',78 from dual union all
select 30192,'1972-W91','1972-01-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 30193,'1972-W910','1972-07-14 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 30194,'1972-W911','1972-07-14 16:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 30195,'1972-W912','1972-07-14 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 30196,'1972-W913','1972-07-17 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 30197,'1972-W914','1972-07-17 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 30198,'1972-W915','1972-07-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 30199,'1972-W916','1972-07-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 302,'1944-S274','1944-12-01 11:34:00','military missile','success',1470,'19848','Ma311','','','Test',661,'',90 from dual union all
select 3020,'1955-A190','1955-03-16 00:00:00','atmospheric rocket','success',836,'3030R','Nike 432R','','','Test',1895,'',10 from dual union all
select 30200,'1972-W917','1972-07-17 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',54 from dual union all
select 30201,'1972-W918','1972-07-17 17:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 30202,'1972-W919','1972-07-17 18:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 30203,'1972-W92','1972-01-19 15:45:00','suborbital rocket','success',631,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 30204,'1972-W920','1972-07-17 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 30205,'1972-W921','1972-07-17 21:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 30206,'1972-W922','1972-07-18 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 30207,'1972-W923','1972-07-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',57 from dual union all
select 30208,'1972-W924','1972-07-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 30209,'1972-W925','1972-07-19 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',55 from dual union all
select 3021,'1955-A191','1955-03-16 00:00:00','atmospheric rocket','success',836,'2996R','Nike 431R','','','Test',1895,'',10 from dual union all
select 30210,'1972-W926','1972-07-19 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 30211,'1972-W927','1972-07-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 30212,'1972-W928','1972-07-19 15:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 30213,'1972-W929','1972-07-19 16:42:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',59 from dual union all
select 30214,'1972-W93','1972-01-19 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',63 from dual union all
select 30215,'1972-W930','1972-07-19 16:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 30216,'1972-W931','1972-07-19 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 30217,'1972-W932','1972-07-19 17:09:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',61 from dual union all
select 30218,'1972-W933','1972-07-19 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',61 from dual union all
select 30219,'1972-W934','1972-07-19 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 3022,'1955-A192','1955-03-17 00:00:00','military missile','success',771,'GM-12483','','','','Test',222,'',12 from dual union all
select 30220,'1972-W935','1972-07-19 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 30221,'1972-W936','1972-07-21 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 30222,'1972-W937','1972-07-21 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',51 from dual union all
select 30223,'1972-W938','1972-07-21 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 30224,'1972-W939','1972-07-21 14:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 30225,'1972-W94','1972-01-19 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 30226,'1972-W940','1972-07-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 30227,'1972-W941','1972-07-21 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 30228,'1972-W942','1972-07-21 18:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 30229,'1972-W943','1972-07-21 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 3023,'1955-A193','1955-03-17 00:00:00','military missile','success',771,'GM-12451','','','','Test',222,'',12 from dual union all
select 30230,'1972-W944','1972-07-21 19:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 30231,'1972-W945','1972-07-21 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 30232,'1972-W946','1972-07-24 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 30233,'1972-W947','1972-07-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 30234,'1972-W948','1972-07-24 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 30235,'1972-W949','1972-07-24 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 30236,'1972-W95','1972-01-19 18:00:00','suborbital rocket','unknown',88,'','Ozone','','','Aeron',1895,'',58 from dual union all
select 30237,'1972-W950','1972-07-24 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 30238,'1972-W951','1972-07-24 19:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 30239,'1972-W952','1972-07-24 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 3024,'1955-A194','1955-03-17 00:00:00','atmospheric rocket','success',836,'1951T','','','','Test',1895,'',10 from dual union all
select 30240,'1972-W953','1972-07-25 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 30241,'1972-W954','1972-07-25 14:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 30242,'1972-W955','1972-07-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 30243,'1972-W956','1972-07-25 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 30244,'1972-W957','1972-07-26 03:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',57 from dual union all
select 30245,'1972-W958','1972-07-26 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 30246,'1972-W959','1972-07-26 13:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 30247,'1972-W96','1972-01-19 18:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',58 from dual;
insert into LAUNCH
select 30248,'1972-W960','1972-07-26 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 30249,'1972-W961','1972-07-26 14:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 3025,'1955-A195','1955-03-18 00:00:00','atmospheric rocket','success',836,'1646T','','','','Test',1895,'',10 from dual union all
select 30250,'1972-W962','1972-07-26 14:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 30251,'1972-W963','1972-07-26 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 30252,'1972-W964','1972-07-26 16:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 30253,'1972-W965','1972-07-26 16:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 30254,'1972-W966','1972-07-26 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 30255,'1972-W967','1972-07-26 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 30256,'1972-W968','1972-07-26 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 30257,'1972-W969','1972-07-27 00:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',54 from dual union all
select 30258,'1972-W97','1972-01-19 19:30:00','suborbital rocket','unknown',88,'','','','','Ionos',1895,'',69 from dual union all
select 30259,'1972-W970','1972-07-27 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 3026,'1955-A196','1955-03-18 00:00:00','atmospheric rocket','success',836,'1660T','','','','Test',1895,'',10 from dual union all
select 30260,'1972-W971','1972-07-28 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 30261,'1972-W972','1972-07-28 14:32:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',60 from dual union all
select 30262,'1972-W973','1972-07-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 30263,'1972-W974','1972-07-28 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 30264,'1972-W975','1972-07-28 17:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',53 from dual union all
select 30265,'1972-W976','1972-07-28 18:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',54 from dual union all
select 30266,'1972-W977','1972-07-28 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 30267,'1972-W978','1972-07-28 19:07:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 30268,'1972-W979','1972-07-28 22:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 30269,'1972-W98','1972-01-19 20:45:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',77 from dual union all
select 3027,'1955-A197','1955-03-18 00:00:00','atmospheric rocket','success',836,'1996T','','','','Test',1895,'',10 from dual union all
select 30270,'1972-W980','1972-07-28 23:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 30271,'1972-W981','1972-08-01 00:00:00','suborbital rocket','unknown',716,'','','','','Meteo',428,'',60 from dual union all
select 30272,'1972-W982','1972-07-31 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 30273,'1972-W983','1972-07-31 14:13:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 30274,'1972-W984','1972-07-31 18:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',56 from dual union all
select 30275,'1972-W985','1972-07-31 18:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 30276,'1972-W986','1972-07-31 23:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 30277,'1972-W987','1972-08-02 13:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 30278,'1972-W988','1972-08-02 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 30279,'1972-W989','1972-08-02 14:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 3028,'1955-A198','1955-03-18 00:00:00','atmospheric rocket','success',836,'1924T','','','','Test',1895,'',10 from dual union all
select 30280,'1972-W99','1972-01-19 22:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 30281,'1972-W990','1972-08-02 14:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 30282,'1972-W991','1972-08-02 14:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 30283,'1972-W992','1972-08-02 14:48:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 30284,'1972-W993','1972-08-02 14:49:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',1470,'',60 from dual union all
select 30285,'1972-W994','1972-08-02 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 30286,'1972-W995','1972-08-02 18:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 30287,'1972-W996','1972-08-02 20:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 30288,'1972-W997','1972-08-02 21:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 30289,'1972-W998','1972-08-03 00:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 3029,'1955-A199','1955-03-19 00:00:00','atmospheric rocket','success',836,'1981T','','','','Test',1895,'',10 from dual union all
select 30290,'1972-W999','1972-08-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 30291,'1972-Y01','1972-01-03 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',91 from dual union all
select 30292,'1972-Y02','1972-01-04 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',90 from dual union all
select 30293,'1972-Y03','1972-01-05 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 30294,'1972-Y04','1972-01-05 09:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 30295,'1972-Y05','1972-01-05 14:33:00','suborbital rocket','unknown',725,'TERLS-400','ISRO-8.54','TERLS-400','','Aeron',1467,'',87 from dual union all
select 30296,'1972-Y06','1972-01-05 14:50:00','suborbital rocket','success',725,'','Chaff','','','Aeron',961,'',83 from dual union all
select 30297,'1972-Y07','1972-01-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 30298,'1972-Y08','1972-01-09 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',90 from dual union all
select 30299,'1972-Y09','1972-01-10 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',91 from dual union all
select 303,'1944-S275','1944-12-05 12:06:00','military missile','success',1470,'19883','Ma346','','','Test',661,'',90 from dual union all
select 3030,'1955-A20','1955-01-18 00:00:00','military missile','unknown',339,'1464U','','','','Test',1895,'',40 from dual union all
select 30300,'1972-Y10','1972-01-11 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',90 from dual union all
select 30301,'1972-Y100','1972-03-15 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 30302,'1972-Y101','1972-03-15 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',93 from dual union all
select 30303,'1972-Y102','1972-03-15 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',90 from dual union all
select 30304,'1972-Y103','1972-03-15 14:34:00','suborbital rocket','unknown',725,'TERLS-425','ISRO-8.64','TERLS-425','','Aeron',1467,'',89 from dual union all
select 30305,'1972-Y104','1972-03-15 15:00:00','suborbital rocket','unknown',725,'','','Spurious?','','Aeron',1467,'',91 from dual union all
select 30306,'1972-Y105','1972-03-17 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',91 from dual union all
select 30307,'1972-Y106','1972-03-18 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',90 from dual union all
select 30308,'1972-Y107','1972-03-19 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',91 from dual union all
select 30309,'1972-Y108','1972-03-20 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',87 from dual union all
select 3031,'1955-A200','1955-03-20 00:00:00','atmospheric rocket','success',836,'1891T','','','','Test',1895,'',10 from dual union all
select 30310,'1972-Y109','1972-03-21 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',90 from dual union all
select 30311,'1972-Y11','1972-01-11 20:00:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',91 from dual union all
select 30312,'1972-Y110','1972-03-21 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 30313,'1972-Y111','1972-03-22 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',96 from dual union all
select 30314,'1972-Y112','1972-03-22 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',91 from dual union all
select 30315,'1972-Y113','1972-03-22 14:30:00','suborbital rocket','unknown',725,'TERLS-427','ISRO-8.65','TERLS-427','','Aeron',1467,'',92 from dual union all
select 30316,'1972-Y114','1972-03-22 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 30317,'1972-Y115','1972-03-22 17:15:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',92 from dual union all
select 30318,'1972-Y116','1972-03-22 19:57:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 30319,'1972-Y117','1972-03-23 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',90 from dual union all
select 3032,'1955-A201','1955-03-20 00:00:00','atmospheric rocket','success',836,'1988T','','','','Test',1895,'',10 from dual union all
select 30320,'1972-Y118','1972-03-24 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',90 from dual union all
select 30321,'1972-Y119','1972-03-25 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',90 from dual union all
select 30322,'1972-Y12','1972-01-12 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 30323,'1972-Y120','1972-03-27 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',83 from dual union all
select 30324,'1972-Y121','1972-03-29 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 30325,'1972-Y122','1972-03-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 30326,'1972-Y123','1972-03-29 14:30:00','suborbital rocket','unknown',725,'TERLS-428','ISRO-8.66','TERLS-428','','Aeron',1467,'',90 from dual union all
select 30327,'1972-Y124','1972-03-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 30328,'1972-Y125','1972-03-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',90 from dual union all
select 30329,'1972-Y126','1972-04-05 14:30:00','suborbital rocket','unknown',725,'TERLS-429','ISRO-8.67','TERLS-429','','Aeron',1467,'',91 from dual union all
select 3033,'1955-A202','1955-03-20 00:00:00','atmospheric rocket','success',836,'1825T','','','','Test',1895,'',10 from dual union all
select 30330,'1972-Y127','1972-04-12 14:30:00','suborbital rocket','unknown',725,'TERLS-431','ISRO-8.68','TERLS-431','','Aeron',1467,'',85 from dual union all
select 30331,'1972-Y128','1972-04-19 09:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',103 from dual union all
select 30332,'1972-Y129','1972-04-19 11:37:00','suborbital rocket','success',1188,'SK45H','','','','Ionospheric',1447,'',87 from dual union all
select 30333,'1972-Y13','1972-01-12 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',90 from dual union all
select 30334,'1972-Y130','1972-04-19 14:30:00','suborbital rocket','unknown',725,'TERLS-432','ISRO-8.69','TERLS-432','','Aeron',1467,'',87 from dual union all
select 30335,'1972-Y131','1972-04-19 16:44:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',92 from dual union all
select 30336,'1972-Y132','1972-04-19 18:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 30337,'1972-Y133','1972-04-26 14:30:00','suborbital rocket','unknown',725,'TERLS-433','ISRO-8.70','TERLS-433','','Aeron',1467,'',85 from dual;
insert into LAUNCH
select 30338,'1972-Y134','1972-05-03 10:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',102 from dual union all
select 30339,'1972-Y135','1972-05-03 14:30:00','suborbital rocket','unknown',725,'TERLS-434','ISRO-8.71','TERLS-434','','Aeron',1467,'',92 from dual union all
select 3034,'1955-A203','1955-03-21 00:00:00','atmospheric rocket','success',836,'1540T','','','','Test',1895,'',10 from dual union all
select 30340,'1972-Y136','1972-05-05 19:07:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',92 from dual union all
select 30341,'1972-Y137','1972-05-10 16:27:00','suborbital rocket','unknown',725,'TERLS-435','ISRO-8.72','TERLS-435','','Aeron',1467,'',91 from dual union all
select 30342,'1972-Y138','1972-05-17 11:34:00','suborbital rocket','success',1188,'SK44H','','','','Ionospheric',1447,'',93 from dual union all
select 30343,'1972-Y139','1972-05-17 15:05:00','suborbital rocket','unknown',725,'TERLS-436','ISRO-8.73','TERLS-436','','Aeron',1467,'',88 from dual union all
select 30344,'1972-Y14','1972-01-12 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 30345,'1972-Y140','1972-05-17 22:35:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 30346,'1972-Y141','1972-05-19 15:29:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 30347,'1972-Y142','1972-05-31 14:37:00','suborbital rocket','unknown',725,'TERLS-444','ISRO-8.74','TERLS-444','','Aeron',1467,'',90 from dual union all
select 30348,'1972-Y143','1972-05-31 21:12:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 30349,'1972-Y144','1972-06-07 14:30:00','suborbital rocket','unknown',725,'TERLS-445','ISRO-8.75','TERLS-445','','Aeron',1467,'',90 from dual union all
select 3035,'1955-A204','1955-03-22 00:00:00','atmospheric rocket','unknown',576,'914RM','','','','Test',1895,'',10 from dual union all
select 30350,'1972-Y145','1972-06-14 14:30:00','suborbital rocket','unknown',725,'TERLS-446','ISRO-8.76','TERLS-446','','Aeron',1467,'',89 from dual union all
select 30351,'1972-Y146','1972-06-14 21:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 30352,'1972-Y147','1972-06-21 14:30:00','suborbital rocket','unknown',725,'TERLS-447','ISRO-8.77','TERLS-447','','Aeron',1467,'',90 from dual union all
select 30353,'1972-Y148','1972-06-28 14:40:00','suborbital rocket','unknown',725,'TERLS-448','ISRO-8.78','TERLS-448','','Aeron',1467,'',88 from dual union all
select 30354,'1972-Y149','1972-06-30 20:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 30355,'1972-Y15','1972-01-12 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',90 from dual union all
select 30356,'1972-Y150','1972-07-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 30357,'1972-Y151','1972-07-01 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',89 from dual union all
select 30358,'1972-Y152','1972-07-03 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',88 from dual union all
select 30359,'1972-Y153','1972-07-05 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 3036,'1955-A205','1955-03-22 00:00:00','atmospheric rocket','unknown',576,'928RM','','','','Test',1895,'',10 from dual union all
select 30360,'1972-Y154','1972-07-05 14:00:00','suborbital rocket','success',725,'','','','','Aeron',961,'',87 from dual union all
select 30361,'1972-Y155','1972-07-05 14:32:00','suborbital rocket','unknown',725,'TERLS-449','ISRO-8.79','TERLS-449','','Aeron',1467,'',87 from dual union all
select 30362,'1972-Y156','1972-07-06 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',83 from dual union all
select 30363,'1972-Y157','1972-07-07 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',90 from dual union all
select 30364,'1972-Y158','1972-07-08 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',83 from dual union all
select 30365,'1972-Y159','1972-07-09 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',84 from dual union all
select 30366,'1972-Y16','1972-01-12 14:30:00','suborbital rocket','unknown',725,'TERLS-403','ISRO-8.55','TERLS-403','','Aeron',1467,'',89 from dual union all
select 30367,'1972-Y160','1972-07-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',91 from dual union all
select 30368,'1972-Y161','1972-07-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 30369,'1972-Y162','1972-07-12 14:30:00','suborbital rocket','unknown',725,'TERLS-451','ISRO-8.80','TERLS-451','','Aeron',1467,'',87 from dual union all
select 3037,'1955-A206','1955-03-22 00:00:00','atmospheric rocket','success',836,'1853T','','','','Test',1895,'',10 from dual union all
select 30370,'1972-Y163','1972-07-12 21:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',93 from dual union all
select 30371,'1972-Y164','1972-07-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',89 from dual union all
select 30372,'1972-Y165','1972-07-15 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',89 from dual union all
select 30373,'1972-Y166','1972-07-16 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',90 from dual union all
select 30374,'1972-Y167','1972-07-19 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 30375,'1972-Y168','1972-07-19 14:00:00','suborbital rocket','success',725,'','','','','Aeron',961,'',85 from dual union all
select 30376,'1972-Y169','1972-07-19 14:05:00','suborbital rocket','unknown',725,'TERLS-452','ISRO-8.81','TERLS-452','','Aeron',1467,'',89 from dual union all
select 30377,'1972-Y17','1972-01-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 30378,'1972-Y170','1972-07-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 30379,'1972-Y171','1972-07-26 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 3038,'1955-A207','1955-03-23 00:00:00','atmospheric rocket','success',365,'','F29 Ramjet','','','Test',1821,'',6 from dual union all
select 30380,'1972-Y172','1972-07-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 30381,'1972-Y173','1972-07-26 14:30:00','suborbital rocket','unknown',725,'TERLS-453','ISRO-8.82','TERLS-453','','Aeron',1467,'',89 from dual union all
select 30382,'1972-Y174','1972-07-26 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',99 from dual union all
select 30383,'1972-Y175','1972-07-26 16:32:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',95 from dual union all
select 30384,'1972-Y176','1972-07-26 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',88 from dual union all
select 30385,'1972-Y177','1972-07-28 22:06:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 30386,'1972-Y178','1972-07-31 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',88 from dual union all
select 30387,'1972-Y179','1972-08-01 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',89 from dual union all
select 30388,'1972-Y18','1972-01-12 15:00:00','suborbital rocket','unknown',725,'','','Spurious?','','Aeron',1467,'',89 from dual union all
select 30389,'1972-Y180','1972-08-02 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 3039,'1955-A208','1955-03-23 00:00:00','atmospheric rocket','unknown',576,'417EM','','','','Test',1895,'',10 from dual union all
select 30390,'1972-Y181','1972-08-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 30391,'1972-Y182','1972-08-02 14:44:00','suborbital rocket','unknown',725,'TERLS-454','ISRO-8.83','TERLS-454','','Aeron',1467,'',88 from dual union all
select 30392,'1972-Y183','1972-08-02 16:07:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 30393,'1972-Y184','1972-08-02 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 30394,'1972-Y185','1972-08-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 30395,'1972-Y186','1972-08-08 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',91 from dual union all
select 30396,'1972-Y187','1972-08-09 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 30397,'1972-Y188','1972-08-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 30398,'1972-Y189','1972-08-09 14:30:00','suborbital rocket','unknown',725,'TERLS-462','ISRO-8.84','TERLS-462','','Aeron',1467,'',87 from dual union all
select 30399,'1972-Y19','1972-01-12 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',83 from dual union all
select 304,'1944-S276','1944-12-05 16:16:00','military missile','success',1470,'19893','Ma340','','','Test',661,'',90 from dual union all
select 3040,'1955-A209','1955-03-23 00:00:00','atmospheric rocket','success',836,'3026R','Nike 433R','','','Test',1895,'',10 from dual union all
select 30400,'1972-Y190','1972-08-09 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',92 from dual union all
select 30401,'1972-Y191','1972-08-10 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 30402,'1972-Y192','1972-08-12 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',88 from dual union all
select 30403,'1972-Y193','1972-08-13 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',88 from dual union all
select 30404,'1972-Y194','1972-08-14 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1366,'SHIR',91 from dual union all
select 30405,'1972-Y195','1972-08-16 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 30406,'1972-Y196','1972-08-16 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 30407,'1972-Y197','1972-08-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 30408,'1972-Y198','1972-08-16 14:30:00','suborbital rocket','unknown',725,'TERLS-464','ISRO-8.85','TERLS-464','','Aeron',1467,'',88 from dual union all
select 30409,'1972-Y199','1972-08-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 3041,'1955-A21','1955-01-18 00:00:00','military missile','unknown',339,'1467R','','','','Test',1895,'',40 from dual union all
select 30410,'1972-Y20','1972-01-12 19:13:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 30411,'1972-Y200','1972-08-23 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 30412,'1972-Y201','1972-08-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 30413,'1972-Y202','1972-08-23 22:17:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',95 from dual union all
select 30414,'1972-Y203','1972-08-24 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.86','TERLS-472','','Aeron',1467,'',87 from dual union all
select 30415,'1972-Y204','1972-08-30 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 30416,'1972-Y205','1972-08-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 30417,'1972-Y206','1972-08-30 14:30:00','suborbital rocket','unknown',725,'TERLS-473','ISRO-8.87','TERLS-473','','Aeron',1467,'',90 from dual union all
select 30418,'1972-Y207','1972-09-05 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 30419,'1972-Y208','1972-09-06 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 3042,'1955-A210','1955-03-23 00:00:00','atmospheric rocket','success',836,'4213R','Nike 434R','','','Test',1895,'',10 from dual union all
select 30420,'1972-Y209','1972-09-06 14:00:00','suborbital rocket','success',725,'','','','','Aeron',961,'',83 from dual union all
select 30421,'1972-Y21','1972-01-13 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',90 from dual union all
select 30422,'1972-Y210','1972-09-06 14:30:00','suborbital rocket','unknown',725,'TERLS-474','ISRO-8.88','TERLS-474','','Aeron',1467,'',87 from dual union all
select 30423,'1972-Y211','1972-09-07 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 30424,'1972-Y212','1972-09-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 30425,'1972-Y213','1972-09-11 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 30426,'1972-Y214','1972-09-12 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 30427,'1972-Y215','1972-09-13 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual;
insert into LAUNCH
select 30428,'1972-Y216','1972-09-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 30429,'1972-Y217','1972-09-13 16:31:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 3043,'1955-A211','1955-03-23 00:00:00','atmospheric rocket','success',836,'2824','','','','Test',1895,'',10 from dual union all
select 30430,'1972-Y218','1972-09-13 16:57:00','suborbital rocket','unknown',725,'TERLS-475','ISRO-8.89','TERLS-475','','Aeron',1467,'',86 from dual union all
select 30431,'1972-Y219','1972-09-14 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 30432,'1972-Y22','1972-01-15 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',93 from dual union all
select 30433,'1972-Y220','1972-09-17 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 30434,'1972-Y221','1972-09-20 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 30435,'1972-Y222','1972-09-20 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 30436,'1972-Y223','1972-09-20 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 30437,'1972-Y224','1972-09-20 14:05:00','suborbital rocket','success',725,'','','','','Aeron',961,'',87 from dual union all
select 30438,'1972-Y225','1972-09-20 14:35:00','suborbital rocket','unknown',725,'TERLS-484','ISRO-8.90','TERLS-484','','Aeron',1467,'',86 from dual union all
select 30439,'1972-Y226','1972-09-20 15:33:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',82 from dual union all
select 3044,'1955-A212','1955-03-23 00:00:00','atmospheric rocket','success',836,'1706T','','','','Test',1895,'',10 from dual union all
select 30440,'1972-Y227','1972-09-20 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 30441,'1972-Y228','1972-09-26 22:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 30442,'1972-Y229','1972-09-27 09:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 30443,'1972-Y23','1972-01-15 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',92 from dual union all
select 30444,'1972-Y230','1972-09-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 30445,'1972-Y231','1972-09-27 14:30:00','suborbital rocket','unknown',725,'TERLS-485','ISRO-8.91','TERLS-485','','Aeron',1467,'',87 from dual union all
select 30446,'1972-Y232','1972-10-03 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 30447,'1972-Y233','1972-10-04 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 30448,'1972-Y234','1972-10-04 14:30:00','suborbital rocket','unknown',725,'TERLS-486','ISRO-8.92','TERLS-486','','Aeron',1467,'',89 from dual union all
select 30449,'1972-Y235','1972-10-04 13:53:00','suborbital rocket','success',725,'','','','','Aeron',961,'',87 from dual union all
select 3045,'1955-A213','1955-03-24 00:00:00','atmospheric rocket','unknown',576,'840PO','','','','Test',1895,'',10 from dual union all
select 30450,'1972-Y236','1972-10-04 16:14:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',80 from dual union all
select 30451,'1972-Y237','1972-10-10 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 30452,'1972-Y238','1972-10-11 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',96 from dual union all
select 30453,'1972-Y239','1972-10-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 30454,'1972-Y24','1972-01-15 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 30455,'1972-Y240','1972-10-11 14:30:00','suborbital rocket','unknown',725,'TERLS-487','ISRO-8.93','TERLS-487','','Aeron',1467,'',90 from dual union all
select 30456,'1972-Y241','1972-10-11 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',95 from dual union all
select 30457,'1972-Y242','1972-10-17 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 30458,'1972-Y243','1972-10-18 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',95 from dual union all
select 30459,'1972-Y244','1972-10-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 3046,'1955-A214','1955-03-24 00:00:00','atmospheric rocket','unknown',576,'837PO','','','','Test',1895,'',10 from dual union all
select 30460,'1972-Y245','1972-10-18 15:06:00','suborbital rocket','unknown',725,'TERLS-490','ISRO-8.94','TERLS-490','','Aeron',1467,'',91 from dual union all
select 30461,'1972-Y246','1972-10-18 16:03:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',93 from dual union all
select 30462,'1972-Y247','1972-10-18 19:03:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 30463,'1972-Y248','1972-10-20 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 30464,'1972-Y249','1972-10-24 21:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 30465,'1972-Y25','1972-01-15 12:14:00','suborbital rocket','success',1188,'SK43H','','','','Ionospheric',1447,'',87 from dual union all
select 30466,'1972-Y250','1972-10-25 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',95 from dual union all
select 30467,'1972-Y251','1972-10-25 14:30:00','suborbital rocket','unknown',725,'TERLS-497','ISRO-8.95','TERLS-497','','Aeron',1467,'',86 from dual union all
select 30468,'1972-Y252','1972-10-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 30469,'1972-Y253','1972-10-25 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 3047,'1955-A215','1955-03-24 00:00:00','atmospheric rocket','success',836,'1717T','','','','Test',1895,'',10 from dual union all
select 30470,'1972-Y254','1972-11-01 14:44:00','suborbital rocket','unknown',725,'TERLS-498','ISRO-8.96','TERLS-498','','Aeron',1467,'',86 from dual union all
select 30471,'1972-Y255','1972-11-01 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 30472,'1972-Y256','1972-11-01 23:15:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 30473,'1972-Y257','1972-11-09 04:10:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',88 from dual union all
select 30474,'1972-Y258','1972-11-09 09:16:00','suborbital rocket','success',725,'','Atomic O','','','Aeron',1010,'',91 from dual union all
select 30475,'1972-Y259','1972-11-09 14:30:00','suborbital rocket','unknown',725,'TERLS-499','ISRO-8.97','TERLS-499','','Aeron',1467,'',88 from dual union all
select 30476,'1972-Y26','1972-01-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',93 from dual union all
select 30477,'1972-Y260','1972-11-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 30478,'1972-Y261','1972-11-15 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 30479,'1972-Y262','1972-11-15 09:26:00','suborbital rocket','success',725,'','Atomic O','','','Aeron',1010,'',91 from dual union all
select 3048,'1955-A216','1955-03-25 00:00:00','military missile','unknown',339,'1561E','','','','Test',1895,'',40 from dual union all
select 30480,'1972-Y263','1972-11-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 30481,'1972-Y264','1972-11-15 14:30:00','suborbital rocket','unknown',725,'TERLS-500','ISRO-8.98','TERLS-500','','Aeron',1467,'',91 from dual union all
select 30482,'1972-Y265','1972-11-15 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',97 from dual union all
select 30483,'1972-Y266','1972-11-17 20:01:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 30484,'1972-Y267','1972-11-20 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 30485,'1972-Y268','1972-11-22 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 30486,'1972-Y269','1972-11-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',98 from dual union all
select 30487,'1972-Y27','1972-01-17 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',90 from dual union all
select 30488,'1972-Y270','1972-11-22 14:30:00','suborbital rocket','unknown',725,'TERLS-501','ISRO-8.99','TERLS-501','','Aeron',1467,'',86 from dual union all
select 30489,'1972-Y271','1972-11-22 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',91 from dual union all
select 3049,'1955-A217','1955-03-25 00:00:00','atmospheric rocket','unknown',576,'M-957-PO','','','','Test',1895,'',10 from dual union all
select 30490,'1972-Y272','1972-11-23 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 30491,'1972-Y273','1972-11-27 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 30492,'1972-Y274','1972-11-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 30493,'1972-Y275','1972-11-29 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 30494,'1972-Y276','1972-11-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 30495,'1972-Y277','1972-11-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 30496,'1972-Y278','1972-11-29 14:30:00','suborbital rocket','unknown',725,'TERLS-502','ISRO-8.100','TERLS-502','','Aeron',1467,'',88 from dual union all
select 30497,'1972-Y279','1972-11-29 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 30498,'1972-Y28','1972-01-18 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',90 from dual union all
select 30499,'1972-Y280','1972-11-29 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 305,'1944-S277','1944-12-05 18:59:00','military missile','success',1470,'19905','Ma339','','','Test',661,'',90 from dual union all
select 3050,'1955-A218','1955-03-25 00:00:00','atmospheric rocket','unknown',576,'M-959-PO','','','','Test',1895,'',10 from dual union all
select 30500,'1972-Y281','1972-12-01 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 30501,'1972-Y282','1972-12-01 20:27:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',87 from dual union all
select 30502,'1972-Y283','1972-12-05 18:14:00','suborbital rocket','success',1281,'NASA 15.95UI','','','','Ionos',1821,'',84 from dual union all
select 30503,'1972-Y284','1972-12-06 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 30504,'1972-Y285','1972-12-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 30505,'1972-Y286','1972-12-06 14:33:00','suborbital rocket','unknown',725,'TERLS-503','ISRO-8.101','TERLS-503','','Aeron',1467,'',83 from dual union all
select 30506,'1972-Y287','1972-12-06 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 30507,'1972-Y288','1972-12-13 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 30508,'1972-Y289','1972-12-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 30509,'1972-Y29','1972-01-19 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',91 from dual union all
select 3051,'1955-A219','1955-03-25 00:00:00','atmospheric rocket','unknown',576,'M-958-PO','','','','Test',1895,'',10 from dual union all
select 30510,'1972-Y290','1972-12-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 30511,'1972-Y291','1972-12-13 14:30:00','suborbital rocket','unknown',725,'TERLS-504','ISRO-8.102','TERLS-504','','Aeron',1467,'',89 from dual union all
select 30512,'1972-Y292','1972-12-20 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 30513,'1972-Y293','1972-12-20 14:30:00','suborbital rocket','success',725,'','','','','Aeron',961,'',82 from dual union all
select 30514,'1972-Y294','1972-12-20 14:30:00','suborbital rocket','unknown',725,'TERLS-509','ISRO-8.103','TERLS-509','','Aeron',1467,'',88 from dual union all
select 30515,'1972-Y295','1972-12-20 16:09:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',91 from dual union all
select 30516,'1972-Y296','1972-12-22 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 30517,'1972-Y297','1972-12-25 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',92 from dual;
insert into LAUNCH
select 30518,'1972-Y298','1972-12-27 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 30519,'1972-Y299','1972-12-27 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',92 from dual union all
select 3052,'1955-A22','1955-01-18 00:00:00','military missile','success',771,'GM-52-1849','','','','Test',222,'',12 from dual union all
select 30520,'1972-Y30','1972-01-19 14:31:00','suborbital rocket','unknown',725,'TERLS-407','ISRO-8.56','TERLS-407','','Aeron',1467,'',91 from dual union all
select 30521,'1972-Y300','1972-12-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 30522,'1972-Y301','1972-12-27 14:30:00','suborbital rocket','unknown',725,'TERLS-510','ISRO-8.104','TERLS-510','','Aeron',1467,'',87 from dual union all
select 30523,'1972-Y302','1972-12-27 18:22:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 30524,'1972-Y303','1972-12-27 22:19:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 30525,'1972-Y304','1972-12-28 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',85 from dual union all
select 30526,'1972-Y305','1972-12-29 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',83 from dual union all
select 30527,'1972-Y306','1972-12-30 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',87 from dual union all
select 30528,'1972-Y307','1972-03-01 03:30:00','suborbital rocket','success',725,'','','','','Aeron',961,'',88 from dual union all
select 30529,'1972-Y308','1972-05-03 14:00:00','suborbital rocket','success',725,'','','','','Aeron',961,'',88 from dual union all
select 3053,'1955-A220','1955-03-25 00:00:00','atmospheric rocket','unknown',576,'M-961-PO','','','','Test',1895,'',10 from dual union all
select 30530,'1972-Y309','1972-05-06 14:10:00','suborbital rocket','success',725,'','','','','Aeron',961,'',88 from dual union all
select 30531,'1972-Y31','1972-01-19 14:40:00','suborbital rocket','success',725,'','','','','Aeron',961,'',86 from dual union all
select 30532,'1972-Y310','1972-05-17 17:50:00','suborbital rocket','success',725,'','','','','Aeron',961,'',88 from dual union all
select 30533,'1972-Y311','1972-06-21 14:00:00','suborbital rocket','success',725,'','','','','Aeron',961,'',88 from dual union all
select 30534,'1972-Y312','1972-06-21 14:05:00','suborbital rocket','success',725,'','','','','Aeron',961,'',88 from dual union all
select 30535,'1972-Y32','1972-01-20 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',94 from dual union all
select 30536,'1972-Y33','1972-01-21 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',91 from dual union all
select 30537,'1972-Y34','1972-01-22 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 30538,'1972-Y35','1972-01-22 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',877,'KORO',90 from dual union all
select 30539,'1972-Y36','1972-01-25 20:04:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',92 from dual union all
select 3054,'1955-A221','1955-03-25 00:00:00','atmospheric rocket','success',836,'1874T','','','','Test',1895,'',10 from dual union all
select 30540,'1972-Y37','1972-01-26 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',95 from dual union all
select 30541,'1972-Y38','1972-01-26 14:35:00','suborbital rocket','success',725,'','','','','Aeron',961,'',88 from dual union all
select 30542,'1972-Y39','1972-01-26 15:00:00','suborbital rocket','unknown',725,'TERLS-408','ISRO-8.57','TERLS-408','','Aeron',1467,'',89 from dual union all
select 30543,'1972-Y40','1972-01-26 16:55:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',89 from dual union all
select 30544,'1972-Y41','1972-01-26 17:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',90 from dual union all
select 30545,'1972-Y42','1972-01-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 30546,'1972-Y43','1972-01-30 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',90 from dual union all
select 30547,'1972-Y44','1972-01-31 18:07:00','suborbital rocket','success',1281,'NASA 15.93UI','','','','Ionospheric',1821,'',82 from dual union all
select 30548,'1972-Y45','1972-02-02 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 30549,'1972-Y46','1972-02-02 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 3055,'1955-A222','1955-03-25 00:00:00','atmospheric rocket','success',836,'1560T','','','','Test',1895,'',10 from dual union all
select 30550,'1972-Y47','1972-02-02 14:30:00','suborbital rocket','unknown',725,'TERLS-411','ISRO-8.58','TERLS-411','','Aeron',1467,'',91 from dual union all
select 30551,'1972-Y48','1972-02-02 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 30552,'1972-Y49','1972-02-02 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',91 from dual union all
select 30553,'1972-Y50','1972-02-05 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 30554,'1972-Y51','1972-02-06 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',88 from dual union all
select 30555,'1972-Y52','1972-02-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',89 from dual union all
select 30556,'1972-Y53','1972-02-08 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',89 from dual union all
select 30557,'1972-Y54','1972-02-09 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 30558,'1972-Y55','1972-02-09 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',98 from dual union all
select 30559,'1972-Y56','1972-02-09 14:30:00','suborbital rocket','unknown',725,'TERLS-413','ISRO-8.59','TERLS-413','','Aeron',1467,'',85 from dual union all
select 3056,'1955-A223','1955-03-25 00:00:00','atmospheric rocket','success',836,'1961T','','','','Test',1895,'',10 from dual union all
select 30560,'1972-Y57','1972-02-10 19:01:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 30561,'1972-Y58','1972-02-11 10:24:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',176,'',92 from dual union all
select 30562,'1972-Y59','1972-02-11 19:14:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',85 from dual union all
select 30563,'1972-Y60','1972-02-12 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 30564,'1972-Y61','1972-02-14 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',87 from dual union all
select 30565,'1972-Y62','1972-02-15 14:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',85 from dual union all
select 30566,'1972-Y63','1972-02-15 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',85 from dual union all
select 30567,'1972-Y64','1972-02-15 18:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',84 from dual union all
select 30568,'1972-Y65','1972-02-16 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 30569,'1972-Y66','1972-02-16 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',95 from dual union all
select 3057,'1955-A224','1955-03-26 00:00:00','atmospheric rocket','success',836,'1954T','','','','Test',1895,'',10 from dual union all
select 30570,'1972-Y67','1972-02-16 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',89 from dual union all
select 30571,'1972-Y68','1972-02-16 15:35:00','suborbital rocket','success',725,'','','','','Aeron',961,'',93 from dual union all
select 30572,'1972-Y69','1972-02-16 18:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',86 from dual union all
select 30573,'1972-Y70','1972-02-17 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',86 from dual union all
select 30574,'1972-Y71','1972-02-17 14:30:00','suborbital rocket','unknown',725,'TERLS-417','ISRO-8.60','TERLS-417','','Aeron',1467,'',90 from dual union all
select 30575,'1972-Y72','1972-02-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',90 from dual union all
select 30576,'1972-Y73','1972-02-17 16:34:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',84 from dual union all
select 30577,'1972-Y74','1972-02-18 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 30578,'1972-Y75','1972-02-18 14:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',86 from dual union all
select 30579,'1972-Y76','1972-02-18 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',84 from dual union all
select 3058,'1955-A225','1955-03-26 00:00:00','atmospheric rocket','success',836,'1831T','','','','Test',1895,'',10 from dual union all
select 30580,'1972-Y77','1972-02-19 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',91 from dual union all
select 30581,'1972-Y78','1972-02-20 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 30582,'1972-Y79','1972-02-21 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',89 from dual union all
select 30583,'1972-Y80','1972-02-22 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1365,'SHIR',89 from dual union all
select 30584,'1972-Y81','1972-02-23 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 30585,'1972-Y82','1972-02-23 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 30586,'1972-Y83','1972-02-23 14:30:00','suborbital rocket','unknown',725,'TERLS-418','ISRO-8.61','TERLS-418','','Aeron',1467,'',88 from dual union all
select 30587,'1972-Y84','1972-02-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 30588,'1972-Y85','1972-02-23 15:00:00','suborbital rocket','unknown',725,'','','Spurious?','','Aeron',1467,'',87 from dual union all
select 30589,'1972-Y86','1972-02-23 15:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',87 from dual union all
select 3059,'1955-A226','1955-03-27 00:00:00','atmospheric rocket','success',836,'1878T','','','','Test',1895,'',10 from dual union all
select 30590,'1972-Y87','1972-02-23 23:32:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 30591,'1972-Y88','1972-02-26 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 30592,'1972-Y89','1972-03-01 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 30593,'1972-Y90','1972-03-01 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 30594,'1972-Y91','1972-03-01 14:30:00','suborbital rocket','unknown',725,'TERLS-419','ISRO-8.62','TERLS-419','','Aeron',1467,'',85 from dual union all
select 30595,'1972-Y92','1972-03-01 15:00:00','suborbital rocket','unknown',725,'','','Spurious?','','Aeron',1467,'',90 from dual union all
select 30596,'1972-Y93','1972-03-08 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 30597,'1972-Y94','1972-03-08 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 30598,'1972-Y95','1972-03-08 15:00:00','suborbital rocket','unknown',725,'','','Spurious?','','Aeron',1467,'',90 from dual union all
select 30599,'1972-Y96','1972-03-08 16:30:00','suborbital rocket','unknown',725,'TERLS-421','ISRO-8.63','TERLS-421','','Aeron',1467,'',91 from dual union all
select 306,'1944-S278','1944-12-06 13:53:00','military missile','success',1470,'19913','Ma338','','','Test',661,'',90 from dual union all
select 3060,'1955-A227','1955-03-27 00:00:00','atmospheric rocket','success',836,'1994T','','','','Test',1895,'',10 from dual union all
select 30600,'1972-Y97','1972-03-08 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',84 from dual union all
select 30601,'1972-Y98','1972-03-08 18:31:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',95 from dual union all
select 30602,'1972-Y99','1972-03-15 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 30603,'1973-001','1973-01-08 06:55:38','deep space','success',953,'259-01','Luna-21','E-8 No. 204','','Sat',587,'',null from dual union all
select 30604,'1973-002','1973-01-11 10:00:00','orbital','success',1508,'78033  -270','Kosmos-543','Zenit-4M','','Sat',562,'',null from dual union all
select 30605,'1973-003','1973-01-20 03:36:05','orbital','success',680,'Yu149-44','Kosmos-544','Tselina-OM','','Sat',456,'',null from dual union all
select 30606,'1973-004','1973-01-24 11:44:50','orbital','success',678,'64653-169','Kosmos-545','DS-P1-Yu No. 62','','Sat',460,'',null from dual union all
select 30607,'1973-005','1973-01-26 11:44:45','orbital','success',680,'Yu149-37','Kosmos-546','Tsiklon GVM','','Sat',622,'',null from dual;
insert into LAUNCH
select 30608,'1973-006','1973-02-01 08:30:00','orbital','success',1508,'78033  -159','Kosmos-547','Zenit-2M','','Sat',562,'',null from dual union all
select 30609,'1973-007','1973-02-03 05:48:59','orbital','success',809,'7801421-514','Molniya-1','Molniya-1 11F67 No. 31','','Sat',480,'',null from dual union all
select 3061,'1955-A228','1955-03-28 00:00:00','atmospheric rocket','success',365,'','E39 Model','','','Test',1821,'',10 from dual union all
select 30610,'1973-008','1973-02-08 13:15:00','orbital','success',1508,'7801422-237','Kosmos-548','Zenit-4M','','Sat',475,'',null from dual union all
select 30611,'1973-009','1973-02-15 01:11:53','orbital','success',807,'M 15000-118','Prognoz-3','SO-M No. 502','','Sat',562,'',null from dual union all
select 30612,'1973-010','1973-02-28 04:37:00','orbital','success',680,'Yu149-45','Kosmos-549','Tselina-OM','','Sat',456,'',null from dual union all
select 30613,'1973-011','1973-03-01 12:40:02','orbital','success',1508,'78033-  730','Kosmos-550','Zenit-4MK','','Sat',473,'',null from dual union all
select 30614,'1973-012','1973-03-06 09:20:00','orbital','success',1508,'7702423-249','Kosmos-551','Zenit-4M','','Sat',562,'',null from dual union all
select 30615,'1973-013','1973-03-06 09:30:00','orbital','success',178,'5202A','AFP-720','RHYOLITE 2','','Sat',228,'',null from dual union all
select 30616,'1973-014','1973-03-09 21:00:00','orbital','success',1426,'23D-6','HEXAGON SV5','HEXAGON SV-5','','Sat',1777,'',null from dual union all
select 30617,'1973-015','1973-03-20 11:20:02','orbital','success',1510,'7702421-335','Meteor','Meteor-M  No. 29','','Sat',473,'',null from dual union all
select 30618,'1973-016','1973-03-22 10:00:00','orbital','success',1508,'78033-  158','Kosmos-552','Zenit-2M','','Sat',475,'',null from dual union all
select 30619,'1973-017','1973-04-03 09:00:00','orbital','success',949,'283-01','Salyut-2','Almaz OPS 1','','Sat',587,'',null from dual union all
select 3062,'1955-A229','1955-03-28 00:00:00','atmospheric rocket','unknown',576,'917RS','','','','Test',1895,'',10 from dual union all
select 30620,'1973-018','1973-04-05 11:11:00','orbital','success',809,'M 15000-115','Molniya-2','Molniya-2','','Sat',473,'',null from dual union all
select 30621,'1973-019','1973-04-06 02:11:00','deep space','success',180,'AC-30','Pioneer 11','Pioneer G','','Sat',269,'',null from dual union all
select 30622,'1973-020','1973-04-12 11:49:55','orbital','success',678,'29547-142','Kosmos-553','DS-P1-Yu No. 55','','Sat',460,'',null from dual union all
select 30623,'1973-021','1973-04-19 08:59:59','orbital','success',1508,'76043  -733','Kosmos-554','Zenit-4MK','','Sat',475,'',null from dual union all
select 30624,'1973-022','1973-04-19 10:19:58','orbital','success',678,'','IK-Kopernik-500','DS-U2-IK Kopernik-500','','Sat',627,'',null from dual union all
select 30625,'1973-023','1973-04-20 23:47:03','orbital','success',380,'583/D94','Anik A2','Anik A2','','Sat',234,'',null from dual union all
select 30626,'1973-024','1973-04-25 10:45:00','orbital','success',1508,'77019  -173','Kosmos-555','Zenit-2M','','Sat',475,'',null from dual union all
select 30627,'1973-025','1973-05-05 07:00:00','orbital','success',1508,'77019  -751','Kosmos-556','Zenit-4MK','','Sat',473,'',null from dual union all
select 30628,'1973-026','1973-05-11 00:20:00','orbital','success',949,'284-01','Kosmos-557','DOS 3','','Sat',587,'',null from dual union all
select 30629,'1973-027','1973-05-14 17:30:00','orbital','success',1122,'SA-513','Skylab 1','Skylab I OWS','','Sat',921,'LUT2',null from dual union all
select 3063,'1955-A23','1955-01-18 00:00:00','military missile','success',771,'GM-52-1865','','','','Test',222,'',12 from dual union all
select 30630,'1973-028','1973-05-16 16:40:00','orbital','success',1403,'24B-7 (3B-41)','KH-8A','GAMBIT-3 Mission 4338','','Sat',1778,'',null from dual union all
select 30631,'1973-029','1973-05-17 13:19:58','orbital','success',678,'45734-143','Kosmos-558','DS-P1-Yu No. 65','','Sat',460,'',null from dual union all
select 30632,'1973-030','1973-05-18 11:00:02','orbital','success',1238,'76043  -731','Kosmos-559','Zenit-4MK','','Sat',474,'',null from dual union all
select 30633,'1973-031','1973-05-23 10:30:00','orbital','success',1508,'7702423-248','Kosmos-560','Zenit-4M','','Sat',475,'',null from dual union all
select 30634,'1973-032','1973-05-25 13:00:00','orbital','success',1121,'SA-206','Skylab 2','Apollo CM-116','','Sat',922,'LUT1',null from dual union all
select 30635,'1973-033','1973-05-25 13:30:01','orbital','success',1508,'78033  -160','Kosmos-561','Zenit-2M','','Sat',475,'',null from dual union all
select 30636,'1973-034','1973-05-29 10:16:03','orbital','success',1510,'7702421-335 (336)','Meteor','Meteor-M  No. 27','','Sat',473,'',null from dual union all
select 30637,'1973-035','1973-06-05 11:29:47','orbital','success',678,'64634-149','Kosmos-562','DS-P1-Yu No. 66','','Sat',460,'',null from dual union all
select 30638,'1973-036','1973-06-06 11:30:00','orbital','success',1508,'7801422-241','Kosmos-563','Zenit-4M','','Sat',475,'',null from dual union all
select 30639,'1973-037','1973-06-08 15:50:00','orbital','success',680,'53753-210','Kosmos-564/571','Strela-1M','','Sat',456,'',null from dual union all
select 3064,'1955-A230','1955-03-28 00:00:00','atmospheric rocket','success',836,'1653T','','','','Test',1895,'',10 from dual union all
select 30640,'1973-038','1973-06-10 10:10:00','orbital','success',1508,'7702423-251','Kosmos-572','Zenit-4M','','Sat',480,'',null from dual union all
select 30641,'1973-039','1973-06-10 14:13:00','deep space','success',379,'581/D95','Explorer 49','RAE B','','Sat',234,'',null from dual union all
select 30642,'1973-040','1973-06-12 07:14:05','orbital','success',1425,'23C-6  (3C-24)','DSP 4','DSP 2','','Sat UPF',274,'',null from dual union all
select 30643,'1973-041','1973-06-15 06:00:00','orbital','success',1223,'S 15000-027','Kosmos-573','Soyuz 7K-T No. 36','','Sat',480,'',null from dual union all
select 30644,'1973-042','1973-06-20 06:16:27','orbital','success',680,'53724-116','Kosmos-574','Zaliv','','Sat',456,'',null from dual union all
select 30645,'1973-043','1973-06-21 13:29:57','orbital','success',1508,'77019  -176','Kosmos-575','Zenit-2M','','Sat',475,'',null from dual union all
select 30646,'1973-044','1973-06-27 11:50:01','orbital','success',1225,'Kh15000-004','Kosmos-576','Zenit-4MT','','Sat',473,'',null from dual union all
select 30647,'1973-045','1973-07-11 09:58:00','orbital','success',809,'76043  -533','Molniya-2','Molniya-2','','Sat',473,'',null from dual union all
select 30648,'1973-046','1973-07-13 20:22:00','orbital','success',1426,'23D-7','HEXAGON SV6','HEXAGON SV-6','','Sat',1777,'',null from dual union all
select 30649,'1973-047','1973-07-21 19:30:59','deep space','success',953,'261-01','Mars-4','3MS No. 52S','','Sat',587,'',null from dual union all
select 3065,'1955-A231','1955-03-29 00:00:00','military missile','unknown',339,'1568R','Corporal R','','','Test',1895,'',40 from dual union all
select 30650,'1973-048','1973-07-25 11:30:00','orbital','success',1508,'7702423-254','Kosmos-577','Zenit-4M','','Sat',475,'',null from dual union all
select 30651,'1973-049','1973-07-25 18:55:48','deep space','success',953,'262-01','Mars-5','3MS No. 53S','','Sat',588,'',null from dual union all
select 30652,'1973-050','1973-07-28 11:10:50','orbital','success',1121,'SA-207','Skylab 3','Apollo CM-117','','Sat',922,'LUT1',null from dual union all
select 30653,'1973-051','1973-08-01 14:00:01','orbital','success',1508,'77019  -174','Kosmos-578','Zenit-2M','','Sat',475,'',null from dual union all
select 30654,'1973-052','1973-08-05 17:45:48','deep space','success',953,'281-01','Mars-6','3MP No. 50P','','Sat',587,'',null from dual union all
select 30655,'1973-053','1973-08-09 17:00:17','deep space','success',953,'281-02','Mars-7','3MP No. 51P','','Sat',588,'',null from dual union all
select 30656,'1973-054','1973-08-17 04:49:12','orbital','success',1365,'291/BIIA-21','DMSP 7529','DMSP Block 5B F-4','','Sat',1720,'',null from dual union all
select 30657,'1973-055','1973-08-21 12:30:01','orbital','success',1508,'78033  -265','Kosmos-579','Zenit-4M','','Sat',473,'',null from dual union all
select 30658,'1973-056','1973-08-21 16:07:00','orbital','success',1404,'33B-3 (3B-38)','OPS 7724','Jumpseat 3','','Sat',1778,'',null from dual union all
select 30659,'1973-057','1973-08-22 11:24:55','orbital','success',678,'45734-141','Kosmos-580','DS-P1-Yu No. 59','','Sat',460,'',null from dual union all
select 3066,'1955-A232','1955-03-29 00:00:00','atmospheric rocket','success',836,'4233','','','','Test',1895,'',10 from dual union all
select 30660,'1973-058','1973-08-23 22:57:02','orbital','success',180,'AC-31','Intelsat 4 F-7','INTELSAT IV F7','','Sat',268,'',null from dual union all
select 30661,'1973-059','1973-08-24 10:59:55','orbital','success',1508,'76043  -273','Kosmos-581','Zenit-4M','','Sat',480,'',null from dual union all
select 30662,'1973-060','1973-08-28 10:08:29','orbital','success',680,'65033-201','Kosmos-582','Tselina-OM','','Sat',457,'',null from dual union all
select 30663,'1973-061','1973-08-30 00:07:59','orbital','success',809,'7705711-499','Molniya-1','Molniya-1','','Sat',473,'',null from dual union all
select 30664,'1973-062','1973-08-30 10:30:02','orbital','success',1508,'76043  -162','Kosmos-583','Zenit-2M','','Sat',480,'',null from dual union all
select 30665,'1973-063','1973-09-06 10:40:01','orbital','success',1508,'77019  -285','Kosmos-584','Zenit-4M','','Sat',473,'',null from dual union all
select 30666,'1973-064','1973-09-08 01:50:00','orbital','success',680,'65027-114','Kosmos-585','Geoid','','Sat',457,'',null from dual union all
select 30667,'1973-065','1973-09-14 00:31:32','orbital','success',680,'53714-106','Kosmos-586','Zaliv','','Sat',457,'',null from dual union all
select 30668,'1973-066','1973-09-21 13:05:01','orbital','success',1238,'76043  -732','Kosmos-587','Zenit-4MK','','Sat',474,'',null from dual union all
select 30669,'1973-067','1973-09-27 12:18:16','orbital','success',1223,'S 15000-026','Soyuz-12','Soyuz 7K-T No. 37','','Sat',480,'',null from dual union all
select 3067,'1955-A233','1955-03-30 00:00:00','atmospheric rocket','unknown',576,'94A-EM','','','','Test',1895,'',10 from dual union all
select 30670,'1973-068','1973-09-27 17:15:00','orbital','success',1403,'24B-8 (3B-42)','KH-8A','GAMBIT-3 Mission 4340','','Sat',1778,'',null from dual union all
select 30671,'1973-069','1973-10-02 21:46:00','orbital','success',680,'53724-115','Kosmos-588/595','Strela-1M','','Sat',457,'',null from dual union all
select 30672,'1973-070','1973-10-03 13:00:00','orbital','success',1508,'7702421-145','Kosmos-596','Zenit-2M','','Sat',473,'',null from dual union all
select 30673,'1973-071','1973-10-06 12:30:00','orbital','success',1508,'76043  -736','Kosmos-597','Zenit-4MK','','Sat',473,'',null from dual union all
select 30674,'1973-072','1973-10-10 10:45:00','orbital','success',1508,'77019  -286','Kosmos-598','Zenit-4M','','Sat',473,'',null from dual union all
select 30675,'1973-073','1973-10-15 08:45:00','orbital','success',1508,'77019  -175','Kosmos-599','Zenit-2M','','Sat',480,'',null from dual union all
select 30676,'1973-074','1973-10-16 12:00:04','orbital','success',1508,'77019  -288','Kosmos-600','Zenit-4M','','Sat',475,'',null from dual union all
select 30677,'1973-075','1973-10-16 14:00:01','orbital','success',678,'29519-181','Kosmos-601','DS-P1-Yu No. 60','','Sat',460,'',null from dual union all
select 30678,'1973-076','1973-10-19 10:26:01','orbital','success',809,'77019  -549','Molniya-2','Molniya-2','','Sat',473,'',null from dual union all
select 30679,'1973-077','1973-10-20 10:14:58','orbital','success',1508,'76019  -752','Kosmos-602','Zenit-4MK','','Sat',475,'',null from dual union all
select 3068,'1955-A234','1955-03-30 00:00:00','atmospheric rocket','success',836,'1884T','','','','Test',1895,'',10 from dual union all
select 30680,'1973-078','1973-10-26 02:26:03','orbital','success',376,'582/D97','Explorer 50','IMP J','','Sat',234,'',null from dual union all
select 30681,'1973-079','1973-10-27 11:09:59','orbital','success',1508,'77019  -287','Kosmos-603','Zenit-4M','','Sat',473,'',null from dual union all
select 30682,'1973-080','1973-10-29 14:00:14','orbital','success',1510,'M 15000-020','Kosmos-604','Tselina-D','','Sat',475,'',null from dual union all
select 30683,'1973-081','1973-10-30 00:37:02','orbital','success',1126,'S178C','NNS O-20 (N-15)','NNS O-20','','Sat',1779,'',null from dual union all
select 30684,'1973-082','1973-10-30 19:00:00','orbital','success',680,'53749-158','Interkosmos-10','DS-U2-IK No. 3','','Sat',457,'',null from dual union all
select 30685,'1973-083','1973-10-31 18:24:59','orbital','success',1238,'A 15000-004','Kosmos-605','Bion No. 1','','Sat',474,'',null from dual union all
select 30686,'1973-084','1973-11-02 13:01:56','orbital','success',809,'M 15000-116','Kosmos-606','Oko','','Sat',473,'',null from dual union all
select 30687,'1973-085','1973-11-03 05:45:00','deep space','success',180,'AC-34','Mariner 10','Mariner 73J','','Sat',269,'',null from dual union all
select 30688,'1973-086','1973-11-06 17:02:00','orbital','success',373,'576/D98','NOAA 3','ITOS F','','Sat',1724,'',null from dual union all
select 30689,'1973-087','1973-11-10 12:38:27','orbital','success',1508,'76029  -758','Kosmos-607','Zenit-4MK','','Sat',475,'',null from dual union all
select 3069,'1955-A235','1955-03-31 00:00:00','atmospheric rocket','success',836,'1778T','','','','Test',1895,'',10 from dual union all
select 30690,'1973-088','1973-11-10 20:12:00','orbital','success',1426,'23D-8','HEXAGON SV7','HEXAGON SV-7','','Sat',1777,'',null from dual union all
select 30691,'1973-089','1973-11-14 20:40:02','orbital','success',809,'78014  -215-15','Molniya-1','Molniya-1 11F67 No. 32','','Sat',480,'',null from dual union all
select 30692,'1973-090','1973-11-16 14:01:23','orbital','success',1121,'SA-208','Skylab 4','CM-118','','Sat',922,'LUT1',null from dual union all
select 30693,'1973-091','1973-11-20 12:29:58','orbital','success',678,'29524-154','Kosmos-608','DS-P1-Yu No. 69','','Sat',460,'',null from dual union all
select 30694,'1973-092','1973-11-21 10:00:00','orbital','success',1508,'77019  -177','Kosmos-609','Zenit-4M','','Sat',480,'',null from dual union all
select 30695,'1973-093','1973-11-27 00:08:01','orbital','success',680,'53724-114','Kosmos-610','Tselina-OM','','Sat',457,'',null from dual union all
select 30696,'1973-094','1973-11-28 09:29:58','orbital','success',678,'64653-170','Kosmos-611','DS-P1-Yu No. 64','','Sat',460,'',null from dual union all
select 30697,'1973-095','1973-11-28 11:43:05','orbital','success',1508,'76029  -759','Kosmos-612','Zenit-4MK','','Sat',475,'',null from dual union all
select 30698,'1973-096','1973-11-30 05:20:00','orbital','success',1223,'S 15000-029','Kosmos-613','Soyuz 7K-T No. 34A','','Sat',480,'',null from dual;
insert into LAUNCH
select 30699,'1973-097','1973-11-30 13:08:59','orbital','success',809,'77019  -550','Molniya-1','Molniya-1','','Sat',473,'',null from dual union all
select 307,'1944-S279','1944-12-07 17:00:00','military missile','success',8,'19874','Ma333','','','Test',676,'',104 from dual union all
select 3070,'1955-A236','1955-04-01 00:00:00','atmospheric rocket','unknown',576,'M-970-PO','','','','Test',1895,'',10 from dual union all
select 30700,'1973-098','1973-12-04 15:00:01','orbital','success',680,'53753-211','Kosmos-614','Strela-2M','','Sat',457,'',null from dual union all
select 30701,'1973-099','1973-12-13 11:10:03','orbital','success',678,'45734-144','Kosmos-615','DS-P1-I No. 13','','Sat',460,'',null from dual union all
select 30702,'1973-100','1973-12-13 23:57:01','orbital','success',1425,'23C-8 (3C-26)','DSCS II 3/4','DSCS II B-3','','Sat UPF',274,'',null from dual union all
select 30703,'1973-101','1973-12-16 06:18:00','orbital','success',377,'585/D99','Explorer 51','AE-C','','Sat',1724,'',null from dual union all
select 30704,'1973-102','1973-12-17 12:00:01','orbital','success',1225,'76043  -825','Kosmos-616','Zenit-4MT','','Sat',473,'',null from dual union all
select 30705,'1973-103','1973-12-18 11:55:00','orbital','success',1223,'S 15000-028','Soyuz-13','Soyuz 7K-T No. 33','','Sat',480,'',null from dual union all
select 30706,'1973-104','1973-12-19 09:43:00','orbital','success',680,'53724-112','Kosmos-617/624','Strela-1M','','Sat',457,'',null from dual union all
select 30707,'1973-105','1973-12-21 12:30:00','orbital','success',1508,'76029  -755','Kosmos-625','Zenit-4MK','','Sat',475,'',null from dual union all
select 30708,'1973-106','1973-12-25 11:17:00','orbital','success',809,'76029  -555','Molniya-2','Molniya-2','','Sat',473,'',null from dual union all
select 30709,'1973-107','1973-12-26 16:30:01','orbital','success',680,'65024-111','Aureole 2','DS-U2-GKA No. 2','','Sat',457,'',null from dual union all
select 3071,'1955-A237','1955-04-01 00:00:00','atmospheric rocket','unknown',576,'M-967-PO','','','','Test',1895,'',10 from dual union all
select 30710,'1973-108','1973-12-27 20:19:58','orbital','success',1446,'45023-413-01','Kosmos-626','US-A reactor','','Sat',590,'',null from dual union all
select 30711,'1973-109','1973-12-29 04:12:00','orbital','success',680,'65033-204','Kosmos-627','Zaliv','','Sat',457,'',null from dual union all
select 30712,'1973-A01','1973-01-11 16:10:00','suborbital rocket','unknown',725,'TERLS-518','ISRO-8.108','TERLS-518','','Aeron',1467,'',23 from dual union all
select 30713,'1973-A02','1973-01-15 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',46 from dual union all
select 30714,'1973-A04','1973-02-01 00:00:00','military missile','success',1472,'','Test','','','ABM Test',183,'',25 from dual union all
select 30715,'1973-A05','1973-02-05 16:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',44 from dual union all
select 30716,'1973-A06','1973-02-08 19:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',47 from dual union all
select 30717,'1973-A07','1973-02-22 00:00:00','military missile','success',1252,'FLA-73','SAFEGUARD M2-41','','','Interceptor',868,'',10 from dual union all
select 30718,'1973-A08','1973-02-22 00:00:00','military missile','success',1252,'FLA-60','SAFEGUARD M2-41','','','Interceptor',868,'',10 from dual union all
select 30719,'1973-A09','1973-02-22 00:00:00','military missile','success',1252,'FLA-75','SAFEGUARD TRM-97','','','Interceptor',868,'',10 from dual union all
select 3072,'1955-A238','1955-04-01 00:00:00','military missile','success',771,'GM-52-1900','','','','Test',222,'',12 from dual union all
select 30720,'1973-A10','1973-02-28 14:30:00','suborbital rocket','unknown',725,'TERLS-530','ISRO-8.114','TERLS-530','','Aeron',1467,'',23 from dual union all
select 30721,'1973-A11','1973-02-28 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',49 from dual union all
select 30722,'1973-A12','1973-03-06 18:29:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',30 from dual union all
select 30723,'1973-A13','1973-03-07 00:22:00','military missile','failure',917,'107','P-254 73 Cape 07','','','OT',262,'',0 from dual union all
select 30724,'1973-A14','1973-03-07 16:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',47 from dual union all
select 30725,'1973-A15','1973-03-12 15:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',42 from dual union all
select 30726,'1973-A16','1973-03-13 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',48 from dual union all
select 30727,'1973-A17','1973-03-16 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',49 from dual union all
select 30728,'1973-A18','1973-03-21 20:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',46 from dual union all
select 30729,'1973-A19','1973-04-08 00:00:00','military missile','failure',1003,'','8K68PM LKI-1','GCh 8F676M','','Test',519,'',0 from dual union all
select 3073,'1955-A239','1955-04-04 00:00:00','atmospheric rocket','unknown',576,'931RM','','','','Test',1895,'',10 from dual union all
select 30730,'1973-A20','1973-04-16 15:03:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',49 from dual union all
select 30731,'1973-A21','1973-04-30 15:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',48 from dual union all
select 30732,'1973-A22','1973-05-24 14:15:00','military missile','failure',1004,'45023-30-4L','15A14 LKI-4','GCh 15F142.500 LM15B86','','Test',483,'',10 from dual union all
select 30733,'1973-A23','1973-05-29 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',43 from dual union all
select 30734,'1973-A24','1973-06-05 00:00:00','military missile','failure',744,'','15A15 LKI-5','','','Test',530,'',0 from dual union all
select 30735,'1973-A25','1973-06-07 00:00:00','military missile','success',1252,'FLA-74','Safeguard M2-22','','','Interceptor',868,'',4 from dual union all
select 30736,'1973-A26','1973-06-11 15:40:00','suborbital rocket','success',776,'Grot-Somit','','','','Aeron',933,'',22 from dual union all
select 30737,'1973-A27','1973-06-12 15:30:00','suborbital rocket','success',776,'Grot-Somit','','','','Aeron',933,'',23 from dual union all
select 30738,'1973-A28','1973-06-14 15:30:00','suborbital rocket','success',779,'Grot-Somit','','','','Aeron',933,'',40 from dual union all
select 30739,'1973-A29','1973-06-15 15:30:00','suborbital rocket','success',779,'Grot-Somit','','','','Aeron',933,'',22 from dual union all
select 3074,'1955-A24','1955-01-18 00:00:00','atmospheric rocket','success',836,'3096R','Nike 416R','','','Test',1895,'',10 from dual union all
select 30740,'1973-A30','1973-06-15 16:20:00','suborbital rocket','success',776,'Grot-Somit','','','','Aeron',933,'',40 from dual union all
select 30741,'1973-A31','1973-06-20 04:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',25 from dual union all
select 30742,'1973-A32','1973-06-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',0 from dual union all
select 30743,'1973-A33','1973-07-20 00:00:00','military missile','success',1252,'FLA-54','Safeguard M2-18','','','Interceptor',866,'',10 from dual union all
select 30744,'1973-A34','1973-07-20 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',48 from dual union all
select 30745,'1973-A35','1973-07-21 04:40:00','military missile','failure',1003,'','8K67PM LKI-3','GCh 8F676M','','Test',518,'',1 from dual union all
select 30746,'1973-A36','1973-07-23 05:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',49 from dual union all
select 30747,'1973-A37','1973-07-23 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',44 from dual union all
select 30748,'1973-A38','1973-07-25 17:20:00','suborbital rocket','success',41,'NASA 26.113GT','','','','Test',1895,'',31 from dual union all
select 30749,'1973-A39','1973-07-30 15:24:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',38 from dual union all
select 3075,'1955-A240','1955-04-04 00:00:00','atmospheric rocket','unknown',576,'922RM','','','','Test',1895,'',10 from dual union all
select 30750,'1973-A40','1973-08-15 00:00:00','atmospheric rocket','success',1309,'','','','','Test',1490,'',5 from dual union all
select 30751,'1973-A41','1973-08-16 13:07:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',45 from dual union all
select 30752,'1973-A42','1973-08-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',0 from dual union all
select 30753,'1973-A43','1973-08-31 14:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',44 from dual union all
select 30754,'1973-A44','1973-09-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',24 from dual union all
select 30755,'1973-A45','1973-09-07 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',24 from dual union all
select 30756,'1973-A46','1973-09-11 21:25:00','military missile','failure',256,'B','Target','','','Target',1689,'',20 from dual union all
select 30757,'1973-A47','1973-09-12 00:00:00','military missile','failure',1004,'10L','15A14 LKI-10','GCh 15F143U1','','Test',510,'',50 from dual union all
select 30758,'1973-A48','1973-09-12 13:45:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',40 from dual union all
select 30759,'1973-A49','1973-09-15 15:30:00','suborbital rocket','success',776,'Grot-Somit','','','','Aeron',933,'',36 from dual union all
select 3076,'1955-A241','1955-04-04 00:00:00','atmospheric rocket','success',836,'1772T','','','','Test',1895,'',10 from dual union all
select 30760,'1973-A50','1973-09-16 15:30:00','suborbital rocket','success',779,'Grot-Somit','','','','Aeron',933,'',44 from dual union all
select 30761,'1973-A51','1973-09-17 15:50:00','suborbital rocket','success',779,'Grot-Somit','','','','Aeron',933,'',45 from dual union all
select 30762,'1973-A52','1973-09-17 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',0 from dual union all
select 30763,'1973-A53','1973-09-18 15:35:00','suborbital rocket','success',779,'Grot-Somit','','','','Aeron',933,'',44 from dual union all
select 30764,'1973-A54','1973-09-19 15:19:00','suborbital rocket','success',88,'NASA 15.113GM','','','','Aeronomy',35,'',49 from dual union all
select 30765,'1973-A55','1973-09-20 04:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',23 from dual union all
select 30766,'1973-A56','1973-09-20 04:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',45 from dual union all
select 30767,'1973-A57','1973-09-28 14:00:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',48 from dual union all
select 30768,'1973-A58','1973-09-28 19:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',24 from dual union all
select 30769,'1973-A59','1973-10-05 00:00:00','military missile','success',1252,'PAVT-1','Safeguard M2-27','','','Interceptor',866,'',10 from dual union all
select 3077,'1955-A242','1955-04-04 00:00:00','atmospheric rocket','success',836,'1957T','','','','Test',1895,'',10 from dual union all
select 30770,'1973-A60','1973-10-08 04:10:00','military missile','failure',1003,'','8K67PM LKI-5','GCh 8F676M?','','Test',518,'',0 from dual union all
select 30771,'1973-A61','1973-10-12 00:00:00','military missile','failure',1004,'11L','15A14 LKI-11','GCh 15F143U1','','Test',483,'',500 from dual union all
select 30772,'1973-A62','1973-10-19 17:15:00','suborbital rocket','success',88,'NASA 15.114GM','','','','Aeronomy',35,'',43 from dual union all
select 30773,'1973-A63','1973-10-21 00:03:00','suborbital rocket','failure',906,'CRL A10.207-3','Release','','','Aeron',394,'',18 from dual union all
select 30774,'1973-A64','1973-11-02 00:00:00','military missile','success',1252,'PAVT-2','Safeguard M2-38','','','Interceptor',866,'',10 from dual union all
select 30775,'1973-A65','1973-11-02 15:06:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',4 from dual union all
select 30776,'1973-A66','1973-11-08 15:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',46 from dual union all
select 30777,'1973-A67','1973-11-08 15:46:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',41 from dual union all
select 30778,'1973-A69','1973-11-15 17:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',49 from dual union all
select 30779,'1973-A70','1973-11-26 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',47 from dual union all
select 3078,'1955-A243','1955-04-04 00:00:00','atmospheric rocket','success',836,'1879T','','','','Test',1895,'',10 from dual union all
select 30780,'1973-A71','1973-11-27 13:52:59','suborbital rocket','success',1061,'','STS','','','Meteo',1895,'',47 from dual union all
select 30781,'1973-A72','1973-11-29 00:00:00','military missile','success',1252,'FLA-65','Safeguard M2-25','','','Interceptor',866,'',10 from dual union all
select 30782,'1973-A73','1973-12-04 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',47 from dual union all
select 30783,'1973-A75','1973-12-07 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',46 from dual union all
select 30784,'1973-A76','1973-12-12 18:14:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',48 from dual union all
select 30785,'1973-A77','1973-12-14 18:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',47 from dual union all
select 30786,'1973-A78','1973-12-17 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 30787,'1973-A79','1973-12-19 15:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',48 from dual union all
select 30788,'1973-A80','1973-10-04 01:20:00','suborbital rocket','failure',1281,'AFCRL Arcas 72-2','ML-TwD','','','Aeron',1895,'',10 from dual;
insert into LAUNCH
select 30789,'1973-A81','1973-06-21 00:00:00','suborbital rocket','failure',1257,'D1-6179 SSL 4-20','Dart F4-1','','','Test',1821,'',21 from dual union all
select 3079,'1955-A244','1955-04-04 00:00:00','atmospheric rocket','success',836,'1606T','','','','Test',1895,'',10 from dual union all
select 30790,'1973-A82','1973-11-27 00:00:00','military missile','success',897,'PRS-1','Test','','','Test',183,'',40 from dual union all
select 30791,'1973-A83','1973-05-24 03:50:00','test rocket','failure',645,'','BOR-3 No. 301','','','Reentry Test',1752,'',20 from dual union all
select 30792,'1973-A84','1973-10-06 00:00:00','suborbital rocket','failure',74,'','Al Tin','','','Test',767,'',3 from dual union all
select 30793,'1973-A85','1973-10-06 00:00:00','suborbital rocket','failure',74,'','Al Tin','','','Test',767,'',3 from dual union all
select 30794,'1973-A86','1973-01-01 00:00:00','atmospheric rocket','success',920,'D','Target','','','Target',2,'',1 from dual union all
select 30795,'1973-A87','1973-07-25 00:00:00','military missile','success',1429,'','','','','Test',636,'',20 from dual union all
select 30796,'1973-A89','1973-02-02 14:08:00','atmospheric rocket','success',1346,'Sandia','SAMS R487101','','','Test',1821,'',4 from dual union all
select 30797,'1973-A90','1973-02-02 14:08:30','atmospheric rocket','success',1346,'Sandia','SAMS R487102','','','Test',1821,'',4 from dual union all
select 30798,'1973-A91','1973-02-27 10:40:00','atmospheric rocket','success',1346,'Sandia','SAMS R487103','','','Test',1821,'',4 from dual union all
select 30799,'1973-A92','1973-02-27 10:40:30','atmospheric rocket','success',1346,'Sandia','SAMS R487104','','','Test',1821,'',4 from dual union all
select 308,'1944-S28','1944-06-01 00:00:00','military missile','success',8,'','Vertical test','','','Test',673,'',120 from dual union all
select 3080,'1955-A245','1955-04-05 00:00:00','military missile','unknown',339,'1567E','','','','Test',1895,'',40 from dual union all
select 30800,'1973-A93','1973-06-28 19:06:00','atmospheric rocket','success',1346,'','','','','Test',1821,'',10 from dual union all
select 30801,'1973-A94','1973-11-01 00:00:00','atmospheric rocket','success',1346,'Sandia','SAMS R487401','','','Test',1821,'',4 from dual union all
select 30802,'1973-E01','1973-06-26 01:22:00','orbital','destroyed before launch',680,'Yu47121-16','','','','Sat',456,'',0 from dual union all
select 30803,'1973-F03','1973-05-25 09:15:34','orbital','failure',680,'65024-109','','Zaliv','','Sat',456,'',20 from dual union all
select 30804,'1973-F04','1973-06-26 17:00:00','orbital','failure',1403,'24B-9  (3B-43)','KH-8A','GAMBIT-3 Mission 4339','','Sat',1778,'',130 from dual union all
select 30805,'1973-F05','1973-07-04 10:59:59','orbital','failure',1508,'7801422-243','','Zenit-4M','','Sat',474,'',50 from dual union all
select 30806,'1973-F06','1973-07-16 17:10:21','orbital','failure',373,'578/D96','ITOS E','ITOS E','','Sat',1724,'',200 from dual union all
select 30807,'1973-F07','1973-09-18 12:12:00','orbital','failure',492,'701-03','JSSW','JSSW','','Sat',810,'',185 from dual union all
select 30808,'1973-M01','1973-01-20 11:38:00','suborbital rocket','success',1189,'INTA MSCL-7301','','','','Ionos',396,'',80 from dual union all
select 30809,'1973-M02','1973-02-01 00:00:00','military missile','success',77,'001','[Spurious]','','','Test',1343,'',50 from dual union all
select 3081,'1955-A246','1955-04-05 00:00:00','atmospheric rocket','unknown',576,'M-829-PO','','','','Test',1895,'',10 from dual union all
select 30810,'1973-M03','1973-01-31 12:57:00','suborbital rocket','success',1191,'INTA MS4CLO-7301','','','','Ionos',396,'',70 from dual union all
select 30811,'1973-M04','1973-03-15 14:15:00','suborbital rocket','success',1189,'SUPARCO/MR/72','','','','Aeron',1419,'',58 from dual union all
select 30812,'1973-M05','1973-03-21 10:11:00','suborbital rocket','success',132,'CRL A30.205-5','','','','Aeron',1047,'',78 from dual union all
select 30813,'1973-M06','1973-03-26 12:35:00','suborbital rocket','success',1189,'INTA MSC-7303','','','','Ionos',396,'',79 from dual union all
select 30814,'1973-M07','1973-04-06 08:45:00','suborbital rocket','success',132,'CRL A30.205-6','Enhanced OH','','','Aeron',1047,'',78 from dual union all
select 30815,'1973-M08','1973-04-18 14:30:00','suborbital rocket','success',1189,'SUPARCO/MR/73','','','','Aeron',1419,'',65 from dual union all
select 30816,'1973-M09','1973-05-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 30817,'1973-M10','1973-05-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 30818,'1973-M11','1973-06-30 20:33:00','suborbital rocket','success',837,'NASA 14.519GI','','','','Ionos',1821,'',77 from dual union all
select 30819,'1973-M12','1973-10-04 00:40:01','suborbital rocket','success',132,'CRL A30.311-1','IR Radiometer','AO30.311-1','','Aeron',1895,'',53 from dual union all
select 3082,'1955-A247','1955-04-05 00:00:00','atmospheric rocket','unknown',576,'M-834-PO','','','','Test',1895,'',10 from dual union all
select 30820,'1973-M13','1973-10-30 18:54:19','suborbital rocket','failure',920,'P147A','High Lat/Particles','','','Auroral',29,'',71 from dual union all
select 30821,'1973-M14','1973-11-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 30822,'1973-M15','1973-05-30 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-21','','','Meteo',350,'',75 from dual union all
select 30823,'1973-M16','1973-03-30 00:00:00','suborbital rocket','success',1257,'D1-6175 SSL 3-16','Robin Dart E4-4','','','Test',1821,'',77 from dual union all
select 30824,'1973-M17','1973-03-30 00:00:00','suborbital rocket','success',1257,'D1-6176 SSL 3-17','Robin Dart E4-4','','','Test',1821,'',75 from dual union all
select 30825,'1973-M18','1973-05-29 00:00:00','suborbital rocket','success',1257,'D1-6178 SSL 3-15','Dart F4-1','','','Test',1821,'',73 from dual union all
select 30826,'1973-M19','1973-04-02 00:00:00','suborbital rocket','success',1257,'D1-6478 SSL 3-19','Dart F4-1','','','Test',1821,'',72 from dual union all
select 30827,'1973-M20','1973-06-21 00:00:00','suborbital rocket','failure',1257,'D1-6180 SSL 4-21','Dart F4-1','','','Test',1821,'',72 from dual union all
select 30828,'1973-M21','1973-06-21 00:00:00','suborbital rocket','failure',1257,'D1-6181 SSL 4-22','Dart F4-1','','','Test',1821,'',66 from dual union all
select 30829,'1973-M22','1973-06-21 00:00:00','suborbital rocket','failure',1257,'D1-6182 SSL 4-23','Dart F4-1','','','Test',1821,'',70 from dual union all
select 3083,'1955-A248','1955-04-05 00:00:00','atmospheric rocket','success',836,'1867T','','','','Test',1895,'',10 from dual union all
select 30830,'1973-M23','1973-06-21 00:00:00','suborbital rocket','failure',1257,'D1-6183 SSL 4-24','Dart F4-1','','','Test',1821,'',70 from dual union all
select 30831,'1973-M24','1973-03-14 18:08:00','suborbital rocket','success',1506,'T 1-6438','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 30832,'1973-M25','1973-04-18 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-19','','','Meteo',350,'',75 from dual union all
select 30833,'1973-M26','1973-05-09 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-20','','','Meteo',350,'',75 from dual union all
select 30834,'1973-M27','1973-06-07 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-22','','','Meteo',350,'',75 from dual union all
select 30835,'1973-M28','1973-09-10 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-23','','','Meteo',350,'',75 from dual union all
select 30836,'1973-M29','1973-09-12 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-24','','','Meteo',350,'',75 from dual union all
select 30837,'1973-M30','1973-10-10 15:15:00','suborbital rocket','success',88,'L1-6576','','','','Meteo',1821,'',70 from dual union all
select 30838,'1973-M31','1973-10-10 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-25','','','Meteo',350,'',75 from dual union all
select 30839,'1973-M32','1973-10-31 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-40','','','Meteo',350,'',75 from dual union all
select 3084,'1955-A249','1955-04-05 00:00:00','atmospheric rocket','success',836,'1758T','','','','Test',1895,'',10 from dual union all
select 30840,'1973-M33','1973-11-29 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-41','','','Meteo',350,'',75 from dual union all
select 30841,'1973-M34','1973-12-12 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-42','','','Meteo',350,'',75 from dual union all
select 30842,'1973-M35','1973-03-28 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-17','','','Meteo',350,'',75 from dual union all
select 30843,'1973-M36','1973-04-04 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-18','','','Meteo',350,'',75 from dual union all
select 30844,'1973-M37','1973-03-21 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-16','','','Meteo',350,'',75 from dual union all
select 30845,'1973-M38','1973-01-10 12:45:00','suborbital rocket','success',716,'INTA HW-7301','','','','Meteo',396,'',60 from dual union all
select 30846,'1973-M39','1973-07-11 12:20:00','suborbital rocket','success',716,'INTA HW-7307','','','','Meteo',396,'',60 from dual union all
select 30847,'1973-M40','1973-11-14 12:12:00','suborbital rocket','success',716,'INTA HW-7308','','','','Meteo',396,'',60 from dual union all
select 30848,'1973-M41','1973-12-12 12:47:00','suborbital rocket','success',716,'INTA HW-7309','','','','Meteo',396,'',60 from dual union all
select 30849,'1973-M42','1973-07-17 14:03:00','suborbital rocket','unknown',1506,'E 1-6678','','','','Meteo',1821,'',60 from dual union all
select 3085,'1955-A25','1955-01-18 00:00:00','atmospheric rocket','success',836,'3102R','Nike 417R','','','Test',1895,'',10 from dual union all
select 30850,'1973-M43','1973-03-30 18:45:00','suborbital rocket','success',88,'T1-6446','','','','Aeron',1821,'',60 from dual union all
select 30851,'1973-M44','1973-04-10 02:15:00','suborbital rocket','success',88,'T1-6428','','','','Aeron',1821,'',60 from dual union all
select 30852,'1973-M46','1973-07-19 18:19:00','suborbital rocket','success',1281,'L1-6574','','','','Aeron',1821,'',60 from dual union all
select 30853,'1973-M47','1973-08-31 17:07:00','suborbital rocket','success',88,'T1-6731','','','','Aeron',1821,'',60 from dual union all
select 30854,'1973-M48','1973-12-04 18:35:00','suborbital rocket','success',88,'NASA L1-6964','','','','Aeron',1821,'',60 from dual union all
select 30855,'1973-M49','1973-08-24 15:55:00','suborbital rocket','success',1281,'T1-6777','','','','Aeron',1821,'',60 from dual union all
select 30856,'1973-M50','1973-09-19 18:39:00','suborbital rocket','success',1281,'T1-6776','','','','Aeron',1821,'',60 from dual union all
select 30857,'1973-M51','1973-10-10 18:04:00','suborbital rocket','success',88,'L1-6577','','','','Meteo?',1821,'',70 from dual union all
select 30858,'1973-S01','1973-01-01 00:00:00','military missile','success',754,'','','','','OT',307,'S655',600 from dual union all
select 30859,'1973-S02','1973-01-01 00:00:00','military missile','success',754,'','','','','OT',307,'S610',600 from dual union all
select 3086,'1955-A250','1955-04-05 00:00:00','atmospheric rocket','success',836,'1568T','','','','Test',1895,'',10 from dual union all
select 30860,'1973-S03','1973-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 30861,'1973-S04','1973-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 30862,'1973-S05','1973-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 30863,'1973-S06','1973-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 30864,'1973-S07','1973-08-17 00:00:00','test rocket','unknown',1497,'1','K65UP No. 1','','','Test/Aeron?',621,'',1500 from dual union all
select 30865,'1973-S08','1973-09-11 00:00:00','military missile','success',1549,'','','','','Test',1626,'PLBR',620 from dual union all
select 30866,'1973-S09','1973-11-23 00:00:00','suborbital rocket','success',1281,'Polar Airglow','AFCRL Arcas 73-1','','','Ionos',1470,'',100 from dual union all
select 30867,'1973-S10','1973-01-30 00:00:00','military missile','success',1456,'','15A20U LKI-13','','','OT',506,'',1000 from dual union all
select 30868,'1973-S100','1973-03-09 10:35:00','military missile','success',795,'FTM-1221','NTV-1 (Mk 12)','ABRES NTV-1 Nosetip','','Reentry Test',1701,'',1300 from dual union all
select 30869,'1973-S101','1973-03-09 04:02:00','military missile','success',1001,'Kh22500-21L','8K67MA LKI-7','GCh 15F143/8F676?','','Test',518,'',1000 from dual union all
select 3087,'1955-A251','1955-04-06 00:00:00','atmospheric rocket','unknown',576,'11A-EM','','','','Test',1895,'',10 from dual union all
select 30870,'1973-S102','1973-03-12 00:00:00','military missile','success',744,'','15A15 LKI-3','','','Test',530,'',1000 from dual union all
select 30871,'1973-S103','1973-03-14 03:15:00','suborbital rocket','success',1211,'SL1004','Cul Wolter XUVS-1','','','Solar X',1839,'',226 from dual union all
select 30872,'1973-S104','1973-03-14 14:15:00','suborbital rocket','unknown',1189,'SUPARCO/MR/71','','','','Aeron',1419,'',100 from dual union all
select 30873,'1973-S105','1973-03-15 00:00:00','military missile','success',1103,'2602','S02C 2602','','','Test',301,'',1000 from dual union all
select 30874,'1973-S106','1973-03-16 10:40:00','suborbital rocket','success',867,'NASA 18.152UE','','','','Plasma',1047,'',249 from dual union all
select 30875,'1973-S107','1973-03-19 16:02:00','military missile','success',917,'111','P-256 73 Cape 09','','','OT',262,'',180 from dual union all
select 30876,'1973-S108','1973-03-19 16:16:00','military missile','success',917,'112','P-257 73 Cape 10','','','OT',262,'',180 from dual union all
select 30877,'1973-S109','1973-03-19 17:20:00','military missile','success',917,'109','P-258 73 Cape 11','','','OT',262,'',180 from dual union all
select 30878,'1973-S11','1973-01-10 12:45:00','suborbital rocket','success',1189,'INTA MSCLO-7301','','','','Ionos',396,'',82 from dual;
insert into LAUNCH
select 30879,'1973-S110','1973-03-19 17:52:00','military missile','success',917,'110','P-259 73 Cape 12','','','OT',262,'',180 from dual union all
select 3088,'1955-A252','1955-04-06 00:00:00','atmospheric rocket','success',836,'4256R','Nike 437R','','','Test',1895,'',10 from dual union all
select 30880,'1973-S111','1973-03-20 03:41:00','suborbital rocket','success',920,'P43H','Dawn chorus','','','Ionospheric',1447,'',129 from dual union all
select 30881,'1973-S112','1973-03-20 04:52:00','suborbital rocket','success',1211,'SL1102','Cul UVS 20','','','Solar UV',1839,'',242 from dual union all
select 30882,'1973-S113','1973-03-21 06:00:00','suborbital rocket','success',867,'NASA 18.144GE','','','','Plasma',1047,'',225 from dual union all
select 30883,'1973-S114','1973-03-22 06:02:00','suborbital rocket','success',867,'NASA 18.145GE','','','','Plasma',1047,'',240 from dual union all
select 30884,'1973-S115','1973-03-22 12:12:00','suborbital rocket','success',228,'CRL A18.006-2','','','','Auroral',1047,'',183 from dual union all
select 30885,'1973-S116','1973-03-22 17:43:00','suborbital rocket','success',1211,'SL1182','RAE SL1182','','','Earth imaging',952,'',240 from dual union all
select 30886,'1973-S117','1973-03-23 12:36:00','suborbital rocket','success',1189,'INTA MSC-7302','','','','Ionos',396,'',92 from dual union all
select 30887,'1973-S118','1973-03-24 03:35:00','suborbital rocket','success',38,'NASA 13.64UG','','','','XR Astron',1895,'',200 from dual union all
select 30888,'1973-S119','1973-03-24 10:31:00','suborbital rocket','success',906,'CRL A10.205-2','ICECAP 73A','','','Auroral',1047,'',213 from dual union all
select 30889,'1973-S12','1973-01-12 08:05:00','military missile','success',136,'H005','RANT','ABRES RANT','','Reentry Test',615,'',200 from dual union all
select 3089,'1955-A253','1955-04-06 00:00:00','atmospheric rocket','success',836,'1901T','','','','Test',1895,'',10 from dual union all
select 30890,'1973-S120','1973-03-25 20:47:25','suborbital rocket','success',1070,'S-210JA-14','','','','Auroral',1458,'',114 from dual union all
select 30891,'1973-S121','1973-03-27 09:37:45','suborbital rocket','success',228,'CRL A18.205-1','','','','Ionos',1047,'',181 from dual union all
select 30892,'1973-S122','1973-03-27 09:38:25','suborbital rocket','success',906,'CRL A10.216-3','ICECAP 73B','','','Aeron',1047,'',235 from dual union all
select 30893,'1973-S123','1973-03-27 12:58:00','suborbital rocket','success',867,'NASA 18.146GE','','','','Plasma',1047,'',233 from dual union all
select 30894,'1973-S124','1973-03-28 17:51:00','suborbital rocket','success',1211,'SL1181','RAE SL1181','','','Earth imaging',952,'',240 from dual union all
select 30895,'1973-S125','1973-03-29 12:50:00','suborbital rocket','success',1189,'INTA MSC-7304','','','','Ionos',396,'',96 from dual union all
select 30896,'1973-S126','1973-03-29 15:38:00','military missile','success',136,'H006','RVTO-3B-3','ABRES RVTO-3B-3','','Reentry Test',615,'',200 from dual union all
select 30897,'1973-S127','1973-03-29 17:23:00','military missile','success',136,'H008','RVTO-3B-1','ABRES RVTO-3B-1','','Reentry Test',615,'',200 from dual union all
select 30898,'1973-S128','1973-03-30 10:43:00','suborbital rocket','success',38,'NASA 13.40UG','MIT BCP 1','','','XR Astron',1895,'',184 from dual union all
select 30899,'1973-S129','1973-03-30 12:34:00','suborbital rocket','success',867,'NASA 18.147GE','','','','Plasma',1047,'',243 from dual union all
select 309,'1944-S280','1944-12-07 17:06:00','military missile','success',1470,'19901','Ma345','','','Test',661,'',90 from dual union all
select 3090,'1955-A254','1955-04-06 00:00:00','atmospheric rocket','success',836,'4222R','Nike 438R','','','Test',1895,'',10 from dual union all
select 30900,'1973-S13','1973-01-12 10:05:00','military missile','success',136,'H009','ABC-1','ABRES ABC-1','','Reentry Test',615,'',200 from dual union all
select 30901,'1973-S130','1973-04-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 30902,'1973-S131','1973-04-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 30903,'1973-S132','1973-03-31 03:17:00','suborbital rocket','success',920,'P44H','Dawn chorus','','','Ionospheric',1447,'',126 from dual union all
select 30904,'1973-S133','1973-03-31 12:27:00','suborbital rocket','success',867,'NASA 18.148GE','','','','Plasma',1047,'',232 from dual union all
select 30905,'1973-S134','1973-04-03 06:09:00','suborbital rocket','success',920,'P131H','D-Region/Solar La/X','','','Iono-solar',1447,'',131 from dual union all
select 30906,'1973-S135','1973-04-03 06:54:00','suborbital rocket','success',920,'P133H','D-Region/Solar La/X','','','Iono-solar',1447,'',132 from dual union all
select 30907,'1973-S136','1973-04-03 08:14:00','suborbital rocket','success',920,'P105H','D-Region/Solar La/X','','','Iono-solar',1447,'',137 from dual union all
select 30908,'1973-S137','1973-04-03 09:48:00','suborbital rocket','success',920,'P132H','D-Region/Solar La/X','','','Iono-solar',1447,'',139 from dual union all
select 30909,'1973-S138','1973-04-03 12:05:00','suborbital rocket','success',1189,'INTA MSC-7305','','','','Ionos',396,'',93 from dual union all
select 3091,'1955-A255','1955-04-06 00:00:00','atmospheric rocket','success',836,'4442R','Nike 436R','','','Test',1895,'',10 from dual union all
select 30910,'1973-S139','1973-04-03 14:52:00','suborbital rocket','success',920,'P134H','D-Region/Solar La/X','','','Iono-solar',1447,'',140 from dual union all
select 30911,'1973-S14','1973-01-12 11:46:00','suborbital rocket','success',39,'KP3.42','','','','XR Astronomy',1869,'',229 from dual union all
select 30912,'1973-S140','1973-04-03 16:15:00','suborbital rocket','success',920,'P104H','D-Region/Solar La/X','','','Iono-solar',1447,'',136 from dual union all
select 30913,'1973-S141','1973-04-03 17:20:00','suborbital rocket','success',228,'NASA 21.11DS-NP','NRL CALROC 1','','','Solar',1873,'',245 from dual union all
select 30914,'1973-S142','1973-04-03 17:44:00','suborbital rocket','success',920,'P135H','D-Region/Solar La/X','','','Iono-solar',1447,'',134 from dual union all
select 30915,'1973-S143','1973-04-06 00:00:00','military missile','success',744,'','15A15 LKI-4','','','Test',541,'',1000 from dual union all
select 30916,'1973-S144','1973-04-06 08:00:00','military missile','success',1004,'67023-30-2L','15A14 LKI-2','GCh 15F141','','Test',489,'',1000 from dual union all
select 30917,'1973-S145','1973-04-06 12:19:00','suborbital rocket','success',1189,'INTA MSC-7306','','','','Ionos',396,'',93 from dual union all
select 30918,'1973-S146','1973-04-09 05:15:00','military missile','success',1452,'51173-72-001','15A30 LKI-1','GCh 15B103T (15F305x6)','','Test',532,'',1000 from dual union all
select 30919,'1973-S147','1973-04-10 06:59:00','suborbital rocket','success',37,'NASA 13.91DG','','','','XR Astron',1895,'',198 from dual union all
select 3092,'1955-A256','1955-04-06 00:00:00','atmospheric rocket','success',836,'4252R','Nike 435R','','','Test',1895,'',10 from dual union all
select 30920,'1973-S148','1973-04-11 17:50:00','suborbital rocket','success',289,'ISRO 5.20','ISRO 5.20','TERLS-552','','Fields',1467,'',151 from dual union all
select 30921,'1973-S149','1973-04-16 10:04:00','suborbital rocket','success',1468,'CRL A09.209-1','Photometers','','','Aeron',1895,'',195 from dual union all
select 30922,'1973-S15','1973-01-13 21:45:00','military missile','success',939,'','DASO-25','','','OT',421,'SSBN 645',500 from dual union all
select 30923,'1973-S150','1973-04-16 11:58:00','suborbital rocket','success',1468,'CRL A09.102-1','Photometers','','','Aeron',1895,'',209 from dual union all
select 30924,'1973-S151','1973-04-18 04:25:00','military missile','success',796,'','Mk 11 RV','FOT GT112M','GLORY TRIP 112GM','OT',1697,'',1300 from dual union all
select 30925,'1973-S152','1973-04-17 10:30:00','suborbital rocket','success',1492,'61M/95','FU-200 3SUV','','','Solar EUV',341,'',200 from dual union all
select 30926,'1973-S153','1973-04-17 15:35:00','suborbital rocket','success',1211,'SL1011','LUX XRS15','','','XR Astronomy',1839,'',214 from dual union all
select 30927,'1973-S154','1973-04-22 23:54:20','suborbital rocket','success',1070,'S-210JA-17','','','','Auroral',1458,'',124 from dual union all
select 30928,'1973-S155','1973-04-25 08:36:00','military missile','success',795,'FTM-373','HK-2','Army STP HK-2','','Reentry Test?',1698,'',1300 from dual union all
select 30929,'1973-S156','1973-04-27 04:50:00','military missile','success',136,'H010','RVTO-3B S/N 104','ABRES RVTO-3B-4','','Reentry Test',615,'',200 from dual union all
select 3093,'1955-A257','1955-04-06 00:00:00','military missile','unknown',844,'B4 1810-5B','Test','','','Test',1895,'',10 from dual union all
select 30930,'1973-S157','1973-04-27 03:23:00','military missile','success',799,'','Mk 12 RV x 3','OT GT21GB','GLORY TRIP 21GB','OT',1711,'',1300 from dual union all
select 30931,'1973-S158','1973-04-26 06:20:00','suborbital rocket','success',588,'','LRL BOX-18','','','XR Astron',1283,'T-AGM-8',262 from dual union all
select 30932,'1973-S159','1973-04-27 04:53:00','suborbital rocket','success',30,'CRL A03.911-1','UV Absorption','','','Aeron',1869,'',121 from dual union all
select 30933,'1973-S16','1973-01-15 23:40:00','suborbital rocket','success',1070,'S-210-8','','','','Ionos',839,'',122 from dual union all
select 30934,'1973-S160','1973-04-28 22:48:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 654',500 from dual union all
select 30935,'1973-S161','1973-04-29 11:13:00','military missile','success',1004,'51023-30-3L','15A14 LKI-3','GCh 15F141','','Test',487,'',1000 from dual union all
select 30936,'1973-S162','1973-06-12 19:17:00','suborbital rocket','success',740,'GMS S-0273','','','','Aeron',120,'VISE',150 from dual union all
select 30937,'1973-S163','1973-05-22 00:00:00','military missile','success',1322,'64553-51-483-6L','15Zh42 LKI-6','','','Test',477,'',1000 from dual union all
select 30938,'1973-S164','1973-05-03 23:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT22GB','GLORY TRIP 22GB','OT',1708,'',1300 from dual union all
select 30939,'1973-S165','1973-05-04 00:00:00','military missile','success',795,'','SSTTP','SSTTP M2-24','','Target',1701,'',1300 from dual union all
select 3094,'1955-A258','1955-04-06 00:00:00','atmospheric rocket','success',836,'1857T','','','','Test',1895,'',10 from dual union all
select 30940,'1973-S166','1973-05-09 23:25:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 655',500 from dual union all
select 30941,'1973-S167','1973-05-12 12:41:00','suborbital rocket','success',228,'A-BBVC-51','DLR A-BBVC-51 Erna','','','Aeronomy/Ionos',29,'',305 from dual union all
select 30942,'1973-S168','1973-05-16 09:19:00','suborbital rocket','success',227,'AAF-VB-37','Tower mod test','','','Test/Aeron/Ionos/So',428,'',301 from dual union all
select 30943,'1973-S169','1973-05-18 16:00:00','suborbital rocket','success',228,'NASA 21.19US-GT','','','','Test/Aeron/Solar EU',1873,'',267 from dual union all
select 30944,'1973-S17','1973-01-16 17:30:00','suborbital rocket','success',837,'NASA 14.510UI','','','','Ionos',1821,'',157 from dual union all
select 30945,'1973-S170','1973-05-19 11:11:00','suborbital rocket','success',920,'P56H','XRB scintillator tel','','','XR Astronomy',1447,'',137 from dual union all
select 30946,'1973-S171','1973-05-19 23:00:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 656',500 from dual union all
select 30947,'1973-S172','1973-05-23 17:05:00','military missile','success',1001,'','8K67PM LKI-2','GCh 8F676M','','Test',518,'',1000 from dual union all
select 30948,'1973-S173','1973-05-23 16:30:00','suborbital rocket','success',35,'NASA 04.235UA','','','','Aeronomy',1869,'',111 from dual union all
select 30949,'1973-S174','1973-05-25 05:50:00','suborbital rocket','success',37,'NASA 13.75UG','','','','UV Astron?',1895,'',178 from dual union all
select 3095,'1955-A259','1955-04-06 00:00:00','military missile','success',1214,'N-69C GM-13107','','','','Test',224,'',10 from dual union all
select 30950,'1973-S175','1973-05-25 15:20:00','suborbital rocket','success',282,'CRL A44.320-01','MAP-III (IR Radiom)','','','Aeron IR',330,'',569 from dual union all
select 30951,'1973-S176','1973-05-29 20:00:00','suborbital rocket','success',740,'','Zarnitsa-1','','','Active plasma',636,'',162 from dual union all
select 30952,'1973-S177','1973-06-13 14:07:00','suborbital rocket','success',740,'GMS S-0373','','','','Aeron',121,'VISE',150 from dual union all
select 30953,'1973-S178','1973-06-01 03:10:00','military missile','success',799,'FTM-5376','Mk 12 RV x 3','PVM-4','','Test',1705,'',1300 from dual union all
select 30954,'1973-S179','1973-06-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 30955,'1973-S18','1973-01-16 20:58:51','suborbital rocket','success',867,'NASA 18.57IE','','','','Plasma',29,'',278 from dual union all
select 30956,'1973-S180','1973-06-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 30957,'1973-S181','1973-06-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 30958,'1973-S182','1973-06-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 30959,'1973-S183','1973-06-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 3096,'1955-A26','1955-01-18 00:00:00','atmospheric rocket','success',836,'3044R','Nike 418R','','','Test',1895,'',10 from dual union all
select 30960,'1973-S184','1973-06-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 30961,'1973-S185','1973-06-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 30962,'1973-S186','1973-06-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 30963,'1973-S187','1973-06-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 30964,'1973-S189','1973-06-19 14:05:00','military missile','success',917,'AO-120','P-260 73 Spring 01','','','OT',433,'',180 from dual union all
select 30965,'1973-S19','1973-01-18 17:55:00','suborbital rocket','success',41,'NASA 26.01CS','LPARL X-ray Spec.','','','Solar X',1895,'',245 from dual union all
select 30966,'1973-S190','1973-06-19 19:00:00','military missile','success',917,'AO-121','P-261 73 Spring 02','','','OT',433,'',180 from dual union all
select 30967,'1973-S191','1973-06-08 00:00:00','military missile','success',1322,'64553-21-731-7L','15Zh42 LKI-7','','','Test',477,'',1000 from dual union all
select 30968,'1973-S192','1973-06-06 06:53:00','military missile','success',1452,'51173-72-002','15A30 LKI-2','GCh 15B103T','','Test',544,'',1000 from dual;
insert into LAUNCH
select 30969,'1973-S193','1973-06-02 16:51:00','suborbital rocket','success',38,'NASA 13.97CS','','','','Solar',1895,'',175 from dual union all
select 3097,'1955-A260','1955-04-07 00:00:00','military missile','unknown',339,'1574R','','','','Test',1895,'',40 from dual union all
select 30970,'1973-S194','1973-06-04 17:10:00','suborbital rocket','failure',228,'NASA 21.21NP-US','HCO CALROC 1','','','Solar EUV',1873,'',132 from dual union all
select 30971,'1973-S195','1973-06-08 03:08:00','military missile','success',795,'','SSTTP','SSTTP M2-22','','Target',1701,'',1300 from dual union all
select 30972,'1973-S196','1973-06-10 20:20:18','suborbital rocket','success',1070,'S-210JA-13','','','','Auroral',1458,'',123 from dual union all
select 30973,'1973-S197','1973-06-11 21:10:56','suborbital rocket','success',1070,'S-210JA-15','','','','Auroral',1458,'',125 from dual union all
select 30974,'1973-S198','1973-06-11 18:30:00','suborbital rocket','success',41,'NASA 26.02CS','ATM Support','','','Solar',1895,'',221 from dual union all
select 30975,'1973-S199','1973-06-12 16:09:00','suborbital rocket','success',1468,'CRL A09.210-1','D/E region ions','','','Aeron/Plasma',428,'',124 from dual union all
select 30976,'1973-S20','1973-01-19 09:00:00','suborbital rocket','success',668,'K-9M-41','VLF','','','Ionos/Active plasma',839,'',330 from dual union all
select 30977,'1973-S200','1973-06-12 16:17:00','suborbital rocket','success',1468,'CRL A09.210-2','D/E region ions','','','Aeron/Plasma',428,'',125 from dual union all
select 30978,'1973-S201','1973-06-13 17:38:00','suborbital rocket','success',228,'NASA 21.12DS-NP','NRL CALROC 2','','','Solar UV/Solar X',1873,'',241 from dual union all
select 30979,'1973-S202','1973-06-16 00:00:00','military missile','success',1004,'5L','15A14 LKI-5','GCh 15F142','','Test',489,'',1000 from dual union all
select 3098,'1955-A261','1955-04-07 00:00:00','atmospheric rocket','success',836,'1797T','','','','Test',1895,'',10 from dual union all
select 30980,'1973-S203','1973-06-16 10:30:00','suborbital rocket','success',1211,'SL1111','Cul FUV-2','','','UV Astronomy',1839,'',227 from dual union all
select 30981,'1973-S204','1973-06-21 00:00:00','military missile','success',744,'','15A15 LKI-6','','','Test',541,'',1000 from dual union all
select 30982,'1973-S205','1973-06-21 10:30:00','suborbital rocket','success',220,'APF-4B-30','Galactic X-rays','','','XR Astron',855,'',686 from dual union all
select 30983,'1973-S206','1973-06-22 00:00:00','military missile','success',1251,'PFT-01','SAFEGUARD M2-45','','','Interceptor',868,'',347 from dual union all
select 30984,'1973-S207','1973-06-22 00:00:00','military missile','success',1251,'800038','SAFEGUARD M2-45','','','Interceptor',868,'',340 from dual union all
select 30985,'1973-S208','1973-06-23 10:45:30','suborbital rocket','success',1347,'Sandia 281-32','LRL ACS-4E (ACS-9)','','','XR Astron',845,'',297 from dual union all
select 30986,'1973-S209','1973-06-27 00:00:00','military missile','success',917,'AO-122','P-262 73 Spring 03','','','OT',433,'',180 from dual union all
select 30987,'1973-S21','1973-01-19 11:49:00','suborbital rocket','success',1189,'INTA MSCLO-7302','','','','Ionos',396,'',82 from dual union all
select 30988,'1973-S210','1973-06-27 00:00:00','military missile','failure',917,'AO-123','P-263 73 Spring 04','','','OT',433,'',50 from dual union all
select 30989,'1973-S211','1973-06-27 08:30:00','military missile','success',796,'FTM-2644','Mk 11C RV','FOT GT114M','GLORY TRIP 114M','OT',1702,'',1300 from dual union all
select 3099,'1955-A262','1955-04-08 00:00:00','atmospheric rocket','unknown',576,'M-116-AES','','','','Test',1895,'',10 from dual union all
select 30990,'1973-S212','1973-06-29 00:00:00','suborbital rocket','success',740,'','FEYERVERK','','','Active plasma',1010,'',160 from dual union all
select 30991,'1973-S213','1973-06-29 11:59:00','suborbital rocket','success',920,'P162H','Composition','','','Ionospheric',1447,'',140 from dual union all
select 30992,'1973-S214','1973-07-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 30993,'1973-S215','1973-07-26 00:00:00','military missile','failure',917,'AO-126','P-266 73 Spring 07','','','OT',433,'',0 from dual union all
select 30994,'1973-S216','1973-07-26 00:00:00','military missile','success',917,'AO-127','P-267 73 Spring 08','','','OT',433,'',180 from dual union all
select 30995,'1973-S217','1973-07-05 00:00:00','military missile','success',1322,'64513-51-481-8L','15Zh42 LKI-8','','','Test',477,'',1000 from dual union all
select 30996,'1973-S218','1973-07-31 05:30:00','military missile','success',1452,'51139-72-003','15A30 LKI-3','GCh 15B103T','','Test',544,'',1000 from dual union all
select 30997,'1973-S219','1973-06-30 10:14:38','suborbital rocket','success',740,'','Eclipse','','','Eclipse',125,'VISE',150 from dual union all
select 30998,'1973-S22','1973-01-21 09:40:00','suborbital rocket','success',289,'ISRO 5.15','ISRO 5.15  C.09','TERLS-520','','XR Astron',1467,'',165 from dual union all
select 30999,'1973-S220','1973-06-30 10:33:00','suborbital rocket','success',39,'KP3.41','Coronal spectrograph','','','Solar UV/Eclipse',999,'',180 from dual union all
select 31,'1943-S09','1943-06-16 00:00:00','military missile','success',8,'31','','','','Test',675,'',95 from dual union all
select 310,'1944-S281','1944-12-08 12:24:00','military missile','success',1470,'19894','Ma342','','','Test',661,'',90 from dual union all
select 3100,'1955-A263','1955-04-08 00:00:00','atmospheric rocket','unknown',576,'M-114 AES','','','','Test',1895,'',10 from dual union all
select 31000,'1973-S221','1973-06-30 13:07:00','suborbital rocket','success',867,'','','','','Solar EUV',1412,'',270 from dual union all
select 31001,'1973-S222','1973-06-30 20:12:00','suborbital rocket','success',839,'NASA 10.412GI','','','','Ionos',1821,'',86 from dual union all
select 31002,'1973-S223','1973-06-30 20:30:00','suborbital rocket','success',132,'NASA 23.01GT','','','','Test',1821,'',93 from dual union all
select 31003,'1973-S224','1973-07-01 10:15:00','suborbital rocket','success',740,'B-37','Eclipse control','','','Ionos',125,'VISE',175 from dual union all
select 31004,'1973-S225','1973-07-02 07:45:00','suborbital rocket','success',37,'NASA 13.87UG','CAL XR 8','','','XR Astron',1895,'',176 from dual union all
select 31005,'1973-S226','1973-07-02 12:30:00','suborbital rocket','success',920,'P163H','Composition','','','Ionospheric',1447,'',136 from dual union all
select 31006,'1973-S227','1973-07-04 00:00:00','military missile','success',1004,'6L','15A14 LKI-6','GCh 15F142','','Test',510,'',1000 from dual union all
select 31007,'1973-S228','1973-07-04 01:15:00','suborbital rocket','failure',289,'ISRO 5.21','ISRO 5.21','TERLS-578','','Fields?',1467,'',1 from dual union all
select 31008,'1973-S229','1973-07-04 03:27:11','suborbital rocket','success',228,'A-BBVC-49','DLR A-BBVC-49 Barbara','','','Aeronomy/Ionos',29,'',260 from dual union all
select 31009,'1973-S23','1973-01-22 11:38:00','suborbital rocket','success',1189,'INTA MSCLO-7303','','','','Ionos',396,'',91 from dual union all
select 3101,'1955-A264','1955-04-08 00:00:00','atmospheric rocket','unknown',576,'M-969-PO','','','','Test',1895,'',10 from dual union all
select 31010,'1973-S230','1973-07-06 00:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT23GB','GLORY TRIP 23GB','OT',1711,'',1300 from dual union all
select 31011,'1973-S231','1973-07-09 00:00:00','military missile','success',744,'','15A15 LKI-7','','','Test',530,'',1000 from dual union all
select 31012,'1973-S232','1973-07-12 14:49:00','suborbital rocket','unknown',740,'B-38','','','','Aeron/Ionos',127,'VISE',175 from dual union all
select 31013,'1973-S233','1973-07-15 19:09:12','suborbital rocket','success',1070,'S-210JA-19','','','','Auroral',1458,'',131 from dual union all
select 31014,'1973-S234','1973-07-17 00:00:00','suborbital rocket','success',1207,'SL1291','SUPER ANTELOPE S2','','','MIRV test',1839,'',210 from dual union all
select 31015,'1973-S235','1973-07-17 14:53:00','military missile','unknown',917,'AO-124','P-264 73 Spring 05','','','OT',433,'',180 from dual union all
select 31016,'1973-S236','1973-07-17 16:19:00','military missile','unknown',917,'AO-125','P-265 73 Spring 06','','','OT',433,'',180 from dual union all
select 31017,'1973-S237','1973-07-17 15:22:00','suborbital rocket','success',839,'NASA 10.416GM','GSFC Grenade 273 W','','','Aeronomy',1821,'',117 from dual union all
select 31018,'1973-S238','1973-07-17 16:41:00','suborbital rocket','success',839,'NASA 10.417GM','GSFC Grenade 274 W','','','Aeronomy',1821,'',116 from dual union all
select 31019,'1973-S239','1973-07-17 22:13:00','suborbital rocket','success',624,'NASA 8.61UA','','','','Aeronomy',1821,'',724 from dual union all
select 3102,'1955-A265','1955-04-08 00:00:00','atmospheric rocket','unknown',576,'M-971-PO','','','','Test',1895,'',10 from dual union all
select 31020,'1973-S24','1973-01-22 23:10:00','suborbital rocket','success',228,'NASA 21.09GT-US','','','','Aeron/Solar EUV',1873,'',277 from dual union all
select 31021,'1973-S240','1973-07-18 04:17:00','suborbital rocket','success',839,'NASA 10.418GM','GSFC Grenade 275 W','','','Aeronomy',1821,'',115 from dual union all
select 31022,'1973-S241','1973-07-18 06:00:00','suborbital rocket','success',839,'NASA 10.419GM','GSFC Grenade 276 W','','','Aeronomy',1821,'',121 from dual union all
select 31023,'1973-S242','1973-07-19 21:21:00','suborbital rocket','success',740,'B-39','','','','Aeron/Ionos',124,'VISE',175 from dual union all
select 31024,'1973-S243','1973-07-21 01:00:00','military missile','success',795,'','SSTTP','SSTTP M2-18','','Target',1701,'',1300 from dual union all
select 31025,'1973-S244','1973-07-20 00:00:00','military missile','success',754,'','','','','OT',301,'',600 from dual union all
select 31026,'1973-S245','1973-07-20 06:48:00','suborbital rocket','unknown',740,'B-40','','','','Aeron/Ionos',127,'VISE',175 from dual union all
select 31027,'1973-S246','1973-07-23 12:23:00','suborbital rocket','success',920,'P45H','XRB scintillator tel','','','XR Astronomy',1447,'',143 from dual union all
select 31028,'1973-S247','1973-07-25 00:00:00','military missile','success',744,'','15A15 LKI-8','','','Test',541,'',1000 from dual union all
select 31029,'1973-S248','1973-07-25 15:45:00','suborbital rocket','success',41,'NASA 26.04GP','','','','Test',1895,'',276 from dual union all
select 3103,'1955-A266','1955-04-08 00:00:00','atmospheric rocket','failure',859,'','Heat Transfer','','','Test',1821,'',18 from dual union all
select 31030,'1973-S249','1973-07-26 21:25:00','military missile','success',796,'','ERCS','ERCS GIANT MOON 4','GIANT MOON 4','Comms',1700,'',1300 from dual union all
select 31031,'1973-S25','1973-01-23 00:00:00','military missile','success',939,'','FOT-5','','','OT',421,'SSBN 628',500 from dual union all
select 31032,'1973-S250','1973-07-28 01:06:00','suborbital rocket','success',867,'NASA 18.153UA','','','','Aeronomy',1895,'',120 from dual union all
select 31033,'1973-S251','1973-07-30 00:00:00','military missile','success',1004,'8L','15A14 LKI-7','GCh 15F141','','Test',487,'',1000 from dual union all
select 31034,'1973-S252','1973-06-29 10:19:00','suborbital rocket','success',740,'8','Eclipse control','','','Ionos',125,'VISE',150 from dual union all
select 31035,'1973-S253','1973-08-01 00:00:00','suborbital rocket','success',740,'9','','','','Aeron?',127,'VISE',150 from dual union all
select 31036,'1973-S254','1973-08-01 00:00:00','suborbital rocket','success',740,'10','','','','Aeron?',127,'VISE',150 from dual union all
select 31037,'1973-S255','1973-08-01 00:00:00','suborbital rocket','success',740,'11','','','','Aeron?',127,'VISE',150 from dual union all
select 31038,'1973-S256','1973-08-01 00:00:00','suborbital rocket','success',740,'12','','','','Aeron?',127,'VISE',150 from dual union all
select 31039,'1973-S257','1973-08-01 00:00:00','suborbital rocket','success',740,'13','','','','Aeron?',127,'VISE',150 from dual union all
select 3104,'1955-A267','1955-04-10 00:00:00','atmospheric rocket','success',836,'1548T','','','','Test',1895,'',10 from dual union all
select 31040,'1973-S258','1973-08-01 00:00:00','suborbital rocket','success',740,'14','','','','Aeron?',127,'VISE',150 from dual union all
select 31041,'1973-S259','1973-08-01 00:00:00','suborbital rocket','success',740,'15','','','','Aeron?',127,'VISE',150 from dual union all
select 31042,'1973-S26','1973-01-23 00:00:00','military missile','success',939,'','FOT-5','','','OT',421,'SSBN 628',500 from dual union all
select 31043,'1973-S260','1973-08-29 00:00:00','military missile','success',1322,'64553-51-484-9L','15Zh42 LKI-9','','','Test',477,'',1000 from dual union all
select 31044,'1973-S261','1973-08-31 05:10:00','military missile','success',1452,'51159-72-005','15A30 LKI-4','GCh 15B103','','Test',532,'',1000 from dual union all
select 31045,'1973-S262','1973-08-01 22:17:50','suborbital rocket','success',837,'NASA 14.515UA','SSC NNA 6','','','Aeronomy',417,'',109 from dual union all
select 31046,'1973-S263','1973-08-01 22:48:39','suborbital rocket','success',837,'NASA 14.516UA','SSC NNA 7','','','Aeronomy',417,'',111 from dual union all
select 31047,'1973-S264','1973-08-02 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT113M','GLORY TRIP 113M','OT',1699,'',1300 from dual union all
select 31048,'1973-S265','1973-08-02 05:35:45','suborbital rocket','success',839,'NASA 10.414GM','SSC NNC 10','','','Aeronomy',417,'',88 from dual union all
select 31049,'1973-S266','1973-08-03 16:00:00','suborbital rocket','success',837,'NASA 14.513UI','','','','Ionos',1821,'',184 from dual union all
select 3105,'1955-A268','1955-04-10 00:00:00','atmospheric rocket','success',836,'1620T','','','','Test',1895,'',10 from dual union all
select 31050,'1973-S267','1973-08-04 08:10:00','suborbital rocket','failure',1209,'SL1081A','BAC ER1','','','Earth imaging',416,'',7 from dual union all
select 31051,'1973-S268','1973-08-05 23:42:36','suborbital rocket','success',837,'NASA 14.507GI','SSC NNA 9','','','Aeronomy',417,'',123 from dual union all
select 31052,'1973-S269','1973-08-06 00:08:25','suborbital rocket','success',837,'NASA 14.517UA','SSC NNA 8','','','Aeronomy',417,'',103 from dual union all
select 31053,'1973-S27','1973-01-23 00:00:00','military missile','success',939,'','FOT-5','','','OT',421,'SSBN 628',500 from dual union all
select 31054,'1973-S270','1973-08-09 00:00:00','suborbital rocket','success',220,'NASA 19.07GT','','','','Test',1821,'',250 from dual union all
select 31055,'1973-S271','1973-08-09 00:00:00','military missile','success',795,'','SSTTP','SSTTP M2-20','PVM-3','Target',1701,'',1300 from dual union all
select 31056,'1973-S272','1973-08-09 17:30:58','suborbital rocket','success',228,'NASA 21.22NP-US','HCO CALROC 2','','','Solar EUV',1873,'',269 from dual union all
select 31057,'1973-S273','1973-08-09 20:12:00','suborbital rocket','success',220,'CRL A16.000-01','HI Rate Midas Conv','','','Test',1821,'',605 from dual;
insert into LAUNCH
select 31058,'1973-S274','1973-08-10 10:52:00','suborbital rocket','success',30,'CRL A03.211-1','Bremstrahlung','','','Plasma',1869,'',168 from dual union all
select 31059,'1973-S275','1973-08-10 15:05:00','suborbital rocket','success',837,'NASA 14.514UI','','','','Ionos',1821,'',188 from dual union all
select 3106,'1955-A269','1955-04-10 00:00:00','atmospheric rocket','success',836,'1572T','','','','Test',1895,'',10 from dual union all
select 31060,'1973-S276','1973-08-14 08:58:00','suborbital rocket','success',334,'WRE C4001','','','','Aeron',1835,'',121 from dual union all
select 31061,'1973-S277','1973-08-15 00:00:00','military missile','success',1004,'7L','15A14 LKI-8','GCh 15B87 (8x 15F143U1)','','Test',483,'',1000 from dual union all
select 31062,'1973-S278','1973-08-15 05:45:00','suborbital rocket','success',837,'NASA 14.512UA','','','','Aeronomy',1895,'',117 from dual union all
select 31063,'1973-S279','1973-08-17 00:00:00','military missile','success',1003,'','8K67PM LKI-4','GCh 8F676M','','Test',519,'',1000 from dual union all
select 31064,'1973-S28','1973-01-23 00:00:00','military missile','success',939,'','FOT-5','','','OT',421,'SSBN 628',500 from dual union all
select 31065,'1973-S280','1973-08-19 09:55:00','suborbital rocket','success',1070,'S-210-9','NO airglow','','','Aeron',839,'',115 from dual union all
select 31066,'1973-S281','1973-08-21 07:00:00','suborbital rocket','success',668,'K-9M-43','','','','Plasma/Ionos',839,'',355 from dual union all
select 31067,'1973-S282','1973-08-22 21:53:00','suborbital rocket','success',1070,'S-210JA-18','Auroral arc','','','Auroral',1458,'',129 from dual union all
select 31068,'1973-S283','1973-08-24 01:35:00','military missile','success',799,'PVM-3','Mk 12 RV x 3','R&D','','Test',1703,'',1300 from dual union all
select 31069,'1973-S284','1973-08-23 03:45:00','suborbital rocket','success',41,'NASA 26.08GG','13-in RC Flight 1','','','UV Astron',1895,'',213 from dual union all
select 3107,'1955-A27','1955-01-18 00:00:00','atmospheric rocket','success',836,'3105R','','','','Test',1895,'',10 from dual union all
select 31070,'1973-S285','1973-08-24 07:15:00','military missile','success',136,'H011','ABC-3','ABRES ABC-3','','Reentry Test',615,'',200 from dual union all
select 31071,'1973-S286','1973-08-27 12:00:00','suborbital rocket','success',668,'K-9M-44','','','','XR Astron',839,'',341 from dual union all
select 31072,'1973-S287','1973-08-29 09:18:00','military missile','success',162,'78F','Mk 11C RV','BMRS RVTO-3A-2/FAIR-2','','Reentry Test',1688,'',1400 from dual union all
select 31073,'1973-S288','1973-08-30 17:01:00','suborbital rocket','success',228,'NASA 21.07US','','','','Solar',1873,'',299 from dual union all
select 31074,'1973-S289','1973-09-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 31075,'1973-S29','1973-01-24 03:10:00','military missile','success',1001,'','8K67MA LKI-6','GCh 15F141','','Test',515,'',1000 from dual union all
select 31076,'1973-S290','1973-09-17 05:25:00','military missile','success',1452,'51159-72-004','15A30 LKI-5','GCh 15B103','','Test',544,'',1000 from dual union all
select 31077,'1973-S291','1973-09-24 04:55:00','military missile','success',1452,'51159-72-006','15A30 LKI-6','GCh 15B103','','Test',532,'',1000 from dual union all
select 31078,'1973-S292','1973-09-03 00:00:00','military missile','success',1004,'9L','15A14 LKI-9','GCh 15F143U1','','Test',489,'',1000 from dual union all
select 31079,'1973-S293','1973-09-03 00:03:00','suborbital rocket','success',920,'P97H','Airglow','','','Atmospheric',1447,'',135 from dual union all
select 3108,'1955-A270','1955-04-11 00:00:00','atmospheric rocket','success',836,'1658T','','','','Test',1895,'',10 from dual union all
select 31080,'1973-S294','1973-09-03 02:04:00','suborbital rocket','success',920,'P98H','Airglow','','','Atmospheric',1447,'',143 from dual union all
select 31081,'1973-S295','1973-09-04 16:45:00','suborbital rocket','success',228,'NASA 21.13DS-NP','NRL CALROC 3','','','Solar UV/Solar X',1873,'',235 from dual union all
select 31082,'1973-S296','1973-09-05 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT43GM','GLORY TRIP 43GM','OT',1697,'',1300 from dual union all
select 31083,'1973-S297','1973-09-06 11:39:00','suborbital rocket','success',920,'P165H','Composition/La','','','Ionospheric',1447,'',131 from dual union all
select 31084,'1973-S298','1973-09-07 00:00:00','military missile','success',795,'','SSTTP','SSTTP M2-44','','Target',1701,'',1300 from dual union all
select 31085,'1973-S299','1973-09-07 01:40:00','suborbital rocket','success',613,'JCR-9','','','','Test',1476,'',125 from dual union all
select 31086,'1973-S30','1973-01-24 11:37:00','suborbital rocket','success',1189,'INTA MSCLO-7304','','','','Ionos',396,'',88 from dual union all
select 31087,'1973-S300','1973-09-10 00:02:00','suborbital rocket','success',920,'P102H','TMA','','','Atmospheric',1447,'',170 from dual union all
select 31088,'1973-S301','1973-09-10 04:18:00','suborbital rocket','success',920,'P103H','TMA','','','Atmospheric',1447,'',168 from dual union all
select 31089,'1973-S302','1973-09-11 03:19:00','suborbital rocket','success',1468,'CRL A09.214-1','Vehicle potential','','','Test',1895,'',160 from dual union all
select 3109,'1955-A271','1955-04-11 00:00:00','atmospheric rocket','success',836,'1694T','','','','Test',1895,'',10 from dual union all
select 31090,'1973-S303','1973-09-13 21:30:00','suborbital rocket','success',37,'NASA 13.69GS','','','','Solar EUV',1895,'',199 from dual union all
select 31091,'1973-S304','1973-09-15 05:25:00','military missile','success',799,'','Mk 12 RV x 3','OT GT25GB','GLORY TRIP 25GB','OT',1712,'',1300 from dual union all
select 31092,'1973-S305','1973-09-15 04:36:00','suborbital rocket','success',920,'P106H','ELF/VLF','','','Ionospheric',1447,'',124 from dual union all
select 31093,'1973-S306','1973-09-15 08:23:00','suborbital rocket','success',740,'X-141','Composition','','','Aeron/Ionos',1010,'',150 from dual union all
select 31094,'1973-S307','1973-09-15 22:15:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 657',500 from dual union all
select 31095,'1973-S308','1973-09-18 00:00:00','military missile','success',1105,'S-1E-2602','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 31096,'1973-S309','1973-09-18 15:50:15','suborbital rocket','success',1468,'CRL A09.312-5','Polar disturbed ion.','','','Ionos',1047,'',127 from dual union all
select 31097,'1973-S31','1973-01-25 00:00:00','military missile','success',1251,'800039','SAFEGUARD M2-26','','','Interceptor',868,'',150 from dual union all
select 31098,'1973-S310','1973-09-18 17:21:39','suborbital rocket','success',1468,'CRL A09.312-4','Polar disturbed ion.','','','Ionos',1047,'',129 from dual union all
select 31099,'1973-S311','1973-09-19 09:17:00','suborbital rocket','unknown',676,'WRE K2007','','','','Aeron',1832,'',121 from dual union all
select 311,'1944-S282','1944-12-08 17:24:00','military missile','success',1470,'19888','Ma344','','','Test',661,'',90 from dual union all
select 3110,'1955-A272','1955-04-12 00:00:00','atmospheric rocket','success',836,'1921T','','','','Test',1895,'',10 from dual union all
select 31100,'1973-S312','1973-09-21 17:00:00','suborbital rocket','success',41,'NASA 26.20GS','EUV solar spectrum','','','Solar EUV',1895,'',290 from dual union all
select 31101,'1973-S313','1973-09-22 09:20:00','suborbital rocket','success',655,'K-10-10','','','','XR Astron/Ionos',839,'',242 from dual union all
select 31102,'1973-S314','1973-09-26 03:35:00','military missile','success',796,'','Mk 11 RV','FOT GT115M-1','GLORY TRIP 115M-1','OT',1699,'',1300 from dual union all
select 31103,'1973-S315','1973-09-25 09:30:00','military missile','unknown',917,'AO-128','P-268 73 Fall 01','','','OT',615,'',180 from dual union all
select 31104,'1973-S316','1973-09-25 10:05:00','military missile','unknown',917,'AO-129','P-269 73 Fall 02','','','OT',615,'',180 from dual union all
select 31105,'1973-S317','1973-09-26 16:00:00','suborbital rocket','success',848,'CRL A07.306-2','Sphere','','','Aeron',330,'',225 from dual union all
select 31106,'1973-S318','1973-09-26 19:42:00','suborbital rocket','success',906,'CRL A10.306-1','O2/O3/X-ray','','','Aeron',330,'',288 from dual union all
select 31107,'1973-S319','1973-09-27 04:27:00','suborbital rocket','success',848,'CRL A07.306-3','Sphere','','','Aeron',330,'',232 from dual union all
select 31108,'1973-S32','1973-01-25 01:15:00','suborbital rocket','success',331,'WRE C','','','','Aeron',1835,'',122 from dual union all
select 31109,'1973-S320','1973-09-27 17:01:00','suborbital rocket','success',740,'X-142','Composition','','','Aeron/Ionos',1010,'',141 from dual union all
select 3111,'1955-A273','1955-04-13 00:00:00','atmospheric rocket','success',836,'4189R','Nike 443R','','','Test',1895,'',10 from dual union all
select 31110,'1973-S321','1973-09-30 00:00:00','military missile','success',162,'108F','ACE FTV-1','BMRS ACE FTV-1','','Reentry Test',1686,'',1400 from dual union all
select 31111,'1973-S322','1973-10-01 00:00:00','suborbital rocket','success',721,'','','','','Aeron?',1832,'',100 from dual union all
select 31112,'1973-S323','1973-10-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 31113,'1973-S324','1973-10-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 31114,'1973-S325','1973-10-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 31115,'1973-S326','1973-10-01 00:00:00','military missile','unknown',837,'Capache','','','','Aeron?',1895,'',100 from dual union all
select 31116,'1973-S33','1973-01-25 11:47:00','suborbital rocket','success',1189,'INTA MSCL-7302','','','','Ionos',396,'',94 from dual union all
select 31117,'1973-S331','1973-10-10 09:28:00','military missile','unknown',917,'AO-130','P-270 73 Fall 03','','','OT',615,'',180 from dual union all
select 31118,'1973-S332','1973-10-10 11:41:00','military missile','unknown',917,'AO-131','P-271 73 Fall 04','','','OT',615,'',180 from dual union all
select 31119,'1973-S333','1973-10-24 10:15:00','military missile','unknown',917,'AO-132','P-272 73 Fall 05','','','OT',615,'',180 from dual union all
select 3112,'1955-A274','1955-04-13 00:00:00','atmospheric rocket','success',836,'1941T','','','','Test',1895,'',10 from dual union all
select 31120,'1973-S334','1973-10-24 11:05:00','military missile','unknown',917,'AO-133','P-273 73 Fall 06','','','OT',615,'',180 from dual union all
select 31121,'1973-S335','1973-10-18 00:00:00','military missile','success',1452,'51159-72-007','15A30 LKI-7','GCh 15B103','','Test',544,'',1000 from dual union all
select 31122,'1973-S336','1973-10-02 00:00:00','military missile','success',744,'','15A15 LKI-9','','','Test',530,'',1000 from dual union all
select 31123,'1973-S337','1973-10-03 06:30:00','military missile','success',796,'','Mk 11 RV','FOT GT116M','GLORY TRIP 116M','OT',1702,'',1300 from dual union all
select 31124,'1973-S338','1973-10-02 01:52:06','suborbital rocket','success',1205,'ESRO S93','DLR H-GR-57','INTA SKY-S93','','Test/UV Astron',395,'',186 from dual union all
select 31125,'1973-S339','1973-10-04 01:27:00','suborbital rocket','success',132,'CRL A30.311-2','IR Radiometer','AO30.311-2','','Aeron',1895,'',100 from dual union all
select 31126,'1973-S34','1973-01-27 11:43:00','suborbital rocket','success',1189,'INTA MSCLO-7305','','','','Ionos',396,'',87 from dual union all
select 31127,'1973-S340','1973-10-04 05:00:00','suborbital rocket','success',132,'CRL A30.311-3','IR Radiometer','AO30.311-3','','Aeron',1895,'',102 from dual union all
select 31128,'1973-S341','1973-10-04 03:40:00','suborbital rocket','success',38,'NASA 13.10GG','GSFC XRPC','','','XR Astron',1895,'',152 from dual union all
select 31129,'1973-S342','1973-10-06 00:00:00','military missile','success',1420,'B-69','Mk 6 SSTTP T5','SAFSCOM M2-27','','Target',1658,'',1300 from dual union all
select 3113,'1955-A275','1955-04-13 00:00:00','atmospheric rocket','success',836,'4347R','Nike 447R','','','Test',1895,'',10 from dual union all
select 31130,'1973-S343','1973-10-06 03:31:34','suborbital rocket','success',1209,'ESRO S92','DLR H-GR-56','INTA SKY-S92','','Opt Astron/UV Astron',395,'',260 from dual union all
select 31131,'1973-S344','1973-12-04 00:33:00','military missile','success',1450,'49239-32-228T','15A20 UBP-13 K','GCh 15B20','','OT',504,'',1000 from dual union all
select 31132,'1973-S345','1973-10-14 00:01:20','suborbital rocket','success',920,'P100A','High Lat/Na','','','Atmospheric',29,'',166 from dual union all
select 31133,'1973-S346','1973-10-16 19:41:39','suborbital rocket','success',920,'P141A','High Lat/Particles','','','Auroral',29,'',148 from dual union all
select 31134,'1973-S347','1973-10-16 19:43:39','suborbital rocket','success',1210,'SL1121','UK High Lat 1','','','Ionospheric/Auroral',29,'',267 from dual union all
select 31135,'1973-S348','1973-10-17 09:33:00','suborbital rocket','unknown',676,'WRE K2008','','','','Aeron',1832,'',118 from dual union all
select 31136,'1973-S349','1973-10-17 17:28:10','suborbital rocket','success',920,'P101A','High Lat/Na','','','Atmospheric',29,'',136 from dual union all
select 31137,'1973-S35','1973-01-27 18:31:55','suborbital rocket','success',867,'NASA 18.138IE','Ferdinand 32/Proton 2','','','Plasma',29,'',222 from dual union all
select 31138,'1973-S350','1973-10-17 18:07:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/CR/Plas',1010,'',186 from dual union all
select 31139,'1973-S351','1973-10-17 21:03:00','suborbital rocket','success',117,'NRL NB24.265','Dummy payload','','','Test',1415,'',270 from dual union all
select 3114,'1955-A276','1955-04-13 00:00:00','atmospheric rocket','success',836,'4743R','Nike 446R','','','Test',1895,'',10 from dual union all
select 31140,'1973-S352','1973-10-18 00:00:00','military missile','success',744,'','15A15 LKI-10','','','Test',541,'',1000 from dual union all
select 31141,'1973-S353','1973-12-26 01:00:00','military missile','success',1450,'47649-24-273P','15A20 UBP-14 ZP','GCh 15B20','','OT',504,'',1000 from dual union all
select 31142,'1973-S354','1973-10-18 23:45:00','suborbital rocket','success',848,'CRL A07.105-1','HAVE MERLIN II','Release','','Aeron',394,'',167 from dual union all
select 31143,'1973-S355','1973-10-19 00:00:00','suborbital rocket','success',867,'CRL A08.105-11','HAVE MERLIN II','Release','','Aeron',394,'',206 from dual union all
select 31144,'1973-S356','1973-10-19 00:20:00','suborbital rocket','success',1468,'CRL A09.301-3','Release','','','Aeron',394,'',228 from dual union all
select 31145,'1973-S357','1973-10-19 00:50:00','suborbital rocket','success',848,'CRL A07.105-2','HAVE MERLIN II','Release','','Aeron',394,'',173 from dual union all
select 31146,'1973-S358','1973-10-19 11:25:00','suborbital rocket','success',867,'CRL A08.005-2','HAVE MERLIN II','Release','','Aeron',394,'',146 from dual union all
select 31147,'1973-S359','1973-10-19 12:36:00','suborbital rocket','success',1189,'INTA MSC2A-7301','','','','Ionos',396,'',92 from dual;
insert into LAUNCH
select 31148,'1973-S36','1973-01-28 05:15:00','suborbital rocket','success',700,'L-4SC-2','TVC test','','','Test',834,'',100 from dual union all
select 31149,'1973-S360','1973-10-19 23:42:00','suborbital rocket','success',1468,'CRL A09.106-3','Release','','','Aeron',394,'',176 from dual union all
select 3115,'1955-A277','1955-04-13 00:00:00','atmospheric rocket','success',836,'4190R','Nike 444R','','','Test',1895,'',10 from dual union all
select 31150,'1973-S361','1973-10-19 23:53:00','suborbital rocket','success',867,'CRL A08.105-12','HAVE MERLIN II','Release YVONNE','','Aeron',394,'',160 from dual union all
select 31151,'1973-S362','1973-10-20 23:43:00','suborbital rocket','success',848,'CRL A07.105-3','HAVE MERLIN II','Release','','Aeron',394,'',184 from dual union all
select 31152,'1973-S363','1973-10-22 09:38:00','suborbital rocket','success',334,'WRE C4002','','','','Aeron',1835,'',138 from dual union all
select 31153,'1973-S364','1973-10-23 12:34:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/Plasma',1010,'',154 from dual union all
select 31154,'1973-S365','1973-10-24 03:05:00','suborbital rocket','success',219,'CRL A16.313-01','VLF Propagation','','','Ionos VLF',394,'',609 from dual union all
select 31155,'1973-S366','1973-10-24 08:15:00','suborbital rocket','success',41,'NASA 26.06UG','JHU UVT F7','','','UV Astron',1895,'',183 from dual union all
select 31156,'1973-S367','1973-10-25 03:07:00','suborbital rocket','success',219,'CRL A16.313-02','VLF Propagation','','','Ionos VLF',394,'',600 from dual union all
select 31157,'1973-S368','1973-10-30 08:08:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Plasma',1010,'',150 from dual union all
select 31158,'1973-S369','1973-10-30 18:50:44','suborbital rocket','success',1210,'SL1122','UK High Lat 2','','','Ionospheric/Auroral',29,'',229 from dual union all
select 31159,'1973-S37','1973-01-29 05:02:00','suborbital rocket','success',220,'AAF-4B-27','','','','Auroral',428,'',760 from dual union all
select 3116,'1955-A278','1955-04-13 00:00:00','atmospheric rocket','success',836,'4335R','Nike 442R','','','Test',1895,'',10 from dual union all
select 31160,'1973-S370','1973-10-30 19:15:54','suborbital rocket','success',920,'P142A','High Lat/e,p','','','Auroral',29,'',133 from dual union all
select 31161,'1973-S371','1973-10-30 20:04:06','suborbital rocket','success',920,'P143A','High Lat/e,p','','','Auroral',29,'',135 from dual union all
select 31162,'1973-S372','1973-11-13 10:15:00','military missile','unknown',917,'AO-134','P-274 73 Fall 07','','','OT',615,'',180 from dual union all
select 31163,'1973-S373','1973-11-13 10:55:00','military missile','unknown',917,'AO-135','P-275 73 Fall 08','','','OT',615,'',180 from dual union all
select 31164,'1973-S374','1973-11-27 00:00:00','military missile','unknown',917,'AO-136','P-276 73 Fall 09','','','OT',615,'',180 from dual union all
select 31165,'1973-S375','1973-11-27 00:00:00','military missile','unknown',917,'AO-137','P-277 73 Fall 10','','','OT',615,'',180 from dual union all
select 31166,'1973-S376','1973-11-22 00:00:00','military missile','success',1452,'51159-72-008','15A30 LKI-8','GCh 15B103','','Test',532,'',1000 from dual union all
select 31167,'1973-S377','1973-11-01 15:30:00','suborbital rocket','success',37,'NASA 13.103UG','Wisc. Aerobee IV','','','XR Astron',1846,'',185 from dual union all
select 31168,'1973-S378','1973-11-01 17:30:00','suborbital rocket','success',37,'NASA 13.65UG','','','','XR Astron',1846,'',165 from dual union all
select 31169,'1973-S379','1973-11-03 00:30:00','military missile','success',795,'','SSTTP','SSTTP M2-38','','Target',1701,'',1300 from dual union all
select 3117,'1955-A279','1955-04-13 00:00:00','atmospheric rocket','success',836,'4191R','Nike 440R','','','Test',1895,'',10 from dual union all
select 31170,'1973-S38','1973-01-29 12:45:00','suborbital rocket','success',1189,'INTA MSCL-7303','','','','Ionos',396,'',90 from dual union all
select 31171,'1973-S380','1973-11-02 16:30:00','suborbital rocket','success',37,'CRL A04.104-2','EUV','','','Solar EUV',1895,'',233 from dual union all
select 31172,'1973-S381','1973-11-04 00:00:00','military missile','success',744,'','15A15 LKI-11','','','Test',530,'',1000 from dual union all
select 31173,'1973-S382','1973-11-05 00:00:00','military missile','success',1004,'12L','15A14 LKI-12','GCh 15F141','','Test',487,'',1000 from dual union all
select 31174,'1973-S383','1973-11-06 15:40:00','suborbital rocket','success',37,'NASA 13.104UG','MIT BCP 2','','','XR Astron',1846,'',167 from dual union all
select 31175,'1973-S384','1973-11-09 17:15:00','suborbital rocket','success',37,'NASA 13.106UG','','','','XR Astron',1846,'',169 from dual union all
select 31176,'1973-S385','1973-11-09 18:00:00','suborbital rocket','success',37,'NASA 13.105UG','MIT LATP 3','','','XR Astron',1846,'',175 from dual union all
select 31177,'1973-S387','1973-11-12 14:00:00','suborbital rocket','success',37,'NASA 13.102UG','Wisc. Aerobee III','','','XR Astron',1846,'',172 from dual union all
select 31178,'1973-S388','1973-11-12 14:50:00','suborbital rocket','success',37,'NASA 13.100DS','','','','Solar',1846,'',154 from dual union all
select 31179,'1973-S389','1973-11-16 22:45:50','suborbital rocket','success',1210,'SL1123','UK High Lat 3','','','Ionospheric/Auroral',29,'',216 from dual union all
select 3118,'1955-A28','1955-01-19 00:00:00','military missile','success',255,'623-10','','','','Test',253,'',20 from dual union all
select 31180,'1973-S39','1973-01-31 01:50:00','military missile','success',799,'PVM-2','Mk 12 RV x 3','R&D','','OT',1703,'',1300 from dual union all
select 31181,'1973-S390','1973-11-16 23:04:26','suborbital rocket','success',920,'P150A','High Lat/E field','','','Ionospheric',29,'',175 from dual union all
select 31182,'1973-S391','1973-11-16 23:42:50','suborbital rocket','success',920,'P145A','High Lat/Na','','','Atmospheric',29,'',175 from dual union all
select 31183,'1973-S392','1973-11-17 00:23:13','suborbital rocket','success',920,'P149A','High Lat/Substorm','','','Auroral',29,'',143 from dual union all
select 31184,'1973-S393','1973-11-17 01:01:20','suborbital rocket','success',920,'P146A','High Lat/Na','','','Atmospheric',29,'',175 from dual union all
select 31185,'1973-S394','1973-11-17 01:22:00','suborbital rocket','success',920,'P148A','High Lat/Substorm','','','Auroral',29,'',132 from dual union all
select 31186,'1973-S395','1973-11-18 21:22:00','suborbital rocket','success',624,'NASA 8.63UI-II','','','','Ionos',330,'',650 from dual union all
select 31187,'1973-S396','1973-11-18 23:34:00','suborbital rocket','success',1058,'EGANI/73','MPE','','','Ionos',310,'',250 from dual union all
select 31188,'1973-S397','1973-11-19 23:35:00','suborbital rocket','success',1058,'EGANI/73','MPE','','','Ionos',310,'',260 from dual union all
select 31189,'1973-S398','1973-11-20 00:00:00','military missile','success',1004,'13L','15A14 LKI-13','GCh 15F141','','Test',489,'',1000 from dual union all
select 3119,'1955-A280','1955-04-13 00:00:00','atmospheric rocket','success',836,'4723R','Nike 439R','','','Test',1895,'',10 from dual union all
select 31190,'1973-S399','1973-11-21 10:08:00','suborbital rocket','unknown',676,'WRE K2009','','','','Aeron',1832,'',114 from dual union all
select 31191,'1973-S40','1973-01-30 06:35:00','suborbital rocket','success',1207,'SL1205','MSSL XR12','','','XR Astronomy',1839,'',234 from dual union all
select 31192,'1973-S400','1973-11-22 09:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT42GM-1','GLORY TRIP 42GM-1','OT',1699,'',1300 from dual union all
select 31193,'1973-S401','1973-11-22 23:39:00','suborbital rocket','success',279,'CR-01','EGANI/73 Ba release','','','Ionos',310,'',406 from dual union all
select 31194,'1973-S402','1973-11-22 21:29:00','suborbital rocket','success',220,'','','','','Ionos/Plasma?',330,'',650 from dual union all
select 31195,'1973-S403','1973-11-22 21:29:00','suborbital rocket','success',624,'NASA 8.64UI-II','','','','Ionos',330,'',650 from dual union all
select 31196,'1973-S405','1973-11-23 00:00:00','suborbital rocket','success',41,'NASA 26.09US','','','','Solar',1895,'',250 from dual union all
select 31197,'1973-S406','1973-11-25 22:47:00','suborbital rocket','failure',226,'AMF-VA-40','','','','Auroral',428,'',103 from dual union all
select 31198,'1973-S407','1973-11-26 05:35:00','suborbital rocket','success',1211,'SL1206','LUX BCS 8 (Mk3 BCS)','','','Solar X',1839,'',238 from dual union all
select 31199,'1973-S408','1973-11-27 01:51:56','suborbital rocket','success',1209,'ESRO S74','DLR H-GR-55','INTA SKY-S74','','Opt Astron/UV Astron',395,'',246 from dual union all
select 312,'1944-S283','1944-12-09 15:44:00','military missile','success',1470,'19900','Ma347','','','Test',661,'',90 from dual union all
select 3120,'1955-A281','1955-04-13 00:00:00','atmospheric rocket','success',836,'4754R','Nike 449R','','','Test',1895,'',10 from dual union all
select 31200,'1973-S409','1973-11-27 15:06:33','suborbital rocket','success',219,'NASA 19.08UA','Skylab Ba+','','','Aeronomy',1047,'',552 from dual union all
select 31201,'1973-S41','1973-02-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 31202,'1973-S410','1973-11-28 22:13:00','suborbital rocket','unknown',740,'MR-12','D-75MG','','','Aeron/Ionos',636,'',150 from dual union all
select 31203,'1973-S411','1973-11-30 01:00:00','military missile','success',795,'','SSTTP','SSTTP M2-25','','Target',1701,'',1300 from dual union all
select 31204,'1973-S412','1974-01-30 00:00:00','military missile','failure',1322,'13L','15Zh42 LKI-13','','','Test',477,'',0 from dual union all
select 31205,'1973-S413','1973-12-21 00:00:00','military missile','success',1452,'51159-72-009','15A30 LKI-9','GCh 15B103','','Test',544,'',1000 from dual union all
select 31206,'1973-S414','1973-11-30 11:30:00','suborbital rocket','success',1189,'INTA MSCLO-7306','','','','Ionos',396,'',81 from dual union all
select 31207,'1973-S415','1973-12-02 14:10:00','suborbital rocket','success',920,'P181A','High Lat/Ba','','','Ionospheric',29,'',140 from dual union all
select 31208,'1973-S416','1973-12-03 11:39:00','suborbital rocket','success',1189,'INTA MSCLO-7307','','','','Ionos',396,'',90 from dual union all
select 31209,'1973-S417','1973-12-04 13:45:00','suborbital rocket','success',1189,'AMM-S2-10','Oxygen profile','','','Aeron',428,'',124 from dual union all
select 3121,'1955-A282','1955-04-13 00:00:00','atmospheric rocket','success',836,'4770R','Nike 448R','','','Test',1895,'',10 from dual union all
select 31210,'1973-S418','1973-12-04 15:20:33','suborbital rocket','success',219,'NASA 19.09UA','Skylab Ba+','','','Aeronomy',1047,'',570 from dual union all
select 31211,'1973-S419','1973-12-04 18:00:00','suborbital rocket','success',1189,'AMM-S2-11','','','','Aeron',428,'',108 from dual union all
select 31212,'1973-S42','1973-02-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 31213,'1973-S420','1973-12-04 22:34:00','suborbital rocket','success',1189,'AMM-S2-12','','','','Aeron',428,'',108 from dual union all
select 31214,'1973-S421','1973-12-05 06:52:00','suborbital rocket','success',1189,'AMM-S2-13','','','','Aeron',428,'',110 from dual union all
select 31215,'1973-S422','1973-12-05 11:46:00','suborbital rocket','success',1189,'INTA MSCLO-7308','','','','Ionos',396,'',90 from dual union all
select 31216,'1973-S423','1973-12-05 14:05:00','suborbital rocket','success',1189,'AMM-S2-14','','','','Aeron',428,'',110 from dual union all
select 31217,'1973-S424','1973-12-05 23:23:58','suborbital rocket','success',1208,'SL1124','UK High Lat 4','','','Auroral/Ionospheric',29,'',222 from dual union all
select 31218,'1973-S425','1973-12-06 10:15:00','military missile','success',917,'AO-138','P-278 73 Fall 11','','','OT',615,'',180 from dual union all
select 31219,'1973-S426','1973-12-06 10:45:00','military missile','success',917,'AO-139             *','P-279 73 Fall 12','','','OT WS-205?',615,'',180 from dual union all
select 3122,'1955-A283','1955-04-13 00:00:00','atmospheric rocket','success',836,'4214R','Nike 445R','','','Test',1895,'',10 from dual union all
select 31220,'1973-S427','1973-12-06 07:11:00','suborbital rocket','success',920,'P96A','High Lat/Ba','','','Ionospheric',29,'',140 from dual union all
select 31221,'1973-S428','1973-12-06 09:18:43','suborbital rocket','success',920,'P71A','High Lat/ELF-VLF','','','Ionospheric',29,'',140 from dual union all
select 31222,'1973-S429','1973-12-06 22:59:00','suborbital rocket','success',1440,'CRL A21.220-1','AFCRL TB2-5','','','Reentry Test',1821,'',303 from dual union all
select 31223,'1973-S43','1973-02-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 31224,'1973-S430','1973-12-07 11:32:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/CR/Plas',1010,'',189 from dual union all
select 31225,'1973-S431','1973-12-07 11:32:00','suborbital rocket','success',1189,'INTA MSCLO-7309','','','','Ionos',396,'',90 from dual union all
select 31226,'1973-S432','1973-12-07 11:56:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',150 from dual union all
select 31227,'1973-S433','1973-12-10 19:10:00','suborbital rocket','success',228,'NASA 21.23NP-US','HCO CALROC 3','','','Solar EUV',1873,'',269 from dual union all
select 31228,'1973-S434','1973-12-10 19:30:00','suborbital rocket','success',1468,'CRL A09.107-4','Composition','','','Aeron',428,'',101 from dual union all
select 31229,'1973-S435','1973-12-11 00:00:00','military missile','success',795,'','HK-4','Army STP HK-4?','','Target',1698,'',1300 from dual union all
select 3123,'1955-A284','1955-04-13 00:00:00','atmospheric rocket','success',836,'4451R','Nike 441R','','','Test',1895,'',10 from dual union all
select 31230,'1973-S436','1973-12-11 10:46:00','suborbital rocket','success',1189,'INTA MSCLO-7310','','','','Ionos',396,'',90 from dual union all
select 31231,'1973-S437','1973-12-12 19:00:00','military missile','success',135,'Athena C','HEART V4-6763','','','Reentry Test',1821,'',200 from dual union all
select 31232,'1973-S438','1973-12-12 11:08:00','suborbital rocket','success',1189,'INTA MSCLO-7311','','','','Ionos',396,'',91 from dual union all
select 31233,'1973-S439','1973-12-14 00:00:00','military missile','success',795,'','SSTTP','SSTTP M2-20A','','Target',1701,'',1300 from dual union all
select 31234,'1973-S44','1973-04-06 00:00:00','military missile','success',1449,'','8K84 UBP-141','','','OT',536,'',1000 from dual union all
select 31235,'1973-S440','1973-12-14 11:36:00','suborbital rocket','success',1189,'INTA MSCLO-7312','','','','Ionos',396,'',85 from dual union all
select 31236,'1973-S441','1973-12-16 11:30:00','suborbital rocket','success',1189,'INTA MSCLO-7313','','','','Ionos',396,'',92 from dual union all
select 31237,'1973-S442','1973-12-17 00:00:00','military missile','success',1004,'15L','15A14 LKI-14','GCh 15F141','','Test',510,'',1000 from dual;
insert into LAUNCH
select 31238,'1973-S443','1973-12-19 22:05:00','suborbital rocket','success',41,'NASA 26.03CS','','','','Solar',1895,'',220 from dual union all
select 31239,'1973-S444','1973-12-21 00:00:00','military missile','success',744,'','15A15 LKI-12','','','Test',541,'',1000 from dual union all
select 3124,'1955-A285','1955-04-14 00:00:00','military missile','unknown',339,'1570E','','','','Test',1895,'',40 from dual union all
select 31240,'1973-S445','1973-12-21 12:20:00','military missile','success',1003,'','8K67PM LKI-6','GCh 8F676M?','','Test',519,'',1000 from dual union all
select 31241,'1973-S446','1973-12-22 08:05:00','military missile','success',799,'STM-8W','Mk 12 RV x 3','R&D','STM-8W','Test',1703,'',1300 from dual union all
select 31242,'1973-S447','1973-12-25 08:08:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/CR/Plas',1010,'',190 from dual union all
select 31243,'1973-S448','1973-12-26 00:00:00','military missile','success',1004,'14L','15A14 LKI-15','GCh 15F143U1','','Test',483,'',1000 from dual union all
select 31244,'1973-S449','1973-12-27 00:00:00','military missile','success',744,'','15A15 LKI-13','','','Test',530,'',1000 from dual union all
select 31245,'1973-S45','1973-01-22 02:30:00','military missile','success',1450,'47653-41-27P','15A20 UBP-5 K','GCh 15B20 (RA-65 x 3)','','Test',504,'',1000 from dual union all
select 31246,'1973-S450','1973-12-27 00:00:00','military missile','success',1452,'51159-72-011','15A30 LKI-10','GCh 15B103','','Test',532,'',1000 from dual union all
select 31247,'1973-S451','1973-10-04 00:10:00','suborbital rocket','success',839,'AFCRL 73-1','ML-TwD','','','Aeron',1895,'',120 from dual union all
select 31248,'1973-S452','1973-10-04 00:54:00','suborbital rocket','success',837,'AFCRL 71-B2','ML-TwD','','','Aeron',1895,'',120 from dual union all
select 31249,'1973-S453','1973-10-04 06:10:00','suborbital rocket','success',1281,'AFCRL Arcas 72-1','ML-TwD','','','Aeron',1895,'',90 from dual union all
select 3125,'1955-A286','1955-04-14 00:00:00','atmospheric rocket','success',836,'1839T','','','','Test',1895,'',10 from dual union all
select 31250,'1973-S454','1973-06-12 11:50:00','suborbital rocket','success',740,'MR-12','','','','Aeron',636,'',120 from dual union all
select 31251,'1973-S455','1973-06-12 16:24:00','suborbital rocket','success',740,'MR-12','','','','Aeron',636,'',120 from dual union all
select 31252,'1973-S456','1973-06-12 21:33:00','suborbital rocket','success',740,'MR-12','','','','Aeron',636,'',150 from dual union all
select 31253,'1973-S457','1973-06-14 12:40:00','suborbital rocket','success',740,'MR-12','','','','Aeron',636,'',150 from dual union all
select 31254,'1973-S458','1973-06-14 03:12:00','suborbital rocket','success',740,'GMS S-0473','','','','Aeron',122,'VISE',150 from dual union all
select 31255,'1973-S459','1973-06-14 07:59:00','suborbital rocket','success',740,'GMS S-0573','','','','Aeron',123,'VISE',150 from dual union all
select 31256,'1973-S46','1973-03-17 04:42:00','military missile','success',1450,'47639-21-273P','15A20 UBP-6 ZP','GCh 15B20','','Test',504,'',1000 from dual union all
select 31257,'1973-S460','1973-06-12 08:01:00','suborbital rocket','success',740,'GMS S-0173','','','','Ionos',126,'VISE',150 from dual union all
select 31258,'1973-S461','1973-06-30 13:05:00','suborbital rocket','success',740,'','Eclipse control','','','Aeron',125,'VISE',150 from dual union all
select 31259,'1973-S462','1973-12-04 00:00:00','military missile','success',1549,'','','','','Test',1626,'PLBR',620 from dual union all
select 3126,'1955-A287','1955-04-14 00:00:00','atmospheric rocket','success',836,'1828T','','','','Test',1895,'',10 from dual union all
select 31260,'1973-S463','1973-08-01 00:00:00','military missile','success',1550,'Mod 2, MIRV','','','','Test',1626,'PLBR',620 from dual union all
select 31261,'1973-S464','1973-01-09 00:00:00','suborbital rocket','success',1257,'D1-6168 SSL 2-9','Robin Dart E4-2','','','Test',1821,'',120 from dual union all
select 31262,'1973-S465','1973-01-09 00:00:00','suborbital rocket','success',1257,'D1-6169 SSL 2-10','Robin Dart E4-2','','','Test',1821,'',120 from dual union all
select 31263,'1973-S466','1973-01-09 00:00:00','suborbital rocket','success',1257,'D1-6170 SSL 2-11','Robin Dart E4-2','','','Test',1821,'',119 from dual union all
select 31264,'1973-S467','1973-01-09 00:00:00','suborbital rocket','success',1257,'D1-6171 SSL 2-12','Robin Dart E4-2','','','Test',1821,'',118 from dual union all
select 31265,'1973-S468','1973-03-29 00:00:00','suborbital rocket','success',1257,'D1-6172 SSL 3-13','Robin Dart E4-3','','','Test',1821,'',104 from dual union all
select 31266,'1973-S469','1973-03-29 00:00:00','suborbital rocket','success',1257,'D1-6173 SSL 3-14','Robin Dart E4-3','','','Test',1821,'',106 from dual union all
select 31267,'1973-S47','1973-03-23 03:40:00','military missile','success',1450,'63039-31-233T','15A20 UBP-7 ZP','GCh 15B20','','Test',504,'',1000 from dual union all
select 31268,'1973-S470','1973-03-30 00:00:00','suborbital rocket','success',1257,'D1-6177 SSL 3-18','Robin Dart E4-4','','','Test',1821,'',112 from dual union all
select 31269,'1973-S471','1973-06-22 00:00:00','suborbital rocket','success',1257,'D1-6184 SSL 4-25','Robin Dart E4-3','','','Test',1821,'',105 from dual union all
select 3127,'1955-A288','1955-04-14 00:00:00','atmospheric rocket','success',836,'1896T','','','','Test',1895,'',10 from dual union all
select 31270,'1973-S472','1973-06-22 00:00:00','suborbital rocket','success',1257,'D1-6185 SSL 4-26','Robin Dart E4-3','','','Test',1821,'',106 from dual union all
select 31271,'1973-S473','1973-06-22 00:00:00','suborbital rocket','success',1257,'D1-6186 SSL 4-27','Robin Dart E4-3','','','Test',1821,'',107 from dual union all
select 31272,'1973-S474','1973-06-26 00:00:00','suborbital rocket','success',1257,'D1-6187 SSL 4-28','Robin Dart E4-3','','','Test',1821,'',104 from dual union all
select 31273,'1973-S475','1973-06-26 00:00:00','suborbital rocket','success',1257,'D1-6188 SSL 4-29','Robin Dart E4-3','','','Test',1821,'',100 from dual union all
select 31274,'1973-S476','1973-06-26 00:00:00','suborbital rocket','success',1257,'D1-6189 SSL 4-30','Robin Dart E4-3','','','Test',1821,'',104 from dual union all
select 31275,'1973-S477','1973-01-01 00:00:00','military missile','success',436,'','','','','Test',810,'',1000 from dual union all
select 31276,'1973-S478','1973-10-22 16:50:00','military missile','success',983,'Scud B','R-17E','','','Weapon',366,'',100 from dual union all
select 31277,'1973-S479','1973-10-22 16:50:00','military missile','success',983,'Scud B','','','','Weapon',366,'',100 from dual union all
select 31278,'1973-S48','1973-06-22 18:40:00','military missile','success',1450,'52439-22-273P','15A20 UBP-8 ZP','GCh 15B20','','Test',504,'',1000 from dual union all
select 31279,'1973-S480','1973-10-22 16:50:00','military missile','success',983,'Scud B','','','','Weapon',366,'',100 from dual union all
select 3128,'1955-A289','1955-04-14 00:00:00','atmospheric rocket','success',836,'1881T','','','','Test',1895,'',10 from dual union all
select 31280,'1973-S481','1973-06-20 23:20:00','suborbital rocket','success',331,'WRE C2004','Li release','','','Aeron',1835,'',130 from dual union all
select 31281,'1973-S482','1973-11-14 00:00:00','atmospheric rocket','success',618,'J-3','Ablation 1','','','Test',1836,'',285 from dual union all
select 31282,'1973-S483','1973-12-06 00:00:00','atmospheric rocket','success',618,'J-4','Ablation 2','','','Test',1836,'',250 from dual union all
select 31283,'1973-S484','1973-09-15 00:00:00','suborbital rocket','success',656,'K-10C-4','M-3C Stage Sep Test','','','Test',839,'',100 from dual union all
select 31284,'1973-S485','1973-06-19 00:00:00','suborbital rocket','success',740,'','Ozone','','','Aeron',636,'',150 from dual union all
select 31285,'1973-S486','1973-07-11 00:00:00','military missile','success',1018,'G 15001-10','8K75 UBP-P5','','','OT',477,'',1160 from dual union all
select 31286,'1973-S487','1973-02-19 00:00:00','military missile','success',982,'R 22604-13T','8K64U UBP-P100','R-16 PL73-01','','OT',477,'',1210 from dual union all
select 31287,'1973-S488','1973-02-21 00:00:00','military missile','success',982,'G 22501-14T','8K64U UBP-P101','R-16 PL73-02','','OT',477,'',1210 from dual union all
select 31288,'1973-S489','1973-03-19 00:00:00','military missile','success',982,'G 22502-05T','8K64U UBP-P102','R-16 PL73-03','','OT',477,'',1210 from dual union all
select 31289,'1973-S49','1973-06-27 23:05:00','military missile','success',1450,'49239-32-216T','15A20 UBP-9 ZP','GCh 15B20','','Test',504,'',1000 from dual union all
select 3129,'1955-A29','1955-01-19 00:00:00','military missile','unknown',339,'1466U','','','','Test',1895,'',40 from dual union all
select 31290,'1973-S490','1973-08-07 00:00:00','military missile','success',982,'R 22601-14','8K64U UBP-P103','R-16 PL73-04','','OT',477,'',1210 from dual union all
select 31291,'1973-S491','1973-11-11 00:00:00','military missile','success',1037,'Yu45713-23','8K98 UBP-19','','','Test',477,'',1000 from dual union all
select 31292,'1973-S492','1973-11-30 00:00:00','military missile','success',1037,'64647-221-32','8K98 UBP-20','','','Test',477,'',1000 from dual union all
select 31293,'1973-S493','1973-12-12 00:00:00','military missile','success',1037,'V 29501-11','8K98 UBP-21','','','Test',477,'',1000 from dual union all
select 31294,'1973-S494','1973-09-29 00:00:00','military missile','success',1322,'10L','15Zh42 LKI-10','','','Test',477,'',1000 from dual union all
select 31295,'1973-S495','1973-11-03 00:00:00','military missile','success',1322,'11L','15Zh42 LKI-11','','','Test',477,'',1000 from dual union all
select 31296,'1973-S496','1973-12-23 00:00:00','military missile','success',1322,'64529-677-12L','15Zh42 LKI-12','','','Test',477,'',1000 from dual union all
select 31297,'1973-S497','1973-10-01 00:00:00','military missile','success',1450,'','15A20 54 RD UBP-1','','','OT',1468,'',1000 from dual union all
select 31298,'1973-S498','1973-05-29 00:00:00','military missile','success',1449,'','8K84 10 RD UBP-4','','','OT',897,'',1000 from dual union all
select 31299,'1973-S499','1973-04-25 00:00:00','military missile','success',981,'','8K64 8 RD UBP-5','Vesna-73','','OT',1916,'',1210 from dual union all
select 313,'1944-S284','1944-12-09 17:10:00','military missile','success',8,'19020','','','','Test',676,'',106 from dual union all
select 3130,'1955-A290','1955-04-15 00:00:00','atmospheric rocket','success',836,'1863T','','','','Test',1895,'',10 from dual union all
select 31300,'1973-S50','1973-08-16 04:55:00','military missile','success',1450,'52439-22-274P','15A20 UBP-10 K','GCh 15B20','','Test',504,'',1000 from dual union all
select 31301,'1973-S500','1973-04-25 00:00:00','military missile','success',1449,'T01','8K84 47 RD UBP-17','Duga-1','','OT',1911,'',1000 from dual union all
select 31302,'1973-S501','1973-06-01 00:00:00','military missile','success',1449,'T02','8K84 47 RD UBP-18','Duga-1','','OT',1911,'',1000 from dual union all
select 31303,'1973-S502','1973-07-01 00:00:00','military missile','success',1449,'T03','8K84 47 RD UBP-20','Duga-1','','OT',1911,'',1000 from dual union all
select 31304,'1973-S503','1973-11-01 00:00:00','military missile','success',1449,'T04','8K84 47 RD UBP-21','Duga-1','','OT',1911,'',1000 from dual union all
select 31305,'1973-S504','1973-11-01 00:00:00','military missile','success',1449,'T05','8K84 47 RD UBP-22','Duga-1','','OT',1911,'',1000 from dual union all
select 31306,'1973-S505','1973-11-01 00:00:00','military missile','success',1449,'T06','8K84 47 RD UBP-23','Duga-1','','OT',1911,'',1000 from dual union all
select 31307,'1973-S506','1973-01-01 00:00:00','military missile','success',1449,'T07','8K84 47 RD UBP-1','Duga-1','','OT',1911,'',1000 from dual union all
select 31308,'1973-S507','1973-01-01 00:00:00','military missile','success',1449,'T08','8K84 47 RD UBP-2','Duga-1','','OT',1911,'',1000 from dual union all
select 31309,'1973-S508','1973-01-01 00:00:00','military missile','success',1449,'T09','8K84 47 RD UBP-3','Duga-1','','OT',1911,'',1000 from dual union all
select 3131,'1955-A291','1955-04-15 00:00:00','atmospheric rocket','success',836,'1885T','','','','Test',1895,'',10 from dual union all
select 31310,'1973-S509','1973-01-01 00:00:00','military missile','success',1449,'T10','8K84 47 RD UBP-4','Duga-1','','OT',1911,'',1000 from dual union all
select 31311,'1973-S51','1973-09-28 03:57:00','military missile','success',1450,'64149-23-273P','15A20 UBP-11 K','GCh 15B20','','Test',504,'',1000 from dual union all
select 31312,'1973-S510','1973-01-01 00:00:00','military missile','success',1449,'T11','8K84 47 RD UBP-5','Duga-1','','OT',1911,'',1000 from dual union all
select 31313,'1973-S511','1973-01-01 00:00:00','military missile','success',1449,'T12','8K84 47 RD UBP-6','Duga-1','','OT',1911,'',1000 from dual union all
select 31314,'1973-S512','1973-01-01 00:00:00','military missile','success',1449,'T13','8K84 47 RD UBP-7','Duga-1','','OT',1911,'',1000 from dual union all
select 31315,'1973-S513','1973-01-01 00:00:00','military missile','success',1449,'T14','8K84 47 RD UBP-8','Duga-1','','OT',1911,'',1000 from dual union all
select 31316,'1973-S514','1973-01-01 00:00:00','military missile','success',1449,'T15','8K84 47 RD UBP-9','Duga-1','','OT',1911,'',1000 from dual union all
select 31317,'1973-S515','1973-01-01 00:00:00','military missile','success',1449,'T16','8K84 47 RD UBP-10','Duga-1','','OT',1911,'',1000 from dual union all
select 31318,'1973-S516','1973-01-01 00:00:00','military missile','success',1449,'T17','8K84 47 RD UBP-11','Duga-1','','OT',1911,'',1000 from dual union all
select 31319,'1973-S517','1973-01-01 00:00:00','military missile','success',1449,'T18','8K84 47 RD UBP-12','Duga-1','','OT',1911,'',1000 from dual union all
select 3132,'1955-A292','1955-04-19 00:00:00','atmospheric rocket','unknown',576,'1046RM','','','','Test',1895,'',10 from dual union all
select 31320,'1973-S518','1973-01-01 00:00:00','military missile','success',1449,'T19','8K84 47 RD UBP-13','Duga-1','','OT',1911,'',1000 from dual union all
select 31321,'1973-S519','1973-01-01 00:00:00','military missile','success',1449,'T20','8K84 47 RD UBP-14','Duga-1','','OT',1911,'',1000 from dual union all
select 31322,'1973-S52','1973-10-27 02:35:00','military missile','success',1450,'51159-33-234T','15A20 UBP-12 ZP','GCh 15B20','','Test',504,'',1000 from dual union all
select 31323,'1973-S520','1973-01-01 00:00:00','military missile','success',1449,'T21','8K84 47 RD UBP-15','Duga-1','','OT',1911,'',1000 from dual union all
select 31324,'1973-S521','1973-01-01 00:00:00','military missile','success',1449,'T22','8K84 47 RD UBP-16','Duga-1','','OT',1911,'',1000 from dual union all
select 31325,'1973-S522','1973-07-01 00:00:00','military missile','success',1449,'T23','8K84 47 RD UBP-19','Duga-1','','OT',1911,'',1000 from dual union all
select 31326,'1973-S523','1973-01-01 00:00:00','military missile','success',1449,'D01','8K84 4 RD UBP-2','Duga-1','','OT',368,'',1000 from dual union all
select 31327,'1973-S524','1973-01-01 00:00:00','military missile','success',1449,'D02','8K84 4 RD UBP-3','Duga-1','','OT',368,'',1000 from dual;
insert into LAUNCH
select 31328,'1973-S525','1973-01-01 00:00:00','military missile','success',1449,'D03','8K84 4 RD UBP-4','Duga-1','','OT',368,'',1000 from dual union all
select 31329,'1973-S526','1973-01-01 00:00:00','military missile','success',1449,'D04','8K84 4 RD UBP-5','Duga-1','','OT',368,'',1000 from dual union all
select 3133,'1955-A293','1955-04-19 00:00:00','atmospheric rocket','unknown',576,'1045RM','','','','Test',1895,'',10 from dual union all
select 31330,'1973-S527','1973-01-01 00:00:00','military missile','success',1449,'D05','8K84 4 RD UBP-6','Duga-1','','OT',368,'',1000 from dual union all
select 31331,'1973-S528','1973-01-01 00:00:00','military missile','success',1449,'D06','8K84 4 RD UBP-7','Duga-1','','OT',368,'',1000 from dual union all
select 31332,'1973-S529','1973-01-01 00:00:00','military missile','success',1449,'D07','8K84 4 RD UBP-8','Duga-1','','OT',368,'',1000 from dual union all
select 31333,'1973-S53','1973-02-01 14:33:30','suborbital rocket','success',228,'','DLR N-BBVC-52','','','Aeron/Ionos',330,'',308 from dual union all
select 31334,'1973-S530','1973-01-01 00:00:00','military missile','success',1449,'D08','8K84 4 RD UBP-9','Duga-1','','OT',368,'',1000 from dual union all
select 31335,'1973-S531','1973-01-01 00:00:00','military missile','success',1449,'D09','8K84 4 RD UBP-10','Duga-1','','OT',368,'',1000 from dual union all
select 31336,'1973-S532','1973-01-01 00:00:00','military missile','success',1449,'D10','8K84 4 RD UBP-11','Duga-1','','OT',368,'',1000 from dual union all
select 31337,'1973-S533','1973-01-01 00:00:00','military missile','success',1449,'D11','8K84 4 RD UBP-12','Duga-1','','OT',368,'',1000 from dual union all
select 31338,'1973-S534','1973-01-01 00:00:00','military missile','success',1449,'D12','8K84 4 RD UBP-13','Duga-1','','OT',368,'',1000 from dual union all
select 31339,'1973-S535','1973-01-01 00:00:00','military missile','success',1449,'D13','8K84 4 RD UBP-14','Duga-1','','OT',368,'',1000 from dual union all
select 3134,'1955-A294','1955-04-19 00:00:00','atmospheric rocket','success',836,'1915T','','','','Test',1895,'',10 from dual union all
select 31340,'1973-S536','1973-01-01 00:00:00','military missile','success',1449,'D14','8K84 4 RD UBP-15','Duga-1','','OT',368,'',1000 from dual union all
select 31341,'1973-S537','1973-01-01 00:00:00','military missile','success',1449,'D15','8K84 4 RD UBP-16','Duga-1','','OT',368,'',1000 from dual union all
select 31342,'1973-S538','1973-01-01 00:00:00','military missile','success',1449,'D16','8K84 4 RD UBP-17','Duga-1','','OT',368,'',1000 from dual union all
select 31343,'1973-S539','1973-01-01 00:00:00','military missile','success',1449,'D17','8K84 4 RD UBP-18','Duga-1','','OT',368,'',1000 from dual union all
select 31344,'1973-S54','1973-02-02 06:15:00','suborbital rocket','success',218,'AAF-3B-67','','','','Auroral',598,'',200 from dual union all
select 31345,'1973-S540','1973-01-01 00:00:00','military missile','success',1449,'D18','8K84 4 RD UBP-19','Duga-1','','OT',368,'',1000 from dual union all
select 31346,'1973-S541','1973-01-01 00:00:00','military missile','success',1449,'D19','8K84 4 RD UBP-20','Duga-1','','OT',368,'',1000 from dual union all
select 31347,'1973-S542','1973-01-21 11:35:00','suborbital rocket','success',1189,'INTA MSC-7301','','','','Ionos',396,'',80 from dual union all
select 31348,'1973-S543','1973-04-18 20:35:00','suborbital rocket','success',331,'WRE C2003','Li release','','','Aeron',1835,'',130 from dual union all
select 31349,'1973-S544','1973-09-19 05:00:00','suborbital rocket','success',331,'WRE C2001','Li release','','','Aeron',1835,'',111 from dual union all
select 3135,'1955-A295','1955-04-19 00:00:00','atmospheric rocket','success',836,'4202R','Nike 451R','','','Test',1895,'',10 from dual union all
select 31350,'1973-S545','1973-11-19 10:19:00','suborbital rocket','success',27,'WRE AH1014','TMA/Grenades','','','Aeron',1835,'',151 from dual union all
select 31351,'1973-S546','1973-03-21 00:00:00','suborbital rocket','success',675,'WRE K1072','','','','Aeron',1832,'',85 from dual union all
select 31352,'1973-S547','1973-04-19 00:00:00','suborbital rocket','success',675,'WRE K1073','','','','Aeron',1832,'',85 from dual union all
select 31353,'1973-S548','1973-07-18 00:00:00','suborbital rocket','success',675,'WRE K1074','','','','Aeron',1832,'',85 from dual union all
select 31354,'1973-S549','1973-08-31 00:00:00','military missile','success',982,'U 22501-18','8K64U UBP-P104','R-16 PL73-04','','OT',477,'',1210 from dual union all
select 31355,'1973-S55','1973-02-02 06:20:00','suborbital rocket','success',227,'AAF-VB-36','Auroral substorm','','','Auroral/Aeron/Plasm',428,'',335 from dual union all
select 31356,'1973-S550','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-1','Duga-1','','OT',1455,'',1000 from dual union all
select 31357,'1973-S551','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-2','Duga-1','','OT',1455,'',1000 from dual union all
select 31358,'1973-S552','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-3','Duga-1','','OT',1455,'',1000 from dual union all
select 31359,'1973-S553','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-4','Duga-1','','OT',1455,'',1000 from dual union all
select 3136,'1955-A296','1955-04-19 00:00:00','atmospheric rocket','success',836,'1291R','Nike 450R','','','Test',1895,'',10 from dual union all
select 31360,'1973-S554','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-5','Duga-1','','OT',1455,'',1000 from dual union all
select 31361,'1973-S555','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-6','Duga-1','','OT',1455,'',1000 from dual union all
select 31362,'1973-S556','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-7','Duga-1','','OT',1455,'',1000 from dual union all
select 31363,'1973-S557','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-8','Duga-1','','OT',1455,'',1000 from dual union all
select 31364,'1973-S558','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-9','Duga-1','','OT',1455,'',1000 from dual union all
select 31365,'1973-S559','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-10','Duga-1','','OT',1455,'',1000 from dual union all
select 31366,'1973-S56','1973-02-02 19:05:00','suborbital rocket','success',289,'ISRO 5.18','ISRO 5.18  CO.07','TERLS-523','','Ionos',1467,'',147 from dual union all
select 31367,'1973-S560','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-11','Duga-1','','OT',1455,'',1000 from dual union all
select 31368,'1973-S561','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-12','Duga-1','','OT',1455,'',1000 from dual union all
select 31369,'1973-S562','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-13','Duga-1','','OT',1455,'',1000 from dual union all
select 3137,'1955-A297','1955-04-19 00:00:00','atmospheric rocket','success',836,'4192R','Nike 452R','','','Test',1895,'',10 from dual union all
select 31370,'1973-S563','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-14','Duga-1','','OT',1455,'',1000 from dual union all
select 31371,'1973-S564','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-15','Duga-1','','OT',1455,'',1000 from dual union all
select 31372,'1973-S565','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-16','Duga-1','','OT',1455,'',1000 from dual union all
select 31373,'1973-S566','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-17','Duga-1','','OT',1455,'',1000 from dual union all
select 31374,'1973-S567','1973-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-18','Duga-1','','OT',1455,'',1000 from dual union all
select 31375,'1973-S568','1973-07-25 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 31376,'1973-S569','1973-05-29 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 31377,'1973-S57','1973-02-03 00:00:00','suborbital rocket','success',228,'','DLR N-BBVC-50','','','Aeron?',330,'',97 from dual union all
select 31378,'1973-S570','1973-07-19 00:00:00','military missile','unknown',979,'','8K65 UBP 32 RD','','','Test',636,'',675 from dual union all
select 31379,'1973-S571','1973-08-25 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 3138,'1955-A298','1955-04-20 00:00:00','atmospheric rocket','unknown',576,'M-830-PO','','','','Test',1895,'',10 from dual union all
select 31380,'1973-S572','1973-12-18 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 31381,'1973-S58','1973-02-03 00:00:00','military missile','success',1251,'800035','SAFEGUARD M2-43','','','Interceptor',868,'',100 from dual union all
select 31382,'1973-S59','1973-02-05 04:48:49','suborbital rocket','failure',1116,'Sandia 152-201','Ba ZURITA','','','Aeron/Fields',1047,'',500 from dual union all
select 31383,'1973-S60','1973-02-06 10:35:00','suborbital rocket','success',867,'Sandia 152-199','Caribou VII','','','Auroral',1047,'',270 from dual union all
select 31384,'1973-S61','1973-02-06 19:59:03','suborbital rocket','success',867,'NASA 18.136IE','Ferd. 30/Electra-2','','','Plasma',29,'',255 from dual union all
select 31385,'1973-S62','1973-02-07 01:50:00','suborbital rocket','success',613,'JCR-8','','','','Test',1476,'',135 from dual union all
select 31386,'1973-S63','1973-02-07 14:09:00','suborbital rocket','success',228,'','DLR N-BBVC-54','','','Aeron?',330,'',211 from dual union all
select 31387,'1973-S64','1973-02-09 00:54:02','suborbital rocket','success',867,'NASA 18.132IA','SSC S7-2 (SNT 7/2)','','','Auroral',408,'',182 from dual union all
select 31388,'1973-S65','1973-02-09 02:53:00','suborbital rocket','success',220,'AAF-4B-28','','','','Aeron/Plasma/Auroral',428,'',760 from dual union all
select 31389,'1973-S66','1973-02-10 04:08:00','suborbital rocket','success',228,'NASA 21.16UG','','UCB EUV Phot/XR PC','','EUV Astron/XR Astron',1873,'',216 from dual union all
select 3139,'1955-A299','1955-04-20 00:00:00','atmospheric rocket','unknown',576,'M-836-PO','','','','Test',1895,'',10 from dual union all
select 31390,'1973-S67','1973-02-13 16:29:59','military missile','success',931,'A3T','DASO','','','OT',421,'UK S23',1000 from dual union all
select 31391,'1973-S68','1973-02-14 00:00:00','military missile','success',1103,'2601','S02C 2601','','','Test',301,'',1000 from dual union all
select 31392,'1973-S69','1973-02-14 23:45:00','suborbital rocket','success',1070,'S-210JA-16','','','','Auroral',1458,'',103 from dual union all
select 31393,'1973-S70','1973-02-16 05:16:00','suborbital rocket','success',228,'','DLR N-BBVC-53','','','Aeron',330,'',315 from dual union all
select 31394,'1973-S71','1973-02-17 23:57:00','suborbital rocket','success',448,'D-308','FU-211 EIDI-3','','','Ionos',307,'',460 from dual union all
select 31395,'1973-S72','1973-02-19 00:22:00','suborbital rocket','success',655,'K-10-9','','','','Aeron/Ionos/Solar UV',839,'',236 from dual union all
select 31396,'1973-S73','1973-02-19 13:17:00','suborbital rocket','success',920,'P116H','Composition','','','Ionospheric',1447,'',124 from dual union all
select 31397,'1973-S74','1973-02-21 06:00:00','military missile','success',1004,'45023-30-1L','15A14 LKI-1','GCh 15F141 TM15B85','','Test',483,'',1000 from dual union all
select 31398,'1973-S75','1973-02-21 15:05:00','military missile','success',917,'101 (FOT)','P-248 73 Cape 01','','','OT',262,'',180 from dual union all
select 31399,'1973-S76','1973-02-21 15:37:00','military missile','success',917,'102','P-249 73 Cape 02','','','OT',262,'',180 from dual union all
select 314,'1944-S285','1944-12-11 15:00:00','military missile','success',1470,'19856','Ma319','','','Test',661,'',90 from dual union all
select 3140,'1955-A30','1955-01-19 00:00:00','atmospheric rocket','unknown',576,'467','','','','Test',1895,'',10 from dual union all
select 31400,'1973-S77','1973-02-21 16:30:00','military missile','success',917,'103','P-250 73 Cape 03','','','OT',262,'',180 from dual union all
select 31401,'1973-S78','1973-02-21 17:06:00','military missile','success',917,'104','P-251 73 Cape 04','','','OT',262,'',180 from dual union all
select 31402,'1973-S79','1973-02-22 03:49:00','suborbital rocket','success',218,'AAD-3B-64','Auroral plasma','','','Aeron',428,'',207 from dual union all
select 31403,'1973-S80','1973-02-22 13:04:00','suborbital rocket','success',920,'P114H','Electrons/La','','','Ionospheric',1447,'',126 from dual union all
select 31404,'1973-S81','1973-02-23 09:38:00','suborbital rocket','success',668,'K-9M-42','Cs Release','','','Ionos',839,'',306 from dual union all
select 31405,'1973-S82','1973-02-23 13:48:00','suborbital rocket','success',920,'P113H','Electrons/La','','','Ionospheric',1447,'',140 from dual union all
select 31406,'1973-S83','1973-02-24 07:20:00','suborbital rocket','success',218,'AAD-3B-65','Auroral plasma','','','Aeron',428,'',190 from dual union all
select 31407,'1973-S84','1973-02-26 03:23:00','suborbital rocket','success',215,'ADD-2-127','Auroral VUV','','','Auroral/Aeron/Ionos',428,'',132 from dual union all
select 31408,'1973-S85','1973-02-28 05:35:00','suborbital rocket','success',30,'NASA 04.330UA','','','','Aeronomy',428,'',180 from dual union all
select 31409,'1973-S86','1973-02-28 21:00:00','suborbital rocket','success',1211,'SL1010','LUX XRS14','','','XR Astronomy',1839,'',199 from dual union all
select 3141,'1955-A300','1955-04-20 00:00:00','military missile','success',771,'GM-11075','','','','Test',222,'',12 from dual union all
select 31410,'1973-S87','1973-03-01 00:00:00','military missile','success',939,'','FOT-6','','','OT',421,'SSBN 633',500 from dual union all
select 31411,'1973-S88','1973-03-01 00:00:00','military missile','success',939,'','FOT-6','','','OT',421,'SSBN 633',500 from dual union all
select 31412,'1973-S89','1973-03-01 00:00:00','military missile','success',939,'','FOT-6','','','OT',421,'SSBN 633',500 from dual union all
select 31413,'1973-S90','1973-03-01 00:00:00','military missile','success',939,'','FOT-6','','','OT',421,'SSBN 633',500 from dual union all
select 31414,'1973-S91','1973-03-01 06:57:00','suborbital rocket','success',837,'NASA 14.503UA','Auroral zone NO','','','Aeronomy',428,'',135 from dual union all
select 31415,'1973-S92','1973-03-03 06:05:00','suborbital rocket','success',289,'ISRO 5.19','CNES FU-198','TERLS-532','','Fields',1467,'',125 from dual union all
select 31416,'1973-S93','1973-03-03 06:50:00','suborbital rocket','success',837,'ISRO 10.45','ISRO 10.45','TERLS-533','','Ionos',1467,'',158 from dual union all
select 31417,'1973-S94','1973-03-05 00:00:00','suborbital rocket','success',837,'NASA 14.444UI','','','','Ionos',1895,'',200 from dual;
insert into LAUNCH
select 31418,'1973-S95','1973-03-05 17:25:00','suborbital rocket','success',37,'NASA 13.32US','','','','Solar',1895,'',241 from dual union all
select 31419,'1973-S96','1973-03-06 23:28:00','military missile','success',917,'105','P-252 73 Cape 05','','','OT',262,'',180 from dual union all
select 3142,'1955-A301','1955-04-20 00:00:00','atmospheric rocket','success',836,'1870T','','','','Test',1895,'',10 from dual union all
select 31420,'1973-S97','1973-03-06 23:48:00','military missile','success',917,'106','P-253 73 Cape 06','','','OT',262,'',180 from dual union all
select 31421,'1973-S98','1973-03-07 00:50:00','military missile','success',917,'108','P-255 73 Cape 08','','','OT',262,'',180 from dual union all
select 31422,'1973-S99','1973-03-08 18:00:00','suborbital rocket','success',37,'NASA 13.29CS','','','','Solar',1895,'',170 from dual union all
select 31423,'1973-U01','1973-04-25 09:10:00','orbital','failure',1446,'Yu22500-17L','','US-A','','Sat',589,'',250 from dual union all
select 31424,'1973-U02','1973-05-21 08:47:00','orbital','failure',432,'No. 5','D-5A/D-5B','D-5B','','Sat',343,'',200 from dual union all
select 31425,'1973-W01','1973-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 31426,'1973-W02','1973-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 31427,'1973-W03','1973-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 31428,'1973-W04','1973-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 31429,'1973-W05','1973-01-02 15:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 3143,'1955-A302','1955-04-20 00:00:00','atmospheric rocket','success',836,'1840T','','','','Test',1895,'',10 from dual union all
select 31430,'1973-W06','1973-01-02 18:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 31431,'1973-W07','1973-01-02 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 31432,'1973-W08','1973-01-03 00:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 31433,'1973-W09','1973-01-03 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',70 from dual union all
select 31434,'1973-W10','1973-01-03 16:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',53 from dual union all
select 31435,'1973-W100','1973-01-17 19:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 31436,'1973-W1000','1973-08-09 14:37:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 31437,'1973-W1001','1973-08-09 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 31438,'1973-W1002','1973-08-09 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 31439,'1973-W1003','1973-08-10 11:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 3144,'1955-A303','1955-04-20 00:00:00','atmospheric rocket','success',836,'4446A','','','','Test',1895,'',10 from dual union all
select 31440,'1973-W1004','1973-08-10 13:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 31441,'1973-W1005','1973-08-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',70 from dual union all
select 31442,'1973-W1006','1973-08-10 15:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 31443,'1973-W1007','1973-08-10 16:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 31444,'1973-W1008','1973-08-10 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 31445,'1973-W1009','1973-08-10 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 31446,'1973-W101','1973-01-17 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 31447,'1973-W1010','1973-08-11 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',65 from dual union all
select 31448,'1973-W1011','1973-08-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',71 from dual union all
select 31449,'1973-W1012','1973-08-13 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 3145,'1955-A304','1955-04-21 00:00:00','atmospheric rocket','success',365,'','Turbojet study','','','Test',1821,'',10 from dual union all
select 31450,'1973-W1013','1973-08-13 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31451,'1973-W1014','1973-08-13 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',73 from dual union all
select 31452,'1973-W1015','1973-08-13 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 31453,'1973-W1016','1973-08-13 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 31454,'1973-W1017','1973-08-13 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 31455,'1973-W1018','1973-08-13 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 31456,'1973-W1019','1973-08-13 21:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 31457,'1973-W102','1973-01-17 20:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 31458,'1973-W1020','1973-08-14 17:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 31459,'1973-W1021','1973-08-14 19:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 3146,'1955-A305','1955-04-21 00:00:00','military missile','success',771,'GM-53-92','','','','Test',222,'',12 from dual union all
select 31460,'1973-W1022','1973-08-15 05:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 31461,'1973-W1023','1973-08-15 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',76 from dual union all
select 31462,'1973-W1024','1973-08-15 13:45:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',70 from dual union all
select 31463,'1973-W1025','1973-08-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 31464,'1973-W1026','1973-08-15 14:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 31465,'1973-W1027','1973-08-15 14:51:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 31466,'1973-W1028','1973-08-15 15:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 31467,'1973-W1029','1973-08-15 16:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 31468,'1973-W103','1973-01-17 21:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 31469,'1973-W1030','1973-08-15 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 3147,'1955-A306','1955-04-21 00:00:00','atmospheric rocket','success',836,'1930T','','','','Test',1895,'',10 from dual union all
select 31470,'1973-W1031','1973-08-15 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 31471,'1973-W1032','1973-08-15 21:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',75 from dual union all
select 31472,'1973-W1033','1973-08-15 22:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 31473,'1973-W1034','1973-08-16 00:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 31474,'1973-W1035','1973-08-16 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31475,'1973-W1036','1973-08-16 16:50:00','suborbital rocket','success',88,'NASA 15.111GM','','','','Aeronomy',35,'',53 from dual union all
select 31476,'1973-W1037','1973-08-16 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 31477,'1973-W1038','1973-08-16 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 31478,'1973-W1039','1973-08-16 18:10:00','suborbital rocket','success',88,'NASA 15.112GM','','','','Aeronomy',35,'',51 from dual union all
select 31479,'1973-W104','1973-01-17 23:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 3148,'1955-A307','1955-04-21 00:00:00','atmospheric rocket','success',836,'1869T','','','','Test',1895,'',10 from dual union all
select 31480,'1973-W1040','1973-08-17 10:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 31481,'1973-W1041','1973-08-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',70 from dual union all
select 31482,'1973-W1042','1973-08-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',71 from dual union all
select 31483,'1973-W1043','1973-08-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 31484,'1973-W1044','1973-08-17 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31485,'1973-W1045','1973-08-17 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',57 from dual union all
select 31486,'1973-W1046','1973-08-20 10:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 31487,'1973-W1047','1973-08-20 13:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 31488,'1973-W1048','1973-08-20 14:47:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31489,'1973-W1049','1973-08-20 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 3149,'1955-A308','1955-04-21 00:00:00','atmospheric rocket','success',836,'1841T','','','','Test',1895,'',10 from dual union all
select 31490,'1973-W105','1973-01-18 19:41:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1047,'',60 from dual union all
select 31491,'1973-W1050','1973-08-20 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 31492,'1973-W1051','1973-08-20 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 31493,'1973-W1052','1973-08-21 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 31494,'1973-W1053','1973-08-21 09:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 31495,'1973-W1054','1973-08-21 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 31496,'1973-W1055','1973-08-22 10:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 31497,'1973-W1056','1973-08-22 13:54:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',52 from dual union all
select 31498,'1973-W1057','1973-08-22 14:48:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31499,'1973-W1058','1973-08-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 315,'1944-S286','1944-12-11 17:31:00','military missile','success',1470,'19858','Ma321','','','Test',661,'',90 from dual union all
select 3150,'1955-A31','1955-01-19 00:00:00','atmospheric rocket','success',836,'1526T','','','','Test',1895,'',10 from dual union all
select 31500,'1973-W1059','1973-08-22 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 31501,'1973-W106','1973-01-18 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 31502,'1973-W1060','1973-08-22 17:03:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',68 from dual union all
select 31503,'1973-W1061','1973-08-22 17:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 31504,'1973-W1062','1973-08-22 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual union all
select 31505,'1973-W1063','1973-08-23 00:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',68 from dual union all
select 31506,'1973-W1064','1973-08-23 02:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31507,'1973-W1065','1973-08-23 09:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual;
insert into LAUNCH
select 31508,'1973-W1066','1973-08-23 12:59:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',52 from dual union all
select 31509,'1973-W1067','1973-08-24 01:43:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 3151,'1955-A310','1955-04-21 00:00:00','atmospheric rocket','success',836,'1855T','','','','Test',1895,'',10 from dual union all
select 31510,'1973-W1068','1973-08-24 07:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31511,'1973-W1069','1973-08-24 09:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 31512,'1973-W107','1973-01-19 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 31513,'1973-W1070','1973-08-24 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual union all
select 31514,'1973-W1071','1973-08-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 31515,'1973-W1072','1973-08-24 14:27:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',67 from dual union all
select 31516,'1973-W1073','1973-08-24 14:52:00','suborbital rocket','success',88,'T1-6723','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 31517,'1973-W1074','1973-08-24 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 31518,'1973-W1075','1973-08-24 20:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 31519,'1973-W1076','1973-08-24 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 3152,'1955-A311','1955-04-22 00:00:00','atmospheric rocket','success',836,'1883T','','','','Test',1895,'',10 from dual union all
select 31520,'1973-W1077','1973-08-25 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 31521,'1973-W1078','1973-08-27 10:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 31522,'1973-W1079','1973-08-27 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',53 from dual union all
select 31523,'1973-W108','1973-01-19 15:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 31524,'1973-W1080','1973-08-27 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 31525,'1973-W1081','1973-08-27 14:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 31526,'1973-W1082','1973-08-27 15:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 31527,'1973-W1083','1973-08-27 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',50 from dual union all
select 31528,'1973-W1084','1973-08-28 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 31529,'1973-W1085','1973-08-28 07:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 3153,'1955-A312','1955-04-22 00:00:00','atmospheric rocket','success',836,'1784T','','','','Test',1895,'',10 from dual union all
select 31530,'1973-W1086','1973-08-28 09:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',69 from dual union all
select 31531,'1973-W1087','1973-08-28 14:18:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',66 from dual union all
select 31532,'1973-W1088','1973-08-28 14:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',64 from dual union all
select 31533,'1973-W1089','1973-08-28 15:21:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',60 from dual union all
select 31534,'1973-W109','1973-01-19 16:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',57 from dual union all
select 31535,'1973-W1090','1973-08-29 09:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 31536,'1973-W1091','1973-08-29 10:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 31537,'1973-W1092','1973-08-29 13:44:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',330,'',61 from dual union all
select 31538,'1973-W1093','1973-08-29 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 31539,'1973-W1094','1973-08-29 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',55 from dual union all
select 3154,'1955-A313','1955-04-26 00:00:00','atmospheric rocket','success',836,'1762T','','','','Test',1895,'',10 from dual union all
select 31540,'1973-W1095','1973-08-29 14:37:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 31541,'1973-W1096','1973-08-29 14:45:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',58 from dual union all
select 31542,'1973-W1097','1973-08-29 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 31543,'1973-W1098','1973-08-29 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 31544,'1973-W1099','1973-08-29 18:54:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 31545,'1973-W11','1973-01-03 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',76 from dual union all
select 31546,'1973-W110','1973-01-19 16:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 31547,'1973-W1100','1973-08-29 19:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 31548,'1973-W1101','1973-08-29 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 31549,'1973-W1102','1973-08-30 00:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 3155,'1955-A314','1955-04-26 00:00:00','atmospheric rocket','success',836,'1794T','','','','Test',1895,'',10 from dual union all
select 31550,'1973-W1103','1973-08-30 09:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 31551,'1973-W1104','1973-08-30 12:49:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',70 from dual union all
select 31552,'1973-W1105','1973-08-30 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 31553,'1973-W1106','1973-08-30 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 31554,'1973-W1107','1973-08-30 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 31555,'1973-W1108','1973-08-30 21:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 31556,'1973-W1109','1973-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 31557,'1973-W111','1973-01-19 16:58:00','suborbital rocket','unknown',88,'','','','','Aeron',1895,'',64 from dual union all
select 31558,'1973-W1110','1973-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 31559,'1973-W1111','1973-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 3156,'1955-A315','1955-04-26 00:00:00','atmospheric rocket','success',836,'4204E','','','','Test',1895,'',10 from dual union all
select 31560,'1973-W1112','1973-08-31 09:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 31561,'1973-W1113','1973-08-31 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 31562,'1973-W1114','1973-08-31 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 31563,'1973-W1115','1973-08-31 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 31564,'1973-W1116','1973-08-31 19:58:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',79 from dual union all
select 31565,'1973-W1117','1973-09-01 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 31566,'1973-W1118','1973-09-04 15:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 31567,'1973-W1119','1973-09-04 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 31568,'1973-W112','1973-01-19 17:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',63 from dual union all
select 31569,'1973-W1120','1973-09-04 18:09:00','suborbital rocket','success',88,'T1-6720','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 3157,'1955-A316','1955-04-26 00:00:00','military missile','success',1214,'N-69C GM-13108','','','','Test',238,'',10 from dual union all
select 31570,'1973-W1121','1973-09-04 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 31571,'1973-W1122','1973-09-05 01:57:00','suborbital rocket','success',761,'MT135P T7','','','','Aeron?',1476,'',57 from dual union all
select 31572,'1973-W1123','1973-09-05 06:30:00','suborbital rocket','success',761,'MT135P T8','','','','Aeron?',1476,'',50 from dual union all
select 31573,'1973-W1124','1973-09-05 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 31574,'1973-W1125','1973-09-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 31575,'1973-W1126','1973-09-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 31576,'1973-W1127','1973-09-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 31577,'1973-W1128','1973-09-05 18:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 31578,'1973-W1129','1973-09-05 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 31579,'1973-W113','1973-01-19 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 3158,'1955-A317','1955-04-27 00:00:00','atmospheric rocket','success',836,'1597T','','','','Test',1895,'',10 from dual union all
select 31580,'1973-W1130','1973-09-05 21:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 31581,'1973-W1131','1973-09-06 00:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 31582,'1973-W1132','1973-09-06 21:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 31583,'1973-W1133','1973-09-06 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 31584,'1973-W1134','1973-09-07 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 31585,'1973-W1135','1973-09-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',69 from dual union all
select 31586,'1973-W1136','1973-09-07 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',52 from dual union all
select 31587,'1973-W1137','1973-09-07 17:37:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 31588,'1973-W1138','1973-09-07 17:37:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 31589,'1973-W1139','1973-09-07 17:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 3159,'1955-A318','1955-04-27 00:00:00','atmospheric rocket','success',836,'1919T','','','','Test',1895,'',10 from dual union all
select 31590,'1973-W114','1973-01-19 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1047,'',61 from dual union all
select 31591,'1973-W1140','1973-09-07 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 31592,'1973-W1141','1973-09-10 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 31593,'1973-W1142','1973-09-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',69 from dual union all
select 31594,'1973-W1143','1973-09-10 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',55 from dual union all
select 31595,'1973-W1144','1973-09-10 16:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 31596,'1973-W1145','1973-09-10 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 31597,'1973-W1146','1973-09-10 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 31598,'1973-W1147','1973-09-10 19:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual;
insert into LAUNCH
select 31599,'1973-W1148','1973-09-11 00:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 316,'1944-S287','1944-12-12 11:14:00','military missile','success',8,'18783','Ma191','','','Test',676,'',81 from dual union all
select 3160,'1955-A319','1955-04-27 00:00:00','atmospheric rocket','success',836,'1800E','','','','Test',1895,'',10 from dual union all
select 31600,'1973-W1149','1973-09-11 07:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 31601,'1973-W115','1973-01-19 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 31602,'1973-W1150','1973-09-11 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31603,'1973-W1151','1973-09-11 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 31604,'1973-W1152','1973-09-12 07:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 31605,'1973-W1153','1973-09-12 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 31606,'1973-W1154','1973-09-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 31607,'1973-W1155','1973-09-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 31608,'1973-W1156','1973-09-12 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 31609,'1973-W1157','1973-09-12 15:18:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 3161,'1955-A32','1955-01-19 00:00:00','atmospheric rocket','success',836,'1673E','','','','Test',1895,'',10 from dual union all
select 31610,'1973-W1158','1973-09-12 15:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 31611,'1973-W1159','1973-09-12 15:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',65 from dual union all
select 31612,'1973-W116','1973-01-19 23:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 31613,'1973-W1160','1973-09-12 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',76 from dual union all
select 31614,'1973-W1161','1973-09-12 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31615,'1973-W1162','1973-09-12 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 31616,'1973-W1163','1973-09-13 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 31617,'1973-W1164','1973-09-13 05:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 31618,'1973-W1165','1973-09-13 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',69 from dual union all
select 31619,'1973-W1166','1973-09-13 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 3162,'1955-A320','1955-04-28 00:00:00','military missile','success',771,'GM-53-205','','','','Test',222,'',12 from dual union all
select 31620,'1973-W1167','1973-09-14 13:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 31621,'1973-W1168','1973-09-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 31622,'1973-W1169','1973-09-14 15:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 31623,'1973-W117','1973-01-22 16:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 31624,'1973-W1170','1973-09-14 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 31625,'1973-W1171','1973-09-14 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 31626,'1973-W1172','1973-09-15 00:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 31627,'1973-W1173','1973-09-16 00:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 31628,'1973-W1174','1973-09-16 01:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 31629,'1973-W1175','1973-09-17 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 3163,'1955-A321','1955-04-28 00:00:00','atmospheric rocket','success',836,'1834T','','','','Test',1895,'',10 from dual union all
select 31630,'1973-W1176','1973-09-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 31631,'1973-W1177','1973-09-17 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 31632,'1973-W1178','1973-09-17 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 31633,'1973-W1179','1973-09-17 18:22:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',52 from dual union all
select 31634,'1973-W118','1973-01-22 16:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 31635,'1973-W1180','1973-09-17 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 31636,'1973-W1181','1973-09-17 19:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 31637,'1973-W1182','1973-09-18 16:24:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 31638,'1973-W1183','1973-09-18 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31639,'1973-W1184','1973-09-19 00:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 3164,'1955-A322','1955-04-28 00:00:00','atmospheric rocket','success',836,'1654T','','','','Test',1895,'',10 from dual union all
select 31640,'1973-W1185','1973-09-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 31641,'1973-W1186','1973-09-19 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31642,'1973-W1187','1973-09-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 31643,'1973-W1188','1973-09-19 15:21:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 31644,'1973-W1189','1973-09-19 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 31645,'1973-W119','1973-01-22 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 31646,'1973-W1190','1973-09-19 17:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 31647,'1973-W1191','1973-09-19 18:15:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',67 from dual union all
select 31648,'1973-W1192','1973-09-19 18:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',75 from dual union all
select 31649,'1973-W1193','1973-09-19 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 3165,'1955-A323','1955-04-28 00:00:00','atmospheric rocket','success',836,'1664T','','','','Test',1895,'',10 from dual union all
select 31650,'1973-W1194','1973-09-20 00:41:00','suborbital rocket','success',1061,'','Chaff,Datasonde','','','Meteo',873,'',79 from dual union all
select 31651,'1973-W1195','1973-09-20 16:24:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 31652,'1973-W1196','1973-09-20 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 31653,'1973-W1197','1973-09-20 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31654,'1973-W1198','1973-09-21 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 31655,'1973-W1199','1973-09-21 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',51 from dual union all
select 31656,'1973-W12','1973-01-03 16:48:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 31657,'1973-W120','1973-01-22 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 31658,'1973-W1200','1973-09-21 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 31659,'1973-W1201','1973-09-21 17:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 3166,'1955-A324','1955-04-28 00:00:00','atmospheric rocket','success',836,'1874T','','','','Test',1895,'',10 from dual union all
select 31660,'1973-W1202','1973-09-21 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31661,'1973-W1203','1973-09-21 17:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 31662,'1973-W1204','1973-09-21 19:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 31663,'1973-W1205','1973-09-24 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 31664,'1973-W1206','1973-09-24 14:15:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 31665,'1973-W1207','1973-09-24 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 31666,'1973-W1208','1973-09-24 15:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 31667,'1973-W1209','1973-09-24 17:31:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 31668,'1973-W121','1973-01-22 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 31669,'1973-W1210','1973-09-24 17:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 3167,'1955-A325','1955-04-28 00:00:00','atmospheric rocket','success',836,'1893T','','','','Test',1895,'',10 from dual union all
select 31670,'1973-W1211','1973-09-24 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 31671,'1973-W1212','1973-09-24 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 31672,'1973-W1213','1973-09-25 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 31673,'1973-W1214','1973-09-25 08:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 31674,'1973-W1215','1973-09-25 08:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 31675,'1973-W1216','1973-09-25 10:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31676,'1973-W1217','1973-09-25 16:24:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31677,'1973-W1218','1973-09-26 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 31678,'1973-W1219','1973-09-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 31679,'1973-W122','1973-01-22 20:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 3168,'1955-A326','1955-04-29 00:00:00','military missile','success',771,'Dummy #2','','','','Test',222,'',12 from dual union all
select 31680,'1973-W1220','1973-09-26 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 31681,'1973-W1221','1973-09-26 15:14:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31682,'1973-W1222','1973-09-26 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 31683,'1973-W1223','1973-09-26 17:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 31684,'1973-W1224','1973-09-26 17:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',55 from dual union all
select 31685,'1973-W1225','1973-09-26 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 31686,'1973-W1226','1973-09-27 01:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 31687,'1973-W1227','1973-09-27 10:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 31688,'1973-W1228','1973-09-27 16:24:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',67 from dual;
insert into LAUNCH
select 31689,'1973-W1229','1973-09-27 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 3169,'1955-A327','1955-04-29 00:00:00','atmospheric rocket','success',836,'1837T','','','','Test',1895,'',10 from dual union all
select 31690,'1973-W123','1973-01-22 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 31691,'1973-W1230','1973-09-27 22:42:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 31692,'1973-W1231','1973-09-28 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 31693,'1973-W1232','1973-09-28 14:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 31694,'1973-W1233','1973-09-28 16:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 31695,'1973-W1234','1973-09-28 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31696,'1973-W1235','1973-09-28 18:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 31697,'1973-W1236','1973-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 31698,'1973-W1237','1973-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 31699,'1973-W1238','1973-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 317,'1944-S288','1944-12-12 21:04:00','military missile','success',1470,'19854','Ma318','','','Test',661,'',90 from dual union all
select 3170,'1955-A328','1955-04-29 00:00:00','atmospheric rocket','success',836,'1905T','','','','Test',1895,'',10 from dual union all
select 31700,'1973-W1239','1973-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 31701,'1973-W124','1973-01-22 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 31702,'1973-W1240','1973-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 31703,'1973-W1241','1973-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 31704,'1973-W1242','1973-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 31705,'1973-W1243','1973-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 31706,'1973-W1244','1973-09-30 06:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 31707,'1973-W1245','1973-10-01 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 31708,'1973-W1246','1973-10-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',69 from dual union all
select 31709,'1973-W1247','1973-10-01 14:04:59','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',52 from dual union all
select 3171,'1955-A329','1955-05-02 00:00:00','atmospheric rocket','success',836,'1860T','','','','Test',1895,'',10 from dual union all
select 31710,'1973-W1248','1973-10-01 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 31711,'1973-W1249','1973-10-02 00:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 31712,'1973-W125','1973-01-23 15:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31713,'1973-W1250','1973-10-02 16:24:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31714,'1973-W1251','1973-10-02 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 31715,'1973-W1252','1973-10-02 17:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 31716,'1973-W1253','1973-10-02 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 31717,'1973-W1254','1973-10-03 14:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 31718,'1973-W1255','1973-10-03 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',58 from dual union all
select 31719,'1973-W1256','1973-10-03 16:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',74 from dual union all
select 3172,'1955-A33','1955-01-19 00:00:00','atmospheric rocket','success',836,'1650E','','','','Test',1895,'',10 from dual union all
select 31720,'1973-W1257','1973-10-03 16:40:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 31721,'1973-W1258','1973-10-03 18:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 31722,'1973-W1259','1973-10-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',51 from dual union all
select 31723,'1973-W126','1973-01-23 15:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 31724,'1973-W1260','1973-10-03 23:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',52 from dual union all
select 31725,'1973-W1261','1973-10-04 01:30:00','suborbital rocket','success',1061,'','Starute,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 31726,'1973-W1262','1973-10-04 02:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 31727,'1973-W1263','1973-10-04 04:19:59','suborbital rocket','success',1061,'','Starute,Arcasonde','','','Meteo',1895,'',73 from dual union all
select 31728,'1973-W1264','1973-10-04 04:49:59','suborbital rocket','success',1061,'','Starute,Arcasonde','','','Meteo',1895,'',62 from dual union all
select 31729,'1973-W1265','1973-10-04 16:24:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 3173,'1955-A330','1955-05-03 00:00:00','atmospheric rocket','unknown',576,'1019RM','','','','Test',1895,'',10 from dual union all
select 31730,'1973-W1266','1973-10-04 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 31731,'1973-W1267','1973-10-05 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 31732,'1973-W1268','1973-10-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 31733,'1973-W1269','1973-10-05 14:25:00','suborbital rocket','success',1061,'','Met Para,AN/DMQ-9','','','Meteo',35,'',52 from dual union all
select 31734,'1973-W127','1973-01-23 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 31735,'1973-W1270','1973-10-05 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',72 from dual union all
select 31736,'1973-W1271','1973-10-05 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31737,'1973-W1272','1973-10-05 21:30:00','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',69 from dual union all
select 31738,'1973-W1273','1973-10-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',68 from dual union all
select 31739,'1973-W1274','1973-10-08 15:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',75 from dual union all
select 3174,'1955-A331','1955-05-03 00:00:00','atmospheric rocket','unknown',576,'1034RM','','','','Test',1895,'',10 from dual union all
select 31740,'1973-W1275','1973-10-09 16:24:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31741,'1973-W1276','1973-10-09 17:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 31742,'1973-W1277','1973-10-09 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31743,'1973-W1278','1973-10-09 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 31744,'1973-W1279','1973-10-09 18:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 31745,'1973-W128','1973-01-23 20:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 31746,'1973-W1280','1973-10-09 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 31747,'1973-W1281','1973-10-10 12:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 31748,'1973-W1282','1973-10-10 14:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 31749,'1973-W1283','1973-10-10 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 3175,'1955-A332','1955-05-03 00:00:00','atmospheric rocket','success',836,'1918E','','','','Test',1895,'',10 from dual union all
select 31750,'1973-W1284','1973-10-10 16:41:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',69 from dual union all
select 31751,'1973-W1285','1973-10-10 18:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',70 from dual union all
select 31752,'1973-W1286','1973-10-10 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 31753,'1973-W1287','1973-10-11 01:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',72 from dual union all
select 31754,'1973-W1288','1973-10-11 10:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 31755,'1973-W1289','1973-10-11 19:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',73 from dual union all
select 31756,'1973-W129','1973-01-23 21:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 31757,'1973-W1290','1973-10-11 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 31758,'1973-W1291','1973-10-12 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',72 from dual union all
select 31759,'1973-W1292','1973-10-12 14:41:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 3176,'1955-A333','1955-05-03 00:00:00','atmospheric rocket','success',836,'1761T','','','','Test',1895,'',10 from dual union all
select 31760,'1973-W1293','1973-10-12 16:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 31761,'1973-W1294','1973-10-12 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 31762,'1973-W1295','1973-10-12 19:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 31763,'1973-W1296','1973-10-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 31764,'1973-W1297','1973-10-15 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 31765,'1973-W1298','1973-10-15 15:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 31766,'1973-W1299','1973-10-15 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31767,'1973-W13','1973-01-03 18:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 31768,'1973-W130','1973-01-23 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 31769,'1973-W1300','1973-10-15 17:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 3177,'1955-A334','1955-05-04 00:00:00','atmospheric rocket','success',836,'1533T','','','','Test',1895,'',10 from dual union all
select 31770,'1973-W1301','1973-10-15 18:03:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',66 from dual union all
select 31771,'1973-W1302','1973-10-15 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 31772,'1973-W1303','1973-10-15 18:45:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',68 from dual union all
select 31773,'1973-W1304','1973-10-15 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 31774,'1973-W1305','1973-10-16 16:24:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 31775,'1973-W1306','1973-10-16 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 31776,'1973-W1307','1973-10-16 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 31777,'1973-W1308','1973-10-17 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',70 from dual union all
select 31778,'1973-W1309','1973-10-17 14:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual;
insert into LAUNCH
select 31779,'1973-W131','1973-01-24 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',61 from dual union all
select 3178,'1955-A335','1955-05-04 00:00:00','atmospheric rocket','success',836,'1922T','','','','Test',1895,'',10 from dual union all
select 31780,'1973-W1310','1973-10-17 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',73 from dual union all
select 31781,'1973-W1311','1973-10-17 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 31782,'1973-W1312','1973-10-17 15:27:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31783,'1973-W1313','1973-10-17 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',73 from dual union all
select 31784,'1973-W1314','1973-10-17 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 31785,'1973-W1315','1973-10-17 18:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 31786,'1973-W1316','1973-10-17 19:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 31787,'1973-W1317','1973-10-17 19:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31788,'1973-W1318','1973-10-17 21:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 31789,'1973-W1319','1973-10-18 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 3179,'1955-A336','1955-05-04 00:00:00','atmospheric rocket','success',836,'1738T','','','','Test',1895,'',10 from dual union all
select 31790,'1973-W132','1973-01-24 15:43:00','suborbital rocket','success',716,'','Chaff,Datasonde','','','Meteo',428,'',67 from dual union all
select 31791,'1973-W1320','1973-10-19 08:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 31792,'1973-W1321','1973-10-19 13:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',76 from dual union all
select 31793,'1973-W1322','1973-10-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',71 from dual union all
select 31794,'1973-W1323','1973-10-19 14:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 31795,'1973-W1324','1973-10-19 15:17:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31796,'1973-W1325','1973-10-19 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31797,'1973-W1326','1973-10-19 17:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',74 from dual union all
select 31798,'1973-W1327','1973-10-19 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 31799,'1973-W1328','1973-10-20 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 318,'1944-S289','1944-12-13 09:40:00','military missile','success',1470,'19859','Ma322','','','Test',661,'',90 from dual union all
select 3180,'1955-A337','1955-05-05 00:00:00','military missile','failure',255,'623-22','Ramjet test RPTV','','','Test',253,'',20 from dual union all
select 31800,'1973-W1329','1973-10-23 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 31801,'1973-W133','1973-01-24 16:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 31802,'1973-W1330','1973-10-23 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 31803,'1973-W1331','1973-10-23 19:47:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',59 from dual union all
select 31804,'1973-W1332','1973-10-23 21:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 31805,'1973-W1333','1973-10-24 08:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 31806,'1973-W1334','1973-10-24 10:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 31807,'1973-W1335','1973-10-24 12:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31808,'1973-W1336','1973-10-24 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 31809,'1973-W1337','1973-10-24 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',54 from dual union all
select 3181,'1955-A338','1955-05-05 00:00:00','atmospheric rocket','success',836,'4445A','','','','Test',1895,'',10 from dual union all
select 31810,'1973-W1338','1973-10-24 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 31811,'1973-W1339','1973-10-24 14:46:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31812,'1973-W134','1973-01-24 16:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 31813,'1973-W1340','1973-10-24 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 31814,'1973-W1341','1973-10-25 00:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 31815,'1973-W1342','1973-10-25 16:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 31816,'1973-W1343','1973-10-25 21:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 31817,'1973-W1344','1973-10-26 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 31818,'1973-W1345','1973-10-26 14:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 31819,'1973-W1346','1973-10-26 15:21:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 3182,'1955-A339','1955-05-05 00:00:00','atmospheric rocket','success',836,'1888T','','','','Test',1895,'',10 from dual union all
select 31820,'1973-W1347','1973-10-26 17:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 31821,'1973-W1348','1973-10-26 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 31822,'1973-W1349','1973-10-26 17:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',71 from dual union all
select 31823,'1973-W135','1973-01-24 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 31824,'1973-W1350','1973-10-26 19:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 31825,'1973-W1351','1973-10-27 00:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 31826,'1973-W1352','1973-10-29 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 31827,'1973-W1353','1973-10-29 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 31828,'1973-W1354','1973-10-29 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 31829,'1973-W1355','1973-10-29 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 3183,'1955-A34','1955-01-20 00:00:00','atmospheric rocket','success',836,'1518T','','','','Test',1895,'',10 from dual union all
select 31830,'1973-W1356','1973-10-30 10:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 31831,'1973-W1357','1973-10-30 16:24:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',70 from dual union all
select 31832,'1973-W1358','1973-10-30 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 31833,'1973-W1359','1973-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 31834,'1973-W136','1973-01-24 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 31835,'1973-W1360','1973-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 31836,'1973-W1361','1973-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 31837,'1973-W1362','1973-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 31838,'1973-W1363','1973-11-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 31839,'1973-W1364','1973-11-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 3184,'1955-A340','1955-05-06 00:00:00','military missile','success',771,'GM-52-1895','','','','Test',272,'',12 from dual union all
select 31840,'1973-W1365','1973-11-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 31841,'1973-W1366','1973-11-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 31842,'1973-W1367','1973-10-31 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 31843,'1973-W1368','1973-10-31 14:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',54 from dual union all
select 31844,'1973-W1369','1973-10-31 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 31845,'1973-W137','1973-01-24 22:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 31846,'1973-W1370','1973-10-31 16:06:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 31847,'1973-W1371','1973-10-31 18:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 31848,'1973-W1372','1973-10-31 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31849,'1973-W1373','1973-10-31 19:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 3185,'1955-A341','1955-05-06 00:00:00','atmospheric rocket','success',836,'1920T','','','','Test',1895,'',10 from dual union all
select 31850,'1973-W1374','1973-10-31 20:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 31851,'1973-W1375','1973-11-01 18:09:00','suborbital rocket','success',1061,'','Starute,Density gauge','','','Meteo',1057,'',59 from dual union all
select 31852,'1973-W1376','1973-11-02 08:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',54 from dual union all
select 31853,'1973-W1377','1973-11-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 31854,'1973-W1378','1973-11-02 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 31855,'1973-W1379','1973-11-02 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 31856,'1973-W138','1973-01-24 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 31857,'1973-W1380','1973-11-02 16:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 31858,'1973-W1381','1973-11-02 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31859,'1973-W1382','1973-11-02 18:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 3186,'1955-A342','1955-05-06 00:00:00','atmospheric rocket','success',836,'4782A','','','','Test',1895,'',10 from dual union all
select 31860,'1973-W1383','1973-11-02 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',77 from dual union all
select 31861,'1973-W1384','1973-11-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',68 from dual union all
select 31862,'1973-W1385','1973-11-05 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 31863,'1973-W1386','1973-11-05 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 31864,'1973-W1387','1973-11-05 16:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 31865,'1973-W1388','1973-11-05 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 31866,'1973-W1389','1973-11-05 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 31867,'1973-W139','1973-01-25 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 31868,'1973-W1390','1973-11-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual;
insert into LAUNCH
select 31869,'1973-W1391','1973-11-07 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 3187,'1955-A343','1955-05-07 00:00:00','atmospheric rocket','success',836,'1625T','','','','Test',1895,'',10 from dual union all
select 31870,'1973-W1392','1973-11-07 15:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 31871,'1973-W1393','1973-11-07 16:20:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31872,'1973-W1394','1973-11-07 17:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 31873,'1973-W1395','1973-11-07 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31874,'1973-W1396','1973-11-07 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 31875,'1973-W1397','1973-11-08 00:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 31876,'1973-W1398','1973-11-08 10:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 31877,'1973-W1399','1973-11-08 16:06:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',63 from dual union all
select 31878,'1973-W14','1973-01-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 31879,'1973-W140','1973-01-26 03:48:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 3188,'1955-A344','1955-05-08 00:00:00','atmospheric rocket','success',836,'1615T','','','','Test',1895,'',10 from dual union all
select 31880,'1973-W1400','1973-11-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 31881,'1973-W1401','1973-11-09 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 31882,'1973-W1402','1973-11-09 14:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',56 from dual union all
select 31883,'1973-W1403','1973-11-09 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 31884,'1973-W1404','1973-11-09 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 31885,'1973-W1405','1973-11-09 18:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',76 from dual union all
select 31886,'1973-W1406','1973-11-09 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 31887,'1973-W1407','1973-11-09 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 31888,'1973-W1408','1973-11-12 15:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',56 from dual union all
select 31889,'1973-W1409','1973-11-12 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 3189,'1955-A345','1955-05-08 00:00:00','atmospheric rocket','success',836,'1616T','','','','Test',1895,'',10 from dual union all
select 31890,'1973-W141','1973-01-26 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 31891,'1973-W1410','1973-11-12 18:47:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 31892,'1973-W1411','1973-11-12 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 31893,'1973-W1412','1973-11-12 20:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 31894,'1973-W1413','1973-11-12 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 31895,'1973-W1414','1973-11-13 09:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 31896,'1973-W1415','1973-11-13 11:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 31897,'1973-W1416','1973-11-13 15:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 31898,'1973-W1417','1973-11-13 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 31899,'1973-W1418','1973-11-13 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 319,'1944-S29','1944-07-24 00:00:00','military missile','success',8,'','Last Blizna launch','','','Test',662,'',90 from dual union all
select 3190,'1955-A346','1955-05-08 00:00:00','atmospheric rocket','success',836,'1630T','','','','Test',1895,'',10 from dual union all
select 31900,'1973-W1419','1973-11-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',67 from dual union all
select 31901,'1973-W142','1973-01-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 31902,'1973-W1420','1973-11-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',70 from dual union all
select 31903,'1973-W1421','1973-11-14 14:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 31904,'1973-W1422','1973-11-14 15:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 31905,'1973-W1423','1973-11-14 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31906,'1973-W1424','1973-11-14 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',52 from dual union all
select 31907,'1973-W1425','1973-11-14 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31908,'1973-W1426','1973-11-14 19:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 31909,'1973-W1427','1973-11-14 21:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 3191,'1955-A347','1955-05-08 00:00:00','atmospheric rocket','success',836,'1601T','','','','Test',1895,'',10 from dual union all
select 31910,'1973-W1428','1973-11-15 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 31911,'1973-W1429','1973-11-15 15:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',55 from dual union all
select 31912,'1973-W143','1973-01-26 15:10:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',53 from dual union all
select 31913,'1973-W1430','1973-11-16 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 31914,'1973-W1431','1973-11-16 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',67 from dual union all
select 31915,'1973-W1432','1973-11-16 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',73 from dual union all
select 31916,'1973-W1433','1973-11-16 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 31917,'1973-W1434','1973-11-16 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 31918,'1973-W1435','1973-11-16 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31919,'1973-W1436','1973-11-16 16:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 3192,'1955-A348','1955-05-09 00:00:00','atmospheric rocket','success',836,'1600T','','','','Test',1895,'',10 from dual union all
select 31920,'1973-W1437','1973-11-16 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 31921,'1973-W1438','1973-11-16 21:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 31922,'1973-W1439','1973-11-16 22:33:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',76 from dual union all
select 31923,'1973-W144','1973-01-26 15:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',57 from dual union all
select 31924,'1973-W1440','1973-11-19 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',70 from dual union all
select 31925,'1973-W1441','1973-11-19 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 31926,'1973-W1442','1973-11-19 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 31927,'1973-W1443','1973-11-19 18:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 31928,'1973-W1444','1973-11-19 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 31929,'1973-W1445','1973-11-19 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 3193,'1955-A349','1955-05-10 00:00:00','atmospheric rocket','unknown',576,'1029RM','','','','Test',1895,'',10 from dual union all
select 31930,'1973-W1446','1973-11-19 23:26:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 31931,'1973-W1447','1973-11-20 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 31932,'1973-W1448','1973-11-20 19:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 31933,'1973-W1449','1973-11-20 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',51 from dual union all
select 31934,'1973-W145','1973-01-26 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 31935,'1973-W1450','1973-11-20 23:30:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31936,'1973-W1451','1973-11-21 08:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 31937,'1973-W1452','1973-11-21 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',69 from dual union all
select 31938,'1973-W1453','1973-11-21 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 31939,'1973-W1454','1973-11-21 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 3194,'1955-A35','1955-01-21 00:00:00','atmospheric rocket','unknown',576,'785M','','','','Test',1895,'',10 from dual union all
select 31940,'1973-W1455','1973-11-21 17:34:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',63 from dual union all
select 31941,'1973-W1456','1973-11-21 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 31942,'1973-W1457','1973-11-21 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 31943,'1973-W1458','1973-11-21 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 31944,'1973-W1459','1973-11-22 00:15:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 31945,'1973-W146','1973-01-26 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 31946,'1973-W1460','1973-11-23 15:20:00','suborbital rocket','success',88,'NASA 15.115GM','','','','Aeronomy',35,'',60 from dual union all
select 31947,'1973-W1461','1973-11-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',57 from dual union all
select 31948,'1973-W1462','1973-11-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',68 from dual union all
select 31949,'1973-W1463','1973-11-26 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 3195,'1955-A350','1955-05-10 00:00:00','atmospheric rocket','unknown',576,'1087RM','','','','Test',1895,'',10 from dual union all
select 31950,'1973-W1464','1973-11-26 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 31951,'1973-W1465','1973-11-26 17:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 31952,'1973-W1466','1973-11-26 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 31953,'1973-W1467','1973-11-26 19:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 31954,'1973-W1468','1973-11-27 10:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 31955,'1973-W1469','1973-11-27 12:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 31956,'1973-W147','1973-01-26 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 31957,'1973-W1470','1973-11-28 00:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 31958,'1973-W1471','1973-11-28 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual;
insert into LAUNCH
select 31959,'1973-W1472','1973-11-28 15:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 3196,'1955-A351','1955-05-10 00:00:00','military missile','success',771,'GM-53-203','','','','Test',222,'',12 from dual union all
select 31960,'1973-W1473','1973-11-28 15:46:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 31961,'1973-W1474','1973-11-28 16:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 31962,'1973-W1475','1973-11-28 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',54 from dual union all
select 31963,'1973-W1476','1973-11-28 21:33:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',69 from dual union all
select 31964,'1973-W1477','1973-11-28 21:45:00','suborbital rocket','success',1061,'','STS','','','Meteo',1895,'',62 from dual union all
select 31965,'1973-W1478','1973-11-28 22:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 31966,'1973-W1479','1973-11-29 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 31967,'1973-W148','1973-01-27 03:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 31968,'1973-W1480','1973-11-29 15:48:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31969,'1973-W1481','1973-11-29 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 3197,'1955-A352','1955-05-11 00:00:00','atmospheric rocket','success',836,'1875T','','','','Test',1895,'',10 from dual union all
select 31970,'1973-W1482','1973-12-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 31971,'1973-W1483','1973-12-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 31972,'1973-W1484','1973-12-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 31973,'1973-W1485','1973-12-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 31974,'1973-W1486','1973-11-30 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',75 from dual union all
select 31975,'1973-W1487','1973-11-30 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual union all
select 31976,'1973-W1488','1973-11-30 15:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 31977,'1973-W1489','1973-11-30 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 31978,'1973-W149','1973-01-29 06:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 31979,'1973-W1490','1973-11-30 16:26:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',60 from dual union all
select 3198,'1955-A353','1955-05-11 00:00:00','atmospheric rocket','success',836,'1868T','','','','Test',1895,'',10 from dual union all
select 31980,'1973-W1491','1973-11-30 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 31981,'1973-W1492','1973-11-30 20:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 31982,'1973-W1493','1973-11-30 21:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 31983,'1973-W1494','1973-12-03 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 31984,'1973-W1495','1973-12-03 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 31985,'1973-W1496','1973-12-03 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 31986,'1973-W1497','1973-12-03 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',69 from dual union all
select 31987,'1973-W1498','1973-12-03 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 31988,'1973-W1499','1973-12-03 20:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 31989,'1973-W15','1973-01-03 19:03:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',68 from dual union all
select 3199,'1955-A354','1955-05-11 00:00:00','atmospheric rocket','success',836,'1744T','','','','Test',1895,'',10 from dual union all
select 31990,'1973-W150','1973-01-29 14:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 31991,'1973-W1500','1973-12-04 15:33:00','suborbital rocket','success',88,'NASA L1-6963','Chaff','','','Meteo',1821,'',67 from dual union all
select 31992,'1973-W1501','1973-12-04 15:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 31993,'1973-W1502','1973-12-04 18:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 31994,'1973-W1503','1973-12-05 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 31995,'1973-W1504','1973-12-05 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 31996,'1973-W1505','1973-12-05 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 31997,'1973-W1506','1973-12-05 17:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 31998,'1973-W1507','1973-12-05 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 31999,'1973-W1508','1973-12-05 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 32,'1943-S10','1943-06-24 00:00:00','military missile','success',8,'30','','','','Test',675,'',95 from dual union all
select 320,'1944-S290','1944-12-13 11:45:00','military missile','success',1470,'19870','Ma320','','','Test',661,'',90 from dual union all
select 3200,'1955-A355','1955-05-11 00:00:00','atmospheric rocket','success',835,'','D22 Ramjet','','','Test',1821,'',10 from dual union all
select 32000,'1973-W1509','1973-12-05 19:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 32001,'1973-W151','1973-01-29 16:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 32002,'1973-W1510','1973-12-06 00:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 32003,'1973-W1511','1973-12-06 09:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 32004,'1973-W1512','1973-12-06 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 32005,'1973-W1513','1973-12-06 16:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 32006,'1973-W1514','1973-12-06 18:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 32007,'1973-W1515','1973-12-06 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',51 from dual union all
select 32008,'1973-W1516','1973-12-07 04:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 32009,'1973-W1517','1973-12-07 12:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 3201,'1955-A356','1955-05-12 00:00:00','atmospheric rocket','success',836,'1757T','','','','Test',1895,'',10 from dual union all
select 32010,'1973-W1518','1973-12-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',60 from dual union all
select 32011,'1973-W1519','1973-12-07 15:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32012,'1973-W152','1973-01-29 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 32013,'1973-W1520','1973-12-07 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 32014,'1973-W1521','1973-12-07 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32015,'1973-W1522','1973-12-07 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 32016,'1973-W1523','1973-12-07 19:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 32017,'1973-W1524','1973-12-07 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 32018,'1973-W1525','1973-12-07 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32019,'1973-W1526','1973-12-10 12:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 3202,'1955-A357','1955-05-12 00:00:00','atmospheric rocket','success',836,'4826A','','','','Test',1895,'',10 from dual union all
select 32020,'1973-W1527','1973-12-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',63 from dual union all
select 32021,'1973-W1528','1973-12-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',52 from dual union all
select 32022,'1973-W1529','1973-12-10 15:46:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 32023,'1973-W153','1973-01-29 20:24:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',54 from dual union all
select 32024,'1973-W1530','1973-12-10 17:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',65 from dual union all
select 32025,'1973-W1531','1973-12-10 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 32026,'1973-W1532','1973-12-10 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',55 from dual union all
select 32027,'1973-W1533','1973-12-10 19:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 32028,'1973-W1534','1973-12-10 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 32029,'1973-W1535','1973-12-11 15:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 3203,'1955-A358','1955-05-13 00:00:00','atmospheric rocket','success',836,'1906T','','','','Test',1895,'',10 from dual union all
select 32030,'1973-W1536','1973-12-11 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 32031,'1973-W1537','1973-12-11 17:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 32032,'1973-W1538','1973-12-11 19:30:00','suborbital rocket','success',1061,'','','','','Meteo',1895,'',64 from dual union all
select 32033,'1973-W1539','1973-12-12 03:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 32034,'1973-W154','1973-01-29 21:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 32035,'1973-W1540','1973-12-12 12:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',69 from dual union all
select 32036,'1973-W1541','1973-12-12 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 32037,'1973-W1542','1973-12-12 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 32038,'1973-W1543','1973-12-12 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 32039,'1973-W1544','1973-12-12 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 3204,'1955-A359','1955-05-13 00:00:00','military missile','success',1214,'N-69A GM-11112','','','','Test',224,'',10 from dual union all
select 32040,'1973-W1545','1973-12-12 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 32041,'1973-W1546','1973-12-12 20:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 32042,'1973-W1547','1973-12-12 20:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 32043,'1973-W1548','1973-12-12 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32044,'1973-W1549','1973-12-13 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 32045,'1973-W155','1973-01-29 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 32046,'1973-W1550','1973-12-13 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 32047,'1973-W1551','1973-12-13 19:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual;
insert into LAUNCH
select 32048,'1973-W1552','1973-12-13 19:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',73 from dual union all
select 32049,'1973-W1553','1973-12-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',69 from dual union all
select 3205,'1955-A36','1955-01-21 00:00:00','atmospheric rocket','unknown',576,'793M','','','','Test',1895,'',10 from dual union all
select 32050,'1973-W1554','1973-12-14 14:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 32051,'1973-W1555','1973-12-14 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 32052,'1973-W1556','1973-12-14 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 32053,'1973-W1557','1973-12-14 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 32054,'1973-W1558','1973-12-14 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 32055,'1973-W1559','1973-12-17 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',67 from dual union all
select 32056,'1973-W156','1973-01-30 18:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32057,'1973-W1560','1973-12-17 14:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',58 from dual union all
select 32058,'1973-W1561','1973-12-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 32059,'1973-W1562','1973-12-17 15:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 3206,'1955-A360','1955-05-14 00:00:00','atmospheric rocket','success',836,'1668T','','','','Test',1895,'',10 from dual union all
select 32060,'1973-W1563','1973-12-17 17:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 32061,'1973-W1564','1973-12-17 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 32062,'1973-W1565','1973-12-17 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 32063,'1973-W1566','1973-12-17 19:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 32064,'1973-W1567','1973-12-18 15:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32065,'1973-W1568','1973-12-18 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 32066,'1973-W1569','1973-12-18 20:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 32067,'1973-W157','1973-01-30 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 32068,'1973-W1570','1973-12-19 07:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 32069,'1973-W1571','1973-12-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',50 from dual union all
select 3207,'1955-A361','1955-05-14 00:00:00','atmospheric rocket','success',836,'1765T','','','','Test',1895,'',10 from dual union all
select 32070,'1973-W1572','1973-12-19 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 32071,'1973-W1573','1973-12-19 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 32072,'1973-W1574','1973-12-19 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 32073,'1973-W1575','1973-12-19 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32074,'1973-W1576','1973-12-20 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 32075,'1973-W1577','1973-12-20 15:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 32076,'1973-W1578','1973-12-20 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32077,'1973-W1579','1973-12-20 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 32078,'1973-W158','1973-01-30 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 32079,'1973-W1580','1973-12-21 10:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 3208,'1955-A362','1955-05-15 00:00:00','atmospheric rocket','success',836,'1612T','','','','Test',1895,'',10 from dual union all
select 32080,'1973-W1581','1973-12-21 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 32081,'1973-W1582','1973-12-21 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 32082,'1973-W1583','1973-12-21 18:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 32083,'1973-W1584','1973-12-21 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 32084,'1973-W1585','1973-12-21 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 32085,'1973-W1586','1973-12-22 04:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 32086,'1973-W1587','1973-12-24 11:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 32087,'1973-W1588','1973-12-24 12:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 32088,'1973-W1589','1973-12-24 13:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 32089,'1973-W159','1973-01-31 14:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 3209,'1955-A363','1955-05-15 00:00:00','atmospheric rocket','success',836,'1562T','','','','Test',1895,'',10 from dual union all
select 32090,'1973-W1590','1973-12-24 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 32091,'1973-W1591','1973-12-24 15:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 32092,'1973-W1592','1973-12-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',55 from dual union all
select 32093,'1973-W1593','1973-12-26 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 32094,'1973-W1594','1973-12-27 15:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 32095,'1973-W1595','1973-12-27 16:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32096,'1973-W1596','1973-12-27 19:18:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 32097,'1973-W1597','1973-12-28 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32098,'1973-W1598','1973-12-28 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 32099,'1973-W1599','1973-12-28 20:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 321,'1944-S291','1944-12-15 12:18:00','military missile','success',1470,'19847','Ma309','','','Test',661,'',90 from dual union all
select 3210,'1955-A364','1955-05-17 00:00:00','military missile','unknown',339,'1411R','','','','Test',1895,'',40 from dual union all
select 32100,'1973-W16','1973-01-03 19:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 32101,'1973-W160','1973-01-31 14:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 32102,'1973-W1600','1973-12-28 21:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 32103,'1973-W1601','1973-09-20 13:54:00','suborbital rocket','success',1281,'CIMO','DMN Sonde','','','Meteo',350,'',73 from dual union all
select 32104,'1973-W1602','1973-09-21 03:32:00','suborbital rocket','success',1281,'CIMO','DMN Sonde','','','Meteo',350,'',74 from dual union all
select 32105,'1973-W1603','1973-09-22 04:37:00','suborbital rocket','success',1281,'CIMO','DMN Sonde','','','Meteo',350,'',74 from dual union all
select 32106,'1973-W1604','1973-09-24 17:15:00','suborbital rocket','success',1281,'CIMO','DMN Sonde','','','Meteo',350,'',73 from dual union all
select 32107,'1973-W1605','1973-09-25 00:56:00','suborbital rocket','success',1281,'CIMO','DMN Sonde','','','Meteo',350,'',77 from dual union all
select 32108,'1973-W1606','1973-09-25 18:40:00','suborbital rocket','success',1281,'CIMO','DMN Sonde','','','Meteo',350,'',70 from dual union all
select 32109,'1973-W1607','1973-09-26 01:06:00','suborbital rocket','success',1281,'CIMO','DMN Sonde','','','Meteo',350,'',70 from dual union all
select 3211,'1955-A365','1955-05-17 00:00:00','military missile','success',771,'GM-52-1902','','','','Test',222,'',12 from dual union all
select 32110,'1973-W1608','1973-09-26 14:42:00','suborbital rocket','success',1281,'CIMO','DMN Sonde','','','Meteo',350,'',72 from dual union all
select 32111,'1973-W1609','1973-09-27 14:26:00','suborbital rocket','success',1281,'CIMO','DMN Sonde','','','Meteo',350,'',73 from dual union all
select 32112,'1973-W161','1973-01-31 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 32113,'1973-W1610','1973-09-28 03:11:00','suborbital rocket','success',1281,'CIMO','DMN Sonde','','','Meteo',350,'',72 from dual union all
select 32114,'1973-W1611','1973-09-29 03:29:00','suborbital rocket','success',1281,'CIMO','DMN Sonde','','','Meteo',350,'',73 from dual union all
select 32115,'1973-W1612','1973-10-01 04:08:00','suborbital rocket','success',1281,'CIMO','DMN Sonde','','','Meteo',350,'',73 from dual union all
select 32116,'1973-W1613','1973-09-20 14:40:00','suborbital rocket','success',725,'CIMO','','','','Meteo',348,'KORO',77 from dual union all
select 32117,'1973-W1614','1973-09-21 15:20:00','suborbital rocket','success',725,'CIMO','','','','Meteo',348,'KORO',80 from dual union all
select 32118,'1973-W1615','1973-09-22 04:10:00','suborbital rocket','success',725,'CIMO','','','','Meteo',348,'KORO',68 from dual union all
select 32119,'1973-W1616','1973-09-24 17:20:00','suborbital rocket','success',725,'CIMO','','','','Meteo',348,'KORO',79 from dual union all
select 3212,'1955-A366','1955-05-17 00:00:00','atmospheric rocket','success',836,'1538T','','','','Test',1895,'',10 from dual union all
select 32120,'1973-W1617','1973-09-25 00:30:00','suborbital rocket','success',725,'CIMO','','','','Meteo',348,'KORO',78 from dual union all
select 32121,'1973-W1618','1973-09-25 18:10:00','suborbital rocket','success',725,'CIMO','','','','Meteo',348,'KORO',77 from dual union all
select 32122,'1973-W1619','1973-09-26 00:20:00','suborbital rocket','success',725,'CIMO','','','','Meteo',348,'KORO',78 from dual union all
select 32123,'1973-W162','1973-01-31 18:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 32124,'1973-W1620','1973-09-29 03:01:00','suborbital rocket','success',725,'CIMO','','','','Meteo',348,'KORO',78 from dual union all
select 32125,'1973-W1621','1973-09-21 03:44:00','suborbital rocket','success',1188,'CIMO','BMO Sonde','','','Meteo',350,'',50 from dual union all
select 32126,'1973-W1622','1973-09-21 15:39:00','suborbital rocket','success',1188,'CIMO','BMO Sonde','','','Meteo',350,'',65 from dual union all
select 32127,'1973-W1623','1973-09-22 04:31:00','suborbital rocket','success',1188,'CIMO','BMO Sonde','','','Meteo',350,'',69 from dual union all
select 32128,'1973-W1624','1973-09-25 00:51:00','suborbital rocket','success',1188,'CIMO','BMO Sonde','','','Meteo',350,'',70 from dual union all
select 32129,'1973-W1625','1973-09-25 18:35:00','suborbital rocket','success',1188,'CIMO','BMO Sonde','','','Meteo',350,'',70 from dual union all
select 3213,'1955-A367','1955-05-17 00:00:00','atmospheric rocket','success',836,'1848T','','','','Test',1895,'',10 from dual union all
select 32130,'1973-W1626','1973-09-26 01:01:00','suborbital rocket','success',1188,'CIMO','BMO Sonde','','','Meteo',350,'',64 from dual union all
select 32131,'1973-W1627','1973-09-26 14:35:00','suborbital rocket','success',1188,'CIMO','BMO Sonde','','','Meteo',350,'',65 from dual union all
select 32132,'1973-W1628','1973-09-27 14:20:00','suborbital rocket','success',1188,'CIMO','BMO Sonde','','','Meteo',350,'',66 from dual union all
select 32133,'1973-W1629','1973-09-28 03:06:00','suborbital rocket','success',1188,'CIMO','BMO Sonde','','','Meteo',350,'',70 from dual union all
select 32134,'1973-W163','1973-01-31 19:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',57 from dual union all
select 32135,'1973-W1630','1973-09-29 03:24:00','suborbital rocket','success',1188,'CIMO','BMO Sonde','','','Meteo',350,'',88 from dual union all
select 32136,'1973-W1631','1973-10-01 04:02:00','suborbital rocket','success',1188,'CIMO','BMO Sonde','','','Meteo',350,'',53 from dual union all
select 32137,'1973-W1633','1973-09-20 13:49:00','suborbital rocket','success',1284,'CIMO','Datasonde','','','Meteo',350,'',69 from dual;
insert into LAUNCH
select 32138,'1973-W1634','1973-09-21 03:28:00','suborbital rocket','success',1284,'CIMO','Datasonde','','','Meteo',350,'',73 from dual union all
select 32139,'1973-W1635','1973-09-21 15:13:00','suborbital rocket','success',1284,'CIMO','Datasonde','','','Meteo',350,'',58 from dual union all
select 3214,'1955-A369','1955-05-17 00:00:00','atmospheric rocket','success',836,'1553T','','','','Test',1895,'',10 from dual union all
select 32140,'1973-W1636','1973-09-22 04:02:00','suborbital rocket','success',1284,'CIMO','Datasonde','','','Meteo',350,'',74 from dual union all
select 32141,'1973-W1637','1973-09-24 17:10:00','suborbital rocket','success',1284,'CIMO','Datasonde','','','Meteo',350,'',70 from dual union all
select 32142,'1973-W1638','1973-09-25 00:19:00','suborbital rocket','success',1284,'CIMO','Datasonde','','','Meteo',350,'',74 from dual union all
select 32143,'1973-W1639','1973-09-25 18:00:00','suborbital rocket','success',1284,'CIMO','Datasonde','','','Meteo',350,'',70 from dual union all
select 32144,'1973-W164','1973-01-31 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 32145,'1973-W1640','1973-09-26 01:39:00','suborbital rocket','success',1284,'CIMO','Datasonde','','','Meteo',350,'',68 from dual union all
select 32146,'1973-W1641','1973-09-26 14:04:00','suborbital rocket','success',1284,'CIMO','Datasonde','','','Meteo',350,'',67 from dual union all
select 32147,'1973-W1642','1973-09-27 14:58:00','suborbital rocket','success',1284,'CIMO','Datasonde','','','Meteo',350,'',74 from dual union all
select 32148,'1973-W1643','1973-09-28 02:33:00','suborbital rocket','success',1284,'CIMO','Datasonde','','','Meteo',350,'',73 from dual union all
select 32149,'1973-W1644','1973-09-29 02:52:00','suborbital rocket','success',1284,'CIMO','Datasonde','','','Meteo',350,'',73 from dual union all
select 3215,'1955-A37','1955-01-21 00:00:00','atmospheric rocket','unknown',576,'794M','','','','Test',1895,'',10 from dual union all
select 32150,'1973-W1645','1973-09-26 15:00:00','suborbital rocket','success',1216,'','Diurnal','','','Meteo',330,'',60 from dual union all
select 32151,'1973-W1646','1973-09-26 16:30:00','suborbital rocket','success',1216,'','Diurnal','','','Meteo',330,'',60 from dual union all
select 32152,'1973-W1647','1973-09-26 21:00:00','suborbital rocket','success',1216,'','Diurnal','','','Meteo',330,'',60 from dual union all
select 32153,'1973-W1648','1973-09-26 00:00:00','suborbital rocket','success',1216,'','Diurnal','','','Meteo',330,'',60 from dual union all
select 32154,'1973-W1649','1973-09-27 00:00:00','suborbital rocket','success',1216,'','Diurnal','','','Meteo',330,'',60 from dual union all
select 32155,'1973-W165','1973-01-31 21:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32156,'1973-W1650','1973-09-27 05:00:00','suborbital rocket','success',1216,'','Diurnal','','','Meteo',330,'',60 from dual union all
select 32157,'1973-W1651','1973-09-27 07:30:00','suborbital rocket','success',1216,'','Diurnal','','','Meteo',330,'',60 from dual union all
select 32158,'1973-W1652','1973-09-27 13:00:00','suborbital rocket','success',1216,'','Diurnal','','','Meteo',330,'',60 from dual union all
select 32159,'1973-W1653','1973-02-02 00:00:00','suborbital rocket','success',88,'USA Ozone','Arcas Starute 7/Ozo','','','Aeron/Test',1895,'',64 from dual union all
select 3216,'1955-A370','1955-05-19 00:00:00','military missile','success',771,'GM-53-98','','','','Test',222,'',12 from dual union all
select 32160,'1973-W1654','1973-10-01 03:30:00','suborbital rocket','success',1284,'CIMO','Datasonde','','','Meteo',350,'',73 from dual union all
select 32161,'1973-W166','1973-01-31 23:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 32162,'1973-W167','1973-02-01 02:24:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32163,'1973-W168','1973-02-01 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 32164,'1973-W169','1973-02-01 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 32165,'1973-W17','1973-01-03 21:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32166,'1973-W170','1973-02-01 21:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 32167,'1973-W171','1973-02-02 01:18:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 32168,'1973-W172','1973-02-02 06:15:00','suborbital rocket','success',1281,'NASA 15.101UI','','','','Ionos',428,'',64 from dual union all
select 32169,'1973-W173','1973-02-02 14:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 3217,'1955-A371','1955-05-19 00:00:00','atmospheric rocket','success',836,'1814T','','','','Test',1895,'',10 from dual union all
select 32170,'1973-W174','1973-02-02 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 32171,'1973-W175','1973-02-02 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 32172,'1973-W176','1973-02-02 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 32173,'1973-W177','1973-02-02 20:21:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',65 from dual union all
select 32174,'1973-W178','1973-02-02 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32175,'1973-W179','1973-02-02 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 32176,'1973-W18','1973-01-03 23:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 32177,'1973-W180','1973-02-03 01:18:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',69 from dual union all
select 32178,'1973-W181','1973-02-05 01:52:00','suborbital rocket','success',761,'MT135P T5','','','','Aeron?',1476,'',58 from dual union all
select 32179,'1973-W182','1973-02-05 16:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',73 from dual union all
select 3218,'1955-A372','1955-05-19 00:00:00','atmospheric rocket','success',836,'1575T','','','','Test',1895,'',10 from dual union all
select 32180,'1973-W183','1973-02-05 18:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 32181,'1973-W184','1973-02-05 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 32182,'1973-W185','1973-02-05 20:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32183,'1973-W186','1973-02-05 23:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32184,'1973-W187','1973-02-05 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',62 from dual union all
select 32185,'1973-W188','1973-02-06 00:51:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 32186,'1973-W189','1973-02-06 14:30:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',61 from dual union all
select 32187,'1973-W19','1973-01-04 15:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 32188,'1973-W190','1973-02-06 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',73 from dual union all
select 32189,'1973-W191','1973-02-06 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 3219,'1955-A373','1955-05-23 00:00:00','military missile','unknown',339,'1477R','','','','Test',1895,'',40 from dual union all
select 32190,'1973-W192','1973-02-06 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 32191,'1973-W193','1973-02-07 00:45:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32192,'1973-W194','1973-02-07 12:07:00','suborbital rocket','success',716,'INTA HW-7302','','','','Meteo',396,'',55 from dual union all
select 32193,'1973-W195','1973-02-07 14:55:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1470,'',62 from dual union all
select 32194,'1973-W196','1973-02-07 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 32195,'1973-W197','1973-02-07 17:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',70 from dual union all
select 32196,'1973-W198','1973-02-07 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 32197,'1973-W199','1973-02-07 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 32198,'1973-W20','1973-01-04 16:54:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',58 from dual union all
select 32199,'1973-W200','1973-02-07 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 322,'1944-S292','1944-12-16 11:29:00','military missile','success',1470,'19837','Ma317','','','Test',661,'',90 from dual union all
select 3220,'1955-A374','1955-05-23 18:05:00','atmospheric rocket','success',1524,'Q-1','','','','Test',253,'',10 from dual union all
select 32200,'1973-W201','1973-02-07 21:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 32201,'1973-W202','1973-02-07 23:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 32202,'1973-W203','1973-02-08 01:30:00','suborbital rocket','success',761,'MT135P T6','','','','Aeron?',1476,'',50 from dual union all
select 32203,'1973-W204','1973-02-08 01:42:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 32204,'1973-W205','1973-02-08 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',64 from dual union all
select 32205,'1973-W206','1973-02-08 14:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 32206,'1973-W207','1973-02-08 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 32207,'1973-W208','1973-02-08 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 32208,'1973-W209','1973-02-08 21:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 32209,'1973-W21','1973-01-04 17:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 3221,'1955-A375','1955-05-23 19:44:00','atmospheric rocket','success',1524,'Q-2','','','','Test',253,'',11 from dual union all
select 32210,'1973-W210','1973-02-09 03:56:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32211,'1973-W211','1973-02-09 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 32212,'1973-W212','1973-02-09 14:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',62 from dual union all
select 32213,'1973-W213','1973-02-09 15:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32214,'1973-W214','1973-02-09 18:03:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32215,'1973-W215','1973-02-09 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 32216,'1973-W216','1973-02-09 20:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 32217,'1973-W217','1973-02-09 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32218,'1973-W218','1973-02-09 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',52 from dual union all
select 32219,'1973-W219','1973-02-12 14:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 3222,'1955-A376','1955-05-24 00:00:00','atmospheric rocket','unknown',576,'1028RM','','','','Test',1895,'',10 from dual union all
select 32220,'1973-W22','1973-01-04 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 32221,'1973-W220','1973-02-12 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 32222,'1973-W221','1973-02-12 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',53 from dual union all
select 32223,'1973-W222','1973-02-12 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 32224,'1973-W223','1973-02-12 18:40:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',61 from dual union all
select 32225,'1973-W224','1973-02-12 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 32226,'1973-W225','1973-02-12 20:32:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',54 from dual union all
select 32227,'1973-W226','1973-02-12 20:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual;
insert into LAUNCH
select 32228,'1973-W227','1973-02-12 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32229,'1973-W228','1973-02-12 23:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 3223,'1955-A377','1955-05-24 00:00:00','atmospheric rocket','unknown',576,'1018RM','','','','Test',1895,'',10 from dual union all
select 32230,'1973-W229','1973-02-13 15:30:00','suborbital rocket','success',88,'NASA 15.107GM','','','','Aeronomy',35,'',58 from dual union all
select 32231,'1973-W23','1973-01-04 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 32232,'1973-W230','1973-02-13 15:45:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 32233,'1973-W231','1973-02-13 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 32234,'1973-W232','1973-02-13 18:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32235,'1973-W233','1973-02-13 19:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 32236,'1973-W234','1973-02-13 21:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32237,'1973-W235','1973-02-14 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',62 from dual union all
select 32238,'1973-W236','1973-02-14 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',53 from dual union all
select 32239,'1973-W237','1973-02-14 15:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',60 from dual union all
select 3224,'1955-A378','1955-05-24 00:00:00','atmospheric rocket','success',836,'1799T','','','','Test',1895,'',10 from dual union all
select 32240,'1973-W238','1973-02-14 15:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 32241,'1973-W239','1973-02-14 16:14:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 32242,'1973-W24','1973-01-05 15:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 32243,'1973-W240','1973-02-14 16:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 32244,'1973-W241','1973-02-14 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 32245,'1973-W242','1973-02-14 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 32246,'1973-W243','1973-02-14 19:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',54 from dual union all
select 32247,'1973-W244','1973-02-14 19:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',50 from dual union all
select 32248,'1973-W245','1973-02-14 21:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 32249,'1973-W246','1973-02-14 23:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 3225,'1955-A379','1955-05-24 00:00:00','atmospheric rocket','success',836,'1619T','','','','Test',1895,'',10 from dual union all
select 32250,'1973-W247','1973-02-15 16:19:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32251,'1973-W248','1973-02-15 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 32252,'1973-W249','1973-02-15 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 32253,'1973-W25','1973-01-05 15:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 32254,'1973-W250','1973-02-15 21:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 32255,'1973-W251','1973-02-16 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 32256,'1973-W252','1973-02-16 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 32257,'1973-W253','1973-02-16 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32258,'1973-W254','1973-02-16 18:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32259,'1973-W255','1973-02-16 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 3226,'1955-A38','1955-01-21 00:00:00','atmospheric rocket','unknown',576,'783M','','','','Test',1895,'',10 from dual union all
select 32260,'1973-W256','1973-02-16 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 32261,'1973-W257','1973-02-19 15:56:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 32262,'1973-W258','1973-02-20 14:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 32263,'1973-W259','1973-02-20 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 32264,'1973-W26','1973-01-05 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 32265,'1973-W260','1973-02-20 15:25:00','suborbital rocket','success',88,'NASA 15.108GM','','','','Aeronomy',35,'',53 from dual union all
select 32266,'1973-W261','1973-02-20 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 32267,'1973-W262','1973-02-20 18:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32268,'1973-W263','1973-02-20 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 32269,'1973-W264','1973-02-21 04:10:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 3227,'1955-A380','1955-05-24 00:00:00','atmospheric rocket','success',836,'1683T','','','','Test',1895,'',10 from dual union all
select 32270,'1973-W265','1973-02-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 32271,'1973-W266','1973-02-21 15:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 32272,'1973-W267','1973-02-21 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',78 from dual union all
select 32273,'1973-W268','1973-02-21 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32274,'1973-W269','1973-02-21 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 32275,'1973-W27','1973-01-05 17:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',73 from dual union all
select 32276,'1973-W270','1973-02-21 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 32277,'1973-W271','1973-02-21 22:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 32278,'1973-W272','1973-02-21 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 32279,'1973-W273','1973-02-22 01:18:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',69 from dual union all
select 3228,'1955-A381','1955-05-24 00:00:00','atmospheric rocket','success',836,'5251A','','','','Test',1895,'',10 from dual union all
select 32280,'1973-W274','1973-02-22 18:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32281,'1973-W275','1973-02-22 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 32282,'1973-W276','1973-02-22 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 32283,'1973-W277','1973-02-23 03:08:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',69 from dual union all
select 32284,'1973-W278','1973-02-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 32285,'1973-W279','1973-02-23 15:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 32286,'1973-W28','1973-01-05 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32287,'1973-W280','1973-02-23 15:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 32288,'1973-W281','1973-02-23 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32289,'1973-W282','1973-02-23 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 3229,'1955-A382','1955-05-24 00:00:00','atmospheric rocket','success',836,'1677T','','','','Test',1895,'',10 from dual union all
select 32290,'1973-W283','1973-02-23 20:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 32291,'1973-W284','1973-02-23 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32292,'1973-W285','1973-02-23 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 32293,'1973-W286','1973-02-24 03:48:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 32294,'1973-W287','1973-02-26 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',79 from dual union all
select 32295,'1973-W288','1973-02-26 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 32296,'1973-W289','1973-02-26 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 32297,'1973-W29','1973-01-05 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 32298,'1973-W290','1973-02-26 20:37:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32299,'1973-W291','1973-02-26 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 323,'1944-S293','1944-12-16 12:11:00','military missile','success',1470,'19838','Ma314','','','Test',661,'',90 from dual union all
select 3230,'1955-A383','1955-05-25 00:00:00','atmospheric rocket','success',836,'4438R','Nike 454R','','','Test',1895,'',10 from dual union all
select 32300,'1973-W292','1973-02-27 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 32301,'1973-W293','1973-02-27 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',68 from dual union all
select 32302,'1973-W294','1973-02-27 20:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',54 from dual union all
select 32303,'1973-W295','1973-02-27 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32304,'1973-W296','1973-02-27 23:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',65 from dual union all
select 32305,'1973-W297','1973-02-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 32306,'1973-W298','1973-02-28 15:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 32307,'1973-W299','1973-02-28 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',56 from dual union all
select 32308,'1973-W30','1973-01-05 21:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32309,'1973-W300','1973-02-28 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',76 from dual union all
select 3231,'1955-A384','1955-05-25 00:00:00','atmospheric rocket','success',836,'1544T','','','','Test',1895,'',10 from dual union all
select 32310,'1973-W301','1973-02-28 19:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',70 from dual union all
select 32311,'1973-W302','1973-02-28 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32312,'1973-W303','1973-02-28 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',65 from dual union all
select 32313,'1973-W304','1973-03-01 04:07:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 32314,'1973-W305','1973-03-01 14:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',57 from dual union all
select 32315,'1973-W306','1973-03-01 17:57:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 32316,'1973-W307','1973-03-01 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 32317,'1973-W308','1973-03-01 21:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual;
insert into LAUNCH
select 32318,'1973-W309','1973-03-02 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 32319,'1973-W31','1973-01-06 00:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 3232,'1955-A385','1955-05-25 00:00:00','atmospheric rocket','success',836,'1569T','','','','Test',1895,'',10 from dual union all
select 32320,'1973-W310','1973-03-02 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 32321,'1973-W311','1973-03-02 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 32322,'1973-W312','1973-03-02 15:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32323,'1973-W313','1973-03-02 15:47:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 32324,'1973-W314','1973-03-02 19:10:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 32325,'1973-W315','1973-03-02 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 32326,'1973-W316','1973-03-02 21:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32327,'1973-W317','1973-03-02 23:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',65 from dual union all
select 32328,'1973-W318','1973-03-05 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',50 from dual union all
select 32329,'1973-W319','1973-03-05 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 3233,'1955-A386','1955-05-25 00:00:00','atmospheric rocket','success',836,'1632T','','','','Test',1895,'',10 from dual union all
select 32330,'1973-W32','1973-01-06 17:52:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 32331,'1973-W320','1973-03-05 15:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32332,'1973-W321','1973-03-05 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 32333,'1973-W322','1973-03-05 17:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32334,'1973-W323','1973-03-05 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 32335,'1973-W324','1973-03-05 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32336,'1973-W325','1973-03-05 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',66 from dual union all
select 32337,'1973-W326','1973-03-06 16:17:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 32338,'1973-W327','1973-03-06 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 32339,'1973-W328','1973-03-06 19:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 3234,'1955-A387','1955-05-25 00:00:00','atmospheric rocket','success',836,'1637T','','','','Test',1895,'',10 from dual union all
select 32340,'1973-W329','1973-03-06 21:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32341,'1973-W33','1973-01-07 17:57:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 32342,'1973-W330','1973-03-07 02:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',76 from dual union all
select 32343,'1973-W331','1973-03-07 13:04:59','suborbital rocket','success',716,'INTA HW-7303','','','','Meteo',396,'',57 from dual union all
select 32344,'1973-W332','1973-03-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 32345,'1973-W333','1973-03-07 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 32346,'1973-W334','1973-03-07 15:14:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',79 from dual union all
select 32347,'1973-W335','1973-03-07 15:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 32348,'1973-W336','1973-03-07 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 32349,'1973-W337','1973-03-07 16:55:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',63 from dual union all
select 3235,'1955-A388','1955-05-25 00:00:00','atmospheric rocket','success',836,'4688R','Nike 455R','','','Test',1895,'',10 from dual union all
select 32350,'1973-W338','1973-03-07 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 32351,'1973-W339','1973-03-07 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 32352,'1973-W34','1973-01-08 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',54 from dual union all
select 32353,'1973-W340','1973-03-07 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32354,'1973-W341','1973-03-08 00:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 32355,'1973-W342','1973-03-08 13:49:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',78 from dual union all
select 32356,'1973-W343','1973-03-08 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 32357,'1973-W344','1973-03-08 16:30:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 32358,'1973-W345','1973-03-08 21:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32359,'1973-W346','1973-03-09 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 3236,'1955-A389','1955-05-25 00:00:00','atmospheric rocket','success',836,'4437R','Nike 453R','','','Test',1895,'',10 from dual union all
select 32360,'1973-W347','1973-03-09 15:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 32361,'1973-W348','1973-03-09 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 32362,'1973-W349','1973-03-09 17:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32363,'1973-W35','1973-01-08 16:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',74 from dual union all
select 32364,'1973-W350','1973-03-09 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',78 from dual union all
select 32365,'1973-W351','1973-03-09 19:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 32366,'1973-W352','1973-03-09 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 32367,'1973-W353','1973-03-12 14:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 32368,'1973-W354','1973-03-12 15:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',79 from dual union all
select 32369,'1973-W355','1973-03-12 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',69 from dual union all
select 3237,'1955-A39','1955-01-21 00:00:00','atmospheric rocket','success',836,'1486T','','','','Test',1895,'',10 from dual union all
select 32370,'1973-W356','1973-03-12 15:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 32371,'1973-W357','1973-03-12 16:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 32372,'1973-W358','1973-03-12 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 32373,'1973-W359','1973-03-12 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 32374,'1973-W36','1973-01-08 16:30:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32375,'1973-W360','1973-03-12 20:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 32376,'1973-W361','1973-03-12 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32377,'1973-W362','1973-03-12 22:08:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32378,'1973-W363','1973-03-13 16:14:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 32379,'1973-W364','1973-03-13 20:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 3238,'1955-A390','1955-05-25 00:00:00','atmospheric rocket','success',836,'1817T','','','','Test',1895,'',10 from dual union all
select 32380,'1973-W365','1973-03-14 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 32381,'1973-W366','1973-03-14 15:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 32382,'1973-W367','1973-03-14 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 32383,'1973-W368','1973-03-14 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 32384,'1973-W369','1973-03-14 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',75 from dual union all
select 32385,'1973-W37','1973-01-08 17:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32386,'1973-W370','1973-03-14 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 32387,'1973-W371','1973-03-14 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 32388,'1973-W372','1973-03-14 20:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32389,'1973-W373','1973-03-14 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 3239,'1955-A391','1955-05-26 00:00:00','atmospheric rocket','success',835,'','10 deg nosecone','','','Test',1821,'',10 from dual union all
select 32390,'1973-W374','1973-03-15 00:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',63 from dual union all
select 32391,'1973-W375','1973-03-15 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 32392,'1973-W376','1973-03-15 15:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',62 from dual union all
select 32393,'1973-W377','1973-03-15 16:34:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 32394,'1973-W378','1973-03-15 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 32395,'1973-W379','1973-03-15 21:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 32396,'1973-W38','1973-01-08 17:48:59','suborbital rocket','success',1061,'','Starute','','','Meteo',1057,'',69 from dual union all
select 32397,'1973-W380','1973-03-16 14:00:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',52 from dual union all
select 32398,'1973-W381','1973-03-16 15:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 32399,'1973-W382','1973-03-16 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 324,'1944-S294','1944-12-17 09:58:00','military missile','success',1470,'19851','Ma273','','','Test',661,'',90 from dual union all
select 3240,'1955-A392','1955-05-26 00:00:00','atmospheric rocket','success',836,'4852A','','','','Test',1895,'',10 from dual union all
select 32400,'1973-W383','1973-03-16 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 32401,'1973-W384','1973-03-16 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 32402,'1973-W385','1973-03-16 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 32403,'1973-W386','1973-03-16 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32404,'1973-W387','1973-03-16 23:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 32405,'1973-W388','1973-03-19 15:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32406,'1973-W389','1973-03-19 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 32407,'1973-W39','1973-01-08 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual;
insert into LAUNCH
select 32408,'1973-W390','1973-03-19 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',75 from dual union all
select 32409,'1973-W391','1973-03-19 18:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 3241,'1955-A393','1955-05-26 00:00:00','atmospheric rocket','success',836,'1820T','','','','Test',1895,'',10 from dual union all
select 32410,'1973-W392','1973-03-19 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',75 from dual union all
select 32411,'1973-W393','1973-03-19 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 32412,'1973-W394','1973-03-19 20:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 32413,'1973-W395','1973-03-19 21:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32414,'1973-W396','1973-03-20 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 32415,'1973-W397','1973-03-20 16:30:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 32416,'1973-W398','1973-03-20 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',76 from dual union all
select 32417,'1973-W399','1973-03-20 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 32418,'1973-W40','1973-01-08 18:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',52 from dual union all
select 32419,'1973-W400','1973-03-20 21:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 3242,'1955-A394','1955-05-26 00:00:00','atmospheric rocket','success',836,'5253A','','','','Test',1895,'',10 from dual union all
select 32420,'1973-W401','1973-03-21 14:40:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',63 from dual union all
select 32421,'1973-W402','1973-03-21 15:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 32422,'1973-W403','1973-03-21 15:53:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 32423,'1973-W404','1973-03-21 16:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 32424,'1973-W405','1973-03-21 17:42:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',50 from dual union all
select 32425,'1973-W406','1973-03-21 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',75 from dual union all
select 32426,'1973-W407','1973-03-21 19:10:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',69 from dual union all
select 32427,'1973-W408','1973-03-21 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32428,'1973-W409','1973-03-22 16:34:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32429,'1973-W41','1973-01-08 20:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 3243,'1955-A395','1955-05-26 00:00:00','atmospheric rocket','success',836,'2788E','','','','Test',1895,'',10 from dual union all
select 32430,'1973-W410','1973-03-22 19:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 32431,'1973-W411','1973-03-22 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32432,'1973-W412','1973-03-23 10:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 32433,'1973-W413','1973-03-23 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',71 from dual union all
select 32434,'1973-W414','1973-03-23 15:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 32435,'1973-W415','1973-03-23 15:20:00','suborbital rocket','success',1061,'','Chaff,Datasonde','','','Meteo',1470,'',50 from dual union all
select 32436,'1973-W416','1973-03-23 15:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 32437,'1973-W417','1973-03-23 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 32438,'1973-W418','1973-03-23 17:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',56 from dual union all
select 32439,'1973-W419','1973-03-23 19:12:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',63 from dual union all
select 3244,'1955-A396','1955-05-26 00:00:00','atmospheric rocket','success',836,'2742E','','','','Test',1895,'',10 from dual union all
select 32440,'1973-W42','1973-01-08 21:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32441,'1973-W420','1973-03-23 19:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 32442,'1973-W421','1973-03-23 21:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 32443,'1973-W422','1973-03-23 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 32444,'1973-W423','1973-03-26 14:10:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',52 from dual union all
select 32445,'1973-W424','1973-03-26 14:15:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',69 from dual union all
select 32446,'1973-W425','1973-03-26 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 32447,'1973-W426','1973-03-26 15:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 32448,'1973-W427','1973-03-26 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 32449,'1973-W428','1973-03-26 19:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 3245,'1955-A397','1955-05-27 00:00:00','atmospheric rocket','unknown',576,'1035RO','','','','Test',1895,'',10 from dual union all
select 32450,'1973-W429','1973-03-26 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',77 from dual union all
select 32451,'1973-W43','1973-01-08 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',64 from dual union all
select 32452,'1973-W430','1973-03-26 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32453,'1973-W431','1973-03-27 00:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',65 from dual union all
select 32454,'1973-W432','1973-03-27 04:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 32455,'1973-W433','1973-03-27 09:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 32456,'1973-W434','1973-03-27 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 32457,'1973-W435','1973-03-27 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 32458,'1973-W436','1973-03-27 19:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 32459,'1973-W437','1973-03-27 21:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 3246,'1955-A398','1955-05-27 00:00:00','atmospheric rocket','unknown',576,'1056RO','','','','Test',1895,'',10 from dual union all
select 32460,'1973-W438','1973-03-28 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 32461,'1973-W439','1973-03-28 14:25:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',67 from dual union all
select 32462,'1973-W44','1973-01-09 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',53 from dual union all
select 32463,'1973-W440','1973-03-28 14:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',61 from dual union all
select 32464,'1973-W441','1973-03-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 32465,'1973-W442','1973-03-28 15:45:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 32466,'1973-W443','1973-03-28 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 32467,'1973-W444','1973-03-28 17:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',59 from dual union all
select 32468,'1973-W445','1973-03-28 19:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 32469,'1973-W446','1973-03-28 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 3247,'1955-A399','1955-05-27 00:00:00','atmospheric rocket','success',836,'1890T','','','','Test',1895,'',10 from dual union all
select 32470,'1973-W447','1973-03-28 22:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 32471,'1973-W448','1973-03-29 04:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 32472,'1973-W449','1973-03-29 16:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 32473,'1973-W45','1973-01-09 19:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32474,'1973-W450','1973-03-29 16:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 32475,'1973-W451','1973-03-29 19:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 32476,'1973-W452','1973-03-29 21:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32477,'1973-W453','1973-03-30 05:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 32478,'1973-W454','1973-03-30 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual union all
select 32479,'1973-W455','1973-03-30 17:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 3248,'1955-A40','1955-01-21 00:00:00','atmospheric rocket','success',836,'1252E','','','','Test',1895,'',10 from dual union all
select 32480,'1973-W456','1973-03-30 18:45:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 32481,'1973-W457','1973-03-30 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 32482,'1973-W458','1973-03-30 20:13:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 32483,'1973-W459','1973-03-30 22:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32484,'1973-W46','1973-01-09 19:32:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 32485,'1973-W460','1973-03-30 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 32486,'1973-W461','1973-04-02 14:49:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',75 from dual union all
select 32487,'1973-W462','1973-04-02 15:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 32488,'1973-W463','1973-04-02 16:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 32489,'1973-W464','1973-04-02 17:04:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',67 from dual union all
select 3249,'1955-A400','1955-02-25 00:00:00','suborbital rocket','success',813,'I','Camp. 1 to Mar 7','','','Test',666,'',40 from dual union all
select 32490,'1973-W465','1973-04-02 17:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 32491,'1973-W466','1973-04-02 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 32492,'1973-W467','1973-04-02 17:44:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',70 from dual union all
select 32493,'1973-W468','1973-04-02 19:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 32494,'1973-W469','1973-04-02 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 32495,'1973-W47','1973-01-09 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 32496,'1973-W470','1973-04-02 21:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 32497,'1973-W471','1973-04-03 04:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 32498,'1973-W472','1973-04-03 13:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',76 from dual;
insert into LAUNCH
select 32499,'1973-W473','1973-04-03 16:30:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',53 from dual union all
select 325,'1944-S295','1944-12-19 12:17:00','military missile','success',1470,'19872','Ma327','','','Test',661,'',90 from dual union all
select 3250,'1955-A401','1955-04-12 00:00:00','suborbital rocket','success',813,'I','Camp. 2 to Apr 19','','','Test/Aeron',666,'',40 from dual union all
select 32500,'1973-W474','1973-04-03 21:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32501,'1973-W475','1973-04-04 14:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 32502,'1973-W476','1973-04-04 15:37:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 32503,'1973-W477','1973-04-04 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 32504,'1973-W478','1973-04-04 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 32505,'1973-W479','1973-04-04 18:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 32506,'1973-W48','1973-01-10 14:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 32507,'1973-W480','1973-04-04 19:19:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1057,'',59 from dual union all
select 32508,'1973-W481','1973-04-04 20:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32509,'1973-W482','1973-04-04 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 3251,'1955-A402','1955-06-02 00:00:00','suborbital rocket','success',813,'I','Camp. 3 to Jun 11','','','Test/Aeron',666,'',40 from dual union all
select 32510,'1973-W483','1973-04-05 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 32511,'1973-W484','1973-04-05 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',68 from dual union all
select 32512,'1973-W485','1973-04-06 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual union all
select 32513,'1973-W486','1973-04-06 15:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 32514,'1973-W487','1973-04-06 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32515,'1973-W488','1973-04-06 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 32516,'1973-W489','1973-04-06 18:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32517,'1973-W49','1973-01-10 15:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32518,'1973-W490','1973-04-06 19:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 32519,'1973-W491','1973-04-06 22:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',69 from dual union all
select 3252,'1955-A403','1955-05-31 00:00:00','atmospheric rocket','success',836,'2213T','','','','Test',1895,'',10 from dual union all
select 32520,'1973-W492','1973-04-09 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual union all
select 32521,'1973-W493','1973-04-09 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 32522,'1973-W494','1973-04-09 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 32523,'1973-W495','1973-04-09 19:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32524,'1973-W496','1973-04-09 19:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 32525,'1973-W497','1973-04-09 21:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32526,'1973-W498','1973-04-10 00:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',65 from dual union all
select 32527,'1973-W499','1973-04-10 04:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 32528,'1973-W50','1973-01-10 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 32529,'1973-W500','1973-04-10 16:30:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 3253,'1955-A404','1955-05-31 00:00:00','atmospheric rocket','success',836,'2248T','','','','Test',1895,'',10 from dual union all
select 32530,'1973-W501','1973-04-10 21:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32531,'1973-W502','1973-04-11 07:34:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',61 from dual union all
select 32532,'1973-W503','1973-04-11 12:20:00','suborbital rocket','success',716,'INTA HW-7304','','','','Meteo',396,'',60 from dual union all
select 32533,'1973-W504','1973-04-11 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',69 from dual union all
select 32534,'1973-W505','1973-04-11 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual union all
select 32535,'1973-W506','1973-04-11 15:36:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 32536,'1973-W507','1973-04-11 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 32537,'1973-W508','1973-04-11 18:03:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',64 from dual union all
select 32538,'1973-W509','1973-04-11 18:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 32539,'1973-W51','1973-01-10 16:44:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 3254,'1955-A405','1955-06-01 00:00:00','military missile','success',771,'GM-12463','','','','Test',222,'',12 from dual union all
select 32540,'1973-W510','1973-04-11 19:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 32541,'1973-W511','1973-04-11 19:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 32542,'1973-W512','1973-04-11 19:41:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',58 from dual union all
select 32543,'1973-W513','1973-04-11 21:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32544,'1973-W514','1973-04-12 00:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 32545,'1973-W515','1973-04-12 16:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 32546,'1973-W516','1973-04-12 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 32547,'1973-W517','1973-04-13 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 32548,'1973-W518','1973-04-13 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 32549,'1973-W519','1973-04-13 15:35:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 3255,'1955-A406','1955-06-01 00:00:00','military missile','success',771,'GM-52-1857','','','','Test',222,'',12 from dual union all
select 32550,'1973-W52','1973-01-10 16:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',60 from dual union all
select 32551,'1973-W520','1973-04-13 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 32552,'1973-W521','1973-04-13 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 32553,'1973-W522','1973-04-13 21:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 32554,'1973-W523','1973-04-16 12:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 32555,'1973-W524','1973-04-16 13:30:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',65 from dual union all
select 32556,'1973-W525','1973-04-16 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 32557,'1973-W526','1973-04-16 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 32558,'1973-W527','1973-04-16 18:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32559,'1973-W528','1973-04-16 19:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 3256,'1955-A407','1955-06-01 00:00:00','atmospheric rocket','success',836,'1582T','','','','Test',1895,'',10 from dual union all
select 32560,'1973-W529','1973-04-16 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32561,'1973-W53','1973-01-10 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',64 from dual union all
select 32562,'1973-W530','1973-04-17 04:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 32563,'1973-W531','1973-04-17 16:34:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32564,'1973-W532','1973-04-17 21:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32565,'1973-W533','1973-04-18 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',71 from dual union all
select 32566,'1973-W534','1973-04-18 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 32567,'1973-W535','1973-04-18 14:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',63 from dual union all
select 32568,'1973-W536','1973-04-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',68 from dual union all
select 32569,'1973-W537','1973-04-18 15:35:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 3257,'1955-A408','1955-06-01 00:00:00','atmospheric rocket','success',836,'2317T','','','','Test',1895,'',10 from dual union all
select 32570,'1973-W538','1973-04-18 15:56:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 32571,'1973-W539','1973-04-18 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 32572,'1973-W54','1973-01-10 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32573,'1973-W540','1973-04-18 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',77 from dual union all
select 32574,'1973-W541','1973-04-18 18:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32575,'1973-W542','1973-04-18 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 32576,'1973-W543','1973-04-18 22:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32577,'1973-W544','1973-04-18 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',55 from dual union all
select 32578,'1973-W545','1973-04-19 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 32579,'1973-W546','1973-04-19 17:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 3258,'1955-A409','1955-06-01 00:00:00','atmospheric rocket','success',836,'1439E','','','','Test',1895,'',10 from dual union all
select 32580,'1973-W547','1973-04-20 12:45:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',70 from dual union all
select 32581,'1973-W548','1973-04-20 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 32582,'1973-W549','1973-04-20 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 32583,'1973-W55','1973-01-10 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 32584,'1973-W550','1973-04-20 20:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 32585,'1973-W551','1973-04-20 22:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32586,'1973-W552','1973-04-23 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',75 from dual union all
select 32587,'1973-W553','1973-04-23 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',78 from dual union all
select 32588,'1973-W554','1973-04-23 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual;
insert into LAUNCH
select 32589,'1973-W555','1973-04-23 20:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 3259,'1955-A41','1955-01-21 00:00:00','atmospheric rocket','success',836,'2828E','','','','Test',1895,'',10 from dual union all
select 32590,'1973-W556','1973-04-23 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32591,'1973-W557','1973-04-23 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 32592,'1973-W558','1973-04-24 04:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 32593,'1973-W559','1973-04-24 16:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32594,'1973-W56','1973-01-10 19:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32595,'1973-W560','1973-04-24 21:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 32596,'1973-W561','1973-04-25 09:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 32597,'1973-W562','1973-04-25 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',63 from dual union all
select 32598,'1973-W563','1973-04-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 32599,'1973-W564','1973-04-25 15:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',77 from dual union all
select 326,'1944-S296','1944-12-20 12:28:00','military missile','success',1470,'19852','Ma269','','','Test',661,'',90 from dual union all
select 3260,'1955-A410','1955-06-01 00:00:00','atmospheric rocket','success',836,'2253E','','','','Test',1895,'',10 from dual union all
select 32600,'1973-W565','1973-04-25 15:34:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',52 from dual union all
select 32601,'1973-W566','1973-04-25 15:43:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 32602,'1973-W567','1973-04-25 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 32603,'1973-W568','1973-04-25 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 32604,'1973-W569','1973-04-25 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',69 from dual union all
select 32605,'1973-W57','1973-01-10 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 32606,'1973-W570','1973-04-25 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 32607,'1973-W571','1973-04-26 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 32608,'1973-W572','1973-04-27 01:14:59','suborbital rocket','success',1061,'','Starute','','','Meteo',615,'',72 from dual union all
select 32609,'1973-W573','1973-04-27 12:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',68 from dual union all
select 3261,'1955-A411','1955-06-01 00:00:00','atmospheric rocket','success',836,'1581T','','','','Test',1895,'',10 from dual union all
select 32610,'1973-W574','1973-04-27 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',78 from dual union all
select 32611,'1973-W575','1973-04-27 16:09:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 32612,'1973-W576','1973-04-27 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 32613,'1973-W577','1973-04-27 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',73 from dual union all
select 32614,'1973-W578','1973-04-27 18:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 32615,'1973-W579','1973-04-27 19:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 32616,'1973-W58','1973-01-10 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 32617,'1973-W580','1973-04-27 22:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 32618,'1973-W581','1973-04-30 14:15:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',50 from dual union all
select 32619,'1973-W582','1973-04-30 14:36:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 3262,'1955-A412','1955-06-01 00:00:00','atmospheric rocket','success',836,'2271T','','','','Test',1895,'',10 from dual union all
select 32620,'1973-W583','1973-04-30 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 32621,'1973-W584','1973-04-30 17:37:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 32622,'1973-W585','1973-04-30 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 32623,'1973-W586','1973-04-30 19:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 32624,'1973-W587','1973-04-30 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32625,'1973-W588','1973-05-02 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',70 from dual union all
select 32626,'1973-W589','1973-05-02 14:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32627,'1973-W59','1973-01-10 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',56 from dual union all
select 32628,'1973-W590','1973-05-02 14:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 32629,'1973-W591','1973-05-02 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 3263,'1955-A413','1955-06-02 00:00:00','atmospheric rocket','unknown',576,'1032RS','','','','Test',1895,'',10 from dual union all
select 32630,'1973-W592','1973-05-02 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',72 from dual union all
select 32631,'1973-W593','1973-05-02 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',75 from dual union all
select 32632,'1973-W594','1973-05-02 17:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 32633,'1973-W595','1973-05-02 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 32634,'1973-W596','1973-05-02 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',76 from dual union all
select 32635,'1973-W597','1973-05-02 21:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 32636,'1973-W598','1973-05-03 01:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',70 from dual union all
select 32637,'1973-W599','1973-05-03 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 32638,'1973-W60','1973-01-11 15:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32639,'1973-W600','1973-05-03 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 3264,'1955-A414','1955-06-02 00:00:00','atmospheric rocket','success',836,'1917T','','','','Test',1895,'',10 from dual union all
select 32640,'1973-W601','1973-05-04 13:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',77 from dual union all
select 32641,'1973-W602','1973-05-04 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',71 from dual union all
select 32642,'1973-W603','1973-05-04 14:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 32643,'1973-W604','1973-05-04 16:14:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 32644,'1973-W605','1973-05-04 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',75 from dual union all
select 32645,'1973-W606','1973-05-04 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 32646,'1973-W607','1973-05-04 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32647,'1973-W608','1973-05-04 21:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 32648,'1973-W609','1973-05-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 32649,'1973-W61','1973-01-11 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',60 from dual union all
select 3265,'1955-A415','1955-06-02 00:00:00','atmospheric rocket','success',836,'1815T','','','','Test',1895,'',10 from dual union all
select 32650,'1973-W610','1973-05-07 17:13:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 32651,'1973-W611','1973-05-07 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 32652,'1973-W612','1973-05-07 19:13:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',59 from dual union all
select 32653,'1973-W613','1973-05-07 19:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 32654,'1973-W614','1973-05-07 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',64 from dual union all
select 32655,'1973-W615','1973-05-08 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 32656,'1973-W616','1973-05-08 16:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 32657,'1973-W617','1973-05-08 17:21:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 32658,'1973-W618','1973-05-08 19:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32659,'1973-W619','1973-05-09 12:33:00','suborbital rocket','success',716,'INTA HW-7305','','','','Meteo',396,'',60 from dual union all
select 3266,'1955-A416','1955-06-02 00:00:00','atmospheric rocket','success',836,'2314T','','','','Test',1895,'',10 from dual union all
select 32660,'1973-W62','1973-01-11 20:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 32661,'1973-W620','1973-05-09 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 32662,'1973-W621','1973-05-09 15:07:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32663,'1973-W622','1973-05-09 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 32664,'1973-W623','1973-05-09 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 32665,'1973-W624','1973-05-09 19:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 32666,'1973-W625','1973-05-09 21:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32667,'1973-W626','1973-05-10 01:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',79 from dual union all
select 32668,'1973-W627','1973-05-10 02:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 32669,'1973-W628','1973-05-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 3267,'1955-A417','1955-06-02 00:00:00','atmospheric rocket','success',836,'2255T','','','','Test',1895,'',10 from dual union all
select 32670,'1973-W629','1973-05-10 14:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32671,'1973-W63','1973-01-11 23:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32672,'1973-W630','1973-05-10 17:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 32673,'1973-W631','1973-05-11 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 32674,'1973-W632','1973-05-11 14:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 32675,'1973-W633','1973-05-11 15:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',76 from dual union all
select 32676,'1973-W634','1973-05-11 16:19:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32677,'1973-W635','1973-05-11 16:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',70 from dual union all
select 32678,'1973-W636','1973-05-11 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual;
insert into LAUNCH
select 32679,'1973-W637','1973-05-11 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 3268,'1955-A418','1955-06-03 00:00:00','atmospheric rocket','unknown',576,'1036RO','','','','Test',1895,'',10 from dual union all
select 32680,'1973-W638','1973-05-11 21:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32681,'1973-W639','1973-05-13 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',79 from dual union all
select 32682,'1973-W64','1973-01-12 05:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',75 from dual union all
select 32683,'1973-W640','1973-05-14 13:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',76 from dual union all
select 32684,'1973-W641','1973-05-14 14:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32685,'1973-W642','1973-05-14 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 32686,'1973-W643','1973-05-14 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 32687,'1973-W644','1973-05-14 18:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 32688,'1973-W645','1973-05-14 19:02:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',55 from dual union all
select 32689,'1973-W646','1973-05-14 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 3269,'1955-A419','1955-06-03 00:00:00','atmospheric rocket','unknown',576,'1023RO','','','','Test',1895,'',10 from dual union all
select 32690,'1973-W647','1973-05-14 23:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 32691,'1973-W648','1973-05-15 16:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 32692,'1973-W649','1973-05-15 18:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32693,'1973-W65','1973-01-12 08:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',615,'',75 from dual union all
select 32694,'1973-W650','1973-05-15 19:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 32695,'1973-W651','1973-05-16 11:06:00','suborbital rocket','success',229,'AAF-VI-01','Dust collector','','','Aeron',428,'',78 from dual union all
select 32696,'1973-W652','1973-05-16 13:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',50 from dual union all
select 32697,'1973-W653','1973-05-16 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 32698,'1973-W654','1973-05-16 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 32699,'1973-W655','1973-05-16 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 327,'1944-S297','1944-12-20 17:29:00','military missile','success',1470,'19853','Ma274','','','Test',661,'',90 from dual union all
select 3270,'1955-A42','1955-01-24 00:00:00','atmospheric rocket','success',836,'1530T','','','','Test',1895,'',10 from dual union all
select 32700,'1973-W656','1973-05-16 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 32701,'1973-W657','1973-05-16 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',66 from dual union all
select 32702,'1973-W658','1973-05-16 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 32703,'1973-W659','1973-05-16 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',50 from dual union all
select 32704,'1973-W66','1973-01-12 08:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 32705,'1973-W660','1973-05-16 17:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 32706,'1973-W661','1973-05-16 19:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 32707,'1973-W662','1973-05-16 19:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32708,'1973-W663','1973-05-17 02:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 32709,'1973-W664','1973-05-17 15:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 3271,'1955-A420','1955-06-03 00:00:00','atmospheric rocket','success',836,'2285T','','','','Test',1895,'',10 from dual union all
select 32710,'1973-W665','1973-05-17 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',61 from dual union all
select 32711,'1973-W666','1973-05-17 21:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 32712,'1973-W667','1973-05-18 13:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 32713,'1973-W668','1973-05-18 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',53 from dual union all
select 32714,'1973-W669','1973-05-18 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 32715,'1973-W67','1973-01-12 10:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 32716,'1973-W670','1973-05-18 16:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32717,'1973-W671','1973-05-18 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 32718,'1973-W672','1973-05-18 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 32719,'1973-W673','1973-05-18 23:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 3272,'1955-A421','1955-06-03 00:00:00','atmospheric rocket','success',836,'4436E','','','','Test',1895,'',10 from dual union all
select 32720,'1973-W674','1973-05-20 01:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 32721,'1973-W675','1973-05-21 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 32722,'1973-W676','1973-05-21 15:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 32723,'1973-W677','1973-05-21 17:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 32724,'1973-W678','1973-05-21 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 32725,'1973-W679','1973-05-21 19:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 32726,'1973-W68','1973-01-12 14:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',73 from dual union all
select 32727,'1973-W680','1973-05-21 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 32728,'1973-W681','1973-05-22 14:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 32729,'1973-W682','1973-05-22 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 3273,'1955-A422','1955-06-03 00:00:00','atmospheric rocket','success',836,'3856E','','','','Test',1895,'',10 from dual union all
select 32730,'1973-W683','1973-05-22 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32731,'1973-W684','1973-05-23 13:49:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',50 from dual union all
select 32732,'1973-W685','1973-05-23 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 32733,'1973-W686','1973-05-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',55 from dual union all
select 32734,'1973-W687','1973-05-23 16:09:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 32735,'1973-W688','1973-05-23 17:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',77 from dual union all
select 32736,'1973-W689','1973-05-23 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 32737,'1973-W69','1973-01-12 15:40:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',76 from dual union all
select 32738,'1973-W690','1973-05-23 19:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 32739,'1973-W691','1973-05-23 23:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 3274,'1955-A423','1955-06-03 00:00:00','atmospheric rocket','success',836,'2239E','','','','Test',1895,'',10 from dual union all
select 32740,'1973-W692','1973-05-24 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',76 from dual union all
select 32741,'1973-W693','1973-05-24 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 32742,'1973-W694','1973-05-24 21:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32743,'1973-W695','1973-05-25 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 32744,'1973-W696','1973-05-25 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 32745,'1973-W697','1973-05-25 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',75 from dual union all
select 32746,'1973-W698','1973-05-25 18:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 32747,'1973-W699','1973-05-25 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 32748,'1973-W70','1973-01-12 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 32749,'1973-W700','1973-05-25 21:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 3275,'1955-A424','1955-06-03 00:00:00','atmospheric rocket','success',836,'1407E','','','','Test',1895,'',10 from dual union all
select 32750,'1973-W701','1973-05-26 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',69 from dual union all
select 32751,'1973-W702','1973-05-28 14:57:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 32752,'1973-W703','1973-05-29 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 32753,'1973-W704','1973-05-29 18:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 32754,'1973-W705','1973-05-29 19:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 32755,'1973-W706','1973-05-29 20:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32756,'1973-W707','1973-05-30 13:44:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',58 from dual union all
select 32757,'1973-W708','1973-05-30 13:44:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',61 from dual union all
select 32758,'1973-W709','1973-05-30 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',64 from dual union all
select 32759,'1973-W71','1973-01-12 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 3276,'1955-A425','1955-06-06 00:00:00','military missile','unknown',339,'1473R','','','','Test',1895,'',40 from dual union all
select 32760,'1973-W710','1973-05-30 14:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 32761,'1973-W711','1973-05-30 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 32762,'1973-W712','1973-05-30 16:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 32763,'1973-W713','1973-05-30 16:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',65 from dual union all
select 32764,'1973-W714','1973-05-30 16:14:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 32765,'1973-W715','1973-05-30 17:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',79 from dual union all
select 32766,'1973-W716','1973-05-30 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 32767,'1973-W717','1973-05-30 19:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 32768,'1973-W718','1973-05-30 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',53 from dual;
insert into LAUNCH
select 32769,'1973-W719','1973-05-31 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 3277,'1955-A426','1955-06-07 00:00:00','atmospheric rocket','success',836,'1781T','','','','Test',1895,'',10 from dual union all
select 32770,'1973-W72','1973-01-12 17:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 32771,'1973-W720','1973-06-01 00:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',67 from dual union all
select 32772,'1973-W721','1973-06-01 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 32773,'1973-W722','1973-06-01 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',53 from dual union all
select 32774,'1973-W723','1973-06-01 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 32775,'1973-W724','1973-06-01 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 32776,'1973-W725','1973-06-01 20:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32777,'1973-W726','1973-06-01 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32778,'1973-W727','1973-06-04 12:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 32779,'1973-W728','1973-06-04 13:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 3278,'1955-A427','1955-06-07 00:00:00','atmospheric rocket','success',836,'1882T','','','','Test',1895,'',10 from dual union all
select 32780,'1973-W729','1973-06-04 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',53 from dual union all
select 32781,'1973-W73','1973-01-12 18:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 32782,'1973-W730','1973-06-04 17:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 32783,'1973-W731','1973-06-04 17:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 32784,'1973-W732','1973-06-04 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 32785,'1973-W733','1973-06-04 18:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32786,'1973-W734','1973-06-04 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',54 from dual union all
select 32787,'1973-W735','1973-06-04 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 32788,'1973-W736','1973-06-05 15:01:00','suborbital rocket','success',88,'NASA 15.109GM','','','','Aeronomy',35,'',56 from dual union all
select 32789,'1973-W737','1973-06-05 21:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 3279,'1955-A428','1955-06-09 00:00:00','military missile','success',771,'GM-54-1','','','','Test',222,'',12 from dual union all
select 32790,'1973-W738','1973-06-06 00:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 32791,'1973-W739','1973-06-06 11:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 32792,'1973-W74','1973-01-12 18:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 32793,'1973-W740','1973-06-06 12:14:59','suborbital rocket','success',716,'INTA HW-7306','','','','Meteo',396,'',61 from dual union all
select 32794,'1973-W741','1973-06-06 12:36:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',60 from dual union all
select 32795,'1973-W742','1973-06-06 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 32796,'1973-W743','1973-06-06 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',50 from dual union all
select 32797,'1973-W744','1973-06-06 14:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32798,'1973-W745','1973-06-06 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 32799,'1973-W746','1973-06-06 16:21:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 328,'1944-S298','1944-12-21 09:23:00','military missile','success',1470,'19186','','','','Test',661,'',90 from dual union all
select 3280,'1955-A429','1955-06-09 00:00:00','atmospheric rocket','success',836,'1872T','','','','Test',1895,'',10 from dual union all
select 32800,'1973-W747','1973-06-06 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 32801,'1973-W748','1973-06-06 17:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 32802,'1973-W749','1973-06-06 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 32803,'1973-W75','1973-01-12 20:24:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 32804,'1973-W750','1973-06-06 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',76 from dual union all
select 32805,'1973-W751','1973-06-06 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 32806,'1973-W752','1973-06-06 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32807,'1973-W753','1973-06-07 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 32808,'1973-W754','1973-06-08 04:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 32809,'1973-W755','1973-06-08 14:00:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',77 from dual union all
select 3281,'1955-A43','1955-01-25 00:00:00','atmospheric rocket','unknown',576,'M-87A','','','','Test',1895,'',10 from dual union all
select 32810,'1973-W756','1973-06-08 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 32811,'1973-W757','1973-06-08 15:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',76 from dual union all
select 32812,'1973-W758','1973-06-08 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 32813,'1973-W759','1973-06-08 16:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 32814,'1973-W76','1973-01-12 21:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 32815,'1973-W760','1973-06-08 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 32816,'1973-W761','1973-06-08 17:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 32817,'1973-W762','1973-06-08 19:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 32818,'1973-W763','1973-06-08 21:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32819,'1973-W764','1973-06-09 00:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 3282,'1955-A430','1955-06-09 00:00:00','atmospheric rocket','success',836,'1801T','','','','Test',1895,'',10 from dual union all
select 32820,'1973-W765','1973-06-09 01:51:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',65 from dual union all
select 32821,'1973-W766','1973-06-09 02:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 32822,'1973-W767','1973-06-09 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 32823,'1973-W768','1973-06-11 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 32824,'1973-W769','1973-06-11 15:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 32825,'1973-W77','1973-01-12 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 32826,'1973-W770','1973-06-11 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 32827,'1973-W771','1973-06-11 17:00:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',77 from dual union all
select 32828,'1973-W772','1973-06-11 17:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32829,'1973-W773','1973-06-11 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 3283,'1955-A431','1955-06-09 00:00:00','atmospheric rocket','success',836,'1926T','','','','Test',1895,'',10 from dual union all
select 32830,'1973-W774','1973-06-11 21:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32831,'1973-W775','1973-06-12 01:54:00','suborbital rocket','success',740,'MR-12','TOFMS/RFMS','','','Aeron',636,'',168 from dual union all
select 32832,'1973-W776','1973-06-12 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32833,'1973-W777','1973-06-13 13:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',62 from dual union all
select 32834,'1973-W778','1973-06-13 14:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 32835,'1973-W779','1973-06-13 14:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',73 from dual union all
select 32836,'1973-W78','1973-01-13 23:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 32837,'1973-W780','1973-06-13 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 32838,'1973-W781','1973-06-13 16:55:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 32839,'1973-W782','1973-06-13 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',60 from dual union all
select 3284,'1955-A432','1955-06-09 00:00:00','atmospheric rocket','success',836,'1966T','','','','Test',1895,'',10 from dual union all
select 32840,'1973-W783','1973-06-13 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 32841,'1973-W784','1973-06-13 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 32842,'1973-W785','1973-06-13 22:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32843,'1973-W786','1973-06-13 22:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 32844,'1973-W787','1973-06-14 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',58 from dual union all
select 32845,'1973-W788','1973-06-14 14:08:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',64 from dual union all
select 32846,'1973-W789','1973-06-14 15:04:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',68 from dual union all
select 32847,'1973-W79','1973-01-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 32848,'1973-W790','1973-06-15 04:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 32849,'1973-W791','1973-06-15 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',276,'',56 from dual union all
select 3285,'1955-A433','1955-06-10 00:00:00','military missile','success',771,'GM-54-3','','','','Test',222,'',12 from dual union all
select 32850,'1973-W792','1973-06-15 13:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32851,'1973-W793','1973-06-15 16:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 32852,'1973-W794','1973-06-15 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 32853,'1973-W795','1973-06-15 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',79 from dual union all
select 32854,'1973-W796','1973-06-15 21:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 32855,'1973-W797','1973-06-16 01:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',52 from dual union all
select 32856,'1973-W798','1973-06-18 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 32857,'1973-W799','1973-06-18 14:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',76 from dual union all
select 32858,'1973-W80','1973-01-15 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',74 from dual;
insert into LAUNCH
select 32859,'1973-W800','1973-06-18 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',54 from dual union all
select 3286,'1955-A434','1955-06-10 00:00:00','military missile','success',771,'GM-54-2','','','','Test',222,'',12 from dual union all
select 32860,'1973-W801','1973-06-18 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',72 from dual union all
select 32861,'1973-W802','1973-06-18 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 32862,'1973-W803','1973-06-18 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 32863,'1973-W804','1973-06-18 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32864,'1973-W805','1973-06-19 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 32865,'1973-W806','1973-06-19 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 32866,'1973-W807','1973-06-20 00:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',66 from dual union all
select 32867,'1973-W808','1973-06-20 04:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 32868,'1973-W809','1973-06-20 14:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 32869,'1973-W81','1973-01-15 16:49:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',60 from dual union all
select 3287,'1955-A435','1955-06-10 00:00:00','atmospheric rocket','success',836,'2246T','','','','Test',1895,'',10 from dual union all
select 32870,'1973-W810','1973-06-20 15:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 32871,'1973-W811','1973-06-20 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 32872,'1973-W812','1973-06-20 16:27:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 32873,'1973-W813','1973-06-20 19:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 32874,'1973-W814','1973-06-20 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 32875,'1973-W815','1973-06-20 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 32876,'1973-W816','1973-06-21 15:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 32877,'1973-W817','1973-06-22 14:47:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',53 from dual union all
select 32878,'1973-W818','1973-06-22 16:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 32879,'1973-W819','1973-06-22 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 3288,'1955-A436','1955-06-11 00:00:00','atmospheric rocket','success',836,'3325T','','','','Test',1895,'',10 from dual union all
select 32880,'1973-W82','1973-01-15 18:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 32881,'1973-W820','1973-06-22 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 32882,'1973-W821','1973-06-22 17:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 32883,'1973-W822','1973-06-22 21:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32884,'1973-W823','1973-06-23 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',69 from dual union all
select 32885,'1973-W824','1973-06-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 32886,'1973-W825','1973-06-25 16:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 32887,'1973-W826','1973-06-25 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 32888,'1973-W827','1973-06-25 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 32889,'1973-W828','1973-06-25 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 3289,'1955-A437','1955-06-11 00:00:00','atmospheric rocket','success',836,'2258T','','','','Test',1895,'',10 from dual union all
select 32890,'1973-W829','1973-06-26 19:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 32891,'1973-W83','1973-01-15 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 32892,'1973-W830','1973-06-27 04:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 32893,'1973-W831','1973-06-27 13:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 32894,'1973-W832','1973-06-27 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',60 from dual union all
select 32895,'1973-W833','1973-06-27 14:38:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 32896,'1973-W834','1973-06-27 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 32897,'1973-W835','1973-06-27 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 32898,'1973-W836','1973-06-27 16:30:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 32899,'1973-W837','1973-06-27 16:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 329,'1944-S299','1944-12-21 10:43:00','military missile','success',1470,'19676','','','','Test',661,'',90 from dual union all
select 3290,'1955-A438','1955-06-12 00:00:00','atmospheric rocket','success',836,'2293T','','','','Test',1895,'',10 from dual union all
select 32900,'1973-W838','1973-06-27 19:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 32901,'1973-W839','1973-06-28 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',66 from dual union all
select 32902,'1973-W84','1973-01-15 18:55:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 32903,'1973-W840','1973-06-28 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',50 from dual union all
select 32904,'1973-W841','1973-06-29 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',54 from dual union all
select 32905,'1973-W842','1973-06-29 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 32906,'1973-W843','1973-06-29 17:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32907,'1973-W844','1973-06-29 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',55 from dual union all
select 32908,'1973-W845','1973-07-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 32909,'1973-W846','1973-07-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 3291,'1955-A439','1955-06-12 00:00:00','atmospheric rocket','success',836,'2254T','','','','Test',1895,'',10 from dual union all
select 32910,'1973-W847','1973-07-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 32911,'1973-W848','1973-06-30 21:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 32912,'1973-W849','1973-07-02 14:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32913,'1973-W85','1973-01-15 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 32914,'1973-W850','1973-07-02 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 32915,'1973-W851','1973-07-02 17:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 32916,'1973-W852','1973-07-02 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 32917,'1973-W853','1973-07-02 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32918,'1973-W854','1973-07-03 14:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32919,'1973-W855','1973-07-03 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 3292,'1955-A44','1955-01-25 00:00:00','atmospheric rocket','unknown',576,'M-34A','','','','Test',1895,'',10 from dual union all
select 32920,'1973-W856','1973-07-04 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 32921,'1973-W857','1973-07-05 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 32922,'1973-W858','1973-07-05 23:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32923,'1973-W859','1973-07-06 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 32924,'1973-W86','1973-01-15 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 32925,'1973-W860','1973-07-06 14:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 32926,'1973-W861','1973-07-06 15:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 32927,'1973-W862','1973-07-06 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 32928,'1973-W863','1973-07-06 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 32929,'1973-W864','1973-07-06 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 3293,'1955-A440','1955-06-12 00:00:00','atmospheric rocket','success',836,'2257T','','','','Test',1895,'',10 from dual union all
select 32930,'1973-W865','1973-07-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',70 from dual union all
select 32931,'1973-W866','1973-07-09 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 32932,'1973-W867','1973-07-09 17:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 32933,'1973-W868','1973-07-09 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 32934,'1973-W869','1973-07-09 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 32935,'1973-W87','1973-01-16 15:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 32936,'1973-W870','1973-07-09 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 32937,'1973-W871','1973-07-11 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',70 from dual union all
select 32938,'1973-W872','1973-07-11 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 32939,'1973-W873','1973-07-11 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 3294,'1955-A441','1955-06-13 21:31:00','military missile','failure',1522,'Q-4','','','','Test',253,'',1 from dual union all
select 32940,'1973-W874','1973-07-11 16:58:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 32941,'1973-W875','1973-07-11 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 32942,'1973-W876','1973-07-11 19:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 32943,'1973-W877','1973-07-11 19:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 32944,'1973-W878','1973-07-11 21:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 32945,'1973-W879','1973-07-12 00:00:00','suborbital rocket','success',88,'','Arcas Starute 8/STS','','','Test/Aeron',1895,'',68 from dual union all
select 32946,'1973-W88','1973-01-16 16:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 32947,'1973-W880','1973-07-12 18:14:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',67 from dual union all
select 32948,'1973-W881','1973-07-12 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual;
insert into LAUNCH
select 32949,'1973-W882','1973-07-13 04:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 3295,'1955-A442','1955-06-14 00:00:00','military missile','unknown',339,'1569R','','','','Test',1895,'',40 from dual union all
select 32950,'1973-W883','1973-07-13 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 32951,'1973-W884','1973-07-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 32952,'1973-W885','1973-07-13 14:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 32953,'1973-W886','1973-07-13 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 32954,'1973-W887','1973-07-13 20:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 32955,'1973-W888','1973-07-16 12:59:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',74 from dual union all
select 32956,'1973-W889','1973-07-16 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 32957,'1973-W89','1973-01-16 18:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 32958,'1973-W890','1973-07-16 17:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 32959,'1973-W891','1973-07-16 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 3296,'1955-A443','1955-06-14 00:00:00','atmospheric rocket','success',836,'4435E','','','','Test',1895,'',10 from dual union all
select 32960,'1973-W892','1973-07-16 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 32961,'1973-W893','1973-07-16 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 32962,'1973-W894','1973-07-16 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 32963,'1973-W895','1973-07-17 00:00:00','suborbital rocket','success',88,'','Arcas Starute 9/STS','','','Test/Aeron',1895,'',57 from dual union all
select 32964,'1973-W896','1973-07-17 00:00:00','suborbital rocket','success',1284,'','','','','Aeron',1821,'',75 from dual union all
select 32965,'1973-W897','1973-07-17 00:00:00','suborbital rocket','success',1284,'','','','','Aeron',1821,'',75 from dual union all
select 32966,'1973-W898','1973-07-17 00:00:00','suborbital rocket','success',1284,'','','','','Aeron',1821,'',75 from dual union all
select 32967,'1973-W899','1973-07-17 00:00:00','suborbital rocket','success',1284,'','','','','Aeron',1821,'',75 from dual union all
select 32968,'1973-W90','1973-01-16 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 32969,'1973-W900','1973-07-17 00:00:00','suborbital rocket','success',1284,'','','','','Aeron',1821,'',75 from dual union all
select 3297,'1955-A444','1955-06-15 00:00:00','atmospheric rocket','success',836,'4667R','Nike 457R','','','Test',1895,'',10 from dual union all
select 32970,'1973-W901','1973-07-17 13:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 32971,'1973-W902','1973-07-17 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 32972,'1973-W903','1973-07-17 17:45:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1895,'',56 from dual union all
select 32973,'1973-W904','1973-07-18 00:00:00','suborbital rocket','success',1284,'','','','','Aeron',1821,'',75 from dual union all
select 32974,'1973-W905','1973-07-18 00:00:00','suborbital rocket','success',1284,'','','','','Aeron',1821,'',75 from dual union all
select 32975,'1973-W906','1973-07-18 00:00:00','suborbital rocket','success',1284,'','','','','Aeron',1821,'',75 from dual union all
select 32976,'1973-W907','1973-07-18 00:00:00','suborbital rocket','success',1284,'','','','','Aeron',1821,'',75 from dual union all
select 32977,'1973-W908','1973-07-18 00:00:00','suborbital rocket','success',1284,'','','','','Aeron',1821,'',75 from dual union all
select 32978,'1973-W909','1973-07-18 04:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',52 from dual union all
select 32979,'1973-W91','1973-01-16 19:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 3298,'1955-A445','1955-07-15 00:00:00','military missile','unknown',845,'B5 1810-8','Test','','','Test',1895,'',10 from dual union all
select 32980,'1973-W910','1973-07-18 05:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 32981,'1973-W911','1973-07-18 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',66 from dual union all
select 32982,'1973-W912','1973-07-18 14:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 32983,'1973-W913','1973-07-18 14:47:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 32984,'1973-W914','1973-07-18 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',56 from dual union all
select 32985,'1973-W915','1973-07-18 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 32986,'1973-W916','1973-07-18 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 32987,'1973-W917','1973-07-18 19:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 32988,'1973-W918','1973-07-19 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 32989,'1973-W919','1973-07-19 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 3299,'1955-A446','1955-06-15 00:00:00','atmospheric rocket','success',836,'4654R','Nike 456R','','','Test',1895,'',10 from dual union all
select 32990,'1973-W92','1973-01-16 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 32991,'1973-W920','1973-07-20 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 32992,'1973-W921','1973-07-20 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 32993,'1973-W922','1973-07-20 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 32994,'1973-W923','1973-07-20 18:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 32995,'1973-W924','1973-07-20 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 32996,'1973-W925','1973-07-23 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 32997,'1973-W926','1973-07-23 16:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 32998,'1973-W927','1973-07-23 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 32999,'1973-W928','1973-07-23 21:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 33,'1943-S11','1943-06-26 00:00:00','military missile','success',8,'36','','','','Test',675,'',95 from dual union all
select 330,'1944-S300','1944-12-21 15:57:00','military missile','success',1470,'19778','','','','Test',661,'',90 from dual union all
select 3300,'1955-A447','1955-06-15 00:00:00','atmospheric rocket','success',836,'4704R','Nike 458R','','','Test',1895,'',10 from dual union all
select 33000,'1973-W929','1973-07-25 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 33001,'1973-W93','1973-01-17 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 33002,'1973-W930','1973-07-25 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 33003,'1973-W931','1973-07-25 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',70 from dual union all
select 33004,'1973-W932','1973-07-25 16:02:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 33005,'1973-W933','1973-07-25 16:27:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 33006,'1973-W934','1973-07-25 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',55 from dual union all
select 33007,'1973-W935','1973-07-25 17:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 33008,'1973-W936','1973-07-25 21:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 33009,'1973-W937','1973-07-26 12:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 3301,'1955-A448','1955-06-15 00:00:00','atmospheric rocket','success',836,'3025R','Nike 459R','','','Test',1895,'',10 from dual union all
select 33010,'1973-W938','1973-07-26 13:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 33011,'1973-W939','1973-07-26 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 33012,'1973-W94','1973-01-17 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',59 from dual union all
select 33013,'1973-W940','1973-07-26 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 33014,'1973-W941','1973-07-26 19:36:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',62 from dual union all
select 33015,'1973-W942','1973-07-26 21:00:00','suborbital rocket','success',1061,'','STS','','','Meteo',1895,'',65 from dual union all
select 33016,'1973-W943','1973-07-27 12:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',74 from dual union all
select 33017,'1973-W944','1973-07-27 14:00:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',51 from dual union all
select 33018,'1973-W945','1973-07-27 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 33019,'1973-W946','1973-07-27 14:26:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',50 from dual union all
select 3302,'1955-A449','1955-06-20 00:00:00','atmospheric rocket','success',365,'','E5 Model','','','Test',1821,'',10 from dual union all
select 33020,'1973-W947','1973-07-27 14:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 33021,'1973-W948','1973-07-27 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 33022,'1973-W949','1973-07-27 15:15:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',65 from dual union all
select 33023,'1973-W95','1973-01-17 15:22:00','suborbital rocket','success',1506,'T 1-6253','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 33024,'1973-W950','1973-07-27 15:45:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1821,'',68 from dual union all
select 33025,'1973-W951','1973-07-27 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',69 from dual union all
select 33026,'1973-W952','1973-07-28 12:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 33027,'1973-W953','1973-07-30 14:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',50 from dual union all
select 33028,'1973-W954','1973-07-30 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 33029,'1973-W955','1973-07-30 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 3303,'1955-A45','1955-01-25 00:00:00','atmospheric rocket','unknown',576,'347RM','','','','Test',1895,'',10 from dual union all
select 33030,'1973-W956','1973-07-30 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 33031,'1973-W957','1973-07-30 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 33032,'1973-W958','1973-07-30 21:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 33033,'1973-W959','1973-08-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 33034,'1973-W96','1973-01-17 16:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 33035,'1973-W960','1973-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 33036,'1973-W961','1973-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 33037,'1973-W962','1973-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual;
insert into LAUNCH
select 33038,'1973-W963','1973-07-31 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 33039,'1973-W964','1973-08-01 00:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 3304,'1955-A450','1955-06-23 19:57:00','military missile','success',1523,'H-1','','','','Test',253,'',20 from dual union all
select 33040,'1973-W965','1973-08-01 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 33041,'1973-W966','1973-08-01 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',64 from dual union all
select 33042,'1973-W967','1973-08-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 33043,'1973-W968','1973-08-01 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',51 from dual union all
select 33044,'1973-W969','1973-08-01 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 33045,'1973-W97','1973-01-17 16:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 33046,'1973-W970','1973-08-01 14:58:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',50 from dual union all
select 33047,'1973-W971','1973-08-01 14:59:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 33048,'1973-W972','1973-08-01 16:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 33049,'1973-W973','1973-08-02 13:14:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',60 from dual union all
select 3305,'1955-A451','1955-06-23 00:00:00','atmospheric rocket','failure',858,'','Heat Transfer','','','Reentry Test',1821,'',30 from dual union all
select 33050,'1973-W974','1973-08-02 16:51:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 33051,'1973-W975','1973-08-02 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 33052,'1973-W976','1973-08-03 07:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 33053,'1973-W977','1973-08-03 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 33054,'1973-W978','1973-08-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 33055,'1973-W979','1973-08-03 14:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 33056,'1973-W98','1973-01-17 17:03:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 33057,'1973-W980','1973-08-03 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 33058,'1973-W981','1973-08-03 21:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 33059,'1973-W982','1973-08-04 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',70 from dual union all
select 3306,'1955-A452','1955-06-28 00:00:00','atmospheric rocket','success',442,'','D35 Model','','','Test',1821,'',10 from dual union all
select 33060,'1973-W983','1973-08-06 13:42:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',53 from dual union all
select 33061,'1973-W984','1973-08-06 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 33062,'1973-W985','1973-08-06 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 33063,'1973-W986','1973-08-06 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 33064,'1973-W987','1973-08-06 18:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 33065,'1973-W988','1973-08-06 18:20:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',35,'',50 from dual union all
select 33066,'1973-W989','1973-08-06 21:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 33067,'1973-W99','1973-01-17 18:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1895,'',65 from dual union all
select 33068,'1973-W990','1973-08-08 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 33069,'1973-W991','1973-08-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',70 from dual union all
select 3307,'1955-A453','1955-06-30 19:41:00','military missile','success',1523,'H-3','','','','Test',253,'',9 from dual union all
select 33070,'1973-W992','1973-08-08 15:07:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 33071,'1973-W993','1973-08-08 15:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',70 from dual union all
select 33072,'1973-W994','1973-08-08 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 33073,'1973-W995','1973-08-08 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 33074,'1973-W996','1973-08-08 19:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 33075,'1973-W997','1973-08-08 20:34:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',59 from dual union all
select 33076,'1973-W998','1973-08-08 21:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 33077,'1973-W999','1973-08-09 00:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 33078,'1973-Y01','1973-01-01 00:00:00','suborbital rocket','unknown',1188,'','','','','Aeron?',340,'',100 from dual union all
select 33079,'1973-Y02','1973-01-01 00:00:00','suborbital rocket','unknown',1188,'','','','','Aeron?',340,'',100 from dual union all
select 3308,'1955-A454','1955-07-08 00:00:00','military missile','success',771,'GM-54-8','','','','Test',222,'',12 from dual union all
select 33080,'1973-Y03','1973-01-01 00:00:00','suborbital rocket','unknown',1188,'','','','','Aeron?',340,'',100 from dual union all
select 33081,'1973-Y04','1973-01-01 00:00:00','suborbital rocket','unknown',1188,'','','','','Aeron?',340,'',100 from dual union all
select 33082,'1973-Y05','1973-01-01 00:00:00','suborbital rocket','unknown',1188,'','','','','Aeron?',340,'',100 from dual union all
select 33083,'1973-Y06','1973-01-01 00:00:00','suborbital rocket','unknown',1188,'','','','','Aeron?',340,'',100 from dual union all
select 33084,'1973-Y07','1973-01-01 00:00:00','suborbital rocket','unknown',1188,'','','','','Aeron?',340,'',100 from dual union all
select 33085,'1973-Y08','1973-01-01 00:00:00','suborbital rocket','unknown',1188,'','','','','Aeron?',340,'',100 from dual union all
select 33086,'1973-Y09','1973-01-01 00:00:00','suborbital rocket','unknown',1188,'','','','','Aeron?',340,'',100 from dual union all
select 33087,'1973-Y10','1973-01-01 00:00:00','suborbital rocket','unknown',1188,'','','','','Aeron?',340,'',100 from dual union all
select 33088,'1973-Y100','1973-02-28 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 33089,'1973-Y101','1973-02-28 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 3309,'1955-A455','1955-07-08 00:00:00','military missile','success',771,'GM-54-5','','','','Test',222,'',12 from dual union all
select 33090,'1973-Y102','1973-02-28 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 33091,'1973-Y103','1973-03-01 14:50:00','suborbital rocket','unknown',725,'TERLS-531','ISRO-8.115','TERLS-531','','Aeron',1467,'',88 from dual union all
select 33092,'1973-Y104','1973-03-01 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',86 from dual union all
select 33093,'1973-Y105','1973-03-01 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',86 from dual union all
select 33094,'1973-Y106','1973-03-01 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',86 from dual union all
select 33095,'1973-Y107','1973-03-02 21:09:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 33096,'1973-Y108','1973-03-04 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',89 from dual union all
select 33097,'1973-Y109','1973-03-04 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',87 from dual union all
select 33098,'1973-Y11','1973-01-01 00:00:00','suborbital rocket','unknown',1188,'','','','','Aeron?',340,'',100 from dual union all
select 33099,'1973-Y110','1973-03-06 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',88 from dual union all
select 331,'1944-S301','1944-12-23 09:46:00','military missile','success',1470,'19886','','','','Test',661,'',90 from dual union all
select 3310,'1955-A456','1955-07-13 00:00:00','military missile','success',1214,'N-69C GM-13112','','','','Test',238,'',10 from dual union all
select 33100,'1973-Y111','1973-03-06 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 33101,'1973-Y112','1973-03-07 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 33102,'1973-Y113','1973-03-07 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',88 from dual union all
select 33103,'1973-Y114','1973-03-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 33104,'1973-Y115','1973-03-07 14:30:00','suborbital rocket','unknown',725,'TERLS-534','ISRO-8.116','TERLS-534','','Aeron',1467,'',87 from dual union all
select 33105,'1973-Y116','1973-03-07 19:19:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 33106,'1973-Y117','1973-03-07 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',87 from dual union all
select 33107,'1973-Y118','1973-03-07 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',87 from dual union all
select 33108,'1973-Y119','1973-03-08 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',88 from dual union all
select 33109,'1973-Y12','1973-01-01 00:00:00','suborbital rocket','unknown',1188,'','','','','Aeron?',340,'',100 from dual union all
select 3311,'1955-A457','1955-07-14 21:34:00','military missile','success',1523,'H-2','','','','Test',253,'',10 from dual union all
select 33110,'1973-Y120','1973-03-09 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',90 from dual union all
select 33111,'1973-Y121','1973-03-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',90 from dual union all
select 33112,'1973-Y122','1973-03-09 15:31:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',83 from dual union all
select 33113,'1973-Y123','1973-03-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',92 from dual union all
select 33114,'1973-Y124','1973-03-10 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',89 from dual union all
select 33115,'1973-Y125','1973-03-11 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',89 from dual union all
select 33116,'1973-Y126','1973-03-12 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',86 from dual union all
select 33117,'1973-Y127','1973-03-12 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',88 from dual union all
select 33118,'1973-Y128','1973-03-14 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 33119,'1973-Y129','1973-03-14 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',86 from dual union all
select 3312,'1955-A458','1955-07-18 00:00:00','military missile','success',771,'GM-54-6','','','','Test',222,'',12 from dual union all
select 33120,'1973-Y13','1973-01-01 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',86 from dual union all
select 33121,'1973-Y130','1973-03-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 33122,'1973-Y131','1973-03-14 14:50:00','suborbital rocket','unknown',725,'TERLS-545','ISRO-8.117','TERLS-545','','Aeron',1467,'',86 from dual union all
select 33123,'1973-Y132','1973-03-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',89 from dual union all
select 33124,'1973-Y133','1973-03-14 18:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 33125,'1973-Y134','1973-03-14 19:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',89 from dual union all
select 33126,'1973-Y135','1973-03-15 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 33127,'1973-Y136','1973-03-15 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',89 from dual;
insert into LAUNCH
select 33128,'1973-Y137','1973-03-19 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',88 from dual union all
select 33129,'1973-Y138','1973-03-19 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',89 from dual union all
select 3313,'1955-A459','1955-07-18 00:00:00','military missile','success',771,'GM-54-7','','','','Test',222,'',12 from dual union all
select 33130,'1973-Y139','1973-03-20 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',89 from dual union all
select 33131,'1973-Y14','1973-01-02 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',91 from dual union all
select 33132,'1973-Y140','1973-03-20 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',83 from dual union all
select 33133,'1973-Y141','1973-03-20 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',90 from dual union all
select 33134,'1973-Y142','1973-03-20 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 33135,'1973-Y143','1973-03-21 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 33136,'1973-Y144','1973-03-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 33137,'1973-Y145','1973-03-21 14:30:00','suborbital rocket','unknown',725,'TERLS-548','ISRO-8.118','TERLS-548','','Aeron',1467,'',87 from dual union all
select 33138,'1973-Y146','1973-03-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',92 from dual union all
select 33139,'1973-Y147','1973-03-21 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',98 from dual union all
select 3314,'1955-A46','1955-01-25 00:00:00','atmospheric rocket','unknown',576,'346RM','','','','Test',1895,'',10 from dual union all
select 33140,'1973-Y148','1973-03-21 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',90 from dual union all
select 33141,'1973-Y149','1973-03-22 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',86 from dual union all
select 33142,'1973-Y15','1973-01-03 14:15:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',97 from dual union all
select 33143,'1973-Y150','1973-03-23 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 33144,'1973-Y151','1973-03-24 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 33145,'1973-Y152','1973-03-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',92 from dual union all
select 33146,'1973-Y153','1973-03-24 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',86 from dual union all
select 33147,'1973-Y154','1973-03-28 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 33148,'1973-Y155','1973-03-28 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 33149,'1973-Y156','1973-03-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 3315,'1955-A460','1955-07-22 00:00:00','military missile','success',771,'GM-54-4','','','','Test',222,'',12 from dual union all
select 33150,'1973-Y157','1973-03-28 14:30:00','suborbital rocket','unknown',725,'TERLS-549','ISRO-8.119','TERLS-549','','Aeron',1467,'',91 from dual union all
select 33151,'1973-Y158','1973-03-28 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',81 from dual union all
select 33152,'1973-Y159','1973-03-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',86 from dual union all
select 33153,'1973-Y16','1973-01-03 14:30:00','suborbital rocket','unknown',725,'TERLS-511','ISRO-8.105','TERLS-511','','Aeron',1467,'',89 from dual union all
select 33154,'1973-Y160','1973-04-04 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 33155,'1973-Y161','1973-04-04 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 33156,'1973-Y162','1973-04-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 33157,'1973-Y163','1973-04-04 14:30:00','suborbital rocket','unknown',725,'TERLS-550','ISRO-8.120','TERLS-550','','Aeron',1467,'',90 from dual union all
select 33158,'1973-Y164','1973-04-04 15:20:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',91 from dual union all
select 33159,'1973-Y165','1973-04-04 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 3316,'1955-A461','1955-07-27 00:00:00','military missile','failure',255,'623-12','','','','Test',253,'',20 from dual union all
select 33160,'1973-Y166','1973-04-05 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 33161,'1973-Y167','1973-04-07 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 33162,'1973-Y168','1973-04-10 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 33163,'1973-Y169','1973-04-11 14:00:00','suborbital rocket','unknown',725,'TERLS-551','ISRO-8.121','TERLS-551','','Aeron',1467,'',90 from dual union all
select 33164,'1973-Y17','1973-01-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 33165,'1973-Y170','1973-04-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 33166,'1973-Y171','1973-04-11 18:58:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 33167,'1973-Y172','1973-04-11 19:09:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',98 from dual union all
select 33168,'1973-Y173','1973-04-11 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',88 from dual union all
select 33169,'1973-Y174','1973-04-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 3317,'1955-A462','1955-07-29 00:00:00','military missile','success',771,'GM-54-9','','','','Test',222,'',12 from dual union all
select 33170,'1973-Y175','1973-04-18 03:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 33171,'1973-Y176','1973-04-18 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 33172,'1973-Y177','1973-04-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 33173,'1973-Y178','1973-04-18 14:30:00','suborbital rocket','unknown',725,'TERLS-553','ISRO-8.122','TERLS-553','','Aeron',1467,'',84 from dual union all
select 33174,'1973-Y179','1973-04-18 15:00:00','suborbital rocket','success',725,'','Atomic O','','','Aeron',856,'',83 from dual union all
select 33175,'1973-Y18','1973-01-03 15:50:00','suborbital rocket','unknown',725,'TERLS-512','ISRO-8.106','TERLS-512','','Aeron',1467,'',86 from dual union all
select 33176,'1973-Y180','1973-04-18 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',98 from dual union all
select 33177,'1973-Y181','1973-04-18 17:01:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',88 from dual union all
select 33178,'1973-Y182','1973-04-18 17:41:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 33179,'1973-Y183','1973-04-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',83 from dual union all
select 3318,'1955-A463','1955-07-29 00:00:00','military missile','success',771,'GM-54-10','','','','Test',222,'',12 from dual union all
select 33180,'1973-Y184','1973-04-21 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 33181,'1973-Y185','1973-04-25 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 33182,'1973-Y186','1973-04-25 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 33183,'1973-Y187','1973-04-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 33184,'1973-Y188','1973-04-25 14:30:00','suborbital rocket','unknown',725,'TERLS-554','ISRO-8.123','TERLS-554','','Aeron',1467,'',89 from dual union all
select 33185,'1973-Y189','1973-04-25 14:56:00','suborbital rocket','success',725,'','Atomic O','','','Aeron',856,'',84 from dual union all
select 33186,'1973-Y19','1973-01-04 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',81 from dual union all
select 33187,'1973-Y190','1973-04-25 18:33:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 33188,'1973-Y191','1973-04-27 14:58:00','suborbital rocket','success',725,'','Atomic O','','','Aeron',856,'',86 from dual union all
select 33189,'1973-Y192','1973-04-29 00:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',80 from dual union all
select 3319,'1955-A464','1955-08-09 00:00:00','military missile','success',1214,'N-69C GM-13113','','','','Test',224,'',10 from dual union all
select 33190,'1973-Y193','1973-05-02 09:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 33191,'1973-Y194','1973-05-02 14:30:00','suborbital rocket','unknown',725,'TERLS-555','ISRO-8.124','TERLS-555','','Aeron',1467,'',84 from dual union all
select 33192,'1973-Y195','1973-05-02 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 33193,'1973-Y196','1973-05-03 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 33194,'1973-Y197','1973-05-07 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 33195,'1973-Y198','1973-05-09 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 33196,'1973-Y199','1973-05-09 14:30:00','suborbital rocket','unknown',725,'TERLS-556','ISRO-8.125','TERLS-556','','Aeron',1467,'',88 from dual union all
select 33197,'1973-Y20','1973-01-04 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',89 from dual union all
select 33198,'1973-Y200','1973-05-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',86 from dual union all
select 33199,'1973-Y201','1973-05-10 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 332,'1944-S302','1944-12-23 12:55:00','military missile','success',1470,'19674','','','','Test',661,'',90 from dual union all
select 3320,'1955-A465','1955-08-16 00:00:00','suborbital rocket','failure',1059,'','','','','Test',986,'F2H2',9 from dual union all
select 33200,'1973-Y202','1973-05-11 15:10:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 33201,'1973-Y203','1973-05-11 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 33202,'1973-Y204','1973-05-14 14:30:00','suborbital rocket','unknown',725,'TERLS-557','ISRO-8.126','TERLS-557','','Aeron',1467,'',86 from dual union all
select 33203,'1973-Y205','1973-05-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 33204,'1973-Y206','1973-05-16 14:30:00','suborbital rocket','unknown',725,'TERLS-558','ISRO-8.127','TERLS-558','','Aeron',1467,'',84 from dual union all
select 33205,'1973-Y207','1973-05-16 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',95 from dual union all
select 33206,'1973-Y208','1973-05-16 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 33207,'1973-Y209','1973-05-18 14:30:00','suborbital rocket','unknown',725,'TERLS-559','ISRO-8.128','TERLS-559','','Aeron',1467,'',88 from dual union all
select 33208,'1973-Y21','1973-01-05 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',86 from dual union all
select 33209,'1973-Y210','1973-05-19 00:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',81 from dual union all
select 3321,'1955-A466','1955-08-25 00:00:00','military missile','success',255,'623-23','','','','Test',254,'',20 from dual union all
select 33210,'1973-Y211','1973-05-19 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',83 from dual union all
select 33211,'1973-Y212','1973-05-21 14:30:00','suborbital rocket','unknown',725,'TERLS-562','ISRO-8.129','TERLS-562','','Aeron',1467,'',87 from dual union all
select 33212,'1973-Y213','1973-05-23 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 33213,'1973-Y214','1973-05-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 33214,'1973-Y215','1973-05-23 14:30:00','suborbital rocket','unknown',725,'TERLS-566','ISRO-8.130','TERLS-566','','Aeron',1467,'',89 from dual union all
select 33215,'1973-Y216','1973-05-23 15:40:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',93 from dual union all
select 33216,'1973-Y217','1973-05-24 02:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',82 from dual union all
select 33217,'1973-Y218','1973-05-25 14:30:00','suborbital rocket','unknown',725,'TERLS-567','ISRO-8.131','TERLS-567','','Aeron',1467,'',90 from dual;
insert into LAUNCH
select 33218,'1973-Y219','1973-05-25 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',84 from dual union all
select 33219,'1973-Y22','1973-01-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',95 from dual union all
select 3322,'1955-A467','1955-08-25 13:00:00','suborbital rocket','failure',49,'NRL 37','XRV-N-13','','','Test',1869,'',4 from dual union all
select 33220,'1973-Y220','1973-05-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 33221,'1973-Y221','1973-05-29 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',85 from dual union all
select 33222,'1973-Y222','1973-05-30 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 33223,'1973-Y223','1973-05-30 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 33224,'1973-Y224','1973-05-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 33225,'1973-Y225','1973-05-30 14:30:00','suborbital rocket','unknown',725,'TERLS-568','ISRO-8.132','TERLS-568','','Aeron',1467,'',86 from dual union all
select 33226,'1973-Y226','1973-05-30 16:30:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 33227,'1973-Y227','1973-06-01 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',87 from dual union all
select 33228,'1973-Y228','1973-06-04 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',86 from dual union all
select 33229,'1973-Y229','1973-06-06 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 3323,'1955-A468','1955-08-26 19:03:00','suborbital rocket','failure',1521,'D-1','','','','Test',253,'',3 from dual union all
select 33230,'1973-Y23','1973-01-06 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',87 from dual union all
select 33231,'1973-Y230','1973-06-06 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 33232,'1973-Y231','1973-06-06 13:44:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',87 from dual union all
select 33233,'1973-Y232','1973-06-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 33234,'1973-Y233','1973-06-06 14:30:00','suborbital rocket','unknown',725,'TERLS-570','ISRO-8.133','TERLS-570','','Aeron',1467,'',88 from dual union all
select 33235,'1973-Y234','1973-06-06 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',86 from dual union all
select 33236,'1973-Y235','1973-06-07 17:15:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',82 from dual union all
select 33237,'1973-Y236','1973-06-08 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',85 from dual union all
select 33238,'1973-Y237','1973-06-08 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 33239,'1973-Y238','1973-06-09 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',82 from dual union all
select 3324,'1955-A469','1955-09-01 00:00:00','atmospheric rocket','failure',835,'','Navaho Model','','','Test',1821,'',1 from dual union all
select 33240,'1973-Y239','1973-06-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 33241,'1973-Y24','1973-01-06 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',86 from dual union all
select 33242,'1973-Y240','1973-06-12 04:30:00','suborbital rocket','success',725,'Diurnal Variation','','','','Aeron',636,'',87 from dual union all
select 33243,'1973-Y241','1973-06-12 06:00:00','suborbital rocket','success',725,'Diurnal Variation','','','','Aeron',636,'',87 from dual union all
select 33244,'1973-Y242','1973-06-12 10:40:00','suborbital rocket','success',725,'Diurnal Variation','','','','Aeron',636,'',85 from dual union all
select 33245,'1973-Y243','1973-06-12 12:50:00','suborbital rocket','success',725,'Diurnal Variation','','','','Aeron',636,'',85 from dual union all
select 33246,'1973-Y244','1973-06-12 18:50:00','suborbital rocket','success',725,'Diurnal Variation','','','','Aeron',636,'',87 from dual union all
select 33247,'1973-Y245','1973-06-13 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',83 from dual union all
select 33248,'1973-Y246','1973-06-12 23:00:00','suborbital rocket','success',725,'Diurnal Variation','','','','Aeron',636,'',87 from dual union all
select 33249,'1973-Y247','1973-06-13 13:44:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',93 from dual union all
select 3325,'1955-A47','1955-01-25 00:00:00','atmospheric rocket','success',836,'3267T','','','','Test',1895,'',10 from dual union all
select 33250,'1973-Y248','1973-06-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 33251,'1973-Y249','1973-06-13 14:00:00','suborbital rocket','success',725,'Diurnal Variation','','','','Aeron',636,'',86 from dual union all
select 33252,'1973-Y25','1973-01-10 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 33253,'1973-Y250','1973-06-13 20:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',89 from dual union all
select 33254,'1973-Y251','1973-06-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 33255,'1973-Y252','1973-06-14 14:30:00','suborbital rocket','unknown',725,'TERLS-573','ISRO-8.134','TERLS-573','','Aeron',1467,'',83 from dual union all
select 33256,'1973-Y253','1973-06-15 18:52:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',87 from dual union all
select 33257,'1973-Y254','1973-06-20 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 33258,'1973-Y255','1973-06-20 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 33259,'1973-Y256','1973-06-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 3326,'1955-A470','1955-09-01 00:00:00','military missile','success',771,'GM-11089','','','','Test',222,'',12 from dual union all
select 33260,'1973-Y257','1973-06-20 14:30:00','suborbital rocket','unknown',725,'TERLS-574','ISRO-8.135','TERLS-574','','Aeron',1467,'',88 from dual union all
select 33261,'1973-Y258','1973-06-20 18:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',93 from dual union all
select 33262,'1973-Y259','1973-06-21 01:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',82 from dual union all
select 33263,'1973-Y26','1973-01-10 14:30:00','suborbital rocket','unknown',725,'TERLS-517','ISRO-8.107','TERLS-517','','Aeron',1467,'',84 from dual union all
select 33264,'1973-Y260','1973-06-22 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',86 from dual union all
select 33265,'1973-Y261','1973-06-22 21:02:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 33266,'1973-Y262','1973-06-24 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 33267,'1973-Y263','1973-06-24 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',84 from dual union all
select 33268,'1973-Y264','1973-06-27 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 33269,'1973-Y265','1973-06-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 3327,'1955-A471','1955-09-15 00:00:00','atmospheric rocket','failure',545,'','Test','','','Test',1821,'',10 from dual union all
select 33270,'1973-Y266','1973-06-27 14:30:00','suborbital rocket','unknown',725,'TERLS-577','ISRO-8.136','TERLS-577','','Aeron',1467,'',88 from dual union all
select 33271,'1973-Y267','1973-06-27 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',85 from dual union all
select 33272,'1973-Y268','1973-06-30 01:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',82 from dual union all
select 33273,'1973-Y269','1973-06-30 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',84 from dual union all
select 33274,'1973-Y27','1973-01-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 33275,'1973-Y270','1973-07-01 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',82 from dual union all
select 33276,'1973-Y271','1973-07-02 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',86 from dual union all
select 33277,'1973-Y272','1973-07-03 00:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 33278,'1973-Y273','1973-07-03 02:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 33279,'1973-Y274','1973-07-03 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',82 from dual union all
select 3328,'1955-A472','1955-09-22 05:51:00','military missile','failure',635,'RS-11    HH','','','','Test',280,'',10 from dual union all
select 33280,'1973-Y275','1973-07-04 09:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 33281,'1973-Y276','1973-07-04 14:30:00','suborbital rocket','unknown',725,'TERLS-579','ISRO-8.137','TERLS-579','','Aeron',1467,'',88 from dual union all
select 33282,'1973-Y277','1973-07-04 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',84 from dual union all
select 33283,'1973-Y278','1973-07-06 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',92 from dual union all
select 33284,'1973-Y279','1973-07-06 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 33285,'1973-Y28','1973-01-10 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',89 from dual union all
select 33286,'1973-Y280','1973-07-07 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',90 from dual union all
select 33287,'1973-Y281','1973-07-11 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 33288,'1973-Y282','1973-07-11 13:49:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',94 from dual union all
select 33289,'1973-Y283','1973-07-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 3329,'1955-A473','1955-09-23 19:10:00','suborbital rocket','failure',1521,'D-2','','','','Test',253,'',5 from dual union all
select 33290,'1973-Y284','1973-07-11 14:30:00','suborbital rocket','unknown',725,'TERLS-580','ISRO-8.138','TERLS-580','','Aeron',1467,'',89 from dual union all
select 33291,'1973-Y285','1973-07-11 20:55:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 33292,'1973-Y286','1973-07-12 02:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',81 from dual union all
select 33293,'1973-Y287','1973-07-14 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 33294,'1973-Y288','1973-07-14 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',89 from dual union all
select 33295,'1973-Y289','1973-07-15 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',85 from dual union all
select 33296,'1973-Y29','1973-01-10 20:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',88 from dual union all
select 33297,'1973-Y290','1973-07-17 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 33298,'1973-Y291','1973-07-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',89 from dual union all
select 33299,'1973-Y292','1973-07-17 16:58:00','suborbital rocket','unknown',1284,'T 1-6682','','','','Test',1821,'',80 from dual union all
select 333,'1944-S303','1944-12-23 14:36:00','military missile','success',1470,'19776','','','','Test',661,'',90 from dual union all
select 3330,'1955-A474','1955-09-25 20:39:00','suborbital rocket','failure',718,'','SUI 41','','','Ionos/Aeron?',101,'AGB-5',11 from dual union all
select 33300,'1973-Y293','1973-07-17 17:35:00','suborbital rocket','unknown',1506,'E 1-6685','','','','Test',1821,'',134 from dual union all
select 33301,'1973-Y294','1973-07-17 18:01:00','suborbital rocket','unknown',1284,'T 1-6684','','','','Test',1821,'',81 from dual union all
select 33302,'1973-Y295','1973-07-18 02:40:00','suborbital rocket','unknown',1506,'E 1-6686','','','','Test',1821,'',141 from dual union all
select 33303,'1973-Y296','1973-07-18 03:14:00','suborbital rocket','unknown',1284,'G 1-6687','','','','Test',1821,'',116 from dual union all
select 33304,'1973-Y297','1973-07-18 05:02:00','suborbital rocket','unknown',1284,'G 1-6689','','','','Test',1821,'',115 from dual union all
select 33305,'1973-Y298','1973-07-18 06:28:00','suborbital rocket','unknown',1284,'T 1-6692','','','','Test',1821,'',82 from dual union all
select 33306,'1973-Y299','1973-07-18 07:31:00','suborbital rocket','unknown',1506,'E 1-6693','','','','Test',1821,'',133 from dual union all
select 33307,'1973-Y30','1973-01-13 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual;
insert into LAUNCH
select 33308,'1973-Y300','1973-07-18 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 33309,'1973-Y301','1973-07-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 3331,'1955-A475','1955-09-27 00:00:00','atmospheric rocket','success',841,'','Hemi-Cyl RV','','','Reentry Test',1821,'',30 from dual union all
select 33310,'1973-Y302','1973-07-18 14:30:00','suborbital rocket','unknown',725,'TERLS-583','ISRO-8.139','TERLS-583','','Aeron',1467,'',88 from dual union all
select 33311,'1973-Y303','1973-07-18 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',993,'NOKV',88 from dual union all
select 33312,'1973-Y304','1973-07-19 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',87 from dual union all
select 33313,'1973-Y305','1973-07-20 16:01:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 33314,'1973-Y306','1973-07-20 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 33315,'1973-Y307','1973-07-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 33316,'1973-Y308','1973-07-24 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',81 from dual union all
select 33317,'1973-Y309','1973-07-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 33318,'1973-Y31','1973-01-16 18:04:00','suborbital rocket','success',1281,'NASA 15.103UI','','','','Ionos',1821,'',95 from dual union all
select 33319,'1973-Y310','1973-07-25 14:30:00','suborbital rocket','unknown',725,'TERLS-584','ISRO-8.140','TERLS-584','','Aeron',1467,'',87 from dual union all
select 3332,'1955-A476','1955-09-27 19:12:00','suborbital rocket','failure',370,'','NRL Rockoon 14','','','Ionos/Aeron?',93,'AGB-5',11 from dual union all
select 33320,'1973-Y311','1973-07-25 20:22:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 33321,'1973-Y312','1973-07-25 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',88 from dual union all
select 33322,'1973-Y313','1973-07-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 33323,'1973-Y314','1973-08-01 02:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 33324,'1973-Y315','1973-08-01 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 33325,'1973-Y316','1973-08-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 33326,'1973-Y317','1973-08-01 14:30:00','suborbital rocket','unknown',725,'TERLS-587','ISRO-8.141','TERLS-587','','Aeron',1467,'',88 from dual union all
select 33327,'1973-Y318','1973-08-01 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 33328,'1973-Y319','1973-08-01 16:02:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',87 from dual union all
select 33329,'1973-Y32','1973-01-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 3333,'1955-A477','1955-09-28 17:40:00','suborbital rocket','failure',370,'','NRL Rockoon 16','','','Ionos/Aeron?',95,'AGB-5',11 from dual union all
select 33330,'1973-Y320','1973-08-01 18:30:00','suborbital rocket','success',1061,'','STS','','','Meteo',1895,'',83 from dual union all
select 33331,'1973-Y321','1973-08-03 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 33332,'1973-Y322','1973-08-07 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',81 from dual union all
select 33333,'1973-Y323','1973-08-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 33334,'1973-Y324','1973-08-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 33335,'1973-Y325','1973-08-08 14:30:00','suborbital rocket','unknown',725,'TERLS-588','ISRO-8.142','TERLS-588','','Aeron',1467,'',87 from dual union all
select 33336,'1973-Y326','1973-08-08 15:26:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 33337,'1973-Y327','1973-08-08 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',95 from dual union all
select 33338,'1973-Y328','1973-08-08 21:23:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 33339,'1973-Y329','1973-08-09 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 3334,'1955-A478','1955-09-29 00:00:00','military missile','success',255,'623-24','','','','Test',254,'',20 from dual union all
select 33340,'1973-Y33','1973-01-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 33341,'1973-Y330','1973-08-11 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',86 from dual union all
select 33342,'1973-Y331','1973-08-14 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 33343,'1973-Y332','1973-08-15 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 33344,'1973-Y333','1973-08-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 33345,'1973-Y334','1973-08-15 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',98 from dual union all
select 33346,'1973-Y335','1973-08-15 20:40:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 33347,'1973-Y336','1973-08-15 21:56:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 33348,'1973-Y337','1973-08-16 14:30:00','suborbital rocket','unknown',725,'TERLS-589','ISRO-8.143','TERLS-589','','Aeron',1467,'',86 from dual union all
select 33349,'1973-Y338','1973-08-19 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',84 from dual union all
select 3335,'1955-A479','1955-10-01 00:00:00','atmospheric rocket','success',365,'','D35 Model','','','Test',1821,'',10 from dual union all
select 33350,'1973-Y339','1973-08-21 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',85 from dual union all
select 33351,'1973-Y34','1973-01-17 14:30:00','suborbital rocket','unknown',725,'TERLS-519','ISRO-8.109','TERLS-519','','Aeron',1467,'',86 from dual union all
select 33352,'1973-Y340','1973-08-22 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 33353,'1973-Y341','1973-08-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 33354,'1973-Y342','1973-08-22 14:30:00','suborbital rocket','unknown',725,'TERLS-590','ISRO-8.144','TERLS-590','','Aeron',1467,'',88 from dual union all
select 33355,'1973-Y343','1973-08-23 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',89 from dual union all
select 33356,'1973-Y344','1973-08-24 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 33357,'1973-Y345','1973-08-24 16:01:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',93 from dual union all
select 33358,'1973-Y346','1973-08-26 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',85 from dual union all
select 33359,'1973-Y347','1973-08-27 00:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',84 from dual union all
select 3336,'1955-A48','1955-01-25 00:00:00','atmospheric rocket','success',836,'2882E','','','','Test',1895,'',10 from dual union all
select 33360,'1973-Y348','1973-08-28 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',83 from dual union all
select 33361,'1973-Y349','1973-08-29 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',87 from dual union all
select 33362,'1973-Y35','1973-01-17 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',99 from dual union all
select 33363,'1973-Y350','1973-08-29 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',81 from dual union all
select 33364,'1973-Y351','1973-08-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 33365,'1973-Y352','1973-08-29 10:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',81 from dual union all
select 33366,'1973-Y353','1973-08-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 33367,'1973-Y354','1973-08-29 14:30:00','suborbital rocket','unknown',725,'TERLS-593','ISRO-8.145','TERLS-593','','Aeron',1467,'',87 from dual union all
select 33368,'1973-Y355','1973-08-29 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',97 from dual union all
select 33369,'1973-Y356','1973-08-30 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',86 from dual union all
select 3337,'1955-A480','1955-10-06 10:54:00','suborbital rocket','failure',370,'','NRL Rockoon 17','','','Ionos/Aeron?',96,'AGB-5',11 from dual union all
select 33370,'1973-Y357','1973-08-30 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',86 from dual union all
select 33371,'1973-Y358','1973-08-31 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',89 from dual union all
select 33372,'1973-Y359','1973-08-31 05:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 33373,'1973-Y36','1973-01-17 20:02:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',88 from dual union all
select 33374,'1973-Y360','1973-08-31 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 33375,'1973-Y361','1973-08-31 16:18:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',88 from dual union all
select 33376,'1973-Y362','1973-08-31 20:44:00','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',85 from dual union all
select 33377,'1973-Y363','1973-08-31 21:13:59','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',85 from dual union all
select 33378,'1973-Y364','1973-08-31 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',87 from dual union all
select 33379,'1973-Y365','1973-09-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',84 from dual union all
select 3338,'1955-A481','1955-10-07 00:00:00','atmospheric rocket','success',858,'','Heat Transfer','','','Reentry Test',1821,'',30 from dual union all
select 33380,'1973-Y366','1973-09-02 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',89 from dual union all
select 33381,'1973-Y367','1973-09-03 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',87 from dual union all
select 33382,'1973-Y368','1973-09-03 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',84 from dual union all
select 33383,'1973-Y369','1973-09-05 01:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 33384,'1973-Y37','1973-01-19 02:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 33385,'1973-Y370','1973-09-05 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 33386,'1973-Y371','1973-09-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 33387,'1973-Y372','1973-09-05 14:30:00','suborbital rocket','unknown',725,'TERLS-594','ISRO-8.146','TERLS-594','','Aeron',1467,'',86 from dual union all
select 33388,'1973-Y373','1973-09-05 17:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',85 from dual union all
select 33389,'1973-Y374','1973-09-08 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',82 from dual union all
select 3339,'1955-A482','1955-10-11 00:00:00','atmospheric rocket','success',365,'','Smoke','','','Aeron/Test',1821,'',8 from dual union all
select 33390,'1973-Y375','1973-09-09 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',87 from dual union all
select 33391,'1973-Y376','1973-09-09 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',83 from dual union all
select 33392,'1973-Y377','1973-09-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',82 from dual union all
select 33393,'1973-Y378','1973-09-12 09:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 33394,'1973-Y379','1973-09-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 33395,'1973-Y38','1973-01-24 05:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 33396,'1973-Y380','1973-09-12 14:30:00','suborbital rocket','unknown',725,'TERLS-595','ISRO-8.147','TERLS-595','','Aeron',1467,'',84 from dual union all
select 33397,'1973-Y381','1973-09-14 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 33398,'1973-Y382','1973-09-14 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',87 from dual;
insert into LAUNCH
select 33399,'1973-Y383','1973-09-18 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',84 from dual union all
select 334,'1944-S304','1944-12-25 10:28:00','military missile','success',1470,'19650','','','','Test',661,'',90 from dual union all
select 3340,'1955-A483','1955-10-11 00:00:00','atmospheric rocket','success',365,'','Smoke','','','Aeron/Test',1821,'',8 from dual union all
select 33400,'1973-Y384','1973-09-19 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 33401,'1973-Y385','1973-09-19 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 33402,'1973-Y386','1973-09-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 33403,'1973-Y387','1973-09-19 14:30:00','suborbital rocket','unknown',725,'TERLS-596','ISRO-8.148','TERLS-596','','Aeron',1467,'',85 from dual union all
select 33404,'1973-Y388','1973-09-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',86 from dual union all
select 33405,'1973-Y389','1973-09-21 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',90 from dual union all
select 33406,'1973-Y39','1973-01-24 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 33407,'1973-Y390','1973-09-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',82 from dual union all
select 33408,'1973-Y391','1973-09-22 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',85 from dual union all
select 33409,'1973-Y392','1973-09-24 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',88 from dual union all
select 3341,'1955-A484','1955-10-11 14:33:00','suborbital rocket','failure',370,'','SUI 52','','','Ionos/Aeron?',87,'AGB-5',20 from dual union all
select 33410,'1973-Y393','1973-09-25 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',84 from dual union all
select 33411,'1973-Y394','1973-09-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 33412,'1973-Y395','1973-09-25 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',83 from dual union all
select 33413,'1973-Y396','1973-09-26 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',82 from dual union all
select 33414,'1973-Y397','1973-09-26 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 33415,'1973-Y398','1973-09-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',90 from dual union all
select 33416,'1973-Y399','1973-09-27 14:30:00','suborbital rocket','unknown',725,'TERLS-597','ISRO-8.149','TERLS-597','','Aeron',1467,'',84 from dual union all
select 33417,'1973-Y40','1973-01-24 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',89 from dual union all
select 33418,'1973-Y400','1973-09-27 22:13:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',86 from dual union all
select 33419,'1973-Y401','1973-09-28 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',91 from dual union all
select 3342,'1955-A485','1955-10-13 15:24:00','suborbital rocket','failure',370,'','SUI 54','','','Ionos/Aeron?',89,'AGB-5',20 from dual union all
select 33420,'1973-Y402','1973-09-29 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1368,'SHIR',84 from dual union all
select 33421,'1973-Y403','1973-09-30 10:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 33422,'1973-Y404','1973-10-03 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 33423,'1973-Y405','1973-10-03 13:16:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 33424,'1973-Y406','1973-10-03 14:30:00','suborbital rocket','unknown',725,'TERLS-598','ISRO-8.150','TERLS-598','','Aeron',1467,'',91 from dual union all
select 33425,'1973-Y407','1973-10-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 33426,'1973-Y408','1973-10-03 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 33427,'1973-Y409','1973-10-03 18:57:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',89 from dual union all
select 33428,'1973-Y41','1973-01-24 14:30:00','suborbital rocket','unknown',725,'TERLS-521','ISRO-8.110','TERLS-521','','Aeron',1467,'',90 from dual union all
select 33429,'1973-Y410','1973-10-04 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',81 from dual union all
select 3343,'1955-A486','1955-10-26 00:00:00','military missile','success',1214,'N-69C 52-10972','','','','Test',224,'',10 from dual union all
select 33430,'1973-Y411','1973-10-04 18:25:59','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',88 from dual union all
select 33431,'1973-Y412','1973-10-04 18:45:59','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',88 from dual union all
select 33432,'1973-Y413','1973-10-05 16:43:59','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',87 from dual union all
select 33433,'1973-Y414','1973-10-05 17:07:00','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',86 from dual union all
select 33434,'1973-Y415','1973-10-05 17:27:59','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',87 from dual union all
select 33435,'1973-Y416','1973-10-05 17:48:59','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',86 from dual union all
select 33436,'1973-Y417','1973-10-05 21:11:00','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',85 from dual union all
select 33437,'1973-Y418','1973-10-05 21:49:59','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',85 from dual union all
select 33438,'1973-Y419','1973-10-06 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',85 from dual union all
select 33439,'1973-Y42','1973-01-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 3344,'1955-A487','1955-10-27 00:00:00','military missile','success',771,'GM-11051','','','','Test',222,'',12 from dual union all
select 33440,'1973-Y420','1973-10-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',89 from dual union all
select 33441,'1973-Y421','1973-10-10 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 33442,'1973-Y422','1973-10-10 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 33443,'1973-Y423','1973-10-10 14:30:00','suborbital rocket','unknown',725,'TERLS-599','ISRO-8.151','TERLS-599','','Aeron',1467,'',89 from dual union all
select 33444,'1973-Y424','1973-10-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 33445,'1973-Y425','1973-10-10 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',98 from dual union all
select 33446,'1973-Y426','1973-10-14 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',90 from dual union all
select 33447,'1973-Y427','1973-10-15 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',95 from dual union all
select 33448,'1973-Y428','1973-10-16 00:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',82 from dual union all
select 33449,'1973-Y429','1973-10-16 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',88 from dual union all
select 3345,'1955-A488','1955-11-10 00:00:00','military missile','success',771,'GM-11055','','','','Test',222,'',12 from dual union all
select 33450,'1973-Y43','1973-01-24 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',101 from dual union all
select 33451,'1973-Y430','1973-10-16 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 33452,'1973-Y431','1973-10-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',88 from dual union all
select 33453,'1973-Y432','1973-10-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 33454,'1973-Y433','1973-10-17 14:30:00','suborbital rocket','unknown',725,'TERLS-600','ISRO-8.152','TERLS-600','','Aeron',1467,'',85 from dual union all
select 33455,'1973-Y434','1973-10-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 33456,'1973-Y435','1973-10-17 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',86 from dual union all
select 33457,'1973-Y436','1973-10-17 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',88 from dual union all
select 33458,'1973-Y437','1973-10-18 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',87 from dual union all
select 33459,'1973-Y438','1973-10-18 13:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 3346,'1955-A489','1955-11-16 00:00:00','atmospheric rocket','success',442,'','Regulus II Model','','','Test',1821,'',10 from dual union all
select 33460,'1973-Y439','1973-10-19 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',89 from dual union all
select 33461,'1973-Y44','1973-01-24 17:57:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',89 from dual union all
select 33462,'1973-Y440','1973-10-23 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 33463,'1973-Y441','1973-10-24 14:30:00','suborbital rocket','unknown',725,'TERLS-601','ISRO-8.153','TERLS-601','','Aeron',1467,'',91 from dual union all
select 33464,'1973-Y442','1973-10-24 14:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',95 from dual union all
select 33465,'1973-Y443','1973-10-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 33466,'1973-Y444','1973-10-24 15:58:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',87 from dual union all
select 33467,'1973-Y445','1973-10-24 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',80 from dual union all
select 33468,'1973-Y446','1973-10-25 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 33469,'1973-Y447','1973-10-30 00:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 3347,'1955-A49','1955-01-26 00:00:00','atmospheric rocket','success',836,'T 1532','','','','Test',1895,'',10 from dual union all
select 33470,'1973-Y448','1973-10-30 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',88 from dual union all
select 33471,'1973-Y449','1973-10-31 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 33472,'1973-Y45','1973-01-26 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',89 from dual union all
select 33473,'1973-Y450','1973-10-31 14:30:00','suborbital rocket','unknown',725,'TERLS-602','ISRO-8.154','TERLS-602','','Aeron',1467,'',86 from dual union all
select 33474,'1973-Y451','1973-10-31 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 33475,'1973-Y452','1973-10-31 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',879,'KORO',89 from dual union all
select 33476,'1973-Y453','1973-11-01 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',81 from dual union all
select 33477,'1973-Y454','1973-11-01 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',879,'KORO',89 from dual union all
select 33478,'1973-Y455','1973-11-02 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',81 from dual union all
select 33479,'1973-Y456','1973-11-02 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',84 from dual union all
select 3348,'1955-A490','1955-11-17 00:00:00','military missile','success',255,'623-25','','','','Test',254,'',20 from dual union all
select 33480,'1973-Y457','1973-11-06 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 33481,'1973-Y458','1973-11-06 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 33482,'1973-Y459','1973-11-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',879,'KORO',92 from dual union all
select 33483,'1973-Y46','1973-01-26 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',89 from dual union all
select 33484,'1973-Y460','1973-11-07 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.155','TERLS-603','','Aeron',1467,'',87 from dual union all
select 33485,'1973-Y461','1973-11-07 17:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',95 from dual union all
select 33486,'1973-Y462','1973-11-09 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',81 from dual union all
select 33487,'1973-Y463','1973-11-09 19:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',80 from dual union all
select 33488,'1973-Y464','1973-11-09 21:03:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual;
insert into LAUNCH
select 33489,'1973-Y465','1973-11-11 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',879,'KORO',82 from dual union all
select 3349,'1955-A491','1955-11-26 00:00:00','military missile','success',1214,'N-69C 52-10977','','','','Test',224,'',10 from dual union all
select 33490,'1973-Y466','1973-11-13 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',88 from dual union all
select 33491,'1973-Y467','1973-11-14 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 33492,'1973-Y468','1973-11-14 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.156','TERLS-604','','Aeron',1467,'',85 from dual union all
select 33493,'1973-Y469','1973-11-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 33494,'1973-Y47','1973-01-27 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 33495,'1973-Y470','1973-11-15 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 33496,'1973-Y471','1973-11-16 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',88 from dual union all
select 33497,'1973-Y472','1973-11-17 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.157','TERLS-605','','Aeron',1467,'',87 from dual union all
select 33498,'1973-Y473','1973-11-18 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',85 from dual union all
select 33499,'1973-Y474','1973-11-21 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 335,'1944-S305','1944-12-25 11:55:00','military missile','success',1470,'19681','','','','Test',661,'',90 from dual union all
select 3350,'1955-A492','1955-12-01 00:00:00','suborbital rocket','success',122,'2','','','','Test',1057,'',30 from dual union all
select 33500,'1973-Y475','1973-11-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 33501,'1973-Y476','1973-11-21 15:15:00','suborbital rocket','unknown',725,'','ISRO-8.158','TERLS-612','','Aeron',1467,'',85 from dual union all
select 33502,'1973-Y477','1973-11-21 17:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',94 from dual union all
select 33503,'1973-Y478','1973-11-22 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 33504,'1973-Y479','1973-11-24 09:10:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',88 from dual union all
select 33505,'1973-Y48','1973-01-28 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',90 from dual union all
select 33506,'1973-Y480','1973-11-24 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.159','TERLS-614','','Aeron',1467,'',90 from dual union all
select 33507,'1973-Y481','1973-11-25 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',90 from dual union all
select 33508,'1973-Y482','1973-11-27 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',87 from dual union all
select 33509,'1973-Y483','1973-11-27 19:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 3351,'1955-A493','1955-11-30 00:00:00','military missile','success',255,'623-26','Last RPTV','','','Test',254,'',20 from dual union all
select 33510,'1973-Y484','1973-11-28 14:38:00','suborbital rocket','unknown',725,'','ISRO-8.160','TERLS-615','','Aeron',1467,'',86 from dual union all
select 33511,'1973-Y485','1973-11-28 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 33512,'1973-Y486','1973-11-28 22:15:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 33513,'1973-Y487','1973-11-30 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',82 from dual union all
select 33514,'1973-Y488','1973-12-01 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.161','TERLS-616','','Aeron',1467,'',83 from dual union all
select 33515,'1973-Y489','1973-12-04 01:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 33516,'1973-Y49','1973-01-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',87 from dual union all
select 33517,'1973-Y490','1973-12-04 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',87 from dual union all
select 33518,'1973-Y491','1973-12-05 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.162','TERLS-617','','Aeron',1467,'',84 from dual union all
select 33519,'1973-Y492','1973-12-05 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 3352,'1955-A494','1955-12-09 00:00:00','military missile','success',1214,'N-69C 52-10973','','','','Test',224,'',10 from dual union all
select 33520,'1973-Y493','1973-12-06 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',86 from dual union all
select 33521,'1973-Y494','1973-12-06 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 33522,'1973-Y495','1973-12-09 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',994,'NOKV',86 from dual union all
select 33523,'1973-Y496','1973-12-11 01:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 33524,'1973-Y497','1973-12-11 18:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 33525,'1973-Y498','1973-12-12 12:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',87 from dual union all
select 33526,'1973-Y499','1973-12-12 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.163','TERLS-618','','Aeron',1467,'',85 from dual union all
select 33527,'1973-Y50','1973-01-30 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 33528,'1973-Y500','1973-12-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 33529,'1973-Y501','1973-12-12 19:34:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 3353,'1955-A495','1955-12-09 14:50:00','suborbital rocket','failure',49,'NRL 38','XRV-N-13','','','Test',1869,'',0 from dual union all
select 33530,'1973-Y502','1973-12-12 22:33:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 33531,'1973-Y503','1973-12-15 02:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',81 from dual union all
select 33532,'1973-Y504','1973-12-15 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.164','TERLS-619','','Aeron',1467,'',87 from dual union all
select 33533,'1973-Y505','1973-12-17 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',82 from dual union all
select 33534,'1973-Y506','1973-12-18 01:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 33535,'1973-Y507','1973-12-19 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 33536,'1973-Y508','1973-12-19 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',82 from dual union all
select 33537,'1973-Y509','1973-12-19 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.165','TERLS-620','','Aeron',1467,'',83 from dual union all
select 33538,'1973-Y51','1973-01-31 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 33539,'1973-Y510','1973-12-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 3354,'1955-A496','1955-12-13 00:00:00','atmospheric rocket','success',442,'','F5D-1 Model','','','Test',1821,'',10 from dual union all
select 33540,'1973-Y511','1973-12-19 21:06:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 33541,'1973-Y512','1973-12-23 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',84 from dual union all
select 33542,'1973-Y513','1973-12-24 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',91 from dual union all
select 33543,'1973-Y514','1973-12-25 01:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 33544,'1973-Y515','1973-12-25 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',89 from dual union all
select 33545,'1973-Y516','1973-12-25 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 33546,'1973-Y517','1973-12-26 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.166','TERLS-621','','Aeron',1467,'',86 from dual union all
select 33547,'1973-Y518','1973-12-26 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 33548,'1973-Y519','1973-12-27 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 33549,'1973-Y52','1973-01-31 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',88 from dual union all
select 3355,'1955-A497','1955-12-14 00:00:00','atmospheric rocket','success',835,'','B-58 Model','','','Test',1821,'',10 from dual union all
select 33550,'1973-Y520','1973-12-27 18:31:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',88 from dual union all
select 33551,'1973-Y521','1973-09-21 04:20:00','suborbital rocket','success',725,'CIMO','','','','Meteo',348,'KORO',80 from dual union all
select 33552,'1973-Y522','1973-09-26 14:10:00','suborbital rocket','success',725,'CIMO','','','','Meteo',348,'KORO',80 from dual union all
select 33553,'1973-Y523','1973-09-27 14:10:00','suborbital rocket','success',725,'CIMO','','','','Meteo',348,'KORO',80 from dual union all
select 33554,'1973-Y524','1973-09-28 02:40:00','suborbital rocket','success',725,'CIMO','','','','Meteo',348,'KORO',80 from dual union all
select 33555,'1973-Y525','1973-10-01 03:40:00','suborbital rocket','success',725,'CIMO','','','','Meteo',348,'KORO',80 from dual union all
select 33556,'1973-Y526','1973-02-27 08:00:00','suborbital rocket','success',725,'','Atomic O','','','Aeron',636,'',85 from dual union all
select 33557,'1973-Y53','1973-01-31 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',88 from dual union all
select 33558,'1973-Y54','1973-01-31 14:37:00','suborbital rocket','unknown',725,'TERLS-522','ISRO-8.111','TERLS-522','','Aeron',1467,'',85 from dual union all
select 33559,'1973-Y55','1973-01-31 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 3356,'1955-A498','1955-12-16 00:00:00','military missile','success',1214,'N-69C 52-10974','','','','Test',224,'',10 from dual union all
select 33560,'1973-Y56','1973-01-31 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 33561,'1973-Y57','1973-01-31 18:58:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',88 from dual union all
select 33562,'1973-Y58','1973-01-31 21:01:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 33563,'1973-Y59','1973-02-01 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',85 from dual union all
select 33564,'1973-Y60','1973-02-02 06:23:00','suborbital rocket','success',1281,'NASA 15.102UI','','','','Ionos',428,'',81 from dual union all
select 33565,'1973-Y61','1973-02-02 20:26:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',88 from dual union all
select 33566,'1973-Y62','1973-02-03 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 33567,'1973-Y63','1973-02-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',83 from dual union all
select 33568,'1973-Y64','1973-02-03 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',88 from dual union all
select 33569,'1973-Y65','1973-02-04 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',90 from dual union all
select 3357,'1955-A499','1955-12-21 00:00:00','atmospheric rocket','failure',545,'','Test','','','Test',1821,'',10 from dual union all
select 33570,'1973-Y66','1973-02-05 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',85 from dual union all
select 33571,'1973-Y67','1973-02-07 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 33572,'1973-Y68','1973-02-07 15:00:00','suborbital rocket','success',725,'','18th Sov Ant.Exp.','','','Aeron',961,'',87 from dual union all
select 33573,'1973-Y69','1973-02-07 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 33574,'1973-Y70','1973-02-07 18:56:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 33575,'1973-Y71','1973-02-07 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',89 from dual union all
select 33576,'1973-Y72','1973-02-08 14:35:00','suborbital rocket','unknown',725,'TERLS-524','ISRO-8.112','TERLS-524','','Aeron',1467,'',90 from dual union all
select 33577,'1973-Y73','1973-02-09 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 33578,'1973-Y74','1973-02-10 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual;
insert into LAUNCH
select 33579,'1973-Y75','1973-02-10 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 3358,'1955-A50','1955-01-27 00:00:00','military missile','unknown',339,'1560R','','','','Test',1895,'',40 from dual union all
select 33580,'1973-Y76','1973-02-13 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 33581,'1973-Y77','1973-02-14 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 33582,'1973-Y78','1973-02-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 33583,'1973-Y79','1973-02-14 17:07:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',95 from dual union all
select 33584,'1973-Y80','1973-02-14 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 33585,'1973-Y81','1973-02-16 17:31:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',80 from dual union all
select 33586,'1973-Y82','1973-02-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 33587,'1973-Y83','1973-02-20 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 33588,'1973-Y84','1973-02-21 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 33589,'1973-Y85','1973-02-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 3359,'1955-A500','1955-12-27 00:00:00','suborbital rocket','success',122,'1','Redwing prototype','','','Test',1057,'',30 from dual union all
select 33590,'1973-Y86','1973-02-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',85 from dual union all
select 33591,'1973-Y87','1973-02-22 14:30:00','suborbital rocket','unknown',725,'TERLS-529','ISRO-8.113','TERLS-529','','Aeron',1467,'',85 from dual union all
select 33592,'1973-Y88','1973-02-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1367,'SHIR',84 from dual union all
select 33593,'1973-Y89','1973-02-23 17:03:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 33594,'1973-Y90','1973-02-23 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',88 from dual union all
select 33595,'1973-Y91','1973-02-23 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',85 from dual union all
select 33596,'1973-Y92','1973-02-23 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',85 from dual union all
select 33597,'1973-Y93','1973-02-24 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 33598,'1973-Y94','1973-02-26 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',85 from dual union all
select 33599,'1973-Y95','1973-02-26 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',86 from dual union all
select 336,'1944-S306','1944-12-27 08:43:00','military missile','success',1470,'20853','Ma421','','','Test',661,'',90 from dual union all
select 3360,'1955-A501','1955-10-02 00:00:00','suborbital rocket','success',813,'I','Camp. 4 to Oct 11','','','Test/Aeron',666,'',40 from dual union all
select 33600,'1973-Y96','1973-02-26 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',878,'KORO',87 from dual union all
select 33601,'1973-Y97','1973-02-27 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',92 from dual union all
select 33602,'1973-Y98','1973-02-27 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',91 from dual union all
select 33603,'1973-Y99','1973-02-28 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 33604,'1974-001','1974-01-17 10:07:30','orbital','success',680,'53753-208','Kosmos-628','Zaliv','','Sat',457,'',null from dual union all
select 33605,'1974-002','1974-01-19 01:38:47','orbital','failure',382,'587/D100','Skynet 2A','Skynet IIA','','Sat',234,'',null from dual union all
select 33606,'1974-003','1974-01-24 15:00:01','orbital','success',1508,'77019  -178','Kosmos-629','Zenit-2M','','Sat',475,'',null from dual union all
select 33607,'1974-004','1974-01-30 11:00:00','orbital','success',1508,'78033  -735','Kosmos-630','Zenit-4MK','','Sat',475,'',null from dual union all
select 33608,'1974-005','1974-02-06 00:33:59','orbital','success',680,'65053-209','Kosmos-631','Tselina-OM','','Sat',457,'',null from dual union all
select 33609,'1974-006','1974-02-12 08:56:00','orbital','success',1508,'77019  -290','Kosmos-632','Zenit-4M','','Sat',562,'',null from dual union all
select 3361,'1955-A502','1955-10-05 00:00:00','military missile','success',1080,'F41','','','','Test',671,'',20 from dual union all
select 33610,'1974-007','1974-02-13 18:00:00','orbital','success',1403,'24B-10 (3B-44)','KH-8A','GAMBIT-3 Mission 4341','','Sat',1778,'',null from dual union all
select 33611,'1974-008','1974-02-16 05:00:00','orbital','success',816,'M-3C-1','Tansei-2','MS-T2','','Sat',835,'',null from dual union all
select 33612,'1974-009','1974-02-18 10:05:28','orbital','success',1131,'S190C','San Marco 4','San Marco C2','','Sat',1412,'',null from dual union all
select 33613,'1974-010','1974-02-27 11:04:58','orbital','success',678,'64643-161','Kosmos-633','DS-P1-Yu No. 71','','Sat',460,'',null from dual union all
select 33614,'1974-011','1974-03-05 11:38:00','orbital','success',1510,'76043  -350','Meteor','Meteor-M  No. 30','','Sat',475,'',null from dual union all
select 33615,'1974-012','1974-03-05 16:04:55','orbital','success',678,'64643-162','Kosmos-634','DS-P1-Yu No. 67','','Sat',460,'',null from dual union all
select 33616,'1974-013','1974-03-09 02:22:11','orbital','success',1131,'S188C','Miranda','X-4','','Sat',1779,'',null from dual union all
select 33617,'1974-014','1974-03-14 10:29:59','orbital','success',1508,'78031  -191','Kosmos-635','Zenit-2M','','Sat',475,'',null from dual union all
select 33618,'1974-015','1974-03-16 08:00:11','orbital','success',1365,'207/BIIA-19','DMSP 8531','DMSP Block 5B F-5','','Sat',1720,'',null from dual union all
select 33619,'1974-016','1974-03-20 08:30:00','orbital','success',1238,'A 15000-006','Kosmos-636','Zenit-4MK','','Sat',562,'',null from dual union all
select 3362,'1955-A503','1955-10-08 00:00:00','military missile','success',1080,'F40','SEPR 505 booster','','','Test',671,'',20 from dual union all
select 33620,'1974-017','1974-03-26 13:35:00','orbital','success',956,'282-01','Kosmos-637','11F638 GVM','','Sat',587,'',null from dual union all
select 33621,'1974-018','1974-04-03 07:30:00','orbital','success',1238,'A 15000-009','Kosmos-638','Soyuz 7K-TM No. 71-EPSA','','Sat',562,'',null from dual union all
select 33622,'1974-019','1974-04-04 08:30:00','orbital','success',1508,'76043  -734','Kosmos-639','Zenit-4MK','','Sat',473,'',null from dual union all
select 33623,'1974-020','1974-04-10 20:20:00','orbital','success',1426,'23D-9','HEXAGON SV8','HEXAGON SV-8','','Sat',1777,'',null from dual union all
select 33624,'1974-021','1974-04-11 12:22:59','orbital','success',1508,'78031  -192','Kosmos-640','Zenit-2M','','Sat',475,'',null from dual union all
select 33625,'1974-022','1974-04-13 23:33:03','orbital','success',385,'588/D101','Westar 1','Westar 1','','Sat',234,'',null from dual union all
select 33626,'1974-023','1974-04-20 20:53:44','orbital','success',809,'7702421-516','Molniya-1','Molniya-1','','Sat',475,'',null from dual union all
select 33627,'1974-024','1974-04-23 14:14:59','orbital','success',680,'65033-202','Kosmos-641/648','Strela-1M','','Sat',457,'',null from dual union all
select 33628,'1974-025','1974-04-24 11:50:00','orbital','success',1510,'78033  -347','Meteor','Meteor-M  No. 22','','Sat',475,'',null from dual union all
select 33629,'1974-026','1974-04-26 14:22:59','orbital','success',809,'76029  -554','Molniya-2','Molniya-2','','Sat',473,'',null from dual union all
select 3363,'1955-A504','1955-06-22 00:00:00','atmospheric rocket','success',836,'1249B-4695','Nike 460R','','','Test',1895,'',10 from dual union all
select 33630,'1974-027','1974-04-29 13:29:59','orbital','success',1508,'76043  -737 ','Kosmos-649','Zenit-4MK','','Sat',475,'',null from dual union all
select 33631,'1974-028','1974-04-29 17:10:00','orbital','success',680,'Yu47121-15','Kosmos-650','Geoid','','Sat',457,'',null from dual union all
select 33632,'1974-029','1974-05-15 07:30:00','orbital','success',1446,'51049-157','Kosmos-651','US-A reactor','','Sat',590,'',null from dual union all
select 33633,'1974-030','1974-05-15 08:30:00','orbital','success',1238,'A 15000-008','Kosmos-652','Zenit-4MK','','Sat',562,'',null from dual union all
select 33634,'1974-031','1974-05-15 12:30:00','orbital','success',1508,'78031  -193','Kosmos-653','Zenit-2M','','Sat',475,'',null from dual union all
select 33635,'1974-032','1974-05-17 06:53:15','orbital','success',1446,'67049-155','Kosmos-654','US-A reactor','','Sat',590,'',null from dual union all
select 33636,'1974-033','1974-05-17 09:31:00','orbital','failure',386,'590/D102','SMS 1','SMS A','','Sat',234,'',null from dual union all
select 33637,'1974-034','1974-05-17 11:00:00','orbital','success',680,'53749-167','Interkosmos-11','DS-U3-IK No. 4','','Sat',622,'',null from dual union all
select 33638,'1974-035','1974-05-21 06:15:59','orbital','success',680,'53719-152','Kosmos-655','Tselina-OM','','Sat',457,'',null from dual union all
select 33639,'1974-036','1974-05-27 07:25:00','orbital','success',1223,'A 15000-032','Kosmos-656','Soyuz 7K-T(A9) No. 61','','Sat',480,'',null from dual union all
select 3364,'1955-A505','1955-06-22 00:00:00','atmospheric rocket','success',836,'1249B-5243','Nike 461R','','','Test',1895,'',10 from dual union all
select 33640,'1974-037','1974-05-29 08:56:51','deep space','success',953,'282-02','Luna-22','E-8LS No. 206','','Sat',588,'',null from dual union all
select 33641,'1974-038','1974-05-30 12:44:59','orbital','success',1508,'76029  -762','Kosmos-657','Zenit-4MK','','Sat',475,'',null from dual union all
select 33642,'1974-039','1974-05-30 13:00:01','orbital','success',1425,'23C-9 (3C-27)','ATS 6','ATS F','','Sat UPF',274,'',null from dual union all
select 33643,'1974-040','1974-06-03 23:09:11','orbital','success',1133,'S191C','Explorer 52','Injun 6','','Sat',1779,'',null from dual union all
select 33644,'1974-041','1974-06-06 06:20:00','orbital','success',1508,'77033  -267','Kosmos-658','Zenit-2M','','Sat',562,'',null from dual union all
select 33645,'1974-042','1974-06-06 16:30:00','orbital','success',1403,'24B-11 (3B-45)','KH-8A','GAMBIT-3 Mission 4342','','Sat',1778,'',null from dual union all
select 33646,'1974-043','1974-06-13 12:29:59','orbital','success',1508,'77019  -753','Kosmos-659','Zenit-4MK','','Sat',475,'',null from dual union all
select 33647,'1974-044','1974-06-18 13:00:00','orbital','success',680,'53714-165','Kosmos-660','Vektor','','Sat',457,'',null from dual union all
select 33648,'1974-045','1974-06-21 09:03:00','orbital','success',680,'65053-205','Kosmos-661','Tselina-OM','','Sat',457,'',null from dual union all
select 33649,'1974-046','1974-06-24 22:38:00','orbital','success',949,'283-02','Salyut-3','Almaz 2 (11F71 No. 101-2','','Sat',587,'',null from dual union all
select 3365,'1955-A506','1955-06-29 00:00:00','atmospheric rocket','success',836,'1249B-4443','Nike 462R','','','Test',1895,'',10 from dual union all
select 33650,'1974-047','1974-06-26 12:29:57','orbital','success',678,'29543-163','Kosmos-662','DS-P1-I No. 14','','Sat',460,'',null from dual union all
select 33651,'1974-048','1974-06-27 15:39:50','orbital','success',680,'53749-160','Kosmos-663','Zaliv','','Sat',456,'',null from dual union all
select 33652,'1974-049','1974-06-29 12:50:02','orbital','success',1225,'7602983-  7','Kosmos-664','Zenit-4MT','','Sat',475,'',null from dual union all
select 33653,'1974-050','1974-06-29 15:59:58','orbital','success',809,'A 15000-121','Kosmos-665','Oko','','Sat',473,'',null from dual union all
select 33654,'1974-051','1974-07-03 18:51:08','orbital','success',1223,'S 15000-031','Soyuz-14','Soyuz 7K-T(A9) No. 62','','Sat',480,'',null from dual union all
select 33655,'1974-052','1974-07-09 14:39:58','orbital','success',1510,'76043  -349','Meteor','Meteor-Priroda No. 1','','Sat',475,'',null from dual union all
select 33656,'1974-053','1974-07-12 12:50:01','orbital','success',1508,'76029  -761','Kosmos-666','Zenit-4MK','','Sat',475,'',null from dual union all
select 33657,'1974-054','1974-07-14 05:17:00','orbital','success',166,'69F','STP P73-3','NTS 1','','Sat',1776,'',null from dual union all
select 33658,'1974-055','1974-07-16 11:51:01','orbital','success',1131,'S186C','Aeros 2','Aeros B','','Sat',1779,'',null from dual union all
select 33659,'1974-056','1974-07-23 01:23:01','orbital','success',809,'78033  -529','Molniya-2','Molniya-2','','Sat',475,'',null from dual union all
select 3366,'1955-A507','1955-06-29 00:00:00','atmospheric rocket','success',836,'1249B-5211','Nike 463R','','','Test',1895,'',10 from dual union all
select 33660,'1974-057','1974-07-25 07:00:00','orbital','success',1508,'77019  -289','Kosmos-667','Zenit-4M','','Sat',562,'',null from dual union all
select 33661,'1974-058','1974-07-25 12:06:00','orbital','success',678,'29543-165','Kosmos-668','DS-P1-Yu No. 74','','Sat',460,'',null from dual union all
select 33662,'1974-059','1974-07-26 06:59:59','orbital','success',1508,'76029  -180','Kosmos-669','Zenit-2M','','Sat',475,'',null from dual union all
select 33663,'1974-060','1974-07-29 12:00:00','orbital','success',956,'287-01','Molniya-1S','Molniya-1 11F658 No. 38','','Sat',588,'',null from dual union all
select 33664,'1974-061','1974-08-06 00:02:00','orbital','success',1238,'K 15000-011','Kosmos-670','Soyuz 7K-S No. 1L','','Sat',480,'',null from dual union all
select 33665,'1974-062','1974-08-07 12:49:59','orbital','success',1508,'76029  -760','Kosmos-671','Zenit-4MK','','Sat',475,'',null from dual union all
select 33666,'1974-063','1974-08-09 03:22:47','orbital','success',1365,'275/BIIA-22','DMSP 9532','DMSP Block 5C F-1','','Sat',1720,'',null from dual union all
select 33667,'1974-064','1974-08-12 06:25:02','orbital','success',1238,'K 15000-007','Kosmos-672','Soyuz 7K-TM No. 72-EPSA','','Sat',562,'',null from dual union all
select 33668,'1974-065','1974-08-14 15:45:00','orbital','success',1403,'24B-12 (3B-46)','KH-8A','GAMBIT-3 Mission 4343','','Sat',1778,'',null from dual;
insert into LAUNCH
select 33669,'1974-066','1974-08-16 03:41:00','orbital','success',1510,'78033  -325','Kosmos-673','Tselina-D','','Sat',475,'',null from dual union all
select 3367,'1955-A508','1955-06-29 00:00:00','atmospheric rocket','success',836,'1249B-5255','Nike 464R','','','Test',1895,'',10 from dual union all
select 33670,'1974-067','1974-08-26 19:58:05','orbital','success',1223,'S 15000-030','Soyuz-15','Soyuz 7K-T(A9) No. 63','','Sat',480,'',null from dual union all
select 33671,'1974-068','1974-08-29 07:39:59','orbital','success',1508,'78031  -774','Kosmos-674','Zenit-4MK','','Sat',562,'',null from dual union all
select 33672,'1974-069','1974-08-29 14:54:59','orbital','success',680,'53719-154','Kosmos-675','Geoid','','Sat',457,'',null from dual union all
select 33673,'1974-070','1974-08-30 14:07:39','orbital','success',1132,'S189C','ANS','ANS','','Sat',1779,'',null from dual union all
select 33674,'1974-071','1974-09-11 17:40:23','orbital','success',680,'53719-151','Kosmos-676','Strela-2M','','Sat',457,'',null from dual union all
select 33675,'1974-072','1974-09-19 14:57:00','orbital','success',680,'53753-206','Kosmos-677/684','Strela-1M','','Sat',457,'',null from dual union all
select 33676,'1974-073','1974-09-20 09:30:00','orbital','success',1508,'78031  -194','Kosmos-685','Zenit-2M','','Sat',562,'',null from dual union all
select 33677,'1974-074','1974-09-26 16:34:56','orbital','success',678,'45753-166','Kosmos-686','DS-P1-Yu No. 72','','Sat',460,'',null from dual union all
select 33678,'1974-075','1974-10-10 22:53:00','orbital','success',385,'589/D103','Westar 2','Westar 2','','Sat',234,'',null from dual union all
select 33679,'1974-076','1974-10-11 11:29:59','orbital','success',680,'53721-251 (225)','Kosmos-687','Vektor','','Sat',457,'',null from dual union all
select 3368,'1955-A509','1955-06-29 00:00:00','atmospheric rocket','success',836,'1249B-4440','Nike 465R','','','Test',1895,'',10 from dual union all
select 33680,'1974-077','1974-10-15 07:47:00','orbital','success',1130,'S187C','Ariel 5','UK 5','','Sat',1412,'',null from dual union all
select 33681,'1974-078','1974-10-18 15:00:00','orbital','success',1508,'77019  -754','Kosmos-688','Zenit-4MK','','Sat',473,'',null from dual union all
select 33682,'1974-079','1974-10-18 22:36:15','orbital','success',680,'53753-207','Kosmos-689','Zaliv','','Sat',457,'',null from dual union all
select 33683,'1974-080','1974-10-22 17:59:59','orbital','success',1238,'K 15000-010','Kosmos-690','Bion No. 2','','Sat',475,'',null from dual union all
select 33684,'1974-081','1974-10-24 12:38:41','orbital','success',809,'78033  -528','Molniya-1','Molniya-1 No. 26','','Sat',473,'',null from dual union all
select 33685,'1974-082','1974-10-25 09:30:01','orbital','success',1238,'K 15000-012','Kosmos-691','Zenit-4MK','','Sat',562,'',null from dual union all
select 33686,'1974-083','1974-10-28 10:17:00','orbital','success',1510,'77019  -352','Meteor','Meteor-M  No. 33','','Sat',475,'',null from dual union all
select 33687,'1974-084','1974-10-28 14:30:32','deep space','success',953,'285-01','Luna-23','E-8-5M No. 410','','Sat',588,'',null from dual union all
select 33688,'1974-085','1974-10-29 19:30:00','orbital','success',1426,'23D-4','HEXAGON SV9','HEXAGON SV-9','','Sat',1777,'',null from dual union all
select 33689,'1974-086','1974-10-31 10:00:00','orbital','success',680,'53721-253','Interkosmos-12','DS-U2-IK No. 4','','Sat',457,'',null from dual union all
select 3369,'1955-A51','1955-01-27 00:00:00','atmospheric rocket','unknown',576,'M-780-AO','','','','Test',1895,'',10 from dual union all
select 33690,'1974-087','1974-11-01 14:20:00','orbital','success',1508,'78031  -179','Kosmos-692','Zenit-2M','','Sat',475,'',null from dual union all
select 33691,'1974-088','1974-11-04 10:40:00','orbital','success',1225,'76029  -836','Kosmos-693','Zenit-4MT','','Sat',473,'',null from dual union all
select 33692,'1974-089','1974-11-15 17:11:00','orbital','success',381,'592/D104','NOAA 4','ITOS G','','Sat',1724,'',null from dual union all
select 33693,'1974-090','1974-11-16 11:45:00','orbital','success',1508,'76029  -763','Kosmos-694','Zenit-4MK','','Sat',475,'',null from dual union all
select 33694,'1974-091','1974-11-20 11:59:58','orbital','success',678,'45753-168','Kosmos-695','DS-P1-Yu No. 73','','Sat',460,'',null from dual union all
select 33695,'1974-092','1974-11-21 10:33:01','orbital','success',809,'78031  -572','Molniya-3','Molniya-3 No. 11','','Sat',473,'',null from dual union all
select 33696,'1974-093','1974-11-21 23:43:59','orbital','success',180,'AC-32','Intelsat 4 F-8','INTELSAT IV F8','','Sat',269,'',null from dual union all
select 33697,'1974-094','1974-11-23 00:28:01','orbital','success',382,'591/D105','Skynet 2B','Skynet IIB','','Sat',234,'',null from dual union all
select 33698,'1974-095','1974-11-27 11:44:59','orbital','success',1508,'78031  -197','Kosmos-696','Zenit-2M','','Sat',475,'',null from dual union all
select 33699,'1974-096','1974-12-02 09:40:00','orbital','success',1238,'K 15000-014','Soyuz-16','Soyuz 7K-TM No. 73-EPSA','','Sat',480,'',null from dual union all
select 337,'1944-S307','1944-12-28 08:34:00','military missile','success',1470,'20844','Ma423','','','Test',661,'',90 from dual union all
select 3370,'1955-A510','1955-07-13 00:00:00','atmospheric rocket','success',836,'1249B-1586','Nike 466R','','','Test',1895,'',10 from dual union all
select 33700,'1974-097','1974-12-10 07:11:02','deep space','success',1427,'23E-2/TC-2','Helios 1','Helios 1','','Sat',275,'',null from dual union all
select 33701,'1974-098','1974-12-13 13:30:00','orbital','success',1238,'A 15000-003','Kosmos-697','Yantar''-2K No. 2','','Sat',474,'',null from dual union all
select 33702,'1974-099','1974-12-17 11:45:00','orbital','success',1510,'77019  -353','Meteor','Meteor-M  No. 32','','Sat',475,'',null from dual union all
select 33703,'1974-100','1974-12-18 14:11:59','orbital','success',680,'53721-256','Kosmos-698','Tselina-OM','','Sat',456,'',null from dual union all
select 33704,'1974-101','1974-12-19 02:39:00','orbital','success',386,'599/D106','Symphonie 1','Symphonie MV1','','Sat',234,'',null from dual union all
select 33705,'1974-102','1974-12-21 02:19:59','orbital','success',809,'78033  -530','Molniya-2','Molniya-2','','Sat',473,'',null from dual union all
select 33706,'1974-103','1974-12-24 11:00:00','orbital','success',1446,'67014-21L','Kosmos-699','US-P','','Sat',591,'',null from dual union all
select 33707,'1974-104','1974-12-26 04:15:00','orbital','success',949,'284-02','Salyut-4','DOS 4 (17K No. 124)','','Sat',588,'',null from dual union all
select 33708,'1974-105','1974-12-26 11:59:59','orbital','success',680,'53731-270','Kosmos-700','Tsiklon-B','','Sat',456,'',null from dual union all
select 33709,'1974-106','1974-12-27 09:10:00','orbital','success',1508,'76043  -738','Kosmos-701','Zenit-4MK','','Sat',562,'',null from dual union all
select 3371,'1955-A511','1955-07-13 00:00:00','atmospheric rocket','success',836,'1249B-1974','Nike 467R','','','Test',1895,'',10 from dual union all
select 33710,'1974-A01','1974-01-01 00:00:00','atmospheric rocket','success',1343,'Sandia NRV','Parachute test','','','Test',1490,'',2 from dual union all
select 33711,'1974-A02','1974-01-07 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',48 from dual union all
select 33712,'1974-A03','1974-01-22 00:00:00','military missile','failure',1251,'PFT-02','SAFEGUARD M2-245','','','Interceptor',868,'',20 from dual union all
select 33713,'1974-A04','1974-02-07 07:41:00','suborbital rocket','failure',1270,'Sandia 428-','LRL ACS 10','','','XR Astron',843,'',5 from dual union all
select 33714,'1974-A05','1974-02-08 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',46 from dual union all
select 33715,'1974-A06','1974-02-12 03:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',47 from dual union all
select 33716,'1974-A07','1974-02-13 00:00:00','military missile','failure',1004,'16L','15A14 LKI-17','GCh 15F142','','Test',510,'',50 from dual union all
select 33717,'1974-A08','1974-02-14 00:00:00','military missile','success',1252,'FLA-79 Block V wh','Safeguard M2-48','','','Interceptor',868,'',10 from dual union all
select 33718,'1974-A09','1974-02-14 06:44:00','suborbital rocket','failure',1468,'CRL A09.307-3','Density HI Bal','','','Aeron',1047,'',10 from dual union all
select 33719,'1974-A10','1974-02-22 12:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',32 from dual union all
select 3372,'1955-A512','1955-07-13 00:00:00','atmospheric rocket','success',836,'1249B-1714','Nike 468R','','','Test',1895,'',10 from dual union all
select 33720,'1974-A11','1974-03-01 03:34:00','suborbital rocket','failure',837,'NASA 14.527UA','','','','Aeronomy',428,'',0 from dual union all
select 33721,'1974-A12','1974-03-20 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 33722,'1974-A13','1974-03-21 12:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',40 from dual union all
select 33723,'1974-A14','1974-03-21 12:00:00','suborbital rocket','failure',1281,'NASA 15.123GM','','Ozone','','Aeronomy',340,'',1 from dual union all
select 33724,'1974-A15','1974-04-08 12:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',40 from dual union all
select 33725,'1974-A16','1974-04-10 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 33726,'1974-A17','1974-04-16 00:00:00','military missile','success',1252,'FLA-81','SAFEGUARD M2-148','','','Interceptor',868,'',10 from dual union all
select 33727,'1974-A18','1974-04-27 00:00:00','military missile','failure',1004,'22L','15A14 LKI-20','GCh 15F141','','Test',490,'',1 from dual union all
select 33728,'1974-A19','1974-04-29 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',49 from dual union all
select 33729,'1974-A20','1974-05-01 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',43 from dual union all
select 3373,'1955-A513','1955-07-13 00:00:00','atmospheric rocket','success',836,'1249B-1972','Nike 469R','','','Test',1895,'',10 from dual union all
select 33730,'1974-A21','1974-05-10 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',49 from dual union all
select 33731,'1974-A22','1974-05-24 14:09:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',46 from dual union all
select 33732,'1974-A23','1974-05-29 19:48:00','suborbital rocket','success',891,'NASA 30.01WT','Hawk Motor Test','','','Test',1821,'',46 from dual union all
select 33733,'1974-A24','1974-05-30 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 33734,'1974-A25','1974-05-31 19:00:00','military missile','failure',917,'105','P-288 74 Cape 09','','','OT',231,'',40 from dual union all
select 33735,'1974-A26','1974-06-03 19:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',49 from dual union all
select 33736,'1974-A27','1974-06-30 01:11:00','suborbital rocket','failure',1468,'CRL A09.001-3','ALADDIN 74 IonMS','','','Aeron',1821,'',16 from dual union all
select 33737,'1974-A28','1974-06-30 01:26:00','suborbital rocket','failure',1468,'CRL A09.301-2','ALADDIN 74 TMA/Ba','','','Aeron Ba',1821,'',0 from dual union all
select 33738,'1974-A29','1974-06-30 08:45:00','suborbital rocket','failure',1468,'CRL A09.001-4','ALADDIN 74 IonMS','','','Aeron',1821,'',17 from dual union all
select 33739,'1974-A30','1974-07-10 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',44 from dual union all
select 3374,'1955-A514','1955-07-13 00:00:00','atmospheric rocket','success',836,'1249B-1898','Nike 470R','','','Test',1895,'',10 from dual union all
select 33740,'1974-A31','1974-08-02 00:00:00','military missile','success',1252,'PAVT-3','SAFEGUARD M2-248','','','Interceptor',868,'',10 from dual union all
select 33741,'1974-A32','1974-08-20 11:09:00','military missile','failure',1099,'','','','','Test',293,'B-52',10 from dual union all
select 33742,'1974-A33','1974-08-28 15:49:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',48 from dual union all
select 33743,'1974-A34','1974-08-28 21:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',49 from dual union all
select 33744,'1974-A35','1974-09-05 00:00:00','military missile','success',1449,'','8K84 UBP-142','Svinets','','OT',540,'',1000 from dual union all
select 33745,'1974-A36','1974-09-06 00:00:00','atmospheric rocket','success',618,'J-5','RAE Aero Model','','','Test',1836,'',15 from dual union all
select 33746,'1974-A37','1974-09-25 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',45 from dual union all
select 33747,'1974-A38','1974-10-01 00:00:00','suborbital rocket','failure',1277,'TM-354','MTV-II-1','','','Reentry Test',855,'',0 from dual union all
select 33748,'1974-A39','1974-09-30 20:40:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',48 from dual union all
select 33749,'1974-A40','1974-10-01 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 3375,'1955-A515','1955-07-22 00:00:00','atmospheric rocket','success',836,'1249B-1982','Nike 471R','','','Test',1895,'',10 from dual union all
select 33750,'1974-A41','1974-10-02 00:00:00','military missile','success',1252,'PAVT-4','SAFEGUARD M2-348','','','Interceptor',868,'',4 from dual union all
select 33751,'1974-A42','1974-09-03 19:40:00','military missile','failure',999,'N22502-07','8K67 UBP-47 OS-67-1A','GCh 8F674','','Test',519,'',100 from dual union all
select 33752,'1974-A43','1974-10-09 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',47 from dual union all
select 33753,'1974-A44','1974-10-16 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 33754,'1974-A45','1974-10-17 00:00:00','military missile','success',1252,'PAVT-5','SAFEGUARD M2-448','','','Interceptor',868,'',25 from dual union all
select 33755,'1974-A46','1974-10-18 13:43:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',42 from dual union all
select 33756,'1974-A47','1974-10-24 00:00:00','military missile','success',792,'','Mk 5 RV','','AIR MOBILE FEASIBILITY','Test',1897,'C-5A',6 from dual union all
select 33757,'1974-A48','1974-10-25 20:06:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',46 from dual union all
select 33758,'1974-A49','1974-10-29 20:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',48 from dual;
insert into LAUNCH
select 33759,'1974-A50','1974-10-31 00:00:00','atmospheric rocket','success',618,'J-6','RAE Aero Model','','','Test',1836,'',15 from dual union all
select 3376,'1955-A516','1955-07-22 00:00:00','atmospheric rocket','success',836,'1249B-1864','Nike 472R','','','Test',1895,'',10 from dual union all
select 33760,'1974-A51','1974-11-11 00:00:00','suborbital rocket','failure',1337,'Sandia 497-002','','','','Test',855,'',10 from dual union all
select 33761,'1974-A52','1974-11-11 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',47 from dual union all
select 33762,'1974-A53','1974-11-12 00:00:00','military missile','failure',1004,'28L','15A14 LKI-28','GCh 15F143U1','','Test',486,'',10 from dual union all
select 33763,'1974-A54','1974-11-20 00:00:00','atmospheric rocket','success',618,'J-7','RAE Aero Model','','','Test',1836,'',15 from dual union all
select 33764,'1974-A55','1974-11-21 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',47 from dual union all
select 33765,'1974-A56','1974-12-03 19:58:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',45 from dual union all
select 33766,'1974-A57','1974-12-18 11:04:00','suborbital rocket','failure',220,'','DLR G-BB4-65 PolCusp','','','Ionospheric',1435,'',0 from dual union all
select 33767,'1974-A58','1974-12-28 06:00:00','suborbital rocket','failure',117,'NRL NB24.275','Crab Occultation','','','XR Astron?',1873,'',10 from dual union all
select 33768,'1974-A59','1974-12-29 00:00:00','military missile','failure',1004,'32L','15A14 LKI-31','GCh 15F143U1','','Test',486,'',10 from dual union all
select 33769,'1974-A60','1974-01-26 09:59:00','suborbital rocket','failure',1349,'Sandia 152-202','LRL ACS-7C','','','XR Astron',845,'',0 from dual union all
select 3377,'1955-A517','1955-07-27 00:00:00','atmospheric rocket','success',836,'1249B-1791','Nike 473R','','','Test',1895,'',10 from dual union all
select 33770,'1974-A61','1974-07-24 03:08:00','suborbital rocket','success',1283,'NRL  NB22.252','FUV Schmidt','','','UV Astron/Aeron',1821,'',300 from dual union all
select 33771,'1974-A62','1974-03-16 17:51:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',4 from dual union all
select 33772,'1974-A63','1974-03-16 18:30:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',4 from dual union all
select 33773,'1974-A64','1974-03-11 22:51:00','atmospheric rocket','success',1309,'','SAMS/TATER','','','Test',1821,'',4 from dual union all
select 33774,'1974-A65','1974-03-29 22:35:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',4 from dual union all
select 33775,'1974-A66','1974-03-29 23:05:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',4 from dual union all
select 33776,'1974-A67','1974-03-30 02:06:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',4 from dual union all
select 33777,'1974-A68','1974-05-02 20:35:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',4 from dual union all
select 33778,'1974-A69','1974-05-07 14:45:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',4 from dual union all
select 33779,'1974-A70','1974-04-08 00:00:00','atmospheric rocket','success',1346,'Sandia','SAMS R487411','','','Test',1821,'',4 from dual union all
select 3378,'1955-A518','1955-08-03 00:00:00','atmospheric rocket','success',836,'1249B-5121','Nike 474R','','','Test',1895,'',10 from dual union all
select 33780,'1974-A71','1974-04-08 00:00:00','atmospheric rocket','success',1346,'Sandia','SAMS R487406','','','Test',1821,'',4 from dual union all
select 33781,'1974-A72','1974-01-26 18:00:00','atmospheric rocket','success',1346,'Sandia','SAMS','','','Test',1821,'',4 from dual union all
select 33782,'1974-A73','1974-04-08 17:54:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',4 from dual union all
select 33783,'1974-A74','1974-04-08 18:09:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',4 from dual union all
select 33784,'1974-A75','1974-05-09 00:00:00','atmospheric rocket','success',1309,'Roll Test Eval','','','','Test',1490,'',4 from dual union all
select 33785,'1974-A76','1974-02-16 16:58:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',4 from dual union all
select 33786,'1974-A77','1974-09-12 00:00:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',4 from dual union all
select 33787,'1974-F01','1974-02-11 13:48:02','orbital','failure',1427,'23E-1/TC-1','Centaur Proof Test','VDS','','Sat',275,'',180 from dual union all
select 33788,'1974-F02','1974-04-12 08:00:01','orbital','failure',1508,'76029  -757','','Zenit-4MK','','Sat',562,'',170 from dual union all
select 33789,'1974-F03','1974-05-23 12:16:51','orbital','failure',1238,'A 15000-005','','Yantar''-2K No. 1','','Sat',474,'',170 from dual union all
select 3379,'1955-A519','1955-08-03 00:00:00','atmospheric rocket','success',836,'1249B-5231','Nike 475R','','','Test',1895,'',10 from dual union all
select 33790,'1974-F04','1974-07-11 10:59:58','orbital','failure',678,'29543-164','','DS-P1-Yu No. 68','','Sat',460,'',30 from dual union all
select 33791,'1974-F05','1974-07-12 13:55:00','orbital','failure',492,'701-04','JSSW','JSSW','','Sat',810,'',185 from dual union all
select 33792,'1974-F06','1974-08-30 09:00:01','orbital','failure',1508,'78031  -196','','Zenit-2M','','Sat',473,'',200 from dual union all
select 33793,'1974-F07','1974-11-05 09:40:00','orbital','failure',296,'CZ2-1    (3)','FSW','FSW No. 1','','Sat',810,'',20 from dual union all
select 33794,'1974-M01','1974-05-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 33795,'1974-M02','1974-05-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 33796,'1974-M04','1974-10-01 00:00:00','military missile','success',77,'002','[Spurious]','','','Test',1343,'',50 from dual union all
select 33797,'1974-M05','1974-11-08 19:33:00','suborbital rocket','success',891,'NASA 30.02WT','','','','Test',1821,'',73 from dual union all
select 33798,'1974-M06','1974-03-21 09:51:00','suborbital rocket','success',1284,'','Temp. Profile','','','Aeron',350,'',70 from dual union all
select 33799,'1974-M07','1974-01-09 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-45','','','Meteo',350,'',75 from dual union all
select 338,'1944-S308','1944-12-28 10:18:00','military missile','success',1470,'20845','Ma422','','','Test',661,'',90 from dual union all
select 3380,'1955-A52','1955-01-27 00:00:00','atmospheric rocket','unknown',576,'804-AO','','','','Test',1895,'',10 from dual union all
select 33800,'1974-M08','1974-02-04 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-46','','','Meteo',350,'',75 from dual union all
select 33801,'1974-M09','1974-02-14 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-47','','','Meteo',350,'',75 from dual union all
select 33802,'1974-M10','1974-02-22 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-43B','EOLE-ISENTROPE No. 2','','Meteo',350,'',75 from dual union all
select 33803,'1974-M11','1974-02-25 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-44B','','','Meteo',350,'',75 from dual union all
select 33804,'1974-M12','1974-02-27 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-48','EOLE-ISENTROPE No. 3','','Meteo',350,'',75 from dual union all
select 33805,'1974-M13','1974-03-13 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-49','','','Meteo',350,'',76 from dual union all
select 33806,'1974-M14','1974-03-27 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-50','','','Meteo',350,'',75 from dual union all
select 33807,'1974-M15','1974-09-04 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-55','EOLE-ISENTROPE No. 4','','Meteo',350,'',75 from dual union all
select 33808,'1974-M16','1974-10-02 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-58','','','Meteo',350,'',75 from dual union all
select 33809,'1974-M17','1974-10-16 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-59','','','Meteo',350,'',75 from dual union all
select 3381,'1955-A520','1955-08-03 00:00:00','atmospheric rocket','success',836,'1249B-4732','Nike 476R','','','Test',1895,'',10 from dual union all
select 33810,'1974-M18','1974-10-30 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-60','','','Meteo',350,'',75 from dual union all
select 33811,'1974-M19','1974-11-13 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-61','','','Meteo',350,'',75 from dual union all
select 33812,'1974-M20','1974-04-17 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-51','','','Meteo',350,'',75 from dual union all
select 33813,'1974-M21','1974-05-02 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-52','','','Meteo',350,'',75 from dual union all
select 33814,'1974-M22','1974-05-15 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-53','','','Meteo',350,'',75 from dual union all
select 33815,'1974-M23','1974-05-29 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-54','','','Meteo',350,'',75 from dual union all
select 33816,'1974-M24','1974-09-11 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-56','','','Meteo',350,'',75 from dual union all
select 33817,'1974-M25','1974-09-18 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-57','','','Meteo',350,'',75 from dual union all
select 33818,'1974-M26','1974-11-27 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-62','','','Meteo',350,'',75 from dual union all
select 33819,'1974-M27','1974-11-29 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-63','','','Meteo',350,'',75 from dual union all
select 3382,'1955-A521','1955-08-10 00:00:00','atmospheric rocket','success',836,'1249B-5123','Nike 477R','','','Test',1895,'',10 from dual union all
select 33820,'1974-M28','1974-12-04 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-64','','','Meteo',350,'',75 from dual union all
select 33821,'1974-M29','1974-12-18 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-65','','','Meteo',350,'',75 from dual union all
select 33822,'1974-M30','1974-01-09 12:15:00','suborbital rocket','success',716,'INTA HW-7401','Prog. Meteo Nacional','','','Meteo',396,'',60 from dual union all
select 33823,'1974-M31','1974-01-11 11:38:00','suborbital rocket','success',716,'INTA HW-7402','Prog. Meteo Nacional','','','Meteo',396,'',60 from dual union all
select 33824,'1974-M32','1974-01-14 11:30:00','suborbital rocket','success',716,'INTA HW-7403','Prog. Meteo Nacional','','','Meteo',396,'',60 from dual union all
select 33825,'1974-M33','1974-01-16 11:30:00','suborbital rocket','success',716,'INTA HW-7404','Prog. Meteo Nacional','','','Meteo',396,'',60 from dual union all
select 33826,'1974-M34','1974-01-18 12:03:00','suborbital rocket','success',716,'INTA HW-7405','Prog. Meteo Nacional','','','Meteo',396,'',60 from dual union all
select 33827,'1974-M35','1974-01-19 12:56:00','suborbital rocket','success',716,'INTA HW-7406','Prog. Meteo Nacional','','','Meteo',396,'',60 from dual union all
select 33828,'1974-M36','1974-01-21 12:58:00','suborbital rocket','success',716,'INTA HW-7407','Prog. Meteo Nacional','','','Meteo',396,'',60 from dual union all
select 33829,'1974-M37','1974-01-23 13:01:00','suborbital rocket','success',716,'INTA HW-7408','Prog. Meteo Nacional','','','Meteo',396,'',60 from dual union all
select 3383,'1955-A522','1955-08-31 00:00:00','atmospheric rocket','success',836,'1249B-5124','Nike 478R','','','Test',1895,'',10 from dual union all
select 33830,'1974-M38','1974-01-25 12:59:00','suborbital rocket','success',716,'INTA HD-7401','Prog. Meteo Nacional','','','Meteo',396,'',60 from dual union all
select 33831,'1974-M39','1974-06-14 18:56:00','suborbital rocket','unknown',1506,'E 1-7845','','','','Meteo',1821,'',60 from dual union all
select 33832,'1974-M40','1974-03-15 19:17:00','suborbital rocket','success',1281,'NASA G1-7207','','','','Meteo',1821,'',60 from dual union all
select 33833,'1974-M41','1974-06-01 00:00:00','suborbital rocket','success',361,'','Dardo de Invest. Met.','','','Meteo',308,'',70 from dual union all
select 33834,'1974-M42','1974-06-01 00:00:00','suborbital rocket','success',361,'','Dardo de Invest. Met.','','','Meteo',308,'',70 from dual union all
select 33835,'1974-M43','1974-05-02 00:00:00','military missile','success',1162,'','','RakSArt Training','','OT',984,'',50 from dual union all
select 33836,'1974-S01','1974-03-19 03:00:00','suborbital rocket','success',721,'1002','','','','Aeron?',1832,'',100 from dual union all
select 33837,'1974-S02','1974-01-01 00:00:00','suborbital rocket','success',722,'','','','','Aeron?',1832,'',140 from dual union all
select 33838,'1974-S03','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33839,'1974-S04','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 3384,'1955-A523','1955-09-14 00:00:00','atmospheric rocket','success',836,'1249B-5126','Nike 479R','','','Test',1895,'',10 from dual union all
select 33840,'1974-S05','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33841,'1974-S06','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33842,'1974-S07','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33843,'1974-S08','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33844,'1974-S09','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33845,'1974-S10','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33846,'1974-S100','1974-02-13 22:01:00','suborbital rocket','success',220,'AAF-4B-31','','','','Auroral/Ionos/Field',428,'',638 from dual union all
select 33847,'1974-S101','1974-02-14 06:07:00','suborbital rocket','success',228,'CRL A18.006-4','CVF','','','Aeron',1047,'',200 from dual union all
select 33848,'1974-S102','1974-02-15 12:15:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',1010,'',160 from dual;
insert into LAUNCH
select 33849,'1974-S103','1974-02-16 08:33:00','suborbital rocket','failure',44,'CRL A35.191-2','HI HI STAR','','','IR Astron',1895,'',111 from dual union all
select 3385,'1955-A524','1955-09-28 00:00:00','atmospheric rocket','success',836,'1249B-5136','Nike 480R','','','Test',1895,'',10 from dual union all
select 33850,'1974-S104','1974-02-16 10:03:00','suborbital rocket','success',1468,'CRL A09.400-1','Density','','','Aeron',1895,'',190 from dual union all
select 33851,'1974-S105','1974-02-28 00:00:00','military missile','success',1452,'-      2-010','15A30 LKI-13','GCh 15B103','','Test',532,'',1000 from dual union all
select 33852,'1974-S106','1974-02-19 07:43:00','suborbital rocket','success',846,'NH74-1','ICECAP 74A','','','Aeron',1047,'',150 from dual union all
select 33853,'1974-S107','1974-02-20 17:01:16','suborbital rocket','success',291,'ESRO C111/1','C111/1','','','Auroral',408,'',155 from dual union all
select 33854,'1974-S108','1974-02-20 18:07:00','suborbital rocket','success',867,'NASA 18.134IE','SSC S10/2 (Substorm)','','','Auroral',413,'',235 from dual union all
select 33855,'1974-S109','1974-02-20 18:22:22','suborbital rocket','success',291,'ESRO C111/2','C111/2','','','Auroral',408,'',152 from dual union all
select 33856,'1974-S11','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33857,'1974-S110','1974-02-20 19:07:00','suborbital rocket','success',867,'NASA 18.133IE','SSC S10/1 (Substorm)','','','Auroral',413,'',235 from dual union all
select 33858,'1974-S111','1974-02-21 00:00:00','military missile','success',1004,'18L','15A14 LKI-18','GCh 15F141','','Test',489,'',1000 from dual union all
select 33859,'1974-S112','1974-03-05 00:00:00','military missile','success',1452,'-      2-014','15A30 LKI-14','GCh 15B103','','Test',544,'',1000 from dual union all
select 3386,'1955-A525','1955-10-05 00:00:00','atmospheric rocket','success',836,'1249B-5161','Nike 481R','','','Test',1895,'',10 from dual union all
select 33860,'1974-S113','1974-02-21 09:16:00','suborbital rocket','success',228,'CRL A18.116-1','HIRIS','','','Aeron',1047,'',180 from dual union all
select 33861,'1974-S114','1974-02-22 04:59:00','suborbital rocket','success',215,'ADD-2A-128','Auroral VUV','','','Auroral/Aeron/Ionos',428,'',131 from dual union all
select 33862,'1974-S115','1974-02-22 06:31:00','suborbital rocket','success',837,'NASA 14.526UA','Auroral zone NO','','','Aeronomy',428,'',137 from dual union all
select 33863,'1974-S116','1974-02-24 00:00:00','military missile','success',1004,'20L','15A14 LKI-19','GCh 15F143U1','','Test',483,'',1000 from dual union all
select 33864,'1974-S117','1974-02-24 14:10:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',1010,'',165 from dual union all
select 33865,'1974-S118','1974-02-25 07:38:00','suborbital rocket','success',228,'CRL A18.219-1','MUTI','','','Aeron',1047,'',195 from dual union all
select 33866,'1974-S119','1974-02-26 00:00:00','military missile','success',744,'','15A15 LKI-15','','','Test',530,'',1000 from dual union all
select 33867,'1974-S12','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33868,'1974-S120','1974-02-26 18:17:00','suborbital rocket','failure',228,'NASA 21.18US','CU-He1 + LPSP-EUV3','','','Solar EUV',1873,'',98 from dual union all
select 33869,'1974-S121','1974-02-28 02:53:00','suborbital rocket','success',227,'AKF-VB-39','ISIS-2 pass','','','Auroral',428,'',246 from dual union all
select 3387,'1955-A526','1955-10-05 00:00:00','atmospheric rocket','success',836,'1249B-5234','Nike 482R','','','Test',1895,'',10 from dual union all
select 33870,'1974-S122','1974-03-01 00:00:00','military missile','success',744,'','15A15 LKI-16','','','Test',541,'',1000 from dual union all
select 33871,'1974-S123','1974-03-01 00:00:00','military missile','success',1420,'B-85','Mk 6 SSTTP T6','SAFSCOM M2-31','','Target',1658,'',1300 from dual union all
select 33872,'1974-S124','1974-03-02 05:27:00','suborbital rocket','success',837,'NASA 14.528UA','','','','Aeronomy',428,'',135 from dual union all
select 33873,'1974-S125','1974-03-05 10:35:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',1010,'',162 from dual union all
select 33874,'1974-S126','1974-03-05 23:36:00','suborbital rocket','success',837,'NASA 14.518GA','Auroral VUV','','','Aeronomy',428,'',156 from dual union all
select 33875,'1974-S127','1974-03-06 00:00:00','military missile','success',162,'73F','USN Mk 4 RV','BMRS Mk 4 SFT-1','','Reentry Test',1686,'',1400 from dual union all
select 33876,'1974-S128','1974-03-09 00:00:00','military missile','success',136,'H013','Mk 12 RV','HAVE MILL 2','','Reentry Test',1784,'',450 from dual union all
select 33877,'1974-S129','1974-03-14 09:20:09','suborbital rocket','success',867,'NASA 18.158IE','','','','Auroral',1047,'',189 from dual union all
select 33878,'1974-S13','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33879,'1974-S130','1974-03-12 00:00:00','military missile','success',796,'','ERCS','ERCS GIANT MOON 5','GIANT MOON 5','Comms',1700,'',1300 from dual union all
select 3388,'1955-A527','1955-10-19 00:00:00','atmospheric rocket','success',836,'1249B-5237','Nike 483R','','','Test',1895,'',10 from dual union all
select 33880,'1974-S131','1974-03-13 19:55:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',1010,'',171 from dual union all
select 33881,'1974-S132','1974-03-14 12:35:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',1010,'',177 from dual union all
select 33882,'1974-S133','1974-03-14 17:15:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',1010,'',177 from dual union all
select 33883,'1974-S134','1974-03-14 19:35:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/CR/Plas',1010,'',162 from dual union all
select 33884,'1974-S135','1974-03-15 20:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',1010,'',173 from dual union all
select 33885,'1974-S136','1974-03-16 03:35:00','suborbital rocket','success',41,'NASA 26.11GG','','','','UV Astron?',1895,'',199 from dual union all
select 33886,'1974-S137','1974-03-16 12:28:15','suborbital rocket','success',1116,'','Contiga Ba ISPIDA','','','Aeron/Fields',1047,'',576 from dual union all
select 33887,'1974-S138','1974-03-16 13:48:00','suborbital rocket','success',867,'Acanta Contiga','','','','Aeronomy',1047,'',270 from dual union all
select 33888,'1974-S139','1974-03-19 19:02:00','suborbital rocket','success',1283,'','EUV Airglow','','','Aeron',1821,'',300 from dual union all
select 33889,'1974-S14','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 3389,'1955-A528','1955-10-19 00:00:00','atmospheric rocket','success',836,'1249B-5217','Nike 484R','','','Test',1895,'',10 from dual union all
select 33890,'1974-S140','1974-03-17 20:15:00','suborbital rocket','unknown',740,'MR-12','Composition','','','Aeron/Ionos/CR/Plas',1010,'',167 from dual union all
select 33891,'1974-S141','1974-03-19 17:30:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/CR/Plasma',1010,'',146 from dual union all
select 33892,'1974-S142','1974-03-21 04:13:00','suborbital rocket','success',41,'NASA 26.29UE','Auroral','','','Plasma',428,'',251 from dual union all
select 33893,'1974-S143','1974-03-21 06:45:00','suborbital rocket','success',839,'NASA 10.411GM','GSFC Ozone','','','Aeronomy',340,'',83 from dual union all
select 33894,'1974-S144','1974-03-21 09:31:00','suborbital rocket','success',839,'NASA 10.420GM','GSFC Ozone','','','Aeronomy',340,'',89 from dual union all
select 33895,'1974-S145','1974-03-21 16:43:00','suborbital rocket','success',740,'MR-12','Ionos','','','Aeron/Ionos',1010,'',178 from dual union all
select 33896,'1974-S146','1974-03-22 06:45:00','suborbital rocket','success',839,'NASA 10.421GM','GSFC Grenade 277 CSG','','','Aeronomy',340,'',119 from dual union all
select 33897,'1974-S147','1974-03-22 09:31:00','suborbital rocket','success',839,'NASA 10.422GM','GSFC Grenade 278 CSG','','','Aeronomy',340,'',121 from dual union all
select 33898,'1974-S148','1974-03-23 00:00:00','military missile','success',162,'97F','ACE FTV-2','BMRS ACE FTV-2','','Reentry Test',1688,'',1400 from dual union all
select 33899,'1974-S149','1974-03-23 08:11:00','suborbital rocket','failure',1273,'Barebird','Acanta-Contiga','','','Auroral',1047,'',200 from dual union all
select 339,'1944-S309','1944-12-30 16:27:00','military missile','success',1470,'20820','Ma414','','','Test',661,'',90 from dual union all
select 3390,'1955-A529','1955-10-19 00:00:00','atmospheric rocket','success',836,'1249B-5174','Nike 485R','','','Test',1895,'',10 from dual union all
select 33900,'1974-S15','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33901,'1974-S150','1974-03-24 00:00:00','military missile','success',136,'H014','Mk 12 RV','HAVE MILL 3','','Reentry Test',1784,'',450 from dual union all
select 33902,'1974-S151','1974-03-28 04:00:00','military missile','success',796,'','Mk 11 RV','FOT GT117M-1','GLORY TRIP 117M-1','OT',1702,'',1300 from dual union all
select 33903,'1974-S152','1974-03-28 04:05:00','suborbital rocket','success',1468,'CRL A09.303-3','Churchill 74','','','Aeron/Ionos',428,'',129 from dual union all
select 33904,'1974-S153','1974-03-29 02:00:01','suborbital rocket','success',220,'','EQUION','','','Ionos/Eclipse',323,'',742 from dual union all
select 33905,'1974-S154','1974-03-29 04:24:00','suborbital rocket','success',740,'IPOCAMP','Franco/Sov (1/4)','','','Aeron/Ionos',1010,'',167 from dual union all
select 33906,'1974-S155','1974-04-03 00:00:00','ballistic missile test','success',1497,'3','K65UP No. 3','','','Ionos/Solar?',621,'',1500 from dual union all
select 33907,'1974-S156','1974-04-01 22:37:00','suborbital rocket','success',920,'P174H','Atomic oxygen','','','Atmospheric',1447,'',142 from dual union all
select 33908,'1974-S157','1974-04-01 22:56:00','suborbital rocket','success',920,'P178H','Atomic oxygen','','','Atmospheric',1447,'',134 from dual union all
select 33909,'1974-S158','1974-04-02 04:18:00','military missile','success',796,'','Mk 11 RV','FOT GT119M','GLORY TRIP 119M','OT',1697,'',1300 from dual union all
select 3391,'1955-A53','1955-01-27 00:00:00','atmospheric rocket','success',836,'1947T','','','','Test',1895,'',10 from dual union all
select 33910,'1974-S159','1974-04-03 00:00:00','military missile','success',744,'','15A15 LKI-17','','','Test',541,'',1000 from dual union all
select 33911,'1974-S16','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33912,'1974-S160','1974-04-03 03:47:00','suborbital rocket','success',1468,'CRL A09.303-1','Churchill 74','','','Ionos',428,'',209 from dual union all
select 33913,'1974-S161','1974-04-03 05:25:00','suborbital rocket','success',920,'P173H','Atomic oxygen','','','Atmospheric',1447,'',133 from dual union all
select 33914,'1974-S162','1974-04-03 05:40:00','suborbital rocket','success',920,'P177H','Atomic oxygen','','','Atmospheric',1447,'',133 from dual union all
select 33915,'1974-S163','1974-04-03 14:22:00','suborbital rocket','success',740,'MR-12','Particles','','','Aeron/Ionos/CR/Plas',1010,'',184 from dual union all
select 33916,'1974-S164','1974-04-03 19:18:00','suborbital rocket','success',1468,'CRL A09.303-4','Churchill 74','','','Aeron',428,'',129 from dual union all
select 33917,'1974-S165','1974-04-05 01:25:00','military missile','success',799,'PVM-5','Mk 12 RV x 3','R&D','PVM-5','Test',1707,'',1300 from dual union all
select 33918,'1974-S166','1974-04-04 13:43:00','suborbital rocket','success',740,'MR-12','Ionos','','','Aeron/Ionos/CR/Plas',1010,'',178 from dual union all
select 33919,'1974-S167','1974-04-06 04:10:00','suborbital rocket','success',44,'NASA 17.12CG','','','','XR Astron',1895,'',237 from dual union all
select 3392,'1955-A530','1955-10-26 00:00:00','atmospheric rocket','success',836,'1249B-5208','Nike 486R','','','Test',1895,'',10 from dual union all
select 33920,'1974-S168','1974-04-09 05:21:00','suborbital rocket','success',1468,'CRL A09.303-2','Churchill 74','','','Aeron',428,'',169 from dual union all
select 33921,'1974-S169','1974-04-11 08:00:00','suborbital rocket','success',850,'NJ-74-1','ICECAP 74B','','','Auroral',1047,'',130 from dual union all
select 33922,'1974-S17','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33923,'1974-S170','1974-04-11 23:38:16','suborbital rocket','failure',132,'CRL A30.413-1','','','','Ionos',1047,'',83 from dual union all
select 33924,'1974-S171','1974-04-12 23:24:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 658',500 from dual union all
select 33925,'1974-S172','1974-04-12 23:25:00','suborbital rocket','success',132,'CRL A30.413-2','VLF','','','Ionos',1047,'',129 from dual union all
select 33926,'1974-S173','1974-04-16 00:00:00','military missile','success',795,'','SSTTP','SSTTP M2-135','','Target',1701,'',1300 from dual union all
select 33927,'1974-S174','1974-04-16 16:25:00','military missile','success',999,'U22504-08','8K67 UBP-46 OS-67-1A','GCh 8F675','','Test',519,'',1000 from dual union all
select 33928,'1974-S175','1974-04-17 10:23:00','suborbital rocket','success',228,'NASA 21.26UE','ECHO III','','','Auroral',1047,'',275 from dual union all
select 33929,'1974-S176','1974-04-18 08:40:15','suborbital rocket','success',906,'CRL A10.312-3','ICECAP 74B Echo 3','','','Plasma',1047,'',185 from dual union all
select 3393,'1955-A531','1955-11-02 00:00:00','atmospheric rocket','success',836,'1249B-5651','Nike 487R','','','Test',1895,'',10 from dual union all
select 33930,'1974-S177','1974-04-19 04:30:00','suborbital rocket','success',837,'NASA 14.520UI','','UIUC Elec Spec.','','Ionos',1821,'',199 from dual union all
select 33931,'1974-S178','1974-04-22 18:02:00','suborbital rocket','success',37,'CRL A04.208-2','EUV','','','Aeron/Solar EUV?',1895,'',212 from dual union all
select 33932,'1974-S179','1974-04-23 01:03:00','suborbital rocket','success',1211,'SL1207','UCL Ionos-Ba','','','Ionos/Aeron',1839,'',177 from dual union all
select 33933,'1974-S18','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33934,'1974-S180','1974-04-24 13:00:00','suborbital rocket','success',1028,'','First test','','','Test',1361,'',280 from dual union all
select 33935,'1974-S181','1974-04-27 08:10:00','military missile','success',795,'','SSTTP','SSTTP M2-35','','Target',1698,'',1300 from dual union all
select 33936,'1974-S182','1974-04-27 00:00:00','military missile','success',744,'','15A15 LKI-18','','','Test',541,'',1000 from dual union all
select 33937,'1974-S183','1974-04-28 00:00:00','military missile','success',744,'','15A15 LKI-19','','','Test',530,'',1000 from dual union all
select 33938,'1974-S184','1974-05-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual;
insert into LAUNCH
select 33939,'1974-S185','1974-04-30 16:55:00','suborbital rocket','success',920,'P72H','Quiet ionosphere','','','Ionospheric',1447,'',126 from dual union all
select 3394,'1955-A532','1955-11-09 00:00:00','atmospheric rocket','success',836,'1249B-5141','Nike 488R','','','Test',1895,'',10 from dual union all
select 33940,'1974-S186','1974-05-01 16:17:00','military missile','success',162,'54F','USN Mk 4 RV','BMRS Mk 4 SFT-2','','Reentry Test',1686,'',1400 from dual union all
select 33941,'1974-S187','1974-05-02 22:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT44GM','GLORY TRIP 44GM','OT',1697,'',1300 from dual union all
select 33942,'1974-S188','1974-05-07 13:00:01','suborbital rocket','success',493,'3','DLR C-FFK-47','','','Test',307,'',120 from dual union all
select 33943,'1974-S189','1974-05-07 19:00:00','military missile','success',917,'103','P-280 74 Cape 01','','','OT',231,'',180 from dual union all
select 33944,'1974-S19','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33945,'1974-S190','1974-05-07 19:15:00','military missile','success',917,'104','P-281 74 Cape 02','','','OT',231,'',180 from dual union all
select 33946,'1974-S191','1974-05-07 20:05:00','military missile','success',917,'102','P-282 74 Cape 03','','','OT',231,'',180 from dual union all
select 33947,'1974-S192','1974-05-07 20:21:00','military missile','success',917,'101','P-283 74 Cape 04','','','OT',231,'',180 from dual union all
select 33948,'1974-S193','1974-05-08 00:00:00','suborbital rocket','failure',133,'NASA 12.21GT','','','','Test',1895,'',155 from dual union all
select 33949,'1974-S194','1974-05-14 18:43:00','suborbital rocket','success',851,'NASA 28.01GT','','','','Test',1821,'',500 from dual union all
select 3395,'1955-A533','1955-11-16 00:00:00','atmospheric rocket','success',836,'1249B-5652','Nike 489R','','','Test',1895,'',10 from dual union all
select 33950,'1974-S195','1974-05-15 21:37:00','suborbital rocket','success',837,'NASA 14.482UI','','','','Aeron/Ionos',1821,'',138 from dual union all
select 33951,'1974-S196','1974-05-15 22:11:00','suborbital rocket','success',41,'NASA 26.18US','','','','Solar',1895,'',281 from dual union all
select 33952,'1974-S197','1974-05-15 23:00:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 659',500 from dual union all
select 33953,'1974-S198','1974-05-15 23:45:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 659',500 from dual union all
select 33954,'1974-S199','1974-05-19 00:00:00','military missile','success',136,'H015','Mk 12 Mod','HAVE MILL 4','','Reentry Test',1784,'',450 from dual union all
select 33955,'1974-S20','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33956,'1974-S200','1974-05-21 13:04:00','military missile','success',917,'109','P-284 74 Cape 05','','','OT',231,'',180 from dual union all
select 33957,'1974-S201','1974-05-21 13:18:00','military missile','success',917,'110','P-285 74 Cape 06','','','OT',231,'',180 from dual union all
select 33958,'1974-S202','1974-05-21 13:28:00','military missile','failure',917,'111','P-286 74 Cape 07','','','OT',231,'',40 from dual union all
select 33959,'1974-S203','1974-05-21 14:43:00','military missile','success',917,'112','P-287 74 Cape 08','','','OT',231,'',180 from dual union all
select 3396,'1955-A534','1955-11-16 00:00:00','atmospheric rocket','success',836,'1249B-5133','Nike 490R','','','Test',1895,'',10 from dual union all
select 33960,'1974-S204','1974-05-28 00:00:00','military missile','success',744,'','15A15 LKI-20','','','Test',541,'',1000 from dual union all
select 33961,'1974-S205','1974-05-29 00:00:00','military missile','success',136,'H016','Mk 12 RV','HAVE MILL 5','','Reentry Test',1784,'',450 from dual union all
select 33962,'1974-S206','1974-05-31 00:00:00','military missile','success',744,'','15A15 LKI-21','','','Test',530,'',1000 from dual union all
select 33963,'1974-S207','1974-06-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 33964,'1974-S208','1974-05-31 14:59:59','military missile','success',931,'A3T','DASO','','','OT',421,'SSBN 600',1000 from dual union all
select 33965,'1974-S209','1974-05-31 19:15:00','military missile','success',917,'106','P-289 74 Cape 10','','','OT',231,'',180 from dual union all
select 33966,'1974-S21','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33967,'1974-S210','1974-05-31 20:05:00','military missile','success',917,'107','P-290 74 Cape 11','','','OT',231,'',180 from dual union all
select 33968,'1974-S211','1974-05-31 20:21:00','military missile','success',917,'108','P-291 74 Cape 12','','','OT',231,'',180 from dual union all
select 33969,'1974-S212','1974-06-09 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 599',1000 from dual union all
select 3397,'1955-A535','1955-11-18 00:00:00','atmospheric rocket','success',836,'1249B-7024','Nike 491R','','','Test',1895,'',10 from dual union all
select 33970,'1974-S213','1974-06-09 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 599',1000 from dual union all
select 33971,'1974-S214','1974-06-09 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 599',1000 from dual union all
select 33972,'1974-S215','1974-06-09 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 599',1000 from dual union all
select 33973,'1974-S216','1974-06-09 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 599',1000 from dual union all
select 33974,'1974-S217','1974-06-12 00:00:00','military missile','success',755,'','','','','OT',301,'',600 from dual union all
select 33975,'1974-S218','1974-06-13 00:00:00','military missile','success',136,'H017','Mk 12 Blunt','HAVE MILL 6','','Reentry Test',1784,'',450 from dual union all
select 33976,'1974-S219','1974-06-15 05:15:00','suborbital rocket','success',228,'NASA 21.17UG','','UCB EUV?/XR','','XR Astron',1873,'',187 from dual union all
select 33977,'1974-S22','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33978,'1974-S220','1974-06-18 00:00:00','suborbital rocket','success',846,'DASA HARSES','','','','Aeron?',1873,'',100 from dual union all
select 33979,'1974-S221','1974-06-18 00:00:00','military missile','success',1105,'S-1E-2604','','','','OT',301,'',1000 from dual union all
select 3398,'1955-A536','1955-11-23 00:00:00','atmospheric rocket','success',836,'1249B-5168','Nike 492R','','','Test',1895,'',10 from dual union all
select 33980,'1974-S222','1974-06-18 05:00:00','suborbital rocket','success',1211,'SL1104','MSSL XR14','','','XR Astronomy',1839,'',234 from dual union all
select 33981,'1974-S223','1974-06-18 14:15:00','suborbital rocket','success',117,'NRL NB24.268','','','','Test/Astron',1873,'',300 from dual union all
select 33982,'1974-S224','1974-06-19 17:00:00','suborbital rocket','success',41,'NASA 26.36US','','','','Solar',1895,'',200 from dual union all
select 33983,'1974-S225','1974-06-20 05:11:00','suborbital rocket','success',1347,'Sandia','Eclipse','','','Solar UV Eclipse',930,'',320 from dual union all
select 33984,'1974-S226','1974-06-20 05:11:00','suborbital rocket','success',1347,'Sandia','Eclipse','','','Solar UV Eclipse',930,'',320 from dual union all
select 33985,'1974-S227','1974-06-20 08:15:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/CR/Plas',1010,'',160 from dual union all
select 33986,'1974-S228','1974-06-20 20:03:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/CR/Plas',1010,'',160 from dual union all
select 33987,'1974-S229','1974-06-21 00:00:00','military missile','success',744,'','15A15 LKI-22','','','Test',541,'',1000 from dual union all
select 33988,'1974-S23','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33989,'1974-S230','1974-06-21 12:06:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',1010,'',160 from dual union all
select 3399,'1955-A537','1955-12-01 00:00:00','atmospheric rocket','success',836,'1249B-5151','Nike 493R','','','Test',1895,'',10 from dual union all
select 33990,'1974-S231','1974-06-22 07:30:00','suborbital rocket','success',41,'NASA 26.26GG','13-in RC Flight 2','','','UV Astron?',1895,'',250 from dual union all
select 33991,'1974-S232','1974-06-23 00:00:00','military missile','success',136,'H018','Mk 12 Decoy','HAVE MILL 7','','Reentry Test',1784,'',450 from dual union all
select 33992,'1974-S233','1974-06-24 00:00:00','military missile','success',1004,'25L','15A14 LKI-21','GCh 15F143U1','','Test',492,'',1000 from dual union all
select 33993,'1974-S234','1974-06-27 18:45:00','suborbital rocket','success',41,'NASA 26.15CS','','','','Solar',1895,'',222 from dual union all
select 33994,'1974-S235','1974-06-28 15:15:00','military missile','success',162,'82F','USN Mk 4 RV','BMRS Mk 4 SFT-3','','Reentry Test',1686,'',1400 from dual union all
select 33995,'1974-S236','1974-06-29 01:10:00','suborbital rocket','success',848,'CRL A07.105-7','ALADDIN 74 Pre-Alad','','','Aeron/Release',1821,'',184 from dual union all
select 33996,'1974-S238','1974-06-29 19:45:00','suborbital rocket','success',37,'CRL A04.208-1','ALADDIN 74 EUV Airg.','','','Aeron',1821,'',211 from dual union all
select 33997,'1974-S239','1974-06-29 20:00:00','suborbital rocket','success',1468,'CRL A09.213-2','ALADDIN 74 NMS/E','','','Aeron/Plasma',1821,'',110 from dual union all
select 33998,'1974-S24','1974-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 33999,'1974-S240','1974-06-29 20:16:00','suborbital rocket','success',228,'NASA 21.28IA','Aladdin 74','','','Aeronomy',1821,'',234 from dual union all
select 34,'1943-S12','1943-06-29 00:00:00','military missile','success',8,'40','','','','Test',675,'',95 from dual union all
select 340,'1944-S31','1944-08-13 16:07:00','military missile','success',1470,'18296','Ma100','','','Training',661,'',90 from dual union all
select 3400,'1955-A538','1955-12-07 00:00:00','atmospheric rocket','success',836,'1249B-5184','Nike 494R','','','Test',1895,'',10 from dual union all
select 34000,'1974-S241','1974-06-29 20:33:00','suborbital rocket','success',867,'CRL A08.306-4','ALADDIN 74 Sphere','','','Aeron',1821,'',189 from dual union all
select 34001,'1974-S242','1974-06-29 21:05:00','suborbital rocket','success',867,'CRL A08.207-2','ALADDIN 74 NO/Photom','','','Aeron',1821,'',186 from dual union all
select 34002,'1974-S243','1974-06-29 21:20:00','suborbital rocket','success',1468,'CRL A09.301-9','ALADDIN 74','','','Aeron',1821,'',150 from dual union all
select 34003,'1974-S244','1974-06-30 00:10:00','suborbital rocket','unknown',839,'E2-7313','ALADDIN 74','','','Aeron/EUV Astron',1821,'',100 from dual union all
select 34004,'1974-S245','1974-07-06 00:00:00','military missile','success',1452,'-      2-019','15A30 LKI-18','GCh 15B103','','Test',532,'',1000 from dual union all
select 34005,'1974-S246','1974-06-30 01:06:00','suborbital rocket','success',1468,'CRL A09.301-1','ALADDIN 74 TMA/Ba','','','Aeron',1821,'',203 from dual union all
select 34006,'1974-S247','1974-06-30 02:20:00','suborbital rocket','success',837,'NASA 14.521UI','ALADDIN 74','','','Ionos',1821,'',196 from dual union all
select 34007,'1974-S248','1974-06-30 03:45:00','suborbital rocket','success',1468,'CRL A09.301-5','ALADDIN 74 Na/Li','','','Aeron',1821,'',201 from dual union all
select 34008,'1974-S249','1974-06-30 04:40:00','suborbital rocket','success',837,'NASA 14.522UI','ALADDIN 74','','','Ionos',1821,'',191 from dual union all
select 34009,'1974-S25','1974-01-10 00:00:00','ballistic missile test','success',1497,'2','K65UP No. 2','','','Ionos/Solar?',621,'',1500 from dual union all
select 3401,'1955-A539','1955-12-14 00:00:00','atmospheric rocket','success',836,'1249B-5860','Nike 495R','','','Test',1895,'',10 from dual union all
select 34010,'1974-S250','1974-06-30 05:30:00','suborbital rocket','success',837,'NASA 14.529UA','ALADDIN 74','','','Aeronomy',1821,'',134 from dual union all
select 34011,'1974-S251','1974-06-30 05:58:00','suborbital rocket','success',1468,'CRL A09.207-4','ALADDIN 74 Sphere','','','Aeron',1821,'',188 from dual union all
select 34012,'1974-S252','1974-06-30 07:10:00','suborbital rocket','success',867,'CRL A08.207-1','ALADDIN 74 NO/Photom','','','Aeron',1821,'',185 from dual union all
select 34013,'1974-S253','1974-06-30 07:30:00','suborbital rocket','success',1468,'CRL A09.301-4','ALADDIN 74 TMA/Phot','','','Aeron',1821,'',178 from dual union all
select 34014,'1974-S254','1974-06-30 07:50:00','suborbital rocket','success',1468,'CRL A09.213-1','ALADDIN 74','','','Aeron',1821,'',140 from dual union all
select 34015,'1974-S255','1974-06-30 08:35:00','suborbital rocket','success',906,'CRL A10.301-6','ALADDIN 74 TMA/Ba','','','Aeron',1821,'',191 from dual union all
select 34016,'1974-S256','1974-06-30 09:00:00','suborbital rocket','success',837,'NASA 14.508UA','ALADDIN 74','','','Aeronomy',1821,'',204 from dual union all
select 34017,'1974-S257','1974-06-30 09:15:00','suborbital rocket','success',837,'NASA 14.523UI','ALADDIN 74','','','Ionos',1821,'',199 from dual union all
select 34018,'1974-S258','1974-06-30 10:01:00','suborbital rocket','success',906,'CRL A10.209-2','ALADDIN 74 Sphere','','','Aeron',1821,'',228 from dual union all
select 34019,'1974-S259','1974-06-30 11:04:00','suborbital rocket','success',1347,'Sandia 281-','LRL ACS-4F','','','XR Astron',855,'',305 from dual union all
select 3402,'1955-A54','1955-01-27 00:00:00','atmospheric rocket','success',836,'1416A','','','','Test',1895,'',10 from dual union all
select 34020,'1974-S26','1974-12-25 00:00:00','military missile','success',1037,'V 29501-07','8K98 UBP-25','','','Test',477,'',1000 from dual union all
select 34021,'1974-S260','1974-06-30 12:55:00','suborbital rocket','success',1468,'CRL A09.105-9','ALADDIN 74 Na-Li','','','Aeron release',1821,'',152 from dual union all
select 34022,'1974-S261','1974-06-30 17:20:00','suborbital rocket','success',1468,'CRL A09.301-7','ALADDIN 74 Na-LI','','','Aeron release',1821,'',180 from dual union all
select 34023,'1974-S262','1974-07-02 12:16:00','suborbital rocket','success',867,'NASA 18.156IE-UE','','','','Plasma',1438,'',236 from dual union all
select 34024,'1974-S263','1974-07-04 00:00:00','military missile','success',1004,'26L','15A14 LKI-22','GCh 15F141','','Test',496,'',1000 from dual union all
select 34025,'1974-S264','1974-07-04 00:30:00','suborbital rocket','success',837,'NASA 14.433GE','Solar CR','','','Plasma',428,'',140 from dual union all
select 34026,'1974-S265','1974-07-05 15:32:00','suborbital rocket','success',837,'NASA 14.488GE','Solar CR','','','Plasma',428,'',141 from dual union all
select 34027,'1974-S266','1974-07-08 15:19:00','suborbital rocket','success',867,'NASA 18.157IE-UE','','','','Plasma',1438,'',237 from dual;
insert into LAUNCH
select 34028,'1974-S267','1974-07-09 00:00:00','military missile','success',796,'','Mk 11C RV','FOT GT120M','GLORY TRIP 120GM','OT',1699,'',1300 from dual union all
select 34029,'1974-S268','1974-07-11 15:40:00','military missile','success',795,'','SAMAST-02/MINT','ABRES SAMAST','','Reentry Test',1698,'',1300 from dual union all
select 3403,'1955-A540','1955-12-14 00:00:00','atmospheric rocket','success',836,'1249B-','Nike 496R','','','Test',1895,'',10 from dual union all
select 34030,'1974-S269','1974-07-16 14:59:59','military missile','success',931,'A3T','DASO','','','OT',421,'UK S26',1000 from dual union all
select 34031,'1974-S27','1974-01-30 12:32:00','military missile','success',135,'','HEART V4-6765','','','Reentry Test',1821,'',200 from dual union all
select 34032,'1974-S270','1974-07-19 00:58:00','military missile','success',795,'','SSTTP','SSTTP M2-146','','Target',1701,'',1300 from dual union all
select 34033,'1974-S271','1974-07-18 00:00:00','military missile','success',1251,'','SAFEGUARD TRM-133','','','Interceptor',868,'',100 from dual union all
select 34034,'1974-S272','1974-07-19 00:00:00','suborbital rocket','success',846,'','Low Light TV','','','Tech',1895,'',150 from dual union all
select 34035,'1974-S273','1974-07-20 06:10:00','suborbital rocket','success',37,'NASA 13.084UG','Wisc. Aerobee IV','','','XR Astron',1895,'',195 from dual union all
select 34036,'1974-S274','1974-07-22 00:00:00','suborbital rocket','success',1207,'SL1292','CHEVALINE S4','','','MIRV test',1839,'',210 from dual union all
select 34037,'1974-S275','1974-07-26 00:00:00','military missile','success',1004,'19L','15A14 LKI-23','GCh 15F142','','Test',510,'',1000 from dual union all
select 34038,'1974-S276','1974-07-26 15:23:00','military missile','success',931,'A3T','DASO','','','OT',421,'SSBN 602',1000 from dual union all
select 34039,'1974-S277','1974-08-01 00:00:00','military missile','success',795,'','SSTTP','SSTTP M2-46','','Target',1698,'',1300 from dual union all
select 3404,'1955-A541','1955-12-16 00:00:00','atmospheric rocket','success',836,'1249B-5139','Nike 497R','','','Test',1895,'',10 from dual union all
select 34040,'1974-S278','1974-08-09 15:12:59','military missile','success',931,'A3T','DASO','','','OT',421,'SSBN 608',1000 from dual union all
select 34041,'1974-S279','1974-08-14 18:08:00','suborbital rocket','success',837,'NASA 14.417UA','','','','Aeronomy',1821,'',133 from dual union all
select 34042,'1974-S28','1974-03-25 22:45:00','military missile','success',135,'','HEART V4-6766','','','Reentry Test',1821,'',200 from dual union all
select 34043,'1974-S280','1974-08-17 16:20:00','military missile','success',799,'SAC OT-45','3 RV','FOT GT45GM','GLORY TRIP 45GM','OT',1697,'',1300 from dual union all
select 34044,'1974-S281','1974-08-17 05:00:00','suborbital rocket','success',1468,'CRL A09.214-2','Vehicle Potential','','','Test',1895,'',161 from dual union all
select 34045,'1974-S282','1974-08-18 08:59:36','suborbital rocket','success',839,'','Ferdinand 36','','','Ionospheric',417,'',106 from dual union all
select 34046,'1974-S283','1974-08-19 00:00:00','military missile','success',1004,'27L','15A14 LKI-24','GCh 15F143U1','','Test',490,'',1000 from dual union all
select 34047,'1974-S284','1974-08-20 09:55:00','suborbital rocket','success',1070,'S-210-10','NNP/NEL/TEL','','','Aeron/Ionos',839,'',112 from dual union all
select 34048,'1974-S285','1974-08-27 15:00:00','military missile','success',931,'A3T','DASO','','','OT',421,'SSBN 600',1000 from dual union all
select 34049,'1974-S286','1974-08-28 00:00:00','military missile','success',744,'','15A15 LKI-23','','','Test',530,'',1000 from dual union all
select 3405,'1955-A542','1955-05-24 00:00:00','atmospheric rocket','success',1527,'83','','','','Test',1895,'B-29',10 from dual union all
select 34050,'1974-S287','1974-08-30 00:00:00','military missile','success',1004,'112T','15A14 K-1 ZP','GCh 15F141','','OT',496,'',1000 from dual union all
select 34051,'1974-S288','1974-08-20 05:30:00','suborbital rocket','success',700,'L-4SC-3','TVC test','','','Test',834,'',100 from dual union all
select 34052,'1974-S289','1974-09-01 00:00:00','military missile','success',1514,'','','','','Test',989,'',1000 from dual union all
select 34053,'1974-S290','1974-09-02 06:00:00','test rocket','success',454,'ETV-1','','','','Test',1475,'',128 from dual union all
select 34054,'1974-S291','1974-09-03 23:25:00','suborbital rocket','success',920,'P99H','Plasmapause flux','','','Ionospheric',1447,'',136 from dual union all
select 34055,'1974-S292','1974-09-04 10:24:00','suborbital rocket','success',40,'CRL A05.391-1','HI STAR SOUTH 1','','','IR Astron',1846,'',193 from dual union all
select 34056,'1974-S293','1974-09-06 23:59:00','suborbital rocket','success',921,'P94H','Plasmapause flux','','','Ionospheric',1447,'',174 from dual union all
select 34057,'1974-S294','1974-09-07 04:35:00','suborbital rocket','success',41,'NASA 26.25DG','','','','XR Astron',1895,'',250 from dual union all
select 34058,'1974-S295','1974-09-08 00:00:00','military missile','success',162,'80F','ACE FTV-3','BMRS ACE FTV-3','','Reentry Test',1686,'',1400 from dual union all
select 34059,'1974-S296','1974-09-16 00:00:00','military missile','success',1004,'23L','15A14 LKI-25','GCh 15F141','','Test',492,'',1000 from dual union all
select 3406,'1955-A547','1955-07-29 00:00:00','military missile','unknown',845,'B6 1810-9','Test','','','Test',1895,'',10 from dual union all
select 34060,'1974-S297','1974-09-11 11:02:00','suborbital rocket','success',920,'P90T/C','Mass spectrometer','','','Atmospheric',1447,'',135 from dual union all
select 34061,'1974-S298','1974-09-11 22:55:00','suborbital rocket','success',40,'CRL A05.391-2','HI STAR SOUTH 2','','','IR Astron',1846,'',200 from dual union all
select 34062,'1974-S299','1974-09-12 00:00:00','suborbital rocket','success',291,'ESRO C99/1 C207','NO-II-1  FU-225','','','Aeron',340,'',140 from dual union all
select 34063,'1974-S30','1974-01-01 00:00:00','suborbital rocket','success',1281,'PCA','','','','Ionos',1470,'',100 from dual union all
select 34064,'1974-S300','1974-09-12 00:00:00','suborbital rocket','success',291,'ESRO C99/2 C201','NO-II-2  FU-225','','','Aeron',340,'',140 from dual union all
select 34065,'1974-S301','1974-09-13 00:00:00','suborbital rocket','success',291,'ESRO C99/3 C209','NO-II-3  FU-225','','','Aeron',340,'',140 from dual union all
select 34066,'1974-S302','1974-09-14 00:00:00','military missile','success',744,'','15A15 LKI-24','','','Test',541,'',1000 from dual union all
select 34067,'1974-S303','1974-09-16 15:40:00','suborbital rocket','success',668,'K-9M-46','Plasma gun M/D','','','Ionos',839,'',336 from dual union all
select 34068,'1974-S304','1974-09-17 15:33:00','suborbital rocket','success',40,'CRL A05.391-3','HI STAR SOUTH 3','','','IR Astron',1846,'',193 from dual union all
select 34069,'1974-S305','1974-09-18 17:43:00','suborbital rocket','success',37,'NASA 13.35US','','','','Solar',1895,'',252 from dual union all
select 3407,'1955-A548','1955-08-16 00:00:00','military missile','unknown',845,'B7 1810-11','Test','','','Test',1895,'',10 from dual union all
select 34070,'1974-S306','1974-09-19 02:00:00','suborbital rocket','success',668,'K-9M-47','','','','Ionos',839,'',328 from dual union all
select 34071,'1974-S307','1974-09-19 14:26:00','suborbital rocket','success',740,'MR-12','Diurnal campaign','','','Solar UV/Solar EUV',636,'',150 from dual union all
select 34072,'1974-S308','1974-09-19 15:47:00','suborbital rocket','success',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',150 from dual union all
select 34073,'1974-S309','1974-09-19 17:31:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',150 from dual union all
select 34074,'1974-S31','1974-09-17 00:00:00','military missile','success',1449,'','8K84 UBP-143','Svinets','','OT',540,'',1000 from dual union all
select 34075,'1974-S310','1974-09-19 19:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',150 from dual union all
select 34076,'1974-S311','1974-09-20 00:00:00','military missile','success',1251,'PFT-02A','SAFEGUARD M2-245A','','','Interceptor',868,'',100 from dual union all
select 34077,'1974-S312','1974-09-20 04:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Plasma',636,'',150 from dual union all
select 34078,'1974-S313','1974-09-20 06:00:00','suborbital rocket','success',740,'MR-12','Impedance probe','','','Aeron/Ionos/CR/Plas',636,'',150 from dual union all
select 34079,'1974-S314','1974-09-20 07:18:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',150 from dual union all
select 3408,'1955-A549','1955-08-19 00:00:00','military missile','unknown',845,'B8 1810-12','Test','','','Test',1895,'',10 from dual union all
select 34080,'1974-S315','1974-09-20 10:00:00','suborbital rocket','success',740,'MR-12','Impedance probe','','','Aeron/Ionos/CR/Plas',636,'',150 from dual union all
select 34081,'1974-S316','1974-09-20 11:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',150 from dual union all
select 34082,'1974-S317','1974-09-20 11:32:00','suborbital rocket','success',668,'K-9M-48','Precip. Electrons','','','Ionos/XUV',839,'',348 from dual union all
select 34083,'1974-S318','1974-09-20 13:30:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',150 from dual union all
select 34084,'1974-S319','1974-09-20 19:36:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',150 from dual union all
select 34085,'1974-S32','1974-10-18 00:00:00','military missile','success',1449,'','8K84 UBP-144','Svinets','','OT',540,'',1000 from dual union all
select 34086,'1974-S320','1974-09-20 22:20:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',150 from dual union all
select 34087,'1974-S321','1974-09-20 22:36:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',150 from dual union all
select 34088,'1974-S322','1974-09-21 00:00:00','military missile','success',922,'','15Zh45 LKI-1','','','Test',636,'',500 from dual union all
select 34089,'1974-S323','1974-09-21 03:10:00','suborbital rocket','unknown',740,'MR-12','','','','Solar UV/Solar EUV',636,'',150 from dual union all
select 3409,'1955-A55','1955-01-28 00:00:00','atmospheric rocket','success',442,'','Delta-wing model','','','Test',1821,'',10 from dual union all
select 34090,'1974-S324','1974-09-21 06:00:00','suborbital rocket','success',740,'MR-12','Ozone','','','Aeron/Ionos/CR/Plas',636,'',150 from dual union all
select 34091,'1974-S325','1974-09-24 18:20:00','suborbital rocket','success',851,'NASA 28.002GT','','','','Test',1821,'',465 from dual union all
select 34092,'1974-S326','1974-09-24 23:17:00','suborbital rocket','success',920,'P50H','Airglow','','','Atmospheric',1447,'',136 from dual union all
select 34093,'1974-S327','1974-09-27 00:00:00','military missile','success',744,'','15A15 LKI-25','','','Test',530,'',1000 from dual union all
select 34094,'1974-S328','1974-09-27 02:09:00','suborbital rocket','success',920,'P57H','Airglow','','','Atmospheric',1447,'',136 from dual union all
select 34095,'1974-S329','1974-09-28 15:34:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT46GB','GLORY TRIP 46GB','OT',1708,'',1300 from dual union all
select 34096,'1974-S33','1974-03-26 03:40:00','military missile','success',1450,'63011-61-263T','15A20 UBP-15 ZP','GCh 15F202','','OT',504,'',1000 from dual union all
select 34097,'1974-S330','1974-09-28 11:32:00','suborbital rocket','success',920,'P161H','Electron density','','','Ionospheric',1447,'',119 from dual union all
select 34098,'1974-S331','1974-10-17 14:10:00','military missile','unknown',917,'AO-140','P-292 74 Fall 01','','','OT',433,'',180 from dual union all
select 34099,'1974-S332','1974-10-17 15:08:00','military missile','unknown',917,'AO-141','P-293 74 Fall 02','','','OT',433,'',180 from dual union all
select 341,'1944-S310','1944-12-30 18:28:00','military missile','success',1470,'20816','Ma413','','','Test',661,'',90 from dual union all
select 3410,'1955-A550','1955-09-02 00:00:00','military missile','unknown',845,'B9 1810-13','Test','','','Test',1895,'',10 from dual union all
select 34100,'1974-S333','1974-10-31 15:02:00','military missile','unknown',917,'AO-142','P-294 74 Fall 03','','','OT',433,'',180 from dual union all
select 34101,'1974-S334','1974-10-31 16:25:00','military missile','unknown',917,'AO-143','P-295 74 Fall 04','','','OT',433,'',180 from dual union all
select 34102,'1974-S335','1974-10-03 05:10:00','suborbital rocket','success',41,'NASA 26.37GG','','','','XR Astron',1895,'',250 from dual union all
select 34103,'1974-S336','1974-10-05 06:10:00','military missile','success',799,'PVM-8','2 RV','R&D','PVM-8','Test',1699,'',1300 from dual union all
select 34104,'1974-S337','1974-10-04 15:40:00','suborbital rocket','success',1211,'SL1012','MSSL XR15 XRCS','','','XR Astronomy',1839,'',196 from dual union all
select 34105,'1974-S338','1974-10-04 16:16:00','suborbital rocket','success',228,'NASA 21.15GP','','','','Test',1873,'',227 from dual union all
select 34106,'1974-S339','1974-10-08 03:35:00','military missile','success',796,'','Mk 11 RV','FOT GT118M','GLORY TRIP 118GM','OT',1697,'',1300 from dual union all
select 34107,'1974-S34','1974-03-30 02:30:00','military missile','success',1450,'47611-51-143P','15A20 UBP-16 ZP','GCh 15F202','','OT',504,'',1000 from dual union all
select 34108,'1974-S340','1974-10-07 00:00:00','military missile','success',1004,'24L','15A14 LKI-26','GCh 15F142','','Test',488,'',1000 from dual union all
select 34109,'1974-S341','1974-10-07 06:33:30','suborbital rocket','success',1211,'SL1304','DLR H-GR-59','Astro 9 (LUX XRS16)','','XR Astronomy  Huelva-3',395,'',190 from dual union all
select 3411,'1955-A551','1955-09-12 00:00:00','military missile','unknown',845,'B10 1810-10','Test','','','Test',1895,'',10 from dual union all
select 34110,'1974-S342','1974-10-09 04:49:00','suborbital rocket','success',599,'INTA IT-P1-7401','F-74001','','','Test',395,'',254 from dual union all
select 34111,'1974-S343','1974-10-10 05:30:00','suborbital rocket','success',1201,'SL1471','Meadowlark','','','Test',1832,'',270 from dual union all
select 34112,'1974-S344','1974-10-10 16:37:00','suborbital rocket','success',132,'NASA 23.05UE','','','','Ionos',1895,'',90 from dual union all
select 34113,'1974-S345','1974-10-12 05:54:00','military missile','success',799,'PVM-6','Mk 12 RV x 3','R&D','PVM-6','Test',1707,'',1300 from dual union all
select 34114,'1974-S346','1974-10-12 08:30:00','military missile','success',799,'PVM-7','Mk 12 RV x 3','R&D','PVM-7','Test',1711,'',1300 from dual union all
select 34115,'1974-S347','1974-10-14 03:46:00','military missile','success',162,'31F','Mk 12 RV','BMRS RVTO-3A-1/FAIR-2','','Reentry Test',1688,'',1400 from dual union all
select 34116,'1974-S348','1974-10-15 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 601',1000 from dual union all
select 34117,'1974-S349','1974-10-15 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 601',1000 from dual;
insert into LAUNCH
select 34118,'1974-S35','1974-06-26 04:10:00','military missile','success',1450,'52411-52-143P','15A20 UBP-18 ZP','GCh 15F202','','OT',504,'',1000 from dual union all
select 34119,'1974-S350','1974-10-15 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 601',1000 from dual union all
select 3412,'1955-A552','1955-10-26 00:00:00','military missile','unknown',845,'B11 1810-17','Test','','','Test',1895,'',10 from dual union all
select 34120,'1974-S351','1974-10-15 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 601',1000 from dual union all
select 34121,'1974-S352','1974-10-15 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 601',1000 from dual union all
select 34122,'1974-S353','1974-10-15 14:53:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',150 from dual union all
select 34123,'1974-S354','1974-10-15 20:30:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',150 from dual union all
select 34124,'1974-S355','1974-10-16 02:14:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',150 from dual union all
select 34125,'1974-S356','1974-10-16 23:24:00','military missile','success',999,'N22502-08','8K67 UBP-48 Yantar''','GCh 8F674','','Test',519,'',1000 from dual union all
select 34126,'1974-S357','1974-10-17 10:20:00','suborbital rocket','success',846,'EX 407.41-1','PRECEDE  (HARSES)','','','Aeron',1895,'',120 from dual union all
select 34127,'1974-S358','1974-10-19 16:00:00','suborbital rocket','success',41,'NASA 26.16AS','','','','Solar',1895,'',201 from dual union all
select 34128,'1974-S359','1974-10-22 00:00:00','military missile','success',744,'','15A15 LKI-26','','','Test',541,'',1000 from dual union all
select 34129,'1974-S36','1974-05-29 13:00:00','military missile','success',1450,'49211-62-263T','15A20 UBP-17 ZP','GCh 15F202','','OT',504,'',1000 from dual union all
select 3413,'1955-A553','1955-11-02 00:00:00','military missile','unknown',845,'B12 1810-16','Test','','','Test',1895,'',10 from dual union all
select 34130,'1974-S360','1974-10-22 14:55:00','military missile','success',796,'','ERCS','ERCS GIANT MOON 6','GIANT MOON 6','Comms',1700,'',1300 from dual union all
select 34131,'1974-S361','1974-10-22 00:00:00','military missile','success',939,'','FOT-7','','','OT',421,'SSBN 659',500 from dual union all
select 34132,'1974-S362','1974-10-22 00:00:00','military missile','success',939,'','FOT-7','','','OT',421,'SSBN 659',500 from dual union all
select 34133,'1974-S363','1974-10-22 00:00:00','military missile','success',939,'','FOT-7','','','OT',421,'SSBN 659',500 from dual union all
select 34134,'1974-S364','1974-10-22 00:00:00','military missile','success',939,'','FOT-7','','','OT',421,'SSBN 659',500 from dual union all
select 34135,'1974-S365','1974-10-23 00:00:00','military missile','success',744,'','15A15 LKI-27','','','Test',530,'',1000 from dual union all
select 34136,'1974-S366','1974-10-23 00:00:00','military missile','success',1004,'31L','15A14 LKI-27','GCh 15F143U1','','Test',496,'',1000 from dual union all
select 34137,'1974-S367','1974-10-25 14:55:00','military missile','success',799,'','Mk 12 RV x 3','OT GT28GM','GLORY TRIP 28GM','OT',1705,'',1300 from dual union all
select 34138,'1974-S368','1974-11-25 14:00:00','military missile','unknown',917,'AO-144','P-296 74 Fall 05','','','OT',433,'',180 from dual union all
select 34139,'1974-S369','1974-11-25 15:01:00','military missile','unknown',917,'AO-145','P-297 74 Fall 06','','','OT',433,'',180 from dual union all
select 3414,'1955-A554','1955-11-16 00:00:00','military missile','unknown',845,'B13 1810-19','Test','','','Test',1895,'',10 from dual union all
select 34140,'1974-S37','1974-07-25 01:00:00','military missile','success',1450,'49211-62-264T','15A20 UBP-19 ZP','GCh 15F202','','OT',504,'',1000 from dual union all
select 34141,'1974-S370','1974-11-01 00:00:00','military missile','success',1251,'PFT-3','SAFEGUARD M2-445','','','Interceptor',868,'',100 from dual union all
select 34142,'1974-S371','1974-11-01 20:43:42','suborbital rocket','success',1210,'SL1221 (ESRO S107)','UK High Lat 5','','','Auroral/Ionospheric',29,'',225 from dual union all
select 34143,'1974-S372','1974-11-03 10:34:55','suborbital rocket','success',1271,'Sandia 428-034','LRL ACS 11','','','XR Astron',843,'',255 from dual union all
select 34144,'1974-S373','1974-11-03 11:57:53','suborbital rocket','success',37,'NASA 13.107UG','CAL XR 10','','','XR Astron',855,'',169 from dual union all
select 34145,'1974-S374','1974-11-10 12:09:00','suborbital rocket','success',1115,'Sandia 281-3','LEINAX III','','','XR Astron',855,'',180 from dual union all
select 34146,'1974-S375','1974-11-12 00:00:00','military missile','success',744,'','15A15 K-1','','','OT',541,'',1000 from dual union all
select 34147,'1974-S376','1974-11-14 09:46:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/CR/Plas',1010,'',150 from dual union all
select 34148,'1974-S377','1974-11-16 05:06:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/CR/Plas',1010,'',150 from dual union all
select 34149,'1974-S378','1974-11-16 09:21:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/CR/Plasma',1010,'',150 from dual union all
select 3415,'1955-A555','1955-11-23 00:00:00','military missile','unknown',845,'B14 1810-18','Test','','','Test',1895,'',10 from dual union all
select 34150,'1974-S379','1974-11-16 09:49:00','suborbital rocket','success',38,'NASA 13.92DG','NRL NB4.274','FUV Schmidt','','UV Astron',1895,'',181 from dual union all
select 34151,'1974-S38','1974-08-23 18:00:00','military missile','success',1450,'47611-51-144P','15A20 UBP-20 ZP','GCh 15F202','','OT',504,'',1000 from dual union all
select 34152,'1974-S380','1974-11-16 10:35:00','suborbital rocket','success',38,'NASA 13.34UG','MIT UBP 1','','','XR Astron',1895,'',166 from dual union all
select 34153,'1974-S381','1974-11-19 02:10:00','suborbital rocket','success',466,'E011B','FAUST FU-215','','','UV Astron',340,'',220 from dual union all
select 34154,'1974-S382','1974-11-20 09:46:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/CR/Plasma',1010,'',150 from dual union all
select 34155,'1974-S383','1974-11-22 00:00:00','military missile','success',939,'','FOT-8','','','OT',421,'SSBN 658',500 from dual union all
select 34156,'1974-S384','1974-11-22 00:00:00','military missile','success',939,'','FOT-8','','','OT',421,'SSBN 658',500 from dual union all
select 34157,'1974-S385','1974-11-22 00:00:00','military missile','success',939,'','FOT-8','','','OT',421,'SSBN 658',500 from dual union all
select 34158,'1974-S386','1974-11-22 00:00:00','military missile','success',939,'','FOT-8','','','OT',421,'SSBN 658',500 from dual union all
select 34159,'1974-S387','1974-11-25 07:22:00','suborbital rocket','success',867,'NASA 18.154UA','','','','Aeronomy',1895,'',133 from dual union all
select 3416,'1955-A556','1955-12-01 00:00:00','military missile','unknown',845,'B15 1810-4','Test','','','Test',1895,'',10 from dual union all
select 34160,'1974-S388','1974-11-25 08:10:00','suborbital rocket','success',228,'NASA 21.31UG','','UCB EUV Photom/XR','','EUV Astron/XR Astron',1873,'',184 from dual union all
select 34161,'1974-S389','1974-11-25 09:15:00','suborbital rocket','success',41,'NASA 26.42UG','','','','UV Astron?',1895,'',207 from dual union all
select 34162,'1974-S39','1974-09-20 03:40:00','military missile','success',1450,'51121-63-263T','15A20 UBP-21 ZP','GCh 15F202','','OT',536,'',1000 from dual union all
select 34163,'1974-S390','1974-11-26 00:00:00','military missile','success',799,'PVM-9','Mk 12 RV x 3','R&D','PVM-9','Test',1703,'',1300 from dual union all
select 34164,'1974-S391','1974-11-27 18:00:00','suborbital rocket','success',41,'NASA 26.41US','','','','Solar',1895,'',297 from dual union all
select 34165,'1974-S392','1974-11-28 16:40:00','suborbital rocket','success',1211,'SL911','Cul FUV-3 Echelle','','','UV Astronomy',1839,'',203 from dual union all
select 34166,'1974-S393','1974-11-29 10:17:00','suborbital rocket','success',920,'P182H','','','','Ionos',1447,'',118 from dual union all
select 34167,'1974-S394','1974-11-29 11:53:00','suborbital rocket','success',920,'P175H','Atomic oxygen','','','Atmospheric',1447,'',134 from dual union all
select 34168,'1974-S395','1974-11-29 12:09:00','suborbital rocket','success',920,'P179H','Atomic oxygen','','','Atmospheric',1447,'',123 from dual union all
select 34169,'1974-S396','1974-12-06 00:00:00','military missile','unknown',917,'AO-146','P-298 74 Fall 07','','','OT',433,'',180 from dual union all
select 3417,'1955-A557','1955-12-07 00:00:00','military missile','unknown',845,'B16 1810-20','Test','','','Test',1895,'',10 from dual union all
select 34170,'1974-S397','1974-12-06 00:00:00','military missile','unknown',917,'AO-147','P-299 74 Fall 08','','','OT WS-213?',433,'',180 from dual union all
select 34171,'1974-S398','1974-12-01 00:00:00','military missile','success',997,'','','','','Test',625,'',1000 from dual union all
select 34172,'1974-S399','1974-12-02 12:36:00','suborbital rocket','success',920,'P91H','','','','Atmospheric',1447,'',129 from dual union all
select 34173,'1974-S40','1974-12-17 23:00:00','military missile','success',1450,'47621-51-144P','15A20 UBP-22 ZP','GCh 15B20','','Test',506,'',1000 from dual union all
select 34174,'1974-S400','1974-12-04 07:20:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT47GB','GLORY TRIP 47GB','OT',1711,'',1300 from dual union all
select 34175,'1974-S401','1974-12-06 20:20:00','suborbital rocket','success',867,'NASA 18.167UA','','','','Aeronomy',1821,'',306 from dual union all
select 34176,'1974-S402','1974-12-06 23:32:06','suborbital rocket','success',227,'AMF-VB-41','Cusp','','','Auroral',334,'',245 from dual union all
select 34177,'1974-S403','1974-12-07 00:00:00','suborbital rocket','success',547,'','HAVE JEEP II A','','','Reentry Test?',873,'',500 from dual union all
select 34178,'1974-S404','1974-12-07 20:37:00','suborbital rocket','success',920,'P183H','','','','Atmospheric',1447,'',101 from dual union all
select 34179,'1974-S405','1974-12-07 20:58:00','suborbital rocket','success',920,'P180H','','','','Aeron/Ionos',1447,'',132 from dual union all
select 3418,'1955-A558','1955-12-14 00:00:00','military missile','unknown',845,'B17 1810-15','Test','','','Test',1895,'',10 from dual union all
select 34180,'1974-S406','1974-12-07 22:26:00','suborbital rocket','success',920,'P176H','','','','Atmospheric',1447,'',137 from dual union all
select 34181,'1974-S407','1974-12-07 22:53:00','suborbital rocket','success',920,'P128H','Airglow','','','Atmospheric',1447,'',138 from dual union all
select 34182,'1974-S408','1974-12-11 00:00:00','military missile','success',1004,'160T','15A14 K-2','GCh 15F143U1','','Test',490,'',1000 from dual union all
select 34183,'1974-S409','1974-12-11 22:48:00','suborbital rocket','success',220,'AAF-4B-32','','','','Auroral/Ionos',334,'',600 from dual union all
select 34184,'1974-S41','1974-03-29 00:00:00','military missile','success',1452,'-      2-015','15A30 LKI-15','GCh 15B103','','Test',544,'',1000 from dual union all
select 34185,'1974-S410','1974-12-13 00:00:00','military missile','success',1322,'64511-646-28L','15Zh42 LKI-28','','','Test',477,'',1000 from dual union all
select 34186,'1974-S411','1974-12-13 00:00:00','military missile','success',1322,'64511-645-29L','15Zh42 LKI-29','','','Test',477,'',1000 from dual union all
select 34187,'1974-S412','1974-12-13 19:27:11','suborbital rocket','success',867,'NASA 18.140IE','Ferdinand 37/Polar 4','','','Auroral',29,'',228 from dual union all
select 34188,'1974-S413','1974-12-13 22:25:00','suborbital rocket','success',1468,'CRL A09.407-1','10-inch Sphere','','','Aeron',1895,'',90 from dual union all
select 34189,'1974-S414','1974-12-16 16:00:02','military missile','success',939,'','DASO','','','OT',421,'SSBN 616',500 from dual union all
select 3419,'1955-A559','1955-12-14 00:00:00','military missile','unknown',845,'B18 1810-22','Test','','','Test',1895,'',10 from dual union all
select 34190,'1974-S415','1974-12-17 00:00:00','military missile','success',744,'','15A15 LKI-28','','','Test',541,'',1000 from dual union all
select 34191,'1974-S416','1974-12-18 02:35:00','military missile','success',796,'','Mk 11 RV','FOT GT121M','GLORY TRIP 121GM','OT',1699,'',1300 from dual union all
select 34192,'1974-S417','1974-12-17 00:00:00','military missile','success',1004,'29L','15A14 LKI-29','GCh 15F143U1','','Test',492,'',1000 from dual union all
select 34193,'1974-S418','1974-12-17 10:55:00','suborbital rocket','success',220,'ESRO D-102A','DLR G-BB4-64 PolCusp','','','Ionospheric',1435,'',280 from dual union all
select 34194,'1974-S419','1974-12-17 11:12:00','suborbital rocket','success',920,'P86G','Polar Cusp Li','','','Atmospheric',1438,'',165 from dual union all
select 34195,'1974-S42','1974-05-27 00:00:00','military missile','success',1452,'-      2-016','15A30 LKI-16','GCh 15B103','','Test',532,'',1000 from dual union all
select 34196,'1974-S420','1974-12-17 12:37:00','suborbital rocket','success',920,'P87G','Polar Cusp Li','','','Atmospheric',1438,'',165 from dual union all
select 34197,'1974-S421','1974-12-18 00:58:00','military missile','success',1164,'SH74-1','Ballistic Test','','','Test',1047,'',500 from dual union all
select 34198,'1974-S422','1974-12-18 11:15:00','suborbital rocket','success',920,'P166G','Polar Cusp Li','','','Atmospheric',1438,'',165 from dual union all
select 34199,'1974-S423','1974-12-18 12:45:00','suborbital rocket','success',920,'P167G','Polar Cusp Li','','','Atmospheric',1438,'',165 from dual union all
select 342,'1944-S311','1944-12-31 12:44:00','military missile','success',1470,'20847','Ma425','','','Test',661,'',90 from dual union all
select 3420,'1955-A56','1955-01-28 00:00:00','atmospheric rocket','unknown',576,'974-AO','','','','Test',1895,'',10 from dual union all
select 34200,'1974-S424','1974-12-21 09:26:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/CR/Plasma',1010,'',150 from dual union all
select 34201,'1974-S425','1974-12-23 00:00:00','military missile','success',1004,'159T','15A14 K-3','GCh 15F142','','Test',488,'',1000 from dual union all
select 34202,'1974-S426','1974-12-24 18:13:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/CR/Plasma',1010,'',150 from dual union all
select 34203,'1974-S427','1974-12-25 00:00:00','military missile','success',1004,'30L','15A14 LKI-30','GCh 15F143U1','','Test',496,'',1000 from dual union all
select 34204,'1974-S428','1974-12-28 06:14:30','suborbital rocket','success',41,'NASA 26.30UG','CAL XR 11','','','XR Astron',1895,'',197 from dual union all
select 34205,'1974-S429','1974-06-30 03:31:00','suborbital rocket','success',30,'NRL NC3.256','NRL P3 Probe','','','Plasma',1873,'',229 from dual union all
select 34206,'1974-S43','1974-05-31 00:00:00','military missile','success',1452,'-      2-017','15A30 LKI-17','GCh 15B103','','Test',508,'',1000 from dual union all
select 34207,'1974-S430','1974-06-26 00:00:00','military missile','success',356,'702009','','','','Test',650,'',500 from dual;
insert into LAUNCH
select 34208,'1974-S431','1974-11-01 00:00:00','military missile','success',356,'','Reentry wake test','','','Test',809,'',500 from dual union all
select 34209,'1974-S432','1974-12-01 00:00:00','military missile','success',356,'','Reentry wake test','','','Test',809,'',500 from dual union all
select 3421,'1955-A560','1955-01-01 00:00:00','military missile','unknown',845,'B19','Test','','','Test',1895,'',10 from dual union all
select 34210,'1974-S433','1974-01-01 00:00:00','military missile','success',353,'','','','','Test',811,'',200 from dual union all
select 34211,'1974-S434','1974-01-01 00:00:00','military missile','success',353,'','','','','Test',811,'',200 from dual union all
select 34212,'1974-S435','1974-07-11 05:01:00','test rocket','success',645,'','BOR-3 No. 302','','','Reentry Test',1752,'',100 from dual union all
select 34213,'1974-S436','1974-10-02 00:00:00','military missile','success',1514,'','','','','Test',190,'PLBR',1000 from dual union all
select 34214,'1974-S437','1974-10-02 00:00:00','military missile','success',1514,'','','','','Test',190,'PLBR',1000 from dual union all
select 34215,'1974-S438','1974-10-01 23:50:00','suborbital rocket','success',334,'WRE C4009','O2 density','','','Aeron',1835,'',137 from dual union all
select 34216,'1974-S439','1974-10-02 07:55:00','suborbital rocket','success',334,'WRE C4007','O2 density','','','Aeron',1835,'',132 from dual union all
select 34217,'1974-S44','1974-08-09 00:00:00','military missile','success',1452,'-      2-020','15A30 LKI-19','GCh 15B103','','Test',508,'',1000 from dual union all
select 34218,'1974-S440','1974-06-28 00:00:00','suborbital rocket','success',334,'WRE C4006','O2 density','','','Aeron',1835,'',130 from dual union all
select 34219,'1974-S441','1974-12-15 04:02:00','suborbital rocket','success',1281,'','AFCRL Arcas 73A-1','','','Aeronomy',1470,'',100 from dual union all
select 3422,'1955-A561','1955-01-01 00:00:00','military missile','unknown',845,'B20','Test','','','Test',1895,'',10 from dual union all
select 34220,'1974-S442','1974-12-01 00:00:00','suborbital rocket','success',547,'','HAVE JEEP II B','','','Reentry Test?',873,'',500 from dual union all
select 34221,'1974-S443','1974-12-01 00:00:00','suborbital rocket','success',547,'','HAVE JEEP II C','','','Reentry Test?',873,'',500 from dual union all
select 34222,'1974-S444','1974-01-21 00:00:00','military missile','success',1018,'G 15001-08','8K75 UBP-P6','','','OT',477,'',1160 from dual union all
select 34223,'1974-S445','1974-05-30 00:00:00','military missile','success',982,'G 22504-04T','8K64U UBP-P105','R-16 PL74-01','','OT',477,'',1210 from dual union all
select 34224,'1974-S446','1974-07-15 00:00:00','military missile','success',982,'G 22502-09T','8K64U UBP-P106','R-16 PL74-02','','OT',477,'',1210 from dual union all
select 34225,'1974-S447','1974-01-16 00:00:00','military missile','success',1037,'29557-211-41','8K98 UBP-22','','','Test',477,'',1000 from dual union all
select 34226,'1974-S448','1974-09-04 00:00:00','military missile','success',1037,'V 29501-05','8K98 UBP-23','','','Test',477,'',1000 from dual union all
select 34227,'1974-S449','1974-09-27 00:00:00','military missile','success',1037,'64647-221-33','8K98 UBP-24','','','Test',477,'',1000 from dual union all
select 34228,'1974-S45','1974-09-30 00:00:00','military missile','success',1452,'-      2-024','15A30 LKI-20','GCh 15B103','','Test',532,'',1000 from dual union all
select 34229,'1974-S450','1974-12-11 00:00:00','military missile','success',1044,'45751-612-01','8K98P UBP-1','','','Test',477,'',1000 from dual union all
select 3423,'1955-A562','1955-12-05 00:00:00','suborbital rocket','success',925,'','POGO','','','Target',1895,'',20 from dual union all
select 34230,'1974-S452','1974-12-29 00:00:00','military missile','success',1322,'64549-680-27L','15Zh42 LKI-30','','','Test',477,'',1000 from dual union all
select 34231,'1974-S453','1974-06-28 00:00:00','military missile','failure',1322,'19L','15Zh42 LKI-19','','','Test',477,'',0 from dual union all
select 34232,'1974-S454','1974-05-07 00:00:00','military missile','success',1322,'64549-682-15L','15Zh42 LKI-15','','','Test',477,'',1000 from dual union all
select 34233,'1974-S455','1974-06-07 00:00:00','military missile','success',1322,'64549-678-16L','15Zh42 LKI-16','','','Test',477,'',1000 from dual union all
select 34234,'1974-S456','1974-06-19 00:00:00','military missile','success',1322,'64511-642-17L','15Zh42 LKI-17','','','Test',477,'',1000 from dual union all
select 34235,'1974-S457','1974-06-25 00:00:00','military missile','success',1322,'64549-684-18L','15Zh42 LKI-18','','','Test',477,'',1000 from dual union all
select 34236,'1974-S458','1974-07-24 00:00:00','military missile','success',1322,'64511-644-20L','15Zh42 LKI-20','','','Test',477,'',1000 from dual union all
select 34237,'1974-S459','1974-08-27 00:00:00','military missile','success',1322,'64553-51-482-21L','15Zh42 LKI-21','','','Test',477,'',1000 from dual union all
select 34238,'1974-S46','1974-11-02 00:00:00','military missile','success',1452,'-      2-025','15A30 LKI-21','GCh 15B103','','Test',508,'',1000 from dual union all
select 34239,'1974-S460','1974-08-30 00:00:00','military missile','success',1322,'64511-648-22L','15Zh42 LKI-22','','','Test',477,'',1000 from dual union all
select 3424,'1955-A563','1955-02-14 00:00:00','atmospheric rocket','unknown',576,'929-RS','Demi John','','','Test',1895,'',10 from dual union all
select 34240,'1974-S461','1974-09-30 00:00:00','military missile','failure',1322,'23L','15Zh42 LKI-23','','','Test',477,'',0 from dual union all
select 34241,'1974-S462','1974-11-04 00:00:00','military missile','success',1322,'64549-681-26L','15Zh42 LKI-24','','','Test',477,'',1000 from dual union all
select 34242,'1974-S463','1974-11-15 00:00:00','military missile','success',1322,'64511-651-25L','15Zh42 LKI-25','','','Test',477,'',1000 from dual union all
select 34243,'1974-S464','1974-11-29 00:00:00','military missile','success',1322,'64511-652-30L','15Zh42 LKI-26','','','Test',477,'',1000 from dual union all
select 34244,'1974-S465','1974-05-26 00:00:00','military missile','success',999,'','8K67 62 RD UBP-2','','','OT',1646,'',1000 from dual union all
select 34245,'1974-S466','1974-05-26 00:00:00','military missile','success',999,'','8K67 62 RD UBP-3','','','OT',1646,'',1000 from dual union all
select 34246,'1974-S467','1974-05-27 00:00:00','military missile','success',999,'','8K67 62 RD UBP-4','','','OT',1646,'',1000 from dual union all
select 34247,'1974-S468','1974-05-27 00:00:00','military missile','success',999,'','8K67 62 RD UBP-5','','','OT',1646,'',1000 from dual union all
select 34248,'1974-S469','1974-05-27 00:00:00','military missile','success',999,'','8K67 62 RD UBP-6','','','OT',1646,'',1000 from dual union all
select 34249,'1974-S47','1974-12-08 00:00:00','military missile','success',1452,'51121-72-026','15A30 LKI-22','GCh 15B103','','Test',508,'',1000 from dual union all
select 3425,'1955-A564','1955-02-14 00:00:00','atmospheric rocket','unknown',576,'925-RS','Demi John','','','Test',1895,'',10 from dual union all
select 34250,'1974-S470','1974-05-27 00:00:00','military missile','success',999,'','8K67 62 RD UBP-7','','','OT',1646,'',1000 from dual union all
select 34251,'1974-S471','1974-01-01 00:00:00','military missile','success',1449,'T24','8K84 47 RD UBP-24','Duga-1','','OT',1911,'',1000 from dual union all
select 34252,'1974-S472','1974-01-01 00:00:00','military missile','success',1449,'T25','8K84 47 RD UBP-25','Duga-1','','OT',1911,'',1000 from dual union all
select 34253,'1974-S473','1974-01-01 00:00:00','military missile','success',1449,'T26','8K84 47 RD UBP-26','Duga-1','','OT',1911,'',1000 from dual union all
select 34254,'1974-S474','1974-01-01 00:00:00','military missile','success',1449,'T27','8K84 47 RD UBP-27','Duga-1','','OT',1911,'',1000 from dual union all
select 34255,'1974-S475','1974-01-01 00:00:00','military missile','success',1449,'T28','8K84 47 RD UBP-28','Duga-1','','OT',1911,'',1000 from dual union all
select 34256,'1974-S476','1974-01-01 00:00:00','military missile','success',1449,'T29','8K84 47 RD UBP-29','Duga-1','','OT',1911,'',1000 from dual union all
select 34257,'1974-S477','1974-01-01 00:00:00','military missile','success',1449,'T30','8K84 47 RD UBP-30','Duga-1','','OT',1911,'',1000 from dual union all
select 34258,'1974-S478','1974-01-01 00:00:00','military missile','success',1449,'T31','8K84 47 RD UBP-31','Duga-1','','OT',1911,'',1000 from dual union all
select 34259,'1974-S479','1974-01-01 00:00:00','military missile','success',1449,'T32','8K84 47 RD UBP-32','Duga-1','','OT',1911,'',1000 from dual union all
select 3426,'1955-A565','1955-08-26 00:00:00','military missile','success',771,'5','','','','Test',646,'',12 from dual union all
select 34260,'1974-S48','1974-12-18 01:50:00','military missile','success',1452,'51121-61-259T','15A30 LKI-23','GCh 15B103','','Test',532,'',1000 from dual union all
select 34261,'1974-S480','1974-01-01 00:00:00','military missile','success',1449,'T33','8K84 47 RD UBP-33','Duga-1','','OT',1911,'',1000 from dual union all
select 34262,'1974-S481','1974-01-01 00:00:00','military missile','success',1449,'T34','8K84 47 RD UBP-34','Duga-1','','OT',1911,'',1000 from dual union all
select 34263,'1974-S482','1974-01-01 00:00:00','military missile','success',1449,'T35','8K84 47 RD UBP-35','Duga-1','','OT',1911,'',1000 from dual union all
select 34264,'1974-S483','1974-01-01 00:00:00','military missile','success',1449,'T36','8K84 47 RD UBP-36','Duga-1','','OT',1911,'',1000 from dual union all
select 34265,'1974-S484','1974-01-01 00:00:00','military missile','success',1449,'T37','8K84 47 RD UBP-37','Duga-1','','OT',1911,'',1000 from dual union all
select 34266,'1974-S485','1974-01-01 00:00:00','military missile','success',1449,'T38','8K84 47 RD UBP-38','Duga-1','','OT',1911,'',1000 from dual union all
select 34267,'1974-S486','1974-01-01 00:00:00','military missile','success',1449,'T39','8K84 47 RD UBP-39','Duga-1','','OT',1911,'',1000 from dual union all
select 34268,'1974-S487','1974-01-01 00:00:00','military missile','success',1449,'T40','8K84 47 RD UBP-40','Duga-1','','OT',1911,'',1000 from dual union all
select 34269,'1974-S488','1974-01-01 00:00:00','military missile','success',1449,'T41','8K84 47 RD UBP-41','Duga-1','','OT',1911,'',1000 from dual union all
select 3427,'1955-A566','1955-04-21 00:00:00','atmospheric rocket','unknown',576,'921-ES','Demi John','','','Test',1895,'',10 from dual union all
select 34270,'1974-S489','1974-01-01 00:00:00','military missile','success',1449,'T42','8K84 47 RD UBP-42','Duga-1','','OT',1911,'',1000 from dual union all
select 34271,'1974-S49','1974-12-22 00:00:00','military missile','success',1452,'-      2-021','15A30 LKI-24','GCh 15B103','','Test',544,'',1000 from dual union all
select 34272,'1974-S490','1974-01-01 00:00:00','military missile','success',1449,'T43','8K84 47 RD UBP-43','Duga-1','','OT',1911,'',1000 from dual union all
select 34273,'1974-S491','1974-01-01 00:00:00','military missile','success',1449,'T44','8K84 47 RD UBP-44','Duga-1','','OT',1911,'',1000 from dual union all
select 34274,'1974-S492','1974-01-01 00:00:00','military missile','success',1449,'T45','8K84 47 RD UBP-45','Duga-1','','OT',1911,'',1000 from dual union all
select 34275,'1974-S493','1974-07-01 00:00:00','military missile','success',1449,'T46','8K84 47 RD UBP-46','Duga-1','','OT',1911,'',1000 from dual union all
select 34276,'1974-S494','1974-07-01 00:00:00','military missile','success',1449,'T47','8K84 47 RD UBP-46','Duga-1','','OT',1911,'',1000 from dual union all
select 34277,'1974-S495','1974-11-29 00:00:00','military missile','success',1322,'64521-61-863-24L','15Zh42 LKI-27','','','Test',477,'',1000 from dual union all
select 34278,'1974-S496','1974-03-25 23:30:00','military missile','success',135,'','HEART V4-6764','','','Reentry Test',1821,'',200 from dual union all
select 34279,'1974-S497','1974-12-13 21:10:00','military missile','success',135,'','HEART V4-7486','','','Reentry Test',1821,'',200 from dual union all
select 3428,'1955-A567','1955-09-22 00:00:00','military missile','success',771,'214','','','','Test',646,'',12 from dual union all
select 34280,'1974-S498','1974-08-28 18:00:00','military missile','success',135,'','HEART V4-6767','','','Reentry Test',1821,'',200 from dual union all
select 34281,'1974-S499','1974-01-27 00:00:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 34282,'1974-S50','1974-12-25 00:00:00','military missile','success',1452,'-      2-022','15A30 LKI-25','GCh 15B103','','Test',508,'',1000 from dual union all
select 34283,'1974-S500','1974-02-20 00:00:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 34284,'1974-S501','1974-02-27 00:00:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 34285,'1974-S502','1974-06-26 04:10:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 34286,'1974-S503','1974-07-25 00:00:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 34287,'1974-S504','1974-09-03 19:40:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 34288,'1974-S505','1974-09-05 00:00:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 34289,'1974-S506','1974-09-16 00:00:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 3429,'1955-A568','1955-10-06 00:00:00','military missile','success',771,'215','','','','Test',646,'',12 from dual union all
select 34290,'1974-S507','1974-09-17 00:00:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 34291,'1974-S508','1974-03-19 02:58:00','suborbital rocket','success',331,'WRE C3004','Li release','','','Aeron',1835,'',113 from dual union all
select 34292,'1974-S509','1974-07-18 00:00:00','military missile','success',980,'','8K65U 35 RD UBP-10','Salvo','','Test',943,'',675 from dual union all
select 34293,'1974-S51','1974-01-05 01:45:00','suborbital rocket','success',41,'NASA 26.23UG-UP','JHU UVS Kohoutek','','','Opt/UV Astron',1895,'',233 from dual union all
select 34294,'1974-S510','1974-07-18 00:00:00','military missile','success',980,'','8K65U 35 RD UBP-11','Salvo','','Test',943,'',675 from dual union all
select 34295,'1974-S511','1974-07-18 00:00:00','military missile','success',980,'','8K65U 35 RD UBP-12','Salvo','','Test',943,'',675 from dual union all
select 34296,'1974-S512','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-19','Duga-1','','OT',1455,'',1000 from dual union all
select 34297,'1974-S513','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-20','Duga-1','','OT',1455,'',1000 from dual union all
select 34298,'1974-S514','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-21','Duga-1','','OT',1455,'',1000 from dual;
insert into LAUNCH
select 34299,'1974-S515','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-22','Duga-1','','OT',1455,'',1000 from dual union all
select 343,'1944-S312','1944-12-31 15:23:00','military missile','success',1470,'20825','Ma408','','','Test',661,'',90 from dual union all
select 3430,'1955-A569','1955-10-12 00:00:00','military missile','success',771,'3090','','','','Test',646,'',12 from dual union all
select 34300,'1974-S516','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-23','Duga-1','','OT',1455,'',1000 from dual union all
select 34301,'1974-S517','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-24','Duga-1','','OT',1455,'',1000 from dual union all
select 34302,'1974-S518','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-25','Duga-1','','OT',1455,'',1000 from dual union all
select 34303,'1974-S519','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-26','Duga-1','','OT',1455,'',1000 from dual union all
select 34304,'1974-S52','1974-01-08 01:40:00','suborbital rocket','success',41,'NASA 26.22DG-DP','NRL NB12.271','','','Aeron/EUV Astron',1895,'',193 from dual union all
select 34305,'1974-S520','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-27','Duga-1','','OT',1455,'',1000 from dual union all
select 34306,'1974-S521','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-28','Duga-1','','OT',1455,'',1000 from dual union all
select 34307,'1974-S522','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-29','Duga-1','','OT',1455,'',1000 from dual union all
select 34308,'1974-S523','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-30','Duga-1','','OT',1455,'',1000 from dual union all
select 34309,'1974-S524','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-31','Duga-1','','OT',1455,'',1000 from dual union all
select 3431,'1955-A57','1955-01-28 00:00:00','atmospheric rocket','unknown',576,'975-AO','','','','Test',1895,'',10 from dual union all
select 34310,'1974-S525','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-32','Duga-1','','OT',1455,'',1000 from dual union all
select 34311,'1974-S526','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-33','Duga-1','','OT',1455,'',1000 from dual union all
select 34312,'1974-S527','1974-01-01 00:00:00','military missile','success',1449,'','8K84 27 RD UBP-34','Duga-1','','OT',1455,'',1000 from dual union all
select 34313,'1974-S528','1974-07-29 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 34314,'1974-S529','1974-07-31 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 34315,'1974-S53','1974-01-10 11:30:00','suborbital rocket','success',1189,'INTA MSCLO-7401','','','','Ionos',396,'',93 from dual union all
select 34316,'1974-S530','1974-08-21 00:00:00','military missile','success',976,'','8K63 43 RD UBP','','','Test',636,'',402 from dual union all
select 34317,'1974-S531','1974-10-02 00:00:00','military missile','unknown',980,'','8K65U UBP 29 RD','','','Test',636,'',675 from dual union all
select 34318,'1974-S532','1974-10-10 00:00:00','military missile','unknown',980,'','8K65U UBP 29 RD','','','Test',636,'',675 from dual union all
select 34319,'1974-S533','1974-11-12 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 3432,'1955-A570','1955-11-07 00:00:00','military missile','success',771,'2','','','','Test',646,'',12 from dual union all
select 34320,'1974-S534','1974-11-29 00:00:00','military missile','unknown',979,'','8K65 UBP 32 RD','','','Test',636,'',675 from dual union all
select 34321,'1974-S535','1974-12-09 00:00:00','military missile','unknown',979,'','8K65 UBP 32 RD','','','Test',636,'',675 from dual union all
select 34322,'1974-S536','1974-12-18 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 34323,'1974-S537','1974-06-22 00:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 34324,'1974-S54','1974-01-11 11:40:00','suborbital rocket','unknown',1189,'INTA MSCLO-7402','','','','Ionos',396,'',100 from dual union all
select 34325,'1974-S55','1974-01-11 22:00:00','military missile','success',999,'N22506-03','8K67 UBP-45 OS-67-1A','GCh 8F676','','Test',519,'',1000 from dual union all
select 34326,'1974-S56','1974-01-12 19:12:33','suborbital rocket','success',1209,'HRC 1','DLR H-GR-58','INTA SKY-N60','','XR Astron',395,'',246 from dual union all
select 34327,'1974-S57','1974-01-13 02:10:00','suborbital rocket','failure',41,'NASA 26.24UG-UP','','','','Aeron',1895,'',82 from dual union all
select 34328,'1974-S58','1974-01-14 11:35:00','suborbital rocket','success',1189,'INTA MSCLO-7403','','','','Ionos',396,'',86 from dual union all
select 34329,'1974-S59','1974-01-15 20:00:00','suborbital rocket','success',228,'NASA 21.14DS-NP','NRL CALROC 4','','','Solar',1873,'',234 from dual union all
select 3433,'1955-A571','1955-12-01 00:00:00','military missile','success',771,'54-3092','','','','Test',646,'',12 from dual union all
select 34330,'1974-S60','1974-01-16 00:00:00','suborbital rocket','success',41,'NASA 26.14CS','','','','Solar',1895,'',250 from dual union all
select 34331,'1974-S61','1974-01-16 02:00:00','suborbital rocket','success',668,'K-9M-45','','','','Iono-solar',839,'',360 from dual union all
select 34332,'1974-S62','1974-01-16 11:38:00','suborbital rocket','success',1189,'INTA MSCLO-7404','','','','Ionos',396,'',86 from dual union all
select 34333,'1974-S63','1974-01-16 17:40:00','suborbital rocket','success',837,'NASA 14.511UA','','','','Aeronomy',1821,'',205 from dual union all
select 34334,'1974-S64','1974-01-17 02:37:00','suborbital rocket','success',867,'NASA 18.160GE','','','','Plasma',1047,'',298 from dual union all
select 34335,'1974-S65','1974-11-06 00:23:00','suborbital rocket','success',466,'E011A','FAUST FU-214','','','UV Astron',340,'',220 from dual union all
select 34336,'1974-S66','1974-01-19 11:34:00','suborbital rocket','success',1191,'INTA M4SCL-7401','','','','Ionos',396,'',110 from dual union all
select 34337,'1974-S67','1974-01-20 00:00:00','military missile','success',1004,'17L','15A14 LKI-16','GCh 15F141?','','Test',487,'',1000 from dual union all
select 34338,'1974-S68','1974-01-21 02:39:00','suborbital rocket','success',867,'NASA 18.161GE','','','','Plasma',1047,'',298 from dual union all
select 34339,'1974-S69','1974-01-21 11:30:00','suborbital rocket','success',1191,'INTA MS4CLO-7401','','','','Ionos',396,'',114 from dual union all
select 3434,'1955-A572','1955-12-07 00:00:00','military missile','success',771,'53-189','','','','Test',646,'',12 from dual union all
select 34340,'1974-S70','1974-01-23 02:30:00','military missile','success',795,'','SSTTP','SSTTP M2-28','','Target',1701,'',1300 from dual union all
select 34341,'1974-S71','1974-01-22 02:41:00','suborbital rocket','success',867,'NASA 18.162GE','','','','Plasma',1047,'',298 from dual union all
select 34342,'1974-S72','1974-01-22 11:00:00','sounding rocket','success',698,'L-3H-8','','','','XR Astron',834,'',1571 from dual union all
select 34343,'1974-S73','1974-01-22 16:45:00','suborbital rocket','success',41,'NASA 26.07CS','','','','Solar EUV/X',1895,'',238 from dual union all
select 34344,'1974-S74','1974-01-23 06:06:00','suborbital rocket','success',227,'AMD-VB-34','','','','Auroral/Aeron/Ionos',428,'',208 from dual union all
select 34345,'1974-S75','1974-01-23 11:30:00','suborbital rocket','success',1191,'INTA MS4CLO-7402','','','','Ionos',396,'',106 from dual union all
select 34346,'1974-S76','1974-01-26 00:00:00','military missile','success',1452,'-      2-013','15A30 LKI-11','GCh 15B103','','Test',544,'',1000 from dual union all
select 34347,'1974-S77','1974-01-25 11:30:00','suborbital rocket','success',1191,'INTA MS4CLO-7403','','','','Ionos',396,'',109 from dual union all
select 34348,'1974-S78','1974-01-26 14:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT24GB-1/SL Targ','GLORY TRIP 24GB-1','OT',1711,'',1300 from dual union all
select 34349,'1974-S79','1974-01-27 00:00:00','military missile','success',1452,'-      2-012','15A30 LKI-12','GCh 15B103','','Test',544,'',1000 from dual union all
select 3435,'1955-A573','1955-04-08 00:00:00','atmospheric rocket','success',762,'5','HTV','','','Heat transfer',1895,'',10 from dual union all
select 34350,'1974-S80','1974-01-27 19:08:46','suborbital rocket','success',867,'NASA 18.139IE','Ferdinand 35/Polar 3','','','Auroral',29,'',238 from dual union all
select 34351,'1974-S81','1974-01-30 00:00:00','military missile','success',1105,'S-1E-2603','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 34352,'1974-S82','1974-02-01 00:00:00','military missile','success',1514,'','','','','Test',989,'',1000 from dual union all
select 34353,'1974-S83','1974-02-01 00:00:00','military missile','success',1514,'','','','','Test',190,'PLBR',1000 from dual union all
select 34354,'1974-S84','1974-03-01 00:00:00','military missile','success',1322,'64549-679-14L','15Zh42 LKI-14','','','Test',477,'',1000 from dual union all
select 34355,'1974-S85','1974-02-01 06:30:00','suborbital rocket','success',613,'JCR-10','','','','Test',1476,'',120 from dual union all
select 34356,'1974-S86','1974-02-04 00:00:00','military missile','success',744,'','15A15 LKI-14','','','Test',541,'',1000 from dual union all
select 34357,'1974-S87','1974-02-04 14:40:00','suborbital rocket','success',1211,'SL1203','MSSL XR13/UCB EUV','','','XR Astron/EUV Astron',1839,'',197 from dual union all
select 34358,'1974-S88','1974-02-06 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 598',1000 from dual union all
select 34359,'1974-S89','1974-02-06 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 598',1000 from dual union all
select 3436,'1955-A574','1955-04-15 00:00:00','atmospheric rocket','success',762,'6','HTV','','','Heat transfer',1895,'',10 from dual union all
select 34360,'1974-S90','1974-02-06 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 598',1000 from dual union all
select 34361,'1974-S91','1974-02-06 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 598',1000 from dual union all
select 34362,'1974-S92','1974-02-06 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 598',1000 from dual union all
select 34363,'1974-S93','1974-02-06 22:48:57','suborbital rocket','success',220,'ESRO D-101','DLR A-BB4-63','','','Auroral',29,'',548 from dual union all
select 34364,'1974-S94','1974-02-08 04:16:21','suborbital rocket','success',132,'CRL A30.311-4','Tracking test','','','Test',1047,'',91 from dual union all
select 34365,'1974-S95','1974-02-09 02:10:00','suborbital rocket','success',41,'NASA 26.21UG','CAL XR 9','','','XR Astron',1895,'',186 from dual union all
select 34366,'1974-S96','1974-02-09 06:30:00','suborbital rocket','failure',693,'LS-C-7','LE-3 Test','','','Test',1476,'',15 from dual union all
select 34367,'1974-S97','1974-02-12 04:00:00','suborbital rocket','success',41,'NASA 26.31UG','','','','UV Astron',1895,'',227 from dual union all
select 34368,'1974-S98','1974-02-13 00:00:00','military missile','success',136,'H012','Mk 12 RV','HAVE MILL 1','','Reentry Test',1784,'',450 from dual union all
select 34369,'1974-S99','1974-02-13 15:36:00','suborbital rocket','success',740,'D-75MG','IPOCAMP','','','Aeron/Ionos/CR/Plas',1010,'',184 from dual union all
select 3437,'1955-A575','1955-06-17 00:00:00','atmospheric rocket','success',563,'7','HTV','','','Heat transfer',1895,'',10 from dual union all
select 34370,'1974-W01','1974-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 34371,'1974-W02','1974-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 34372,'1974-W03','1974-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34373,'1974-W04','1974-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34374,'1974-W05','1974-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34375,'1974-W06','1974-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34376,'1974-W07','1974-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34377,'1974-W08','1974-01-02 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 34378,'1974-W09','1974-01-02 17:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 34379,'1974-W10','1974-01-02 18:32:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 3438,'1955-A576','1955-06-28 00:00:00','atmospheric rocket','success',563,'8','HTV','','','Heat transfer',1895,'',10 from dual union all
select 34380,'1974-W100','1974-01-23 14:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 34381,'1974-W1000','1974-08-02 19:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 34382,'1974-W1001','1974-08-02 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',74 from dual union all
select 34383,'1974-W1002','1974-08-02 21:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 34384,'1974-W1003','1974-08-03 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 34385,'1974-W1004','1974-08-05 10:19:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',66 from dual union all
select 34386,'1974-W1005','1974-08-05 12:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',75 from dual union all
select 34387,'1974-W1006','1974-08-05 15:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 34388,'1974-W1007','1974-08-05 15:55:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',68 from dual;
insert into LAUNCH
select 34389,'1974-W1008','1974-08-05 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 3439,'1955-A577','1955-06-30 00:00:00','atmospheric rocket','success',563,'9','HTV','','','Heat transfer',1895,'',10 from dual union all
select 34390,'1974-W1009','1974-08-05 18:32:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1821,'',57 from dual union all
select 34391,'1974-W101','1974-01-23 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',67 from dual union all
select 34392,'1974-W1010','1974-08-05 21:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 34393,'1974-W1011','1974-08-06 14:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 34394,'1974-W1012','1974-08-07 09:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 34395,'1974-W1013','1974-08-07 09:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',63 from dual union all
select 34396,'1974-W1014','1974-08-07 11:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 34397,'1974-W1015','1974-08-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',73 from dual union all
select 34398,'1974-W1016','1974-08-07 15:24:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',71 from dual union all
select 34399,'1974-W1017','1974-08-07 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 344,'1944-S313','1944-06-01 00:00:00','military missile','success',8,'','Vertical test','','','Test',673,'',110 from dual union all
select 3440,'1955-A578','1955-10-19 00:00:00','atmospheric rocket','success',563,'10','HTV 1366','','','Heat transfer',1895,'',10 from dual union all
select 34400,'1974-W1018','1974-08-07 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34401,'1974-W1019','1974-08-07 17:06:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 34402,'1974-W102','1974-01-23 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 34403,'1974-W1020','1974-08-07 19:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',75 from dual union all
select 34404,'1974-W1021','1974-08-08 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 34405,'1974-W1022','1974-08-08 09:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',61 from dual union all
select 34406,'1974-W1023','1974-08-08 11:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 34407,'1974-W1024','1974-08-08 15:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 34408,'1974-W1025','1974-08-09 09:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',64 from dual union all
select 34409,'1974-W1026','1974-08-09 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 3441,'1955-A579','1955-10-26 00:00:00','atmospheric rocket','success',563,'11','HTV 1366','','','Heat transfer',1895,'',10 from dual union all
select 34410,'1974-W1027','1974-08-09 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',72 from dual union all
select 34411,'1974-W1028','1974-08-09 16:21:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 34412,'1974-W1029','1974-08-09 16:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 34413,'1974-W103','1974-01-23 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',63 from dual union all
select 34414,'1974-W1030','1974-08-09 20:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 34415,'1974-W1031','1974-08-12 09:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',73 from dual union all
select 34416,'1974-W1032','1974-08-12 11:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 34417,'1974-W1033','1974-08-12 15:00:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',60 from dual union all
select 34418,'1974-W1034','1974-08-12 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 34419,'1974-W1035','1974-08-12 16:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 3442,'1955-A58','1955-01-28 00:00:00','atmospheric rocket','success',836,'1960T','','','','Test',1895,'',10 from dual union all
select 34420,'1974-W1036','1974-08-12 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 34421,'1974-W1037','1974-08-12 21:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 34422,'1974-W1038','1974-08-13 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 34423,'1974-W1039','1974-08-14 09:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 34424,'1974-W104','1974-01-23 18:33:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',77 from dual union all
select 34425,'1974-W1040','1974-08-14 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',58 from dual union all
select 34426,'1974-W1041','1974-08-14 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 34427,'1974-W1042','1974-08-14 14:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 34428,'1974-W1043','1974-08-14 14:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 34429,'1974-W1044','1974-08-14 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 3443,'1955-A580','1955-11-22 00:00:00','atmospheric rocket','success',563,'14','HTV 1366','','','Heat transfer',1895,'',10 from dual union all
select 34430,'1974-W1045','1974-08-15 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 34431,'1974-W1046','1974-08-15 09:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',64 from dual union all
select 34432,'1974-W1047','1974-08-15 13:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',76 from dual union all
select 34433,'1974-W1048','1974-08-15 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 34434,'1974-W1049','1974-08-16 10:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 34435,'1974-W105','1974-01-23 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 34436,'1974-W1050','1974-08-16 14:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 34437,'1974-W1051','1974-08-16 14:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 34438,'1974-W1052','1974-08-16 14:57:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 34439,'1974-W1053','1974-08-16 15:14:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',75 from dual union all
select 3444,'1955-A581','1955-12-08 00:00:00','atmospheric rocket','success',563,'15','HTV 1366','','','Heat transfer',1895,'',10 from dual union all
select 34440,'1974-W1054','1974-08-16 16:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 34441,'1974-W1055','1974-08-16 21:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 34442,'1974-W1056','1974-08-17 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 34443,'1974-W1057','1974-08-19 09:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',66 from dual union all
select 34444,'1974-W1058','1974-08-19 12:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 34445,'1974-W1059','1974-08-19 14:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 34446,'1974-W106','1974-01-23 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 34447,'1974-W1060','1974-08-19 16:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 34448,'1974-W1061','1974-08-19 16:42:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 34449,'1974-W1062','1974-08-19 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 3445,'1955-A582','1955-04-28 00:00:00','military missile','unknown',339,'1575R','','','','Test',1895,'',40 from dual union all
select 34450,'1974-W1063','1974-08-19 18:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 34451,'1974-W1064','1974-08-19 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 34452,'1974-W1065','1974-08-20 13:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',73 from dual union all
select 34453,'1974-W1066','1974-08-20 16:34:59','suborbital rocket','success',1061,'','Met Para,STS-','','','Meteo',1895,'',62 from dual union all
select 34454,'1974-W1067','1974-08-20 17:30:00','suborbital rocket','success',1061,'','Met Para,STS-','','','Meteo',1895,'',58 from dual union all
select 34455,'1974-W1068','1974-08-21 09:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',57 from dual union all
select 34456,'1974-W1069','1974-08-21 13:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',75 from dual union all
select 34457,'1974-W107','1974-01-23 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 34458,'1974-W1070','1974-08-21 14:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 34459,'1974-W1071','1974-08-21 14:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',72 from dual union all
select 3446,'1955-A583','1955-06-21 00:00:00','military missile','unknown',339,'1602T','','','','Test',1895,'',40 from dual union all
select 34460,'1974-W1072','1974-08-21 15:22:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 34461,'1974-W1073','1974-08-21 16:04:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',75 from dual union all
select 34462,'1974-W1074','1974-08-21 16:43:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 34463,'1974-W1075','1974-08-21 17:49:59','suborbital rocket','success',1061,'','Met Para,STS-','','','Meteo',1895,'',73 from dual union all
select 34464,'1974-W1076','1974-08-21 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 34465,'1974-W1077','1974-08-21 18:24:59','suborbital rocket','success',1061,'','Met Para,STS-','','','Meteo',1895,'',70 from dual union all
select 34466,'1974-W1078','1974-08-21 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 34467,'1974-W1079','1974-08-22 01:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',72 from dual union all
select 34468,'1974-W108','1974-01-24 01:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 34469,'1974-W1080','1974-08-22 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 3447,'1955-A584','1955-06-24 00:00:00','military missile','unknown',339,'1603T','','','','Test',1895,'',40 from dual union all
select 34470,'1974-W1081','1974-08-23 09:01:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',70 from dual union all
select 34471,'1974-W1082','1974-08-23 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 34472,'1974-W1083','1974-08-23 14:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 34473,'1974-W1084','1974-08-23 14:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 34474,'1974-W1085','1974-08-23 16:41:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 34475,'1974-W1086','1974-08-23 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 34476,'1974-W1087','1974-08-23 20:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34477,'1974-W1088','1974-08-26 09:15:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',66 from dual union all
select 34478,'1974-W1089','1974-08-26 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual;
insert into LAUNCH
select 34479,'1974-W109','1974-01-24 09:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 3448,'1955-A585','1955-06-28 00:00:00','military missile','unknown',339,'1601T','','','','Test',1895,'',40 from dual union all
select 34480,'1974-W1090','1974-08-26 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',79 from dual union all
select 34481,'1974-W1091','1974-08-26 16:46:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 34482,'1974-W1092','1974-08-26 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34483,'1974-W1093','1974-08-26 21:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 34484,'1974-W1094','1974-08-27 20:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 34485,'1974-W1095','1974-08-28 09:01:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',59 from dual union all
select 34486,'1974-W1096','1974-08-28 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 34487,'1974-W1097','1974-08-28 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 34488,'1974-W1098','1974-08-28 16:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 34489,'1974-W1099','1974-08-28 16:53:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 3449,'1955-A586','1955-07-01 00:00:00','military missile','unknown',339,'1604T','','','','Test',1895,'',40 from dual union all
select 34490,'1974-W11','1974-01-02 19:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 34491,'1974-W110','1974-01-24 15:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 34492,'1974-W1100','1974-08-28 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 34493,'1974-W1101','1974-08-28 18:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 34494,'1974-W1102','1974-08-28 19:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 34495,'1974-W1103','1974-08-29 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',71 from dual union all
select 34496,'1974-W1104','1974-08-29 09:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',52 from dual union all
select 34497,'1974-W1105','1974-08-29 14:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 34498,'1974-W1106','1974-08-29 15:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 34499,'1974-W1107','1974-08-29 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 345,'1944-S314','1944-06-01 00:00:00','military missile','success',8,'','Vertical test','','','Test',673,'',110 from dual union all
select 3450,'1955-A587','1955-07-11 00:00:00','military missile','unknown',339,'1571E','','','','Test',1895,'',40 from dual union all
select 34500,'1974-W1108','1974-08-30 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 34501,'1974-W1109','1974-08-30 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',72 from dual union all
select 34502,'1974-W111','1974-01-24 20:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 34503,'1974-W1110','1974-08-30 15:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 34504,'1974-W1111','1974-08-30 15:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 34505,'1974-W1112','1974-08-30 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 34506,'1974-W1113','1974-09-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 34507,'1974-W1114','1974-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34508,'1974-W1115','1974-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34509,'1974-W1116','1974-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 3451,'1955-A588','1955-07-18 00:00:00','military missile','unknown',339,'T1331','','','','Test',1895,'',40 from dual union all
select 34510,'1974-W1117','1974-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34511,'1974-W1118','1974-09-02 16:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 34512,'1974-W1119','1974-09-03 01:30:00','suborbital rocket','success',761,'MT135P T11','','','','Aeron?',1476,'',50 from dual union all
select 34513,'1974-W112','1974-01-25 13:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 34514,'1974-W1120','1974-09-03 06:00:00','suborbital rocket','success',761,'MT135P T12','','','','Aeron?',1476,'',50 from dual union all
select 34515,'1974-W1121','1974-09-03 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 34516,'1974-W1122','1974-09-03 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 34517,'1974-W1123','1974-09-03 19:52:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',55 from dual union all
select 34518,'1974-W1124','1974-09-04 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',60 from dual union all
select 34519,'1974-W1125','1974-09-04 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 3452,'1955-A589','1955-07-19 00:00:00','military missile','unknown',339,'T1337','','','','Test',1895,'',40 from dual union all
select 34520,'1974-W1126','1974-09-04 15:26:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 34521,'1974-W1127','1974-09-04 17:02:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',65 from dual union all
select 34522,'1974-W1128','1974-09-04 17:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 34523,'1974-W1129','1974-09-04 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34524,'1974-W113','1974-01-25 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 34525,'1974-W1130','1974-09-04 17:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 34526,'1974-W1131','1974-09-04 20:02:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',62 from dual union all
select 34527,'1974-W1132','1974-09-04 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 34528,'1974-W1133','1974-09-05 01:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 34529,'1974-W1134','1974-09-05 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 3453,'1955-A59','1955-01-28 00:00:00','atmospheric rocket','success',836,'1481T','','','','Test',1895,'',10 from dual union all
select 34530,'1974-W1135','1974-09-05 17:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34531,'1974-W1136','1974-09-05 19:58:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',63 from dual union all
select 34532,'1974-W1137','1974-09-06 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 34533,'1974-W1138','1974-09-06 09:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 34534,'1974-W1139','1974-09-06 14:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 34535,'1974-W114','1974-01-25 16:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 34536,'1974-W1140','1974-09-06 14:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 34537,'1974-W1141','1974-09-06 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 34538,'1974-W1142','1974-09-06 15:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 34539,'1974-W1143','1974-09-06 15:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',74 from dual union all
select 3454,'1955-A590','1955-07-19 00:00:00','military missile','unknown',339,'1566','Corporal EU','','','Test',1895,'',40 from dual union all
select 34540,'1974-W1144','1974-09-06 15:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 34541,'1974-W1145','1974-09-06 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 34542,'1974-W1146','1974-09-07 05:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 34543,'1974-W1147','1974-09-08 05:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 34544,'1974-W1148','1974-09-09 14:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 34545,'1974-W1149','1974-09-09 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 34546,'1974-W115','1974-01-25 17:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 34547,'1974-W1150','1974-09-09 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 34548,'1974-W1151','1974-09-09 22:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 34549,'1974-W1152','1974-09-10 09:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 3455,'1955-A591','1955-07-26 00:00:00','military missile','unknown',339,'JS1584','Corporal C','','','Test',1895,'',40 from dual union all
select 34550,'1974-W1153','1974-09-10 13:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',76 from dual union all
select 34551,'1974-W1154','1974-09-10 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',52 from dual union all
select 34552,'1974-W1155','1974-09-10 19:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1390,'SHOK',79 from dual union all
select 34553,'1974-W1156','1974-09-10 19:49:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',51 from dual union all
select 34554,'1974-W1157','1974-09-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',68 from dual union all
select 34555,'1974-W1158','1974-09-11 14:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',75 from dual union all
select 34556,'1974-W1159','1974-09-11 15:21:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 34557,'1974-W116','1974-01-25 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 34558,'1974-W1160','1974-09-11 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 34559,'1974-W1161','1974-09-11 15:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 3456,'1955-A592','1955-07-26 00:00:00','military missile','unknown',339,'1465','Corporal T','','','Test',1895,'',40 from dual union all
select 34560,'1974-W1162','1974-09-11 15:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 34561,'1974-W1163','1974-09-11 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 34562,'1974-W1164','1974-09-11 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34563,'1974-W1165','1974-09-11 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',62 from dual union all
select 34564,'1974-W1166','1974-09-11 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 34565,'1974-W1167','1974-09-12 10:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 34566,'1974-W1168','1974-09-12 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34567,'1974-W1169','1974-09-12 20:02:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',57 from dual union all
select 34568,'1974-W117','1974-01-25 19:41:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual;
insert into LAUNCH
select 34569,'1974-W1170','1974-09-12 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 3457,'1955-A593','1955-07-28 00:00:00','military missile','unknown',339,'1340','Corporal T','','','Test',1895,'',40 from dual union all
select 34570,'1974-W1171','1974-09-13 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',73 from dual union all
select 34571,'1974-W1172','1974-09-13 14:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 34572,'1974-W1173','1974-09-13 14:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 34573,'1974-W1174','1974-09-13 14:37:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 34574,'1974-W1175','1974-09-13 16:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',50 from dual union all
select 34575,'1974-W1176','1974-09-13 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 34576,'1974-W1177','1974-09-14 11:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 34577,'1974-W1178','1974-09-16 09:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 34578,'1974-W1179','1974-09-16 13:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',72 from dual union all
select 34579,'1974-W118','1974-01-25 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 3458,'1955-A594','1955-07-28 00:00:00','military missile','unknown',339,'1579','Corporal E','','','Test',1895,'',40 from dual union all
select 34580,'1974-W1180','1974-09-16 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 34581,'1974-W1181','1974-09-16 14:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 34582,'1974-W1182','1974-09-16 15:20:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',74 from dual union all
select 34583,'1974-W1183','1974-09-16 16:42:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 34584,'1974-W1184','1974-09-16 16:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 34585,'1974-W1185','1974-09-16 16:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 34586,'1974-W1186','1974-09-16 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 34587,'1974-W1187','1974-09-16 19:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 34588,'1974-W1188','1974-09-16 20:34:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',68 from dual union all
select 34589,'1974-W1189','1974-09-17 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 3459,'1955-A595','1955-08-04 00:00:00','military missile','unknown',339,'1585','Corporal EU','','','Test',1895,'',40 from dual union all
select 34590,'1974-W119','1974-01-26 10:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',66 from dual union all
select 34591,'1974-W1190','1974-09-17 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 34592,'1974-W1191','1974-09-18 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 34593,'1974-W1192','1974-09-18 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 34594,'1974-W1193','1974-09-18 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',70 from dual union all
select 34595,'1974-W1194','1974-09-18 15:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 34596,'1974-W1195','1974-09-18 15:06:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 34597,'1974-W1196','1974-09-18 15:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 34598,'1974-W1197','1974-09-18 15:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 34599,'1974-W1198','1974-09-18 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 346,'1944-S315','1944-06-01 00:00:00','military missile','success',8,'','Vertical test','','','Test',673,'',110 from dual union all
select 3460,'1955-A596','1955-08-17 00:00:00','military missile','unknown',339,'1589','Corporal EU','','','Test',1895,'',40 from dual union all
select 34600,'1974-W1199','1974-09-18 20:26:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',53 from dual union all
select 34601,'1974-W12','1974-01-02 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 34602,'1974-W120','1974-01-28 13:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',50 from dual union all
select 34603,'1974-W1200','1974-09-18 20:37:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 34604,'1974-W1201','1974-09-19 01:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',67 from dual union all
select 34605,'1974-W1202','1974-09-19 12:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',52 from dual union all
select 34606,'1974-W1203','1974-09-19 14:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 34607,'1974-W1204','1974-09-19 16:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 34608,'1974-W1205','1974-09-19 18:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 34609,'1974-W1206','1974-09-19 18:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 3461,'1955-A597','1955-08-19 00:00:00','military missile','unknown',339,'1628','Corporal U II','','','Test',1895,'',40 from dual union all
select 34610,'1974-W1207','1974-09-19 19:42:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 34611,'1974-W1208','1974-09-20 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 34612,'1974-W1209','1974-09-20 16:35:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 34613,'1974-W121','1974-01-28 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 34614,'1974-W1210','1974-09-20 20:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 34615,'1974-W1211','1974-09-20 20:24:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',57 from dual union all
select 34616,'1974-W1212','1974-09-21 01:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 34617,'1974-W1213','1974-09-23 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',72 from dual union all
select 34618,'1974-W1214','1974-09-23 14:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 34619,'1974-W1215','1974-09-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 3462,'1955-A598','1955-08-23 00:00:00','military missile','unknown',339,'1607-JS','Corporal EU','','','Test',1895,'',40 from dual union all
select 34620,'1974-W1216','1974-09-23 16:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 34621,'1974-W1217','1974-09-23 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 34622,'1974-W1218','1974-09-23 20:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 34623,'1974-W1219','1974-09-24 12:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 34624,'1974-W122','1974-01-28 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 34625,'1974-W1220','1974-09-24 14:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',74 from dual union all
select 34626,'1974-W1221','1974-09-24 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 34627,'1974-W1222','1974-09-24 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 34628,'1974-W1223','1974-09-25 00:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',61 from dual union all
select 34629,'1974-W1224','1974-09-25 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 3463,'1955-A599','1955-08-26 00:00:00','military missile','unknown',339,'1388','Corporal T','','','Test',1895,'',40 from dual union all
select 34630,'1974-W1225','1974-09-25 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 34631,'1974-W1226','1974-09-25 14:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 34632,'1974-W1227','1974-09-25 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 34633,'1974-W1228','1974-09-25 15:19:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 34634,'1974-W1229','1974-09-25 18:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 34635,'1974-W123','1974-01-28 16:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 34636,'1974-W1230','1974-09-25 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 34637,'1974-W1231','1974-09-26 00:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',71 from dual union all
select 34638,'1974-W1232','1974-09-26 15:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 34639,'1974-W1233','1974-09-26 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 3464,'1955-A60','1955-01-28 00:00:00','atmospheric rocket','success',836,'2838A','','','','Test',1895,'',10 from dual union all
select 34640,'1974-W1234','1974-09-26 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',60 from dual union all
select 34641,'1974-W1235','1974-09-27 00:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 34642,'1974-W1236','1974-09-27 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 34643,'1974-W1237','1974-09-27 14:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 34644,'1974-W1238','1974-09-27 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 34645,'1974-W1239','1974-09-27 16:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 34646,'1974-W124','1974-01-28 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 34647,'1974-W1240','1974-09-28 11:37:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 34648,'1974-W1241','1974-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 34649,'1974-W1242','1974-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 3465,'1955-A600','1955-08-26 00:00:00','military missile','unknown',339,'1389','Corporal T','','','Test',1895,'',40 from dual union all
select 34650,'1974-W1243','1974-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 34651,'1974-W1244','1974-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 34652,'1974-W1245','1974-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 34653,'1974-W1246','1974-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 34654,'1974-W1247','1974-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34655,'1974-W1248','1974-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34656,'1974-W1249','1974-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34657,'1974-W125','1974-01-28 19:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 34658,'1974-W1250','1974-09-30 14:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual;
insert into LAUNCH
select 34659,'1974-W1251','1974-09-30 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 3466,'1955-A601','1955-08-30 00:00:00','military missile','unknown',339,'1618','Corporal II U-4','','','Test',1895,'',40 from dual union all
select 34660,'1974-W1252','1974-09-30 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 34661,'1974-W1253','1974-09-30 16:11:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 34662,'1974-W1254','1974-09-30 16:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 34663,'1974-W1255','1974-09-30 16:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 34664,'1974-W1256','1974-09-30 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 34665,'1974-W1257','1974-09-30 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 34666,'1974-W1258','1974-10-01 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 34667,'1974-W1259','1974-10-02 09:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 34668,'1974-W126','1974-01-28 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 34669,'1974-W1260','1974-10-02 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',75 from dual union all
select 3467,'1955-A602','1955-09-01 00:00:00','military missile','unknown',339,'1614','Corporal II U-3','','','Test',1895,'',40 from dual union all
select 34670,'1974-W1261','1974-10-02 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',72 from dual union all
select 34671,'1974-W1262','1974-10-02 14:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 34672,'1974-W1263','1974-10-02 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 34673,'1974-W1264','1974-10-02 16:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',52 from dual union all
select 34674,'1974-W1265','1974-10-02 17:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',74 from dual union all
select 34675,'1974-W1266','1974-10-02 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',64 from dual union all
select 34676,'1974-W1267','1974-10-02 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 34677,'1974-W1268','1974-10-02 20:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',72 from dual union all
select 34678,'1974-W1269','1974-10-02 21:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 34679,'1974-W127','1974-01-29 09:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 3468,'1955-A603','1955-09-12 00:00:00','military missile','unknown',339,'1608','Corporal EU','','','Test',1895,'',40 from dual union all
select 34680,'1974-W1270','1974-10-03 00:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 34681,'1974-W1271','1974-10-03 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',70 from dual union all
select 34682,'1974-W1272','1974-10-03 17:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34683,'1974-W1273','1974-10-03 20:08:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',58 from dual union all
select 34684,'1974-W1274','1974-10-04 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 34685,'1974-W1275','1974-10-04 14:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 34686,'1974-W1276','1974-10-04 14:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 34687,'1974-W1277','1974-10-04 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',63 from dual union all
select 34688,'1974-W1278','1974-10-04 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 34689,'1974-W1279','1974-10-04 21:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 3469,'1955-A604','1955-09-20 00:00:00','military missile','unknown',339,'1654','Corporal TA-28','','','Test',1895,'',40 from dual union all
select 34690,'1974-W128','1974-01-29 12:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',69 from dual union all
select 34691,'1974-W1280','1974-10-07 09:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',61 from dual union all
select 34692,'1974-W1281','1974-10-07 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 34693,'1974-W1282','1974-10-07 15:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 34694,'1974-W1283','1974-10-07 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',73 from dual union all
select 34695,'1974-W1284','1974-10-07 17:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 34696,'1974-W1285','1974-10-07 23:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 34697,'1974-W1286','1974-10-08 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 34698,'1974-W1287','1974-10-08 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 34699,'1974-W1288','1974-10-08 19:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',79 from dual union all
select 347,'1944-S316','1944-06-01 00:00:00','military missile','success',8,'','Vertical test','','','Test',673,'',110 from dual union all
select 3470,'1955-A605','1955-09-20 00:00:00','military missile','unknown',339,'1621','Corporal EU','','','Test',1895,'',40 from dual union all
select 34700,'1974-W1289','1974-10-08 19:49:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',66 from dual union all
select 34701,'1974-W129','1974-01-29 15:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',70 from dual union all
select 34702,'1974-W1290','1974-10-08 22:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',77 from dual union all
select 34703,'1974-W1291','1974-10-09 14:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 34704,'1974-W1292','1974-10-09 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 34705,'1974-W1293','1974-10-09 15:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 34706,'1974-W1294','1974-10-09 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34707,'1974-W1295','1974-10-09 20:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',66 from dual union all
select 34708,'1974-W1296','1974-10-09 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 34709,'1974-W1297','1974-10-09 21:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 3471,'1955-A606','1955-09-22 00:00:00','military missile','unknown',339,'1660','Corporal TA-29','','','Test',1895,'',40 from dual union all
select 34710,'1974-W1298','1974-10-09 21:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 34711,'1974-W1299','1974-10-10 01:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',60 from dual union all
select 34712,'1974-W13','1974-01-03 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 34713,'1974-W130','1974-01-29 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 34714,'1974-W1300','1974-10-10 09:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 34715,'1974-W1301','1974-10-10 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',53 from dual union all
select 34716,'1974-W1302','1974-10-10 21:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 34717,'1974-W1303','1974-10-11 14:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 34718,'1974-W1304','1974-10-11 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',70 from dual union all
select 34719,'1974-W1305','1974-10-11 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 3472,'1955-A607','1955-09-23 00:00:00','military missile','unknown',339,'1659','Corporal T-01','','','Test',1895,'',40 from dual union all
select 34720,'1974-W1306','1974-10-11 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 34721,'1974-W1307','1974-10-11 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',68 from dual union all
select 34722,'1974-W1308','1974-10-11 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',61 from dual union all
select 34723,'1974-W1309','1974-10-12 01:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 34724,'1974-W131','1974-01-29 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 34725,'1974-W1310','1974-10-14 00:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 34726,'1974-W1311','1974-10-14 04:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 34727,'1974-W1312','1974-10-15 09:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 34728,'1974-W1313','1974-10-15 13:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',74 from dual union all
select 34729,'1974-W1314','1974-10-15 16:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 3473,'1955-A608','1955-09-29 00:00:00','military missile','unknown',339,'1622','Corporal EU','','','Test',1895,'',40 from dual union all
select 34730,'1974-W1315','1974-10-15 17:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 34731,'1974-W1316','1974-10-15 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 34732,'1974-W1317','1974-10-15 20:35:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',68 from dual union all
select 34733,'1974-W1318','1974-10-16 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 34734,'1974-W1319','1974-10-16 14:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 34735,'1974-W132','1974-01-29 19:17:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 34736,'1974-W1320','1974-10-16 14:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',73 from dual union all
select 34737,'1974-W1321','1974-10-16 15:09:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 34738,'1974-W1322','1974-10-16 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',73 from dual union all
select 34739,'1974-W1323','1974-10-16 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 3474,'1955-A609','1955-09-30 00:00:00','military missile','unknown',339,'1657','Corporal T-02','','','Test',1895,'',40 from dual union all
select 34740,'1974-W1324','1974-10-16 20:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',67 from dual union all
select 34741,'1974-W1325','1974-10-16 21:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 34742,'1974-W1326','1974-10-17 01:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 34743,'1974-W1327','1974-10-17 13:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34744,'1974-W1328','1974-10-17 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34745,'1974-W1329','1974-10-18 13:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 34746,'1974-W133','1974-01-30 13:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',72 from dual union all
select 34747,'1974-W1330','1974-10-18 14:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 34748,'1974-W1331','1974-10-18 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',69 from dual;
insert into LAUNCH
select 34749,'1974-W1332','1974-10-18 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 3475,'1955-A61','1955-01-28 00:00:00','atmospheric rocket','success',836,'2790A','','','','Test',1895,'',10 from dual union all
select 34750,'1974-W1333','1974-10-18 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 34751,'1974-W1334','1974-10-18 17:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 34752,'1974-W1335','1974-10-18 17:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 34753,'1974-W1336','1974-10-18 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',67 from dual union all
select 34754,'1974-W1337','1974-10-18 21:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 34755,'1974-W1338','1974-10-21 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',53 from dual union all
select 34756,'1974-W1339','1974-10-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 34757,'1974-W134','1974-01-30 13:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 34758,'1974-W1340','1974-10-21 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',52 from dual union all
select 34759,'1974-W1341','1974-10-21 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 3476,'1955-A610','1955-10-07 00:00:00','military missile','unknown',339,'1616','Corporal U-5','','','Test',1895,'',40 from dual union all
select 34760,'1974-W1342','1974-10-21 16:07:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 34761,'1974-W1343','1974-10-21 17:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 34762,'1974-W1344','1974-10-21 18:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 34763,'1974-W1345','1974-10-21 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',69 from dual union all
select 34764,'1974-W1346','1974-10-22 14:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',73 from dual union all
select 34765,'1974-W1347','1974-10-23 12:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',75 from dual union all
select 34766,'1974-W1348','1974-10-23 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 34767,'1974-W1349','1974-10-23 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 34768,'1974-W135','1974-01-30 14:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 34769,'1974-W1350','1974-10-23 13:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 3477,'1955-A611','1955-10-11 00:00:00','military missile','unknown',339,'1588','Corporal UM-7','','','Test',1895,'',40 from dual union all
select 34770,'1974-W1351','1974-10-23 15:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 34771,'1974-W1352','1974-10-23 15:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',69 from dual union all
select 34772,'1974-W1353','1974-10-23 17:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',71 from dual union all
select 34773,'1974-W1354','1974-10-23 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 34774,'1974-W1355','1974-10-23 20:15:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',64 from dual union all
select 34775,'1974-W1356','1974-10-23 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 34776,'1974-W1357','1974-10-24 13:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',69 from dual union all
select 34777,'1974-W1358','1974-10-24 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 34778,'1974-W1359','1974-10-25 09:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 34779,'1974-W136','1974-01-30 15:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 3478,'1955-A612','1955-10-12 00:00:00','military missile','unknown',339,'1617','Corporal II U-6','','','Test',1895,'',40 from dual union all
select 34780,'1974-W1360','1974-10-25 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 34781,'1974-W1361','1974-10-25 13:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 34782,'1974-W1362','1974-10-25 16:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 34783,'1974-W1363','1974-10-25 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 34784,'1974-W1364','1974-10-28 16:30:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 34785,'1974-W1365','1974-10-28 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 34786,'1974-W1366','1974-10-29 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 34787,'1974-W1367','1974-10-29 20:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 34788,'1974-W1368','1974-10-29 20:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 34789,'1974-W1369','1974-10-29 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',60 from dual union all
select 3479,'1955-A613','1955-10-14 00:00:00','military missile','unknown',339,'1662','Corporal TA-30','','','Test',1895,'',40 from dual union all
select 34790,'1974-W137','1974-01-30 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 34791,'1974-W1370','1974-10-30 09:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 34792,'1974-W1371','1974-10-30 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',75 from dual union all
select 34793,'1974-W1372','1974-10-30 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 34794,'1974-W1373','1974-10-30 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 34795,'1974-W1374','1974-10-30 16:18:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 34796,'1974-W1375','1974-10-30 16:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 34797,'1974-W1376','1974-10-30 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 34798,'1974-W1377','1974-10-30 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',58 from dual union all
select 34799,'1974-W1378','1974-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 348,'1944-S317','1944-06-01 00:00:00','military missile','success',8,'','Vertical test','','','Test',673,'',110 from dual union all
select 3480,'1955-A614','1955-10-19 00:00:00','military missile','unknown',339,'1652','Corporal TA-32','','','Test',1895,'',40 from dual union all
select 34800,'1974-W1379','1974-11-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34801,'1974-W138','1974-01-30 15:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 34802,'1974-W1380','1974-11-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34803,'1974-W1381','1974-11-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34804,'1974-W1382','1974-11-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34805,'1974-W1383','1974-10-31 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 34806,'1974-W1384','1974-10-31 15:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 34807,'1974-W1385','1974-10-31 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 34808,'1974-W1386','1974-11-01 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',72 from dual union all
select 34809,'1974-W1387','1974-11-01 15:55:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 3481,'1955-A615','1955-10-20 00:00:00','military missile','unknown',339,'1578','Corporal U-II U-8','','','Test',1895,'',40 from dual union all
select 34810,'1974-W1388','1974-11-01 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 34811,'1974-W1389','1974-11-01 20:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 34812,'1974-W139','1974-01-30 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',73 from dual union all
select 34813,'1974-W1390','1974-11-01 21:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',66 from dual union all
select 34814,'1974-W1391','1974-11-04 00:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',65 from dual union all
select 34815,'1974-W1392','1974-11-04 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',73 from dual union all
select 34816,'1974-W1393','1974-11-04 15:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 34817,'1974-W1394','1974-11-04 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 34818,'1974-W1395','1974-11-04 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 34819,'1974-W1396','1974-11-05 15:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 3482,'1955-A616','1955-10-24 00:00:00','military missile','unknown',339,'1630','Corporal U-9','','','Test',1895,'',40 from dual union all
select 34820,'1974-W1397','1974-11-05 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34821,'1974-W1398','1974-11-05 16:21:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 34822,'1974-W1399','1974-11-05 20:02:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',54 from dual union all
select 34823,'1974-W14','1974-01-03 15:56:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 34824,'1974-W140','1974-01-30 18:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',60 from dual union all
select 34825,'1974-W1400','1974-11-06 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 34826,'1974-W1401','1974-11-06 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',67 from dual union all
select 34827,'1974-W1402','1974-11-06 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 34828,'1974-W1403','1974-11-06 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 34829,'1974-W1404','1974-11-06 16:14:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 3483,'1955-A617','1955-10-24 00:00:00','military missile','unknown',339,'1658','Corporal TA-33','','','Test',1895,'',40 from dual union all
select 34830,'1974-W1405','1974-11-06 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34831,'1974-W1406','1974-11-06 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 34832,'1974-W1407','1974-11-06 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',67 from dual union all
select 34833,'1974-W1408','1974-11-07 09:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 34834,'1974-W1409','1974-11-07 12:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',73 from dual union all
select 34835,'1974-W141','1974-01-30 18:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',75 from dual union all
select 34836,'1974-W1410','1974-11-07 15:45:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 34837,'1974-W1411','1974-11-07 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 34838,'1974-W1412','1974-11-08 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual;
insert into LAUNCH
select 34839,'1974-W1413','1974-11-08 15:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',78 from dual union all
select 3484,'1955-A618','1955-10-26 00:00:00','military missile','unknown',339,'1631','Corporal U-10','','','Test',1895,'',40 from dual union all
select 34840,'1974-W1414','1974-11-08 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34841,'1974-W1415','1974-11-08 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',52 from dual union all
select 34842,'1974-W1416','1974-11-08 17:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 34843,'1974-W1417','1974-11-08 20:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',60 from dual union all
select 34844,'1974-W1418','1974-11-11 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 34845,'1974-W1419','1974-11-11 15:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 34846,'1974-W142','1974-01-30 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 34847,'1974-W1420','1974-11-11 15:59:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',70 from dual union all
select 34848,'1974-W1421','1974-11-11 17:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 34849,'1974-W1422','1974-11-11 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',64 from dual union all
select 3485,'1955-A619','1955-10-31 00:00:00','military missile','unknown',339,'1626','Corporal UM-11','','','Test',1895,'',40 from dual union all
select 34850,'1974-W1423','1974-11-12 13:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',63 from dual union all
select 34851,'1974-W1424','1974-11-12 15:37:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 34852,'1974-W1425','1974-11-13 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 34853,'1974-W1426','1974-11-13 14:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 34854,'1974-W1427','1974-11-13 14:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 34855,'1974-W1428','1974-11-13 16:10:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',51 from dual union all
select 34856,'1974-W1429','1974-11-13 16:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 34857,'1974-W143','1974-01-30 21:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 34858,'1974-W1430','1974-11-13 16:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 34859,'1974-W1431','1974-11-13 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',75 from dual union all
select 3486,'1955-A62','1955-02-01 00:00:00','atmospheric rocket','unknown',576,'510-RM','','','','Test',1895,'',10 from dual union all
select 34860,'1974-W1432','1974-11-13 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34861,'1974-W1433','1974-11-13 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',56 from dual union all
select 34862,'1974-W1434','1974-11-14 16:01:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 34863,'1974-W1435','1974-11-14 17:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 34864,'1974-W1436','1974-11-14 18:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 34865,'1974-W1437','1974-11-15 12:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',74 from dual union all
select 34866,'1974-W1438','1974-11-15 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 34867,'1974-W1439','1974-11-15 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 34868,'1974-W144','1974-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 34869,'1974-W1440','1974-11-15 16:19:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 3487,'1955-A620','1955-11-02 00:00:00','military missile','unknown',339,'1623','Corporal EU','','','Test',1895,'',40 from dual union all
select 34870,'1974-W1441','1974-11-15 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34871,'1974-W1442','1974-11-15 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 34872,'1974-W1443','1974-11-15 17:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 34873,'1974-W1444','1974-11-15 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',59 from dual union all
select 34874,'1974-W1445','1974-11-18 14:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 34875,'1974-W1446','1974-11-18 15:58:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 34876,'1974-W1447','1974-11-18 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 34877,'1974-W1448','1974-11-18 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 34878,'1974-W1449','1974-11-18 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 34879,'1974-W145','1974-02-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 3488,'1955-A621','1955-11-08 00:00:00','military missile','unknown',339,'1665','Corporal TA-36','','','Test',1895,'',40 from dual union all
select 34880,'1974-W1450','1974-11-18 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',65 from dual union all
select 34881,'1974-W1451','1974-11-18 21:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 34882,'1974-W1452','1974-11-19 15:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 34883,'1974-W1453','1974-11-19 18:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 34884,'1974-W1454','1974-11-20 15:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 34885,'1974-W1455','1974-11-20 16:22:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 34886,'1974-W1456','1974-11-20 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 34887,'1974-W1457','1974-11-20 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 34888,'1974-W1458','1974-11-20 17:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 34889,'1974-W1459','1974-11-20 18:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 3489,'1955-A622','1955-11-10 00:00:00','military missile','unknown',339,'1664','Corporal TA','','','Test',1895,'',40 from dual union all
select 34890,'1974-W146','1974-02-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34891,'1974-W1460','1974-11-20 18:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 34892,'1974-W1461','1974-11-21 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 34893,'1974-W1462','1974-11-21 20:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 34894,'1974-W1463','1974-11-22 15:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 34895,'1974-W1464','1974-11-22 15:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 34896,'1974-W1465','1974-11-22 16:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',72 from dual union all
select 34897,'1974-W1466','1974-11-22 17:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 34898,'1974-W1467','1974-11-22 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',67 from dual union all
select 34899,'1974-W1468','1974-11-22 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 349,'1944-S318','1944-07-31 00:00:00','military missile','success',8,'V205','','','','Test',675,'',90 from dual union all
select 3490,'1955-A623','1955-11-17 00:00:00','military missile','unknown',339,'1414','Corporal C','','','Test',1895,'',40 from dual union all
select 34900,'1974-W1469','1974-11-25 13:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 34901,'1974-W147','1974-02-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34902,'1974-W1470','1974-11-25 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',75 from dual union all
select 34903,'1974-W1471','1974-11-25 15:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 34904,'1974-W1472','1974-11-25 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual union all
select 34905,'1974-W1473','1974-11-25 16:26:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 34906,'1974-W1474','1974-11-25 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 34907,'1974-W1475','1974-11-25 20:15:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',65 from dual union all
select 34908,'1974-W1476','1974-11-25 21:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 34909,'1974-W1477','1974-11-26 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 3491,'1955-A624','1955-11-18 00:00:00','military missile','unknown',339,'1459','Corporal Chem','','','Test',1895,'',40 from dual union all
select 34910,'1974-W1478','1974-11-26 15:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 34911,'1974-W1479','1974-11-26 20:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 34912,'1974-W148','1974-01-31 00:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 34913,'1974-W1480','1974-11-27 00:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 34914,'1974-W1481','1974-11-27 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 34915,'1974-W1482','1974-11-27 14:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 34916,'1974-W1483','1974-11-27 15:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34917,'1974-W1484','1974-11-27 16:46:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 34918,'1974-W1485','1974-11-27 16:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',55 from dual union all
select 34919,'1974-W1486','1974-11-27 18:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',52 from dual union all
select 3492,'1955-A625','1955-11-21 00:00:00','military missile','unknown',339,'1651','Corporal TA-34','','','Test',1895,'',40 from dual union all
select 34920,'1974-W1487','1974-11-27 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 34921,'1974-W1488','1974-11-27 22:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 34922,'1974-W1489','1974-11-28 01:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 34923,'1974-W149','1974-01-31 15:00:00','suborbital rocket','success',88,'NASA 15.51UE','','','','Plasma',35,'',60 from dual union all
select 34924,'1974-W1490','1974-11-28 16:18:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 34925,'1974-W1491','1974-11-29 15:51:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 34926,'1974-W1492','1974-11-29 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 34927,'1974-W1493','1974-12-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 34928,'1974-W1494','1974-12-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual;
insert into LAUNCH
select 34929,'1974-W1495','1974-12-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 3493,'1955-A626','1955-11-21 00:00:00','military missile','unknown',339,'1667','Corporal TA-35','','','Test',1895,'',40 from dual union all
select 34930,'1974-W1496','1974-12-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34931,'1974-W1497','1974-12-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34932,'1974-W1498','1974-12-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 34933,'1974-W1499','1974-12-02 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 34934,'1974-W15','1974-01-03 16:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 34935,'1974-W150','1974-01-31 15:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 34936,'1974-W1500','1974-12-02 15:45:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 34937,'1974-W1501','1974-12-02 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 34938,'1974-W1502','1974-12-02 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 34939,'1974-W1503','1974-12-02 17:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 3494,'1955-A627','1955-11-22 00:00:00','military missile','unknown',339,'1595','Corporal EU','','','Test',1895,'',40 from dual union all
select 34940,'1974-W1504','1974-12-02 18:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 34941,'1974-W1505','1974-12-02 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',64 from dual union all
select 34942,'1974-W1506','1974-12-03 16:10:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 34943,'1974-W1507','1974-12-03 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 34944,'1974-W1508','1974-12-03 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 34945,'1974-W1509','1974-12-04 01:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 34946,'1974-W151','1974-01-31 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 34947,'1974-W1510','1974-12-04 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',67 from dual union all
select 34948,'1974-W1511','1974-12-04 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 34949,'1974-W1512','1974-12-04 14:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 3495,'1955-A628','1955-12-01 00:00:00','military missile','unknown',339,'1636','Corporal TA-40','','','Test',1895,'',40 from dual union all
select 34950,'1974-W1513','1974-12-04 16:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 34951,'1974-W1514','1974-12-04 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 34952,'1974-W1515','1974-12-04 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 34953,'1974-W1516','1974-12-04 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',54 from dual union all
select 34954,'1974-W1517','1974-12-05 01:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 34955,'1974-W1518','1974-12-05 15:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 34956,'1974-W1519','1974-12-05 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 34957,'1974-W152','1974-01-31 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 34958,'1974-W1520','1974-12-05 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34959,'1974-W1521','1974-12-05 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',65 from dual union all
select 3496,'1955-A629','1955-12-02 00:00:00','military missile','unknown',339,'1677','Corporal TA-38','','','Test',1895,'',40 from dual union all
select 34960,'1974-W1522','1974-12-06 13:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 34961,'1974-W1523','1974-12-06 15:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 34962,'1974-W1524','1974-12-06 15:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',72 from dual union all
select 34963,'1974-W1525','1974-12-06 16:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 34964,'1974-W1526','1974-12-06 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 34965,'1974-W1527','1974-12-06 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 34966,'1974-W1528','1974-12-06 17:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 34967,'1974-W1529','1974-12-06 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 34968,'1974-W153','1974-02-01 14:23:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 34969,'1974-W1530','1974-12-06 21:59:59','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',64 from dual union all
select 3497,'1955-A63','1955-02-01 00:00:00','atmospheric rocket','unknown',576,'469-RM','','','','Test',1895,'',10 from dual union all
select 34970,'1974-W1531','1974-12-08 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',65 from dual union all
select 34971,'1974-W1532','1974-12-09 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 34972,'1974-W1533','1974-12-09 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34973,'1974-W1534','1974-12-09 18:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 34974,'1974-W1535','1974-12-09 18:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 34975,'1974-W1536','1974-12-09 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 34976,'1974-W1537','1974-12-10 03:58:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 34977,'1974-W1538','1974-12-10 09:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 34978,'1974-W1539','1974-12-10 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 34979,'1974-W154','1974-02-01 14:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 3498,'1955-A630','1955-12-02 00:00:00','military missile','unknown',339,'1637','Corporal TA-41','','','Test',1895,'',40 from dual union all
select 34980,'1974-W1540','1974-12-10 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 34981,'1974-W1541','1974-12-10 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',65 from dual union all
select 34982,'1974-W1542','1974-12-10 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 34983,'1974-W1543','1974-12-10 22:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 34984,'1974-W1544','1974-12-11 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 34985,'1974-W1545','1974-12-11 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 34986,'1974-W1546','1974-12-11 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 34987,'1974-W1547','1974-12-11 16:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 34988,'1974-W1548','1974-12-11 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 34989,'1974-W1549','1974-12-11 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',60 from dual union all
select 3499,'1955-A631','1955-12-06 00:00:00','military missile','unknown',339,'1412','Corporal C','','','Test',1895,'',40 from dual union all
select 34990,'1974-W155','1974-02-01 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',53 from dual union all
select 34991,'1974-W1550','1974-12-11 21:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 34992,'1974-W1551','1974-12-12 01:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 34993,'1974-W1552','1974-12-12 09:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 34994,'1974-W1553','1974-12-12 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 34995,'1974-W1554','1974-12-12 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 34996,'1974-W1555','1974-12-12 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 34997,'1974-W1556','1974-12-12 22:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 34998,'1974-W1557','1974-12-12 22:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 34999,'1974-W1558','1974-12-12 23:00:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 35,'1943-S13','1943-11-09 00:00:00','military missile','success',8,'43','','','','Test',675,'',95 from dual union all
select 350,'1944-S319','1944-09-14 00:00:00','military missile','success',1470,'','London','','','Weapon',1732,'',95 from dual union all
select 3500,'1955-A632','1955-12-07 00:00:00','military missile','unknown',339,'1656','Corporal TA-39','','','Test',1895,'',40 from dual union all
select 35000,'1974-W1559','1974-12-13 05:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 35001,'1974-W156','1974-02-01 15:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 35002,'1974-W1560','1974-12-13 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 35003,'1974-W1561','1974-12-13 16:21:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 35004,'1974-W1562','1974-12-13 20:10:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',68 from dual union all
select 35005,'1974-W1563','1974-12-13 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 35006,'1974-W1564','1974-12-13 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 35007,'1974-W1565','1974-12-13 22:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 35008,'1974-W1566','1974-12-13 23:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',52 from dual union all
select 35009,'1974-W1567','1974-12-13 23:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 3501,'1955-A633','1955-12-08 00:00:00','military missile','unknown',339,'1668','Corporal II U-14','','','Test',1895,'',40 from dual union all
select 35010,'1974-W1568','1974-12-16 15:45:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 35011,'1974-W1569','1974-12-16 17:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 35012,'1974-W157','1974-02-01 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 35013,'1974-W1570','1974-12-16 18:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 35014,'1974-W1571','1974-12-16 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35015,'1974-W1572','1974-12-16 20:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 35016,'1974-W1573','1974-12-16 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',68 from dual union all
select 35017,'1974-W1574','1974-12-16 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual;
insert into LAUNCH
select 35018,'1974-W1575','1974-12-17 09:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 35019,'1974-W1576','1974-12-17 14:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 3502,'1955-A634','1955-12-08 00:00:00','military missile','unknown',339,'1627','Corporal U-13','','','Test',1895,'',40 from dual union all
select 35020,'1974-W1577','1974-12-17 15:46:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 35021,'1974-W1578','1974-12-17 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 35022,'1974-W1579','1974-12-17 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 35023,'1974-W158','1974-02-01 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',52 from dual union all
select 35024,'1974-W1580','1974-12-17 23:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 35025,'1974-W1581','1974-12-18 12:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 35026,'1974-W1582','1974-12-18 14:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',74 from dual union all
select 35027,'1974-W1583','1974-12-18 16:19:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 35028,'1974-W1584','1974-12-18 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 35029,'1974-W1585','1974-12-18 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 3503,'1955-A635','1955-12-13 00:00:00','military missile','unknown',339,'1605','Corporal TA-42','','','Test',1895,'',40 from dual union all
select 35030,'1974-W1586','1974-12-18 20:01:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',65 from dual union all
select 35031,'1974-W1587','1974-12-18 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 35032,'1974-W1588','1974-12-19 01:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 35033,'1974-W1589','1974-12-19 10:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 35034,'1974-W159','1974-02-02 01:30:00','suborbital rocket','success',761,'MT135P T9','','','','Aeron?',1476,'',50 from dual union all
select 35035,'1974-W1590','1974-12-19 15:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 35036,'1974-W1591','1974-12-19 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 35037,'1974-W1592','1974-12-19 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 35038,'1974-W1593','1974-12-20 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 35039,'1974-W1594','1974-12-20 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',52 from dual union all
select 3504,'1955-A636','1955-12-14 00:00:00','military missile','unknown',339,'1596','Corporal TA-43','','','Test',1895,'',40 from dual union all
select 35040,'1974-W1595','1974-12-20 16:30:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 35041,'1974-W1596','1974-12-20 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 35042,'1974-W1597','1974-12-20 17:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 35043,'1974-W1598','1974-12-20 21:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',64 from dual union all
select 35044,'1974-W1599','1974-12-20 21:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 35045,'1974-W16','1974-01-03 19:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 35046,'1974-W160','1974-02-04 14:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 35047,'1974-W1600','1974-12-21 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 35048,'1974-W1601','1974-12-23 14:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',55 from dual union all
select 35049,'1974-W1602','1974-12-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 3505,'1955-A637','1955-09-07 00:00:00','atmospheric rocket','success',1527,'40','','','','Test',1895,'B-29',10 from dual union all
select 35050,'1974-W1603','1974-12-23 16:23:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 35051,'1974-W1604','1974-12-23 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35052,'1974-W1605','1974-12-23 17:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 35053,'1974-W1606','1974-12-23 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 35054,'1974-W1607','1974-12-24 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 35055,'1974-W1608','1974-12-24 13:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 35056,'1974-W1609','1974-12-24 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 35057,'1974-W161','1974-02-04 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 35058,'1974-W1610','1974-12-24 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 35059,'1974-W1611','1974-12-26 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 3506,'1955-A638','1955-09-09 00:00:00','atmospheric rocket','success',1527,'2','','','','Test',1895,'B-29',10 from dual union all
select 35060,'1974-W1612','1974-12-27 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',53 from dual union all
select 35061,'1974-W1613','1974-12-27 20:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 35062,'1974-W1614','1974-12-27 21:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 35063,'1974-W1615','1974-12-28 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',72 from dual union all
select 35064,'1974-W1616','1974-12-30 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 35065,'1974-W1617','1974-12-30 15:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 35066,'1974-W1618','1974-12-30 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 35067,'1974-W1619','1974-12-30 16:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 35068,'1974-W162','1974-02-04 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 35069,'1974-W1620','1974-12-30 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 3507,'1955-A639','1955-09-12 00:00:00','atmospheric rocket','success',1527,'2','','','','Test',1895,'B-29',10 from dual union all
select 35070,'1974-W1621','1974-12-30 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 35071,'1974-W1622','1974-12-30 18:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 35072,'1974-W1623','1974-12-31 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 35073,'1974-W1624','1974-12-31 16:07:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 35074,'1974-W1625','1974-12-31 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',66 from dual union all
select 35075,'1974-W1626','1974-01-29 02:00:00','suborbital rocket','success',761,'MT135P-R139','','','','Meteo',1332,'',60 from dual union all
select 35076,'1974-W1627','1974-01-09 02:00:00','suborbital rocket','success',761,'MT135P-R144','','','','Meteo',1332,'',60 from dual union all
select 35077,'1974-W1628','1974-01-23 02:00:00','suborbital rocket','success',761,'MT135P-R145','','','','Meteo',1332,'',60 from dual union all
select 35078,'1974-W1629','1974-01-31 02:00:00','suborbital rocket','success',761,'MT135P-R146','','','','Meteo',1332,'',60 from dual union all
select 35079,'1974-W163','1974-02-04 16:04:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 3508,'1955-A64','1955-02-01 00:00:00','atmospheric rocket','success',836,'2812E','','','','Test',1895,'',10 from dual union all
select 35080,'1974-W1630','1974-02-06 02:05:00','suborbital rocket','success',761,'MT135P-R147','','','','Meteo',1332,'',60 from dual union all
select 35081,'1974-W1631','1974-02-20 02:00:00','suborbital rocket','success',761,'MT135P-R148','','','','Meteo',1332,'',60 from dual union all
select 35082,'1974-W1632','1974-02-27 02:00:00','suborbital rocket','success',761,'MT135P-R149','','','','Meteo',1332,'',60 from dual union all
select 35083,'1974-W1633','1974-03-13 02:04:00','suborbital rocket','success',761,'MT135P-R150','','','','Meteo',1332,'',60 from dual union all
select 35084,'1974-W1634','1974-03-20 02:00:00','suborbital rocket','success',761,'MT135P-R160','','','','Meteo',1332,'',60 from dual union all
select 35085,'1974-W1635','1974-03-27 02:00:00','suborbital rocket','success',761,'MT135P-R151','','','','Meteo',1332,'',60 from dual union all
select 35086,'1974-W1636','1974-04-04 02:00:00','suborbital rocket','success',761,'MT135P-R152','','','','Meteo',1332,'',60 from dual union all
select 35087,'1974-W1637','1974-04-10 02:09:00','suborbital rocket','success',761,'MT135P-R153','','','','Meteo',1332,'',60 from dual union all
select 35088,'1974-W1638','1974-04-17 02:05:00','suborbital rocket','success',761,'MT135P-R154','','','','Meteo',1332,'',60 from dual union all
select 35089,'1974-W1639','1974-04-24 02:00:00','suborbital rocket','success',761,'MT135P-R155','','','','Meteo',1332,'',60 from dual union all
select 3509,'1955-A640','1955-09-20 00:00:00','atmospheric rocket','success',1527,'47','','','','Test',1895,'B-29',10 from dual union all
select 35090,'1974-W164','1974-02-04 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 35091,'1974-W1640','1974-05-01 02:00:00','suborbital rocket','success',761,'MT135P-R156','','','','Meteo',1332,'',60 from dual union all
select 35092,'1974-W1641','1974-05-08 02:00:00','suborbital rocket','success',761,'MT135P-R158','','','','Meteo',1332,'',60 from dual union all
select 35093,'1974-W1642','1974-05-16 02:00:00','suborbital rocket','success',761,'MT135P-R157','','','','Meteo',1332,'',60 from dual union all
select 35094,'1974-W1643','1974-05-22 02:10:00','suborbital rocket','success',761,'MT135P-R159','','','','Meteo',1332,'',60 from dual union all
select 35095,'1974-W1644','1974-06-13 02:30:00','suborbital rocket','success',761,'MT135P-R161','','','','Meteo',1332,'',60 from dual union all
select 35096,'1974-W1645','1974-06-26 02:00:00','suborbital rocket','success',761,'MT135P-R162','','','','Meteo',1332,'',60 from dual union all
select 35097,'1974-W1646','1974-07-17 02:05:00','suborbital rocket','success',761,'MT135P-R163','','','','Meteo',1332,'',60 from dual union all
select 35098,'1974-W1647','1974-07-24 02:20:00','suborbital rocket','success',761,'MT135P-R164','','','','Meteo',1332,'',60 from dual union all
select 35099,'1974-W1648','1974-08-07 02:00:00','suborbital rocket','success',761,'MT135P-R165','','','','Meteo',1332,'',60 from dual union all
select 351,'1944-S32','1944-08-13 18:15:00','military missile','success',1470,'18292','Ma106','','','Training',661,'',90 from dual union all
select 3510,'1955-A641','1955-09-27 00:00:00','atmospheric rocket','success',1527,'1B','','','','Test',1895,'B-29',10 from dual union all
select 35100,'1974-W1649','1974-08-21 02:00:00','suborbital rocket','success',761,'MT135P-R166','','','','Meteo',1332,'',60 from dual union all
select 35101,'1974-W165','1974-02-04 21:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 35102,'1974-W1650','1974-08-28 02:05:00','suborbital rocket','success',761,'MT135P-R167','','','','Meteo',1332,'',60 from dual union all
select 35103,'1974-W1651','1974-09-04 02:10:00','suborbital rocket','success',761,'MT135P-R169','','','','Meteo',1332,'',60 from dual union all
select 35104,'1974-W1652','1974-09-11 02:00:00','suborbital rocket','success',761,'MT135P-R170','','','','Meteo',1332,'',60 from dual union all
select 35105,'1974-W1653','1974-09-18 02:30:00','suborbital rocket','success',761,'MT135P-R168','','','','Meteo',1332,'',60 from dual union all
select 35106,'1974-W1654','1974-10-02 02:00:00','suborbital rocket','success',761,'MT135P-R171','','','','Meteo',1332,'',60 from dual union all
select 35107,'1974-W1655','1974-10-09 02:00:00','suborbital rocket','success',761,'MT135P-R172','','','','Meteo',1332,'',60 from dual;
insert into LAUNCH
select 35108,'1974-W1656','1974-10-16 02:00:00','suborbital rocket','success',761,'MT135P-R173','','','','Meteo',1332,'',60 from dual union all
select 35109,'1974-W1657','1974-10-30 02:00:00','suborbital rocket','success',761,'MT135P-R174','','','','Meteo',1332,'',60 from dual union all
select 3511,'1955-A642','1955-09-30 00:00:00','atmospheric rocket','success',1527,'3','','','','Test',1895,'B-29',10 from dual union all
select 35110,'1974-W1658','1974-11-06 02:00:00','suborbital rocket','success',761,'MT135P-R175','','','','Meteo',1332,'',60 from dual union all
select 35111,'1974-W1659','1974-11-13 02:00:00','suborbital rocket','success',761,'MT135P-R176','','','','Meteo',1332,'',60 from dual union all
select 35112,'1974-W166','1974-02-05 17:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 35113,'1974-W1660','1974-11-20 02:00:00','suborbital rocket','success',761,'MT135P-R177','','','','Meteo',1332,'',60 from dual union all
select 35114,'1974-W1661','1974-11-27 02:02:00','suborbital rocket','success',761,'MT135P-R178','','','','Meteo',1332,'',60 from dual union all
select 35115,'1974-W1662','1974-12-04 02:00:00','suborbital rocket','success',761,'MT135P-R179','','','','Meteo',1332,'',60 from dual union all
select 35116,'1974-W1663','1974-12-11 02:00:00','suborbital rocket','success',761,'MT135P-R180','','','','Meteo',1332,'',60 from dual union all
select 35117,'1974-W1664','1974-12-25 02:00:00','suborbital rocket','success',761,'MT135P-R181','','','','Meteo',1332,'',60 from dual union all
select 35118,'1974-W1665','1974-05-15 16:10:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',71 from dual union all
select 35119,'1974-W1666','1974-05-21 00:00:00','suborbital rocket','success',229,'2-74','STS','','','Test',1895,'',75 from dual union all
select 3512,'1955-A643','1955-10-13 00:00:00','atmospheric rocket','success',1527,'42','','','','Test',1895,'B-29',10 from dual union all
select 35120,'1974-W1667','1974-05-22 00:00:00','suborbital rocket','success',229,'3-74','STS','','','Test',1895,'',74 from dual union all
select 35121,'1974-W1668','1974-05-23 00:00:00','suborbital rocket','success',229,'4-74','STS','','','Test',1895,'',73 from dual union all
select 35122,'1974-W1669','1974-05-23 00:00:00','suborbital rocket','success',229,'5-74','STS','','','Test',1895,'',67 from dual union all
select 35123,'1974-W167','1974-02-05 17:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 35124,'1974-W1670','1974-05-29 00:00:00','suborbital rocket','success',229,'6-74','STS','','','Test',1895,'',71 from dual union all
select 35125,'1974-W1671','1974-05-30 00:00:00','suborbital rocket','success',229,'7-74','STS','','','Test',1895,'',85 from dual union all
select 35126,'1974-W1672','1974-05-31 00:00:00','suborbital rocket','success',229,'8-74','STS','','','Test',1895,'',86 from dual union all
select 35127,'1974-W1673','1974-06-12 00:00:00','suborbital rocket','success',229,'9-74','STS','','','Test',1895,'',67 from dual union all
select 35128,'1974-W1674','1974-06-13 00:00:00','suborbital rocket','success',229,'10-74','STS','','','Test',1895,'',74 from dual union all
select 35129,'1974-W1675','1974-06-18 00:00:00','suborbital rocket','success',229,'11-74','STS','','','Test',1895,'',57 from dual union all
select 3513,'1955-A644','1955-10-18 00:00:00','atmospheric rocket','success',1527,'42TV10/6','','','','Test',1895,'B-29',10 from dual union all
select 35130,'1974-W1676','1974-06-19 00:00:00','suborbital rocket','success',229,'12-74','STS','','','Test',1895,'',69 from dual union all
select 35131,'1974-W1677','1974-08-08 00:00:00','suborbital rocket','success',229,'13-74','STS','','','Test',1895,'',69 from dual union all
select 35132,'1974-W1678','1974-08-09 00:00:00','suborbital rocket','success',229,'14-74','STS','','','Test',1895,'',70 from dual union all
select 35133,'1974-W1679','1974-08-21 00:00:00','suborbital rocket','success',229,'15-74','STS/Starute','','','Test',1895,'',81 from dual union all
select 35134,'1974-W168','1974-02-05 19:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 35135,'1974-W1680','1974-08-21 00:00:00','suborbital rocket','success',229,'16-74','STS/Starute','','','Test',1895,'',75 from dual union all
select 35136,'1974-W1681','1974-08-29 00:00:00','suborbital rocket','success',229,'17-74','STS/Starute','','','Test',1895,'',70 from dual union all
select 35137,'1974-W1682','1974-09-04 00:00:00','suborbital rocket','success',229,'18-74','Ozone/Starute','','','Test',1895,'',72 from dual union all
select 35138,'1974-W1683','1974-10-20 00:00:00','suborbital rocket','success',229,'19-74','Ozone/Starute','','','Test',1895,'',67 from dual union all
select 35139,'1974-W1684','1974-11-15 00:00:00','suborbital rocket','success',229,'20-74','STS/Starute','','','Test',1895,'',70 from dual union all
select 3514,'1955-A645','1955-10-28 00:00:00','atmospheric rocket','success',1527,'11','','','','Test',1895,'B-29',10 from dual union all
select 35140,'1974-W1685','1974-11-15 00:00:00','suborbital rocket','success',229,'21-74','STS/Starute','','','Test',1895,'',70 from dual union all
select 35141,'1974-W1686','1974-11-19 00:00:00','suborbital rocket','success',229,'22-74','STS/Starute','','','Test',1895,'',71 from dual union all
select 35142,'1974-W1687','1974-11-21 00:00:00','suborbital rocket','success',229,'23-74','STS/Starute','','','Test',1895,'',82 from dual union all
select 35143,'1974-W1688','1974-11-21 00:00:00','suborbital rocket','success',229,'24-74','STS/Starute','','','Test',1895,'',82 from dual union all
select 35144,'1974-W1689','1974-11-26 00:00:00','suborbital rocket','success',229,'25-74','STS/Starute','','','Test',1895,'',84 from dual union all
select 35145,'1974-W169','1974-02-06 12:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',65 from dual union all
select 35146,'1974-W1690','1974-12-10 00:00:00','suborbital rocket','success',229,'26-74','STS/Starute','','','Test',1895,'',78 from dual union all
select 35147,'1974-W1691','1974-05-20 00:00:00','suborbital rocket','success',229,'1-74','Quanah User Program','','','Test',1895,'',78 from dual union all
select 35148,'1974-W17','1974-01-04 15:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 35149,'1974-W170','1974-02-06 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 3515,'1955-A646','1955-11-09 00:00:00','atmospheric rocket','success',1527,'GTV-2','','','','Test',1895,'B-29',10 from dual union all
select 35150,'1974-W171','1974-02-06 14:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 35151,'1974-W172','1974-02-06 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 35152,'1974-W173','1974-02-06 16:09:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 35153,'1974-W174','1974-02-06 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 35154,'1974-W175','1974-02-07 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 35155,'1974-W176','1974-02-07 15:45:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 35156,'1974-W177','1974-02-07 15:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 35157,'1974-W178','1974-02-07 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 35158,'1974-W179','1974-02-07 19:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 35159,'1974-W18','1974-01-04 15:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 3516,'1955-A647','1955-11-18 00:00:00','atmospheric rocket','success',1527,'45','','','','Test',1895,'B-29',10 from dual union all
select 35160,'1974-W180','1974-02-08 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 35161,'1974-W181','1974-02-08 15:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 35162,'1974-W182','1974-02-08 18:56:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',68 from dual union all
select 35163,'1974-W183','1974-02-10 06:30:00','suborbital rocket','success',761,'MT135P T10','','','','Aeron?',1476,'',50 from dual union all
select 35164,'1974-W184','1974-02-11 15:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 35165,'1974-W185','1974-02-11 16:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 35166,'1974-W186','1974-02-11 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 35167,'1974-W187','1974-02-12 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 35168,'1974-W188','1974-02-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 35169,'1974-W189','1974-02-12 15:30:00','suborbital rocket','success',1061,'','Starute','','','Meteo',1821,'',64 from dual union all
select 3517,'1955-A648','1955-12-01 00:00:00','atmospheric rocket','success',1527,'GTV-2-3','','','','Test',1895,'B-29',10 from dual union all
select 35170,'1974-W19','1974-01-04 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',76 from dual union all
select 35171,'1974-W190','1974-02-12 15:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 35172,'1974-W191','1974-02-12 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 35173,'1974-W192','1974-02-12 17:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 35174,'1974-W193','1974-02-12 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 35175,'1974-W194','1974-02-13 00:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 35176,'1974-W195','1974-02-13 07:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 35177,'1974-W196','1974-02-13 13:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',76 from dual union all
select 35178,'1974-W197','1974-02-13 13:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 35179,'1974-W198','1974-02-13 14:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 3518,'1955-A649','1955-12-06 00:00:00','atmospheric rocket','success',1527,'46','','','','Test',1895,'B-29',10 from dual union all
select 35180,'1974-W199','1974-02-13 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 35181,'1974-W20','1974-01-04 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 35182,'1974-W200','1974-02-13 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',75 from dual union all
select 35183,'1974-W201','1974-02-14 00:59:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 35184,'1974-W202','1974-02-14 05:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 35185,'1974-W203','1974-02-14 16:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 35186,'1974-W204','1974-02-14 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 35187,'1974-W205','1974-02-15 00:00:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 35188,'1974-W206','1974-02-15 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 35189,'1974-W207','1974-02-15 14:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',73 from dual union all
select 3519,'1955-A65','1955-02-01 00:00:00','atmospheric rocket','success',836,'2866E','','','','Test',1895,'',10 from dual union all
select 35190,'1974-W208','1974-02-15 15:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 35191,'1974-W209','1974-02-15 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 35192,'1974-W21','1974-01-04 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 35193,'1974-W210','1974-02-15 16:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 35194,'1974-W211','1974-02-15 19:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 35195,'1974-W212','1974-02-15 19:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 35196,'1974-W213','1974-02-16 05:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 35197,'1974-W214','1974-02-16 09:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 35198,'1974-W215','1974-02-16 10:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual;
insert into LAUNCH
select 35199,'1974-W216','1974-02-17 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',79 from dual union all
select 352,'1944-S320','1944-09-14 00:00:00','military missile','success',1470,'','London','','','Weapon',1732,'',95 from dual union all
select 3520,'1955-A650','1955-12-08 00:00:00','atmospheric rocket','success',1527,'16','','','','Test',1895,'B-29',10 from dual union all
select 35200,'1974-W217','1974-02-18 05:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 35201,'1974-W218','1974-02-19 00:37:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 35202,'1974-W219','1974-02-19 06:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 35203,'1974-W22','1974-01-04 22:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 35204,'1974-W220','1974-02-19 08:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 35205,'1974-W221','1974-02-19 14:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 35206,'1974-W222','1974-02-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 35207,'1974-W223','1974-02-19 15:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 35208,'1974-W224','1974-02-19 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35209,'1974-W225','1974-02-20 06:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 3521,'1955-A651','1955-12-13 00:00:00','atmospheric rocket','success',1527,'01','','','','Test',1895,'B-29',10 from dual union all
select 35210,'1974-W226','1974-02-20 07:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 35211,'1974-W227','1974-02-20 13:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 35212,'1974-W228','1974-02-20 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',51 from dual union all
select 35213,'1974-W229','1974-02-20 14:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 35214,'1974-W23','1974-01-05 22:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 35215,'1974-W230','1974-02-20 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 35216,'1974-W231','1974-02-20 18:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 35217,'1974-W232','1974-02-21 00:33:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 35218,'1974-W233','1974-02-21 02:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 35219,'1974-W234','1974-02-21 15:21:00','suborbital rocket','success',1284,'G 1-6480','','','','Test',1821,'',69 from dual union all
select 3522,'1955-A652','1955-12-20 00:00:00','atmospheric rocket','success',1527,'47','','','','Test',1895,'B-29',10 from dual union all
select 35220,'1974-W235','1974-02-21 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 35221,'1974-W236','1974-02-21 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 35222,'1974-W237','1974-02-21 19:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 35223,'1974-W238','1974-02-21 22:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 35224,'1974-W239','1974-02-22 13:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 35225,'1974-W24','1974-01-07 13:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 35226,'1974-W240','1974-02-22 15:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35227,'1974-W241','1974-02-23 02:17:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 35228,'1974-W242','1974-02-25 06:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 35229,'1974-W243','1974-02-25 13:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 3523,'1955-A653','1955-12-22 00:00:00','atmospheric rocket','success',1527,'2','','','','Test',1895,'B-29',10 from dual union all
select 35230,'1974-W244','1974-02-25 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 35231,'1974-W245','1974-02-25 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 35232,'1974-W246','1974-02-25 18:00:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',63 from dual union all
select 35233,'1974-W247','1974-02-25 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 35234,'1974-W248','1974-02-25 18:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 35235,'1974-W249','1974-02-26 00:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 35236,'1974-W25','1974-01-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 35237,'1974-W250','1974-02-26 09:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',68 from dual union all
select 35238,'1974-W251','1974-02-26 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35239,'1974-W252','1974-02-26 18:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 3524,'1955-A654','1955-12-23 00:00:00','atmospheric rocket','success',1527,'3','','','','Test',1895,'B-29',10 from dual union all
select 35240,'1974-W253','1974-02-26 18:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 35241,'1974-W254','1974-02-26 18:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 35242,'1974-W255','1974-02-27 00:57:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 35243,'1974-W256','1974-02-27 03:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 35244,'1974-W257','1974-02-27 05:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 35245,'1974-W258','1974-02-27 13:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 35246,'1974-W259','1974-02-27 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',70 from dual union all
select 35247,'1974-W26','1974-01-07 15:03:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 35248,'1974-W260','1974-02-27 15:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 35249,'1974-W261','1974-02-27 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 3525,'1955-A655','1955-01-18 00:00:00','atmospheric rocket','success',1526,'MX-883','','','','Test',1895,'B-29',10 from dual union all
select 35250,'1974-W262','1974-02-27 18:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',53 from dual union all
select 35251,'1974-W263','1974-02-27 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 35252,'1974-W264','1974-02-27 21:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 35253,'1974-W265','1974-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 35254,'1974-W266','1974-03-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 35255,'1974-W267','1974-03-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 35256,'1974-W268','1974-03-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 35257,'1974-W269','1974-03-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 35258,'1974-W27','1974-01-07 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 35259,'1974-W270','1974-02-28 01:02:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 3526,'1955-A656','1955-02-01 00:00:00','atmospheric rocket','success',1526,'MX-883','','','','Test',1895,'B-29',10 from dual union all
select 35260,'1974-W271','1974-02-28 06:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 35261,'1974-W272','1974-02-28 09:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',55 from dual union all
select 35262,'1974-W273','1974-02-28 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',72 from dual union all
select 35263,'1974-W274','1974-03-01 01:08:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 35264,'1974-W275','1974-03-01 12:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',75 from dual union all
select 35265,'1974-W276','1974-03-01 14:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 35266,'1974-W277','1974-03-01 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 35267,'1974-W278','1974-03-01 15:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 35268,'1974-W279','1974-03-01 21:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',75 from dual union all
select 35269,'1974-W28','1974-01-07 19:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 3527,'1955-A657','1955-02-24 00:00:00','atmospheric rocket','success',1526,'MX-883','ST-30 TV14','','','Test',1895,'B-29',10 from dual union all
select 35270,'1974-W280','1974-03-01 21:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 35271,'1974-W281','1974-03-02 01:14:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 35272,'1974-W282','1974-03-02 11:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',65 from dual union all
select 35273,'1974-W283','1974-03-04 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 35274,'1974-W284','1974-03-04 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',76 from dual union all
select 35275,'1974-W285','1974-03-04 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 35276,'1974-W286','1974-03-04 15:34:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',56 from dual union all
select 35277,'1974-W287','1974-03-04 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 35278,'1974-W288','1974-03-04 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 35279,'1974-W289','1974-03-05 00:47:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 3528,'1955-A658','1955-04-29 00:00:00','atmospheric rocket','success',1526,'MX-883','32','','','Test',1895,'B-29',10 from dual union all
select 35280,'1974-W29','1974-01-07 20:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 35281,'1974-W290','1974-03-05 16:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 35282,'1974-W291','1974-03-05 17:46:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 35283,'1974-W292','1974-03-06 12:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 35284,'1974-W293','1974-03-06 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',56 from dual union all
select 35285,'1974-W294','1974-03-06 14:06:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',56 from dual union all
select 35286,'1974-W295','1974-03-06 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',74 from dual union all
select 35287,'1974-W296','1974-03-06 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 35288,'1974-W297','1974-03-06 16:08:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',70 from dual;
insert into LAUNCH
select 35289,'1974-W298','1974-03-06 18:20:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 3529,'1955-A659','1955-06-03 00:00:00','atmospheric rocket','success',1527,'36','','','','Test',1895,'B-29',10 from dual union all
select 35290,'1974-W299','1974-03-06 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 35291,'1974-W30','1974-01-08 17:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 35292,'1974-W300','1974-03-06 19:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 35293,'1974-W301','1974-03-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 35294,'1974-W302','1974-03-07 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 35295,'1974-W303','1974-03-07 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 35296,'1974-W304','1974-03-08 09:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',69 from dual union all
select 35297,'1974-W305','1974-03-08 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',77 from dual union all
select 35298,'1974-W306','1974-03-08 14:47:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',60 from dual union all
select 35299,'1974-W307','1974-03-08 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 353,'1944-S321','1944-09-15 02:04:00','military missile','success',1470,'','London/Sunbury','','','Weapon',1745,'',90 from dual union all
select 3530,'1955-A66','1955-02-02 00:00:00','atmospheric rocket','success',836,'3328T','','','','Test',1895,'',10 from dual union all
select 35300,'1974-W308','1974-03-08 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 35301,'1974-W309','1974-03-08 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',75 from dual union all
select 35302,'1974-W31','1974-01-08 17:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 35303,'1974-W310','1974-03-08 17:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 35304,'1974-W311','1974-03-08 20:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 35305,'1974-W312','1974-03-08 22:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 35306,'1974-W313','1974-03-09 02:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 35307,'1974-W314','1974-03-11 12:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',74 from dual union all
select 35308,'1974-W315','1974-03-11 14:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 35309,'1974-W316','1974-03-11 15:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 3531,'1955-A660','1955-06-21 00:00:00','atmospheric rocket','success',1527,'36','','','','Test',1895,'B-29',10 from dual union all
select 35310,'1974-W317','1974-03-11 16:04:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 35311,'1974-W318','1974-03-11 17:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 35312,'1974-W319','1974-03-11 17:56:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',57 from dual union all
select 35313,'1974-W32','1974-01-09 13:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 35314,'1974-W320','1974-03-12 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 35315,'1974-W321','1974-03-12 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 35316,'1974-W322','1974-03-12 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 35317,'1974-W323','1974-03-13 09:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',70 from dual union all
select 35318,'1974-W324','1974-03-13 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',74 from dual union all
select 35319,'1974-W325','1974-03-13 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 3532,'1955-A661','1955-07-14 00:00:00','atmospheric rocket','success',1527,'TV-13','','','','Test',1895,'B-29',10 from dual union all
select 35320,'1974-W326','1974-03-13 15:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 35321,'1974-W327','1974-03-13 16:17:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 35322,'1974-W328','1974-03-13 16:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 35323,'1974-W329','1974-03-13 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35324,'1974-W33','1974-01-09 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 35325,'1974-W330','1974-03-13 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 35326,'1974-W331','1974-03-13 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 35327,'1974-W332','1974-03-14 04:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',72 from dual union all
select 35328,'1974-W333','1974-03-14 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 35329,'1974-W334','1974-03-14 15:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 3533,'1955-A662','1955-08-11 00:00:00','atmospheric rocket','success',1527,'38','','','','Test',1895,'B-29',10 from dual union all
select 35330,'1974-W335','1974-03-14 16:19:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 35331,'1974-W336','1974-03-14 20:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 35332,'1974-W337','1974-03-14 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 35333,'1974-W338','1974-03-15 07:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 35334,'1974-W339','1974-03-15 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',74 from dual union all
select 35335,'1974-W34','1974-01-09 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',63 from dual union all
select 35336,'1974-W340','1974-03-15 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',59 from dual union all
select 35337,'1974-W341','1974-03-15 14:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 35338,'1974-W342','1974-03-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 35339,'1974-W343','1974-03-15 15:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 3534,'1955-A663','1955-08-25 00:00:00','atmospheric rocket','success',1527,'39','','','','Test',1895,'B-29',10 from dual union all
select 35340,'1974-W344','1974-03-15 15:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 35341,'1974-W345','1974-03-15 20:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 35342,'1974-W346','1974-03-16 01:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 35343,'1974-W347','1974-03-18 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 35344,'1974-W348','1974-03-18 17:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 35345,'1974-W349','1974-03-18 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 35346,'1974-W35','1974-01-09 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 35347,'1974-W350','1974-03-18 21:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 35348,'1974-W351','1974-03-19 00:17:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 35349,'1974-W352','1974-03-19 00:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 3535,'1955-A664','1955-09-16 00:00:00','atmospheric rocket','success',563,'AM-1','HTV Atlas AM-1','','','Heat transfer',1895,'',10 from dual union all
select 35350,'1974-W353','1974-03-19 14:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 35351,'1974-W354','1974-03-19 15:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',350,'',73 from dual union all
select 35352,'1974-W355','1974-03-19 15:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',428,'',66 from dual union all
select 35353,'1974-W356','1974-03-19 15:01:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',35,'',63 from dual union all
select 35354,'1974-W357','1974-03-19 15:40:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',310,'',56 from dual union all
select 35355,'1974-W358','1974-03-19 16:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-10A','','','Aeron',176,'',65 from dual union all
select 35356,'1974-W359','1974-03-19 16:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',330,'',67 from dual union all
select 35357,'1974-W36','1974-01-09 17:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 35358,'1974-W360','1974-03-19 16:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Meteo',1363,'',62 from dual union all
select 35359,'1974-W361','1974-03-19 16:05:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',1821,'',63 from dual union all
select 3536,'1955-A665','1955-09-19 00:00:00','atmospheric rocket','success',563,'HS-2','HTV Atlas HS-1','','','Heat transfer',1895,'',10 from dual union all
select 35360,'1974-W362','1974-03-19 18:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',350,'',75 from dual union all
select 35361,'1974-W363','1974-03-19 18:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',428,'',64 from dual union all
select 35362,'1974-W364','1974-03-19 18:20:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',35,'',69 from dual union all
select 35363,'1974-W365','1974-03-19 18:31:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',310,'',62 from dual union all
select 35364,'1974-W366','1974-03-19 19:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-10A','','','Aeron',176,'',64 from dual union all
select 35365,'1974-W367','1974-03-19 19:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Meteo',1363,'',66 from dual union all
select 35366,'1974-W368','1974-03-19 19:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',330,'',66 from dual union all
select 35367,'1974-W369','1974-03-19 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 35368,'1974-W37','1974-01-09 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 35369,'1974-W370','1974-03-19 19:54:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',1821,'',67 from dual union all
select 3537,'1955-A666','1955-09-21 00:00:00','atmospheric rocket','success',563,'AM-2','HTV Atlas AM-2','','','Heat transfer',1895,'',10 from dual union all
select 35370,'1974-W371','1974-03-19 21:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',350,'',75 from dual union all
select 35371,'1974-W372','1974-03-19 21:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',35,'',73 from dual union all
select 35372,'1974-W373','1974-03-19 21:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',428,'',62 from dual union all
select 35373,'1974-W374','1974-03-19 21:30:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',310,'',60 from dual union all
select 35374,'1974-W375','1974-03-19 22:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Meteo',1363,'',64 from dual union all
select 35375,'1974-W376','1974-03-19 22:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',1821,'',78 from dual union all
select 35376,'1974-W377','1974-03-19 22:18:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',330,'',62 from dual union all
select 35377,'1974-W378','1974-03-19 22:20:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',350,'',75 from dual union all
select 35378,'1974-W379','1974-03-20 00:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',350,'',76 from dual;
insert into LAUNCH
select 35379,'1974-W38','1974-01-09 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 3538,'1955-A667','1955-02-02 00:00:00','military missile','failure',971,'K3-10','','','','Test',636,'',0 from dual union all
select 35380,'1974-W380','1974-03-20 00:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',428,'',62 from dual union all
select 35381,'1974-W381','1974-03-20 00:05:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',1821,'',78 from dual union all
select 35382,'1974-W382','1974-03-20 00:21:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',1821,'',76 from dual union all
select 35383,'1974-W383','1974-03-20 00:30:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',310,'',62 from dual union all
select 35384,'1974-W384','1974-03-20 00:47:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',1821,'',64 from dual union all
select 35385,'1974-W385','1974-03-20 01:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-10A','','','Aeron',176,'',63 from dual union all
select 35386,'1974-W386','1974-03-20 01:00:00','suborbital rocket','success',716,'PWN-8B','DIURNAL PWN-8B','','','Meteo',1363,'',62 from dual union all
select 35387,'1974-W387','1974-03-20 01:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',330,'',62 from dual union all
select 35388,'1974-W388','1974-03-20 01:08:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',1821,'',77 from dual union all
select 35389,'1974-W389','1974-03-20 03:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',350,'',74 from dual union all
select 3539,'1955-A668','1955-03-02 00:00:00','military missile','failure',992,'K3-5D','UGCh','','','OT',636,'',15 from dual union all
select 35390,'1974-W39','1974-01-10 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 35391,'1974-W390','1974-03-20 03:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',35,'',74 from dual union all
select 35392,'1974-W391','1974-03-20 03:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',428,'',64 from dual union all
select 35393,'1974-W392','1974-03-20 03:30:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',310,'',53 from dual union all
select 35394,'1974-W393','1974-03-20 04:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Meteo',1363,'',64 from dual union all
select 35395,'1974-W394','1974-03-20 04:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-10A','','','Aeron',176,'',66 from dual union all
select 35396,'1974-W395','1974-03-20 04:53:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',1821,'',62 from dual union all
select 35397,'1974-W396','1974-03-20 06:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',350,'',75 from dual union all
select 35398,'1974-W397','1974-03-20 06:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',428,'',62 from dual union all
select 35399,'1974-W398','1974-03-20 06:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',35,'',72 from dual union all
select 354,'1944-S322','1944-09-15 00:00:00','military missile','success',1470,'','London/All Hallows','','','Weapon',1745,'',90 from dual union all
select 3540,'1955-A669','1955-07-26 00:00:00','military missile','failure',992,'S0-4','UGCh','','','Test',636,'',15 from dual union all
select 35400,'1974-W399','1974-03-20 06:30:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',310,'',63 from dual union all
select 35401,'1974-W40','1974-01-10 15:53:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 35402,'1974-W400','1974-03-20 07:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Meteo',1363,'',61 from dual union all
select 35403,'1974-W401','1974-03-20 07:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-10A','','','Aeron',176,'',69 from dual union all
select 35404,'1974-W402','1974-03-20 07:06:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',1821,'',63 from dual union all
select 35405,'1974-W403','1974-03-20 07:48:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',330,'',64 from dual union all
select 35406,'1974-W404','1974-03-20 08:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 35407,'1974-W405','1974-03-20 09:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',428,'',65 from dual union all
select 35408,'1974-W406','1974-03-20 09:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',350,'',67 from dual union all
select 35409,'1974-W407','1974-03-20 09:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',35,'',72 from dual union all
select 3541,'1955-A67','1955-02-03 00:00:00','military missile','unknown',844,'B2 1810-2R','Test','','','Test',1895,'',10 from dual union all
select 35410,'1974-W408','1974-03-20 09:31:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',310,'',62 from dual union all
select 35411,'1974-W409','1974-03-20 10:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-10A','','','Aeron',176,'',65 from dual union all
select 35412,'1974-W41','1974-01-10 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 35413,'1974-W410','1974-03-20 10:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',330,'',65 from dual union all
select 35414,'1974-W411','1974-03-20 10:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',1821,'',59 from dual union all
select 35415,'1974-W412','1974-03-20 10:15:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Meteo',1363,'',55 from dual union all
select 35416,'1974-W413','1974-03-20 10:40:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',1821,'',62 from dual union all
select 35417,'1974-W414','1974-03-20 12:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',350,'',75 from dual union all
select 35418,'1974-W415','1974-03-20 12:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',428,'',62 from dual union all
select 35419,'1974-W416','1974-03-20 12:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',35,'',74 from dual union all
select 3542,'1955-A670','1955-04-07 00:00:00','atmospheric rocket','success',1526,'MX-883','ST-31','','','Test',1895,'B-29',10 from dual union all
select 35420,'1974-W417','1974-03-20 12:40:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',1821,'',63 from dual union all
select 35421,'1974-W418','1974-03-20 13:03:00','suborbital rocket','success',1284,'','DIURNAL PWN-10A','','','Aeron',176,'',64 from dual union all
select 35422,'1974-W419','1974-03-20 13:15:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Meteo',1363,'',67 from dual union all
select 35423,'1974-W42','1974-01-10 18:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 35424,'1974-W420','1974-03-20 13:21:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',330,'',66 from dual union all
select 35425,'1974-W421','1974-03-20 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 35426,'1974-W422','1974-03-20 15:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',350,'',77 from dual union all
select 35427,'1974-W423','1974-03-20 15:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',35,'',63 from dual union all
select 35428,'1974-W424','1974-03-20 16:00:00','suborbital rocket','success',1284,'','DIURNAL PWN-10A','','','Aeron',176,'',63 from dual union all
select 35429,'1974-W425','1974-03-20 16:00:00','suborbital rocket','success',716,'','DIURNAL PWN-8B','','','Aeron',330,'',60 from dual union all
select 3543,'1955-A68','1955-02-03 00:00:00','atmospheric rocket','success',836,'1483T','','','','Test',1895,'',10 from dual union all
select 35430,'1974-W426','1974-03-20 16:39:00','suborbital rocket','success',1284,'','DIURNAL PWN-11A','','','Aeron',1821,'',61 from dual union all
select 35431,'1974-W427','1974-03-20 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 35432,'1974-W428','1974-03-20 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 35433,'1974-W429','1974-03-21 01:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 35434,'1974-W43','1974-01-10 22:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',73 from dual union all
select 35435,'1974-W430','1974-03-21 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 35436,'1974-W431','1974-03-21 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 35437,'1974-W432','1974-03-22 03:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 35438,'1974-W433','1974-03-22 13:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 35439,'1974-W434','1974-03-22 13:58:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 3544,'1955-A69','1955-02-03 00:00:00','atmospheric rocket','success',836,'1887T','','','','Test',1895,'',10 from dual union all
select 35440,'1974-W435','1974-03-22 16:56:00','suborbital rocket','failure',229,'AAF-VI-02','Dust collector','','','Aeron',428,'',78 from dual union all
select 35441,'1974-W436','1974-03-22 17:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',53 from dual union all
select 35442,'1974-W437','1974-03-22 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',74 from dual union all
select 35443,'1974-W438','1974-03-22 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 35444,'1974-W439','1974-03-23 04:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 35445,'1974-W44','1974-01-11 00:00:00','suborbital rocket','success',88,'PSU Gerdien','Arcas Starute 10','','','Ionos',1895,'',70 from dual union all
select 35446,'1974-W440','1974-03-23 09:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',68 from dual union all
select 35447,'1974-W441','1974-03-24 02:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 35448,'1974-W442','1974-03-25 13:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',62 from dual union all
select 35449,'1974-W443','1974-03-25 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',55 from dual union all
select 3545,'1955-A70','1955-02-04 00:00:00','atmospheric rocket','success',836,'1488T','','','','Test',1895,'',10 from dual union all
select 35450,'1974-W444','1974-03-25 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 35451,'1974-W445','1974-03-25 15:07:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 35452,'1974-W446','1974-03-25 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 35453,'1974-W447','1974-03-25 20:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 35454,'1974-W448','1974-03-25 21:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35455,'1974-W449','1974-03-26 14:43:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 35456,'1974-W45','1974-01-11 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 35457,'1974-W450','1974-03-27 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',66 from dual union all
select 35458,'1974-W451','1974-03-27 13:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 35459,'1974-W452','1974-03-27 14:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 3546,'1955-A71','1955-02-04 00:00:00','atmospheric rocket','success',836,'1543T','','','','Test',1895,'',10 from dual union all
select 35460,'1974-W453','1974-03-27 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',59 from dual union all
select 35461,'1974-W454','1974-03-27 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 35462,'1974-W455','1974-03-27 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',71 from dual union all
select 35463,'1974-W456','1974-03-28 00:00:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 35464,'1974-W457','1974-03-28 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 35465,'1974-W458','1974-03-28 14:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 35466,'1974-W459','1974-03-28 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',60 from dual union all
select 35467,'1974-W46','1974-01-11 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 35468,'1974-W460','1974-03-28 17:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual;
insert into LAUNCH
select 35469,'1974-W461','1974-03-29 00:54:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 3547,'1955-A72','1955-02-04 00:00:00','atmospheric rocket','success',836,'3640T','','','','Test',1895,'',10 from dual union all
select 35470,'1974-W462','1974-03-29 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual union all
select 35471,'1974-W463','1974-03-29 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 35472,'1974-W464','1974-03-29 16:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 35473,'1974-W465','1974-03-29 17:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 35474,'1974-W466','1974-03-29 21:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 35475,'1974-W467','1974-03-30 01:36:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 35476,'1974-W468','1974-04-01 13:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual union all
select 35477,'1974-W469','1974-04-01 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',78 from dual union all
select 35478,'1974-W47','1974-01-11 14:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',56 from dual union all
select 35479,'1974-W470','1974-04-01 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 3548,'1955-A73','1955-02-04 00:00:00','atmospheric rocket','success',836,'1822T','','','','Test',1895,'',10 from dual union all
select 35480,'1974-W471','1974-04-01 17:32:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 35481,'1974-W472','1974-04-01 17:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 35482,'1974-W473','1974-04-01 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 35483,'1974-W474','1974-04-02 00:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 35484,'1974-W475','1974-04-02 18:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 35485,'1974-W476','1974-04-03 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 35486,'1974-W477','1974-04-03 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 35487,'1974-W478','1974-04-03 17:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 35488,'1974-W479','1974-04-03 19:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 35489,'1974-W48','1974-01-11 14:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 3549,'1955-A74','1955-02-05 00:00:00','atmospheric rocket','success',836,'1833T','','','','Test',1895,'',10 from dual union all
select 35490,'1974-W480','1974-04-03 20:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',66 from dual union all
select 35491,'1974-W481','1974-04-03 21:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 35492,'1974-W482','1974-04-03 22:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 35493,'1974-W483','1974-04-04 13:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 35494,'1974-W484','1974-04-04 18:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 35495,'1974-W485','1974-04-04 20:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 35496,'1974-W486','1974-04-04 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 35497,'1974-W487','1974-04-05 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',54 from dual union all
select 35498,'1974-W488','1974-04-05 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 35499,'1974-W489','1974-04-05 17:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 355,'1944-S323','1944-10-22 00:00:00','military missile','success',1470,'','Brussels/Deurne','','','Weapon',1627,'',90 from dual union all
select 3550,'1955-A75','1955-02-06 00:00:00','atmospheric rocket','success',836,'1854T','','','','Test',1895,'',10 from dual union all
select 35500,'1974-W49','1974-01-11 15:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 35501,'1974-W490','1974-04-05 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 35502,'1974-W491','1974-04-05 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 35503,'1974-W492','1974-04-06 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 35504,'1974-W493','1974-04-06 01:36:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 35505,'1974-W494','1974-04-08 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 35506,'1974-W495','1974-04-08 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 35507,'1974-W496','1974-04-09 01:34:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 35508,'1974-W497','1974-04-09 06:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 35509,'1974-W498','1974-04-09 13:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 3551,'1955-A76','1955-02-06 00:00:00','atmospheric rocket','success',836,'1506T','','','','Test',1895,'',10 from dual union all
select 35510,'1974-W499','1974-04-09 20:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 35511,'1974-W50','1974-01-11 18:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 35512,'1974-W500','1974-04-10 12:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',76 from dual union all
select 35513,'1974-W501','1974-04-10 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 35514,'1974-W502','1974-04-10 14:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 35515,'1974-W503','1974-04-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 35516,'1974-W504','1974-04-10 18:09:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 35517,'1974-W505','1974-04-10 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 35518,'1974-W506','1974-04-11 06:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 35519,'1974-W507','1974-04-11 14:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 3552,'1955-A77','1955-02-06 00:00:00','atmospheric rocket','success',836,'1933T','','','','Test',1895,'',10 from dual union all
select 35520,'1974-W508','1974-04-11 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 35521,'1974-W509','1974-04-12 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 35522,'1974-W51','1974-01-11 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35523,'1974-W510','1974-04-12 17:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 35524,'1974-W511','1974-04-12 18:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 35525,'1974-W512','1974-04-12 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 35526,'1974-W513','1974-04-13 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 35527,'1974-W514','1974-04-13 01:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',53 from dual union all
select 35528,'1974-W515','1974-04-15 09:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 35529,'1974-W516','1974-04-15 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',73 from dual union all
select 3553,'1955-A78','1955-02-07 00:00:00','atmospheric rocket','success',836,'1823T','','','','Test',1895,'',10 from dual union all
select 35530,'1974-W517','1974-04-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 35531,'1974-W518','1974-04-15 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 35532,'1974-W519','1974-04-15 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 35533,'1974-W52','1974-01-11 19:33:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',74 from dual union all
select 35534,'1974-W520','1974-04-15 18:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',73 from dual union all
select 35535,'1974-W521','1974-04-15 20:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 35536,'1974-W522','1974-04-16 04:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 35537,'1974-W523','1974-04-16 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 35538,'1974-W524','1974-04-16 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 35539,'1974-W525','1974-04-16 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 3554,'1955-A79','1955-02-07 00:00:00','atmospheric rocket','success',836,'1537T','','','','Test',1895,'',10 from dual union all
select 35540,'1974-W526','1974-04-16 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 35541,'1974-W527','1974-04-17 06:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 35542,'1974-W528','1974-04-17 13:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',55 from dual union all
select 35543,'1974-W529','1974-04-17 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 35544,'1974-W53','1974-01-11 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 35545,'1974-W530','1974-04-17 15:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 35546,'1974-W531','1974-04-17 17:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 35547,'1974-W532','1974-04-17 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 35548,'1974-W533','1974-04-17 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 35549,'1974-W534','1974-04-17 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 3555,'1955-A80','1955-02-08 00:00:00','atmospheric rocket','unknown',576,'960PO','','','','Test',1895,'',10 from dual union all
select 35550,'1974-W535','1974-04-18 01:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 35551,'1974-W536','1974-04-18 12:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',71 from dual union all
select 35552,'1974-W537','1974-04-18 20:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 35553,'1974-W538','1974-04-19 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 35554,'1974-W539','1974-04-19 13:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 35555,'1974-W54','1974-01-12 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 35556,'1974-W540','1974-04-19 14:07:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 35557,'1974-W541','1974-04-19 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 35558,'1974-W542','1974-04-19 17:00:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',62 from dual;
insert into LAUNCH
select 35559,'1974-W543','1974-04-19 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 3556,'1955-A81','1955-02-08 00:00:00','atmospheric rocket','unknown',576,'923RM','','','','Test',1895,'',10 from dual union all
select 35560,'1974-W544','1974-04-19 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 35561,'1974-W545','1974-04-20 01:07:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 35562,'1974-W546','1974-04-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 35563,'1974-W547','1974-04-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 35564,'1974-W548','1974-04-22 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',72 from dual union all
select 35565,'1974-W549','1974-04-22 16:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',56 from dual union all
select 35566,'1974-W55','1974-01-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',67 from dual union all
select 35567,'1974-W550','1974-04-22 16:40:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',65 from dual union all
select 35568,'1974-W551','1974-04-22 17:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 35569,'1974-W552','1974-04-22 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 3557,'1955-A82','1955-02-08 00:00:00','atmospheric rocket','unknown',576,'924RM','','','','Test',1895,'',10 from dual union all
select 35570,'1974-W553','1974-04-22 21:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 35571,'1974-W554','1974-04-23 00:41:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 35572,'1974-W555','1974-04-23 04:31:00','suborbital rocket','success',229,'AKF-VI-03','Auroral X-rays','','','Auroral',428,'',76 from dual union all
select 35573,'1974-W556','1974-04-23 08:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 35574,'1974-W557','1974-04-24 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 35575,'1974-W558','1974-04-24 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 35576,'1974-W559','1974-04-24 16:15:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 35577,'1974-W56','1974-01-14 15:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 35578,'1974-W560','1974-04-24 17:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 35579,'1974-W561','1974-04-24 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 3558,'1955-A83','1955-02-08 00:00:00','atmospheric rocket','success',836,'1945T','','','','Test',1895,'',10 from dual union all
select 35580,'1974-W562','1974-04-24 18:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 35581,'1974-W563','1974-04-24 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 35582,'1974-W564','1974-04-24 21:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 35583,'1974-W565','1974-04-25 06:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 35584,'1974-W566','1974-04-25 09:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',70 from dual union all
select 35585,'1974-W567','1974-04-25 17:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',70 from dual union all
select 35586,'1974-W568','1974-04-25 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 35587,'1974-W569','1974-04-26 04:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 35588,'1974-W57','1974-01-14 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 35589,'1974-W570','1974-04-26 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',76 from dual union all
select 3559,'1955-A84','1955-02-08 00:00:00','atmospheric rocket','success',836,'2932E','','','','Test',1895,'',10 from dual union all
select 35590,'1974-W571','1974-04-26 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',56 from dual union all
select 35591,'1974-W572','1974-04-26 14:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 35592,'1974-W573','1974-04-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 35593,'1974-W574','1974-04-26 15:55:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 35594,'1974-W575','1974-04-26 16:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 35595,'1974-W576','1974-04-26 16:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 35596,'1974-W577','1974-04-26 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 35597,'1974-W578','1974-04-27 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 35598,'1974-W579','1974-04-28 12:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35599,'1974-W58','1974-01-14 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 356,'1944-S324','1944-10-02 00:00:00','military missile','success',1470,'','Test (post bombing)','','','Test',677,'',90 from dual union all
select 3560,'1955-A85','1955-02-09 00:00:00','atmospheric rocket','success',836,'1953T','','','','Test',1895,'',10 from dual union all
select 35600,'1974-W580','1974-04-29 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',55 from dual union all
select 35601,'1974-W581','1974-04-29 13:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 35602,'1974-W582','1974-04-29 14:42:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 35603,'1974-W583','1974-04-29 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 35604,'1974-W584','1974-04-29 17:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 35605,'1974-W585','1974-04-29 17:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 35606,'1974-W586','1974-04-29 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 35607,'1974-W587','1974-05-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 35608,'1974-W588','1974-05-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 35609,'1974-W589','1974-05-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 3561,'1955-A86','1955-02-09 00:00:00','atmospheric rocket','success',836,'3098R','Nike 419R','','','Test',1895,'',10 from dual union all
select 35610,'1974-W59','1974-01-14 18:42:00','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',58 from dual union all
select 35611,'1974-W590','1974-05-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 35612,'1974-W591','1974-04-30 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 35613,'1974-W592','1974-04-30 19:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 35614,'1974-W593','1974-05-01 12:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 35615,'1974-W594','1974-05-01 13:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 35616,'1974-W595','1974-05-01 14:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 35617,'1974-W596','1974-05-01 14:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 35618,'1974-W597','1974-05-01 15:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 35619,'1974-W598','1974-05-01 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 3562,'1955-A87','1955-02-09 20:15:00','military missile','failure',1053,'RS-8     XL','','','','Test',272,'',0 from dual union all
select 35620,'1974-W599','1974-05-01 15:30:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 35621,'1974-W60','1974-01-14 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 35622,'1974-W600','1974-05-01 16:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 35623,'1974-W601','1974-05-01 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 35624,'1974-W602','1974-05-01 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 35625,'1974-W603','1974-05-02 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1363,'',57 from dual union all
select 35626,'1974-W604','1974-05-02 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35627,'1974-W605','1974-05-03 12:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 35628,'1974-W606','1974-05-03 14:45:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 35629,'1974-W607','1974-05-03 15:45:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',68 from dual union all
select 3563,'1955-A88','1955-02-09 21:06:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 35630,'1974-W608','1974-05-03 17:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 35631,'1974-W609','1974-05-03 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 35632,'1974-W61','1974-01-15 07:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',69 from dual union all
select 35633,'1974-W610','1974-05-03 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 35634,'1974-W611','1974-05-03 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 35635,'1974-W612','1974-05-04 06:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 35636,'1974-W613','1974-05-04 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35637,'1974-W614','1974-05-05 08:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 35638,'1974-W615','1974-05-05 08:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 35639,'1974-W616','1974-05-05 09:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 3564,'1955-A89','1955-02-09 21:19:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 35640,'1974-W617','1974-05-05 09:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',60 from dual union all
select 35641,'1974-W618','1974-05-05 11:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 35642,'1974-W619','1974-05-05 11:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',72 from dual union all
select 35643,'1974-W62','1974-01-15 15:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',68 from dual union all
select 35644,'1974-W620','1974-05-05 12:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 35645,'1974-W621','1974-05-05 12:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',60 from dual union all
select 35646,'1974-W622','1974-05-05 14:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 35647,'1974-W623','1974-05-05 15:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 35648,'1974-W624','1974-05-05 15:59:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',58 from dual;
insert into LAUNCH
select 35649,'1974-W625','1974-05-05 17:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',59 from dual union all
select 3565,'1955-A90','1955-02-10 00:00:00','atmospheric rocket','success',836,'1502T','','','','Test',1895,'',10 from dual union all
select 35650,'1974-W626','1974-05-05 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 35651,'1974-W627','1974-05-05 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 35652,'1974-W628','1974-05-05 18:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 35653,'1974-W629','1974-05-05 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35654,'1974-W63','1974-01-15 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 35655,'1974-W630','1974-05-05 20:00:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',52 from dual union all
select 35656,'1974-W631','1974-05-06 14:42:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 35657,'1974-W632','1974-05-06 14:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 35658,'1974-W633','1974-05-06 18:46:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',58 from dual union all
select 35659,'1974-W634','1974-05-06 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 3566,'1955-A91','1955-02-10 00:00:00','military missile','success',1214,'N-69C GM-13106','','','','Test',238,'',10 from dual union all
select 35660,'1974-W635','1974-05-07 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 35661,'1974-W636','1974-05-07 16:34:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 35662,'1974-W637','1974-05-07 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35663,'1974-W638','1974-05-07 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 35664,'1974-W639','1974-05-08 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 35665,'1974-W64','1974-01-16 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',69 from dual union all
select 35666,'1974-W640','1974-05-08 15:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 35667,'1974-W641','1974-05-08 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 35668,'1974-W642','1974-05-08 16:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 35669,'1974-W643','1974-05-08 19:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 3567,'1955-A92','1955-02-11 00:00:00','atmospheric rocket','unknown',576,'289-ES','','','','Test',1895,'',10 from dual union all
select 35670,'1974-W644','1974-05-09 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 35671,'1974-W645','1974-05-09 15:49:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 35672,'1974-W646','1974-05-09 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 35673,'1974-W647','1974-05-09 16:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 35674,'1974-W648','1974-05-10 05:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 35675,'1974-W649','1974-05-10 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 35676,'1974-W65','1974-01-16 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 35677,'1974-W650','1974-05-10 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 35678,'1974-W651','1974-05-10 18:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 35679,'1974-W652','1974-05-10 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 3568,'1955-A93','1955-02-11 00:00:00','atmospheric rocket','unknown',576,'126-ES','','','','Test',1895,'',10 from dual union all
select 35680,'1974-W653','1974-05-10 19:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',54 from dual union all
select 35681,'1974-W654','1974-05-10 21:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 35682,'1974-W655','1974-05-13 16:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 35683,'1974-W656','1974-05-13 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',71 from dual union all
select 35684,'1974-W657','1974-05-13 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 35685,'1974-W658','1974-05-13 17:49:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',64 from dual union all
select 35686,'1974-W659','1974-05-13 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 35687,'1974-W66','1974-01-16 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',61 from dual union all
select 35688,'1974-W660','1974-05-14 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 35689,'1974-W661','1974-05-14 16:34:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 3569,'1955-A94','1955-02-11 00:00:00','atmospheric rocket','success',836,'1942T','','','','Test',1895,'',10 from dual union all
select 35690,'1974-W662','1974-05-15 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 35691,'1974-W663','1974-05-15 14:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 35692,'1974-W664','1974-05-15 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',77 from dual union all
select 35693,'1974-W665','1974-05-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 35694,'1974-W666','1974-05-15 15:14:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 35695,'1974-W667','1974-05-15 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 35696,'1974-W669','1974-05-15 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 35697,'1974-W67','1974-01-16 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 35698,'1974-W670','1974-05-15 21:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 35699,'1974-W671','1974-05-16 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 357,'1944-S325','1944-10-03 00:00:00','military missile','success',1470,'','London','','','Weapon',1734,'',90 from dual union all
select 3570,'1955-A95','1955-02-12 00:00:00','atmospheric rocket','success',836,'1739T','','','','Test',1895,'',10 from dual union all
select 35700,'1974-W672','1974-05-16 17:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 35701,'1974-W673','1974-05-16 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 35702,'1974-W674','1974-05-17 13:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 35703,'1974-W675','1974-05-17 13:58:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',64 from dual union all
select 35704,'1974-W676','1974-05-17 16:09:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 35705,'1974-W677','1974-05-17 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 35706,'1974-W678','1974-05-17 18:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',71 from dual union all
select 35707,'1974-W679','1974-05-17 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 35708,'1974-W68','1974-01-16 17:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 35709,'1974-W680','1974-05-18 07:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 3571,'1955-A96','1955-02-13 00:00:00','atmospheric rocket','success',836,'1892T','','','','Test',1895,'',10 from dual union all
select 35710,'1974-W681','1974-05-20 15:59:59','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',75 from dual union all
select 35711,'1974-W682','1974-05-20 17:56:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',55 from dual union all
select 35712,'1974-W683','1974-05-20 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 35713,'1974-W684','1974-05-20 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',50 from dual union all
select 35714,'1974-W685','1974-05-21 16:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 35715,'1974-W686','1974-05-21 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 35716,'1974-W687','1974-05-21 18:00:00','suborbital rocket','success',1061,'','Met Para,STS','','','Meteo',1895,'',70 from dual union all
select 35717,'1974-W688','1974-05-21 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 35718,'1974-W689','1974-05-22 15:17:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 35719,'1974-W69','1974-01-16 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 3572,'1955-A97','1955-02-13 00:00:00','atmospheric rocket','success',836,'1618T','','','','Test',1895,'',10 from dual union all
select 35720,'1974-W690','1974-05-22 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 35721,'1974-W691','1974-05-22 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 35722,'1974-W692','1974-05-22 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 35723,'1974-W693','1974-05-22 18:14:59','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',65 from dual union all
select 35724,'1974-W694','1974-05-22 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 35725,'1974-W695','1974-05-22 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 35726,'1974-W696','1974-05-23 01:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 35727,'1974-W697','1974-05-23 10:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 35728,'1974-W698','1974-05-23 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 35729,'1974-W699','1974-05-23 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 3573,'1955-A98','1955-02-14 00:00:00','atmospheric rocket','success',836,'1578T','','','','Test',1895,'',10 from dual union all
select 35730,'1974-W70','1974-01-16 18:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 35731,'1974-W700','1974-05-24 07:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 35732,'1974-W701','1974-05-24 16:14:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 35733,'1974-W702','1974-05-24 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35734,'1974-W703','1974-05-24 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 35735,'1974-W704','1974-05-27 14:38:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 35736,'1974-W705','1974-05-27 15:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 35737,'1974-W706','1974-05-28 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 35738,'1974-W707','1974-05-28 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual;
insert into LAUNCH
select 35739,'1974-W708','1974-05-28 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 3574,'1955-A99','1955-02-14 00:00:00','atmospheric rocket','success',836,'1795T','','','','Test',1895,'',10 from dual union all
select 35740,'1974-W709','1974-05-28 21:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 35741,'1974-W71','1974-01-16 19:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 35742,'1974-W710','1974-05-29 00:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 35743,'1974-W711','1974-05-29 14:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 35744,'1974-W712','1974-05-29 15:18:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 35745,'1974-W713','1974-05-29 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 35746,'1974-W714','1974-05-29 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 35747,'1974-W715','1974-05-29 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35748,'1974-W716','1974-05-30 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 35749,'1974-W717','1974-05-30 10:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 3575,'1955-M01','1955-02-10 22:38:00','suborbital rocket','success',51,'USAF 53','AF/Utah D-layer','','','Ionos',640,'',76 from dual union all
select 35750,'1974-W718','1974-05-30 15:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',51 from dual union all
select 35751,'1974-W719','1974-05-30 16:30:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',72 from dual union all
select 35752,'1974-W72','1974-01-16 20:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 35753,'1974-W720','1974-05-30 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 35754,'1974-W721','1974-06-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 35755,'1974-W722','1974-06-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 35756,'1974-W723','1974-06-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 35757,'1974-W724','1974-06-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 35758,'1974-W725','1974-05-31 07:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 35759,'1974-W726','1974-05-31 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 3576,'1955-M02','1955-06-23 12:40:00','suborbital rocket','failure',49,'USAF 57','AJ11-6 test','','','Test',640,'',64 from dual union all
select 35760,'1974-W727','1974-05-31 15:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 35761,'1974-W728','1974-05-31 15:40:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',74 from dual union all
select 35762,'1974-W729','1974-05-31 16:14:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 35763,'1974-W73','1974-01-17 15:55:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 35764,'1974-W730','1974-05-31 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 35765,'1974-W731','1974-05-31 20:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 35766,'1974-W732','1974-05-31 21:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 35767,'1974-W733','1974-06-03 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',70 from dual union all
select 35768,'1974-W734','1974-06-03 15:11:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',59 from dual union all
select 35769,'1974-W735','1974-06-03 15:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 3577,'1955-M03','1955-07-13 06:59:00','suborbital rocket','success',55,'NRL 27','Mass spectrometers','','','Aeronomy',1869,'',69 from dual union all
select 35770,'1974-W736','1974-06-03 15:59:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 35771,'1974-W737','1974-06-03 16:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 35772,'1974-W738','1974-06-03 16:14:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',72 from dual union all
select 35773,'1974-W739','1974-06-04 16:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 35774,'1974-W74','1974-01-17 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',76 from dual union all
select 35775,'1974-W740','1974-06-04 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 35776,'1974-W741','1974-06-05 07:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 35777,'1974-W742','1974-06-05 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',69 from dual union all
select 35778,'1974-W743','1974-06-05 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',70 from dual union all
select 35779,'1974-W744','1974-06-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 3578,'1955-M04','1955-08-16 00:00:00','suborbital rocket','success',1059,'','','','','Test',986,'F2H2',55 from dual union all
select 35780,'1974-W745','1974-06-05 15:22:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 35781,'1974-W746','1974-06-05 16:09:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',50 from dual union all
select 35782,'1974-W747','1974-06-05 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 35783,'1974-W748','1974-06-05 18:40:00','suborbital rocket','success',1061,'','Met Para,STS-','','','Meteo',1895,'',57 from dual union all
select 35784,'1974-W749','1974-06-05 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 35785,'1974-W75','1974-01-17 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',75 from dual union all
select 35786,'1974-W750','1974-06-05 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 35787,'1974-W751','1974-06-06 00:00:00','suborbital rocket','success',779,'','','','','Aeron',933,'',65 from dual union all
select 35788,'1974-W752','1974-06-06 02:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 35789,'1974-W753','1974-06-06 13:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 3579,'1955-M05','1955-08-16 00:00:00','suborbital rocket','failure',1059,'','','','','Test',986,'F2H2',50 from dual union all
select 35790,'1974-W754','1974-06-06 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 35791,'1974-W755','1974-06-06 15:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',55 from dual union all
select 35792,'1974-W756','1974-06-06 18:00:00','suborbital rocket','success',1061,'','Met Para,STS-','','','Meteo',1895,'',62 from dual union all
select 35793,'1974-W757','1974-06-06 20:00:00','suborbital rocket','success',1061,'','Met Para,STS-','','','Meteo',1895,'',57 from dual union all
select 35794,'1974-W758','1974-06-07 12:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 35795,'1974-W759','1974-06-07 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 35796,'1974-W76','1974-01-17 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 35797,'1974-W760','1974-06-07 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual union all
select 35798,'1974-W761','1974-06-07 16:19:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 35799,'1974-W762','1974-06-07 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 358,'1944-S326','1944-10-03 00:00:00','military missile','success',1470,'','London','','','Weapon',1734,'',90 from dual union all
select 3580,'1955-M06','1955-09-30 15:50:00','suborbital rocket','success',47,'USAF 59','Colorado UVS','','','Solar UV',640,'',74 from dual union all
select 35800,'1974-W763','1974-06-07 19:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 35801,'1974-W764','1974-06-07 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 35802,'1974-W765','1974-06-07 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 35803,'1974-W766','1974-06-08 13:44:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',74 from dual union all
select 35804,'1974-W767','1974-06-10 12:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',67 from dual union all
select 35805,'1974-W768','1974-06-10 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 35806,'1974-W769','1974-06-10 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',67 from dual union all
select 35807,'1974-W77','1974-01-18 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 35808,'1974-W770','1974-06-10 16:19:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',60 from dual union all
select 35809,'1974-W771','1974-06-10 16:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 3581,'1955-M07','1955-10-13 06:37:00','suborbital rocket','success',370,'','NRL Rockoon 19','','','Ionos/Aeron?',98,'AGB-5',60 from dual union all
select 35810,'1974-W772','1974-06-10 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 35811,'1974-W773','1974-06-10 17:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 35812,'1974-W774','1974-06-10 19:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 35813,'1974-W775','1974-06-10 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 35814,'1974-W776','1974-06-11 11:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 35815,'1974-W777','1974-06-11 16:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 35816,'1974-W778','1974-06-12 12:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',60 from dual union all
select 35817,'1974-W779','1974-06-12 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',64 from dual union all
select 35818,'1974-W78','1974-01-18 14:04:59','suborbital rocket','success',1061,'','Met Para,Arcasonde','','','Meteo',1821,'',54 from dual union all
select 35819,'1974-W780','1974-06-12 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 3582,'1955-M08','1955-10-13 13:42:00','suborbital rocket','failure',370,'','NRL Rockoon 20','','','Ionos/Aeron?',99,'AGB-5',50 from dual union all
select 35820,'1974-W781','1974-06-12 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 35821,'1974-W782','1974-06-12 15:19:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 35822,'1974-W783','1974-06-12 17:40:00','suborbital rocket','success',1506,'E 1-7345','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 35823,'1974-W784','1974-06-12 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 35824,'1974-W785','1974-06-12 20:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 35825,'1974-W786','1974-06-12 21:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 35826,'1974-W787','1974-06-13 03:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 35827,'1974-W788','1974-06-13 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',76 from dual union all
select 35828,'1974-W789','1974-06-13 17:00:00','suborbital rocket','success',1061,'','Met Para,STS-','','','Meteo',1895,'',62 from dual;
insert into LAUNCH
select 35829,'1974-W79','1974-01-18 15:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',75 from dual union all
select 3583,'1955-M09','1955-11-01 00:00:00','suborbital rocket','failure',1059,'','','','','Test',986,'F2H2',50 from dual union all
select 35830,'1974-W790','1974-06-14 12:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 35831,'1974-W791','1974-06-14 14:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 35832,'1974-W792','1974-06-14 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 35833,'1974-W793','1974-06-14 16:04:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',51 from dual union all
select 35834,'1974-W794','1974-06-14 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',64 from dual union all
select 35835,'1974-W795','1974-06-14 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 35836,'1974-W796','1974-06-17 16:40:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',70 from dual union all
select 35837,'1974-W797','1974-06-17 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 35838,'1974-W798','1974-06-17 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 35839,'1974-W799','1974-06-17 21:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 3584,'1955-M10','1955-11-01 00:00:00','suborbital rocket','failure',1059,'','','','','Test',986,'F2H2',50 from dual union all
select 35840,'1974-W80','1974-01-18 17:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 35841,'1974-W800','1974-06-18 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 35842,'1974-W801','1974-06-18 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 35843,'1974-W802','1974-06-18 17:45:00','suborbital rocket','success',1061,'','Met Para','','','Meteo',1895,'',50 from dual union all
select 35844,'1974-W803','1974-06-19 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',66 from dual union all
select 35845,'1974-W804','1974-06-19 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 35846,'1974-W805','1974-06-19 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 35847,'1974-W806','1974-06-19 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1470,'',69 from dual union all
select 35848,'1974-W807','1974-06-19 16:09:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 35849,'1974-W808','1974-06-19 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',69 from dual union all
select 3585,'1955-M84','1965-09-05 00:00:00','military missile','success',1162,'','','RB250 Training','','OT',1447,'',50 from dual union all
select 35850,'1974-W809','1974-06-19 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 35851,'1974-W81','1974-01-18 19:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 35852,'1974-W810','1974-06-19 17:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 35853,'1974-W811','1974-06-19 20:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 35854,'1974-W812','1974-06-19 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 35855,'1974-W813','1974-06-20 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',72 from dual union all
select 35856,'1974-W814','1974-06-20 20:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 35857,'1974-W815','1974-06-21 15:15:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',73 from dual union all
select 35858,'1974-W816','1974-06-21 15:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 35859,'1974-W817','1974-06-21 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 3586,'1955-M85','1965-08-01 00:00:00','military missile','success',1162,'','','RB150 Training','','OT',1447,'',50 from dual union all
select 35860,'1974-W818','1974-06-21 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 35861,'1974-W819','1974-06-21 21:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',54 from dual union all
select 35862,'1974-W82','1974-01-19 00:30:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 35863,'1974-W820','1974-06-21 21:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 35864,'1974-W821','1974-06-23 00:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',68 from dual union all
select 35865,'1974-W822','1974-06-24 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 35866,'1974-W823','1974-06-24 20:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',69 from dual union all
select 35867,'1974-W824','1974-06-24 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 35868,'1974-W825','1974-06-25 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',54 from dual union all
select 35869,'1974-W826','1974-06-26 14:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 3587,'1955-S01','1955-01-12 00:00:00','military missile','success',971,'K3-3','','','','Test',636,'',150 from dual union all
select 35870,'1974-W827','1974-06-26 15:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 35871,'1974-W828','1974-06-26 15:59:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 35872,'1974-W829','1974-06-26 16:24:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',77 from dual union all
select 35873,'1974-W83','1974-01-21 09:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 35874,'1974-W830','1974-06-26 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 35875,'1974-W831','1974-06-26 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 35876,'1974-W832','1974-06-26 20:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 35877,'1974-W833','1974-06-26 21:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 35878,'1974-W834','1974-06-27 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 35879,'1974-W835','1974-06-27 16:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 3588,'1955-S02','1955-01-06 00:00:00','military missile','success',1009,'K3-13','8A62 ZI','','','Test',636,'',300 from dual union all
select 35880,'1974-W836','1974-06-28 10:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 35881,'1974-W837','1974-06-28 15:55:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',52 from dual union all
select 35882,'1974-W838','1974-06-28 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 35883,'1974-W839','1974-06-28 18:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 35884,'1974-W84','1974-01-21 13:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 35885,'1974-W840','1974-06-28 18:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 35886,'1974-W841','1974-06-28 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 35887,'1974-W842','1974-06-29 17:00:00','suborbital rocket','success',1506,'Starute,Datasonde','ALADDIN 74','','','Meteo',1821,'',67 from dual union all
select 35888,'1974-W843','1974-06-29 18:20:00','suborbital rocket','unknown',1506,'E 1-7314','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35889,'1974-W844','1974-06-29 17:40:00','suborbital rocket','unknown',1284,'T 1-7328','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 3589,'1955-S03','1955-01-08 00:00:00','military missile','success',1009,'K3-12','8A62 ZI','','','Test',636,'',300 from dual union all
select 35890,'1974-W845','1974-06-29 19:30:00','suborbital rocket','unknown',1284,'T 1-7329','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35891,'1974-W846','1974-06-29 20:25:00','suborbital rocket','unknown',1281,'E 1-7323','ALADDIN 74','','','Aeron',1821,'',60 from dual union all
select 35892,'1974-W847','1974-06-29 20:35:00','suborbital rocket','unknown',1284,'T 1-3330','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35893,'1974-W848','1974-06-29 20:50:00','suborbital rocket','unknown',1506,'E 1-7315','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35894,'1974-W849','1974-06-29 21:40:00','suborbital rocket','unknown',1284,'T 1-7331','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35895,'1974-W85','1974-01-21 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',67 from dual union all
select 35896,'1974-W850','1974-06-29 23:30:00','suborbital rocket','unknown',1284,'T 1-7332','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35897,'1974-W851','1974-07-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 35898,'1974-W852','1974-07-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 35899,'1974-W853','1974-07-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 359,'1944-S327','1944-10-03 00:00:00','military missile','success',1470,'','London','','','Weapon',1734,'',90 from dual union all
select 3590,'1955-S04','1955-01-17 00:00:00','military missile','success',1009,'K3-14','8A62 ZI','','','Test',636,'',300 from dual union all
select 35900,'1974-W854','1974-07-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 35901,'1974-W855','1974-07-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 35902,'1974-W856','1974-07-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 35903,'1974-W857','1974-07-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 35904,'1974-W858','1974-07-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 35905,'1974-W859','1974-07-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 35906,'1974-W86','1974-01-21 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',75 from dual union all
select 35907,'1974-W860','1974-06-30 00:35:00','suborbital rocket','unknown',1506,'E 1-7316','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35908,'1974-W861','1974-06-30 02:00:00','suborbital rocket','unknown',1284,'T 1-7333','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35909,'1974-W862','1974-06-30 03:15:00','suborbital rocket','success',1281,'NASA 15.126UE','ALADDIN 74','','','Plasma',1821,'',72 from dual union all
select 3591,'1955-S05','1955-01-20 00:00:00','military missile','success',1013,'S1-2','8K51 LKI-1','','','Test',636,'',500 from dual union all
select 35910,'1974-W863','1974-06-30 04:55:00','suborbital rocket','unknown',1284,'T 1-7334','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35911,'1974-W864','1974-06-30 06:20:00','suborbital rocket','unknown',1284,'T 1-7335','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35912,'1974-W865','1974-06-30 06:30:00','suborbital rocket','unknown',1506,'E 1-7318','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35913,'1974-W866','1974-06-30 07:20:00','suborbital rocket','unknown',1284,'T 1-7336','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35914,'1974-W867','1974-06-30 08:40:00','suborbital rocket','unknown',1506,'E 1-7319','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35915,'1974-W868','1974-06-30 09:10:00','suborbital rocket','unknown',1284,'T 1-7337','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35916,'1974-W869','1974-06-30 10:05:00','suborbital rocket','unknown',1506,'E 1-7320','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35917,'1974-W87','1974-01-21 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 35918,'1974-W870','1974-06-30 10:20:00','suborbital rocket','unknown',1284,'T 1-7338','ALADDIN 74','','','Aeron',1821,'',75 from dual;
insert into LAUNCH
select 35919,'1974-W871','1974-06-30 11:10:00','suborbital rocket','unknown',1284,'T 1-7339','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 3592,'1955-S06','1955-01-22 00:54:00','suborbital rocket','success',51,'USAF 51','Na Release 1','','','Aeronomy',640,'',94 from dual union all
select 35920,'1974-W872','1974-06-30 13:15:00','suborbital rocket','unknown',1506,'E 1-7321','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35921,'1974-W873','1974-06-30 13:45:00','suborbital rocket','unknown',1284,'T 1-7340','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35922,'1974-W874','1974-06-30 13:52:00','suborbital rocket','success',1506,'Starute,Datasonde','ALADDIN 74','','','Meteo',1821,'',68 from dual union all
select 35923,'1974-W875','1974-06-30 14:30:00','suborbital rocket','unknown',1506,'E 1-7322','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35924,'1974-W876','1974-06-30 17:00:00','suborbital rocket','unknown',1284,'T 1-7341','ALADDIN 74','','','Aeron',1821,'',75 from dual union all
select 35925,'1974-W877','1974-07-01 13:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 35926,'1974-W878','1974-07-01 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 35927,'1974-W879','1974-07-01 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 35928,'1974-W88','1974-01-21 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',73 from dual union all
select 35929,'1974-W880','1974-07-01 19:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 3593,'1955-S07','1955-01-25 00:00:00','suborbital rocket','success',988,'1','1RE','','','Test/Bio',636,'',100 from dual union all
select 35930,'1974-W881','1974-07-01 20:02:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',63 from dual union all
select 35931,'1974-W882','1974-07-01 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 35932,'1974-W883','1974-07-02 13:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 35933,'1974-W884','1974-07-02 14:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 35934,'1974-W885','1974-07-03 09:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',65 from dual union all
select 35935,'1974-W886','1974-07-03 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',72 from dual union all
select 35936,'1974-W887','1974-07-03 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 35937,'1974-W888','1974-07-03 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 35938,'1974-W889','1974-07-03 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',70 from dual union all
select 35939,'1974-W89','1974-01-21 16:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 3594,'1955-S08','1955-01-28 00:00:00','military missile','success',970,'No 720','','','','OT',636,'',100 from dual union all
select 35940,'1974-W890','1974-07-03 17:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 35941,'1974-W891','1974-07-03 20:34:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',55 from dual union all
select 35942,'1974-W892','1974-07-03 21:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 35943,'1974-W893','1974-07-05 09:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 35944,'1974-W894','1974-07-05 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 35945,'1974-W895','1974-07-05 13:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',73 from dual union all
select 35946,'1974-W896','1974-07-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',52 from dual union all
select 35947,'1974-W897','1974-07-08 09:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',67 from dual union all
select 35948,'1974-W898','1974-07-08 17:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 35949,'1974-W899','1974-07-08 18:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 3595,'1955-S09','1955-01-29 00:00:00','military missile','success',970,'No 920','','','','OT',636,'',100 from dual union all
select 35950,'1974-W90','1974-01-21 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 35951,'1974-W900','1974-07-08 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',56 from dual union all
select 35952,'1974-W901','1974-07-08 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 35953,'1974-W902','1974-07-09 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',61 from dual union all
select 35954,'1974-W903','1974-07-09 14:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 35955,'1974-W904','1974-07-09 16:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 35956,'1974-W905','1974-07-10 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',64 from dual union all
select 35957,'1974-W906','1974-07-10 13:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 35958,'1974-W907','1974-07-10 14:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 35959,'1974-W908','1974-07-10 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 3596,'1955-S10','1955-01-28 00:00:00','military missile','success',971,'K3-14','','','','Test',636,'',150 from dual union all
select 35960,'1974-W909','1974-07-10 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',73 from dual union all
select 35961,'1974-W91','1974-01-21 19:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 35962,'1974-W910','1974-07-10 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',63 from dual union all
select 35963,'1974-W911','1974-07-10 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',70 from dual union all
select 35964,'1974-W912','1974-07-10 23:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 35965,'1974-W913','1974-07-11 12:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',68 from dual union all
select 35966,'1974-W914','1974-07-11 14:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',62 from dual union all
select 35967,'1974-W915','1974-07-11 20:10:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',60 from dual union all
select 35968,'1974-W916','1974-07-12 13:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',67 from dual union all
select 35969,'1974-W917','1974-07-12 13:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 3597,'1955-S100','1955-11-04 00:00:00','suborbital rocket','success',988,'3','','','','Bio',636,'',100 from dual union all
select 35970,'1974-W918','1974-07-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',54 from dual union all
select 35971,'1974-W919','1974-07-12 14:37:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',68 from dual union all
select 35972,'1974-W92','1974-01-22 00:54:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 35973,'1974-W920','1974-07-12 15:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',72 from dual union all
select 35974,'1974-W921','1974-07-12 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 35975,'1974-W922','1974-07-12 19:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 35976,'1974-W923','1974-07-15 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 35977,'1974-W924','1974-07-15 16:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 35978,'1974-W925','1974-07-15 17:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 35979,'1974-W926','1974-07-15 17:29:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',67 from dual union all
select 3598,'1955-S101','1955-11-04 15:30:00','suborbital rocket','success',57,'NRL 36','Solar La/airglow','','','Solar UV',1869,'',135 from dual union all
select 35980,'1974-W927','1974-07-15 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 35981,'1974-W928','1974-07-16 12:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',72 from dual union all
select 35982,'1974-W929','1974-07-16 17:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 35983,'1974-W93','1974-01-22 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',72 from dual union all
select 35984,'1974-W930','1974-07-16 20:13:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',61 from dual union all
select 35985,'1974-W931','1974-07-17 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',63 from dual union all
select 35986,'1974-W932','1974-07-17 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',57 from dual union all
select 35987,'1974-W933','1974-07-17 13:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',73 from dual union all
select 35988,'1974-W934','1974-07-17 13:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 35989,'1974-W935','1974-07-17 14:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',71 from dual union all
select 3599,'1955-S102','1955-11-17 09:15:00','suborbital rocket','success',54,'NRL 25','','','','Meteorites/UV Astron',1869,'',105 from dual union all
select 35990,'1974-W936','1974-07-17 14:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 35991,'1974-W937','1974-07-17 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',66 from dual union all
select 35992,'1974-W938','1974-07-17 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 35993,'1974-W939','1974-07-18 14:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 35994,'1974-W94','1974-01-22 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 35995,'1974-W940','1974-07-18 16:40:00','suborbital rocket','success',1061,'','Met Para,STS-','','','Meteo',1895,'',56 from dual union all
select 35996,'1974-W941','1974-07-18 17:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 35997,'1974-W942','1974-07-19 14:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 35998,'1974-W943','1974-07-19 14:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',51 from dual union all
select 35999,'1974-W944','1974-07-19 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 36,'1943-S14','1943-11-10 00:00:00','military missile','unknown',8,'73','','','','Test',675,'',95 from dual union all
select 360,'1944-S328','1944-10-03 00:00:00','military missile','success',1470,'','London','','','Weapon',1734,'',90 from dual union all
select 3600,'1955-S103','1955-11-19 00:00:00','military missile','success',1013,'S1-22','8K51 LKI-24','','','Test',636,'',500 from dual union all
select 36000,'1974-W945','1974-07-19 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',54 from dual union all
select 36001,'1974-W946','1974-07-19 20:00:00','suborbital rocket','success',1061,'','Met Para,STS-','','','Meteo',1895,'',63 from dual union all
select 36002,'1974-W947','1974-07-19 21:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 36003,'1974-W948','1974-07-22 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',75 from dual union all
select 36004,'1974-W949','1974-07-22 14:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 36005,'1974-W95','1974-01-22 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 36006,'1974-W950','1974-07-22 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',60 from dual union all
select 36007,'1974-W951','1974-07-22 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual;
insert into LAUNCH
select 36008,'1974-W952','1974-07-22 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',54 from dual union all
select 36009,'1974-W953','1974-07-22 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 3601,'1955-S104','1955-11-29 17:16:00','suborbital rocket','success',54,'NRL 24','Mass spectrometers','','','Ionos',1869,'',132 from dual union all
select 36010,'1974-W954','1974-07-23 09:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',67 from dual union all
select 36011,'1974-W955','1974-07-24 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',76 from dual union all
select 36012,'1974-W956','1974-07-24 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 36013,'1974-W957','1974-07-24 16:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 36014,'1974-W958','1974-07-24 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 36015,'1974-W959','1974-07-24 20:00:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',58 from dual union all
select 36016,'1974-W96','1974-01-22 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 36017,'1974-W960','1974-07-24 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 36018,'1974-W961','1974-07-25 18:00:00','suborbital rocket','success',1281,'NASA 15.106UI','','','','Ionos',1895,'',74 from dual union all
select 36019,'1974-W962','1974-07-26 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 3602,'1955-S105','1955-12-01 20:56:00','suborbital rocket','success',1521,'D-3','','','','Test',253,'',40 from dual union all
select 36020,'1974-W963','1974-07-26 14:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 36021,'1974-W964','1974-07-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',67 from dual union all
select 36022,'1974-W965','1974-07-26 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 36023,'1974-W966','1974-07-26 20:30:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',68 from dual union all
select 36024,'1974-W967','1974-07-29 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',66 from dual union all
select 36025,'1974-W968','1974-07-29 15:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 36026,'1974-W969','1974-07-29 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 36027,'1974-W97','1974-01-23 00:19:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 36028,'1974-W970','1974-07-29 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',63 from dual union all
select 36029,'1974-W971','1974-07-29 20:08:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',61 from dual union all
select 3603,'1955-S106','1955-12-06 00:46:00','military missile','success',635,'RS-12    HU','','','','Test',280,'',90 from dual union all
select 36030,'1974-W972','1974-07-30 14:22:00','suborbital rocket','unknown',1284,'G 1-7460','','','','Test/Aeron',1821,'',72 from dual union all
select 36031,'1974-W973','1974-07-30 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 36032,'1974-W974','1974-07-30 17:16:00','suborbital rocket','unknown',1284,'G 1-7461','','','','Test/Aeron',1821,'',72 from dual union all
select 36033,'1974-W975','1974-07-30 19:00:00','suborbital rocket','unknown',1284,'G 1-7462','','','','Test/Aeron',1821,'',75 from dual union all
select 36034,'1974-W976','1974-07-30 20:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 36035,'1974-W977','1974-08-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 36036,'1974-W978','1974-08-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 36037,'1974-W979','1974-08-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 36038,'1974-W98','1974-01-23 13:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 36039,'1974-W980','1974-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 3604,'1955-S107','1955-12-13 05:00:00','suborbital rocket','success',55,'NRL 28','Airglow/Mass spec','','','Aeronomy',1869,'',142 from dual union all
select 36040,'1974-W981','1974-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 36041,'1974-W982','1974-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 36042,'1974-W983','1974-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 36043,'1974-W984','1974-07-31 09:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',176,'',67 from dual union all
select 36044,'1974-W985','1974-07-31 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',53 from dual union all
select 36045,'1974-W986','1974-07-31 14:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 36046,'1974-W987','1974-07-31 15:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',76 from dual union all
select 36047,'1974-W988','1974-07-31 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 36048,'1974-W989','1974-07-31 20:35:00','suborbital rocket','success',1061,'','Met Para,Datasonde','','','Meteo',1047,'',58 from dual union all
select 36049,'1974-W99','1974-01-23 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',58 from dual union all
select 3605,'1955-S108','1955-12-13 17:58:00','suborbital rocket','success',47,'USAF 61','Colorado UVS','','','Solar UV',640,'',138 from dual union all
select 36050,'1974-W990','1974-07-31 21:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 36051,'1974-W991','1974-08-01 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 36052,'1974-W992','1974-08-01 14:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 36053,'1974-W993','1974-08-01 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 36054,'1974-W994','1974-08-01 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 36055,'1974-W995','1974-08-02 13:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',35,'',74 from dual union all
select 36056,'1974-W996','1974-08-02 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',59 from dual union all
select 36057,'1974-W997','1974-08-02 14:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 36058,'1974-W998','1974-08-02 15:55:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',70 from dual union all
select 36059,'1974-W999','1974-08-02 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 3606,'1955-S109','1955-12-30 00:00:00','military missile','failure',975,'S1-5','LKI 1A','','','Test',636,'',10 from dual union all
select 36060,'1974-Y01','1974-01-02 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 36061,'1974-Y02','1974-01-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 36062,'1974-Y03','1974-01-02 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.167','TERLS-622','','Aeron',1467,'',85 from dual union all
select 36063,'1974-Y04','1974-01-02 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 36064,'1974-Y05','1974-01-03 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',81 from dual union all
select 36065,'1974-Y06','1974-01-05 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 36066,'1974-Y07','1974-01-08 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 36067,'1974-Y08','1974-01-09 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 36068,'1974-Y09','1974-01-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 36069,'1974-Y10','1974-01-09 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.168','TERLS-623','','Aeron',1467,'',86 from dual union all
select 3607,'1955-S11','1955-02-09 00:00:00','military missile','success',1013,'S1-3','8K51 LKI-2','','','Test',636,'',500 from dual union all
select 36070,'1974-Y100','1974-03-06 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 36071,'1974-Y101','1974-03-07 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',93 from dual union all
select 36072,'1974-Y102','1974-03-10 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',88 from dual union all
select 36073,'1974-Y103','1974-03-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 36074,'1974-Y104','1974-03-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 36075,'1974-Y105','1974-03-13 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.177','TERLS-647','','Aeron',1467,'',83 from dual union all
select 36076,'1974-Y106','1974-03-13 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 36077,'1974-Y107','1974-03-13 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',95 from dual union all
select 36078,'1974-Y108','1974-03-14 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',81 from dual union all
select 36079,'1974-Y109','1974-03-14 20:59:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 3608,'1955-S110','1955-01-04 00:00:00','military missile','success',992,'409','Generator-2','','','Test',636,'',170 from dual union all
select 36080,'1974-Y11','1974-01-09 15:56:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 36081,'1974-Y110','1974-03-14 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',84 from dual union all
select 36082,'1974-Y111','1974-03-17 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',84 from dual union all
select 36083,'1974-Y112','1974-03-20 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',90 from dual union all
select 36084,'1974-Y113','1974-03-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',86 from dual union all
select 36085,'1974-Y114','1974-03-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 36086,'1974-Y115','1974-03-20 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.178','TERLS-654','','Aeron',1467,'',86 from dual union all
select 36087,'1974-Y116','1974-03-20 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 36088,'1974-Y117','1974-03-20 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',88 from dual union all
select 36089,'1974-Y118','1974-03-20 21:18:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 3609,'1955-S111','1955-01-14 00:00:00','military missile','success',971,'K3-4','','','','Test',636,'',150 from dual union all
select 36090,'1974-Y119','1974-03-20 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',85 from dual union all
select 36091,'1974-Y12','1974-01-09 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',94 from dual union all
select 36092,'1974-Y120','1974-03-22 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',91 from dual union all
select 36093,'1974-Y121','1974-03-22 14:45:00','suborbital rocket','failure',1281,'NASA 15.124GM','','Ozone','','Aeronomy',340,'',89 from dual union all
select 36094,'1974-Y122','1974-03-22 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 36095,'1974-Y123','1974-03-23 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',86 from dual union all
select 36096,'1974-Y124','1974-03-23 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',82 from dual union all
select 36097,'1974-Y125','1974-03-23 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',86 from dual union all
select 36098,'1974-Y126','1974-03-24 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',88 from dual;
insert into LAUNCH
select 36099,'1974-Y127','1974-03-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 361,'1944-S329','1944-10-03 00:00:00','military missile','success',1470,'','London','','','Weapon',1734,'',90 from dual union all
select 3610,'1955-S112','1955-01-21 00:00:00','military missile','success',971,'K3-6','','','','Test',636,'',150 from dual union all
select 36100,'1974-Y128','1974-03-26 21:30:00','suborbital rocket','success',229,'AAF-VI-04','Dust collector','','','Aeron',428,'',81 from dual union all
select 36101,'1974-Y129','1974-03-26 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 36102,'1974-Y13','1974-01-12 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 36103,'1974-Y130','1974-03-27 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 36104,'1974-Y131','1974-03-27 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',93 from dual union all
select 36105,'1974-Y132','1974-03-27 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',82 from dual union all
select 36106,'1974-Y133','1974-03-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 36107,'1974-Y134','1974-03-27 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',91 from dual union all
select 36108,'1974-Y135','1974-03-27 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.179','TERLS-655','','Aeron',1467,'',88 from dual union all
select 36109,'1974-Y136','1974-03-29 21:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 3611,'1955-S113','1955-01-21 00:00:00','military missile','success',971,'K3-5','','','','Test',636,'',150 from dual union all
select 36110,'1974-Y137','1974-03-31 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',87 from dual union all
select 36111,'1974-Y138','1974-03-31 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',93 from dual union all
select 36112,'1974-Y139','1974-04-02 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 36113,'1974-Y14','1974-01-15 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 36114,'1974-Y140','1974-04-03 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 36115,'1974-Y141','1974-04-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 36116,'1974-Y142','1974-04-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 36117,'1974-Y143','1974-04-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 36118,'1974-Y144','1974-04-03 16:00:00','suborbital rocket','unknown',725,'','ISRO 8.180','TERLS-656','','Aeron',1467,'',87 from dual union all
select 36119,'1974-Y145','1974-04-03 17:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 3612,'1955-S115','1955-01-15 00:00:00','military missile','success',992,'408','Generator-2','','','Test',636,'',170 from dual union all
select 36120,'1974-Y146','1974-04-04 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 36121,'1974-Y147','1974-04-05 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 36122,'1974-Y148','1974-04-05 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 36123,'1974-Y149','1974-04-09 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 36124,'1974-Y15','1974-01-16 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 36125,'1974-Y150','1974-04-09 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 36126,'1974-Y151','1974-04-10 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 36127,'1974-Y152','1974-04-10 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 36128,'1974-Y153','1974-04-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',82 from dual union all
select 36129,'1974-Y154','1974-04-10 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.181','TERLS-660','','Aeron',1467,'',88 from dual union all
select 3613,'1955-S116','1955-01-21 00:00:00','military missile','success',1009,'K3-17','8A62 ZI','','','Test',636,'',300 from dual union all
select 36130,'1974-Y155','1974-04-10 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 36131,'1974-Y156','1974-04-12 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 36132,'1974-Y157','1974-04-12 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 36133,'1974-Y158','1974-04-12 17:29:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 36134,'1974-Y159','1974-04-17 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 36135,'1974-Y16','1974-01-16 13:00:00','suborbital rocket','success',1284,'INTA SHR-7401','DLR H-SD-60','','','Aeron',396,'',100 from dual union all
select 36136,'1974-Y160','1974-04-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 36137,'1974-Y161','1974-04-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 36138,'1974-Y162','1974-04-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 36139,'1974-Y163','1974-04-17 14:30:00','suborbital rocket','success',725,'','ISRO 8.182','TERLS-661','','Aeron',1467,'',84 from dual union all
select 3614,'1955-S117','1955-01-22 00:00:00','military missile','success',1009,'K3-18','8A62 ZI','','','Test',636,'',300 from dual union all
select 36140,'1974-Y164','1974-04-17 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 36141,'1974-Y165','1974-04-17 18:59:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',86 from dual union all
select 36142,'1974-Y166','1974-04-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',90 from dual union all
select 36143,'1974-Y167','1974-04-20 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',90 from dual union all
select 36144,'1974-Y168','1974-04-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',97 from dual union all
select 36145,'1974-Y169','1974-04-22 16:30:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',82 from dual union all
select 36146,'1974-Y17','1974-01-16 15:00:00','suborbital rocket','unknown',725,'','ISRO-8.169','TERLS-624','','Aeron',1467,'',86 from dual union all
select 36147,'1974-Y170','1974-04-22 16:49:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',92 from dual union all
select 36148,'1974-Y171','1974-04-22 19:01:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',97 from dual union all
select 36149,'1974-Y172','1974-04-22 19:49:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1895,'',92 from dual union all
select 3615,'1955-S118','1955-01-25 00:00:00','military missile','success',1009,'K3-19','8A62 ZI','','','Test',636,'',300 from dual union all
select 36150,'1974-Y173','1974-04-23 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 36151,'1974-Y174','1974-04-24 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 36152,'1974-Y175','1974-04-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 36153,'1974-Y176','1974-04-24 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',92 from dual union all
select 36154,'1974-Y177','1974-04-24 16:00:00','suborbital rocket','unknown',725,'','ISRO 8.183','TERLS-662','','Aeron',1467,'',84 from dual union all
select 36155,'1974-Y178','1974-04-24 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',95 from dual union all
select 36156,'1974-Y179','1974-04-25 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 36157,'1974-Y18','1974-01-16 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 36158,'1974-Y180','1974-04-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 36159,'1974-Y181','1974-04-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',92 from dual union all
select 3616,'1955-S119','1955-01-29 00:00:00','military missile','success',1009,'K3-20','8A62 ZI','','','Test',636,'',300 from dual union all
select 36160,'1974-Y182','1974-04-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 36161,'1974-Y183','1974-04-30 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 36162,'1974-Y184','1974-04-30 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 36163,'1974-Y185','1974-05-01 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.184','TERLS-674','','Aeron',1467,'',86 from dual union all
select 36164,'1974-Y186','1974-05-05 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',87 from dual union all
select 36165,'1974-Y187','1974-05-07 22:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 36166,'1974-Y188','1974-05-08 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 36167,'1974-Y189','1974-05-08 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',99 from dual union all
select 36168,'1974-Y19','1974-01-16 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 36169,'1974-Y190','1974-05-08 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.185','TERLS-681','','Aeron',1467,'',83 from dual union all
select 3617,'1955-S12','1955-02-14 00:00:00','military missile','success',1013,'S1-4','8K51 LKI-3','','','Test',636,'',500 from dual union all
select 36170,'1974-Y191','1974-05-09 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',89 from dual union all
select 36171,'1974-Y192','1974-05-10 21:11:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',82 from dual union all
select 36172,'1974-Y193','1974-05-14 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 36173,'1974-Y194','1974-05-15 07:45:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',87 from dual union all
select 36174,'1974-Y195','1974-05-15 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 36175,'1974-Y196','1974-05-15 15:48:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 36176,'1974-Y197','1974-05-16 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.186','TERLS-684','','Aeron',1467,'',84 from dual union all
select 36177,'1974-Y198','1974-05-18 06:40:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',85 from dual union all
select 36178,'1974-Y199','1974-05-21 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 36179,'1974-Y20','1974-01-16 18:13:00','suborbital rocket','success',1281,'NASA 15.105UI','','','','Ionos',1821,'',103 from dual union all
select 3618,'1955-S120','1955-01-25 00:00:00','military missile','success',992,'403','Generator-2','','','Test',636,'',170 from dual union all
select 36180,'1974-Y200','1974-05-22 06:40:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',87 from dual union all
select 36181,'1974-Y201','1974-05-22 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 36182,'1974-Y202','1974-05-22 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 36183,'1974-Y203','1974-05-22 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.187','TERLS-689','','Aeron',1467,'',81 from dual union all
select 36184,'1974-Y204','1974-05-22 16:04:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 36185,'1974-Y205','1974-05-25 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',87 from dual union all
select 36186,'1974-Y206','1974-05-27 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',85 from dual union all
select 36187,'1974-Y207','1974-05-28 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36188,'1974-Y208','1974-05-28 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual;
insert into LAUNCH
select 36189,'1974-Y209','1974-05-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 3619,'1955-S121','1955-01-31 00:00:00','military missile','success',992,'402','Generator-2','','','Test',636,'',170 from dual union all
select 36190,'1974-Y21','1974-01-16 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36191,'1974-Y210','1974-05-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',97 from dual union all
select 36192,'1974-Y211','1974-05-29 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.188','TERLS-692','','Aeron',1467,'',86 from dual union all
select 36193,'1974-Y212','1974-06-05 00:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 36194,'1974-Y213','1974-06-05 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 36195,'1974-Y214','1974-06-07 13:44:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',95 from dual union all
select 36196,'1974-Y215','1974-06-12 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 36197,'1974-Y216','1974-06-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 36198,'1974-Y217','1974-06-12 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.190','TERLS-694','','Aeron',1467,'',84 from dual union all
select 36199,'1974-Y218','1974-06-12 15:33:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 362,'1944-S33','1944-08-14 09:33:00','military missile','success',1470,'18291','Ma108','','','Training',661,'',90 from dual union all
select 3620,'1955-S123','1955-02-08 00:00:00','military missile','success',992,'K5-20','','','','OT',636,'',170 from dual union all
select 36200,'1974-Y219','1974-06-12 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',97 from dual union all
select 36201,'1974-Y22','1974-01-18 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 36202,'1974-Y220','1974-06-12 21:15:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36203,'1974-Y221','1974-06-18 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 36204,'1974-Y222','1974-06-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 36205,'1974-Y223','1974-06-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',95 from dual union all
select 36206,'1974-Y224','1974-06-26 09:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 36207,'1974-Y225','1974-06-26 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 36208,'1974-Y226','1974-06-29 21:10:00','suborbital rocket','success',1281,'NASA 15.104UI','ALADDIN 74','','','Ionos',1821,'',89 from dual union all
select 36209,'1974-Y227','1974-07-01 00:05:00','suborbital rocket','success',1281,'NASA 15.125UI','ALADDIN 74','','','Ionos',1821,'',83 from dual union all
select 3621,'1955-S124','1955-02-08 00:00:00','military missile','success',992,'K4-20','','','','OT',636,'',170 from dual union all
select 36210,'1974-Y228','1974-06-30 10:35:00','suborbital rocket','unknown',1281,'E 1-7324','ALADDIN 74','','','Aeron',1821,'',90 from dual union all
select 36211,'1974-Y229','1974-07-03 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 36212,'1974-Y23','1974-01-19 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 36213,'1974-Y230','1974-07-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 36214,'1974-Y231','1974-07-03 20:55:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36215,'1974-Y232','1974-07-10 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 36216,'1974-Y233','1974-07-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',95 from dual union all
select 36217,'1974-Y234','1974-07-10 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 36218,'1974-Y235','1974-07-10 20:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 36219,'1974-Y236','1974-07-10 23:03:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 3622,'1955-S125','1955-02-10 00:00:00','military missile','success',971,'K3-9','','','','Test',636,'',150 from dual union all
select 36220,'1974-Y237','1974-07-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 36221,'1974-Y238','1974-07-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',96 from dual union all
select 36222,'1974-Y239','1974-07-17 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',97 from dual union all
select 36223,'1974-Y24','1974-01-21 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 36224,'1974-Y240','1974-07-19 20:55:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36225,'1974-Y241','1974-07-24 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 36226,'1974-Y242','1974-07-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 36227,'1974-Y243','1974-07-24 18:24:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 36228,'1974-Y244','1974-07-24 21:04:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36229,'1974-Y245','1974-07-31 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 3623,'1955-S126','1955-02-11 00:00:00','military missile','success',971,'K3-8','','','','Test',636,'',150 from dual union all
select 36230,'1974-Y246','1974-07-31 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 36231,'1974-Y247','1974-07-31 15:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 36232,'1974-Y248','1974-07-31 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 36233,'1974-Y249','1974-07-31 20:55:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36234,'1974-Y25','1974-01-23 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 36235,'1974-Y250','1974-08-07 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 36236,'1974-Y251','1974-08-07 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 36237,'1974-Y252','1974-08-07 19:06:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 36238,'1974-Y253','1974-08-09 21:19:00','suborbital rocket','success',1061,'','','','','Meteo',855,'',81 from dual union all
select 36239,'1974-Y254','1974-08-14 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 3624,'1955-S127','1955-02-15 00:00:00','military missile','success',971,'S3-12','','','','Test',636,'',150 from dual union all
select 36240,'1974-Y255','1974-08-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 36241,'1974-Y256','1974-08-14 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 36242,'1974-Y257','1974-08-14 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1390,'SHOK',90 from dual union all
select 36243,'1974-Y258','1974-08-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 36244,'1974-Y259','1974-08-18 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1390,'SHOK',91 from dual union all
select 36245,'1974-Y26','1974-01-23 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 36246,'1974-Y260','1974-08-20 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1390,'SHOK',87 from dual union all
select 36247,'1974-Y261','1974-08-20 19:08:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 36248,'1974-Y262','1974-08-20 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 36249,'1974-Y263','1974-08-21 11:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 3625,'1955-S128','1955-02-17 00:00:00','military missile','success',971,'S3-15','','','','Test',636,'',150 from dual union all
select 36250,'1974-Y264','1974-08-21 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',97 from dual union all
select 36251,'1974-Y265','1974-08-23 07:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 36252,'1974-Y266','1974-08-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 36253,'1974-Y267','1974-08-23 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1390,'SHOK',85 from dual union all
select 36254,'1974-Y268','1974-08-24 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1390,'SHOK',90 from dual union all
select 36255,'1974-Y269','1974-08-26 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 36256,'1974-Y27','1974-01-23 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.170','TERLS-625','','Aeron',1467,'',87 from dual union all
select 36257,'1974-Y270','1974-08-26 13:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 36258,'1974-Y271','1974-08-27 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 36259,'1974-Y272','1974-08-28 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 3626,'1955-S129','1955-02-17 00:00:00','military missile','success',971,'S3-7','','','','Test',636,'',150 from dual union all
select 36260,'1974-Y273','1974-08-28 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 36261,'1974-Y274','1974-08-28 16:44:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',98 from dual union all
select 36262,'1974-Y275','1974-08-28 17:58:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',88 from dual union all
select 36263,'1974-Y276','1974-08-30 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 36264,'1974-Y277','1974-09-02 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 36265,'1974-Y278','1974-09-03 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1390,'SHOK',88 from dual union all
select 36266,'1974-Y279','1974-09-03 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 36267,'1974-Y28','1974-01-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 36268,'1974-Y280','1974-09-04 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 36269,'1974-Y281','1974-09-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 3627,'1955-S13','1955-02-03 00:00:00','military missile','success',970,'No 820','','','','OT',636,'',100 from dual union all
select 36270,'1974-Y282','1974-09-04 16:38:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 36271,'1974-Y283','1974-09-04 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36272,'1974-Y284','1974-09-05 19:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1390,'SHOK',80 from dual union all
select 36273,'1974-Y285','1974-09-06 05:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 36274,'1974-Y286','1974-09-10 22:00:00','suborbital rocket','success',725,'','Diurnal Variation','','','Aeron',636,'',85 from dual union all
select 36275,'1974-Y287','1974-09-11 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 36276,'1974-Y288','1974-09-11 10:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 36277,'1974-Y289','1974-09-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 36278,'1974-Y29','1974-01-23 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',95 from dual;
insert into LAUNCH
select 36279,'1974-Y290','1974-09-11 20:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 3628,'1955-S130','1955-02-19 00:00:00','military missile','success',971,'S3-13','','','','Test',636,'',150 from dual union all
select 36280,'1974-Y291','1974-09-13 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1390,'SHOK',91 from dual union all
select 36281,'1974-Y292','1974-09-15 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1390,'SHOK',86 from dual union all
select 36282,'1974-Y293','1974-09-16 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1390,'SHOK',83 from dual union all
select 36283,'1974-Y294','1974-09-18 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 36284,'1974-Y295','1974-09-18 13:00:00','suborbital rocket','success',725,'','Diurnal Variation','','','Aeron',636,'',90 from dual union all
select 36285,'1974-Y296','1974-09-18 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 36286,'1974-Y297','1974-09-19 14:40:00','suborbital rocket','success',725,'','Diurnal Variation','','','Aeron',636,'',89 from dual union all
select 36287,'1974-Y298','1974-09-19 21:00:00','suborbital rocket','success',725,'','Diurnal Variation','','','Aeron',636,'',91 from dual union all
select 36288,'1974-Y299','1974-09-20 00:30:00','suborbital rocket','success',725,'','Diurnal Variation','','','Aeron',636,'',88 from dual union all
select 36289,'1974-Y30','1974-01-23 18:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 3629,'1955-S131','1955-02-21 00:00:00','military missile','success',971,'S3-16','','','','Test',636,'',150 from dual union all
select 36290,'1974-Y300','1974-09-20 03:10:00','suborbital rocket','success',725,'','Diurnal Variation','','','Aeron',636,'',87 from dual union all
select 36291,'1974-Y301','1974-09-20 07:40:00','suborbital rocket','success',725,'','Diurnal Variation','','','Aeron',636,'',89 from dual union all
select 36292,'1974-Y302','1974-09-20 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 36293,'1974-Y303','1974-09-20 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 36294,'1974-Y304','1974-09-20 18:40:00','suborbital rocket','success',725,'','Diurnal Variation','','','Aeron',636,'',87 from dual union all
select 36295,'1974-Y305','1974-09-20 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1390,'SHOK',88 from dual union all
select 36296,'1974-Y306','1974-09-21 04:40:00','suborbital rocket','success',725,'','Diurnal Variation','','','Aeron',636,'',91 from dual union all
select 36297,'1974-Y307','1974-09-21 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 36298,'1974-Y308','1974-09-21 11:30:00','suborbital rocket','success',725,'','Diurnal Variation','','','Aeron',636,'',88 from dual union all
select 36299,'1974-Y309','1974-09-24 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 363,'1944-S330','1944-10-03 21:00:00','military missile','success',1470,'','London/Wanstead UK43','','','Weapon',1734,'',90 from dual union all
select 3630,'1955-S132','1955-02-22 00:00:00','military missile','success',971,'S3-11','','','','Test',636,'',150 from dual union all
select 36300,'1974-Y31','1974-01-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 36301,'1974-Y310','1974-09-25 09:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 36302,'1974-Y311','1974-09-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 36303,'1974-Y312','1974-09-25 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 36304,'1974-Y313','1974-10-01 18:25:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 36305,'1974-Y314','1974-10-01 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 36306,'1974-Y315','1974-10-01 23:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 36307,'1974-Y316','1974-10-02 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 36308,'1974-Y317','1974-10-02 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 36309,'1974-Y318','1974-10-02 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',97 from dual union all
select 3631,'1955-S133','1955-02-25 00:00:00','military missile','success',992,'K7-20','','','','OT',636,'',170 from dual union all
select 36310,'1974-Y319','1974-10-02 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36311,'1974-Y32','1974-01-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',85 from dual union all
select 36312,'1974-Y320','1974-10-03 09:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 36313,'1974-Y321','1974-10-05 03:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 36314,'1974-Y322','1974-10-07 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',94 from dual union all
select 36315,'1974-Y323','1974-10-08 23:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 36316,'1974-Y324','1974-10-09 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 36317,'1974-Y325','1974-10-09 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 36318,'1974-Y326','1974-10-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 36319,'1974-Y327','1974-10-10 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 3632,'1955-S134','1955-02-28 00:00:00','military missile','success',992,'K8-20','','','','OT',636,'',170 from dual union all
select 36320,'1974-Y328','1974-10-10 21:02:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36321,'1974-Y329','1974-10-15 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 36322,'1974-Y33','1974-01-28 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',95 from dual union all
select 36323,'1974-Y330','1974-10-16 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 36324,'1974-Y331','1974-10-16 15:37:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 36325,'1974-Y332','1974-10-16 16:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',95 from dual union all
select 36326,'1974-Y333','1974-10-16 21:11:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36327,'1974-Y334','1974-10-22 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',881,'KORO',87 from dual union all
select 36328,'1974-Y335','1974-10-22 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 36329,'1974-Y336','1974-10-23 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 3633,'1955-S135','1955-03-01 00:00:00','military missile','success',992,'K6-20','','','','OT',636,'',170 from dual union all
select 36330,'1974-Y337','1974-10-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 36331,'1974-Y338','1974-10-23 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',82 from dual union all
select 36332,'1974-Y339','1974-10-27 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',881,'KORO',84 from dual union all
select 36333,'1974-Y34','1974-01-29 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',81 from dual union all
select 36334,'1974-Y340','1974-10-30 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',881,'KORO',90 from dual union all
select 36335,'1974-Y341','1974-10-30 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 36336,'1974-Y342','1974-10-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 36337,'1974-Y343','1974-10-30 17:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',96 from dual union all
select 36338,'1974-Y344','1974-10-30 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 36339,'1974-Y345','1974-11-02 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',881,'KORO',81 from dual union all
select 3634,'1955-S136','1955-03-21 00:00:00','military missile','success',992,'K0-10','UGCh','','','OT',636,'',170 from dual union all
select 36340,'1974-Y346','1974-11-05 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',881,'KORO',84 from dual union all
select 36341,'1974-Y347','1974-11-05 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 36342,'1974-Y348','1974-11-05 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 36343,'1974-Y349','1974-11-06 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',95 from dual union all
select 36344,'1974-Y35','1974-01-30 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 36345,'1974-Y350','1974-11-13 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 36346,'1974-Y351','1974-11-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',95 from dual union all
select 36347,'1974-Y352','1974-11-13 18:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',95 from dual union all
select 36348,'1974-Y353','1974-11-19 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 36349,'1974-Y354','1974-11-20 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 3635,'1955-S137','1955-04-04 00:00:00','military missile','success',992,'K0-11','UGCh','','','OT',636,'',170 from dual union all
select 36350,'1974-Y355','1974-11-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 36351,'1974-Y356','1974-11-22 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 36352,'1974-Y357','1974-11-26 10:30:00','suborbital rocket','unknown',725,'','','','','Aeron',881,'KORO',88 from dual union all
select 36353,'1974-Y358','1974-11-26 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',881,'KORO',91 from dual union all
select 36354,'1974-Y359','1974-11-27 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',97 from dual union all
select 36355,'1974-Y36','1974-01-30 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 36356,'1974-Y360','1974-11-27 17:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',98 from dual union all
select 36357,'1974-Y361','1974-11-27 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 36358,'1974-Y362','1974-11-27 21:48:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36359,'1974-Y363','1974-11-27 22:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 3636,'1955-S138','1955-05-18 00:00:00','military missile','success',992,'S1-20 TAD','','','','OT',636,'',170 from dual union all
select 36360,'1974-Y364','1974-11-30 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 36361,'1974-Y365','1974-12-02 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 36362,'1974-Y366','1974-12-03 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 36363,'1974-Y367','1974-12-04 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 36364,'1974-Y368','1974-12-04 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 36365,'1974-Y369','1974-12-04 17:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',97 from dual union all
select 36366,'1974-Y37','1974-01-30 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.171','TERLS-626','','Aeron',1467,'',82 from dual union all
select 36367,'1974-Y370','1974-12-04 20:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36368,'1974-Y371','1974-12-11 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual;
insert into LAUNCH
select 36369,'1974-Y372','1974-12-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 3637,'1955-S139','1955-05-20 00:00:00','military missile','success',992,'S2-20 TAD','','','','OT',636,'',170 from dual union all
select 36370,'1974-Y373','1974-12-11 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 36371,'1974-Y374','1974-12-11 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36372,'1974-Y375','1974-12-14 09:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 36373,'1974-Y376','1974-12-18 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 36374,'1974-Y377','1974-12-18 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 36375,'1974-Y378','1974-12-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 36376,'1974-Y379','1974-12-20 17:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',100 from dual union all
select 36377,'1974-Y38','1974-01-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 36378,'1974-Y380','1974-12-22 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1370,'SHIR',88 from dual union all
select 36379,'1974-Y381','1974-12-24 18:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 3638,'1955-S14','1955-02-04 21:55:00','suborbital rocket','success',1500,'12','Viking 12','','','Reentry test',1889,'',231 from dual union all
select 36380,'1974-Y382','1974-12-25 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1370,'SHIR',86 from dual union all
select 36381,'1974-Y383','1974-12-25 01:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 36382,'1974-Y384','1974-12-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 36383,'1974-Y385','1974-12-27 20:49:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36384,'1974-Y386','1974-12-28 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1370,'SHIR',91 from dual union all
select 36385,'1974-Y387','1974-12-31 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 36386,'1974-Y388','1974-06-01 00:00:00','suborbital rocket','unknown',725,'','ISRO 8.189','TERLS-693','','Aeron',1467,'',80 from dual union all
select 36387,'1974-Y39','1974-01-30 20:44:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 36388,'1974-Y40','1974-02-01 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 36389,'1974-Y41','1974-02-01 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',85 from dual union all
select 3639,'1955-S140','1955-05-28 00:00:00','military missile','success',992,'K8-12','','','','OT',636,'',170 from dual union all
select 36390,'1974-Y42','1974-02-03 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',86 from dual union all
select 36391,'1974-Y43','1974-02-05 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 36392,'1974-Y44','1974-02-06 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 36393,'1974-Y45','1974-02-06 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.172','TERLS-627','','Aeron',1467,'',84 from dual union all
select 36394,'1974-Y46','1974-02-06 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',87 from dual union all
select 36395,'1974-Y47','1974-02-07 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',89 from dual union all
select 36396,'1974-Y48','1974-02-07 18:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 36397,'1974-Y49','1974-02-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 36398,'1974-Y50','1974-02-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',90 from dual union all
select 36399,'1974-Y51','1974-02-09 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',90 from dual union all
select 364,'1944-S331','1944-10-30 05:10:00','military missile','success',1470,'','London/Barking','','','Weapon',1627,'',90 from dual union all
select 3640,'1955-S141','1955-05-31 00:00:00','military missile','success',992,'K2-9','','','','OT',636,'',170 from dual union all
select 36400,'1974-Y52','1974-02-09 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',86 from dual union all
select 36401,'1974-Y53','1974-02-11 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',91 from dual union all
select 36402,'1974-Y54','1974-02-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 36403,'1974-Y55','1974-02-11 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 36404,'1974-Y56','1974-02-11 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',88 from dual union all
select 36405,'1974-Y57','1974-02-13 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 36406,'1974-Y58','1974-02-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 36407,'1974-Y59','1974-02-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',91 from dual union all
select 36408,'1974-Y60','1974-02-13 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.173','TERLS-633','','Aeron',1467,'',85 from dual union all
select 36409,'1974-Y61','1974-02-13 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',89 from dual union all
select 3641,'1955-S142','1955-06-01 00:00:00','military missile','success',992,'K6-9','','','','OT',636,'',170 from dual union all
select 36410,'1974-Y62','1974-02-14 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',87 from dual union all
select 36411,'1974-Y63','1974-02-15 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 36412,'1974-Y64','1974-02-15 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 36413,'1974-Y65','1974-02-15 18:33:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',84 from dual union all
select 36414,'1974-Y66','1974-02-18 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',81 from dual union all
select 36415,'1974-Y67','1974-02-18 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',90 from dual union all
select 36416,'1974-Y68','1974-02-19 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',880,'KORO',95 from dual union all
select 36417,'1974-Y69','1974-02-20 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',95 from dual union all
select 36418,'1974-Y70','1974-02-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 36419,'1974-Y71','1974-02-20 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.174','TLERS-634','','Aeron',1467,'',86 from dual union all
select 3642,'1955-S143','1955-06-20 00:00:00','military missile','success',992,'S3-20','','','','OT',636,'',170 from dual union all
select 36420,'1974-Y72','1974-02-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 36421,'1974-Y73','1974-02-20 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 36422,'1974-Y74','1974-02-22 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',97 from dual union all
select 36423,'1974-Y75','1974-02-23 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 36424,'1974-Y76','1974-02-24 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',91 from dual union all
select 36425,'1974-Y77','1974-02-26 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',85 from dual union all
select 36426,'1974-Y78','1974-02-26 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 36427,'1974-Y79','1974-02-27 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 36428,'1974-Y80','1974-02-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 36429,'1974-Y81','1974-02-27 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',90 from dual union all
select 3643,'1955-S144','1955-06-28 00:00:00','military missile','success',992,'S6-20','','','','OT',636,'',170 from dual union all
select 36430,'1974-Y82','1974-02-27 16:00:00','suborbital rocket','unknown',725,'','ISRO-8.175','TERLS-641','','Aeron',1467,'',89 from dual union all
select 36431,'1974-Y83','1974-02-27 22:10:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',84 from dual union all
select 36432,'1974-Y84','1974-02-28 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 36433,'1974-Y85','1974-02-28 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',92 from dual union all
select 36434,'1974-Y86','1974-03-01 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1369,'SHIR',89 from dual union all
select 36435,'1974-Y87','1974-03-02 05:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 36436,'1974-Y88','1974-03-02 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',93 from dual union all
select 36437,'1974-Y89','1974-03-02 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 36438,'1974-Y90','1974-03-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 36439,'1974-Y91','1974-03-04 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 3644,'1955-S145','1955-07-15 00:00:00','military missile','success',992,'S0-1','','','','OT',636,'',170 from dual union all
select 36440,'1974-Y92','1974-03-05 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1389,'SHOK',90 from dual union all
select 36441,'1974-Y93','1974-03-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 36442,'1974-Y94','1974-03-05 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 36443,'1974-Y95','1974-03-05 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 36444,'1974-Y96','1974-03-06 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 36445,'1974-Y97','1974-03-06 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 36446,'1974-Y98','1974-03-06 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 36447,'1974-Y99','1974-03-06 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.176','TERLS-646','','Aeron',1467,'',83 from dual union all
select 36448,'1975-001','1975-01-10 21:43:37','orbital','success',1223,'Kh15000-022','Soyuz-17','Soyuz 7K-T No. 38','','Sat',480,'',null from dual union all
select 36449,'1975-002','1975-01-17 09:00:00','orbital','success',1508,'Kh76001- 33','Kosmos-702','Zenit-2M','','Sat',562,'',null from dual union all
select 3645,'1955-S146','1955-08-03 00:00:00','military missile','success',992,'S7-20','','','','OT',636,'',170 from dual union all
select 36450,'1975-003','1975-01-21 11:04:57','orbital','success',678,'45753-167','Kosmos-703','DS-P1-Yu No. 70','','Sat',460,'',null from dual union all
select 36451,'1975-004','1975-01-22 17:55:52','orbital','success',383,'598/D107','Landsat 2','Landsat B','','Sat',1724,'',null from dual union all
select 36452,'1975-005','1975-01-23 11:00:00','orbital','success',1508,'78031  -775','Kosmos-704','Zenit-4MK','','Sat',473,'',null from dual union all
select 36453,'1975-006','1975-01-28 12:05:01','orbital','success',678,'64643-171','Kosmos-705','DS-P1-Yu No. 75','','Sat',460,'',null from dual union all
select 36454,'1975-007','1975-01-30 15:02:00','orbital','success',809,'78031  -574','Kosmos-706','Oko','','Sat',473,'',null from dual union all
select 36455,'1975-008','1975-02-05 13:15:00','orbital','success',680,'53753-212','Kosmos-707','Tselina-OM','','Sat',456,'',null from dual union all
select 36456,'1975-009','1975-02-06 04:49:00','orbital','success',809,'76043  -532','Molniya-2','Molniya-2','','Sat',473,'',null from dual union all
select 36457,'1975-010','1975-02-06 16:35:00','orbital','success',433,'No. 1','Starlette','Starlette','','Sat',343,'',null from dual union all
select 36458,'1975-011','1975-02-06 22:04:00','orbital','success',386,'593/D108','SMS 2','SMS B','','Sat',234,'',null from dual;
insert into LAUNCH
select 36459,'1975-012','1975-02-12 03:30:00','orbital','success',680,'53749-166','Kosmos-708','Geoid','','Sat',456,'',null from dual union all
select 3646,'1955-S147','1955-08-06 00:00:00','military missile','success',992,'406','Generator-2','','','OT',636,'',170 from dual union all
select 36460,'1975-013','1975-02-12 14:30:00','orbital','success',1508,'78031  -777 (177)','Kosmos-709','Zenit-4MK','','Sat',473,'',null from dual union all
select 36461,'1975-014','1975-02-24 05:25:00','orbital','success',816,'M-3C-2','Taiyo','SRATS','','Sat',835,'',null from dual union all
select 36462,'1975-015','1975-02-26 09:00:00','orbital','success',1508,'78031  -778','Kosmos-710','Zenit-4MK','','Sat',562,'',null from dual union all
select 36463,'1975-016','1975-02-28 14:01:59','orbital','success',680,'53749-157','Kosmos-711/718','Strela-1M','','Sat',457,'',null from dual union all
select 36464,'1975-017','1975-03-10 04:41:00','orbital','success',1405,'34B-1 (3B-50)','OPS 2439','Jumpseat 4','','Sat',1778,'',null from dual union all
select 36465,'1975-018','1975-03-12 08:55:00','orbital','success',1508,'77019  -750','Kosmos-719','Zenit-4MK','','Sat',562,'',null from dual union all
select 36466,'1975-019','1975-03-21 06:50:00','orbital','success',1238,'76041  -851','Kosmos-720','Zenit-4MT','','Sat',474,'',null from dual union all
select 36467,'1975-020','1975-03-26 08:50:00','orbital','success',1508,'76041  -199','Kosmos-721','Zenit-2M','','Sat',473,'',null from dual union all
select 36468,'1975-021','1975-03-27 08:00:00','orbital','success',1508,'78031  -781','Kosmos-722','Zenit-4MK','','Sat',562,'',null from dual union all
select 36469,'1975-022','1975-03-27 14:30:00','orbital','success',680,'53721-268','Interkosmos-13','DS-U2-IK No. 5','','Sat',456,'',null from dual union all
select 3647,'1955-S148','1955-08-08 00:00:00','military missile','success',992,'S0-5','UGCh','','','OT',636,'',170 from dual union all
select 36470,'1975-023','1975-04-01 12:30:00','orbital','success',1510,'76041  -366','Meteor','Meteor-M  No. 31','','Sat',473,'',null from dual union all
select 36471,'1975-024','1975-04-02 11:00:00','orbital','success',1446,'67049-158','Kosmos-723','US-A reactor','','Sat',591,'',null from dual union all
select 36472,'1975-025','1975-04-07 11:00:00','orbital','success',1446,'45023-413-02','Kosmos-724','US-A reactor','','Sat',591,'',null from dual union all
select 36473,'1975-026','1975-04-08 18:29:56','orbital','success',678,'64643-172','Kosmos-725','DS-P1-Yu No. 77','','Sat',460,'',null from dual union all
select 36474,'1975-027','1975-04-09 23:58:02','orbital','success',375,'584/D109','Geos 3','Geos C','','Sat',1724,'',null from dual union all
select 36475,'1975-028','1975-04-11 07:57:14','orbital','success',680,'65033-203','Kosmos-726','Tsiklon-B','','Sat',456,'',null from dual union all
select 36476,'1975-029','1975-04-14 17:52:59','orbital','success',809,'76043  -531','Molniya-3','Molniya-3 No. 12','','Sat',473,'',null from dual union all
select 36477,'1975-030','1975-04-16 08:00:01','orbital','success',1238,'76041  -849','Kosmos-727','Zenit-4MK','','Sat',562,'',null from dual union all
select 36478,'1975-031','1975-04-18 10:00:00','orbital','success',1508,'77016  -212','Kosmos-728','Zenit-2M','','Sat',473,'',null from dual union all
select 36479,'1975-032','1975-04-18 16:45:00','orbital','success',1403,'24B-14 (3B-48)','KH-8A','GAMBIT-3 Mission 4344','','Sat',1778,'',null from dual union all
select 3648,'1955-S149','1955-08-24 00:00:00','military missile','success',992,'S0-8','UGCh','','','OT',636,'',170 from dual union all
select 36480,'1975-033','1975-04-19 07:58:55','orbital','success',680,'53731-279','Aryabhata','ISS','','Sat',623,'',null from dual union all
select 36481,'1975-034','1975-04-22 21:10:37','orbital','success',680,'53731-274','Kosmos-729','Zaliv','','Sat',456,'',null from dual union all
select 36482,'1975-035','1975-04-24 08:04:59','orbital','success',1508,'76041  -788','Kosmos-730','Zenit-4MK','','Sat',474,'',null from dual union all
select 36483,'1975-036','1975-04-29 10:23:59','orbital','success',809,'M 15000-064','Molniya-1','Molniya-1','','Sat',473,'',null from dual union all
select 36484,'1975-037','1975-05-07 22:45:01','orbital','success',1134,'S194C','Explorer 53','SAS C','','Sat',1412,'',null from dual union all
select 36485,'1975-038','1975-05-07 23:35:26','orbital','success',385,'596/D110','Anik A3','Anik A3','','Sat',234,'',null from dual union all
select 36486,'1975-039','1975-05-17 10:32:00','orbital','success',433,'No. 2','D-5A/D-5B','D-5A','','Sat',343,'',null from dual union all
select 36487,'1975-040','1975-05-20 14:03:48','orbital','success',1425,'23C-7 (3C-25)','DSCS II 5/6','DSCS II B-5','','Sat',274,'',null from dual union all
select 36488,'1975-041','1975-05-21 06:59:32','orbital','success',1508,'78031  -195','Kosmos-731','Zenit-2M','','Sat',562,'',null from dual union all
select 36489,'1975-042','1975-05-22 22:04:00','orbital','success',180,'AC-35','Intelsat 4 F-1','INTELSAT IV F1','','Sat',268,'',null from dual union all
select 3649,'1955-S15','1955-02-05 00:00:00','suborbital rocket','success',988,'2','1RE','','','Test/Bio',636,'',100 from dual union all
select 36490,'1975-043','1975-05-24 03:22:41','orbital','success',1365,'197/BIIA-24','DMSP 10533','DMSP Block 5C F-2','','Sat',1720,'',null from dual union all
select 36491,'1975-044','1975-05-24 14:58:10','orbital','success',1223,'F 15000-033','Soyuz-18','Soyuz 7K-T No. 40','','Sat',480,'',null from dual union all
select 36492,'1975-045','1975-05-28 00:25:01','orbital','success',680,'53731-272','Kosmos-732/739','Strela-1M','','Sat',456,'',null from dual union all
select 36493,'1975-046','1975-05-28 07:29:59','orbital','success',1508,'77016  -655','Kosmos-740','Zenit-4MK','','Sat',562,'',null from dual union all
select 36494,'1975-047','1975-05-30 06:45:00','orbital','success',1508,'77016  -215','Kosmos-741','Gektor-Priroda','','Sat',474,'',null from dual union all
select 36495,'1975-048','1975-06-03 13:20:48','orbital','success',1508,'76029  -756','Kosmos-742','Zenit-4MK','','Sat',474,'',null from dual union all
select 36496,'1975-049','1975-06-05 01:37:59','orbital','success',809,'76041  -579','Molniya-1','Molniya-1 No. 24','','Sat',473,'',null from dual union all
select 36497,'1975-050','1975-06-08 02:38:00','deep space','success',953,'286-01','Venera-9','4V-1 No. 660','','Sat',588,'',null from dual union all
select 36498,'1975-051','1975-06-08 18:30:00','orbital','success',1426,'23D-10','HEXAGON SV10','HEXAGON SV-10','','Sat',1777,'',null from dual union all
select 36499,'1975-052','1975-06-12 08:12:01','orbital','success',383,'595/D111','Nimbus 6','Nimbus F','','Sat',1724,'',null from dual union all
select 365,'1944-S332','1944-10-30 00:00:00','military missile','success',1470,'','London/West Ham','','','Weapon',1627,'',90 from dual union all
select 3650,'1955-S150','1955-08-27 00:00:00','military missile','success',992,'S0-7','UGCh','','','OT',636,'',170 from dual union all
select 36500,'1975-053','1975-06-12 12:30:27','orbital','success',1238,'76041  -850','Kosmos-743','Zenit-4MK','','Sat',474,'',null from dual union all
select 36501,'1975-054','1975-06-14 03:00:31','deep space','success',953,'285-02','Venera-10','4V-1 No. 661','','Sat',588,'',null from dual union all
select 36502,'1975-055','1975-06-18 09:00:00','orbital','success',178,'5506A','AFP-827','CANYON 6','','Sat',228,'',null from dual union all
select 36503,'1975-056','1975-06-20 06:54:00','orbital','success',1510,'76043  -324','Kosmos-744','Tselina-D','','Sat',473,'',null from dual union all
select 36504,'1975-057','1975-06-21 11:43:00','orbital','success',378,'586/D112','OSO 8','OSO I','','Sat',234,'',null from dual union all
select 36505,'1975-058','1975-06-24 12:04:58','orbital','success',678,'45753-173','Kosmos-745','DS-P1-Yu No. 76','','Sat',460,'',null from dual union all
select 36506,'1975-059','1975-06-25 12:59:58','orbital','success',1508,'76041  -785','Kosmos-746','Zenit-4MK','','Sat',474,'',null from dual union all
select 36507,'1975-060','1975-06-27 13:00:00','orbital','success',1508,'76041  -198','Kosmos-747','Zenit-2M','','Sat',473,'',null from dual union all
select 36508,'1975-061','1975-07-03 13:40:00','orbital','success',1508,'76041  -776','Kosmos-748','Zenit-4MK','','Sat',473,'',null from dual union all
select 36509,'1975-062','1975-07-04 00:56:07','orbital','success',680,'53721-259','Kosmos-749','Tselina-OM','','Sat',456,'',null from dual union all
select 3651,'1955-S151','1955-08-30 00:00:00','military missile','success',992,'S8-20','','','','OT',636,'',170 from dual union all
select 36510,'1975-063','1975-07-08 05:05:00','orbital','success',809,'78019  -552','Molniya-2','Molniya-2','','Sat',473,'',null from dual union all
select 36511,'1975-064','1975-07-11 04:15:00','orbital','success',1510,'77016  -383','Meteor-2','Meteor-2 No. 1','','Sat',473,'',null from dual union all
select 36512,'1975-065','1975-07-15 12:20:00','orbital','success',1238,'F 15000-017','Soyuz-19','Soyuz 7K-TM No. 75','','Sat',480,'',null from dual union all
select 36513,'1975-066','1975-07-15 19:50:01','orbital','success',1121,'SA-210','ASTP','Apollo CM-111','','Sat',922,'LUT1',null from dual union all
select 36514,'1975-067','1975-07-17 09:10:01','orbital','success',678,'64653-175','Kosmos-750','DS-P1-I No. 15','','Sat',460,'',null from dual union all
select 36515,'1975-068','1975-07-23 13:00:00','orbital','success',1508,'76041  -200','Kosmos-751','Zenit-2M','','Sat',474,'',null from dual union all
select 36516,'1975-069','1975-07-24 19:00:01','orbital','success',680,'53721-254','Kosmos-752','Vektor','','Sat',456,'',null from dual union all
select 36517,'1975-070','1975-07-26 13:30:00','orbital','success',492,'701-05','JSSW','JSSW','','Sat',810,'',null from dual union all
select 36518,'1975-071','1975-07-31 13:00:00','orbital','success',1508,'76041  -786','Kosmos-753','Zenit-4MK','','Sat',474,'',null from dual union all
select 36519,'1975-072','1975-08-09 01:50:00','orbital','success',384,'602/D113','COS-B','COS-B','','Sat',1724,'',null from dual union all
select 3652,'1955-S152','1955-08-31 00:00:00','military missile','success',992,'S9-20','','','','OT',636,'',170 from dual union all
select 36520,'1975-073','1975-08-13 07:21:04','orbital','success',1508,'78031  -782','Kosmos-754','Zenit-4MK','','Sat',562,'',null from dual union all
select 36521,'1975-074','1975-08-14 13:29:21','orbital','success',680,'53731-288','Kosmos-755','Tsiklon-B','','Sat',456,'',null from dual union all
select 36522,'1975-075','1975-08-20 21:22:00','deep space','success',1427,'23E-4/TC-4','Viking 1','Viking Orbiter 1','','Sat',275,'',null from dual union all
select 36523,'1975-076','1975-08-22 02:10:59','orbital','success',1510,'76031  -371','Kosmos-756','Tselina-D','','Sat',473,'',null from dual union all
select 36524,'1975-077','1975-08-27 01:42:00','orbital','success',386,'594/D114','Symphonie 2','Symphonie MV2','','Sat',233,'',null from dual union all
select 36525,'1975-078','1975-08-27 14:45:00','orbital','success',1508,'76041  -787','Kosmos-757','Zenit-4MK','','Sat',473,'',null from dual union all
select 36526,'1975-079','1975-09-02 13:09:00','orbital','success',809,'78031  -571','Molniya-1','Molniya-1','','Sat',473,'',null from dual union all
select 36527,'1975-080','1975-09-05 14:49:59','orbital','success',1238,'A 15000-016','Kosmos-758','Yantar''-2K No. 3','','Sat',474,'',null from dual union all
select 36528,'1975-081','1975-09-09 00:19:06','orbital','success',809,'76029  -560','Molniya-2','Molniya-2','','Sat',473,'',null from dual union all
select 36529,'1975-082','1975-09-09 05:30:00','orbital','success',822,'N-1(F)','Kiku-1','ETS 1','','Sat',1474,'',null from dual union all
select 3653,'1955-S153','1955-09-29 00:00:00','military missile','success',992,'S11-20','','','','OT',636,'',170 from dual union all
select 36530,'1975-083','1975-09-09 18:39:00','deep space','success',1427,'23E-3/TC-3','Viking 2','Viking Orbiter 2','','Sat',275,'',null from dual union all
select 36531,'1975-084','1975-09-12 05:30:00','orbital','success',1238,'77016  -662','Kosmos-759','Zenit-4MT','','Sat',474,'',null from dual union all
select 36532,'1975-085','1975-09-16 09:00:00','orbital','success',1508,'76041  -789','Kosmos-760','Zenit-4MK','','Sat',562,'',null from dual union all
select 36533,'1975-086','1975-09-17 07:10:00','orbital','success',680,'53746-311','Kosmos-761/768','Strela-1M','','Sat',456,'',null from dual union all
select 36534,'1975-087','1975-09-18 00:12:00','orbital','success',1510,'76029  -354','Meteor','Meteor-M  No. 28','','Sat',473,'',null from dual union all
select 36535,'1975-088','1975-09-23 09:59:59','orbital','success',1508,'77016  -214','Kosmos-769','Zenit-2M','','Sat',473,'',null from dual union all
select 36536,'1975-089','1975-09-24 12:00:00','orbital','success',680,'53721-260','Kosmos-770','Geoid','','Sat',456,'',null from dual union all
select 36537,'1975-090','1975-09-25 09:49:59','orbital','success',1238,'77016  -011','Kosmos-771','Fram','','Sat',474,'',null from dual union all
select 36538,'1975-091','1975-09-26 00:17:00','orbital','success',180,'AC-36','Intelsat 4A F-1','INTELSAT IVA F1','','Sat',269,'',null from dual union all
select 36539,'1975-092','1975-09-27 08:37:00','orbital','success',433,'No. 3','D-2B','D-2B','','Sat',343,'',null from dual union all
select 3654,'1955-S154','1955-10-01 00:00:00','military missile','success',992,'S10-20','','','','OT',636,'',170 from dual union all
select 36540,'1975-093','1975-09-29 04:15:00','orbital','success',1238,'F 15000-021','Kosmos-772','Soyuz 7K-S No. 2L','','Sat',480,'',null from dual union all
select 36541,'1975-094','1975-09-30 18:37:59','orbital','success',680,'53721-258','Kosmos-773','Strela-2M','','Sat',456,'',null from dual union all
select 36542,'1975-095','1975-10-01 08:30:00','orbital','success',1508,'76041  -790','Kosmos-774','Zenit-4MK','','Sat',562,'',null from dual union all
select 36543,'1975-096','1975-10-06 09:00:50','orbital','success',383,'600/D115','Explorer 54','AE-D','','Sat',1724,'',null from dual union all
select 36544,'1975-097','1975-10-08 00:30:00','orbital','success',956,'286-02','Kosmos-775','Oko?','','Sat',587,'',null from dual union all
select 36545,'1975-098','1975-10-09 19:15:00','orbital','success',1403,'24B-13 (3B-47)','KH-8A','GAMBIT-3 Mission 4345','','Sat',1778,'',null from dual union all
select 36546,'1975-099','1975-10-12 06:39:36','orbital','success',1132,'S195C','TIP 2','TIP 2','','Sat',1779,'',null from dual union all
select 36547,'1975-100','1975-10-16 22:40:00','orbital','success',386,'597/D116','GOES 1','GOES A','','Sat',234,'',null from dual union all
select 36548,'1975-101','1975-10-17 14:29:59','orbital','success',1508,'77041  -216','Kosmos-776','Zenit-2M','','Sat',473,'',null from dual;
insert into LAUNCH
select 36549,'1975-102','1975-10-29 11:00:00','orbital','success',1446,'51014-420L','Kosmos-777','US-P','','Sat',589,'',null from dual union all
select 3655,'1955-S155','1955-10-31 00:00:00','military missile','success',992,'S12-20','','','','OT',636,'',170 from dual union all
select 36550,'1975-103','1975-11-04 10:12:53','orbital','success',680,'53731-275','Kosmos-778','Tsiklon-B','','Sat',456,'',null from dual union all
select 36551,'1975-104','1975-11-04 15:19:58','orbital','success',1508,'76041  -791','Kosmos-779','Zenit-4MK','','Sat',474,'',null from dual union all
select 36552,'1975-105','1975-11-14 19:13:58','orbital','success',809,'77016  -588','Molniya-3','Molniya-3 No. 13','','Sat',474,'',null from dual union all
select 36553,'1975-106','1975-11-17 14:36:37','orbital','success',1238,'K 15000- 15','Soyuz-20','Soyuz 7K-T(A9) No. 64','','Sat',480,'',null from dual union all
select 36554,'1975-107','1975-11-20 02:06:48','orbital','success',383,'604/D117','Explorer 55','AE-E','','Sat',234,'',null from dual union all
select 36555,'1975-108','1975-11-21 09:20:00','orbital','success',1508,'77016  -213','Kosmos-780','Zenit-2M','','Sat',562,'',null from dual union all
select 36556,'1975-109','1975-11-21 17:11:00','orbital','success',680,'53731-271','Kosmos-781','Tselina-OK','','Sat',456,'',null from dual union all
select 36557,'1975-110','1975-11-25 17:00:00','orbital','success',1238,'F 15000-022','Kosmos-782','Bion No. 3','','Sat',474,'',null from dual union all
select 36558,'1975-111','1975-11-26 03:27:52','orbital','success',301,'CZ2C-1   (4)','FSW','FSW No. 2','','Sat',810,'',null from dual union all
select 36559,'1975-112','1975-11-28 00:10:08','orbital','success',680,'53721-252','Kosmos-783','Strela-2M','','Sat',456,'',null from dual union all
select 3656,'1955-S156','1955-11-19 00:00:00','military missile','success',992,'S13-20','','','','OT',636,'',170 from dual union all
select 36560,'1975-113','1975-12-03 10:00:00','orbital','success',1508,'76041  -201','Kosmos-784','Zenit-2M','','Sat',474,'',null from dual union all
select 36561,'1975-114','1975-12-04 20:30:00','orbital','success',1426,'23D-13','HEXAGON SV11','HEXAGON SV-11','','Sat',1777,'',null from dual union all
select 36562,'1975-115','1975-12-11 17:00:01','orbital','success',680,'53746-308','Interkosmos-14','DS-U2-IK No. 6','','Sat',456,'',null from dual union all
select 36563,'1975-116','1975-12-12 12:45:00','orbital','success',1446,'51011-110','Kosmos-785','US-A reactor','','Sat',589,'',null from dual union all
select 36564,'1975-117','1975-12-13 01:56:14','orbital','success',390,'607/D118','Satcom 1','RCA-A','','Sat',233,'',null from dual union all
select 36565,'1975-118','1975-12-14 05:15:00','orbital','success',1425,'23C-11 (3C-29)','DSP 5','DSP 8','','Sat',274,'',null from dual union all
select 36566,'1975-119','1975-12-16 09:21:00','orbital','success',492,'701-06','JSSW 701-06','JSSW','','Sat',810,'',null from dual union all
select 36567,'1975-120','1975-12-16 09:50:00','orbital','success',1508,'76041  -792','Kosmos-786','Zenit-4MK','','Sat',562,'',null from dual union all
select 36568,'1975-121','1975-12-17 11:06:01','orbital','success',809,'78031  -567','Molniya-2','Molniya-2','','Sat',474,'',null from dual union all
select 36569,'1975-122','1975-12-22 02:08:00','orbital','success',807,'76041  -575','Prognoz-4','SO-M No. 504','','Sat',562,'',null from dual union all
select 3657,'1955-S157','1955-11-23 00:00:00','military missile','success',992,'S14-20','','','','OT',636,'',170 from dual union all
select 36570,'1975-123','1975-12-22 13:00:00','orbital','success',956,'288-01','Raduga','Gran'' No. 11L','','Sat',588,'',null from dual union all
select 36571,'1975-124','1975-12-25 19:00:00','orbital','success',1510,'76041  -368','Meteor','Meteor No. 38','','Sat',473,'',null from dual union all
select 36572,'1975-125','1975-12-27 10:22:02','orbital','success',809,'F 15000-142','Molniya-3','Molniya-3 No. 15','','Sat',474,'',null from dual union all
select 36573,'1975-A01','1975-01-09 03:01:00','atmospheric rocket','success',1309,'Q3-7681','','','','Test',1821,'',4 from dual union all
select 36574,'1975-A02','1975-01-09 03:00:00','atmospheric rocket','success',1309,'Q3-7680','','','','Test',1821,'',4 from dual union all
select 36575,'1975-A03','1975-01-08 09:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',48 from dual union all
select 36576,'1975-A04','1975-01-21 16:46:00','military missile','success',999,'N22502-03','8K67 UBP-50 Svinets-5','GCh 8F674','','Test',519,'',1000 from dual union all
select 36577,'1975-A05','1975-01-30 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',49 from dual union all
select 36578,'1975-A06','1975-02-04 18:35:00','atmospheric rocket','success',908,'FLAME 1','FLAME','','','Test',1821,'F-4',17 from dual union all
select 36579,'1975-A07','1975-02-07 15:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',49 from dual union all
select 3658,'1955-S158','1955-11-23 00:00:00','military missile','success',992,'415','Generator-2','','','OT',636,'',170 from dual union all
select 36580,'1975-A08','1975-02-24 19:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',48 from dual union all
select 36581,'1975-A09','1975-02-27 00:00:00','atmospheric rocket','success',908,'FLAME 2','FLAME','','','Test',1490,'F-4',17 from dual union all
select 36582,'1975-A10','1975-03-03 18:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',48 from dual union all
select 36583,'1975-A11','1975-03-07 21:05:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',4 from dual union all
select 36584,'1975-A12','1975-03-18 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',45 from dual union all
select 36585,'1975-A13','1975-03-20 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 36586,'1975-A14','1975-03-20 17:00:00','suborbital rocket','success',1061,'','Starute,STS-','','','Meteo',1895,'',33 from dual union all
select 36587,'1975-A15','1975-03-25 11:01:00','military missile','success',1099,'','','','','Test',293,'B-52',10 from dual union all
select 36588,'1975-A16','1975-03-27 00:00:00','atmospheric rocket','success',835,'','','','','Test',1490,'',10 from dual union all
select 36589,'1975-A17','1975-03-28 21:04:00','atmospheric rocket','success',908,'FLAME 3','FLAME','','','Test',1821,'F-4',18 from dual union all
select 3659,'1955-S159','1955-12-16 00:00:00','military missile','success',992,'S15-20','','','','OT',636,'',170 from dual union all
select 36590,'1975-A18','1975-04-04 12:00:00','suborbital rocket','failure',867,'NASA 18.172GE','','','','Plasma',1047,'',10 from dual union all
select 36591,'1975-A19','1975-04-10 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 36592,'1975-A20','1975-04-11 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',47 from dual union all
select 36593,'1975-A21','1975-04-15 12:15:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',4 from dual union all
select 36594,'1975-A22','1975-04-18 00:00:00','atmospheric rocket','failure',835,'','','','','Test',1490,'',0 from dual union all
select 36595,'1975-A23','1975-04-18 00:00:00','atmospheric rocket','failure',908,'FLAME 4','FLAME','','','Test',1490,'F-4',17 from dual union all
select 36596,'1975-A24','1975-04-21 12:35:00','military missile','success',1099,'','','','','Test',293,'FB-111',10 from dual union all
select 36597,'1975-A25','1975-04-30 00:00:00','military missile','success',1252,'PAVT-6','SAFEGUARD M2-548','','','Interceptor',868,'',4 from dual union all
select 36598,'1975-A26','1975-05-07 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',45 from dual union all
select 36599,'1975-A27','1975-05-12 11:24:00','military missile','success',1099,'','','','','Test',293,'B-52',10 from dual union all
select 366,'1944-S333','1944-10-30 00:00:00','military missile','success',1470,'','London/Wapping','','','Weapon',1627,'',90 from dual union all
select 3660,'1955-S16','1955-02-07 00:00:00','military missile','success',1009,'K3-8','8A62 ZI','','','Test',636,'',300 from dual union all
select 36600,'1975-A28','1975-05-12 11:54:00','military missile','success',1099,'','','','','Test',293,'B-52',10 from dual union all
select 36601,'1975-A29','1975-05-22 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 36602,'1975-A30','1975-05-23 11:35:00','military missile','success',1099,'','','','','Test',293,'FB-111',10 from dual union all
select 36603,'1975-A31','1975-05-23 11:58:00','military missile','success',1099,'','','','','Test',293,'FB-111',10 from dual union all
select 36604,'1975-A32','1975-06-03 00:00:00','atmospheric rocket','success',908,'FLAME 5','FLAME','Tungsten nosetip test','','Test',1490,'F-4',18 from dual union all
select 36605,'1975-A33','1975-06-03 00:00:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',5 from dual union all
select 36606,'1975-A34','1975-06-03 16:14:00','suborbital rocket','failure',867,'NASA 18.149GE','Antarqui Fields','','','Plasma',323,'',20 from dual union all
select 36607,'1975-A35','1975-06-04 00:00:00','atmospheric rocket','success',542,'','','','','Test',1490,'',10 from dual union all
select 36608,'1975-A36','1975-06-04 00:00:00','atmospheric rocket','failure',908,'FLAME 6','FLAME','','','Test',1490,'F-4',17 from dual union all
select 36609,'1975-A37','1975-06-09 18:42:00','suborbital rocket','failure',37,'NASA 13.112UA','','','','Aeronomy',1895,'',29 from dual union all
select 3661,'1955-S160','1955-12-17 00:00:00','military missile','success',992,'S16-20','','','','OT',636,'',170 from dual union all
select 36610,'1975-A38','1975-06-13 00:19:00','suborbital rocket','failure',1161,'S75-1','PF-SGT-100 Ballistic','','','Test',1047,'',8 from dual union all
select 36611,'1975-A39','1975-06-16 00:00:00','military missile','failure',1452,'63046-41-382T','15A30 UBP-2 ZP','GCh 15B103','','OT',508,'',0 from dual union all
select 36612,'1975-A40','1975-06-24 12:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',41 from dual union all
select 36613,'1975-A41','1975-07-02 14:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',47 from dual union all
select 36614,'1975-A42','1975-07-04 00:00:00','military missile','failure',1452,'51121-72-027','15A30 UBP-3 ZP','GCh 15B103','','OT',532,'',0 from dual union all
select 36615,'1975-A43','1975-07-23 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 36616,'1975-A44','1975-07-25 18:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',39 from dual union all
select 36617,'1975-A45','1975-08-05 16:20:00','military missile','failure',917,'FOT-42','P-313 75 Cape 06','','','OT',231,'',40 from dual union all
select 36618,'1975-A46','1975-08-12 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',40 from dual union all
select 36619,'1975-A47','1975-08-25 15:54:00','military missile','failure',917,'FOT-45','P-316 75 Cape 09','','','OT',231,'',40 from dual union all
select 3662,'1955-S161','1955-12-24 00:00:00','military missile','success',992,'S18-10','','','','OT',636,'',170 from dual union all
select 36620,'1975-A48','1975-08-26 18:49:00','military missile','failure',917,'FOT-48','P-319 75 Cape 12','','','OT',231,'',40 from dual union all
select 36621,'1975-A49','1975-08-27 00:00:00','atmospheric rocket','success',542,'','','','','Test',1490,'',10 from dual union all
select 36622,'1975-A50','1975-09-03 16:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',42 from dual union all
select 36623,'1975-A51','1975-09-05 02:35:00','military missile','failure',1450,'64146-43-155P','15A20 UBP-25 ZP','GCh 15F202','','OT',536,'',100 from dual union all
select 36624,'1975-A52','1975-09-05 17:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',45 from dual union all
select 36625,'1975-A53','1975-09-29 20:30:00','military missile','failure',1450,'64146-43-156P','15A20 UBP-26 ZP','GCh 15F202','','OT',536,'',20 from dual union all
select 36626,'1975-A54','1975-10-01 13:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',45 from dual union all
select 36627,'1975-A55','1975-10-08 23:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',49 from dual union all
select 36628,'1975-A56','1975-10-10 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',46 from dual union all
select 36629,'1975-A57','1975-10-13 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',48 from dual union all
select 3663,'1955-S162','1955-12-27 00:00:00','military missile','success',992,'S17-20','','','','OT',636,'',170 from dual union all
select 36630,'1975-A58','1975-10-29 00:00:00','atmospheric rocket','success',835,'','','','','Test',1490,'',10 from dual union all
select 36631,'1975-A59','1975-10-31 00:00:00','atmospheric rocket','success',835,'','','','','Test',1490,'',10 from dual union all
select 36632,'1975-A60','1975-11-04 17:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',44 from dual union all
select 36633,'1975-A61','1975-11-05 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',41 from dual union all
select 36634,'1975-A62','1975-11-13 13:16:00','military missile','success',1099,'','','','','Test',293,'B-52',10 from dual union all
select 36635,'1975-A63','1975-11-14 12:30:00','military missile','success',1099,'','','','','Test',293,'B-52',10 from dual union all
select 36636,'1975-A64','1975-11-18 16:30:00','suborbital rocket','failure',839,'NASA 10.409GM','GSFC Grenade 281 W','','','Aeronomy',1821,'',21 from dual union all
select 36637,'1975-A65','1975-11-24 13:15:00','military missile','success',1099,'','','','','Test',293,'B-52',10 from dual union all
select 36638,'1975-A66','1975-12-01 16:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',49 from dual;
insert into LAUNCH
select 36639,'1975-A67','1975-12-01 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',19 from dual union all
select 3664,'1955-S163','1955-12-28 00:00:00','military missile','success',992,'S19-20','','','','OT',636,'',170 from dual union all
select 36640,'1975-A68','1975-12-01 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',28 from dual union all
select 36641,'1975-A69','1975-12-03 00:00:00','atmospheric rocket','success',1309,'','','','','Test',1821,'',5 from dual union all
select 36642,'1975-A70','1975-12-03 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',38 from dual union all
select 36643,'1975-A71','1975-12-05 22:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',49 from dual union all
select 36644,'1975-A72','1975-12-08 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',32 from dual union all
select 36645,'1975-A73','1975-12-08 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',33 from dual union all
select 36646,'1975-A74','1975-12-08 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',33 from dual union all
select 36647,'1975-A75','1975-12-16 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 36648,'1975-A76','1975-12-20 16:24:00','suborbital rocket','unknown',1284,'FLIGHT 120','','','','Aeron',35,'',49 from dual union all
select 36649,'1975-A77','1975-12-31 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',0 from dual union all
select 3665,'1955-S164','1955-12-30 00:00:00','military missile','success',992,'S19-19','','','','OT',636,'',170 from dual union all
select 36650,'1975-A78','1975-12-31 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',48 from dual union all
select 36651,'1975-A79','1975-07-08 06:00:00','suborbital rocket','success',773,'','Gerdien probe','','','Aeron',1467,'',40 from dual union all
select 36652,'1975-A80','1975-05-12 00:00:00','suborbital rocket','failure',229,'11-75','12cm Sphere','','','Test',1895,'',0 from dual union all
select 36653,'1975-A81','1975-01-01 00:00:00','military missile','success',1429,'','','','','Test',636,'',20 from dual union all
select 36654,'1975-F01','1975-02-20 23:35:00','orbital','failure',180,'AC-33','Intelsat 4 F-6','INTELSAT IV F6','','Sat',268,'',200 from dual union all
select 36655,'1975-F02','1975-04-05 11:04:54','orbital','failure',1223,'Kh15000-023','Soyuz','Soyuz 7K-T No. 39','','Sat',480,'',192 from dual union all
select 36656,'1975-F03','1975-04-13 00:51:00','orbital','failure',162,'71F','STP P72-2','P72-2','','Sat',1776,'',0 from dual union all
select 36657,'1975-F04','1975-06-03 09:00:00','orbital','failure',680,'53721-257','','DS-U3-IK No. 5','','Sat',623,'',20 from dual union all
select 36658,'1975-F05','1975-10-16 04:04:56','orbital','failure',954,'287-02','','E-8-5M No. 412','','Sat',587,'',190 from dual union all
select 36659,'1975-F06','1975-12-06 03:35:01','orbital','failure',1134,'S196C','DAD A/B','DAD-A','','Sat',1779,'',100 from dual union all
select 3666,'1955-S165','1955-06-15 00:00:00','military missile','success',1013,'S1-10','8K51 LKI-8','','','Test',636,'',500 from dual union all
select 36660,'1975-F07','1975-12-19 14:00:00','orbital','failure',680,'53721-263','','Lira','','Sat',456,'',675 from dual union all
select 36661,'1975-M01','1975-04-04 00:00:00','military missile','success',77,'003','[Spurious]','','','Test',1343,'',50 from dual union all
select 36662,'1975-M02','1975-07-15 15:03:00','suborbital rocket','success',132,'NASA 23.02UE','','','','Plasma',1895,'',76 from dual union all
select 36663,'1975-M03','1975-01-09 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-66','','','Meteo',350,'',75 from dual union all
select 36664,'1975-M04','1975-02-26 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-70','','','Meteo',350,'',75 from dual union all
select 36665,'1975-M05','1975-06-11 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-78','','','Meteo',350,'',75 from dual union all
select 36666,'1975-M06','1975-09-29 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-82','','','Meteo',350,'',75 from dual union all
select 36667,'1975-M07','1975-12-18 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-84','','','Meteo',350,'',75 from dual union all
select 36668,'1975-M08','1975-01-30 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-67','','','Meteo',350,'',75 from dual union all
select 36669,'1975-M09','1975-02-13 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-69','','','Meteo',350,'',75 from dual union all
select 3667,'1955-S17','1955-02-07 00:00:00','military missile','success',970,'No 1020','','','','OT',636,'',100 from dual union all
select 36670,'1975-M10','1975-03-12 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-71','','','Meteo',350,'',75 from dual union all
select 36671,'1975-M11','1975-03-26 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-72','','','Meteo',350,'',75 from dual union all
select 36672,'1975-M12','1975-04-16 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-73','','','Meteo',350,'',75 from dual union all
select 36673,'1975-M13','1975-04-29 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-74','','','Meteo',350,'',75 from dual union all
select 36674,'1975-M14','1975-05-14 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-75','','','Meteo',350,'',75 from dual union all
select 36675,'1975-M15','1975-05-28 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-77','','','Meteo',350,'',75 from dual union all
select 36676,'1975-M16','1975-06-18 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-79','','','Meteo',350,'',75 from dual union all
select 36677,'1975-M17','1975-09-17 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-81','','','Meteo',350,'',75 from dual union all
select 36678,'1975-M18','1975-10-29 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-83','','','Meteo',350,'',75 from dual union all
select 36679,'1975-M19','1975-09-09 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-80','','','Meteo',350,'',75 from dual union all
select 3668,'1955-S18','1955-02-07 18:51:00','suborbital rocket','success',51,'USAF 52','AF/Utah D-layer','','','Ionos',640,'',120 from dual union all
select 36680,'1975-M20','1975-04-01 00:00:00','military missile','success',1162,'','','30FA B2 Training','','OT',984,'',50 from dual union all
select 36681,'1975-M21','1975-04-01 00:00:00','military missile','success',1162,'','','30FA B2 Training','','OT',984,'',50 from dual union all
select 36682,'1975-S01','1975-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 36683,'1975-S02','1975-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 36684,'1975-S03','1975-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 36685,'1975-S04','1975-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 36686,'1975-S05','1975-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 36687,'1975-S06','1975-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 36688,'1975-S07','1975-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 36689,'1975-S08','1975-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 3669,'1955-S19','1955-02-21 18:25:00','suborbital rocket','success',57,'NRL 29','Sunfollower/UVS','','','Solar',1869,'',115 from dual union all
select 36690,'1975-S09','1975-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 36691,'1975-S10','1975-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 36692,'1975-S100','1975-03-12 02:32:00','suborbital rocket','success',837,'NASA 14.416GA','','','','Aeronomy',428,'',138 from dual union all
select 36693,'1975-S101','1975-03-12 07:48:10','suborbital rocket','success',1164,'IC519.07-1B','ICECAP 75A','','','Auroral',1047,'',500 from dual union all
select 36694,'1975-S102','1975-03-13 17:02:00','suborbital rocket','success',837,'SSC S16/1','SSC S16/1 NOX','','','Aeronomy',408,'',106 from dual union all
select 36695,'1975-S103','1975-03-14 04:52:00','suborbital rocket','success',1211,'ESRO S98 (Astro 1/2)','DLR W-GR-67','','','Solar X?',1839,'',270 from dual union all
select 36696,'1975-S104','1975-03-15 04:10:00','suborbital rocket','success',44,'NASA 17.15UG','SAO XRT 1','','','XR Astron',1895,'',214 from dual union all
select 36697,'1975-S105','1975-03-15 07:00:57','suborbital rocket','success',41,'NASA 26.39UG','JHU UVT F8 (Saturn)','','','UV Astron',1895,'',261 from dual union all
select 36698,'1975-S106','1975-03-15 16:34:00','suborbital rocket','success',920,'P70T/C','ISRO-18.16','TERLS-739','','Ionospheric',1467,'',147 from dual union all
select 36699,'1975-S107','1975-03-16 00:00:00','suborbital rocket','success',920,'P184T/C','ISRO-18.18','TERLS-740','','Ionospheric',1467,'',120 from dual union all
select 367,'1944-S334','1944-10-31 00:51:00','military missile','success',1470,'','London/Hanwell','','','Weapon',1627,'',90 from dual union all
select 3670,'1955-S20','1955-04-29 00:00:00','military missile','failure',1013,'S1-5','8K51 LKI-4','','','Test',636,'',300 from dual union all
select 36700,'1975-S108','1975-03-18 00:00:00','military missile','success',1004,'35L','15A14 LKI-33','GCh 15F142','','Test',486,'',1000 from dual union all
select 36701,'1975-S109','1975-03-18 08:27:00','suborbital rocket','success',867,'NASA 18.165UE','','','','Plasma',1047,'',270 from dual union all
select 36702,'1975-S11','1975-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 36703,'1975-S110','1975-03-19 09:21:00','suborbital rocket','success',920,'P185T/C','ISRO-18.19','TERLS-741','','Ionospheric',1467,'',107 from dual union all
select 36704,'1975-S111','1975-03-23 11:18:20','suborbital rocket','success',867,'NASA 18.166UE','','','','Plasma',1047,'',230 from dual union all
select 36705,'1975-S112','1975-03-24 00:06:00','suborbital rocket','success',920,'P160T/C','ISRO-18.20','TERLS-743','','Fields',1467,'',90 from dual union all
select 36706,'1975-S113','1975-03-30 00:00:00','military missile','success',1004,'33L','15A14 LKI-34','GCh 15F142','','Test',490,'',1000 from dual union all
select 36707,'1975-S114','1975-04-03 00:00:00','military missile','success',1251,'PFT-4','SAFEGUARD M2-345','','','Interceptor',868,'',100 from dual union all
select 36708,'1975-S115','1975-04-08 19:05:00','suborbital rocket','success',839,'NASA 10.377UM','','','','Aeronomy',1821,'',90 from dual union all
select 36709,'1975-S116','1975-04-10 04:40:00','suborbital rocket','success',38,'NASA 13.88GG','','','','UV Astron?',1895,'',176 from dual union all
select 3671,'1955-S21','1955-02-01 00:00:00','military missile','success',1009,'K3-15','8A62 ZI','','','Test',636,'',300 from dual union all
select 36710,'1975-S117','1975-04-10 09:54:00','suborbital rocket','success',906,'CRL A10.302-3','AEOLUS Release,Sph','','','Aeron',428,'',216 from dual union all
select 36711,'1975-S118','1975-04-10 09:54:30','suborbital rocket','success',906,'CRL A10.302-4','AEOLUS Release','','','Aeron',428,'',219 from dual union all
select 36712,'1975-S119','1975-04-10 10:00:00','suborbital rocket','success',906,'CRL A10.403-3','AEOLUS Mass Spec/E','','','Aeron',428,'',211 from dual union all
select 36713,'1975-S12','1975-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 36714,'1975-S120','1975-04-12 14:59:00','military missile','success',931,'A3T','DASO','','','OT',421,'SSBN 610',1000 from dual union all
select 36715,'1975-S121','1975-04-13 08:15:00','suborbital rocket','success',867,'NASA 18.174GE','','','','Plasma',1047,'',299 from dual union all
select 36716,'1975-S122','1975-04-13 09:46:00','suborbital rocket','success',278,'DNA EX531.43-1','EXCEDE','','','Active Plasma',1047,'',100 from dual union all
select 36717,'1975-S123','1975-04-13 10:38:00','suborbital rocket','success',867,'NASA 18.173GE','','','','Plasma',1047,'',299 from dual union all
select 36718,'1975-S124','1975-04-16 09:15:00','suborbital rocket','success',867,'NASA 18.175GE','','','','Plasma',1047,'',299 from dual union all
select 36719,'1975-S125','1975-04-17 00:00:00','military missile','success',1251,'PFT-5','SAFEGUARD M2-545','','','Interceptor',868,'',100 from dual union all
select 3672,'1955-S22','1955-03-19 06:00:00','suborbital rocket','success',54,'NRL 26','Airglow','','','Aeronomy',1869,'',115 from dual union all
select 36720,'1975-S126','1975-04-19 00:28:00','military missile','success',939,'','FOT-9','','','OT',421,'SSBN 645',500 from dual union all
select 36721,'1975-S127','1975-04-19 00:33:34','military missile','success',939,'','FOT-9','','','OT',421,'SSBN 645',500 from dual union all
select 36722,'1975-S128','1975-04-21 08:11:00','suborbital rocket','success',290,'ISRO 5.26','ISRO 5.26 CO.22','TERLS-747','','Ionos',1467,'',157 from dual union all
select 36723,'1975-S129','1975-04-21 09:07:00','suborbital rocket','success',906,'CRL A10.302-1','AEOLUS Release/E','','','Aeron',428,'',213 from dual union all
select 36724,'1975-S13','1975-07-06 00:00:00','test rocket','unknown',1497,'4','K65UP No. 4','','','Aeron/Ionos?',621,'',1500 from dual union all
select 36725,'1975-S130','1975-04-21 09:07:30','suborbital rocket','success',906,'CRL A10.302-2','AEOLUS Release','','','Aeron',428,'',220 from dual union all
select 36726,'1975-S131','1975-04-21 09:12:00','suborbital rocket','success',906,'CRL A10.403-1','AEOLUS Mass Spec','','','Aeron',428,'',222 from dual union all
select 36727,'1975-S132','1975-04-21 17:30:00','suborbital rocket','success',290,'ISRO 5.27','ISRO 5.27 CO.21','TERLS-748','','Ionos',1467,'',156 from dual union all
select 36728,'1975-S133','1975-04-23 00:00:00','military missile','success',744,'','15A15 K-2','','','OT',530,'',1000 from dual;
insert into LAUNCH
select 36729,'1975-S134','1975-04-24 00:00:00','military missile','success',939,'','FOT-10','','','OT',421,'SSBN 633',500 from dual union all
select 3673,'1955-S23','1955-03-29 16:47:00','suborbital rocket','success',51,'USAF 54','Solar FUV/LyA','','','Solar/Aeron',640,'',112 from dual union all
select 36730,'1975-S135','1975-04-24 00:00:00','military missile','success',1004,'37L','15A14 LKI-35','GCh 15F143U1','','Test',492,'',1000 from dual union all
select 36731,'1975-S136','1975-04-25 04:13:00','suborbital rocket','success',906,'CRL A10.302-5','AEOLUS TMA/Ba','','','Aeron',428,'',214 from dual union all
select 36732,'1975-S137','1975-04-25 04:13:30','suborbital rocket','success',906,'CRL A10.302-7','AEOLUS TMA/Ba','','','Aeron',428,'',220 from dual union all
select 36733,'1975-S138','1975-04-25 04:18:00','suborbital rocket','success',906,'CRL A10.403-2','AEOLUS Mass Spec','','','Aeron',428,'',217 from dual union all
select 36734,'1975-S139','1975-04-26 15:10:00','military missile','success',931,'A3T','DASO','','','OT',421,'SSBN 609',1000 from dual union all
select 36735,'1975-S14','1975-09-02 00:00:00','test rocket','unknown',1497,'5','K65UP No. 5','','','Aero/Ionos?',621,'',1500 from dual union all
select 36736,'1975-S140','1975-05-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36737,'1975-S141','1975-05-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36738,'1975-S142','1975-05-03 19:00:00','suborbital rocket','success',920,'P164H','Electron density','','','Ionospheric',1447,'',111 from dual union all
select 36739,'1975-S143','1975-06-04 11:22:00','suborbital rocket','success',1189,'INTA MSCL-7503','','','','Aeron/Ionos',396,'',85 from dual union all
select 3674,'1955-S24','1955-06-30 00:00:00','military missile','success',1013,'S1-8','8K51 LKI-12','','','Test',636,'',500 from dual union all
select 36740,'1975-S144','1975-06-04 11:42:00','suborbital rocket','unknown',920,'INTA MPGFD-7503','','','','Ionos',396,'',81 from dual union all
select 36741,'1975-S145','1975-05-07 04:20:00','military missile','success',799,'PVM-10','Mk 12 RV x 3','R&D','PVM-10','Test',1712,'',1300 from dual union all
select 36742,'1975-S146','1975-06-07 11:00:00','suborbital rocket','success',1189,'INTA MSCL-7504','','','','Aeron/Ionos',396,'',94 from dual union all
select 36743,'1975-S147','1975-06-07 11:18:00','suborbital rocket','unknown',920,'INTA MPGFD-7504','','','','Ionos',396,'',86 from dual union all
select 36744,'1975-S148','1975-05-09 00:00:00','military missile','success',795,'FTM 1328','Mk 500 RB 102','Evader SFT-102','SFT-102','Reentry Test',1701,'',1300 from dual union all
select 36745,'1975-S149','1975-05-09 00:00:00','suborbital rocket','success',487,'F.0','','','','Test',1836,'',119 from dual union all
select 36746,'1975-S15','1975-01-01 00:00:00','military missile','success',1507,'VMF','','','','Test',989,'',1000 from dual union all
select 36747,'1975-S150','1975-05-09 00:00:00','suborbital rocket','success',1347,'Sandia','','','','Monitoring',855,'',406 from dual union all
select 36748,'1975-S151','1975-05-12 00:00:00','suborbital rocket','success',1337,'Sandia 497-004','','','','Test',855,'',485 from dual union all
select 36749,'1975-S152','1975-05-13 09:25:00','military missile','unknown',917,'AO-148','P-300 75 Spring 01','','','OT',615,'',180 from dual union all
select 3675,'1955-S25','1955-07-07 00:00:00','military missile','success',1013,'S1-13','8K51 LKI-13','','','Test',636,'',500 from dual union all
select 36750,'1975-S153','1975-05-13 09:52:00','military missile','unknown',917,'AO-149','P-301 75 Spring 02','','','OT',615,'',180 from dual union all
select 36751,'1975-S154','1975-05-14 00:00:00','suborbital rocket','success',1349,'Sandia','','','','Plasma Ba',855,'',360 from dual union all
select 36752,'1975-S155','1975-05-15 00:00:00','suborbital rocket','success',1349,'Sandia','','','','Plasma Ba',855,'',360 from dual union all
select 36753,'1975-S156','1975-05-15 15:25:00','suborbital rocket','success',133,'NASA 25.03GT-NP','','','','Test',1895,'',224 from dual union all
select 36754,'1975-S157','1975-05-16 00:00:00','military missile','success',799,'STM-9W/FTM 5647','Mk 12A RV x 3','PAVE PEPPER','PAVE PEPPER 1','Test',1697,'',1300 from dual union all
select 36755,'1975-S158','1975-05-23 00:00:00','military missile','success',795,'FTM 1406','NTV-3','ABRES NTV-3','','Reentry Test',1698,'',1300 from dual union all
select 36756,'1975-S159','1975-05-24 01:00:00','suborbital rocket','success',867,'NASA 18.170GE','Antarqui Radiation','','','Plasma',323,'',338 from dual union all
select 36757,'1975-S16','1975-01-01 00:00:00','military missile','success',997,'','','','','Test',636,'',1000 from dual union all
select 36758,'1975-S160','1975-05-24 06:55:00','suborbital rocket','success',867,'NASA 18.171GE','Antarqui Radiation','','','Plasma',323,'',324 from dual union all
select 36759,'1975-S161','1975-05-27 19:02:00','suborbital rocket','success',837,'NASA 14.531UA','Antarqui Pandora','','','Aeronomy',323,'',127 from dual union all
select 3676,'1955-S26','1955-04-08 00:00:00','military missile','success',970,'No 410','','','','OT',636,'',100 from dual union all
select 36760,'1975-S162','1975-05-27 08:01:00','suborbital rocket','success',837,'NASA 14.530UA','Antarqui Pandora','','','Aeronomy',323,'',127 from dual union all
select 36761,'1975-S163','1975-05-28 09:38:00','military missile','unknown',917,'AO-150','P-302 75 Spring 03','','','OT',615,'',180 from dual union all
select 36762,'1975-S164','1975-05-28 10:04:00','military missile','unknown',917,'AO-151','P-303 75 Spring 04','','','OT',615,'',180 from dual union all
select 36763,'1975-S165','1975-05-28 00:00:00','military missile','success',1004,'38L','15A14 LKI-36','GCh 15F143U1','','Test',490,'',1000 from dual union all
select 36764,'1975-S166','1975-05-28 20:26:00','suborbital rocket','success',837,'NASA 14.532UE','Antarqui Ionos','','','Plasma',323,'',191 from dual union all
select 36765,'1975-S167','1975-05-28 23:00:02','military missile','success',939,'','DASO-34','','','OT',421,'SSBN 617',500 from dual union all
select 36766,'1975-S168','1975-05-29 00:02:02','military missile','success',939,'3063','DASO-35','','','OT',421,'SSBN 617',500 from dual union all
select 36767,'1975-S169','1975-06-01 03:55:00','suborbital rocket','success',837,'NASA 14.538UA','Antarqui Mass Spec','','','Aeronomy',323,'',131 from dual union all
select 36768,'1975-S17','1975-01-20 04:10:00','suborbital rocket','success',1073,'S-310-1','','','','Test',839,'',192 from dual union all
select 36769,'1975-S170','1975-05-30 04:35:00','suborbital rocket','success',837,'NASA 14.524UE','Antarqui Ionos','','','Plasma',323,'',187 from dual union all
select 3677,'1955-S27','1955-04-08 15:19:00','suborbital rocket','success',841,'','DAN-1','','','Test',1821,'',108 from dual union all
select 36770,'1975-S171','1975-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36771,'1975-S172','1975-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36772,'1975-S173','1975-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36773,'1975-S174','1975-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36774,'1975-S175','1975-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36775,'1975-S176','1975-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36776,'1975-S177','1975-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36777,'1975-S178','1975-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36778,'1975-S179','1975-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36779,'1975-S18','1975-04-02 18:30:00','suborbital rocket','success',135,'Athena D','HEARTS II RV 112','V4-7489','','Reentry Test',1821,'',200 from dual union all
select 3678,'1955-S28','1955-04-09 00:00:00','military missile','success',970,'No 1811','','','','OT',636,'',100 from dual union all
select 36780,'1975-S180','1975-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36781,'1975-S181','1975-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36782,'1975-S182','1975-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36783,'1975-S183','1975-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36784,'1975-S184','1975-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36785,'1975-S185','1975-06-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 36786,'1975-S186','1975-06-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 36787,'1975-S187','1975-06-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 36788,'1975-S188','1975-05-30 00:00:00','suborbital rocket','failure',1492,'','FU-216 FAUST','','','UV Astron',341,'',172 from dual union all
select 36789,'1975-S189','1975-06-02 05:11:00','suborbital rocket','success',837,'NASA 14.525UE','Antarqui Ionos','','','Plasma',323,'',187 from dual union all
select 3679,'1955-S29','1955-04-12 00:00:00','military missile','success',970,'No 405','','','','OT',636,'',100 from dual union all
select 36790,'1975-S19','1975-01-01 00:00:00','military missile','success',922,'','15Zh45 LKI K-1','','','Test',822,'',500 from dual union all
select 36791,'1975-S190','1975-06-03 16:41:00','suborbital rocket','success',837,'NASA 14.540CA','GCA Release Li Ant.','','','Aeronomy',323,'',204 from dual union all
select 36792,'1975-S191','1975-06-04 00:00:00','military missile','success',1004,'34L','15A14 LKI-37','GCh 15F143U1','','Test',482,'',1000 from dual union all
select 36793,'1975-S192','1975-06-07 00:00:00','military missile','success',1004,'36L','15A14 LKI-38','GCh 15F143U1','','Test',496,'',1000 from dual union all
select 36794,'1975-S193','1975-06-07 16:07:00','suborbital rocket','success',867,'NASA 18.150GE','Antarqui Fields','','','Plasma',323,'',259 from dual union all
select 36795,'1975-S194','1975-06-07 16:45:00','suborbital rocket','success',837,'NASA 14.541CA','Antarqui Release','GCA Release LI','','Aeronomy',323,'',197 from dual union all
select 36796,'1975-S195','1975-06-10 10:15:00','suborbital rocket','success',30,'CRL A03.103-1','Density/Laser Backs.','','','Aeron',1869,'',158 from dual union all
select 36797,'1975-S196','1975-06-10 13:18:56','suborbital rocket','success',839,'','Ferdinand 42','','','Auroral',29,'',100 from dual union all
select 36798,'1975-S197','1975-06-11 14:30:00','military missile','success',799,'','Mk 12 RV x 3','OT GT30GM','GLORY TRIP 30GM','OT',1703,'',1300 from dual union all
select 36799,'1975-S198','1975-06-11 09:22:00','military missile','success',917,'AO-152','P-304 75 Spring 05','','','OT',615,'',180 from dual union all
select 368,'1944-S335','1944-10-31 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3680,'1955-S30','1955-04-15 00:00:00','military missile','success',970,'No 215','','','','OT',636,'',100 from dual union all
select 36800,'1975-S199','1975-06-11 10:00:00','military missile','success',917,'AO-153','P-305 75 Spring 06','','','OT',615,'',180 from dual union all
select 36801,'1975-S20','1975-04-14 00:00:00','military missile','success',1322,'64521-61-875-35L','15Zh42 UBP-1','','','Test',477,'',1000 from dual union all
select 36802,'1975-S200','1975-06-13 00:00:00','military missile','success',744,'','15A15 LKI-29','','','Test',530,'',1000 from dual union all
select 36803,'1975-S201','1975-06-18 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 602',1000 from dual union all
select 36804,'1975-S202','1975-06-18 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 602',1000 from dual union all
select 36805,'1975-S203','1975-06-18 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 602',1000 from dual union all
select 36806,'1975-S204','1975-06-18 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 602',1000 from dual union all
select 36807,'1975-S205','1975-06-18 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 602',1000 from dual union all
select 36808,'1975-S206','1975-06-20 00:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT31GM','GLORY TRIP 31GM','OT',1705,'',1300 from dual union all
select 36809,'1975-S207','1975-06-23 00:00:00','military missile','unknown',939,'','FOT-11','','','OT',421,'SSBN 627',500 from dual union all
select 3681,'1955-S31','1955-04-20 06:51:00','military missile','success',1053,'RS-9     XE','','','','Test',280,'',90 from dual union all
select 36810,'1975-S208','1975-06-23 00:00:00','military missile','unknown',939,'','FOT-11','','','OT',421,'SSBN 627',500 from dual union all
select 36811,'1975-S209','1975-06-24 00:00:00','military missile','success',1105,'S-1E-2605','','','','OT',301,'',1000 from dual union all
select 36812,'1975-S21','1975-04-15 00:00:00','military missile','success',1322,'64536-51-853-31L','15Zh42 UBP-2','','','Test',477,'',1000 from dual union all
select 36813,'1975-S210','1975-06-24 00:00:00','military missile','unknown',939,'','FOT-12','','','OT',421,'SSBN 656',500 from dual union all
select 36814,'1975-S211','1975-06-24 00:00:00','military missile','unknown',939,'','FOT-12','','','OT',421,'SSBN 656',500 from dual union all
select 36815,'1975-S212','1975-06-24 11:18:00','suborbital rocket','success',1211,'SL1105','LUX XRS17','','','XR Astronomy',1839,'',180 from dual union all
select 36816,'1975-S213','1975-06-25 00:00:00','military missile','unknown',917,'AO-154','P-306 75 Spring 07','','','OT',615,'',180 from dual union all
select 36817,'1975-S214','1975-06-25 00:00:00','military missile','unknown',917,'AO-155','P-307 75 Spring 08','','','OT',615,'',180 from dual union all
select 36818,'1975-S215','1975-06-25 02:50:00','military missile','success',999,'N22501-01','8K67 UBP-52 OS-67-1A','GCh 8F674','','Test',515,'',1000 from dual;
insert into LAUNCH
select 36819,'1975-S216','1975-06-27 20:10:00','suborbital rocket','failure',1337,'NASA 29.02GT','','','','Test',1821,'',283 from dual union all
select 3682,'1955-S32','1955-04-21 15:58:00','suborbital rocket','success',49,'USAF 55','AJ11-6 test','','','Test',640,'',198 from dual union all
select 36820,'1975-S217','1975-07-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36821,'1975-S218','1975-07-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',138,'ZUB',150 from dual union all
select 36822,'1975-S219','1975-06-30 00:00:00','military missile','success',1004,'39L','15A14 LKI-39','GCh 15F143U2','','Test',510,'',1000 from dual union all
select 36823,'1975-S22','1975-01-27 00:00:00','military missile','success',1449,'','8K84 UBP-145 Svinets-5','','','OT',540,'',1000 from dual union all
select 36824,'1975-S220','1975-07-01 00:00:00','military missile','success',922,'','15Zh45 LKI','','','Test',636,'',500 from dual union all
select 36825,'1975-S221','1975-07-01 00:00:00','military missile','success',744,'','15A15 LKI-30','','','Test',541,'',1000 from dual union all
select 36826,'1975-S222','1975-07-01 00:00:00','military missile','success',799,'PVM-11','Mk 12 RV x 3','R&D','','Test',1712,'',1300 from dual union all
select 36827,'1975-S223','1975-07-03 00:00:00','suborbital rocket','success',1211,'SL1295','CHEVALINE S6','','','MIRV test',1839,'',270 from dual union all
select 36828,'1975-S224','1975-07-10 08:00:00','suborbital rocket','success',37,'NASA 13.73GG','13-in RC Flight 4','','','UV Astron?',1895,'',179 from dual union all
select 36829,'1975-S225','1975-07-15 03:35:00','suborbital rocket','success',1014,'','R-5V Geophysical','','','Ionos',636,'',503 from dual union all
select 3683,'1955-S33','1955-05-01 00:00:00','military missile','success',974,'','','','','Test',636,'',150 from dual union all
select 36830,'1975-S226','1975-07-21 15:31:00','suborbital rocket','success',228,'NASA 21.25DS','HRTS-1','','','Solar UV',1873,'',220 from dual union all
select 36831,'1975-S227','1975-07-23 15:04:00','military missile','success',917,'FOT-37','P-308 75 Cape 01','','','OT',231,'',180 from dual union all
select 36832,'1975-S228','1975-07-23 16:38:00','military missile','success',917,'FOT-38','P-309 75 Cape 02','','','OT',231,'',180 from dual union all
select 36833,'1975-S229','1975-07-23 16:53:00','military missile','success',917,'FOT-39','P-310 75 Cape 03','','','OT',231,'',180 from dual union all
select 36834,'1975-S23','1975-02-25 02:50:00','military missile','success',1450,'47656-41-143P','15A20 UBP-23 ZP','GCh 15F202','','OT',504,'',1000 from dual union all
select 36835,'1975-S230','1975-07-24 05:20:00','suborbital rocket','success',839,'NASA 10.413GM Icomp','GSFC Grenade 279 W','','','Aeronomy',1821,'',120 from dual union all
select 36836,'1975-S231','1975-07-24 18:49:00','military missile','success',917,'FOT-40','P-311 75 Cape 04','','','OT',231,'',180 from dual union all
select 36837,'1975-S232','1975-07-26 00:00:00','military missile','success',799,'STM-10W','Mk 12A RV x 3','PAVE PEPPER','PAVE PEPPER 2','Test',1697,'',1300 from dual union all
select 36838,'1975-S233','1975-07-28 17:44:00','suborbital rocket','success',228,'NASA 21.29US','','','','Solar UV',1873,'',278 from dual union all
select 36839,'1975-S234','1975-08-05 16:04:00','military missile','success',917,'FOT-41','P-312 75 Cape 05','','','OT',231,'',180 from dual union all
select 3684,'1955-S34','1955-05-01 00:00:00','military missile','success',974,'','','','','Test',636,'',150 from dual union all
select 36840,'1975-S235','1975-08-05 18:16:00','military missile','success',917,'FOT-43','P-314 75 Cape 07','','','OT',231,'',180 from dual union all
select 36841,'1975-S236','1975-08-05 18:32:00','military missile','success',917,'FOT-44','P-315 75 Cape 08','','','OT',231,'',180 from dual union all
select 36842,'1975-S237','1975-08-07 00:00:00','suborbital rocket','success',277,'','','','','Tech',871,'',200 from dual union all
select 36843,'1975-S238','1975-08-07 00:00:00','military missile','success',1420,'B-52','Mk 6 RV','DG-2','','Target',1658,'',1300 from dual union all
select 36844,'1975-S239','1975-08-07 03:30:00','suborbital rocket','success',37,'CRL A04.308-1','Density/Bremss','','','Aeron',1895,'',174 from dual union all
select 36845,'1975-S24','1975-05-22 17:03:00','military missile','success',1450,'52456-42-157P','15A20 UBP-24 ZP','GCh 15F202','','OT',504,'',1000 from dual union all
select 36846,'1975-S240','1975-08-07 16:10:00','suborbital rocket','success',839,'NASA 10.407GM','GSFC Grenade 280 W','','','Aeronomy',1821,'',119 from dual union all
select 36847,'1975-S241','1975-08-12 00:00:00','military missile','success',744,'','15A15 K-3','','','Test',530,'',1000 from dual union all
select 36848,'1975-S242','1975-08-12 00:00:00','military missile','success',744,'','15A15 K-4','','','Test',541,'',1000 from dual union all
select 36849,'1975-S243','1975-08-15 04:25:00','military missile','success',796,'FTM 2396','Mk 11C RV','FOT GT122M-1','GLORY TRIP 122M-1','OT',1699,'',1300 from dual union all
select 3685,'1955-S35','1955-05-01 00:00:00','military missile','success',974,'','','','','Test',636,'',150 from dual union all
select 36850,'1975-S244','1975-08-15 05:21:00','suborbital rocket','success',906,'CRL A10.406-2','ESCAPE I (Sphere)','Sphere (10in)','','Aeron',1774,'',272 from dual union all
select 36851,'1975-S245','1975-08-16 00:00:00','military missile','success',1004,'40L','15A14 LKI-40','GCh 15F143U2','','Test',492,'',1000 from dual union all
select 36852,'1975-S246','1975-08-16 07:40:00','suborbital rocket','success',37,'NASA 13.50UH','MIT UBP 2','','','XR Astron',1895,'',172 from dual union all
select 36853,'1975-S247','1975-08-17 06:00:00','suborbital rocket','success',1298,'TT-210-1F','','','','Test',1476,'',113 from dual union all
select 36854,'1975-S248','1975-08-18 12:45:00','suborbital rocket','success',117,'NASA 12.26GT-IT','IMS Prototype','','','Test',1873,'',417 from dual union all
select 36855,'1975-S249','1975-08-22 00:00:00','military missile','success',795,'','Mk 500 RB 103','Evader SFT-103','SFT-103','Reentry Test',1701,'',1300 from dual union all
select 36856,'1975-S25','1975-10-10 03:25:00','military missile','success',1450,'47656-131','15A20S LKI-1','GCh 15F202P2 (15F204 x3)','','OT',504,'',1000 from dual union all
select 36857,'1975-S250','1975-08-22 17:08:40','military missile','unknown',939,'','FOT-13','','','OT',421,'SSBN 634',500 from dual union all
select 36858,'1975-S251','1975-08-22 17:09:08','military missile','unknown',939,'','FOT-13','','','OT',421,'SSBN 634',500 from dual union all
select 36859,'1975-S252','1975-08-22 17:26:10','military missile','unknown',939,'','FOT-13','','','OT',421,'SSBN 634',500 from dual union all
select 3686,'1955-S36','1955-05-20 00:00:00','military missile','success',1013,'S1-6','8K51 LKI-5','','','Test',636,'',500 from dual union all
select 36860,'1975-S253','1975-08-25 17:32:00','military missile','success',917,'FOT-46','P-317 75 Cape 10','','','OT',231,'',180 from dual union all
select 36861,'1975-S254','1975-08-26 00:00:00','military missile','success',1004,'518T','15A14 K-5 ZP','GCh 15F143U1','','OT',496,'',1000 from dual union all
select 36862,'1975-S255','1975-08-26 10:40:00','suborbital rocket','success',668,'K-9M-53','','','','Ionos',839,'',351 from dual union all
select 36863,'1975-S256','1975-08-26 18:32:00','military missile','success',917,'FOT-47','P-318 75 Cape 11','','','OT',231,'',180 from dual union all
select 36864,'1975-S257','1975-08-26 18:50:00','suborbital rocket','success',37,'NASA 13.74UA','','','','Aeronomy',1895,'',180 from dual union all
select 36865,'1975-S258','1975-08-27 00:00:00','military missile','success',1004,'41L','15A14 LKI-41','GCh 15F143U2','','Test',490,'',1000 from dual union all
select 36866,'1975-S259','1975-08-29 14:30:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT49GM','GLORY TRIP 49GM','OT',1703,'',1300 from dual union all
select 36867,'1975-S26','1975-10-30 00:40:00','military missile','success',1450,'64146-43-157P','15A20 UBP-27 ZP','GCh 15F202','','OT',504,'',1000 from dual union all
select 36868,'1975-S260','1975-08-29 17:24:26','military missile','unknown',939,'','FOT-14','','','OT',421,'SSBN 645',500 from dual union all
select 36869,'1975-S261','1975-08-29 17:24:44','military missile','unknown',939,'','FOT-14','','','OT',421,'SSBN 645',500 from dual union all
select 3687,'1955-S37','1955-05-26 00:00:00','military missile','success',1013,'S1-1','8K51 LKI-6','','','Test',636,'',500 from dual union all
select 36870,'1975-S262','1975-08-30 04:10:00','suborbital rocket','success',1073,'S-310-2','','','','Ionos',839,'',191 from dual union all
select 36871,'1975-S263','1975-09-02 04:40:00','suborbital rocket','success',1014,'','Vertikal''-3','VZA','','Aeron/Solar X',636,'',502 from dual union all
select 36872,'1975-S264','1975-09-02 12:00:00','suborbital rocket','success',668,'K-9M-51','','','','Ionos',839,'',311 from dual union all
select 36873,'1975-S265','1975-09-03 10:30:00','suborbital rocket','success',864,'Sandia 152-','LRL ACS-1D','','','XR Astron',1490,'',270 from dual union all
select 36874,'1975-S266','1975-09-04 16:35:00','suborbital rocket','unknown',740,'MR-12','Spolokh-1','','','Plasma,Ba release',636,'',170 from dual union all
select 36875,'1975-S267','1975-09-05 04:25:00','military missile','success',796,'','ERCS','ERCS GIANT MOON 7','GIANT MOON 7','Comms',1700,'',1300 from dual union all
select 36876,'1975-S268','1975-09-09 00:56:00','suborbital rocket','success',920,'P136H','Atomic oxygen','','','Atmospheric',1447,'',141 from dual union all
select 36877,'1975-S269','1975-09-09 00:30:00','suborbital rocket','success',920,'P168H','Airglow','','','Atmospheric',1447,'',130 from dual union all
select 36878,'1975-S27','1975-04-05 00:00:00','military missile','success',1452,'63046-41-381P','15A30 UBP-1 ZP','GCh 15B103','','OT',508,'',1000 from dual union all
select 36879,'1975-S270','1975-09-09 00:00:00','suborbital rocket','success',920,'P153H','Electron density','','','Ionospheric',1447,'',129 from dual union all
select 3688,'1955-S38','1955-05-21 00:00:00','military missile','success',974,'','','','','Test',636,'',150 from dual union all
select 36880,'1975-S271','1975-09-09 19:32:00','suborbital rocket','success',842,'NASA 12.28WT','','','','Test/Aeron',1821,'',162 from dual union all
select 36881,'1975-S272','1975-09-10 00:00:00','military missile','success',795,'','Mk 500 RB 104','Evader SFT-104','SFT-104','Reentry Test',1701,'',1300 from dual union all
select 36882,'1975-S273','1975-09-11 20:40:00','suborbital rocket','success',740,'MR-12','Zarnitsa-2','','','Plasma',636,'',154 from dual union all
select 36883,'1975-S274','1975-09-16 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT123M-1','GLORY TRIP 123M-1','OT',1702,'',1300 from dual union all
select 36884,'1975-S275','1975-09-18 07:30:00','suborbital rocket','success',1278,'','NRV Nosetip Recovery','','','Reentry Test',1821,'',170 from dual union all
select 36885,'1975-S276','1975-09-18 19:47:00','suborbital rocket','success',906,'CRL A10.304-1','NMS','','','Aeron',1895,'',128 from dual union all
select 36886,'1975-S277','1975-09-19 00:00:00','military missile','success',1373,'201','DG-5','','','Target',773,'',500 from dual union all
select 36887,'1975-S278','1975-09-19 05:15:00','suborbital rocket','success',906,'CRL A10.304-2','NMS','','','Aeron',1895,'',128 from dual union all
select 36888,'1975-S279','1975-09-24 01:44:00','military missile','success',796,'OSL','Mk 11 RV','Ogden Special Launch','OSL','Test',1699,'',1300 from dual union all
select 36889,'1975-S28','1975-08-29 00:00:00','military missile','success',1452,'51156-79-001T','15A30 UBP-4','','','OT',508,'',1000 from dual union all
select 3689,'1955-S39','1955-05-25 04:24:00','military missile','success',1053,'RS-10    HX','','','','Test',280,'',90 from dual union all
select 36890,'1975-S280','1975-09-23 10:02:23','suborbital rocket','success',864,'Sandia 152-','LRL ACS-1E','','','XR Astron',1490,'',270 from dual union all
select 36891,'1975-S281','1975-09-23 12:00:00','suborbital rocket','success',668,'K-9M-52','Soft XR','','','XR Astron',839,'',336 from dual union all
select 36892,'1975-S282','1975-09-24 05:00:00','suborbital rocket','success',655,'K-10-11','','','','XR Astron/Solar UV',839,'',196 from dual union all
select 36893,'1975-S283','1975-09-24 21:27:00','suborbital rocket','success',867,'NASA 18.163UE','UTD FERRET 1','','','Plasma',1821,'',273 from dual union all
select 36894,'1975-S284','1975-09-24 23:00:02','military missile','success',939,'','DASO','','','OT',421,'SSBN 623',500 from dual union all
select 36895,'1975-S285','1975-09-29 00:00:00','military missile','success',1004,'42L','15A14 LKI-42','GCh 15F143U2','','Test',486,'',1000 from dual union all
select 36896,'1975-S286','1975-09-30 10:02:00','military missile','success',795,'','USN Mk 4 RV','Mk 4 SFT-05','SFT-05','Reentry Test',1698,'',1300 from dual union all
select 36897,'1975-S287','1975-10-07 00:00:00','military missile','success',916,'AO-156','P-320 75 Fall 01','','','OT',433,'',180 from dual union all
select 36898,'1975-S288','1975-10-07 00:00:00','military missile','success',916,'AO-157','P-321 75 Fall 02','','','OT',433,'',180 from dual union all
select 36899,'1975-S289','1975-10-21 09:35:00','military missile','success',916,'AO-158','P-322 75 Fall 03','','','OT',433,'',180 from dual union all
select 369,'1944-S336','1944-10-31 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3690,'1955-S40','1955-06-01 00:00:00','military missile','success',974,'','','','','Test',636,'',150 from dual union all
select 36900,'1975-S29','1975-09-20 00:00:00','military missile','success',1452,'49256-42-378T','15A30 UBP-5 ZP','GCh 15B103','','OT',532,'',1000 from dual union all
select 36901,'1975-S290','1975-09-30 00:00:00','suborbital rocket','success',279,'CR-02','Ba release','','','Ionos',1747,'',400 from dual union all
select 36902,'1975-S291','1975-10-01 00:00:00','military missile','success',1004,'43L','15A14 LKI-43','GCh 15F143U2','','Test',482,'',1000 from dual union all
select 36903,'1975-S292','1975-10-01 15:35:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',162 from dual union all
select 36904,'1975-S293','1975-10-02 00:00:00','suborbital rocket','success',1181,'MSAC 2L 7503','','','','Ionos',1895,'',90 from dual union all
select 36905,'1975-S294','1975-10-02 09:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',168 from dual union all
select 36906,'1975-S295','1975-10-02 15:35:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',164 from dual union all
select 36907,'1975-S296','1975-10-02 21:05:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',160 from dual union all
select 36908,'1975-S297','1975-10-03 00:00:00','suborbital rocket','success',279,'CR-03','Ba release','','','Ionos',1747,'',400 from dual;
insert into LAUNCH
select 36909,'1975-S298','1975-10-03 03:15:00','suborbital rocket','success',41,'NASA 26.47DH','','','','XR Astron',1895,'',212 from dual union all
select 3691,'1955-S41','1955-06-01 00:00:00','military missile','success',974,'','','','','Test',636,'',150 from dual union all
select 36910,'1975-S299','1975-10-09 04:49:25','suborbital rocket','success',1211,'ESRO S103','DLR H-GR-68 (Astro 7)','INTA SKY-A7','','UV Astronomy',395,'',254 from dual union all
select 36911,'1975-S30','1975-10-13 00:00:00','military missile','success',1452,'63056-41-383T','15A30 UBP-6 ZP','GCh 15B103','','OT',544,'',1000 from dual union all
select 36912,'1975-S300','1975-10-09 10:45:00','suborbital rocket','success',37,'NASA 13.117UL','','','','UV Astron',1895,'',203 from dual union all
select 36913,'1975-S301','1975-10-10 23:00:02','military missile','success',939,'','DASO','','','OT',421,'SSBN 619',500 from dual union all
select 36914,'1975-S302','1975-10-14 10:30:00','suborbital rocket','success',133,'CRL A31.320-02','IPRCE-4/Zodi IR','','','IR Astron',1895,'',265 from dual union all
select 36915,'1975-S303','1975-10-16 02:10:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',165 from dual union all
select 36916,'1975-S304','1975-10-16 16:30:00','suborbital rocket','success',906,'CRL A10.000-1','DCST','','','Test',1895,'',263 from dual union all
select 36917,'1975-S305','1975-10-18 04:45:00','suborbital rocket','success',37,'NASA 13.108UH','MIT UBP 3','','','XR Astron',1895,'',184 from dual union all
select 36918,'1975-S306','1975-10-21 11:42:00','suborbital rocket','success',599,'INTA IT-P2-7501','F-75001','','','Aeron/Test',396,'',240 from dual union all
select 36919,'1975-S307','1975-10-25 00:00:00','military missile','success',999,'','','','','OT?',596,'',1000 from dual union all
select 3692,'1955-S42','1955-06-01 00:00:00','military missile','success',974,'','','','','Test',636,'',150 from dual union all
select 36920,'1975-S308','1975-10-29 02:39:00','suborbital rocket','success',38,'NASA 13.48GG','','','','UV Astron?',1895,'',186 from dual union all
select 36921,'1975-S309','1975-10-29 14:28:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',164 from dual union all
select 36922,'1975-S31','1975-01-06 23:49:02','suborbital rocket','success',220,'Tordo I','Periquito','','','Plasma Ba',334,'',540 from dual union all
select 36923,'1975-S310','1975-10-30 23:52:31','suborbital rocket','success',1162,'EX530.90-1/S75-2','PF-SGT-101','','','Test',1047,'',100 from dual union all
select 36924,'1975-S311','1975-11-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',139,'ZUB',150 from dual union all
select 36925,'1975-S312','1975-11-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',139,'ZUB',150 from dual union all
select 36926,'1975-S313','1975-10-31 11:17:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',152 from dual union all
select 36927,'1975-S314','1975-11-06 00:00:00','military missile','success',1373,'274','DG-3','','','Target',773,'',500 from dual union all
select 36928,'1975-S315','1975-11-08 00:00:00','military missile','unknown',939,'','FOT-15','','','OT',421,'SSBN 635',500 from dual union all
select 36929,'1975-S316','1975-11-08 00:00:00','military missile','unknown',939,'','FOT-15','','','OT',421,'SSBN 635',500 from dual union all
select 3693,'1955-S43','1955-06-06 00:00:00','military missile','success',1013,'S1-7','8K51 LKI-7','','','Test',636,'',500 from dual union all
select 36930,'1975-S317','1975-11-08 05:20:00','suborbital rocket','success',37,'NASA 13.049UH','Wisc. Aerobee IV','','','XR Astron',1895,'',177 from dual union all
select 36931,'1975-S318','1975-11-13 00:00:00','military missile','success',795,'FTM 1349','USN Mk 4 RV','Mk 4 SFT-06','SFT-06','Reentry Test',1698,'',1300 from dual union all
select 36932,'1975-S319','1975-11-13 00:00:00','military missile','unknown',939,'','FOT-16','','','OT',421,'SSBN 642',500 from dual union all
select 36933,'1975-S32','1975-01-10 02:50:00','suborbital rocket','success',277,'','SOFT','','','Reentry Test',871,'',200 from dual union all
select 36934,'1975-S320','1975-11-14 17:10:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT50GM','GLORY TRIP 50GM','OT',1703,'',1300 from dual union all
select 36935,'1975-S321','1975-11-18 19:01:00','suborbital rocket','success',850,'NASA 32.01WT','','','','Test',1821,'',163 from dual union all
select 36936,'1975-S322','1975-11-19 17:45:00','suborbital rocket','success',839,'NASA 10.415GM','GSFC Grenade 282 W','','','Aeronomy',1821,'',120 from dual union all
select 36937,'1975-S323','1975-11-21 04:11:00','suborbital rocket','success',839,'NASA 10.423GM','GSFC Grenade 283 W','','','Aeronomy',1821,'',121 from dual union all
select 36938,'1975-S324','1975-11-21 16:37:36','suborbital rocket','success',837,'SSC SO1 (T1)','SSC Range Test','','','Test',417,'',131 from dual union all
select 36939,'1975-S325','1975-11-22 16:00:02','military missile','success',931,'A3T','DASO','','','OT',421,'UK S27',1000 from dual union all
select 3694,'1955-S44','1955-06-16 01:11:00','suborbital rocket','success',47,'USAF 56','','','','Classified',640,'',203 from dual union all
select 36940,'1975-S326','1975-11-24 15:15:00','suborbital rocket','success',1213,'SL1112','LUX XRS18/Saclay','','','XR Astronomy',1839,'',251 from dual union all
select 36941,'1975-S327','1975-11-25 23:39:18','suborbital rocket','success',220,'AEC-IVA-03','Periquito','','','Plasma Ba',334,'',500 from dual union all
select 36942,'1975-S328','1975-11-28 12:28:00','suborbital rocket','success',920,'P152H','Electron density','','','Ionospheric',1447,'',129 from dual union all
select 36943,'1975-S329','1975-11-28 12:56:00','suborbital rocket','success',920,'P151H','Atomic oxygen','','','Atmospheric',1447,'',147 from dual union all
select 36944,'1975-S33','1975-01-11 00:25:02','suborbital rocket','success',220,'Tordo II','Periquito','','','Plasma Ba',334,'',540 from dual union all
select 36945,'1975-S330','1975-11-28 23:39:18','suborbital rocket','success',220,'AEC-IVA-04','Periquito','','','Plasma Ba',334,'',500 from dual union all
select 36946,'1975-S331','1975-12-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',139,'ZUB',150 from dual union all
select 36947,'1975-S332','1975-12-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',139,'ZUB',150 from dual union all
select 36948,'1975-S333','1975-12-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',139,'ZUB',150 from dual union all
select 36949,'1975-S334','1975-12-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',139,'ZUB',150 from dual union all
select 3695,'1955-S45','1955-06-18 00:00:00','military missile','success',970,'No 318','','','','OT',636,'',100 from dual union all
select 36950,'1975-S335','1975-12-02 12:57:00','suborbital rocket','success',132,'CRL A30.311-8','Twilight D-Region','','','Aeron/Tech',1895,'',120 from dual union all
select 36951,'1975-S336','1975-12-02 13:50:00','suborbital rocket','success',132,'CRL A30.311-5','Twilight D-Region','','','Aeron/Tech',1895,'',124 from dual union all
select 36952,'1975-S337','1975-12-02 17:00:00','suborbital rocket','success',132,'CRL A30.311-7','Twilight D-Region','','','Aeron/Tech',1895,'',125 from dual union all
select 36953,'1975-S338','1975-12-03 00:35:00','suborbital rocket','success',132,'CRL A30.413-5','Twilight D-Region','','','Aeron/Tech',1895,'',125 from dual union all
select 36954,'1975-S339','1975-12-03 00:59:00','suborbital rocket','success',132,'CRL A30.205-7','Twilight D-Region','','','Aeron/Tech',1895,'',125 from dual union all
select 36955,'1975-S34','1975-01-10 02:27:56','military missile','success',1420,'B-27','Mk 6 RV','SOFT','','Target',1658,'',1300 from dual union all
select 36956,'1975-S340','1975-12-03 02:00:00','suborbital rocket','success',132,'CRL A30.413-4','Twilight D-Region','','','Aeron/Tech',1895,'',127 from dual union all
select 36957,'1975-S341','1975-12-03 16:57:00','suborbital rocket','success',839,'NASA 10.424GM','GSFC Grenade 284 W','','','Aeronomy',1821,'',122 from dual union all
select 36958,'1975-S342','1975-12-04 00:00:00','suborbital rocket','success',210,'AAF-NVB-01','','','','Test',428,'',340 from dual union all
select 36959,'1975-S343','1975-12-04 00:00:00','suborbital rocket','success',277,'','','','','Tech',871,'',200 from dual union all
select 3696,'1955-S46','1955-06-22 00:00:00','military missile','success',1013,'S1-9','8K51 LKI-9','','','Test',636,'',500 from dual union all
select 36960,'1975-S344','1975-12-04 00:00:00','military missile','success',1420,'B-41','Mk 6 RV','DG-4','','Target',1658,'',1300 from dual union all
select 36961,'1975-S345','1975-12-04 03:22:00','suborbital rocket','success',44,'CRL A35.191-1','Super HI STAR','','','IR Astron',1895,'',286 from dual union all
select 36962,'1975-S346','1975-12-04 05:25:00','suborbital rocket','success',839,'NASA 10.410GM','GSFC Grenade 285 W','','','Aeronomy',1821,'',120 from dual union all
select 36963,'1975-S347','1975-12-04 18:52:00','suborbital rocket','success',338,'WRE Co1002','','','','Aeron',1832,'',220 from dual union all
select 36964,'1975-S348','1975-12-06 06:05:00','suborbital rocket','success',44,'NASA 17.16UH','SAO XRT 2','','','XR Astron',1895,'',221 from dual union all
select 36965,'1975-S349','1975-12-06 07:42:00','suborbital rocket','success',37,'NASA 13.118DG','','','','UV Astron?',1895,'',186 from dual union all
select 36966,'1975-S35','1975-01-11 10:53:00','suborbital rocket','success',220,'','DLR G-BB4-66','','','Ionospheric',1435,'',280 from dual union all
select 36967,'1975-S350','1975-12-11 16:15:00','suborbital rocket','success',228,'NASA 21.32NP','SPAR 1','','','Micrograv',1873,'',225 from dual union all
select 36968,'1975-S351','1975-12-12 05:09:00','suborbital rocket','success',1468,'CRL A09.406-1','ESCAPE II (Sphere)','Sphere AF-4 (7in)','','Aeron',1774,'',164 from dual union all
select 36969,'1975-S352','1975-12-16 16:00:00','suborbital rocket','success',867,'NASA 12.31GT','','','','Test',1895,'',241 from dual union all
select 3697,'1955-S47','1955-06-23 00:00:00','military missile','success',1013,'S1-12','8K51 LKI-10','','','Test',636,'',500 from dual union all
select 36970,'1975-S353','1975-12-17 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT51GB','GLORY TRIP 51GB','OT',1708,'',1300 from dual union all
select 36971,'1975-S354','1975-12-17 11:04:00','suborbital rocket','success',1190,'INTA MS3CL-7501','Winter Anom','','','Aeron/Ionos',396,'',94 from dual union all
select 36972,'1975-S355','1975-09-08 00:00:00','military missile','success',77,'1','','','','Test',1343,'',110 from dual union all
select 36973,'1975-S356','1975-10-23 00:00:00','military missile','success',77,'2','','','','Test',1343,'',100 from dual union all
select 36974,'1975-S357','1975-01-01 00:00:00','suborbital rocket','success',1025,'','First Flight','','','Test',1467,'',130 from dual union all
select 36975,'1975-S358','1975-10-21 10:10:00','military missile','success',916,'AO-159             *','P-323 75 Fall 04','','','OT WS-225?',433,'',180 from dual union all
select 36976,'1975-S359','1975-04-28 23:45:00','suborbital rocket','success',334,'WRE C4018','O2 density','','','Aeron',1835,'',124 from dual union all
select 36977,'1975-S36','1975-01-14 20:56:00','suborbital rocket','success',1468,'CRL A09.301-8','ALADDIN 75 Rel Test','','','Aeron/Test',1821,'',235 from dual union all
select 36978,'1975-S360','1975-08-23 00:00:00','suborbital rocket','success',656,'K-10C-5','M-3S SRMC test','','','Test',839,'',100 from dual union all
select 36979,'1975-S361','1975-01-20 20:00:00','military missile','success',135,'Athena C','HEARTS II RV 106','V4-7487','','Reentry Test',1821,'',200 from dual union all
select 3698,'1955-S48','1955-06-24 00:00:00','military missile','success',970,'No 305','','','','OT',636,'',100 from dual union all
select 36980,'1975-S362','1975-01-01 00:00:00','military missile','success',644,'','MATSESTA 1','','','Reentry Test',636,'',null from dual union all
select 36981,'1975-S363','1975-01-01 00:00:00','military missile','success',644,'','MATSESTA 2','','','Reentry Test',636,'',null from dual union all
select 36982,'1975-S364','1975-11-06 19:08:00','suborbital rocket','success',331,'WRE C3009','Li release','','','Aeron',1835,'',150 from dual union all
select 36983,'1975-S365','1975-01-13 00:00:00','military missile','success',1044,'45721-612-02','8K98P UBP-2','','','Test',477,'',1000 from dual union all
select 36984,'1975-S366','1975-03-10 00:00:00','military missile','success',1044,'45721-612-03','8K98P UBP-3','','','Test',477,'',1000 from dual union all
select 36985,'1975-S367','1975-08-12 00:00:00','military missile','success',1044,'64646-812-54','8K98P UBP-4','','','Test',477,'',1000 from dual union all
select 36986,'1975-S368','1975-09-23 00:00:00','military missile','success',1037,'64647-221-44','8K98 UBP-26','','','Test',477,'',1000 from dual union all
select 36987,'1975-S369','1975-10-29 00:00:00','military missile','success',1037,'64647-221-41','8K98 UBP-27','','','Test',477,'',1000 from dual union all
select 36988,'1975-S37','1975-01-15 22:46:00','suborbital rocket','success',1468,'CRL A09.105-8','ALADDIN 75 Rel Al','','','Aeron',1821,'',159 from dual union all
select 36989,'1975-S370','1975-12-18 00:00:00','military missile','success',1044,'45721-612-04','8K98P UBP-5','','','Test',477,'',1000 from dual union all
select 3699,'1955-S49','1955-06-24 18:04:00','suborbital rocket','success',841,'','DAN-2 Sphere','AM7.02 ?','','Aeron',1821,'',105 from dual union all
select 36990,'1975-S371','1975-12-25 00:00:00','military missile','success',1037,'64647-221-42','8K98 UBP-28','','','Test',477,'',1000 from dual union all
select 36991,'1975-S372','1975-05-19 00:00:00','military missile','success',1322,'64521-61-869-34L','15Zh42 UBP-3','','','Test',477,'',1000 from dual union all
select 36992,'1975-S373','1975-08-26 00:00:00','military missile','success',1322,'64521-61-874-32L','15Zh42 UBP-4','','','Test',477,'',1000 from dual union all
select 36993,'1975-S374','1975-05-29 00:00:00','military missile','success',1449,'','8K84 10 RD UBP-5','','','OT',897,'',1000 from dual union all
select 36994,'1975-S375','1975-07-02 00:00:00','military missile','success',982,'','8K64 42 RD UBP-8','','','OT',1002,'',1210 from dual union all
select 36995,'1975-S376','1975-08-27 00:00:00','military missile','success',982,'','8K64 42 RD UBP-9','','','OT',1002,'',1210 from dual union all
select 36996,'1975-S377','1975-08-27 00:00:00','military missile','success',982,'','8K64 42 RD UBP-10','','','OT',1002,'',1210 from dual union all
select 36997,'1975-S378','1975-07-23 00:00:00','military missile','success',981,'','8K64 8 RD UBP-6','','','OT',1916,'',1210 from dual union all
select 36998,'1975-S379','1975-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-1','Duga-1','','OT',1911,'',1210 from dual union all
select 36999,'1975-S38','1975-01-17 09:35:00','suborbital rocket','success',668,'K-9M-49','Airglow','','','Aeron/Ionos',839,'',363 from dual;
insert into LAUNCH
select 37,'1944-A01','1944-01-27 00:00:00','military missile','failure',8,'17003','Mittelwerk Eval','','','Test',675,'',0 from dual union all
select 370,'1944-S337','1944-10-31 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3700,'1955-S50','1955-07-01 00:00:00','military missile','success',974,'','','','','Test',636,'',150 from dual union all
select 37000,'1975-S380','1975-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-2','Duga-1','','OT',1911,'',1210 from dual union all
select 37001,'1975-S381','1975-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-3','Duga-1','','OT',1911,'',1210 from dual union all
select 37002,'1975-S382','1975-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-4','Duga-1','','OT',1911,'',1210 from dual union all
select 37003,'1975-S383','1975-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-5','Duga-1','','OT',1911,'',1210 from dual union all
select 37004,'1975-S384','1975-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-6','Duga-1','','OT',1911,'',1210 from dual union all
select 37005,'1975-S385','1975-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-7','Duga-1','','OT',1911,'',1210 from dual union all
select 37006,'1975-S386','1975-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-8','Duga-1','','OT',1911,'',1210 from dual union all
select 37007,'1975-S387','1975-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-9','Duga-1','','OT',1911,'',1210 from dual union all
select 37008,'1975-S388','1975-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-10','Duga-1','','OT',1911,'',1210 from dual union all
select 37009,'1975-S389','1975-01-01 00:00:00','military missile','success',1450,'T01K','15A20 47 RD UBP-1','Duga-1','','OT',1911,'',1000 from dual union all
select 3701,'1955-S51','1955-07-01 00:00:00','military missile','success',974,'','','','','Test',636,'',150 from dual union all
select 37010,'1975-S39','1975-01-17 22:43:00','suborbital rocket','success',1468,'CRL A09.301-9','ALADDIN 75 Sph/TMA','Sphere 79/AF-1','','Aeron/Plasma',1821,'',190 from dual union all
select 37011,'1975-S390','1975-01-01 00:00:00','military missile','success',982,'','8K64U 4 RD UBP-1','Duga-1','','OT',368,'',1210 from dual union all
select 37012,'1975-S391','1975-01-01 00:00:00','military missile','success',982,'','8K64U 4 RD UBP-2','Duga-1','','OT',368,'',1210 from dual union all
select 37013,'1975-S392','1975-01-01 00:00:00','military missile','success',982,'','8K64U 4 RD UBP-3','Duga-1','','OT',368,'',1210 from dual union all
select 37014,'1975-S393','1975-01-01 00:00:00','military missile','success',982,'','8K64U 4 RD UBP-4','Duga-1','','OT',368,'',1210 from dual union all
select 37015,'1975-S394','1975-01-01 00:00:00','military missile','success',982,'','8K64U 4 RD UBP-5','Duga-1','','OT',368,'',1210 from dual union all
select 37016,'1975-S396','1975-06-25 11:57:00','suborbital rocket','success',1284,'INTA SLW-7501','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 37017,'1975-S397','1975-07-01 11:25:00','suborbital rocket','success',1284,'INTA SLW-7502','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 37018,'1975-S398','1975-09-03 11:20:00','suborbital rocket','success',1284,'INTA SLW-7503','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 37019,'1975-S399','1975-11-19 12:00:00','suborbital rocket','success',1284,'INTA SLW-7504','','','','Aeron',396,'',100 from dual union all
select 3702,'1955-S52','1955-07-01 00:00:00','military missile','success',974,'','','','','Test',636,'',150 from dual union all
select 37020,'1975-S40','1975-01-18 04:35:00','suborbital rocket','success',906,'CRL A10.213-1','ALADDIN 75 Mass Spec','','','Aeron',1821,'',159 from dual union all
select 37021,'1975-S400','1975-12-17 12:00:00','suborbital rocket','success',1284,'INTA SLW-7505','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 37022,'1975-S401','1975-12-31 10:46:00','suborbital rocket','success',1284,'INTA SLW-7506','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 37023,'1975-S402','1975-04-24 20:30:00','suborbital rocket','success',27,'WRE AH1015','Grenade/TMA','','','Aeron',1833,'',165 from dual union all
select 37024,'1975-S403','1975-08-18 04:31:00','suborbital rocket','success',334,'WRE C4008 (C3008)','Impedance probe','','','Aeron',1835,'',145 from dual union all
select 37025,'1975-S404','1975-10-03 17:45:00','suborbital rocket','success',1506,'T1-8039','','Instrumented Sphere 1','','Aeron/Test',1821,'',100 from dual union all
select 37026,'1975-S405','1975-01-01 00:00:00','military missile','success',922,'','15Zh45 LKI K-2','','','Test',822,'',500 from dual union all
select 37027,'1975-S406','1975-01-01 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 37028,'1975-S407','1975-01-01 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 37029,'1975-S408','1975-07-17 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 3703,'1955-S53','1955-06-28 00:00:00','military missile','success',1013,'S1-11','8K51 LKI-11','','','Test',636,'',500 from dual union all
select 37030,'1975-S409','1975-03-01 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 37031,'1975-S41','1975-01-18 04:45:00','suborbital rocket','success',906,'CRL A10.301-10','ALADDIN 75 NO/Photom','','','Aeron',1821,'',183 from dual union all
select 37032,'1975-S410','1975-03-01 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 37033,'1975-S411','1975-08-03 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 37034,'1975-S412','1975-10-23 00:00:00','military missile','unknown',980,'','8K65U UBP 29 RD','','','Test',636,'',675 from dual union all
select 37035,'1975-S42','1975-01-19 11:12:00','military missile','success',795,'','USN Mk 4 RV','Mk 4 SFT-04','SFT-04','Reentry Test',1698,'',1300 from dual union all
select 37036,'1975-S43','1975-01-19 18:36:00','military missile','success',999,'N22501-02','8K67 UBP-49 Svinets-5','GCh 8F674','','Test',519,'',1000 from dual union all
select 37037,'1975-S44','1975-01-20 18:30:00','suborbital rocket','success',228,'NASA 21.24US','CU-He2 + LPSP-EUV4','','','Solar EUV',1873,'',279 from dual union all
select 37038,'1975-S45','1975-01-23 11:05:00','suborbital rocket','success',668,'K-9M-50','','','','XR Astron',839,'',353 from dual union all
select 37039,'1975-S46','1975-01-25 12:40:00','suborbital rocket','unknown',920,'INTA MPGFD-7501','','','','Ionos',396,'',100 from dual union all
select 3704,'1955-S54','1955-07-20 00:00:00','military missile','success',992,'S0-3','','','','OT',636,'',170 from dual union all
select 37040,'1975-S47','1975-01-25 13:33:00','suborbital rocket','success',1189,'INTA MSCLO-7501','','','','Aeron/Ionos',396,'',91 from dual union all
select 37041,'1975-S48','1975-01-26 02:37:00','suborbital rocket','success',466,'E012A','ARAKS Northward','','','Ionos/Plasma',856,'',190 from dual union all
select 37042,'1975-S49','1975-01-27 20:05:00','military missile','success',999,'N22506-04','8K67 UBP-51 Svinets-5','GCh 8F675','','Test',515,'',1000 from dual union all
select 37043,'1975-S50','1975-01-28 00:00:00','suborbital rocket','success',867,'Barium cloud','','','','Magnetospheric',873,'',185 from dual union all
select 37044,'1975-S51','1975-01-29 15:20:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT48GM','GLORY TRIP 48GM','OT',1699,'',1300 from dual union all
select 37045,'1975-S52','1975-01-30 02:15:00','suborbital rocket','success',227,'AED-VB-43','','','','Auroral/Aeron',428,'',244 from dual union all
select 37046,'1975-S53','1975-01-30 12:15:00','suborbital rocket','unknown',920,'INTA MPGFD-7502','','','','Ionos',396,'',82 from dual union all
select 37047,'1975-S54','1975-01-30 14:10:00','suborbital rocket','success',1191,'INTA MS4CL-7501','','','','Aeron/Ionos',396,'',117 from dual union all
select 37048,'1975-S55','1975-01-31 00:00:00','suborbital rocket','success',867,'Barium cloud','','','','Magnetospheric',873,'',185 from dual union all
select 37049,'1975-S56','1975-02-01 03:00:00','suborbital rocket','success',41,'NASA 26.27GG','13-in RC Flight 3','','','UV Astron?',1895,'',220 from dual union all
select 3705,'1955-S55','1955-07-08 08:39:00','suborbital rocket','success',54,'NRL 23','Mass spectrometers','','','Ionos',1869,'',113 from dual union all
select 37050,'1975-S57','1975-02-03 09:16:00','suborbital rocket','success',44,'NASA 17.13CG','','LPARL XRT','','XR Astron',1895,'',253 from dual union all
select 37051,'1975-S58','1975-02-04 00:00:00','military missile','success',1004,'21L','15A14 LKI-32','GCh 15F142','','Test',510,'',1000 from dual union all
select 37052,'1975-S59','1975-02-05 14:40:00','military missile','failure',799,'','Mk 12 RV x 3','OT GT29GM-1','GLORY TRIP 29GM-1','OT',1705,'',500 from dual union all
select 37053,'1975-S60','1975-02-05 00:00:00','suborbital rocket','success',867,'Barium cloud','','','','Magnetospheric',873,'',185 from dual union all
select 37054,'1975-S61','1975-02-05 06:00:00','test rocket','success',454,'ETV-2','','','','Test',1475,'',167 from dual union all
select 37055,'1975-S62','1975-02-05 13:02:00','suborbital rocket','success',1189,'INTA MSCLO-7502','','','','Aeron/Ionos',396,'',90 from dual union all
select 37056,'1975-S63','1975-02-09 05:35:00','suborbital rocket','success',920,'P155T/C Electrojet','ISRO-18.12','TERLS-729','','Atmospheric',1467,'',170 from dual union all
select 37057,'1975-S64','1975-02-09 00:24:00','suborbital rocket','success',920,'P154T/C','ISRO-18.09','TERLS-726','','Atmospheric',1467,'',165 from dual union all
select 37058,'1975-S65','1975-02-09 00:29:00','suborbital rocket','success',920,'P158T/C','ISRO-18.10','TERLS-727','','Ionospheric',1467,'',130 from dual union all
select 37059,'1975-S66','1975-02-09 05:30:00','suborbital rocket','success',920,'P137T/C','ISRO-18.11','TERLS-728','','Ionos',1467,'',102 from dual union all
select 3706,'1955-S56','1955-07-09 00:00:00','military missile','success',1013,'S1-14','8K51 LKI-14','','','Test',636,'',500 from dual union all
select 37060,'1975-S67','1975-02-09 05:50:00','suborbital rocket','failure',290,'ISRO 5.23 CE-1','ISRO 5.23','TERLS-730','','Ionospheric',1467,'',150 from dual union all
select 37061,'1975-S68','1975-02-09 07:08:00','suborbital rocket','success',920,'P159T/C','ISRO-18.13','TERLS-731','','Ionospheric',1467,'',130 from dual union all
select 37062,'1975-S69','1975-02-10 05:28:10','suborbital rocket','success',837,'NASA 14.536UA','EUV airglow','','','Aeronomy',428,'',142 from dual union all
select 37063,'1975-S70','1975-02-11 02:16:56','suborbital rocket','success',624,'NASA 8.65UE','','','','Ionos',428,'',791 from dual union all
select 37064,'1975-S71','1975-02-13 04:43:00','suborbital rocket','success',837,'NASA 14.535UA','Auroral NO','','','Aeronomy',428,'',136 from dual union all
select 37065,'1975-S72','1975-02-13 13:50:00','suborbital rocket','success',41,'NASA 26.40UE','','','','Plasma',1895,'',295 from dual union all
select 37066,'1975-S73','1975-02-15 01:36:00','suborbital rocket','success',466,'E012B','ARAKS Eastward','','','Ionos/Plasma',856,'',185 from dual union all
select 37067,'1975-S74','1975-02-19 05:35:00','suborbital rocket','success',290,'ISRO 5.25 CE-4','ISRO 5.25 CO.04','TERLS-732','','Ionos',1467,'',160 from dual union all
select 37068,'1975-S75','1975-02-19 08:37:00','suborbital rocket','success',920,'P156T/C','ISRO-18.15','TERLS-733','','Atmospheric',1467,'',170 from dual union all
select 37069,'1975-S76','1975-02-19 08:42:00','suborbital rocket','success',920,'P138T/C','ISRO-18.14','TERLS-734','','Ionos',1467,'',135 from dual union all
select 3707,'1955-S57','1955-07-25 00:00:00','military missile','success',970,'No 104','','','','OT',636,'',100 from dual union all
select 37070,'1975-S77','1975-02-19 09:04:00','suborbital rocket','success',290,'ISRO 5.24 CE-2','ISRO 5.24','TERLS-735','','Ionospheric',1467,'',164 from dual union all
select 37071,'1975-S78','1975-02-19 13:32:00','suborbital rocket','success',920,'P157T/C','ISRO-18.17','TERLS-736','','Atmospheric',1467,'',165 from dual union all
select 37072,'1975-S79','1975-02-25 01:35:00','suborbital rocket','success',1213,'SL1301','Cul Wolter XUVS-2','','','Solar X',1839,'',282 from dual union all
select 37073,'1975-S80','1975-02-25 05:32:00','suborbital rocket','success',837,'NASA 14.537UA','Auroral NO','','','Aeronomy',428,'',138 from dual union all
select 37074,'1975-S81','1975-02-26 00:00:00','military missile','success',1004,'517T','15A14 K-4 ZP','GCh 15F143U1','','OT',496,'',1000 from dual union all
select 37075,'1975-S82','1975-02-26 22:50:00','suborbital rocket','success',132,'CRL A30.413-3','Gerdian Condenser','','','Aeron',1047,'',104 from dual union all
select 37076,'1975-S83','1975-02-26 22:45:00','military missile','success',1001,'','8K67MA LKI-8','GCh 15F142','','Test',511,'',1000 from dual union all
select 37077,'1975-S84','1975-02-28 00:00:00','suborbital rocket','success',1207,'SL1293','CHEVALINE S5','','','MIRV test',1839,'',210 from dual union all
select 37078,'1975-S85','1975-02-28 03:10:00','military missile','success',1001,'','8K67MA LKI-9','GCh 15F142','','Test',515,'',1000 from dual union all
select 37079,'1975-S86','1975-03-01 01:00:00','suborbital rocket','success',132,'CRL A30.311-6','IR/OH','','','Aeron',1047,'',111 from dual union all
select 3708,'1955-S58','1955-07-26 00:00:00','military missile','success',970,'No 119','','','','OT',636,'',100 from dual union all
select 37080,'1975-S87','1975-03-01 06:55:00','suborbital rocket','success',867,'NASA 18.159UE','','','','Plasma',1047,'',190 from dual union all
select 37081,'1975-S88','1975-03-04 07:39:30','suborbital rocket','success',850,'DNA IC506.14-2','ICECAP 75A','','','Auroral',1047,'',130 from dual union all
select 37082,'1975-S89','1975-03-05 16:36:18','suborbital rocket','success',837,'SSC S16/2','SSC S16/2 NOX','','','Aeronomy',408,'',102 from dual union all
select 37083,'1975-S90','1975-03-06 08:36:00','military missile','success',795,'','Mk 500 RB 101','Evader SFT-101','SFT-101','Reentry Test',1701,'',1300 from dual union all
select 37084,'1975-S91','1975-03-06 03:43:00','suborbital rocket','success',846,'DNA IC507.11-1A','ICECAP 75A','','','Auroral',1047,'',150 from dual union all
select 37085,'1975-S92','1975-03-06 06:16:30','suborbital rocket','success',846,'DNA IC507.11-3','ICECAP 75A','','','Auroral',1047,'',150 from dual union all
select 37086,'1975-S93','1975-03-07 00:22:55','suborbital rocket','success',291,'SSC S13 (C100)','SSC S13','','','Aeronomy',408,'',109 from dual union all
select 37087,'1975-S94','1975-03-08 00:00:00','suborbital rocket','success',41,'NASA 26.32UG','','','','XR Astron',1895,'',250 from dual union all
select 37088,'1975-S95','1975-03-10 09:12:20','suborbital rocket','success',846,'DNA IC507.11-2A','ICECAP 75A','','','Auroral',1047,'',152 from dual;
insert into LAUNCH
select 37089,'1975-S96','1975-03-10 21:19:00','suborbital rocket','success',36,'NASA 04.334UA','JHU Aurora 10','','','Aeronomy',428,'',172 from dual union all
select 3709,'1955-S59','1955-07-29 00:00:00','military missile','success',970,'No 0427','','','','OT',636,'',100 from dual union all
select 37090,'1975-S97','1975-03-11 19:30:00','suborbital rocket','success',1278,'','MTV-II-2','','','Reentry Test',1821,'',170 from dual union all
select 37091,'1975-S98','1975-03-11 06:33:09','suborbital rocket','success',548,'DNA IC511.21-1A','ICECAP','','','Auroral?',1047,'',183 from dual union all
select 37092,'1975-S99','1975-03-11 06:34:11','suborbital rocket','success',850,'DNA IC506.36-1','ICECAP 75A','','','Auroral',1047,'',130 from dual union all
select 37093,'1975-W01','1975-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 37094,'1975-W02','1975-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 37095,'1975-W03','1975-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 37096,'1975-W04','1975-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 37097,'1975-W05','1975-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 37098,'1975-W06','1975-01-02 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 37099,'1975-W07','1975-01-02 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 371,'1944-S338','1944-10-31 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3710,'1955-S60','1955-07-30 00:00:00','military missile','success',974,'','','','','Test',636,'',150 from dual union all
select 37100,'1975-W08','1975-01-03 15:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 37101,'1975-W09','1975-01-03 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 37102,'1975-W10','1975-01-03 15:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 37103,'1975-W100','1975-01-27 18:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 37104,'1975-W1000','1975-09-05 14:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 37105,'1975-W1001','1975-09-05 16:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37106,'1975-W1002','1975-09-05 18:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 37107,'1975-W1003','1975-09-06 00:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 37108,'1975-W1004','1975-09-08 12:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 37109,'1975-W1005','1975-09-08 16:26:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',52 from dual union all
select 3711,'1955-S61','1955-08-12 00:00:00','military missile','success',1013,'S1-17','8K51 LKI-16','','','Test',636,'',500 from dual union all
select 37110,'1975-W1006','1975-09-08 22:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',67 from dual union all
select 37111,'1975-W1007','1975-09-08 23:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',72 from dual union all
select 37112,'1975-W1008','1975-09-09 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 37113,'1975-W1009','1975-09-10 01:30:00','suborbital rocket','success',761,'MT135P T15','','','','Aeron?',1476,'',50 from dual union all
select 37114,'1975-W101','1975-01-27 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 37115,'1975-W1010','1975-09-10 02:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 37116,'1975-W1011','1975-09-10 06:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',77 from dual union all
select 37117,'1975-W1012','1975-09-10 09:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 37118,'1975-W1013','1975-09-10 14:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 37119,'1975-W1014','1975-09-10 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 3712,'1955-S62','1955-08-05 00:00:00','military missile','success',992,'S0-2','UGCh','','','Test',636,'',170 from dual union all
select 37120,'1975-W1015','1975-09-10 16:35:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 37121,'1975-W1016','1975-09-11 00:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 37122,'1975-W1017','1975-09-12 11:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 37123,'1975-W1018','1975-09-12 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 37124,'1975-W1019','1975-09-12 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 37125,'1975-W102','1975-01-27 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 37126,'1975-W1020','1975-09-12 16:52:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37127,'1975-W1021','1975-09-12 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37128,'1975-W1022','1975-09-12 20:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 37129,'1975-W1023','1975-09-12 22:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 3713,'1955-S63','1955-08-01 00:00:00','military missile','success',970,'No 0430','','','','OT',636,'',100 from dual union all
select 37130,'1975-W1024','1975-09-15 11:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 37131,'1975-W1025','1975-09-15 15:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 37132,'1975-W1026','1975-09-15 16:14:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 37133,'1975-W1027','1975-09-15 16:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 37134,'1975-W1028','1975-09-15 16:19:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',73 from dual union all
select 37135,'1975-W1029','1975-09-15 22:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 37136,'1975-W103','1975-01-28 15:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 37137,'1975-W1030','1975-09-16 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',65 from dual union all
select 37138,'1975-W1031','1975-09-16 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 37139,'1975-W1032','1975-09-16 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 3714,'1955-S64','1955-08-09 00:00:00','military missile','success',1013,'S1-18','8K51 LKI-15','','','Test',636,'',500 from dual union all
select 37140,'1975-W1033','1975-09-16 18:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 37141,'1975-W1034','1975-09-16 22:10:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',65 from dual union all
select 37142,'1975-W1035','1975-09-17 12:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 37143,'1975-W1036','1975-09-17 13:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 37144,'1975-W1037','1975-09-17 14:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',56 from dual union all
select 37145,'1975-W1038','1975-09-17 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',61 from dual union all
select 37146,'1975-W1039','1975-09-17 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 37147,'1975-W104','1975-01-28 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 37148,'1975-W1040','1975-09-17 15:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 37149,'1975-W1041','1975-09-17 16:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 3715,'1955-S65','1955-08-24 00:00:00','military missile','success',970,'No 302','','','','OT',636,'',100 from dual union all
select 37150,'1975-W1042','1975-09-17 16:56:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37151,'1975-W1043','1975-09-18 13:55:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37152,'1975-W1044','1975-09-18 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 37153,'1975-W1045','1975-09-19 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',60 from dual union all
select 37154,'1975-W1046','1975-09-19 16:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',50 from dual union all
select 37155,'1975-W1047','1975-09-19 17:57:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 37156,'1975-W1048','1975-09-19 21:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 37157,'1975-W1049','1975-09-20 18:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37158,'1975-W105','1975-01-28 20:13:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 37159,'1975-W1050','1975-09-21 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',50 from dual union all
select 3716,'1955-S66','1955-08-25 00:00:00','military missile','success',970,'No 0315','','','','OT',636,'',100 from dual union all
select 37160,'1975-W1051','1975-09-22 11:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 37161,'1975-W1052','1975-09-22 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 37162,'1975-W1053','1975-09-22 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 37163,'1975-W1054','1975-09-22 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 37164,'1975-W1055','1975-09-22 16:30:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37165,'1975-W1056','1975-09-22 18:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',55 from dual union all
select 37166,'1975-W1057','1975-09-23 12:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37167,'1975-W1058','1975-09-23 19:00:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',62 from dual union all
select 37168,'1975-W1059','1975-09-24 00:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 37169,'1975-W106','1975-01-29 00:35:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 3717,'1955-S67','1955-08-16 00:00:00','military missile','failure',1013,'S1-16','8K51 LKI-17','','','Test',636,'',100 from dual union all
select 37170,'1975-W1060','1975-09-24 12:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 37171,'1975-W1061','1975-09-24 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 37172,'1975-W1062','1975-09-24 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 37173,'1975-W1063','1975-09-24 15:49:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',66 from dual union all
select 37174,'1975-W1064','1975-09-24 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 37175,'1975-W1065','1975-09-24 17:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 37176,'1975-W1066','1975-09-24 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37177,'1975-W1067','1975-09-24 19:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 37178,'1975-W1068','1975-09-24 22:43:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',51 from dual;
insert into LAUNCH
select 37179,'1975-W1069','1975-09-25 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',77 from dual union all
select 3718,'1955-S68','1955-09-19 00:00:00','military missile','success',1013,'S1-20','8K51 LKI-18','','','Test',636,'',500 from dual union all
select 37180,'1975-W107','1975-01-29 09:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 37181,'1975-W1070','1975-09-25 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',70 from dual union all
select 37182,'1975-W1071','1975-09-25 10:55:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 37183,'1975-W1072','1975-09-25 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37184,'1975-W1073','1975-09-25 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 37185,'1975-W1074','1975-09-25 18:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 37186,'1975-W1075','1975-09-25 21:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 37187,'1975-W1076','1975-09-25 23:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 37188,'1975-W1077','1975-09-26 11:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37189,'1975-W1078','1975-09-26 12:59:59','suborbital rocket','success',88,'Gerdien','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 3719,'1955-S69','1955-09-23 00:00:00','military missile','success',1013,'S1-19','8K51 LKI-19','','','Test',636,'',500 from dual union all
select 37190,'1975-W1079','1975-09-26 13:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37191,'1975-W108','1975-01-29 11:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',72 from dual union all
select 37192,'1975-W1080','1975-09-26 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 37193,'1975-W1081','1975-09-26 15:45:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',58 from dual union all
select 37194,'1975-W1082','1975-09-26 16:45:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 37195,'1975-W1083','1975-09-26 16:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 37196,'1975-W1084','1975-09-29 12:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 37197,'1975-W1085','1975-09-29 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 37198,'1975-W1086','1975-09-29 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',56 from dual union all
select 37199,'1975-W1087','1975-09-29 17:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 372,'1944-S339','1944-10-31 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3720,'1955-S70','1955-08-31 00:11:00','military missile','success',1053,'RS-7     XI','','','','Test',280,'',90 from dual union all
select 37200,'1975-W1088','1975-09-29 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 37201,'1975-W1089','1975-09-29 22:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 37202,'1975-W109','1975-01-29 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',76 from dual union all
select 37203,'1975-W1090','1975-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 37204,'1975-W1091','1975-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 37205,'1975-W1092','1975-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 37206,'1975-W1093','1975-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 37207,'1975-W1094','1975-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 37208,'1975-W1095','1975-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 37209,'1975-W1096','1975-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 3721,'1955-S71','1955-09-12 14:32:00','military missile','success',974,'','','','','Test',190,'B-67',150 from dual union all
select 37210,'1975-W1097','1975-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 37211,'1975-W1098','1975-09-30 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 37212,'1975-W1099','1975-10-01 15:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 37213,'1975-W11','1975-01-03 16:43:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 37214,'1975-W110','1975-01-29 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 37215,'1975-W1100','1975-10-01 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 37216,'1975-W1101','1975-10-01 17:13:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 37217,'1975-W1102','1975-10-01 17:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 37218,'1975-W1103','1975-10-01 21:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',54 from dual union all
select 37219,'1975-W1104','1975-10-02 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 3722,'1955-S72','1955-09-14 13:30:00','suborbital rocket','success',47,'USAF 58','UM wind vanes','','','Test/Aeron',640,'',95 from dual union all
select 37220,'1975-W1105','1975-10-02 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37221,'1975-W1106','1975-10-02 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37222,'1975-W1107','1975-10-03 13:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 37223,'1975-W1108','1975-10-03 14:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 37224,'1975-W1109','1975-10-03 14:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 37225,'1975-W111','1975-01-30 01:30:00','suborbital rocket','success',761,'MT135P T13','','','','Aeron?',1476,'',50 from dual union all
select 37226,'1975-W1110','1975-10-03 15:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',68 from dual union all
select 37227,'1975-W1111','1975-10-03 15:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',57 from dual union all
select 37228,'1975-W1112','1975-10-03 16:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 37229,'1975-W1113','1975-10-03 19:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 3723,'1955-S73','1955-09-23 20:34:00','suborbital rocket','success',718,'','SUI 38','','','Ionos/Aeron?',81,'AGB-5',100 from dual union all
select 37230,'1975-W1114','1975-10-03 22:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 37231,'1975-W1115','1975-10-04 01:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 37232,'1975-W1116','1975-10-04 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 37233,'1975-W1117','1975-10-04 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37234,'1975-W1118','1975-10-05 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37235,'1975-W1119','1975-10-06 15:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 37236,'1975-W112','1975-01-30 02:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 37237,'1975-W1120','1975-10-06 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 37238,'1975-W1121','1975-10-06 18:04:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37239,'1975-W1122','1975-10-06 18:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',57 from dual union all
select 3724,'1955-S74','1955-09-24 16:35:00','suborbital rocket','success',370,'','SUI 39','','','Ionos/Aeron?',92,'AGB-5',100 from dual union all
select 37240,'1975-W1123','1975-10-07 08:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 37241,'1975-W1124','1975-10-07 09:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37242,'1975-W1125','1975-10-07 10:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37243,'1975-W1126','1975-10-07 16:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 37244,'1975-W1127','1975-10-08 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 37245,'1975-W1128','1975-10-08 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 37246,'1975-W1129','1975-10-08 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',57 from dual union all
select 37247,'1975-W113','1975-01-30 04:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37248,'1975-W1130','1975-10-08 16:27:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 37249,'1975-W1131','1975-10-08 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 3725,'1955-S75','1955-09-24 21:09:00','suborbital rocket','success',718,'','SUI 40','','','Ionos/Aeron?',100,'AGB-5',100 from dual union all
select 37250,'1975-W1132','1975-10-08 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37251,'1975-W1133','1975-10-08 19:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 37252,'1975-W1134','1975-10-09 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 37253,'1975-W1135','1975-10-09 18:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37254,'1975-W1136','1975-10-09 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 37255,'1975-W1137','1975-10-10 14:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37256,'1975-W1138','1975-10-10 15:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',75 from dual union all
select 37257,'1975-W1139','1975-10-10 15:52:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 37258,'1975-W114','1975-01-30 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 37259,'1975-W1140','1975-10-10 16:19:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',76 from dual union all
select 3726,'1955-S76','1955-09-27 19:43:00','suborbital rocket','success',370,'','NRL Rockoon 13','','','Ionos/Aeron?',91,'AGB-5',90 from dual union all
select 37260,'1975-W1141','1975-10-10 19:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',70 from dual union all
select 37261,'1975-W1142','1975-10-10 22:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',67 from dual union all
select 37262,'1975-W1143','1975-10-13 09:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 37263,'1975-W1144','1975-10-13 13:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 37264,'1975-W1145','1975-10-13 15:40:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',59 from dual union all
select 37265,'1975-W1146','1975-10-14 08:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 37266,'1975-W1147','1975-10-14 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 37267,'1975-W1148','1975-10-14 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 37268,'1975-W1149','1975-10-14 20:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual;
insert into LAUNCH
select 37269,'1975-W115','1975-02-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 3727,'1955-S77','1955-09-27 20:54:00','suborbital rocket','success',370,'','SUI 42','','','Ionos/Aeron?',102,'AGB-5',100 from dual union all
select 37270,'1975-W1150','1975-10-15 13:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',69 from dual union all
select 37271,'1975-W1151','1975-10-15 13:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 37272,'1975-W1152','1975-10-15 13:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 37273,'1975-W1153','1975-10-15 16:09:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',70 from dual union all
select 37274,'1975-W1154','1975-10-15 16:35:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 37275,'1975-W1155','1975-10-15 17:37:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 37276,'1975-W1156','1975-10-15 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 37277,'1975-W1157','1975-10-16 12:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 37278,'1975-W1158','1975-10-16 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 37279,'1975-W1159','1975-10-16 18:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',71 from dual union all
select 3728,'1955-S78','1955-09-28 01:13:00','suborbital rocket','success',718,'','SUI 43','','','Ionos/Aeron?',103,'AGB-5',100 from dual union all
select 37280,'1975-W116','1975-02-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 37281,'1975-W1160','1975-10-16 18:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 37282,'1975-W1161','1975-10-16 22:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 37283,'1975-W1162','1975-10-17 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',55 from dual union all
select 37284,'1975-W1163','1975-10-17 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 37285,'1975-W1164','1975-10-17 15:34:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',61 from dual union all
select 37286,'1975-W1165','1975-10-17 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',71 from dual union all
select 37287,'1975-W1166','1975-10-17 23:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',77 from dual union all
select 37288,'1975-W1167','1975-10-20 13:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37289,'1975-W1168','1975-10-20 15:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',68 from dual union all
select 3729,'1955-S79','1955-09-28 12:45:00','suborbital rocket','success',370,'','NRL Rockoon 15','','','Ionos/Aeron?',94,'AGB-5',90 from dual union all
select 37290,'1975-W1169','1975-10-20 16:51:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37291,'1975-W117','1975-01-31 09:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 37292,'1975-W1170','1975-10-20 17:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 37293,'1975-W1171','1975-10-20 17:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 37294,'1975-W1172','1975-10-20 18:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',75 from dual union all
select 37295,'1975-W1173','1975-10-20 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37296,'1975-W1174','1975-10-21 00:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 37297,'1975-W1175','1975-10-21 10:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 37298,'1975-W1176','1975-10-21 15:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 37299,'1975-W1177','1975-10-21 17:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 373,'1944-S34','1944-08-14 09:52:00','military missile','success',1470,'18301','Ma118','','','Training',661,'',90 from dual union all
select 3730,'1955-S80','1955-09-28 14:54:00','suborbital rocket','success',370,'','SUI 44','','','Ionos/Aeron?',104,'AGB-5',100 from dual union all
select 37300,'1975-W1178','1975-10-21 22:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 37301,'1975-W1179','1975-10-22 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 37302,'1975-W118','1975-01-31 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',78 from dual union all
select 37303,'1975-W1180','1975-10-22 13:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 37304,'1975-W1181','1975-10-22 14:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 37305,'1975-W1182','1975-10-22 14:55:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',53 from dual union all
select 37306,'1975-W1183','1975-10-22 15:12:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',53 from dual union all
select 37307,'1975-W1184','1975-10-22 15:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 37308,'1975-W1185','1975-10-22 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 37309,'1975-W1186','1975-10-22 19:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 3731,'1955-S81','1955-09-28 19:22:00','suborbital rocket','failure',372,'','SUI 45','','','Test',105,'AGB-5',102 from dual union all
select 37310,'1975-W1187','1975-10-22 23:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 37311,'1975-W1188','1975-10-22 23:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',52 from dual union all
select 37312,'1975-W1189','1975-10-23 00:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',69 from dual union all
select 37313,'1975-W119','1975-01-31 15:37:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',78 from dual union all
select 37314,'1975-W1190','1975-10-23 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 37315,'1975-W1191','1975-10-23 21:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 37316,'1975-W1192','1975-10-24 00:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',71 from dual union all
select 37317,'1975-W1193','1975-10-24 12:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 37318,'1975-W1194','1975-10-24 15:20:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',74 from dual union all
select 37319,'1975-W1195','1975-10-24 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 3732,'1955-S82','1955-09-29 13:42:00','suborbital rocket','success',370,'','SUI 46','','','Ionos/Aeron?',106,'AGB-5',100 from dual union all
select 37320,'1975-W1196','1975-10-24 16:34:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37321,'1975-W1197','1975-10-24 17:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 37322,'1975-W1198','1975-10-24 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 37323,'1975-W1199','1975-10-24 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37324,'1975-W12','1975-01-03 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 37325,'1975-W120','1975-01-31 16:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 37326,'1975-W1200','1975-10-24 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',51 from dual union all
select 37327,'1975-W1201','1975-10-27 15:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',69 from dual union all
select 37328,'1975-W1202','1975-10-27 16:30:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37329,'1975-W1203','1975-10-28 09:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 3733,'1955-S83','1955-09-29 19:13:00','suborbital rocket','success',370,'','SUI 47','','','Ionos/Aeron?',82,'AGB-5',100 from dual union all
select 37330,'1975-W1204','1975-10-28 17:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 37331,'1975-W1205','1975-10-28 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 37332,'1975-W1206','1975-10-29 12:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 37333,'1975-W1207','1975-10-29 14:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',62 from dual union all
select 37334,'1975-W1208','1975-10-29 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 37335,'1975-W1209','1975-10-29 17:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 37336,'1975-W121','1975-01-31 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37337,'1975-W1210','1975-10-29 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 37338,'1975-W1211','1975-10-29 17:25:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',70 from dual union all
select 37339,'1975-W1212','1975-10-29 18:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',53 from dual union all
select 3734,'1955-S84','1955-09-29 21:52:00','suborbital rocket','failure',372,'','SUI 48','','','Test',83,'AGB-5',100 from dual union all
select 37340,'1975-W1213','1975-10-29 21:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 37341,'1975-W1214','1975-10-29 23:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',52 from dual union all
select 37342,'1975-W1215','1975-10-30 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 37343,'1975-W1216','1975-10-30 16:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',56 from dual union all
select 37344,'1975-W1217','1975-10-30 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 37345,'1975-W1218','1975-10-31 00:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',71 from dual union all
select 37346,'1975-W1219','1975-10-31 07:12:00','suborbital rocket','success',229,'AKF-VI-07','[Not in FC]','','','Auroral',428,'',64 from dual union all
select 37347,'1975-W122','1975-01-31 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 37348,'1975-W1220','1975-10-31 15:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 37349,'1975-W1221','1975-10-31 15:49:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',75 from dual union all
select 3735,'1955-S85','1955-09-27 00:00:00','military missile','success',1013,'S1-24','8K51 LKI-20','','','Test',636,'',500 from dual union all
select 37350,'1975-W1222','1975-10-31 17:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 37351,'1975-W1223','1975-10-31 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37352,'1975-W1224','1975-10-31 21:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 37353,'1975-W1225','1975-11-03 12:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 37354,'1975-W1226','1975-11-03 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 37355,'1975-W1227','1975-11-03 16:34:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',54 from dual union all
select 37356,'1975-W1228','1975-11-03 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 37357,'1975-W1229','1975-11-03 19:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 37358,'1975-W123','1975-02-01 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',72 from dual;
insert into LAUNCH
select 37359,'1975-W1230','1975-11-04 00:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',71 from dual union all
select 3736,'1955-S86','1955-10-01 00:00:00','military missile','success',1013,'S1-25','8K51 LKI-21','','','Test',636,'',500 from dual union all
select 37360,'1975-W1231','1975-11-04 15:14:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',76 from dual union all
select 37361,'1975-W1232','1975-11-04 15:59:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 37362,'1975-W1233','1975-11-04 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37363,'1975-W1234','1975-11-04 19:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 37364,'1975-W1235','1975-11-04 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 37365,'1975-W1236','1975-11-05 12:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 37366,'1975-W1237','1975-11-05 15:45:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',67 from dual union all
select 37367,'1975-W1238','1975-11-05 16:46:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 37368,'1975-W1239','1975-11-05 17:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 37369,'1975-W124','1975-02-03 06:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 3737,'1955-S87','1955-09-30 20:10:00','suborbital rocket','success',718,'','SUI 49','','','Ionos/Aeron?',84,'AGB-5',100 from dual union all
select 37370,'1975-W1240','1975-11-05 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 37371,'1975-W1241','1975-11-05 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 37372,'1975-W1242','1975-11-05 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 37373,'1975-W1243','1975-11-06 12:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 37374,'1975-W1244','1975-11-06 15:40:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',78 from dual union all
select 37375,'1975-W1245','1975-11-06 16:14:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 37376,'1975-W1246','1975-11-06 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37377,'1975-W1247','1975-11-06 19:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',72 from dual union all
select 37378,'1975-W1248','1975-11-07 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',75 from dual union all
select 37379,'1975-W1249','1975-11-07 15:45:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',60 from dual union all
select 3738,'1955-S88','1955-10-04 21:18:00','suborbital rocket','success',718,'','SUI 50','','','Ionos/Aeron?',85,'AGB-5',100 from dual union all
select 37380,'1975-W125','1975-02-03 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37381,'1975-W1250','1975-11-07 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 37382,'1975-W1251','1975-11-07 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 37383,'1975-W1252','1975-11-07 19:07:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 37384,'1975-W1253','1975-11-07 20:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 37385,'1975-W1254','1975-11-07 22:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37386,'1975-W1255','1975-11-10 12:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 37387,'1975-W1256','1975-11-10 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 37388,'1975-W1257','1975-11-10 16:54:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 37389,'1975-W1258','1975-11-10 17:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 3739,'1955-S89','1955-10-05 00:00:00','military missile','success',970,'No K1-17','','','','OT',636,'',100 from dual union all
select 37390,'1975-W1259','1975-11-10 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37391,'1975-W126','1975-02-03 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 37392,'1975-W1260','1975-11-10 22:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 37393,'1975-W1261','1975-11-11 03:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',69 from dual union all
select 37394,'1975-W1262','1975-11-11 15:49:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',74 from dual union all
select 37395,'1975-W1263','1975-11-11 16:14:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37396,'1975-W1264','1975-11-12 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37397,'1975-W1265','1975-11-12 15:55:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',72 from dual union all
select 37398,'1975-W1266','1975-11-12 16:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 37399,'1975-W1267','1975-11-12 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 374,'1944-S340','1944-11-01 00:10:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3740,'1955-S90','1955-10-06 19:10:00','suborbital rocket','success',370,'','SUI 51','','','Ionos/Aeron?',86,'AGB-5',100 from dual union all
select 37400,'1975-W1268','1975-11-12 17:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 37401,'1975-W1269','1975-11-12 19:04:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37402,'1975-W127','1975-02-03 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 37403,'1975-W1270','1975-11-12 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 37404,'1975-W1271','1975-11-12 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 37405,'1975-W1272','1975-11-13 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 37406,'1975-W1273','1975-11-13 15:40:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',71 from dual union all
select 37407,'1975-W1274','1975-11-13 16:41:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37408,'1975-W1275','1975-11-13 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37409,'1975-W1276','1975-11-13 20:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',53 from dual union all
select 3741,'1955-S91','1955-10-08 00:00:00','military missile','success',970,'No K1-19','','','','OT',636,'',100 from dual union all
select 37410,'1975-W1277','1975-11-14 15:40:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',66 from dual union all
select 37411,'1975-W1278','1975-11-14 15:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 37412,'1975-W1279','1975-11-14 16:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 37413,'1975-W128','1975-02-03 18:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 37414,'1975-W1280','1975-11-14 17:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 37415,'1975-W1281','1975-11-17 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',76 from dual union all
select 37416,'1975-W1282','1975-11-17 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 37417,'1975-W1283','1975-11-17 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 37418,'1975-W1284','1975-11-17 21:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 37419,'1975-W1285','1975-11-17 22:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 3742,'1955-S92','1955-10-11 14:37:00','suborbital rocket','success',370,'','NRL Rockoon 18','','','Ionos/Aeron?',97,'AGB-5',90 from dual union all
select 37420,'1975-W1286','1975-11-18 15:32:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',1821,'',73 from dual union all
select 37421,'1975-W1287','1975-11-18 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 37422,'1975-W1288','1975-11-18 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 37423,'1975-W1289','1975-11-19 14:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',59 from dual union all
select 37424,'1975-W129','1975-02-03 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 37425,'1975-W1290','1975-11-19 16:01:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 37426,'1975-W1291','1975-11-19 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 37427,'1975-W1292','1975-11-19 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 37428,'1975-W1293','1975-11-19 17:16:00','suborbital rocket','success',1284,'G 1-7935','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37429,'1975-W1294','1975-11-19 17:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 3743,'1955-S93','1955-10-13 01:00:00','suborbital rocket','success',51,'USAF 60','Na Release 2','','','Aeronomy',640,'',101 from dual union all
select 37430,'1975-W1295','1975-11-19 18:00:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',1821,'',77 from dual union all
select 37431,'1975-W1296','1975-11-19 18:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',70 from dual union all
select 37432,'1975-W1297','1975-11-19 21:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',74 from dual union all
select 37433,'1975-W1298','1975-11-20 01:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 37434,'1975-W1299','1975-11-20 12:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37435,'1975-W13','1975-01-03 21:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 37436,'1975-W130','1975-02-03 21:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',78 from dual union all
select 37437,'1975-W1300','1975-11-20 16:20:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 37438,'1975-W1301','1975-11-20 18:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 37439,'1975-W1302','1975-11-21 01:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 3744,'1955-S94','1955-10-13 12:44:00','suborbital rocket','success',370,'','SUI 53','','','Ionos/Aeron?',88,'AGB-5',100 from dual union all
select 37440,'1975-W1303','1975-11-21 16:06:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37441,'1975-W1304','1975-11-21 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37442,'1975-W1305','1975-11-24 09:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 37443,'1975-W1306','1975-11-24 15:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 37444,'1975-W1307','1975-11-24 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 37445,'1975-W1308','1975-11-24 17:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 37446,'1975-W1309','1975-11-25 16:18:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37447,'1975-W131','1975-02-04 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 37448,'1975-W1310','1975-11-25 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual;
insert into LAUNCH
select 37449,'1975-W1311','1975-11-25 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 3745,'1955-S95','1955-10-13 20:13:00','suborbital rocket','success',370,'','SUI 55','','','Ionos/Aeron?',90,'AGB-5',100 from dual union all
select 37450,'1975-W1312','1975-11-26 14:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 37451,'1975-W1313','1975-11-26 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',72 from dual union all
select 37452,'1975-W1314','1975-11-26 16:17:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37453,'1975-W1315','1975-11-26 16:18:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 37454,'1975-W1316','1975-11-26 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',76 from dual union all
select 37455,'1975-W1317','1975-11-26 18:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',72 from dual union all
select 37456,'1975-W1318','1975-11-26 19:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 37457,'1975-W1319','1975-11-27 17:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 37458,'1975-W132','1975-02-04 16:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',76 from dual union all
select 37459,'1975-W1320','1975-11-27 18:59:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 3746,'1955-S96','1955-10-18 22:49:00','suborbital rocket','success',57,'NRL 34','Solar La/airglow','','','Solar UV/Solar X',1869,'',101 from dual union all
select 37460,'1975-W1321','1975-11-28 16:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37461,'1975-W1322','1975-12-01 16:17:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37462,'1975-W1323','1975-12-01 18:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 37463,'1975-W1324','1975-12-01 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37464,'1975-W1325','1975-12-01 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',73 from dual union all
select 37465,'1975-W1326','1975-12-01 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 37466,'1975-W1327','1975-12-01 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 37467,'1975-W1328','1975-12-02 12:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 37468,'1975-W1329','1975-12-02 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',56 from dual union all
select 37469,'1975-W133','1975-02-04 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 3747,'1955-S97','1955-10-22 00:20:00','suborbital rocket','success',57,'NRL 35','Solar La/airglow','','','Solar UV',1869,'',185 from dual union all
select 37470,'1975-W1330','1975-12-02 13:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',55 from dual union all
select 37471,'1975-W1331','1975-12-02 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37472,'1975-W1332','1975-12-02 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 37473,'1975-W1333','1975-12-02 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',52 from dual union all
select 37474,'1975-W1334','1975-12-02 17:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',72 from dual union all
select 37475,'1975-W1335','1975-12-02 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37476,'1975-W1336','1975-12-02 19:04:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37477,'1975-W1337','1975-12-02 21:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 37478,'1975-W1338','1975-12-02 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 37479,'1975-W1339','1975-12-02 22:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',55 from dual union all
select 3748,'1955-S98','1955-11-01 00:00:00','military missile','success',1013,'S1-23','8K51 LKI-22','','','Test',636,'',500 from dual union all
select 37480,'1975-W134','1975-02-04 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',76 from dual union all
select 37481,'1975-W1340','1975-12-03 14:24:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',56 from dual union all
select 37482,'1975-W1341','1975-12-03 15:32:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',1821,'',79 from dual union all
select 37483,'1975-W1342','1975-12-03 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 37484,'1975-W1343','1975-12-03 17:26:59','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',1821,'',74 from dual union all
select 37485,'1975-W1344','1975-12-03 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 37486,'1975-W1345','1975-12-03 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 37487,'1975-W1346','1975-12-03 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',56 from dual union all
select 37488,'1975-W1347','1975-12-04 06:00:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',1821,'',79 from dual union all
select 37489,'1975-W1348','1975-12-04 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 3749,'1955-S99','1955-11-04 00:00:00','military missile','success',1013,'S1-21','8K51 LKI-23','','','Test',636,'',500 from dual union all
select 37490,'1975-W1349','1975-12-04 19:13:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37491,'1975-W135','1975-02-05 04:23:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37492,'1975-W1350','1975-12-05 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 37493,'1975-W1351','1975-12-05 15:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',73 from dual union all
select 37494,'1975-W1352','1975-12-05 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37495,'1975-W1353','1975-12-05 22:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 37496,'1975-W1354','1975-12-08 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 37497,'1975-W1355','1975-12-08 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 37498,'1975-W1356','1975-12-08 17:25:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',67 from dual union all
select 37499,'1975-W1357','1975-12-08 17:47:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 375,'1944-S341','1944-11-01 03:08:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3750,'1956-A01','1956-01-01 00:00:00','suborbital rocket','success',122,'3','','','','Test',1057,'',30 from dual union all
select 37500,'1975-W1358','1975-12-08 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',73 from dual union all
select 37501,'1975-W1359','1975-12-09 16:14:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37502,'1975-W136','1975-02-05 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 37503,'1975-W1360','1975-12-09 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 37504,'1975-W1361','1975-12-09 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 37505,'1975-W1362','1975-12-09 22:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 37506,'1975-W1363','1975-12-10 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 37507,'1975-W1364','1975-12-10 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',72 from dual union all
select 37508,'1975-W1365','1975-12-10 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 37509,'1975-W1366','1975-12-10 17:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 3751,'1956-A02','1956-01-01 00:00:00','suborbital rocket','success',122,'4','','','','Test',1057,'',30 from dual union all
select 37510,'1975-W1367','1975-12-10 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37511,'1975-W1368','1975-12-10 18:36:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37512,'1975-W1369','1975-12-11 05:00:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',65 from dual union all
select 37513,'1975-W137','1975-02-05 17:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',76 from dual union all
select 37514,'1975-W1370','1975-12-11 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 37515,'1975-W1371','1975-12-11 16:26:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37516,'1975-W1372','1975-12-11 17:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 37517,'1975-W1373','1975-12-11 21:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37518,'1975-W1374','1975-12-12 04:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',66 from dual union all
select 37519,'1975-W1375','1975-12-12 05:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',72 from dual union all
select 3752,'1956-A03','1956-01-17 00:00:00','suborbital rocket','success',813,'I','Camp. 5 to Feb  2','','','Test/Release',666,'',40 from dual union all
select 37520,'1975-W1376','1975-12-12 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 37521,'1975-W1377','1975-12-12 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 37522,'1975-W1378','1975-12-12 18:57:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 37523,'1975-W1379','1975-12-12 22:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 37524,'1975-W138','1975-02-05 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37525,'1975-W1380','1975-12-13 02:15:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',50 from dual union all
select 37526,'1975-W1381','1975-12-14 02:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',65 from dual union all
select 37527,'1975-W1382','1975-12-15 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 37528,'1975-W1383','1975-12-15 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 37529,'1975-W1384','1975-12-15 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 3753,'1956-A04','1956-01-19 00:00:00','military missile','success',771,'GM-54-17','','','','Test',222,'',12 from dual union all
select 37530,'1975-W1385','1975-12-15 16:43:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',51 from dual union all
select 37531,'1975-W1386','1975-12-15 18:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 37532,'1975-W1387','1975-12-15 21:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 37533,'1975-W1388','1975-12-16 02:00:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',72 from dual union all
select 37534,'1975-W1389','1975-12-16 04:19:00','suborbital rocket','success',1281,'NASA 15.110GM','','','','Aeronomy',1470,'',77 from dual union all
select 37535,'1975-W139','1975-02-05 20:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',72 from dual union all
select 37536,'1975-W1390','1975-12-16 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37537,'1975-W1391','1975-12-16 22:43:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 37538,'1975-W1392','1975-12-17 04:14:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',68 from dual;
insert into LAUNCH
select 37539,'1975-W1393','1975-12-17 14:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',59 from dual union all
select 3754,'1956-A06','1956-01-25 00:00:00','military missile','success',771,'GM-54-18','','','','Test',222,'',12 from dual union all
select 37540,'1975-W1394','1975-12-17 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 37541,'1975-W1395','1975-12-17 16:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',59 from dual union all
select 37542,'1975-W1396','1975-12-17 18:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37543,'1975-W1397','1975-12-17 18:56:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 37544,'1975-W1398','1975-12-17 22:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',74 from dual union all
select 37545,'1975-W1399','1975-12-18 00:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 37546,'1975-W14','1975-01-04 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 37547,'1975-W140','1975-02-06 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',71 from dual union all
select 37548,'1975-W1400','1975-12-18 03:24:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',73 from dual union all
select 37549,'1975-W1401','1975-12-18 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 3755,'1956-A07','1956-01-26 00:00:00','atmospheric rocket','success',1308,'','E15 Model','','','Test',1821,'',10 from dual union all
select 37550,'1975-W1402','1975-12-18 17:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37551,'1975-W1403','1975-12-18 17:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 37552,'1975-W1404','1975-12-18 23:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 37553,'1975-W1405','1975-12-19 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',51 from dual union all
select 37554,'1975-W1406','1975-12-19 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',72 from dual union all
select 37555,'1975-W1407','1975-12-19 16:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 37556,'1975-W1408','1975-12-19 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37557,'1975-W1409','1975-12-19 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 37558,'1975-W141','1975-02-06 17:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 37559,'1975-W1410','1975-12-19 19:19:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 3756,'1956-A08','1956-01-27 00:00:00','military missile','success',771,'GM-54-16','','','','Test',222,'',12 from dual union all
select 37560,'1975-W1411','1975-12-20 02:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 37561,'1975-W1412','1975-12-20 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',59 from dual union all
select 37562,'1975-W1413','1975-12-21 22:19:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',58 from dual union all
select 37563,'1975-W1414','1975-12-22 02:13:00','suborbital rocket','success',1281,'NASA 15.146GM','','','','Aeronomy',1470,'',78 from dual union all
select 37564,'1975-W1415','1975-12-22 03:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',50 from dual union all
select 37565,'1975-W1416','1975-12-22 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 37566,'1975-W1417','1975-12-22 18:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37567,'1975-W1418','1975-12-22 18:56:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 37568,'1975-W1419','1975-12-23 16:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 37569,'1975-W142','1975-02-06 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 3757,'1956-A09','1956-01-27 00:00:00','military missile','success',1214,'N-69C 52-10971','','','','Test',224,'',10 from dual union all
select 37570,'1975-W1420','1975-12-23 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 37571,'1975-W1421','1975-12-23 16:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 37572,'1975-W1422','1975-12-24 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 37573,'1975-W1423','1975-12-24 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 37574,'1975-W1424','1975-12-24 19:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 37575,'1975-W1425','1975-12-24 23:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',65 from dual union all
select 37576,'1975-W1426','1975-12-26 22:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',54 from dual union all
select 37577,'1975-W1427','1975-12-29 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 37578,'1975-W1428','1975-12-29 19:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 37579,'1975-W1429','1975-12-29 21:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',54 from dual union all
select 3758,'1956-A10','1956-02-01 00:00:00','atmospheric rocket','success',365,'','Delta-wing model','','','Test',1821,'',10 from dual union all
select 37580,'1975-W143','1975-02-06 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37581,'1975-W1430','1975-12-30 00:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 37582,'1975-W1431','1975-12-30 18:34:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37583,'1975-W1432','1975-12-31 16:56:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 37584,'1975-W1433','1975-12-31 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 37585,'1975-W1434','1975-11-18 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37586,'1975-W1435','1975-11-18 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37587,'1975-W1436','1975-11-18 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37588,'1975-W1437','1975-11-19 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37589,'1975-W1438','1975-11-19 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 3759,'1956-A100','1956-10-05 00:00:00','atmospheric rocket','success',861,'','Heat Transfer','','','Test',1821,'',4 from dual union all
select 37590,'1975-W1439','1975-11-19 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37591,'1975-W144','1975-02-06 21:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 37592,'1975-W1440','1975-11-21 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37593,'1975-W1441','1975-11-21 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37594,'1975-W1442','1975-11-21 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37595,'1975-W1443','1975-11-21 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37596,'1975-W1444','1975-11-21 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37597,'1975-W1445','1975-12-03 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37598,'1975-W1446','1975-12-03 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37599,'1975-W1447','1975-12-03 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 376,'1944-S342','1944-11-01 16:27:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3760,'1956-A101','1956-10-09 00:00:00','atmospheric rocket','success',835,'','F23 Ramjet','','','Test',1821,'',10 from dual union all
select 37600,'1975-W1448','1975-12-04 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37601,'1975-W1449','1975-12-04 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37602,'1975-W145','1975-02-07 00:46:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 37603,'1975-W1450','1975-12-04 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37604,'1975-W1451','1975-12-04 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37605,'1975-W1452','1975-12-04 00:00:00','suborbital rocket','success',1284,'','Nimbus Comparison','','','Aeron',1821,'',70 from dual union all
select 37606,'1975-W1453','1975-02-05 00:00:00','suborbital rocket','success',229,'PF-1-75','Arctic test','','','Test',1047,'',71 from dual union all
select 37607,'1975-W1454','1975-02-05 00:00:00','suborbital rocket','success',229,'PF-2-75','Arctic test','','','Test',1047,'',72 from dual union all
select 37608,'1975-W1455','1975-02-06 00:00:00','suborbital rocket','success',229,'PF-3-75','Arctic test','','','Test',1047,'',73 from dual union all
select 37609,'1975-W1456','1975-02-06 00:00:00','suborbital rocket','success',229,'PF-4-75','Arctic test','','','Test',1047,'',69 from dual union all
select 3761,'1956-A102','1956-10-12 00:00:00','military missile','success',255,'624-6','','','','OT',254,'',20 from dual union all
select 37610,'1975-W1457','1975-02-07 00:00:00','suborbital rocket','success',229,'PF-5-75','Arctic test','','','Test',1047,'',70 from dual union all
select 37611,'1975-W1458','1975-02-07 00:00:00','suborbital rocket','success',229,'PF-6-75','Arctic test','','','Test',1047,'',74 from dual union all
select 37612,'1975-W1459','1975-03-04 00:00:00','suborbital rocket','success',229,'1-75','STS/Starute','','','Test',1895,'',74 from dual union all
select 37613,'1975-W146','1975-02-07 01:00:00','suborbital rocket','success',761,'MT135P T14','','','','Aeron?',1476,'',50 from dual union all
select 37614,'1975-W1460','1975-03-04 00:00:00','suborbital rocket','success',229,'2-75','STS/Starute','','','Test',1895,'',79 from dual union all
select 37615,'1975-W1461','1975-03-18 00:00:00','suborbital rocket','success',229,'3-75','STS/Starute','','','Test',1895,'',76 from dual union all
select 37616,'1975-W1462','1975-03-18 00:00:00','suborbital rocket','success',229,'4-75','STS/Starute','','','Test',1895,'',78 from dual union all
select 37617,'1975-W1463','1975-03-20 00:00:00','suborbital rocket','success',229,'5-75','12cm Sphere','','','Test',1895,'',78 from dual union all
select 37618,'1975-W1464','1975-03-25 00:00:00','suborbital rocket','success',229,'6-75','STS/Starute','','','Test',1895,'',82 from dual union all
select 37619,'1975-W1465','1975-03-25 00:00:00','suborbital rocket','success',229,'7-75','STS/Starute','','','Test',1895,'',79 from dual union all
select 3762,'1956-A103','1956-10-12 00:00:00','military missile','success',771,'GM-54-15','','','','Test',222,'',12 from dual union all
select 37620,'1975-W1466','1975-03-28 00:00:00','suborbital rocket','success',229,'8-75','STS/Starute','','','Test',1895,'',70 from dual union all
select 37621,'1975-W1467','1975-03-28 00:00:00','suborbital rocket','success',229,'9-75','STS/Starute','','','Test',1895,'',73 from dual union all
select 37622,'1975-W1468','1975-05-02 00:00:00','suborbital rocket','success',229,'10-75','12cm Sphere','','','Test',1895,'',78 from dual union all
select 37623,'1975-W147','1975-02-07 11:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',79 from dual union all
select 37624,'1975-W148','1975-02-07 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37625,'1975-W149','1975-02-07 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',78 from dual union all
select 37626,'1975-W15','1975-01-06 09:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 37627,'1975-W150','1975-02-07 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 37628,'1975-W151','1975-02-07 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual;
insert into LAUNCH
select 37629,'1975-W152','1975-02-08 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',71 from dual union all
select 3763,'1956-A104','1956-10-31 00:00:00','military missile','success',845,'1810-44','Drone intercept','','','Test',1895,'',10 from dual union all
select 37630,'1975-W153','1975-02-10 10:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 37631,'1975-W154','1975-02-10 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',56 from dual union all
select 37632,'1975-W155','1975-02-10 14:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 37633,'1975-W156','1975-02-10 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 37634,'1975-W157','1975-02-10 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37635,'1975-W158','1975-02-10 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 37636,'1975-W159','1975-02-11 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 37637,'1975-W16','1975-01-06 15:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37638,'1975-W160','1975-02-11 15:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',65 from dual union all
select 37639,'1975-W161','1975-02-11 18:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 3764,'1956-A105','1956-10-31 00:00:00','military missile','success',1214,'N-69C 51-17572','','','','Test',224,'',10 from dual union all
select 37640,'1975-W162','1975-02-11 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37641,'1975-W163','1975-02-11 20:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 37642,'1975-W164','1975-02-12 00:51:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 37643,'1975-W165','1975-02-12 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 37644,'1975-W166','1975-02-12 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37645,'1975-W167','1975-02-12 19:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 37646,'1975-W168','1975-02-12 19:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',72 from dual union all
select 37647,'1975-W169','1975-02-12 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 37648,'1975-W17','1975-01-06 19:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 37649,'1975-W170','1975-02-13 00:47:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 3765,'1956-A106','1956-10-31 02:04:00','military missile','failure',635,'RS-25    US','Jupiter A','','','Test',280,'',0 from dual union all
select 37650,'1975-W171','1975-02-13 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 37651,'1975-W172','1975-02-13 09:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37652,'1975-W173','1975-02-13 15:24:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',72 from dual union all
select 37653,'1975-W174','1975-02-13 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 37654,'1975-W175','1975-02-13 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 37655,'1975-W176','1975-02-13 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 37656,'1975-W177','1975-02-14 14:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',52 from dual union all
select 37657,'1975-W178','1975-02-14 14:45:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',72 from dual union all
select 37658,'1975-W179','1975-02-14 15:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 37659,'1975-W18','1975-01-06 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 3766,'1956-A107','1956-11-05 07:50:00','suborbital rocket','failure',49,'NRL 45 (RV-N-13b)','NRL NN3.02','IGY NN3.02','','Auroral',428,'',0 from dual union all
select 37660,'1975-W180','1975-02-14 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 37661,'1975-W181','1975-02-14 18:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 37662,'1975-W182','1975-02-14 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 37663,'1975-W183','1975-02-15 00:37:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37664,'1975-W184','1975-02-17 14:45:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',73 from dual union all
select 37665,'1975-W185','1975-02-18 00:47:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37666,'1975-W186','1975-02-18 15:40:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',65 from dual union all
select 37667,'1975-W187','1975-02-18 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 37668,'1975-W188','1975-02-18 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 37669,'1975-W189','1975-02-18 22:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 3767,'1956-A108','1956-11-06 00:00:00','military missile','success',832,'03','G-26 No. 1','','','Test',281,'',2 from dual union all
select 37670,'1975-W19','1975-01-07 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 37671,'1975-W190','1975-02-19 00:47:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 37672,'1975-W191','1975-02-19 09:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37673,'1975-W192','1975-02-19 15:24:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',67 from dual union all
select 37674,'1975-W193','1975-02-19 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',54 from dual union all
select 37675,'1975-W194','1975-02-19 18:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',72 from dual union all
select 37676,'1975-W195','1975-02-19 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37677,'1975-W196','1975-02-19 19:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',74 from dual union all
select 37678,'1975-W197','1975-02-19 22:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 37679,'1975-W198','1975-02-20 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 3768,'1956-A109','1956-11-06 00:00:00','atmospheric rocket','success',1051,'','Heat Transfer','','','Test',1821,'F2H2',4 from dual union all
select 37680,'1975-W199','1975-02-20 18:11:00','suborbital rocket','unknown',1284,'G 1-7753','','','','Test',1821,'',69 from dual union all
select 37681,'1975-W20','1975-01-07 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 37682,'1975-W200','1975-02-20 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 37683,'1975-W201','1975-02-20 19:27:00','suborbital rocket','unknown',1284,'G 1-7754','','','','Test',1821,'',67 from dual union all
select 37684,'1975-W202','1975-02-20 22:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 37685,'1975-W203','1975-02-20 22:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 37686,'1975-W204','1975-02-21 00:52:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 37687,'1975-W205','1975-02-21 01:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 37688,'1975-W206','1975-02-21 13:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 37689,'1975-W207','1975-02-21 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 3769,'1956-A11','1956-02-02 00:00:00','military missile','success',255,'623-13','','','','Test',272,'',20 from dual union all
select 37690,'1975-W208','1975-02-21 16:14:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',69 from dual union all
select 37691,'1975-W209','1975-02-21 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 37692,'1975-W21','1975-01-07 18:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 37693,'1975-W210','1975-02-21 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 37694,'1975-W211','1975-02-21 19:10:00','suborbital rocket','unknown',1284,'G 1-7755','','','','Aeron',1821,'',70 from dual union all
select 37695,'1975-W212','1975-02-21 19:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 37696,'1975-W213','1975-02-21 21:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 37697,'1975-W214','1975-02-22 01:03:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 37698,'1975-W215','1975-02-24 17:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 37699,'1975-W216','1975-02-24 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 377,'1944-S343','1944-11-05 00:31:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3770,'1956-A110','1956-11-09 00:00:00','military missile','success',771,'GM-54-127','','','','Test',222,'',12 from dual union all
select 37700,'1975-W217','1975-02-24 18:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 37701,'1975-W218','1975-02-24 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 37702,'1975-W219','1975-02-25 00:46:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 37703,'1975-W22','1975-01-07 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 37704,'1975-W220','1975-02-25 01:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 37705,'1975-W221','1975-02-25 14:49:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',65 from dual union all
select 37706,'1975-W222','1975-02-25 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 37707,'1975-W223','1975-02-25 18:47:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 37708,'1975-W224','1975-02-25 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 37709,'1975-W225','1975-02-25 22:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 3771,'1956-A111','1956-11-14 00:00:00','military missile','success',1214,'N-69D 52-10982','','','','Test',238,'',10 from dual union all
select 37710,'1975-W226','1975-02-26 13:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 37711,'1975-W227','1975-02-26 15:40:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',66 from dual union all
select 37712,'1975-W228','1975-02-26 16:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 37713,'1975-W229','1975-02-26 17:04:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 37714,'1975-W23','1975-01-08 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 37715,'1975-W230','1975-02-26 17:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 37716,'1975-W231','1975-02-26 20:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',64 from dual union all
select 37717,'1975-W232','1975-02-26 22:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 37718,'1975-W233','1975-02-27 00:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual;
insert into LAUNCH
select 37719,'1975-W234','1975-02-27 13:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 3772,'1956-A112','1956-11-21 00:00:00','military missile','success',771,'GM-54-128','','','','Test',222,'',12 from dual union all
select 37720,'1975-W235','1975-02-27 15:04:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',75 from dual union all
select 37721,'1975-W236','1975-02-27 16:41:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37722,'1975-W237','1975-02-27 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',58 from dual union all
select 37723,'1975-W238','1975-02-27 19:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 37724,'1975-W239','1975-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 37725,'1975-W24','1975-01-08 15:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 37726,'1975-W240','1975-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 37727,'1975-W241','1975-03-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 37728,'1975-W242','1975-03-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 37729,'1975-W243','1975-03-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 3773,'1956-A113','1956-11-27 00:00:00','atmospheric rocket','success',365,'','AGARD Model 2','','','Test',1821,'',10 from dual union all
select 37730,'1975-W244','1975-02-28 01:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 37731,'1975-W245','1975-02-28 12:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 37732,'1975-W246','1975-02-28 15:49:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',64 from dual union all
select 37733,'1975-W247','1975-02-28 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37734,'1975-W248','1975-02-28 18:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',57 from dual union all
select 37735,'1975-W249','1975-02-28 18:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 37736,'1975-W25','1975-01-08 15:48:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 37737,'1975-W250','1975-02-28 22:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 37738,'1975-W251','1975-03-01 00:00:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37739,'1975-W252','1975-03-01 00:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 3774,'1956-A114','1956-11-29 00:00:00','military missile','success',255,'624-2','','','','OT',254,'',20 from dual union all
select 37740,'1975-W253','1975-03-03 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 37741,'1975-W254','1975-03-03 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 37742,'1975-W255','1975-03-03 17:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 37743,'1975-W256','1975-03-03 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 37744,'1975-W257','1975-03-03 20:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 37745,'1975-W258','1975-03-03 23:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 37746,'1975-W259','1975-03-04 13:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 37747,'1975-W26','1975-01-08 21:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',72 from dual union all
select 37748,'1975-W260','1975-03-04 16:03:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',66 from dual union all
select 37749,'1975-W261','1975-03-04 16:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',78 from dual union all
select 3775,'1956-A115','1956-11-30 00:00:00','military missile','success',771,'GM-54-129','','','','Test',222,'',12 from dual union all
select 37750,'1975-W262','1975-03-04 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',70 from dual union all
select 37751,'1975-W263','1975-03-04 18:00:00','suborbital rocket','success',1061,'','Met Para,STS-','','','Meteo',1895,'',72 from dual union all
select 37752,'1975-W264','1975-03-04 18:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',60 from dual union all
select 37753,'1975-W265','1975-03-05 01:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 37754,'1975-W266','1975-03-05 02:36:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 37755,'1975-W267','1975-03-05 12:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 37756,'1975-W268','1975-03-05 14:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 37757,'1975-W269','1975-03-05 15:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 37758,'1975-W27','1975-01-09 01:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 37759,'1975-W270','1975-03-05 15:55:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',65 from dual union all
select 3776,'1956-A116','1956-12-03 04:20:00','atmospheric rocket','success',654,'K-1-4','K-I-T-1','','','Test',7,'',5 from dual union all
select 37760,'1975-W271','1975-03-05 18:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 37761,'1975-W272','1975-03-05 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37762,'1975-W273','1975-03-05 18:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',72 from dual union all
select 37763,'1975-W274','1975-03-06 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',72 from dual union all
select 37764,'1975-W275','1975-03-06 00:58:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 37765,'1975-W276','1975-03-06 04:42:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 37766,'1975-W277','1975-03-06 12:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 37767,'1975-W278','1975-03-06 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 37768,'1975-W279','1975-03-06 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',72 from dual union all
select 37769,'1975-W28','1975-01-09 15:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 3777,'1956-A117','1956-12-05 00:00:00','military missile','failure',1214,'N-69D 53-8172','Impact in Brazil','','','Test',224,'',10 from dual union all
select 37770,'1975-W280','1975-03-06 19:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 37771,'1975-W281','1975-03-07 00:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',66 from dual union all
select 37772,'1975-W282','1975-03-07 01:02:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37773,'1975-W283','1975-03-07 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',73 from dual union all
select 37774,'1975-W284','1975-03-07 16:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',75 from dual union all
select 37775,'1975-W285','1975-03-07 17:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 37776,'1975-W286','1975-03-07 19:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 37777,'1975-W287','1975-03-07 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 37778,'1975-W288','1975-03-07 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37779,'1975-W289','1975-03-07 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 3778,'1956-A118','1956-12-07 00:00:00','atmospheric rocket','success',839,'','Hemi-Cyl RV','','','Reentry Test',1821,'',30 from dual union all
select 37780,'1975-W29','1975-01-09 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',65 from dual union all
select 37781,'1975-W290','1975-03-08 01:02:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37782,'1975-W291','1975-03-10 13:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37783,'1975-W292','1975-03-10 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',51 from dual union all
select 37784,'1975-W293','1975-03-10 17:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 37785,'1975-W294','1975-03-10 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',60 from dual union all
select 37786,'1975-W295','1975-03-10 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37787,'1975-W296','1975-03-10 23:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 37788,'1975-W297','1975-03-11 01:09:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 37789,'1975-W298','1975-03-11 12:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 3779,'1956-A119','1956-12-08 03:32:00','atmospheric rocket','success',654,'K-1-5','K-I-T-2','','','Test',7,'',5 from dual union all
select 37790,'1975-W299','1975-03-11 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 37791,'1975-W30','1975-01-09 19:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 37792,'1975-W300','1975-03-11 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 37793,'1975-W301','1975-03-12 01:44:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',67 from dual union all
select 37794,'1975-W302','1975-03-12 12:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 37795,'1975-W303','1975-03-12 15:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 37796,'1975-W304','1975-03-12 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 37797,'1975-W305','1975-03-12 19:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 37798,'1975-W306','1975-03-12 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 37799,'1975-W307','1975-03-12 19:49:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',79 from dual union all
select 378,'1944-S344','1944-11-05 01:30:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3780,'1956-A12','1956-02-06 00:00:00','suborbital rocket','success',122,'1','','','','Test',1895,'',30 from dual union all
select 37800,'1975-W308','1975-03-12 20:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',76 from dual union all
select 37801,'1975-W309','1975-03-13 12:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 37802,'1975-W31','1975-01-10 09:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 37803,'1975-W310','1975-03-13 16:15:00','suborbital rocket','success',1281,'NASA 15.137GM','','','','Aeronomy',1895,'',78 from dual union all
select 37804,'1975-W311','1975-03-13 17:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 37805,'1975-W312','1975-03-13 17:26:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 37806,'1975-W313','1975-03-13 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',57 from dual union all
select 37807,'1975-W314','1975-03-14 11:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 37808,'1975-W315','1975-03-14 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',50 from dual;
insert into LAUNCH
select 37809,'1975-W316','1975-03-14 17:08:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 3781,'1956-A120','1956-12-11 03:26:00','atmospheric rocket','success',654,'K-1-6','K-I-T-3','','','Test',7,'',5 from dual union all
select 37810,'1975-W317','1975-03-14 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',53 from dual union all
select 37811,'1975-W318','1975-03-14 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 37812,'1975-W319','1975-03-14 18:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 37813,'1975-W32','1975-01-10 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 37814,'1975-W320','1975-03-14 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 37815,'1975-W321','1975-03-14 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 37816,'1975-W322','1975-03-17 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',51 from dual union all
select 37817,'1975-W323','1975-03-17 15:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37818,'1975-W324','1975-03-17 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 37819,'1975-W325','1975-03-17 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 3782,'1956-A121','1956-12-13 00:00:00','suborbital rocket','success',1060,'RA-1','Rockair 2710','','','Test',646,'F-86D',44 from dual union all
select 37820,'1975-W326','1975-03-17 17:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 37821,'1975-W327','1975-03-17 17:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 37822,'1975-W328','1975-03-17 22:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 37823,'1975-W329','1975-03-18 09:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 37824,'1975-W33','1975-01-10 16:35:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 37825,'1975-W330','1975-03-18 15:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 37826,'1975-W331','1975-03-18 16:30:00','suborbital rocket','success',1061,'','Starute,STS-','','','Meteo',1895,'',73 from dual union all
select 37827,'1975-W332','1975-03-18 16:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 37828,'1975-W333','1975-03-18 17:30:00','suborbital rocket','success',1061,'','Starute,STS-','','','Meteo',1895,'',72 from dual union all
select 37829,'1975-W334','1975-03-18 17:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',59 from dual union all
select 3783,'1956-A122','1956-12-13 02:10:00','atmospheric rocket','success',654,'K-1-7','K-I-T-4','','','Test',7,'',5 from dual union all
select 37830,'1975-W335','1975-03-18 22:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 37831,'1975-W336','1975-03-19 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 37832,'1975-W337','1975-03-19 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37833,'1975-W338','1975-03-19 17:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',53 from dual union all
select 37834,'1975-W339','1975-03-19 17:35:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 37835,'1975-W34','1975-01-10 17:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 37836,'1975-W340','1975-03-19 18:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 37837,'1975-W341','1975-03-20 13:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 37838,'1975-W342','1975-03-20 15:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 37839,'1975-W343','1975-03-20 16:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 3784,'1956-A123','1956-12-14 00:00:00','suborbital rocket','success',1060,'RA-2','','','','Test',646,'F-86D',42 from dual union all
select 37840,'1975-W344','1975-03-20 16:57:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 37841,'1975-W345','1975-03-21 13:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37842,'1975-W346','1975-03-21 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 37843,'1975-W347','1975-03-21 13:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 37844,'1975-W348','1975-03-21 16:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 37845,'1975-W349','1975-03-21 17:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',51 from dual union all
select 37846,'1975-W35','1975-01-10 17:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37847,'1975-W350','1975-03-21 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',77 from dual union all
select 37848,'1975-W351','1975-03-21 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37849,'1975-W352','1975-03-21 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',77 from dual union all
select 3785,'1956-A124','1956-12-17 00:00:00','suborbital rocket','success',1060,'RA-3','','','','Test',646,'F-86D',43 from dual union all
select 37850,'1975-W353','1975-03-22 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 37851,'1975-W354','1975-03-24 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37852,'1975-W355','1975-03-24 17:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 37853,'1975-W356','1975-03-24 17:24:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 37854,'1975-W357','1975-03-24 17:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',71 from dual union all
select 37855,'1975-W358','1975-03-24 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 37856,'1975-W359','1975-03-24 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37857,'1975-W36','1975-01-10 19:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 37858,'1975-W360','1975-03-25 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 37859,'1975-W361','1975-03-25 14:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 3786,'1956-A125','1956-12-19 00:00:00','suborbital rocket','success',1060,'RA-4','','','','Test',646,'F-86D',45 from dual union all
select 37860,'1975-W362','1975-03-25 17:06:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 37861,'1975-W363','1975-03-25 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 37862,'1975-W364','1975-03-25 18:00:00','suborbital rocket','success',1061,'','Starute,STS-','','','Meteo',1895,'',73 from dual union all
select 37863,'1975-W365','1975-03-25 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 37864,'1975-W366','1975-03-26 13:08:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37865,'1975-W367','1975-03-26 14:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',73 from dual union all
select 37866,'1975-W368','1975-03-26 16:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 37867,'1975-W369','1975-03-26 16:47:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 37868,'1975-W37','1975-01-11 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 37869,'1975-W370','1975-03-26 17:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 3787,'1956-A126','1956-12-20 00:00:00','atmospheric rocket','success',552,'','Heat Transfer','','','Reentry Test',1821,'',30 from dual union all
select 37870,'1975-W371','1975-03-26 20:51:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',65 from dual union all
select 37871,'1975-W372','1975-03-26 22:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 37872,'1975-W373','1975-03-27 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 37873,'1975-W374','1975-03-27 12:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 37874,'1975-W375','1975-03-27 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 37875,'1975-W376','1975-03-27 18:10:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 37876,'1975-W377','1975-03-27 19:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 37877,'1975-W378','1975-03-28 00:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 37878,'1975-W379','1975-03-28 12:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 37879,'1975-W38','1975-01-13 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 3788,'1956-A127','1956-12-20 00:00:00','military missile','success',1214,'N-69D 53-8171','','','','Test',238,'',10 from dual union all
select 37880,'1975-W380','1975-03-28 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 37881,'1975-W381','1975-03-28 17:15:00','suborbital rocket','success',1061,'','Starute,STS-','','','Meteo',1895,'',68 from dual union all
select 37882,'1975-W382','1975-03-28 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 37883,'1975-W383','1975-04-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 37884,'1975-W384','1975-04-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 37885,'1975-W385','1975-04-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 37886,'1975-W386','1975-03-31 13:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 37887,'1975-W387','1975-03-31 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 37888,'1975-W388','1975-03-31 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',54 from dual union all
select 37889,'1975-W389','1975-03-31 17:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 3789,'1956-A128','1956-10-10 00:00:00','atmospheric rocket','success',836,'','BTL PIT 1','','','Test',1890,'',10 from dual union all
select 37890,'1975-W39','1975-01-13 17:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 37891,'1975-W390','1975-03-31 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 37892,'1975-W391','1975-03-31 21:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 37893,'1975-W392','1975-04-01 01:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 37894,'1975-W393','1975-04-01 12:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 37895,'1975-W394','1975-04-01 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',77 from dual union all
select 37896,'1975-W395','1975-04-01 18:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 37897,'1975-W396','1975-04-01 23:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 37898,'1975-W397','1975-04-02 00:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 37899,'1975-W398','1975-04-02 12:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual;
insert into LAUNCH
select 379,'1944-S345','1944-11-05 05:40:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3790,'1956-A129','1956-01-11 00:00:00','atmospheric rocket','success',836,'1249B-7033','Nike 498R','','','Test',1895,'',10 from dual union all
select 37900,'1975-W399','1975-04-02 17:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 37901,'1975-W40','1975-01-13 20:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 37902,'1975-W400','1975-04-02 19:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',73 from dual union all
select 37903,'1975-W401','1975-04-02 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37904,'1975-W402','1975-04-02 23:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',70 from dual union all
select 37905,'1975-W403','1975-04-03 00:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 37906,'1975-W404','1975-04-03 12:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 37907,'1975-W405','1975-04-03 15:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 37908,'1975-W406','1975-04-03 17:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 37909,'1975-W407','1975-04-03 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 3791,'1956-A13','1956-02-06 00:00:00','military missile','success',771,'GM-54-12','','','','Test',222,'',12 from dual union all
select 37910,'1975-W408','1975-04-03 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 37911,'1975-W409','1975-04-04 10:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 37912,'1975-W41','1975-01-13 20:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 37913,'1975-W410','1975-04-04 13:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37914,'1975-W411','1975-04-04 17:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 37915,'1975-W412','1975-04-04 19:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',63 from dual union all
select 37916,'1975-W413','1975-04-05 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 37917,'1975-W414','1975-04-07 08:57:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 37918,'1975-W415','1975-04-07 12:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 37919,'1975-W416','1975-04-07 17:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 3792,'1956-A130','1956-01-11 00:00:00','atmospheric rocket','success',836,'1249B-7027','Nike 499R','','','Test',1895,'',10 from dual union all
select 37920,'1975-W417','1975-04-07 17:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 37921,'1975-W418','1975-04-07 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 37922,'1975-W419','1975-04-07 18:51:00','suborbital rocket','unknown',1284,'G 1-7756','','','','Aeron',1821,'',70 from dual union all
select 37923,'1975-W42','1975-01-14 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 37924,'1975-W420','1975-04-07 23:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 37925,'1975-W421','1975-04-08 01:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',68 from dual union all
select 37926,'1975-W422','1975-04-08 08:55:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 37927,'1975-W423','1975-04-08 17:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',76 from dual union all
select 37928,'1975-W424','1975-04-08 17:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',72 from dual union all
select 37929,'1975-W425','1975-04-08 18:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 3793,'1956-A131','1956-01-11 00:00:00','atmospheric rocket','success',836,'1249B-5840','Nike 500R','','','Test',1895,'',10 from dual union all
select 37930,'1975-W426','1975-04-08 22:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 37931,'1975-W427','1975-04-09 08:47:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 37932,'1975-W428','1975-04-09 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',69 from dual union all
select 37933,'1975-W429','1975-04-09 14:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 37934,'1975-W43','1975-01-14 14:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 37935,'1975-W430','1975-04-09 16:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',58 from dual union all
select 37936,'1975-W431','1975-04-09 17:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 37937,'1975-W432','1975-04-09 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 37938,'1975-W433','1975-04-09 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37939,'1975-W434','1975-04-10 00:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 3794,'1956-A132','1956-01-11 00:00:00','atmospheric rocket','success',836,'1249B-6204','Nike 501R','','','Test',1895,'',10 from dual union all
select 37940,'1975-W435','1975-04-10 08:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',57 from dual union all
select 37941,'1975-W436','1975-04-10 10:27:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 37942,'1975-W437','1975-04-10 12:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37943,'1975-W438','1975-04-10 16:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',77 from dual union all
select 37944,'1975-W439','1975-04-10 21:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 37945,'1975-W44','1975-01-14 16:24:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 37946,'1975-W440','1975-04-11 09:11:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 37947,'1975-W441','1975-04-11 14:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 37948,'1975-W442','1975-04-11 16:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 37949,'1975-W443','1975-04-11 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 3795,'1956-A133','1956-01-11 00:00:00','atmospheric rocket','success',836,'1249B-5154','Nike 502R','','','Test',1895,'',10 from dual union all
select 37950,'1975-W444','1975-04-14 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 37951,'1975-W445','1975-04-14 15:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 37952,'1975-W446','1975-04-14 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',64 from dual union all
select 37953,'1975-W447','1975-04-14 17:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 37954,'1975-W448','1975-04-14 19:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37955,'1975-W449','1975-04-14 21:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 37956,'1975-W45','1975-01-14 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 37957,'1975-W450','1975-04-15 12:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 37958,'1975-W451','1975-04-15 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 37959,'1975-W452','1975-04-16 09:46:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 3796,'1956-A134','1956-01-18 00:00:00','atmospheric rocket','success',836,'1249B-5179','Nike 503R','','','Test',1895,'',10 from dual union all
select 37960,'1975-W453','1975-04-16 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 37961,'1975-W454','1975-04-16 14:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 37962,'1975-W455','1975-04-16 15:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 37963,'1975-W456','1975-04-16 17:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',72 from dual union all
select 37964,'1975-W457','1975-04-16 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',52 from dual union all
select 37965,'1975-W458','1975-04-17 16:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 37966,'1975-W459','1975-04-17 17:24:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',72 from dual union all
select 37967,'1975-W46','1975-01-14 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 37968,'1975-W460','1975-04-17 20:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 37969,'1975-W461','1975-04-18 09:35:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 3797,'1956-A135','1956-01-18 00:00:00','atmospheric rocket','success',836,'1249B-6961','Nike 504R','','','Test',1895,'',10 from dual union all
select 37970,'1975-W462','1975-04-18 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',69 from dual union all
select 37971,'1975-W463','1975-04-18 14:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',72 from dual union all
select 37972,'1975-W464','1975-04-18 16:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',71 from dual union all
select 37973,'1975-W465','1975-04-19 01:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 37974,'1975-W466','1975-04-21 17:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 37975,'1975-W467','1975-04-21 17:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',67 from dual union all
select 37976,'1975-W468','1975-04-21 17:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 37977,'1975-W469','1975-04-21 21:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 37978,'1975-W47','1975-01-17 21:00:00','suborbital rocket','success',1284,'','ALADDIN 75','','','Aeron',1821,'',75 from dual union all
select 37979,'1975-W470','1975-04-22 00:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 3798,'1956-A136','1956-01-18 00:00:00','atmospheric rocket','success',836,'1249B-6972','Nike 505R','','','Test',1895,'',10 from dual union all
select 37980,'1975-W471','1975-04-22 12:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',54 from dual union all
select 37981,'1975-W472','1975-04-22 14:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 37982,'1975-W473','1975-04-22 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 37983,'1975-W474','1975-04-23 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 37984,'1975-W475','1975-04-23 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 37985,'1975-W476','1975-04-23 14:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',74 from dual union all
select 37986,'1975-W477','1975-04-23 17:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',61 from dual union all
select 37987,'1975-W478','1975-04-23 19:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 37988,'1975-W479','1975-04-23 20:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 37989,'1975-W48','1975-01-17 21:50:00','suborbital rocket','success',1284,'','ALADDIN 75','','','Aeron',1821,'',75 from dual;
insert into LAUNCH
select 3799,'1956-A137','1956-01-25 00:00:00','atmospheric rocket','success',836,'1249B-5155','Nike 506R','','','Test',1895,'',10 from dual union all
select 37990,'1975-W480','1975-04-23 21:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 37991,'1975-W481','1975-04-24 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 37992,'1975-W482','1975-04-24 02:54:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',54 from dual union all
select 37993,'1975-W483','1975-04-24 12:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 37994,'1975-W484','1975-04-24 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 37995,'1975-W485','1975-04-24 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 37996,'1975-W486','1975-04-24 21:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 37997,'1975-W487','1975-04-25 14:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 37998,'1975-W488','1975-04-25 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',69 from dual union all
select 37999,'1975-W489','1975-04-25 20:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 38,'1944-A02','1944-01-29 21:30:00','military missile','failure',8,'','','','','Test',662,'',0 from dual union all
select 380,'1944-S346','1944-11-05 08:50:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3800,'1956-A138','1956-01-25 00:00:00','atmospheric rocket','success',836,'1249B-6962','Nike 507R','','','Test',1895,'',10 from dual union all
select 38000,'1975-W49','1975-01-15 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 38001,'1975-W490','1975-04-26 00:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',66 from dual union all
select 38002,'1975-W491','1975-04-28 15:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',53 from dual union all
select 38003,'1975-W492','1975-04-28 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 38004,'1975-W493','1975-04-28 16:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 38005,'1975-W494','1975-04-28 17:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',72 from dual union all
select 38006,'1975-W495','1975-04-28 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 38007,'1975-W496','1975-04-29 00:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 38008,'1975-W497','1975-04-29 12:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 38009,'1975-W498','1975-04-29 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 3801,'1956-A139','1956-01-25 00:00:00','atmospheric rocket','success',836,'1249B-5250','Nike 508R','','','Test',1895,'',10 from dual union all
select 38010,'1975-W499','1975-04-29 16:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',62 from dual union all
select 38011,'1975-W50','1975-01-15 14:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 38012,'1975-W500','1975-04-29 21:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 38013,'1975-W501','1975-05-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 38014,'1975-W502','1975-05-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38015,'1975-W503','1975-05-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38016,'1975-W504','1975-05-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38017,'1975-W505','1975-05-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38018,'1975-W506','1975-04-30 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 38019,'1975-W507','1975-04-30 15:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 3802,'1956-A14','1956-02-06 00:00:00','military missile','success',771,'GM-54-11','','','','Test',222,'',12 from dual union all
select 38020,'1975-W508','1975-04-30 16:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 38021,'1975-W509','1975-04-30 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 38022,'1975-W51','1975-01-15 16:07:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 38023,'1975-W510','1975-04-30 18:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',75 from dual union all
select 38024,'1975-W511','1975-05-01 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 38025,'1975-W512','1975-05-01 11:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 38026,'1975-W513','1975-05-02 16:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 38027,'1975-W514','1975-05-02 16:58:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 38028,'1975-W515','1975-05-02 17:03:59','suborbital rocket','success',1061,'','Starute,Exp. Sonde','','','Meteo',1821,'',66 from dual union all
select 38029,'1975-W516','1975-05-02 17:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 3803,'1956-A140','1956-02-01 00:00:00','atmospheric rocket','success',836,'1249B-5244','Nike 509R','','','Test',1895,'',10 from dual union all
select 38030,'1975-W517','1975-05-02 19:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 38031,'1975-W518','1975-05-02 20:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 38032,'1975-W519','1975-05-03 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 38033,'1975-W52','1975-01-15 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 38034,'1975-W520','1975-05-05 09:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 38035,'1975-W521','1975-05-05 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',54 from dual union all
select 38036,'1975-W522','1975-05-05 14:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 38037,'1975-W523','1975-05-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 38038,'1975-W524','1975-05-05 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',69 from dual union all
select 38039,'1975-W525','1975-05-05 17:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 3804,'1956-A141','1956-02-01 00:00:00','atmospheric rocket','success',836,'1249B-5252','Nike 510R','','','Test',1895,'',10 from dual union all
select 38040,'1975-W526','1975-05-05 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 38041,'1975-W527','1975-05-05 23:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',68 from dual union all
select 38042,'1975-W528','1975-05-06 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 38043,'1975-W529','1975-05-06 20:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 38044,'1975-W53','1975-01-15 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 38045,'1975-W530','1975-05-07 15:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 38046,'1975-W531','1975-05-07 15:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 38047,'1975-W532','1975-05-07 16:32:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 38048,'1975-W533','1975-05-07 18:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 38049,'1975-W534','1975-05-07 20:01:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',76 from dual union all
select 3805,'1956-A142','1956-02-01 00:00:00','atmospheric rocket','success',836,'1249B-5156','Nike 511R','','','Test',1895,'',10 from dual union all
select 38050,'1975-W535','1975-05-07 21:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 38051,'1975-W536','1975-05-08 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 38052,'1975-W537','1975-05-08 21:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 38053,'1975-W538','1975-05-09 04:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 38054,'1975-W539','1975-05-09 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 38055,'1975-W54','1975-01-15 21:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 38056,'1975-W540','1975-05-09 14:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 38057,'1975-W541','1975-05-09 14:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 38058,'1975-W542','1975-05-09 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 38059,'1975-W543','1975-05-09 16:27:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 3806,'1956-A143','1956-02-08 00:00:00','atmospheric rocket','success',836,'1249B-6390','Nike 512R','','','Test',1895,'',10 from dual union all
select 38060,'1975-W544','1975-05-09 17:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',75 from dual union all
select 38061,'1975-W545','1975-05-09 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 38062,'1975-W546','1975-05-09 21:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 38063,'1975-W547','1975-05-12 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 38064,'1975-W548','1975-05-12 14:46:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 38065,'1975-W549','1975-05-12 16:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 38066,'1975-W55','1975-01-16 01:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',72 from dual union all
select 38067,'1975-W550','1975-05-12 17:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 38068,'1975-W551','1975-05-12 17:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 38069,'1975-W552','1975-05-13 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 3807,'1956-A144','1956-02-15 00:00:00','atmospheric rocket','success',836,'1249B-5160','Nike 513R','','','Test',1895,'',10 from dual union all
select 38070,'1975-W553','1975-05-13 08:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 38071,'1975-W554','1975-05-13 10:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 38072,'1975-W555','1975-05-13 15:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 38073,'1975-W556','1975-05-13 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 38074,'1975-W557','1975-05-13 22:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',64 from dual union all
select 38075,'1975-W558','1975-05-14 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 38076,'1975-W559','1975-05-14 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 38077,'1975-W56','1975-01-16 14:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 38078,'1975-W560','1975-05-14 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual;
insert into LAUNCH
select 38079,'1975-W561','1975-05-14 15:20:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',53 from dual union all
select 3808,'1956-A145','1956-02-15 00:00:00','atmospheric rocket','success',836,'1249B-5224','Nike 514R','','','Test',1895,'',10 from dual union all
select 38080,'1975-W562','1975-05-14 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 38081,'1975-W563','1975-05-14 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 38082,'1975-W564','1975-05-14 19:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 38083,'1975-W565','1975-05-14 21:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',68 from dual union all
select 38084,'1975-W566','1975-05-15 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 38085,'1975-W567','1975-05-16 05:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 38086,'1975-W568','1975-05-16 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 38087,'1975-W569','1975-05-16 13:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',60 from dual union all
select 38088,'1975-W57','1975-01-16 16:19:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 38089,'1975-W570','1975-05-16 14:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 3809,'1956-A146','1956-02-15 00:00:00','atmospheric rocket','success',836,'1249B-6717','Nike 515R','','','Test',1895,'',10 from dual union all
select 38090,'1975-W571','1975-05-16 15:34:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',65 from dual union all
select 38091,'1975-W572','1975-05-16 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 38092,'1975-W573','1975-05-16 17:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',60 from dual union all
select 38093,'1975-W574','1975-05-16 18:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 38094,'1975-W575','1975-05-16 20:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 38095,'1975-W576','1975-05-17 20:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',79 from dual union all
select 38096,'1975-W577','1975-05-19 14:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 38097,'1975-W578','1975-05-19 14:42:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',62 from dual union all
select 38098,'1975-W579','1975-05-19 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',70 from dual union all
select 38099,'1975-W58','1975-01-16 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 381,'1944-S347','1944-11-05 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3810,'1956-A147','1956-02-20 00:00:00','atmospheric rocket','success',836,'1249B-6964','Nike 516R','','','Test',1895,'',10 from dual union all
select 38100,'1975-W580','1975-05-19 19:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 38101,'1975-W581','1975-05-19 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 38102,'1975-W582','1975-05-19 22:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',63 from dual union all
select 38103,'1975-W583','1975-05-20 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 38104,'1975-W584','1975-05-20 11:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 38105,'1975-W585','1975-05-20 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',68 from dual union all
select 38106,'1975-W586','1975-05-21 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 38107,'1975-W587','1975-05-21 14:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 38108,'1975-W588','1975-05-21 14:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 38109,'1975-W589','1975-05-21 14:41:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',52 from dual union all
select 3811,'1956-A148','1956-02-20 00:00:00','atmospheric rocket','success',836,'1249B-7022','Nike 517R','','','Test',1895,'',10 from dual union all
select 38110,'1975-W59','1975-01-16 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 38111,'1975-W590','1975-05-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',66 from dual union all
select 38112,'1975-W591','1975-05-21 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 38113,'1975-W592','1975-05-21 15:20:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 38114,'1975-W593','1975-05-21 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',65 from dual union all
select 38115,'1975-W594','1975-05-21 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 38116,'1975-W595','1975-05-21 19:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 38117,'1975-W596','1975-05-21 22:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',66 from dual union all
select 38118,'1975-W597','1975-05-22 01:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 38119,'1975-W598','1975-05-23 07:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',70 from dual union all
select 3812,'1956-A149','1956-02-23 00:00:00','atmospheric rocket','success',836,'1249B-6188','Nike 518R','','','Test',1895,'',10 from dual union all
select 38120,'1975-W599','1975-05-23 11:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 38121,'1975-W60','1975-01-16 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 38122,'1975-W600','1975-05-23 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 38123,'1975-W601','1975-05-23 14:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 38124,'1975-W602','1975-05-23 15:10:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',66 from dual union all
select 38125,'1975-W603','1975-05-23 18:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',72 from dual union all
select 38126,'1975-W604','1975-05-23 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 38127,'1975-W605','1975-05-23 22:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',53 from dual union all
select 38128,'1975-W606','1975-05-24 01:52:00','suborbital rocket','success',1281,'NASA 15.133GM','Antarqui Ozone','','','Aeronomy',323,'',71 from dual union all
select 38129,'1975-W607','1975-05-24 08:42:00','suborbital rocket','success',1281,'NASA 15.135GM','Antarqui Ozone','','','Aeronomy',323,'',67 from dual union all
select 3813,'1956-A15','1956-02-08 00:00:00','military missile','success',1214,'N-69C 52-10978','','','','Test',224,'',10 from dual union all
select 38130,'1975-W608','1975-05-24 14:24:00','suborbital rocket','unknown',1284,'GU1-7743','Antarqui SL Ozone 1','','','Aeron',323,'',70 from dual union all
select 38131,'1975-W609','1975-05-24 19:00:00','suborbital rocket','unknown',1284,'GU1-7742','Antarqui SL Ozone ','','','Aeron',323,'',70 from dual union all
select 38132,'1975-W61','1975-01-18 00:01:00','suborbital rocket','success',1284,'','ALADDIN 75','','','Aeron',1821,'',75 from dual union all
select 38133,'1975-W610','1975-05-25 19:44:00','suborbital rocket','unknown',1284,'GU1-7744','Antarqui SL Ozone 2','','','Aeron',323,'',69 from dual union all
select 38134,'1975-W611','1975-05-26 14:41:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',56 from dual union all
select 38135,'1975-W612','1975-05-26 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',73 from dual union all
select 38136,'1975-W613','1975-05-26 22:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',64 from dual union all
select 38137,'1975-W614','1975-05-27 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 38138,'1975-W615','1975-05-27 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 38139,'1975-W616','1975-05-27 20:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 3814,'1956-A150','1956-02-23 00:00:00','atmospheric rocket','success',836,'1249B-6162','Nike 519R','','','Test',1895,'',10 from dual union all
select 38140,'1975-W617','1975-05-27 23:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 38141,'1975-W618','1975-05-28 08:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 38142,'1975-W619','1975-05-28 09:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 38143,'1975-W62','1975-01-17 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 38144,'1975-W620','1975-05-28 10:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 38145,'1975-W621','1975-05-28 15:17:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 38146,'1975-W622','1975-05-28 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',63 from dual union all
select 38147,'1975-W623','1975-05-28 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',71 from dual union all
select 38148,'1975-W624','1975-05-28 17:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 38149,'1975-W625','1975-05-28 17:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 3815,'1956-A151','1956-02-23 00:00:00','atmospheric rocket','success',836,'1249B-5687','Nike 520R','','','Test',1895,'',10 from dual union all
select 38150,'1975-W626','1975-05-28 22:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',75 from dual union all
select 38151,'1975-W627','1975-05-28 22:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',64 from dual union all
select 38152,'1975-W628','1975-05-29 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',72 from dual union all
select 38153,'1975-W629','1975-05-29 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',53 from dual union all
select 38154,'1975-W63','1975-01-17 15:45:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',65 from dual union all
select 38155,'1975-W630','1975-05-29 20:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 38156,'1975-W631','1975-05-29 21:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 38157,'1975-W632','1975-05-30 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 38158,'1975-W633','1975-05-30 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 38159,'1975-W634','1975-05-30 14:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 3816,'1956-A152','1956-02-27 00:00:00','atmospheric rocket','success',836,'1249B-6966','Nike 521R','','','Test',1895,'',10 from dual union all
select 38160,'1975-W635','1975-05-30 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',56 from dual union all
select 38161,'1975-W636','1975-05-30 16:32:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 38162,'1975-W637','1975-05-30 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 38163,'1975-W638','1975-05-30 19:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 38164,'1975-W639','1975-05-30 21:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',52 from dual union all
select 38165,'1975-W64','1975-01-17 17:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 38166,'1975-W640','1975-06-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 38167,'1975-W641','1975-06-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 38168,'1975-W642','1975-06-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual;
insert into LAUNCH
select 38169,'1975-W643','1975-06-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 3817,'1956-A153','1956-02-27 00:00:00','atmospheric rocket','success',836,'1249B-5158','Nike 522R','','','Test',1895,'',10 from dual union all
select 38170,'1975-W644','1975-06-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 38171,'1975-W645','1975-06-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 38172,'1975-W646','1975-06-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 38173,'1975-W647','1975-06-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 38174,'1975-W648','1975-06-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 38175,'1975-W649','1975-06-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38176,'1975-W65','1975-01-17 18:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 38177,'1975-W650','1975-06-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38178,'1975-W651','1975-06-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38179,'1975-W652','1975-06-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 3818,'1956-A154','1956-02-27 00:00:00','atmospheric rocket','success',836,'1249B-6970','Nike 523R','','','Test',1895,'',10 from dual union all
select 38180,'1975-W653','1975-06-02 16:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 38181,'1975-W654','1975-06-02 16:34:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',62 from dual union all
select 38182,'1975-W655','1975-06-02 17:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',70 from dual union all
select 38183,'1975-W656','1975-06-02 19:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 38184,'1975-W657','1975-06-03 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',68 from dual union all
select 38185,'1975-W658','1975-06-03 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',54 from dual union all
select 38186,'1975-W659','1975-06-03 14:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 38187,'1975-W66','1975-01-17 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 38188,'1975-W660','1975-06-03 17:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 38189,'1975-W661','1975-06-04 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 3819,'1956-A155','1956-02-27 00:00:00','atmospheric rocket','success',836,'1249B-6971','Nike 524R','','','Test',1895,'',10 from dual union all
select 38190,'1975-W662','1975-06-04 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',55 from dual union all
select 38191,'1975-W663','1975-06-04 15:08:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 38192,'1975-W664','1975-06-04 15:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 38193,'1975-W665','1975-06-04 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 38194,'1975-W666','1975-06-04 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 38195,'1975-W667','1975-06-04 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 38196,'1975-W668','1975-06-05 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 38197,'1975-W669','1975-06-05 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 38198,'1975-W67','1975-01-17 21:49:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',51 from dual union all
select 38199,'1975-W670','1975-06-05 17:00:00','suborbital rocket','success',1061,'','STS-','','','Meteo',1895,'',71 from dual union all
select 382,'1944-S348','1944-11-05 14:36:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3820,'1956-A156','1956-02-28 00:00:00','atmospheric rocket','success',836,'1249B-5221','Nike 525R','','','Test',1895,'',10 from dual union all
select 38200,'1975-W671','1975-06-05 19:19:59','suborbital rocket','success',1061,'','STS-','','','Meteo',1895,'',67 from dual union all
select 38201,'1975-W672','1975-06-05 21:11:59','suborbital rocket','success',1061,'','Starute,STS-','','','Meteo',1895,'',56 from dual union all
select 38202,'1975-W673','1975-06-05 22:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',65 from dual union all
select 38203,'1975-W674','1975-06-06 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 38204,'1975-W675','1975-06-06 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',52 from dual union all
select 38205,'1975-W676','1975-06-06 14:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 38206,'1975-W677','1975-06-06 16:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 38207,'1975-W678','1975-06-06 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 38208,'1975-W679','1975-06-06 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',67 from dual union all
select 38209,'1975-W68','1975-01-18 00:12:00','suborbital rocket','success',1284,'','ALADDIN 75','','','Aeron',1821,'',75 from dual union all
select 3821,'1956-A157','1956-02-28 00:00:00','atmospheric rocket','success',836,'1249B-5743','Nike 526R','','','Test',1895,'',10 from dual union all
select 38210,'1975-W680','1975-06-06 21:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 38211,'1975-W681','1975-06-09 15:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 38212,'1975-W682','1975-06-09 15:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',64 from dual union all
select 38213,'1975-W683','1975-06-09 16:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 38214,'1975-W684','1975-06-09 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 38215,'1975-W685','1975-06-09 17:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 38216,'1975-W686','1975-06-09 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 38217,'1975-W687','1975-06-11 08:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 38218,'1975-W688','1975-06-11 09:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 38219,'1975-W689','1975-06-11 10:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 3822,'1956-A158','1956-02-28 00:00:00','atmospheric rocket','success',836,'1249B-6470','Nike 527R','','','Test',1895,'',10 from dual union all
select 38220,'1975-W69','1975-01-18 01:14:00','suborbital rocket','success',1284,'','ALADDIN 75','','','Aeron',1821,'',75 from dual union all
select 38221,'1975-W690','1975-06-11 14:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 38222,'1975-W691','1975-06-11 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 38223,'1975-W692','1975-06-11 15:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 38224,'1975-W693','1975-06-11 15:18:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 38225,'1975-W694','1975-06-11 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',60 from dual union all
select 38226,'1975-W695','1975-06-12 01:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 38227,'1975-W696','1975-06-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',78 from dual union all
select 38228,'1975-W697','1975-06-13 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 38229,'1975-W698','1975-06-13 13:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 3823,'1956-A159','1956-02-29 00:00:00','atmospheric rocket','success',836,'1249B-5273','Nike 528R','','','Test',1895,'',10 from dual union all
select 38230,'1975-W699','1975-06-13 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 38231,'1975-W70','1975-01-18 01:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 38232,'1975-W700','1975-06-13 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 38233,'1975-W701','1975-06-13 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 38234,'1975-W702','1975-06-16 09:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 38235,'1975-W703','1975-06-17 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 38236,'1975-W704','1975-06-17 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 38237,'1975-W705','1975-06-17 21:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 38238,'1975-W706','1975-06-18 09:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 38239,'1975-W707','1975-06-18 13:07:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',53 from dual union all
select 3824,'1956-A16','1956-02-10 00:00:00','suborbital rocket','success',122,'2','','','','Test',1895,'',30 from dual union all
select 38240,'1975-W708','1975-06-18 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 38241,'1975-W709','1975-06-18 15:20:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 38242,'1975-W71','1975-01-19 08:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',71 from dual union all
select 38243,'1975-W710','1975-06-18 16:19:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',64 from dual union all
select 38244,'1975-W711','1975-06-18 19:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',75 from dual union all
select 38245,'1975-W712','1975-06-18 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 38246,'1975-W713','1975-06-18 20:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 38247,'1975-W714','1975-06-18 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',66 from dual union all
select 38248,'1975-W715','1975-06-19 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 38249,'1975-W716','1975-06-19 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 3825,'1956-A160','1956-02-29 00:00:00','atmospheric rocket','success',836,'1249B-6710','Nike 529R','','','Test',1895,'',10 from dual union all
select 38250,'1975-W717','1975-06-20 09:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 38251,'1975-W718','1975-06-20 13:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',51 from dual union all
select 38252,'1975-W719','1975-06-20 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 38253,'1975-W72','1975-01-20 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 38254,'1975-W720','1975-06-20 15:20:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',64 from dual union all
select 38255,'1975-W721','1975-06-20 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 38256,'1975-W722','1975-06-20 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',65 from dual union all
select 38257,'1975-W723','1975-06-20 21:22:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 38258,'1975-W724','1975-06-21 01:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual;
insert into LAUNCH
select 38259,'1975-W725','1975-06-23 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 3826,'1956-A161','1956-02-29 00:00:00','atmospheric rocket','success',836,'1249B-5725','Nike 530R','','','Test',1895,'',10 from dual union all
select 38260,'1975-W726','1975-06-23 15:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 38261,'1975-W727','1975-06-23 15:40:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',64 from dual union all
select 38262,'1975-W728','1975-06-23 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 38263,'1975-W729','1975-06-23 22:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',69 from dual union all
select 38264,'1975-W73','1975-01-20 16:15:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 38265,'1975-W730','1975-06-24 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 38266,'1975-W731','1975-06-24 16:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 38267,'1975-W732','1975-06-24 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',67 from dual union all
select 38268,'1975-W733','1975-06-25 08:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 38269,'1975-W734','1975-06-25 10:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',57 from dual union all
select 3827,'1956-A162','1956-05-02 00:00:00','atmospheric rocket','success',836,'1249B-5765','Nike 531R','','','Test',1895,'',10 from dual union all
select 38270,'1975-W735','1975-06-25 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 38271,'1975-W736','1975-06-25 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 38272,'1975-W737','1975-06-25 14:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 38273,'1975-W738','1975-06-25 16:52:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',62 from dual union all
select 38274,'1975-W739','1975-06-25 19:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 38275,'1975-W74','1975-01-20 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 38276,'1975-W740','1975-06-25 20:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',76 from dual union all
select 38277,'1975-W741','1975-06-25 21:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',68 from dual union all
select 38278,'1975-W742','1975-06-26 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 38279,'1975-W743','1975-06-26 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 3828,'1956-A163','1956-06-14 00:00:00','atmospheric rocket','success',836,'1249B-8951','Nike 532R','','','Test',1895,'',10 from dual union all
select 38280,'1975-W744','1975-06-27 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 38281,'1975-W745','1975-06-27 15:45:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',54 from dual union all
select 38282,'1975-W746','1975-06-27 16:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 38283,'1975-W747','1975-06-27 20:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 38284,'1975-W748','1975-06-28 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 38285,'1975-W749','1975-07-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 38286,'1975-W75','1975-01-20 17:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 38287,'1975-W750','1975-07-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38288,'1975-W751','1975-07-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38289,'1975-W752','1975-07-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 3829,'1956-A164','1956-06-22 00:00:00','atmospheric rocket','success',836,'1249B-7946','Nike 533R','','','Test',1895,'',10 from dual union all
select 38290,'1975-W753','1975-07-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38291,'1975-W754','1975-07-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38292,'1975-W755','1975-06-30 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 38293,'1975-W756','1975-06-30 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 38294,'1975-W757','1975-06-30 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 38295,'1975-W758','1975-06-30 18:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 38296,'1975-W759','1975-07-01 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 38297,'1975-W76','1975-01-20 21:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 38298,'1975-W760','1975-07-01 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',71 from dual union all
select 38299,'1975-W761','1975-07-02 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 383,'1944-S349','1944-11-05 15:07:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3830,'1956-A165','1956-07-25 00:00:00','atmospheric rocket','success',836,'1249B-5220','Nike 534R','','','Test',1895,'',10 from dual union all
select 38300,'1975-W762','1975-07-02 15:11:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 38301,'1975-W763','1975-07-02 17:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 38302,'1975-W764','1975-07-02 19:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 38303,'1975-W765','1975-07-02 19:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',72 from dual union all
select 38304,'1975-W766','1975-07-02 21:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 38305,'1975-W767','1975-07-03 01:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 38306,'1975-W768','1975-07-03 19:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 38307,'1975-W769','1975-07-07 09:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 38308,'1975-W77','1975-01-21 15:48:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',64 from dual union all
select 38309,'1975-W770','1975-07-07 14:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 3831,'1956-A166','1956-07-25 00:00:00','atmospheric rocket','success',836,'1249B-5238','Nike 535R','','','Test',1895,'',10 from dual union all
select 38310,'1975-W771','1975-07-07 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 38311,'1975-W772','1975-07-07 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 38312,'1975-W773','1975-07-07 23:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',66 from dual union all
select 38313,'1975-W774','1975-07-08 00:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 38314,'1975-W775','1975-07-08 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 38315,'1975-W776','1975-07-09 09:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 38316,'1975-W777','1975-07-09 13:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',60 from dual union all
select 38317,'1975-W778','1975-07-09 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 38318,'1975-W779','1975-07-09 16:29:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 38319,'1975-W78','1975-01-21 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 3832,'1956-A167','1956-08-01 00:00:00','atmospheric rocket','success',836,'1249B-5771','Nike 536R','','','Test',1895,'',10 from dual union all
select 38320,'1975-W780','1975-07-09 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 38321,'1975-W781','1975-07-09 18:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',72 from dual union all
select 38322,'1975-W782','1975-07-09 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 38323,'1975-W783','1975-07-10 01:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',66 from dual union all
select 38324,'1975-W784','1975-07-10 19:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 38325,'1975-W785','1975-07-11 00:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 38326,'1975-W786','1975-07-11 12:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 38327,'1975-W787','1975-07-11 14:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 38328,'1975-W788','1975-07-11 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',68 from dual union all
select 38329,'1975-W789','1975-07-11 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 3833,'1956-A168','1956-08-08 00:00:00','atmospheric rocket','success',836,'1249B-3013','Nike 537R','','','Test',1895,'',10 from dual union all
select 38330,'1975-W79','1975-01-21 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 38331,'1975-W790','1975-07-11 19:42:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 38332,'1975-W791','1975-07-11 21:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 38333,'1975-W792','1975-07-12 15:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',65 from dual union all
select 38334,'1975-W793','1975-07-13 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 38335,'1975-W794','1975-07-14 14:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 38336,'1975-W795','1975-07-14 15:45:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',70 from dual union all
select 38337,'1975-W796','1975-07-14 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 38338,'1975-W797','1975-07-14 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',60 from dual union all
select 38339,'1975-W798','1975-07-14 23:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 3834,'1956-A169','1956-08-09 00:00:00','atmospheric rocket','success',836,'1249B-8600','Nike 538R','','','Test',1895,'',10 from dual union all
select 38340,'1975-W799','1975-07-15 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 38341,'1975-W80','1975-01-21 20:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 38342,'1975-W800','1975-07-15 10:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 38343,'1975-W801','1975-07-15 14:59:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',68 from dual union all
select 38344,'1975-W802','1975-07-15 21:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 38345,'1975-W803','1975-07-16 10:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 38346,'1975-W804','1975-07-16 14:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',54 from dual union all
select 38347,'1975-W805','1975-07-16 14:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 38348,'1975-W806','1975-07-16 15:40:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',51 from dual;
insert into LAUNCH
select 38349,'1975-W807','1975-07-16 18:15:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 3835,'1956-A17','1956-02-10 00:00:00','suborbital rocket','success',122,'4','','','','Test',1895,'',30 from dual union all
select 38350,'1975-W808','1975-07-16 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 38351,'1975-W809','1975-07-16 23:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',73 from dual union all
select 38352,'1975-W81','1975-01-22 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 38353,'1975-W810','1975-07-17 00:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',62 from dual union all
select 38354,'1975-W811','1975-07-17 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 38355,'1975-W812','1975-07-18 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 38356,'1975-W813','1975-07-18 14:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 38357,'1975-W814','1975-07-18 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',71 from dual union all
select 38358,'1975-W815','1975-07-18 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 38359,'1975-W816','1975-07-18 20:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 3836,'1956-A170','1956-08-09 00:00:00','atmospheric rocket','success',836,'1249B-7531','Nike 539R','','','Test',1895,'',10 from dual union all
select 38360,'1975-W817','1975-07-21 13:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 38361,'1975-W818','1975-07-21 15:10:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',61 from dual union all
select 38362,'1975-W819','1975-07-21 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',59 from dual union all
select 38363,'1975-W82','1975-01-22 14:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 38364,'1975-W820','1975-07-21 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',56 from dual union all
select 38365,'1975-W821','1975-07-21 23:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',75 from dual union all
select 38366,'1975-W822','1975-07-22 21:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 38367,'1975-W823','1975-07-23 01:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 38368,'1975-W824','1975-07-23 16:11:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',73 from dual union all
select 38369,'1975-W825','1975-07-23 17:30:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 3837,'1956-A171','1956-08-16 00:00:00','atmospheric rocket','success',836,'1249B-5778','Nike 540R','','','Test',1895,'',10 from dual union all
select 38370,'1975-W826','1975-07-23 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 38371,'1975-W827','1975-07-23 17:40:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',73 from dual union all
select 38372,'1975-W828','1975-07-23 18:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 38373,'1975-W829','1975-07-23 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 38374,'1975-W83','1975-01-22 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 38375,'1975-W830','1975-07-24 04:43:59','suborbital rocket','success',1061,'','Thermistor Intercomp','','','Meteo',1821,'',61 from dual union all
select 38376,'1975-W831','1975-07-24 19:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 38377,'1975-W832','1975-07-25 00:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 38378,'1975-W833','1975-07-25 12:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 38379,'1975-W834','1975-07-25 16:04:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',71 from dual union all
select 3838,'1956-A172','1956-08-16 00:00:00','atmospheric rocket','success',836,'1249B-6949','Nike 541R','','','Test',1895,'',10 from dual union all
select 38380,'1975-W835','1975-07-25 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 38381,'1975-W836','1975-07-25 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',56 from dual union all
select 38382,'1975-W837','1975-07-25 22:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 38383,'1975-W838','1975-07-28 09:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 38384,'1975-W839','1975-07-28 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 38385,'1975-W84','1975-01-22 16:18:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',66 from dual union all
select 38386,'1975-W840','1975-07-28 15:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 38387,'1975-W841','1975-07-28 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 38388,'1975-W842','1975-07-28 18:15:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 38389,'1975-W843','1975-07-28 23:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',64 from dual union all
select 3839,'1956-A173','1956-08-16 00:00:00','atmospheric rocket','success',836,'1249B-6978','Nike 542R','','','Test',1895,'',10 from dual union all
select 38390,'1975-W844','1975-07-29 01:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 38391,'1975-W845','1975-07-29 15:34:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',58 from dual union all
select 38392,'1975-W846','1975-07-29 17:17:00','suborbital rocket','unknown',1284,'T 1-7959','','','','Aeron',1821,'',72 from dual union all
select 38393,'1975-W847','1975-07-29 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',78 from dual union all
select 38394,'1975-W848','1975-07-29 19:20:00','suborbital rocket','success',1281,'NASA 15.136GM','Antarqui Ozone','','','Aeronomy',1821,'',73 from dual union all
select 38395,'1975-W849','1975-07-29 19:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',79 from dual union all
select 38396,'1975-W85','1975-01-22 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 38397,'1975-W850','1975-07-30 12:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 38398,'1975-W851','1975-07-30 14:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 38399,'1975-W852','1975-07-30 14:48:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 384,'1944-S35','1944-08-14 11:23:00','military missile','success',1470,'18303','Ma120','','','Training',661,'',90 from dual union all
select 3840,'1956-A174','1956-08-24 00:00:00','atmospheric rocket','success',836,'1249B-5254','Nike 543R','','','Test',1895,'',10 from dual union all
select 38400,'1975-W853','1975-07-30 15:20:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',68 from dual union all
select 38401,'1975-W854','1975-07-30 17:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 38402,'1975-W855','1975-07-30 17:34:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 38403,'1975-W856','1975-07-30 21:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 38404,'1975-W857','1975-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38405,'1975-W858','1975-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38406,'1975-W859','1975-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38407,'1975-W86','1975-01-22 20:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 38408,'1975-W860','1975-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38409,'1975-W861','1975-07-31 01:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 3841,'1956-A175','1956-09-05 00:00:00','atmospheric rocket','success',836,'1249B-7966','Nike 544R','','','Test',1895,'',10 from dual union all
select 38410,'1975-W862','1975-07-31 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 38411,'1975-W863','1975-07-31 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',68 from dual union all
select 38412,'1975-W864','1975-08-01 09:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 38413,'1975-W865','1975-08-01 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 38414,'1975-W866','1975-08-01 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 38415,'1975-W867','1975-08-01 14:48:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 38416,'1975-W868','1975-08-01 14:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 38417,'1975-W869','1975-08-01 15:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 38418,'1975-W87','1975-01-22 23:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',75 from dual union all
select 38419,'1975-W870','1975-08-01 22:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 3842,'1956-A176','1956-09-12 00:00:00','atmospheric rocket','success',836,'1249B-7918','Nike 545R','','','Test',1895,'',10 from dual union all
select 38420,'1975-W871','1975-08-02 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 38421,'1975-W872','1975-08-04 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 38422,'1975-W873','1975-08-04 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 38423,'1975-W874','1975-08-04 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 38424,'1975-W875','1975-08-04 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 38425,'1975-W876','1975-08-04 20:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',55 from dual union all
select 38426,'1975-W877','1975-08-04 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',56 from dual union all
select 38427,'1975-W878','1975-08-05 01:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 38428,'1975-W879','1975-08-05 15:30:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',65 from dual union all
select 38429,'1975-W88','1975-01-23 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 3843,'1956-A177','1956-09-26 00:00:00','atmospheric rocket','success',836,'1249B-5773','Nike 546R','','','Test',1895,'',10 from dual union all
select 38430,'1975-W880','1975-08-05 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 38431,'1975-W881','1975-08-05 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 38432,'1975-W882','1975-08-05 19:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 38433,'1975-W883','1975-08-06 12:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 38434,'1975-W884','1975-08-06 12:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',68 from dual union all
select 38435,'1975-W885','1975-08-06 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 38436,'1975-W886','1975-08-06 16:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 38437,'1975-W887','1975-08-06 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 38438,'1975-W888','1975-08-06 17:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',57 from dual;
insert into LAUNCH
select 38439,'1975-W889','1975-08-06 18:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 3844,'1956-A178','1956-09-26 00:00:00','atmospheric rocket','success',836,'1249B-7787','Nike 547R','','','Test',1895,'',10 from dual union all
select 38440,'1975-W89','1975-01-23 09:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 38441,'1975-W890','1975-08-06 21:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 38442,'1975-W891','1975-08-06 23:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',60 from dual union all
select 38443,'1975-W892','1975-08-07 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 38444,'1975-W893','1975-08-07 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 38445,'1975-W894','1975-08-07 15:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',77 from dual union all
select 38446,'1975-W895','1975-08-07 17:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 38447,'1975-W896','1975-08-08 00:12:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 38448,'1975-W897','1975-08-08 09:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 38449,'1975-W898','1975-08-08 15:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 3845,'1956-A179','1956-09-26 00:00:00','atmospheric rocket','success',836,'1249B-7025','Nike 549R','','','Test',1895,'',10 from dual union all
select 38450,'1975-W899','1975-08-08 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',61 from dual union all
select 38451,'1975-W90','1975-01-23 15:42:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 38452,'1975-W900','1975-08-09 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 38453,'1975-W901','1975-08-11 12:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 38454,'1975-W902','1975-08-11 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1895,'',66 from dual union all
select 38455,'1975-W903','1975-08-11 15:55:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',56 from dual union all
select 38456,'1975-W904','1975-08-11 16:51:00','suborbital rocket','success',1284,'T 1-8409','','','','Aeron',1821,'',54 from dual union all
select 38457,'1975-W905','1975-08-12 15:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',74 from dual union all
select 38458,'1975-W906','1975-08-12 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 38459,'1975-W907','1975-08-12 16:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 3846,'1956-A18','1956-02-10 00:00:00','suborbital rocket','success',122,'5','','','','Test',1895,'',30 from dual union all
select 38460,'1975-W908','1975-08-12 18:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1895,'',64 from dual union all
select 38461,'1975-W909','1975-08-12 21:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',71 from dual union all
select 38462,'1975-W91','1975-01-23 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 38463,'1975-W910','1975-08-12 22:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',75 from dual union all
select 38464,'1975-W911','1975-08-12 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',60 from dual union all
select 38465,'1975-W912','1975-08-13 00:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 38466,'1975-W913','1975-08-13 11:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 38467,'1975-W914','1975-08-13 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 38468,'1975-W915','1975-08-13 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 38469,'1975-W916','1975-08-13 16:46:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 3847,'1956-A180','1956-10-03 00:00:00','atmospheric rocket','success',836,'1249B-9481','Nike 550R','','','Test',1895,'',10 from dual union all
select 38470,'1975-W917','1975-08-13 17:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 38471,'1975-W918','1975-08-13 17:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1895,'',63 from dual union all
select 38472,'1975-W919','1975-08-13 18:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1895,'',66 from dual union all
select 38473,'1975-W92','1975-01-23 20:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 38474,'1975-W920','1975-08-13 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',63 from dual union all
select 38475,'1975-W921','1975-08-13 22:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 38476,'1975-W922','1975-08-14 01:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',79 from dual union all
select 38477,'1975-W923','1975-08-14 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1362,'',60 from dual union all
select 38478,'1975-W924','1975-08-15 04:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 38479,'1975-W925','1975-08-15 05:21:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',65 from dual union all
select 3848,'1956-A181','1956-10-11 00:00:00','atmospheric rocket','success',836,'1249B-7509','Nike 551R','','','Test',1895,'',10 from dual union all
select 38480,'1975-W926','1975-08-15 05:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',75 from dual union all
select 38481,'1975-W927','1975-08-15 13:42:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 38482,'1975-W928','1975-08-15 15:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 38483,'1975-W929','1975-08-15 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',75 from dual union all
select 38484,'1975-W93','1975-01-24 06:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',78 from dual union all
select 38485,'1975-W930','1975-08-15 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 38486,'1975-W931','1975-08-15 16:31:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',61 from dual union all
select 38487,'1975-W932','1975-08-15 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1895,'',66 from dual union all
select 38488,'1975-W933','1975-08-15 20:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 38489,'1975-W934','1975-08-16 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',59 from dual union all
select 3849,'1956-A182','1956-10-31 00:00:00','atmospheric rocket','success',836,'1249B-9946','Nike 552R','','','Test',1895,'',10 from dual union all
select 38490,'1975-W935','1975-08-18 11:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',69 from dual union all
select 38491,'1975-W936','1975-08-18 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 38492,'1975-W937','1975-08-18 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 38493,'1975-W938','1975-08-18 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 38494,'1975-W939','1975-08-18 17:40:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',58 from dual union all
select 38495,'1975-W94','1975-01-24 14:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',76 from dual union all
select 38496,'1975-W940','1975-08-18 18:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 38497,'1975-W941','1975-08-18 22:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 38498,'1975-W942','1975-08-19 18:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1895,'',64 from dual union all
select 38499,'1975-W943','1975-08-20 00:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 385,'1944-S350','1944-11-06 07:40:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3850,'1956-A183','1956-10-31 00:00:00','atmospheric rocket','success',836,'1249B-7930','Nike 553R','','','Test',1895,'',10 from dual union all
select 38500,'1975-W944','1975-08-20 11:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 38501,'1975-W945','1975-08-20 17:00:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 38502,'1975-W946','1975-08-20 17:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 38503,'1975-W947','1975-08-20 21:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',67 from dual union all
select 38504,'1975-W948','1975-08-21 00:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 38505,'1975-W949','1975-08-21 15:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 38506,'1975-W95','1975-01-24 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 38507,'1975-W950','1975-08-22 01:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',75 from dual union all
select 38508,'1975-W951','1975-08-22 13:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 38509,'1975-W952','1975-08-22 14:43:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 3851,'1956-A184','1956-10-31 00:00:00','atmospheric rocket','success',836,'1249B-7460','Nike 554R','','','Test',1895,'',10 from dual union all
select 38510,'1975-W953','1975-08-22 14:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 38511,'1975-W954','1975-08-22 15:24:59','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',54 from dual union all
select 38512,'1975-W955','1975-08-22 15:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',61 from dual union all
select 38513,'1975-W956','1975-08-22 22:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 38514,'1975-W957','1975-08-25 12:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 38515,'1975-W958','1975-08-25 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',59 from dual union all
select 38516,'1975-W959','1975-08-25 16:49:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',62 from dual union all
select 38517,'1975-W96','1975-01-24 15:44:00','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 38518,'1975-W960','1975-08-25 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 38519,'1975-W961','1975-08-25 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 3852,'1956-A185','1956-10-31 00:00:00','atmospheric rocket','success',836,'1249B-8601','Nike 555R','','','Test',1895,'',10 from dual union all
select 38520,'1975-W962','1975-08-25 20:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 38521,'1975-W963','1975-08-26 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 38522,'1975-W964','1975-08-27 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 38523,'1975-W965','1975-08-27 15:45:00','suborbital rocket','success',1061,'','Starute,AN/DMQ-9-','','','Meteo',1470,'',54 from dual union all
select 38524,'1975-W966','1975-08-27 16:09:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1895,'',63 from dual union all
select 38525,'1975-W967','1975-08-27 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 38526,'1975-W968','1975-08-27 16:38:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',60 from dual union all
select 38527,'1975-W969','1975-08-27 18:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 38528,'1975-W97','1975-01-27 09:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual;
insert into LAUNCH
select 38529,'1975-W970','1975-08-27 19:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 3853,'1956-A186','1956-11-07 00:00:00','atmospheric rocket','success',836,'1249B-7467','Nike 556R','','','Test',1895,'',10 from dual union all
select 38530,'1975-W971','1975-08-27 20:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 38531,'1975-W972','1975-08-28 00:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',74 from dual union all
select 38532,'1975-W973','1975-08-28 11:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 38533,'1975-W974','1975-08-28 18:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1895,'',63 from dual union all
select 38534,'1975-W975','1975-08-29 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 38535,'1975-W976','1975-08-29 16:19:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 38536,'1975-W977','1975-08-29 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 38537,'1975-W978','1975-08-29 22:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',65 from dual union all
select 38538,'1975-W979','1975-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38539,'1975-W98','1975-01-27 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 3854,'1956-A187','1956-11-21 00:00:00','atmospheric rocket','success',836,'1249B-7534','Nike 560R','','','Test',1895,'',10 from dual union all
select 38540,'1975-W980','1975-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38541,'1975-W981','1975-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38542,'1975-W982','1975-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 38543,'1975-W983','1975-09-02 12:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 38544,'1975-W984','1975-09-02 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',64 from dual union all
select 38545,'1975-W985','1975-09-02 15:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 38546,'1975-W986','1975-09-02 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 38547,'1975-W987','1975-09-02 18:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 38548,'1975-W988','1975-09-03 00:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',77 from dual union all
select 38549,'1975-W989','1975-09-03 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 3855,'1956-A188','1956-11-29 00:00:00','atmospheric rocket','success',836,'1249B-6947','Nike 561R','','','Test',1895,'',10 from dual union all
select 38550,'1975-W99','1975-01-27 16:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 38551,'1975-W990','1975-09-03 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',50 from dual union all
select 38552,'1975-W991','1975-09-03 16:31:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',63 from dual union all
select 38553,'1975-W992','1975-09-03 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 38554,'1975-W993','1975-09-03 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 38555,'1975-W994','1975-09-03 18:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 38556,'1975-W995','1975-09-03 22:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',74 from dual union all
select 38557,'1975-W996','1975-09-03 23:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',76 from dual union all
select 38558,'1975-W997','1975-09-04 12:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 38559,'1975-W998','1975-09-04 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 3856,'1956-A189','1956-12-12 00:00:00','atmospheric rocket','success',836,'1249B-9201','Nike 562R','','','Test',1895,'',10 from dual union all
select 38560,'1975-W999','1975-09-04 20:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 38561,'1975-Y01','1975-01-02 13:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 38562,'1975-Y02','1975-01-02 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 38563,'1975-Y03','1975-01-03 16:22:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',87 from dual union all
select 38564,'1975-Y04','1975-01-03 21:02:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',86 from dual union all
select 38565,'1975-Y05','1975-01-04 13:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 38566,'1975-Y06','1975-01-04 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1370,'SHIR',92 from dual union all
select 38567,'1975-Y07','1975-01-06 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 38568,'1975-Y08','1975-01-06 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 38569,'1975-Y09','1975-01-07 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 3857,'1956-A19','1956-02-10 00:00:00','suborbital rocket','success',122,'6','','','','Test',1895,'',30 from dual union all
select 38570,'1975-Y10','1975-01-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 38571,'1975-Y100','1975-04-08 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 38572,'1975-Y101','1975-04-09 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 38573,'1975-Y102','1975-04-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',90 from dual union all
select 38574,'1975-Y103','1975-04-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 38575,'1975-Y104','1975-04-11 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 38576,'1975-Y105','1975-04-11 15:57:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 38577,'1975-Y106','1975-04-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 38578,'1975-Y107','1975-04-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 38579,'1975-Y108','1975-04-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 3858,'1956-A190','1956-12-14 00:00:00','atmospheric rocket','success',836,'1249B-50003','Nike 564R','','','Test',1895,'',10 from dual union all
select 38580,'1975-Y109','1975-04-16 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 38581,'1975-Y11','1975-01-08 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 38582,'1975-Y110','1975-04-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 38583,'1975-Y111','1975-04-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 38584,'1975-Y112','1975-04-16 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 38585,'1975-Y113','1975-04-18 12:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 38586,'1975-Y114','1975-04-18 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 38587,'1975-Y115','1975-04-23 01:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 38588,'1975-Y116','1975-04-23 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 38589,'1975-Y117','1975-04-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',89 from dual union all
select 3859,'1956-A191','1956-12-14 00:00:00','atmospheric rocket','success',836,'1249B-50004','Nike 565R','','','Test',1895,'',10 from dual union all
select 38590,'1975-Y118','1975-04-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 38591,'1975-Y119','1975-04-23 21:02:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 38592,'1975-Y12','1975-01-08 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',97 from dual union all
select 38593,'1975-Y120','1975-04-25 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',90 from dual union all
select 38594,'1975-Y121','1975-04-25 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 38595,'1975-Y122','1975-04-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 38596,'1975-Y123','1975-04-28 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 38597,'1975-Y124','1975-04-28 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 38598,'1975-Y125','1975-04-28 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 38599,'1975-Y126','1975-04-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 386,'1944-S351','1944-11-10 12:15:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3860,'1956-A192','1956-12-19 00:00:00','atmospheric rocket','success',836,'1249B-5170','Nike 566R','','','Test',1895,'',10 from dual union all
select 38600,'1975-Y127','1975-04-30 20:10:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 38601,'1975-Y128','1975-05-02 16:03:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 38602,'1975-Y129','1975-05-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 38603,'1975-Y13','1975-01-09 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 38604,'1975-Y130','1975-05-05 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',94 from dual union all
select 38605,'1975-Y131','1975-05-06 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',88 from dual union all
select 38606,'1975-Y132','1975-05-07 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 38607,'1975-Y133','1975-05-07 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 38608,'1975-Y134','1975-05-07 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',87 from dual union all
select 38609,'1975-Y135','1975-05-07 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',90 from dual union all
select 3861,'1956-A193','1956-01-01 00:00:00','military missile','unknown',845,'B21','Test','','','Test',1895,'',10 from dual union all
select 38610,'1975-Y136','1975-05-07 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 38611,'1975-Y137','1975-05-07 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 38612,'1975-Y138','1975-05-08 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',95 from dual union all
select 38613,'1975-Y139','1975-05-08 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',87 from dual union all
select 38614,'1975-Y14','1975-01-10 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 38615,'1975-Y140','1975-05-08 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',88 from dual union all
select 38616,'1975-Y141','1975-05-09 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',89 from dual union all
select 38617,'1975-Y142','1975-05-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',90 from dual union all
select 38618,'1975-Y143','1975-05-09 16:54:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',87 from dual;
insert into LAUNCH
select 38619,'1975-Y144','1975-05-11 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',85 from dual union all
select 3862,'1956-A194','1956-01-12 00:00:00','military missile','unknown',845,'B22 1810-24','Test','','','Test',1895,'',10 from dual union all
select 38620,'1975-Y145','1975-05-13 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',85 from dual union all
select 38621,'1975-Y146','1975-05-14 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 38622,'1975-Y147','1975-05-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',91 from dual union all
select 38623,'1975-Y148','1975-05-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 38624,'1975-Y149','1975-05-14 18:55:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 38625,'1975-Y15','1975-01-10 16:43:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 38626,'1975-Y150','1975-05-14 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 38627,'1975-Y151','1975-05-14 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',83 from dual union all
select 38628,'1975-Y152','1975-05-17 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',81 from dual union all
select 38629,'1975-Y153','1975-05-19 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.191','TERLS-749','','Aeron',1467,'',89 from dual union all
select 3863,'1956-A195','1956-01-19 00:00:00','military missile','unknown',845,'B23 1810-25','Test','','','Test',1895,'',10 from dual union all
select 38630,'1975-Y154','1975-05-20 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 38631,'1975-Y155','1975-05-21 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 38632,'1975-Y156','1975-05-21 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',92 from dual union all
select 38633,'1975-Y157','1975-05-21 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.192','TERLS-751','','Aeron',1467,'',86 from dual union all
select 38634,'1975-Y158','1975-05-22 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 38635,'1975-Y159','1975-05-23 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.193','TERLS-752','','Aeron',1467,'',86 from dual union all
select 38636,'1975-Y16','1975-01-10 19:30:00','suborbital rocket','unknown',725,'','Indian Ocean','','','Aeron',1370,'SHIR',80 from dual union all
select 38637,'1975-Y160','1975-05-24 01:05:00','suborbital rocket','success',1281,'NASA 15.139UE','Antarqui PSU','','','Plasma',323,'',93 from dual union all
select 38638,'1975-Y161','1975-05-24 07:05:00','suborbital rocket','success',1281,'NASA 15.140UE','Antarqui PSU','','','Plasma',323,'',91 from dual union all
select 38639,'1975-Y162','1975-05-24 07:44:00','suborbital rocket','success',1281,'NASA 15.134GM','Antarqui Ozone','','','Aeronomy',323,'',88 from dual union all
select 3864,'1956-A196','1956-01-31 00:00:00','military missile','unknown',845,'B24 1810-23','Test','','','Test',1895,'',10 from dual union all
select 38640,'1975-Y163','1975-05-24 10:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',89 from dual union all
select 38641,'1975-Y164','1975-05-25 20:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',86 from dual union all
select 38642,'1975-Y165','1975-05-26 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.194','TERLS-753','','Aeron',1467,'',84 from dual union all
select 38643,'1975-Y166','1975-05-28 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 38644,'1975-Y167','1975-05-28 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 38645,'1975-Y168','1975-05-28 14:40:00','suborbital rocket','unknown',725,'','ISRO 8.195','TERLS-755','','Aeron',1467,'',84 from dual union all
select 38646,'1975-Y169','1975-05-28 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',82 from dual union all
select 38647,'1975-Y17','1975-01-11 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1370,'SHIR',81 from dual union all
select 38648,'1975-Y170','1975-05-28 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 38649,'1975-Y171','1975-05-28 19:30:00','suborbital rocket','success',1281,'NASA 15.142UE','Antarqui PSU','','','Plasma',323,'',86 from dual union all
select 3865,'1956-A197','1956-02-09 00:00:00','military missile','unknown',845,'B25 1810-26','Test','','','Test',1895,'',10 from dual union all
select 38650,'1975-Y172','1975-05-28 21:16:00','suborbital rocket','success',1281,'NASA 15.141UE','Antarqui PSU','','','Plasma',323,'',82 from dual union all
select 38651,'1975-Y173','1975-05-28 21:40:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',86 from dual union all
select 38652,'1975-Y174','1975-05-29 18:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',87 from dual union all
select 38653,'1975-Y175','1975-05-30 16:00:00','suborbital rocket','unknown',725,'','ISRO 8.196','TERLS-757','','Aeron',1467,'',86 from dual union all
select 38654,'1975-Y176','1975-06-03 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 38655,'1975-Y177','1975-06-04 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 38656,'1975-Y178','1975-06-04 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 38657,'1975-Y179','1975-06-04 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 38658,'1975-Y18','1975-01-12 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1370,'SHIR',83 from dual union all
select 38659,'1975-Y180','1975-06-04 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.197','TERLS-758','','Aeron',1467,'',88 from dual union all
select 3866,'1956-A198','1956-02-10 00:00:00','military missile','unknown',845,'B26 1810-14','Test','','','Test',1895,'',10 from dual union all
select 38660,'1975-Y181','1975-06-06 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',90 from dual union all
select 38661,'1975-Y182','1975-06-06 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',81 from dual union all
select 38662,'1975-Y183','1975-06-06 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',89 from dual union all
select 38663,'1975-Y184','1975-06-07 21:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',90 from dual union all
select 38664,'1975-Y185','1975-06-08 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',86 from dual union all
select 38665,'1975-Y186','1975-06-10 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',86 from dual union all
select 38666,'1975-Y187','1975-06-10 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 38667,'1975-Y188','1975-06-11 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 38668,'1975-Y189','1975-06-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 38669,'1975-Y19','1975-01-13 20:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1370,'SHIR',87 from dual union all
select 3867,'1956-A199','1956-02-24 00:00:00','military missile','unknown',845,'B27 1810-28','Test','','','Test',1895,'',10 from dual union all
select 38670,'1975-Y190','1975-06-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',96 from dual union all
select 38671,'1975-Y191','1975-06-11 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.198','TERLS-760','','Aeron',1467,'',88 from dual union all
select 38672,'1975-Y192','1975-06-11 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',89 from dual union all
select 38673,'1975-Y193','1975-06-12 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',90 from dual union all
select 38674,'1975-Y194','1975-06-13 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',87 from dual union all
select 38675,'1975-Y195','1975-06-14 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',83 from dual union all
select 38676,'1975-Y196','1975-06-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',88 from dual union all
select 38677,'1975-Y197','1975-06-17 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',86 from dual union all
select 38678,'1975-Y198','1975-06-17 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 38679,'1975-Y199','1975-06-17 23:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 3868,'1956-A20','1956-02-14 18:07:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 38680,'1975-Y20','1975-01-14 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1370,'SHIR',89 from dual union all
select 38681,'1975-Y200','1975-06-18 07:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 38682,'1975-Y201','1975-06-18 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 38683,'1975-Y202','1975-06-18 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',95 from dual union all
select 38684,'1975-Y203','1975-06-18 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.199','TERLS-763','','Aeron',1467,'',88 from dual union all
select 38685,'1975-Y204','1975-06-20 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',81 from dual union all
select 38686,'1975-Y205','1975-06-20 19:56:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',86 from dual union all
select 38687,'1975-Y206','1975-06-20 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 38688,'1975-Y207','1975-06-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1372,'SHIR',86 from dual union all
select 38689,'1975-Y208','1975-06-24 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',82 from dual union all
select 3869,'1956-A200','1956-02-29 00:00:00','military missile','unknown',845,'B28 1810-29','Test','','','Test',1895,'',10 from dual union all
select 38690,'1975-Y209','1975-06-25 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 38691,'1975-Y21','1975-01-15 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 38692,'1975-Y210','1975-06-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 38693,'1975-Y211','1975-06-25 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.200','TERLS-765','','Aeron',1467,'',88 from dual union all
select 38694,'1975-Y212','1975-06-25 20:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 38695,'1975-Y213','1975-06-26 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',87 from dual union all
select 38696,'1975-Y214','1975-06-26 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',83 from dual union all
select 38697,'1975-Y215','1975-06-28 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1391,'SHOK',83 from dual union all
select 38698,'1975-Y216','1975-07-02 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 38699,'1975-Y217','1975-07-02 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.201','TERLS-766','','Aeron',1467,'',87 from dual union all
select 387,'1944-S352','1944-11-12 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3870,'1956-A201','1956-05-31 00:00:00','military missile','unknown',845,'B29 1810-36','Test','','','Test',1895,'',10 from dual union all
select 38700,'1975-Y218','1975-07-02 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 38701,'1975-Y219','1975-07-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',96 from dual union all
select 38702,'1975-Y22','1975-01-15 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 38703,'1975-Y220','1975-07-09 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 38704,'1975-Y221','1975-07-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 38705,'1975-Y222','1975-07-09 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.202','TERLS-768','','Aeron',1467,'',87 from dual union all
select 38706,'1975-Y223','1975-07-11 19:14:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 38707,'1975-Y224','1975-07-15 13:18:00','suborbital rocket','success',88,'NASA 15.48WT','','','','Fields',1895,'',90 from dual union all
select 38708,'1975-Y225','1975-07-15 15:49:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',87 from dual;
insert into LAUNCH
select 38709,'1975-Y226','1975-07-16 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 3871,'1956-A202','1956-06-06 00:00:00','military missile','unknown',845,'B30 1810-37','Test','','','Test',1895,'',10 from dual union all
select 38710,'1975-Y227','1975-07-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 38711,'1975-Y228','1975-07-16 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.203','TERLS-769','','Aeron',1467,'',87 from dual union all
select 38712,'1975-Y229','1975-07-21 23:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 38713,'1975-Y23','1975-01-15 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 38714,'1975-Y230','1975-07-23 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 38715,'1975-Y231','1975-07-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 38716,'1975-Y232','1975-07-23 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.204','TERLS-770','','Aeron',1467,'',87 from dual union all
select 38717,'1975-Y233','1975-07-24 05:47:00','suborbital rocket','success',1281,'NASA 15.143GM','Sphere (Intercomp)','','','Aeronomy',1821,'',85 from dual union all
select 38718,'1975-Y234','1975-07-29 16:34:00','suborbital rocket','success',1281,'NASA 15.144GM','','','','Aeronomy',1821,'',81 from dual union all
select 38719,'1975-Y235','1975-07-30 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 3872,'1956-A203','1956-06-22 00:00:00','military missile','unknown',845,'B31 1810-38','Test','','','Test',1895,'',10 from dual union all
select 38720,'1975-Y236','1975-07-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 38721,'1975-Y237','1975-07-30 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.205','TERLS-771','','Aeron',1467,'',88 from dual union all
select 38722,'1975-Y238','1975-08-01 16:21:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',88 from dual union all
select 38723,'1975-Y239','1975-08-05 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 38724,'1975-Y24','1975-01-15 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',96 from dual union all
select 38725,'1975-Y240','1975-08-06 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 38726,'1975-Y241','1975-08-06 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.206','TERLS-772','','Aeron',1467,'',87 from dual union all
select 38727,'1975-Y242','1975-08-06 15:55:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',86 from dual union all
select 38728,'1975-Y243','1975-08-06 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 38729,'1975-Y244','1975-08-07 23:52:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',86 from dual union all
select 3873,'1956-A204','1956-06-29 00:00:00','military missile','unknown',845,'B32 1810-39','Test','','','Test',1895,'',10 from dual union all
select 38730,'1975-Y245','1975-08-13 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 38731,'1975-Y246','1975-08-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 38732,'1975-Y247','1975-08-13 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.207','TERLS-773','','Aeron',1467,'',89 from dual union all
select 38733,'1975-Y248','1975-08-13 17:01:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',87 from dual union all
select 38734,'1975-Y249','1975-08-13 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 38735,'1975-Y25','1975-01-15 19:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1370,'SHIR',85 from dual union all
select 38736,'1975-Y250','1975-08-15 03:45:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',87 from dual union all
select 38737,'1975-Y251','1975-08-15 06:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',88 from dual union all
select 38738,'1975-Y252','1975-08-19 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.208','TERLS-774','','Aeron',1467,'',87 from dual union all
select 38739,'1975-Y253','1975-08-20 00:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 3874,'1956-A205','1956-07-25 00:00:00','military missile','success',845,'1810-30','System Demo','','','Test',1895,'',10 from dual union all
select 38740,'1975-Y254','1975-08-20 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 38741,'1975-Y255','1975-08-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 38742,'1975-Y256','1975-08-20 16:54:59','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',88 from dual union all
select 38743,'1975-Y257','1975-08-20 20:42:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',86 from dual union all
select 38744,'1975-Y258','1975-08-22 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 38745,'1975-Y259','1975-08-27 08:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 38746,'1975-Y26','1975-01-15 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 38747,'1975-Y260','1975-08-27 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 38748,'1975-Y261','1975-08-27 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.209','TERLS-775','','Aeron',1467,'',84 from dual union all
select 38749,'1975-Y262','1975-08-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 3875,'1956-A206','1956-09-10 00:00:00','military missile','success',845,'1810-35','QB-17 intercept','','','Test',1895,'',10 from dual union all
select 38750,'1975-Y263','1975-09-02 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 38751,'1975-Y264','1975-09-02 17:58:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',86 from dual union all
select 38752,'1975-Y265','1975-09-03 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 38753,'1975-Y266','1975-09-03 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 38754,'1975-Y267','1975-09-03 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.210','TERLS-782','','Aeron',1467,'',85 from dual union all
select 38755,'1975-Y268','1975-09-03 17:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',94 from dual union all
select 38756,'1975-Y269','1975-09-03 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 38757,'1975-Y27','1975-01-16 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1370,'SHIR',87 from dual union all
select 38758,'1975-Y270','1975-09-03 22:22:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',88 from dual union all
select 38759,'1975-Y271','1975-09-09 22:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 3876,'1956-A207','1956-12-14 00:00:00','military missile','success',845,'1810-48 B47','Firing no. 47','','','Test',1895,'',10 from dual union all
select 38760,'1975-Y272','1975-09-10 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 38761,'1975-Y273','1975-09-10 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 38762,'1975-Y274','1975-09-10 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.211','TERLS-783','','Aeron',1467,'',84 from dual union all
select 38763,'1975-Y275','1975-09-11 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 38764,'1975-Y276','1975-09-12 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',96 from dual union all
select 38765,'1975-Y277','1975-09-12 21:37:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',86 from dual union all
select 38766,'1975-Y278','1975-09-16 21:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 38767,'1975-Y279','1975-09-16 23:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 38768,'1975-Y28','1975-01-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 38769,'1975-Y280','1975-09-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 3877,'1956-A208','1956-07-25 00:00:00','military missile','success',845,'1810-31','System Demo','','','Test',1895,'',10 from dual union all
select 38770,'1975-Y281','1975-09-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 38771,'1975-Y282','1975-09-17 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.212','TERLS-786','','Aeron',1467,'',87 from dual union all
select 38772,'1975-Y283','1975-09-18 00:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 38773,'1975-Y284','1975-09-19 04:00:00','suborbital rocket','unknown',725,'','Ozone','','','Aeron',636,'',89 from dual union all
select 38774,'1975-Y285','1975-09-19 15:33:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',86 from dual union all
select 38775,'1975-Y286','1975-09-22 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 38776,'1975-Y287','1975-09-22 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 38777,'1975-Y288','1975-09-23 22:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 38778,'1975-Y289','1975-09-24 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 38779,'1975-Y29','1975-01-26 02:26:00','suborbital rocket','success',1281,'NASA 15.129UE','ARAKS','','','Plasma',856,'',80 from dual union all
select 3878,'1956-A209','1956-02-10 00:00:00','suborbital rocket','success',925,'','Talos target','','','Target',1895,'',20 from dual union all
select 38780,'1975-Y290','1975-09-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 38781,'1975-Y291','1975-09-24 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.213','TERLS-789','','Aeron',1467,'',88 from dual union all
select 38782,'1975-Y292','1975-09-30 19:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 38783,'1975-Y293','1975-09-30 22:52:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 38784,'1975-Y294','1975-10-01 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 38785,'1975-Y295','1975-10-01 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 38786,'1975-Y296','1975-10-01 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.214','TERLS-791','','Aeron',1467,'',83 from dual union all
select 38787,'1975-Y297','1975-10-01 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 38788,'1975-Y298','1975-10-02 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',87 from dual union all
select 38789,'1975-Y299','1975-10-04 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',86 from dual union all
select 3879,'1956-A21','1956-02-17 00:00:00','military missile','success',1214,'N-69C 52-10975','','','','Test',224,'',10 from dual union all
select 38790,'1975-Y30','1975-01-26 02:36:00','suborbital rocket','success',1281,'NASA 15.130UE','ARAKS','','','Plasma',856,'',80 from dual union all
select 38791,'1975-Y300','1975-10-08 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 38792,'1975-Y301','1975-10-08 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 38793,'1975-Y302','1975-10-08 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.215','TERLS-794','','Aeron',1467,'',86 from dual union all
select 38794,'1975-Y303','1975-10-08 19:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 38795,'1975-Y304','1975-10-10 22:34:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',89 from dual union all
select 38796,'1975-Y305','1975-10-14 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 38797,'1975-Y306','1975-10-15 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 38798,'1975-Y307','1975-10-15 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 38799,'1975-Y308','1975-10-15 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.216','TERLS-795','','Aeron',1467,'',85 from dual;
insert into LAUNCH
select 388,'1944-S353','1944-11-16 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1627,'',90 from dual union all
select 3880,'1956-A210','1956-02-17 00:00:00','suborbital rocket','success',925,'','Talos target','','','Target',1895,'',20 from dual union all
select 38800,'1975-Y309','1975-10-17 23:03:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 38801,'1975-Y31','1975-01-22 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 38802,'1975-Y310','1975-10-22 12:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 38803,'1975-Y311','1975-10-22 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',88 from dual union all
select 38804,'1975-Y312','1975-10-22 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 38805,'1975-Y313','1975-10-22 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.217','TERLS-796','','Aeron',1467,'',85 from dual union all
select 38806,'1975-Y314','1975-10-22 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 38807,'1975-Y315','1975-10-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 38808,'1975-Y316','1975-10-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 38809,'1975-Y317','1975-10-29 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.218','TERLS-797','','Aeron',1467,'',84 from dual union all
select 3881,'1956-A211','1956-05-16 00:00:00','suborbital rocket','success',925,'','Pogo Hi II','','','Target',1895,'',20 from dual union all
select 38810,'1975-Y318','1975-10-29 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 38811,'1975-Y319','1975-10-31 21:26:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',90 from dual union all
select 38812,'1975-Y32','1975-01-22 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 38813,'1975-Y320','1975-11-04 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 38814,'1975-Y321','1975-11-05 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 38815,'1975-Y322','1975-11-05 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 38816,'1975-Y323','1975-11-05 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.219','TERLS-798','','Aeron',1467,'',83 from dual union all
select 38817,'1975-Y324','1975-11-06 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',94 from dual union all
select 38818,'1975-Y325','1975-11-07 21:06:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 38819,'1975-Y326','1975-11-08 01:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',81 from dual union all
select 3882,'1956-A212','1956-05-31 00:00:00','suborbital rocket','success',925,'','Pogo Hi','','','Target',1895,'',20 from dual union all
select 38820,'1975-Y327','1975-11-11 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.220','TERLS-800','','Aeron',1467,'',85 from dual union all
select 38821,'1975-Y328','1975-11-12 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 38822,'1975-Y329','1975-11-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 38823,'1975-Y33','1975-01-22 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',91 from dual union all
select 38824,'1975-Y330','1975-11-12 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 38825,'1975-Y331','1975-11-14 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',93 from dual union all
select 38826,'1975-Y332','1975-11-14 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 38827,'1975-Y333','1975-11-18 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 38828,'1975-Y334','1975-11-18 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 38829,'1975-Y335','1975-11-19 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 3883,'1956-A213','1956-05-31 00:00:00','suborbital rocket','success',925,'','Pogo Hi','','','Target',1895,'',20 from dual union all
select 38830,'1975-Y336','1975-11-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 38831,'1975-Y337','1975-11-19 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.221','TERLS-802','','Aeron',1467,'',91 from dual union all
select 38832,'1975-Y338','1975-11-19 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',88 from dual union all
select 38833,'1975-Y339','1975-11-21 03:45:00','suborbital rocket','success',1284,'','Starute,Datasonde','','','Meteo',1821,'',84 from dual union all
select 38834,'1975-Y34','1975-01-22 20:33:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',90 from dual union all
select 38835,'1975-Y340','1975-11-21 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 38836,'1975-Y341','1975-11-21 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 38837,'1975-Y342','1975-11-26 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 38838,'1975-Y343','1975-11-26 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 38839,'1975-Y344','1975-11-26 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 3884,'1956-A214','1956-06-29 00:00:00','suborbital rocket','success',925,'1038','Pogo Hi','','','Target',1895,'',20 from dual union all
select 38840,'1975-Y345','1975-11-26 17:57:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',87 from dual union all
select 38841,'1975-Y346','1975-11-28 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 38842,'1975-Y347','1975-11-29 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 38843,'1975-Y348','1975-12-03 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 38844,'1975-Y349','1975-12-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 38845,'1975-Y35','1975-01-22 23:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 38846,'1975-Y350','1975-12-03 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.223','TERLS-807','','Aeron',1467,'',83 from dual union all
select 38847,'1975-Y351','1975-12-03 19:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 38848,'1975-Y352','1975-12-05 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',95 from dual union all
select 38849,'1975-Y353','1975-12-06 05:34:00','suborbital rocket','unknown',725,'','ISRO 8.224','TERLS-808','','Aeron',1467,'',92 from dual union all
select 3885,'1956-A215','1956-07-06 00:00:00','suborbital rocket','success',925,'','Pogo Hi II','','','Target',1895,'',20 from dual union all
select 38850,'1975-Y354','1975-12-10 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 38851,'1975-Y355','1975-12-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 38852,'1975-Y356','1975-12-10 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.225','TERLS-809','','Aeron',1467,'',90 from dual union all
select 38853,'1975-Y357','1975-12-10 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 38854,'1975-Y358','1975-12-10 23:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',82 from dual union all
select 38855,'1975-Y359','1975-12-11 16:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 38856,'1975-Y36','1975-01-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 38857,'1975-Y360','1975-12-12 03:36:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',86 from dual union all
select 38858,'1975-Y361','1975-12-12 05:09:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 38859,'1975-Y362','1975-12-12 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',95 from dual union all
select 3886,'1956-A216','1956-07-06 00:00:00','suborbital rocket','success',925,'','Pogo Hi II','','','Target',1895,'',20 from dual union all
select 38860,'1975-Y363','1975-12-13 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',81 from dual union all
select 38861,'1975-Y364','1975-12-15 22:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',82 from dual union all
select 38862,'1975-Y365','1975-12-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 38863,'1975-Y366','1975-12-17 11:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',82 from dual union all
select 38864,'1975-Y367','1975-12-17 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.226','TERLS-810','','Aeron',1467,'',87 from dual union all
select 38865,'1975-Y368','1975-12-17 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 38866,'1975-Y369','1975-12-17 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 38867,'1975-Y37','1975-01-29 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',95 from dual union all
select 38868,'1975-Y370','1975-12-17 22:14:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 38869,'1975-Y371','1975-12-17 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 3887,'1956-A217','1956-10-18 00:00:00','suborbital rocket','success',925,'','Pogo Hi Loki Wasp','','','Target',1895,'',20 from dual union all
select 38870,'1975-Y372','1975-12-18 02:10:00','suborbital rocket','success',1281,'NASA 15.145GM','','','','Aeronomy',1470,'',81 from dual union all
select 38871,'1975-Y373','1975-12-18 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.227','TERLS-811','','Aeron',1467,'',87 from dual union all
select 38872,'1975-Y374','1975-12-19 20:35:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 38873,'1975-Y375','1975-12-19 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',95 from dual union all
select 38874,'1975-Y376','1975-12-22 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',95 from dual union all
select 38875,'1975-Y377','1975-12-22 18:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 38876,'1975-Y378','1975-12-22 22:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',873,'',80 from dual union all
select 38877,'1975-Y379','1975-12-23 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 38878,'1975-Y38','1975-01-29 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 38879,'1975-Y380','1975-12-23 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 3888,'1956-A218','1956-12-20 00:00:00','suborbital rocket','success',925,'','Pogo Hi II','','','Target',1895,'',20 from dual union all
select 38880,'1975-Y381','1975-12-24 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 38881,'1975-Y382','1975-12-24 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.228','TERLS-812','','Aeron',1467,'',84 from dual union all
select 38882,'1975-Y383','1975-12-24 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',94 from dual union all
select 38883,'1975-Y384','1975-12-24 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 38884,'1975-Y385','1975-12-27 23:15:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 38885,'1975-Y386','1975-12-28 09:35:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 38886,'1975-Y387','1975-12-29 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',96 from dual union all
select 38887,'1975-Y388','1975-12-30 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 38888,'1975-Y389','1975-12-31 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 38889,'1975-Y39','1975-01-31 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual;
insert into LAUNCH
select 3889,'1956-A219','1956-01-05 00:00:00','military missile','success',771,'5','','','','Test',646,'',12 from dual union all
select 38890,'1975-Y390','1975-12-31 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 38891,'1975-Y391','1975-12-31 15:00:00','suborbital rocket','unknown',725,'','ISRO 8.229','TERLS-813','','Aeron',1467,'',81 from dual union all
select 38892,'1975-Y392','1975-11-26 14:30:00','suborbital rocket','unknown',725,'','ISRO 8.222','TERLS-803','','Aeron',1467,'',84 from dual union all
select 38893,'1975-Y40','1975-02-03 04:15:00','suborbital rocket','success',1281,'NASA 15.138UE','ARAKS','','','Plasma',856,'',80 from dual union all
select 38894,'1975-Y41','1975-02-05 17:02:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',87 from dual union all
select 38895,'1975-Y42','1975-02-05 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 38896,'1975-Y43','1975-02-05 20:30:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',89 from dual union all
select 38897,'1975-Y44','1975-02-07 16:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',96 from dual union all
select 38898,'1975-Y45','1975-02-12 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 38899,'1975-Y46','1975-02-12 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 389,'1944-S354','1944-11-16 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1627,'',90 from dual union all
select 3890,'1956-A22','1956-02-21 00:00:00','atmospheric rocket','success',365,'','E42 Model','','','Test',1821,'',10 from dual union all
select 38900,'1975-Y47','1975-02-14 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 38901,'1975-Y48','1975-02-15 01:25:00','suborbital rocket','success',1281,'NASA 15.131UE','ARAKS','','','Plasma',856,'',80 from dual union all
select 38902,'1975-Y49','1975-02-15 01:36:00','suborbital rocket','success',1281,'NASA 15.132UE','ARAKS','','','Plasma',856,'',80 from dual union all
select 38903,'1975-Y50','1975-02-18 19:15:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 38904,'1975-Y51','1975-02-18 20:35:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 38905,'1975-Y52','1975-02-19 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 38906,'1975-Y53','1975-02-19 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 38907,'1975-Y54','1975-02-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 38908,'1975-Y55','1975-02-21 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 38909,'1975-Y56','1975-02-21 20:58:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 3891,'1956-A220','1956-01-12 00:00:00','military missile','success',771,'RTV 9','','','','Test',646,'',12 from dual union all
select 38910,'1975-Y57','1975-02-25 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 38911,'1975-Y58','1975-02-26 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 38912,'1975-Y59','1975-02-26 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 38913,'1975-Y60','1975-02-26 20:08:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 38914,'1975-Y61','1975-02-28 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 38915,'1975-Y62','1975-02-28 22:24:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 38916,'1975-Y63','1975-03-01 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1371,'SHIR',91 from dual union all
select 38917,'1975-Y64','1975-03-03 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1371,'SHIR',89 from dual union all
select 38918,'1975-Y65','1975-03-03 17:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1371,'SHIR',88 from dual union all
select 38919,'1975-Y66','1975-03-04 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 3892,'1956-A221','1956-01-26 00:00:00','military missile','success',771,'54-3095','','','','Test',646,'',12 from dual union all
select 38920,'1975-Y67','1975-03-05 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 38921,'1975-Y68','1975-03-05 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 38922,'1975-Y69','1975-03-06 18:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1371,'SHIR',83 from dual union all
select 38923,'1975-Y70','1975-03-07 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1371,'SHIR',86 from dual union all
select 38924,'1975-Y71','1975-03-07 19:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1371,'SHIR',89 from dual union all
select 38925,'1975-Y72','1975-03-09 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',882,'KORO',81 from dual union all
select 38926,'1975-Y73','1975-03-11 16:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1371,'SHIR',82 from dual union all
select 38927,'1975-Y74','1975-03-11 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 38928,'1975-Y75','1975-03-12 11:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 38929,'1975-Y76','1975-03-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 3893,'1956-A222','1956-02-10 00:00:00','military missile','success',771,'GM-53-190','','','','Test',646,'',12 from dual union all
select 38930,'1975-Y77','1975-03-13 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1371,'SHIR',89 from dual union all
select 38931,'1975-Y78','1975-03-14 18:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1371,'SHIR',89 from dual union all
select 38932,'1975-Y79','1975-03-15 15:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1371,'SHIR',87 from dual union all
select 38933,'1975-Y80','1975-03-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1371,'SHIR',87 from dual union all
select 38934,'1975-Y81','1975-03-18 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',882,'KORO',83 from dual union all
select 38935,'1975-Y82','1975-03-18 18:40:00','suborbital rocket','unknown',725,'','','','','Aeron',882,'KORO',81 from dual union all
select 38936,'1975-Y83','1975-03-18 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 38937,'1975-Y84','1975-03-19 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 38938,'1975-Y85','1975-03-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 38939,'1975-Y86','1975-03-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 3894,'1956-A223','1956-02-21 00:00:00','military missile','success',771,'2096','','','','Test',646,'',12 from dual union all
select 38940,'1975-Y87','1975-03-19 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1371,'SHIR',88 from dual union all
select 38941,'1975-Y88','1975-03-20 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1371,'SHIR',87 from dual union all
select 38942,'1975-Y89','1975-03-22 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1371,'SHIR',89 from dual union all
select 38943,'1975-Y90','1975-03-25 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 38944,'1975-Y91','1975-03-26 11:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 38945,'1975-Y92','1975-03-26 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 38946,'1975-Y93','1975-03-26 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 38947,'1975-Y94','1975-04-02 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 38948,'1975-Y95','1975-04-02 15:00:00','suborbital rocket','success',725,'','','','','Aeron',856,'',89 from dual union all
select 38949,'1975-Y96','1975-04-02 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 3895,'1956-A224','1956-03-01 00:00:00','military missile','success',771,'53192','','','','Test',646,'',12 from dual union all
select 38950,'1975-Y97','1975-04-02 19:23:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',89 from dual union all
select 38951,'1975-Y98','1975-04-02 22:52:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',86 from dual union all
select 38952,'1975-Y99','1975-04-03 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 38953,'1976-001','1976-01-06 04:52:02','orbital','success',680,'53731-269','Kosmos-787','Tselina-OM','','Sat',456,'',null from dual union all
select 38954,'1976-002','1976-01-07 15:34:58','orbital','success',1508,'76041  -793','Kosmos-788','Zenit-4MK','','Sat',474,'',null from dual union all
select 38955,'1976-003','1976-01-15 05:34:00','deep space','success',1427,'23E-5/TC-5','Helios 2','Helios 2','','Sat',275,'',null from dual union all
select 38956,'1976-004','1976-01-17 23:27:54','orbital','success',386,'606/D119','CTS','CTS','','Sat',234,'',null from dual union all
select 38957,'1976-005','1976-01-20 17:07:41','orbital','success',680,'53749-164','Kosmos-789','Tsiklon-B','','Sat',456,'',null from dual union all
select 38958,'1976-006','1976-01-22 11:38:00','orbital','success',809,'76029  -559','Molniya-1','Molniya-1 11F658 No. 40','','Sat',480,'',null from dual union all
select 38959,'1976-007','1976-01-22 22:26:01','orbital','success',680,'53731-285','Kosmos-790','Tselina-OM','','Sat',456,'',null from dual union all
select 3896,'1956-A225','1956-03-15 00:00:00','military missile','success',771,'6','','','','Test',646,'',12 from dual union all
select 38960,'1976-008','1976-01-28 10:39:01','orbital','success',680,'53731-281','Kosmos-791/798','Strela-1M','','Sat',456,'',null from dual union all
select 38961,'1976-009','1976-01-29 08:30:00','orbital','success',1508,'77016  -218','Kosmos-799','Zenit-2M','','Sat',562,'',null from dual union all
select 38962,'1976-010','1976-01-29 23:56:00','orbital','success',180,'AC-37','Intelsat 4A F-2','INTELSAT IVA F2','','Sat',269,'',null from dual union all
select 38963,'1976-011','1976-02-03 08:16:00','orbital','success',680,'53721-282','Kosmos-800','Zaliv','','Sat',456,'',null from dual union all
select 38964,'1976-012','1976-02-05 14:30:03','orbital','success',678,'64653-174','Kosmos-801','DS-P1-I No. 16','','Sat',460,'',null from dual union all
select 38965,'1976-013','1976-02-11 08:50:00','orbital','success',1508,'77016  -658','Kosmos-802','Zenit-4MK','','Sat',562,'',null from dual union all
select 38966,'1976-014','1976-02-12 13:00:01','orbital','success',680,'53731-286','Kosmos-803','Lira','','Sat',457,'',null from dual union all
select 38967,'1976-015','1976-02-16 08:29:00','orbital','success',1446,'45049-156','Kosmos-804','IS','','Sat',589,'',null from dual union all
select 38968,'1976-016','1976-02-19 07:52:52','orbital','success',1365,'182/BIIA-25','DMSP 11534','DMSP Block 5C F-3','','Sat',1720,'',null from dual union all
select 38969,'1976-017','1976-02-19 22:32:00','orbital','success',386,'603/D120','Marisat 1','Marisat A','','Sat',234,'',null from dual union all
select 3897,'1956-A226','1956-03-29 00:00:00','military missile','success',771,'54-3098','','','','Test',646,'',12 from dual union all
select 38970,'1976-018','1976-02-20 14:01:02','orbital','success',1238,'K 15000-013','Kosmos-805','Yantar''-2K No. 4','','Sat',474,'',null from dual union all
select 38971,'1976-019','1976-02-29 03:30:00','orbital','success',822,'N-2(F)','Ume','ISS','','Sat',1474,'',null from dual union all
select 38972,'1976-020','1976-03-10 08:00:00','orbital','success',1238,'78026  -666','Kosmos-806','Zenit-4MK','','Sat',562,'',null from dual union all
select 38973,'1976-021','1976-03-11 19:45:00','orbital','success',809,'78031  -575','Molniya-1','Molniya-1','','Sat',473,'',null from dual union all
select 38974,'1976-022','1976-03-12 13:30:00','orbital','success',680,'53746-310','Kosmos-807','Vektor','','Sat',456,'',null from dual union all
select 38975,'1976-023','1976-03-15 01:25:40','orbital','success',1425,'23C-12 (3C-30)','P74-1','LES 8','','Sat',274,'',null from dual union all
select 38976,'1976-024','1976-03-16 17:22:00','orbital','success',1510,'78026  -387','Kosmos-808','Tselina-D','','Sat',473,'',null from dual union all
select 38977,'1976-025','1976-03-18 09:15:00','orbital','success',1238,'78026  -222','Kosmos-809','Zenit-2M','','Sat',562,'',null from dual union all
select 38978,'1976-026','1976-03-19 19:31:00','orbital','success',809,'76029  -558','Molniya-1','Molniya-1 11F658 No. 42','','Sat',480,'',null from dual union all
select 38979,'1976-027','1976-03-22 18:14:00','orbital','success',1403,'24B-18 (3B-52)','KH-8A','GAMBIT-3 Mission 4346','','Sat',1778,'',null from dual;
insert into LAUNCH
select 3898,'1956-A227','1956-04-16 00:00:00','military missile','success',771,'221','','','','Test',646,'',12 from dual union all
select 38980,'1976-028','1976-03-26 15:00:00','orbital','success',1508,'77016  -656','Kosmos-810','Zenit-4MK','','Sat',474,'',null from dual union all
select 38981,'1976-029','1976-03-26 22:47:00','orbital','success',390,'610/D121','Satcom 2','RCA-B','','Sat',233,'',null from dual union all
select 38982,'1976-030','1976-03-31 12:50:03','orbital','success',1225,'76029  -838','Kosmos-811','Orion','','Sat',473,'',null from dual union all
select 38983,'1976-031','1976-04-06 04:14:00','orbital','success',680,'53731-280','Kosmos-812','Tselina-OM','','Sat',456,'',null from dual union all
select 38984,'1976-032','1976-04-07 13:05:00','orbital','success',1510,'76041  -369','Meteor','Meteor No. 37','','Sat',473,'',null from dual union all
select 38985,'1976-033','1976-04-09 08:30:00','orbital','success',1508,'77016  -217','Kosmos-813','Zenit-2M','','Sat',474,'',null from dual union all
select 38986,'1976-034','1976-04-13 17:15:37','orbital','success',1446,'51019-154','Kosmos-814','IS','','Sat',589,'',null from dual union all
select 38987,'1976-035','1976-04-22 20:46:00','orbital','success',386,'608/D122','NATO 3A','NATO 3A','','Sat',234,'',null from dual union all
select 38988,'1976-036','1976-04-28 09:30:00','orbital','success',1508,'76041  -784','Kosmos-815','Zenit-4MK','','Sat',474,'',null from dual union all
select 38989,'1976-037','1976-04-28 13:30:00','orbital','success',680,'53746-301','Kosmos-816','Romb 1','','Sat',456,'',null from dual union all
select 3899,'1956-A228','1956-04-25 00:00:00','military missile','success',771,'GM 53-195','','','','Test',646,'',12 from dual union all
select 38990,'1976-038','1976-04-30 19:12:00','orbital','success',164,'59F','OPS 6431    [NOSS1]','PARCAE 1','','Sat',1776,'',null from dual union all
select 38991,'1976-039','1976-05-04 08:00:00','orbital','success',384,'609/D123','Lageos','Lageos','','Sat',1724,'',null from dual union all
select 38992,'1976-040','1976-05-05 07:50:00','orbital','success',1508,'77016  -660','Kosmos-817','Zenit-4MK','','Sat',562,'',null from dual union all
select 38993,'1976-041','1976-05-12 17:57:00','orbital','success',809,'77016  -598','Molniya-3','Molniya-3 No. 16','','Sat',473,'',null from dual union all
select 38994,'1976-042','1976-05-13 22:28:00','orbital','success',180,'AC-38','Comstar D-1','Comstar D-1','','Sat',268,'',null from dual union all
select 38995,'1976-043','1976-05-15 13:30:00','orbital','success',1510,'76042  -388','Meteor','Meteor-Priroda No. 2-1','','Sat',474,'',null from dual union all
select 38996,'1976-044','1976-05-18 10:59:57','orbital','success',678,'29519-182','Kosmos-818','DS-P1-Yu No. 78','','Sat',460,'',null from dual union all
select 38997,'1976-045','1976-05-20 09:00:00','orbital','success',1508,'77016  -219','Kosmos-819','Zenit-2M','','Sat',562,'',null from dual union all
select 38998,'1976-046','1976-05-21 07:00:00','orbital','success',1238,'76042  -022','Kosmos-820','Fram','','Sat',474,'',null from dual union all
select 38999,'1976-047','1976-05-22 07:42:16','orbital','success',1130,'S179CR','P76-5','NNS O-15','','Sat',1779,'',null from dual union all
select 39,'1944-A03','1944-02-29 00:00:00','military missile','failure',1519,'2','','','','Test',673,'',0 from dual union all
select 390,'1944-S355','1944-11-16 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1627,'',90 from dual union all
select 3900,'1956-A229','1956-05-14 00:00:00','military missile','success',771,'M-208','RTV','','','Test',646,'',12 from dual union all
select 39000,'1976-048','1976-05-26 09:00:00','orbital','success',1508,'77016  -659','Kosmos-821','Zenit-4MK','','Sat',474,'',null from dual union all
select 39001,'1976-049','1976-05-28 15:00:01','orbital','success',680,'53716-328','Kosmos-822','Vektor','','Sat',456,'',null from dual union all
select 39002,'1976-050','1976-06-02 20:56:00','orbital','success',1405,'34B-5  (3B-55)','OPS 7837','SDS 1','','Sat',1778,'',null from dual union all
select 39003,'1976-051','1976-06-02 22:30:15','orbital','success',680,'53731-283','Kosmos-823','Zaliv','','Sat',457,'',null from dual union all
select 39004,'1976-052','1976-06-08 07:00:00','orbital','success',1508,'77016  -657','Kosmos-824','Zenit-4MK','','Sat',562,'',null from dual union all
select 39005,'1976-053','1976-06-10 00:09:00','orbital','success',386,'601/D124','Marisat 2','Marisat B','','Sat',233,'',null from dual union all
select 39006,'1976-054','1976-06-15 13:19:01','orbital','success',680,'53731-277','Kosmos-825/832','Strela-1M','','Sat',457,'',null from dual union all
select 39007,'1976-055','1976-06-16 13:09:58','orbital','success',1508,'77016  -654','Kosmos-833','Zenit-4MK','','Sat',474,'',null from dual union all
select 39008,'1976-056','1976-06-19 16:00:01','orbital','success',680,'53731-276','Interkosmos-15','AUOS-Z-T-IK','','Sat',456,'',null from dual union all
select 39009,'1976-057','1976-06-22 18:04:00','orbital','success',949,'290-02','Salyut-5','Almaz OPS 3','','Sat',587,'',null from dual union all
select 3901,'1956-A23','1956-02-22 20:54:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 39010,'1976-058','1976-06-24 07:10:01','orbital','success',1238,'78026  -220','Kosmos-834','Zenit-2M','','Sat',474,'',null from dual union all
select 39011,'1976-059','1976-06-26 03:00:01','orbital','success',1425,'23C-10 (3C-28)','DSP 6','DSP 7','','Sat',274,'',null from dual union all
select 39012,'1976-060','1976-06-29 07:20:00','orbital','success',1508,'77016  -661','Kosmos-835','Zenit-4MK','','Sat',562,'',null from dual union all
select 39013,'1976-061','1976-06-29 08:12:00','orbital','success',680,'53731-284','Kosmos-836','Strela-2M','','Sat',457,'',null from dual union all
select 39014,'1976-062','1976-07-01 08:05:38','orbital','failure',809,'77016  -594','Kosmos-837','Molniya-2','','Sat',475,'',null from dual union all
select 39015,'1976-063','1976-07-02 10:30:00','orbital','success',1446,'67012-101T','Kosmos-838','US-P','','Sat',589,'',null from dual union all
select 39016,'1976-064','1976-07-06 12:08:45','orbital','success',1223,'F 15000-034','Soyuz-21','Soyuz 7K-T No. 41','','Sat',480,'',null from dual union all
select 39017,'1976-065','1976-07-08 18:30:00','orbital','success',1426,'23D-14','HEXAGON SV12','HEXAGON SV-12','','Sat',1777,'',null from dual union all
select 39018,'1976-066','1976-07-08 23:31:00','orbital','success',385,'611/D125','Palapa A1','Palapa A1','','Sat',233,'',null from dual union all
select 39019,'1976-067','1976-07-08 21:08:00','orbital','success',680,'53716-324','Kosmos-839','Lira','','Sat',456,'',null from dual union all
select 3902,'1956-A230','1956-05-24 00:00:00','military missile','success',771,'543099','','','','Test',646,'',12 from dual union all
select 39020,'1976-068','1976-07-14 09:00:00','orbital','success',1238,'78026  -221','Kosmos-840','Zenit-2M','','Sat',475,'',null from dual union all
select 39021,'1976-069','1976-07-15 13:11:00','orbital','success',680,'53746-304','Kosmos-841','Strela-2M','','Sat',456,'',null from dual union all
select 39022,'1976-070','1976-07-21 10:20:00','orbital','success',680,'53731-266','Kosmos-842','Geoid','','Sat',456,'',null from dual union all
select 39023,'1976-071','1976-07-21 15:14:00','orbital','success',1446,'45012-102T','Kosmos-843','IS','','Sat',589,'',null from dual union all
select 39024,'1976-072','1976-07-22 15:40:01','orbital','success',1238,'78026  -958','Kosmos-844','Yantar''-2K No. 5','','Sat',474,'',null from dual union all
select 39025,'1976-073','1976-07-22 22:04:00','orbital','success',180,'AC-40','Comstar D-2','Comstar D-2','','Sat',269,'',null from dual union all
select 39026,'1976-074','1976-07-23 15:49:00','orbital','success',809,'76029  -556','Molniya-1','Molniya-1 11F658 No. 43','','Sat',480,'',null from dual union all
select 39027,'1976-075','1976-07-27 05:21:00','orbital','success',680,'53746-307','Kosmos-845','Tselina-OM','','Sat',457,'',null from dual union all
select 39028,'1976-076','1976-07-27 12:00:02','orbital','success',680,'53746-316','Interkosmos-16','DS-U3-IK No. 6?','','Sat',623,'',null from dual union all
select 39029,'1976-077','1976-07-29 17:07:00','orbital','success',381,'605/D126','NOAA 5','ITOS H','','Sat',1724,'',null from dual union all
select 3903,'1956-A231','1956-05-25 00:00:00','military missile','success',771,'53-191','RTV','','','Test',646,'',12 from dual union all
select 39030,'1976-078','1976-07-29 20:02:44','orbital','success',680,'53716-329','Kosmos-846','Zaliv','','Sat',456,'',null from dual union all
select 39031,'1976-079','1976-08-04 13:40:00','orbital','success',1238,'78026  -668','Kosmos-847','Zenit-4MK','','Sat',475,'',null from dual union all
select 39032,'1976-080','1976-08-06 22:21:00','orbital','success',1405,'34B-6  (3B-56)','OPS 7940','SDS 2','','Sat',1778,'',null from dual union all
select 39033,'1976-081','1976-08-09 15:04:12','deep space','success',954,'288-02','Luna-24','E-8-5M No. 413','','Sat',587,'',null from dual union all
select 39034,'1976-082','1976-08-12 13:30:00','orbital','success',1238,'76042  -241','Kosmos-848','Zenit-2M','','Sat',474,'',null from dual union all
select 39035,'1976-083','1976-08-18 09:30:01','orbital','success',678,'64619-184','Kosmos-849','DS-P1-I No. 17','','Sat',460,'',null from dual union all
select 39036,'1976-084','1976-08-26 10:59:57','orbital','success',678,'29519-183','Kosmos-850','DS-P1-Yu No. 79','','Sat',460,'',null from dual union all
select 39037,'1976-085','1976-08-27 14:34:58','orbital','success',1510,'78026  -385','Kosmos-851','Tselina-D','','Sat',475,'',null from dual union all
select 39038,'1976-086','1976-08-28 09:00:00','orbital','success',1238,'76042  -690','Kosmos-852','Zenit-4MK','','Sat',562,'',null from dual union all
select 39039,'1976-087','1976-08-30 11:45:00','orbital','success',492,'701-07','JSSW 701-07','JSSW','','Sat',810,'',null from dual union all
select 3904,'1956-A232','1956-06-11 00:00:00','military missile','success',771,'GM 53-194','RTV','','','Test',646,'',12 from dual union all
select 39040,'1976-088','1976-09-01 03:23:00','orbital','failure',809,'76041  -576','Kosmos-853','Molniya-2','','Sat',474,'',null from dual union all
select 39041,'1976-089','1976-09-01 21:14:02','orbital','success',1132,'S197C','TIP 3','TIP 3','','Sat',1779,'',null from dual union all
select 39042,'1976-090','1976-09-03 09:20:00','orbital','success',1238,'78026  -665','Kosmos-854','Zenit-4MK','','Sat',475,'',null from dual union all
select 39043,'1976-091','1976-09-11 08:00:55','orbital','success',1374,'172','DMSP 5D F-1','DMSP Block 5D S-1','','Sat',1720,'',null from dual union all
select 39044,'1976-092','1976-09-11 18:24:00','orbital','success',956,'289-01','Raduga','Gran'' No. 12L','','Sat',588,'',null from dual union all
select 39045,'1976-093','1976-09-15 09:48:30','orbital','success',1238,'E 15000-069','Soyuz-22','Soyuz 7K-TM No. 74','','Sat',480,'',null from dual union all
select 39046,'1976-094','1976-09-15 18:50:00','orbital','success',1403,'24B-17 (3B-51)','KH-8A','GAMBIT-3 Mission 4347','','Sat',1778,'',null from dual union all
select 39047,'1976-095','1976-09-21 11:40:00','orbital','success',1238,'78026  -862','Kosmos-855','Orion','','Sat',474,'',null from dual union all
select 39048,'1976-096','1976-09-22 09:30:00','orbital','success',1238,'76042  -240','Kosmos-856','Zenit-2M','','Sat',562,'',null from dual union all
select 39049,'1976-097','1976-09-24 15:00:00','orbital','success',1238,'78026  -669','Kosmos-857','Zenit-4MK','','Sat',474,'',null from dual union all
select 3905,'1956-A233','1956-10-03 00:00:00','military missile','success',771,'','','','','Test',646,'',12 from dual union all
select 39050,'1976-098','1976-09-29 07:04:01','orbital','success',680,'53716-333','Kosmos-858','Strela-2M','','Sat',457,'',null from dual union all
select 39051,'1976-099','1976-10-10 09:35:00','orbital','success',1238,'78026  -664','Kosmos-859','Zenit-4MK','','Sat',562,'',null from dual union all
select 39052,'1976-100','1976-10-14 17:39:18','orbital','success',1223,'E 15000-035','Soyuz-23','Soyuz 7K-T(A9) No. 65','','Sat',480,'',null from dual union all
select 39053,'1976-101','1976-10-14 22:44:00','orbital','success',386,'614/D127','Marisat 3','Marisat C','','Sat',233,'',null from dual union all
select 39054,'1976-102','1976-10-15 22:59:38','orbital','success',1510,'76041  -370','Meteor','Meteor No. 35','','Sat',474,'',null from dual union all
select 39055,'1976-103','1976-10-17 18:06:43','orbital','success',1446,'45051-109','Kosmos-860','US-A reactor','','Sat',589,'',null from dual union all
select 39056,'1976-104','1976-10-21 16:53:00','orbital','success',1446,'67011-111','Kosmos-861','US-A reactor','','Sat',589,'',null from dual union all
select 39057,'1976-105','1976-10-22 09:11:59','orbital','success',809,'78026  -600','Kosmos-862','Oko','','Sat',475,'',null from dual union all
select 39058,'1976-106','1976-10-25 14:30:00','orbital','success',1238,'78026  -686','Kosmos-863','Zenit-4MK','','Sat',475,'',null from dual union all
select 39059,'1976-107','1976-10-26 14:50:00','orbital','success',956,'290-01','Ekran','Ekran No. 11L','','Sat',588,'',null from dual union all
select 3906,'1956-A234','1956-11-01 00:00:00','military missile','success',771,'','','','','Test',646,'',12 from dual union all
select 39060,'1976-108','1976-10-29 12:39:44','orbital','success',680,'53772-414','Kosmos-864','Parus','','Sat',457,'',null from dual union all
select 39061,'1976-109','1976-11-01 11:20:00','orbital','success',1238,'78026  -223','Kosmos-865','Zenit-2M','','Sat',475,'',null from dual union all
select 39062,'1976-110','1976-11-11 10:45:00','orbital','success',1238,'78026  -673','Kosmos-866','Zenit-4MK','','Sat',562,'',null from dual union all
select 39063,'1976-111','1976-11-23 16:27:00','orbital','success',1238,'E 15000-052','Kosmos-867','Zenit-6','','Sat',475,'',null from dual union all
select 39064,'1976-112','1976-11-25 03:59:00','orbital','success',807,'E 15000-153','Prognoz-5','SO-M No. 505','','Sat',562,'',null from dual union all
select 39065,'1976-113','1976-11-26 14:30:00','orbital','success',1446,'45012-102','Kosmos-868','US-P','','Sat',589,'',null from dual union all
select 39066,'1976-114','1976-11-29 16:00:00','orbital','success',1238,'E 15000-071','Kosmos-869','Soyuz 7K-S No. 3L','','Sat',480,'',null from dual union all
select 39067,'1976-115','1976-12-02 00:17:00','orbital','success',680,'53731-273','Kosmos-870','Tselina-OM','','Sat',457,'',null from dual union all
select 39068,'1976-116','1976-12-02 02:44:57','orbital','success',809,'78026  -597','Molniya-2','Molniya-2','','Sat',475,'',null from dual;
insert into LAUNCH
select 39069,'1976-117','1976-12-07 04:38:00','orbital','success',301,'CZ2C-2   (5)','FSW','FSW No. 3','','Sat',810,'',null from dual union all
select 3907,'1956-A235','1956-11-14 00:00:00','military missile','success',771,'','','','','Test',646,'',12 from dual union all
select 39070,'1976-118','1976-12-07 10:23:00','orbital','success',680,'53716-330','Kosmos-871/878','Strela-1M','','Sat',456,'',null from dual union all
select 39071,'1976-119','1976-12-09 10:00:00','orbital','success',1238,'76042  -238','Kosmos-879','Zenit-2M','','Sat',475,'',null from dual union all
select 39072,'1976-120','1976-12-09 20:00:00','orbital','success',680,'53719-153','Kosmos-880','Lira','','Sat',457,'',null from dual union all
select 39073,'1976-121','1976-12-15 01:30:00','orbital','success',949,'289-02','Kosmos-881/882','TKS VA No. 009A/1','','Sat',588,'',null from dual union all
select 39074,'1976-122','1976-12-15 13:59:59','orbital','success',680,'53782-426','Kosmos-883','Tsikada','','Sat',456,'',null from dual union all
select 39075,'1976-123','1976-12-17 09:30:00','orbital','success',1238,'78026  -674','Kosmos-884','Zenit-4MK','','Sat',562,'',null from dual union all
select 39076,'1976-124','1976-12-17 12:00:00','orbital','success',680,'53721-255','Kosmos-885','Romb 2','','Sat',457,'',null from dual union all
select 39077,'1976-125','1976-12-19 18:19:00','orbital','success',1426,'23D-15','KH-11','KH11-1 KENNAN','','Sat',1777,'',null from dual union all
select 39078,'1976-126','1976-12-27 12:05:00','orbital','success',1446,'45023-301T','Kosmos-886','IS','','Sat',589,'',null from dual union all
select 39079,'1976-127','1976-12-28 06:38:00','orbital','success',809,'76042  -602','Molniya-3','Molniya-3 No. 17','','Sat',475,'',null from dual union all
select 3908,'1956-A236','1956-01-19 00:00:00','atmospheric rocket','success',563,'13','HTV 1366','','','Heat transfer',1895,'',10 from dual union all
select 39080,'1976-128','1976-12-28 07:49:56','orbital','success',680,'65072-415','Kosmos-887','Parus','','Sat',456,'',null from dual union all
select 39081,'1976-A01','1976-01-09 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',48 from dual union all
select 39082,'1976-A02','1976-01-19 16:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',48 from dual union all
select 39083,'1976-A03','1976-01-21 15:40:00','suborbital rocket','failure',1189,'INTA MSCL-7603','Winter Anomaly','','','Aeron/Ionos',396,'',15 from dual union all
select 39084,'1976-A04','1976-01-22 00:00:00','atmospheric rocket','success',908,'FLAME 7','FLAME','','','Test',1490,'F-4',17 from dual union all
select 39085,'1976-A05','1976-01-26 00:00:00','atmospheric rocket','success',908,'FLAME 8','FLAME','','','Test',1490,'F-4',17 from dual union all
select 39086,'1976-A06','1976-01-27 17:30:00','suborbital rocket','failure',1209,'Heatpipe I','DLR K-GR-73','','','Tech',416,'',2 from dual union all
select 39087,'1976-A07','1976-01-28 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',48 from dual union all
select 39088,'1976-A08','1976-02-02 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',48 from dual union all
select 39089,'1976-A09','1976-02-05 19:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',45 from dual union all
select 3909,'1956-A237','1956-02-21 00:00:00','atmospheric rocket','success',563,'2A24','HTV 1366','','','Heat transfer',1895,'',10 from dual union all
select 39090,'1976-A10','1976-02-07 00:05:10','suborbital rocket','success',835,'','GI-PFRR Launcher T.','','','Test',1047,'',1 from dual union all
select 39091,'1976-A11','1976-02-26 00:00:00','suborbital rocket','failure',1219,'XV-01','Serido','','','Test',330,'',47 from dual union all
select 39092,'1976-A12','1976-03-02 03:24:30','suborbital rocket','failure',837,'NASA 14.461UE','Auroral NO','','','Plasma',428,'',0 from dual union all
select 39093,'1976-A13','1976-03-05 21:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 39094,'1976-A14','1976-03-05 21:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',43 from dual union all
select 39095,'1976-A15','1976-03-08 22:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 39096,'1976-A16','1976-03-10 09:10:00','suborbital rocket','failure',220,'Astro 6-1','DLR H-BB4-74','INTA BB4-BAC-7601','','EUV Astronomy',396,'',0 from dual union all
select 39097,'1976-A17','1976-03-10 14:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',44 from dual union all
select 39098,'1976-A18','1976-03-12 12:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',47 from dual union all
select 39099,'1976-A19','1976-03-29 06:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',48 from dual union all
select 391,'1944-S356','1944-11-17 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1627,'',90 from dual union all
select 3910,'1956-A238','1956-03-02 00:00:00','atmospheric rocket','success',563,'2A25','HTV 1366','','','Heat transfer',1895,'',10 from dual union all
select 39100,'1976-A20','1976-03-31 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',46 from dual union all
select 39101,'1976-A21','1976-04-12 22:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 39102,'1976-A22','1976-04-14 12:01:00','military missile','success',1099,'','','','','Test',293,'FB-111',10 from dual union all
select 39103,'1976-A23','1976-04-14 12:21:00','military missile','success',1099,'','','','','Test',293,'FB-111',10 from dual union all
select 39104,'1976-A24','1976-04-26 06:48:00','suborbital rocket','failure',906,'GL A10.403-4','Auroral ions','','','Aeron',428,'',19 from dual union all
select 39105,'1976-A25','1976-05-03 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',48 from dual union all
select 39106,'1976-A26','1976-05-05 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 39107,'1976-A27','1976-05-07 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',0 from dual union all
select 39108,'1976-A28','1976-05-11 05:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',48 from dual union all
select 39109,'1976-A29','1976-05-11 11:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',0 from dual union all
select 3911,'1956-A239','1956-03-13 00:00:00','atmospheric rocket','success',563,'2F36','HTV RS','','','Heat transfer',1895,'',10 from dual union all
select 39110,'1976-A30','1976-05-19 14:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',39 from dual union all
select 39111,'1976-A31','1976-05-24 00:00:00','military missile','failure',1452,'63022-82-440T','15A30 UBP-8 ZP','GCh 15B103?','','OT',532,'',0 from dual union all
select 39112,'1976-A32','1976-05-24 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',49 from dual union all
select 39113,'1976-A33','1976-06-03 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 39114,'1976-A34','1976-06-16 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',0 from dual union all
select 39115,'1976-A35','1976-06-23 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',45 from dual union all
select 39116,'1976-A36','1976-06-28 15:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',46 from dual union all
select 39117,'1976-A37','1976-06-28 21:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 39118,'1976-A38','1976-06-30 17:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',49 from dual union all
select 39119,'1976-A39','1976-07-09 20:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 3912,'1956-A24','1956-02-28 22:16:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 39120,'1976-A40','1976-07-12 21:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 39121,'1976-A41','1976-07-13 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 39122,'1976-A42','1976-07-14 21:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 39123,'1976-A43','1976-07-16 21:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 39124,'1976-A44','1976-07-18 01:23:00','suborbital rocket','failure',624,'NASA 12.1776GT','Chem Release test','','','Test/Plasma',1821,'',25 from dual union all
select 39125,'1976-A45','1976-07-21 23:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 39126,'1976-A46','1976-07-23 12:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',46 from dual union all
select 39127,'1976-A47','1976-08-04 14:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',330,'',48 from dual union all
select 39128,'1976-A48','1976-08-06 20:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 39129,'1976-A49','1976-08-13 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 3913,'1956-A240','1956-03-29 00:00:00','atmospheric rocket','success',563,'2A26','HTV 1366','','','Heat transfer',1895,'',10 from dual union all
select 39130,'1976-A50','1976-08-19 20:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 39131,'1976-A51','1976-08-21 03:55:00','military missile','failure',1452,'49242-83-440T','15A30 UBP-10 ZP','GCh 15B103','','OT',532,'',0 from dual union all
select 39132,'1976-A52','1976-08-21 03:40:00','suborbital rocket','failure',846,'GL A11.605','TREP ABRES Density','Sphere','','Aeron',871,'',15 from dual union all
select 39133,'1976-A53','1976-09-03 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',49 from dual union all
select 39134,'1976-A54','1976-09-03 17:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',49 from dual union all
select 39135,'1976-A55','1976-09-08 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',46 from dual union all
select 39136,'1976-A56','1976-09-10 15:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',31 from dual union all
select 39137,'1976-A57','1976-09-15 14:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',308,'',44 from dual union all
select 39138,'1976-A58','1976-09-21 14:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',0 from dual union all
select 39139,'1976-A59','1976-09-22 14:13:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',330,'',39 from dual union all
select 3914,'1956-A241','1956-04-03 00:00:00','atmospheric rocket','success',563,'AF27','HTV RS','','','Heat transfer',1895,'',10 from dual union all
select 39140,'1976-A60','1976-09-22 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',0 from dual union all
select 39141,'1976-A61','1976-09-23 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 39142,'1976-A62','1976-10-06 15:04:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',9 from dual union all
select 39143,'1976-A63','1976-10-06 19:30:00','suborbital rocket','failure',839,'NASA 10.317UI','','','','Ionos',1821,'',20 from dual union all
select 39144,'1976-A64','1976-10-09 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 39145,'1976-A65','1976-10-13 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',42 from dual union all
select 39146,'1976-A66','1976-10-13 17:12:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',276,'',9 from dual union all
select 39147,'1976-A67','1976-10-14 17:17:00','suborbital rocket','failure',290,'ISRO 5.35','ISRO 5.35','','','Plasma',1467,'',0 from dual union all
select 39148,'1976-A68','1976-10-15 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 39149,'1976-A69','1976-10-21 00:00:00','military missile','failure',744,'','15A15 LKI-36','','','Test',524,'',0 from dual union all
select 3915,'1956-A242','1956-04-05 00:00:00','atmospheric rocket','success',563,'2F38','HTV RS','','','Heat transfer',1895,'',10 from dual union all
select 39150,'1976-A70','1976-10-22 13:00:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',43 from dual union all
select 39151,'1976-A71','1976-10-28 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 39152,'1976-A72','1976-10-28 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 39153,'1976-A73','1976-11-05 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',47 from dual union all
select 39154,'1976-A74','1976-11-15 16:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',41 from dual union all
select 39155,'1976-A75','1976-12-07 16:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',48 from dual union all
select 39156,'1976-A76','1976-12-14 00:00:00','military missile','failure',1452,'51122-79-008T','15A30 UBP-13 ZP','GCh 15B103?','','OT',535,'',0 from dual union all
select 39157,'1976-A77','1976-12-14 12:00:00','military missile','success',1099,'','','','','Test',1049,'FB-111',10 from dual union all
select 39158,'1976-A78','1976-12-14 12:20:00','military missile','success',1099,'','','','','Test',1049,'FB-111',10 from dual;
insert into LAUNCH
select 39159,'1976-A79','1976-12-15 14:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',45 from dual union all
select 3916,'1956-A243','1956-04-11 00:00:00','atmospheric rocket','success',563,'2AM39','HTV RS','','','Heat transfer',1895,'',10 from dual union all
select 39160,'1976-A80','1976-12-21 17:32:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',47 from dual union all
select 39161,'1976-A81','1976-12-28 01:00:00','military missile','failure',1452,'51142-79-011T','15A30 LKI-B7','GCh 15B126','','OT',534,'',0 from dual union all
select 39162,'1976-A82','1976-12-30 00:00:00','military missile','failure',744,'','15A15 LKI-40','','','Test',541,'',0 from dual union all
select 39163,'1976-A83','1976-07-24 00:00:00','suborbital rocket','failure',765,'Sandia 497-007','','','','Test',1489,'',17 from dual union all
select 39164,'1976-A84','1976-06-09 18:00:00','atmospheric rocket','success',835,'G1-8283','','','','Test?',1821,'',20 from dual union all
select 39165,'1976-A85','1976-03-09 13:03:00','atmospheric rocket','success',1309,'Sandia 487-509','SAMS/TATER 509','','','Test',1821,'',5 from dual union all
select 39166,'1976-A86','1976-01-14 00:00:00','atmospheric rocket','success',1309,'Sandia 487-602','SAMS/TATER 602','','','RV Test',1490,'',4 from dual union all
select 39167,'1976-A87','1976-03-27 22:58:00','atmospheric rocket','success',1309,'Sandia 487-516','SAMS/TATER 516','','','RV Test',1490,'',4 from dual union all
select 39168,'1976-A88','1976-04-25 20:47:00','atmospheric rocket','success',1309,'Sandia 487-512','SAMS/TATER 512','','','RV Test',1490,'',4 from dual union all
select 39169,'1976-A89','1976-11-30 00:00:00','atmospheric rocket','failure',12,'WSRL ADTV1','','','','Test',1832,'',1 from dual union all
select 3917,'1956-A244','1956-04-26 00:00:00','atmospheric rocket','success',563,'2A27','HTV','','','Heat transfer',1895,'',10 from dual union all
select 39170,'1976-F01','1976-02-04 06:00:00','orbital','failure',816,'M-3C-3','CORSA','CORSA','','Sat',835,'',84 from dual union all
select 39171,'1976-F02','1976-10-04 11:00:00','orbital','failure',1238,'E 15000-056','','Fram','','Sat',475,'',40 from dual union all
select 39172,'1976-F03','1976-11-10 09:05:00','orbital','failure',492,'701-08','JSSW','JSSW?','','Sat',810,'',185 from dual union all
select 39173,'1976-M01','1976-01-18 19:28:00','suborbital rocket','success',132,'NASA 23.03UE','','','','Plasma',1821,'',76 from dual union all
select 39174,'1976-M02','1976-01-20 13:40:00','suborbital rocket','success',920,'INTA MPGAFD-7602','Winter Anomaly','','','Ionos',396,'',60 from dual union all
select 39175,'1976-M03','1976-01-20 16:10:00','suborbital rocket','success',1190,'INTA MS3CLO-7603','Winter Anomaly','','','Aeron/Ionos',396,'',80 from dual union all
select 39176,'1976-M04','1976-01-21 13:40:00','suborbital rocket','success',920,'INTA MPGAFD-7603','Winter Anomaly','','','Aeron/Ionos',396,'',79 from dual union all
select 39177,'1976-M05','1976-01-23 10:06:40','suborbital rocket','success',1162,'WA630.50-1','Optical Track Test','','','Test',1047,'',80 from dual union all
select 39178,'1976-M06','1976-01-23 19:32:00','suborbital rocket','success',132,'NASA 23.04UE','','','','Plasma',1821,'',76 from dual union all
select 39179,'1976-M07','1976-02-02 02:08:30','suborbital rocket','failure',227,'AAF-VB-35','','','','Auroral/Ionos',428,'',55 from dual union all
select 3918,'1956-A245','1956-05-22 00:00:00','atmospheric rocket','success',563,'2M40','HTV','','','Heat transfer',1895,'',10 from dual union all
select 39180,'1976-M08','1976-02-29 13:04:00','suborbital rocket','success',290,'ISRO 5.29','ISRO 5.29','','','Aeron/Ionos',1467,'',56 from dual union all
select 39181,'1976-M09','1976-09-30 12:53:25','suborbital rocket','success',867,'NASA 18.180GE','','','','Plasma',1047,'',65 from dual union all
select 39182,'1976-M10','1976-02-18 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-87','','','Meteo',350,'',75 from dual union all
select 39183,'1976-M11','1976-04-14 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-90','','','Meteo',350,'',75 from dual union all
select 39184,'1976-M12','1976-04-21 18:33:00','suborbital rocket','unknown',1506,'T1-8278','','Instrumented Sphere 4','','Aeron',1821,'',100 from dual union all
select 39185,'1976-M13','1976-06-09 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-94','','','Meteo',350,'',75 from dual union all
select 39186,'1976-M14','1976-01-21 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-85','','','Meteo',350,'',75 from dual union all
select 39187,'1976-M15','1976-01-28 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-86','','','Meteo',350,'',75 from dual union all
select 39188,'1976-M16','1976-03-17 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-88','','','Meteo',350,'',75 from dual union all
select 39189,'1976-M17','1976-04-01 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-89','','','Meteo',350,'',75 from dual union all
select 3919,'1956-A246','1956-06-07 00:00:00','atmospheric rocket','success',563,'S16','HTV','','','Heat transfer',1895,'',10 from dual union all
select 39190,'1976-M18','1976-04-29 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-91','','','Meteo',350,'',75 from dual union all
select 39191,'1976-M19','1976-05-12 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-92','','','Meteo',350,'',75 from dual union all
select 39192,'1976-M20','1976-09-15 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-95','','','Meteo',350,'',75 from dual union all
select 39193,'1976-M21','1976-09-29 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-96','','','Meteo',350,'',75 from dual union all
select 39194,'1976-M22','1976-10-13 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-97','','','Meteo',350,'',75 from dual union all
select 39195,'1976-M23','1976-12-01 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-100','','','Meteo',350,'',75 from dual union all
select 39196,'1976-M24','1976-01-07 18:54:00','suborbital rocket','success',1506,'T1-8059','','Instrumented Sphere 2','','Aeron',1821,'',100 from dual union all
select 39197,'1976-M25','1976-06-02 00:00:00','suborbital rocket','success',1281,'','EXAMETNET 35-93','','','Meteo',350,'',75 from dual union all
select 39198,'1976-M26','1976-01-07 19:58:00','suborbital rocket','success',1506,'T1-8060','','Instrumented Sphere 3','','Aeron',1821,'',100 from dual union all
select 39199,'1976-M27','1976-04-22 15:30:00','suborbital rocket','unknown',1506,'T1-8279','','Instrumented Sphere 5','','Aeron',1821,'',100 from dual union all
select 392,'1944-S357','1944-11-19 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3920,'1956-A247','1956-06-19 00:00:00','atmospheric rocket','success',563,'S17','HTV','','','Heat transfer',1895,'',10 from dual union all
select 39200,'1976-S01','1976-01-01 00:00:00','military missile','success',6,'','LKI','','','Test',183,'',300 from dual union all
select 39201,'1976-S02','1976-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',139,'ZUB',150 from dual union all
select 39202,'1976-S03','1976-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',139,'ZUB',150 from dual union all
select 39203,'1976-S04','1976-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',139,'ZUB',150 from dual union all
select 39204,'1976-S05','1976-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',139,'ZUB',150 from dual union all
select 39205,'1976-S06','1976-01-15 10:00:00','military missile','unknown',917,'AO-160','P-324 76 Winter 01','','','OT',433,'',180 from dual union all
select 39206,'1976-S07','1976-01-15 11:00:00','military missile','unknown',917,'AO-161','P-325 76 Winter 02','','','OT',433,'',180 from dual union all
select 39207,'1976-S08','1976-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 39208,'1976-S09','1976-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 39209,'1976-S10','1976-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 3921,'1956-A248','1956-07-10 00:00:00','atmospheric rocket','success',563,'HAT-1','HTV HAT','','','Heat transfer',1895,'',10 from dual union all
select 39210,'1976-S100','1976-02-13 09:45:00','suborbital rocket','success',1073,'S-310JA-1','VLF/HF','','','Ionos',1458,'',216 from dual union all
select 39211,'1976-S101','1976-02-14 22:00:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 624',500 from dual union all
select 39212,'1976-S102','1976-02-18 00:00:00','military missile','success',1004,'1T','15A14 UBP-3 SLI','GCh 15F141','','OT',482,'',1000 from dual union all
select 39213,'1976-S103','1976-02-18 07:05:59','suborbital rocket','success',867,'NASA 18.1001UE','','','','Plasma',1047,'',237 from dual union all
select 39214,'1976-S104','1976-02-18 17:40:00','suborbital rocket','success',228,'NASA 21.30US-IS','OSO-I Cal','','','Solar UV',1873,'',282 from dual union all
select 39215,'1976-S105','1976-02-20 03:20:00','military missile','success',796,'','Mk 11 RV','FOT GT126M','GLORY TRIP 126M','OT',1700,'',1300 from dual union all
select 39216,'1976-S106','1976-02-20 00:00:00','suborbital rocket','success',487,'F.01','','','','Test',1836,'',117 from dual union all
select 39217,'1976-S107','1976-02-21 19:42:32','suborbital rocket','success',837,'SSC S18-2','SSC S18/2 D-Layer','','','Ionos',408,'',105 from dual union all
select 39218,'1976-S108','1976-02-22 01:18:00','suborbital rocket','success',906,'GL A10.504-1','ICECAP 76','','','Aeron',1047,'',248 from dual union all
select 39219,'1976-S109','1976-02-25 05:15:00','military missile','success',796,'','Mk 11 RV','FOT GT124M','GLORY TRIP 124M','OT',1699,'',1300 from dual union all
select 3922,'1956-A249','1956-07-26 00:00:00','atmospheric rocket','success',563,'W-2','HTV 107A','','','Heat transfer',1895,'',10 from dual union all
select 39220,'1976-S11','1976-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 39221,'1976-S110','1976-02-24 19:20:00','suborbital rocket','success',37,'GL A04.305-1','XUV','','','Solar X/UV',1895,'',221 from dual union all
select 39222,'1976-S111','1976-02-24 02:30:00','suborbital rocket','success',722,'2013','','','','Aeron?',1832,'',107 from dual union all
select 39223,'1976-S112','1976-02-28 02:00:00','military missile','success',795,'','ANT-1 RV x 4','ABRES ANT-1','','Reentry Test',1698,'',1300 from dual union all
select 39224,'1976-S113','1976-02-28 00:55:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',1010,'',185 from dual union all
select 39225,'1976-S114','1976-02-28 05:46:40','suborbital rocket','success',1162,'EX630.42-1','EXCEDE SWIR','','','Active Plasma',1047,'',100 from dual union all
select 39226,'1976-S115','1976-02-28 12:12:50','suborbital rocket','success',867,'NASA 18.164UE','UTD FERRET II/18.1003UE','','','Auroral/Ionos',1047,'',281 from dual union all
select 39227,'1976-S116','1976-02-28 14:22:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/CR/Plasma',1010,'',176 from dual union all
select 39228,'1976-S117','1976-02-28 21:01:00','suborbital rocket','success',740,'MR-12','E Field','','','Auroral',1010,'',190 from dual union all
select 39229,'1976-S118','1976-02-29 01:15:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',1010,'',180 from dual union all
select 3923,'1956-A25','1956-02-28 22:27:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 39230,'1976-S119','1976-03-01 10:45:45','suborbital rocket','success',867,'NASA 18.1002UE','','','','Plasma',1047,'',246 from dual union all
select 39231,'1976-S12','1976-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 39232,'1976-S120','1976-03-02 23:49:30','suborbital rocket','success',837,'SSC S21-1','SSC S21/1 TRIGGER','','','Ionos',417,'',160 from dual union all
select 39233,'1976-S121','1976-03-03 00:10:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/CR/Plas',1010,'',164 from dual union all
select 39234,'1976-S122','1976-03-03 17:44:00','suborbital rocket','success',906,'GL A10.507-1','ICECAP 76 Sphere/Ph.','Sphere (10in)','','Aeron',1047,'',210 from dual union all
select 39235,'1976-S123','1976-03-04 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT52GB','GLORY TRIP 52GB','OT',1711,'',1300 from dual union all
select 39236,'1976-S124','1976-03-04 00:01:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/CR/Plas',1010,'',175 from dual union all
select 39237,'1976-S125','1976-03-05 11:36:00','suborbital rocket','success',37,'NASA 13.128UC','Comet West','','','UV Astron',1895,'',215 from dual union all
select 39238,'1976-S126','1976-03-05 11:40:00','suborbital rocket','success',41,'NASA 26.50UC','JHU UVS Comet West','','','UV Astron',1895,'',209 from dual union all
select 39239,'1976-S127','1976-03-06 18:48:00','suborbital rocket','unknown',740,'MR-12','','','','Auroral',1010,'',180 from dual union all
select 3924,'1956-A250','1956-08-22 00:00:00','atmospheric rocket','success',563,'HAT-2','HTV HAT','','','Heat transfer',1895,'',10 from dual union all
select 39240,'1976-S128','1976-03-06 21:00:16','suborbital rocket','success',867,'SSC S17-2','SSC S17/2 M/Dtr','','','Ionos',417,'',210 from dual union all
select 39241,'1976-S129','1976-03-05 23:45:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/CR/Plas',1010,'',176 from dual union all
select 39242,'1976-S13','1976-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 39243,'1976-S130','1976-03-06 23:30:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/CR/Plas',1010,'',142 from dual union all
select 39244,'1976-S131','1976-03-08 22:56:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',1010,'',150 from dual union all
select 39245,'1976-S132','1976-03-09 00:00:00','military missile','success',135,'SAMSO-37','ABRES','','','Reentry Test',1821,'',200 from dual union all
select 39246,'1976-S133','1976-03-10 10:58:00','suborbital rocket','success',37,'NASA 13.129GC','Comet West','','','UV Astron',1895,'',189 from dual union all
select 39247,'1976-S134','1976-03-12 00:00:00','military missile','success',744,'','15A15 LKI-31','','','Test',530,'',1000 from dual union all
select 39248,'1976-S135','1976-03-14 00:00:00','military missile','success',799,'PVM-13','Mk 12 RV x 3','R&D','PVM-13','Test',1712,'',1300 from dual;
insert into LAUNCH
select 39249,'1976-S136','1976-03-17 20:35:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/CR/Plasma',1010,'',150 from dual union all
select 3925,'1956-A251','1956-08-31 00:00:00','atmospheric rocket','success',563,'','HTV 107A','','','Heat transfer',1895,'',10 from dual union all
select 39250,'1976-S137','1976-03-18 00:28:00','suborbital rocket','unknown',740,'MR-12','','','','CR/Plasma/Fields',1010,'',150 from dual union all
select 39251,'1976-S138','1976-03-18 08:08:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/CR/Plasma',1010,'',150 from dual union all
select 39252,'1976-S139','1976-03-19 17:15:00','suborbital rocket','success',839,'NASA 10.425GM','','','','Aeronomy',1821,'',90 from dual union all
select 39253,'1976-S14','1976-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 39254,'1976-S140','1976-03-22 06:43:40','suborbital rocket','success',38,'NASA 13.116UE','Auroral UV/particles','','','Plasma',428,'',182 from dual union all
select 39255,'1976-S141','1976-03-23 06:00:00','suborbital rocket','success',41,'NASA 26.55GG','GSFC NGC 4151','','','UV Astron',1895,'',216 from dual union all
select 39256,'1976-S142','1976-03-24 00:40:00','suborbital rocket','success',290,'ISRO 5.30','ISRO 5.30','','','Ionos/Fields',1467,'',142 from dual union all
select 39257,'1976-S143','1976-03-26 01:35:00','suborbital rocket','success',290,'ISRO 5.31','ISRO 5.31','','','Aeron/Ionos',1467,'',139 from dual union all
select 39258,'1976-S144','1976-03-26 02:58:26','suborbital rocket','success',37,'NASA 13.077UA','JHU Aurora 11','','','Auroral',428,'',175 from dual union all
select 39259,'1976-S145','1976-03-26 13:40:00','suborbital rocket','success',846,'DNA IC607.11-2C','','','','Auroral',1047,'',150 from dual union all
select 3926,'1956-A252','1956-09-06 00:00:00','atmospheric rocket','success',563,'HAT-83','HTV 1366','','','Heat transfer',1895,'',10 from dual union all
select 39260,'1976-S146','1976-03-26 21:00:00','suborbital rocket','success',41,'NASA 26.017AS','','','','Solar X/UV',1895,'',203 from dual union all
select 39261,'1976-S147','1976-03-27 00:00:00','military missile','success',135,'SAMSO-38','ABRES','','','Reentry Test',1821,'',200 from dual union all
select 39262,'1976-S148','1976-03-27 06:43:00','suborbital rocket','success',867,'NASA 18.176GE','','','','Plasma',1047,'',230 from dual union all
select 39263,'1976-S149','1976-03-27 12:50:00','suborbital rocket','success',867,'NASA 18.194GE','','','','Plasma',1047,'',230 from dual union all
select 39264,'1976-S15','1976-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 39265,'1976-S150','1976-03-28 13:00:00','suborbital rocket','success',867,'NASA 18.195GE','','','','Plasma',1047,'',230 from dual union all
select 39266,'1976-S151','1976-03-29 16:00:00','military missile','success',931,'A3T','DASO','','','OT',421,'SSBN 618',1000 from dual union all
select 39267,'1976-S152','1976-03-30 12:30:00','suborbital rocket','success',867,'NASA 18.177GE','','','','Plasma',1047,'',230 from dual union all
select 39268,'1976-S153','1976-03-30 12:56:00','suborbital rocket','success',846,'NSF','Ba','','','Aeron/Fields',1047,'',150 from dual union all
select 39269,'1976-S154','1976-03-30 16:50:00','suborbital rocket','success',133,'NASA 25.13CS-NT','LPARL Solar Corona','','','Solar X/Test',1895,'',230 from dual union all
select 3927,'1956-A253','1956-09-11 00:00:00','atmospheric rocket','success',563,'W-4','HTV 107A','','','Heat transfer',1895,'',10 from dual union all
select 39270,'1976-S155','1976-03-30 20:27:00','suborbital rocket','failure',117,'NASA 24.01IE','DLR K-AR-75','Porcupine 1','','Magnetospheric',407,'',360 from dual union all
select 39271,'1976-S156','1976-03-31 06:39:00','suborbital rocket','success',228,'NASA 21.03UI','','','','Ionos/Auroral',428,'',231 from dual union all
select 39272,'1976-S157','1976-04-01 08:05:20','suborbital rocket','success',1162,'IC630.02-1A','ICECAP 76 HIRIS','','','Auroral',1047,'',130 from dual union all
select 39273,'1976-S158','1976-04-01 09:28:20','suborbital rocket','success',543,'ARPA WS650.27-1','Ballistics','','','Test',1047,'',150 from dual union all
select 39274,'1976-S159','1976-04-02 21:18:00','military missile','success',135,'Athena D','MSV-1','','','Reentry Test',1821,'',200 from dual union all
select 39275,'1976-S16','1976-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 39276,'1976-S160','1976-04-07 00:00:00','military missile','success',755,'','','','','OT',307,'S613',600 from dual union all
select 39277,'1976-S161','1976-04-18 23:55:00','suborbital rocket','success',290,'ISRO 5.32','ISRO 5.32','','','Aeron/Ionos',1467,'',152 from dual union all
select 39278,'1976-S162','1976-04-20 20:08:00','military missile','success',939,'','FOT-18','','','OT',421,'SSBN 656',500 from dual union all
select 39279,'1976-S163','1976-04-20 20:09:00','military missile','success',939,'','FOT-18','','','OT',421,'SSBN 656',500 from dual union all
select 3928,'1956-A254','1956-09-27 00:00:00','atmospheric rocket','success',563,'W-5','HTV 107A','','','Heat transfer',1895,'',10 from dual union all
select 39280,'1976-S164','1976-04-21 03:15:00','suborbital rocket','success',41,'NASA 26.049DH','NRL XR','','','XR Astron',1895,'',202 from dual union all
select 39281,'1976-S165','1976-04-23 09:46:00','military missile','success',939,'','FOT-19','','','OT',421,'SSBN 644',500 from dual union all
select 39282,'1976-S166','1976-04-23 09:53:00','military missile','success',939,'','FOT-19','','','OT',421,'SSBN 644',500 from dual union all
select 39283,'1976-S167','1976-04-25 00:00:00','military missile','success',135,'SAMSO-39','ABRES','','','Reentry Test',1821,'',200 from dual union all
select 39284,'1976-S168','1976-04-28 20:35:00','suborbital rocket','success',338,'WRE Co1003','','','','Aeron',1832,'',200 from dual union all
select 39285,'1976-S169','1976-05-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 39286,'1976-S17','1976-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 39287,'1976-S170','1976-05-04 14:02:00','military missile','unknown',917,'AO-164','P-328 76 Spring 01','','','OT',433,'',180 from dual union all
select 39288,'1976-S171','1976-05-04 14:40:00','military missile','unknown',917,'AO-165','P-329 76 Spring 02','','','OT',433,'',180 from dual union all
select 39289,'1976-S172','1976-05-18 09:10:00','military missile','unknown',917,'AO-166','P-330 76 Spring 03','','','OT',433,'',180 from dual union all
select 3929,'1956-A255','1956-10-16 00:00:00','atmospheric rocket','success',563,'TM-41','HTV Douglas','','','Heat transfer',1895,'',10 from dual union all
select 39290,'1976-S173','1976-05-18 09:35:00','military missile','unknown',917,'AO-167','P-331 76 Spring 04','','','OT',433,'',180 from dual union all
select 39291,'1976-S174','1976-04-30 04:35:00','suborbital rocket','success',906,'GL A10.001-2','Auroral ions','','','Aeron',428,'',218 from dual union all
select 39292,'1976-S175','1976-05-05 04:43:00','suborbital rocket','success',1189,'INTA MSCL-7608','Winter Anom','','','Ionos',396,'',95 from dual union all
select 39293,'1976-S176','1976-05-06 11:44:00','suborbital rocket','success',920,'P186H','Mass spectrometer','','','Ionospheric',1447,'',149 from dual union all
select 39294,'1976-S177','1976-05-06 12:21:00','suborbital rocket','success',920,'P115H','Faraday rotation','','','Ionospheric',1447,'',137 from dual union all
select 39295,'1976-S178','1976-05-11 08:55:00','suborbital rocket','success',41,'NASA 26.038GH','CIT Cyg X-1','','','XR Astron',1895,'',175 from dual union all
select 39296,'1976-S179','1976-05-11 17:08:00','suborbital rocket','success',40,'NASA 26.045CS','','','','Solar',1895,'',248 from dual union all
select 39297,'1976-S18','1976-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 39298,'1976-S180','1976-05-12 10:50:00','suborbital rocket','success',1213,'SL1115','MSSL XR16-Bir XR1','','','XR Astronomy',1839,'',231 from dual union all
select 39299,'1976-S181','1976-05-13 02:51:00','suborbital rocket','success',132,'NASA 23.06UE','','','','Plasma',1895,'',90 from dual union all
select 393,'1944-S358','1944-11-19 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3930,'1956-A256','1956-11-19 00:00:00','atmospheric rocket','success',563,'','HTV 107A','','','Heat transfer',1895,'',10 from dual union all
select 39300,'1976-S182','1976-05-17 00:00:00','military missile','success',744,'','15A15 LKI-32','','','Test',541,'',1000 from dual union all
select 39301,'1976-S183','1976-05-17 00:00:00','military missile','success',1004,'4T','15A14 UBP-4 SLI','GCh 15F142','','OT',490,'',1000 from dual union all
select 39302,'1976-S184','1976-05-17 16:00:00','suborbital rocket','success',228,'NASA 21.33NP','SPAR 2','','','Micrograv',1873,'',188 from dual union all
select 39303,'1976-S185','1976-05-18 17:00:00','suborbital rocket','success',30,'GL A03.410-1','Solar XUV/UV','','','Solar UV/X',1869,'',190 from dual union all
select 39304,'1976-S186','1976-05-23 09:30:00','suborbital rocket','success',228,'NASA 21.48NP','SFHe Experiment','','','Tech',855,'',196 from dual union all
select 39305,'1976-S187','1976-05-25 22:00:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 620',500 from dual union all
select 39306,'1976-S188','1976-05-27 00:30:00','suborbital rocket','success',1212,'SL1211/1271','RAE/BAE Test','Li release','','Test/Ionos Li',1839,'',254 from dual union all
select 39307,'1976-S189','1976-05-29 09:49:30','suborbital rocket','success',1347,'Sandia','LEINAX IV','','','XR Astron',855,'',271 from dual union all
select 39308,'1976-S19','1976-03-30 00:00:00','test rocket','unknown',1497,'6','K65UP No. 6','','','Ionos/Solar?',621,'',1500 from dual union all
select 39309,'1976-S190','1976-06-02 15:19:00','military missile','unknown',917,'AO-168','P-332 76 Spring 05','','','OT',433,'',180 from dual union all
select 3931,'1956-A257','1956-11-28 00:00:00','atmospheric rocket','success',563,'','HTV 107A','','','Heat transfer',1895,'',10 from dual union all
select 39310,'1976-S191','1976-06-02 15:45:00','military missile','unknown',917,'AO-169','P-333 76 Spring 06','','','OT',433,'',180 from dual union all
select 39311,'1976-S192','1976-06-29 13:08:00','military missile','success',917,'AO-170','P-334 76 Spring 07','','','OT',437,'',180 from dual union all
select 39312,'1976-S193','1976-06-29 14:56:00','military missile','success',917,'AO-171','P-335 76 Spring 08','','','OT WS-237+6+CC',437,'',180 from dual union all
select 39313,'1976-S194','1976-06-01 00:00:00','military missile','success',1105,'S-1E-2606','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 39314,'1976-S195','1976-06-04 04:23:00','suborbital rocket','success',920,'P187H','E field/VLF','','','Ionospheric',1447,'',143 from dual union all
select 39315,'1976-S196','1976-06-04 04:39:00','suborbital rocket','success',920,'P144H','VLF','','','Ionospheric',1447,'',132 from dual union all
select 39316,'1976-S197','1976-06-06 08:44:00','suborbital rocket','success',1347,'Sandia','','','','XR Astron',855,'',430 from dual union all
select 39317,'1976-S198','1976-06-09 07:19:00','military missile','success',796,'','Mk 11 RV','FOT GT127M','GLORY TRIP 127M','OT',1702,'',1300 from dual union all
select 39318,'1976-S199','1976-06-09 00:00:00','military missile','success',1004,'3T','15A14 UBP-5 SLI','GCh 15F143U2','','OT',492,'',1000 from dual union all
select 39319,'1976-S20','1976-12-29 00:00:00','test rocket','unknown',1497,'8','K65UP No. 8','','','Ionos/Solar?',621,'',1500 from dual union all
select 3932,'1956-A258','1956-01-13 00:00:00','military missile','unknown',339,'1669','Corporal EU','','','Test',1895,'',40 from dual union all
select 39320,'1976-S200','1976-06-10 17:40:00','suborbital rocket','success',1213,'SL1212','MSSL XR17 XRCS','','','XR Astronomy',1839,'',280 from dual union all
select 39321,'1976-S201','1976-06-11 18:58:00','suborbital rocket','success',850,'NASA 32.004WT','T2-7036','','','Test',1821,'',130 from dual union all
select 39322,'1976-S202','1976-06-15 00:00:00','military missile','success',744,'','15A15 LKI-33','','','Test',524,'',1000 from dual union all
select 39323,'1976-S203','1976-06-17 05:30:00','suborbital rocket','success',44,'NASA 17.017UH','SAO XRT 3','','','XR Astron',1895,'',232 from dual union all
select 39324,'1976-S204','1976-06-17 07:47:00','suborbital rocket','success',1213,'SL1501','Bir XR2 coded mask','','','XR Astronomy',1839,'',256 from dual union all
select 39325,'1976-S205','1976-06-21 07:01:00','military missile','success',799,'','Mk 12 RV x 3','OT GT33GM','GLORY TRIP 33GM','OT',1703,'',1300 from dual union all
select 39326,'1976-S206','1976-06-23 06:00:00','military missile','success',796,'','Mk 11 RV','FOT GT128M','GLORY TRIP 128M','OT',1700,'',1300 from dual union all
select 39327,'1976-S207','1976-06-22 20:16:00','military missile','success',939,'','FOT-20','','','OT',421,'SSBN 632',500 from dual union all
select 39328,'1976-S208','1976-06-22 20:17:00','military missile','success',939,'','FOT-20','','','OT',421,'SSBN 632',500 from dual union all
select 39329,'1976-S209','1976-06-24 23:40:00','suborbital rocket','success',1070,'S-210JA-20','VLF/HF/ne','','','Auroral',1458,'',118 from dual union all
select 3933,'1956-A259','1956-01-17 00:00:00','military missile','unknown',339,'1676','Corporal TA-60','','','Test',1895,'',40 from dual union all
select 39330,'1976-S21','1976-01-01 00:00:00','military missile','success',1514,'','','','','OT',989,'',1000 from dual union all
select 39331,'1976-S210','1976-06-26 00:00:00','military missile','success',744,'','15A15 LKI-34','','','Test',530,'',1000 from dual union all
select 39332,'1976-S211','1976-06-26 08:58:00','military missile','success',939,'','FOT-21','','','OT',421,'SSBN 640',500 from dual union all
select 39333,'1976-S212','1976-06-26 09:06:00','military missile','success',939,'','FOT-21','','','OT',421,'SSBN 640',500 from dual union all
select 39334,'1976-S213','1976-06-27 00:00:00','military missile','success',277,'','','','','Tech',871,'',200 from dual union all
select 39335,'1976-S214','1976-06-27 08:58:00','suborbital rocket','success',220,'Astro 6-2','DLR H-BB4B-76','INTA BB4-BAC-7602','','EUV Astronomy',396,'',700 from dual union all
select 39336,'1976-S215','1976-06-28 02:40:53','military missile','success',1420,'B-17','Mk 6 RV','ITF-1','','Target',1658,'',1300 from dual union all
select 39337,'1976-S216','1976-06-29 03:00:00','military missile','success',744,'67026-527 (27L)','15A15 LKI-35','Experimental','','Test',542,'',1000 from dual union all
select 39338,'1976-S217','1976-06-29 00:00:00','military missile','success',1004,'1-22-118T','15A14 UBP-6 ZP','GCh 15F143U1','','OT',486,'',1000 from dual;
insert into LAUNCH
select 39339,'1976-S218','1976-06-29 18:20:00','suborbital rocket','success',867,'NASA 18.181GA','','','','Aeronomy',1895,'',270 from dual union all
select 3934,'1956-A26','1956-03-04 13:58:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 39340,'1976-S219','1976-06-29 18:20:00','suborbital rocket','success',37,'NASA 13.130GS','Solar Constant 1','','','Solar',1895,'',225 from dual union all
select 39341,'1976-S22','1976-04-28 00:00:00','military missile','success',1322,'64536-51-859-33L','15Zh42 UBP-5','','','Test',477,'',1000 from dual union all
select 39342,'1976-S220','1976-06-30 00:00:00','suborbital rocket','success',677,'','','','','Aeron?',1832,'',120 from dual union all
select 39343,'1976-S221','1976-06-30 09:19:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT54GM','GLORY TRIP 54GM','OT',1705,'',1300 from dual union all
select 39344,'1976-S222','1976-07-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 39345,'1976-S223','1976-07-01 00:00:00','military missile','success',1548,'','','','','Test',1626,'PLBR',620 from dual union all
select 39346,'1976-S224','1976-07-05 11:44:00','suborbital rocket','success',920,'P188H','Atomic oxygen','','','Atmospheric',1447,'',135 from dual union all
select 39347,'1976-S225','1976-07-13 17:48:00','suborbital rocket','success',38,'NASA 12.032GT','Aerobee Reuse Test','','','Test',1870,'',183 from dual union all
select 39348,'1976-S226','1976-07-15 00:00:00','military missile','success',799,'STM-11W','Mk 12 RV x 3','STM-11W','STM-11W','Test',1707,'',1300 from dual union all
select 39349,'1976-S227','1976-07-17 06:10:00','suborbital rocket','success',1213,'SL1402 (DLR H-GR-77)','MSSL XR18 XRCS-MPE','INTA SKY.7-7601','','XR Astronomy',395,'',256 from dual union all
select 3935,'1956-A260','1956-01-19 00:00:00','military missile','unknown',339,'1672','Corporal EU','','','Test',1895,'',40 from dual union all
select 39350,'1976-S228','1976-07-22 00:00:00','suborbital rocket','success',765,'Sandia 497-006','','','','Test',1489,'',146 from dual union all
select 39351,'1976-S23','1976-01-31 00:00:00','military missile','success',1450,'','15A20S LKI-3','GCh 15F202P2 (15F204x3)','','OT',504,'',1000 from dual union all
select 39352,'1976-S230','1976-07-26 00:23:00','suborbital rocket','success',1070,'S-210JA-21','VLF/HF/ne/ISIS-2','','','Auroral',1458,'',116 from dual union all
select 39353,'1976-S231','1976-07-26 15:59:00','suborbital rocket','success',38,'NASA 12.033GT','Aerobee Reuse Test','','','Test',1895,'',170 from dual union all
select 39354,'1976-S232','1976-07-27 04:45:00','suborbital rocket','success',290,'ISRO 5.33','ISRO 5.33','','','Test',1467,'',97 from dual union all
select 39355,'1976-S233','1976-07-29 00:00:00','suborbital rocket','success',547,'HAVE JEEP IV-1','','','','Test',873,'',500 from dual union all
select 39356,'1976-S234','1976-08-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39357,'1976-S235','1976-08-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39358,'1976-S236','1976-08-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39359,'1976-S237','1976-08-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 3936,'1956-A261','1956-01-20 00:00:00','military missile','unknown',339,'1597','Corporal TA-45','','','Test',1895,'',40 from dual union all
select 39360,'1976-S238','1976-08-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39361,'1976-S239','1976-08-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39362,'1976-S24','1976-04-20 00:00:00','military missile','success',1450,'','15A20S LKI-4','GCh 15F202P2 (15F204x3)','','OT',504,'',1000 from dual union all
select 39363,'1976-S240','1976-08-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39364,'1976-S241','1976-08-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39365,'1976-S242','1976-08-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39366,'1976-S243','1976-08-03 19:16:00','suborbital rocket','success',44,'GL A35.191-4','ELE (Earth Limb Expt)','','','IR Astron',1895,'',257 from dual union all
select 39367,'1976-S244','1976-08-04 17:39:00','suborbital rocket','success',41,'NASA 26.043US','','','','Solar UV',1895,'',222 from dual union all
select 39368,'1976-S245','1976-08-11 15:45:00','suborbital rocket','success',132,'NASA 23.07UE','','','','Plasma',1895,'',90 from dual union all
select 39369,'1976-S246','1976-08-11 19:12:00','suborbital rocket','success',1337,'NASA 12.36GT','','','','Test',1821,'',612 from dual union all
select 3937,'1956-A262','1956-01-20 00:00:00','military missile','unknown',339,'1679','Corporal TA-44','','','Test',1895,'',40 from dual union all
select 39370,'1976-S247','1976-08-12 00:00:00','military missile','success',744,'','15A15 K-5','','','OT',530,'',1000 from dual union all
select 39371,'1976-S248','1976-08-12 15:54:00','suborbital rocket','success',867,'NASA 18.168UE','NASA 18.1006UE','','','Plasma/Aeron',1821,'',148 from dual union all
select 39372,'1976-S249','1976-08-16 23:54:00','suborbital rocket','success',1070,'S-210JA-24','E/B/ne','','','Auroral',1458,'',118 from dual union all
select 39373,'1976-S25','1976-05-20 00:00:00','military missile','success',1450,'','15A20S LKI-5','GCh 15B126?','','OT',504,'',1000 from dual union all
select 39374,'1976-S250','1976-08-17 10:15:00','suborbital rocket','success',1070,'S-210-11','','','','Aeron/Ionos',839,'',112 from dual union all
select 39375,'1976-S251','1976-08-18 16:59:00','suborbital rocket','success',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',160 from dual union all
select 39376,'1976-S252','1976-08-18 20:40:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',160 from dual union all
select 39377,'1976-S253','1976-08-19 00:00:00','military missile','success',795,'','SAMAST','ABRES','SAMAST III/MINT II','Reentry Test',1701,'',1300 from dual union all
select 39378,'1976-S254','1976-08-19 00:43:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',160 from dual union all
select 39379,'1976-S255','1976-08-19 05:15:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',160 from dual union all
select 3938,'1956-A263','1956-02-01 00:00:00','military missile','unknown',339,'1641','Corporal TA-47','','','Test',1895,'',40 from dual union all
select 39380,'1976-S256','1976-08-19 09:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Plasma',636,'',160 from dual union all
select 39381,'1976-S257','1976-08-19 10:45:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/Plasma',636,'',160 from dual union all
select 39382,'1976-S258','1976-08-19 16:52:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',160 from dual union all
select 39383,'1976-S259','1976-08-19 20:35:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',160 from dual union all
select 39384,'1976-S26','1976-05-20 00:00:00','military missile','success',1450,'','15A20S LKI-6','GCh 15B126?','','OT',536,'',1000 from dual union all
select 39385,'1976-S260','1976-08-20 00:42:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',636,'',160 from dual union all
select 39386,'1976-S261','1976-08-20 02:15:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/CR/Plasma',636,'',160 from dual union all
select 39387,'1976-S262','1976-08-20 05:15:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/CR/Plasma',636,'',160 from dual union all
select 39388,'1976-S263','1976-08-20 09:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/CR/Plasma',636,'',160 from dual union all
select 39389,'1976-S264','1976-08-20 13:45:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/CR/Plasma',636,'',160 from dual union all
select 3939,'1956-A264','1956-02-01 00:00:00','military missile','unknown',339,'1678','Corporal TA-46','','','Test',1895,'',40 from dual union all
select 39390,'1976-S265','1976-08-20 22:48:00','military missile','success',939,'','FOT-22','','','OT',421,'SSBN 636',500 from dual union all
select 39391,'1976-S266','1976-08-20 22:50:00','military missile','success',939,'','FOT-22','','','OT',421,'SSBN 636',500 from dual union all
select 39392,'1976-S267','1976-08-21 00:47:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/CR/Plasma',636,'',160 from dual union all
select 39393,'1976-S268','1976-08-21 12:10:00','suborbital rocket','success',1073,'S-310-3','','','','Ionos',839,'',195 from dual union all
select 39394,'1976-S269','1976-08-22 17:55:00','suborbital rocket','success',867,'NASA 18.209IE','CUSP II','','','Plasma/Auroral',1438,'',210 from dual union all
select 39395,'1976-S27','1976-08-26 00:00:00','military missile','success',1450,'','15A20S LKI-7','GCh 15F202P2 (15F204x3)','','OT',504,'',1000 from dual union all
select 39396,'1976-S270','1976-08-26 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT129M','GLORY TRIP 129M','OT',1702,'',1300 from dual union all
select 39397,'1976-S271','1976-08-27 11:08:00','suborbital rocket','success',867,'NASA 18.210IE','','','','Plasma',1438,'',270 from dual union all
select 39398,'1976-S272','1976-08-28 09:42:00','military missile','success',939,'','FOT-23','','','OT',421,'SSBN 641',500 from dual union all
select 39399,'1976-S273','1976-08-28 09:43:00','military missile','success',939,'','FOT-23','','','OT',421,'SSBN 641',500 from dual union all
select 394,'1944-S359','1944-11-21 00:00:00','military missile','success',1470,'','London 226','','','Weapon',1627,'',90 from dual union all
select 3940,'1956-A265','1956-02-01 00:00:00','military missile','unknown',339,'1347','Corporal Chem','','','Test',1895,'',40 from dual union all
select 39400,'1976-S274','1976-08-30 19:55:00','suborbital rocket','success',668,'K-9M-57','','','','Ionos/Fields',839,'',288 from dual union all
select 39401,'1976-S275','1976-09-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39402,'1976-S276','1976-09-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39403,'1976-S277','1976-09-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39404,'1976-S278','1976-09-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39405,'1976-S279','1976-09-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39406,'1976-S28','1976-10-21 00:00:00','military missile','success',1450,'','15A20S LKI-8','GCh 15F202P2 (15F204x3)','','OT',504,'',1000 from dual union all
select 39407,'1976-S280','1976-09-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39408,'1976-S281','1976-09-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39409,'1976-S282','1976-09-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 3941,'1956-A266','1956-02-03 00:00:00','military missile','unknown',339,'1632','Corporal TA-61','','','Test',1895,'',40 from dual union all
select 39410,'1976-S283','1976-08-31 03:40:00','suborbital rocket','success',846,'GL A11.408-1','TREP SAMSO 1202-B','Sphere','','Aeron',871,'',188 from dual union all
select 39411,'1976-S284','1976-09-01 00:00:00','suborbital rocket','success',1070,'S-210JA-25','E/B/ne','','','Auroral?',1458,'',125 from dual union all
select 39412,'1976-S285','1976-09-02 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 610',1000 from dual union all
select 39413,'1976-S286','1976-09-02 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 610',1000 from dual union all
select 39414,'1976-S287','1976-09-03 00:00:00','military missile','success',1004,'51026-32-517T','15A14 UBP-7 ZP','GCh 15F141','','OT',496,'',1000 from dual union all
select 39415,'1976-S288','1976-09-07 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 609',1000 from dual union all
select 39416,'1976-S289','1976-09-07 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 609',1000 from dual union all
select 39417,'1976-S29','1976-10-28 02:59:00','military missile','success',1450,'64146-136(6L)','15A20-T-OCh LKI-1','GCh 15F202P2 (15F204x3)','','OT',534,'',1000 from dual union all
select 39418,'1976-S290','1976-09-13 04:31:00','suborbital rocket','success',1070,'S-210JA-23','NO/Ne/ISIS-2','','','Auroral?',1458,'',120 from dual union all
select 39419,'1976-S291','1976-09-16 02:00:00','suborbital rocket','success',668,'K-9M-55','','','','Ionos/Fields',839,'',359 from dual union all
select 3942,'1956-A267','1956-02-06 00:00:00','military missile','unknown',339,'CD-5','Corporal Chem','','','Test',1895,'',40 from dual union all
select 39420,'1976-S292','1976-09-16 17:00:00','suborbital rocket','success',38,'NASA 13.134CS','','','','Solar X',1895,'',177 from dual union all
select 39421,'1976-S293','1976-09-18 10:00:00','suborbital rocket','success',668,'K-9M-56','','','','Ionos/Fields',839,'',300 from dual union all
select 39422,'1976-S294','1976-09-18 17:25:00','military missile','success',931,'A3T','DASO','','','OT',421,'SSBN 611',1000 from dual union all
select 39423,'1976-S295','1976-09-21 00:00:00','suborbital rocket','success',1219,'XV-02','','','','Test',330,'',543 from dual union all
select 39424,'1976-S296','1976-09-21 12:52:00','suborbital rocket','success',867,'NASA 18.178GE','','','','Plasma',1047,'',253 from dual union all
select 39425,'1976-S297','1976-09-23 11:37:10','suborbital rocket','success',867,'NASA 18.179GE','','','','Plasma',1047,'',235 from dual union all
select 39426,'1976-S298','1976-09-24 01:30:00','suborbital rocket','success',1298,'TT-210-3F','','','','Test',1476,'',100 from dual union all
select 39427,'1976-S299','1976-10-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39428,'1976-S30','1976-11-26 00:00:00','military missile','success',1450,'','15A20S LKI-9','GCh 15F202P2 (15F204x3)','','OT',504,'',1000 from dual;
insert into LAUNCH
select 39429,'1976-S300','1976-10-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 3943,'1956-A268','1956-02-10 00:00:00','military missile','unknown',339,'1609M','Corporal EU','','','Test',1895,'',40 from dual union all
select 39430,'1976-S301','1976-10-01 00:00:00','suborbital rocket','success',740,'MR-12','','','','Aeron',128,'VISE',150 from dual union all
select 39431,'1976-S302','1976-10-01 00:00:00','suborbital rocket','failure',1028,'','3-t engine test','','','Test',1361,'',100 from dual union all
select 39432,'1976-S303','1976-10-08 17:44:00','suborbital rocket','success',290,'ISRO 5.34','ISRO 5.34','','','Plasma',1467,'',120 from dual union all
select 39433,'1976-S304','1976-10-14 10:50:00','ballistic missile test','success',1497,'7','Vertikal''-4','VZA','','Ionos/Solar?',621,'',1512 from dual union all
select 39434,'1976-S305','1976-10-22 15:00:00','suborbital rocket','success',228,'NASA 21.36DS','','','','Solar',1873,'',239 from dual union all
select 39435,'1976-S306','1976-10-26 00:00:00','military missile','success',744,'','15A15 LKI-37','','','Test',541,'',1000 from dual union all
select 39436,'1976-S307','1976-10-27 03:00:00','military missile','success',796,'','ERCS','ERCS GIANT MOON 8','GIANT MOON 8','Comms',1700,'',1300 from dual union all
select 39437,'1976-S308','1976-10-26 18:06:00','military missile','success',917,'FOT-49','P-336 76 Cape 01','','','OT',231,'',180 from dual union all
select 39438,'1976-S309','1976-10-26 19:12:00','military missile','success',917,'FOT-50','P-337 76 Cape 02','','','OT',231,'',180 from dual union all
select 39439,'1976-S31','1976-12-08 00:00:00','military missile','success',1450,'','15A20S LKI-10','GCh 15F202P2 (15F204x3)','','OT',536,'',1000 from dual union all
select 3944,'1956-A269','1956-02-14 00:00:00','military missile','unknown',339,'1711','Corporal TA-48','','','Test',1895,'',40 from dual union all
select 39440,'1976-S310','1976-10-26 20:21:00','military missile','success',917,'FOT-51','P-338 76 Cape 03','','','OT',231,'',180 from dual union all
select 39441,'1976-S311','1976-10-26 20:39:00','military missile','success',917,'FOT-52','P-339 76 Cape 04','','','OT',231,'',180 from dual union all
select 39442,'1976-S312','1976-10-27 16:29:00','suborbital rocket','success',290,'ISRO 5.36 C.34','ISRO 5.36 C-300','','','XR Astron',1467,'',148 from dual union all
select 39443,'1976-S313','1976-10-29 00:00:00','military missile','success',1472,'','Intercept','','','ABM Test',183,'',180 from dual union all
select 39444,'1976-S314','1976-10-29 22:11:00','suborbital rocket','success',867,'NASA 18.155UA','','','','Aeronomy',1821,'',123 from dual union all
select 39445,'1976-S315','1976-10-30 03:30:00','suborbital rocket','success',41,'NASA 26.056DG','NRL Mk II UV Camera','','','UV Astron',1895,'',182 from dual union all
select 39446,'1976-S316','1976-11-01 00:00:00','suborbital rocket','success',740,'MR-12  IOR','','','','Aeron',735,'VISE',150 from dual union all
select 39447,'1976-S317','1976-11-01 00:00:00','military missile','success',1507,'VMF','','','','Test',190,'PLBR',1000 from dual union all
select 39448,'1976-S318','1976-11-04 07:45:00','suborbital rocket','success',1211,'SL1306','LUX XRS19 LAPC','','','XR Astronomy',1839,'',191 from dual union all
select 39449,'1976-S319','1976-11-05 08:20:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT55GM','GLORY TRIP 55GM','OT',1703,'',1300 from dual union all
select 3945,'1956-A27','1956-03-04 14:06:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 39450,'1976-S32','1976-01-01 00:00:00','military missile','success',1450,'','15A20S LKI-2','','','OT',596,'',1000 from dual union all
select 39451,'1976-S320','1976-11-08 10:54:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT56GB','GLORY TRIP 56GB','OT',1712,'',1300 from dual union all
select 39452,'1976-S321','1976-11-09 12:32:00','suborbital rocket','success',227,'APD-VB-42','Cosmic rays','','','XR Astron',1849,'',215 from dual union all
select 39453,'1976-S322','1976-11-12 00:00:00','military missile','success',799,'STM-12W','Mk 12 RV x 3','R&D','STM-12W','Test',1707,'',1300 from dual union all
select 39454,'1976-S323','1976-11-13 10:31:30','suborbital rocket','success',547,'ARPA WS614.51-3','WIDEBAND','','','Ionos',1047,'',500 from dual union all
select 39455,'1976-S324','1976-11-15 03:18:20','suborbital rocket','success',1304,'DNA TE634.52-1','Ballistics','','','Test',1047,'',500 from dual union all
select 39456,'1976-S325','1976-11-17 18:30:00','suborbital rocket','success',211,'NASA 27.001CS','','','','Solar',1895,'',261 from dual union all
select 39457,'1976-S326','1976-11-18 04:25:00','suborbital rocket','success',290,'ISRO 5.37','ISRO 5.37','','','Ionos',1467,'',128 from dual union all
select 39458,'1976-S327','1976-11-19 04:06:40','suborbital rocket','success',1303,'DNA EX651.92-1','EXCEDE','','','Plasma',1047,'',300 from dual union all
select 39459,'1976-S328','1976-11-19 09:25:00','suborbital rocket','success',211,'NASA 27.002UH','UCB EUV telescope','','','EUV Astron',1895,'',295 from dual union all
select 3946,'1956-A270','1956-02-21 00:00:00','military missile','unknown',339,'1714','Corporal TA-49','','','Test',1895,'',40 from dual union all
select 39460,'1976-S329','1976-11-19 17:07:00','military missile','success',939,'','FOT-24','','','OT',421,'SSBN 617',500 from dual union all
select 39461,'1976-S33','1976-01-30 00:00:00','military missile','success',1456,'','15A20U UPB-1','','','OT',506,'',1000 from dual union all
select 39462,'1976-S330','1976-11-19 17:07:00','military missile','success',939,'','FOT-24','','','OT',421,'SSBN 617',500 from dual union all
select 39463,'1976-S331','1976-11-19 19:08:00','military missile','success',939,'','FOT-24','','','OT',421,'SSBN 617',500 from dual union all
select 39464,'1976-S332','1976-11-19 19:09:00','military missile','success',939,'','FOT-24','','','OT',421,'SSBN 617',500 from dual union all
select 39465,'1976-S333','1976-11-21 21:14:09','suborbital rocket','success',1202,'SL1422','UK High Lat 6','','','Auroral suprathermal',29,'',715 from dual union all
select 39466,'1976-S334','1976-11-21 21:14:47','suborbital rocket','success',495,'F2','','','','Auroral',29,'',137 from dual union all
select 39467,'1976-S335','1976-11-23 00:00:00','military missile','success',939,'','FOT-25','','','OT',1897,'SSBN 655',500 from dual union all
select 39468,'1976-S336','1976-11-23 00:00:00','military missile','success',939,'','FOT-25','','','OT',1897,'SSBN 655',500 from dual union all
select 39469,'1976-S337','1976-11-26 10:25:40','suborbital rocket','success',547,'ARPA WS614.51-4','WIDEBAND','','','Ionos',1047,'',500 from dual union all
select 3947,'1956-A271','1956-03-01 00:00:00','military missile','unknown',339,'1713','Corporal TA-50','','','Test',1895,'',40 from dual union all
select 39470,'1976-S338','1976-11-27 20:51:41','suborbital rocket','success',867,'NASA 18.206IE','Ferdinand 39 (Harang)','','','Auroral',29,'',210 from dual union all
select 39471,'1976-S339','1976-11-27 20:51:41','suborbital rocket','success',1337,'NASA 29.05IE','Ferdinand 38 (Harang)','','','Auroral',29,'',538 from dual union all
select 39472,'1976-S34','1976-02-18 00:00:00','military missile','success',1452,'51122-81-435T','15A30 UBP-7 ZP','GCh 15B103','','OT',544,'',1000 from dual union all
select 39473,'1976-S340','1976-12-01 06:30:00','military missile','success',799,'','Mk 12 RV x 3','OT GT32GM-1','GLORY TRIP 32GM-1','OT',1705,'',1300 from dual union all
select 39474,'1976-S341','1976-12-02 14:40:00','suborbital rocket','success',1213,'SL1114','Cul FUV-4 Echelle','','','UV Astronomy',1839,'',263 from dual union all
select 39475,'1976-S342','1976-12-03 00:00:00','military missile','success',1106,'V1','','','','Test',301,'',1000 from dual union all
select 39476,'1976-S343','1976-12-07 11:20:00','suborbital rocket','unknown',740,'MR-12','','','','Solar UV/Solar EUV',636,'',165 from dual union all
select 39477,'1976-S344','1976-12-08 11:20:00','suborbital rocket','unknown',740,'MR-12','','','','Solar UV/Solar EUV',636,'',165 from dual union all
select 39478,'1976-S345','1976-12-11 00:14:38','suborbital rocket','success',1202,'SL1425','UK High Lat 7','','','Auroral',29,'',694 from dual union all
select 39479,'1976-S346','1976-12-11 02:40:24','suborbital rocket','success',495,'F5','','','','Auroral/Aeron/Ionos',29,'',214 from dual union all
select 3948,'1956-A272','1956-03-01 00:00:00','military missile','unknown',339,'1670','Corporal U','','','Test',1895,'',40 from dual union all
select 39480,'1976-S347','1976-12-14 00:00:00','military missile','success',744,'','15A15 LKI-38','','','Test',524,'',1000 from dual union all
select 39481,'1976-S348','1976-12-14 19:35:00','suborbital rocket','success',228,'NASA 21.34NP','SPAR 3','','','Micrograv',1873,'',174 from dual union all
select 39482,'1976-S349','1976-12-15 23:18:00','military missile','success',135,'Athena D','MSV-2','','','Reentry Test',1821,'',200 from dual union all
select 39483,'1976-S35','1976-03-12 00:00:00','military missile','success',1452,'51156-79-003T','15A30 LKI-B1','GCh 15B126 (15F306 x1)','','OT',508,'',1000 from dual union all
select 39484,'1976-S350','1976-12-16 00:00:00','military missile','success',744,'','15A15 LKI-39','','','Test',530,'',1000 from dual union all
select 39485,'1976-S351','1976-12-16 01:10:00','suborbital rocket','success',117,'NRL NB24.277','IMS Test Rocket','','','Test/Astron',1873,'',512 from dual union all
select 39486,'1976-S352','1976-12-22 04:30:00','military missile','success',997,'','','','','Test',190,'K-140',1000 from dual union all
select 39487,'1976-S353','1976-12-22 00:00:00','military missile','success',1004,'114T','15A14 UBP-8 DP','GCh 15F143U3','','OT',492,'',1000 from dual union all
select 39488,'1976-S354','1976-04-06 14:46:00','military missile','success',77,'3','','','','Test',1343,'',100 from dual union all
select 39489,'1976-S355','1976-01-15 12:00:00','military missile','unknown',917,'AO-162','P-326 76 Winter 03','','','OT',433,'',180 from dual union all
select 3949,'1956-A273','1956-03-01 00:00:00','military missile','unknown',339,'1707','Corporal TA-51','','','Test',1895,'',40 from dual union all
select 39490,'1976-S356','1976-01-15 13:55:00','military missile','unknown',917,'AO-163','P-327 76 Winter 04','','','OT',433,'',180 from dual union all
select 39491,'1976-S357','1976-08-30 05:35:00','suborbital rocket','success',700,'L-4SC-4','Radar transponder','','','Test',834,'',100 from dual union all
select 39492,'1976-S358','1976-05-14 21:55:00','military missile','success',359,'702601','Performance test','','','Test',809,'',1000 from dual union all
select 39493,'1976-S359','1976-08-24 22:00:00','military missile','success',359,'702602','Performance test','','','Test',809,'',1000 from dual union all
select 39494,'1976-S36','1976-05-17 02:15:00','military missile','success',1452,'51156-79-005T','15A30 LKI-B2','GCh 15B126','','OT',535,'',1000 from dual union all
select 39495,'1976-S360','1976-06-07 14:24:16','suborbital rocket','success',1274,'','LASL Buaro','','','Release',855,'',450 from dual union all
select 39496,'1976-S361','1976-05-27 03:00:00','suborbital rocket','success',331,'WRE C4020','','','','Ionos',1849,'',103 from dual union all
select 39497,'1976-S362','1976-06-30 01:36:00','suborbital rocket','success',722,'2012','','','','Ionos',1849,'',105 from dual union all
select 39498,'1976-S363','1976-08-10 00:00:00','suborbital rocket','failure',721,'1002','Mass Spec','','','Aeron',1849,'',140 from dual union all
select 39499,'1976-S364','1976-08-11 00:00:00','suborbital rocket','failure',721,'1003','Mass Spec','','','Aeron',1849,'',140 from dual union all
select 395,'1944-S36','1944-08-14 12:53:00','military missile','success',1470,'18305','Ma123','','','Training',661,'',90 from dual union all
select 3950,'1956-A274','1956-03-02 00:00:00','military missile','unknown',339,'1671','Corporal U-12','','','Test',1895,'',40 from dual union all
select 39500,'1976-S365','1976-12-01 23:08:37','suborbital rocket','success',543,'DNA ST610.31-1','STRESS ANNE','','','Aeron/Ionos',389,'',180 from dual union all
select 39501,'1976-S366','1976-01-01 00:00:00','military missile','success',644,'','MATSESTA 3','','','Reentry Test',636,'',null from dual union all
select 39502,'1976-S367','1976-02-25 10:03:00','suborbital rocket','success',676,'WRE K2021','','','','Aeron',1832,'',100 from dual union all
select 39503,'1976-S368','1976-02-02 00:00:00','military missile','failure',1044,'','8K98P UBP-6','','','Test',477,'',0 from dual union all
select 39504,'1976-S369','1976-03-30 00:00:00','military missile','success',1044,'29542-713-04','8K98P UBP-7','','','Test',477,'',1000 from dual union all
select 39505,'1976-S37','1976-05-31 04:55:00','military missile','success',1452,'51156-79-004T','15A30 LKI-B3','GCh 15B126','','OT',508,'',1000 from dual union all
select 39506,'1976-S370','1976-08-31 00:00:00','military missile','success',1037,'V 29501-10','8K98 UBP-29','','','Test',477,'',1000 from dual union all
select 39507,'1976-S371','1976-11-17 00:00:00','military missile','success',1037,'V 29502-07','8K98 UBP-30','','','Test',477,'',1000 from dual union all
select 39508,'1976-S372','1976-11-24 00:00:00','military missile','success',1037,'V 29503-02','8K98 UBP-31','','','Test',477,'',1000 from dual union all
select 39509,'1976-S373','1976-12-07 00:00:00','military missile','failure',1044,'','8K98P UBP-8','','','Test',477,'',0 from dual union all
select 3951,'1956-A275','1956-03-02 00:00:00','military missile','unknown',339,'1639','Corporal EU','','','Test',1895,'',40 from dual union all
select 39510,'1976-S374','1976-12-17 00:00:00','military missile','success',1037,'V 29502-04','8K98 UBP-32','','','Test',477,'',1000 from dual union all
select 39511,'1976-S375','1976-12-24 00:00:00','military missile','success',1037,'V 29503-04','8K98 UBP-33','','','Test',477,'',1000 from dual union all
select 39512,'1976-S376','1976-02-19 00:00:00','military missile','success',1449,'','8K84 10 RD UBP-6','','','OT',897,'',1000 from dual union all
select 39513,'1976-S377','1976-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-11','Duga-1','','OT',1911,'',1210 from dual union all
select 39514,'1976-S378','1976-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-12','Duga-1','','OT',1911,'',1210 from dual union all
select 39515,'1976-S379','1976-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-13','Duga-1','','OT',1911,'',1210 from dual union all
select 39516,'1976-S38','1976-06-16 03:30:00','military missile','success',1452,'51122-79-007T','15A30 UBP-9 ZP','GCh 15B103','','OT',532,'',1000 from dual union all
select 39517,'1976-S380','1976-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-14','Duga-1','','OT',1911,'',1210 from dual union all
select 39518,'1976-S381','1976-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-15','Duga-1','','OT',1911,'',1210 from dual;
insert into LAUNCH
select 39519,'1976-S382','1976-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-16','Duga-1','','OT',1911,'',1210 from dual union all
select 3952,'1956-A276','1956-03-02 00:00:00','military missile','unknown',339,'1638','Corporal TA-52','','','Test',1895,'',40 from dual union all
select 39520,'1976-S383','1976-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-17','Duga-1','','OT',1911,'',1210 from dual union all
select 39521,'1976-S384','1976-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-18','Duga-1','','OT',1911,'',1210 from dual union all
select 39522,'1976-S385','1976-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-19','Duga-1','','OT',1911,'',1210 from dual union all
select 39523,'1976-S386','1976-01-01 00:00:00','military missile','success',982,'','8K64U 47RD UBP-20','Duga-1','','OT',1911,'',1210 from dual union all
select 39524,'1976-S387','1976-01-01 00:00:00','military missile','success',1450,'T02K','15A20 47 RD UBP-2','Duga-1','','OT',1911,'',1000 from dual union all
select 39525,'1976-S388','1976-01-01 00:00:00','military missile','success',982,'','8K64U 4 RD UBP-6','Duga-1','','OT',368,'',1210 from dual union all
select 39526,'1976-S389','1976-01-01 00:00:00','military missile','success',982,'','8K64U 4 RD UBP-7','Duga-1','','OT',368,'',1210 from dual union all
select 39527,'1976-S39','1976-08-05 01:50:00','military missile','success',1452,'63056-79-004A','15A30 LKI-B4','GCh 15B126','','OT',534,'',1000 from dual union all
select 39528,'1976-S390','1976-01-01 00:00:00','military missile','success',982,'','8K64U 4 RD UBP-8','Duga-1','','OT',368,'',1210 from dual union all
select 39529,'1976-S391','1976-01-01 00:00:00','military missile','success',982,'','8K64U 4 RD UBP-9','Duga-1','','OT',368,'',1210 from dual union all
select 3953,'1956-A277','1956-03-13 00:00:00','military missile','unknown',339,'1413','Corporal C','','','Test',1895,'',40 from dual union all
select 39530,'1976-S392','1976-01-01 00:00:00','military missile','success',982,'','8K64U 4 RD UBP-10','Duga-1','','OT',368,'',1210 from dual union all
select 39531,'1976-S393','1976-05-08 16:18:00','suborbital rocket','success',920,'INTA MPGAFD-7605','Winter Anom','','','Aeron/Ionos',396,'',85 from dual union all
select 39532,'1976-S394','1976-05-15 16:08:00','suborbital rocket','success',920,'INTA MPGAFD-7606','Winter Anom','','','Aeron/Ionos',396,'',85 from dual union all
select 39533,'1976-S395','1976-01-04 16:52:00','suborbital rocket','success',1284,'INTA SLW-7602','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39534,'1976-S396','1976-01-06 16:40:00','suborbital rocket','success',1284,'INTA SLW-7603','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39535,'1976-S397','1976-01-08 12:00:00','suborbital rocket','success',1284,'INTA SLW-7604','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39536,'1976-S398','1976-01-09 15:53:00','suborbital rocket','success',1284,'INTA SLW-7605','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39537,'1976-S399','1976-01-12 13:50:00','suborbital rocket','success',1284,'INTA SLW-7606','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39538,'1976-S40','1976-09-27 02:30:00','military missile','success',1452,'49242-83-441T','15A30 UBP-11 ZP','GCh 15B103','','OT',532,'',1000 from dual union all
select 39539,'1976-S400','1976-01-14 13:50:00','suborbital rocket','success',1284,'INTA SLW-7607','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 3954,'1956-A278','1956-03-13 00:00:00','military missile','unknown',339,'1635','Corporal EU','','','Test',1895,'',40 from dual union all
select 39540,'1976-S401','1976-01-20 15:50:00','suborbital rocket','success',1284,'INTA SLW-7608','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39541,'1976-S402','1976-01-21 16:40:00','suborbital rocket','success',1284,'INTA SLW-7609','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39542,'1976-S403','1976-01-22 15:56:00','suborbital rocket','success',1284,'INTA SLW-7610','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39543,'1976-S404','1976-01-23 15:50:00','suborbital rocket','success',1284,'INTA SLW-7611','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39544,'1976-S405','1976-01-24 16:00:00','suborbital rocket','success',1284,'INTA SLW-7612','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39545,'1976-S406','1976-01-24 16:41:00','suborbital rocket','success',1284,'INTA SLW-7613','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39546,'1976-S407','1976-01-26 15:59:00','suborbital rocket','success',1284,'INTA SLW-7614','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39547,'1976-S408','1976-01-28 15:50:00','suborbital rocket','success',1284,'INTA SLW-7615','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39548,'1976-S409','1976-01-30 15:38:00','suborbital rocket','success',1284,'INTA SLW-7616','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39549,'1976-S41','1976-10-22 02:10:00','military missile','success',1452,'51122-79-006T','15A30 LKI-B5','GCh 15B126','','OT',534,'',1000 from dual union all
select 3955,'1956-A279','1956-03-21 00:00:00','military missile','unknown',339,'1673','Corporal U-15','','','Test',1895,'',40 from dual union all
select 39550,'1976-S410','1976-02-05 16:26:00','suborbital rocket','success',1284,'INTA SLW-7617','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39551,'1976-S411','1976-02-08 16:20:00','suborbital rocket','success',1284,'INTA SLW-7618','Winter Anomaly','','','Aeron',396,'',90 from dual union all
select 39552,'1976-S412','1976-05-04 21:30:00','suborbital rocket','unknown',1191,'INTA MS4CLOF-7604','Winter Anom','','','Aeron/Ionos',396,'',100 from dual union all
select 39553,'1976-S413','1976-05-05 21:30:00','suborbital rocket','unknown',1191,'INTA MS4CLOF-7605','Winter Anom','','','Aeron/Ionos',396,'',100 from dual union all
select 39554,'1976-S414','1976-05-08 15:31:00','suborbital rocket','unknown',1189,'INTA MSCLO-7601','Winter Anom','','','Aeron/Ionos',396,'',100 from dual union all
select 39555,'1976-S415','1976-05-15 15:31:00','suborbital rocket','unknown',1189,'INTA MSCLO-7602','Winter Anom','','','Aeron/Ionos',396,'',100 from dual union all
select 39556,'1976-S416','1976-09-09 20:34:00','suborbital rocket','failure',722,'2005','','','','Aeron',1849,'',140 from dual union all
select 39557,'1976-S417','1976-02-23 21:30:00','suborbital rocket','success',331,'WRE C3010','','','','Aeron',1832,'',145 from dual union all
select 39558,'1976-S418','1976-09-26 07:30:00','suborbital rocket','success',725,'','','Ozone test','','Aeron',1467,'',80 from dual union all
select 39559,'1976-S419','1976-02-23 23:30:00','suborbital rocket','success',722,'2011','','','','Aeron?',1832,'',118 from dual union all
select 3956,'1956-A28','1956-03-04 14:48:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 39560,'1976-S42','1976-12-13 01:00:00','military missile','success',1452,'51122-81-436T','15A30 UBP-12 ZP','GCh 15B103','','OT',532,'',1000 from dual union all
select 39561,'1976-S420','1976-06-30 00:30:00','suborbital rocket','success',722,'2015','','','','Aeron?',1832,'',150 from dual union all
select 39562,'1976-S421','1976-06-02 15:23:00','suborbital rocket','success',1284,'','','','','Aeron/Test',1821,'',80 from dual union all
select 39563,'1976-S422','1976-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-1','Duga-1','','OT',368,'',1000 from dual union all
select 39564,'1976-S423','1976-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-2','Duga-1','','OT',368,'',1000 from dual union all
select 39565,'1976-S424','1976-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-3','Duga-1','','OT',368,'',1000 from dual union all
select 39566,'1976-S425','1976-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-4','Duga-1','','OT',368,'',1000 from dual union all
select 39567,'1976-S426','1976-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-5','Duga-1','','OT',368,'',1000 from dual union all
select 39568,'1976-S427','1976-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-6','Duga-1','','OT',368,'',1000 from dual union all
select 39569,'1976-S428','1976-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-7','Duga-1','','OT',368,'',1000 from dual union all
select 3957,'1956-A280','1956-03-22 00:00:00','military missile','unknown',339,'1686','Corporal EU','','','Test',1895,'',40 from dual union all
select 39570,'1976-S429','1976-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-8','Duga-1','','OT',368,'',1000 from dual union all
select 39571,'1976-S43','1976-12-16 00:00:00','military missile','success',1452,'51142-79-010T','15A30 LKI-B6','GCh 15B126 (15F306 x1)','','OT',534,'',1000 from dual union all
select 39572,'1976-S431','1976-01-10 00:00:00','military missile','success',922,'','15Zh45 LKI K-4','','','Test',822,'',500 from dual union all
select 39573,'1976-S432','1976-01-09 00:00:00','military missile','success',922,'','15Zh45 LKI-21','','','Test',636,'',500 from dual union all
select 39574,'1976-S433','1976-06-18 00:00:00','military missile','success',922,'','15Zh45 33 RD UBP-1','','','Test',636,'',500 from dual union all
select 39575,'1976-S434','1976-12-12 00:00:00','military missile','success',922,'','15Zh45 33 RD UBP-2','','','Test',636,'',500 from dual union all
select 39576,'1976-S435','1976-07-01 00:00:00','military missile','success',922,'','15Zh45 LKI M-1','','','Test',636,'',500 from dual union all
select 39577,'1976-S436','1976-10-01 00:00:00','military missile','success',922,'','15Zh45 LKI M-2','','','Test',636,'',500 from dual union all
select 39578,'1976-S44','1976-01-02 14:59:00','suborbital rocket','success',1190,'INTA MS3CLO-7601','Winter Anom','','','Aeron/Ionos',396,'',94 from dual union all
select 39579,'1976-S440','1976-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-9','Duga-1','','OT',368,'',1000 from dual union all
select 3958,'1956-A281','1956-03-22 00:00:00','military missile','unknown',339,'1688','Corporal TA-53','','','Test',1895,'',40 from dual union all
select 39580,'1976-S441','1976-12-23 00:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 39581,'1976-S442','1976-12-23 00:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 39582,'1976-S443','1976-12-23 00:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 39583,'1976-S444','1976-12-08 00:00:00','suborbital rocket','success',725,'','','Elec conductivity','','Ionos',636,'',85 from dual union all
select 39584,'1976-S45','1976-01-04 13:40:00','suborbital rocket','unknown',920,'INTA MPGA-7601','Winter Anomaly','','','Ionos',396,'',84 from dual union all
select 39585,'1976-S46','1976-01-04 14:30:00','suborbital rocket','success',837,'DLR H-NA-70','Winter Anom B-IV-1','INTA MNAE-7601','','Aeron/Ionos/Solar UV',396,'',129 from dual union all
select 39586,'1976-S47','1976-01-04 15:30:03','suborbital rocket','success',1207,'DLR H-CR-69','Winter Anom B-II-1','INTA SKY.3-7601','','Aeron/Ionos/Solar UV/X',395,'',116 from dual union all
select 39587,'1976-S48','1976-01-04 15:47:00','suborbital rocket','success',1190,'INTA MS3CL-7601','Winter Anom','','','Aeron/Ionos',396,'',103 from dual union all
select 39588,'1976-S49','1976-01-04 16:00:00','suborbital rocket','success',920,'INTA UCLP-7601','Winter Anom B-VI-1','','','Aeron',396,'',135 from dual union all
select 39589,'1976-S50','1976-01-04 22:58:00','suborbital rocket','unknown',1191,'INTA MS4CLOF-7601','Winter Anom','','','Aeron/Ionos',396,'',100 from dual union all
select 3959,'1956-A282','1956-03-23 00:00:00','military missile','unknown',339,'1695','Corporal TA-54','','','Test',1895,'',40 from dual union all
select 39590,'1976-S51','1976-01-06 15:46:00','suborbital rocket','success',1189,'INTA M3CL-7601','Winter Anom','','','Aeron/Ionos',396,'',96 from dual union all
select 39591,'1976-S52','1976-01-08 00:00:00','military missile','success',799,'PVM-12','Mk 12 RV x 3','R&D','','Test',1712,'',1300 from dual union all
select 39592,'1976-S53','1976-01-10 00:00:00','military missile','success',922,'','15Zh45 LKI K-3','','','Test',822,'',500 from dual union all
select 39593,'1976-S54','1976-01-09 16:11:00','suborbital rocket','unknown',1189,'INTA MSCL-7602','Winter Anom','','','Aeron/Ionos',396,'',100 from dual union all
select 39594,'1976-S55','1976-01-10 14:16:00','suborbital rocket','success',228,'NASA 12.29GT-IT','SSC S19A','','','Tech',1818,'',290 from dual union all
select 39595,'1976-S56','1976-01-12 14:20:00','suborbital rocket','success',1190,'INTA MS3CLO-7602','Winter Anom','','','Aeron/Ionos',396,'',91 from dual union all
select 39596,'1976-S57','1976-01-14 14:10:00','suborbital rocket','success',1190,'INTA MS3CL-7602','Winter Anom','','','Aeron/Ionos',396,'',93 from dual union all
select 39597,'1976-S58','1976-01-15 18:30:00','suborbital rocket','failure',1337,'NASA 12.34CT-GT','','','','Test',1821,'',81 from dual union all
select 39598,'1976-S59','1976-01-16 21:00:00','suborbital rocket','success',668,'K-9M-54','','','','Magnetospheric',839,'',367 from dual union all
select 39599,'1976-S60','1976-01-18 00:00:00','suborbital rocket','success',867,'NASA 18.1007UE','Wiinter Anom TiCl-TMA','','','Plasma',1821,'',270 from dual union all
select 396,'1944-S360','1944-11-22 00:00:00','military missile','success',1470,'','London 236','','','Weapon',1627,'',90 from dual union all
select 3960,'1956-A283','1956-03-23 00:00:00','military missile','unknown',339,'1674','Corporal U-16','','','Test',1895,'',40 from dual union all
select 39600,'1976-S61','1976-01-18 05:20:00','suborbital rocket','success',655,'K-10-12','','','','Ionos',839,'',350 from dual union all
select 39601,'1976-S62','1976-01-20 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 608',1000 from dual union all
select 39602,'1976-S63','1976-01-20 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 608',1000 from dual union all
select 39603,'1976-S64','1976-01-20 01:42:00','suborbital rocket','success',220,'GL A16.501-01','VLF','','','Ionos',428,'',632 from dual union all
select 39604,'1976-S66','1976-01-21 14:32:09','suborbital rocket','success',837,'DLR H-NA-72/Ferd 43','Winter Anom B-IV-2','INTA MNAE-7602','','Aeron/Ionos',396,'',120 from dual union all
select 39605,'1976-S67','1976-01-21 15:31:00','suborbital rocket','success',1207,'DLR H-CR-71','Winter Anom B-II-2','INTA SKY.3-7602','','Aeron/Ionos/Solar UV/X',395,'',116 from dual union all
select 39606,'1976-S68','1976-01-21 15:55:00','suborbital rocket','success',920,'INTA UCLP-7602','Winter Anom B-VI-2','','','Aeron',396,'',140 from dual union all
select 39607,'1976-S69','1976-01-21 18:17:00','suborbital rocket','success',920,'INTA UCLP-7603','Winter Anom B-VI-3','','','Aeron',396,'',135 from dual union all
select 39608,'1976-S70','1976-01-21 22:30:00','suborbital rocket','success',1191,'INTA MS4CLOF-7602','Winter Anomaly','','','Auroral/Aeron/Ionos',396,'',99 from dual;
insert into LAUNCH
select 39609,'1976-S71','1976-01-22 16:16:00','suborbital rocket','success',1189,'INTA MSCL-7604','Winter Anom','','','Aeron/Ionos',396,'',93 from dual union all
select 3961,'1956-A284','1956-04-05 00:00:00','military missile','unknown',339,'1640','Corporal EU','','','Test',1895,'',40 from dual union all
select 39610,'1976-S72','1976-01-22 18:58:36','suborbital rocket','success',867,'SSC S17-1 (EF8)','SSC S17/1 M/Dtr','','','Ionos',417,'',208 from dual union all
select 39611,'1976-S73','1976-01-23 00:00:00','military missile','success',795,'FTM 1313','Mk 500 RB 105','Evader SFT-105','SFT-105','Reentry Test',1701,'',1300 from dual union all
select 39612,'1976-S74','1976-01-23 16:10:00','suborbital rocket','success',1189,'INTA MSCL-7605','Winter Anom','','','Aeron/Ionos',396,'',99 from dual union all
select 39613,'1976-S75','1976-01-23 22:34:00','suborbital rocket','success',1468,'GL A09.402-2','Winter Anomaly','','','Aeron',1821,'',124 from dual union all
select 39614,'1976-S76','1976-01-23 22:45:00','suborbital rocket','success',867,'GL A08.608-1','Winter Anomaly','','','Aeron',1821,'',117 from dual union all
select 39615,'1976-S77','1976-01-24 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 600',1000 from dual union all
select 39616,'1976-S78','1976-01-24 00:00:00','military missile','success',931,'A3T','FOT','','','OT',1897,'SSBN 600',1000 from dual union all
select 39617,'1976-S79','1976-01-25 23:20:00','suborbital rocket','success',1070,'S-210JA-22','NO/Ne','','','Aeron/Ionos',1458,'',127 from dual union all
select 39618,'1976-S80','1976-01-26 16:19:00','suborbital rocket','success',1190,'INTA MS3CL-7603','Winter Anom','','','Aeron/Ionos',396,'',100 from dual union all
select 39619,'1976-S81','1976-01-27 06:00:00','suborbital rocket','success',1298,'TT-210-2F','','','','Test',1476,'',100 from dual union all
select 3962,'1956-A285','1956-04-10 00:00:00','military missile','unknown',339,'1461','Corporal Chem','','','Test',1895,'',40 from dual union all
select 39620,'1976-S82','1976-01-28 16:10:00','suborbital rocket','unknown',1190,'INTA MS3CL-7604','Winter Anom','','','Aeron/Ionos',396,'',100 from dual union all
select 39621,'1976-S83','1976-01-30 04:50:00','military missile','success',796,'','Mk 11 RV','FOT GT125M','GLORY TRIP 125M','OT',1702,'',1300 from dual union all
select 39622,'1976-S84','1976-01-30 00:00:00','military missile','success',1004,'118T','15A14 UBP-1','GCh 15F143U1','','OT',486,'',1000 from dual union all
select 39623,'1976-S85','1976-01-30 01:07:00','suborbital rocket','success',1213,'SL1302','MSSL SX9 Coronal','','','Solar X',1839,'',278 from dual union all
select 39624,'1976-S86','1976-02-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',139,'ZUB',150 from dual union all
select 39625,'1976-S87','1976-02-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',139,'ZUB',150 from dual union all
select 39626,'1976-S88','1976-01-31 06:44:10','suborbital rocket','success',228,'NASA 21.27UE','ECHO IV','','','Auroral',1047,'',225 from dual union all
select 39627,'1976-S89','1976-02-01 19:28:55','suborbital rocket','success',867,'NASA 18.208UE','Ferdinand 45/Polar 5','','','Auroral/Active',29,'',221 from dual union all
select 39628,'1976-S90','1976-02-03 21:15:00','military missile','success',939,'','FOT-17','','','OT',421,'SSBN 643',500 from dual union all
select 39629,'1976-S91','1976-02-03 21:16:00','military missile','success',939,'','FOT-17','','','OT',421,'SSBN 643',500 from dual union all
select 3963,'1956-A286','1956-04-12 00:00:00','military missile','unknown',339,'1687','Corporal EU','','','Test',1895,'',40 from dual union all
select 39630,'1976-S92','1976-02-03 21:47:00','military missile','success',939,'','FOT-17','','','OT',421,'SSBN 643',500 from dual union all
select 39631,'1976-S93','1976-02-05 05:22:00','suborbital rocket','success',220,'AAF-4B-33','','','','Test/Auroral/Ionos',428,'',730 from dual union all
select 39632,'1976-S94','1976-02-05 15:51:00','suborbital rocket','success',1189,'INTA MSCL-7606','Winter Anom','','','Aeron/Ionos',396,'',102 from dual union all
select 39633,'1976-S95','1976-02-05 22:30:00','suborbital rocket','success',1191,'INTA MS4CLOF-7603','Winter Anom','','','Auroral/Aeron/Ionos',396,'',101 from dual union all
select 39634,'1976-S96','1976-02-06 00:00:00','military missile','failure',799,'','Mk 12 RV x 3','OT GT26GM-4','GLORY TRIP 26GM-4','OT',1705,'',200 from dual union all
select 39635,'1976-S97','1976-02-08 14:26:00','suborbital rocket','success',920,'INTA MPGAFD-7604','Winter Anom','','','Aeron/Ionos',396,'',81 from dual union all
select 39636,'1976-S98','1976-02-08 15:34:00','suborbital rocket','success',1189,'INTA MSCL-7607','Winter Anom','','','Aeron/Ionos',396,'',100 from dual union all
select 39637,'1976-S99','1976-02-12 00:00:00','military missile','success',1004,'2T','15A14 UBP-2 SLI','GCh 15F142U2 -','','OT',510,'',1000 from dual union all
select 39638,'1976-U01','1976-08-19 05:25:00','lunar return','success',450,'No. 413','Luna-24','VA','','Sat',337,'',null from dual union all
select 39639,'1976-U02','1976-06-18 11:41:00','ballistic missile test','success',1132,'S193C','GP-A','GP-A','','Physics',1814,'',10230 from dual union all
select 3964,'1956-A287','1956-04-24 00:00:00','military missile','unknown',339,'1684','Corporal EU','','','Test',1895,'',40 from dual union all
select 39640,'1976-W01','1976-01-01 01:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 39641,'1976-W02','1976-01-03 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 39642,'1976-W03','1976-01-05 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',69 from dual union all
select 39643,'1976-W04','1976-01-05 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',78 from dual union all
select 39644,'1976-W05','1976-01-05 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 39645,'1976-W06','1976-01-05 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',53 from dual union all
select 39646,'1976-W07','1976-01-05 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 39647,'1976-W08','1976-01-05 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 39648,'1976-W09','1976-01-05 18:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',55 from dual union all
select 39649,'1976-W10','1976-01-05 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 3965,'1956-A288','1956-05-03 00:00:00','military missile','unknown',339,'1685','Corporal EU','','','Test',1895,'',40 from dual union all
select 39650,'1976-W100','1976-01-21 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 39651,'1976-W1000','1976-07-21 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',330,'',54 from dual union all
select 39652,'1976-W1001','1976-07-21 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 39653,'1976-W1002','1976-07-21 15:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',67 from dual union all
select 39654,'1976-W1003','1976-07-21 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 39655,'1976-W1004','1976-07-21 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 39656,'1976-W1005','1976-07-21 17:02:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 39657,'1976-W1006','1976-07-21 17:35:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',74 from dual union all
select 39658,'1976-W1007','1976-07-21 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 39659,'1976-W1008','1976-07-21 18:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 3966,'1956-A289','1956-05-04 00:00:00','military missile','unknown',339,'1795','Corporal TA-55','','','Test',1895,'',40 from dual union all
select 39660,'1976-W1009','1976-07-21 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 39661,'1976-W101','1976-01-21 22:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',75 from dual union all
select 39662,'1976-W1010','1976-07-21 19:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',65 from dual union all
select 39663,'1976-W1011','1976-07-21 21:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 39664,'1976-W1012','1976-07-22 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 39665,'1976-W1013','1976-07-22 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 39666,'1976-W1014','1976-07-22 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',50 from dual union all
select 39667,'1976-W1015','1976-07-22 23:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 39668,'1976-W1016','1976-07-23 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 39669,'1976-W1017','1976-07-23 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',59 from dual union all
select 3967,'1956-A29','1956-03-04 14:59:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 39670,'1976-W1018','1976-07-23 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 39671,'1976-W1019','1976-07-23 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',65 from dual union all
select 39672,'1976-W102','1976-01-22 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 39673,'1976-W1020','1976-07-26 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 39674,'1976-W1021','1976-07-26 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 39675,'1976-W1022','1976-07-26 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',67 from dual union all
select 39676,'1976-W1023','1976-07-26 18:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 39677,'1976-W1024','1976-07-26 19:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',61 from dual union all
select 39678,'1976-W1025','1976-07-27 11:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 39679,'1976-W1026','1976-07-27 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',52 from dual union all
select 3968,'1956-A290','1956-05-15 00:00:00','military missile','unknown',339,'1805','Corporal TA-56','','','Test',1895,'',40 from dual union all
select 39680,'1976-W1027','1976-07-27 16:54:59','suborbital rocket','success',1061,'','Chaff,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 39681,'1976-W1028','1976-07-27 19:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',62 from dual union all
select 39682,'1976-W1029','1976-07-27 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 39683,'1976-W103','1976-01-22 01:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 39684,'1976-W1030','1976-07-27 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',72 from dual union all
select 39685,'1976-W1031','1976-07-27 23:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 39686,'1976-W1032','1976-07-28 12:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',58 from dual union all
select 39687,'1976-W1033','1976-07-28 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 39688,'1976-W1034','1976-07-28 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 39689,'1976-W1035','1976-07-28 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 3969,'1956-A291','1956-05-16 00:00:00','military missile','unknown',339,'1675','Corporal II U-17','','','Test',1895,'',40 from dual union all
select 39690,'1976-W1036','1976-07-28 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 39691,'1976-W1037','1976-07-28 16:36:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 39692,'1976-W1038','1976-07-28 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 39693,'1976-W1039','1976-07-28 17:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',308,'',60 from dual union all
select 39694,'1976-W104','1976-01-22 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',60 from dual union all
select 39695,'1976-W1040','1976-07-28 18:14:59','suborbital rocket','success',1061,'','Chaff,Arcasonde','','','Meteo',1895,'',51 from dual union all
select 39696,'1976-W1041','1976-07-28 18:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',64 from dual union all
select 39697,'1976-W1042','1976-07-28 21:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 39698,'1976-W1043','1976-07-28 21:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 39699,'1976-W1044','1976-07-28 21:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual;
insert into LAUNCH
select 397,'1944-S361','1944-11-24 00:00:00','military missile','success',1470,'','London 250','','','Weapon',1627,'',90 from dual union all
select 3970,'1956-A292','1956-05-16 00:00:00','military missile','unknown',339,'1801','Corporal TA-57','','','Test',1895,'',40 from dual union all
select 39700,'1976-W1045','1976-07-29 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',330,'',63 from dual union all
select 39701,'1976-W1046','1976-07-29 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 39702,'1976-W1047','1976-07-29 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',51 from dual union all
select 39703,'1976-W1048','1976-07-30 12:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 39704,'1976-W1049','1976-07-30 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',61 from dual union all
select 39705,'1976-W105','1976-01-22 16:10:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 39706,'1976-W1050','1976-07-30 15:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 39707,'1976-W1051','1976-07-30 16:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 39708,'1976-W1052','1976-07-30 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 39709,'1976-W1053','1976-07-30 21:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 3971,'1956-A293','1956-05-17 00:00:00','military missile','unknown',339,'1807','Corporal TA-58','','','Test',1895,'',40 from dual union all
select 39710,'1976-W1054','1976-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 39711,'1976-W1055','1976-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 39712,'1976-W1056','1976-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 39713,'1976-W1057','1976-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 39714,'1976-W1058','1976-08-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 39715,'1976-W1059','1976-08-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 39716,'1976-W106','1976-01-22 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 39717,'1976-W1060','1976-08-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 39718,'1976-W1061','1976-07-31 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',76 from dual union all
select 39719,'1976-W1062','1976-07-31 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',58 from dual union all
select 3972,'1956-A294','1956-05-18 00:00:00','military missile','unknown',339,'1827','Corporal II U-18','','','Test',1895,'',40 from dual union all
select 39720,'1976-W1063','1976-08-02 12:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 39721,'1976-W1064','1976-08-02 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 39722,'1976-W1065','1976-08-02 14:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 39723,'1976-W1066','1976-08-02 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 39724,'1976-W1067','1976-08-02 17:23:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',57 from dual union all
select 39725,'1976-W1068','1976-08-02 17:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 39726,'1976-W1069','1976-08-02 18:10:00','suborbital rocket','success',1061,'','Chaff,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 39727,'1976-W107','1976-01-22 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 39728,'1976-W1070','1976-08-02 20:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',62 from dual union all
select 39729,'1976-W1071','1976-08-02 23:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 3973,'1956-A295','1956-05-22 00:00:00','military missile','unknown',339,'1818','Corporal EU AEO-24','','','Test',1895,'',40 from dual union all
select 39730,'1976-W1072','1976-08-03 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 39731,'1976-W1073','1976-08-04 13:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 39732,'1976-W1074','1976-08-04 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 39733,'1976-W1075','1976-08-04 14:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 39734,'1976-W1076','1976-08-04 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',62 from dual union all
select 39735,'1976-W1077','1976-08-04 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 39736,'1976-W1078','1976-08-04 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 39737,'1976-W1079','1976-08-04 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',57 from dual union all
select 39738,'1976-W108','1976-01-22 18:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 39739,'1976-W1080','1976-08-04 16:48:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 3974,'1956-A296','1956-05-23 00:00:00','military missile','unknown',339,'1823','Corporal TA-59','','','Test',1895,'',40 from dual union all
select 39740,'1976-W1081','1976-08-04 17:10:00','suborbital rocket','success',1061,'','Chaff,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 39741,'1976-W1082','1976-08-04 20:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',74 from dual union all
select 39742,'1976-W1083','1976-08-04 21:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',50 from dual union all
select 39743,'1976-W1084','1976-08-04 21:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 39744,'1976-W1085','1976-08-05 12:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 39745,'1976-W1086','1976-08-05 16:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',308,'',63 from dual union all
select 39746,'1976-W1087','1976-08-06 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',59 from dual union all
select 39747,'1976-W1088','1976-08-06 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 39748,'1976-W1089','1976-08-06 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 39749,'1976-W109','1976-01-22 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 3975,'1956-A297','1956-05-23 00:00:00','military missile','unknown',339,'1824','Corporal TA-62','','','Test',1895,'',40 from dual union all
select 39750,'1976-W1090','1976-08-06 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 39751,'1976-W1091','1976-08-06 16:44:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 39752,'1976-W1092','1976-08-06 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 39753,'1976-W1093','1976-08-06 18:00:00','suborbital rocket','success',1061,'','Chaff,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 39754,'1976-W1094','1976-08-06 20:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',53 from dual union all
select 39755,'1976-W1095','1976-08-09 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 39756,'1976-W1096','1976-08-09 13:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 39757,'1976-W1097','1976-08-09 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',57 from dual union all
select 39758,'1976-W1098','1976-08-09 16:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 39759,'1976-W1099','1976-08-09 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 3976,'1956-A298','1956-06-07 00:00:00','military missile','unknown',339,'1822','Corporal TA-64','','','Test',1895,'',40 from dual union all
select 39760,'1976-W11','1976-01-05 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 39761,'1976-W110','1976-01-22 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 39762,'1976-W1100','1976-08-09 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 39763,'1976-W1101','1976-08-09 16:58:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 39764,'1976-W1102','1976-08-09 18:40:00','suborbital rocket','success',1061,'','Chaff,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 39765,'1976-W1103','1976-08-09 20:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',65 from dual union all
select 39766,'1976-W1104','1976-08-09 21:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 39767,'1976-W1105','1976-08-09 23:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 39768,'1976-W1106','1976-08-11 12:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 39769,'1976-W1107','1976-08-11 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 3977,'1956-A299','1956-06-07 00:00:00','military missile','unknown',339,'1821','Corporal TA-63','','','Test',1895,'',40 from dual union all
select 39770,'1976-W1108','1976-08-11 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',53 from dual union all
select 39771,'1976-W1109','1976-08-11 14:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',330,'',53 from dual union all
select 39772,'1976-W111','1976-01-23 10:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 39773,'1976-W1110','1976-08-11 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 39774,'1976-W1111','1976-08-11 16:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 39775,'1976-W1112','1976-08-11 16:21:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 39776,'1976-W1113','1976-08-11 16:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',50 from dual union all
select 39777,'1976-W1114','1976-08-11 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 39778,'1976-W1115','1976-08-11 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 39779,'1976-W1116','1976-08-11 21:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',65 from dual union all
select 3978,'1956-A30','1956-03-04 15:43:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 39780,'1976-W1117','1976-08-11 22:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 39781,'1976-W1118','1976-08-12 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 39782,'1976-W1119','1976-08-12 02:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1632,'USHAKOV',60 from dual union all
select 39783,'1976-W112','1976-01-23 13:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 39784,'1976-W1120','1976-08-12 03:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 39785,'1976-W1121','1976-08-12 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',308,'',56 from dual union all
select 39786,'1976-W1122','1976-08-12 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 39787,'1976-W1123','1976-08-12 23:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 39788,'1976-W1124','1976-08-13 12:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 39789,'1976-W1125','1976-08-13 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',59 from dual;
insert into LAUNCH
select 3979,'1956-A300','1956-06-13 00:00:00','military missile','unknown',339,'1820','Corporal EU AEO-26','','','Test',1895,'',40 from dual union all
select 39790,'1976-W1126','1976-08-13 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 39791,'1976-W1127','1976-08-13 16:08:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 39792,'1976-W1128','1976-08-13 16:26:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 39793,'1976-W1129','1976-08-13 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 39794,'1976-W113','1976-01-23 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',58 from dual union all
select 39795,'1976-W1130','1976-08-13 17:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 39796,'1976-W1131','1976-08-13 20:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',65 from dual union all
select 39797,'1976-W1132','1976-08-13 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 39798,'1976-W1133','1976-08-16 14:45:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',52 from dual union all
select 39799,'1976-W1134','1976-08-16 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',50 from dual union all
select 398,'1944-S362','1944-11-25 11:11:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 3980,'1956-A301','1956-06-13 00:00:00','military missile','unknown',339,'1373','Corporal Chem 8','','','Test',1895,'',40 from dual union all
select 39800,'1976-W1135','1976-08-16 15:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 39801,'1976-W1136','1976-08-16 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 39802,'1976-W1137','1976-08-16 17:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 39803,'1976-W1138','1976-08-16 18:00:00','suborbital rocket','success',1061,'','Chaff,Arcasonde','','','Meteo',1895,'',66 from dual union all
select 39804,'1976-W1139','1976-08-16 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',61 from dual union all
select 39805,'1976-W114','1976-01-23 16:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 39806,'1976-W1140','1976-08-16 21:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 39807,'1976-W1141','1976-08-17 00:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 39808,'1976-W1142','1976-08-17 05:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 39809,'1976-W1143','1976-08-17 08:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',55 from dual union all
select 3981,'1956-A302','1956-06-15 00:00:00','military missile','unknown',339,'1348','Corporal Chem 7','','','Test',1895,'',40 from dual union all
select 39810,'1976-W1144','1976-08-17 09:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 39811,'1976-W1145','1976-08-17 12:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 39812,'1976-W1146','1976-08-17 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 39813,'1976-W1147','1976-08-17 15:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 39814,'1976-W1148','1976-08-17 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 39815,'1976-W1149','1976-08-18 12:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 39816,'1976-W115','1976-01-23 16:26:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 39817,'1976-W1150','1976-08-18 14:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',330,'',57 from dual union all
select 39818,'1976-W1151','1976-08-18 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',63 from dual union all
select 39819,'1976-W1152','1976-08-18 15:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 3982,'1956-A303','1956-06-16 00:00:00','military missile','unknown',339,'1781','Corporal EU','','','Test',1895,'',40 from dual union all
select 39820,'1976-W1153','1976-08-18 16:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 39821,'1976-W1154','1976-08-18 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',308,'',65 from dual union all
select 39822,'1976-W1155','1976-08-18 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',50 from dual union all
select 39823,'1976-W1156','1976-08-18 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 39824,'1976-W1157','1976-08-18 18:00:00','suborbital rocket','success',1061,'','Chaff,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 39825,'1976-W1158','1976-08-18 19:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',75 from dual union all
select 39826,'1976-W1159','1976-08-18 21:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 39827,'1976-W116','1976-01-23 18:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 39828,'1976-W1160','1976-08-18 22:08:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 39829,'1976-W1161','1976-08-19 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.271','','','Aeron',1467,'',68 from dual union all
select 3983,'1956-A304','1956-06-21 00:00:00','military missile','unknown',339,'1686','Corporal TA-66','','','Test',1895,'',40 from dual union all
select 39830,'1976-W1162','1976-08-19 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 39831,'1976-W1163','1976-08-19 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 39832,'1976-W1164','1976-08-19 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',50 from dual union all
select 39833,'1976-W1165','1976-08-19 15:14:59','suborbital rocket','success',1061,'','Chaff,Arcasonde','','','Meteo',1895,'',61 from dual union all
select 39834,'1976-W1166','1976-08-19 16:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 39835,'1976-W1167','1976-08-19 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 39836,'1976-W1168','1976-08-20 12:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 39837,'1976-W1169','1976-08-20 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 39838,'1976-W117','1976-01-23 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 39839,'1976-W1170','1976-08-20 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 3984,'1956-A305','1956-06-23 00:00:00','military missile','unknown',339,'1712','Corporal TA-67','','','Test',1895,'',40 from dual union all
select 39840,'1976-W1171','1976-08-20 16:23:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 39841,'1976-W1172','1976-08-20 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',64 from dual union all
select 39842,'1976-W1173','1976-08-21 01:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 39843,'1976-W1174','1976-08-22 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 39844,'1976-W1175','1976-08-23 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 39845,'1976-W1176','1976-08-23 14:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',76 from dual union all
select 39846,'1976-W1177','1976-08-23 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 39847,'1976-W1178','1976-08-23 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 39848,'1976-W1179','1976-08-23 16:11:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 39849,'1976-W118','1976-01-23 21:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 3985,'1956-A306','1956-06-30 00:00:00','military missile','unknown',339,'1829','Corporal II U-19','','','Test',1895,'',40 from dual union all
select 39850,'1976-W1180','1976-08-23 16:19:59','suborbital rocket','success',1061,'','Chaff,Arcasonde','','','Meteo',1895,'',62 from dual union all
select 39851,'1976-W1181','1976-08-23 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',64 from dual union all
select 39852,'1976-W1182','1976-08-23 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 39853,'1976-W1183','1976-08-24 14:58:00','suborbital rocket','unknown',1284,'T 1-8439','','','','Ionos',1821,'',66 from dual union all
select 39854,'1976-W1184','1976-08-24 16:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 39855,'1976-W1185','1976-08-24 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 39856,'1976-W1186','1976-08-25 12:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 39857,'1976-W1187','1976-08-25 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',53 from dual union all
select 39858,'1976-W1188','1976-08-25 14:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 39859,'1976-W1189','1976-08-25 14:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',330,'',57 from dual union all
select 3986,'1956-A307','1956-07-06 00:00:00','military missile','unknown',339,'1780','Corporal EU','','','Test',1895,'',40 from dual union all
select 39860,'1976-W119','1976-01-23 23:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 39861,'1976-W1190','1976-08-25 14:45:00','suborbital rocket','unknown',1284,'T 1-8440','','','','Ionos',1821,'',66 from dual union all
select 39862,'1976-W1191','1976-08-25 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 39863,'1976-W1192','1976-08-25 16:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 39864,'1976-W1193','1976-08-25 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',54 from dual union all
select 39865,'1976-W1194','1976-08-25 20:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',71 from dual union all
select 39866,'1976-W1195','1976-08-25 20:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 39867,'1976-W1196','1976-08-26 01:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 39868,'1976-W1197','1976-08-26 14:57:00','suborbital rocket','unknown',1284,'T 1-8437','','','','Ionos',1821,'',67 from dual union all
select 39869,'1976-W1198','1976-08-26 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',54 from dual union all
select 3987,'1956-A308','1956-07-06 00:00:00','military missile','unknown',339,'1350','Corporal TA-68','','','Test',1895,'',40 from dual union all
select 39870,'1976-W1199','1976-08-26 15:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 39871,'1976-W12','1976-01-05 23:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 39872,'1976-W120','1976-01-24 00:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 39873,'1976-W1200','1976-08-26 16:04:00','suborbital rocket','unknown',1284,'T 1-8450','','','','Ionos',1821,'',67 from dual union all
select 39874,'1976-W1201','1976-08-27 12:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 39875,'1976-W1202','1976-08-27 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 39876,'1976-W1203','1976-08-27 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 39877,'1976-W1204','1976-08-27 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 39878,'1976-W1205','1976-08-27 16:27:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 39879,'1976-W1206','1976-08-27 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',53 from dual;
insert into LAUNCH
select 3988,'1956-A309','1956-07-12 00:00:00','military missile','unknown',339,'1700','Corporal EU','','','Test',1895,'',40 from dual union all
select 39880,'1976-W1207','1976-08-27 20:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',64 from dual union all
select 39881,'1976-W1208','1976-08-28 02:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1632,'USHAKOV',61 from dual union all
select 39882,'1976-W1209','1976-08-28 16:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 39883,'1976-W121','1976-01-26 05:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 39884,'1976-W1210','1976-08-29 15:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 39885,'1976-W1211','1976-08-30 12:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',56 from dual union all
select 39886,'1976-W1212','1976-08-30 13:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',54 from dual union all
select 39887,'1976-W1213','1976-08-30 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 39888,'1976-W1214','1976-08-30 17:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 39889,'1976-W1215','1976-08-30 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',62 from dual union all
select 3989,'1956-A31','1956-03-04 15:53:00','atmospheric rocket','success',836,'','Cold Weather Test','','','Test',428,'',10 from dual union all
select 39890,'1976-W1216','1976-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 39891,'1976-W1217','1976-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 39892,'1976-W1218','1976-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 39893,'1976-W1219','1976-09-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 39894,'1976-W122','1976-01-26 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',56 from dual union all
select 39895,'1976-W1220','1976-09-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 39896,'1976-W1221','1976-09-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 39897,'1976-W1222','1976-09-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 39898,'1976-W1223','1976-08-31 01:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 39899,'1976-W1224','1976-08-31 04:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 399,'1944-S363','1944-11-27 10:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1627,'',90 from dual union all
select 3990,'1956-A310','1956-07-12 00:00:00','military missile','unknown',339,'1698','Corporal EU','','','Test',1895,'',40 from dual union all
select 39900,'1976-W1225','1976-08-31 05:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 39901,'1976-W1226','1976-08-31 15:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 39902,'1976-W1227','1976-08-31 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 39903,'1976-W1228','1976-08-31 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',64 from dual union all
select 39904,'1976-W1229','1976-08-31 20:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 39905,'1976-W123','1976-01-26 16:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 39906,'1976-W1230','1976-08-31 21:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 39907,'1976-W1231','1976-09-01 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',54 from dual union all
select 39908,'1976-W1232','1976-09-01 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 39909,'1976-W1233','1976-09-01 14:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',330,'',59 from dual union all
select 3991,'1956-A311','1956-07-13 00:00:00','military missile','unknown',339,'1798','Corporal II U-20','','','Test',1895,'',40 from dual union all
select 39910,'1976-W1234','1976-09-01 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 39911,'1976-W1235','1976-09-01 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 39912,'1976-W1236','1976-09-01 16:34:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 39913,'1976-W1237','1976-09-01 16:42:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 39914,'1976-W1238','1976-09-01 18:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 39915,'1976-W1239','1976-09-01 19:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',71 from dual union all
select 39916,'1976-W124','1976-01-26 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 39917,'1976-W1240','1976-09-01 21:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 39918,'1976-W1241','1976-09-01 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 39919,'1976-W1242','1976-09-02 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 3992,'1956-A312','1956-07-13 00:00:00','military missile','unknown',339,'1828','Corporal II U-21','','','Test',1895,'',40 from dual union all
select 39920,'1976-W1243','1976-09-02 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',58 from dual union all
select 39921,'1976-W1244','1976-09-03 12:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 39922,'1976-W1245','1976-09-03 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 39923,'1976-W1246','1976-09-03 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',52 from dual union all
select 39924,'1976-W1247','1976-09-03 16:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 39925,'1976-W1248','1976-09-03 23:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 39926,'1976-W1249','1976-09-04 00:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 39927,'1976-W125','1976-01-26 17:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',52 from dual union all
select 39928,'1976-W1250','1976-09-04 03:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1632,'USHAKOV',62 from dual union all
select 39929,'1976-W1251','1976-09-04 15:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 3993,'1956-A313','1956-07-14 00:00:00','military missile','unknown',339,'1794','Corporal II U-23','','','Test',1895,'',40 from dual union all
select 39930,'1976-W1252','1976-09-05 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 39931,'1976-W1253','1976-09-06 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 39932,'1976-W1254','1976-09-06 15:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 39933,'1976-W1255','1976-09-07 15:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 39934,'1976-W1256','1976-09-07 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 39935,'1976-W1257','1976-09-07 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 39936,'1976-W1258','1976-09-07 17:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 39937,'1976-W1259','1976-09-07 18:00:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',50 from dual union all
select 39938,'1976-W126','1976-01-26 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 39939,'1976-W1260','1976-09-07 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',69 from dual union all
select 3994,'1956-A314','1956-07-16 00:00:00','military missile','unknown',339,'1696','Corporal EU','','','Test',1895,'',40 from dual union all
select 39940,'1976-W1261','1976-09-07 20:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 39941,'1976-W1262','1976-09-07 20:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 39942,'1976-W1263','1976-09-07 23:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 39943,'1976-W1264','1976-09-08 03:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1632,'USHAKOV',61 from dual union all
select 39944,'1976-W1265','1976-09-08 13:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 39945,'1976-W1266','1976-09-08 14:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 39946,'1976-W1267','1976-09-08 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 39947,'1976-W1268','1976-09-08 15:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 39948,'1976-W1269','1976-09-08 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',63 from dual union all
select 39949,'1976-W127','1976-01-26 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 3995,'1956-A315','1956-07-24 00:00:00','military missile','unknown',339,'1817','Corporal EU AEB-25','','','Test',1895,'',40 from dual union all
select 39950,'1976-W1270','1976-09-08 16:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',55 from dual union all
select 39951,'1976-W1271','1976-09-08 16:07:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 39952,'1976-W1272','1976-09-08 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 39953,'1976-W1273','1976-09-08 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 39954,'1976-W1274','1976-09-08 19:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',66 from dual union all
select 39955,'1976-W1275','1976-09-08 21:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 39956,'1976-W1276','1976-09-08 22:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 39957,'1976-W1277','1976-09-09 02:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1632,'USHAKOV',61 from dual union all
select 39958,'1976-W1278','1976-09-09 12:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 39959,'1976-W1279','1976-09-09 14:16:00','suborbital rocket','unknown',1284,'T 1-8454','','','','Ionos',1821,'',70 from dual union all
select 3996,'1956-A316','1956-08-20 00:00:00','military missile','unknown',339,'1691','Corporal TA-69','','','Test',1895,'',40 from dual union all
select 39960,'1976-W128','1976-01-26 20:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 39961,'1976-W1280','1976-09-09 14:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',330,'',56 from dual union all
select 39962,'1976-W1281','1976-09-09 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 39963,'1976-W1282','1976-09-09 15:30:00','suborbital rocket','unknown',1284,'T 1-8455','','','','Ionos',1821,'',68 from dual union all
select 39964,'1976-W1283','1976-09-09 17:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 39965,'1976-W1284','1976-09-09 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 39966,'1976-W1285','1976-09-10 00:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 39967,'1976-W1286','1976-09-10 14:48:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',54 from dual union all
select 39968,'1976-W1287','1976-09-10 16:29:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 39969,'1976-W1288','1976-09-10 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual;
insert into LAUNCH
select 3997,'1956-A317','1956-08-21 00:00:00','military missile','unknown',339,'1793','Corporal TA-72','','','Test',1895,'',40 from dual union all
select 39970,'1976-W1289','1976-09-10 19:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',63 from dual union all
select 39971,'1976-W129','1976-01-26 23:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 39972,'1976-W1290','1976-09-10 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 39973,'1976-W1291','1976-09-13 12:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 39974,'1976-W1292','1976-09-13 14:56:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',55 from dual union all
select 39975,'1976-W1293','1976-09-13 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 39976,'1976-W1294','1976-09-13 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 39977,'1976-W1295','1976-09-13 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 39978,'1976-W1296','1976-09-13 19:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',60 from dual union all
select 39979,'1976-W1297','1976-09-14 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 3998,'1956-A318','1956-08-21 00:00:00','military missile','unknown',339,'1694','Corporal TA-70','','','Test',1895,'',40 from dual union all
select 39980,'1976-W1298','1976-09-15 02:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1632,'USHAKOV',61 from dual union all
select 39981,'1976-W1299','1976-09-15 13:30:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',54 from dual union all
select 39982,'1976-W13','1976-01-06 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 39983,'1976-W130','1976-01-27 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 39984,'1976-W1300','1976-09-15 14:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',330,'',57 from dual union all
select 39985,'1976-W1301','1976-09-15 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 39986,'1976-W1302','1976-09-15 15:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 39987,'1976-W1303','1976-09-15 16:24:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 39988,'1976-W1304','1976-09-15 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 39989,'1976-W1305','1976-09-15 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 3999,'1956-A319','1956-08-23 00:00:00','military missile','unknown',339,'1792','Corporal TA-73','','','Test',1895,'',40 from dual union all
select 39990,'1976-W1306','1976-09-15 17:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 39991,'1976-W1307','1976-09-15 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 39992,'1976-W1308','1976-09-15 20:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',63 from dual union all
select 39993,'1976-W1309','1976-09-15 21:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 39994,'1976-W131','1976-01-27 20:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 39995,'1976-W1310','1976-09-16 15:21:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',51 from dual union all
select 39996,'1976-W1311','1976-09-16 16:55:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 39997,'1976-W1312','1976-09-17 00:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 39998,'1976-W1313','1976-09-17 12:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 39999,'1976-W1314','1976-09-17 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 4,'1942-A04','1942-11-28 00:00:00','military missile','failure',8,'7','','','','Test',675,'',5 from dual union all
select 40,'1944-A04','1944-03-08 00:00:00','military missile','success',1519,'3','','','','Test',673,'',1 from dual union all
select 400,'1944-S364','1944-11-27 20:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 4000,'1956-A32','1956-03-09 00:00:00','atmospheric rocket','failure',554,'','Test','','','Test',1821,'',1 from dual union all
select 40000,'1976-W1315','1976-09-17 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 40001,'1976-W1316','1976-09-17 16:21:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 40002,'1976-W1317','1976-09-17 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 40003,'1976-W1318','1976-09-17 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 40004,'1976-W1319','1976-09-17 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 40005,'1976-W132','1976-01-28 01:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 40006,'1976-W1320','1976-09-17 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 40007,'1976-W1321','1976-09-19 07:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',70 from dual union all
select 40008,'1976-W1322','1976-09-20 06:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',70 from dual union all
select 40009,'1976-W1323','1976-09-20 12:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 4001,'1956-A320','1956-08-24 00:00:00','military missile','unknown',339,'1706','Corporal TA-71','','','Test',1895,'',40 from dual union all
select 40010,'1976-W1324','1976-09-20 14:35:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',53 from dual union all
select 40011,'1976-W1325','1976-09-20 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 40012,'1976-W1326','1976-09-20 16:07:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 40013,'1976-W1327','1976-09-20 16:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 40014,'1976-W1328','1976-09-20 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 40015,'1976-W1329','1976-09-20 18:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 40016,'1976-W133','1976-01-28 13:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 40017,'1976-W1330','1976-09-21 02:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1632,'USHAKOV',61 from dual union all
select 40018,'1976-W1331','1976-09-21 06:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',65 from dual union all
select 40019,'1976-W1332','1976-09-21 14:03:00','suborbital rocket','unknown',1281,'TM1-8169','','','','Aeron',1047,'',70 from dual union all
select 4002,'1956-A321','1956-09-04 00:00:00','military missile','unknown',339,'1826','Corporal TA-76','','','Test',1895,'',40 from dual union all
select 40020,'1976-W1333','1976-09-21 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 40021,'1976-W1334','1976-09-21 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 40022,'1976-W1335','1976-09-22 07:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',77 from dual union all
select 40023,'1976-W1336','1976-09-22 12:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 40024,'1976-W1337','1976-09-22 13:49:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',54 from dual union all
select 40025,'1976-W1338','1976-09-22 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',60 from dual union all
select 40026,'1976-W1339','1976-09-22 16:00:00','suborbital rocket','unknown',1284,'T 1-8401','','','','Aeron',1821,'',70 from dual union all
select 40027,'1976-W134','1976-01-28 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',62 from dual union all
select 40028,'1976-W1340','1976-09-22 16:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',308,'',57 from dual union all
select 40029,'1976-W1341','1976-09-22 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 4003,'1956-A322','1956-09-05 00:00:00','military missile','unknown',339,'1819','Corporal EU','','','Test',1895,'',40 from dual union all
select 40030,'1976-W1342','1976-09-22 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 40031,'1976-W1343','1976-09-22 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',78 from dual union all
select 40032,'1976-W1344','1976-09-22 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 40033,'1976-W1345','1976-09-22 20:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 40034,'1976-W1346','1976-09-23 01:30:00','suborbital rocket','success',761,'MT135P T16','','','','Aeron?',1476,'',50 from dual union all
select 40035,'1976-W1347','1976-09-23 01:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 40036,'1976-W1348','1976-09-23 14:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',72 from dual union all
select 40037,'1976-W1349','1976-09-23 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 40038,'1976-W135','1976-01-28 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 40039,'1976-W1350','1976-09-23 15:30:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',54 from dual union all
select 4004,'1956-A323','1956-09-07 00:00:00','military missile','unknown',339,'1791','Corporal TA-74','','','Test',1895,'',40 from dual union all
select 40040,'1976-W1351','1976-09-23 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 40041,'1976-W1352','1976-09-24 00:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 40042,'1976-W1353','1976-09-24 06:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',61 from dual union all
select 40043,'1976-W1354','1976-09-24 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 40044,'1976-W1355','1976-09-24 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 40045,'1976-W1356','1976-09-24 16:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 40046,'1976-W1357','1976-09-24 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 40047,'1976-W1358','1976-09-24 16:34:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 40048,'1976-W1359','1976-09-24 17:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 40049,'1976-W136','1976-01-28 16:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 4005,'1956-A324','1956-09-10 00:00:00','military missile','unknown',339,'1837','Corporal EU AES-29-A','','','Test',1895,'',40 from dual union all
select 40050,'1976-W1360','1976-09-24 17:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',68 from dual union all
select 40051,'1976-W1361','1976-09-24 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 40052,'1976-W1362','1976-09-25 01:30:00','suborbital rocket','success',761,'MT135P T17','','','','Aeron?',1476,'',50 from dual union all
select 40053,'1976-W1363','1976-09-25 06:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',67 from dual union all
select 40054,'1976-W1364','1976-09-25 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 40055,'1976-W1365','1976-09-26 06:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',67 from dual union all
select 40056,'1976-W1366','1976-09-26 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 40057,'1976-W1367','1976-09-27 07:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',67 from dual;
insert into LAUNCH
select 40058,'1976-W1368','1976-09-27 12:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 40059,'1976-W1369','1976-09-27 14:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',69 from dual union all
select 4006,'1956-A325','1956-09-11 00:00:00','military missile','unknown',339,'1910','Corporal T UK-2','','','Test',1895,'',40 from dual union all
select 40060,'1976-W137','1976-01-28 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 40061,'1976-W1370','1976-09-27 14:45:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',55 from dual union all
select 40062,'1976-W1371','1976-09-27 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 40063,'1976-W1372','1976-09-27 16:10:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 40064,'1976-W1373','1976-09-27 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 40065,'1976-W1374','1976-09-27 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 40066,'1976-W1375','1976-09-27 21:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 40067,'1976-W1376','1976-09-27 22:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 40068,'1976-W1377','1976-09-28 07:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',68 from dual union all
select 40069,'1976-W1378','1976-09-28 15:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',56 from dual union all
select 4007,'1956-A326','1956-09-11 00:00:00','military missile','unknown',339,'1825','Corporal TA-75','','','Test',1895,'',40 from dual union all
select 40070,'1976-W1379','1976-09-28 19:47:00','suborbital rocket','unknown',1284,'TM1-8521','','','','Aeron',1047,'',70 from dual union all
select 40071,'1976-W138','1976-01-28 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 40072,'1976-W1380','1976-09-28 20:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 40073,'1976-W1381','1976-09-28 22:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',66 from dual union all
select 40074,'1976-W1382','1976-09-29 00:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',65 from dual union all
select 40075,'1976-W1383','1976-09-29 03:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1632,'USHAKOV',61 from dual union all
select 40076,'1976-W1384','1976-09-29 07:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',66 from dual union all
select 40077,'1976-W1385','1976-09-29 12:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 40078,'1976-W1386','1976-09-29 14:00:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',62 from dual union all
select 40079,'1976-W1387','1976-09-29 14:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',56 from dual union all
select 4008,'1956-A327','1956-09-18 00:00:00','military missile','unknown',339,'1906','Corporal T UK-1','','','Test',1895,'',40 from dual union all
select 40080,'1976-W1388','1976-09-29 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 40081,'1976-W1389','1976-09-29 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 40082,'1976-W139','1976-01-28 20:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 40083,'1976-W1390','1976-09-29 16:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',68 from dual union all
select 40084,'1976-W1391','1976-09-29 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',79 from dual union all
select 40085,'1976-W1392','1976-09-29 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 40086,'1976-W1393','1976-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 40087,'1976-W1394','1976-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 40088,'1976-W1395','1976-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 40089,'1976-W1396','1976-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 4009,'1956-A328','1956-10-12 00:00:00','military missile','unknown',339,'1730','Corporal EU AES-30','','','Test',1895,'',40 from dual union all
select 40090,'1976-W1397','1976-10-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 40091,'1976-W1398','1976-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40092,'1976-W1399','1976-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40093,'1976-W14','1976-01-06 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',67 from dual union all
select 40094,'1976-W140','1976-01-28 21:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 40095,'1976-W1400','1976-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40096,'1976-W1401','1976-09-30 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 40097,'1976-W1402','1976-09-30 06:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',67 from dual union all
select 40098,'1976-W1403','1976-09-30 15:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',59 from dual union all
select 40099,'1976-W1404','1976-09-30 15:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 401,'1944-S365','1944-12-06 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 4010,'1956-A329','1956-10-23 00:00:00','military missile','unknown',339,'1363','Corporal Chem 9','','','Test',1895,'',40 from dual union all
select 40100,'1976-W1405','1976-09-30 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 40101,'1976-W1406','1976-09-30 22:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 40102,'1976-W1407','1976-10-01 06:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',66 from dual union all
select 40103,'1976-W1408','1976-10-01 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 40104,'1976-W1409','1976-10-01 15:34:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',59 from dual union all
select 40105,'1976-W141','1976-01-28 23:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 40106,'1976-W1410','1976-10-01 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',51 from dual union all
select 40107,'1976-W1411','1976-10-01 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 40108,'1976-W1412','1976-10-01 16:36:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',65 from dual union all
select 40109,'1976-W1413','1976-10-01 19:15:00','suborbital rocket','unknown',1284,'TM1-8522','','','','Aeron',1047,'',74 from dual union all
select 4011,'1956-A33','1956-03-15 00:00:00','military missile','success',255,'623-14','','','','Test',254,'',20 from dual union all
select 40110,'1976-W1414','1976-10-01 20:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',68 from dual union all
select 40111,'1976-W1415','1976-10-01 22:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 40112,'1976-W1416','1976-10-02 06:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',66 from dual union all
select 40113,'1976-W1417','1976-10-04 12:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 40114,'1976-W1418','1976-10-04 14:20:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',64 from dual union all
select 40115,'1976-W1419','1976-10-04 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 40116,'1976-W142','1976-01-29 01:51:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 40117,'1976-W1420','1976-10-04 16:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 40118,'1976-W1421','1976-10-04 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',51 from dual union all
select 40119,'1976-W1422','1976-10-04 16:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 4012,'1956-A330','1956-10-23 00:00:00','military missile','unknown',339,'1364','Corporal Chem 10','','','Test',1895,'',40 from dual union all
select 40120,'1976-W1423','1976-10-04 18:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',63 from dual union all
select 40121,'1976-W1424','1976-10-04 20:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',62 from dual union all
select 40122,'1976-W1425','1976-10-04 23:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 40123,'1976-W1426','1976-10-05 02:00:00','suborbital rocket','success',738,'','','','','Aeron',1632,'USHAKOV',60 from dual union all
select 40124,'1976-W1427','1976-10-05 15:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',64 from dual union all
select 40125,'1976-W1428','1976-10-06 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 40126,'1976-W1429','1976-10-06 14:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',330,'',63 from dual union all
select 40127,'1976-W143','1976-01-29 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 40128,'1976-W1430','1976-10-06 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.284','','','Aeron',1467,'',66 from dual union all
select 40129,'1976-W1431','1976-10-06 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',76 from dual union all
select 4013,'1956-A331','1956-10-24 00:00:00','military missile','unknown',339,'1895','Corporal TA-77','','','Test',1895,'',40 from dual union all
select 40130,'1976-W1432','1976-10-06 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 40131,'1976-W1433','1976-10-06 15:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',51 from dual union all
select 40132,'1976-W1434','1976-10-06 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 40133,'1976-W1435','1976-10-06 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 40134,'1976-W1436','1976-10-06 16:30:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',53 from dual union all
select 40135,'1976-W1437','1976-10-06 18:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 40136,'1976-W1438','1976-10-06 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 40137,'1976-W1439','1976-10-06 19:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',60 from dual union all
select 40138,'1976-W144','1976-01-29 19:25:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 40139,'1976-W1440','1976-10-07 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 4014,'1956-A332','1956-10-25 00:00:00','military missile','unknown',339,'1893','Corporal TA-78','','','Test',1895,'',40 from dual union all
select 40140,'1976-W1441','1976-10-07 12:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 40141,'1976-W1442','1976-10-07 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 40142,'1976-W1443','1976-10-07 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 40143,'1976-W1444','1976-10-07 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',63 from dual union all
select 40144,'1976-W1445','1976-10-07 21:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 40145,'1976-W1446','1976-10-08 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',56 from dual union all
select 40146,'1976-W1447','1976-10-08 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 40147,'1976-W1448','1976-10-08 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual;
insert into LAUNCH
select 40148,'1976-W1449','1976-10-08 16:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 40149,'1976-W145','1976-01-30 00:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 4015,'1956-A333','1956-10-26 00:00:00','military missile','unknown',339,'1799','Corporal TA-79','','','Test',1895,'',40 from dual union all
select 40150,'1976-W1450','1976-10-08 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 40151,'1976-W1451','1976-10-11 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 40152,'1976-W1452','1976-10-11 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',61 from dual union all
select 40153,'1976-W1453','1976-10-12 15:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 40154,'1976-W1454','1976-10-12 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 40155,'1976-W1455','1976-10-12 20:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',60 from dual union all
select 40156,'1976-W1456','1976-10-12 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 40157,'1976-W1457','1976-10-13 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1632,'USHAKOV',61 from dual union all
select 40158,'1976-W1458','1976-10-13 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 40159,'1976-W1459','1976-10-13 14:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',330,'',63 from dual union all
select 4016,'1956-A334','1956-11-15 00:00:00','military missile','unknown',339,'1894','Corporal TA-80','','','Test',1895,'',40 from dual union all
select 40160,'1976-W146','1976-01-30 01:25:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 40161,'1976-W1460','1976-10-13 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 40162,'1976-W1461','1976-10-13 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 40163,'1976-W1462','1976-10-13 16:32:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 40164,'1976-W1463','1976-10-13 16:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',63 from dual union all
select 40165,'1976-W1464','1976-10-13 16:55:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 40166,'1976-W1465','1976-10-13 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 40167,'1976-W1466','1976-10-13 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 40168,'1976-W1467','1976-10-13 18:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 40169,'1976-W1468','1976-10-13 20:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',65 from dual union all
select 4017,'1956-A335','1956-11-16 00:00:00','military missile','unknown',339,'1962','Corporal TA-81','','','Test',1895,'',40 from dual union all
select 40170,'1976-W1469','1976-10-13 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',64 from dual union all
select 40171,'1976-W147','1976-01-30 01:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 40172,'1976-W1470','1976-10-13 21:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 40173,'1976-W1471','1976-10-13 21:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 40174,'1976-W1472','1976-10-14 13:04:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',65 from dual union all
select 40175,'1976-W1473','1976-10-15 12:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 40176,'1976-W1474','1976-10-15 14:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 40177,'1976-W1475','1976-10-15 14:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 40178,'1976-W1476','1976-10-15 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 40179,'1976-W1477','1976-10-15 16:23:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 4018,'1956-A336','1956-11-20 00:00:00','military missile','unknown',339,'1836','Corporal EU AFS-31','','','Test',1895,'',40 from dual union all
select 40180,'1976-W1478','1976-10-15 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 40181,'1976-W1479','1976-10-15 19:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',58 from dual union all
select 40182,'1976-W148','1976-01-30 16:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 40183,'1976-W1480','1976-10-15 20:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',59 from dual union all
select 40184,'1976-W1481','1976-10-15 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 40185,'1976-W1482','1976-10-15 23:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 40186,'1976-W1483','1976-10-16 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1297,'PRILIV',61 from dual union all
select 40187,'1976-W1484','1976-10-18 14:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 40188,'1976-W1485','1976-10-18 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 40189,'1976-W1486','1976-10-18 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 4019,'1956-A337','1956-11-27 00:00:00','military missile','unknown',339,'1960','Corporal TA-83','','','Test',1895,'',40 from dual union all
select 40190,'1976-W1487','1976-10-18 16:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',58 from dual union all
select 40191,'1976-W1488','1976-10-18 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',51 from dual union all
select 40192,'1976-W1489','1976-10-19 00:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 40193,'1976-W149','1976-01-30 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 40194,'1976-W1490','1976-10-19 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1297,'PRILIV',64 from dual union all
select 40195,'1976-W1491','1976-10-19 18:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',62 from dual union all
select 40196,'1976-W1492','1976-10-20 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',78 from dual union all
select 40197,'1976-W1493','1976-10-20 13:44:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',57 from dual union all
select 40198,'1976-W1494','1976-10-20 14:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',63 from dual union all
select 40199,'1976-W1495','1976-10-20 14:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 402,'1944-S366','1944-12-06 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 4020,'1956-A338','1956-11-29 00:00:00','military missile','unknown',339,'1961','Corporal TA-84','','','Test',1895,'',40 from dual union all
select 40200,'1976-W1496','1976-10-20 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 40201,'1976-W1497','1976-10-20 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 40202,'1976-W1498','1976-10-20 16:10:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 40203,'1976-W1499','1976-10-20 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',57 from dual union all
select 40204,'1976-W15','1976-01-06 15:45:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 40205,'1976-W150','1976-01-30 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 40206,'1976-W1500','1976-10-20 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 40207,'1976-W1501','1976-10-20 19:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',60 from dual union all
select 40208,'1976-W1502','1976-10-20 22:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 40209,'1976-W1503','1976-10-21 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 4021,'1956-A339','1956-11-30 00:00:00','military missile','unknown',339,'1957','Corporal TA-82','','','Test',1895,'',40 from dual union all
select 40210,'1976-W1504','1976-10-21 01:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 40211,'1976-W1505','1976-10-21 17:00:00','suborbital rocket','unknown',1284,'T 1-8402','','','','Aeron',1821,'',70 from dual union all
select 40212,'1976-W1506','1976-10-21 18:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 40213,'1976-W1507','1976-10-22 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 40214,'1976-W1508','1976-10-22 14:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',61 from dual union all
select 40215,'1976-W1509','1976-10-22 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 40216,'1976-W151','1976-01-30 17:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',64 from dual union all
select 40217,'1976-W1510','1976-10-22 15:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 40218,'1976-W1511','1976-10-22 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 40219,'1976-W1512','1976-10-22 18:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 4022,'1956-A34','1956-03-23 00:00:00','military missile','success',771,'GM-54-22','','','','Test',222,'',12 from dual union all
select 40220,'1976-W1513','1976-10-22 22:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 40221,'1976-W1514','1976-10-22 23:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 40222,'1976-W1515','1976-10-23 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',64 from dual union all
select 40223,'1976-W1516','1976-10-24 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',76 from dual union all
select 40224,'1976-W1517','1976-10-25 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 40225,'1976-W1518','1976-10-25 16:32:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 40226,'1976-W1519','1976-10-26 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 40227,'1976-W152','1976-01-30 19:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 40228,'1976-W1520','1976-10-26 17:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 40229,'1976-W1521','1976-10-26 19:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',59 from dual union all
select 4023,'1956-A340','1956-12-04 00:00:00','military missile','unknown',339,'1835','Corporal AEB-32','','','Test',1895,'',40 from dual union all
select 40230,'1976-W1522','1976-10-26 20:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 40231,'1976-W1523','1976-10-26 21:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 40232,'1976-W1524','1976-10-26 22:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 40233,'1976-W1525','1976-10-26 22:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',76 from dual union all
select 40234,'1976-W1526','1976-10-27 13:19:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',58 from dual union all
select 40235,'1976-W1527','1976-10-27 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 40236,'1976-W1528','1976-10-27 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',63 from dual union all
select 40237,'1976-W1529','1976-10-27 14:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual;
insert into LAUNCH
select 40238,'1976-W153','1976-02-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 40239,'1976-W1530','1976-10-27 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 4024,'1956-A341','1956-01-13 00:00:00','atmospheric rocket','success',1527,'48-TV18-1','','','','Test',1895,'B-29',10 from dual union all
select 40240,'1976-W1531','1976-10-27 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 40241,'1976-W1532','1976-10-27 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 40242,'1976-W1533','1976-10-27 16:44:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',54 from dual union all
select 40243,'1976-W1534','1976-10-27 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 40244,'1976-W1535','1976-10-27 20:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',56 from dual union all
select 40245,'1976-W1536','1976-10-27 21:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',53 from dual union all
select 40246,'1976-W1537','1976-10-28 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',61 from dual union all
select 40247,'1976-W1538','1976-10-28 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',76 from dual union all
select 40248,'1976-W1539','1976-10-28 12:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 40249,'1976-W154','1976-02-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 4025,'1956-A342','1956-02-10 00:00:00','atmospheric rocket','success',1527,'49','','','','Test',1895,'B-29',10 from dual union all
select 40250,'1976-W1540','1976-10-28 23:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 40251,'1976-W1541','1976-10-29 14:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 40252,'1976-W1542','1976-10-29 14:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 40253,'1976-W1543','1976-10-29 15:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',54 from dual union all
select 40254,'1976-W1544','1976-10-29 15:30:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',62 from dual union all
select 40255,'1976-W1545','1976-10-29 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 40256,'1976-W1546','1976-10-29 16:18:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 40257,'1976-W1547','1976-10-29 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 40258,'1976-W1548','1976-10-29 19:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 40259,'1976-W1549','1976-10-29 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 4026,'1956-A343','1956-03-09 00:00:00','atmospheric rocket','success',1527,'50','','','','Test',1895,'B-29',10 from dual union all
select 40260,'1976-W155','1976-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40261,'1976-W1550','1976-10-29 21:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',67 from dual union all
select 40262,'1976-W1551','1976-10-29 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',69 from dual union all
select 40263,'1976-W1552','1976-10-30 00:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 40264,'1976-W1553','1976-11-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 40265,'1976-W1554','1976-11-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 40266,'1976-W1555','1976-11-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 40267,'1976-W1556','1976-11-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 40268,'1976-W1557','1976-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40269,'1976-W1558','1976-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 4027,'1956-A344','1956-03-21 00:00:00','atmospheric rocket','success',1527,'TV-16','','','','Test',1895,'B-29',10 from dual union all
select 40270,'1976-W1559','1976-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40271,'1976-W156','1976-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40272,'1976-W1560','1976-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40273,'1976-W1561','1976-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40274,'1976-W1562','1976-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40275,'1976-W1563','1976-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40276,'1976-W1564','1976-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40277,'1976-W1565','1976-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40278,'1976-W1566','1976-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40279,'1976-W1567','1976-11-01 12:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 4028,'1956-A345','1956-07-31 00:00:00','atmospheric rocket','success',1527,'','','','','Test',1895,'B-29',10 from dual union all
select 40280,'1976-W1568','1976-11-01 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',57 from dual union all
select 40281,'1976-W1569','1976-11-01 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 40282,'1976-W157','1976-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40283,'1976-W1570','1976-11-01 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',54 from dual union all
select 40284,'1976-W1571','1976-11-01 16:47:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 40285,'1976-W1572','1976-11-01 17:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 40286,'1976-W1573','1976-11-01 18:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 40287,'1976-W1574','1976-11-01 18:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',61 from dual union all
select 40288,'1976-W1575','1976-11-01 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',65 from dual union all
select 40289,'1976-W1576','1976-11-01 23:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 4029,'1956-A346','1956-08-18 00:00:00','atmospheric rocket','success',1528,'F-191','','','','Test',1895,'B-29',10 from dual union all
select 40290,'1976-W1577','1976-11-01 23:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 40291,'1976-W1578','1976-11-02 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 40292,'1976-W1579','1976-11-03 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 40293,'1976-W158','1976-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40294,'1976-W1580','1976-11-03 14:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',52 from dual union all
select 40295,'1976-W1581','1976-11-03 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',73 from dual union all
select 40296,'1976-W1582','1976-11-03 16:42:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 40297,'1976-W1583','1976-11-03 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',59 from dual union all
select 40298,'1976-W1584','1976-11-03 17:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 40299,'1976-W1585','1976-11-03 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 403,'1944-S367','1944-12-09 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1627,'',90 from dual union all
select 4030,'1956-A347','1956-08-18 00:00:00','atmospheric rocket','success',1528,'F-243','','','','Test',1895,'B-29',10 from dual union all
select 40300,'1976-W1586','1976-11-03 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 40301,'1976-W1587','1976-11-03 21:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 40302,'1976-W1588','1976-11-03 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',61 from dual union all
select 40303,'1976-W1589','1976-11-04 12:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 40304,'1976-W159','1976-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40305,'1976-W1590','1976-11-04 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 40306,'1976-W1591','1976-11-04 15:43:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',54 from dual union all
select 40307,'1976-W1592','1976-11-05 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',61 from dual union all
select 40308,'1976-W1593','1976-11-05 15:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 40309,'1976-W1594','1976-11-05 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 4031,'1956-A348','1956-08-18 00:00:00','atmospheric rocket','success',1528,'F-192','','','','Test',1895,'B-29',10 from dual union all
select 40310,'1976-W1595','1976-11-05 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',51 from dual union all
select 40311,'1976-W1596','1976-11-05 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 40312,'1976-W1597','1976-11-05 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',67 from dual union all
select 40313,'1976-W1598','1976-11-05 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 40314,'1976-W1599','1976-11-05 23:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 40315,'1976-W16','1976-01-06 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 40316,'1976-W160','1976-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40317,'1976-W1600','1976-11-08 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 40318,'1976-W1601','1976-11-08 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',64 from dual union all
select 40319,'1976-W1602','1976-11-08 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 4032,'1956-A349','1956-08-18 00:00:00','atmospheric rocket','success',1528,'F-190','','','','Test',1895,'B-29',10 from dual union all
select 40320,'1976-W1603','1976-11-08 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',67 from dual union all
select 40321,'1976-W1604','1976-11-08 16:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 40322,'1976-W1605','1976-11-08 16:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 40323,'1976-W1606','1976-11-08 18:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 40324,'1976-W1607','1976-11-08 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',55 from dual union all
select 40325,'1976-W1608','1976-11-08 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 40326,'1976-W1609','1976-11-08 23:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 40327,'1976-W161','1976-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual;
insert into LAUNCH
select 40328,'1976-W1610','1976-11-09 16:54:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 40329,'1976-W1611','1976-11-10 12:15:00','suborbital rocket','unknown',1284,'T 1-8404','','','','Aeron',1821,'',70 from dual union all
select 4033,'1956-A35','1956-03-23 00:00:00','military missile','success',771,'GM-54-26','','','','Test',222,'',12 from dual union all
select 40330,'1976-W1612','1976-11-10 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 40331,'1976-W1613','1976-11-10 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',51 from dual union all
select 40332,'1976-W1614','1976-11-10 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 40333,'1976-W1615','1976-11-10 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 40334,'1976-W1616','1976-11-10 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 40335,'1976-W1617','1976-11-10 16:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 40336,'1976-W1618','1976-11-10 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 40337,'1976-W1619','1976-11-10 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',61 from dual union all
select 40338,'1976-W162','1976-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40339,'1976-W1620','1976-11-10 19:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 4034,'1956-A350','1956-10-18 00:00:00','atmospheric rocket','success',1527,'','','','','Test',1895,'B-29',10 from dual union all
select 40340,'1976-W1621','1976-11-10 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',67 from dual union all
select 40341,'1976-W1622','1976-11-10 21:00:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',77 from dual union all
select 40342,'1976-W1623','1976-11-10 21:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 40343,'1976-W1624','1976-11-11 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 40344,'1976-W1625','1976-11-11 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 40345,'1976-W1626','1976-11-11 12:16:00','suborbital rocket','unknown',1284,'T 1-8407','','','','Aeron',1821,'',71 from dual union all
select 40346,'1976-W1627','1976-11-11 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 40347,'1976-W1628','1976-11-11 14:00:00','suborbital rocket','unknown',1284,'T 1-8403','','','','Test',1821,'',71 from dual union all
select 40348,'1976-W1629','1976-11-11 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 40349,'1976-W163','1976-01-31 07:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 4035,'1956-A351','1956-10-31 00:00:00','atmospheric rocket','success',1527,'','','','','Test',1895,'B-29',10 from dual union all
select 40350,'1976-W1630','1976-11-12 12:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 40351,'1976-W1631','1976-11-12 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',62 from dual union all
select 40352,'1976-W1632','1976-11-12 15:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',58 from dual union all
select 40353,'1976-W1633','1976-11-12 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',63 from dual union all
select 40354,'1976-W1634','1976-11-12 16:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 40355,'1976-W1635','1976-11-12 16:41:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 40356,'1976-W1636','1976-11-12 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 40357,'1976-W1637','1976-11-12 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 40358,'1976-W1638','1976-11-12 23:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 40359,'1976-W1639','1976-11-13 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 4036,'1956-A352','1956-11-09 00:00:00','atmospheric rocket','success',1527,'','','','','Test',1895,'B-29',10 from dual union all
select 40360,'1976-W164','1976-02-02 14:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 40361,'1976-W1640','1976-11-13 06:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',67 from dual union all
select 40362,'1976-W1641','1976-11-14 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1297,'PRILIV',60 from dual union all
select 40363,'1976-W1642','1976-11-15 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',58 from dual union all
select 40364,'1976-W1643','1976-11-15 14:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',68 from dual union all
select 40365,'1976-W1644','1976-11-15 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',53 from dual union all
select 40366,'1976-W1645','1976-11-15 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 40367,'1976-W1646','1976-11-15 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 40368,'1976-W1647','1976-11-15 18:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',64 from dual union all
select 40369,'1976-W1648','1976-11-15 23:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 4037,'1956-A353','1956-11-30 00:00:00','atmospheric rocket','success',1527,'','','','','Test',1895,'B-29',10 from dual union all
select 40370,'1976-W1649','1976-11-16 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 40371,'1976-W165','1976-02-02 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 40372,'1976-W1650','1976-11-16 05:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',62 from dual union all
select 40373,'1976-W1651','1976-11-17 12:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 40374,'1976-W1652','1976-11-17 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 40375,'1976-W1653','1976-11-17 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 40376,'1976-W1654','1976-11-17 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 40377,'1976-W1655','1976-11-17 15:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',55 from dual union all
select 40378,'1976-W1656','1976-11-17 16:21:00','suborbital rocket','unknown',1284,'T 1-8405','','','','Aeron',1821,'',68 from dual union all
select 40379,'1976-W1657','1976-11-17 16:33:00','suborbital rocket','unknown',1284,'T 1-8406','','','','Aeron',1821,'',70 from dual union all
select 4038,'1956-A354','1956-12-06 00:00:00','atmospheric rocket','success',1527,'','','','','Test',1895,'B-29',10 from dual union all
select 40380,'1976-W1658','1976-11-17 16:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 40381,'1976-W1659','1976-11-17 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 40382,'1976-W166','1976-02-02 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 40383,'1976-W1660','1976-11-17 17:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 40384,'1976-W1661','1976-11-17 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 40385,'1976-W1662','1976-11-17 21:25:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',61 from dual union all
select 40386,'1976-W1663','1976-11-17 21:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 40387,'1976-W1664','1976-11-18 04:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',69 from dual union all
select 40388,'1976-W1665','1976-11-18 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1297,'PRILIV',60 from dual union all
select 40389,'1976-W1666','1976-11-18 12:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 4039,'1956-A355','1956-12-21 00:00:00','atmospheric rocket','success',1527,'','','','','Test',1895,'B-29',10 from dual union all
select 40390,'1976-W1667','1976-11-18 16:01:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 40391,'1976-W1668','1976-11-19 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 40392,'1976-W1669','1976-11-19 15:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 40393,'1976-W167','1976-02-02 19:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 40394,'1976-W1670','1976-11-19 16:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',58 from dual union all
select 40395,'1976-W1671','1976-11-19 16:43:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 40396,'1976-W1672','1976-11-19 16:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 40397,'1976-W1673','1976-11-19 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 40398,'1976-W1674','1976-11-19 18:57:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',57 from dual union all
select 40399,'1976-W1675','1976-11-19 21:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 404,'1944-S368','1944-12-14 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 4040,'1956-A356','1956-05-19 00:00:00','atmospheric rocket','success',563,'','HTV Atlas 107A','','','Heat transfer',1895,'',10 from dual union all
select 40400,'1976-W1676','1976-11-21 06:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',68 from dual union all
select 40401,'1976-W1677','1976-11-22 12:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 40402,'1976-W1678','1976-11-22 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 40403,'1976-W1679','1976-11-22 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 40404,'1976-W168','1976-02-02 20:03:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 40405,'1976-W1680','1976-11-22 16:41:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 40406,'1976-W1681','1976-11-22 16:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 40407,'1976-W1682','1976-11-22 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',59 from dual union all
select 40408,'1976-W1683','1976-11-22 19:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 40409,'1976-W1684','1976-11-22 22:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 4041,'1956-A357','1956-05-19 00:00:00','atmospheric rocket','success',563,'','HTV Atlas 107A','','','Heat transfer',1895,'',10 from dual union all
select 40410,'1976-W1685','1976-11-22 23:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 40411,'1976-W1686','1976-11-23 05:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',67 from dual union all
select 40412,'1976-W1687','1976-11-23 16:00:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 40413,'1976-W1688','1976-11-23 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 40414,'1976-W1689','1976-11-24 06:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',62 from dual union all
select 40415,'1976-W169','1976-02-02 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 40416,'1976-W1690','1976-11-24 13:14:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',70 from dual union all
select 40417,'1976-W1691','1976-11-24 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual;
insert into LAUNCH
select 40418,'1976-W1692','1976-11-24 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 40419,'1976-W1693','1976-11-24 15:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 4042,'1956-A358','1956-06-09 00:00:00','atmospheric rocket','success',563,'DT-2','HTV Atlas DT-2','','','Heat transfer',1895,'',10 from dual union all
select 40420,'1976-W1694','1976-11-24 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 40421,'1976-W1695','1976-11-24 16:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 40422,'1976-W1696','1976-11-24 17:19:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 40423,'1976-W1697','1976-11-24 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 40424,'1976-W1698','1976-11-24 18:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 40425,'1976-W1699','1976-11-24 19:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',52 from dual union all
select 40426,'1976-W17','1976-01-06 18:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 40427,'1976-W170','1976-02-02 23:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 40428,'1976-W1700','1976-11-24 21:00:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',71 from dual union all
select 40429,'1976-W1701','1976-11-24 23:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 4043,'1956-A359','1956-12-11 00:00:00','suborbital rocket','failure',716,'WASP','1','','','Test',1895,'',1 from dual union all
select 40430,'1976-W1702','1976-11-25 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',77 from dual union all
select 40431,'1976-W1703','1976-11-25 14:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',308,'',59 from dual union all
select 40432,'1976-W1704','1976-11-25 16:07:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 40433,'1976-W1705','1976-11-25 22:31:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',78 from dual union all
select 40434,'1976-W1706','1976-11-26 09:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',68 from dual union all
select 40435,'1976-W1707','1976-11-26 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',63 from dual union all
select 40436,'1976-W1708','1976-11-26 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 40437,'1976-W1709','1976-11-26 15:40:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',52 from dual union all
select 40438,'1976-W171','1976-02-03 13:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 40439,'1976-W1710','1976-11-26 16:12:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 4044,'1956-A36','1956-03-26 00:00:00','military missile','success',771,'GM-54-31','','','','Test',222,'',12 from dual union all
select 40440,'1976-W1711','1976-11-26 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 40441,'1976-W1712','1976-11-26 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 40442,'1976-W1713','1976-11-27 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 40443,'1976-W1714','1976-11-29 05:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',62 from dual union all
select 40444,'1976-W1715','1976-11-29 14:17:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',62 from dual union all
select 40445,'1976-W1716','1976-11-29 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 40446,'1976-W1717','1976-11-29 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 40447,'1976-W1718','1976-11-29 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 40448,'1976-W1719','1976-11-29 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 40449,'1976-W172','1976-02-03 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 4045,'1956-A360','1956-12-11 00:00:00','suborbital rocket','failure',716,'WASP','3','','','Test',1895,'',1 from dual union all
select 40450,'1976-W1720','1976-11-29 18:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 40451,'1976-W1721','1976-11-29 22:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 40452,'1976-W1722','1976-11-29 23:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 40453,'1976-W1723','1976-11-30 12:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',58 from dual union all
select 40454,'1976-W1724','1976-11-30 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 40455,'1976-W1725','1976-12-01 05:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',58 from dual union all
select 40456,'1976-W1726','1976-12-01 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 40457,'1976-W1727','1976-12-01 14:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',276,'',72 from dual union all
select 40458,'1976-W1728','1976-12-01 14:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',58 from dual union all
select 40459,'1976-W1729','1976-12-01 15:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 4046,'1956-A361','1956-12-12 00:00:00','suborbital rocket','failure',716,'WASP','11','','','Test',1895,'',1 from dual union all
select 40460,'1976-W173','1976-02-03 16:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',51 from dual union all
select 40461,'1976-W1730','1976-12-01 15:17:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',64 from dual union all
select 40462,'1976-W1731','1976-12-01 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 40463,'1976-W1732','1976-12-01 16:09:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 40464,'1976-W1733','1976-12-01 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',54 from dual union all
select 40465,'1976-W1734','1976-12-01 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 40466,'1976-W1735','1976-12-01 17:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',79 from dual union all
select 40467,'1976-W1736','1976-12-01 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 40468,'1976-W1737','1976-12-01 23:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',62 from dual union all
select 40469,'1976-W1738','1976-12-02 15:44:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 4047,'1956-A362','1956-12-19 00:00:00','military missile','success',845,'1810-50','','','','Test',1895,'',10 from dual union all
select 40470,'1976-W1739','1976-12-02 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',79 from dual union all
select 40471,'1976-W174','1976-02-03 16:49:00','suborbital rocket','unknown',1284,'FLIGHT 122','','','','Aeron',35,'',54 from dual union all
select 40472,'1976-W1740','1976-12-02 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 40473,'1976-W1741','1976-12-02 19:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',59 from dual union all
select 40474,'1976-W1742','1976-12-02 21:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 40475,'1976-W1743','1976-12-03 14:15:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',66 from dual union all
select 40476,'1976-W1744','1976-12-03 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 40477,'1976-W1745','1976-12-03 14:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 40478,'1976-W1746','1976-12-03 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',66 from dual union all
select 40479,'1976-W1747','1976-12-03 15:51:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 4048,'1956-A363','1956-01-11 00:00:00','military missile','success',975,'S1-7','LKI 1A','','','Test',636,'',40 from dual union all
select 40480,'1976-W1748','1976-12-03 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 40481,'1976-W1749','1976-12-03 19:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',65 from dual union all
select 40482,'1976-W175','1976-02-03 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 40483,'1976-W1750','1976-12-03 21:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 40484,'1976-W1751','1976-12-04 12:53:00','suborbital rocket','success',1284,'FLIGHT-141','AE-5 Cal','','','Aeron',35,'',51 from dual union all
select 40485,'1976-W1752','1976-12-04 14:04:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',73 from dual union all
select 40486,'1976-W1753','1976-12-06 14:10:00','suborbital rocket','success',1284,'FLIGHT-142','Nimbus 4 Cal','','','Aeron',35,'',55 from dual union all
select 40487,'1976-W1754','1976-12-06 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 40488,'1976-W1755','1976-12-06 15:30:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',59 from dual union all
select 40489,'1976-W1756','1976-12-06 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 4049,'1956-A364','1956-01-14 00:00:00','military missile','success',975,'S1-8','LKI 1A','','','Test',636,'',40 from dual union all
select 40490,'1976-W1757','1976-12-06 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 40491,'1976-W1758','1976-12-06 18:00:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',56 from dual union all
select 40492,'1976-W1759','1976-12-06 18:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',58 from dual union all
select 40493,'1976-W176','1976-02-03 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 40494,'1976-W1760','1976-12-06 18:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 40495,'1976-W1761','1976-12-06 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 40496,'1976-W1762','1976-12-06 20:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',67 from dual union all
select 40497,'1976-W1763','1976-12-06 23:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 40498,'1976-W1764','1976-12-07 12:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 40499,'1976-W1765','1976-12-07 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 405,'1944-S369','1944-12-14 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1627,'',90 from dual union all
select 4050,'1956-A365','1956-02-06 00:00:00','military missile','success',975,'S1-19','LKI 1A','','','Test',636,'',40 from dual union all
select 40500,'1976-W1766','1976-12-07 17:33:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 40501,'1976-W1767','1976-12-07 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 40502,'1976-W1768','1976-12-08 12:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 40503,'1976-W1769','1976-12-08 15:00:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',72 from dual union all
select 40504,'1976-W177','1976-02-04 12:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 40505,'1976-W1770','1976-12-08 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 40506,'1976-W1771','1976-12-08 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 40507,'1976-W1772','1976-12-08 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual;
insert into LAUNCH
select 40508,'1976-W1773','1976-12-08 17:37:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 40509,'1976-W1774','1976-12-08 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 4051,'1956-A366','1956-02-08 00:00:00','military missile','success',975,'S1-11','LKI 1A','','','Test',636,'',40 from dual union all
select 40510,'1976-W1775','1976-12-08 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 40511,'1976-W1776','1976-12-08 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',50 from dual union all
select 40512,'1976-W1777','1976-12-08 20:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',58 from dual union all
select 40513,'1976-W1778','1976-12-09 00:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 40514,'1976-W1779','1976-12-09 12:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',70 from dual union all
select 40515,'1976-W178','1976-02-04 14:31:00','suborbital rocket','unknown',1284,'FLIGHT 123','','','','Aeron',35,'',55 from dual union all
select 40516,'1976-W1780','1976-12-09 14:11:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',308,'',58 from dual union all
select 40517,'1976-W1781','1976-12-09 14:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',64 from dual union all
select 40518,'1976-W1782','1976-12-09 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',61 from dual union all
select 40519,'1976-W1783','1976-12-09 15:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',51 from dual union all
select 4052,'1956-A367','1956-02-09 00:00:00','military missile','success',975,'S1-12','LKI 1A','','','Test',636,'',40 from dual union all
select 40520,'1976-W1784','1976-12-09 15:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 40521,'1976-W1785','1976-12-09 15:37:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 40522,'1976-W1786','1976-12-09 15:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',60 from dual union all
select 40523,'1976-W1787','1976-12-09 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 40524,'1976-W1788','1976-12-09 18:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',54 from dual union all
select 40525,'1976-W1789','1976-12-10 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 40526,'1976-W179','1976-02-04 15:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',61 from dual union all
select 40527,'1976-W1790','1976-12-10 15:34:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 40528,'1976-W1791','1976-12-10 15:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 40529,'1976-W1792','1976-12-10 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 4053,'1956-A368','1956-02-11 00:00:00','military missile','success',975,'S1-2','LKI 1A','','','Test',636,'',40 from dual union all
select 40530,'1976-W1793','1976-12-10 18:59:59','suborbital rocket','success',1284,'','','','','Meteo',1895,'',72 from dual union all
select 40531,'1976-W1794','1976-12-10 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',67 from dual union all
select 40532,'1976-W1795','1976-12-10 21:00:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',70 from dual union all
select 40533,'1976-W1796','1976-12-10 22:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 40534,'1976-W1797','1976-12-11 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 40535,'1976-W1798','1976-12-12 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 40536,'1976-W1799','1976-12-13 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',74 from dual union all
select 40537,'1976-W18','1976-01-07 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 40538,'1976-W180','1976-02-04 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 40539,'1976-W1800','1976-12-13 14:45:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',65 from dual union all
select 4054,'1956-A369','1956-04-20 00:00:00','military missile','failure',971,'A1-1','Production launch','','','Test',636,'',10 from dual union all
select 40540,'1976-W1801','1976-12-13 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 40541,'1976-W1802','1976-12-13 15:37:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 40542,'1976-W1803','1976-12-13 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 40543,'1976-W1804','1976-12-13 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 40544,'1976-W1805','1976-12-13 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',55 from dual union all
select 40545,'1976-W1806','1976-12-13 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 40546,'1976-W1807','1976-12-13 19:08:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 40547,'1976-W1808','1976-12-13 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',66 from dual union all
select 40548,'1976-W1809','1976-12-13 22:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 40549,'1976-W181','1976-02-04 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 4055,'1956-A37','1956-03-26 00:00:00','military missile','success',771,'GM-54-19','','','','Test',222,'',12 from dual union all
select 40550,'1976-W1810','1976-12-14 12:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 40551,'1976-W1811','1976-12-14 15:37:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',54 from dual union all
select 40552,'1976-W1812','1976-12-14 15:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 40553,'1976-W1813','1976-12-14 16:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 40554,'1976-W1814','1976-12-14 17:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 40555,'1976-W1815','1976-12-14 20:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',58 from dual union all
select 40556,'1976-W1816','1976-12-14 20:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 40557,'1976-W1817','1976-12-15 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',50 from dual union all
select 40558,'1976-W1818','1976-12-15 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 40559,'1976-W1819','1976-12-15 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 4056,'1956-A370','1956-06-28 00:00:00','military missile','failure',971,'A1-5','Production launch','','','Test',636,'',30 from dual union all
select 40560,'1976-W182','1976-02-04 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 40561,'1976-W1820','1976-12-15 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 40562,'1976-W1821','1976-12-15 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 40563,'1976-W1822','1976-12-15 15:37:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 40564,'1976-W1823','1976-12-15 16:14:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',66 from dual union all
select 40565,'1976-W1824','1976-12-15 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 40566,'1976-W1825','1976-12-15 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',61 from dual union all
select 40567,'1976-W1826','1976-12-16 00:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 40568,'1976-W1827','1976-12-16 12:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 40569,'1976-W1828','1976-12-16 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 4057,'1956-A371','1956-08-28 00:00:00','military missile','success',975,'A1-13','LKI 1B','','','Test',636,'',40 from dual union all
select 40570,'1976-W1829','1976-12-16 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 40571,'1976-W183','1976-02-04 19:41:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',67 from dual union all
select 40572,'1976-W1830','1976-12-16 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 40573,'1976-W1831','1976-12-16 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 40574,'1976-W1832','1976-12-16 20:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',64 from dual union all
select 40575,'1976-W1833','1976-12-16 21:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 40576,'1976-W1834','1976-12-17 01:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 40577,'1976-W1835','1976-12-17 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 40578,'1976-W1836','1976-12-17 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 40579,'1976-W1837','1976-12-17 15:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 4058,'1956-A372','1956-09-13 00:00:00','military missile','success',975,'A1-16','LKI 1B','','','Test',636,'',40 from dual union all
select 40580,'1976-W1838','1976-12-17 15:37:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 40581,'1976-W1839','1976-12-17 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 40582,'1976-W184','1976-02-04 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 40583,'1976-W1840','1976-12-17 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 40584,'1976-W1841','1976-12-18 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 40585,'1976-W1842','1976-12-20 12:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 40586,'1976-W1843','1976-12-20 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',62 from dual union all
select 40587,'1976-W1844','1976-12-20 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 40588,'1976-W1845','1976-12-20 14:26:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',67 from dual union all
select 40589,'1976-W1846','1976-12-20 15:38:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 4059,'1956-A373','1956-09-20 00:00:00','military missile','success',975,'A1-17','LKI 1B','','','Test',636,'',40 from dual union all
select 40590,'1976-W1847','1976-12-20 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 40591,'1976-W1848','1976-12-20 16:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 40592,'1976-W1849','1976-12-20 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',65 from dual union all
select 40593,'1976-W185','1976-02-05 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 40594,'1976-W1850','1976-12-20 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 40595,'1976-W1851','1976-12-20 21:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',58 from dual union all
select 40596,'1976-W1852','1976-12-20 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 40597,'1976-W1853','1976-12-20 23:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 40598,'1976-W1854','1976-12-21 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1297,'PRILIV',65 from dual;
insert into LAUNCH
select 40599,'1976-W1855','1976-12-21 17:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',60 from dual union all
select 406,'1944-S37','1944-08-14 19:28:00','military missile','success',1470,'18309','Ma127','','','Training',661,'',90 from dual union all
select 4060,'1956-A38','1956-03-26 00:00:00','military missile','success',771,'GM-54-35','','','','Test',222,'',12 from dual union all
select 40600,'1976-W1856','1976-12-21 20:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 40601,'1976-W1857','1976-12-22 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 40602,'1976-W1858','1976-12-22 14:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 40603,'1976-W1859','1976-12-22 15:08:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',57 from dual union all
select 40604,'1976-W186','1976-02-05 01:36:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 40605,'1976-W1860','1976-12-22 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 40606,'1976-W1861','1976-12-22 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',62 from dual union all
select 40607,'1976-W1862','1976-12-22 16:03:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',68 from dual union all
select 40608,'1976-W1863','1976-12-22 16:04:00','suborbital rocket','unknown',1284,'T 1-8408','','','','Aeron',1821,'',70 from dual union all
select 40609,'1976-W1864','1976-12-22 16:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 4061,'1956-A39','1956-03-28 00:00:00','military missile','success',771,'GM-54-21','','','','Test',222,'',12 from dual union all
select 40610,'1976-W1865','1976-12-22 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 40611,'1976-W1866','1976-12-22 17:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 40612,'1976-W1867','1976-12-22 17:43:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 40613,'1976-W1868','1976-12-22 21:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 40614,'1976-W1869','1976-12-22 22:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 40615,'1976-W187','1976-02-05 15:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',62 from dual union all
select 40616,'1976-W1870','1976-12-23 15:36:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 40617,'1976-W1871','1976-12-23 19:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 40618,'1976-W1872','1976-12-24 00:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 40619,'1976-W1873','1976-12-24 14:35:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',63 from dual union all
select 4062,'1956-A40','1956-03-28 00:00:00','military missile','success',771,'GM-54-30','','','','Test',222,'',12 from dual union all
select 40620,'1976-W1874','1976-12-24 15:40:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 40621,'1976-W1875','1976-12-24 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 40622,'1976-W1876','1976-12-27 16:55:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',56 from dual union all
select 40623,'1976-W1877','1976-12-27 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 40624,'1976-W1878','1976-12-27 23:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 40625,'1976-W1879','1976-12-28 15:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 40626,'1976-W188','1976-02-05 16:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 40627,'1976-W1880','1976-12-28 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 40628,'1976-W1881','1976-12-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',79 from dual union all
select 40629,'1976-W1882','1976-12-29 14:30:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',69 from dual union all
select 4063,'1956-A41','1956-03-28 00:00:00','military missile','success',771,'GM-54-25','','','','Test',222,'',12 from dual union all
select 40630,'1976-W1883','1976-12-29 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',58 from dual union all
select 40631,'1976-W1884','1976-12-29 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 40632,'1976-W1885','1976-12-29 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 40633,'1976-W1886','1976-12-29 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 40634,'1976-W1887','1976-12-29 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 40635,'1976-W1888','1976-12-29 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 40636,'1976-W1889','1976-12-29 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 40637,'1976-W189','1976-02-05 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 40638,'1976-W1890','1976-12-29 21:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 40639,'1976-W1891','1976-12-30 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 4064,'1956-A42','1956-03-28 00:00:00','military missile','success',771,'GM-54-36','','','','Test',222,'',12 from dual union all
select 40640,'1976-W1892','1976-12-30 12:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',58 from dual union all
select 40641,'1976-W1893','1976-12-30 14:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',330,'',64 from dual union all
select 40642,'1976-W1894','1976-12-30 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 40643,'1976-W1895','1976-12-30 21:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 40644,'1976-W1896','1976-12-31 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1297,'PRILIV',60 from dual union all
select 40645,'1976-W1897','1976-12-31 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1297,'PRILIV',61 from dual union all
select 40646,'1976-W1898','1976-12-31 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 40647,'1976-W1899','1976-01-22 18:35:00','suborbital rocket','success',1284,'','Conductivity','','','Ionos',1821,'',70 from dual union all
select 40648,'1976-W19','1976-01-07 15:59:59','suborbital rocket','success',716,'','Starute,Datasonde','','','Meteo',428,'',59 from dual union all
select 40649,'1976-W190','1976-02-05 19:17:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',53 from dual union all
select 4065,'1956-A43','1956-04-04 00:00:00','military missile','success',771,'GM-54-28','','','','Test',222,'',12 from dual union all
select 40650,'1976-W1900','1976-01-22 18:35:00','suborbital rocket','success',1061,'','Blunt Probe','','','Ionos',1821,'',70 from dual union all
select 40651,'1976-W1901','1976-01-30 20:25:00','suborbital rocket','success',1061,'','Gerdien Condenser','','','Ionos',1821,'',70 from dual union all
select 40652,'1976-W1902','1976-01-02 15:59:00','suborbital rocket','success',1284,'INTA SLW-7601','Winter Anomaly','','','Meteo',396,'',70 from dual union all
select 40653,'1976-W191','1976-02-05 19:28:00','suborbital rocket','unknown',1284,'FLIGHT 124','','','','Aeron',35,'',54 from dual union all
select 40654,'1976-W192','1976-02-06 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',76 from dual union all
select 40655,'1976-W193','1976-02-06 12:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 40656,'1976-W194','1976-02-06 16:07:00','suborbital rocket','unknown',1284,'FLIGHT 125','','','','Aeron',35,'',54 from dual union all
select 40657,'1976-W195','1976-02-06 16:08:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',65 from dual union all
select 40658,'1976-W196','1976-02-06 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 40659,'1976-W197','1976-02-06 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 4066,'1956-A44','1956-04-06 00:00:00','military missile','success',771,'GM-54-34','','','','Test',222,'',12 from dual union all
select 40660,'1976-W198','1976-02-06 20:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 40661,'1976-W199','1976-02-06 22:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 40662,'1976-W20','1976-01-07 18:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 40663,'1976-W200','1976-02-07 07:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 40664,'1976-W201','1976-02-09 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 40665,'1976-W202','1976-02-09 16:18:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',65 from dual union all
select 40666,'1976-W203','1976-02-09 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 40667,'1976-W204','1976-02-09 18:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 40668,'1976-W205','1976-02-09 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 40669,'1976-W206','1976-02-09 21:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 4067,'1956-A45','1956-04-06 00:00:00','military missile','success',771,'GM-54-23','','','','Test',222,'',12 from dual union all
select 40670,'1976-W207','1976-02-09 23:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 40671,'1976-W208','1976-02-10 07:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1757,'VOLNA',57 from dual union all
select 40672,'1976-W209','1976-02-10 11:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 40673,'1976-W21','1976-01-07 18:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',55 from dual union all
select 40674,'1976-W210','1976-02-10 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 40675,'1976-W211','1976-02-10 22:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 40676,'1976-W212','1976-02-11 12:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 40677,'1976-W213','1976-02-11 16:07:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 40678,'1976-W214','1976-02-11 16:24:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',67 from dual union all
select 40679,'1976-W215','1976-02-11 17:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 4068,'1956-A46','1956-04-09 00:00:00','atmospheric rocket','success',545,'','Test','','','Test',1821,'',10 from dual union all
select 40680,'1976-W216','1976-02-11 18:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 40681,'1976-W217','1976-02-11 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 40682,'1976-W218','1976-02-11 19:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 40683,'1976-W219','1976-02-11 20:14:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',68 from dual union all
select 40684,'1976-W22','1976-01-07 19:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 40685,'1976-W220','1976-02-12 07:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1757,'VOLNA',58 from dual union all
select 40686,'1976-W221','1976-02-12 15:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',59 from dual union all
select 40687,'1976-W222','1976-02-12 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 40688,'1976-W223','1976-02-12 15:55:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',67 from dual;
insert into LAUNCH
select 40689,'1976-W224','1976-02-12 18:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 4069,'1956-A47','1956-04-14 00:00:00','military missile','success',255,'623-15','','','','Test',254,'',20 from dual union all
select 40690,'1976-W225','1976-02-12 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 40691,'1976-W226','1976-02-12 21:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 40692,'1976-W227','1976-02-12 23:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 40693,'1976-W228','1976-02-13 05:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 40694,'1976-W229','1976-02-13 06:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',72 from dual union all
select 40695,'1976-W23','1976-01-07 20:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 40696,'1976-W230','1976-02-13 09:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1757,'VOLNA',59 from dual union all
select 40697,'1976-W231','1976-02-13 12:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 40698,'1976-W232','1976-02-13 15:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 40699,'1976-W233','1976-02-13 16:18:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',52 from dual union all
select 407,'1944-S370','1944-12-16 13:15:00','military missile','success',1470,'','Antwerp','','','Weapon',1729,'',90 from dual union all
select 4070,'1956-A48','1956-05-01 00:00:00','suborbital rocket','success',122,'','Redwing 1','','','Aeronomy',199,'',30 from dual union all
select 40700,'1976-W234','1976-02-13 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',54 from dual union all
select 40701,'1976-W235','1976-02-13 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 40702,'1976-W236','1976-02-13 22:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',76 from dual union all
select 40703,'1976-W237','1976-02-14 00:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 40704,'1976-W238','1976-02-14 23:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 40705,'1976-W239','1976-02-16 06:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 40706,'1976-W24','1976-01-07 20:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',70 from dual union all
select 40707,'1976-W240','1976-02-16 07:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1757,'VOLNA',61 from dual union all
select 40708,'1976-W241','1976-02-16 16:04:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 40709,'1976-W242','1976-02-16 17:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 4071,'1956-A49','1956-05-01 00:00:00','suborbital rocket','success',122,'','Redwing 2 (to 40)','','','Aeronomy',199,'',30 from dual union all
select 40710,'1976-W243','1976-02-17 02:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 40711,'1976-W244','1976-02-17 09:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1757,'VOLNA',57 from dual union all
select 40712,'1976-W245','1976-02-17 11:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 40713,'1976-W246','1976-02-17 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',52 from dual union all
select 40714,'1976-W247','1976-02-17 16:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 40715,'1976-W248','1976-02-17 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 40716,'1976-W249','1976-02-17 23:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 40717,'1976-W25','1976-01-07 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 40718,'1976-W250','1976-02-18 12:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 40719,'1976-W251','1976-02-18 14:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 4072,'1956-A50','1956-05-01 22:05:00','suborbital rocket','failure',49,'NRL 39','RV-N-13a','','','Test',1869,'',4 from dual union all
select 40720,'1976-W252','1976-02-18 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',79 from dual union all
select 40721,'1976-W253','1976-02-18 15:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 40722,'1976-W254','1976-02-18 16:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 40723,'1976-W255','1976-02-18 16:21:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 40724,'1976-W256','1976-02-18 16:30:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 40725,'1976-W257','1976-02-18 16:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 40726,'1976-W258','1976-02-18 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 40727,'1976-W259','1976-02-18 21:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 40728,'1976-W26','1976-01-08 01:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 40729,'1976-W260','1976-02-18 22:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 4073,'1956-A51','1956-05-02 00:00:00','military missile','failure',771,'GM-11065','','','','Test',222,'',12 from dual union all
select 40730,'1976-W261','1976-02-19 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 40731,'1976-W262','1976-02-19 15:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',63 from dual union all
select 40732,'1976-W263','1976-02-19 16:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 40733,'1976-W264','1976-02-19 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 40734,'1976-W265','1976-02-20 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 40735,'1976-W266','1976-02-20 16:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 40736,'1976-W267','1976-02-20 16:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 40737,'1976-W268','1976-02-20 17:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 40738,'1976-W269','1976-02-20 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 40739,'1976-W27','1976-01-08 14:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 4074,'1956-A52','1956-05-09 00:00:00','atmospheric rocket','success',835,'','Avro Arrow Model','','','Test',1821,'',10 from dual union all
select 40740,'1976-W270','1976-02-20 20:58:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 40741,'1976-W271','1976-02-20 22:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 40742,'1976-W272','1976-02-21 00:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',73 from dual union all
select 40743,'1976-W273','1976-02-21 11:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',69 from dual union all
select 40744,'1976-W274','1976-02-22 02:48:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 40745,'1976-W275','1976-02-23 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 40746,'1976-W276','1976-02-23 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 40747,'1976-W277','1976-02-23 17:13:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 40748,'1976-W278','1976-02-23 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',69 from dual union all
select 40749,'1976-W279','1976-02-23 22:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 4075,'1956-A53','1956-05-15 00:00:00','atmospheric rocket','success',835,'','Avro Arrow Model','','','Test',1821,'',10 from dual union all
select 40750,'1976-W28','1976-01-08 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',54 from dual union all
select 40751,'1976-W280','1976-02-23 23:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 40752,'1976-W281','1976-02-24 01:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 40753,'1976-W282','1976-02-24 02:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',68 from dual union all
select 40754,'1976-W283','1976-02-24 17:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 40755,'1976-W284','1976-02-24 17:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 40756,'1976-W285','1976-02-24 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 40757,'1976-W286','1976-02-25 00:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 40758,'1976-W287','1976-02-25 02:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',69 from dual union all
select 40759,'1976-W288','1976-02-25 10:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',68 from dual union all
select 4076,'1956-A54','1956-05-15 00:00:00','atmospheric rocket','success',835,'','F4H-1 Model','','','Test',1821,'',10 from dual union all
select 40760,'1976-W289','1976-02-25 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 40761,'1976-W29','1976-01-08 15:57:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 40762,'1976-W290','1976-02-25 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 40763,'1976-W291','1976-02-25 15:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',65 from dual union all
select 40764,'1976-W292','1976-02-25 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 40765,'1976-W293','1976-02-25 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 40766,'1976-W294','1976-02-25 20:24:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',64 from dual union all
select 40767,'1976-W295','1976-02-25 20:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',72 from dual union all
select 40768,'1976-W296','1976-02-25 21:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 40769,'1976-W297','1976-02-25 21:43:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',72 from dual union all
select 4077,'1956-A55','1956-05-20 00:00:00','military missile','success',771,'GM-11091','','','','Test',283,'',12 from dual union all
select 40770,'1976-W298','1976-02-26 02:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 40771,'1976-W299','1976-02-26 03:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 40772,'1976-W30','1976-01-08 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 40773,'1976-W300','1976-02-26 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 40774,'1976-W301','1976-02-26 20:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 40775,'1976-W302','1976-02-27 02:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 40776,'1976-W303','1976-02-27 07:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1757,'VOLNA',63 from dual union all
select 40777,'1976-W304','1976-02-27 10:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',69 from dual union all
select 40778,'1976-W305','1976-02-27 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual;
insert into LAUNCH
select 40779,'1976-W306','1976-02-27 15:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 4078,'1956-A56','1956-05-21 00:00:00','military missile','failure',255,'623-16','','','','Test',272,'',20 from dual union all
select 40780,'1976-W307','1976-02-27 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 40781,'1976-W308','1976-02-27 17:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 40782,'1976-W309','1976-02-27 18:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 40783,'1976-W31','1976-01-09 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 40784,'1976-W310','1976-02-27 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 40785,'1976-W311','1976-02-27 21:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',54 from dual union all
select 40786,'1976-W312','1976-02-27 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 40787,'1976-W313','1976-02-28 01:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 40788,'1976-W314','1976-02-28 07:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 40789,'1976-W315','1976-03-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 4079,'1956-A57','1956-06-01 00:00:00','suborbital rocket','success',122,'','Redwing 2 (to 40)','','','Aeronomy',199,'',30 from dual union all
select 40790,'1976-W316','1976-03-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 40791,'1976-W317','1976-03-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 40792,'1976-W318','1976-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40793,'1976-W319','1976-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40794,'1976-W32','1976-01-09 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 40795,'1976-W320','1976-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40796,'1976-W321','1976-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40797,'1976-W322','1976-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 40798,'1976-W323','1976-02-29 11:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',69 from dual union all
select 40799,'1976-W324','1976-03-01 05:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 408,'1944-S371','1944-12-16 14:55:00','military missile','success',1470,'','Antwerp','','','Weapon',1729,'',90 from dual union all
select 4080,'1956-A58','1956-06-01 00:00:00','suborbital rocket','success',716,'WASP 1','','','','Aeronomy',199,'DD-787',30 from dual union all
select 40800,'1976-W325','1976-03-01 11:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 40801,'1976-W326','1976-03-01 12:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 40802,'1976-W327','1976-03-01 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 40803,'1976-W328','1976-03-01 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 40804,'1976-W329','1976-03-01 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 40805,'1976-W33','1976-01-09 18:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 40806,'1976-W330','1976-03-01 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',56 from dual union all
select 40807,'1976-W331','1976-03-01 21:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 40808,'1976-W332','1976-03-01 23:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 40809,'1976-W333','1976-03-02 03:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',62 from dual union all
select 4081,'1956-A59','1956-06-01 00:00:00','suborbital rocket','success',716,'WASP 1','','','','Aeronomy',199,'DD-790',30 from dual union all
select 40810,'1976-W334','1976-03-02 05:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',69 from dual union all
select 40811,'1976-W335','1976-03-02 11:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 40812,'1976-W336','1976-03-02 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 40813,'1976-W337','1976-03-02 21:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 40814,'1976-W338','1976-03-03 09:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1757,'VOLNA',60 from dual union all
select 40815,'1976-W339','1976-03-03 12:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 40816,'1976-W34','1976-01-09 21:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 40817,'1976-W340','1976-03-03 16:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 40818,'1976-W341','1976-03-03 16:32:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 40819,'1976-W342','1976-03-03 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 4082,'1956-A60','1956-06-07 00:00:00','atmospheric rocket','success',852,'','E41 Model','','','Test',1821,'',10 from dual union all
select 40820,'1976-W343','1976-03-03 16:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 40821,'1976-W344','1976-03-03 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 40822,'1976-W345','1976-03-03 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 40823,'1976-W346','1976-03-03 17:52:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',63 from dual union all
select 40824,'1976-W347','1976-03-03 18:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',75 from dual union all
select 40825,'1976-W348','1976-03-03 18:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 40826,'1976-W349','1976-03-03 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 40827,'1976-W35','1976-01-09 22:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 40828,'1976-W350','1976-03-03 21:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',76 from dual union all
select 40829,'1976-W351','1976-03-03 21:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 4083,'1956-A61','1956-06-08 00:00:00','atmospheric rocket','success',555,'','Test','','','Test',1821,'',30 from dual union all
select 40830,'1976-W352','1976-03-03 23:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 40831,'1976-W353','1976-03-04 02:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 40832,'1976-W354','1976-03-04 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',51 from dual union all
select 40833,'1976-W355','1976-03-04 17:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1895,'',66 from dual union all
select 40834,'1976-W356','1976-03-04 21:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 40835,'1976-W357','1976-03-04 22:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 40836,'1976-W358','1976-03-05 02:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',61 from dual union all
select 40837,'1976-W359','1976-03-05 10:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1757,'VOLNA',61 from dual union all
select 40838,'1976-W36','1976-01-09 22:45:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',76 from dual union all
select 40839,'1976-W360','1976-03-05 10:30:00','suborbital rocket','success',1061,'','','','','Meteo',1895,'',64 from dual union all
select 4084,'1956-A62','1956-06-19 00:00:00','military missile','success',255,'623-17','','','','Test',254,'',20 from dual union all
select 40840,'1976-W361','1976-03-05 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 40841,'1976-W362','1976-03-05 15:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 40842,'1976-W363','1976-03-05 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 40843,'1976-W364','1976-03-05 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 40844,'1976-W365','1976-03-05 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 40845,'1976-W366','1976-03-06 10:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 40846,'1976-W367','1976-03-07 08:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1757,'VOLNA',59 from dual union all
select 40847,'1976-W368','1976-03-08 08:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1757,'VOLNA',60 from dual union all
select 40848,'1976-W369','1976-03-08 12:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 40849,'1976-W37','1976-01-12 16:07:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',65 from dual union all
select 4085,'1956-A63','1956-06-22 00:00:00','atmospheric rocket','success',1308,'','Inlet model','','','Test',1821,'',10 from dual union all
select 40850,'1976-W370','1976-03-08 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 40851,'1976-W371','1976-03-08 16:07:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 40852,'1976-W372','1976-03-08 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 40853,'1976-W373','1976-03-08 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 40854,'1976-W374','1976-03-08 18:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',66 from dual union all
select 40855,'1976-W375','1976-03-08 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 40856,'1976-W376','1976-03-08 21:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 40857,'1976-W377','1976-03-09 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 40858,'1976-W378','1976-03-09 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 40859,'1976-W379','1976-03-09 16:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 4086,'1956-A64','1956-06-22 19:42:00','suborbital rocket','failure',54,'NRL 22','D-region','','','Ionos',1869,'',5 from dual union all
select 40860,'1976-W38','1976-01-12 17:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 40861,'1976-W380','1976-03-09 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 40862,'1976-W381','1976-03-10 12:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 40863,'1976-W382','1976-03-10 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',61 from dual union all
select 40864,'1976-W383','1976-03-10 15:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 40865,'1976-W384','1976-03-10 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 40866,'1976-W385','1976-03-10 16:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 40867,'1976-W386','1976-03-10 16:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 40868,'1976-W387','1976-03-10 18:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual;
insert into LAUNCH
select 40869,'1976-W388','1976-03-10 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 4087,'1956-A65','1956-06-27 00:00:00','atmospheric rocket','success',442,'','MW-2 Model/T-40','','','Test',1821,'',10 from dual union all
select 40870,'1976-W389','1976-03-10 21:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 40871,'1976-W39','1976-01-12 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 40872,'1976-W390','1976-03-10 22:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 40873,'1976-W391','1976-03-11 09:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1757,'VOLNA',59 from dual union all
select 40874,'1976-W392','1976-03-11 10:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1757,'VOLNA',62 from dual union all
select 40875,'1976-W393','1976-03-11 10:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 40876,'1976-W394','1976-03-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',76 from dual union all
select 40877,'1976-W395','1976-03-11 15:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 40878,'1976-W396','1976-03-11 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 40879,'1976-W397','1976-03-11 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',74 from dual union all
select 4088,'1956-A66','1956-07-01 00:00:00','suborbital rocket','success',122,'','Redwing 2 (to 40)','','','Aeronomy',199,'',30 from dual union all
select 40880,'1976-W398','1976-03-12 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 40881,'1976-W399','1976-03-12 15:33:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 40882,'1976-W40','1976-01-12 18:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 40883,'1976-W400','1976-03-12 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 40884,'1976-W401','1976-03-12 16:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 40885,'1976-W402','1976-03-12 22:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 40886,'1976-W403','1976-03-12 23:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 40887,'1976-W404','1976-03-15 08:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 40888,'1976-W405','1976-03-15 09:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1757,'VOLNA',61 from dual union all
select 40889,'1976-W406','1976-03-15 10:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1757,'VOLNA',61 from dual union all
select 4089,'1956-A67','1956-07-01 00:00:00','suborbital rocket','success',716,'WASP 1','','','','Aeronomy',199,'DD-787',30 from dual union all
select 40890,'1976-W407','1976-03-15 12:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 40891,'1976-W408','1976-03-15 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',53 from dual union all
select 40892,'1976-W409','1976-03-15 16:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 40893,'1976-W41','1976-01-12 19:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',58 from dual union all
select 40894,'1976-W410','1976-03-15 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 40895,'1976-W411','1976-03-15 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 40896,'1976-W412','1976-03-15 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',74 from dual union all
select 40897,'1976-W413','1976-03-15 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 40898,'1976-W414','1976-03-15 23:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 40899,'1976-W415','1976-03-16 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 409,'1944-S372','1944-12-18 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1746,'',90 from dual union all
select 4090,'1956-A68','1956-07-01 00:00:00','suborbital rocket','success',716,'WASP 1','','','','Aeronomy',199,'DD-790',30 from dual union all
select 40900,'1976-W416','1976-03-16 18:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',75 from dual union all
select 40901,'1976-W417','1976-03-16 19:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 40902,'1976-W418','1976-03-17 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.241','','','Aeron',1467,'',78 from dual union all
select 40903,'1976-W419','1976-03-17 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',60 from dual union all
select 40904,'1976-W42','1976-01-12 19:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 40905,'1976-W420','1976-03-17 15:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',61 from dual union all
select 40906,'1976-W421','1976-03-17 16:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 40907,'1976-W422','1976-03-17 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 40908,'1976-W423','1976-03-17 17:30:00','suborbital rocket','unknown',1284,'T 1-8198','','','','Aeron',1821,'',67 from dual union all
select 40909,'1976-W424','1976-03-17 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 4091,'1956-A69','1956-07-05 00:00:00','atmospheric rocket','success',553,'','Heat Transfer','','','Test',1821,'',27 from dual union all
select 40910,'1976-W425','1976-03-17 17:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',51 from dual union all
select 40911,'1976-W426','1976-03-17 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',71 from dual union all
select 40912,'1976-W427','1976-03-17 19:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 40913,'1976-W428','1976-03-17 23:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 40914,'1976-W429','1976-03-18 05:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',68 from dual union all
select 40915,'1976-W43','1976-01-12 21:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 40916,'1976-W430','1976-03-18 15:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 40917,'1976-W431','1976-03-18 22:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',67 from dual union all
select 40918,'1976-W432','1976-03-19 08:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 40919,'1976-W433','1976-03-19 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 4092,'1956-A70','1956-07-10 00:00:00','military missile','success',1214,'N-69C 51-17577','','','','Test',224,'',10 from dual union all
select 40920,'1976-W434','1976-03-19 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 40921,'1976-W435','1976-03-19 15:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 40922,'1976-W436','1976-03-19 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 40923,'1976-W437','1976-03-19 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 40924,'1976-W438','1976-03-19 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1307,'',66 from dual union all
select 40925,'1976-W439','1976-03-19 19:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 40926,'1976-W44','1976-01-12 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 40927,'1976-W440','1976-03-19 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 40928,'1976-W441','1976-03-19 22:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 40929,'1976-W442','1976-03-20 03:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 4093,'1956-A71','1956-07-13 00:00:00','military missile','success',255,'623-18','','','','Test',254,'',20 from dual union all
select 40930,'1976-W443','1976-03-22 12:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 40931,'1976-W444','1976-03-22 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 40932,'1976-W445','1976-03-22 16:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 40933,'1976-W446','1976-03-22 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 40934,'1976-W447','1976-03-22 17:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 40935,'1976-W448','1976-03-22 18:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 40936,'1976-W449','1976-03-22 18:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 40937,'1976-W45','1976-01-13 13:08:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 40938,'1976-W450','1976-03-22 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 40939,'1976-W451','1976-03-22 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 4094,'1956-A72','1956-07-20 00:00:00','military missile','failure',970,'No 5301','','','','OT',636,'',45 from dual union all
select 40940,'1976-W452','1976-03-23 01:12:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 40941,'1976-W453','1976-03-23 05:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 40942,'1976-W454','1976-03-24 05:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 40943,'1976-W455','1976-03-24 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 40944,'1976-W456','1976-03-24 14:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',60 from dual union all
select 40945,'1976-W457','1976-03-24 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',69 from dual union all
select 40946,'1976-W458','1976-03-24 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',57 from dual union all
select 40947,'1976-W459','1976-03-24 15:57:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 40948,'1976-W46','1976-01-13 13:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 40949,'1976-W460','1976-03-24 16:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 4095,'1956-A73','1956-07-21 00:00:00','suborbital rocket','success',122,'','PMR Redwing 41','','','Aeronomy',199,'',30 from dual union all
select 40950,'1976-W461','1976-03-24 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 40951,'1976-W462','1976-03-24 18:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 40952,'1976-W463','1976-03-24 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 40953,'1976-W464','1976-03-24 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 40954,'1976-W465','1976-03-25 01:13:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 40955,'1976-W466','1976-03-25 12:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 40956,'1976-W467','1976-03-25 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 40957,'1976-W468','1976-03-25 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 40958,'1976-W469','1976-03-25 21:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual;
insert into LAUNCH
select 40959,'1976-W47','1976-01-13 16:36:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 4096,'1956-A74','1956-07-21 22:00:00','suborbital rocket','failure',370,'','NRL NN5.31','IGY NN5.31','','Aeron',1290,'LSD-18',11 from dual union all
select 40960,'1976-W470','1976-03-26 01:14:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 40961,'1976-W471','1976-03-26 05:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 40962,'1976-W472','1976-03-26 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 40963,'1976-W473','1976-03-26 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 40964,'1976-W474','1976-03-26 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 40965,'1976-W475','1976-03-26 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 40966,'1976-W476','1976-03-26 16:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 40967,'1976-W477','1976-03-26 20:13:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',70 from dual union all
select 40968,'1976-W478','1976-03-26 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 40969,'1976-W479','1976-03-26 21:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 4097,'1956-A75','1956-07-24 19:10:00','suborbital rocket','failure',370,'','NRL NN5.33','IGY NN5.33','','Solar UV/Solar X',1292,'LSD-18',11 from dual union all
select 40970,'1976-W48','1976-01-13 17:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 40971,'1976-W480','1976-03-27 01:26:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 40972,'1976-W481','1976-03-29 12:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 40973,'1976-W482','1976-03-29 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 40974,'1976-W483','1976-03-29 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 40975,'1976-W484','1976-03-29 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 40976,'1976-W485','1976-03-29 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',71 from dual union all
select 40977,'1976-W486','1976-03-29 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 40978,'1976-W487','1976-03-29 18:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 40979,'1976-W488','1976-03-29 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 4098,'1956-A76','1956-07-26 00:00:00','military missile','success',1214,'N-69C 52-10976','','','','Test',224,'',10 from dual union all
select 40980,'1976-W489','1976-03-29 20:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',59 from dual union all
select 40981,'1976-W49','1976-01-13 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 40982,'1976-W490','1976-03-30 06:17:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 40983,'1976-W491','1976-03-30 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 40984,'1976-W492','1976-03-31 01:25:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 40985,'1976-W493','1976-03-31 06:38:00','suborbital rocket','success',1281,'NASA 15.128UE','','','','Plasma',428,'',61 from dual union all
select 40986,'1976-W494','1976-03-31 07:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',68 from dual union all
select 40987,'1976-W495','1976-03-31 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 40988,'1976-W496','1976-03-31 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 40989,'1976-W497','1976-03-31 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 4099,'1956-A77','1956-07-28 02:12:00','suborbital rocket','failure',1521,'R-3','4203-4','','','Reentry Test',253,'',145 from dual union all
select 40990,'1976-W498','1976-03-31 16:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 40991,'1976-W499','1976-03-31 17:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 40992,'1976-W50','1976-01-13 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 40993,'1976-W500','1976-03-31 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 40994,'1976-W501','1976-03-31 22:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 40995,'1976-W502','1976-04-01 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 40996,'1976-W503','1976-04-01 22:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 40997,'1976-W504','1976-04-02 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',50 from dual union all
select 40998,'1976-W505','1976-04-02 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 40999,'1976-W506','1976-04-02 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 41,'1944-A05','1944-05-26 14:00:00','military missile','failure',8,'','','','','Test',675,'',1 from dual union all
select 410,'1944-S373','1944-12-23 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1740,'',90 from dual union all
select 4100,'1956-A78','1956-08-03 12:56:00','suborbital rocket','failure',51,'USAF 72','UV Spectrograph','','','Solar UV',640,'',2 from dual union all
select 41000,'1976-W507','1976-04-02 15:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',62 from dual union all
select 41001,'1976-W508','1976-04-02 16:32:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 41002,'1976-W509','1976-04-02 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 41003,'1976-W51','1976-01-14 12:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 41004,'1976-W510','1976-04-02 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 41005,'1976-W511','1976-04-02 21:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 41006,'1976-W512','1976-04-02 21:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',73 from dual union all
select 41007,'1976-W513','1976-04-02 22:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 41008,'1976-W514','1976-04-05 12:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 41009,'1976-W515','1976-04-05 17:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 4101,'1956-A79','1956-08-18 08:31:00','suborbital rocket','failure',1521,'R-4','4203-2','','','Reentry Test',253,'',1 from dual union all
select 41010,'1976-W516','1976-04-05 18:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 41011,'1976-W517','1976-04-05 18:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 41012,'1976-W518','1976-04-05 19:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',68 from dual union all
select 41013,'1976-W519','1976-04-05 23:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 41014,'1976-W52','1976-01-14 14:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 41015,'1976-W520','1976-04-06 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 41016,'1976-W521','1976-04-06 22:10:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 41017,'1976-W522','1976-04-07 13:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',51 from dual union all
select 41018,'1976-W523','1976-04-07 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 41019,'1976-W524','1976-04-07 16:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 4102,'1956-A80','1956-08-24 00:00:00','atmospheric rocket','failure',552,'','Heat Transfer','','','Reentry Test',1821,'',1 from dual union all
select 41020,'1976-W525','1976-04-07 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 41021,'1976-W526','1976-04-07 16:45:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 41022,'1976-W527','1976-04-07 17:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 41023,'1976-W528','1976-04-07 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 41024,'1976-W529','1976-04-07 17:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 41025,'1976-W53','1976-01-14 16:29:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 41026,'1976-W530','1976-04-07 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 41027,'1976-W531','1976-04-07 22:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 41028,'1976-W532','1976-04-08 01:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 41029,'1976-W533','1976-04-09 12:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 4103,'1956-A81','1956-08-29 00:00:00','military missile','success',771,'GM-55-528','','','','Test',222,'',12 from dual union all
select 41030,'1976-W534','1976-04-09 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 41031,'1976-W535','1976-04-09 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',75 from dual union all
select 41032,'1976-W536','1976-04-09 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 41033,'1976-W537','1976-04-09 16:41:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 41034,'1976-W538','1976-04-09 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 41035,'1976-W539','1976-04-09 17:57:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',58 from dual union all
select 41036,'1976-W54','1976-01-14 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 41037,'1976-W540','1976-04-09 18:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 41038,'1976-W541','1976-04-09 20:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 41039,'1976-W542','1976-04-09 21:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 4104,'1956-A82','1956-08-30 00:00:00','military missile','success',771,'GM-55-529','','','','Test',222,'',12 from dual union all
select 41040,'1976-W543','1976-04-12 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 41041,'1976-W544','1976-04-12 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 41042,'1976-W545','1976-04-12 15:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 41043,'1976-W546','1976-04-12 16:26:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 41044,'1976-W547','1976-04-12 17:13:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 41045,'1976-W548','1976-04-12 18:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 41046,'1976-W549','1976-04-12 18:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 41047,'1976-W55','1976-01-14 16:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual;
insert into LAUNCH
select 41048,'1976-W550','1976-04-12 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 41049,'1976-W551','1976-04-12 21:33:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 4105,'1956-A83','1956-08-31 00:00:00','military missile','success',1214,'N-69C 51-17571','','','','Test',224,'',10 from dual union all
select 41050,'1976-W552','1976-04-13 00:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',67 from dual union all
select 41051,'1976-W553','1976-04-13 11:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 41052,'1976-W554','1976-04-13 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',51 from dual union all
select 41053,'1976-W555','1976-04-13 23:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',61 from dual union all
select 41054,'1976-W556','1976-04-14 12:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 41055,'1976-W557','1976-04-14 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',52 from dual union all
select 41056,'1976-W558','1976-04-14 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 41057,'1976-W559','1976-04-14 15:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 41058,'1976-W56','1976-01-14 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 41059,'1976-W560','1976-04-14 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 4106,'1956-A84','1956-09-10 00:00:00','atmospheric rocket','success',862,'','Heat Transfer','','','Test',1821,'',10 from dual union all
select 41060,'1976-W561','1976-04-14 16:34:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',53 from dual union all
select 41061,'1976-W562','1976-04-14 16:43:00','suborbital rocket','unknown',1284,'T 1-8199','','','','Aeron',1821,'',69 from dual union all
select 41062,'1976-W563','1976-04-14 16:46:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 41063,'1976-W564','1976-04-14 17:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 41064,'1976-W565','1976-04-14 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 41065,'1976-W566','1976-04-14 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 41066,'1976-W567','1976-04-14 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 41067,'1976-W568','1976-04-14 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 41068,'1976-W569','1976-04-15 01:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 41069,'1976-W57','1976-01-14 19:10:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 4107,'1956-A85','1956-09-12 00:00:00','military missile','failure',771,'GM-55-527','','','','Test',222,'',12 from dual union all
select 41070,'1976-W570','1976-04-15 12:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 41071,'1976-W571','1976-04-15 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 41072,'1976-W572','1976-04-15 16:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 41073,'1976-W573','1976-04-16 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',54 from dual union all
select 41074,'1976-W574','1976-04-16 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',57 from dual union all
select 41075,'1976-W575','1976-04-16 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',60 from dual union all
select 41076,'1976-W576','1976-04-16 16:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 41077,'1976-W577','1976-04-16 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 41078,'1976-W578','1976-04-16 21:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 41079,'1976-W579','1976-04-19 12:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 4108,'1956-A86','1956-09-12 00:00:00','military missile','success',771,'GM-54-132','','','','Test',222,'',12 from dual union all
select 41080,'1976-W58','1976-01-14 21:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',66 from dual union all
select 41081,'1976-W580','1976-04-19 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 41082,'1976-W581','1976-04-19 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',77 from dual union all
select 41083,'1976-W582','1976-04-19 18:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 41084,'1976-W583','1976-04-19 18:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 41085,'1976-W584','1976-04-19 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 41086,'1976-W585','1976-04-19 21:48:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 41087,'1976-W586','1976-04-20 00:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 41088,'1976-W587','1976-04-20 01:49:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 41089,'1976-W588','1976-04-20 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 4109,'1956-A87','1956-09-13 00:00:00','military missile','success',1214,'N-69C 52-10961','','','','Test',224,'',10 from dual union all
select 41090,'1976-W589','1976-04-21 03:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1631,'USHAKOV',60 from dual union all
select 41091,'1976-W59','1976-01-14 21:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 41092,'1976-W590','1976-04-21 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 41093,'1976-W591','1976-04-21 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',62 from dual union all
select 41094,'1976-W592','1976-04-21 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 41095,'1976-W593','1976-04-21 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 41096,'1976-W594','1976-04-21 17:23:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 41097,'1976-W595','1976-04-21 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 41098,'1976-W596','1976-04-21 19:06:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 41099,'1976-W597','1976-04-21 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 411,'1944-S374','1944-12-26 19:21:00','military missile','success',1470,'','London/Islington','','','Weapon',1627,'',90 from dual union all
select 4110,'1956-A88','1956-09-20 00:00:00','military missile','success',771,'GM-54-13','','','','Test',222,'',12 from dual union all
select 41100,'1976-W598','1976-04-21 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',69 from dual union all
select 41101,'1976-W599','1976-04-21 22:20:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',68 from dual union all
select 41102,'1976-W60','1976-01-14 22:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',73 from dual union all
select 41103,'1976-W600','1976-04-22 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 41104,'1976-W601','1976-04-22 12:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 41105,'1976-W602','1976-04-22 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41106,'1976-W603','1976-04-22 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 41107,'1976-W604','1976-04-22 21:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',68 from dual union all
select 41108,'1976-W605','1976-04-22 21:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 41109,'1976-W606','1976-04-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 4111,'1956-A89','1956-09-21 00:00:00','suborbital rocket','failure',1324,'M1','','','','Test',1821,'',16 from dual union all
select 41110,'1976-W607','1976-04-23 15:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 41111,'1976-W608','1976-04-23 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 41112,'1976-W609','1976-04-23 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 41113,'1976-W61','1976-01-14 22:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',73 from dual union all
select 41114,'1976-W610','1976-04-23 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 41115,'1976-W611','1976-04-24 02:34:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 41116,'1976-W612','1976-04-26 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 41117,'1976-W613','1976-04-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 41118,'1976-W614','1976-04-26 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 41119,'1976-W615','1976-04-26 16:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 4112,'1956-A90','1956-09-24 00:02:00','atmospheric rocket','success',654,'K-1-1','K-I-S-1','','','Test',7,'',5 from dual union all
select 41120,'1976-W616','1976-04-26 17:47:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',64 from dual union all
select 41121,'1976-W617','1976-04-26 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 41122,'1976-W618','1976-04-26 21:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 41123,'1976-W619','1976-04-26 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 41124,'1976-W62','1976-01-15 09:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 41125,'1976-W620','1976-04-27 02:26:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 41126,'1976-W621','1976-04-27 17:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 41127,'1976-W622','1976-04-28 02:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1631,'USHAKOV',63 from dual union all
select 41128,'1976-W623','1976-04-28 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 41129,'1976-W624','1976-04-28 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 4113,'1956-A91','1956-09-26 00:00:00','military missile','success',771,'GM-54-133','','','','Test',222,'',12 from dual union all
select 41130,'1976-W625','1976-04-28 14:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41131,'1976-W626','1976-04-28 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 41132,'1976-W627','1976-04-28 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',51 from dual union all
select 41133,'1976-W628','1976-04-28 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 41134,'1976-W629','1976-04-28 16:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 41135,'1976-W63','1976-01-15 14:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',55 from dual union all
select 41136,'1976-W630','1976-04-28 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 41137,'1976-W631','1976-04-28 21:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',75 from dual;
insert into LAUNCH
select 41138,'1976-W632','1976-04-28 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 41139,'1976-W633','1976-04-29 00:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 4114,'1956-A92','1956-09-26 00:00:00','military missile','failure',771,'GM-54-230','','','','Test',222,'',12 from dual union all
select 41140,'1976-W634','1976-04-29 02:37:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 41141,'1976-W635','1976-04-29 12:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 41142,'1976-W636','1976-04-29 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 41143,'1976-W637','1976-04-29 16:52:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 41144,'1976-W638','1976-04-29 18:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 41145,'1976-W639','1976-04-29 21:13:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',57 from dual union all
select 41146,'1976-W64','1976-01-15 15:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 41147,'1976-W640','1976-05-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 41148,'1976-W641','1976-05-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 41149,'1976-W642','1976-05-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 4115,'1956-A93','1956-09-26 00:00:00','military missile','success',1214,'N-69C 52-17578','','','','Test',224,'',10 from dual union all
select 41150,'1976-W643','1976-05-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 41151,'1976-W644','1976-05-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 41152,'1976-W645','1976-05-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 41153,'1976-W646','1976-05-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 41154,'1976-W647','1976-05-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 41155,'1976-W648','1976-04-30 14:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41156,'1976-W649','1976-04-30 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 41157,'1976-W65','1976-01-15 16:01:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 41158,'1976-W650','1976-04-30 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 41159,'1976-W651','1976-04-30 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 4116,'1956-A94','1956-09-28 02:15:00','atmospheric rocket','success',654,'K-1-2','K-I-S-2','','','Test',7,'',5 from dual union all
select 41160,'1976-W652','1976-04-30 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 41161,'1976-W653','1976-04-30 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 41162,'1976-W654','1976-04-30 17:35:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 41163,'1976-W655','1976-04-30 19:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 41164,'1976-W656','1976-05-01 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 41165,'1976-W657','1976-05-01 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 41166,'1976-W658','1976-05-03 02:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1631,'USHAKOV',61 from dual union all
select 41167,'1976-W659','1976-05-03 15:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',76 from dual union all
select 41168,'1976-W66','1976-01-15 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 41169,'1976-W660','1976-05-03 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 4117,'1956-A95','1956-09-29 01:50:00','atmospheric rocket','success',654,'K-1-3','K-I-S-3','','','Test',7,'',5 from dual union all
select 41170,'1976-W661','1976-05-03 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 41171,'1976-W662','1976-05-03 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 41172,'1976-W663','1976-05-03 16:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 41173,'1976-W664','1976-05-03 16:42:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 41174,'1976-W665','1976-05-03 17:24:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',59 from dual union all
select 41175,'1976-W666','1976-05-03 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',55 from dual union all
select 41176,'1976-W667','1976-05-03 20:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 41177,'1976-W668','1976-05-03 21:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 41178,'1976-W669','1976-05-04 11:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 41179,'1976-W67','1976-01-15 22:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',75 from dual union all
select 4118,'1956-A96','1956-10-02 00:00:00','atmospheric rocket','success',835,'','F4H-1 Model','','','Test',1821,'',10 from dual union all
select 41180,'1976-W670','1976-05-04 13:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 41181,'1976-W671','1976-05-04 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 41182,'1976-W672','1976-05-04 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 41183,'1976-W673','1976-05-05 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 41184,'1976-W674','1976-05-05 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',52 from dual union all
select 41185,'1976-W675','1976-05-05 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 41186,'1976-W676','1976-05-05 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 41187,'1976-W677','1976-05-05 15:33:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',54 from dual union all
select 41188,'1976-W678','1976-05-05 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',74 from dual union all
select 41189,'1976-W679','1976-05-05 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 4119,'1956-A97','1956-10-02 00:00:00','military missile','success',1214,'N-69C 52-10979','','','','Test',224,'',10 from dual union all
select 41190,'1976-W68','1976-01-16 12:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 41191,'1976-W680','1976-05-05 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 41192,'1976-W681','1976-05-05 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',53 from dual union all
select 41193,'1976-W682','1976-05-05 17:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',68 from dual union all
select 41194,'1976-W683','1976-05-05 18:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 41195,'1976-W684','1976-05-05 18:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 41196,'1976-W685','1976-05-05 22:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',68 from dual union all
select 41197,'1976-W686','1976-05-06 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 41198,'1976-W687','1976-05-06 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 41199,'1976-W688','1976-05-06 17:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',57 from dual union all
select 412,'1944-S375','1944-12-28 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 4120,'1956-A98','1956-10-03 00:00:00','atmospheric rocket','success',835,'','B-58 Model','','','Test',1821,'',10 from dual union all
select 41200,'1976-W689','1976-05-07 13:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 41201,'1976-W69','1976-01-16 14:02:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 41202,'1976-W690','1976-05-07 13:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',69 from dual union all
select 41203,'1976-W691','1976-05-07 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',75 from dual union all
select 41204,'1976-W692','1976-05-07 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 41205,'1976-W693','1976-05-07 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 41206,'1976-W694','1976-05-07 16:40:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 41207,'1976-W695','1976-05-07 17:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 41208,'1976-W696','1976-05-07 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 41209,'1976-W697','1976-05-07 22:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 4121,'1956-A99','1956-10-04 00:00:00','military missile','success',771,'GM-54-14','','','','Test',222,'',12 from dual union all
select 41210,'1976-W698','1976-05-08 00:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 41211,'1976-W699','1976-05-08 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 41212,'1976-W70','1976-01-16 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',51 from dual union all
select 41213,'1976-W700','1976-05-09 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 41214,'1976-W701','1976-05-10 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 41215,'1976-W702','1976-05-10 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',50 from dual union all
select 41216,'1976-W703','1976-05-10 14:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 41217,'1976-W704','1976-05-10 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 41218,'1976-W705','1976-05-10 16:15:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 41219,'1976-W706','1976-05-10 16:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',60 from dual union all
select 4122,'1956-M01','1956-01-19 00:00:00','military missile','unknown',1162,'1','JPL-329 motor','','','Test',1852,'',50 from dual union all
select 41220,'1976-W707','1976-05-10 17:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41221,'1976-W708','1976-05-10 17:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 41222,'1976-W709','1976-05-10 22:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 41223,'1976-W71','1976-01-16 16:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 41224,'1976-W710','1976-05-10 22:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 41225,'1976-W711','1976-05-11 15:49:00','suborbital rocket','success',229,'AAF-VI-10','Sep system tes','','','Test',428,'',60 from dual union all
select 41226,'1976-W712','1976-05-11 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 41227,'1976-W713','1976-05-12 03:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1631,'USHAKOV',60 from dual;
insert into LAUNCH
select 41228,'1976-W714','1976-05-12 12:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 41229,'1976-W715','1976-05-12 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',54 from dual union all
select 4123,'1956-M02','1956-02-28 00:00:00','military missile','unknown',1162,'2','JPL-329','','','Test',1852,'',50 from dual union all
select 41230,'1976-W716','1976-05-12 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41231,'1976-W717','1976-05-12 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',64 from dual union all
select 41232,'1976-W718','1976-05-12 15:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 41233,'1976-W719','1976-05-12 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 41234,'1976-W72','1976-01-16 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 41235,'1976-W720','1976-05-12 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 41236,'1976-W721','1976-05-12 17:03:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 41237,'1976-W722','1976-05-12 17:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 41238,'1976-W723','1976-05-12 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 41239,'1976-W724','1976-05-12 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 4124,'1956-M03','1956-04-24 00:00:00','military missile','unknown',1162,'3','JPL-329','','','Test',1852,'',50 from dual union all
select 41240,'1976-W725','1976-05-13 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 41241,'1976-W726','1976-05-13 12:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 41242,'1976-W727','1976-05-13 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 41243,'1976-W728','1976-05-13 21:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',78 from dual union all
select 41244,'1976-W729','1976-05-14 14:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41245,'1976-W73','1976-01-16 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 41246,'1976-W730','1976-05-14 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 41247,'1976-W731','1976-05-14 15:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',60 from dual union all
select 41248,'1976-W732','1976-05-14 16:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 41249,'1976-W733','1976-05-14 17:07:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 4125,'1956-M04','1956-06-04 14:13:00','suborbital rocket','failure',49,'NRL 46','RV-N-13a','','','Solar UV',1869,'',58 from dual union all
select 41250,'1976-W734','1976-05-14 19:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 41251,'1976-W735','1976-05-15 00:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 41252,'1976-W736','1976-05-17 12:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 41253,'1976-W737','1976-05-17 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 41254,'1976-W738','1976-05-17 15:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',54 from dual union all
select 41255,'1976-W739','1976-05-17 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',61 from dual union all
select 41256,'1976-W74','1976-01-16 18:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41257,'1976-W740','1976-05-17 16:36:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 41258,'1976-W741','1976-05-17 16:58:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',62 from dual union all
select 41259,'1976-W742','1976-05-17 17:29:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 4126,'1956-M05','1956-10-11 00:00:00','suborbital rocket','success',545,'','Polished Cone RV','','','Reentry Test',1821,'',70 from dual union all
select 41260,'1976-W743','1976-05-17 17:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 41261,'1976-W744','1976-05-17 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 41262,'1976-W745','1976-05-17 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 41263,'1976-W746','1976-05-17 21:53:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',60 from dual union all
select 41264,'1976-W747','1976-05-18 08:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 41265,'1976-W748','1976-05-18 09:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 41266,'1976-W749','1976-05-18 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',76 from dual union all
select 41267,'1976-W75','1976-01-16 19:01:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 41268,'1976-W750','1976-05-18 21:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 41269,'1976-W751','1976-05-19 02:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1631,'USHAKOV',59 from dual union all
select 4127,'1956-M06','1956-11-01 12:57:00','suborbital rocket','success',48,'USAF 73','Na Release','','','Aeronomy',640,'',66 from dual union all
select 41270,'1976-W752','1976-05-19 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 41271,'1976-W753','1976-05-19 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.250','','','Aeron',1467,'',75 from dual union all
select 41272,'1976-W754','1976-05-19 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 41273,'1976-W755','1976-05-19 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 41274,'1976-W756','1976-05-19 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 41275,'1976-W757','1976-05-19 16:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',61 from dual union all
select 41276,'1976-W758','1976-05-19 17:34:00','suborbital rocket','unknown',1284,'T 1-8286','','','','Aeron',1821,'',71 from dual union all
select 41277,'1976-W759','1976-05-19 17:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 41278,'1976-W76','1976-01-16 21:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 41279,'1976-W760','1976-05-19 18:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 4128,'1956-M07','1956-11-03 00:00:00','military missile','unknown',1162,'4','JPL-477','','','Test',1852,'',50 from dual union all
select 41280,'1976-W761','1976-05-19 18:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',67 from dual union all
select 41281,'1976-W762','1976-05-19 19:36:59','suborbital rocket','success',1061,'','Starute,Exp. Sonde','','','Meteo',1821,'',67 from dual union all
select 41282,'1976-W763','1976-05-19 19:37:00','suborbital rocket','unknown',1284,'T 1-8288','','','','Ionos',1821,'',71 from dual union all
select 41283,'1976-W764','1976-05-19 22:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',74 from dual union all
select 41284,'1976-W765','1976-05-20 00:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 41285,'1976-W766','1976-05-20 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 41286,'1976-W767','1976-05-20 12:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 41287,'1976-W768','1976-05-20 17:02:00','suborbital rocket','unknown',1284,'T 1-8287','','','','Aeron',1821,'',71 from dual union all
select 41288,'1976-W769','1976-05-20 18:09:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41289,'1976-W77','1976-01-16 22:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 4129,'1956-M08','1956-11-12 11:47:47','suborbital rocket','success',51,'SM1.01','Grenades','IGY SM1.01','','Aeronomy',428,'',67 from dual union all
select 41290,'1976-W770','1976-05-20 21:37:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 41291,'1976-W771','1976-05-21 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 41292,'1976-W772','1976-05-21 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 41293,'1976-W773','1976-05-21 16:13:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 41294,'1976-W774','1976-05-21 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',51 from dual union all
select 41295,'1976-W775','1976-05-21 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 41296,'1976-W776','1976-05-21 19:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 41297,'1976-W777','1976-05-22 00:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 41298,'1976-W778','1976-05-23 08:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 41299,'1976-W779','1976-05-23 12:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 413,'1944-S376','1944-12-28 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 4130,'1956-M09','1956-12-11 00:00:00','suborbital rocket','success',716,'WASP','2','','','Test',1895,'',70 from dual union all
select 41300,'1976-W78','1976-01-18 18:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41301,'1976-W780','1976-05-24 14:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',52 from dual union all
select 41302,'1976-W781','1976-05-24 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 41303,'1976-W782','1976-05-24 16:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 41304,'1976-W783','1976-05-24 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 41305,'1976-W784','1976-05-24 16:55:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',58 from dual union all
select 41306,'1976-W785','1976-05-24 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41307,'1976-W786','1976-05-24 20:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 41308,'1976-W787','1976-05-25 03:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1631,'USHAKOV',58 from dual union all
select 41309,'1976-W788','1976-05-25 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 4131,'1956-M10','1956-12-11 00:00:00','suborbital rocket','success',716,'WASP','4','','','Test',1895,'',70 from dual union all
select 41310,'1976-W789','1976-05-25 23:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 41311,'1976-W79','1976-01-18 20:25:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',66 from dual union all
select 41312,'1976-W790','1976-05-26 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 41313,'1976-W791','1976-05-26 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 41314,'1976-W792','1976-05-26 14:34:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 41315,'1976-W793','1976-05-26 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 41316,'1976-W794','1976-05-26 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 41317,'1976-W795','1976-05-26 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual;
insert into LAUNCH
select 41318,'1976-W796','1976-05-26 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 41319,'1976-W797','1976-05-26 17:44:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 4132,'1956-M11','1956-12-13 00:00:00','suborbital rocket','success',716,'WASP','12','','','Test',1895,'',70 from dual union all
select 41320,'1976-W798','1976-05-26 18:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 41321,'1976-W799','1976-05-26 18:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 41322,'1976-W80','1976-01-19 12:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 41323,'1976-W800','1976-05-27 03:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1631,'USHAKOV',60 from dual union all
select 41324,'1976-W801','1976-05-27 12:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 41325,'1976-W802','1976-05-27 17:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 41326,'1976-W803','1976-05-27 21:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',50 from dual union all
select 41327,'1976-W804','1976-05-28 12:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 41328,'1976-W805','1976-05-28 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',69 from dual union all
select 41329,'1976-W806','1976-05-28 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 4133,'1956-M12','1956-12-13 00:00:00','suborbital rocket','success',716,'WASP','6','','','Test',1895,'',70 from dual union all
select 41330,'1976-W807','1976-05-28 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 41331,'1976-W808','1976-05-28 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 41332,'1976-W809','1976-05-28 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 41333,'1976-W81','1976-01-19 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',56 from dual union all
select 41334,'1976-W810','1976-05-28 21:31:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 41335,'1976-W811','1976-05-29 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 41336,'1976-W812','1976-06-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 41337,'1976-W813','1976-06-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 41338,'1976-W814','1976-06-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 41339,'1976-W815','1976-06-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 4134,'1956-M13','1956-12-13 00:00:00','suborbital rocket','success',716,'WASP','22','','','Test',1895,'',70 from dual union all
select 41340,'1976-W816','1976-05-31 15:48:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 41341,'1976-W817','1976-05-31 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 41342,'1976-W818','1976-06-01 13:07:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',68 from dual union all
select 41343,'1976-W819','1976-06-01 14:36:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',63 from dual union all
select 41344,'1976-W82','1976-01-19 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',62 from dual union all
select 41345,'1976-W820','1976-06-01 15:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 41346,'1976-W821','1976-06-01 15:54:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 41347,'1976-W822','1976-06-01 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',61 from dual union all
select 41348,'1976-W823','1976-06-01 16:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 41349,'1976-W824','1976-06-01 19:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 4135,'1956-M14','1956-12-14 00:00:00','suborbital rocket','success',716,'WASP','17','','','Test',1895,'',70 from dual union all
select 41350,'1976-W825','1976-06-02 13:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',62 from dual union all
select 41351,'1976-W826','1976-06-02 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 41352,'1976-W827','1976-06-02 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 41353,'1976-W828','1976-06-02 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 41354,'1976-W829','1976-06-02 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 41355,'1976-W83','1976-01-19 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',55 from dual union all
select 41356,'1976-W830','1976-06-02 15:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 41357,'1976-W831','1976-06-02 15:45:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 41358,'1976-W832','1976-06-02 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 41359,'1976-W833','1976-06-02 15:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 4136,'1956-M15','1956-12-14 00:00:00','suborbital rocket','success',716,'WASP','18','','','Test',1895,'',70 from dual union all
select 41360,'1976-W834','1976-06-02 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',69 from dual union all
select 41361,'1976-W835','1976-06-02 17:12:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 41362,'1976-W836','1976-06-02 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',69 from dual union all
select 41363,'1976-W837','1976-06-02 22:35:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 41364,'1976-W838','1976-06-03 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 41365,'1976-W839','1976-06-03 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 41366,'1976-W84','1976-01-19 18:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 41367,'1976-W840','1976-06-04 12:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 41368,'1976-W841','1976-06-04 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41369,'1976-W842','1976-06-04 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 4137,'1956-M16','1956-12-14 00:00:00','suborbital rocket','success',716,'WASP','19','','','Test',1895,'',70 from dual union all
select 41370,'1976-W843','1976-06-04 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',61 from dual union all
select 41371,'1976-W844','1976-06-04 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 41372,'1976-W845','1976-06-07 14:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 41373,'1976-W846','1976-06-07 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',77 from dual union all
select 41374,'1976-W847','1976-06-07 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 41375,'1976-W848','1976-06-07 17:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual union all
select 41376,'1976-W849','1976-06-07 23:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 41377,'1976-W85','1976-01-19 20:23:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41378,'1976-W850','1976-06-08 17:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 41379,'1976-W851','1976-06-08 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',53 from dual union all
select 4138,'1956-M17','1956-12-14 00:00:00','suborbital rocket','success',716,'WASP','23','','','Test',1895,'',70 from dual union all
select 41380,'1976-W852','1976-06-09 03:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1631,'USHAKOV',62 from dual union all
select 41381,'1976-W853','1976-06-09 12:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 41382,'1976-W854','1976-06-09 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 41383,'1976-W855','1976-06-09 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 41384,'1976-W856','1976-06-09 16:35:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',52 from dual union all
select 41385,'1976-W857','1976-06-09 16:56:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1057,'',57 from dual union all
select 41386,'1976-W858','1976-06-09 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 41387,'1976-W859','1976-06-09 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 41388,'1976-W86','1976-01-19 21:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',56 from dual union all
select 41389,'1976-W860','1976-06-09 19:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',66 from dual union all
select 4139,'1956-M18','1956-12-14 00:00:00','suborbital rocket','success',716,'WASP','21','','','Test',1895,'',70 from dual union all
select 41390,'1976-W861','1976-06-09 23:37:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 41391,'1976-W862','1976-06-10 00:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 41392,'1976-W863','1976-06-11 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',66 from dual union all
select 41393,'1976-W864','1976-06-11 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 41394,'1976-W865','1976-06-11 14:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 41395,'1976-W866','1976-06-11 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 41396,'1976-W867','1976-06-11 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 41397,'1976-W868','1976-06-11 16:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 41398,'1976-W869','1976-06-11 16:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',54 from dual union all
select 41399,'1976-W87','1976-01-19 22:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 414,'1944-S377','1944-12-28 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 4140,'1956-M19','1956-12-14 00:00:00','suborbital rocket','success',716,'WASP','20','','','Test',1895,'',70 from dual union all
select 41400,'1976-W870','1976-06-14 12:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 41401,'1976-W871','1976-06-14 14:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',61 from dual union all
select 41402,'1976-W872','1976-06-14 16:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',71 from dual union all
select 41403,'1976-W873','1976-06-14 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',50 from dual union all
select 41404,'1976-W874','1976-06-14 23:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 41405,'1976-W875','1976-06-15 05:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1631,'USHAKOV',62 from dual union all
select 41406,'1976-W876','1976-06-15 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 41407,'1976-W877','1976-06-16 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.258','','','Aeron',1467,'',56 from dual;
insert into LAUNCH
select 41408,'1976-W878','1976-06-16 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 41409,'1976-W879','1976-06-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 4141,'1956-M20','1956-12-14 00:00:00','suborbital rocket','success',716,'WASP','24','','','Test',1895,'',70 from dual union all
select 41410,'1976-W88','1976-01-20 00:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 41411,'1976-W880','1976-06-16 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 41412,'1976-W881','1976-06-16 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 41413,'1976-W882','1976-06-16 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',62 from dual union all
select 41414,'1976-W883','1976-06-16 17:15:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 41415,'1976-W884','1976-06-16 18:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 41416,'1976-W885','1976-06-16 22:07:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41417,'1976-W886','1976-06-17 00:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 41418,'1976-W887','1976-06-17 06:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1631,'USHAKOV',61 from dual union all
select 41419,'1976-W888','1976-06-17 12:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 4142,'1956-M21','1956-12-18 00:00:00','suborbital rocket','success',716,'WASP','14','','','Test',1895,'',70 from dual union all
select 41420,'1976-W889','1976-06-17 16:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',60 from dual union all
select 41421,'1976-W89','1976-01-20 03:08:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 41422,'1976-W890','1976-06-17 17:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 41423,'1976-W891','1976-06-17 21:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',62 from dual union all
select 41424,'1976-W892','1976-06-18 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 41425,'1976-W893','1976-06-21 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 41426,'1976-W894','1976-06-21 14:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 41427,'1976-W895','1976-06-21 15:55:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 41428,'1976-W896','1976-06-21 15:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 41429,'1976-W897','1976-06-21 17:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',66 from dual union all
select 4143,'1956-M22','1956-12-18 00:00:00','suborbital rocket','success',716,'WASP','15','','','Test',1895,'',70 from dual union all
select 41430,'1976-W898','1976-06-21 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41431,'1976-W899','1976-06-21 23:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 41432,'1976-W90','1976-01-20 16:52:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',65 from dual union all
select 41433,'1976-W900','1976-06-22 21:42:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 41434,'1976-W901','1976-06-23 14:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',65 from dual union all
select 41435,'1976-W902','1976-06-23 14:35:00','suborbital rocket','unknown',728,'','ISRO-8.259','','','Aeron',1467,'',79 from dual union all
select 41436,'1976-W903','1976-06-23 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 41437,'1976-W904','1976-06-23 15:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41438,'1976-W905','1976-06-23 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',60 from dual union all
select 41439,'1976-W906','1976-06-23 15:49:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',54 from dual union all
select 4144,'1956-M23','1956-12-18 00:00:00','suborbital rocket','success',716,'WASP','16','','','Test',1895,'',70 from dual union all
select 41440,'1976-W907','1976-06-23 16:09:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',67 from dual union all
select 41441,'1976-W908','1976-06-23 17:01:00','suborbital rocket','unknown',1284,'T 1-8332','','','','Aeron',1821,'',57 from dual union all
select 41442,'1976-W909','1976-06-23 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 41443,'1976-W91','1976-01-20 17:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 41444,'1976-W910','1976-06-23 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',59 from dual union all
select 41445,'1976-W911','1976-06-23 21:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 41446,'1976-W912','1976-06-24 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',55 from dual union all
select 41447,'1976-W913','1976-06-24 16:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 41448,'1976-W914','1976-06-24 19:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 41449,'1976-W915','1976-06-24 21:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',63 from dual union all
select 4145,'1956-M24','1956-12-18 00:00:00','suborbital rocket','success',716,'WASP','8','','','Test',1895,'',70 from dual union all
select 41450,'1976-W916','1976-06-25 14:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',63 from dual union all
select 41451,'1976-W917','1976-06-25 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 41452,'1976-W918','1976-06-25 21:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 41453,'1976-W919','1976-06-28 16:34:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 41454,'1976-W92','1976-01-20 19:42:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 41455,'1976-W920','1976-06-28 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 41456,'1976-W921','1976-06-28 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',58 from dual union all
select 41457,'1976-W922','1976-06-29 11:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',66 from dual union all
select 41458,'1976-W923','1976-06-29 14:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',64 from dual union all
select 41459,'1976-W924','1976-06-29 14:37:00','suborbital rocket','unknown',1284,'T 1-8289','','','','Ionos',1821,'',71 from dual union all
select 4146,'1956-M25','1956-12-18 00:00:00','suborbital rocket','success',716,'WASP','9','','','Test',1895,'',70 from dual union all
select 41460,'1976-W925','1976-06-29 15:11:59','suborbital rocket','success',1061,'','Starute,Exp. Sonde','','','Meteo',1821,'',68 from dual union all
select 41461,'1976-W926','1976-06-29 16:22:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',70 from dual union all
select 41462,'1976-W927','1976-06-29 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',68 from dual union all
select 41463,'1976-W928','1976-06-30 14:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1821,'',68 from dual union all
select 41464,'1976-W929','1976-06-30 16:27:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 41465,'1976-W93','1976-01-21 13:32:00','suborbital rocket','unknown',1284,'T 1-8161','','','','Test/Aeron',1821,'',57 from dual union all
select 41466,'1976-W930','1976-06-30 17:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 41467,'1976-W931','1976-06-30 18:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 41468,'1976-W932','1976-06-30 19:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',63 from dual union all
select 41469,'1976-W933','1976-06-30 21:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',855,'',61 from dual union all
select 4147,'1956-M26','1956-12-18 00:00:00','suborbital rocket','success',716,'WASP','10','','','Test',1895,'',70 from dual union all
select 41470,'1976-W934','1976-07-01 15:45:00','suborbital rocket','success',1061,'','Chaff,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 41471,'1976-W935','1976-07-01 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 41472,'1976-W936','1976-07-01 21:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 41473,'1976-W937','1976-07-02 13:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 41474,'1976-W938','1976-07-02 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 41475,'1976-W939','1976-07-02 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 41476,'1976-W94','1976-01-21 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 41477,'1976-W940','1976-07-02 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 41478,'1976-W941','1976-07-02 19:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',59 from dual union all
select 41479,'1976-W942','1976-07-05 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 4148,'1956-M27','1956-12-18 00:00:00','suborbital rocket','success',716,'WASP','24','','','Test',1895,'',70 from dual union all
select 41480,'1976-W943','1976-07-06 15:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',51 from dual union all
select 41481,'1976-W944','1976-07-06 18:30:00','suborbital rocket','success',1061,'','Chaff,Arcasonde','','','Meteo',1895,'',63 from dual union all
select 41482,'1976-W945','1976-07-06 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 41483,'1976-W946','1976-07-07 02:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 41484,'1976-W947','1976-07-07 12:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 41485,'1976-W948','1976-07-07 13:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 41486,'1976-W949','1976-07-07 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 41487,'1976-W95','1976-01-21 15:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 41488,'1976-W950','1976-07-07 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 41489,'1976-W951','1976-07-07 16:51:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 4149,'1956-S01','1956-01-09 00:00:00','military missile','failure',975,'S1-6','LKI 1A','','','Test',636,'',40 from dual union all
select 41490,'1976-W952','1976-07-07 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 41491,'1976-W953','1976-07-07 18:30:00','suborbital rocket','success',1061,'','Chaff,Arcasonde','','','Meteo',1895,'',65 from dual union all
select 41492,'1976-W954','1976-07-07 22:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 41493,'1976-W955','1976-07-08 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 41494,'1976-W956','1976-07-08 16:19:59','suborbital rocket','success',1061,'','Chaff,Arcasonde','','','Meteo',1895,'',64 from dual union all
select 41495,'1976-W957','1976-07-08 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 41496,'1976-W958','1976-07-09 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',58 from dual union all
select 41497,'1976-W959','1976-07-09 14:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 41498,'1976-W96','1976-01-21 16:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1363,'',59 from dual;
insert into LAUNCH
select 41499,'1976-W960','1976-07-09 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',76 from dual union all
select 415,'1944-S378','1944-12-28 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 4150,'1956-S02','1956-01-11 00:00:00','military missile','success',1013,'S1-26','8K51 ZI-1 Certification','','','Test',636,'',500 from dual union all
select 41500,'1976-W961','1976-07-09 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 41501,'1976-W962','1976-07-09 16:37:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 41502,'1976-W963','1976-07-09 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 41503,'1976-W964','1976-07-10 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 41504,'1976-W965','1976-07-12 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',58 from dual union all
select 41505,'1976-W966','1976-07-12 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',58 from dual union all
select 41506,'1976-W967','1976-07-12 14:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',57 from dual union all
select 41507,'1976-W968','1976-07-12 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',51 from dual union all
select 41508,'1976-W969','1976-07-12 16:27:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 41509,'1976-W97','1976-01-21 16:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 4151,'1956-S03','1956-01-17 00:00:00','military missile','success',1013,'S1-27','8K51 ZI-2 Certificatio','','','Test',636,'',500 from dual union all
select 41510,'1976-W970','1976-07-12 16:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 41511,'1976-W971','1976-07-12 17:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 41512,'1976-W972','1976-07-12 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 41513,'1976-W973','1976-07-12 22:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 41514,'1976-W974','1976-07-13 00:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 41515,'1976-W975','1976-07-14 13:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',76 from dual union all
select 41516,'1976-W976','1976-07-14 14:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',330,'',60 from dual union all
select 41517,'1976-W977','1976-07-14 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 41518,'1976-W978','1976-07-14 16:19:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 41519,'1976-W979','1976-07-14 16:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 4152,'1956-S04','1956-01-21 00:00:00','military missile','success',1013,'S1-28','8K51 ZI-3 Certification','','','Test',636,'',500 from dual union all
select 41520,'1976-W98','1976-01-21 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',77 from dual union all
select 41521,'1976-W980','1976-07-14 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',57 from dual union all
select 41522,'1976-W981','1976-07-14 17:54:00','suborbital rocket','unknown',1284,'T 1-8333','','','','Aeron',1821,'',72 from dual union all
select 41523,'1976-W982','1976-07-14 18:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 41524,'1976-W983','1976-07-14 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',57 from dual union all
select 41525,'1976-W984','1976-07-14 21:07:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',59 from dual union all
select 41526,'1976-W985','1976-07-14 21:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 41527,'1976-W986','1976-07-15 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 41528,'1976-W987','1976-07-15 20:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 41529,'1976-W988','1976-07-16 12:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 4153,'1956-S05','1956-02-06 00:00:00','military missile','success',1013,'S1-29','8K51 ZI-5 Certification','','','Test',636,'',500 from dual union all
select 41530,'1976-W989','1976-07-16 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 41531,'1976-W99','1976-01-21 17:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1895,'',65 from dual union all
select 41532,'1976-W990','1976-07-16 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 41533,'1976-W991','1976-07-19 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',55 from dual union all
select 41534,'1976-W992','1976-07-19 17:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 41535,'1976-W993','1976-07-19 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 41536,'1976-W994','1976-07-19 18:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 41537,'1976-W995','1976-07-19 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',64 from dual union all
select 41538,'1976-W996','1976-07-20 12:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 41539,'1976-W997','1976-07-20 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 4154,'1956-S06','1956-01-19 00:00:00','military missile','success',975,'S1-1','LKI 1A','','','Test',636,'',40 from dual union all
select 41540,'1976-W998','1976-07-20 21:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 41541,'1976-W999','1976-07-21 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 41542,'1976-Y01','1976-01-02 15:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',94 from dual union all
select 41543,'1976-Y02','1976-01-04 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 41544,'1976-Y03','1976-01-07 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.230','','','Aeron',1467,'',83 from dual union all
select 41545,'1976-Y04','1976-01-07 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 41546,'1976-Y05','1976-01-07 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',82 from dual union all
select 41547,'1976-Y06','1976-01-07 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 41548,'1976-Y07','1976-01-07 19:57:59','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',87 from dual union all
select 41549,'1976-Y08','1976-01-09 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 4155,'1956-S07','1956-01-20 18:06:00','suborbital rocket','success',1521,'D-4','','','','Test',253,'',132 from dual union all
select 41550,'1976-Y09','1976-01-09 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 41551,'1976-Y10','1976-01-09 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',92 from dual union all
select 41552,'1976-Y100','1976-02-27 04:05:00','suborbital rocket','success',1281,'NASA 15.147GM','COPNE (Polar ozone)','','','Aeronomy',1470,'',84 from dual union all
select 41553,'1976-Y101','1976-02-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',83 from dual union all
select 41554,'1976-Y102','1976-03-02 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 41555,'1976-Y103','1976-03-03 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.239','','','Aeron',1467,'',84 from dual union all
select 41556,'1976-Y104','1976-03-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 41557,'1976-Y105','1976-03-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 41558,'1976-Y106','1976-03-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',83 from dual union all
select 41559,'1976-Y107','1976-03-03 21:14:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 4156,'1956-S08','1956-01-21 00:00:00','military missile','success',970,'No 101','','','','OT',636,'',100 from dual union all
select 41560,'1976-Y108','1976-03-04 05:22:00','suborbital rocket','success',1281,'NASA 15.148GM','COPNE','','','Aeronomy',1470,'',84 from dual union all
select 41561,'1976-Y109','1976-03-04 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 41562,'1976-Y11','1976-01-09 22:18:59','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',855,'',86 from dual union all
select 41563,'1976-Y110','1976-03-08 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',86 from dual union all
select 41564,'1976-Y111','1976-03-09 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 41565,'1976-Y112','1976-03-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',94 from dual union all
select 41566,'1976-Y113','1976-03-10 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.240','','','Aeron',1467,'',85 from dual union all
select 41567,'1976-Y114','1976-03-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 41568,'1976-Y115','1976-03-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 41569,'1976-Y116','1976-03-11 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 4157,'1956-S09','1956-03-07 00:00:00','military missile','success',1013,'M-5RD A1-4','M5-RD 2','R-7 component test','','Test',636,'',500 from dual union all
select 41570,'1976-Y117','1976-03-12 17:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',96 from dual union all
select 41571,'1976-Y118','1976-03-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',82 from dual union all
select 41572,'1976-Y119','1976-03-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',80 from dual union all
select 41573,'1976-Y12','1976-01-10 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',87 from dual union all
select 41574,'1976-Y120','1976-03-14 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',84 from dual union all
select 41575,'1976-Y121','1976-03-15 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',88 from dual union all
select 41576,'1976-Y122','1976-03-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',85 from dual union all
select 41577,'1976-Y123','1976-03-16 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 41578,'1976-Y124','1976-03-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 41579,'1976-Y125','1976-03-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 4158,'1956-S10','1956-02-02 07:30:00','military missile','success',1013,'S1-30','8K51 ZI-4 BAYKAL','','','Nuclear Test',636,'',500 from dual union all
select 41580,'1976-Y126','1976-03-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 41581,'1976-Y127','1976-03-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',83 from dual union all
select 41582,'1976-Y128','1976-03-18 21:38:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 41583,'1976-Y129','1976-03-23 19:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',98 from dual union all
select 41584,'1976-Y13','1976-01-11 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',92 from dual union all
select 41585,'1976-Y130','1976-03-23 20:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 41586,'1976-Y131','1976-03-23 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 41587,'1976-Y132','1976-03-23 23:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 41588,'1976-Y133','1976-03-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual;
insert into LAUNCH
select 41589,'1976-Y134','1976-03-24 16:23:00','suborbital rocket','unknown',728,'','ISRO-8.242','','','Aeron',1467,'',86 from dual union all
select 4159,'1956-S100','1956-11-15 19:32:00','suborbital rocket','success',49,'NRL 47 (RV-N-13b)','NRL NN3.07F','IGY NN3.07','','Ionos',428,'',129 from dual union all
select 41590,'1976-Y135','1976-03-24 21:22:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 41591,'1976-Y136','1976-03-26 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 41592,'1976-Y137','1976-03-26 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',95 from dual union all
select 41593,'1976-Y138','1976-03-26 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',86 from dual union all
select 41594,'1976-Y139','1976-03-26 19:26:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',88 from dual union all
select 41595,'1976-Y14','1976-01-12 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 41596,'1976-Y140','1976-03-27 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 41597,'1976-Y141','1976-03-28 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',84 from dual union all
select 41598,'1976-Y142','1976-03-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 41599,'1976-Y143','1976-03-30 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 416,'1944-S379','1944-12-28 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 4160,'1956-S101','1956-11-16 14:38:00','suborbital rocket','success',1521,'R-14','4203-14','','','Reentry Test',253,'',107 from dual union all
select 41600,'1976-Y144','1976-03-31 14:35:00','suborbital rocket','unknown',728,'','ISRO-8.243','','','Aeron',1467,'',90 from dual union all
select 41601,'1976-Y145','1976-03-31 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 41602,'1976-Y146','1976-03-31 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',86 from dual union all
select 41603,'1976-Y147','1976-03-31 17:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',96 from dual union all
select 41604,'1976-Y148','1976-03-31 19:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41605,'1976-Y149','1976-03-31 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 41606,'1976-Y15','1976-01-14 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 41607,'1976-Y150','1976-04-02 20:55:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',89 from dual union all
select 41608,'1976-Y151','1976-04-02 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 41609,'1976-Y152','1976-04-03 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',86 from dual union all
select 4161,'1956-S102','1956-11-17 16:48:00','suborbital rocket','success',49,'NRL 43 (RV-N-13b)','NRL NN3.12','IGY NN3.12','','Aeronomy/Solar La,X',428,'',209 from dual union all
select 41610,'1976-Y153','1976-04-06 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 41611,'1976-Y154','1976-04-06 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 41612,'1976-Y155','1976-04-07 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.244','','','Aeron',1467,'',88 from dual union all
select 41613,'1976-Y156','1976-04-07 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 41614,'1976-Y157','1976-04-07 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 41615,'1976-Y158','1976-04-07 17:07:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',86 from dual union all
select 41616,'1976-Y159','1976-04-08 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 41617,'1976-Y16','1976-01-14 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 41618,'1976-Y160','1976-04-09 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',86 from dual union all
select 41619,'1976-Y161','1976-04-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 4162,'1956-S103','1956-11-21 05:21:11','suborbital rocket','success',49,'NRL 48 (RV-N-13c)','NRL NN3.17','IGY NN3.17','','Ionos/Auroral',428,'',250 from dual union all
select 41620,'1976-Y162','1976-04-10 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',88 from dual union all
select 41621,'1976-Y163','1976-04-11 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',83 from dual union all
select 41622,'1976-Y164','1976-04-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 41623,'1976-Y165','1976-04-12 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',87 from dual union all
select 41624,'1976-Y166','1976-04-13 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',90 from dual union all
select 41625,'1976-Y167','1976-04-13 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 41626,'1976-Y168','1976-04-14 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.245','','','Aeron',1467,'',83 from dual union all
select 41627,'1976-Y169','1976-04-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 41628,'1976-Y17','1976-01-14 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.231','','','Aeron',1467,'',88 from dual union all
select 41629,'1976-Y170','1976-04-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 4163,'1956-S104','1956-11-23 13:57:00','suborbital rocket','success',1521,'R-15','4203-15','','','Reentry Test',253,'',143 from dual union all
select 41630,'1976-Y171','1976-04-14 17:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',94 from dual union all
select 41631,'1976-Y172','1976-04-14 21:11:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 41632,'1976-Y173','1976-04-15 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 41633,'1976-Y174','1976-04-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 41634,'1976-Y175','1976-04-16 23:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 41635,'1976-Y176','1976-04-20 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 41636,'1976-Y177','1976-04-20 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 41637,'1976-Y178','1976-04-21 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.246','','','Aeron',1467,'',88 from dual union all
select 41638,'1976-Y179','1976-04-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 41639,'1976-Y18','1976-01-14 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 4164,'1956-S105','1956-11-29 13:23:00','military missile','success',635,'CC-15    HS','Jupiter A','','','Test',280,'',90 from dual union all
select 41640,'1976-Y180','1976-04-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',91 from dual union all
select 41641,'1976-Y181','1976-04-21 17:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',95 from dual union all
select 41642,'1976-Y182','1976-04-21 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',99 from dual union all
select 41643,'1976-Y183','1976-04-21 21:49:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 41644,'1976-Y184','1976-04-22 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 41645,'1976-Y185','1976-04-24 00:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 41646,'1976-Y186','1976-04-26 23:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 41647,'1976-Y187','1976-04-27 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 41648,'1976-Y188','1976-04-27 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 41649,'1976-Y189','1976-04-28 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.247','','','Aeron',1467,'',87 from dual union all
select 4165,'1956-S106','1956-12-03 15:36:00','suborbital rocket','success',1521,'R-16','4203-16','','','Reentry Test',253,'',125 from dual union all
select 41650,'1976-Y19','1976-01-14 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 41651,'1976-Y190','1976-04-28 14:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',97 from dual union all
select 41652,'1976-Y191','1976-04-28 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 41653,'1976-Y192','1976-04-28 20:59:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 41654,'1976-Y193','1976-04-28 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 41655,'1976-Y194','1976-04-29 11:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 41656,'1976-Y195','1976-05-04 01:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 41657,'1976-Y196','1976-05-04 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 41658,'1976-Y197','1976-05-04 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 41659,'1976-Y198','1976-05-05 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.248','','','Aeron',1467,'',85 from dual union all
select 4166,'1956-S107','1956-12-08 06:03:00','test rocket','success',1500,'13/TV0','Vanguard TV0','','','Test',235,'',204 from dual union all
select 41660,'1976-Y199','1976-05-05 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 41661,'1976-Y20','1976-01-14 21:48:59','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',87 from dual union all
select 41662,'1976-Y200','1976-05-05 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 41663,'1976-Y201','1976-05-05 20:21:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',87 from dual union all
select 41664,'1976-Y202','1976-05-05 21:53:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 41665,'1976-Y203','1976-05-06 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 41666,'1976-Y204','1976-05-11 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 41667,'1976-Y205','1976-05-11 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 41668,'1976-Y206','1976-05-12 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.249','','','Aeron',1467,'',88 from dual union all
select 41669,'1976-Y207','1976-05-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 4167,'1956-S108','1956-12-11 12:37:00','suborbital rocket','success',1521,'R-17','4203-17','','','Reentry Test',253,'',144 from dual union all
select 41670,'1976-Y208','1976-05-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 41671,'1976-Y209','1976-05-13 20:47:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',86 from dual union all
select 41672,'1976-Y21','1976-01-15 22:31:59','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',855,'',87 from dual union all
select 41673,'1976-Y210','1976-05-18 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 41674,'1976-Y211','1976-05-19 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',94 from dual union all
select 41675,'1976-Y212','1976-05-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 41676,'1976-Y213','1976-05-19 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 41677,'1976-Y214','1976-05-19 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 41678,'1976-Y215','1976-05-19 18:02:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual;
insert into LAUNCH
select 41679,'1976-Y216','1976-05-19 21:33:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 4168,'1956-S109','1956-12-13 21:44:00','suborbital rocket','success',49,'USAF 75','AJ11-6 Rocket test','','','Test',640,'',193 from dual union all
select 41680,'1976-Y217','1976-05-21 14:35:00','suborbital rocket','unknown',728,'','ISRO-8.251','','','Aeron',1467,'',86 from dual union all
select 41681,'1976-Y218','1976-05-21 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 41682,'1976-Y219','1976-05-21 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 41683,'1976-Y22','1976-01-16 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',87 from dual union all
select 41684,'1976-Y220','1976-05-21 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',96 from dual union all
select 41685,'1976-Y221','1976-05-24 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.252','','','Aeron',1467,'',84 from dual union all
select 41686,'1976-Y222','1976-05-24 23:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 41687,'1976-Y223','1976-05-25 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 41688,'1976-Y224','1976-05-25 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 41689,'1976-Y225','1976-05-25 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 4169,'1956-S11','1956-02-13 00:00:00','military missile','success',970,'No 431','','','','OT',636,'',100 from dual union all
select 41690,'1976-Y226','1976-05-26 01:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41691,'1976-Y227','1976-05-26 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.253','','','Aeron',1467,'',86 from dual union all
select 41692,'1976-Y228','1976-05-26 16:06:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 41693,'1976-Y229','1976-05-27 00:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 41694,'1976-Y23','1976-01-16 18:35:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 41695,'1976-Y230','1976-05-28 14:33:00','suborbital rocket','unknown',728,'','ISRO-8.254','','','Aeron',1467,'',83 from dual union all
select 41696,'1976-Y231','1976-05-28 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',93 from dual union all
select 41697,'1976-Y232','1976-05-28 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 41698,'1976-Y233','1976-05-31 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.255','','','Aeron',1467,'',83 from dual union all
select 41699,'1976-Y234','1976-06-01 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 417,'1944-S38','1944-08-15 10:08:00','military missile','success',8,'18311','Ma113','','','Training',661,'',90 from dual union all
select 4170,'1956-S110','1956-12-19 02:30:00','military missile','success',635,'RS-22    UU','Jupiter A','','','Test',280,'',90 from dual union all
select 41700,'1976-Y235','1976-06-01 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 41701,'1976-Y236','1976-06-02 14:38:00','suborbital rocket','unknown',728,'','ISRO-8.256','','','Aeron',1467,'',85 from dual union all
select 41702,'1976-Y237','1976-06-02 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 41703,'1976-Y238','1976-06-02 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',95 from dual union all
select 41704,'1976-Y239','1976-06-04 00:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 41705,'1976-Y24','1976-01-18 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',86 from dual union all
select 41706,'1976-Y240','1976-06-08 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 41707,'1976-Y241','1976-06-09 15:12:00','suborbital rocket','unknown',728,'','ISRO-8.257','','','Aeron',1467,'',85 from dual union all
select 41708,'1976-Y242','1976-06-09 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 41709,'1976-Y243','1976-06-09 23:11:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',88 from dual union all
select 4171,'1956-S111','1956-12-20 00:00:00','suborbital rocket','success',5,'','','','','Ionos?',636,'',100 from dual union all
select 41710,'1976-Y244','1976-06-11 15:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',95 from dual union all
select 41711,'1976-Y245','1976-06-15 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 41712,'1976-Y246','1976-06-15 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 41713,'1976-Y247','1976-06-16 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',96 from dual union all
select 41714,'1976-Y248','1976-06-17 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',86 from dual union all
select 41715,'1976-Y249','1976-06-18 00:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 41716,'1976-Y25','1976-01-19 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',87 from dual union all
select 41717,'1976-Y250','1976-06-22 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',90 from dual union all
select 41718,'1976-Y251','1976-06-22 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 41719,'1976-Y252','1976-06-23 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 4172,'1956-S112','1956-12-26 00:00:00','military missile','success',974,'','','','','Test',190,'B-67',150 from dual union all
select 41720,'1976-Y253','1976-06-23 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',86 from dual union all
select 41721,'1976-Y254','1976-06-23 21:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',85 from dual union all
select 41722,'1976-Y255','1976-06-24 01:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 41723,'1976-Y256','1976-06-25 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',86 from dual union all
select 41724,'1976-Y257','1976-06-25 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 41725,'1976-Y258','1976-06-26 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',89 from dual union all
select 41726,'1976-Y259','1976-06-28 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',89 from dual union all
select 41727,'1976-Y26','1976-01-21 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 41728,'1976-Y260','1976-06-28 23:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 41729,'1976-Y261','1976-06-29 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 4173,'1956-S114','1956-01-24 00:00:00','military missile','failure',970,'No 465','','','','OT',636,'',100 from dual union all
select 41730,'1976-Y262','1976-06-29 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 41731,'1976-Y263','1976-06-30 14:34:00','suborbital rocket','unknown',728,'','ISRO-8.260','','','Aeron',1467,'',85 from dual union all
select 41732,'1976-Y264','1976-06-30 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',95 from dual union all
select 41733,'1976-Y265','1976-06-30 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',89 from dual union all
select 41734,'1976-Y266','1976-07-01 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',89 from dual union all
select 41735,'1976-Y267','1976-07-02 18:57:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual union all
select 41736,'1976-Y268','1976-07-02 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 41737,'1976-Y269','1976-07-06 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 41738,'1976-Y27','1976-01-21 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.232','','','Aeron',1467,'',83 from dual union all
select 41739,'1976-Y270','1976-07-06 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 4174,'1956-S115','1956-01-29 00:00:00','military missile','success',971,'S1-1','Production launch','','','Test',636,'',150 from dual union all
select 41740,'1976-Y271','1976-07-07 15:00:00','suborbital rocket','unknown',728,'','ISRO-8.261','','','Aeron',1467,'',86 from dual union all
select 41741,'1976-Y272','1976-07-07 17:22:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',86 from dual union all
select 41742,'1976-Y273','1976-07-07 18:20:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 41743,'1976-Y274','1976-07-07 21:32:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',83 from dual union all
select 41744,'1976-Y275','1976-07-07 22:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',94 from dual union all
select 41745,'1976-Y276','1976-07-08 01:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 41746,'1976-Y277','1976-07-08 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 41747,'1976-Y278','1976-07-09 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 41748,'1976-Y279','1976-07-13 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 41749,'1976-Y28','1976-01-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 4175,'1956-S116','1956-01-31 00:00:00','military missile','success',971,'S1-3','Production launch','','','Test',636,'',150 from dual union all
select 41750,'1976-Y280','1976-07-13 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 41751,'1976-Y281','1976-07-14 14:50:00','suborbital rocket','unknown',728,'','ISRO-8.262','','','Aeron',1467,'',85 from dual union all
select 41752,'1976-Y282','1976-07-14 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',96 from dual union all
select 41753,'1976-Y283','1976-07-14 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',82 from dual union all
select 41754,'1976-Y284','1976-07-15 00:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',83 from dual union all
select 41755,'1976-Y285','1976-07-16 22:37:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 41756,'1976-Y286','1976-07-16 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 41757,'1976-Y287','1976-07-17 01:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',83 from dual union all
select 41758,'1976-Y288','1976-07-17 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',89 from dual union all
select 41759,'1976-Y289','1976-07-19 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',92 from dual union all
select 4176,'1956-S118','1956-02-17 00:00:00','military missile','success',992,'A1-20','','','','OT',636,'',170 from dual union all
select 41760,'1976-Y29','1976-01-23 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 41761,'1976-Y290','1976-07-19 23:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 41762,'1976-Y291','1976-07-20 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 41763,'1976-Y292','1976-07-20 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',85 from dual union all
select 41764,'1976-Y293','1976-07-20 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',82 from dual union all
select 41765,'1976-Y294','1976-07-20 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',95 from dual union all
select 41766,'1976-Y295','1976-07-20 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 41767,'1976-Y296','1976-07-21 14:45:00','suborbital rocket','unknown',728,'','ISRO-8.263','','','Aeron',1467,'',85 from dual union all
select 41768,'1976-Y297','1976-07-21 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',85 from dual;
insert into LAUNCH
select 41769,'1976-Y298','1976-07-21 21:31:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',88 from dual union all
select 4177,'1956-S119','1956-02-25 00:00:00','military missile','success',975,'S1-3','LKI 1A','','','Test',636,'',80 from dual union all
select 41770,'1976-Y299','1976-07-22 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',82 from dual union all
select 41771,'1976-Y30','1976-01-24 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',89 from dual union all
select 41772,'1976-Y300','1976-07-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',86 from dual union all
select 41773,'1976-Y301','1976-07-23 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 41774,'1976-Y302','1976-07-26 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',86 from dual union all
select 41775,'1976-Y303','1976-07-26 23:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 41776,'1976-Y304','1976-07-27 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 41777,'1976-Y305','1976-07-28 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.264','','','Aeron',1467,'',83 from dual union all
select 41778,'1976-Y306','1976-07-28 14:55:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 41779,'1976-Y307','1976-07-28 18:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 4178,'1956-S12','1956-02-14 00:00:00','military missile','success',970,'No 453','','','','OT',636,'',100 from dual union all
select 41780,'1976-Y308','1976-07-28 19:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 41781,'1976-Y309','1976-07-28 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 41782,'1976-Y31','1976-01-28 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 41783,'1976-Y310','1976-07-29 00:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 41784,'1976-Y311','1976-07-29 03:05:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41785,'1976-Y312','1976-07-29 06:15:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 41786,'1976-Y313','1976-07-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 41787,'1976-Y314','1976-07-29 11:45:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 41788,'1976-Y315','1976-07-29 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 41789,'1976-Y316','1976-07-29 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',83 from dual union all
select 4179,'1956-S120','1956-02-25 00:00:00','military missile','success',975,'S1-4','LKI 1A','','','Test',636,'',80 from dual union all
select 41790,'1976-Y317','1976-07-29 17:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 41791,'1976-Y318','1976-07-29 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 41792,'1976-Y319','1976-07-29 23:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',82 from dual union all
select 41793,'1976-Y32','1976-01-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 41794,'1976-Y320','1976-07-30 00:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 41795,'1976-Y321','1976-07-30 02:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 41796,'1976-Y322','1976-07-30 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 41797,'1976-Y323','1976-07-30 11:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41798,'1976-Y324','1976-07-30 14:59:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 41799,'1976-Y325','1976-07-30 16:15:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 418,'1944-S380','1944-12-28 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 4180,'1956-S121','1956-02-27 00:00:00','military missile','success',975,'S1-9','LKI 1A','','','Test',636,'',80 from dual union all
select 41800,'1976-Y326','1976-07-30 17:06:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 41801,'1976-Y327','1976-07-30 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 41802,'1976-Y328','1976-08-01 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',85 from dual union all
select 41803,'1976-Y329','1976-08-01 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',86 from dual union all
select 41804,'1976-Y33','1976-01-28 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.233','','','Aeron',1467,'',85 from dual union all
select 41805,'1976-Y330','1976-08-03 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',82 from dual union all
select 41806,'1976-Y331','1976-08-03 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 41807,'1976-Y332','1976-08-03 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 41808,'1976-Y333','1976-08-04 14:33:00','suborbital rocket','unknown',728,'','ISRO-8.265','','','Aeron',1467,'',84 from dual union all
select 41809,'1976-Y334','1976-08-04 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 4181,'1956-S122','1956-03-09 00:00:00','military missile','success',992,'A2-20','','','','OT',636,'',170 from dual union all
select 41810,'1976-Y335','1976-08-04 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 41811,'1976-Y336','1976-08-04 19:56:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 41812,'1976-Y337','1976-08-04 22:10:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',85 from dual union all
select 41813,'1976-Y338','1976-08-05 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 41814,'1976-Y339','1976-08-05 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',87 from dual union all
select 41815,'1976-Y34','1976-01-28 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 41816,'1976-Y340','1976-08-05 23:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 41817,'1976-Y341','1976-08-06 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',86 from dual union all
select 41818,'1976-Y342','1976-08-06 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 41819,'1976-Y343','1976-08-08 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',86 from dual union all
select 4182,'1956-S123','1956-04-29 00:00:00','military missile','success',992,'A3-20','','','','OT',636,'',170 from dual union all
select 41820,'1976-Y344','1976-08-08 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',86 from dual union all
select 41821,'1976-Y345','1976-08-10 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',80 from dual union all
select 41822,'1976-Y346','1976-08-10 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 41823,'1976-Y347','1976-08-11 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.266','','','Aeron',1467,'',85 from dual union all
select 41824,'1976-Y348','1976-08-11 15:55:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual union all
select 41825,'1976-Y349','1976-08-11 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 41826,'1976-Y35','1976-01-28 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 41827,'1976-Y350','1976-08-11 18:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 41828,'1976-Y351','1976-08-13 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 41829,'1976-Y352','1976-08-17 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 4183,'1956-S124','1956-05-08 00:00:00','military missile','success',992,'0105','','','','OT',636,'',170 from dual union all
select 41830,'1976-Y353','1976-08-17 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 41831,'1976-Y354','1976-08-18 14:27:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41832,'1976-Y355','1976-08-18 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.267','','','Aeron',1467,'',85 from dual union all
select 41833,'1976-Y356','1976-08-18 17:49:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 41834,'1976-Y357','1976-08-18 18:05:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 41835,'1976-Y358','1976-08-18 20:30:00','suborbital rocket','unknown',728,'','ISRO-8.268','','','Aeron',1467,'',83 from dual union all
select 41836,'1976-Y359','1976-08-18 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 41837,'1976-Y36','1976-01-29 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',90 from dual union all
select 41838,'1976-Y360','1976-08-18 23:15:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 41839,'1976-Y361','1976-08-19 01:48:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',95 from dual union all
select 4184,'1956-S125','1956-05-08 00:00:00','military missile','success',990,'1T','Project T-3','','','Test',636,'',100 from dual union all
select 41840,'1976-Y362','1976-08-19 02:30:00','suborbital rocket','unknown',728,'','ISRO-8.269','','','Aeron',1467,'',84 from dual union all
select 41841,'1976-Y363','1976-08-19 06:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 41842,'1976-Y364','1976-08-19 08:47:00','suborbital rocket','unknown',728,'','ISRO-8.270','','','Aeron',1467,'',83 from dual union all
select 41843,'1976-Y365','1976-08-19 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 41844,'1976-Y366','1976-08-19 11:46:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41845,'1976-Y367','1976-08-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 41846,'1976-Y368','1976-08-19 15:48:00','suborbital rocket','unknown',728,'','ISRO-8.272','','','Aeron',1467,'',85 from dual union all
select 41847,'1976-Y369','1976-08-19 16:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',85 from dual union all
select 41848,'1976-Y37','1976-01-29 20:02:00','suborbital rocket','success',1061,'','Met Sphere,MK-1','','','Meteo',1057,'',86 from dual union all
select 41849,'1976-Y370','1976-08-19 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 4185,'1956-S126','1956-05-10 00:00:00','military missile','success',990,'4T-1','Project T-3','','','Test',636,'',100 from dual union all
select 41850,'1976-Y371','1976-08-19 20:30:00','suborbital rocket','unknown',728,'','ISRO-8.273','','','Aeron',1467,'',84 from dual union all
select 41851,'1976-Y372','1976-08-19 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 41852,'1976-Y373','1976-08-19 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 41853,'1976-Y374','1976-08-19 23:15:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 41854,'1976-Y375','1976-08-20 02:30:00','suborbital rocket','unknown',728,'','ISRO-8.274','','','Aeron',1467,'',86 from dual union all
select 41855,'1976-Y376','1976-08-20 03:33:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 41856,'1976-Y377','1976-08-20 06:12:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 41857,'1976-Y378','1976-08-20 08:25:00','suborbital rocket','unknown',728,'','ISRO-8.275','','','Aeron',1467,'',86 from dual union all
select 41858,'1976-Y379','1976-08-20 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual;
insert into LAUNCH
select 41859,'1976-Y38','1976-01-30 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 4186,'1956-S127','1956-05-16 00:00:00','military missile','success',990,'3T-1','Project T-3','','','Test',636,'',100 from dual union all
select 41860,'1976-Y380','1976-08-20 11:44:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41861,'1976-Y381','1976-08-20 14:42:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 41862,'1976-Y382','1976-08-20 17:42:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 41863,'1976-Y383','1976-08-20 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 41864,'1976-Y384','1976-08-20 21:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 41865,'1976-Y385','1976-08-21 17:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',89 from dual union all
select 41866,'1976-Y386','1976-08-22 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',84 from dual union all
select 41867,'1976-Y387','1976-08-23 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',92 from dual union all
select 41868,'1976-Y388','1976-08-24 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',87 from dual union all
select 41869,'1976-Y389','1976-08-24 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',86 from dual union all
select 4187,'1956-S128','1956-06-06 00:00:00','military missile','success',990,'5T-2','Project T-3','','','Test',636,'',100 from dual union all
select 41870,'1976-Y39','1976-01-30 15:25:00','suborbital rocket','success',1281,'NASA 15.49UE','','','','Plasma',1821,'',85 from dual union all
select 41871,'1976-Y390','1976-08-24 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 41872,'1976-Y391','1976-08-24 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 41873,'1976-Y392','1976-08-25 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.276','','','Aeron',1467,'',80 from dual union all
select 41874,'1976-Y393','1976-08-25 16:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 41875,'1976-Y394','1976-08-25 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',84 from dual union all
select 41876,'1976-Y395','1976-08-25 17:22:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 41877,'1976-Y396','1976-08-26 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',89 from dual union all
select 41878,'1976-Y397','1976-08-26 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',89 from dual union all
select 41879,'1976-Y398','1976-08-26 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 4188,'1956-S129','1956-06-08 00:00:00','military missile','success',990,'2T-1','Project T-3','','','Test',636,'',100 from dual union all
select 41880,'1976-Y399','1976-08-27 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',85 from dual union all
select 41881,'1976-Y40','1976-01-31 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',87 from dual union all
select 41882,'1976-Y400','1976-08-28 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',88 from dual union all
select 41883,'1976-Y401','1976-08-28 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',86 from dual union all
select 41884,'1976-Y402','1976-08-30 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',89 from dual union all
select 41885,'1976-Y403','1976-08-30 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1393,'SHOK',86 from dual union all
select 41886,'1976-Y404','1976-08-31 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 41887,'1976-Y405','1976-08-31 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 41888,'1976-Y406','1976-09-01 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.277','','','Aeron',1467,'',81 from dual union all
select 41889,'1976-Y407','1976-09-01 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 4189,'1956-S13','1956-02-16 00:00:00','military missile','success',1013,'M-5RD S1-3','M5-RD 1','R-7 component test','','Test',636,'',500 from dual union all
select 41890,'1976-Y408','1976-09-01 18:22:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 41891,'1976-Y409','1976-09-02 01:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 41892,'1976-Y41','1976-01-31 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',88 from dual union all
select 41893,'1976-Y410','1976-09-02 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 41894,'1976-Y411','1976-09-03 17:52:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 41895,'1976-Y412','1976-09-07 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 41896,'1976-Y413','1976-09-08 16:25:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 41897,'1976-Y414','1976-09-08 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 41898,'1976-Y415','1976-09-08 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 41899,'1976-Y416','1976-09-09 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.278','','','Aeron',1467,'',84 from dual union all
select 419,'1944-S381','1944-12-28 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 4190,'1956-S130','1956-06-12 00:00:00','military missile','success',990,'7T-2','Project T-3','','','Test',636,'',100 from dual union all
select 41900,'1976-Y417','1976-09-09 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 41901,'1976-Y418','1976-09-14 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 41902,'1976-Y419','1976-09-14 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',80 from dual union all
select 41903,'1976-Y42','1976-02-01 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',89 from dual union all
select 41904,'1976-Y420','1976-09-15 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.279','','','Aeron',1467,'',84 from dual union all
select 41905,'1976-Y421','1976-09-15 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41906,'1976-Y422','1976-09-15 16:53:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual union all
select 41907,'1976-Y423','1976-09-15 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',96 from dual union all
select 41908,'1976-Y424','1976-09-15 18:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 41909,'1976-Y425','1976-09-15 20:56:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 4191,'1956-S131','1956-06-12 00:00:00','military missile','success',990,'6T-2','Project T-3','','','Test',636,'',100 from dual union all
select 41910,'1976-Y426','1976-09-17 02:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 41911,'1976-Y427','1976-09-21 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 41912,'1976-Y428','1976-09-21 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 41913,'1976-Y429','1976-09-22 14:54:00','suborbital rocket','unknown',725,'','ISRO-8.280','','','Aeron',1467,'',86 from dual union all
select 41914,'1976-Y43','1976-02-02 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',85 from dual union all
select 41915,'1976-Y430','1976-09-22 16:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 41916,'1976-Y431','1976-09-22 19:23:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 41917,'1976-Y432','1976-09-23 12:55:00','suborbital rocket','unknown',1281,'TM1-8170','','','','Aeron',1047,'',86 from dual union all
select 41918,'1976-Y433','1976-09-23 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.281','','','Aeron',1467,'',87 from dual union all
select 41919,'1976-Y434','1976-09-24 15:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 4192,'1956-S132','1956-06-18 00:00:00','military missile','success',990,'8T-2','Project T-3','','','Test',636,'',100 from dual union all
select 41920,'1976-Y435','1976-09-24 17:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 41921,'1976-Y436','1976-09-24 19:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 41922,'1976-Y437','1976-09-24 20:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 41923,'1976-Y438','1976-09-25 07:15:00','suborbital rocket','unknown',728,'','ISRO-8.282','','','Solar UV',1467,'',90 from dual union all
select 41924,'1976-Y439','1976-09-25 07:56:00','suborbital rocket','unknown',1281,'TM1-8171','','','','Aeron',1047,'',82 from dual union all
select 41925,'1976-Y44','1976-02-02 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',91 from dual union all
select 41926,'1976-Y440','1976-09-28 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 41927,'1976-Y441','1976-09-28 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 41928,'1976-Y442','1976-09-29 14:30:00','suborbital rocket','unknown',725,'','ISRO-8.283','','','Aeron',1467,'',86 from dual union all
select 41929,'1976-Y443','1976-09-29 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 4193,'1956-S133','1956-06-20 00:00:00','military missile','success',990,'10T-3','Project T-3','','','Test',636,'',100 from dual union all
select 41930,'1976-Y444','1976-09-30 21:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41931,'1976-Y445','1976-10-01 01:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 41932,'1976-Y446','1976-10-01 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 41933,'1976-Y447','1976-10-05 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 41934,'1976-Y448','1976-10-05 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 41935,'1976-Y449','1976-10-06 16:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41936,'1976-Y45','1976-02-03 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',87 from dual union all
select 41937,'1976-Y450','1976-10-06 16:52:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 41938,'1976-Y451','1976-10-06 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41939,'1976-Y452','1976-10-06 19:09:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 4194,'1956-S134','1956-06-21 00:00:00','military missile','success',990,'9T-3','Project T-3','','','Test',636,'',100 from dual union all
select 41940,'1976-Y453','1976-10-08 16:52:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41941,'1976-Y454','1976-10-12 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 41942,'1976-Y455','1976-10-12 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 41943,'1976-Y456','1976-10-13 10:05:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 41944,'1976-Y457','1976-10-13 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.285','','','Aeron',1467,'',80 from dual union all
select 41945,'1976-Y458','1976-10-13 15:58:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',86 from dual union all
select 41946,'1976-Y459','1976-10-13 18:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 41947,'1976-Y46','1976-02-04 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 41948,'1976-Y460','1976-10-14 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual;
insert into LAUNCH
select 41949,'1976-Y461','1976-10-15 16:15:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 4195,'1956-S135','1956-07-12 00:00:00','military missile','success',992,'207','','','','OT',636,'',170 from dual union all
select 41950,'1976-Y462','1976-10-19 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 41951,'1976-Y463','1976-10-19 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 41952,'1976-Y464','1976-10-20 10:55:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 41953,'1976-Y465','1976-10-20 14:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41954,'1976-Y466','1976-10-20 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.286','','','Aeron',1467,'',82 from dual union all
select 41955,'1976-Y467','1976-10-20 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',95 from dual union all
select 41956,'1976-Y468','1976-10-20 15:40:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 41957,'1976-Y469','1976-10-22 11:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',85 from dual union all
select 41958,'1976-Y47','1976-02-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 41959,'1976-Y470','1976-10-26 13:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',85 from dual union all
select 4196,'1956-S136','1956-07-28 00:00:00','military missile','success',992,'A3-5','','','','OT',636,'',170 from dual union all
select 41960,'1976-Y471','1976-10-26 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 41961,'1976-Y472','1976-10-26 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 41962,'1976-Y473','1976-10-27 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 41963,'1976-Y474','1976-10-27 11:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41964,'1976-Y475','1976-10-27 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',85 from dual union all
select 41965,'1976-Y476','1976-10-27 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',96 from dual union all
select 41966,'1976-Y477','1976-10-27 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.287','','','Aeron',1467,'',82 from dual union all
select 41967,'1976-Y478','1976-10-27 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 41968,'1976-Y479','1976-10-27 21:33:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',87 from dual union all
select 41969,'1976-Y48','1976-02-04 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.234','','','Aeron',1467,'',84 from dual union all
select 4197,'1956-S137','1956-07-28 00:00:00','military missile','success',992,'S19-18','','','','OT',636,'',170 from dual union all
select 41970,'1976-Y480','1976-10-29 18:57:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual union all
select 41971,'1976-Y481','1976-10-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',85 from dual union all
select 41972,'1976-Y482','1976-11-02 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',80 from dual union all
select 41973,'1976-Y483','1976-11-02 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 41974,'1976-Y484','1976-11-02 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 41975,'1976-Y485','1976-11-03 09:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41976,'1976-Y486','1976-11-03 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.288','','','Aeron',1467,'',83 from dual union all
select 41977,'1976-Y487','1976-11-03 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',90 from dual union all
select 41978,'1976-Y488','1976-11-03 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',96 from dual union all
select 41979,'1976-Y489','1976-11-06 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',83 from dual union all
select 4198,'1956-S138','1956-08-25 00:00:00','military missile','success',1013,'S1-10','8K51 KS-1','Control launch','','Test',636,'',500 from dual union all
select 41980,'1976-Y49','1976-02-04 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 41981,'1976-Y490','1976-11-09 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 41982,'1976-Y491','1976-11-09 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 41983,'1976-Y492','1976-11-10 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 41984,'1976-Y493','1976-11-10 15:00:00','suborbital rocket','unknown',727,'','ISRO-8.289','','','Aeron',1467,'',83 from dual union all
select 41985,'1976-Y494','1976-11-10 15:45:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 41986,'1976-Y495','1976-11-10 16:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 41987,'1976-Y496','1976-11-10 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',95 from dual union all
select 41988,'1976-Y497','1976-11-12 17:26:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 41989,'1976-Y498','1976-11-16 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',80 from dual union all
select 4199,'1956-S139','1956-09-16 00:00:00','military missile','success',1013,'A1-1TS','8K51 KS-2','','','Test',636,'',500 from dual union all
select 41990,'1976-Y499','1976-11-16 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 41991,'1976-Y50','1976-02-04 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 41992,'1976-Y500','1976-11-16 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 41993,'1976-Y501','1976-11-17 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.290','','','Aeron',1467,'',85 from dual union all
select 41994,'1976-Y502','1976-11-17 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 41995,'1976-Y503','1976-11-17 17:31:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',96 from dual union all
select 41996,'1976-Y504','1976-11-17 19:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',96 from dual union all
select 41997,'1976-Y505','1976-11-17 23:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 41998,'1976-Y506','1976-11-18 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',85 from dual union all
select 41999,'1976-Y507','1976-11-22 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',85 from dual union all
select 42,'1944-A06','1944-06-01 00:00:00','military missile','failure',8,'','','','','Test',675,'',0 from dual union all
select 420,'1944-S382','1944-12-28 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 4200,'1956-S14','1956-03-15 00:00:00','military missile','success',1013,'M-5RD A1-1','M5-RD 3','R-7 component test','','Test',636,'',500 from dual union all
select 42000,'1976-Y508','1976-11-23 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 42001,'1976-Y509','1976-11-23 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 42002,'1976-Y51','1976-02-04 17:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',95 from dual union all
select 42003,'1976-Y510','1976-11-23 22:31:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 42004,'1976-Y511','1976-11-24 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.291','','','Aeron',1467,'',82 from dual union all
select 42005,'1976-Y512','1976-11-24 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 42006,'1976-Y513','1976-11-24 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',96 from dual union all
select 42007,'1976-Y514','1976-11-24 17:09:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 42008,'1976-Y515','1976-11-24 17:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 42009,'1976-Y516','1976-11-26 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 4201,'1956-S140','1956-09-19 00:00:00','military missile','success',1013,'A1-2TS','8K51 KS-3','','','Test',636,'',500 from dual union all
select 42010,'1976-Y517','1976-11-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',81 from dual union all
select 42011,'1976-Y518','1976-11-30 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 42012,'1976-Y519','1976-11-30 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 42013,'1976-Y52','1976-02-05 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',84 from dual union all
select 42014,'1976-Y520','1976-12-01 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.292','','','Aeron',1467,'',81 from dual union all
select 42015,'1976-Y521','1976-12-01 15:56:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',95 from dual union all
select 42016,'1976-Y522','1976-12-01 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',86 from dual union all
select 42017,'1976-Y523','1976-12-01 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',96 from dual union all
select 42018,'1976-Y524','1976-12-01 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',86 from dual union all
select 42019,'1976-Y525','1976-12-03 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',81 from dual union all
select 4202,'1956-S141','1956-09-13 00:00:00','military missile','success',975,'A1-6','LKI 1B','','','Test',636,'',80 from dual union all
select 42020,'1976-Y526','1976-12-05 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',82 from dual union all
select 42021,'1976-Y527','1976-12-06 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',93 from dual union all
select 42022,'1976-Y528','1976-12-07 08:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 42023,'1976-Y529','1976-12-07 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 42024,'1976-Y53','1976-02-05 22:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',88 from dual union all
select 42025,'1976-Y530','1976-12-07 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 42026,'1976-Y531','1976-12-08 11:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 42027,'1976-Y532','1976-12-08 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.293','','','Aeron',1467,'',86 from dual union all
select 42028,'1976-Y533','1976-12-08 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',96 from dual union all
select 42029,'1976-Y534','1976-12-08 17:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 4203,'1956-S142','1956-09-29 00:00:00','military missile','success',992,'A4-20','','','','OT',636,'',170 from dual union all
select 42030,'1976-Y535','1976-12-08 18:24:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 42031,'1976-Y536','1976-12-09 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',85 from dual union all
select 42032,'1976-Y537','1976-12-10 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 42033,'1976-Y538','1976-12-10 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 42034,'1976-Y539','1976-12-10 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 42035,'1976-Y54','1976-02-06 15:33:59','suborbital rocket','success',716,'','Met Sphere','','','Meteo',428,'',101 from dual union all
select 42036,'1976-Y540','1976-12-10 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 42037,'1976-Y541','1976-12-10 21:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',86 from dual;
insert into LAUNCH
select 42038,'1976-Y542','1976-12-12 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',80 from dual union all
select 42039,'1976-Y543','1976-12-12 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',82 from dual union all
select 4204,'1956-S143','1956-10-24 00:00:00','military missile','success',970,'No 102','','','','OT',636,'',100 from dual union all
select 42040,'1976-Y544','1976-12-13 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',82 from dual union all
select 42041,'1976-Y545','1976-12-13 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 42042,'1976-Y546','1976-12-14 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 42043,'1976-Y547','1976-12-14 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 42044,'1976-Y548','1976-12-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 42045,'1976-Y549','1976-12-15 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',97 from dual union all
select 42046,'1976-Y55','1976-02-06 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',91 from dual union all
select 42047,'1976-Y550','1976-12-15 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',87 from dual union all
select 42048,'1976-Y551','1976-12-15 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 42049,'1976-Y552','1976-12-16 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.294','','','Aeron',1467,'',85 from dual union all
select 4205,'1956-S144','1956-10-25 00:00:00','military missile','success',970,'No 103','','','','OT',636,'',100 from dual union all
select 42050,'1976-Y553','1976-12-16 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',98 from dual union all
select 42051,'1976-Y554','1976-12-16 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',83 from dual union all
select 42052,'1976-Y555','1976-12-17 07:36:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 42053,'1976-Y556','1976-12-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 42054,'1976-Y557','1976-12-17 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',84 from dual union all
select 42055,'1976-Y558','1976-12-17 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',84 from dual union all
select 42056,'1976-Y559','1976-12-18 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',87 from dual union all
select 42057,'1976-Y56','1976-02-07 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',87 from dual union all
select 42058,'1976-Y560','1976-12-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',87 from dual union all
select 42059,'1976-Y561','1976-12-21 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 4206,'1956-S145','1956-10-25 00:00:00','military missile','success',992,'0310','','','','OT',636,'',170 from dual union all
select 42060,'1976-Y562','1976-12-21 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 42061,'1976-Y563','1976-12-22 08:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 42062,'1976-Y564','1976-12-22 09:35:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 42063,'1976-Y565','1976-12-22 14:30:00','suborbital rocket','unknown',727,'','ISRO-8.295','','','Aeron',1467,'',81 from dual union all
select 42064,'1976-Y566','1976-12-22 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 42065,'1976-Y567','1976-12-22 22:01:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',86 from dual union all
select 42066,'1976-Y568','1976-12-23 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 42067,'1976-Y569','1976-12-23 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',87 from dual union all
select 42068,'1976-Y57','1976-02-07 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',98 from dual union all
select 42069,'1976-Y570','1976-12-23 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',87 from dual union all
select 4207,'1956-S146','1956-10-25 00:00:00','military missile','success',992,'A3-10','','','','OT',636,'',170 from dual union all
select 42070,'1976-Y571','1976-12-25 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',87 from dual union all
select 42071,'1976-Y572','1976-12-25 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',90 from dual union all
select 42072,'1976-Y573','1976-12-25 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',86 from dual union all
select 42073,'1976-Y574','1976-12-26 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',85 from dual union all
select 42074,'1976-Y575','1976-12-26 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',85 from dual union all
select 42075,'1976-Y576','1976-12-27 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',81 from dual union all
select 42076,'1976-Y577','1976-12-27 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',87 from dual union all
select 42077,'1976-Y578','1976-12-28 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',88 from dual union all
select 42078,'1976-Y579','1976-12-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',84 from dual union all
select 42079,'1976-Y58','1976-02-08 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',87 from dual union all
select 4208,'1956-S147','1956-10-26 00:00:00','military missile','success',970,'No 104','','','','OT',636,'',100 from dual union all
select 42080,'1976-Y580','1976-12-28 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 42081,'1976-Y581','1976-12-29 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 42082,'1976-Y582','1976-12-29 14:30:00','suborbital rocket','unknown',728,'','ISRO 8.296','','','Aeron',1467,'',83 from dual union all
select 42083,'1976-Y583','1976-12-29 17:15:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',94 from dual union all
select 42084,'1976-Y584','1976-12-29 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 42085,'1976-Y585','1976-12-30 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',84 from dual union all
select 42086,'1976-Y586','1976-12-30 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',86 from dual union all
select 42087,'1976-Y587','1976-12-30 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',86 from dual union all
select 42088,'1976-Y59','1976-02-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',88 from dual union all
select 42089,'1976-Y60','1976-02-09 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',89 from dual union all
select 4209,'1956-S148','1956-11-03 00:00:00','military missile','success',1013,'A1-3TS','8K51 KS-4','','','Test',636,'',500 from dual union all
select 42090,'1976-Y61','1976-02-11 10:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 42091,'1976-Y62','1976-02-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 42092,'1976-Y63','1976-02-11 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.235','','','Aeron',1467,'',83 from dual union all
select 42093,'1976-Y64','1976-02-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 42094,'1976-Y65','1976-02-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',89 from dual union all
select 42095,'1976-Y66','1976-02-11 17:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',95 from dual union all
select 42096,'1976-Y67','1976-02-12 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.236','','','Aeron',1467,'',86 from dual union all
select 42097,'1976-Y68','1976-02-13 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 42098,'1976-Y69','1976-02-13 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',90 from dual union all
select 42099,'1976-Y70','1976-02-13 21:38:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',89 from dual union all
select 421,'1944-S383','1944-12-31 21:35:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 4210,'1956-S149','1956-11-03 00:00:00','military missile','success',992,'A5-20','','','','OT',636,'',170 from dual union all
select 42100,'1976-Y71','1976-02-14 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',89 from dual union all
select 42101,'1976-Y72','1976-02-15 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',89 from dual union all
select 42102,'1976-Y73','1976-02-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',89 from dual union all
select 42103,'1976-Y74','1976-02-16 18:43:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 42104,'1976-Y75','1976-02-18 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 42105,'1976-Y76','1976-02-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 42106,'1976-Y77','1976-02-18 14:30:00','suborbital rocket','unknown',728,'','ISRO-8.237','','','Aeron',1467,'',91 from dual union all
select 42107,'1976-Y78','1976-02-18 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',85 from dual union all
select 42108,'1976-Y79','1976-02-18 15:59:59','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',1057,'',85 from dual union all
select 42109,'1976-Y80','1976-02-18 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 4211,'1956-S15','1956-03-17 00:00:00','military missile','success',1013,'M-5RD A1-5','M5-RD 4','R-7 component test','','Test',636,'',500 from dual union all
select 42110,'1976-Y81','1976-02-18 21:08:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',87 from dual union all
select 42111,'1976-Y82','1976-02-19 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 42112,'1976-Y83','1976-02-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',91 from dual union all
select 42113,'1976-Y84','1976-02-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',82 from dual union all
select 42114,'1976-Y85','1976-02-20 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 42115,'1976-Y86','1976-02-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',83 from dual union all
select 42116,'1976-Y87','1976-02-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1373,'SHIR',91 from dual union all
select 42117,'1976-Y88','1976-02-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',89 from dual union all
select 42118,'1976-Y89','1976-02-22 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',88 from dual union all
select 42119,'1976-Y90','1976-02-22 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',89 from dual union all
select 4212,'1956-S150','1956-11-12 00:00:00','military missile','success',970,'No 303','','','','OT',636,'',100 from dual union all
select 42120,'1976-Y91','1976-02-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1392,'SHOK',80 from dual union all
select 42121,'1976-Y92','1976-02-24 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 42122,'1976-Y93','1976-02-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 42123,'1976-Y94','1976-02-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',82 from dual union all
select 42124,'1976-Y95','1976-02-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',94 from dual union all
select 42125,'1976-Y96','1976-02-25 16:00:00','suborbital rocket','unknown',728,'','ISRO-8.238','','','Aeron',1467,'',87 from dual union all
select 42126,'1976-Y97','1976-02-25 21:06:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',855,'',86 from dual union all
select 42127,'1976-Y98','1976-02-26 01:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual;
insert into LAUNCH
select 42128,'1976-Y99','1976-02-26 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 42129,'1977-001','1977-01-06 09:40:00','orbital','success',1238,'78026  -675','Kosmos-888','Zenit-4MK','','Sat',562,'',null from dual union all
select 4213,'1956-S151','1956-11-13 00:00:00','military missile','success',1013,'A1-6TS','8K51 KS-5','','','Test',636,'',500 from dual union all
select 42130,'1977-002','1977-01-06 23:17:00','orbital','success',1510,'76042  -390','Meteor-2','Meteor-2 No. 2','','Sat',474,'',null from dual union all
select 42131,'1977-003','1977-01-20 08:30:00','orbital','success',1238,'78026  -224','Kosmos-889','Zenit-2M','','Sat',562,'',null from dual union all
select 42132,'1977-004','1977-01-20 20:05:00','orbital','success',680,'53719-155','Kosmos-890','Zaliv','','Sat',456,'',null from dual union all
select 42133,'1977-005','1977-01-28 00:50:00','orbital','success',386,'613/D128','NATO 3B','NATO 3B','','Sat',233,'',null from dual union all
select 42134,'1977-006','1977-02-02 12:30:00','orbital','success',680,'53746-306','Kosmos-891','Vektor','','Sat',457,'',null from dual union all
select 42135,'1977-007','1977-02-06 06:00:01','orbital','success',1425,'23C-5  (3C-23)','DSP 9','DSP 9','','Sat',274,'',null from dual union all
select 42136,'1977-008','1977-02-07 16:11:50','orbital','success',1238,'E 15000-092','Soyuz-24','Soyuz 7K-T(A9) No. 66','','Sat',480,'',null from dual union all
select 42137,'1977-009','1977-02-09 11:30:00','orbital','success',1238,'78026  -667','Kosmos-892','Zenit-4MK','','Sat',474,'',null from dual union all
select 42138,'1977-010','1977-02-11 14:57:00','orbital','success',809,'76041  -577','Molniya-2','Molniya-2','','Sat',475,'',null from dual union all
select 42139,'1977-011','1977-02-15 11:00:00','orbital','success',680,'53721-261','Kosmos-893','GVM DS-U2-IK','','Sat',457,'',null from dual union all
select 4214,'1956-S152','1956-11-16 00:00:00','military missile','success',1013,'A1-4TS','8K51 KS-6','','','Test',636,'',500 from dual union all
select 42140,'1977-012','1977-02-19 05:15:00','orbital','success',818,'M-3H-1','Tansei-3/KM-A','MS-T3','','Sat',835,'',null from dual union all
select 42141,'1977-013','1977-02-21 17:20:00','orbital','success',680,'47182-428','Kosmos-894','Parus','','Sat',456,'',null from dual union all
select 42142,'1977-014','1977-02-23 08:50:00','orbital','success',822,'N-3(F)','Kiku 2','ETS-2','','Sat',1474,'',null from dual union all
select 42143,'1977-015','1977-02-26 21:18:00','orbital','success',1510,'77052  -391','Kosmos-895','Tselina-D','','Sat',475,'',null from dual union all
select 42144,'1977-016','1977-03-03 10:30:00','orbital','success',1238,'77052  -023','Kosmos-896','Zenit-6','','Sat',475,'',null from dual union all
select 42145,'1977-017','1977-03-10 11:00:00','orbital','success',1238,'78026  -670','Kosmos-897','Zenit-4MK','','Sat',474,'',null from dual union all
select 42146,'1977-018','1977-03-10 23:16:00','orbital','success',386,'612/D129','Palapa A2','Palapa A2','','Sat',233,'',null from dual union all
select 42147,'1977-019','1977-03-13 18:41:10','orbital','success',1403,'24B-19 (3B-54)','KH-8A','GAMBIT-3 Mission 4348','','Sat',1778,'',null from dual union all
select 42148,'1977-020','1977-03-17 08:30:00','orbital','success',1238,'76042  -235','Kosmos-898','Zenit-2M','','Sat',474,'',null from dual union all
select 42149,'1977-021','1977-03-24 11:51:00','orbital','success',809,'78026  -599','Molniya-1','Molniya-1','','Sat',475,'',null from dual union all
select 4215,'1956-S153','1956-11-24 00:00:00','military missile','success',1013,'A1-20','8K51 KS-7','','','Test',636,'',500 from dual union all
select 42150,'1977-022','1977-03-24 22:11:00','orbital','success',680,'53749-159','Kosmos-899','Tselina-OM','','Sat',457,'',null from dual union all
select 42151,'1977-023','1977-03-29 23:00:00','orbital','success',680,'53749-168','Kosmos-900','AUOS-Z-R-O Oval','','Sat',457,'',null from dual union all
select 42152,'1977-024','1977-04-05 02:05:00','orbital','success',1510,'78033  -348','Meteor','Meteor-M  No. 39','','Sat',474,'',null from dual union all
select 42153,'1977-025','1977-04-05 10:30:00','orbital','success',678,'45739-186','Kosmos-901','DS-P1-I No. 18','','Sat',460,'',null from dual union all
select 42154,'1977-026','1977-04-07 08:59:00','orbital','success',1238,'78026  -671','Kosmos-902','Zenit-4MK','','Sat',475,'',null from dual union all
select 42155,'1977-027','1977-04-11 01:38:00','orbital','success',809,'E 15000-160','Kosmos-903','Oko','','Sat',474,'',null from dual union all
select 42156,'1977-028','1977-04-20 09:00:00','orbital','success',1238,'76042  -236','Kosmos-904','Zenit-2M','','Sat',562,'',null from dual union all
select 42157,'1977-029','1977-04-20 10:15:00','orbital','failure',386,'617/D130','Geos 1','Geos 1','','Sat',234,'',null from dual union all
select 42158,'1977-030','1977-04-26 14:45:00','orbital','success',1238,'E 15000-031','Kosmos-905','Yantar''-2K No. 6','','Sat',474,'',null from dual union all
select 42159,'1977-031','1977-04-27 03:30:00','orbital','success',680,'53731-287','Kosmos-906','ER763-4','','Sat',622,'',null from dual union all
select 4216,'1956-S154','1956-12-06 00:00:00','military missile','success',1013,'A1-5TS','8K51 KS-8','','','Test',636,'',500 from dual union all
select 42160,'1977-032','1977-04-28 09:10:00','orbital','success',809,'76026  -604','Molniya-3','Molniya-3 No. 19','','Sat',475,'',null from dual union all
select 42161,'1977-033','1977-05-05 14:00:00','orbital','success',1238,'76042  -687','Kosmos-907','Zenit-4MK','','Sat',474,'',null from dual union all
select 42162,'1977-034','1977-05-12 14:26:58','orbital','success',1425,'23C-14 (3C-32)','DSCS II 7/8','DSCS II C-7','','Sat',274,'',null from dual union all
select 42163,'1977-035','1977-05-17 10:10:00','orbital','success',1238,'76042  -688','Kosmos-908','Zenit-4MK','','Sat',562,'',null from dual union all
select 42164,'1977-036','1977-05-19 16:30:01','orbital','success',680,'53719-156','Kosmos-909','Lira','','Sat',457,'',null from dual union all
select 42165,'1977-037','1977-05-23 12:14:51','orbital','success',1446,'51026-502','Kosmos-910','IS','','Sat',589,'',null from dual union all
select 42166,'1977-038','1977-05-23 18:13:00','orbital','success',178,'5507A','AFP-827','CANYON 7','','Sat',228,'',null from dual union all
select 42167,'1977-039','1977-05-25 11:00:00','orbital','success',680,'53749-163','Kosmos-911','Geoid','','Sat',457,'',null from dual union all
select 42168,'1977-040','1977-05-26 07:00:00','orbital','success',1238,'78018  -034','Kosmos-912','Fram','','Sat',475,'',null from dual union all
select 42169,'1977-041','1977-05-26 21:47:01','orbital','success',180,'AC-39','Intelsat 4A F-4','INTELSAT IVA F4','','Sat',268,'',null from dual union all
select 4217,'1956-S155','1956-12-12 00:00:00','military missile','success',1013,'A2-20','8K51 KS-9','','','Test',636,'',500 from dual union all
select 42170,'1977-042','1977-05-30 22:30:00','orbital','success',680,'53749-161','Kosmos-913','Romb 3','','Sat',456,'',null from dual union all
select 42171,'1977-043','1977-05-31 07:30:00','orbital','success',1238,'76042  -237','Kosmos-914','Zenit-2M','','Sat',562,'',null from dual union all
select 42172,'1977-044','1977-06-05 02:59:02','orbital','success',1374,'183','DMSP 5D F-2','DMSP Block 5D S-2','','Sat',1720,'',null from dual union all
select 42173,'1977-045','1977-06-08 14:00:00','orbital','success',1238,'76042  -689','Kosmos-915','Zenit-4MK','','Sat',475,'',null from dual union all
select 42174,'1977-046','1977-06-10 08:00:00','orbital','success',1238,'78026  -863','Kosmos-916','Orion','','Sat',474,'',null from dual union all
select 42175,'1977-047','1977-06-16 01:58:00','orbital','success',809,'D 15000-162','Kosmos-917','Oko','','Sat',475,'',null from dual union all
select 42176,'1977-048','1977-06-16 10:51:00','orbital','success',386,'616/D131','GOES 2','GOES B','','Sat',234,'',null from dual union all
select 42177,'1977-049','1977-06-17 03:30:00','orbital','success',680,'47182-422','Signe 3','Signe 3','','Sat',622,'',null from dual union all
select 42178,'1977-050','1977-06-17 07:23:10','orbital','success',1446,'45046-501','Kosmos-918','IS','','Sat',589,'',null from dual union all
select 42179,'1977-051','1977-06-18 10:30:00','orbital','success',678,'64619-185','Kosmos-919','DS-P1-I No. 19','','Sat',460,'',null from dual union all
select 4218,'1956-S156','1956-12-06 00:00:00','military missile','success',970,'No 103','','','','OT',636,'',100 from dual union all
select 42180,'1977-052','1977-06-22 08:00:00','orbital','success',1238,'76042  -691','Kosmos-920','Zenit-4MK','','Sat',562,'',null from dual union all
select 42181,'1977-053','1977-06-23 08:17:00','orbital','success',167,'65F','P76-4','NTS 2','','Sat',1776,'',null from dual union all
select 42182,'1977-054','1977-06-24 05:41:00','orbital','success',809,'76041  -580','Molniya-1','Molniya-1 11F658 No. 45','','Sat',480,'',null from dual union all
select 42183,'1977-055','1977-06-24 10:30:00','orbital','success',1448,'No. 2L','Kosmos-921','EPN No. 03.0380?','','Sat',470,'',null from dual union all
select 42184,'1977-056','1977-06-27 18:30:00','orbital','success',1426,'23D-17','HEXAGON SV13','HEXAGON SV-13','','Sat',1777,'',null from dual union all
select 42185,'1977-057','1977-06-29 18:34:00','orbital','success',1510,'77016  -382','Meteor-Priroda','Meteor-Priroda No. 2-2','','Sat',562,'',null from dual union all
select 42186,'1977-058','1977-06-30 14:00:00','orbital','success',1238,'76042  -239','Kosmos-922','Zenit-2M','','Sat',475,'',null from dual union all
select 42187,'1977-059','1977-07-01 11:52:00','orbital','success',680,'53749-162','Kosmos-923','Strela-2M','','Sat',456,'',null from dual union all
select 42188,'1977-060','1977-07-04 22:20:00','orbital','success',680,'53746-303','Kosmos-924','Tselina-OK','','Sat',457,'',null from dual union all
select 42189,'1977-061','1977-07-07 07:25:00','orbital','success',1510,'76041  -367','Kosmos-925','Tselina-D','','Sat',475,'',null from dual union all
select 4219,'1956-S16','1956-03-05 21:20:00','suborbital rocket','success',1521,'D-5','','','','Test',253,'',116 from dual union all
select 42190,'1977-062','1977-07-08 17:30:00','orbital','success',680,'53731-262','Kosmos-926','Tsikada','','Sat',456,'',null from dual union all
select 42191,'1977-063','1977-07-12 09:00:00','orbital','success',1238,'76042  -692','Kosmos-927','Zenit-4MKM','','Sat',475,'',null from dual union all
select 42192,'1977-064','1977-07-13 05:02:00','orbital','success',680,'53716-331','Kosmos-928','Parus','','Sat',456,'',null from dual union all
select 42193,'1977-065','1977-07-14 10:39:00','orbital','success',386,'618/D132','GMS','GMS','','Sat',234,'',null from dual union all
select 42194,'1977-066','1977-07-17 09:00:00','orbital','success',949,'293-02','Kosmos-929','TKS No. 16101','','Sat',588,'',null from dual union all
select 42195,'1977-067','1977-07-19 08:40:00','orbital','success',680,'53721-264','Kosmos-930','Vektor','','Sat',457,'',null from dual union all
select 42196,'1977-068','1977-07-20 04:44:00','orbital','success',809,'77052  -609','Kosmos-931','Oko','','Sat',475,'',null from dual union all
select 42197,'1977-069','1977-07-20 07:35:00','orbital','success',1238,'76042  -693','Kosmos-932','Zenit-4MKM','','Sat',562,'',null from dual union all
select 42198,'1977-070','1977-07-22 10:00:00','orbital','success',680,'53731-267','Kosmos-933','Vektor','','Sat',456,'',null from dual union all
select 42199,'1977-071','1977-07-23 21:15:00','orbital','success',956,'291-01','Raduga','Gran'' No. 13L','','Sat',554,'',null from dual union all
select 422,'1944-S39','1944-08-15 10:54:00','military missile','success',8,'18312','Ma114','','','Training',661,'',90 from dual union all
select 4220,'1956-S17','1956-03-12 21:15:00','suborbital rocket','success',51,'USAF 62','Release B (NO)','','','Ionos',640,'',95 from dual union all
select 42200,'1977-072','1977-07-27 18:07:00','orbital','success',1238,'E 15000-082','Kosmos-934','Zenit-6','','Sat',475,'',null from dual union all
select 42201,'1977-073','1977-07-29 08:00:00','orbital','success',1238,'77052  -244','Kosmos-935','Zenit-2M','','Sat',475,'',null from dual union all
select 42202,'1977-074','1977-08-03 14:01:00','orbital','success',1238,'D 15000-113','Kosmos-936','Bion No. 4','','Sat',474,'',null from dual union all
select 42203,'1977-075','1977-08-12 06:29:31','orbital','success',180,'AC-45','HEAO 1','HEAO A','','Sat',269,'',null from dual union all
select 42204,'1977-076','1977-08-20 14:29:44','deep space','success',1427,'23E-7/TC-7','Voyager 2','Voyager 2','','Sat',275,'',null from dual union all
select 42205,'1977-077','1977-08-24 07:07:00','orbital','success',1446,'51052-103T','Kosmos-937','US-P','','Sat',589,'',null from dual union all
select 42206,'1977-078','1977-08-24 14:29:59','orbital','success',1238,'77052  -695','Kosmos-938','Zenit-4MKM','','Sat',475,'',null from dual union all
select 42207,'1977-079','1977-08-24 18:20:00','orbital','success',680,'53716-326','Kosmos-939/946','Strela-1M','','Sat',456,'',null from dual union all
select 42208,'1977-080','1977-08-25 23:49:59','orbital','success',382,'615/D133','Sirio','Sirio 1','','Sat',234,'',null from dual union all
select 42209,'1977-081','1977-08-27 10:09:00','orbital','success',1238,'78018  -110','Kosmos-947','Zenit-2M','','Sat',474,'',null from dual union all
select 4221,'1956-S18','1956-03-14 08:45:00','suborbital rocket','success',51,'USAF 63','Release (NO)','','','Ionos/Aeronomy',640,'',106 from dual union all
select 42210,'1977-082','1977-08-30 18:06:00','orbital','success',809,'77016  -593','Molniya-1','Molniya-1','','Sat',474,'',null from dual union all
select 42211,'1977-083','1977-09-02 09:00:00','orbital','success',1238,'76038  -035','Kosmos-948','Fram','','Sat',475,'',null from dual union all
select 42212,'1977-084','1977-09-05 12:56:01','deep space','success',1427,'23E-6/TC-6','Voyager 1','Voyager 1','','Sat',275,'',null from dual union all
select 42213,'1977-085','1977-09-06 17:30:00','orbital','success',1238,'77052  -961','Kosmos-949','Yantar''-2K No. 7','','Sat',474,'',null from dual union all
select 42214,'1977-086','1977-09-13 15:10:00','orbital','success',1238,'76038  -111','Kosmos-950','Zenit-2M','','Sat',474,'',null from dual union all
select 42215,'1977-087','1977-09-13 19:59:00','orbital','success',680,'65082-436','Kosmos-951','Parus','','Sat',456,'',null from dual union all
select 42216,'1977-088','1977-09-16 14:25:00','orbital','success',1446,'51026-505','Kosmos-952','US-A reactor','','Sat',589,'',null from dual union all
select 42217,'1977-089','1977-09-16 14:30:00','orbital','success',1238,'77052  -696','Kosmos-953','Zenit-4MKM','','Sat',474,'',null from dual;
insert into LAUNCH
select 42218,'1977-090','1977-09-18 13:48:00','orbital','success',1446,'67012-101','Kosmos-954','US-A','','Sat',589,'',null from dual union all
select 42219,'1977-091','1977-09-20 01:00:59','orbital','success',1510,'78026  -393','Kosmos-955','Tselina-D','','Sat',475,'',null from dual union all
select 4222,'1956-S19','1956-03-15 00:36:00','military missile','success',635,'RS-18    HL','Jupiter A','','','Test',280,'',90 from dual union all
select 42220,'1977-092','1977-09-20 17:28:00','orbital','success',956,'291-02','Ekran','Ekran No. 12L','','Sat',554,'',null from dual union all
select 42221,'1977-093','1977-09-22 00:51:00','orbital','success',807,'D 15000-163','Prognoz-6','SO-M No. 506','','Sat',562,'',null from dual union all
select 42222,'1977-094','1977-09-23 18:34:00','orbital','success',1403,'24B-23 (3B-58)','KH-8A','GAMBIT-3 Mission 4349','','Sat',1778,'',null from dual union all
select 42223,'1977-095','1977-09-24 10:15:00','orbital','success',1448,'45028-201','Kosmos-956','EPN No. 03.0380?','','Sat',470,'',null from dual union all
select 42224,'1977-096','1977-09-24 16:30:00','orbital','success',680,'53731-278','Interkosmos-17','AUOS-Z-R-IK','','Sat',456,'',null from dual union all
select 42225,'1977-097','1977-09-29 06:50:00','orbital','success',949,'295-01','Salyut-6','DOS 5','','Sat',588,'',null from dual union all
select 42226,'1977-098','1977-09-30 09:46:00','orbital','success',1238,'77052  -697','Kosmos-957','Zenit-4MKM','','Sat',562,'',null from dual union all
select 42227,'1977-099','1977-10-09 02:40:35','orbital','success',1238,'D 15000-103','Soyuz-25','Soyuz 7K-T No. 42','','Sat',480,'',null from dual union all
select 42228,'1977-100','1977-10-11 15:14:00','orbital','success',1238,'D 15000-099','Kosmos-958','Zenit-6','','Sat',475,'',null from dual union all
select 42229,'1977-101','1977-10-21 10:05:00','orbital','success',680,'53716-325','Kosmos-959','Lira','','Sat',456,'',null from dual union all
select 4223,'1956-S20','1956-03-23 00:00:00','military missile','success',1013,'M-5RD S1-2','M5-RD 5','R-7 component test','','Test',636,'',500 from dual union all
select 42230,'1977-102','1977-10-22 13:53:00','orbital','success',386,'623/D135','ISEE 1/2','ISEE A','','Sat',234,'',null from dual union all
select 42231,'1977-103','1977-10-25 05:25:00','orbital','success',680,'53746-309','Kosmos-960','Tselina-OM','','Sat',456,'',null from dual union all
select 42232,'1977-104','1977-10-26 05:14:00','orbital','success',1446,'67026-503','Kosmos-961','IS','','Sat',589,'',null from dual union all
select 42233,'1977-105','1977-10-28 01:37:00','orbital','success',809,'77052  -606','Molniya-3','Molniya-3 No. 18','','Sat',474,'',null from dual union all
select 42234,'1977-106','1977-10-28 04:52:04','orbital','success',1132,'S200C','Transat','NNS O-11','','Sat',1779,'',null from dual union all
select 42235,'1977-107','1977-10-28 16:00:00','orbital','success',680,'53768-313','Kosmos-962','Zaliv','','Sat',456,'',null from dual union all
select 42236,'1977-108','1977-11-23 01:35:00','orbital','success',386,'620/D136','Meteosat 1','Meteosat 1','','Sat',233,'',null from dual union all
select 42237,'1977-109','1977-11-24 14:30:00','orbital','success',680,'53746-302','Kosmos-963','Geoid','','Sat',456,'',null from dual union all
select 42238,'1977-110','1977-12-04 12:00:00','orbital','success',1238,'77052  -698','Kosmos-964','Zenit-4MKM','','Sat',475,'',null from dual union all
select 42239,'1977-111','1977-12-08 11:00:00','orbital','success',680,'65068-308','Kosmos-965','Romb 4','','Sat',457,'',null from dual union all
select 4224,'1956-S21','1956-03-28 00:00:00','military missile','success',970,'No 0316','','','','OT',636,'',100 from dual union all
select 42240,'1977-112','1977-12-08 17:45:00','orbital','success',164,'50F','OPS 8781    [NOSS2]','PARCAE 2','','Sat',1776,'',null from dual union all
select 42241,'1977-113','1977-12-10 01:18:40','orbital','success',1238,'D 15000-093','Soyuz-26','Soyuz 7K-T No. 43','','Sat',480,'',null from dual union all
select 42242,'1977-114','1977-12-11 22:45:01','orbital','success',178,'5504A','OPS 4258','AQUACADE 3','','Sat',228,'',null from dual union all
select 42243,'1977-115','1977-12-12 09:40:00','orbital','success',1238,'78018  -105','Kosmos-966','Zenit-2M','','Sat',562,'',null from dual union all
select 42244,'1977-116','1977-12-13 15:53:00','orbital','success',680,'53716-332','Kosmos-967','Lira','','Sat',456,'',null from dual union all
select 42245,'1977-117','1977-12-14 09:30:00','orbital','success',1510,'78018  -305','Meteor-2','Meteor-2 No. 3','','Sat',475,'',null from dual union all
select 42246,'1977-118','1977-12-15 00:47:03','orbital','success',386,'624/D137','CS','CS','','Sat',234,'',null from dual union all
select 42247,'1977-119','1977-12-16 04:25:00','orbital','success',680,'53746-305','Kosmos-968','Strela-2M','','Sat',456,'',null from dual union all
select 42248,'1977-120','1977-12-20 15:50:00','orbital','success',1238,'77052  -699','Kosmos-969','Zenit-4MKM','','Sat',475,'',null from dual union all
select 42249,'1977-121','1977-12-21 10:35:00','orbital','success',1446,'67023-302T','Kosmos-970','IS','','Sat',589,'',null from dual union all
select 4225,'1956-S22','1956-04-12 02:05:00','suborbital rocket','success',51,'USAF 64','Na Release 3','','','Ionos/Aeronomy',640,'',106 from dual union all
select 42250,'1977-122','1977-12-23 16:24:00','orbital','success',680,'53716-322','Kosmos-971','Parus','','Sat',456,'',null from dual union all
select 42251,'1977-123','1977-12-27 08:00:00','orbital','success',1448,'51028-202','Kosmos-972','EPN No. 03.0380?','','Sat',470,'',null from dual union all
select 42252,'1977-124','1977-12-27 09:20:00','orbital','success',1238,'77052  -243','Kosmos-973','Zenit-2M','','Sat',562,'',null from dual union all
select 42253,'1977-A01','1977-01-03 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',48 from dual union all
select 42254,'1977-A02','1977-01-04 01:08:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 42255,'1977-A03','1977-01-11 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 42256,'1977-A04','1977-01-14 13:19:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',49 from dual union all
select 42257,'1977-A05','1977-01-18 05:43:20','suborbital rocket','failure',1337,'NASA 29.04UE','','','','Auroral/Plasma',1047,'',11 from dual union all
select 42258,'1977-A06','1977-01-19 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',48 from dual union all
select 42259,'1977-A07','1977-01-21 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',48 from dual union all
select 4226,'1956-S23','1956-04-16 00:00:00','military missile','success',970,'No 0317','','','','OT',636,'',100 from dual union all
select 42260,'1977-A08','1977-02-04 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 42261,'1977-A09','1977-02-07 22:05:00','suborbital rocket','success',920,'P195H','','','','Aeron/Ionos',1447,'',46 from dual union all
select 42262,'1977-A10','1977-02-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',36 from dual union all
select 42263,'1977-A11','1977-02-18 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',47 from dual union all
select 42264,'1977-A12','1977-02-25 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',47 from dual union all
select 42265,'1977-A13','1977-03-02 18:53:00','military missile','failure',917,'FOT-55','P-342 76 Cape 07','','','OT',231,'',0 from dual union all
select 42266,'1977-A14','1977-03-02 21:42:00','military missile','success',917,'FOT-57','P-344 76 Cape 09','','','OT CC-102',231,'',180 from dual union all
select 42267,'1977-A15','1977-03-14 16:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',37 from dual union all
select 42268,'1977-A16','1977-03-21 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',47 from dual union all
select 42269,'1977-A17','1977-03-24 16:04:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',47 from dual union all
select 4227,'1956-S24','1956-04-17 10:30:00','suborbital rocket','success',1521,'R-1','','','','Reentry Test',253,'',100 from dual union all
select 42270,'1977-A18','1977-04-01 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 42271,'1977-A19','1977-04-14 11:59:00','military missile','success',1099,'Case 29','','','','Test',398,'B-52',10 from dual union all
select 42272,'1977-A20','1977-04-15 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 42273,'1977-A21','1977-04-20 16:56:00','suborbital rocket','unknown',1284,'T 1-8721','','','','Aeron',1821,'',49 from dual union all
select 42274,'1977-A22','1977-05-04 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',27 from dual union all
select 42275,'1977-A23','1977-05-09 12:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',46 from dual union all
select 42276,'1977-A24','1977-05-11 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 42277,'1977-A25','1977-05-13 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 42278,'1977-A26','1977-05-13 13:44:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',49 from dual union all
select 42279,'1977-A27','1977-05-14 00:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 4228,'1956-S25','1956-05-22 00:00:00','suborbital rocket','success',813,'I','Camp. 6 to jun 13','','','Test',313,'',90 from dual union all
select 42280,'1977-A28','1977-05-16 15:14:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',29 from dual union all
select 42281,'1977-A29','1977-05-17 08:15:00','suborbital rocket','success',885,'1','4x9m','','','Test',1355,'',12 from dual union all
select 42282,'1977-A30','1977-05-20 23:09:00','suborbital rocket','failure',38,'NASA 13.131DA','','','','Aeronomy',1895,'',23 from dual union all
select 42283,'1977-A31','1977-06-01 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',49 from dual union all
select 42284,'1977-A32','1977-06-13 21:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',41 from dual union all
select 42285,'1977-A33','1977-06-29 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',46 from dual union all
select 42286,'1977-A34','1977-07-01 00:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 42287,'1977-A35','1977-07-08 20:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',49 from dual union all
select 42288,'1977-A36','1977-07-13 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 42289,'1977-A37','1977-07-14 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 4229,'1956-S26','1956-05-01 00:00:00','suborbital rocket','success',813,'I','','','','Test',313,'',90 from dual union all
select 42290,'1977-A38','1977-08-09 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 42291,'1977-A39','1977-09-07 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',45 from dual union all
select 42292,'1977-A40','1977-09-09 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',46 from dual union all
select 42293,'1977-A41','1977-10-05 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',48 from dual union all
select 42294,'1977-A42','1977-10-20 00:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 42295,'1977-A43','1977-10-24 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',43 from dual union all
select 42296,'1977-A44','1977-10-29 00:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 42297,'1977-A45','1977-11-04 20:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',43 from dual union all
select 42298,'1977-A46','1977-11-09 01:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 42299,'1977-A47','1977-11-15 01:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 423,'1944-S40','1944-08-15 11:57:00','military missile','success',8,'18310','Ma128','','','Training',661,'',90 from dual union all
select 4230,'1956-S27','1956-05-08 14:54:00','suborbital rocket','success',48,'USAF 65','FUV spec/La imager','','','Solar UV',640,'',143 from dual union all
select 42300,'1977-A48','1977-11-19 23:52:00','suborbital rocket','success',835,'NERDD','','','','Test',1047,'',1 from dual union all
select 42301,'1977-A49','1977-12-09 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 42302,'1977-A50','1977-12-12 23:00:00','suborbital rocket','failure',210,'AAF-NVB-04','Dayside Cusp','','','Auroral/Ionos/Field',334,'',10 from dual union all
select 42303,'1977-A51','1977-12-14 12:54:00','military missile','success',1099,'Case 33','','','','Test',187,'B-52',10 from dual union all
select 42304,'1977-A52','1977-12-28 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',47 from dual union all
select 42305,'1977-A53','1977-01-01 00:00:00','suborbital rocket','success',773,'','Parachute Test','','','Test',1467,'',40 from dual union all
select 42306,'1977-A54','1977-08-24 00:00:00','atmospheric rocket','failure',12,'WSRL ADTV2','','','','Test',1832,'',1 from dual union all
select 42307,'1977-A55','1977-10-25 00:00:00','atmospheric rocket','success',12,'WSRL ADTV3','Falling Sphere','','','Test',1832,'',17 from dual;
insert into LAUNCH
select 42308,'1977-A56','1977-11-22 00:00:00','atmospheric rocket','success',12,'WSRL ADTV4','Falling Sphere','','','Test',1832,'',18 from dual union all
select 42309,'1977-A57','1977-11-23 00:00:00','atmospheric rocket','success',12,'WSRL ADTV4','Falling Sphere','','','Test',1832,'',18 from dual union all
select 4231,'1956-S28','1956-05-08 15:15:00','suborbital rocket','failure',49,'NRL 42','RV-N-13a','','','Test',1869,'',188 from dual union all
select 42310,'1977-A58','1977-06-01 00:00:00','military missile','success',702,'Test firing','','','','Test',984,'',10 from dual union all
select 42311,'1977-F01','1977-02-22 09:19:00','orbital','failure',1238,'78026  -672','','Zenit-4MK','','Sat',562,'',10 from dual union all
select 42312,'1977-F02','1977-08-04 22:00:00','orbital','failure',949,'293-01','','TKS VA No. 009A/P','','Sat',588,'',20 from dual union all
select 42313,'1977-F03','1977-08-10 10:40:00','orbital','failure',1238,'76042  -694','','Zenit-4MKM','','Sat',562,'',10 from dual union all
select 42314,'1977-F04','1977-09-13 23:31:00','orbital','failure',390,'619/D134','OTS 1','OTS 1','','Sat',233,'',10 from dual union all
select 42315,'1977-F05','1977-09-30 01:02:59','orbital','failure',180,'AC-43','Intelsat 4A F-5','INTELSAT IVA F5','','Sat',268,'',10 from dual union all
select 42316,'1977-F06','1977-11-29 07:05:00','orbital','failure',680,'53721-265','','Tsikada?','','Sat',456,'',85 from dual union all
select 42317,'1977-M01','1977-07-14 12:30:00','suborbital rocket','success',891,'NASA 12.27GT','Parachute Test','','','Test',1895,'',59 from dual union all
select 42318,'1977-M02','1977-08-12 04:19:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 2  Datasonde','','','Meteo',1821,'',73 from dual union all
select 42319,'1977-M03','1977-08-12 06:59:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 3  Datasonde','','','Meteo',1821,'',75 from dual union all
select 4232,'1956-S29','1956-05-14 00:00:00','suborbital rocket','success',988,'4','','','','Bio',636,'',100 from dual union all
select 42320,'1977-M04','1977-08-12 11:00:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 4  Datasonde','','','Meteo',1821,'',72 from dual union all
select 42321,'1977-M05','1977-08-15 17:44:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 5  Datasonde','','','Meteo',1821,'',72 from dual union all
select 42322,'1977-M06','1977-08-15 19:30:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 6  Datasonde','','','Meteo',1821,'',79 from dual union all
select 42323,'1977-M07','1977-08-15 19:42:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 6  Sphere','','','Meteo',1821,'',113 from dual union all
select 42324,'1977-M08','1977-08-16 01:13:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 7  Datasonde','','','Meteo',1821,'',77 from dual union all
select 42325,'1977-M09','1977-08-22 23:08:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 17 Datasonde','','','Meteo',1821,'',74 from dual union all
select 42326,'1977-M10','1977-08-16 20:00:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 9  Datasonde','','','Meteo',1821,'',78 from dual union all
select 42327,'1977-M11','1977-08-17 20:25:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 10 Datasonde','','','Meteo',1821,'',78 from dual union all
select 42328,'1977-M12','1977-08-19 01:16:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 11 Datasonde','','','Meteo',1821,'',79 from dual union all
select 42329,'1977-M13','1977-08-19 03:00:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 12 Datasonde','','','Meteo',1821,'',77 from dual union all
select 4233,'1956-S30','1956-05-31 00:00:00','military missile','success',1013,'R-5R S1-1','R-5R  1','Radio guidance test','','Test',636,'',500 from dual union all
select 42330,'1977-M14','1977-08-19 05:37:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 13 Datasonde','','','Meteo',1821,'',76 from dual union all
select 42331,'1977-M15','1977-08-16 01:05:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 7','','','Meteo',1822,'KORO',79 from dual union all
select 42332,'1977-M16','1977-08-19 20:00:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 14 Datasonde','','','Meteo',1821,'',77 from dual union all
select 42333,'1977-M17','1977-08-20 01:00:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 16 Datasonde','','','Meteo',1821,'',76 from dual union all
select 42334,'1977-M18','1977-08-23 01:00:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 18 Datasonde','','','Meteo',1821,'',74 from dual union all
select 42335,'1977-M19','1977-08-23 03:39:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 19 Datasonde','','','Meteo',1821,'',76 from dual union all
select 42336,'1977-M20','1977-08-24 01:28:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 20 Datasonde','','','Meteo',1821,'',73 from dual union all
select 42337,'1977-M21','1977-05-28 02:30:00','suborbital rocket','success',1284,'Intercomp prep 4A','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 42338,'1977-M22','1977-05-28 06:47:00','suborbital rocket','success',1284,'Intercomp prep 5B','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 42339,'1977-M23','1977-08-10 14:00:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 1  Datasonde','','','Meteo',1821,'',72 from dual union all
select 4234,'1956-S31','1956-05-16 04:21:00','military missile','success',635,'RS-19    HE','Jupiter A','','','Test',280,'',90 from dual union all
select 42340,'1977-M24','1977-05-28 10:42:00','suborbital rocket','success',1284,'Intercomp prep 6A','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 42341,'1977-M25','1977-05-28 10:47:00','suborbital rocket','success',1284,'Intercomp prep 6B','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 42342,'1977-M26','1977-04-27 00:00:00','military missile','success',1162,'','','RB350 Training','','OT',984,'',50 from dual union all
select 42343,'1977-S01','1977-01-01 00:00:00','military missile','success',6,'','LKI','','','Test',183,'',300 from dual union all
select 42344,'1977-S02','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42345,'1977-S03','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42346,'1977-S04','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42347,'1977-S05','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42348,'1977-S06','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42349,'1977-S07','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 4235,'1956-S32','1956-05-16 15:40:00','suborbital rocket','failure',49,'USAF 66','AJ11-6 test','','','Test',640,'',169 from dual union all
select 42350,'1977-S08','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42351,'1977-S09','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42352,'1977-S10','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42353,'1977-S100','1977-03-06 23:31:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/Plasma',1010,'',170 from dual union all
select 42354,'1977-S101','1977-03-07 23:58:00','suborbital rocket','success',543,'DNA ST710.31-4','STRESS DIANNE','','','Aeron/Ionos',389,'',186 from dual union all
select 42355,'1977-S102','1977-03-07 15:53:00','suborbital rocket','success',740,'MR-12','','','','Aeron',1010,'',170 from dual union all
select 42356,'1977-S103','1977-03-08 04:01:00','suborbital rocket','success',220,'AAF-4B-34','Pitch angles','','','Test/Auroral/Ionos',428,'',700 from dual union all
select 42357,'1977-S104','1977-03-09 00:00:00','suborbital rocket','success',860,'NASA 31.02UA','Auroral NO','','','Aeronomy',428,'',138 from dual union all
select 42358,'1977-S105','1977-03-09 16:40:00','suborbital rocket','success',228,'NASA 21.41US','','','','Solar',1873,'',191 from dual union all
select 42359,'1977-S106','1977-03-12 03:30:00','suborbital rocket','success',133,'NASA 25.20UG','','','','UV Astron',1895,'',225 from dual union all
select 4236,'1956-S34','1956-06-07 00:00:00','military missile','success',1013,'R-5R A1-3','R-5R  2','Radio guidance test','','Test',636,'',500 from dual union all
select 42360,'1977-S107','1977-03-13 22:58:00','suborbital rocket','success',543,'DNA ST710.31-5','STRESS ESTHER','','','Aeron/Ionos',389,'',189 from dual union all
select 42361,'1977-S108','1977-03-13 21:31:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/Plasma',1010,'',163 from dual union all
select 42362,'1977-S109','1977-03-14 22:43:00','suborbital rocket','success',543,'DNA ST710.31-6','STRESS FERN','','','Aeron/Ionos',389,'',186 from dual union all
select 42363,'1977-S11','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42364,'1977-S110','1977-03-14 17:38:00','suborbital rocket','success',837,'DFVLR Firefly','DLR K-NA-83','','','Auroral?',417,'',250 from dual union all
select 42365,'1977-S111','1977-03-16 00:00:00','military missile','success',1105,'S-1E-2607','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 42366,'1977-S112','1977-03-16 18:10:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Fields',1010,'',172 from dual union all
select 42367,'1977-S113','1977-03-16 20:05:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/Plasma',1010,'',160 from dual union all
select 42368,'1977-S114','1977-03-16 22:04:40','suborbital rocket','success',1212,'PHA F4D AB3-F4','DLR A-GR-81 Christ.','','','Aeron',29,'',263 from dual union all
select 42369,'1977-S115','1977-03-17 00:00:00','military missile','success',1004,'5T','15A14 UBP-10 SLI','GCh 15F143U2','','OT',490,'',1000 from dual union all
select 4237,'1956-S35','1956-05-31 02:57:00','suborbital rocket','success',988,'5','','','','Bio/Solar UV',636,'',100 from dual union all
select 42370,'1977-S116','1977-03-17 03:59:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos/Fields',1010,'',170 from dual union all
select 42371,'1977-S117','1977-03-18 00:00:00','military missile','success',1514,'','','','','Test',201,'PLBR',1000 from dual union all
select 42372,'1977-S118','1977-03-18 20:10:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/Plasma',1010,'',162 from dual union all
select 42373,'1977-S119','1977-03-18 21:05:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Fields',1010,'',170 from dual union all
select 42374,'1977-S12','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42375,'1977-S120','1977-03-19 06:06:00','suborbital rocket','success',133,'NASA 25.25UE','Auroral Oxides','','','Plasma',428,'',195 from dual union all
select 42376,'1977-S121','1977-03-20 00:00:00','military missile','success',1514,'','','','','Test',201,'PLBR',1000 from dual union all
select 42377,'1977-S122','1977-03-20 00:00:00','military missile','success',135,'Athena D','MSV-3','','','Reentry Test',1821,'',200 from dual union all
select 42378,'1977-S123','1977-03-20 19:22:00','suborbital rocket','success',117,'NASA 24.02IE','DLR K-AR-82','Porcupine 2','','Magnetospheric Ba',407,'',460 from dual union all
select 42379,'1977-S124','1977-03-21 06:22:00','suborbital rocket','success',227,'AND-VB-44','CMB spectrum','','','Submm Astron',428,'',178 from dual union all
select 4238,'1956-S37','1956-06-07 00:00:00','suborbital rocket','success',858,'','Heat Transfer','','','Reentry Test',1821,'',100 from dual union all
select 42380,'1977-S125','1977-03-22 06:30:00','suborbital rocket','success',41,'NASA 26.59UG','','','','UV Astron?',1895,'',189 from dual union all
select 42381,'1977-S126','1977-03-24 13:10:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos/Plasma/Fields',1010,'',160 from dual union all
select 42382,'1977-S127','1977-03-24 20:47:00','military missile','success',939,'','OT-28','','','OT',421,'SSBN 624',500 from dual union all
select 42383,'1977-S128','1977-03-24 20:47:00','military missile','success',939,'','OT-28','','','OT',421,'SSBN 624',500 from dual union all
select 42384,'1977-S129','1977-03-26 00:00:00','military missile','success',744,'','15A15 K-6','','','OT',530,'',1000 from dual union all
select 42385,'1977-S13','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42386,'1977-S130','1977-03-27 03:55:00','suborbital rocket','success',1070,'S-210JA-28','NO/O3/Ne','','','Aeron',1458,'',106 from dual union all
select 42387,'1977-S131','1977-03-28 15:17:00','military missile','success',1442,'C4X-3','','','','Test',246,'',1000 from dual union all
select 42388,'1977-S132','1977-03-28 19:06:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/Plasma',1010,'',155 from dual union all
select 42389,'1977-S133','1977-03-28 22:31:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/Plasma',1010,'',169 from dual union all
select 4239,'1956-S38','1956-06-07 00:00:00','suborbital rocket','success',989,'6','','','','Bio',636,'',100 from dual union all
select 42390,'1977-S134','1977-03-29 00:06:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/Plasma',1010,'',170 from dual union all
select 42391,'1977-S135','1977-03-29 02:16:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/Plasma',1010,'',168 from dual union all
select 42392,'1977-S136','1977-03-29 03:37:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/Plasma',1010,'',165 from dual union all
select 42393,'1977-S137','1977-03-30 12:02:00','military missile','success',939,'','OT-29','','','OT',421,'SSBN 654',500 from dual union all
select 42394,'1977-S138','1977-03-30 13:36:00','military missile','success',939,'','OT-29','','','OT',421,'SSBN 654',500 from dual union all
select 42395,'1977-S139','1977-04-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 42396,'1977-S14','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42397,'1977-S140','1977-04-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 42398,'1977-S141','1977-03-31 00:00:00','suborbital rocket','success',1219,'XV-03','','','','Ionos?',330,'',546 from dual;
insert into LAUNCH
select 42399,'1977-S142','1977-04-06 01:55:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos/CR/Plasma/Fie',1010,'',174 from dual union all
select 424,'1944-S41','1944-08-15 14:41:00','military missile','success',8,'18314','Ma131','','','Training',661,'',90 from dual union all
select 4240,'1956-S39','1956-06-13 20:51:00','suborbital rocket','success',48,'USAF 67','E-layer','','','Ionos',640,'',138 from dual union all
select 42400,'1977-S143','1977-04-06 03:01:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',1010,'',171 from dual union all
select 42401,'1977-S144','1977-04-08 06:23:50','suborbital rocket','success',867,'NASA 18.1013GA','Auroral EUV','','','Aeronomy',428,'',157 from dual union all
select 42402,'1977-S145','1977-04-11 05:00:00','suborbital rocket','success',1070,'S-210JA-26','NO/O3/Ne','','','Aeron',1458,'',105 from dual union all
select 42403,'1977-S146','1977-04-14 00:00:00','military missile','success',744,'','15A15 LKI-41','','','Test',541,'',1000 from dual union all
select 42404,'1977-S147','1977-04-14 00:00:00','military missile','success',1106,'V2','','','','Test',301,'',1000 from dual union all
select 42405,'1977-S148','1977-04-16 05:00:00','suborbital rocket','success',228,'NASA 21.54UG','JHU FOT 1','','','UV Astron',1873,'',136 from dual union all
select 42406,'1977-S149','1977-04-21 17:00:00','suborbital rocket','unknown',37,'GL A04.410-02','','','','Solar UV/Solar EUV',1895,'',237 from dual union all
select 42407,'1977-S15','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42408,'1977-S150','1977-04-28 12:25:00','suborbital rocket','success',1213,'SL1115A','MSSL XR19 Tel','','','XR Astronomy',1839,'',244 from dual union all
select 42409,'1977-S151','1977-04-29 14:09:00','military missile','success',1442,'C4X-6','','','','Test',246,'',1000 from dual union all
select 4241,'1956-S41','1956-06-15 00:00:00','military missile','success',1013,'R-5R A1-4','R-5R  3','Radio guidance test','','Test',636,'',500 from dual union all
select 42410,'1977-S152','1977-05-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',129,'VISE',150 from dual union all
select 42411,'1977-S153','1977-05-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',129,'VISE',150 from dual union all
select 42412,'1977-S154','1977-05-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 42413,'1977-S155','1977-05-03 15:49:00','military missile','success',917,'AO-172','P-345  77 Spring 01','','','OT WS-244 - off by 1',433,'',180 from dual union all
select 42414,'1977-S156','1977-05-03 16:20:00','military missile','success',917,'AO-173','P-346  77 Spring 02','','','OT',433,'',180 from dual union all
select 42415,'1977-S157','1977-05-17 15:13:00','military missile','success',917,'AO-174','P-347  77 Spring 03','','','OT',433,'',180 from dual union all
select 42416,'1977-S158','1977-05-17 16:53:00','military missile','success',917,'AO-175','P-348  77 Spring 04','','','OT WS-247',433,'',180 from dual union all
select 42417,'1977-S159','1977-06-01 15:19:00','military missile','success',917,'AO-176','P-349  77 Spring 05','P-340?','','OT',433,'',180 from dual union all
select 42418,'1977-S16','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42419,'1977-S160','1977-06-01 15:45:00','military missile','success',917,'AO-177','P-350  77 Spring 06','P-341?','','OT',433,'',180 from dual union all
select 4242,'1956-S42','1956-06-18 20:42:00','suborbital rocket','success',48,'USAF 68','E-layer','','','Ionos',640,'',136 from dual union all
select 42420,'1977-S161','1977-06-14 14:07:00','military missile','success',917,'AO-178','P-351  77 Spring 07','','','OT WS-250',433,'',180 from dual union all
select 42421,'1977-S162','1977-06-14 14:42:00','military missile','failure',917,'AO-179','P-352  77 Spring 08','','','OT',433,'',0 from dual union all
select 42422,'1977-S165','1977-05-17 00:00:00','military missile','success',1004,'6T','15A14 UBP-11 SLI','GCh 15F143U2','','OT',496,'',1000 from dual union all
select 42423,'1977-S166','1977-05-18 00:00:00','military missile','success',795,'','TDV-1','ABRES TDV-1','TDV-1','Reentry Test',1701,'',1300 from dual union all
select 42424,'1977-S167','1977-05-18 13:43:00','suborbital rocket','success',846,'GL A11.712-1','ABRES Density/TDV-1','Sphere (10in)','','Aeron',871,'',188 from dual union all
select 42425,'1977-S168','1977-05-25 00:00:00','military missile','success',744,'','15A15 LKI-42','','','OT?',530,'',1000 from dual union all
select 42426,'1977-S169','1977-05-25 07:25:00','suborbital rocket','success',41,'NASA 26.061UH','Wisc. Aerobee IV','','','XR Astron',1895,'',192 from dual union all
select 42427,'1977-S17','1977-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 42428,'1977-S170','1977-05-25 22:57:00','military missile','success',939,'','OT-30','','','OT',421,'SSBN 643',500 from dual union all
select 42429,'1977-S171','1977-05-25 23:27:00','military missile','success',939,'','OT-30','','','OT',421,'SSBN 643',500 from dual union all
select 4243,'1956-S43','1956-06-21 18:48:00','suborbital rocket','success',48,'USAF 69','E-layer','','','Ionos',640,'',146 from dual union all
select 42430,'1977-S172','1977-05-25 23:27:00','military missile','success',939,'','OT-30','','','OT',421,'SSBN 643',500 from dual union all
select 42431,'1977-S173','1977-05-28 08:10:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT58GB','GLORY TRIP 58GB','OT',1712,'',1300 from dual union all
select 42432,'1977-S174','1977-05-28 00:00:00','military missile','success',1004,'27T','15A14 UBP-12 ZP','GCh 15F141U1','','OT',490,'',1000 from dual union all
select 42433,'1977-S175','1977-06-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',129,'VISE',150 from dual union all
select 42434,'1977-S176','1977-06-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',129,'VISE',150 from dual union all
select 42435,'1977-S177','1977-06-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',129,'VISE',150 from dual union all
select 42436,'1977-S178','1977-06-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',129,'VISE',150 from dual union all
select 42437,'1977-S179','1977-06-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',129,'VISE',150 from dual union all
select 42438,'1977-S18','1977-08-30 00:00:00','test rocket','unknown',1497,'9','K65UP No. 9','','','Ionos?',621,'',1500 from dual union all
select 42439,'1977-S180','1977-06-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',129,'VISE',150 from dual union all
select 4244,'1956-S45','1956-06-26 13:24:00','suborbital rocket','success',1521,'D-6A','','','','Test',253,'',140 from dual union all
select 42440,'1977-S181','1977-06-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',129,'VISE',150 from dual union all
select 42441,'1977-S182','1977-06-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',129,'VISE',150 from dual union all
select 42442,'1977-S19','1977-12-27 00:00:00','test rocket','unknown',1497,'11','K65UP No. 11','','','Ionos?',621,'',1500 from dual union all
select 42443,'1977-S191','1977-06-02 07:08:00','military missile','success',799,'','Mk 12 RV x 3','OT GT34GM-1','GLORY TRIP 34GM-1','OT',1705,'',1300 from dual union all
select 42444,'1977-S192','1977-06-01 21:46:00','military missile','success',939,'','OT-31','','','OT',421,'SSBN 616',500 from dual union all
select 42445,'1977-S193','1977-06-01 21:47:00','military missile','success',939,'','OT-31','','','OT',421,'SSBN 616',500 from dual union all
select 42446,'1977-S194','1977-06-06 22:34:00','suborbital rocket','success',544,'TE720.93-1','Ballistics','','','Test',1047,'',300 from dual union all
select 42447,'1977-S195','1977-06-08 00:00:00','suborbital rocket','success',1207,'SL1298','CHEVALINE S7','','','MIRV test',1839,'',210 from dual union all
select 42448,'1977-S196','1977-06-09 05:40:00','suborbital rocket','success',211,'NASA 27.08UH','SAO XRT 4','','','XR Astron',1895,'',205 from dual union all
select 42449,'1977-S197','1977-06-10 03:06:19','suborbital rocket','success',228,'DLR H-BBV-84','Heatpipe 2','INTA BB5-BAC-7701','','Tech',396,'',322 from dual union all
select 4245,'1956-S46','1956-06-26 18:26:00','suborbital rocket','success',48,'USAF 70','E-layer','','','Ionos',640,'',111 from dual union all
select 42450,'1977-S198','1977-06-16 00:00:00','military missile','success',799,'STM-14W','Mk 12A RV x 3','R&D','STM-14W','Test',1707,'',1300 from dual union all
select 42451,'1977-S199','1977-06-16 00:00:00','military missile','success',932,'A3T','FOT','','','OT',1897,'SSBN 610',1000 from dual union all
select 42452,'1977-S20','1977-01-01 00:00:00','military missile','success',1507,'VMF','','','','Test',1626,'PLBR',1000 from dual union all
select 42453,'1977-S200','1977-06-16 00:00:00','military missile','success',932,'A3T','FOT','','','OT',1897,'SSBN 610',1000 from dual union all
select 42454,'1977-S201','1977-06-16 00:00:00','military missile','success',932,'A3T','FOT','','','OT',1897,'SSBN 610',1000 from dual union all
select 42455,'1977-S202','1977-06-16 00:00:00','military missile','success',932,'A3T','FOT','','','OT',1897,'SSBN 610',1000 from dual union all
select 42456,'1977-S203','1977-06-16 00:00:00','military missile','success',932,'A3T','FOT','','','OT',1897,'SSBN 610',1000 from dual union all
select 42457,'1977-S204','1977-06-21 16:00:00','suborbital rocket','success',228,'NASA 21.45NP','SPAR 4','','','Micrograv',1873,'',179 from dual union all
select 42458,'1977-S205','1977-06-27 11:00:01','military missile','success',1442,'C4X-8','Evader 207','Mk 500 RB 207','','Test',246,'',1000 from dual union all
select 42459,'1977-S206','1977-06-27 11:48:00','suborbital rocket','success',920,'P196H','Atomic oxygen','','','Atmospheric',1447,'',129 from dual union all
select 4246,'1956-S47','1956-06-29 19:09:00','suborbital rocket','success',49,'NRL 50 (RV-N-13b)','D-region','','','Ionos',1869,'',264 from dual union all
select 42460,'1977-S207','1977-06-27 12:06:00','suborbital rocket','success',920,'P190H','O2/ne','','','Atmospheric',1447,'',124 from dual union all
select 42461,'1977-S208','1977-06-29 06:22:00','military missile','success',796,'','Mk 11 RV','FOT GT134M','GLORY TRIP 134M','OT',1699,'',1300 from dual union all
select 42462,'1977-S209','1977-07-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',129,'VISE',150 from dual union all
select 42463,'1977-S21','1977-01-01 00:00:00','military missile','success',997,'','','','','Test',1626,'K-140',1000 from dual union all
select 42464,'1977-S210','1977-07-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',129,'VISE',150 from dual union all
select 42465,'1977-S211','1977-06-30 00:00:00','military missile','success',744,'','15A15 K-7','','','OT',530,'',1000 from dual union all
select 42466,'1977-S212','1977-06-30 00:00:00','military missile','success',1106,'V3','','','','Test',301,'',1000 from dual union all
select 42467,'1977-S213','1977-07-12 05:45:00','suborbital rocket','success',860,'NASA 31.05UA','','','','Aeronomy',1895,'',113 from dual union all
select 42468,'1977-S214','1977-07-12 16:15:00','suborbital rocket','success',1070,'S-210JA-29','E/B/ne/ISIS-2','','','Auroral',1458,'',118 from dual union all
select 42469,'1977-S215','1977-07-14 16:00:00','military missile','success',931,'A3T','DASO','','','OT',421,'UK S22',1000 from dual union all
select 4247,'1956-S48','1956-08-07 00:00:00','military missile','success',1013,'M-5RD A2-8','M5-RD 7','R-7 component test','','Test',636,'',500 from dual union all
select 42470,'1977-S216','1977-07-19 19:00:00','suborbital rocket','success',1319,'NASA 12.1001WT','','','','Test',1821,'',240 from dual union all
select 42471,'1977-S217','1977-07-21 09:00:00','suborbital rocket','success',133,'NASA 25.16GG','','','','UV Astron?',1895,'',206 from dual union all
select 42472,'1977-S218','1977-07-26 15:35:29','suborbital rocket','success',1073,'S-310JA-3','VLF/plasma/B','','','Ionos',1458,'',221 from dual union all
select 42473,'1977-S219','1977-07-27 04:40:00','suborbital rocket','success',211,'NASA 27.31UH','SAO XRT 5','','','XR Astron',1895,'',203 from dual union all
select 42474,'1977-S22','1977-01-01 00:00:00','suborbital rocket','success',1028,'','TVC test','','','Test',1361,'',100 from dual union all
select 42475,'1977-S220','1977-07-27 05:35:00','suborbital rocket','success',133,'NASA 25.21UH','MIT WIXT 1','','','XR Astron',1895,'',190 from dual union all
select 42476,'1977-S221','1977-07-28 00:00:00','military missile','success',1472,'','Intercept','','','ABM Test',183,'',180 from dual union all
select 42477,'1977-S222','1977-07-29 00:00:00','suborbital rocket','success',547,'SRK-11','HAVE JEEP IV','','','Reentry Test?',871,'',500 from dual union all
select 42478,'1977-S223','1977-07-30 00:00:00','military missile','success',744,'','15A15 LKI-43','','','Test',542,'',1000 from dual union all
select 42479,'1977-S224','1977-08-03 08:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT60GB','GLORY TRIP 60GB','OT',1712,'',1300 from dual union all
select 4248,'1956-S49','1956-06-30 00:00:00','military missile','success',970,'No 5112','','','','OT',636,'',100 from dual union all
select 42480,'1977-S225','1977-08-03 16:00:02','military missile','success',939,'','DASO','','','OT',421,'SSBN 622',500 from dual union all
select 42481,'1977-S226','1977-08-09 16:10:00','suborbital rocket','success',30,'GL A03.509-1','SOLAR UV','','','Solar UV',1869,'',168 from dual union all
select 42482,'1977-S227','1977-08-10 07:10:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT59GM-1','GLORY TRIP 59GM-1','OT',1703,'',1300 from dual union all
select 42483,'1977-S228','1977-08-10 05:01:00','suborbital rocket','success',837,'NASA 14.534UE','','','','Plasma',1821,'',200 from dual union all
select 42484,'1977-S229','1977-08-10 12:47:00','suborbital rocket','success',1070,'S-210JA-27','NO airglow','','','Auroral?',1458,'',120 from dual union all
select 42485,'1977-S23','1977-02-10 00:00:00','military missile','success',1450,'47622-127(17L)','15A20-T-VCh LKI-1','GCh 15F842 (15F843x1)','','Test',504,'',1000 from dual union all
select 42486,'1977-S230','1977-08-16 11:15:00','suborbital rocket','success',698,'L-3H-9','','','','Aeron/Ionos/Plasma',834,'',1294 from dual union all
select 42487,'1977-S231','1977-08-17 17:10:00','suborbital rocket','failure',867,'NASA 18.182GA','','','','Aeronomy',1895,'',198 from dual union all
select 42488,'1977-S232','1977-08-18 19:18:01','military missile','success',1442,'C4X-10','','','','Test',246,'',1000 from dual;
insert into LAUNCH
select 42489,'1977-S233','1977-08-18 23:35:00','suborbital rocket','success',867,'NASA 18.1009UE','','','','Plasma',1895,'',267 from dual union all
select 4249,'1956-S50','1956-07-05 07:52:00','suborbital rocket','success',57,'NRL 33','Night airglow/B','','','Magnetospheric/Aeron',1869,'',162 from dual union all
select 42490,'1977-S234','1977-08-19 00:00:00','military missile','success',799,'PVM-14','Mk 12 RV x 3','R&D','PVM-14','Test',1707,'',1300 from dual union all
select 42491,'1977-S235','1977-08-25 01:30:00','suborbital rocket','success',1299,'TT-500-2F','','','','Test',1476,'',230 from dual union all
select 42492,'1977-S236','1977-08-30 00:00:00','suborbital rocket','success',1014,'','Vertikal''-5','VZAF','','Solar UV/Solar X',636,'',500 from dual union all
select 42493,'1977-S237','1977-09-01 00:00:00','suborbital rocket','success',846,'HARSES','','','','Aeron?',1895,'',150 from dual union all
select 42494,'1977-S238','1977-09-02 05:50:54','suborbital rocket','success',1347,'Lagopedo Uno','','','','Active Ionos',855,'',261 from dual union all
select 42495,'1977-S239','1977-09-03 00:00:00','military missile','success',795,'','TDV-2','ABRES TDV-2','TDV-2','Reentry Test',1701,'',1300 from dual union all
select 42496,'1977-S24','1977-02-10 02:30:00','military missile','success',1450,'64122-126(16L)','15A20-T-VCh LKI-2','GCh 15F842 (15F843x1)','','Test',536,'',1000 from dual union all
select 42497,'1977-S240','1977-09-03 10:00:00','suborbital rocket','success',668,'K-9M-59','','','','Aeron/Ionos/Solar EUV',839,'',376 from dual union all
select 42498,'1977-S241','1977-09-03 14:16:00','military missile','success',1442,'C4X-9','','','','Test',246,'',1000 from dual union all
select 42499,'1977-S242','1977-09-06 21:00:01','military missile','success',939,'','DASO','','','OT',421,'SSBN 625',500 from dual union all
select 425,'1944-S42','1944-08-15 17:04:00','military missile','success',8,'18321','Ma143','','','Training',661,'',90 from dual union all
select 4250,'1956-S51','1956-07-06 18:00:00','suborbital rocket','success',839,'N-C 1','UM AM6.01 Sphere','IGY AM6.01','','Aeron',1821,'',129 from dual union all
select 42500,'1977-S243','1977-09-10 00:00:00','suborbital rocket','success',1337,'Sandia 497-008','','','','Test',1821,'',500 from dual union all
select 42501,'1977-S244','1977-09-12 06:00:09','suborbital rocket','success',1347,'Lagopedo Dos','LASL Lagopedo II','','','Active Ionos',855,'',286 from dual union all
select 42502,'1977-S245','1977-09-12 23:30:00','military missile','success',932,'A3TK-PA','CHEVALINE P-A','','','Test',250,'',1000 from dual union all
select 42503,'1977-S246','1977-09-14 00:15:00','test rocket','success',492,'701(II)-01','DDDS','','','Test',810,'',200 from dual union all
select 42504,'1977-S247','1977-09-14 00:00:00','suborbital rocket','success',740,'','IKO','','','Test',636,'',93 from dual union all
select 42505,'1977-S248','1977-09-14 07:10:00','military missile','success',799,'','Mk 12 RV x 3','OT GT35GM','GLORY TRIP 35GM','OT',1705,'',1300 from dual union all
select 42506,'1977-S249','1977-09-14 12:22:00','suborbital rocket','success',655,'K-10-13','','','','UV Astron/EUV Astron',839,'',204 from dual union all
select 42507,'1977-S25','1977-03-18 13:40:00','military missile','success',1450,'52422-128(18L)','15A20-T-VCh LKI-3','GCh 15F843?(15F843x1)','','Test',536,'',1000 from dual union all
select 42508,'1977-S250','1977-09-15 09:30:00','suborbital rocket','success',1070,'S-210-12','','','','Aeron/Ionos',839,'',114 from dual union all
select 42509,'1977-S251','1977-09-21 08:35:00','suborbital rocket','success',1073,'S-310-4','','','','Ionos',839,'',188 from dual union all
select 4251,'1956-S52','1956-07-17 07:28:00','suborbital rocket','success',1521,'R-2','4203-3','','','Reentry Test',253,'',142 from dual union all
select 42510,'1977-S252','1977-09-24 11:55:00','suborbital rocket','success',848,'GL A07.707-1','DMSP CHEM','','','Aeronomy',1895,'',189 from dual union all
select 42511,'1977-S253','1977-09-24 12:15:00','suborbital rocket','success',906,'GL A10.705-1','DMSP MS SP','','','Aeron',1895,'',195 from dual union all
select 42512,'1977-S254','1977-09-28 15:33:16','suborbital rocket','success',1303,'DNA IC733.03-1','SPIRE I','','','Aurora/IR Background',1047,'',275 from dual union all
select 42513,'1977-S255','1977-09-29 17:45:00','suborbital rocket','success',133,'NASA 25.22UL','','','','Aeron',1895,'',257 from dual union all
select 42514,'1977-S256','1977-10-02 03:00:00','suborbital rocket','success',41,'NASA 26.65IH-UH','SAO XR/Italy XR','','','XR Astron',1895,'',198 from dual union all
select 42515,'1977-S257','1977-10-04 02:20:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',170 from dual union all
select 42516,'1977-S258','1977-10-13 21:26:00','suborbital rocket','success',1202,'DLR A-GRC-85','T/NL 3A Dagmar','','','Auroral',29,'',536 from dual union all
select 42517,'1977-S259','1977-10-13 21:59:00','suborbital rocket','success',1202,'SL1423','UK High Lat 8','','','Auroral',29,'',788 from dual union all
select 42518,'1977-S26','1977-04-15 13:20:00','military missile','success',1450,'64122-129(19L)','15A20-T-VCh LKI-4','GCh 15F842 (15F843x1)','','Test',536,'',1000 from dual union all
select 42519,'1977-S260','1977-10-16 22:23:26','suborbital rocket','success',495,'F3','Wind/T','','','Atmospheric',29,'',247 from dual union all
select 4252,'1956-S53','1956-07-17 19:40:00','suborbital rocket','success',370,'','NRL NN5.27','IGY NN5.27','','Solar UV/Solar X',1285,'LSD-18',120 from dual union all
select 42520,'1977-S261','1977-10-16 22:33:36','suborbital rocket','success',921,'P207A','Na','','','Atmospheric',29,'',207 from dual union all
select 42521,'1977-S262','1977-10-19 00:00:00','suborbital rocket','success',1219,'XV-04','','','','Ionos?',330,'',590 from dual union all
select 42522,'1977-S263','1977-10-19 18:08:00','military missile','success',1442,'C4X-11','','','','Test',246,'',1000 from dual union all
select 42523,'1977-S264','1977-10-21 08:49:00','suborbital rocket','success',211,'SSC S19B','S19 Guidance Test','','','Test',412,'',374 from dual union all
select 42524,'1977-S265','1977-10-25 00:30:00','military missile','success',745,'45028-201(1L)','15A16 LKI-1','GCh 15F161 (15F162x4)','','OT',542,'',1000 from dual union all
select 42525,'1977-S266','1977-10-25 12:25:00','ballistic missile test','success',1497,'10','Vertikal''-6','VZA','','Aeron/Ionos',621,'',1500 from dual union all
select 42526,'1977-S267','1977-10-26 01:55:00','military missile','success',1453,'49278-82-001','15A35 LKI-1','GCh 15F355 (15F356 x6)','','OT',508,'',1000 from dual union all
select 42527,'1977-S268','1977-10-29 00:00:00','suborbital rocket','success',740,'','','','','Aeron?',636,'',150 from dual union all
select 42528,'1977-S269','1977-10-30 00:00:00','suborbital rocket','success',740,'','','','','Aeron?',636,'',150 from dual union all
select 42529,'1977-S27','1977-10-06 00:00:00','military missile','success',1450,'','15A20S LKI-11','GCh 15F202P2 (15F204x3)','','OT',536,'',1000 from dual union all
select 4253,'1956-S54','1956-07-18 20:00:00','suborbital rocket','success',370,'','NRL NN5.28','IGY NN5.28','','Solar UV/Solar X',1287,'LSD-18',120 from dual union all
select 42530,'1977-S270','1977-12-08 15:05:00','military missile','unknown',917,'AO-181','P-354  77 Winter 02','P-II RV test 2','P2AD','OT WS-253 (OR)',433,'',180 from dual union all
select 42531,'1977-S271','1978-02-21 00:00:00','military missile','failure',917,'AO-182','P-355  77 Winter 03','P-II RV test 3','P2AD','OT',433,'',1 from dual union all
select 42532,'1977-S272','1977-11-01 00:00:00','military missile','success',1507,'VMF','','','','Test',1626,'PLBR',1000 from dual union all
select 42533,'1977-S273','1977-11-01 00:00:00','military missile','success',1507,'VMF','','','','Test',1626,'PLBR',1000 from dual union all
select 42534,'1977-S274','1977-11-01 00:00:00','military missile','success',1507,'VMF','','','','Test',1626,'PLBR',1000 from dual union all
select 42535,'1977-S275','1977-10-31 03:20:00','military missile','success',1006,'45028-201(1L)','15A18-01 LKI-1','GCh 15F183-1','','OT',489,'',1000 from dual union all
select 42536,'1977-S276','1977-11-02 20:15:00','suborbital rocket','success',1202,'DLR A-GRC-87','T/NL 1B','','','Auroral',29,'',541 from dual union all
select 42537,'1977-S277','1977-11-03 00:00:00','military missile','success',799,'PVM-15','Mk 12 RV x 3','R&D','PVM-15','Test',1707,'',1300 from dual union all
select 42538,'1977-S278','1977-11-08 22:30:00','military missile','success',932,'A3TK-PB','CHEVALINE P-B','','','Test',252,'',1000 from dual union all
select 42539,'1977-S279','1977-11-11 04:00:00','suborbital rocket','success',117,'GL A24.609-01','TEM-1','','','ABM Tech IR',1873,'',228 from dual union all
select 4254,'1956-S55','1956-07-19 08:45:00','military missile','success',635,'CC-13    HN','Jupiter A','','','Test',279,'',90 from dual union all
select 42540,'1977-S28','1977-10-21 08:25:00','military missile','success',1450,'52432-122(12L)','15A20-T-OCh LKI-2','GCh 15F202P2 (15F204x3)','','OT',504,'',1000 from dual union all
select 42541,'1977-S280','1977-11-13 00:00:00','military missile','success',795,'','TDV-3','ABRES TDV-3','TDV-3','Reentry Test',1701,'',1300 from dual union all
select 42542,'1977-S281','1977-11-13 08:55:00','suborbital rocket','success',1162,'DNA IC730.09-1','FWIF I','','','Aeron/Auroral/Solar',1047,'',141 from dual union all
select 42543,'1977-S282','1977-11-15 10:33:40','suborbital rocket','success',543,'ARPA WS710.51-1','WIDEBAND','','','Ionos',1047,'',170 from dual union all
select 42544,'1977-S283','1977-11-17 00:00:00','military missile','success',1106,'V4','','','','Test',301,'',1000 from dual union all
select 42545,'1977-S284','1977-11-17 21:32:36','suborbital rocket','success',1202,'SL1421','UK High Lat 9','','','Auroral',29,'',718 from dual union all
select 42546,'1977-S285','1977-11-17 21:34:06','suborbital rocket','success',495,'F1','Electrons/Ions','','','Ionospheric',29,'',261 from dual union all
select 42547,'1977-S286','1977-11-18 15:04:00','military missile','unknown',917,'AO-180','P-353  77 Winter 01','P-II RV test 1','P2AD','OT',433,'',180 from dual union all
select 42548,'1977-S287','1977-11-29 00:00:00','military missile','success',1004,'217T','15A14 UBP-13 ZP','GCh 15F141U1','','OT',496,'',1000 from dual union all
select 42549,'1977-S288','1977-12-01 02:50:00','military missile','success',796,'','Mk 11 RV','FOT GT135M','GLORY TRIP 135M','OT',1702,'',1300 from dual union all
select 4255,'1956-S56','1956-07-19 19:32:00','suborbital rocket','success',370,'','NRL NN5.29','IGY NN5.29','','Solar UV/Solar X',1288,'LSD-18',120 from dual union all
select 42550,'1977-S29','1977-10-28 00:00:00','military missile','success',1450,'','15A20S LKI-12','GCh 15F202P2?','','OT',536,'',1000 from dual union all
select 42551,'1977-S291','1977-12-01 00:00:00','military missile','success',1507,'VMF','','','','Test',1626,'PLBR',1000 from dual union all
select 42552,'1977-S292','1977-12-20 01:00:00','military missile','success',1453,'49278-82-002','15A35 LKI-2','GCh 15F355','','OT',508,'',1000 from dual union all
select 42553,'1977-S293','1977-12-02 00:00:00','suborbital rocket','success',1319,'NASA 12.1002WT','','','','Test',1821,'',247 from dual union all
select 42554,'1977-S294','1977-12-05 02:08:10','suborbital rocket','success',495,'F4','HLC 2B','','','Auroral',29,'',255 from dual union all
select 42555,'1977-S295','1977-12-05 14:08:00','military missile','success',1442,'C4X-12','','','','Test',246,'',1000 from dual union all
select 42556,'1977-S296','1977-12-07 06:50:00','military missile','success',799,'','Mk 12 RV x 3','OT GT36GM','GLORY TRIP 36GM','OT',1705,'',1300 from dual union all
select 42557,'1977-S297','1977-12-06 00:05:00','suborbital rocket','success',210,'AAF-NVB-03','Dayside Cusp','','','Ionospheric',334,'',300 from dual union all
select 42558,'1977-S298','1977-12-11 06:00:00','suborbital rocket','success',41,'NASA 26.60GG','','','','UV Astron?',1895,'',177 from dual union all
select 42559,'1977-S299','1977-12-11 07:45:00','suborbital rocket','success',133,'NASA 25.17GG','UIT-Proto F1','','','UV Astron',1895,'',226 from dual union all
select 4256,'1956-S57','1956-07-20 00:00:00','military missile','success',1013,'M-5RD A1-7','M5-RD 6','R-7 component test','','Test',636,'',500 from dual union all
select 42560,'1977-S30','1977-02-12 02:00:00','military missile','success',1452,'49238-79-012T','15A30 LKI-B8','GCh 15B126','','OT',535,'',1000 from dual union all
select 42561,'1977-S300','1977-12-11 16:30:00','suborbital rocket','success',867,'NASA 18.183GA','','','','Aeronomy',1895,'',196 from dual union all
select 42562,'1977-S301','1977-12-12 10:49:00','suborbital rocket','success',288,'INTA TCIS-7701','C-3-2','','','Ionos',396,'',150 from dual union all
select 42563,'1977-S302','1977-12-13 05:50:00','suborbital rocket','success',37,'GL A04.602','PRECEDE','','','Active Plasma',1895,'',102 from dual union all
select 42564,'1977-S303','1977-12-13 08:46:39','suborbital rocket','success',1212,'DLR K-GR-90','DLR TEXUS 1','','','Micrograv',416,'',265 from dual union all
select 42565,'1977-S304','1977-12-14 05:00:00','suborbital rocket','success',860,'NASA 31.03UA','','','','Aeronomy',1895,'',140 from dual union all
select 42566,'1977-S305','1977-12-16 10:35:00','suborbital rocket','success',288,'INTA TCIS-7702','C-3-2','','','Ionos',396,'',150 from dual union all
select 42567,'1977-S306','1977-12-27 00:00:00','military missile','success',1004,'107T','15A14 UBP-14 SLI','GCh 15F141U1','','OT',510,'',1000 from dual union all
select 42568,'1977-S307','1977-12-29 01:00:00','military missile','success',745,'51048-202(2L)','15A16 LKI-2','GCh 15F161','','OT',542,'',1000 from dual union all
select 42569,'1977-S308','1977-12-30 00:00:00','military missile','success',1004,'117T','15A14 UBP-15 ZP','GCh 15F143U2','','OT',496,'',1000 from dual union all
select 4257,'1956-S59','1956-07-20 19:15:00','suborbital rocket','success',370,'8','NRL NN5.30','IGY NN5.30','','Solar UV/Solar X',1289,'LSD-18',120 from dual union all
select 42570,'1977-S309','1977-06-04 05:58:00','suborbital rocket','success',290,'ISRO','ISRO 5.40 (CO.40)','','','Ionos',1467,'',150 from dual union all
select 42571,'1977-S31','1977-02-17 02:30:00','military missile','success',1452,'49238-21-133T','15A30 UBP-14 ZP','GCh 15B103','','OT',508,'',1000 from dual union all
select 42572,'1977-S310','1977-05-18 07:26:00','suborbital rocket','success',1284,'Flight 2025','Sphere','','','Aeron',873,'',90 from dual union all
select 42573,'1977-S311','1977-05-18 09:30:00','suborbital rocket','success',1284,'Flight 2026','Sphere','','','Aeron',873,'',90 from dual union all
select 42574,'1977-S312','1977-05-18 12:37:00','suborbital rocket','success',1284,'Flight 2028','Sphere','','','Aeron',873,'',90 from dual union all
select 42575,'1977-S313','1977-05-18 13:20:00','suborbital rocket','success',1284,'Flight 2029','Sphere','','','Aeron',873,'',90 from dual union all
select 42576,'1977-S314','1977-05-18 13:43:00','suborbital rocket','success',1284,'Flight AC6','Accelerometer Sphere','','','Aeron',873,'',90 from dual union all
select 42577,'1977-S315','1977-08-23 00:00:00','suborbital rocket','failure',220,'Equatorial Wideband','','','','Plasma',871,'',160 from dual union all
select 42578,'1977-S316','1977-09-30 00:00:00','military missile','success',359,'','Full range test','','','Test',809,'',1000 from dual;
insert into LAUNCH
select 42579,'1977-S317','1977-11-22 00:00:00','military missile','success',359,'','Full range test','','','Test',809,'',1000 from dual union all
select 4258,'1956-S60','1956-07-22 19:10:00','suborbital rocket','success',370,'','NRL NN5.32','IGY NN5.32','','Solar UV/Solar X',1291,'LSD-18',120 from dual union all
select 42580,'1977-S318','1977-10-01 00:00:00','military missile','success',983,'Scud B','','','','Training',761,'',100 from dual union all
select 42581,'1977-S319','1977-10-01 00:00:00','military missile','success',983,'Scud B','','','','Training',761,'',100 from dual union all
select 42582,'1977-S32','1977-04-18 14:30:00','military missile','success',1452,'49238-21-134T','15A30 UBP-15 K','GCh 15B103','','OT',508,'',1000 from dual union all
select 42583,'1977-S320','1977-10-01 00:00:00','military missile','success',983,'Scud B','','','','Training',761,'',100 from dual union all
select 42584,'1977-S321','1977-10-01 00:00:00','military missile','success',983,'Scud B','','','','Training',761,'',100 from dual union all
select 42585,'1977-S322','1977-10-01 00:00:00','military missile','success',983,'Scud B','','','','Training',761,'',100 from dual union all
select 42586,'1977-S323','1977-10-01 00:00:00','military missile','success',983,'Scud B','','','','Training',761,'',100 from dual union all
select 42587,'1977-S324','1977-03-08 00:14:05','suborbital rocket','success',846,'DNA ST707.51-1','STRESS DIANNE Probe','','','Ionos',388,'',213 from dual union all
select 42588,'1977-S325','1977-03-08 00:32:04','suborbital rocket','success',846,'DNA ST707.51-2','STRESS DIANNE Probe','','','Ionos',388,'',203 from dual union all
select 42589,'1977-S326','1977-03-13 23:27:00','suborbital rocket','success',846,'DNA ST707.51-3','STRESS ESTHER Probe','','','Ionos',388,'',220 from dual union all
select 4259,'1956-S61','1956-07-24 00:00:00','suborbital rocket','success',839,'N-C 4 (HUGO-1)','Hurricane photo','','','Aeron',1821,'',112 from dual union all
select 42590,'1977-S327','1977-03-13 23:44:50','suborbital rocket','success',846,'DNA ST707.51-4','STRESS ESTHER Probe','','','Ionos',389,'',199 from dual union all
select 42591,'1977-S328','1977-03-14 23:25:20','suborbital rocket','success',846,'DNA ST707.51-5','STRESS FERN Probe','','','Ionos',388,'',249 from dual union all
select 42592,'1977-S329','1977-03-15 00:03:10','suborbital rocket','success',846,'DNA ST707.51-6','STRESS FERN Probe','','','Ionos',388,'',222 from dual union all
select 42593,'1977-S33','1977-05-25 14:00:00','military missile','success',1452,'51138-22-134T','15A30 UBP-16 ZP','GCh 15B103','','OT',532,'',1000 from dual union all
select 42594,'1977-S330','1977-01-01 00:00:00','military missile','success',644,'','MATSESTA 4','','','Reentry Test',636,'',null from dual union all
select 42595,'1977-S331','1977-11-01 00:00:00','military missile','success',984,'Aerofon','','','','Test',636,'',86 from dual union all
select 42596,'1977-S332','1977-09-01 00:00:00','military missile','success',886,'','First launch','','','Test',636,'',120 from dual union all
select 42597,'1977-S333','1977-08-12 06:06:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 3','','','Meteo',1822,'KORO',82 from dual union all
select 42598,'1977-S334','1977-08-12 11:05:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 4','','','Meteo',1822,'KORO',81 from dual union all
select 42599,'1977-S335','1977-08-15 17:35:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 5','','','Meteo',1822,'KORO',84 from dual union all
select 426,'1944-S43','1944-08-15 17:24:00','military missile','success',8,'18315','Ma132','','','Training',661,'',90 from dual union all
select 4260,'1956-S62','1956-07-24 00:00:00','military missile','success',970,'No 0409','','','','OT',636,'',100 from dual union all
select 42600,'1977-S336','1977-08-17 20:15:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 10','','','Meteo',1822,'KORO',86 from dual union all
select 42601,'1977-S337','1977-08-16 20:29:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 9','','','Meteo',1822,'KORO',84 from dual union all
select 42602,'1977-S339','1977-04-07 00:00:00','military missile','success',1044,'29532-713-10','8K98P UBP-9','','','Test',477,'',1000 from dual union all
select 42603,'1977-S34','1977-08-23 03:30:00','military missile','success',1452,'63078-23-124T','15A30 UBP-17 ZP','GCh 15B103','','OT',532,'',1000 from dual union all
select 42604,'1977-S340','1977-04-22 00:00:00','military missile','success',1044,'45738-114-04','8K98P UBP-10','','','Test',477,'',1000 from dual union all
select 42605,'1977-S341','1977-05-26 00:00:00','military missile','success',1037,'V 29502-06','8K98 UBP-35','','','Test',477,'',1000 from dual union all
select 42606,'1977-S342','1977-06-30 00:00:00','military missile','success',1037,'V 29503-03','8K98 UBP-36','','','Test',477,'',1000 from dual union all
select 42607,'1977-S343','1977-08-25 00:00:00','military missile','success',1044,'45738-114-06','8K98P UBP-11','','','Test',477,'',1000 from dual union all
select 42608,'1977-S344','1977-09-14 00:00:00','military missile','success',1044,'V 29503-05','8K98P UBP-12','','','Test',477,'',1000 from dual union all
select 42609,'1977-S345','1977-09-29 00:00:00','military missile','success',1037,'V 29503-01','8K98 UBP-37','','','Test',477,'',1000 from dual union all
select 4261,'1956-S63','1956-07-25 21:15:00','suborbital rocket','success',370,'','NRL NN5.34','IGY NN5.34','','Solar UV/Solar X',1293,'LSD-18',120 from dual union all
select 42610,'1977-S346','1977-10-14 00:00:00','military missile','success',1037,'Yu29501-07','8K98 UBP-38','','','Test',477,'',1000 from dual union all
select 42611,'1977-S347','1977-12-27 00:00:00','military missile','success',1037,'Yu29501-11','8K98 UBP-39','','','Test',477,'',1000 from dual union all
select 42612,'1977-S348','1977-01-01 00:00:00','military missile','success',1450,'','15A20 54 RD UBP-2','','','OT',1468,'',1000 from dual union all
select 42613,'1977-S349','1977-09-01 00:00:00','military missile','success',1450,'','15A20 54 RD UBP-3','','','OT',1468,'',1000 from dual union all
select 42614,'1977-S35','1977-01-07 07:45:00','suborbital rocket','success',227,'AED-VB-46','','','','Auroral/Aeron',428,'',261 from dual union all
select 42615,'1977-S350','1977-01-01 00:00:00','military missile','success',1450,'K01','15A20 47 RD UBP-3','Duga-1','','OT',1911,'',1000 from dual union all
select 42616,'1977-S351','1977-01-01 00:00:00','military missile','success',1450,'K02','15A20 47 RD UBP-4','Duga-1','','OT',1911,'',1000 from dual union all
select 42617,'1977-S352','1977-01-01 00:00:00','military missile','success',1450,'K03','15A20 47 RD UBP-5','Duga-1','','OT',1911,'',1000 from dual union all
select 42618,'1977-S353','1977-01-01 00:00:00','military missile','success',1450,'K04','15A20 47 RD UBP-6','Duga-1','','OT',1911,'',1000 from dual union all
select 42619,'1977-S354','1977-01-01 00:00:00','military missile','success',1450,'K05','15A20 47 RD UBP-7','Duga-1','','OT',1911,'',1000 from dual union all
select 4262,'1956-S64','1956-07-26 00:00:00','military missile','success',970,'No 0311','','','','OT',636,'',100 from dual union all
select 42620,'1977-S355','1977-01-01 00:00:00','military missile','success',1450,'K06','15A20 47 RD UBP-8','Duga-1','','OT',1911,'',1000 from dual union all
select 42621,'1977-S356','1977-01-01 00:00:00','military missile','success',1450,'K07','15A20 47 RD UBP-9','Duga-1','','OT',1911,'',1000 from dual union all
select 42622,'1977-S357','1977-01-01 00:00:00','military missile','success',1450,'K08','15A20 47 RD UBP-10','Duga-1','','OT',1911,'',1000 from dual union all
select 42623,'1977-S358','1977-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-10','Duga-1','','OT',368,'',1000 from dual union all
select 42624,'1977-S359','1977-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-17','Duga-1','','OT',368,'',1000 from dual union all
select 42625,'1977-S36','1977-01-08 19:03:00','suborbital rocket','success',867,'NASA 18.1008UE','','','','Plasma',1821,'',138 from dual union all
select 42626,'1977-S360','1977-02-28 00:00:00','military missile','success',1037,'V 29501-12','8K98 UBP-34','','','Test',477,'',1000 from dual union all
select 42627,'1977-S361','1977-01-17 11:36:00','suborbital rocket','success',1284,'INTA SLW-7701','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42628,'1977-S362','1977-01-19 11:33:00','suborbital rocket','success',1284,'INTA SLW-7702','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42629,'1977-S363','1977-01-19 12:08:00','suborbital rocket','success',1284,'INTA SLW-7703','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 4263,'1956-S65','1956-07-26 21:00:00','suborbital rocket','success',370,'NN5.35','NRL NN5.35','IGY NN5.35','','Solar UV/Solar X',1294,'LSD-18',120 from dual union all
select 42630,'1977-S364','1977-01-21 11:34:00','suborbital rocket','success',1284,'INTA SLW-7704','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42631,'1977-S365','1977-01-24 11:31:00','suborbital rocket','success',1284,'INTA SLW-7705','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42632,'1977-S366','1977-01-24 12:00:00','suborbital rocket','success',1284,'INTA SLW-7706','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42633,'1977-S367','1977-01-26 11:34:00','suborbital rocket','success',1284,'INTA SLW-7707','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42634,'1977-S368','1977-01-26 12:21:00','suborbital rocket','success',1284,'INTA SLW-7708','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42635,'1977-S369','1977-01-28 11:33:00','suborbital rocket','success',1284,'INTA SLW-7709','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42636,'1977-S37','1977-01-08 19:38:00','suborbital rocket','success',132,'NASA 23.08UE','','','','Plasma',1821,'',83 from dual union all
select 42637,'1977-S370','1977-01-31 11:55:00','suborbital rocket','success',1284,'INTA SLW-7710','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42638,'1977-S371','1977-02-04 11:32:00','suborbital rocket','success',1284,'INTA SLW-7712','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42639,'1977-S372','1977-02-05 11:09:00','suborbital rocket','success',1284,'INTA SLW-7713','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 4264,'1956-S66','1956-07-27 17:05:00','suborbital rocket','failure',370,'NN5.36','NRL NN5.36','IGY NN5.36','','Solar UV/Solar X',1286,'LSD-18',5 from dual union all
select 42640,'1977-S373','1977-02-05 11:50:00','suborbital rocket','success',1284,'INTA SLW-7714','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42641,'1977-S374','1977-02-07 11:34:00','suborbital rocket','success',1284,'INTA SLW-7715','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42642,'1977-S375','1977-02-09 11:32:00','suborbital rocket','success',1284,'INTA SLW-7716','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42643,'1977-S376','1977-02-11 11:37:00','suborbital rocket','success',1284,'INTA SLW-7717','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42644,'1977-S377','1977-02-16 11:33:00','suborbital rocket','success',1284,'INTA SLW-7718','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42645,'1977-S378','1977-02-23 11:53:00','suborbital rocket','success',1284,'INTA SLW-7719','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42646,'1977-S379','1977-03-02 11:54:00','suborbital rocket','success',1284,'INTA SLW-7720','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42647,'1977-S38','1977-01-13 04:22:00','suborbital rocket','success',226,'AAD-VA-45','Auroral UV/IR','','','Auroral/Ionos',428,'',152 from dual union all
select 42648,'1977-S380','1977-03-09 11:31:00','suborbital rocket','success',1284,'INTA SLW-7721','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42649,'1977-S381','1977-03-16 11:30:00','suborbital rocket','success',1284,'INTA SLW-7722','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 4265,'1956-S67','1956-07-28 00:00:00','military missile','success',970,'No 0307','','','','OT',636,'',100 from dual union all
select 42650,'1977-S382','1977-03-23 11:35:00','suborbital rocket','success',1284,'INTA SLW-7723','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42651,'1977-S383','1977-03-30 11:41:00','suborbital rocket','success',1284,'INTA SLW-7724','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42652,'1977-S384','1977-04-01 11:31:00','suborbital rocket','success',1284,'INTA SLW-7725','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42653,'1977-S385','1977-04-04 11:31:00','suborbital rocket','success',1284,'INTA SLW-7726','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42654,'1977-S386','1977-04-11 11:51:00','suborbital rocket','success',1284,'INTA SLW-7727','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42655,'1977-S387','1977-04-13 11:34:00','suborbital rocket','success',1284,'INTA SLW-7728','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42656,'1977-S388','1977-04-15 11:31:00','suborbital rocket','success',1284,'INTA SLW-7729','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42657,'1977-S389','1977-04-18 11:31:00','suborbital rocket','success',1284,'INTA SLW-7730','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42658,'1977-S39','1977-01-14 10:18:10','suborbital rocket','success',1337,'NASA 29.03AE','','','','Auroral/Aeron/Ionos',1047,'',500 from dual union all
select 42659,'1977-S390','1977-04-20 11:31:00','suborbital rocket','success',1284,'INTA SLW-7731','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 4266,'1956-S68','1956-08-10 00:00:00','military missile','success',1013,'M-5RD A2-6','M5-RD 8','R-7 component test','','Test',636,'',500 from dual union all
select 42660,'1977-S391','1977-04-22 11:31:00','suborbital rocket','success',1284,'INTA SLW-7732','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42661,'1977-S392','1977-04-25 11:32:00','suborbital rocket','success',1284,'INTA SLW-7733','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42662,'1977-S393','1977-04-27 11:39:00','suborbital rocket','success',1284,'INTA SLW-7734','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42663,'1977-S394','1977-04-29 11:31:00','suborbital rocket','success',1284,'INTA SLW-7735','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42664,'1977-S395','1977-05-04 11:32:00','suborbital rocket','success',1284,'INTA SLW-7736','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42665,'1977-S396','1977-05-11 11:54:00','suborbital rocket','success',1284,'INTA SLW-7737','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42666,'1977-S397','1977-06-08 11:32:00','suborbital rocket','success',1284,'INTA SLW-7738','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42667,'1977-S398','1977-06-22 11:31:00','suborbital rocket','success',1284,'INTA SLW-7739','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42668,'1977-S399','1977-07-29 00:00:00','suborbital rocket','success',1284,'INTA SLW-7740','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual;
insert into LAUNCH
select 42669,'1977-S40','1977-01-15 00:50:02','suborbital rocket','success',837,'SSC S18-1','SSC S18/1 D Layer','','','Ionos',408,'',104 from dual union all
select 4267,'1956-S69','1956-09-25 00:00:00','military missile','success',1013,'M-5RD A2-9','M5-RD 9','R-7 component test','','Test',636,'',500 from dual union all
select 42670,'1977-S400','1977-08-01 12:14:00','suborbital rocket','success',1284,'INTA SLW-7741','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42671,'1977-S401','1977-08-05 11:35:00','suborbital rocket','success',1284,'INTA SLW-7742','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42672,'1977-S402','1977-08-12 11:31:00','suborbital rocket','success',1284,'INTA SLW-7743','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42673,'1977-S403','1977-08-19 11:39:00','suborbital rocket','success',1284,'INTA SLW-7744','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42674,'1977-S404','1977-09-07 11:31:00','suborbital rocket','success',1284,'INTA SLW-7745','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42675,'1977-S405','1977-09-07 12:08:00','suborbital rocket','success',1284,'INTA SLW-7746','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42676,'1977-S406','1977-09-14 11:31:00','suborbital rocket','success',1284,'INTA SLW-7747','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42677,'1977-S407','1977-09-21 11:31:00','suborbital rocket','success',1284,'INTA SLW-7748','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42678,'1977-S408','1977-09-28 12:30:00','suborbital rocket','success',1284,'INTA SLW-7749','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42679,'1977-S409','1977-10-05 12:34:00','suborbital rocket','success',1284,'INTA SLW-7750','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 4268,'1956-S70','1956-07-31 00:56:00','suborbital rocket','success',48,'USAF 71','Airglow','','','Aeronomy',640,'',129 from dual union all
select 42680,'1977-S41','1977-01-15 01:40:00','suborbital rocket','success',228,'NASA 21.49UL','Colo. microchannel','','','UV Astron',1873,'',268 from dual union all
select 42681,'1977-S410','1977-10-19 12:34:00','suborbital rocket','success',1284,'INTA SLW-7751','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42682,'1977-S411','1977-11-02 12:31:00','suborbital rocket','success',1284,'INTA SLW-7752','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42683,'1977-S412','1977-11-16 12:31:00','suborbital rocket','success',1284,'INTA SLW-7753','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42684,'1977-S413','1977-11-30 12:31:00','suborbital rocket','success',1284,'INTA SLW-7754','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42685,'1977-S414','1977-11-30 12:57:00','suborbital rocket','success',1284,'INTA SLW-7755','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42686,'1977-S415','1977-12-21 12:42:00','suborbital rocket','success',1284,'INTA SLW-7756','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42687,'1977-S416','1977-02-02 11:38:00','suborbital rocket','success',1284,'INTA SLW-7711','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 42688,'1977-S417','1977-06-01 07:45:00','suborbital rocket','success',725,'','','Ozone test','','Aeron',1467,'',80 from dual union all
select 42689,'1977-S418','1977-09-28 04:30:00','suborbital rocket','success',725,'','','Ozonesonde','','Aeron',1467,'',80 from dual union all
select 4269,'1956-S71','1956-08-08 08:25:00','military missile','success',635,'RS-20    UX','Jupiter A','','','Test',280,'',90 from dual union all
select 42690,'1977-S419','1977-07-01 00:00:00','suborbital rocket','success',1311,'01','','','','Test',310,'',100 from dual union all
select 42691,'1977-S42','1977-01-15 06:17:00','suborbital rocket','success',38,'NASA 13.122UH','Wisc. Aerobee IV','','','XR Astron',1895,'',196 from dual union all
select 42692,'1977-S420','1977-08-12 04:05:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 2','','','Meteo',1822,'KORO',82 from dual union all
select 42693,'1977-S421','1977-08-19 22:05:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 15','','','Meteo',1822,'KORO',82 from dual union all
select 42694,'1977-S422','1977-08-10 14:05:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 1','','','Meteo',1822,'KORO',82 from dual union all
select 42695,'1977-S423','1977-08-12 11:11:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 4  Sphere','','','Meteo',1821,'',112 from dual union all
select 42696,'1977-S424','1977-08-15 19:35:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 6','','','Meteo',1822,'KORO',80 from dual union all
select 42697,'1977-S425','1977-08-16 18:40:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 8','','','Meteo',1822,'KORO',85 from dual union all
select 42698,'1977-S426','1977-08-19 01:20:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 11','','','Meteo',1822,'KORO',84 from dual union all
select 42699,'1977-S427','1977-08-19 03:08:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 12','','','Meteo',1822,'KORO',84 from dual union all
select 427,'1944-S44','1944-08-15 19:37:00','military missile','success',8,'18319','Ma140','','','Training',661,'',90 from dual union all
select 4270,'1956-S72','1956-08-08 23:00:00','suborbital rocket','success',839,'N-C 2 (770A/5656)','UM AM6.30','IGY AM6.30','','Aeron',1895,'',100 from dual union all
select 42700,'1977-S428','1977-08-19 05:07:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 13','','','Meteo',1822,'KORO',83 from dual union all
select 42701,'1977-S429','1977-08-19 20:05:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 14','','','Meteo',1822,'KORO',83 from dual union all
select 42702,'1977-S43','1977-01-16 12:45:00','suborbital rocket','success',668,'K-9M-58','NIR Galactic Plane','','','IR Astron/Ionos',839,'',324 from dual union all
select 42703,'1977-S430','1977-08-20 01:05:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 16','','','Meteo',1822,'KORO',83 from dual union all
select 42704,'1977-S431','1977-08-22 23:12:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 17','','','Meteo',1822,'KORO',83 from dual union all
select 42705,'1977-S432','1977-08-23 01:05:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 18','','','Meteo',1822,'KORO',84 from dual union all
select 42706,'1977-S433','1977-08-23 03:43:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 19','','','Meteo',1822,'KORO',84 from dual union all
select 42707,'1977-S434','1977-08-24 01:07:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 20','','','Meteo',1822,'KORO',84 from dual union all
select 42708,'1977-S435','1977-08-16 01:23:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 7  Sphere','','','Meteo',1821,'',111 from dual union all
select 42709,'1977-S436','1977-08-16 20:32:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 9  Sphere','','','Meteo',1821,'',115 from dual union all
select 4271,'1956-S73','1956-08-09 15:53:00','suborbital rocket','success',59,'SC 34  (S/N 17)','Sampling','','','Aeronomy',1869,'',85 from dual union all
select 42710,'1977-S437','1977-08-19 01:25:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 11 Sphere','','','Meteo',1821,'',113 from dual union all
select 42711,'1977-S438','1977-08-19 05:24:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 13 Sphere','','','Meteo',1821,'',114 from dual union all
select 42712,'1977-S439','1977-08-19 22:10:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 15 Datasonde','','','Meteo',1821,'',80 from dual union all
select 42713,'1977-S44','1977-01-18 19:03:00','military missile','success',1442,'C4X-1','','','','Test',246,'',1000 from dual union all
select 42714,'1977-S440','1977-09-17 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 42715,'1977-S441','1977-09-17 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 42716,'1977-S442','1977-11-01 00:00:00','military missile','success',976,'','8K63 43 RD UBP','','','Test',636,'',402 from dual union all
select 42717,'1977-S443','1977-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-13','Duga-1','','OT',368,'',1000 from dual union all
select 42718,'1977-S444','1977-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-14','Duga-1','','OT',368,'',1000 from dual union all
select 42719,'1977-S445','1977-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-15','Duga-1','','OT',368,'',1000 from dual union all
select 4272,'1956-S74','1956-08-09 22:47:00','suborbital rocket','success',839,'N-C 3 (5651/BRL)','BRL OB6.00','IGY OB6.00','','Aeron',1895,'',164 from dual union all
select 42720,'1977-S446','1977-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-16','Duga-1','','OT',368,'',1000 from dual union all
select 42721,'1977-S447','1977-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-11','Duga-1','','OT',368,'',1000 from dual union all
select 42722,'1977-S448','1977-01-01 00:00:00','military missile','success',1450,'','15A20 4 RD UBP-12','Duga-1','','OT',368,'',1000 from dual union all
select 42723,'1977-S449','1977-01-01 00:00:00','military missile','success',922,'','15Zh45 LKI M-3','','','Test',636,'',500 from dual union all
select 42724,'1977-S45','1977-01-21 08:04:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT53GM-1','GLORY TRIP 35GM-1','OT',1703,'',1300 from dual union all
select 42725,'1977-S450','1977-09-15 00:00:00','military missile','success',922,'','15Zh45 39 RD UBP','','','Test',636,'',500 from dual union all
select 42726,'1977-S46','1977-01-22 21:28:00','suborbital rocket','success',1212,'PHA F2A AF6-F2','DLR A-GR-78 Eveline','','','Aeron/Auroral EUV',29,'',277 from dual union all
select 42727,'1977-S47','1977-01-23 19:18:06','suborbital rocket','success',867,'NASA 18.1005UE','','','','Plasma',29,'',222 from dual union all
select 42728,'1977-S48','1977-01-25 01:30:00','suborbital rocket','success',1299,'TT-500-1F','','','','Test',1476,'',75 from dual union all
select 42729,'1977-S49','1977-01-26 08:25:00','suborbital rocket','success',117,'NRL NB24.276','Astro 8-1  (A8)','','','XR Astron',1873,'',347 from dual union all
select 4273,'1956-S75','1956-08-10 15:22:00','suborbital rocket','success',59,'SC 35  (S/N 10)','Sampling','','','Aeronomy',1869,'',85 from dual union all
select 42730,'1977-S50','1977-01-27 19:35:00','suborbital rocket','unknown',740,'MR-12','','','','Auroral/Aeron/Ionos',1010,'',157 from dual union all
select 42731,'1977-S51','1977-01-28 22:36:00','military missile','success',939,'','OT-26','','','OT',421,'SSBN 623',500 from dual union all
select 42732,'1977-S52','1977-01-28 22:37:00','military missile','success',939,'','OT-26','','','OT',421,'SSBN 623',500 from dual union all
select 42733,'1977-S53','1977-01-28 22:37:00','military missile','success',939,'','OT-26','','','OT',421,'SSBN 623',500 from dual union all
select 42734,'1977-S54','1977-01-28 22:37:00','military missile','success',939,'','OT-26','','','OT',421,'SSBN 623',500 from dual union all
select 42735,'1977-S55','1977-01-30 00:00:00','military missile','success',799,'STM-13W','Mk 12A RV x 3','R&D','STM-13W','Test',1707,'',1300 from dual union all
select 42736,'1977-S56','1977-02-01 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT132M','GLORY TRIP 132M','OT',1702,'',1300 from dual union all
select 42737,'1977-S57','1977-02-03 00:00:00','military missile','success',1004,'21T','15A14 UBP-9 ZP','GCh 15F143U1','','OT',486,'',1000 from dual union all
select 42738,'1977-S58','1977-02-05 03:03:00','military missile','success',939,'+1 other','FOT-27','','','OT',421,'SSBN 628',500 from dual union all
select 42739,'1977-S59','1977-02-06 20:50:43','suborbital rocket','success',867,'NASA 18.1004UE','','','','Plasma',29,'',224 from dual union all
select 4274,'1956-S76','1956-09-26 00:00:00','military missile','success',1013,'M-5RD A2-10','M5-RD 10','R-7 component test','','Test',636,'',500 from dual union all
select 42740,'1977-S60','1977-02-07 04:51:40','suborbital rocket','success',867,'NASA 18.1011UE','','','','Plasma',1047,'',270 from dual union all
select 42741,'1977-S61','1977-02-07 21:40:00','suborbital rocket','success',920,'MOP1','Ozone/La/ne','','','Atmospheric',1447,'',127 from dual union all
select 42742,'1977-S62','1977-02-07 23:09:00','suborbital rocket','success',920,'P192H','','','','Atmospheric',1447,'',146 from dual union all
select 42743,'1977-S63','1977-02-07 23:26:00','suborbital rocket','success',920,'P193H','','','','Ionos/Plasma',1447,'',135 from dual union all
select 42744,'1977-S64','1977-02-08 17:58:20','suborbital rocket','success',1206,'S22','SSC S22 Auroral Arc','','','Auroral',416,'',212 from dual union all
select 42745,'1977-S65','1977-02-08 22:11:51','suborbital rocket','success',1212,'PHA F1B AF6-F1','DLR A-GR-79 Silvia','','','Aeron/Auroral EUV',29,'',272 from dual union all
select 42746,'1977-S66','1977-02-09 17:29:00','suborbital rocket','unknown',740,'MR-12','','','','Auroral/Aeron/Ionos',1010,'',167 from dual union all
select 42747,'1977-S67','1977-02-10 00:22:50','suborbital rocket','success',1073,'S-310JA-2','VLF/HF','','','Ionos',1458,'',212 from dual union all
select 42748,'1977-S68','1977-02-10 03:15:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/Plasma',1010,'',152 from dual union all
select 42749,'1977-S69','1977-02-11 04:48:40','suborbital rocket','success',867,'NASA 18.1012UE','','','','Plasma',1047,'',212 from dual union all
select 4275,'1956-S77','1956-08-23 16:32:00','suborbital rocket','success',1521,'R-5','4203-5','','','Reentry Test',253,'',142 from dual union all
select 42750,'1977-S70','1977-02-11 12:33:00','suborbital rocket','success',920,'P194H','','','','Ionos',1447,'',140 from dual union all
select 42751,'1977-S71','1977-02-11 13:18:00','suborbital rocket','success',920,'P191H','','','','Ionos',1447,'',132 from dual union all
select 42752,'1977-S72','1977-02-11 13:59:00','suborbital rocket','success',920,'P189H','Atomic oxygen','','','Atmospheric',1447,'',142 from dual union all
select 42753,'1977-S73','1977-02-11 20:49:07','suborbital rocket','success',867,'NASA 18.213IE','SSC S21/2 TRIGGER','','','Auroral',417,'',220 from dual union all
select 42754,'1977-S74','1977-02-13 12:19:00','suborbital rocket','success',740,'MR-12','Composition','','','Aeron/Ionos/Plasma',1010,'',151 from dual union all
select 42755,'1977-S75','1977-02-15 12:50:00','suborbital rocket','success',837,'NASA 14.500UA','Auroral NO','','','Aeronomy',428,'',136 from dual union all
select 42756,'1977-S76','1977-02-15 14:38:00','suborbital rocket','success',41,'NASA 26.54UH','CAL XR 12','','','XR Astron',1846,'',176 from dual union all
select 42757,'1977-S77','1977-02-15 17:10:00','suborbital rocket','success',41,'NASA 26.52DH','NRL XR','','','XR Astron',1846,'',189 from dual union all
select 42758,'1977-S78','1977-02-15 18:48:10','suborbital rocket','success',837,'SSC S21/3','SSC S21/3 TRIGGER','','','Ionos',417,'',170 from dual;
insert into LAUNCH
select 42759,'1977-S79','1977-02-15 23:10:00','military missile','success',1442,'C4X-2','','','','Test',246,'',1000 from dual union all
select 4276,'1956-S78','1956-08-28 13:02:00','suborbital rocket','success',1521,'R-6','4203-6','','','Reentry Test',253,'',100 from dual union all
select 42760,'1977-S80','1977-02-16 08:26:00','military missile','success',799,'','Mk 12 RV x 3','OT GT27GM-2','GLORY TRIP 27GM-2','OT',1705,'',1300 from dual union all
select 42761,'1977-S81','1977-02-16 03:00:00','suborbital rocket','success',133,'NASA 25.15GG','','','','UV Astron',1895,'',198 from dual union all
select 42762,'1977-S82','1977-02-16 20:45:07','suborbital rocket','success',867,'NASA 18.212GE-IE','Ferdinand 46','','','Aeron/Auroral',29,'',248 from dual union all
select 42763,'1977-S83','1977-02-17 08:30:00','suborbital rocket','success',37,'NASA 13.124UG','JHU UVS (Sirius B)','','','UV Astron',1846,'',192 from dual union all
select 42764,'1977-S84','1977-02-17 14:05:00','suborbital rocket','success',41,'NASA 26.57GG','','','','UV Astron',1846,'',197 from dual union all
select 42765,'1977-S85','1977-02-17 18:12:00','military missile','success',917,'FOT-53','P-340 76 Cape 05','','','OT',231,'',180 from dual union all
select 42766,'1977-S86','1977-02-20 00:00:00','suborbital rocket','success',133,'','LUX/ASE Wolter I','','','Solar X',1895,'',270 from dual union all
select 42767,'1977-S87','1977-02-20 21:14:00','suborbital rocket','success',1212,'PHA F3C AB3-F3','DLR A-GR-80 Georgine','','','Aeron',29,'',256 from dual union all
select 42768,'1977-S88','1977-02-21 01:37:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Fields',1010,'',153 from dual union all
select 42769,'1977-S89','1977-02-21 14:30:00','suborbital rocket','success',41,'NASA 26.51UG','JHU UVT F9 (Al. Cen)','','','UV Astron',1846,'',174 from dual union all
select 4277,'1956-S79','1956-09-08 16:09:00','suborbital rocket','failure',1521,'R-7','4203-7','','','Reentry Test',253,'',394 from dual union all
select 42770,'1977-S90','1977-02-21 21:06:00','suborbital rocket','success',37,'NASA 13.123IS','UV Extinction','','','Aeron',1846,'',177 from dual union all
select 42771,'1977-S91','1977-02-23 13:20:00','suborbital rocket','success',41,'NASA 26.53DH','','','','XR Astron',1846,'',184 from dual union all
select 42772,'1977-S92','1977-02-24 01:30:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Fields',1010,'',170 from dual union all
select 42773,'1977-S93','1977-02-26 23:49:33','suborbital rocket','success',543,'DNA ST710.31-2','STRESS BETTY','','','Aeron/Ionos',389,'',179 from dual union all
select 42774,'1977-S94','1977-02-26 20:02:00','suborbital rocket','success',860,'NASA 31.01UA','Auroral NO','','','Aeronomy',428,'',142 from dual union all
select 42775,'1977-S95','1977-03-02 23:51:00','suborbital rocket','success',543,'DNA ST710.31-3','STRESS CAROLYN','','','Aeron/Ionos',389,'',191 from dual union all
select 42776,'1977-S96','1977-03-02 08:17:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT57GB','GLORY TRIP 57GB','OT',1712,'',1300 from dual union all
select 42777,'1977-S97','1977-03-02 18:28:00','military missile','failure',917,'FOT-54','P-341 76 Cape 06','','','OT',231,'',50 from dual union all
select 42778,'1977-S98','1977-03-02 20:31:00','military missile','success',917,'FOT-56','P-343 76 Cape 08','','','OT',231,'',180 from dual union all
select 42779,'1977-S99','1977-03-05 04:47:00','suborbital rocket','success',210,'AAF-NVB-02','Auroral','','','Auroral/Ionos/Field',428,'',327 from dual union all
select 4278,'1956-S80','1956-09-20 06:45:00','test rocket','success',637,'RS-27    UI','Jupiter C/Dummy Nose','','','Reentry Test',279,'',1097 from dual union all
select 42780,'1977-W01','1977-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 42781,'1977-W02','1977-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 42782,'1977-W03','1977-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 42783,'1977-W04','1977-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 42784,'1977-W05','1977-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 42785,'1977-W06','1977-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 42786,'1977-W07','1977-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 42787,'1977-W08','1977-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 42788,'1977-W09','1977-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 42789,'1977-W10','1977-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 4279,'1956-S81','1956-09-21 00:00:00','suborbital rocket','success',1324,'M2','','','','Test',1821,'',120 from dual union all
select 42790,'1977-W100','1977-01-17 16:44:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',73 from dual union all
select 42791,'1977-W1000','1977-06-15 04:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',59 from dual union all
select 42792,'1977-W1001','1977-06-15 13:44:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',68 from dual union all
select 42793,'1977-W1002','1977-06-15 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',55 from dual union all
select 42794,'1977-W1003','1977-06-15 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 42795,'1977-W1004','1977-06-15 14:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 42796,'1977-W1005','1977-06-15 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',71 from dual union all
select 42797,'1977-W1006','1977-06-15 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 42798,'1977-W1007','1977-06-15 15:35:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',63 from dual union all
select 42799,'1977-W1008','1977-06-15 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 428,'1944-S45','1944-08-16 07:23:00','military missile','success',8,'18318','Ma138','','','Training',661,'',90 from dual union all
select 4280,'1956-S82','1956-10-01 21:21:00','suborbital rocket','success',1521,'R-8','4203-8','','','Reentry Test',253,'',145 from dual union all
select 42800,'1977-W1009','1977-06-15 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 42801,'1977-W101','1977-01-17 16:52:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 42802,'1977-W1010','1977-06-15 16:32:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',65 from dual union all
select 42803,'1977-W1011','1977-06-15 16:36:00','suborbital rocket','unknown',1284,'FLIGHT 156     /T 1-','','','','Aeron',1821,'',70 from dual union all
select 42804,'1977-W1012','1977-06-15 17:21:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',63 from dual union all
select 42805,'1977-W1013','1977-06-15 17:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 42806,'1977-W1014','1977-06-15 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 42807,'1977-W1015','1977-06-15 21:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 42808,'1977-W1016','1977-06-15 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 42809,'1977-W1017','1977-06-16 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 4281,'1956-S83','1956-10-05 14:32:00','suborbital rocket','success',1521,'R-9','4203-9','','','Reentry Test',253,'',117 from dual union all
select 42810,'1977-W1018','1977-06-16 14:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 42811,'1977-W1019','1977-06-16 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 42812,'1977-W102','1977-01-17 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 42813,'1977-W1020','1977-06-16 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',68 from dual union all
select 42814,'1977-W1021','1977-06-16 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 42815,'1977-W1022','1977-06-16 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 42816,'1977-W1023','1977-06-17 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 42817,'1977-W1024','1977-06-17 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',58 from dual union all
select 42818,'1977-W1025','1977-06-17 12:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 42819,'1977-W1026','1977-06-17 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 4282,'1956-S84','1956-10-13 17:46:00','suborbital rocket','success',1521,'R-10','4203-10','','','Reentry Test',253,'',102 from dual union all
select 42820,'1977-W1027','1977-06-17 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 42821,'1977-W1028','1977-06-17 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 42822,'1977-W1029','1977-06-17 18:14:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',69 from dual union all
select 42823,'1977-W103','1977-01-17 19:30:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 42824,'1977-W1030','1977-06-17 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 42825,'1977-W1031','1977-06-17 18:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 42826,'1977-W1032','1977-06-17 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 42827,'1977-W1033','1977-06-18 00:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 42828,'1977-W1034','1977-06-18 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 42829,'1977-W1035','1977-06-19 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 4283,'1956-S85','1956-10-18 13:49:00','suborbital rocket','success',1521,'R-11','4203-11','','','Reentry Test',253,'',155 from dual union all
select 42830,'1977-W1036','1977-06-20 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 42831,'1977-W1037','1977-06-20 14:03:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',74 from dual union all
select 42832,'1977-W1038','1977-06-20 15:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 42833,'1977-W1039','1977-06-20 16:23:59','suborbital rocket','success',1061,'','','','','Meteo',276,'',69 from dual union all
select 42834,'1977-W104','1977-01-17 20:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',60 from dual union all
select 42835,'1977-W1040','1977-06-20 17:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 42836,'1977-W1041','1977-06-20 23:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 42837,'1977-W1042','1977-06-21 00:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 42838,'1977-W1043','1977-06-21 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',57 from dual union all
select 42839,'1977-W1044','1977-06-21 11:41:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 4284,'1956-S86','1956-10-18 09:05:00','military missile','success',635,'CC-14    HT','Jupiter A','','','Test',280,'',90 from dual union all
select 42840,'1977-W1045','1977-06-21 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 42841,'1977-W1046','1977-06-21 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 42842,'1977-W1047','1977-06-22 13:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 42843,'1977-W1048','1977-06-22 14:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',60 from dual union all
select 42844,'1977-W1049','1977-06-22 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 42845,'1977-W105','1977-01-17 20:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',62 from dual union all
select 42846,'1977-W1050','1977-06-22 15:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',76 from dual union all
select 42847,'1977-W1051','1977-06-22 16:09:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',57 from dual union all
select 42848,'1977-W1052','1977-06-22 16:25:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',65 from dual;
insert into LAUNCH
select 42849,'1977-W1053','1977-06-22 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 4285,'1956-S87','1956-10-20 22:01:00','suborbital rocket','success',839,'N-C 13','UM AM6.31','IGY AM6.31','','Aeron',428,'',113 from dual union all
select 42850,'1977-W1054','1977-06-22 16:40:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',59 from dual union all
select 42851,'1977-W1055','1977-06-22 18:00:00','suborbital rocket','unknown',1284,'FLIGHT 157     /TH1-','','','','Aeron',428,'',75 from dual union all
select 42852,'1977-W1056','1977-06-22 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 42853,'1977-W1057','1977-06-22 19:11:00','suborbital rocket','unknown',1284,'FLIGHT 158     /TH1-','','','','Aeron',428,'',72 from dual union all
select 42854,'1977-W1058','1977-06-22 20:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 42855,'1977-W1059','1977-06-22 21:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',73 from dual union all
select 42856,'1977-W106','1977-01-17 23:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 42857,'1977-W1060','1977-06-22 22:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 42858,'1977-W1061','1977-06-22 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 42859,'1977-W1062','1977-06-23 12:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 4286,'1956-S88','1956-10-23 08:40:00','suborbital rocket','success',48,'UM AM2.21','Pressure','IGY AM2.21','','Aeronomy',428,'',145 from dual union all
select 42860,'1977-W1063','1977-06-23 15:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 42861,'1977-W1064','1977-06-23 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 42862,'1977-W1065','1977-06-23 23:23:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 42863,'1977-W1066','1977-06-24 14:23:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 42864,'1977-W1067','1977-06-24 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',55 from dual union all
select 42865,'1977-W1068','1977-06-24 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 42866,'1977-W1069','1977-06-24 15:59:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',74 from dual union all
select 42867,'1977-W107','1977-01-18 12:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 42868,'1977-W1070','1977-06-24 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 42869,'1977-W1071','1977-06-24 20:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 4287,'1956-S89','1956-10-25 17:29:00','suborbital rocket','success',1521,'R-12','4203-12','','','Reentry Test',253,'',124 from dual union all
select 42870,'1977-W1072','1977-06-24 23:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 42871,'1977-W1073','1977-06-25 23:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 42872,'1977-W1074','1977-06-26 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',58 from dual union all
select 42873,'1977-W1075','1977-06-26 23:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 42874,'1977-W1076','1977-06-27 12:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 42875,'1977-W1077','1977-06-27 12:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 42876,'1977-W1078','1977-06-27 14:03:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',65 from dual union all
select 42877,'1977-W1079','1977-06-27 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 42878,'1977-W108','1977-01-18 16:07:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 42879,'1977-W1080','1977-06-27 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 4288,'1956-S90','1956-10-27 21:24:00','suborbital rocket','success',839,'N-C 8','UM AM6.08 Sphere','IGY AM6.08','','Aeron',108,'LSD-14',161 from dual union all
select 42880,'1977-W1081','1977-06-27 18:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 42881,'1977-W1082','1977-06-27 18:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 42882,'1977-W1083','1977-06-27 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 42883,'1977-W1084','1977-06-27 23:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 42884,'1977-W1085','1977-06-28 00:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 42885,'1977-W1086','1977-06-28 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 42886,'1977-W1087','1977-06-28 23:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 42887,'1977-W1088','1977-06-29 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 42888,'1977-W1089','1977-06-29 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',59 from dual union all
select 42889,'1977-W109','1977-01-18 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 4289,'1956-S91','1956-10-31 00:00:00','military missile','success',974,'','','','','Test',190,'B-67',150 from dual union all
select 42890,'1977-W1090','1977-06-29 04:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',59 from dual union all
select 42891,'1977-W1091','1977-06-29 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 42892,'1977-W1092','1977-06-29 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 42893,'1977-W1093','1977-06-29 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 42894,'1977-W1094','1977-06-29 14:53:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',54 from dual union all
select 42895,'1977-W1095','1977-06-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',76 from dual union all
select 42896,'1977-W1096','1977-06-29 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 42897,'1977-W1097','1977-06-29 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 42898,'1977-W1098','1977-06-29 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',52 from dual union all
select 42899,'1977-W1099','1977-06-29 17:07:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 429,'1944-S46','1944-08-16 07:38:00','military missile','success',8,'18317','Ma136','','','Training',661,'',90 from dual union all
select 4290,'1956-S92','1956-10-31 00:00:00','military missile','success',974,'','','','','Test',190,'B-67',150 from dual union all
select 42900,'1977-W11','1977-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 42901,'1977-W110','1977-01-18 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 42902,'1977-W1100','1977-06-29 19:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 42903,'1977-W1101','1977-07-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 42904,'1977-W1102','1977-07-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 42905,'1977-W1103','1977-06-30 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 42906,'1977-W1104','1977-06-30 12:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 42907,'1977-W1105','1977-06-30 15:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 42908,'1977-W1106','1977-06-30 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 42909,'1977-W1107','1977-06-30 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 4291,'1956-S93','1956-11-02 05:39:00','suborbital rocket','success',48,'USAF 74','Na Release 4','','','Aeronomy',640,'',146 from dual union all
select 42910,'1977-W1108','1977-06-30 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 42911,'1977-W1109','1977-07-01 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 42912,'1977-W111','1977-01-18 17:11:00','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',70 from dual union all
select 42913,'1977-W1110','1977-07-01 17:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 42914,'1977-W1111','1977-07-01 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 42915,'1977-W1112','1977-07-01 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 42916,'1977-W1113','1977-07-02 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 42917,'1977-W1114','1977-07-03 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 42918,'1977-W1115','1977-07-04 15:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 42919,'1977-W1116','1977-07-04 23:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 4292,'1956-S94','1956-11-02 18:40:00','suborbital rocket','success',839,'N-C 9','UM AM6.09 Sphere','IGY AM6.09','','Aeron',109,'LSD-14',131 from dual union all
select 42920,'1977-W1117','1977-07-05 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 42921,'1977-W1118','1977-07-05 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 42922,'1977-W1119','1977-07-05 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 42923,'1977-W112','1977-01-18 18:10:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',69 from dual union all
select 42924,'1977-W1120','1977-07-05 21:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 42925,'1977-W1121','1977-07-05 23:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 42926,'1977-W1122','1977-07-06 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 42927,'1977-W1123','1977-07-06 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 42928,'1977-W1124','1977-07-06 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 42929,'1977-W1125','1977-07-06 14:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',61 from dual union all
select 4293,'1956-S95','1956-11-04 18:54:00','suborbital rocket','success',839,'N-C 10','UM AM6.10 Sphere','IGY AM6.09','','Aeron',110,'LSD-14',162 from dual union all
select 42930,'1977-W1126','1977-07-06 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',79 from dual union all
select 42931,'1977-W1127','1977-07-06 15:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 42932,'1977-W1128','1977-07-06 15:38:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',54 from dual union all
select 42933,'1977-W1129','1977-07-06 18:10:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',72 from dual union all
select 42934,'1977-W113','1977-01-18 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 42935,'1977-W1130','1977-07-06 19:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 42936,'1977-W1131','1977-07-06 21:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 42937,'1977-W1132','1977-07-07 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 42938,'1977-W1133','1977-07-07 20:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual;
insert into LAUNCH
select 42939,'1977-W1134','1977-07-07 23:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 4294,'1956-S96','1956-11-05 13:09:00','suborbital rocket','success',1521,'R-13','4203-13','','','Reentry Test',253,'',118 from dual union all
select 42940,'1977-W1135','1977-07-08 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 42941,'1977-W1136','1977-07-08 13:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 42942,'1977-W1137','1977-07-08 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 42943,'1977-W1138','1977-07-08 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 42944,'1977-W1139','1977-07-08 23:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 42945,'1977-W114','1977-01-18 19:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',62 from dual union all
select 42946,'1977-W1140','1977-07-09 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 42947,'1977-W1141','1977-07-09 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 42948,'1977-W1142','1977-07-10 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 42949,'1977-W1143','1977-07-11 14:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 4295,'1956-S97','1956-11-07 15:02:00','suborbital rocket','success',839,'N-C 11','UM AM6.11 Sphere','IGY AM6.11','','Aeron',111,'LSD-14',169 from dual union all
select 42950,'1977-W1144','1977-07-11 15:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',62 from dual union all
select 42951,'1977-W1145','1977-07-11 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 42952,'1977-W1146','1977-07-11 18:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 42953,'1977-W1147','1977-07-11 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 42954,'1977-W1148','1977-07-12 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 42955,'1977-W1149','1977-07-12 17:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 42956,'1977-W115','1977-01-18 22:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 42957,'1977-W1150','1977-07-12 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 42958,'1977-W1151','1977-07-12 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 42959,'1977-W1152','1977-07-13 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',77 from dual union all
select 4296,'1956-S98','1956-11-10 15:17:00','suborbital rocket','success',839,'N-C 12','UM AM6.12 Sphere','IGY AM6.12','','Aeron',112,'LSD-14',161 from dual union all
select 42960,'1977-W1153','1977-07-13 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 42961,'1977-W1154','1977-07-13 18:10:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',72 from dual union all
select 42962,'1977-W1155','1977-07-13 20:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',60 from dual union all
select 42963,'1977-W1156','1977-07-13 20:36:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',78 from dual union all
select 42964,'1977-W1157','1977-07-13 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',72 from dual union all
select 42965,'1977-W1158','1977-07-13 21:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',68 from dual union all
select 42966,'1977-W1159','1977-07-13 23:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 42967,'1977-W116','1977-01-19 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 42968,'1977-W1160','1977-07-14 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 42969,'1977-W1161','1977-07-14 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 4297,'1956-S99','1956-11-14 02:05:00','military missile','success',635,'RS-28    UL','Jupiter A/Sandia PL','','','Test',280,'',90 from dual union all
select 42970,'1977-W1162','1977-07-14 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 42971,'1977-W1163','1977-07-15 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 42972,'1977-W1164','1977-07-15 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 42973,'1977-W1165','1977-07-15 15:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 42974,'1977-W1166','1977-07-15 18:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 42975,'1977-W1167','1977-07-15 18:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 42976,'1977-W1168','1977-07-15 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 42977,'1977-W1169','1977-07-15 22:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 42978,'1977-W117','1977-01-19 15:22:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',74 from dual union all
select 42979,'1977-W1170','1977-07-15 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 4298,'1956-W01','1956-02-07 00:00:00','suborbital rocket','success',716,'WASP 1','','','','Test',1057,'',75 from dual union all
select 42980,'1977-W1171','1977-07-16 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 42981,'1977-W1172','1977-07-16 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 42982,'1977-W1173','1977-07-16 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 42983,'1977-W1174','1977-07-17 14:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 42984,'1977-W1175','1977-07-17 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 42985,'1977-W1176','1977-07-17 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 42986,'1977-W1177','1977-07-17 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 42987,'1977-W1178','1977-07-18 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 42988,'1977-W1179','1977-07-18 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 42989,'1977-W118','1977-01-19 15:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 4299,'1956-W02','1956-04-01 00:00:00','suborbital rocket','success',716,'WASP 1','','','','Aeronomy',199,'DD-787',75 from dual union all
select 42990,'1977-W1180','1977-07-18 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 42991,'1977-W1181','1977-07-18 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 42992,'1977-W1182','1977-07-18 19:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',63 from dual union all
select 42993,'1977-W1183','1977-07-18 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 42994,'1977-W1184','1977-07-18 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 42995,'1977-W1185','1977-07-18 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 42996,'1977-W1186','1977-07-18 23:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 42997,'1977-W1187','1977-07-19 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 42998,'1977-W1188','1977-07-19 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 42999,'1977-W1189','1977-07-19 17:03:00','suborbital rocket','unknown',1284,'FLIGHT 159     /T 1-','ASRP Ozone','','','Aeron',1821,'',73 from dual union all
select 43,'1944-A07','1944-08-14 06:37:00','military missile','failure',1470,'18298','Ma115','','','Training',661,'',0 from dual union all
select 430,'1944-S47','1944-08-16 10:40:00','military missile','success',1470,'18295','Ma111','','','Training',661,'',90 from dual union all
select 4300,'1956-W03','1956-04-01 00:00:00','suborbital rocket','success',716,'WASP 1','','','','Aeronomy',199,'DD-790',75 from dual union all
select 43000,'1977-W119','1977-01-19 16:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 43001,'1977-W1190','1977-07-19 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 43002,'1977-W1191','1977-07-19 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',53 from dual union all
select 43003,'1977-W1192','1977-07-19 22:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 43004,'1977-W1193','1977-07-19 23:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 43005,'1977-W1194','1977-07-20 15:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 43006,'1977-W1195','1977-07-20 16:41:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',54 from dual union all
select 43007,'1977-W1196','1977-07-20 17:06:00','suborbital rocket','unknown',1284,'FLIGHT 160     /T 1-','ASRP Ozone','','','Aeron',1821,'',73 from dual union all
select 43008,'1977-W1197','1977-07-20 17:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',53 from dual union all
select 43009,'1977-W1198','1977-07-20 18:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',62 from dual union all
select 4301,'1956-W04','1956-05-01 00:00:00','suborbital rocket','success',716,'WASP 1','','','','Aeronomy',199,'DD-787',75 from dual union all
select 43010,'1977-W1199','1977-07-20 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 43011,'1977-W12','1977-01-03 11:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1297,'PRILIV',58 from dual union all
select 43012,'1977-W120','1977-01-19 16:34:59','suborbital rocket','success',1061,'','Chaff','','','Meteo',1821,'',70 from dual union all
select 43013,'1977-W1200','1977-07-20 18:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43014,'1977-W1201','1977-07-20 18:20:00','suborbital rocket','unknown',1284,'FLIGHT 161     /TH1-','','','','Aeron',428,'',73 from dual union all
select 43015,'1977-W1202','1977-07-20 18:30:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',72 from dual union all
select 43016,'1977-W1203','1977-07-20 18:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 43017,'1977-W1204','1977-07-20 20:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',62 from dual union all
select 43018,'1977-W1205','1977-07-20 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',72 from dual union all
select 43019,'1977-W1206','1977-07-20 22:13:00','suborbital rocket','unknown',1284,'FLIGHT 162     /T 1-','ASRP Ozone','','','Aeron',1821,'',72 from dual union all
select 4302,'1956-W05','1956-05-01 00:00:00','suborbital rocket','success',716,'WASP 1','','','','Aeronomy',199,'DD-790',75 from dual union all
select 43020,'1977-W1207','1977-07-20 23:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 43021,'1977-W1208','1977-07-21 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 43022,'1977-W1209','1977-07-21 11:54:00','suborbital rocket','unknown',1284,'FLIGHT 163     /T 1-','ASRP Ozone','','','Aeron',1821,'',73 from dual union all
select 43023,'1977-W121','1977-01-19 17:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 43024,'1977-W1210','1977-07-21 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 43025,'1977-W1211','1977-07-21 14:28:00','suborbital rocket','unknown',1284,'FLIGHT 164     /T 1-','ASRP Ozone','','','Aeron',1821,'',73 from dual union all
select 43026,'1977-W1212','1977-07-21 17:03:00','suborbital rocket','unknown',1284,'FLIGHT 165     /T 1-','ASRP Ozone','','','Aeron',1821,'',74 from dual union all
select 43027,'1977-W1213','1977-07-21 17:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual;
insert into LAUNCH
select 43028,'1977-W1214','1977-07-21 18:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 43029,'1977-W1215','1977-07-21 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 4303,'1956-W06','1956-05-09 00:00:00','suborbital rocket','unknown',716,'1-12','','','','Test',1895,'',70 from dual union all
select 43030,'1977-W1216','1977-07-21 19:38:00','suborbital rocket','unknown',1284,'FLIGHT 166     /T 1-','ASRP Ozone','','','Aeron',1821,'',74 from dual union all
select 43031,'1977-W1217','1977-07-21 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 43032,'1977-W1218','1977-07-21 22:13:00','suborbital rocket','unknown',1284,'FLIGHT 167     /T 1-','ASRP Ozone','','','Aeron',1821,'',73 from dual union all
select 43033,'1977-W1219','1977-07-21 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 43034,'1977-W122','1977-01-19 17:15:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 43035,'1977-W1220','1977-07-22 12:10:00','suborbital rocket','unknown',1284,'FLIGHT 168     /T 1-','ASRP Ozone','','','Aeron',1821,'',74 from dual union all
select 43036,'1977-W1221','1977-07-22 15:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43037,'1977-W1222','1977-07-22 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 43038,'1977-W1223','1977-07-22 17:22:00','suborbital rocket','unknown',1284,'FLIGHT 169     /T 1-','ASRP Ozone','','','Aeron',1821,'',72 from dual union all
select 43039,'1977-W1224','1977-07-22 18:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 4304,'1956-W07','1956-05-10 00:00:00','suborbital rocket','success',716,'13-20','','','','Test',1895,'',75 from dual union all
select 43040,'1977-W1225','1977-07-22 18:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 43041,'1977-W1226','1977-07-22 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',69 from dual union all
select 43042,'1977-W1227','1977-07-22 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 43043,'1977-W1228','1977-07-22 21:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 43044,'1977-W1229','1977-07-22 23:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',53 from dual union all
select 43045,'1977-W123','1977-01-19 17:30:00','suborbital rocket','unknown',1284,'T 1-8414','Winter Strato','','','Aeron',1821,'',71 from dual union all
select 43046,'1977-W1230','1977-07-23 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 43047,'1977-W1231','1977-07-23 04:02:00','suborbital rocket','success',1281,'T 1-9103','','','','Aeron',1821,'',72 from dual union all
select 43048,'1977-W1232','1977-07-23 04:15:00','suborbital rocket','success',1281,'T 1-9104','','','','Aeron',1821,'',71 from dual union all
select 43049,'1977-W1233','1977-07-23 14:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 4305,'1957 ALP','1957-10-04 19:28:34','orbital','success',1255,'M1-PS','PS-1','PS-1','','Sat',480,'',null from dual union all
select 43050,'1977-W1234','1977-07-23 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',67 from dual union all
select 43051,'1977-W1235','1977-07-23 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 43052,'1977-W1236','1977-07-23 23:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 43053,'1977-W1237','1977-07-24 14:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',59 from dual union all
select 43054,'1977-W1238','1977-07-24 20:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',69 from dual union all
select 43055,'1977-W1239','1977-07-24 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 43056,'1977-W124','1977-01-19 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 43057,'1977-W1240','1977-07-25 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 43058,'1977-W1241','1977-07-25 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 43059,'1977-W1242','1977-07-25 17:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 4306,'1957 BET','1957-11-03 02:30:42','orbital','success',1255,'M1-2PS','PS-2','PS-2','','Sat',480,'',null from dual union all
select 43060,'1977-W1243','1977-07-25 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 43061,'1977-W1244','1977-07-25 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 43062,'1977-W1245','1977-07-26 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 43063,'1977-W1246','1977-07-26 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 43064,'1977-W1247','1977-07-26 18:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',64 from dual union all
select 43065,'1977-W1248','1977-07-26 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 43066,'1977-W1249','1977-07-26 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 43067,'1977-W125','1977-01-19 18:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',66 from dual union all
select 43068,'1977-W1250','1977-07-27 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 43069,'1977-W1251','1977-07-27 04:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1634,'USHAKOV',60 from dual union all
select 4307,'1957-A01','1957-01-01 00:00:00','atmospheric rocket','success',1052,'','Heat Transfer','','','Test',1821,'F2H2',4 from dual union all
select 43070,'1977-W1252','1977-07-27 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 43071,'1977-W1253','1977-07-27 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 43072,'1977-W1254','1977-07-27 17:10:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 43073,'1977-W1255','1977-07-27 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 43074,'1977-W1256','1977-07-27 18:30:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',72 from dual union all
select 43075,'1977-W1257','1977-07-27 19:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',59 from dual union all
select 43076,'1977-W1258','1977-07-27 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 43077,'1977-W1259','1977-07-27 21:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',58 from dual union all
select 43078,'1977-W126','1977-01-19 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 43079,'1977-W1260','1977-07-27 21:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',76 from dual union all
select 4308,'1957-A02','1957-01-03 00:00:00','military missile','success',255,'624-3','','','','OT',293,'',20 from dual union all
select 43080,'1977-W1261','1977-07-27 21:27:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',64 from dual union all
select 43081,'1977-W1262','1977-07-27 21:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',68 from dual union all
select 43082,'1977-W1263','1977-07-28 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 43083,'1977-W1264','1977-07-28 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 43084,'1977-W1265','1977-07-28 18:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 43085,'1977-W1266','1977-07-28 19:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 43086,'1977-W1267','1977-07-28 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 43087,'1977-W1268','1977-07-28 22:17:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 43088,'1977-W1269','1977-07-29 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',78 from dual union all
select 43089,'1977-W127','1977-01-19 21:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',65 from dual union all
select 4309,'1957-A03','1957-01-11 00:00:00','military missile','success',1214,'N-69D 52-10983','','','','Test',224,'',10 from dual union all
select 43090,'1977-W1270','1977-07-29 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',66 from dual union all
select 43091,'1977-W1271','1977-07-29 15:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 43092,'1977-W1272','1977-07-29 19:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',61 from dual union all
select 43093,'1977-W1273','1977-07-29 19:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 43094,'1977-W1274','1977-07-29 20:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',60 from dual union all
select 43095,'1977-W1275','1977-07-29 21:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 43096,'1977-W1276','1977-07-29 21:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 43097,'1977-W1277','1977-07-29 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 43098,'1977-W1278','1977-07-30 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 43099,'1977-W1279','1977-07-30 14:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 431,'1944-S48','1944-08-16 10:53:00','military missile','success',8,'18323','Ma129','','','Training',661,'',90 from dual union all
select 4310,'1957-A04','1957-01-23 00:00:00','military missile','success',1214,'N-69D 53-8173','','','','Test',224,'',10 from dual union all
select 43100,'1977-W128','1977-01-19 21:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 43101,'1977-W1280','1977-07-30 18:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 43102,'1977-W1281','1977-07-30 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 43103,'1977-W1282','1977-07-30 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 43104,'1977-W1283','1977-08-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43105,'1977-W1284','1977-08-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43106,'1977-W1285','1977-08-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43107,'1977-W1286','1977-08-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43108,'1977-W1287','1977-07-31 14:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 43109,'1977-W1288','1977-07-31 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 4311,'1957-A05','1957-01-24 00:00:00','atmospheric rocket','success',545,'','Cone RV','','','Reentry Test',1821,'',10 from dual union all
select 43110,'1977-W1289','1977-07-31 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 43111,'1977-W129','1977-01-19 21:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',74 from dual union all
select 43112,'1977-W1290','1977-07-31 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43113,'1977-W1291','1977-08-01 14:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',65 from dual union all
select 43114,'1977-W1292','1977-08-01 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 43115,'1977-W1293','1977-08-01 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43116,'1977-W1294','1977-08-01 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 43117,'1977-W1295','1977-08-01 18:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual;
insert into LAUNCH
select 43118,'1977-W1296','1977-08-01 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 43119,'1977-W1297','1977-08-01 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 4312,'1957-A06','1957-01-26 00:45:00','military missile','failure',1366,'101','R&D Series I','','','Test',234,'',0 from dual union all
select 43120,'1977-W1298','1977-08-02 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 43121,'1977-W1299','1977-08-02 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 43122,'1977-W13','1977-01-03 15:45:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',52 from dual union all
select 43123,'1977-W130','1977-01-19 21:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 43124,'1977-W1300','1977-08-02 17:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 43125,'1977-W1301','1977-08-02 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 43126,'1977-W1302','1977-08-02 23:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43127,'1977-W1303','1977-08-03 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 43128,'1977-W1304','1977-08-03 14:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',65 from dual union all
select 43129,'1977-W1305','1977-08-03 16:26:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 4313,'1957-A07','1957-01-29 16:30:00','suborbital rocket','failure',1521,'R-20','4203-20','','','Reentry Test',253,'',2 from dual union all
select 43130,'1977-W1306','1977-08-03 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43131,'1977-W1307','1977-08-03 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 43132,'1977-W1308','1977-08-03 18:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 43133,'1977-W1309','1977-08-03 22:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1634,'USHAKOV',60 from dual union all
select 43134,'1977-W131','1977-01-20 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',66 from dual union all
select 43135,'1977-W1310','1977-08-03 22:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',75 from dual union all
select 43136,'1977-W1311','1977-08-04 00:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 43137,'1977-W1312','1977-08-04 14:40:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',68 from dual union all
select 43138,'1977-W1313','1977-08-04 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 43139,'1977-W1314','1977-08-04 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 4314,'1957-A08','1957-02-05 00:00:00','military missile','success',1214,'N-69D 52-10983','','','','Test',224,'',10 from dual union all
select 43140,'1977-W1315','1977-08-05 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 43141,'1977-W1316','1977-08-05 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 43142,'1977-W1317','1977-08-05 14:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 43143,'1977-W1318','1977-08-05 16:17:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',51 from dual union all
select 43144,'1977-W1319','1977-08-05 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',56 from dual union all
select 43145,'1977-W132','1977-01-20 12:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 43146,'1977-W1320','1977-08-05 17:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',62 from dual union all
select 43147,'1977-W1321','1977-08-05 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 43148,'1977-W1322','1977-08-05 21:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1634,'USHAKOV',63 from dual union all
select 43149,'1977-W1323','1977-08-05 21:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 4315,'1957-A09','1957-02-13 00:00:00','military missile','success',255,'624-4','','','','OT',293,'',20 from dual union all
select 43150,'1977-W1324','1977-08-05 23:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 43151,'1977-W1325','1977-08-06 14:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',66 from dual union all
select 43152,'1977-W1326','1977-08-06 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 43153,'1977-W1327','1977-08-06 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 43154,'1977-W1328','1977-08-07 18:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43155,'1977-W1329','1977-08-07 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 43156,'1977-W133','1977-01-20 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 43157,'1977-W1330','1977-08-08 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 43158,'1977-W1331','1977-08-08 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',61 from dual union all
select 43159,'1977-W1332','1977-08-08 14:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 4316,'1957-A10','1957-02-13 00:00:00','atmospheric rocket','success',365,'','Delta-wing model','','','Test',1821,'',10 from dual union all
select 43160,'1977-W1333','1977-08-08 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 43161,'1977-W1334','1977-08-08 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 43162,'1977-W1335','1977-08-08 18:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 43163,'1977-W1336','1977-08-08 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43164,'1977-W1337','1977-08-09 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 43165,'1977-W1338','1977-08-09 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 43166,'1977-W1339','1977-08-09 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 43167,'1977-W134','1977-01-20 15:20:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',63 from dual union all
select 43168,'1977-W1340','1977-08-09 23:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 43169,'1977-W1341','1977-08-10 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',72 from dual union all
select 4317,'1957-A100','1957-09-18 17:42:00','suborbital rocket','failure',841,'NN7.48F','','IGY NN7.48F','','Solar UV/Solar X',1415,'',21 from dual union all
select 43170,'1977-W1343','1977-08-10 14:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 43171,'1977-W1344','1977-08-10 16:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 43172,'1977-W1345','1977-08-10 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 43173,'1977-W1346','1977-08-10 18:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 43174,'1977-W1347','1977-08-10 21:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1634,'USHAKOV',62 from dual union all
select 43175,'1977-W1348','1977-08-10 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43176,'1977-W1349','1977-08-11 21:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 43177,'1977-W135','1977-01-20 16:10:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 43178,'1977-W1350','1977-08-12 16:41:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',54 from dual union all
select 43179,'1977-W1351','1977-08-12 17:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 4318,'1957-A101','1957-09-19 14:45:00','military missile','success',1214,'N-69E N-3322','','','','Test',224,'',10 from dual union all
select 43180,'1977-W1352','1977-08-12 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 43181,'1977-W1353','1977-08-12 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43182,'1977-W1354','1977-08-12 19:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 43183,'1977-W1355','1977-08-12 21:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1634,'USHAKOV',61 from dual union all
select 43184,'1977-W1356','1977-08-12 21:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 43185,'1977-W1357','1977-08-15 15:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 43186,'1977-W1358','1977-08-15 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 43187,'1977-W1359','1977-08-15 16:19:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 43188,'1977-W136','1977-01-20 17:25:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 43189,'1977-W1360','1977-08-15 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',55 from dual union all
select 4319,'1957-A102','1957-09-20 10:03:00','suborbital rocket','success',660,'K-4-1','','','','CR',7,'',40 from dual union all
select 43190,'1977-W1361','1977-08-15 20:08:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 43191,'1977-W1362','1977-08-15 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',50 from dual union all
select 43192,'1977-W1363','1977-08-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',69 from dual union all
select 43193,'1977-W1364','1977-08-16 18:35:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 8  Datasonde','','','Meteo',1821,'',76 from dual union all
select 43194,'1977-W1365','1977-08-17 15:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 43195,'1977-W1366','1977-08-17 15:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 43196,'1977-W1367','1977-08-17 16:36:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 43197,'1977-W1368','1977-08-17 17:00:00','suborbital rocket','unknown',1284,'FLIGHT 170     /T 1-','','','','Aeron',1821,'',73 from dual union all
select 43198,'1977-W1369','1977-08-17 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 43199,'1977-W137','1977-01-20 19:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 432,'1944-S49','1944-08-16 14:30:00','military missile','success',8,'18328','Ma133','','','Training',661,'',90 from dual union all
select 4320,'1957-A103','1957-09-22 10:00:00','suborbital rocket','success',660,'K-4-2','','','','CR',7,'',40 from dual union all
select 43200,'1977-W1370','1977-08-17 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 43201,'1977-W1371','1977-08-17 18:00:00','suborbital rocket','unknown',1284,'FLIGHT 171     /TH1-','','','','Aeron',428,'',72 from dual union all
select 43202,'1977-W1372','1977-08-17 18:21:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',72 from dual union all
select 43203,'1977-W1373','1977-08-17 20:48:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 43204,'1977-W1374','1977-08-17 21:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1634,'USHAKOV',63 from dual union all
select 43205,'1977-W1375','1977-08-17 21:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',74 from dual union all
select 43206,'1977-W1376','1977-08-18 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 43207,'1977-W1377','1977-08-19 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual;
insert into LAUNCH
select 43208,'1977-W1378','1977-08-19 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 43209,'1977-W1379','1977-08-19 17:09:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 4321,'1957-A104','1957-09-25 00:00:00','military missile','success',771,'GM-54-177','','','','Test',222,'',12 from dual union all
select 43210,'1977-W138','1977-01-20 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 43211,'1977-W1380','1977-08-19 21:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1634,'USHAKOV',63 from dual union all
select 43212,'1977-W1381','1977-08-19 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 43213,'1977-W1382','1977-08-19 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 43214,'1977-W1383','1977-08-22 14:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 43215,'1977-W1384','1977-08-22 16:54:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 43216,'1977-W1385','1977-08-22 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',68 from dual union all
select 43217,'1977-W1386','1977-08-22 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 43218,'1977-W1387','1977-08-22 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 43219,'1977-W1388','1977-08-22 20:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 4322,'1957-A105','1957-09-25 00:00:00','sounding rocket','failure',488,'','Shot 1','','','Test/Ionos',405,'Balloon',20 from dual union all
select 43220,'1977-W1389','1977-08-22 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 43221,'1977-W139','1977-01-21 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',883,'KORO',78 from dual union all
select 43222,'1977-W1390','1977-08-23 01:30:00','suborbital rocket','success',761,'MT135P T19','','','','Aeron?',1476,'',50 from dual union all
select 43223,'1977-W1391','1977-08-24 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 43224,'1977-W1392','1977-08-24 14:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 43225,'1977-W1393','1977-08-24 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 43226,'1977-W1394','1977-08-24 15:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',56 from dual union all
select 43227,'1977-W1395','1977-08-24 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',67 from dual union all
select 43228,'1977-W1396','1977-08-24 18:47:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',56 from dual union all
select 43229,'1977-W1397','1977-08-24 19:29:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',69 from dual union all
select 4323,'1957-A106','1957-09-25 19:57:00','military missile','failure',142,'6A','R&D Fairing','R&D','','Test',229,'',4 from dual union all
select 43230,'1977-W1398','1977-08-24 21:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43231,'1977-W1399','1977-08-24 23:18:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 43232,'1977-W14','1977-01-03 17:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 43233,'1977-W140','1977-01-21 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 43234,'1977-W1400','1977-08-25 16:10:59','suborbital rocket','success',1284,'','Chaff','','','Meteo',1895,'',57 from dual union all
select 43235,'1977-W1401','1977-08-25 21:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 43236,'1977-W1402','1977-08-26 01:30:00','suborbital rocket','success',761,'MT135P T20','','','','Aeron?',1476,'',55 from dual union all
select 43237,'1977-W1403','1977-08-26 15:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 43238,'1977-W1404','1977-08-26 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43239,'1977-W1405','1977-08-26 16:24:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 4324,'1957-A107','1957-09-26 00:00:00','military missile','success',262,'4','','','','Test',243,'',4 from dual union all
select 43240,'1977-W1406','1977-08-26 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 43241,'1977-W1407','1977-08-26 18:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 43242,'1977-W1408','1977-08-26 21:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1634,'USHAKOV',61 from dual union all
select 43243,'1977-W1409','1977-08-26 22:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 43244,'1977-W141','1977-01-21 15:14:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 43245,'1977-W1410','1977-08-29 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',52 from dual union all
select 43246,'1977-W1411','1977-08-29 15:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 43247,'1977-W1412','1977-08-29 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 43248,'1977-W1413','1977-08-29 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43249,'1977-W1414','1977-08-29 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 4325,'1957-A108','1957-09-26 00:00:00','military missile','success',771,'GM-54-27','','','','Test',222,'',12 from dual union all
select 43250,'1977-W1415','1977-08-29 17:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 43251,'1977-W1416','1977-08-29 19:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',63 from dual union all
select 43252,'1977-W1417','1977-08-29 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 43253,'1977-W1418','1977-08-29 21:02:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 43254,'1977-W1419','1977-08-29 21:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',66 from dual union all
select 43255,'1977-W142','1977-01-21 15:45:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',74 from dual union all
select 43256,'1977-W1420','1977-09-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43257,'1977-W1421','1977-08-31 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 43258,'1977-W1422','1977-08-31 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 43259,'1977-W1423','1977-08-31 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 4326,'1957-A109','1957-09-26 20:00:00','suborbital rocket','failure',838,'NN8.50F','','IGY NN8.50F','','Test',1415,'',16 from dual union all
select 43260,'1977-W1424','1977-08-31 16:07:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 43261,'1977-W1425','1977-08-31 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',53 from dual union all
select 43262,'1977-W1426','1977-08-31 18:10:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',72 from dual union all
select 43263,'1977-W1427','1977-08-31 22:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1634,'USHAKOV',61 from dual union all
select 43264,'1977-W1428','1977-09-01 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 43265,'1977-W1429','1977-09-01 21:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 43266,'1977-W143','1977-01-21 15:53:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 43267,'1977-W1430','1977-09-02 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 43268,'1977-W1431','1977-09-02 16:22:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',53 from dual union all
select 43269,'1977-W1432','1977-09-02 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 4327,'1957-A11','1957-02-13 01:51:00','suborbital rocket','success',1193,'SL01','RAE Test 1','','','Test',1839,'',12 from dual union all
select 43270,'1977-W1433','1977-09-02 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 43271,'1977-W1434','1977-09-03 06:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43272,'1977-W1435','1977-09-03 14:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 43273,'1977-W1436','1977-09-06 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 43274,'1977-W1437','1977-09-06 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 43275,'1977-W1438','1977-09-06 20:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 43276,'1977-W1439','1977-09-06 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 43277,'1977-W144','1977-01-21 15:59:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 43278,'1977-W1440','1977-09-06 22:07:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 43279,'1977-W1441','1977-09-07 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1634,'USHAKOV',59 from dual union all
select 4328,'1957-A110','1957-09-27 00:00:00','military missile','success',255,'624-10','','','','OT',293,'',20 from dual union all
select 43280,'1977-W1442','1977-09-07 14:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',78 from dual union all
select 43281,'1977-W1443','1977-09-07 16:47:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',53 from dual union all
select 43282,'1977-W1444','1977-09-07 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 43283,'1977-W1445','1977-09-07 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 43284,'1977-W1446','1977-09-07 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 43285,'1977-W1447','1977-09-08 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 43286,'1977-W1448','1977-09-08 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 43287,'1977-W1449','1977-09-09 14:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 43288,'1977-W145','1977-01-21 16:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 43289,'1977-W1450','1977-09-09 16:43:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 4329,'1957-A111','1957-10-01 00:00:00','suborbital rocket','success',122,'','Plumbob','','','Aeronomy',963,'',30 from dual union all
select 43290,'1977-W1451','1977-09-09 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 43291,'1977-W1452','1977-09-09 17:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 43292,'1977-W1453','1977-09-09 21:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 43293,'1977-W1454','1977-09-09 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',52 from dual union all
select 43294,'1977-W1455','1977-09-09 21:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 43295,'1977-W1456','1977-09-12 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 43296,'1977-W1457','1977-09-12 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 43297,'1977-W1458','1977-09-12 20:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 43298,'1977-W1459','1977-09-13 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual;
insert into LAUNCH
select 43299,'1977-W146','1977-01-21 17:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 433,'1944-S50','1944-08-17 12:42:00','military missile','success',8,'18320','Ma142','','','Training',661,'',90 from dual union all
select 4330,'1957-A112','1957-10-01 00:00:00','suborbital rocket','success',122,'','Plumbob','','','Aeronomy',963,'',30 from dual union all
select 43300,'1977-W1460','1977-09-13 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 43301,'1977-W1461','1977-09-13 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',76 from dual union all
select 43302,'1977-W1462','1977-09-14 13:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43303,'1977-W1463','1977-09-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',67 from dual union all
select 43304,'1977-W1464','1977-09-14 15:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',76 from dual union all
select 43305,'1977-W1465','1977-09-14 15:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 43306,'1977-W1466','1977-09-14 18:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 43307,'1977-W1467','1977-09-14 18:02:00','suborbital rocket','unknown',1284,'FLIGHT 172     /T 1-','','','','Aeron',1821,'',68 from dual union all
select 43308,'1977-W1468','1977-09-14 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 43309,'1977-W1469','1977-09-14 20:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 4331,'1957-A113','1957-10-01 00:00:00','military missile','success',771,'GM-54-140','','','','Test',222,'',12 from dual union all
select 43310,'1977-W147','1977-01-21 19:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 43311,'1977-W1470','1977-09-14 21:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',78 from dual union all
select 43312,'1977-W1471','1977-09-14 21:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43313,'1977-W1472','1977-09-16 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 43314,'1977-W1473','1977-09-16 14:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 43315,'1977-W1474','1977-09-16 15:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 43316,'1977-W1475','1977-09-16 16:01:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 43317,'1977-W1476','1977-09-16 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 43318,'1977-W1477','1977-09-16 19:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 43319,'1977-W1478','1977-09-16 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 4332,'1957-A114','1957-10-01 00:00:00','military missile','success',771,'GM-54-38','','','','Test',222,'',12 from dual union all
select 43320,'1977-W1479','1977-09-19 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 43321,'1977-W148','1977-01-21 20:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',64 from dual union all
select 43322,'1977-W1480','1977-09-19 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',55 from dual union all
select 43323,'1977-W1481','1977-09-19 15:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 43324,'1977-W1482','1977-09-19 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 43325,'1977-W1483','1977-09-19 16:52:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 43326,'1977-W1484','1977-09-19 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 43327,'1977-W1485','1977-09-19 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 43328,'1977-W1486','1977-09-19 20:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 43329,'1977-W1487','1977-09-20 17:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 4333,'1957-A115','1957-10-01 00:00:00','military missile','success',1214,'N-69D N-3316','','','','Test',238,'',10 from dual union all
select 43330,'1977-W1488','1977-09-21 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 43331,'1977-W1489','1977-09-21 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 43332,'1977-W149','1977-01-21 23:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 43333,'1977-W1490','1977-09-21 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 43334,'1977-W1491','1977-09-21 16:38:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 43335,'1977-W1492','1977-09-21 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',76 from dual union all
select 43336,'1977-W1493','1977-09-21 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 43337,'1977-W1494','1977-09-21 18:10:00','suborbital rocket','unknown',1284,'FLIGHT 173     /TH1-','','','','Aeron',428,'',72 from dual union all
select 43338,'1977-W1495','1977-09-21 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 43339,'1977-W1496','1977-09-21 21:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',73 from dual union all
select 4334,'1957-A116','1957-10-03 17:13:00','military missile','failure',1366,'107','R&D Series I','','','Test',233,'',0 from dual union all
select 43340,'1977-W1497','1977-09-21 21:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 43341,'1977-W1498','1977-09-22 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',52 from dual union all
select 43342,'1977-W1499','1977-09-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',78 from dual union all
select 43343,'1977-W15','1977-01-03 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 43344,'1977-W150','1977-01-22 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 43345,'1977-W1500','1977-09-22 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 43346,'1977-W1501','1977-09-22 21:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 43347,'1977-W1502','1977-09-22 22:07:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 43348,'1977-W1503','1977-09-23 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 43349,'1977-W1504','1977-09-23 14:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 4335,'1957-A117','1957-10-08 00:00:00','atmospheric rocket','success',552,'','Glide Rocket','','','Reentry Test',1821,'',30 from dual union all
select 43350,'1977-W1505','1977-09-23 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 43351,'1977-W1506','1977-09-23 15:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 43352,'1977-W1507','1977-09-23 15:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 43353,'1977-W1508','1977-09-23 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',53 from dual union all
select 43354,'1977-W1509','1977-09-23 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 43355,'1977-W151','1977-01-22 15:30:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 43356,'1977-W1510','1977-09-24 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 43357,'1977-W1511','1977-09-25 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 43358,'1977-W1512','1977-09-26 14:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 43359,'1977-W1513','1977-09-26 15:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 4336,'1957-A118','1957-10-10 00:00:00','atmospheric rocket','success',861,'','Heat Transfer','','','Reentry Test',1821,'',10 from dual union all
select 43360,'1977-W1514','1977-09-26 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 43361,'1977-W1515','1977-09-26 16:32:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 43362,'1977-W1516','1977-09-26 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 43363,'1977-W1517','1977-09-26 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 43364,'1977-W1518','1977-09-26 20:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',50 from dual union all
select 43365,'1977-W1519','1977-09-27 18:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 43366,'1977-W152','1977-01-22 16:54:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 43367,'1977-W1520','1977-09-27 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 43368,'1977-W1521','1977-09-27 22:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 43369,'1977-W1522','1977-09-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',72 from dual union all
select 4337,'1957-A119','1957-10-11 20:30:00','military missile','success',255,'624-11','','','','OT',293,'',20 from dual union all
select 43370,'1977-W1523','1977-09-28 14:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',52 from dual union all
select 43371,'1977-W1524','1977-09-28 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 43372,'1977-W1525','1977-09-28 15:45:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',73 from dual union all
select 43373,'1977-W1526','1977-09-28 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',53 from dual union all
select 43374,'1977-W1527','1977-09-28 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 43375,'1977-W1528','1977-09-28 17:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',66 from dual union all
select 43376,'1977-W1529','1977-09-28 18:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43377,'1977-W153','1977-01-22 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 43378,'1977-W1530','1977-09-28 20:26:59','suborbital rocket','success',1284,'','','','','Meteo',1895,'',71 from dual union all
select 43379,'1977-W1531','1977-09-28 20:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',74 from dual union all
select 4338,'1957-A12','1957-02-15 00:00:00','atmospheric rocket','success',564,'','D27 Model','','','Test',1821,'',5 from dual union all
select 43380,'1977-W1532','1977-09-29 18:07:00','suborbital rocket','success',1284,'','Chaff','','','Meteo',1895,'',55 from dual union all
select 43381,'1977-W1533','1977-09-29 19:30:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',68 from dual union all
select 43382,'1977-W1534','1977-09-29 22:19:59','suborbital rocket','success',1284,'','Chaff','','','Meteo',1895,'',54 from dual union all
select 43383,'1977-W1535','1977-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43384,'1977-W1536','1977-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43385,'1977-W1537','1977-09-30 02:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 43386,'1977-W1538','1977-09-30 13:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 43387,'1977-W1539','1977-09-30 14:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 43388,'1977-W154','1977-01-22 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',58 from dual;
insert into LAUNCH
select 43389,'1977-W1540','1977-09-30 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 4339,'1957-A120','1957-10-11 00:00:00','sounding rocket','failure',488,'','Shot 4','','','Test/Ionos',405,'Balloon',30 from dual union all
select 43390,'1977-W1541','1977-09-30 16:19:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 43391,'1977-W1542','1977-09-30 16:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 43392,'1977-W1543','1977-09-30 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 43393,'1977-W1544','1977-09-30 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 43394,'1977-W1545','1977-09-30 19:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 43395,'1977-W1546','1977-09-30 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 43396,'1977-W1547','1977-10-03 14:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 43397,'1977-W1548','1977-10-03 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 43398,'1977-W1549','1977-10-03 17:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 43399,'1977-W155','1977-01-22 23:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',69 from dual union all
select 434,'1944-S51','1944-08-17 13:14:00','military missile','success',8,'18316','Ma134','','','Training',661,'',90 from dual union all
select 4340,'1957-A121','1957-10-17 21:16:00','suborbital rocket','failure',718,'II5.58F','SUI 82','IGY II5.58F','','Aeron/Fields',733,'AGB-4',44 from dual union all
select 43400,'1977-W1550','1977-10-03 18:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 43401,'1977-W1551','1977-10-03 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 43402,'1977-W1552','1977-10-04 00:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 43403,'1977-W1553','1977-10-04 18:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 43404,'1977-W1554','1977-10-05 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 43405,'1977-W1555','1977-10-05 15:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 43406,'1977-W1556','1977-10-05 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',73 from dual union all
select 43407,'1977-W1557','1977-10-05 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43408,'1977-W1558','1977-10-05 16:23:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',51 from dual union all
select 43409,'1977-W1559','1977-10-05 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 4341,'1957-A122','1957-10-23 17:25:00','military missile','success',255,'624-16','','','','OT',293,'',20 from dual union all
select 43410,'1977-W156','1977-01-22 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43411,'1977-W1560','1977-10-05 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',73 from dual union all
select 43412,'1977-W1561','1977-10-05 21:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 43413,'1977-W1562','1977-10-07 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 43414,'1977-W1563','1977-10-07 16:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 43415,'1977-W1564','1977-10-07 16:08:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 43416,'1977-W1565','1977-10-07 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',67 from dual union all
select 43417,'1977-W1566','1977-10-07 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 43418,'1977-W1567','1977-10-07 21:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 43419,'1977-W1568','1977-10-08 01:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 4342,'1957-A123','1957-10-25 00:00:00','atmospheric rocket','success',549,'','GE RV Model','','','Reentry Test',1821,'',30 from dual union all
select 43420,'1977-W1569','1977-10-08 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',901,'KRENKEL''',59 from dual union all
select 43421,'1977-W157','1977-01-23 10:36:00','suborbital rocket','unknown',725,'','','','','Aeron',883,'KORO',77 from dual union all
select 43422,'1977-W1570','1977-10-08 04:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',59 from dual union all
select 43423,'1977-W1571','1977-10-10 16:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 43424,'1977-W1572','1977-10-11 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43425,'1977-W1573','1977-10-11 19:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 43426,'1977-W1574','1977-10-11 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 43427,'1977-W1575','1977-10-11 21:00:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 43428,'1977-W1576','1977-10-12 01:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 43429,'1977-W1577','1977-10-12 15:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 4343,'1957-A124','1957-10-27 00:00:00','military missile','success',262,'5','','','','Test',243,'',11 from dual union all
select 43430,'1977-W1578','1977-10-12 15:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43431,'1977-W1579','1977-10-12 17:00:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 43432,'1977-W158','1977-01-23 15:46:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 43433,'1977-W1580','1977-10-12 17:00:00','suborbital rocket','unknown',1284,'FLIGHT 175     /T 1-','','','','Aeron',1821,'',73 from dual union all
select 43434,'1977-W1581','1977-10-12 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 43435,'1977-W1582','1977-10-12 17:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 43436,'1977-W1583','1977-10-12 21:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',74 from dual union all
select 43437,'1977-W1584','1977-10-13 01:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',67 from dual union all
select 43438,'1977-W1585','1977-10-13 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',56 from dual union all
select 43439,'1977-W1586','1977-10-13 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 4344,'1957-A125','1957-10-30 00:00:00','military missile','failure',268,'2/3','','','','Test',1752,'',0 from dual union all
select 43440,'1977-W1587','1977-10-13 19:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 43441,'1977-W1588','1977-10-13 21:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 43442,'1977-W1589','1977-10-14 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43443,'1977-W159','1977-01-23 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 43444,'1977-W1590','1977-10-14 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 43445,'1977-W1591','1977-10-14 20:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 43446,'1977-W1592','1977-10-15 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',57 from dual union all
select 43447,'1977-W1593','1977-10-15 04:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',64 from dual union all
select 43448,'1977-W1594','1977-10-17 15:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43449,'1977-W1595','1977-10-17 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 4345,'1957-A126','1957-10-31 22:21:00','military missile','success',1214,'N-69E N-3324','','','','Test',224,'',10 from dual union all
select 43450,'1977-W1596','1977-10-17 16:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',59 from dual union all
select 43451,'1977-W1597','1977-10-17 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 43452,'1977-W1598','1977-10-17 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 43453,'1977-W1599','1977-10-17 22:01:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 43454,'1977-W16','1977-01-03 20:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 43455,'1977-W160','1977-01-23 18:00:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 43456,'1977-W1600','1977-10-18 01:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',67 from dual union all
select 43457,'1977-W1601','1977-10-19 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 43458,'1977-W1602','1977-10-19 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 43459,'1977-W1603','1977-10-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',78 from dual union all
select 4346,'1957-A127','1957-10-31 04:52:00','military missile','failure',635,'CC-41    TH','Jupiter A','','','Test',280,'',40 from dual union all
select 43460,'1977-W1604','1977-10-19 16:45:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',77 from dual union all
select 43461,'1977-W1605','1977-10-19 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 43462,'1977-W1606','1977-10-19 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 43463,'1977-W1607','1977-10-19 18:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',67 from dual union all
select 43464,'1977-W1608','1977-10-19 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 43465,'1977-W1609','1977-10-19 21:23:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',73 from dual union all
select 43466,'1977-W161','1977-01-23 22:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',64 from dual union all
select 43467,'1977-W1610','1977-10-19 21:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',73 from dual union all
select 43468,'1977-W1611','1977-10-20 01:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',66 from dual union all
select 43469,'1977-W1612','1977-10-20 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 4347,'1957-A128','1957-11-13 17:28:00','military missile','success',832,'11','G-26 No. 7','','','Test',281,'',20 from dual union all
select 43470,'1977-W1613','1977-10-20 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',65 from dual union all
select 43471,'1977-W1614','1977-10-20 14:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 43472,'1977-W1615','1977-10-20 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 43473,'1977-W1616','1977-10-21 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 43474,'1977-W1617','1977-10-21 15:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 43475,'1977-W1618','1977-10-21 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 43476,'1977-W1619','1977-10-21 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 43477,'1977-W162','1977-01-23 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 43478,'1977-W1620','1977-10-21 17:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual;
insert into LAUNCH
select 43479,'1977-W1621','1977-10-21 18:00:00','suborbital rocket','unknown',1284,'FLIGHT 176     /TH1-','','','','Aeron',428,'',72 from dual union all
select 4348,'1957-A129','1957-11-14 00:00:00','military missile','success',255,'624-12','','','','OT',293,'',20 from dual union all
select 43480,'1977-W1622','1977-10-21 19:31:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 43481,'1977-W1623','1977-10-21 19:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 43482,'1977-W1624','1977-10-22 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',61 from dual union all
select 43483,'1977-W1625','1977-10-24 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 43484,'1977-W1626','1977-10-24 14:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 43485,'1977-W1627','1977-10-25 15:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 43486,'1977-W1628','1977-10-25 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 43487,'1977-W1629','1977-10-25 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 43488,'1977-W163','1977-01-24 00:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',64 from dual union all
select 43489,'1977-W1630','1977-10-25 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 4349,'1957-A13','1957-02-18 00:00:00','military missile','success',1214,'N-69D 53-8175','','','','Test',224,'',10 from dual union all
select 43490,'1977-W1631','1977-10-26 02:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',63 from dual union all
select 43491,'1977-W1632','1977-10-26 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 43492,'1977-W1633','1977-10-26 15:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 43493,'1977-W1634','1977-10-26 15:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 43494,'1977-W1635','1977-10-26 16:00:00','suborbital rocket','unknown',725,'ISRO 8.350','','','','Aeron',1467,'',79 from dual union all
select 43495,'1977-W1636','1977-10-26 17:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 43496,'1977-W1637','1977-10-26 19:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 43497,'1977-W1638','1977-10-26 19:36:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',75 from dual union all
select 43498,'1977-W1639','1977-10-26 21:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',73 from dual union all
select 43499,'1977-W164','1977-01-24 14:30:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',68 from dual union all
select 435,'1944-S52','1944-08-17 13:31:00','military missile','success',1470,'18290','Ma109','','','Training',661,'',90 from dual union all
select 4350,'1957-A130','1957-11-14 00:00:00','military missile','success',771,'GM-54-138','','','','Test',222,'',12 from dual union all
select 43500,'1977-W1640','1977-10-27 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',62 from dual union all
select 43501,'1977-W1641','1977-10-27 15:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 43502,'1977-W1642','1977-10-28 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43503,'1977-W1643','1977-10-28 16:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 43504,'1977-W1644','1977-10-28 16:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 43505,'1977-W1645','1977-10-28 16:19:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 43506,'1977-W1646','1977-10-28 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 43507,'1977-W1647','1977-10-28 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 43508,'1977-W1648','1977-10-29 01:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43509,'1977-W1649','1977-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 4351,'1957-A131','1957-11-20 00:00:00','atmospheric rocket','success',441,'','MW-2 Model','','','Test',1821,'',10 from dual union all
select 43510,'1977-W165','1977-01-24 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 43511,'1977-W1650','1977-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43512,'1977-W1651','1977-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43513,'1977-W1652','1977-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43514,'1977-W1653','1977-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43515,'1977-W1654','1977-10-31 15:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 43516,'1977-W1655','1977-10-31 16:00:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 43517,'1977-W1656','1977-10-31 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43518,'1977-W1657','1977-10-31 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 43519,'1977-W1658','1977-10-31 17:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 4352,'1957-A132','1957-11-20 00:00:00','military missile','success',1214,'N-69D N-3316-2','','','','Test',224,'',10 from dual union all
select 43520,'1977-W1659','1977-10-31 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 43521,'1977-W166','1977-01-24 15:30:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',67 from dual union all
select 43522,'1977-W1660','1977-10-31 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 43523,'1977-W1661','1977-11-01 20:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 43524,'1977-W1662','1977-11-01 22:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',64 from dual union all
select 43525,'1977-W1663','1977-11-02 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 43526,'1977-W1664','1977-11-02 15:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 43527,'1977-W1665','1977-11-02 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 43528,'1977-W1666','1977-11-02 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 43529,'1977-W1667','1977-11-02 16:47:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 4353,'1957-A133','1957-11-26 16:46:00','military missile','success',262,'6','','','','Test',243,'',13 from dual union all
select 43530,'1977-W1668','1977-11-02 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43531,'1977-W1669','1977-11-02 20:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 43532,'1977-W167','1977-01-24 15:41:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 43533,'1977-W1670','1977-11-02 21:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 43534,'1977-W1671','1977-11-03 01:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 43535,'1977-W1672','1977-11-03 21:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',62 from dual union all
select 43536,'1977-W1673','1977-11-03 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 43537,'1977-W1674','1977-11-04 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 43538,'1977-W1675','1977-11-04 16:49:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 43539,'1977-W1676','1977-11-04 21:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 4354,'1957-A134','1957-11-27 02:10:00','military missile','failure',638,'AM-3A','R&D','','','Test',249,'',20 from dual union all
select 43540,'1977-W1677','1977-11-05 00:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 43541,'1977-W1678','1977-11-05 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',53 from dual union all
select 43542,'1977-W1679','1977-11-05 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',68 from dual union all
select 43543,'1977-W168','1977-01-24 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 43544,'1977-W1680','1977-11-05 12:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 43545,'1977-W1681','1977-11-05 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 43546,'1977-W1682','1977-11-07 15:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 43547,'1977-W1683','1977-11-07 16:36:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 43548,'1977-W1684','1977-11-07 16:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 43549,'1977-W1685','1977-11-07 17:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 4355,'1957-A135','1957-12-06 04:00:00','military missile','success',1214,'N-69E N-3326','','','','Test',224,'',10 from dual union all
select 43550,'1977-W1686','1977-11-07 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 43551,'1977-W1687','1977-11-07 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 43552,'1977-W1688','1977-11-07 20:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 43553,'1977-W1689','1977-11-07 21:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 43554,'1977-W169','1977-01-24 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 43555,'1977-W1690','1977-11-08 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 43556,'1977-W1691','1977-11-08 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',67 from dual union all
select 43557,'1977-W1692','1977-11-08 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',75 from dual union all
select 43558,'1977-W1693','1977-11-09 15:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 43559,'1977-W1694','1977-11-09 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 4356,'1957-A136','1957-12-09 00:00:00','military missile','success',255,'624-14','','','','OT',293,'',20 from dual union all
select 43560,'1977-W1695','1977-11-09 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 43561,'1977-W1696','1977-11-09 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 43562,'1977-W1697','1977-11-09 16:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 43563,'1977-W1698','1977-11-09 16:54:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 43564,'1977-W1699','1977-11-09 16:54:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',77 from dual union all
select 43565,'1977-W17','1977-01-04 00:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 43566,'1977-W170','1977-01-24 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43567,'1977-W1700','1977-11-09 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 43568,'1977-W1701','1977-11-09 17:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual;
insert into LAUNCH
select 43569,'1977-W1702','1977-11-09 21:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',62 from dual union all
select 4357,'1957-A137','1957-12-10 21:36:00','suborbital rocket','failure',839,'SS6.39','','IGY SS6.39','','Aeron',428,'',2 from dual union all
select 43570,'1977-W1703','1977-11-10 04:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',75 from dual union all
select 43571,'1977-W1704','1977-11-10 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 43572,'1977-W1705','1977-11-10 16:26:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 43573,'1977-W1706','1977-11-10 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 43574,'1977-W1707','1977-11-11 14:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 43575,'1977-W1708','1977-11-11 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 43576,'1977-W1709','1977-11-11 19:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 43577,'1977-W171','1977-01-24 18:00:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 43578,'1977-W1710','1977-11-11 20:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 43579,'1977-W1711','1977-11-12 06:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',50 from dual union all
select 4358,'1957-A138','1957-12-13 00:00:00','atmospheric rocket','success',862,'','Heat Transfer','','','Test',1821,'',10 from dual union all
select 43580,'1977-W1712','1977-11-13 05:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 43581,'1977-W1713','1977-11-13 10:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',67 from dual union all
select 43582,'1977-W1714','1977-11-13 12:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',59 from dual union all
select 43583,'1977-W1715','1977-11-14 16:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43584,'1977-W1716','1977-11-14 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43585,'1977-W1717','1977-11-14 17:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 43586,'1977-W1718','1977-11-14 17:26:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 43587,'1977-W1719','1977-11-14 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 43588,'1977-W172','1977-01-24 22:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 43589,'1977-W1720','1977-11-14 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 4359,'1957-A139','1957-12-14 00:00:00','military missile','success',1080,'F79','','','','Test',671,'',20 from dual union all
select 43590,'1977-W1721','1977-11-14 22:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 43591,'1977-W1722','1977-11-15 09:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 43592,'1977-W1723','1977-11-15 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',79 from dual union all
select 43593,'1977-W1724','1977-11-15 16:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 43594,'1977-W1725','1977-11-15 16:32:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 43595,'1977-W1726','1977-11-16 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43596,'1977-W1727','1977-11-16 15:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',52 from dual union all
select 43597,'1977-W1728','1977-11-16 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',76 from dual union all
select 43598,'1977-W1729','1977-11-16 15:56:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 43599,'1977-W173','1977-01-24 23:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',65 from dual union all
select 436,'1944-S53','1944-08-17 16:39:00','military missile','success',8,'18327','Ma147','','','Training',661,'',90 from dual union all
select 4360,'1957-A14','1957-02-28 00:00:00','military missile','success',255,'624-5','','','','OT',293,'',20 from dual union all
select 43600,'1977-W1730','1977-11-16 16:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 43601,'1977-W1731','1977-11-16 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 43602,'1977-W1732','1977-11-16 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',76 from dual union all
select 43603,'1977-W1733','1977-11-16 17:37:00','suborbital rocket','unknown',1284,'FLIGHT 177     /T 1-','','','','Aeron',1821,'',72 from dual union all
select 43604,'1977-W1734','1977-11-16 18:00:00','suborbital rocket','unknown',1284,'FLIGHT 178     /TH1-','','','','Aeron',428,'',75 from dual union all
select 43605,'1977-W1735','1977-11-16 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43606,'1977-W1736','1977-11-16 21:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',74 from dual union all
select 43607,'1977-W1737','1977-11-16 23:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 43608,'1977-W1738','1977-11-17 01:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',58 from dual union all
select 43609,'1977-W1739','1977-11-17 10:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',73 from dual union all
select 4361,'1957-A140','1957-12-16 00:00:00','military missile','success',1080,'F80','','','','Test',671,'',20 from dual union all
select 43610,'1977-W174','1977-01-25 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',73 from dual union all
select 43611,'1977-W1740','1977-11-17 16:40:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 43612,'1977-W1741','1977-11-17 16:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 43613,'1977-W1742','1977-11-17 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 43614,'1977-W1743','1977-11-17 21:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 43615,'1977-W1744','1977-11-18 13:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 43616,'1977-W1745','1977-11-18 15:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 43617,'1977-W1746','1977-11-18 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 43618,'1977-W1747','1977-11-18 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 43619,'1977-W1748','1977-11-18 19:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 4362,'1957-A141','1957-12-15 00:38:00','suborbital rocket','failure',839,'AM6.34','','IGY AM6.34','','Aeron',428,'',9 from dual union all
select 43620,'1977-W1749','1977-11-19 10:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 43621,'1977-W175','1977-01-25 12:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 43622,'1977-W1750','1977-11-21 15:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',58 from dual union all
select 43623,'1977-W1751','1977-11-21 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 43624,'1977-W1752','1977-11-21 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 43625,'1977-W1753','1977-11-21 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 43626,'1977-W1754','1977-11-21 20:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 43627,'1977-W1755','1977-11-22 16:44:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 43628,'1977-W1756','1977-11-22 20:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 43629,'1977-W1757','1977-11-22 22:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 4363,'1957-A142','1957-12-17 00:00:00','military missile','success',1080,'F81','','','','Test',671,'',20 from dual union all
select 43630,'1977-W1758','1977-11-22 22:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 43631,'1977-W1759','1977-11-23 15:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 43632,'1977-W176','1977-01-25 15:14:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 43633,'1977-W1760','1977-11-23 16:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 43634,'1977-W1761','1977-11-23 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 43635,'1977-W1762','1977-11-23 16:54:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 43636,'1977-W1763','1977-11-23 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 43637,'1977-W1764','1977-11-23 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 43638,'1977-W1765','1977-11-23 20:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',71 from dual union all
select 43639,'1977-W1766','1977-11-24 01:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 4364,'1957-A143','1957-12-19 00:00:00','military missile','success',1080,'F84','','','','Test',671,'',20 from dual union all
select 43640,'1977-W1767','1977-11-24 02:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 43641,'1977-W1768','1977-11-25 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',60 from dual union all
select 43642,'1977-W1769','1977-11-25 16:25:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',54 from dual union all
select 43643,'1977-W177','1977-01-25 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 43644,'1977-W1770','1977-11-25 18:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 43645,'1977-W1771','1977-11-26 01:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',58 from dual union all
select 43646,'1977-W1772','1977-11-26 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',74 from dual union all
select 43647,'1977-W1773','1977-11-26 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',71 from dual union all
select 43648,'1977-W1774','1977-11-28 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 43649,'1977-W1775','1977-11-28 16:42:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 4365,'1957-A144','1957-12-23 00:00:00','suborbital rocket','failure',839,'HUGO 2','Hurricane photo','','','Aeron',1821,'',10 from dual union all
select 43650,'1977-W1776','1977-11-28 16:48:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 43651,'1977-W1777','1977-11-28 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',77 from dual union all
select 43652,'1977-W1778','1977-11-28 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 43653,'1977-W1779','1977-11-28 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',70 from dual union all
select 43654,'1977-W178','1977-01-25 20:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',62 from dual union all
select 43655,'1977-W1780','1977-11-28 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 43656,'1977-W1781','1977-11-28 21:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 43657,'1977-W1782','1977-11-29 01:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',66 from dual union all
select 43658,'1977-W1783','1977-11-29 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',50 from dual;
insert into LAUNCH
select 43659,'1977-W1784','1977-11-29 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',79 from dual union all
select 4366,'1957-A145','1957-03-13 00:00:00','atmospheric rocket','success',836,'','BTL PIT 100','','','Test',1890,'',10 from dual union all
select 43660,'1977-W1785','1977-11-29 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 43661,'1977-W1786','1977-11-29 23:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 43662,'1977-W1787','1977-12-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43663,'1977-W1788','1977-12-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43664,'1977-W1789','1977-12-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43665,'1977-W179','1977-01-25 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 43666,'1977-W1790','1977-12-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43667,'1977-W1791','1977-12-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43668,'1977-W1792','1977-12-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43669,'1977-W1793','1977-12-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 4367,'1957-A146','1957-01-16 00:00:00','atmospheric rocket','success',836,'1249B-7970','Nike 567R','','','Test',1895,'',10 from dual union all
select 43670,'1977-W1794','1977-11-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',79 from dual union all
select 43671,'1977-W1795','1977-11-30 15:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',57 from dual union all
select 43672,'1977-W1796','1977-11-30 16:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',50 from dual union all
select 43673,'1977-W1797','1977-11-30 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 43674,'1977-W1798','1977-11-30 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 43675,'1977-W1799','1977-11-30 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43676,'1977-W18','1977-01-04 12:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 43677,'1977-W180','1977-01-26 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 43678,'1977-W1800','1977-11-30 19:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 43679,'1977-W1801','1977-11-30 20:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',72 from dual union all
select 4368,'1957-A147','1957-02-06 00:00:00','atmospheric rocket','success',836,'1249B-5775','Nike 572R','','','Test',1895,'',10 from dual union all
select 43680,'1977-W1802','1977-11-30 21:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 43681,'1977-W1803','1977-12-01 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',60 from dual union all
select 43682,'1977-W1804','1977-12-01 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 43683,'1977-W1805','1977-12-01 20:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 43684,'1977-W1806','1977-12-02 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',971,'MUSSON',66 from dual union all
select 43685,'1977-W1807','1977-12-02 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 43686,'1977-W1808','1977-12-02 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43687,'1977-W1809','1977-12-02 17:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 43688,'1977-W181','1977-01-26 14:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 43689,'1977-W1810','1977-12-02 19:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 4369,'1957-A148','1957-02-20 00:00:00','atmospheric rocket','success',836,'1249B-7035','Nike 574R','','','Test',1895,'',10 from dual union all
select 43690,'1977-W1811','1977-12-02 20:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',54 from dual union all
select 43691,'1977-W1812','1977-12-02 21:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 43692,'1977-W1813','1977-12-03 01:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 43693,'1977-W1814','1977-12-03 02:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',63 from dual union all
select 43694,'1977-W1815','1977-12-05 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',55 from dual union all
select 43695,'1977-W1816','1977-12-05 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 43696,'1977-W1817','1977-12-05 16:33:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 43697,'1977-W1818','1977-12-05 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 43698,'1977-W1819','1977-12-05 17:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43699,'1977-W182','1977-01-26 15:30:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 437,'1944-S54','1944-08-17 17:23:00','military missile','success',8,'18324','Ma144','','','Training',661,'',90 from dual union all
select 4370,'1957-A149','1957-05-22 00:00:00','atmospheric rocket','success',836,'1249B-50041','Nike 594R','','','Test',1895,'',10 from dual union all
select 43700,'1977-W1820','1977-12-05 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 43701,'1977-W1821','1977-12-05 21:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43702,'1977-W1822','1977-12-06 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 43703,'1977-W1823','1977-12-06 16:45:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 43704,'1977-W1824','1977-12-06 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 43705,'1977-W1825','1977-12-07 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',971,'MUSSON',65 from dual union all
select 43706,'1977-W1826','1977-12-07 15:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',79 from dual union all
select 43707,'1977-W1827','1977-12-07 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',63 from dual union all
select 43708,'1977-W1828','1977-12-07 16:02:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 43709,'1977-W1829','1977-12-07 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 4371,'1957-A15','1957-03-01 21:51:00','military missile','failure',638,'AM-1A','R&D','','','Test',279,'',14 from dual union all
select 43710,'1977-W183','1977-01-26 15:42:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',67 from dual union all
select 43711,'1977-W1830','1977-12-07 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 43712,'1977-W1831','1977-12-07 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43713,'1977-W1832','1977-12-07 16:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',51 from dual union all
select 43714,'1977-W1833','1977-12-07 18:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 43715,'1977-W1834','1977-12-07 19:53:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',79 from dual union all
select 43716,'1977-W1835','1977-12-08 13:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 43717,'1977-W1836','1977-12-08 17:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 43718,'1977-W1837','1977-12-08 21:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 43719,'1977-W1838','1977-12-09 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 4372,'1957-A150','1957-05-22 00:00:00','atmospheric rocket','success',836,'1249B-50042','Nike 595R','','','Test',1895,'',10 from dual union all
select 43720,'1977-W1839','1977-12-09 16:30:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 43721,'1977-W184','1977-01-26 16:07:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 43722,'1977-W1840','1977-12-09 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',50 from dual union all
select 43723,'1977-W1841','1977-12-09 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 43724,'1977-W1842','1977-12-09 19:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 43725,'1977-W1843','1977-12-12 15:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',54 from dual union all
select 43726,'1977-W1844','1977-12-12 16:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 43727,'1977-W1845','1977-12-12 16:15:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 43728,'1977-W1846','1977-12-12 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43729,'1977-W1847','1977-12-12 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 4373,'1957-A151','1957-08-21 00:00:00','atmospheric rocket','success',836,'1249B-7956','Nike 600R','','','Test',1895,'',10 from dual union all
select 43730,'1977-W1848','1977-12-12 17:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 43731,'1977-W1849','1977-12-12 20:35:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 43732,'1977-W185','1977-01-26 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 43733,'1977-W1850','1977-12-13 16:19:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 43734,'1977-W1851','1977-12-13 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 43735,'1977-W1852','1977-12-13 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 43736,'1977-W1853','1977-12-13 20:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 43737,'1977-W1854','1977-12-13 22:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 43738,'1977-W1855','1977-12-14 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',971,'MUSSON',56 from dual union all
select 43739,'1977-W1856','1977-12-14 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 4374,'1957-A152','1957-09-11 00:00:00','atmospheric rocket','success',836,'1249B-5685','Nike 602R','','','Test',1895,'',10 from dual union all
select 43740,'1977-W1857','1977-12-14 15:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 43741,'1977-W1858','1977-12-14 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43742,'1977-W1859','1977-12-14 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 43743,'1977-W186','1977-01-26 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',65 from dual union all
select 43744,'1977-W1860','1977-12-14 16:43:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 43745,'1977-W1861','1977-12-14 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 43746,'1977-W1862','1977-12-14 17:12:00','suborbital rocket','unknown',1284,'FLIGHT 179     /T 1-','','','','Aeron',1821,'',56 from dual union all
select 43747,'1977-W1863','1977-12-14 18:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 43748,'1977-W1864','1977-12-14 19:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual;
insert into LAUNCH
select 43749,'1977-W1865','1977-12-14 20:37:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 4375,'1957-A153','1957-04-25 00:00:00','military missile','success',845,'','Drone intercept','','','Test',1895,'',10 from dual union all
select 43750,'1977-W1866','1977-12-15 02:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43751,'1977-W1867','1977-12-15 15:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 43752,'1977-W1868','1977-12-15 16:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 43753,'1977-W1869','1977-12-15 16:56:00','suborbital rocket','unknown',1284,'FLIGHT 180     /T 1-','','','','Aeron',1821,'',67 from dual union all
select 43754,'1977-W187','1977-01-26 20:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 43755,'1977-W1870','1977-12-15 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 43756,'1977-W1871','1977-12-15 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 43757,'1977-W1872','1977-12-15 23:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 43758,'1977-W1873','1977-12-16 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43759,'1977-W1874','1977-12-16 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 4376,'1957-A154','1957-12-01 00:00:00','military missile','success',845,'','Liquid stage 67','','','Test',1895,'',10 from dual union all
select 43760,'1977-W1875','1977-12-16 17:03:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 43761,'1977-W1876','1977-12-16 18:00:00','suborbital rocket','unknown',1284,'FLIGHT 181     /TH1-','','','','Aeron',428,'',75 from dual union all
select 43762,'1977-W1877','1977-12-16 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 43763,'1977-W1878','1977-12-16 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 43764,'1977-W1879','1977-12-19 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 43765,'1977-W188','1977-01-26 21:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 43766,'1977-W1880','1977-12-19 17:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43767,'1977-W1881','1977-12-19 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 43768,'1977-W1882','1977-12-20 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 43769,'1977-W1883','1977-12-20 14:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 4377,'1957-A155','1957-12-01 00:00:00','military missile','success',843,'','XM30 No.35','','','Test',1895,'',10 from dual union all
select 43770,'1977-W1884','1977-12-20 16:12:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 43771,'1977-W1885','1977-12-21 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',971,'MUSSON',66 from dual union all
select 43772,'1977-W1886','1977-12-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',78 from dual union all
select 43773,'1977-W1887','1977-12-21 15:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 43774,'1977-W1888','1977-12-21 15:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 43775,'1977-W1889','1977-12-21 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 43776,'1977-W189','1977-01-26 21:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 43777,'1977-W1890','1977-12-21 17:48:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',53 from dual union all
select 43778,'1977-W1891','1977-12-21 18:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 43779,'1977-W1892','1977-12-21 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 4378,'1957-A157','1957-05-07 00:00:00','military missile','success',843,'','Contr. system  test','','','Test',1895,'',10 from dual union all
select 43780,'1977-W1893','1977-12-21 21:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 43781,'1977-W1894','1977-12-21 23:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 43782,'1977-W1895','1977-12-22 16:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 43783,'1977-W1896','1977-12-22 16:41:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 43784,'1977-W1897','1977-12-23 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43785,'1977-W1898','1977-12-23 14:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 43786,'1977-W1899','1977-12-23 16:29:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',51 from dual union all
select 43787,'1977-W19','1977-01-04 16:15:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 43788,'1977-W190','1977-01-27 02:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 43789,'1977-W1900','1977-12-23 17:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 4379,'1957-A158','1957-04-09 00:00:00','military missile','success',975,'M1-25','','','','Test',636,'',40 from dual union all
select 43790,'1977-W1901','1977-12-23 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 43791,'1977-W1902','1977-12-24 01:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 43792,'1977-W1903','1977-12-28 01:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43793,'1977-W1904','1977-12-28 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',971,'MUSSON',59 from dual union all
select 43794,'1977-W1905','1977-12-29 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 43795,'1977-W1906','1977-12-29 21:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 43796,'1977-W1907','1977-12-30 14:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 43797,'1977-W1908','1977-12-30 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 43798,'1977-W1909','1977-12-30 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 43799,'1977-W191','1977-01-27 15:14:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 438,'1944-S55','1944-08-17 17:35:00','military missile','success',1470,'18306','Ma124','','','Training',661,'',90 from dual union all
select 4380,'1957-A159','1957-04-11 00:00:00','military missile','success',975,'M1-22','','','','Test',636,'',40 from dual union all
select 43800,'1977-W1910','1977-08-24 03:00:00','suborbital rocket','success',1284,'US-USSR Intercomp','No. 21 Datasonde','','','Meteo',1821,'',75 from dual union all
select 43801,'1977-W1911','1977-08-24 03:08:00','suborbital rocket','success',728,'US-USSR Intercomp','No. 21','','','Meteo',1822,'KORO',84 from dual union all
select 43802,'1977-W1912','1977-07-18 06:31:00','suborbital rocket','success',728,'Intercomp prep','','','','Meteo',1260,'KORO',70 from dual union all
select 43803,'1977-W1913','1977-07-18 07:51:00','suborbital rocket','success',728,'Intercomp prep','','','','Meteo',1261,'KORO',70 from dual union all
select 43804,'1977-W1914','1977-07-21 20:30:00','suborbital rocket','success',728,'Intercomp prep','','','','Meteo',1262,'KORO',70 from dual union all
select 43805,'1977-W1915','1977-07-21 22:00:00','suborbital rocket','success',728,'Intercomp prep','','','','Meteo',1263,'KORO',70 from dual union all
select 43806,'1977-W1916','1977-07-21 23:30:00','suborbital rocket','success',728,'Intercomp prep','','','','Meteo',1264,'KORO',70 from dual union all
select 43807,'1977-W1917','1977-07-25 05:09:00','suborbital rocket','success',728,'Intercomp prep','','','','Meteo',1265,'KORO',70 from dual union all
select 43808,'1977-W1918','1977-07-25 06:40:00','suborbital rocket','success',728,'Intercomp prep','','','','Meteo',1266,'KORO',70 from dual union all
select 43809,'1977-W1919','1977-07-25 08:10:00','suborbital rocket','success',728,'Intercomp prep','','','','Meteo',1267,'KORO',70 from dual union all
select 4381,'1957-A16','1957-03-12 00:00:00','military missile','success',1214,'N-69D 53-8174','','','','Test',224,'',10 from dual union all
select 43810,'1977-W192','1977-01-27 15:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 43811,'1977-W1920','1977-07-27 03:04:00','suborbital rocket','success',728,'Intercomp prep','','','','Meteo',1268,'KORO',70 from dual union all
select 43812,'1977-W1921','1977-07-27 04:35:00','suborbital rocket','success',728,'Intercomp prep','','','','Meteo',1269,'KORO',70 from dual union all
select 43813,'1977-W1922','1977-06-01 00:00:00','suborbital rocket','success',728,'Intercomp prep','','[Dubious]','','Meteo',1270,'KORO',70 from dual union all
select 43814,'1977-W1923','1977-06-01 00:00:00','suborbital rocket','success',728,'Intercomp prep','','[Dubious]','','Meteo',1270,'KORO',70 from dual union all
select 43815,'1977-W193','1977-01-27 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43816,'1977-W194','1977-01-27 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 43817,'1977-W195','1977-01-27 18:00:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 43818,'1977-W196','1977-01-27 19:42:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 43819,'1977-W197','1977-01-27 20:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',75 from dual union all
select 4382,'1957-A160','1957-07-10 00:00:00','military missile','success',969,'0414','','','','OT',636,'',40 from dual union all
select 43820,'1977-W198','1977-01-27 21:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 43821,'1977-W199','1977-01-28 02:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 43822,'1977-W20','1977-01-04 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 43823,'1977-W200','1977-01-28 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 43824,'1977-W201','1977-01-28 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 43825,'1977-W202','1977-01-28 14:50:00','suborbital rocket','unknown',725,'','','','','Aeron',883,'KORO',77 from dual union all
select 43826,'1977-W203','1977-01-28 15:59:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',73 from dual union all
select 43827,'1977-W204','1977-01-28 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 43828,'1977-W205','1977-01-28 16:23:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 43829,'1977-W206','1977-01-28 16:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 4383,'1957-A161','1957-07-13 00:00:00','military missile','success',969,'5208413','','','','OT',636,'',40 from dual union all
select 43830,'1977-W207','1977-01-28 18:00:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 43831,'1977-W208','1977-01-28 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',73 from dual union all
select 43832,'1977-W209','1977-01-28 20:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 43833,'1977-W21','1977-01-04 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 43834,'1977-W210','1977-01-28 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',73 from dual union all
select 43835,'1977-W211','1977-01-28 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',57 from dual union all
select 43836,'1977-W212','1977-01-28 23:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43837,'1977-W213','1977-01-29 00:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',60 from dual union all
select 43838,'1977-W214','1977-01-29 15:44:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',64 from dual;
insert into LAUNCH
select 43839,'1977-W215','1977-01-29 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 4384,'1957-A162','1957-07-24 00:00:00','military missile','success',969,'53010116','','','','OT',636,'',40 from dual union all
select 43840,'1977-W216','1977-01-29 18:00:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 43841,'1977-W217','1977-01-29 22:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',50 from dual union all
select 43842,'1977-W218','1977-01-29 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43843,'1977-W219','1977-01-29 23:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',66 from dual union all
select 43844,'1977-W22','1977-01-04 17:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',60 from dual union all
select 43845,'1977-W220','1977-01-30 15:36:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 43846,'1977-W221','1977-01-30 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 43847,'1977-W222','1977-01-30 18:00:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 43848,'1977-W223','1977-01-30 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',53 from dual union all
select 43849,'1977-W224','1977-01-30 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 4385,'1957-A163','1957-12-11 00:00:00','military missile','success',971,'M6815','','','','Test',636,'',40 from dual union all
select 43850,'1977-W225','1977-02-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 43851,'1977-W226','1977-02-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 43852,'1977-W227','1977-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43853,'1977-W228','1977-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43854,'1977-W229','1977-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43855,'1977-W23','1977-01-04 20:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',53 from dual union all
select 43856,'1977-W230','1977-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43857,'1977-W231','1977-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43858,'1977-W232','1977-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43859,'1977-W233','1977-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 4386,'1957-A164','1957-12-11 00:00:00','military missile','success',971,'M6818','','','','Test',636,'',40 from dual union all
select 43860,'1977-W234','1977-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 43861,'1977-W235','1977-01-31 14:00:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',70 from dual union all
select 43862,'1977-W236','1977-01-31 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 43863,'1977-W237','1977-01-31 16:18:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 43864,'1977-W238','1977-01-31 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',53 from dual union all
select 43865,'1977-W239','1977-01-31 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 43866,'1977-W24','1977-01-04 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 43867,'1977-W240','1977-01-31 17:00:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 43868,'1977-W241','1977-01-31 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 43869,'1977-W242','1977-01-31 18:30:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 4387,'1957-A165','1957-12-12 00:00:00','military missile','success',971,'M6813','','','','Test',636,'',40 from dual union all
select 43870,'1977-W243','1977-01-31 19:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',58 from dual union all
select 43871,'1977-W244','1977-01-31 23:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 43872,'1977-W245','1977-02-01 16:26:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',64 from dual union all
select 43873,'1977-W246','1977-02-01 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 43874,'1977-W247','1977-02-01 18:59:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 43875,'1977-W248','1977-02-01 20:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',55 from dual union all
select 43876,'1977-W249','1977-02-01 21:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 43877,'1977-W25','1977-01-04 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',64 from dual union all
select 43878,'1977-W250','1977-02-01 22:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',67 from dual union all
select 43879,'1977-W251','1977-02-02 10:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',77 from dual union all
select 4388,'1957-A166','1957-01-11 00:00:00','military missile','success',936,'Thrust reversal test','','','','Test',1057,'',10 from dual union all
select 43880,'1977-W252','1977-02-02 12:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 43881,'1977-W253','1977-02-02 13:44:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',77 from dual union all
select 43882,'1977-W254','1977-02-02 14:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 43883,'1977-W255','1977-02-02 16:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 43884,'1977-W256','1977-02-02 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 43885,'1977-W257','1977-02-02 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 43886,'1977-W258','1977-02-02 17:02:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 43887,'1977-W259','1977-02-02 22:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',73 from dual union all
select 43888,'1977-W26','1977-01-05 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1297,'PRILIV',59 from dual union all
select 43889,'1977-W260','1977-02-02 23:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 4389,'1957-A167','1957-01-29 00:00:00','military missile','success',936,'Thrust reversal test','','','','Test',1057,'',10 from dual union all
select 43890,'1977-W261','1977-02-03 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 43891,'1977-W262','1977-02-03 03:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 43892,'1977-W263','1977-02-03 15:26:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 43893,'1977-W264','1977-02-03 16:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 43894,'1977-W265','1977-02-03 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 43895,'1977-W266','1977-02-03 17:00:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 43896,'1977-W267','1977-02-03 19:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',57 from dual union all
select 43897,'1977-W268','1977-02-03 21:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',69 from dual union all
select 43898,'1977-W269','1977-02-04 00:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 43899,'1977-W27','1977-01-05 10:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 439,'1944-S56','1944-08-17 20:45:00','military missile','success',8,'18325','Ma107','','','Training',661,'',90 from dual union all
select 4390,'1957-A168','1957-02-07 00:00:00','military missile','success',936,'Thrust reversal test','','','','Test',1057,'',10 from dual union all
select 43900,'1977-W270','1977-02-04 01:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43901,'1977-W271','1977-02-04 12:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 43902,'1977-W272','1977-02-04 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 43903,'1977-W273','1977-02-04 15:45:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 43904,'1977-W274','1977-02-04 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 43905,'1977-W275','1977-02-04 16:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 43906,'1977-W276','1977-02-04 16:49:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',75 from dual union all
select 43907,'1977-W277','1977-02-04 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 43908,'1977-W278','1977-02-04 19:13:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 43909,'1977-W279','1977-02-04 20:46:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',68 from dual union all
select 4391,'1957-A169','1957-02-14 00:00:00','military missile','success',936,'Thrust reversal test','','','','Test',1057,'',10 from dual union all
select 43910,'1977-W28','1977-01-05 11:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 43911,'1977-W280','1977-02-04 21:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 43912,'1977-W281','1977-02-04 21:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 43913,'1977-W282','1977-02-04 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',63 from dual union all
select 43914,'1977-W283','1977-02-04 23:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 43915,'1977-W284','1977-02-05 00:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 43916,'1977-W285','1977-02-05 23:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 43917,'1977-W286','1977-02-06 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',899,'KRENKEL''',60 from dual union all
select 43918,'1977-W287','1977-02-06 23:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 43919,'1977-W288','1977-02-07 12:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 4392,'1957-A17','1957-03-13 00:00:00','military missile','success',262,'Dummy','','','','Test',243,'',1 from dual union all
select 43920,'1977-W289','1977-02-07 15:15:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',75 from dual union all
select 43921,'1977-W29','1977-01-05 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 43922,'1977-W290','1977-02-07 15:30:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',76 from dual union all
select 43923,'1977-W291','1977-02-07 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 43924,'1977-W292','1977-02-07 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43925,'1977-W293','1977-02-07 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 43926,'1977-W294','1977-02-07 18:14:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 43927,'1977-W295','1977-02-07 19:07:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',60 from dual union all
select 43928,'1977-W296','1977-02-07 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual;
insert into LAUNCH
select 43929,'1977-W297','1977-02-07 21:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',68 from dual union all
select 4393,'1957-A170','1957-03-03 00:00:00','military missile','success',936,'Thrust reversal test','','','','Test',1057,'',10 from dual union all
select 43930,'1977-W298','1977-02-07 23:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 43931,'1977-W299','1977-02-08 14:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 43932,'1977-W30','1977-01-05 14:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 43933,'1977-W300','1977-02-08 15:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 43934,'1977-W301','1977-02-08 15:41:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 43935,'1977-W302','1977-02-08 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 43936,'1977-W303','1977-02-08 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 43937,'1977-W304','1977-02-08 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 43938,'1977-W305','1977-02-08 18:10:00','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 43939,'1977-W306','1977-02-08 18:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',53 from dual union all
select 4394,'1957-A171','1957-10-11 00:00:00','military missile','success',843,'','Contr. system  test','','','Test',1895,'',10 from dual union all
select 43940,'1977-W307','1977-02-08 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',70 from dual union all
select 43941,'1977-W308','1977-02-08 23:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 43942,'1977-W309','1977-02-08 23:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',58 from dual union all
select 43943,'1977-W31','1977-01-05 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 43944,'1977-W310','1977-02-09 14:15:00','suborbital rocket','success',1061,'','','','','Meteo',176,'',63 from dual union all
select 43945,'1977-W311','1977-02-09 15:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 43946,'1977-W312','1977-02-09 15:59:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',77 from dual union all
select 43947,'1977-W313','1977-02-09 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',79 from dual union all
select 43948,'1977-W314','1977-02-09 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 43949,'1977-W315','1977-02-09 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 4395,'1957-A18','1957-03-13 00:00:00','military missile','unknown',843,'B57','XM30 No.1','','','Test',1895,'',10 from dual union all
select 43950,'1977-W316','1977-02-09 17:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 43951,'1977-W317','1977-02-09 18:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 43952,'1977-W318','1977-02-09 19:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 43953,'1977-W319','1977-02-09 19:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 43954,'1977-W32','1977-01-05 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 43955,'1977-W320','1977-02-09 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',75 from dual union all
select 43956,'1977-W321','1977-02-09 21:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',70 from dual union all
select 43957,'1977-W322','1977-02-09 22:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 43958,'1977-W323','1977-02-10 15:58:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 43959,'1977-W324','1977-02-10 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 4396,'1957-A19','1957-03-13 00:00:00','military missile','success',1214,'N-69D Inertial 1','','','','Test',224,'',10 from dual union all
select 43960,'1977-W325','1977-02-10 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 43961,'1977-W326','1977-02-10 18:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',56 from dual union all
select 43962,'1977-W327','1977-02-10 18:59:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 43963,'1977-W328','1977-02-10 21:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',70 from dual union all
select 43964,'1977-W329','1977-02-11 01:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',61 from dual union all
select 43965,'1977-W33','1977-01-05 15:00:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',53 from dual union all
select 43966,'1977-W330','1977-02-11 12:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 43967,'1977-W331','1977-02-11 14:30:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',74 from dual union all
select 43968,'1977-W332','1977-02-11 15:33:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 43969,'1977-W333','1977-02-11 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',63 from dual union all
select 4397,'1957-A20','1957-03-18 00:00:00','atmospheric rocket','success',552,'','Reentry blackout','','','Reentry Test',1821,'',30 from dual union all
select 43970,'1977-W334','1977-02-11 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 43971,'1977-W335','1977-02-11 18:59:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 43972,'1977-W336','1977-02-11 19:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 43973,'1977-W337','1977-02-11 20:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 43974,'1977-W338','1977-02-11 22:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',60 from dual union all
select 43975,'1977-W339','1977-02-12 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',64 from dual union all
select 43976,'1977-W34','1977-01-05 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 43977,'1977-W340','1977-02-12 23:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 43978,'1977-W341','1977-02-13 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',899,'KRENKEL''',61 from dual union all
select 43979,'1977-W342','1977-02-13 23:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 4398,'1957-A21','1957-03-22 00:00:00','military missile','failure',832,'06','G-26 No. 2','','','Test',281,'',9 from dual union all
select 43980,'1977-W343','1977-02-14 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 43981,'1977-W344','1977-02-14 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 43982,'1977-W345','1977-02-14 16:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 43983,'1977-W346','1977-02-14 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 43984,'1977-W347','1977-02-14 17:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',51 from dual union all
select 43985,'1977-W348','1977-02-14 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',66 from dual union all
select 43986,'1977-W349','1977-02-14 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 43987,'1977-W35','1977-01-05 16:09:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',50 from dual union all
select 43988,'1977-W350','1977-02-14 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',62 from dual union all
select 43989,'1977-W351','1977-02-14 21:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 4399,'1957-A22','1957-04-10 00:00:00','atmospheric rocket','success',545,'','Heat Transfer','','','Reentry Test',1821,'',30 from dual union all
select 43990,'1977-W352','1977-02-15 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 43991,'1977-W353','1977-02-15 18:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',57 from dual union all
select 43992,'1977-W354','1977-02-15 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',57 from dual union all
select 43993,'1977-W355','1977-02-16 00:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 43994,'1977-W356','1977-02-16 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 43995,'1977-W357','1977-02-16 14:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 43996,'1977-W358','1977-02-16 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 43997,'1977-W359','1977-02-16 15:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 43998,'1977-W36','1977-01-05 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 43999,'1977-W360','1977-02-16 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 44,'1944-A08','1944-08-14 07:17:00','military missile','failure',1470,'18307','Ma125','','','Training',661,'',40 from dual union all
select 440,'1944-S57','1944-08-18 07:46:00','military missile','success',8,'18326','Ma146','','','Training',661,'',90 from dual union all
select 4400,'1957-A23','1957-04-10 00:00:00','military missile','success',771,'','','','','Test',222,'',12 from dual union all
select 44000,'1977-W361','1977-02-16 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 44001,'1977-W362','1977-02-16 17:32:00','suborbital rocket','unknown',1284,'T 1-8415','','','','Aeron',1821,'',73 from dual union all
select 44002,'1977-W363','1977-02-16 18:07:00','suborbital rocket','unknown',1284,'TH1-8708','','','','Aeron',428,'',73 from dual union all
select 44003,'1977-W364','1977-02-16 18:12:00','suborbital rocket','unknown',1284,'T 1-8418','','','','Aeron',1821,'',51 from dual union all
select 44004,'1977-W365','1977-02-16 18:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44005,'1977-W366','1977-02-16 19:44:59','suborbital rocket','success',1284,'','','','','Meteo',1895,'',60 from dual union all
select 44006,'1977-W367','1977-02-16 21:20:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',56 from dual union all
select 44007,'1977-W368','1977-02-16 21:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 44008,'1977-W369','1977-02-17 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',64 from dual union all
select 44009,'1977-W37','1977-01-05 17:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 4401,'1957-A24','1957-04-11 00:00:00','military missile','success',771,'','','','','Test',222,'',12 from dual union all
select 44010,'1977-W370','1977-02-17 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 44011,'1977-W371','1977-02-17 20:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',54 from dual union all
select 44012,'1977-W372','1977-02-17 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 44013,'1977-W373','1977-02-17 21:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 44014,'1977-W374','1977-02-17 23:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 44015,'1977-W375','1977-02-18 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 44016,'1977-W376','1977-02-18 15:30:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',68 from dual union all
select 44017,'1977-W377','1977-02-18 16:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual;
insert into LAUNCH
select 44018,'1977-W378','1977-02-18 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 44019,'1977-W379','1977-02-18 19:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 4402,'1957-A25','1957-04-15 00:00:00','military missile','success',1214,'N-69D Inertial 2','','','','Test',224,'',10 from dual union all
select 44020,'1977-W38','1977-01-05 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',55 from dual union all
select 44021,'1977-W380','1977-02-21 14:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 44022,'1977-W381','1977-02-21 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 44023,'1977-W382','1977-02-22 12:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 44024,'1977-W383','1977-02-22 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 44025,'1977-W384','1977-02-22 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',53 from dual union all
select 44026,'1977-W385','1977-02-22 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 44027,'1977-W386','1977-02-22 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',55 from dual union all
select 44028,'1977-W387','1977-02-22 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',66 from dual union all
select 44029,'1977-W388','1977-02-22 21:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 4403,'1957-A26','1957-04-16 00:00:00','military missile','success',1214,'N-69D 53-8176','','','','Test',224,'',10 from dual union all
select 44030,'1977-W389','1977-02-23 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 44031,'1977-W39','1977-01-05 19:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 44032,'1977-W390','1977-02-23 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 44033,'1977-W391','1977-02-23 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 44034,'1977-W392','1977-02-23 15:30:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',70 from dual union all
select 44035,'1977-W393','1977-02-23 17:00:00','suborbital rocket','unknown',1284,'FLIGHT 148     /T 1-','','','','Aeron',1821,'',71 from dual union all
select 44036,'1977-W394','1977-02-23 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 44037,'1977-W395','1977-02-23 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 44038,'1977-W396','1977-02-23 18:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',60 from dual union all
select 44039,'1977-W397','1977-02-23 20:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',55 from dual union all
select 4404,'1957-A27','1957-04-17 00:00:00','military missile','success',255,'624-1','','','','OT',272,'',20 from dual union all
select 44040,'1977-W398','1977-02-23 21:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',74 from dual union all
select 44041,'1977-W399','1977-02-23 22:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 44042,'1977-W40','1977-01-05 21:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 44043,'1977-W400','1977-02-24 01:30:00','suborbital rocket','success',761,'MT135P T18','','','','Aeron?',1476,'',50 from dual union all
select 44044,'1977-W401','1977-02-24 02:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',67 from dual union all
select 44045,'1977-W402','1977-02-24 03:07:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 44046,'1977-W403','1977-02-24 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 44047,'1977-W404','1977-02-24 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 44048,'1977-W405','1977-02-24 20:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 44049,'1977-W406','1977-02-24 22:07:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 4405,'1957-A28','1957-04-20 04:31:00','military missile','failure',1366,'102','R&D Series I','','','Test',234,'',0 from dual union all
select 44050,'1977-W407','1977-02-25 12:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 44051,'1977-W408','1977-02-25 13:44:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',72 from dual union all
select 44052,'1977-W409','1977-02-25 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 44053,'1977-W41','1977-01-06 02:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',58 from dual union all
select 44054,'1977-W410','1977-02-25 18:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 44055,'1977-W411','1977-02-25 18:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',56 from dual union all
select 44056,'1977-W412','1977-02-25 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 44057,'1977-W413','1977-02-25 20:27:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 44058,'1977-W414','1977-02-27 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',899,'KRENKEL''',60 from dual union all
select 44059,'1977-W415','1977-03-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 4406,'1957-A29','1957-04-25 00:00:00','military missile','failure',832,'07','G-26 No. 5','','','Test',281,'',0 from dual union all
select 44060,'1977-W416','1977-03-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 44061,'1977-W417','1977-03-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 44062,'1977-W418','1977-03-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',75 from dual union all
select 44063,'1977-W419','1977-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 44064,'1977-W42','1977-01-06 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 44065,'1977-W420','1977-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 44066,'1977-W421','1977-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 44067,'1977-W422','1977-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 44068,'1977-W423','1977-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 44069,'1977-W424','1977-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 4407,'1957-A30','1957-04-26 20:12:00','military missile','failure',638,'AM-1B','R&D','','','Test',279,'',18 from dual union all
select 44070,'1977-W425','1977-02-28 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 44071,'1977-W426','1977-02-28 14:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 44072,'1977-W427','1977-02-28 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 44073,'1977-W428','1977-02-28 16:55:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',65 from dual union all
select 44074,'1977-W429','1977-02-28 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 44075,'1977-W43','1977-01-06 15:24:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',71 from dual union all
select 44076,'1977-W430','1977-02-28 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 44077,'1977-W431','1977-02-28 23:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 44078,'1977-W432','1977-02-28 23:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 44079,'1977-W433','1977-03-01 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 4408,'1957-A31','1957-05-01 00:00:00','suborbital rocket','success',122,'','Plumbob test','','','Test',1895,'',30 from dual union all
select 44080,'1977-W434','1977-03-02 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',899,'KRENKEL''',62 from dual union all
select 44081,'1977-W435','1977-03-02 03:26:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 44082,'1977-W436','1977-03-02 12:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 44083,'1977-W437','1977-03-02 13:44:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',74 from dual union all
select 44084,'1977-W438','1977-03-02 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 44085,'1977-W439','1977-03-02 14:45:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',74 from dual union all
select 44086,'1977-W44','1977-01-06 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 44087,'1977-W440','1977-03-02 15:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 44088,'1977-W441','1977-03-02 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 44089,'1977-W442','1977-03-02 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 4409,'1957-A32','1957-05-01 00:00:00','suborbital rocket','failure',122,'','Plumbob test','','','Test',1895,'',0 from dual union all
select 44090,'1977-W443','1977-03-02 18:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 44091,'1977-W444','1977-03-02 19:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',76 from dual union all
select 44092,'1977-W445','1977-03-02 20:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',66 from dual union all
select 44093,'1977-W446','1977-03-02 20:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 44094,'1977-W447','1977-03-02 21:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 44095,'1977-W448','1977-03-02 21:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 44096,'1977-W449','1977-03-02 22:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 44097,'1977-W45','1977-01-06 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 44098,'1977-W450','1977-03-02 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 44099,'1977-W451','1977-03-03 15:30:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',65 from dual union all
select 441,'1944-S58','1944-08-18 12:59:00','military missile','success',8,'18345','Ma150','','','Training',661,'',90 from dual union all
select 4410,'1957-A33','1957-04-30 00:00:00','suborbital rocket','success',658,'K-2-1','','','','Test',7,'',30 from dual union all
select 44100,'1977-W452','1977-03-03 23:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',60 from dual union all
select 44101,'1977-W453','1977-03-04 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',899,'KRENKEL''',62 from dual union all
select 44102,'1977-W454','1977-03-04 02:26:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',53 from dual union all
select 44103,'1977-W455','1977-03-04 12:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 44104,'1977-W456','1977-03-04 14:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 44105,'1977-W457','1977-03-04 16:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 44106,'1977-W458','1977-03-04 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 44107,'1977-W459','1977-03-04 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual;
insert into LAUNCH
select 44108,'1977-W46','1977-01-07 08:40:00','suborbital rocket','unknown',725,'','','','','Aeron',883,'KORO',78 from dual union all
select 44109,'1977-W460','1977-03-04 18:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',66 from dual union all
select 4411,'1957-A34','1957-05-02 00:00:00','suborbital rocket','success',659,'K-3-1','','','','Test',7,'',30 from dual union all
select 44110,'1977-W461','1977-03-04 19:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 44111,'1977-W462','1977-03-05 04:47:00','suborbital rocket','success',229,'AKF-VI-09','Auroral X-rays','','','Auroral',428,'',60 from dual union all
select 44112,'1977-W463','1977-03-07 12:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 44113,'1977-W464','1977-03-07 14:12:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',73 from dual union all
select 44114,'1977-W465','1977-03-07 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',56 from dual union all
select 44115,'1977-W466','1977-03-07 16:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 44116,'1977-W467','1977-03-07 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 44117,'1977-W468','1977-03-07 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 44118,'1977-W469','1977-03-07 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 44119,'1977-W47','1977-01-07 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 4412,'1957-A35','1957-05-02 00:00:00','atmospheric rocket','success',835,'','Slender missile','','','Test',1821,'',10 from dual union all
select 44120,'1977-W470','1977-03-07 19:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 44121,'1977-W471','1977-03-08 00:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 44122,'1977-W472','1977-03-08 04:06:00','suborbital rocket','success',229,'AKF-VI-08','Auroral X-rays','','','Auroral',428,'',60 from dual union all
select 44123,'1977-W473','1977-03-08 15:49:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 44124,'1977-W474','1977-03-08 18:04:59','suborbital rocket','success',1284,'','','','','Meteo',1895,'',60 from dual union all
select 44125,'1977-W475','1977-03-09 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',899,'KRENKEL''',64 from dual union all
select 44126,'1977-W476','1977-03-09 10:32:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 44127,'1977-W477','1977-03-09 12:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 44128,'1977-W478','1977-03-09 13:14:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',58 from dual union all
select 44129,'1977-W479','1977-03-09 15:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 4413,'1957-A36','1957-05-03 00:00:00','military missile','failure',1214,'N-69D 53-8178','','','','Test',224,'',10 from dual union all
select 44130,'1977-W48','1977-01-07 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 44131,'1977-W480','1977-03-09 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 44132,'1977-W481','1977-03-09 17:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 44133,'1977-W482','1977-03-09 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 44134,'1977-W483','1977-03-09 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 44135,'1977-W484','1977-03-09 19:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 44136,'1977-W485','1977-03-09 19:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 44137,'1977-W486','1977-03-10 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 44138,'1977-W487','1977-03-10 06:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 44139,'1977-W488','1977-03-10 23:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 4414,'1957-A37','1957-05-13 00:00:00','military missile','success',262,'Dummy','','','','Test',243,'',1 from dual union all
select 44140,'1977-W489','1977-03-11 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',899,'KRENKEL''',61 from dual union all
select 44141,'1977-W49','1977-01-07 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 44142,'1977-W490','1977-03-11 12:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 44143,'1977-W491','1977-03-11 15:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 44144,'1977-W492','1977-03-11 16:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 44145,'1977-W493','1977-03-11 16:14:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',52 from dual union all
select 44146,'1977-W494','1977-03-11 16:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 44147,'1977-W495','1977-03-11 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 44148,'1977-W496','1977-03-11 16:49:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',71 from dual union all
select 44149,'1977-W497','1977-03-11 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 4415,'1957-A39','1957-05-16 00:00:00','military missile','success',771,'','','','','Test',222,'',12 from dual union all
select 44150,'1977-W498','1977-03-11 17:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 44151,'1977-W499','1977-03-11 20:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',68 from dual union all
select 44152,'1977-W50','1977-01-07 20:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',67 from dual union all
select 44153,'1977-W500','1977-03-12 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',884,'KORO',78 from dual union all
select 44154,'1977-W501','1977-03-14 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 44155,'1977-W502','1977-03-14 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',73 from dual union all
select 44156,'1977-W503','1977-03-14 15:20:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',73 from dual union all
select 44157,'1977-W504','1977-03-14 16:43:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 44158,'1977-W505','1977-03-14 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',57 from dual union all
select 44159,'1977-W506','1977-03-14 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 4416,'1957-A40','1957-05-17 00:00:00','military missile','success',771,'','','','','Test',222,'',12 from dual union all
select 44160,'1977-W507','1977-03-14 19:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 44161,'1977-W508','1977-03-14 20:25:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 44162,'1977-W509','1977-03-15 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 44163,'1977-W51','1977-01-07 21:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 44164,'1977-W510','1977-03-15 17:46:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 44165,'1977-W511','1977-03-16 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 44166,'1977-W512','1977-03-16 12:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 44167,'1977-W513','1977-03-16 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 44168,'1977-W514','1977-03-16 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 44169,'1977-W515','1977-03-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',78 from dual union all
select 4417,'1957-A41','1957-05-22 05:00:00','military missile','destroyed before launch',1366,'103','R&D Series I (pad ex)','','','Test',234,'',0 from dual union all
select 44170,'1977-W516','1977-03-16 15:20:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',73 from dual union all
select 44171,'1977-W517','1977-03-16 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',60 from dual union all
select 44172,'1977-W518','1977-03-16 16:12:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 44173,'1977-W519','1977-03-16 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 44174,'1977-W52','1977-01-07 21:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 44175,'1977-W520','1977-03-16 17:00:00','suborbital rocket','unknown',1284,'T 1-8416','','','','Aeron',1821,'',71 from dual union all
select 44176,'1977-W521','1977-03-16 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 44177,'1977-W522','1977-03-16 18:19:00','suborbital rocket','unknown',1284,'FLIGHT 150     /TH1-','','','','Aeron',428,'',73 from dual union all
select 44178,'1977-W523','1977-03-16 18:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 44179,'1977-W524','1977-03-16 23:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 4418,'1957-A42','1957-05-23 00:00:00','atmospheric rocket','success',442,'','MW-2 Model/T-40','','','Test',1821,'',10 from dual union all
select 44180,'1977-W525','1977-03-17 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',899,'KRENKEL''',60 from dual union all
select 44181,'1977-W526','1977-03-17 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1758,'VOLNA',60 from dual union all
select 44182,'1977-W527','1977-03-17 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',54 from dual union all
select 44183,'1977-W528','1977-03-17 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 44184,'1977-W529','1977-03-17 23:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 44185,'1977-W53','1977-01-08 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 44186,'1977-W530','1977-03-18 02:40:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 44187,'1977-W531','1977-03-18 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1758,'VOLNA',61 from dual union all
select 44188,'1977-W532','1977-03-18 12:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 44189,'1977-W533','1977-03-18 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 4419,'1957-A43','1957-05-27 00:00:00','military missile','failure',1080,'F77','SEPR 732 booster','','','Test',671,'',20 from dual union all
select 44190,'1977-W534','1977-03-18 15:14:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',74 from dual union all
select 44191,'1977-W535','1977-03-18 15:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 44192,'1977-W536','1977-03-18 16:08:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 44193,'1977-W537','1977-03-18 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 44194,'1977-W538','1977-03-18 19:07:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 44195,'1977-W539','1977-03-18 20:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 44196,'1977-W54','1977-01-08 17:54:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',64 from dual union all
select 44197,'1977-W540','1977-03-19 02:48:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 44198,'1977-W541','1977-03-21 14:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual;
insert into LAUNCH
select 44199,'1977-W542','1977-03-21 15:00:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',51 from dual union all
select 442,'1944-S59','1944-08-18 16:45:00','military missile','success',8,'18336','Ma121','','','Training',661,'',90 from dual union all
select 4420,'1957-A44','1957-05-28 00:00:00','military missile','success',1214,'N-69D 53-8177','Last Northrop launch','','','Test',224,'',10 from dual union all
select 44200,'1977-W543','1977-03-21 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 44201,'1977-W544','1977-03-21 19:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',56 from dual union all
select 44202,'1977-W545','1977-03-22 12:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 44203,'1977-W546','1977-03-22 16:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 44204,'1977-W547','1977-03-22 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',76 from dual union all
select 44205,'1977-W548','1977-03-22 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 44206,'1977-W549','1977-03-23 00:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 44207,'1977-W55','1977-01-08 21:32:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',67 from dual union all
select 44208,'1977-W550','1977-03-23 04:00:00','suborbital rocket','unknown',738,'','','','','Aeron',899,'KRENKEL''',64 from dual union all
select 44209,'1977-W551','1977-03-23 11:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1758,'VOLNA',62 from dual union all
select 4421,'1957-A45','1957-06-01 00:00:00','suborbital rocket','success',122,'','Plumbob test','','','Test',1895,'',30 from dual union all
select 44210,'1977-W552','1977-03-23 12:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 44211,'1977-W553','1977-03-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',75 from dual union all
select 44212,'1977-W554','1977-03-23 15:10:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',70 from dual union all
select 44213,'1977-W555','1977-03-23 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 44214,'1977-W556','1977-03-23 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 44215,'1977-W557','1977-03-23 16:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 44216,'1977-W558','1977-03-23 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 44217,'1977-W559','1977-03-23 17:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44218,'1977-W56','1977-01-08 21:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 44219,'1977-W560','1977-03-23 19:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 4422,'1957-A46','1957-06-01 00:00:00','suborbital rocket','success',122,'','Plumbob test','','','Test',1895,'',30 from dual union all
select 44220,'1977-W561','1977-03-23 19:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 44221,'1977-W562','1977-03-23 21:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 44222,'1977-W563','1977-03-24 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 44223,'1977-W564','1977-03-24 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 44224,'1977-W565','1977-03-25 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',899,'KRENKEL''',62 from dual union all
select 44225,'1977-W566','1977-03-25 02:00:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',69 from dual union all
select 44226,'1977-W567','1977-03-25 03:00:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',67 from dual union all
select 44227,'1977-W568','1977-03-25 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1758,'VOLNA',57 from dual union all
select 44228,'1977-W569','1977-03-25 12:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 44229,'1977-W57','1977-01-10 12:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 4423,'1957-A47','1957-06-01 00:00:00','suborbital rocket','success',122,'','Plumbob test','','','Test',998,'',30 from dual union all
select 44230,'1977-W570','1977-03-25 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 44231,'1977-W571','1977-03-25 15:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 44232,'1977-W572','1977-03-25 16:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 44233,'1977-W573','1977-03-25 16:20:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 44234,'1977-W574','1977-03-25 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 44235,'1977-W575','1977-03-25 23:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',75 from dual union all
select 44236,'1977-W576','1977-03-26 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 44237,'1977-W577','1977-03-28 13:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 44238,'1977-W578','1977-03-28 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 44239,'1977-W579','1977-03-28 14:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 4424,'1957-A48','1957-06-01 00:00:00','suborbital rocket','success',122,'','Plumbob test','','','Test',1490,'',30 from dual union all
select 44240,'1977-W58','1977-01-10 14:37:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',72 from dual union all
select 44241,'1977-W580','1977-03-28 15:57:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 44242,'1977-W581','1977-03-28 16:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 44243,'1977-W582','1977-03-28 17:17:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',73 from dual union all
select 44244,'1977-W583','1977-03-28 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 44245,'1977-W584','1977-03-28 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 44246,'1977-W585','1977-03-28 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',59 from dual union all
select 44247,'1977-W586','1977-03-28 19:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 44248,'1977-W587','1977-03-29 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 44249,'1977-W588','1977-03-29 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 4425,'1957-A49','1957-06-08 00:00:00','military missile','success',1080,'F78','','','','Test',671,'',20 from dual union all
select 44250,'1977-W589','1977-03-29 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 44251,'1977-W59','1977-01-10 16:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 44252,'1977-W590','1977-03-29 23:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 44253,'1977-W591','1977-03-30 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1758,'VOLNA',59 from dual union all
select 44254,'1977-W592','1977-03-30 12:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 44255,'1977-W593','1977-03-30 14:10:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',73 from dual union all
select 44256,'1977-W594','1977-03-30 14:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 44257,'1977-W595','1977-03-30 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 44258,'1977-W596','1977-03-30 16:18:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 44259,'1977-W597','1977-03-30 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 4426,'1957-A50','1957-06-11 19:37:00','military missile','failure',142,'4A','R&D Fairing','R&D','','Test',229,'',3 from dual union all
select 44260,'1977-W598','1977-03-30 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44261,'1977-W599','1977-03-30 18:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 44262,'1977-W60','1977-01-10 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 44263,'1977-W600','1977-03-30 19:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 44264,'1977-W601','1977-03-30 20:19:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 44265,'1977-W602','1977-03-31 04:00:00','suborbital rocket','unknown',738,'','','','','Aeron',899,'KRENKEL''',59 from dual union all
select 44266,'1977-W603','1977-03-31 06:00:00','suborbital rocket','unknown',738,'','','','','Aeron',899,'KRENKEL''',61 from dual union all
select 44267,'1977-W604','1977-03-31 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 44268,'1977-W605','1977-03-31 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 44269,'1977-W606','1977-04-01 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 4427,'1957-A51','1957-06-19 00:00:00','atmospheric rocket','success',552,'','Copper RV','','','Reentry Test',1821,'',30 from dual union all
select 44270,'1977-W607','1977-04-01 14:40:00','suborbital rocket','unknown',1284,'T 1-8722','','','','Aeron',1821,'',72 from dual union all
select 44271,'1977-W608','1977-04-01 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 44272,'1977-W609','1977-04-01 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 44273,'1977-W61','1977-01-10 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 44274,'1977-W610','1977-04-01 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 44275,'1977-W611','1977-04-01 16:09:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',76 from dual union all
select 44276,'1977-W612','1977-04-01 16:38:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 44277,'1977-W613','1977-04-01 18:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 44278,'1977-W614','1977-04-01 19:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 44279,'1977-W615','1977-04-01 20:26:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 4428,'1957-A52','1957-06-19 00:00:00','atmospheric rocket','success',835,'','F4H-1 Model','','','Test',1821,'',10 from dual union all
select 44280,'1977-W616','1977-04-01 22:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 44281,'1977-W617','1977-04-02 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 44282,'1977-W618','1977-04-03 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',899,'KRENKEL''',62 from dual union all
select 44283,'1977-W619','1977-04-04 04:00:00','suborbital rocket','unknown',738,'','','','','Aeron',899,'KRENKEL''',60 from dual union all
select 44284,'1977-W62','1977-01-10 19:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 44285,'1977-W620','1977-04-04 12:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 44286,'1977-W621','1977-04-04 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 44287,'1977-W622','1977-04-04 14:45:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',72 from dual union all
select 44288,'1977-W623','1977-04-04 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual;
insert into LAUNCH
select 44289,'1977-W624','1977-04-04 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 4429,'1957-A53','1957-06-20 00:00:00','military missile','failure',1214,'N-69E N-3321','','','','Test',223,'',0 from dual union all
select 44290,'1977-W625','1977-04-04 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 44291,'1977-W626','1977-04-04 17:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 44292,'1977-W627','1977-04-04 18:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',57 from dual union all
select 44293,'1977-W628','1977-04-04 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 44294,'1977-W629','1977-04-05 23:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 44295,'1977-W63','1977-01-10 20:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',60 from dual union all
select 44296,'1977-W630','1977-04-06 12:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 44297,'1977-W631','1977-04-06 14:53:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',65 from dual union all
select 44298,'1977-W632','1977-04-06 14:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 44299,'1977-W633','1977-04-06 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 443,'1944-S60','1944-08-18 19:29:00','military missile','success',8,'18339','Ma157','','','Training',661,'',90 from dual union all
select 4430,'1957-A54','1957-06-22 00:00:00','suborbital rocket','success',659,'K-3-2','','','','Test',7,'',30 from dual union all
select 44300,'1977-W634','1977-04-06 16:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 44301,'1977-W635','1977-04-06 16:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 44302,'1977-W636','1977-04-06 17:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 44303,'1977-W637','1977-04-06 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 44304,'1977-W638','1977-04-06 20:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44305,'1977-W639','1977-04-06 21:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 44306,'1977-W64','1977-01-11 00:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 44307,'1977-W640','1977-04-06 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 44308,'1977-W641','1977-04-07 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 44309,'1977-W642','1977-04-07 21:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 4431,'1957-A55','1957-07-26 00:00:00','suborbital rocket','success',659,'K-3-3','','','','Test',7,'',30 from dual union all
select 44310,'1977-W643','1977-04-08 14:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 44311,'1977-W644','1977-04-08 14:40:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',67 from dual union all
select 44312,'1977-W645','1977-04-08 15:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',59 from dual union all
select 44313,'1977-W646','1977-04-08 16:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 44314,'1977-W647','1977-04-08 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 44315,'1977-W648','1977-04-08 21:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 44316,'1977-W649','1977-04-11 12:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 44317,'1977-W65','1977-01-11 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 44318,'1977-W650','1977-04-11 14:45:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',71 from dual union all
select 44319,'1977-W651','1977-04-11 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 4432,'1957-A56','1957-06-26 18:00:00','military missile','failure',832,'08','G-26 No. 3','','','Test',281,'',12 from dual union all
select 44320,'1977-W652','1977-04-11 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 44321,'1977-W653','1977-04-11 18:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 44322,'1977-W654','1977-04-11 18:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',58 from dual union all
select 44323,'1977-W655','1977-04-12 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 44324,'1977-W656','1977-04-12 23:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 44325,'1977-W657','1977-04-13 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1633,'USHAKOV',61 from dual union all
select 44326,'1977-W658','1977-04-13 12:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 44327,'1977-W659','1977-04-13 14:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 44328,'1977-W66','1977-01-11 16:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',57 from dual union all
select 44329,'1977-W660','1977-04-13 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 4433,'1957-A57','1957-06-27 18:35:00','military missile','success',262,'2','','','','Test',243,'',6 from dual union all
select 44330,'1977-W661','1977-04-13 16:06:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 44331,'1977-W662','1977-04-13 16:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 44332,'1977-W663','1977-04-13 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 44333,'1977-W664','1977-04-13 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 44334,'1977-W665','1977-04-13 17:30:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',72 from dual union all
select 44335,'1977-W666','1977-04-13 18:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44336,'1977-W667','1977-04-13 19:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 44337,'1977-W668','1977-04-13 20:09:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',69 from dual union all
select 44338,'1977-W669','1977-04-13 20:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',72 from dual union all
select 44339,'1977-W67','1977-01-11 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 4434,'1957-A58','1957-07-01 00:00:00','suborbital rocket','success',122,'','Plumbob test','','','Test',963,'',30 from dual union all
select 44340,'1977-W670','1977-04-14 00:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 44341,'1977-W671','1977-04-14 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',77 from dual union all
select 44342,'1977-W672','1977-04-14 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1758,'VOLNA',60 from dual union all
select 44343,'1977-W673','1977-04-14 12:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 44344,'1977-W674','1977-04-14 15:14:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',71 from dual union all
select 44345,'1977-W675','1977-04-15 14:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 44346,'1977-W676','1977-04-15 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 44347,'1977-W677','1977-04-15 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 44348,'1977-W678','1977-04-15 16:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 44349,'1977-W679','1977-04-15 16:27:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 4435,'1957-A59','1957-07-01 00:00:00','suborbital rocket','success',122,'','Plumbob','','','Aeronomy',963,'',30 from dual union all
select 44350,'1977-W68','1977-01-11 20:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 44351,'1977-W680','1977-04-15 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 44352,'1977-W681','1977-04-15 21:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 44353,'1977-W682','1977-04-15 21:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 44354,'1977-W683','1977-04-18 12:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 44355,'1977-W684','1977-04-18 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 44356,'1977-W685','1977-04-18 16:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 44357,'1977-W686','1977-04-18 17:00:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',73 from dual union all
select 44358,'1977-W687','1977-04-18 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 44359,'1977-W688','1977-04-18 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 4436,'1957-A60','1957-07-01 00:00:00','suborbital rocket','success',122,'','Plumbob','','','Aeronomy',963,'',30 from dual union all
select 44360,'1977-W689','1977-04-18 19:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',54 from dual union all
select 44361,'1977-W69','1977-01-11 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',79 from dual union all
select 44362,'1977-W690','1977-04-18 23:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 44363,'1977-W691','1977-04-19 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1758,'VOLNA',59 from dual union all
select 44364,'1977-W692','1977-04-19 21:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 44365,'1977-W693','1977-04-20 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1758,'VOLNA',60 from dual union all
select 44366,'1977-W694','1977-04-20 12:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 44367,'1977-W695','1977-04-20 13:44:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',70 from dual union all
select 44368,'1977-W696','1977-04-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',79 from dual union all
select 44369,'1977-W697','1977-04-20 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 4437,'1957-A61','1957-07-05 06:17:56','suborbital rocket','failure',49,'NN3.09F','NRL NN3.09F','IGY NN3.09F','','Ionos',428,'',16 from dual union all
select 44370,'1977-W698','1977-04-20 15:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',64 from dual union all
select 44371,'1977-W699','1977-04-20 16:15:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 44372,'1977-W70','1977-01-12 04:25:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 44373,'1977-W700','1977-04-20 17:04:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',67 from dual union all
select 44374,'1977-W701','1977-04-20 17:30:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',55 from dual union all
select 44375,'1977-W702','1977-04-20 18:00:00','suborbital rocket','unknown',1284,'TH1-8710','','','','Aeron',428,'',73 from dual union all
select 44376,'1977-W703','1977-04-20 18:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 44377,'1977-W704','1977-04-20 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44378,'1977-W705','1977-04-20 19:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual;
insert into LAUNCH
select 44379,'1977-W706','1977-04-20 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',71 from dual union all
select 4438,'1957-A62','1957-07-12 12:53:00','military missile','failure',1016,'M1-7','8K71 LKI I-2','','','Test',480,'',20 from dual union all
select 44380,'1977-W707','1977-04-21 01:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 44381,'1977-W708','1977-04-21 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1758,'VOLNA',59 from dual union all
select 44382,'1977-W709','1977-04-21 15:40:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',68 from dual union all
select 44383,'1977-W71','1977-01-12 08:40:00','suborbital rocket','unknown',725,'','','','','Aeron',883,'KORO',79 from dual union all
select 44384,'1977-W710','1977-04-22 12:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',55 from dual union all
select 44385,'1977-W711','1977-04-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',79 from dual union all
select 44386,'1977-W712','1977-04-22 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 44387,'1977-W713','1977-04-22 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',54 from dual union all
select 44388,'1977-W714','1977-04-22 16:12:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 44389,'1977-W715','1977-04-22 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 4439,'1957-A63','1957-07-15 21:23:00','suborbital rocket','failure',841,'NN7.39F','NRL NN7.39F','IGY NN7.39F','','Solar UV/X',1415,'',28 from dual union all
select 44390,'1977-W716','1977-04-22 16:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 44391,'1977-W717','1977-04-22 18:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 44392,'1977-W718','1977-04-22 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 44393,'1977-W719','1977-04-23 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 44394,'1977-W72','1977-01-12 12:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 44395,'1977-W720','1977-04-23 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1758,'VOLNA',62 from dual union all
select 44396,'1977-W721','1977-04-25 12:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 44397,'1977-W722','1977-04-25 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 44398,'1977-W723','1977-04-25 16:06:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 44399,'1977-W724','1977-04-25 18:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 444,'1944-S61','1944-08-18 19:40:00','military missile','success',8,'18349','Ma135','','','Training',661,'',90 from dual union all
select 4440,'1957-A64','1957-07-17 00:00:00','military missile','success',1214,'N-69D N-3313','','','','Test',224,'',10 from dual union all
select 44400,'1977-W725','1977-04-25 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 44401,'1977-W726','1977-04-25 18:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 44402,'1977-W727','1977-04-25 20:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 44403,'1977-W728','1977-04-26 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1758,'VOLNA',63 from dual union all
select 44404,'1977-W729','1977-04-26 15:42:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',70 from dual union all
select 44405,'1977-W73','1977-01-12 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 44406,'1977-W730','1977-04-26 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44407,'1977-W731','1977-04-26 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',56 from dual union all
select 44408,'1977-W732','1977-04-26 21:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 44409,'1977-W733','1977-04-27 12:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 4441,'1957-A65','1957-07-22 00:00:00','military missile','success',255,'624-7','','','','OT',272,'',20 from dual union all
select 44410,'1977-W734','1977-04-27 14:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 44411,'1977-W735','1977-04-27 15:14:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',73 from dual union all
select 44412,'1977-W736','1977-04-27 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 44413,'1977-W737','1977-04-27 16:26:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 44414,'1977-W738','1977-04-27 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 44415,'1977-W739','1977-04-27 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 44416,'1977-W74','1977-01-12 15:24:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',54 from dual union all
select 44417,'1977-W740','1977-04-27 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44418,'1977-W741','1977-04-27 20:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 44419,'1977-W742','1977-04-27 21:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 4442,'1957-A66','1957-07-27 00:00:00','suborbital rocket','success',439,'','','','','Test',1490,'',24 from dual union all
select 44420,'1977-W743','1977-04-27 23:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',54 from dual union all
select 44421,'1977-W744','1977-04-28 21:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 44422,'1977-W745','1977-04-29 05:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1633,'USHAKOV',65 from dual union all
select 44423,'1977-W746','1977-04-29 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 44424,'1977-W747','1977-04-29 14:47:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 44425,'1977-W748','1977-04-29 14:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 44426,'1977-W749','1977-04-29 15:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',76 from dual union all
select 44427,'1977-W75','1977-01-12 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 44428,'1977-W750','1977-04-29 15:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 44429,'1977-W751','1977-04-29 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 4443,'1957-A67','1957-07-30 18:10:02','suborbital rocket','failure',839,'AM6.32','UM AM6.32','IGY AM6.32','','Aeron',428,'',25 from dual union all
select 44430,'1977-W752','1977-04-29 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44431,'1977-W753','1977-04-29 19:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 44432,'1977-W754','1977-04-29 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 44433,'1977-W755','1977-04-29 22:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 44434,'1977-W756','1977-05-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 44435,'1977-W757','1977-04-30 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1758,'VOLNA',60 from dual union all
select 44436,'1977-W758','1977-04-30 11:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1758,'VOLNA',68 from dual union all
select 44437,'1977-W759','1977-05-02 14:00:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',70 from dual union all
select 44438,'1977-W76','1977-01-12 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 44439,'1977-W760','1977-05-02 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 4444,'1957-A68','1957-08-01 00:00:00','suborbital rocket','success',122,'','Plumbob','','','Aeronomy',963,'',30 from dual union all
select 44440,'1977-W761','1977-05-02 16:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 44441,'1977-W762','1977-05-02 16:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 44442,'1977-W763','1977-05-02 16:23:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 44443,'1977-W764','1977-05-02 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44444,'1977-W765','1977-05-02 17:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 44445,'1977-W766','1977-05-02 18:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 44446,'1977-W767','1977-05-02 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 44447,'1977-W768','1977-05-02 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 44448,'1977-W769','1977-05-03 02:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 44449,'1977-W77','1977-01-12 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 4445,'1957-A69','1957-08-01 00:00:00','suborbital rocket','success',122,'','Plumbob','','','Aeronomy',963,'',30 from dual union all
select 44450,'1977-W770','1977-05-03 11:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1758,'VOLNA',62 from dual union all
select 44451,'1977-W771','1977-05-03 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 44452,'1977-W772','1977-05-03 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 44453,'1977-W773','1977-05-04 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1633,'USHAKOV',59 from dual union all
select 44454,'1977-W774','1977-05-04 13:54:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',72 from dual union all
select 44455,'1977-W775','1977-05-04 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 44456,'1977-W776','1977-05-04 14:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 44457,'1977-W777','1977-05-04 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 44458,'1977-W778','1977-05-04 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',56 from dual union all
select 44459,'1977-W779','1977-05-04 16:32:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 4446,'1957-A70','1957-08-01 00:00:00','military missile','failure',268,'2/1','','','','Test',1752,'',0 from dual union all
select 44460,'1977-W78','1977-01-12 17:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',61 from dual union all
select 44461,'1977-W780','1977-05-04 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 44462,'1977-W781','1977-05-04 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 44463,'1977-W782','1977-05-04 18:56:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',57 from dual union all
select 44464,'1977-W783','1977-05-04 20:48:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 44465,'1977-W784','1977-05-05 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 44466,'1977-W785','1977-05-05 01:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 44467,'1977-W786','1977-05-05 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 44468,'1977-W787','1977-05-05 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual;
insert into LAUNCH
select 44469,'1977-W788','1977-05-06 05:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1633,'USHAKOV',62 from dual union all
select 4447,'1957-A71','1957-08-02 00:00:00','atmospheric rocket','success',545,'','Flat RV','','','Reentry Test',1821,'',10 from dual union all
select 44470,'1977-W789','1977-05-06 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 44471,'1977-W79','1977-01-12 19:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 44472,'1977-W790','1977-05-06 14:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 44473,'1977-W791','1977-05-06 14:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 44474,'1977-W792','1977-05-06 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 44475,'1977-W793','1977-05-06 15:59:59','suborbital rocket','success',1284,'','','','','Meteo',1895,'',68 from dual union all
select 44476,'1977-W794','1977-05-06 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 44477,'1977-W795','1977-05-06 16:36:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',53 from dual union all
select 44478,'1977-W796','1977-05-06 16:40:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',73 from dual union all
select 44479,'1977-W797','1977-05-06 19:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 4448,'1957-A72','1957-08-05 19:10:00','suborbital rocket','failure',841,'NN7.41F','NRL NN7.41F','IGY NN7.41F','','Solar UV/X',1415,'',14 from dual union all
select 44480,'1977-W798','1977-05-06 20:21:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 44481,'1977-W799','1977-05-06 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 44482,'1977-W80','1977-01-12 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',64 from dual union all
select 44483,'1977-W800','1977-05-06 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 44484,'1977-W801','1977-05-07 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 44485,'1977-W802','1977-05-09 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 44486,'1977-W803','1977-05-09 15:14:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',73 from dual union all
select 44487,'1977-W804','1977-05-09 16:30:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 44488,'1977-W805','1977-05-09 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',56 from dual union all
select 44489,'1977-W806','1977-05-09 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 4449,'1957-A73','1957-08-07 00:00:00','atmospheric rocket','success',1052,'','Heat Transfer','','','Test',1821,'F2H2',4 from dual union all
select 44490,'1977-W807','1977-05-09 18:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 44491,'1977-W808','1977-05-09 20:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 44492,'1977-W809','1977-05-09 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 44493,'1977-W81','1977-01-13 00:49:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 44494,'1977-W810','1977-05-10 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44495,'1977-W811','1977-05-10 22:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 44496,'1977-W812','1977-05-11 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1633,'USHAKOV',61 from dual union all
select 44497,'1977-W813','1977-05-11 14:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 44498,'1977-W814','1977-05-11 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',76 from dual union all
select 44499,'1977-W815','1977-05-11 15:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 445,'1944-S62','1944-08-18 20:28:00','military missile','success',8,'18346','Ma112','','','Training',661,'',90 from dual union all
select 4450,'1957-A74','1957-08-07 03:28:00','suborbital rocket','failure',718,'II5.05F','SUI 60','IGY II5.05F','','Aeron/CR',692,'LSD-29',10 from dual union all
select 44500,'1977-W816','1977-05-11 15:14:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',72 from dual union all
select 44501,'1977-W817','1977-05-11 16:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 44502,'1977-W818','1977-05-11 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',77 from dual union all
select 44503,'1977-W819','1977-05-11 16:44:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',63 from dual union all
select 44504,'1977-W82','1977-01-13 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',77 from dual union all
select 44505,'1977-W820','1977-05-11 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',57 from dual union all
select 44506,'1977-W821','1977-05-11 18:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44507,'1977-W822','1977-05-11 20:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 44508,'1977-W823','1977-05-12 16:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 44509,'1977-W824','1977-05-13 14:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',62 from dual union all
select 4451,'1957-A75','1957-08-07 22:04:00','suborbital rocket','failure',718,'II5.06F','SUI 61','IGY II5.06F','','Fields',693,'LSD-29',10 from dual union all
select 44510,'1977-W825','1977-05-13 15:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 44511,'1977-W826','1977-05-13 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 44512,'1977-W827','1977-05-13 16:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44513,'1977-W828','1977-05-13 16:25:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 44514,'1977-W829','1977-05-13 16:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 44515,'1977-W83','1977-01-13 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 44516,'1977-W830','1977-05-13 19:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 44517,'1977-W831','1977-05-13 20:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 44518,'1977-W832','1977-05-13 23:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 44519,'1977-W833','1977-05-16 15:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 4452,'1957-A76','1957-08-11 05:16:00','suborbital rocket','failure',718,'II5.10F','SUI 65','IGY II5.10F','','Aeron/CR',680,'LSD-29',10 from dual union all
select 44520,'1977-W834','1977-05-16 16:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 44521,'1977-W835','1977-05-16 16:18:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',51 from dual union all
select 44522,'1977-W836','1977-05-16 17:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 44523,'1977-W837','1977-05-16 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 44524,'1977-W838','1977-05-16 18:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',51 from dual union all
select 44525,'1977-W839','1977-05-16 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 44526,'1977-W84','1977-01-13 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 44527,'1977-W840','1977-05-16 21:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 44528,'1977-W841','1977-05-16 23:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 44529,'1977-W842','1977-05-17 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 4453,'1957-A77','1957-08-12 00:00:00','military missile','success',832,'09','G-26 No. 4','','','Test    [Not LC10]',281,'',25 from dual union all
select 44530,'1977-W843','1977-05-17 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 44531,'1977-W844','1977-05-17 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 44532,'1977-W845','1977-05-17 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44533,'1977-W846','1977-05-18 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1633,'USHAKOV',65 from dual union all
select 44534,'1977-W847','1977-05-18 06:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',64 from dual union all
select 44535,'1977-W848','1977-05-18 13:44:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',65 from dual union all
select 44536,'1977-W849','1977-05-18 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 44537,'1977-W85','1977-01-13 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 44538,'1977-W850','1977-05-18 14:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 44539,'1977-W851','1977-05-18 15:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 4454,'1957-A78','1957-08-12 07:48:00','suborbital rocket','failure',718,'II5.12F','SUI 67','IGY II5.12F','','Aeron/Auroral/CR',682,'LSD-29',10 from dual union all
select 44540,'1977-W852','1977-05-18 16:00:59','suborbital rocket','success',1284,'','','','','Meteo',1895,'',58 from dual union all
select 44541,'1977-W853','1977-05-18 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 44542,'1977-W854','1977-05-18 16:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',71 from dual union all
select 44543,'1977-W855','1977-05-18 16:32:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 44544,'1977-W856','1977-05-18 16:37:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',69 from dual union all
select 44545,'1977-W857','1977-05-18 16:38:00','suborbital rocket','unknown',1284,'FLIGHT 154     /T 1-','','','','Aeron',1821,'',70 from dual union all
select 44546,'1977-W858','1977-05-18 17:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 44547,'1977-W859','1977-05-18 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 44548,'1977-W86','1977-01-13 21:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 44549,'1977-W860','1977-05-18 18:13:00','suborbital rocket','unknown',1284,'FLIGHT 155     /TH1-','','','','Aeron',428,'',70 from dual union all
select 4455,'1957-A79','1957-08-13 00:00:00','atmospheric rocket','success',861,'','Heat Transfer','','','Test',1821,'',10 from dual union all
select 44550,'1977-W861','1977-05-18 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 44551,'1977-W862','1977-05-18 19:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 44552,'1977-W863','1977-05-18 21:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',51 from dual union all
select 44553,'1977-W864','1977-05-20 14:20:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',72 from dual union all
select 44554,'1977-W865','1977-05-20 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 44555,'1977-W866','1977-05-20 14:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 44556,'1977-W867','1977-05-20 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 44557,'1977-W868','1977-05-20 16:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',76 from dual union all
select 44558,'1977-W869','1977-05-20 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual;
insert into LAUNCH
select 44559,'1977-W87','1977-01-14 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 4456,'1957-A80','1957-08-15 00:00:00','military missile','success',255,'624-8','','','','OT',293,'',20 from dual union all
select 44560,'1977-W870','1977-05-20 17:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 44561,'1977-W871','1977-05-20 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44562,'1977-W872','1977-05-20 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 44563,'1977-W873','1977-05-20 21:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 44564,'1977-W874','1977-05-21 00:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 44565,'1977-W875','1977-05-21 00:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 44566,'1977-W876','1977-05-23 12:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 44567,'1977-W877','1977-05-23 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',59 from dual union all
select 44568,'1977-W878','1977-05-23 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 44569,'1977-W879','1977-05-23 19:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 4457,'1957-A81','1957-08-16 00:00:00','military missile','success',1214,'N-69E N-3323','','','','Test',224,'',10 from dual union all
select 44570,'1977-W88','1977-01-14 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 44571,'1977-W880','1977-05-23 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 44572,'1977-W881','1977-05-23 23:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 44573,'1977-W882','1977-05-24 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 44574,'1977-W883','1977-05-25 03:20:00','suborbital rocket','unknown',738,'','','','','Aeron',1633,'USHAKOV',60 from dual union all
select 44575,'1977-W884','1977-05-25 12:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 44576,'1977-W885','1977-05-25 13:44:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',69 from dual union all
select 44577,'1977-W886','1977-05-25 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 44578,'1977-W887','1977-05-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',64 from dual union all
select 44579,'1977-W888','1977-05-25 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 4458,'1957-A82','1957-08-20 00:00:00','military missile','success',262,'3','','','','Test',243,'',10 from dual union all
select 44580,'1977-W889','1977-05-25 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 44581,'1977-W89','1977-01-14 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 44582,'1977-W890','1977-05-25 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 44583,'1977-W891','1977-05-25 18:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 44584,'1977-W892','1977-05-25 20:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 44585,'1977-W893','1977-05-25 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',73 from dual union all
select 44586,'1977-W894','1977-05-26 00:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 44587,'1977-W895','1977-05-26 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 44588,'1977-W896','1977-05-26 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 44589,'1977-W897','1977-05-27 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 4459,'1957-A83','1957-08-20 02:29:51','suborbital rocket','success',28,'SM1.05 (SC 40)','Grenades','IGY SM1.05','','Aeron',428,'',88 from dual union all
select 44590,'1977-W898','1977-05-27 14:34:00','suborbital rocket','success',1284,'Intercomp prep 1A','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 44591,'1977-W899','1977-05-27 14:40:00','suborbital rocket','success',1284,'Intercomp prep 1B','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 44592,'1977-W90','1977-01-14 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 44593,'1977-W900','1977-05-27 14:53:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',71 from dual union all
select 44594,'1977-W901','1977-05-27 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',76 from dual union all
select 44595,'1977-W902','1977-05-27 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 44596,'1977-W903','1977-05-27 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 44597,'1977-W904','1977-05-27 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 44598,'1977-W905','1977-05-27 16:48:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 44599,'1977-W906','1977-05-27 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 446,'1944-S63','1944-08-22 19:06:00','military missile','success',8,'18322','Ma141','','','Training',661,'',90 from dual union all
select 4460,'1957-A84','1957-08-24 06:00:00','suborbital rocket','failure',839,'SS6.38','Grenade test','IGY SS6.38','','Aeron',428,'',1 from dual union all
select 44600,'1977-W907','1977-05-27 18:38:59','suborbital rocket','success',1284,'Intercomp prep 2A','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 44601,'1977-W908','1977-05-27 18:45:00','suborbital rocket','success',1284,'Intercomp prep 2B','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 44602,'1977-W909','1977-05-27 18:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 44603,'1977-W91','1977-01-14 16:35:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 44604,'1977-W910','1977-05-27 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 44605,'1977-W911','1977-05-27 21:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 44606,'1977-W912','1977-05-27 22:56:59','suborbital rocket','success',1284,'Intercomp prep 3A','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 44607,'1977-W913','1977-05-27 23:03:00','suborbital rocket','success',1284,'Intercomp prep 3B','Arcasonde','','','Meteo',1821,'',60 from dual union all
select 44608,'1977-W914','1977-05-27 23:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 44609,'1977-W915','1977-05-28 00:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 4461,'1957-A85','1957-08-27 00:00:00','military missile','success',1214,'N-69D N-3317','','','','Test',224,'',10 from dual union all
select 44610,'1977-W916','1977-05-28 02:40:00','suborbital rocket','success',1284,'Intercomp prep 4B','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 44611,'1977-W917','1977-05-28 06:30:00','suborbital rocket','success',1284,'Intercomp prep 5A','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 44612,'1977-W918','1977-05-28 10:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',76 from dual union all
select 44613,'1977-W919','1977-05-28 10:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 44614,'1977-W92','1977-01-14 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 44615,'1977-W920','1977-05-30 15:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 44616,'1977-W921','1977-05-30 16:26:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 44617,'1977-W922','1977-05-31 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 44618,'1977-W923','1977-05-31 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 44619,'1977-W924','1977-05-31 15:45:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',69 from dual union all
select 4462,'1957-A86','1957-08-27 15:54:00','suborbital rocket','failure',841,'NN7.43F','NRL NN7.43F','IGY NN7.43F','','Solar UV/X',1415,'',16 from dual union all
select 44620,'1977-W925','1977-05-31 17:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 44621,'1977-W926','1977-05-31 21:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 44622,'1977-W927','1977-05-31 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 44623,'1977-W928','1977-06-01 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1633,'USHAKOV',63 from dual union all
select 44624,'1977-W929','1977-06-01 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 44625,'1977-W93','1977-01-14 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 44626,'1977-W930','1977-06-01 14:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 44627,'1977-W931','1977-06-01 15:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 44628,'1977-W932','1977-06-01 15:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 44629,'1977-W933','1977-06-01 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 4463,'1957-A87','1957-08-29 00:00:00','atmospheric rocket','success',552,'','Glide Rocket','','','Reentry Test',1821,'',27 from dual union all
select 44630,'1977-W934','1977-06-01 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 44631,'1977-W935','1977-06-01 17:12:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 44632,'1977-W936','1977-06-01 18:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 44633,'1977-W937','1977-06-01 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 44634,'1977-W938','1977-06-01 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 44635,'1977-W939','1977-06-01 22:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',74 from dual union all
select 44636,'1977-W94','1977-01-14 20:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1047,'',60 from dual union all
select 44637,'1977-W940','1977-06-02 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 44638,'1977-W941','1977-06-02 01:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 44639,'1977-W942','1977-06-02 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',56 from dual union all
select 4464,'1957-A88','1957-08-29 00:00:00','military missile','success',771,'GM-54-137','','','','Test',222,'',12 from dual union all
select 44640,'1977-W943','1977-06-02 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 44641,'1977-W944','1977-06-02 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 44642,'1977-W945','1977-06-03 04:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1633,'USHAKOV',60 from dual union all
select 44643,'1977-W946','1977-06-03 14:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 44644,'1977-W947','1977-06-03 15:00:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',72 from dual union all
select 44645,'1977-W948','1977-06-03 16:55:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 44646,'1977-W949','1977-06-03 18:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 44647,'1977-W95','1977-01-14 21:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 44648,'1977-W950','1977-06-03 20:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual;
insert into LAUNCH
select 44649,'1977-W951','1977-06-03 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 4465,'1957-A89','1957-08-30 20:10:00','military missile','failure',1366,'104','R&D Series I','','','Test',233,'',0 from dual union all
select 44650,'1977-W952','1977-06-04 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 44651,'1977-W953','1977-06-04 04:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1633,'USHAKOV',61 from dual union all
select 44652,'1977-W954','1977-06-06 12:59:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',72 from dual union all
select 44653,'1977-W955','1977-06-06 16:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 44654,'1977-W956','1977-06-06 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 44655,'1977-W957','1977-06-06 20:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 44656,'1977-W958','1977-06-06 20:38:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 44657,'1977-W959','1977-06-07 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 44658,'1977-W96','1977-01-15 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 44659,'1977-W960','1977-06-07 12:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',57 from dual union all
select 4466,'1957-A90','1957-09-01 00:00:00','suborbital rocket','success',122,'','Plumbob','','','Aeronomy',963,'',30 from dual union all
select 44660,'1977-W961','1977-06-08 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',61 from dual union all
select 44661,'1977-W962','1977-06-08 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',62 from dual union all
select 44662,'1977-W963','1977-06-08 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 44663,'1977-W964','1977-06-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',79 from dual union all
select 44664,'1977-W965','1977-06-08 14:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',77 from dual union all
select 44665,'1977-W966','1977-06-08 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',64 from dual union all
select 44666,'1977-W967','1977-06-08 16:04:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',51 from dual union all
select 44667,'1977-W968','1977-06-08 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',69 from dual union all
select 44668,'1977-W969','1977-06-08 17:00:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',65 from dual union all
select 44669,'1977-W97','1977-01-17 14:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 4467,'1957-A91','1957-09-01 00:00:00','suborbital rocket','success',122,'','Plumbob','','','Aeronomy',963,'',30 from dual union all
select 44670,'1977-W970','1977-06-08 17:01:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 44671,'1977-W971','1977-06-08 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 44672,'1977-W972','1977-06-08 18:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 44673,'1977-W973','1977-06-08 20:32:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',70 from dual union all
select 44674,'1977-W974','1977-06-09 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 44675,'1977-W975','1977-06-09 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 44676,'1977-W976','1977-06-09 12:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 44677,'1977-W977','1977-06-09 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 44678,'1977-W978','1977-06-09 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 44679,'1977-W979','1977-06-10 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 4468,'1957-A92','1957-09-01 00:00:00','military missile','failure',268,'2/1','','','','Test',1752,'',0 from dual union all
select 44680,'1977-W98','1977-01-17 16:00:59','suborbital rocket','success',1061,'Winter Strato','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 44681,'1977-W980','1977-06-10 14:59:00','suborbital rocket','success',1061,'','','','','Meteo',35,'',70 from dual union all
select 44682,'1977-W981','1977-06-10 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 44683,'1977-W982','1977-06-11 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 44684,'1977-W983','1977-06-11 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',67 from dual union all
select 44685,'1977-W984','1977-06-12 23:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 44686,'1977-W985','1977-06-13 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 44687,'1977-W986','1977-06-13 13:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 44688,'1977-W987','1977-06-13 15:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 44689,'1977-W988','1977-06-13 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 4469,'1957-A93','1957-09-03 00:00:00','military missile','success',771,'GM-54-20','','','','Test',222,'',12 from dual union all
select 44690,'1977-W989','1977-06-13 15:34:59','suborbital rocket','success',1061,'','','','','Meteo',35,'',75 from dual union all
select 44691,'1977-W99','1977-01-17 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',54 from dual union all
select 44692,'1977-W990','1977-06-13 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 44693,'1977-W991','1977-06-13 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 44694,'1977-W992','1977-06-13 23:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 44695,'1977-W993','1977-06-14 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 44696,'1977-W994','1977-06-14 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 44697,'1977-W995','1977-06-14 15:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 44698,'1977-W996','1977-06-14 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 44699,'1977-W997','1977-06-14 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 447,'1944-S64','1944-08-23 19:46:00','military missile','success',8,'18210','Ma30','','','Training',661,'',90 from dual union all
select 4470,'1957-A94','1957-09-05 00:00:00','military missile','success',771,'GM-54-139','','','','Test',222,'',12 from dual union all
select 44700,'1977-W998','1977-06-14 23:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 44701,'1977-W999','1977-06-15 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',900,'KRENKEL''',59 from dual union all
select 44702,'1977-Y01','1977-01-01 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',85 from dual union all
select 44703,'1977-Y02','1977-01-02 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',83 from dual union all
select 44704,'1977-Y03','1977-01-02 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1394,'SHOK',83 from dual union all
select 44705,'1977-Y04','1977-01-03 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',87 from dual union all
select 44706,'1977-Y05','1977-01-04 12:40:00','suborbital rocket','unknown',725,'','','','','Aeron',883,'KORO',86 from dual union all
select 44707,'1977-Y06','1977-01-04 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 44708,'1977-Y07','1977-01-04 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 44709,'1977-Y08','1977-01-05 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',86 from dual union all
select 4471,'1957-A95','1957-09-05 00:00:00','military missile','success',771,'GM-54-32','','','','Test',222,'',12 from dual union all
select 44710,'1977-Y09','1977-01-05 09:50:00','suborbital rocket','unknown',725,'','','','','Aeron',883,'KORO',87 from dual union all
select 44711,'1977-Y10','1977-01-05 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 44712,'1977-Y100','1977-03-15 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 44713,'1977-Y101','1977-03-16 13:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 44714,'1977-Y102','1977-03-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 44715,'1977-Y103','1977-03-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',90 from dual union all
select 44716,'1977-Y104','1977-03-16 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 44717,'1977-Y105','1977-03-16 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 44718,'1977-Y106','1977-03-17 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 44719,'1977-Y107','1977-03-17 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 4472,'1957-A96','1957-09-12 15:19:30','suborbital rocket','failure',841,'NN7.46F','NRL NN7.46F','IGY NN7.46F','','Solar UV/X',1415,'',3 from dual union all
select 44720,'1977-Y108','1977-03-18 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',884,'KORO',82 from dual union all
select 44721,'1977-Y109','1977-03-18 14:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 44722,'1977-Y11','1977-01-05 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 44723,'1977-Y110','1977-03-18 15:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 44724,'1977-Y111','1977-03-18 16:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 44725,'1977-Y112','1977-03-18 18:29:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 44726,'1977-Y113','1977-03-19 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',884,'KORO',84 from dual union all
select 44727,'1977-Y114','1977-03-20 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',884,'KORO',88 from dual union all
select 44728,'1977-Y115','1977-03-21 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',884,'KORO',83 from dual union all
select 44729,'1977-Y116','1977-03-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',91 from dual union all
select 4473,'1957-A97','1957-09-15 20:43:00','suborbital rocket','failure',841,'NN7.47F','NRL NN7.47F','IGY NN7.47F','','Solar UV/X',1415,'',90 from dual union all
select 44730,'1977-Y117','1977-03-22 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',884,'KORO',83 from dual union all
select 44731,'1977-Y118','1977-03-22 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 44732,'1977-Y119','1977-03-23 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',884,'KORO',85 from dual union all
select 44733,'1977-Y12','1977-01-08 20:21:00','suborbital rocket','success',1281,'NASA 15.152UE','','','','Plasma',1821,'',86 from dual union all
select 44734,'1977-Y120','1977-03-23 13:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 44735,'1977-Y121','1977-03-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',93 from dual union all
select 44736,'1977-Y122','1977-03-23 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 44737,'1977-Y123','1977-03-23 16:44:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',96 from dual union all
select 44738,'1977-Y124','1977-03-23 17:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual;
insert into LAUNCH
select 44739,'1977-Y125','1977-03-23 19:20:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 4474,'1957-A98','1957-09-16 19:20:00','military missile','success',255,'624-9','','','','OT',293,'',20 from dual union all
select 44740,'1977-Y126','1977-03-24 18:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 44741,'1977-Y127','1977-03-25 14:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 44742,'1977-Y128','1977-03-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',89 from dual union all
select 44743,'1977-Y129','1977-03-26 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',884,'KORO',85 from dual union all
select 44744,'1977-Y13','1977-01-11 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 44745,'1977-Y130','1977-03-28 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',884,'KORO',82 from dual union all
select 44746,'1977-Y131','1977-03-28 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',884,'KORO',81 from dual union all
select 44747,'1977-Y132','1977-03-30 13:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 44748,'1977-Y133','1977-03-30 14:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 44749,'1977-Y134','1977-03-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 4475,'1957-A99','1957-09-18 21:15:00','military missile','success',832,'10','G-26 No. 6','','','Test',281,'',23 from dual union all
select 44750,'1977-Y135','1977-03-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 44751,'1977-Y136','1977-03-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 44752,'1977-Y137','1977-03-30 16:31:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',97 from dual union all
select 44753,'1977-Y138','1977-03-30 23:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 44754,'1977-Y139','1977-04-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 44755,'1977-Y14','1977-01-12 05:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 44756,'1977-Y140','1977-04-01 18:51:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 44757,'1977-Y141','1977-04-01 21:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',87 from dual union all
select 44758,'1977-Y142','1977-04-06 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 44759,'1977-Y143','1977-04-06 11:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 4476,'1957-F01','1957-12-06 16:44:35','orbital','failure',1486,'TV-3','Vanguard TV3','Vanguard Test Satellite','','Sat',235,'',0 from dual union all
select 44760,'1977-Y144','1977-04-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 44761,'1977-Y145','1977-04-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 44762,'1977-Y146','1977-04-06 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',92 from dual union all
select 44763,'1977-Y147','1977-04-06 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 44764,'1977-Y148','1977-04-06 15:25:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',96 from dual union all
select 44765,'1977-Y149','1977-04-06 20:15:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 44766,'1977-Y15','1977-01-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 44767,'1977-Y150','1977-04-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 44768,'1977-Y151','1977-04-08 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 44769,'1977-Y152','1977-04-13 07:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 4477,'1957-M01','1957-01-31 00:00:00','military missile','success',1162,'5','','','','OT?',1852,'',50 from dual union all
select 44770,'1977-Y153','1977-04-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 44771,'1977-Y154','1977-04-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 44772,'1977-Y155','1977-04-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 44773,'1977-Y156','1977-04-13 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',89 from dual union all
select 44774,'1977-Y157','1977-04-13 17:55:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 44775,'1977-Y158','1977-04-13 19:25:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 44776,'1977-Y159','1977-04-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 44777,'1977-Y16','1977-01-12 15:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',93 from dual union all
select 44778,'1977-Y160','1977-04-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',91 from dual union all
select 44779,'1977-Y161','1977-04-18 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 4478,'1957-M02','1957-02-12 20:30:00','suborbital rocket','success',718,'','II5.097','IGY II5.97','','Fields',637,'',75 from dual union all
select 44780,'1977-Y162','1977-04-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',91 from dual union all
select 44781,'1977-Y163','1977-04-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 44782,'1977-Y164','1977-04-20 17:37:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',81 from dual union all
select 44783,'1977-Y165','1977-04-20 18:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 44784,'1977-Y166','1977-04-22 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 44785,'1977-Y167','1977-04-22 17:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',94 from dual union all
select 44786,'1977-Y168','1977-04-27 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 44787,'1977-Y169','1977-04-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 44788,'1977-Y17','1977-01-12 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 44789,'1977-Y170','1977-04-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 4479,'1957-M03','1957-02-14 20:05:00','suborbital rocket','success',718,'','II5.098','IGY II5.98','','Fields',637,'',75 from dual union all
select 44790,'1977-Y171','1977-04-27 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 44791,'1977-Y172','1977-04-27 18:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 44792,'1977-Y173','1977-04-27 21:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 44793,'1977-Y174','1977-05-04 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 44794,'1977-Y175','1977-05-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 44795,'1977-Y176','1977-05-04 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 44796,'1977-Y177','1977-05-04 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 44797,'1977-Y178','1977-05-04 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',95 from dual union all
select 44798,'1977-Y179','1977-05-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1395,'SHOK',87 from dual union all
select 44799,'1977-Y18','1977-01-12 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 448,'1944-S65','1944-08-24 14:39:00','military missile','success',8,'18209','Ma29','','','Training',661,'',90 from dual union all
select 4480,'1957-M04','1957-02-17 21:36:00','suborbital rocket','success',718,'','II5.099','IGY II5.99','','Fields',637,'',75 from dual union all
select 44800,'1977-Y180','1977-05-11 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 44801,'1977-Y181','1977-05-11 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 44802,'1977-Y182','1977-05-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 44803,'1977-Y183','1977-05-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1395,'SHOK',85 from dual union all
select 44804,'1977-Y184','1977-05-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 44805,'1977-Y185','1977-05-11 15:14:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',95 from dual union all
select 44806,'1977-Y186','1977-05-11 15:21:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 44807,'1977-Y187','1977-05-11 18:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 44808,'1977-Y188','1977-05-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 44809,'1977-Y189','1977-05-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 4481,'1957-M05','1957-04-01 00:00:00','suborbital rocket','success',440,'','Test 17 (last R&D)','','','Test',1490,'',80 from dual union all
select 44810,'1977-Y19','1977-01-17 07:30:00','suborbital rocket','unknown',725,'','','','','Aeron',883,'KORO',85 from dual union all
select 44811,'1977-Y190','1977-05-17 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1395,'SHOK',82 from dual union all
select 44812,'1977-Y191','1977-05-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 44813,'1977-Y192','1977-05-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 44814,'1977-Y193','1977-05-18 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 44815,'1977-Y194','1977-05-18 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 44816,'1977-Y195','1977-05-18 15:14:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',96 from dual union all
select 44817,'1977-Y196','1977-05-18 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',100 from dual union all
select 44818,'1977-Y197','1977-05-18 18:38:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',86 from dual union all
select 44819,'1977-Y198','1977-05-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1395,'SHOK',85 from dual union all
select 4482,'1957-M06','1957-04-16 00:00:00','military missile','success',1162,'6','','','','OT?',1852,'',50 from dual union all
select 44820,'1977-Y199','1977-05-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 44821,'1977-Y20','1977-01-18 02:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',86 from dual union all
select 44822,'1977-Y200','1977-05-20 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 44823,'1977-Y201','1977-05-21 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1395,'SHOK',85 from dual union all
select 44824,'1977-Y202','1977-05-21 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1395,'SHOK',80 from dual union all
select 44825,'1977-Y203','1977-05-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 44826,'1977-Y204','1977-05-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 44827,'1977-Y205','1977-05-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 44828,'1977-Y206','1977-05-25 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',95 from dual;
insert into LAUNCH
select 44829,'1977-Y207','1977-05-25 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 4483,'1957-M07','1957-04-09 15:19:00','suborbital rocket','success',46,'USAF 77','','','','Meteorites',640,'',61 from dual union all
select 44830,'1977-Y208','1977-05-25 18:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',102 from dual union all
select 44831,'1977-Y209','1977-05-25 18:15:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 44832,'1977-Y21','1977-01-18 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 44833,'1977-Y210','1977-05-25 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',87 from dual union all
select 44834,'1977-Y211','1977-05-27 19:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 44835,'1977-Y212','1977-05-27 20:45:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 44836,'1977-Y213','1977-05-27 21:09:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',87 from dual union all
select 44837,'1977-Y214','1977-05-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 44838,'1977-Y215','1977-06-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 44839,'1977-Y216','1977-06-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 4484,'1957-M08','1957-05-22 05:20:00','suborbital rocket','success',1193,'SL02','RAE Test 2','','','Test',1839,'',75 from dual union all
select 44840,'1977-Y217','1977-06-01 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 44841,'1977-Y218','1977-06-01 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 44842,'1977-Y219','1977-06-01 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',90 from dual union all
select 44843,'1977-Y22','1977-01-18 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 44844,'1977-Y220','1977-06-01 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',94 from dual union all
select 44845,'1977-Y221','1977-06-01 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',100 from dual union all
select 44846,'1977-Y222','1977-06-01 21:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 44847,'1977-Y223','1977-06-04 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1395,'SHOK',80 from dual union all
select 44848,'1977-Y224','1977-06-04 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1395,'SHOK',81 from dual union all
select 44849,'1977-Y225','1977-06-06 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 4485,'1957-M09','1957-08-10 23:36:00','suborbital rocket','success',718,'II5.09F','SUI 64','IGY II5.09F','','Aeron/Auroral/CR',696,'LSD-29',77 from dual union all
select 44850,'1977-Y226','1977-06-08 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 44851,'1977-Y227','1977-06-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 44852,'1977-Y228','1977-06-08 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 44853,'1977-Y229','1977-06-08 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',94 from dual union all
select 44854,'1977-Y23','1977-01-19 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 44855,'1977-Y230','1977-06-10 19:34:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 44856,'1977-Y231','1977-06-15 12:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 44857,'1977-Y232','1977-06-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 44858,'1977-Y233','1977-06-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 44859,'1977-Y234','1977-06-15 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 4486,'1957-M10','1957-08-12 15:59:31','suborbital rocket','success',28,'SM1.04','Grenades','IGY SM1.04','','Aeron',428,'',74 from dual union all
select 44860,'1977-Y235','1977-06-15 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',95 from dual union all
select 44861,'1977-Y236','1977-06-15 16:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',101 from dual union all
select 44862,'1977-Y237','1977-06-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 44863,'1977-Y238','1977-06-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 44864,'1977-Y239','1977-06-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 44865,'1977-Y24','1977-01-19 01:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',100 from dual union all
select 44866,'1977-Y240','1977-06-22 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 44867,'1977-Y241','1977-06-22 15:31:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',94 from dual union all
select 44868,'1977-Y242','1977-06-22 21:24:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 44869,'1977-Y243','1977-06-22 21:27:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 4487,'1957-M11','1957-08-25 02:29:00','suborbital rocket','success',28,'SM2.05','','[IGY SM2.05]','','Aeron',428,'',51 from dual union all
select 44870,'1977-Y244','1977-06-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 44871,'1977-Y245','1977-06-24 17:08:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',102 from dual union all
select 44872,'1977-Y246','1977-06-29 10:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 44873,'1977-Y247','1977-06-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 44874,'1977-Y248','1977-06-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 44875,'1977-Y249','1977-06-29 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',104 from dual union all
select 44876,'1977-Y25','1977-01-19 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 44877,'1977-Y250','1977-06-29 19:03:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 44878,'1977-Y251','1977-06-29 23:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',82 from dual union all
select 44879,'1977-Y252','1977-07-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 4488,'1957-M12','1957-09-17 14:04:00','suborbital rocket','success',54,'NRL 21','NRL UV Echelle','IGY NN1.25','','Solar UV',1869,'',51 from dual union all
select 44880,'1977-Y253','1977-07-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 44881,'1977-Y254','1977-07-06 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 44882,'1977-Y255','1977-07-06 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 44883,'1977-Y256','1977-07-06 21:06:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 44884,'1977-Y257','1977-07-06 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 44885,'1977-Y258','1977-07-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 44886,'1977-Y259','1977-07-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 44887,'1977-Y26','1977-01-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 44888,'1977-Y260','1977-07-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 44889,'1977-Y261','1977-07-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 4489,'1957-M13','1957-09-18 17:54:00','suborbital rocket','success',841,'NN7.49F','','IGY NN7.49F','','Solar UV/Solar X',1415,'',77 from dual union all
select 44890,'1977-Y262','1977-07-14 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 44891,'1977-Y263','1977-07-15 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 44892,'1977-Y264','1977-07-15 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1395,'SHOK',81 from dual union all
select 44893,'1977-Y265','1977-07-16 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 44894,'1977-Y266','1977-07-20 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 44895,'1977-Y267','1977-07-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 44896,'1977-Y268','1977-07-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 44897,'1977-Y269','1977-07-20 18:20:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 44898,'1977-Y27','1977-01-19 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',95 from dual union all
select 44899,'1977-Y270','1977-07-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 449,'1944-S66','1944-08-26 05:36:00','military missile','success',8,'18337','Ma148','','','Training',661,'',90 from dual union all
select 4490,'1957-M14','1957-10-03 00:00:00','military missile','success',1162,'8','JPL-477','','','OT?',1852,'',50 from dual union all
select 44900,'1977-Y271','1977-07-22 14:42:00','suborbital rocket','success',1281,'T 1-9105','','','','Aeron',1821,'',85 from dual union all
select 44901,'1977-Y272','1977-07-22 21:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',87 from dual union all
select 44902,'1977-Y273','1977-07-24 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',82 from dual union all
select 44903,'1977-Y274','1977-07-25 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 44904,'1977-Y275','1977-07-27 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 44905,'1977-Y276','1977-07-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 44906,'1977-Y277','1977-07-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 44907,'1977-Y278','1977-07-27 18:20:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 44908,'1977-Y279','1977-07-27 23:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',82 from dual union all
select 44909,'1977-Y28','1977-01-19 18:21:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual union all
select 4491,'1957-M15','1957-11-26 00:00:00','military missile','success',1162,'7','JPL-477','','','OT?',1852,'',50 from dual union all
select 44910,'1977-Y280','1977-07-28 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',82 from dual union all
select 44911,'1977-Y281','1977-07-29 09:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',82 from dual union all
select 44912,'1977-Y282','1977-08-01 23:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 44913,'1977-Y283','1977-08-03 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 44914,'1977-Y284','1977-08-03 10:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 44915,'1977-Y285','1977-08-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 44916,'1977-Y286','1977-08-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',90 from dual union all
select 44917,'1977-Y287','1977-08-03 17:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 44918,'1977-Y288','1977-08-03 18:31:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual;
insert into LAUNCH
select 44919,'1977-Y289','1977-08-03 21:27:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',85 from dual union all
select 4492,'1957-S01','1957-01-01 00:00:00','suborbital rocket','success',1324,'','','','','CR?',1821,'',100 from dual union all
select 44920,'1977-Y29','1977-01-19 20:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 44921,'1977-Y290','1977-08-03 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 44922,'1977-Y291','1977-08-05 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 44923,'1977-Y292','1977-08-10 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 44924,'1977-Y293','1977-08-10 10:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 44925,'1977-Y294','1977-08-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 44926,'1977-Y295','1977-08-10 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 44927,'1977-Y296','1977-08-10 19:11:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 44928,'1977-Y297','1977-08-12 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 44929,'1977-Y298','1977-08-15 23:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 4493,'1957-S02','1957-01-01 00:00:00','suborbital rocket','success',1324,'','','','','CR?',1821,'',100 from dual union all
select 44930,'1977-Y299','1977-08-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 44931,'1977-Y30','1977-01-20 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',87 from dual union all
select 44932,'1977-Y300','1977-08-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 44933,'1977-Y301','1977-08-17 16:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 44934,'1977-Y302','1977-08-17 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 44935,'1977-Y303','1977-08-19 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 44936,'1977-Y304','1977-08-23 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 44937,'1977-Y305','1977-08-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 44938,'1977-Y306','1977-08-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 44939,'1977-Y307','1977-08-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',88 from dual union all
select 4494,'1957-S03','1957-01-01 00:00:00','suborbital rocket','success',1324,'','','','','CR?',1821,'',100 from dual union all
select 44940,'1977-Y308','1977-08-24 19:18:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 44941,'1977-Y309','1977-08-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 44942,'1977-Y31','1977-01-23 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',84 from dual union all
select 44943,'1977-Y310','1977-08-26 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 44944,'1977-Y311','1977-08-31 11:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 44945,'1977-Y312','1977-08-31 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 44946,'1977-Y313','1977-08-31 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 44947,'1977-Y314','1977-08-31 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 44948,'1977-Y315','1977-08-31 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 44949,'1977-Y316','1977-08-31 21:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 4495,'1957-S04','1957-01-01 00:00:00','suborbital rocket','success',1324,'','','','','CR?',1821,'',100 from dual union all
select 44950,'1977-Y317','1977-09-01 21:34:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',88 from dual union all
select 44951,'1977-Y318','1977-09-02 15:29:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 44952,'1977-Y319','1977-09-03 17:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',82 from dual union all
select 44953,'1977-Y32','1977-01-23 15:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 44954,'1977-Y320','1977-09-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 44955,'1977-Y321','1977-09-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 44956,'1977-Y322','1977-09-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 44957,'1977-Y323','1977-09-07 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',100 from dual union all
select 44958,'1977-Y324','1977-09-07 19:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 44959,'1977-Y325','1977-09-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 4496,'1957-S05','1957-01-01 00:00:00','suborbital rocket','success',440,'','','','','Test',1490,'',84 from dual union all
select 44960,'1977-Y326','1977-09-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 44961,'1977-Y327','1977-09-14 17:53:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 44962,'1977-Y328','1977-09-14 18:20:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 44963,'1977-Y329','1977-09-16 21:03:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',87 from dual union all
select 44964,'1977-Y33','1977-01-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',87 from dual union all
select 44965,'1977-Y330','1977-09-17 02:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 44966,'1977-Y331','1977-09-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 44967,'1977-Y332','1977-09-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 44968,'1977-Y333','1977-09-21 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 44969,'1977-Y334','1977-09-23 12:56:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 4497,'1957-S06','1957-01-08 17:59:00','suborbital rocket','success',1521,'R-18','4203-18','','','Reentry Test',253,'',140 from dual union all
select 44970,'1977-Y335','1977-09-24 11:15:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',103 from dual union all
select 44971,'1977-Y336','1977-09-24 12:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',101 from dual union all
select 44972,'1977-Y337','1977-09-24 12:34:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 44973,'1977-Y338','1977-09-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 44974,'1977-Y339','1977-09-28 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 44975,'1977-Y34','1977-01-25 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 44976,'1977-Y340','1977-09-28 20:17:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 44977,'1977-Y341','1977-09-28 20:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',88 from dual union all
select 44978,'1977-Y342','1977-09-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 44979,'1977-Y343','1977-09-29 18:22:00','suborbital rocket','unknown',1284,'FLIGHT 174     /TW1-','','','','Aeron',1895,'',84 from dual union all
select 4498,'1957-S07','1957-01-15 16:30:00','suborbital rocket','failure',1521,'R-19','4203-19','','','Reentry Test',253,'',146 from dual union all
select 44980,'1977-Y344','1977-09-30 17:22:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 44981,'1977-Y345','1977-10-05 04:43:00','suborbital rocket','success',725,'ISRO 8.345','','Ozonesonde','','Aeron',1467,'',90 from dual union all
select 44982,'1977-Y346','1977-10-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 44983,'1977-Y347','1977-10-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 44984,'1977-Y348','1977-10-05 15:00:00','suborbital rocket','unknown',725,'ISRO 8.346','','','','Aeron',1467,'',85 from dual union all
select 44985,'1977-Y349','1977-10-05 17:00:00','suborbital rocket','unknown',725,'ISRO 8.347','','','','Aeron',1467,'',81 from dual union all
select 44986,'1977-Y35','1977-01-26 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 44987,'1977-Y350','1977-10-05 19:54:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',85 from dual union all
select 44988,'1977-Y351','1977-10-05 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',94 from dual union all
select 44989,'1977-Y352','1977-10-05 21:16:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 4499,'1957-S08','1957-01-19 00:00:00','military missile','success',1533,'V-300 S-25M','ZUR-215','','','Nuclear Test',636,'',10 from dual union all
select 44990,'1977-Y353','1977-10-11 14:00:00','suborbital rocket','unknown',725,'ISRO 8.348','','','','Aeron',1467,'',82 from dual union all
select 44991,'1977-Y354','1977-10-12 12:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 44992,'1977-Y355','1977-10-12 13:45:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 44993,'1977-Y356','1977-10-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 44994,'1977-Y357','1977-10-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 44995,'1977-Y358','1977-10-12 17:15:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 44996,'1977-Y359','1977-10-12 18:17:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 44997,'1977-Y36','1977-01-26 09:10:00','suborbital rocket','unknown',725,'','','','','Aeron',883,'KORO',83 from dual union all
select 44998,'1977-Y360','1977-10-12 21:22:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',87 from dual union all
select 44999,'1977-Y361','1977-10-13 20:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',94 from dual union all
select 45,'1944-A09','1944-08-14 09:04:00','military missile','failure',1470,'18302','Ma119','','','Training',661,'',1 from dual union all
select 450,'1944-S67','1944-08-26 09:54:00','military missile','success',8,'18333','Ma153','','','Training',661,'',90 from dual union all
select 4500,'1957-S09','1957-01-19 01:37:00','military missile','success',635,'CC-16    HV','Jupiter A','','','Test',280,'',90 from dual union all
select 45000,'1977-Y362','1977-10-18 21:28:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 45001,'1977-Y363','1977-10-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 45002,'1977-Y364','1977-10-19 14:00:00','suborbital rocket','unknown',725,'ISRO 8.349','','','','Aeron',1467,'',82 from dual union all
select 45003,'1977-Y365','1977-10-19 17:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45004,'1977-Y366','1977-10-19 20:54:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',88 from dual union all
select 45005,'1977-Y367','1977-10-21 11:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 45006,'1977-Y368','1977-10-21 12:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 45007,'1977-Y369','1977-10-25 11:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual;
insert into LAUNCH
select 45008,'1977-Y37','1977-01-26 10:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 45009,'1977-Y370','1977-10-25 12:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 4501,'1957-S10','1957-01-01 00:00:00','suborbital rocket','success',813,'','','','','Test',313,'',90 from dual union all
select 45010,'1977-Y371','1977-10-25 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 45011,'1977-Y372','1977-10-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 45012,'1977-Y373','1977-10-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 45013,'1977-Y374','1977-10-26 19:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45014,'1977-Y375','1977-10-26 19:27:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',86 from dual union all
select 45015,'1977-Y376','1977-10-26 20:15:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45016,'1977-Y377','1977-10-26 20:18:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45017,'1977-Y378','1977-10-26 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 45018,'1977-Y379','1977-11-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 45019,'1977-Y38','1977-01-26 20:36:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 4502,'1957-S100','1957-08-29 21:12:25','suborbital rocket','success',841,'NN7.45F/DAN-23','NRL NN7.45F','IGY NN7.45F','','Solar UV/X',1415,'',113 from dual union all
select 45020,'1977-Y380','1977-11-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 45021,'1977-Y381','1977-11-02 16:00:00','suborbital rocket','unknown',725,'ISRO 8.351','','','','Aeron',1467,'',85 from dual union all
select 45022,'1977-Y382','1977-11-02 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 45023,'1977-Y383','1977-11-02 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 45024,'1977-Y384','1977-11-02 19:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 45025,'1977-Y385','1977-11-02 19:52:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',86 from dual union all
select 45026,'1977-Y386','1977-11-04 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 45027,'1977-Y387','1977-11-04 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 45028,'1977-Y388','1977-11-04 21:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',86 from dual union all
select 45029,'1977-Y389','1977-11-04 21:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 4503,'1957-S101','1957-08-29 00:00:00','military missile','success',976,'M3-6','63M No. 7 LKI1-7','','','Test',636,'',402 from dual union all
select 45030,'1977-Y39','1977-01-27 08:10:00','suborbital rocket','unknown',725,'','','','','Aeron',883,'KORO',81 from dual union all
select 45031,'1977-Y390','1977-11-04 23:00:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',98 from dual union all
select 45032,'1977-Y391','1977-11-05 00:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45033,'1977-Y392','1977-11-05 03:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45034,'1977-Y393','1977-11-05 04:09:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 45035,'1977-Y394','1977-11-05 07:00:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45036,'1977-Y395','1977-11-05 08:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 45037,'1977-Y396','1977-11-05 09:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 45038,'1977-Y397','1977-11-05 11:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 45039,'1977-Y398','1977-11-05 12:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 4504,'1957-S102','1957-08-31 04:57:00','suborbital rocket','success',839,'II6.23F','Auroral particles','IGY II6.23F','','Auroral',428,'',115 from dual union all
select 45040,'1977-Y399','1977-11-05 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',82 from dual union all
select 45041,'1977-Y40','1977-01-27 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 45042,'1977-Y400','1977-11-05 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 45043,'1977-Y401','1977-11-05 16:09:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45044,'1977-Y402','1977-11-05 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45045,'1977-Y403','1977-11-05 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45046,'1977-Y404','1977-11-05 23:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 45047,'1977-Y405','1977-11-07 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',82 from dual union all
select 45048,'1977-Y406','1977-11-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 45049,'1977-Y407','1977-11-09 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 4505,'1957-S103','1957-08-31 05:30:00','suborbital rocket','success',994,'No. 4','','','','Ionos/Bio',636,'',185 from dual union all
select 45050,'1977-Y408','1977-11-09 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',82 from dual union all
select 45051,'1977-Y409','1977-11-09 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 45052,'1977-Y41','1977-02-01 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 45053,'1977-Y410','1977-11-09 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',96 from dual union all
select 45054,'1977-Y411','1977-11-09 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45055,'1977-Y412','1977-11-09 18:02:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45056,'1977-Y413','1977-11-09 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 45057,'1977-Y414','1977-11-09 21:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',85 from dual union all
select 45058,'1977-Y415','1977-11-10 15:00:00','suborbital rocket','unknown',725,'ISRO 8.352','','','','Aeron',1467,'',86 from dual union all
select 45059,'1977-Y416','1977-11-11 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',86 from dual union all
select 4506,'1957-S104','1957-09-01 22:28:00','suborbital rocket','success',49,'AM4.001','Ion probe','IGY AM4.01','','Aeron',428,'',160 from dual union all
select 45060,'1977-Y417','1977-11-13 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',81 from dual union all
select 45061,'1977-Y418','1977-11-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 45062,'1977-Y419','1977-11-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 45063,'1977-Y42','1977-02-01 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 45064,'1977-Y420','1977-11-16 15:00:00','suborbital rocket','unknown',725,'ISRO 8.353','','','','Aeron',1467,'',86 from dual union all
select 45065,'1977-Y421','1977-11-16 16:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 45066,'1977-Y422','1977-11-16 17:03:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 45067,'1977-Y423','1977-11-16 17:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45068,'1977-Y424','1977-11-16 17:08:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45069,'1977-Y425','1977-11-17 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',85 from dual union all
select 4507,'1957-S105','1957-09-02 00:00:00','military missile','success',975,'M1-30','','','','Test',636,'',86 from dual union all
select 45070,'1977-Y426','1977-11-21 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',82 from dual union all
select 45071,'1977-Y427','1977-11-22 15:00:00','suborbital rocket','unknown',725,'ISRO 8.354','','','','Aeron',1467,'',82 from dual union all
select 45072,'1977-Y428','1977-11-22 16:50:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',84 from dual union all
select 45073,'1977-Y429','1977-11-23 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',86 from dual union all
select 45074,'1977-Y43','1977-02-02 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 45075,'1977-Y430','1977-11-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 45076,'1977-Y431','1977-11-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 45077,'1977-Y432','1977-11-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 45078,'1977-Y433','1977-11-23 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',84 from dual union all
select 45079,'1977-Y434','1977-11-23 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 4508,'1957-S106','1957-09-09 13:45:00','military missile','success',934,'1-204-6 Polaris jg.','Jetavator','','','Reentry Test',253,'',20 from dual union all
select 45080,'1977-Y435','1977-11-23 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45081,'1977-Y436','1977-11-23 19:02:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45082,'1977-Y437','1977-11-23 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45083,'1977-Y438','1977-11-24 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',86 from dual union all
select 45084,'1977-Y439','1977-11-24 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',88 from dual union all
select 45085,'1977-Y44','1977-02-02 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 45086,'1977-Y440','1977-11-28 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',85 from dual union all
select 45087,'1977-Y441','1977-11-28 15:00:00','suborbital rocket','unknown',725,'ISRO 8.355','','','','Aeron',1467,'',85 from dual union all
select 45088,'1977-Y442','1977-11-29 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',85 from dual union all
select 45089,'1977-Y443','1977-11-29 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',82 from dual union all
select 4509,'1957-S107','1957-09-07 11:39:00','military missile','success',1016,'M1-9','8K71 LKI I-4','','','Test',480,'',1350 from dual union all
select 45090,'1977-Y444','1977-11-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 45091,'1977-Y445','1977-11-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 45092,'1977-Y446','1977-11-30 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',85 from dual union all
select 45093,'1977-Y447','1977-11-30 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 45094,'1977-Y448','1977-11-30 16:09:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45095,'1977-Y449','1977-11-30 16:26:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 45096,'1977-Y45','1977-02-03 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',84 from dual union all
select 45097,'1977-Y450','1977-11-30 19:37:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45098,'1977-Y451','1977-11-30 21:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual;
insert into LAUNCH
select 45099,'1977-Y452','1977-11-30 22:09:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 451,'1944-S68','1944-08-30 11:25:00','military missile','success',8,'18299','Ma117','','','Test',676,'',93 from dual union all
select 4510,'1957-S108','1957-09-09 15:50:00','suborbital rocket','success',994,'No. 5','','','','Ionos/Bio',636,'',212 from dual union all
select 45100,'1977-Y453','1977-12-01 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',104 from dual union all
select 45101,'1977-Y454','1977-12-01 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',84 from dual union all
select 45102,'1977-Y455','1977-12-02 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',88 from dual union all
select 45103,'1977-Y456','1977-12-02 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',82 from dual union all
select 45104,'1977-Y457','1977-12-03 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',85 from dual union all
select 45105,'1977-Y458','1977-12-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 45106,'1977-Y459','1977-12-07 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 45107,'1977-Y46','1977-02-03 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',82 from dual union all
select 45108,'1977-Y460','1977-12-07 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45109,'1977-Y461','1977-12-09 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',85 from dual union all
select 4511,'1957-S109','1957-09-11 02:41:00','military missile','success',635,'CC-38    NL','Jupiter A','','','Test',280,'',90 from dual union all
select 45110,'1977-Y462','1977-12-10 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',83 from dual union all
select 45111,'1977-Y463','1977-12-11 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',82 from dual union all
select 45112,'1977-Y464','1977-12-14 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',80 from dual union all
select 45113,'1977-Y465','1977-12-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 45114,'1977-Y466','1977-12-14 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 45115,'1977-Y467','1977-12-14 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 45116,'1977-Y468','1977-12-14 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 45117,'1977-Y469','1977-12-14 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 45118,'1977-Y47','1977-02-03 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 45119,'1977-Y470','1977-12-15 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',83 from dual union all
select 4512,'1957-S11','1957-02-01 00:00:00','suborbital rocket','success',813,'','','','','Test',313,'',90 from dual union all
select 45120,'1977-Y471','1977-12-16 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',85 from dual union all
select 45121,'1977-Y472','1977-12-18 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',80 from dual union all
select 45122,'1977-Y473','1977-12-19 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',85 from dual union all
select 45123,'1977-Y474','1977-12-20 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',87 from dual union all
select 45124,'1977-Y475','1977-12-21 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 45125,'1977-Y476','1977-12-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 45126,'1977-Y477','1977-12-21 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 45127,'1977-Y478','1977-12-21 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',85 from dual union all
select 45128,'1977-Y479','1977-12-21 18:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 45129,'1977-Y48','1977-02-04 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',85 from dual union all
select 4513,'1957-S110','1957-09-19 16:30:00','suborbital rocket','success',28,'','Release A (Cs)','','','Aeron',640,'',100 from dual union all
select 45130,'1977-Y480','1977-12-22 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',87 from dual union all
select 45131,'1977-Y481','1977-12-26 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',84 from dual union all
select 45132,'1977-Y482','1977-12-27 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 45133,'1977-Y483','1977-12-27 07:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 45134,'1977-Y484','1977-12-27 21:17:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 45135,'1977-Y485','1977-12-27 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 45136,'1977-Y486','1977-12-28 10:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 45137,'1977-Y487','1977-12-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 45138,'1977-Y488','1977-12-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 45139,'1977-Y489','1977-12-28 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 4514,'1957-S111','1957-09-20 14:25:00','military missile','success',1366,'105','R&D Series I','','','Test',234,'',520 from dual union all
select 45140,'1977-Y49','1977-02-04 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',86 from dual union all
select 45141,'1977-Y490','1977-12-28 18:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 45142,'1977-Y491','1977-12-29 13:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 45143,'1977-Y492','1977-12-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 45144,'1977-Y493','1977-12-29 16:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 45145,'1977-Y494','1977-12-29 17:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 45146,'1977-Y50','1977-02-06 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',84 from dual union all
select 45147,'1977-Y51','1977-02-06 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1374,'SHIR',86 from dual union all
select 45148,'1977-Y52','1977-02-08 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 45149,'1977-Y53','1977-02-08 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 4515,'1957-S112','1957-08-21 00:00:00','suborbital rocket','success',28,'USAF 84','NO probe','','','Aeron',640,'',100 from dual union all
select 45150,'1977-Y54','1977-02-09 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 45151,'1977-Y55','1977-02-09 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 45152,'1977-Y56','1977-02-09 19:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',86 from dual union all
select 45153,'1977-Y57','1977-02-09 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',88 from dual union all
select 45154,'1977-Y58','1977-02-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 45155,'1977-Y59','1977-02-10 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 45156,'1977-Y60','1977-02-10 23:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 45157,'1977-Y61','1977-02-11 20:50:45','suborbital rocket','success',1281,'NASA 15.154UE','TRIGGER AR-1','','','Auroral',417,'',80 from dual union all
select 45158,'1977-Y62','1977-02-12 00:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 45159,'1977-Y63','1977-02-15 18:48:20','suborbital rocket','success',1281,'NASA 15.155UE','TRIGGER AR-2','','','Auroral',417,'',85 from dual union all
select 4516,'1957-S113','1957-09-26 18:21:00','suborbital rocket','success',718,'II5.50F','SUI 74','IGY II5.50F','','Test',698,'AGB-4',100 from dual union all
select 45160,'1977-Y64','1977-02-15 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 45161,'1977-Y65','1977-02-15 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 45162,'1977-Y66','1977-02-16 10:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 45163,'1977-Y67','1977-02-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',83 from dual union all
select 45164,'1977-Y68','1977-02-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 45165,'1977-Y69','1977-02-17 21:12:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 45166,'1977-Y70','1977-02-18 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',92 from dual union all
select 45167,'1977-Y71','1977-02-18 18:35:59','suborbital rocket','success',1061,'','','','','Meteo',1057,'',86 from dual union all
select 45168,'1977-Y72','1977-02-22 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 45169,'1977-Y73','1977-02-22 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 4517,'1957-S114','1957-09-27 14:27:00','suborbital rocket','success',718,'II5.51F','SUI 75','IGY II5.51F','','CR',709,'AGB-4',119 from dual union all
select 45170,'1977-Y74','1977-02-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 45171,'1977-Y75','1977-02-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 45172,'1977-Y76','1977-02-23 21:03:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 45173,'1977-Y77','1977-02-24 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',95 from dual union all
select 45174,'1977-Y78','1977-02-24 23:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',82 from dual union all
select 45175,'1977-Y79','1977-02-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 45176,'1977-Y80','1977-02-27 05:20:00','suborbital rocket','success',229,'AQF-VI-05','Non-acoustic waves','','','Aeron',428,'',95 from dual union all
select 45177,'1977-Y81','1977-03-01 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 45178,'1977-Y82','1977-03-02 10:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 45179,'1977-Y83','1977-03-02 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 4518,'1957-S115','1957-10-01 00:00:00','suborbital rocket','success',720,'LT1','','','','Test',1839,'',100 from dual union all
select 45180,'1977-Y84','1977-03-02 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',86 from dual union all
select 45181,'1977-Y85','1977-03-02 18:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 45182,'1977-Y86','1977-03-02 20:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 45183,'1977-Y87','1977-03-02 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 45184,'1977-Y88','1977-03-03 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 45185,'1977-Y89','1977-03-04 15:25:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',96 from dual union all
select 45186,'1977-Y90','1977-03-04 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 45187,'1977-Y91','1977-03-08 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 45188,'1977-Y92','1977-03-08 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual;
insert into LAUNCH
select 45189,'1977-Y93','1977-03-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',94 from dual union all
select 4519,'1957-S116','1957-10-02 19:29:00','military missile','success',635,'CC-39    NE','Jupiter A','','','Test',280,'',90 from dual union all
select 45190,'1977-Y94','1977-03-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 45191,'1977-Y95','1977-03-09 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 45192,'1977-Y96','1977-03-10 21:06:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 45193,'1977-Y97','1977-03-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',90 from dual union all
select 45194,'1977-Y98','1977-03-14 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',884,'KORO',84 from dual union all
select 45195,'1977-Y99','1977-03-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',92 from dual union all
select 45196,'1978-001','1978-01-06 15:50:00','orbital','success',1238,'77052  -702','Kosmos-974','Zenit-4MKM','','Sat',474,'',null from dual union all
select 45197,'1978-002','1978-01-07 00:15:00','orbital','success',180,'AC-46','Intelsat 4A F-3','INTELSAT IVA F3','','Sat',269,'',null from dual union all
select 45198,'1978-003','1978-01-10 12:26:00','orbital','success',1238,'D 15000-106','Soyuz-27','Soyuz 7K-T No. 44','','Sat',480,'',null from dual union all
select 45199,'1978-004','1978-01-10 13:23:00','orbital','success',1510,'78026  -394','Kosmos-975','Tselina-D','','Sat',475,'',null from dual union all
select 452,'1944-S69','1944-08-30 15:44:00','military missile','success',8,'18334','Ma154','','','Training',661,'',90 from dual union all
select 4520,'1957-S117','1957-10-03 00:00:00','sounding rocket','success',488,'','Shot 2','','','Test/Ionos',405,'Balloon',800 from dual union all
select 45200,'1978-005','1978-01-10 20:51:00','orbital','success',680,'53716-319','Kosmos-976/983','Strela-1M','','Sat',457,'',null from dual union all
select 45201,'1978-006','1978-01-13 15:15:00','orbital','success',1238,'78018  -106','Kosmos-984','Zenit-2M','','Sat',474,'',null from dual union all
select 45202,'1978-007','1978-01-17 03:26:00','orbital','success',680,'53716-327','Kosmos-985','Parus','','Sat',456,'',null from dual union all
select 45203,'1978-008','1978-01-20 08:24:40','orbital','success',1238,'E 15000-075','Progress-1','Progress 7K-TG No. 102','','Sat',562,'',null from dual union all
select 45204,'1978-009','1978-01-24 06:51:00','orbital','success',809,'78026  -596','Molniya-3','Molniya-3 No. 20','','Sat',474,'',null from dual union all
select 45205,'1978-010','1978-01-24 09:50:00','orbital','success',1238,'77052  -700','Kosmos-986','Zenit-4MKM','','Sat',562,'',null from dual union all
select 45206,'1978-011','1978-01-26 05:00:00','orbital','success',301,'CZ2C-3   (6)','FSW','FSW No. 4','','Sat',810,'',null from dual union all
select 45207,'1978-012','1978-01-26 17:36:00','orbital','success',386,'628/D138','IUE','IUE','','Sat',233,'',null from dual union all
select 45208,'1978-013','1978-01-31 14:50:00','orbital','success',1238,'77052  -703','Kosmos-987','Zenit-4MKM','','Sat',475,'',null from dual union all
select 45209,'1978-014','1978-02-04 07:00:00','orbital','success',818,'M-3H-2','Kyokko/KM -A','EXOS-A','','Sat',835,'',null from dual union all
select 4521,'1957-S118','1957-10-04 20:36:00','suborbital rocket','success',718,'II5.52F','SUI 76','IGY II5.52F','','CR',720,'AGB-4',113 from dual union all
select 45210,'1978-015','1978-02-08 12:15:00','orbital','success',1238,'77052  -875','Kosmos-988','Orion','','Sat',474,'',null from dual union all
select 45211,'1978-016','1978-02-09 21:17:01','orbital','success',180,'AC-44','FLTSATCOM 1','FLTSATCOM F1','','Sat',268,'',null from dual union all
select 45212,'1978-017','1978-02-14 09:30:00','orbital','success',1238,'77052  -701','Kosmos-989','Zenit-4MKM','','Sat',562,'',null from dual union all
select 45213,'1978-018','1978-02-16 04:00:00','orbital','success',822,'N-4(F)','Ume 2','ISS-b','','Sat',1474,'',null from dual union all
select 45214,'1978-019','1978-02-17 16:33:00','orbital','success',680,'53746-312','Kosmos-990','Strela-2M','','Sat',457,'',null from dual union all
select 45215,'1978-020','1978-02-22 23:44:00','orbital','success',167,'64F','GPS 1','Navstar GPS SVN 1','','Sat',1775,'',null from dual union all
select 45216,'1978-021','1978-02-25 05:00:00','orbital','success',1405,'34B-2  (3B-49)','OPS 6031','Jumpseat 5','','Sat',1778,'',null from dual union all
select 45217,'1978-022','1978-02-28 06:43:00','orbital','success',680,'53716-323','Kosmos-991','Parus','','Sat',457,'',null from dual union all
select 45218,'1978-023','1978-03-02 15:28:10','orbital','success',1238,'D 15000-132','Soyuz-28','Soyuz 7K-T No. 45','','Sat',480,'',null from dual union all
select 45219,'1978-024','1978-03-02 22:07:00','orbital','success',809,'76041  -581','Molniya-1','Molniya-1','','Sat',474,'',null from dual union all
select 4522,'1957-S119','1957-10-07 00:00:00','sounding rocket','failure',488,'','Shot 3','','','Test/Ionos',405,'Balloon',700 from dual union all
select 45220,'1978-025','1978-03-04 07:40:00','orbital','success',1238,'76042  -242','Kosmos-992','Zenit-2M','','Sat',562,'',null from dual union all
select 45221,'1978-026','1978-03-05 17:54:00','orbital','success',383,'621/D139','Landsat 3','Landsat C','','Sat',1724,'',null from dual union all
select 45222,'1978-027','1978-03-10 10:42:00','orbital','success',1238,'77052  -704','Kosmos-993','Zenit-4MKM','','Sat',474,'',null from dual union all
select 45223,'1978-028','1978-03-15 15:57:00','orbital','success',680,'53746-315','Kosmos-994','Zaliv','','Sat',457,'',null from dual union all
select 45224,'1978-029','1978-03-16 18:40:00','orbital','success',1426,'23D-20','HEXAGON SV14','HEXAGON SV-14','','Sat',1777,'',null from dual union all
select 45225,'1978-030','1978-03-17 10:50:00','orbital','success',1238,'78018  -107','Kosmos-995','Zenit-2M','','Sat',474,'',null from dual union all
select 45226,'1978-031','1978-03-28 01:30:00','orbital','success',680,'53746-314','Kosmos-996','Parus','','Sat',456,'',null from dual union all
select 45227,'1978-032','1978-03-30 00:00:00','orbital','success',949,'292-01','Kosmos-997/998','TKS VA No. 009A/P2','','Sat',588,'',null from dual union all
select 45228,'1978-033','1978-03-30 07:50:00','orbital','success',1238,'78018  -705','Kosmos-999','Zenit-4MKM','','Sat',562,'',null from dual union all
select 45229,'1978-034','1978-03-31 14:01:00','orbital','success',680,'53768-307','Kosmos-1000','Tsikada','','Sat',456,'',null from dual union all
select 4523,'1957-S12','1957-02-02 08:05:00','suborbital rocket','success',48,'USAF 76','Firefly (Ethelyne)','','','Aeronomy release',640,'',143 from dual union all
select 45230,'1978-035','1978-03-31 23:36:01','orbital','success',180,'AC-48','Intelsat 4A F-6','INTELSAT IVA F6','','Sat',269,'',null from dual union all
select 45231,'1978-036','1978-04-04 15:00:00','orbital','success',1238,'D 15000-123','Kosmos-1001','Soyuz 7K-ST No. 4L','','Sat',480,'',null from dual union all
select 45232,'1978-037','1978-04-06 09:10:00','orbital','success',1238,'78018  -109','Kosmos-1002','Zenit-2M','','Sat',562,'',null from dual union all
select 45233,'1978-038','1978-04-07 00:45:01','orbital','success',178,'5505A','OPS 8790','AQUACADE 4','','Sat',228,'',null from dual union all
select 45234,'1978-039','1978-04-07 22:01:00','orbital','success',386,'626/D140','BSE','BSE','','Sat',234,'',null from dual union all
select 45235,'1978-040','1978-04-20 15:30:00','orbital','success',1238,'78018  -706','Kosmos-1003','Zenit-4MKM','','Sat',475,'',null from dual union all
select 45236,'1978-041','1978-04-26 10:20:00','orbital','success',1132,'S201C','HCMM','AEM-A','','Sat',1779,'',null from dual union all
select 45237,'1978-042','1978-05-01 03:05:04','orbital','success',1374,'143','DMSP 5D F-3','DMSP Block 5D S-3','','Sat',1720,'',null from dual union all
select 45238,'1978-043','1978-05-05 15:30:00','orbital','success',1238,'78018  -108','Kosmos-1004','Zenit-2M','','Sat',474,'',null from dual union all
select 45239,'1978-044','1978-05-11 22:59:00','orbital','success',390,'627/D141','OTS 2','OTS 2','','Sat',233,'',null from dual union all
select 4524,'1957-S120','1957-10-11 16:33:00','military missile','success',1366,'108','R&D Series I','','','Test',234,'',520 from dual union all
select 45240,'1978-045','1978-05-12 04:07:00','orbital','success',1510,'76038  -309','Kosmos-1005','Tselina-D','','Sat',475,'',null from dual union all
select 45241,'1978-046','1978-05-12 11:00:00','orbital','success',680,'53746-318','Kosmos-1006','Vektor','','Sat',456,'',null from dual union all
select 45242,'1978-047','1978-05-13 10:34:00','orbital','success',167,'49F','GPS 2','Navstar GPS SVN 2','','Sat',1775,'',null from dual union all
select 45243,'1978-048','1978-05-16 10:40:00','orbital','success',1238,'78018  -707','Kosmos-1007','Zenit-4MKM','','Sat',474,'',null from dual union all
select 45244,'1978-049','1978-05-17 14:39:00','orbital','success',680,'53798-316','Kosmos-1008','Tselina-OK','','Sat',456,'',null from dual union all
select 45245,'1978-050','1978-05-19 00:21:00','orbital','success',1446,'51053-303T','Kosmos-1009','IS','','Sat',589,'',null from dual union all
select 45246,'1978-051','1978-05-20 13:13:00','deep space','success',180,'AC-50','Pioneer Venus 1','Pioneer Venus Orbiter','','Sat',268,'',null from dual union all
select 45247,'1978-052','1978-05-23 07:30:00','orbital','success',1238,'77025  -046','Kosmos-1010','Fram','','Sat',475,'',null from dual union all
select 45248,'1978-053','1978-05-23 16:57:00','orbital','success',680,'53768-310','Kosmos-1011','Parus','','Sat',456,'',null from dual union all
select 45249,'1978-054','1978-05-25 14:30:00','orbital','success',1238,'76038  -112','Kosmos-1012','Zenit-2M','','Sat',474,'',null from dual union all
select 4525,'1957-S121','1957-10-13 18:15:00','suborbital rocket','success',718,'II5.53F','SUI 77','IGY II5.53F','','CR',728,'AGB-4',100 from dual union all
select 45250,'1978-055','1978-06-02 12:12:00','orbital','success',809,'78026  -595','Molniya-1','Molniya-1','','Sat',474,'',null from dual union all
select 45251,'1978-056','1978-06-07 22:00:00','orbital','success',680,'53716-321','Kosmos-1013/1020','Strela-1M','','Sat',456,'',null from dual union all
select 45252,'1978-057','1978-06-10 08:35:00','orbital','success',1238,'78018  -711','Kosmos-1021','Zenit-4MKM','','Sat',562,'',null from dual union all
select 45253,'1978-058','1978-06-10 19:08:00','orbital','success',1425,'23C-15 (3C-33)','OPS 9454','CHALET 1','','Sat',274,'',null from dual union all
select 45254,'1978-059','1978-06-12 10:30:00','orbital','success',1238,'78018  -708','Kosmos-1022','Zenit-4MKM','','Sat',474,'',null from dual union all
select 45255,'1978-060','1978-06-14 18:23:00','orbital','success',1426,'23D-18','KH-11','KH11-2 KENNAN','','Sat',1777,'',null from dual union all
select 45256,'1978-061','1978-06-15 20:16:45','orbital','success',1238,'D 15000-133','Soyuz-29','Soyuz 7K-T No. 46','','Sat',480,'',null from dual union all
select 45257,'1978-062','1978-06-16 10:49:00','orbital','success',386,'625/D142','GOES 3','GOES C','','Sat',234,'',null from dual union all
select 45258,'1978-063','1978-06-21 09:27:00','orbital','success',680,'53716-317','Kosmos-1023','Strela-2M','','Sat',456,'',null from dual union all
select 45259,'1978-064','1978-06-27 01:12:00','orbital','success',163,'23F','Seasat','Seasat A','','Sat',1776,'',null from dual union all
select 4526,'1957-S122','1957-10-14 15:08:00','suborbital rocket','success',28,'USAF 87','','','','Meteorites',640,'',146 from dual union all
select 45260,'1978-065','1978-06-27 15:27:21','orbital','success',1238,'Ye15000-147','Soyuz-30','Soyuz 7K-T(A9) No. 67','','Sat',480,'',null from dual union all
select 45261,'1978-066','1978-06-28 02:58:00','orbital','success',809,'77052  -608','Kosmos-1024','Oko','','Sat',474,'',null from dual union all
select 45262,'1978-067','1978-06-28 17:35:00','orbital','success',1448,'51035-501','Kosmos-1025','EPN No. 03.0380?','','Sat',470,'',null from dual union all
select 45263,'1978-068','1978-06-29 22:24:59','orbital','success',180,'AC-41','Comstar D-3','Comstar D-3','','Sat',269,'',null from dual union all
select 45264,'1978-069','1978-07-02 09:30:00','orbital','success',1238,'76038  -113','Kosmos-1026','Energiya No. 2','','Sat',562,'',null from dual union all
select 45265,'1978-070','1978-07-07 11:26:16','orbital','success',1238,'S 15000-128','Progress-2','Progress 7K-TG No. 101','','Sat',562,'',null from dual union all
select 45266,'1978-071','1978-07-14 10:43:00','orbital','success',386,'631/D143','Geos 2','Geos 2','','Sat',233,'',null from dual union all
select 45267,'1978-072','1978-07-14 15:00:00','orbital','success',809,'76038  -508','Molniya-1','Molniya-1','','Sat',475,'',null from dual union all
select 45268,'1978-073','1978-07-18 21:59:00','orbital','success',956,'292-02','Raduga','Gran'' No. 14L','','Sat',554,'',null from dual union all
select 45269,'1978-074','1978-07-27 04:49:00','orbital','success',680,'53716-313','Kosmos-1027','Zaliv','','Sat',457,'',null from dual union all
select 4527,'1957-S123','1957-10-14 22:31:00','suborbital rocket','success',718,'II5.54F','SUI 78','IGY II5.54F','','Aeron/Fields',729,'AGB-4',113 from dual union all
select 45270,'1978-075','1978-08-05 05:00:00','orbital','success',1405,'34B-7  (3B-57)','OPS 7310','SDS 3','','Sat',1778,'',null from dual union all
select 45271,'1978-076','1978-08-05 15:00:00','orbital','success',1238,'77025  -295','Kosmos-1028','Feniks No. 14','','Sat',474,'',null from dual union all
select 45272,'1978-077','1978-08-07 22:31:22','orbital','success',1238,'Ye15000-138','Progress-3','Progress 7K-TG No. 103','','Sat',562,'',null from dual union all
select 45273,'1978-078','1978-08-08 07:33:00','deep space','success',180,'AC-51','Pioneer Venus 2','Pioneer Venus Multiprobe','','Sat',268,'',null from dual union all
select 45274,'1978-079','1978-08-12 15:12:00','deep space','success',386,'633/D144','ISEE 3','ISEE C','','Sat',234,'',null from dual union all
select 45275,'1978-080','1978-08-22 23:47:00','orbital','success',809,'78018  -501','Molniya-1','Molniya-1','','Sat',475,'',null from dual union all
select 45276,'1978-081','1978-08-26 14:51:30','orbital','success',1238,'Ye15000-159','Soyuz-31','Soyuz 7K-T No. 47','','Sat',480,'',null from dual union all
select 45277,'1978-082','1978-08-29 15:00:00','orbital','success',1238,'78018  -709','Kosmos-1029','Zenit-4MKM','','Sat',475,'',null from dual union all
select 45278,'1978-083','1978-09-06 03:04:00','orbital','success',809,'76038  -507','Kosmos-1030','Oko','','Sat',475,'',null from dual;
insert into LAUNCH
select 45279,'1978-084','1978-09-09 03:25:39','deep space','success',954,'296-01','Venera-11','4V-1 No. 360','','Sat',587,'',null from dual union all
select 4528,'1957-S124','1957-10-16 21:17:00','suborbital rocket','success',718,'II5.55F','SUI 79','IGY II5.55F','','Aeron/Fields',730,'AGB-4',100 from dual union all
select 45280,'1978-085','1978-09-09 15:00:00','orbital','success',1238,'76038  -716 ','Kosmos-1031','Zenit-4MKM','','Sat',473,'',null from dual union all
select 45281,'1978-086','1978-09-14 02:25:13','deep space','success',954,'296-02','Venera-12','4V-1 No. 361','','Sat',588,'',null from dual union all
select 45282,'1978-087','1978-09-16 05:00:00','orbital','success',818,'M-3H-3','Jikiken/KM -B','EXOS-B','','Sat',835,'',null from dual union all
select 45283,'1978-088','1978-09-19 08:05:00','orbital','success',1238,'77025  -124','Kosmos-1032','Zenit-2M','','Sat',474,'',null from dual union all
select 45284,'1978-089','1978-10-03 11:00:00','orbital','success',1238,'77025  -047','Kosmos-1033','Fram','','Sat',474,'',null from dual union all
select 45285,'1978-090','1978-10-03 23:09:30','orbital','success',1238,'Ye15000-152','Progress-4','Progress 7K-TG No. 105','','Sat',480,'',null from dual union all
select 45286,'1978-091','1978-10-04 03:49:00','orbital','success',680,'53782-429','Kosmos-1034/1041','Strela-1M','','Sat',456,'',null from dual union all
select 45287,'1978-092','1978-10-06 15:30:00','orbital','success',1238,'76038  -715','Kosmos-1042','Zenit-4MKM','','Sat',474,'',null from dual union all
select 45288,'1978-093','1978-10-07 00:28:00','orbital','success',167,'47F','GPS 3','Navstar GPS SVN 3','','Sat',1775,'',null from dual union all
select 45289,'1978-094','1978-10-10 19:44:00','orbital','success',1510,'76038  -310','Kosmos-1043','Tselina-D','','Sat',475,'',null from dual union all
select 4529,'1957-S125','1957-10-17 00:09:00','suborbital rocket','failure',718,'II5.56F','SUI 80','IGY II5.56F','','Aeron/Fields',731,'AGB-4',0 from dual union all
select 45290,'1978-095','1978-10-13 05:19:00','orbital','success',809,'76038  -511','Molniya-3','Molniya-3 No. 22','','Sat',474,'',null from dual union all
select 45291,'1978-096','1978-10-13 11:23:00','orbital','success',162,'29F','Tiros N','Tiros-N','','Sat',1776,'',null from dual union all
select 45292,'1978-097','1978-10-17 15:00:00','orbital','success',1238,'77025  -125','Kosmos-1044','Zenit-2M','','Sat',475,'',null from dual union all
select 45293,'1978-098','1978-10-24 08:15:00','orbital','success',383,'630/D145','Nimbus 7','Nimbus G','','Sat',1724,'',null from dual union all
select 45294,'1978-099','1978-10-24 19:00:00','orbital','success',680,'65055-106','Interkosmos-18','AUOS-Z Mag-IK','','Sat',456,'',null from dual union all
select 45295,'1978-100','1978-10-26 07:00:00','orbital','success',1448,'67055-502','Kosmos-1045','Meteor-2 GVM','','Sat',470,'',null from dual union all
select 45296,'1978-101','1978-10-30 05:23:00','orbital','success',807,'D 15000-173/801','Prognoz-7','SO-M No. 507','','Sat',480,'',null from dual union all
select 45297,'1978-102','1978-11-01 12:00:00','orbital','success',1238,'78026  -864','Kosmos-1046','Orion','','Sat',473,'',null from dual union all
select 45298,'1978-103','1978-11-13 05:24:00','orbital','success',180,'AC-52','HEAO 2','HEAO B','','Sat',269,'',null from dual union all
select 45299,'1978-104','1978-11-15 11:45:00','orbital','success',1238,'78038  -714','Kosmos-1047','Zenit-4MKM','','Sat',473,'',null from dual union all
select 453,'1944-S70','1944-08-30 16:10:00','military missile','success',8,'18338','Ma137','','','Training',661,'',90 from dual union all
select 4530,'1957-S126','1957-10-17 02:18:00','suborbital rocket','failure',718,'II5.57F','SUI 81','IGY II5.57F','','Aeron/Fields',732,'AGB-4',10 from dual union all
select 45300,'1978-105','1978-11-16 21:45:00','orbital','success',680,'53755-111','Kosmos-1048','Strela-2M','','Sat',457,'',null from dual union all
select 45301,'1978-106','1978-11-19 00:46:00','orbital','success',386,'634/D146','NATO 3C','NATO 3C','','Sat',234,'',null from dual union all
select 45302,'1978-107','1978-11-21 12:00:00','orbital','success',1238,'77025  -729','Kosmos-1049','Zenit-4MKM','','Sat',475,'',null from dual union all
select 45303,'1978-108','1978-11-28 16:20:00','orbital','success',1238,'D 15000-086','Kosmos-1050','Zenit-6','','Sat',475,'',null from dual union all
select 45304,'1978-109','1978-12-05 18:12:00','orbital','success',680,'53716-320','Kosmos-1051/1058','Strela-1M','','Sat',457,'',null from dual union all
select 45305,'1978-110','1978-12-07 15:30:00','orbital','success',1238,'78038  -717','Kosmos-1059','Zenit-4MKM','','Sat',473,'',null from dual union all
select 45306,'1978-111','1978-12-08 09:30:00','orbital','success',1238,'E 15000-144','Kosmos-1060','Zenit-2M','','Sat',562,'',null from dual union all
select 45307,'1978-112','1978-12-11 03:59:00','orbital','success',167,'39F','GPS 4','Navstar GPS SVN 4','','Sat',1775,'',null from dual union all
select 45308,'1978-113','1978-12-14 00:40:02','orbital','success',1425,'23C-18 (3C-36)','DSCS II 11/12','DSCS II C-11','','Sat',274,'',null from dual union all
select 45309,'1978-114','1978-12-14 15:20:00','orbital','success',1238,'78055  -129','Kosmos-1061','Zenit-2M','','Sat',475,'',null from dual union all
select 4531,'1957-S128','1957-10-18 00:59:00','suborbital rocket','success',718,'II5.59F','SUI 83','IGY II5.59F','','Fields',699,'AGB-4',127 from dual union all
select 45310,'1978-115','1978-12-15 13:19:00','orbital','success',680,'53782-408','Kosmos-1062','Tselina-OM','','Sat',457,'',null from dual union all
select 45311,'1978-116','1978-12-16 00:21:00','orbital','success',390,'632/D147','Anik B','Anik B','','Sat',233,'',null from dual union all
select 45312,'1978-117','1978-12-19 01:35:00','orbital','success',1510,'76038  -311','Kosmos-1063','Tselina-D','','Sat',475,'',null from dual union all
select 45313,'1978-118','1978-12-19 12:15:00','orbital','success',956,'295-02','Gorizont','Gorizont No. 11L','','Sat',554,'',null from dual union all
select 45314,'1978-119','1978-12-20 20:43:00','orbital','failure',680,'47182-425','Kosmos-1064','Parus','','Sat',456,'',null from dual union all
select 45315,'1978-120','1978-12-22 22:00:00','orbital','success',680,'65068-305','Kosmos-1065','Romb 5','','Sat',622,'',null from dual union all
select 45316,'1978-121','1978-12-23 08:39:00','orbital','success',1510,'78026  -384','Kosmos-1066','Astrofizika 11F653 No. 1','','Sat',474,'',null from dual union all
select 45317,'1978-122','1978-12-26 13:30:00','orbital','success',680,'53782-430','Kosmos-1067','Geoid','','Sat',457,'',null from dual union all
select 45318,'1978-123','1978-12-26 15:30:00','orbital','success',1238,'76038  -718','Kosmos-1068','Zenit-4MKM','','Sat',475,'',null from dual union all
select 45319,'1978-124','1978-12-28 16:30:00','orbital','success',1238,'77052  -876','Kosmos-1069','Orion','','Sat',473,'',null from dual union all
select 4532,'1957-S129','1957-10-18 09:35:00','suborbital rocket','success',718,'II5.60F','SUI 84','IGY II5.60F','','CR',700,'AGB-4',10 from dual union all
select 45320,'1978-A01','1978-01-04 00:00:00','suborbital rocket','failure',1281,'NASA 15.160UE','','','','Plasma',1409,'',1 from dual union all
select 45321,'1978-A02','1978-01-26 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',44 from dual union all
select 45322,'1978-A03','1978-01-27 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',46 from dual union all
select 45323,'1978-A04','1978-01-27 21:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',49 from dual union all
select 45324,'1978-A05','1978-02-04 08:20:00','suborbital rocket','failure',41,'NASA 26.063UH','Wisc. Aerobee IV','','','XR Astron',1895,'',29 from dual union all
select 45325,'1978-A06','1978-02-17 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',46 from dual union all
select 45326,'1978-A07','1978-02-20 16:15:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',47 from dual union all
select 45327,'1978-A08','1978-03-01 12:00:00','military missile','success',1099,'Case 34','','','','Test',1321,'B-52G',10 from dual union all
select 45328,'1978-A09','1978-03-07 00:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 45329,'1978-A10','1978-03-13 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',44 from dual union all
select 4533,'1957-S13','1957-02-07 13:00:00','suborbital rocket','success',1521,'R-21','4203-21','','','Reentry Test',253,'',107 from dual union all
select 45330,'1978-A11','1978-03-24 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 45331,'1978-A12','1978-03-27 21:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 45332,'1978-A13','1978-03-31 21:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',45 from dual union all
select 45333,'1978-A14','1978-04-05 11:58:00','suborbital rocket','failure',846,'GL A11.712-3','DENSITY sphere','','','Aeron',873,'',26 from dual union all
select 45334,'1978-A15','1978-04-15 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 45335,'1978-A16','1978-04-19 01:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 45336,'1978-A17','1978-04-20 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 45337,'1978-A18','1978-04-21 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 45338,'1978-A19','1978-04-27 03:10:00','military missile','failure',745,'51035-501(4L)','15A16 LKI-4','GCh 15F161','','OT',524,'',200 from dual union all
select 45339,'1978-A20','1978-05-04 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 4534,'1957-S130','1957-10-18 20:58:00','suborbital rocket','success',718,'II5.61F','SUI 85','IGY II5.61F','','Aeron/Fields',701,'AGB-4',121 from dual union all
select 45340,'1978-A21','1978-05-19 22:01:00','suborbital rocket','success',885,'2','4x9m','','','Test',1355,'',9 from dual union all
select 45341,'1978-A22','1978-05-23 00:00:00','suborbital rocket','failure',487,'F.1','CHEVALINE F1','','','Test',1836,'',2 from dual union all
select 45342,'1978-A23','1978-05-23 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 45343,'1978-A24','1978-05-24 20:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 45344,'1978-A25','1978-05-26 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 45345,'1978-A26','1978-06-02 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',47 from dual union all
select 45346,'1978-A27','1978-06-05 11:40:00','suborbital rocket','failure',885,'3a','4x15m','','','Test',1355,'',0 from dual union all
select 45347,'1978-A28','1978-06-26 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 45348,'1978-A29','1978-06-28 04:00:00','suborbital rocket','failure',599,'INTA IT.SIB-7801','Martelli','','','Aeron',396,'',25 from dual union all
select 45349,'1978-A30','1978-06-28 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 4535,'1957-S131','1957-10-19 00:59:00','suborbital rocket','success',718,'II5.62F','SUI 86','IGY II5.62F','','Fields',702,'AGB-4',121 from dual union all
select 45350,'1978-A31','1978-06-30 13:00:00','military missile','failure',1006,'67035-502(6L)','15A18 LKI-4','GCh 15F183','','OT',487,'',10 from dual union all
select 45351,'1978-A32','1978-07-07 16:18:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',46 from dual union all
select 45352,'1978-A33','1978-07-10 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',47 from dual union all
select 45353,'1978-A34','1978-07-20 11:49:00','military missile','success',1099,'Case 30','','','','Test',453,'B-52',10 from dual union all
select 45354,'1978-A35','1978-07-26 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',39 from dual union all
select 45355,'1978-A36','1978-08-15 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 45356,'1978-A37','1978-08-18 21:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 45357,'1978-A38','1978-08-21 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',44 from dual union all
select 45358,'1978-A39','1978-08-23 17:31:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',39 from dual union all
select 45359,'1978-A40','1978-08-24 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 4536,'1957-S132','1957-10-19 20:07:00','suborbital rocket','success',718,'SUI 87','II5.63F','IGY II5.63F','','Fields',703,'AGB-4',122 from dual union all
select 45360,'1978-A41','1978-08-28 15:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',48 from dual union all
select 45361,'1978-A42','1978-08-29 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 45362,'1978-A43','1978-09-05 20:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 45363,'1978-A44','1978-09-08 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',44 from dual union all
select 45364,'1978-A46','1978-09-13 00:53:00','suborbital rocket','success',852,'GL A45.709-1','SMOKE','','','Aeron',428,'',45 from dual union all
select 45365,'1978-A47','1978-09-13 11:13:00','suborbital rocket','success',852,'GL A45.709-2','SMOKE','','','Aeron',428,'',45 from dual union all
select 45366,'1978-A48','1978-09-18 20:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 45367,'1978-A49','1978-09-29 03:50:00','military missile','failure',1006,'67048-33-204(3L)','15A18 LKI-7','GCh 15F183','','OT',485,'',1000 from dual union all
select 45368,'1978-A50','1978-10-10 20:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual;
insert into LAUNCH
select 45369,'1978-A51','1978-10-19 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',36 from dual union all
select 4537,'1957-S133','1957-10-20 00:00:00','sounding rocket','success',488,'','Shot 5','','','Test/Ionos',405,'Balloon',5000 from dual union all
select 45370,'1978-A52','1978-11-09 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',43 from dual union all
select 45371,'1978-A54','1978-11-29 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 45372,'1978-A55','1978-12-13 11:58:00','military missile','success',1099,'Case 33','','','','Test',1901,'B-52',10 from dual union all
select 45373,'1978-A56','1978-12-16 17:43:00','military missile','failure',1442,'C4X-20','','','','Test',246,'',1 from dual union all
select 45374,'1978-A57','1978-12-22 21:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 45375,'1978-A58','1978-12-27 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',49 from dual union all
select 45376,'1978-A59','1978-01-01 00:00:00','military missile','success',897,'PRS-1','Test','','','Test',183,'',40 from dual union all
select 45377,'1978-A60','1978-04-29 00:00:00','military missile','success',903,'1','NHK-1','','','Test',33,'',20 from dual union all
select 45378,'1978-A61','1978-05-01 00:00:00','military missile','failure',903,'2','NHK-1','','','Test',33,'',20 from dual union all
select 45379,'1978-A62','1978-04-01 00:00:00','military missile','failure',903,'3','NHK-1','','','Test',33,'',20 from dual union all
select 4538,'1957-S134','1957-10-20 02:19:00','suborbital rocket','success',718,'SUI 88','II5.64F','IGY II5.64F','','Aeron/Fields',704,'AGB-4',104 from dual union all
select 45380,'1978-A63','1978-04-01 00:00:00','military missile','success',903,'4','NHK-1','','','Test',33,'',20 from dual union all
select 45381,'1978-A64','1978-07-01 00:00:00','military missile','failure',903,'5','NHK-1','','','Test',33,'',20 from dual union all
select 45382,'1978-A65','1978-07-01 00:00:00','military missile','success',903,'6','NHK-1','','','Test',33,'',20 from dual union all
select 45383,'1978-A66','1978-07-01 00:00:00','military missile','failure',903,'7','NHK-1','','','Test',33,'',20 from dual union all
select 45384,'1978-A67','1978-09-16 00:00:00','military missile','success',903,'8','NHK-1','','','Test',33,'',20 from dual union all
select 45385,'1978-A68','1978-09-26 00:00:00','military missile','success',903,'9','NHK-1','','','Test',33,'',40 from dual union all
select 45386,'1978-A69','1978-03-02 00:00:00','military missile','success',702,'Test firing','','','','Test',984,'',10 from dual union all
select 45387,'1978-F01','1978-03-25 18:09:00','orbital','failure',1425,'23C-17 (3C-35)','DSCS II 9/10','DSCS II C-9','','Sat',274,'',150 from dual union all
select 45388,'1978-F02','1978-05-27 01:25:00','orbital','failure',956,'294-02','','Ekran No. 13L','','Sat',554,'',40 from dual union all
select 45389,'1978-F03','1978-08-17 20:02:00','orbital','failure',956,'297-02','','Ekran No. 15L','','Sat',554,'',100 from dual union all
select 4539,'1957-S135','1957-10-20 03:57:00','suborbital rocket','success',718,'SUI 89','II5.65F','IGY II5.65F','','CR',705,'AGB-4',100 from dual union all
select 45390,'1978-F04','1978-10-17 16:04:00','orbital','failure',956,'298-01','','Ekran No. 14L','','Sat',554,'',100 from dual union all
select 45391,'1978-M01','1978-02-02 11:10:00','suborbital rocket','success',891,'DLR A-O-89','A-OR-90 Orion 1','','','Range Test',29,'',57 from dual union all
select 45392,'1978-M02','1978-05-20 09:29:00','suborbital rocket','success',852,'GL A45.710-1','SMOKE','','','Aeron',1821,'',53 from dual union all
select 45393,'1978-M03','1978-05-22 09:27:00','suborbital rocket','success',852,'GL A45.710-2','SMOKE','','','Aeron',1821,'',54 from dual union all
select 45394,'1978-M04','1978-06-08 20:00:00','suborbital rocket','success',1162,'EX830.56-1','Recovery test','','','Test',1047,'',105 from dual union all
select 45395,'1978-S01','1978-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 45396,'1978-S02','1978-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 45397,'1978-S03','1978-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 45398,'1978-S04','1978-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 45399,'1978-S05','1978-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 454,'1944-S71','1944-09-08 00:00:00','military missile','success',8,'','','','','Test',661,'',90 from dual union all
select 4540,'1957-S136','1957-10-20 20:11:00','suborbital rocket','success',718,'SUI 90','II5.66F','IGY II5.66F','','CR',706,'AGB-4',100 from dual union all
select 45400,'1978-S06','1978-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 45401,'1978-S07','1978-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 45402,'1978-S08','1978-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 45403,'1978-S09','1978-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 45404,'1978-S10','1978-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 45405,'1978-S100','1978-04-05 00:00:00','military missile','success',795,'','TREP-1','ABRES TREP-1','TREP-1','Reentry Test',1701,'',1300 from dual union all
select 45406,'1978-S101','1978-04-05 00:00:00','military missile','success',1106,'V5','','','','Test',301,'',1000 from dual union all
select 45407,'1978-S102','1978-04-05 12:26:00','suborbital rocket','success',846,'GL A11.712-4','DENSITY sphere','','','Aeron',873,'',175 from dual union all
select 45408,'1978-S103','1978-04-07 10:45:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',130,'VISE',178 from dual union all
select 45409,'1978-S104','1978-04-09 04:50:00','suborbital rocket','success',210,'NASA 27.10AE','E // B','','','Auroral active',428,'',247 from dual union all
select 4541,'1957-S137','1957-10-22 00:00:00','sounding rocket','success',488,'','Shot 6','','','Test/Ionos',405,'Balloon',5000 from dual union all
select 45410,'1978-S105','1978-04-11 04:00:00','suborbital rocket','success',211,'NASA 27.26UH','','','','XR Astron',1895,'',303 from dual union all
select 45411,'1978-S106','1978-04-11 21:59:00','military missile','success',939,'','OT-32','','','OT',421,'SSBN 619',500 from dual union all
select 45412,'1978-S107','1978-04-11 21:59:00','military missile','success',939,'','OT-32','','','OT',421,'SSBN 619',500 from dual union all
select 45413,'1978-S108','1978-04-11 22:00:00','military missile','success',939,'','OT-32','','','OT',421,'SSBN 619',500 from dual union all
select 45414,'1978-S109','1978-04-11 22:00:00','military missile','success',939,'','OT-32','','','OT',421,'SSBN 619',500 from dual union all
select 45415,'1978-S11','1978-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 45416,'1978-S110','1978-04-12 01:43:00','suborbital rocket','success',226,'ADD-VA-49','O/Ozone profile','','','Auroral',428,'',140 from dual union all
select 45417,'1978-S111','1978-04-12 15:31:00','military missile','failure',1442,'C4X-15','','','','Test',246,'',100 from dual union all
select 45418,'1978-S112','1978-04-15 16:39:00','test rocket','success',492,'701(II)-02','DDDS','','','Test',810,'',200 from dual union all
select 45419,'1978-S113','1978-04-13 19:22:56','suborbital rocket','success',860,'NASA 31.06UE','SSC S27 TWILIGHT','','','Test/Aeron',412,'',153 from dual union all
select 4542,'1957-S138','1957-10-22 22:31:00','suborbital rocket','success',718,'SUI 91','II5.67F','IGY II5.67F','','CR',707,'AGB-4',100 from dual union all
select 45420,'1978-S114','1978-04-18 00:00:00','suborbital rocket','success',1025,'','AST 1','','','Technology',1467,'',130 from dual union all
select 45421,'1978-S115','1978-04-19 10:35:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',130,'VISE',161 from dual union all
select 45422,'1978-S116','1978-04-19 15:37:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',130,'VISE',170 from dual union all
select 45423,'1978-S117','1978-04-21 05:29:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Plasma/Solar',130,'VISE',170 from dual union all
select 45424,'1978-S118','1978-04-21 06:26:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Plasma/Solar',130,'VISE',170 from dual union all
select 45425,'1978-S119','1978-04-21 07:52:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Plasma/Solar',130,'VISE',150 from dual union all
select 45426,'1978-S12','1978-05-30 00:00:00','test rocket','unknown',1497,'12','K65UP No. 12','','','Ionos?',621,'',1500 from dual union all
select 45427,'1978-S120','1978-04-21 17:40:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Plasma/Solar',130,'VISE',170 from dual union all
select 45428,'1978-S121','1978-04-21 18:30:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Plasma/Solar',130,'VISE',170 from dual union all
select 45429,'1978-S122','1978-04-24 09:35:00','military missile','success',939,'','OT-33','','','OT',421,'SSBN 645',500 from dual union all
select 4543,'1957-S139','1957-10-23 00:00:00','suborbital rocket','success',28,'','','','','Aeron',640,'',100 from dual union all
select 45430,'1978-S123','1978-04-24 09:35:00','military missile','success',939,'','OT-33','','','OT',421,'SSBN 645',500 from dual union all
select 45431,'1978-S124','1978-04-24 09:36:00','military missile','success',939,'','OT-33','','','OT',421,'SSBN 645',500 from dual union all
select 45432,'1978-S125','1978-04-24 09:36:00','military missile','success',939,'','OT-33','','','OT',421,'SSBN 645',500 from dual union all
select 45433,'1978-S126','1978-04-26 00:00:00','suborbital rocket','success',1219,'XV-05','','','','Ionos?',330,'',615 from dual union all
select 45434,'1978-S127','1978-05-04 06:14:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Plasma/Solar',130,'VISE',168 from dual union all
select 45435,'1978-S128','1978-05-04 07:05:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Plasma/Solar',130,'VISE',165 from dual union all
select 45436,'1978-S129','1978-05-04 18:29:00','suborbital rocket','unknown',740,'MR-12','','','','Plasma/Solar UV/Sol',130,'VISE',168 from dual union all
select 45437,'1978-S13','1978-09-16 00:00:00','test rocket','unknown',1497,'13','K65UP No. 13','','','Ionos?',621,'',1500 from dual union all
select 45438,'1978-S130','1978-05-05 06:30:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Plasma',130,'VISE',160 from dual union all
select 45439,'1978-S131','1978-05-06 07:00:00','suborbital rocket','success',38,'NASA 13.137UH','Wisc. Aerobee IV','','','XR Astron',1895,'',200 from dual union all
select 4544,'1957-S14','1957-02-14 14:12:00','suborbital rocket','success',1521,'R-22','4203-22','','','Reentry Test',253,'',141 from dual union all
select 45440,'1978-S132','1978-05-12 05:01:00','suborbital rocket','success',1213,'SL1305','Cul UVS 21','','','Solar UV',1839,'',283 from dual union all
select 45441,'1978-S133','1978-05-12 23:11:00','military missile','success',939,'','OT-34','','','OT',421,'SSBN 623',500 from dual union all
select 45442,'1978-S134','1978-05-12 23:12:00','military missile','success',939,'','OT-34','','','OT',421,'SSBN 623',500 from dual union all
select 45443,'1978-S135','1978-05-12 23:12:00','military missile','success',939,'','OT-34','','','OT',421,'SSBN 623',500 from dual union all
select 45444,'1978-S136','1978-05-12 23:13:00','military missile','success',939,'','OT-34','','','OT',421,'SSBN 623',500 from dual union all
select 45445,'1978-S137','1978-05-15 09:05:00','suborbital rocket','success',133,'NASA 25.26DG','NRL Mk II UV Camera','','','UV Astron',1895,'',211 from dual union all
select 45446,'1978-S138','1978-05-15 21:00:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 629',500 from dual union all
select 45447,'1978-S139','1978-05-16 20:30:00','suborbital rocket','success',37,'GL A04.606-1','EUV/Sphere','','','Aeron/Ionos/Solar EUV',1895,'',191 from dual union all
select 45448,'1978-S14','1978-01-01 00:00:00','military missile','success',997,'','','','','Test',1626,'K-140',1000 from dual union all
select 45449,'1978-S140','1978-05-18 13:00:00','military missile','success',917,'AO-184','P-357 77 Winter 05','P-II RV test 5','P2AD','OT',433,'',180 from dual union all
select 4545,'1957-S140','1957-10-22 15:50:00','military missile','success',934,'1-204-7','Fin guidance','','','Reentry Test',253,'',20 from dual union all
select 45450,'1978-S141','1978-05-21 00:00:00','military missile','success',939,'','OT-35','','','OT',421,'SSBN 658',500 from dual union all
select 45451,'1978-S142','1978-05-21 12:22:00','military missile','success',939,'','OT-35','','','OT',421,'SSBN 658',500 from dual union all
select 45452,'1978-S143','1978-05-21 12:23:00','military missile','success',939,'','OT-35','','','OT',421,'SSBN 658',500 from dual union all
select 45453,'1978-S144','1978-05-21 12:55:00','military missile','success',939,'','OT-35','','','OT',421,'SSBN 658',500 from dual union all
select 45454,'1978-S145','1978-05-30 20:35:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos/Plasma/Fields',636,'',170 from dual union all
select 45455,'1978-S146','1978-06-03 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT136M','GLORY TRIP 136M','OT',1699,'',1300 from dual union all
select 45456,'1978-S147','1978-06-05 00:00:00','military missile','success',744,'','15A15 UBP-3','','','OT',530,'',1000 from dual union all
select 45457,'1978-S148','1978-06-05 18:50:00','military missile','success',745,'67035-502(5L)','15A16 LKI-5','GCh 15F161','','OT',542,'',1000 from dual union all
select 45458,'1978-S149','1978-06-06 17:41:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',180 from dual;
insert into LAUNCH
select 45459,'1978-S15','1978-03-01 02:00:00','military missile','success',1006,'51048-202(2L)','15A18-01 LKI-2','GCh 15F183-1(15F162 x10)','','OT',489,'',1000 from dual union all
select 4546,'1957-S141','1957-10-23 01:07:00','military missile','success',638,'AM-3','R&D','','','Test',249,'',500 from dual union all
select 45460,'1978-S150','1978-06-06 20:35:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',176 from dual union all
select 45461,'1978-S151','1978-06-06 23:21:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',176 from dual union all
select 45462,'1978-S152','1978-06-08 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT63GB','GLORY TRIP 63GB','OT',1712,'',1300 from dual union all
select 45463,'1978-S153','1978-06-11 06:32:00','suborbital rocket','success',837,'NASA 14.539UE','JASPIC Rel TMA','','','Plasma',1821,'',220 from dual union all
select 45464,'1978-S154','1978-06-11 06:27:00','suborbital rocket','unknown',740,'D-75MG','JASPIC','','','Aeron/Ionos/Plasma/',1821,'VISE',170 from dual union all
select 45465,'1978-S155','1978-06-12 08:45:00','suborbital rocket','success',117,'Astro 8-2','Astro 8-2','','','XR Astron/Microgravity',1873,'',355 from dual union all
select 45466,'1978-S156','1978-06-14 00:00:00','military missile','success',1004,'08T','15A14 UBP-18','GCh 15F143U1','','Test',496,'',1000 from dual union all
select 45467,'1978-S157','1978-06-15 00:00:00','suborbital rocket','success',867,'NASA 18.1023UE','','','','Plasma',1821,'',265 from dual union all
select 45468,'1978-S158','1978-06-16 00:00:00','military missile','success',795,'','ANT-3','ABRES ANT-3','ANT-3','Reentry Test',1698,'',1300 from dual union all
select 45469,'1978-S159','1978-06-20 04:10:00','suborbital rocket','unknown',740,'D-75MG','JASPIC','','','Plasma/Solar UV/Sol',1821,'VISE',175 from dual union all
select 4547,'1957-S142','1957-10-23 19:22:54','test rocket','success',1486,'TV-2','Vanguard TV2','','','Test',235,'',175 from dual union all
select 45470,'1978-S16','1978-05-12 04:20:00','military missile','success',1006,'51035-501(5L)','15A18 LKI-3','GCh 15F183','','OT',483,'',1000 from dual union all
select 45471,'1978-S160','1978-06-20 04:15:00','suborbital rocket','success',837,'NASA 14.543UE','JASPIC','','','Plasma',1821,'',184 from dual union all
select 45472,'1978-S161','1978-06-20 00:00:00','military missile','success',1106,'V6','','','','Test',301,'',1000 from dual union all
select 45473,'1978-S162','1978-06-22 00:00:00','military missile','success',799,'STM-16W','Mk 12A RV x 2','R&D','STM-16W','Test',1707,'',1300 from dual union all
select 45474,'1978-S163','1978-06-22 11:49:00','military missile','failure',1442,'C4X-17','','','','Test',246,'',1000 from dual union all
select 45475,'1978-S164','1978-06-24 02:13:00','suborbital rocket','unknown',740,'D-75MG','JASPIC','','','Plasma/Solar UV/Sol',1821,'VISE',168 from dual union all
select 45476,'1978-S165','1978-06-26 02:08:00','suborbital rocket','success',867,'NASA 18.1019CE','JASPIC','','','Plasma',1821,'',315 from dual union all
select 45477,'1978-S166','1978-06-26 01:31:00','suborbital rocket','unknown',740,'D-75MG','JASPIC','','','Plasma/Solar UV/Sol',1821,'VISE',165 from dual union all
select 45478,'1978-S167','1978-06-27 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT64GM','GLORY TRIP 64GM','OT',1703,'',1300 from dual union all
select 45479,'1978-S168','1978-06-28 20:10:00','military missile','success',745,'51035-504(7L)','15A16 LKI-6','GCh 15F161','','OT',524,'',1000 from dual union all
select 4548,'1957-S143','1957-10-24 12:30:00','military missile','success',935,'3-204-3','','','','Reentry Test',253,'',100 from dual union all
select 45480,'1978-S169','1978-06-28 21:00:00','military missile','success',931,'A3TA','DASO','','','OT',421,'SSBN 599',1000 from dual union all
select 45481,'1978-S17','1978-07-27 03:30:00','military missile','success',1006,'45035-503(7L)','15A18-01 LKI-5','GCh 15F183-1','','OT',487,'',1000 from dual union all
select 45482,'1978-S170','1978-06-29 00:00:00','military missile','success',1004,'215T','15A14 UBP-19 ZP','GCh 15F143U1','','OT',484,'',1000 from dual union all
select 45483,'1978-S171','1978-07-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',130,'VISE',150 from dual union all
select 45484,'1978-S172','1978-07-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',130,'VISE',150 from dual union all
select 45485,'1978-S173','1978-07-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',130,'VISE',150 from dual union all
select 45486,'1978-S174','1978-07-04 00:00:00','military missile','success',795,'','ANT-2','ABRES ANT-2','ANT-2','Reentry Test',1701,'',1300 from dual union all
select 45487,'1978-S175','1978-07-07 00:00:00','military missile','success',931,'A3TA','FOT','','','OT',1897,'SSBN 618',1000 from dual union all
select 45488,'1978-S176','1978-07-07 00:00:00','military missile','success',931,'A3TA','FOT','','','OT',1897,'SSBN 618',1000 from dual union all
select 45489,'1978-S177','1978-07-12 00:00:00','military missile','success',931,'A3TA','FOT','','','OT',1897,'SSBN 611',1000 from dual union all
select 4549,'1957-S144','1957-10-24 16:38:00','military missile','success',1366,'109','R&D Series I','','','Test',233,'',520 from dual union all
select 45490,'1978-S178','1978-07-12 00:00:00','military missile','success',931,'A3TA','FOT','','','OT',1897,'SSBN 611',1000 from dual union all
select 45491,'1978-S179','1978-07-12 00:00:00','military missile','success',931,'A3TA','FOT','','','OT',1897,'SSBN 611',1000 from dual union all
select 45492,'1978-S18','1978-08-07 14:00:00','military missile','success',1006,'51035-504(8L)','15A18 LKI-6','GCh 15F183','','OT',483,'',1000 from dual union all
select 45493,'1978-S180','1978-07-13 14:37:00','military missile','success',917,'FOT-111','P-359 78 Cape 01','','','OT  R=740',231,'',180 from dual union all
select 45494,'1978-S181','1978-07-13 16:57:00','military missile','success',917,'FOT-112','P-360 78 Cape 02','','','OT',231,'',180 from dual union all
select 45495,'1978-S182','1978-07-13 18:09:00','military missile','success',917,'FOT-113','P-361 78 Cape 03','','','OT',231,'',180 from dual union all
select 45496,'1978-S183','1978-07-20 04:30:00','suborbital rocket','success',133,'NASA 25.37UH','','','','XR Astron',1895,'',190 from dual union all
select 45497,'1978-S184','1978-07-25 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT133M-1','GLORY TRIP 133M-1','OT',1705,'',1300 from dual union all
select 45498,'1978-S185','1978-07-25 04:05:00','suborbital rocket','success',30,'GL A03.604','DENSITY Laser','','','Aeron',1869,'',144 from dual union all
select 45499,'1978-S186','1978-07-25 14:30:00','military missile','success',917,'FOT-115','P-362 78 Cape 04','','','OT',231,'',180 from dual union all
select 455,'1944-S72','1944-09-01 00:00:00','military missile','success',1470,'','Liege','','','Weapon',1734,'',95 from dual union all
select 4550,'1957-S145','1957-10-25 00:00:00','suborbital rocket','success',28,'','','','','Ionos',640,'',100 from dual union all
select 45500,'1978-S187','1978-07-25 14:48:00','military missile','success',917,'FOT-114','P-363 78 Cape 05','','','OT',231,'',180 from dual union all
select 45501,'1978-S188','1978-07-25 15:06:00','military missile','success',917,'FOT-116','P-364 78 Cape 06','','','OT',231,'',180 from dual union all
select 45502,'1978-S189','1978-07-27 23:30:00','military missile','success',932,'A3TK P-1','CHEVALINE P1','','','Test',250,'',1000 from dual union all
select 45503,'1978-S19','1978-10-15 01:30:00','military missile','success',1006,'45035-506(10L)','15A18 LKI-8','GCh 15F183','','OT',489,'',1000 from dual union all
select 45504,'1978-S190','1978-07-29 00:00:00','military missile','success',1004,'44L','15A14 UBP-20 DP','GCh 15F678/EM Mayak','','OT',492,'',1000 from dual union all
select 45505,'1978-S191','1978-07-30 23:13:00','suborbital rocket','success',860,'','SSC S32 NLC-OBS','','','Aeronomy',417,'',134 from dual union all
select 45506,'1978-S192','1978-07-30 22:41:18','suborbital rocket','success',920,'P205K','Atomic oxygen','','','Aeronomy',413,'',149 from dual union all
select 45507,'1978-S193','1978-07-30 23:33:13','suborbital rocket','success',1200,'SSC S26/1 ION','SSC S26A Noctilucent','','','Ionospheric',416,'',114 from dual union all
select 45508,'1978-S194','1978-08-11 00:00:00','military missile','success',744,'','15A15 UBP-4','','','OT',530,'',1000 from dual union all
select 45509,'1978-S195','1978-08-11 02:20:00','military missile','success',1442,'C4X-18','','','','Test',246,'',1000 from dual union all
select 4551,'1957-S146','1957-10-26 19:47:00','suborbital rocket','success',718,'SUI 92','II5.68F','IGY II5.68F','','CR',708,'AGB-4',100 from dual union all
select 45510,'1978-S196','1978-08-13 00:58:15','suborbital rocket','success',920,'P206K','Atomic oxygen','','','Aeronomy',413,'',150 from dual union all
select 45511,'1978-S197','1978-08-14 14:45:00','suborbital rocket','failure',211,'NASA 27.34DS','HRTS-3','','','Solar',1895,'',220 from dual union all
select 45512,'1978-S198','1978-08-20 11:30:00','suborbital rocket','success',668,'K-9M-64','NIR Galactic Plane','','','Test/IR Astron/UV Astron',839,'',319 from dual union all
select 45513,'1978-S199','1978-08-25 03:00:00','military missile','success',745,'45035-503(6L)','15A16 LKI-7','GCh 15F161','','OT',546,'',1000 from dual union all
select 45514,'1978-S20','1978-10-16 15:20:00','military missile','success',1006,'67051-21-109(4L)','15A18 LKI-9','GCh 15F183','','OT',491,'',1000 from dual union all
select 45515,'1978-S200','1978-08-25 01:30:00','suborbital rocket','success',1299,'TT-500-4F','','','','Test',1476,'',300 from dual union all
select 45516,'1978-S201','1978-08-25 13:06:00','military missile','success',917,'FOT-117','P-365 78 Cape 07','','','OT',231,'',180 from dual union all
select 45517,'1978-S202','1978-08-25 13:23:00','military missile','success',917,'FOT-118','P-366 78 Cape 08','','','OT',231,'',180 from dual union all
select 45518,'1978-S203','1978-08-25 13:53:00','military missile','success',917,'FOT-119','P-367 78 Cape 09','','','OT',231,'',180 from dual union all
select 45519,'1978-S204','1978-09-01 00:00:00','military missile','success',1507,'VMF','','','','Test',1626,'PLBR',1000 from dual union all
select 4552,'1957-S147','1957-10-27 02:46:00','suborbital rocket','failure',718,'SUI 93','II5.69F','IGY II5.69F','','CR',710,'AGB-4',10 from dual union all
select 45520,'1978-S205','1978-09-06 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT65GB','GLORY TRIP 65GB','OT',1712,'',1300 from dual union all
select 45521,'1978-S206','1978-09-11 16:00:00','suborbital rocket','success',228,'NASA 21.46NP','SPAR 5','','','Micrograv',1873,'',165 from dual union all
select 45522,'1978-S207','1978-09-12 00:00:00','military missile','success',1004,'21T','15A14 UBP-21 ZP','GCh 15F143U2','','OT',510,'',1000 from dual union all
select 45523,'1978-S208','1978-09-13 19:36:00','suborbital rocket','success',891,'DLR K-O-91','K-OR-91','','','Range Test',417,'',62 from dual union all
select 45524,'1978-S209','1978-09-15 00:00:00','suborbital rocket','success',487,'F.2','CHEVALINE F2','','','Test',1836,'',97 from dual union all
select 45525,'1978-S21','1978-11-20 13:40:00','military missile','success',1006,'51055-507(11L)','15A18 LKI-10','GCh 15F183','','OT',483,'',1000 from dual union all
select 45526,'1978-S210','1978-09-15 21:00:00','suborbital rocket','success',867,'GL A08.708-1','CLUSTER ION','','','Ionos',1895,'',114 from dual union all
select 45527,'1978-S211','1978-09-19 18:30:00','suborbital rocket','success',37,'GL A04.711-1','SOLAR UV','','','Solar UV',1895,'',192 from dual union all
select 45528,'1978-S212','1978-09-20 02:58:00','suborbital rocket','success',133,'NASA 25.28UL','','','','EUV Astron',1895,'',224 from dual union all
select 45529,'1978-S213','1978-09-20 16:00:00','military missile','success',932,'A3TA','DASO','','','OT',421,'SSBN 598',1000 from dual union all
select 4553,'1957-S148','1957-10-29 00:13:00','suborbital rocket','failure',718,'SUI 94','II5.70F','IGY II5.70F','','Aeron/CR',711,'AGB-4',8 from dual union all
select 45530,'1978-S214','1978-09-26 05:05:00','suborbital rocket','success',133,'NASA 25.42GG','','','','UV Astron',1895,'',231 from dual union all
select 45531,'1978-S215','1978-09-27 05:30:00','suborbital rocket','success',837,'NASA 14.542UE','JASPIC','','','Plasma',1821,'',183 from dual union all
select 45532,'1978-S216','1978-09-27 00:00:00','suborbital rocket','success',1217,'XV-06','','','','Ionos?',330,'',230 from dual union all
select 45533,'1978-S217','1978-10-02 09:11:00','military missile','success',745,'67035-505(8L)','15A16 LKI-9','GCh 15F161','','OT',528,'',1000 from dual union all
select 45534,'1978-S218','1978-10-06 01:55:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',636,'',125 from dual union all
select 45535,'1978-S219','1978-10-26 09:16:31','suborbital rocket','success',846,'DNA IC807.15-1','SWIR DYNAMICS','','','Auroral',1047,'',122 from dual union all
select 45536,'1978-S22','1978-12-20 14:10:00','military missile','success',1006,'67055-508(12L)','15A18 LKI-11','GCh 15F183','','OT',485,'',1000 from dual union all
select 45537,'1978-S220','1978-10-26 09:20:20','suborbital rocket','failure',850,'DNA IC806.35-1','DYNAMICS','','','Auroral',1047,'',130 from dual union all
select 45538,'1978-S221','1978-10-26 09:29:00','suborbital rocket','success',846,'DNA IR807.57-1','ENERGY','','','Auroral IR',1047,'',160 from dual union all
select 45539,'1978-S222','1978-10-26 14:53:00','military missile','success',1442,'C4X-16','','','','Test',246,'',1000 from dual union all
select 4554,'1957-S149','1957-10-30 22:50:00','suborbital rocket','success',718,'SUI 95','II5.71F','IGY II5.71F','','Aeron/CR',712,'AGB-4',100 from dual union all
select 45540,'1978-S223','1978-10-29 05:02:00','suborbital rocket','success',1303,'DNA EX851.44-1','EXCEDE 1','','','Active Plasma',1047,'',137 from dual union all
select 45541,'1978-S224','1978-10-31 19:47:00','suborbital rocket','success',1320,'NASA 12.1005WT','','','','Test',1821,'',554 from dual union all
select 45542,'1978-S225','1978-11-23 00:00:00','suborbital rocket','success',466,'E015','Ariane range test','','','Test',340,'',350 from dual union all
select 45543,'1978-S226','1978-11-03 01:20:00','suborbital rocket','success',38,'NASA 13.135UE','UM Airglow Pay.','','','Plasma',1895,'',151 from dual union all
select 45544,'1978-S227','1978-11-03 11:45:00','ballistic missile test','success',1497,'14','Vertikal''-7','VZA','','Aeron/Ionos',621,'',1500 from dual union all
select 45545,'1978-S228','1978-11-10 04:14:22','suborbital rocket','success',1202,'SL1424','UK High Lat 10','','','Auroral  VLF/TMA',29,'',796 from dual union all
select 45546,'1978-S229','1978-11-13 12:44:30','suborbital rocket','success',1162,'DNA IC830.09-1A','FWIF II','','','Auroral IR',1047,'',141 from dual union all
select 45547,'1978-S23','1978-12-25 13:45:00','military missile','success',1006,'67055-505(9L)','15A18 LKI-12','GCh 15F183','','OT',487,'',1000 from dual union all
select 45548,'1978-S230','1978-11-15 00:00:00','suborbital rocket','success',1207,'SL1296','CHEVALINE S8','','','MIRV test',1839,'',210 from dual;
insert into LAUNCH
select 45549,'1978-S231','1978-11-16 00:00:00','military missile','failure',796,'','Mk 11 RV','FOT GT130M-2','GLORY TRIP 130M-2','OT',1702,'',1300 from dual union all
select 4555,'1957-S15','1957-03-01 11:58:00','suborbital rocket','success',1521,'R-23','4203-23','','','Reentry Test',253,'',151 from dual union all
select 45550,'1978-S232','1978-11-16 07:49:00','suborbital rocket','success',1212,'DLR K-GR-94','DLR TEXUS 2','','','Micrograv',416,'',265 from dual union all
select 45551,'1978-S233','1978-11-16 18:15:00','suborbital rocket','success',38,'NASA 13.138GS','Solar Constant 2','','','Solar',1895,'',182 from dual union all
select 45552,'1978-S234','1978-11-17 20:17:00','military missile','success',1442,'C4X-19','','','','Test',246,'',1000 from dual union all
select 45553,'1978-S235','1978-11-20 17:23:00','military missile','success',932,'A3TA','DASO','','','OT',421,'SSBN 601',1000 from dual union all
select 45554,'1978-S236','1978-11-25 04:44:10','suborbital rocket','success',210,'AAF-NVB-05','CMB spectrum 2','','','Submm Astron',428,'',366 from dual union all
select 45555,'1978-S237','1978-11-27 00:00:00','military missile','success',795,'','ANT-3A','ABRES ANT-3A','ANT-3A','Reentry Test',1701,'',1300 from dual union all
select 45556,'1978-S238','1978-11-27 18:55:00','suborbital rocket','success',867,'NASA 18.207IE','Ferdinand 41 (Elec 3)','','','Plasma',29,'',202 from dual union all
select 45557,'1978-S239','1978-11-27 18:55:35','suborbital rocket','success',867,'NASA 18.216IE','Ferdinand 40 (Elec 2)','','','Plasma',29,'',193 from dual union all
select 45558,'1978-S24','1978-12-26 12:40:00','military missile','success',1006,'45055-509(13L)','15A18 LKI-13','GCh 15F183','','OT',491,'',1000 from dual union all
select 45559,'1978-S240','1978-11-30 12:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',170 from dual union all
select 4556,'1957-S150','1957-10-31 01:44:00','suborbital rocket','success',718,'SUI 96','II5.72F','IGY II5.72F','','Aeron/CR',713,'AGB-4',100 from dual union all
select 45560,'1978-S241','1978-11-30 16:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',170 from dual union all
select 45561,'1978-S242','1978-11-30 18:55:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/Plasma/',636,'',170 from dual union all
select 45562,'1978-S243','1978-11-30 22:00:00','military missile','success',932,'A3TK P-2','CHEVALINE P2','','','Test',250,'',1000 from dual union all
select 45563,'1978-S244','1978-11-30 22:55:00','suborbital rocket','unknown',740,'MR-12','','','','Plasma/Fields/Solar',636,'',190 from dual union all
select 45564,'1978-S245','1978-12-01 00:55:00','suborbital rocket','success',740,'MR-12','Stereotop','','','Aeron/Ionos/Plasma',636,'',165 from dual union all
select 45565,'1978-S246','1978-12-01 07:20:00','suborbital rocket','success',133,'NASA 25.38UL','JHU UVT F9 (Jupiter)','','','UV Astron',1895,'',238 from dual union all
select 45566,'1978-S247','1978-12-02 00:00:00','military missile','success',1106,'V7','','','','Test',301,'',1000 from dual union all
select 45567,'1978-S248','1978-12-05 00:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT38GM','GLORY TRIP 38GM','OT',1705,'',1300 from dual union all
select 45568,'1978-S249','1978-12-05 00:00:00','military missile','success',277,'','DOT 1','','','Interceptor',871,'',235 from dual union all
select 45569,'1978-S25','1978-01-19 07:30:00','military missile','success',1452,'51165-52-252T','15A30 UBP-18 K','GCh 15B103','','OT',532,'',1000 from dual union all
select 4557,'1957-S151','1957-10-31 19:51:00','suborbital rocket','success',718,'SUI 97','II5.73F','IGY II5.73F','','Aeron/Fields',714,'AGB-4',100 from dual union all
select 45570,'1978-S250','1978-12-06 00:00:00','suborbital rocket','success',487,'F.3','CHEVALINE F3','','','Test',1836,'',98 from dual union all
select 45571,'1978-S251','1978-12-08 00:00:00','military missile','success',799,'STM-17W','Mk 12A RV x 3','R&D','STM-17W','Test',1707,'',1300 from dual union all
select 45572,'1978-S252','1978-12-11 00:00:00','suborbital rocket','success',867,'NASA 18.1024UA','','','','Aeronomy',1895,'',228 from dual union all
select 45573,'1978-S253','1978-12-11 03:00:00','suborbital rocket','success',133,'NASA 25.01UH','XUV Telescope','','','EUV Astron',1895,'',164 from dual union all
select 45574,'1978-S254','1978-12-13 00:00:00','suborbital rocket','success',1207,'SL1297','CHEVALINE S9','','','MIRV test',1839,'',210 from dual union all
select 45575,'1978-S255','1978-12-15 01:57:00','suborbital rocket','success',227,'APD-VB-47','','','','Auroral',428,'',316 from dual union all
select 45576,'1978-S256','1978-12-21 00:00:00','military missile','success',1004,'45L','15A14 UBP-22 DP','GCh 15F678','','OT',492,'',1000 from dual union all
select 45577,'1978-S257','1978-02-21 09:22:00','suborbital rocket','success',290,'ISRO','ISRO 5.47 (CO.47)','','','Ionos',1467,'',150 from dual union all
select 45578,'1978-S258','1977-02-14 13:00:00','suborbital rocket','success',1028,'560.T','ISRO (Langmuir)','','','Ionos',1361,'',300 from dual union all
select 45579,'1978-S259','1978-12-20 20:45:00','suborbital rocket','success',290,'ISRO','ISRO 5.51 (CO.51)','','','Ionos',1467,'',160 from dual union all
select 4558,'1957-S152','1957-11-01 01:00:00','suborbital rocket','failure',718,'SUI 98','II5.74F','IGY II5.74F','','Aeron/Fields',715,'AGB-4',10 from dual union all
select 45580,'1978-S26','1978-06-12 11:20:00','military missile','success',1452,'51165-52-253T','15A30 UBP-19 ZP','GCh 15B103','','OT',534,'',1000 from dual union all
select 45581,'1978-S260','1978-12-21 00:45:00','suborbital rocket','success',290,'ISRO','ISRO 5.52 (CO.52)','','','Ionos',1467,'',135 from dual union all
select 45582,'1978-S261','1978-07-30 23:12:30','suborbital rocket','success',920,'MPI1','','','','Aeronomy?',413,'',127 from dual union all
select 45583,'1978-S262','1978-08-13 00:26:57','suborbital rocket','success',920,'MPI2','','','','Aeronomy?',413,'',129 from dual union all
select 45584,'1978-S263','1978-04-05 08:55:00','suborbital rocket','success',1284,'Flight 2018','Sphere','','','Aeron',873,'',90 from dual union all
select 45585,'1978-S264','1978-04-05 10:41:00','suborbital rocket','success',1284,'Flight 2019','Sphere','','','Aeron',873,'',90 from dual union all
select 45586,'1978-S265','1978-04-05 12:43:00','suborbital rocket','success',1284,'Flight 2021','Sphere','','','Aeron',873,'',90 from dual union all
select 45587,'1978-S267','1978-10-31 14:20:00','suborbital rocket','success',290,'ISRO','ISRO 5.49','','','Ionos',1467,'',150 from dual union all
select 45588,'1978-S268','1978-09-12 04:10:00','military missile','destroyed before launch',745,'45055-506(9L)','15A16 LKI-8','GCh 15F161','','OT',542,'',0 from dual union all
select 45589,'1978-S269','1978-11-27 13:20:00','military missile','failure',745,'67055-508(11L)','15A16 LKI-10','GCh 15F161','','OT',545,'',50 from dual union all
select 4559,'1957-S153','1957-11-03 20:08:00','suborbital rocket','failure',718,'SUI 99','II5.75F','IGY II5.75F','','Aeron/Fields',716,'AGB-4',10 from dual union all
select 45590,'1978-S27','1978-07-21 01:15:00','military missile','success',1452,'63065-53-252T','15A30 UBP-20 ZP','GCh 15B103','','OT',532,'',1000 from dual union all
select 45591,'1978-S270','1978-06-26 02:31:00','suborbital rocket','unknown',740,'D-75MG','JASPIC','','','Plasma/Solar UV/Sol',1821,'VISE',165 from dual union all
select 45592,'1978-S271','1978-05-08 11:39:00','suborbital rocket','success',867,'','DNA Avefria Uno','','','Release',1490,'',192 from dual union all
select 45593,'1978-S272','1978-05-18 11:45:00','suborbital rocket','success',867,'','DNA Avefria Dos','','','Release',1490,'',192 from dual union all
select 45594,'1978-S273','1978-08-13 23:38:17','suborbital rocket','success',1200,'SSC S26/2 ION','SSC S26B Ion','','','Ionospheric',416,'',127 from dual union all
select 45595,'1978-S274','1978-03-30 14:04:00','military missile','unknown',917,'AO-183','P-356 77 Winter 04','P-II RV test 4','P2AD','OT',433,'',180 from dual union all
select 45596,'1978-S275','1978-01-01 00:00:00','military missile','unknown',917,'AO-185','P-358','Unknown','','OT',433,'',180 from dual union all
select 45597,'1978-S276','1978-12-05 00:00:00','military missile','success',917,'AO-192','P-374 78 Fall 07','','','OT',433,'',180 from dual union all
select 45598,'1978-S277','1978-12-05 00:00:00','military missile','success',917,'AO-193','P-375 78 Fall 08','','','OT',433,'',180 from dual union all
select 45599,'1978-S279','1978-01-01 00:00:00','military missile','success',644,'','MATSESTA 5','','','Reentry Test',636,'',null from dual union all
select 456,'1944-S73','1944-09-01 00:00:00','military missile','success',1470,'','Hasselt','','','Weapon',1728,'',95 from dual union all
select 4560,'1957-S154','1957-11-04 00:39:00','suborbital rocket','success',718,'SUI 100','II5.76F','IGY II5.76F','','Aeron/CR',717,'AGB-4',100 from dual union all
select 45600,'1978-S28','1978-02-27 02:20:00','military missile','success',1453,'49278-82-003T','15A35 LKI-3','GCh 15F355','','OT',535,'',1000 from dual union all
select 45601,'1978-S280','1978-01-01 00:00:00','military missile','success',984,'Aerofon','','','','Test',636,'',86 from dual union all
select 45602,'1978-S281','1978-08-15 00:15:00','suborbital rocket','success',668,'K-9M-63','','','','Ionos?',839,'',354 from dual union all
select 45603,'1978-S282','1978-04-25 00:00:00','military missile','success',1044,'64675-215-01','8K98P UBP-13','','','Test',477,'',1000 from dual union all
select 45604,'1978-S283','1978-05-18 00:00:00','military missile','success',1037,'Yu29501-03','8K98 UBP-40','','','Test',477,'',1000 from dual union all
select 45605,'1978-S284','1978-06-23 00:00:00','military missile','success',1037,'Yu29501-09','8K98 UBP-41','','','Test',477,'',1000 from dual union all
select 45606,'1978-S285','1978-07-12 00:00:00','military missile','success',1037,'Yu29501-10','8K98 UBP-42','','','Test',477,'',1000 from dual union all
select 45607,'1978-S286','1978-07-31 00:00:00','military missile','success',1044,'64675-215-05','8K98P UBP-14','','','Test',477,'',1000 from dual union all
select 45608,'1978-S287','1978-10-05 00:00:00','military missile','success',1037,'Yu29501-05','8K98 UBP-43','','','Test',477,'',1000 from dual union all
select 45609,'1978-S288','1978-01-01 00:00:00','military missile','success',1452,'','15A30 60 RD UBP-1','Off site test','','OT',1461,'',1000 from dual union all
select 4561,'1957-S155','1957-11-04 02:50:00','suborbital rocket','success',718,'SUI 101','II5.77F','IGY II5.77F','','Aeron/Fields',718,'AGB-4',113 from dual union all
select 45610,'1978-S289','1978-07-22 00:00:00','military missile','success',744,'','15A15 10 RD UBP-1','','','OT',897,'',1000 from dual union all
select 45611,'1978-S29','1978-03-23 02:35:00','military missile','success',1453,'49265-82-004T','15A35 LKI-4','GCh 15F355','','OT',508,'',1000 from dual union all
select 45612,'1978-S290','1978-08-23 00:00:00','military missile','success',744,'','15A15 10 RD UBP-2','','','OT',897,'',1000 from dual union all
select 45613,'1978-S291','1978-01-01 00:00:00','military missile','success',1450,'K09','15A20 47 RD UBP-11','Duga-1','','OT',1911,'',1000 from dual union all
select 45614,'1978-S292','1978-01-01 00:00:00','military missile','success',1450,'K10','15A20 47 RD UBP-12','Duga-1','','OT',1911,'',1000 from dual union all
select 45615,'1978-S293','1978-01-01 00:00:00','military missile','success',1450,'K11','15A20 47 RD UBP-13','Duga-1','','OT',1911,'',1000 from dual union all
select 45616,'1978-S294','1978-01-01 00:00:00','military missile','success',1450,'K12','15A20 47 RD UBP-14','Duga-1','','OT',1911,'',1000 from dual union all
select 45617,'1978-S295','1978-01-01 00:00:00','military missile','success',1450,'K13','15A20 47 RD UBP-15','Duga-1','','OT',1911,'',1000 from dual union all
select 45618,'1978-S296','1978-01-01 00:00:00','military missile','success',1450,'K14','15A20 47 RD UBP-16','Duga-1','','OT',1911,'',1000 from dual union all
select 45619,'1978-S297','1978-01-01 00:00:00','military missile','success',1450,'K15','15A20 47 RD UBP-17','Duga-1','','OT',1911,'',1000 from dual union all
select 4562,'1957-S156','1957-11-04 03:47:00','suborbital rocket','failure',718,'SUI 102','II5.78F','IGY II5.78F','','Aeron/Fields',719,'AGB-4',10 from dual union all
select 45620,'1978-S298','1978-01-01 00:00:00','military missile','success',1450,'K16','15A20 47 RD UBP-18','Duga-1','','OT',1911,'',1000 from dual union all
select 45621,'1978-S30','1978-04-26 14:00:00','military missile','success',1453,'49265-82-005T','15A35 LKI-5','GCh 15F355','','OT',535,'',1000 from dual union all
select 45622,'1978-S301','1978-01-01 00:00:00','military missile','success',436,'','','','','Test',810,'',1000 from dual union all
select 45623,'1978-S302','1978-01-11 12:35:00','suborbital rocket','success',1284,'INTA SLW-7801','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45624,'1978-S303','1978-01-13 12:42:00','suborbital rocket','success',1284,'INTA SLW-7802','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45625,'1978-S304','1978-01-16 12:43:00','suborbital rocket','success',1284,'INTA SLW-7803','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45626,'1978-S305','1978-01-16 13:05:00','suborbital rocket','success',1284,'INTA SLW-7804','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45627,'1978-S306','1978-01-18 12:31:00','suborbital rocket','success',1284,'INTA SLW-7805','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45628,'1978-S307','1978-01-20 12:47:00','suborbital rocket','success',1284,'INTA SLW-7806','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45629,'1978-S308','1978-01-23 12:31:00','suborbital rocket','success',1284,'INTA SLW-7807','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 4563,'1957-S157','1957-11-04 07:16:00','suborbital rocket','failure',718,'SUI 103','II5.79F','IGY II5.79F','','Aeron/CR',721,'AGB-4',10 from dual union all
select 45630,'1978-S309','1978-01-27 12:31:00','suborbital rocket','success',1284,'INTA SLW-7808','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45631,'1978-S31','1978-06-16 13:00:00','military missile','success',1453,'49265-82-006T','15A35 LKI-6','GCh 15F355','','OT',508,'',1000 from dual union all
select 45632,'1978-S310','1978-01-30 12:35:00','suborbital rocket','success',1284,'INTA SLW-7809','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45633,'1978-S311','1978-02-03 12:35:00','suborbital rocket','success',1284,'INTA SLW-7810','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45634,'1978-S312','1978-02-06 12:36:00','suborbital rocket','success',1284,'INTA SLW-7811','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45635,'1978-S313','1978-02-13 12:31:00','suborbital rocket','success',1284,'INTA SLW-7812','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45636,'1978-S314','1978-07-17 11:31:00','suborbital rocket','success',1284,'INTA SLW-7813','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45637,'1978-S315','1978-07-17 12:27:00','suborbital rocket','success',1284,'INTA SLW-7814','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45638,'1978-S316','1978-07-18 14:12:00','suborbital rocket','success',1284,'INTA SLW-7815','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual;
insert into LAUNCH
select 45639,'1978-S317','1978-07-26 11:31:00','suborbital rocket','success',1284,'INTA SLW-7816','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 4564,'1957-S158','1957-11-04 18:52:00','suborbital rocket','success',718,'SUI 104','II5.80F','IGY II5.80F','','Aeron/Fields',722,'AGB-4',10 from dual union all
select 45640,'1978-S318','1978-08-04 11:44:00','suborbital rocket','success',1284,'INTA SLW-7817','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45641,'1978-S319','1978-08-18 11:36:00','suborbital rocket','success',1284,'INTA SLW-7818','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45642,'1978-S32','1978-07-05 15:50:00','military missile','success',1453,'49265-82-007T','15A35 LKI-7','GCh 15F355','','OT',535,'',1000 from dual union all
select 45643,'1978-S320','1978-10-18 11:45:00','suborbital rocket','success',1284,'INTA SLW-7819','Prog. Meteo Nacional','','','Aeron',396,'',100 from dual union all
select 45644,'1978-S321','1978-06-21 00:00:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 45645,'1978-S322','1978-11-29 00:00:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 45646,'1978-S323','1978-06-10 22:56:50','suborbital rocket','success',1073,'S-310JA-5','','','','Ionos',839,'',226 from dual union all
select 45647,'1978-S324','1978-08-18 00:32:00','suborbital rocket','success',1073,'S-310JA-4','','','','Ionos',839,'',199 from dual union all
select 45648,'1978-S325','1978-03-27 19:15:50','suborbital rocket','success',1073,'S-310JA-7','','','','Auroral',839,'',220 from dual union all
select 45649,'1978-S326','1978-08-28 21:56:00','suborbital rocket','success',1073,'S-310JA-6','','','','Ionos',839,'',199 from dual union all
select 4565,'1957-S159','1957-11-05 01:25:00','suborbital rocket','success',718,'SUI 105','II5.81F','IGY II5.81F','','Aeron/CR',723,'AGB-4',100 from dual union all
select 45650,'1978-S327','1978-06-21 00:00:00','military missile','success',1004,'','15A14 62 RD UBP-1','','','Test',1646,'',1000 from dual union all
select 45651,'1978-S328','1978-02-19 00:00:00','military missile','success',922,'','15Zh45 32 RD UBP','','','Test',636,'',500 from dual union all
select 45652,'1978-S329','1978-04-18 00:00:00','military missile','success',922,'','15Zh45 42 RD UBP','','','Test',636,'',500 from dual union all
select 45653,'1978-S33','1978-08-10 03:40:00','military missile','success',1453,'49265-82-008T','15A35 LKI-8','GCh 15F355','','OT',508,'',1000 from dual union all
select 45654,'1978-S330','1978-11-27 00:00:00','military missile','success',922,'','15Zh45  8 RD UBP','','','Test',636,'',500 from dual union all
select 45655,'1978-S331','1978-06-01 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 45656,'1978-S332','1978-06-01 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 45657,'1978-S333','1978-10-01 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 45658,'1978-S334','1978-10-03 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 45659,'1978-S335','1978-11-27 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 4566,'1957-S16','1957-03-11 13:07:00','suborbital rocket','success',1521,'R-24','4203-A3','','','Reentry Test',253,'',134 from dual union all
select 45660,'1978-S336','1978-12-01 00:00:00','suborbital rocket','success',1014,'','R-5V Geophysical','Vol''t','','Active Plasma',636,'',380 from dual union all
select 45661,'1978-S337','1978-06-29 18:05:00','suborbital rocket','success',740,'','Spolokh-2','','','Ionos/Barium release',636,'',155 from dual union all
select 45662,'1978-S338','1978-10-19 14:05:00','military missile','success',917,'AO-186','P-368 78 Fall 01','','','OT',433,'',180 from dual union all
select 45663,'1978-S339','1978-10-19 14:35:00','military missile','success',917,'AO-187','P-369 78 Fall 02','','','OT',433,'',180 from dual union all
select 45664,'1978-S34','1978-08-31 03:40:00','military missile','success',1453,'49265-82-009T','15A35 LKI-9','GCh 15F355','','OT',535,'',1000 from dual union all
select 45665,'1978-S340','1978-11-07 00:00:00','military missile','success',917,'AO-188','P-370 78 Fall 03','','','OT',433,'',180 from dual union all
select 45666,'1978-S341','1978-11-07 00:00:00','military missile','success',917,'AO-189','P-371 78 Fall 04','','','OT',433,'',180 from dual union all
select 45667,'1978-S342','1978-11-20 15:05:00','military missile','success',917,'AO-190','P-372 78 Fall 05','','','OT',433,'',180 from dual union all
select 45668,'1978-S343','1978-11-20 15:40:00','military missile','success',917,'AO-191','P-373 78 Fall 06','','','OT',433,'',180 from dual union all
select 45669,'1978-S35','1978-10-03 00:00:00','military missile','success',1453,'65085-82-010T','15A35 LKI-10','GCh 15F355','','OT',535,'',1000 from dual union all
select 4567,'1957-S160','1957-11-05 20:23:00','suborbital rocket','success',718,'SUI 106','II5.82F','IGY II5.82F','','Aeron/Fields',724,'AGB-4',97 from dual union all
select 45670,'1978-S36','1978-10-09 00:00:00','military missile','success',1453,'49285-82-011T','15A35 LKI-11','GCh 15F355','','OT',505,'',1000 from dual union all
select 45671,'1978-S37','1978-11-24 00:00:00','military missile','success',1453,'49285-82-013T','15A35 LKI-12','','','OT',505,'',1000 from dual union all
select 45672,'1978-S38','1978-11-28 00:00:00','military missile','success',1453,'49285-51-240T','15A35 LKI-13','','','OT',535,'',1000 from dual union all
select 45673,'1978-S39','1978-12-08 00:00:00','military missile','success',1453,'49285-82-012T','15A35 LKI-14','','','OT',508,'',1000 from dual union all
select 45674,'1978-S40','1978-12-21 00:00:00','military missile','success',1453,'49285-82-014T','15A35 LKI-15','','','OT',505,'',1000 from dual union all
select 45675,'1978-S41','1978-01-05 05:01:00','suborbital rocket','success',837,'NASA 14.533UE','','','','Plasma',1821,'',197 from dual union all
select 45676,'1978-S42','1978-01-06 00:00:00','military missile','success',799,'STM-15W','Mk 12A RV x 3','R&D','STM-15W','Test',1707,'',1300 from dual union all
select 45677,'1978-S43','1978-01-09 20:00:00','suborbital rocket','success',133,'NASA 25.29GA','','','','Aeronomy',1895,'',262 from dual union all
select 45678,'1978-S44','1978-01-10 06:15:00','suborbital rocket','success',133,'NASA 25.12UH','','','','XR Astron',1895,'',196 from dual union all
select 45679,'1978-S45','1978-01-11 23:55:00','suborbital rocket','success',740,'MR-12','STERETOP','','','Active plasma',636,'',150 from dual union all
select 4568,'1957-S161','1957-11-05 23:17:00','suborbital rocket','success',718,'SUI 107','II5.83F','IGY II5.83F','','Aeron/Fields',725,'AGB-4',100 from dual union all
select 45680,'1978-S46','1978-01-16 01:30:00','suborbital rocket','success',1299,'TT-500-3F','','','','Test',1476,'',300 from dual union all
select 45681,'1978-S47','1978-01-17 20:21:00','military missile','success',1442,'C4X-13','Imp. Evader 106A','','','Test',246,'',1000 from dual union all
select 45682,'1978-S48','1978-01-21 09:01:00','suborbital rocket','success',133,'GL A31.603','CHARGING','','','Tech',1895,'',258 from dual union all
select 45683,'1978-S49','1978-01-22 02:00:00','suborbital rocket','success',668,'K-9M-62','','','','Ionos/Fields/Solar',839,'',369 from dual union all
select 45684,'1978-S50','1978-01-27 11:00:00','suborbital rocket','success',668,'K-9M-61','','','','Aeron/Ionos/Plasma',839,'',292 from dual union all
select 45685,'1978-S51','1978-01-28 20:20:00','suborbital rocket','success',1070,'S-210JA-30','','','','Auroral?',1458,'',125 from dual union all
select 45686,'1978-S52','1978-01-30 20:52:10','suborbital rocket','success',867,'NASA 18.211IE/UE','','','','Auroral',29,'',220 from dual union all
select 45687,'1978-S53','1978-01-30 21:37:00','suborbital rocket','success',1202,'DLR A-GRC-86','T/NL 4C  Gitti','','','Auroral',29,'',540 from dual union all
select 45688,'1978-S54','1978-01-30 22:10:00','suborbital rocket','success',1202,'DLR A-GRC-88','T/NL 2D  Susanne','','','Auroral',29,'',541 from dual union all
select 45689,'1978-S55','1978-01-31 18:35:00','suborbital rocket','success',211,'NASA 27.029CS','','','','Solar',1895,'',273 from dual union all
select 4569,'1957-S162','1957-11-07 16:05:00','suborbital rocket','success',28,'USAF-89','','','','Ionos',640,'',100 from dual union all
select 45690,'1978-S56','1978-02-02 08:54:00','suborbital rocket','success',1337,'NASA 29.08UE','','','','Plasma',1047,'',616 from dual union all
select 45691,'1978-S57','1978-02-02 13:04:00','suborbital rocket','success',920,'P199H','Atomic oxygen','','','Atmospheric',1447,'',140 from dual union all
select 45692,'1978-S58','1978-02-02 13:33:00','suborbital rocket','success',920,'P203H','Ion composition','','','Ionospheric',1447,'',112 from dual union all
select 45693,'1978-S59','1978-02-02 13:56:00','suborbital rocket','success',920,'P112H','Solar La','','','Ionospheric/Solar UV',1447,'',135 from dual union all
select 45694,'1978-S60','1978-02-06 00:20:00','suborbital rocket','success',920,'P200H','UV airglow','','','Atmospheric',1447,'',136 from dual union all
select 45695,'1978-S61','1978-02-06 01:00:00','suborbital rocket','success',920,'P202H','La/ne','','','Ionospheric',1447,'',125 from dual union all
select 45696,'1978-S62','1978-02-06 18:55:00','suborbital rocket','success',1070,'S-210JA-31','','','','Auroral?',1458,'',116 from dual union all
select 45697,'1978-S63','1978-02-07 00:30:00','suborbital rocket','success',920,'P204H','Ion composition','','','Ionospheric',1447,'',108 from dual union all
select 45698,'1978-S64','1978-02-07 01:01:00','suborbital rocket','success',920,'P198H','Atomic oxygen','','','Atmospheric',1447,'',139 from dual union all
select 45699,'1978-S65','1978-02-08 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT62GB','GLORY TRIP 62GB','OT',1712,'',1300 from dual union all
select 457,'1944-S74','1944-09-01 00:00:00','military missile','success',1470,'','Hasselt','','','Weapon',1728,'',95 from dual union all
select 4570,'1957-S163','1957-11-08 00:00:00','military missile','success',935,'3-204-4','','','','Reentry Test',253,'',100 from dual union all
select 45700,'1978-S66','1978-02-10 05:30:00','suborbital rocket','success',227,'NASA 21.56UG','JHU FOT 2','','','UV Astron',1873,'',238 from dual union all
select 45701,'1978-S67','1978-02-11 00:00:00','military missile','success',1004,'20T','15A14 UBP-16 ZP','GCh 15F141','','OT',496,'',1000 from dual union all
select 45702,'1978-S68','1978-02-13 00:00:00','suborbital rocket','success',920,'MOP4','Ozone','','','Atmospheric',1447,'',140 from dual union all
select 45703,'1978-S69','1978-02-13 00:00:00','suborbital rocket','success',920,'MOP2','Ozone','','','Atmospheric',1447,'',140 from dual union all
select 45704,'1978-S70','1978-02-13 17:15:00','suborbital rocket','success',228,'NASA 21.42DS','HRTS 2','','','Solar UV',1873,'',210 from dual union all
select 45705,'1978-S71','1978-02-15 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT61GM','GLORY TRIP 61GM','OT',1703,'',1300 from dual union all
select 45706,'1978-S72','1978-02-15 00:00:00','military missile','success',1004,'16T','15A14 UBP-17 ZP','GCh 15F143U2','','OT',510,'',1000 from dual union all
select 45707,'1978-S73','1978-02-14 21:30:00','military missile','failure',1442,'C4X-14','Imp. Evader 209','','','Test',246,'',1000 from dual union all
select 45708,'1978-S74','1978-02-26 11:00:00','suborbital rocket','success',668,'K-9M-60','','','','UV Astron/EUV Astron',839,'',340 from dual union all
select 45709,'1978-S75','1978-02-27 00:00:00','military missile','success',744,'','15A15 UBP-1','','','OT',542,'',1000 from dual union all
select 4571,'1957-S164','1957-11-08 14:57:00','suborbital rocket','failure',28,'USAF-90','','','','Test/Ionos',640,'',100 from dual union all
select 45710,'1978-S76','1978-02-27 05:30:00','suborbital rocket','success',220,'AAF-4B-35','','','','Auroral/Ionos',428,'',785 from dual union all
select 45711,'1978-S77','1978-02-27 05:43:36','suborbital rocket','success',867,'NASA 18.1017UE','','','','Plasma',1047,'',270 from dual union all
select 45712,'1978-S78','1978-03-01 00:00:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',130,'VISE',150 from dual union all
select 45713,'1978-S80','1978-02-28 04:17:00','suborbital rocket','success',867,'NASA 18.1015UE','','','','Aeron Ba',1047,'',270 from dual union all
select 45714,'1978-S81','1978-02-28 08:10:50','suborbital rocket','success',1164,'DNA IC819.08-1','MULTI-WIDEBAND','','','Tech IR',1047,'',458 from dual union all
select 45715,'1978-S82','1978-02-28 11:44:40','suborbital rocket','success',1304,'DNA WS834.55-1','Ballistics','','','Test',1047,'',500 from dual union all
select 45716,'1978-S83','1978-02-28 17:52:00','suborbital rocket','success',37,'NASA 13.136DA','','','','Aeronomy',1895,'',223 from dual union all
select 45717,'1978-S84','1978-03-01 01:13:00','suborbital rocket','success',837,'Ferdinand 47','Ferd. 47/Bugatti 1','','','Ionospheric',29,'',128 from dual union all
select 45718,'1978-S85','1978-03-01 01:13:01','suborbital rocket','success',837,'Ferdinand 48','Ferd. 48/Trinom 1','','','Ionospheric',29,'',109 from dual union all
select 45719,'1978-S86','1978-03-01 19:06:00','suborbital rocket','success',543,'ARPA WS810.51-2','WIDEBAND','','','Ionos',1047,'',170 from dual union all
select 4572,'1957-S165','1957-11-08 22:00:00','suborbital rocket','success',718,'SUI 108','II5.84F','IGY II5.84F','','Aeron/CR',726,'AGB-4',110 from dual union all
select 45720,'1978-S87','1978-03-03 04:41:00','military missile','success',799,'','Mk 12 RV x 3','OT GT37GM','GLORY TRIP 37GM','OT',1705,'',1300 from dual union all
select 45721,'1978-S88','1978-03-02 04:23:00','suborbital rocket','success',867,'NASA 18.1016UE','','','','Aeron Ba',1047,'',270 from dual union all
select 45722,'1978-S89','1978-03-08 04:00:00','suborbital rocket','success',133,'NASA 25.23UH','MIT WIXT 2','','','XR Astronomy',1895,'',190 from dual union all
select 45723,'1978-S90','1978-03-09 08:13:10','suborbital rocket','success',1337,'NASA 29.07UE','','','','Plasma',1047,'',336 from dual union all
select 45724,'1978-S91','1978-03-10 13:15:31','suborbital rocket','success',867,'NASA 18.1018UE','','','','Plasma',1047,'',270 from dual union all
select 45725,'1978-S92','1978-03-13 04:40:00','suborbital rocket','success',1319,'NASA 33.01UA','EUV airglow','','','Aeronomy',428,'',204 from dual union all
select 45726,'1978-S93','1978-03-13 07:05:00','suborbital rocket','success',133,'NASA 25.31UE','Auroral NO','','','Plasma',428,'',198 from dual union all
select 45727,'1978-S94','1978-03-13 07:50:00','suborbital rocket','success',1319,'NASA 33.02GA','Auroral UV','','','Aeronomy',428,'',186 from dual union all
select 45728,'1978-S95','1978-03-26 00:00:00','military missile','success',744,'','15A15 UBP-2','','','OT',530,'',1000 from dual;
insert into LAUNCH
select 45729,'1978-S96','1978-03-27 10:27:00','suborbital rocket','success',867,'NASA 18.215GM','','','','Aeron',1047,'',219 from dual union all
select 4573,'1957-S166','1957-11-09 00:00:00','suborbital rocket','success',718,'SUI 109','II5.85F','IGY II5.85F','','Aeron/CR',727,'AGB-4',110 from dual union all
select 45730,'1978-S97','1978-03-29 23:00:01','military missile','success',939,'','DASO','','','OT',421,'SSBN 626',500 from dual union all
select 45731,'1978-S98','1978-03-29 16:50:00','suborbital rocket','success',867,'NASA 18.214GM','','','','Aeron',1047,'',231 from dual union all
select 45732,'1978-S99','1978-03-31 03:30:00','military missile','success',745,'67048-803(3L)','15A16 LKI-3','GCh 15F161','','OT',542,'',1000 from dual union all
select 45733,'1978-W01','1978-01-03 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 45734,'1978-W02','1978-01-03 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 45735,'1978-W03','1978-01-03 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 45736,'1978-W04','1978-01-03 21:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 45737,'1978-W05','1978-01-03 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 45738,'1978-W06','1978-01-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',79 from dual union all
select 45739,'1978-W07','1978-01-04 14:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 4574,'1957-S167','1957-11-09 16:54:00','suborbital rocket','success',5,'','','','','Ionos/Aeron',636,'',330 from dual union all
select 45740,'1978-W08','1978-01-04 15:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',66 from dual union all
select 45741,'1978-W09','1978-01-04 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 45742,'1978-W10','1978-01-04 17:20:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',68 from dual union all
select 45743,'1978-W100','1978-01-26 16:14:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 45744,'1978-W1000','1978-09-25 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 45745,'1978-W1001','1978-09-25 21:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 45746,'1978-W1002','1978-09-25 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 45747,'1978-W1003','1978-09-27 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',902,'KRENKEL''',57 from dual union all
select 45748,'1978-W1004','1978-09-27 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 45749,'1978-W1005','1978-09-27 15:51:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 4575,'1957-S168','1957-11-10 00:00:00','suborbital rocket','success',28,'','','','','Aeron',640,'',100 from dual union all
select 45750,'1978-W1006','1978-09-27 16:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 45751,'1978-W1007','1978-09-27 17:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 45752,'1978-W1008','1978-09-27 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 45753,'1978-W1009','1978-09-27 17:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 45754,'1978-W101','1978-01-26 16:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 45755,'1978-W1010','1978-09-27 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 45756,'1978-W1011','1978-09-28 00:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 45757,'1978-W1012','1978-09-29 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',902,'KRENKEL''',60 from dual union all
select 45758,'1978-W1013','1978-09-29 14:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',60 from dual union all
select 45759,'1978-W1014','1978-09-29 16:18:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 4576,'1957-S169','1957-11-13 11:52:00','suborbital rocket','success',1193,'SL04','UCL Grenade/Chaff 1','','','Test/Aeronomy',1839,'',127 from dual union all
select 45760,'1978-W1015','1978-09-29 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 45761,'1978-W1016','1978-09-29 17:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',77 from dual union all
select 45762,'1978-W1017','1978-09-29 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 45763,'1978-W1018','1978-09-29 18:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 45764,'1978-W1019','1978-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 45765,'1978-W102','1978-01-27 15:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 45766,'1978-W1020','1978-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 45767,'1978-W1021','1978-10-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 45768,'1978-W1022','1978-09-30 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 45769,'1978-W1023','1978-10-02 15:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 4577,'1957-S17','1957-03-14 08:12:00','military missile','success',635,'CC-32    NU','Jupiter A','','','Test',280,'',90 from dual union all
select 45770,'1978-W1024','1978-10-02 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 45771,'1978-W1025','1978-10-02 16:34:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 45772,'1978-W1026','1978-10-02 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 45773,'1978-W1027','1978-10-02 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 45774,'1978-W1028','1978-10-02 18:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 45775,'1978-W1029','1978-10-03 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 45776,'1978-W103','1978-01-27 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 45777,'1978-W1030','1978-10-04 15:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 45778,'1978-W1031','1978-10-04 16:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 45779,'1978-W1032','1978-10-04 17:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 4578,'1957-S170','1957-11-15 00:00:00','military missile','success',934,'1-204-8','Jetavator','','','Reentry Test',253,'',20 from dual union all
select 45780,'1978-W1033','1978-10-04 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 45781,'1978-W1034','1978-10-04 19:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 45782,'1978-W1035','1978-10-04 20:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 45783,'1978-W1036','1978-10-04 23:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 45784,'1978-W1037','1978-10-06 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 45785,'1978-W1038','1978-10-06 16:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 45786,'1978-W1039','1978-10-06 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 45787,'1978-W104','1978-01-27 16:58:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',52 from dual union all
select 45788,'1978-W1040','1978-10-06 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 45789,'1978-W1041','1978-10-07 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 4579,'1957-S171','1957-11-19 16:29:56','suborbital rocket','success',28,'Smoke Puff','Release 1, KNO3-Al','','','Aeron',640,'',121 from dual union all
select 45790,'1978-W1042','1978-10-09 15:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 45791,'1978-W1043','1978-10-10 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 45792,'1978-W1044','1978-10-10 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 45793,'1978-W1045','1978-10-10 21:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 45794,'1978-W1046','1978-10-11 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 45795,'1978-W1047','1978-10-11 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',61 from dual union all
select 45796,'1978-W1048','1978-10-11 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 45797,'1978-W1049','1978-10-11 14:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 45798,'1978-W105','1978-01-27 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 45799,'1978-W1050','1978-10-11 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 458,'1944-S75','1944-09-01 00:00:00','military missile','success',1470,'','Hasselt x4','','','Weapon',1728,'',95 from dual union all
select 4580,'1957-S172','1957-11-26 00:00:00','suborbital rocket','success',28,'','Na','','','Aeron',640,'',100 from dual union all
select 45800,'1978-W1051','1978-10-11 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 45801,'1978-W1052','1978-10-11 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 45802,'1978-W1053','1978-10-11 19:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 45803,'1978-W1054','1978-10-11 21:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 45804,'1978-W1055','1978-10-11 22:07:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 45805,'1978-W1056','1978-10-13 14:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 45806,'1978-W1057','1978-10-13 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 45807,'1978-W1058','1978-10-13 15:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 45808,'1978-W1059','1978-10-13 16:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 45809,'1978-W106','1978-01-27 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 4581,'1957-S173','1957-12-07 22:11:00','military missile','success',1366,'112','R&D Series II','','','Test',234,'',520 from dual union all
select 45810,'1978-W1060','1978-10-13 17:13:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 45811,'1978-W1061','1978-10-13 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 45812,'1978-W1062','1978-10-13 20:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 45813,'1978-W1063','1978-10-13 21:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 45814,'1978-W1064','1978-10-16 13:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 45815,'1978-W1065','1978-10-16 15:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 45816,'1978-W1066','1978-10-16 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 45817,'1978-W1067','1978-10-16 19:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 45818,'1978-W1068','1978-10-16 21:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual;
insert into LAUNCH
select 45819,'1978-W1069','1978-10-16 21:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 4582,'1957-S174','1957-12-10 17:35:00','suborbital rocket','success',839,'OB6.02F','','IGY OB6.02F','','Ionos/Fields',1895,'',145 from dual union all
select 45820,'1978-W107','1978-01-30 14:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 45821,'1978-W1070','1978-10-17 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 45822,'1978-W1071','1978-10-18 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 45823,'1978-W1072','1978-10-18 15:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 45824,'1978-W1073','1978-10-18 15:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 45825,'1978-W1074','1978-10-18 17:00:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 45826,'1978-W1075','1978-10-18 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 45827,'1978-W1076','1978-10-18 17:21:00','suborbital rocket','unknown',1284,'FLIGHT 208     /T 1-','','','','Aeron',1821,'',75 from dual union all
select 45828,'1978-W1077','1978-10-18 17:52:00','suborbital rocket','unknown',1284,'FLIGHT 209     /T 1-','','','','Aeron',1821,'',64 from dual union all
select 45829,'1978-W1078','1978-10-18 18:05:00','suborbital rocket','unknown',1284,'FLIGHT 210     /TH1-','','','','Aeron',428,'',53 from dual union all
select 4583,'1957-S175','1957-12-10 18:12:00','suborbital rocket','success',838,'NN8.51F','NRL NN8.51F','IGY NN8.51F','','Test',1415,'',169 from dual union all
select 45830,'1978-W1079','1978-10-18 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 45831,'1978-W108','1978-01-30 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 45832,'1978-W1080','1978-10-18 21:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 45833,'1978-W1081','1978-10-19 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 45834,'1978-W1082','1978-10-19 21:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 45835,'1978-W1083','1978-10-20 15:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 45836,'1978-W1084','1978-10-20 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 45837,'1978-W1085','1978-10-20 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 45838,'1978-W1086','1978-10-20 19:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 45839,'1978-W1087','1978-10-20 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 4584,'1957-S176','1957-12-10 16:50:00','military missile','success',934,'1-204-9','Jetavator','','','Reentry Test',253,'',20 from dual union all
select 45840,'1978-W1088','1978-10-23 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 45841,'1978-W1089','1978-10-23 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 45842,'1978-W109','1978-01-30 16:40:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 45843,'1978-W1090','1978-10-23 17:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 45844,'1978-W1091','1978-10-23 18:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 45845,'1978-W1092','1978-10-24 21:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 45846,'1978-W1093','1978-10-25 15:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 45847,'1978-W1094','1978-10-25 15:25:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 45848,'1978-W1095','1978-10-25 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 45849,'1978-W1096','1978-10-25 17:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 4585,'1957-S177','1957-12-11 00:36:00','military missile','success',635,'CC-42    TU','Jupiter A','','','Test',280,'',90 from dual union all
select 45850,'1978-W1097','1978-10-25 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 45851,'1978-W1098','1978-10-25 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 45852,'1978-W1099','1978-10-25 22:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 45853,'1978-W11','1978-01-04 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 45854,'1978-W110','1978-01-30 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 45855,'1978-W1100','1978-10-25 23:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 45856,'1978-W1101','1978-10-26 17:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 45857,'1978-W1102','1978-10-27 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 45858,'1978-W1103','1978-10-27 15:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 45859,'1978-W1104','1978-10-27 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 4586,'1957-S178','1957-12-12 04:00:00','suborbital rocket','success',28,'SM1.07','Grenades','IGY SM1.07','','Aeron',428,'',80 from dual union all
select 45860,'1978-W1105','1978-10-27 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 45861,'1978-W1106','1978-10-27 18:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 45862,'1978-W1107','1978-10-28 01:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 45863,'1978-W1108','1978-10-30 14:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',63 from dual union all
select 45864,'1978-W1109','1978-10-30 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 45865,'1978-W111','1978-01-30 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 45866,'1978-W1110','1978-10-30 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 45867,'1978-W1111','1978-10-30 19:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 45868,'1978-W1112','1978-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 45869,'1978-W1113','1978-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 4587,'1957-S179','1957-12-14 21:00:00','suborbital rocket','success',28,'SM1.08','Grenades','IGY SM1.08','','Aeron',428,'',97 from dual union all
select 45870,'1978-W1114','1978-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 45871,'1978-W1115','1978-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 45872,'1978-W1116','1978-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 45873,'1978-W1117','1978-11-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 45874,'1978-W1118','1978-10-31 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 45875,'1978-W1119','1978-11-01 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',973,'MUSSON',59 from dual union all
select 45876,'1978-W112','1978-01-30 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 45877,'1978-W1120','1978-11-01 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',61 from dual union all
select 45878,'1978-W1121','1978-11-01 16:22:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',53 from dual union all
select 45879,'1978-W1122','1978-11-01 16:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 4588,'1957-S18','1957-03-21 13:54:00','suborbital rocket','success',1521,'R-25','4203-A4','','','Reentry Test',253,'',103 from dual union all
select 45880,'1978-W1123','1978-11-01 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 45881,'1978-W1124','1978-11-01 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 45882,'1978-W1125','1978-11-01 19:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 45883,'1978-W1126','1978-11-02 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 45884,'1978-W1127','1978-11-02 16:02:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',54 from dual union all
select 45885,'1978-W1128','1978-11-03 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',973,'MUSSON',59 from dual union all
select 45886,'1978-W1129','1978-11-03 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 45887,'1978-W113','1978-01-30 22:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 45888,'1978-W1130','1978-11-03 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 45889,'1978-W1131','1978-11-03 19:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 4589,'1957-S180','1957-12-17 17:39:00','military missile','success',142,'12A','R&D Fairing','R&D','','Test',229,'',120 from dual union all
select 45890,'1978-W1132','1978-11-03 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',52 from dual union all
select 45891,'1978-W1133','1978-11-06 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 45892,'1978-W1134','1978-11-06 14:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 45893,'1978-W1135','1978-11-06 15:49:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 45894,'1978-W1136','1978-11-06 17:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 45895,'1978-W1137','1978-11-06 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 45896,'1978-W1138','1978-11-06 21:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 45897,'1978-W1139','1978-11-07 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 45898,'1978-W114','1978-01-31 03:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',56 from dual union all
select 45899,'1978-W1140','1978-11-08 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',973,'MUSSON',58 from dual union all
select 459,'1944-S76','1944-09-01 00:00:00','military missile','success',1470,'','Tournai x6','','','Weapon',1728,'',95 from dual union all
select 4590,'1957-S181','1957-12-19 00:07:00','military missile','failure',638,'AM-4','R&D','','','Test',249,'',92 from dual union all
select 45900,'1978-W1141','1978-11-08 14:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 45901,'1978-W1142','1978-11-08 15:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 45902,'1978-W1143','1978-11-08 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 45903,'1978-W1144','1978-11-08 16:52:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 45904,'1978-W1145','1978-11-08 16:55:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 45905,'1978-W1146','1978-11-08 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 45906,'1978-W1147','1978-11-08 21:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 45907,'1978-W1148','1978-11-08 23:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 45908,'1978-W1149','1978-11-09 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual;
insert into LAUNCH
select 45909,'1978-W115','1978-01-31 13:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 4591,'1957-S182','1957-12-19 19:57:00','military missile','success',1366,'113','R&D Series II','','','Test',233,'',520 from dual union all
select 45910,'1978-W1150','1978-11-09 17:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 45911,'1978-W1151','1978-11-09 18:14:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 45912,'1978-W1152','1978-11-10 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',973,'MUSSON',62 from dual union all
select 45913,'1978-W1153','1978-11-10 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 45914,'1978-W1154','1978-11-10 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',52 from dual union all
select 45915,'1978-W1155','1978-11-10 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 45916,'1978-W1156','1978-11-13 15:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',61 from dual union all
select 45917,'1978-W1157','1978-11-13 16:19:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 45918,'1978-W1158','1978-11-13 17:11:00','suborbital rocket','unknown',1284,'FLIGHT 211     /TH1-','','','','Aeron',428,'',75 from dual union all
select 45919,'1978-W1159','1978-11-13 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 4592,'1957-S183','1957-05-15 16:01:00','military missile','failure',1016,'M1-5','8K71 LKI I-1','','','Test',480,'',100 from dual union all
select 45920,'1978-W116','1978-01-31 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',66 from dual union all
select 45921,'1978-W1160','1978-11-13 18:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',63 from dual union all
select 45922,'1978-W1161','1978-11-14 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 45923,'1978-W1162','1978-11-14 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 45924,'1978-W1163','1978-11-15 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',973,'MUSSON',63 from dual union all
select 45925,'1978-W1164','1978-11-15 14:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 45926,'1978-W1165','1978-11-15 16:37:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',50 from dual union all
select 45927,'1978-W1166','1978-11-15 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 45928,'1978-W1167','1978-11-15 17:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 45929,'1978-W1168','1978-11-15 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 4593,'1957-S184','1957-01-13 00:00:00','military missile','success',970,'No 0406','','','','OT',636,'',100 from dual union all
select 45930,'1978-W1169','1978-11-15 21:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 45931,'1978-W117','1978-01-31 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 45932,'1978-W1170','1978-11-15 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 45933,'1978-W1171','1978-11-16 16:07:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 45934,'1978-W1172','1978-11-16 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 45935,'1978-W1173','1978-11-17 01:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 45936,'1978-W1174','1978-11-17 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',973,'MUSSON',60 from dual union all
select 45937,'1978-W1175','1978-11-17 15:49:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',51 from dual union all
select 45938,'1978-W1176','1978-11-17 16:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',60 from dual union all
select 45939,'1978-W1177','1978-11-17 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 4594,'1957-S185','1957-01-14 00:00:00','military missile','success',970,'No 0408','','','','OT',636,'',100 from dual union all
select 45940,'1978-W1178','1978-11-17 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 45941,'1978-W1179','1978-11-17 21:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 45942,'1978-W118','1978-02-01 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1635,'USHAKOV',61 from dual union all
select 45943,'1978-W1180','1978-11-17 22:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 45944,'1978-W1181','1978-11-19 15:08:00','suborbital rocket','unknown',1284,'FLIGHT 212','','','','Aeron',330,'',74 from dual union all
select 45945,'1978-W1182','1978-11-19 15:59:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 45946,'1978-W1183','1978-11-19 17:00:00','suborbital rocket','unknown',1284,'FLIGHT 213     /TH1-','','','','Aeron',428,'',75 from dual union all
select 45947,'1978-W1184','1978-11-20 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 45948,'1978-W1185','1978-11-20 15:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 45949,'1978-W1186','1978-11-20 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 4595,'1957-S186','1957-01-04 00:00:00','military missile','failure',971,'A1-4T','','','','Test',636,'',80 from dual union all
select 45950,'1978-W1187','1978-11-20 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 45951,'1978-W1188','1978-11-20 16:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 45952,'1978-W1189','1978-11-20 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 45953,'1978-W119','1978-02-01 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 45954,'1978-W1190','1978-11-20 19:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 45955,'1978-W1191','1978-11-21 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 45956,'1978-W1192','1978-11-21 16:20:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 45957,'1978-W1193','1978-11-22 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',72 from dual union all
select 45958,'1978-W1194','1978-11-22 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 45959,'1978-W1195','1978-11-22 14:08:00','suborbital rocket','unknown',1284,'FLIGHT 214     /TN1-','','','','Aeron',330,'',73 from dual union all
select 4596,'1957-S187','1957-01-25 00:00:00','military missile','success',971,'A1-2','','','','Test',636,'',150 from dual union all
select 45960,'1978-W1196','1978-11-22 14:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 45961,'1978-W1197','1978-11-22 15:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 45962,'1978-W1198','1978-11-22 15:48:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 45963,'1978-W1199','1978-11-22 16:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 45964,'1978-W12','1978-01-04 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',77 from dual union all
select 45965,'1978-W120','1978-02-01 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 45966,'1978-W1200','1978-11-22 17:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 45967,'1978-W1201','1978-11-22 21:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 45968,'1978-W1202','1978-11-22 21:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 45969,'1978-W1203','1978-11-23 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 4597,'1957-S188','1957-02-01 00:00:00','military missile','success',971,'A1-7','','','','Test',636,'',150 from dual union all
select 45970,'1978-W1204','1978-11-24 15:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',57 from dual union all
select 45971,'1978-W1205','1978-11-24 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 45972,'1978-W1206','1978-11-24 17:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 45973,'1978-W1207','1978-11-25 01:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 45974,'1978-W1208','1978-11-25 15:10:00','suborbital rocket','unknown',1284,'FLIGHT 215','','','','Aeron',330,'',73 from dual union all
select 45975,'1978-W1209','1978-11-27 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 45976,'1978-W121','1978-02-01 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 45977,'1978-W1210','1978-11-27 15:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',51 from dual union all
select 45978,'1978-W1211','1978-11-27 15:55:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',66 from dual union all
select 45979,'1978-W1212','1978-11-27 17:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',52 from dual union all
select 4598,'1957-S189','1957-02-02 00:00:00','military missile','success',971,'A1-1','','','','Test',636,'',150 from dual union all
select 45980,'1978-W1213','1978-11-27 19:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',77 from dual union all
select 45981,'1978-W1214','1978-11-27 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 45982,'1978-W1215','1978-11-27 22:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 45983,'1978-W1216','1978-11-28 03:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 45984,'1978-W1217','1978-11-28 13:58:00','suborbital rocket','unknown',1284,'FLIGHT 216     /TN1-','','','','Aeron',330,'',73 from dual union all
select 45985,'1978-W1218','1978-11-28 20:12:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 45986,'1978-W1219','1978-11-29 16:13:00','suborbital rocket','unknown',1284,'FLIGHT 217     /T 1-','','','','Aeron',1821,'',74 from dual union all
select 45987,'1978-W122','1978-02-01 17:03:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 45988,'1978-W1220','1978-11-29 16:40:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',51 from dual union all
select 45989,'1978-W1221','1978-11-29 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 4599,'1957-S19','1957-03-25 00:00:00','military missile','success',975,'M1-21','','','','Test',636,'',86 from dual union all
select 45990,'1978-W1222','1978-11-29 17:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 45991,'1978-W1223','1978-11-29 19:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 45992,'1978-W1224','1978-11-29 23:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',60 from dual union all
select 45993,'1978-W1225','1978-11-30 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 45994,'1978-W1226','1978-11-30 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 45995,'1978-W1227','1978-11-30 21:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 45996,'1978-W1228','1978-12-01 15:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',58 from dual union all
select 45997,'1978-W1229','1978-12-01 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 45998,'1978-W123','1978-02-01 18:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 45999,'1978-W1230','1978-12-01 18:59:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual;
insert into LAUNCH
select 46,'1944-A10','1944-08-14 17:30:00','military missile','failure',1470,'18308','Ma126','','','Training',661,'',0 from dual union all
select 460,'1944-S77','1944-09-01 00:00:00','military missile','success',1470,'','Maastricht x8','','','Weapon',1728,'',95 from dual union all
select 4600,'1957-S190','1957-02-06 00:00:00','military missile','success',971,'A1-4','','','','Test',636,'',150 from dual union all
select 46000,'1978-W1231','1978-12-01 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46001,'1978-W1232','1978-12-04 15:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 46002,'1978-W1233','1978-12-04 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 46003,'1978-W1234','1978-12-04 21:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 46004,'1978-W1235','1978-12-05 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 46005,'1978-W1236','1978-12-05 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 46006,'1978-W1237','1978-12-05 18:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 46007,'1978-W1238','1978-12-05 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 46008,'1978-W1239','1978-12-06 15:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 46009,'1978-W124','1978-02-01 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 4601,'1957-S191','1957-02-19 00:00:00','military missile','success',970,'No 0920','','','','OT',636,'',100 from dual union all
select 46010,'1978-W1240','1978-12-06 15:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 46011,'1978-W1241','1978-12-06 17:15:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',65 from dual union all
select 46012,'1978-W1242','1978-12-06 18:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',67 from dual union all
select 46013,'1978-W1243','1978-12-06 20:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 46014,'1978-W1244','1978-12-06 22:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 46015,'1978-W1245','1978-12-07 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46016,'1978-W1246','1978-12-07 18:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 46017,'1978-W1247','1978-12-07 19:40:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 46018,'1978-W1248','1978-12-07 21:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 46019,'1978-W1249','1978-12-08 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 4602,'1957-S192','1957-02-20 00:00:00','military missile','success',971,'A2-1','','','','Test',636,'',150 from dual union all
select 46020,'1978-W125','1978-02-01 20:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 46021,'1978-W1250','1978-12-08 20:11:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 46022,'1978-W1251','1978-12-08 20:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46023,'1978-W1252','1978-12-08 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 46024,'1978-W1253','1978-12-08 21:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46025,'1978-W1254','1978-12-08 23:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 46026,'1978-W1255','1978-12-11 15:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 46027,'1978-W1256','1978-12-11 15:37:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 46028,'1978-W1257','1978-12-11 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 46029,'1978-W1258','1978-12-11 21:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 4603,'1957-S193','1957-02-20 00:00:00','military missile','success',971,'A1-20','','','','Test',636,'',150 from dual union all
select 46030,'1978-W1259','1978-12-12 15:56:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 46031,'1978-W126','1978-02-02 16:20:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 46032,'1978-W1260','1978-12-12 19:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 46033,'1978-W1261','1978-12-12 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 46034,'1978-W1262','1978-12-13 15:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',66 from dual union all
select 46035,'1978-W1263','1978-12-13 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 46036,'1978-W1264','1978-12-13 19:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',69 from dual union all
select 46037,'1978-W1265','1978-12-13 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 46038,'1978-W1266','1978-12-13 19:55:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 46039,'1978-W1267','1978-12-13 22:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 4604,'1957-S194','1957-02-28 00:00:00','military missile','success',992,'M1-25','','','','OT',636,'',170 from dual union all
select 46040,'1978-W1268','1978-12-13 23:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 46041,'1978-W1269','1978-12-14 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',58 from dual union all
select 46042,'1978-W127','1978-02-02 18:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',56 from dual union all
select 46043,'1978-W1270','1978-12-14 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 46044,'1978-W1271','1978-12-15 04:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 46045,'1978-W1272','1978-12-15 05:41:00','suborbital rocket','success',1281,'NASA 15.149GM','','','','Aeronomy',1895,'',70 from dual union all
select 46046,'1978-W1273','1978-12-15 14:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',63 from dual union all
select 46047,'1978-W1274','1978-12-15 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46048,'1978-W1275','1978-12-15 19:26:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 46049,'1978-W1276','1978-12-15 21:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',53 from dual union all
select 4605,'1957-S195','1957-03-11 00:00:00','military missile','success',1013,'A2-12TS','8K51 ST-1 special test','TMP-2 GCh','','Test',636,'',500 from dual union all
select 46050,'1978-W1277','1978-12-18 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 46051,'1978-W1278','1978-12-18 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46052,'1978-W1279','1978-12-18 17:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46053,'1978-W128','1978-02-03 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1635,'USHAKOV',61 from dual union all
select 46054,'1978-W1280','1978-12-18 20:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 46055,'1978-W1281','1978-12-19 18:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 46056,'1978-W1282','1978-12-19 19:41:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',52 from dual union all
select 46057,'1978-W1283','1978-12-20 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',62 from dual union all
select 46058,'1978-W1284','1978-12-20 15:42:00','suborbital rocket','unknown',1284,'FLIGHT 218     /T 1-','','','','Aeron',1821,'',66 from dual union all
select 46059,'1978-W1285','1978-12-20 16:18:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 4606,'1957-S196','1957-03-18 00:00:00','military missile','success',1013,'A2-13TS','8K51 ST-2','TMP-2 GCh','','Test',636,'',500 from dual union all
select 46060,'1978-W1286','1978-12-20 16:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 46061,'1978-W1287','1978-12-20 16:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46062,'1978-W1288','1978-12-20 17:13:00','suborbital rocket','unknown',1284,'FLIGHT 219     /TH1-','','','','Aeron',428,'',77 from dual union all
select 46063,'1978-W1289','1978-12-20 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 46064,'1978-W129','1978-02-03 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 46065,'1978-W1290','1978-12-20 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 46066,'1978-W1291','1978-12-20 23:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 46067,'1978-W1292','1978-12-21 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 46068,'1978-W1293','1978-12-21 18:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',61 from dual union all
select 46069,'1978-W1294','1978-12-22 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 4607,'1957-S197','1957-03-18 00:00:00','military missile','success',1013,'M5-7TK','8K51 ST-3','VV GCh','','Test',636,'',500 from dual union all
select 46070,'1978-W1295','1978-12-22 19:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 46071,'1978-W1296','1978-12-27 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',66 from dual union all
select 46072,'1978-W1297','1978-12-27 21:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 46073,'1978-W1298','1978-12-29 18:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 46074,'1978-W13','1978-01-04 21:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',69 from dual union all
select 46075,'1978-W130','1978-02-03 15:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 46076,'1978-W131','1978-02-03 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46077,'1978-W132','1978-02-03 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 46078,'1978-W133','1978-02-03 20:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46079,'1978-W134','1978-02-04 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 4608,'1957-S198','1957-03-21 00:00:00','military missile','success',1013,'A2-15TS','8K51 ST-4','TMP-2 GCh','','Test',636,'',500 from dual union all
select 46080,'1978-W135','1978-02-06 14:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 46081,'1978-W136','1978-02-06 15:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 46082,'1978-W137','1978-02-06 15:56:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',51 from dual union all
select 46083,'1978-W138','1978-02-06 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',56 from dual union all
select 46084,'1978-W139','1978-02-06 16:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 46085,'1978-W14','1978-01-04 21:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 46086,'1978-W140','1978-02-06 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 46087,'1978-W141','1978-02-06 19:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 46088,'1978-W142','1978-02-06 20:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual;
insert into LAUNCH
select 46089,'1978-W143','1978-02-07 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 4609,'1957-S199','1957-03-28 00:00:00','military missile','success',1013,'M1-20','8K51 OT-1','Series production','','Test',636,'',500 from dual union all
select 46090,'1978-W144','1978-02-07 19:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',66 from dual union all
select 46091,'1978-W145','1978-02-08 00:53:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 46092,'1978-W146','1978-02-08 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1635,'USHAKOV',64 from dual union all
select 46093,'1978-W147','1978-02-08 15:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 46094,'1978-W148','1978-02-08 15:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 46095,'1978-W149','1978-02-08 16:24:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 46096,'1978-W15','1978-01-05 16:11:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',51 from dual union all
select 46097,'1978-W150','1978-02-08 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46098,'1978-W151','1978-02-08 16:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46099,'1978-W152','1978-02-08 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 461,'1944-S78','1944-09-01 00:00:00','military missile','success',1470,'','Maastricht x4','','','Weapon',1728,'',95 from dual union all
select 4610,'1957-S20','1957-03-28 01:22:00','military missile','success',635,'CC-30    NX','Jupiter A','','','Test',280,'',90 from dual union all
select 46100,'1978-W153','1978-02-08 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 46101,'1978-W154','1978-02-08 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',72 from dual union all
select 46102,'1978-W155','1978-02-09 16:48:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 46103,'1978-W156','1978-02-09 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 46104,'1978-W157','1978-02-09 21:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46105,'1978-W158','1978-02-09 22:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 46106,'1978-W159','1978-02-10 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1635,'USHAKOV',62 from dual union all
select 46107,'1978-W16','1978-01-05 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 46108,'1978-W160','1978-02-10 15:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 46109,'1978-W161','1978-02-10 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 4611,'1957-S200','1957-05-29 00:00:00','military missile','success',992,'M2-25','','','','OT',636,'',170 from dual union all
select 46110,'1978-W162','1978-02-10 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 46111,'1978-W163','1978-02-10 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46112,'1978-W164','1978-02-10 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46113,'1978-W165','1978-02-13 15:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 46114,'1978-W166','1978-02-13 16:44:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',51 from dual union all
select 46115,'1978-W167','1978-02-13 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 46116,'1978-W168','1978-02-13 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46117,'1978-W169','1978-02-13 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46118,'1978-W17','1978-01-06 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 46119,'1978-W170','1978-02-14 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 4612,'1957-S201','1957-06-01 00:00:00','military missile','success',970,'0418','','','','OT',636,'',100 from dual union all
select 46120,'1978-W171','1978-02-14 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46121,'1978-W172','1978-02-14 16:52:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 46122,'1978-W173','1978-02-14 17:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 46123,'1978-W174','1978-02-14 22:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 46124,'1978-W175','1978-02-15 04:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1635,'USHAKOV',69 from dual union all
select 46125,'1978-W176','1978-02-15 11:10:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',64 from dual union all
select 46126,'1978-W177','1978-02-15 15:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46127,'1978-W178','1978-02-15 15:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 46128,'1978-W179','1978-02-15 16:11:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 46129,'1978-W18','1978-01-06 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',66 from dual union all
select 4613,'1957-S202','1957-06-01 00:00:00','military missile','success',970,'0419','','','','OT',636,'',100 from dual union all
select 46130,'1978-W180','1978-02-15 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 46131,'1978-W181','1978-02-15 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',52 from dual union all
select 46132,'1978-W182','1978-02-15 17:03:00','suborbital rocket','unknown',1284,'FLIGHT 184     /T 1-','','','','Aeron',1821,'',70 from dual union all
select 46133,'1978-W183','1978-02-15 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',52 from dual union all
select 46134,'1978-W184','1978-02-15 18:03:00','suborbital rocket','unknown',1284,'FLIGHT 185     /TH1-','','','','Aeron',428,'',74 from dual union all
select 46135,'1978-W185','1978-02-15 19:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 46136,'1978-W186','1978-02-16 01:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 46137,'1978-W187','1978-02-16 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 46138,'1978-W188','1978-02-16 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 46139,'1978-W189','1978-02-16 16:22:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 4614,'1957-S203','1957-06-05 00:00:00','military missile','success',992,'0405','','','','OT',636,'',170 from dual union all
select 46140,'1978-W19','1978-01-06 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 46141,'1978-W190','1978-02-16 16:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',63 from dual union all
select 46142,'1978-W191','1978-02-17 00:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46143,'1978-W192','1978-02-17 01:30:00','suborbital rocket','success',761,'MT135P T21','','','','Aeron?',1476,'',50 from dual union all
select 46144,'1978-W193','1978-02-17 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1635,'USHAKOV',65 from dual union all
select 46145,'1978-W194','1978-02-17 15:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 46146,'1978-W195','1978-02-17 15:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 46147,'1978-W196','1978-02-17 16:33:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',53 from dual union all
select 46148,'1978-W197','1978-02-17 16:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 46149,'1978-W198','1978-02-17 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 4615,'1957-S204','1957-06-07 00:00:00','military missile','success',992,'0406','','','','OT',636,'',170 from dual union all
select 46150,'1978-W199','1978-02-18 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 46151,'1978-W20','1978-01-06 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 46152,'1978-W200','1978-02-19 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46153,'1978-W201','1978-02-19 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',65 from dual union all
select 46154,'1978-W202','1978-02-20 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 46155,'1978-W203','1978-02-20 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 46156,'1978-W204','1978-02-20 15:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',65 from dual union all
select 46157,'1978-W205','1978-02-20 16:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 46158,'1978-W206','1978-02-21 13:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 46159,'1978-W207','1978-02-21 16:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 4616,'1957-S205','1957-06-14 00:00:00','military missile','success',1013,'A683-V','8K51 ST-5','Vibrator','','Test',636,'',500 from dual union all
select 46160,'1978-W208','1978-02-21 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 46161,'1978-W209','1978-02-21 16:34:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 46162,'1978-W21','1978-01-06 18:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 46163,'1978-W210','1978-02-21 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 46164,'1978-W211','1978-02-21 17:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 46165,'1978-W212','1978-02-21 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 46166,'1978-W213','1978-02-21 20:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 46167,'1978-W214','1978-02-22 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1635,'USHAKOV',68 from dual union all
select 46168,'1978-W215','1978-02-22 10:50:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',69 from dual union all
select 46169,'1978-W216','1978-02-22 15:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',63 from dual union all
select 4617,'1957-S206','1957-06-28 00:00:00','military missile','success',1013,'A681-V','8K51 ST-6','Vibrator','','Test',636,'',500 from dual union all
select 46170,'1978-W217','1978-02-22 15:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',66 from dual union all
select 46171,'1978-W218','1978-02-22 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 46172,'1978-W219','1978-02-22 16:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46173,'1978-W22','1978-01-06 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 46174,'1978-W220','1978-02-22 16:51:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',78 from dual union all
select 46175,'1978-W221','1978-02-22 16:56:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',65 from dual union all
select 46176,'1978-W222','1978-02-22 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 46177,'1978-W223','1978-02-22 17:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',58 from dual union all
select 46178,'1978-W224','1978-02-23 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual;
insert into LAUNCH
select 46179,'1978-W225','1978-02-23 15:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 4618,'1957-S207','1957-06-28 00:00:00','military missile','success',1013,'M2-20T','8K51 OT-2','','','Test',636,'',500 from dual union all
select 46180,'1978-W226','1978-02-23 17:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46181,'1978-W227','1978-02-23 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 46182,'1978-W228','1978-02-23 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 46183,'1978-W229','1978-02-23 22:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',66 from dual union all
select 46184,'1978-W23','1978-01-06 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 46185,'1978-W230','1978-02-24 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 46186,'1978-W231','1978-02-24 02:42:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 46187,'1978-W232','1978-02-24 03:35:00','suborbital rocket','unknown',738,'','','','','Aeron',1635,'USHAKOV',64 from dual union all
select 46188,'1978-W233','1978-02-24 15:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',58 from dual union all
select 46189,'1978-W234','1978-02-24 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 4619,'1957-S208','1957-07-07 00:00:00','military missile','success',992,'0417RD','','','','OT',634,'',170 from dual union all
select 46190,'1978-W235','1978-02-24 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 46191,'1978-W236','1978-02-24 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',53 from dual union all
select 46192,'1978-W237','1978-02-24 17:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 46193,'1978-W238','1978-02-25 15:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 46194,'1978-W239','1978-02-26 01:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 46195,'1978-W24','1978-01-06 23:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 46196,'1978-W240','1978-02-26 15:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',64 from dual union all
select 46197,'1978-W241','1978-02-27 05:32:00','suborbital rocket','success',229,'AKF-VI-14','Auroral X-rays','','','Auroral',428,'',55 from dual union all
select 46198,'1978-W242','1978-02-27 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 46199,'1978-W243','1978-02-27 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',54 from dual union all
select 462,'1944-S79','1944-09-01 00:00:00','military missile','success',1470,'','Liege x4','','','Weapon',1728,'',95 from dual union all
select 4620,'1957-S209','1957-07-08 00:00:00','military missile','success',969,'52080416','','','','OT',636,'',100 from dual union all
select 46200,'1978-W244','1978-02-27 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 46201,'1978-W245','1978-02-27 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 46202,'1978-W246','1978-02-27 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 46203,'1978-W247','1978-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 46204,'1978-W248','1978-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 46205,'1978-W249','1978-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 46206,'1978-W25','1978-01-09 15:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 46207,'1978-W250','1978-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 46208,'1978-W251','1978-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 46209,'1978-W252','1978-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 4621,'1957-S21','1957-03-29 04:51:00','suborbital rocket','success',57,'NRL 31','NRL NN2.26','IGY NN2.26','','UV Astronomy',1869,'',135 from dual union all
select 46210,'1978-W253','1978-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 46211,'1978-W254','1978-03-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 46212,'1978-W255','1978-02-28 00:00:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 46213,'1978-W256','1978-02-28 03:09:00','suborbital rocket','success',229,'AKF-VI-15','Auroral X-rays','','','Auroral',428,'',57 from dual union all
select 46214,'1978-W257','1978-02-28 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 46215,'1978-W258','1978-02-28 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 46216,'1978-W259','1978-02-28 19:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 46217,'1978-W26','1978-01-09 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 46218,'1978-W260','1978-03-01 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',79 from dual union all
select 46219,'1978-W261','1978-03-01 15:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 4622,'1957-S210','1957-07-09 00:00:00','military missile','success',992,'K2-5','','','','OT',636,'',170 from dual union all
select 46220,'1978-W262','1978-03-01 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',62 from dual union all
select 46221,'1978-W263','1978-03-01 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46222,'1978-W264','1978-03-01 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 46223,'1978-W265','1978-03-01 19:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 46224,'1978-W266','1978-03-02 01:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 46225,'1978-W267','1978-03-02 01:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 46226,'1978-W268','1978-03-03 14:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 46227,'1978-W269','1978-03-03 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 46228,'1978-W27','1978-01-09 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 46229,'1978-W270','1978-03-03 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 4623,'1957-S211','1957-07-09 00:00:00','military missile','success',969,'0415','','','','OT',636,'',100 from dual union all
select 46230,'1978-W271','1978-03-03 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46231,'1978-W272','1978-03-03 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 46232,'1978-W273','1978-03-04 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 46233,'1978-W274','1978-03-06 15:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 46234,'1978-W275','1978-03-06 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46235,'1978-W276','1978-03-06 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',57 from dual union all
select 46236,'1978-W277','1978-03-06 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46237,'1978-W278','1978-03-08 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',58 from dual union all
select 46238,'1978-W279','1978-03-08 02:04:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',53 from dual union all
select 46239,'1978-W28','1978-01-09 22:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 4624,'1957-S212','1957-07-09 00:00:00','military missile','success',1013,'A2-17TS','8K51 OT-3','GCh/RUD','','Test',636,'',500 from dual union all
select 46240,'1978-W280','1978-03-08 14:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 46241,'1978-W281','1978-03-08 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',66 from dual union all
select 46242,'1978-W282','1978-03-08 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 46243,'1978-W283','1978-03-08 17:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 46244,'1978-W284','1978-03-08 17:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 46245,'1978-W285','1978-03-08 17:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 46246,'1978-W286','1978-03-08 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',62 from dual union all
select 46247,'1978-W287','1978-03-08 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 46248,'1978-W288','1978-03-08 19:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 46249,'1978-W289','1978-03-08 20:58:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',74 from dual union all
select 4625,'1957-S213','1957-07-10 00:00:00','military missile','success',992,'0409','','','','OT',636,'',170 from dual union all
select 46250,'1978-W29','1978-01-09 22:46:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 46251,'1978-W290','1978-03-08 21:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',73 from dual union all
select 46252,'1978-W291','1978-03-08 21:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 46253,'1978-W292','1978-03-09 00:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 46254,'1978-W293','1978-03-09 01:41:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 46255,'1978-W294','1978-03-09 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',59 from dual union all
select 46256,'1978-W295','1978-03-10 01:49:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 46257,'1978-W296','1978-03-10 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46258,'1978-W297','1978-03-10 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46259,'1978-W298','1978-03-10 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',68 from dual union all
select 4626,'1957-S214','1957-07-18 00:00:00','military missile','success',992,'0407','','','','OT',636,'',170 from dual union all
select 46260,'1978-W299','1978-03-10 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 46261,'1978-W30','1978-01-09 23:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 46262,'1978-W300','1978-03-10 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46263,'1978-W301','1978-03-10 19:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 46264,'1978-W302','1978-03-10 19:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46265,'1978-W303','1978-03-10 20:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 46266,'1978-W304','1978-03-11 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 46267,'1978-W305','1978-03-13 14:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 46268,'1978-W306','1978-03-13 14:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual;
insert into LAUNCH
select 46269,'1978-W307','1978-03-13 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 4627,'1957-S215','1957-07-19 00:00:00','military missile','success',992,'0408','','','','OT',636,'',170 from dual union all
select 46270,'1978-W308','1978-03-13 19:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46271,'1978-W309','1978-03-13 22:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 46272,'1978-W31','1978-01-10 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 46273,'1978-W310','1978-03-15 01:21:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 46274,'1978-W311','1978-03-15 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 46275,'1978-W312','1978-03-15 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 46276,'1978-W313','1978-03-15 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 46277,'1978-W314','1978-03-15 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46278,'1978-W315','1978-03-15 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46279,'1978-W316','1978-03-15 17:14:00','suborbital rocket','unknown',1284,'FLIGHT 187     /T 1-','','','','Aeron',1821,'',76 from dual union all
select 4628,'1957-S216','1957-07-18 00:00:00','military missile','success',1013,'A2-16TS','8K51 OT-4','GCh/RUD','','Test',636,'',500 from dual union all
select 46280,'1978-W317','1978-03-15 18:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',67 from dual union all
select 46281,'1978-W318','1978-03-15 21:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 46282,'1978-W319','1978-03-16 01:29:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 46283,'1978-W32','1978-01-10 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 46284,'1978-W320','1978-03-16 01:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 46285,'1978-W321','1978-03-17 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',972,'MUSSON',59 from dual union all
select 46286,'1978-W322','1978-03-17 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',972,'MUSSON',64 from dual union all
select 46287,'1978-W323','1978-03-17 14:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',63 from dual union all
select 46288,'1978-W324','1978-03-17 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 46289,'1978-W325','1978-03-17 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 4629,'1957-S217','1957-07-24 00:00:00','military missile','success',1013,'A2-18TS','8K51 OT-5','GCh/RUD','','Test',636,'',500 from dual union all
select 46290,'1978-W326','1978-03-17 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',57 from dual union all
select 46291,'1978-W327','1978-03-18 01:19:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 46292,'1978-W328','1978-03-19 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1759,'VOLNA',59 from dual union all
select 46293,'1978-W329','1978-03-20 14:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',51 from dual union all
select 46294,'1978-W33','1978-01-11 15:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 46295,'1978-W330','1978-03-20 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46296,'1978-W331','1978-03-20 17:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 46297,'1978-W332','1978-03-20 17:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',65 from dual union all
select 46298,'1978-W333','1978-03-20 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',52 from dual union all
select 46299,'1978-W334','1978-03-20 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',59 from dual union all
select 463,'1944-S80','1944-09-01 00:00:00','military missile','success',1470,'','Liege x8','','','Weapon',1728,'',95 from dual union all
select 4630,'1957-S218','1957-07-22 00:00:00','military missile','success',992,'0419','','','','OT',634,'',170 from dual union all
select 46300,'1978-W335','1978-03-21 00:00:00','suborbital rocket','success',1281,'NASA 15.165GA','','','','Aeronomy',1047,'',62 from dual union all
select 46301,'1978-W336','1978-03-21 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1759,'VOLNA',60 from dual union all
select 46302,'1978-W337','1978-03-21 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 46303,'1978-W338','1978-03-21 21:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46304,'1978-W339','1978-03-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',70 from dual union all
select 46305,'1978-W34','1978-01-11 15:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',63 from dual union all
select 46306,'1978-W340','1978-03-22 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 46307,'1978-W341','1978-03-22 15:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 46308,'1978-W342','1978-03-22 16:40:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 46309,'1978-W343','1978-03-22 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 4631,'1957-S219','1957-07-24 00:00:00','military missile','success',992,'0319RD','','','','OT',634,'',170 from dual union all
select 46310,'1978-W344','1978-03-22 17:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 46311,'1978-W345','1978-03-22 18:01:00','suborbital rocket','unknown',1284,'FLIGHT 186     /TH1-','','','','Aeron',428,'',74 from dual union all
select 46312,'1978-W346','1978-03-22 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46313,'1978-W347','1978-03-22 18:35:00','suborbital rocket','unknown',1284,'FLIGHT 188     /TH1-','','','','Aeron',428,'',67 from dual union all
select 46314,'1978-W348','1978-03-22 20:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46315,'1978-W349','1978-03-22 21:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',59 from dual union all
select 46316,'1978-W35','1978-01-11 16:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 46317,'1978-W350','1978-03-22 21:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',71 from dual union all
select 46318,'1978-W351','1978-03-23 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 46319,'1978-W352','1978-03-23 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 4632,'1957-S22','1957-03-27 00:00:00','military missile','success',975,'M1-24','','','','Test',636,'',86 from dual union all
select 46320,'1978-W353','1978-03-24 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1759,'VOLNA',60 from dual union all
select 46321,'1978-W354','1978-03-24 08:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 46322,'1978-W355','1978-03-24 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 46323,'1978-W356','1978-03-24 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 46324,'1978-W357','1978-03-24 22:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',65 from dual union all
select 46325,'1978-W358','1978-03-25 01:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 46326,'1978-W359','1978-03-26 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1759,'VOLNA',60 from dual union all
select 46327,'1978-W36','1978-01-11 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',69 from dual union all
select 46328,'1978-W360','1978-03-27 00:00:00','suborbital rocket','success',1281,'NASA 15.167GA','','','','Aeronomy',1047,'',67 from dual union all
select 46329,'1978-W361','1978-03-27 00:00:00','suborbital rocket','success',1281,'NASA 15.169GA','','','','Aeronomy',1047,'',67 from dual union all
select 4633,'1957-S220','1957-07-29 00:00:00','military missile','success',992,'0217RD','','','','OT',634,'',170 from dual union all
select 46330,'1978-W362','1978-03-27 00:00:00','suborbital rocket','success',1281,'NASA 15.170UE','','','','Plasma',1047,'',75 from dual union all
select 46331,'1978-W363','1978-03-27 16:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 46332,'1978-W364','1978-03-27 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 46333,'1978-W365','1978-03-27 17:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 46334,'1978-W366','1978-03-29 00:00:00','suborbital rocket','success',1281,'NASA 15.164GA','','','','Aeronomy',1047,'',63 from dual union all
select 46335,'1978-W367','1978-03-29 00:00:00','suborbital rocket','success',1281,'NASA 15.166GA','','','','Aeronomy',1047,'',62 from dual union all
select 46336,'1978-W368','1978-03-29 00:00:00','suborbital rocket','success',1281,'NASA 15.168GA','','','','Aeronomy',1047,'',60 from dual union all
select 46337,'1978-W369','1978-03-29 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',972,'MUSSON',61 from dual union all
select 46338,'1978-W37','1978-01-11 20:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 46339,'1978-W370','1978-03-29 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',972,'MUSSON',62 from dual union all
select 4634,'1957-S221','1957-08-05 00:00:00','military missile','success',992,'0216','','','','OT',634,'',170 from dual union all
select 46340,'1978-W371','1978-03-29 14:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',52 from dual union all
select 46341,'1978-W372','1978-03-29 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 46342,'1978-W373','1978-03-29 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 46343,'1978-W374','1978-03-29 17:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 46344,'1978-W375','1978-03-29 19:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 46345,'1978-W376','1978-03-30 00:00:00','suborbital rocket','success',1281,'NASA 15.171UE','','','','Plasma',1047,'',58 from dual union all
select 46346,'1978-W377','1978-03-30 01:53:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',66 from dual union all
select 46347,'1978-W378','1978-03-30 12:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46348,'1978-W379','1978-03-30 16:36:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 46349,'1978-W38','1978-01-11 22:11:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',78 from dual union all
select 4635,'1957-S222','1957-08-06 00:00:00','military missile','success',992,'0215','','','','OT',634,'',170 from dual union all
select 46350,'1978-W380','1978-03-30 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 46351,'1978-W381','1978-04-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 46352,'1978-W382','1978-04-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 46353,'1978-W383','1978-04-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 46354,'1978-W384','1978-03-31 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',972,'MUSSON',62 from dual union all
select 46355,'1978-W385','1978-03-31 14:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46356,'1978-W386','1978-03-31 15:54:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 46357,'1978-W387','1978-03-31 16:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 46358,'1978-W388','1978-03-31 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual;
insert into LAUNCH
select 46359,'1978-W389','1978-03-31 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 4636,'1957-S223','1957-08-07 00:00:00','military missile','success',992,'K3-12','','','','OT',636,'',170 from dual union all
select 46360,'1978-W39','1978-01-11 23:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 46361,'1978-W390','1978-04-01 16:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1636,'USHAKOV',59 from dual union all
select 46362,'1978-W391','1978-04-01 17:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1636,'USHAKOV',58 from dual union all
select 46363,'1978-W392','1978-04-03 14:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 46364,'1978-W393','1978-04-03 16:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46365,'1978-W394','1978-04-03 19:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46366,'1978-W395','1978-04-05 16:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 46367,'1978-W396','1978-04-05 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46368,'1978-W397','1978-04-05 17:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',59 from dual union all
select 46369,'1978-W398','1978-04-05 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 4637,'1957-S224','1957-08-08 00:00:00','military missile','success',992,'K3-9','','','','OT',636,'',170 from dual union all
select 46370,'1978-W399','1978-04-05 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',72 from dual union all
select 46371,'1978-W40','1978-01-11 23:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 46372,'1978-W400','1978-04-06 03:22:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 46373,'1978-W401','1978-04-06 22:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',60 from dual union all
select 46374,'1978-W402','1978-04-07 16:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 46375,'1978-W403','1978-04-07 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 46376,'1978-W404','1978-04-07 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 46377,'1978-W405','1978-04-07 17:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46378,'1978-W406','1978-04-07 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 46379,'1978-W407','1978-04-08 00:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 4638,'1957-S225','1957-08-20 00:00:00','military missile','success',992,'K3-14','','','','OT',636,'',170 from dual union all
select 46380,'1978-W408','1978-04-08 02:14:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 46381,'1978-W409','1978-04-09 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',972,'MUSSON',61 from dual union all
select 46382,'1978-W41','1978-01-12 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46383,'1978-W410','1978-04-10 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 46384,'1978-W411','1978-04-10 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 46385,'1978-W412','1978-04-10 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 46386,'1978-W413','1978-04-10 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 46387,'1978-W414','1978-04-10 21:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',61 from dual union all
select 46388,'1978-W415','1978-04-11 00:00:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 46389,'1978-W416','1978-04-11 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 4639,'1957-S227','1957-08-22 00:00:00','military missile','success',971,'M686','ZI','','','Test',636,'',150 from dual union all
select 46390,'1978-W417','1978-04-12 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',972,'MUSSON',59 from dual union all
select 46391,'1978-W418','1978-04-12 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',972,'MUSSON',60 from dual union all
select 46392,'1978-W419','1978-04-12 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',62 from dual union all
select 46393,'1978-W42','1978-01-12 19:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 46394,'1978-W420','1978-04-12 15:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 46395,'1978-W421','1978-04-12 16:47:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 46396,'1978-W422','1978-04-12 17:00:00','suborbital rocket','unknown',1284,'FLIGHT 189     /T 1-','','','','Aeron',1895,'',72 from dual union all
select 46397,'1978-W423','1978-04-12 17:00:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',66 from dual union all
select 46398,'1978-W424','1978-04-12 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 46399,'1978-W425','1978-04-12 18:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 464,'1944-S81','1944-09-08 07:34:00','military missile','success',1470,'','Paris','','','Weapon',1738,'',95 from dual union all
select 4640,'1957-S228','1957-08-23 00:00:00','military missile','success',971,'M581','ZI','','','Test',636,'',150 from dual union all
select 46400,'1978-W426','1978-04-12 18:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',52 from dual union all
select 46401,'1978-W427','1978-04-12 19:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46402,'1978-W428','1978-04-12 21:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',72 from dual union all
select 46403,'1978-W429','1978-04-13 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',61 from dual union all
select 46404,'1978-W43','1978-01-12 21:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 46405,'1978-W430','1978-04-13 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46406,'1978-W431','1978-04-14 15:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46407,'1978-W432','1978-04-14 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 46408,'1978-W433','1978-04-14 16:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46409,'1978-W434','1978-04-14 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 4641,'1957-S229','1957-08-23 00:00:00','military missile','success',971,'M583','ZI','','','Test',636,'',150 from dual union all
select 46410,'1978-W435','1978-04-14 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 46411,'1978-W436','1978-04-14 20:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 46412,'1978-W437','1978-04-15 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',972,'MUSSON',58 from dual union all
select 46413,'1978-W438','1978-04-17 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',58 from dual union all
select 46414,'1978-W439','1978-04-17 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 46415,'1978-W44','1978-01-13 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',67 from dual union all
select 46416,'1978-W440','1978-04-17 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46417,'1978-W441','1978-04-17 18:59:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 46418,'1978-W442','1978-04-17 19:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46419,'1978-W443','1978-04-18 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 4642,'1957-S23','1957-04-03 00:00:00','military missile','success',975,'M1-23','','','','Test',636,'',86 from dual union all
select 46420,'1978-W444','1978-04-18 19:18:00','suborbital rocket','success',229,'AAF-VI-13','Dust collector','','','Meteorites',428,'',75 from dual union all
select 46421,'1978-W445','1978-04-19 13:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 46422,'1978-W446','1978-04-19 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 46423,'1978-W447','1978-04-19 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 46424,'1978-W448','1978-04-19 15:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 46425,'1978-W449','1978-04-19 18:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 46426,'1978-W45','1978-01-13 17:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 46427,'1978-W450','1978-04-19 20:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',73 from dual union all
select 46428,'1978-W451','1978-04-20 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 46429,'1978-W452','1978-04-21 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 4643,'1957-S230','1957-08-26 00:00:00','military missile','success',975,'M1-28','','','','Test',636,'',86 from dual union all
select 46430,'1978-W453','1978-04-21 15:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46431,'1978-W454','1978-04-21 16:00:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',66 from dual union all
select 46432,'1978-W455','1978-04-22 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 46433,'1978-W456','1978-04-24 14:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 46434,'1978-W457','1978-04-24 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 46435,'1978-W458','1978-04-24 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 46436,'1978-W459','1978-04-25 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 46437,'1978-W46','1978-01-13 20:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 46438,'1978-W460','1978-04-26 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 46439,'1978-W461','1978-04-26 16:13:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 4644,'1957-S231','1957-08-27 00:00:00','military missile','success',975,'M1-29','','','','Test',636,'',86 from dual union all
select 46440,'1978-W462','1978-04-26 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 46441,'1978-W463','1978-04-26 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46442,'1978-W464','1978-04-27 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 46443,'1978-W465','1978-04-27 21:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46444,'1978-W466','1978-04-28 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46445,'1978-W467','1978-04-28 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 46446,'1978-W468','1978-04-28 17:40:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 46447,'1978-W469','1978-04-28 19:00:00','suborbital rocket','unknown',1284,'FLIGHT 190     /TH1-','','','','Aeron',428,'',73 from dual union all
select 46448,'1978-W47','1978-01-13 20:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual;
insert into LAUNCH
select 46449,'1978-W470','1978-04-28 20:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 4645,'1957-S232','1957-08-28 00:00:00','military missile','success',975,'M1-20','','','','Test',636,'',86 from dual union all
select 46450,'1978-W471','1978-05-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 46451,'1978-W472','1978-05-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 46452,'1978-W473','1978-05-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 46453,'1978-W474','1978-05-01 14:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 46454,'1978-W475','1978-05-01 16:27:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 46455,'1978-W476','1978-05-01 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 46456,'1978-W477','1978-05-01 17:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',63 from dual union all
select 46457,'1978-W478','1978-05-01 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 46458,'1978-W479','1978-05-02 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',57 from dual union all
select 46459,'1978-W48','1978-01-13 21:47:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 4646,'1957-S233','1957-08-30 00:00:00','military missile','success',975,'M1-10','','','','Test',636,'',86 from dual union all
select 46460,'1978-W480','1978-05-02 21:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46461,'1978-W481','1978-05-03 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 46462,'1978-W482','1978-05-03 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 46463,'1978-W483','1978-05-03 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 46464,'1978-W484','1978-05-03 16:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 46465,'1978-W485','1978-05-03 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 46466,'1978-W486','1978-05-03 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 46467,'1978-W487','1978-05-04 01:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46468,'1978-W488','1978-05-04 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46469,'1978-W489','1978-05-05 14:33:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 4647,'1957-S234','1957-09-01 00:00:00','military missile','success',975,'M1-27','','','','Test',636,'',86 from dual union all
select 46470,'1978-W49','1978-01-14 01:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46471,'1978-W490','1978-05-05 14:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 46472,'1978-W491','1978-05-05 15:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 46473,'1978-W492','1978-05-05 15:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',57 from dual union all
select 46474,'1978-W493','1978-05-05 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 46475,'1978-W494','1978-05-05 21:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',62 from dual union all
select 46476,'1978-W495','1978-05-08 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 46477,'1978-W496','1978-05-08 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 46478,'1978-W497','1978-05-08 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46479,'1978-W498','1978-05-08 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 4648,'1957-S235','1957-08-22 00:00:00','military missile','success',992,'K3-4','','','','OT',636,'',170 from dual union all
select 46480,'1978-W499','1978-05-08 17:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',62 from dual union all
select 46481,'1978-W50','1978-01-16 14:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 46482,'1978-W500','1978-05-08 20:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 46483,'1978-W501','1978-05-09 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 46484,'1978-W502','1978-05-09 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46485,'1978-W503','1978-05-09 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 46486,'1978-W504','1978-05-10 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',54 from dual union all
select 46487,'1978-W505','1978-05-10 05:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',62 from dual union all
select 46488,'1978-W506','1978-05-10 14:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 46489,'1978-W507','1978-05-10 16:15:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 4649,'1957-S236','1957-08-23 00:00:00','military missile','success',992,'K3-13','','','','OT',636,'',170 from dual union all
select 46490,'1978-W508','1978-05-10 18:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46491,'1978-W509','1978-05-10 19:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 46492,'1978-W51','1978-01-16 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 46493,'1978-W510','1978-05-10 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 46494,'1978-W511','1978-05-10 20:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 46495,'1978-W512','1978-05-10 21:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 46496,'1978-W513','1978-05-11 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 46497,'1978-W514','1978-05-12 14:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 46498,'1978-W515','1978-05-12 16:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46499,'1978-W516','1978-05-12 16:07:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 465,'1944-S82','1944-09-08 16:35:00','military missile','success',1470,'','London','','','Weapon',1731,'',95 from dual union all
select 4650,'1957-S237','1957-08-27 00:00:00','military missile','success',992,'K3-7','','','','OT',636,'',170 from dual union all
select 46500,'1978-W517','1978-05-12 16:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',56 from dual union all
select 46501,'1978-W518','1978-05-12 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',51 from dual union all
select 46502,'1978-W519','1978-05-12 21:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 46503,'1978-W52','1978-01-16 19:55:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 46504,'1978-W520','1978-05-13 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 46505,'1978-W521','1978-05-15 14:00:00','suborbital rocket','unknown',725,'ISRO 8.371','','','','Aeron',1467,'',75 from dual union all
select 46506,'1978-W522','1978-05-15 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 46507,'1978-W523','1978-05-15 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 46508,'1978-W524','1978-05-15 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 46509,'1978-W525','1978-05-15 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 4651,'1957-S238','1957-08-27 00:00:00','military missile','success',992,'K3-2','','','','OT',636,'',170 from dual union all
select 46510,'1978-W526','1978-05-15 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 46511,'1978-W527','1978-05-16 01:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46512,'1978-W528','1978-05-16 18:33:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 46513,'1978-W529','1978-05-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',77 from dual union all
select 46514,'1978-W53','1978-01-16 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 46515,'1978-W530','1978-05-17 14:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46516,'1978-W531','1978-05-17 15:02:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',65 from dual union all
select 46517,'1978-W532','1978-05-17 15:03:00','suborbital rocket','unknown',1284,'T 1-9409','','','','Aeron',1895,'',71 from dual union all
select 46518,'1978-W533','1978-05-17 15:14:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',65 from dual union all
select 46519,'1978-W534','1978-05-17 15:15:00','suborbital rocket','unknown',1284,'T 1-9410','','','','Aeron',1895,'',71 from dual union all
select 4652,'1957-S239','1957-08-27 00:00:00','military missile','success',992,'K3-1','','','','OT',636,'',170 from dual union all
select 46520,'1978-W535','1978-05-17 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 46521,'1978-W536','1978-05-17 16:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 46522,'1978-W537','1978-05-17 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 46523,'1978-W538','1978-05-17 17:02:00','suborbital rocket','unknown',1284,'FLIGHT 191     /T 1-','','','','Aeron',1895,'',71 from dual union all
select 46524,'1978-W539','1978-05-17 17:02:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',65 from dual union all
select 46525,'1978-W54','1978-01-17 01:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 46526,'1978-W540','1978-05-17 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 46527,'1978-W541','1978-05-17 18:43:00','suborbital rocket','unknown',1284,'T 1-9411','','','','Aeron',1895,'',70 from dual union all
select 46528,'1978-W542','1978-05-17 18:43:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',66 from dual union all
select 46529,'1978-W543','1978-05-17 19:04:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',64 from dual union all
select 4653,'1957-S24','1957-04-11 16:31:00','suborbital rocket','success',49,'NRL 40 (RV-N-13c)','Vanguard test','','','Sat test',1869,'',204 from dual union all
select 46530,'1978-W544','1978-05-17 19:05:00','suborbital rocket','unknown',1284,'T 1-9412','','','','Aeron',1895,'',67 from dual union all
select 46531,'1978-W545','1978-05-17 19:47:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',67 from dual union all
select 46532,'1978-W546','1978-05-17 19:48:00','suborbital rocket','unknown',1284,'T 1-9413','','','','Aeron',1895,'',72 from dual union all
select 46533,'1978-W547','1978-05-17 20:00:00','suborbital rocket','unknown',1284,'T 1-9414','','','','Aeron',1895,'',66 from dual union all
select 46534,'1978-W548','1978-05-17 20:00:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',54 from dual union all
select 46535,'1978-W549','1978-05-18 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 46536,'1978-W55','1978-01-17 13:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 46537,'1978-W550','1978-05-18 11:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 46538,'1978-W551','1978-05-18 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual;
insert into LAUNCH
select 46539,'1978-W552','1978-05-18 15:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 4654,'1957-S240','1957-08-21 00:00:00','military missile','success',1013,'B7-20T','8K51 OT-6','','','Test',636,'',500 from dual union all
select 46540,'1978-W553','1978-05-18 15:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',55 from dual union all
select 46541,'1978-W554','1978-05-18 17:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 46542,'1978-W555','1978-05-18 18:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',61 from dual union all
select 46543,'1978-W556','1978-05-18 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 46544,'1978-W557','1978-05-18 22:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 46545,'1978-W558','1978-05-19 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 46546,'1978-W559','1978-05-19 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',50 from dual union all
select 46547,'1978-W56','1978-01-17 16:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 46548,'1978-W560','1978-05-19 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 46549,'1978-W561','1978-05-19 17:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 4655,'1957-S241','1957-09-05 00:00:00','military missile','success',1013,'M1-1','Generator-5 No. M1-1','','','Test',636,'',500 from dual union all
select 46550,'1978-W562','1978-05-19 17:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',58 from dual union all
select 46551,'1978-W563','1978-05-19 21:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 46552,'1978-W564','1978-05-22 14:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',57 from dual union all
select 46553,'1978-W565','1978-05-22 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 46554,'1978-W566','1978-05-22 15:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 46555,'1978-W567','1978-05-22 15:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 46556,'1978-W568','1978-05-22 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46557,'1978-W569','1978-05-22 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 46558,'1978-W57','1978-01-17 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 46559,'1978-W570','1978-05-22 19:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',60 from dual union all
select 4656,'1957-S242','1957-09-12 00:00:00','military missile','success',1013,'A2-19TS','8K51 OT-7','TMP-2','','Test',636,'',500 from dual union all
select 46560,'1978-W571','1978-05-23 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46561,'1978-W572','1978-05-23 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 46562,'1978-W573','1978-05-23 14:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 46563,'1978-W574','1978-05-23 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46564,'1978-W575','1978-05-23 20:18:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 46565,'1978-W576','1978-05-24 05:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',67 from dual union all
select 46566,'1978-W577','1978-05-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',77 from dual union all
select 46567,'1978-W578','1978-05-24 14:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 46568,'1978-W579','1978-05-24 15:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',59 from dual union all
select 46569,'1978-W58','1978-01-18 15:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 4657,'1957-S243','1957-09-14 00:00:00','military missile','success',1013,'A2-14TS','8K51 OT-8','TMP-2','','Test',636,'',500 from dual union all
select 46570,'1978-W580','1978-05-24 15:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',77 from dual union all
select 46571,'1978-W581','1978-05-24 17:00:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 46572,'1978-W582','1978-05-24 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 46573,'1978-W583','1978-05-24 18:00:00','suborbital rocket','unknown',1284,'FLIGHT 192     /TH1-','','','','Aeron',428,'',71 from dual union all
select 46574,'1978-W584','1978-05-24 18:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 46575,'1978-W585','1978-05-24 19:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46576,'1978-W586','1978-05-25 14:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 46577,'1978-W587','1978-05-25 14:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 46578,'1978-W588','1978-05-25 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 46579,'1978-W589','1978-05-26 14:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 4658,'1957-S244','1957-09-21 00:00:00','military missile','success',992,'0410','Tactical launcher','','','OT',1628,'',170 from dual union all
select 46580,'1978-W59','1978-01-18 16:18:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 46581,'1978-W590','1978-05-26 14:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 46582,'1978-W591','1978-05-26 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 46583,'1978-W592','1978-05-26 14:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46584,'1978-W593','1978-05-26 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 46585,'1978-W594','1978-05-26 18:12:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',73 from dual union all
select 46586,'1978-W595','1978-05-26 18:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46587,'1978-W596','1978-05-26 21:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 46588,'1978-W597','1978-05-29 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 46589,'1978-W598','1978-05-29 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',60 from dual union all
select 4659,'1957-S245','1957-09-21 00:00:00','military missile','success',992,'0412','Tactical launcher','','','OT',1628,'',170 from dual union all
select 46590,'1978-W599','1978-05-30 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46591,'1978-W60','1978-01-18 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 46592,'1978-W600','1978-05-30 18:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 46593,'1978-W601','1978-05-30 18:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',64 from dual union all
select 46594,'1978-W602','1978-05-31 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',79 from dual union all
select 46595,'1978-W603','1978-05-31 14:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 46596,'1978-W604','1978-05-31 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46597,'1978-W605','1978-05-31 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 46598,'1978-W606','1978-05-31 14:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',61 from dual union all
select 46599,'1978-W607','1978-05-31 15:54:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',54 from dual union all
select 466,'1944-S83','1944-09-08 16:36:00','military missile','success',1470,'','London','','','Weapon',1731,'',95 from dual union all
select 4660,'1957-S246','1957-09-22 00:00:00','military missile','success',992,'0411','Tactical launcher','','','OT',1628,'',170 from dual union all
select 46600,'1978-W608','1978-05-31 18:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46601,'1978-W609','1978-05-31 18:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 46602,'1978-W61','1978-01-18 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',57 from dual union all
select 46603,'1978-W610','1978-05-31 20:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',72 from dual union all
select 46604,'1978-W611','1978-06-01 17:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 46605,'1978-W612','1978-06-02 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 46606,'1978-W613','1978-06-02 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 46607,'1978-W614','1978-06-02 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 46608,'1978-W615','1978-06-02 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',53 from dual union all
select 46609,'1978-W616','1978-06-02 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 4661,'1957-S247','1957-09-23 00:00:00','military missile','success',992,'K3-15','Tactical launcher','','','OT',1628,'',170 from dual union all
select 46610,'1978-W617','1978-06-03 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 46611,'1978-W618','1978-06-05 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 46612,'1978-W619','1978-06-05 15:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 46613,'1978-W62','1978-01-18 17:08:00','suborbital rocket','unknown',1284,'FLIGHT 182     /T 1-','','','','Aeron',1821,'',72 from dual union all
select 46614,'1978-W620','1978-06-05 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 46615,'1978-W621','1978-06-05 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',57 from dual union all
select 46616,'1978-W622','1978-06-05 20:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 46617,'1978-W623','1978-06-07 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',76 from dual union all
select 46618,'1978-W624','1978-06-07 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',68 from dual union all
select 46619,'1978-W625','1978-06-07 14:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 4662,'1957-S248','1957-09-29 00:00:00','military missile','success',992,'0414','Tactical launcher','','','OT',1628,'',170 from dual union all
select 46620,'1978-W626','1978-06-07 14:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 46621,'1978-W627','1978-06-07 16:42:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46622,'1978-W628','1978-06-07 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 46623,'1978-W629','1978-06-07 16:51:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 46624,'1978-W63','1978-01-18 17:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 46625,'1978-W630','1978-06-07 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 46626,'1978-W631','1978-06-07 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 46627,'1978-W632','1978-06-09 15:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 46628,'1978-W633','1978-06-09 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual;
insert into LAUNCH
select 46629,'1978-W634','1978-06-09 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',59 from dual union all
select 4663,'1957-S249','1957-09-30 00:00:00','military missile','success',992,'0413','Tactical launcher','','','OT',1628,'',170 from dual union all
select 46630,'1978-W635','1978-06-09 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 46631,'1978-W636','1978-06-09 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46632,'1978-W637','1978-06-12 16:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46633,'1978-W638','1978-06-12 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 46634,'1978-W639','1978-06-12 17:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',56 from dual union all
select 46635,'1978-W64','1978-01-18 18:04:00','suborbital rocket','unknown',1284,'FLIGHT 183     /TH1-','','','','Aeron',428,'',74 from dual union all
select 46636,'1978-W640','1978-06-12 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 46637,'1978-W641','1978-06-12 21:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 46638,'1978-W642','1978-06-13 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 46639,'1978-W643','1978-06-14 14:00:00','suborbital rocket','unknown',725,'ISRO 8.379','','','','Aeron',1467,'',78 from dual union all
select 4664,'1957-S25','1957-04-12 00:00:00','military missile','success',994,'','','','','Test',636,'',100 from dual union all
select 46640,'1978-W644','1978-06-14 14:33:00','suborbital rocket','unknown',728,'ISRO 8.','','','','Aeron',1467,'',78 from dual union all
select 46641,'1978-W645','1978-06-14 15:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 46642,'1978-W646','1978-06-14 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 46643,'1978-W647','1978-06-14 17:11:00','suborbital rocket','unknown',1284,'FLIGHT 193     /T 1-','','','','Aeron',1895,'',71 from dual union all
select 46644,'1978-W648','1978-06-14 17:11:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',57 from dual union all
select 46645,'1978-W649','1978-06-14 17:17:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',51 from dual union all
select 46646,'1978-W65','1978-01-18 19:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 46647,'1978-W650','1978-06-14 18:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 46648,'1978-W651','1978-06-14 18:06:00','suborbital rocket','unknown',1284,'FLIGHT 194     /TH1-','','','','Aeron',428,'',73 from dual union all
select 46649,'1978-W652','1978-06-14 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 4665,'1957-S250','1957-10-01 00:00:00','military missile','success',992,'K2-11','Tactical launcher','','','OT',1628,'',170 from dual union all
select 46650,'1978-W653','1978-06-14 18:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 46651,'1978-W654','1978-06-14 18:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 46652,'1978-W655','1978-06-15 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 46653,'1978-W656','1978-06-16 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',56 from dual union all
select 46654,'1978-W657','1978-06-16 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 46655,'1978-W658','1978-06-16 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 46656,'1978-W659','1978-06-16 15:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',59 from dual union all
select 46657,'1978-W66','1978-01-18 21:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',71 from dual union all
select 46658,'1978-W660','1978-06-16 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46659,'1978-W661','1978-06-17 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 4666,'1957-S251','1957-10-02 00:00:00','military missile','success',992,'K3-11','Tactical launcher','','','OT',1628,'',170 from dual union all
select 46660,'1978-W662','1978-06-17 00:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',64 from dual union all
select 46661,'1978-W663','1978-06-19 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 46662,'1978-W664','1978-06-19 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 46663,'1978-W665','1978-06-19 17:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46664,'1978-W666','1978-06-19 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46665,'1978-W667','1978-06-19 19:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 46666,'1978-W668','1978-06-19 20:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 46667,'1978-W669','1978-06-20 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 46668,'1978-W67','1978-01-19 01:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 46669,'1978-W670','1978-06-20 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 4667,'1957-S252','1957-10-06 00:00:00','military missile','success',969,'5108088419','','','','OT',636,'',100 from dual union all
select 46670,'1978-W671','1978-06-21 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 46671,'1978-W672','1978-06-21 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 46672,'1978-W673','1978-06-21 15:45:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',65 from dual union all
select 46673,'1978-W674','1978-06-21 15:55:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',53 from dual union all
select 46674,'1978-W675','1978-06-21 16:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 46675,'1978-W676','1978-06-21 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 46676,'1978-W677','1978-06-21 17:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 46677,'1978-W678','1978-06-23 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 46678,'1978-W679','1978-06-23 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 46679,'1978-W68','1978-01-19 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 4668,'1957-S253','1957-10-06 00:00:00','military missile','success',969,'5108088423','','','','OT',636,'',100 from dual union all
select 46680,'1978-W680','1978-06-23 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46681,'1978-W681','1978-06-23 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 46682,'1978-W682','1978-06-23 17:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 46683,'1978-W683','1978-06-23 20:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 46684,'1978-W684','1978-06-26 14:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 46685,'1978-W685','1978-06-26 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46686,'1978-W686','1978-06-26 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 46687,'1978-W687','1978-06-26 18:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',59 from dual union all
select 46688,'1978-W688','1978-06-27 15:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 46689,'1978-W689','1978-06-27 22:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 4669,'1957-S254','1957-10-12 00:00:00','military missile','success',992,'52120311','','','','OT',636,'',170 from dual union all
select 46690,'1978-W69','1978-01-19 19:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 46691,'1978-W690','1978-06-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',78 from dual union all
select 46692,'1978-W691','1978-06-28 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',77 from dual union all
select 46693,'1978-W692','1978-06-28 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 46694,'1978-W693','1978-06-28 15:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 46695,'1978-W694','1978-06-28 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46696,'1978-W695','1978-06-28 15:45:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',66 from dual union all
select 46697,'1978-W696','1978-06-28 16:35:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 46698,'1978-W697','1978-06-28 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 46699,'1978-W698','1978-06-28 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 467,'1944-S84','1944-09-10 19:24:00','military missile','success',1470,'','London','','','Weapon',1744,'',95 from dual union all
select 4670,'1957-S255','1957-10-12 00:00:00','military missile','success',992,'52120312','','','','OT',636,'',170 from dual union all
select 46700,'1978-W699','1978-06-28 21:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46701,'1978-W70','1978-01-20 14:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 46702,'1978-W700','1978-06-28 21:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',70 from dual union all
select 46703,'1978-W701','1978-06-29 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 46704,'1978-W702','1978-06-30 14:45:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 46705,'1978-W703','1978-06-30 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 46706,'1978-W704','1978-06-30 17:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 46707,'1978-W705','1978-06-30 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46708,'1978-W706','1978-06-30 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',53 from dual union all
select 46709,'1978-W707','1978-07-01 02:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 4671,'1957-S256','1957-10-13 00:00:00','military missile','success',969,'5108088422','','','','OT',636,'',100 from dual union all
select 46710,'1978-W708','1978-07-03 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 46711,'1978-W709','1978-07-03 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46712,'1978-W71','1978-01-20 16:37:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 46713,'1978-W710','1978-07-04 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 46714,'1978-W711','1978-07-04 10:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',60 from dual union all
select 46715,'1978-W712','1978-07-05 14:00:00','suborbital rocket','unknown',725,'ISRO 8.382','','','','Aeron',1467,'',76 from dual union all
select 46716,'1978-W713','1978-07-05 14:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',63 from dual union all
select 46717,'1978-W714','1978-07-05 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 46718,'1978-W715','1978-07-05 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual;
insert into LAUNCH
select 46719,'1978-W716','1978-07-05 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 4672,'1957-S257','1957-10-13 00:00:00','military missile','success',992,'52120313','','','','OT',636,'',170 from dual union all
select 46720,'1978-W717','1978-07-05 16:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 46721,'1978-W718','1978-07-05 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 46722,'1978-W719','1978-07-05 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 46723,'1978-W72','1978-01-20 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 46724,'1978-W720','1978-07-05 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',71 from dual union all
select 46725,'1978-W721','1978-07-05 21:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46726,'1978-W722','1978-07-06 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 46727,'1978-W723','1978-07-07 14:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',55 from dual union all
select 46728,'1978-W724','1978-07-07 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 46729,'1978-W725','1978-07-07 19:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 4673,'1957-S258','1957-10-13 00:00:00','military missile','success',992,'K3-6','','','','OT',636,'',170 from dual union all
select 46730,'1978-W726','1978-07-08 01:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46731,'1978-W727','1978-07-10 13:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 46732,'1978-W728','1978-07-10 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 46733,'1978-W729','1978-07-10 14:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46734,'1978-W73','1978-01-20 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46735,'1978-W730','1978-07-10 17:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 46736,'1978-W731','1978-07-10 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46737,'1978-W732','1978-07-10 22:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 46738,'1978-W733','1978-07-12 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 46739,'1978-W734','1978-07-12 14:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 4674,'1957-S259','1957-10-14 00:00:00','military missile','success',969,'0101','','','','OT',636,'',100 from dual union all
select 46740,'1978-W735','1978-07-12 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 46741,'1978-W736','1978-07-12 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 46742,'1978-W737','1978-07-12 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 46743,'1978-W738','1978-07-12 17:04:00','suborbital rocket','unknown',1284,'FLIGHT 195     /T 1-','','','','Aeron',1821,'',74 from dual union all
select 46744,'1978-W739','1978-07-12 17:20:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 46745,'1978-W74','1978-01-20 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 46746,'1978-W740','1978-07-12 18:36:00','suborbital rocket','unknown',1284,'FLIGHT 196     /TH1-','','','','Aeron',428,'',73 from dual union all
select 46747,'1978-W741','1978-07-12 19:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 46748,'1978-W742','1978-07-13 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46749,'1978-W743','1978-07-14 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 4675,'1957-S26','1957-04-13 00:00:00','military missile','failure',937,'VI-1','','','','Reentry Test',253,'',10 from dual union all
select 46750,'1978-W744','1978-07-14 16:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 46751,'1978-W745','1978-07-14 16:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 46752,'1978-W746','1978-07-14 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 46753,'1978-W747','1978-07-14 20:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',52 from dual union all
select 46754,'1978-W748','1978-07-17 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 46755,'1978-W749','1978-07-17 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 46756,'1978-W75','1978-01-20 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 46757,'1978-W750','1978-07-17 16:41:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 46758,'1978-W751','1978-07-17 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',52 from dual union all
select 46759,'1978-W752','1978-07-17 17:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',52 from dual union all
select 4676,'1957-S260','1957-10-14 00:00:00','military missile','success',969,'5108088429','','','','OT',636,'',100 from dual union all
select 46760,'1978-W753','1978-07-18 23:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 46761,'1978-W754','1978-07-19 05:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 46762,'1978-W755','1978-07-19 11:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 46763,'1978-W756','1978-07-19 14:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 46764,'1978-W757','1978-07-19 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 46765,'1978-W758','1978-07-19 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 46766,'1978-W759','1978-07-19 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',76 from dual union all
select 46767,'1978-W76','1978-01-20 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 46768,'1978-W760','1978-07-19 16:42:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',65 from dual union all
select 46769,'1978-W761','1978-07-19 17:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',57 from dual union all
select 4677,'1957-S261','1957-10-24 00:00:00','military missile','success',969,'53040818','','','','OT',636,'',100 from dual union all
select 46770,'1978-W762','1978-07-19 18:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 46771,'1978-W763','1978-07-19 18:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 46772,'1978-W764','1978-07-19 23:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',63 from dual union all
select 46773,'1978-W765','1978-07-20 06:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 46774,'1978-W766','1978-07-20 11:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 46775,'1978-W767','1978-07-20 16:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46776,'1978-W768','1978-07-20 21:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 46777,'1978-W769','1978-07-21 02:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 46778,'1978-W77','1978-01-22 13:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 46779,'1978-W770','1978-07-21 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 4678,'1957-S262','1957-10-24 00:00:00','military missile','success',969,'53091511','','','','OT',636,'',100 from dual union all
select 46780,'1978-W771','1978-07-21 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 46781,'1978-W772','1978-07-21 22:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 46782,'1978-W773','1978-07-24 14:53:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 46783,'1978-W774','1978-07-24 14:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',61 from dual union all
select 46784,'1978-W775','1978-07-24 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 46785,'1978-W776','1978-07-24 17:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 46786,'1978-W777','1978-07-25 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 46787,'1978-W778','1978-07-25 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',52 from dual union all
select 46788,'1978-W779','1978-07-26 02:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 46789,'1978-W78','1978-01-23 14:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 4679,'1957-S263','1957-10-24 00:00:00','military missile','success',969,'53081319','','','','OT',636,'',100 from dual union all
select 46790,'1978-W780','1978-07-26 13:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 46791,'1978-W781','1978-07-26 14:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 46792,'1978-W782','1978-07-26 15:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 46793,'1978-W783','1978-07-26 16:07:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 46794,'1978-W784','1978-07-26 17:23:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',59 from dual union all
select 46795,'1978-W785','1978-07-26 20:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',72 from dual union all
select 46796,'1978-W786','1978-07-26 22:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 46797,'1978-W787','1978-07-28 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 46798,'1978-W788','1978-07-28 15:00:00','suborbital rocket','unknown',738,'','','','','Aeron',902,'KRENKEL''',59 from dual union all
select 46799,'1978-W789','1978-07-28 16:00:00','suborbital rocket','unknown',738,'','','','','Aeron',902,'KRENKEL''',58 from dual union all
select 468,'1944-S85','1944-09-11 07:02:00','military missile','success',1470,'','London','','','Weapon',1744,'',95 from dual union all
select 4680,'1957-S264','1957-10-26 00:00:00','military missile','success',992,'53050110','','','','OT',636,'',170 from dual union all
select 46800,'1978-W79','1978-01-23 16:21:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',54 from dual union all
select 46801,'1978-W790','1978-07-28 16:02:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 46802,'1978-W791','1978-07-28 16:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',55 from dual union all
select 46803,'1978-W792','1978-07-28 18:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 46804,'1978-W793','1978-07-28 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 46805,'1978-W794','1978-07-29 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',64 from dual union all
select 46806,'1978-W795','1978-07-30 15:00:00','suborbital rocket','unknown',738,'','','','','Aeron',902,'KRENKEL''',59 from dual union all
select 46807,'1978-W796','1978-07-30 16:00:00','suborbital rocket','unknown',738,'','','','','Aeron',902,'KRENKEL''',57 from dual union all
select 46808,'1978-W797','1978-07-31 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual;
insert into LAUNCH
select 46809,'1978-W798','1978-07-31 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 4681,'1957-S265','1957-10-27 00:00:00','military missile','success',992,'52120309','','','','OT',636,'',170 from dual union all
select 46810,'1978-W799','1978-07-31 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 46811,'1978-W80','1978-01-23 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 46812,'1978-W800','1978-07-31 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 46813,'1978-W801','1978-07-31 21:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 46814,'1978-W802','1978-08-02 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 46815,'1978-W803','1978-08-02 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 46816,'1978-W804','1978-08-02 16:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 46817,'1978-W805','1978-08-02 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 46818,'1978-W806','1978-08-02 18:00:00','suborbital rocket','unknown',738,'','','','','Aeron',902,'KRENKEL''',58 from dual union all
select 46819,'1978-W807','1978-08-02 22:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',68 from dual union all
select 4682,'1957-S266','1957-11-04 00:00:00','military missile','success',992,'52120308','','','','OT',636,'',170 from dual union all
select 46820,'1978-W808','1978-08-03 02:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 46821,'1978-W809','1978-08-03 22:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',66 from dual union all
select 46822,'1978-W81','1978-01-23 17:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 46823,'1978-W810','1978-08-04 15:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',62 from dual union all
select 46824,'1978-W811','1978-08-04 16:38:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 46825,'1978-W812','1978-08-04 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 46826,'1978-W813','1978-08-04 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46827,'1978-W814','1978-08-07 15:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',62 from dual union all
select 46828,'1978-W815','1978-08-07 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 46829,'1978-W816','1978-08-07 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',51 from dual union all
select 4683,'1957-S267','1957-09-26 00:00:00','military missile','success',1013,'M3-20','8K51 OT-9','','','Test',636,'',500 from dual union all
select 46830,'1978-W817','1978-08-07 17:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46831,'1978-W818','1978-08-07 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 46832,'1978-W819','1978-08-08 20:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',71 from dual union all
select 46833,'1978-W82','1978-01-23 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 46834,'1978-W820','1978-08-08 21:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 46835,'1978-W821','1978-08-09 14:51:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',50 from dual union all
select 46836,'1978-W822','1978-08-09 14:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',60 from dual union all
select 46837,'1978-W823','1978-08-09 15:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 46838,'1978-W824','1978-08-09 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46839,'1978-W825','1978-08-09 17:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 4684,'1957-S268','1957-11-03 00:00:00','military missile','success',1013,'M1-2','Generator-5 No. M1-2','','','Test',636,'',500 from dual union all
select 46840,'1978-W826','1978-08-09 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 46841,'1978-W827','1978-08-09 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',54 from dual union all
select 46842,'1978-W828','1978-08-10 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',65 from dual union all
select 46843,'1978-W829','1978-08-10 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 46844,'1978-W83','1978-01-23 20:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 46845,'1978-W830','1978-08-11 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 46846,'1978-W831','1978-08-11 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 46847,'1978-W832','1978-08-11 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 46848,'1978-W833','1978-08-11 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 46849,'1978-W834','1978-08-11 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',53 from dual union all
select 4685,'1957-S269','1957-09-27 00:00:00','military missile','success',971,'M689','','','','Test',636,'',150 from dual union all
select 46850,'1978-W835','1978-08-11 21:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 46851,'1978-W836','1978-08-14 14:33:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 46852,'1978-W837','1978-08-14 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 46853,'1978-W838','1978-08-14 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 46854,'1978-W839','1978-08-14 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 46855,'1978-W84','1978-01-23 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 46856,'1978-W840','1978-08-14 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 46857,'1978-W841','1978-08-15 00:00:00','suborbital rocket','success',1281,'NASA 15.158UU','','','','Aeron?',1821,'',61 from dual union all
select 46858,'1978-W842','1978-08-15 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46859,'1978-W843','1978-08-15 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 4686,'1957-S27','1957-04-14 00:00:00','military missile','success',994,'','','','','Test',636,'',100 from dual union all
select 46860,'1978-W844','1978-08-16 13:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 46861,'1978-W845','1978-08-16 14:00:00','suborbital rocket','unknown',725,'ISRO 8.389','','','','Aeron',1467,'',74 from dual union all
select 46862,'1978-W846','1978-08-16 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 46863,'1978-W847','1978-08-16 17:06:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',50 from dual union all
select 46864,'1978-W848','1978-08-16 17:07:00','suborbital rocket','unknown',1284,'FLIGHT 197     /T 1-','','','','Aeron',1821,'',68 from dual union all
select 46865,'1978-W849','1978-08-16 17:29:00','suborbital rocket','unknown',1284,'FLIGHT 198     /T 1-','','','','Aeron',1821,'',74 from dual union all
select 46866,'1978-W85','1978-01-24 15:46:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 46867,'1978-W850','1978-08-16 18:16:00','suborbital rocket','unknown',1284,'FLIGHT 199     /TH1-','','','','Aeron',428,'',69 from dual union all
select 46868,'1978-W851','1978-08-16 18:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 46869,'1978-W852','1978-08-16 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 4687,'1957-S270','1957-09-27 00:00:00','military missile','success',971,'M681','','','','Test',636,'',150 from dual union all
select 46870,'1978-W853','1978-08-16 20:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 46871,'1978-W854','1978-08-16 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 46872,'1978-W855','1978-08-16 21:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46873,'1978-W856','1978-08-18 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 46874,'1978-W857','1978-08-18 16:03:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 46875,'1978-W858','1978-08-18 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46876,'1978-W859','1978-08-18 16:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 46877,'1978-W86','1978-01-24 20:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46878,'1978-W860','1978-08-18 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46879,'1978-W861','1978-08-18 20:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',61 from dual union all
select 4688,'1957-S271','1957-10-06 00:00:00','military missile','success',971,'M6816','','','','Test',636,'',150 from dual union all
select 46880,'1978-W862','1978-08-19 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 46881,'1978-W863','1978-08-21 14:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46882,'1978-W864','1978-08-21 15:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 46883,'1978-W865','1978-08-21 15:48:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 46884,'1978-W866','1978-08-21 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 46885,'1978-W867','1978-08-21 18:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',64 from dual union all
select 46886,'1978-W868','1978-08-21 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 46887,'1978-W869','1978-08-22 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 46888,'1978-W87','1978-01-24 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 46889,'1978-W870','1978-08-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',78 from dual union all
select 4689,'1957-S272','1957-11-18 00:00:00','military missile','success',971,'M6812','','','','Test',636,'',150 from dual union all
select 46890,'1978-W871','1978-08-23 14:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 46891,'1978-W872','1978-08-23 16:22:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',54 from dual union all
select 46892,'1978-W873','1978-08-23 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 46893,'1978-W874','1978-08-23 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 46894,'1978-W875','1978-08-23 17:34:00','suborbital rocket','unknown',1284,'FLIGHT 200     /T 1-','','','','Aeron',1821,'',64 from dual union all
select 46895,'1978-W876','1978-08-23 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 46896,'1978-W877','1978-08-23 18:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',64 from dual union all
select 46897,'1978-W878','1978-08-23 20:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 46898,'1978-W879','1978-08-23 23:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',67 from dual union all
select 46899,'1978-W88','1978-01-24 23:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual;
insert into LAUNCH
select 469,'1944-S86','1944-09-11 07:25:00','military missile','success',1470,'','London','','','Weapon',1744,'',95 from dual union all
select 4690,'1957-S273','1957-11-20 00:00:00','military missile','success',971,'M68688','','','','Test',636,'',150 from dual union all
select 46900,'1978-W880','1978-08-24 01:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',79 from dual union all
select 46901,'1978-W881','1978-08-24 01:45:00','suborbital rocket','success',761,'MT135P T22','','','','Aeron?',1476,'',50 from dual union all
select 46902,'1978-W882','1978-08-24 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46903,'1978-W883','1978-08-24 21:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 46904,'1978-W884','1978-08-25 14:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 46905,'1978-W885','1978-08-25 16:52:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 46906,'1978-W886','1978-08-25 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46907,'1978-W887','1978-08-25 17:03:00','suborbital rocket','unknown',1284,'FLIGHT 201     /T 1-','','','','Aeron',1821,'',64 from dual union all
select 46908,'1978-W888','1978-08-25 18:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 46909,'1978-W889','1978-08-26 01:30:00','suborbital rocket','success',761,'MT135P T23','','','','Aeron?',1476,'',50 from dual union all
select 4691,'1957-S274','1957-11-26 00:00:00','military missile','success',971,'M6814','','','','Test',636,'',150 from dual union all
select 46910,'1978-W89','1978-01-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',78 from dual union all
select 46911,'1978-W890','1978-08-28 00:00:00','suborbital rocket','success',1281,'NASA 15.157UU','','','','Aeron?',1821,'',63 from dual union all
select 46912,'1978-W891','1978-08-28 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 46913,'1978-W892','1978-08-28 16:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 46914,'1978-W893','1978-08-28 16:29:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 46915,'1978-W894','1978-08-28 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46916,'1978-W895','1978-08-28 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 46917,'1978-W896','1978-08-29 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',63 from dual union all
select 46918,'1978-W897','1978-08-29 19:37:59','suborbital rocket','success',1281,'','MidAir Recovery Test','','','Aeron/Test',1821,'',66 from dual union all
select 46919,'1978-W898','1978-08-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',79 from dual union all
select 4692,'1957-S275','1957-11-27 00:00:00','military missile','success',971,'M6811','','','','Test',636,'',150 from dual union all
select 46920,'1978-W899','1978-08-30 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 46921,'1978-W90','1978-01-25 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 46922,'1978-W900','1978-08-30 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',62 from dual union all
select 46923,'1978-W901','1978-08-30 15:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',61 from dual union all
select 46924,'1978-W902','1978-08-30 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46925,'1978-W903','1978-08-30 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 46926,'1978-W904','1978-08-30 19:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 46927,'1978-W905','1978-08-30 20:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 46928,'1978-W906','1978-08-30 23:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',63 from dual union all
select 46929,'1978-W907','1978-08-31 13:03:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',55 from dual union all
select 4693,'1957-S276','1957-11-29 00:00:00','military missile','success',971,'M6817','','','','Test',636,'',150 from dual union all
select 46930,'1978-W908','1978-08-31 13:04:00','suborbital rocket','unknown',1284,'FLIGHT 202     /T 1-','','','','Aeron',1821,'',61 from dual union all
select 46931,'1978-W909','1978-08-31 13:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 46932,'1978-W91','1978-01-25 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 46933,'1978-W910','1978-08-31 17:08:00','suborbital rocket','unknown',1284,'FLIGHT 203     /T 1-','','','','Aeron',1821,'',63 from dual union all
select 46934,'1978-W911','1978-08-31 18:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',57 from dual union all
select 46935,'1978-W912','1978-08-31 20:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 46936,'1978-W913','1978-09-01 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',902,'KRENKEL''',59 from dual union all
select 46937,'1978-W914','1978-09-01 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 46938,'1978-W915','1978-09-01 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 46939,'1978-W916','1978-09-01 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 4694,'1957-S277','1957-12-16 00:00:00','military missile','success',971,'M2-2','','','','Test',636,'',150 from dual union all
select 46940,'1978-W917','1978-09-01 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 46941,'1978-W918','1978-09-01 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46942,'1978-W919','1978-09-01 18:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 46943,'1978-W92','1978-01-25 16:35:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 46944,'1978-W920','1978-09-01 21:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 46945,'1978-W921','1978-09-02 00:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 46946,'1978-W922','1978-09-02 01:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 46947,'1978-W923','1978-09-04 15:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 46948,'1978-W924','1978-09-05 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 46949,'1978-W925','1978-09-06 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',902,'KRENKEL''',57 from dual union all
select 4695,'1957-S278','1957-12-16 00:00:00','military missile','success',971,'M2-25','','','','Test',636,'',150 from dual union all
select 46950,'1978-W926','1978-09-06 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46951,'1978-W927','1978-09-06 14:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 46952,'1978-W928','1978-09-06 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',59 from dual union all
select 46953,'1978-W929','1978-09-06 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 46954,'1978-W93','1978-01-25 16:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46955,'1978-W930','1978-09-06 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 46956,'1978-W931','1978-09-06 21:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',63 from dual union all
select 46957,'1978-W932','1978-09-06 22:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 46958,'1978-W933','1978-09-07 03:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 46959,'1978-W934','1978-09-08 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',902,'KRENKEL''',61 from dual union all
select 4696,'1957-S279','1957-12-26 00:00:00','military missile','success',971,'M3-25T','','','','Test',636,'',150 from dual union all
select 46960,'1978-W935','1978-09-08 12:17:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 46961,'1978-W936','1978-09-08 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 46962,'1978-W937','1978-09-08 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 46963,'1978-W938','1978-09-08 18:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',61 from dual union all
select 46964,'1978-W939','1978-09-08 18:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 46965,'1978-W94','1978-01-25 16:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 46966,'1978-W940','1978-09-09 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 46967,'1978-W941','1978-09-11 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46968,'1978-W942','1978-09-11 16:15:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 46969,'1978-W943','1978-09-11 18:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 4697,'1957-S28','1957-05-28 00:00:00','suborbital rocket','success',813,'','','','','Test',313,'',90 from dual union all
select 46970,'1978-W944','1978-09-11 18:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',62 from dual union all
select 46971,'1978-W945','1978-09-11 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',59 from dual union all
select 46972,'1978-W946','1978-09-12 03:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 46973,'1978-W947','1978-09-12 21:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 46974,'1978-W948','1978-09-13 01:22:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',52 from dual union all
select 46975,'1978-W949','1978-09-13 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',902,'KRENKEL''',60 from dual union all
select 46976,'1978-W95','1978-01-25 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 46977,'1978-W950','1978-09-13 11:37:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 46978,'1978-W951','1978-09-13 15:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 46979,'1978-W952','1978-09-13 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 4698,'1957-S280','1957-12-28 00:00:00','military missile','success',971,'M3-9','','','','Test',636,'',150 from dual union all
select 46980,'1978-W953','1978-09-13 15:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 46981,'1978-W954','1978-09-13 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',56 from dual union all
select 46982,'1978-W955','1978-09-13 17:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 46983,'1978-W956','1978-09-13 22:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 46984,'1978-W957','1978-09-13 23:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',65 from dual union all
select 46985,'1978-W958','1978-09-14 21:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',65 from dual union all
select 46986,'1978-W959','1978-09-15 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',902,'KRENKEL''',60 from dual union all
select 46987,'1978-W96','1978-01-25 18:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 46988,'1978-W960','1978-09-15 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 46989,'1978-W961','1978-09-15 15:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual;
insert into LAUNCH
select 4699,'1957-S281','1957-12-31 00:00:00','military missile','success',971,'M4-4T','','','','Test',636,'',150 from dual union all
select 46990,'1978-W962','1978-09-15 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 46991,'1978-W963','1978-09-15 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 46992,'1978-W964','1978-09-18 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 46993,'1978-W965','1978-09-18 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 46994,'1978-W966','1978-09-18 16:42:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 46995,'1978-W967','1978-09-18 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 46996,'1978-W968','1978-09-18 17:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 46997,'1978-W969','1978-09-19 01:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 46998,'1978-W97','1978-01-25 21:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 46999,'1978-W970','1978-09-19 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 47,'1944-A11','1944-08-17 12:56:00','military missile','failure',1470,'18300','Ma116','','','Training',661,'',10 from dual union all
select 470,'1944-S87','1944-09-12 06:10:00','military missile','success',1470,'','London','','','Weapon',1744,'',95 from dual union all
select 4700,'1957-S282','1957-12-31 00:00:00','military missile','success',971,'M4-6','','','','Test',636,'',150 from dual union all
select 47000,'1978-W971','1978-09-20 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',902,'KRENKEL''',59 from dual union all
select 47001,'1978-W972','1978-09-20 13:14:00','suborbital rocket','unknown',1284,'FLIGHT 204     /T 1-','','','','Aeron',1821,'',57 from dual union all
select 47002,'1978-W973','1978-09-20 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 47003,'1978-W974','1978-09-20 15:29:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',62 from dual union all
select 47004,'1978-W975','1978-09-20 15:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 47005,'1978-W976','1978-09-20 17:00:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 47006,'1978-W977','1978-09-20 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',53 from dual union all
select 47007,'1978-W978','1978-09-20 17:32:00','suborbital rocket','unknown',1284,'FLIGHT 205     /T 1-','','','','Aeron',1821,'',64 from dual union all
select 47008,'1978-W979','1978-09-20 17:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 47009,'1978-W98','1978-01-25 22:07:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 4701,'1957-S283','1957-11-30 00:00:00','military missile','success',1013,'M4-20','8K51 OT-10','','','Test',636,'',500 from dual union all
select 47010,'1978-W980','1978-09-20 18:00:00','suborbital rocket','unknown',1284,'FLIGHT 206     /TH1-','','','','Aeron',428,'',60 from dual union all
select 47011,'1978-W981','1978-09-20 18:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 47012,'1978-W982','1978-09-20 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 47013,'1978-W983','1978-09-20 20:30:00','suborbital rocket','unknown',1284,'FLIGHT 207     /T 1-','','','','Aeron',1821,'',63 from dual union all
select 47014,'1978-W984','1978-09-20 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 47015,'1978-W985','1978-09-20 21:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 47016,'1978-W986','1978-09-20 21:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 47017,'1978-W987','1978-09-22 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',902,'KRENKEL''',60 from dual union all
select 47018,'1978-W988','1978-09-22 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 47019,'1978-W989','1978-09-22 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 4702,'1957-S284','1957-12-26 00:00:00','military missile','success',1013,'M1-3','Generator-5 No. M1-3','','','Test',636,'',500 from dual union all
select 47020,'1978-W99','1978-01-26 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 47021,'1978-W990','1978-09-22 14:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 47022,'1978-W991','1978-09-22 15:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',55 from dual union all
select 47023,'1978-W992','1978-09-22 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 47024,'1978-W993','1978-09-22 20:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 47025,'1978-W994','1978-09-22 21:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 47026,'1978-W995','1978-09-25 14:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 47027,'1978-W996','1978-09-25 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 47028,'1978-W997','1978-09-25 16:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 47029,'1978-W998','1978-09-25 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 4703,'1957-S285','1957-08-19 00:00:00','suborbital rocket','success',28,'USAF 83','Na','','','Aeron',640,'',178 from dual union all
select 47030,'1978-W999','1978-09-25 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',53 from dual union all
select 47031,'1978-Y01','1978-01-04 11:40:00','suborbital rocket','success',1281,'NASA 15.162UE','','','','Plasma',1409,'',80 from dual union all
select 47032,'1978-Y02','1978-01-04 00:00:00','suborbital rocket','success',1281,'NASA 15.161UE','','','','Plasma',1409,'',80 from dual union all
select 47033,'1978-Y03','1978-01-04 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 47034,'1978-Y04','1978-01-04 14:00:00','suborbital rocket','unknown',725,'ISRO 8.356','','','','Aeron',1467,'',86 from dual union all
select 47035,'1978-Y05','1978-01-04 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 47036,'1978-Y06','1978-01-04 17:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47037,'1978-Y07','1978-01-04 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',88 from dual union all
select 47038,'1978-Y08','1978-01-05 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 47039,'1978-Y09','1978-01-05 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',82 from dual union all
select 4704,'1957-S29','1957-05-01 00:00:00','suborbital rocket','success',813,'','','','','Test',313,'',90 from dual union all
select 47040,'1978-Y10','1978-01-10 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',81 from dual union all
select 47041,'1978-Y100','1978-03-03 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 47042,'1978-Y101','1978-03-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 47043,'1978-Y102','1978-03-05 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',91 from dual union all
select 47044,'1978-Y103','1978-03-05 10:15:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',83 from dual union all
select 47045,'1978-Y104','1978-03-07 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 47046,'1978-Y105','1978-03-07 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 47047,'1978-Y106','1978-03-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 47048,'1978-Y107','1978-03-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 47049,'1978-Y108','1978-03-08 07:40:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',89 from dual union all
select 4705,'1957-S30','1957-04-30 15:10:00','suborbital rocket','success',49,'NRL 41 (RV-N-13c)','RV-N-13c test','','','Test/Meteorites',1869,'',289 from dual union all
select 47050,'1978-Y109','1978-03-08 09:10:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',83 from dual union all
select 47051,'1978-Y11','1978-01-11 10:15:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 47052,'1978-Y110','1978-03-08 14:00:00','suborbital rocket','unknown',725,'ISRO 8.365','','','','Aeron',1467,'',84 from dual union all
select 47053,'1978-Y111','1978-03-08 17:37:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 47054,'1978-Y112','1978-03-15 14:00:00','suborbital rocket','unknown',725,'ISRO 8.366','','','','Aeron',1467,'',83 from dual union all
select 47055,'1978-Y113','1978-03-15 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 47056,'1978-Y114','1978-03-15 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47057,'1978-Y115','1978-03-15 18:03:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual union all
select 47058,'1978-Y116','1978-03-15 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 47059,'1978-Y117','1978-03-15 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 4706,'1957-S31','1957-05-01 06:29:00','test rocket','success',1500,'14/TV1','Vanguard TV1','','','Test',235,'',195 from dual union all
select 47060,'1978-Y118','1978-03-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',80 from dual union all
select 47061,'1978-Y119','1978-03-16 21:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 47062,'1978-Y12','1978-01-11 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',90 from dual union all
select 47063,'1978-Y120','1978-03-19 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1375,'SHIR',88 from dual union all
select 47064,'1978-Y121','1978-03-22 13:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 47065,'1978-Y122','1978-03-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 47066,'1978-Y123','1978-03-22 14:00:00','suborbital rocket','unknown',725,'ISRO 8.367','','','','Aeron',1467,'',85 from dual union all
select 47067,'1978-Y124','1978-03-22 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 47068,'1978-Y125','1978-03-22 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47069,'1978-Y126','1978-03-22 20:55:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 4707,'1957-S32','1957-05-03 14:04:00','suborbital rocket','success',49,'NRL 44 (RV-N-13c)','Sunfollower/UVS','','','Solar UV',1869,'',204 from dual union all
select 47070,'1978-Y127','1978-03-26 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1375,'SHIR',92 from dual union all
select 47071,'1978-Y128','1978-03-26 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1375,'SHIR',87 from dual union all
select 47072,'1978-Y129','1978-03-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 47073,'1978-Y13','1978-01-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 47074,'1978-Y130','1978-03-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 47075,'1978-Y131','1978-03-29 14:00:00','suborbital rocket','unknown',725,'ISRO 8.368','','','','Aeron',1467,'',85 from dual union all
select 47076,'1978-Y132','1978-03-29 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 47077,'1978-Y133','1978-03-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 47078,'1978-Y134','1978-03-29 17:40:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual;
insert into LAUNCH
select 47079,'1978-Y135','1978-03-30 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1375,'SHIR',88 from dual union all
select 4708,'1957-S33','1957-05-15 07:55:00','test rocket','success',637,'RS-34    NT','Jupiter C/Jupiter RV','','','Reentry Test',280,'',655 from dual union all
select 47080,'1978-Y136','1978-03-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1375,'SHIR',83 from dual union all
select 47081,'1978-Y137','1978-03-31 14:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 47082,'1978-Y138','1978-03-31 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 47083,'1978-Y139','1978-04-03 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',886,'KORO',88 from dual union all
select 47084,'1978-Y14','1978-01-11 14:00:00','suborbital rocket','unknown',725,'ISRO 8.357','','','','Aeron',1467,'',86 from dual union all
select 47085,'1978-Y140','1978-04-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',886,'KORO',85 from dual union all
select 47086,'1978-Y141','1978-04-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 47087,'1978-Y142','1978-04-05 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 47088,'1978-Y143','1978-04-05 16:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47089,'1978-Y144','1978-04-05 17:27:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 4709,'1957-S34','1957-05-16 02:14:00','suborbital rocket','success',994,'No. 1','','','','Test',636,'',200 from dual union all
select 47090,'1978-Y145','1978-04-05 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 47091,'1978-Y146','1978-04-05 20:26:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 47092,'1978-Y147','1978-04-06 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',886,'KORO',83 from dual union all
select 47093,'1978-Y148','1978-04-07 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',886,'KORO',87 from dual union all
select 47094,'1978-Y149','1978-04-07 12:30:00','suborbital rocket','unknown',725,'','','','','Aeron',886,'KORO',85 from dual union all
select 47095,'1978-Y15','1978-01-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 47096,'1978-Y150','1978-04-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 47097,'1978-Y151','1978-04-07 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 47098,'1978-Y152','1978-04-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 47099,'1978-Y153','1978-04-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 471,'1944-S88','1944-09-12 06:14:00','military missile','success',1470,'','London','','','Weapon',1744,'',95 from dual union all
select 4710,'1957-S35','1957-05-16 03:18:00','suborbital rocket','success',994,'','','','','Test/Bio',636,'',200 from dual union all
select 47100,'1978-Y154','1978-04-12 16:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 47101,'1978-Y155','1978-04-12 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 47102,'1978-Y156','1978-04-12 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 47103,'1978-Y157','1978-04-19 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 47104,'1978-Y158','1978-04-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 47105,'1978-Y159','1978-04-19 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 47106,'1978-Y16','1978-01-11 20:02:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47107,'1978-Y160','1978-04-19 18:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47108,'1978-Y161','1978-04-19 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47109,'1978-Y162','1978-04-19 20:07:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 4711,'1957-S36','1957-05-24 00:00:00','suborbital rocket','success',994,'No. 2','','','','Test/Bio',636,'',200 from dual union all
select 47110,'1978-Y163','1978-04-20 17:00:00','suborbital rocket','unknown',725,'ISRO 8.369','','','','Aeron',1467,'',80 from dual union all
select 47111,'1978-Y164','1978-04-21 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',886,'KORO',84 from dual union all
select 47112,'1978-Y165','1978-04-22 08:35:00','suborbital rocket','unknown',725,'','','','','Aeron',886,'KORO',88 from dual union all
select 47113,'1978-Y166','1978-04-22 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',886,'KORO',88 from dual union all
select 47114,'1978-Y167','1978-04-23 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',886,'KORO',83 from dual union all
select 47115,'1978-Y168','1978-04-24 08:50:00','suborbital rocket','unknown',725,'','','','','Aeron',886,'KORO',87 from dual union all
select 47116,'1978-Y169','1978-04-25 08:50:00','suborbital rocket','unknown',725,'','','','','Aeron',886,'KORO',84 from dual union all
select 47117,'1978-Y17','1978-01-11 22:33:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual union all
select 47118,'1978-Y170','1978-04-26 04:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 47119,'1978-Y171','1978-04-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 4712,'1957-S37','1957-05-31 18:08:00','military missile','success',638,'AM-1','R&D','','','Test',279,'',500 from dual union all
select 47120,'1978-Y172','1978-04-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 47121,'1978-Y173','1978-04-26 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47122,'1978-Y174','1978-04-26 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',886,'KORO',81 from dual union all
select 47123,'1978-Y175','1978-04-27 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',886,'KORO',81 from dual union all
select 47124,'1978-Y176','1978-05-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 47125,'1978-Y177','1978-05-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 47126,'1978-Y178','1978-05-03 15:58:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 47127,'1978-Y179','1978-05-10 14:00:00','suborbital rocket','unknown',725,'ISRO 8.370','','','','Aeron',1467,'',81 from dual union all
select 47128,'1978-Y18','1978-01-12 09:57:30','suborbital rocket','success',1281,'NASA 15.163UE','','','','Plasma',1409,'',80 from dual union all
select 47129,'1978-Y180','1978-05-10 14:17:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 4713,'1957-S38','1957-06-07 00:00:00','military missile','success',994,'[These dubious]','NIIP-A target','','','Target',634,'',100 from dual union all
select 47130,'1978-Y181','1978-05-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 47131,'1978-Y182','1978-05-10 17:49:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47132,'1978-Y183','1978-05-10 19:26:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 47133,'1978-Y184','1978-05-10 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 47134,'1978-Y185','1978-05-15 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47135,'1978-Y186','1978-05-15 20:52:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',96 from dual union all
select 47136,'1978-Y187','1978-05-17 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 47137,'1978-Y188','1978-05-17 14:00:00','suborbital rocket','unknown',725,'ISRO 8.372','','','','Aeron',1467,'',82 from dual union all
select 47138,'1978-Y189','1978-05-17 16:19:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47139,'1978-Y19','1978-01-12 08:20:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',84 from dual union all
select 4714,'1957-S39','1957-06-10 00:00:00','military missile','success',994,'','NIIP-A target','','','Target',634,'',100 from dual union all
select 47140,'1978-Y190','1978-05-17 16:36:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',86 from dual union all
select 47141,'1978-Y191','1978-05-17 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 47142,'1978-Y192','1978-05-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 47143,'1978-Y193','1978-05-19 14:00:00','suborbital rocket','unknown',725,'ISRO 8.373','','','','Aeron',1467,'',81 from dual union all
select 47144,'1978-Y194','1978-05-23 14:00:00','suborbital rocket','unknown',725,'ISRO 8.374','','','','Aeron',1467,'',81 from dual union all
select 47145,'1978-Y195','1978-05-24 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 47146,'1978-Y196','1978-05-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 47147,'1978-Y197','1978-05-24 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 47148,'1978-Y198','1978-05-24 21:32:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 47149,'1978-Y199','1978-05-26 00:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',88 from dual union all
select 4715,'1957-S40','1957-06-18 14:00:00','suborbital rocket','success',49,'USAF 78','AU4.26','IGY AU4.26','','Ionos',640,'',171 from dual union all
select 47150,'1978-Y20','1978-01-12 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',84 from dual union all
select 47151,'1978-Y200','1978-05-26 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 47152,'1978-Y201','1978-05-29 14:00:00','suborbital rocket','unknown',725,'ISRO 8.375','','','','Aeron',1467,'',81 from dual union all
select 47153,'1978-Y202','1978-05-30 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 47154,'1978-Y203','1978-05-31 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 47155,'1978-Y204','1978-05-31 14:00:00','suborbital rocket','unknown',725,'ISRO 8.376','','','','Aeron',1467,'',81 from dual union all
select 47156,'1978-Y205','1978-05-31 17:54:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual union all
select 47157,'1978-Y206','1978-05-31 18:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 47158,'1978-Y207','1978-05-31 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 47159,'1978-Y208','1978-06-02 14:00:00','suborbital rocket','unknown',725,'ISRO 8.377','','','','Aeron',1467,'',84 from dual union all
select 4716,'1957-S41','1957-06-22 00:00:00','military missile','success',976,'M2-3','63M No. 1 LKI1-1','','','Test',636,'',402 from dual union all
select 47160,'1978-Y209','1978-06-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 47161,'1978-Y21','1978-01-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 47162,'1978-Y210','1978-06-07 14:00:00','suborbital rocket','unknown',725,'ISRO 8.378','','','','Aeron',1467,'',82 from dual union all
select 47163,'1978-Y211','1978-06-07 15:06:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',89 from dual union all
select 47164,'1978-Y212','1978-06-07 16:19:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 47165,'1978-Y213','1978-06-07 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 47166,'1978-Y214','1978-06-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 47167,'1978-Y215','1978-06-13 22:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 47168,'1978-Y216','1978-06-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual;
insert into LAUNCH
select 47169,'1978-Y217','1978-06-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 4717,'1957-S42','1957-06-22 00:00:00','military missile','success',994,'','NIIP-A target','','','Target',634,'',100 from dual union all
select 47170,'1978-Y218','1978-06-14 15:06:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 47171,'1978-Y219','1978-06-14 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47172,'1978-Y22','1978-01-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',82 from dual union all
select 47173,'1978-Y220','1978-06-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 47174,'1978-Y221','1978-06-20 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 47175,'1978-Y222','1978-06-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 47176,'1978-Y223','1978-06-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 47177,'1978-Y224','1978-06-21 16:13:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',91 from dual union all
select 47178,'1978-Y225','1978-06-21 16:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47179,'1978-Y226','1978-06-21 17:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 4718,'1957-S43','1957-06-23 00:00:00','military missile','success',994,'','NIIP-A target','','','Target',634,'',100 from dual union all
select 47180,'1978-Y227','1978-06-21 17:55:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 47181,'1978-Y228','1978-06-22 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 47182,'1978-Y229','1978-06-22 14:00:00','suborbital rocket','unknown',725,'ISRO 8.380','','','','Aeron',1467,'',84 from dual union all
select 47183,'1978-Y23','1978-01-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',84 from dual union all
select 47184,'1978-Y230','1978-06-22 14:12:00','suborbital rocket','unknown',728,'ISRO 8.','','','','Aeron',1467,'',84 from dual union all
select 47185,'1978-Y231','1978-06-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 47186,'1978-Y232','1978-06-27 21:05:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 47187,'1978-Y233','1978-06-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 47188,'1978-Y234','1978-06-28 14:00:00','suborbital rocket','unknown',725,'ISRO 8.381','','','','Aeron',1467,'',83 from dual union all
select 47189,'1978-Y235','1978-06-28 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 4719,'1957-S44','1957-06-25 14:07:00','suborbital rocket','success',49,'USAF 79','AU4.27','IGY AU4.27','','Ionos',640,'',202 from dual union all
select 47190,'1978-Y236','1978-06-28 17:25:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 47191,'1978-Y237','1978-06-28 18:53:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 47192,'1978-Y238','1978-06-28 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 47193,'1978-Y239','1978-06-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 47194,'1978-Y24','1978-01-18 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',81 from dual union all
select 47195,'1978-Y240','1978-07-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 47196,'1978-Y241','1978-07-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 47197,'1978-Y242','1978-07-05 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',80 from dual union all
select 47198,'1978-Y243','1978-07-05 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47199,'1978-Y244','1978-07-05 17:22:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 472,'1944-S89','1944-09-12 06:47:00','military missile','success',1470,'','London','','','Weapon',1744,'',95 from dual union all
select 4720,'1957-S45','1957-06-26 11:09:00','military missile','success',635,'CC-31    NH','Jupiter A','','','Test',280,'',90 from dual union all
select 47200,'1978-Y245','1978-07-05 19:57:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 47201,'1978-Y246','1978-07-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 47202,'1978-Y247','1978-07-07 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 47203,'1978-Y248','1978-07-10 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 47204,'1978-Y249','1978-07-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 47205,'1978-Y25','1978-01-18 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',82 from dual union all
select 47206,'1978-Y250','1978-07-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 47207,'1978-Y251','1978-07-12 14:00:00','suborbital rocket','unknown',725,'ISRO 8.383','','','','Aeron',1467,'',85 from dual union all
select 47208,'1978-Y252','1978-07-12 14:07:00','suborbital rocket','unknown',728,'ISRO 8.','','','','Aeron',1467,'',85 from dual union all
select 47209,'1978-Y253','1978-07-12 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 4721,'1957-S46','1957-06-27 00:00:00','military missile','success',936,'4-204-1','','','','Reentry Test',253,'',10 from dual union all
select 47210,'1978-Y254','1978-07-12 16:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47211,'1978-Y255','1978-07-12 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47212,'1978-Y256','1978-07-12 17:15:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47213,'1978-Y257','1978-07-12 18:55:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 47214,'1978-Y258','1978-07-13 14:00:00','suborbital rocket','unknown',725,'ISRO 8.384','','','','Aeron',1467,'',84 from dual union all
select 47215,'1978-Y259','1978-07-13 22:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 47216,'1978-Y26','1978-01-18 12:55:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 47217,'1978-Y260','1978-07-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 47218,'1978-Y261','1978-07-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 47219,'1978-Y262','1978-07-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 4722,'1957-S47','1957-06-28 00:00:00','military missile','success',994,'','NIIP-A target','','','Target',634,'',100 from dual union all
select 47220,'1978-Y263','1978-07-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 47221,'1978-Y264','1978-07-19 14:34:00','suborbital rocket','unknown',728,'ISRO 8.385','','','','Aeron',1467,'',82 from dual union all
select 47222,'1978-Y265','1978-07-19 15:00:00','suborbital rocket','unknown',725,'ISRO 8.','','','','Aeron',1467,'',82 from dual union all
select 47223,'1978-Y266','1978-07-19 15:23:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 47224,'1978-Y267','1978-07-19 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47225,'1978-Y268','1978-07-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 47226,'1978-Y269','1978-07-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 47227,'1978-Y27','1978-01-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 47228,'1978-Y270','1978-07-26 14:00:00','suborbital rocket','unknown',725,'ISRO 8.386','','','','Aeron',1467,'',82 from dual union all
select 47229,'1978-Y271','1978-07-26 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 4723,'1957-S48','1957-06-29 00:00:00','military missile','success',994,'','NIIP-A target','','','Target',634,'',100 from dual union all
select 47230,'1978-Y272','1978-07-26 16:52:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 47231,'1978-Y273','1978-07-26 20:18:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 47232,'1978-Y274','1978-07-26 22:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47233,'1978-Y275','1978-08-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 47234,'1978-Y276','1978-08-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 47235,'1978-Y277','1978-08-02 15:00:00','suborbital rocket','unknown',725,'ISRO 8.387','','','','Aeron',1467,'',84 from dual union all
select 47236,'1978-Y278','1978-08-02 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 47237,'1978-Y279','1978-08-02 16:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47238,'1978-Y28','1978-01-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 47239,'1978-Y280','1978-08-02 16:24:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 4724,'1957-S49','1957-07-04 00:00:00','military missile','success',976,'M2-4','63M No. 2 LKI1-2','','','Test',636,'',402 from dual union all
select 47240,'1978-Y281','1978-08-02 20:20:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47241,'1978-Y282','1978-08-02 21:36:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 47242,'1978-Y283','1978-08-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 47243,'1978-Y284','1978-08-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 47244,'1978-Y285','1978-08-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 47245,'1978-Y286','1978-08-09 14:00:00','suborbital rocket','unknown',728,'ISRO 8.388','','','','Aeron',1467,'',83 from dual union all
select 47246,'1978-Y287','1978-08-09 15:00:00','suborbital rocket','unknown',725,'ISRO 8.','','','','Aeron',1467,'',83 from dual union all
select 47247,'1978-Y288','1978-08-09 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',86 from dual union all
select 47248,'1978-Y289','1978-08-09 17:01:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 47249,'1978-Y29','1978-01-18 14:00:00','suborbital rocket','unknown',725,'ISRO 8.358','','','','Aeron',1467,'',86 from dual union all
select 4725,'1957-S50','1957-07-13 00:00:00','military missile','success',976,'M3-5','63M No. 3 LKI1-3','','','Test',636,'',402 from dual union all
select 47250,'1978-Y290','1978-08-09 17:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47251,'1978-Y291','1978-08-09 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 47252,'1978-Y292','1978-08-09 21:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47253,'1978-Y293','1978-08-10 14:20:00','suborbital rocket','unknown',728,'','','','','Aeron/Ionos',636,'',85 from dual union all
select 47254,'1978-Y294','1978-08-10 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 47255,'1978-Y295','1978-08-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 47256,'1978-Y296','1978-08-14 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 47257,'1978-Y297','1978-08-14 14:20:00','suborbital rocket','unknown',728,'','','','','Aeron/Ionos',636,'',85 from dual union all
select 47258,'1978-Y298','1978-08-14 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual;
insert into LAUNCH
select 47259,'1978-Y299','1978-08-16 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',81 from dual union all
select 4726,'1957-S51','1957-07-27 00:00:00','military missile','success',976,'M4-7','63M No. 4 LKI1-4','','','Test',636,'',402 from dual union all
select 47260,'1978-Y30','1978-01-18 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 47261,'1978-Y300','1978-08-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 47262,'1978-Y301','1978-08-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 47263,'1978-Y302','1978-08-16 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',94 from dual union all
select 47264,'1978-Y303','1978-08-16 16:58:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 47265,'1978-Y304','1978-08-16 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 47266,'1978-Y305','1978-08-16 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 47267,'1978-Y306','1978-08-16 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47268,'1978-Y307','1978-08-16 22:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 47269,'1978-Y308','1978-08-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 4727,'1957-S52','1957-07-01 19:00:00','suborbital rocket','success',841,'NN7.37F','NRL NN7.37F','IGY NN7.37F','','Solar UV/X',1415,'',93 from dual union all
select 47270,'1978-Y309','1978-08-18 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 47271,'1978-Y31','1978-01-18 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 47272,'1978-Y310','1978-08-19 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 47273,'1978-Y311','1978-08-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 47274,'1978-Y312','1978-08-23 14:00:00','suborbital rocket','unknown',725,'ISRO 8.390','','','','Aeron',1467,'',84 from dual union all
select 47275,'1978-Y313','1978-08-23 14:12:00','suborbital rocket','unknown',728,'ISRO 8.391','','','','Aeron',1467,'',85 from dual union all
select 47276,'1978-Y314','1978-08-23 16:25:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 47277,'1978-Y315','1978-08-23 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47278,'1978-Y316','1978-08-23 17:00:00','suborbital rocket','unknown',728,'','','','','Aeron',856,'',89 from dual union all
select 47279,'1978-Y317','1978-08-23 20:34:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 4728,'1957-S53','1957-07-03 16:29:00','suborbital rocket','success',841,'NN7.38F','NRL NN7.38F','IGY NN7.38F','','Solar UV/X',1415,'',113 from dual union all
select 47280,'1978-Y318','1978-08-25 14:17:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 47281,'1978-Y319','1978-08-28 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 47282,'1978-Y32','1978-01-18 20:54:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',87 from dual union all
select 47283,'1978-Y320','1978-08-29 16:06:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 47284,'1978-Y321','1978-08-30 04:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 47285,'1978-Y322','1978-08-30 04:20:00','suborbital rocket','unknown',728,'','','','','Aeron/Ionos',636,'',85 from dual union all
select 47286,'1978-Y323','1978-08-30 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',92 from dual union all
select 47287,'1978-Y324','1978-08-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 47288,'1978-Y325','1978-08-30 14:00:00','suborbital rocket','unknown',725,'ISRO 8.392','','','','Aeron',1467,'',84 from dual union all
select 47289,'1978-Y326','1978-08-30 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 4729,'1957-S54','1957-07-04 00:00:00','military missile','success',994,'','NIIP-A target','','','Target',634,'',100 from dual union all
select 47290,'1978-Y327','1978-09-01 01:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 47291,'1978-Y328','1978-09-01 14:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 47292,'1978-Y329','1978-09-04 04:29:00','suborbital rocket','success',725,'ISRO 8.393','O2 profile','','','Aeron',1467,'',89 from dual union all
select 47293,'1978-Y33','1978-01-19 21:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 47294,'1978-Y330','1978-09-04 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 47295,'1978-Y331','1978-09-05 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 47296,'1978-Y332','1978-09-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 47297,'1978-Y333','1978-09-06 04:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 47298,'1978-Y334','1978-09-06 14:17:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 47299,'1978-Y335','1978-09-06 15:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 473,'1944-S90','1944-09-12 12:15:00','military missile','success',8,'18769','Ma184','','','Training',661,'',90 from dual union all
select 4730,'1957-S55','1957-07-04 18:15:40','suborbital rocket','success',49,'NN3.08F','NRL NN3.08F','IGY NN3.08F','','Ionos',428,'',237 from dual union all
select 47300,'1978-Y336','1978-09-06 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 47301,'1978-Y337','1978-09-06 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47302,'1978-Y338','1978-09-06 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 47303,'1978-Y339','1978-09-08 15:36:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',82 from dual union all
select 47304,'1978-Y34','1978-01-20 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',85 from dual union all
select 47305,'1978-Y340','1978-09-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 47306,'1978-Y341','1978-09-13 02:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 47307,'1978-Y342','1978-09-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 47308,'1978-Y343','1978-09-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 47309,'1978-Y344','1978-09-13 17:42:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 4731,'1957-S56','1957-07-05 00:00:00','military missile','success',994,'','NIIP-A target','','','Target',634,'',100 from dual union all
select 47310,'1978-Y345','1978-09-13 21:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 47311,'1978-Y346','1978-09-15 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 47312,'1978-Y347','1978-09-15 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 47313,'1978-Y348','1978-09-15 21:22:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 47314,'1978-Y349','1978-09-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 47315,'1978-Y35','1978-01-20 11:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',85 from dual union all
select 47316,'1978-Y350','1978-09-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 47317,'1978-Y351','1978-09-20 15:56:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 47318,'1978-Y352','1978-09-20 16:07:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 47319,'1978-Y353','1978-09-20 18:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 4732,'1957-S57','1957-07-12 06:30:00','military missile','success',635,'CC-35    NS','Jupiter A','','','Test',280,'',90 from dual union all
select 47320,'1978-Y354','1978-09-20 21:32:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 47321,'1978-Y355','1978-09-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 47322,'1978-Y356','1978-09-22 15:53:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 47323,'1978-Y357','1978-09-26 21:32:00','suborbital rocket','unknown',728,'','','','','Bio',636,'',85 from dual union all
select 47324,'1978-Y358','1978-09-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 47325,'1978-Y359','1978-09-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 47326,'1978-Y36','1978-01-23 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',86 from dual union all
select 47327,'1978-Y360','1978-09-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 47328,'1978-Y361','1978-09-27 16:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 47329,'1978-Y362','1978-09-27 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 4733,'1957-S58','1957-07-16 13:30:00','suborbital rocket','success',28,'USAF 80','','','','Meteorites',640,'',122 from dual union all
select 47330,'1978-Y363','1978-09-27 18:56:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',85 from dual union all
select 47331,'1978-Y364','1978-09-27 19:07:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 47332,'1978-Y365','1978-10-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 47333,'1978-Y366','1978-10-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 47334,'1978-Y367','1978-10-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 47335,'1978-Y368','1978-10-04 14:14:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',1821,'',90 from dual union all
select 47336,'1978-Y369','1978-10-04 19:22:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 47337,'1978-Y37','1978-01-25 10:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 47338,'1978-Y370','1978-10-04 21:25:00','suborbital rocket','unknown',728,'','','','','Bio',636,'',85 from dual union all
select 47339,'1978-Y371','1978-10-04 23:17:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 4734,'1957-S59','1957-07-16 00:00:00','military missile','success',936,'4-204-2','','','','Reentry Test',253,'',10 from dual union all
select 47340,'1978-Y372','1978-10-05 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 47341,'1978-Y373','1978-10-11 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 47342,'1978-Y374','1978-10-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 47343,'1978-Y375','1978-10-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 47344,'1978-Y376','1978-10-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 47345,'1978-Y377','1978-10-11 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47346,'1978-Y378','1978-10-11 21:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',86 from dual union all
select 47347,'1978-Y379','1978-10-18 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 47348,'1978-Y38','1978-01-25 14:00:00','suborbital rocket','unknown',725,'ISRO 8.359','','','','Aeron',1467,'',82 from dual;
insert into LAUNCH
select 47349,'1978-Y380','1978-10-18 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 4735,'1957-S60','1957-07-18 14:30:00','suborbital rocket','failure',28,'USAF 81','','','','Meteorites',640,'',10 from dual union all
select 47350,'1978-Y381','1978-10-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 47351,'1978-Y382','1978-10-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 47352,'1978-Y383','1978-10-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 47353,'1978-Y384','1978-10-18 14:37:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',89 from dual union all
select 47354,'1978-Y385','1978-10-18 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47355,'1978-Y386','1978-10-18 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 47356,'1978-Y387','1978-10-19 02:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',82 from dual union all
select 47357,'1978-Y388','1978-10-20 17:32:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 47358,'1978-Y389','1978-10-25 01:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 47359,'1978-Y39','1978-01-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 4736,'1957-S61','1957-07-19 00:00:00','military missile','success',935,'3-204-1','0.4-scale RV','','','Reentry Test',253,'',130 from dual union all
select 47360,'1978-Y390','1978-10-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 47361,'1978-Y391','1978-10-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 47362,'1978-Y392','1978-10-25 14:37:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 47363,'1978-Y393','1978-10-25 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47364,'1978-Y394','1978-10-25 21:27:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 47365,'1978-Y395','1978-10-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 47366,'1978-Y396','1978-11-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 47367,'1978-Y397','1978-11-01 14:27:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 47368,'1978-Y398','1978-11-01 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 47369,'1978-Y399','1978-11-01 15:17:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',85 from dual union all
select 4737,'1957-S62','1957-08-22 15:03:00','suborbital rocket','success',1521,'R-26','','','','Reentry Test',253,'',114 from dual union all
select 47370,'1978-Y40','1978-01-25 16:20:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',86 from dual union all
select 47371,'1978-Y400','1978-11-01 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 47372,'1978-Y401','1978-11-01 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 47373,'1978-Y402','1978-11-01 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47374,'1978-Y403','1978-11-01 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 47375,'1978-Y404','1978-11-07 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 47376,'1978-Y405','1978-11-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 47377,'1978-Y406','1978-11-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 47378,'1978-Y407','1978-11-08 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 47379,'1978-Y408','1978-11-08 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 4738,'1957-S63','1957-07-22 04:16:28','suborbital rocket','success',28,'SM1.02','Grenades','IGY SM1.02','','Aeron',428,'',92 from dual union all
select 47380,'1978-Y409','1978-11-08 21:29:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 47381,'1978-Y41','1978-01-25 18:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47382,'1978-Y410','1978-11-09 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 47383,'1978-Y411','1978-11-15 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 47384,'1978-Y412','1978-11-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 47385,'1978-Y413','1978-11-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 47386,'1978-Y414','1978-11-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 47387,'1978-Y415','1978-11-15 17:01:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 47388,'1978-Y416','1978-11-15 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47389,'1978-Y417','1978-11-15 20:53:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 4739,'1957-S64','1957-07-23 03:02:00','suborbital rocket','success',1193,'SL03','RAE Test 3/QUB AGP1','','','Test/Airglow',1839,'',85 from dual union all
select 47390,'1978-Y418','1978-11-17 18:27:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 47391,'1978-Y419','1978-11-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 47392,'1978-Y42','1978-01-26 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',84 from dual union all
select 47393,'1978-Y420','1978-11-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 47394,'1978-Y421','1978-11-22 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 47395,'1978-Y422','1978-11-22 15:11:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 47396,'1978-Y423','1978-11-22 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47397,'1978-Y424','1978-11-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 47398,'1978-Y425','1978-11-29 14:59:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 47399,'1978-Y426','1978-11-29 15:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 474,'1944-S91','1944-09-12 15:50:00','military missile','success',1470,'','London','','','Weapon',1744,'',95 from dual union all
select 4740,'1957-S65','1957-07-23 23:31:52','suborbital rocket','success',841,'NN7.40F','NRL NN7.40F','IGY NN7.40F','','Solar UV/X',1415,'',129 from dual union all
select 47400,'1978-Y427','1978-11-29 19:06:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47401,'1978-Y428','1978-11-29 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 47402,'1978-Y429','1978-11-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 47403,'1978-Y43','1978-01-26 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1396,'SHOK',83 from dual union all
select 47404,'1978-Y430','1978-12-05 01:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 47405,'1978-Y431','1978-12-06 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 47406,'1978-Y432','1978-12-06 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 47407,'1978-Y433','1978-12-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 47408,'1978-Y434','1978-12-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 47409,'1978-Y435','1978-12-06 15:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 4741,'1957-S66','1957-07-24 05:29:50','suborbital rocket','success',28,'SM1.03','Grenades','IGY SM1.03','','Aeron',428,'',88 from dual union all
select 47410,'1978-Y436','1978-12-06 17:56:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual union all
select 47411,'1978-Y437','1978-12-06 20:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47412,'1978-Y438','1978-12-13 04:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 47413,'1978-Y439','1978-12-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 47414,'1978-Y44','1978-01-27 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',90 from dual union all
select 47415,'1978-Y440','1978-12-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 47416,'1978-Y441','1978-12-13 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 47417,'1978-Y442','1978-12-13 21:33:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 47418,'1978-Y443','1978-12-15 00:00:00','suborbital rocket','success',1281,'NASA 15.197GM','','','','Aeronomy',1821,'',80 from dual union all
select 47419,'1978-Y444','1978-12-20 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 4742,'1957-S67','1957-07-26 04:17:00','military missile','success',635,'CC-37    NI','Jupiter A','','','Test',280,'',90 from dual union all
select 47420,'1978-Y445','1978-12-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',80 from dual union all
select 47421,'1978-Y446','1978-12-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 47422,'1978-Y447','1978-12-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 47423,'1978-Y448','1978-12-20 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 47424,'1978-Y449','1978-12-20 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47425,'1978-Y45','1978-01-27 21:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 47426,'1978-Y450','1978-12-22 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',82 from dual union all
select 47427,'1978-Y451','1978-12-27 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 47428,'1978-Y452','1978-12-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 47429,'1978-Y453','1978-12-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 4743,'1957-S68','1957-07-29 21:59:00','suborbital rocket','success',49,'NN3.13F','NRL NN3.13F','IGY NN3.13F','','Aeron La',428,'',210 from dual union all
select 47430,'1978-Y454','1978-12-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 47431,'1978-Y46','1978-01-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 47432,'1978-Y47','1978-01-31 01:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',82 from dual union all
select 47433,'1978-Y48','1978-02-01 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',81 from dual union all
select 47434,'1978-Y49','1978-02-01 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 47435,'1978-Y50','1978-02-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 47436,'1978-Y51','1978-02-01 14:00:00','suborbital rocket','unknown',725,'ISRO 8.360','','','','Aeron',1467,'',82 from dual union all
select 47437,'1978-Y52','1978-02-01 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 47438,'1978-Y53','1978-02-01 20:07:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual;
insert into LAUNCH
select 47439,'1978-Y54','1978-02-03 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',82 from dual union all
select 4744,'1957-S69','1957-08-23 00:00:00','military missile','success',975,'M1-26','','','','Test',636,'',86 from dual union all
select 47440,'1978-Y55','1978-02-03 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 47441,'1978-Y56','1978-02-03 20:24:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual union all
select 47442,'1978-Y57','1978-02-07 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',82 from dual union all
select 47443,'1978-Y58','1978-02-08 04:23:00','suborbital rocket','success',229,'AQF-VI-06','Coherent pulse radar','','','Ionos',428,'',95 from dual union all
select 47444,'1978-Y59','1978-02-08 10:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 47445,'1978-Y60','1978-02-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 47446,'1978-Y61','1978-02-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 47447,'1978-Y62','1978-02-08 14:00:00','suborbital rocket','unknown',725,'ISRO 8.361','','','','Aeron',1467,'',83 from dual union all
select 47448,'1978-Y63','1978-02-08 16:34:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47449,'1978-Y64','1978-02-08 21:02:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 4745,'1957-S70','1957-08-02 00:00:00','military missile','failure',976,'M4-8','63M No. 5 LKI1-5','','','Test',636,'',300 from dual union all
select 47450,'1978-Y65','1978-02-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 47451,'1978-Y66','1978-02-15 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 47452,'1978-Y67','1978-02-15 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',84 from dual union all
select 47453,'1978-Y68','1978-02-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 47454,'1978-Y69','1978-02-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 47455,'1978-Y70','1978-02-15 14:00:00','suborbital rocket','unknown',725,'ISRO 8.362','','','','Aeron',1467,'',81 from dual union all
select 47456,'1978-Y71','1978-02-15 16:55:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual union all
select 47457,'1978-Y72','1978-02-15 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47458,'1978-Y73','1978-02-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',84 from dual union all
select 47459,'1978-Y74','1978-02-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 4746,'1957-S71','1957-08-15 00:00:00','military missile','success',976,'M5-9','63M No. 6 LKI1-6','','','Test',636,'',402 from dual union all
select 47460,'1978-Y75','1978-02-19 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',81 from dual union all
select 47461,'1978-Y76','1978-02-21 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 47462,'1978-Y77','1978-02-21 11:20:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',87 from dual union all
select 47463,'1978-Y78','1978-02-22 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 47464,'1978-Y79','1978-02-22 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 47465,'1978-Y80','1978-02-22 12:10:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',91 from dual union all
select 47466,'1978-Y81','1978-02-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 47467,'1978-Y82','1978-02-22 14:00:00','suborbital rocket','unknown',725,'ISRO 8.363','','','','Aeron',1467,'',85 from dual union all
select 47468,'1978-Y83','1978-02-22 16:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47469,'1978-Y84','1978-02-23 08:30:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',88 from dual union all
select 4747,'1957-S72','1957-08-05 13:22:00','suborbital rocket','success',718,'II5.01F','SUI 56','IGY II5.01F','','Aeron/CR',679,'LSD-29',100 from dual union all
select 47470,'1978-Y85','1978-02-24 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',85 from dual union all
select 47471,'1978-Y86','1978-02-24 10:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',88 from dual union all
select 47472,'1978-Y87','1978-02-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 47473,'1978-Y88','1978-02-25 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 47474,'1978-Y89','1978-02-27 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 47475,'1978-Y90','1978-02-27 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',81 from dual union all
select 47476,'1978-Y91','1978-02-28 01:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 47477,'1978-Y92','1978-03-01 01:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 47478,'1978-Y93','1978-03-01 07:30:00','suborbital rocket','unknown',725,'','','','','Aeron',885,'KORO',87 from dual union all
select 47479,'1978-Y94','1978-03-01 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 4748,'1957-S73','1957-08-05 16:59:00','suborbital rocket','success',718,'II5.02F','SUI 57','IGY II5.02F','','Fields',689,'LSD-29',100 from dual union all
select 47480,'1978-Y95','1978-03-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 47481,'1978-Y96','1978-03-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 47482,'1978-Y97','1978-03-01 14:00:00','suborbital rocket','unknown',725,'ISRO 8.364','','','','Aeron',1467,'',83 from dual union all
select 47483,'1978-Y98','1978-03-01 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 47484,'1978-Y99','1978-03-03 11:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 47485,'1979-001','1979-01-11 15:00:00','orbital','success',1238,'78055  -130','Kosmos-1070','Zenit-2M','','Sat',474,'',null from dual union all
select 47486,'1979-002','1979-01-13 15:30:00','orbital','success',1238,'77025  -730','Kosmos-1071','Zenit-4MKM','','Sat',475,'',null from dual union all
select 47487,'1979-003','1979-01-16 17:37:00','orbital','success',680,'47182-437','Kosmos-1072','Parus','','Sat',456,'',null from dual union all
select 47488,'1979-004','1979-01-18 15:42:00','orbital','success',809,'78018  -504','Molniya-3','Molniya-3 No. 23','','Sat',475,'',null from dual union all
select 47489,'1979-005','1979-01-25 05:43:00','orbital','success',1510,'78026  -386','Meteor-Priroda','Meteor-Priroda No. 2-3','','Sat',562,'',null from dual union all
select 4749,'1957-S74','1957-08-06 15:30:00','suborbital rocket','success',48,'USAF 82','Solar UV','IGY AC2.24','','Solar UV',640,'',145 from dual union all
select 47490,'1979-006','1979-01-30 15:15:00','orbital','success',1238,'77025  -731','Kosmos-1073','Zenit-4MKM','','Sat',475,'',null from dual union all
select 47491,'1979-007','1979-01-30 21:42:00','orbital','success',386,'629/D148','Scatha','P78-2','','Sat',234,'',null from dual union all
select 47492,'1979-008','1979-01-31 09:00:00','orbital','success',1238,'Ye15000-142','Kosmos-1074','Soyuz 7K-ST No. 5L','','Sat',562,'',null from dual union all
select 47493,'1979-009','1979-02-06 08:46:00','orbital','failure',822,'N-5(F)','Ayame','ECS','','Sat',1474,'',null from dual union all
select 47494,'1979-010','1979-02-08 10:00:00','orbital','success',680,'53782-417','Kosmos-1075','Vektor','','Sat',457,'',null from dual union all
select 47495,'1979-011','1979-02-12 13:00:00','orbital','success',1448,'45055-503','Kosmos-1076','Okean-E NKh No. 1','','Sat',470,'',null from dual union all
select 47496,'1979-012','1979-02-13 21:41:00','orbital','success',1510,'76038  -312','Kosmos-1077','Tselina-D','','Sat',475,'',null from dual union all
select 47497,'1979-013','1979-02-18 16:18:00','orbital','success',1132,'S202C','SAGE','AEM-B','','Sat',1814,'',null from dual union all
select 47498,'1979-014','1979-02-21 05:00:00','orbital','success',816,'M-3C-4','Hakucho','CORSA-b','','Sat',835,'',null from dual union all
select 47499,'1979-015','1979-02-21 07:49:00','orbital','success',956,'294-01','Ekran','Ekran No. 16L','','Sat',554,'',null from dual union all
select 475,'1944-S92','1944-09-13 03:03:00','military missile','success',8,'18767','Ma173','','','Training',661,'',90 from dual union all
select 4750,'1957-S75','1957-08-06 13:13:00','suborbital rocket','success',718,'II5.03F','SUI 58','IGY II5.03F','','Aeron/Auroral/CR',690,'LSD-29',116 from dual union all
select 47500,'1979-016','1979-02-22 12:10:00','orbital','success',1238,'77025  -732','Kosmos-1078','Zenit-4MKM','','Sat',473,'',null from dual union all
select 47501,'1979-017','1979-02-24 08:20:00','orbital','success',165,'27F','P78-1','P78-1','','Sat',1776,'',null from dual union all
select 47502,'1979-018','1979-02-25 11:53:49','orbital','success',1238,'Ye15000-155','Soyuz-32','Soyuz 7K-T No. 48','','Sat',562,'',null from dual union all
select 47503,'1979-019','1979-02-27 15:00:00','orbital','success',1238,'77025  -926','Kosmos-1079','Feniks No. 15','','Sat',474,'',null from dual union all
select 47504,'1979-020','1979-02-27 17:00:00','orbital','success',680,'47155-107','Interkosmos-19','AUOS-Z Ionozond-IK','','Sat',457,'',null from dual union all
select 47505,'1979-021','1979-03-01 18:45:00','orbital','success',1510,'78018  -307','Meteor-2','Meteor-2 No. 4','','Sat',475,'',null from dual union all
select 47506,'1979-022','1979-03-12 05:47:28','orbital','success',1238,'Ye15000-162','Progress-5','Progress 7K-TG No. 104','','Sat',562,'',null from dual union all
select 47507,'1979-023','1979-03-14 10:50:00','orbital','success',1238,'78055  -737','Kosmos-1080','Zenit-4MKM','','Sat',473,'',null from dual union all
select 47508,'1979-024','1979-03-15 02:58:00','orbital','success',680,'47168-312','Kosmos-1081/1088','Strela-1M','','Sat',457,'',null from dual union all
select 47509,'1979-025','1979-03-16 18:30:00','orbital','success',1426,'23D-21','HEXAGON SV15','HEXAGON SV-15','','Sat',1777,'',null from dual union all
select 4751,'1957-S76','1957-08-06 17:23:00','suborbital rocket','success',718,'II5.04F','SUI 59','IGY II5.04F','','Ionos',691,'LSD-29',117 from dual union all
select 47510,'1979-026','1979-03-21 04:13:00','orbital','success',680,'47172-413','Kosmos-1089','Parus','','Sat',457,'',null from dual union all
select 47511,'1979-027','1979-03-31 10:45:00','orbital','success',1238,'77025  -127','Kosmos-1090','Zenit-2M','','Sat',473,'',null from dual union all
select 47512,'1979-028','1979-04-07 06:20:00','orbital','success',680,'65082-424','Kosmos-1091','Parus','','Sat',456,'',null from dual union all
select 47513,'1979-029','1979-04-10 17:34:34','orbital','success',1238,'Zh15000-182','Soyuz-33','Soyuz 7K-T No. 49','','Sat',562,'',null from dual union all
select 47514,'1979-030','1979-04-11 21:51:00','orbital','success',680,'53782-423','Kosmos-1092','Tsikada','','Sat',456,'',null from dual union all
select 47515,'1979-031','1979-04-12 00:28:00','orbital','success',809,'76042  -605','Molniya-1','Molniya-1','','Sat',473,'',null from dual union all
select 47516,'1979-032','1979-04-14 05:27:00','orbital','success',1510,'78055  -326','Kosmos-1093','Tselina-D','','Sat',474,'',null from dual union all
select 47517,'1979-033','1979-04-18 12:00:00','orbital','success',1446,'45026-504','Kosmos-1094','US-P','','Sat',589,'',null from dual union all
select 47518,'1979-034','1979-04-20 11:30:00','orbital','success',1238,'76038  -407','Kosmos-1095','Zenit-6','','Sat',474,'',null from dual union all
select 47519,'1979-035','1979-04-25 03:44:00','orbital','success',956,'298-02','Raduga','Gran'' No. 15L','','Sat',554,'',null from dual union all
select 4752,'1957-S77','1957-08-21 00:00:00','military missile','success',971,'M684','ZI','','','Test',636,'',150 from dual union all
select 47520,'1979-036','1979-04-25 10:00:00','orbital','success',1446,'45052-105','Kosmos-1096','US-P','','Sat',589,'',null from dual union all
select 47521,'1979-037','1979-04-27 17:15:00','orbital','success',1238,'E 15000-048','Kosmos-1097','Yantar''-4K1 No. 1','','Sat',474,'',null from dual union all
select 47522,'1979-038','1979-05-04 18:57:00','orbital','success',180,'AC-47','FLTSATCOM 2','FLTSATCOM F2','','Sat',268,'',null from dual union all
select 47523,'1979-039','1979-05-13 04:17:10','orbital','success',1238,'Zh15000-175','Progress-6','Progress 7K-TG No. 106','','Sat',562,'',null from dual union all
select 47524,'1979-040','1979-05-15 11:40:00','orbital','success',1238,'78055  -736','Kosmos-1098','Zenit-4MKM','','Sat',473,'',null from dual union all
select 47525,'1979-041','1979-05-17 07:10:00','orbital','success',1238,'78055  -048','Kosmos-1099','Fram','','Sat',475,'',null from dual union all
select 47526,'1979-042','1979-05-22 23:00:00','orbital','success',949,'300-02','Kosmos-1100/1101','TKS VA No. 0102A','','Sat',588,'',null from dual union all
select 47527,'1979-043','1979-05-25 07:00:00','orbital','success',1238,'76047  -152','Kosmos-1102','Gektor-Priroda','','Sat',473,'',null from dual union all
select 47528,'1979-044','1979-05-28 18:30:00','orbital','success',1403,'24B-25 (3B-61)','KH-8A','GAMBIT-3 Mission 4350','','Sat',1778,'',null from dual;
insert into LAUNCH
select 47529,'1979-045','1979-05-31 16:30:00','orbital','success',1238,'78055  -420','Kosmos-1103','Zenit-6','','Sat',474,'',null from dual union all
select 4753,'1957-S78','1957-08-08 06:59:00','test rocket','success',637,'RS/CC-40 TX','Jupiter RV','','','Reentry Test',280,'',460 from dual union all
select 47530,'1979-046','1979-05-31 17:58:00','orbital','success',680,'65082-418','Kosmos-1104','Parus','','Sat',456,'',null from dual union all
select 47531,'1979-047','1979-06-02 23:26:00','orbital','success',1132,'S198C','Ariel 6','Ariel 6','','Sat',1814,'',null from dual union all
select 47532,'1979-048','1979-06-05 23:28:00','orbital','success',809,'76038  -509','Molniya-3','Molniya-3 No. 21','','Sat',475,'',null from dual union all
select 47533,'1979-049','1979-06-06 18:12:41','orbital','success',1238,'Zh15000-178','Soyuz-34','Soyuz 7K-T No. 50','','Sat',562,'',null from dual union all
select 47534,'1979-050','1979-06-06 18:22:12','orbital','success',1374,'264','DMSP 5D F-4','DMSP Block 5D S-5','','Sat',1720,'',null from dual union all
select 47535,'1979-051','1979-06-07 10:30:00','orbital','success',680,'65055-109','Bhaskara','SEO','','Sat',622,'',null from dual union all
select 47536,'1979-052','1979-06-08 07:10:00','orbital','success',1238,'76047  -060','Kosmos-1105','Fram','','Sat',473,'',null from dual union all
select 47537,'1979-053','1979-06-10 13:30:00','orbital','success',1425,'23C-13 (3C-31)','DSP 11','DSP 11','','Sat',274,'',null from dual union all
select 47538,'1979-054','1979-06-12 07:00:00','orbital','success',1238,'77025  -128','Kosmos-1106','Gektor-Priroda','','Sat',475,'',null from dual union all
select 47539,'1979-055','1979-06-15 10:50:00','orbital','success',1238,'D 15000-083','Kosmos-1107','Zenit-6','','Sat',474,'',null from dual union all
select 4754,'1957-S79','1957-08-08 07:17:00','suborbital rocket','success',718,'II5.07F','SUI 62','IGY II5.07F','','Aeron/CR',694,'LSD-29',132 from dual union all
select 47540,'1979-056','1979-06-22 07:00:00','orbital','success',1238,'78055  -049','Kosmos-1108','Fram','','Sat',475,'',null from dual union all
select 47541,'1979-057','1979-06-27 15:51:59','orbital','success',162,'25F','NOAA 6','NOAA A','','Sat',1776,'',null from dual union all
select 47542,'1979-058','1979-06-27 18:11:23','orbital','success',809,'76038  -512','Kosmos-1109','Oko','','Sat',473,'',null from dual union all
select 47543,'1979-059','1979-06-28 09:25:11','orbital','success',1238,'Zh15000-192','Progress-7','Progress 7K-TG No. 107','','Sat',562,'',null from dual union all
select 47544,'1979-060','1979-06-28 20:09:00','orbital','success',680,'53768-304','Kosmos-1110','Strela-2M','','Sat',456,'',null from dual union all
select 47545,'1979-061','1979-06-29 16:00:00','orbital','success',1238,'78055  -421','Kosmos-1111','Zenit-6','','Sat',474,'',null from dual union all
select 47546,'1979-062','1979-07-05 23:19:00','orbital','success',956,'299-01','Gorizont','Gorizont No. 12L','','Sat',554,'',null from dual union all
select 47547,'1979-063','1979-07-06 08:20:00','orbital','success',680,'65075-124','Kosmos-1112','Romb','','Sat',622,'',null from dual union all
select 47548,'1979-064','1979-07-10 09:00:00','orbital','success',1238,'76038-  712','Kosmos-1113','Zenit-4MKM','','Sat',562,'',null from dual union all
select 47549,'1979-065','1979-07-11 15:41:00','orbital','success',680,'47155-104','Kosmos-1114','Tselina-OM','','Sat',456,'',null from dual union all
select 4755,'1957-S80','1957-08-09 00:00:00','military missile','success',935,'3-204-2','0.4-scale RV','','','Reentry Test',253,'',116 from dual union all
select 47550,'1979-066','1979-07-13 08:25:00','orbital','success',1238,'76047  -062','Kosmos-1115','Fram','','Sat',475,'',null from dual union all
select 47551,'1979-067','1979-07-20 11:58:00','orbital','success',1510,'77025  -324','Kosmos-1116','Tselina-D','','Sat',475,'',null from dual union all
select 47552,'1979-068','1979-07-25 15:20:00','orbital','success',1238,'77025  -733','Kosmos-1117','Zenit-4MKM','','Sat',474,'',null from dual union all
select 47553,'1979-069','1979-07-27 07:30:00','orbital','success',1238,'76047  -153','Kosmos-1118','Gektor-Priroda','','Sat',475,'',null from dual union all
select 47554,'1979-070','1979-07-31 03:56:00','orbital','success',809,'78055  -534','Molniya-1','Molniya-1','','Sat',474,'',null from dual union all
select 47555,'1979-071','1979-08-03 10:45:00','orbital','success',1238,'76038  -818','Kosmos-1119','Orion','','Sat',474,'',null from dual union all
select 47556,'1979-072','1979-08-10 00:20:00','orbital','success',386,'638/D149','Westar 3','Westar 3','','Sat',233,'',null from dual union all
select 47557,'1979-073','1979-08-11 09:15:00','orbital','success',1238,'76047  -759','Kosmos-1120','Zenit-4MKM','','Sat',562,'',null from dual union all
select 47558,'1979-074','1979-08-14 15:30:00','orbital','success',1238,'78055  -929','Kosmos-1121','Feniks No. 929','','Sat',474,'',null from dual union all
select 47559,'1979-075','1979-08-17 07:45:00','orbital','success',1238,'77025  -126','Kosmos-1122','Gektor-Priroda','','Sat',475,'',null from dual union all
select 4756,'1957-S81','1957-08-10 06:29:00','suborbital rocket','success',718,'II5.08F','SUI 63','IGY II5.08F','','Aeron/Auroral/CR',695,'LSD-29',117 from dual union all
select 47560,'1979-076','1979-08-21 11:10:00','orbital','success',1238,'76047  -061','Kosmos-1123','Fram','','Sat',473,'',null from dual union all
select 47561,'1979-077','1979-08-28 00:17:04','orbital','success',809,'78055  -531','Kosmos-1124','Oko','','Sat',475,'',null from dual union all
select 47562,'1979-078','1979-08-28 00:55:00','orbital','success',680,'47182-419','Kosmos-1125','Strela-2M','','Sat',456,'',null from dual union all
select 47563,'1979-079','1979-08-31 11:30:00','orbital','success',1238,'76047  -444','Kosmos-1126','Zenit-6','','Sat',475,'',null from dual union all
select 47564,'1979-080','1979-09-05 10:20:00','orbital','success',1238,'Zh15000-201','Kosmos-1127','Resurs-F1 17F41 No. 11','','Sat',473,'',null from dual union all
select 47565,'1979-081','1979-09-14 15:30:00','orbital','success',1238,'78055  -738','Kosmos-1128','Zenit-4MKM','','Sat',475,'',null from dual union all
select 47566,'1979-082','1979-09-20 05:28:00','orbital','success',180,'AC-53','HEAO 3','HEAO C','','Sat',269,'',null from dual union all
select 47567,'1979-083','1979-09-25 15:30:00','orbital','success',1238,'Zh15000-193','Kosmos-1129','Bion No. 5','','Sat',473,'',null from dual union all
select 47568,'1979-084','1979-09-25 21:00:00','orbital','success',680,'47147-243','Kosmos-1130/1137','Strela-1M','','Sat',457,'',null from dual union all
select 47569,'1979-085','1979-09-28 12:20:00','orbital','success',1238,'77025  -418','Kosmos-1138','Zenit-6','','Sat',474,'',null from dual union all
select 4757,'1957-S82','1957-08-11 20:30:00','suborbital rocket','success',718,'II5.11F','SUI 66','IGY II5.11F','','Aeron/CR',681,'LSD-29',100 from dual union all
select 47570,'1979-086','1979-10-01 11:22:00','orbital','success',1425,'23C-16 (3C-34)','OPS 1948','VORTEX 2','','Sat',274,'',null from dual union all
select 47571,'1979-087','1979-10-03 17:12:00','orbital','success',956,'302-02','Ekran','Ekran No. 17L','','Sat',554,'',null from dual union all
select 47572,'1979-088','1979-10-05 11:30:00','orbital','success',1238,'78055  -817','Kosmos-1139','Orion','','Sat',473,'',null from dual union all
select 47573,'1979-089','1979-10-11 16:36:00','orbital','success',680,'47172-407','Kosmos-1140','Strela-2M','','Sat',457,'',null from dual union all
select 47574,'1979-090','1979-10-16 12:17:00','orbital','success',680,'47172-410','Kosmos-1141','Parus','','Sat',457,'',null from dual union all
select 47575,'1979-091','1979-10-20 07:03:00','orbital','success',809,'78055  -528','Molniya-1','Molniya-1','','Sat',473,'',null from dual union all
select 47576,'1979-092','1979-10-22 12:30:00','orbital','success',1238,'76047  -445','Kosmos-1142','Zenit-6','','Sat',474,'',null from dual union all
select 47577,'1979-093','1979-10-26 18:12:00','orbital','success',1510,'78018  -306 ','Kosmos-1143','Tselina-D','','Sat',475,'',null from dual union all
select 47578,'1979-094','1979-10-30 14:16:00','orbital','success',1135,'S203C','Magsat','AEM-C','','Sat',1779,'',null from dual union all
select 47579,'1979-095','1979-10-31 09:25:00','orbital','success',1510,'78055  -328','Meteor-2','Meteor-2 No. 7','','Sat',475,'',null from dual union all
select 4758,'1957-S83','1957-08-12 15:15:00','suborbital rocket','success',718,'II5.13F','SUI 68','IGY II5.13F','','Auroral/Fields',683,'LSD-29',100 from dual union all
select 47580,'1979-096','1979-11-01 08:05:00','orbital','success',680,'53755-105','Interkosmos-20','AUOS-Z ?','','Sat',457,'',null from dual union all
select 47581,'1979-097','1979-11-02 16:00:00','orbital','success',1238,'76047  -939','Kosmos-1144','Feniks No. 939','','Sat',474,'',null from dual union all
select 47582,'1979-098','1979-11-21 02:09:36','orbital','success',1425,'23C-19 (3C-37)','DSCS II 13/14','DSCS II D-13','','Sat',274,'',null from dual union all
select 47583,'1979-099','1979-11-27 09:55:00','orbital','success',1510,'78055  -327','Kosmos-1145','Tselina-D','','Sat',475,'',null from dual union all
select 47584,'1979-100','1979-12-05 09:00:00','orbital','success',680,'47168-309','Kosmos-1146','Taifun-1B','','Sat',457,'',null from dual union all
select 47585,'1979-101','1979-12-07 01:35:00','orbital','success',390,'622/D150','Satcom 3','RCA-C','','Sat',233,'',null from dual union all
select 47586,'1979-102','1979-12-12 12:30:00','orbital','success',1238,'76047  -442','Kosmos-1147','Zenit-6','','Sat',474,'',null from dual union all
select 47587,'1979-103','1979-12-16 12:29:50','orbital','success',1238,'78018  -912','Soyuz T','Soyuz 7K-ST No. 6L','','Sat',562,'',null from dual union all
select 47588,'1979-104','1979-12-24 17:14:38','orbital','success',91,'L01','','CAT','','Sat',344,'',null from dual union all
select 47589,'1979-105','1979-12-28 11:51:00','orbital','success',956,'303-01','Gorizont','Gorizont No. 13L','','Sat',554,'',null from dual union all
select 4759,'1957-S84','1957-08-13 00:00:00','suborbital rocket','success',994,'','SOI','','','Solar UV',636,'',100 from dual union all
select 47590,'1979-106','1979-12-28 13:00:00','orbital','success',1238,'76047  -760','Kosmos-1148','Zenit-4MKM','','Sat',474,'',null from dual union all
select 47591,'1979-A01','1979-01-05 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',49 from dual union all
select 47592,'1979-A02','1979-01-17 12:52:41','military missile','unknown',1099,'','','','','Test',293,'FB-111',10 from dual union all
select 47593,'1979-A03','1979-01-19 12:29:38','military missile','unknown',1099,'','','','','Test',293,'FB-111',10 from dual union all
select 47594,'1979-A04','1979-01-19 11:59:20','military missile','unknown',1099,'','','','','Test',293,'FB-111',10 from dual union all
select 47595,'1979-A05','1979-01-30 19:51:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',48 from dual union all
select 47596,'1979-A06','1979-02-05 20:11:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',49 from dual union all
select 47597,'1979-A07','1979-02-05 23:26:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',48 from dual union all
select 47598,'1979-A08','1979-02-07 11:57:09','military missile','unknown',1099,'','','','','Test',293,'B-52',10 from dual union all
select 47599,'1979-A09','1979-02-09 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',41 from dual union all
select 476,'1944-S93','1944-09-13 21:00:00','military missile','success',1470,'','London','','','Weapon',1744,'',95 from dual union all
select 4760,'1957-S85','1957-08-13 01:58:00','suborbital rocket','success',718,'II5.14F','SUI 69','IGY II5.14F','','Aeron/Auroral/CR',684,'LSD-29',100 from dual union all
select 47600,'1979-A10','1979-02-12 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 47601,'1979-A11','1979-02-19 14:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',45 from dual union all
select 47602,'1979-A12','1979-02-27 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 47603,'1979-A13','1979-03-01 00:00:00','military missile','success',10,'','Test','','','ABM Test',183,'',10 from dual union all
select 47604,'1979-A14','1979-03-19 00:34:48','suborbital rocket','failure',495,'F6','','','','Aeron',29,'',15 from dual union all
select 47605,'1979-A15','1979-03-19 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',22 from dual union all
select 47606,'1979-A17','1979-03-22 06:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',48 from dual union all
select 47607,'1979-A18','1979-03-27 21:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 47608,'1979-A19','1979-04-02 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 47609,'1979-A20','1979-04-09 15:59:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',46 from dual union all
select 4761,'1957-S86','1957-08-14 09:24:00','suborbital rocket','success',718,'II5.15F','SUI 70','IGY II5.15F','','Aeron/Auroral/CR',685,'LSD-29',100 from dual union all
select 47610,'1979-A21','1979-04-10 10:00:00','military missile','failure',1442,'C4T-1','PEM-1','','','OT',421,'SSBN 657',10 from dual union all
select 47611,'1979-A22','1979-05-09 17:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',48 from dual union all
select 47612,'1979-A23','1979-05-18 15:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',44 from dual union all
select 47613,'1979-A24','1979-05-21 20:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',47 from dual union all
select 47614,'1979-A25','1979-05-30 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',43 from dual union all
select 47615,'1979-A26','1979-06-04 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',37 from dual union all
select 47616,'1979-A27','1979-06-08 21:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',48 from dual union all
select 47617,'1979-A28','1979-06-13 13:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',49 from dual union all
select 47618,'1979-A29','1979-06-25 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',49 from dual;
insert into LAUNCH
select 47619,'1979-A30','1979-06-25 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 4762,'1957-S87','1957-08-14 15:07:00','suborbital rocket','success',718,'II5.16F','SUI 71','IGY II5.16F','','Auroral/Fields',686,'LSD-29',97 from dual union all
select 47620,'1979-A31','1979-06-29 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 47621,'1979-A32','1979-07-03 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 47622,'1979-A33','1979-07-06 18:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',46 from dual union all
select 47623,'1979-A34','1979-07-12 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 47624,'1979-A35','1979-08-14 19:40:00','suborbital rocket','failure',867,'GL A08.706-2','DMSP DENS','','','Aeron',1895,'',18 from dual union all
select 47625,'1979-A36','1979-08-15 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 47626,'1979-A37','1979-08-17 00:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 47627,'1979-A38','1979-08-22 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',49 from dual union all
select 47628,'1979-A39','1979-08-27 15:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',49 from dual union all
select 47629,'1979-A40','1979-09-17 20:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 4763,'1957-S88','1957-08-14 21:19:00','suborbital rocket','success',718,'II5.17F','SUI 72','IGY II5.17F','','Aeron/Auroral/CR',687,'LSD-29',130 from dual union all
select 47630,'1979-A41','1979-09-24 05:32:00','suborbital rocket','failure',921,'P217E','INTA PSU/P.217E','Ba release','','Ionospheric',396,'',30 from dual union all
select 47631,'1979-A42','1979-09-24 21:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',44 from dual union all
select 47632,'1979-A43','1979-09-25 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 47633,'1979-A44','1979-10-03 15:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',49 from dual union all
select 47634,'1979-A45','1979-11-28 12:01:28','military missile','unknown',1099,'','','','','Test',293,'B-52',10 from dual union all
select 47635,'1979-A46','1979-12-12 12:25:12','military missile','unknown',1099,'','','','','Test',293,'FB-111',10 from dual union all
select 47636,'1979-A47','1979-12-12 11:59:36','military missile','unknown',1099,'','','','','Test',293,'FB-111',10 from dual union all
select 47637,'1979-A48','1979-12-12 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',47 from dual union all
select 47638,'1979-A49','1979-08-01 00:00:00','military missile','success',3,'','Ejection Test','','','Test',183,'',0 from dual union all
select 47639,'1979-F01','1979-02-16 15:00:00','orbital','failure',1238,'77025  -131','','Zenit-2M No. 131','','Sat',473,'',200 from dual union all
select 4764,'1957-S89','1957-08-15 00:11:00','suborbital rocket','success',718,'II5.18F','SUI 73','IGY II5.18F','','Aeron/Auroral/CR',688,'LSD-29',97 from dual union all
select 47640,'1979-F02','1979-07-27 21:28:00','orbital','failure',492,'XCZ-1-02','SJ-2','SKW','','Sat',810,'',240 from dual union all
select 47641,'1979-F03','1979-08-10 02:28:00','orbital','failure',1084,'SLV-3-E1','RS-1','Rohini RS-1','','Sat',1360,'',300 from dual union all
select 47642,'1979-M01','1979-10-23 23:04:00','suborbital rocket','success',852,'GL A45.803-1','SMOKE','','','Aeron release',323,'',50 from dual union all
select 47643,'1979-M02','1979-10-24 23:24:00','suborbital rocket','success',852,'GL A45.803-2','SMOKE','','','Aeron release',323,'',50 from dual union all
select 47644,'1979-M03','1979-10-29 23:04:00','suborbital rocket','success',852,'GL A45.803-3','SMOKE','','','Aeron release',323,'',50 from dual union all
select 47645,'1979-M04','1979-07-11 13:18:00','suborbital rocket','success',1023,'MONEX-79','Meteo Chaff','','','Meteo',1361,'',60 from dual union all
select 47646,'1979-M05','1979-01-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47647,'1979-M06','1979-01-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47648,'1979-M07','1979-01-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47649,'1979-M08','1979-01-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 4765,'1957-S90','1957-08-16 00:00:00','military missile','success',937,'5-204-1','','','','Reentry Test',253,'',169 from dual union all
select 47650,'1979-M09','1979-01-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47651,'1979-M10','1979-01-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47652,'1979-M100','1979-08-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47653,'1979-M101','1979-08-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47654,'1979-M102','1979-08-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47655,'1979-M103','1979-08-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47656,'1979-M104','1979-08-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47657,'1979-M105','1979-08-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47658,'1979-M106','1979-08-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47659,'1979-M107','1979-08-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 4766,'1957-S91','1957-08-20 16:50:04','suborbital rocket','success',841,'NN7.42F/DAN-20','NRL NN7.42F','IGY NN7.42F','','Solar UV/X',1415,'',96 from dual union all
select 47660,'1979-M108','1979-08-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47661,'1979-M109','1979-08-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47662,'1979-M11','1979-01-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47663,'1979-M110','1979-08-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47664,'1979-M111','1979-08-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47665,'1979-M112','1979-09-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47666,'1979-M113','1979-09-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47667,'1979-M114','1979-09-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47668,'1979-M115','1979-09-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47669,'1979-M116','1979-09-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 4767,'1957-S92','1957-08-21 12:25:00','military missile','success',1016,'M1-8','8K71 LKI I-3','','','Test',480,'',1350 from dual union all
select 47670,'1979-M117','1979-09-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47671,'1979-M118','1979-09-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47672,'1979-M119','1979-10-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47673,'1979-M12','1979-01-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47674,'1979-M120','1979-10-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47675,'1979-M121','1979-10-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47676,'1979-M122','1979-10-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47677,'1979-M123','1979-10-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47678,'1979-M124','1979-10-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47679,'1979-M125','1979-10-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 4768,'1957-S93','1957-08-23 21:54:05','suborbital rocket','success',839,'RP6.X1 igniter test','RP6.1','IGY RP6.X1','','Test',428,'',114 from dual union all
select 47680,'1979-M126','1979-10-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47681,'1979-M127','1979-10-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47682,'1979-M128','1979-11-07 11:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47683,'1979-M129','1979-11-07 12:11:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47684,'1979-M13','1979-01-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47685,'1979-M130','1979-11-07 13:18:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47686,'1979-M131','1979-11-07 13:57:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47687,'1979-M132','1979-11-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47688,'1979-M133','1979-11-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47689,'1979-M134','1979-11-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 4769,'1957-S94','1957-08-25 02:23:00','suborbital rocket','success',994,'No. 3','','','','Ionos/Bio',636,'',206 from dual union all
select 47690,'1979-M135','1979-11-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47691,'1979-M136','1979-11-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47692,'1979-M137','1979-11-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47693,'1979-M138','1979-11-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47694,'1979-M139','1979-12-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47695,'1979-M14','1979-01-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47696,'1979-M140','1979-12-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47697,'1979-M141','1979-12-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47698,'1979-M142','1979-12-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47699,'1979-M143','1979-12-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 477,'1944-S94','1944-09-13 15:08:00','military missile','success',8,'18765','Ma178','','','Training',661,'',90 from dual union all
select 4770,'1957-S95','1957-08-25 03:27:00','suborbital rocket','success',994,'[Same as above]','','','','Test/',636,'',208 from dual union all
select 47700,'1979-M144','1979-12-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47701,'1979-M145','1979-12-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47702,'1979-M146','1979-12-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47703,'1979-M147','1979-02-21 15:05:00','suborbital rocket','success',290,'ISRO','','','','Ionos',1467,'',72 from dual union all
select 47704,'1979-M148','1979-01-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 47705,'1979-M149','1979-02-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 47706,'1979-M15','1979-01-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47707,'1979-M150','1979-05-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 47708,'1979-M151','1979-06-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual;
insert into LAUNCH
select 47709,'1979-M152','1979-06-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 4771,'1957-S96','1957-08-25 14:08:05','suborbital rocket','success',28,'SM2.06','Grenades,Temp, wind','IGY SM2.06','','Aeron',428,'',130 from dual union all
select 47710,'1979-M153','1979-06-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 47711,'1979-M154','1979-07-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 47712,'1979-M155','1979-07-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 47713,'1979-M156','1979-09-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 47714,'1979-M157','1979-10-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 47715,'1979-M158','1979-10-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 47716,'1979-M159','1979-10-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 47717,'1979-M16','1979-01-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47718,'1979-M160','1979-10-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 47719,'1979-M161','1979-11-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 4772,'1957-S97','1957-08-28 04:15:03','suborbital rocket','success',839,'II6.22F','Auroral particles','IGY II6.22F','','Auroral',428,'',80 from dual union all
select 47720,'1979-M162','1979-11-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 47721,'1979-M163','1979-11-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47722,'1979-M164','1979-07-24 15:25:00','suborbital rocket','success',1281,'','VLF Probe 1','','','Ionos',1821,'',75 from dual union all
select 47723,'1979-M165','1979-07-24 15:55:00','suborbital rocket','success',1281,'','VLF Probe 2','','','Ionos',1821,'',75 from dual union all
select 47724,'1979-M17','1979-01-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47725,'1979-M18','1979-02-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47726,'1979-M19','1979-02-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47727,'1979-M20','1979-02-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47728,'1979-M21','1979-02-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47729,'1979-M22','1979-02-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 4773,'1957-S98','1957-08-28 20:21:40','suborbital rocket','success',841,'NN7.44F','NRL NN7.44F','IGY NN7.44F','','Solar UV/X',1415,'',96 from dual union all
select 47730,'1979-M23','1979-02-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47731,'1979-M24','1979-02-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47732,'1979-M25','1979-02-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47733,'1979-M26','1979-02-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47734,'1979-M27','1979-02-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47735,'1979-M28','1979-02-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47736,'1979-M29','1979-02-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47737,'1979-M30','1979-02-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47738,'1979-M31','1979-02-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47739,'1979-M32','1979-02-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 4774,'1957-S99','1957-08-28 21:02:00','military missile','success',638,'AM-2','R&D','','','Test',248,'',500 from dual union all
select 47740,'1979-M33','1979-02-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47741,'1979-M34','1979-03-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47742,'1979-M35','1979-03-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47743,'1979-M36','1979-03-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47744,'1979-M37','1979-03-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47745,'1979-M38','1979-03-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47746,'1979-M39','1979-03-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47747,'1979-M40','1979-03-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47748,'1979-M41','1979-03-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47749,'1979-M42','1979-03-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 4775,'1957-U01','1957-10-17 05:05:00','suborbital rocket','success',28,'USAF 88','Artificial Meteor','','','Meteorites',640,'',114 from dual union all
select 47750,'1979-M43','1979-03-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47751,'1979-M44','1979-04-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47752,'1979-M45','1979-04-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47753,'1979-M46','1979-04-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47754,'1979-M47','1979-04-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47755,'1979-M48','1979-04-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47756,'1979-M49','1979-04-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47757,'1979-M50','1979-04-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47758,'1979-M51','1979-04-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47759,'1979-M52','1979-05-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 4776,'1957-W01','1957-01-01 00:00:00','suborbital rocket','success',125,'','Test','','','Test',1895,'',50 from dual union all
select 47760,'1979-M53','1979-05-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47761,'1979-M54','1979-05-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47762,'1979-M55','1979-05-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47763,'1979-M56','1979-05-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47764,'1979-M57','1979-05-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47765,'1979-M58','1979-05-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47766,'1979-M59','1979-05-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47767,'1979-M60','1979-05-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47768,'1979-M61','1979-05-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47769,'1979-M62','1979-05-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 4777,'1957-W02','1957-01-01 00:00:00','suborbital rocket','success',125,'','Test','','','Test',1895,'',50 from dual union all
select 47770,'1979-M63','1979-05-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47771,'1979-M64','1979-06-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47772,'1979-M65','1979-06-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47773,'1979-M66','1979-06-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47774,'1979-M67','1979-06-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47775,'1979-M68','1979-06-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47776,'1979-M69','1979-06-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47777,'1979-M70','1979-06-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47778,'1979-M71','1979-06-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47779,'1979-M72','1979-06-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 4778,'1957-W03','1957-01-01 00:00:00','suborbital rocket','success',124,'','Test','','','Test',1895,'',50 from dual union all
select 47780,'1979-M73','1979-06-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47781,'1979-M74','1979-06-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47782,'1979-M75','1979-06-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47783,'1979-M76','1979-06-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47784,'1979-M77','1979-06-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47785,'1979-M78','1979-06-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47786,'1979-M79','1979-06-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47787,'1979-M80','1979-07-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47788,'1979-M81','1979-07-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47789,'1979-M82','1979-07-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 4779,'1957-Y01','1957-07-11 02:05:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 47790,'1979-M83','1979-07-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47791,'1979-M84','1979-07-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47792,'1979-M85','1979-07-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47793,'1979-M86','1979-07-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47794,'1979-M87','1979-07-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47795,'1979-M88','1979-07-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47796,'1979-M89','1979-07-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47797,'1979-M90','1979-07-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47798,'1979-M91','1979-07-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 47799,'1979-M92','1979-07-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual;
insert into LAUNCH
select 478,'1944-S95','1944-09-14 00:00:00','military missile','success',8,'','Vertical test','','','Test',673,'',175 from dual union all
select 4780,'1957-Y02','1957-07-27 02:20:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 47800,'1979-M93','1979-08-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47801,'1979-M94','1979-08-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47802,'1979-M95','1979-08-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47803,'1979-M96','1979-08-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47804,'1979-M97','1979-08-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47805,'1979-M98','1979-08-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47806,'1979-M99','1979-08-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 47807,'1979-S01','1979-01-01 00:00:00','suborbital rocket','success',741,'MR-20','LKI','','','Test',636,'',250 from dual union all
select 47808,'1979-S02','1979-12-26 00:00:00','military missile','success',914,'67067-401(1L)','15A11 LKI-1','GCh 15B99 No. 01L','','Test',546,'',1000 from dual union all
select 47809,'1979-S03','1979-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 4781,'1957-Y03','1957-08-14 04:23:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 47810,'1979-S04','1979-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 47811,'1979-S05','1979-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 47812,'1979-S06','1979-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 47813,'1979-S07','1979-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 47814,'1979-S08','1979-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 47815,'1979-S09','1979-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 47816,'1979-S10','1979-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 47817,'1979-S100','1979-03-21 03:45:00','suborbital rocket','success',133,'NASA 25.33UG','','','','UV Astron',1895,'',221 from dual union all
select 47818,'1979-S101','1979-03-22 23:53:00','suborbital rocket','success',279,'CR-05','Spread F','','','Ba release',323,'',268 from dual union all
select 47819,'1979-S102','1979-03-23 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT137M','GLORY TRIP 137M','OT',1702,'',1300 from dual union all
select 4782,'1957-Y04','1957-08-19 03:13:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 47820,'1979-S103','1979-03-23 22:25:00','military missile','success',939,'C3E','DASO','','','OT',421,'SSBN 630',500 from dual union all
select 47821,'1979-S104','1979-03-24 11:04:40','suborbital rocket','success',1320,'NASA 12.1003UE','','','','Plasma',1047,'',500 from dual union all
select 47822,'1979-S105','1979-03-26 02:53:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',1010,'',165 from dual union all
select 47823,'1979-S106','1979-03-28 00:00:00','military missile','success',799,'','Mk 12 RV x 3','OT GT39GM','GLORY TRIP 39GM','OT',1705,'',1300 from dual union all
select 47824,'1979-S107','1979-03-28 15:38:24','suborbital rocket','success',921,'P208A','VLF waves','','','Ionospheric',29,'',169 from dual union all
select 47825,'1979-S108','1979-03-31 02:45:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',1010,'',165 from dual union all
select 47826,'1979-S109','1979-03-31 22:29:00','suborbital rocket','success',117,'NASA 24.06IE','DLR K-AR-93','Porcupine 4','','Magnetospheric',407,'',452 from dual union all
select 47827,'1979-S11','1979-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 47828,'1979-S110','1979-04-01 10:31:41','suborbital rocket','success',1320,'NASA 12.1004UE','','','','Plasma',1047,'',500 from dual union all
select 47829,'1979-S111','1979-04-04 00:00:00','military missile','success',932,'A3TA','FOT','','','OT',1897,'SSBN 608',1000 from dual union all
select 4783,'1957-Y05','1957-09-14 03:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 47830,'1979-S112','1979-04-04 00:00:00','military missile','success',932,'A3TA','FOT','','','OT',1897,'SSBN 608',1000 from dual union all
select 47831,'1979-S113','1979-04-04 00:00:00','suborbital rocket','success',487,'F.5','CHEVALINE F5','','','Test',1836,'',103 from dual union all
select 47832,'1979-S114','1979-04-04 23:47:00','military missile','success',932,'A3TK P-3','CHEVALINE P3','','','Test',251,'',1000 from dual union all
select 47833,'1979-S115','1979-04-05 00:34:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/Plasma',1010,'',175 from dual union all
select 47834,'1979-S116','1979-04-07 01:48:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',1010,'',170 from dual union all
select 47835,'1979-S117','1979-04-11 00:00:00','military missile','success',932,'A3TA','FOT','','','OT',1897,'SSBN 599',1000 from dual union all
select 47836,'1979-S118','1979-04-11 00:00:00','military missile','success',932,'A3TA','FOT','','','OT',1897,'SSBN 599',1000 from dual union all
select 47837,'1979-S119','1979-04-11 00:00:00','military missile','success',932,'A3TA','FOT','','','OT',1897,'SSBN 599',1000 from dual union all
select 47838,'1979-S12','1979-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 47839,'1979-S120','1979-04-11 10:45:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/Plasma',1010,'',180 from dual union all
select 4784,'1957-Y06','1957-09-17 03:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 47840,'1979-S121','1979-04-13 22:05:00','suborbital rocket','success',211,'NASA 27.33AS-US','CfA/HAO Rocket Coron. 1','','','Solar Opt/Solar UV',1895,'',268 from dual union all
select 47841,'1979-S122','1979-04-14 18:44:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/Plasma',1010,'',175 from dual union all
select 47842,'1979-S123','1979-04-15 09:25:00','suborbital rocket','success',867,'NASA 18.218GE','','','','Plasma',1047,'',270 from dual union all
select 47843,'1979-S124','1979-04-15 09:31:01','suborbital rocket','success',1337,'NASA 29.11GE','','','','Plasma',1047,'',671 from dual union all
select 47844,'1979-S125','1979-04-18 09:09:00','suborbital rocket','success',1337,'NASA 29.12GE','','','','Plasma',1047,'',500 from dual union all
select 47845,'1979-S126','1979-04-18 09:15:00','suborbital rocket','success',867,'NASA 18.217GE','','','','Plasma',1047,'',270 from dual union all
select 47846,'1979-S127','1979-04-19 00:00:00','suborbital rocket','success',765,'Sandia 497-009','Malemute II','','','Test',1490,'',165 from dual union all
select 47847,'1979-S128','1979-04-19 00:00:00','military missile','success',799,'PVM-17','Mk 12A RV x 2','R&D','PVM-17','Test',1707,'',1300 from dual union all
select 47848,'1979-S129','1979-04-19 00:00:00','military missile','success',1004,'46L','15A14 UBP-23 DP','GCh 15F678','','OT',492,'',1000 from dual union all
select 47849,'1979-S13','1979-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 4785,'1957-Y07','1957-09-20 02:30:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 47850,'1979-S130','1979-04-21 10:10:00','suborbital rocket','success',211,'NASA 27.38UE','','','','Plasma',1895,'',317 from dual union all
select 47851,'1979-S131','1979-04-25 12:24:00','suborbital rocket','success',920,'P201H','','','','Ionospheric',1447,'',130 from dual union all
select 47852,'1979-S132','1979-04-25 15:03:00','military missile','unknown',917,'FOT-125','P-379 78 Cape 13','','','OT',231,'',180 from dual union all
select 47853,'1979-S133','1979-04-25 15:17:00','military missile','unknown',917,'FOT-124','P-380 78 Cape 14','','','OT',231,'',180 from dual union all
select 47854,'1979-S134','1979-04-25 15:40:00','military missile','unknown',917,'FOT-123','P-381 78 Cape 15','','','OT',231,'',180 from dual union all
select 47855,'1979-S135','1979-05-09 17:00:00','military missile','unknown',917,'FOT-128','P-382 78 Cape 16','','','OT',231,'',180 from dual union all
select 47856,'1979-S136','1979-05-09 18:29:00','military missile','unknown',917,'FOT-126','P-383 78 Cape 17','','','OT',231,'',180 from dual union all
select 47857,'1979-S137','1979-05-09 18:50:00','military missile','unknown',917,'FOT-127','P-384 78 Cape 18','','','OT',231,'',180 from dual union all
select 47858,'1979-S138','1979-05-21 13:15:00','military missile','success',745,'45055-509(12L)','15A16 LKI-11','GCh 15F161','','OT',524,'',1000 from dual union all
select 47859,'1979-S139','1979-05-21 05:00:00','suborbital rocket','success',133,'NASA 25.43GG','UIT Proto','','','UV Astron',1895,'',226 from dual union all
select 4786,'1957-Y08','1957-11-04 07:55:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 47860,'1979-S14','1979-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 47861,'1979-S140','1979-05-21 06:40:00','suborbital rocket','success',133,'NASA 25.44GG','','','','UV Astron?',1895,'',240 from dual union all
select 47862,'1979-S141','1979-05-26 22:15:03','military missile','success',939,'C3E','DASO','','','OT',421,'SSBN 631',500 from dual union all
select 47863,'1979-S142','1979-05-31 12:45:00','military missile','success',745,'51055-510(13L)','15A16 LKI-12','GCh 15F161','','OT',546,'',1000 from dual union all
select 47864,'1979-S143','1979-06-02 00:00:00','suborbital rocket','success',133,'NASA 25.27DG','NRL FUV Spectrograph','','','UV Astron',1895,'',200 from dual union all
select 47865,'1979-S144','1979-06-05 17:15:00','suborbital rocket','success',211,'NASA 27.28US','','','','Solar',1895,'',325 from dual union all
select 47866,'1979-S145','1979-06-07 00:00:00','suborbital rocket','success',132,'NASA 23.12UU','','','','Aeron',1821,'',90 from dual union all
select 47867,'1979-S146','1979-06-08 18:37:00','military missile','success',1442,'C4T-3','PEM-3','','','OT',421,'SSBN 657',1000 from dual union all
select 47868,'1979-S147','1979-06-13 00:00:00','suborbital rocket','success',466,'E017','Ariane range test','','','Test',340,'',350 from dual union all
select 47869,'1979-S148','1979-06-15 18:30:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',170 from dual union all
select 4787,'1957-Y09','1957-12-16 04:35:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 47870,'1979-S149','1979-06-19 18:00:00','military missile','success',1442,'C4T-2','PEM-2','','','OT',421,'SSBN 657',1000 from dual union all
select 47871,'1979-S15','1979-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 47872,'1979-S150','1979-06-21 05:25:00','suborbital rocket','success',211,'NASA 27.040UG','','','','XR Astron',1895,'',215 from dual union all
select 47873,'1979-S151','1979-06-24 17:06:00','suborbital rocket','success',1028,'560.12A','','','','XR Astron',1361,'',332 from dual union all
select 47874,'1979-S152','1979-06-26 04:00:00','military missile','success',745,'67087-401(14L)','15A16 LKI-13','GCh 15F161','','OT',546,'',1000 from dual union all
select 47875,'1979-S153','1979-06-29 12:55:00','military missile','success',745,'67087-514-20T','15A16 LKI-14 K','GCh 15F161','','OT',524,'',1000 from dual union all
select 47876,'1979-S154','1979-06-29 08:15:00','suborbital rocket','success',227,'NASA 21.59UG','JHU FOT 3','','','UV Astron',1873,'',238 from dual union all
select 47877,'1979-S155','1979-07-01 00:00:00','military missile','success',755,'','','','','OT',307,'S613',600 from dual union all
select 47878,'1979-S156','1979-07-03 16:30:00','suborbital rocket','success',211,'NASA 27.32CS','XSST/TRC 1','','','Solar UV/X',1895,'',300 from dual union all
select 47879,'1979-S157','1979-07-06 00:00:00','military missile','success',795,'','Mk 12A LBRV-1','ABRES LBRV-1','LBRV-1','Reentry Test',1698,'',1300 from dual union all
select 4788,'1957-Y10','1957-12-16 04:35:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 47880,'1979-S158','1979-07-06 01:20:00','military missile','success',932,'A3TK C9E','CHEVALINE P4','','','Test',250,'',1000 from dual union all
select 47881,'1979-S159','1979-07-10 17:39:00','military missile','success',799,'Global Shield 79','Mk 12 RV x 3','OT GT40GM','GLORY TRIP 40GM','OT',1705,'',1300 from dual union all
select 47882,'1979-S16','1979-09-26 00:00:00','test rocket','unknown',1497,'15','K65UP No. 15','','','Ionos/Solar?',621,'',1500 from dual union all
select 47883,'1979-S160','1979-07-10 17:39:12','military missile','success',799,'Global Shield 79','Mk 12 RV x 3','FOT GT68GM','GLORY TRIP 68GM','OT',1703,'',1300 from dual union all
select 47884,'1979-S161','1979-07-14 18:00:00','military missile','success',1442,'C4T-5','PEM-4','','','OT',421,'SSBN 657',1000 from dual union all
select 47885,'1979-S162','1979-07-15 00:00:00','military missile','success',436,'','','','','Test',810,'',1000 from dual union all
select 47886,'1979-S163','1979-07-16 00:00:00','military missile','success',796,'Global Shield 79','ECRS','ERCS GIANT MOON 9','GIANT MOON 9','Comms',1699,'',1300 from dual union all
select 47887,'1979-S164','1979-07-16 00:00:00','military missile','success',1106,'TS-2601','Qualification','','','Test',301,'',1000 from dual union all
select 47888,'1979-S165','1979-07-16 12:31:30','suborbital rocket','success',1337,'Sandia 497-011','DNA PLUMEX I','','','Aeron/Release',873,'',590 from dual union all
select 47889,'1979-S166','1979-07-20 12:20:00','military missile','success',745,'51055-507(10L)','15A16 LKI-15','GCh 15F161','','OT',528,'',1000 from dual;
insert into LAUNCH
select 4789,'1957-Y11','1957-12-21 04:40:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 47890,'1979-S167','1979-07-20 00:00:00','suborbital rocket','success',867,'NASA 18.1026GU','','','','Aeron',1895,'',270 from dual union all
select 47891,'1979-S168','1979-07-20 00:00:00','suborbital rocket','success',920,'P211H','Atomic oxygen','','','Aeronomy',1447,'',145 from dual union all
select 47892,'1979-S169','1979-07-20 00:00:00','suborbital rocket','success',920,'P210H','Atomic oxygen','','','Aeronomy',1447,'',149 from dual union all
select 47893,'1979-S17','1979-10-24 00:00:00','test rocket','unknown',1497,'16','K65UP No. 16','','','Ionos/Solar?',621,'',1500 from dual union all
select 47894,'1979-S170','1979-07-20 00:00:00','suborbital rocket','success',920,'P209H','Atomic oxygen','','','Aeronomy',1447,'',149 from dual union all
select 47895,'1979-S171','1979-07-22 18:00:00','military missile','success',1442,'C4TE-4-PEM-5','Imp. Evader 210','','','OT',421,'SSBN 657',1000 from dual union all
select 47896,'1979-S172','1979-07-23 09:57:30','suborbital rocket','success',1337,'Sandia 497-012','DNA PLUMEX II','','','Aeron/Release',873,'',580 from dual union all
select 47897,'1979-S173','1979-07-26 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT69GB','GLORY TRIP 69GB','OT',1712,'',1300 from dual union all
select 47898,'1979-S174','1979-08-01 00:00:00','military missile','success',997,'','','','','Test',1626,'K-140',1000 from dual union all
select 47899,'1979-S175','1979-07-31 02:15:00','military missile','success',1442,'C4T-7','PEM-7','','','OT',421,'SSBN 657',1000 from dual union all
select 479,'1944-S96','1944-09-14 00:00:00','military missile','success',1470,'','Lille','','','Weapon',1730,'',95 from dual union all
select 4790,'1957-Y12','1957-12-21 04:40:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 47900,'1979-S176','1979-07-31 04:33:00','military missile','success',1442,'C4T-6','PEM-6','','','OT',421,'SSBN 657',1000 from dual union all
select 47901,'1979-S177','1979-08-02 11:30:00','military missile','success',745,'51087-403(16L)','15A16 LKI-16','GCh 15F161','','OT',524,'',1000 from dual union all
select 47902,'1979-S178','1979-08-03 18:54:00','military missile','success',745,'67087-404(17L)','15A16 LKI-17','GCh 15F161','','OT',546,'',1000 from dual union all
select 47903,'1979-S179','1979-08-03 00:00:00','military missile','failure',795,'','STREP-2','ABRES STREP-2','STREP-2','Reentry Test',1701,'',1300 from dual union all
select 47904,'1979-S18','1979-04-25 12:40:00','military missile','success',1006,'67087-401(14L)','15A18 LKI-14','GCh 15F183','','OT',485,'',1000 from dual union all
select 47905,'1979-S180','1979-08-03 09:20:00','suborbital rocket','success',133,'GL A31.702','FWIR III','','','Aeron/IR Astron',1895,'',172 from dual union all
select 47906,'1979-S181','1979-08-07 17:14:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron',636,'',160 from dual union all
select 47907,'1979-S182','1979-08-07 19:50:00','suborbital rocket','success',228,'GL A18.805','POSTBO','','','ABM Tech',1873,'',219 from dual union all
select 47908,'1979-S183','1979-08-10 17:12:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',160 from dual union all
select 47909,'1979-S184','1979-08-14 18:10:00','suborbital rocket','success',860,'SSC T3 Test','Dummy Orion','','','Test',417,'',11 from dual union all
select 4791,'1957-Y13','1957-12-21 08:45:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 47910,'1979-S185','1979-08-14 16:50:00','suborbital rocket','success',867,'GL A08.705-2','DMSP MS','','','Aeron',1895,'',190 from dual union all
select 47911,'1979-S186','1979-08-14 20:20:00','suborbital rocket','success',37,'GL A04.703','DMSP UV','','','Aeron',1895,'',178 from dual union all
select 47912,'1979-S187','1979-08-17 21:15:00','suborbital rocket','success',133,'NASA 25.11AS','','','','Solar',1895,'',225 from dual union all
select 47913,'1979-S188','1979-08-18 07:00:00','suborbital rocket','success',668,'K-9M-67','TED/TPI/NEL/PWN','','','Ionos',839,'',365 from dual union all
select 47914,'1979-S189','1979-08-21 00:00:00','military missile','success',436,'','','','','Test',810,'',1000 from dual union all
select 47915,'1979-S19','1979-04-26 02:20:00','military missile','success',1006,'67087-514(24T)','15A18 K-1','GCh 15F183','','OT',489,'',1000 from dual union all
select 47916,'1979-S190','1979-08-21 01:14:16','suborbital rocket','success',860,'NASA 31.16UE','SSC S27B Twilight B','','','Aeronomy',412,'',149 from dual union all
select 47917,'1979-S191','1979-08-22 16:42:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',155 from dual union all
select 47918,'1979-S192','1979-08-23 00:00:00','suborbital rocket','success',1218,'XV-07','','','','Ionos?',330,'',248 from dual union all
select 47919,'1979-S193','1979-08-25 01:40:00','suborbital rocket','success',1299,'TT-500-6F','','','','Test',1476,'',300 from dual union all
select 4792,'1958 ALP','1958-02-01 03:47:56','orbital','success',636,'RS-29    UE','Explorer 1','Explorer 1','','Sat',248,'',null from dual union all
select 47920,'1979-S194','1979-08-29 00:30:00','military missile','success',1442,'','DASO-1','','','OT',421,'SSBN 657',1000 from dual union all
select 47921,'1979-S195','1979-08-30 17:00:00','military missile','success',745,'45087-405(18L)','15A16 LKI-18','GCh 15F161','','OT',524,'',1000 from dual union all
select 47922,'1979-S196','1979-08-30 00:00:00','military missile','success',799,'STM-18W','Mk 12A RV x 3','R&D','STM-18W','Test',1707,'',1300 from dual union all
select 47923,'1979-S197','1979-09-01 04:16:00','military missile','success',932,'A3TK P-5','CHEVALINE P5','','','Test',251,'',1000 from dual union all
select 47924,'1979-S198','1979-09-04 00:00:00','military missile','success',436,'','','','','Test',1494,'',1000 from dual union all
select 47925,'1979-S199','1979-09-11 00:00:00','suborbital rocket','success',860,'NASA 31.04IU','Ozone payload test','','','Aeron',1821,'',83 from dual union all
select 47926,'1979-S20','1979-05-30 03:50:00','military missile','success',1006,'45087-402(15L)','15A18 LKI-15','GCh 15F183','','OT',487,'',1000 from dual union all
select 47927,'1979-S200','1979-09-11 01:00:00','suborbital rocket','success',668,'K-9M-68','SUV/AUV/NEL/EPA','','','Ionos/Solar UV',839,'',353 from dual union all
select 47928,'1979-S201','1979-09-14 19:20:00','military missile','success',745,'45087-402(15L)','15A16 LKI-19','GCh 15F161','','OT',528,'',1000 from dual union all
select 47929,'1979-S202','1979-09-15 11:00:00','suborbital rocket','success',1073,'S-310-7','','','','Aeron',839,'',166 from dual union all
select 4793,'1958 BET','1958-03-17 12:15:41','orbital','success',1486,'TV-4','Vanguard I','Vanguard Test Satellite','','Sat',235,'',null from dual union all
select 47930,'1979-S203','1979-09-16 00:00:00','suborbital rocket','success',132,'NASA 23.16UE','','','','Plasma',1821,'',90 from dual union all
select 47931,'1979-S204','1979-09-20 00:00:00','suborbital rocket','success',132,'NASA 23.13UU','','','','Aeron',1821,'',90 from dual union all
select 47932,'1979-S205','1979-09-20 21:00:00','suborbital rocket','success',228,'NASA 21.57UL','','','','Aeron',1873,'',278 from dual union all
select 47933,'1979-S206','1979-09-22 00:00:00','military missile','success',795,'','ABRV-2','ABRES ABRV-2','ABRV-2','Reentry Test',1701,'',1300 from dual union all
select 47934,'1979-S207','1979-09-24 11:15:00','suborbital rocket','success',133,'NASA 25.39UG','JHU UVS','','','UV Astron',1895,'',246 from dual union all
select 47935,'1979-S208','1979-09-24 20:20:10','suborbital rocket','success',211,'NASA 27.39UE','SSC S29 Barium-GEOS','','','Auroral/release',412,'',397 from dual union all
select 47936,'1979-S209','1979-09-25 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT138M','GLORY TRIP 138GM','OT',1699,'',1300 from dual union all
select 47937,'1979-S21','1979-06-27 12:30:00','military missile','success',1006,'51087-403(16L)','15A18 LKI-16','GCh 15F183','','OT',483,'',1000 from dual union all
select 47938,'1979-S210','1979-09-25 00:00:00','suborbital rocket','success',1164,'ABRES','HAVE LENT IV','','','Test?',855,'',400 from dual union all
select 47939,'1979-S211','1979-09-26 03:20:00','suborbital rocket','success',1014,'','Vertikal''-8','VZAF','','Solar UV/Solar X',636,'',505 from dual union all
select 4794,'1958 DEL','1958-05-15 07:00:35','orbital','success',1254,'B1-1','ISZ D-1 No. 2','D-1 No. 2','','Sat',480,'',null from dual union all
select 47940,'1979-S212','1979-09-26 23:44:00','military missile','success',1442,'','DASO-2','','','OT',421,'SSBN 657',1000 from dual union all
select 47941,'1979-S213','1979-09-27 11:30:00','suborbital rocket','success',228,'NASA 21.62GS','','','','Solar',1873,'',247 from dual union all
select 47942,'1979-S214','1979-09-28 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT70GM','GLORY TRIP 70GM','OT',1703,'',1300 from dual union all
select 47943,'1979-S215','1979-10-01 00:00:00','suborbital rocket','success',132,'NASA 23.14UU','','','','Aeron',1821,'',90 from dual union all
select 47944,'1979-S216','1979-10-03 18:42:00','suborbital rocket','success',921,'P218E','INTA PSU/P.218E','Ba release','','Ionospheric',396,'',145 from dual union all
select 47945,'1979-S217','1979-10-12 14:09:00','suborbital rocket','success',1203,'DLR N-GRC-97','Astro-Hel','','','FUV Astron',329,'',829 from dual union all
select 47946,'1979-S218','1979-10-13 00:00:00','suborbital rocket','success',1337,'Sandia 497-010','LASL Aguila','','','Test/Release',855,'',533 from dual union all
select 47947,'1979-S22','1979-10-17 12:50:00','military missile','success',1006,'67087-404(17L)','15A18 LKI-17','GCh 15F183','','OT',487,'',1000 from dual union all
select 47948,'1979-S220','1979-10-17 12:40:00','suborbital rocket','success',211,'NASA 27.16NP','SPAR 6','','','Micrograv',1873,'',185 from dual union all
select 47949,'1979-S221','1979-10-17 14:50:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos/Plasma/Fields',636,'',160 from dual union all
select 4795,'1958 EPS','1958-07-26 15:00:57','orbital','success',636,'RS/CC-44 TT','Explorer 4','Explorer 4','','Sat',279,'',null from dual union all
select 47950,'1979-S222','1979-10-18 00:55:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos/Plasma/Fields',636,'',165 from dual union all
select 47951,'1979-S223','1979-10-19 05:46:40','suborbital rocket','success',1303,'GL A51.970','EXCEDE 2 (SPECTRAL)','','','Active Plasma',1047,'',128 from dual union all
select 47952,'1979-S224','1979-10-21 16:25:00','suborbital rocket','success',860,'NASA 31.10IU','WI T2-9915','','','Aeron',1821,'',140 from dual union all
select 47953,'1979-S225','1979-10-21 17:48:00','suborbital rocket','success',860,'NASA 31.09IU','WI T2-9914','','','Aeron',1821,'',140 from dual union all
select 47954,'1979-S226','1979-10-21 00:00:00','suborbital rocket','success',860,'NASA 31.08IU','','','','Aeron',1821,'',140 from dual union all
select 47955,'1979-S227','1979-10-22 09:00:00','suborbital rocket','unknown',740,'MR-12','','','','Ionos',636,'',170 from dual union all
select 47956,'1979-S228','1979-10-22 16:50:00','suborbital rocket','success',860,'DLR N-NO-98','Natal 6','','','Test',330,'',140 from dual union all
select 47957,'1979-S229','1979-10-23 03:15:00','suborbital rocket','success',133,'NASA 25.49GG','','','','UV Astron?',1895,'',227 from dual union all
select 47958,'1979-S23','1979-10-20 03:05:00','military missile','success',1006,'45087-524-23T','15A18 K-2','GCh 15F183','','OT',489,'',1000 from dual union all
select 47959,'1979-S230','1979-10-23 21:51:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 628',500 from dual union all
select 4796,'1958 ETA','1958-10-11 08:42:13','orbital','success',1353,'130','Pioneer 1','Able 2','','Sat',233,'',null from dual union all
select 47960,'1979-S231','1979-10-25 20:05:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/Plasma/',636,'',150 from dual union all
select 47961,'1979-S232','1979-10-26 21:45:00','suborbital rocket','success',860,'DLR N-NO-99','Natal 6','','','Test',330,'',140 from dual union all
select 47962,'1979-S233','1979-11-01 00:00:00','suborbital rocket','success',860,'NASA 31.13IU','Ozone Intercomp','','','Aeron',1821,'',140 from dual union all
select 47963,'1979-S234','1979-11-01 00:00:00','suborbital rocket','success',860,'NASA 31.12IU','Ozone Intercomp','','','Aeron',1821,'',140 from dual union all
select 47964,'1979-S235','1979-11-01 00:00:00','suborbital rocket','success',860,'NASA 31.11IU','Ozone Intercomp','','','Aeron',1821,'',140 from dual union all
select 47965,'1979-S236','1979-11-04 09:27:00','suborbital rocket','success',891,'NASA 30.09IU','','','','Aeron',1821,'',82 from dual union all
select 47966,'1979-S237','1979-11-07 20:50:00','suborbital rocket','success',211,'NASA 27.30CS','','','','Solar',1895,'',271 from dual union all
select 47967,'1979-S238','1979-11-08 00:00:00','suborbital rocket','success',1319,'NASA 33.08UL','','','','Aeron',1873,'',200 from dual union all
select 47968,'1979-S239','1979-11-08 22:00:00','military missile','success',932,'A3TK P-6','CHEVALINE P6','','','Test',251,'',1000 from dual union all
select 47969,'1979-S24','1979-11-23 15:00:00','military missile','success',1006,'45087-405(18L)','15A18 LKI-18','GCh 15F183','','OT',483,'',1000 from dual union all
select 4797,'1958 GAM','1958-03-26 17:38:03','orbital','success',636,'RS-24    UT','Explorer 3','Explorer 3','','Sat',279,'',null from dual union all
select 47970,'1979-S240','1979-11-13 04:45:00','suborbital rocket','success',1269,'NASA 12.37UE','ECHO V','','','Auroral',1047,'',215 from dual union all
select 47971,'1979-S241','1979-11-15 12:55:00','military missile','success',745,'45067-524-14T','15A16 UBP-1 ZP','GCh 15F161','','OT',528,'',1000 from dual union all
select 47972,'1979-S242','1979-11-15 00:00:00','suborbital rocket','success',290,'','AST 2','','','Tech',1467,'',148 from dual union all
select 47973,'1979-S243','1979-11-16 17:00:00','suborbital rocket','success',30,'NASA 04.337CS','','','','Solar',1869,'',154 from dual union all
select 47974,'1979-S244','1979-11-26 00:00:00','military missile','success',436,'','','','','Test',1494,'',1000 from dual union all
select 47975,'1979-S245','1979-12-01 00:00:00','military missile','success',10,'','','','','ABM Test',183,'',100 from dual union all
select 47976,'1979-S246','1979-12-03 05:32:47','suborbital rocket','success',210,'AAF-NVB-06','Auroral substorm','','','Auroral/Ionos',428,'',344 from dual union all
select 47977,'1979-S247','1979-12-12 01:05:00','suborbital rocket','success',30,'NASA 04.338UE','UM Airglow Pay.','','','Magnetospheric',1869,'',122 from dual union all
select 47978,'1979-S248','1979-12-13 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT139M','GLORY TRIP 139M','OT',1702,'',1300 from dual union all
select 47979,'1979-S249','1979-12-13 17:30:00','military missile','success',1442,'','DASO-3','','','OT',421,'SSBN 658',1000 from dual;
insert into LAUNCH
select 4798,'1958 THE','1958-12-06 05:44:52','orbital','success',634,'AM-11','Pioneer 3','Pioneer 3','','Sat',279,'',null from dual union all
select 47980,'1979-S25','1979-11-27 14:25:00','military missile','success',1006,'51067-406(19L)','15A18 LKI-19','GCh 15F183','','OT',487,'',1000 from dual union all
select 47981,'1979-S250','1979-12-14 00:00:00','suborbital rocket','success',133,'NASA 25.48UG','','0.3m UVT','','UV Astron',1895,'',200 from dual union all
select 47982,'1979-S251','1979-12-17 06:27:00','suborbital rocket','success',226,'ADD-VA-48','','','','Auroral',428,'',138 from dual union all
select 47983,'1979-S252','1979-12-18 00:00:00','suborbital rocket','success',132,'NASA 23.15UU','','','','Aeron',1821,'',90 from dual union all
select 47984,'1979-S253','1979-12-19 01:57:00','suborbital rocket','unknown',740,'MR-12','','','','Solar UV/Solar EUV',636,'',165 from dual union all
select 47985,'1979-S254','1979-12-20 00:00:00','military missile','success',795,'','AMARV FTV-1','ABRES AMARV-1','AMARV-1','Reentry Test',1698,'',1300 from dual union all
select 47986,'1979-S255','1979-12-20 11:58:00','suborbital rocket','unknown',740,'MR-12','','','','Solar UV/Solar EUV',636,'',165 from dual union all
select 47987,'1979-S256','1979-03-26 00:00:00','military missile','success',744,'','15A15 UBP-5','','','OT',596,'',1000 from dual union all
select 47988,'1979-S257','1979-12-26 14:05:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',170 from dual union all
select 47989,'1979-S258','1979-12-27 14:07:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',636,'',165 from dual union all
select 4799,'1958 ZET','1958-12-18 23:02:00','orbital','success',147,'10B','SCORE','SCORE','','Sat',226,'',1484 from dual union all
select 47990,'1979-S259','1979-02-08 00:00:00','suborbital rocket','success',364,'DA006','Ariane range test','','','Test',340,'',135 from dual union all
select 47991,'1979-S26','1979-11-29 11:45:00','military missile','success',1006,'51067-534-23T','15A18 UBP-1 ZP','GCh 15F183','','OT',489,'',1000 from dual union all
select 47992,'1979-S260','1979-08-25 06:21:00','suborbital rocket','success',1025,'','ISRO-CNES I','','','Ionos',1467,'',150 from dual union all
select 47993,'1979-S261','1979-08-31 05:51:00','suborbital rocket','success',1025,'','ISRO-CNES II','','','Ionos',1467,'',150 from dual union all
select 47994,'1979-S262','1979-09-29 00:00:00','military missile','success',984,'Aerofon','','','','Test',636,'',86 from dual union all
select 47995,'1979-S263','1979-09-20 02:30:00','suborbital rocket','success',700,'L-4SC-5','TVC test','','','Test',834,'',82 from dual union all
select 47996,'1979-S264','1979-11-13 15:06:00','military missile','unknown',917,'AO-194','P-385 79 Fall 01','','','OT',433,'',180 from dual union all
select 47997,'1979-S265','1979-12-04 15:04:00','military missile','unknown',917,'AO-195','P-386 79 Fall 02','','','OT',433,'',180 from dual union all
select 47998,'1979-S27','1979-02-25 00:00:00','military missile','success',1453,'51117-92-573T/302P','15A35 LKI-16','','','OT',535,'',1000 from dual union all
select 47999,'1979-S271','1979-08-10 00:00:00','military missile','success',923,'U01','15Zh53 LKI-1','','','Test',636,'',500 from dual union all
select 48,'1944-A12','1944-08-18 16:33:00','military missile','failure',8,'18350','Ma156','','','Training',661,'',40 from dual union all
select 480,'1944-S97','1944-09-14 00:00:00','military missile','success',1470,'','Mons','','','Weapon',1730,'',95 from dual union all
select 4800,'1958-A01','1958-01-09 15:35:00','suborbital rocket','success',838,'II8.10','','IGY II8.10','','Test/Solar UV/Solar',1895,'',15 from dual union all
select 48000,'1979-S272','1979-01-01 00:00:00','military missile','success',923,'U02','15Zh53 LKI-2','','','Test',636,'',500 from dual union all
select 48001,'1979-S273','1979-01-01 00:00:00','military missile','success',923,'U03','15Zh53 LKI-3','','','Test',636,'',500 from dual union all
select 48002,'1979-S274','1979-01-01 00:00:00','military missile','success',923,'U04','15Zh53 LKI-4','','','Test',636,'',500 from dual union all
select 48003,'1979-S275','1979-01-01 00:00:00','military missile','success',923,'U05','15Zh53 LKI-5','','','Test',636,'',500 from dual union all
select 48004,'1979-S276','1979-08-01 00:00:00','military missile','success',886,'','Stage 1, last','','','Test',636,'',120 from dual union all
select 48005,'1979-S277','1979-09-01 00:00:00','military missile','success',886,'','Stage 2, first','','','Test',636,'',120 from dual union all
select 48006,'1979-S278','1979-01-17 00:00:00','military missile','success',1037,'Yu64612-31','8K98 UBP-44','','','Test',477,'',1000 from dual union all
select 48007,'1979-S279','1979-02-28 00:00:00','military missile','success',1037,'Yu64612-25','8K98 UBP-45','','','Test',477,'',1000 from dual union all
select 48008,'1979-S28','1979-05-24 00:00:00','military missile','success',1453,'49217-82-016T','15A35 LKI-17','','','OT',535,'',1000 from dual union all
select 48009,'1979-S280','1979-04-17 00:00:00','military missile','success',1037,'Yu64612-24','8K98 UBP-46','','','Test',477,'',1000 from dual union all
select 4801,'1958-A02','1958-01-10 19:38:00','military missile','success',832,'13','G-26 No. 9','','','Test',281,'',22 from dual union all
select 48010,'1979-S281','1979-04-25 00:00:00','military missile','success',1044,'29597-166-04','8K98P UBP-15','','','Test',477,'',1000 from dual union all
select 48011,'1979-S282','1979-07-30 00:00:00','military missile','success',1044,'29597-166-05','8K98P UBP-16','','','Test',477,'',1000 from dual union all
select 48012,'1979-S283','1979-09-26 00:00:00','military missile','success',1037,'Yu29501-13','8K98 UBP-47','','','Test',477,'',1000 from dual union all
select 48013,'1979-S284','1979-10-24 00:00:00','military missile','success',1037,'Yu29501-12','8K98 UBP-48','','','Test',477,'',1000 from dual union all
select 48014,'1979-S285','1979-12-05 00:00:00','military missile','success',1044,'Yu64612-26','8K98 UBP-49','','','Test',477,'',1000 from dual union all
select 48015,'1979-S286','1979-01-01 00:00:00','military missile','success',1452,'','15A30 60 RD UBP-2','','','OT',1461,'',1000 from dual union all
select 48016,'1979-S287','1979-01-01 00:00:00','military missile','success',1452,'','15A30 60 RD UBP-3','','','OT',1461,'',1000 from dual union all
select 48017,'1979-S288','1979-01-01 00:00:00','military missile','success',1452,'','15A30 60 RD UBP-4','','','OT',1461,'',1000 from dual union all
select 48018,'1979-S289','1979-08-03 00:00:00','military missile','success',1449,'','8K84 10 RD UBP-7','','','OT',897,'',1000 from dual union all
select 48019,'1979-S29','1979-05-29 00:00:00','military missile','success',1453,'49217-82-017T','15A35 LKI-18','','','OT',505,'',1000 from dual union all
select 4802,'1958-A03','1958-01-14 16:45:00','military missile','success',255,'624-','','','','OT',293,'',20 from dual union all
select 48020,'1979-S290','1979-01-01 00:00:00','military missile','success',436,'','','','','Test',1494,'',1000 from dual union all
select 48021,'1979-S291','1979-10-15 00:00:00','military missile','success',436,'','','','','Test',1494,'',1000 from dual union all
select 48022,'1979-S292','1979-12-04 15:36:00','military missile','unknown',917,'AO-196','P-386 79 Fall 03','','','OT',433,'',180 from dual union all
select 48023,'1979-S293','1979-04-17 10:02:00','suborbital rocket','success',725,'','','Ozonesonde','','Aeron',1467,'',80 from dual union all
select 48024,'1979-S294','1979-06-06 09:49:00','suborbital rocket','success',725,'','','Ozonesonde','','Aeron',1467,'',80 from dual union all
select 48025,'1979-S295','1979-10-02 02:00:00','suborbital rocket','success',920,'MOP5','Ozone','','','Ozone',1447,'',150 from dual union all
select 48026,'1979-S296','1979-10-02 06:00:00','suborbital rocket','success',920,'MOP6','Ozone','','','Ozone',1447,'',150 from dual union all
select 48027,'1979-S297','1979-10-02 09:30:00','suborbital rocket','success',920,'MOP7','Ozone','','','Ozone',1447,'',150 from dual union all
select 48028,'1979-S298','1979-10-02 15:30:00','suborbital rocket','success',920,'MOP8','Ozone','','','Ozone',1447,'',150 from dual union all
select 48029,'1979-S299','1979-03-21 23:49:00','suborbital rocket','success',279,'CR-04','Spread F','','','Ba release',323,'',250 from dual union all
select 4803,'1958-A04','1958-01-23 00:00:00','atmospheric rocket','success',545,'','Heat Transfer','','','Reentry Test',1821,'',10 from dual union all
select 48030,'1979-S30','1979-06-28 00:00:00','military missile','success',1453,'49217-82-018T','15A35 LKI-19','','','OT',535,'',1000 from dual union all
select 48031,'1979-S300','1979-02-02 00:00:00','suborbital rocket','success',1311,'02','','','','Test',310,'',100 from dual union all
select 48032,'1979-S301','1979-05-31 00:00:00','military missile','success',922,'','15Zh45  8 RD UBP','','','Test',636,'',500 from dual union all
select 48033,'1979-S302','1979-09-01 00:00:00','military missile','success',922,'','15Zh45 33 RD UBP','','','Test',636,'',500 from dual union all
select 48034,'1979-S303','1979-04-02 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 48035,'1979-S304','1979-11-01 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 48036,'1979-S305','1979-12-01 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 48037,'1979-S306','1979-12-01 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 48038,'1979-S307','1979-10-19 00:00:00','military missile','unknown',979,'','8K65 UBP 43 RD','','','Test',636,'',675 from dual union all
select 48039,'1979-S308','1979-10-26 00:00:00','military missile','unknown',980,'','8K65U UBP 29 RD','','','Test',636,'',675 from dual union all
select 4804,'1958-A05','1958-01-25 00:00:00','military missile','success',1214,'N-69E N-3327','','','','Test',224,'',10 from dual union all
select 48040,'1979-S31','1979-07-09 00:00:00','military missile','success',1453,'63097-93-373T','15A35 UBP-1 ZP','','','OT',505,'',1000 from dual union all
select 48041,'1979-S32','1979-09-12 00:00:00','military missile','success',1453,'51117-92-574T','15A35 UBP-2 ZP','','','OT',508,'',1000 from dual union all
select 48042,'1979-S33','1979-11-14 00:00:00','military missile','success',1453,'49297-94-558T','15A35 UBP-3 ZP','','','OT',535,'',1000 from dual union all
select 48043,'1979-S34','1979-01-07 00:00:00','military missile','success',436,'','','','','Test',1494,'',1000 from dual union all
select 48044,'1979-S35','1979-01-09 11:53:00','suborbital rocket','success',133,'NASA 25.32UL','','','','Aeron',1895,'',214 from dual union all
select 48045,'1979-S36','1979-01-16 20:50:00','suborbital rocket','success',668,'K-9M-65','','','','Aeron/Ionos',839,'',353 from dual union all
select 48046,'1979-S37','1979-01-17 20:00:00','suborbital rocket','success',133,'NASA 25.10AS','','','','Solar',1895,'',226 from dual union all
select 48047,'1979-S38','1979-01-19 00:00:00','military missile','success',795,'','STREP-1','ABRES STREP-1','STREP-1','Reentry Test',1701,'',1300 from dual union all
select 48048,'1979-S39','1979-01-19 08:50:00','suborbital rocket','success',1073,'S-310-6','','','','Aeron/Ionos',839,'',212 from dual union all
select 48049,'1979-S40','1979-01-21 06:16:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos',1010,'',180 from dual union all
select 4805,'1958-A06','1958-01-28 20:16:00','military missile','failure',1366,'114','R&D Series II','','','Test',233,'',0 from dual union all
select 48050,'1979-S41','1979-01-21 09:06:00','suborbital rocket','success',668,'K-9M-66','','','','Test/Aeron/Ionos/Pl',839,'',336 from dual union all
select 48051,'1979-S42','1979-01-23 15:00:00','military missile','success',1442,'C4X-21','','','','Test',246,'',1000 from dual union all
select 48052,'1979-S43','1979-01-24 00:00:00','military missile','success',795,'','ABRV-1','ABRES ABRV-1','ABRV-1','Reentry Test',1698,'',1300 from dual union all
select 48053,'1979-S44','1979-01-25 21:07:33','suborbital rocket','success',921,'P216K','Auroral pulsations','','','Ionospheric',413,'',179 from dual union all
select 48054,'1979-S45','1979-01-26 11:01:00','suborbital rocket','failure',1337,'NASA 29.13UE','','','','Plasma',1047,'',186 from dual union all
select 48055,'1979-S46','1979-01-27 01:30:00','suborbital rocket','success',1299,'TT-500-5F','','','','Test',1476,'',300 from dual union all
select 48056,'1979-S47','1979-01-27 17:12:32','suborbital rocket','success',228,'SSC S23-L1','Substorm-GEOS','','','Auroral',411,'',267 from dual union all
select 48057,'1979-S48','1979-01-27 20:04:55','suborbital rocket','success',921,'P215K','Auroral pulsations','','','Ionospheric',413,'',180 from dual union all
select 48058,'1979-S49','1979-01-27 21:01:59','suborbital rocket','success',211,'SSC S23H','Substorm-GEOS','','','Auroral',412,'',414 from dual union all
select 48059,'1979-S50','1979-01-27 21:52:20','suborbital rocket','success',228,'SSC S23-L2','Substorm-GEOS','','','Auroral',411,'',266 from dual union all
select 4806,'1958-A07','1958-01-31 00:00:00','military missile','success',262,'7','','','','Test',243,'',10 from dual union all
select 48060,'1979-S51','1979-01-28 03:07:00','suborbital rocket','failure',133,'NASA 25.41UE','Nitric Oxide','','','Aeron',428,'',100 from dual union all
select 48061,'1979-S52','1979-01-28 06:06:00','suborbital rocket','success',117,'GL A24.7S2-01','SPICE 1','','','IR Astron',1873,'',249 from dual union all
select 48062,'1979-S53','1979-01-30 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT66GM','GLORY TRIP 66GM','OT',1703,'',1300 from dual union all
select 48063,'1979-S54','1979-02-01 00:00:00','suborbital rocket','success',1268,'SWERVE','','','','Reentry Test',855,'',200 from dual union all
select 48064,'1979-S55','1979-11-14 00:00:00','suborbital rocket','success',466,'E016','Ariane range test','','','Test',340,'',350 from dual union all
select 48065,'1979-S56','1979-01-31 02:00:00','suborbital rocket','success',1073,'S-310-5','','','','Ionos/Fields',839,'',185 from dual union all
select 48066,'1979-S57','1979-02-06 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT67GB','GLORY TRIP 67GB','OT',1712,'',1300 from dual union all
select 48067,'1979-S58','1979-02-08 04:37:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/CR/Plas',1010,'',155 from dual union all
select 48068,'1979-S59','1979-02-14 00:00:00','suborbital rocket','success',487,'F.4','CHEVALINE F4','','','Test',1836,'',93 from dual;
insert into LAUNCH
select 48069,'1979-S60','1979-02-15 00:00:00','military missile','success',799,'PVM-16','Mk 12A RV x 3','R&D, Mk 12 Mod 1A','PVM-16','Test',1707,'',1300 from dual union all
select 4807,'1958-A08','1958-01-31 00:00:00','atmospheric rocket','success',1052,'','Heat Transfer','','','Test',1821,'F2H2',4 from dual union all
select 48070,'1979-S61','1979-02-19 03:32:00','military missile','success',939,'','FOT-36','','','OT',421,'SSBN 645',500 from dual union all
select 48071,'1979-S62','1979-02-19 03:33:00','military missile','success',939,'','FOT-36','','','OT',421,'SSBN 645',500 from dual union all
select 48072,'1979-S63','1979-02-19 03:35:00','military missile','success',939,'','FOT-36','','','OT',421,'SSBN 645',500 from dual union all
select 48073,'1979-S64','1979-02-19 05:00:00','military missile','success',939,'','FOT-36','','','OT',421,'SSBN 645',500 from dual union all
select 48074,'1979-S65','1979-02-20 00:00:00','suborbital rocket','success',132,'NASA 23.11UU','Nitric Oxide','','','Aeron',428,'',90 from dual union all
select 48075,'1979-S66','1979-02-20 17:00:00','military missile','success',932,'A3T','DASO','','','OT',421,'UK S23',1000 from dual union all
select 48076,'1979-S67','1979-02-22 12:15:30','suborbital rocket','success',1211,'DLR W-GR-96','Astro 4-2','','','Solar X',1839,'',192 from dual union all
select 48077,'1979-S68','1979-02-22 14:43:00','military missile','success',917,'FOT-121','P-376 78 Cape 10','','','OT',231,'',180 from dual union all
select 48078,'1979-S69','1979-02-22 15:00:00','military missile','success',917,'FOT-122','P-377 78 Cape 11','','','OT Wrong azimuth',231,'',180 from dual union all
select 48079,'1979-S70','1979-02-22 15:19:00','military missile','success',917,'FOT-120','P-378 78 Cape 12','','','OT',231,'',180 from dual union all
select 4808,'1958-A09','1958-02-14 00:00:00','military missile','success',1214,'N-69E N-3328','','','','Test',224,'',10 from dual union all
select 48080,'1979-S71','1979-02-23 14:09:00','suborbital rocket','success',740,'MR-12','','','','Aeron/Ionos/CR/Pla',1010,'',187 from dual union all
select 48081,'1979-S72','1979-02-24 16:52:00','suborbital rocket','success',867,'NASA 18.1020UE','Range check','','','Test',1317,'',130 from dual union all
select 48082,'1979-S73','1979-02-26 16:50:45','suborbital rocket','success',226,'AMF-VA-51','','','','Eclipse',1318,'',132 from dual union all
select 48083,'1979-S74','1979-02-26 16:53:30','suborbital rocket','success',1319,'NASA 33.04UE','','','','Aeronomy',1317,'',200 from dual union all
select 48084,'1979-S75','1979-02-26 16:53:45','suborbital rocket','success',1319,'NASA 33.03UA','','','','Aeronomy',1317,'',200 from dual union all
select 48085,'1979-S76','1979-02-26 23:48:00','suborbital rocket','success',848,'GL A07.712-2','Eclipse DEN Sphere','','','Eclipse/Aeron',1317,'',215 from dual union all
select 48086,'1979-S77','1979-02-26 16:28:00','suborbital rocket','success',860,'GL A12.9A1','Eclipse ASL-SE-79A1','','','Eclipse Aeron',1317,'',158 from dual union all
select 48087,'1979-S78','1979-02-26 16:28:30','suborbital rocket','success',860,'GL A12.9B1','Eclipse ASL-SE-79B1','','','Eclipse Aeron',1317,'',158 from dual union all
select 48088,'1979-S79','1979-02-26 16:50:50','suborbital rocket','success',132,'NASA 23.09UE','','','','Plasma/Eclipse',1319,'',85 from dual union all
select 48089,'1979-S80','1979-02-26 16:51:55','suborbital rocket','success',860,'GL A12.9A2','Eclipse LWIR','','','Eclipse/Aeron',1317,'',141 from dual union all
select 4809,'1958-A10','1958-02-25 17:22:00','military missile','failure',832,'12','G-26 No. 8','','','Test',281,'',2 from dual union all
select 48090,'1979-S81','1979-02-26 16:52:00','suborbital rocket','success',867,'NASA 18.1021UE','Eclipse','','','Eclipse/Aeron',1317,'',132 from dual union all
select 48091,'1979-S82','1979-02-26 16:52:30','suborbital rocket','success',906,'GL A10.802-1','','','','Eclipse/Aeron',1317,'',115 from dual union all
select 48092,'1979-S83','1979-02-26 16:54:10','suborbital rocket','success',867,'NASA 18.1022UE','Eclipse','','','Eclipse/Aeron',1317,'',132 from dual union all
select 48093,'1979-S84','1979-02-26 17:41:00','suborbital rocket','success',906,'GL A10.802-2','','','','Eclipse/Aeron',1317,'',116 from dual union all
select 48094,'1979-S85','1979-02-27 12:00:00','suborbital rocket','success',132,'NASA 23.10UE','','','','Plasma/Eclipse',1319,'',85 from dual union all
select 48095,'1979-S86','1979-03-01 00:00:00','suborbital rocket','success',1211,'SL1299','CHEVALINE S10','','','MIRV test',1839,'',270 from dual union all
select 48096,'1979-S87','1979-03-01 09:44:00','suborbital rocket','success',228,'NASA 21.60DS','HRTS 3','','','Solar',1873,'',210 from dual union all
select 48097,'1979-S88','1979-03-05 18:47:00','military missile','success',939,'C3','OT-37','','','OT',421,'SSBN 642',500 from dual union all
select 48098,'1979-S89','1979-03-05 18:48:00','military missile','success',939,'C3','OT-37','','','OT',421,'SSBN 642',500 from dual union all
select 48099,'1979-S90','1979-03-05 20:07:00','military missile','success',939,'C3','OT-37','','','OT',421,'SSBN 642',500 from dual union all
select 481,'1944-S98','1944-09-14 10:46:00','military missile','success',8,'18745','Ma160','','','Training',661,'',90 from dual union all
select 4810,'1958-A100','1958-08-25 00:00:00','suborbital rocket','success',365,'','Pogo Target','','','Test',165,'AVM1',30 from dual union all
select 48100,'1979-S91','1979-03-05 22:25:00','military missile','success',939,'C3','OT-37','','','OT',421,'SSBN 642',500 from dual union all
select 48101,'1979-S92','1979-03-06 15:27:00','suborbital rocket','unknown',740,'MR-12','','','','Aeron/Ionos/Plasma',1010,'',160 from dual union all
select 48102,'1979-S93','1979-03-10 18:43:00','suborbital rocket','unknown',740,'MR-12','','','','Plasma/Fields',1010,'',160 from dual union all
select 48103,'1979-S94','1979-03-13 00:30:00','suborbital rocket','success',1213,'DLR W-GR-95','Astro 4-1','','','Solar X',1839,'',258 from dual union all
select 48104,'1979-S95','1979-03-13 21:52:00','suborbital rocket','success',740,'MR-12','','','','Aeron/Ionos/Plasma',1010,'',188 from dual union all
select 48105,'1979-S96','1979-03-15 05:20:00','suborbital rocket','success',133,'NASA 25.045UH','Wisc. Aerobee IV','','','XR Astron',1895,'',216 from dual union all
select 48106,'1979-S97','1979-03-17 17:42:00','suborbital rocket','success',740,'MR-12','','','','Aeron/Ionos/Plasma',1010,'',187 from dual union all
select 48107,'1979-S98','1979-03-19 22:57:00','suborbital rocket','success',117,'NASA 24.05IE','DLR K-AR-92','Porcupine 3','','Magnetospheric',407,'',460 from dual union all
select 48108,'1979-S99','1979-03-20 00:00:00','military missile','success',1106,'V8','','','','Test',301,'',1000 from dual union all
select 48109,'1979-U01','1979-10-12 12:30:00','orbital','failure',1238,'77025  -419','','Zenit-6','','Sat',474,'',200 from dual union all
select 4811,'1958-A101','1958-08-25 00:00:00','suborbital rocket','success',365,'','Pogo Target','','','Test',165,'AVM1',30 from dual union all
select 48110,'1979-W01','1979-01-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 48111,'1979-W02','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48112,'1979-W03','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48113,'1979-W04','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48114,'1979-W05','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48115,'1979-W06','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48116,'1979-W07','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48117,'1979-W08','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48118,'1979-W09','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48119,'1979-W10','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 4812,'1958-A102','1958-08-25 00:00:00','suborbital rocket','success',365,'','Pogo Target','','','Test',165,'AVM1',30 from dual union all
select 48120,'1979-W100','1979-01-17 21:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',73 from dual union all
select 48121,'1979-W1000','1979-08-15 13:55:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',58 from dual union all
select 48122,'1979-W1001','1979-08-15 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48123,'1979-W1002','1979-08-15 18:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 48124,'1979-W1003','1979-08-15 18:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',59 from dual union all
select 48125,'1979-W1004','1979-08-16 00:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48126,'1979-W1005','1979-08-16 21:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 48127,'1979-W1006','1979-08-17 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 48128,'1979-W1007','1979-08-17 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 48129,'1979-W1008','1979-08-17 15:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 4813,'1958-A103','1958-08-25 00:00:00','military missile','success',1214,'N-69E N-3411','','','','Test',238,'',10 from dual union all
select 48130,'1979-W1009','1979-08-17 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 48131,'1979-W101','1979-01-18 01:42:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48132,'1979-W1010','1979-08-17 18:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 48133,'1979-W1011','1979-08-18 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 48134,'1979-W1012','1979-08-19 04:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48135,'1979-W1013','1979-08-20 13:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 48136,'1979-W1014','1979-08-20 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',51 from dual union all
select 48137,'1979-W1015','1979-08-20 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',67 from dual union all
select 48138,'1979-W1016','1979-08-20 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',59 from dual union all
select 48139,'1979-W1017','1979-08-21 04:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 4814,'1958-A104','1958-08-26 00:00:00','suborbital rocket','success',365,'','Pogo Target','','','Test',165,'AVM1',30 from dual union all
select 48140,'1979-W1018','1979-08-22 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 48141,'1979-W1019','1979-08-22 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 48142,'1979-W102','1979-01-18 17:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',58 from dual union all
select 48143,'1979-W1020','1979-08-22 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',62 from dual union all
select 48144,'1979-W1021','1979-08-22 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 48145,'1979-W1022','1979-08-22 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',51 from dual union all
select 48146,'1979-W1023','1979-08-22 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 48147,'1979-W1024','1979-08-22 23:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 48148,'1979-W1025','1979-08-23 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 48149,'1979-W1026','1979-08-24 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 4815,'1958-A105','1958-08-28 00:00:00','military missile','success',262,'13','','','','Test',243,'',10 from dual union all
select 48150,'1979-W1027','1979-08-24 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 48151,'1979-W1028','1979-08-24 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 48152,'1979-W1029','1979-08-24 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 48153,'1979-W103','1979-01-18 19:37:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',65 from dual union all
select 48154,'1979-W1030','1979-08-24 17:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48155,'1979-W1031','1979-08-25 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 48156,'1979-W1032','1979-08-27 01:30:00','suborbital rocket','success',761,'MT135P T25','','','','Aeron?',1476,'',50 from dual union all
select 48157,'1979-W1033','1979-08-27 06:00:00','suborbital rocket','success',761,'MT135P T26','','','','Aeron?',1476,'',50 from dual union all
select 48158,'1979-W1034','1979-08-27 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual;
insert into LAUNCH
select 48159,'1979-W1035','1979-08-27 13:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 4816,'1958-A106','1958-08-30 00:00:00','military missile','success',1214,'N-69E N-3412','','','','Test',224,'',10 from dual union all
select 48160,'1979-W1036','1979-08-27 16:10:00','suborbital rocket','unknown',1284,'FLIGHT 244     /T 1-','','','','Aeron',1821,'',66 from dual union all
select 48161,'1979-W1037','1979-08-27 17:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48162,'1979-W1038','1979-08-27 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 48163,'1979-W1039','1979-08-28 04:07:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 48164,'1979-W104','1979-01-19 06:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48165,'1979-W1040','1979-08-28 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 48166,'1979-W1041','1979-08-28 20:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 48167,'1979-W1042','1979-08-29 02:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',67 from dual union all
select 48168,'1979-W1043','1979-08-29 03:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',75 from dual union all
select 48169,'1979-W1044','1979-08-29 03:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 4817,'1958-A107','1958-09-03 00:00:00','military missile','success',255,'624-XY3','','','','OT',278,'',20 from dual union all
select 48170,'1979-W1045','1979-08-29 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48171,'1979-W1046','1979-08-29 14:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',61 from dual union all
select 48172,'1979-W1047','1979-08-29 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 48173,'1979-W1048','1979-08-29 22:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 48174,'1979-W1049','1979-08-30 03:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48175,'1979-W105','1979-01-19 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',77 from dual union all
select 48176,'1979-W1050','1979-08-30 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',75 from dual union all
select 48177,'1979-W1051','1979-08-30 15:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 48178,'1979-W1052','1979-08-31 00:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 48179,'1979-W1053','1979-08-31 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 4818,'1958-A108','1958-09-05 00:00:00','suborbital rocket','failure',575,'1','High Virgo','','','Interceptor',21,'B-58',13 from dual union all
select 48180,'1979-W1054','1979-08-31 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 48181,'1979-W1055','1979-08-31 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 48182,'1979-W1056','1979-08-31 17:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',60 from dual union all
select 48183,'1979-W1057','1979-08-31 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 48184,'1979-W1058','1979-08-31 21:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 48185,'1979-W1059','1979-09-01 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 48186,'1979-W106','1979-01-19 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 48187,'1979-W1060','1979-09-01 06:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',71 from dual union all
select 48188,'1979-W1061','1979-09-03 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 48189,'1979-W1062','1979-09-04 20:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',51 from dual union all
select 4819,'1958-A109','1958-09-07 23:30:00','suborbital rocket','failure',716,'II9.04F','','IGY II9.04F','','CR/Plasma',428,'',19 from dual union all
select 48190,'1979-W1063','1979-09-05 12:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48191,'1979-W1064','1979-09-05 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',68 from dual union all
select 48192,'1979-W1065','1979-09-05 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 48193,'1979-W1066','1979-09-05 14:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 48194,'1979-W1067','1979-09-05 14:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',58 from dual union all
select 48195,'1979-W1068','1979-09-05 16:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48196,'1979-W1069','1979-09-05 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 48197,'1979-W107','1979-01-19 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 48198,'1979-W1070','1979-09-05 23:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 48199,'1979-W1071','1979-09-07 00:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 482,'1944-S99','1944-09-14 14:12:00','military missile','success',8,'18755','Ma174','','','Training',661,'',90 from dual union all
select 4820,'1958-A11','1958-02-26 22:17:40','suborbital rocket','failure',839,'RP6.X2','Long nose cone test','IGY RP6.X2','','Test',428,'',37 from dual union all
select 48200,'1979-W1072','1979-09-07 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48201,'1979-W1073','1979-09-07 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 48202,'1979-W1074','1979-09-07 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 48203,'1979-W1075','1979-09-07 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 48204,'1979-W1076','1979-09-07 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 48205,'1979-W1077','1979-09-07 20:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 48206,'1979-W1078','1979-09-08 02:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48207,'1979-W1079','1979-09-10 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48208,'1979-W108','1979-01-19 15:33:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',67 from dual union all
select 48209,'1979-W1080','1979-09-10 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 4821,'1958-A110','1958-09-08 00:00:00','atmospheric rocket','success',442,'','E15 Model','','','Test',1821,'',10 from dual union all
select 48210,'1979-W1081','1979-09-10 14:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48211,'1979-W1082','1979-09-10 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 48212,'1979-W1083','1979-09-10 16:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48213,'1979-W1084','1979-09-10 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 48214,'1979-W1085','1979-09-11 02:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48215,'1979-W1086','1979-09-12 02:03:00','suborbital rocket','success',761,'MT135P-R360','','','','Meteo',1332,'',59 from dual union all
select 48216,'1979-W1087','1979-09-12 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 48217,'1979-W1088','1979-09-12 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 48218,'1979-W1089','1979-09-12 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 48219,'1979-W109','1979-01-19 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 4822,'1958-A111','1958-09-08 00:00:00','atmospheric rocket','failure',835,'','E15 Model/Loki','','','Test',1821,'',10 from dual union all
select 48220,'1979-W1090','1979-09-12 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 48221,'1979-W1091','1979-09-12 14:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 48222,'1979-W1092','1979-09-12 18:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 48223,'1979-W1093','1979-09-12 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 48224,'1979-W1094','1979-09-12 20:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 48225,'1979-W1095','1979-09-12 21:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 48226,'1979-W1096','1979-09-12 22:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 48227,'1979-W1097','1979-09-13 08:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 48228,'1979-W1098','1979-09-13 09:14:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',63 from dual union all
select 48229,'1979-W1099','1979-09-13 10:17:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',62 from dual union all
select 4823,'1958-A112','1958-09-10 00:00:00','military missile','success',268,'1-01','','','','Test',1752,'',15 from dual union all
select 48230,'1979-W11','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48231,'1979-W110','1979-01-19 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 48232,'1979-W1100','1979-09-13 12:00:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',59 from dual union all
select 48233,'1979-W1101','1979-09-13 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 48234,'1979-W1102','1979-09-13 16:14:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',62 from dual union all
select 48235,'1979-W1103','1979-09-13 16:15:00','suborbital rocket','unknown',1284,'FLIGHT 245     /T 1-','','','','Aeron',1821,'',66 from dual union all
select 48236,'1979-W1104','1979-09-14 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 48237,'1979-W1105','1979-09-14 13:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 48238,'1979-W1106','1979-09-14 14:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 48239,'1979-W1107','1979-09-14 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 4824,'1958-A113','1958-09-11 00:00:00','military missile','success',832,'14','G-26 No. 15/RISE 1','','','Test',281,'',25 from dual union all
select 48240,'1979-W1108','1979-09-14 19:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48241,'1979-W1109','1979-09-15 00:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 48242,'1979-W111','1979-01-19 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',51 from dual union all
select 48243,'1979-W1110','1979-09-15 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 48244,'1979-W1111','1979-09-17 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48245,'1979-W1112','1979-09-17 13:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',54 from dual union all
select 48246,'1979-W1113','1979-09-17 14:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48247,'1979-W1114','1979-09-17 18:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 48248,'1979-W1115','1979-09-18 02:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',63 from dual;
insert into LAUNCH
select 48249,'1979-W1116','1979-09-18 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 4825,'1958-A114','1958-09-12 00:00:00','military missile','success',262,'11','','','','Test',240,'',10 from dual union all
select 48250,'1979-W1117','1979-09-18 17:55:00','suborbital rocket','unknown',1284,'FLIGHT 246     /TH1-','','','','Aeron',1307,'',66 from dual union all
select 48251,'1979-W1118','1979-09-18 18:30:00','suborbital rocket','unknown',1284,'FLIGHT 247     /TH1-','','','','Aeron',1307,'',57 from dual union all
select 48252,'1979-W1119','1979-09-19 12:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 48253,'1979-W112','1979-01-19 21:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 48254,'1979-W1120','1979-09-19 14:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 48255,'1979-W1121','1979-09-19 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48256,'1979-W1122','1979-09-19 15:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',53 from dual union all
select 48257,'1979-W1123','1979-09-19 18:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 48258,'1979-W1124','1979-09-19 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',58 from dual union all
select 48259,'1979-W1125','1979-09-19 22:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 4826,'1958-A115','1958-09-12 00:00:00','atmospheric rocket','success',1052,'','','','','Test',1821,'B-57A',12 from dual union all
select 48260,'1979-W1126','1979-09-19 22:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',74 from dual union all
select 48261,'1979-W1127','1979-09-20 17:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',54 from dual union all
select 48262,'1979-W1128','1979-09-21 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 48263,'1979-W1129','1979-09-21 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 48264,'1979-W113','1979-01-20 03:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 48265,'1979-W1130','1979-09-21 14:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 48266,'1979-W1131','1979-09-21 15:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 48267,'1979-W1132','1979-09-21 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 48268,'1979-W1133','1979-09-22 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 48269,'1979-W1134','1979-09-22 05:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 4827,'1958-A116','1958-09-12 01:31:00','suborbital rocket','success',662,'K-6-3','K-6-3','','','Test',7,'',4 from dual union all
select 48270,'1979-W1135','1979-09-22 09:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 48271,'1979-W1136','1979-09-24 08:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 48272,'1979-W1137','1979-09-24 12:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 48273,'1979-W1138','1979-09-24 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 48274,'1979-W1139','1979-09-24 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 48275,'1979-W114','1979-01-20 05:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 48276,'1979-W1140','1979-09-24 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48277,'1979-W1141','1979-09-24 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 48278,'1979-W1142','1979-09-25 03:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 48279,'1979-W1143','1979-09-26 02:00:00','suborbital rocket','success',761,'MT135P-R361','','','','Meteo',1332,'',57 from dual union all
select 4828,'1958-A117','1958-09-14 02:40:00','suborbital rocket','success',662,'K-6-4','K-6-4','','','Test',7,'',40 from dual union all
select 48280,'1979-W1144','1979-09-26 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 48281,'1979-W1145','1979-09-26 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 48282,'1979-W1146','1979-09-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',79 from dual union all
select 48283,'1979-W1147','1979-09-26 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',59 from dual union all
select 48284,'1979-W1148','1979-09-26 14:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 48285,'1979-W1149','1979-09-26 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 48286,'1979-W115','1979-01-20 06:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 48287,'1979-W1150','1979-09-26 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 48288,'1979-W1151','1979-09-26 18:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 48289,'1979-W1152','1979-09-27 02:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 4829,'1958-A118','1958-09-18 00:00:00','military missile','success',262,'Dummy 4','','','','Test',243,'',10 from dual union all
select 48290,'1979-W1153','1979-09-28 00:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 48291,'1979-W1154','1979-09-28 03:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 48292,'1979-W1155','1979-09-28 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48293,'1979-W1156','1979-09-28 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 48294,'1979-W1157','1979-09-28 14:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48295,'1979-W1158','1979-09-28 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 48296,'1979-W1159','1979-09-28 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 48297,'1979-W116','1979-01-20 07:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 48298,'1979-W1160','1979-09-28 22:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 48299,'1979-W1161','1979-09-29 03:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 483,'1945-A01','1945-01-01 14:50:00','military missile','failure',1470,'20852','Ma426','','','Test',661,'',20 from dual union all
select 4830,'1958-A119','1958-09-18 00:00:00','atmospheric rocket','success',839,'','Polaris RV model','','','Reentry Test',1821,'',9 from dual union all
select 48300,'1979-W1162','1979-10-01 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 48301,'1979-W1163','1979-10-01 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',66 from dual union all
select 48302,'1979-W1164','1979-10-01 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 48303,'1979-W1165','1979-10-01 14:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48304,'1979-W1166','1979-10-01 17:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 48305,'1979-W1167','1979-10-01 17:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',57 from dual union all
select 48306,'1979-W1168','1979-10-01 20:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 48307,'1979-W1169','1979-10-01 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 48308,'1979-W117','1979-01-20 09:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 48309,'1979-W1170','1979-10-02 01:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 4831,'1958-A12','1958-02-28 00:00:00','atmospheric rocket','success',549,'','GE RV Model','','','Reentry Test',1821,'',30 from dual union all
select 48310,'1979-W1171','1979-10-02 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 48311,'1979-W1172','1979-10-03 02:00:00','suborbital rocket','success',761,'MT135P-R362','','','','Meteo',1332,'',59 from dual union all
select 48312,'1979-W1173','1979-10-03 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 48313,'1979-W1174','1979-10-03 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 48314,'1979-W1175','1979-10-03 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',57 from dual union all
select 48315,'1979-W1176','1979-10-03 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 48316,'1979-W1177','1979-10-03 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 48317,'1979-W1178','1979-10-03 16:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 48318,'1979-W1179','1979-10-03 17:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',55 from dual union all
select 48319,'1979-W118','1979-01-22 14:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 4832,'1958-A120','1958-09-19 00:00:00','military missile','success',1214,'N-69E N-3414','','','','Test',238,'',10 from dual union all
select 48320,'1979-W1180','1979-10-03 18:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',58 from dual union all
select 48321,'1979-W1181','1979-10-04 01:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 48322,'1979-W1182','1979-10-04 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 48323,'1979-W1183','1979-10-04 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 48324,'1979-W1184','1979-10-05 04:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48325,'1979-W1185','1979-10-05 12:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48326,'1979-W1186','1979-10-05 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 48327,'1979-W1187','1979-10-05 15:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 48328,'1979-W1188','1979-10-05 21:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 48329,'1979-W1189','1979-10-06 04:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 4833,'1958-A121','1958-09-22 00:00:00','suborbital rocket','success',716,'','Oriole','','','Aeron/Test',1821,'',30 from dual union all
select 48330,'1979-W119','1979-01-22 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',75 from dual union all
select 48331,'1979-W1190','1979-10-08 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 48332,'1979-W1191','1979-10-08 14:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48333,'1979-W1192','1979-10-08 14:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 48334,'1979-W1193','1979-10-08 16:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 48335,'1979-W1194','1979-10-09 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 48336,'1979-W1195','1979-10-09 20:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 48337,'1979-W1196','1979-10-10 00:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48338,'1979-W1197','1979-10-10 10:10:00','suborbital rocket','unknown',1284,'FLIGHT 248     /TN1-','','','','Aeron',330,'',66 from dual;
insert into LAUNCH
select 48339,'1979-W1198','1979-10-10 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 4834,'1958-A122','1958-09-22 00:00:00','suborbital rocket','success',716,'','Oriole','','','Aeron/Test',1821,'',30 from dual union all
select 48340,'1979-W1199','1979-10-10 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 48341,'1979-W12','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48342,'1979-W120','1979-01-22 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 48343,'1979-W1200','1979-10-10 13:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 48344,'1979-W1201','1979-10-10 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 48345,'1979-W1202','1979-10-10 15:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 48346,'1979-W1203','1979-10-10 21:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 48347,'1979-W1204','1979-10-10 22:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 48348,'1979-W1205','1979-10-12 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 48349,'1979-W1206','1979-10-12 16:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 4835,'1958-A123','1958-09-22 00:00:00','suborbital rocket','success',716,'','Oriole','','','Aeron/Test',1821,'',30 from dual union all
select 48350,'1979-W1207','1979-10-12 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 48351,'1979-W1208','1979-10-12 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 48352,'1979-W1209','1979-10-13 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 48353,'1979-W121','1979-01-22 16:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 48354,'1979-W1210','1979-10-15 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48355,'1979-W1211','1979-10-15 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 48356,'1979-W1212','1979-10-15 14:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',51 from dual union all
select 48357,'1979-W1213','1979-10-15 16:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 48358,'1979-W1214','1979-10-15 16:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 48359,'1979-W1215','1979-10-15 20:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 4836,'1958-A124','1958-09-24 00:00:00','military missile','success',255,'624-XY4','','','','OT',272,'',20 from dual union all
select 48360,'1979-W1216','1979-10-16 04:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',58 from dual union all
select 48361,'1979-W1217','1979-10-17 02:00:00','suborbital rocket','success',761,'MT135P-R363','','','','Meteo',1332,'',57 from dual union all
select 48362,'1979-W1218','1979-10-17 03:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 48363,'1979-W1219','1979-10-17 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48364,'1979-W122','1979-01-22 17:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 48365,'1979-W1220','1979-10-17 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 48366,'1979-W1221','1979-10-17 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',61 from dual union all
select 48367,'1979-W1222','1979-10-17 14:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 48368,'1979-W1223','1979-10-17 15:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 48369,'1979-W1224','1979-10-17 22:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 4837,'1958-A125','1958-09-24 17:18:00','military missile','failure',933,'AX-1','','','','Test',244,'',12 from dual union all
select 48370,'1979-W1225','1979-10-19 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 48371,'1979-W1226','1979-10-19 14:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 48372,'1979-W1227','1979-10-19 14:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',50 from dual union all
select 48373,'1979-W1228','1979-10-19 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 48374,'1979-W1229','1979-10-19 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 48375,'1979-W123','1979-01-23 01:24:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',53 from dual union all
select 48376,'1979-W1230','1979-10-19 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 48377,'1979-W1231','1979-10-19 19:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 48378,'1979-W1232','1979-10-19 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',51 from dual union all
select 48379,'1979-W1233','1979-10-20 00:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 4838,'1958-A126','1958-09-25 00:00:00','military missile','success',511,'Dummy 5','','','','Test',240,'',10 from dual union all
select 48380,'1979-W1234','1979-10-20 01:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48381,'1979-W1235','1979-10-21 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',52 from dual union all
select 48382,'1979-W1236','1979-10-21 15:37:00','suborbital rocket','success',1281,'NASA 15.203GM','','','','Aeronomy',1821,'',74 from dual union all
select 48383,'1979-W1237','1979-10-21 15:48:00','suborbital rocket','unknown',1284,'FLIGHT 249     /T 1-','','','','Aeron',1821,'',66 from dual union all
select 48384,'1979-W1238','1979-10-21 16:35:00','suborbital rocket','unknown',1284,'FLIGHT 250     /T 1-','','','','Aeron',1821,'',64 from dual union all
select 48385,'1979-W1239','1979-10-21 17:15:00','suborbital rocket','success',1281,'NASA 15.205GM','','','','Aeronomy',1821,'',75 from dual union all
select 48386,'1979-W124','1979-01-23 16:49:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',75 from dual union all
select 48387,'1979-W1240','1979-10-21 17:32:00','suborbital rocket','unknown',1284,'FLIGHT 251     /T 1-','','','','Aeron',1821,'',67 from dual union all
select 48388,'1979-W1241','1979-10-22 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48389,'1979-W1242','1979-10-22 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 4839,'1958-A127','1958-09-25 00:00:00','suborbital rocket','success',716,'','Oriole','','','Aeron/Test',1821,'',30 from dual union all
select 48390,'1979-W1243','1979-10-22 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48391,'1979-W1244','1979-10-22 15:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 48392,'1979-W1245','1979-10-22 17:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 48393,'1979-W1246','1979-10-22 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 48394,'1979-W1247','1979-10-23 00:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 48395,'1979-W1248','1979-10-23 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 48396,'1979-W1249','1979-10-24 02:00:00','suborbital rocket','success',761,'MT135P-R364','','','','Meteo',1332,'',58 from dual union all
select 48397,'1979-W125','1979-01-23 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',69 from dual union all
select 48398,'1979-W1250','1979-10-24 04:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 48399,'1979-W1251','1979-10-24 12:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 484,'1945-A02','1945-01-04 11:43:00','military missile','failure',1470,'20814','Ma410','','','Test',661,'',30 from dual union all
select 4840,'1958-A128','1958-09-25 00:00:00','suborbital rocket','success',716,'','Oriole','','','Aeron/Test',1821,'',30 from dual union all
select 48400,'1979-W1252','1979-10-24 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48401,'1979-W1253','1979-10-24 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 48402,'1979-W1254','1979-10-24 14:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48403,'1979-W1255','1979-10-24 18:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 48404,'1979-W1256','1979-10-24 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 48405,'1979-W1257','1979-10-25 04:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48406,'1979-W1258','1979-10-25 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',52 from dual union all
select 48407,'1979-W1259','1979-10-26 04:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48408,'1979-W126','1979-01-23 18:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48409,'1979-W1260','1979-10-26 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 4841,'1958-A129','1958-09-25 02:55:00','suborbital rocket','success',662,'K-6-TW-3','K-6-7','S3','','Aeron',7,'',44 from dual union all
select 48410,'1979-W1261','1979-10-26 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',69 from dual union all
select 48411,'1979-W1262','1979-10-26 14:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',71 from dual union all
select 48412,'1979-W1263','1979-10-26 14:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48413,'1979-W1264','1979-10-26 18:22:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',72 from dual union all
select 48414,'1979-W1265','1979-10-26 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 48415,'1979-W1266','1979-10-26 18:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48416,'1979-W1267','1979-10-26 21:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 48417,'1979-W1268','1979-10-29 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 48418,'1979-W1269','1979-10-29 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',76 from dual union all
select 48419,'1979-W127','1979-01-23 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 4842,'1958-A13','1958-02-28 13:08:00','military missile','failure',1366,'120','R&D Series II','','','Test',234,'',0 from dual union all
select 48420,'1979-W1270','1979-10-29 15:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 48421,'1979-W1271','1979-10-29 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 48422,'1979-W1272','1979-10-29 16:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 48423,'1979-W1273','1979-10-30 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 48424,'1979-W1274','1979-10-31 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',68 from dual union all
select 48425,'1979-W1275','1979-10-31 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 48426,'1979-W1276','1979-10-31 15:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 48427,'1979-W1277','1979-10-31 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 48428,'1979-W1278','1979-10-31 19:07:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual;
insert into LAUNCH
select 48429,'1979-W1279','1979-10-31 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 4843,'1958-A130','1958-09-30 00:00:00','military missile','success',511,'14','','','','Test',243,'',10 from dual union all
select 48430,'1979-W128','1979-01-23 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',74 from dual union all
select 48431,'1979-W1280','1979-10-31 23:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 48432,'1979-W1281','1979-11-01 12:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 48433,'1979-W1282','1979-11-01 13:15:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',59 from dual union all
select 48434,'1979-W1283','1979-11-01 13:16:00','suborbital rocket','unknown',1284,'FLIGHT 252     /T 1-','','','','Aeron',1821,'',64 from dual union all
select 48435,'1979-W1284','1979-11-01 13:55:00','suborbital rocket','unknown',1284,'FLIGHT 253     /T 1-','','','','Aeron',1821,'',65 from dual union all
select 48436,'1979-W1285','1979-11-01 15:05:00','suborbital rocket','unknown',1284,'FLIGHT 254     /T 1-','','','','Aeron',1821,'',66 from dual union all
select 48437,'1979-W1286','1979-11-01 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 48438,'1979-W1287','1979-11-01 17:51:00','suborbital rocket','unknown',1284,'FLIGHT 255     /TH1-','','','','Aeron',1307,'',57 from dual union all
select 48439,'1979-W1288','1979-11-01 19:42:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 4844,'1958-A131','1958-10-01 10:45:00','suborbital rocket','success',726,'','','','','Aeron',636,'',24 from dual union all
select 48440,'1979-W1289','1979-11-01 20:30:00','suborbital rocket','unknown',1284,'FLIGHT 256     /T 1-','','','','Aeron',1821,'',69 from dual union all
select 48441,'1979-W129','1979-01-24 05:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 48442,'1979-W1290','1979-11-01 20:49:00','suborbital rocket','unknown',1284,'FLIGHT 257     /T 1-','','','','Aeron',1821,'',64 from dual union all
select 48443,'1979-W1291','1979-11-01 21:40:00','suborbital rocket','unknown',738,'','','','','Aeron',1637,'USHAKOV',55 from dual union all
select 48444,'1979-W1292','1979-11-01 22:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',61 from dual union all
select 48445,'1979-W1293','1979-11-02 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 48446,'1979-W1294','1979-11-02 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',66 from dual union all
select 48447,'1979-W1295','1979-11-02 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',71 from dual union all
select 48448,'1979-W1296','1979-11-02 14:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 48449,'1979-W1297','1979-11-02 16:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 4845,'1958-A132','1958-10-06 00:00:00','atmospheric rocket','success',551,'','Polaris RTV','','','Reentry Test',1821,'',26 from dual union all
select 48450,'1979-W1298','1979-11-03 02:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 48451,'1979-W1299','1979-11-04 07:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 48452,'1979-W13','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48453,'1979-W130','1979-01-24 05:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48454,'1979-W1300','1979-11-04 08:50:00','suborbital rocket','success',1281,'NASA 15.209GM','','','','Aeronomy',1821,'',78 from dual union all
select 48455,'1979-W1301','1979-11-04 10:40:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',79 from dual union all
select 48456,'1979-W1302','1979-11-04 10:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 48457,'1979-W1303','1979-11-05 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 48458,'1979-W1304','1979-11-05 14:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 48459,'1979-W1305','1979-11-05 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 4846,'1958-A133','1958-10-10 03:49:00','military missile','failure',638,'AM-9','R&D','','','Test',249,'',0 from dual union all
select 48460,'1979-W1306','1979-11-05 16:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',71 from dual union all
select 48461,'1979-W1307','1979-11-05 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 48462,'1979-W1308','1979-11-05 20:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 48463,'1979-W1309','1979-11-06 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 48464,'1979-W131','1979-01-24 09:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 48465,'1979-W1310','1979-11-07 02:14:00','suborbital rocket','success',761,'MT135P-R365','','','','Meteo',1332,'',59 from dual union all
select 48466,'1979-W1311','1979-11-07 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',906,'KRENKEL''',57 from dual union all
select 48467,'1979-W1312','1979-11-07 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1637,'USHAKOV',59 from dual union all
select 48468,'1979-W1313','1979-11-07 12:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48469,'1979-W1314','1979-11-07 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 4847,'1958-A134','1958-10-13 00:00:00','atmospheric rocket','success',549,'','E41 Model','','','Test',1821,'',10 from dual union all
select 48470,'1979-W1315','1979-11-07 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 48471,'1979-W1316','1979-11-07 15:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 48472,'1979-W1317','1979-11-07 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 48473,'1979-W1318','1979-11-07 16:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 48474,'1979-W1319','1979-11-07 20:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',74 from dual union all
select 48475,'1979-W132','1979-01-24 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 48476,'1979-W1320','1979-11-07 23:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 48477,'1979-W1321','1979-11-09 00:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 48478,'1979-W1322','1979-11-09 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 48479,'1979-W1323','1979-11-09 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 4848,'1958-A135','1958-10-15 00:00:00','military missile','failure',933,'AX-2','','','','Test',244,'',1 from dual union all
select 48480,'1979-W1324','1979-11-09 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48481,'1979-W1325','1979-11-09 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',55 from dual union all
select 48482,'1979-W1326','1979-11-09 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 48483,'1979-W1327','1979-11-09 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 48484,'1979-W1328','1979-11-12 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 48485,'1979-W1329','1979-11-12 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',77 from dual union all
select 48486,'1979-W133','1979-01-24 13:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48487,'1979-W1330','1979-11-13 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 48488,'1979-W1331','1979-11-13 20:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 48489,'1979-W1332','1979-11-14 02:00:00','suborbital rocket','success',761,'MT135P-R366','','','','Meteo',1332,'',58 from dual union all
select 4849,'1958-A136','1958-10-16 00:00:00','military missile','success',1469,'','LKI','','','Test',182,'',15 from dual union all
select 48490,'1979-W1333','1979-11-14 03:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 48491,'1979-W1334','1979-11-14 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 48492,'1979-W1335','1979-11-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',79 from dual union all
select 48493,'1979-W1336','1979-11-14 14:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 48494,'1979-W1337','1979-11-14 16:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 48495,'1979-W1338','1979-11-14 17:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',54 from dual union all
select 48496,'1979-W1339','1979-11-14 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 48497,'1979-W134','1979-01-24 14:13:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 48498,'1979-W1340','1979-11-14 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',74 from dual union all
select 48499,'1979-W1341','1979-11-14 22:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 485,'1945-A04','1945-09-27 00:00:00','suborbital rocket','success',1515,'1','Dummy','','','Test',1852,'',2 from dual union all
select 4850,'1958-A137','1958-10-21 00:00:00','military missile','success',255,'624-XY5','','','','OT',278,'',20 from dual union all
select 48500,'1979-W1342','1979-11-15 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',79 from dual union all
select 48501,'1979-W1343','1979-11-16 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',69 from dual union all
select 48502,'1979-W1344','1979-11-16 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 48503,'1979-W1345','1979-11-16 15:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48504,'1979-W1346','1979-11-16 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',67 from dual union all
select 48505,'1979-W1347','1979-11-16 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 48506,'1979-W1348','1979-11-16 19:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48507,'1979-W1349','1979-11-17 00:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48508,'1979-W135','1979-01-24 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 48509,'1979-W1350','1979-11-19 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 4851,'1958-A138','1958-10-21 00:00:00','military missile','success',255,'624-XY6','','','','OT',272,'',20 from dual union all
select 48510,'1979-W1351','1979-11-19 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 48511,'1979-W1352','1979-11-19 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 48512,'1979-W1353','1979-11-21 03:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48513,'1979-W1354','1979-11-21 07:40:00','suborbital rocket','unknown',738,'','','','','Aeron',906,'KRENKEL''',56 from dual union all
select 48514,'1979-W1355','1979-11-21 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 48515,'1979-W1356','1979-11-21 14:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48516,'1979-W1357','1979-11-21 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 48517,'1979-W1358','1979-11-21 17:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',58 from dual union all
select 48518,'1979-W1359','1979-11-21 19:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual;
insert into LAUNCH
select 48519,'1979-W136','1979-01-24 16:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 4852,'1958-A139','1958-10-23 10:10:00','military missile','success',1214,'N-69D N-3318','','','','Test',223,'',10 from dual union all
select 48520,'1979-W1360','1979-11-21 19:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',55 from dual union all
select 48521,'1979-W1361','1979-11-21 21:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 48522,'1979-W1362','1979-11-22 03:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 48523,'1979-W1363','1979-11-23 16:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 48524,'1979-W1364','1979-11-24 00:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 48525,'1979-W1365','1979-11-26 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 48526,'1979-W1366','1979-11-26 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 48527,'1979-W1367','1979-11-26 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 48528,'1979-W1368','1979-11-26 14:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48529,'1979-W1369','1979-11-26 20:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 4853,'1958-A14','1958-03-08 00:00:00','military missile','success',1214,'N-69E N-3330','','','','Test',224,'',10 from dual union all
select 48530,'1979-W137','1979-01-24 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 48531,'1979-W1370','1979-11-28 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 48532,'1979-W1371','1979-11-28 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 48533,'1979-W1372','1979-11-28 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48534,'1979-W1373','1979-11-28 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 48535,'1979-W1374','1979-11-28 17:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',67 from dual union all
select 48536,'1979-W1375','1979-11-28 21:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',73 from dual union all
select 48537,'1979-W1376','1979-11-28 22:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48538,'1979-W1377','1979-11-29 16:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 48539,'1979-W1378','1979-11-30 04:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 4854,'1958-A140','1958-11-06 00:00:00','atmospheric rocket','success',542,'','Corvus Model','','','Test',1821,'',10 from dual union all
select 48540,'1979-W1379','1979-11-30 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',906,'KRENKEL''',57 from dual union all
select 48541,'1979-W138','1979-01-24 19:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48542,'1979-W1380','1979-11-30 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48543,'1979-W1381','1979-11-30 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 48544,'1979-W1382','1979-12-03 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 48545,'1979-W1383','1979-12-03 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 48546,'1979-W1384','1979-12-03 19:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 48547,'1979-W1385','1979-12-04 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 48548,'1979-W1386','1979-12-04 16:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',59 from dual union all
select 48549,'1979-W1387','1979-12-04 17:53:00','suborbital rocket','unknown',1284,'FLIGHT 258     /TH1-','','','','Aeron',1307,'',53 from dual union all
select 4855,'1958-A141','1958-11-08 09:25:00','military missile','success',1214,'N-69D N-3317','','','','Test',224,'',10 from dual union all
select 48550,'1979-W1388','1979-12-04 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 48551,'1979-W1389','1979-12-05 02:00:00','suborbital rocket','success',761,'MT135P-R369','','','','Meteo',1332,'',55 from dual union all
select 48552,'1979-W139','1979-01-24 23:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48553,'1979-W1390','1979-12-05 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 48554,'1979-W1391','1979-12-05 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 48555,'1979-W1392','1979-12-05 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 48556,'1979-W1393','1979-12-05 16:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 48557,'1979-W1394','1979-12-05 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 48558,'1979-W1395','1979-12-05 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 48559,'1979-W1396','1979-12-05 22:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 4856,'1958-A142','1958-11-12 00:00:00','military missile','success',771,'Training #1','','','','Test',222,'',12 from dual union all
select 48560,'1979-W1397','1979-12-07 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',63 from dual union all
select 48561,'1979-W1398','1979-12-07 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 48562,'1979-W1399','1979-12-07 17:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 48563,'1979-W14','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48564,'1979-W140','1979-01-25 15:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',70 from dual union all
select 48565,'1979-W1400','1979-12-10 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 48566,'1979-W1401','1979-12-10 18:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 48567,'1979-W1402','1979-12-10 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 48568,'1979-W1403','1979-12-12 00:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 48569,'1979-W1404','1979-12-12 02:00:00','suborbital rocket','success',761,'MT135P-R368','','','','Meteo',1332,'',57 from dual union all
select 4857,'1958-A143','1958-11-13 00:00:00','military missile','success',771,'Training #2','','','','Test',222,'',12 from dual union all
select 48570,'1979-W1405','1979-12-12 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',1637,'USHAKOV',60 from dual union all
select 48571,'1979-W1406','1979-12-12 14:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 48572,'1979-W1407','1979-12-12 14:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 48573,'1979-W1408','1979-12-12 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',76 from dual union all
select 48574,'1979-W1409','1979-12-12 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 48575,'1979-W141','1979-01-25 16:22:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',61 from dual union all
select 48576,'1979-W1410','1979-12-12 20:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 48577,'1979-W1411','1979-12-12 21:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 48578,'1979-W1412','1979-12-12 22:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48579,'1979-W1413','1979-12-13 20:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',74 from dual union all
select 4858,'1958-A144','1958-11-14 00:00:00','military missile','success',511,'15','','','','Test',243,'',10 from dual union all
select 48580,'1979-W1414','1979-12-14 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 48581,'1979-W1415','1979-12-14 17:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',71 from dual union all
select 48582,'1979-W1416','1979-12-14 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 48583,'1979-W1417','1979-12-14 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 48584,'1979-W1418','1979-12-17 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',59 from dual union all
select 48585,'1979-W1419','1979-12-17 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',62 from dual union all
select 48586,'1979-W142','1979-01-25 18:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 48587,'1979-W1420','1979-12-17 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 48588,'1979-W1421','1979-12-17 16:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',72 from dual union all
select 48589,'1979-W1422','1979-12-17 19:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 4859,'1958-A145','1958-11-18 21:03:00','military missile','failure',832,'04','G-26 No. 14/RISE 2','','','Test',281,'',23 from dual union all
select 48590,'1979-W1423','1979-12-18 02:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 48591,'1979-W1424','1979-12-18 05:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',66 from dual union all
select 48592,'1979-W1425','1979-12-18 17:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 48593,'1979-W1426','1979-12-18 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 48594,'1979-W1427','1979-12-19 02:00:00','suborbital rocket','success',761,'MT135P-R370','','','','Meteo',1332,'',57 from dual union all
select 48595,'1979-W1428','1979-12-19 03:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48596,'1979-W1429','1979-12-19 05:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 48597,'1979-W143','1979-01-25 21:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 48598,'1979-W1430','1979-12-19 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1637,'USHAKOV',58 from dual union all
select 48599,'1979-W1431','1979-12-19 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 486,'1945-A05','1945-09-28 00:00:00','suborbital rocket','success',1515,'2','Dummy','','','Test',1852,'',2 from dual union all
select 4860,'1958-A146','1958-11-20 00:00:00','atmospheric rocket','success',552,'','Telemetry test','','','Reentry Test',1821,'',30 from dual union all
select 48600,'1979-W1432','1979-12-19 14:23:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48601,'1979-W1433','1979-12-19 14:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 48602,'1979-W1434','1979-12-19 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 48603,'1979-W1435','1979-12-19 19:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 48604,'1979-W1436','1979-12-19 21:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',76 from dual union all
select 48605,'1979-W1437','1979-12-20 03:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48606,'1979-W1438','1979-12-20 05:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 48607,'1979-W1439','1979-12-20 09:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 48608,'1979-W144','1979-01-26 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',903,'KRENKEL''',57 from dual;
insert into LAUNCH
select 48609,'1979-W1440','1979-12-20 15:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',78 from dual union all
select 4861,'1958-A147','1958-11-20 00:00:00','suborbital rocket','failure',1163,'','Mercury-style RV','','','Reentry Test',1821,'',30 from dual union all
select 48610,'1979-W1441','1979-12-20 16:01:00','suborbital rocket','unknown',1284,'FLIGHT 259     /T 1-','','','','Aeron',1821,'',66 from dual union all
select 48611,'1979-W1442','1979-12-21 02:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48612,'1979-W1443','1979-12-21 12:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 48613,'1979-W1444','1979-12-21 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 48614,'1979-W1445','1979-12-21 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 48615,'1979-W1446','1979-12-21 18:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',68 from dual union all
select 48616,'1979-W1447','1979-12-24 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',1637,'USHAKOV',58 from dual union all
select 48617,'1979-W1448','1979-12-24 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',69 from dual union all
select 48618,'1979-W1449','1979-12-24 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1760,'VOLNA',60 from dual union all
select 48619,'1979-W145','1979-01-26 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 4862,'1958-A148','1958-11-20 03:27:00','suborbital rocket','failure',716,'II9.13F','','IGY II9.13F','','CR/Plasma',428,'',3 from dual union all
select 48620,'1979-W1450','1979-12-24 14:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 48621,'1979-W1451','1979-12-24 15:45:00','suborbital rocket','unknown',738,'','','','','Aeron',1760,'VOLNA',62 from dual union all
select 48622,'1979-W1452','1979-12-26 02:01:00','suborbital rocket','success',761,'MT135P-R371','','','','Meteo',1332,'',54 from dual union all
select 48623,'1979-W1453','1979-12-26 07:15:00','suborbital rocket','unknown',738,'','','','','Aeron',1637,'USHAKOV',59 from dual union all
select 48624,'1979-W1454','1979-12-26 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',50 from dual union all
select 48625,'1979-W1455','1979-12-26 15:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48626,'1979-W1456','1979-12-26 23:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 48627,'1979-W1457','1979-12-28 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48628,'1979-W1458','1979-12-28 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 48629,'1979-W1459','1979-12-28 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 4863,'1958-A149','1958-11-20 04:25:00','suborbital rocket','failure',716,'II9.14F','','IGY II9.14F','','CR/Plasma',428,'',10 from dual union all
select 48630,'1979-W146','1979-01-26 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48631,'1979-W1460','1979-12-28 16:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 48632,'1979-W1461','1979-12-29 03:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48633,'1979-W1462','1979-12-31 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48634,'1979-W1463','1979-12-31 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 48635,'1979-W1464','1979-12-31 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 48636,'1979-W1465','1979-01-10 02:00:00','suborbital rocket','success',761,'MT135P-R336','','','','Meteo',1332,'',63 from dual union all
select 48637,'1979-W1466','1979-01-17 02:10:00','suborbital rocket','success',761,'MT135P-R337','','','','Meteo',1332,'',64 from dual union all
select 48638,'1979-W1467','1979-01-24 02:04:00','suborbital rocket','success',761,'MT135P-R338','','','','Meteo',1332,'',62 from dual union all
select 48639,'1979-W1468','1979-01-31 02:07:00','suborbital rocket','success',761,'MT135P-R339','','','','Meteo',1332,'',63 from dual union all
select 4864,'1958-A15','1958-03-13 00:00:00','military missile','success',771,'GM-56-1963','','','','Test',222,'',12 from dual union all
select 48640,'1979-W1469','1979-02-07 02:00:00','suborbital rocket','success',761,'MT135P-R340','','','','Meteo',1332,'',60 from dual union all
select 48641,'1979-W147','1979-01-26 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 48642,'1979-W1470','1979-02-14 02:02:00','suborbital rocket','success',761,'MT135P-R341','','','','Meteo',1332,'',60 from dual union all
select 48643,'1979-W1471','1979-02-21 02:00:00','suborbital rocket','success',761,'MT135P-R342','','','','Meteo',1332,'',63 from dual union all
select 48644,'1979-W1472','1979-03-07 02:00:00','suborbital rocket','success',761,'MT135P-R343','','','','Meteo',1332,'',61 from dual union all
select 48645,'1979-W1473','1979-03-14 02:00:00','suborbital rocket','success',761,'MT135P-R344','','','','Meteo',1332,'',64 from dual union all
select 48646,'1979-W1474','1979-03-28 02:00:00','suborbital rocket','success',761,'MT135P-R345','','','','Meteo',1332,'',59 from dual union all
select 48647,'1979-W1475','1979-04-11 02:04:00','suborbital rocket','success',761,'MT135P-R346','','','','Meteo',1332,'',64 from dual union all
select 48648,'1979-W1476','1979-04-18 02:11:00','suborbital rocket','success',761,'MT135P-R347','','','','Meteo',1332,'',65 from dual union all
select 48649,'1979-W1477','1979-04-25 02:00:00','suborbital rocket','success',761,'MT135P-R348','','','','Meteo',1332,'',65 from dual union all
select 4865,'1958-A150','1958-11-21 15:40:00','military missile','success',255,'624-XY7','','','','OT',272,'',20 from dual union all
select 48650,'1979-W1478','1979-05-09 02:00:00','suborbital rocket','success',761,'MT135P-R349','','','','Meteo',1332,'',61 from dual union all
select 48651,'1979-W1479','1979-05-16 02:15:00','suborbital rocket','success',761,'MT135P-R350','','','','Meteo',1332,'',60 from dual union all
select 48652,'1979-W148','1979-01-26 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48653,'1979-W1480','1979-05-23 02:00:00','suborbital rocket','success',761,'MT135P-R351','','','','Meteo',1332,'',65 from dual union all
select 48654,'1979-W1481','1979-05-30 02:00:00','suborbital rocket','success',761,'MT135P-R352','','','','Meteo',1332,'',61 from dual union all
select 48655,'1979-W1482','1979-06-06 02:00:00','suborbital rocket','success',761,'MT135P-R353','','','','Meteo',1332,'',62 from dual union all
select 48656,'1979-W1483','1979-06-20 02:00:00','suborbital rocket','success',761,'MT135P-R354','','','','Meteo',1332,'',64 from dual union all
select 48657,'1979-W1484','1979-09-06 02:00:00','suborbital rocket','success',761,'MT135P-R359','','','','Meteo',1332,'',64 from dual union all
select 48658,'1979-W1485','1979-11-28 02:00:00','suborbital rocket','success',761,'MT135P-R367','','','','Meteo',1332,'',57 from dual union all
select 48659,'1979-W149','1979-01-26 17:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 4866,'1958-A151','1958-11-24 08:00:00','suborbital rocket','failure',839,'AA6.17','Atm p,rho, Sphere','IGY AA6.17','','Aeron',428,'',13 from dual union all
select 48660,'1979-W15','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48661,'1979-W150','1979-01-26 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 48662,'1979-W151','1979-01-26 21:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 48663,'1979-W152','1979-01-27 02:02:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 48664,'1979-W153','1979-01-27 20:05:00','suborbital rocket','success',1281,'NASA 15.150GM','','','','Aeronomy',1047,'',60 from dual union all
select 48665,'1979-W154','1979-01-28 08:33:00','suborbital rocket','success',1281,'NASA 15.151GM','','','','Aeronomy',1047,'',69 from dual union all
select 48666,'1979-W155','1979-01-29 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 48667,'1979-W156','1979-01-29 14:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 48668,'1979-W157','1979-01-29 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48669,'1979-W158','1979-01-29 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 4867,'1958-A152','1958-11-28 00:00:00','military missile','success',511,'18','','','','Test',243,'',10 from dual union all
select 48670,'1979-W159','1979-01-29 17:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 48671,'1979-W16','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48672,'1979-W160','1979-01-29 19:08:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 48673,'1979-W161','1979-01-29 21:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 48674,'1979-W162','1979-01-30 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 48675,'1979-W163','1979-01-30 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',54 from dual union all
select 48676,'1979-W164','1979-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 48677,'1979-W165','1979-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 48678,'1979-W166','1979-02-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 48679,'1979-W167','1979-01-31 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',903,'KRENKEL''',58 from dual union all
select 4868,'1958-A153','1958-11-28 03:05:00','suborbital rocket','failure',662,'K-6-CP-1','K-6-10','','','CR/Aeron',7,'',36 from dual union all
select 48680,'1979-W168','1979-01-31 13:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48681,'1979-W169','1979-01-31 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 48682,'1979-W17','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48683,'1979-W170','1979-01-31 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 48684,'1979-W171','1979-01-31 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 48685,'1979-W172','1979-01-31 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',62 from dual union all
select 48686,'1979-W173','1979-01-31 16:01:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 48687,'1979-W174','1979-01-31 16:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 48688,'1979-W175','1979-01-31 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',73 from dual union all
select 48689,'1979-W176','1979-01-31 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 4869,'1958-A154','1958-11-29 03:05:00','suborbital rocket','failure',662,'K-6-RS-2','K-6-11','','','Solar UV',7,'',40 from dual union all
select 48690,'1979-W177','1979-01-31 18:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48691,'1979-W178','1979-01-31 21:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',69 from dual union all
select 48692,'1979-W179','1979-01-31 23:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 48693,'1979-W18','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48694,'1979-W180','1979-02-01 15:57:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 48695,'1979-W181','1979-02-01 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 48696,'1979-W182','1979-02-01 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 48697,'1979-W183','1979-02-02 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',974,'MUSSON',61 from dual union all
select 48698,'1979-W184','1979-02-02 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 48699,'1979-W185','1979-02-02 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',57 from dual;
insert into LAUNCH
select 487,'1945-A06','1945-10-01 00:00:00','suborbital rocket','success',1515,'3','1/4 Charge','','','Test',1852,'',8 from dual union all
select 4870,'1958-A155','1958-12-01 00:00:00','atmospheric rocket','success',551,'','Polaris RTV','','','Reentry Test',1821,'',26 from dual union all
select 48700,'1979-W186','1979-02-02 14:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 48701,'1979-W187','1979-02-02 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 48702,'1979-W188','1979-02-02 18:35:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 48703,'1979-W189','1979-02-02 20:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 48704,'1979-W19','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48705,'1979-W190','1979-02-05 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',60 from dual union all
select 48706,'1979-W191','1979-02-05 13:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 48707,'1979-W192','1979-02-05 14:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 48708,'1979-W193','1979-02-05 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 48709,'1979-W194','1979-02-05 17:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 4871,'1958-A156','1958-11-30 04:00:00','suborbital rocket','success',662,'K-6-CP-2','K-6-12','','','CR/Aeron',7,'',49 from dual union all
select 48710,'1979-W195','1979-02-05 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 48711,'1979-W196','1979-02-05 21:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 48712,'1979-W197','1979-02-06 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48713,'1979-W198','1979-02-06 18:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48714,'1979-W199','1979-02-06 19:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 48715,'1979-W20','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48716,'1979-W200','1979-02-06 20:01:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 48717,'1979-W201','1979-02-07 01:30:00','suborbital rocket','success',761,'MT135P T24','','','','Aeron?',1476,'',50 from dual union all
select 48718,'1979-W202','1979-02-07 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',974,'MUSSON',61 from dual union all
select 48719,'1979-W203','1979-02-07 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 4872,'1958-A157','1958-12-02 00:00:00','military missile','success',771,'Training #3','','','','Test',222,'',12 from dual union all
select 48720,'1979-W204','1979-02-07 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 48721,'1979-W205','1979-02-07 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 48722,'1979-W206','1979-02-07 15:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 48723,'1979-W207','1979-02-07 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 48724,'1979-W208','1979-02-07 16:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',51 from dual union all
select 48725,'1979-W209','1979-02-07 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 48726,'1979-W21','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48727,'1979-W210','1979-02-07 18:50:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',78 from dual union all
select 48728,'1979-W211','1979-02-07 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 48729,'1979-W212','1979-02-07 19:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',66 from dual union all
select 4873,'1958-A158','1958-12-03 00:00:00','military missile','success',771,'Training #4','','','','Test',222,'',12 from dual union all
select 48730,'1979-W213','1979-02-07 22:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',72 from dual union all
select 48731,'1979-W214','1979-02-07 22:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 48732,'1979-W215','1979-02-07 23:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 48733,'1979-W216','1979-02-08 18:44:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 48734,'1979-W217','1979-02-08 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 48735,'1979-W218','1979-02-09 01:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48736,'1979-W219','1979-02-09 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',974,'MUSSON',61 from dual union all
select 48737,'1979-W22','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48738,'1979-W220','1979-02-09 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',64 from dual union all
select 48739,'1979-W221','1979-02-09 15:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 4874,'1958-A159','1958-12-04 00:00:00','military missile','success',771,'Training #5','','','','Test',222,'',12 from dual union all
select 48740,'1979-W222','1979-02-09 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',57 from dual union all
select 48741,'1979-W223','1979-02-09 17:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 48742,'1979-W224','1979-02-09 17:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 48743,'1979-W225','1979-02-09 18:57:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 48744,'1979-W226','1979-02-09 22:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 48745,'1979-W227','1979-02-10 03:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 48746,'1979-W228','1979-02-12 13:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 48747,'1979-W229','1979-02-12 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 48748,'1979-W23','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 48749,'1979-W230','1979-02-12 17:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 4875,'1958-A16','1958-03-13 00:00:00','military missile','success',771,'GM-54-24','','','','Test',222,'',12 from dual union all
select 48750,'1979-W231','1979-02-12 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 48751,'1979-W232','1979-02-12 18:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48752,'1979-W233','1979-02-12 20:48:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 48753,'1979-W234','1979-02-12 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 48754,'1979-W235','1979-02-13 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 48755,'1979-W236','1979-02-13 20:33:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 48756,'1979-W237','1979-02-14 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',974,'MUSSON',60 from dual union all
select 48757,'1979-W238','1979-02-14 14:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48758,'1979-W239','1979-02-14 14:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 48759,'1979-W24','1979-01-01 00:00:00','suborbital rocket','success',1284,'','','','','Meteo',428,'',70 from dual union all
select 4876,'1958-A160','1958-12-04 00:00:00','military missile','success',771,'Training #6','','','','Test',222,'',12 from dual union all
select 48760,'1979-W240','1979-02-14 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 48761,'1979-W241','1979-02-14 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',75 from dual union all
select 48762,'1979-W242','1979-02-14 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',54 from dual union all
select 48763,'1979-W243','1979-02-14 19:35:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 48764,'1979-W244','1979-02-14 19:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 48765,'1979-W245','1979-02-14 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 48766,'1979-W246','1979-02-14 21:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',72 from dual union all
select 48767,'1979-W247','1979-02-14 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',75 from dual union all
select 48768,'1979-W248','1979-02-15 01:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 48769,'1979-W249','1979-02-15 15:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 4877,'1958-A161','1958-12-05 00:00:00','military missile','success',511,'6','','','','Test',240,'',10 from dual union all
select 48770,'1979-W25','1979-01-02 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 48771,'1979-W250','1979-02-15 17:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 48772,'1979-W251','1979-02-15 19:35:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',56 from dual union all
select 48773,'1979-W252','1979-02-15 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 48774,'1979-W253','1979-02-16 01:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48775,'1979-W254','1979-02-16 13:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 48776,'1979-W255','1979-02-16 13:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 48777,'1979-W256','1979-02-16 14:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 48778,'1979-W257','1979-02-16 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 48779,'1979-W258','1979-02-16 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 4878,'1958-A162','1958-12-11 00:00:00','military missile','success',1214,'N-69D N-3319','','','','Test',224,'',10 from dual union all
select 48780,'1979-W259','1979-02-16 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 48781,'1979-W26','1979-01-03 09:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',903,'KRENKEL''',60 from dual union all
select 48782,'1979-W260','1979-02-16 20:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 48783,'1979-W261','1979-02-16 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 48784,'1979-W262','1979-02-19 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 48785,'1979-W263','1979-02-19 13:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 48786,'1979-W264','1979-02-19 14:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',69 from dual union all
select 48787,'1979-W265','1979-02-19 16:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 48788,'1979-W266','1979-02-19 18:31:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 48789,'1979-W267','1979-02-20 03:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual;
insert into LAUNCH
select 4879,'1958-A163','1958-12-13 00:00:00','military missile','success',255,'624-XY8','','','','OT',272,'',20 from dual union all
select 48790,'1979-W268','1979-02-20 17:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 48791,'1979-W269','1979-02-20 18:48:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',66 from dual union all
select 48792,'1979-W27','1979-01-03 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 48793,'1979-W270','1979-02-20 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 48794,'1979-W271','1979-02-20 22:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 48795,'1979-W272','1979-02-21 02:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48796,'1979-W273','1979-02-21 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48797,'1979-W274','1979-02-21 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',67 from dual union all
select 48798,'1979-W275','1979-02-21 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 48799,'1979-W276','1979-02-21 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 488,'1945-A07','1945-10-02 00:00:00','suborbital rocket','success',1515,'4','1/4 Charge','','','Test',1852,'',8 from dual union all
select 4880,'1958-A164','1958-12-16 10:00:00','military missile','success',1214,'N-69D N-3308','','','','Test',224,'',10 from dual union all
select 48800,'1979-W277','1979-02-21 16:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 48801,'1979-W278','1979-02-21 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 48802,'1979-W279','1979-02-21 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 48803,'1979-W28','1979-01-03 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 48804,'1979-W280','1979-02-21 19:07:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 48805,'1979-W281','1979-02-21 22:44:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 48806,'1979-W282','1979-02-21 22:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 48807,'1979-W283','1979-02-22 16:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 48808,'1979-W284','1979-02-22 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 48809,'1979-W285','1979-02-22 20:38:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 4881,'1958-A165','1958-12-17 00:00:00','atmospheric rocket','success',545,'','','','','Reentry Test',1821,'',30 from dual union all
select 48810,'1979-W286','1979-02-23 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 48811,'1979-W287','1979-02-23 14:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 48812,'1979-W288','1979-02-23 14:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 48813,'1979-W289','1979-02-23 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 48814,'1979-W29','1979-01-03 14:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 48815,'1979-W290','1979-02-23 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 48816,'1979-W291','1979-02-23 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 48817,'1979-W292','1979-02-23 20:29:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 48818,'1979-W293','1979-02-23 20:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',67 from dual union all
select 48819,'1979-W294','1979-02-23 20:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 4882,'1958-A166','1958-12-18 00:00:00','atmospheric rocket','success',542,'','Corvus Model','','','Test',1821,'',5 from dual union all
select 48820,'1979-W295','1979-02-23 23:55:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 48821,'1979-W296','1979-02-24 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48822,'1979-W297','1979-02-26 12:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48823,'1979-W298','1979-02-26 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',75 from dual union all
select 48824,'1979-W299','1979-02-26 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',54 from dual union all
select 48825,'1979-W30','1979-01-03 16:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',63 from dual union all
select 48826,'1979-W300','1979-02-26 16:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',71 from dual union all
select 48827,'1979-W301','1979-02-26 17:00:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',65 from dual union all
select 48828,'1979-W302','1979-02-26 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 48829,'1979-W303','1979-02-26 17:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 4883,'1958-A167','1958-12-19 00:00:00','military missile','success',255,'624-XY9','','','','OT',293,'',20 from dual union all
select 48830,'1979-W304','1979-02-26 18:23:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',53 from dual union all
select 48831,'1979-W305','1979-02-26 20:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 48832,'1979-W306','1979-02-26 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 48833,'1979-W307','1979-02-26 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',68 from dual union all
select 48834,'1979-W308','1979-02-27 01:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48835,'1979-W309','1979-02-27 04:30:00','suborbital rocket','success',1284,'CMSL-09','Eclipse Blunt probe','','','Aeron',1317,'',75 from dual union all
select 48836,'1979-W31','1979-01-03 19:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 48837,'1979-W310','1979-02-27 13:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 48838,'1979-W311','1979-02-27 14:40:00','suborbital rocket','success',1284,'CMSL-10','Eclipse Blunt Probe','','','Aeron',1317,'',75 from dual union all
select 48839,'1979-W312','1979-02-27 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 4884,'1958-A168','1958-12-24 00:00:00','military missile','success',255,'624-XY10','','','','OT',293,'',20 from dual union all
select 48840,'1979-W313','1979-02-27 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 48841,'1979-W314','1979-02-27 18:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48842,'1979-W315','1979-02-27 18:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 48843,'1979-W316','1979-02-28 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',974,'MUSSON',57 from dual union all
select 48844,'1979-W317','1979-02-28 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 48845,'1979-W318','1979-02-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',66 from dual union all
select 48846,'1979-W319','1979-02-28 14:00:00','suborbital rocket','unknown',738,'','','','','Aeron',961,'',57 from dual union all
select 48847,'1979-W32','1979-01-03 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 48848,'1979-W320','1979-02-28 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48849,'1979-W321','1979-02-28 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',54 from dual union all
select 4885,'1958-A17','1958-03-13 00:00:00','military missile','success',771,'GM-54-142','','','','Test',222,'',12 from dual union all
select 48850,'1979-W322','1979-02-28 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 48851,'1979-W323','1979-02-28 15:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 48852,'1979-W324','1979-02-28 16:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 48853,'1979-W325','1979-02-28 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 48854,'1979-W326','1979-02-28 17:50:00','suborbital rocket','unknown',1284,'FLIGHT 224     /T 1-','','','','Aeron',1821,'',67 from dual union all
select 48855,'1979-W327','1979-02-28 17:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 48856,'1979-W328','1979-02-28 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 48857,'1979-W329','1979-02-28 22:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 48858,'1979-W33','1979-01-04 21:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 48859,'1979-W330','1979-03-01 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',57 from dual union all
select 4886,'1958-A170','1958-12-28 00:00:00','military missile','success',268,'2-04','','','','Test',1752,'',15 from dual union all
select 48860,'1979-W331','1979-03-01 17:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48861,'1979-W332','1979-03-01 19:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 48862,'1979-W333','1979-03-02 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 48863,'1979-W334','1979-03-02 18:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 48864,'1979-W335','1979-03-02 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 48865,'1979-W336','1979-03-02 20:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 48866,'1979-W337','1979-03-02 21:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 48867,'1979-W338','1979-03-02 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',68 from dual union all
select 48868,'1979-W339','1979-03-05 14:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 48869,'1979-W34','1979-01-05 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 4887,'1958-A171','1958-12-30 15:28:00','military missile','failure',933,'AX-3','','','','Test',244,'',40 from dual union all
select 48870,'1979-W340','1979-03-05 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 48871,'1979-W341','1979-03-05 19:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',78 from dual union all
select 48872,'1979-W342','1979-03-05 19:52:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 48873,'1979-W343','1979-03-05 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 48874,'1979-W344','1979-03-05 23:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 48875,'1979-W345','1979-03-06 01:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48876,'1979-W346','1979-03-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',66 from dual union all
select 48877,'1979-W347','1979-03-07 14:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48878,'1979-W348','1979-03-07 15:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 48879,'1979-W349','1979-03-07 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',56 from dual;
insert into LAUNCH
select 4888,'1958-A172','1958-12-31 02:02:00','military missile','failure',1367,'149','R&D Series IV','','','Test',236,'',0 from dual union all
select 48880,'1979-W35','1979-01-05 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',75 from dual union all
select 48881,'1979-W350','1979-03-07 19:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48882,'1979-W351','1979-03-07 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 48883,'1979-W352','1979-03-07 20:04:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 48884,'1979-W353','1979-03-07 22:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 48885,'1979-W354','1979-03-07 23:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',69 from dual union all
select 48886,'1979-W355','1979-03-08 19:54:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 48887,'1979-W356','1979-03-08 20:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 48888,'1979-W357','1979-03-09 14:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 48889,'1979-W358','1979-03-09 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 4889,'1958-A173','1958-11-04 17:00:00','suborbital rocket','success',88,'1','Test','','','Test',1895,'',20 from dual union all
select 48890,'1979-W359','1979-03-09 18:47:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 48891,'1979-W36','1979-01-05 14:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 48892,'1979-W360','1979-03-09 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 48893,'1979-W361','1979-03-09 21:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',59 from dual union all
select 48894,'1979-W362','1979-03-09 23:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',51 from dual union all
select 48895,'1979-W363','1979-03-10 01:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48896,'1979-W364','1979-03-12 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 48897,'1979-W365','1979-03-12 17:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 48898,'1979-W366','1979-03-12 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 48899,'1979-W367','1979-03-12 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 489,'1945-A08','1945-10-04 13:13:33','military missile','failure',1470,'','Backfire 2','','','Test',353,'',17 from dual union all
select 4890,'1958-A174','1958-11-28 17:45:00','suborbital rocket','success',88,'2','Test','','','Test',1895,'',22 from dual union all
select 48900,'1979-W368','1979-03-12 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',50 from dual union all
select 48901,'1979-W369','1979-03-13 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 48902,'1979-W37','1979-01-05 16:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 48903,'1979-W370','1979-03-13 20:45:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 48904,'1979-W371','1979-03-14 09:08:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',74 from dual union all
select 48905,'1979-W372','1979-03-14 15:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 48906,'1979-W373','1979-03-14 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',57 from dual union all
select 48907,'1979-W374','1979-03-14 19:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',78 from dual union all
select 48908,'1979-W375','1979-03-14 20:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 48909,'1979-W376','1979-03-14 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',73 from dual union all
select 4891,'1958-A175','1958-12-08 15:00:00','suborbital rocket','success',88,'3','Test','','','Test',1895,'',28 from dual union all
select 48910,'1979-W377','1979-03-14 23:23:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 48911,'1979-W378','1979-03-14 23:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',72 from dual union all
select 48912,'1979-W379','1979-03-15 16:07:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 48913,'1979-W38','1979-01-05 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',58 from dual union all
select 48914,'1979-W380','1979-03-15 17:08:00','suborbital rocket','unknown',1284,'FLIGHT 225     /TH1-','','','','Aeron',428,'',75 from dual union all
select 48915,'1979-W381','1979-03-15 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 48916,'1979-W382','1979-03-15 21:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',77 from dual union all
select 48917,'1979-W383','1979-03-16 04:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',79 from dual union all
select 48918,'1979-W384','1979-03-16 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 48919,'1979-W385','1979-03-16 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 4892,'1958-A176','1958-06-17 00:00:00','atmospheric rocket','success',603,'1RO','','','','Test',1895,'',2 from dual union all
select 48920,'1979-W386','1979-03-16 21:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 48921,'1979-W387','1979-03-16 21:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 48922,'1979-W388','1979-03-19 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 48923,'1979-W389','1979-03-19 15:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 48924,'1979-W39','1979-01-05 19:41:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 48925,'1979-W390','1979-03-19 15:49:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 48926,'1979-W391','1979-03-19 16:12:00','suborbital rocket','unknown',1284,'FLIGHT 226     /T 1-','','','','Aeron',1821,'',65 from dual union all
select 48927,'1979-W392','1979-03-19 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 48928,'1979-W393','1979-03-19 20:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 48929,'1979-W394','1979-03-19 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 4893,'1958-A177','1958-07-09 00:00:00','atmospheric rocket','success',603,'2RO','','','','Test',1895,'',2 from dual union all
select 48930,'1979-W395','1979-03-20 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 48931,'1979-W396','1979-03-20 06:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 48932,'1979-W397','1979-03-20 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 48933,'1979-W398','1979-03-21 14:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 48934,'1979-W399','1979-03-21 16:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 48935,'1979-W40','1979-01-07 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',61 from dual union all
select 48936,'1979-W400','1979-03-21 17:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 48937,'1979-W401','1979-03-21 18:58:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 48938,'1979-W402','1979-03-21 19:31:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',74 from dual union all
select 48939,'1979-W403','1979-03-21 19:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 4894,'1958-A178','1958-07-24 00:00:00','atmospheric rocket','success',603,'3RO','','','','Test',1895,'',2 from dual union all
select 48940,'1979-W404','1979-03-21 19:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48941,'1979-W405','1979-03-21 22:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',72 from dual union all
select 48942,'1979-W406','1979-03-22 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 48943,'1979-W407','1979-03-23 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 48944,'1979-W408','1979-03-23 18:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 48945,'1979-W409','1979-03-23 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 48946,'1979-W41','1979-01-07 17:27:00','suborbital rocket','success',1284,'F-220/T 1-9631','','','','Aeron',1821,'',64 from dual union all
select 48947,'1979-W410','1979-03-23 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 48948,'1979-W411','1979-03-23 20:43:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 48949,'1979-W412','1979-03-23 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 4895,'1958-A179','1958-08-06 00:00:00','atmospheric rocket','success',603,'4RM','Warhead test','','','Test',1895,'',2 from dual union all
select 48950,'1979-W413','1979-03-24 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 48951,'1979-W414','1979-03-24 05:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 48952,'1979-W415','1979-03-24 14:23:00','suborbital rocket','unknown',1284,'FLIGHT 227     /TN1-','','','','Aeron',330,'',74 from dual union all
select 48953,'1979-W416','1979-03-26 15:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 48954,'1979-W417','1979-03-26 17:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 48955,'1979-W418','1979-03-26 19:49:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 48956,'1979-W419','1979-03-26 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 48957,'1979-W42','1979-01-07 17:49:00','suborbital rocket','success',1284,'F-221/T 1-9645','','','','Aeron',1821,'',65 from dual union all
select 48958,'1979-W420','1979-03-26 21:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 48959,'1979-W421','1979-03-27 02:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 4896,'1958-A18','1958-03-13 00:00:00','military missile','success',771,'GM-56-1962','','','','Test',222,'',12 from dual union all
select 48960,'1979-W422','1979-03-27 07:14:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',50 from dual union all
select 48961,'1979-W423','1979-03-27 19:33:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 48962,'1979-W424','1979-03-28 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 48963,'1979-W425','1979-03-28 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 48964,'1979-W426','1979-03-28 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 48965,'1979-W427','1979-03-28 17:22:00','suborbital rocket','unknown',1284,'FLIGHT 228     /T 1-','','','','Aeron',1821,'',65 from dual union all
select 48966,'1979-W428','1979-03-28 18:40:00','suborbital rocket','unknown',1284,'T 1-9640','','','','Aeron',1821,'',59 from dual union all
select 48967,'1979-W429','1979-03-28 19:51:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',61 from dual union all
select 48968,'1979-W43','1979-01-08 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',75 from dual union all
select 48969,'1979-W430','1979-03-28 19:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',67 from dual;
insert into LAUNCH
select 4897,'1958-A180','1958-08-12 00:00:00','atmospheric rocket','success',603,'5RO','','','','Test',1895,'',2 from dual union all
select 48970,'1979-W431','1979-03-28 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 48971,'1979-W432','1979-03-28 22:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',65 from dual union all
select 48972,'1979-W433','1979-03-29 07:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',51 from dual union all
select 48973,'1979-W434','1979-03-29 21:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 48974,'1979-W435','1979-03-30 14:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 48975,'1979-W436','1979-03-30 15:55:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',60 from dual union all
select 48976,'1979-W437','1979-03-30 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 48977,'1979-W438','1979-03-30 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 48978,'1979-W439','1979-03-30 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 48979,'1979-W44','1979-01-08 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 4898,'1958-A181','1958-08-19 00:00:00','atmospheric rocket','success',603,'Special','','','','Test',1915,'',2 from dual union all
select 48980,'1979-W440','1979-04-01 00:00:00','suborbital rocket','success',716,'','','','','Meteo',428,'',60 from dual union all
select 48981,'1979-W441','1979-03-31 07:19:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 48982,'1979-W442','1979-04-02 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',59 from dual union all
select 48983,'1979-W443','1979-04-02 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 48984,'1979-W444','1979-04-02 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 48985,'1979-W445','1979-04-02 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 48986,'1979-W446','1979-04-02 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 48987,'1979-W447','1979-04-02 20:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 48988,'1979-W448','1979-04-02 22:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 48989,'1979-W449','1979-04-04 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',76 from dual union all
select 4899,'1958-A182','1958-08-25 00:00:00','atmospheric rocket','success',603,'6RO','','','','Test',1895,'',2 from dual union all
select 48990,'1979-W45','1979-01-08 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 48991,'1979-W450','1979-04-04 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 48992,'1979-W451','1979-04-04 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',55 from dual union all
select 48993,'1979-W452','1979-04-04 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 48994,'1979-W453','1979-04-04 16:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 48995,'1979-W454','1979-04-04 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 48996,'1979-W455','1979-04-04 20:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 48997,'1979-W456','1979-04-04 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 48998,'1979-W457','1979-04-04 21:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 48999,'1979-W458','1979-04-04 22:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 49,'1944-A13','1944-08-26 04:57:00','military missile','failure',8,'18344','Ma155','','','Training',661,'',10 from dual union all
select 490,'1945-A09','1945-10-13 00:00:00','atmospheric rocket','success',564,'','R-1','','','Test',1895,'',5 from dual union all
select 4900,'1958-A183','1958-08-25 00:00:00','atmospheric rocket','success',603,'7RO','','','','Test',1895,'',2 from dual union all
select 49000,'1979-W459','1979-04-04 23:46:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',53 from dual union all
select 49001,'1979-W46','1979-01-08 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 49002,'1979-W460','1979-04-05 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 49003,'1979-W461','1979-04-05 01:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49004,'1979-W462','1979-04-05 17:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 49005,'1979-W463','1979-04-05 18:38:00','suborbital rocket','unknown',1284,'FLIGHT 229     /TH1-','','','','Aeron',1307,'',70 from dual union all
select 49006,'1979-W464','1979-04-05 21:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 49007,'1979-W465','1979-04-05 21:54:00','suborbital rocket','unknown',1284,'FLIGHT 230     /T 1-','','','','Aeron',1821,'',67 from dual union all
select 49008,'1979-W466','1979-04-05 22:13:00','suborbital rocket','unknown',1284,'FLIGHT 231     /T 1-','','','','Aeron',1821,'',65 from dual union all
select 49009,'1979-W467','1979-04-05 22:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',51 from dual union all
select 4901,'1958-A184','1958-09-23 00:00:00','atmospheric rocket','success',603,'8RO','','','','Test',1895,'',2 from dual union all
select 49010,'1979-W468','1979-04-06 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 49011,'1979-W469','1979-04-06 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 49012,'1979-W47','1979-01-08 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 49013,'1979-W470','1979-04-06 15:52:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 49014,'1979-W471','1979-04-06 15:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 49015,'1979-W472','1979-04-06 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49016,'1979-W473','1979-04-06 19:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 49017,'1979-W474','1979-04-06 20:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 49018,'1979-W475','1979-04-07 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 49019,'1979-W476','1979-04-09 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 4902,'1958-A185','1958-09-30 00:00:00','atmospheric rocket','success',603,'10RO','','','','Test',1895,'',2 from dual union all
select 49020,'1979-W477','1979-04-09 15:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 49021,'1979-W478','1979-04-09 20:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 49022,'1979-W479','1979-04-09 21:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 49023,'1979-W48','1979-01-08 19:55:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',62 from dual union all
select 49024,'1979-W480','1979-04-09 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 49025,'1979-W481','1979-04-10 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 49026,'1979-W482','1979-04-11 13:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 49027,'1979-W483','1979-04-11 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 49028,'1979-W484','1979-04-11 14:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 49029,'1979-W485','1979-04-11 15:30:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 4903,'1958-A186','1958-01-01 00:00:00','military missile','unknown',845,'','Liquid stage 72/last','','','Test',1895,'',10 from dual union all
select 49030,'1979-W486','1979-04-11 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 49031,'1979-W487','1979-04-11 17:52:00','suborbital rocket','unknown',1284,'FLIGHT 232     /TH1-','','','','Aeron',1307,'',68 from dual union all
select 49032,'1979-W488','1979-04-11 18:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 49033,'1979-W489','1979-04-11 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 49034,'1979-W49','1979-01-08 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 49035,'1979-W490','1979-04-11 22:03:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',69 from dual union all
select 49036,'1979-W491','1979-04-11 22:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 49037,'1979-W492','1979-04-12 19:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 49038,'1979-W493','1979-04-13 13:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 49039,'1979-W494','1979-04-13 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 4904,'1958-A187','1958-04-11 00:00:00','military missile','success',843,'','Ord Eng Eval Test 1','','','Test',1895,'',10 from dual union all
select 49040,'1979-W495','1979-04-13 15:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 49041,'1979-W496','1979-04-13 18:10:00','suborbital rocket','unknown',1284,'FLIGHT 233     /TN1-','','','','Aeron',330,'',65 from dual union all
select 49042,'1979-W497','1979-04-13 18:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 49043,'1979-W498','1979-04-13 23:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 49044,'1979-W499','1979-04-16 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 49045,'1979-W50','1979-01-09 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 49046,'1979-W500','1979-04-16 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49047,'1979-W501','1979-04-16 16:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',59 from dual union all
select 49048,'1979-W502','1979-04-16 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 49049,'1979-W503','1979-04-16 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 4905,'1958-A188','1958-04-28 00:00:00','military missile','success',843,'','Package Training 1','','','Test',433,'',10 from dual union all
select 49050,'1979-W504','1979-04-17 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 49051,'1979-W505','1979-04-17 06:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 49052,'1979-W506','1979-04-18 13:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 49053,'1979-W507','1979-04-18 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49054,'1979-W508','1979-04-18 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 49055,'1979-W509','1979-04-18 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 49056,'1979-W51','1979-01-09 06:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 49057,'1979-W510','1979-04-18 15:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 49058,'1979-W511','1979-04-18 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual;
insert into LAUNCH
select 49059,'1979-W512','1979-04-18 19:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 4906,'1958-A189','1958-12-01 00:00:00','military missile','success',843,'','XM30 No.111','','','Test',1895,'',10 from dual union all
select 49060,'1979-W513','1979-04-18 20:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 49061,'1979-W514','1979-04-19 01:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 49062,'1979-W515','1979-04-19 21:59:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',75 from dual union all
select 49063,'1979-W516','1979-04-19 22:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 49064,'1979-W517','1979-04-20 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 49065,'1979-W518','1979-04-20 14:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 49066,'1979-W519','1979-04-20 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 49067,'1979-W52','1979-01-09 15:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',58 from dual union all
select 49068,'1979-W520','1979-04-20 17:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 49069,'1979-W521','1979-04-20 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 4907,'1958-A19','1958-03-18 00:00:00','military missile','success',262,'9','','','','Test',243,'',10 from dual union all
select 49070,'1979-W522','1979-04-20 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 49071,'1979-W523','1979-04-20 23:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 49072,'1979-W524','1979-04-21 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 49073,'1979-W525','1979-04-21 01:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 49074,'1979-W526','1979-04-21 12:02:00','suborbital rocket','unknown',728,'','','','','Aeron',1376,'SHIR',78 from dual union all
select 49075,'1979-W527','1979-04-21 13:58:00','suborbital rocket','unknown',1284,'FLIGHT 234     /TN1-','','','','Aeron',330,'',67 from dual union all
select 49076,'1979-W528','1979-04-21 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',78 from dual union all
select 49077,'1979-W529','1979-04-23 13:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 49078,'1979-W53','1979-01-09 15:49:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 49079,'1979-W530','1979-04-23 14:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 4908,'1958-A190','1958-05-01 00:00:00','military missile','success',843,'','Package Training 2','','','Test',433,'',10 from dual union all
select 49080,'1979-W531','1979-04-23 17:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1298,'PRILIV',60 from dual union all
select 49081,'1979-W532','1979-04-23 17:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 49082,'1979-W533','1979-04-23 19:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 49083,'1979-W534','1979-04-23 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 49084,'1979-W535','1979-04-23 23:45:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 49085,'1979-W536','1979-04-25 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',904,'KRENKEL''',58 from dual union all
select 49086,'1979-W537','1979-04-25 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 49087,'1979-W538','1979-04-25 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 49088,'1979-W539','1979-04-25 16:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',53 from dual union all
select 49089,'1979-W54','1979-01-09 16:28:00','suborbital rocket','unknown',1284,'FLIGHT 222     /T 1-','','','','Aeron',1821,'',64 from dual union all
select 4909,'1958-A191','1958-05-01 00:00:00','military missile','success',843,'','Package Training 3','','','Test',433,'',10 from dual union all
select 49090,'1979-W540','1979-04-25 16:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 49091,'1979-W541','1979-04-25 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',71 from dual union all
select 49092,'1979-W542','1979-04-25 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 49093,'1979-W543','1979-04-25 21:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 49094,'1979-W544','1979-04-25 22:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 49095,'1979-W545','1979-04-25 23:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 49096,'1979-W546','1979-04-26 21:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',73 from dual union all
select 49097,'1979-W547','1979-04-27 06:09:00','suborbital rocket','failure',229,'AKF-VI-16','Auroral X-rays','','','Auroral',428,'',50 from dual union all
select 49098,'1979-W548','1979-04-27 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',904,'KRENKEL''',59 from dual union all
select 49099,'1979-W549','1979-04-27 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',75 from dual union all
select 491,'1945-A10','1945-10-13 00:00:00','atmospheric rocket','success',564,'','R-2','','','Test',1895,'',5 from dual union all
select 4910,'1958-A192','1958-05-01 00:00:00','military missile','success',843,'','Package Training 4','','','Test',433,'',10 from dual union all
select 49100,'1979-W55','1979-01-09 19:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 49101,'1979-W550','1979-04-27 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 49102,'1979-W551','1979-04-27 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 49103,'1979-W552','1979-04-27 19:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',52 from dual union all
select 49104,'1979-W553','1979-04-27 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 49105,'1979-W554','1979-04-27 23:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 49106,'1979-W555','1979-04-28 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 49107,'1979-W556','1979-04-28 06:44:00','suborbital rocket','success',229,'AKF-VI-17','Auroral X-rays','','','Auroral',428,'',60 from dual union all
select 49108,'1979-W557','1979-04-30 14:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 49109,'1979-W558','1979-04-30 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 4911,'1958-A193','1958-05-01 00:00:00','military missile','success',843,'','Package Training 5','','','Test',433,'',10 from dual union all
select 49110,'1979-W559','1979-04-30 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',56 from dual union all
select 49111,'1979-W56','1979-01-09 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 49112,'1979-W560','1979-04-30 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 49113,'1979-W561','1979-04-30 20:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',52 from dual union all
select 49114,'1979-W562','1979-04-30 21:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 49115,'1979-W563','1979-04-30 22:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 49116,'1979-W564','1979-05-02 13:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',75 from dual union all
select 49117,'1979-W565','1979-05-02 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 49118,'1979-W566','1979-05-02 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',52 from dual union all
select 49119,'1979-W567','1979-05-02 15:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 4912,'1958-A194','1958-05-01 00:00:00','military missile','success',843,'','Package Training 6','','','Test',433,'',10 from dual union all
select 49120,'1979-W568','1979-05-02 15:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',51 from dual union all
select 49121,'1979-W569','1979-05-02 17:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 49122,'1979-W57','1979-01-10 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 49123,'1979-W570','1979-05-02 19:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',70 from dual union all
select 49124,'1979-W571','1979-05-02 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 49125,'1979-W572','1979-05-02 22:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 49126,'1979-W573','1979-05-02 23:56:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 49127,'1979-W574','1979-05-03 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',59 from dual union all
select 49128,'1979-W575','1979-05-03 16:35:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',59 from dual union all
select 49129,'1979-W576','1979-05-03 16:36:00','suborbital rocket','unknown',1284,'FLIGHT 235     /T 1-','','','','Aeron',1821,'',64 from dual union all
select 4913,'1958-A195','1958-06-01 00:00:00','military missile','success',843,'','Package Training 7','','','Test',433,'',10 from dual union all
select 49130,'1979-W577','1979-05-03 18:05:00','suborbital rocket','unknown',1284,'FLIGHT 236     /TH1-','','','','Aeron',1307,'',68 from dual union all
select 49131,'1979-W578','1979-05-03 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 49132,'1979-W579','1979-05-03 22:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',74 from dual union all
select 49133,'1979-W58','1979-01-10 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 49134,'1979-W580','1979-05-04 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49135,'1979-W581','1979-05-04 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',71 from dual union all
select 49136,'1979-W582','1979-05-04 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',50 from dual union all
select 49137,'1979-W583','1979-05-04 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 49138,'1979-W584','1979-05-04 15:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 49139,'1979-W585','1979-05-04 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 4914,'1958-A196','1958-06-01 00:00:00','military missile','success',843,'','Package Training 8','','','Test',433,'',10 from dual union all
select 49140,'1979-W586','1979-05-04 18:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',60 from dual union all
select 49141,'1979-W587','1979-05-04 19:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 49142,'1979-W588','1979-05-04 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 49143,'1979-W589','1979-05-04 22:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',52 from dual union all
select 49144,'1979-W59','1979-01-10 15:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',68 from dual union all
select 49145,'1979-W590','1979-05-05 00:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',60 from dual union all
select 49146,'1979-W591','1979-05-07 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 49147,'1979-W592','1979-05-07 14:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 49148,'1979-W593','1979-05-07 15:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual;
insert into LAUNCH
select 49149,'1979-W594','1979-05-07 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 4915,'1958-A197','1958-06-01 00:00:00','military missile','success',843,'','SNODGRASS IMB 1','','','Test',433,'',10 from dual union all
select 49150,'1979-W595','1979-05-07 18:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 49151,'1979-W596','1979-05-07 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 49152,'1979-W597','1979-05-08 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 49153,'1979-W598','1979-05-09 02:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 49154,'1979-W599','1979-05-09 14:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 49155,'1979-W60','1979-01-10 15:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',62 from dual union all
select 49156,'1979-W600','1979-05-09 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',56 from dual union all
select 49157,'1979-W601','1979-05-09 17:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',53 from dual union all
select 49158,'1979-W602','1979-05-09 17:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',57 from dual union all
select 49159,'1979-W603','1979-05-09 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',73 from dual union all
select 4916,'1958-A198','1958-06-01 00:00:00','military missile','success',843,'','SNODGRASS IMB 2','','','Test',433,'',10 from dual union all
select 49160,'1979-W604','1979-05-09 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49161,'1979-W605','1979-05-09 19:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 49162,'1979-W606','1979-05-09 19:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',69 from dual union all
select 49163,'1979-W607','1979-05-09 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 49164,'1979-W608','1979-05-09 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',58 from dual union all
select 49165,'1979-W609','1979-05-09 22:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 49166,'1979-W61','1979-01-10 16:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',76 from dual union all
select 49167,'1979-W610','1979-05-11 01:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 49168,'1979-W611','1979-05-11 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49169,'1979-W612','1979-05-11 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 4917,'1958-A199','1958-07-01 00:00:00','military missile','success',843,'','Project AMMO','','','Test',1895,'',30 from dual union all
select 49170,'1979-W613','1979-05-11 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 49171,'1979-W614','1979-05-11 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 49172,'1979-W615','1979-05-11 15:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 49173,'1979-W616','1979-05-11 19:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 49174,'1979-W617','1979-05-11 19:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 49175,'1979-W618','1979-05-11 23:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 49176,'1979-W619','1979-05-12 16:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',55 from dual union all
select 49177,'1979-W62','1979-01-10 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 49178,'1979-W620','1979-05-12 17:46:00','suborbital rocket','unknown',1284,'FLIGHT 237','','','','Aeron',1821,'',65 from dual union all
select 49179,'1979-W621','1979-05-14 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 4918,'1958-A20','1958-03-20 00:00:00','military missile','success',255,'624-17','','','','OT',293,'',20 from dual union all
select 49180,'1979-W622','1979-05-14 14:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',56 from dual union all
select 49181,'1979-W623','1979-05-14 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 49182,'1979-W624','1979-05-14 15:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 49183,'1979-W625','1979-05-14 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 49184,'1979-W626','1979-05-14 18:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',77 from dual union all
select 49185,'1979-W627','1979-05-14 20:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 49186,'1979-W628','1979-05-14 23:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',68 from dual union all
select 49187,'1979-W629','1979-05-15 01:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 49188,'1979-W63','1979-01-10 18:40:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',64 from dual union all
select 49189,'1979-W630','1979-05-15 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 4919,'1958-A200','1958-07-14 00:00:00','military missile','success',843,'','SNODGRASS Ph3-1','','','Test',394,'',10 from dual union all
select 49190,'1979-W631','1979-05-16 02:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',61 from dual union all
select 49191,'1979-W632','1979-05-16 13:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 49192,'1979-W633','1979-05-16 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 49193,'1979-W634','1979-05-16 15:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',55 from dual union all
select 49194,'1979-W635','1979-05-16 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',54 from dual union all
select 49195,'1979-W636','1979-05-16 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 49196,'1979-W637','1979-05-16 19:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 49197,'1979-W638','1979-05-16 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 49198,'1979-W639','1979-05-16 21:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1298,'PRILIV',55 from dual union all
select 49199,'1979-W64','1979-01-10 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',74 from dual union all
select 492,'1945-A11','1945-10-13 00:00:00','atmospheric rocket','success',564,'','R-3','','','Test',1895,'',5 from dual union all
select 4920,'1958-A201','1958-07-17 00:00:00','military missile','success',843,'','SNODGRASS Ph3-2','','','Test',394,'',10 from dual union all
select 49200,'1979-W640','1979-05-16 23:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 49201,'1979-W641','1979-05-17 01:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 49202,'1979-W642','1979-05-17 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 49203,'1979-W643','1979-05-18 02:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 49204,'1979-W644','1979-05-18 14:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 49205,'1979-W645','1979-05-18 14:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',55 from dual union all
select 49206,'1979-W646','1979-05-18 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 49207,'1979-W647','1979-05-18 17:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',56 from dual union all
select 49208,'1979-W648','1979-05-18 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 49209,'1979-W649','1979-05-18 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 4921,'1958-A202','1958-07-24 00:00:00','military missile','success',843,'','SNODGRASS Ph3-3','','','Test',394,'',2 from dual union all
select 49210,'1979-W65','1979-01-10 21:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 49211,'1979-W650','1979-05-18 19:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 49212,'1979-W651','1979-05-18 20:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1298,'PRILIV',58 from dual union all
select 49213,'1979-W652','1979-05-19 02:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 49214,'1979-W653','1979-05-21 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 49215,'1979-W654','1979-05-21 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 49216,'1979-W655','1979-05-21 15:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 49217,'1979-W656','1979-05-21 16:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 49218,'1979-W657','1979-05-21 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 49219,'1979-W658','1979-05-21 19:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 4922,'1958-A203','1958-07-24 00:00:00','military missile','success',843,'','SNODGRASS Ph3-4','','','Test',394,'',2 from dual union all
select 49220,'1979-W659','1979-05-21 20:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1298,'PRILIV',59 from dual union all
select 49221,'1979-W66','1979-01-10 21:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 49222,'1979-W660','1979-05-21 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 49223,'1979-W661','1979-05-21 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 49224,'1979-W662','1979-05-22 01:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 49225,'1979-W663','1979-05-23 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',77 from dual union all
select 49226,'1979-W664','1979-05-23 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 49227,'1979-W665','1979-05-23 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 49228,'1979-W666','1979-05-23 14:32:00','suborbital rocket','unknown',728,'','','','','Aeron',1376,'SHIR',78 from dual union all
select 49229,'1979-W667','1979-05-23 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 4923,'1958-A204','1958-07-29 00:00:00','military missile','success',843,'','SNODGRASS Ph3-5','','','Test',394,'',10 from dual union all
select 49230,'1979-W668','1979-05-23 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 49231,'1979-W669','1979-05-23 15:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 49232,'1979-W67','1979-01-10 23:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 49233,'1979-W670','1979-05-23 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 49234,'1979-W671','1979-05-23 17:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',57 from dual union all
select 49235,'1979-W672','1979-05-23 19:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 49236,'1979-W673','1979-05-23 20:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1298,'PRILIV',58 from dual union all
select 49237,'1979-W674','1979-05-23 22:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 49238,'1979-W675','1979-05-23 22:54:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',78 from dual;
insert into LAUNCH
select 49239,'1979-W676','1979-05-24 00:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 4924,'1958-A205','1958-07-29 00:00:00','military missile','success',843,'','SNODGRASS Ph3-6','','','Test',394,'',10 from dual union all
select 49240,'1979-W677','1979-05-24 20:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1298,'PRILIV',60 from dual union all
select 49241,'1979-W678','1979-05-25 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 49242,'1979-W679','1979-05-25 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 49243,'1979-W68','1979-01-11 16:18:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',59 from dual union all
select 49244,'1979-W680','1979-05-25 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',55 from dual union all
select 49245,'1979-W681','1979-05-25 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',52 from dual union all
select 49246,'1979-W682','1979-05-25 16:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 49247,'1979-W683','1979-05-25 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 49248,'1979-W684','1979-05-25 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 49249,'1979-W685','1979-05-25 18:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 4925,'1958-A206','1958-07-28 00:00:00','atmospheric rocket','success',272,'','Sea impact test','','','Test',1821,'',10 from dual union all
select 49250,'1979-W686','1979-05-25 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 49251,'1979-W687','1979-05-25 21:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 49252,'1979-W688','1979-05-25 23:15:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 49253,'1979-W689','1979-05-26 03:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 49254,'1979-W69','1979-01-11 17:14:00','suborbital rocket','unknown',1284,'FLIGHT 223     /TH1-','','','','Aeron',428,'',76 from dual union all
select 49255,'1979-W690','1979-05-27 03:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1376,'SHIR',79 from dual union all
select 49256,'1979-W691','1979-05-27 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',975,'MUSSON',58 from dual union all
select 49257,'1979-W692','1979-05-27 20:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1298,'PRILIV',59 from dual union all
select 49258,'1979-W693','1979-05-27 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',79 from dual union all
select 49259,'1979-W694','1979-05-28 03:16:00','suborbital rocket','unknown',728,'','','','','Aeron',1376,'SHIR',79 from dual union all
select 4926,'1958-A207','1958-01-01 00:00:00','atmospheric rocket','success',925,'Terrier Target','POGO','','','Target',158,'DDG-1',10 from dual union all
select 49260,'1979-W695','1979-05-28 14:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 49261,'1979-W696','1979-05-28 20:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1298,'PRILIV',58 from dual union all
select 49262,'1979-W697','1979-05-29 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 49263,'1979-W698','1979-05-29 14:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 49264,'1979-W699','1979-05-29 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 49265,'1979-W70','1979-01-11 19:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 49266,'1979-W700','1979-05-29 20:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1298,'PRILIV',58 from dual union all
select 49267,'1979-W701','1979-05-29 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',51 from dual union all
select 49268,'1979-W702','1979-05-29 21:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',58 from dual union all
select 49269,'1979-W703','1979-05-30 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 4927,'1958-A208','1958-01-01 00:00:00','atmospheric rocket','success',925,'Terrier Target','POGO','','','Target',158,'DDG-1',10 from dual union all
select 49270,'1979-W704','1979-05-30 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 49271,'1979-W705','1979-05-30 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 49272,'1979-W706','1979-05-30 15:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 49273,'1979-W707','1979-05-30 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 49274,'1979-W708','1979-05-30 18:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 49275,'1979-W709','1979-05-30 19:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 49276,'1979-W71','1979-01-11 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 49277,'1979-W710','1979-05-30 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 49278,'1979-W711','1979-05-30 21:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',75 from dual union all
select 49279,'1979-W712','1979-05-30 22:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 4928,'1958-A209','1958-01-01 00:00:00','atmospheric rocket','success',926,'','Pogo Hi Type IIC5a','','','Target',1895,'',36 from dual union all
select 49280,'1979-W713','1979-05-31 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 49281,'1979-W714','1979-06-01 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',975,'MUSSON',59 from dual union all
select 49282,'1979-W715','1979-06-01 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49283,'1979-W716','1979-06-01 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 49284,'1979-W717','1979-06-01 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 49285,'1979-W718','1979-06-01 17:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 49286,'1979-W719','1979-06-01 21:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 49287,'1979-W72','1979-01-12 01:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 49288,'1979-W720','1979-06-01 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 49289,'1979-W721','1979-06-02 01:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 4929,'1958-A21','1958-03-21 00:00:00','military missile','failure',268,'2/2','','','','Test',1752,'',0 from dual union all
select 49290,'1979-W722','1979-06-04 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 49291,'1979-W723','1979-06-04 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49292,'1979-W724','1979-06-04 14:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 49293,'1979-W725','1979-06-04 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 49294,'1979-W726','1979-06-04 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 49295,'1979-W727','1979-06-04 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 49296,'1979-W728','1979-06-04 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 49297,'1979-W729','1979-06-05 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 49298,'1979-W73','1979-01-12 13:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',54 from dual union all
select 49299,'1979-W730','1979-06-06 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',975,'MUSSON',59 from dual union all
select 493,'1945-A12','1945-10-13 00:00:00','atmospheric rocket','success',564,'','R-4','','','Test',1895,'',5 from dual union all
select 4930,'1958-A210','1958-01-01 00:00:00','suborbital rocket','success',925,'','Pogo Hi Type IC5a','','','Target',1895,'',18 from dual union all
select 49300,'1979-W731','1979-06-06 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 49301,'1979-W732','1979-06-06 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 49302,'1979-W733','1979-06-06 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 49303,'1979-W734','1979-06-06 14:23:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 49304,'1979-W735','1979-06-06 14:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',55 from dual union all
select 49305,'1979-W736','1979-06-06 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 49306,'1979-W737','1979-06-06 16:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 49307,'1979-W738','1979-06-06 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 49308,'1979-W739','1979-06-06 17:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 49309,'1979-W74','1979-01-12 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 4931,'1958-A211','1958-01-01 00:00:00','suborbital rocket','success',925,'','Pogo Hi Type IC7a','','','Target',1895,'',21 from dual union all
select 49310,'1979-W740','1979-06-06 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 49311,'1979-W741','1979-06-06 22:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 49312,'1979-W742','1979-06-08 01:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 49313,'1979-W743','1979-06-08 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',975,'MUSSON',59 from dual union all
select 49314,'1979-W744','1979-06-08 13:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',77 from dual union all
select 49315,'1979-W745','1979-06-08 14:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 49316,'1979-W746','1979-06-08 18:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 49317,'1979-W747','1979-06-08 19:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',66 from dual union all
select 49318,'1979-W748','1979-06-10 13:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',75 from dual union all
select 49319,'1979-W749','1979-06-11 14:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',64 from dual union all
select 4932,'1958-A212','1958-01-01 00:00:00','suborbital rocket','success',925,'','Pogo Hi Type IC5c','','','Target',1895,'',18 from dual union all
select 49320,'1979-W75','1979-01-12 15:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',54 from dual union all
select 49321,'1979-W750','1979-06-11 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 49322,'1979-W751','1979-06-11 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 49323,'1979-W752','1979-06-11 16:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 49324,'1979-W753','1979-06-11 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',65 from dual union all
select 49325,'1979-W754','1979-06-11 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 49326,'1979-W755','1979-06-12 01:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 49327,'1979-W756','1979-06-13 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',975,'MUSSON',58 from dual union all
select 49328,'1979-W757','1979-06-13 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual;
insert into LAUNCH
select 49329,'1979-W758','1979-06-13 13:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',75 from dual union all
select 4933,'1958-A213','1958-01-01 00:00:00','suborbital rocket','success',925,'','Pogo Hi Type IC5c','','','Target',1895,'',18 from dual union all
select 49330,'1979-W759','1979-06-13 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 49331,'1979-W76','1979-01-12 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 49332,'1979-W760','1979-06-13 14:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 49333,'1979-W761','1979-06-13 15:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 49334,'1979-W762','1979-06-13 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 49335,'1979-W763','1979-06-13 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 49336,'1979-W764','1979-06-13 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 49337,'1979-W765','1979-06-13 18:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 49338,'1979-W766','1979-06-14 03:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 49339,'1979-W767','1979-06-15 02:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 4934,'1958-A214','1958-01-01 00:00:00','atmospheric rocket','success',926,'','Pogo Hi Type IIC5c','','','Target',1895,'',36 from dual union all
select 49340,'1979-W768','1979-06-15 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',975,'MUSSON',59 from dual union all
select 49341,'1979-W769','1979-06-15 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 49342,'1979-W77','1979-01-12 18:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 49343,'1979-W770','1979-06-15 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 49344,'1979-W771','1979-06-15 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',54 from dual union all
select 49345,'1979-W772','1979-06-15 15:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 49346,'1979-W773','1979-06-15 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 49347,'1979-W774','1979-06-15 17:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 49348,'1979-W775','1979-06-15 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 49349,'1979-W776','1979-06-15 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',54 from dual union all
select 4935,'1958-A215','1958-07-29 00:00:00','military missile','failure',992,'51120301','','','','OT',636,'',40 from dual union all
select 49350,'1979-W777','1979-06-18 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 49351,'1979-W778','1979-06-18 14:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 49352,'1979-W779','1979-06-18 14:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 49353,'1979-W78','1979-01-12 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 49354,'1979-W780','1979-06-18 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',73 from dual union all
select 49355,'1979-W781','1979-06-18 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 49356,'1979-W782','1979-06-18 19:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',62 from dual union all
select 49357,'1979-W783','1979-06-18 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 49358,'1979-W784','1979-06-18 22:24:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',64 from dual union all
select 49359,'1979-W785','1979-06-20 01:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 4936,'1958-A216','1958-09-08 00:00:00','military missile','failure',971,'M2-24','','','','Test',636,'',40 from dual union all
select 49360,'1979-W786','1979-06-20 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',975,'MUSSON',60 from dual union all
select 49361,'1979-W787','1979-06-20 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',53 from dual union all
select 49362,'1979-W788','1979-06-20 14:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',55 from dual union all
select 49363,'1979-W789','1979-06-20 14:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 49364,'1979-W79','1979-01-12 19:17:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',66 from dual union all
select 49365,'1979-W790','1979-06-20 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',58 from dual union all
select 49366,'1979-W791','1979-06-20 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 49367,'1979-W792','1979-06-20 18:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 49368,'1979-W793','1979-06-20 23:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 49369,'1979-W794','1979-06-22 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',975,'MUSSON',59 from dual union all
select 4937,'1958-A22','1958-03-24 00:00:00','atmospheric rocket','success',835,'','F4H-1 Model','','','Test',1821,'',10 from dual union all
select 49370,'1979-W795','1979-06-22 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 49371,'1979-W796','1979-06-22 15:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 49372,'1979-W797','1979-06-22 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 49373,'1979-W798','1979-06-22 18:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 49374,'1979-W799','1979-06-22 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',53 from dual union all
select 49375,'1979-W80','1979-01-15 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 49376,'1979-W800','1979-06-22 20:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 49377,'1979-W801','1979-06-22 23:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',66 from dual union all
select 49378,'1979-W802','1979-06-23 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 49379,'1979-W803','1979-06-25 02:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 4938,'1958-A23','1958-03-28 00:00:00','atmospheric rocket','success',1052,'','','','','Test',1821,'F2H2',4 from dual union all
select 49380,'1979-W804','1979-06-25 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 49381,'1979-W805','1979-06-25 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 49382,'1979-W806','1979-06-25 14:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 49383,'1979-W807','1979-06-25 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 49384,'1979-W808','1979-06-25 17:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 49385,'1979-W809','1979-06-25 17:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 49386,'1979-W81','1979-01-15 14:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 49387,'1979-W810','1979-06-25 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 49388,'1979-W811','1979-06-25 20:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 49389,'1979-W812','1979-06-25 22:49:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 4939,'1958-A24','1958-03-31 00:00:00','military missile','success',1080,'F75','','','','Test',671,'',20 from dual union all
select 49390,'1979-W813','1979-06-26 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 49391,'1979-W814','1979-06-27 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',975,'MUSSON',61 from dual union all
select 49392,'1979-W815','1979-06-27 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 49393,'1979-W816','1979-06-27 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 49394,'1979-W817','1979-06-27 14:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 49395,'1979-W818','1979-06-27 14:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',50 from dual union all
select 49396,'1979-W819','1979-06-27 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',55 from dual union all
select 49397,'1979-W82','1979-01-15 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 49398,'1979-W820','1979-06-27 16:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 49399,'1979-W821','1979-06-27 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 494,'1945-A13','1945-10-14 00:00:00','atmospheric rocket','success',564,'','R-5','','','Test',1895,'',5 from dual union all
select 4940,'1958-A25','1958-03-31 00:00:00','military missile','success',1080,'F85','','','','Test',671,'',20 from dual union all
select 49400,'1979-W822','1979-06-27 19:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 49401,'1979-W823','1979-06-27 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',56 from dual union all
select 49402,'1979-W824','1979-06-27 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',55 from dual union all
select 49403,'1979-W825','1979-06-27 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 49404,'1979-W826','1979-06-29 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 49405,'1979-W827','1979-06-29 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 49406,'1979-W828','1979-06-29 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 49407,'1979-W829','1979-06-29 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49408,'1979-W83','1979-01-15 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 49409,'1979-W830','1979-06-29 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',71 from dual union all
select 4941,'1958-A26','1958-04-01 00:00:00','military missile','failure',1080,'F83','','','','Test',671,'',20 from dual union all
select 49410,'1979-W831','1979-06-29 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 49411,'1979-W832','1979-06-29 20:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 49412,'1979-W833','1979-06-29 22:30:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 49413,'1979-W834','1979-06-30 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 49414,'1979-W835','1979-07-02 13:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 49415,'1979-W836','1979-07-02 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 49416,'1979-W837','1979-07-02 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 49417,'1979-W838','1979-07-02 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 49418,'1979-W839','1979-07-02 20:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',52 from dual;
insert into LAUNCH
select 49419,'1979-W84','1979-01-15 20:49:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',58 from dual union all
select 4942,'1958-A27','1958-04-01 20:32:00','suborbital rocket','success',1193,'SL06','RAE Test 4/Dum probe','','','Test',1839,'',42 from dual union all
select 49420,'1979-W840','1979-07-02 23:10:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',57 from dual union all
select 49421,'1979-W841','1979-07-03 00:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 49422,'1979-W842','1979-07-03 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',57 from dual union all
select 49423,'1979-W843','1979-07-04 02:00:00','suborbital rocket','success',761,'MT135P-R355','','','','Meteo',1332,'',64 from dual union all
select 49424,'1979-W844','1979-07-04 02:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 49425,'1979-W845','1979-07-04 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',905,'KRENKEL''',58 from dual union all
select 49426,'1979-W846','1979-07-04 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 49427,'1979-W847','1979-07-05 02:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 49428,'1979-W848','1979-07-06 02:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 49429,'1979-W849','1979-07-06 14:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 4943,'1958-A28','1958-04-02 16:20:00','military missile','success',255,'624-18','','','','OT',293,'',20 from dual union all
select 49430,'1979-W85','1979-01-15 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 49431,'1979-W850','1979-07-06 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 49432,'1979-W851','1979-07-06 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 49433,'1979-W852','1979-07-06 18:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 49434,'1979-W853','1979-07-06 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',66 from dual union all
select 49435,'1979-W854','1979-07-06 20:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',50 from dual union all
select 49436,'1979-W855','1979-07-07 01:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 49437,'1979-W856','1979-07-09 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 49438,'1979-W857','1979-07-09 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 49439,'1979-W858','1979-07-09 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 4944,'1958-A29','1958-04-03 00:00:00','military missile','success',1214,'N-69E N-3329','','','','Test',224,'',10 from dual union all
select 49440,'1979-W859','1979-07-09 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',63 from dual union all
select 49441,'1979-W86','1979-01-16 19:29:00','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',66 from dual union all
select 49442,'1979-W860','1979-07-09 18:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 49443,'1979-W861','1979-07-09 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 49444,'1979-W862','1979-07-09 23:40:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 49445,'1979-W863','1979-07-10 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 49446,'1979-W864','1979-07-10 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 49447,'1979-W865','1979-07-11 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',905,'KRENKEL''',59 from dual union all
select 49448,'1979-W866','1979-07-11 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 49449,'1979-W867','1979-07-11 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',62 from dual union all
select 4945,'1958-A30','1958-04-08 01:31:00','suborbital rocket','success',642,'K150S-1','','','','Test',7,'',6 from dual union all
select 49450,'1979-W868','1979-07-11 14:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 49451,'1979-W869','1979-07-11 14:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',61 from dual union all
select 49452,'1979-W87','1979-01-16 21:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 49453,'1979-W870','1979-07-11 16:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 49454,'1979-W871','1979-07-11 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',58 from dual union all
select 49455,'1979-W872','1979-07-11 17:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',73 from dual union all
select 49456,'1979-W873','1979-07-11 18:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 49457,'1979-W874','1979-07-11 23:15:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',63 from dual union all
select 49458,'1979-W875','1979-07-12 03:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 49459,'1979-W876','1979-07-12 14:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1298,'PRILIV',57 from dual union all
select 4946,'1958-A31','1958-04-14 00:00:00','atmospheric rocket','success',442,'','D3/D18 Model','','','Test',1821,'',10 from dual union all
select 49460,'1979-W877','1979-07-12 15:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1298,'PRILIV',58 from dual union all
select 49461,'1979-W878','1979-07-12 16:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1298,'PRILIV',58 from dual union all
select 49462,'1979-W879','1979-07-12 21:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 49463,'1979-W88','1979-01-16 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',58 from dual union all
select 49464,'1979-W880','1979-07-13 02:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 49465,'1979-W881','1979-07-13 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 49466,'1979-W882','1979-07-13 15:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',63 from dual union all
select 49467,'1979-W883','1979-07-13 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 49468,'1979-W884','1979-07-13 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49469,'1979-W885','1979-07-13 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 4947,'1958-A32','1958-04-18 00:00:00','military missile','failure',262,'10','','','','Test',243,'',10 from dual union all
select 49470,'1979-W886','1979-07-13 20:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 49471,'1979-W887','1979-07-13 21:00:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',65 from dual union all
select 49472,'1979-W888','1979-07-14 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',76 from dual union all
select 49473,'1979-W889','1979-07-16 13:20:00','suborbital rocket','success',1281,'NASA 15.196GM','','','','Aeronomy',1047,'',72 from dual union all
select 49474,'1979-W89','1979-01-17 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 49475,'1979-W890','1979-07-16 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 49476,'1979-W891','1979-07-16 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 49477,'1979-W892','1979-07-16 14:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',59 from dual union all
select 49478,'1979-W893','1979-07-16 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 49479,'1979-W894','1979-07-16 16:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 4948,'1958-A33','1958-04-19 13:30:00','military missile','failure',1366,'121','R&D Series II','','','Test',234,'',0 from dual union all
select 49480,'1979-W895','1979-07-16 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 49481,'1979-W896','1979-07-16 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 49482,'1979-W897','1979-07-16 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 49483,'1979-W898','1979-07-17 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',66 from dual union all
select 49484,'1979-W899','1979-07-17 10:52:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 49485,'1979-W90','1979-01-17 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 49486,'1979-W900','1979-07-17 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 49487,'1979-W901','1979-07-18 00:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 49488,'1979-W902','1979-07-18 02:00:00','suborbital rocket','success',761,'MT135P-R356','','','','Meteo',1332,'',59 from dual union all
select 49489,'1979-W903','1979-07-18 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',905,'KRENKEL''',57 from dual union all
select 4949,'1958-A34','1958-04-20 00:00:00','suborbital rocket','success',661,'K-5-1','','','','Test',7,'',9 from dual union all
select 49490,'1979-W904','1979-07-18 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 49491,'1979-W905','1979-07-18 14:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 49492,'1979-W906','1979-07-18 14:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',53 from dual union all
select 49493,'1979-W907','1979-07-18 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 49494,'1979-W908','1979-07-18 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 49495,'1979-W909','1979-07-18 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',74 from dual union all
select 49496,'1979-W91','1979-01-17 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 49497,'1979-W910','1979-07-18 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 49498,'1979-W911','1979-07-18 18:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 49499,'1979-W912','1979-07-19 09:51:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',61 from dual union all
select 495,'1945-A14','1945-10-14 00:00:00','atmospheric rocket','success',564,'','R-6','','','Test',1895,'',5 from dual union all
select 4950,'1958-A36','1958-04-24 00:00:00','atmospheric rocket','success',552,'','GE RV Model','','','Reentry Test',1821,'',30 from dual union all
select 49500,'1979-W913','1979-07-20 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49501,'1979-W914','1979-07-20 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 49502,'1979-W915','1979-07-20 14:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 49503,'1979-W916','1979-07-21 01:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 49504,'1979-W917','1979-07-22 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 49505,'1979-W918','1979-07-23 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 49506,'1979-W919','1979-07-23 14:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 49507,'1979-W92','1979-01-17 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 49508,'1979-W920','1979-07-23 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual;
insert into LAUNCH
select 49509,'1979-W921','1979-07-23 18:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 4951,'1958-A37','1958-04-24 00:30:00','suborbital rocket','success',642,'K150T-1','','','','Test',7,'',6 from dual union all
select 49510,'1979-W922','1979-07-23 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 49511,'1979-W923','1979-07-23 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 49512,'1979-W924','1979-07-25 00:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 49513,'1979-W925','1979-07-25 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 49514,'1979-W926','1979-07-25 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 49515,'1979-W927','1979-07-25 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 49516,'1979-W928','1979-07-25 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',78 from dual union all
select 49517,'1979-W929','1979-07-25 14:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 49518,'1979-W93','1979-01-17 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',74 from dual union all
select 49519,'1979-W930','1979-07-25 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 4952,'1958-A38','1958-04-24 04:00:00','suborbital rocket','success',642,'K150T-2','','','','Test',7,'',6 from dual union all
select 49520,'1979-W931','1979-07-25 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',57 from dual union all
select 49521,'1979-W932','1979-07-25 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 49522,'1979-W933','1979-07-25 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',59 from dual union all
select 49523,'1979-W934','1979-07-25 21:04:59','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 49524,'1979-W935','1979-07-25 21:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 49525,'1979-W936','1979-07-26 20:20:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',62 from dual union all
select 49526,'1979-W937','1979-07-27 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 49527,'1979-W938','1979-07-27 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 49528,'1979-W939','1979-07-27 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 49529,'1979-W94','1979-01-17 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 4953,'1958-A39','1958-04-28 00:00:00','military missile','success',268,'1-03','','','','Test',1752,'',15 from dual union all
select 49530,'1979-W940','1979-07-27 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 49531,'1979-W941','1979-07-27 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',61 from dual union all
select 49532,'1979-W942','1979-07-27 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 49533,'1979-W943','1979-07-28 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 49534,'1979-W944','1979-07-30 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49535,'1979-W945','1979-07-30 15:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 49536,'1979-W946','1979-07-30 15:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 49537,'1979-W947','1979-07-30 15:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 49538,'1979-W948','1979-07-30 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',60 from dual union all
select 49539,'1979-W949','1979-07-30 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 4954,'1958-A40','1958-05-01 15:45:00','military missile','success',255,'624-19','','','','OT',272,'',20 from dual union all
select 49540,'1979-W95','1979-01-17 16:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',60 from dual union all
select 49541,'1979-W950','1979-07-30 22:30:00','suborbital rocket','success',1061,'','Starute,Datasonde','','','Meteo',1047,'',67 from dual union all
select 49542,'1979-W951','1979-07-31 05:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 49543,'1979-W952','1979-08-01 02:00:00','suborbital rocket','success',761,'MT135P-R357','','','','Meteo',1332,'',62 from dual union all
select 49544,'1979-W953','1979-08-01 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 49545,'1979-W954','1979-08-01 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 49546,'1979-W955','1979-08-01 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49547,'1979-W956','1979-08-01 14:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',57 from dual union all
select 49548,'1979-W957','1979-08-01 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 49549,'1979-W958','1979-08-01 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 4955,'1958-A41','1958-05-06 00:00:00','military missile','success',1214,'N-69E N-3409','','','','Test',224,'',10 from dual union all
select 49550,'1979-W959','1979-08-01 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 49551,'1979-W96','1979-01-17 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1363,'',64 from dual union all
select 49552,'1979-W960','1979-08-01 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 49553,'1979-W961','1979-08-01 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 49554,'1979-W962','1979-08-01 21:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 49555,'1979-W963','1979-08-02 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 49556,'1979-W964','1979-08-02 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 49557,'1979-W965','1979-08-03 04:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 49558,'1979-W966','1979-08-03 07:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 49559,'1979-W967','1979-08-03 08:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 4956,'1958-A42','1958-05-15 16:02:00','military missile','success',262,'8','','','','Test',243,'',10 from dual union all
select 49560,'1979-W968','1979-08-03 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 49561,'1979-W969','1979-08-03 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 49562,'1979-W97','1979-01-17 19:32:59','suborbital rocket','success',716,'','Arcasonde','','','Meteo',428,'',63 from dual union all
select 49563,'1979-W970','1979-08-03 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 49564,'1979-W971','1979-08-03 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 49565,'1979-W972','1979-08-04 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 49566,'1979-W973','1979-08-06 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 49567,'1979-W974','1979-08-06 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 49568,'1979-W975','1979-08-06 14:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 49569,'1979-W976','1979-08-06 20:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 4957,'1958-A43','1958-05-16 00:00:00','atmospheric rocket','success',545,'','E41 Model','','','Test',1821,'',10 from dual union all
select 49570,'1979-W977','1979-08-06 21:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 49571,'1979-W978','1979-08-07 02:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 49572,'1979-W979','1979-08-07 16:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 49573,'1979-W98','1979-01-17 19:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 49574,'1979-W980','1979-08-08 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 49575,'1979-W981','1979-08-08 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 49576,'1979-W982','1979-08-08 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 49577,'1979-W983','1979-08-08 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',62 from dual union all
select 49578,'1979-W984','1979-08-08 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 49579,'1979-W985','1979-08-09 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 4958,'1958-A44','1958-05-20 00:00:00','military missile','success',255,'624-21','','','','OT',293,'',20 from dual union all
select 49580,'1979-W986','1979-08-09 20:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 49581,'1979-W987','1979-08-10 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 49582,'1979-W988','1979-08-10 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 49583,'1979-W989','1979-08-11 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 49584,'1979-W99','1979-01-17 20:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',72 from dual union all
select 49585,'1979-W990','1979-08-13 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 49586,'1979-W991','1979-08-13 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 49587,'1979-W992','1979-08-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',78 from dual union all
select 49588,'1979-W993','1979-08-13 15:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 49589,'1979-W994','1979-08-13 16:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',51 from dual union all
select 4959,'1958-A45','1958-05-22 00:00:00','military missile','success',268,'2-01','','','','Test',1752,'',17 from dual union all
select 49590,'1979-W995','1979-08-14 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 49591,'1979-W996','1979-08-14 19:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 49592,'1979-W997','1979-08-15 02:00:00','suborbital rocket','success',761,'MT135P-R358','','','','Meteo',1332,'',61 from dual union all
select 49593,'1979-W998','1979-08-15 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 49594,'1979-W999','1979-08-15 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 49595,'1979-Y01','1979-01-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 49596,'1979-Y02','1979-01-03 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 49597,'1979-Y03','1979-01-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 49598,'1979-Y04','1979-01-03 15:48:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 49599,'1979-Y05','1979-01-03 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual;
insert into LAUNCH
select 496,'1945-A15','1945-10-16 00:00:00','suborbital rocket','success',1516,'7','','','','Test',1852,'',27 from dual union all
select 4960,'1958-A46','1958-05-26 00:00:00','test rocket','success',253,'Missile 1','','','','Test',293,'B-47',8 from dual union all
select 49600,'1979-Y06','1979-01-03 19:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49601,'1979-Y07','1979-01-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 49602,'1979-Y08','1979-01-05 15:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',84 from dual union all
select 49603,'1979-Y09','1979-01-10 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 49604,'1979-Y10','1979-01-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 49605,'1979-Y100','1979-02-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 49606,'1979-Y101','1979-02-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 49607,'1979-Y102','1979-02-14 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 49608,'1979-Y103','1979-02-14 18:24:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 49609,'1979-Y104','1979-02-14 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 4961,'1958-A47','1958-05-26 03:41:00','suborbital rocket','success',661,'K-5-L','','','','Test',7,'',15 from dual union all
select 49610,'1979-Y105','1979-02-14 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 49611,'1979-Y106','1979-02-14 20:57:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 49612,'1979-Y107','1979-02-14 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',84 from dual union all
select 49613,'1979-Y108','1979-02-15 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',85 from dual union all
select 49614,'1979-Y109','1979-02-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 49615,'1979-Y11','1979-01-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 49616,'1979-Y110','1979-02-16 15:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 49617,'1979-Y111','1979-02-16 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 49618,'1979-Y112','1979-02-16 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',91 from dual union all
select 49619,'1979-Y113','1979-02-17 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',83 from dual union all
select 4962,'1958-A48','1958-05-27 03:45:00','suborbital rocket','success',642,'K150G','','','','Test',7,'',5 from dual union all
select 49620,'1979-Y114','1979-02-18 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',84 from dual union all
select 49621,'1979-Y115','1979-02-21 01:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 49622,'1979-Y116','1979-02-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 49623,'1979-Y117','1979-02-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 49624,'1979-Y118','1979-02-21 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 49625,'1979-Y119','1979-02-21 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 49626,'1979-Y12','1979-01-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 49627,'1979-Y120','1979-02-23 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',86 from dual union all
select 49628,'1979-Y121','1979-02-23 23:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 49629,'1979-Y122','1979-02-24 17:22:00','suborbital rocket','success',1281,'CMSA-10','Eclipse Prep','','','Aeronomy',1318,'',80 from dual union all
select 4963,'1958-A49','1958-05-28 00:00:00','military missile','success',1214,'N-69E N-3410','','','','Test',224,'',10 from dual union all
select 49630,'1979-Y123','1979-02-25 17:30:00','suborbital rocket','success',1281,'CMSA-05','Eclipse Prep','','','Aeronomy',1318,'',80 from dual union all
select 49631,'1979-Y124','1979-02-26 16:53:00','suborbital rocket','success',1281,'CMSA-06','Eclipse Gerdien','','','Aeronomy',1318,'',80 from dual union all
select 49632,'1979-Y125','1979-02-26 17:38:00','suborbital rocket','success',1281,'CMSA-07','Eclipse Gerdien','','','Aeronomy',1318,'',80 from dual union all
select 49633,'1979-Y126','1979-02-26 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',84 from dual union all
select 49634,'1979-Y127','1979-02-27 03:30:00','suborbital rocket','success',1281,'CMSA-08','Eclipse Gerdien','','','Aeronomy',1318,'',80 from dual union all
select 49635,'1979-Y128','1979-02-27 13:06:00','suborbital rocket','success',1281,'CMSA-09','Eclipse Gerdien','','','Aeronomy',1318,'',80 from dual union all
select 49636,'1979-Y129','1979-02-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 49637,'1979-Y13','1979-01-10 15:11:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 49638,'1979-Y130','1979-02-28 14:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 49639,'1979-Y131','1979-02-28 16:26:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 4964,'1958-A50','1958-06-05 00:00:00','atmospheric rocket','success',836,'','Control test','','','Test',1821,'',20 from dual union all
select 49640,'1979-Y132','1979-02-28 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49641,'1979-Y133','1979-03-01 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',83 from dual union all
select 49642,'1979-Y134','1979-03-01 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',81 from dual union all
select 49643,'1979-Y135','1979-03-02 21:24:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 49644,'1979-Y136','1979-03-07 01:45:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 49645,'1979-Y137','1979-03-07 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 49646,'1979-Y138','1979-03-07 17:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 49647,'1979-Y139','1979-03-07 19:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49648,'1979-Y14','1979-01-10 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',88 from dual union all
select 49649,'1979-Y140','1979-03-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 4965,'1958-A51','1958-06-09 00:00:00','military missile','success',255,'624-23','','','','OT',293,'',20 from dual union all
select 49650,'1979-Y141','1979-03-14 04:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 49651,'1979-Y142','1979-03-14 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49652,'1979-Y143','1979-03-14 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 49653,'1979-Y144','1979-03-14 15:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 49654,'1979-Y145','1979-03-14 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49655,'1979-Y146','1979-03-14 19:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 49656,'1979-Y147','1979-03-14 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',84 from dual union all
select 49657,'1979-Y148','1979-03-14 21:56:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',88 from dual union all
select 49658,'1979-Y149','1979-03-15 04:35:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 49659,'1979-Y15','1979-01-10 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 4966,'1958-A52','1958-06-11 00:00:00','military missile','failure',268,'3/1','','','','Test',1752,'',0 from dual union all
select 49660,'1979-Y150','1979-03-15 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 49661,'1979-Y151','1979-03-16 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 49662,'1979-Y152','1979-03-16 07:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 49663,'1979-Y153','1979-03-16 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 49664,'1979-Y154','1979-03-17 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 49665,'1979-Y155','1979-03-17 07:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 49666,'1979-Y156','1979-03-18 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 49667,'1979-Y157','1979-03-18 08:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 49668,'1979-Y158','1979-03-18 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 49669,'1979-Y159','1979-03-18 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',84 from dual union all
select 4967,'1958-A53','1958-06-14 01:11:00','suborbital rocket','success',643,'','','','','Test',7,'',1 from dual union all
select 49670,'1979-Y16','1979-01-10 20:51:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 49671,'1979-Y160','1979-03-18 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',87 from dual union all
select 49672,'1979-Y161','1979-03-19 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 49673,'1979-Y162','1979-03-19 07:25:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 49674,'1979-Y163','1979-03-19 11:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49675,'1979-Y164','1979-03-20 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 49676,'1979-Y165','1979-03-20 07:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 49677,'1979-Y166','1979-03-20 11:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 49678,'1979-Y167','1979-03-20 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 49679,'1979-Y168','1979-03-21 04:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 4968,'1958-A54','1958-06-16 02:36:00','suborbital rocket','unknown',662,'K-6-1','K-6-1','','','Test?',7,'',30 from dual union all
select 49680,'1979-Y169','1979-03-21 07:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 49681,'1979-Y17','1979-01-12 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 49682,'1979-Y170','1979-03-21 11:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49683,'1979-Y171','1979-03-21 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 49684,'1979-Y172','1979-03-21 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49685,'1979-Y173','1979-03-21 19:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 49686,'1979-Y174','1979-03-21 21:32:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 49687,'1979-Y175','1979-03-22 01:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 49688,'1979-Y176','1979-03-22 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 49689,'1979-Y177','1979-03-22 06:55:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual;
insert into LAUNCH
select 4969,'1958-A55','1958-06-18 10:35:00','suborbital rocket','failure',1194,'SL18','UCL Grenade/Chaff 3','','','Aeronomy',1839,'',3 from dual union all
select 49690,'1979-Y178','1979-03-22 11:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 49691,'1979-Y179','1979-03-23 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 49692,'1979-Y18','1979-01-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 49693,'1979-Y180','1979-03-23 06:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 49694,'1979-Y181','1979-03-24 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 49695,'1979-Y182','1979-03-24 06:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 49696,'1979-Y183','1979-03-24 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 49697,'1979-Y184','1979-03-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49698,'1979-Y185','1979-03-25 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 49699,'1979-Y186','1979-03-25 06:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 497,'1945-A16','1945-10-25 00:00:00','suborbital rocket','success',1516,'9','','','','Test',1852,'',20 from dual union all
select 4970,'1958-A56','1958-06-20 18:30:00','military missile','success',255,'624-24','','','','OT',293,'',20 from dual union all
select 49700,'1979-Y187','1979-03-26 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 49701,'1979-Y188','1979-03-26 05:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 49702,'1979-Y189','1979-03-26 09:55:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 49703,'1979-Y19','1979-01-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 49704,'1979-Y190','1979-03-26 11:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 49705,'1979-Y191','1979-03-27 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 49706,'1979-Y192','1979-03-27 06:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 49707,'1979-Y193','1979-03-27 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',86 from dual union all
select 49708,'1979-Y194','1979-03-27 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',85 from dual union all
select 49709,'1979-Y195','1979-03-28 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 4971,'1958-A57','1958-06-20 06:15:00','suborbital rocket','success',662,'K-6-2','K-6-2','','','Test',7,'',45 from dual union all
select 49710,'1979-Y196','1979-03-28 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 49711,'1979-Y197','1979-03-28 11:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49712,'1979-Y198','1979-03-28 16:09:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',89 from dual union all
select 49713,'1979-Y199','1979-03-28 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 49714,'1979-Y20','1979-01-13 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',87 from dual union all
select 49715,'1979-Y200','1979-03-28 19:09:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49716,'1979-Y201','1979-03-29 05:55:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 49717,'1979-Y202','1979-03-30 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 49718,'1979-Y203','1979-03-30 05:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 49719,'1979-Y204','1979-03-30 11:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 4972,'1958-A58','1958-06-24 01:51:00','suborbital rocket','success',662,'K-6-TW-1','K-6-5','S1','','Aeron',7,'',25 from dual union all
select 49720,'1979-Y205','1979-03-31 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 49721,'1979-Y206','1979-03-31 05:35:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 49722,'1979-Y207','1979-03-31 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',86 from dual union all
select 49723,'1979-Y208','1979-03-31 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',84 from dual union all
select 49724,'1979-Y209','1979-04-01 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 49725,'1979-Y21','1979-01-14 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',83 from dual union all
select 49726,'1979-Y210','1979-04-01 05:25:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 49727,'1979-Y211','1979-04-01 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49728,'1979-Y212','1979-04-02 05:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 49729,'1979-Y213','1979-04-03 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 4973,'1958-A59','1958-06-26 00:00:00','military missile','success',255,'624-20','','','','OT',293,'',20 from dual union all
select 49730,'1979-Y214','1979-04-03 05:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 49731,'1979-Y215','1979-04-03 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 49732,'1979-Y216','1979-04-04 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 49733,'1979-Y217','1979-04-04 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 49734,'1979-Y218','1979-04-04 15:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',86 from dual union all
select 49735,'1979-Y219','1979-04-04 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 49736,'1979-Y22','1979-01-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 49737,'1979-Y220','1979-04-04 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49738,'1979-Y221','1979-04-04 19:49:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 49739,'1979-Y222','1979-04-05 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 4974,'1958-A60','1958-06-27 00:00:00','atmospheric rocket','success',552,'','GE RV Model','','','Reentry Test',1821,'',30 from dual union all
select 49740,'1979-Y223','1979-04-05 04:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 49741,'1979-Y224','1979-04-05 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49742,'1979-Y225','1979-04-05 23:14:00','suborbital rocket','unknown',1281,'NASA 15.176GM  /T 1-','','','','Aeron',1821,'',81 from dual union all
select 49743,'1979-Y226','1979-04-06 04:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 49744,'1979-Y227','1979-04-06 04:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 49745,'1979-Y228','1979-04-06 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 49746,'1979-Y229','1979-04-07 03:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 49747,'1979-Y23','1979-01-16 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',87 from dual union all
select 49748,'1979-Y230','1979-04-07 04:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 49749,'1979-Y231','1979-04-07 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 4975,'1958-A61','1958-06-27 00:00:00','test rocket','failure',253,'Missile 2','','','','Test',293,'B-47',12 from dual union all
select 49750,'1979-Y232','1979-04-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 49751,'1979-Y233','1979-04-08 03:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 49752,'1979-Y234','1979-04-08 04:25:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 49753,'1979-Y235','1979-04-09 03:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 49754,'1979-Y236','1979-04-09 04:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 49755,'1979-Y237','1979-04-09 05:53:00','suborbital rocket','success',725,'','','Ozonesonde','','Aeron',1467,'',95 from dual union all
select 49756,'1979-Y238','1979-04-09 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49757,'1979-Y239','1979-04-10 03:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 49758,'1979-Y24','1979-01-17 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 49759,'1979-Y240','1979-04-10 03:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 4976,'1958-A62','1958-06-27 00:00:00','suborbital rocket','success',1163,'','Mercury-style RV','','','Reentry Test',1821,'',34 from dual union all
select 49760,'1979-Y241','1979-04-10 04:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 49761,'1979-Y242','1979-04-11 03:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 49762,'1979-Y243','1979-04-11 03:55:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 49763,'1979-Y244','1979-04-11 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49764,'1979-Y245','1979-04-11 15:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 49765,'1979-Y246','1979-04-11 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 49766,'1979-Y247','1979-04-12 03:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 49767,'1979-Y248','1979-04-14 12:01:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 49768,'1979-Y249','1979-04-17 21:31:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 49769,'1979-Y25','1979-01-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 4977,'1958-A63','1958-06-27 00:00:00','military missile','success',1214,'N-69E N-3413','','','','Test',238,'',10 from dual union all
select 49770,'1979-Y250','1979-04-18 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 49771,'1979-Y251','1979-04-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 49772,'1979-Y252','1979-04-18 15:20:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 49773,'1979-Y253','1979-04-18 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49774,'1979-Y254','1979-04-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 49775,'1979-Y255','1979-04-20 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 49776,'1979-Y256','1979-04-22 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',82 from dual union all
select 49777,'1979-Y257','1979-04-24 11:39:00','suborbital rocket','unknown',728,'','','','','Aeron',1376,'SHIR',84 from dual union all
select 49778,'1979-Y258','1979-04-24 20:10:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',84 from dual union all
select 49779,'1979-Y259','1979-04-25 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual;
insert into LAUNCH
select 4978,'1958-A64','1958-07-09 00:00:00','test rocket','failure',1521,'Winder 2','Argus test','','','Test',1338,'AVM1',10 from dual union all
select 49780,'1979-Y26','1979-01-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 49781,'1979-Y260','1979-04-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49782,'1979-Y261','1979-04-25 15:17:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 49783,'1979-Y262','1979-04-25 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 49784,'1979-Y263','1979-04-25 21:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49785,'1979-Y264','1979-04-26 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',84 from dual union all
select 49786,'1979-Y265','1979-04-27 12:22:00','suborbital rocket','unknown',728,'','','','','Aeron',1376,'SHIR',84 from dual union all
select 49787,'1979-Y266','1979-04-29 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',85 from dual union all
select 49788,'1979-Y267','1979-05-01 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',85 from dual union all
select 49789,'1979-Y268','1979-05-02 03:25:00','suborbital rocket','unknown',728,'','','','','Aeron',1376,'SHIR',90 from dual union all
select 4979,'1958-A65','1958-07-17 00:00:00','test rocket','failure',1521,'Winder 3','Argus test','','','Test',1338,'AVM1',0 from dual union all
select 49790,'1979-Y269','1979-05-02 11:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 49791,'1979-Y27','1979-01-17 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 49792,'1979-Y270','1979-05-02 14:09:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 49793,'1979-Y271','1979-05-02 17:31:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 49794,'1979-Y272','1979-05-02 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 49795,'1979-Y273','1979-05-02 19:31:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49796,'1979-Y274','1979-05-02 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',82 from dual union all
select 49797,'1979-Y275','1979-05-03 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 49798,'1979-Y276','1979-05-03 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 49799,'1979-Y277','1979-05-03 22:02:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',86 from dual union all
select 498,'1945-A17','1945-10-25 00:00:00','suborbital rocket','success',1516,'10','','','','Test',1852,'',20 from dual union all
select 4980,'1958-A66','1958-06-30 07:52:00','suborbital rocket','success',662,'K-6-TW-2','K-6-6','S2','','Aeron',7,'',45 from dual union all
select 49800,'1979-Y278','1979-05-09 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 49801,'1979-Y279','1979-05-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 49802,'1979-Y28','1979-01-17 16:31:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',89 from dual union all
select 49803,'1979-Y280','1979-05-09 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 49804,'1979-Y281','1979-05-09 17:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 49805,'1979-Y282','1979-05-09 18:52:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 49806,'1979-Y283','1979-05-10 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 49807,'1979-Y284','1979-05-10 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 49808,'1979-Y285','1979-05-13 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',83 from dual union all
select 49809,'1979-Y286','1979-05-16 00:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 4981,'1958-A67','1958-07-03 00:00:00','military missile','success',268,'3/1','','','','Test',1752,'',15 from dual union all
select 49810,'1979-Y287','1979-05-16 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 49811,'1979-Y288','1979-05-16 09:45:00','suborbital rocket','success',725,'','','Ozonesonde','','Aeron',1467,'',94 from dual union all
select 49812,'1979-Y289','1979-05-16 13:24:00','suborbital rocket','unknown',728,'','','','','Aeron',1376,'SHIR',82 from dual union all
select 49813,'1979-Y29','1979-01-17 19:21:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 49814,'1979-Y290','1979-05-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 49815,'1979-Y291','1979-05-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',88 from dual union all
select 49816,'1979-Y292','1979-05-16 19:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 49817,'1979-Y293','1979-05-16 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',82 from dual union all
select 49818,'1979-Y294','1979-05-18 00:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 49819,'1979-Y295','1979-05-18 13:24:00','suborbital rocket','unknown',728,'','','','','Aeron',1376,'SHIR',83 from dual union all
select 4982,'1958-A68','1958-07-04 00:00:00','suborbital rocket','failure',945,'','','','','Test',996,'',0 from dual union all
select 49820,'1979-Y296','1979-05-18 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 49821,'1979-Y297','1979-05-18 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',84 from dual union all
select 49822,'1979-Y298','1979-05-19 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',88 from dual union all
select 49823,'1979-Y299','1979-05-21 13:25:00','suborbital rocket','unknown',728,'','','','','Aeron',1376,'SHIR',84 from dual union all
select 49824,'1979-Y30','1979-01-17 19:54:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',90 from dual union all
select 49825,'1979-Y300','1979-05-21 22:00:00','suborbital rocket','unknown',1284,'','','','','Aeron',1822,'KORO',84 from dual union all
select 49826,'1979-Y301','1979-05-21 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',84 from dual union all
select 49827,'1979-Y302','1979-05-21 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',84 from dual union all
select 49828,'1979-Y303','1979-05-22 22:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 49829,'1979-Y304','1979-05-23 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 4983,'1958-A69','1958-07-10 07:42:00','military missile','failure',1016,'B1-4','8K71 LKI II-5','','','Test',480,'',40 from dual union all
select 49830,'1979-Y305','1979-05-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 49831,'1979-Y306','1979-05-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 49832,'1979-Y307','1979-05-23 15:21:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 49833,'1979-Y308','1979-05-23 16:52:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 49834,'1979-Y309','1979-05-23 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',89 from dual union all
select 49835,'1979-Y31','1979-01-17 21:01:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 49836,'1979-Y310','1979-05-24 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',85 from dual union all
select 49837,'1979-Y311','1979-05-25 13:23:00','suborbital rocket','unknown',728,'','','','','Aeron',1376,'SHIR',85 from dual union all
select 49838,'1979-Y312','1979-05-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 49839,'1979-Y313','1979-05-26 00:40:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',84 from dual union all
select 4984,'1958-A70','1958-07-11 18:00:00','military missile','success',255,'624-25','','','','OT',293,'',20 from dual union all
select 49840,'1979-Y314','1979-05-26 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',89 from dual union all
select 49841,'1979-Y315','1979-05-28 02:11:00','suborbital rocket','unknown',728,'','','','','Aeron',1376,'SHIR',83 from dual union all
select 49842,'1979-Y316','1979-05-29 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 49843,'1979-Y317','1979-05-29 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',888,'KORO',81 from dual union all
select 49844,'1979-Y318','1979-05-30 14:12:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 49845,'1979-Y319','1979-05-30 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 49846,'1979-Y32','1979-01-18 15:50:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',80 from dual union all
select 49847,'1979-Y320','1979-05-30 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 49848,'1979-Y321','1979-05-30 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49849,'1979-Y322','1979-05-30 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',80 from dual union all
select 4985,'1958-A71','1958-07-11 00:00:00','suborbital rocket','failure',623,'J-A','','','','Test',1821,'',20 from dual union all
select 49850,'1979-Y323','1979-05-30 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 49851,'1979-Y324','1979-06-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49852,'1979-Y325','1979-06-02 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',86 from dual union all
select 49853,'1979-Y326','1979-06-04 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 49854,'1979-Y327','1979-06-05 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 49855,'1979-Y328','1979-06-05 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 49856,'1979-Y329','1979-06-06 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 49857,'1979-Y33','1979-01-19 03:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 49858,'1979-Y330','1979-06-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 49859,'1979-Y331','1979-06-06 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',86 from dual union all
select 4986,'1958-A72','1958-07-13 00:00:00','military missile','success',268,'3/2','','','','Test',1752,'',15 from dual union all
select 49860,'1979-Y332','1979-06-06 16:51:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 49861,'1979-Y333','1979-06-08 11:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 49862,'1979-Y334','1979-06-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 49863,'1979-Y335','1979-06-08 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',86 from dual union all
select 49864,'1979-Y336','1979-06-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 49865,'1979-Y337','1979-06-13 10:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 49866,'1979-Y338','1979-06-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 49867,'1979-Y339','1979-06-13 14:32:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 49868,'1979-Y34','1979-01-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 49869,'1979-Y340','1979-06-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',89 from dual;
insert into LAUNCH
select 4987,'1958-A73','1958-07-15 00:00:00','atmospheric rocket','success',1052,'','','','','Test',1821,'B-57A',14 from dual union all
select 49870,'1979-Y341','1979-06-13 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49871,'1979-Y342','1979-06-13 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 49872,'1979-Y343','1979-06-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 49873,'1979-Y344','1979-06-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 49874,'1979-Y345','1979-06-20 02:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 49875,'1979-Y346','1979-06-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 49876,'1979-Y347','1979-06-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 49877,'1979-Y348','1979-06-20 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',86 from dual union all
select 49878,'1979-Y349','1979-06-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 49879,'1979-Y35','1979-01-19 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',81 from dual union all
select 4988,'1958-A74','1958-07-16 00:00:00','suborbital rocket','failure',89,'NRL-1','','','','Test',1821,'',1 from dual union all
select 49880,'1979-Y350','1979-06-20 17:52:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 49881,'1979-Y351','1979-06-20 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 49882,'1979-Y352','1979-06-20 21:02:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49883,'1979-Y353','1979-06-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 49884,'1979-Y354','1979-06-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',88 from dual union all
select 49885,'1979-Y355','1979-06-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 49886,'1979-Y356','1979-06-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 49887,'1979-Y357','1979-06-24 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',889,'KORO',82 from dual union all
select 49888,'1979-Y358','1979-06-25 12:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',88 from dual union all
select 49889,'1979-Y359','1979-06-26 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',889,'KORO',92 from dual union all
select 4989,'1958-A75','1958-07-17 00:00:00','suborbital rocket','failure',623,'J-B','','','','Test',1821,'',20 from dual union all
select 49890,'1979-Y36','1979-01-19 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 49891,'1979-Y360','1979-06-26 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 49892,'1979-Y361','1979-06-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 49893,'1979-Y362','1979-06-27 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',84 from dual union all
select 49894,'1979-Y363','1979-06-27 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',89 from dual union all
select 49895,'1979-Y364','1979-06-27 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',89 from dual union all
select 49896,'1979-Y365','1979-06-27 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 49897,'1979-Y366','1979-06-27 18:56:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 49898,'1979-Y367','1979-06-27 19:49:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49899,'1979-Y368','1979-06-29 13:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',89 from dual union all
select 499,'1945-A18','1945-04-01 00:00:00','military missile','success',944,'1','Dummy sustainer','','','Test',435,'',1 from dual union all
select 4990,'1958-A76','1958-07-18 00:00:00','suborbital rocket','failure',945,'','','','','Test',996,'',0 from dual union all
select 49900,'1979-Y369','1979-06-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 49901,'1979-Y37','1979-01-20 15:25:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',80 from dual union all
select 49902,'1979-Y370','1979-06-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 49903,'1979-Y371','1979-06-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 49904,'1979-Y372','1979-07-03 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 49905,'1979-Y373','1979-07-04 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 49906,'1979-Y374','1979-07-04 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 49907,'1979-Y375','1979-07-04 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',89 from dual union all
select 49908,'1979-Y376','1979-07-05 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 49909,'1979-Y377','1979-07-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 4991,'1958-A77','1958-07-19 17:36:00','military missile','failure',147,'3B','Mk 2-1 RV','R&D','','Test',226,'',10 from dual union all
select 49910,'1979-Y378','1979-07-06 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',95 from dual union all
select 49911,'1979-Y379','1979-07-06 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',87 from dual union all
select 49912,'1979-Y38','1979-01-21 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',82 from dual union all
select 49913,'1979-Y380','1979-07-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 49914,'1979-Y381','1979-07-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',84 from dual union all
select 49915,'1979-Y382','1979-07-09 14:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',90 from dual union all
select 49916,'1979-Y383','1979-07-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',86 from dual union all
select 49917,'1979-Y384','1979-07-11 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49918,'1979-Y385','1979-07-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 49919,'1979-Y386','1979-07-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',90 from dual union all
select 4992,'1958-A78','1958-07-21 00:00:00','suborbital rocket','failure',89,'SEL-1','','','','Test',1821,'',1 from dual union all
select 49920,'1979-Y387','1979-07-11 16:35:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 49921,'1979-Y388','1979-07-11 17:53:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 49922,'1979-Y389','1979-07-11 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 49923,'1979-Y39','1979-01-22 13:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',86 from dual union all
select 49924,'1979-Y390','1979-07-12 20:56:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 49925,'1979-Y391','1979-07-13 13:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',85 from dual union all
select 49926,'1979-Y392','1979-07-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 49927,'1979-Y393','1979-07-14 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',84 from dual union all
select 49928,'1979-Y394','1979-07-16 12:13:00','suborbital rocket','success',1281,'NASA 15.194GM','','','','Aeronomy',1047,'',88 from dual union all
select 49929,'1979-Y395','1979-07-17 12:23:00','suborbital rocket','success',1281,'NASA 15.195GM','','','','Aeronomy',1047,'',81 from dual union all
select 4993,'1958-A79','1958-07-22 00:00:00','suborbital rocket','failure',89,'','','','','Test',1821,'',20 from dual union all
select 49930,'1979-Y396','1979-07-18 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 49931,'1979-Y397','1979-07-18 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 49932,'1979-Y398','1979-07-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 49933,'1979-Y399','1979-07-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 49934,'1979-Y40','1979-01-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 49935,'1979-Y400','1979-07-18 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',84 from dual union all
select 49936,'1979-Y401','1979-07-18 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49937,'1979-Y402','1979-07-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 49938,'1979-Y403','1979-07-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49939,'1979-Y404','1979-07-22 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',81 from dual union all
select 4994,'1958-A80','1958-07-24 00:00:00','military missile','success',262,'12','','','','Test',240,'',10 from dual union all
select 49940,'1979-Y405','1979-07-22 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',82 from dual union all
select 49941,'1979-Y406','1979-07-22 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',94 from dual union all
select 49942,'1979-Y407','1979-07-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 49943,'1979-Y408','1979-07-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 49944,'1979-Y409','1979-07-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 49945,'1979-Y41','1979-01-22 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',88 from dual union all
select 49946,'1979-Y410','1979-07-25 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 49947,'1979-Y411','1979-07-25 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49948,'1979-Y412','1979-07-25 23:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 49949,'1979-Y413','1979-07-27 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',85 from dual union all
select 4995,'1958-A81','1958-07-24 00:00:00','atmospheric rocket','success',1052,'','','','','Test',1821,'B-57A',14 from dual union all
select 49950,'1979-Y414','1979-07-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 49951,'1979-Y415','1979-07-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',87 from dual union all
select 49952,'1979-Y416','1979-07-27 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 49953,'1979-Y417','1979-08-01 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 49954,'1979-Y418','1979-08-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 49955,'1979-Y419','1979-08-01 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 49956,'1979-Y42','1979-01-23 15:10:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',85 from dual union all
select 49957,'1979-Y420','1979-08-01 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 49958,'1979-Y421','1979-08-01 20:57:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',87 from dual union all
select 49959,'1979-Y422','1979-08-02 01:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual;
insert into LAUNCH
select 4996,'1958-A82','1958-07-26 06:40:00','military missile','failure',1366,'126','R&D Series III','','','Test',234,'',0 from dual union all
select 49960,'1979-Y423','1979-08-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 49961,'1979-Y424','1979-08-03 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',93 from dual union all
select 49962,'1979-Y425','1979-08-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 49963,'1979-Y426','1979-08-08 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 49964,'1979-Y427','1979-08-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 49965,'1979-Y428','1979-08-08 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49966,'1979-Y429','1979-08-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',92 from dual union all
select 49967,'1979-Y43','1979-01-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 49968,'1979-Y430','1979-08-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 49969,'1979-Y431','1979-08-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',85 from dual union all
select 4997,'1958-A83','1958-08-01 09:50:00','suborbital rocket','success',439,'','','','','Aeron',803,'',24 from dual union all
select 49970,'1979-Y432','1979-08-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 49971,'1979-Y433','1979-08-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 49972,'1979-Y434','1979-08-14 22:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 49973,'1979-Y435','1979-08-15 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 49974,'1979-Y436','1979-08-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 49975,'1979-Y437','1979-08-15 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',89 from dual union all
select 49976,'1979-Y438','1979-08-15 17:54:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 49977,'1979-Y439','1979-08-16 20:54:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',86 from dual union all
select 49978,'1979-Y44','1979-01-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 49979,'1979-Y440','1979-08-17 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',88 from dual union all
select 4998,'1958-A84','1958-08-01 09:50:00','suborbital rocket','success',439,'','','','','Aeron',803,'',24 from dual union all
select 49980,'1979-Y441','1979-08-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',87 from dual union all
select 49981,'1979-Y442','1979-08-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',889,'KORO',82 from dual union all
select 49982,'1979-Y443','1979-08-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',85 from dual union all
select 49983,'1979-Y444','1979-08-22 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 49984,'1979-Y445','1979-08-22 04:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 49985,'1979-Y446','1979-08-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 49986,'1979-Y447','1979-08-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 49987,'1979-Y448','1979-08-23 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',87 from dual union all
select 49988,'1979-Y449','1979-08-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',85 from dual union all
select 49989,'1979-Y45','1979-01-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 4999,'1958-A85','1958-08-01 09:50:00','suborbital rocket','success',439,'','','','','Aeron',803,'',24 from dual union all
select 49990,'1979-Y450','1979-08-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 49991,'1979-Y451','1979-08-25 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',86 from dual union all
select 49992,'1979-Y452','1979-08-28 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',85 from dual union all
select 49993,'1979-Y453','1979-08-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1398,'SHOK',85 from dual union all
select 49994,'1979-Y454','1979-08-29 00:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 49995,'1979-Y455','1979-08-29 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 49996,'1979-Y456','1979-08-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 49997,'1979-Y457','1979-08-29 14:17:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 49998,'1979-Y458','1979-08-29 14:40:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',94 from dual union all
select 49999,'1979-Y459','1979-08-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 5,'1942-A05','1942-12-12 00:00:00','military missile','failure',8,'9','','','','Test',675,'',0 from dual union all
select 50,'1944-A14','1944-09-02 15:45:00','military missile','success',8,'18293','Ma93','','','Test',676,'',35 from dual union all
select 500,'1945-A19','1945-04-01 00:00:00','military missile','success',944,'2','Dummy sustainer','','','Test',435,'',1 from dual union all
select 5000,'1958-A86','1958-08-01 09:50:00','suborbital rocket','success',439,'','','','','Aeron',803,'',24 from dual union all
select 50000,'1979-Y46','1979-01-24 15:35:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',83 from dual union all
select 50001,'1979-Y460','1979-08-29 22:27:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 50002,'1979-Y461','1979-09-04 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 50003,'1979-Y462','1979-09-05 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 50004,'1979-Y463','1979-09-05 14:08:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 50005,'1979-Y464','1979-09-05 16:26:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 50006,'1979-Y465','1979-09-05 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 50007,'1979-Y466','1979-09-06 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 50008,'1979-Y467','1979-09-07 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',95 from dual union all
select 50009,'1979-Y468','1979-09-10 17:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 5001,'1958-A87','1958-08-07 00:00:00','military missile','success',255,'624-XY1','','','','OT',272,'',20 from dual union all
select 50010,'1979-Y469','1979-09-11 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 50011,'1979-Y47','1979-01-24 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 50012,'1979-Y470','1979-09-12 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 50013,'1979-Y471','1979-09-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 50014,'1979-Y472','1979-09-12 14:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 50015,'1979-Y473','1979-09-12 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',84 from dual union all
select 50016,'1979-Y474','1979-09-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 50017,'1979-Y475','1979-09-12 17:56:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 50018,'1979-Y476','1979-09-13 00:00:00','suborbital rocket','success',1281,'NASA 15.193UE','','','','Plasma',1821,'',80 from dual union all
select 50019,'1979-Y477','1979-09-13 00:00:00','suborbital rocket','success',1281,'NASA 15.188UE','','','','Plasma',1821,'',80 from dual union all
select 5002,'1958-A88','1958-08-11 09:30:00','suborbital rocket','success',439,'','','','','Aeron',803,'',24 from dual union all
select 50020,'1979-Y478','1979-09-14 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 50021,'1979-Y479','1979-09-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 50022,'1979-Y48','1979-01-24 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 50023,'1979-Y480','1979-09-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 50024,'1979-Y481','1979-09-19 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 50025,'1979-Y482','1979-09-19 18:38:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 50026,'1979-Y483','1979-09-19 22:15:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',88 from dual union all
select 50027,'1979-Y484','1979-09-25 22:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 50028,'1979-Y485','1979-09-26 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 50029,'1979-Y486','1979-09-26 14:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 5003,'1958-A89','1958-08-12 09:27:00','military missile','success',1053,'CC-51    SH','Hardtack Orange','','','Nuclear Test',771,'',41 from dual union all
select 50030,'1979-Y487','1979-09-28 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',90 from dual union all
select 50031,'1979-Y488','1979-09-29 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',82 from dual union all
select 50032,'1979-Y489','1979-10-02 22:45:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 50033,'1979-Y49','1979-01-25 14:55:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',80 from dual union all
select 50034,'1979-Y490','1979-10-03 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 50035,'1979-Y491','1979-10-03 14:08:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 50036,'1979-Y492','1979-10-03 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 50037,'1979-Y493','1979-10-09 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 50038,'1979-Y494','1979-10-10 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 50039,'1979-Y495','1979-10-10 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',95 from dual union all
select 5004,'1958-A90','1958-08-14 00:00:00','suborbital rocket','success',716,'','Oriole','','','Aeron/Test',1821,'',30 from dual union all
select 50040,'1979-Y496','1979-10-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 50041,'1979-Y497','1979-10-17 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 50042,'1979-Y498','1979-10-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 50043,'1979-Y499','1979-10-17 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 50044,'1979-Y50','1979-01-25 20:33:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',87 from dual union all
select 50045,'1979-Y500','1979-10-17 20:43:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 50046,'1979-Y501','1979-10-21 16:30:00','suborbital rocket','success',1281,'NASA 15.204GM','','','','Aeronomy',1821,'',82 from dual union all
select 50047,'1979-Y502','1979-10-21 18:51:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual;
insert into LAUNCH
select 50048,'1979-Y503','1979-10-23 22:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 50049,'1979-Y504','1979-10-24 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 5005,'1958-A91','1958-08-14 00:00:00','suborbital rocket','success',716,'','Oriole','','','Aeron/Test',1821,'',30 from dual union all
select 50050,'1979-Y505','1979-10-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 50051,'1979-Y506','1979-10-24 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',89 from dual union all
select 50052,'1979-Y507','1979-10-25 14:12:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 50053,'1979-Y508','1979-10-30 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 50054,'1979-Y509','1979-10-31 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 50055,'1979-Y51','1979-01-26 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',84 from dual union all
select 50056,'1979-Y510','1979-10-31 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 50057,'1979-Y511','1979-10-31 17:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 50058,'1979-Y512','1979-10-31 18:37:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 50059,'1979-Y513','1979-11-01 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 5006,'1958-A92','1958-08-14 00:00:00','suborbital rocket','success',716,'','Oriole','','','Aeron/Test',1821,'',30 from dual union all
select 50060,'1979-Y514','1979-11-02 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 50061,'1979-Y515','1979-11-02 04:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 50062,'1979-Y516','1979-11-02 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',84 from dual union all
select 50063,'1979-Y517','1979-11-02 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',86 from dual union all
select 50064,'1979-Y518','1979-11-04 09:25:00','suborbital rocket','success',1281,'NASA 15.208GM','','','','Aeronomy',1821,'',82 from dual union all
select 50065,'1979-Y519','1979-11-04 10:00:00','suborbital rocket','success',1281,'NASA 15.207GM','','','','Aeronomy',1821,'',81 from dual union all
select 50066,'1979-Y52','1979-01-27 00:00:00','suborbital rocket','success',1281,'NASA 15.198GM','','','','Aeronomy',1821,'',80 from dual union all
select 50067,'1979-Y520','1979-11-05 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',82 from dual union all
select 50068,'1979-Y521','1979-11-05 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',81 from dual union all
select 50069,'1979-Y522','1979-11-05 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 5007,'1958-A93','1958-08-15 00:00:00','military missile','success',255,'624-XY2','','','','OT',293,'',20 from dual union all
select 50070,'1979-Y523','1979-11-07 04:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 50071,'1979-Y524','1979-11-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 50072,'1979-Y525','1979-11-07 15:56:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 50073,'1979-Y526','1979-11-08 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',81 from dual union all
select 50074,'1979-Y527','1979-11-08 19:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 50075,'1979-Y528','1979-11-09 20:25:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 50076,'1979-Y529','1979-11-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',86 from dual union all
select 50077,'1979-Y53','1979-01-27 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 50078,'1979-Y530','1979-11-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',85 from dual union all
select 50079,'1979-Y531','1979-11-11 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',84 from dual union all
select 5008,'1958-A94','1958-08-15 00:00:00','suborbital rocket','success',716,'','Oriole','','','Aeron/Test',1821,'',30 from dual union all
select 50080,'1979-Y532','1979-11-11 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',86 from dual union all
select 50081,'1979-Y533','1979-11-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',80 from dual union all
select 50082,'1979-Y534','1979-11-12 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',85 from dual union all
select 50083,'1979-Y535','1979-11-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',82 from dual union all
select 50084,'1979-Y536','1979-11-13 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',88 from dual union all
select 50085,'1979-Y537','1979-11-13 17:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',85 from dual union all
select 50086,'1979-Y538','1979-11-14 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 50087,'1979-Y539','1979-11-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 50088,'1979-Y54','1979-01-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 50089,'1979-Y540','1979-11-14 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',82 from dual union all
select 5009,'1958-A95','1958-08-15 00:00:00','suborbital rocket','success',716,'','Oriole','','','Aeron/Test',1821,'',30 from dual union all
select 50090,'1979-Y541','1979-11-14 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 50091,'1979-Y542','1979-11-14 17:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',89 from dual union all
select 50092,'1979-Y543','1979-11-14 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',81 from dual union all
select 50093,'1979-Y544','1979-11-15 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',80 from dual union all
select 50094,'1979-Y545','1979-11-16 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 50095,'1979-Y546','1979-11-16 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',906,'KRENKEL''',87 from dual union all
select 50096,'1979-Y547','1979-11-20 23:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 50097,'1979-Y548','1979-11-21 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 50098,'1979-Y549','1979-11-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 50099,'1979-Y55','1979-01-27 14:55:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',81 from dual union all
select 501,'1945-A20','1945-04-01 00:00:00','military missile','success',944,'3','','','','Test',435,'',1 from dual union all
select 5010,'1958-A96','1958-08-15 00:00:00','suborbital rocket','success',716,'','Oriole','','','Aeron/Test',1821,'',30 from dual union all
select 50100,'1979-Y550','1979-11-21 18:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 50101,'1979-Y551','1979-11-21 19:03:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 50102,'1979-Y552','1979-11-23 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 50103,'1979-Y553','1979-11-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',81 from dual union all
select 50104,'1979-Y554','1979-11-27 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',82 from dual union all
select 50105,'1979-Y555','1979-11-27 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 50106,'1979-Y556','1979-11-28 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 50107,'1979-Y557','1979-11-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 50108,'1979-Y558','1979-11-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',91 from dual union all
select 50109,'1979-Y559','1979-11-28 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 5011,'1958-A97','1958-08-16 00:00:00','suborbital rocket','failure',945,'','','','','Test',996,'',0 from dual union all
select 50110,'1979-Y56','1979-01-27 15:25:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 50111,'1979-Y560','1979-11-28 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 50112,'1979-Y561','1979-11-28 21:24:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 50113,'1979-Y562','1979-11-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',84 from dual union all
select 50114,'1979-Y563','1979-11-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',85 from dual union all
select 50115,'1979-Y564','1979-11-29 16:03:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 50116,'1979-Y565','1979-11-30 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 50117,'1979-Y566','1979-11-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',82 from dual union all
select 50118,'1979-Y567','1979-12-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',82 from dual union all
select 50119,'1979-Y568','1979-12-04 15:37:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 5012,'1958-A98','1958-08-17 00:00:00','suborbital rocket','failure',945,'','','','','Test',996,'',0 from dual union all
select 50120,'1979-Y569','1979-12-05 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 50121,'1979-Y57','1979-01-28 00:00:00','suborbital rocket','success',1281,'NASA 15.199GM','','','','Aeronomy',1821,'',80 from dual union all
select 50122,'1979-Y570','1979-12-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 50123,'1979-Y571','1979-12-05 17:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 50124,'1979-Y572','1979-12-05 18:45:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 50125,'1979-Y573','1979-12-05 20:26:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 50126,'1979-Y574','1979-12-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',89 from dual union all
select 50127,'1979-Y575','1979-12-07 01:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',82 from dual union all
select 50128,'1979-Y576','1979-12-07 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 50129,'1979-Y577','1979-12-10 19:00:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 5013,'1958-A99','1958-08-21 16:15:00','military missile','success',255,'624-22','','','','OT',293,'',20 from dual union all
select 50130,'1979-Y578','1979-12-11 11:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',82 from dual union all
select 50131,'1979-Y579','1979-12-12 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 50132,'1979-Y58','1979-01-28 15:35:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',84 from dual union all
select 50133,'1979-Y580','1979-12-12 11:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',89 from dual union all
select 50134,'1979-Y581','1979-12-12 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 50135,'1979-Y582','1979-12-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',85 from dual union all
select 50136,'1979-Y583','1979-12-12 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',85 from dual union all
select 50137,'1979-Y584','1979-12-12 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual;
insert into LAUNCH
select 50138,'1979-Y585','1979-12-13 21:49:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',96 from dual union all
select 50139,'1979-Y586','1979-12-14 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 5014,'1958-F01','1958-02-05 07:33:00','orbital','failure',1486,'TV-3BU','Vanguard TV3BU','Vanguard Test Satellite','','Sat',235,'',6 from dual union all
select 50140,'1979-Y587','1979-12-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 50141,'1979-Y588','1979-12-15 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',81 from dual union all
select 50142,'1979-Y589','1979-12-19 02:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 50143,'1979-Y59','1979-01-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 50144,'1979-Y590','1979-12-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',89 from dual union all
select 50145,'1979-Y591','1979-12-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 50146,'1979-Y592','1979-12-19 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 50147,'1979-Y593','1979-12-19 18:51:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 50148,'1979-Y594','1979-12-19 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',83 from dual union all
select 50149,'1979-Y595','1979-12-19 19:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 5015,'1958-F02','1958-03-05 18:27:57','orbital','failure',636,'RS/CC-26 UV','Explorer 2','Explorer 2','','Sat',248,'',379 from dual union all
select 50150,'1979-Y596','1979-12-19 20:57:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 50151,'1979-Y597','1979-12-20 14:52:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',88 from dual union all
select 50152,'1979-Y598','1979-12-21 02:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 50153,'1979-Y599','1979-12-26 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 50154,'1979-Y60','1979-01-29 15:35:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',82 from dual union all
select 50155,'1979-Y600','1979-12-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 50156,'1979-Y601','1979-12-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',89 from dual union all
select 50157,'1979-Y602','1979-12-26 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',84 from dual union all
select 50158,'1979-Y603','1979-12-26 17:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 50159,'1979-Y604','1979-12-28 02:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 5016,'1958-F03','1958-04-27 09:01:00','orbital','failure',1254,'B1-2','ISZ D-1 No. 1','D-1 No. 1','','Sat',480,'',30 from dual union all
select 50160,'1979-Y605','1979-12-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 50161,'1979-Y606','1979-02-24 16:54:50','suborbital rocket','success',1281,'CMSA-1','Eclipse Ne','','','Ionos',1318,'',80 from dual union all
select 50162,'1979-Y607','1979-02-19 20:38:00','suborbital rocket','success',1284,'','Eclipse pre Sonde','','','Meteo',1318,'',80 from dual union all
select 50163,'1979-Y608','1979-02-23 18:00:00','suborbital rocket','success',1284,'','Eclipse pre Sonde','','','Meteo',1318,'',80 from dual union all
select 50164,'1979-Y609','1979-02-24 15:51:00','suborbital rocket','success',1284,'','Eclipse pre Sonde','','','Meteo',1318,'',80 from dual union all
select 50165,'1979-Y61','1979-01-30 17:55:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',82 from dual union all
select 50166,'1979-Y610','1979-02-25 17:00:00','suborbital rocket','success',1281,'','Eclipse Ne','','','Aeronomy',1318,'',80 from dual union all
select 50167,'1979-Y611','1979-02-25 18:30:00','suborbital rocket','success',1284,'','Eclipse Sonde','','','Meteo',1318,'',80 from dual union all
select 50168,'1979-Y612','1979-02-26 18:40:00','suborbital rocket','success',1281,'','Eclipse Ne','','','Aeronomy',1318,'',80 from dual union all
select 50169,'1979-Y613','1979-02-26 19:15:00','suborbital rocket','success',1284,'','Eclipse Sonde','','','Meteo',1318,'',80 from dual union all
select 5017,'1958-F04','1958-04-29 02:53:00','orbital','failure',1486,'TV-5','Vanguard TV5','X-ray satellite','','Sat',235,'',550 from dual union all
select 50170,'1979-Y614','1979-02-27 05:30:00','suborbital rocket','success',1284,'','Eclipse Met Probe','','','Aeron',1317,'',75 from dual union all
select 50171,'1979-Y615','1979-02-27 15:45:00','suborbital rocket','success',1284,'','Eclipse Met Probe','','','Aeron',1317,'',80 from dual union all
select 50172,'1979-Y616','1979-02-27 14:10:00','suborbital rocket','success',1281,'','Eclipse Ne','','','Aeronomy',1318,'',80 from dual union all
select 50173,'1979-Y62','1979-01-31 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 50174,'1979-Y63','1979-01-31 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 50175,'1979-Y64','1979-01-31 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 50176,'1979-Y65','1979-01-31 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 50177,'1979-Y66','1979-01-31 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',83 from dual union all
select 50178,'1979-Y67','1979-01-31 15:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 50179,'1979-Y68','1979-01-31 16:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 5018,'1958-F05','1958-05-28 03:46:20','orbital','failure',1486,'SLV-1','Vanguard SLV1','Lyman Alpha satellite','','Sat',235,'',2500 from dual union all
select 50180,'1979-Y69','1979-01-31 20:56:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',88 from dual union all
select 50181,'1979-Y70','1979-02-01 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',82 from dual union all
select 50182,'1979-Y71','1979-02-02 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 50183,'1979-Y72','1979-02-02 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 50184,'1979-Y73','1979-02-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 50185,'1979-Y74','1979-02-02 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',81 from dual union all
select 50186,'1979-Y75','1979-02-03 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',81 from dual union all
select 50187,'1979-Y76','1979-02-04 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',86 from dual union all
select 50188,'1979-Y77','1979-02-04 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',83 from dual union all
select 50189,'1979-Y78','1979-02-05 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',86 from dual union all
select 5019,'1958-F06','1958-06-26 05:00:52','orbital','failure',1486,'SLV-2','Vanguard SLV2','X-ray satellite','','Sat',235,'',165 from dual union all
select 50190,'1979-Y79','1979-02-05 21:20:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',81 from dual union all
select 50191,'1979-Y80','1979-02-06 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',81 from dual union all
select 50192,'1979-Y81','1979-02-07 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 50193,'1979-Y82','1979-02-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 50194,'1979-Y83','1979-02-07 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 50195,'1979-Y84','1979-02-07 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 50196,'1979-Y85','1979-02-07 15:49:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',82 from dual union all
select 50197,'1979-Y86','1979-02-07 19:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 50198,'1979-Y87','1979-02-09 03:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 50199,'1979-Y88','1979-02-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 502,'1945-A21','1945-04-01 00:00:00','military missile','success',944,'4','','','','Test',435,'',1 from dual union all
select 5020,'1958-F07','1958-07-25 00:00:00','orbital','failure',946,'1','Diagnostic 1','Diagnostic Payload 1','','Sat',997,'F4D-1 747',12 from dual union all
select 50200,'1979-Y89','1979-02-09 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',80 from dual union all
select 50201,'1979-Y90','1979-02-09 20:20:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',83 from dual union all
select 50202,'1979-Y91','1979-02-10 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',82 from dual union all
select 50203,'1979-Y92','1979-02-11 19:50:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',85 from dual union all
select 50204,'1979-Y93','1979-02-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 50205,'1979-Y94','1979-02-12 20:50:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',85 from dual union all
select 50206,'1979-Y95','1979-02-13 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1397,'SHOK',80 from dual union all
select 50207,'1979-Y96','1979-02-13 20:10:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',83 from dual union all
select 50208,'1979-Y97','1979-02-14 13:50:00','suborbital rocket','unknown',725,'','','','','Aeron',887,'KORO',83 from dual union all
select 50209,'1979-Y98','1979-02-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 5021,'1958-F08','1958-08-12 00:00:00','orbital','failure',946,'2','Diagnostic 2','Diagnostic Payload 2','','Sat',997,'F4D-1 747',12 from dual union all
select 50210,'1979-Y99','1979-02-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 50211,'1980-001','1980-01-09 12:15:00','orbital','success',1238,'76047  -443','Kosmos-1149','Zenit-6','','Sat',474,'',null from dual union all
select 50212,'1980-002','1980-01-11 12:28:00','orbital','success',809,'78018  -505','Molniya-1','Molniya-1','','Sat',473,'',null from dual union all
select 50213,'1980-003','1980-01-14 19:49:00','orbital','success',680,'65067-248','Kosmos-1150','Parus','','Sat',457,'',null from dual union all
select 50214,'1980-004','1980-01-18 01:26:00','orbital','success',180,'AC-49','FLTSATCOM 3','FLTSATCOM F3','','Sat',268,'',null from dual union all
select 50215,'1980-005','1980-01-23 07:00:00','orbital','success',1448,'67067-401','Kosmos-1151','Okean-E NKh No. 2','','Sat',469,'',null from dual union all
select 50216,'1980-006','1980-01-24 15:45:00','orbital','success',1238,'76038  -914','Kosmos-1152','Feniks No. 928','','Sat',474,'',null from dual union all
select 50217,'1980-007','1980-01-25 20:36:00','orbital','success',680,'47167-249','Kosmos-1153','Parus','','Sat',457,'',null from dual union all
select 50218,'1980-008','1980-01-30 12:51:00','orbital','success',1510,'78055  -329','Kosmos-1154','Tselina-D','','Sat',475,'',null from dual union all
select 50219,'1980-009','1980-02-07 11:00:00','orbital','success',1238,'78017  -452','Kosmos-1155','Zenit-6','','Sat',475,'',null from dual union all
select 5022,'1958-F09','1958-08-17 12:18:00','orbital','failure',1353,'127','Able/Pioneer','Able 1','','Sat',233,'',16 from dual union all
select 50220,'1980-010','1980-02-07 21:10:00','orbital','success',1426,'23D-19','KH-11','KH11-3 KENNAN','','Sat',1777,'',null from dual union all
select 50221,'1980-011','1980-02-09 23:08:00','orbital','success',167,'35F','GPS 5','Navstar GPS SVN 5','','Sat',1775,'',null from dual union all
select 50222,'1980-012','1980-02-11 23:32:00','orbital','success',680,'53772-402','Kosmos-1156/1163','Strela-1M','','Sat',457,'',null from dual union all
select 50223,'1980-013','1980-02-12 00:52:37','orbital','failure',809,'78055  -532','Kosmos-1164','Oko','','Sat',475,'',578 from dual union all
select 50224,'1980-014','1980-02-14 15:57:00','orbital','success',387,'635/D151','Solar Max','SMM','','Sat',233,'',null from dual union all
select 50225,'1980-015','1980-02-17 00:40:00','orbital','success',819,'M-3S-1','Tansei-4','MS-T4','','Sat',835,'',null from dual union all
select 50226,'1980-016','1980-02-20 08:05:00','orbital','success',956,'297-01','Raduga','Gran'' No. 16L','','Sat',553,'',null from dual union all
select 50227,'1980-017','1980-02-21 12:00:00','orbital','success',1238,'76038  -713','Kosmos-1165','Zenit-4MKM','','Sat',475,'',null from dual;
insert into LAUNCH
select 50228,'1980-018','1980-02-22 08:35:00','orbital','success',822,'N-6(F)','Ayame 2','ECS-b','','Sat',1474,'',null from dual union all
select 50229,'1980-019','1980-03-03 09:27:00','orbital','success',164,'67F','OPS 7245    [NOSS3]','PARCAE 3','','Sat',1776,'',null from dual union all
select 5023,'1958-F10','1958-08-22 00:00:00','orbital','failure',946,'3','Diagnostic 3','Diagnostic Payload 3','','Sat',997,'F4D-1 747',80 from dual union all
select 50230,'1980-020','1980-03-04 10:30:00','orbital','success',1238,'78017  -447','Kosmos-1166','Zenit-6','','Sat',475,'',null from dual union all
select 50231,'1980-021','1980-03-14 10:40:00','orbital','success',1446,'51052-106','Kosmos-1167','US-P','','Sat',589,'',null from dual union all
select 50232,'1980-022','1980-03-17 21:37:00','orbital','success',680,'65098-314','Kosmos-1168','Tsikada','','Sat',456,'',null from dual union all
select 50233,'1980-023','1980-03-27 07:30:00','orbital','success',680,'65098-311','Kosmos-1169','Vektor','','Sat',456,'',null from dual union all
select 50234,'1980-024','1980-03-27 18:53:31','orbital','success',1238,'Zh15000-200','Progress-8','Progress 7K-TG No. 108','','Sat',562,'',null from dual union all
select 50235,'1980-025','1980-04-01 08:00:00','orbital','success',1238,'77025  -734','Kosmos-1170','Zenit-4MKM','','Sat',480,'',null from dual union all
select 50236,'1980-026','1980-04-03 07:40:00','orbital','success',680,'47198-315','Kosmos-1171','Lira','','Sat',457,'',null from dual union all
select 50237,'1980-027','1980-04-09 13:38:22','orbital','success',1238,'Zh15000-189','Soyuz-35','Soyuz 7K-T No. 51','','Sat',562,'',null from dual union all
select 50238,'1980-028','1980-04-12 20:17:57','orbital','success',809,'77025  -526','Kosmos-1172','Oko','','Sat',473,'',null from dual union all
select 50239,'1980-029','1980-04-17 08:30:00','orbital','success',1238,'77025  -735','Kosmos-1173','Zenit-4MKM','','Sat',480,'',null from dual union all
select 5024,'1958-F11','1958-08-24 06:17:22','orbital','failure',636,'RS/CC-47 HN','Explorer 5','Explorer 5','','Sat',279,'',284 from dual union all
select 50240,'1980-030','1980-04-18 00:51:00','orbital','success',1446,'45055-506','Kosmos-1174','IS','','Sat',589,'',null from dual union all
select 50241,'1980-031','1980-04-18 17:31:00','orbital','failure',809,'78055  -506','Kosmos-1175','Molniya-3 No. 26','','Sat',473,'',null from dual union all
select 50242,'1980-032','1980-04-26 10:00:00','orbital','success',167,'34F','GPS 6','Navstar GPS SVN 6','','Sat',1775,'',null from dual union all
select 50243,'1980-033','1980-04-27 06:24:11','orbital','success',1238,'Zh15000-210','Progress-9','Progress 7K-TG No. 109','','Sat',480,'',null from dual union all
select 50244,'1980-034','1980-04-29 11:40:00','orbital','success',1446,'51052-103','Kosmos-1176','US-A reactor','','Sat',589,'',null from dual union all
select 50245,'1980-035','1980-04-29 13:30:00','orbital','success',1238,'Zh15000-213','Kosmos-1177','Yantar''-4K1 No. 2','','Sat',474,'',null from dual union all
select 50246,'1980-036','1980-05-07 13:00:00','orbital','success',1238,'78017  -449','Kosmos-1178','Zenit-6','','Sat',473,'',null from dual union all
select 50247,'1980-037','1980-05-14 13:00:00','orbital','success',680,'53783-456','Kosmos-1179','Taifun-1B','','Sat',456,'',null from dual union all
select 50248,'1980-038','1980-05-15 05:35:00','orbital','success',1238,'78017  -448','Kosmos-1180','Orion','','Sat',474,'',null from dual union all
select 50249,'1980-039','1980-05-20 09:21:00','orbital','success',680,'47175-116','Kosmos-1181','Parus','','Sat',457,'',null from dual union all
select 5025,'1958-F12','1958-08-25 00:00:00','orbital','failure',946,'4','Radiation 1','Radiation Payload 1','','Sat',997,'F4D-1 747',12 from dual union all
select 50250,'1980-040','1980-05-23 07:10:00','orbital','success',1238,'78017  -064','Kosmos-1182','Fram','','Sat',474,'',null from dual union all
select 50251,'1980-041','1980-05-26 18:20:39','orbital','success',1238,'P 15000-226','Soyuz-36','Soyuz 7K-T No. 52','','Sat',562,'',null from dual union all
select 50252,'1980-042','1980-05-28 12:00:00','orbital','success',1238,'76047  -446','Kosmos-1183','Zenit-6','','Sat',473,'',null from dual union all
select 50253,'1980-043','1980-05-29 10:53:00','orbital','success',162,'19F','NOAA B','NOAA B','','Sat',1776,'',null from dual union all
select 50254,'1980-044','1980-06-04 07:34:00','orbital','success',1510,'76047  -351','Kosmos-1184','Tselina-D','','Sat',474,'',null from dual union all
select 50255,'1980-045','1980-06-05 14:19:30','orbital','success',1238,'Zh15000-196','Soyuz T-2','Soyuz 7K-ST No. 7L','','Sat',480,'',null from dual union all
select 50256,'1980-046','1980-06-06 07:00:00','orbital','success',1238,'78055  -928','Kosmos-1185','Resurs-F1 17F41 No. 12','','Sat',473,'',null from dual union all
select 50257,'1980-047','1980-06-06 11:00:00','orbital','success',680,'47168-306','Kosmos-1186','Romb','','Sat',456,'',null from dual union all
select 50258,'1980-048','1980-06-12 12:30:00','orbital','success',1238,'78017  -451','Kosmos-1187','Zenit-6','','Sat',473,'',null from dual union all
select 50259,'1980-049','1980-06-14 00:49:50','orbital','success',956,'303-02','Gorizont','Gorizont No. 15L','','Sat',553,'',null from dual union all
select 5026,'1958-F13','1958-08-26 00:00:00','orbital','failure',946,'5','Radiation 2','Radiation Payload 2','','Sat',997,'F4D-1 747',12 from dual union all
select 50260,'1980-050','1980-06-14 20:51:34','orbital','success',809,'77025  -527','Kosmos-1188','Oko','','Sat',474,'',null from dual union all
select 50261,'1980-051','1980-06-18 06:14:12','orbital','success',1510,'76038  -313','Meteor-Priroda','Meteor-Priroda No. 3-1','','Sat',562,'',null from dual union all
select 50262,'1980-052','1980-06-18 18:30:00','orbital','success',1426,'23D-16','HEXAGON SV16','HEXAGON SV-16','','Sat',1777,'',null from dual union all
select 50263,'1980-053','1980-06-21 18:34:00','orbital','success',809,'76047  -550','Molniya-1','Molniya-1','','Sat',473,'',null from dual union all
select 50264,'1980-054','1980-06-26 12:20:00','orbital','success',1238,'78017  -450','Kosmos-1189','Zenit-6','','Sat',473,'',null from dual union all
select 50265,'1980-055','1980-06-29 04:40:42','orbital','success',1238,'P 15000-232','Progress-10','Progress 7K-TG No. 110','','Sat',480,'',null from dual union all
select 50266,'1980-056','1980-07-01 07:12:00','orbital','success',680,'47182-416','Kosmos-1190','Strela-2M','','Sat',457,'',null from dual union all
select 50267,'1980-057','1980-07-02 00:53:41','orbital','success',809,'77025  -529','Kosmos-1191','Oko','','Sat',473,'',null from dual union all
select 50268,'1980-058','1980-07-09 00:42:00','orbital','success',680,'65072-409','Kosmos-1192/1199','Strela-1M','','Sat',457,'',null from dual union all
select 50269,'1980-059','1980-07-09 12:40:00','orbital','success',1238,'78017  -453','Kosmos-1200','Zenit-6','','Sat',473,'',null from dual union all
select 5027,'1958-F14','1958-08-28 00:00:00','orbital','failure',946,'6','Radiation 3','Radiation Payload 3','','Sat',997,'F4D-1 747',12 from dual union all
select 50270,'1980-060','1980-07-14 22:35:00','orbital','success',956,'301-01','Ekran','Ekran No. 19L','','Sat',554,'',null from dual union all
select 50271,'1980-061','1980-07-15 07:30:00','orbital','success',1238,'76047  -063','Kosmos-1201','Fram','','Sat',474,'',null from dual union all
select 50272,'1980-062','1980-07-18 02:33:45','orbital','success',1084,'SLV-3-E2','RS-1','Rohini RS-1','','Sat',1360,'',null from dual union all
select 50273,'1980-063','1980-07-18 10:37:00','orbital','success',809,'76047  -551','Molniya-3','Molniya-3 No. 27','','Sat',474,'',null from dual union all
select 50274,'1980-064','1980-07-23 18:33:03','orbital','success',1238,'P 15000-238','Soyuz-37','Soyuz 7K-T No. 53','','Sat',480,'',null from dual union all
select 50275,'1980-065','1980-07-24 12:40:00','orbital','success',1238,'78017  -454','Kosmos-1202','Zenit-6','','Sat',474,'',null from dual union all
select 50276,'1980-066','1980-07-31 07:45:00','orbital','success',1238,'77023  -796','Kosmos-1203','Resurs-F1 17F41 No. 13','','Sat',474,'',null from dual union all
select 50277,'1980-067','1980-07-31 10:20:00','orbital','success',680,'47175-122','Kosmos-1204','Romb','','Sat',622,'',null from dual union all
select 50278,'1980-068','1980-08-12 11:50:00','orbital','success',1238,'78017  -455','Kosmos-1205','Zenit-6','','Sat',474,'',null from dual union all
select 50279,'1980-069','1980-08-15 05:34:00','orbital','success',1510,'76047  -352','Kosmos-1206','Tselina-D','','Sat',474,'',null from dual union all
select 5028,'1958-F15','1958-09-23 07:40:23','orbital','failure',1513,'B1-3','Luna','E-1 No. 1','','Sat',480,'',80 from dual union all
select 50280,'1980-070','1980-08-22 10:00:00','orbital','success',1238,'76047  -065','Kosmos-1207','Fram','','Sat',473,'',null from dual union all
select 50281,'1980-071','1980-08-26 15:30:00','orbital','success',1238,'77052  -960','Kosmos-1208','Feniks No. 927','','Sat',473,'',null from dual union all
select 50282,'1980-072','1980-09-03 10:20:00','orbital','success',1238,'76053  -476','Kosmos-1209','Resurs-F1 17F41 No. 14','','Sat',473,'',null from dual union all
select 50283,'1980-073','1980-09-09 11:00:00','orbital','success',1510,'78018  -308','Meteor-2','Meteor-2 No. 5','','Sat',474,'',null from dual union all
select 50284,'1980-074','1980-09-09 22:27:00','orbital','success',390,'637/D152','GOES 4','GOES D','','Sat',233,'',null from dual union all
select 50285,'1980-075','1980-09-18 19:11:04','orbital','success',1238,'P 15000-242','Soyuz-38','Soyuz 7K-T No. 54','','Sat',480,'',null from dual union all
select 50286,'1980-076','1980-09-19 10:10:00','orbital','success',1238,'77023  -466','Kosmos-1210','Zenit-6','','Sat',473,'',null from dual union all
select 50287,'1980-077','1980-09-23 10:30:00','orbital','success',1238,'78055  -840','Kosmos-1211','Orion','','Sat',473,'',null from dual union all
select 50288,'1980-078','1980-09-26 10:10:00','orbital','success',1238,'77023  -076','Kosmos-1212','Fram','','Sat',473,'',null from dual union all
select 50289,'1980-079','1980-09-28 15:09:55','orbital','success',1238,'P 15000-219','Progress-11','Progress 7K-TG No. 111','','Sat',480,'',null from dual union all
select 5029,'1958-F17','1958-10-11 21:41:58','orbital','failure',1513,'B1-4','Luna','E-1 No. 2','','Sat',480,'',80 from dual union all
select 50290,'1980-080','1980-10-03 12:00:00','orbital','success',1238,'77023  -469','Kosmos-1213','Zenit-6','','Sat',473,'',null from dual union all
select 50291,'1980-081','1980-10-05 17:10:00','orbital','success',956,'300-01','Raduga','Gran'' No. 17L','','Sat',553,'',null from dual union all
select 50292,'1980-082','1980-10-10 13:10:00','orbital','success',1238,'77023  -467','Kosmos-1214','Zenit-4MKM','','Sat',473,'',null from dual union all
select 50293,'1980-083','1980-10-14 20:41:00','orbital','success',680,'47172-401','Kosmos-1215','Tselina-OM','','Sat',456,'',null from dual union all
select 50294,'1980-084','1980-10-16 12:20:00','orbital','success',1238,'77023  -470','Kosmos-1216','Zenit-6','','Sat',473,'',null from dual union all
select 50295,'1980-085','1980-10-24 10:53:29','orbital','success',809,'76047  -548','Kosmos-1217','Oko','','Sat',473,'',null from dual union all
select 50296,'1980-086','1980-10-30 10:00:00','orbital','success',1238,'P 15000-233','Kosmos-1218','Yantar''-4K1 No. 3','','Sat',562,'',null from dual union all
select 50297,'1980-087','1980-10-31 03:54:00','orbital','success',180,'AC-57','FLTSATCOM 4','FLTSATCOM F4','','Sat',268,'',null from dual union all
select 50298,'1980-088','1980-10-31 12:00:00','orbital','success',1238,'77023  -471','Kosmos-1219','Zenit-6','','Sat',473,'',null from dual union all
select 50299,'1980-089','1980-11-04 15:04:00','orbital','success',1446,'51028-202','Kosmos-1220','US-P','','Sat',589,'',null from dual union all
select 503,'1945-A22','1945-04-01 00:00:00','military missile','success',944,'5','','','','Test',435,'',1 from dual union all
select 5030,'1958-F18','1958-10-23 03:21:04','orbital','failure',636,'RS/CC-49 HE','Beacon 1','Beacon','','Sat',279,'',100 from dual union all
select 50300,'1980-090','1980-11-12 12:30:00','orbital','success',1238,'77023  -472','Kosmos-1221','Zenit-6','','Sat',473,'',null from dual union all
select 50301,'1980-091','1980-11-15 22:49:00','orbital','success',388,'636/D153','SBS 1','SBS 1','','Sat',233,'',null from dual union all
select 50302,'1980-092','1980-11-16 04:18:00','orbital','success',809,'77025  -524','Molniya-1','Molniya-1','','Sat',473,'',null from dual union all
select 50303,'1980-093','1980-11-21 11:53:00','orbital','success',1510,'76047  -353','Kosmos-1222','Tselina-D','','Sat',474,'',null from dual union all
select 50304,'1980-094','1980-11-27 14:18:28','orbital','success',1238,'Zh15000-202','Soyuz T-3','Soyuz 7K-ST No. 8L','','Sat',480,'',null from dual union all
select 50305,'1980-095','1980-11-27 21:37:10','orbital','success',809,'78055  -536','Kosmos-1223','Oko','','Sat',473,'',null from dual union all
select 50306,'1980-096','1980-12-01 12:15:00','orbital','success',1238,'77023  -473','Kosmos-1224','Zenit-6','','Sat',474,'',null from dual union all
select 50307,'1980-097','1980-12-05 04:23:00','orbital','success',680,'65098-317','Kosmos-1225','Parus','','Sat',456,'',null from dual union all
select 50308,'1980-098','1980-12-06 23:31:00','orbital','success',180,'AC-54','Intelsat 502','INTELSAT V F2','','Sat',269,'',null from dual union all
select 50309,'1980-099','1980-12-10 20:53:00','orbital','success',680,'65083-464','Kosmos-1226','Tsikada','','Sat',456,'',null from dual union all
select 5031,'1958-F19','1958-11-08 07:30:21','orbital','failure',1353,'129','Pioneer 2','Able 3','','Sat',233,'',1550 from dual union all
select 50310,'1980-100','1980-12-13 16:04:00','orbital','success',1405,'34B-3  (3B-53)','OPS 5805','SDS 4','','Sat',1778,'',null from dual union all
select 50311,'1980-101','1980-12-16 12:15:00','orbital','success',1238,'77023  -475','Kosmos-1227','Zenit-6','','Sat',474,'',null from dual union all
select 50312,'1980-102','1980-12-23 22:48:00','orbital','success',680,'47198-318','Kosmos-1228/1235','Strela-1M','','Sat',456,'',null from dual union all
select 50313,'1980-103','1980-12-25 04:02:00','orbital','success',807,'P 15000-203','Prognoz-8','SO-M No. 508','','Sat',562,'',null from dual union all
select 50314,'1980-104','1980-12-26 11:49:00','orbital','success',956,'304-01','Ekran','Ekran No. 20L','','Sat',554,'',null from dual union all
select 50315,'1980-105','1980-12-26 16:10:00','orbital','success',1238,'76038  -913','Kosmos-1236','Feniks No. 941','','Sat',473,'',null from dual union all
select 50316,'1980-A01','1980-01-11 22:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 50317,'1980-A02','1980-01-21 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual;
insert into LAUNCH
select 50318,'1980-A03','1980-01-28 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',48 from dual union all
select 50319,'1980-A04','1980-01-29 21:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 5032,'1958-F20','1958-12-04 18:18:45','orbital','failure',1513,'B1-5','Luna','E-1 No. 3','','Sat',480,'',100 from dual union all
select 50320,'1980-A05','1980-02-01 15:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',46 from dual union all
select 50321,'1980-A06','1980-02-06 12:01:42','military missile','unknown',1099,'','','','','Test',293,'B-52',10 from dual union all
select 50322,'1980-A07','1980-02-22 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',48 from dual union all
select 50323,'1980-A08','1980-02-28 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 50324,'1980-A09','1980-03-12 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 50325,'1980-A10','1980-03-14 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',24 from dual union all
select 50326,'1980-A11','1980-03-14 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 50327,'1980-A12','1980-03-24 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',48 from dual union all
select 50328,'1980-A13','1980-03-26 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',37 from dual union all
select 50329,'1980-A14','1980-03-31 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',49 from dual union all
select 5033,'1958-M01','1958-01-21 00:00:00','military missile','success',1162,'9','','','','OT?',1852,'',50 from dual union all
select 50330,'1980-A15','1980-04-01 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 50331,'1980-A16','1980-04-02 21:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 50332,'1980-A17','1980-04-11 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 50333,'1980-A18','1980-05-02 20:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 50334,'1980-A19','1980-05-12 18:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',38 from dual union all
select 50335,'1980-A20','1980-05-30 15:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',37 from dual union all
select 50336,'1980-A21','1980-06-04 13:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',49 from dual union all
select 50337,'1980-A22','1980-06-09 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 50338,'1980-A23','1980-06-09 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',44 from dual union all
select 50339,'1980-A24','1980-06-11 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',45 from dual union all
select 5034,'1958-M02','1958-03-14 00:00:00','military missile','success',1162,'10','','','','OT?',1853,'',50 from dual union all
select 50340,'1980-A25','1980-07-02 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',48 from dual union all
select 50341,'1980-A26','1980-07-09 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 50342,'1980-A27','1980-07-11 14:42:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',42 from dual union all
select 50343,'1980-A28','1980-07-11 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 50344,'1980-A29','1980-07-14 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 50345,'1980-A30','1980-07-21 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 50346,'1980-A31','1980-07-30 00:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 50347,'1980-A32','1980-08-01 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 50348,'1980-A33','1980-09-05 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',41 from dual union all
select 50349,'1980-A34','1980-09-08 12:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',48 from dual union all
select 5035,'1958-M03','1958-04-25 00:00:00','suborbital rocket','success',839,'','Beacon Test 1R','','','Balloon test',1821,'',73 from dual union all
select 50350,'1980-A35','1980-09-09 15:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',49 from dual union all
select 50351,'1980-A36','1980-09-23 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 50352,'1980-A37','1980-09-30 00:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 50353,'1980-A38','1980-10-15 19:23:00','suborbital rocket','success',1284,'TY1-6423','','','','Aeron',29,'',46 from dual union all
select 50354,'1980-A39','1980-10-24 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 50355,'1980-A40','1980-10-31 22:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',44 from dual union all
select 50356,'1980-A41','1980-11-03 22:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 50357,'1980-A42','1980-11-05 12:07:49','military missile','unknown',1099,'','','','','Test',293,'FB-111',10 from dual union all
select 50358,'1980-A43','1980-11-05 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 50359,'1980-A44','1980-11-07 22:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 5036,'1958-M04','1958-05-01 00:00:00','military missile','success',1162,'11','','','','OT?',1853,'',50 from dual union all
select 50360,'1980-A45','1980-11-17 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 50361,'1980-A46','1980-11-18 01:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 50362,'1980-A47','1980-11-18 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',44 from dual union all
select 50363,'1980-A48','1980-11-21 20:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',49 from dual union all
select 50364,'1980-A49','1980-11-21 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 50365,'1980-A50','1980-11-24 17:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',46 from dual union all
select 50366,'1980-A51','1980-12-09 03:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',40 from dual union all
select 50367,'1980-A52','1980-12-11 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 50368,'1980-A53','1980-12-12 14:23:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',43 from dual union all
select 50369,'1980-A54','1980-12-19 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 5037,'1958-M05','1958-05-09 00:00:00','military missile','success',1162,'12','','','','OT?',1853,'',50 from dual union all
select 50370,'1980-A55','1980-12-29 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',44 from dual union all
select 50371,'1980-A56','1980-10-03 00:00:00','military missile','failure',1057,'5      59A/45x','','','','Test',989,'',40 from dual union all
select 50372,'1980-A57','1980-12-03 00:00:00','military missile','failure',1057,'6      54A/46x','','','','Test',989,'',40 from dual union all
select 50373,'1980-E01','1980-03-18 16:01:00','orbital','destroyed before launch',1510,'78055  -330','','','','Sat',475,'',0 from dual union all
select 50374,'1980-E02','1980-09-19 08:00:00','military missile','destroyed before launch',1420,'','','','','Explosion',936,'',0 from dual union all
select 50375,'1980-F01','1980-05-23 14:29:39','orbital','failure',91,'L02','Firewheel','CAT','','Sat',344,'',20 from dual union all
select 50376,'1980-F02','1980-07-15 02:22:11','orbital','failure',1374,'304','DMSP 5D F-5','DMSP Block 5D S-4','','Sat',1720,'',820 from dual union all
select 50377,'1980-M01','1980-02-16 08:25:00','suborbital rocket','success',132,'NASA 23.17UE','','','','Plasma',1412,'',78 from dual union all
select 50378,'1980-M02','1980-02-16 10:30:00','suborbital rocket','failure',1023,'','Eclipse','','','Meteo',320,'',50 from dual union all
select 50379,'1980-M03','1980-02-16 20:30:00','suborbital rocket','success',132,'NASA 23.18UE','','','','Plasma',1412,'',76 from dual union all
select 5038,'1958-M06','1958-06-30 00:00:00','military missile','success',1162,'13','Off-pad test','','','OT?',1853,'',50 from dual union all
select 50380,'1980-M04','1980-02-17 10:30:00','suborbital rocket','success',1023,'','Eclipse control','','','Meteo',320,'',70 from dual union all
select 50381,'1980-M05','1980-07-14 19:11:00','suborbital rocket','success',891,'NASA 30.03UU','','','','Aeron',1821,'',76 from dual union all
select 50382,'1980-M06','1980-10-16 18:54:00','suborbital rocket','success',891,'NASA 30.04UU','','','','Aeron',1821,'',75 from dual union all
select 50383,'1980-M07','1980-01-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50384,'1980-M08','1980-01-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50385,'1980-M09','1980-01-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50386,'1980-M10','1980-01-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50387,'1980-M100','1980-09-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50388,'1980-M101','1980-09-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50389,'1980-M102','1980-09-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5039,'1958-M07','1958-08-27 00:00:00','military missile','success',1162,'16','','','','OT?',1895,'',50 from dual union all
select 50390,'1980-M103','1980-10-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50391,'1980-M104','1980-10-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50392,'1980-M105','1980-10-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50393,'1980-M106','1980-10-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50394,'1980-M107','1980-10-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50395,'1980-M108','1980-10-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50396,'1980-M109','1980-10-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50397,'1980-M11','1980-01-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50398,'1980-M110','1980-10-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50399,'1980-M111','1980-10-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 504,'1945-A23','1945-04-01 00:00:00','military missile','success',944,'6','','','','Test',435,'',1 from dual union all
select 5040,'1958-M08','1958-08-27 00:00:00','military missile','success',1162,'15','','','','OT?',1895,'',50 from dual union all
select 50400,'1980-M112','1980-10-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50401,'1980-M113','1980-10-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50402,'1980-M114','1980-10-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50403,'1980-M115','1980-10-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50404,'1980-M116','1980-10-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50405,'1980-M117','1980-10-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50406,'1980-M118','1980-10-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50407,'1980-M119','1980-11-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual;
insert into LAUNCH
select 50408,'1980-M12','1980-01-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50409,'1980-M120','1980-11-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 5041,'1958-M09','1958-08-28 00:00:00','military missile','success',1162,'14','','','','OT?',1895,'',50 from dual union all
select 50410,'1980-M121','1980-11-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50411,'1980-M122','1980-11-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50412,'1980-M123','1980-11-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50413,'1980-M124','1980-11-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50414,'1980-M125','1980-11-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50415,'1980-M126','1980-11-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50416,'1980-M127','1980-11-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50417,'1980-M128','1980-11-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50418,'1980-M129','1980-11-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50419,'1980-M13','1980-01-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5042,'1958-M10','1958-08-01 09:47:00','military missile','success',1053,'CC-50    SX','Hardtack Teak','','','Nuclear Test',771,'',76 from dual union all
select 50420,'1980-M130','1980-11-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50421,'1980-M131','1980-11-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50422,'1980-M132','1980-11-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50423,'1980-M133','1980-11-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50424,'1980-M134','1980-11-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50425,'1980-M135','1980-11-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50426,'1980-M136','1980-11-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50427,'1980-M137','1980-12-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50428,'1980-M138','1980-12-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50429,'1980-M139','1980-12-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 5043,'1958-M11','1958-09-25 05:50:00','suborbital rocket','success',662,'K-6-RS-1','K-6-8','S4','','Solar UV',7,'',51 from dual union all
select 50430,'1980-M14','1980-01-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50431,'1980-M140','1980-12-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50432,'1980-M141','1980-12-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50433,'1980-M142','1980-12-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50434,'1980-M143','1980-12-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50435,'1980-M144','1980-12-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50436,'1980-M145','1980-05-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 50437,'1980-M146','1980-06-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 50438,'1980-M147','1980-08-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 50439,'1980-M148','1980-08-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 5044,'1958-M12','1958-09-26 03:50:00','suborbital rocket','success',662,'K-6-TW-4','K-6-9','S5','','Aeron',7,'',58 from dual union all
select 50440,'1980-M149','1980-08-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 50441,'1980-M15','1980-01-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50442,'1980-M150','1980-08-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 50443,'1980-M151','1980-02-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50444,'1980-M152','1980-08-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50445,'1980-M153','1980-08-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50446,'1980-M154','1980-10-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50447,'1980-M155','1980-02-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50448,'1980-M156','1980-03-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50449,'1980-M157','1980-03-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5045,'1958-M13','1958-10-21 00:00:00','military missile','success',1162,'18','','','','OT?',1895,'',50 from dual union all
select 50450,'1980-M158','1980-04-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50451,'1980-M159','1980-07-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50452,'1980-M16','1980-01-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50453,'1980-M160','1980-07-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50454,'1980-M161','1980-10-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50455,'1980-M162','1980-10-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50456,'1980-M163','1980-10-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50457,'1980-M164','1980-08-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50458,'1980-M165','1980-09-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50459,'1980-M166','1980-10-29 15:10:00','suborbital rocket','success',1281,'','VLF Probe 3','','','Ionos',1821,'',75 from dual union all
select 5046,'1958-M14','1958-10-23 00:00:00','military missile','success',1162,'17','','','','OT?',1895,'',50 from dual union all
select 50460,'1980-M167','1980-10-29 15:40:00','suborbital rocket','success',1281,'','VLF Probe 4','','','Ionos',1821,'',75 from dual union all
select 50461,'1980-M17','1980-01-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50462,'1980-M18','1980-01-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50463,'1980-M19','1980-01-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50464,'1980-M20','1980-01-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50465,'1980-M21','1980-02-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50466,'1980-M22','1980-02-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50467,'1980-M23','1980-02-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50468,'1980-M24','1980-02-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50469,'1980-M25','1980-02-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 5047,'1958-M15','1958-10-01 19:42:00','suborbital rocket','success',838,'NN8.58F','','IGY NN8.58F','','Test/Solar UV/Solar',1135,'LSD-31',56 from dual union all
select 50470,'1980-M26','1980-02-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50471,'1980-M27','1980-02-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50472,'1980-M28','1980-03-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50473,'1980-M29','1980-03-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50474,'1980-M30','1980-03-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50475,'1980-M31','1980-04-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50476,'1980-M32','1980-04-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50477,'1980-M33','1980-04-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50478,'1980-M34','1980-04-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50479,'1980-M35','1980-04-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5048,'1958-M16','1958-11-03 12:52:00','suborbital rocket','success',45,'SS12.050','Grenades','IGY SS12.50F','','Aeron',637,'',54 from dual union all
select 50480,'1980-M36','1980-05-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50481,'1980-M37','1980-05-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50482,'1980-M38','1980-05-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50483,'1980-M39','1980-05-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50484,'1980-M40','1980-05-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50485,'1980-M41','1980-05-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50486,'1980-M42','1980-05-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50487,'1980-M43','1980-06-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50488,'1980-M44','1980-06-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50489,'1980-M45','1980-06-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 5049,'1958-M17','1958-11-07 12:25:38','suborbital rocket','success',45,'SS12.051','Grenades','IGY SS12.51F','','Aeron',637,'',59 from dual union all
select 50490,'1980-M46','1980-06-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50491,'1980-M47','1980-06-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50492,'1980-M48','1980-06-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50493,'1980-M49','1980-06-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50494,'1980-M50','1980-06-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50495,'1980-M51','1980-06-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50496,'1980-M52','1980-06-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50497,'1980-M53','1980-06-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50498,'1980-M54','1980-06-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual;
insert into LAUNCH
select 50499,'1980-M55','1980-06-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 505,'1945-A24','1945-04-01 00:00:00','military missile','success',944,'7','','','','Test',435,'',1 from dual union all
select 5050,'1958-M18','1958-11-12 11:40:00','suborbital rocket','success',839,'SS6.52','Grenades','IGY SS6.52F','','Aeron',637,'',74 from dual union all
select 50500,'1980-M56','1980-07-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50501,'1980-M57','1980-07-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50502,'1980-M58','1980-07-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50503,'1980-M59','1980-07-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50504,'1980-M60','1980-07-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50505,'1980-M61','1980-07-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50506,'1980-M62','1980-07-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50507,'1980-M63','1980-07-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50508,'1980-M64','1980-07-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50509,'1980-M65','1980-07-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5051,'1958-M19','1958-11-14 11:09:43','suborbital rocket','success',839,'SS6.53','Grenades','IGY SS6.53F','','Aeron',637,'',75 from dual union all
select 50510,'1980-M66','1980-07-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50511,'1980-M67','1980-07-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50512,'1980-M68','1980-07-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50513,'1980-M69','1980-08-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50514,'1980-M70','1980-08-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50515,'1980-M71','1980-08-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50516,'1980-M72','1980-08-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50517,'1980-M73','1980-08-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50518,'1980-M74','1980-08-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50519,'1980-M75','1980-08-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 5052,'1958-M20','1958-11-19 18:01:40','suborbital rocket','success',839,'SS6.54','Grenades','IGY SS6.54F','','Aeron',637,'',71 from dual union all
select 50520,'1980-M76','1980-08-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50521,'1980-M77','1980-08-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50522,'1980-M78','1980-08-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50523,'1980-M79','1980-08-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50524,'1980-M80','1980-08-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50525,'1980-M81','1980-08-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50526,'1980-M82','1980-08-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50527,'1980-M83','1980-08-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50528,'1980-M84','1980-08-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50529,'1980-M85','1980-09-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 5053,'1958-M21','1958-11-19 20:32:00','suborbital rocket','success',366,'','RW-104','','','Aeron',1490,'',78 from dual union all
select 50530,'1980-M86','1980-09-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50531,'1980-M87','1980-09-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50532,'1980-M88','1980-09-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50533,'1980-M89','1980-09-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50534,'1980-M90','1980-09-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50535,'1980-M91','1980-09-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50536,'1980-M92','1980-09-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50537,'1980-M93','1980-09-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 50538,'1980-M94','1980-09-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50539,'1980-M95','1980-09-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5054,'1958-M22','1958-11-20 02:25:00','suborbital rocket','success',366,'','RW-105','','','Aeron',1490,'',76 from dual union all
select 50540,'1980-M96','1980-09-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50541,'1980-M97','1980-09-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50542,'1980-M98','1980-09-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50543,'1980-M99','1980-09-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 50544,'1980-S01','1980-08-20 00:00:00','military missile','success',744,'','15A15 UBP-6','','','OT',528,'',1000 from dual union all
select 50545,'1980-S02','1980-01-01 00:00:00','military missile','success',755,'','','','','OT',307,'S614',600 from dual union all
select 50546,'1980-S03','1980-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 50547,'1980-S04','1980-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 50548,'1980-S05','1980-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 50549,'1980-S06','1980-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 5055,'1958-M23','1958-11-20 14:38:40','suborbital rocket','success',839,'SS6.55','Grenades','IGY SS6.55F','','Aeron',637,'',69 from dual union all
select 50550,'1980-S07','1980-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 50551,'1980-S08','1980-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 50552,'1980-S09','1980-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 50553,'1980-S10','1980-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 50554,'1980-S100','1980-05-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50555,'1980-S101','1980-05-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50556,'1980-S102','1980-05-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50557,'1980-S103','1980-05-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50558,'1980-S104','1980-05-07 14:40:00','military missile','unknown',917,'FOT-130','P-393 80 Cape 07','','','OT',231,'',180 from dual union all
select 50559,'1980-S105','1980-05-07 14:59:00','military missile','unknown',917,'FOT-136','P-394 80 Cape 08','','','OT',231,'',180 from dual union all
select 5056,'1958-M24','1958-11-22 17:06:00','suborbital rocket','success',45,'SS12.057','Grenades','IGY SS12.57F','','Aeron',637,'',59 from dual union all
select 50560,'1980-S106','1980-05-07 15:58:00','military missile','unknown',917,'FOT-137','P-395 80 Cape 09','','','OT',231,'',180 from dual union all
select 50561,'1980-S107','1980-05-14 00:00:00','suborbital rocket','success',211,'NASA 27.18NP','SPAR 7','','','Micrograv',1895,'',204 from dual union all
select 50562,'1980-S108','1980-05-15 14:51:00','military missile','unknown',917,'FOT-138','P-396 80 Cape 10','','','OT',231,'',180 from dual union all
select 50563,'1980-S109','1980-05-18 00:00:00','military missile','success',436,'','','','','Test',810,'',1000 from dual union all
select 50564,'1980-S11','1980-04-23 00:00:00','test rocket','unknown',1497,'17','K65UP No. 17','','','Aeron/Ionos?',621,'',1500 from dual union all
select 50565,'1980-S110','1980-05-19 00:30:00','military missile','success',932,'A3TK PT-3','CHEVALINE PT3','','','Test',250,'',1000 from dual union all
select 50566,'1980-S111','1980-05-21 02:00:00','military missile','success',436,'','','','','Test',810,'',1000 from dual union all
select 50567,'1980-S112','1980-05-21 12:08:00','suborbital rocket','success',117,'GL A24.609-02','TEM-2','','','ABM Tech IR/UV',1873,'',232 from dual union all
select 50568,'1980-S113','1980-05-22 15:00:00','suborbital rocket','success',133,'NASA 25.63GS','Solar Constant 3','','','Solar',1895,'',249 from dual union all
select 50569,'1980-S114','1980-05-28 06:23:00','military missile','success',1442,'','OT-3','','','OT',421,'SSBN 657',1000 from dual union all
select 5057,'1958-M25','1958-11-24 17:27:18','suborbital rocket','success',839,'SS6.58','Grenades','IGY SS6.58F','','Aeron',637,'',79 from dual union all
select 50570,'1980-S115','1980-05-28 06:24:00','military missile','success',1442,'','OT-3','','','OT',421,'SSBN 657',1000 from dual union all
select 50571,'1980-S116','1980-05-28 06:52:00','military missile','success',1442,'','OT-3','','','OT',421,'SSBN 657',1000 from dual union all
select 50572,'1980-S117','1980-05-28 06:52:00','military missile','success',1442,'','OT-3','','','OT',421,'SSBN 657',1000 from dual union all
select 50573,'1980-S118','1980-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50574,'1980-S119','1980-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50575,'1980-S12','1980-09-03 00:00:00','test rocket','unknown',1497,'18','K65UP No. 18','','','Aeron/Ionos?',621,'',1500 from dual union all
select 50576,'1980-S120','1980-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50577,'1980-S121','1980-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50578,'1980-S122','1980-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50579,'1980-S123','1980-06-10 11:45:00','suborbital rocket','success',211,'NASA 27.46AS','','','','Solar',1895,'',298 from dual union all
select 5058,'1958-M26','1958-12-11 00:00:00','military missile','success',1162,'19','JPL-477','','','OT?',1895,'',50 from dual union all
select 50580,'1980-S124','1980-06-16 08:00:00','suborbital rocket','success',133,'NASA 25.58DG','NRL FUV Spectrograph','','','UV Astron',1895,'',217 from dual union all
select 50581,'1980-S125','1980-06-22 14:30:00','military missile','success',799,'Global Shield 80','Mk 12 RV x 3','FOT GT74GM','GLORY TRIP 74GM','OT',1703,'',1300 from dual union all
select 50582,'1980-S126','1980-06-24 05:30:00','suborbital rocket','success',211,'NASA 27.41UH','','','','XR Astron',1895,'',213 from dual union all
select 50583,'1980-S127','1980-06-27 05:39:00','military missile','success',1442,'','OT-4','','','OT',421,'SSBN 658',1000 from dual union all
select 50584,'1980-S128','1980-06-27 05:39:00','military missile','success',1442,'','OT-4','','','OT',421,'SSBN 658',1000 from dual union all
select 50585,'1980-S129','1980-06-27 05:53:00','military missile','success',1442,'','OT-4','','','OT',421,'SSBN 658',1000 from dual union all
select 50586,'1980-S13','1980-12-24 00:00:00','test rocket','unknown',1497,'19','K65UP No. 19','','','Aeron/Ionos?',621,'',1500 from dual union all
select 50587,'1980-S130','1980-06-27 05:53:00','military missile','success',1442,'','OT-4','','','OT',421,'SSBN 658',1000 from dual union all
select 50588,'1980-S131','1980-06-27 19:01:00','suborbital rocket','success',133,'NASA 25.46UE','','','','Plasma',1895,'',260 from dual;
insert into LAUNCH
select 50589,'1980-S132','1980-07-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 5059,'1958-M27','1958-12-04 01:13:00','suborbital rocket','success',26,'','AL1','','','Aeron',1837,'',78 from dual union all
select 50590,'1980-S133','1980-07-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50591,'1980-S134','1980-07-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50592,'1980-S135','1980-07-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50593,'1980-S136','1980-07-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50594,'1980-S137','1980-07-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50595,'1980-S138','1980-07-04 00:00:00','military missile','success',1004,'49L','15A14 UBP-26 DP','GCh 15F678','','OT',496,'',1000 from dual union all
select 50596,'1980-S139','1980-07-11 16:01:00','military missile','success',939,'','FOT-41','','','OT',421,'SSBN 642',500 from dual union all
select 50597,'1980-S14','1980-01-01 00:00:00','military missile','success',1507,'VMF','','','','Test',190,'PLBR',1000 from dual union all
select 50598,'1980-S140','1980-07-11 16:01:00','military missile','success',939,'','FOT-41','','','OT',421,'SSBN 642',500 from dual union all
select 50599,'1980-S141','1980-07-11 16:02:00','military missile','success',939,'','FOT-41','','','OT',421,'SSBN 642',500 from dual union all
select 506,'1945-A25','1945-04-01 00:00:00','military missile','success',944,'8','','','','Test',435,'',1 from dual union all
select 5060,'1958-M28','1958-12-15 21:37:00','suborbital rocket','success',366,'','RW-108','','','Aeron',1490,'',80 from dual union all
select 50600,'1980-S142','1980-07-11 16:02:00','military missile','success',939,'','FOT-41','','','OT',421,'SSBN 642',500 from dual union all
select 50601,'1980-S143','1980-07-14 22:00:00','military missile','success',939,'','DASO-48','','','OT',421,'SSBN 634',500 from dual union all
select 50602,'1980-S144','1980-07-15 17:00:00','suborbital rocket','success',211,'NASA 27.44US','','','','Solar',1895,'',322 from dual union all
select 50603,'1980-S145','1980-07-22 03:10:00','military missile','success',1442,'','DASO-5','','','OT',421,'SSBN 629',1000 from dual union all
select 50604,'1980-S146','1980-07-26 23:38:00','suborbital rocket','success',921,'P223K','VLF propagation','','','Ionospheric',413,'',168 from dual union all
select 50605,'1980-S147','1980-07-28 19:00:00','suborbital rocket','success',1319,'NASA 33.13UE','','','','Plasma',1821,'',229 from dual union all
select 50606,'1980-S148','1980-07-31 12:18:00','suborbital rocket','success',556,'NASA 33.14GE','E-Field M/DTR','','','Plasma/Test',1821,'',111 from dual union all
select 50607,'1980-S149','1980-08-09 07:30:00','suborbital rocket','success',133,'NASA 25.53GG','UIT Proto','','','UV Astron',1895,'',214 from dual union all
select 50608,'1980-S15','1980-02-28 03:20:00','military missile','success',1006,'45033-416-44T','15A18 UBP-2 ZP','GCh 15F183','','OT',485,'',1000 from dual union all
select 50609,'1980-S150','1980-08-14 02:02:00','suborbital rocket','success',1319,'NASA 33.05UA','','','','Aeronomy',1873,'',192 from dual union all
select 5061,'1958-M29','1958-12-19 00:00:00','test rocket','success',575,'2','High Virgo','','','Test',21,'B-58',76 from dual union all
select 50610,'1980-S151','1980-08-14 02:40:00','suborbital rocket','success',860,'NASA 31.21UE','','','','Plasma',1821,'',90 from dual union all
select 50611,'1980-S152','1980-08-18 09:50:00','suborbital rocket','success',117,'GL A24.6S1-01','ZIP I (BMP)','','','IR Astron',1873,'',401 from dual union all
select 50612,'1980-S153','1980-08-20 00:00:00','military missile','success',799,'Long range OT','Mk 12 RV x 3','FOT GT76GB','GLORY TRIP 76GB','OT',1712,'',1300 from dual union all
select 50613,'1980-S154','1980-08-20 21:19:00','military missile','success',939,'','FOT-42','','','OT',421,'SSBN 625',500 from dual union all
select 50614,'1980-S155','1980-08-20 21:19:00','military missile','success',939,'','FOT-42','','','OT',421,'SSBN 625',500 from dual union all
select 50615,'1980-S156','1980-08-20 21:27:00','military missile','success',939,'','FOT-42','','','OT',421,'SSBN 625',500 from dual union all
select 50616,'1980-S157','1980-08-20 21:32:00','military missile','success',939,'','FOT-42','','','OT',421,'SSBN 625',500 from dual union all
select 50617,'1980-S158','1980-08-26 11:40:00','suborbital rocket','success',655,'K-10-14','GIR/GXV/STS','','','XR Astron/IR Astron',839,'',219 from dual union all
select 50618,'1980-S159','1980-08-27 00:00:00','suborbital rocket','success',1217,'XV-08','','','','Ionos?',330,'',245 from dual union all
select 50619,'1980-S16','1980-05-27 03:55:00','military missile','success',1006,'51033-426-43T','15A18 UBP-3 ZP','GCh 15F183','','OT',487,'',1000 from dual union all
select 5062,'1958-M30','1958-12-23 03:03:00','suborbital rocket','success',662,'K-6-TW-5','K-6-13','Grenade','','Aeron',7,'',60 from dual union all
select 50620,'1980-S160','1980-08-27 02:19:00','military missile','success',1442,'','OT-5','','','OT',421,'SSBN 655',1000 from dual union all
select 50621,'1980-S161','1980-08-27 02:20:00','military missile','success',1442,'','OT-5','','','OT',421,'SSBN 655',1000 from dual union all
select 50622,'1980-S162','1980-08-27 02:20:00','military missile','success',1442,'','OT-5','','','OT',421,'SSBN 655',1000 from dual union all
select 50623,'1980-S163','1980-08-29 00:00:00','military missile','success',1004,'50L','15A14 UBP-27 DP','GCh 15F678','','OT',492,'',1000 from dual union all
select 50624,'1980-S164','1980-09-01 00:00:00','suborbital rocket','success',1268,'SWERVE II-1','','','','Reentry Test',855,'',200 from dual union all
select 50625,'1980-S165','1980-09-01 00:00:00','suborbital rocket','success',1268,'SWERVE II-2','','','','Reentry Test',855,'',200 from dual union all
select 50626,'1980-S166','1980-09-02 10:40:00','suborbital rocket','success',668,'K-9M-70','','','','Test/Aeron/Ionos',839,'',230 from dual union all
select 50627,'1980-S167','1980-09-04 12:45:00','suborbital rocket','success',668,'K-9M-71','','','','Aeron?',839,'',308 from dual union all
select 50628,'1980-S168','1980-09-13 22:00:00','suborbital rocket','success',1300,'TT-500A-8F','','','','Test',1476,'',320 from dual union all
select 50629,'1980-S169','1980-09-15 00:00:00','suborbital rocket','success',1319,'NASA 33.06GU','','','','Aeronomy',1873,'',200 from dual union all
select 5063,'1958-M31','1958-05-23 22:39:00','suborbital rocket','success',45,'SS12.48','Grenades','IGY SS12.48','','Aeron',1895,'',60 from dual union all
select 50630,'1980-S17','1980-09-30 02:30:00','military missile','success',1006,'67083-436-43T','15A18 UBP-4 ZP','GCh 15F183','','OT',487,'',1000 from dual union all
select 50631,'1980-S170','1980-09-16 00:15:00','military missile','success',795,'','STREP-3','ABRES STREP-3','','Reentry Test',1701,'',1300 from dual union all
select 50632,'1980-S171','1980-09-17 00:00:00','military missile','success',799,'Long range OT','Mk 12 RV x 3','FOT GT77GM','GLORY TRIP 77GM','OT, Long Range test',1705,'',1400 from dual union all
select 50633,'1980-S172','1980-09-17 00:00:00','military missile','success',362,'','DOT 3','','','Interceptor',871,'',296 from dual union all
select 50634,'1980-S173','1980-09-18 16:30:00','suborbital rocket','success',37,'GL A04.801','UV','','','Solar UV',1895,'',197 from dual union all
select 50635,'1980-S174','1980-09-20 01:30:00','suborbital rocket','success',117,'NASA 24.03CH','LCXT','','','XR Astron',1873,'',309 from dual union all
select 50636,'1980-S175','1980-09-23 20:25:00','suborbital rocket','success',211,'NASA 27.36CS','XSST/TRC 2','','','Solar UV/X',1895,'',296 from dual union all
select 50637,'1980-S176','1980-09-24 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT78GM','GLORY TRIP 78GM','OT',1703,'',1300 from dual union all
select 50638,'1980-S177','1980-09-26 22:35:00','suborbital rocket','success',211,'NASA 27.35DH','','','','XR Astron',1895,'',300 from dual union all
select 50639,'1980-S178','1980-09-30 18:16:00','suborbital rocket','success',228,'NASA 21.65UL','','','','Aeron',1873,'',281 from dual union all
select 5064,'1958-S01','1958-01-01 00:00:00','suborbital rocket','success',720,'LT2','','','','Test',1839,'',100 from dual union all
select 50640,'1980-S179','1980-10-07 03:36:42','suborbital rocket','success',891,'NASA 30.10GU','TY2-7688','','','Ionospheric',29,'',87 from dual union all
select 50641,'1980-S18','1980-10-31 03:05:00','military missile','success',1006,'51067-534-22T','15A18 UBP-5 ZP','GCh 15F183','','OT',485,'',1000 from dual union all
select 50642,'1980-S180','1980-10-07 03:37:42','suborbital rocket','success',556,'NASA 33.15GE','E-Field (MAE)','','','Plasma',29,'',122 from dual union all
select 50643,'1980-S181','1980-10-08 00:00:00','military missile','success',795,'','AMARV FTV-2','ABRES AMARV-2','','Reentry Test',1698,'',1300 from dual union all
select 50644,'1980-S182','1980-10-08 18:39:00','suborbital rocket','success',1319,'NASA 33.07UL','','','','Aeronomy',1873,'',200 from dual union all
select 50645,'1980-S183','1980-10-11 21:57:54','suborbital rocket','success',556,'NASA 33.16GE','E-Field (MAE)','','','Plasma',29,'',120 from dual union all
select 50646,'1980-S184','1980-10-11 21:57:54','suborbital rocket','success',891,'NASA 30.11GU','TY2-7684','','','Ionospheric',29,'',83 from dual union all
select 50647,'1980-S185','1980-10-07 16:10:00','military missile','unknown',917,'AO-197','P-397  80 Fall 01','','','OT',433,'',180 from dual union all
select 50648,'1980-S186','1980-10-07 16:54:00','military missile','unknown',917,'AO-198','P-398  80 Fall 02','','','OT',433,'',180 from dual union all
select 50649,'1980-S187','1980-10-15 14:02:00','military missile','unknown',917,'AO-199','P-399  80 Fall 03','','','OT',433,'',180 from dual union all
select 5065,'1958-S02','1958-01-01 00:00:00','suborbital rocket','success',720,'LT3','','','','Test',1839,'',100 from dual union all
select 50650,'1980-S188','1980-10-15 14:34:00','military missile','unknown',917,'AO-200','P-400  80 Fall 04','','','OT',433,'',180 from dual union all
select 50651,'1980-S189','1980-10-18 02:08:14','suborbital rocket','success',556,'NASA 33.17GE','E-Field (MAE)','','','Plasma',29,'',117 from dual union all
select 50652,'1980-S19','1980-01-28 00:00:00','military missile','failure',1057,'1      -','','','','Test',989,'',100 from dual union all
select 50653,'1980-S190','1980-10-18 02:50:25','suborbital rocket','success',891,'NASA 30.12GU','','','','Ionospheric',29,'',85 from dual union all
select 50654,'1980-S191','1980-10-22 20:31:00','suborbital rocket','success',906,'GL A10.901-1','SPE-1','','','Plasma',1047,'',88 from dual union all
select 50655,'1980-S192','1980-10-28 21:25:00','military missile','success',1442,'','DASO-6','','','OT',421,'SSBN 630',1000 from dual union all
select 50656,'1980-S193','1980-11-01 06:44:00','suborbital rocket','success',133,'NASA 25.50UG','','0.3m UTV','','UV Astron',1895,'',227 from dual union all
select 50657,'1980-S194','1980-11-11 00:12:00','suborbital rocket','success',921,'P221K','EBC-C E5','','','Aeronomy',413,'',170 from dual union all
select 50658,'1980-S195','1980-11-11 00:12:00','suborbital rocket','success',837,'Ferdinand 54','EBC-C E4  Bugatti II','','','Ionospheric',29,'',123 from dual union all
select 50659,'1980-S196','1980-11-11 00:12:12','suborbital rocket','success',860,'DLR A-NO-101','EBC-C Ferd.57/Trinom','','','Ionos',29,'',146 from dual union all
select 5066,'1958-S03','1958-01-01 00:00:00','suborbital rocket','success',720,'LT4','','','','Test',1837,'',100 from dual union all
select 50660,'1980-S197','1980-11-11 00:27:00','suborbital rocket','success',860,'DLR K-NO-105','EBC-C E6C','','','Ionos/Aeron',417,'',198 from dual union all
select 50661,'1980-S198','1980-11-11 00:32:00','suborbital rocket','success',1189,'MPSC-8001A','EBC-C E7','','','Aeron',29,'',99 from dual union all
select 50662,'1980-S199','1980-11-14 20:00:00','military missile','success',932,'A3TK','CHEVALINE PS1','','','Test',421,'UK S26',1000 from dual union all
select 50663,'1980-S20','1980-04-04 00:00:00','military missile','failure',1057,'2      -','','','','Test',989,'',100 from dual union all
select 50664,'1980-S200','1980-11-16 03:14:18','suborbital rocket','success',921,'P220K','EBC-B E5','','','Aeronomy',413,'',174 from dual union all
select 50665,'1980-S201','1980-11-16 03:13:00','suborbital rocket','success',1212,'DLR K-GR-104','EBC-B E2 (ENERGY EB1)','','','Ionos/Plasma',416,'',231 from dual union all
select 50666,'1980-S202','1980-11-16 03:16:00','suborbital rocket','success',1319,'A13.073','ENERGY B','','','Ionos',29,'',195 from dual union all
select 50667,'1980-S203','1980-11-16 03:31:00','suborbital rocket','success',837,'Ferdinand 53','EBC-B E4  Bugatti II','','','Ionospheric',29,'',124 from dual union all
select 50668,'1980-S204','1980-11-16 03:31:12','suborbital rocket','success',860,'DLR A-NO-102','EBC-B Ferd.55/Trinom','','','Ionos',29,'',149 from dual union all
select 50669,'1980-S205','1980-11-16 03:46:00','suborbital rocket','success',1189,'MPSC-8002A','EBC-B E7','','','Aeron',29,'',87 from dual union all
select 5067,'1958-S04','1958-01-01 00:00:00','suborbital rocket','success',720,'LT5','','','','Test',1837,'',100 from dual union all
select 50670,'1980-S206','1980-11-16 03:49:40','suborbital rocket','success',1319,'NASA 33.10UE','EBC-B E11a','','','Auroral',412,'',180 from dual union all
select 50671,'1980-S207','1980-11-16 04:15:00','suborbital rocket','success',1319,'NASA 33.11UE','EBC-B E11b','','','Auroral',412,'',181 from dual union all
select 50672,'1980-S208','1980-11-16 04:47:00','suborbital rocket','success',860,'DLR K-NO-106','EBC-B E6A1 S10.12-1','','','Ionos/Aeron',417,'',165 from dual union all
select 50673,'1980-S209','1980-11-18 16:30:00','military missile','success',932,'A3TK','CHEVALINE PS2','','','Test',421,'UK S26',1000 from dual union all
select 50674,'1980-S21','1980-06-17 00:00:00','military missile','failure',1057,'3      -','','','','Test',989,'',100 from dual union all
select 50675,'1980-S210','1980-11-19 00:50:00','suborbital rocket','success',211,'NASA 27.19NP','SPAR 8 (R-19)','','','Micrograv',1895,'',198 from dual union all
select 50676,'1980-S211','1980-11-21 07:52:00','suborbital rocket','success',740,'','Energy Budget','','','Aeron/Ionos',1010,'',166 from dual union all
select 50677,'1980-S212','1980-11-25 08:10:00','suborbital rocket','success',740,'','Energy Budget','','','Aeron/Ionos',1010,'',166 from dual union all
select 50678,'1980-S213','1980-11-28 00:00:00','military missile','success',756,'1','','','','Test',301,'',800 from dual;
insert into LAUNCH
select 50679,'1980-S214','1980-11-28 03:24:36','suborbital rocket','success',837,'Ferdinand 52','EBC-A E4  Bugatti II','','','Ionospheric',29,'',97 from dual union all
select 5068,'1958-S05','1958-01-01 00:00:00','suborbital rocket','success',720,'LT6','','','','Test',1837,'',100 from dual union all
select 50680,'1980-S215','1980-11-28 03:24:48','suborbital rocket','success',860,'DLR A-NO-103','EBC-A Ferd.56 Trinom','','','Ionos',29,'',146 from dual union all
select 50681,'1980-S216','1980-11-28 03:44:36','suborbital rocket','success',1189,'MPSC-8003A','EBC-A2 E7','','','Aeron',29,'',94 from dual union all
select 50682,'1980-S217','1980-11-30 23:45:30','suborbital rocket','success',921,'P222K','EBC-A2 E5','','','Aeronomy',413,'',169 from dual union all
select 50683,'1980-S218','1980-11-30 23:44:30','suborbital rocket','success',1319,'NASA 33.09UE','EBC-A2 E11a','','','Auroral',412,'',171 from dual union all
select 50684,'1980-S219','1980-12-01 00:09:00','suborbital rocket','success',860,'DLR K-NO-107','EBC-A2 E6A2','','','Ionos/Aeron',417,'',161 from dual union all
select 50685,'1980-S22','1980-12-27 00:00:00','military missile','success',1057,'7      56A/49x','5 RV','','','Test',989,'',1000 from dual union all
select 50686,'1980-S220','1980-12-01 01:23:40','suborbital rocket','success',860,'DLR K-NO-108','EBC-A2 E6B','','','Ionos/Aeron',417,'',172 from dual union all
select 50687,'1980-S221','1980-12-04 23:05:00','suborbital rocket','success',1349,'','DNA PLACES GAIL','','','Plasma/Ionos',386,'',300 from dual union all
select 50688,'1980-S222','1980-12-05 02:00:00','test rocket','success',647,'','BOR-4','','','Reentry Test',621,'',198 from dual union all
select 50689,'1980-S223','1980-12-05 08:20:00','suborbital rocket','success',740,'','Energy Budget','','','Aeron/Ionos',1010,'',166 from dual union all
select 5069,'1958-S06','1958-01-08 00:00:00','suborbital rocket','success',838,'II8.35','','','','Test',1895,'',145 from dual union all
select 50690,'1980-S224','1980-12-06 23:05:00','suborbital rocket','success',1349,'','DNA PLACES HOPE','','','Plasma/Ionos',386,'',300 from dual union all
select 50691,'1980-S225','1980-12-08 23:10:00','suborbital rocket','success',1349,'','DNA PLACES IRIS','','','Plasma/Ionos',386,'',300 from dual union all
select 50692,'1980-S226','1980-12-08 23:38:36','suborbital rocket','success',1349,'','DNA PLACES beacon 1','','','Plasma/Ionos',386,'',300 from dual union all
select 50693,'1980-S227','1980-12-08 23:51:36','suborbital rocket','success',1349,'','DNA PLACES beacon 2','','','Plasma/Ionos',386,'',300 from dual union all
select 50694,'1980-S228','1980-12-09 06:30:00','suborbital rocket','success',211,'NASA 27.47GG','','','','UV Astron?',1895,'',301 from dual union all
select 50695,'1980-S229','1980-12-10 00:00:00','military missile','success',1106,'2601','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 50696,'1980-S23','1980-04-19 00:00:00','military missile','success',1453,'51163-92-854T','15A35 UBP-4 ZP','','','OT',505,'',1000 from dual union all
select 50697,'1980-S230','1980-12-12 01:05:00','suborbital rocket','success',740,'','Energy Budget','','','Aeron/Ionos',1010,'',166 from dual union all
select 50698,'1980-S231','1980-12-12 01:45:00','suborbital rocket','success',740,'','Energy Budget','','','Aeron/Ionos',1010,'',166 from dual union all
select 50699,'1980-S232','1980-12-12 07:15:00','suborbital rocket','success',740,'','Energy Budget','','','Aeron/Ionos',1010,'',166 from dual union all
select 507,'1945-A26','1945-04-01 00:00:00','military missile','success',944,'9','','','','Test',435,'',1 from dual union all
select 5070,'1958-S07','1958-01-10 15:48:00','military missile','success',142,'10A','R&D Fairing','R&D','','Test',227,'',120 from dual union all
select 50700,'1980-S233','1980-12-12 17:19:00','suborbital rocket','success',867,'NASA 18.203UE','','','','Ionospheric',1408,'',211 from dual union all
select 50701,'1980-S234','1980-12-12 23:11:00','suborbital rocket','success',1349,'','DNA PLACES JAN Ba','','','Plasma/Ionos',386,'',300 from dual union all
select 50702,'1980-S235','1980-12-12 23:42:50','suborbital rocket','success',1349,'','DNA PLACES JAN Probe','','','Plasma/Ionos',386,'',241 from dual union all
select 50703,'1980-S236','1980-12-16 00:00:00','military missile','success',795,'','ABRV-4','ABRES ABRV-4','','Reentry Test',1701,'',1300 from dual union all
select 50704,'1980-S237','1980-12-17 01:20:00','suborbital rocket','success',740,'','Energy Budget','','','Aeron/Ionos',1010,'',166 from dual union all
select 50705,'1980-S238','1980-12-18 23:00:00','suborbital rocket','success',740,'','Energy Budget','','','Aeron/Ionos',1010,'',166 from dual union all
select 50706,'1980-S239','1980-12-20 17:32:00','suborbital rocket','success',867,'NASA 18.204UE','','','','Ionospheric',1408,'',200 from dual union all
select 50707,'1980-S24','1980-08-26 00:00:00','military missile','success',1453,'','15A35 UBP-5 ZP','','','OT',535,'',1000 from dual union all
select 50708,'1980-S240','1980-12-22 00:00:00','suborbital rocket','success',1219,'XV-09','','','','Ionos?',330,'',600 from dual union all
select 50709,'1980-S241','1980-10-01 15:33:00','suborbital rocket','success',1028,'F-16/560.16','Spread F (360/80)','','','Ionos',1361,'',350 from dual union all
select 5071,'1958-S09','1958-01-15 01:24:00','military missile','success',635,'CC-45    TS','Hardtack pod test','','','Test',280,'',90 from dual union all
select 50710,'1980-S242','1980-02-18 01:50:00','suborbital rocket','success',290,'','O3 Photometer','','','Aeron',1467,'',100 from dual union all
select 50711,'1980-S246','1980-02-09 00:00:00','military missile','success',359,'','Test','','','Test',809,'',1000 from dual union all
select 50712,'1980-S247','1980-08-02 15:01:00','military missile','success',359,'802013','','','','Test',809,'',1000 from dual union all
select 50713,'1980-S248','1980-10-31 00:00:00','military missile','success',359,'','','','','Test',810,'',1000 from dual union all
select 50714,'1980-S249','1980-11-01 00:00:00','military missile','success',983,'Scud B','69 against Iran','','','Weapon',761,'',100 from dual union all
select 50715,'1980-S25','1980-12-26 00:00:00','military missile','success',1453,'51163-92-853','15A35 UBP-6 ZP','','','OT',505,'',1000 from dual union all
select 50716,'1980-S250','1980-10-28 00:00:00','military missile','unknown',917,'AO-201','P-401 80 Fall 05','','','OT',433,'',180 from dual union all
select 50717,'1980-S251','1980-10-28 00:00:00','military missile','unknown',917,'AO-202','P-402 80 Fall 06','','','OT',433,'',180 from dual union all
select 50718,'1980-S252','1980-11-18 00:00:00','military missile','unknown',917,'AO-203','P-403 80 Fall 07','','','OT',433,'',180 from dual union all
select 50719,'1980-S253','1980-11-18 00:00:00','military missile','unknown',917,'AO-204','P-404  80 Fall 08','','','OT',433,'',180 from dual union all
select 5072,'1958-S10','1958-01-17 12:59:00','military missile','success',934,'1-204-10','Jetavator','','','Test',253,'',20 from dual union all
select 50720,'1980-S254','1980-01-01 00:00:00','military missile','success',923,'U06','15Zh53 LKI-6','','','Test',636,'',500 from dual union all
select 50721,'1980-S255','1980-01-01 00:00:00','military missile','success',923,'U07','15Zh53 LKI-7','','','Test',636,'',500 from dual union all
select 50722,'1980-S256','1980-01-01 00:00:00','military missile','success',923,'U08','15Zh53 LKI-8','','','Test',636,'',500 from dual union all
select 50723,'1980-S257','1980-01-01 00:00:00','military missile','success',923,'U09','15Zh53 LKI-9','','','Test',636,'',500 from dual union all
select 50724,'1980-S258','1980-08-14 00:00:00','military missile','success',923,'U10','15Zh53 LKI-10','','','Test',636,'',500 from dual union all
select 50725,'1980-S259','1980-07-01 00:00:00','military missile','success',886,'','Stage 2, last','','','Test',636,'',120 from dual union all
select 50726,'1980-S26','1980-01-16 03:00:00','suborbital rocket','success',668,'K-9M-69','TPE-1','','','Ionos/Tether',839,'',328 from dual union all
select 50727,'1980-S260','1980-11-11 01:21:30','suborbital rocket','success',1189,'E7','EBC   E7','','','Aeron',417,'',97 from dual union all
select 50728,'1980-S261','1980-11-16 05:37:30','suborbital rocket','success',1189,'E7','EBC   E7','','','Aeron',417,'',96 from dual union all
select 50729,'1980-S262','1980-12-01 02:09:30','suborbital rocket','success',1189,'E7','EBC   E7','','','Aeron',417,'',92 from dual union all
select 5073,'1958-S100','1958-07-12 06:36:00','military missile','success',1366,'123','R&D Series III','','','Test',234,'',520 from dual union all
select 50730,'1980-S263','1980-04-18 04:00:00','military missile','success',745,'45067-524-13T','15A16 UBP-3 ZP','GCh 15F161','','OT',528,'',1000 from dual union all
select 50731,'1980-S264','1980-11-27 00:00:00','military missile','success',1004,'504','15A14 UBP-28 DKh','GCh 15F142','','OT',596,'',1000 from dual union all
select 50732,'1980-S265','1980-04-23 00:00:00','military missile','success',1044,'45733-317-04','8K98P UBP-17','','','Test',477,'',1000 from dual union all
select 50733,'1980-S266','1980-05-27 00:00:00','military missile','success',1037,'Yu64612-30','8K98 UBP-50','','','Test',477,'',1000 from dual union all
select 50734,'1980-S267','1980-06-10 00:00:00','military missile','success',1037,'Yu64612-22','8K98 UBP-51','','','Test',477,'',1000 from dual union all
select 50735,'1980-S268','1980-07-03 00:00:00','military missile','success',1037,'Yu64612-23','8K98 UBP-52','','','Test',477,'',1000 from dual union all
select 50736,'1980-S269','1980-08-26 00:00:00','military missile','failure',1044,'','8K98P UBP-18','','','Test',477,'',0 from dual union all
select 50737,'1980-S27','1980-01-18 06:20:00','suborbital rocket','success',1074,'S-520-1','Heat Pipe','','','Aeron/Ionos',833,'',329 from dual union all
select 50738,'1980-S270','1980-09-25 00:00:00','military missile','success',1037,'Kh29521-31','8K98 UBP-53','','','Test',477,'',1000 from dual union all
select 50739,'1980-S271','1980-10-30 00:00:00','military missile','success',1037,'Yu45713-27','8K98 UBP-54','','','Test',477,'',1000 from dual union all
select 5074,'1958-S101','1958-07-15 20:07:31','suborbital rocket','success',839,'AM6.39','','IGY AM6.39','','Aeron',428,'',126 from dual union all
select 50740,'1980-S272','1980-11-28 00:00:00','military missile','success',1037,'V29502-05','8K98 UBP-55','','','Test',477,'',1000 from dual union all
select 50741,'1980-S273','1980-06-05 00:00:00','military missile','success',1452,'49242-83-427','15A30 60 RD UBP-5','','','OT',1461,'',1000 from dual union all
select 50742,'1980-S274','1980-07-01 00:00:00','military missile','success',1452,'','15A30 60 RD UBP-6','','','OT',1461,'',1000 from dual union all
select 50743,'1980-S275','1980-01-01 00:00:00','military missile','success',1450,'','15A20 54 RD UBP-4','','','OT',1468,'',1000 from dual union all
select 50744,'1980-S276','1980-01-01 00:00:00','military missile','success',1450,'','15A20 54 RD UBP-5','','','OT',1468,'',1000 from dual union all
select 50745,'1980-S277','1980-02-15 00:00:00','military missile','success',436,'','','','','Test',1494,'',1000 from dual union all
select 50746,'1980-S278','1980-12-16 12:49:00','suborbital rocket','success',837,'INTA NA/FL-8001','','','','Ionospheric',396,'',150 from dual union all
select 50747,'1980-S279','1980-12-18 11:44:00','suborbital rocket','success',837,'INTA NA/FL-8002','','','','Ionospheric',396,'',150 from dual union all
select 50748,'1980-S28','1980-01-21 08:05:00','suborbital rocket','success',133,'NASA 25.051UH','Wisc. Aerobee IV','','','XR Astron',1895,'',216 from dual union all
select 50749,'1980-S280','1980-01-01 00:00:00','military missile','success',1450,'','15A20 54 RD UBP-6','','','OT',1468,'',1000 from dual union all
select 5075,'1958-S102','1958-07-16 22:40:28','suborbital rocket','success',839,'OB6.04','B/ne probe','IGY OB6.04','','Fields',428,'',142 from dual union all
select 50750,'1980-S281','1980-03-21 00:00:00','military missile','success',922,'','15Zh45  8 RD UBP','','','Test',636,'',500 from dual union all
select 50751,'1980-S282','1980-04-18 00:00:00','military missile','success',922,'','15Zh45 42 RD UBP','','','Test',636,'',500 from dual union all
select 50752,'1980-S283','1980-05-24 00:00:00','military missile','success',922,'','15Zh45 39 RD UBP','','','Test',636,'',500 from dual union all
select 50753,'1980-S284','1980-07-30 00:00:00','military missile','success',922,'','15Zh45 42 RD UBP','','','Test',636,'',500 from dual union all
select 50754,'1980-S285','1980-10-31 00:00:00','military missile','success',922,'','15Zh45  8 RD UBP','','','Test',636,'',500 from dual union all
select 50755,'1980-S286','1980-11-01 00:00:00','military missile','success',922,'','15Zh45 33 RD UBP','','','Test',636,'',500 from dual union all
select 50756,'1980-S287','1980-06-01 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 50757,'1980-S288','1980-06-01 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 50758,'1980-S289','1980-06-01 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 50759,'1980-S29','1980-01-24 17:21:59','suborbital rocket','success',867,'Ferdinand 50','Ferd. 50/COROBIER','','','Test/Ionos/Plasma',29,'',202 from dual union all
select 5076,'1958-S103','1958-07-17 07:21:41','suborbital rocket','success',839,'AA6.15','','IGY AA6.15','','Aeron',428,'',169 from dual union all
select 50760,'1980-S290','1980-10-01 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 50761,'1980-S291','1980-10-01 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 50762,'1980-S292','1980-07-17 00:00:00','military missile','unknown',980,'','8K65U UBP 29 RD','','','Test',636,'',675 from dual union all
select 50763,'1980-S293','1980-07-31 00:00:00','military missile','unknown',980,'','8K65U UBP 29 RD','','','Test',636,'',675 from dual union all
select 50764,'1980-S30','1980-01-27 09:17:00','suborbital rocket','success',1337,'NASA 29.14UE','ARCS 1','','','Plasma',1047,'',433 from dual union all
select 50765,'1980-S31','1980-01-28 01:30:00','suborbital rocket','success',1299,'TT-500-7F','','','','Test',1476,'',300 from dual union all
select 50766,'1980-S32','1980-01-31 00:00:00','military missile','success',799,'PVM-18','Mk 12 RV x 3','R&D','PVM-18','Test',1707,'',1300 from dual union all
select 50767,'1980-S33','1980-02-02 08:47:00','suborbital rocket','success',1073,'S-310-8','','','','Aeron',839,'',179 from dual union all
select 50768,'1980-S34','1980-02-03 01:12:00','military missile','success',939,'','FOT-38','','','OT',421,'SSBN 654',500 from dual;
insert into LAUNCH
select 50769,'1980-S35','1980-02-03 01:13:00','military missile','success',939,'','FOT-38','','','OT',421,'SSBN 654',500 from dual union all
select 5077,'1958-S104','1958-07-17 09:04:00','military missile','success',638,'AM-6B','Gaslight','','','Reentry Test',249,'',500 from dual union all
select 50770,'1980-S36','1980-02-03 01:13:00','military missile','success',939,'','FOT-38','','','OT',421,'SSBN 654',500 from dual union all
select 50771,'1980-S37','1980-02-04 02:42:00','military missile','success',939,'','FOT-39','','','OT',421,'SSBN 654',500 from dual union all
select 50772,'1980-S38','1980-02-05 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT71GM','GLORY TRIP 71GM','OT',1705,'',1300 from dual union all
select 50773,'1980-S39','1980-02-05 00:00:00','military missile','success',362,'','DOT 2','','','Interceptor',871,'',324 from dual union all
select 50774,'1980-S40','1980-02-12 15:51:00','military missile','unknown',917,'FOT-131','P-387 80 Cape 01','','','OT',231,'',180 from dual union all
select 50775,'1980-S41','1980-02-12 16:27:00','military missile','unknown',917,'FOT-129','P-388 80 Cape 02','','','OT',231,'',180 from dual union all
select 50776,'1980-S42','1980-02-13 22:00:02','military missile','success',939,'','DASO','','','OT',421,'SSBN 635',500 from dual union all
select 50777,'1980-S43','1980-02-15 11:43:24','suborbital rocket','success',227,'ADD-VB-50','Pulsating aurora','','','Auroral',1350,'',156 from dual union all
select 50778,'1980-S44','1980-02-16 08:23:00','suborbital rocket','success',211,'NASA 27.43AS','LASL Eclipse La','','','Solar Eclipse',1412,'',319 from dual union all
select 50779,'1980-S45','1980-02-16 08:23:30','suborbital rocket','success',211,'NASA 27.42AS','LASL Eclipse La','','','Solar Eclipse',1412,'',298 from dual union all
select 5078,'1958-S105','1958-07-17 17:48:00','suborbital rocket','success',838,'NN8.54F','','IGY NN8.54F','','Test/Solar UV/Solar',1415,'',225 from dual union all
select 50780,'1980-S46','1980-02-16 08:50:00','suborbital rocket','success',1028,'F-14','Eclipse control','','','Eclipse',1361,'',330 from dual union all
select 50781,'1980-S47','1980-02-16 09:24:00','suborbital rocket','success',290,'ISRO 5.60','Eclipse D1','','','Eclipse Ionos',1467,'',140 from dual union all
select 50782,'1980-S48','1980-02-16 16:32:00','suborbital rocket','success',211,'NASA 27.37US','CfA/HAO Rocket Coron. 2','','','Solar UV/Solar Opt',1895,'',268 from dual union all
select 50783,'1980-S49','1980-02-16 09:52:00','suborbital rocket','success',290,'ISRO 5.61','Eclipse D2','','','Eclipse Ionos',1467,'',140 from dual union all
select 50784,'1980-S50','1980-02-16 10:15:00','suborbital rocket','failure',1028,'F-15','Eclipse F-region','','','Eclipse',1361,'',0 from dual union all
select 50785,'1980-S51','1980-02-16 13:25:00','suborbital rocket','success',290,'ISRO 5.63','Eclipse Ba/Li','','','Eclipse Aeron',1467,'',140 from dual union all
select 50786,'1980-S52','1980-02-17 00:00:00','suborbital rocket','success',290,'ISRO 5.65/C.59','NO Photometer','','','Aeron',1467,'',150 from dual union all
select 50787,'1980-S53','1980-02-17 09:52:00','suborbital rocket','success',290,'ISRO 5.64','Eclipse D3','','','Eclipse Ionos',1467,'',140 from dual union all
select 50788,'1980-S54','1980-02-21 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT72GB','GLORY TRIP 72GB','OT',1712,'',1300 from dual union all
select 50789,'1980-S55','1980-02-21 00:53:49','suborbital rocket','success',867,'Ferdinand 49','PULSAUR I','','','Test/Auroral/Ionos/',29,'',196 from dual union all
select 5079,'1958-S106','1958-07-18 00:00:00','test rocket','success',253,'Missile 3','','','','Test',293,'B-47',100 from dual union all
select 50790,'1980-S56','1980-02-23 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT140M','GLORY TRIP 140M','OT',1699,'',1300 from dual union all
select 50791,'1980-S57','1980-02-23 10:05:09','suborbital rocket','success',227,'ADD-VB-52','Pulsating aurora','','','Auroral',1350,'',148 from dual union all
select 50792,'1980-S58','1980-02-27 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT73GM','GLORY TRIP 73GM','OT',1703,'',1300 from dual union all
select 50793,'1980-S59','1980-02-28 13:13:00','military missile','success',1442,'','OT-1','','','OT',421,'SSBN 657',1000 from dual union all
select 50794,'1980-S60','1980-02-28 13:13:00','military missile','success',1442,'','OT-1','','','OT',421,'SSBN 657',1000 from dual union all
select 50795,'1980-S61','1980-02-28 14:15:00','military missile','success',1442,'','OT-1','','','OT',421,'SSBN 657',1000 from dual union all
select 50796,'1980-S62','1980-02-28 14:15:00','military missile','success',1442,'','OT-1','','','OT',421,'SSBN 657',1000 from dual union all
select 50797,'1980-S63','1980-03-06 12:00:00','military missile','success',745,'45033-416-45','15A16 UBP-2 ZP','GCh 15F161','','OT',524,'',1000 from dual union all
select 50798,'1980-S64','1980-03-06 00:00:00','military missile','success',1004,'48L','15A14 UBP-24 DP','GCh 15F678','','OT',496,'',1000 from dual union all
select 50799,'1980-S65','1980-03-15 00:00:00','military missile','success',795,'','ABRV-3','ABRES ABRV-3','ABRV-3','Reentry Test',1701,'',1300 from dual union all
select 508,'1945-A27','1945-04-01 00:00:00','military missile','success',944,'10','','','','Test',435,'',1 from dual union all
select 5080,'1958-S107','1958-07-23 22:13:00','military missile','success',1352,'119','Able RTV','','','Reentry Test',233,'',1600 from dual union all
select 50800,'1980-S66','1980-03-16 11:55:39','suborbital rocket','success',1320,'NASA 34.03UE','KING CRAB','','','Plasma Ba',1047,'',571 from dual union all
select 50801,'1980-S67','1980-03-19 07:59:37','suborbital rocket','success',1320,'NASA 34.01UE','','','','Plasma',1047,'',596 from dual union all
select 50802,'1980-S68','1980-03-21 23:30:00','military missile','success',932,'A3TK PT-2','CHEVALINE PT2','','','Test',250,'',1000 from dual union all
select 50803,'1980-S69','1980-03-22 10:30:00','suborbital rocket','success',133,'NASA 25.47UH','Soft XRB 1','','','XR Astron',1895,'',217 from dual union all
select 50804,'1980-S70','1980-03-22 11:36:37','suborbital rocket','success',1320,'NASA 34.02UE','','','','Plasma',1047,'',618 from dual union all
select 50805,'1980-S71','1980-03-26 12:19:00','military missile','success',1442,'','DASO-4','','','OT',421,'SSBN 655',1000 from dual union all
select 50806,'1980-S72','1980-03-27 00:00:00','military missile','success',799,'PVM-19','Mk 12 RV x 3','R&D','PVM-19','Test',1707,'',1300 from dual union all
select 50807,'1980-S73','1980-03-28 06:26:00','suborbital rocket','success',133,'NASA 25.59UH','UM Airglow Payload','Mich. Auroral Probe','','Aurora',428,'',198 from dual union all
select 50808,'1980-S74','1980-04-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50809,'1980-S75','1980-04-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 5081,'1958-S108','1958-07-24 00:00:00','test rocket','success',1521,'Winder 4','Argus test','','','Test',1338,'AVM1',672 from dual union all
select 50810,'1980-S76','1980-04-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50811,'1980-S77','1980-04-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50812,'1980-S78','1980-04-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50813,'1980-S79','1980-04-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50814,'1980-S80','1980-04-01 12:36:00','military missile','success',1442,'','OT-2','','','OT',421,'SSBN 658',1000 from dual union all
select 50815,'1980-S81','1980-04-02 12:39:00','military missile','success',1442,'','OT-2','','','OT',421,'SSBN 658',1000 from dual union all
select 50816,'1980-S82','1980-04-02 14:23:00','military missile','success',1442,'','OT-2','','','OT',421,'SSBN 658',1000 from dual union all
select 50817,'1980-S83','1980-04-02 14:23:00','military missile','success',1442,'','OT-2','','','OT',421,'SSBN 658',1000 from dual union all
select 50818,'1980-S84','1980-04-06 03:33:01','suborbital rocket','success',210,'AAF-NVB-07','WATERHOLE 1','','','Auroral/Aeron/Ionos',428,'',307 from dual union all
select 50819,'1980-S85','1980-04-10 00:00:00','military missile','success',795,'','STREP-4','ABRES STREP-4','STREP-4','Reentry Test',1698,'',1300 from dual union all
select 5082,'1958-S109','1958-07-25 05:01:00','suborbital rocket','success',838,'NN8.55F','','IGY NN8.55F','','Test/Solar UV/Solar',1415,'',200 from dual union all
select 50820,'1980-S86','1980-04-11 00:15:00','military missile','success',932,'A3TK PT-1','CHEVALINE PT1','','','Test',250,'',1000 from dual union all
select 50821,'1980-S87','1980-04-21 21:47:00','military missile','success',939,'','FOT-40','','','OT',421,'SSBN 656',500 from dual union all
select 50822,'1980-S88','1980-04-21 21:47:00','military missile','success',939,'','FOT-40','','','OT',421,'SSBN 656',500 from dual union all
select 50823,'1980-S89','1980-04-21 21:47:00','military missile','success',939,'','FOT-40','','','OT',421,'SSBN 656',500 from dual union all
select 50824,'1980-S90','1980-04-21 21:48:00','military missile','success',939,'','FOT-40','','','OT',421,'SSBN 656',500 from dual union all
select 50825,'1980-S91','1980-04-23 15:04:00','military missile','unknown',917,'FOT-132','P-389 80 Cape 03','','','OT',231,'',180 from dual union all
select 50826,'1980-S92','1980-04-23 15:22:00','military missile','unknown',917,'FOT-133','P-390 80 Cape 04','','','OT',231,'',180 from dual union all
select 50827,'1980-S93','1980-04-23 15:39:00','military missile','unknown',917,'FOT-134','P-391 80 Cape 05','','','OT',231,'',180 from dual union all
select 50828,'1980-S94','1980-04-23 16:05:00','military missile','unknown',917,'FOT-135','P-392 80 Cape 06','','','OT',231,'',180 from dual union all
select 50829,'1980-S95','1980-04-25 00:00:00','military missile','success',1004,'47L','15A14 UBP-25 DP','GCh 15F678','','OT',492,'',1000 from dual union all
select 5083,'1958-S11','1958-01-25 19:11:05','suborbital rocket','success',839,'AM6.02','Sphere','IGY AM6.02','','Aeron',428,'',157 from dual union all
select 50830,'1980-S96','1980-04-28 02:09:00','suborbital rocket','unknown',740,'MMR-12','Ariel-4','','','Aeron/Ionos',636,'',155 from dual union all
select 50831,'1980-S97','1980-04-28 07:10:02','suborbital rocket','success',1212,'DLR K-GR-100','DLR TEXUS 3A','','','Micrograv',416,'',253 from dual union all
select 50832,'1980-S98','1980-04-28 15:00:00','suborbital rocket','unknown',740,'','Aelita-2','','','Aeron',1010,'',155 from dual union all
select 50833,'1980-S99','1980-05-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',140,'ZUB',150 from dual union all
select 50834,'1980-U01','1980-12-09 07:18:00','orbital','failure',157,'68E','-           [NOSS4]','PARCAE 4','','Sat',1776,'',1050 from dual union all
select 50835,'1980-W01','1980-01-02 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 50836,'1980-W02','1980-01-02 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 50837,'1980-W03','1980-01-02 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 50838,'1980-W04','1980-01-02 15:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 50839,'1980-W05','1980-01-02 17:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',66 from dual union all
select 5084,'1958-S110','1958-07-29 05:03:00','suborbital rocket','success',838,'NN8.56F','','IGY NN8.56F','','Test/Solar UV/Solar',1415,'',200 from dual union all
select 50840,'1980-W06','1980-01-03 00:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 50841,'1980-W07','1980-01-03 01:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 50842,'1980-W08','1980-01-03 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 50843,'1980-W09','1980-01-04 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 50844,'1980-W10','1980-01-04 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',67 from dual union all
select 50845,'1980-W100','1980-01-30 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 50846,'1980-W1000','1980-10-27 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 50847,'1980-W1001','1980-10-27 19:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 50848,'1980-W1002','1980-10-27 20:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 50849,'1980-W1003','1980-10-28 17:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 5085,'1958-S111','1958-08-01 00:00:00','suborbital rocket','success',440,'','','','','Aeron',405,'',84 from dual union all
select 50850,'1980-W1004','1980-10-28 19:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 50851,'1980-W1005','1980-10-28 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 50852,'1980-W1006','1980-10-29 02:00:00','suborbital rocket','success',761,'MT135P-R401','','','','Meteo',1332,'',50 from dual union all
select 50853,'1980-W1007','1980-10-29 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',209,'BUGAEV',59 from dual union all
select 50854,'1980-W1008','1980-10-29 13:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 50855,'1980-W1009','1980-10-29 16:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 50856,'1980-W101','1980-01-30 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 50857,'1980-W1010','1980-10-29 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 50858,'1980-W1011','1980-10-29 19:10:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',68 from dual;
insert into LAUNCH
select 50859,'1980-W1012','1980-10-29 19:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 5086,'1958-S112','1958-08-20 00:00:00','military missile','success',976,'B1-7','K63 SI-1','','','Test',636,'',402 from dual union all
select 50860,'1980-W1013','1980-10-30 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 50861,'1980-W1014','1980-10-31 12:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 50862,'1980-W1015','1980-10-31 16:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 50863,'1980-W1016','1980-11-03 09:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 50864,'1980-W1017','1980-11-03 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 50865,'1980-W1018','1980-11-04 02:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 50866,'1980-W1019','1980-11-04 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 50867,'1980-W102','1980-01-30 22:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 50868,'1980-W1020','1980-11-05 17:19:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',60 from dual union all
select 50869,'1980-W1021','1980-11-05 20:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 5087,'1958-S113','1958-08-23 00:00:00','military missile','success',976,'B1-12','K63 SI-2','','','Test',636,'',402 from dual union all
select 50870,'1980-W1022','1980-11-05 21:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 50871,'1980-W1023','1980-11-06 12:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 50872,'1980-W1024','1980-11-07 14:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 50873,'1980-W1025','1980-11-07 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 50874,'1980-W1026','1980-11-07 19:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 50875,'1980-W1027','1980-11-10 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 50876,'1980-W1028','1980-11-10 15:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 50877,'1980-W1029','1980-11-10 17:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 50878,'1980-W103','1980-01-31 11:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 50879,'1980-W1030','1980-11-10 19:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 5088,'1958-S114','1958-08-01 00:00:00','military missile','success',992,'53120416RD','NIIP-A target','','','Target',634,'',100 from dual union all
select 50880,'1980-W1031','1980-11-10 21:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 50881,'1980-W1032','1980-11-11 09:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 50882,'1980-W1033','1980-11-11 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',54 from dual union all
select 50883,'1980-W1034','1980-11-12 02:00:00','suborbital rocket','success',761,'MT135P-R402','','','','Meteo',1332,'',57 from dual union all
select 50884,'1980-W1035','1980-11-12 09:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 50885,'1980-W1036','1980-11-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',71 from dual union all
select 50886,'1980-W1037','1980-11-12 14:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 50887,'1980-W1038','1980-11-12 17:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 50888,'1980-W1039','1980-11-12 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 50889,'1980-W104','1980-01-31 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 5089,'1958-S115','1958-08-01 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 50890,'1980-W1040','1980-11-14 07:20:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',78 from dual union all
select 50891,'1980-W1041','1980-11-14 09:20:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',76 from dual union all
select 50892,'1980-W1042','1980-11-14 12:20:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',75 from dual union all
select 50893,'1980-W1043','1980-11-14 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 50894,'1980-W1044','1980-11-14 21:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 50895,'1980-W1045','1980-11-14 21:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 50896,'1980-W1046','1980-11-17 12:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 50897,'1980-W1047','1980-11-17 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',69 from dual union all
select 50898,'1980-W1048','1980-11-17 19:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 50899,'1980-W1049','1980-11-17 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 509,'1945-A28','1945-04-01 00:00:00','military missile','success',944,'11','','','','Test',435,'',1 from dual union all
select 5090,'1958-S116','1958-08-01 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 50900,'1980-W105','1980-01-31 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 50901,'1980-W1050','1980-11-18 10:20:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',76 from dual union all
select 50902,'1980-W1051','1980-11-18 11:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',76 from dual union all
select 50903,'1980-W1052','1980-11-18 17:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 50904,'1980-W1053','1980-11-18 18:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',71 from dual union all
select 50905,'1980-W1054','1980-11-18 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 50906,'1980-W1055','1980-11-18 19:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 50907,'1980-W1056','1980-11-19 15:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 50908,'1980-W1057','1980-11-19 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 50909,'1980-W1058','1980-11-19 22:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 5091,'1958-S117','1958-08-01 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 50910,'1980-W1059','1980-11-19 22:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 50911,'1980-W106','1980-01-31 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 50912,'1980-W1060','1980-11-20 21:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 50913,'1980-W1061','1980-11-21 09:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 50914,'1980-W1062','1980-11-21 14:13:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 50915,'1980-W1063','1980-11-21 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',64 from dual union all
select 50916,'1980-W1064','1980-11-21 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 50917,'1980-W1065','1980-11-24 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',55 from dual union all
select 50918,'1980-W1066','1980-11-24 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 50919,'1980-W1067','1980-11-24 21:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 5092,'1958-S118','1958-08-01 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 50920,'1980-W1068','1980-11-24 21:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 50921,'1980-W1069','1980-11-26 02:00:00','suborbital rocket','success',761,'MT135P-R404','','','','Meteo',1332,'',56 from dual union all
select 50922,'1980-W107','1980-02-01 00:04:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',72 from dual union all
select 50923,'1980-W1070','1980-11-26 15:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 50924,'1980-W1071','1980-11-26 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 50925,'1980-W1072','1980-11-26 21:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 50926,'1980-W1073','1980-11-26 22:41:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',72 from dual union all
select 50927,'1980-W1074','1980-11-27 00:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 50928,'1980-W1075','1980-11-28 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 50929,'1980-W1076','1980-11-28 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',54 from dual union all
select 5093,'1958-S119','1958-08-01 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 50930,'1980-W1077','1980-11-28 21:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 50931,'1980-W1078','1980-12-01 13:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 50932,'1980-W1079','1980-12-01 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 50933,'1980-W108','1980-02-01 00:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 50934,'1980-W1080','1980-12-01 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 50935,'1980-W1081','1980-12-01 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 50936,'1980-W1082','1980-12-01 18:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 50937,'1980-W1083','1980-12-01 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',69 from dual union all
select 50938,'1980-W1084','1980-12-02 17:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 50939,'1980-W1085','1980-12-03 02:02:00','suborbital rocket','success',761,'MT135P-R405','','','','Meteo',1332,'',59 from dual union all
select 5094,'1958-S12','1958-01-26 04:18:56','suborbital rocket','success',49,'NN3.03F','NRL NN3.03F','IGY NN3.03F','','Auroral',428,'',180 from dual union all
select 50940,'1980-W1086','1980-12-03 18:12:00','suborbital rocket','unknown',1284,'FLIGHT 286     /TH1-','','','','Aeron',1307,'',66 from dual union all
select 50941,'1980-W1087','1980-12-03 18:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 50942,'1980-W1088','1980-12-03 20:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 50943,'1980-W1089','1980-12-03 21:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 50944,'1980-W109','1980-02-01 00:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',77 from dual union all
select 50945,'1980-W1090','1980-12-04 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 50946,'1980-W1091','1980-12-04 19:10:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',52 from dual union all
select 50947,'1980-W1092','1980-12-04 19:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',77 from dual union all
select 50948,'1980-W1093','1980-12-05 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual;
insert into LAUNCH
select 50949,'1980-W1094','1980-12-05 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',64 from dual union all
select 5095,'1958-S120','1958-08-01 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 50950,'1980-W1095','1980-12-08 09:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 50951,'1980-W1096','1980-12-08 13:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',64 from dual union all
select 50952,'1980-W1097','1980-12-08 14:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 50953,'1980-W1098','1980-12-08 17:13:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 50954,'1980-W1099','1980-12-08 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 50955,'1980-W11','1980-01-04 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',53 from dual union all
select 50956,'1980-W110','1980-02-01 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 50957,'1980-W1100','1980-12-08 21:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 50958,'1980-W1101','1980-12-08 22:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 50959,'1980-W1102','1980-12-09 12:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 5096,'1958-S121','1958-08-01 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 50960,'1980-W1103','1980-12-09 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',68 from dual union all
select 50961,'1980-W1104','1980-12-10 02:00:00','suborbital rocket','success',761,'MT135P-R406','','','','Meteo',1332,'',52 from dual union all
select 50962,'1980-W1105','1980-12-10 15:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 50963,'1980-W1106','1980-12-10 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 50964,'1980-W1107','1980-12-11 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 50965,'1980-W1108','1980-12-11 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 50966,'1980-W1109','1980-12-12 02:30:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',70 from dual union all
select 50967,'1980-W111','1980-02-01 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 50968,'1980-W1110','1980-12-12 06:20:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',77 from dual union all
select 50969,'1980-W1111','1980-12-12 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 5097,'1958-S122','1958-08-01 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 50970,'1980-W1112','1980-12-12 20:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',62 from dual union all
select 50971,'1980-W1113','1980-12-12 21:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 50972,'1980-W1114','1980-12-15 12:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',56 from dual union all
select 50973,'1980-W1115','1980-12-15 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 50974,'1980-W1116','1980-12-16 05:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 50975,'1980-W1117','1980-12-16 09:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 50976,'1980-W1118','1980-12-16 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 50977,'1980-W1119','1980-12-16 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 50978,'1980-W112','1980-02-01 20:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 50979,'1980-W1120','1980-12-16 21:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 5098,'1958-S123','1958-08-01 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 50980,'1980-W1121','1980-12-17 02:03:00','suborbital rocket','success',761,'MT135P-R407','','','','Meteo',1332,'',58 from dual union all
select 50981,'1980-W1122','1980-12-17 04:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',69 from dual union all
select 50982,'1980-W1123','1980-12-17 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 50983,'1980-W1124','1980-12-17 10:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 50984,'1980-W1125','1980-12-17 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',59 from dual union all
select 50985,'1980-W1126','1980-12-17 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 50986,'1980-W1127','1980-12-17 21:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 50987,'1980-W1128','1980-12-17 22:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 50988,'1980-W1129','1980-12-18 09:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 50989,'1980-W113','1980-02-04 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',69 from dual union all
select 5099,'1958-S124','1958-08-01 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 50990,'1980-W1130','1980-12-18 23:30:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',60 from dual union all
select 50991,'1980-W1132','1980-12-19 17:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',64 from dual union all
select 50992,'1980-W1133','1980-12-22 09:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 50993,'1980-W1134','1980-12-22 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',56 from dual union all
select 50994,'1980-W1135','1980-12-22 18:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 50995,'1980-W1136','1980-12-22 21:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 50996,'1980-W1137','1980-12-23 16:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 50997,'1980-W1138','1980-12-24 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 50998,'1980-W1139','1980-12-24 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 50999,'1980-W114','1980-02-04 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 51,'1944-A15','1944-09-06 09:30:00','military missile','failure',1470,'18589','Paris','','','Weapon',1743,'',1 from dual union all
select 510,'1945-A29','1945-04-01 00:00:00','military missile','success',944,'12','','','','Test',435,'',1 from dual union all
select 5100,'1958-S125','1958-08-01 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51000,'1980-W1140','1980-12-24 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',64 from dual union all
select 51001,'1980-W1141','1980-12-24 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 51002,'1980-W1143','1980-12-26 12:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51003,'1980-W1144','1980-12-29 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51004,'1980-W1145','1980-12-30 21:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51005,'1980-W1146','1980-12-30 21:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 51006,'1980-W1147','1980-12-31 12:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 51007,'1980-W1148','1980-12-31 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 51008,'1980-W1149','1980-02-27 02:00:00','suborbital rocket','success',761,'MT135P-R377','','','','Meteo',1332,'',60 from dual union all
select 51009,'1980-W115','1980-02-04 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',61 from dual union all
select 5101,'1958-S126','1958-08-01 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51010,'1980-W1150','1980-03-13 02:20:00','suborbital rocket','success',761,'MT135P-R379','','','','Meteo',1332,'',60 from dual union all
select 51011,'1980-W1151','1980-07-09 02:28:00','suborbital rocket','success',761,'MT135P-R392','','','','Meteo',1332,'',60 from dual union all
select 51012,'1980-W1152','1980-11-19 02:00:00','suborbital rocket','success',761,'MT135P-R403','','','','Meteo',1332,'',60 from dual union all
select 51013,'1980-W116','1980-02-04 17:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 51014,'1980-W117','1980-02-04 21:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',50 from dual union all
select 51015,'1980-W118','1980-02-04 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 51016,'1980-W119','1980-02-06 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1027,'PASSAT',56 from dual union all
select 51017,'1980-W12','1980-01-04 15:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51018,'1980-W120','1980-02-06 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 51019,'1980-W121','1980-02-06 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 5102,'1958-S127','1958-08-01 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51020,'1980-W122','1980-02-06 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 51021,'1980-W123','1980-02-06 22:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 51022,'1980-W124','1980-02-08 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',66 from dual union all
select 51023,'1980-W125','1980-02-08 14:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51024,'1980-W126','1980-02-08 16:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',56 from dual union all
select 51025,'1980-W127','1980-02-08 21:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 51026,'1980-W128','1980-02-09 15:40:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',59 from dual union all
select 51027,'1980-W129','1980-02-11 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',67 from dual union all
select 51028,'1980-W13','1980-01-04 16:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 51029,'1980-W130','1980-02-11 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 5103,'1958-S128','1958-08-01 00:00:00','suborbital rocket','success',623,'J-CX','','','','Test',1821,'',750 from dual union all
select 51030,'1980-W131','1980-02-11 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 51031,'1980-W132','1980-02-11 17:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 51032,'1980-W133','1980-02-11 18:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 51033,'1980-W134','1980-02-11 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 51034,'1980-W135','1980-02-12 01:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 51035,'1980-W136','1980-02-12 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 51036,'1980-W137','1980-02-13 02:00:00','suborbital rocket','success',761,'MT135P-R375','','','','Meteo',1332,'',55 from dual union all
select 51037,'1980-W138','1980-02-13 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual;
insert into LAUNCH
select 51038,'1980-W139','1980-02-13 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51039,'1980-W14','1980-01-04 19:08:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 5104,'1958-S129','1958-08-01 09:50:00','suborbital rocket','success',440,'','','','','Aeron',803,'',83 from dual union all
select 51040,'1980-W140','1980-02-13 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 51041,'1980-W141','1980-02-13 17:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 51042,'1980-W142','1980-02-13 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51043,'1980-W143','1980-02-13 23:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51044,'1980-W144','1980-02-14 16:12:00','suborbital rocket','unknown',1284,'FLIGHT 261     /T 1-','','','','Aeron',1821,'',66 from dual union all
select 51045,'1980-W145','1980-02-14 16:42:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 51046,'1980-W146','1980-02-14 23:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',76 from dual union all
select 51047,'1980-W147','1980-02-14 23:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',54 from dual union all
select 51048,'1980-W148','1980-02-15 00:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 51049,'1980-W149','1980-02-15 08:30:00','suborbital rocket','unknown',738,'','','','','Aeron',1027,'PASSAT',60 from dual union all
select 5105,'1958-S13','1958-01-27 06:04:00','suborbital rocket','success',28,'SM1.09','Grenades','IGY SM1.09','','Aeron',428,'',98 from dual union all
select 51050,'1980-W15','1980-01-05 01:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 51051,'1980-W150','1980-02-15 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51052,'1980-W151','1980-02-15 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 51053,'1980-W152','1980-02-15 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 51054,'1980-W153','1980-02-15 14:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51055,'1980-W154','1980-02-15 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',71 from dual union all
select 51056,'1980-W155','1980-02-15 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 51057,'1980-W156','1980-02-16 01:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 51058,'1980-W157','1980-02-16 09:10:00','suborbital rocket','success',1281,'NASA 15.202UE','','','','Plasma',1412,'',78 from dual union all
select 51059,'1980-W158','1980-02-18 14:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 5106,'1958-S130','1958-08-01 09:50:00','suborbital rocket','success',440,'','','','','Aeron',803,'',83 from dual union all
select 51060,'1980-W159','1980-02-18 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 51061,'1980-W16','1980-01-05 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',76 from dual union all
select 51062,'1980-W160','1980-02-18 19:06:00','suborbital rocket','unknown',1284,'FLIGHT 262     /TN1-','','','','Aeron',330,'',75 from dual union all
select 51063,'1980-W161','1980-02-18 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 51064,'1980-W162','1980-02-19 15:10:00','suborbital rocket','unknown',1284,'FLIGHT 263     /TN1-','','','','Aeron',330,'',64 from dual union all
select 51065,'1980-W163','1980-02-19 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 51066,'1980-W164','1980-02-20 02:00:00','suborbital rocket','success',761,'MT135P-R376','','','','Meteo',1332,'',57 from dual union all
select 51067,'1980-W165','1980-02-20 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1027,'PASSAT',59 from dual union all
select 51068,'1980-W166','1980-02-20 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51069,'1980-W167','1980-02-20 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 5107,'1958-S131','1958-08-01 09:50:00','suborbital rocket','success',440,'','','','','Aeron',803,'',83 from dual union all
select 51070,'1980-W168','1980-02-20 15:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51071,'1980-W169','1980-02-20 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',67 from dual union all
select 51072,'1980-W17','1980-01-07 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 51073,'1980-W170','1980-02-20 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',55 from dual union all
select 51074,'1980-W171','1980-02-21 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51075,'1980-W172','1980-02-22 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51076,'1980-W173','1980-02-22 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51077,'1980-W174','1980-02-22 19:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 51078,'1980-W175','1980-02-23 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 51079,'1980-W176','1980-02-23 01:55:00','suborbital rocket','success',761,'MT135P T27','','','','Aeron-',1476,'',55 from dual union all
select 5108,'1958-S132','1958-08-01 09:50:00','suborbital rocket','success',440,'','','','','Aeron',803,'',83 from dual union all
select 51080,'1980-W177','1980-02-24 15:10:00','suborbital rocket','unknown',1284,'FLIGHT 264     /TN1-','','','','Aeron',330,'',54 from dual union all
select 51081,'1980-W178','1980-02-25 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',68 from dual union all
select 51082,'1980-W179','1980-02-25 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 51083,'1980-W18','1980-01-07 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 51084,'1980-W180','1980-02-25 14:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',59 from dual union all
select 51085,'1980-W181','1980-02-25 14:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51086,'1980-W182','1980-02-25 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 51087,'1980-W183','1980-02-25 16:30:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',59 from dual union all
select 51088,'1980-W184','1980-02-25 18:30:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',60 from dual union all
select 51089,'1980-W185','1980-02-26 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 5109,'1958-S133','1958-08-02 05:47:00','suborbital rocket','success',994,'No. 6','','','','Ionos/Bio',636,'',211 from dual union all
select 51090,'1980-W186','1980-02-26 14:30:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',60 from dual union all
select 51091,'1980-W187','1980-02-26 18:40:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',58 from dual union all
select 51092,'1980-W188','1980-02-27 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1027,'PASSAT',52 from dual union all
select 51093,'1980-W189','1980-02-27 13:20:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',59 from dual union all
select 51094,'1980-W19','1980-01-07 15:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 51095,'1980-W190','1980-02-27 14:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',55 from dual union all
select 51096,'1980-W191','1980-02-27 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 51097,'1980-W192','1980-02-27 17:30:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',59 from dual union all
select 51098,'1980-W193','1980-02-27 22:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51099,'1980-W194','1980-02-28 13:20:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',59 from dual union all
select 511,'1945-A30','1945-04-01 00:00:00','military missile','success',944,'13','','','','Test',435,'',1 from dual union all
select 5110,'1958-S134','1958-08-02 22:16:00','military missile','success',147,'4B','Mk 2-1 RV','R&D','','Test',228,'',900 from dual union all
select 51100,'1980-W195','1980-02-28 14:20:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',60 from dual union all
select 51101,'1980-W196','1980-02-28 15:30:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',59 from dual union all
select 51102,'1980-W197','1980-02-28 17:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',59 from dual union all
select 51103,'1980-W198','1980-02-28 18:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',60 from dual union all
select 51104,'1980-W199','1980-02-28 19:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 51105,'1980-W20','1980-01-07 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 51106,'1980-W200','1980-02-28 23:08:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',72 from dual union all
select 51107,'1980-W201','1980-02-28 23:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',78 from dual union all
select 51108,'1980-W202','1980-02-28 23:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',63 from dual union all
select 51109,'1980-W203','1980-02-29 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',78 from dual union all
select 5111,'1958-S135','1958-08-06 20:24:00','military missile','success',1366,'117','R&D Series III','','','Test',236,'',520 from dual union all
select 51110,'1980-W204','1980-02-29 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',1027,'PASSAT',57 from dual union all
select 51111,'1980-W205','1980-02-29 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1027,'PASSAT',57 from dual union all
select 51112,'1980-W206','1980-02-29 12:20:00','suborbital rocket','unknown',738,'','','','','Aeron',1761,'VOLNA',59 from dual union all
select 51113,'1980-W207','1980-02-29 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51114,'1980-W208','1980-02-29 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',67 from dual union all
select 51115,'1980-W209','1980-02-29 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 51116,'1980-W21','1980-01-07 17:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',64 from dual union all
select 51117,'1980-W210','1980-02-29 15:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',78 from dual union all
select 51118,'1980-W211','1980-02-29 19:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 51119,'1980-W212','1980-02-29 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 5112,'1958-S136','1958-08-07 00:00:00','military missile','success',992,'S13-11D','NIIP-A target','','','Target',634,'',100 from dual union all
select 51120,'1980-W213','1980-03-01 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 51121,'1980-W214','1980-03-01 13:10:00','suborbital rocket','unknown',1284,'FLIGHT 265     /TN1-','','','','Aeron',330,'',65 from dual union all
select 51122,'1980-W215','1980-03-03 04:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1027,'PASSAT',57 from dual union all
select 51123,'1980-W216','1980-03-03 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 51124,'1980-W217','1980-03-03 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 51125,'1980-W218','1980-03-03 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 51126,'1980-W219','1980-03-03 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',63 from dual union all
select 51127,'1980-W22','1980-01-07 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual;
insert into LAUNCH
select 51128,'1980-W220','1980-03-03 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 51129,'1980-W221','1980-03-04 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 5113,'1958-S137','1958-08-11 09:30:00','suborbital rocket','success',440,'','','','','Aeron',803,'',83 from dual union all
select 51130,'1980-W222','1980-03-04 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 51131,'1980-W223','1980-03-05 02:02:00','suborbital rocket','success',761,'MT135P-R378','','','','Meteo',1332,'',53 from dual union all
select 51132,'1980-W224','1980-03-05 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1638,'USHAKOV',57 from dual union all
select 51133,'1980-W225','1980-03-05 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 51134,'1980-W226','1980-03-05 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 51135,'1980-W227','1980-03-05 21:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',73 from dual union all
select 51136,'1980-W228','1980-03-05 22:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 51137,'1980-W229','1980-03-06 19:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 51138,'1980-W23','1980-01-07 19:37:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',62 from dual union all
select 51139,'1980-W230','1980-03-06 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 5114,'1958-S138','1958-08-11 09:30:00','suborbital rocket','success',440,'','','','','Aeron',803,'',83 from dual union all
select 51140,'1980-W231','1980-03-07 00:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51141,'1980-W232','1980-03-07 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 51142,'1980-W233','1980-03-07 15:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 51143,'1980-W234','1980-03-07 16:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 51144,'1980-W235','1980-03-07 20:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 51145,'1980-W236','1980-03-07 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51146,'1980-W237','1980-03-10 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51147,'1980-W238','1980-03-10 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51148,'1980-W239','1980-03-10 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',55 from dual union all
select 51149,'1980-W24','1980-01-07 19:38:00','suborbital rocket','unknown',1284,'FLIGHT 260     /T 1-','','','','Aeron',1821,'',67 from dual union all
select 5115,'1958-S139','1958-08-11 09:30:00','suborbital rocket','success',440,'','','','','Aeron',803,'',83 from dual union all
select 51150,'1980-W240','1980-03-10 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 51151,'1980-W241','1980-03-10 18:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 51152,'1980-W242','1980-03-10 19:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 51153,'1980-W243','1980-03-10 21:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 51154,'1980-W244','1980-03-11 03:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 51155,'1980-W245','1980-03-11 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 51156,'1980-W246','1980-03-12 00:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51157,'1980-W247','1980-03-12 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1638,'USHAKOV',56 from dual union all
select 51158,'1980-W248','1980-03-12 13:11:00','suborbital rocket','unknown',1284,'FLIGHT 266     /TN1-','','','','Aeron',330,'',65 from dual union all
select 51159,'1980-W249','1980-03-12 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 5116,'1958-S14','1958-01-27 19:17:45','suborbital rocket','success',839,'AM6.36','Pressure/Temp','IGY AM6.36','','Aeron',428,'',129 from dual union all
select 51160,'1980-W25','1980-01-07 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 51161,'1980-W250','1980-03-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',66 from dual union all
select 51162,'1980-W251','1980-03-12 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 51163,'1980-W252','1980-03-12 14:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51164,'1980-W253','1980-03-12 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',52 from dual union all
select 51165,'1980-W254','1980-03-12 19:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 51166,'1980-W255','1980-03-13 00:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',78 from dual union all
select 51167,'1980-W256','1980-03-13 00:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 51168,'1980-W257','1980-03-13 00:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',78 from dual union all
select 51169,'1980-W258','1980-03-13 02:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 5117,'1958-S140','1958-08-11 09:30:00','suborbital rocket','success',440,'','','','','Aeron',803,'',83 from dual union all
select 51170,'1980-W259','1980-03-13 16:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 51171,'1980-W26','1980-01-07 21:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 51172,'1980-W260','1980-03-14 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1638,'USHAKOV',57 from dual union all
select 51173,'1980-W261','1980-03-14 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 51174,'1980-W262','1980-03-14 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 51175,'1980-W263','1980-03-15 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51176,'1980-W264','1980-03-15 06:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51177,'1980-W265','1980-03-17 14:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51178,'1980-W266','1980-03-17 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 51179,'1980-W267','1980-03-17 16:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 5118,'1958-S141','1958-08-11 09:30:00','suborbital rocket','success',440,'','','','','Aeron',803,'',83 from dual union all
select 51180,'1980-W268','1980-03-17 16:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 51181,'1980-W269','1980-03-17 17:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 51182,'1980-W27','1980-01-08 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 51183,'1980-W270','1980-03-17 18:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 51184,'1980-W271','1980-03-17 21:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 51185,'1980-W272','1980-03-17 23:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',59 from dual union all
select 51186,'1980-W273','1980-03-18 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 51187,'1980-W274','1980-03-18 04:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 51188,'1980-W275','1980-03-18 20:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',76 from dual union all
select 51189,'1980-W276','1980-03-18 21:05:00','suborbital rocket','unknown',1284,'FLIGHT 267     /T 1-','','','','Aeron',1821,'',65 from dual union all
select 5119,'1958-S142','1958-08-11 09:30:00','suborbital rocket','success',440,'','','','','Aeron',803,'',83 from dual union all
select 51190,'1980-W277','1980-03-19 02:00:00','suborbital rocket','success',761,'MT135P-R380','','','','Meteo',1332,'',55 from dual union all
select 51191,'1980-W278','1980-03-19 04:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51192,'1980-W279','1980-03-19 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1638,'USHAKOV',57 from dual union all
select 51193,'1980-W28','1980-01-09 02:00:00','suborbital rocket','success',761,'MT135P-R372','','','','Meteo',1332,'',54 from dual union all
select 51194,'1980-W280','1980-03-19 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 51195,'1980-W281','1980-03-19 15:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 51196,'1980-W282','1980-03-19 17:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 51197,'1980-W283','1980-03-19 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 51198,'1980-W284','1980-03-19 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',67 from dual union all
select 51199,'1980-W285','1980-03-19 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 512,'1945-A31','1945-04-01 00:00:00','military missile','success',943,'14','Private A-1','','','Test',435,'',8 from dual union all
select 5120,'1958-S143','1958-08-11 09:30:00','suborbital rocket','success',440,'','','','','Aeron',803,'',83 from dual union all
select 51200,'1980-W286','1980-03-19 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51201,'1980-W287','1980-03-20 03:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51202,'1980-W288','1980-03-21 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1638,'USHAKOV',57 from dual union all
select 51203,'1980-W289','1980-03-21 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51204,'1980-W29','1980-01-09 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 51205,'1980-W290','1980-03-21 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51206,'1980-W291','1980-03-21 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 51207,'1980-W292','1980-03-21 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51208,'1980-W293','1980-03-21 21:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 51209,'1980-W294','1980-03-22 01:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 5121,'1958-S144','1958-08-12 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51210,'1980-W295','1980-03-22 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 51211,'1980-W296','1980-03-22 07:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 51212,'1980-W297','1980-03-24 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',69 from dual union all
select 51213,'1980-W298','1980-03-24 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 51214,'1980-W299','1980-03-24 14:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',58 from dual union all
select 51215,'1980-W30','1980-01-09 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 51216,'1980-W300','1980-03-24 15:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51217,'1980-W301','1980-03-24 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual;
insert into LAUNCH
select 51218,'1980-W302','1980-03-24 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 51219,'1980-W303','1980-03-24 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 5122,'1958-S145','1958-08-12 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51220,'1980-W304','1980-03-25 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51221,'1980-W305','1980-03-26 02:00:00','suborbital rocket','success',761,'MT135P-R381','','','','Meteo',1332,'',57 from dual union all
select 51222,'1980-W306','1980-03-26 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1638,'USHAKOV',58 from dual union all
select 51223,'1980-W307','1980-03-26 14:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',63 from dual union all
select 51224,'1980-W308','1980-03-26 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51225,'1980-W309','1980-03-26 15:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',79 from dual union all
select 51226,'1980-W31','1980-01-09 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 51227,'1980-W310','1980-03-26 18:53:00','suborbital rocket','success',1061,'','STS','','','Meteo',276,'',65 from dual union all
select 51228,'1980-W311','1980-03-26 19:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 51229,'1980-W312','1980-03-26 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 5123,'1958-S146','1958-08-12 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51230,'1980-W313','1980-03-27 04:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 51231,'1980-W314','1980-03-28 04:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 51232,'1980-W315','1980-03-28 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1638,'USHAKOV',58 from dual union all
select 51233,'1980-W316','1980-03-28 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51234,'1980-W317','1980-03-28 14:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',64 from dual union all
select 51235,'1980-W318','1980-03-28 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 51236,'1980-W319','1980-03-28 16:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 51237,'1980-W32','1980-01-09 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 51238,'1980-W320','1980-03-28 19:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',66 from dual union all
select 51239,'1980-W321','1980-03-29 00:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 5124,'1958-S147','1958-08-12 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51240,'1980-W322','1980-03-31 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51241,'1980-W323','1980-03-31 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 51242,'1980-W324','1980-03-31 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 51243,'1980-W325','1980-03-31 16:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 51244,'1980-W326','1980-03-31 23:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',79 from dual union all
select 51245,'1980-W327','1980-04-01 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',79 from dual union all
select 51246,'1980-W328','1980-04-01 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 51247,'1980-W329','1980-04-02 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 51248,'1980-W33','1980-01-09 22:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51249,'1980-W330','1980-04-02 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 5125,'1958-S148','1958-08-12 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51250,'1980-W331','1980-04-02 15:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51251,'1980-W332','1980-04-02 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 51252,'1980-W333','1980-04-03 00:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 51253,'1980-W334','1980-04-03 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 51254,'1980-W335','1980-04-03 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 51255,'1980-W336','1980-04-03 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51256,'1980-W337','1980-04-04 15:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 51257,'1980-W338','1980-04-04 15:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 51258,'1980-W339','1980-04-04 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51259,'1980-W34','1980-01-10 01:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 5126,'1958-S149','1958-08-12 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51260,'1980-W340','1980-04-05 03:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',67 from dual union all
select 51261,'1980-W341','1980-04-07 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51262,'1980-W342','1980-04-07 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 51263,'1980-W343','1980-04-07 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51264,'1980-W344','1980-04-08 05:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',66 from dual union all
select 51265,'1980-W345','1980-04-09 01:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 51266,'1980-W346','1980-04-09 02:04:59','suborbital rocket','success',761,'MT135P-R382','','','','Meteo',1332,'',57 from dual union all
select 51267,'1980-W347','1980-04-09 13:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 51268,'1980-W348','1980-04-09 14:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51269,'1980-W349','1980-04-09 14:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 5127,'1958-S15','1958-01-27 18:49:00','suborbital rocket','success',28,'SM2.10','Grenades/Sphere','IGY SM2.10','','Aeron',428,'',143 from dual union all
select 51270,'1980-W35','1980-01-11 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51271,'1980-W350','1980-04-09 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 51272,'1980-W351','1980-04-09 15:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 51273,'1980-W352','1980-04-09 16:09:00','suborbital rocket','unknown',1284,'FLIGHT 268     /T 1-','','','','Aeron',1821,'',63 from dual union all
select 51274,'1980-W353','1980-04-09 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51275,'1980-W354','1980-04-09 22:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 51276,'1980-W355','1980-04-09 23:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 51277,'1980-W356','1980-04-10 00:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 51278,'1980-W357','1980-04-10 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 51279,'1980-W358','1980-04-11 01:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 5128,'1958-S150','1958-08-12 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51280,'1980-W359','1980-04-11 02:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',59 from dual union all
select 51281,'1980-W36','1980-01-11 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 51282,'1980-W360','1980-04-11 05:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 51283,'1980-W361','1980-04-11 06:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 51284,'1980-W362','1980-04-11 06:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 51285,'1980-W363','1980-04-11 08:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 51286,'1980-W364','1980-04-11 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51287,'1980-W365','1980-04-11 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51288,'1980-W366','1980-04-11 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 51289,'1980-W367','1980-04-11 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 5129,'1958-S151','1958-08-12 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51290,'1980-W368','1980-04-12 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 51291,'1980-W369','1980-04-14 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 51292,'1980-W37','1980-01-11 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51293,'1980-W370','1980-04-14 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 51294,'1980-W371','1980-04-14 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51295,'1980-W372','1980-04-14 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51296,'1980-W373','1980-04-15 03:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51297,'1980-W374','1980-04-15 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 51298,'1980-W375','1980-04-16 02:10:00','suborbital rocket','success',761,'MT135P-R383','','','','Meteo',1332,'',58 from dual union all
select 51299,'1980-W376','1980-04-16 02:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 513,'1945-A32','1945-04-01 00:00:00','military missile','success',944,'15','','','','Test',435,'',1 from dual union all
select 5130,'1958-S152','1958-08-12 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51300,'1980-W377','1980-04-16 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 51301,'1980-W378','1980-04-16 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 51302,'1980-W379','1980-04-16 14:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51303,'1980-W38','1980-01-11 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 51304,'1980-W380','1980-04-16 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 51305,'1980-W381','1980-04-16 18:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 51306,'1980-W382','1980-04-16 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 51307,'1980-W383','1980-04-17 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual;
insert into LAUNCH
select 51308,'1980-W384','1980-04-17 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 51309,'1980-W385','1980-04-18 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 5131,'1958-S153','1958-08-12 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51310,'1980-W386','1980-04-18 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 51311,'1980-W387','1980-04-18 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 51312,'1980-W388','1980-04-18 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 51313,'1980-W389','1980-04-19 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 51314,'1980-W39','1980-01-11 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 51315,'1980-W390','1980-04-19 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1299,'PRILIV',56 from dual union all
select 51316,'1980-W391','1980-04-21 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 51317,'1980-W392','1980-04-21 15:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51318,'1980-W393','1980-04-21 15:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51319,'1980-W394','1980-04-21 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 5132,'1958-S154','1958-08-12 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51320,'1980-W395','1980-04-21 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 51321,'1980-W396','1980-04-22 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51322,'1980-W397','1980-04-22 23:20:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',77 from dual union all
select 51323,'1980-W398','1980-04-23 00:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 51324,'1980-W399','1980-04-23 00:20:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',73 from dual union all
select 51325,'1980-W40','1980-01-11 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 51326,'1980-W400','1980-04-23 00:27:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',75 from dual union all
select 51327,'1980-W401','1980-04-23 04:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 51328,'1980-W402','1980-04-23 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1299,'PRILIV',58 from dual union all
select 51329,'1980-W403','1980-04-23 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 5133,'1958-S155','1958-08-12 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51330,'1980-W404','1980-04-23 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 51331,'1980-W405','1980-04-23 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 51332,'1980-W406','1980-04-23 16:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 51333,'1980-W407','1980-04-23 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 51334,'1980-W408','1980-04-23 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51335,'1980-W409','1980-04-24 03:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 51336,'1980-W41','1980-01-11 17:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 51337,'1980-W410','1980-04-24 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 51338,'1980-W411','1980-04-24 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 51339,'1980-W412','1980-04-25 01:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 5134,'1958-S156','1958-08-12 00:00:00','suborbital rocket','success',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51340,'1980-W413','1980-04-25 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1299,'PRILIV',61 from dual union all
select 51341,'1980-W414','1980-04-25 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51342,'1980-W415','1980-04-25 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 51343,'1980-W416','1980-04-25 19:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 51344,'1980-W417','1980-04-25 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 51345,'1980-W418','1980-04-28 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51346,'1980-W419','1980-04-28 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 51347,'1980-W42','1980-01-12 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 51348,'1980-W420','1980-04-28 15:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 51349,'1980-W421','1980-04-28 20:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 5135,'1958-S157','1958-08-12 00:00:00','suborbital rocket','failure',21,'','Hardtack','','','Aeronomy',803,'',100 from dual union all
select 51350,'1980-W422','1980-04-29 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51351,'1980-W423','1980-04-29 20:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 51352,'1980-W424','1980-04-30 02:15:00','suborbital rocket','success',761,'MT135P-R384','','','','Meteo',1332,'',56 from dual union all
select 51353,'1980-W425','1980-04-30 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 51354,'1980-W426','1980-04-30 15:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51355,'1980-W427','1980-04-30 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 51356,'1980-W428','1980-05-01 00:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 51357,'1980-W429','1980-05-01 03:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51358,'1980-W43','1980-01-14 10:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',73 from dual union all
select 51359,'1980-W430','1980-05-01 20:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 5136,'1958-S158','1958-08-13 00:00:00','suborbital rocket','success',28,'','','','','Ionos',640,'',100 from dual union all
select 51360,'1980-W431','1980-05-02 00:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',76 from dual union all
select 51361,'1980-W432','1980-05-02 00:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 51362,'1980-W433','1980-05-02 00:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',78 from dual union all
select 51363,'1980-W434','1980-05-02 12:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 51364,'1980-W435','1980-05-02 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 51365,'1980-W436','1980-05-02 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51366,'1980-W437','1980-05-02 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 51367,'1980-W438','1980-05-02 21:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 51368,'1980-W439','1980-05-03 03:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 51369,'1980-W44','1980-01-14 12:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',72 from dual union all
select 5137,'1958-S159','1958-08-13 02:00:00','suborbital rocket','success',994,'No. 7','','','','Ionos/Bio',636,'',208 from dual union all
select 51370,'1980-W440','1980-05-05 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',58 from dual union all
select 51371,'1980-W441','1980-05-05 14:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 51372,'1980-W442','1980-05-05 14:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 51373,'1980-W443','1980-05-05 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 51374,'1980-W444','1980-05-05 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51375,'1980-W445','1980-05-05 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 51376,'1980-W446','1980-05-05 20:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 51377,'1980-W447','1980-05-07 12:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 51378,'1980-W448','1980-05-07 13:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51379,'1980-W449','1980-05-07 15:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 5138,'1958-S16','1958-01-29 19:06:17','suborbital rocket','success',839,'AM6.03','Sphere','IGY AM6.03','','Aeron',428,'',170 from dual union all
select 51380,'1980-W45','1980-01-14 13:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',76 from dual union all
select 51381,'1980-W450','1980-05-07 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 51382,'1980-W451','1980-05-07 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 51383,'1980-W452','1980-05-07 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51384,'1980-W453','1980-05-07 21:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 51385,'1980-W454','1980-05-08 02:01:00','suborbital rocket','success',761,'MT135P-R385','','','','Meteo',1332,'',57 from dual union all
select 51386,'1980-W455','1980-05-08 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 51387,'1980-W456','1980-05-09 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',75 from dual union all
select 51388,'1980-W457','1980-05-09 14:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 51389,'1980-W458','1980-05-09 15:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 5139,'1958-S160','1958-08-14 00:40:00','suborbital rocket','success',5,'','','','','Ionos/Aeron',1010,'',100 from dual union all
select 51390,'1980-W459','1980-05-09 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 51391,'1980-W46','1980-01-14 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',69 from dual union all
select 51392,'1980-W460','1980-05-09 20:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 51393,'1980-W461','1980-05-09 21:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 51394,'1980-W462','1980-05-12 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51395,'1980-W463','1980-05-12 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 51396,'1980-W464','1980-05-12 14:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',57 from dual union all
select 51397,'1980-W465','1980-05-12 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 51398,'1980-W466','1980-05-12 18:00:00','suborbital rocket','unknown',1284,'FLIGHT 269     /TH1-','','','','Aeron',1307,'',75 from dual;
insert into LAUNCH
select 51399,'1980-W467','1980-05-12 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 514,'1945-A33','1945-04-01 00:00:00','military missile','success',944,'16','','','','Test',435,'',1 from dual union all
select 5140,'1958-S161','1958-08-14 13:28:00','suborbital rocket','success',839,'OB6.01F','','IGY OB6.01F','','Test',1895,'',132 from dual union all
select 51400,'1980-W468','1980-05-12 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 51401,'1980-W469','1980-05-12 20:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 51402,'1980-W47','1980-01-14 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 51403,'1980-W470','1980-05-13 03:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 51404,'1980-W471','1980-05-13 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51405,'1980-W472','1980-05-14 02:00:00','suborbital rocket','success',761,'MT135P-R386','','','','Meteo',1332,'',58 from dual union all
select 51406,'1980-W473','1980-05-14 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 51407,'1980-W474','1980-05-14 14:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51408,'1980-W475','1980-05-14 15:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 51409,'1980-W476','1980-05-14 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 5141,'1958-S162','1958-08-15 04:16:00','suborbital rocket','success',623,'Jason 1822','','','','Nuclear Obs',225,'',693 from dual union all
select 51410,'1980-W477','1980-05-14 19:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 51411,'1980-W478','1980-05-14 22:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51412,'1980-W479','1980-05-15 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 51413,'1980-W48','1980-01-14 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 51414,'1980-W480','1980-05-16 00:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',78 from dual union all
select 51415,'1980-W481','1980-05-16 00:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',77 from dual union all
select 51416,'1980-W482','1980-05-16 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51417,'1980-W483','1980-05-16 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 51418,'1980-W484','1980-05-16 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51419,'1980-W485','1980-05-16 16:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',53 from dual union all
select 5142,'1958-S163','1958-08-15 15:45:00','suborbital rocket','success',28,'','','','','Test/Ionos',640,'',200 from dual union all
select 51420,'1980-W486','1980-05-16 20:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 51421,'1980-W487','1980-05-17 11:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1299,'PRILIV',58 from dual union all
select 51422,'1980-W488','1980-05-19 14:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',73 from dual union all
select 51423,'1980-W489','1980-05-19 15:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51424,'1980-W49','1980-01-15 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 51425,'1980-W490','1980-05-19 16:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 51426,'1980-W491','1980-05-19 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 51427,'1980-W492','1980-05-19 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 51428,'1980-W493','1980-05-19 20:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 51429,'1980-W494','1980-05-20 02:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 5143,'1958-S164','1958-08-17 00:00:00','military missile','success',992,'53120418RD','NIIP-A target','','','Target',634,'',100 from dual union all
select 51430,'1980-W495','1980-05-20 02:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 51431,'1980-W496','1980-05-20 14:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 51432,'1980-W497','1980-05-21 02:00:00','suborbital rocket','success',761,'MT135P-R387','','','','Meteo',1332,'',57 from dual union all
select 51433,'1980-W498','1980-05-21 03:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 51434,'1980-W499','1980-05-21 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 51435,'1980-W50','1980-01-15 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 51436,'1980-W500','1980-05-21 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',60 from dual union all
select 51437,'1980-W501','1980-05-21 14:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51438,'1980-W502','1980-05-21 18:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 51439,'1980-W503','1980-05-21 18:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',76 from dual union all
select 5144,'1958-S165','1958-08-20 11:27:00','suborbital rocket','success',623,'Jason 1841','','','','Nuclear Obs',1312,'',800 from dual union all
select 51440,'1980-W504','1980-05-21 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 51441,'1980-W505','1980-05-22 12:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 51442,'1980-W506','1980-05-22 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 51443,'1980-W507','1980-05-23 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51444,'1980-W508','1980-05-23 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 51445,'1980-W509','1980-05-23 13:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',75 from dual union all
select 51446,'1980-W51','1980-01-15 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 51447,'1980-W510','1980-05-23 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',64 from dual union all
select 51448,'1980-W511','1980-05-23 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',62 from dual union all
select 51449,'1980-W512','1980-05-23 21:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',79 from dual union all
select 5145,'1958-S166','1958-08-25 18:17:00','suborbital rocket','success',623,'Jason 1859','','','','Nuclear Obs',1821,'',800 from dual union all
select 51450,'1980-W513','1980-05-26 18:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 51451,'1980-W514','1980-05-26 21:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 51452,'1980-W515','1980-05-26 22:00:00','suborbital rocket','unknown',1284,'FLIGHT 270     /T 1-','','','','Aeron',1821,'',65 from dual union all
select 51453,'1980-W516','1980-05-27 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 51454,'1980-W517','1980-05-27 19:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',79 from dual union all
select 51455,'1980-W518','1980-05-27 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 51456,'1980-W519','1980-05-28 02:10:00','suborbital rocket','success',761,'MT135P-R388','','','','Meteo',1332,'',58 from dual union all
select 51457,'1980-W52','1980-01-16 02:00:00','suborbital rocket','success',761,'MT135P-R373','','','','Meteo',1332,'',54 from dual union all
select 51458,'1980-W520','1980-05-28 02:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',75 from dual union all
select 51459,'1980-W521','1980-05-28 02:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 5146,'1958-S167','1958-08-27 00:00:00','military missile','success',992,'S13-15D','NIIP-A target','','','Target',634,'',100 from dual union all
select 51460,'1980-W522','1980-05-28 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 51461,'1980-W523','1980-05-28 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 51462,'1980-W524','1980-05-28 14:23:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 51463,'1980-W525','1980-05-28 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 51464,'1980-W526','1980-05-28 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',54 from dual union all
select 51465,'1980-W527','1980-05-28 15:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 51466,'1980-W528','1980-05-28 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 51467,'1980-W529','1980-05-29 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1299,'PRILIV',56 from dual union all
select 51468,'1980-W53','1980-01-16 12:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',68 from dual union all
select 51469,'1980-W530','1980-05-29 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',53 from dual union all
select 5147,'1958-S168','1958-08-27 02:20:00','test rocket','success',1521,'','Argus I','','','Nuclear Test',162,'AVM1',160 from dual union all
select 51470,'1980-W531','1980-05-30 02:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 51471,'1980-W532','1980-05-30 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51472,'1980-W533','1980-05-30 14:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 51473,'1980-W534','1980-05-30 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 51474,'1980-W535','1980-05-31 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1299,'PRILIV',62 from dual union all
select 51475,'1980-W536','1980-05-31 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1299,'PRILIV',57 from dual union all
select 51476,'1980-W537','1980-06-02 10:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51477,'1980-W538','1980-06-02 11:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1299,'PRILIV',58 from dual union all
select 51478,'1980-W539','1980-06-02 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 51479,'1980-W54','1980-01-16 13:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',73 from dual union all
select 5148,'1958-S169','1958-08-27 03:33:00','suborbital rocket','success',623,'Jason 1909','','','','Nuclear Obs',225,'',937 from dual union all
select 51480,'1980-W540','1980-06-02 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 51481,'1980-W541','1980-06-02 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51482,'1980-W542','1980-06-02 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 51483,'1980-W543','1980-06-03 00:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 51484,'1980-W544','1980-06-03 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51485,'1980-W545','1980-06-04 02:00:00','suborbital rocket','success',761,'MT135P-R389','','','','Meteo',1332,'',56 from dual union all
select 51486,'1980-W546','1980-06-04 12:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51487,'1980-W547','1980-06-04 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 51488,'1980-W548','1980-06-04 15:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',74 from dual;
insert into LAUNCH
select 51489,'1980-W549','1980-06-04 16:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',64 from dual union all
select 5149,'1958-S17','1958-01-29 21:15:00','military missile','failure',1016,'M1-11','8K71 LKI II-1','','','Test',480,'',1350 from dual union all
select 51490,'1980-W55','1980-01-16 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',74 from dual union all
select 51491,'1980-W550','1980-06-04 18:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 51492,'1980-W551','1980-06-05 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 51493,'1980-W552','1980-06-05 18:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 51494,'1980-W553','1980-06-05 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51495,'1980-W554','1980-06-05 21:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 51496,'1980-W555','1980-06-05 21:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 51497,'1980-W556','1980-06-06 10:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51498,'1980-W557','1980-06-06 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',52 from dual union all
select 51499,'1980-W558','1980-06-06 13:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 515,'1945-A34','1945-04-13 00:00:00','military missile','success',944,'17','','','','Test',435,'',1 from dual union all
select 5150,'1958-S170','1958-08-27 04:06:00','suborbital rocket','success',994,'','','','','Ionos/Bio',636,'',209 from dual union all
select 51500,'1980-W559','1980-06-06 15:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51501,'1980-W56','1980-01-16 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 51502,'1980-W560','1980-06-06 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51503,'1980-W561','1980-06-06 21:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51504,'1980-W562','1980-06-06 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 51505,'1980-W563','1980-06-07 03:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 51506,'1980-W564','1980-06-09 12:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 51507,'1980-W565','1980-06-09 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 51508,'1980-W566','1980-06-09 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 51509,'1980-W567','1980-06-09 15:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',59 from dual union all
select 5151,'1958-S171','1958-08-27 04:24:00','suborbital rocket','success',623,'Jason 1914','','','','Nuclear Obs',1312,'',800 from dual union all
select 51510,'1980-W568','1980-06-09 16:17:00','suborbital rocket','unknown',1284,'FLIGHT 271     /T 1-','','','','Aeron',1821,'',67 from dual union all
select 51511,'1980-W569','1980-06-09 16:17:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',62 from dual union all
select 51512,'1980-W57','1980-01-16 14:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51513,'1980-W570','1980-06-09 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 51514,'1980-W571','1980-06-09 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',65 from dual union all
select 51515,'1980-W572','1980-06-09 18:04:00','suborbital rocket','unknown',1284,'FLIGHT 272     /TH1-','','','','Aeron',1307,'',69 from dual union all
select 51516,'1980-W573','1980-06-09 18:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 51517,'1980-W574','1980-06-11 02:01:00','suborbital rocket','success',761,'MT135P-R390','','','','Meteo',1332,'',57 from dual union all
select 51518,'1980-W575','1980-06-11 10:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51519,'1980-W576','1980-06-11 13:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 5152,'1958-S172','1958-08-27 06:05:00','suborbital rocket','success',1011,'A1-8','R-5A Geophysical 2','IR atmos','','Aeron/Ionos/Fields',636,'',450 from dual union all
select 51520,'1980-W577','1980-06-11 13:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 51521,'1980-W578','1980-06-11 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',68 from dual union all
select 51522,'1980-W579','1980-06-11 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 51523,'1980-W58','1980-01-16 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',79 from dual union all
select 51524,'1980-W580','1980-06-11 15:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',70 from dual union all
select 51525,'1980-W581','1980-06-11 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51526,'1980-W582','1980-06-11 21:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 51527,'1980-W583','1980-06-11 22:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51528,'1980-W584','1980-06-12 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 51529,'1980-W585','1980-06-13 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 5153,'1958-S173','1958-08-27 06:42:00','suborbital rocket','success',623,'Jason 1917','','','','Nuclear Obs',1312,'',817 from dual union all
select 51530,'1980-W586','1980-06-13 15:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51531,'1980-W587','1980-06-13 15:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51532,'1980-W588','1980-06-13 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51533,'1980-W589','1980-06-13 19:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 51534,'1980-W59','1980-01-16 19:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 51535,'1980-W590','1980-06-13 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',52 from dual union all
select 51536,'1980-W591','1980-06-14 00:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 51537,'1980-W592','1980-06-14 05:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 51538,'1980-W593','1980-06-16 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51539,'1980-W594','1980-06-16 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 5154,'1958-S174','1958-08-27 07:29:00','suborbital rocket','success',623,'Jason 1913 J-15','','','','Nuclear Obs',1821,'',800 from dual union all
select 51540,'1980-W595','1980-06-16 14:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 51541,'1980-W596','1980-06-16 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',55 from dual union all
select 51542,'1980-W597','1980-06-16 17:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 51543,'1980-W598','1980-06-16 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 51544,'1980-W599','1980-06-17 02:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 51545,'1980-W60','1980-01-16 19:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 51546,'1980-W600','1980-06-17 16:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 51547,'1980-W601','1980-06-18 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 51548,'1980-W602','1980-06-18 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 51549,'1980-W603','1980-06-18 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',74 from dual union all
select 5155,'1958-S176','1958-08-27 23:15:00','military missile','success',638,'AM-7','R&D','','','Test',248,'',500 from dual union all
select 51550,'1980-W604','1980-06-18 14:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',73 from dual union all
select 51551,'1980-W605','1980-06-18 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51552,'1980-W606','1980-06-18 18:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual union all
select 51553,'1980-W607','1980-06-18 19:56:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',66 from dual union all
select 51554,'1980-W608','1980-06-18 20:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 51555,'1980-W609','1980-06-20 00:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 51556,'1980-W61','1980-01-16 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 51557,'1980-W610','1980-06-20 10:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51558,'1980-W611','1980-06-20 13:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',66 from dual union all
select 51559,'1980-W612','1980-06-20 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 5156,'1958-S177','1958-08-29 04:30:00','military missile','success',147,'5B','Mk 2-1 RV/AFSWC-1','R&D','','Test',226,'',900 from dual union all
select 51560,'1980-W613','1980-06-20 14:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 51561,'1980-W614','1980-06-20 14:23:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51562,'1980-W615','1980-06-20 18:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',62 from dual union all
select 51563,'1980-W616','1980-06-23 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 51564,'1980-W617','1980-06-23 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 51565,'1980-W618','1980-06-23 15:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51566,'1980-W619','1980-06-23 20:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 51567,'1980-W62','1980-01-16 23:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 51568,'1980-W620','1980-06-23 21:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 51569,'1980-W621','1980-06-24 02:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 5157,'1958-S178','1958-08-30 03:10:00','test rocket','success',1521,'','Argus II','','','Nuclear Test',163,'AVM1',293 from dual union all
select 51570,'1980-W622','1980-06-24 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 51571,'1980-W623','1980-06-25 00:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51572,'1980-W624','1980-06-25 02:03:00','suborbital rocket','success',761,'MT135P-R391','','','','Meteo',1332,'',59 from dual union all
select 51573,'1980-W625','1980-06-25 10:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 51574,'1980-W626','1980-06-25 14:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 51575,'1980-W627','1980-06-25 14:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 51576,'1980-W628','1980-06-25 16:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 51577,'1980-W629','1980-06-25 20:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 51578,'1980-W63','1980-01-18 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual;
insert into LAUNCH
select 51579,'1980-W630','1980-06-26 03:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 5158,'1958-S179','1958-08-30 03:48:00','suborbital rocket','success',623,'Jason 2019 J-16','','','','Nuclear Obs',1821,'',817 from dual union all
select 51580,'1980-W631','1980-06-26 15:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 51581,'1980-W632','1980-06-27 01:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual union all
select 51582,'1980-W633','1980-06-27 08:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',71 from dual union all
select 51583,'1980-W634','1980-06-27 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 51584,'1980-W635','1980-06-27 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',71 from dual union all
select 51585,'1980-W636','1980-06-27 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 51586,'1980-W637','1980-06-27 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 51587,'1980-W638','1980-06-28 00:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual union all
select 51588,'1980-W639','1980-06-30 13:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',53 from dual union all
select 51589,'1980-W64','1980-01-18 14:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 5159,'1958-S180','1958-08-30 04:31:00','suborbital rocket','success',623,'Jason 2022','','','','Nuclear Obs',225,'',878 from dual union all
select 51590,'1980-W640','1980-06-30 14:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',58 from dual union all
select 51591,'1980-W641','1980-06-30 15:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51592,'1980-W642','1980-06-30 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',63 from dual union all
select 51593,'1980-W643','1980-06-30 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51594,'1980-W644','1980-07-01 00:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 51595,'1980-W645','1980-07-01 20:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 51596,'1980-W646','1980-07-02 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 51597,'1980-W647','1980-07-02 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',67 from dual union all
select 51598,'1980-W648','1980-07-02 15:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51599,'1980-W649','1980-07-02 15:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 516,'1945-A35','1945-09-26 17:00:00','suborbital rocket','success',1401,'A','Booster test','','','Test',1852,'',4 from dual union all
select 5160,'1958-S181','1958-08-30 05:18:00','suborbital rocket','success',623,'Jason 2021 J-17','','','','Nuclear Obs',1821,'',830 from dual union all
select 51600,'1980-W65','1980-01-18 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 51601,'1980-W650','1980-07-02 17:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 51602,'1980-W651','1980-07-02 22:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 51603,'1980-W652','1980-07-03 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 51604,'1980-W653','1980-07-03 21:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51605,'1980-W654','1980-07-04 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 51606,'1980-W655','1980-07-05 18:30:00','suborbital rocket','unknown',1284,'FLIGHT 273','','','','Aeron',330,'',66 from dual union all
select 51607,'1980-W656','1980-07-06 14:03:00','suborbital rocket','unknown',1284,'FLIGHT 274','','','','Aeron',330,'',62 from dual union all
select 51608,'1980-W657','1980-07-07 12:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51609,'1980-W658','1980-07-07 13:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',67 from dual union all
select 5161,'1958-S182','1958-08-30 05:52:00','suborbital rocket','success',623,'Jason 2023','','','','Nuclear Obs',1312,'',825 from dual union all
select 51610,'1980-W659','1980-07-07 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 51611,'1980-W66','1980-01-18 15:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',64 from dual union all
select 51612,'1980-W660','1980-07-07 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51613,'1980-W661','1980-07-07 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 51614,'1980-W662','1980-07-07 17:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 51615,'1980-W663','1980-07-07 21:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51616,'1980-W664','1980-07-07 21:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 51617,'1980-W665','1980-07-09 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 51618,'1980-W666','1980-07-09 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 51619,'1980-W667','1980-07-09 14:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',73 from dual union all
select 5162,'1958-S183','1958-08-30 06:36:00','suborbital rocket','success',623,'Jason 2025','','','','Nuclear Obs',225,'',699 from dual union all
select 51620,'1980-W668','1980-07-09 15:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 51621,'1980-W669','1980-07-09 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',53 from dual union all
select 51622,'1980-W67','1980-01-18 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',79 from dual union all
select 51623,'1980-W670','1980-07-10 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 51624,'1980-W671','1980-07-11 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 51625,'1980-W672','1980-07-11 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 51626,'1980-W673','1980-07-11 15:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51627,'1980-W674','1980-07-11 16:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',60 from dual union all
select 51628,'1980-W675','1980-07-11 18:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 51629,'1980-W676','1980-07-12 01:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 5163,'1958-S184','1958-08-30 07:21:00','suborbital rocket','success',623,'Jason 2024 J-18','','','','Nuclear Obs',1821,'',815 from dual union all
select 51630,'1980-W677','1980-07-14 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51631,'1980-W678','1980-07-14 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 51632,'1980-W679','1980-07-14 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51633,'1980-W68','1980-01-18 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51634,'1980-W680','1980-07-14 15:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51635,'1980-W681','1980-07-14 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 51636,'1980-W682','1980-07-14 17:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',76 from dual union all
select 51637,'1980-W683','1980-07-15 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 51638,'1980-W684','1980-07-16 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51639,'1980-W685','1980-07-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',79 from dual union all
select 5164,'1958-S185','1958-08-30 22:02:00','suborbital rocket','success',623,'Jason 2027 J-19','','','','Nuclear Obs',1821,'',745 from dual union all
select 51640,'1980-W686','1980-07-16 14:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 51641,'1980-W687','1980-07-16 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 51642,'1980-W688','1980-07-16 16:07:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 51643,'1980-W689','1980-07-16 20:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 51644,'1980-W69','1980-01-21 05:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 51645,'1980-W690','1980-07-17 00:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 51646,'1980-W691','1980-07-17 13:47:00','suborbital rocket','unknown',1284,'FLIGHT 275     /TN1-','','','','Aeron',330,'',64 from dual union all
select 51647,'1980-W692','1980-07-17 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 51648,'1980-W693','1980-07-17 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 51649,'1980-W694','1980-07-18 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',60 from dual union all
select 5165,'1958-S186','1958-08-30 23:03:00','suborbital rocket','success',623,'Jason 2026','','','','Nuclear Obs',1312,'',800 from dual union all
select 51650,'1980-W695','1980-07-18 15:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 51651,'1980-W696','1980-07-18 19:08:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 51652,'1980-W697','1980-07-18 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 51653,'1980-W698','1980-07-21 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51654,'1980-W699','1980-07-21 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',68 from dual union all
select 51655,'1980-W70','1980-01-21 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51656,'1980-W700','1980-07-21 22:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 51657,'1980-W701','1980-07-23 02:00:00','suborbital rocket','success',761,'MT135P-R393','','','','Meteo',1332,'',53 from dual union all
select 51658,'1980-W702','1980-07-23 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51659,'1980-W703','1980-07-23 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 5166,'1958-S187','1958-09-08 00:00:00','military missile','success',976,'B1-5','K63 SI-3','','','Test',636,'',402 from dual union all
select 51660,'1980-W704','1980-07-23 14:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 51661,'1980-W705','1980-07-23 15:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 51662,'1980-W706','1980-07-23 16:14:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',61 from dual union all
select 51663,'1980-W707','1980-07-23 16:15:00','suborbital rocket','unknown',1284,'FLIGHT 276     /T 1-','','','','Aeron',1821,'',65 from dual union all
select 51664,'1980-W708','1980-07-23 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',67 from dual union all
select 51665,'1980-W709','1980-07-23 19:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 51666,'1980-W71','1980-01-21 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 51667,'1980-W710','1980-07-23 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51668,'1980-W711','1980-07-24 03:20:00','suborbital rocket','unknown',1284,'FLIGHT 277     /TM1-','','','','Aeron',1047,'',78 from dual;
insert into LAUNCH
select 51669,'1980-W712','1980-07-24 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 5167,'1958-S188','1958-09-23 00:00:00','military missile','success',976,'B1-4','K63 SI-4','','','Test',636,'',402 from dual union all
select 51670,'1980-W713','1980-07-24 22:35:00','suborbital rocket','unknown',1284,'FLIGHT 278     /TM1-','','','','Aeron',1047,'',70 from dual union all
select 51671,'1980-W714','1980-07-25 03:41:00','suborbital rocket','unknown',1284,'FLIGHT 279     /TM1-','','','','Aeron',1047,'',66 from dual union all
select 51672,'1980-W715','1980-07-25 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51673,'1980-W716','1980-07-25 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51674,'1980-W717','1980-07-25 14:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',52 from dual union all
select 51675,'1980-W718','1980-07-25 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 51676,'1980-W719','1980-07-25 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51677,'1980-W72','1980-01-21 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 51678,'1980-W720','1980-07-28 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 51679,'1980-W721','1980-07-28 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',69 from dual union all
select 5168,'1958-S189','1958-08-31 00:07:00','suborbital rocket','success',623,'Jason 2020','','','','Nuclear Obs',225,'',800 from dual union all
select 51680,'1980-W722','1980-07-28 15:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51681,'1980-W723','1980-07-28 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 51682,'1980-W724','1980-07-30 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',67 from dual union all
select 51683,'1980-W725','1980-07-30 15:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51684,'1980-W726','1980-07-30 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 51685,'1980-W727','1980-07-30 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51686,'1980-W728','1980-07-30 21:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 51687,'1980-W729','1980-07-31 12:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 51688,'1980-W73','1980-01-21 18:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 51689,'1980-W730','1980-08-01 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 5169,'1958-S19','1958-02-03 18:02:20','suborbital rocket','success',49,'NN3.10F','NRL NN3.10F','IGY NN3.10F','','Test/Ionos',428,'',138 from dual union all
select 51690,'1980-W731','1980-08-01 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',68 from dual union all
select 51691,'1980-W732','1980-08-01 14:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51692,'1980-W733','1980-08-01 17:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 51693,'1980-W734','1980-08-01 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',67 from dual union all
select 51694,'1980-W735','1980-08-01 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 51695,'1980-W736','1980-08-04 12:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51696,'1980-W737','1980-08-04 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 51697,'1980-W738','1980-08-04 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51698,'1980-W739','1980-08-04 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51699,'1980-W74','1980-01-22 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 517,'1945-A36','1945-09-26 00:00:00','suborbital rocket','success',1401,'B','Booster test','','','Test',1852,'',4 from dual union all
select 5170,'1958-S190','1958-09-02 19:02:00','suborbital rocket','success',623,'Jason 2041','','','','Nuclear Obs',1312,'',800 from dual union all
select 51700,'1980-W740','1980-08-04 21:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 51701,'1980-W741','1980-08-05 13:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 51702,'1980-W742','1980-08-05 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 51703,'1980-W743','1980-08-06 01:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 51704,'1980-W744','1980-08-06 02:00:00','suborbital rocket','success',761,'MT135P-R394','','','','Meteo',1332,'',58 from dual union all
select 51705,'1980-W745','1980-08-06 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',54 from dual union all
select 51706,'1980-W746','1980-08-06 14:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 51707,'1980-W747','1980-08-06 15:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51708,'1980-W748','1980-08-06 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51709,'1980-W749','1980-08-07 15:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 5171,'1958-S191','1958-09-02 20:03:00','suborbital rocket','success',623,'Jason 2042 J-20','','','','Nuclear Obs',1821,'',789 from dual union all
select 51710,'1980-W75','1980-01-23 02:00:00','suborbital rocket','success',761,'MT135P-R374','','','','Meteo',1332,'',53 from dual union all
select 51711,'1980-W750','1980-08-07 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 51712,'1980-W751','1980-08-07 21:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 51713,'1980-W752','1980-08-08 15:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 51714,'1980-W753','1980-08-08 18:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 51715,'1980-W754','1980-08-08 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51716,'1980-W755','1980-08-09 04:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 51717,'1980-W756','1980-08-11 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51718,'1980-W757','1980-08-11 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 51719,'1980-W758','1980-08-11 14:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 5172,'1958-S192','1958-09-02 22:15:00','suborbital rocket','success',623,'Jason 2043','','','','Nuclear Obs',225,'',789 from dual union all
select 51720,'1980-W759','1980-08-11 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 51721,'1980-W76','1980-01-23 02:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',78 from dual union all
select 51722,'1980-W760','1980-08-11 17:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',63 from dual union all
select 51723,'1980-W761','1980-08-11 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51724,'1980-W762','1980-08-12 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 51725,'1980-W763','1980-08-13 14:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 51726,'1980-W764','1980-08-13 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',66 from dual union all
select 51727,'1980-W765','1980-08-13 16:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 51728,'1980-W766','1980-08-13 18:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 51729,'1980-W767','1980-08-13 21:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 5173,'1958-S193','1958-09-06 00:00:00','military missile','success',974,'','','','','Test',1062,'B-62',86 from dual union all
select 51730,'1980-W768','1980-08-14 00:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 51731,'1980-W769','1980-08-14 00:19:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',59 from dual union all
select 51732,'1980-W77','1980-01-23 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 51733,'1980-W770','1980-08-14 03:42:00','suborbital rocket','success',1061,'','','','','Meteo',1821,'',58 from dual union all
select 51734,'1980-W771','1980-08-14 11:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 51735,'1980-W772','1980-08-14 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 51736,'1980-W773','1980-08-15 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 51737,'1980-W774','1980-08-15 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',63 from dual union all
select 51738,'1980-W775','1980-08-15 14:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51739,'1980-W776','1980-08-15 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 5174,'1958-S194','1958-09-06 22:05:00','test rocket','success',1521,'','Argus III','','','Nuclear Test',164,'AVM1',750 from dual union all
select 51740,'1980-W777','1980-08-15 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 51741,'1980-W778','1980-08-15 21:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',69 from dual union all
select 51742,'1980-W779','1980-08-18 13:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 51743,'1980-W78','1980-01-23 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 51744,'1980-W780','1980-08-18 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',69 from dual union all
select 51745,'1980-W781','1980-08-18 14:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 51746,'1980-W782','1980-08-18 18:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 51747,'1980-W783','1980-08-18 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51748,'1980-W784','1980-08-19 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51749,'1980-W785','1980-08-20 00:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 5175,'1958-S195','1958-09-07 10:33:00','military missile','success',239,'BK.01','H01/Proving Trial','','','Test',1843,'',225 from dual union all
select 51750,'1980-W786','1980-08-20 02:01:00','suborbital rocket','success',761,'MT135P-R395','','','','Meteo',1332,'',59 from dual union all
select 51751,'1980-W787','1980-08-20 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51752,'1980-W788','1980-08-20 14:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51753,'1980-W789','1980-08-20 15:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 51754,'1980-W79','1980-01-23 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 51755,'1980-W790','1980-08-20 16:14:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',52 from dual union all
select 51756,'1980-W791','1980-08-20 16:15:00','suborbital rocket','unknown',1284,'FLIGHT 280     /T 1-','','','','Aeron',1821,'',53 from dual union all
select 51757,'1980-W792','1980-08-20 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 51758,'1980-W793','1980-08-20 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual;
insert into LAUNCH
select 51759,'1980-W794','1980-08-20 17:50:00','suborbital rocket','unknown',716,'FLIGHT 281     /TH1-','','','','Aeron',1307,'',70 from dual union all
select 5176,'1958-S196','1958-09-11 00:00:00','suborbital rocket','success',839,'','Beacon Test 3','','','Balloon Test',1821,'',122 from dual union all
select 51760,'1980-W795','1980-08-21 21:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 51761,'1980-W796','1980-08-22 13:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 51762,'1980-W797','1980-08-22 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51763,'1980-W798','1980-08-22 14:25:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',67 from dual union all
select 51764,'1980-W799','1980-08-22 14:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',70 from dual union all
select 51765,'1980-W80','1980-01-23 17:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 51766,'1980-W800','1980-08-22 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 51767,'1980-W801','1980-08-22 21:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51768,'1980-W802','1980-08-25 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 51769,'1980-W803','1980-08-25 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 5177,'1958-S197','1958-09-14 05:24:00','military missile','success',147,'8B','Mk 2-1CX RV/AFSWC-2','R&D','','Test',229,'',900 from dual union all
select 51770,'1980-W804','1980-08-25 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 51771,'1980-W805','1980-08-25 21:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',66 from dual union all
select 51772,'1980-W806','1980-08-26 02:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 51773,'1980-W807','1980-08-26 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51774,'1980-W808','1980-08-26 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',79 from dual union all
select 51775,'1980-W809','1980-08-27 04:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',70 from dual union all
select 51776,'1980-W81','1980-01-23 17:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 51777,'1980-W810','1980-08-27 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',53 from dual union all
select 51778,'1980-W811','1980-08-27 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 51779,'1980-W812','1980-08-27 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 5178,'1958-S198','1958-09-15 16:32:00','suborbital rocket','success',839,'','Release 5 (Cs)','','','Aeron',646,'',82 from dual union all
select 51780,'1980-W813','1980-08-28 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 51781,'1980-W814','1980-08-29 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 51782,'1980-W815','1980-08-29 14:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 51783,'1980-W816','1980-08-29 15:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 51784,'1980-W817','1980-08-29 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51785,'1980-W818','1980-09-01 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 51786,'1980-W819','1980-09-02 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',51 from dual union all
select 51787,'1980-W82','1980-01-23 22:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51788,'1980-W820','1980-09-02 19:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',55 from dual union all
select 51789,'1980-W821','1980-09-03 02:00:00','suborbital rocket','success',761,'MT135P-R396','','','','Meteo',1332,'',58 from dual union all
select 5179,'1958-S199','1958-09-16 16:31:00','suborbital rocket','success',839,'','Release 6 (Cs) 69km','','','Aeron',646,'',80 from dual union all
select 51790,'1980-W822','1980-09-03 03:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51791,'1980-W823','1980-09-03 10:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',69 from dual union all
select 51792,'1980-W824','1980-09-03 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',54 from dual union all
select 51793,'1980-W825','1980-09-03 14:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',78 from dual union all
select 51794,'1980-W826','1980-09-03 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',65 from dual union all
select 51795,'1980-W827','1980-09-03 16:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 51796,'1980-W828','1980-09-03 18:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 51797,'1980-W829','1980-09-03 19:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',77 from dual union all
select 51798,'1980-W83','1980-01-24 01:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 51799,'1980-W830','1980-09-03 20:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 518,'1945-A37','1945-09-26 00:00:00','suborbital rocket','success',1401,'C','Booster test','','','Test',1852,'',4 from dual union all
select 5180,'1958-S20','1958-02-04 06:16:58','suborbital rocket','success',49,'NN3.11F','NRL NN3.11F','IGY NN3.11F','','Ionos',428,'',234 from dual union all
select 51800,'1980-W831','1980-09-04 03:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51801,'1980-W832','1980-09-05 00:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 51802,'1980-W833','1980-09-05 08:20:00','suborbital rocket','unknown',738,'','','','','Aeron',907,'KRENKEL''',54 from dual union all
select 51803,'1980-W834','1980-09-05 09:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',52 from dual union all
select 51804,'1980-W835','1980-09-05 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 51805,'1980-W836','1980-09-05 14:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51806,'1980-W837','1980-09-05 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 51807,'1980-W838','1980-09-05 19:51:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',79 from dual union all
select 51808,'1980-W839','1980-09-05 20:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 51809,'1980-W84','1980-01-24 17:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 5181,'1958-S200','1958-09-17 18:00:00','military missile','success',1053,'CC-56    SV','','','','Test',280,'',90 from dual union all
select 51810,'1980-W840','1980-09-05 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 51811,'1980-W841','1980-09-08 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',72 from dual union all
select 51812,'1980-W842','1980-09-08 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 51813,'1980-W843','1980-09-08 15:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',72 from dual union all
select 51814,'1980-W844','1980-09-08 18:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 51815,'1980-W845','1980-09-08 21:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 51816,'1980-W846','1980-09-09 16:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',78 from dual union all
select 51817,'1980-W847','1980-09-09 18:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',75 from dual union all
select 51818,'1980-W848','1980-09-09 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 51819,'1980-W849','1980-09-09 22:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 5182,'1958-S201','1958-09-18 16:49:58','suborbital rocket','success',839,'','Release 7 (Cs)','','','Aeron',646,'',91 from dual union all
select 51820,'1980-W85','1980-01-25 12:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51821,'1980-W850','1980-09-10 07:35:00','suborbital rocket','unknown',738,'','','','','Aeron',907,'KRENKEL''',57 from dual union all
select 51822,'1980-W851','1980-09-10 13:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',71 from dual union all
select 51823,'1980-W852','1980-09-10 13:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 51824,'1980-W853','1980-09-10 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 51825,'1980-W854','1980-09-10 16:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 51826,'1980-W855','1980-09-10 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 51827,'1980-W856','1980-09-11 21:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 51828,'1980-W857','1980-09-12 04:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 51829,'1980-W858','1980-09-12 07:35:00','suborbital rocket','unknown',738,'','','','','Aeron',907,'KRENKEL''',56 from dual union all
select 5183,'1958-S202','1958-09-18 21:27:00','military missile','failure',147,'6B','Low Drag Fairing','R&D','','Test',228,'',100 from dual union all
select 51830,'1980-W859','1980-09-12 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 51831,'1980-W86','1980-01-25 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 51832,'1980-W860','1980-09-12 15:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51833,'1980-W861','1980-09-12 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',66 from dual union all
select 51834,'1980-W862','1980-09-13 01:30:00','suborbital rocket','success',761,'MT135P T28','','','','Aeron?',1476,'',50 from dual union all
select 51835,'1980-W863','1980-09-15 01:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 51836,'1980-W864','1980-09-15 01:30:00','suborbital rocket','success',761,'MT135P T29','','','','Aeron?',1476,'',50 from dual union all
select 51837,'1980-W865','1980-09-15 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 51838,'1980-W866','1980-09-15 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 51839,'1980-W867','1980-09-15 14:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 5184,'1958-S203','1958-09-19 00:48:00','suborbital rocket','success',1011,'A1-10','R-5A Geophysical 3','E-1 Kometa (Na test)','','Aeron/Ionos/Meteori',636,'',430 from dual union all
select 51840,'1980-W868','1980-09-15 15:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51841,'1980-W869','1980-09-15 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 51842,'1980-W87','1980-01-25 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 51843,'1980-W870','1980-09-15 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 51844,'1980-W871','1980-09-15 19:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 51845,'1980-W872','1980-09-15 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51846,'1980-W873','1980-09-16 04:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51847,'1980-W874','1980-09-17 02:00:00','suborbital rocket','success',761,'MT135P-R397','','','','Meteo',1332,'',58 from dual union all
select 51848,'1980-W875','1980-09-17 12:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',58 from dual;
insert into LAUNCH
select 51849,'1980-W876','1980-09-17 14:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',66 from dual union all
select 5185,'1958-S204','1958-09-22 03:47:00','suborbital rocket','success',726,'','','','','Aeron',636,'',85 from dual union all
select 51850,'1980-W877','1980-09-17 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 51851,'1980-W878','1980-09-17 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 51852,'1980-W879','1980-09-17 21:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 51853,'1980-W88','1980-01-25 15:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51854,'1980-W880','1980-09-18 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 51855,'1980-W881','1980-09-18 21:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 51856,'1980-W882','1980-09-19 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',907,'KRENKEL''',57 from dual union all
select 51857,'1980-W883','1980-09-19 13:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 51858,'1980-W884','1980-09-19 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',52 from dual union all
select 51859,'1980-W885','1980-09-19 14:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 5186,'1958-S205','1958-09-25 00:00:00','suborbital rocket','success',470,'','Model 2','','','Test',1821,'',460 from dual union all
select 51860,'1980-W886','1980-09-19 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 51861,'1980-W887','1980-09-19 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 51862,'1980-W888','1980-09-19 16:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 51863,'1980-W889','1980-09-19 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 51864,'1980-W89','1980-01-25 16:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',52 from dual union all
select 51865,'1980-W890','1980-09-19 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 51866,'1980-W891','1980-09-20 02:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51867,'1980-W892','1980-09-22 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',54 from dual union all
select 51868,'1980-W893','1980-09-22 16:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 51869,'1980-W894','1980-09-22 22:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 5187,'1958-S206','1958-09-25 00:00:00','test rocket','success',253,'','','','','Test',293,'B-47',100 from dual union all
select 51870,'1980-W895','1980-09-23 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 51871,'1980-W896','1980-09-24 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',907,'KRENKEL''',58 from dual union all
select 51872,'1980-W897','1980-09-24 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 51873,'1980-W898','1980-09-24 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 51874,'1980-W899','1980-09-24 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',51 from dual union all
select 51875,'1980-W90','1980-01-25 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 51876,'1980-W900','1980-09-24 15:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',73 from dual union all
select 51877,'1980-W901','1980-09-24 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 51878,'1980-W902','1980-09-24 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51879,'1980-W903','1980-09-25 18:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 5188,'1958-S207','1958-09-27 03:48:00','suborbital rocket','success',726,'','','','','Aeron',636,'',85 from dual union all
select 51880,'1980-W904','1980-09-25 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51881,'1980-W905','1980-09-26 12:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',52 from dual union all
select 51882,'1980-W906','1980-09-26 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',53 from dual union all
select 51883,'1980-W907','1980-09-26 14:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',67 from dual union all
select 51884,'1980-W908','1980-09-26 15:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 51885,'1980-W909','1980-09-26 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 51886,'1980-W91','1980-01-26 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 51887,'1980-W910','1980-09-26 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 51888,'1980-W911','1980-09-29 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 51889,'1980-W912','1980-09-29 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 5189,'1958-S208','1958-09-28 19:42:00','suborbital rocket','success',838,'NN8.57F','','IGY NN8.57F','','Test/Solar UV/Solar',1135,'LSD-31',225 from dual union all
select 51890,'1980-W913','1980-09-29 14:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',68 from dual union all
select 51891,'1980-W914','1980-09-29 17:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 51892,'1980-W915','1980-09-29 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 51893,'1980-W916','1980-09-29 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',53 from dual union all
select 51894,'1980-W917','1980-09-29 21:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 51895,'1980-W918','1980-09-30 02:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 51896,'1980-W919','1980-09-30 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 51897,'1980-W92','1980-01-28 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51898,'1980-W920','1980-10-01 02:13:00','suborbital rocket','success',761,'MT135P-R398','','','','Meteo',1332,'',55 from dual union all
select 51899,'1980-W921','1980-10-01 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',907,'KRENKEL''',57 from dual union all
select 519,'1945-A38','1945-09-27 00:00:00','suborbital rocket','success',1401,'D','Booster test','','','Test',1852,'',4 from dual union all
select 5190,'1958-S209','1958-10-02 00:00:00','military missile','success',976,'B1-3','K63 SI-5','','','Test',636,'',402 from dual union all
select 51900,'1980-W922','1980-10-01 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',209,'BUGAEV',57 from dual union all
select 51901,'1980-W923','1980-10-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',78 from dual union all
select 51902,'1980-W924','1980-10-01 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 51903,'1980-W925','1980-10-01 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 51904,'1980-W926','1980-10-01 22:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 51905,'1980-W927','1980-10-01 22:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51906,'1980-W928','1980-10-02 14:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 51907,'1980-W929','1980-10-02 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 51908,'1980-W93','1980-01-28 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',54 from dual union all
select 51909,'1980-W930','1980-10-02 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 5191,'1958-S21','1958-02-07 19:37:00','military missile','failure',142,'13A','R&D Fairing','R&D','','Test',229,'',120 from dual union all
select 51910,'1980-W931','1980-10-03 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',209,'BUGAEV',57 from dual union all
select 51911,'1980-W932','1980-10-03 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 51912,'1980-W933','1980-10-03 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 51913,'1980-W934','1980-10-06 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 51914,'1980-W935','1980-10-06 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51915,'1980-W936','1980-10-06 20:34:00','suborbital rocket','success',1284,'TY1-6190','','','','Test',29,'',70 from dual union all
select 51916,'1980-W937','1980-10-06 21:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 51917,'1980-W938','1980-10-07 04:28:57','suborbital rocket','success',1284,'TY1-6878','','','','Ionos',29,'',60 from dual union all
select 51918,'1980-W939','1980-10-07 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 51919,'1980-W94','1980-01-28 14:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 5192,'1958-S210','1958-10-14 00:00:00','military missile','success',976,'B1-6','K63 SI-6','','','Test',636,'',402 from dual union all
select 51920,'1980-W940','1980-10-07 15:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51921,'1980-W941','1980-10-07 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 51922,'1980-W942','1980-10-08 00:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 51923,'1980-W943','1980-10-08 02:04:59','suborbital rocket','success',761,'MT135P-R399','','','','Meteo',1332,'',58 from dual union all
select 51924,'1980-W944','1980-10-08 05:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51925,'1980-W945','1980-10-08 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',209,'BUGAEV',57 from dual union all
select 51926,'1980-W946','1980-10-08 09:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 51927,'1980-W947','1980-10-08 15:03:00','suborbital rocket','unknown',1284,'FLIGHT 282     /TN1-','','','','Aeron',330,'',71 from dual union all
select 51928,'1980-W948','1980-10-08 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 51929,'1980-W949','1980-10-09 12:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 5193,'1958-S211','1958-10-03 03:32:00','suborbital rocket','success',726,'','','','','Aeron',636,'',85 from dual union all
select 51930,'1980-W95','1980-01-28 14:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1470,'',69 from dual union all
select 51931,'1980-W950','1980-10-09 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 51932,'1980-W951','1980-10-10 21:29:12','suborbital rocket','success',1284,'TY1-6195','','','','Aeron',29,'',67 from dual union all
select 51933,'1980-W952','1980-10-10 22:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51934,'1980-W953','1980-10-11 20:33:20','suborbital rocket','success',1284,'TY1-6196','','','','Aeron',29,'',69 from dual union all
select 51935,'1980-W954','1980-10-11 23:28:30','suborbital rocket','success',1284,'TY1-7128','','','','Ionos',29,'',65 from dual union all
select 51936,'1980-W955','1980-10-14 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 51937,'1980-W956','1980-10-14 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 51938,'1980-W957','1980-10-14 15:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual;
insert into LAUNCH
select 51939,'1980-W958','1980-10-15 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',209,'BUGAEV',57 from dual union all
select 5194,'1958-S212','1958-10-04 14:00:00','suborbital rocket','success',5,'','','','','Iono-solar',636,'',110 from dual union all
select 51940,'1980-W959','1980-10-15 12:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 51941,'1980-W96','1980-01-28 19:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',75 from dual union all
select 51942,'1980-W960','1980-10-15 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 51943,'1980-W961','1980-10-15 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',71 from dual union all
select 51944,'1980-W962','1980-10-15 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 51945,'1980-W963','1980-10-15 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 51946,'1980-W964','1980-10-15 18:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 51947,'1980-W965','1980-10-15 20:53:00','suborbital rocket','success',1284,'TY1-6424','','','','Aeron',29,'',67 from dual union all
select 51948,'1980-W966','1980-10-15 21:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 51949,'1980-W967','1980-10-16 08:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 5195,'1958-S213','1958-10-04 15:08:00','suborbital rocket','success',972,'1-2','','','','Aeron?',636,'',98 from dual union all
select 51950,'1980-W968','1980-10-16 15:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 51951,'1980-W969','1980-10-16 16:45:00','suborbital rocket','unknown',1284,'FLIGHT 284     /T 1-','','','','Aeron',1821,'',65 from dual union all
select 51952,'1980-W97','1980-01-28 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 51953,'1980-W970','1980-10-16 18:20:00','suborbital rocket','unknown',1284,'FLIGHT 285     /T 1-','','','','Aeron',1821,'',66 from dual union all
select 51954,'1980-W971','1980-10-16 19:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',76 from dual union all
select 51955,'1980-W972','1980-10-17 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',209,'BUGAEV',57 from dual union all
select 51956,'1980-W973','1980-10-17 13:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 51957,'1980-W974','1980-10-17 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',69 from dual union all
select 51958,'1980-W975','1980-10-17 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',61 from dual union all
select 51959,'1980-W976','1980-10-18 02:07:14','suborbital rocket','success',1284,'TY1-7129','','','','Ionos',29,'',52 from dual union all
select 5196,'1958-S214','1958-10-06 10:30:00','suborbital rocket','success',726,'','','','','Aeron/SB',636,'',85 from dual union all
select 51960,'1980-W977','1980-10-18 02:13:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',66 from dual union all
select 51961,'1980-W978','1980-10-18 03:52:00','suborbital rocket','success',1284,'TY1-7130','','','','Ionos',29,'',63 from dual union all
select 51962,'1980-W979','1980-10-18 04:35:02','suborbital rocket','success',1284,'TY1-6875','','','','Aeron',29,'',75 from dual union all
select 51963,'1980-W98','1980-01-29 01:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 51964,'1980-W980','1980-10-20 12:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 51965,'1980-W981','1980-10-20 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',69 from dual union all
select 51966,'1980-W982','1980-10-20 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 51967,'1980-W983','1980-10-22 02:10:00','suborbital rocket','success',761,'MT135P-R400','','','','Meteo',1332,'',58 from dual union all
select 51968,'1980-W984','1980-10-22 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',209,'BUGAEV',57 from dual union all
select 51969,'1980-W985','1980-10-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',78 from dual union all
select 5197,'1958-S215','1958-10-07 00:00:00','suborbital rocket','success',839,'','Hi Ball 3','','','Balloon Test',1821,'',244 from dual union all
select 51970,'1980-W986','1980-10-22 15:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 51971,'1980-W987','1980-10-22 16:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',59 from dual union all
select 51972,'1980-W988','1980-10-22 18:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 51973,'1980-W989','1980-10-22 20:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 51974,'1980-W99','1980-01-30 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 51975,'1980-W990','1980-10-23 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 51976,'1980-W991','1980-10-23 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 51977,'1980-W992','1980-10-24 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',209,'BUGAEV',60 from dual union all
select 51978,'1980-W993','1980-10-24 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 51979,'1980-W994','1980-10-24 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',68 from dual union all
select 5198,'1958-S216','1958-10-08 03:50:00','suborbital rocket','success',726,'','','','','Aeron',636,'',85 from dual union all
select 51980,'1980-W995','1980-10-24 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 51981,'1980-W996','1980-10-24 19:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',61 from dual union all
select 51982,'1980-W997','1980-10-24 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 51983,'1980-W998','1980-10-24 21:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 51984,'1980-W999','1980-10-27 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 51985,'1980-Y01','1980-01-01 02:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 51986,'1980-Y02','1980-01-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',84 from dual union all
select 51987,'1980-Y03','1980-01-02 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 51988,'1980-Y04','1980-01-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 51989,'1980-Y05','1980-01-02 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 5199,'1958-S217','1958-10-10 00:00:00','suborbital rocket','success',972,'1-1','','','','Aeron?',636,'',103 from dual union all
select 51990,'1980-Y06','1980-01-02 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',90 from dual union all
select 51991,'1980-Y07','1980-01-02 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 51992,'1980-Y08','1980-01-03 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',84 from dual union all
select 51993,'1980-Y09','1980-01-03 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',83 from dual union all
select 51994,'1980-Y10','1980-01-04 02:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 51995,'1980-Y100','1980-02-25 17:50:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',83 from dual union all
select 51996,'1980-Y101','1980-02-25 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',84 from dual union all
select 51997,'1980-Y102','1980-02-25 19:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 51998,'1980-Y103','1980-02-25 19:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',85 from dual union all
select 51999,'1980-Y104','1980-02-26 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',89 from dual union all
select 52,'1944-A16','1944-09-06 10:40:00','military missile','failure',1470,'18593','Paris','','','Weapon',1743,'',1 from dual union all
select 520,'1945-A39','1945-03-23 07:00:00','military missile','failure',1470,'','London','','','Weapon',1734,'',1 from dual union all
select 5200,'1958-S218','1958-10-10 00:00:00','test rocket','success',253,'','','','','Test',293,'B-47',100 from dual union all
select 52000,'1980-Y105','1980-02-26 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',80 from dual union all
select 52001,'1980-Y106','1980-02-26 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',82 from dual union all
select 52002,'1980-Y107','1980-02-26 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',87 from dual union all
select 52003,'1980-Y108','1980-02-26 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',89 from dual union all
select 52004,'1980-Y109','1980-02-26 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',85 from dual union all
select 52005,'1980-Y11','1980-01-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 52006,'1980-Y110','1980-02-26 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',86 from dual union all
select 52007,'1980-Y111','1980-02-26 18:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',82 from dual union all
select 52008,'1980-Y112','1980-02-27 04:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 52009,'1980-Y113','1980-02-27 12:10:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',84 from dual union all
select 5201,'1958-S219','1958-10-10 13:50:00','suborbital rocket','success',5,'','','','','Iono-solar',636,'',110 from dual union all
select 52010,'1980-Y114','1980-02-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 52011,'1980-Y115','1980-02-27 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 52012,'1980-Y116','1980-02-27 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',86 from dual union all
select 52013,'1980-Y117','1980-02-27 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 52014,'1980-Y118','1980-02-27 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',84 from dual union all
select 52015,'1980-Y119','1980-02-27 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',83 from dual union all
select 52016,'1980-Y12','1980-01-04 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 52017,'1980-Y120','1980-02-27 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',82 from dual union all
select 52018,'1980-Y121','1980-02-28 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',86 from dual union all
select 52019,'1980-Y122','1980-02-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',88 from dual union all
select 5202,'1958-S22','1958-02-12 00:54:00','military missile','success',1053,'CC-46    TV','','','','Test',280,'',90 from dual union all
select 52020,'1980-Y123','1980-02-28 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',83 from dual union all
select 52021,'1980-Y124','1980-02-28 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',83 from dual union all
select 52022,'1980-Y125','1980-02-28 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',85 from dual union all
select 52023,'1980-Y126','1980-02-28 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',83 from dual union all
select 52024,'1980-Y127','1980-02-28 18:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',84 from dual union all
select 52025,'1980-Y128','1980-02-28 23:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',80 from dual union all
select 52026,'1980-Y129','1980-02-29 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',83 from dual union all
select 52027,'1980-Y13','1980-01-05 01:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual;
insert into LAUNCH
select 52028,'1980-Y130','1980-02-29 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',87 from dual union all
select 52029,'1980-Y131','1980-02-29 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',88 from dual union all
select 5203,'1958-S220','1958-10-12 19:32:06','suborbital rocket','success',838,'NN8.59F','','IGY NN8.59F','','Solar EUV/Solar X',1135,'LSD-31',222 from dual union all
select 52030,'1980-Y132','1980-02-29 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',84 from dual union all
select 52031,'1980-Y133','1980-02-29 13:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',84 from dual union all
select 52032,'1980-Y134','1980-02-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',84 from dual union all
select 52033,'1980-Y135','1980-02-29 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',84 from dual union all
select 52034,'1980-Y136','1980-02-29 15:15:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',88 from dual union all
select 52035,'1980-Y137','1980-02-29 15:50:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',85 from dual union all
select 52036,'1980-Y138','1980-03-05 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 52037,'1980-Y139','1980-03-05 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 52038,'1980-Y14','1980-01-05 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1399,'SHOK',85 from dual union all
select 52039,'1980-Y140','1980-03-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 5204,'1958-S221','1958-10-12 19:42:03','suborbital rocket','success',838,'NN8.60F','','IGY NN8.60F','','Solar EUV/Solar X',1135,'LSD-31',236 from dual union all
select 52040,'1980-Y141','1980-03-05 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 52041,'1980-Y142','1980-03-05 20:55:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 52042,'1980-Y143','1980-03-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',88 from dual union all
select 52043,'1980-Y144','1980-03-07 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 52044,'1980-Y145','1980-03-07 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',84 from dual union all
select 52045,'1980-Y146','1980-03-10 15:11:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52046,'1980-Y147','1980-03-12 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 52047,'1980-Y148','1980-03-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 52048,'1980-Y149','1980-03-12 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 52049,'1980-Y15','1980-01-07 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 5205,'1958-S222','1958-10-12 19:43:18','suborbital rocket','success',838,'NN8.61F','','IGY NN8.61F','','Solar EUV/Solar X',1135,'LSD-31',242 from dual union all
select 52050,'1980-Y150','1980-03-18 00:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',81 from dual union all
select 52051,'1980-Y151','1980-03-18 19:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',88 from dual union all
select 52052,'1980-Y152','1980-03-19 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 52053,'1980-Y153','1980-03-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 52054,'1980-Y154','1980-03-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 52055,'1980-Y155','1980-03-19 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 52056,'1980-Y156','1980-03-19 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 52057,'1980-Y157','1980-03-19 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',94 from dual union all
select 52058,'1980-Y158','1980-03-19 19:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 52059,'1980-Y159','1980-03-19 20:56:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 5206,'1958-S223','1958-10-12 19:52:49','suborbital rocket','success',838,'NN8.62F','','IGY NN8.62F','','Solar EUV/Solar X',1135,'LSD-31',240 from dual union all
select 52060,'1980-Y16','1980-01-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 52061,'1980-Y160','1980-03-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 52062,'1980-Y161','1980-03-21 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 52063,'1980-Y162','1980-03-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',83 from dual union all
select 52064,'1980-Y163','1980-03-24 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52065,'1980-Y164','1980-03-26 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 52066,'1980-Y165','1980-03-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52067,'1980-Y166','1980-03-26 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 52068,'1980-Y167','1980-03-26 18:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 52069,'1980-Y168','1980-03-26 18:52:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 5207,'1958-S224','1958-10-12 20:10:09','suborbital rocket','success',838,'NN8.63F','','IGY NN8.63F','','Solar EUV/Solar X',1135,'LSD-31',88 from dual union all
select 52070,'1980-Y169','1980-03-26 19:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 52071,'1980-Y17','1980-01-07 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 52072,'1980-Y170','1980-03-26 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 52073,'1980-Y171','1980-03-27 07:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 52074,'1980-Y172','1980-03-27 10:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 52075,'1980-Y173','1980-03-28 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',92 from dual union all
select 52076,'1980-Y174','1980-03-29 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',84 from dual union all
select 52077,'1980-Y175','1980-03-30 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',81 from dual union all
select 52078,'1980-Y176','1980-03-31 23:12:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 52079,'1980-Y177','1980-03-31 23:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',82 from dual union all
select 5208,'1958-S225','1958-10-13 19:39:56','suborbital rocket','success',838,'NN8.64F','','IGY NN8.64F','','Test',1135,'LSD-31',225 from dual union all
select 52080,'1980-Y178','1980-04-02 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 52081,'1980-Y179','1980-04-02 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 52082,'1980-Y18','1980-01-07 18:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52083,'1980-Y180','1980-04-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 52084,'1980-Y181','1980-04-02 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 52085,'1980-Y182','1980-04-02 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',89 from dual union all
select 52086,'1980-Y183','1980-04-09 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 52087,'1980-Y184','1980-04-09 02:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52088,'1980-Y185','1980-04-09 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 52089,'1980-Y186','1980-04-09 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 5209,'1958-S226','1958-10-14 12:00:42','suborbital rocket','success',839,'AA6.204','','IGY AA6.204','','Meteorites',428,'',137 from dual union all
select 52090,'1980-Y187','1980-04-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 52091,'1980-Y188','1980-04-09 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 52092,'1980-Y189','1980-04-09 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 52093,'1980-Y19','1980-01-09 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 52094,'1980-Y190','1980-04-09 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 52095,'1980-Y191','1980-04-09 18:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',82 from dual union all
select 52096,'1980-Y192','1980-04-09 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 52097,'1980-Y193','1980-04-09 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 52098,'1980-Y194','1980-04-10 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 52099,'1980-Y195','1980-04-10 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 521,'1945-M01','1945-01-24 00:00:00','military missile','success',9,'2','A-9','','','Test',674,'',77 from dual union all
select 5210,'1958-S227','1958-10-15 00:00:00','military missile','success',992,'','NIIP-A target','','','Target',634,'',100 from dual union all
select 52100,'1980-Y196','1980-04-10 10:04:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 52101,'1980-Y197','1980-04-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 52102,'1980-Y198','1980-04-10 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 52103,'1980-Y199','1980-04-10 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52104,'1980-Y20','1980-01-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 52105,'1980-Y200','1980-04-11 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 52106,'1980-Y201','1980-04-11 06:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52107,'1980-Y202','1980-04-14 15:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',89 from dual union all
select 52108,'1980-Y203','1980-04-16 02:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 52109,'1980-Y204','1980-04-16 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 5211,'1958-S228','1958-10-15 01:04:36','suborbital rocket','success',49,'AM4.012','Vehicle test','IGY AM4.12','','Test/Aeron',428,'',156 from dual union all
select 52110,'1980-Y205','1980-04-16 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 52111,'1980-Y206','1980-04-22 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 52112,'1980-Y207','1980-04-23 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 52113,'1980-Y208','1980-04-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 52114,'1980-Y209','1980-04-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 52115,'1980-Y21','1980-01-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 52116,'1980-Y210','1980-04-23 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 52117,'1980-Y211','1980-04-26 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual;
insert into LAUNCH
select 52118,'1980-Y212','1980-04-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 52119,'1980-Y213','1980-04-29 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',89 from dual union all
select 5212,'1958-S229','1958-10-15 04:00:01','suborbital rocket','success',839,'AA6.203','','IGY AA6.203','','Meteorites',428,'',151 from dual union all
select 52120,'1980-Y214','1980-04-29 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',82 from dual union all
select 52121,'1980-Y215','1980-04-29 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',83 from dual union all
select 52122,'1980-Y216','1980-04-29 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 52123,'1980-Y217','1980-04-30 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 52124,'1980-Y218','1980-04-30 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 52125,'1980-Y219','1980-04-30 04:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 52126,'1980-Y22','1980-01-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 52127,'1980-Y220','1980-04-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 52128,'1980-Y221','1980-04-30 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 52129,'1980-Y222','1980-04-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 5213,'1958-S23','1958-02-13 06:37:04','suborbital rocket','success',839,'II6.24F','Auroral particles','IGY II6.24F','','Auroral',428,'',129 from dual union all
select 52130,'1980-Y223','1980-05-01 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',891,'KORO',90 from dual union all
select 52131,'1980-Y224','1980-05-01 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',81 from dual union all
select 52132,'1980-Y225','1980-05-01 23:27:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 52133,'1980-Y226','1980-05-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',86 from dual union all
select 52134,'1980-Y227','1980-05-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',82 from dual union all
select 52135,'1980-Y228','1980-05-05 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',82 from dual union all
select 52136,'1980-Y229','1980-05-06 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',891,'KORO',94 from dual union all
select 52137,'1980-Y23','1980-01-11 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 52138,'1980-Y230','1980-05-06 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',82 from dual union all
select 52139,'1980-Y231','1980-05-06 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',84 from dual union all
select 5214,'1958-S230','1958-10-16 04:00:00','suborbital rocket','success',726,'','','','','Aeron',636,'',85 from dual union all
select 52140,'1980-Y232','1980-05-06 22:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',82 from dual union all
select 52141,'1980-Y233','1980-05-07 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 52142,'1980-Y234','1980-05-07 11:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 52143,'1980-Y235','1980-05-07 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',891,'KORO',89 from dual union all
select 52144,'1980-Y236','1980-05-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 52145,'1980-Y237','1980-05-07 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 52146,'1980-Y238','1980-05-07 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 52147,'1980-Y239','1980-05-07 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 52148,'1980-Y24','1980-01-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 52149,'1980-Y240','1980-05-07 20:58:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 5215,'1958-S231','1958-10-17 04:05:00','suborbital rocket','success',726,'','','','','Aeron',636,'',85 from dual union all
select 52150,'1980-Y241','1980-05-07 21:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',84 from dual union all
select 52151,'1980-Y242','1980-05-08 22:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',83 from dual union all
select 52152,'1980-Y243','1980-05-09 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',89 from dual union all
select 52153,'1980-Y244','1980-05-09 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',90 from dual union all
select 52154,'1980-Y245','1980-05-09 22:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',83 from dual union all
select 52155,'1980-Y246','1980-05-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 52156,'1980-Y247','1980-05-12 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 52157,'1980-Y248','1980-05-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 52158,'1980-Y249','1980-05-14 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 52159,'1980-Y25','1980-01-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 5216,'1958-S232','1958-10-18 03:25:06','suborbital rocket','success',839,'AA6.205','','IGY AA6.205','','Meteorites',428,'',143 from dual union all
select 52160,'1980-Y250','1980-05-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 52161,'1980-Y251','1980-05-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52162,'1980-Y252','1980-05-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',91 from dual union all
select 52163,'1980-Y253','1980-05-14 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 52164,'1980-Y254','1980-05-14 18:20:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 52165,'1980-Y255','1980-05-14 18:54:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 52166,'1980-Y256','1980-05-16 00:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',82 from dual union all
select 52167,'1980-Y257','1980-05-16 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',89 from dual union all
select 52168,'1980-Y258','1980-05-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',89 from dual union all
select 52169,'1980-Y259','1980-05-19 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 5217,'1958-S233','1958-10-20 22:01:01','suborbital rocket','success',839,'ABM6.207','Ionospheric Te','IGY ABM6.207','','Aeron/Ionos',428,'',130 from dual union all
select 52170,'1980-Y26','1980-01-14 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 52171,'1980-Y260','1980-05-19 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',86 from dual union all
select 52172,'1980-Y261','1980-05-20 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',891,'KORO',91 from dual union all
select 52173,'1980-Y262','1980-05-20 13:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52174,'1980-Y263','1980-05-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',80 from dual union all
select 52175,'1980-Y264','1980-05-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',891,'KORO',91 from dual union all
select 52176,'1980-Y265','1980-05-20 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',85 from dual union all
select 52177,'1980-Y266','1980-05-21 02:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 52178,'1980-Y267','1980-05-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52179,'1980-Y268','1980-05-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',88 from dual union all
select 5218,'1958-S234','1958-10-22 03:45:00','suborbital rocket','success',839,'AA6.206','','IGY AA6.206','','Meteorites',428,'',158 from dual union all
select 52180,'1980-Y269','1980-05-21 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 52181,'1980-Y27','1980-01-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 52182,'1980-Y270','1980-05-21 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 52183,'1980-Y271','1980-05-21 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',85 from dual union all
select 52184,'1980-Y272','1980-05-22 22:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',82 from dual union all
select 52185,'1980-Y273','1980-05-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',90 from dual union all
select 52186,'1980-Y274','1980-05-24 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',82 from dual union all
select 52187,'1980-Y275','1980-05-25 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',82 from dual union all
select 52188,'1980-Y276','1980-05-26 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',891,'KORO',85 from dual union all
select 52189,'1980-Y277','1980-05-26 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',891,'KORO',84 from dual union all
select 5219,'1958-S235','1958-10-22 14:22:00','suborbital rocket','failure',838,'II8.11','','IGY II8.11','','Test/Meteorites',1895,'',10 from dual union all
select 52190,'1980-Y278','1980-05-26 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',83 from dual union all
select 52191,'1980-Y279','1980-05-26 20:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52192,'1980-Y28','1980-01-14 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 52193,'1980-Y280','1980-05-27 12:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',86 from dual union all
select 52194,'1980-Y281','1980-05-27 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',84 from dual union all
select 52195,'1980-Y282','1980-05-28 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',891,'KORO',82 from dual union all
select 52196,'1980-Y283','1980-05-28 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',891,'KORO',88 from dual union all
select 52197,'1980-Y284','1980-05-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 52198,'1980-Y285','1980-05-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 52199,'1980-Y286','1980-05-28 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 522,'1945-M02','1945-10-11 00:00:00','suborbital rocket','success',1516,'5','','','','Test',1852,'',72 from dual union all
select 5220,'1958-S236','1958-10-22 16:47:18','suborbital rocket','success',42,'AA10.01','','IGY AA10.01','','Meteorites',428,'',177 from dual union all
select 52200,'1980-Y287','1980-05-28 19:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 52201,'1980-Y288','1980-05-29 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 52202,'1980-Y289','1980-05-30 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52203,'1980-Y29','1980-01-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 52204,'1980-Y290','1980-05-30 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',83 from dual union all
select 52205,'1980-Y291','1980-06-01 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',84 from dual union all
select 52206,'1980-Y292','1980-06-02 13:43:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 52207,'1980-Y293','1980-06-03 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',891,'KORO',84 from dual;
insert into LAUNCH
select 52208,'1980-Y294','1980-06-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',891,'KORO',81 from dual union all
select 52209,'1980-Y295','1980-06-04 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 5221,'1958-S237','1958-10-23 02:20:00','suborbital rocket','success',726,'','','','','Aeron',1010,'',85 from dual union all
select 52210,'1980-Y296','1980-06-04 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52211,'1980-Y297','1980-06-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 52212,'1980-Y298','1980-06-04 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 52213,'1980-Y299','1980-06-04 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',85 from dual union all
select 52214,'1980-Y30','1980-01-16 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 52215,'1980-Y300','1980-06-05 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',84 from dual union all
select 52216,'1980-Y301','1980-06-05 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 52217,'1980-Y302','1980-06-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 52218,'1980-Y303','1980-06-06 21:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1378,'SHIR',85 from dual union all
select 52219,'1980-Y304','1980-06-11 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 5222,'1958-S238','1958-10-25 19:28:00','suborbital rocket','success',42,'AA10.02','','IGY AA10.02','','Meteorites',428,'',418 from dual union all
select 52220,'1980-Y305','1980-06-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 52221,'1980-Y306','1980-06-11 16:27:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 52222,'1980-Y307','1980-06-11 19:10:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 52223,'1980-Y308','1980-06-11 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 52224,'1980-Y309','1980-06-11 21:22:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 52225,'1980-Y31','1980-01-16 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',84 from dual union all
select 52226,'1980-Y310','1980-06-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 52227,'1980-Y311','1980-06-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 52228,'1980-Y312','1980-06-18 03:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 52229,'1980-Y313','1980-06-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 5223,'1958-S239','1958-10-31 00:00:00','suborbital rocket','success',972,'','First R-11A at OK','','','Aeron?',1010,'',103 from dual union all
select 52230,'1980-Y314','1980-06-18 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 52231,'1980-Y315','1980-06-18 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 52232,'1980-Y316','1980-06-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52233,'1980-Y317','1980-06-20 13:45:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 52234,'1980-Y318','1980-06-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 52235,'1980-Y319','1980-06-24 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 52236,'1980-Y32','1980-01-16 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 52237,'1980-Y320','1980-06-25 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 52238,'1980-Y321','1980-06-25 02:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 52239,'1980-Y322','1980-06-25 13:14:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 5224,'1958-S24','1958-02-16 06:16:38','suborbital rocket','success',839,'II6.25F','Auroral particles','IGY II6.25F','','Auroral',428,'',121 from dual union all
select 52240,'1980-Y323','1980-06-25 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 52241,'1980-Y324','1980-06-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 52242,'1980-Y325','1980-06-27 07:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',85 from dual union all
select 52243,'1980-Y326','1980-06-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 52244,'1980-Y327','1980-07-02 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 52245,'1980-Y328','1980-07-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52246,'1980-Y329','1980-07-02 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 52247,'1980-Y33','1980-01-17 15:24:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52248,'1980-Y330','1980-07-02 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 52249,'1980-Y331','1980-07-04 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 5225,'1958-S240','1958-11-24 00:00:00','military missile','success',976,'B1-15','K63 SI-7','','','Test',636,'',402 from dual union all
select 52250,'1980-Y332','1980-07-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 52251,'1980-Y333','1980-07-09 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 52252,'1980-Y334','1980-07-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 52253,'1980-Y335','1980-07-09 14:03:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52254,'1980-Y336','1980-07-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 52255,'1980-Y337','1980-07-09 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 52256,'1980-Y338','1980-07-09 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 52257,'1980-Y339','1980-07-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 52258,'1980-Y34','1980-01-18 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 52259,'1980-Y340','1980-07-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 5226,'1958-S241','1958-10-31 11:54:00','suborbital rocket','success',1011,'A1-11','R-5A Geophysical 4','IR atmos','','Iono-solar/Bio',636,'',452 from dual union all
select 52260,'1980-Y341','1980-07-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 52261,'1980-Y342','1980-07-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 52262,'1980-Y343','1980-07-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52263,'1980-Y344','1980-07-16 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 52264,'1980-Y345','1980-07-16 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 52265,'1980-Y346','1980-07-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 52266,'1980-Y347','1980-07-18 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 52267,'1980-Y348','1980-07-23 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 52268,'1980-Y349','1980-07-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 52269,'1980-Y35','1980-01-18 10:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',80 from dual union all
select 5227,'1958-S242','1958-10-31 19:59:14','suborbital rocket','success',49,'NN3.15F','','IGY NN3.15F','','Aeron',428,'',188 from dual union all
select 52270,'1980-Y350','1980-07-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 52271,'1980-Y351','1980-07-23 14:40:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',87 from dual union all
select 52272,'1980-Y352','1980-07-23 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 52273,'1980-Y353','1980-07-23 18:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 52274,'1980-Y354','1980-07-23 18:56:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 52275,'1980-Y355','1980-07-23 20:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 52276,'1980-Y356','1980-07-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 52277,'1980-Y357','1980-07-28 09:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',84 from dual union all
select 52278,'1980-Y358','1980-07-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52279,'1980-Y359','1980-07-30 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 5228,'1958-S243','1958-10-31 20:36:19','suborbital rocket','success',839,'OB6.11','Arctic cloud cover','IGY OB6.11','','Meteo/Imaging',428,'',119 from dual union all
select 52280,'1980-Y36','1980-01-18 13:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',81 from dual union all
select 52281,'1980-Y360','1980-07-30 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 52282,'1980-Y361','1980-07-30 17:35:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 52283,'1980-Y362','1980-07-30 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 52284,'1980-Y363','1980-07-30 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 52285,'1980-Y364','1980-08-06 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 52286,'1980-Y365','1980-08-06 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 52287,'1980-Y366','1980-08-06 13:56:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52288,'1980-Y367','1980-08-06 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 52289,'1980-Y368','1980-08-06 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 5229,'1958-S244','1958-11-01 00:00:00','military missile','success',1013,'B9-20TS','8K51 UBP-14','GCh 4R','','Nuclear Test',636,'',500 from dual union all
select 52290,'1980-Y369','1980-08-13 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 52291,'1980-Y37','1980-01-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52292,'1980-Y370','1980-08-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 52293,'1980-Y371','1980-08-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 52294,'1980-Y372','1980-08-13 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 52295,'1980-Y373','1980-08-13 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 52296,'1980-Y374','1980-08-13 20:58:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 52297,'1980-Y375','1980-08-13 23:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',80 from dual union all
select 52298,'1980-Y376','1980-08-14 03:11:00','suborbital rocket','success',1281,'NASA 15.210UE','','','','Plasma',1801,'',94 from dual;
insert into LAUNCH
select 52299,'1980-Y377','1980-08-16 11:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 523,'1945-M03','1945-10-12 00:00:00','suborbital rocket','success',1516,'6','','','','Test',1852,'',72 from dual union all
select 5230,'1958-S245','1958-11-03 00:00:00','military missile','success',1013,'','Nuclear test','','','Nuclear Test',636,'',500 from dual union all
select 52300,'1980-Y378','1980-08-19 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 52301,'1980-Y379','1980-08-20 02:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 52302,'1980-Y38','1980-01-21 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 52303,'1980-Y380','1980-08-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 52304,'1980-Y381','1980-08-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 52305,'1980-Y382','1980-08-20 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52306,'1980-Y383','1980-08-25 13:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',85 from dual union all
select 52307,'1980-Y384','1980-08-26 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 52308,'1980-Y385','1980-08-26 21:20:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',82 from dual union all
select 52309,'1980-Y386','1980-08-27 02:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 5231,'1958-S246','1958-11-03 00:00:00','military missile','success',1013,'M1-2 8K51D','8K51D Fakel Target 1','GCh Fakel','','Target',321,'',500 from dual union all
select 52310,'1980-Y387','1980-08-27 03:20:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',86 from dual union all
select 52311,'1980-Y388','1980-08-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 52312,'1980-Y389','1980-08-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 52313,'1980-Y39','1980-01-21 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',84 from dual union all
select 52314,'1980-Y390','1980-08-27 17:55:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 52315,'1980-Y391','1980-09-03 05:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 52316,'1980-Y392','1980-09-03 12:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52317,'1980-Y393','1980-09-03 13:52:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52318,'1980-Y394','1980-09-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 52319,'1980-Y395','1980-09-03 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 5232,'1958-S247','1958-11-03 16:20:05','suborbital rocket','success',49,'NN3.20F','Composition','IGY NN3.20F','','Aeron/Ionos',428,'',211 from dual union all
select 52320,'1980-Y396','1980-09-03 17:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',81 from dual union all
select 52321,'1980-Y397','1980-09-03 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 52322,'1980-Y398','1980-09-03 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 52323,'1980-Y399','1980-09-09 21:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 52324,'1980-Y40','1980-01-21 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',84 from dual union all
select 52325,'1980-Y400','1980-09-10 02:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 52326,'1980-Y401','1980-09-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 52327,'1980-Y402','1980-09-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 52328,'1980-Y403','1980-09-10 14:04:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',86 from dual union all
select 52329,'1980-Y404','1980-09-10 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 5233,'1958-S249','1958-11-04 16:50:24','suborbital rocket','success',839,'OB6.12','Arctic cloud cover','IGY OB6.12','','Meteo/Imaging',428,'',122 from dual union all
select 52330,'1980-Y405','1980-09-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 52331,'1980-Y406','1980-09-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 52332,'1980-Y407','1980-09-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 52333,'1980-Y408','1980-09-17 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52334,'1980-Y409','1980-09-17 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 52335,'1980-Y41','1980-01-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 52336,'1980-Y410','1980-09-17 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',86 from dual union all
select 52337,'1980-Y411','1980-09-17 17:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 52338,'1980-Y412','1980-09-19 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 52339,'1980-Y413','1980-09-24 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 5234,'1958-S25','1958-02-18 17:21:00','suborbital rocket','success',29,'NRL 51','Aerobee Jr','','','Test/Imaging',1895,'',90 from dual union all
select 52340,'1980-Y414','1980-09-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 52341,'1980-Y415','1980-09-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52342,'1980-Y416','1980-09-24 16:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 52343,'1980-Y417','1980-09-24 19:07:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 52344,'1980-Y418','1980-10-01 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 52345,'1980-Y419','1980-10-01 13:44:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',86 from dual union all
select 52346,'1980-Y42','1980-01-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 52347,'1980-Y420','1980-10-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 52348,'1980-Y421','1980-10-01 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 52349,'1980-Y422','1980-10-01 18:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 5235,'1958-S251','1958-11-05 08:53:00','military missile','success',1367,'138','R&D Series IV','','','Test',234,'',520 from dual union all
select 52350,'1980-Y423','1980-10-06 23:27:00','suborbital rocket','success',1284,'TY1-6191','','','','Aeron',29,'',84 from dual union all
select 52351,'1980-Y424','1980-10-07 06:01:52','suborbital rocket','success',1284,'TY1-6192','','','','Aeron',29,'',80 from dual union all
select 52352,'1980-Y425','1980-10-08 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 52353,'1980-Y426','1980-10-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 52354,'1980-Y427','1980-10-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 52355,'1980-Y428','1980-10-08 16:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 52356,'1980-Y429','1980-10-09 01:15:00','suborbital rocket','success',1284,'TY1-6193','','','','Aeron',29,'',80 from dual union all
select 52357,'1980-Y43','1980-01-21 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',96 from dual union all
select 52358,'1980-Y430','1980-10-09 02:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 52359,'1980-Y431','1980-10-09 22:02:00','suborbital rocket','success',1284,'TY1-6194','','','','Aeron',29,'',85 from dual union all
select 5236,'1958-S252','1958-11-06 00:00:00','suborbital rocket','success',972,'','','','','Aeron?',1010,'',103 from dual union all
select 52360,'1980-Y432','1980-10-11 22:31:00','suborbital rocket','success',1284,'TY1-6197','','','','Aeron',29,'',81 from dual union all
select 52361,'1980-Y433','1980-10-12 00:26:00','suborbital rocket','success',1284,'TY1-6422','','','','Aeron',29,'',82 from dual union all
select 52362,'1980-Y434','1980-10-15 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 52363,'1980-Y435','1980-10-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 52364,'1980-Y436','1980-10-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 52365,'1980-Y437','1980-10-15 17:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 52366,'1980-Y438','1980-10-15 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 52367,'1980-Y439','1980-10-15 23:04:00','suborbital rocket','success',1284,'TY1-6425','','','','Aeron',29,'',80 from dual union all
select 52368,'1980-Y44','1980-01-22 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',92 from dual union all
select 52369,'1980-Y440','1980-10-16 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',89 from dual union all
select 5237,'1958-S253','1958-11-06 00:43:00','military missile','success',1053,'CC-57    SI','','','','Test',280,'',90 from dual union all
select 52370,'1980-Y441','1980-10-18 03:17:35','suborbital rocket','success',1284,'TY1-6874','','','','Aeron',29,'',84 from dual union all
select 52371,'1980-Y442','1980-10-22 02:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 52372,'1980-Y443','1980-10-22 13:52:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52373,'1980-Y444','1980-10-22 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 52374,'1980-Y445','1980-10-22 17:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',86 from dual union all
select 52375,'1980-Y446','1980-10-22 20:43:00','suborbital rocket','success',1281,'A14.021-1','SPE-1A','','','Plasma',1047,'',90 from dual union all
select 52376,'1980-Y447','1980-10-23 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 52377,'1980-Y448','1980-10-24 19:02:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 52378,'1980-Y449','1980-10-29 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 52379,'1980-Y45','1980-01-22 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 5238,'1958-S254','1958-11-08 01:53:19','suborbital rocket','success',839,'CC6.01','IR airglow','IGY CC6.01','','Aeron',428,'',139 from dual union all
select 52380,'1980-Y450','1980-10-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 52381,'1980-Y451','1980-10-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 52382,'1980-Y452','1980-10-29 19:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 52383,'1980-Y453','1980-10-30 04:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 52384,'1980-Y454','1980-10-31 00:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 52385,'1980-Y455','1980-11-04 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 52386,'1980-Y456','1980-11-04 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 52387,'1980-Y457','1980-11-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 52388,'1980-Y458','1980-11-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual;
insert into LAUNCH
select 52389,'1980-Y459','1980-11-05 15:02:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 5239,'1958-S255','1958-11-13 08:59:05','suborbital rocket','success',839,'CC6.02','IR airglow','IGY CC6.02','','Aeron',428,'',142 from dual union all
select 52390,'1980-Y46','1980-01-23 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',85 from dual union all
select 52391,'1980-Y460','1980-11-05 19:14:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',82 from dual union all
select 52392,'1980-Y461','1980-11-11 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 52393,'1980-Y462','1980-11-12 14:44:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52394,'1980-Y463','1980-11-12 14:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 52395,'1980-Y464','1980-11-12 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 52396,'1980-Y465','1980-11-13 20:37:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 52397,'1980-Y466','1980-11-18 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 52398,'1980-Y467','1980-11-19 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 52399,'1980-Y468','1980-11-19 13:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 524,'1945-M04','1945-10-19 00:00:00','suborbital rocket','success',1516,'8','','','','Test',1852,'',72 from dual union all
select 5240,'1958-S256','1958-11-13 17:34:06','suborbital rocket','success',839,'OB6.13','Arctic cloud cover','IGY OB6.13F','','Meteo/Imaging',428,'',119 from dual union all
select 52400,'1980-Y469','1980-11-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 52401,'1980-Y47','1980-01-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 52402,'1980-Y470','1980-11-19 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 52403,'1980-Y471','1980-11-19 20:15:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52404,'1980-Y472','1980-11-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 52405,'1980-Y473','1980-11-21 08:20:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',82 from dual union all
select 52406,'1980-Y474','1980-11-25 09:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',86 from dual union all
select 52407,'1980-Y475','1980-11-25 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',89 from dual union all
select 52408,'1980-Y476','1980-11-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 52409,'1980-Y477','1980-11-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 5241,'1958-S257','1958-11-16 06:56:53','suborbital rocket','success',49,'NN3.06F','Auroral particles','IGY NN3.06F','','Auroral/Fields',428,'',163 from dual union all
select 52410,'1980-Y478','1980-11-26 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 52411,'1980-Y479','1980-11-26 19:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 52412,'1980-Y48','1980-01-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 52413,'1980-Y480','1980-12-03 02:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',88 from dual union all
select 52414,'1980-Y481','1980-12-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 52415,'1980-Y482','1980-12-03 14:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 52416,'1980-Y483','1980-12-03 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 52417,'1980-Y484','1980-12-05 09:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',80 from dual union all
select 52418,'1980-Y485','1980-12-09 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52419,'1980-Y486','1980-12-10 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 5242,'1958-S258','1958-11-17 00:00:00','test rocket','success',253,'Missile 5','','','','Test',293,'B-47',100 from dual union all
select 52420,'1980-Y487','1980-12-10 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 52421,'1980-Y488','1980-12-10 18:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 52422,'1980-Y489','1980-12-11 16:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 52423,'1980-Y49','1980-01-23 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 52424,'1980-Y490','1980-12-12 10:20:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',96 from dual union all
select 52425,'1980-Y491','1980-12-12 11:17:00','suborbital rocket','success',1281,'NASA 15.189UE','','','','Plasma',1407,'',80 from dual union all
select 52426,'1980-Y492','1980-12-12 19:27:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 52427,'1980-Y493','1980-12-16 21:30:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',92 from dual union all
select 52428,'1980-Y494','1980-12-17 02:10:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',91 from dual union all
select 52429,'1980-Y495','1980-12-17 05:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 5243,'1958-S259','1958-11-18 04:00:00','military missile','success',147,'9B','Mk 2-1 RV','R&D','','Test',226,'',800 from dual union all
select 52430,'1980-Y496','1980-12-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 52431,'1980-Y497','1980-12-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 52432,'1980-Y498','1980-12-17 14:35:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',83 from dual union all
select 52433,'1980-Y499','1980-12-18 14:48:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 52434,'1980-Y50','1980-01-25 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 52435,'1980-Y500','1980-12-20 17:30:00','suborbital rocket','success',1281,'NASA 15.190UE','','','','Plasma',1407,'',80 from dual union all
select 52436,'1980-Y501','1980-12-23 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',90 from dual union all
select 52437,'1980-Y502','1980-12-24 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 52438,'1980-Y503','1980-12-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',93 from dual union all
select 52439,'1980-Y504','1980-12-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 5244,'1958-S26','1958-02-20 17:46:00','military missile','failure',142,'11A','R&D Fairing','R&D','','Test',227,'',90 from dual union all
select 52440,'1980-Y505','1980-12-24 17:52:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 52441,'1980-Y506','1980-12-24 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 52442,'1980-Y507','1980-12-30 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 52443,'1980-Y508','1980-12-30 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 52444,'1980-Y509','1980-12-31 01:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 52445,'1980-Y51','1980-01-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 52446,'1980-Y510','1980-12-31 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 52447,'1980-Y511','1980-12-31 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 52448,'1980-Y512','1980-09-24 09:30:00','suborbital rocket','success',725,'Ozone M-08-554','ISRO-8.554','','','Aeron',1467,'',81 from dual union all
select 52449,'1980-Y513','1980-09-24 21:30:00','suborbital rocket','success',725,'Ozone M-08-556','ISRO-8.556','','','Aeron',1467,'',81 from dual union all
select 5245,'1958-S260','1958-11-18 16:46:51','suborbital rocket','success',839,'OB6.14','Arctic cloud cover','IGY OB6.14F','','Meteo/Imaging',428,'',116 from dual union all
select 52450,'1980-Y514','1980-11-07 22:00:00','suborbital rocket','success',1284,'','EBC-C E9B','','','Aeron',409,'',85 from dual union all
select 52451,'1980-Y515','1980-11-07 22:50:00','suborbital rocket','success',1284,'PWN-12A','EBC-C E9A','','','Aeron',409,'',104 from dual union all
select 52452,'1980-Y516','1980-11-10 00:10:00','suborbital rocket','success',1284,'','EBC-C E9B','','','Aeron',409,'',85 from dual union all
select 52453,'1980-Y517','1980-11-10 01:00:00','suborbital rocket','success',1284,'','EBC-C E9A','','','Aeron',409,'',104 from dual union all
select 52454,'1980-Y518','1980-11-10 02:18:00','suborbital rocket','success',1284,'','EBC-C E9A','','','Aeron',409,'',104 from dual union all
select 52455,'1980-Y519','1980-11-10 22:00:00','suborbital rocket','success',1284,'','EBC-C E9B','','','Aeron',409,'',85 from dual union all
select 52456,'1980-Y52','1980-01-25 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 52457,'1980-Y520','1980-11-10 23:46:00','suborbital rocket','success',1284,'','EBC-C E9A','','','Aeron',409,'',104 from dual union all
select 52458,'1980-Y521','1980-11-11 01:01:30','suborbital rocket','success',1284,'','EBC-C E9A','','','Aeron',409,'',104 from dual union all
select 52459,'1980-Y522','1980-11-11 01:55:30','suborbital rocket','success',1284,'','EBC-C E9A','','','Aeron',409,'',104 from dual union all
select 5246,'1958-S261','1958-11-19 00:00:00','suborbital rocket','success',972,'','','','','Aeron?',1010,'',103 from dual union all
select 52460,'1980-Y524','1980-11-11 02:26:30','suborbital rocket','success',1284,'','EBC-C E9B','','','Aeron',409,'',85 from dual union all
select 52461,'1980-Y525','1980-11-12 00:20:30','suborbital rocket','success',1284,'','EBC-C E9B','','','Aeron',409,'',85 from dual union all
select 52462,'1980-Y526','1980-11-12 01:07:00','suborbital rocket','success',1284,'','EBC-C E9A','','','Aeron',409,'',104 from dual union all
select 52463,'1980-Y527','1980-11-15 22:35:30','suborbital rocket','success',1284,'','EBC-B E9B','','','Aeron',409,'',85 from dual union all
select 52464,'1980-Y528','1980-11-16 05:12:30','suborbital rocket','success',1284,'','EBC-B E9A','','','Aeron',409,'',104 from dual union all
select 52465,'1980-Y529','1980-11-16 06:33:00','suborbital rocket','success',1284,'','EBC-B E9B','','','Aeron',409,'',85 from dual union all
select 52466,'1980-Y53','1980-01-28 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 52467,'1980-Y530','1980-11-16 07:22:00','suborbital rocket','success',1284,'','EBC-B E9A','','','Aeron',409,'',104 from dual union all
select 52468,'1980-Y531','1980-11-16 07:51:30','suborbital rocket','success',1284,'','EBC-B E9A','','','Aeron',409,'',104 from dual union all
select 52469,'1980-Y532','1980-11-16 08:23:00','suborbital rocket','success',1284,'','EBC-B E9B','','','Aeron',409,'',85 from dual union all
select 5247,'1958-S262','1958-11-19 21:50:54','suborbital rocket','success',839,'OB6.06','B/ne probe','IGY OB6.06F','','Fields',428,'',140 from dual union all
select 52470,'1980-Y533','1980-11-27 22:45:30','suborbital rocket','success',1284,'','EBC-A E9B','','','Aeron',409,'',85 from dual union all
select 52471,'1980-Y534','1980-11-28 00:47:00','suborbital rocket','success',1284,'','EBC-A E9A','','','Aeron',409,'',104 from dual union all
select 52472,'1980-Y535','1980-11-28 03:29:00','suborbital rocket','success',1284,'','EBC-A E9A','','','Aeron',409,'',104 from dual union all
select 52473,'1980-Y536','1980-11-28 04:19:10','suborbital rocket','success',1284,'','EBC-A E9B','','','Aeron',409,'',85 from dual union all
select 52474,'1980-Y537','1980-12-01 00:24:00','suborbital rocket','success',1284,'','EBC-A2 E9A','','','Aeron',409,'',104 from dual union all
select 52475,'1980-Y538','1980-12-01 01:39:00','suborbital rocket','success',1284,'','EBC-A2 E9A','','','Aeron',409,'',104 from dual union all
select 52476,'1980-Y539','1980-12-01 02:33:30','suborbital rocket','success',1284,'','EBC-A2 E9B','','','Aeron',409,'',85 from dual union all
select 52477,'1980-Y54','1980-01-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 52478,'1980-Y540','1980-12-01 03:24:00','suborbital rocket','success',1284,'','EBC-A2 E9B','','','Aeron',409,'',85 from dual;
insert into LAUNCH
select 52479,'1980-Y55','1980-01-28 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',95 from dual union all
select 5248,'1958-S263','1958-11-20 08:18:00','suborbital rocket','success',366,'','RW-106','','','Aeron',1490,'',83 from dual union all
select 52480,'1980-Y56','1980-01-28 15:24:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',89 from dual union all
select 52481,'1980-Y57','1980-01-30 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 52482,'1980-Y58','1980-01-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 52483,'1980-Y59','1980-01-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 52484,'1980-Y60','1980-01-30 15:14:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',86 from dual union all
select 52485,'1980-Y61','1980-01-30 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 52486,'1980-Y62','1980-02-01 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 52487,'1980-Y63','1980-02-01 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 52488,'1980-Y64','1980-02-04 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 52489,'1980-Y65','1980-02-04 19:53:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',89 from dual union all
select 5249,'1958-S264','1958-11-20 14:10:00','suborbital rocket','success',366,'','RW-107','','','Aeron',1490,'',84 from dual union all
select 52490,'1980-Y66','1980-02-05 01:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 52491,'1980-Y67','1980-02-06 02:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 52492,'1980-Y68','1980-02-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 52493,'1980-Y69','1980-02-06 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',80 from dual union all
select 52494,'1980-Y70','1980-02-06 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',86 from dual union all
select 52495,'1980-Y71','1980-02-06 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 52496,'1980-Y72','1980-02-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 52497,'1980-Y73','1980-02-08 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 52498,'1980-Y74','1980-02-09 01:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 52499,'1980-Y75','1980-02-11 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 525,'1945-S01','1945-01-01 00:00:00','military missile','success',1470,'','','','','Weapon',1734,'',90 from dual union all
select 5250,'1958-S265','1958-11-21 06:08:00','suborbital rocket','success',1013,'B1-1 8K52','8K52 Zvezda-1','Zvezda radio guidance','','Ionos',636,'',460 from dual union all
select 52500,'1980-Y76','1980-02-13 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 52501,'1980-Y77','1980-02-13 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 52502,'1980-Y78','1980-02-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 52503,'1980-Y79','1980-02-13 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 52504,'1980-Y80','1980-02-13 18:45:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 52505,'1980-Y81','1980-02-14 23:11:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 52506,'1980-Y82','1980-02-15 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 52507,'1980-Y83','1980-02-15 08:25:00','suborbital rocket','success',1281,'NASA 15.200UE','','','','Plasma',1412,'',88 from dual union all
select 52508,'1980-Y84','1980-02-15 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 52509,'1980-Y85','1980-02-16 07:23:00','suborbital rocket','success',725,'ISRO 8.506','Met','','','Meteo/Eclipse',1467,'',84 from dual union all
select 5251,'1958-S266','1958-11-22 09:57:59','suborbital rocket','success',839,'SS6.56','Grenades','IGY SS6.56F','','Aeron',637,'',85 from dual union all
select 52510,'1980-Y86','1980-02-16 08:25:00','suborbital rocket','success',1281,'NASA 15.201UE','','','','Plasma',1412,'',84 from dual union all
select 52511,'1980-Y87','1980-02-16 11:05:00','suborbital rocket','success',725,'ISRO 8.507','Met','','','Meteo/Eclipse',1467,'',86 from dual union all
select 52512,'1980-Y88','1980-02-20 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 52513,'1980-Y89','1980-02-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 52514,'1980-Y90','1980-02-20 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 52515,'1980-Y91','1980-02-20 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',88 from dual union all
select 52516,'1980-Y92','1980-02-20 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 52517,'1980-Y93','1980-02-22 02:25:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 52518,'1980-Y94','1980-02-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',87 from dual union all
select 52519,'1980-Y95','1980-02-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',81 from dual union all
select 5252,'1958-S267','1958-11-23 22:02:00','suborbital rocket','success',839,'AA6.16','Atm. p,rho, Sphere','IGY AA6.16','','Aeron',428,'',131 from dual union all
select 52520,'1980-Y96','1980-02-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',83 from dual union all
select 52521,'1980-Y97','1980-02-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',83 from dual union all
select 52522,'1980-Y98','1980-02-25 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',890,'KORO',86 from dual union all
select 52523,'1980-Y99','1980-02-25 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1377,'SHIR',83 from dual union all
select 52524,'1981-001','1981-01-06 12:15:00','orbital','success',1238,'77023  -474','Kosmos-1237','Zenit-6','','Sat',473,'',null from dual union all
select 52525,'1981-002','1981-01-09 14:57:00','orbital','success',809,'77023  -617','Molniya-3','Molniya-3 No. 25','','Sat',473,'',null from dual union all
select 52526,'1981-003','1981-01-16 09:00:00','orbital','success',680,'53798-319','Kosmos-1238','Vektor','','Sat',456,'',null from dual union all
select 52527,'1981-004','1981-01-16 12:00:00','orbital','success',1238,'78017  -821','Kosmos-1239','Orion','','Sat',473,'',null from dual union all
select 52528,'1981-005','1981-01-20 11:00:00','orbital','success',1238,'77023  -975','Kosmos-1240','Feniks No. 975','','Sat',562,'',null from dual union all
select 52529,'1981-006','1981-01-21 08:29:00','orbital','success',680,'65082-427','Kosmos-1241','Lira','','Sat',456,'',null from dual union all
select 5253,'1958-S268','1958-11-24 06:23:34','suborbital rocket','success',49,'NN3.21F','Electron density','IGY NN3.21F','','Aeron/Ionos',428,'',207 from dual union all
select 52530,'1981-007','1981-01-24 14:18:02','orbital','success',1238,'P 15000-235','Progress-12','Progress 7K-TG No. 113','','Sat',480,'',null from dual union all
select 52531,'1981-008','1981-01-27 14:58:00','orbital','success',1510,'78017  -354','Kosmos-1242','Tselina-D','','Sat',474,'',null from dual union all
select 52532,'1981-009','1981-01-30 16:27:00','orbital','success',809,'76038  -510','Molniya-1','Molniya-1','','Sat',474,'',null from dual union all
select 52533,'1981-010','1981-02-02 02:19:00','orbital','success',1446,'45028-201','Kosmos-1243','IS','','Sat',589,'',null from dual union all
select 52534,'1981-011','1981-02-06 08:00:00','orbital','success',680,'53793-478','Interkosmos-21','AUOS-Z R-P-IK','','Sat',456,'',null from dual union all
select 52535,'1981-012','1981-02-11 08:30:00','orbital','success',825,'N-7(F)','Kiku-3','ETS-4','','Sat',1474,'',null from dual union all
select 52536,'1981-013','1981-02-12 18:21:00','orbital','success',680,'65098-320','Kosmos-1244','Parus','','Sat',456,'',null from dual union all
select 52537,'1981-014','1981-02-13 11:15:00','orbital','success',1238,'76053  -478','Kosmos-1245','Zenit-6','','Sat',474,'',null from dual union all
select 52538,'1981-015','1981-02-18 09:00:00','orbital','success',1238,'Ts15000-264','Kosmos-1246','Siluet No. 1','','Sat',562,'',null from dual union all
select 52539,'1981-016','1981-02-19 10:00:26','orbital','success',809,'76047  -549','Kosmos-1247','Oko','','Sat',462,'',null from dual union all
select 5254,'1958-S269','1958-11-26 09:09:00','military missile','success',1367,'140','R&D Series IV','','','Test',234,'',520 from dual union all
select 52540,'1981-017','1981-02-21 00:30:00','orbital','success',819,'M-3S-2','Hinotori','ASTRO-A','','Sat',835,'',null from dual union all
select 52541,'1981-018','1981-02-21 23:23:00','orbital','success',180,'AC-42','Comstar D-4','Comstar D-4','','Sat',268,'',null from dual union all
select 52542,'1981-019','1981-02-28 19:10:00','orbital','success',1403,'24B-24 (3B-59)','KH-8A','GAMBIT-3 Mission 4351','','Sat',1778,'',null from dual union all
select 52543,'1981-020','1981-03-05 15:00:00','orbital','success',1238,'77023  -475 ','Kosmos-1248','Feniks No. 940','','Sat',473,'',null from dual union all
select 52544,'1981-021','1981-03-05 18:09:00','orbital','success',1446,'67052-104','Kosmos-1249','US-A reactor','','Sat',589,'',null from dual union all
select 52545,'1981-022','1981-03-06 11:31:00','orbital','success',680,'53767-247','Kosmos-1250/1257','Strela-1M','','Sat',457,'',null from dual union all
select 52546,'1981-023','1981-03-12 19:00:11','orbital','success',1238,'Ts15000-266','Soyuz T-4','Soyuz 7K-ST No. 10L','','Sat',480,'',null from dual union all
select 52547,'1981-024','1981-03-14 16:55:00','orbital','success',1446,'45048-204','Kosmos-1258','IS','','Sat',589,'',null from dual union all
select 52548,'1981-025','1981-03-16 19:24:00','orbital','success',1425,'23C-22 (3C-40)','DSP 10','DSP 10','','Sat',274,'',null from dual union all
select 52549,'1981-026','1981-03-17 08:40:00','orbital','success',1238,'77023  -468','Kosmos-1259','Zenit-6','','Sat',562,'',null from dual union all
select 5255,'1958-S27','1958-02-21 07:40:00','suborbital rocket','success',1011,'A1-7','R-5A Geophysical 1','','','Ionos/Solar UV/Met',636,'',473 from dual union all
select 52550,'1981-027','1981-03-18 04:40:00','orbital','success',956,'306-01','Raduga','Gran'' No. 18L','','Sat',554,'',null from dual union all
select 52551,'1981-028','1981-03-20 23:45:00','orbital','success',1446,'45035-503','Kosmos-1260','US-P','','Sat',589,'',null from dual union all
select 52552,'1981-029','1981-03-22 14:58:55','orbital','success',1238,'P 15000-257','Soyuz-39','Soyuz 7K-T No. 55','','Sat',562,'',null from dual union all
select 52553,'1981-030','1981-03-24 03:31:00','orbital','success',809,'77025  -525','Molniya-3','Molniya-3 No. 24','','Sat',473,'',null from dual union all
select 52554,'1981-031','1981-03-31 09:39:49','orbital','success',809,'78017  -553','Kosmos-1261','Oko','','Sat',473,'',null from dual union all
select 52555,'1981-032','1981-04-07 10:51:00','orbital','success',1238,'76053  -480','Kosmos-1262','Zenit-6','','Sat',474,'',null from dual union all
select 52556,'1981-033','1981-04-09 12:00:00','orbital','success',680,'53798-322','Kosmos-1263','Vektor','','Sat',456,'',null from dual union all
select 52557,'1981-034','1981-04-12 12:00:04','orbital','success',1243,'STS-1','OFT-1','OV-102 OFT-1','','Sat',921,'MLP1',null from dual union all
select 52558,'1981-035','1981-04-15 10:30:00','orbital','success',1238,'76053  -482','Kosmos-1264','Zenit-6','','Sat',562,'',null from dual union all
select 52559,'1981-036','1981-04-16 11:30:00','orbital','success',1238,'76053  -479','Kosmos-1265','Zenit-6','','Sat',473,'',null from dual union all
select 5256,'1958-S270','1958-11-29 02:27:00','military missile','success',147,'12B','Low Drag Fairing/AFWSC-3','R&D','','Test',229,'',900 from dual union all
select 52560,'1981-037','1981-04-21 03:45:00','orbital','success',1446,'51035-504','Kosmos-1266','US-A reactor','','Sat',589,'',null from dual union all
select 52561,'1981-038','1981-04-24 21:32:00','orbital','success',1405,'34B-8  (3B-60)','OPS 7225','Jumpseat 6','','Sat',1778,'',null from dual union all
select 52562,'1981-039','1981-04-25 02:01:00','orbital','success',949,'299-02','Kosmos-1267','TKS No. 16301','','Sat',553,'',null from dual union all
select 52563,'1981-040','1981-04-28 09:00:00','orbital','success',1238,'77053  -484','Kosmos-1268','Zenit-6','','Sat',562,'',null from dual union all
select 52564,'1981-041','1981-05-07 13:21:00','orbital','success',680,'47198-321','Kosmos-1269','Strela-2M','','Sat',456,'',null from dual union all
select 52565,'1981-042','1981-05-14 17:16:38','orbital','success',1238,'P 15000-246','Soyuz-40','Soyuz 7K-T No. 56','','Sat',480,'',null from dual union all
select 52566,'1981-043','1981-05-14 21:45:00','orbital','success',1510,'77052  -392','Meteor-2','Meteor-2 No. 8','','Sat',474,'',null from dual union all
select 52567,'1981-044','1981-05-15 06:07:17','orbital','success',1135,'S192C','Nova 1','Nova 1','','Sat',1779,'',null from dual union all
select 52568,'1981-045','1981-05-18 11:50:00','orbital','success',1238,'76053  -485','Kosmos-1270','Feniks No. 980','','Sat',562,'',null from dual;
insert into LAUNCH
select 52569,'1981-046','1981-05-19 03:49:00','orbital','success',1510,'78017  -355','Kosmos-1271','Tselina-D','','Sat',474,'',null from dual union all
select 5257,'1958-S271','1958-11-29 16:30:30','suborbital rocket','success',49,'NN3.16F','Auroral/UV Phot','IGY NN3.16F','','Auroral',428,'',202 from dual union all
select 52570,'1981-047','1981-05-21 09:10:00','orbital','success',1238,'78039  -418','Kosmos-1272','Zenit-6','','Sat',562,'',null from dual union all
select 52571,'1981-048','1981-05-22 07:10:00','orbital','success',1238,'76053  -077','Kosmos-1273','Fram','','Sat',473,'',null from dual union all
select 52572,'1981-049','1981-05-22 22:29:00','orbital','success',390,'645/D154','GOES 5','GOES E','','Sat',233,'',null from dual union all
select 52573,'1981-050','1981-05-23 22:42:00','orbital','success',180,'AC-56','Intelsat 501','INTELSAT V F1','','Sat',269,'',null from dual union all
select 52574,'1981-051','1981-05-31 05:00:00','orbital','success',1084,'SLV-3-D3','RS-D-1','Rohini RS-D-1','','Sat',1360,'',null from dual union all
select 52575,'1981-052','1981-06-03 14:00:00','orbital','success',1238,'76053  -483','Kosmos-1274','Feniks No. 942','','Sat',473,'',null from dual union all
select 52576,'1981-053','1981-06-04 15:41:00','orbital','success',680,'65098-323','Kosmos-1275','Parus','','Sat',457,'',null from dual union all
select 52577,'1981-054','1981-06-09 03:33:00','orbital','success',809,'76053  -622','Molniya-3','Molniya-3 No. 30','','Sat',473,'',null from dual union all
select 52578,'1981-055','1981-06-16 07:00:00','orbital','success',1238,'78039  -078','Kosmos-1276','Fram','','Sat',474,'',null from dual union all
select 52579,'1981-056','1981-06-17 09:30:00','orbital','success',1238,'78039  -417','Kosmos-1277','Zenit-6','','Sat',562,'',null from dual union all
select 5258,'1958-S272','1958-12-04 00:00:00','military missile','success',976,'B1-13','K63 SI-8','','','Test',636,'',402 from dual union all
select 52580,'1981-057','1981-06-19 12:32:59','orbital','success',91,'L03','Meteosat 2/Apple','Meteosat F2','','Sat',344,'',null from dual union all
select 52581,'1981-058','1981-06-19 19:37:04','orbital','success',809,'76047  -554','Kosmos-1278','Oko','','Sat',474,'',null from dual union all
select 52582,'1981-059','1981-06-23 10:52:59','orbital','success',162,'87F','NOAA 7','NOAA C','','Sat',1776,'',null from dual union all
select 52583,'1981-060','1981-06-24 17:47:00','orbital','success',809,'76038  -607','Molniya-1','Molniya-1','','Sat',474,'',null from dual union all
select 52584,'1981-061','1981-06-25 23:55:00','orbital','success',956,'305-01','Ekran','Ekran No. 21L','','Sat',554,'',null from dual union all
select 52585,'1981-062','1981-07-01 09:30:00','orbital','success',1238,'78039  -422','Kosmos-1279','Zenit-6','','Sat',562,'',null from dual union all
select 52586,'1981-063','1981-07-02 07:10:00','orbital','success',1238,'78026  -663','Kosmos-1280','Resurs-F1 17F41 No. 15','','Sat',474,'',null from dual union all
select 52587,'1981-064','1981-07-07 12:30:00','orbital','success',1238,'76053  -477','Kosmos-1281','Zenit-6','','Sat',474,'',null from dual union all
select 52588,'1981-065','1981-07-10 05:14:00','orbital','success',1510,'76042  -389','Meteor-Priroda','Meteor-Priroda No. 2-4','','Sat',562,'',null from dual union all
select 52589,'1981-066','1981-07-15 13:00:00','orbital','success',1238,'78017  -951','Kosmos-1282','Feniks No. 951','','Sat',562,'',null from dual union all
select 5259,'1958-S273','1958-12-10 00:00:00','military missile','success',976,'B1-14','K63 SI-9','','','Test',636,'',402 from dual union all
select 52590,'1981-067','1981-07-17 08:00:00','orbital','success',1238,'76047  -940','Kosmos-1283','Zenit-6','','Sat',473,'',null from dual union all
select 52591,'1981-068','1981-07-29 11:55:00','orbital','success',1238,'78039  -421','Kosmos-1284','Zenit-6','','Sat',473,'',null from dual union all
select 52592,'1981-069','1981-07-30 21:38:00','orbital','success',956,'301-02','Raduga','Gran'' No. 19L','','Sat',553,'',null from dual union all
select 52593,'1981-070','1981-08-03 09:56:00','orbital','success',389,'642/D155','DE 1/DE 2','DE A','','Sat',1724,'',null from dual union all
select 52594,'1981-071','1981-08-04 00:12:58','orbital','success',809,'76047  -522','Kosmos-1285','Oko','','Sat',462,'',null from dual union all
select 52595,'1981-072','1981-08-04 08:28:00','orbital','success',1446,'67028-203','Kosmos-1286','US-PM1','','Sat',589,'',null from dual union all
select 52596,'1981-073','1981-08-06 08:16:00','orbital','success',180,'AC-59','FLTSATCOM 5','FLTSATCOM F5','','Sat',268,'',null from dual union all
select 52597,'1981-074','1981-08-06 11:49:00','orbital','success',680,'65055-112','Kosmos-1287/1294','Strela-1M','','Sat',457,'',null from dual union all
select 52598,'1981-075','1981-08-07 13:35:00','orbital','success',1510,'78039  -375','IK Bulgaria-1300','IK Bulgaria-1300','','Sat',474,'',null from dual union all
select 52599,'1981-076','1981-08-10 20:03:00','orbital','success',825,'N-8(F)','GMS 2','GMS 2','','Sat',1474,'',null from dual union all
select 526,'1945-S02','1945-01-01 00:00:00','military missile','success',1470,'','','','','Weapon',1734,'',90 from dual union all
select 5260,'1958-S274','1958-11-30 18:36:36','suborbital rocket','success',42,'ABM10.200','Te probe','IGY ABM10.200','','Test/Ionos',428,'',304 from dual union all
select 52600,'1981-077','1981-08-12 05:46:00','orbital','success',680,'47198-324','Kosmos-1295','Parus','','Sat',457,'',null from dual union all
select 52601,'1981-078','1981-08-13 16:20:00','orbital','success',1238,'76053  -978','Kosmos-1296','Feniks No. 943','','Sat',473,'',null from dual union all
select 52602,'1981-079','1981-08-18 09:30:00','orbital','success',1238,'76053  -481','Kosmos-1297','Zenit-6','','Sat',473,'',null from dual union all
select 52603,'1981-080','1981-08-21 10:20:00','orbital','success',1238,'78039  -115','Kosmos-1298','Yantar''-4K2','','Sat',562,'',null from dual union all
select 52604,'1981-081','1981-08-24 16:37:00','orbital','success',1446,'51048-205','Kosmos-1299','US-A reactor','','Sat',589,'',null from dual union all
select 52605,'1981-082','1981-08-24 21:40:00','orbital','success',1448,'51099-801','Kosmos-1300','Tselina-D','','Sat',469,'',null from dual union all
select 52606,'1981-083','1981-08-27 10:30:00','orbital','success',1238,'77049  -116','Kosmos-1301','Resurs-F1 17F41 No. 16','','Sat',473,'',null from dual union all
select 52607,'1981-084','1981-08-28 16:18:00','orbital','success',680,'47155-101','Kosmos-1302','Strela-2M','','Sat',456,'',null from dual union all
select 52608,'1981-085','1981-09-03 18:29:00','orbital','success',1426,'23D-22','KH-11','KH11-4 KENNAN','','Sat',1777,'',null from dual union all
select 52609,'1981-086','1981-09-04 08:00:00','orbital','success',1238,'78017  -941','Kosmos-1303','Zenit-6','','Sat',562,'',null from dual union all
select 5261,'1958-S275','1958-11-30 22:27:01','suborbital rocket','success',839,'IGY OB6.15','B/ne probe','IGY OB6.15F','','Fields',428,'',152 from dual union all
select 52610,'1981-087','1981-09-04 11:06:00','orbital','success',680,'53755-108','Kosmos-1304','Tsikada','','Sat',456,'',null from dual union all
select 52611,'1981-088','1981-09-11 08:43:10','orbital','success',809,'78055  -535','Kosmos-1305','Molniya-3 No. 28','','Sat',474,'',null from dual union all
select 52612,'1981-089','1981-09-14 20:31:00','orbital','success',1446,'67055-505','Kosmos-1306','US-PM','','Sat',589,'',null from dual union all
select 52613,'1981-090','1981-09-15 11:30:00','orbital','success',1238,'78039  -416','Kosmos-1307','Zenit-6','','Sat',474,'',null from dual union all
select 52614,'1981-091','1981-09-18 03:34:00','orbital','success',680,'47175-113','Kosmos-1308','Parus','','Sat',456,'',null from dual union all
select 52615,'1981-092','1981-09-18 09:30:00','orbital','success',1238,'77023  -802','Kosmos-1309','Orion','','Sat',474,'',null from dual union all
select 52616,'1981-093','1981-09-19 21:28:40','orbital','success',492,'XCZ-1-02','SJ-2','shijian erhao B xing','','Sat',810,'',null from dual union all
select 52617,'1981-094','1981-09-21 13:10:00','orbital','success',1448,'67099-802','Aureole 3','AUOS-Z 401 M-A-IK','','Sat',469,'',null from dual union all
select 52618,'1981-095','1981-09-23 08:00:00','orbital','success',680,'53719-527','Kosmos-1310','Vektor','','Sat',457,'',null from dual union all
select 52619,'1981-096','1981-09-24 23:09:00','orbital','success',388,'641/D156','SBS 2','SBS 2','','Sat',233,'',null from dual union all
select 5262,'1958-S276','1958-12-01 19:34:40','suborbital rocket','failure',42,'NN10.001','Field/ne probe','IGY NN10.01','','Ionos',428,'',97 from dual union all
select 52620,'1981-097','1981-09-28 21:00:00','orbital','success',680,'53775-114','Kosmos-1311','Romb','','Sat',457,'',null from dual union all
select 52621,'1981-098','1981-09-30 08:00:00','orbital','success',1448,'51099-801','Kosmos-1312','Musson No. 12','','Sat',469,'',null from dual union all
select 52622,'1981-099','1981-10-01 09:00:00','orbital','success',1238,'78039  -427','Kosmos-1313','Zenit-6','','Sat',562,'',null from dual union all
select 52623,'1981-100','1981-10-06 11:27:00','orbital','success',381,'639/D157','SME','Solar Mesosphere Explore','','Sat',1724,'',null from dual union all
select 52624,'1981-101','1981-10-09 10:40:00','orbital','success',1238,'77049  -079','Kosmos-1314','Fram','','Sat',473,'',null from dual union all
select 52625,'1981-102','1981-10-09 16:59:00','orbital','success',956,'310-01','Raduga','Gran'' No. 20L','','Sat',553,'',null from dual union all
select 52626,'1981-103','1981-10-13 23:01:00','orbital','success',1510,'78017  -357','Kosmos-1315','Tselina-D','','Sat',474,'',null from dual union all
select 52627,'1981-104','1981-10-15 09:15:00','orbital','success',1238,'78039  -423','Kosmos-1316','Zenit-6','','Sat',562,'',null from dual union all
select 52628,'1981-105','1981-10-17 05:59:00','orbital','success',809,'78055  -530','Molniya-3','Molniya-3 No. 31','','Sat',473,'',null from dual union all
select 52629,'1981-106','1981-10-30 06:04:22','deep space','success',954,'311-01','Venera-13','4V-1M No. 760','','Sat',554,'',null from dual union all
select 5263,'1958-S277','1958-12-03 10:13:00','suborbital rocket','success',1193,'SL11','QUB Na-1','','','Aeronomy',1839,'',126 from dual union all
select 52630,'1981-107','1981-10-31 09:22:00','orbital','success',1425,'23C-21 (3C-39)','OPS 4029','VORTEX 3','','Sat',274,'',null from dual union all
select 52631,'1981-108','1981-10-31 22:53:31','orbital','success',809,'76047  -555','Kosmos-1317','Oko','','Sat',462,'',null from dual union all
select 52632,'1981-109','1981-11-03 13:00:00','orbital','success',1238,'77023  -977','Kosmos-1318','Feniks No. 944','','Sat',473,'',null from dual union all
select 52633,'1981-110','1981-11-04 05:31:16','deep space','success',954,'311-02','Venera-14','4V-1M No. 761','','Sat',553,'',null from dual union all
select 52634,'1981-111','1981-11-12 15:10:00','orbital','success',1243,'STS-2','OFT-2/OSTA-1','OV-102 OFT-2','','Sat',921,'MLP1',null from dual union all
select 52635,'1981-112','1981-11-13 09:30:00','orbital','success',1238,'78039  -429','Kosmos-1319','Zenit-6','','Sat',562,'',null from dual union all
select 52636,'1981-113','1981-11-17 15:25:00','orbital','success',809,'78026  -601','Molniya-1','Molniya-1','','Sat',473,'',null from dual union all
select 52637,'1981-114','1981-11-20 01:37:00','orbital','success',388,'640/D158','Satcom 3R','RCA-D','','Sat',233,'',null from dual union all
select 52638,'1981-115','1981-11-20 08:30:00','orbital','success',680,'47193-468','Bhaskara 2','SEO B','','Sat',623,'',null from dual union all
select 52639,'1981-116','1981-11-28 18:08:00','orbital','success',680,'65093-121','Kosmos-1320/1327','Strela-1M','','Sat',457,'',null from dual union all
select 5264,'1958-S278','1958-12-03 18:38:44','suborbital rocket','success',42,'NN10.002','Field/ne probe','IGY NN10.02','','Ionos',428,'',225 from dual union all
select 52640,'1981-117','1981-12-03 11:47:00','orbital','success',1448,'45029-803','Kosmos-1328','Tselina-D','','Sat',469,'',null from dual union all
select 52641,'1981-118','1981-12-04 09:50:00','orbital','success',1238,'76053  -980','Kosmos-1329','Zenit-6','','Sat',562,'',null from dual union all
select 52642,'1981-119','1981-12-15 23:35:00','orbital','success',180,'AC-55','Intelsat 503','INTELSAT V F3','','Sat',268,'',null from dual union all
select 52643,'1981-120','1981-12-17 11:00:00','orbital','success',680,'53775-120','RS-3/8','RS-3','','Sat',457,'',null from dual union all
select 52644,'1981-121','1981-12-19 11:50:00','orbital','success',1238,'77049  -428','Kosmos-1330','Feniks No. 952','','Sat',562,'',null from dual union all
select 52645,'1981-122','1981-12-20 01:29:00','orbital','success',91,'L04','Marecs 1','Marecs A','','Sat',344,'',null from dual union all
select 52646,'1981-123','1981-12-23 13:15:00','orbital','success',809,'78039  -640','Molniya-1','Molniya-1 11F658 No. 55','','Sat',480,'',null from dual union all
select 52647,'1981-A01','1981-01-01 00:00:00','military missile','failure',756,'3','','','','Test',301,'',0 from dual union all
select 52648,'1981-A02','1981-02-09 16:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',48 from dual union all
select 52649,'1981-A03','1981-03-04 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',49 from dual union all
select 5265,'1958-S279','1958-12-06 00:41:00','military missile','success',1367,'145','R&D Series IV','','','Test',236,'',520 from dual union all
select 52650,'1981-A04','1981-03-11 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',42 from dual union all
select 52651,'1981-A05','1981-03-13 21:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 52652,'1981-A06','1981-03-17 21:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 52653,'1981-A07','1981-03-30 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 52654,'1981-A08','1981-04-15 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 52655,'1981-A09','1981-05-01 15:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',44 from dual union all
select 52656,'1981-A10','1981-05-13 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',44 from dual union all
select 52657,'1981-A11','1981-05-22 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',46 from dual union all
select 52658,'1981-A12','1981-06-16 17:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',41 from dual;
insert into LAUNCH
select 52659,'1981-A13','1981-06-17 13:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',46 from dual union all
select 5266,'1958-S280','1958-12-08 00:00:00','test rocket','failure',254,'','','','','Test',293,'B-47',11 from dual union all
select 52660,'1981-A14','1981-06-18 16:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',37 from dual union all
select 52661,'1981-A15','1981-06-23 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',48 from dual union all
select 52662,'1981-A16','1981-06-23 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',46 from dual union all
select 52663,'1981-A17','1981-06-29 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',48 from dual union all
select 52664,'1981-A18','1981-07-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',43 from dual union all
select 52665,'1981-A19','1981-07-23 19:00:00','suborbital rocket','success',1284,'','Ozone','','','Meteo',1821,'',70 from dual union all
select 52666,'1981-A20','1981-07-27 14:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',49 from dual union all
select 52667,'1981-A21','1981-09-04 17:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',18 from dual union all
select 52668,'1981-A22','1981-09-08 00:30:00','military missile','failure',1452,'49285-79-014T(10E)','15A30 UBP-25','GCh 15B103P(15F355P3 x6)','','OT',534,'',15 from dual union all
select 52669,'1981-A23','1981-09-08 19:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',48 from dual union all
select 5267,'1958-S281','1958-12-05 00:00:00','suborbital rocket','success',839,'HUGO 3','Weather photo','','','Aeron',1821,'',100 from dual union all
select 52670,'1981-A24','1981-09-13 16:04:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',0 from dual union all
select 52671,'1981-A25','1981-09-14 15:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',46 from dual union all
select 52672,'1981-A26','1981-09-15 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',43 from dual union all
select 52673,'1981-A27','1981-10-28 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',43 from dual union all
select 52674,'1981-A28','1981-11-11 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 52675,'1981-A29','1981-11-15 17:50:00','military missile','failure',1442,'DASO-8','Adv Evader 211','','','OT',421,'SSBN 640',20 from dual union all
select 52676,'1981-A30','1981-11-23 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',46 from dual union all
select 52677,'1981-A31','1981-11-25 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',46 from dual union all
select 52678,'1981-A32','1981-12-08 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',48 from dual union all
select 52679,'1981-A33','1981-12-09 12:00:52','military missile','unknown',1099,'Case 85','','','','Test',293,'FB-111',10 from dual union all
select 5268,'1958-S282','1958-12-05 00:00:00','suborbital rocket','success',839,'HUGO 4','Weather photo','','','Aeron',1821,'',100 from dual union all
select 52680,'1981-A34','1981-12-18 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',46 from dual union all
select 52681,'1981-A35','1981-12-18 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 52682,'1981-A36','1981-04-22 00:00:00','military missile','failure',1057,'8      31P/53x','','','','Test',989,'',30 from dual union all
select 52683,'1981-A37','1981-07-01 00:00:00','military missile','success',897,'PRS-1','Test','','','Test',183,'',40 from dual union all
select 52684,'1981-A38','1981-10-30 00:00:00','military missile','success',1434,'','15Zh58 B-2','Ejection test','','Test',477,'',1 from dual union all
select 52685,'1981-F01','1981-01-23 11:20:00','orbital','failure',1448,'45083-621','','Musson No. 11','','Sat',469,'',200 from dual union all
select 52686,'1981-F02','1981-03-28 09:30:00','orbital','failure',1238,'78055  -927','','Feniks No. 979','','Sat',562,'',20 from dual union all
select 52687,'1981-F03','1981-12-19 01:10:00','orbital','failure',160,'76E','GPS 7','Navstar GPS SVN 7','','Sat',1775,'',0 from dual union all
select 52688,'1981-M01','1981-02-27 18:27:00','suborbital rocket','success',891,'NASA 30.07GU','','','','Aeron',1821,'',77 from dual union all
select 52689,'1981-M02','1981-06-07 00:00:00','suborbital rocket','success',885,'4','4x15m','','','Test',1345,'',50 from dual union all
select 5269,'1958-S283','1958-12-11 00:00:00','military missile','success',1013,'','8K51D Fakel Target 3','GCh Fakel NIIP-A','','Target',321,'',500 from dual union all
select 52690,'1981-M03','1981-09-17 00:00:00','suborbital rocket','success',885,'5','1x15m','','','Test',1345,'',50 from dual union all
select 52691,'1981-M04','1981-10-01 00:00:00','suborbital rocket','success',885,'6','1x15m','','','Test',1345,'',50 from dual union all
select 52692,'1981-M05','1981-10-24 00:00:00','suborbital rocket','success',885,'7','1x15m','','','Test',1345,'',50 from dual union all
select 52693,'1981-M06','1981-11-19 00:00:00','suborbital rocket','success',885,'8','1x15m','','','Test',1345,'',50 from dual union all
select 52694,'1981-M07','1981-12-12 00:00:00','suborbital rocket','success',885,'9','1x15m','','','Test',1345,'',50 from dual union all
select 52695,'1981-M08','1982-06-02 00:00:00','suborbital rocket','success',885,'10','1x15m','','','Test',1345,'',50 from dual union all
select 52696,'1981-M09','1982-06-24 00:00:00','suborbital rocket','success',885,'11','1x15m RSO test','','','Test',1345,'',10 from dual union all
select 52697,'1981-M10','1982-09-02 00:00:00','suborbital rocket','success',885,'12','1x15m','','','Test',1345,'',50 from dual union all
select 52698,'1981-M11','1982-09-11 00:00:00','suborbital rocket','success',885,'13','1x15m','','','Test',1345,'',50 from dual union all
select 52699,'1981-M12','1982-11-10 00:00:00','suborbital rocket','success',885,'14','1x15m','','','Test',1345,'',50 from dual union all
select 527,'1945-S03','1945-01-01 00:00:00','military missile','success',1470,'','','','','Weapon',1746,'',90 from dual union all
select 5270,'1958-S284','1958-12-12 00:00:00','military missile','success',976,'B1-18','8K63 KI-1 Kontrol''niy','','','OT',636,'',402 from dual union all
select 52700,'1981-M13','1982-11-16 00:00:00','suborbital rocket','success',885,'15','1x15m','','','Test',1345,'',50 from dual union all
select 52701,'1981-M14','1982-12-09 00:00:00','suborbital rocket','success',885,'16','1x15m','','','Test',1345,'',50 from dual union all
select 52702,'1981-M15','1981-09-24 13:50:00','suborbital rocket','success',891,'NASA 30.13UE','','','','Plasma',1821,'',71 from dual union all
select 52703,'1981-M16','1981-11-14 09:15:00','suborbital rocket','success',891,'DLR K-O-111','DFVLR Strafam 1/1','','','Aeron',417,'',60 from dual union all
select 52704,'1981-M17','1981-12-13 21:31:38','suborbital rocket','failure',220,'AAF-4B-39','CENTAUR','','','Auroral',334,'',50 from dual union all
select 52705,'1981-M19','1981-01-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52706,'1981-M20','1981-01-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52707,'1981-M21','1981-01-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52708,'1981-M22','1981-01-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52709,'1981-M23','1981-01-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 5271,'1958-S285','1958-12-13 08:53:44','ballistic missile test','success',638,'AM-13','Bioflight 1','','','Bio',249,'',500 from dual union all
select 52710,'1981-M24','1981-01-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52711,'1981-M25','1981-01-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52712,'1981-M26','1981-01-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52713,'1981-M27','1981-01-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52714,'1981-M28','1981-01-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52715,'1981-M29','1981-01-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52716,'1981-M30','1981-01-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52717,'1981-M31','1981-01-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52718,'1981-M32','1981-01-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52719,'1981-M33','1981-01-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5272,'1958-S286','1958-12-16 00:00:00','military missile','success',1013,'','8K51D Fakel Target 4','GCh Fakel NIIP-A','','Target',321,'',500 from dual union all
select 52720,'1981-M34','1981-01-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52721,'1981-M35','1981-01-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52722,'1981-M36','1981-02-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52723,'1981-M37','1981-02-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52724,'1981-M38','1981-02-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52725,'1981-M39','1981-02-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52726,'1981-M40','1981-02-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52727,'1981-M41','1981-02-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52728,'1981-M42','1981-02-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52729,'1981-M43','1981-03-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 5273,'1958-S287','1958-12-16 00:00:00','test rocket','success',254,'','','','','Test',293,'B-47',200 from dual union all
select 52730,'1981-M44','1981-03-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52731,'1981-M45','1981-03-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52732,'1981-M46','1981-03-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52733,'1981-M47','1981-03-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52734,'1981-M48','1981-04-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52735,'1981-M49','1981-04-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52736,'1981-M50','1981-04-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52737,'1981-M51','1981-04-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52738,'1981-M52','1981-04-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52739,'1981-M53','1981-04-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 5274,'1958-S288','1958-12-17 04:00:00','military missile','success',1367,'146','R&D Series IV','','','Test',234,'',520 from dual union all
select 52740,'1981-M54','1981-04-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52741,'1981-M55','1981-04-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52742,'1981-M56','1981-04-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52743,'1981-M57','1981-04-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52744,'1981-M58','1981-05-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52745,'1981-M59','1981-05-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52746,'1981-M60','1981-05-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52747,'1981-M61','1981-05-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52748,'1981-M62','1981-05-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual;
insert into LAUNCH
select 52749,'1981-M63','1981-05-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5275,'1958-S289','1958-12-16 03:25:00','suborbital rocket','success',366,'','RW-109','','','Aeron',1490,'',83 from dual union all
select 52750,'1981-M64','1981-05-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52751,'1981-M65','1981-05-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52752,'1981-M66','1981-05-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52753,'1981-M67','1981-05-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52754,'1981-M68','1981-06-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52755,'1981-M69','1981-06-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52756,'1981-M70','1981-06-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52757,'1981-M71','1981-06-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 52758,'1981-M72','1981-02-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 52759,'1981-M73','1981-02-19 04:12:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 5276,'1958-S29','1958-02-21 09:40:00','suborbital rocket','success',5,'','','','','Iono-solar',636,'',206 from dual union all
select 52760,'1981-M74','1981-02-19 05:02:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 52761,'1981-M75','1981-02-19 05:50:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 52762,'1981-M76','1981-02-19 06:37:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 52763,'1981-M77','1981-02-19 07:19:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 52764,'1981-M78','1981-02-20 05:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 52765,'1981-M79','1981-02-20 05:45:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 52766,'1981-M80','1981-02-20 06:34:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 52767,'1981-M81','1981-02-20 07:34:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 52768,'1981-M82','1981-02-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 52769,'1981-M83','1981-12-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 5277,'1958-S290','1958-12-16 08:49:00','suborbital rocket','success',366,'','RW-110','','','Aeron',1490,'',83 from dual union all
select 52770,'1981-M84','1981-02-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52771,'1981-M85','1981-05-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52772,'1981-M86','1981-05-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52773,'1981-M87','1981-06-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 52774,'1981-S01','1981-11-21 08:49:23','military missile','success',914,'51033-625(7L)','15A11 LKI-2','GCh 15B99','','Test',546,'',1000 from dual union all
select 52775,'1981-S02','1981-01-01 00:00:00','military missile','success',756,'2','','','','Test',301,'',800 from dual union all
select 52776,'1981-S03','1981-01-01 00:00:00','military missile','success',756,'4','','','','Test',301,'',800 from dual union all
select 52777,'1981-S04','1981-01-01 00:00:00','military missile','success',756,'5','','','','Test',301,'',800 from dual union all
select 52778,'1981-S05','1981-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 52779,'1981-S06','1981-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 5278,'1958-S291','1958-12-16 14:47:00','suborbital rocket','success',366,'','RW-111','','','Aeron',1490,'',84 from dual union all
select 52780,'1981-S07','1981-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 52781,'1981-S08','1981-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 52782,'1981-S09','1981-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 52783,'1981-S10','1981-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 52784,'1981-S100','1981-09-17 19:20:00','military missile','success',745,'45067-524-11','15A16 UBP-4','GCh 15F161','','OT',528,'',1000 from dual union all
select 52785,'1981-S101','1981-04-27 23:15:00','suborbital rocket','success',117,'NASA 24.10UG','JHU Aries-UVT','','','UV Astron',1873,'',263 from dual union all
select 52786,'1981-S102','1981-05-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 52787,'1981-S103','1981-05-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 52788,'1981-S104','1981-05-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 52789,'1981-S105','1981-05-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 5279,'1958-S292','1958-12-16 23:44:45','military missile','success',1367,'151','OT','','TUNE UP','OT',1673,'',520 from dual union all
select 52790,'1981-S106','1981-04-30 06:00:00','suborbital rocket','success',1212,'DLR K-GR-109','DLR TEXUS 3B','','','Micrograv',416,'',253 from dual union all
select 52791,'1981-S107','1981-05-04 07:55:00','suborbital rocket','success',211,'NASA 27.58UH','Soft XRB 2','','','XR Astron',1895,'',294 from dual union all
select 52792,'1981-S108','1981-05-06 08:00:00','suborbital rocket','success',211,'NASA 27.60UL','','','','Aeron',1895,'',322 from dual union all
select 52793,'1981-S109','1981-05-08 05:27:00','suborbital rocket','success',1212,'DLR K-GR-110','DLR TEXUS 4','','','Micrograv',416,'',258 from dual union all
select 52794,'1981-S11','1981-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 52795,'1981-S110','1981-05-27 04:25:00','suborbital rocket','success',228,'NASA 21.67UG','JHU FOT 4 (Io)','','','UV Astron',1873,'',225 from dual union all
select 52796,'1981-S111','1981-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 52797,'1981-S112','1981-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 52798,'1981-S113','1981-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 52799,'1981-S114','1981-06-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 528,'1945-S04','1945-01-02 12:05:00','military missile','success',1470,'20815','Ma411','','','Test',661,'',90 from dual union all
select 5280,'1958-S293','1958-12-23 00:00:00','suborbital rocket','success',972,'1-6','','','','Aeron?',1010,'',100 from dual union all
select 52800,'1981-S115','1981-06-03 00:00:00','military missile','success',1106,'2602','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 52801,'1981-S116','1981-06-12 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT82GM','GLORY TRIP 82GM','OT',1705,'',1300 from dual union all
select 52802,'1981-S117','1981-06-12 00:00:00','military missile','success',362,'','DOT 4','','','Interceptor',871,'',303 from dual union all
select 52803,'1981-S118','1981-06-19 01:15:00','military missile','success',939,'','FOT-43','','','OT',421,'SSBN 644',500 from dual union all
select 52804,'1981-S119','1981-06-19 01:15:00','military missile','success',939,'','FOT-43','','','OT',421,'SSBN 644',500 from dual union all
select 52805,'1981-S12','1981-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 52806,'1981-S120','1981-06-19 01:16:00','military missile','success',939,'','FOT-43','','','OT',421,'SSBN 644',500 from dual union all
select 52807,'1981-S121','1981-06-19 01:16:00','military missile','success',939,'','FOT-43','','','OT',421,'SSBN 644',500 from dual union all
select 52808,'1981-S122','1981-06-26 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT83GB','GLORY TRIP 83GB','OT',1712,'',1300 from dual union all
select 52809,'1981-S123','1981-07-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 5281,'1958-S294','1958-12-23 13:00:00','suborbital rocket','success',5,'','','','','Ionos/Aeron',636,'',110 from dual union all
select 52810,'1981-S124','1981-07-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 52811,'1981-S125','1981-07-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 52812,'1981-S126','1981-07-11 14:48:00','military missile','success',939,'','FOT-44','','','OT',421,'SSBN 654',500 from dual union all
select 52813,'1981-S127','1981-07-11 14:48:00','military missile','success',939,'','FOT-44','','','OT',421,'SSBN 654',500 from dual union all
select 52814,'1981-S128','1981-07-11 14:49:00','military missile','success',939,'','FOT-44','','','OT',421,'SSBN 654',500 from dual union all
select 52815,'1981-S129','1981-07-11 14:49:00','military missile','success',939,'','FOT-44','','','OT',421,'SSBN 654',500 from dual union all
select 52816,'1981-S13','1981-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 52817,'1981-S130','1981-07-23 17:30:00','suborbital rocket','success',891,'NASA 30.14UU','UM NO Profile','','','Aeron',1821,'',73 from dual union all
select 52818,'1981-S131','1981-07-23 18:20:00','suborbital rocket','success',891,'NASA 30.08UU','UM NO Profile','','','Aeron',1821,'',75 from dual union all
select 52819,'1981-S132','1981-08-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 5282,'1958-S295','1958-12-24 00:00:00','military missile','success',976,'B1-17','8K63 KI-2','Tac pad','','OT',636,'',402 from dual union all
select 52820,'1981-S133','1981-08-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 52821,'1981-S134','1981-08-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 52822,'1981-S135','1981-08-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',141,'ZUB',150 from dual union all
select 52823,'1981-S136','1981-07-31 10:50:00','suborbital rocket','success',117,'GL A24.6S1-02','ZIP II (BMP)','','','IR Astron',1873,'',395 from dual union all
select 52824,'1981-S137','1981-08-01 22:00:00','suborbital rocket','success',1300,'TT-500A-10F','','','','Test',1476,'',320 from dual union all
select 52825,'1981-S138','1981-08-09 00:00:00','suborbital rocket','success',1319,'NASA 33.22UE','','','','Plasma',1821,'',160 from dual union all
select 52826,'1981-S139','1981-08-09 00:00:00','suborbital rocket','success',860,'NASA 31.22UE','','','','Plasma',1821,'',92 from dual union all
select 52827,'1981-S14','1981-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 52828,'1981-S140','1981-08-13 02:26:00','military missile','success',1442,'','OT-6','','','OT',421,'SSBN 630',1000 from dual union all
select 52829,'1981-S141','1981-08-13 02:46:00','military missile','success',1442,'','OT-6','','','OT',421,'SSBN 630',1000 from dual union all
select 5283,'1958-S296','1958-12-24 04:45:00','military missile','success',150,'3C','Mk 2 RV','R&D','','Test',227,'',900 from dual union all
select 52830,'1981-S142','1981-08-13 03:32:00','military missile','success',1442,'','OT-6','','','OT',421,'SSBN 630',1000 from dual union all
select 52831,'1981-S143','1981-08-13 03:32:00','military missile','success',1442,'','OT-6','','','OT',421,'SSBN 630',1000 from dual union all
select 52832,'1981-S144','1981-08-14 07:43:00','suborbital rocket','success',231,'NASA 12.38GT','','','','Test',1821,'',625 from dual union all
select 52833,'1981-S145','1981-08-22 01:00:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 636',500 from dual union all
select 52834,'1981-S146','1981-08-24 00:00:00','suborbital rocket','success',891,'NASA 30.16UE','','','','Plasma',1821,'',85 from dual union all
select 52835,'1981-S147','1981-08-24 12:00:00','suborbital rocket','success',1073,'S-310-10','','','','Test/Auroral/Aeron',839,'',187 from dual union all
select 52836,'1981-S148','1981-08-25 00:00:00','suborbital rocket','success',1319,'NASA 33.06UL','','','','Aeronomy',1873,'',200 from dual union all
select 52837,'1981-S149','1981-08-28 02:40:00','suborbital rocket','success',1014,'','Vertikal''-9','VZAF','','Solar UV/Solar X',636,'',500 from dual union all
select 52838,'1981-S15','1981-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual;
insert into LAUNCH
select 52839,'1981-S150','1981-08-28 00:03:00','military missile','success',1442,'','OT-7','','','OT',421,'SSBN 641',1000 from dual union all
select 5284,'1958-S297','1958-12-25 00:00:00','suborbital rocket','success',972,'1-7','','','','Aeron?',1010,'',102 from dual union all
select 52840,'1981-S151','1981-08-28 00:04:00','military missile','success',1442,'','OT-7','','','OT',421,'SSBN 641',1000 from dual union all
select 52841,'1981-S152','1981-08-28 00:04:00','military missile','success',1442,'','OT-7','','','OT',421,'SSBN 641',1000 from dual union all
select 52842,'1981-S153','1981-08-28 00:04:00','military missile','success',1442,'','OT-7','','','OT',421,'SSBN 641',1000 from dual union all
select 52843,'1981-S154','1981-09-05 01:00:00','suborbital rocket','success',1074,'S-520-4','PL Recovery Test','','','Test/Ionos',833,'',224 from dual union all
select 52844,'1981-S155','1981-09-07 09:38:00','suborbital rocket','success',1073,'S-310-11','Submm atm. spectra','','','Aeron',839,'',193 from dual union all
select 52845,'1981-S156','1981-09-12 00:00:00','military missile','success',795,'','Mk500-PAS-2','ABRES Mk500-PAS-2','','Reentry Test',1701,'',1300 from dual union all
select 52846,'1981-S157','1981-09-18 17:59:58','ballistic missile test','success',1497,'21','Gruziya-60-Spurt','','','Active Plasma',621,'',1514 from dual union all
select 52847,'1981-S158','1981-09-30 00:00:00','suborbital rocket','success',228,'NASA 21.63GS','','','','Solar',1873,'',243 from dual union all
select 52848,'1981-S159','1981-09-30 23:20:00','suborbital rocket','success',211,'SSC S33','PIRAT','','','IR Astron',412,'',270 from dual union all
select 52849,'1981-S16','1981-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 5285,'1958-S298','1958-12-25 00:00:00','military missile','success',976,'B2-15T','8K63 KI-3','','','OT',636,'',402 from dual union all
select 52850,'1981-S160','1981-10-04 00:00:00','military missile','success',795,'','AMARV-3','ABRES AMARV-3','','Reentry Test',1698,'',1300 from dual union all
select 52851,'1981-S161','1981-10-14 05:47:00','suborbital rocket','success',860,'NASA 31.14UE','UIUC TR','UIUC EPS','','Plasma',1821,'',197 from dual union all
select 52852,'1981-S162','1981-10-16 00:00:00','suborbital rocket','success',867,'NASA 18.1027GU','','','','Aeron',1895,'',270 from dual union all
select 52853,'1981-S163','1981-10-17 00:00:00','suborbital rocket','success',133,'NASA 25.40UH','MIT WFSXC 1','','','XR Astron',1895,'',200 from dual union all
select 52854,'1981-S164','1981-10-21 00:00:00','military missile','success',917,'AO-212   Pos 2-1','P-421 81 Fall 08','','','Test',433,'',180 from dual union all
select 52855,'1981-S165','1981-10-25 19:01:00','suborbital rocket','success',867,'Polar 6','Ferdinand 51','','','Auroral',29,'',184 from dual union all
select 52856,'1981-S166','1981-10-26 21:30:00','suborbital rocket','success',906,'GL A10.901-2','SPE-2','','','Ionos/Particles',1047,'',96 from dual union all
select 52857,'1981-S167','1981-11-07 12:03:40','suborbital rocket','success',1162,'GL A30.175','FWIF V','','','Auroral',1047,'',130 from dual union all
select 52858,'1981-S168','1981-11-09 00:00:00','suborbital rocket','success',1319,'NASA 33.21UL','','','','Aeron',1873,'',200 from dual union all
select 52859,'1981-S169','1981-11-17 00:00:00','suborbital rocket','success',210,'NASA 27.57UH','CAL XR 13','','','XR Astron',1895,'',253 from dual union all
select 5286,'1958-S299','1958-12-25 13:10:00','suborbital rocket','success',5,'','','','','Ionos/Aeron',636,'',110 from dual union all
select 52860,'1981-S17','1981-08-28 00:00:00','test rocket','unknown',1497,'20','K65UP No. 20','','','Ionos?',621,'',1500 from dual union all
select 52861,'1981-S170','1981-11-17 00:00:00','suborbital rocket','success',210,'NASA 27.09DH','','','','XR Astron',1895,'',253 from dual union all
select 52862,'1981-S171','1981-11-23 18:35:00','suborbital rocket','success',211,'NASA 27.52US','','','','Solar',1895,'',322 from dual union all
select 52863,'1981-S172','1981-11-24 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT84GM-1','GLORY TRIP 84GM-1','OT',1705,'',1300 from dual union all
select 52864,'1981-S173','1981-12-27 00:00:00','military missile','success',1057,'15     45x/65A','4 RV','','','Test',989,'',1000 from dual union all
select 52865,'1981-S174','1981-12-01 15:50:00','suborbital rocket','success',920,'P224K','Li/Na','','','Aeronomy',413,'',187 from dual union all
select 52866,'1981-S175','1981-12-01 01:38:00','suborbital rocket','success',231,'NASA 35.001UE','CENTAUR','','','Plasma',334,'',612 from dual union all
select 52867,'1981-S176','1981-12-06 23:14:58','suborbital rocket','success',220,'AAF-4B-38','CENTAUR','','','Auroral',334,'',615 from dual union all
select 52868,'1981-S177','1981-12-07 00:00:00','military missile','success',436,'','','','','Test',1494,'',1000 from dual union all
select 52869,'1981-S178','1981-12-07 00:00:40','suborbital rocket','success',1337,'NASA 29.17CE','CENTAUR','','','Plasma',334,'',451 from dual union all
select 5287,'1958-S300','1958-12-27 00:00:00','military missile','success',976,'B1-16','8K63 KI-4','','','OT',636,'',402 from dual union all
select 52870,'1981-S179','1981-12-08 00:00:00','suborbital rocket','success',30,'NASA 04.339UE','UM Airglow Pay.','','','Magnetospheric',1869,'',200 from dual union all
select 52871,'1981-S18','1981-01-01 00:00:00','military missile','success',1548,'','','','','OT',1011,'PLBR',620 from dual union all
select 52872,'1981-S180','1981-12-09 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT85GB','GLORY TRIP 85GB','OT',1712,'',1300 from dual union all
select 52873,'1981-S181','1981-12-09 17:17:00','suborbital rocket','success',921,'P225K','Li/Na','','','Aeronomy',413,'',188 from dual union all
select 52874,'1981-S182','1981-12-09 21:25:35','suborbital rocket','success',1212,'DLR K-GR-112','EBC81 E2 (ENERGY EB2)','','','Ionos/Plasma',416,'',237 from dual union all
select 52875,'1981-S183','1981-12-10 00:00:00','suborbital rocket','success',1217,'XV-10','','','','Ionos?',330,'',239 from dual union all
select 52876,'1981-S184','1981-12-13 22:54:25','suborbital rocket','success',231,'NASA 35.002UE','CENTAUR','','','Plasma',334,'',662 from dual union all
select 52877,'1981-S185','1981-12-16 14:10:00','suborbital rocket','success',920,'P226K','LITHIUM','','','Aeronomy',413,'',187 from dual union all
select 52878,'1981-S186','1981-12-19 20:00:00','suborbital rocket','success',920,'INTA FOCCA 1-1/81','FOCCA I','IR photom','','Aeron',396,'',106 from dual union all
select 52879,'1981-S187','1981-12-21 18:37:00','ballistic missile test','success',1497,'22','Vertikal''-10','VZA','','Aeron/Ionos',621,'',1508 from dual union all
select 5288,'1958-S301','1958-12-27 09:50:00','suborbital rocket','success',726,'','','','','Aeron',636,'',85 from dual union all
select 52880,'1981-S188','1981-02-04 05:00:00','suborbital rocket','success',287,'ISRO C.76','Langmuir/E Field','','','Ionos',1467,'',150 from dual union all
select 52881,'1981-S189','1981-02-12 05:27:00','suborbital rocket','success',290,'ISRO C.77','Langmuir/E Field','','','Ionos',1467,'',158 from dual union all
select 52882,'1981-S19','1981-01-01 00:00:00','military missile','success',1548,'','','','','OT',1011,'PLBR',620 from dual union all
select 52883,'1981-S190','1982-01-30 05:59:00','suborbital rocket','success',287,'ISRO C.73','Langmuir/E Field','','','Ionos',1467,'',150 from dual union all
select 52884,'1981-S191','1981-09-29 00:00:00','military missile','success',1434,'','15Zh58 B-1','Ejection test','','Test',467,'',1 from dual union all
select 52885,'1981-S192','1981-01-27 00:00:00','military missile','success',1057,'21     34x/51x','5 RV','','','Test',989,'',1000 from dual union all
select 52886,'1981-S193','1981-04-01 00:00:00','military missile','failure',1057,'9      38x/50x','','','','Test',989,'',100 from dual union all
select 52887,'1981-S195','1981-11-27 00:00:00','military missile','success',1057,'10     38P/56A','5 RV','','','Test',989,'',1000 from dual union all
select 52888,'1981-S196','1981-12-22 00:00:00','military missile','success',1057,'13     37P/63A','','','','Test',989,'',1000 from dual union all
select 52889,'1981-S197','1981-04-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 5289,'1958-S302','1958-12-30 00:00:00','military missile','success',976,'B3-20T','8K63 UBP-1','','','OT',636,'',402 from dual union all
select 52890,'1981-S198','1981-04-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 52891,'1981-S199','1981-04-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 52892,'1981-S20','1981-01-01 00:00:00','military missile','success',1548,'','','','','OT',1011,'PLBR',620 from dual union all
select 52893,'1981-S200','1981-04-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 52894,'1981-S201','1981-04-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 52895,'1981-S202','1981-06-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 52896,'1981-S203','1981-06-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 52897,'1981-S204','1981-06-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 52898,'1981-S205','1981-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 52899,'1981-S206','1981-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 529,'1945-S05','1945-01-02 15:35:00','military missile','success',1470,'20817','Ma415','','','Test',661,'',90 from dual union all
select 5290,'1958-S303','1958-04-24 00:10:00','military missile','failure',1352,'116','Able RTV','','','Reentry Test',233,'',80 from dual union all
select 52900,'1981-S207','1981-12-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 52901,'1981-S208','1981-12-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 52902,'1981-S209','1981-09-22 16:15:00','military missile','success',917,'AO-205','P-414 81 Fall 01','','','OT',433,'',180 from dual union all
select 52903,'1981-S21','1981-01-01 00:00:00','military missile','success',1548,'','','','','OT',1011,'PLBR',620 from dual union all
select 52904,'1981-S210','1981-09-22 16:48:00','military missile','success',917,'AO-206','P-415 81 Fall 02','','','OT',433,'',180 from dual union all
select 52905,'1981-S211','1981-09-29 00:00:00','military missile','unknown',917,'AO-207','P-416 81 Fall 03','','','OT',433,'',180 from dual union all
select 52906,'1981-S212','1981-09-29 00:00:00','military missile','unknown',917,'AO-208','P-417 81 Fall 04','','','OT',433,'',180 from dual union all
select 52907,'1981-S213','1981-10-13 16:00:00','military missile','success',917,'AO-209','P-418 81 Fall 05','','','OT',433,'',180 from dual union all
select 52908,'1981-S214','1981-10-13 16:30:00','military missile','success',917,'AO-210','P-419 81 Fall 06','','','OT',433,'',180 from dual union all
select 52909,'1981-S215','1981-10-21 00:00:00','military missile','unknown',917,'AO-211','P-420 81 Fall 07','','','OT',433,'',180 from dual union all
select 5291,'1958-S304','1958-01-10 00:00:00','military missile','success',969,'53030610','','','','OT',636,'',100 from dual union all
select 52910,'1981-S216','1981-11-27 17:45:00','military missile','success',1006,'51029-748-06T','15A18 UBP-10','GCh 15F183','','OT',485,'',1000 from dual union all
select 52911,'1981-S217','1981-05-21 00:00:00','military missile','success',1037,'V29502-03','8K98 UBP-56','','','Test',477,'',1000 from dual union all
select 52912,'1981-S218','1981-06-15 00:00:00','military missile','success',1044,'45799-418-04','8K98P UBP-19','','','Test',477,'',1000 from dual union all
select 52913,'1981-S219','1981-06-30 00:00:00','military missile','success',1044,'45763-317-09','8K98P UBP-20','','','Test',477,'',1000 from dual union all
select 52914,'1981-S22','1981-01-01 00:00:00','military missile','success',1548,'','','','','OT',1011,'PLBR',620 from dual union all
select 52915,'1981-S220','1981-07-29 00:00:00','military missile','success',1044,'64646-812-53','8K98P UBP-21','','','Test',477,'',1000 from dual union all
select 52916,'1981-S221','1981-08-14 00:00:00','military missile','success',1037,'29557-211-43','8K98 UBP-57','','','Test',477,'',1000 from dual union all
select 52917,'1981-S222','1981-09-15 00:00:00','military missile','success',1037,'Yu45713-29','8K98 UBP-58','','','Test',477,'',1000 from dual union all
select 52918,'1981-S223','1981-09-29 00:00:00','military missile','success',1037,'29557-211-45','8K98 UBP-59','','','Test',477,'',1000 from dual union all
select 52919,'1981-S224','1981-11-12 00:00:00','military missile','success',1037,'64647-221-34','8K98 UBP-60','','','Test',477,'',1000 from dual union all
select 5292,'1958-S305','1958-01-10 00:00:00','military missile','success',969,'53071201','','','','OT',636,'',100 from dual union all
select 52920,'1981-S225','1981-12-23 00:00:00','military missile','success',1037,'Kh29521-34','8K98 UBP-61','','','Test',477,'',1000 from dual union all
select 52921,'1981-S227','1981-04-03 00:00:00','military missile','success',1452,'63042-82-444','15A30 60 RD UBP-7','','','OT',1461,'',1000 from dual union all
select 52922,'1981-S228','1981-04-12 00:00:00','military missile','failure',1452,'49242-83-424','15A30 60 RD UBP-8','','','OT',1461,'',1000 from dual union all
select 52923,'1981-S229','1981-06-16 00:00:00','military missile','success',744,'','15A15 10 RD UBP-3','','','OT',897,'',1000 from dual union all
select 52924,'1981-S23','1981-01-01 00:00:00','military missile','success',1548,'','','','','OT',1011,'PLBR',620 from dual union all
select 52925,'1981-S230','1981-07-09 00:00:00','military missile','success',744,'','15A15 10 RD UBP-4','','','OT',897,'',1000 from dual union all
select 52926,'1981-S231','1981-07-15 00:00:00','military missile','success',744,'','15A15 10 RD UBP-5','','','OT',897,'',1000 from dual union all
select 52927,'1981-S232','1981-07-23 00:00:00','military missile','success',744,'','15A15 10 RD UBP-6','','','OT',897,'',1000 from dual union all
select 52928,'1981-S233','1981-09-14 00:00:00','military missile','success',744,'','15A15 10 RD UBP-7','','','OT',897,'',1000 from dual;
insert into LAUNCH
select 52929,'1981-S234','1981-09-18 00:00:00','military missile','success',744,'','15A15 10 RD UBP-8','','','OT',897,'',1000 from dual union all
select 5293,'1958-S306','1958-01-11 00:00:00','military missile','success',969,'53091713','','','','OT',636,'',100 from dual union all
select 52930,'1981-S235','1981-09-11 00:00:00','military missile','success',1450,'','15A20 52 RD UBP','','','OT',195,'',1000 from dual union all
select 52931,'1981-S236','1981-02-10 00:00:00','military missile','success',1004,'512','15A14 62 RD UBP-2','GCh 15F141','','OT',1646,'',1000 from dual union all
select 52932,'1981-S237','1981-02-11 00:00:00','military missile','success',1004,'511','15A14 62 RD UBP-3','GCh 15F141','','OT',1646,'',1000 from dual union all
select 52933,'1981-S238','1981-03-05 00:00:00','military missile','success',1004,'507','15A14 62 RD UBP-4','GCh 15F141','','OT',1646,'',1000 from dual union all
select 52934,'1981-S239','1981-03-05 00:00:00','military missile','success',1004,'502','15A14 62 RD UBP-5','GCh 15F141','','OT',1646,'',1000 from dual union all
select 52935,'1981-S24','1981-01-01 00:00:00','military missile','success',1548,'','','','','OT',1011,'PLBR',620 from dual union all
select 52936,'1981-S240','1981-03-06 00:00:00','military missile','failure',1004,'501','15A14 62 RD UBP-6','GCh 15F141','','OT',1646,'',1000 from dual union all
select 52937,'1981-S241','1981-09-01 00:00:00','military missile','success',1450,'','15A20 36 RD UBP-1','','','OT',898,'',1000 from dual union all
select 52938,'1981-S242','1981-09-01 00:00:00','military missile','success',1450,'','15A20 36 RD UBP-2','','','OT',898,'',1000 from dual union all
select 52939,'1981-S243','1981-04-01 00:00:00','military missile','success',1004,'503','15A14 UBP 59 RD','GCh 15F143U1','','OT',830,'',1000 from dual union all
select 5294,'1958-S307','1958-01-17 00:00:00','military missile','success',992,'53120317','','','','OT',636,'',170 from dual union all
select 52940,'1981-S244','1981-04-28 00:00:00','military missile','success',1004,'509','15A14 UBP 59 RD','GCh 15F143U1','','OT',830,'',1000 from dual union all
select 52941,'1981-S245','1981-04-29 00:00:00','military missile','success',1004,'506','15A14 UBP 59 RD','GCh 15F143U1','','OT',830,'',1000 from dual union all
select 52942,'1981-S247','1981-11-25 14:43:00','suborbital rocket','success',1284,'INTA SLD-8101','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 52943,'1981-S248','1981-12-16 14:34:00','suborbital rocket','success',1284,'INTA SLD-8102','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 52944,'1981-S249','1981-12-22 14:40:00','suborbital rocket','success',1284,'INTA SLD-8103','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 52945,'1981-S25','1981-01-01 00:00:00','military missile','success',1548,'','','','','OT',1011,'PLBR',620 from dual union all
select 52946,'1981-S250','1981-12-10 18:00:00','suborbital rocket','success',1311,'09','Earth Resources Camera','','','Earth Imaging',310,'',131 from dual union all
select 52947,'1981-S251','1981-09-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-1','','','Test',368,'',500 from dual union all
select 52948,'1981-S252','1981-09-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-2','','','Test',368,'',500 from dual union all
select 52949,'1981-S253','1981-05-01 00:00:00','military missile','success',922,'','15Zh45  8 RD UBP','','','Test',636,'',500 from dual union all
select 5295,'1958-S308','1958-01-31 00:00:00','military missile','success',969,'D2111','','','','OT',636,'',100 from dual union all
select 52950,'1981-S254','1981-08-27 00:00:00','military missile','success',923,'','15Zh53 32 RD UBP','','','Test',636,'',500 from dual union all
select 52951,'1981-S255','1981-08-01 00:00:00','military missile','success',976,'','8K63 29 RD UBP','','','Test',636,'',402 from dual union all
select 52952,'1981-S256','1981-12-01 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 52953,'1981-S257','1981-12-01 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 52954,'1981-S26','1981-01-01 00:00:00','military missile','success',1507,'VMF','','','','OT',1011,'PLBR',1000 from dual union all
select 52955,'1981-S27','1981-01-01 00:00:00','military missile','success',1507,'VMF','','','','OT',1011,'PLBR',1000 from dual union all
select 52956,'1981-S28','1981-01-01 00:00:00','military missile','success',1507,'VMF','','','','OT',1011,'PLBR',1000 from dual union all
select 52957,'1981-S29','1981-01-01 00:00:00','military missile','success',1514,'','','','','OT',1011,'PLBR',1000 from dual union all
select 52958,'1981-S30','1981-01-01 00:00:00','military missile','success',1514,'','','','','OT',1011,'PLBR',1000 from dual union all
select 52959,'1981-S31','1981-01-01 00:00:00','military missile','success',1514,'','','','','OT',1011,'PLBR',1000 from dual union all
select 5296,'1958-S309','1958-02-11 00:00:00','military missile','success',969,'519088433','','','','OT',636,'',100 from dual union all
select 52960,'1981-S32','1981-01-01 00:00:00','military missile','success',1514,'','','','','OT',1011,'PLBR',1000 from dual union all
select 52961,'1981-S33','1981-02-17 14:30:00','military missile','success',1006,'51099-718-09T','15A18 UBP-6 ZP','GCh 15F183','','OT',487,'',1000 from dual union all
select 52962,'1981-S34','1981-05-29 00:50:00','military missile','success',1006,'67099-728-08T','15A18 UBP-7','GCh 15F183','','OT',487,'',1000 from dual union all
select 52963,'1981-S35','1981-07-18 01:00:00','military missile','success',1006,'67083-436-42T','15A18 UBP-8','GCh 15F183','','OT',485,'',1000 from dual union all
select 52964,'1981-S36','1981-10-02 01:45:00','military missile','success',1006,'45029-739-07T','15A18 UBP-9','GCh 15F183','','OT',487,'',1000 from dual union all
select 52965,'1981-S37','1981-08-18 00:00:00','military missile','success',1006,'211','15A14 UBP 59 RD','GCh 15F143U1','','Training',826,'',1000 from dual union all
select 52966,'1981-S38','1981-05-18 23:30:00','military missile','success',1452,'63085-53-241(2E)','15A30 UBP-21','GCh 15B103','','OT',532,'',1000 from dual union all
select 52967,'1981-S39','1981-06-05 18:45:00','military missile','success',1452,'63085-53-251T(3E)','15A30 UBP-22','GCh 15B103','','OT',534,'',1000 from dual union all
select 52968,'1981-S40','1981-07-01 15:50:00','military missile','success',1452,'63085-53-248(5E)','15A30 UBP-23','GCh 15B103','','OT',534,'',1000 from dual union all
select 52969,'1981-S41','1981-07-31 00:05:00','military missile','success',1452,'63085-53-249(8E)','15A30 UBP-24','GCh 15B103','','OT',534,'',1000 from dual union all
select 5297,'1958-S31','1958-02-22 02:02:26','suborbital rocket','success',49,'NN3.18F','NRL NN3.18F','IGY NN3.18F','','Ionos,AuroralX',428,'',225 from dual union all
select 52970,'1981-S42','1981-09-09 23:00:00','military missile','success',1452,'49285-79-013T(11E)','15A30 UBP-26','GCh 15B103P','','OT',532,'',1000 from dual union all
select 52971,'1981-S43','1981-04-21 00:00:00','military missile','success',1453,'49219-21-133T(1E)','15A35 UBP-7 PED','','','OT',535,'',1000 from dual union all
select 52972,'1981-S44','1981-06-23 00:00:00','military missile','success',1453,'49243-94-841T(4E)','15A35 UBP-8','','','OT',535,'',1000 from dual union all
select 52973,'1981-S45','1981-07-19 01:40:00','military missile','success',1453,'51139-22-121T(6E)','15A35 UBP-9','GCh 15F355','','OT',505,'',1000 from dual union all
select 52974,'1981-S46','1981-07-28 00:25:00','military missile','success',1453,'51139-22-120T(7E)','15A35 UBP-10','GCh 15F355','','OT',596,'',1000 from dual union all
select 52975,'1981-S47','1981-08-21 00:00:00','military missile','success',1453,'51139-82-019T(9E)','15A35 UBP-11','GCh 15F355','','OT',535,'',1000 from dual union all
select 52976,'1981-S48','1981-09-12 01:00:00','military missile','success',1453,'51139-82-020(12E)','15A35 UBP-12','GCh 15F355','','OT',505,'',1000 from dual union all
select 52977,'1981-S49','1981-01-10 18:22:00','suborbital rocket','success',867,'NASA 18.205UE','','','','Ionospheric',1408,'',198 from dual union all
select 52978,'1981-S50','1981-01-14 22:00:00','suborbital rocket','success',1300,'TT-500A-9F','','','','Test',1476,'',320 from dual union all
select 52979,'1981-S51','1981-01-19 03:37:27','suborbital rocket','success',227,'AED-VB-54','Upper Atm Compos.','','','Aeron',428,'',263 from dual union all
select 5298,'1958-S310','1958-02-20 00:00:00','military missile','success',992,'K3-18','','','','OT',636,'',170 from dual union all
select 52980,'1981-S52','1981-01-20 04:50:00','suborbital rocket','success',133,'NASA 25.62DG','NRL FUV Spectrograph','','','UV Astron',1895,'',209 from dual union all
select 52981,'1981-S53','1981-01-21 00:25:00','suborbital rocket','success',211,'NASA 27.17NP','SPAR 9','','','Micrograv',1895,'',209 from dual union all
select 52982,'1981-S54','1981-01-21 21:46:00','suborbital rocket','success',1073,'S-310-9','','','','Aeron',839,'',174 from dual union all
select 52983,'1981-S55','1981-01-26 07:20:06','suborbital rocket','success',220,'AAF-4B-36','Substorm','','','Auroral',428,'',584 from dual union all
select 52984,'1981-S56','1981-01-29 04:12:00','suborbital rocket','success',133,'NASA 25.60UE','UM Airglow Pay.','','','Auroral',428,'',189 from dual union all
select 52985,'1981-S57','1981-01-29 07:00:00','suborbital rocket','success',1074,'S-520-2','TPE-2','','','Ionos/Tether',833,'',323 from dual union all
select 52986,'1981-S58','1981-02-04 01:10:00','suborbital rocket','success',117,'GL A24.7S1-01','IRBS (BMP)','','','ABM Tech IR/IR Astron',1873,'',387 from dual union all
select 52987,'1981-S59','1981-02-04 15:00:00','military missile','unknown',917,'FOT-141','P-405 81 Cape 01','','','OT',231,'',180 from dual union all
select 52988,'1981-S60','1981-02-04 15:33:00','military missile','unknown',917,'FOT-139','P-406 81 Cape 02','','','OT',231,'',180 from dual union all
select 52989,'1981-S61','1981-02-04 15:53:00','military missile','unknown',917,'FOT-140','P-407 81 Cape 03','','','OT',231,'',180 from dual union all
select 5299,'1958-S311','1958-03-08 00:00:00','military missile','success',971,'B1-19','','','','Test',636,'',150 from dual union all
select 52990,'1981-S62','1981-02-05 11:25:00','suborbital rocket','success',1162,'GL A30.072','FWIF IV','','','Auroral',1047,'',130 from dual union all
select 52991,'1981-S63','1981-02-07 20:13:45','suborbital rocket','success',921,'P219K','Atomic oxygen','','','Aeronomy',413,'',171 from dual union all
select 52992,'1981-S64','1981-02-07 20:30:40','suborbital rocket','success',860,'SSC S35','OXYGEN','','','Aeronomy',416,'',145 from dual union all
select 52993,'1981-S65','1981-02-09 14:31:00','military missile','success',799,'Global Shield 81','Mk 12 RV x 3','FOT GT79GM','GLORY TRIP 79GM','OT',1703,'',1300 from dual union all
select 52994,'1981-S66','1981-02-09 16:38:00','military missile','success',799,'Global Shield 81','Mk 12 RV x 3','FOT GT80GM','GLORY TRIP 80GM','OT',1705,'',1300 from dual union all
select 52995,'1981-S67','1981-02-13 17:15:00','suborbital rocket','success',211,'NASA 27.50CS','','','','Solar',1895,'',265 from dual union all
select 52996,'1981-S68','1981-02-17 17:15:00','military missile','unknown',917,'FOT-143','P-408 81 Cape 04','','','OT',231,'',180 from dual union all
select 52997,'1981-S69','1981-02-17 17:51:00','military missile','unknown',917,'FOT-142','P-409 81 Cape 05','','','OT',231,'',180 from dual union all
select 52998,'1981-S70','1981-02-17 19:17:00','military missile','unknown',917,'FOT-144','P-410 81 Cape 06','','','OT',231,'',180 from dual union all
select 52999,'1981-S71','1981-02-18 11:12:00','suborbital rocket','success',599,'INTA IT-P4-8101','','','','Tech',396,'',285 from dual union all
select 53,'1944-A17','1944-09-15 10:37:00','military missile','failure',8,'18756','Ma171','','','Training',661,'',1 from dual union all
select 530,'1945-S06','1945-01-02 18:37:00','military missile','success',1470,'20818','Ma412','','','Test',661,'',90 from dual union all
select 5300,'1958-S312','1958-05-25 00:00:00','military missile','success',969,'K6-7','','','','OT',636,'',100 from dual union all
select 53000,'1981-S72','1981-02-18 00:00:00','military missile','success',795,'','IRS-1','ABRES IRS-1','','Reentry Test',1701,'',1300 from dual union all
select 53001,'1981-S73','1981-02-21 14:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT81GB','GLORY TRIP 81GB','OT',1712,'',1300 from dual union all
select 53002,'1981-S74','1981-02-25 17:20:00','suborbital rocket','success',1164,'','HAVE SLED','','','Test',1047,'',400 from dual union all
select 53003,'1981-S75','1981-03-01 09:20:00','suborbital rocket','success',885,'3','4x15m','','','Test',1345,'',150 from dual union all
select 53004,'1981-S76','1981-03-05 16:48:00','suborbital rocket','success',1164,'','HAVE SLED','','','Test',1047,'',400 from dual union all
select 53005,'1981-S77','1981-03-07 08:09:00','suborbital rocket','success',1319,'A13.30','AURORAL E1','','','Auroral',1047,'',156 from dual union all
select 53006,'1981-S78','1981-03-07 08:09:00','suborbital rocket','success',1319,'A13.20','AURORAL E3','','','Aeron/Ionos/Plasma',1047,'',203 from dual union all
select 53007,'1981-S79','1981-03-07 08:26:00','suborbital rocket','success',1319,'A13.31','AURORAL E2','','','Auroral',1047,'',169 from dual union all
select 53008,'1981-S80','1981-03-07 08:38:00','suborbital rocket','success',906,'GL A10.901-3','SPE-3/AURORAL E4','','','Auroral',1047,'',187 from dual union all
select 53009,'1981-S81','1981-03-08 16:00:00','suborbital rocket','success',860,'NASA 31.07UL','CWAS 1','','','Aeronomy',428,'',140 from dual union all
select 5301,'1958-S313','1958-05-25 00:00:00','military missile','success',969,'K2-19','','','','OT',636,'',100 from dual union all
select 53010,'1981-S82','1981-03-12 04:30:00','suborbital rocket','success',1164,'','HAVE SLED','','','Test',1047,'',400 from dual union all
select 53011,'1981-S83','1981-03-13 16:00:00','suborbital rocket','success',860,'NASA 31.17UL','CWAS 2','','','Aeronomy',428,'',140 from dual union all
select 53012,'1981-S84','1981-03-15 00:00:00','military missile','success',795,'','TDV-4','ABRES TDV-4','','Reentry Test',1698,'',1300 from dual union all
select 53013,'1981-S85','1981-03-24 03:00:00','suborbital rocket','success',211,'NASA 27.55UH','XUV Telescope','','','EUV Astron',1895,'',261 from dual union all
select 53014,'1981-S86','1981-03-25 08:05:16','suborbital rocket','success',1320,'NASA 34.05UE','BUBBLE MACHINE 1','','','Plasma',1047,'',496 from dual union all
select 53015,'1981-S87','1981-03-28 01:54:00','suborbital rocket','success',220,'AAF-4B-37','FALCONS','','','Ionos/Plasma/Fields',428,'',675 from dual union all
select 53016,'1981-S88','1981-03-28 01:54:06','suborbital rocket','success',227,'AAF-VB-55','FALCONS','','','Ionos/Plasma/Fields',428,'',327 from dual union all
select 53017,'1981-S89','1981-03-30 05:52:55','suborbital rocket','success',860,'','LADY LOU  Ba','','','Aeron/Fields',1047,'',140 from dual;
insert into LAUNCH
select 53018,'1981-S90','1981-04-01 13:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT75GM-2','GLORY TRIP 75GM-2','OT',1699,'',1300 from dual union all
select 53019,'1981-S91','1981-04-01 09:04:35','suborbital rocket','success',1320,'NASA 34.04UE','','','','Plasma',1047,'',500 from dual union all
select 5302,'1958-S314','1958-05-27 00:00:00','military missile','success',969,'K9-9','','','','OT',636,'',100 from dual union all
select 53020,'1981-S92','1981-04-01 09:10:00','military missile','success',1442,'','DASO-7','','','OT',421,'SSBN 641',1000 from dual union all
select 53021,'1981-S93','1981-04-02 09:48:00','suborbital rocket','success',1319,'NASA 33.20GE','','','','Plasma',1821,'',220 from dual union all
select 53022,'1981-S94','1981-04-03 09:51:00','suborbital rocket','success',1319,'NASA 33.19GE','','','','Plasma',1821,'',210 from dual union all
select 53023,'1981-S95','1981-04-03 06:20:55','suborbital rocket','success',860,'','KLONDIKE KATE Ba','','','Aeron/Fields',1047,'',140 from dual union all
select 53024,'1981-S96','1981-04-04 00:00:00','military missile','success',795,'','Mk500-PAS-1','ABRES Mk500-PAS-1','','Reentry Test',1701,'',1300 from dual union all
select 53025,'1981-S97','1981-04-07 14:00:00','military missile','unknown',917,'FOT-146','P-411 81 Cape 07','','','OT',231,'',180 from dual union all
select 53026,'1981-S98','1981-04-07 14:19:00','military missile','unknown',917,'FOT-147','P-412 81 Cape 08','','','OT',231,'',180 from dual union all
select 53027,'1981-S99','1981-04-07 14:40:00','military missile','unknown',917,'FOT-148','P-413 81 Cape 09','','','OT',231,'',180 from dual union all
select 53028,'1981-W01','1981-01-02 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 53029,'1981-W02','1981-01-02 15:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 5303,'1958-S315','1958-05-27 00:00:00','military missile','success',969,'K1-3','','','','OT',636,'',100 from dual union all
select 53030,'1981-W03','1981-01-05 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 53031,'1981-W04','1981-01-07 02:03:59','suborbital rocket','success',761,'MT135P-R408','','','','Meteo',1332,'',57 from dual union all
select 53032,'1981-W05','1981-01-07 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53033,'1981-W06','1981-01-08 13:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',64 from dual union all
select 53034,'1981-W07','1981-01-09 14:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 53035,'1981-W08','1981-01-12 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53036,'1981-W09','1981-01-12 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 53037,'1981-W10','1981-01-12 18:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 53038,'1981-W100','1981-02-20 15:00:00','suborbital rocket','unknown',738,'FLIGHT 296','','','','Aeron',1300,'PRILIV',64 from dual union all
select 53039,'1981-W101','1981-02-20 17:00:00','suborbital rocket','unknown',738,'','','','','Aeron',908,'KRENKEL''',56 from dual union all
select 5304,'1958-S316','1958-07-04 00:00:00','military missile','success',992,'52120418','','','','OT',636,'',170 from dual union all
select 53040,'1981-W102','1981-02-20 18:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',66 from dual union all
select 53041,'1981-W103','1981-02-20 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 53042,'1981-W104','1981-02-20 21:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 53043,'1981-W105','1981-02-22 13:20:00','suborbital rocket','unknown',738,'','','','','Aeron',1762,'VOLNA',58 from dual union all
select 53044,'1981-W106','1981-02-23 10:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53045,'1981-W107','1981-02-23 15:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1762,'VOLNA',62 from dual union all
select 53046,'1981-W108','1981-02-23 15:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 53047,'1981-W109','1981-02-23 16:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 53048,'1981-W11','1981-01-14 02:00:00','suborbital rocket','success',761,'MT135P-R409','','','','Meteo',1332,'',53 from dual union all
select 53049,'1981-W110','1981-02-23 20:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 5305,'1958-S317','1958-07-04 00:00:00','military missile','success',969,'52080408','','','','OT',636,'',100 from dual union all
select 53050,'1981-W111','1981-02-23 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 53051,'1981-W112','1981-02-24 00:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 53052,'1981-W113','1981-02-24 13:40:00','suborbital rocket','unknown',738,'','','','','Aeron',1762,'VOLNA',57 from dual union all
select 53053,'1981-W114','1981-02-24 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 53054,'1981-W115','1981-02-25 00:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 53055,'1981-W116','1981-02-25 12:30:00','suborbital rocket','unknown',738,'','','','','Aeron',1300,'PRILIV',56 from dual union all
select 53056,'1981-W117','1981-02-25 12:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53057,'1981-W118','1981-02-25 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 53058,'1981-W119','1981-02-26 01:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 53059,'1981-W12','1981-01-14 04:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',77 from dual union all
select 5306,'1958-S318','1958-07-08 00:00:00','military missile','success',992,'52120416','','','','OT',636,'',170 from dual union all
select 53060,'1981-W120','1981-02-26 02:00:00','suborbital rocket','success',761,'MT135P-R414','','','','Meteo',1332,'',61 from dual union all
select 53061,'1981-W121','1981-02-26 15:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 53062,'1981-W122','1981-02-26 16:28:00','suborbital rocket','unknown',1284,'FLIGHT 291     /T 1-','','','','Aeron',1821,'',66 from dual union all
select 53063,'1981-W123','1981-02-26 19:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 53064,'1981-W124','1981-02-26 19:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',76 from dual union all
select 53065,'1981-W125','1981-02-26 20:45:00','suborbital rocket','unknown',1284,'FLIGHT 292     /T 1-','','','','Aeron',1821,'',65 from dual union all
select 53066,'1981-W126','1981-02-26 22:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 53067,'1981-W127','1981-02-27 10:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 53068,'1981-W128','1981-02-27 14:24:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',78 from dual union all
select 53069,'1981-W129','1981-02-27 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 5307,'1958-S319','1958-03-30 00:00:00','military missile','success',1013,'B3-20T','8K51 OT-15','','','Test',636,'',500 from dual union all
select 53070,'1981-W13','1981-01-14 13:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1762,'VOLNA',54 from dual union all
select 53071,'1981-W130','1981-02-27 21:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 53072,'1981-W131','1981-02-27 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 53073,'1981-W132','1981-03-02 05:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',63 from dual union all
select 53074,'1981-W133','1981-03-02 15:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 53075,'1981-W134','1981-03-02 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 53076,'1981-W135','1981-03-02 21:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 53077,'1981-W136','1981-03-02 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 53078,'1981-W137','1981-03-02 21:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 53079,'1981-W138','1981-03-03 13:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 5308,'1958-S32','1958-02-22 05:34:31','suborbital rocket','success',839,'II6.26F','Auroral particles','IGY II6.26F','','Auroral',428,'',129 from dual union all
select 53080,'1981-W139','1981-03-03 17:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',77 from dual union all
select 53081,'1981-W14','1981-01-14 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53082,'1981-W140','1981-03-04 15:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 53083,'1981-W141','1981-03-04 17:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 53084,'1981-W142','1981-03-04 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 53085,'1981-W143','1981-03-05 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 53086,'1981-W144','1981-03-06 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53087,'1981-W145','1981-03-06 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 53088,'1981-W146','1981-03-06 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 53089,'1981-W147','1981-03-06 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 5309,'1958-S320','1958-04-22 00:00:00','military missile','success',1013,'A6-85V','8K51 OT-16','Vibrator','','Test',636,'',500 from dual union all
select 53090,'1981-W148','1981-03-06 18:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 53091,'1981-W149','1981-03-06 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 53092,'1981-W15','1981-01-14 15:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1300,'PRILIV',55 from dual union all
select 53093,'1981-W150','1981-03-07 00:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 53094,'1981-W151','1981-03-07 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',77 from dual union all
select 53095,'1981-W152','1981-03-09 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 53096,'1981-W153','1981-03-09 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53097,'1981-W154','1981-03-09 16:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 53098,'1981-W155','1981-03-09 20:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 53099,'1981-W156','1981-03-09 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 531,'1945-S07','1945-01-03 12:19:00','military missile','success',1470,'20838','Ma418','','','Test',661,'',90 from dual union all
select 5310,'1958-S321','1958-04-30 00:00:00','military missile','success',1013,'B4-20TS','8K51 OT-17','Minsredmash flight','','Test',636,'',500 from dual union all
select 53100,'1981-W157','1981-03-10 00:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',79 from dual union all
select 53101,'1981-W158','1981-03-10 17:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',55 from dual union all
select 53102,'1981-W159','1981-03-11 02:01:00','suborbital rocket','success',761,'MT135P-R415','','','','Meteo',1332,'',56 from dual union all
select 53103,'1981-W16','1981-01-15 13:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1762,'VOLNA',61 from dual union all
select 53104,'1981-W160','1981-03-11 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53105,'1981-W161','1981-03-11 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 53106,'1981-W162','1981-03-11 21:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 53107,'1981-W163','1981-03-11 21:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual;
insert into LAUNCH
select 53108,'1981-W164','1981-03-12 18:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 53109,'1981-W165','1981-03-12 21:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 5311,'1958-S322','1958-05-09 00:00:00','military missile','success',1013,'M5-6TK','8K51 OT-18','VV GCh','','Test',636,'',500 from dual union all
select 53110,'1981-W166','1981-03-13 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53111,'1981-W167','1981-03-13 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 53112,'1981-W168','1981-03-15 06:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 53113,'1981-W169','1981-03-15 09:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 53114,'1981-W17','1981-01-16 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 53115,'1981-W170','1981-03-15 15:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 53116,'1981-W171','1981-03-16 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53117,'1981-W172','1981-03-16 21:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 53118,'1981-W173','1981-03-17 14:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 53119,'1981-W174','1981-03-17 20:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',63 from dual union all
select 5312,'1958-S323','1958-05-16 00:00:00','military missile','success',1013,'A6-82V','8K51 OT-19','Vibrator','','Test',636,'',500 from dual union all
select 53120,'1981-W175','1981-03-18 02:03:59','suborbital rocket','success',761,'MT135P-R416','','','','Meteo',1332,'',58 from dual union all
select 53121,'1981-W176','1981-03-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',72 from dual union all
select 53122,'1981-W177','1981-03-18 15:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53123,'1981-W178','1981-03-18 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 53124,'1981-W179','1981-03-18 20:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',61 from dual union all
select 53125,'1981-W18','1981-01-16 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53126,'1981-W180','1981-03-18 21:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 53127,'1981-W181','1981-03-19 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',76 from dual union all
select 53128,'1981-W182','1981-03-19 20:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 53129,'1981-W183','1981-03-20 19:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 5313,'1958-S324','1958-05-29 00:00:00','military missile','success',1013,'B5-20T','8K51 UBP-1','','','Test',636,'',500 from dual union all
select 53130,'1981-W184','1981-03-20 21:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 53131,'1981-W185','1981-03-20 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 53132,'1981-W186','1981-03-20 21:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',61 from dual union all
select 53133,'1981-W187','1981-03-23 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 53134,'1981-W188','1981-03-23 15:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1300,'PRILIV',58 from dual union all
select 53135,'1981-W189','1981-03-23 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 53136,'1981-W19','1981-01-17 14:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1300,'PRILIV',58 from dual union all
select 53137,'1981-W190','1981-03-24 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 53138,'1981-W191','1981-03-24 21:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 53139,'1981-W192','1981-03-25 12:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 5314,'1958-S325','1958-05-30 00:00:00','military missile','success',1013,'M5-4TK','8K51 UBP-2','VV GCh','','Test',636,'',500 from dual union all
select 53140,'1981-W193','1981-03-25 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 53141,'1981-W194','1981-03-25 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 53142,'1981-W195','1981-03-25 18:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',63 from dual union all
select 53143,'1981-W196','1981-03-25 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 53144,'1981-W197','1981-03-25 22:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 53145,'1981-W198','1981-03-26 02:03:59','suborbital rocket','success',761,'MT135P-R417','','','','Meteo',1332,'',52 from dual union all
select 53146,'1981-W199','1981-03-26 15:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1300,'PRILIV',55 from dual union all
select 53147,'1981-W20','1981-01-19 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53148,'1981-W200','1981-03-26 15:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',62 from dual union all
select 53149,'1981-W201','1981-03-26 16:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1762,'VOLNA',58 from dual union all
select 5315,'1958-S326','1958-06-03 00:00:00','military missile','success',1013,'M5-10TK','8K51 UBP-3','VV GCh','','Test',636,'',500 from dual union all
select 53150,'1981-W202','1981-03-26 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',54 from dual union all
select 53151,'1981-W203','1981-03-26 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 53152,'1981-W204','1981-03-26 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',72 from dual union all
select 53153,'1981-W205','1981-03-27 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 53154,'1981-W206','1981-03-27 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53155,'1981-W207','1981-03-27 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 53156,'1981-W208','1981-03-27 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 53157,'1981-W209','1981-03-30 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 53158,'1981-W21','1981-01-19 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',58 from dual union all
select 53159,'1981-W210','1981-03-30 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 5316,'1958-S327','1958-06-23 00:00:00','military missile','failure',1013,'M5-8TK','8K51 UBP-4','VV GCh','','Test',636,'',300 from dual union all
select 53160,'1981-W211','1981-03-30 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 53161,'1981-W212','1981-03-30 22:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 53162,'1981-W213','1981-03-31 19:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 53163,'1981-W214','1981-04-01 00:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 53164,'1981-W215','1981-04-01 11:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',72 from dual union all
select 53165,'1981-W216','1981-04-01 18:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 53166,'1981-W217','1981-04-01 23:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53167,'1981-W218','1981-04-02 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 53168,'1981-W219','1981-04-02 15:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1300,'PRILIV',55 from dual union all
select 53169,'1981-W22','1981-01-19 23:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 5317,'1958-S328','1958-06-26 00:00:00','military missile','success',1013,'B6-20T','8K51 UBP-5','','','Test',636,'',500 from dual union all
select 53170,'1981-W220','1981-04-03 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53171,'1981-W221','1981-04-03 16:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 53172,'1981-W222','1981-04-03 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',69 from dual union all
select 53173,'1981-W223','1981-04-03 20:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 53174,'1981-W224','1981-04-03 20:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 53175,'1981-W225','1981-04-05 16:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1300,'PRILIV',56 from dual union all
select 53176,'1981-W226','1981-04-06 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53177,'1981-W227','1981-04-06 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 53178,'1981-W228','1981-04-06 19:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53179,'1981-W229','1981-04-07 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',71 from dual union all
select 5318,'1958-S329','1958-05-31 00:00:00','military missile','success',971,'B2-3','','','','Test',636,'',150 from dual union all
select 53180,'1981-W23','1981-01-20 13:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1300,'PRILIV',58 from dual union all
select 53181,'1981-W230','1981-04-07 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',55 from dual union all
select 53182,'1981-W231','1981-04-07 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 53183,'1981-W232','1981-04-08 02:00:00','suborbital rocket','success',761,'MT135P-R418','','','','Meteo',1332,'',55 from dual union all
select 53184,'1981-W233','1981-04-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',70 from dual union all
select 53185,'1981-W234','1981-04-08 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 53186,'1981-W235','1981-04-08 16:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual union all
select 53187,'1981-W236','1981-04-08 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 53188,'1981-W237','1981-04-08 20:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 53189,'1981-W238','1981-04-09 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual union all
select 5319,'1958-S33','1958-02-24 06:59:58','suborbital rocket','success',49,'NN3.14F','NRL NN3.14F','IGY NN3.14F','','Aeron',428,'',207 from dual union all
select 53190,'1981-W239','1981-04-09 16:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',55 from dual union all
select 53191,'1981-W24','1981-01-20 13:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1762,'VOLNA',55 from dual union all
select 53192,'1981-W240','1981-04-10 05:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 53193,'1981-W241','1981-04-10 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53194,'1981-W242','1981-04-10 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',52 from dual union all
select 53195,'1981-W243','1981-04-10 19:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 53196,'1981-W244','1981-04-10 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 53197,'1981-W245','1981-04-10 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 53198,'1981-W246','1981-04-12 05:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual;
insert into LAUNCH
select 53199,'1981-W247','1981-04-12 12:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',58 from dual union all
select 532,'1945-S08','1945-01-03 14:50:00','military missile','success',1470,'20841','Ma420','','','Test',661,'',90 from dual union all
select 5320,'1958-S330','1958-05-31 00:00:00','military missile','success',971,'B2-67','','','','Test',636,'',150 from dual union all
select 53200,'1981-W248','1981-04-12 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',66 from dual union all
select 53201,'1981-W249','1981-04-12 15:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',75 from dual union all
select 53202,'1981-W25','1981-01-21 04:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 53203,'1981-W250','1981-04-12 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 53204,'1981-W251','1981-04-12 17:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 53205,'1981-W252','1981-04-12 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 53206,'1981-W253','1981-04-12 22:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 53207,'1981-W254','1981-04-13 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 53208,'1981-W255','1981-04-13 11:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',52 from dual union all
select 53209,'1981-W256','1981-04-13 12:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 5321,'1958-S331','1958-07-08 00:00:00','military missile','success',992,'52120415','','','','OT',636,'',170 from dual union all
select 53210,'1981-W257','1981-04-13 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 53211,'1981-W258','1981-04-13 15:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 53212,'1981-W259','1981-04-13 15:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',74 from dual union all
select 53213,'1981-W26','1981-01-21 13:30:00','suborbital rocket','unknown',738,'','','','','Aeron',1762,'VOLNA',60 from dual union all
select 53214,'1981-W260','1981-04-13 17:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 53215,'1981-W261','1981-04-13 20:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53216,'1981-W262','1981-04-13 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 53217,'1981-W263','1981-04-14 00:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 53218,'1981-W264','1981-04-14 09:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 53219,'1981-W265','1981-04-14 12:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 5322,'1958-S332','1958-07-08 00:00:00','military missile','success',969,'52080407','','','','OT',636,'',100 from dual union all
select 53220,'1981-W266','1981-04-14 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 53221,'1981-W267','1981-04-14 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',69 from dual union all
select 53222,'1981-W268','1981-04-14 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 53223,'1981-W269','1981-04-14 17:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 53224,'1981-W27','1981-01-21 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53225,'1981-W270','1981-04-14 17:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',72 from dual union all
select 53226,'1981-W271','1981-04-14 17:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 53227,'1981-W272','1981-04-14 19:08:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',67 from dual union all
select 53228,'1981-W273','1981-04-14 19:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',72 from dual union all
select 53229,'1981-W274','1981-04-14 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',52 from dual union all
select 5323,'1958-S333','1958-07-08 00:00:00','military missile','success',969,'52080410','','','','OT',636,'',100 from dual union all
select 53230,'1981-W275','1981-04-14 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 53231,'1981-W276','1981-04-15 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 53232,'1981-W277','1981-04-15 02:00:00','suborbital rocket','success',761,'MT135P-R419','','','','Meteo',1332,'',56 from dual union all
select 53233,'1981-W278','1981-04-15 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53234,'1981-W279','1981-04-15 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 53235,'1981-W28','1981-01-21 13:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',79 from dual union all
select 53236,'1981-W280','1981-04-15 20:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 53237,'1981-W281','1981-04-17 13:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',58 from dual union all
select 53238,'1981-W282','1981-04-17 20:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53239,'1981-W283','1981-04-17 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 5324,'1958-S334','1958-07-08 00:00:00','military missile','success',971,'M2-12','','','','Test',636,'',150 from dual union all
select 53240,'1981-W284','1981-04-20 20:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 53241,'1981-W285','1981-04-20 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 53242,'1981-W286','1981-04-21 14:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',67 from dual union all
select 53243,'1981-W287','1981-04-21 20:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 53244,'1981-W288','1981-04-22 02:03:59','suborbital rocket','success',761,'MT135P-R420','','','','Meteo',1332,'',56 from dual union all
select 53245,'1981-W289','1981-04-22 03:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual union all
select 53246,'1981-W29','1981-01-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',61 from dual union all
select 53247,'1981-W290','1981-04-22 04:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',58 from dual union all
select 53248,'1981-W291','1981-04-22 16:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53249,'1981-W292','1981-04-22 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 5325,'1958-S335','1958-07-15 00:00:00','military missile','success',969,'52080411','','','','OT',636,'',100 from dual union all
select 53250,'1981-W293','1981-04-22 21:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 53251,'1981-W294','1981-04-23 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 53252,'1981-W295','1981-04-24 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53253,'1981-W296','1981-04-24 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 53254,'1981-W297','1981-04-24 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 53255,'1981-W298','1981-04-24 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 53256,'1981-W299','1981-04-26 21:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',73 from dual union all
select 53257,'1981-W30','1981-01-21 18:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 53258,'1981-W300','1981-04-27 13:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',71 from dual union all
select 53259,'1981-W301','1981-04-27 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 5326,'1958-S336','1958-07-15 00:00:00','military missile','success',992,'52120417','','','','OT',636,'',170 from dual union all
select 53260,'1981-W302','1981-04-27 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 53261,'1981-W303','1981-04-27 14:20:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',76 from dual union all
select 53262,'1981-W304','1981-04-27 15:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 53263,'1981-W305','1981-04-27 16:35:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',79 from dual union all
select 53264,'1981-W306','1981-04-27 17:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',77 from dual union all
select 53265,'1981-W307','1981-04-27 20:13:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53266,'1981-W308','1981-04-29 02:00:00','suborbital rocket','success',761,'MT135P-R421','','','','Meteo',1332,'',55 from dual union all
select 53267,'1981-W309','1981-04-29 13:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',61 from dual union all
select 53268,'1981-W31','1981-01-22 02:00:00','suborbital rocket','success',761,'MT135P-R410','','','','Meteo',1332,'',57 from dual union all
select 53269,'1981-W310','1981-04-29 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 5327,'1958-S337','1958-07-24 00:00:00','military missile','success',969,'51120504','','','','OT',636,'',100 from dual union all
select 53270,'1981-W311','1981-04-29 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 53271,'1981-W312','1981-04-29 18:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',58 from dual union all
select 53272,'1981-W313','1981-04-29 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 53273,'1981-W314','1981-04-29 20:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 53274,'1981-W315','1981-04-30 20:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 53275,'1981-W316','1981-05-01 07:50:00','suborbital rocket','unknown',738,'','','','','Aeron',909,'KRENKEL''',58 from dual union all
select 53276,'1981-W317','1981-05-02 00:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 53277,'1981-W318','1981-05-04 15:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 53278,'1981-W319','1981-05-04 18:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 53279,'1981-W32','1981-01-22 15:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',62 from dual union all
select 5328,'1958-S338','1958-07-24 00:00:00','military missile','success',969,'51120106','','','','OT',636,'',100 from dual union all
select 53280,'1981-W320','1981-05-04 22:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 53281,'1981-W321','1981-05-05 10:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 53282,'1981-W322','1981-05-05 14:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 53283,'1981-W323','1981-05-06 08:30:00','suborbital rocket','unknown',738,'','','','','Aeron',909,'KRENKEL''',57 from dual union all
select 53284,'1981-W324','1981-05-06 15:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 53285,'1981-W325','1981-05-07 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 53286,'1981-W326','1981-05-08 00:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',67 from dual union all
select 53287,'1981-W327','1981-05-08 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',909,'KRENKEL''',55 from dual union all
select 53288,'1981-W328','1981-05-08 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',75 from dual;
insert into LAUNCH
select 53289,'1981-W329','1981-05-08 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 5329,'1958-S339','1958-07-25 00:00:00','military missile','success',969,'51120107','','','','OT',636,'',100 from dual union all
select 53290,'1981-W33','1981-01-22 16:17:00','suborbital rocket','unknown',1284,'FLIGHT 289     /T 1-','','','','Aeron',1821,'',67 from dual union all
select 53291,'1981-W330','1981-05-08 20:24:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',79 from dual union all
select 53292,'1981-W331','1981-05-08 20:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 53293,'1981-W332','1981-05-11 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53294,'1981-W333','1981-05-11 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 53295,'1981-W334','1981-05-11 15:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 53296,'1981-W335','1981-05-11 16:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',59 from dual union all
select 53297,'1981-W336','1981-05-12 18:10:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',76 from dual union all
select 53298,'1981-W337','1981-05-13 02:15:59','suborbital rocket','success',761,'MT135P-R422','','','','Meteo',1332,'',58 from dual union all
select 53299,'1981-W338','1981-05-13 10:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 533,'1945-S09','1945-01-03 17:05:00','military missile','success',1470,'20851','Ma424','','','Test',661,'',90 from dual union all
select 5330,'1958-S34','1958-02-24 07:34:42','suborbital rocket','success',839,'AM6.37','','IGY AM6.37','','Aeron',428,'',145 from dual union all
select 53300,'1981-W339','1981-05-13 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 53301,'1981-W34','1981-01-23 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53302,'1981-W340','1981-05-13 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',60 from dual union all
select 53303,'1981-W341','1981-05-13 15:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',53 from dual union all
select 53304,'1981-W342','1981-05-13 18:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 53305,'1981-W343','1981-05-14 12:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53306,'1981-W344','1981-05-14 19:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 53307,'1981-W345','1981-05-14 19:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 53308,'1981-W346','1981-05-15 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',909,'KRENKEL''',57 from dual union all
select 53309,'1981-W347','1981-05-15 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 5331,'1958-S340','1958-07-10 00:00:00','military missile','success',1013,'M5-1TSK','8K51 UBP-6','VV GCh','','Test',636,'',500 from dual union all
select 53310,'1981-W348','1981-05-15 21:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 53311,'1981-W349','1981-05-18 10:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53312,'1981-W35','1981-01-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',74 from dual union all
select 53313,'1981-W350','1981-05-18 13:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',66 from dual union all
select 53314,'1981-W351','1981-05-18 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',76 from dual union all
select 53315,'1981-W352','1981-05-19 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',79 from dual union all
select 53316,'1981-W353','1981-05-19 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 53317,'1981-W354','1981-05-20 02:00:00','suborbital rocket','success',761,'MT135P-R423','','','','Meteo',1332,'',56 from dual union all
select 53318,'1981-W355','1981-05-20 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',909,'KRENKEL''',59 from dual union all
select 53319,'1981-W356','1981-05-20 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 5332,'1958-S341','1958-07-11 00:00:00','military missile','success',1013,'M5-9TSK','8K51 UBP-7','VV GCh','','Test',636,'',500 from dual union all
select 53320,'1981-W357','1981-05-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',78 from dual union all
select 53321,'1981-W358','1981-05-20 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 53322,'1981-W359','1981-05-20 17:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 53323,'1981-W36','1981-01-23 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 53324,'1981-W360','1981-05-20 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 53325,'1981-W361','1981-05-21 21:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 53326,'1981-W362','1981-05-22 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',909,'KRENKEL''',58 from dual union all
select 53327,'1981-W363','1981-05-22 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 53328,'1981-W364','1981-05-22 21:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',68 from dual union all
select 53329,'1981-W365','1981-05-22 21:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 5333,'1958-S342','1958-07-29 00:00:00','military missile','success',1013,'A2-2','8K51 UBP-8','No BRK-2 guidance','','Test',636,'',500 from dual union all
select 53330,'1981-W366','1981-05-24 08:30:00','suborbital rocket','unknown',738,'','','','','Aeron',909,'KRENKEL''',56 from dual union all
select 53331,'1981-W367','1981-05-25 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 53332,'1981-W368','1981-05-25 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 53333,'1981-W369','1981-05-26 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',77 from dual union all
select 53334,'1981-W37','1981-01-26 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',67 from dual union all
select 53335,'1981-W370','1981-05-26 20:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 53336,'1981-W371','1981-05-26 21:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 53337,'1981-W372','1981-05-27 02:03:00','suborbital rocket','success',761,'MT135P-R424','','','','Meteo',1332,'',57 from dual union all
select 53338,'1981-W373','1981-05-27 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',909,'KRENKEL''',58 from dual union all
select 53339,'1981-W374','1981-05-27 10:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 5334,'1958-S343','1958-07-29 00:00:00','military missile','failure',992,'51120303','','','','OT',636,'',40 from dual union all
select 53340,'1981-W375','1981-05-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',63 from dual union all
select 53341,'1981-W376','1981-05-27 14:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 53342,'1981-W377','1981-05-27 15:15:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',75 from dual union all
select 53343,'1981-W378','1981-05-27 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 53344,'1981-W379','1981-05-27 18:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 53345,'1981-W38','1981-01-26 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 53346,'1981-W380','1981-05-27 20:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 53347,'1981-W381','1981-05-28 00:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 53348,'1981-W382','1981-05-28 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 53349,'1981-W383','1981-05-29 08:40:00','suborbital rocket','unknown',738,'','','','','Aeron',909,'KRENKEL''',57 from dual union all
select 5335,'1958-S344','1958-08-16 00:00:00','military missile','success',969,'52090501','','','','OT',636,'',100 from dual union all
select 53350,'1981-W384','1981-05-29 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53351,'1981-W385','1981-05-29 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',54 from dual union all
select 53352,'1981-W386','1981-05-29 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 53353,'1981-W387','1981-05-29 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 53354,'1981-W388','1981-05-30 00:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 53355,'1981-W389','1981-05-30 00:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 53356,'1981-W39','1981-01-26 15:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',58 from dual union all
select 53357,'1981-W390','1981-06-01 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53358,'1981-W391','1981-06-01 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 53359,'1981-W392','1981-06-01 16:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',55 from dual union all
select 5336,'1958-S345','1958-09-01 00:00:00','military missile','success',992,'M2-18','','','','OT',636,'',170 from dual union all
select 53360,'1981-W393','1981-06-01 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 53361,'1981-W394','1981-06-01 20:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 53362,'1981-W395','1981-06-02 15:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 53363,'1981-W396','1981-06-03 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',210,'BUGAEV',59 from dual union all
select 53364,'1981-W397','1981-06-03 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 53365,'1981-W398','1981-06-03 19:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',62 from dual union all
select 53366,'1981-W399','1981-06-03 21:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 53367,'1981-W40','1981-01-28 02:29:00','suborbital rocket','success',761,'MT135P-R411','','','','Meteo',1332,'',58 from dual union all
select 53368,'1981-W400','1981-06-03 22:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 53369,'1981-W401','1981-06-04 09:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 5337,'1958-S346','1958-09-01 00:00:00','military missile','success',992,'K3-3','','','','OT',636,'',170 from dual union all
select 53370,'1981-W402','1981-06-04 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 53371,'1981-W403','1981-06-05 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',210,'BUGAEV',59 from dual union all
select 53372,'1981-W404','1981-06-05 12:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 53373,'1981-W405','1981-06-05 14:14:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 53374,'1981-W406','1981-06-05 19:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 53375,'1981-W407','1981-06-05 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 53376,'1981-W408','1981-06-08 21:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 53377,'1981-W409','1981-06-09 09:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53378,'1981-W41','1981-01-28 12:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual;
insert into LAUNCH
select 53379,'1981-W410','1981-06-09 14:23:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 5338,'1958-S347','1958-09-08 00:00:00','military missile','success',969,'D2115','','','','OT',636,'',100 from dual union all
select 53380,'1981-W411','1981-06-09 17:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',62 from dual union all
select 53381,'1981-W412','1981-06-10 02:00:00','suborbital rocket','success',761,'MT135P-R425','','','','Meteo',1332,'',55 from dual union all
select 53382,'1981-W413','1981-06-10 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',210,'BUGAEV',57 from dual union all
select 53383,'1981-W414','1981-06-10 13:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53384,'1981-W415','1981-06-10 17:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 53385,'1981-W416','1981-06-10 19:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 53386,'1981-W417','1981-06-11 12:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53387,'1981-W418','1981-06-11 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',68 from dual union all
select 53388,'1981-W419','1981-06-12 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',210,'BUGAEV',58 from dual union all
select 53389,'1981-W42','1981-01-28 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',58 from dual union all
select 5339,'1958-S348','1958-09-08 00:00:00','military missile','success',992,'M2-3D','','','','OT',636,'',170 from dual union all
select 53390,'1981-W420','1981-06-12 21:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 53391,'1981-W421','1981-06-16 00:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 53392,'1981-W422','1981-06-16 12:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53393,'1981-W423','1981-06-17 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',210,'BUGAEV',54 from dual union all
select 53394,'1981-W424','1981-06-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',79 from dual union all
select 53395,'1981-W425','1981-06-17 20:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 53396,'1981-W426','1981-06-17 21:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 53397,'1981-W427','1981-06-18 09:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53398,'1981-W428','1981-06-18 20:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 53399,'1981-W429','1981-06-19 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',210,'BUGAEV',57 from dual union all
select 534,'1945-S10','1945-01-04 11:15:00','military missile','success',1470,'20823','Ma417','','','Test',661,'',90 from dual union all
select 5340,'1958-S349','1958-09-13 00:00:00','military missile','success',992,'K2-16','','','','OT',636,'',170 from dual union all
select 53400,'1981-W43','1981-01-28 23:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 53401,'1981-W430','1981-06-19 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 53402,'1981-W431','1981-06-22 04:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual union all
select 53403,'1981-W432','1981-06-22 06:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',52 from dual union all
select 53404,'1981-W433','1981-06-22 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',53 from dual union all
select 53405,'1981-W434','1981-06-22 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',73 from dual union all
select 53406,'1981-W435','1981-06-22 12:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53407,'1981-W436','1981-06-22 14:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 53408,'1981-W437','1981-06-22 19:55:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 53409,'1981-W438','1981-06-22 21:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 5341,'1958-S35','1958-02-26 02:35:00','suborbital rocket','success',5,'','','','','Ionos/Aeron',1010,'',100 from dual union all
select 53410,'1981-W439','1981-06-23 05:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual union all
select 53411,'1981-W44','1981-01-30 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53412,'1981-W440','1981-06-23 06:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',72 from dual union all
select 53413,'1981-W441','1981-06-23 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',79 from dual union all
select 53414,'1981-W442','1981-06-24 02:00:00','suborbital rocket','success',761,'MT135P-R426','','','','Meteo',1332,'',54 from dual union all
select 53415,'1981-W443','1981-06-24 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',210,'BUGAEV',57 from dual union all
select 53416,'1981-W444','1981-06-24 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53417,'1981-W445','1981-06-24 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',65 from dual union all
select 53418,'1981-W446','1981-06-24 16:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 53419,'1981-W447','1981-06-24 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 5342,'1958-S350','1958-09-16 00:00:00','military missile','success',992,'A3-6','','','','OT',636,'',170 from dual union all
select 53420,'1981-W448','1981-06-24 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 53421,'1981-W449','1981-06-24 17:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',69 from dual union all
select 53422,'1981-W45','1981-01-30 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 53423,'1981-W450','1981-06-25 19:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 53424,'1981-W451','1981-06-26 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',210,'BUGAEV',57 from dual union all
select 53425,'1981-W452','1981-06-26 09:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53426,'1981-W453','1981-06-26 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 53427,'1981-W454','1981-06-26 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 53428,'1981-W455','1981-06-28 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',210,'BUGAEV',57 from dual union all
select 53429,'1981-W456','1981-06-29 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 5343,'1958-S351','1958-09-16 00:00:00','military missile','failure',992,'A3-6 (DUP)','','','','OT',636,'',170 from dual union all
select 53430,'1981-W457','1981-06-29 22:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 53431,'1981-W458','1981-06-30 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 53432,'1981-W459','1981-06-30 21:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 53433,'1981-W46','1981-02-02 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 53434,'1981-W460','1981-07-01 09:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53435,'1981-W461','1981-07-01 21:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 53436,'1981-W462','1981-07-01 22:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 53437,'1981-W463','1981-07-02 00:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 53438,'1981-W464','1981-07-02 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',75 from dual union all
select 53439,'1981-W465','1981-07-02 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 5344,'1958-S352','1958-09-08 00:00:00','military missile','success',1013,'M3-18','8K51 UBP-9','','','Test',636,'',500 from dual union all
select 53440,'1981-W466','1981-07-02 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 53441,'1981-W467','1981-07-03 12:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53442,'1981-W468','1981-07-06 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53443,'1981-W469','1981-07-06 14:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 53444,'1981-W47','1981-02-02 18:08:00','suborbital rocket','unknown',1284,'FLIGHT 290     /TH1-','','','','Aeron',1307,'',65 from dual union all
select 53445,'1981-W470','1981-07-06 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 53446,'1981-W471','1981-07-06 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53447,'1981-W472','1981-07-08 00:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 53448,'1981-W473','1981-07-08 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',77 from dual union all
select 53449,'1981-W474','1981-07-08 02:00:00','suborbital rocket','success',761,'MT135P-R427','','','','Meteo',1332,'',57 from dual union all
select 5345,'1958-S353','1958-09-01 00:00:00','military missile','success',971,'B2-23','','','','Test',636,'',150 from dual union all
select 53450,'1981-W475','1981-07-08 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 53451,'1981-W476','1981-07-08 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 53452,'1981-W477','1981-07-08 18:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 53453,'1981-W478','1981-07-08 21:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 53454,'1981-W479','1981-07-09 12:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53455,'1981-W48','1981-02-02 19:55:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 53456,'1981-W480','1981-07-09 13:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 53457,'1981-W481','1981-07-09 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 53458,'1981-W482','1981-07-09 21:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 53459,'1981-W483','1981-07-10 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 5346,'1958-S354','1958-09-08 00:00:00','military missile','success',971,'M2-9','','','','Test',636,'',150 from dual union all
select 53460,'1981-W484','1981-07-10 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 53461,'1981-W485','1981-07-14 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 53462,'1981-W486','1981-07-14 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 53463,'1981-W487','1981-07-15 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53464,'1981-W488','1981-07-15 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 53465,'1981-W489','1981-07-16 18:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 53466,'1981-W49','1981-02-03 12:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53467,'1981-W490','1981-07-16 22:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 53468,'1981-W491','1981-07-17 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual;
insert into LAUNCH
select 53469,'1981-W492','1981-07-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',79 from dual union all
select 5347,'1958-S355','1958-09-18 00:00:00','military missile','success',971,'B2-8','','','','Test',636,'',150 from dual union all
select 53470,'1981-W493','1981-07-17 20:52:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',68 from dual union all
select 53471,'1981-W494','1981-07-17 21:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',69 from dual union all
select 53472,'1981-W495','1981-07-17 21:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 53473,'1981-W496','1981-07-20 12:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53474,'1981-W497','1981-07-20 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 53475,'1981-W498','1981-07-20 20:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',68 from dual union all
select 53476,'1981-W499','1981-07-22 09:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 53477,'1981-W50','1981-02-04 02:04:59','suborbital rocket','success',761,'MT135P-R412','','','','Meteo',1332,'',59 from dual union all
select 53478,'1981-W500','1981-07-22 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',69 from dual union all
select 53479,'1981-W501','1981-07-22 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',57 from dual union all
select 5348,'1958-S356','1958-07-31 00:00:00','military missile','success',971,'M2-21','','','','Test',636,'',150 from dual union all
select 53480,'1981-W502','1981-07-22 21:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 53481,'1981-W503','1981-07-23 02:00:00','suborbital rocket','success',761,'MT135P-R428','','','','Meteo',1332,'',53 from dual union all
select 53482,'1981-W504','1981-07-23 19:18:59','suborbital rocket','success',1284,'','Ozone','','','Meteo',1821,'',75 from dual union all
select 53483,'1981-W505','1981-07-23 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 53484,'1981-W506','1981-07-23 20:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 53485,'1981-W507','1981-07-24 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 53486,'1981-W508','1981-07-24 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 53487,'1981-W509','1981-07-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',78 from dual union all
select 53488,'1981-W51','1981-02-04 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',908,'KRENKEL''',54 from dual union all
select 53489,'1981-W510','1981-07-24 22:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 5349,'1958-S357','1958-09-18 00:00:00','military missile','success',971,'B2-2','','','','Test',636,'',150 from dual union all
select 53490,'1981-W511','1981-07-25 00:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 53491,'1981-W512','1981-07-27 09:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53492,'1981-W513','1981-07-27 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 53493,'1981-W514','1981-07-27 20:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 53494,'1981-W515','1981-07-27 21:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 53495,'1981-W516','1981-07-29 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53496,'1981-W517','1981-07-29 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 53497,'1981-W518','1981-07-29 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 53498,'1981-W519','1981-07-30 01:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 53499,'1981-W52','1981-02-04 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 535,'1945-S11','1945-01-08 12:57:00','military missile','success',1470,'20843','Ma419','','','Test',661,'',90 from dual union all
select 5350,'1958-S358','1958-09-23 00:00:00','military missile','success',971,'B2-4','','','','Test',636,'',150 from dual union all
select 53500,'1981-W520','1981-07-30 22:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 53501,'1981-W521','1981-07-31 12:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53502,'1981-W522','1981-07-31 13:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',69 from dual union all
select 53503,'1981-W523','1981-07-31 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',79 from dual union all
select 53504,'1981-W524','1981-07-31 14:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 53505,'1981-W525','1981-07-31 20:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 53506,'1981-W526','1981-07-31 21:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 53507,'1981-W527','1981-08-03 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53508,'1981-W528','1981-08-04 17:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',55 from dual union all
select 53509,'1981-W529','1981-08-04 20:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 5351,'1958-S359','1958-08-01 00:00:00','military missile','success',971,'M2-7','','','','Test',636,'',150 from dual union all
select 53510,'1981-W53','1981-02-04 16:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 53511,'1981-W530','1981-08-05 09:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',55 from dual union all
select 53512,'1981-W531','1981-08-05 12:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',59 from dual union all
select 53513,'1981-W532','1981-08-05 13:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 53514,'1981-W533','1981-08-05 17:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 53515,'1981-W534','1981-08-06 02:07:00','suborbital rocket','success',761,'MT135P-R429','','','','Meteo',1332,'',54 from dual union all
select 53516,'1981-W535','1981-08-06 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 53517,'1981-W536','1981-08-07 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 53518,'1981-W537','1981-08-07 16:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 53519,'1981-W538','1981-08-07 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 5352,'1958-S36','1958-02-25 05:48:32','suborbital rocket','success',839,'II6.27F','Auroral particles','IGY II6.27F','','Auroral',428,'',129 from dual union all
select 53520,'1981-W539','1981-08-07 22:08:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 53521,'1981-W54','1981-02-04 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 53522,'1981-W540','1981-08-09 02:06:00','suborbital rocket','success',1281,'NASA 15.211UE','E field/conduct.','','','Plasma',1821,'',75 from dual union all
select 53523,'1981-W541','1981-08-10 13:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 53524,'1981-W542','1981-08-10 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 53525,'1981-W543','1981-08-10 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 53526,'1981-W544','1981-08-11 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 53527,'1981-W545','1981-08-13 04:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53528,'1981-W546','1981-08-13 18:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 53529,'1981-W547','1981-08-13 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 5353,'1958-S360','1958-08-06 00:00:00','military missile','success',971,'M2-22','','','','Test',636,'',150 from dual union all
select 53530,'1981-W548','1981-08-14 09:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',69 from dual union all
select 53531,'1981-W549','1981-08-14 13:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 53532,'1981-W55','1981-02-04 18:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 53533,'1981-W550','1981-08-14 16:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 53534,'1981-W551','1981-08-17 12:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',71 from dual union all
select 53535,'1981-W552','1981-08-17 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',57 from dual union all
select 53536,'1981-W553','1981-08-17 21:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',65 from dual union all
select 53537,'1981-W554','1981-08-18 09:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 53538,'1981-W555','1981-08-18 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 53539,'1981-W556','1981-08-18 20:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 5354,'1958-S361','1958-09-23 00:00:00','military missile','success',1013,'M4-26','8K51 UBP-10','','','Test',636,'',500 from dual union all
select 53540,'1981-W557','1981-08-19 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',68 from dual union all
select 53541,'1981-W558','1981-08-20 02:00:00','suborbital rocket','success',761,'MT135P-R430','','','','Meteo',1332,'',54 from dual union all
select 53542,'1981-W559','1981-08-21 13:42:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',69 from dual union all
select 53543,'1981-W56','1981-02-04 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 53544,'1981-W560','1981-08-21 16:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 53545,'1981-W561','1981-08-21 20:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 53546,'1981-W562','1981-08-22 00:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 53547,'1981-W563','1981-08-24 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',70 from dual union all
select 53548,'1981-W564','1981-08-24 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 53549,'1981-W565','1981-08-24 17:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 5355,'1958-S362','1958-09-23 00:00:00','military missile','success',1013,'M5-2TSK','8K51 UBP-11','','','Test',636,'',500 from dual union all
select 53550,'1981-W566','1981-08-24 22:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 53551,'1981-W567','1981-08-26 20:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',53 from dual union all
select 53552,'1981-W568','1981-08-26 20:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 53553,'1981-W569','1981-08-26 21:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 53554,'1981-W57','1981-02-04 22:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53555,'1981-W570','1981-08-28 02:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',69 from dual union all
select 53556,'1981-W571','1981-08-28 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 53557,'1981-W572','1981-08-31 09:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 53558,'1981-W573','1981-08-31 13:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual;
insert into LAUNCH
select 53559,'1981-W574','1981-08-31 17:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 5356,'1958-S363','1958-09-23 00:00:00','military missile','success',1013,'M4-17','8K51 UBP-12','','','Test',636,'',500 from dual union all
select 53560,'1981-W575','1981-08-31 20:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 53561,'1981-W576','1981-08-31 20:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 53562,'1981-W577','1981-09-01 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53563,'1981-W578','1981-09-02 01:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 53564,'1981-W579','1981-09-02 02:20:00','suborbital rocket','success',761,'MT135P-R431','','','','Meteo',1332,'',56 from dual union all
select 53565,'1981-W58','1981-02-04 22:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 53566,'1981-W580','1981-09-02 13:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',69 from dual union all
select 53567,'1981-W581','1981-09-02 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',66 from dual union all
select 53568,'1981-W582','1981-09-02 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',79 from dual union all
select 53569,'1981-W583','1981-09-02 18:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 5357,'1958-S364','1958-09-24 00:00:00','military missile','success',1013,'B8-20T','8K51 UBP-13','','','Test',636,'',500 from dual union all
select 53570,'1981-W584','1981-09-04 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',976,'MUSSON',60 from dual union all
select 53571,'1981-W585','1981-09-04 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',71 from dual union all
select 53572,'1981-W586','1981-09-04 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 53573,'1981-W587','1981-09-04 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 53574,'1981-W588','1981-09-04 17:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',70 from dual union all
select 53575,'1981-W589','1981-09-04 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 53576,'1981-W59','1981-02-05 09:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53577,'1981-W590','1981-09-04 22:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53578,'1981-W591','1981-09-08 13:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',72 from dual union all
select 53579,'1981-W592','1981-09-08 20:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',72 from dual union all
select 5358,'1958-S365','1958-09-30 00:00:00','military missile','success',971,'B2-11','','','','Test',636,'',150 from dual union all
select 53580,'1981-W593','1981-09-08 20:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 53581,'1981-W594','1981-09-08 22:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 53582,'1981-W595','1981-09-09 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',976,'MUSSON',60 from dual union all
select 53583,'1981-W596','1981-09-09 09:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',71 from dual union all
select 53584,'1981-W597','1981-09-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',73 from dual union all
select 53585,'1981-W598','1981-09-09 17:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',55 from dual union all
select 53586,'1981-W599','1981-09-10 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 53587,'1981-W60','1981-02-05 14:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 53588,'1981-W600','1981-09-10 22:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',68 from dual union all
select 53589,'1981-W601','1981-09-11 07:40:00','suborbital rocket','unknown',738,'','','','','Aeron',976,'MUSSON',60 from dual union all
select 5359,'1958-S366','1958-02-12 00:00:00','military missile','success',1013,'M5-3TSK','VV GCh','','','Test',636,'',500 from dual union all
select 53590,'1981-W602','1981-09-11 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',68 from dual union all
select 53591,'1981-W603','1981-09-11 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 53592,'1981-W604','1981-09-11 19:42:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 53593,'1981-W605','1981-09-12 05:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 53594,'1981-W606','1981-09-12 08:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',67 from dual union all
select 53595,'1981-W607','1981-09-13 16:04:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',61 from dual union all
select 53596,'1981-W608','1981-09-13 16:05:00','suborbital rocket','success',1284,'F-293/T 1-6617','','','','Aeron',1821,'',66 from dual union all
select 53597,'1981-W609','1981-09-14 12:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',70 from dual union all
select 53598,'1981-W61','1981-02-05 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 53599,'1981-W610','1981-09-14 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 536,'1945-S12','1945-01-08 15:35:00','military missile','success',1470,'20819','Ma416','','','Test',661,'',90 from dual union all
select 5360,'1958-S367','1958-10-05 00:00:00','military missile','success',975,'B1-6T','','','','Test',636,'',80 from dual union all
select 53600,'1981-W611','1981-09-14 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53601,'1981-W612','1981-09-15 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',71 from dual union all
select 53602,'1981-W613','1981-09-15 15:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 53603,'1981-W614','1981-09-15 22:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 53604,'1981-W615','1981-09-16 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',976,'MUSSON',60 from dual union all
select 53605,'1981-W616','1981-09-16 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 53606,'1981-W617','1981-09-16 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 53607,'1981-W618','1981-09-17 18:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 53608,'1981-W619','1981-09-17 21:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 53609,'1981-W62','1981-02-06 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',908,'KRENKEL''',58 from dual union all
select 5361,'1958-S368','1958-10-08 00:00:00','military missile','success',975,'B1-11','','','','Test',636,'',80 from dual union all
select 53610,'1981-W620','1981-09-21 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',58 from dual union all
select 53611,'1981-W621','1981-09-21 22:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53612,'1981-W622','1981-09-21 22:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 53613,'1981-W623','1981-09-22 09:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53614,'1981-W624','1981-09-22 13:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 53615,'1981-W625','1981-09-22 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 53616,'1981-W626','1981-09-22 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',64 from dual union all
select 53617,'1981-W627','1981-09-22 21:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 53618,'1981-W628','1981-09-22 21:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 53619,'1981-W629','1981-09-23 13:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 5362,'1958-S369','1958-10-30 00:00:00','military missile','success',975,'B2-3','','','','Test',636,'',80 from dual union all
select 53620,'1981-W63','1981-02-06 08:45:00','suborbital rocket','unknown',738,'','','','','Aeron',908,'KRENKEL''',59 from dual union all
select 53621,'1981-W630','1981-09-23 18:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 53622,'1981-W631','1981-09-24 00:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 53623,'1981-W632','1981-09-25 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',976,'MUSSON',61 from dual union all
select 53624,'1981-W633','1981-09-25 13:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53625,'1981-W634','1981-09-28 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 53626,'1981-W635','1981-09-28 13:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 53627,'1981-W636','1981-09-28 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 53628,'1981-W637','1981-09-28 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 53629,'1981-W638','1981-09-28 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 5363,'1958-S37','1958-02-27 19:59:00','military missile','success',1053,'CC-43    TN','','','','Test',280,'',90 from dual union all
select 53630,'1981-W639','1981-09-29 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 53631,'1981-W64','1981-02-06 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 53632,'1981-W640','1981-09-30 00:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 53633,'1981-W641','1981-09-30 02:00:00','suborbital rocket','success',761,'MT135P-R433','','','','Meteo',1332,'',54 from dual union all
select 53634,'1981-W642','1981-09-30 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',976,'MUSSON',60 from dual union all
select 53635,'1981-W643','1981-09-30 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53636,'1981-W644','1981-09-30 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 53637,'1981-W645','1981-09-30 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 53638,'1981-W646','1981-10-01 15:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 53639,'1981-W647','1981-10-01 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 5364,'1958-S370','1958-10-31 00:00:00','military missile','success',975,'B2-9T','','','','Test',636,'',80 from dual union all
select 53640,'1981-W648','1981-10-01 21:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53641,'1981-W649','1981-10-02 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',976,'MUSSON',57 from dual union all
select 53642,'1981-W65','1981-02-06 20:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',61 from dual union all
select 53643,'1981-W650','1981-10-02 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',57 from dual union all
select 53644,'1981-W651','1981-10-02 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 53645,'1981-W652','1981-10-02 14:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',62 from dual union all
select 53646,'1981-W653','1981-10-02 20:34:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 53647,'1981-W654','1981-10-04 05:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 53648,'1981-W655','1981-10-04 09:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual;
insert into LAUNCH
select 53649,'1981-W656','1981-10-05 09:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 5365,'1958-S371','1958-12-19 00:00:00','suborbital rocket','success',1013,'B1-5 8K52','8K52 Zvezda-3','Zvezda','','Ionos',636,'',460 from dual union all
select 53650,'1981-W657','1981-10-05 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 53651,'1981-W658','1981-10-05 16:10:00','suborbital rocket','unknown',1284,'FLIGHT 294     /T 1-','','','','Aeron',1821,'',67 from dual union all
select 53652,'1981-W659','1981-10-05 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 53653,'1981-W66','1981-02-06 21:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 53654,'1981-W660','1981-10-05 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 53655,'1981-W661','1981-10-06 12:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53656,'1981-W662','1981-10-06 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 53657,'1981-W663','1981-10-07 02:00:00','suborbital rocket','success',761,'MT135P-R434','','','','Meteo',1332,'',51 from dual union all
select 53658,'1981-W664','1981-10-07 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',976,'MUSSON',60 from dual union all
select 53659,'1981-W665','1981-10-07 19:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 5366,'1958-S372','1958-11-28 00:00:00','suborbital rocket','success',1013,'B1-3 8K52','8K52 Zvezda-2','Zvezda','','Ionos',636,'',460 from dual union all
select 53660,'1981-W666','1981-10-07 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 53661,'1981-W667','1981-10-09 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 53662,'1981-W668','1981-10-09 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',54 from dual union all
select 53663,'1981-W669','1981-10-09 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 53664,'1981-W67','1981-02-06 22:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 53665,'1981-W670','1981-10-12 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53666,'1981-W671','1981-10-13 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 53667,'1981-W672','1981-10-13 15:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 53668,'1981-W673','1981-10-13 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 53669,'1981-W674','1981-10-13 21:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 5367,'1958-S373','1958-12-02 00:00:00','military missile','success',1013,'B10-20T','8K51 UBP-15','','','Test',636,'',500 from dual union all
select 53670,'1981-W675','1981-10-14 09:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53671,'1981-W676','1981-10-14 12:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 53672,'1981-W677','1981-10-14 22:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 53673,'1981-W678','1981-10-14 23:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 53674,'1981-W679','1981-10-16 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53675,'1981-W68','1981-02-09 10:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53676,'1981-W680','1981-10-16 20:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53677,'1981-W681','1981-10-19 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 53678,'1981-W682','1981-10-20 14:15:00','suborbital rocket','success',1061,'','','','','Meteo',276,'',59 from dual union all
select 53679,'1981-W683','1981-10-20 18:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',65 from dual union all
select 5368,'1958-S374','1958-11-21 00:00:00','military missile','success',975,'B3-11','','','','Test',636,'',80 from dual union all
select 53680,'1981-W684','1981-10-21 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',211,'BUGAEV',57 from dual union all
select 53681,'1981-W685','1981-10-21 13:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 53682,'1981-W686','1981-10-21 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 53683,'1981-W687','1981-10-21 15:00:00','suborbital rocket','success',1284,'','','','','Meteo',1895,'',60 from dual union all
select 53684,'1981-W688','1981-10-21 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 53685,'1981-W689','1981-10-22 17:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',50 from dual union all
select 53686,'1981-W69','1981-02-09 17:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',57 from dual union all
select 53687,'1981-W690','1981-10-23 12:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53688,'1981-W691','1981-10-23 14:24:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 53689,'1981-W692','1981-10-26 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 5369,'1958-S375','1958-11-28 00:00:00','military missile','success',975,'B1-3','','','','Test',636,'',80 from dual union all
select 53690,'1981-W693','1981-10-26 21:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 53691,'1981-W694','1981-10-27 14:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 53692,'1981-W695','1981-10-27 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',75 from dual union all
select 53693,'1981-W696','1981-10-28 02:00:00','suborbital rocket','success',761,'MT135P-R436','','','','Meteo',1332,'',54 from dual union all
select 53694,'1981-W697','1981-10-28 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',211,'BUGAEV',61 from dual union all
select 53695,'1981-W698','1981-10-28 10:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53696,'1981-W699','1981-10-28 14:10:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',62 from dual union all
select 53697,'1981-W70','1981-02-09 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',50 from dual union all
select 53698,'1981-W700','1981-10-28 18:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 53699,'1981-W701','1981-10-28 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 537,'1945-S13','1945-01-10 12:19:00','military missile','success',8,'20805','Ma400','','','Test',661,'',90 from dual union all
select 5370,'1958-S376','1958-12-02 00:00:00','military missile','success',975,'B3-6T','','','','Test',636,'',80 from dual union all
select 53700,'1981-W702','1981-10-28 23:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 53701,'1981-W703','1981-10-30 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53702,'1981-W704','1981-10-30 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',71 from dual union all
select 53703,'1981-W705','1981-10-30 17:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 53704,'1981-W706','1981-10-30 21:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 53705,'1981-W707','1981-11-02 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 53706,'1981-W708','1981-11-02 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 53707,'1981-W709','1981-11-02 22:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 53708,'1981-W71','1981-02-09 22:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 53709,'1981-W710','1981-11-03 12:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 5371,'1958-S377','1958-12-24 16:00:00','military missile','failure',1016,'B3-16','8K71 LKI III-1','','','Test',480,'',200 from dual union all
select 53710,'1981-W711','1981-11-04 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53711,'1981-W712','1981-11-04 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 53712,'1981-W713','1981-11-04 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 53713,'1981-W714','1981-11-04 22:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 53714,'1981-W715','1981-11-05 09:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53715,'1981-W716','1981-11-06 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',211,'BUGAEV',61 from dual union all
select 53716,'1981-W717','1981-11-06 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 53717,'1981-W718','1981-11-06 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 53718,'1981-W719','1981-11-06 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53719,'1981-W72','1981-02-10 14:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1300,'PRILIV',60 from dual union all
select 5372,'1958-S378','1958-12-27 00:00:00','military missile','success',971,'B1-8T','','','','Test',636,'',150 from dual union all
select 53720,'1981-W720','1981-11-09 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',58 from dual union all
select 53721,'1981-W721','1981-11-09 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 53722,'1981-W722','1981-11-10 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 53723,'1981-W723','1981-11-11 02:02:00','suborbital rocket','success',761,'MT135P-R437','','','','Meteo',1332,'',52 from dual union all
select 53724,'1981-W724','1981-11-11 07:40:00','suborbital rocket','unknown',738,'','','','','Aeron',211,'BUGAEV',56 from dual union all
select 53725,'1981-W725','1981-11-12 14:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 53726,'1981-W726','1981-11-12 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',68 from dual union all
select 53727,'1981-W727','1981-11-12 18:29:00','suborbital rocket','unknown',1284,'FLIGHT 295     /T 1-','','','','Aeron',1821,'',75 from dual union all
select 53728,'1981-W728','1981-11-12 18:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 53729,'1981-W729','1981-11-12 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',71 from dual union all
select 5373,'1958-S379','1958-12-30 00:00:00','military missile','success',975,'B4-10','','','','Test',636,'',80 from dual union all
select 53730,'1981-W73','1981-02-10 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 53731,'1981-W730','1981-11-12 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 53732,'1981-W731','1981-11-12 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 53733,'1981-W732','1981-11-13 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 53734,'1981-W733','1981-11-13 18:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 53735,'1981-W734','1981-11-13 20:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 53736,'1981-W735','1981-11-13 21:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 53737,'1981-W736','1981-11-14 16:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 53738,'1981-W737','1981-11-14 18:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual;
insert into LAUNCH
select 53739,'1981-W738','1981-11-14 20:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 5374,'1958-S38','1958-03-19 01:12:00','suborbital rocket','success',28,'USAF-93','NO/O2 composition','IGY AA1.25','','Aeron',640,'',93 from dual union all
select 53740,'1981-W739','1981-11-14 20:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 53741,'1981-W74','1981-02-10 23:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 53742,'1981-W740','1981-11-14 20:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 53743,'1981-W741','1981-11-14 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',64 from dual union all
select 53744,'1981-W742','1981-11-14 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 53745,'1981-W743','1981-11-14 21:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 53746,'1981-W744','1981-11-14 21:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 53747,'1981-W745','1981-11-14 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',66 from dual union all
select 53748,'1981-W746','1981-11-14 23:12:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 53749,'1981-W747','1981-11-16 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 5375,'1958-S380','1958-12-30 00:00:00','military missile','success',975,'B4-2T','','','','Test',636,'',80 from dual union all
select 53750,'1981-W748','1981-11-16 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 53751,'1981-W749','1981-11-17 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 53752,'1981-W75','1981-02-11 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53753,'1981-W750','1981-11-18 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',211,'BUGAEV',62 from dual union all
select 53754,'1981-W751','1981-11-18 09:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53755,'1981-W752','1981-11-18 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 53756,'1981-W753','1981-11-18 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',73 from dual union all
select 53757,'1981-W754','1981-11-18 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',54 from dual union all
select 53758,'1981-W755','1981-11-18 22:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',58 from dual union all
select 53759,'1981-W756','1981-11-19 02:00:00','suborbital rocket','success',761,'MT135P-R438','','','','Meteo',1332,'',56 from dual union all
select 5376,'1958-S381','1958-02-04 00:00:00','military missile','success',1013,'B1-20T','8K51 OT-11','','','Test',636,'',500 from dual union all
select 53760,'1981-W757','1981-11-20 12:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 53761,'1981-W758','1981-11-20 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',56 from dual union all
select 53762,'1981-W759','1981-11-20 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 53763,'1981-W76','1981-02-11 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 53764,'1981-W760','1981-11-20 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 53765,'1981-W761','1981-11-23 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53766,'1981-W762','1981-11-23 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',54 from dual union all
select 53767,'1981-W763','1981-11-23 16:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',54 from dual union all
select 53768,'1981-W764','1981-11-23 20:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 53769,'1981-W765','1981-11-23 21:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 5377,'1958-S382','1958-03-07 00:00:00','military missile','success',1013,'M5-5TK','8K51 OT-12','VV GCh','','Test',636,'',500 from dual union all
select 53770,'1981-W766','1981-11-25 02:04:59','suborbital rocket','success',761,'MT135P-R439','','','','Meteo',1332,'',55 from dual union all
select 53771,'1981-W767','1981-11-25 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',211,'BUGAEV',61 from dual union all
select 53772,'1981-W768','1981-11-25 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53773,'1981-W769','1981-11-25 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 53774,'1981-W77','1981-02-11 19:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 53775,'1981-W770','1981-11-25 17:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 53776,'1981-W771','1981-11-26 01:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',59 from dual union all
select 53777,'1981-W772','1981-11-27 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 53778,'1981-W773','1981-11-30 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 53779,'1981-W774','1981-11-30 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',65 from dual union all
select 5378,'1958-S383','1958-03-28 00:00:00','military missile','success',1013,'A6-84V','8K51 OT-13','Vibrator','','Test',636,'',500 from dual union all
select 53780,'1981-W775','1981-11-30 20:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',67 from dual union all
select 53781,'1981-W776','1981-11-30 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 53782,'1981-W777','1981-12-01 12:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53783,'1981-W778','1981-12-02 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53784,'1981-W779','1981-12-02 18:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 53785,'1981-W78','1981-02-11 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',58 from dual union all
select 53786,'1981-W780','1981-12-02 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 53787,'1981-W781','1981-12-02 22:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',59 from dual union all
select 53788,'1981-W782','1981-12-03 02:00:00','suborbital rocket','success',761,'MT135P-R440','','','','Meteo',1332,'',58 from dual union all
select 53789,'1981-W783','1981-12-03 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',64 from dual union all
select 5379,'1958-S384','1958-03-29 00:00:00','military missile','success',1013,'B2-20T','8K51 OT-14','Series prod.','','Test',636,'',500 from dual union all
select 53790,'1981-W784','1981-12-03 19:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 53791,'1981-W785','1981-12-03 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',62 from dual union all
select 53792,'1981-W786','1981-12-04 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 53793,'1981-W787','1981-12-04 18:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',52 from dual union all
select 53794,'1981-W788','1981-12-04 21:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 53795,'1981-W789','1981-12-04 21:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 53796,'1981-W79','1981-02-12 01:30:00','suborbital rocket','success',761,'MT135P T30','','','','Aeron?',1476,'',50 from dual union all
select 53797,'1981-W790','1981-12-07 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53798,'1981-W791','1981-12-07 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 53799,'1981-W792','1981-12-07 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',50 from dual union all
select 538,'1945-S14','1945-01-10 18:37:00','military missile','success',8,'20808','Ma404','','','Test',661,'',90 from dual union all
select 5380,'1958-S385','1958-05-30 02:31:00','suborbital rocket','success',839,'SS6.49','Grenades','IGY SS6.49','','Aeron',1895,'',100 from dual union all
select 53800,'1981-W793','1981-12-08 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 53801,'1981-W794','1981-12-09 02:04:59','suborbital rocket','success',761,'MT135P-R441','','','','Meteo',1332,'',56 from dual union all
select 53802,'1981-W795','1981-12-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',79 from dual union all
select 53803,'1981-W796','1981-12-09 18:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 53804,'1981-W797','1981-12-09 18:48:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',64 from dual union all
select 53805,'1981-W798','1981-12-09 19:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 53806,'1981-W799','1981-12-10 09:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53807,'1981-W80','1981-02-12 14:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',70 from dual union all
select 53808,'1981-W800','1981-12-10 14:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',53 from dual union all
select 53809,'1981-W801','1981-12-10 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',59 from dual union all
select 5381,'1958-S386','1958-04-24 16:44:00','suborbital rocket','success',839,'NC AF-1','','','','Meteorites',646,'',167 from dual union all
select 53810,'1981-W802','1981-12-11 16:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 53811,'1981-W803','1981-12-11 21:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',68 from dual union all
select 53812,'1981-W804','1981-12-14 14:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 53813,'1981-W805','1981-12-14 15:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53814,'1981-W806','1981-12-14 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 53815,'1981-W807','1981-12-15 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 53816,'1981-W808','1981-12-16 02:00:00','suborbital rocket','success',761,'MT135P-R442','','','','Meteo',1332,'',54 from dual union all
select 53817,'1981-W809','1981-12-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',78 from dual union all
select 53818,'1981-W81','1981-02-12 21:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 53819,'1981-W810','1981-12-16 15:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 5382,'1958-S39','1958-03-04 06:01:36','suborbital rocket','success',839,'AM6.04','Sphere','IGY AM6.04','','Aeron',428,'',100 from dual union all
select 53820,'1981-W811','1981-12-16 20:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',55 from dual union all
select 53821,'1981-W812','1981-12-16 21:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',52 from dual union all
select 53822,'1981-W813','1981-12-16 22:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',58 from dual union all
select 53823,'1981-W814','1981-12-17 17:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 53824,'1981-W815','1981-12-17 20:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 53825,'1981-W816','1981-12-21 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 53826,'1981-W817','1981-12-21 15:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53827,'1981-W818','1981-12-21 16:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 53828,'1981-W819','1981-12-21 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',59 from dual;
insert into LAUNCH
select 53829,'1981-W82','1981-02-13 08:30:00','suborbital rocket','unknown',738,'','','','','Aeron',908,'KRENKEL''',58 from dual union all
select 5383,'1958-S40','1958-03-04 19:30:09','suborbital rocket','success',839,'AM6.05','Sphere','IGY AM6.05','','Aeron',428,'',181 from dual union all
select 53830,'1981-W820','1981-12-22 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 53831,'1981-W821','1981-12-23 02:00:00','suborbital rocket','success',761,'MT135P-R443','','','','Meteo',1332,'',55 from dual union all
select 53832,'1981-W822','1981-12-23 15:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53833,'1981-W823','1981-12-23 22:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 53834,'1981-W824','1981-12-24 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53835,'1981-W825','1981-12-29 12:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 53836,'1981-W826','1981-12-30 22:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 53837,'1981-W827','1981-12-31 15:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 53838,'1981-W828','1981-09-17 02:00:00','suborbital rocket','success',761,'MT135P-R432','','','','Meteo',1332,'',60 from dual union all
select 53839,'1981-W829','1981-10-21 02:00:00','suborbital rocket','success',761,'MT135P-R435','','','','Meteo',1332,'',60 from dual union all
select 5384,'1958-S41','1958-03-16 04:54:15','suborbital rocket','success',49,'NN3.04','NRL NN3.04','IGY NN3.04F','','Auroral/Fields',428,'',144 from dual union all
select 53840,'1981-W83','1981-02-13 10:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 53841,'1981-W84','1981-02-13 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',50 from dual union all
select 53842,'1981-W85','1981-02-13 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 53843,'1981-W86','1981-02-13 22:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 53844,'1981-W87','1981-02-17 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',72 from dual union all
select 53845,'1981-W88','1981-02-17 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',72 from dual union all
select 53846,'1981-W89','1981-02-17 20:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 53847,'1981-W90','1981-02-18 02:00:00','suborbital rocket','success',761,'MT135P-R413','','','','Meteo',1332,'',57 from dual union all
select 53848,'1981-W91','1981-02-18 07:20:00','suborbital rocket','unknown',738,'','','','','Aeron',908,'KRENKEL''',60 from dual union all
select 53849,'1981-W92','1981-02-18 08:40:00','suborbital rocket','unknown',738,'','','','','Aeron',908,'KRENKEL''',57 from dual union all
select 5385,'1958-S42','1958-03-18 00:00:00','suborbital rocket','success',837,'AA5.025','','','','Solar UV',1895,'',107 from dual union all
select 53850,'1981-W93','1981-02-18 10:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual union all
select 53851,'1981-W94','1981-02-18 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 53852,'1981-W95','1981-02-18 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 53853,'1981-W96','1981-02-18 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 53854,'1981-W97','1981-02-19 02:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 53855,'1981-W98','1981-02-19 16:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 53856,'1981-W99','1981-02-20 13:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',65 from dual union all
select 53857,'1981-Y01','1981-01-01 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 53858,'1981-Y02','1981-01-04 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 53859,'1981-Y03','1981-01-05 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 5386,'1958-S43','1958-03-21 00:00:00','suborbital rocket','success',28,'','','','','Ionos',640,'',100 from dual union all
select 53860,'1981-Y04','1981-01-06 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 53861,'1981-Y05','1981-01-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 53862,'1981-Y06','1981-01-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 53863,'1981-Y07','1981-01-07 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 53864,'1981-Y08','1981-01-08 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 53865,'1981-Y09','1981-01-09 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 53866,'1981-Y10','1981-01-10 18:18:00','suborbital rocket','success',1281,'NASA 15.191UE','','','','Plasma',1407,'',80 from dual union all
select 53867,'1981-Y100','1981-03-17 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 53868,'1981-Y101','1981-03-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 53869,'1981-Y102','1981-03-18 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 5387,'1958-S44','1958-03-22 06:41:17','suborbital rocket','success',49,'NN3.05','','IGY NN3.05F','','Auroral/Fields',428,'',168 from dual union all
select 53870,'1981-Y103','1981-03-19 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 53871,'1981-Y104','1981-03-20 07:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 53872,'1981-Y105','1981-03-24 20:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 53873,'1981-Y106','1981-03-24 21:22:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 53874,'1981-Y107','1981-03-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 53875,'1981-Y108','1981-03-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 53876,'1981-Y109','1981-03-25 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 53877,'1981-Y11','1981-01-11 11:27:00','suborbital rocket','success',1281,'NASA 15.192UE','','','','Plasma',1407,'',80 from dual union all
select 53878,'1981-Y110','1981-03-25 16:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 53879,'1981-Y111','1981-03-25 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 5388,'1958-S46','1958-03-23 18:07:08','suborbital rocket','success',49,'NN3.19F','Neutral gas','IGY NN3.19F','','Auroral/Aeron La',428,'',202 from dual union all
select 53880,'1981-Y112','1981-03-27 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 53881,'1981-Y113','1981-03-28 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',83 from dual union all
select 53882,'1981-Y114','1981-03-28 17:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',80 from dual union all
select 53883,'1981-Y115','1981-03-28 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',84 from dual union all
select 53884,'1981-Y116','1981-03-29 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',81 from dual union all
select 53885,'1981-Y117','1981-03-29 17:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',87 from dual union all
select 53886,'1981-Y118','1981-03-29 18:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',85 from dual union all
select 53887,'1981-Y119','1981-03-30 12:46:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',86 from dual union all
select 53888,'1981-Y12','1981-01-11 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 53889,'1981-Y120','1981-03-30 16:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',87 from dual union all
select 5389,'1958-S47','1958-03-24 18:30:25','suborbital rocket','success',839,'OB6.08','Horizon imaging','IGY OB6.08','','Aeron/Imaging',428,'',119 from dual union all
select 53890,'1981-Y121','1981-03-30 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',86 from dual union all
select 53891,'1981-Y122','1981-03-30 18:33:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',83 from dual union all
select 53892,'1981-Y123','1981-03-31 21:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 53893,'1981-Y124','1981-04-01 10:09:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',86 from dual union all
select 53894,'1981-Y125','1981-04-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 53895,'1981-Y126','1981-04-01 14:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 53896,'1981-Y127','1981-04-01 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 53897,'1981-Y128','1981-04-01 16:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',81 from dual union all
select 53898,'1981-Y129','1981-04-01 19:26:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 53899,'1981-Y13','1981-01-12 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 539,'1945-S15','1945-01-10 19:22:00','military missile','success',8,'20809','Ma405','','','Test',661,'',90 from dual union all
select 5390,'1958-S48','1958-03-24 22:00:00','suborbital rocket','success',839,'AM6.38','Pressure','IGY AM6.38','','Aeron',428,'',137 from dual union all
select 53900,'1981-Y130','1981-04-06 08:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 53901,'1981-Y131','1981-04-06 10:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 53902,'1981-Y132','1981-04-07 07:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 53903,'1981-Y133','1981-04-07 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 53904,'1981-Y134','1981-04-08 14:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',81 from dual union all
select 53905,'1981-Y135','1981-04-08 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 53906,'1981-Y136','1981-04-12 13:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',83 from dual union all
select 53907,'1981-Y137','1981-04-12 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',89 from dual union all
select 53908,'1981-Y138','1981-04-13 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',88 from dual union all
select 53909,'1981-Y139','1981-04-13 15:09:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',89 from dual union all
select 5391,'1958-S49','1958-03-29 14:43:00','military missile','success',1016,'M1-10','8K71 LKI II-2','','','Test',480,'',1350 from dual union all
select 53910,'1981-Y14','1981-01-13 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 53911,'1981-Y140','1981-04-14 10:17:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',93 from dual union all
select 53912,'1981-Y141','1981-04-14 17:02:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 53913,'1981-Y142','1981-04-14 17:04:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',92 from dual union all
select 53914,'1981-Y143','1981-04-14 18:20:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',92 from dual union all
select 53915,'1981-Y144','1981-04-14 18:49:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 53916,'1981-Y145','1981-04-14 19:55:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 53917,'1981-Y146','1981-04-14 20:17:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 53918,'1981-Y147','1981-04-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual;
insert into LAUNCH
select 53919,'1981-Y148','1981-04-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 5392,'1958-S50','1958-03-31 00:00:00','military missile','success',974,'','','','','Test',190,'B-67',86 from dual union all
select 53920,'1981-Y149','1981-04-15 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',84 from dual union all
select 53921,'1981-Y15','1981-01-14 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 53922,'1981-Y150','1981-04-15 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 53923,'1981-Y151','1981-04-20 20:55:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',95 from dual union all
select 53924,'1981-Y152','1981-04-21 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 53925,'1981-Y153','1981-04-22 01:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 53926,'1981-Y154','1981-04-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 53927,'1981-Y155','1981-04-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 53928,'1981-Y156','1981-04-22 14:11:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 53929,'1981-Y157','1981-04-22 20:52:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',87 from dual union all
select 5393,'1958-S51','1958-04-04 15:30:00','military missile','success',1016,'B1-12','8K71 LKI II-3','','','Test',480,'',1350 from dual union all
select 53930,'1981-Y158','1981-04-24 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 53931,'1981-Y159','1981-04-28 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 53932,'1981-Y16','1981-01-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 53933,'1981-Y160','1981-04-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 53934,'1981-Y161','1981-04-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 53935,'1981-Y162','1981-04-29 14:15:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 53936,'1981-Y163','1981-04-29 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 53937,'1981-Y164','1981-05-02 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',85 from dual union all
select 53938,'1981-Y165','1981-05-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 53939,'1981-Y166','1981-05-04 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',84 from dual union all
select 5394,'1958-S52','1958-04-05 17:01:00','military missile','failure',142,'15A','R&D Fairing','R&D','','Test',229,'',100 from dual union all
select 53940,'1981-Y167','1981-05-05 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 53941,'1981-Y168','1981-05-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 53942,'1981-Y169','1981-05-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 53943,'1981-Y17','1981-01-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 53944,'1981-Y170','1981-05-07 15:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 53945,'1981-Y171','1981-05-09 18:35:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',84 from dual union all
select 53946,'1981-Y172','1981-05-12 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',82 from dual union all
select 53947,'1981-Y173','1981-05-12 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 53948,'1981-Y174','1981-05-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 53949,'1981-Y175','1981-05-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 5395,'1958-S53','1958-04-07 00:00:00','suborbital rocket','success',839,'','Hi Ball 1','','','Balloon test',1821,'',244 from dual union all
select 53950,'1981-Y176','1981-05-13 15:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 53951,'1981-Y177','1981-05-13 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 53952,'1981-Y178','1981-05-13 18:22:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',92 from dual union all
select 53953,'1981-Y179','1981-05-14 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',88 from dual union all
select 53954,'1981-Y18','1981-01-14 14:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 53955,'1981-Y180','1981-05-14 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',90 from dual union all
select 53956,'1981-Y181','1981-05-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 53957,'1981-Y182','1981-05-15 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',80 from dual union all
select 53958,'1981-Y183','1981-05-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',92 from dual union all
select 53959,'1981-Y184','1981-05-17 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',89 from dual union all
select 5396,'1958-S55','1958-04-17 10:57:00','suborbital rocket','success',1193,'SL07','UCL Grenade/Chaff 2','','','Test/Aeronomy',1839,'',152 from dual union all
select 53960,'1981-Y185','1981-05-19 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',86 from dual union all
select 53961,'1981-Y186','1981-05-19 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 53962,'1981-Y187','1981-05-20 13:44:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 53963,'1981-Y188','1981-05-20 14:35:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 53964,'1981-Y189','1981-05-20 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 53965,'1981-Y19','1981-01-14 21:00:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 53966,'1981-Y190','1981-05-20 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 53967,'1981-Y191','1981-05-20 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',84 from dual union all
select 53968,'1981-Y192','1981-05-20 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',86 from dual union all
select 53969,'1981-Y193','1981-05-21 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',84 from dual union all
select 5397,'1958-S56','1958-04-18 13:05:00','military missile','success',934,'1-204-11','Jetavator','','','Test',244,'',20 from dual union all
select 53970,'1981-Y194','1981-05-22 02:45:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',80 from dual union all
select 53971,'1981-Y195','1981-05-22 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 53972,'1981-Y196','1981-05-22 20:56:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 53973,'1981-Y197','1981-05-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 53974,'1981-Y198','1981-05-27 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',99 from dual union all
select 53975,'1981-Y199','1981-05-27 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 53976,'1981-Y20','1981-01-15 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 53977,'1981-Y200','1981-05-27 17:40:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 53978,'1981-Y201','1981-05-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 53979,'1981-Y202','1981-06-02 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 5398,'1958-S57','1958-04-24 00:00:00','suborbital rocket','success',839,'','Beacon Test 1','','','Balloon test',1821,'',122 from dual union all
select 53980,'1981-Y203','1981-06-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 53981,'1981-Y204','1981-06-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 53982,'1981-Y205','1981-06-03 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 53983,'1981-Y206','1981-06-03 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 53984,'1981-Y207','1981-06-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 53985,'1981-Y208','1981-06-05 19:29:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 53986,'1981-Y209','1981-06-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 53987,'1981-Y21','1981-01-16 14:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 53988,'1981-Y210','1981-06-08 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',89 from dual union all
select 53989,'1981-Y211','1981-06-09 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 5399,'1958-S59','1958-05-27 00:00:00','military missile','success',976,'B1-9','K63 No. 1 LKI2-1','','','Test',636,'',402 from dual union all
select 53990,'1981-Y212','1981-06-09 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 53991,'1981-Y213','1981-06-10 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 53992,'1981-Y214','1981-06-10 03:15:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 53993,'1981-Y215','1981-06-10 13:49:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 53994,'1981-Y216','1981-06-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 53995,'1981-Y217','1981-06-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 53996,'1981-Y218','1981-06-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',83 from dual union all
select 53997,'1981-Y219','1981-06-10 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 53998,'1981-Y22','1981-01-18 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 53999,'1981-Y220','1981-06-11 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 54,'1944-A18','1944-09-15 17:54:00','military missile','failure',8,'18763','Ma177','','','Training',661,'',5 from dual union all
select 540,'1945-S16','1945-01-11 11:10:00','military missile','success',8,'20810','Ma406','','','Test',661,'',90 from dual union all
select 5400,'1958-S60','1958-05-01 12:33:00','suborbital rocket','success',839,'NC AF-2','','','','Meteorites',646,'',137 from dual union all
select 54000,'1981-Y221','1981-06-11 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',87 from dual union all
select 54001,'1981-Y222','1981-06-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 54002,'1981-Y223','1981-06-12 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 54003,'1981-Y224','1981-06-12 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',82 from dual union all
select 54004,'1981-Y225','1981-06-13 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',83 from dual union all
select 54005,'1981-Y226','1981-06-14 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',86 from dual union all
select 54006,'1981-Y227','1981-06-14 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',85 from dual union all
select 54007,'1981-Y228','1981-06-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual;
insert into LAUNCH
select 54008,'1981-Y229','1981-06-15 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',85 from dual union all
select 54009,'1981-Y23','1981-01-20 21:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 5401,'1958-S61','1958-05-05 14:15:00','suborbital rocket','success',49,'NN3.22F','','IGY NN3.22F','','Solar UV',1869,'',214 from dual union all
select 54010,'1981-Y230','1981-06-16 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',82 from dual union all
select 54011,'1981-Y231','1981-06-17 07:12:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 54012,'1981-Y232','1981-06-17 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 54013,'1981-Y233','1981-06-17 10:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',94 from dual union all
select 54014,'1981-Y234','1981-06-17 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 54015,'1981-Y235','1981-06-17 13:44:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',86 from dual union all
select 54016,'1981-Y236','1981-06-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 54017,'1981-Y237','1981-06-17 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 54018,'1981-Y238','1981-06-17 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 54019,'1981-Y239','1981-06-17 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',84 from dual union all
select 5402,'1958-S62','1958-05-08 15:36:00','military missile','success',934,'1-204-12','Jetavator','','','Test',244,'',20 from dual union all
select 54020,'1981-Y24','1981-01-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 54021,'1981-Y240','1981-06-18 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 54022,'1981-Y241','1981-06-19 07:12:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 54023,'1981-Y242','1981-06-19 10:12:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 54024,'1981-Y243','1981-06-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 54025,'1981-Y244','1981-06-21 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',87 from dual union all
select 54026,'1981-Y245','1981-06-22 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 54027,'1981-Y246','1981-06-22 02:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',80 from dual union all
select 54028,'1981-Y247','1981-06-23 08:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 54029,'1981-Y248','1981-06-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 5403,'1958-S63','1958-05-17 00:05:00','military missile','success',1053,'RS-1002','','','','OT',279,'',90 from dual union all
select 54030,'1981-Y249','1981-06-23 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 54031,'1981-Y25','1981-01-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 54032,'1981-Y250','1981-06-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 54033,'1981-Y251','1981-06-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 54034,'1981-Y252','1981-06-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 54035,'1981-Y253','1981-06-24 16:49:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 54036,'1981-Y254','1981-06-24 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',84 from dual union all
select 54037,'1981-Y255','1981-06-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 54038,'1981-Y256','1981-06-25 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',89 from dual union all
select 54039,'1981-Y257','1981-06-25 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',892,'KORO',85 from dual union all
select 5404,'1958-S64','1958-05-18 05:05:00','military missile','success',638,'AM-5','Gaslight','','','Reentry Test',249,'',500 from dual union all
select 54040,'1981-Y258','1981-06-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 54041,'1981-Y259','1981-06-30 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 54042,'1981-Y26','1981-01-21 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',95 from dual union all
select 54043,'1981-Y260','1981-07-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 54044,'1981-Y261','1981-07-01 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 54045,'1981-Y262','1981-07-01 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',85 from dual union all
select 54046,'1981-Y263','1981-07-01 21:15:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 54047,'1981-Y264','1981-07-01 21:32:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',93 from dual union all
select 54048,'1981-Y265','1981-07-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 54049,'1981-Y266','1981-07-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 5405,'1958-S65','1958-05-20 09:43:35','suborbital rocket','success',839,'','Release 2 Cs/Na','','','Aeron',646,'',101 from dual union all
select 54050,'1981-Y267','1981-07-07 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 54051,'1981-Y268','1981-07-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 54052,'1981-Y269','1981-07-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 54053,'1981-Y27','1981-01-22 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 54054,'1981-Y270','1981-07-08 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',81 from dual union all
select 54055,'1981-Y271','1981-07-08 15:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 54056,'1981-Y272','1981-07-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 54057,'1981-Y273','1981-07-14 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 54058,'1981-Y274','1981-07-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 54059,'1981-Y275','1981-07-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 5406,'1958-S66','1958-05-20 04:26:00','suborbital rocket','success',1194,'SL05','RAE Test 5/QUB AGP2','','','Test/Aeronomy',1839,'',150 from dual union all
select 54060,'1981-Y276','1981-07-15 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 54061,'1981-Y277','1981-07-15 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 54062,'1981-Y278','1981-07-16 18:22:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 54063,'1981-Y279','1981-07-20 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',84 from dual union all
select 54064,'1981-Y28','1981-01-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',92 from dual union all
select 54065,'1981-Y280','1981-07-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',82 from dual union all
select 54066,'1981-Y281','1981-07-21 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 54067,'1981-Y282','1981-07-21 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 54068,'1981-Y283','1981-07-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 54069,'1981-Y284','1981-07-22 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',87 from dual union all
select 5407,'1958-S67','1958-05-21 11:31:59','suborbital rocket','success',839,'','Release 3 Cs/Na','','','Aeron',646,'',128 from dual union all
select 54070,'1981-Y285','1981-07-22 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 54071,'1981-Y286','1981-07-23 14:18:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',84 from dual union all
select 54072,'1981-Y287','1981-07-23 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',83 from dual union all
select 54073,'1981-Y288','1981-07-24 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',86 from dual union all
select 54074,'1981-Y289','1981-07-24 21:25:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 54075,'1981-Y29','1981-01-25 20:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 54076,'1981-Y290','1981-07-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 54077,'1981-Y291','1981-07-28 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 54078,'1981-Y292','1981-07-28 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 54079,'1981-Y293','1981-07-28 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 5408,'1958-S68','1958-05-22 11:29:59','suborbital rocket','success',839,'','Release 4 (Na)','','','Aeron',646,'',116 from dual union all
select 54080,'1981-Y294','1981-07-28 23:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 54081,'1981-Y295','1981-07-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',856,'',91 from dual union all
select 54082,'1981-Y296','1981-07-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 54083,'1981-Y297','1981-07-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 54084,'1981-Y298','1981-07-30 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 54085,'1981-Y299','1981-08-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 54086,'1981-Y30','1981-01-26 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 54087,'1981-Y300','1981-08-04 07:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 54088,'1981-Y301','1981-08-04 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 54089,'1981-Y302','1981-08-04 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 5409,'1958-S69','1958-05-22 00:57:00','suborbital rocket','success',839,'','Hi Ball 2','','','Balloon Test',1821,'',240 from dual union all
select 54090,'1981-Y303','1981-08-04 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 54091,'1981-Y304','1981-08-04 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 54092,'1981-Y305','1981-08-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 54093,'1981-Y306','1981-08-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 54094,'1981-Y307','1981-08-05 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 54095,'1981-Y308','1981-08-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 54096,'1981-Y309','1981-08-11 02:08:00','suborbital rocket','success',1281,'NASA 15.212UE','','','','Plasma',1821,'',82 from dual union all
select 54097,'1981-Y31','1981-01-27 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 54098,'1981-Y310','1981-08-11 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual;
insert into LAUNCH
select 54099,'1981-Y311','1981-08-11 20:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 541,'1945-S17','1945-01-11 11:55:00','military missile','success',8,'20807','Ma402','','','Test',661,'',90 from dual union all
select 5410,'1958-S70','1958-05-24 10:30:00','military missile','failure',1016,'B1-3','8K71 LKI II-4','','','Test',480,'',1350 from dual union all
select 54100,'1981-Y312','1981-08-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 54101,'1981-Y313','1981-08-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 54102,'1981-Y314','1981-08-12 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 54103,'1981-Y315','1981-08-12 17:20:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 54104,'1981-Y316','1981-08-13 03:22:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',85 from dual union all
select 54105,'1981-Y317','1981-08-13 17:48:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 54106,'1981-Y318','1981-08-18 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',81 from dual union all
select 54107,'1981-Y319','1981-08-18 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 54108,'1981-Y32','1981-01-28 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 54109,'1981-Y320','1981-08-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 5411,'1958-S71','1958-05-25 00:00:00','suborbital rocket','success',839,'','Beacon Test 2','','','Balloon Test',1821,'',122 from dual union all
select 54110,'1981-Y321','1981-08-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 54111,'1981-Y322','1981-08-19 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',86 from dual union all
select 54112,'1981-Y323','1981-08-25 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 54113,'1981-Y324','1981-08-25 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 54114,'1981-Y325','1981-08-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 54115,'1981-Y326','1981-08-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 54116,'1981-Y327','1981-08-26 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',84 from dual union all
select 54117,'1981-Y328','1981-08-26 18:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 54118,'1981-Y329','1981-08-26 20:02:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 54119,'1981-Y33','1981-01-28 13:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 5412,'1958-S72','1958-05-25 00:00:00','military missile','success',974,'','','','','Test',190,'B-67',86 from dual union all
select 54120,'1981-Y330','1981-09-02 13:14:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',84 from dual union all
select 54121,'1981-Y331','1981-09-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 54122,'1981-Y332','1981-09-02 17:54:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 54123,'1981-Y333','1981-09-03 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 54124,'1981-Y334','1981-09-03 11:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 54125,'1981-Y335','1981-09-04 21:40:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 54126,'1981-Y336','1981-09-08 20:35:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 54127,'1981-Y337','1981-09-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 54128,'1981-Y338','1981-09-09 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 54129,'1981-Y339','1981-09-15 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 5413,'1958-S73','1958-05-26 00:00:00','military missile','success',974,'','','','','Test',190,'B-67',86 from dual union all
select 54130,'1981-Y34','1981-01-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 54131,'1981-Y340','1981-09-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 54132,'1981-Y341','1981-09-16 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 54133,'1981-Y342','1981-09-16 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 54134,'1981-Y343','1981-09-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 54135,'1981-Y344','1981-09-18 13:44:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 54136,'1981-Y345','1981-09-22 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 54137,'1981-Y346','1981-09-22 20:25:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 54138,'1981-Y347','1981-09-23 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',80 from dual union all
select 54139,'1981-Y348','1981-09-23 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 5414,'1958-S74','1958-06-04 00:00:00','military missile','success',976,'B1-8','63M No. 8 LKI1-8','','','Test',636,'',402 from dual union all
select 54140,'1981-Y349','1981-09-29 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 54141,'1981-Y35','1981-01-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 54142,'1981-Y350','1981-09-30 13:44:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',83 from dual union all
select 54143,'1981-Y351','1981-09-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 54144,'1981-Y352','1981-09-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 54145,'1981-Y353','1981-09-30 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 54146,'1981-Y354','1981-10-01 15:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 54147,'1981-Y355','1981-10-06 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 54148,'1981-Y356','1981-10-06 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 54149,'1981-Y357','1981-10-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 5415,'1958-S75','1958-06-13 00:00:00','military missile','success',976,'B1-10','K63 LKI2-2','','','Test',636,'',402 from dual union all
select 54150,'1981-Y358','1981-10-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 54151,'1981-Y359','1981-10-09 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 54152,'1981-Y36','1981-01-28 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',82 from dual union all
select 54153,'1981-Y360','1981-10-13 20:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 54154,'1981-Y361','1981-10-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 54155,'1981-Y362','1981-10-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 54156,'1981-Y363','1981-10-14 14:29:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',84 from dual union all
select 54157,'1981-Y364','1981-10-14 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 54158,'1981-Y365','1981-10-14 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',91 from dual union all
select 54159,'1981-Y366','1981-10-15 00:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 5416,'1958-S76','1958-06-19 00:00:00','military missile','success',976,'B1-11','K63 LKI2-3','','','Test',636,'',402 from dual union all
select 54160,'1981-Y367','1981-10-16 20:26:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 54161,'1981-Y368','1981-10-20 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 54162,'1981-Y369','1981-10-20 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 54163,'1981-Y37','1981-01-28 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 54164,'1981-Y370','1981-10-20 23:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 54165,'1981-Y371','1981-10-21 00:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 54166,'1981-Y372','1981-10-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 54167,'1981-Y373','1981-10-21 14:50:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 54168,'1981-Y374','1981-10-21 19:12:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 54169,'1981-Y375','1981-10-23 17:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 5417,'1958-S77','1958-06-03 04:50:00','military missile','success',1053,'CC-1004','','Bat B','','OT',1854,'',90 from dual union all
select 54170,'1981-Y376','1981-10-27 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 54171,'1981-Y377','1981-10-27 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 54172,'1981-Y378','1981-10-27 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 54173,'1981-Y379','1981-10-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 54174,'1981-Y38','1981-01-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 54175,'1981-Y380','1981-10-28 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 54176,'1981-Y381','1981-10-28 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 54177,'1981-Y382','1981-11-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 54178,'1981-Y383','1981-11-03 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 54179,'1981-Y384','1981-11-04 00:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 5418,'1958-S78','1958-06-03 21:28:00','military missile','success',142,'16A','R&D Fairing','R&D','','Test',227,'',120 from dual union all
select 54180,'1981-Y385','1981-11-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 54181,'1981-Y386','1981-11-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 54182,'1981-Y387','1981-11-04 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 54183,'1981-Y388','1981-11-05 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 54184,'1981-Y389','1981-11-06 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',95 from dual union all
select 54185,'1981-Y39','1981-01-30 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 54186,'1981-Y390','1981-11-10 20:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',94 from dual union all
select 54187,'1981-Y391','1981-11-10 23:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 54188,'1981-Y392','1981-11-11 09:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual;
insert into LAUNCH
select 54189,'1981-Y393','1981-11-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 5419,'1958-S79','1958-06-04 21:17:00','military missile','success',1366,'115','R&D Series III','','','Test',236,'',520 from dual union all
select 54190,'1981-Y394','1981-11-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 54191,'1981-Y395','1981-11-11 21:35:00','suborbital rocket','success',725,'Ozone M-08-621','ISRO-8.621','','','Aeron',1467,'',82 from dual union all
select 54192,'1981-Y396','1981-11-12 09:30:00','suborbital rocket','success',725,'Ozone M-08-622','ISRO-8.622','','','Aeron',1467,'',84 from dual union all
select 54193,'1981-Y397','1981-11-12 13:51:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 54194,'1981-Y398','1981-11-12 18:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',89 from dual union all
select 54195,'1981-Y399','1981-11-12 18:55:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 54196,'1981-Y40','1981-02-02 12:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',90 from dual union all
select 54197,'1981-Y400','1981-11-13 14:25:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',85 from dual union all
select 54198,'1981-Y401','1981-11-13 19:15:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 54199,'1981-Y402','1981-11-13 21:17:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',87 from dual union all
select 542,'1945-S18','1945-01-11 15:02:00','military missile','success',8,'20811','Ma398','','','Test',661,'',90 from dual union all
select 5420,'1958-S80','1958-06-04 15:15:00','suborbital rocket','success',49,'','Solar EUV Spectrum','','','Solar EUV',640,'',200 from dual union all
select 54200,'1981-Y403','1981-11-14 20:20:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 54201,'1981-Y404','1981-11-14 20:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 54202,'1981-Y405','1981-11-14 20:49:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 54203,'1981-Y406','1981-11-14 22:21:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 54204,'1981-Y407','1981-11-16 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 54205,'1981-Y408','1981-11-17 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 54206,'1981-Y409','1981-11-17 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 54207,'1981-Y41','1981-02-03 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',90 from dual union all
select 54208,'1981-Y410','1981-11-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 54209,'1981-Y411','1981-11-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 5421,'1958-S81','1958-06-05 11:30:00','suborbital rocket','failure',1193,'SL22','RAE Test 6','','','Test',1839,'',153 from dual union all
select 54210,'1981-Y412','1981-11-23 10:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 54211,'1981-Y413','1981-11-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 54212,'1981-Y414','1981-11-23 20:56:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 54213,'1981-Y415','1981-11-24 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 54214,'1981-Y416','1981-11-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 54215,'1981-Y417','1981-11-25 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 54216,'1981-Y418','1981-11-25 17:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 54217,'1981-Y419','1981-11-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 54218,'1981-Y42','1981-02-03 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 54219,'1981-Y420','1981-12-01 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 5422,'1958-S82','1958-06-06 00:00:00','military missile','success',934,'1-204-13','Jetavator','','','Reentry Test',244,'',20 from dual union all
select 54220,'1981-Y421','1981-12-02 06:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 54221,'1981-Y422','1981-12-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 54222,'1981-Y423','1981-12-02 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 54223,'1981-Y424','1981-12-04 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 54224,'1981-Y425','1981-12-04 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 54225,'1981-Y426','1981-12-08 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 54226,'1981-Y427','1981-12-09 22:01:59','suborbital rocket','success',1191,'E8','EBC81 E8','','','Aeronomy?',417,'',70 from dual union all
select 54227,'1981-Y428','1981-12-09 22:55:30','suborbital rocket','success',1189,'E7','EBC81 E7','','','Aeronomy?',417,'',94 from dual union all
select 54228,'1981-Y429','1981-12-09 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 54229,'1981-Y43','1981-02-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 5423,'1958-S83','1958-06-12 01:59:00','military missile','success',635,'CC-54    ST','','','','Test',280,'',90 from dual union all
select 54230,'1981-Y430','1981-12-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',88 from dual union all
select 54231,'1981-Y431','1981-12-09 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 54232,'1981-Y432','1981-12-11 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 54233,'1981-Y433','1981-12-11 21:11:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',87 from dual union all
select 54234,'1981-Y434','1981-12-15 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 54235,'1981-Y435','1981-12-16 01:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 54236,'1981-Y436','1981-12-16 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',84 from dual union all
select 54237,'1981-Y437','1981-12-16 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 54238,'1981-Y438','1981-12-17 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 54239,'1981-Y439','1981-12-18 19:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 5424,'1958-S84','1958-06-13 15:06:00','military missile','success',1366,'122','R&D Series III','','','Test',234,'',520 from dual union all
select 54240,'1981-Y44','1981-02-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',87 from dual union all
select 54241,'1981-Y440','1981-12-22 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 54242,'1981-Y441','1981-12-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 54243,'1981-Y442','1981-12-23 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 54244,'1981-Y443','1981-12-23 20:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 54245,'1981-Y444','1981-12-24 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 54246,'1981-Y445','1981-12-25 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 54247,'1981-Y446','1981-12-29 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 54248,'1981-Y447','1981-12-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 54249,'1981-Y448','1981-12-31 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 5425,'1958-S85','1958-06-18 15:15:00','suborbital rocket','success',1194,'SL09','UCL Grenade/Chaff 4','','','Aeronomy',1839,'',144 from dual union all
select 54250,'1981-Y449','1981-12-31 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',86 from dual union all
select 54251,'1981-Y45','1981-02-04 21:33:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',95 from dual union all
select 54252,'1981-Y450','1981-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 54253,'1981-Y451','1981-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 54254,'1981-Y452','1981-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 54255,'1981-Y453','1981-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 54256,'1981-Y46','1981-02-05 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 54257,'1981-Y47','1981-02-06 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 54258,'1981-Y48','1981-02-10 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 54259,'1981-Y49','1981-02-10 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 5426,'1958-S86','1958-09-19 21:15:00','suborbital rocket','success',1194,'SL08','UCL Grenade/Chaff 5','','','Test/Aeronomy',1839,'',154 from dual union all
select 54260,'1981-Y50','1981-02-11 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 54261,'1981-Y51','1981-02-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 54262,'1981-Y52','1981-02-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 54263,'1981-Y53','1981-02-11 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 54264,'1981-Y54','1981-02-11 16:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 54265,'1981-Y55','1981-02-12 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 54266,'1981-Y56','1981-02-13 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 54267,'1981-Y57','1981-02-15 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',84 from dual union all
select 54268,'1981-Y58','1981-02-17 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',84 from dual union all
select 54269,'1981-Y59','1981-02-17 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',84 from dual union all
select 5427,'1958-S87','1958-06-24 13:00:00','military missile','success',934,'1-204-14','Jetavator','','','Reentry Test',244,'',20 from dual union all
select 54270,'1981-Y60','1981-02-18 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',83 from dual union all
select 54271,'1981-Y61','1981-02-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 54272,'1981-Y62','1981-02-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 54273,'1981-Y63','1981-02-18 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',83 from dual union all
select 54274,'1981-Y64','1981-02-18 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 54275,'1981-Y65','1981-02-19 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 54276,'1981-Y66','1981-02-19 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 54277,'1981-Y67','1981-02-20 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 54278,'1981-Y68','1981-02-20 20:15:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual;
insert into LAUNCH
select 54279,'1981-Y69','1981-02-21 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',80 from dual union all
select 5428,'1958-S88','1958-06-25 03:36:00','military missile','success',1053,'CC-48    TL','','','','Test',280,'',90 from dual union all
select 54280,'1981-Y70','1981-02-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',81 from dual union all
select 54281,'1981-Y71','1981-02-24 13:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',87 from dual union all
select 54282,'1981-Y72','1981-02-24 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 54283,'1981-Y73','1981-02-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 54284,'1981-Y74','1981-02-25 14:07:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 54285,'1981-Y75','1981-02-25 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',84 from dual union all
select 54286,'1981-Y76','1981-02-25 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',90 from dual union all
select 54287,'1981-Y77','1981-02-25 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 54288,'1981-Y78','1981-02-25 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 54289,'1981-Y79','1981-02-26 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 5429,'1958-S89','1958-06-26 00:00:00','suborbital rocket','success',470,'','Model 1','','','Test',1821,'',370 from dual union all
select 54290,'1981-Y80','1981-02-26 22:53:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 54291,'1981-Y81','1981-02-26 23:15:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 54292,'1981-Y82','1981-02-27 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 54293,'1981-Y83','1981-02-27 21:25:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 54294,'1981-Y84','1981-03-02 00:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 54295,'1981-Y85','1981-03-03 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 54296,'1981-Y86','1981-03-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 54297,'1981-Y87','1981-03-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 54298,'1981-Y88','1981-03-04 16:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',89 from dual union all
select 54299,'1981-Y89','1981-03-04 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 543,'1945-S19','1945-01-11 15:12:00','military missile','success',8,'20804','Ma401','','','Test',661,'',90 from dual union all
select 5430,'1958-S90','1958-07-02 00:00:00','test rocket','success',1521,'Winder 1','Argus test','','',' Test',1338,'AVM1',560 from dual union all
select 54300,'1981-Y90','1981-03-05 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 54301,'1981-Y91','1981-03-06 18:11:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 54302,'1981-Y92','1981-03-06 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 54303,'1981-Y93','1981-03-07 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1379,'SHIR',82 from dual union all
select 54304,'1981-Y94','1981-03-10 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',80 from dual union all
select 54305,'1981-Y95','1981-03-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 54306,'1981-Y96','1981-03-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 54307,'1981-Y97','1981-03-11 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 54308,'1981-Y98','1981-03-13 02:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 54309,'1981-Y99','1981-03-13 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 5431,'1958-S91','1958-07-02 05:00:00','suborbital rocket','success',994,'','','','','CR',636,'',210 from dual union all
select 54310,'1982-001','1982-01-07 15:38:00','orbital','success',680,'53755-102','Kosmos-1331','Strela-2M','','Sat',457,'',null from dual union all
select 54311,'1982-002','1982-01-12 12:30:00','orbital','success',1238,'77023  -076 976','Kosmos-1332','Orion','','Sat',473,'',null from dual union all
select 54312,'1982-003','1982-01-14 07:51:00','orbital','success',680,'53767-250','Kosmos-1333','Parus','','Sat',457,'',null from dual union all
select 54313,'1982-004','1982-01-16 01:55:00','orbital','success',388,'643/D159','Satcom 4','RCA-C''','','Sat',233,'',null from dual union all
select 54314,'1982-005','1982-01-20 11:30:00','orbital','success',1238,'78039  -424','Kosmos-1334','Zenit-6','','Sat',462,'',null from dual union all
select 54315,'1982-006','1982-01-21 19:30:01','orbital','success',1403,'24B-26 (3B-62)','KH-8A','GAMBIT-3 Mission 4352','','Sat',1778,'',null from dual union all
select 54316,'1982-007','1982-01-29 11:00:00','orbital','success',680,'65075-115','Kosmos-1335','Romb','','Sat',456,'',null from dual union all
select 54317,'1982-008','1982-01-30 11:30:00','orbital','success',1238,'78039  -953','Kosmos-1336','Feniks No. 953','','Sat',562,'',null from dual union all
select 54318,'1982-009','1982-02-05 09:12:00','orbital','success',956,'308-01','Ekran','Ekran No. 22L','','Sat',554,'',null from dual union all
select 54319,'1982-010','1982-02-11 01:11:00','orbital','success',1446,'51035-501','Kosmos-1337','US-PM1','','Sat',589,'',null from dual union all
select 5432,'1958-S92','1958-07-03 16:52:00','suborbital rocket','success',28,'AU3.07F','Charge density','IGY AU3.07F','','Ionos',428,'',262 from dual union all
select 54320,'1982-011','1982-02-16 11:10:00','orbital','success',1238,'78039  -419','Kosmos-1338','Zenit-6','','Sat',473,'',null from dual union all
select 54321,'1982-012','1982-02-17 21:56:00','orbital','success',680,'65067-261','Kosmos-1339','Tsikada','','Sat',457,'',null from dual union all
select 54322,'1982-013','1982-02-19 01:42:00','orbital','success',1510,'77023  -338','Kosmos-1340','Tselina-D','','Sat',462,'',null from dual union all
select 54323,'1982-014','1982-02-26 00:04:44','orbital','success',388,'644/D160','Westar 4','Westar 4','','Sat',233,'',null from dual union all
select 54324,'1982-015','1982-02-26 20:10:00','orbital','success',809,'77023  -620','Molniya-1','Molniya-1','','Sat',473,'',null from dual union all
select 54325,'1982-016','1982-03-03 05:44:38','orbital','success',809,'77023  -618','Kosmos-1341','Oko','','Sat',462,'',null from dual union all
select 54326,'1982-017','1982-03-05 00:23:00','orbital','success',180,'AC-58','Intelsat 504','INTELSAT V F4','','Sat',269,'',null from dual union all
select 54327,'1982-018','1982-03-05 10:50:00','orbital','success',1238,'78039  -420','Kosmos-1342','Zenit-6','','Sat',473,'',null from dual union all
select 54328,'1982-019','1982-03-06 19:25:00','orbital','success',1425,'23C-20 (3C-38)','DSP 13','DSP 13','','Sat',274,'',null from dual union all
select 54329,'1982-020','1982-03-15 04:39:00','orbital','success',956,'305-02','Gorizont','Gorizont No. 14L','','Sat',553,'',null from dual union all
select 5433,'1958-S93','1958-07-06 12:02:18','suborbital rocket','success',28,'AU3.10F','Charge density','IGY AU3.10F','','Ionos',428,'',250 from dual union all
select 54330,'1982-021','1982-03-17 10:30:00','orbital','success',1238,'78039  -425','Kosmos-1343','Zenit-6','','Sat',473,'',null from dual union all
select 54331,'1982-022','1982-03-22 16:00:00','orbital','success',1243,'STS-3','OFT-3/OSS-1','OV-102 OFT-3','','Sat',921,'MLP1',null from dual union all
select 54332,'1982-023','1982-03-24 00:12:00','orbital','success',809,'78039  -638','Molniya-3','Molniya-3 No. 29','','Sat',473,'',null from dual union all
select 54333,'1982-024','1982-03-24 19:47:00','orbital','success',680,'53767-253','Kosmos-1344','Parus','','Sat',456,'',null from dual union all
select 54334,'1982-025','1982-03-25 09:50:00','orbital','success',1448,'67029-805','Meteor-2','Meteor-2 No. 12','','Sat',469,'',null from dual union all
select 54335,'1982-026','1982-03-31 09:00:00','orbital','success',680,'53747-241','Kosmos-1345','Tselina-OM','','Sat',457,'',null from dual union all
select 54336,'1982-027','1982-03-31 16:27:00','orbital','success',1510,'77023  -339','Kosmos-1346','Tselina-D','','Sat',462,'',null from dual union all
select 54337,'1982-028','1982-04-02 10:15:00','orbital','success',1238,'Sh15000-310','Kosmos-1347','Yantar''-4K2','','Sat',562,'',null from dual union all
select 54338,'1982-029','1982-04-07 13:41:47','orbital','success',809,'77023  -621','Kosmos-1348','Oko','','Sat',462,'',null from dual union all
select 54339,'1982-030','1982-04-08 00:15:00','orbital','success',680,'65047-242','Kosmos-1349','Parus','','Sat',457,'',null from dual union all
select 5434,'1958-S94','1958-07-07 18:50:28','suborbital rocket','success',839,'OB6.09','Horizon imaging','IGY OB6.09','','Aeron/Imaging',428,'',121 from dual union all
select 54340,'1982-031','1982-04-10 06:47:00','orbital','success',388,'647/D161','Insat 1A','Insat 1A','','Sat',233,'',null from dual union all
select 54341,'1982-032','1982-04-15 14:30:00','orbital','success',1238,'76053  -979','Kosmos-1350','Feniks No. 978','','Sat',473,'',null from dual union all
select 54342,'1982-033','1982-04-19 19:45:00','orbital','success',949,'306-02','Salyut-7','DOS 6 (17K No. 125-02)','','Sat',554,'',null from dual union all
select 54343,'1982-034','1982-04-21 01:40:00','orbital','success',680,'53767-256','Kosmos-1351','Romb','','Sat',622,'',null from dual union all
select 54344,'1982-035','1982-04-21 09:15:00','orbital','success',1238,'78039  -426','Kosmos-1352','Zenit-6','','Sat',562,'',null from dual union all
select 54345,'1982-036','1982-04-23 09:40:00','orbital','success',1238,'76054  -080','Kosmos-1353','Fram','','Sat',473,'',null from dual union all
select 54346,'1982-037','1982-04-28 02:52:00','orbital','success',680,'47175-125','Kosmos-1354','Strela-2M','','Sat',456,'',null from dual union all
select 54347,'1982-038','1982-04-29 09:55:00','orbital','success',1446,'67035-502','Kosmos-1355','US-PM1','','Sat',589,'',null from dual union all
select 54348,'1982-039','1982-05-05 08:01:00','orbital','success',1510,'77023  -340','Kosmos-1356','Tselina-D','','Sat',462,'',null from dual union all
select 54349,'1982-040','1982-05-06 18:07:00','orbital','success',680,'53775-123','Kosmos-1357/1364','Strela-1M','','Sat',457,'',null from dual union all
select 5435,'1958-S95','1958-07-07 23:18:00','suborbital rocket','success',838,'NN8.52F','','IGY NN8.52F','','Test/Solar UV/Solar',1415,'',225 from dual union all
select 54350,'1982-041','1982-05-11 18:35:00','orbital','success',1426,'23D-24','HEXAGON SV17','HEXAGON SV-17','','Sat',1777,'',null from dual union all
select 54351,'1982-042','1982-05-13 09:58:05','orbital','success',1238,'Ts15000-272','Soyuz T-5','Soyuz 7K-ST No. 11L','','Sat',480,'',null from dual union all
select 54352,'1982-043','1982-05-14 19:39:00','orbital','success',1446,'51055-507','Kosmos-1365','US-A reactor','','Sat',589,'',null from dual union all
select 54353,'1982-044','1982-05-17 23:50:00','orbital','success',956,'310-02','Kosmos-1366','Geizer No. 11L','','Sat',553,'',null from dual union all
select 54354,'1982-045','1982-05-20 13:08:31','orbital','success',809,'76053  -624','Kosmos-1367','Oko','','Sat',473,'',null from dual union all
select 54355,'1982-046','1982-05-21 12:40:00','orbital','success',1238,'78018  -710','Kosmos-1368','Zenit-6','','Sat',562,'',null from dual union all
select 54356,'1982-047','1982-05-23 05:56:41','orbital','success',1238,'Ts15000-283','Progress-13','Progress 7K-TG No. 114','','Sat',480,'',null from dual union all
select 54357,'1982-048','1982-05-25 09:00:00','orbital','success',1238,'76054  -117','Kosmos-1369','Resurs-F1 17F41 No. 17','','Sat',474,'',null from dual union all
select 54358,'1982-049','1982-05-28 09:10:00','orbital','success',1238,'Ts15000-290','Kosmos-1370','Siluet No. 2','','Sat',562,'',null from dual union all
select 54359,'1982-050','1982-05-28 22:02:59','orbital','success',809,'78017  -556','Molniya-1','Molniya-1','','Sat',474,'',null from dual union all
select 5436,'1958-S96','1958-07-08 23:15:00','suborbital rocket','success',838,'NN8.53F','','IGY NN8.53F','','Test/Solar UV/Solar',1415,'',225 from dual union all
select 54360,'1982-051','1982-06-01 04:37:00','orbital','success',680,'47167-258','Kosmos-1371','Strela-2M','','Sat',456,'',null from dual union all
select 54361,'1982-052','1982-06-01 13:58:00','orbital','success',1446,'51083-622','Kosmos-1372','US-A reactor','','Sat',589,'',null from dual union all
select 54362,'1982-053','1982-06-02 12:50:00','orbital','success',1238,'77049  -430','Kosmos-1373','Zenit-6','','Sat',562,'',null from dual union all
select 54363,'1982-054','1982-06-03 21:30:00','orbital','success',647,'','Kosmos-1374','BOR-4 No. 404','','Sat',622,'',null from dual union all
select 54364,'1982-055','1982-06-06 17:10:00','orbital','success',680,'65067-257','Kosmos-1375','Lira','','Sat',457,'',null from dual union all
select 54365,'1982-056','1982-06-08 07:45:00','orbital','success',1238,'76054  -118','Kosmos-1376','Resurs-F1 17F41 No. 18','','Sat',474,'',null from dual union all
select 54366,'1982-057','1982-06-08 12:00:00','orbital','success',1238,'78039  -215','Kosmos-1377','Oktan No. 215','','Sat',562,'',null from dual union all
select 54367,'1982-058','1982-06-09 00:24:00','orbital','success',388,'649/D162','Westar 5','Westar 5','','Sat',233,'',null from dual union all
select 54368,'1982-059','1982-06-10 17:37:00','orbital','success',1448,'51029-804','Kosmos-1378','Tselina-D','','Sat',469,'',null from dual;
insert into LAUNCH
select 54369,'1982-060','1982-06-18 11:04:00','orbital','success',1446,'45083-624','Kosmos-1379','IS-P Uran','','Sat',589,'',null from dual union all
select 5437,'1958-S97','1958-07-10 02:49:00','military missile','success',1352,'118','Able RTV','','','Reentry Test',233,'',1600 from dual union all
select 54370,'1982-061','1982-06-18 11:58:00','orbital','failure',680,'53783-460','Kosmos-1380','Parus','','Sat',457,'',665 from dual union all
select 54371,'1982-062','1982-06-18 13:00:00','orbital','success',1238,'77049  -434','Kosmos-1381','Zenit-6','','Sat',562,'',null from dual union all
select 54372,'1982-063','1982-06-24 16:29:48','orbital','success',1238,'Ts15000-269','Soyuz T-6','Soyuz 7K-ST No. 9L','','Sat',480,'',null from dual union all
select 54373,'1982-064','1982-06-25 02:28:02','orbital','success',809,'76053  -626','Kosmos-1382','Oko','','Sat',474,'',null from dual union all
select 54374,'1982-065','1982-06-27 15:00:00','orbital','success',1243,'STS-4','OFT-4/DoD','OV-102 OFT-4','','Sat',921,'MLP1',null from dual union all
select 54375,'1982-066','1982-06-29 21:45:41','orbital','success',680,'47183-457','Kosmos-1383','Nadezhda 11F643N No. 514','','Sat',456,'',null from dual union all
select 54376,'1982-067','1982-06-30 15:00:00','orbital','success',1238,'78017  -054','Kosmos-1384','Feniks No. 954','','Sat',473,'',null from dual union all
select 54377,'1982-068','1982-07-06 07:50:00','orbital','success',1238,'77049  -433','Kosmos-1385','Zenit-6','','Sat',473,'',null from dual union all
select 54378,'1982-069','1982-07-07 09:47:00','orbital','success',680,'65093-473','Kosmos-1386','Parus','','Sat',456,'',null from dual union all
select 54379,'1982-070','1982-07-10 09:57:44','orbital','success',1238,'Sh15000-318','Progress-14','Progress 7K-TG No. 117','','Sat',480,'',null from dual union all
select 5438,'1958-S98','1958-07-11 18:30:07','suborbital rocket','success',28,'AU3.11F','Charge density','IGY AU3.11F','','Ionos',428,'',243 from dual union all
select 54380,'1982-071','1982-07-13 08:00:00','orbital','success',1238,'76054  -081','Kosmos-1387','Fram','','Sat',474,'',null from dual union all
select 54381,'1982-072','1982-07-16 17:59:00','orbital','success',391,'648/D163','Landsat 4','Landsat D','','Sat',1724,'',null from dual union all
select 54382,'1982-073','1982-07-21 06:31:00','orbital','success',680,'65055-103','Kosmos-1388/1395','Strela-1M','','Sat',457,'',null from dual union all
select 54383,'1982-074','1982-07-21 09:40:02','orbital','success',809,'76054  -652','Molniya-1','Molniya-1 11F658 No. 62','','Sat',480,'',null from dual union all
select 54384,'1982-075','1982-07-27 12:30:00','orbital','success',1238,'77049  -432','Kosmos-1396','Zenit-6','','Sat',462,'',null from dual union all
select 54385,'1982-076','1982-07-29 19:40:00','orbital','success',680,'53793-472','Kosmos-1397','Romb','','Sat',622,'',null from dual union all
select 54386,'1982-077','1982-08-03 11:30:00','orbital','success',1238,'77049  -803','Kosmos-1398','Orion','','Sat',474,'',null from dual union all
select 54387,'1982-078','1982-08-04 11:30:00','orbital','success',1238,'77049  -216','Kosmos-1399','Oktan No. 216','','Sat',562,'',null from dual union all
select 54388,'1982-079','1982-08-05 06:56:00','orbital','success',1510,'76053  -341','Kosmos-1400','Tselina-D','','Sat',462,'',null from dual union all
select 54389,'1982-080','1982-08-19 17:11:52','orbital','success',1238,'Ts15000-276','Soyuz T-7','Soyuz 7K-ST No. 12L','','Sat',480,'',null from dual union all
select 5439,'1958-S99','1958-07-12 20:30:00','suborbital rocket','success',839,'OB6.10','Horizon imaging','IGY OB6.10','','Aeron/Imaging',428,'',114 from dual union all
select 54390,'1982-081','1982-08-20 09:50:00','orbital','success',1238,'76054  -119','Kosmos-1401','Resurs-F1 17F41 No. 19','','Sat',473,'',null from dual union all
select 54391,'1982-082','1982-08-26 23:10:00','orbital','success',393,'651/D164','Anik D-1','Anik D-1','','Sat',234,'',null from dual union all
select 54392,'1982-083','1982-08-27 00:02:00','orbital','success',809,'76047  -547','Molniya-3','Molniya-3 No. 33','','Sat',473,'',null from dual union all
select 54393,'1982-084','1982-08-30 10:06:00','orbital','success',1446,'45033-621','Kosmos-1402','US-A','','Sat',589,'',null from dual union all
select 54394,'1982-085','1982-09-01 09:00:00','orbital','success',1238,'76054  -763','Kosmos-1403','Zenit-6','','Sat',562,'',null from dual union all
select 54395,'1982-086','1982-09-01 11:40:00','orbital','success',1238,'78017  -944','Kosmos-1404','Zenit-6','','Sat',474,'',null from dual union all
select 54396,'1982-087','1982-09-03 05:00:00','orbital','success',822,'N-9(F)','Kiku-4','ETS 3','','Sat',1474,'',null from dual union all
select 54397,'1982-088','1982-09-04 17:50:00','orbital','success',1446,'67083-623','Kosmos-1405','US-P','','Sat',589,'',null from dual union all
select 54398,'1982-089','1982-09-08 10:20:00','orbital','success',1238,'77024  -082','Kosmos-1406','Fram','','Sat',473,'',null from dual union all
select 54399,'1982-090','1982-09-09 07:19:00','orbital','success',301,'Y1       (7)','FSW','FSW No. 5','','Sat',810,'',null from dual union all
select 544,'1945-S20','1945-01-11 17:05:00','military missile','success',8,'20806','Ma399','','','Test',661,'',90 from dual union all
select 5440,'1958-U01','1958-09-26 15:38:00','orbital','failure',1486,'SLV-3','Vanguard SLV3','Cloud cover satellite','','Sat',235,'',426 from dual union all
select 54400,'1982-091','1982-09-15 15:30:00','orbital','success',1238,'77049  -955','Kosmos-1407','Feniks No. 955','','Sat',473,'',null from dual union all
select 54401,'1982-092','1982-09-16 04:55:00','orbital','success',1448,'45029-806','Kosmos-1408','Tselina-D','','Sat',470,'',null from dual union all
select 54402,'1982-093','1982-09-16 18:31:00','orbital','success',956,'309-01','Ekran','Ekran No. 24L','','Sat',554,'',null from dual union all
select 54403,'1982-094','1982-09-18 04:58:54','orbital','success',1238,'Ts15000-292','Progress-15','Progress 7K-TG No. 112','','Sat',480,'',null from dual union all
select 54404,'1982-095','1982-09-22 06:23:11','orbital','success',809,'76053  -639','Kosmos-1409','Oko','','Sat',462,'',null from dual union all
select 54405,'1982-096','1982-09-24 09:15:00','orbital','success',1448,'45029-803','Kosmos-1410','Musson No. 13','','Sat',469,'',null from dual union all
select 54406,'1982-097','1982-09-28 23:17:00','orbital','success',180,'AC-60','Intelsat 505','INTELSAT V F5','','Sat',269,'',null from dual union all
select 54407,'1982-098','1982-09-30 11:50:00','orbital','success',1238,'76054  -761','Kosmos-1411','Zenit-6','','Sat',462,'',null from dual union all
select 54408,'1982-099','1982-10-02 00:01:00','orbital','success',1446,'51067-403','Kosmos-1412','US-A reactor','','Sat',589,'',null from dual union all
select 54409,'1982-100','1982-10-12 14:56:59','orbital','success',958,'315-01','Kosmos-1413/1415','Uragan No. 11L','','Sat',553,'',null from dual union all
select 5441,'1958-W01','1958-01-01 00:00:00','suborbital rocket','success',124,'','Test','','','Test',1895,'',50 from dual union all
select 54410,'1982-101','1982-10-14 09:10:00','orbital','success',1238,'76054  -764','Kosmos-1416','Zenit-6','','Sat',562,'',null from dual union all
select 54411,'1982-102','1982-10-19 05:58:00','orbital','success',680,'53734-137','Kosmos-1417','Parus','','Sat',456,'',null from dual union all
select 54412,'1982-103','1982-10-20 16:26:00','orbital','success',956,'312-01','Gorizont','Gorizont No. 16L','','Sat',554,'',null from dual union all
select 54413,'1982-104','1982-10-21 14:00:00','orbital','success',680,'47193-471','Kosmos-1418','Taifun-1B','','Sat',622,'',null from dual union all
select 54414,'1982-105','1982-10-28 01:27:49','orbital','success',394,'652/D165','Satcom 5','RCA-E','','Sat',234,'',null from dual union all
select 54415,'1982-106','1982-10-30 04:05:01','orbital','success',1407,'34D-1  (04D-5)','DSCS II/DSCS III','DSCS II F-16','','Sat',274,'',null from dual union all
select 54416,'1982-107','1982-10-31 11:20:36','orbital','success',1238,'Sh15000-335','Progress-16','Progress 7K-TG No. 115','','Sat',480,'',null from dual union all
select 54417,'1982-108','1982-11-02 09:30:00','orbital','success',1238,'76054  -765','Kosmos-1419','Zenit-6','','Sat',562,'',null from dual union all
select 54418,'1982-109','1982-11-11 06:14:00','orbital','success',680,'47167-252','Kosmos-1420','Strela-2M','','Sat',456,'',null from dual union all
select 54419,'1982-110','1982-11-11 12:19:00','orbital','success',1243,'STS-5','PAM-D Deploy','OV-102 PAM deploy','','Sat',921,'MLP1',null from dual union all
select 5442,'1958-W02','1958-08-01 00:00:00','suborbital rocket','success',716,'','Argus training','','','Test',148,'DD-843',70 from dual union all
select 54420,'1982-111','1982-11-17 21:18:00','orbital','success',1426,'23D-23','KH-11','KH11-5 KENNAN','','Sat',1777,'',null from dual union all
select 54421,'1982-112','1982-11-18 09:25:00','orbital','success',1238,'76054  -767','Kosmos-1421','Zenit-6','','Sat',562,'',null from dual union all
select 54422,'1982-113','1982-11-26 14:13:00','orbital','success',956,'313-01','Raduga','Gran'' No. 21L','','Sat',553,'',null from dual union all
select 54423,'1982-114','1982-12-03 12:00:00','orbital','success',1238,'76054  -766','Kosmos-1422','Zenit-6','','Sat',474,'',null from dual union all
select 54424,'1982-115','1982-12-08 13:46:00','orbital','failure',809,'77024  -656','Kosmos-1423','Molniya-1 11F658 No. 60','','Sat',480,'',488 from dual union all
select 54425,'1982-116','1982-12-14 22:30:00','orbital','success',1510,'78017  -356','Meteor-2','Meteor-2 No. 9','','Sat',474,'',null from dual union all
select 54426,'1982-117','1982-12-16 10:00:00','orbital','success',1238,'77049  -217','Kosmos-1424','Oktan No. 217','','Sat',562,'',null from dual union all
select 54427,'1982-118','1982-12-21 02:38:00','orbital','success',155,'60E','DMSP F6','DMSP Block 5D-2 S-6','','Sat',1776,'',null from dual union all
select 54428,'1982-119','1982-12-23 09:10:00','orbital','success',1242,'77024  -777 (002)','Kosmos-1425','Zenit-6','','Sat',480,'',null from dual union all
select 54429,'1982-120','1982-12-28 12:00:00','orbital','success',1238,'Sh15000-332','Kosmos-1426','Yantar''-4KS1','','Sat',562,'',null from dual union all
select 5443,'1958-W03','1958-08-01 00:00:00','suborbital rocket','success',716,'','Argus training','','','Test',148,'DD-843',70 from dual union all
select 54430,'1982-121','1982-12-29 12:00:02','orbital','success',680,'53783-454','Kosmos-1427','Taifun-1B','','Sat',456,'',null from dual union all
select 54431,'1982-A01','1982-01-28 15:15:00','military missile','failure',914,'45033-621(2L)','15A11 LKI-3','GCh 15B99','','Test',540,'',1 from dual union all
select 54432,'1982-A02','1982-02-03 16:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',48 from dual union all
select 54433,'1982-A03','1982-02-05 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 54434,'1982-A04','1982-02-24 02:00:00','suborbital rocket','success',761,'MT135P-R450','','','','Meteo',1332,'',45 from dual union all
select 54435,'1982-A05','1982-02-28 19:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',47 from dual union all
select 54436,'1982-A06','1982-03-10 02:01:00','suborbital rocket','success',761,'MT135P-R451','','','','Meteo',1332,'',24 from dual union all
select 54437,'1982-A07','1982-03-16 05:10:00','suborbital rocket','failure',224,'AAF-IXC-1','WATERHOLE 2','','','Auroral/Aeron/Ionos',428,'',3 from dual union all
select 54438,'1982-A08','1982-03-17 02:20:00','suborbital rocket','success',761,'MT135P-R452','','','','Meteo',1332,'',49 from dual union all
select 54439,'1982-A09','1982-04-21 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 5444,'1958-W04','1958-08-01 00:00:00','suborbital rocket','success',716,'','Argus training','','','Test',148,'DD-843',70 from dual union all
select 54440,'1982-A10','1982-05-05 15:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',48 from dual union all
select 54441,'1982-A11','1982-05-12 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',43 from dual union all
select 54442,'1982-A12','1982-05-19 02:00:00','suborbital rocket','success',761,'MT135P-R459','','','','Meteo',1332,'',43 from dual union all
select 54443,'1982-A13','1982-05-31 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',42 from dual union all
select 54444,'1982-A14','1982-06-16 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',44 from dual union all
select 54445,'1982-A15','1982-07-13 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 54446,'1982-A16','1982-07-14 00:00:00','military missile','unknown',256,'B','Target','','','Target',1690,'',20 from dual union all
select 54447,'1982-A17','1982-07-14 00:00:00','military missile','unknown',256,'B','Target','','','Target',1689,'',20 from dual union all
select 54448,'1982-A18','1982-08-24 02:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 54449,'1982-A19','1982-09-09 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',42 from dual union all
select 5445,'1958-W05','1958-08-01 00:00:00','suborbital rocket','success',716,'','Argus training','','','Test',148,'DD-843',70 from dual union all
select 54450,'1982-A20','1982-09-15 22:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 54451,'1982-A21','1982-09-17 23:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 54452,'1982-A22','1982-09-29 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 54453,'1982-A23','1982-10-06 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 54454,'1982-A24','1982-10-08 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 54455,'1982-A25','1982-10-13 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',44 from dual union all
select 54456,'1982-A26','1982-10-18 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 54457,'1982-A27','1982-10-20 15:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',48 from dual union all
select 54458,'1982-A28','1982-10-20 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual;
insert into LAUNCH
select 54459,'1982-A29','1982-10-25 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 5446,'1958-W06','1958-08-01 00:00:00','suborbital rocket','success',716,'','Argus training','','','Test',148,'DD-843',70 from dual union all
select 54460,'1982-A30','1982-10-29 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 54461,'1982-A31','1982-11-04 03:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 54462,'1982-A32','1982-11-09 17:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',48 from dual union all
select 54463,'1982-A33','1982-11-12 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',40 from dual union all
select 54464,'1982-A34','1982-11-16 18:30:00','suborbital rocket','failure',211,'NASA 27.73US','','','','Solar',1895,'',10 from dual union all
select 54465,'1982-A35','1982-12-11 03:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 54466,'1982-A36','1982-12-13 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 54467,'1982-A37','1982-12-21 03:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 54468,'1982-A38','1982-12-22 00:55:00','military missile','failure',1455,'63024-82-025T(1L)','15A35S LKI-1','GCh 15F357P1 (15F358 x6)','','OT',505,'',0 from dual union all
select 54469,'1982-A39','1982-12-22 22:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',44 from dual union all
select 5447,'1958-W07','1958-08-01 00:00:00','suborbital rocket','success',716,'','Argus training','','','Test',148,'DD-843',70 from dual union all
select 54470,'1982-A40','1982-12-29 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 54471,'1982-A41','1982-04-01 00:00:00','military missile','success',897,'PRS-1','Intercept Test','','','ABM Test',183,'',40 from dual union all
select 54472,'1982-A42','1982-06-18 00:00:00','military missile','success',897,'PRS-1','Intercept Test','','','ABM Test',183,'',40 from dual union all
select 54473,'1982-A43','1982-08-25 00:00:00','military missile','success',1434,'','15Zh58 B-3','Ejection test','','Test',477,'',1 from dual union all
select 54474,'1982-F01','1982-03-04 16:41:00','orbital','failure',680,'53739-530','','Romb?','','Sat',622,'',20 from dual union all
select 54475,'1982-F02','1982-05-15 14:20:00','orbital','failure',1238,'77049  -431','','Zenit-6','','Sat',473,'',10 from dual union all
select 54476,'1982-F03','1982-06-12 09:00:00','orbital','failure',1238,'77049  -435','','Zenit-6','','Sat',562,'',80 from dual union all
select 54477,'1982-F04','1982-07-22 22:11:00','orbital','failure',956,'307-02','','Ekran No. 23L','','Sat',554,'',10 from dual union all
select 54478,'1982-F05','1982-08-30 19:55:00','orbital','failure',680,'65047-236','','Strela-2M','','Sat',457,'',800 from dual union all
select 54479,'1982-F06','1982-09-09 02:12:00','orbital','failure',91,'L5','Marecs B/Sirio B','Marecs B','','Sat',344,'',220 from dual union all
select 5448,'1958-W08','1958-08-01 00:00:00','suborbital rocket','success',716,'','Argus training','','','Test',148,'DD-843',70 from dual union all
select 54480,'1982-F07','1982-11-24 11:00:00','orbital','failure',680,'47183-465','','Strela-1M','','Sat',456,'',40 from dual union all
select 54481,'1982-F08','1982-12-24 12:00:00','orbital','failure',956,'314-01','','Gran'' No. 22L','','Sat',553,'',60 from dual union all
select 54482,'1982-M01','1982-02-01 00:00:00','suborbital rocket','success',1023,'','Thermistor test','','','Meteo',1467,'',70 from dual union all
select 54483,'1982-M02','1982-03-01 00:00:00','suborbital rocket','success',1023,'','Thermistor test','','','Meteo',1467,'',70 from dual union all
select 54484,'1982-M03','1982-03-01 00:00:00','suborbital rocket','success',1023,'','Thermistor test','','','Meteo',1467,'',70 from dual union all
select 54485,'1982-M04','1982-04-01 00:00:00','suborbital rocket','success',1023,'','Thermistor test','','','Meteo',1467,'',70 from dual union all
select 54486,'1982-M05','1982-06-23 15:40:00','suborbital rocket','success',860,'NASA 31.30GE','VLF SEEP 1','','','Plasma',1821,'',85 from dual union all
select 54487,'1982-M06','1982-07-07 09:01:00','suborbital rocket','success',860,'NASA 31.31GE','VLF SEEP 2','','','Plasma',1821,'',80 from dual union all
select 54488,'1982-M07','1982-03-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 54489,'1982-M08','1982-03-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 5449,'1958-W09','1958-08-01 00:00:00','suborbital rocket','success',716,'','Argus training','','','Test',148,'DD-843',70 from dual union all
select 54490,'1982-M09','1982-03-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 54491,'1982-M10','1982-03-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54492,'1982-M11','1982-03-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54493,'1982-M12','1982-03-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54494,'1982-M13','1982-03-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54495,'1982-M14','1982-03-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54496,'1982-M15','1982-04-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 54497,'1982-M16','1982-04-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 54498,'1982-M17','1982-04-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 54499,'1982-M18','1982-04-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 545,'1945-S21','1945-02-08 00:00:00','military missile','unknown',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5450,'1958-W10','1958-08-01 00:00:00','suborbital rocket','success',716,'','Argus training','','','Test',148,'DD-843',70 from dual union all
select 54500,'1982-M19','1982-04-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54501,'1982-M20','1982-04-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54502,'1982-M21','1982-04-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54503,'1982-M22','1982-10-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54504,'1982-M23','1982-10-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54505,'1982-M24','1982-11-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54506,'1982-M25','1982-11-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54507,'1982-M26','1982-11-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54508,'1982-M27','1982-11-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54509,'1982-M28','1982-11-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5451,'1958-W11','1958-08-01 00:00:00','suborbital rocket','success',716,'','Argus training','','','Test',148,'DD-843',70 from dual union all
select 54510,'1982-M29','1982-11-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54511,'1982-M30','1982-11-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54512,'1982-M31','1982-12-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54513,'1982-M32','1982-12-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54514,'1982-M33','1982-12-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54515,'1982-M34','1982-12-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54516,'1982-M35','1982-05-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54517,'1982-M36','1982-06-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 54518,'1982-M37','1982-05-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 54519,'1982-M38','1982-05-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5452,'1958-W12','1958-08-01 00:00:00','suborbital rocket','success',716,'','Argus training','','','Test',148,'DD-843',70 from dual union all
select 54520,'1982-M39','1982-05-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54521,'1982-M40','1982-01-22 05:40:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 54522,'1982-M41','1982-01-22 06:30:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 54523,'1982-M42','1982-01-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 54524,'1982-M43','1982-03-03 13:52:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 54525,'1982-M44','1982-03-03 16:16:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 54526,'1982-M45','1982-04-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 54527,'1982-M46','1982-03-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 54528,'1982-M47','1982-04-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1361,'',60 from dual union all
select 54529,'1982-M48','1982-05-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5453,'1958-W13','1958-08-01 00:00:00','suborbital rocket','success',716,'','Argus training','','','Test',148,'DD-843',70 from dual union all
select 54530,'1982-M49','1982-06-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 54531,'1982-M50','1982-04-02 15:32:00','suborbital rocket','success',1281,'','VLF Probe 5','','','Ionos',1821,'',75 from dual union all
select 54532,'1982-M51','1982-10-30 00:00:00','military missile','success',592,'','','','','Test',33,'',50 from dual union all
select 54533,'1982-S01','1982-04-26 15:19:59','military missile','success',914,'67033-623(4L)','15A11 LKI-4','GCh 15B99','','Test',540,'',1000 from dual union all
select 54534,'1982-S02','1982-06-29 15:30:07','military missile','success',914,'45033-624(5L)','15A11 LKI-5','GCh 15B99','','Test',546,'',1000 from dual union all
select 54535,'1982-S03','1982-10-22 11:14:55','military missile','success',914,'45067-402(6L)','15A11 LKI-6','GCh 15B99','','Test',540,'',1000 from dual union all
select 54536,'1982-S04','1982-11-24 00:00:00','military missile','success',914,'51033-622(3L)','15A11 LKI-7','GCh 15B99','','Test',546,'',1000 from dual union all
select 54537,'1982-S05','1982-01-01 00:00:00','military missile','success',756,'6','','','','Test',301,'',800 from dual union all
select 54538,'1982-S06','1982-01-01 00:00:00','military missile','success',756,'7','','','','Test',301,'',800 from dual union all
select 54539,'1982-S07','1982-01-01 00:00:00','military missile','success',756,'','','','','Test',307,'S655',800 from dual union all
select 5454,'1958-W14','1958-08-12 00:00:00','suborbital rocket','success',716,'','Argus training','','','Test',148,'DD-843',70 from dual union all
select 54540,'1982-S08','1982-01-01 00:00:00','military missile','success',756,'','','','','Test',307,'S655',800 from dual union all
select 54541,'1982-S09','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54542,'1982-S10','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54543,'1982-S100','1982-04-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54544,'1982-S101','1982-04-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54545,'1982-S102','1982-04-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54546,'1982-S103','1982-04-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54547,'1982-S104','1982-03-31 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT87GB','GLORY TRIP 87GB','OT',1712,'',1300 from dual union all
select 54548,'1982-S105','1982-04-15 07:20:00','suborbital rocket','success',867,'NASA 18.1028GE','','','','Plasma',1047,'',270 from dual;
insert into LAUNCH
select 54549,'1982-S106','1982-04-17 06:15:00','suborbital rocket','success',211,'NASA 27.59GG','UIT','','','UV Astron',1895,'',279 from dual union all
select 5455,'1958-W15','1958-08-22 00:00:00','suborbital rocket','success',716,'','Argus training','','','Test',148,'DD-843',70 from dual union all
select 54550,'1982-S107','1982-04-16 06:29:00','sounding rocket','success',231,'NASA 35.003GE','','','','Plasma',1047,'',1500 from dual union all
select 54551,'1982-S108','1982-04-16 07:32:00','suborbital rocket','success',867,'NASA 18.1029GE','','','','Plasma',1047,'',270 from dual union all
select 54552,'1982-S109','1982-04-20 06:30:00','suborbital rocket','success',231,'NASA 35.004GE','','','','Plasma',1047,'',1195 from dual union all
select 54553,'1982-S11','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54554,'1982-S110','1982-04-24 17:52:00','military missile','success',1442,'','DASO-12','','','OT',421,'SSBN 643',1000 from dual union all
select 54555,'1982-S111','1982-04-29 05:20:00','suborbital rocket','success',1212,'DLR K-GR-113','DLR TEXUS 5','','','Micrograv',416,'',256 from dual union all
select 54556,'1982-S112','1982-04-30 00:00:00','military missile','success',614,'CSS-NX-3','','','','Test',1909,'HULU',500 from dual union all
select 54557,'1982-S113','1982-05-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54558,'1982-S114','1982-05-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54559,'1982-S115','1982-05-08 05:30:00','suborbital rocket','success',1212,'DLR K-GR-114','DLR TEXUS 6','','','Micrograv',416,'',256 from dual union all
select 5456,'1958-W16','1958-09-03 17:03:00','suborbital rocket','success',716,'II9.01F','Stability test','IGY II9.01F','','Test/CR/Plasma',428,'',75 from dual union all
select 54560,'1982-S116','1982-05-17 00:00:00','suborbital rocket','success',1319,'NASA 33.28UU','','','','Aeron',1873,'',200 from dual union all
select 54561,'1982-S117','1982-05-25 00:00:00','military missile','success',1106,'2603','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 54562,'1982-S118','1982-05-25 00:00:00','military missile','success',939,'','FOT-46','','','OT',421,'SSBN 619',500 from dual union all
select 54563,'1982-S119','1982-05-25 00:00:00','military missile','success',939,'','FOT-46','','','OT',421,'SSBN 619',500 from dual union all
select 54564,'1982-S12','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54565,'1982-S120','1982-05-25 00:00:00','military missile','success',939,'','FOT-46','','','OT',421,'SSBN 619',500 from dual union all
select 54566,'1982-S121','1982-05-25 23:13:00','military missile','success',939,'','FOT-46','','','OT',421,'SSBN 619',500 from dual union all
select 54567,'1982-S122','1982-05-28 15:47:00','suborbital rocket','success',117,'GL A24.609-03','TEM-3','','','ABM Tech IR',1873,'',236 from dual union all
select 54568,'1982-S123','1982-06-18 00:00:00','military missile','success',10,'','Intercept Test','','','ABM Test',183,'',100 from dual union all
select 54569,'1982-S124','1982-06-05 20:00:00','military missile','success',1442,'','DASO-13','','','OT',421,'SSBN 627',1000 from dual union all
select 5457,'1958-W17','1958-09-03 19:30:00','suborbital rocket','success',716,'II9.02F','Radiation','IGY II9.02F','','CR/Plasma',428,'',75 from dual union all
select 54570,'1982-S125','1982-06-18 00:00:00','military missile','success',922,'','ABM Target','','','Target',636,'',500 from dual union all
select 54571,'1982-S128','1982-06-18 04:45:00','suborbital rocket','success',117,'NASA 24.04UG','1m UVT','','','EUV Astron',1873,'',247 from dual union all
select 54572,'1982-S129','1982-06-22 14:27:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT88GM','GLORY TRIP 88GM','OT',1703,'',1300 from dual union all
select 54573,'1982-S13','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54574,'1982-S130','1982-06-22 00:00:00','suborbital rocket','success',211,'NASA 27.68AH','','','','XR Astron',1895,'',245 from dual union all
select 54575,'1982-S131','1982-06-26 00:00:00','military missile','success',1106,'2605','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 54576,'1982-S132','1982-06-26 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 620',500 from dual union all
select 54577,'1982-S133','1982-06-26 02:57:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 620',500 from dual union all
select 54578,'1982-S134','1982-06-29 11:01:00','suborbital rocket','success',1319,'GL A13.277','TRACER','','','Aeron',1821,'',147 from dual union all
select 54579,'1982-S135','1982-06-30 00:00:00','suborbital rocket','success',1219,'XV-11','BIME Test Flight','','','Test',330,'',549 from dual union all
select 5458,'1958-W18','1958-09-04 19:30:00','suborbital rocket','success',716,'II9.03F','','IGY II9.03F','','CR/Plasma',428,'',75 from dual union all
select 54580,'1982-S136','1982-07-13 16:30:00','suborbital rocket','success',211,'NASA 27.54CS','XSST/TRC 3','','','Solar UV,X',1895,'',302 from dual union all
select 54581,'1982-S137','1982-07-15 00:00:00','suborbital rocket','success',860,'NASA 31.15UE','','','','Plasma',1821,'',140 from dual union all
select 54582,'1982-S138','1982-07-15 19:25:00','military missile','success',1004,'45046-315-08','15A14 UBP-29','GCh 15F143U1','','Test',497,'',1000 from dual union all
select 54583,'1982-S139','1982-07-19 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT142M','GLORY TRIP 142M','OT',1699,'',1300 from dual union all
select 54584,'1982-S14','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54585,'1982-S140','1982-07-20 00:00:00','suborbital rocket','success',211,'NASA 27.53US','CfA/HAO Rocket Coron. 3','','','Solar Opt/Solar UV',1895,'',254 from dual union all
select 54586,'1982-S141','1982-07-22 14:49:00','military missile','failure',919,'ED-1','P-431 P-II F1','','','Test',231,'',1 from dual union all
select 54587,'1982-S142','1982-08-04 00:16:00','suborbital rocket','success',860,'','CAMP TAD','','','Aeronomy',416,'',153 from dual union all
select 54588,'1982-S143','1982-08-03 00:00:00','suborbital rocket','success',891,'NASA 30.21GU','','','','Aeron',1895,'',85 from dual union all
select 54589,'1982-S144','1982-08-03 23:19:10','suborbital rocket','success',860,'DLR K-NO-115','Strafam 1/2','','','Tech/Aeron',410,'',114 from dual union all
select 5459,'1958-W19','1958-09-08 02:04:00','suborbital rocket','success',716,'II9.05F','','IGY II9.05F','','CR/Plasma',428,'',75 from dual union all
select 54590,'1982-S145','1982-08-03 23:02:00','suborbital rocket','success',860,'SSC S37 CAMP-N','Cold Arctic Mesopause','','','Aeronomy',408,'',119 from dual union all
select 54591,'1982-S146','1982-08-03 23:32:00','suborbital rocket','success',860,'SSC S37 CAMP-P','CAMP-P','','','Aeronomy',408,'',119 from dual union all
select 54592,'1982-S147','1982-08-03 23:48:30','suborbital rocket','success',860,'SSC S37 SOAP/1','SOAP-1','','','Aeronomy',412,'',134 from dual union all
select 54593,'1982-S148','1982-08-05 02:05:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT89GM','GLORY TRIP 89GM','OT',1705,'',1300 from dual union all
select 54594,'1982-S149','1982-08-04 00:03:00','suborbital rocket','success',920,'P235K','CAMP Support','','','Aeronomy',415,'',140 from dual union all
select 54595,'1982-S15','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54596,'1982-S150','1982-08-09 15:00:00','suborbital rocket','success',860,'NASA 31.34UU','CWAS 4','','','Aeronomy',1895,'',140 from dual union all
select 54597,'1982-S151','1982-08-10 17:16:00','suborbital rocket','success',211,'NASA 27.76UL','EUV Abs Cal','','','Solar EUV',1895,'',323 from dual union all
select 54598,'1982-S152','1982-08-12 00:30:15','suborbital rocket','success',860,'SSC S37 SOAP/2','SOAP-2','','','Aeronomy',412,'',135 from dual union all
select 54599,'1982-S154','1982-08-11 00:35:00','suborbital rocket','success',1319,'NASA 33.29UL','','','','Aeron',1047,'',200 from dual union all
select 546,'1945-S22','1945-02-10 08:53:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5460,'1958-W20','1958-09-08 02:56:00','suborbital rocket','success',716,'II9.06F','','IGY II9.06F','','CR/Plasma',428,'',75 from dual union all
select 54600,'1982-S155','1982-08-11 23:30:00','suborbital rocket','success',920,'P214K','CAMP Support','','','Aeronomy',415,'',129 from dual union all
select 54601,'1982-S156','1982-08-12 19:05:00','military missile','success',1004,'45046-315-05','15A14 UBP-30','GCh 15F143U1','','Test',493,'',1000 from dual union all
select 54602,'1982-S157','1982-08-15 23:15:00','suborbital rocket','success',1300,'TT-500A-11F','','','','Test',1476,'',320 from dual union all
select 54603,'1982-S158','1982-09-06 02:00:00','suborbital rocket','success',1074,'S-520-5','0.1m telescope','','','Solar EUV',833,'',237 from dual union all
select 54604,'1982-S159','1982-09-08 21:42:00','suborbital rocket','success',210,'GL A19.124-1','BIME','','','Ionos',330,'',321 from dual union all
select 54605,'1982-S16','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54606,'1982-S160','1982-09-08 21:52:00','suborbital rocket','success',1219,'XV-12','GL A20.123-01 BIME','','','Ionos',330,'',525 from dual union all
select 54607,'1982-S161','1982-09-09 10:17:00','suborbital rocket','success',117,'DLR M-AR-116','Conestoga 1','','','Test',948,'',309 from dual union all
select 54608,'1982-S162','1982-09-12 12:00:00','suborbital rocket','success',1070,'S-210-13','','','','Aeron',839,'',97 from dual union all
select 54609,'1982-S163','1982-09-13 12:30:00','suborbital rocket','success',668,'K-9M-75','NIR Background','','','IR Astron',839,'',320 from dual union all
select 5461,'1958-W21','1958-11-11 21:18:00','suborbital rocket','success',716,'II9.07F','Radiation','IGY II9.07F','','CR/Plasma',428,'',75 from dual union all
select 54610,'1982-S164','1982-09-13 21:05:00','suborbital rocket','success',210,'GL A19.124-2','BIME','','','Ionos',330,'',333 from dual union all
select 54611,'1982-S165','1982-09-13 21:05:00','suborbital rocket','success',1219,'XV-13','GL A20.123-02 BIME','','','Ionos',330,'',506 from dual union all
select 54612,'1982-S166','1982-09-15 04:49:00','suborbital rocket','success',117,'GL A24.7S2-03','SPICE 2','','','IR Astron',1873,'',359 from dual union all
select 54613,'1982-S167','1982-09-15 18:15:00','suborbital rocket','success',228,'AMF-VC-53','','','','Solar IR',1873,'',243 from dual union all
select 54614,'1982-S168','1982-09-17 20:56:00','suborbital rocket','success',1219,'XV-14','Colored Bubbles','','','Ionos',330,'',335 from dual union all
select 54615,'1982-S169','1982-09-18 20:45:00','suborbital rocket','success',1219,'XV-15','Colored Bubbles','','','Ionos',330,'',335 from dual union all
select 54616,'1982-S17','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54617,'1982-S170','1982-09-24 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT90GB','GLORY TRIP 90GB','OT',1712,'',1300 from dual union all
select 54618,'1982-S171','1982-09-25 09:15:00','suborbital rocket','success',1073,'S-310-12','','','','Aeron',839,'',200 from dual union all
select 54619,'1982-S172','1982-10-05 15:10:24','suborbital rocket','success',867,'Ferdinand 61','Hero 4','','','Ionospheric/Active',24,'',296 from dual union all
select 5462,'1958-W22','1958-11-13 21:15:00','suborbital rocket','success',716,'II9.08F','','IGY II9.08F','','CR/Plasma',428,'',75 from dual union all
select 54620,'1982-S173','1982-10-08 00:00:00','military missile','success',795,'','STREP-21','RSLP STREP-21','','Reentry Test',1698,'',1300 from dual union all
select 54621,'1982-S174','1982-10-12 00:00:00','military missile','success',614,'','','','','Test',1909,'GOLF-C',500 from dual union all
select 54622,'1982-S175','1982-10-14 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT143MS-1','GLORY TRIP 143MS-1','OT',1703,'',1300 from dual union all
select 54623,'1982-S176','1982-10-14 00:00:00','military missile','success',1057,'26     /36','4 RV','','','Test',190,'TK-208',1000 from dual union all
select 54624,'1982-S177','1982-10-14 00:00:00','military missile','success',1057,'32     8/93','1 RV','','','Test',190,'TK-208',1000 from dual union all
select 54625,'1982-S178','1982-10-14 00:00:00','military missile','success',1057,'28     9/','4 RV','','','Test',190,'TK-208',1000 from dual union all
select 54626,'1982-S18','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54627,'1982-S180','1982-10-23 06:17:08','suborbital rocket','success',867,'Ferdinand 60','Hero 3','','','Ionospheric/Active',24,'',297 from dual union all
select 54628,'1982-S181','1982-10-23 08:00:00','suborbital rocket','success',211,'NASA 27.48UG','FUV Echelle','','','UV Astron',1895,'',262 from dual union all
select 54629,'1982-S182','1982-10-26 00:00:00','military missile','success',1039,'48764-251-1L','15Zh44 LKI-1','silo','','Test',466,'',1000 from dual union all
select 5463,'1958-W23','1958-11-13 21:45:00','suborbital rocket','success',716,'II9.09F','','IGY II9.09F','','CR/Plasma',428,'',75 from dual union all
select 54630,'1982-S183','1982-10-27 00:00:00','military missile','success',1434,'','15Zh58 LKI-1-1','','','Test',636,'',100 from dual union all
select 54631,'1982-S186','1982-11-01 08:16:00','suborbital rocket','success',228,'NASA 21.69UH','MIT WFSXC 2','','','XR Astron',1873,'',210 from dual union all
select 54632,'1982-S187','1982-11-09 17:45:00','suborbital rocket','success',227,'NASA 21.64GS','EUV line profiles','','','Solar',1873,'',233 from dual union all
select 54633,'1982-S188','1982-11-14 07:55:15','suborbital rocket','success',224,'NASA 36.01UE','ARCS 2','','','Plasma',1047,'',473 from dual union all
select 54634,'1982-S189','1982-11-14 17:00:00','military missile','success',1442,'','DASO','','','OT',421,'SSBN 632',1000 from dual union all
select 54635,'1982-S19','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54636,'1982-S190','1982-11-15 08:00:00','suborbital rocket','success',44,'NASA 17.19DG','','','','UV Astron',1895,'',300 from dual union all
select 54637,'1982-S191','1982-11-19 18:30:00','military missile','success',919,'ED-3','P-436','','','Test',433,'',320 from dual union all
select 54638,'1982-S192','1982-11-21 17:34:00','military missile','success',1442,'DASO-15','Adv Evader211A','','','OT',421,'SSBN 727',1000 from dual;
insert into LAUNCH
select 54639,'1982-S193','1982-11-24 13:39:48','suborbital rocket','success',1319,'NASA 33.24CE','','','','Plasma',1047,'',200 from dual union all
select 5464,'1958-W24','1958-11-17 18:00:00','suborbital rocket','success',716,'II9.10F','','IGY II9.10F','','CR/Plasma',428,'',75 from dual union all
select 54640,'1982-S194','1982-12-28 00:00:00','military missile','success',1039,'48764-252-2L','15Zh44 LKI-2','','','Test',477,'',1000 from dual union all
select 54641,'1982-S195','1982-12-01 15:30:24','suborbital rocket','success',867,'Ferdinand 59','Hero 2','','','Ionospheric/Active',24,'',295 from dual union all
select 54642,'1982-S196','1982-12-02 13:45:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT91GM','GLORY TRIP 91GM','OT',1705,'',1300 from dual union all
select 54643,'1982-S198','1982-12-13 10:20:00','suborbital rocket','success',228,'NASA 21.74UG','JHU UVT F10','','','UV Astron',1873,'',232 from dual union all
select 54644,'1982-S199','1982-12-14 14:22:09','suborbital rocket','success',867,'Ferdinand 58','Hero 1','','','Ionospheric/Active',24,'',296 from dual union all
select 54645,'1982-S20','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54646,'1982-S200','1982-12-23 13:20:00','military missile','success',1004,'45046-315-15','15A14 UBP-31','GCh 15F143U1','','Test',497,'',1000 from dual union all
select 54647,'1982-S201','1982-11-23 15:10:00','suborbital rocket','success',1028,'F-25/560.25','Spread F','','','Ionos',1361,'',330 from dual union all
select 54648,'1982-S202','1982-11-29 16:50:00','suborbital rocket','success',1028,'F-26/560.26','Spread F','','','Ionos',1361,'',330 from dual union all
select 54649,'1982-S203','1982-02-16 13:39:00','suborbital rocket','success',1028,'ASV.26','','','','Ionos',1361,'',364 from dual union all
select 5465,'1958-W25','1958-11-19 18:30:00','suborbital rocket','success',716,'II9.11F','','IGY II9.11F','','CR/Plasma',428,'',75 from dual union all
select 54650,'1982-S204','1982-03-01 13:35:00','suborbital rocket','success',1028,'ASV.30','','','','Ionos',1361,'',364 from dual union all
select 54651,'1982-S205','1982-01-28 00:00:00','military missile','success',1057,'16     37A/66A','5 RV','','','Test',989,'',1000 from dual union all
select 54652,'1982-S206','1982-02-12 00:00:00','military missile','success',1057,'17     41P/70A','3 RV','','','Test',989,'',1000 from dual union all
select 54653,'1982-S207','1982-03-03 00:00:00','military missile','failure',1057,'23     47A/67A','','','','Test',989,'',100 from dual union all
select 54654,'1982-S208','1982-04-11 00:00:00','military missile','success',1057,'14     41P/73A','5 RV','','','Test',989,'',1000 from dual union all
select 54655,'1982-S209','1982-04-23 00:00:00','military missile','success',1057,'18     76A/74A','2 RV','','','Test',989,'',1000 from dual union all
select 54656,'1982-S21','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54657,'1982-S210','1982-06-09 00:00:00','military missile','success',1057,'20     75/76','4 RV','','','Test',989,'',1000 from dual union all
select 54658,'1982-S211','1982-06-29 00:00:00','military missile','success',1057,'30     79/82','3 RV','','','Test',190,'TK-208',1000 from dual union all
select 54659,'1982-S212','1982-07-21 00:00:00','military missile','success',1057,'27     -','4 RV','','','Test',190,'TK-208',1000 from dual union all
select 5466,'1958-W26','1958-11-20 02:14:00','suborbital rocket','success',716,'II9.12F','','IGY II9.12F','','CR/Plasma',428,'',75 from dual union all
select 54660,'1982-S213','1982-08-12 00:00:00','military missile','success',1057,'33     54P/85','2 RV','','','Test',190,'TK-208',1000 from dual union all
select 54661,'1982-S214','1982-08-12 00:00:00','military missile','success',1057,'24     82/88','3 RV','','','Test',190,'TK-208',1000 from dual union all
select 54662,'1982-S215','1982-09-01 00:00:00','military missile','success',1057,'22     61A/86','','','','Test',190,'TK-208',1000 from dual union all
select 54663,'1982-S216','1982-12-02 00:00:00','military missile','success',1057,'34     84/Akv','2 RV','','','Test',190,'TK-208',1000 from dual union all
select 54664,'1982-S217','1982-12-12 00:00:00','military missile','success',1057,'35     63/','5 RV','','','Test',190,'TK-208',1000 from dual union all
select 54665,'1982-S218','1982-12-12 00:00:00','military missile','success',1057,'36','2 RV','','','Test',190,'TK-208',1000 from dual union all
select 54666,'1982-S219','1982-12-12 00:00:00','military missile','success',1057,'37','4 RV','','','Test',190,'TK-208',1000 from dual union all
select 54667,'1982-S22','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54668,'1982-S220','1982-07-22 00:00:00','military missile','success',1057,'31     83/84','Mil variant','','','Test',190,'TK-208',1000 from dual union all
select 54669,'1982-S221','1982-09-01 00:00:00','military missile','success',1057,'19     72/75','','','','Test',190,'TK-208',1000 from dual union all
select 5467,'1958-Y01','1958-01-10 07:27:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54670,'1982-S222','1982-09-01 00:00:00','military missile','success',1057,'29     86/87','','','','Test',190,'TK-208',1000 from dual union all
select 54671,'1982-S223','1982-01-01 00:00:00','military missile','success',356,'','Long life test','','','Test',810,'',500 from dual union all
select 54672,'1982-S224','1982-01-01 00:00:00','military missile','success',356,'','Long life test','','','Test',810,'',500 from dual union all
select 54673,'1982-S225','1982-05-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54674,'1982-S226','1982-05-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54675,'1982-S227','1982-07-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54676,'1982-S228','1982-07-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54677,'1982-S229','1982-07-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54678,'1982-S23','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54679,'1982-S230','1982-07-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 5468,'1958-Y02','1958-01-19 08:45:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54680,'1982-S231','1982-07-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54681,'1982-S232','1982-07-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54682,'1982-S233','1982-07-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54683,'1982-S234','1982-07-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54684,'1982-S235','1982-09-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54685,'1982-S236','1982-09-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54686,'1982-S237','1982-09-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54687,'1982-S238','1982-09-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54688,'1982-S239','1982-09-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54689,'1982-S24','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 5469,'1958-Y03','1958-01-19 09:45:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54690,'1982-S240','1982-09-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54691,'1982-S241','1982-09-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54692,'1982-S242','1982-09-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54693,'1982-S243','1982-09-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54694,'1982-S244','1982-09-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54695,'1982-S245','1982-09-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54696,'1982-S246','1982-09-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54697,'1982-S247','1982-09-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54698,'1982-S248','1982-09-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54699,'1982-S249','1982-12-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 547,'1945-S23','1945-02-14 15:40:00','military missile','unknown',1470,'4299','Last HVP','','','Test',675,'',90 from dual union all
select 5470,'1958-Y04','1958-01-19 23:21:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54700,'1982-S25','1982-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 54701,'1982-S250','1982-12-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 54702,'1982-S251','1982-08-04 00:00:00','military missile','success',277,'','DOT 5','','','Interceptor',871,'',230 from dual union all
select 54703,'1982-S252','1982-08-24 16:34:00','military missile','unknown',917,'AO-213','P-432 82 Fall 01','','','OT',433,'',180 from dual union all
select 54704,'1982-S253','1982-08-24 17:00:00','military missile','unknown',917,'AO-214','P-433 82 Fall 02','','','OT',433,'',180 from dual union all
select 54705,'1982-S254','1982-09-01 16:00:00','military missile','unknown',917,'AO-215','P-434 82 Fall 03','','','OT',433,'',180 from dual union all
select 54706,'1982-S255','1982-09-01 16:46:00','military missile','unknown',917,'AO-216','P-435 82 Fall 04','','','OT',433,'',180 from dual union all
select 54707,'1982-S256','1982-06-28 13:05:00','military missile','success',1006,'67074-327-08T','15A18 UBP-13','GCh 15F183','','Test',489,'',1000 from dual union all
select 54708,'1982-S257','1982-10-21 03:35:00','military missile','success',1453,'51184-62-664T(3P)','15A35 UBP-15','GCh 15F355','','OT',505,'',1000 from dual union all
select 54709,'1982-S258','1982-12-08 00:45:00','military missile','success',1453,'49217-91-572(4P)','15A35 UBP-16','GCh 15F355P5','','OT',535,'',1000 from dual union all
select 5471,'1958-Y05','1958-02-10 08:45:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54710,'1982-S259','1982-04-27 00:00:00','military missile','success',1037,'29557-211-40','8K98 UBP-62','','','Test',477,'',1000 from dual union all
select 54711,'1982-S26','1982-07-02 00:00:00','test rocket','unknown',1497,'23','K65UP No. 23','','','Ionos?',621,'',1500 from dual union all
select 54712,'1982-S260','1982-08-20 00:00:00','military missile','success',1037,'Kh29521-35','8K98 UBP-63','','','Test',477,'',1000 from dual union all
select 54713,'1982-S262','1982-01-08 00:00:00','military missile','success',1452,'63065-53-233','15A30 60 RD UBP-9','','','OT',1461,'',1000 from dual union all
select 54714,'1982-S263','1982-01-01 00:00:00','military missile','success',1450,'','15A20 54 RD UBP-7','','','OT',1468,'',1000 from dual union all
select 54715,'1982-S264','1982-05-20 00:00:00','military missile','success',1449,'','8K84 10 RD UBP-8','','','OT',897,'',1000 from dual union all
select 54716,'1982-S265','1982-02-20 00:00:00','military missile','success',744,'','15A15 10 RD UBP-9','','','OT',897,'',1000 from dual union all
select 54717,'1982-S266','1982-03-22 00:00:00','military missile','success',744,'','15A15 10 RD UBP-10','','','OT',897,'',1000 from dual union all
select 54718,'1982-S267','1982-03-23 00:00:00','military missile','success',744,'','15A15 10 RD UBP-11','','','OT',897,'',1000 from dual union all
select 54719,'1982-S268','1982-02-16 00:00:00','military missile','success',745,'','15A16 10 RD UBP-1','','','OT',897,'',1000 from dual union all
select 5472,'1958-Y06','1958-02-11 09:44:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54720,'1982-S269','1982-06-11 00:00:00','military missile','success',1450,'','15A20 52 RD UBP','','','OT',195,'',1000 from dual union all
select 54721,'1982-S27','1982-12-17 00:00:00','test rocket','unknown',1497,'24','K65UP No. 24','','','Ionos?',621,'',1500 from dual union all
select 54722,'1982-S270','1982-08-13 00:00:00','military missile','success',1450,'','15A20 36 RD UBP-3','','','OT',898,'',1000 from dual union all
select 54723,'1982-S271','1982-01-08 14:02:00','suborbital rocket','success',1284,'INTA SLD-8201','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 54724,'1982-S272','1982-01-18 14:16:00','suborbital rocket','success',1284,'INTA SLD-8202','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 54725,'1982-S273','1982-01-20 14:29:00','suborbital rocket','success',1284,'INTA SLD-8203','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 54726,'1982-S274','1982-01-27 14:23:00','suborbital rocket','success',1284,'INTA SLD-8204','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 54727,'1982-S275','1982-02-05 14:23:00','suborbital rocket','success',1284,'INTA SLD-8205','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 54728,'1982-S276','1982-02-08 14:18:00','suborbital rocket','success',1284,'INTA SLD-8206','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual;
insert into LAUNCH
select 54729,'1982-S277','1982-02-15 14:42:00','suborbital rocket','success',1284,'INTA SLD-8207','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 5473,'1958-Y07','1958-02-18 13:10:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54730,'1982-S278','1982-10-14 00:00:00','military missile','success',922,'','15Zh45 35 RD UBP','','','Test',636,'',500 from dual union all
select 54731,'1982-S279','1982-03-01 00:00:00','military missile','success',923,'','15Zh53 32 RD UBP','','','Test',636,'',500 from dual union all
select 54732,'1982-S28','1982-03-16 19:05:00','military missile','failure',1006,'51074-317-10T','15A18 UBP-11','GCh 15F183','','OT',485,'',100 from dual union all
select 54733,'1982-S280','1982-03-01 00:00:00','military missile','success',923,'','15Zh53 32 RD UBP','','','Test',636,'',500 from dual union all
select 54734,'1982-S281','1982-10-01 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 54735,'1982-S29','1982-06-03 19:20:00','military missile','success',1006,'51074-317-03T','15A18 UBP-12','GCh 15F183','','OT',485,'',1000 from dual union all
select 54736,'1982-S30','1982-07-27 00:00:00','military missile','success',1006,'2508T','15A18 UBP 59 RD','GCh 15F183','','OT',828,'',1000 from dual union all
select 54737,'1982-S32','1982-01-07 11:47:00','military missile','success',1453,'51139-22-122T(1P)','15A35 UBP-13','GCh 15F355','','OT',505,'',1000 from dual union all
select 54738,'1982-S33','1982-04-26 15:20:00','military missile','success',1453,'51124-62-662T(2P)','15A35 UBP-14','GCh 15F355','','OT',505,'',1000 from dual union all
select 54739,'1982-S34','1982-01-15 09:20:00','suborbital rocket','success',668,'K-9M-73','','','','Ionos',839,'',303 from dual union all
select 5474,'1958-Y08','1958-02-18 14:31:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54740,'1982-S35','1982-01-17 14:11:00','military missile','success',1442,'','DASO-9','','','OT',421,'SSBN 726',1000 from dual union all
select 54741,'1982-S37','1982-01-20 00:30:15','suborbital rocket','success',860,'NASA 31.18GE','','','','Plasma',1047,'',94 from dual union all
select 54742,'1982-S38','1982-01-20 14:59:00','military missile','unknown',917,'','P-422 82 Cape 01','','','OT',231,'',180 from dual union all
select 54743,'1982-S39','1982-01-20 15:19:00','military missile','unknown',917,'','P-423 82 Cape 02','','','OT',231,'',180 from dual union all
select 54744,'1982-S40','1982-01-20 19:06:00','military missile','unknown',917,'','P-424 82 Cape 03','','','OT',231,'',180 from dual union all
select 54745,'1982-S41','1982-01-21 01:31:00','suborbital rocket','success',1337,'NASA 29.18UE','','','','Plasma',334,'',633 from dual union all
select 54746,'1982-S42','1982-01-23 08:00:00','suborbital rocket','success',117,'GL A24.7S2-02','FIRSSE (BMP)','','','ABM Tech IR/IR Astron',1873,'',378 from dual union all
select 54747,'1982-S43','1982-01-23 12:12:00','suborbital rocket','success',860,'NASA 31.25GE','','','','Plasma',1047,'',140 from dual union all
select 54748,'1982-S44','1982-01-23 12:13:00','suborbital rocket','success',860,'NASA 31.19GE','','','','Plasma',1047,'',101 from dual union all
select 54749,'1982-S45','1982-01-24 12:45:00','suborbital rocket','success',860,'NASA 31.24GE','','','','Plasma',1047,'',140 from dual union all
select 5475,'1958-Y09','1958-02-26 03:35:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54750,'1982-S46','1982-01-26 12:50:00','military missile','success',1442,'','DASO-10','','','OT',421,'SSBN 634',1000 from dual union all
select 54751,'1982-S47','1982-01-27 03:52:00','suborbital rocket','success',211,'NASA 27.45UE','SCEX','','','Ionos active',428,'',239 from dual union all
select 54752,'1982-S48','1982-01-29 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT86GM','GLORY TRIP 86GM','OT',1703,'',1300 from dual union all
select 54753,'1982-S49','1982-01-30 16:30:00','military missile','success',932,'A3TK','CHEVALINE DASO','','','OT',421,'UK S26',1000 from dual union all
select 54754,'1982-S50','1982-02-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54755,'1982-S51','1982-02-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54756,'1982-S52','1982-02-01 18:30:00','military missile','success',932,'A3TK','CHEVALINE DASO','','','OT',421,'UK S26',1000 from dual union all
select 54757,'1982-S53','1982-02-03 16:30:00','military missile','success',932,'A3TK','CHEVALINE DASO','','','OT',421,'UK S26',1000 from dual union all
select 54758,'1982-S54','1982-02-08 18:30:00','military missile','success',932,'A3TK','CHEVALINE DASO','','','OT',421,'UK S26',1000 from dual union all
select 54759,'1982-S55','1982-02-13 02:00:00','suborbital rocket','success',668,'K-9M-72','','','','Test/Ionos/Plasma/F',839,'',328 from dual union all
select 5476,'1958-Y10','1958-02-26 04:35:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54760,'1982-S56','1982-02-14 10:50:00','suborbital rocket','success',1074,'S-520-3','0.3m telescope test','','','UV Astronomy',833,'',266 from dual union all
select 54761,'1982-S57','1982-02-18 12:40:00','suborbital rocket','success',668,'K-9M-74','','','','UV/XR Astron',839,'',315 from dual union all
select 54762,'1982-S58','1982-02-19 16:00:00','suborbital rocket','success',860,'NASA 31.23UL','CWAS 3','','','Aeronomy',428,'',140 from dual union all
select 54763,'1982-S59','1982-02-24 14:04:00','military missile','unknown',917,'','P-425 82 Cape 04','','','OT',231,'',180 from dual union all
select 54764,'1982-S60','1982-02-24 14:55:00','military missile','unknown',917,'','P-426 82 Cape 05','','','OT',231,'',180 from dual union all
select 54765,'1982-S61','1982-02-24 15:14:00','military missile','unknown',917,'','P-427 82 Cape 06','','','OT',231,'',180 from dual union all
select 54766,'1982-S62','1982-02-26 03:50:00','suborbital rocket','success',1319,'NASA 33.25UL','','','','Ionos',428,'',160 from dual union all
select 54767,'1982-S63','1982-03-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54768,'1982-S64','1982-03-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54769,'1982-S65','1982-03-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 5477,'1958-Y11','1958-03-17 03:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54770,'1982-S66','1982-03-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54771,'1982-S67','1982-03-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54772,'1982-S68','1982-03-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54773,'1982-S69','1982-03-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54774,'1982-S70','1982-03-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54775,'1982-S71','1982-02-28 00:00:00','military missile','success',939,'','FOT-45','','','OT',421,'SSBN 628',500 from dual union all
select 54776,'1982-S72','1982-02-28 00:00:00','military missile','success',939,'','FOT-45','','','OT',421,'SSBN 628',500 from dual union all
select 54777,'1982-S73','1982-03-06 08:47:00','suborbital rocket','success',1337,'NASA 29.16CE','','','','Plasma',1047,'',434 from dual union all
select 54778,'1982-S74','1982-03-10 00:00:00','military missile','success',756,'8','','','','Test',307,'S655',800 from dual union all
select 54779,'1982-S75','1982-03-10 15:01:00','military missile','unknown',917,'','P-428 82 Cape 07','','','OT',231,'',180 from dual union all
select 5478,'1958-Y12','1958-03-20 21:45:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54780,'1982-S76','1982-03-10 15:17:00','military missile','unknown',917,'','P-429 82 Cape 08','','','OT',231,'',180 from dual union all
select 54781,'1982-S77','1982-03-10 16:26:00','military missile','unknown',917,'','P-430 82 Cape 09','','','OT',231,'',180 from dual union all
select 54782,'1982-S78','1982-03-13 00:00:00','suborbital rocket','success',44,'NASA 17.18DG','NRL Mk II UV Camera','','','UV Astron',1895,'',300 from dual union all
select 54783,'1982-S79','1982-03-15 18:00:00','suborbital rocket','failure',920,'INTA FOCCA 1-1/82','FOCCA II','IR photom','','Aeron',396,'',140 from dual union all
select 54784,'1982-S80','1982-03-13 17:00:00','military missile','success',1442,'','DASO-11','','','OT',421,'SSBN 726',1000 from dual union all
select 54785,'1982-S81','1982-03-15 06:38:20','suborbital rocket','success',1320,'NASA 34.08UE','BUBBLE MACHINE II','','','Plasma',1047,'',460 from dual union all
select 54786,'1982-S82','1982-03-17 00:00:00','military missile','success',1442,'','OT-8','','','OT',421,'SSBN 658',1000 from dual union all
select 54787,'1982-S83','1982-03-17 00:00:00','military missile','success',1442,'','OT-8','','','OT',421,'SSBN 658',1000 from dual union all
select 54788,'1982-S84','1982-03-17 00:00:00','military missile','success',1442,'','OT-8','','','OT',421,'SSBN 658',1000 from dual union all
select 54789,'1982-S85','1982-03-17 10:34:00','military missile','success',1442,'','OT-8','','','OT',421,'SSBN 658',1000 from dual union all
select 5479,'1958-Y13','1958-03-21 03:35:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54790,'1982-S86','1982-03-18 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT141MS','GLORY TRIP 141MS','OT',1699,'',1300 from dual union all
select 54791,'1982-S87','1982-03-23 21:27:00','suborbital rocket','success',920,'P230H','ETON','','','Aeronomy',1447,'',132 from dual union all
select 54792,'1982-S88','1982-03-23 21:48:00','suborbital rocket','success',920,'P232H','ETON','','','Aeronomy',1447,'',138 from dual union all
select 54793,'1982-S89','1982-03-23 22:10:00','suborbital rocket','success',920,'P227H','ETON','','','Aeronomy',1447,'',129 from dual union all
select 54794,'1982-S90','1982-03-23 22:57:00','suborbital rocket','unknown',920,'P229H','ETON','','','Aeron',1447,'',139 from dual union all
select 54795,'1982-S91','1982-03-23 23:12:00','suborbital rocket','success',920,'P231H','ETON','','','Aeronomy',1447,'',131 from dual union all
select 54796,'1982-S92','1982-03-23 23:28:00','suborbital rocket','success',920,'P228H','ETON','','','Aeronomy',1447,'',127 from dual union all
select 54797,'1982-S93','1982-03-23 23:55:00','suborbital rocket','success',920,'P234H','ETON','','','Aeronomy',1447,'',129 from dual union all
select 54798,'1982-S94','1982-03-30 10:26:35','suborbital rocket','success',1320,'NASA 34.07UE','LIMERICK','','','Plasma Ba',1047,'',606 from dual union all
select 54799,'1982-S95','1982-04-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 548,'1945-S24','1945-03-17 00:00:00','military missile','success',1470,'','Remagen','','','Weapon',1736,'',90 from dual union all
select 5480,'1958-Y14','1958-03-28 03:35:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54800,'1982-S96','1982-04-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54801,'1982-S97','1982-04-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54802,'1982-S98','1982-04-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54803,'1982-S99','1982-04-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',131,'VISE',150 from dual union all
select 54804,'1982-W01','1982-01-01 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',977,'MUSSON',61 from dual union all
select 54805,'1982-W02','1982-01-04 12:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 54806,'1982-W03','1982-01-04 19:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 54807,'1982-W04','1982-01-04 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 54808,'1982-W05','1982-01-05 09:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 54809,'1982-W06','1982-01-06 02:00:00','suborbital rocket','success',761,'MT135P-R444','','','','Meteo',1332,'',57 from dual union all
select 5481,'1958-Y15','1958-04-01 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54810,'1982-W07','1982-01-06 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',977,'MUSSON',58 from dual union all
select 54811,'1982-W08','1982-01-06 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 54812,'1982-W09','1982-01-06 22:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 54813,'1982-W10','1982-01-06 23:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 54814,'1982-W100','1982-02-05 17:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',52 from dual union all
select 54815,'1982-W101','1982-02-08 09:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',58 from dual union all
select 54816,'1982-W102','1982-02-08 20:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 54817,'1982-W103','1982-02-10 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',79 from dual union all
select 54818,'1982-W104','1982-02-10 13:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',63 from dual;
insert into LAUNCH
select 54819,'1982-W105','1982-02-10 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 5482,'1958-Y16','1958-04-20 15:40:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54820,'1982-W106','1982-02-10 20:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 54821,'1982-W107','1982-02-10 22:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 54822,'1982-W108','1982-02-11 20:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 54823,'1982-W109','1982-02-12 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',1028,'PASSAT',61 from dual union all
select 54824,'1982-W11','1982-01-07 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',75 from dual union all
select 54825,'1982-W110','1982-02-12 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 54826,'1982-W111','1982-02-12 14:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',64 from dual union all
select 54827,'1982-W112','1982-02-12 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 54828,'1982-W113','1982-02-15 09:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 54829,'1982-W114','1982-02-15 17:08:00','suborbital rocket','unknown',1284,'FLIGHT 301     /TZ1-','','','','Aeron',1747,'',70 from dual union all
select 5483,'1958-Y17','1958-05-18 00:50:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54830,'1982-W115','1982-02-15 19:15:00','suborbital rocket','unknown',1284,'FLIGHT 302     /TZ1-','','','','Aeron',1747,'',69 from dual union all
select 54831,'1982-W116','1982-02-15 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 54832,'1982-W117','1982-02-16 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',61 from dual union all
select 54833,'1982-W118','1982-02-16 20:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 54834,'1982-W119','1982-02-16 20:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 54835,'1982-W12','1982-01-08 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',977,'MUSSON',59 from dual union all
select 54836,'1982-W120','1982-02-16 21:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 54837,'1982-W121','1982-02-17 02:00:00','suborbital rocket','success',761,'MT135P-R449','','','','Meteo',1332,'',55 from dual union all
select 54838,'1982-W122','1982-02-17 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1028,'PASSAT',62 from dual union all
select 54839,'1982-W123','1982-02-17 09:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 5484,'1958-Y18','1958-05-18 01:56:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54840,'1982-W124','1982-02-17 19:07:00','suborbital rocket','unknown',1284,'FLIGHT 304     /TZ1-','','','','Aeron',1747,'',70 from dual union all
select 54841,'1982-W125','1982-02-17 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 54842,'1982-W126','1982-02-17 22:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 54843,'1982-W127','1982-02-18 13:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 54844,'1982-W128','1982-02-18 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 54845,'1982-W129','1982-02-19 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1028,'PASSAT',62 from dual union all
select 54846,'1982-W13','1982-01-08 14:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 54847,'1982-W130','1982-02-19 13:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',57 from dual union all
select 54848,'1982-W131','1982-02-19 15:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',63 from dual union all
select 54849,'1982-W132','1982-02-19 18:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 5485,'1958-Y19','1958-05-19 01:05:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54850,'1982-W133','1982-02-19 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 54851,'1982-W134','1982-02-21 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1028,'PASSAT',58 from dual union all
select 54852,'1982-W135','1982-02-21 16:53:00','suborbital rocket','unknown',1284,'FLIGHT 305     /TZ1-','','','','Aeron',1747,'',70 from dual union all
select 54853,'1982-W136','1982-02-22 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1028,'PASSAT',60 from dual union all
select 54854,'1982-W137','1982-02-22 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',53 from dual union all
select 54855,'1982-W138','1982-02-22 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 54856,'1982-W139','1982-02-23 13:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',56 from dual union all
select 54857,'1982-W14','1982-01-11 12:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 54858,'1982-W140','1982-02-23 19:11:00','suborbital rocket','unknown',1284,'T 1-0105','','','','Test/Aeron',1821,'',79 from dual union all
select 54859,'1982-W141','1982-02-24 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1028,'PASSAT',60 from dual union all
select 5486,'1958-Y20','1958-05-29 01:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54860,'1982-W142','1982-02-24 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',73 from dual union all
select 54861,'1982-W143','1982-02-24 22:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 54862,'1982-W144','1982-02-25 09:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 54863,'1982-W145','1982-02-25 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',61 from dual union all
select 54864,'1982-W146','1982-02-25 19:00:00','suborbital rocket','unknown',1284,'T 1-0107','','','','Test/Aeron',1821,'',74 from dual union all
select 54865,'1982-W147','1982-02-25 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',77 from dual union all
select 54866,'1982-W148','1982-02-25 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 54867,'1982-W149','1982-02-26 07:00:00','suborbital rocket','unknown',738,'MMR-06','','','','Aeron',1028,'PASSAT',60 from dual union all
select 54868,'1982-W15','1982-01-11 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 54869,'1982-W150','1982-02-26 16:51:00','suborbital rocket','unknown',1284,'FLIGHT 306     /TZ1-','','','','Aeron',1747,'',75 from dual union all
select 5487,'1958-Y21','1958-05-31 01:45:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54870,'1982-W151','1982-02-27 19:00:00','suborbital rocket','unknown',716,'T 1-0109','','','','Test/Aeron',1821,'',60 from dual union all
select 54871,'1982-W152','1982-02-28 08:50:00','suborbital rocket','unknown',738,'','','','','Aeron',1028,'PASSAT',62 from dual union all
select 54872,'1982-W153','1982-02-28 19:46:00','suborbital rocket','unknown',1284,'T 1-0112','','','','Test/Aeron',1821,'',79 from dual union all
select 54873,'1982-W154','1982-03-01 20:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',57 from dual union all
select 54874,'1982-W155','1982-03-01 20:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 54875,'1982-W156','1982-03-03 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 54876,'1982-W157','1982-03-04 16:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 54877,'1982-W158','1982-03-04 19:00:00','suborbital rocket','unknown',1284,'T 1-0118','','','','Test/Aeron',1821,'',79 from dual union all
select 54878,'1982-W159','1982-03-05 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 54879,'1982-W16','1982-01-11 19:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 5488,'1958-Y22','1958-06-09 00:50:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54880,'1982-W160','1982-03-05 19:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',72 from dual union all
select 54881,'1982-W161','1982-03-05 21:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 54882,'1982-W163','1982-03-06 19:02:00','suborbital rocket','unknown',716,'T 1-0120','','','','Test/Aeron',1821,'',60 from dual union all
select 54883,'1982-W164','1982-03-07 15:20:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',74 from dual union all
select 54884,'1982-W165','1982-03-07 19:01:00','suborbital rocket','unknown',716,'T 1-0121','','','','Test/Aeron',1821,'',59 from dual union all
select 54885,'1982-W166','1982-03-08 19:08:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 54886,'1982-W167','1982-03-09 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',69 from dual union all
select 54887,'1982-W168','1982-03-09 19:00:00','suborbital rocket','unknown',1284,'T 1-0123','','','','Test/Aeron',1821,'',78 from dual union all
select 54888,'1982-W169','1982-03-09 20:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 54889,'1982-W17','1982-01-11 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 5489,'1958-Y23','1958-06-24 00:45:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54890,'1982-W170','1982-03-10 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 54891,'1982-W171','1982-03-10 19:00:00','suborbital rocket','unknown',1284,'T 1-0124','','','','Test/Aeron',1821,'',76 from dual union all
select 54892,'1982-W172','1982-03-10 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 54893,'1982-W173','1982-03-11 19:00:00','suborbital rocket','unknown',1284,'T 1-0125','','','','Test/Aeron',1821,'',79 from dual union all
select 54894,'1982-W174','1982-03-12 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',75 from dual union all
select 54895,'1982-W175','1982-03-12 19:00:00','suborbital rocket','unknown',1284,'T 1-0126','','','','Test/Aeron',1821,'',79 from dual union all
select 54896,'1982-W177','1982-03-13 19:03:00','suborbital rocket','unknown',716,'T 1-0127','','','','Test/Aeron',1821,'',61 from dual union all
select 54897,'1982-W178','1982-03-14 19:01:00','suborbital rocket','unknown',716,'T 1-0128','','','','Test/Aeron',1821,'',61 from dual union all
select 54898,'1982-W179','1982-03-15 19:00:00','suborbital rocket','unknown',716,'T 1-0129','','','','Test/Aeron',1821,'',74 from dual union all
select 54899,'1982-W18','1982-01-11 21:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',70 from dual union all
select 549,'1945-S25','1945-03-16 00:00:00','military missile','success',1470,'','Antwerp','','','Weapon',1735,'',90 from dual union all
select 5490,'1958-Y24','1958-06-24 01:45:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54900,'1982-W180','1982-03-15 20:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 54901,'1982-W181','1982-03-16 02:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 54902,'1982-W182','1982-03-16 19:00:00','suborbital rocket','unknown',1284,'T 1-0130','','','','Test/Aeron',1821,'',76 from dual union all
select 54903,'1982-W183','1982-03-17 18:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 54904,'1982-W184','1982-03-17 19:23:00','suborbital rocket','unknown',1284,'T 1-0131','','','','Test/Aeron',1821,'',79 from dual union all
select 54905,'1982-W185','1982-03-17 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 54906,'1982-W186','1982-03-18 19:01:00','suborbital rocket','unknown',1284,'T 1-0132','','','','Test/Aeron',1821,'',60 from dual union all
select 54907,'1982-W187','1982-03-19 19:00:00','suborbital rocket','unknown',1284,'T 1-0133','','','','Test/Aeron',1821,'',76 from dual union all
select 54908,'1982-W188','1982-03-20 19:00:00','suborbital rocket','unknown',716,'T 1-0134','','','','Test/Aeron',1821,'',62 from dual;
insert into LAUNCH
select 54909,'1982-W189','1982-03-20 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',74 from dual union all
select 5491,'1958-Y25','1958-06-29 01:30:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54910,'1982-W19','1982-01-12 16:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 54911,'1982-W190','1982-03-21 19:20:00','suborbital rocket','unknown',716,'T 1-0288','','','','Test/Aeron',1821,'',61 from dual union all
select 54912,'1982-W191','1982-03-22 16:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 54913,'1982-W192','1982-03-22 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',50 from dual union all
select 54914,'1982-W193','1982-03-22 19:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 54915,'1982-W194','1982-03-22 19:24:00','suborbital rocket','unknown',1284,'T 1-0290','','','','Test/Aeron',1821,'',75 from dual union all
select 54916,'1982-W195','1982-03-22 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 54917,'1982-W196','1982-03-23 17:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 54918,'1982-W197','1982-03-23 19:00:00','suborbital rocket','unknown',716,'T 1-0291','','','','Test/Aeron',1821,'',78 from dual union all
select 54919,'1982-W198','1982-03-23 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 5492,'1958-Y26','1958-07-15 20:45:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54920,'1982-W199','1982-03-23 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 54921,'1982-W20','1982-01-13 02:00:00','suborbital rocket','success',761,'MT135P-R445','','','','Meteo',1332,'',53 from dual union all
select 54922,'1982-W200','1982-03-24 02:01:00','suborbital rocket','success',761,'MT135P-R453','','','','Meteo',1332,'',58 from dual union all
select 54923,'1982-W201','1982-03-24 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',79 from dual union all
select 54924,'1982-W202','1982-03-24 16:58:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',79 from dual union all
select 54925,'1982-W203','1982-03-24 17:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 54926,'1982-W204','1982-03-24 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 54927,'1982-W205','1982-03-24 19:01:00','suborbital rocket','unknown',1284,'T 1-0292','','','','Test/Aeron',1821,'',74 from dual union all
select 54928,'1982-W206','1982-03-24 20:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 54929,'1982-W207','1982-03-25 17:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 5493,'1958-Y27','1958-07-15 21:45:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54930,'1982-W208','1982-03-25 19:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 54931,'1982-W209','1982-03-26 17:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 54932,'1982-W21','1982-01-13 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',977,'MUSSON',59 from dual union all
select 54933,'1982-W210','1982-03-26 20:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 54934,'1982-W211','1982-03-27 16:57:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',78 from dual union all
select 54935,'1982-W212','1982-03-27 17:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 54936,'1982-W213','1982-03-27 19:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 54937,'1982-W214','1982-03-28 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 54938,'1982-W215','1982-03-28 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 54939,'1982-W216','1982-03-29 12:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 5494,'1958-Y28','1958-07-16 02:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54940,'1982-W217','1982-03-29 13:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',62 from dual union all
select 54941,'1982-W218','1982-03-29 14:05:00','suborbital rocket','unknown',1284,'T 1-0283','','','','Test/Aeron',1821,'',75 from dual union all
select 54942,'1982-W219','1982-03-29 14:57:00','suborbital rocket','unknown',1284,'T 1-0293','','','','Test/Aeron',1821,'',71 from dual union all
select 54943,'1982-W22','1982-01-13 09:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',57 from dual union all
select 54944,'1982-W220','1982-03-29 15:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',51 from dual union all
select 54945,'1982-W221','1982-03-29 20:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 54946,'1982-W222','1982-03-29 20:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 54947,'1982-W223','1982-03-30 10:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',67 from dual union all
select 54948,'1982-W224','1982-03-30 14:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 54949,'1982-W225','1982-03-30 14:20:00','suborbital rocket','unknown',1284,'T 1-0297','','','','Test/Aeron',1821,'',65 from dual union all
select 5495,'1958-Y29','1958-07-17 01:05:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54950,'1982-W226','1982-03-30 15:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 54951,'1982-W227','1982-03-30 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 54952,'1982-W228','1982-03-30 16:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 54953,'1982-W229','1982-03-30 17:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 54954,'1982-W23','1982-01-13 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 54955,'1982-W230','1982-03-31 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',79 from dual union all
select 54956,'1982-W231','1982-03-31 20:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 54957,'1982-W232','1982-03-31 22:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 54958,'1982-W233','1982-04-02 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',58 from dual union all
select 54959,'1982-W234','1982-04-05 20:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 5496,'1958-Y30','1958-07-27 02:20:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54960,'1982-W235','1982-04-06 22:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 54961,'1982-W236','1982-04-07 02:00:00','suborbital rocket','success',761,'MT135P-R454','','','','Meteo',1332,'',53 from dual union all
select 54962,'1982-W237','1982-04-07 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',978,'MUSSON',60 from dual union all
select 54963,'1982-W238','1982-04-07 16:10:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 54964,'1982-W239','1982-04-07 21:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 54965,'1982-W24','1982-01-13 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 54966,'1982-W240','1982-04-09 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',978,'MUSSON',58 from dual union all
select 54967,'1982-W241','1982-04-09 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 54968,'1982-W242','1982-04-11 07:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',63 from dual union all
select 54969,'1982-W243','1982-04-14 02:02:00','suborbital rocket','success',761,'MT135P-R455','','','','Meteo',1332,'',55 from dual union all
select 5497,'1958-Y31','1958-07-27 15:10:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54970,'1982-W244','1982-04-14 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 54971,'1982-W245','1982-04-14 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 54972,'1982-W246','1982-04-14 22:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 54973,'1982-W247','1982-04-15 21:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 54974,'1982-W248','1982-04-16 02:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',56 from dual union all
select 54975,'1982-W249','1982-04-16 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',978,'MUSSON',60 from dual union all
select 54976,'1982-W25','1982-01-13 20:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 54977,'1982-W250','1982-04-19 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 54978,'1982-W251','1982-04-19 21:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 54979,'1982-W252','1982-04-20 14:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 5498,'1958-Y32','1958-07-31 02:20:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 54980,'1982-W253','1982-04-20 15:59:00','suborbital rocket','unknown',1284,'FLIGHT 307     /T 1-','','','','Aeron',1821,'',64 from dual union all
select 54981,'1982-W254','1982-04-20 16:41:00','suborbital rocket','unknown',1284,'FLIGHT 308     /T 1-','','','','Aeron',1821,'',65 from dual union all
select 54982,'1982-W255','1982-04-20 16:46:00','suborbital rocket','unknown',1284,'FLIGHT 309     /T 1-','','','','Aeron',1821,'',66 from dual union all
select 54983,'1982-W256','1982-04-20 17:17:00','suborbital rocket','unknown',1284,'FLIGHT 310     /T 1-','','','','Aeron',1821,'',63 from dual union all
select 54984,'1982-W257','1982-04-20 17:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',69 from dual union all
select 54985,'1982-W258','1982-04-21 02:00:00','suborbital rocket','success',761,'MT135P-R456','','','','Meteo',1332,'',55 from dual union all
select 54986,'1982-W259','1982-04-21 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',978,'MUSSON',59 from dual union all
select 54987,'1982-W26','1982-01-13 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',58 from dual union all
select 54988,'1982-W260','1982-04-21 18:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 54989,'1982-W261','1982-04-21 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 5499,'1958-Y33','1958-08-07 08:45:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 54990,'1982-W262','1982-04-22 20:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 54991,'1982-W263','1982-04-23 02:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 54992,'1982-W264','1982-04-23 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',978,'MUSSON',59 from dual union all
select 54993,'1982-W265','1982-04-23 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 54994,'1982-W266','1982-04-26 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 54995,'1982-W267','1982-04-28 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 54996,'1982-W268','1982-04-28 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 54997,'1982-W269','1982-04-28 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 54998,'1982-W27','1982-01-13 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',58 from dual union all
select 54999,'1982-W270','1982-04-29 17:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual;
insert into LAUNCH
select 55,'1944-A19','1944-09-23 09:28:00','military missile','failure',8,'18823','Ma212','','','Training',661,'',20 from dual union all
select 550,'1945-S26','1945-03-27 20:43:00','military missile','success',1470,'Last launch','Antwerp/Mortsel','','','Weapon',1734,'',90 from dual union all
select 5500,'1958-Y34','1958-08-12 03:10:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 55000,'1982-W271','1982-04-30 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',978,'MUSSON',59 from dual union all
select 55001,'1982-W272','1982-04-30 20:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 55002,'1982-W273','1982-04-30 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 55003,'1982-W274','1982-05-03 06:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1029,'PASSAT',56 from dual union all
select 55004,'1982-W275','1982-05-03 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',978,'MUSSON',59 from dual union all
select 55005,'1982-W276','1982-05-03 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 55006,'1982-W277','1982-05-05 06:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1029,'PASSAT',57 from dual union all
select 55007,'1982-W278','1982-05-05 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 55008,'1982-W279','1982-05-05 22:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 55009,'1982-W28','1982-01-13 22:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 5501,'1958-Y35','1958-08-12 04:40:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 55010,'1982-W280','1982-05-07 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1029,'PASSAT',60 from dual union all
select 55011,'1982-W281','1982-05-07 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 55012,'1982-W282','1982-05-10 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1029,'PASSAT',55 from dual union all
select 55013,'1982-W283','1982-05-10 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',58 from dual union all
select 55014,'1982-W284','1982-05-11 02:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 55015,'1982-W285','1982-05-12 02:00:00','suborbital rocket','success',761,'MT135P-R458','','','','Meteo',1332,'',54 from dual union all
select 55016,'1982-W286','1982-05-12 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1029,'PASSAT',58 from dual union all
select 55017,'1982-W287','1982-05-12 19:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 55018,'1982-W288','1982-05-13 20:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 55019,'1982-W289','1982-05-14 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1029,'PASSAT',62 from dual union all
select 5502,'1958-Y36','1958-08-14 01:40:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55020,'1982-W29','1982-01-15 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',977,'MUSSON',59 from dual union all
select 55021,'1982-W290','1982-05-14 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 55022,'1982-W291','1982-05-17 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1029,'PASSAT',61 from dual union all
select 55023,'1982-W292','1982-05-17 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 55024,'1982-W293','1982-05-18 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 55025,'1982-W294','1982-05-18 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 55026,'1982-W295','1982-05-18 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 55027,'1982-W296','1982-05-19 16:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 55028,'1982-W297','1982-05-21 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1029,'PASSAT',60 from dual union all
select 55029,'1982-W298','1982-05-21 20:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 5503,'1958-Y37','1958-08-15 04:40:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 55030,'1982-W299','1982-05-24 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1029,'PASSAT',60 from dual union all
select 55031,'1982-W30','1982-01-16 11:19:00','suborbital rocket','unknown',716,'TM1-0224','','','','Aeron',1047,'',75 from dual union all
select 55032,'1982-W300','1982-05-24 17:00:00','suborbital rocket','unknown',1284,'T 1-0298','','','','Test/Aeron',1821,'',61 from dual union all
select 55033,'1982-W301','1982-05-25 21:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',65 from dual union all
select 55034,'1982-W302','1982-05-26 02:00:00','suborbital rocket','success',761,'MT135P-R460','','','','Meteo',1332,'',53 from dual union all
select 55035,'1982-W303','1982-05-26 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1029,'PASSAT',59 from dual union all
select 55036,'1982-W304','1982-05-26 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',69 from dual union all
select 55037,'1982-W305','1982-05-26 17:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 55038,'1982-W306','1982-05-26 21:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 55039,'1982-W307','1982-05-26 23:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',57 from dual union all
select 5504,'1958-Y38','1958-09-06 04:25:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 55040,'1982-W308','1982-05-27 14:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 55041,'1982-W309','1982-05-27 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 55042,'1982-W31','1982-01-16 11:44:00','suborbital rocket','unknown',716,'TM1-0225','','','','Aeron',1047,'',57 from dual union all
select 55043,'1982-W310','1982-05-28 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 55044,'1982-W311','1982-05-28 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 55045,'1982-W312','1982-05-30 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',79 from dual union all
select 55046,'1982-W313','1982-06-02 02:02:00','suborbital rocket','success',761,'MT135P-R461','','','','Meteo',1332,'',57 from dual union all
select 55047,'1982-W314','1982-06-02 15:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 55048,'1982-W315','1982-06-02 23:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 55049,'1982-W316','1982-06-03 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 5505,'1958-Y39','1958-09-14 04:40:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55050,'1982-W317','1982-06-04 19:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 55051,'1982-W318','1982-06-07 20:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 55052,'1982-W319','1982-06-07 21:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 55053,'1982-W32','1982-01-17 11:40:00','suborbital rocket','unknown',716,'TM1-0226','','','','Aeron',1047,'',57 from dual union all
select 55054,'1982-W320','1982-06-08 14:37:00','suborbital rocket','unknown',1284,'T 1-0301','','','','Test/Aeron',1821,'',64 from dual union all
select 55055,'1982-W321','1982-06-08 21:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 55056,'1982-W322','1982-06-09 02:02:00','suborbital rocket','success',761,'MT135P-R462','','','','Meteo',1332,'',56 from dual union all
select 55057,'1982-W323','1982-06-09 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 55058,'1982-W324','1982-06-09 23:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 55059,'1982-W325','1982-06-14 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 5506,'1958-Y40','1958-09-17 02:30:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55060,'1982-W326','1982-06-16 17:02:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 55061,'1982-W327','1982-06-17 22:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 55062,'1982-W328','1982-06-18 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 55063,'1982-W329','1982-06-21 21:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 55064,'1982-W33','1982-01-17 16:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',67 from dual union all
select 55065,'1982-W330','1982-06-22 15:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',50 from dual union all
select 55066,'1982-W331','1982-06-23 02:00:00','suborbital rocket','success',761,'MT135P-R463','','','','Meteo',1332,'',56 from dual union all
select 55067,'1982-W332','1982-06-23 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 55068,'1982-W333','1982-06-23 22:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 55069,'1982-W334','1982-06-24 21:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 5507,'1958-Y41','1958-09-17 04:43:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 55070,'1982-W335','1982-06-27 14:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 55071,'1982-W336','1982-06-27 16:55:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 55072,'1982-W337','1982-06-28 15:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 55073,'1982-W338','1982-06-29 14:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 55074,'1982-W339','1982-06-29 16:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',68 from dual union all
select 55075,'1982-W34','1982-01-18 11:27:00','suborbital rocket','unknown',716,'TM1-0227','','','','Aeron',1047,'',55 from dual union all
select 55076,'1982-W340','1982-06-30 22:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 55077,'1982-W341','1982-06-30 22:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 55078,'1982-W342','1982-07-01 13:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 55079,'1982-W343','1982-07-01 16:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',70 from dual union all
select 5508,'1958-Y42','1958-09-19 03:50:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55080,'1982-W344','1982-07-03 16:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',70 from dual union all
select 55081,'1982-W345','1982-07-03 16:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 55082,'1982-W346','1982-07-04 10:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 55083,'1982-W347','1982-07-04 11:20:00','suborbital rocket','unknown',1284,'T 1-0308','','','','Test/Aeron',1821,'',66 from dual union all
select 55084,'1982-W348','1982-07-04 13:42:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 55085,'1982-W349','1982-07-04 14:09:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 55086,'1982-W35','1982-01-18 14:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',55 from dual union all
select 55087,'1982-W350','1982-07-04 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 55088,'1982-W351','1982-07-04 15:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',67 from dual;
insert into LAUNCH
select 55089,'1982-W352','1982-07-04 16:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 5509,'1958-Y43','1958-09-20 05:35:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 55090,'1982-W353','1982-07-04 16:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 55091,'1982-W354','1982-07-04 17:11:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 55092,'1982-W355','1982-07-07 02:00:00','suborbital rocket','success',761,'MT135P-R464','','','','Meteo',1332,'',56 from dual union all
select 55093,'1982-W356','1982-07-07 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 55094,'1982-W357','1982-07-07 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 55095,'1982-W358','1982-07-07 22:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 55096,'1982-W359','1982-07-09 21:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',69 from dual union all
select 55097,'1982-W36','1982-01-18 15:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',60 from dual union all
select 55098,'1982-W360','1982-07-09 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 55099,'1982-W361','1982-07-14 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 551,'1945-S27','1945-10-02 13:41:12','military missile','success',1470,'','Backfire 1','','','Test',353,'',69 from dual union all
select 5510,'1958-Y44','1958-09-22 02:30:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55100,'1982-W362','1982-07-14 22:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',67 from dual union all
select 55101,'1982-W363','1982-07-15 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 55102,'1982-W364','1982-07-16 20:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 55103,'1982-W365','1982-07-16 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',58 from dual union all
select 55104,'1982-W366','1982-07-16 22:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 55105,'1982-W367','1982-07-19 16:20:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 55106,'1982-W368','1982-07-19 22:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 55107,'1982-W369','1982-07-19 22:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 55108,'1982-W37','1982-01-18 18:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 55109,'1982-W370','1982-07-20 20:36:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 5511,'1958-Y45','1958-10-01 09:41:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55110,'1982-W371','1982-07-21 02:25:00','suborbital rocket','success',761,'MT135P-R465','','','','Meteo',1332,'',53 from dual union all
select 55111,'1982-W372','1982-07-21 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 55112,'1982-W373','1982-07-21 22:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 55113,'1982-W374','1982-07-26 21:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 55114,'1982-W375','1982-07-28 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 55115,'1982-W376','1982-07-28 22:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 55116,'1982-W377','1982-07-29 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',57 from dual union all
select 55117,'1982-W378','1982-08-05 02:00:00','suborbital rocket','success',761,'MT135P-R466','','','','Meteo',1332,'',55 from dual union all
select 55118,'1982-W379','1982-08-05 21:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 55119,'1982-W38','1982-01-18 21:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',72 from dual union all
select 5512,'1958-Y46','1958-10-23 03:20:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55120,'1982-W380','1982-08-06 20:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 55121,'1982-W381','1982-08-09 19:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 55122,'1982-W382','1982-08-09 21:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 55123,'1982-W383','1982-08-11 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 55124,'1982-W384','1982-08-18 02:12:00','suborbital rocket','success',761,'MT135P-R467','','','','Meteo',1332,'',54 from dual union all
select 55125,'1982-W385','1982-08-18 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 55126,'1982-W386','1982-08-18 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',66 from dual union all
select 55127,'1982-W387','1982-08-18 22:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',64 from dual union all
select 55128,'1982-W388','1982-08-19 23:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 55129,'1982-W389','1982-08-20 16:07:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',60 from dual union all
select 5513,'1958-Y47','1958-10-25 08:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55130,'1982-W39','1982-01-19 15:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',64 from dual union all
select 55131,'1982-W390','1982-08-24 18:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',55 from dual union all
select 55132,'1982-W391','1982-08-25 21:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 55133,'1982-W392','1982-08-27 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 55134,'1982-W393','1982-08-27 20:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 55135,'1982-W394','1982-08-27 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 55136,'1982-W395','1982-08-30 21:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',69 from dual union all
select 55137,'1982-W396','1982-09-01 02:00:00','suborbital rocket','success',761,'MT135P-R468','','','','Meteo',1332,'',55 from dual union all
select 55138,'1982-W397','1982-09-01 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 55139,'1982-W398','1982-09-01 17:13:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 5514,'1958-Y48','1958-10-28 20:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55140,'1982-W399','1982-09-01 23:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 55141,'1982-W40','1982-01-20 01:27:00','suborbital rocket','unknown',716,'TM1-0228','','','','Aeron',1047,'',62 from dual union all
select 55142,'1982-W400','1982-09-03 18:45:00','suborbital rocket','unknown',1284,'T 1-0309','','','','Test/Aeron',1821,'',74 from dual union all
select 55143,'1982-W401','1982-09-05 01:30:00','suborbital rocket','success',761,'MT135P T33','','','','Aeron?',1476,'',50 from dual union all
select 55144,'1982-W402','1982-09-07 21:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 55145,'1982-W403','1982-09-08 02:00:00','suborbital rocket','success',761,'MT135P-R469','','','','Meteo',1332,'',56 from dual union all
select 55146,'1982-W404','1982-09-09 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 55147,'1982-W405','1982-09-10 15:05:00','suborbital rocket','unknown',1284,'T 1-0311','','','','Test/Aeron',1821,'',72 from dual union all
select 55148,'1982-W406','1982-09-15 02:00:00','suborbital rocket','success',761,'MT135P-R470','','','','Meteo',1332,'',55 from dual union all
select 55149,'1982-W407','1982-09-16 00:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 5515,'1958-Y49','1958-10-31 08:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55150,'1982-W408','1982-09-16 14:00:00','suborbital rocket','unknown',1284,'T 1-0312','','','','Aeron',1821,'',73 from dual union all
select 55151,'1982-W409','1982-09-16 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 55152,'1982-W41','1982-01-20 05:30:00','suborbital rocket','unknown',716,'TM1-0229','','','','Aeron',1047,'',60 from dual union all
select 55153,'1982-W410','1982-09-16 19:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 55154,'1982-W411','1982-09-21 02:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 55155,'1982-W412','1982-09-21 22:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',60 from dual union all
select 55156,'1982-W413','1982-09-23 00:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 55157,'1982-W414','1982-09-24 13:21:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 55158,'1982-W415','1982-09-27 14:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 55159,'1982-W416','1982-09-27 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 5516,'1958-Y50','1958-11-04 12:20:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55160,'1982-W417','1982-09-27 21:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 55161,'1982-W418','1982-09-29 02:00:00','suborbital rocket','success',761,'MT135P-R471','','','','Meteo',1332,'',57 from dual union all
select 55162,'1982-W419','1982-09-29 19:00:00','suborbital rocket','unknown',1284,'T 1-0310','','','','Test/Aeron',1821,'',77 from dual union all
select 55163,'1982-W42','1982-01-20 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',78 from dual union all
select 55164,'1982-W420','1982-09-29 22:41:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 55165,'1982-W421','1982-10-01 14:26:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 55166,'1982-W422','1982-10-02 02:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 55167,'1982-W423','1982-10-04 21:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 55168,'1982-W424','1982-10-05 03:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 55169,'1982-W425','1982-10-05 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 5517,'1958-Y51','1958-11-10 20:45:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55170,'1982-W426','1982-10-06 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 55171,'1982-W427','1982-10-07 02:00:00','suborbital rocket','success',761,'MT135P-R472','','','','Meteo',1332,'',55 from dual union all
select 55172,'1982-W428','1982-10-08 10:54:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 55173,'1982-W429','1982-10-13 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 55174,'1982-W43','1982-01-20 16:09:00','suborbital rocket','unknown',1284,'FLIGHT 296     /T 1-','','','','Aeron',1821,'',66 from dual union all
select 55175,'1982-W430','1982-10-13 17:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',60 from dual union all
select 55176,'1982-W431','1982-10-15 21:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 55177,'1982-W432','1982-10-20 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',68 from dual union all
select 55178,'1982-W433','1982-10-20 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual;
insert into LAUNCH
select 55179,'1982-W434','1982-10-20 21:47:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 5518,'1958-Y52','1958-11-14 08:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55180,'1982-W435','1982-10-21 02:00:00','suborbital rocket','success',761,'MT135P-R473','','','','Meteo',1332,'',56 from dual union all
select 55181,'1982-W436','1982-10-22 00:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 55182,'1982-W437','1982-10-22 08:22:00','suborbital rocket','unknown',716,'T 1-0442','','','','Test/Aeron',1821,'',59 from dual union all
select 55183,'1982-W438','1982-10-27 02:00:00','suborbital rocket','success',761,'MT135P-R474','','','','Meteo',1332,'',56 from dual union all
select 55184,'1982-W439','1982-10-27 03:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 55185,'1982-W44','1982-01-20 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',73 from dual union all
select 55186,'1982-W440','1982-10-27 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 55187,'1982-W441','1982-10-27 23:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 55188,'1982-W442','1982-11-01 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',910,'KRENKEL''',59 from dual union all
select 55189,'1982-W443','1982-11-01 19:55:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',64 from dual union all
select 5519,'1958-Y53','1958-11-18 08:45:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55190,'1982-W444','1982-11-01 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 55191,'1982-W445','1982-11-03 07:10:00','suborbital rocket','unknown',738,'','','','','Aeron',910,'KRENKEL''',59 from dual union all
select 55192,'1982-W446','1982-11-03 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',79 from dual union all
select 55193,'1982-W447','1982-11-04 15:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',57 from dual union all
select 55194,'1982-W448','1982-11-05 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',910,'KRENKEL''',58 from dual union all
select 55195,'1982-W449','1982-11-10 02:00:00','suborbital rocket','success',761,'MT135P-R475','','','','Meteo',1332,'',57 from dual union all
select 55196,'1982-W45','1982-01-20 20:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',71 from dual union all
select 55197,'1982-W450','1982-11-10 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',910,'KRENKEL''',59 from dual union all
select 55198,'1982-W451','1982-11-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',70 from dual union all
select 55199,'1982-W452','1982-11-10 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 552,'1945-S28','1945-10-15 14:06:26','military missile','success',1470,'','Backfire 3','','','Test',353,'',64 from dual union all
select 5520,'1958-Y54','1958-11-28 20:06:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55200,'1982-W453','1982-11-10 22:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',68 from dual union all
select 55201,'1982-W454','1982-11-11 13:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 55202,'1982-W455','1982-11-11 14:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',62 from dual union all
select 55203,'1982-W456','1982-11-12 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',910,'KRENKEL''',59 from dual union all
select 55204,'1982-W457','1982-11-12 21:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 55205,'1982-W458','1982-11-13 14:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 55206,'1982-W459','1982-11-13 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 55207,'1982-W46','1982-01-20 23:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 55208,'1982-W460','1982-11-15 07:10:00','suborbital rocket','unknown',738,'','','','','Aeron',910,'KRENKEL''',57 from dual union all
select 55209,'1982-W461','1982-11-15 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 5521,'1958-Y55','1958-12-08 19:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55210,'1982-W462','1982-11-15 14:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 55211,'1982-W463','1982-11-16 08:24:00','suborbital rocket','unknown',716,'T 1-0443','','','','Test/Aeron',1821,'',59 from dual union all
select 55212,'1982-W464','1982-11-16 08:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 55213,'1982-W465','1982-11-16 09:46:00','suborbital rocket','unknown',1284,'T 1-0448','','','','Test/Aeron',1821,'',78 from dual union all
select 55214,'1982-W466','1982-11-16 12:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 55215,'1982-W467','1982-11-16 12:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 55216,'1982-W468','1982-11-16 12:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 55217,'1982-W469','1982-11-16 13:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 55218,'1982-W47','1982-01-21 02:00:00','suborbital rocket','success',761,'MT135P-R446','','','','Meteo',1332,'',52 from dual union all
select 55219,'1982-W470','1982-11-16 14:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 5522,'1958-Y56','1958-12-10 11:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55220,'1982-W471','1982-11-16 20:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 55221,'1982-W472','1982-11-17 23:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',57 from dual union all
select 55222,'1982-W473','1982-11-18 02:00:00','suborbital rocket','success',761,'MT135P-R476','','','','Meteo',1332,'',54 from dual union all
select 55223,'1982-W474','1982-11-19 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',910,'KRENKEL''',60 from dual union all
select 55224,'1982-W475','1982-11-19 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 55225,'1982-W476','1982-11-22 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',910,'KRENKEL''',60 from dual union all
select 55226,'1982-W477','1982-11-24 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',910,'KRENKEL''',60 from dual union all
select 55227,'1982-W478','1982-11-24 08:33:00','suborbital rocket','unknown',716,'T 1-0444','','','','Test/Aeron',1821,'',63 from dual union all
select 55228,'1982-W479','1982-11-24 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 55229,'1982-W48','1982-01-21 21:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',56 from dual union all
select 5523,'1958-Y57','1958-12-11 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55230,'1982-W480','1982-11-25 01:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 55231,'1982-W481','1982-11-26 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',910,'KRENKEL''',61 from dual union all
select 55232,'1982-W482','1982-11-27 03:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 55233,'1982-W483','1982-11-29 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',910,'KRENKEL''',60 from dual union all
select 55234,'1982-W484','1982-11-29 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 55235,'1982-W485','1982-12-01 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',212,'BUGAEV',59 from dual union all
select 55236,'1982-W486','1982-12-01 15:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 55237,'1982-W487','1982-12-01 22:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 55238,'1982-W488','1982-12-02 02:00:00','suborbital rocket','success',761,'MT135P-R477','','','','Meteo',1332,'',55 from dual union all
select 55239,'1982-W489','1982-12-02 08:27:00','suborbital rocket','unknown',716,'T 1-0450','','','','Test/Aeron',1821,'',60 from dual union all
select 5524,'1958-Y58','1958-12-12 11:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 55240,'1982-W49','1982-01-22 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',977,'MUSSON',60 from dual union all
select 55241,'1982-W490','1982-12-03 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',212,'BUGAEV',59 from dual union all
select 55242,'1982-W491','1982-12-03 08:15:00','suborbital rocket','unknown',716,'T 1-0451','','','','Test/Aeron',1821,'',62 from dual union all
select 55243,'1982-W492','1982-12-06 08:20:00','suborbital rocket','unknown',738,'','','','','Aeron',212,'BUGAEV',61 from dual union all
select 55244,'1982-W493','1982-12-07 20:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 55245,'1982-W494','1982-12-08 02:00:00','suborbital rocket','success',761,'MT135P-R478','','','','Meteo',1332,'',53 from dual union all
select 55246,'1982-W495','1982-12-08 08:40:00','suborbital rocket','unknown',738,'','','','','Aeron',212,'BUGAEV',60 from dual union all
select 55247,'1982-W496','1982-12-08 14:41:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',79 from dual union all
select 55248,'1982-W497','1982-12-08 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 55249,'1982-W498','1982-12-08 20:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 5525,'1959 ALP','1959-02-17 15:55:00','orbital','success',1486,'SLV-4','Vanguard II','Cloud cover satellite','','Sat',235,'',null from dual union all
select 55250,'1982-W499','1982-12-13 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 55251,'1982-W50','1982-01-22 09:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',59 from dual union all
select 55252,'1982-W500','1982-12-15 02:00:00','suborbital rocket','success',761,'MT135P-R479','','','','Meteo',1332,'',56 from dual union all
select 55253,'1982-W501','1982-12-15 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',79 from dual union all
select 55254,'1982-W502','1982-12-15 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',52 from dual union all
select 55255,'1982-W503','1982-12-15 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 55256,'1982-W504','1982-12-16 20:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 55257,'1982-W505','1982-12-17 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',212,'BUGAEV',58 from dual union all
select 55258,'1982-W506','1982-12-20 08:10:00','suborbital rocket','unknown',716,'T 1-0452','','','','Test/Aeron',1821,'',61 from dual union all
select 55259,'1982-W507','1982-12-20 20:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 5526,'1959 BET','1959-02-28 21:49:16','orbital','failure',1360,'163/FTV 1022','Discoverer 1','CORONA Test Vehicle 2','FLYING YANKEE','Sat TAG1',1678,'',295 from dual union all
select 55260,'1982-W508','1982-12-21 20:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 55261,'1982-W509','1982-12-22 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',212,'BUGAEV',59 from dual union all
select 55262,'1982-W51','1982-01-22 14:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',59 from dual union all
select 55263,'1982-W510','1982-12-29 02:00:00','suborbital rocket','success',761,'MT135P-R480','','','','Meteo',1332,'',55 from dual union all
select 55264,'1982-W511','1982-12-29 07:20:00','suborbital rocket','unknown',738,'','','','','Aeron',212,'BUGAEV',64 from dual union all
select 55265,'1982-W512','1982-12-29 08:45:00','suborbital rocket','unknown',738,'','','','','Aeron',212,'BUGAEV',61 from dual union all
select 55266,'1982-W513','1982-12-31 00:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 55267,'1982-W514','1982-12-31 21:43:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 55268,'1982-W515','1982-04-28 02:00:00','suborbital rocket','success',761,'MT135P-R457','','','','Meteo',1332,'',60 from dual;
insert into LAUNCH
select 55269,'1982-W516','1982-08-04 00:31:30','suborbital rocket','success',1284,'SuLo 11/1','CAMP Datasonde','','','Aeron',409,'',84 from dual union all
select 5527,'1959 DEL','1959-08-07 14:24:20','orbital','success',1356,'134','Explorer 6','NASA S-2','','Sat',233,'',null from dual union all
select 55270,'1982-W517','1982-08-12 00:44:00','suborbital rocket','success',1284,'SuLo 11/2','CAMP Datasonde','','','Aeron',409,'',84 from dual union all
select 55271,'1982-W52','1982-01-23 13:48:00','suborbital rocket','unknown',716,'TM1-0231','','','','Aeron',1047,'',68 from dual union all
select 55272,'1982-W53','1982-01-24 13:05:00','suborbital rocket','unknown',716,'TM1-0232','','','','Aeron',1047,'',59 from dual union all
select 55273,'1982-W54','1982-01-24 14:32:00','suborbital rocket','unknown',716,'TM1-0233','','','','Aeron',1047,'',65 from dual union all
select 55274,'1982-W55','1982-01-25 16:00:00','suborbital rocket','unknown',1284,'FLIGHT 297     /TN1-','','','','Aeron',330,'',67 from dual union all
select 55275,'1982-W56','1982-01-25 16:40:00','suborbital rocket','unknown',1284,'FLIGHT 298     /TN1-','','','','Aeron',330,'',65 from dual union all
select 55276,'1982-W57','1982-01-25 17:55:00','suborbital rocket','success',1281,'TN1-0145','','','','Aeron',330,'',69 from dual union all
select 55277,'1982-W58','1982-01-25 18:30:00','suborbital rocket','unknown',1284,'FLIGHT 299     /TN1-','','','','Aeron',330,'',62 from dual union all
select 55278,'1982-W59','1982-01-25 19:13:00','suborbital rocket','success',1281,'TN1-0146','','','','Aeron',330,'',68 from dual union all
select 55279,'1982-W60','1982-01-25 19:30:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',58 from dual union all
select 5528,'1959 EPS','1959-08-13 19:00:08','orbital','success',1360,'192/FTV 1029','Discoverer 5','CORONA C-2','FLY HIGH','Sat TAG5',1678,'',null from dual union all
select 55280,'1982-W61','1982-01-25 21:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 55281,'1982-W62','1982-01-26 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 55282,'1982-W63','1982-01-26 15:44:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',61 from dual union all
select 55283,'1982-W64','1982-01-27 02:00:00','suborbital rocket','success',761,'MT135P-R447','','','','Meteo',1332,'',55 from dual union all
select 55284,'1982-W65','1982-01-27 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',977,'MUSSON',57 from dual union all
select 55285,'1982-W66','1982-01-27 12:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',59 from dual union all
select 55286,'1982-W67','1982-01-27 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',56 from dual union all
select 55287,'1982-W68','1982-01-27 15:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 55288,'1982-W69','1982-01-27 16:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',51 from dual union all
select 55289,'1982-W70','1982-01-27 23:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 5529,'1959 ETA','1959-09-18 05:20:07','orbital','success',1485,'SLV-7','Vanguard III','Magne-Ray satellite','','Sat',235,'',null from dual union all
select 55290,'1982-W71','1982-01-28 17:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 55291,'1982-W72','1982-01-28 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',78 from dual union all
select 55292,'1982-W73','1982-01-29 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',977,'MUSSON',58 from dual union all
select 55293,'1982-W74','1982-01-29 13:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',57 from dual union all
select 55294,'1982-W75','1982-01-29 18:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 55295,'1982-W76','1982-01-29 20:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 55296,'1982-W77','1982-01-29 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',58 from dual union all
select 55297,'1982-W78','1982-01-30 18:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',74 from dual union all
select 55298,'1982-W79','1982-01-30 19:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',62 from dual union all
select 55299,'1982-W80','1982-01-31 14:16:00','suborbital rocket','success',1281,'TN1-0137','','','','Aeron',330,'',71 from dual union all
select 553,'1945-S29','1945-02-11 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5530,'1959 GAM','1959-04-13 21:18:39','orbital','success',1360,'170/FTV 1018','Discoverer 2','CORONA BIO 1','EARLY TIME','Sat TAG2',1678,'',null from dual union all
select 55300,'1982-W81','1982-01-31 14:46:00','suborbital rocket','success',1281,'TN1-0138','','','','Aeron',330,'',76 from dual union all
select 55301,'1982-W82','1982-01-31 15:23:00','suborbital rocket','unknown',1284,'FLIGHT 300     /TN1-','','','','Aeron',330,'',65 from dual union all
select 55302,'1982-W83','1982-02-01 12:57:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 55303,'1982-W84','1982-02-01 20:03:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',72 from dual union all
select 55304,'1982-W85','1982-02-01 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',58 from dual union all
select 55305,'1982-W86','1982-02-02 01:30:00','suborbital rocket','success',761,'MT135P T31','','','','Aeron?',1476,'',50 from dual union all
select 55306,'1982-W87','1982-02-02 20:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 55307,'1982-W88','1982-02-03 01:30:00','suborbital rocket','success',761,'MT135P T32','','','','Aeron?',1476,'',50 from dual union all
select 55308,'1982-W89','1982-02-03 02:15:59','suborbital rocket','success',761,'MT135P-R448','','','','Meteo',1332,'',56 from dual union all
select 55309,'1982-W90','1982-02-03 15:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 5531,'1959 IOT','1959-10-13 15:30:04','orbital','success',634,'AM-19A','Explorer 7','NASA S-1A','','Sat',279,'',null from dual union all
select 55310,'1982-W91','1982-02-03 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',70 from dual union all
select 55311,'1982-W92','1982-02-03 19:04:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',35,'',71 from dual union all
select 55312,'1982-W93','1982-02-03 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',58 from dual union all
select 55313,'1982-W94','1982-02-03 21:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 55314,'1982-W95','1982-02-03 22:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 55315,'1982-W96','1982-02-04 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',276,'',60 from dual union all
select 55316,'1982-W97','1982-02-04 16:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 55317,'1982-W98','1982-02-04 19:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 55318,'1982-W99','1982-02-05 15:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',176,'',62 from dual union all
select 55319,'1982-Y01','1982-01-01 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 5532,'1959 KAP','1959-11-07 20:28:41','orbital','success',1360,'206/FTV 1051','Discoverer 7','CORONA C-4','CARGO NET','Sat TAG7',1678,'',null from dual union all
select 55320,'1982-Y02','1982-01-04 11:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 55321,'1982-Y03','1982-01-05 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 55322,'1982-Y04','1982-01-06 03:45:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 55323,'1982-Y05','1982-01-06 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',80 from dual union all
select 55324,'1982-Y06','1982-01-06 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 55325,'1982-Y07','1982-01-06 19:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 55326,'1982-Y08','1982-01-06 22:02:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 55327,'1982-Y09','1982-01-12 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 55328,'1982-Y10','1982-01-13 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 55329,'1982-Y100','1982-03-24 01:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 5533,'1959 LAM','1959-11-20 19:25:24','orbital','success',1360,'212/FTV 1050','Discoverer 8','CORONA C-5','LIVID LADY','Sat TAG8',1679,'',null from dual union all
select 55330,'1982-Y101','1982-03-24 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 55331,'1982-Y102','1982-03-24 10:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 55332,'1982-Y103','1982-03-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 55333,'1982-Y104','1982-03-24 19:04:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',91 from dual union all
select 55334,'1982-Y105','1982-03-25 16:51:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 55335,'1982-Y106','1982-03-25 17:25:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',88 from dual union all
select 55336,'1982-Y107','1982-03-25 19:04:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',95 from dual union all
select 55337,'1982-Y108','1982-03-25 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 55338,'1982-Y109','1982-03-26 17:07:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',92 from dual union all
select 55339,'1982-Y11','1982-01-13 14:26:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',82 from dual union all
select 5534,'1959 MU','1959-01-02 16:41:21','deep space','success',1513,'B1-6','Luna-1','E-1 No. 4','','Sat',480,'',null from dual union all
select 55340,'1982-Y110','1982-03-26 19:18:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',95 from dual union all
select 55341,'1982-Y111','1982-03-27 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',85 from dual union all
select 55342,'1982-Y112','1982-03-27 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',85 from dual union all
select 55343,'1982-Y113','1982-03-27 19:04:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 55344,'1982-Y114','1982-03-28 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',90 from dual union all
select 55345,'1982-Y115','1982-03-28 16:57:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 55346,'1982-Y116','1982-03-28 17:35:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',90 from dual union all
select 55347,'1982-Y118','1982-03-29 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',89 from dual union all
select 55348,'1982-Y119','1982-03-29 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',90 from dual union all
select 55349,'1982-Y12','1982-01-13 18:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 5535,'1959 NU','1959-03-03 05:10:56','deep space','success',634,'AM-14','Pioneer 4','Pioneer 4','','Sat',279,'',null from dual union all
select 55350,'1982-Y120','1982-03-29 19:37:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',91 from dual union all
select 55351,'1982-Y121','1982-03-30 13:30:00','suborbital rocket','unknown',1284,'T 1-0294','','','','Test/Aeron',1821,'',97 from dual union all
select 55352,'1982-Y122','1982-03-30 13:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 55353,'1982-Y123','1982-03-30 13:47:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 55354,'1982-Y124','1982-03-30 14:59:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',93 from dual union all
select 55355,'1982-Y125','1982-03-30 15:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 55356,'1982-Y126','1982-03-30 16:54:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',91 from dual union all
select 55357,'1982-Y127','1982-03-30 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 55358,'1982-Y128','1982-03-31 01:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual;
insert into LAUNCH
select 55359,'1982-Y129','1982-03-31 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 5536,'1959 THE','1959-10-04 00:43:39','deep space','success',1513,'I1-8','Luna-3','E-2A','','Sat',480,'',null from dual union all
select 55360,'1982-Y13','1982-01-14 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 55361,'1982-Y130','1982-03-31 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 55362,'1982-Y131','1982-04-05 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 55363,'1982-Y132','1982-04-06 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 55364,'1982-Y133','1982-04-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 55365,'1982-Y134','1982-04-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 55366,'1982-Y135','1982-04-07 16:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 55367,'1982-Y136','1982-04-07 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 55368,'1982-Y137','1982-04-07 19:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 55369,'1982-Y138','1982-04-14 03:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 5537,'1959 XI','1959-09-12 06:39:42','deep space','success',1513,'I1-7B','Luna-2','E-1 No. 6','','Sat',480,'',null from dual union all
select 55370,'1982-Y139','1982-04-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 55371,'1982-Y14','1982-01-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 55372,'1982-Y140','1982-04-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 55373,'1982-Y141','1982-04-14 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 55374,'1982-Y142','1982-04-19 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 55375,'1982-Y143','1982-04-19 09:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 55376,'1982-Y144','1982-04-19 20:54:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 55377,'1982-Y145','1982-04-20 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 55378,'1982-Y146','1982-04-21 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 55379,'1982-Y147','1982-04-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 5538,'1959 ZET','1959-08-19 19:24:44','orbital','success',1360,'200/FTV 1028','Discoverer 6','CORONA C-3','HURRY UP','Sat TAG6',1679,'',null from dual union all
select 55380,'1982-Y148','1982-04-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 55381,'1982-Y149','1982-04-21 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 55382,'1982-Y15','1982-01-14 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 55383,'1982-Y150','1982-04-22 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 55384,'1982-Y151','1982-04-27 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 55385,'1982-Y152','1982-04-28 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 55386,'1982-Y153','1982-04-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 55387,'1982-Y154','1982-04-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 55388,'1982-Y155','1982-04-28 16:55:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',96 from dual union all
select 55389,'1982-Y156','1982-04-28 17:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 5539,'1959-A01','1959-01-01 00:00:00','suborbital rocket','success',123,'','Cleansweep I','','','Test',1490,'',30 from dual union all
select 55390,'1982-Y157','1982-04-28 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 55391,'1982-Y158','1982-04-28 18:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 55392,'1982-Y159','1982-04-28 21:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 55393,'1982-Y16','1982-01-15 13:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',85 from dual union all
select 55394,'1982-Y160','1982-05-03 07:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 55395,'1982-Y161','1982-05-05 02:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 55396,'1982-Y162','1982-05-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 55397,'1982-Y163','1982-05-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 55398,'1982-Y164','1982-05-05 17:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',96 from dual union all
select 55399,'1982-Y165','1982-05-07 05:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 554,'1945-S30','1945-02-22 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5540,'1959-A02','1959-01-01 00:00:00','atmospheric rocket','success',551,'','Polaris RTV','','','Reentry Test',1821,'',26 from dual union all
select 55400,'1982-Y166','1982-05-07 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 55401,'1982-Y167','1982-05-11 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 55402,'1982-Y168','1982-05-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 55403,'1982-Y169','1982-05-12 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 55404,'1982-Y17','1982-01-17 15:34:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',85 from dual union all
select 55405,'1982-Y170','1982-05-12 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 55406,'1982-Y171','1982-05-13 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 55407,'1982-Y172','1982-05-18 17:26:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 55408,'1982-Y173','1982-05-18 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 55409,'1982-Y174','1982-05-19 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 5541,'1959-A03','1959-01-01 00:00:00','atmospheric rocket','success',552,'','','','','Reentry Test',1821,'',25 from dual union all
select 55410,'1982-Y175','1982-05-19 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 55411,'1982-Y176','1982-05-19 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 55412,'1982-Y177','1982-05-19 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 55413,'1982-Y178','1982-05-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 55414,'1982-Y179','1982-05-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 55415,'1982-Y18','1982-01-19 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 55416,'1982-Y180','1982-05-26 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 55417,'1982-Y181','1982-05-26 20:57:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 55418,'1982-Y182','1982-05-30 20:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',84 from dual union all
select 55419,'1982-Y183','1982-05-31 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',80 from dual union all
select 5542,'1959-A04','1959-01-01 00:00:00','atmospheric rocket','success',552,'','','','','Reentry Test',1821,'',25 from dual union all
select 55420,'1982-Y184','1982-06-01 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',84 from dual union all
select 55421,'1982-Y185','1982-06-01 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 55422,'1982-Y186','1982-06-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 55423,'1982-Y187','1982-06-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 55424,'1982-Y188','1982-06-02 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 55425,'1982-Y189','1982-06-02 21:25:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',83 from dual union all
select 55426,'1982-Y19','1982-01-19 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 55427,'1982-Y190','1982-06-04 02:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 55428,'1982-Y191','1982-06-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 55429,'1982-Y192','1982-06-04 18:55:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',93 from dual union all
select 5543,'1959-A05','1959-01-01 00:00:00','atmospheric rocket','success',552,'','','','','Reentry Test',1821,'',25 from dual union all
select 55430,'1982-Y193','1982-06-09 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 55431,'1982-Y194','1982-06-09 08:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 55432,'1982-Y195','1982-06-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 55433,'1982-Y196','1982-06-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 55434,'1982-Y197','1982-06-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 55435,'1982-Y198','1982-06-09 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 55436,'1982-Y199','1982-06-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 55437,'1982-Y20','1982-01-20 01:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 55438,'1982-Y200','1982-06-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 55439,'1982-Y201','1982-06-15 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 5544,'1959-A06','1959-01-01 00:00:00','military missile','failure',974,'','','','','Test',190,'B-67',0 from dual union all
select 55440,'1982-Y202','1982-06-16 08:15:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 55441,'1982-Y203','1982-06-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 55442,'1982-Y204','1982-06-16 17:50:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 55443,'1982-Y205','1982-06-17 10:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 55444,'1982-Y206','1982-06-17 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 55445,'1982-Y207','1982-06-18 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 55446,'1982-Y208','1982-06-23 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 55447,'1982-Y209','1982-06-23 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 55448,'1982-Y21','1982-01-20 06:45:00','suborbital rocket','success',1281,'NASA 15.213UE','','','','Plasma',1047,'',80 from dual;
insert into LAUNCH
select 55449,'1982-Y210','1982-06-25 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 5545,'1959-A07','1959-01-01 00:00:00','atmospheric rocket','success',1051,'','Blunt cone Stab.','','','Test',1821,'',6 from dual union all
select 55450,'1982-Y211','1982-06-25 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',86 from dual union all
select 55451,'1982-Y212','1982-06-27 15:52:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',95 from dual union all
select 55452,'1982-Y213','1982-06-29 10:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 55453,'1982-Y214','1982-06-29 11:45:00','suborbital rocket','unknown',1284,'T 1-0303','','','','Test/Aeron',1821,'',95 from dual union all
select 55454,'1982-Y215','1982-06-29 13:49:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 55455,'1982-Y216','1982-06-29 14:25:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',83 from dual union all
select 55456,'1982-Y217','1982-06-29 15:10:00','suborbital rocket','unknown',1284,'T 1-0302','','','','Test/Aeron',1821,'',95 from dual union all
select 55457,'1982-Y218','1982-06-29 15:48:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 55458,'1982-Y219','1982-06-29 16:12:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',93 from dual union all
select 55459,'1982-Y22','1982-01-20 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 5546,'1959-A08','1959-01-01 00:00:00','atmospheric rocket','success',1051,'','Ogive Stab.','','','Test',1821,'',4 from dual union all
select 55460,'1982-Y220','1982-06-29 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 55461,'1982-Y221','1982-06-30 10:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 55462,'1982-Y222','1982-06-30 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 55463,'1982-Y223','1982-06-30 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 55464,'1982-Y224','1982-06-30 17:35:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 55465,'1982-Y225','1982-06-30 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 55466,'1982-Y226','1982-07-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 55467,'1982-Y227','1982-07-01 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 55468,'1982-Y228','1982-07-01 15:48:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',95 from dual union all
select 55469,'1982-Y229','1982-07-01 22:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',80 from dual union all
select 5547,'1959-A09','1959-01-07 00:00:00','military missile','success',771,'Training #7','','','','Test',222,'',12 from dual union all
select 55470,'1982-Y23','1982-01-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 55471,'1982-Y230','1982-07-02 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 55472,'1982-Y231','1982-07-02 22:07:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',90 from dual union all
select 55473,'1982-Y232','1982-07-03 13:53:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 55474,'1982-Y233','1982-07-03 14:43:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 55475,'1982-Y234','1982-07-03 15:11:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 55476,'1982-Y235','1982-07-03 15:48:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 55477,'1982-Y236','1982-07-03 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',85 from dual union all
select 55478,'1982-Y237','1982-07-04 10:40:00','suborbital rocket','unknown',1284,'T 1-0306','','','','Test/Aeron',1821,'',97 from dual union all
select 55479,'1982-Y238','1982-07-04 11:00:00','suborbital rocket','unknown',1284,'T 1-0307','','','','Test/Aeron',1821,'',95 from dual union all
select 5548,'1959-A10','1959-01-08 00:00:00','military missile','success',771,'Training #8','','','','Test',222,'',12 from dual union all
select 55480,'1982-Y239','1982-07-04 11:09:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 55481,'1982-Y24','1982-01-21 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 55482,'1982-Y240','1982-07-04 13:12:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 55483,'1982-Y241','1982-07-04 13:47:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 55484,'1982-Y242','1982-07-04 14:12:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 55485,'1982-Y243','1982-07-04 14:59:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',95 from dual union all
select 55486,'1982-Y244','1982-07-04 15:23:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 55487,'1982-Y245','1982-07-04 15:48:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 55488,'1982-Y246','1982-07-04 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',87 from dual union all
select 55489,'1982-Y247','1982-07-05 22:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',89 from dual union all
select 5549,'1959-A100','1959-06-16 21:45:00','military missile','failure',1367,'191','IWST','','RIFLE SHOT','Test',1676,'',10 from dual union all
select 55490,'1982-Y248','1982-07-06 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',85 from dual union all
select 55491,'1982-Y249','1982-07-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 55492,'1982-Y25','1982-01-24 15:22:00','suborbital rocket','success',1281,'NASA 15.216UE','','','','Plasma',1047,'',80 from dual union all
select 55493,'1982-Y250','1982-07-07 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 55494,'1982-Y251','1982-07-07 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 55495,'1982-Y252','1982-07-07 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',84 from dual union all
select 55496,'1982-Y253','1982-07-08 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',80 from dual union all
select 55497,'1982-Y254','1982-07-09 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 55498,'1982-Y255','1982-07-09 14:58:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 55499,'1982-Y256','1982-07-09 22:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',82 from dual union all
select 555,'1945-S31','1945-02-26 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5550,'1959-A101','1959-06-18 00:00:00','atmospheric rocket','success',861,'','Mercury model','','','Test',1821,'',10 from dual union all
select 55500,'1982-Y257','1982-07-10 21:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',85 from dual union all
select 55501,'1982-Y258','1982-07-11 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',83 from dual union all
select 55502,'1982-Y259','1982-07-12 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',83 from dual union all
select 55503,'1982-Y26','1982-01-26 20:08:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 55504,'1982-Y260','1982-07-12 20:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 55505,'1982-Y261','1982-07-13 17:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 55506,'1982-Y262','1982-07-13 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',90 from dual union all
select 55507,'1982-Y263','1982-07-13 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 55508,'1982-Y264','1982-07-13 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 55509,'1982-Y265','1982-07-14 02:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 5551,'1959-A102','1959-06-29 00:00:00','atmospheric rocket','failure',1065,'','','','','Test',2,'',0 from dual union all
select 55510,'1982-Y266','1982-07-14 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 55511,'1982-Y267','1982-07-14 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 55512,'1982-Y268','1982-07-14 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1380,'SHIR',85 from dual union all
select 55513,'1982-Y269','1982-07-16 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 55514,'1982-Y27','1982-01-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 55515,'1982-Y270','1982-07-16 21:22:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 55516,'1982-Y271','1982-07-19 15:32:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 55517,'1982-Y272','1982-07-19 21:44:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',85 from dual union all
select 55518,'1982-Y273','1982-07-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 55519,'1982-Y274','1982-07-21 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 5552,'1959-A103','1959-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55520,'1982-Y275','1982-07-21 17:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 55521,'1982-Y276','1982-07-23 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 55522,'1982-Y277','1982-07-26 20:52:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',81 from dual union all
select 55523,'1982-Y278','1982-07-27 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 55524,'1982-Y279','1982-07-27 22:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 55525,'1982-Y28','1982-01-27 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',84 from dual union all
select 55526,'1982-Y280','1982-07-28 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 55527,'1982-Y281','1982-07-28 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 55528,'1982-Y282','1982-07-28 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 55529,'1982-Y283','1982-07-30 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 5553,'1959-A104','1959-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55530,'1982-Y284','1982-08-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 55531,'1982-Y285','1982-08-04 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 55532,'1982-Y286','1982-08-04 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 55533,'1982-Y287','1982-08-05 20:57:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',95 from dual union all
select 55534,'1982-Y288','1982-08-06 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 55535,'1982-Y289','1982-08-09 18:53:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 55536,'1982-Y29','1982-01-27 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 55537,'1982-Y290','1982-08-09 21:24:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 55538,'1982-Y291','1982-08-10 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual;
insert into LAUNCH
select 55539,'1982-Y292','1982-08-11 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 5554,'1959-A105','1959-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55540,'1982-Y293','1982-08-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 55541,'1982-Y294','1982-08-11 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 55542,'1982-Y295','1982-08-11 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 55543,'1982-Y296','1982-08-11 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 55544,'1982-Y297','1982-08-17 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 55545,'1982-Y298','1982-08-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 55546,'1982-Y299','1982-08-18 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 55547,'1982-Y30','1982-01-29 03:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 55548,'1982-Y300','1982-08-18 16:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 55549,'1982-Y301','1982-08-18 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 5555,'1959-A106','1959-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55550,'1982-Y302','1982-08-19 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 55551,'1982-Y303','1982-08-20 15:33:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 55552,'1982-Y304','1982-08-24 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 55553,'1982-Y305','1982-08-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',90 from dual union all
select 55554,'1982-Y306','1982-08-25 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 55555,'1982-Y307','1982-08-25 16:55:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 55556,'1982-Y308','1982-08-25 19:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 55557,'1982-Y309','1982-08-25 19:52:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 55558,'1982-Y31','1982-01-29 17:58:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 55559,'1982-Y310','1982-08-25 22:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',80 from dual union all
select 5556,'1959-A107','1959-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55560,'1982-Y311','1982-09-01 01:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 55561,'1982-Y312','1982-09-01 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 55562,'1982-Y313','1982-09-01 02:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 55563,'1982-Y314','1982-09-01 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 55564,'1982-Y315','1982-09-01 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 55565,'1982-Y316','1982-09-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 55566,'1982-Y317','1982-09-07 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 55567,'1982-Y318','1982-09-07 20:53:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',96 from dual union all
select 55568,'1982-Y319','1982-09-08 03:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 55569,'1982-Y32','1982-02-02 19:20:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 5557,'1959-A108','1959-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55570,'1982-Y320','1982-09-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 55571,'1982-Y321','1982-09-08 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 55572,'1982-Y322','1982-09-08 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 55573,'1982-Y323','1982-09-08 17:32:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',82 from dual union all
select 55574,'1982-Y324','1982-09-09 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 55575,'1982-Y325','1982-09-09 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',96 from dual union all
select 55576,'1982-Y326','1982-09-14 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 55577,'1982-Y327','1982-09-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 55578,'1982-Y328','1982-09-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 55579,'1982-Y329','1982-09-15 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 5558,'1959-A109','1959-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55580,'1982-Y33','1982-02-02 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 55581,'1982-Y330','1982-09-21 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 55582,'1982-Y331','1982-09-21 21:42:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',95 from dual union all
select 55583,'1982-Y332','1982-09-22 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 55584,'1982-Y333','1982-09-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',89 from dual union all
select 55585,'1982-Y334','1982-09-22 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 55586,'1982-Y335','1982-09-22 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 55587,'1982-Y336','1982-09-28 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 55588,'1982-Y337','1982-09-28 21:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 55589,'1982-Y338','1982-09-29 07:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 5559,'1959-A11','1959-01-10 00:00:00','military missile','failure',1080,'F86','','','','Test',671,'',20 from dual union all
select 55590,'1982-Y339','1982-09-29 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 55591,'1982-Y34','1982-02-03 03:25:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 55592,'1982-Y340','1982-09-29 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 55593,'1982-Y341','1982-09-30 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 55594,'1982-Y342','1982-10-05 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 55595,'1982-Y343','1982-10-05 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 55596,'1982-Y344','1982-10-05 21:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 55597,'1982-Y345','1982-10-06 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 55598,'1982-Y346','1982-10-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 55599,'1982-Y347','1982-10-06 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 556,'1945-S32','1945-03-07 01:15:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5560,'1959-A110','1959-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55600,'1982-Y348','1982-10-07 15:48:00','suborbital rocket','success',1281,'NASA 15.217UE','','','','Plasma',1821,'',80 from dual union all
select 55601,'1982-Y349','1982-10-12 19:15:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 55602,'1982-Y35','1982-02-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 55603,'1982-Y350','1982-10-12 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 55604,'1982-Y351','1982-10-13 04:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 55605,'1982-Y352','1982-10-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 55606,'1982-Y353','1982-10-13 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 55607,'1982-Y354','1982-10-18 19:58:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 55608,'1982-Y355','1982-10-19 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 55609,'1982-Y356','1982-10-20 04:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 5561,'1959-A111','1959-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55610,'1982-Y357','1982-10-20 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 55611,'1982-Y358','1982-10-20 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 55612,'1982-Y359','1982-10-26 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 55613,'1982-Y36','1982-02-03 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 55614,'1982-Y360','1982-10-27 05:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',80 from dual union all
select 55615,'1982-Y361','1982-10-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 55616,'1982-Y362','1982-10-27 17:01:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 55617,'1982-Y363','1982-11-02 20:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 55618,'1982-Y364','1982-11-03 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 55619,'1982-Y365','1982-11-03 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 5562,'1959-A112','1959-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55620,'1982-Y366','1982-11-09 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 55621,'1982-Y367','1982-11-10 05:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 55622,'1982-Y368','1982-11-10 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 55623,'1982-Y369','1982-11-11 12:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 55624,'1982-Y37','1982-02-03 21:23:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 55625,'1982-Y370','1982-11-11 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',91 from dual union all
select 55626,'1982-Y371','1982-11-13 12:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 55627,'1982-Y372','1982-11-13 13:36:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 55628,'1982-Y373','1982-11-13 14:43:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual;
insert into LAUNCH
select 55629,'1982-Y374','1982-11-15 13:26:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',93 from dual union all
select 5563,'1959-A113','1959-07-02 00:00:00','military missile','success',1214,'N-69D N-3320','','','','OT',224,'',10 from dual union all
select 55630,'1982-Y375','1982-11-15 14:04:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 55631,'1982-Y376','1982-11-16 08:56:00','suborbital rocket','success',1284,'CONDOR Test','','','','Aeron',1821,'',86 from dual union all
select 55632,'1982-Y377','1982-11-16 09:11:00','suborbital rocket','success',1284,'CONDOR Test','','','','Aeron',1821,'',84 from dual union all
select 55633,'1982-Y378','1982-11-16 09:31:00','suborbital rocket','success',1284,'CONDOR Test','','','','Aeron',1821,'',88 from dual union all
select 55634,'1982-Y379','1982-11-16 10:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 55635,'1982-Y38','1982-02-04 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 55636,'1982-Y380','1982-11-16 11:09:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 55637,'1982-Y381','1982-11-16 11:33:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 55638,'1982-Y382','1982-11-16 11:54:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 55639,'1982-Y383','1982-11-16 12:11:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 5564,'1959-A114','1959-07-07 00:00:00','atmospheric rocket','failure',1065,'','','','','Test',2,'',0 from dual union all
select 55640,'1982-Y384','1982-11-16 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 55641,'1982-Y385','1982-11-16 13:33:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 55642,'1982-Y386','1982-11-16 14:08:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 55643,'1982-Y387','1982-11-16 14:31:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 55644,'1982-Y388','1982-11-16 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 55645,'1982-Y389','1982-11-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 55646,'1982-Y39','1982-02-08 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',85 from dual union all
select 55647,'1982-Y390','1982-11-17 14:50:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 55648,'1982-Y391','1982-11-23 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 55649,'1982-Y392','1982-11-23 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 5565,'1959-A115','1959-07-12 00:00:00','atmospheric rocket','success',552,'','GE Ablative RV','','','Reentry Test',1821,'',30 from dual union all
select 55650,'1982-Y393','1982-11-24 08:47:00','suborbital rocket','unknown',1284,'T 1-0455','','','','Test/Aeron',1821,'',110 from dual union all
select 55651,'1982-Y394','1982-11-24 09:03:00','suborbital rocket','unknown',1284,'T 1-0456','','','','Test/Aeron',1821,'',117 from dual union all
select 55652,'1982-Y395','1982-11-24 09:50:00','suborbital rocket','unknown',1284,'T 1-0457','','','','Test/Aeron',1821,'',105 from dual union all
select 55653,'1982-Y396','1982-11-24 11:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 55654,'1982-Y397','1982-11-24 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 55655,'1982-Y398','1982-11-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 55656,'1982-Y399','1982-11-25 17:51:00','suborbital rocket','unknown',725,'','','','','Aeron',1753,'VOKV',82 from dual union all
select 55657,'1982-Y40','1982-02-09 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 55658,'1982-Y400','1982-11-29 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1753,'VOKV',83 from dual union all
select 55659,'1982-Y401','1982-11-30 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 5566,'1959-A116','1959-07-14 16:58:00','military missile','success',256,'631-2','','','','OT',254,'',20 from dual union all
select 55660,'1982-Y402','1982-12-01 11:15:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 55661,'1982-Y403','1982-12-01 12:45:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 55662,'1982-Y404','1982-12-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 55663,'1982-Y405','1982-12-01 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 55664,'1982-Y406','1982-12-01 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1753,'VOKV',87 from dual union all
select 55665,'1982-Y407','1982-12-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 55666,'1982-Y408','1982-12-02 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 55667,'1982-Y409','1982-12-03 16:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',93 from dual union all
select 55668,'1982-Y41','1982-02-10 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 55669,'1982-Y410','1982-12-05 19:25:00','suborbital rocket','unknown',725,'','','','','Aeron',1753,'VOKV',87 from dual union all
select 5567,'1959-A117','1959-07-15 16:59:00','military missile','failure',933,'AX-11','','','','Test',244,'',2 from dual union all
select 55670,'1982-Y411','1982-12-07 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1753,'VOKV',87 from dual union all
select 55671,'1982-Y412','1982-12-07 20:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 55672,'1982-Y413','1982-12-08 11:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 55673,'1982-Y414','1982-12-08 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 55674,'1982-Y415','1982-12-09 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1753,'VOKV',85 from dual union all
select 55675,'1982-Y416','1982-12-09 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 55676,'1982-Y417','1982-12-10 11:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 55677,'1982-Y418','1982-12-10 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 55678,'1982-Y419','1982-12-14 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 55679,'1982-Y42','1982-02-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 5568,'1959-A118','1959-07-17 17:40:00','suborbital rocket','failure',838,'NN8.66','','','','Solar',1015,'',10 from dual union all
select 55680,'1982-Y420','1982-12-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 55681,'1982-Y421','1982-12-15 15:15:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 55682,'1982-Y422','1982-12-16 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 55683,'1982-Y423','1982-12-17 03:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 55684,'1982-Y424','1982-12-20 08:30:00','suborbital rocket','unknown',1284,'T 1-0466','','','','Test/Aeron',1821,'',97 from dual union all
select 55685,'1982-Y425','1982-12-20 09:08:00','suborbital rocket','unknown',1284,'T 1-0467','','','','Test/Aeron',1821,'',91 from dual union all
select 55686,'1982-Y426','1982-12-21 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 55687,'1982-Y427','1982-12-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 55688,'1982-Y428','1982-12-22 15:45:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 55689,'1982-Y429','1982-12-23 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 5569,'1959-A119','1959-07-21 07:33:00','military missile','failure',1367,'203','R&D Series IV','','','Test',234,'',1 from dual union all
select 55690,'1982-Y43','1982-02-10 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 55691,'1982-Y430','1982-12-24 14:10:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 55692,'1982-Y431','1982-12-28 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 55693,'1982-Y432','1982-12-28 23:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1753,'VOKV',85 from dual union all
select 55694,'1982-Y433','1982-12-29 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 55695,'1982-Y434','1982-12-29 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 55696,'1982-Y435','1982-12-29 17:46:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 55697,'1982-Y436','1982-12-30 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',85 from dual union all
select 55698,'1982-Y437','1982-12-31 03:45:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 55699,'1982-Y438','1982-12-31 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1753,'VOKV',90 from dual union all
select 557,'1945-S33','1945-03-08 10:57:00','military missile','success',1470,'','London/Smithfield','','','Weapon',1627,'',90 from dual union all
select 5570,'1959-A12','1959-01-13 00:00:00','military missile','success',1080,'F87','','','','Test',671,'',40 from dual union all
select 55700,'1982-Y439','1982-12-31 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1753,'VOKV',90 from dual union all
select 55701,'1982-Y44','1982-02-10 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 55702,'1982-Y440','1982-11-29 05:21:00','suborbital rocket','success',725,'','Gerdien probe','','','Aeron',1467,'',83 from dual union all
select 55703,'1982-Y45','1982-02-11 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 55704,'1982-Y46','1982-02-16 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 55705,'1982-Y47','1982-02-17 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 55706,'1982-Y48','1982-02-17 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 55707,'1982-Y49','1982-02-17 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 55708,'1982-Y50','1982-02-18 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',86 from dual union all
select 55709,'1982-Y51','1982-02-19 06:35:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 5571,'1959-A120','1959-07-24 00:00:00','suborbital spaceplane','success',1520,'2-C-1','','','','Test',379,'NB-52 003',14 from dual union all
select 55710,'1982-Y52','1982-02-19 09:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 55711,'1982-Y53','1982-02-19 12:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',90 from dual union all
select 55712,'1982-Y54','1982-02-23 21:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 55713,'1982-Y55','1982-02-24 03:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 55714,'1982-Y56','1982-02-24 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 55715,'1982-Y57','1982-02-24 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 55716,'1982-Y58','1982-02-24 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 55717,'1982-Y59','1982-02-24 19:00:00','suborbital rocket','unknown',1284,'T 1-0106','','','','Test/Aeron',1821,'',82 from dual union all
select 55718,'1982-Y60','1982-02-26 19:20:00','suborbital rocket','unknown',1284,'T 1-0108','','','','Test/Aeron',1821,'',80 from dual;
insert into LAUNCH
select 55719,'1982-Y61','1982-03-01 20:00:00','suborbital rocket','unknown',1284,'T 1-0113','','','','Test/Aeron',1821,'',80 from dual union all
select 5572,'1959-A121','1959-07-31 00:00:00','suborbital rocket','failure',88,'','','','','Aeron',1821,'',10 from dual union all
select 55720,'1982-Y62','1982-03-02 19:55:00','suborbital rocket','unknown',1284,'T 1-0116','','','','Test/Aeron',1821,'',81 from dual union all
select 55721,'1982-Y63','1982-03-02 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 55722,'1982-Y64','1982-03-03 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 55723,'1982-Y65','1982-03-03 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 55724,'1982-Y66','1982-03-03 19:00:00','suborbital rocket','unknown',1284,'T 1-0117','','','','Test/Aeron',1821,'',83 from dual union all
select 55725,'1982-Y67','1982-03-03 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 55726,'1982-Y68','1982-03-04 15:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',85 from dual union all
select 55727,'1982-Y69','1982-03-04 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 55728,'1982-Y70','1982-03-05 19:03:00','suborbital rocket','unknown',1284,'T 1-0119','','','','Test/Aeron',1821,'',81 from dual union all
select 55729,'1982-Y71','1982-03-07 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',89 from dual union all
select 5573,'1959-A122','1959-07-31 00:00:00','suborbital rocket','failure',88,'','','','','Aeron',1821,'',10 from dual union all
select 55730,'1982-Y72','1982-03-08 19:08:00','suborbital rocket','unknown',1284,'T 1-0122','','','','Test/Aeron',1821,'',81 from dual union all
select 55731,'1982-Y73','1982-03-09 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 55732,'1982-Y74','1982-03-10 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 55733,'1982-Y75','1982-03-10 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',87 from dual union all
select 55734,'1982-Y76','1982-03-10 15:20:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',93 from dual union all
select 55735,'1982-Y77','1982-03-10 16:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 55736,'1982-Y78','1982-03-10 18:50:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',89 from dual union all
select 55737,'1982-Y79','1982-03-11 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 55738,'1982-Y80','1982-03-12 18:25:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',91 from dual union all
select 55739,'1982-Y81','1982-03-14 08:40:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',91 from dual union all
select 5574,'1959-A123','1959-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55740,'1982-Y82','1982-03-15 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',93 from dual union all
select 55741,'1982-Y83','1982-03-16 05:10:00','suborbital rocket','success',1281,'NASA 15.214UE','WATERHOLE support','','','Plasma',428,'',80 from dual union all
select 55742,'1982-Y84','1982-03-16 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',83 from dual union all
select 55743,'1982-Y85','1982-03-16 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 55744,'1982-Y86','1982-03-17 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 55745,'1982-Y87','1982-03-17 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 55746,'1982-Y88','1982-03-17 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 55747,'1982-Y89','1982-03-17 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',87 from dual union all
select 55748,'1982-Y90','1982-03-18 17:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',88 from dual union all
select 55749,'1982-Y91','1982-03-18 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 5575,'1959-A124','1959-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55750,'1982-Y92','1982-03-19 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',91 from dual union all
select 55751,'1982-Y93','1982-03-21 17:20:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',91 from dual union all
select 55752,'1982-Y94','1982-03-22 16:15:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',80 from dual union all
select 55753,'1982-Y95','1982-03-22 18:42:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',96 from dual union all
select 55754,'1982-Y96','1982-03-22 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',893,'KORO',89 from dual union all
select 55755,'1982-Y97','1982-03-23 17:01:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 55756,'1982-Y98','1982-03-23 19:04:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 55757,'1982-Y99','1982-03-23 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 55758,'1983-001','1983-01-12 14:02:00','orbital','success',680,'65093-479','Kosmos-1428','Parus','','Sat',457,'',null from dual union all
select 55759,'1983-002','1983-01-19 02:25:00','orbital','success',680,'53793-481','Kosmos-1429/1436','Strela-1M','','Sat',456,'',null from dual union all
select 5576,'1959-A125','1959-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55760,'1983-003','1983-01-20 17:26:00','orbital','success',1510,'76053  -342','Kosmos-1437','Tselina-D','','Sat',462,'',null from dual union all
select 55761,'1983-004','1983-01-26 02:17:00','orbital','success',387,'650/D166','IRAS','IRAS','','Sat',1724,'',null from dual union all
select 55762,'1983-005','1983-01-27 08:30:00','orbital','success',1238,'77024  -773','Kosmos-1438','Zenit-6','','Sat',562,'',null from dual union all
select 55763,'1983-006','1983-02-04 08:37:00','orbital','success',825,'N-10(F)','CS 2A','CS 2A','','Sat',1474,'',null from dual union all
select 55764,'1983-007','1983-02-06 11:31:00','orbital','success',1238,'77049  -956','Kosmos-1439','Feniks No. 956','','Sat',562,'',null from dual union all
select 55765,'1983-008','1983-02-09 13:47:00','orbital','success',170,'6001H','OPS 0252    [NOSS5]','PARCAE 5','','Sat',1775,'',null from dual union all
select 55766,'1983-009','1983-02-10 07:15:00','orbital','success',1238,'77024  -120','Kosmos-1440','Resurs-F1 17F41 No. 20','','Sat',473,'',null from dual union all
select 55767,'1983-010','1983-02-16 10:03:00','orbital','success',1510,'76053  -343','Kosmos-1441','Tselina-D','','Sat',462,'',null from dual union all
select 55768,'1983-011','1983-02-20 05:10:00','orbital','success',819,'M-3S-3','Tenma','ASTRO-B','','Sat',837,'',null from dual union all
select 55769,'1983-012','1983-02-25 12:45:00','orbital','success',1238,'76054  -248','Kosmos-1442','Oktan No. 248','','Sat',473,'',null from dual union all
select 5577,'1959-A126','1959-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55770,'1983-013','1983-03-02 09:37:08','orbital','success',949,'309-02','Kosmos-1443','TKS-M No. 16401','','Sat',553,'',null from dual union all
select 55771,'1983-014','1983-03-02 10:50:00','orbital','success',1238,'78017  -943','Kosmos-1444','Zenit-6','','Sat',473,'',null from dual union all
select 55772,'1983-015','1983-03-11 15:29:00','orbital','success',805,'78039  -641','Molniya-3','Molniya-3 No. 34','','Sat',473,'',null from dual union all
select 55773,'1983-016','1983-03-12 14:00:00','orbital','success',956,'304-02','Ekran','Ekran No. 18L','','Sat',554,'',null from dual union all
select 55774,'1983-017','1983-03-15 22:30:00','orbital','success',647,'','Kosmos-1445','BOR-4 No. 403','','Sat',622,'',null from dual union all
select 55775,'1983-018','1983-03-16 08:50:00','orbital','success',1242,'77024  -778 (001)','Kosmos-1446','Zenit-6','','Sat',480,'',null from dual union all
select 55776,'1983-019','1983-03-16 18:14:00','orbital','success',805,'76053  -625','Molniya-1','Molniya-1','','Sat',473,'',null from dual union all
select 55777,'1983-020','1983-03-23 12:45:06','orbital','success',954,'307-01','Astron','1A No. 602','','Sat',553,'',null from dual union all
select 55778,'1983-021','1983-03-24 20:55:50','orbital','success',680,'53739-548','Kosmos-1447','Nadezhda 11F643N No. 530','','Sat',457,'',null from dual union all
select 55779,'1983-022','1983-03-28 15:52:00','orbital','success',155,'73E','NOAA 8','NOAA E','','Sat',1776,'',null from dual union all
select 5578,'1959-A127','1959-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55780,'1983-023','1983-03-30 01:10:00','orbital','success',680,'65019-522','Kosmos-1448','Parus','','Sat',456,'',null from dual union all
select 55781,'1983-024','1983-03-31 10:50:00','orbital','success',1238,'77024  -770','Kosmos-1449','Zenit-6','','Sat',462,'',null from dual union all
select 55782,'1983-025','1983-04-02 02:02:00','orbital','success',805,'77024  -654','Molniya-1','Molniya-1 11F658T No. 68','','Sat',480,'',null from dual union all
select 55783,'1983-026','1983-04-04 18:30:00','orbital','success',1243,'STS-6','TDRS-A','OV-099 IUS deploy','','Sat',921,'MLP2',null from dual union all
select 55784,'1983-027','1983-04-06 12:00:00','orbital','success',680,'53719-512','Kosmos-1450','Vektor','','Sat',457,'',null from dual union all
select 55785,'1983-028','1983-04-08 04:45:00','orbital','success',956,'315-02','Raduga','Gran'' No. 23L','','Sat',554,'',null from dual union all
select 55786,'1983-029','1983-04-08 08:30:00','orbital','success',1238,'77024  -772','Kosmos-1451','Zenit-6','','Sat',475,'',null from dual union all
select 55787,'1983-030','1983-04-11 22:39:00','orbital','success',394,'653/D167','Satcom 1R','RCA-F','','Sat',234,'',null from dual union all
select 55788,'1983-031','1983-04-12 18:20:00','orbital','success',680,'65019-519','Kosmos-1452','Strela-2M','','Sat',457,'',null from dual union all
select 55789,'1983-032','1983-04-15 18:45:00','orbital','success',1403,'24B-27 (3B-63)','KH-8A','GAMBIT-3 Mission 4353','','Sat',1778,'',null from dual union all
select 5579,'1959-A128','1959-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55790,'1983-033','1983-04-17 05:46:00','orbital','success',1083,'SLV-3-D4','RS-D-2','Rohini RS-D-2','','Sat',1360,'',null from dual union all
select 55791,'1983-034','1983-04-19 12:00:00','orbital','success',680,'47119-520','Kosmos-1453','Romb','','Sat',457,'',null from dual union all
select 55792,'1983-035','1983-04-20 13:10:54','orbital','success',1238,'Sh15000-342','Soyuz T-8','Soyuz 7K-ST No. 13L','','Sat',480,'',null from dual union all
select 55793,'1983-036','1983-04-22 14:30:00','orbital','success',1238,'76054  -957','Kosmos-1454','Feniks No. 957','','Sat',473,'',null from dual union all
select 55794,'1983-037','1983-04-23 14:30:00','orbital','success',1448,'51029-807','Kosmos-1455','Tselina-D','','Sat',470,'',null from dual union all
select 55795,'1983-038','1983-04-25 19:33:43','orbital','success',803,'77049  -646','Kosmos-1456','Oko','','Sat',462,'',null from dual union all
select 55796,'1983-039','1983-04-26 10:00:00','orbital','success',1238,'78039  -214','Kosmos-1457','Oktan No. 214','','Sat',562,'',null from dual union all
select 55797,'1983-040','1983-04-28 08:30:00','orbital','success',1238,'76054  -768','Kosmos-1458','Fram','','Sat',473,'',null from dual union all
select 55798,'1983-041','1983-04-28 22:26:00','orbital','success',390,'D168','GOES 6','GOES F','','Sat',233,'',null from dual union all
select 55799,'1983-042','1983-05-06 03:00:00','orbital','success',680,'65019-513','Kosmos-1459','Parus','','Sat',457,'',null from dual union all
select 558,'1945-S34','1945-03-09 00:00:00','military missile','success',1470,'','Remagen','','','Weapon',1736,'',90 from dual union all
select 5580,'1959-A129','1959-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55800,'1983-043','1983-05-06 09:10:00','orbital','success',1238,'77024  -771','Kosmos-1460','Zenit-6','','Sat',562,'',null from dual union all
select 55801,'1983-044','1983-05-07 10:30:00','orbital','success',1446,'45099-806','Kosmos-1461','US-P','','Sat',589,'',null from dual union all
select 55802,'1983-045','1983-05-17 08:00:00','orbital','success',1238,'78055  -839','Kosmos-1462','Resurs-F1 17F41 No. 21','','Sat',473,'',null from dual union all
select 55803,'1983-046','1983-05-19 12:00:00','orbital','success',680,'47119-511','Kosmos-1463','Taifun-1B','','Sat',456,'',null from dual union all
select 55804,'1983-047','1983-05-19 22:26:00','orbital','success',180,'AC-61','Intelsat 506','INTELSAT V F6','','Sat',268,'',null from dual union all
select 55805,'1983-048','1983-05-24 02:59:00','orbital','success',680,'47139-541','Kosmos-1464','Parus','','Sat',456,'',null from dual union all
select 55806,'1983-049','1983-05-26 05:00:00','orbital','success',680,'65083-458','Kosmos-1465','Romb','','Sat',622,'',null from dual union all
select 55807,'1983-050','1983-05-26 12:00:00','orbital','success',1238,'77024  -250','Kosmos-1466','Oktan No. 250','','Sat',562,'',null from dual union all
select 55808,'1983-051','1983-05-26 15:18:00','orbital','success',390,'D169','Exosat','Exosat','','Sat',1724,'',null from dual;
insert into LAUNCH
select 55809,'1983-052','1983-05-31 11:40:00','orbital','success',1238,'77024  -775','Kosmos-1467','Zenit-6','','Sat',475,'',null from dual union all
select 5581,'1959-A130','1959-07-31 00:00:00','atmospheric rocket','success',542,'','Corvus Model','','','Test',1895,'',5 from dual union all
select 55810,'1983-053','1983-06-02 02:38:40','deep space','success',954,'321-01','Venera-15','4V-2 No. 860','','Sat',553,'',null from dual union all
select 55811,'1983-054','1983-06-07 02:32:51','deep space','success',954,'321-02','Venera-16','4V-2 No. 861','','Sat',554,'',null from dual union all
select 55812,'1983-055','1983-06-07 07:50:00','orbital','success',1238,'78012  -122','Kosmos-1468','Resurs-F1 17F41 No. 22','','Sat',473,'',null from dual union all
select 55813,'1983-056','1983-06-09 23:23:00','orbital','success',170,'6002H','OPS 6432    [NOSS6]','PARCAE 6','','Sat',1775,'',null from dual union all
select 55814,'1983-057','1983-06-14 12:15:00','orbital','success',1238,'76054  -762','Kosmos-1469','Zenit-6','','Sat',475,'',null from dual union all
select 55815,'1983-058','1983-06-16 11:59:03','orbital','success',91,'L6','ECS 1','ECS 1','','Sat',344,'',null from dual union all
select 55816,'1983-059','1983-06-18 11:33:00','orbital','success',1243,'STS-7','PAM-D Deploy/SPAS','OV-099 PAM deploy/FF','','Sat',921,'MLP1',null from dual union all
select 55817,'1983-060','1983-06-20 18:45:00','orbital','success',1406,'34D-5  (04D-3)','HEXAGON SV18','HEXAGON SV-18','','Sat',1777,'',null from dual union all
select 55818,'1983-061','1983-06-22 23:58:00','orbital','success',1448,'45074-703','Kosmos-1470','Tselina-D','','Sat',470,'',null from dual union all
select 55819,'1983-062','1983-06-27 09:12:18','orbital','success',1238,'Sh15000-349','Soyuz T-9','Soyuz 7K-ST No. 14L','','Sat',480,'',null from dual union all
select 5582,'1959-A131','1959-08-04 00:00:00','atmospheric rocket','success',861,'','Mercury model','','','Test',1821,'',10 from dual union all
select 55820,'1983-063','1983-06-27 15:37:09','orbital','success',1132,'S205C','P83-1','NNS O-16','','Sat',1779,'',null from dual union all
select 55821,'1983-064','1983-06-28 15:00:00','orbital','success',1238,'77024  -958','Kosmos-1471','Feniks No. 958','','Sat',473,'',null from dual union all
select 55822,'1983-065','1983-06-28 23:08:00','orbital','success',393,'D170','Galaxy 1','Galaxy I','','Sat',234,'',null from dual union all
select 55823,'1983-066','1983-06-30 23:56:00','orbital','success',956,'314-02','Gorizont','Gorizont No. 17L','','Sat',553,'',null from dual union all
select 55824,'1983-067','1983-07-01 12:17:00','deep space','success',807,'Ts15000-214','Prognoz-9','SO-M No. 509','','Sat',562,'',null from dual union all
select 55825,'1983-068','1983-07-05 07:50:00','orbital','success',1238,'78012  -123','Kosmos-1472','Zenit-6','','Sat',473,'',null from dual union all
select 55826,'1983-069','1983-07-06 00:31:00','orbital','success',680,'47183-459','Kosmos-1473/1480','Strela-1M','','Sat',457,'',null from dual union all
select 55827,'1983-070','1983-07-08 19:21:00','orbital','success',803,'76054  -647','Kosmos-1481','Oko','','Sat',474,'',null from dual union all
select 55828,'1983-071','1983-07-13 09:40:00','orbital','success',1238,'78012  -781','Kosmos-1482','Zenit-6','','Sat',480,'',null from dual union all
select 55829,'1983-072','1983-07-14 10:21:00','orbital','success',159,'75E','GPS 8','Navstar GPS SVN 8','','Sat',1776,'',null from dual union all
select 5583,'1959-A132','1959-08-06 00:00:00','military missile','failure',933,'AX-15','','','','Test',244,'',10 from dual union all
select 55830,'1983-073','1983-07-19 15:14:00','orbital','success',805,'78012  -660','Molniya-1','Molniya-1 11F658 No. 66','','Sat',480,'',null from dual union all
select 55831,'1983-074','1983-07-20 08:00:00','orbital','success',1238,'78012  -083','Kosmos-1483','Resurs-F1 17F41 No. 23','','Sat',475,'',null from dual union all
select 55832,'1983-075','1983-07-24 05:50:37','orbital','success',1510,'77049  -376','Kosmos-1484','Resurs-OE No. 3-2','','Sat',562,'',null from dual union all
select 55833,'1983-076','1983-07-26 12:00:00','orbital','success',1238,'76054  -769 (269)','Kosmos-1485','Zenit-6','','Sat',462,'',null from dual union all
select 55834,'1983-077','1983-07-28 22:49:00','orbital','success',393,'D171','Telstar 301','Telstar 301','','Sat',233,'',null from dual union all
select 55835,'1983-078','1983-07-31 15:41:00','orbital','success',1405,'34B-9  (3B-65)','OPS 7304','Jumpseat 7','','Sat',1778,'',null from dual union all
select 55836,'1983-079','1983-08-03 12:40:00','orbital','success',680,'53783-463','Kosmos-1486','Strela-2M','','Sat',457,'',null from dual union all
select 55837,'1983-080','1983-08-05 09:20:00','orbital','success',1238,'78012  -125','Kosmos-1487','Resurs-F1 17F41 No. 25','','Sat',475,'',null from dual union all
select 55838,'1983-081','1983-08-05 20:29:00','orbital','success',825,'N-11(F)','CS 2B','CS-2B','','Sat',1474,'',null from dual union all
select 55839,'1983-082','1983-08-09 11:20:00','orbital','success',1238,'77024  -774','Kosmos-1488','Zenit-6','','Sat',475,'',null from dual union all
select 5584,'1959-A133','1959-08-07 00:00:00','suborbital rocket','failure',89,'NASA 2.04GT','','','','Test',1821,'',20 from dual union all
select 55840,'1983-083','1983-08-10 13:00:00','orbital','success',1238,'77024  -252','Kosmos-1489','Oktan No. 252','','Sat',480,'',null from dual union all
select 55841,'1983-084','1983-08-10 18:24:26','orbital','success',958,'317-01','Kosmos-1490/1492','Uragan No. 12L','','Sat',553,'',null from dual union all
select 55842,'1983-085','1983-08-17 12:08:23','orbital','success',1238,'Ts15000-302','Progress-17','Progress 7K-TG No. 119','','Sat',480,'',null from dual union all
select 55843,'1983-086','1983-08-19 06:00:00','orbital','success',301,'Y2       (8)','FSW','FSW No. 6','','Sat',810,'',null from dual union all
select 55844,'1983-087','1983-08-23 11:05:00','orbital','success',1238,'77024  -776','Kosmos-1493','Zenit-6','','Sat',475,'',null from dual union all
select 55845,'1983-088','1983-08-25 20:02:00','orbital','success',956,'316-02','Raduga','Gran'' No. 24L','','Sat',554,'',null from dual union all
select 55846,'1983-089','1983-08-30 06:32:00','orbital','success',1243,'STS-8','PAM-D Deploy/PFTA','OV-099 PAM deploy','','Sat',921,'MLP2',null from dual union all
select 55847,'1983-090','1983-08-30 22:49:00','orbital','success',805,'76054  -650','Molniya-3','Molniya-3 No. 32','','Sat',473,'',null from dual union all
select 55848,'1983-091','1983-08-31 06:30:00','orbital','success',680,'47139-547','Kosmos-1494','Romb','','Sat',622,'',null from dual union all
select 55849,'1983-092','1983-09-03 10:15:00','orbital','success',1238,'78012  -084','Kosmos-1495','Fram','','Sat',475,'',null from dual union all
select 5585,'1959-A134','1959-08-12 15:00:00','military missile','success',256,'631-3','','','','OT',254,'',20 from dual union all
select 55850,'1983-093','1983-09-07 13:24:00','orbital','success',1238,'78012  -782','Kosmos-1496','Oktan No. 251','','Sat',462,'',null from dual union all
select 55851,'1983-094','1983-09-08 22:52:00','orbital','success',394,'D172','Satcom 2R','RCA-G','','Sat',234,'',null from dual union all
select 55852,'1983-095','1983-09-09 11:00:00','orbital','success',1238,'78012  -783','Kosmos-1497','Zenit-6','','Sat',475,'',null from dual union all
select 55853,'1983-096','1983-09-14 10:25:00','orbital','success',1238,'78039  -952','Kosmos-1498','Resurs-F1 17F41 No. 24','','Sat',473,'',null from dual union all
select 55854,'1983-097','1983-09-17 11:15:00','orbital','success',1238,'77024  -251','Kosmos-1499','Zenit-6','','Sat',473,'',null from dual union all
select 55855,'1983-098','1983-09-22 22:16:00','orbital','success',393,'D173','Galaxy 2','Galaxy II','','Sat',233,'',null from dual union all
select 55856,'1983-099','1983-09-28 07:59:00','orbital','success',1448,'51042-302','Kosmos-1500','Okean-OE NKhM No. 1','','Sat',469,'',null from dual union all
select 55857,'1983-100','1983-09-29 17:37:00','orbital','success',956,'318-01','Ekran','Ekran No. 25L','','Sat',554,'',null from dual union all
select 55858,'1983-101','1983-09-30 11:00:00','orbital','success',680,'65093-482','Kosmos-1501','Romb','','Sat',456,'',null from dual union all
select 55859,'1983-102','1983-10-05 12:00:00','orbital','success',680,'65039-540','Kosmos-1502','Taifun-1B','','Sat',456,'',null from dual union all
select 5586,'1959-A135','1959-08-12 00:00:00','suborbital rocket','failure',1327,'Terasca','UV Scanner','','','Test',1023,'',0 from dual union all
select 55860,'1983-103','1983-10-12 00:20:00','orbital','success',680,'65039-543','Kosmos-1503','Strela-2M','','Sat',456,'',null from dual union all
select 55861,'1983-104','1983-10-14 10:00:00','orbital','success',1238,'77024  -803','Kosmos-1504','Yantar''-4K2','','Sat',562,'',null from dual union all
select 55862,'1983-105','1983-10-19 00:45:36','orbital','success',91,'L7','Intelsat 507','INTELSAT V F7','','Sat',344,'',null from dual union all
select 55863,'1983-106','1983-10-20 09:59:05','orbital','success',1238,'Ts15000-287','Progress-18','Progress 7K-TG No. 118','','Sat',562,'',null from dual union all
select 55864,'1983-107','1983-10-21 12:10:00','orbital','success',1238,'78012  -784','Kosmos-1505','Zenit-6','','Sat',462,'',null from dual union all
select 55865,'1983-108','1983-10-26 17:20:00','orbital','success',680,'47122-331','Kosmos-1506','Tsikada','','Sat',456,'',null from dual union all
select 55866,'1983-109','1983-10-28 09:00:00','orbital','success',1510,'77025  -325','Meteor-2','Meteor-2 No. 10','','Sat',462,'',null from dual union all
select 55867,'1983-110','1983-10-29 08:30:00','orbital','success',1446,'45083-627','Kosmos-1507','US-PM1','','Sat',589,'',null from dual union all
select 55868,'1983-111','1983-11-11 12:30:00','orbital','success',680,'65093-470','Kosmos-1508','Vektor','','Sat',457,'',null from dual union all
select 55869,'1983-112','1983-11-17 12:15:00','orbital','success',1238,'78012  -785','Kosmos-1509','Zenit-6','','Sat',462,'',null from dual union all
select 5587,'1959-A136','1959-08-13 00:00:00','military missile','failure',976,'I01-067B','8K63 UBP-19','','','Test',636,'',1 from dual union all
select 55870,'1983-113','1983-11-18 06:32:00','orbital','success',155,'58E','DMSP','DMSP Block 5D-2 S-7','','Sat',1776,'',null from dual union all
select 55871,'1983-114','1983-11-23 16:45:00','orbital','success',805,'77023  -619','Molniya-1','Molniya-1!No. 48','','Sat',473,'',null from dual union all
select 55872,'1983-115','1983-11-24 12:33:00','orbital','success',1448,'51074-701','Kosmos-1510','Musson No. 14','','Sat',470,'',null from dual union all
select 55873,'1983-116','1983-11-28 16:00:00','orbital','success',1243,'STS-9','Spacelab 1','OV-102 Spacelab','','Sat',921,'MLP1',null from dual union all
select 55874,'1983-117','1983-11-30 13:45:00','orbital','success',1238,'77024  -249','Kosmos-1511','Oktan No. 249','','Sat',473,'',null from dual union all
select 55875,'1983-118','1983-11-30 13:51:00','orbital','success',956,'308-02','Gorizont','Gorizont No. 18L','','Sat',553,'',null from dual union all
select 55876,'1983-119','1983-12-07 12:10:00','orbital','success',1238,'78012  -786','Kosmos-1512','Zenit-6','','Sat',473,'',null from dual union all
select 55877,'1983-120','1983-12-08 06:13:00','orbital','success',680,'65019-528','Kosmos-1513','Parus','','Sat',457,'',null from dual union all
select 55878,'1983-121','1983-12-14 07:00:00','orbital','success',1238,'Yu15000-371','Kosmos-1514','Bion No. 6','','Sat',473,'',null from dual union all
select 55879,'1983-122','1983-12-15 12:25:00','orbital','success',1448,'67014-708','Kosmos-1515','Tselina-D','','Sat',470,'',null from dual union all
select 5588,'1959-A137','1959-08-14 17:56:00','military missile','failure',1419,'B-5','P13 RV','','','Test',237,'',0 from dual union all
select 55880,'1983-123','1983-12-21 06:07:59','orbital','success',805,'76054  -648','Molniya-3','Molniya-3 No. 35','','Sat',473,'',null from dual union all
select 55881,'1983-124','1983-12-27 09:30:00','orbital','success',1238,'Sh15000-345','Kosmos-1516','Siluet No. 3','','Sat',562,'',null from dual union all
select 55882,'1983-125','1983-12-27 10:00:00','orbital','success',647,'','Kosmos-1517','BOR-4 No. 405','','Sat',621,'',null from dual union all
select 55883,'1983-126','1983-12-28 03:48:05','orbital','success',803,'78039  -642','Kosmos-1518','Oko','','Sat',462,'',null from dual union all
select 55884,'1983-127','1983-12-29 00:52:24','orbital','success',958,'320-02','Kosmos-1519/1521','Uragan No. 14L','','Sat',554,'',null from dual union all
select 55885,'1983-A01','1983-01-04 03:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 55886,'1983-A02','1983-01-07 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 55887,'1983-A03','1983-01-28 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',44 from dual union all
select 55888,'1983-A04','1983-02-02 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',46 from dual union all
select 55889,'1983-A05','1983-02-04 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 5589,'1959-A138','1959-08-14 19:36:04','military missile','failure',1367,'190','IWST','','SHORT SKIP','Test',1675,'',0 from dual union all
select 55890,'1983-A06','1983-02-16 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 55891,'1983-A07','1983-02-21 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 55892,'1983-A08','1983-03-04 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 55893,'1983-A09','1983-03-07 23:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 55894,'1983-A10','1983-03-09 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 55895,'1983-A11','1983-03-16 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 55896,'1983-A12','1983-03-19 03:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 55897,'1983-A13','1983-03-25 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',41 from dual union all
select 55898,'1983-A14','1983-03-28 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 55899,'1983-A15','1983-03-30 22:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual;
insert into LAUNCH
select 559,'1945-S35','1945-03-25 20:55:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5590,'1959-A139','1959-08-18 00:00:00','atmospheric rocket','success',542,'','Corvus Model','','','Test',1895,'',5 from dual union all
select 55900,'1983-A16','1983-04-01 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',39 from dual union all
select 55901,'1983-A17','1983-04-07 03:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',48 from dual union all
select 55902,'1983-A18','1983-04-18 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 55903,'1983-A19','1983-04-19 17:00:00','military missile','failure',1442,'','DASO','','','OT',421,'SSBN 633',20 from dual union all
select 55904,'1983-A20','1983-04-27 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 55905,'1983-A21','1983-05-05 13:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',46 from dual union all
select 55906,'1983-A22','1983-05-18 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',37 from dual union all
select 55907,'1983-A23','1983-05-24 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 55908,'1983-A24','1983-06-09 15:00:00','military missile','failure',932,'A3TK','CHEVALINE DASO','','','OT',421,'UK S27',10 from dual union all
select 55909,'1983-A25','1983-06-22 10:58:41','military missile','unknown',1099,'','','','','Test',293,'B-52',10 from dual union all
select 5591,'1959-A14','1959-01-27 00:00:00','military missile','success',255,'624-XY16','','','','OT',272,'',20 from dual union all
select 55910,'1983-A26','1983-07-25 14:47:00','suborbital rocket','failure',1319,'NASA 33.32GU','','','','Aeron',1873,'',10 from dual union all
select 55911,'1983-A27','1983-07-26 11:29:54','military missile','unknown',1099,'','','','','Test',293,'B-52',10 from dual union all
select 55912,'1983-A28','1983-09-01 22:40:00','suborbital rocket','failure',1318,'NASA 12.39WT','','','','Test',1821,'',10 from dual union all
select 55913,'1983-A29','1983-09-02 00:00:00','military missile','failure',1453,'','15A35 UBP-20','','','OT',508,'',40 from dual union all
select 55914,'1983-A30','1983-09-20 11:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',48 from dual union all
select 55915,'1983-A31','1983-09-26 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 55916,'1983-A32','1983-09-28 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',42 from dual union all
select 55917,'1983-A33','1983-09-30 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',49 from dual union all
select 55918,'1983-A34','1983-10-03 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',40 from dual union all
select 55919,'1983-A35','1983-10-05 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 5592,'1959-A140','1959-08-19 00:34:00','suborbital rocket','failure',838,'NASA 03.14CA','GCA Release 2 Na','','','Aeronomy',1821,'',30 from dual union all
select 55920,'1983-A36','1983-10-07 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 55921,'1983-A37','1983-10-11 20:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',42 from dual union all
select 55922,'1983-A38','1983-10-13 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',49 from dual union all
select 55923,'1983-A39','1983-10-13 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 55924,'1983-A40','1983-10-14 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',42 from dual union all
select 55925,'1983-A41','1983-10-18 02:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 55926,'1983-A42','1983-10-19 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 55927,'1983-A43','1983-10-21 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',47 from dual union all
select 55928,'1983-A44','1983-10-24 21:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',45 from dual union all
select 55929,'1983-A45','1983-10-28 20:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',42 from dual union all
select 5593,'1959-A141','1959-08-21 15:00:00','test rocket','failure',710,'LJ-1','','','','SC Test',1797,'',1 from dual union all
select 55930,'1983-A46','1983-10-31 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',44 from dual union all
select 55931,'1983-A47','1983-11-02 21:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',42 from dual union all
select 55932,'1983-A48','1983-11-16 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',46 from dual union all
select 55933,'1983-A49','1983-12-03 02:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',38 from dual union all
select 55934,'1983-A50','1983-12-07 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',40 from dual union all
select 55935,'1983-A51','1983-12-08 16:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',48 from dual union all
select 55936,'1983-A52','1983-12-21 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',39 from dual union all
select 55937,'1983-A53','1983-12-30 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',39 from dual union all
select 55938,'1983-A56','1983-08-10 00:00:00','military missile','failure',1434,'4L','15Zh58 LKI-4','','','Test',477,'',0 from dual union all
select 55939,'1983-F01','1983-01-25 12:00:00','orbital','failure',680,'47193-480','','Romb','','Sat',457,'',60 from dual union all
select 5594,'1959-A142','1959-08-25 00:00:00','military missile','success',771,'GM-54-46','','','','Test',222,'',12 from dual union all
select 55940,'1983-F02','1983-09-26 19:37:51','orbital','failure',1238,'Yu15000-363','Soyuz T','Soyuz 7K-ST No. 16L','','Sat',480,'',1 from dual union all
select 55941,'1983-M01','1983-09-19 05:59:00','suborbital rocket','failure',885,'DLR K-OL-120','1-3-B 4x9m','','','Test',410,'',4 from dual union all
select 55942,'1983-M02','1983-11-14 10:20:00','suborbital rocket','failure',1219,'XV-18','GL A20.327-02  (IMS)','','','Ionos',1821,'',15 from dual union all
select 55943,'1983-M03','1983-12-07 18:23:00','suborbital rocket','success',860,'DLR A-NO-121','Ferdinand 71 M-T 1','','','Ionos',29,'',78 from dual union all
select 55944,'1983-M04','1983-01-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55945,'1983-M05','1983-01-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55946,'1983-M06','1983-01-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55947,'1983-M07','1983-01-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55948,'1983-M08','1983-02-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55949,'1983-M09','1983-02-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5595,'1959-A143','1959-08-25 00:00:00','military missile','success',771,'GM-54-47','','','','Test',222,'',12 from dual union all
select 55950,'1983-M10','1983-02-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55951,'1983-M11','1983-02-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55952,'1983-M12','1983-03-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55953,'1983-M13','1983-03-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55954,'1983-M14','1983-03-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55955,'1983-M15','1983-03-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55956,'1983-M16','1983-04-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55957,'1983-M17','1983-04-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55958,'1983-M18','1983-05-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55959,'1983-M19','1983-05-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5596,'1959-A144','1959-08-25 16:30:00','military missile','failure',933,'AX-18','','','','Test',244,'',10 from dual union all
select 55960,'1983-M20','1983-05-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55961,'1983-M21','1983-05-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55962,'1983-M22','1983-06-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55963,'1983-M23','1983-06-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55964,'1983-M24','1983-06-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55965,'1983-M25','1983-06-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55966,'1983-M26','1983-06-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55967,'1983-M27','1983-07-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55968,'1983-M28','1983-07-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55969,'1983-M29','1983-07-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5597,'1959-A145','1959-08-26 17:35:00','military missile','failure',873,'ZW-1/101','1','','','Test',1884,'',0 from dual union all
select 55970,'1983-M30','1983-07-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55971,'1983-M31','1983-08-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55972,'1983-M32','1983-08-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55973,'1983-M33','1983-08-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55974,'1983-M34','1983-08-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55975,'1983-M35','1983-08-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55976,'1983-M36','1983-08-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55977,'1983-M37','1983-08-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55978,'1983-M38','1983-09-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55979,'1983-M39','1983-09-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5598,'1959-A146','1959-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55980,'1983-M40','1983-09-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55981,'1983-M41','1983-09-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55982,'1983-M42','1983-09-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55983,'1983-M43','1983-09-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55984,'1983-M44','1983-09-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55985,'1983-M45','1983-10-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55986,'1983-M46','1983-10-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55987,'1983-M47','1983-11-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55988,'1983-M48','1983-11-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55989,'1983-M49','1983-11-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual;
insert into LAUNCH
select 5599,'1959-A147','1959-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 55990,'1983-M50','1983-11-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55991,'1983-M51','1983-11-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55992,'1983-M52','1983-11-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55993,'1983-M53','1983-11-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55994,'1983-M54','1983-12-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55995,'1983-M55','1983-12-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55996,'1983-M56','1983-12-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55997,'1983-M57','1983-12-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 55998,'1983-M58','1983-08-23 16:25:00','suborbital rocket','success',1284,'','ROCOZ-A 316','','','Aeron Ozone',1821,'',75 from dual union all
select 55999,'1983-M59','1983-08-23 17:00:00','suborbital rocket','success',1284,'','Datasonde','','','Aeron Ozone',1821,'',75 from dual union all
select 56,'1944-A20','1944-11-22 20:31:00','military missile','failure',1470,'19832','Ma302','','','Test',661,'',0 from dual union all
select 560,'1945-S36','1945-03-27 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5600,'1959-A148','1959-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 56000,'1983-M60','1983-09-26 17:32:00','suborbital rocket','success',1284,'','ROCOZ-A 317','','','Aeron',1821,'',75 from dual union all
select 56001,'1983-M61','1983-09-26 17:39:00','suborbital rocket','success',1284,'','ROCOZ-A 318','','','Aeron Ozone',1821,'',75 from dual union all
select 56002,'1983-M62','1983-09-26 18:17:00','suborbital rocket','success',1284,'','ROCOZ-A 319','','','Aeron Ozone',1821,'',75 from dual union all
select 56003,'1983-M63','1983-09-26 18:45:00','suborbital rocket','success',1284,'','ROCOZ-A 320','','','Aeron Ozone',1821,'',75 from dual union all
select 56004,'1983-M64','1983-10-27 16:24:00','suborbital rocket','success',1284,'','ROCOZ-A 321','','','Aeron Ozone',1821,'',75 from dual union all
select 56005,'1983-M65','1983-10-27 16:31:00','suborbital rocket','success',1284,'','ROCOZ-A 322','','','Aeron Ozone',1821,'',75 from dual union all
select 56006,'1983-M66','1983-10-27 17:20:00','suborbital rocket','success',1284,'','ROCOZ-A 323','','','Aeron Ozone',1821,'',75 from dual union all
select 56007,'1983-M67','1983-10-27 17:54:00','suborbital rocket','success',1284,'','ROCOZ-A 324','','','Aeron Ozone',1821,'',75 from dual union all
select 56008,'1983-M68','1983-11-22 16:30:00','suborbital rocket','success',1284,'','ROCOZ-A 325','','','Aeron Ozone',1821,'',75 from dual union all
select 56009,'1983-M69','1983-12-09 16:50:00','suborbital rocket','success',1284,'','ROCOZ-A 326','','','Aeron Ozone',1821,'',75 from dual union all
select 5601,'1959-A149','1959-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 56010,'1983-M70','1983-12-15 16:50:00','suborbital rocket','success',1284,'','ROCOZ-A 327','','','Aeron Ozone',1821,'',75 from dual union all
select 56011,'1983-M71','1983-06-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56012,'1983-M72','1983-06-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56013,'1983-M73','1983-07-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56014,'1983-M74','1983-07-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56015,'1983-M75','1983-07-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56016,'1983-M76','1983-07-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56017,'1983-M77','1983-08-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56018,'1983-M78','1983-08-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56019,'1983-M79','1983-08-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 5602,'1959-A15','1959-01-27 00:00:00','military missile','success',255,'624-XY11','','','','OT',278,'',20 from dual union all
select 56020,'1983-M80','1983-08-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56021,'1983-M81','1983-08-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56022,'1983-M82','1983-09-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56023,'1983-M83','1983-09-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56024,'1983-M84','1983-09-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56025,'1983-M85','1983-10-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56026,'1983-M86','1983-10-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56027,'1983-M87','1983-11-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56028,'1983-M88','1983-11-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56029,'1983-M89','1983-11-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 5603,'1959-A150','1959-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 56030,'1983-M90','1983-11-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56031,'1983-M91','1983-12-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56032,'1983-M92','1983-12-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',1467,'',60 from dual union all
select 56033,'1983-S01','1983-03-17 11:40:00','military missile','success',914,'67083-626(8L)','15A11 UBP-1','GCh 15B99','','Test',540,'',1000 from dual union all
select 56034,'1983-S02','1983-01-01 00:00:00','military missile','success',756,'','','','','Test',206,'S655',800 from dual union all
select 56035,'1983-S03','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 56036,'1983-S04','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 56037,'1983-S05','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 56038,'1983-S06','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 56039,'1983-S07','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 5604,'1959-A151','1959-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 56040,'1983-S08','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 56041,'1983-S09','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 56042,'1983-S10','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 56043,'1983-S100','1983-05-14 06:55:00','suborbital rocket','success',211,'NASA 27.67UH','CAL XR 14','','','XR Astron',1895,'',246 from dual union all
select 56044,'1983-S101','1983-05-15 15:46:00','military missile','success',932,'A3TK','CHEVALINE DASO','','','OT',421,'UK S27',1000 from dual union all
select 56045,'1983-S102','1983-05-16 15:30:00','military missile','success',932,'A3TK','CHEVALINE DASO','','','OT',421,'UK S27',1000 from dual union all
select 56046,'1983-S103','1983-05-27 14:48:00','military missile','success',919,'ED-11','P-445','','','Test',231,'',180 from dual union all
select 56047,'1983-S104','1983-05-28 00:00:00','military missile','success',795,'','HOE Target','RSLP STREP-12 (HOE)','','Target',1698,'',1300 from dual union all
select 56048,'1983-S105','1983-05-28 00:00:00','military missile','success',558,'HOE 2','HOE 2','HOE 2','','ABM Test',868,'',200 from dual union all
select 56049,'1983-S106','1983-10-25 00:00:00','military missile','success',1434,'45524-403-5L','15Zh58 LKI-5','','','Test',477,'',1000 from dual union all
select 5605,'1959-A152','1959-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 56050,'1983-S107','1983-06-01 00:00:00','military missile','success',1179,'VMF','LKI','','','Test',989,'',1000 from dual union all
select 56051,'1983-S108','1983-06-01 00:00:00','suborbital rocket','success',1219,'XV-16','Manicore','','','Ionos?',330,'',599 from dual union all
select 56052,'1983-S109','1983-06-02 14:46:00','military missile','success',919,'ED-14','P-446','','','Test',231,'',180 from dual union all
select 56053,'1983-S11','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 56054,'1983-S110','1983-06-09 16:05:00','military missile','success',932,'A3TK','CHEVALINE DASO','','','OT',421,'UK S27',1000 from dual union all
select 56055,'1983-S111','1983-06-11 08:10:00','suborbital rocket','success',228,'ADD-VC-56','OASIS','','','Aeron',1873,'',180 from dual union all
select 56056,'1983-S112','1983-06-11 15:00:00','military missile','success',932,'A3TK','CHEVALINE DASO','','','OT',421,'UK S27',1000 from dual union all
select 56057,'1983-S113','1983-06-11 16:43:00','military missile','success',932,'A3TK','CHEVALINE DASO','','','OT',421,'UK S27',1000 from dual union all
select 56058,'1983-S114','1983-06-16 06:51:00','suborbital rocket','success',846,'GL A11.074','STATE','','','Aeron',1047,'',134 from dual union all
select 56059,'1983-S115','1983-06-18 02:10:00','military missile','success',907,'FTM-1','Mk 12A RV x 6','','','Test',1726,'',1000 from dual union all
select 5606,'1959-A153','1959-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 56060,'1983-S116','1983-06-17 09:00:00','suborbital rocket','success',211,'NASA 27.72NP','SPAR 10','','','Micrograv',1895,'',212 from dual union all
select 56061,'1983-S117','1983-06-19 15:12:00','military missile','success',918,'ED-13','P-447','','','Test',433,'',180 from dual union all
select 56062,'1983-S118','1983-06-21 20:15:00','suborbital rocket','success',1319,'NASA 33.33UL','','','','Aeron',1873,'',200 from dual union all
select 56063,'1983-S119','1983-06-24 15:31:00','military missile','success',918,'ED','P-448','','','Test',433,'',180 from dual union all
select 56064,'1983-S12','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 56065,'1983-S120','1983-06-25 14:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT95GB','GLORY TRIP 95GB','OT',1712,'',1300 from dual union all
select 56066,'1983-S121','1983-06-25 11:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT94GM','GLORY TRIP 94GM','OT',1705,'',1300 from dual union all
select 56067,'1983-S122','1983-06-25 00:00:00','military missile','success',1106,'2606','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 56068,'1983-S123','1983-06-29 04:35:00','suborbital rocket','success',211,'NASA 27.27UH','','','','XR Astron',1895,'',289 from dual union all
select 56069,'1983-S124','1983-07-12 00:00:00','military missile','success',756,'','','','','Test',307,'S655',800 from dual union all
select 5607,'1959-A154','1959-09-02 00:00:00','military missile','success',256,'631-4','','','','OT',272,'',20 from dual union all
select 56070,'1983-S125','1983-07-14 13:20:00','military missile','success',1004,'51026-325-02','15A14 UBP-33 Osnashch.','GCh 15F143U','','OT',497,'',1000 from dual union all
select 56071,'1983-S126','1983-07-15 08:49:00','suborbital rocket','success',1319,'NASA 33.34GE','GSFC Release','','','Plasma',1821,'',220 from dual union all
select 56072,'1983-S127','1983-07-16 15:13:00','military missile','success',918,'ED','P-449','','','Test',433,'',365 from dual union all
select 56073,'1983-S128','1983-07-20 08:54:00','suborbital rocket','success',1319,'NASA 33.35GE','GSFC Release','','','Plasma',1821,'',210 from dual union all
select 56074,'1983-S129','1983-07-23 19:15:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 645',500 from dual union all
select 56075,'1983-S13','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 56076,'1983-S130','1983-07-25 13:15:00','suborbital rocket','success',211,'NASA 27.81US','','','','Solar',1895,'',318 from dual union all
select 56077,'1983-S131','1983-07-27 14:15:00','military missile','failure',919,'ED-17','P-450','','','Test',231,'',90 from dual union all
select 56078,'1983-S132','1983-07-28 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 622',500 from dual;
insert into LAUNCH
select 56079,'1983-S133','1983-07-28 00:00:00','military missile','failure',939,'','FOT','','','OT',421,'SSBN 622',500 from dual union all
select 5608,'1959-A155','1959-09-02 00:00:00','atmospheric rocket','success',542,'','Corvus Model','','','Test',1821,'',5 from dual union all
select 56080,'1983-S134','1983-07-28 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 622',500 from dual union all
select 56081,'1983-S135','1983-07-28 05:22:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 622',500 from dual union all
select 56082,'1983-S136','1983-08-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56083,'1983-S137','1983-08-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56084,'1983-S138','1983-08-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56085,'1983-S139','1983-08-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56086,'1983-S14','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 56087,'1983-S140','1983-08-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56088,'1983-S141','1983-08-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56089,'1983-S142','1983-08-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 5609,'1959-A156','1959-09-02 00:00:00','military missile','failure',976,'ID3-19Sh','8K63 ShPU Test 1','','','Test',629,'',50 from dual union all
select 56090,'1983-S143','1983-08-06 00:00:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 641',1000 from dual union all
select 56091,'1983-S144','1983-08-06 00:38:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 641',1000 from dual union all
select 56092,'1983-S145','1983-08-08 06:00:00','suborbital rocket','success',211,'NASA 27.61UE','','','','Plasma',1895,'',239 from dual union all
select 56093,'1983-S146','1983-08-08 06:00:00','suborbital rocket','success',227,'','CHARGE-1 (TPE-3)','','','Tether/Ionos',1895,'',218 from dual union all
select 56094,'1983-S147','1983-08-16 17:21:00','suborbital rocket','success',211,'NASA 27.79UL','EUV Abs Cal','','','Aeron/Solar EUV',1895,'',314 from dual union all
select 56095,'1983-S148','1983-08-18 23:00:00','suborbital rocket','success',1300,'TT-500A-13F','','','','Micrograv',1476,'',320 from dual union all
select 56096,'1983-S149','1983-08-21 18:00:00','military missile','success',1442,'','DASO','','','OT',421,'SSBN 728',1000 from dual union all
select 56097,'1983-S15','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 56098,'1983-S150','1983-08-29 10:00:00','suborbital rocket','success',1074,'S-520-6','MINIX','','','Plasma physics',833,'',238 from dual union all
select 56099,'1983-S151','1983-08-29 13:20:00','suborbital rocket','success',30,'NASA 04.340UG','JHU Airglow 13 (HUT)','','','Airglow',1869,'',200 from dual union all
select 561,'1945-S37','1945-03-27 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5610,'1959-A157','1959-09-04 14:23:00','suborbital spaceplane','failure',1520,'2-A-2','','','','Test',379,'NB-52 003',14 from dual union all
select 56100,'1983-S152','1983-08-30 15:00:00','suborbital rocket','success',860,'NASA 31.35UU','CWAS 5','','','Aeronomy',1895,'',140 from dual union all
select 56101,'1983-S153','1983-09-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56102,'1983-S154','1983-09-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56103,'1983-S155','1983-09-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56104,'1983-S156','1983-09-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56105,'1983-S157','1983-09-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56106,'1983-S158','1983-09-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56107,'1983-S159','1983-09-02 16:00:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 616',500 from dual union all
select 56108,'1983-S16','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 56109,'1983-S161','1983-09-07 15:00:00','suborbital rocket','success',860,'NASA 31.40UU','CWAS 6','','','Aeronomy',1895,'',140 from dual union all
select 5611,'1959-A158','1959-09-09 00:00:00','atmospheric rocket','success',551,'','Mercury model','','','Reentry Test',1821,'',10 from dual union all
select 56110,'1983-S162','1983-09-07 17:04:00','military missile','success',919,'ED-18','P-451','','','Test',231,'',320 from dual union all
select 56111,'1983-S163','1983-09-12 22:00:00','suborbital rocket','success',1073,'S-310-13','','','','Ionos',839,'',227 from dual union all
select 56112,'1983-S164','1983-09-16 18:16:00','suborbital rocket','success',1073,'S-310-14','','','','Aeron',839,'',218 from dual union all
select 56113,'1983-S165','1983-09-18 15:00:00','military missile','success',918,'ED-15/20','P-452','','','Test',433,'',317 from dual union all
select 56114,'1983-S166','1983-09-20 16:51:00','suborbital rocket','success',211,'NASA 27.56GS','SERTS-1','','','Solar EUV',1895,'',308 from dual union all
select 56115,'1983-S167','1983-09-21 00:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT96GM','GLORY TRIP 96GM','OT',1703,'',1300 from dual union all
select 56116,'1983-S168','1983-09-21 13:55:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT97GM','GLORY TRIP 97GM','OT',1705,'',1300 from dual union all
select 56117,'1983-S169','1983-09-22 21:45:00','suborbital rocket','success',30,'NASA 04.341UE','UM Airglow Pay.','','','Magnetospheric',1869,'',200 from dual union all
select 56118,'1983-S17','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 56119,'1983-S170','1983-09-23 12:05:00','military missile','unknown',917,'','P-453 83 Cape 01','','','OT',231,'',180 from dual union all
select 5612,'1959-A159','1959-09-15 00:00:00','military missile','success',793,'','Silo Test','','','Test',381,'',0 from dual union all
select 56120,'1983-S171','1983-09-23 13:21:00','military missile','unknown',917,'','P-454 83 Cape 02','','','OT',231,'',180 from dual union all
select 56121,'1983-S172','1983-09-23 14:34:00','military missile','unknown',917,'','P-455 83 Cape 03','','','OT',231,'',180 from dual union all
select 56122,'1983-S173','1983-09-23 14:55:00','military missile','unknown',917,'','P-456 83 Cape 04','','','OT',231,'',180 from dual union all
select 56123,'1983-S174','1983-09-23 16:17:00','military missile','unknown',917,'','P-457 83 Cape 05','','','OT',231,'',180 from dual union all
select 56124,'1983-S175','1983-09-29 02:00:00','military missile','success',796,'','Mk 11 RV','FOT GT145MS','GLORY TRIP 145MS','OT',1699,'',1300 from dual union all
select 56125,'1983-S176','1983-10-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56126,'1983-S177','1983-10-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56127,'1983-S178','1983-10-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56128,'1983-S179','1983-10-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56129,'1983-S18','1983-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 5613,'1959-A16','1959-01-30 20:40:00','military missile','success',843,'','Cold weather test','','','Test',428,'',10 from dual union all
select 56130,'1983-S180','1983-10-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56131,'1983-S181','1983-10-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56132,'1983-S182','1983-10-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',132,'VISE',150 from dual union all
select 56133,'1983-S183','1983-10-03 18:34:00','military missile','success',1442,'','DASO','','','OT',421,'SSBN 728',1000 from dual union all
select 56134,'1983-S184','1983-10-04 00:00:00','military missile','success',1106,'2604','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 56135,'1983-S185','1983-10-07 08:05:00','suborbital rocket','success',210,'NASA 27.69UG','FUV Echelle','','','UV Astron',1895,'',258 from dual union all
select 56136,'1983-S186','1983-10-07 10:30:00','suborbital rocket','success',210,'NASA 27.80UL','','','','Aeron',1895,'',311 from dual union all
select 56137,'1983-S187','1983-10-11 04:50:00','military missile','success',1004,'45046-315-10','15A14 UBP-35 Osn.','GCh 15F143U','','OT',491,'',1000 from dual union all
select 56138,'1983-S188','1983-10-13 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 623',500 from dual union all
select 56139,'1983-S189','1983-10-13 06:41:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 623',500 from dual union all
select 5614,'1959-A160','1959-09-17 16:08:48','suborbital spaceplane','success',1520,'2-1-3','','','','Test',379,'NB-52 003',15 from dual union all
select 56140,'1983-S19','1983-10-20 00:00:00','test rocket','unknown',1497,'25','K65UP No. 25','','','Ionos?',621,'',1500 from dual union all
select 56141,'1983-S190','1983-10-13 16:25:00','military missile','unknown',917,'','P-458 83 Cape 06','','','OT',231,'',180 from dual union all
select 56142,'1983-S191','1983-10-13 16:50:00','military missile','unknown',917,'','P-459 83 Cape 07','','','OT',231,'',180 from dual union all
select 56143,'1983-S192','1983-10-13 17:13:00','military missile','unknown',917,'','P-460 83 Cape 08','','','OT',231,'',180 from dual union all
select 56144,'1983-S193','1983-10-13 17:43:00','military missile','unknown',917,'','P-461 83 Cape 09','','','OT',231,'',180 from dual union all
select 56145,'1983-S194','1983-10-14 16:55:00','military missile','success',907,'FTM-2','Mk 12A RV x 6','','','Test',1726,'',1000 from dual union all
select 56146,'1983-S195','1983-10-20 04:25:00','suborbital rocket','success',1014,'','Vertikal''-11','VZAF','','Solar UV/Solar X',636,'',500 from dual union all
select 56147,'1983-S196','1983-10-25 11:26:00','suborbital rocket','success',117,'GL A24.260','ELC-I (IR limb)','','','IR Astron',1873,'',282 from dual union all
select 56148,'1983-S197','1983-10-31 22:35:00','suborbital rocket','failure',1219,'XV-17','GL A20.327-01  (IMS)','','','Ionos',1821,'',25 from dual union all
select 56149,'1983-S198','1983-11-07 00:00:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 632',1000 from dual union all
select 5615,'1959-A161','1959-09-21 20:00:00','atmospheric rocket','success',835,'Smoke 002','','','','Aeronomy',1821,'',12 from dual union all
select 56150,'1983-S199','1983-11-07 18:02:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 632',1000 from dual union all
select 56151,'1983-S20','1983-01-01 00:00:00','military missile','success',1548,'','','','','OT',1011,'PLBR',620 from dual union all
select 56152,'1983-S200','1983-11-09 05:15:00','suborbital rocket','success',867,'GL A08.708-2','IMS Release (SF6)','','','Plasma release',1821,'',348 from dual union all
select 56153,'1983-S201','1983-11-11 21:08:39','suborbital rocket','success',1337,'NASA 29.22GE/IE','Ferdinand 63/Pol.Leap','','','Auroral',29,'',454 from dual union all
select 56154,'1983-S202','1983-11-17 10:41:00','suborbital rocket','success',860,'NASA 31.36GE','','','','Plasma',1821,'',140 from dual union all
select 56155,'1983-S203','1983-11-19 10:41:00','suborbital rocket','success',860,'NASA 31.37GE','VLF Trigg Elec','','','Plasma',1821,'',140 from dual union all
select 56156,'1983-S204','1983-11-19 10:46:00','suborbital rocket','success',860,'NASA 31.39GE','VLF Trigg Elec','','','Plasma',1821,'',140 from dual union all
select 56157,'1983-S205','1983-11-22 00:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT98GB','GLORY TRIP 98GB','OT',1712,'',1300 from dual union all
select 56158,'1983-S206','1983-11-23 00:00:00','military missile','success',1039,'48712-101-6L','15Zh44 LKI-6','','','Test',477,'',1000 from dual union all
select 56159,'1983-S207','1983-11-23 00:00:00','military missile','success',756,'13','','','','Test',307,'S655',800 from dual union all
select 5616,'1959-A162','1959-09-22 00:00:00','military missile','success',771,'GM-54-48','','','','Test',222,'',12 from dual union all
select 56160,'1983-S208','1983-11-27 23:15:00','suborbital rocket','success',117,'NASA 24.15UG','','','','UV Astron',1873,'',300 from dual union all
select 56161,'1983-S209','1983-12-07 18:00:00','suborbital rocket','success',211,'NASA 27.77UU','','','','Aeron',1895,'',357 from dual union all
select 56162,'1983-S21','1983-02-08 12:10:00','military missile','success',1006,'45042-613-08T','15A18 UBP-14','GCh 15F183','','OT',489,'',1000 from dual union all
select 56163,'1983-S210','1983-12-14 19:57:00','suborbital rocket','success',1318,'NASA 12.40WT','','','','Test/Ionos',1821,'',700 from dual union all
select 56164,'1983-S211','1983-12-15 00:00:00','military missile','success',795,'','HOE Target','RSLP STREP-12A (HOE)','','Target',1698,'',1300 from dual union all
select 56165,'1983-S212','1983-12-15 00:00:00','military missile','success',558,'HOE 3','HOE 3','HOE 3','','ABM Test',868,'',200 from dual union all
select 56166,'1983-S213','1983-12-16 16:00:06','suborbital rocket','success',1304,'','HAVE SLED II','','','Target?',1047,'',500 from dual union all
select 56167,'1983-S214','1983-12-20 17:00:00','military missile','success',907,'FTM-3','Mk 12A RV x 8','','','Test',1726,'',1000 from dual union all
select 56168,'1983-S215','1983-12-06 00:00:00','military missile','success',1442,'','WTR to Wake','','','OT',1897,'SSBN UNK',1000 from dual;
insert into LAUNCH
select 56169,'1983-S216','1983-06-27 00:00:00','military missile','success',1057,'12  95/','4 RV','','','Test',989,'',1000 from dual union all
select 5617,'1959-A163','1959-09-22 00:00:00','military missile','success',771,'GM-54-43','','','','Test',222,'',12 from dual union all
select 56170,'1983-S217','1983-03-09 02:25:00','suborbital rocket','failure',1284,'TU1-0381 PWN-12A','CONDOR Sphere','','','Aeron',322,'',80 from dual union all
select 56171,'1983-S218','1983-03-09 05:20:00','suborbital rocket','failure',1284,'TU1-0383 PWN-12A','CONDOR Sphere','','','Aeron',322,'',80 from dual union all
select 56172,'1983-S219','1983-03-09 05:49:00','suborbital rocket','failure',1284,'TU1-0383 PWN-12A','CONDOR Sphere','','','Aeron',322,'',80 from dual union all
select 56173,'1983-S22','1983-03-17 12:38:00','military missile','success',1006,'67014-327-07','15A18 UBP-15','GCh 15F183','','OT',485,'',1000 from dual union all
select 56174,'1983-S220','1983-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56175,'1983-S221','1983-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56176,'1983-S222','1983-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56177,'1983-S223','1983-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56178,'1983-S224','1983-05-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56179,'1983-S225','1983-05-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 5618,'1959-A164','1959-09-22 00:00:00','test rocket','failure',575,'','High Virgo','','','Interceptor',21,'B-58',12 from dual union all
select 56180,'1983-S226','1983-05-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56181,'1983-S227','1983-05-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56182,'1983-S228','1983-05-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56183,'1983-S229','1983-05-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56184,'1983-S23','1983-07-13 10:25:00','military missile','success',1006,'45029-738-05T','15A18 UBP-16 K','GCh 15F183','','OT',485,'',1000 from dual union all
select 56185,'1983-S230','1983-05-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56186,'1983-S231','1983-08-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56187,'1983-S232','1983-08-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56188,'1983-S233','1983-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56189,'1983-S234','1983-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 5619,'1959-A165','1959-09-25 00:00:00','military missile','success',1214,'SM-62A N-3415','','','','OT',224,'',10 from dual union all
select 56190,'1983-S235','1983-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56191,'1983-S236','1983-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56192,'1983-S237','1983-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56193,'1983-S238','1983-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56194,'1983-S239','1983-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56195,'1983-S24','1983-10-26 04:50:00','military missile','success',1006,'45029-738-03','15A18 UPB-17 K','GCh 15F183','','OT',485,'',1000 from dual union all
select 56196,'1983-S240','1983-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56197,'1983-S241','1983-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56198,'1983-S242','1983-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56199,'1983-S243','1983-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 562,'1945-S38','1945-01-03 06:45:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5620,'1959-A166','1959-09-28 17:06:00','military missile','failure',933,'AX-14','','','','Test',244,'',10 from dual union all
select 56200,'1983-S244','1983-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56201,'1983-S245','1983-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56202,'1983-S246','1983-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56203,'1983-S247','1983-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56204,'1983-S248','1983-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56205,'1983-S249','1983-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56206,'1983-S250','1983-11-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56207,'1983-S251','1983-12-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56208,'1983-S252','1983-12-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56209,'1983-S253','1983-12-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 5621,'1959-A167','1959-10-02 00:00:00','military missile','success',268,'2/4','','','','Test',1752,'',17 from dual union all
select 56210,'1983-S254','1983-12-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56211,'1983-S255','1983-12-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56212,'1983-S256','1983-12-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 56213,'1983-S258','1983-11-08 17:21:00','military missile','unknown',917,'AO-217','P-462 83 Fall 01','','','OT',433,'',180 from dual union all
select 56214,'1983-S259','1983-11-08 17:45:00','military missile','unknown',917,'AO-218','P-463 83 Fall 02','','','OT',433,'',180 from dual union all
select 56215,'1983-S260','1983-11-16 00:00:00','military missile','unknown',917,'AO-219','P-464 83 Fall 03','','','OT',433,'',180 from dual union all
select 56216,'1983-S261','1983-11-18 00:00:00','military missile','unknown',917,'AO-220','P-465 83 Fall 04','','','OT',433,'',180 from dual union all
select 56217,'1983-S262','1983-01-01 00:00:00','military missile','success',984,'Aerofon','UGCh','ZLKI 1','','Test',636,'',86 from dual union all
select 56218,'1983-S263','1983-01-01 00:00:00','military missile','success',984,'Aerofon','UGCh','ZLKI 2','','Test',636,'',86 from dual union all
select 56219,'1983-S265','1983-05-31 00:00:00','military missile','success',1434,'45579-207-3L','15Zh58 LKI-3','','','Test',477,'',1000 from dual union all
select 5622,'1959-A168','1959-10-02 00:00:00','military missile','success',793,'','Silo Test','','','Test',381,'',0 from dual union all
select 56220,'1983-S266','1983-01-28 00:00:00','military missile','success',1452,'49242-83-426','15A30 60 RD UBP-10','','','OT',1461,'',1000 from dual union all
select 56221,'1983-S267','1983-01-28 00:00:00','military missile','success',1453,'49242-83-425','15A35 60 RD UBP-1','','','OT',1461,'',1000 from dual union all
select 56222,'1983-S268','1983-01-01 00:00:00','military missile','success',1450,'','15A20 54 RD UBP-8','','','OT',1468,'',1000 from dual union all
select 56223,'1983-S269','1983-01-01 00:00:00','military missile','success',1450,'','15A20 54 RD UBP-9','','','OT',1468,'',1000 from dual union all
select 56224,'1983-S270','1983-03-03 00:00:00','military missile','success',1449,'','8K84 10 RD UBP-9','','','OT',897,'',1000 from dual union all
select 56225,'1983-S271','1983-03-03 00:00:00','military missile','success',1449,'','8K84 10 RD UBP-10','','','OT',897,'',1000 from dual union all
select 56226,'1983-S272','1983-10-04 00:00:00','military missile','success',745,'','15A16 10 RD UBP-2','','','OT',897,'',1000 from dual union all
select 56227,'1983-S273','1983-07-19 00:00:00','military missile','success',1004,'','15A14 UBP 38 RD','','','OT',363,'',1000 from dual union all
select 56228,'1983-S274','1983-01-12 12:41:00','suborbital rocket','success',1284,'INTA SLD-8301','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 56229,'1983-S275','1983-01-21 13:27:00','suborbital rocket','success',1284,'INTA SLD-8302','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 5623,'1959-A169','1959-10-02 18:11:00','military missile','failure',933,'AX-20','','','','Test',244,'',2 from dual union all
select 56230,'1983-S276','1983-01-28 12:30:00','suborbital rocket','success',1284,'INTA SLD-8303','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 56231,'1983-S277','1983-01-30 12:26:00','suborbital rocket','success',1284,'INTA SLD-8304','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 56232,'1983-S278','1983-02-01 12:31:00','suborbital rocket','success',1284,'INTA SLD-8305','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 56233,'1983-S279','1983-02-04 12:28:00','suborbital rocket','success',1284,'INTA SLD-8306','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 56234,'1983-S280','1983-02-09 12:34:00','suborbital rocket','success',1284,'INTA SLD-8307','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 56235,'1983-S281','1983-02-16 12:02:00','suborbital rocket','success',1284,'INTA SLD-8308','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 56236,'1983-S282','1983-02-17 13:21:00','suborbital rocket','success',1284,'INTA SLD-8309','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 56237,'1983-S283','1983-02-19 12:15:00','suborbital rocket','success',1284,'INTA SLD-8310','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 56238,'1983-S284','1983-09-09 08:31:00','suborbital rocket','success',1284,'INTA SLD-8311','GLOBUS','','','Aeron',396,'',100 from dual union all
select 56239,'1983-S285','1983-09-10 11:03:00','suborbital rocket','success',1284,'INTA SLD-8312','GLOBUS','','','Aeron',396,'',100 from dual union all
select 5624,'1959-A17','1959-02-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 56240,'1983-S286','1983-09-14 03:21:00','suborbital rocket','success',1284,'INTA SLD-8313','GLOBUS','','','Aeron',396,'',100 from dual union all
select 56241,'1983-S287','1983-09-14 11:05:00','suborbital rocket','success',1284,'INTA SLD-8314','GLOBUS','','','Aeron',396,'',100 from dual union all
select 56242,'1983-S288','1983-09-20 12:43:00','suborbital rocket','success',1284,'INTA SLD-8315','GLOBUS','','','Aeron',396,'',100 from dual union all
select 56243,'1983-S289','1983-09-24 11:00:00','suborbital rocket','success',1284,'INTA SLD-8316','GLOBUS','','','Aeron',396,'',100 from dual union all
select 56244,'1983-S29','1983-03-15 00:00:00','military missile','failure',1039,'3L','15Zh44 LKI-3','','','Test',477,'',0 from dual union all
select 56245,'1983-S290','1983-12-13 12:23:00','suborbital rocket','success',1284,'INTA SLD-8317','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 56246,'1983-S291','1983-12-25 00:00:00','military missile','success',1039,'48722-103-8L','15Zh44 LKI-7','','','Test',477,'',1000 from dual union all
select 56247,'1983-S292','1983-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-3','Duga-2','','Test',368,'',500 from dual union all
select 56248,'1983-S293','1983-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-4','Duga-2','','Test',368,'',500 from dual union all
select 56249,'1983-S294','1983-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-5','Duga-2','','Test',368,'',500 from dual union all
select 5625,'1959-A170','1959-10-08 20:00:00','suborbital rocket','success',716,'','','','','Aeron',1895,'',44 from dual union all
select 56250,'1983-S295','1983-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-6','Duga-2','','Test',368,'',500 from dual union all
select 56251,'1983-S296','1983-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-7','Duga-2','','Test',368,'',500 from dual union all
select 56252,'1983-S297','1983-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-8','Duga-2','','Test',368,'',500 from dual union all
select 56253,'1983-S298','1983-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-9','Duga-2','','Test',368,'',500 from dual union all
select 56254,'1983-S299','1983-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-10','Duga-2','','Test',368,'',500 from dual union all
select 56255,'1983-S30','1983-09-06 00:00:00','military missile','failure',1039,'5L','15Zh44 LKI-5','','','Test',477,'',0 from dual union all
select 56256,'1983-S300','1983-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-11','Duga-2','','Test',368,'',500 from dual union all
select 56257,'1983-S301','1983-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-12','Duga-2','','Test',368,'',500 from dual union all
select 56258,'1983-S302','1983-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-13','Duga-2','','Test',368,'',500 from dual;
insert into LAUNCH
select 56259,'1983-S303','1983-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-14','Duga-2','','Test',368,'',500 from dual union all
select 5626,'1959-A171','1959-10-09 00:00:00','military missile','failure',976,'B2-3','8K63 UBP-27','','','Test',636,'',50 from dual union all
select 56260,'1983-S304','1983-03-14 00:00:00','military missile','success',922,'','15Zh45 35 RD UBP','','','Test',636,'',500 from dual union all
select 56261,'1983-S305','1983-03-21 00:00:00','military missile','success',922,'','15Zh45 35 RD UBP','','','Test',636,'',500 from dual union all
select 56262,'1983-S306','1983-07-01 00:00:00','military missile','success',923,'','15Zh53 29 RD UBP','','','Test',636,'',500 from dual union all
select 56263,'1983-S307','1983-08-01 00:00:00','military missile','success',923,'','15Zh53 29 RD UBP','','','Test',636,'',500 from dual union all
select 56264,'1983-S308','1983-11-30 00:00:00','military missile','success',923,'','15Zh53 32 RD UBP','','','Test',636,'',500 from dual union all
select 56265,'1983-S309','1983-01-01 00:00:00','suborbital rocket','success',1025,'','IPP-10 propellant test','','','Test',1467,'',150 from dual union all
select 56266,'1983-S31','1983-02-02 00:00:00','military missile','success',1453,'49282-71-393T(5P)','15A35 UBP-17','','','OT',505,'',1000 from dual union all
select 56267,'1983-S32','1983-03-04 00:00:00','military missile','success',1455,'63084-82-027(2L)','15A35S LKI-2','GCh 15F357','','OT',505,'',1000 from dual union all
select 56268,'1983-S33','1983-03-23 13:00:00','military missile','success',1453,'51165-52-247(6P)','15A35 UBP-18','','','OT',535,'',1000 from dual union all
select 56269,'1983-S34','1983-05-12 00:00:00','military missile','success',1455,'51139-82-022T(3L)','15A35S LKI-3','GCh 15F357','','OT',508,'',1000 from dual union all
select 5627,'1959-A172','1959-10-10 00:00:00','suborbital spaceplane','failure',1520,'2-A-4','','','','Test',379,'NB-52 003',14 from dual union all
select 56270,'1983-S35','1983-05-13 00:00:00','military missile','success',1455,'63024-82-023T(4L)','15A35S LKI-4','GCh 15F357','','Test',508,'',1000 from dual union all
select 56271,'1983-S36','1983-05-27 00:00:00','military missile','success',1455,'63024-82-024T(5L)','15A35S LKI-5','GCh 15F357','','Test',508,'',1000 from dual union all
select 56272,'1983-S37','1983-07-13 00:00:00','military missile','success',1453,'63085-53-250','15A35 UBP-19','','','Test',505,'',1000 from dual union all
select 56273,'1983-S38','1983-08-26 00:00:00','military missile','success',1455,'-      2-026T(6L)','15A35S LKI-6','GCh 15F357','','Test',505,'',1000 from dual union all
select 56274,'1983-S39','1983-12-06 00:00:00','military missile','success',1453,'','15A35 UBP-21','','','Test',508,'',1000 from dual union all
select 56275,'1983-S40','1983-12-16 00:00:00','military missile','success',1455,'(7L)','15A35S LKI-7','GCh 15F357','','Test',505,'',1000 from dual union all
select 56276,'1983-S41','1983-01-07 00:00:00','military missile','success',795,'','Mk 12A LBRV-2','RSLP LBRV-2','','Reentry Test',1698,'',1300 from dual union all
select 56277,'1983-S42','1983-01-10 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 624',500 from dual union all
select 56278,'1983-S43','1983-01-10 09:19:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 624',500 from dual union all
select 56279,'1983-S44','1983-01-12 19:00:00','suborbital rocket','success',1319,'NASA 33.30UU','','','','Aeron',1873,'',200 from dual union all
select 5628,'1959-A173','1959-10-12 18:18:00','military missile','failure',928,'A1X-2','','','','Test',251,'',10 from dual union all
select 56280,'1983-S45','1983-01-13 17:31:00','suborbital rocket','success',891,'NASA 30.15UU','UM NO Profile','','','Aeron',1821,'',70 from dual union all
select 56281,'1983-S46','1983-01-13 18:39:00','suborbital rocket','success',891,'NASA 30.18UU','UM NO Profile','','','Aeron',1821,'',75 from dual union all
select 56282,'1983-S47','1983-01-14 20:40:00','suborbital rocket','success',668,'K-9M-76','','','','Aeron/Ionos',839,'',349 from dual union all
select 56283,'1983-S48','1983-01-19 23:12:38','suborbital rocket','success',224,'SSC S36','AURELD-HIGH','','','Auroral',412,'',494 from dual union all
select 56284,'1983-S49','1983-01-21 14:48:00','military missile','success',919,'ED-10','P-437','','','Test',231,'',314 from dual union all
select 56285,'1983-S50','1983-01-22 13:02:10','suborbital rocket','success',1319,'NASA 33.23CE','','','','Plasma',1047,'',200 from dual union all
select 56286,'1983-S51','1983-01-26 23:00:00','suborbital rocket','success',1300,'TT-500A-12F','','','','Micrograv',1476,'',320 from dual union all
select 56287,'1983-S52','1983-02-07 00:00:00','military missile','success',795,'','HOE Target','RSLP STREP-11 (HOE)','','Reentry Test',1701,'',1300 from dual union all
select 56288,'1983-S53','1983-02-07 00:00:00','military missile','success',558,'HOE 1','HOE 1','HOE 1','','ABM Test',868,'',200 from dual union all
select 56289,'1983-S54','1983-02-07 00:00:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 640',1000 from dual union all
select 5629,'1959-A174','1959-10-14 00:00:00','suborbital spaceplane','failure',1520,'2-A-5','','','','Test',379,'NB-52 003',14 from dual union all
select 56290,'1983-S55','1983-02-07 02:14:00','suborbital rocket','success',224,'AAF-IXC-2','WATERHOLE 3','','','Plasma',428,'',333 from dual union all
select 56291,'1983-S56','1983-02-07 13:42:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 640',1000 from dual union all
select 56292,'1983-S57','1983-02-18 00:00:00','military missile','success',1434,'45519-205-1L','15Zh58 LKI-1','','','Test',477,'',1000 from dual union all
select 56293,'1983-S58','1983-02-09 15:06:00','military missile','success',919,'ED-2','P-438','','','Test',231,'',342 from dual union all
select 56294,'1983-S59','1983-02-12 07:39:15','suborbital rocket','success',1319,'NASA 33.31UL','','','','Aeron',1047,'',200 from dual union all
select 56295,'1983-S60','1983-02-17 19:40:00','military missile','success',1004,'45046-315-03','15A14 UBP-32','GCh 15F143U1','','OT',493,'',1000 from dual union all
select 56296,'1983-S61','1983-02-24 15:57:00','military missile','failure',799,'','Mk 12/12A RV x 3','FOT GT92GB','GLORY TRIP 92GB','OT',1712,'',100 from dual union all
select 56297,'1983-S62','1983-02-27 16:33:00','suborbital rocket','success',860,'NASA 31.28UE','CONDOR UIUC Ionos','','','Plasma',322,'',206 from dual union all
select 56298,'1983-S63','1983-02-26 16:55:00','military missile','success',918,'ED-5','P-439','','','Test',433,'',209 from dual union all
select 56299,'1983-S64','1983-03-02 02:17:00','suborbital rocket','success',133,'GL A31.200','Sensor Ejection Sys','','','Tech',1895,'',197 from dual union all
select 563,'1945-S39','1945-01-04 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5630,'1959-A175','1959-10-17 18:13:07','suborbital spaceplane','success',1520,'2-2-6','','','','Test',379,'NB-52 003',18 from dual union all
select 56300,'1983-S65','1983-03-02 02:51:11','suborbital rocket','success',1337,'NASA 29.20UE','CONDOR Cornell ESF','','','Plasma',322,'',587 from dual union all
select 56301,'1983-S66','1983-03-03 03:54:00','suborbital rocket','success',1319,'NASA 33.26UE','CONDOR Fields','','','Plasma',322,'',115 from dual union all
select 56302,'1983-S67','1983-03-07 16:00:00','suborbital rocket','success',228,'NASA 21.84DS','HRTS 4','','','Solar UV',1873,'',200 from dual union all
select 56303,'1983-S68','1983-03-08 16:35:00','suborbital rocket','success',860,'NASA 31.26GE','CONDOR GSFC-PSU','','','Plasma',322,'',108 from dual union all
select 56304,'1983-S69','1983-03-09 03:06:00','suborbital rocket','success',860,'NASA 31.27GE','CONDOR GSFC-PSU','','','Plasma',322,'',108 from dual union all
select 56305,'1983-S70','1983-03-09 03:27:00','suborbital rocket','success',860,'NASA 31.32GE','CONDOR GSFC E','','','Plasma',322,'',140 from dual union all
select 56306,'1983-S71','1983-03-09 08:57:00','suborbital rocket','success',860,'NASA 31.33GE','CONDOR GSFC E','','','Plasma',322,'',140 from dual union all
select 56307,'1983-S72','1983-03-11 00:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT93GM','GLORY TRIP 93GM','OT',1699,'',1300 from dual union all
select 56308,'1983-S73','1983-03-12 15:35:00','suborbital rocket','success',1319,'NASA 33.27UE','CONDOR Fields','','','Plasma',322,'',135 from dual union all
select 56309,'1983-S74','1983-03-12 17:09:00','suborbital rocket','success',860,'NASA 31.29UE','CONDOR UIUC Ionos','','','Plasma',322,'',205 from dual union all
select 5631,'1959-A176','1959-10-22 00:00:00','atmospheric rocket','success',542,'','Corvus Model','','','Test',1895,'',5 from dual union all
select 56310,'1983-S75','1983-03-13 16:35:00','military missile','success',918,'ED-4','P-440 (1-stage)','','','Test',433,'',316 from dual union all
select 56311,'1983-S76','1983-03-15 02:33:19','suborbital rocket','success',1337,'NASA 29.19UE','CONDOR Cornell ESF','','','Plasma',322,'',535 from dual union all
select 56312,'1983-S77','1983-03-16 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT144MS','GLORY TRIP 144MS','OT',1703,'',1300 from dual union all
select 56313,'1983-S78','1983-03-19 06:29:45','suborbital rocket','success',1303,'GL A51.971','ELIAS','','','Aeron/Auroral',1047,'',305 from dual union all
select 56314,'1983-S79','1983-03-21 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 635',500 from dual union all
select 56315,'1983-S80','1983-03-21 02:54:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 635',500 from dual union all
select 56316,'1983-S81','1983-03-21 09:32:00','suborbital rocket','success',1320,'NASA 34.10UE','CONDOR/Star ofCondor','','','Plasma',322,'',555 from dual union all
select 56317,'1983-S82','1983-03-23 09:49:00','suborbital rocket','success',1320,'NASA 34.09UE','CONDOR/Star of Lima','','','Plasma',322,'',425 from dual union all
select 56318,'1983-S83','1983-03-28 15:29:00','military missile','success',919,'ED-9','P-441','','','Test',231,'',330 from dual union all
select 56319,'1983-S84','1983-08-30 11:35:00','military missile','success',1004,'45046-315-04','15A14 UBP-34 Osn.','GCh 15F143U','','OT',493,'',1000 from dual union all
select 5632,'1959-A177','1959-10-22 00:00:00','military missile','success',793,'','Silo Test','','','Test',381,'',0 from dual union all
select 56320,'1983-S85','1983-03-30 06:59:51','suborbital rocket','success',224,'NASA 36.02UE','ECHO VI','','','Plasma',1047,'',191 from dual union all
select 56321,'1983-S86','1983-04-08 19:00:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 642',500 from dual union all
select 56322,'1983-S87','1983-04-10 07:47:00','suborbital rocket','success',231,'NASA 35.005UG','UMD ISM Helium','','','Particle Astron',1821,'',667 from dual union all
select 56323,'1983-S88','1983-04-10 19:55:00','military missile','success',919,'ED-8','P-442','','','Test',231,'',180 from dual union all
select 56324,'1983-S89','1983-04-13 09:06:23','suborbital rocket','success',1162,'GL A30.276','FWIF VI','','','Auroral',1047,'',140 from dual union all
select 56325,'1983-S90','1983-04-19 20:00:00','suborbital rocket','success',37,'GL A04.902','UV','','','Ionos/Solar EUV',1895,'',204 from dual union all
select 56326,'1983-S91','1983-04-24 15:13:00','military missile','success',918,'ED-6','P-443','','','Test',433,'',180 from dual union all
select 56327,'1983-S92','1983-04-27 05:30:00','suborbital rocket','success',891,'DLR K-O-117','DFVLR Strafam 1/3','','','Aeron',417,'',58 from dual union all
select 56328,'1983-S93','1983-05-26 00:00:00','military missile','failure',1039,'4L','15Zh44 LKI-4','','','Test',477,'',0 from dual union all
select 56329,'1983-S94','1983-05-05 00:00:00','military missile','success',1434,'45524-401-2L','15Zh58 LKI-2','','','Test',477,'',1000 from dual union all
select 5633,'1959-A178','1959-10-22 00:00:00','atmospheric rocket','failure',705,'L2B','','','','Test',2,'',0 from dual union all
select 56330,'1983-S95','1983-05-05 00:00:00','military missile','success',795,'','STREP-22','RSLP STREP-22','','Reentry Test',1698,'',1300 from dual union all
select 56331,'1983-S96','1983-05-05 05:05:00','suborbital rocket','success',1212,'DLR K-GR-118','DLR TEXUS 7','','','Micrograv',416,'',227 from dual union all
select 56332,'1983-S97','1983-05-06 18:12:00','military missile','success',918,'ED-7','P-444','','','Test',433,'',316 from dual union all
select 56333,'1983-S98','1983-05-12 05:46:56','suborbital rocket','success',210,'AAF-VIIIB-8','SABRE','','','Auroral/Ionos/Release',428,'',350 from dual union all
select 56334,'1983-S99','1983-05-13 11:37:00','suborbital rocket','success',1212,'DLR K-GR-119','DLR TEXUS 8','','','Micrograv',416,'',264 from dual union all
select 56335,'1983-W01','1983-01-05 08:27:00','suborbital rocket','unknown',1284,'T 1-0470','','','','Test/Aeron',1821,'',78 from dual union all
select 56336,'1983-W02','1983-01-05 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 56337,'1983-W03','1983-01-05 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1753,'VOKV',52 from dual union all
select 56338,'1983-W04','1983-01-05 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 56339,'1983-W05','1983-01-05 21:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',56 from dual union all
select 5634,'1959-A179','1959-10-22 00:00:00','suborbital spaceplane','failure',1520,'2-A-7','','','','Test',379,'NB-52 003',14 from dual union all
select 56340,'1983-W06','1983-01-06 00:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',59 from dual union all
select 56341,'1983-W07','1983-01-12 02:00:00','suborbital rocket','success',761,'MT135P-R481','','','','Meteo',1332,'',55 from dual union all
select 56342,'1983-W08','1983-01-12 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',53 from dual union all
select 56343,'1983-W09','1983-01-13 19:02:59','suborbital rocket','success',1284,'Orion support','Arcasonde','','','Meteo',1821,'',67 from dual union all
select 56344,'1983-W10','1983-01-14 08:22:00','suborbital rocket','unknown',716,'T 1-0471','','','','Test/Aeron',1821,'',66 from dual union all
select 56345,'1983-W100','1983-03-29 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 56346,'1983-W101','1983-03-30 19:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',53 from dual union all
select 56347,'1983-W102','1983-03-30 19:25:00','suborbital rocket','unknown',716,'T 1-0506','','','','Test/Aeron',1821,'',55 from dual union all
select 56348,'1983-W103','1983-03-31 20:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual;
insert into LAUNCH
select 56349,'1983-W104','1983-04-04 17:06:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',62 from dual union all
select 5635,'1959-A18','1959-02-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 56350,'1983-W105','1983-04-04 18:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 56351,'1983-W106','1983-04-06 02:00:00','suborbital rocket','success',761,'MT135P-R490','','','','Meteo',1332,'',57 from dual union all
select 56352,'1983-W107','1983-04-06 08:20:00','suborbital rocket','unknown',716,'T 1-0490','','','','Test/Aeron',1821,'',50 from dual union all
select 56353,'1983-W108','1983-04-06 08:41:00','suborbital rocket','unknown',716,'T 1-0529','','','','Test/Aeron',1821,'',58 from dual union all
select 56354,'1983-W109','1983-04-06 17:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',54 from dual union all
select 56355,'1983-W11','1983-01-15 03:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 56356,'1983-W110','1983-04-08 17:15:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',66 from dual union all
select 56357,'1983-W111','1983-04-08 17:46:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',64 from dual union all
select 56358,'1983-W112','1983-04-08 18:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',68 from dual union all
select 56359,'1983-W113','1983-04-09 13:02:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 5636,'1959-A180','1959-10-26 17:00:00','suborbital rocket','failure',839,'AA6.188C','','','','Ionos',428,'',0 from dual union all
select 56360,'1983-W114','1983-04-09 15:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 56361,'1983-W115','1983-04-09 16:43:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 56362,'1983-W116','1983-04-09 16:53:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 56363,'1983-W117','1983-04-09 17:59:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 56364,'1983-W118','1983-04-09 18:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 56365,'1983-W119','1983-04-09 18:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 56366,'1983-W12','1983-01-17 19:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',67 from dual union all
select 56367,'1983-W120','1983-04-13 02:00:00','suborbital rocket','success',761,'MT135P-R491','','','','Meteo',1332,'',56 from dual union all
select 56368,'1983-W121','1983-04-13 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 56369,'1983-W122','1983-04-13 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 5637,'1959-A181','1959-10-27 00:00:00','military missile','success',771,'GM-54-45','','','','Test',222,'',12 from dual union all
select 56370,'1983-W123','1983-04-15 19:35:00','suborbital rocket','unknown',716,'T 1-0166','','','','Test/Aeron',1821,'',63 from dual union all
select 56371,'1983-W124','1983-04-18 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',52 from dual union all
select 56372,'1983-W125','1983-04-20 02:00:00','suborbital rocket','success',761,'MT135P-R492','','','','Meteo',1332,'',59 from dual union all
select 56373,'1983-W126','1983-04-20 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 56374,'1983-W127','1983-04-20 21:45:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 56375,'1983-W128','1983-04-20 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 56376,'1983-W129','1983-04-24 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 56377,'1983-W13','1983-01-18 02:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',77 from dual union all
select 56378,'1983-W130','1983-04-25 20:35:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 56379,'1983-W131','1983-04-27 02:00:00','suborbital rocket','success',761,'MT135P-R493','','','','Meteo',1332,'',57 from dual union all
select 5638,'1959-A182','1959-10-27 00:00:00','military missile','success',771,'GM-54-49','','','','Test',222,'',12 from dual union all
select 56380,'1983-W132','1983-04-28 15:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',56 from dual union all
select 56381,'1983-W133','1983-05-02 19:35:00','suborbital rocket','unknown',716,'T 1-0498','','','','Test/Aeron',1821,'',63 from dual union all
select 56382,'1983-W134','1983-05-04 07:10:00','suborbital rocket','unknown',738,'','','','','Aeron',979,'MUSSON',60 from dual union all
select 56383,'1983-W135','1983-05-04 15:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',58 from dual union all
select 56384,'1983-W136','1983-05-04 16:25:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',58 from dual union all
select 56385,'1983-W137','1983-05-04 20:52:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',70 from dual union all
select 56386,'1983-W138','1983-05-05 12:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',69 from dual union all
select 56387,'1983-W139','1983-05-05 17:47:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 56388,'1983-W14','1983-01-19 02:00:00','suborbital rocket','success',761,'MT135P-R482','','','','Meteo',1332,'',54 from dual union all
select 56389,'1983-W140','1983-05-05 21:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 5639,'1959-A183','1959-10-27 00:00:00','military missile','failure',976,'B2-9','8K63 UBP-30','','','Test',636,'',1 from dual union all
select 56390,'1983-W141','1983-05-06 07:20:00','suborbital rocket','unknown',738,'','','','','Aeron',979,'MUSSON',60 from dual union all
select 56391,'1983-W142','1983-05-06 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',54 from dual union all
select 56392,'1983-W143','1983-05-09 20:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 56393,'1983-W144','1983-05-10 12:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 56394,'1983-W145','1983-05-10 19:30:00','suborbital rocket','unknown',716,'T 1-0525','','','','Test/Aeron',1821,'',57 from dual union all
select 56395,'1983-W146','1983-05-11 02:00:00','suborbital rocket','success',761,'MT135P-R494','','','','Meteo',1332,'',55 from dual union all
select 56396,'1983-W147','1983-05-11 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',979,'MUSSON',60 from dual union all
select 56397,'1983-W148','1983-05-11 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',71 from dual union all
select 56398,'1983-W149','1983-05-11 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 56399,'1983-W15','1983-01-19 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 564,'1945-S40','1945-01-04 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5640,'1959-A184','1959-10-28 14:00:00','military missile','success',256,'631-5','','','','OT',272,'',20 from dual union all
select 56400,'1983-W150','1983-05-11 15:14:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',58 from dual union all
select 56401,'1983-W151','1983-05-11 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 56402,'1983-W152','1983-05-13 00:25:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 56403,'1983-W153','1983-05-13 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',979,'MUSSON',59 from dual union all
select 56404,'1983-W154','1983-05-13 20:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 56405,'1983-W155','1983-05-13 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 56406,'1983-W156','1983-05-16 08:10:00','suborbital rocket','unknown',738,'','','','','Aeron',979,'MUSSON',60 from dual union all
select 56407,'1983-W157','1983-05-16 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 56408,'1983-W158','1983-05-17 08:20:00','suborbital rocket','unknown',716,'T 1-0527','','','','Test/Aeron',1821,'',63 from dual union all
select 56409,'1983-W159','1983-05-18 02:00:00','suborbital rocket','success',761,'MT135P-R495','','','','Meteo',1332,'',57 from dual union all
select 5641,'1959-A185','1959-10-29 00:00:00','military missile','success',764,'56-2884','','','','Test',243,'',12 from dual union all
select 56410,'1983-W16','1983-01-19 23:23:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 56411,'1983-W160','1983-05-18 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',979,'MUSSON',59 from dual union all
select 56412,'1983-W161','1983-05-18 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 56413,'1983-W162','1983-05-20 07:40:00','suborbital rocket','unknown',738,'','','','','Aeron',979,'MUSSON',60 from dual union all
select 56414,'1983-W163','1983-05-20 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 56415,'1983-W164','1983-05-24 14:30:00','suborbital rocket','unknown',716,'T 1-0541','','','','Test/Aeron',1821,'',66 from dual union all
select 56416,'1983-W165','1983-05-25 05:40:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',58 from dual union all
select 56417,'1983-W166','1983-05-25 07:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',77 from dual union all
select 56418,'1983-W167','1983-05-25 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',979,'MUSSON',63 from dual union all
select 56419,'1983-W168','1983-05-25 14:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',53 from dual union all
select 5642,'1959-A186','1959-10-31 00:00:00','suborbital spaceplane','failure',1520,'2-A-8','','','','Test',379,'NB-52 003',14 from dual union all
select 56420,'1983-W169','1983-05-25 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 56421,'1983-W17','1983-01-21 08:25:00','suborbital rocket','unknown',716,'T 1-0472','','','','Test/Aeron',1821,'',61 from dual union all
select 56422,'1983-W170','1983-05-25 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 56423,'1983-W171','1983-05-26 02:00:00','suborbital rocket','success',761,'MT135P-R496','','','','Meteo',1332,'',55 from dual union all
select 56424,'1983-W172','1983-05-26 20:13:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 56425,'1983-W173','1983-05-27 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',979,'MUSSON',60 from dual union all
select 56426,'1983-W174','1983-06-01 02:15:00','suborbital rocket','success',761,'MT135P-R497','','','','Meteo',1332,'',57 from dual union all
select 56427,'1983-W175','1983-06-01 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 56428,'1983-W176','1983-06-01 19:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 56429,'1983-W177','1983-06-02 08:25:00','suborbital rocket','unknown',716,'T 1-0530','','','','Test/Aeron',1821,'',60 from dual union all
select 5643,'1959-A187','1959-11-04 14:30:00','test rocket','success',708,'LJ-1A','','','','SC Test',1797,'',14 from dual union all
select 56430,'1983-W178','1983-06-07 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 56431,'1983-W179','1983-06-08 17:15:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',56 from dual union all
select 56432,'1983-W18','1983-01-21 16:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 56433,'1983-W180','1983-06-08 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',50 from dual union all
select 56434,'1983-W181','1983-06-08 18:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',59 from dual union all
select 56435,'1983-W182','1983-06-08 20:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual union all
select 56436,'1983-W183','1983-06-09 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 56437,'1983-W184','1983-06-10 08:15:00','suborbital rocket','unknown',716,'T 1-0532','','','','Test/Aeron',1821,'',62 from dual union all
select 56438,'1983-W185','1983-06-13 19:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',51 from dual;
insert into LAUNCH
select 56439,'1983-W186','1983-06-13 19:15:00','suborbital rocket','unknown',716,'T 1-0550','','','','Test/Aeron',1821,'',56 from dual union all
select 5644,'1959-A188','1959-11-05 17:39:28','suborbital spaceplane','success',1520,'2-3-9','','','','Test',379,'NB-52 003',13 from dual union all
select 56440,'1983-W187','1983-06-15 02:03:59','suborbital rocket','success',761,'MT135P-R498','','','','Meteo',1332,'',59 from dual union all
select 56441,'1983-W188','1983-06-15 15:31:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 56442,'1983-W189','1983-06-17 14:15:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 56443,'1983-W19','1983-01-25 02:57:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 56444,'1983-W190','1983-06-17 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',72 from dual union all
select 56445,'1983-W191','1983-06-19 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 56446,'1983-W192','1983-06-20 19:20:00','suborbital rocket','unknown',716,'T 1-0534','','','','Test/Aeron',1821,'',61 from dual union all
select 56447,'1983-W193','1983-06-22 00:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',71 from dual union all
select 56448,'1983-W194','1983-06-22 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 56449,'1983-W195','1983-06-24 07:52:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',67 from dual union all
select 5645,'1959-A189','1959-11-07 00:00:00','military missile','success',1214,'SM-62A N-3417','','','','OT',238,'',10 from dual union all
select 56450,'1983-W196','1983-06-24 09:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 56451,'1983-W197','1983-06-24 14:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 56452,'1983-W198','1983-06-24 14:37:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 56453,'1983-W199','1983-06-24 14:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 56454,'1983-W20','1983-01-25 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',77 from dual union all
select 56455,'1983-W200','1983-06-24 15:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',55 from dual union all
select 56456,'1983-W201','1983-06-27 08:15:00','suborbital rocket','unknown',716,'T 1-0536','','','','Test/Aeron',1821,'',60 from dual union all
select 56457,'1983-W202','1983-06-28 19:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',59 from dual union all
select 56458,'1983-W203','1983-06-28 19:38:00','suborbital rocket','unknown',716,'T 1-0538','','','','Test/Aeron',1821,'',62 from dual union all
select 56459,'1983-W204','1983-06-29 02:04:59','suborbital rocket','success',761,'MT135P-R499','','','','Meteo',1332,'',57 from dual union all
select 5646,'1959-A19','1959-02-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 56460,'1983-W205','1983-06-29 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',68 from dual union all
select 56461,'1983-W206','1983-06-30 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 56462,'1983-W207','1983-07-01 00:01:00','suborbital rocket','unknown',716,'T 1-0642','','','','Test/Aeron',1821,'',65 from dual union all
select 56463,'1983-W208','1983-07-05 17:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',55 from dual union all
select 56464,'1983-W209','1983-07-05 21:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 56465,'1983-W21','1983-01-26 02:18:00','suborbital rocket','success',761,'MT135P-R483','','','','Meteo',1332,'',57 from dual union all
select 56466,'1983-W210','1983-07-07 19:00:00','suborbital rocket','unknown',716,'T 1-0644','','','','Test/Aeron',1821,'',64 from dual union all
select 56467,'1983-W211','1983-07-13 08:15:00','suborbital rocket','unknown',716,'T 1-0646','','','','Test/Aeron',1821,'',60 from dual union all
select 56468,'1983-W212','1983-07-13 16:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',58 from dual union all
select 56469,'1983-W213','1983-07-14 02:00:00','suborbital rocket','success',761,'MT135P-R500','','','','Meteo',1332,'',56 from dual union all
select 5647,'1959-A190','1959-11-10 00:00:00','military missile','success',771,'GM-54-52','','','','Test',222,'',12 from dual union all
select 56470,'1983-W214','1983-07-15 19:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 56471,'1983-W215','1983-07-15 19:38:00','suborbital rocket','unknown',716,'T 1-0648','','','','Test/Aeron',1821,'',63 from dual union all
select 56472,'1983-W216','1983-07-16 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',67 from dual union all
select 56473,'1983-W217','1983-07-18 21:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 56474,'1983-W218','1983-07-19 20:51:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 56475,'1983-W219','1983-07-21 08:29:00','suborbital rocket','unknown',716,'T 1-0661','','','','Test/Aeron',1821,'',65 from dual union all
select 56476,'1983-W22','1983-01-26 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',79 from dual union all
select 56477,'1983-W220','1983-07-25 16:58:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',76 from dual union all
select 56478,'1983-W221','1983-07-25 18:01:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 56479,'1983-W222','1983-07-27 02:00:00','suborbital rocket','success',761,'MT135P-R501','','','','Meteo',1332,'',59 from dual union all
select 5648,'1959-A191','1959-11-19 03:00:00','sounding rocket','failure',1267,'OB11.02','Ion density','','','Test/Ionos',1821,'',32 from dual union all
select 56480,'1983-W223','1983-07-27 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 56481,'1983-W224','1983-07-29 08:15:00','suborbital rocket','unknown',716,'T 1-0654','','','','Test/Aeron',1821,'',64 from dual union all
select 56482,'1983-W225','1983-07-29 12:50:00','suborbital rocket','unknown',716,'T 1-0656','','','','Test/Aeron',1821,'',63 from dual union all
select 56483,'1983-W226','1983-08-02 19:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 56484,'1983-W227','1983-08-03 07:50:00','suborbital rocket','unknown',738,'','','','','Aeron',980,'MUSSON',57 from dual union all
select 56485,'1983-W228','1983-08-03 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 56486,'1983-W229','1983-08-04 21:36:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',64 from dual union all
select 56487,'1983-W23','1983-01-26 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 56488,'1983-W230','1983-08-05 07:40:00','suborbital rocket','unknown',738,'','','','','Aeron',980,'MUSSON',59 from dual union all
select 56489,'1983-W231','1983-08-08 18:25:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',76 from dual union all
select 5649,'1959-A192','1959-11-22 18:35:00','suborbital rocket','failure',42,'AA10.186C','UM ionos probe','','','Ionos',428,'',35 from dual union all
select 56490,'1983-W232','1983-08-10 02:00:00','suborbital rocket','success',761,'MT135P-R502','','','','Meteo',1332,'',55 from dual union all
select 56491,'1983-W233','1983-08-10 07:20:00','suborbital rocket','unknown',738,'','','','','Aeron',980,'MUSSON',57 from dual union all
select 56492,'1983-W234','1983-08-10 12:30:00','suborbital rocket','unknown',716,'T 1-0658','','','','Test/Aeron',1821,'',52 from dual union all
select 56493,'1983-W235','1983-08-12 00:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',72 from dual union all
select 56494,'1983-W236','1983-08-12 07:40:00','suborbital rocket','unknown',738,'','','','','Aeron',980,'MUSSON',56 from dual union all
select 56495,'1983-W237','1983-08-12 15:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 56496,'1983-W238','1983-08-16 17:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 56497,'1983-W239','1983-08-18 08:03:00','suborbital rocket','unknown',738,'','','','','Aeron',980,'MUSSON',59 from dual union all
select 56498,'1983-W24','1983-01-26 17:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',58 from dual union all
select 56499,'1983-W240','1983-08-23 00:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 565,'1945-S41','1945-01-04 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5650,'1959-A193','1959-11-24 00:00:00','military missile','success',771,'GM-54-50','','','','Test',222,'',12 from dual union all
select 56500,'1983-W241','1983-08-24 07:06:00','suborbital rocket','unknown',738,'','','','','Aeron',980,'MUSSON',58 from dual union all
select 56501,'1983-W242','1983-08-24 16:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 56502,'1983-W243','1983-08-25 02:30:00','suborbital rocket','success',761,'MT135P-R503','','','','Meteo',1332,'',55 from dual union all
select 56503,'1983-W244','1983-08-26 07:23:00','suborbital rocket','unknown',738,'','','','','Aeron',980,'MUSSON',58 from dual union all
select 56504,'1983-W245','1983-08-29 07:04:00','suborbital rocket','unknown',738,'','','','','Aeron',980,'MUSSON',58 from dual union all
select 56505,'1983-W246','1983-08-30 10:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 56506,'1983-W247','1983-08-30 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 56507,'1983-W248','1983-09-01 09:32:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 56508,'1983-W249','1983-09-01 12:45:00','suborbital rocket','unknown',716,'T 1-0666','','','','Test/Aeron',1821,'',62 from dual union all
select 56509,'1983-W25','1983-02-02 23:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 5651,'1959-A194','1959-11-24 00:00:00','military missile','success',771,'GM-54-51','','','','Test',222,'',12 from dual union all
select 56510,'1983-W250','1983-09-02 18:20:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 56511,'1983-W251','1983-09-03 03:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 56512,'1983-W252','1983-09-03 09:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 56513,'1983-W253','1983-09-03 20:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1754,'VOKV',61 from dual union all
select 56514,'1983-W254','1983-09-05 06:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 56515,'1983-W255','1983-09-05 07:57:00','suborbital rocket','unknown',1284,'T 1-0664','','','','Test/Aeron',1821,'',76 from dual union all
select 56516,'1983-W256','1983-09-05 08:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 56517,'1983-W257','1983-09-05 08:55:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 56518,'1983-W258','1983-09-05 09:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 56519,'1983-W259','1983-09-06 15:53:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',62 from dual union all
select 5652,'1959-A195','1959-11-24 00:00:00','military missile','success',771,'GM-54-44','','','','Test',222,'',12 from dual union all
select 56520,'1983-W26','1983-02-03 02:00:00','suborbital rocket','success',761,'MT135P-R484','','','','Meteo',1332,'',58 from dual union all
select 56521,'1983-W260','1983-09-06 21:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 56522,'1983-W261','1983-09-07 02:15:00','suborbital rocket','success',761,'MT135P-R504','','','','Meteo',1332,'',56 from dual union all
select 56523,'1983-W262','1983-09-07 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 56524,'1983-W263','1983-09-07 23:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',67 from dual union all
select 56525,'1983-W264','1983-09-12 19:22:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 56526,'1983-W265','1983-09-12 21:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 56527,'1983-W266','1983-09-14 02:00:00','suborbital rocket','success',761,'MT135P-R505','','','','Meteo',1332,'',50 from dual union all
select 56528,'1983-W267','1983-09-14 15:40:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual;
insert into LAUNCH
select 56529,'1983-W268','1983-09-14 17:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 5653,'1959-A196','1959-11-25 00:00:00','atmospheric rocket','success',552,'','F131 Heat Transfer','','','Reentry Test',1821,'',26 from dual union all
select 56530,'1983-W269','1983-09-14 23:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',63 from dual union all
select 56531,'1983-W27','1983-02-07 08:17:00','suborbital rocket','unknown',716,'T 1-0482','','','','Test/Aeron',1821,'',64 from dual union all
select 56532,'1983-W270','1983-09-15 12:45:00','suborbital rocket','unknown',716,'T 1-0668','','','','Test/Aeron',1821,'',60 from dual union all
select 56533,'1983-W271','1983-09-16 20:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',55 from dual union all
select 56534,'1983-W272','1983-09-18 13:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 56535,'1983-W273','1983-09-19 19:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',51 from dual union all
select 56536,'1983-W274','1983-09-19 19:14:00','suborbital rocket','unknown',716,'T 1-0670','','','','Test/Aeron',1821,'',55 from dual union all
select 56537,'1983-W275','1983-09-20 12:45:00','suborbital rocket','unknown',716,'T 1-0672','','','','Test/Aeron',1821,'',52 from dual union all
select 56538,'1983-W276','1983-09-20 21:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',65 from dual union all
select 56539,'1983-W277','1983-09-21 00:15:00','suborbital rocket','unknown',1284,'T 1-0674','','','','Test/Aeron',1821,'',75 from dual union all
select 5654,'1959-A197','1959-11-25 00:00:00','atmospheric rocket','success',839,'','Paraglider test','','','Test',1821,'',45 from dual union all
select 56540,'1983-W278','1983-09-21 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 56541,'1983-W279','1983-09-21 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 56542,'1983-W28','1983-02-09 07:10:00','suborbital rocket','unknown',738,'','','','','Aeron',911,'KRENKEL''',59 from dual union all
select 56543,'1983-W280','1983-09-22 20:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 56544,'1983-W281','1983-09-26 15:20:00','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',72 from dual union all
select 56545,'1983-W282','1983-09-27 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',76 from dual union all
select 56546,'1983-W283','1983-09-28 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',78 from dual union all
select 56547,'1983-W284','1983-09-28 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 56548,'1983-W285','1983-09-29 15:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 56549,'1983-W286','1983-10-04 23:55:00','suborbital rocket','unknown',716,'T 1-0712','','','','Test/Aeron',1821,'',58 from dual union all
select 5655,'1959-A198','1959-12-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 56550,'1983-W287','1983-10-05 15:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',52 from dual union all
select 56551,'1983-W288','1983-10-05 17:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 56552,'1983-W289','1983-10-06 02:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 56553,'1983-W29','1983-02-09 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 56554,'1983-W290','1983-10-12 02:00:00','suborbital rocket','success',761,'MT135P-R507','','','','Meteo',1332,'',51 from dual union all
select 56555,'1983-W291','1983-10-12 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 56556,'1983-W292','1983-10-14 00:00:00','suborbital rocket','unknown',716,'T 1-0714','','','','Test/Aeron',1821,'',62 from dual union all
select 56557,'1983-W293','1983-10-17 21:49:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',67 from dual union all
select 56558,'1983-W294','1983-10-19 02:00:00','suborbital rocket','success',761,'MT135P-R508','','','','Meteo',1332,'',52 from dual union all
select 56559,'1983-W295','1983-10-19 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 5656,'1959-A199','1959-12-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 56560,'1983-W296','1983-10-20 17:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 56561,'1983-W297','1983-10-21 18:33:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',56 from dual union all
select 56562,'1983-W298','1983-10-25 20:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 56563,'1983-W299','1983-10-26 02:00:00','suborbital rocket','success',761,'MT135P-R509','','','','Meteo',1332,'',57 from dual union all
select 56564,'1983-W30','1983-02-09 22:46:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 56565,'1983-W300','1983-10-26 17:25:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 56566,'1983-W301','1983-10-27 15:25:59','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 56567,'1983-W302','1983-10-28 00:12:00','suborbital rocket','unknown',716,'T 1-0791','','','','Test/Aeron',1821,'',54 from dual union all
select 56568,'1983-W303','1983-10-28 19:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',65 from dual union all
select 56569,'1983-W304','1983-10-31 12:58:00','suborbital rocket','unknown',716,'T 1-0718','','','','Test/Aeron',1821,'',56 from dual union all
select 5657,'1959-A20','1959-02-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 56570,'1983-W305','1983-11-01 21:12:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',63 from dual union all
select 56571,'1983-W306','1983-11-02 02:00:00','suborbital rocket','success',761,'MT135P-R510','','','','Meteo',1332,'',56 from dual union all
select 56572,'1983-W307','1983-11-02 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',66 from dual union all
select 56573,'1983-W308','1983-11-04 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 56574,'1983-W309','1983-11-07 19:23:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 56575,'1983-W31','1983-02-10 18:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 56576,'1983-W310','1983-11-08 17:01:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',63 from dual union all
select 56577,'1983-W311','1983-11-08 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 56578,'1983-W312','1983-11-09 18:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',64 from dual union all
select 56579,'1983-W313','1983-11-09 20:45:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',54 from dual union all
select 5658,'1959-A200','1959-12-02 05:29:00','military missile','failure',1367,'265','CTL','','HARD RIGHT','OT',1673,'',0 from dual union all
select 56580,'1983-W314','1983-11-09 20:58:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 56581,'1983-W315','1983-11-10 23:50:00','suborbital rocket','unknown',716,'T 1-0779','','','','Aeron',1821,'',60 from dual union all
select 56582,'1983-W316','1983-11-14 12:45:00','suborbital rocket','unknown',716,'T 1-0781','','','','Aeron',1821,'',61 from dual union all
select 56583,'1983-W317','1983-11-16 02:00:00','suborbital rocket','success',761,'MT135P-R511','','','','Meteo',1332,'',53 from dual union all
select 56584,'1983-W318','1983-11-16 16:19:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',65 from dual union all
select 56585,'1983-W319','1983-11-16 18:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',60 from dual union all
select 56586,'1983-W32','1983-02-13 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',911,'KRENKEL''',58 from dual union all
select 56587,'1983-W320','1983-11-17 20:33:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 56588,'1983-W321','1983-11-18 15:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 56589,'1983-W322','1983-11-21 19:31:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',66 from dual union all
select 5659,'1959-A201','1959-12-04 00:00:00','military missile','success',764,'56-2893','','','','Test',243,'',12 from dual union all
select 56590,'1983-W323','1983-11-22 00:42:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 56591,'1983-W324','1983-11-22 15:30:00','suborbital rocket','unknown',1284,'T 1-0771','','','','Aeron',1821,'',74 from dual union all
select 56592,'1983-W325','1983-11-23 12:50:00','suborbital rocket','unknown',716,'T 1-0783','','','','Aeron',1821,'',60 from dual union all
select 56593,'1983-W326','1983-11-23 18:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',50 from dual union all
select 56594,'1983-W327','1983-11-23 19:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',61 from dual union all
select 56595,'1983-W328','1983-11-28 16:09:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 56596,'1983-W329','1983-11-28 16:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 56597,'1983-W33','1983-02-14 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',911,'KRENKEL''',60 from dual union all
select 56598,'1983-W330','1983-11-30 02:00:00','suborbital rocket','success',761,'MT135P-R512','','','','Meteo',1332,'',53 from dual union all
select 56599,'1983-W331','1983-11-30 15:37:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',71 from dual union all
select 566,'1945-S42','1945-01-04 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5660,'1959-A202','1959-12-09 00:00:00','military missile','success',771,'GM-54-53','','','','Test',222,'',12 from dual union all
select 56600,'1983-W332','1983-11-30 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 56601,'1983-W333','1983-11-30 21:20:00','suborbital rocket','success',1284,'MAP/WINE M-M','M-D 01L','','','Aeron',962,'',75 from dual union all
select 56602,'1983-W334','1983-11-30 23:08:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',75 from dual union all
select 56603,'1983-W335','1983-12-02 16:18:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 56604,'1983-W336','1983-12-02 18:01:00','suborbital rocket','success',1502,'MAP/WINE','M-F 1','','','Aeron',29,'',75 from dual union all
select 56605,'1983-W337','1983-12-03 22:25:00','suborbital rocket','success',1284,'MAP/WINE M-M','M-D 02L','','','Aeron',962,'',75 from dual union all
select 56606,'1983-W338','1983-12-04 15:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 56607,'1983-W339','1983-12-06 10:07:00','suborbital rocket','success',1284,'MAP/WINE','M-C 1','','','Aeron',29,'',75 from dual union all
select 56608,'1983-W34','1983-02-14 21:15:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',68 from dual union all
select 56609,'1983-W340','1983-12-06 10:49:30','suborbital rocket','success',1284,'MAP/WINE','M-C 2','','','Aeron',29,'',75 from dual union all
select 5661,'1959-A203','1959-12-11 00:00:00','military missile','success',1214,'SM-62A N-3416','','','','OT',238,'',10 from dual union all
select 56610,'1983-W341','1983-12-06 15:56:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',69 from dual union all
select 56611,'1983-W342','1983-12-06 21:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 56612,'1983-W343','1983-12-07 02:00:00','suborbital rocket','success',761,'MT135P-R513','','','','Meteo',1332,'',56 from dual union all
select 56613,'1983-W344','1983-12-07 12:55:00','suborbital rocket','unknown',716,'T 1-0787','','','','Aeron',1821,'',62 from dual union all
select 56614,'1983-W345','1983-12-07 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',63 from dual union all
select 56615,'1983-W346','1983-12-07 17:12:00','suborbital rocket','success',1284,'MAP/WINE','M-C 3','','','Aeron',29,'',75 from dual union all
select 56616,'1983-W347','1983-12-07 18:49:00','suborbital rocket','success',1284,'MAP/WINE','M-C 4','','','Aeron',29,'',75 from dual union all
select 56617,'1983-W348','1983-12-07 20:27:00','suborbital rocket','success',1284,'MAP/WINE M-M','M-D 03L','','','Aeron',962,'',75 from dual union all
select 56618,'1983-W349','1983-12-07 23:23:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',60 from dual;
insert into LAUNCH
select 56619,'1983-W35','1983-02-15 08:22:00','suborbital rocket','unknown',716,'T 1-0484','','','','Test/Aeron',1821,'',62 from dual union all
select 5662,'1959-A204','1959-12-12 17:11:00','military missile','failure',1419,'C-3','RVX-3','','','Test',231,'',0 from dual union all
select 56620,'1983-W350','1983-12-08 10:44:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',73 from dual union all
select 56621,'1983-W351','1983-12-08 17:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 56622,'1983-W352','1983-12-08 18:45:00','suborbital rocket','unknown',1284,'T 1-0798','','','','Test/Aeron',1821,'',77 from dual union all
select 56623,'1983-W353','1983-12-08 22:18:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 56624,'1983-W354','1983-12-08 23:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',68 from dual union all
select 56625,'1983-W355','1983-12-08 23:44:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',72 from dual union all
select 56626,'1983-W356','1983-12-09 00:00:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',52 from dual union all
select 56627,'1983-W357','1983-12-09 00:02:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',79 from dual union all
select 56628,'1983-W358','1983-12-09 00:07:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',70 from dual union all
select 56629,'1983-W359','1983-12-09 00:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',63 from dual union all
select 5663,'1959-A205','1959-12-15 02:14:00','military missile','failure',1367,'185','IWST','','TALL GIRL','Test',1674,'',0 from dual union all
select 56630,'1983-W36','1983-02-16 02:01:00','suborbital rocket','success',761,'MT135P-R485','','','','Meteo',1332,'',54 from dual union all
select 56631,'1983-W360','1983-12-09 00:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 56632,'1983-W361','1983-12-09 15:50:00','suborbital rocket','unknown',1284,'T 1-0767','','','','Aeron',1821,'',77 from dual union all
select 56633,'1983-W362','1983-12-10 20:10:00','suborbital rocket','success',1284,'MAP/WINE M-M','M-D 04L','','','Aeron',962,'',75 from dual union all
select 56634,'1983-W363','1983-12-13 16:26:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',76 from dual union all
select 56635,'1983-W364','1983-12-14 02:03:59','suborbital rocket','success',761,'MT135P-R514','','','','Meteo',1332,'',56 from dual union all
select 56636,'1983-W365','1983-12-15 01:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 56637,'1983-W366','1983-12-15 15:58:59','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',70 from dual union all
select 56638,'1983-W367','1983-12-15 21:00:00','suborbital rocket','success',1284,'MAP/WINE M-M','M-D 05L','','','Aeron',962,'',75 from dual union all
select 56639,'1983-W368','1983-12-17 15:56:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',73 from dual union all
select 5664,'1959-A206','1959-12-16 00:00:00','military missile','success',1214,'N-69D N-3317-3','','','','OT',223,'',10 from dual union all
select 56640,'1983-W369','1983-12-17 18:41:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',74 from dual union all
select 56641,'1983-W37','1983-02-16 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',911,'KRENKEL''',58 from dual union all
select 56642,'1983-W370','1983-12-19 18:17:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',74 from dual union all
select 56643,'1983-W371','1983-12-20 00:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',78 from dual union all
select 56644,'1983-W372','1983-12-21 02:03:00','suborbital rocket','success',761,'MT135P-R515','','','','Meteo',1332,'',56 from dual union all
select 56645,'1983-W373','1983-12-21 12:55:00','suborbital rocket','unknown',716,'T 1-0789','','','','Aeron',1821,'',59 from dual union all
select 56646,'1983-W374','1983-12-21 19:28:00','suborbital rocket','success',1284,'MAP/WINE M-M','M-D 06L','','','Aeron',962,'',75 from dual union all
select 56647,'1983-W375','1983-12-21 21:22:00','suborbital rocket','success',1284,'MAP/WINE M-M','M-D 07L','','','Aeron',962,'',75 from dual union all
select 56648,'1983-W376','1983-12-21 23:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',64 from dual union all
select 56649,'1983-W377','1983-12-22 15:58:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',66 from dual union all
select 5665,'1959-A207','1959-12-16 00:00:00','military missile','failure',873,'ZW-3/104','3','','','Test',1884,'',10 from dual union all
select 56650,'1983-W378','1983-12-22 18:35:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',62 from dual union all
select 56651,'1983-W379','1983-12-28 02:00:00','suborbital rocket','success',761,'MT135P-R516','','','','Meteo',1332,'',56 from dual union all
select 56652,'1983-W38','1983-02-16 20:03:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',72 from dual union all
select 56653,'1983-W380','1983-12-28 08:32:00','suborbital rocket','success',1284,'MAP/WINE','M-D 08A','','','Aeron',29,'',75 from dual union all
select 56654,'1983-W381','1983-12-28 10:19:00','suborbital rocket','success',1284,'MAP/WINE','M-D 09A','','','Aeron',29,'',75 from dual union all
select 56655,'1983-W382','1983-12-29 19:50:00','suborbital rocket','success',1284,'MAP/WINE M-M','M-D 08L','','','Aeron',962,'',75 from dual union all
select 56656,'1983-W383','1983-12-31 17:00:00','suborbital rocket','success',1284,'MAP/WINE','M-D 11A','','','Aeron',29,'',72 from dual union all
select 56657,'1983-W384','1983-09-28 02:00:00','suborbital rocket','success',761,'MT135P-R506','','','','Meteo',1332,'',60 from dual union all
select 56658,'1983-W39','1983-02-18 00:56:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',65 from dual union all
select 56659,'1983-W40','1983-02-18 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',911,'KRENKEL''',61 from dual union all
select 5666,'1959-A208','1959-12-16 00:00:00','suborbital spaceplane','failure',1520,'1-A-6','','','','Test',379,'NB-52',14 from dual union all
select 56660,'1983-W41','1983-02-18 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',63 from dual union all
select 56661,'1983-W42','1983-02-21 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',911,'KRENKEL''',59 from dual union all
select 56662,'1983-W43','1983-02-22 18:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',71 from dual union all
select 56663,'1983-W44','1983-02-23 00:27:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 56664,'1983-W45','1983-02-23 02:00:00','suborbital rocket','success',761,'MT135P-R486','','','','Meteo',1332,'',56 from dual union all
select 56665,'1983-W46','1983-02-23 07:50:00','suborbital rocket','unknown',738,'','','','','Aeron',911,'KRENKEL''',61 from dual union all
select 56666,'1983-W47','1983-02-23 08:22:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',56 from dual union all
select 56667,'1983-W48','1983-02-23 08:48:00','suborbital rocket','unknown',716,'T 1-0517','','','','Test/Aeron',1821,'',56 from dual union all
select 56668,'1983-W49','1983-02-23 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',66 from dual union all
select 56669,'1983-W50','1983-02-26 02:48:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 5667,'1959-A209','1959-12-22 00:00:00','military missile','success',793,'','Silo Test','','','Test',381,'',0 from dual union all
select 56670,'1983-W51','1983-02-26 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',62 from dual union all
select 56671,'1983-W52','1983-02-27 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',911,'KRENKEL''',60 from dual union all
select 56672,'1983-W53','1983-02-28 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 56673,'1983-W54','1983-03-01 00:54:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 56674,'1983-W55','1983-03-02 08:55:00','suborbital rocket','unknown',738,'','','','','Aeron',213,'BUGAEV',56 from dual union all
select 56675,'1983-W56','1983-03-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',74 from dual union all
select 56676,'1983-W57','1983-03-02 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',65 from dual union all
select 56677,'1983-W58','1983-03-02 20:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 56678,'1983-W59','1983-03-03 21:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',53 from dual union all
select 56679,'1983-W60','1983-03-04 08:15:00','suborbital rocket','unknown',716,'T 1-0492','','','','Test/Aeron',1821,'',61 from dual union all
select 5668,'1959-A21','1959-01-31 00:00:00','suborbital rocket','success',439,'','','','','Aeron',1490,'',24 from dual union all
select 56680,'1983-W61','1983-03-04 08:40:00','suborbital rocket','unknown',738,'','','','','Aeron',213,'BUGAEV',58 from dual union all
select 56681,'1983-W62','1983-03-07 08:15:00','suborbital rocket','unknown',738,'','','','','Aeron',213,'BUGAEV',60 from dual union all
select 56682,'1983-W63','1983-03-07 21:13:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',59 from dual union all
select 56683,'1983-W64','1983-03-08 19:17:00','suborbital rocket','unknown',1284,'TU1-0388 PWN-11A','CONDOR Datasonde','','','Aeron',322,'',65 from dual union all
select 56684,'1983-W65','1983-03-09 02:00:00','suborbital rocket','success',761,'MT135P-R487','','','','Meteo',1332,'',53 from dual union all
select 56685,'1983-W66','1983-03-09 05:32:00','suborbital rocket','unknown',1284,'TU1-0359 PWN-11A','CONDOR Sphere','','','Aeron',322,'',79 from dual union all
select 56686,'1983-W67','1983-03-09 08:20:00','suborbital rocket','unknown',738,'','','','','Aeron',213,'BUGAEV',62 from dual union all
select 56687,'1983-W68','1983-03-10 18:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',59 from dual union all
select 56688,'1983-W69','1983-03-10 20:38:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',74 from dual union all
select 56689,'1983-W70','1983-03-11 08:20:00','suborbital rocket','unknown',738,'','','','','Aeron',213,'BUGAEV',61 from dual union all
select 5669,'1959-A210','1959-12-23 00:00:00','military missile','failure',928,'A1X-5','','','','Test',245,'',1 from dual union all
select 56690,'1983-W71','1983-03-11 08:35:00','suborbital rocket','unknown',716,'T 1-0494','','','','Test/Aeron',1821,'',61 from dual union all
select 56691,'1983-W72','1983-03-11 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',50 from dual union all
select 56692,'1983-W73','1983-03-12 18:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',63 from dual union all
select 56693,'1983-W74','1983-03-13 15:00:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',54 from dual union all
select 56694,'1983-W75','1983-03-14 07:35:00','suborbital rocket','unknown',738,'','','','','Aeron',213,'BUGAEV',58 from dual union all
select 56695,'1983-W76','1983-03-14 19:20:00','suborbital rocket','unknown',716,'T 1-0496','','','','Test/Aeron',1821,'',60 from dual union all
select 56696,'1983-W77','1983-03-14 21:14:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',51 from dual union all
select 56697,'1983-W78','1983-03-15 16:32:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',59 from dual union all
select 56698,'1983-W79','1983-03-15 20:21:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',79 from dual union all
select 56699,'1983-W80','1983-03-16 02:00:00','suborbital rocket','success',761,'MT135P-R488','','','','Meteo',1332,'',54 from dual union all
select 567,'1945-S43','1945-01-04 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5670,'1959-A211','1959-10-20 00:00:00','suborbital rocket','success',715,'Loki I','','','','Aeron',1057,'',35 from dual union all
select 56700,'1983-W81','1983-03-16 08:40:00','suborbital rocket','unknown',738,'','','','','Aeron',213,'BUGAEV',61 from dual union all
select 56701,'1983-W82','1983-03-16 13:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',68 from dual union all
select 56702,'1983-W83','1983-03-16 18:59:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',64 from dual union all
select 56703,'1983-W84','1983-03-18 07:45:00','suborbital rocket','unknown',738,'','','','','Aeron',213,'BUGAEV',60 from dual union all
select 56704,'1983-W85','1983-03-18 21:19:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',855,'',66 from dual union all
select 56705,'1983-W86','1983-03-21 21:11:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',56 from dual union all
select 56706,'1983-W87','1983-03-22 16:30:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1307,'',61 from dual union all
select 56707,'1983-W88','1983-03-22 19:24:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1821,'',52 from dual union all
select 56708,'1983-W89','1983-03-22 19:25:00','suborbital rocket','unknown',716,'T 1-0500','','','','Test/Aeron',1821,'',54 from dual;
insert into LAUNCH
select 56709,'1983-W90','1983-03-23 02:00:00','suborbital rocket','success',761,'MT135P-R489','','','','Meteo',1332,'',54 from dual union all
select 5671,'1959-A212','1959-10-21 00:00:00','suborbital rocket','success',715,'Loki I','','','','Aeron',1057,'',31 from dual union all
select 56710,'1983-W91','1983-03-23 07:40:00','suborbital rocket','unknown',738,'','','','','Aeron',213,'BUGAEV',58 from dual union all
select 56711,'1983-W92','1983-03-23 18:34:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1895,'',58 from dual union all
select 56712,'1983-W93','1983-03-23 18:52:00','suborbital rocket','unknown',1284,'TU1-0391','CONDOR','','','Aeron',322,'',78 from dual union all
select 56713,'1983-W94','1983-03-23 21:10:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1362,'',54 from dual union all
select 56714,'1983-W95','1983-03-24 00:41:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',76 from dual union all
select 56715,'1983-W96','1983-03-28 07:35:00','suborbital rocket','unknown',738,'','','','','Aeron',213,'BUGAEV',58 from dual union all
select 56716,'1983-W97','1983-03-28 08:30:00','suborbital rocket','unknown',716,'T 1-0502','','','','Test/Aeron',1821,'',66 from dual union all
select 56717,'1983-W98','1983-03-28 18:48:59','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',1057,'',61 from dual union all
select 56718,'1983-W99','1983-03-29 00:29:00','suborbital rocket','success',1061,'','Arcasonde','','','Meteo',873,'',73 from dual union all
select 56719,'1983-Y01','1983-01-03 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1753,'VOKV',91 from dual union all
select 5672,'1959-A213','1959-03-10 19:00:00','suborbital rocket','success',835,'','Cree III 28-1','','','Parachute test',385,'',10 from dual union all
select 56720,'1983-Y02','1983-01-04 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1753,'VOKV',88 from dual union all
select 56721,'1983-Y03','1983-01-04 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 56722,'1983-Y04','1983-01-05 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 56723,'1983-Y05','1983-01-05 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 56724,'1983-Y06','1983-01-05 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 56725,'1983-Y07','1983-01-06 14:55:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 56726,'1983-Y08','1983-01-08 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1753,'VOKV',82 from dual union all
select 56727,'1983-Y09','1983-01-09 21:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1753,'VOKV',85 from dual union all
select 56728,'1983-Y10','1983-01-11 20:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 56729,'1983-Y100','1983-03-16 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',96 from dual union all
select 5673,'1959-A214','1959-04-23 00:00:00','suborbital rocket','success',852,'','Cree III 29-2','','','Parachute test',385,'',12 from dual union all
select 56730,'1983-Y101','1983-03-16 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',83 from dual union all
select 56731,'1983-Y102','1983-03-17 18:25:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',90 from dual union all
select 56732,'1983-Y103','1983-03-18 18:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',86 from dual union all
select 56733,'1983-Y104','1983-03-19 20:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',85 from dual union all
select 56734,'1983-Y105','1983-03-20 18:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',88 from dual union all
select 56735,'1983-Y106','1983-03-21 19:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',86 from dual union all
select 56736,'1983-Y107','1983-03-22 20:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 56737,'1983-Y108','1983-03-22 21:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',83 from dual union all
select 56738,'1983-Y109','1983-03-23 04:50:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 56739,'1983-Y11','1983-01-12 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 5674,'1959-A215','1959-06-19 00:00:00','suborbital rocket','success',852,'','Cree III 30-3','','','Parachute test',385,'',20 from dual union all
select 56740,'1983-Y110','1983-03-23 05:45:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 56741,'1983-Y111','1983-03-23 09:10:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 56742,'1983-Y112','1983-03-23 10:20:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 56743,'1983-Y113','1983-03-23 11:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 56744,'1983-Y114','1983-03-23 13:55:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 56745,'1983-Y115','1983-03-23 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 56746,'1983-Y116','1983-03-23 14:30:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 56747,'1983-Y117','1983-03-23 15:45:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 56748,'1983-Y118','1983-03-28 10:00:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 56749,'1983-Y119','1983-03-28 11:03:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 5675,'1959-A216','1959-07-10 12:45:00','suborbital rocket','success',852,'','Cree III 31-4','','','Parachute test',385,'',20 from dual union all
select 56750,'1983-Y12','1983-01-12 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 56751,'1983-Y120','1983-03-28 15:00:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 56752,'1983-Y121','1983-03-28 17:00:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 56753,'1983-Y122','1983-03-28 18:05:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 56754,'1983-Y123','1983-03-28 18:24:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 56755,'1983-Y124','1983-03-29 08:50:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 56756,'1983-Y125','1983-03-29 20:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 56757,'1983-Y126','1983-03-30 14:05:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',87 from dual union all
select 56758,'1983-Y127','1983-03-30 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 56759,'1983-Y128','1983-03-31 10:30:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 5676,'1959-A217','1959-09-02 00:00:00','suborbital rocket','success',852,'','Cree III 32-5','','','Parachute test',385,'',23 from dual union all
select 56760,'1983-Y129','1983-03-31 11:50:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 56761,'1983-Y13','1983-01-12 14:55:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 56762,'1983-Y130','1983-03-31 14:10:00','suborbital rocket','success',725,'','Ozone Intercomp.','','','Aeron',1467,'',85 from dual union all
select 56763,'1983-Y131','1983-04-01 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',96 from dual union all
select 56764,'1983-Y132','1983-04-04 16:23:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 56765,'1983-Y133','1983-04-04 18:41:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',86 from dual union all
select 56766,'1983-Y134','1983-04-06 13:15:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 56767,'1983-Y135','1983-04-06 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',88 from dual union all
select 56768,'1983-Y136','1983-04-06 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 56769,'1983-Y137','1983-04-06 19:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 5677,'1959-A218','1959-01-22 00:00:00','atmospheric rocket','success',603,'21RO','','','','Test',1895,'',2 from dual union all
select 56770,'1983-Y138','1983-04-09 12:33:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 56771,'1983-Y139','1983-04-09 14:30:00','suborbital rocket','unknown',1284,'T 1-0460','','','','Test/Aeron',1821,'',109 from dual union all
select 56772,'1983-Y14','1983-01-12 18:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 56773,'1983-Y140','1983-04-09 14:50:00','suborbital rocket','unknown',1284,'T 1-0459','','','','Test/Aeron',1821,'',81 from dual union all
select 56774,'1983-Y141','1983-04-09 15:17:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 56775,'1983-Y142','1983-04-09 15:37:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',93 from dual union all
select 56776,'1983-Y143','1983-04-09 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',93 from dual union all
select 56777,'1983-Y144','1983-04-09 16:23:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 56778,'1983-Y145','1983-04-09 17:59:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 56779,'1983-Y146','1983-04-11 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',92 from dual union all
select 5678,'1959-A219','1959-01-22 00:00:00','atmospheric rocket','success',603,'22RO','','','','Test',1895,'',2 from dual union all
select 56780,'1983-Y147','1983-04-13 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 56781,'1983-Y148','1983-04-13 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 56782,'1983-Y149','1983-04-13 19:00:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',96 from dual union all
select 56783,'1983-Y15','1983-01-13 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 56784,'1983-Y150','1983-04-13 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 56785,'1983-Y151','1983-04-14 19:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 56786,'1983-Y152','1983-04-20 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',80 from dual union all
select 56787,'1983-Y153','1983-04-20 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 56788,'1983-Y154','1983-04-20 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 56789,'1983-Y155','1983-04-20 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 5679,'1959-A22','1959-02-03 20:25:00','military missile','success',843,'','Cold weather test','','','Test',428,'',10 from dual union all
select 56790,'1983-Y156','1983-04-20 21:24:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 56791,'1983-Y157','1983-04-21 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 56792,'1983-Y158','1983-04-25 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 56793,'1983-Y159','1983-04-25 19:55:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',83 from dual union all
select 56794,'1983-Y16','1983-01-18 20:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 56795,'1983-Y160','1983-04-25 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 56796,'1983-Y161','1983-04-26 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 56797,'1983-Y162','1983-04-27 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 56798,'1983-Y163','1983-04-27 15:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',90 from dual union all
select 56799,'1983-Y164','1983-04-27 16:35:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',95 from dual;
insert into LAUNCH
select 568,'1945-S44','1945-01-04 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5680,'1959-A220','1959-03-25 00:00:00','atmospheric rocket','success',603,'E1RO','','','','Test',1895,'',2 from dual union all
select 56800,'1983-Y165','1983-04-28 16:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 56801,'1983-Y166','1983-05-03 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 56802,'1983-Y167','1983-05-04 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 56803,'1983-Y168','1983-05-04 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 56804,'1983-Y169','1983-05-05 17:13:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual union all
select 56805,'1983-Y17','1983-01-19 01:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 56806,'1983-Y170','1983-05-05 21:32:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',92 from dual union all
select 56807,'1983-Y171','1983-05-10 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 56808,'1983-Y172','1983-05-11 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 56809,'1983-Y173','1983-05-18 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 5681,'1959-A221','1959-03-25 00:00:00','atmospheric rocket','success',603,'E2RO','','','','Test',1895,'',2 from dual union all
select 56810,'1983-Y174','1983-05-24 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 56811,'1983-Y175','1983-05-25 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 56812,'1983-Y176','1983-05-26 16:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 56813,'1983-Y177','1983-05-30 23:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',92 from dual union all
select 56814,'1983-Y178','1983-05-31 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 56815,'1983-Y179','1983-06-01 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 56816,'1983-Y18','1983-01-19 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 56817,'1983-Y180','1983-06-01 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 56818,'1983-Y181','1983-06-01 18:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 56819,'1983-Y182','1983-06-02 02:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 5682,'1959-A222','1959-04-09 00:00:00','atmospheric rocket','success',603,'E3RO','','','','Test',1895,'',2 from dual union all
select 56820,'1983-Y183','1983-06-02 04:25:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',90 from dual union all
select 56821,'1983-Y184','1983-06-03 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 56822,'1983-Y185','1983-06-07 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 56823,'1983-Y186','1983-06-08 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',82 from dual union all
select 56824,'1983-Y187','1983-06-08 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 56825,'1983-Y188','1983-06-08 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 56826,'1983-Y189','1983-06-08 17:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 56827,'1983-Y19','1983-01-20 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',90 from dual union all
select 56828,'1983-Y190','1983-06-10 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 56829,'1983-Y191','1983-06-15 02:30:00','suborbital rocket','success',1281,'A14.311-1','STATE 1','','','Ionos',1047,'',101 from dual union all
select 5683,'1959-A223','1959-04-01 00:00:00','atmospheric rocket','success',603,'E4RO','','','','Test',1895,'',2 from dual union all
select 56830,'1983-Y192','1983-06-15 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',85 from dual union all
select 56831,'1983-Y193','1983-06-15 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 56832,'1983-Y194','1983-06-15 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 56833,'1983-Y195','1983-06-16 07:07:00','suborbital rocket','success',1281,'A14.311-2','STATE 2','','','Ionos',1047,'',106 from dual union all
select 56834,'1983-Y196','1983-06-16 15:56:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',93 from dual union all
select 56835,'1983-Y197','1983-06-17 15:06:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 56836,'1983-Y198','1983-06-17 18:15:00','suborbital rocket','success',1281,'A14.311-3','STATE 3','','','Ionos',1047,'',98 from dual union all
select 56837,'1983-Y199','1983-06-19 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',90 from dual union all
select 56838,'1983-Y20','1983-01-21 01:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',81 from dual union all
select 56839,'1983-Y200','1983-06-19 15:30:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',85 from dual union all
select 5684,'1959-A224','1959-04-01 00:00:00','atmospheric rocket','success',603,'E5RO','','','','Test',1895,'',2 from dual union all
select 56840,'1983-Y201','1983-06-20 20:47:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 56841,'1983-Y202','1983-06-21 13:00:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',83 from dual union all
select 56842,'1983-Y203','1983-06-21 14:50:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',82 from dual union all
select 56843,'1983-Y204','1983-06-21 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 56844,'1983-Y205','1983-06-22 02:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 56845,'1983-Y206','1983-06-22 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 56846,'1983-Y207','1983-06-22 15:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 56847,'1983-Y208','1983-06-22 20:53:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 56848,'1983-Y209','1983-06-24 11:30:00','suborbital rocket','unknown',1284,'T 1-0548','','','','Test/Aeron',1821,'',110 from dual union all
select 56849,'1983-Y21','1983-01-25 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',82 from dual union all
select 5685,'1959-A225','1959-04-01 00:00:00','atmospheric rocket','success',603,'E6RO','','','','Test',1895,'',2 from dual union all
select 56850,'1983-Y210','1983-06-24 12:01:00','suborbital rocket','unknown',1284,'T 1-0547','','','','Test/Aeron',1821,'',81 from dual union all
select 56851,'1983-Y211','1983-06-24 13:23:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 56852,'1983-Y212','1983-06-24 13:35:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 56853,'1983-Y213','1983-06-24 13:56:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 56854,'1983-Y214','1983-06-24 14:08:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',93 from dual union all
select 56855,'1983-Y215','1983-06-24 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',81 from dual union all
select 56856,'1983-Y216','1983-06-24 14:24:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',88 from dual union all
select 56857,'1983-Y217','1983-06-24 16:17:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',92 from dual union all
select 56858,'1983-Y218','1983-06-28 09:45:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',85 from dual union all
select 56859,'1983-Y219','1983-06-28 11:07:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',88 from dual union all
select 5686,'1959-A226','1959-04-01 00:00:00','atmospheric rocket','success',603,'E7RO','','','','Test',1895,'',2 from dual union all
select 56860,'1983-Y22','1983-01-26 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 56861,'1983-Y220','1983-06-29 14:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 56862,'1983-Y221','1983-06-29 19:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 56863,'1983-Y222','1983-06-30 10:50:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',85 from dual union all
select 56864,'1983-Y223','1983-06-30 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 56865,'1983-Y224','1983-07-01 15:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 56866,'1983-Y225','1983-07-04 10:00:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',87 from dual union all
select 56867,'1983-Y226','1983-07-05 09:30:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',88 from dual union all
select 56868,'1983-Y227','1983-07-05 17:15:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 56869,'1983-Y228','1983-07-05 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',84 from dual union all
select 5687,'1959-A227','1959-04-01 00:00:00','atmospheric rocket','success',603,'E8RO','','','','Test',1895,'',2 from dual union all
select 56870,'1983-Y229','1983-07-05 21:26:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',80 from dual union all
select 56871,'1983-Y23','1983-01-27 14:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',88 from dual union all
select 56872,'1983-Y230','1983-07-06 14:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 56873,'1983-Y231','1983-07-06 15:33:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 56874,'1983-Y232','1983-07-06 19:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',92 from dual union all
select 56875,'1983-Y233','1983-07-08 14:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 56876,'1983-Y234','1983-07-13 14:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 56877,'1983-Y235','1983-07-15 14:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 56878,'1983-Y236','1983-07-19 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 56879,'1983-Y237','1983-07-19 20:26:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 5688,'1959-A228','1959-04-01 00:00:00','atmospheric rocket','success',603,'E9RO','','','','Test',1895,'',2 from dual union all
select 56880,'1983-Y238','1983-07-20 14:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',83 from dual union all
select 56881,'1983-Y239','1983-07-20 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 56882,'1983-Y24','1983-02-01 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 56883,'1983-Y240','1983-07-22 14:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 56884,'1983-Y241','1983-07-22 15:45:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',88 from dual union all
select 56885,'1983-Y242','1983-07-22 17:50:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',82 from dual union all
select 56886,'1983-Y243','1983-07-27 14:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 56887,'1983-Y244','1983-07-27 17:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 56888,'1983-Y245','1983-07-29 14:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 56889,'1983-Y246','1983-08-02 19:09:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual;
insert into LAUNCH
select 5689,'1959-A229','1959-04-01 00:00:00','atmospheric rocket','success',603,'E10RO','','','','Test',1895,'',2 from dual union all
select 56890,'1983-Y247','1983-08-02 20:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 56891,'1983-Y248','1983-08-03 15:11:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 56892,'1983-Y249','1983-08-04 16:32:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',83 from dual union all
select 56893,'1983-Y25','1983-02-02 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',86 from dual union all
select 56894,'1983-Y250','1983-08-05 18:11:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',86 from dual union all
select 56895,'1983-Y251','1983-08-06 17:46:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',87 from dual union all
select 56896,'1983-Y252','1983-08-06 19:19:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',86 from dual union all
select 56897,'1983-Y253','1983-08-07 14:11:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 56898,'1983-Y254','1983-08-10 14:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 56899,'1983-Y255','1983-08-10 16:32:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',85 from dual union all
select 569,'1945-S45','1945-01-04 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5690,'1959-A23','1959-02-05 00:00:00','atmospheric rocket','success',542,'','Corvus Model','','','Test',1821,'',5 from dual union all
select 56900,'1983-Y256','1983-08-10 17:56:00','suborbital rocket','unknown',725,'','','','','Aeron',894,'KORO',84 from dual union all
select 56901,'1983-Y257','1983-08-10 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 56902,'1983-Y258','1983-08-16 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 56903,'1983-Y259','1983-08-17 14:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 56904,'1983-Y26','1983-02-02 14:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 56905,'1983-Y260','1983-08-17 16:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',91 from dual union all
select 56906,'1983-Y261','1983-08-17 16:20:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',82 from dual union all
select 56907,'1983-Y262','1983-08-23 20:10:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',89 from dual union all
select 56908,'1983-Y263','1983-08-24 14:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',87 from dual union all
select 56909,'1983-Y264','1983-08-24 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 5691,'1959-A230','1959-05-01 00:00:00','atmospheric rocket','success',603,'E11RO','','','','Test',1895,'',2 from dual union all
select 56910,'1983-Y265','1983-08-30 08:38:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual union all
select 56911,'1983-Y266','1983-08-31 14:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',85 from dual union all
select 56912,'1983-Y267','1983-08-31 14:24:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',85 from dual union all
select 56913,'1983-Y268','1983-08-31 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1754,'VOKV',87 from dual union all
select 56914,'1983-Y269','1983-09-01 08:01:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 56915,'1983-Y27','1983-02-05 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 56916,'1983-Y270','1983-09-01 08:22:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 56917,'1983-Y271','1983-09-02 17:48:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 56918,'1983-Y272','1983-09-03 07:25:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 56919,'1983-Y273','1983-09-03 18:55:00','suborbital rocket','unknown',725,'','','','','Aeron',1754,'VOKV',87 from dual union all
select 5692,'1959-A231','1959-05-05 00:00:00','atmospheric rocket','success',603,'E12RO','','','','Test',1895,'',2 from dual union all
select 56920,'1983-Y274','1983-09-05 06:13:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 56921,'1983-Y275','1983-09-05 07:30:00','suborbital rocket','unknown',1284,'T 1-0663','','','','Test/Aeron',1821,'',107 from dual union all
select 56922,'1983-Y276','1983-09-05 07:41:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',92 from dual union all
select 56923,'1983-Y277','1983-09-05 08:02:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 56924,'1983-Y278','1983-09-07 14:22:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',84 from dual union all
select 56925,'1983-Y279','1983-09-07 17:05:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',84 from dual union all
select 56926,'1983-Y28','1983-02-06 01:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 56927,'1983-Y280','1983-09-07 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 56928,'1983-Y281','1983-09-12 21:31:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 56929,'1983-Y282','1983-09-13 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 5693,'1959-A232','1959-06-02 00:00:00','atmospheric rocket','success',603,'E13RO','XM-31E2 test','','','Test',1895,'',2 from dual union all
select 56930,'1983-Y283','1983-09-13 21:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 56931,'1983-Y284','1983-09-14 13:54:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 56932,'1983-Y285','1983-09-20 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',88 from dual union all
select 56933,'1983-Y286','1983-09-20 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 56934,'1983-Y287','1983-09-21 13:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 56935,'1983-Y288','1983-09-21 14:30:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 56936,'1983-Y289','1983-10-20 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 56937,'1983-Y29','1983-02-07 07:14:25','suborbital rocket','success',1281,'NASA 15.224UE','Auroral X','','','Plasma',428,'',71 from dual union all
select 56938,'1983-Y290','1983-11-02 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 56939,'1983-Y291','1983-11-16 20:37:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 5694,'1959-A233','1959-06-02 00:00:00','atmospheric rocket','success',603,'E14RO','XM-31E2 test','','','Test',1895,'',2 from dual union all
select 56940,'1983-Y292','1983-11-17 09:26:00','suborbital rocket','unknown',1284,'T 1-0763','','','','Aeron',1821,'',81 from dual union all
select 56941,'1983-Y293','1983-11-19 09:45:00','suborbital rocket','unknown',1284,'T 1-0794','','','','Aeron',1821,'',80 from dual union all
select 56942,'1983-Y294','1983-11-21 19:07:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 56943,'1983-Y295','1983-11-23 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 56944,'1983-Y296','1983-12-02 15:22:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 56945,'1983-Y297','1983-12-03 20:00:00','suborbital rocket','success',1284,'MAP/WINE','M-D 01A','','','Aeron',29,'',80 from dual union all
select 56946,'1983-Y298','1983-12-04 15:01:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 56947,'1983-Y299','1983-12-05 20:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 2','','','Aeron',29,'',115 from dual union all
select 56948,'1983-Y30','1983-02-07 07:21:25','suborbital rocket','success',1281,'NASA 15.225UE','Auroral X','','','Plasma',428,'',71 from dual union all
select 56949,'1983-Y300','1983-12-07 17:44:00','suborbital rocket','success',1502,'MAP/WINE','M-F 3','','','Aeron',29,'',108 from dual union all
select 5695,'1959-A234','1959-06-02 00:00:00','atmospheric rocket','success',603,'E15RM','Warhead test','','','Test',1895,'',2 from dual union all
select 56950,'1983-Y301','1983-12-07 19:55:00','suborbital rocket','success',1284,'MAP/WINE','M-D 02A','','','Aeron',29,'',85 from dual union all
select 56951,'1983-Y302','1983-12-08 10:02:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 56952,'1983-Y303','1983-12-08 18:25:00','suborbital rocket','unknown',1284,'T 1-0797','','','','Test/Aeron',1821,'',109 from dual union all
select 56953,'1983-Y304','1983-12-08 20:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 4','','','Aeron',29,'',103 from dual union all
select 56954,'1983-Y305','1983-12-08 21:06:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 56955,'1983-Y306','1983-12-08 22:09:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 56956,'1983-Y307','1983-12-10 21:34:00','suborbital rocket','success',1284,'MAP/WINE','M-D 03A','','','Aeron',29,'',83 from dual union all
select 56957,'1983-Y308','1983-12-12 20:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 5','','','Aeron',29,'',118 from dual union all
select 56958,'1983-Y309','1983-12-13 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 56959,'1983-Y31','1983-02-07 09:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',92 from dual union all
select 5696,'1959-A235','1959-06-05 00:00:00','atmospheric rocket','success',603,'E16RO','XM-31E2 test','','','Test',1895,'',2 from dual union all
select 56960,'1983-Y310','1983-12-14 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 56961,'1983-Y311','1983-12-15 20:00:00','suborbital rocket','success',1284,'MAP/WINE','M-D 04A','','','Aeron',29,'',88 from dual union all
select 56962,'1983-Y312','1983-12-15 20:46:00','suborbital rocket','success',1284,'MAP/WINE','M-D 05A','','','Aeron',29,'',83 from dual union all
select 56963,'1983-Y313','1983-12-16 11:29:00','suborbital rocket','success',1502,'MAP/WINE','M-F 6','','','Aeron',29,'',113 from dual union all
select 56964,'1983-Y314','1983-12-16 12:12:00','suborbital rocket','success',1502,'MAP/WINE','M-F 7','','','Aeron',29,'',106 from dual union all
select 56965,'1983-Y315','1983-12-17 19:50:00','suborbital rocket','success',1284,'MAP/WINE','M-D 06A','','','Aeron',29,'',82 from dual union all
select 56966,'1983-Y316','1983-12-19 17:52:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 56967,'1983-Y317','1983-12-19 19:46:00','suborbital rocket','success',1502,'MAP/WINE','M-F 8','','','Aeron',29,'',108 from dual union all
select 56968,'1983-Y318','1983-12-19 20:15:00','suborbital rocket','success',1502,'MAP/WINE','M-F 9','','','Aeron',29,'',113 from dual union all
select 56969,'1983-Y319','1983-12-21 19:00:00','suborbital rocket','success',1284,'MAP/WINE','M-D 07A','','','Aeron',29,'',84 from dual union all
select 5697,'1959-A236','1959-06-05 00:00:00','atmospheric rocket','success',603,'E17RO','XM-31E2 test','','','Test',1895,'',2 from dual union all
select 56970,'1983-Y32','1983-02-07 09:25:00','suborbital rocket','unknown',1284,'T 1-0507','','','','Test/Aeron',1821,'',111 from dual union all
select 56971,'1983-Y320','1983-12-29 11:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 10','','','Aeron',29,'',118 from dual union all
select 56972,'1983-Y321','1983-12-29 11:34:00','suborbital rocket','success',1502,'MAP/WINE','M-F 11','','','Aeron',29,'',119 from dual union all
select 56973,'1983-Y322','1983-12-31 15:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 12','','','Aeron',29,'',100 from dual union all
select 56974,'1983-Y323','1983-12-31 15:56:00','suborbital rocket','success',1284,'MAP/WINE','M-D 10A','','','Aeron',29,'',88 from dual union all
select 56975,'1983-Y324','1983-10-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 56976,'1983-Y325','1983-10-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 56977,'1983-Y326','1983-10-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 56978,'1983-Y327','1983-10-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 56979,'1983-Y328','1983-11-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual;
insert into LAUNCH
select 5698,'1959-A237','1959-06-19 00:00:00','atmospheric rocket','success',603,'E18RO','Acc test','','','Test',1895,'',2 from dual union all
select 56980,'1983-Y329','1983-11-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 56981,'1983-Y33','1983-02-07 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',92 from dual union all
select 56982,'1983-Y330','1983-11-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 56983,'1983-Y331','1983-11-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 56984,'1983-Y332','1983-11-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 56985,'1983-Y333','1983-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 56986,'1983-Y334','1983-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 56987,'1983-Y335','1983-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 56988,'1983-Y336','1983-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 56989,'1983-Y34','1983-02-08 01:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',83 from dual union all
select 5699,'1959-A238','1959-07-02 00:00:00','atmospheric rocket','success',603,'E37RO','Acc test','','','Test',1895,'',2 from dual union all
select 56990,'1983-Y35','1983-02-09 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 56991,'1983-Y36','1983-02-09 15:15:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',85 from dual union all
select 56992,'1983-Y37','1983-02-10 14:17:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 56993,'1983-Y38','1983-02-10 19:03:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 56994,'1983-Y39','1983-02-15 09:44:00','suborbital rocket','unknown',1284,'T 1-0512','','','','Test/Aeron',1821,'',109 from dual union all
select 56995,'1983-Y40','1983-02-16 02:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 56996,'1983-Y41','1983-02-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',81 from dual union all
select 56997,'1983-Y42','1983-02-16 16:50:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 56998,'1983-Y43','1983-02-16 19:26:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',93 from dual union all
select 56999,'1983-Y44','1983-02-21 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',91 from dual union all
select 57,'1944-A21','1944-11-24 19:56:00','military missile','failure',1470,'19834','Ma304','','','Test',661,'',20 from dual union all
select 570,'1945-S46','1945-01-04 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5700,'1959-A239','1959-12-17 00:10:00','military missile','success',843,'562Art Btn 2 Bat B','Service firing test','','','Test',443,'',3 from dual union all
select 57000,'1983-Y45','1983-02-22 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',86 from dual union all
select 57001,'1983-Y46','1983-02-22 21:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',91 from dual union all
select 57002,'1983-Y47','1983-02-23 09:38:00','suborbital rocket','unknown',1284,'T 1-0488','','','','Test/Aeron',1821,'',102 from dual union all
select 57003,'1983-Y48','1983-02-23 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',86 from dual union all
select 57004,'1983-Y49','1983-02-23 16:20:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 57005,'1983-Y50','1983-02-23 16:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',86 from dual union all
select 57006,'1983-Y51','1983-02-23 20:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',81 from dual union all
select 57007,'1983-Y52','1983-02-24 18:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',83 from dual union all
select 57008,'1983-Y53','1983-02-25 18:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',87 from dual union all
select 57009,'1983-Y54','1983-02-26 18:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',83 from dual union all
select 5701,'1959-A24','1959-02-07 00:00:00','military missile','success',843,'','Cold weather test','','','Test',428,'',10 from dual union all
select 57010,'1983-Y55','1983-02-27 18:45:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',85 from dual union all
select 57011,'1983-Y56','1983-02-28 18:52:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',81 from dual union all
select 57012,'1983-Y57','1983-03-01 19:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',85 from dual union all
select 57013,'1983-Y58','1983-03-01 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 57014,'1983-Y59','1983-03-02 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',91 from dual union all
select 57015,'1983-Y60','1983-03-02 15:45:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',95 from dual union all
select 57016,'1983-Y61','1983-03-02 17:05:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',96 from dual union all
select 57017,'1983-Y62','1983-03-02 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',86 from dual union all
select 57018,'1983-Y63','1983-03-03 19:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',85 from dual union all
select 57019,'1983-Y64','1983-03-04 04:00:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',95 from dual union all
select 5702,'1959-A240','1959-12-17 20:11:00','military missile','success',843,'562Art Btn 2 Bat B','Service firing test','','','Test',443,'',3 from dual union all
select 57020,'1983-Y65','1983-03-04 18:40:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',83 from dual union all
select 57021,'1983-Y66','1983-03-05 18:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',84 from dual union all
select 57022,'1983-Y67','1983-03-06 18:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',83 from dual union all
select 57023,'1983-Y68','1983-03-07 18:50:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',85 from dual union all
select 57024,'1983-Y69','1983-03-08 15:15:00','suborbital rocket','unknown',1284,'TU1-0380 PWN-12A','CONDOR Sphere','','','Aeron',322,'',82 from dual union all
select 57025,'1983-Y70','1983-03-08 17:16:00','suborbital rocket','success',1281,'NASA 15.218UE','CONDOR PSU E1','','','Plasma',322,'',80 from dual union all
select 57026,'1983-Y71','1983-03-08 20:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',90 from dual union all
select 57027,'1983-Y72','1983-03-08 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',87 from dual union all
select 57028,'1983-Y73','1983-03-08 22:00:00','suborbital rocket','failure',1281,'NASA 15.223UE','CONDOR PSU E2','','','Plasma',322,'',50 from dual union all
select 57029,'1983-Y74','1983-03-08 23:27:03','suborbital rocket','success',1281,'NASA 15.221UE','CONDOR PSU E3','','','Plasma',322,'',80 from dual union all
select 5703,'1959-A241','1959-06-24 00:00:00','military missile','failure',843,'','Cell launcher test','','','Test',1852,'',10 from dual union all
select 57030,'1983-Y75','1983-03-09 01:00:00','suborbital rocket','success',1281,'NASA 15.220UE','CONDOR PSU E4','','','Plasma',322,'',80 from dual union all
select 57031,'1983-Y76','1983-03-09 02:30:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',88 from dual union all
select 57032,'1983-Y77','1983-03-09 02:38:00','suborbital rocket','unknown',1284,'TU1-0358 PWN-12A','CONDOR Sphere','','','Aeron',322,'',88 from dual union all
select 57033,'1983-Y78','1983-03-09 03:52:00','suborbital rocket','unknown',1284,'TU1-0389 PWN-11A','CONDOR Datasonde','','','Aeron',322,'',81 from dual union all
select 57034,'1983-Y79','1983-03-09 05:00:00','suborbital rocket','unknown',1284,'TU1-0382 PWN-12A','CONDOR Sphere','','','Aeron',322,'',82 from dual union all
select 57035,'1983-Y80','1983-03-09 06:10:00','suborbital rocket','unknown',1284,'TU1-0385 PWN-12A','CONDOR Sphere','','','Aeron',322,'',88 from dual union all
select 57036,'1983-Y81','1983-03-09 07:46:00','suborbital rocket','failure',1284,'TU1-0390 PWN-11A','CONDOR Datasonde','','','Aeron',322,'',80 from dual union all
select 57037,'1983-Y82','1983-03-09 08:20:00','suborbital rocket','unknown',1284,'TU1-0386 PWN-12A','CONDOR Sphere','','','Aeron',322,'',88 from dual union all
select 57038,'1983-Y83','1983-03-09 09:20:00','suborbital rocket','unknown',1284,'TU1-0361 PWN-11A','CONDOR Datasonde','','','Aeron',322,'',85 from dual union all
select 57039,'1983-Y84','1983-03-09 10:11:00','suborbital rocket','success',1281,'NASA 15.219UE','CONDOR PSU E5','','','Plasma',322,'',80 from dual union all
select 5704,'1959-A242','1959-12-19 00:00:00','military missile','failure',843,'','Cell launcher test','','','Test',1852,'',10 from dual union all
select 57040,'1983-Y85','1983-03-09 12:00:00','suborbital rocket','success',1281,'NASA 15.222UE','CONDOR PSU E6','','','Plasma',322,'',80 from dual union all
select 57041,'1983-Y86','1983-03-09 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',83 from dual union all
select 57042,'1983-Y87','1983-03-09 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',80 from dual union all
select 57043,'1983-Y88','1983-03-09 19:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',82 from dual union all
select 57044,'1983-Y89','1983-03-10 17:49:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 57045,'1983-Y90','1983-03-10 18:35:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',85 from dual union all
select 57046,'1983-Y91','1983-03-11 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',85 from dual union all
select 57047,'1983-Y92','1983-03-11 20:20:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',86 from dual union all
select 57048,'1983-Y93','1983-03-13 18:25:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',85 from dual union all
select 57049,'1983-Y94','1983-03-14 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',87 from dual union all
select 5705,'1959-A243','1959-12-01 00:00:00','military missile','success',843,'','XM30 No.205','','','Test',1895,'',10 from dual union all
select 57050,'1983-Y95','1983-03-15 18:30:00','suborbital rocket','unknown',725,'','','','','Aeron',1381,'SHIR',84 from dual union all
select 57051,'1983-Y96','1983-03-15 20:05:00','suborbital rocket','unknown',725,'','','','','Aeron',1010,'',93 from dual union all
select 57052,'1983-Y97','1983-03-16 10:40:00','suborbital rocket','unknown',725,'','','','','Aeron',636,'',89 from dual union all
select 57053,'1983-Y98','1983-03-16 14:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',84 from dual union all
select 57054,'1983-Y99','1983-03-16 14:20:00','suborbital rocket','unknown',725,'','','','','Aeron',961,'',88 from dual union all
select 57055,'1984-001','1984-01-05 20:09:00','orbital','success',680,'47119-529','Kosmos-1522/1529','Strela-1M','','Sat',457,'',null from dual union all
select 57056,'1984-002','1984-01-11 12:20:00','orbital','success',1238,'78012  -787','Kosmos-1530','Zenit-6','','Sat',475,'',null from dual union all
select 57057,'1984-003','1984-01-11 18:08:00','orbital','success',680,'65039-546','Kosmos-1531','Parus','','Sat',457,'',null from dual union all
select 57058,'1984-004','1984-01-13 14:40:00','orbital','success',1238,'78012  -806     *','Kosmos-1532','Kobal''t','','Sat',473,'',null from dual union all
select 57059,'1984-005','1984-01-23 07:58:00','orbital','success',825,'N-12(F)','BS 2A','BS 2A','','Sat',1474,'',null from dual union all
select 5706,'1959-A244','1959-02-26 00:00:00','military missile','success',843,'','Field qual','','','Test',1895,'',10 from dual union all
select 57060,'1984-006','1984-01-26 08:50:00','orbital','success',1242,'76032 -791 (-003)','Kosmos-1533','Zenit-6','','Sat',562,'',null from dual union all
select 57061,'1984-007','1984-01-26 12:00:00','orbital','success',680,'53793-469','Kosmos-1534','Vektor','','Sat',457,'',null from dual union all
select 57062,'1984-008','1984-01-29 12:25:00','orbital','success',308,'Y1       (9)','STW','STW-1','','Sat',1904,'',null from dual union all
select 57063,'1984-009','1984-01-31 03:08:01','orbital','success',1408,'34D-10 (05D-1)','OPS 0441','VORTEX 4','','Sat',274,'',null from dual union all
select 57064,'1984-010','1984-02-02 17:38:00','orbital','success',680,'47139-532','Kosmos-1535','Parus','','Sat',456,'',null from dual union all
select 57065,'1984-011','1984-02-03 13:00:00','orbital','success',1243,'STS-11/41-B','PAM-D Deploy/MMU','OV-099 PAM deploy/FF','','Sat',921,'MLP2',null from dual union all
select 57066,'1984-012','1984-02-05 18:44:00','orbital','success',170,'6003H','OPS 8737    [NOSS7]','PARCAE 7','','Sat',1775,'',null from dual union all
select 57067,'1984-013','1984-02-08 09:23:00','orbital','success',1448,'45014-706','Kosmos-1536','Tselina-D','','Sat',470,'',null from dual union all
select 57068,'1984-014','1984-02-08 12:07:26','orbital','success',1238,'Yu15000-357','Soyuz T-10','Soyuz 7K-ST No. 15L','','Sat',562,'',null from dual;
insert into LAUNCH
select 57069,'1984-015','1984-02-14 08:00:00','orbital','success',819,'M-3S-4','Ohzora','EXOS-C','','Sat',837,'',null from dual union all
select 5707,'1959-A245','1959-07-10 00:00:00','military missile','success',843,'','Field qual','','','Test',433,'',10 from dual union all
select 57070,'1984-016','1984-02-15 08:46:00','orbital','success',956,'318-02','Raduga','Gran'' No. 25L','','Sat',553,'',null from dual union all
select 57071,'1984-017','1984-02-16 08:15:00','orbital','success',1238,'76032  -126','Kosmos-1537','Resurs-F1 17F41 No. 27','','Sat',473,'',null from dual union all
select 57072,'1984-018','1984-02-21 06:46:05','orbital','success',1238,'Yu15000-367','Progress-19','Progress 7K-TG No. 120','','Sat',562,'',null from dual union all
select 57073,'1984-019','1984-02-21 15:36:00','orbital','success',680,'65039-537','Kosmos-1538','Strela-2M','','Sat',456,'',null from dual union all
select 57074,'1984-020','1984-02-28 13:59:59','orbital','success',1238,'78012  -789','Kosmos-1539','Kobal''t','','Sat',462,'',null from dual union all
select 57075,'1984-021','1984-03-01 17:59:00','orbital','success',391,'D174','Landsat 5','Landsat D''','','Sat',1724,'',null from dual union all
select 57076,'1984-022','1984-03-02 03:54:00','orbital','success',956,'316-01','Kosmos-1540','Geizer No. 12L','','Sat',554,'',null from dual union all
select 57077,'1984-023','1984-03-05 00:50:03','orbital','success',91,'L8','Intelsat 508','INTELSAT V F8','','Sat',344,'',null from dual union all
select 57078,'1984-024','1984-03-06 17:10:18','orbital','success',805,'76054  -649','Kosmos-1541','Oko','','Sat',462,'',null from dual union all
select 57079,'1984-025','1984-03-07 08:00:00','orbital','success',1242,'76032  -792 (-004)','Kosmos-1542','Zenit-6','','Sat',562,'',null from dual union all
select 5708,'1959-A246','1959-07-31 00:00:00','military missile','success',843,'','Field qual','','','Test',1895,'',10 from dual union all
select 57080,'1984-026','1984-03-10 17:00:00','orbital','success',1238,'Ts15000-309','Kosmos-1543','Efir No. 1','','Sat',473,'',null from dual union all
select 57081,'1984-027','1984-03-15 17:05:00','orbital','success',1448,'67014-705','Kosmos-1544','Tselina-D','','Sat',470,'',null from dual union all
select 57082,'1984-028','1984-03-16 14:00:00','orbital','success',956,'322-01','Ekran','Ekran No. 26L','','Sat',553,'',null from dual union all
select 57083,'1984-029','1984-03-16 23:29:59','orbital','success',803,'78039  -637','Molniya-1','Molniya-1!No. 51','','Sat',473,'',null from dual union all
select 57084,'1984-030','1984-03-21 11:05:00','orbital','success',1241,'77024  -779','Kosmos-1545','Zenit-6','','Sat',475,'',null from dual union all
select 57085,'1984-031','1984-03-29 05:52:56','orbital','success',956,'319-02','Kosmos-1546','Oko-S F1','','Sat',554,'',null from dual union all
select 57086,'1984-032','1984-04-03 13:08:42','orbital','success',1238,'Sc15000-395','Soyuz T-11','Soyuz 7K-ST No. 17L','','Sat',562,'',null from dual union all
select 57087,'1984-033','1984-04-04 01:40:04','orbital','success',803,'77049  -643','Kosmos-1547','Oko','','Sat',462,'',null from dual union all
select 57088,'1984-034','1984-04-06 13:58:00','orbital','success',1243,'STS-13/41-C','SMM Repair/LDEF','OV-099 LEO deploy/Rend','','Sat',921,'MLP1',null from dual union all
select 57089,'1984-035','1984-04-08 11:20:00','orbital','success',308,'Y2       (10)','STW','STW','','Sat',1904,'',null from dual union all
select 5709,'1959-A247','1959-01-01 00:00:00','suborbital rocket','failure',927,'','Pogo Target Test','','','Test',1895,'',0 from dual union all
select 57090,'1984-036','1984-04-10 14:00:00','orbital','success',1238,'76032  -808','Kosmos-1548','Kobal''t','','Sat',473,'',null from dual union all
select 57091,'1984-037','1984-04-14 16:52:02','orbital','success',1408,'34D-11 (05D-2)','DSP 12','DSP 12','','Sat',274,'',null from dual union all
select 57092,'1984-038','1984-04-15 08:12:53','orbital','success',1242,'Yu15000-005','Progress-20','Progress 7K-TG No. 121','','Sat',562,'',null from dual union all
select 57093,'1984-039','1984-04-17 18:45:00','orbital','success',1403,'24B-28 (3B-67)','KH-8A','GAMBIT-3 Mission 4354','','Sat',1778,'',null from dual union all
select 57094,'1984-040','1984-04-19 11:40:00','orbital','success',1238,'76032  -788','Kosmos-1549','Zenit-6','','Sat',462,'',null from dual union all
select 57095,'1984-041','1984-04-22 04:21:00','orbital','success',956,'312-02','Gorizont','Gorizont No. 19L','','Sat',553,'',null from dual union all
select 57096,'1984-042','1984-05-07 22:47:15','orbital','success',1238,'Sc15000-399','Progress-21','Progress 7K-TG No. 116','','Sat',562,'',null from dual union all
select 57097,'1984-043','1984-05-11 06:19:00','orbital','success',680,'53739-533','Kosmos-1550','Parus','','Sat',457,'',null from dual union all
select 57098,'1984-044','1984-05-11 13:00:00','orbital','success',1238,'76032  -811','Kosmos-1551','Zenit-6','','Sat',473,'',null from dual union all
select 57099,'1984-045','1984-05-14 14:00:00','orbital','success',1238,'Yu15000-364','Kosmos-1552','Yantar''-4KS1','','Sat',562,'',null from dual union all
select 571,'1945-S47','1945-01-04 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5710,'1959-A248','1959-01-01 00:00:00','atmospheric rocket','success',926,'','Pogo Hi Type IID3a','','','Target',1895,'',24 from dual union all
select 57100,'1984-046','1984-05-17 14:43:26','orbital','success',680,'47139-538','Kosmos-1553','Tsikada','','Sat',456,'',null from dual union all
select 57101,'1984-047','1984-05-19 15:10:42','orbital','success',958,'323-02','Kosmos-1554/1556','Uragan No. 16L','','Sat',554,'',null from dual union all
select 57102,'1984-048','1984-05-22 08:30:00','orbital','success',1238,'77046  -050','Kosmos-1557','Fram','','Sat',475,'',null from dual union all
select 57103,'1984-049','1984-05-23 01:33:29','orbital','success',91,'V9  L9','Spacenet 1','Spacenet F1','','Sat',344,'',null from dual union all
select 57104,'1984-050','1984-05-25 11:30:00','orbital','success',1238,'76032  -810','Kosmos-1558','Kobal''t','','Sat',462,'',null from dual union all
select 57105,'1984-051','1984-05-28 14:12:52','orbital','success',1238,'Sc15000-405','Progress-22','Progress 7K-TG No. 122','','Sat',562,'',null from dual union all
select 57106,'1984-052','1984-05-28 21:52:00','orbital','success',680,'65039-531','Kosmos-1559/1566','Strela-1M','','Sat',457,'',null from dual union all
select 57107,'1984-053','1984-05-30 18:46:00','orbital','success',1446,'45067-402','Kosmos-1567','US-PM1','','Sat',589,'',null from dual union all
select 57108,'1984-054','1984-06-01 13:50:00','orbital','success',1238,'77046  -148','Kosmos-1568','Zenit-6','','Sat',475,'',null from dual union all
select 57109,'1984-055','1984-06-06 15:33:32','orbital','success',803,'77024  -657','Kosmos-1569','Oko','','Sat',462,'',null from dual union all
select 5711,'1959-A249','1959-01-01 00:00:00','atmospheric rocket','success',926,'','Pogo Hi Type IIF3a','','','Target',1895,'',24 from dual union all
select 57110,'1984-056','1984-06-08 11:28:00','orbital','success',680,'53739-542','Kosmos-1570','Strela-2M','','Sat',456,'',null from dual union all
select 57111,'1984-057','1984-06-09 23:03:00','orbital','success',169,'AC-62','Intelsat 509','INTELSAT V F9','','Sat',269,'',null from dual union all
select 57112,'1984-058','1984-06-11 08:40:00','orbital','success',1238,'76032  -795','Kosmos-1571','Oblik','','Sat',480,'',null from dual union all
select 57113,'1984-059','1984-06-13 11:37:00','orbital','success',159,'42E','USA 1 (GPS 9)','Navstar GPS SVN 9','','Sat',1776,'',null from dual union all
select 57114,'1984-060','1984-06-15 08:20:00','orbital','success',1238,'76032  -793','Kosmos-1572','Resurs-F1 17F41 No. 26','','Sat',473,'',null from dual union all
select 57115,'1984-061','1984-06-19 10:55:00','orbital','success',1238,'76032  -796','Kosmos-1573','Zenit-6','','Sat',462,'',null from dual union all
select 57116,'1984-062','1984-06-21 19:40:03','orbital','success',680,'53719-521','Kosmos-1574','Nadezhda 11F643N No. 531','','Sat',457,'',null from dual union all
select 57117,'1984-063','1984-06-22 00:20:00','orbital','success',956,'319-01','Raduga','Gran'' No. 27L','','Sat',553,'',null from dual union all
select 57118,'1984-064','1984-06-22 07:40:00','orbital','success',1238,'76032  -790','Kosmos-1575','Resurs-F1 17F41 No. 48','','Sat',473,'',null from dual union all
select 57119,'1984-065','1984-06-25 18:47:00','orbital','success',1406,'34D-4  (04D-1)','USA 2/3 [HEX SV19]','HEXAGON SV-19','','Sat',1777,'',null from dual union all
select 5712,'1959-A25','1959-02-10 21:30:00','military missile','success',843,'','Cold weather test','','','Test',428,'',10 from dual union all
select 57120,'1984-066','1984-06-26 15:35:00','orbital','success',1238,'76032  -807','Kosmos-1576','Kobal''t','','Sat',473,'',null from dual union all
select 57121,'1984-067','1984-06-27 04:59:00','orbital','success',680,'65039-549','Kosmos-1577','Parus','','Sat',457,'',null from dual union all
select 57122,'1984-068','1984-06-28 13:10:00','orbital','success',680,'47134-133','Kosmos-1578','Taifun-1B','','Sat',622,'',null from dual union all
select 57123,'1984-069','1984-06-29 00:21:00','orbital','success',1446,'67067-401','Kosmos-1579','US-A reactor','','Sat',589,'',null from dual union all
select 57124,'1984-070','1984-06-29 15:00:00','orbital','success',1238,'77046  -306','Kosmos-1580','Oblik No. 793','','Sat',475,'',null from dual union all
select 57125,'1984-071','1984-07-03 21:30:42','orbital','success',803,'78012  -659','Kosmos-1581','Oko','','Sat',475,'',null from dual union all
select 57126,'1984-072','1984-07-05 03:35:00','orbital','success',1448,'67074-702','Meteor-2','Meteor-2 No. 16','','Sat',470,'',null from dual union all
select 57127,'1984-073','1984-07-17 17:40:54','orbital','success',1242,'Yu15000-006','Soyuz T-12','Soyuz 7K-ST No. 18L','','Sat',562,'',null from dual union all
select 57128,'1984-074','1984-07-19 08:30:00','orbital','success',1238,'77046  -149','Kosmos-1582','Resurs-F1 17F41 No. 49','','Sat',475,'',null from dual union all
select 57129,'1984-075','1984-07-24 12:40:00','orbital','success',1238,'77046  -150','Kosmos-1583','Oblik','','Sat',462,'',null from dual union all
select 5713,'1959-A250','1959-01-01 00:00:00','atmospheric rocket','success',926,'','Pogo Hi Type IIF3c','','','Target',1895,'',24 from dual union all
select 57130,'1984-076','1984-07-27 08:59:59','orbital','success',1238,'77046  -307','Kosmos-1584','Oblik (Priroda)','','Sat',473,'',null from dual union all
select 57131,'1984-077','1984-07-31 12:29:54','orbital','success',1238,'77046  -823','Kosmos-1585','Kobal''t','','Sat',480,'',null from dual union all
select 57132,'1984-078','1984-08-01 21:37:00','orbital','success',956,'324-01','Gorizont','Gorizont No. 20L','','Sat',554,'',null from dual union all
select 57133,'1984-079','1984-08-02 08:38:16','orbital','success',803,'76032  -667','Kosmos-1586','Oko','','Sat',462,'',null from dual union all
select 57134,'1984-080','1984-08-02 20:30:00','orbital','success',825,'N-13(F)','GMS 3','GMS 3','','Sat',1474,'',null from dual union all
select 57135,'1984-081','1984-08-04 13:32:54','orbital','success',93,'V10 L12','ECS 2/Telecom 1A','ECS 2','','Sat',344,'',null from dual union all
select 57136,'1984-082','1984-08-06 14:00:00','orbital','success',1238,'76032  -794','Kosmos-1587','Oblik','','Sat',473,'',null from dual union all
select 57137,'1984-083','1984-08-07 22:50:34','orbital','success',1446,'51099-807','Kosmos-1588','US-PM1','','Sat',589,'',null from dual union all
select 57138,'1984-084','1984-08-08 12:08:00','orbital','success',1448,'45042-301','Kosmos-1589','Musson No. 15','','Sat',470,'',null from dual union all
select 57139,'1984-085','1984-08-10 00:03:58','orbital','success',805,'76032  -666 ','Molniya-1','Molniya-1!No. 53','','Sat',473,'',null from dual union all
select 5714,'1959-A251','1959-01-01 00:00:00','atmospheric rocket','success',926,'','Pogo Hi Type IIE3c','','','Target',1895,'',24 from dual union all
select 57140,'1984-086','1984-08-14 06:28:15','orbital','success',1238,'Sc15000-411','Progress-23','Progress 7K-TG No. 124','','Sat',480,'',null from dual union all
select 57141,'1984-087','1984-08-16 09:50:00','orbital','success',1238,'77046  -309','Kosmos-1590','Resurs-F1 17F41 No. 50','','Sat',473,'',null from dual union all
select 57142,'1984-088','1984-08-16 14:48:00','orbital','success',394,'D175','AMPTE','AMPTE-CCE','','Sat',233,'',null from dual union all
select 57143,'1984-089','1984-08-24 08:26:59','orbital','success',805,'78012  -661','Molniya-1','Molniya-1!No. 54','','Sat',475,'',null from dual union all
select 57144,'1984-090','1984-08-24 19:50:00','orbital','success',956,'324-02','Ekran','Ekran No. 27L','','Sat',553,'',null from dual union all
select 57145,'1984-091','1984-08-28 18:03:00','orbital','success',1405,'34B-4  (3B-64)','USA 4        [JS7]','SDS 5','','Sat',1778,'',null from dual union all
select 57146,'1984-092','1984-08-30 10:10:00','orbital','success',1238,'77046  -151','Kosmos-1591','Resurs-F1 17F41 No. 51','','Sat',475,'',null from dual union all
select 57147,'1984-093','1984-08-30 12:41:50','orbital','success',1243,'STS-14/41-D','PAM-D/Leasat Deploy','OV-103 PAM deploy','','Sat',921,'MLP2',null from dual union all
select 57148,'1984-094','1984-09-04 10:20:00','orbital','success',1238,'77046  -308','Kosmos-1592','Oblik','','Sat',462,'',null from dual union all
select 57149,'1984-095','1984-09-04 15:49:53','orbital','success',958,'320-01','Kosmos-1593/1595','Uragan No. 18L','','Sat',554,'',null from dual union all
select 5715,'1959-A252','1959-11-28 00:00:00','military missile','failure',992,'S3-8','','','','OT',636,'',20 from dual union all
select 57150,'1984-096','1984-09-07 19:12:58','orbital','success',803,'76054  -651','Kosmos-1596','Oko','','Sat',462,'',null from dual union all
select 57151,'1984-097','1984-09-08 21:41:00','orbital','success',159,'14E','USA 5 (GPS 10)','Navstar GPS SVN 10','','Sat',1776,'',null from dual union all
select 57152,'1984-098','1984-09-12 05:44:00','orbital','success',301,'Y3       (11)','FSW','FSW No. 7','','Sat',810,'',null from dual union all
select 57153,'1984-099','1984-09-13 10:25:00','orbital','success',1238,'77046  -051','Kosmos-1597','Fram','','Sat',475,'',null from dual union all
select 57154,'1984-100','1984-09-13 15:54:08','orbital','success',680,'53739-539','Kosmos-1598','Parus','','Sat',456,'',null from dual union all
select 57155,'1984-101','1984-09-21 22:18:00','orbital','success',393,'D176','Galaxy 3','Galaxy III','','Sat',234,'',null from dual union all
select 57156,'1984-102','1984-09-25 14:30:00','orbital','success',1238,'77046  -902','Kosmos-1599','Kobal''t','','Sat',475,'',null from dual union all
select 57157,'1984-103','1984-09-27 08:10:00','orbital','success',1238,'77046  -320 (310)','Kosmos-1600','Oblik','','Sat',480,'',null from dual union all
select 57158,'1984-104','1984-09-27 09:30:00','orbital','success',680,'53796-180','Kosmos-1601','Romb','','Sat',457,'',null from dual;
insert into LAUNCH
select 57159,'1984-105','1984-09-28 06:00:00','orbital','success',1448,'51032-311','Kosmos-1602','Okean-OE NKhM No. 2','','Sat',470,'',null from dual union all
select 5716,'1959-A253','1959-12-25 00:00:00','military missile','failure',992,'S4-8','','','','OT',636,'',0 from dual union all
select 57160,'1984-106','1984-09-28 14:00:00','orbital','success',958,'327-02','Kosmos-1603','Tselina-2','','Sat',553,'',null from dual union all
select 57161,'1984-107','1984-10-04 19:49:13','orbital','success',803,'78012  -662','Kosmos-1604','Oko','','Sat',462,'',null from dual union all
select 57162,'1984-108','1984-10-05 11:03:00','orbital','success',1243,'STS-17/41-G','ERBS/OSTA-3','OV-099 LEO deploy','','Sat',921,'MLP1',null from dual union all
select 57163,'1984-109','1984-10-11 14:43:00','orbital','success',680,'53734-125','Kosmos-1605','Parus','','Sat',457,'',null from dual union all
select 57164,'1984-110','1984-10-12 01:43:34','orbital','success',1135,'S208C','Nova 3','Nova 3','','Sat',1779,'',null from dual union all
select 57165,'1984-111','1984-10-18 17:46:00','orbital','success',1448,'51074-701','Kosmos-1606','Tselina-D','','Sat',470,'',null from dual union all
select 57166,'1984-112','1984-10-31 12:29:00','orbital','success',1446,'51083-625','Kosmos-1607','US-A reactor','','Sat',589,'',null from dual union all
select 57167,'1984-113','1984-11-08 12:15:00','orbital','success',1243,'STS-19/51-A','HS-376 Rescue/PAM-D','OV-103 PAM deploy/Rend','','Sat',921,'MLP2',null from dual union all
select 57168,'1984-114','1984-11-10 01:14:18','orbital','success',93,'V11 L13','Spacenet 2/Marecs 2','Spacenet F2','','Sat',344,'',null from dual union all
select 57169,'1984-115','1984-11-14 00:34:00','orbital','success',390,'D177','NATO 3D','NATO 3D','','Sat',233,'',null from dual union all
select 5717,'1959-A254','1959-07-09 00:00:00','atmospheric rocket','success',512,'','Jaguar R&D','','','Test',1849,'',20 from dual union all
select 57170,'1984-116','1984-11-14 07:40:00','orbital','success',1238,'Yu15000-352','Kosmos-1608','Siluet No. 4','','Sat',480,'',null from dual union all
select 57171,'1984-117','1984-11-14 12:20:00','orbital','success',1238,'77046  -311','Kosmos-1609','Oblik','','Sat',475,'',null from dual union all
select 57172,'1984-118','1984-11-15 06:40:00','orbital','success',680,'47139-124','Kosmos-1610','Parus','','Sat',456,'',null from dual union all
select 57173,'1984-119','1984-11-21 10:30:00','orbital','success',1238,'78056  -825','Kosmos-1611','Kobal''t','','Sat',480,'',null from dual union all
select 57174,'1984-120','1984-11-27 14:22:00','orbital','failure',1448,'51014-707','Kosmos-1612','Meteor-3 No. 1','','Sat',469,'',null from dual union all
select 57175,'1984-121','1984-11-29 14:00:00','orbital','success',1241,'78024  -780','Kosmos-1613','Oblik','','Sat',475,'',null from dual union all
select 57176,'1984-122','1984-12-04 18:03:00','orbital','success',1406,'34D-6  (04D-4)','USA 6       [KH11-6]','KH11-6 KENNAN','','Sat',1777,'',null from dual union all
select 57177,'1984-123','1984-12-12 10:42:00','orbital','success',155,'39E','NOAA 9','NOAA F','','Sat',1776,'',null from dual union all
select 57178,'1984-124','1984-12-14 20:40:59','orbital','success',805,'76032  -672','Molniya-1','Molniya-1!No. 55','','Sat',473,'',null from dual union all
select 57179,'1984-125','1984-12-15 09:16:24','deep space','success',954,'329-01','Vega-1','5VK No. 901','','Sat',553,'',null from dual union all
select 5718,'1959-A255','1959-08-21 00:00:00','atmospheric rocket','success',512,'','Jaguar R&D','','','Test',1849,'',20 from dual union all
select 57180,'1984-126','1984-12-19 03:55:00','orbital','success',647,'','Kosmos-1614','BOR-4 No. 406','','Sat',622,'',null from dual union all
select 57181,'1984-127','1984-12-20 13:00:00','orbital','success',680,'47134-127','Kosmos-1615','Taifun-1B','','Sat',457,'',null from dual union all
select 57182,'1984-128','1984-12-21 09:13:52','deep space','success',954,'325-02','Vega-2','5VK No. 902','','Sat',554,'',null from dual union all
select 57183,'1984-129','1984-12-22 00:02:03','orbital','success',1408,'34D-13 (05D-3)','USA 7        [DSP 6]','DSP 6R','','Sat',274,'',null from dual union all
select 57184,'1984-A01','1984-01-04 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',40 from dual union all
select 57185,'1984-A02','1984-01-09 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',47 from dual union all
select 57186,'1984-A03','1984-01-11 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',43 from dual union all
select 57187,'1984-A04','1984-01-16 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',46 from dual union all
select 57188,'1984-A05','1984-01-18 12:45:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',49 from dual union all
select 57189,'1984-A06','1984-01-18 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',43 from dual union all
select 5719,'1959-A256','1959-11-18 00:00:00','atmospheric rocket','success',512,'','Jaguar R&D','','','Test',1849,'',20 from dual union all
select 57190,'1984-A07','1984-01-30 12:57:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',44 from dual union all
select 57191,'1984-A08','1984-01-31 02:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',49 from dual union all
select 57192,'1984-A09','1984-02-13 22:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',44 from dual union all
select 57193,'1984-A10','1984-02-28 21:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',48 from dual union all
select 57194,'1984-A11','1984-03-05 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',49 from dual union all
select 57195,'1984-A12','1984-03-09 12:51:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',44 from dual union all
select 57196,'1984-A13','1984-03-13 02:31:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',49 from dual union all
select 57197,'1984-A14','1984-03-28 21:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',44 from dual union all
select 57198,'1984-A15','1984-04-05 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',45 from dual union all
select 57199,'1984-A16','1984-04-18 02:10:00','suborbital rocket','success',761,'MT135P-R527','','','','Meteo',1332,'',49 from dual union all
select 572,'1945-S48','1945-01-04 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5720,'1959-A26','1959-02-12 10:58:00','military missile','success',1214,'N-69D N-3312-2','','','','Test',224,'',10 from dual union all
select 57200,'1984-A17','1984-04-30 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',48 from dual union all
select 57201,'1984-A18','1984-05-04 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',48 from dual union all
select 57202,'1984-A19','1984-05-08 19:00:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',48 from dual union all
select 57203,'1984-A20','1984-05-09 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',48 from dual union all
select 57204,'1984-A21','1984-05-14 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',49 from dual union all
select 57205,'1984-A22','1984-05-21 20:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',37 from dual union all
select 57206,'1984-A23','1984-05-28 19:31:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',48 from dual union all
select 57207,'1984-A24','1984-06-08 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',39 from dual union all
select 57208,'1984-A25','1984-06-12 19:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',43 from dual union all
select 57209,'1984-A26','1984-06-27 19:54:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',49 from dual union all
select 5721,'1959-A27','1959-02-12 19:48:00','military missile','success',843,'','Cold weather test','','','Test',428,'',10 from dual union all
select 57210,'1984-A27','1984-07-23 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',46 from dual union all
select 57211,'1984-A28','1984-08-03 00:00:00','suborbital rocket','failure',435,'','','','','Test',1337,'',0 from dual union all
select 57212,'1984-A29','1984-08-08 15:48:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',42 from dual union all
select 57213,'1984-A30','1984-08-24 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',48 from dual union all
select 57214,'1984-A31','1984-09-07 21:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',49 from dual union all
select 57215,'1984-A32','1984-10-05 11:03:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',0 from dual union all
select 57216,'1984-A33','1984-10-26 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',47 from dual union all
select 57217,'1984-A34','1984-10-29 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',47 from dual union all
select 57218,'1984-A35','1984-11-02 21:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',48 from dual union all
select 57219,'1984-A36','1984-11-16 12:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',47 from dual union all
select 5722,'1959-A28','1959-02-13 00:00:00','military missile','success',255,'624-XY12','','','','OT',293,'',20 from dual union all
select 57220,'1984-A37','1984-12-06 00:00:00','military missile','failure',1453,'49256-42-361','15A35 UBP-24','','','OT',508,'',0 from dual union all
select 57221,'1984-A38','1984-12-18 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',48 from dual union all
select 57222,'1984-A39','1984-12-21 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',43 from dual union all
select 57223,'1984-A40','1984-12-01 00:00:00','atmospheric rocket','success',542,'Sandia','Stores parachute 1','','','Test',1490,'',12 from dual union all
select 57224,'1984-A41','1984-05-25 12:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1361,'',42 from dual union all
select 57225,'1984-M01','1984-01-17 00:00:00','suborbital rocket','success',1108,'ST-735-1','SRB Sep test','','','Test',833,'',50 from dual union all
select 57226,'1984-M02','1984-02-27 16:00:01','suborbital rocket','success',860,'NASA 31.43UU','CWAS 8','','','Aeronomy',428,'',73 from dual union all
select 57227,'1984-M03','1984-01-31 21:05:36','suborbital rocket','success',891,'DLR K-O-129 DROPMAS','DFVLR Strafam 1/4','','','Aeron',417,'',57 from dual union all
select 57228,'1984-M04','1984-05-23 12:45:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1361,'',64 from dual union all
select 57229,'1984-M05','1984-05-23 12:50:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',320,'',57 from dual union all
select 5723,'1959-A29','1959-02-16 17:00:00','military missile','failure',1053,'CC-1011','','','','OT 46AG',1854,'',10 from dual union all
select 57230,'1984-M06','1984-05-23 12:48:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1467,'',60 from dual union all
select 57231,'1984-M07','1984-05-23 13:54:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1467,'',61 from dual union all
select 57232,'1984-M08','1984-05-25 12:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1467,'',68 from dual union all
select 57233,'1984-M09','1984-05-25 13:35:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',320,'',69 from dual union all
select 57234,'1984-M10','1984-05-25 13:36:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1361,'',67 from dual union all
select 57235,'1984-M100','1984-05-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57236,'1984-M101','1984-05-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57237,'1984-M102','1984-06-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57238,'1984-M103','1984-06-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57239,'1984-M104','1984-07-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5724,'1959-A30','1959-02-16 21:03:00','military missile','success',445,'S/N 1','Alpha Draco','','','Test',225,'',30 from dual union all
select 57240,'1984-M11','1984-05-27 12:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1361,'',60 from dual union all
select 57241,'1984-M12','1984-05-27 12:34:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1467,'',64 from dual union all
select 57242,'1984-M13','1984-05-27 12:45:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',320,'',65 from dual union all
select 57243,'1984-M14','1984-05-27 13:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1361,'',64 from dual union all
select 57244,'1984-M15','1984-05-29 12:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1361,'',72 from dual union all
select 57245,'1984-M16','1984-05-29 12:37:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1467,'',66 from dual union all
select 57246,'1984-M17','1984-05-29 12:46:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',320,'',68 from dual union all
select 57247,'1984-M18','1984-05-31 12:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',320,'',59 from dual union all
select 57248,'1984-M19','1984-05-31 12:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1361,'',68 from dual;
insert into LAUNCH
select 57249,'1984-M20','1984-05-31 12:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1467,'',69 from dual union all
select 5725,'1959-A31','1959-02-19 18:04:00','military missile','success',843,'','Cold weather test','','','Test',428,'',10 from dual union all
select 57250,'1984-M21','1984-06-02 12:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1361,'',55 from dual union all
select 57251,'1984-M22','1984-06-02 12:40:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',320,'',62 from dual union all
select 57252,'1984-M23','1984-06-02 13:57:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1467,'',68 from dual union all
select 57253,'1984-M24','1984-06-04 12:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1467,'',72 from dual union all
select 57254,'1984-M25','1984-06-04 12:40:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1361,'',73 from dual union all
select 57255,'1984-M26','1984-06-04 13:39:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',320,'',60 from dual union all
select 57256,'1984-M27','1984-06-04 14:20:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',320,'',67 from dual union all
select 57257,'1984-M28','1984-06-06 12:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1361,'',61 from dual union all
select 57258,'1984-M29','1984-06-06 12:50:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 57259,'1984-M30','1984-06-08 12:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1467,'',58 from dual union all
select 5726,'1959-A32','1959-02-25 00:00:00','military missile','success',771,'Training #9','','','','Test',222,'',12 from dual union all
select 57260,'1984-M31','1984-06-08 12:33:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',320,'',64 from dual union all
select 57261,'1984-M32','1984-06-08 13:01:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1361,'',72 from dual union all
select 57262,'1984-M33','1984-06-10 12:40:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1361,'',72 from dual union all
select 57263,'1984-M34','1984-06-10 12:45:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',320,'',62 from dual union all
select 57264,'1984-M35','1984-06-10 13:01:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1467,'',60 from dual union all
select 57265,'1984-M36','1984-06-12 12:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',320,'',60 from dual union all
select 57266,'1984-M37','1984-06-12 12:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1361,'',72 from dual union all
select 57267,'1984-M38','1984-06-12 12:30:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',1467,'',59 from dual union all
select 57268,'1984-M39','1984-06-12 13:50:00','suborbital rocket','success',1023,'Eq. Wave Campaign I','Meteo Chaff','','','Meteo',320,'',66 from dual union all
select 57269,'1984-M40','1984-02-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5727,'1959-A33','1959-02-25 00:00:00','military missile','success',771,'Training #10','','','','Test',222,'',12 from dual union all
select 57270,'1984-M41','1984-02-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57271,'1984-M42','1984-02-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57272,'1984-M43','1984-02-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57273,'1984-M44','1984-02-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57274,'1984-M45','1984-03-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57275,'1984-M46','1984-03-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57276,'1984-M47','1984-03-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57277,'1984-M48','1984-03-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57278,'1984-M49','1984-04-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57279,'1984-M50','1984-04-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5728,'1959-A34','1959-02-25 00:00:00','military missile','success',771,'Training #11','','','','Test',222,'',12 from dual union all
select 57280,'1984-M51','1984-04-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57281,'1984-M52','1984-04-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57282,'1984-M53','1984-07-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57283,'1984-M54','1984-07-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57284,'1984-M55','1984-07-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57285,'1984-M56','1984-08-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57286,'1984-M57','1984-08-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57287,'1984-M58','1984-08-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57288,'1984-M59','1984-08-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57289,'1984-M60','1985-03-25 17:35:00','suborbital rocket','success',1284,'','ROCOZ-A 339','','','Aeron Ozone',330,'',75 from dual union all
select 5729,'1959-A35','1959-02-26 00:00:00','military missile','success',771,'Training #12','','','','Test',222,'',12 from dual union all
select 57290,'1984-M61','1985-03-27 17:35:00','suborbital rocket','success',1284,'','ROCOZ-A 340','','','Aeron Ozone',330,'',75 from dual union all
select 57291,'1984-M63','1985-04-05 10:36:00','suborbital rocket','success',1284,'','ROCOZ-A 342','','','Aeron Ozone',330,'',75 from dual union all
select 57292,'1984-M64','1985-04-06 10:31:00','suborbital rocket','success',1284,'','ROCOZ-A 343','','','Aeron Ozone',330,'',75 from dual union all
select 57293,'1984-M65','1985-04-13 17:55:00','suborbital rocket','success',1284,'','ROCOZ-A 344','','','Aeron Ozone',330,'',75 from dual union all
select 57294,'1984-M66','1985-04-15 15:19:00','suborbital rocket','success',1284,'','ROCOZ-A 345','','','Aeron Ozone',330,'',75 from dual union all
select 57295,'1984-M67','1984-10-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57296,'1984-M68','1984-10-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57297,'1984-M69','1984-11-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57298,'1984-M70','1984-11-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57299,'1984-M71','1984-11-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 573,'1945-S49','1945-01-04 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5730,'1959-A36','1959-02-27 19:16:00','military missile','failure',933,'AX-5','','','','Test',244,'',2 from dual union all
select 57300,'1984-M72','1984-11-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57301,'1984-M73','1984-12-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57302,'1984-M74','1984-12-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57303,'1984-M75','1984-12-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57304,'1984-M76','1984-12-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57305,'1984-M77','1984-02-23 12:00:00','suborbital rocket','success',1506,'','E field','','','Ionos/Aeron',1821,'',75 from dual union all
select 57306,'1984-M78','1984-02-09 17:30:00','suborbital rocket','success',1284,'','ROCOZ-A 328','','','Aeron Ozone',1821,'',75 from dual union all
select 57307,'1984-M79','1984-02-09 17:30:00','suborbital rocket','success',1284,'','ROCOZ-A 329','','','Aeron Ozone',1821,'',75 from dual union all
select 57308,'1984-M80','1984-08-15 16:11:00','suborbital rocket','success',1284,'','ROCOZ-A 330','','','Aeron Ozone',1821,'',75 from dual union all
select 57309,'1984-M81','1984-09-18 14:58:00','suborbital rocket','success',1284,'','ROCOZ-A 331','','','Aeron Ozone',1821,'',75 from dual union all
select 5731,'1959-A37','1959-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 57310,'1984-M82','1984-09-18 15:05:00','suborbital rocket','success',1284,'','ROCOZ-A 332','','','Aeron Ozone',1821,'',75 from dual union all
select 57311,'1984-M83','1984-09-18 16:21:00','suborbital rocket','success',1284,'','ROCOZ-A 333','','','Aeron Ozone',1821,'',75 from dual union all
select 57312,'1984-M84','1984-09-18 16:28:00','suborbital rocket','success',1284,'','ROCOZ-A 334','','','Aeron Ozone',1821,'',75 from dual union all
select 57313,'1984-M85','1984-10-22 16:30:00','suborbital rocket','success',1284,'','ROCOZ-A 335','','','Aeron Ozone',1821,'',75 from dual union all
select 57314,'1984-M86','1984-11-13 16:15:00','suborbital rocket','success',1284,'','ROCOZ-A 336','','','Aeron Ozone',1821,'',75 from dual union all
select 57315,'1984-M87','1985-04-04 15:23:00','suborbital rocket','success',1284,'','ROCOZ-A 341','','','Aeron Ozone',330,'',75 from dual union all
select 57316,'1984-M88','1984-08-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57317,'1984-M89','1984-09-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57318,'1984-M90','1984-09-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57319,'1984-M91','1984-09-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5732,'1959-A38','1959-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 57320,'1984-M92','1984-09-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57321,'1984-M93','1984-10-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57322,'1984-M94','1984-10-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57323,'1984-M95','1984-01-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57324,'1984-M96','1984-01-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57325,'1984-M97','1984-01-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57326,'1984-M98','1984-01-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57327,'1984-M99','1984-05-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 57328,'1984-S01','1984-01-27 16:25:00','military missile','success',914,'51099-801(9L)','15A11 UBP-2','GCh 15B99','','Test',540,'',1000 from dual union all
select 57329,'1984-S02','1984-10-31 18:35:00','military missile','success',744,'45026-31-510','15A15 UBP-7','GCh 15F154 (15F158U1 x1)','','OT',546,'',1000 from dual union all
select 5733,'1959-A39','1959-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 57330,'1984-S03','1984-01-31 19:15:00','suborbital rocket','success',860,'SSC S39/1','SSC SOAP/WINE','','','Aeronomy',412,'',149 from dual union all
select 57331,'1984-S04','1984-02-10 03:00:00','suborbital rocket','success',860,'SSC S39/2','SSC SOAP/WINE','','','Aeronomy',412,'',148 from dual union all
select 57332,'1984-S05','1984-01-01 00:00:00','military missile','unknown',646,'','Dzho 2','','','Reentry Test',621,'',675 from dual union all
select 57333,'1984-S06','1984-01-01 00:00:00','military missile','unknown',646,'','Barkhat-1','','','Reentry Test',621,'',675 from dual union all
select 57334,'1984-S07','1984-01-01 00:00:00','military missile','unknown',646,'','Barkhat-1','','','Reentry Test',621,'',675 from dual union all
select 57335,'1984-S08','1984-01-01 00:00:00','military missile','unknown',646,'','Oval (VMF RV)','','','Reentry Test',621,'',675 from dual union all
select 57336,'1984-S09','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57337,'1984-S10','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57338,'1984-S100','1984-07-12 00:00:00','military missile','success',756,'','','','','Test',307,'S615',800 from dual;
insert into LAUNCH
select 57339,'1984-S101','1984-07-24 10:40:50','suborbital rocket','success',227,'NASA 21.91GE','GSFC M/DTR','','','Ionos',1821,'',98 from dual union all
select 5734,'1959-A40','1959-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 57340,'1984-S102','1984-07-24 17:00:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 654',500 from dual union all
select 57341,'1984-S103','1984-07-26 18:05:00','suborbital rocket','success',1219,'XV-19','','','','Ionos',330,'',566 from dual union all
select 57342,'1984-S104','1984-08-01 04:18:00','suborbital rocket','success',1318,'NASA 38.02UE','SPINEX 1','','','Plasma',1821,'',423 from dual union all
select 57343,'1984-S105','1984-08-07 01:00:00','suborbital rocket','success',1319,'NASA 33.43UE','Electrojet','','','Plasma',1821,'',115 from dual union all
select 57344,'1984-S106','1984-08-07 12:00:00','military missile','success',919,'ED-22','P-467','','','OT',231,'',320 from dual union all
select 57345,'1984-S107','1984-08-16 20:40:00','suborbital rocket','success',891,'NASA 30.19UU','','','','Aeron',1821,'',85 from dual union all
select 57346,'1984-S108','1984-08-23 09:11:00','suborbital rocket','success',860,'NASA 31.42GE','VLF Trig Elec','','','Plasma',1821,'',140 from dual union all
select 57347,'1984-S109','1984-08-23 10:05:00','suborbital rocket','success',891,'NASA 30.22UE','PSU E','','','Plasma',1821,'',85 from dual union all
select 57348,'1984-S11','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57349,'1984-S110','1984-08-24 00:00:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 630',1000 from dual union all
select 5735,'1959-A41','1959-03-03 21:45:00','military missile','success',843,'','Cold weather test','','','Test',428,'',10 from dual union all
select 57350,'1984-S111','1984-08-24 21:43:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 630',1000 from dual union all
select 57351,'1984-S112','1984-09-01 00:00:00','military missile','success',579,'','','','','Test',1480,'',86 from dual union all
select 57352,'1984-S113','1984-09-01 00:00:00','military missile','success',579,'','','','','Test',1480,'',86 from dual union all
select 57353,'1984-S114','1984-09-13 11:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT103GM','GLORY TRIP 103GM','OT',1705,'',1300 from dual union all
select 57354,'1984-S115','1984-09-18 17:45:00','suborbital rocket','success',860,'NASA 31.46GU','','','','Aeron',1895,'',140 from dual union all
select 57355,'1984-S116','1984-09-19 00:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT104GB','GLORY TRIP 104GB','OT',1712,'',1300 from dual union all
select 57356,'1984-S117','1984-09-19 03:46:00','suborbital rocket','success',210,'NASA 27.91UL','','','','Aeron',1895,'',308 from dual union all
select 57357,'1984-S118','1984-09-20 14:30:00','military missile','success',919,'AO-1','P-468 Hot Shot','','','OT',231,'',320 from dual union all
select 57358,'1984-S119','1984-10-01 14:34:00','military missile','success',907,'FTM-6','Mk 12A/21 RV x 6','','','Test',1726,'',1000 from dual union all
select 57359,'1984-S12','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 5736,'1959-A42','1959-03-04 18:07:00','military missile','success',255,'624-XY13','','','','OT',293,'',20 from dual union all
select 57360,'1984-S120','1984-10-03 14:22:00','military missile','success',919,'AO-2','P-469 Hot Shot','','','OT',231,'',320 from dual union all
select 57361,'1984-S121','1984-10-04 11:45:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT105GM','GLORY TRIP 105GM','OT',1699,'',1300 from dual union all
select 57362,'1984-S122','1984-10-06 07:00:00','suborbital rocket','success',211,'NASA 27.71UG','IMAPS 1','','','FUV Astron',1895,'',273 from dual union all
select 57363,'1984-S123','1984-10-09 21:05:00','military missile','success',1442,'','DASO','','','OT',421,'SSBN 658',1000 from dual union all
select 57364,'1984-S124','1984-10-18 00:00:00','military missile','success',795,'','IPA-1','RSLP IPA-1','','Reentry Test',1698,'',1300 from dual union all
select 57365,'1984-S125','1984-10-22 06:27:00','suborbital rocket','success',1319,'NASA 33.47UE','UM UV Glow 1','','','Plasma',1873,'',150 from dual union all
select 57366,'1984-S126','1984-10-24 16:58:20','suborbital rocket','success',228,'SSC S38','TOR','','','Auroral/E Field/Releases',412,'',188 from dual union all
select 57367,'1984-S127','1984-10-24 02:19:59','suborbital rocket','success',1319,'NASA 33.41UE','MINI (Metal Ions)','','','Plasma',1873,'',170 from dual union all
select 57368,'1984-S128','1984-04-17 17:42:00','military missile','success',914,'67099-802(10L)','15A11 UBP-3','GCh 15B99','','Test',540,'',1000 from dual union all
select 57369,'1984-S129','1984-11-13 23:22:00','military missile','success',1006,'51076-811-14T','15A18 UBP-19 K','GCh 15F183','','OT',485,'',1000 from dual union all
select 5737,'1959-A43','1959-03-04 11:14:00','suborbital rocket','failure',1194,'SL21','UCL GC-6/QUB Na-2','','','Aeronomy',1839,'',30 from dual union all
select 57370,'1984-S13','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57371,'1984-S130','1984-11-13 20:17:00','test rocket','success',20,'','','','','Interceptor',383,'F-15',1000 from dual union all
select 57372,'1984-S131','1984-11-21 00:00:00','suborbital rocket','success',1220,'PT-01','','','','Test',330,'',616 from dual union all
select 57373,'1984-S132','1984-12-04 19:54:00','military missile','success',1442,'','DASO','','','OT',421,'SSBN 730',1000 from dual union all
select 57374,'1984-S133','1984-12-10 19:20:00','suborbital rocket','success',211,'NASA 27.101GU','GSFC Solar UVI 1','','','Solar UV',1895,'',312 from dual union all
select 57375,'1984-S134','1984-12-15 20:44:00','military missile','success',1442,'','DASO','','','OT',421,'SSBN 655',1000 from dual union all
select 57376,'1984-S135','1984-06-03 00:00:00','military missile','success',1442,'','WTR to Wake','','','OT',1897,'SSBN UNK',1000 from dual union all
select 57377,'1984-S136','1984-07-02 00:00:00','military missile','success',1442,'','WTR to Wake','','','OT',1897,'SSBN UNK',1000 from dual union all
select 57378,'1984-S137','1984-06-11 11:04:00','suborbital rocket','success',597,'INTA MZ-8401','Test','Zorzal-1 (Maqueta)','','Test',396,'',1 from dual union all
select 57379,'1984-S138','1984-06-12 11:03:00','suborbital rocket','success',597,'INTA MZ-8402','Test','Zorzal-2 (Maqueta)','','Test',396,'',1 from dual union all
select 5738,'1959-A44','1959-03-07 19:34:00','suborbital rocket','failure',1490,'AGI18','Na release','','','Aeronomy',668,'',35 from dual union all
select 57380,'1984-S139','1984-06-13 11:04:00','suborbital rocket','success',597,'INTA RP-8401','Test (Ballast)','Rocio-1','','Test',396,'',45 from dual union all
select 57381,'1984-S14','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57382,'1984-S140','1984-09-24 00:00:00','military missile','success',984,'Aerofon','','','','Test',636,'',86 from dual union all
select 57383,'1984-S141','1984-10-31 00:00:00','military missile','success',984,'Aerofon','','ZLKI 5','','Test',636,'',86 from dual union all
select 57384,'1984-S142','1984-09-07 07:10:00','military missile','success',1507,'VMF','Caused UFO sighting','','','OT',201,'PLBR',1000 from dual union all
select 57385,'1984-S143','1984-01-25 00:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 57386,'1984-S144','1984-01-30 00:00:00','suborbital rocket','success',725,'','','','','Aeron',6,'',90 from dual union all
select 57387,'1984-S145','1984-02-01 00:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 57388,'1984-S146','1984-02-16 00:00:00','suborbital rocket','success',725,'','','','','Aeron',6,'',90 from dual union all
select 57389,'1984-S147','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 5739,'1959-A45','1959-03-10 09:01:00','military missile','success',1214,'N-69D N-3320','','','','Test',224,'',10 from dual union all
select 57390,'1984-S148','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57391,'1984-S149','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57392,'1984-S15','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57393,'1984-S150','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57394,'1984-S151','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57395,'1984-S152','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57396,'1984-S153','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57397,'1984-S154','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57398,'1984-S155','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57399,'1984-S156','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 574,'1945-S50','1945-01-04 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5740,'1959-A46','1959-03-10 17:50:00','suborbital spaceplane','success',1520,'1-C-1','','','','Test',379,'NB-52 003',11 from dual union all
select 57400,'1984-S157','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57401,'1984-S158','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57402,'1984-S159','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57403,'1984-S16','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57404,'1984-S160','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57405,'1984-S161','1984-02-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57406,'1984-S162','1984-03-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57407,'1984-S163','1984-03-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57408,'1984-S164','1984-03-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57409,'1984-S165','1984-03-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 5741,'1959-A47','1959-03-16 00:00:00','military missile','success',445,'S/N 2','Alpha Draco','','','Test',225,'',30 from dual union all
select 57410,'1984-S166','1984-03-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57411,'1984-S167','1984-03-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57412,'1984-S168','1984-03-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57413,'1984-S169','1984-03-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57414,'1984-S17','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57415,'1984-S170','1984-03-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57416,'1984-S171','1984-03-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57417,'1984-S172','1984-03-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57418,'1984-S173','1984-06-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57419,'1984-S174','1984-06-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 5742,'1959-A48','1959-03-17 00:00:00','atmospheric rocket','failure',552,'','GE Ablative RV','','','Reentry Test',1821,'',30 from dual union all
select 57420,'1984-S175','1984-06-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57421,'1984-S176','1984-06-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57422,'1984-S177','1984-06-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57423,'1984-S178','1984-06-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57424,'1984-S179','1984-06-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57425,'1984-S18','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57426,'1984-S181','1984-06-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57427,'1984-S182','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57428,'1984-S183','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual;
insert into LAUNCH
select 57429,'1984-S184','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 5743,'1959-A49','1959-03-29 00:00:00','military missile','success',268,'3-04','','','','Test',1752,'',15 from dual union all
select 57430,'1984-S185','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57431,'1984-S186','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57432,'1984-S187','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57433,'1984-S188','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57434,'1984-S189','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57435,'1984-S19','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57436,'1984-S190','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57437,'1984-S191','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57438,'1984-S192','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57439,'1984-S193','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 5744,'1959-A51','1959-03-31 00:00:00','military missile','success',255,'624-XY14','','','','OT',293,'',20 from dual union all
select 57440,'1984-S194','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57441,'1984-S195','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57442,'1984-S196','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57443,'1984-S197','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57444,'1984-S198','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57445,'1984-S199','1984-10-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57446,'1984-S20','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57447,'1984-S200','1984-06-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 57448,'1984-S205','1984-11-13 15:36:00','military missile','unknown',917,'AO-225','P-470','','','OT',433,'',180 from dual union all
select 57449,'1984-S206','1984-11-13 16:03:00','military missile','unknown',917,'AO-226','P-471','','','OT',433,'',180 from dual union all
select 5745,'1959-A52','1959-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 57450,'1984-S207','1984-11-13 18:10:00','military missile','unknown',917,'AO-227','P-472','','','OT',433,'',180 from dual union all
select 57451,'1984-S208','1984-11-13 18:30:00','military missile','unknown',917,'AO-228','P-473','','','OT',433,'',180 from dual union all
select 57452,'1984-S209','1984-01-01 00:00:00','military missile','success',644,'','Dzho 1','','','Reentry Test',636,'',null from dual union all
select 57453,'1984-S21','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57454,'1984-S210','1984-01-01 00:00:00','military missile','success',984,'Aerofon','UGCh','ZLKI 3','','Test',636,'',86 from dual union all
select 57455,'1984-S211','1984-04-27 18:35:00','military missile','success',1006,'51086-111-5T','15A18 UBP-18 K','GCh 15F183','','OT',485,'',1000 from dual union all
select 57456,'1984-S212','1984-04-17 00:00:00','military missile','success',1044,'64646-812-51','8K98P UBP-22','','','Test',477,'',1000 from dual union all
select 57457,'1984-S213','1984-06-26 00:00:00','military missile','success',1037,'Yu45713-30','8K98 UBP-64','','','Test',477,'',1000 from dual union all
select 57458,'1984-S214','1984-07-26 00:00:00','military missile','success',1037,'Yu45713-25','8K98 UBP-65','','','Test',477,'',1000 from dual union all
select 57459,'1984-S215','1984-08-21 00:00:00','military missile','failure',1037,'','8K98 UBP-66','','','Test',477,'',0 from dual union all
select 5746,'1959-A53','1959-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 57460,'1984-S216','1984-08-31 00:00:00','military missile','failure',1044,'','8K98P UBP-23','','','Test',477,'',0 from dual union all
select 57461,'1984-S217','1984-12-14 00:00:00','military missile','success',1037,'29557-211-42','8K98 UBP-67','','','Test',477,'',1000 from dual union all
select 57462,'1984-S218','1984-12-28 00:00:00','military missile','success',1037,'64647-221-46','8K98 UBP-68','','','Test',477,'',1000 from dual union all
select 57463,'1984-S22','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57464,'1984-S220','1984-02-20 00:00:00','military missile','success',1434,'45524-404-6L','15Zh58 LKI-6','','','Test',477,'',1000 from dual union all
select 57465,'1984-S221','1984-03-27 00:00:00','military missile','success',1434,'45584-406-8L','15Zh58 LKI-7','','','Test',477,'',1000 from dual union all
select 57466,'1984-S222','1984-04-23 00:00:00','military missile','success',1434,'45584-407-9L','15Zh58 LKI-8','','','Test',477,'',1000 from dual union all
select 57467,'1984-S223','1984-05-23 00:00:00','military missile','success',1434,'45584-405-7L','15Zh58 LKI-9','','','Test',477,'',1000 from dual union all
select 57468,'1984-S224','1984-07-26 00:00:00','military missile','success',1434,'45584-409-10L','15Zh58 LKI-10','','','Test',477,'',1000 from dual union all
select 57469,'1984-S225','1984-09-10 00:00:00','military missile','success',1434,'45584-410-11L','15Zh58 LKI-11','','','Test',477,'',1000 from dual union all
select 5747,'1959-A54','1959-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 57470,'1984-S226','1984-10-02 00:00:00','military missile','failure',1434,'12L','15Zh58 LKI-12','','','Test',477,'',0 from dual union all
select 57471,'1984-S227','1984-11-20 00:00:00','military missile','success',1434,'64516-214-37','15Zh58 K-1 Kontrol''niy','','','Test',477,'',1000 from dual union all
select 57472,'1984-S228','1984-12-06 00:00:00','military missile','success',1434,'45542-222-13L','15Zh58 LKI-13','','','Test',477,'',1000 from dual union all
select 57473,'1984-S229','1984-02-16 00:00:00','military missile','failure',1039,'7L','15Zh44 LKI-8','','','Test',477,'',0 from dual union all
select 57474,'1984-S23','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57475,'1984-S230','1984-03-29 00:00:00','military missile','failure',1040,'2L','15Zh52 LKI-2','','','Test',477,'',0 from dual union all
select 57476,'1984-S231','1984-06-08 00:00:00','military missile','success',1040,'48722-108-3L','15Zh52 LKI-3','','','Test',477,'',1000 from dual union all
select 57477,'1984-S232','1984-06-28 00:00:00','military missile','success',1040,'48722-109-4L','15Zh52 LKI-4','','','Test',477,'',1000 from dual union all
select 57478,'1984-S233','1984-09-07 00:00:00','military missile','success',1040,'48736-201-5L','15Zh52 LKI-5','','','Test',477,'',1000 from dual union all
select 57479,'1984-S234','1984-10-16 00:00:00','military missile','success',1040,'48736-203-7L','15Zh52 LKI-6','','','Test',477,'',1000 from dual union all
select 5748,'1959-A55','1959-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 57480,'1984-S235','1984-11-01 00:00:00','military missile','success',1040,'48736-202-6L','15Zh52 LKI-7','','','Test',477,'',1000 from dual union all
select 57481,'1984-S236','1984-12-12 00:00:00','military missile','success',1040,'48736-204-8L','15Zh52 LKI-8','','','Test',477,'',1000 from dual union all
select 57482,'1984-S237','1984-12-26 00:00:00','military missile','success',1040,'48736-205-10L','15Zh52 LKI-9','','','Test',477,'',1000 from dual union all
select 57483,'1984-S238','1984-07-10 00:00:00','military missile','failure',1453,'51163-92-844','15A35 60 RD UBP-2','','','OT',1461,'',0 from dual union all
select 57484,'1984-S239','1984-05-25 00:00:00','military missile','success',1450,'','15A20 52 RD UBP','','','OT',195,'',1000 from dual union all
select 57485,'1984-S24','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57486,'1984-S240','1984-01-10 13:01:00','suborbital rocket','success',1284,'INTA SLD-8401','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 57487,'1984-S241','1984-01-18 12:53:00','suborbital rocket','success',1284,'INTA SLD-8402','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 57488,'1984-S242','1984-01-21 12:52:00','suborbital rocket','success',1284,'INTA SLD-8403','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 57489,'1984-S243','1984-01-24 13:16:00','suborbital rocket','success',1284,'INTA SLD-8404','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 5749,'1959-A56','1959-04-01 00:00:00','atmospheric rocket','success',853,'F40','F40 Heat transfer','','','Test',1821,'',45 from dual union all
select 57490,'1984-S244','1984-01-30 13:57:00','suborbital rocket','success',1284,'INTA SLD-8405','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 57491,'1984-S245','1984-03-29 12:05:00','suborbital rocket','success',1284,'INTA SLD-8406','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 57492,'1984-S246','1984-06-13 12:35:00','suborbital rocket','success',1284,'INTA SLD-8407','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 57493,'1984-S247','1984-06-15 11:05:00','suborbital rocket','success',1284,'INTA SLD-8408','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 57494,'1984-S248','1984-09-21 12:06:00','suborbital rocket','success',1284,'INTA SLD-8409','Winter Anom/Dyn.Atm.','','','Aeron',396,'',100 from dual union all
select 57495,'1984-S249','1984-12-06 00:00:00','military missile','success',1434,'45542-223-14L','15Zh58 LKI-14','','','Test',477,'',1000 from dual union all
select 57496,'1984-S25','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57497,'1984-S250','1984-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-15','Duga-2','','Test',368,'',500 from dual union all
select 57498,'1984-S251','1984-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-16','Duga-2','','Test',368,'',500 from dual union all
select 57499,'1984-S252','1984-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-17','Duga-2','','Test',368,'',500 from dual union all
select 575,'1945-S51','1945-01-04 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5750,'1959-A57','1959-04-01 00:00:00','suborbital spaceplane','failure',1520,'1-A-2','','','','Test',379,'NB-52 003',11 from dual union all
select 57500,'1984-S253','1984-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-18','Duga-2','','Test',368,'',500 from dual union all
select 57501,'1984-S254','1984-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-19','Duga-2','','Test',368,'',500 from dual union all
select 57502,'1984-S255','1984-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-20','Duga-2','','Test',368,'',500 from dual union all
select 57503,'1984-S256','1984-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-21','Duga-2','','Test',368,'',500 from dual union all
select 57504,'1984-S257','1984-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-22','Duga-2','','Test',368,'',500 from dual union all
select 57505,'1984-S258','1984-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-23','Duga-2','','Test',368,'',500 from dual union all
select 57506,'1984-S259','1984-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-24','Duga-2','','Test',368,'',500 from dual union all
select 57507,'1984-S26','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57508,'1984-S260','1984-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-25','Duga-2','','Test',368,'',500 from dual union all
select 57509,'1984-S261','1984-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-26','Duga-2','','Test',368,'',500 from dual union all
select 5751,'1959-A58','1959-04-02 07:00:00','suborbital rocket','success',716,'','','','','Aeron',1895,'',40 from dual union all
select 57510,'1984-S262','1984-04-01 00:00:00','military missile','success',922,'','15Zh45 4 RD Salvo Test 1','S-300 Target','','Test',368,'',500 from dual union all
select 57511,'1984-S263','1984-04-01 00:00:00','military missile','success',922,'','15Zh45 4 RD Salvo Test 2','S-300 Target','','Test',368,'',500 from dual union all
select 57512,'1984-S264','1984-04-01 00:00:00','military missile','success',922,'','15Zh45 4 RD Salvo 1','S-300 Target','','Test',368,'',500 from dual union all
select 57513,'1984-S265','1984-04-01 00:00:00','military missile','success',922,'','15Zh45 4 RD Salvo 3','S-300 Target','','Test',368,'',500 from dual union all
select 57514,'1984-S266','1984-04-01 00:00:00','military missile','success',922,'','15Zh45 4 RD Salvo 4','S-300 Target','','Test',368,'',500 from dual union all
select 57515,'1984-S267','1984-04-01 00:00:00','military missile','success',922,'','15Zh45 4 RD Salvo 5','S-300 Target','','Test',368,'',500 from dual union all
select 57516,'1984-S268','1984-04-01 00:00:00','military missile','success',922,'','15Zh45 4 RD Salvo 6','S-300 Target','','Test',368,'',500 from dual union all
select 57517,'1984-S269','1984-04-01 00:00:00','military missile','success',922,'','15Zh45 4 RD Salvo 7','S-300 Target','','Test',368,'',500 from dual union all
select 57518,'1984-S27','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual;
insert into LAUNCH
select 57519,'1984-S270','1984-07-01 00:00:00','military missile','success',922,'','15Zh45 31 RD UBP','','','Test',636,'',500 from dual union all
select 5752,'1959-A59','1959-04-07 01:26:00','military missile','success',1214,'SM-62A N-3422','','','','OT',224,'',10 from dual union all
select 57520,'1984-S271','1984-12-27 00:00:00','military missile','success',922,'','15Zh45 50 RD UBP','','','Test',636,'',500 from dual union all
select 57521,'1984-S272','1984-04-23 00:00:00','military missile','success',923,'','15Zh53 29 RD UBP','','','Test',636,'',500 from dual union all
select 57522,'1984-S273','1984-07-01 00:00:00','military missile','success',923,'','15Zh53 32 RD UBP','','','Test',636,'',500 from dual union all
select 57523,'1984-S274','1984-10-25 00:00:00','military missile','success',923,'','15Zh53 32 RD UBP','','','Test',636,'',500 from dual union all
select 57524,'1984-S275','1984-04-27 00:00:00','military missile','success',922,'','15Zh56 LKI-1','Gorn','','Test/Comms',636,'',500 from dual union all
select 57525,'1984-S276','1984-12-01 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 57526,'1984-S277','1984-12-01 00:00:00','military missile','success',976,'','8K63 40 RD UBP','','','Test',636,'',402 from dual union all
select 57527,'1984-S278','1984-04-01 00:00:00','military missile','success',922,'','15Zh45 4 RD Salvo 2','S-300 Target','','Test',368,'',500 from dual union all
select 57528,'1984-S279','1984-04-01 00:00:00','military missile','success',579,'','','','','Test',1480,'',86 from dual union all
select 57529,'1984-S28','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 5753,'1959-A60','1959-04-10 16:24:00','suborbital spaceplane','failure',1520,'1-A-3','','','','Test',379,'NB-52 003',11 from dual union all
select 57530,'1984-S280','1984-04-01 00:00:00','military missile','success',579,'','','','','Test',1480,'',86 from dual union all
select 57531,'1984-S281','1984-09-01 00:00:00','military missile','success',579,'','','','','Test',1480,'',86 from dual union all
select 57532,'1984-S29','1984-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 57533,'1984-S30','1984-08-09 00:00:00','military missile','success',1004,'','15A18 UBP 59 RD','','','OT',827,'',1000 from dual union all
select 57534,'1984-S31','1984-01-01 00:00:00','military missile','success',1451,'','OT','','','OT',596,'',1000 from dual union all
select 57535,'1984-S32','1984-01-01 00:00:00','military missile','success',1451,'','OT','','','OT',596,'',1000 from dual union all
select 57536,'1984-S33','1984-01-01 00:00:00','military missile','success',1451,'','OT','','','OT',596,'',1000 from dual union all
select 57537,'1984-S34','1984-11-02 16:55:00','military missile','success',1452,'49256-42-374','15A30 UBP-27 K','GCh 15F355P3','','OT',534,'',1000 from dual union all
select 57538,'1984-S35','1984-01-01 00:00:00','military missile','success',1452,'','OT','','','OT',596,'',1000 from dual union all
select 57539,'1984-S36','1984-01-17 00:00:00','military missile','success',1455,'63032-82-029T(8L)','15A35S LKI-8','GCh 15F357','','OT',508,'',1000 from dual union all
select 5754,'1959-A61','1959-04-14 21:46:00','military missile','failure',154,'3D','Mk 2-2A','R&D','','Test',228,'',1 from dual union all
select 57540,'1984-S37','1984-01-27 00:00:00','military missile','success',1453,'51126-31-285T','15A35 UBP-22','','','OT',505,'',1000 from dual union all
select 57541,'1984-S38','1984-05-07 00:00:00','military missile','success',1453,'63085-53-240','15A35 UBP-23 K','','','OT',508,'',1000 from dual union all
select 57542,'1984-S39','1984-01-01 00:00:00','military missile','success',1453,'','OT','','','OT',596,'',1000 from dual union all
select 57543,'1984-S40','1984-01-06 21:55:00','suborbital rocket','success',860,'DLR A-NO-122','Ferdinand 66 M-T 2','','','Aeron',29,'',127 from dual union all
select 57544,'1984-S41','1984-01-13 19:30:00','suborbital rocket','success',668,'K-9M-77','CIR','','','IR Astron',839,'',317 from dual union all
select 57545,'1984-S42','1984-01-13 20:00:00','suborbital rocket','success',860,'DLR A-NO-123','Ferdinand 68 M-T 3','','','Ionos',29,'',122 from dual union all
select 57546,'1984-S43','1984-01-16 18:20:00','suborbital rocket','success',1319,'NASA 33.36UU','','','','Aeron',1873,'',200 from dual union all
select 57547,'1984-S44','1984-01-21 18:52:30','test rocket','success',20,'','','','','Test',383,'F-15',1000 from dual union all
select 57548,'1984-S45','1984-01-25 11:15:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT99GM','GLORY TRIP 99GM','OT',1705,'',1300 from dual union all
select 57549,'1984-S46','1984-01-25 13:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT100GM','GLORY TRIP 100GM','OT',1703,'',1300 from dual union all
select 5755,'1959-A62','1959-04-19 00:00:00','military missile','success',268,'2-05','','','','Test',1752,'',17 from dual union all
select 57550,'1984-S47','1984-01-25 16:39:00','suborbital rocket','success',860,'DLR A-NO-124','Ferdinand 67 M-T 4','','','Aeron',29,'',118 from dual union all
select 57551,'1984-S48','1984-01-26 15:52:00','suborbital rocket','success',1304,'','HAVE SLED II','','','Target?',1047,'',500 from dual union all
select 57552,'1984-S49','1984-01-27 19:37:00','suborbital rocket','success',1202,'DLR A-GRC-131','CAESAR I','','','Auroral',29,'',676 from dual union all
select 57553,'1984-S50','1984-01-31 18:31:00','suborbital rocket','success',860,'DLR A-NO-125','Ferdinand 70 M-T 5','','','Ionos',29,'',115 from dual union all
select 57554,'1984-S51','1984-01-31 18:53:00','suborbital rocket','success',860,'MADAME 1','Ferdinand 73 M-M 1','','','Aeron',29,'',113 from dual union all
select 57555,'1984-S53','1984-02-02 15:36:00','suborbital rocket','success',1304,'','HAVE SLED II','','','Target?',1047,'',500 from dual union all
select 57556,'1984-S54','1984-02-07 09:09:00','suborbital rocket','success',231,'NASA 35.006UE','Bidarca','','','Plasma',1047,'',1040 from dual union all
select 57557,'1984-S55','1984-02-07 09:10:40','suborbital rocket','success',1337,'NASA 29.21UE','Bidarca','','','Plasma',1912,'',640 from dual union all
select 57558,'1984-S56','1984-02-10 02:40:00','suborbital rocket','success',860,'DLR A-NO-126','Ferdinand 69 M-T 6','','','Ionos',29,'',117 from dual union all
select 57559,'1984-S57','1984-02-10 03:05:00','suborbital rocket','success',860,'MADAME 2','Ferdinand 72 M-M 2','','','Aeron',29,'',115 from dual union all
select 5756,'1959-A63','1959-04-21 00:00:00','military missile','success',255,'624-XY15','','','','OT',272,'',20 from dual union all
select 57560,'1984-S58','1984-02-10 04:12:00','suborbital rocket','success',1210,'DLR K-GR-130','MAP/WINE M-I 1','','','Aeron',416,'',180 from dual union all
select 57561,'1984-S59','1984-02-16 01:20:00','suborbital rocket','success',860,'DLR A-NO-127','Ferdinand 64 M-T 7','','','Aeron',29,'',117 from dual union all
select 57562,'1984-S60','1984-02-18 00:22:00','suborbital rocket','success',860,'DLR A-NO-128','Ferdinand 65 M-T 8','','','Aeron',29,'',113 from dual union all
select 57563,'1984-S61','1984-02-18 16:29:30','suborbital rocket','success',860,'NASA 31.44UU','CWAS 7','','','Aeronomy',428,'',99 from dual union all
select 57564,'1984-S62','1984-02-22 00:00:00','military missile','success',756,'14','','','','Test',307,'S655',800 from dual union all
select 57565,'1984-S63','1984-02-23 12:10:00','suborbital rocket','success',860,'NASA 31.38GE','GSFC E','','','Plasma',1821,'',140 from dual union all
select 57566,'1984-S64','1984-02-23 20:34:47','suborbital rocket','success',227,'NASA 21.86UE','','','','Ionos/Auroral',428,'',189 from dual union all
select 57567,'1984-S65','1984-02-28 03:53:44','suborbital rocket','success',211,'ADD-VIIIC-10','ARIES B','','','Auroral/Aeron/Ionos',428,'',204 from dual union all
select 57568,'1984-S66','1984-02-28 17:13:00','suborbital rocket','success',1319,'NASA 33.39UL','','','','Aeron',1047,'',200 from dual union all
select 57569,'1984-S67','1984-02-29 00:00:00','military missile','success',796,'','Mk 11 RV','FOT GT146MS','GLORY TRIP 146MS','OT   Last M2 SOT',1699,'',1300 from dual union all
select 5757,'1959-A64','1959-04-21 08:55:00','military missile','success',1214,'N-69D N-3320-1','','','','OT',224,'',10 from dual union all
select 57570,'1984-S68','1984-03-03 03:06:24','suborbital rocket','success',211,'ADD-VIIIC-9','ARIES A','','','Auroral/Aeron/Ionos',428,'',206 from dual union all
select 57571,'1984-S69','1984-03-03 04:40:00','suborbital rocket','success',227,'NASA 21.85UG','JHU UVT F11','','','UV Astron',1873,'',206 from dual union all
select 57572,'1984-S70','1984-03-19 17:00:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 644',500 from dual union all
select 57573,'1984-S71','1984-03-30 15:30:00','military missile','success',907,'FTM-4','Mk 12A RV x 10','','','Test',1726,'',1000 from dual union all
select 57574,'1984-S72','1984-03-30 11:00:00','suborbital rocket','success',231,'NASA 35.007GE','','','','Plasma',1047,'',948 from dual union all
select 57575,'1984-S73','1984-04-01 00:00:00','military missile','success',579,'','','','','Test',1480,'',86 from dual union all
select 57576,'1984-S74','1984-01-18 00:00:00','military missile','failure',1040,'1L','15Zh52 LKI-1','RailCar Launch','','Test',465,'',0 from dual union all
select 57577,'1984-S75','1984-04-01 07:19:00','suborbital rocket','success',231,'NASA 35.008GE','','','','Plasma',1047,'',954 from dual union all
select 57578,'1984-S76','1984-04-02 07:15:50','suborbital rocket','success',1318,'NASA 38.01UE','','','','Plasma',1047,'',500 from dual union all
select 57579,'1984-S77','1984-04-02 07:17:57','suborbital rocket','success',1319,'NASA 33.42UE','','','','Plasma',1047,'',200 from dual union all
select 5758,'1959-A65','1959-04-22 00:00:00','military missile','success',771,'Training #13','','','','Test',222,'',12 from dual union all
select 57580,'1984-S78','1984-04-04 19:27:01','suborbital rocket','success',1073,'S-310JA-8','AURORA I','','','Ionos',1458,'',201 from dual union all
select 57581,'1984-S79','1984-04-07 07:20:00','military missile','success',1442,'','DASO','','','OT',421,'SSBN 729',1000 from dual union all
select 57582,'1984-S80','1984-04-08 00:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT101GB','GLORY TRIP 101GB','OT',1712,'',1300 from dual union all
select 57583,'1984-S81','1984-04-08 00:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT102GM','GLORY TRIP 102GM','OT',1705,'',1300 from dual union all
select 57584,'1984-S82','1984-04-09 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 645',500 from dual union all
select 57585,'1984-S83','1984-04-09 05:16:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 645',500 from dual union all
select 57586,'1984-S84','1984-05-03 04:13:00','suborbital rocket','success',1212,'DLR K-GR-132','DLR TEXUS 9','','','Micrograv',416,'',258 from dual union all
select 57587,'1984-S85','1984-05-03 22:14:10','suborbital rocket','success',1073,'S-310JA-9','AURORA II','','','Ionos',1458,'',204 from dual union all
select 57588,'1984-S86','1984-05-09 06:30:00','suborbital rocket','success',44,'NASA 17.20UH','Soft X-ray ISM','','','XR Astron',1895,'',191 from dual union all
select 57589,'1984-S87','1984-05-10 08:57:00','suborbital rocket','success',860,'NASA 31.41GE','VLF Trig Elec','','','Plasma',1821,'',89 from dual union all
select 5759,'1959-A66','1959-04-22 00:00:00','military missile','success',771,'Training #14','','','','Test',222,'',12 from dual union all
select 57590,'1984-S88','1984-05-11 11:22:31','suborbital rocket','success',217,'ARS-3A-69','SMERF 1','','','Aeron',428,'',103 from dual union all
select 57591,'1984-S89','1984-05-15 07:46:00','suborbital rocket','success',1212,'DLR K-GR-133','DLR TEXUS 10','','','Micrograv',416,'',216 from dual union all
select 57592,'1984-S90','1984-05-16 14:31:00','military missile','success',919,'ED-21','P-466','','','OT',231,'',320 from dual union all
select 57593,'1984-S91','1984-05-28 23:17:13','suborbital rocket','success',1073,'S-310JA-10','AURORA III','','','Ionos',1458,'',209 from dual union all
select 57594,'1984-S92','1984-06-10 00:00:00','military missile','success',795,'','HOE Target','RSLP STREP-13 (HOE)','','Target',1698,'',1300 from dual union all
select 57595,'1984-S93','1984-06-10 00:00:00','military missile','success',558,'HOE 4','HOE 4','HOE 4','','ABM Test',868,'',200 from dual union all
select 57596,'1984-S94','1984-06-15 16:50:00','military missile','success',907,'FTM-5','Mk 12A RV x 5, Mk 21 RV','','','Test',1726,'',1000 from dual union all
select 57597,'1984-S95','1984-06-19 13:05:00','suborbital rocket','success',211,'NASA 27.89US','','','','Solar',1895,'',316 from dual union all
select 57598,'1984-S96','1984-06-24 06:15:00','suborbital rocket','success',210,'NASA 27.84UG','FUV Echelle','','','UV Astron',1895,'',248 from dual union all
select 57599,'1984-S97','1984-06-29 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 625',500 from dual union all
select 576,'1945-S52','1945-01-13 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5760,'1959-A67','1959-04-23 00:00:00','military missile','success',771,'Training #15','','','','Test',222,'',12 from dual union all
select 57600,'1984-S98','1984-06-29 20:03:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 625',500 from dual union all
select 57601,'1984-S99','1984-07-05 20:20:00','test rocket','failure',647,'','BOR-5 No. 501','','','Reentry Test',622,'',210 from dual union all
select 57602,'1984-W01','1984-01-03 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',59 from dual union all
select 57603,'1984-W02','1984-01-04 20:00:00','suborbital rocket','success',1284,'MAP/WINE','M-D 12A','','','Aeron',29,'',78 from dual union all
select 57604,'1984-W03','1984-01-04 20:58:00','suborbital rocket','success',1284,'MAP/WINE M-M','M-D 09L','','','Aeron',962,'',75 from dual union all
select 57605,'1984-W04','1984-01-04 21:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',67 from dual union all
select 57606,'1984-W05','1984-01-04 23:21:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',56 from dual union all
select 57607,'1984-W06','1984-01-06 22:44:00','suborbital rocket','success',1284,'MAP/WINE','M-D 13A','','','Aeron',29,'',79 from dual union all
select 57608,'1984-W07','1984-01-07 20:00:00','suborbital rocket','success',1284,'MAP/WINE M-M','M-D 10L','','','Aeron',962,'',75 from dual;
insert into LAUNCH
select 57609,'1984-W08','1984-01-09 21:52:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',66 from dual union all
select 5761,'1959-A68','1959-04-23 00:00:00','military missile','success',771,'Training #16','','','','Test',222,'',12 from dual union all
select 57610,'1984-W09','1984-01-09 23:59:00','suborbital rocket','unknown',716,'T 1-0800','','','','Aeron',1821,'',58 from dual union all
select 57611,'1984-W10','1984-01-10 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',64 from dual union all
select 57612,'1984-W100','1984-03-02 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',75 from dual union all
select 57613,'1984-W101','1984-03-07 02:00:00','suborbital rocket','success',761,'MT135P-R523','','','','Meteo',1332,'',55 from dual union all
select 57614,'1984-W102','1984-03-08 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',59 from dual union all
select 57615,'1984-W103','1984-03-12 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 57616,'1984-W104','1984-03-12 21:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',65 from dual union all
select 57617,'1984-W105','1984-03-14 02:00:00','suborbital rocket','success',761,'MT135P-R524','','','','Meteo',1332,'',53 from dual union all
select 57618,'1984-W106','1984-03-14 12:51:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',62 from dual union all
select 57619,'1984-W107','1984-03-14 18:42:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',61 from dual union all
select 5762,'1959-A69','1959-04-27 00:00:00','military missile','failure',445,'S/N 3','Alpha Draco','','','Test',225,'',19 from dual union all
select 57620,'1984-W108','1984-03-15 00:14:00','suborbital rocket','success',716,'','T 1-0881','','','Aeron',1821,'',57 from dual union all
select 57621,'1984-W109','1984-03-15 21:18:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',61 from dual union all
select 57622,'1984-W11','1984-01-11 02:03:59','suborbital rocket','success',761,'MT135P-R517','','','','Meteo',1332,'',56 from dual union all
select 57623,'1984-W110','1984-03-20 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',53 from dual union all
select 57624,'1984-W111','1984-03-21 12:56:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',61 from dual union all
select 57625,'1984-W112','1984-03-21 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',58 from dual union all
select 57626,'1984-W113','1984-03-21 18:21:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 57627,'1984-W114','1984-03-21 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1895,'',64 from dual union all
select 57628,'1984-W115','1984-03-21 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',55 from dual union all
select 57629,'1984-W116','1984-03-22 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',58 from dual union all
select 5763,'1959-A70','1959-05-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 57630,'1984-W117','1984-03-23 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 57631,'1984-W118','1984-03-26 12:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',61 from dual union all
select 57632,'1984-W119','1984-03-26 21:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 57633,'1984-W12','1984-01-11 15:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',61 from dual union all
select 57634,'1984-W120','1984-03-28 02:00:00','suborbital rocket','success',761,'MT135P-R525','','','','Meteo',1332,'',56 from dual union all
select 57635,'1984-W121','1984-03-28 16:09:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',62 from dual union all
select 57636,'1984-W122','1984-03-29 00:26:00','suborbital rocket','unknown',716,'T 1-0867','','','','Aeron',1821,'',52 from dual union all
select 57637,'1984-W123','1984-03-29 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 57638,'1984-W124','1984-03-30 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 57639,'1984-W125','1984-03-30 19:42:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',64 from dual union all
select 5764,'1959-A71','1959-05-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 57640,'1984-W126','1984-04-02 19:56:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',66 from dual union all
select 57641,'1984-W127','1984-04-03 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',63 from dual union all
select 57642,'1984-W128','1984-04-04 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',51 from dual union all
select 57643,'1984-W129','1984-04-06 13:58:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',60 from dual union all
select 57644,'1984-W13','1984-01-11 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1895,'',64 from dual union all
select 57645,'1984-W130','1984-04-06 14:09:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',65 from dual union all
select 57646,'1984-W131','1984-04-06 14:29:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',63 from dual union all
select 57647,'1984-W132','1984-04-06 15:57:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 57648,'1984-W133','1984-04-06 17:27:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 57649,'1984-W134','1984-04-06 21:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',52 from dual union all
select 5765,'1959-A72','1959-05-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 57650,'1984-W135','1984-04-08 14:03:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',72 from dual union all
select 57651,'1984-W136','1984-04-08 14:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 57652,'1984-W137','1984-04-08 14:31:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',64 from dual union all
select 57653,'1984-W138','1984-04-09 20:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 57654,'1984-W139','1984-04-10 13:11:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',66 from dual union all
select 57655,'1984-W14','1984-01-12 17:49:00','suborbital rocket','success',1502,'MAP/WINE','M-F 18','','','Aeron',29,'',75 from dual union all
select 57656,'1984-W140','1984-04-10 13:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 57657,'1984-W141','1984-04-10 14:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 57658,'1984-W142','1984-04-11 02:02:00','suborbital rocket','success',761,'MT135P-R526','','','','Meteo',1332,'',56 from dual union all
select 57659,'1984-W143','1984-04-12 12:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',65 from dual union all
select 5766,'1959-A73','1959-05-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 57660,'1984-W144','1984-04-12 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 57661,'1984-W145','1984-04-12 13:09:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',62 from dual union all
select 57662,'1984-W146','1984-04-13 06:38:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',67 from dual union all
select 57663,'1984-W147','1984-04-13 13:07:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',75 from dual union all
select 57664,'1984-W148','1984-04-13 13:47:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',68 from dual union all
select 57665,'1984-W149','1984-04-13 13:48:00','suborbital rocket','unknown',1284,'T 1-0884','','','','Aeron',1821,'',73 from dual union all
select 57666,'1984-W15','1984-01-12 20:19:00','suborbital rocket','success',1284,'MAP/WINE','M-D 16A','','','Aeron',29,'',78 from dual union all
select 57667,'1984-W150','1984-04-13 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',71 from dual union all
select 57668,'1984-W151','1984-04-13 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',77 from dual union all
select 57669,'1984-W152','1984-04-13 14:03:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',67 from dual union all
select 5767,'1959-A74','1959-05-01 00:00:00','suborbital rocket','failure',1327,'Terasca','','','','Test',1023,'',0 from dual union all
select 57670,'1984-W153','1984-04-13 14:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 57671,'1984-W154','1984-04-16 20:31:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 57672,'1984-W155','1984-04-18 12:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 57673,'1984-W156','1984-04-18 18:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',59 from dual union all
select 57674,'1984-W157','1984-04-18 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',52 from dual union all
select 57675,'1984-W158','1984-04-20 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',52 from dual union all
select 57676,'1984-W159','1984-04-20 21:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',66 from dual union all
select 57677,'1984-W16','1984-01-12 22:06:00','suborbital rocket','success',1284,'MAP/WINE M-M','M-D 11L','','','Aeron',962,'',75 from dual union all
select 57678,'1984-W160','1984-04-23 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 57679,'1984-W161','1984-04-24 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 5768,'1959-A75','1959-05-06 03:36:00','military missile','success',1214,'SM-62A N-3423','','','','OT',224,'',10 from dual union all
select 57680,'1984-W162','1984-04-25 02:00:00','suborbital rocket','success',761,'MT135P-R528','','','','Meteo',1332,'',55 from dual union all
select 57681,'1984-W163','1984-04-25 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 57682,'1984-W164','1984-04-25 18:31:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 57683,'1984-W165','1984-04-25 20:31:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 57684,'1984-W166','1984-04-25 23:45:00','suborbital rocket','success',716,'','T 1-0859','','','Aeron',1821,'',62 from dual union all
select 57685,'1984-W167','1984-04-26 18:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 57686,'1984-W168','1984-04-27 19:02:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',68 from dual union all
select 57687,'1984-W169','1984-04-27 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 57688,'1984-W17','1984-01-13 23:16:00','suborbital rocket','success',1284,'MAP/WINE','M-D 17A','','','Aeron',29,'',74 from dual union all
select 57689,'1984-W170','1984-05-01 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',63 from dual union all
select 5769,'1959-A77','1959-05-15 22:10:00','suborbital rocket','failure',89,'NASA 2.03GT','','','','Test',1821,'',1 from dual union all
select 57690,'1984-W171','1984-05-01 21:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',62 from dual union all
select 57691,'1984-W172','1984-05-02 15:51:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',59 from dual union all
select 57692,'1984-W173','1984-05-02 19:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 57693,'1984-W174','1984-05-07 21:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',64 from dual union all
select 57694,'1984-W175','1984-05-08 11:45:00','suborbital rocket','unknown',716,'T 1-0857','','','','Aeron',1821,'',62 from dual union all
select 57695,'1984-W176','1984-05-08 12:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 57696,'1984-W177','1984-05-09 02:02:00','suborbital rocket','success',761,'MT135P-R529','','','','Meteo',1332,'',55 from dual union all
select 57697,'1984-W178','1984-05-09 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 57698,'1984-W179','1984-05-10 09:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 57699,'1984-W18','1984-01-15 20:20:00','suborbital rocket','success',1284,'MAP/WINE M-M','M-D 12L','','','Aeron',962,'',75 from dual;
insert into LAUNCH
select 577,'1945-S53','1945-01-20 08:47:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5770,'1959-A78','1959-05-18 23:16:00','military missile','failure',933,'AX-7','','','','Test',244,'',10 from dual union all
select 57700,'1984-W180','1984-05-11 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 57701,'1984-W181','1984-05-16 02:00:00','suborbital rocket','success',761,'MT135P-R530','','','','Meteo',1332,'',52 from dual union all
select 57702,'1984-W182','1984-05-16 12:44:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',59 from dual union all
select 57703,'1984-W183','1984-05-16 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 57704,'1984-W184','1984-05-17 02:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 57705,'1984-W185','1984-05-21 20:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',71 from dual union all
select 57706,'1984-W186','1984-05-23 02:00:00','suborbital rocket','success',761,'MT135P-R531','','','','Meteo',1332,'',51 from dual union all
select 57707,'1984-W187','1984-05-23 19:56:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 57708,'1984-W188','1984-05-25 14:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 57709,'1984-W189','1984-05-25 20:33:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',60 from dual union all
select 5771,'1959-A79','1959-05-19 04:30:00','military missile','failure',154,'7D','RVX-2','R&D','','Test',229,'',1 from dual union all
select 57710,'1984-W19','1984-01-16 21:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 57711,'1984-W190','1984-05-30 02:03:00','suborbital rocket','success',761,'MT135P-R532','','','','Meteo',1332,'',51 from dual union all
select 57712,'1984-W191','1984-05-30 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',62 from dual union all
select 57713,'1984-W192','1984-05-30 14:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 57714,'1984-W193','1984-05-30 20:25:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',73 from dual union all
select 57715,'1984-W194','1984-06-01 14:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 57716,'1984-W195','1984-06-06 14:33:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 57717,'1984-W196','1984-06-06 18:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',71 from dual union all
select 57718,'1984-W197','1984-06-07 00:16:00','suborbital rocket','unknown',1284,'PWN-11D','','','','Aeron',1821,'',68 from dual union all
select 57719,'1984-W198','1984-06-08 02:35:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 5772,'1959-A80','1959-05-21 00:00:00','suborbital spaceplane','failure',1520,'1-A-4','','','','Test',379,'NB-52 003',11 from dual union all
select 57720,'1984-W199','1984-06-12 20:44:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',70 from dual union all
select 57721,'1984-W20','1984-01-16 21:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',61 from dual union all
select 57722,'1984-W200','1984-06-13 02:00:00','suborbital rocket','success',761,'MT135P-R533','','','','Meteo',1332,'',53 from dual union all
select 57723,'1984-W201','1984-06-13 15:35:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 57724,'1984-W202','1984-06-15 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 57725,'1984-W203','1984-06-15 18:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',66 from dual union all
select 57726,'1984-W204','1984-06-15 18:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',70 from dual union all
select 57727,'1984-W205','1984-06-18 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',59 from dual union all
select 57728,'1984-W206','1984-06-19 23:59:00','suborbital rocket','unknown',1284,'PWN-11D','','','','Aeron',1821,'',75 from dual union all
select 57729,'1984-W207','1984-06-20 02:00:00','suborbital rocket','success',761,'MT135P-R534','','','','Meteo',1332,'',55 from dual union all
select 5773,'1959-A81','1959-05-25 00:00:00','atmospheric rocket','success',549,'','Glide Rocket','','','Reentry Test',1821,'',30 from dual union all
select 57730,'1984-W208','1984-06-20 14:25:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',60 from dual union all
select 57731,'1984-W209','1984-06-20 15:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 57732,'1984-W21','1984-01-17 18:36:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1895,'',60 from dual union all
select 57733,'1984-W210','1984-06-22 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 57734,'1984-W211','1984-06-25 13:07:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',73 from dual union all
select 57735,'1984-W212','1984-06-25 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 57736,'1984-W213','1984-06-26 13:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',75 from dual union all
select 57737,'1984-W214','1984-06-27 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 57738,'1984-W215','1984-06-27 23:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',59 from dual union all
select 57739,'1984-W216','1984-06-28 15:52:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',64 from dual union all
select 5774,'1959-A82','1959-05-27 02:37:00','military missile','success',1214,'SM-62A N-3424','','','','OT',224,'',10 from dual union all
select 57740,'1984-W217','1984-06-29 19:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 57741,'1984-W218','1984-07-02 14:29:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',72 from dual union all
select 57742,'1984-W219','1984-07-02 17:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',55 from dual union all
select 57743,'1984-W22','1984-01-18 12:15:00','suborbital rocket','success',1284,'','T 1-0851','','','Aeron',1821,'',74 from dual union all
select 57744,'1984-W220','1984-07-03 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 57745,'1984-W221','1984-07-04 02:00:00','suborbital rocket','success',761,'MT135P-R535','','','','Meteo',1332,'',54 from dual union all
select 57746,'1984-W222','1984-07-05 14:25:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',71 from dual union all
select 57747,'1984-W223','1984-07-05 20:03:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',53 from dual union all
select 57748,'1984-W224','1984-07-06 12:43:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',61 from dual union all
select 57749,'1984-W225','1984-07-06 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 5775,'1959-A83','1959-05-27 13:47:00','military missile','success',256,'631-1','','','','OT',272,'',20 from dual union all
select 57750,'1984-W226','1984-07-06 20:25:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',63 from dual union all
select 57751,'1984-W227','1984-07-09 21:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',58 from dual union all
select 57752,'1984-W228','1984-07-11 12:48:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',50 from dual union all
select 57753,'1984-W229','1984-07-11 13:25:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 57754,'1984-W23','1984-01-18 14:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',60 from dual union all
select 57755,'1984-W230','1984-07-11 22:47:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',73 from dual union all
select 57756,'1984-W231','1984-07-16 12:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',60 from dual union all
select 57757,'1984-W232','1984-07-16 14:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',60 from dual union all
select 57758,'1984-W233','1984-07-18 02:00:00','suborbital rocket','success',761,'MT135P-R536','','','','Meteo',1332,'',53 from dual union all
select 57759,'1984-W234','1984-07-18 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',56 from dual union all
select 5776,'1959-A84','1959-05-27 00:00:00','atmospheric rocket','success',542,'','Corvus Model','','','Test',1821,'',5 from dual union all
select 57760,'1984-W235','1984-07-20 14:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 57761,'1984-W236','1984-07-20 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 57762,'1984-W237','1984-07-23 21:26:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',63 from dual union all
select 57763,'1984-W238','1984-07-24 20:55:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',73 from dual union all
select 57764,'1984-W239','1984-07-25 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',51 from dual union all
select 57765,'1984-W24','1984-01-18 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',276,'',62 from dual union all
select 57766,'1984-W240','1984-07-25 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 57767,'1984-W241','1984-07-27 19:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',53 from dual union all
select 57768,'1984-W242','1984-08-01 02:00:00','suborbital rocket','success',761,'MT135P-R537','','','','Meteo',1332,'',52 from dual union all
select 57769,'1984-W243','1984-08-01 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 5777,'1959-A86','1959-06-01 00:00:00','military missile','unknown',339,'','','','','Test',434,'',40 from dual union all
select 57770,'1984-W244','1984-08-01 15:33:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',53 from dual union all
select 57771,'1984-W245','1984-08-01 17:42:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',60 from dual union all
select 57772,'1984-W246','1984-08-03 18:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',59 from dual union all
select 57773,'1984-W247','1984-08-07 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',60 from dual union all
select 57774,'1984-W248','1984-08-07 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',56 from dual union all
select 57775,'1984-W249','1984-08-08 21:33:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',53 from dual union all
select 57776,'1984-W25','1984-01-18 20:00:00','suborbital rocket','success',1284,'MAP/WINE','M-D 18A','','','Aeron',29,'',76 from dual union all
select 57777,'1984-W250','1984-08-08 23:32:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',71 from dual union all
select 57778,'1984-W251','1984-08-13 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',64 from dual union all
select 57779,'1984-W252','1984-08-14 00:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 5778,'1959-A87','1959-06-01 00:00:00','military missile','unknown',339,'','','','','Test',434,'',40 from dual union all
select 57780,'1984-W253','1984-08-15 09:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 57781,'1984-W254','1984-08-15 12:47:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',58 from dual union all
select 57782,'1984-W255','1984-08-15 15:11:00','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',67 from dual union all
select 57783,'1984-W256','1984-08-15 23:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',76 from dual union all
select 57784,'1984-W257','1984-08-17 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 57785,'1984-W258','1984-08-17 20:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',58 from dual union all
select 57786,'1984-W259','1984-08-21 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',54 from dual union all
select 57787,'1984-W26','1984-01-19 02:00:00','suborbital rocket','success',761,'MT135P-R518','','','','Meteo',1332,'',55 from dual union all
select 57788,'1984-W260','1984-08-21 20:31:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',72 from dual union all
select 57789,'1984-W261','1984-08-22 16:51:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual;
insert into LAUNCH
select 5779,'1959-A88','1959-06-01 00:00:00','military missile','unknown',339,'','','','','Test',434,'',40 from dual union all
select 57790,'1984-W262','1984-08-23 09:26:59','suborbital rocket','success',1284,'VLF-TE support','Datasonde','','','Meteo',1821,'',60 from dual union all
select 57791,'1984-W263','1984-08-23 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',60 from dual union all
select 57792,'1984-W264','1984-08-24 17:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',71 from dual union all
select 57793,'1984-W265','1984-08-24 19:01:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',65 from dual union all
select 57794,'1984-W266','1984-08-26 14:35:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',60 from dual union all
select 57795,'1984-W267','1984-08-27 14:35:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',60 from dual union all
select 57796,'1984-W268','1984-08-29 02:00:00','suborbital rocket','success',761,'MT135P-R539','','','','Meteo',1332,'',53 from dual union all
select 57797,'1984-W269','1984-08-29 19:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',59 from dual union all
select 57798,'1984-W27','1984-01-20 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 57799,'1984-W270','1984-08-29 23:26:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',75 from dual union all
select 578,'1945-S54','1945-01-20 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5780,'1959-A89','1959-06-01 00:00:00','military missile','unknown',339,'','','','','Test',434,'',40 from dual union all
select 57800,'1984-W271','1984-08-30 13:36:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',54 from dual union all
select 57801,'1984-W272','1984-08-30 14:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',74 from dual union all
select 57802,'1984-W273','1984-08-30 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',62 from dual union all
select 57803,'1984-W274','1984-08-30 14:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 57804,'1984-W275','1984-08-30 19:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 57805,'1984-W276','1984-08-31 00:29:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',59 from dual union all
select 57806,'1984-W277','1984-08-31 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',54 from dual union all
select 57807,'1984-W278','1984-08-31 16:53:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',55 from dual union all
select 57808,'1984-W279','1984-09-01 14:24:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',64 from dual union all
select 57809,'1984-W28','1984-01-21 14:44:00','suborbital rocket','success',1284,'MAP/WINE','M-D 19A','','','Aeron',29,'',73 from dual union all
select 5781,'1959-A90','1959-06-01 00:00:00','military missile','unknown',339,'','','','','Test',434,'',40 from dual union all
select 57810,'1984-W280','1984-09-01 14:31:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 57811,'1984-W281','1984-09-01 14:38:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',64 from dual union all
select 57812,'1984-W282','1984-09-03 14:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',66 from dual union all
select 57813,'1984-W283','1984-09-03 14:33:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 57814,'1984-W284','1984-09-05 08:26:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',75 from dual union all
select 57815,'1984-W285','1984-09-05 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',65 from dual union all
select 57816,'1984-W286','1984-09-05 13:37:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',73 from dual union all
select 57817,'1984-W287','1984-09-05 13:38:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',56 from dual union all
select 57818,'1984-W288','1984-09-05 13:52:00','suborbital rocket','unknown',1284,'PWN-11D','PWN-11D/TI-904','','','Test/Aeron',1821,'',72 from dual union all
select 57819,'1984-W289','1984-09-05 14:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',67 from dual union all
select 5782,'1959-A91','1959-06-01 00:00:00','military missile','success',1080,'F88','','','','Test',671,'',40 from dual union all
select 57820,'1984-W29','1984-01-21 16:37:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 57821,'1984-W290','1984-09-05 14:22:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 57822,'1984-W291','1984-09-05 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',59 from dual union all
select 57823,'1984-W292','1984-09-05 15:08:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',74 from dual union all
select 57824,'1984-W293','1984-09-06 02:00:00','suborbital rocket','success',761,'MT135P-R540','','','','Meteo',1332,'',55 from dual union all
select 57825,'1984-W294','1984-09-10 12:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',52 from dual union all
select 57826,'1984-W295','1984-09-10 16:10:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',67 from dual union all
select 57827,'1984-W296','1984-09-10 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 57828,'1984-W297','1984-09-10 20:10:00','suborbital rocket','success',1284,'','PWN-11D/TI-889','','','Aeron',1821,'',70 from dual union all
select 57829,'1984-W298','1984-09-12 00:36:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',76 from dual union all
select 5783,'1959-A92','1959-06-05 00:00:00','military missile','success',1080,'F76','','','','Test',671,'',40 from dual union all
select 57830,'1984-W299','1984-09-12 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',51 from dual union all
select 57831,'1984-W30','1984-01-21 18:17:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',75 from dual union all
select 57832,'1984-W300','1984-09-13 19:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',52 from dual union all
select 57833,'1984-W301','1984-09-14 21:43:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',67 from dual union all
select 57834,'1984-W302','1984-09-18 12:46:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',55 from dual union all
select 57835,'1984-W303','1984-09-18 14:00:00','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',69 from dual union all
select 57836,'1984-W304','1984-09-19 02:00:00','suborbital rocket','success',761,'MT135P-R541','','','','Meteo',1332,'',54 from dual union all
select 57837,'1984-W305','1984-09-19 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',53 from dual union all
select 57838,'1984-W306','1984-09-20 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 57839,'1984-W307','1984-09-20 21:45:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',62 from dual union all
select 5784,'1959-A93','1959-06-08 16:38:40','suborbital spaceplane','success',1520,'1-1-5','','','','Test',379,'NB-52 003',11 from dual union all
select 57840,'1984-W308','1984-09-25 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',53 from dual union all
select 57841,'1984-W309','1984-09-26 15:45:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',59 from dual union all
select 57842,'1984-W31','1984-01-21 20:02:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',51 from dual union all
select 57843,'1984-W310','1984-09-26 20:37:00','suborbital rocket','unknown',1284,'PWN-11D','PWN-11D','','','Test/Aeron',1821,'',78 from dual union all
select 57844,'1984-W311','1984-09-26 21:51:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',65 from dual union all
select 57845,'1984-W312','1984-09-26 23:35:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',77 from dual union all
select 57846,'1984-W313','1984-09-27 19:38:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 57847,'1984-W314','1984-09-29 00:35:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',68 from dual union all
select 57848,'1984-W315','1984-10-03 15:22:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',59 from dual union all
select 57849,'1984-W316','1984-10-03 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',66 from dual union all
select 5785,'1959-A95','1959-06-10 00:00:00','military missile','success',771,'Training #17','','','','Test',222,'',12 from dual union all
select 57850,'1984-W317','1984-10-04 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',69 from dual union all
select 57851,'1984-W318','1984-10-05 11:27:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',63 from dual union all
select 57852,'1984-W319','1984-10-05 12:15:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',63 from dual union all
select 57853,'1984-W32','1984-01-23 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',64 from dual union all
select 57854,'1984-W320','1984-10-05 14:03:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 57855,'1984-W321','1984-10-05 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 57856,'1984-W322','1984-10-07 16:18:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',61 from dual union all
select 57857,'1984-W323','1984-10-07 16:18:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',73 from dual union all
select 57858,'1984-W324','1984-10-07 16:21:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 57859,'1984-W325','1984-10-09 15:42:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 5786,'1959-A96','1959-06-10 00:00:00','military missile','success',771,'Training #18','','','','Test',222,'',12 from dual union all
select 57860,'1984-W326','1984-10-09 16:21:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',63 from dual union all
select 57861,'1984-W327','1984-10-09 17:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 57862,'1984-W328','1984-10-09 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 57863,'1984-W329','1984-10-10 02:00:00','suborbital rocket','success',761,'MT135P-R542','','','','Meteo',1332,'',55 from dual union all
select 57864,'1984-W33','1984-01-23 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',52 from dual union all
select 57865,'1984-W330','1984-10-11 16:18:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',60 from dual union all
select 57866,'1984-W331','1984-10-11 16:21:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 57867,'1984-W332','1984-10-11 16:21:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',66 from dual union all
select 57868,'1984-W333','1984-10-11 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',51 from dual union all
select 57869,'1984-W334','1984-10-13 10:54:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',61 from dual union all
select 5787,'1959-A97','1959-06-11 00:00:00','military missile','success',771,'Training #19','','','','Test',222,'',12 from dual union all
select 57870,'1984-W335','1984-10-13 16:21:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',66 from dual union all
select 57871,'1984-W336','1984-10-13 16:58:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 57872,'1984-W337','1984-10-13 16:59:00','suborbital rocket','success',1284,'','PWN-11D/TI-964','','','Aeron',1821,'',79 from dual union all
select 57873,'1984-W338','1984-10-13 17:08:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 57874,'1984-W339','1984-10-13 17:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 57875,'1984-W34','1984-01-24 19:31:00','suborbital rocket','success',1284,'MAP/WINE','M-D 20A','','','Aeron',29,'',72 from dual union all
select 57876,'1984-W340','1984-10-13 17:34:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',64 from dual union all
select 57877,'1984-W341','1984-10-15 21:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',64 from dual union all
select 57878,'1984-W342','1984-10-17 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',58 from dual union all
select 57879,'1984-W343','1984-10-17 13:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',60 from dual;
insert into LAUNCH
select 5788,'1959-A98','1959-06-11 00:00:00','military missile','success',771,'Training #20','','','','Test',222,'',12 from dual union all
select 57880,'1984-W344','1984-10-17 13:51:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',59 from dual union all
select 57881,'1984-W345','1984-10-17 16:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',66 from dual union all
select 57882,'1984-W346','1984-10-17 16:53:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',57 from dual union all
select 57883,'1984-W347','1984-10-18 02:00:00','suborbital rocket','success',761,'MT135P-R543','','','','Meteo',1332,'',55 from dual union all
select 57884,'1984-W348','1984-10-18 12:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',65 from dual union all
select 57885,'1984-W349','1984-10-22 12:45:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',59 from dual union all
select 57886,'1984-W35','1984-01-25 02:00:00','suborbital rocket','success',761,'MT135P-R519','','','','Meteo',1332,'',57 from dual union all
select 57887,'1984-W350','1984-10-22 15:30:00','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',64 from dual union all
select 57888,'1984-W351','1984-10-24 02:01:00','suborbital rocket','success',761,'MT135P-R544','','','','Meteo',1332,'',56 from dual union all
select 57889,'1984-W352','1984-10-24 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',56 from dual union all
select 5789,'1959-A99','1959-06-12 18:57:00','military missile','failure',933,'AX-10','','','','Test',244,'',10 from dual union all
select 57890,'1984-W353','1984-10-24 18:33:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',59 from dual union all
select 57891,'1984-W354','1984-10-24 21:56:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',61 from dual union all
select 57892,'1984-W355','1984-10-26 09:15:00','suborbital rocket','success',1284,'','PWN-11D','','','Aeron',1821,'',79 from dual union all
select 57893,'1984-W356','1984-10-26 15:51:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',65 from dual union all
select 57894,'1984-W357','1984-10-26 16:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',61 from dual union all
select 57895,'1984-W358','1984-10-27 00:29:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 57896,'1984-W359','1984-10-29 21:27:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',72 from dual union all
select 57897,'1984-W36','1984-01-25 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',276,'',60 from dual union all
select 57898,'1984-W360','1984-10-31 02:01:00','suborbital rocket','success',761,'MT135P-R545','','','','Meteo',1332,'',55 from dual union all
select 57899,'1984-W361','1984-10-31 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 579,'1945-S55','1945-01-20 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5790,'1959-E01','1959-01-21 00:00:00','orbital','destroyed before launch',1360,'160/FTV 1019','Discoverer 0','','','Sat TAG0',1678,'',0 from dual union all
select 57900,'1984-W362','1984-10-31 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 57901,'1984-W363','1984-10-31 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',51 from dual union all
select 57902,'1984-W364','1984-10-31 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',61 from dual union all
select 57903,'1984-W365','1984-10-31 18:57:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',73 from dual union all
select 57904,'1984-W366','1984-10-31 23:09:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',76 from dual union all
select 57905,'1984-W367','1984-11-05 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',54 from dual union all
select 57906,'1984-W368','1984-11-06 15:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',61 from dual union all
select 57907,'1984-W369','1984-11-07 02:00:00','suborbital rocket','success',761,'MT135P-R546','','','','Meteo',1332,'',56 from dual union all
select 57908,'1984-W37','1984-01-25 18:45:00','suborbital rocket','success',1284,'MAP/WINE','M-D 21A','','','Aeron',29,'',78 from dual union all
select 57909,'1984-W370','1984-11-07 21:08:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',65 from dual union all
select 5791,'1959-E02','1959-09-24 15:12:00','orbital','destroyed before launch',151,'9C','Static Test','Able IVA','','Sat',227,'',0 from dual union all
select 57910,'1984-W371','1984-11-08 12:38:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',58 from dual union all
select 57911,'1984-W372','1984-11-08 12:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',53 from dual union all
select 57912,'1984-W373','1984-11-08 13:27:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',61 from dual union all
select 57913,'1984-W374','1984-11-08 13:55:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 57914,'1984-W375','1984-11-09 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',52 from dual union all
select 57915,'1984-W376','1984-11-10 11:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',74 from dual union all
select 57916,'1984-W377','1984-11-10 12:19:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',65 from dual union all
select 57917,'1984-W378','1984-11-10 12:21:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',61 from dual union all
select 57918,'1984-W379','1984-11-10 12:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 57919,'1984-W38','1984-01-26 20:24:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',76 from dual union all
select 5792,'1959-F01','1959-04-14 02:49:46','orbital','failure',1486,'SLV-5','Vanguard SLV5','Magnetometer satellite','','Sat',235,'',100 from dual union all
select 57920,'1984-W380','1984-11-12 11:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 57921,'1984-W381','1984-11-12 12:25:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',66 from dual union all
select 57922,'1984-W382','1984-11-13 15:17:00','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',60 from dual union all
select 57923,'1984-W383','1984-11-13 16:48:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 57924,'1984-W384','1984-11-13 19:42:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',68 from dual union all
select 57925,'1984-W385','1984-11-13 21:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',58 from dual union all
select 57926,'1984-W386','1984-11-14 02:00:00','suborbital rocket','success',761,'MT135P-R547','','','','Meteo',1332,'',55 from dual union all
select 57927,'1984-W387','1984-11-14 11:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 57928,'1984-W388','1984-11-14 11:57:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',63 from dual union all
select 57929,'1984-W389','1984-11-14 12:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',72 from dual union all
select 5793,'1959-F02','1959-06-03 20:09:20','orbital','failure',1360,'174/FTV 1020','Discoverer 3','CORONA BIO 2','GOLD DUKE','Sat TAG3',1678,'',250 from dual union all
select 57930,'1984-W39','1984-01-26 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 57931,'1984-W390','1984-11-14 12:51:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',59 from dual union all
select 57932,'1984-W391','1984-11-14 12:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 57933,'1984-W392','1984-11-14 20:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',56 from dual union all
select 57934,'1984-W393','1984-11-16 06:33:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',67 from dual union all
select 57935,'1984-W394','1984-11-16 11:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',53 from dual union all
select 57936,'1984-W395','1984-11-16 11:57:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',63 from dual union all
select 57937,'1984-W396','1984-11-16 12:15:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',76 from dual union all
select 57938,'1984-W397','1984-11-16 12:17:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 57939,'1984-W398','1984-11-16 12:22:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',66 from dual union all
select 5794,'1959-F03','1959-06-22 20:16:09','orbital','failure',1486,'SLV-6','Vanguard SLV6','Radiation Balance sat','','Sat',235,'',140 from dual union all
select 57940,'1984-W399','1984-11-16 12:35:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',59 from dual union all
select 57941,'1984-W40','1984-01-27 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',70 from dual union all
select 57942,'1984-W400','1984-11-16 12:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 57943,'1984-W401','1984-11-16 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',52 from dual union all
select 57944,'1984-W402','1984-11-19 15:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 57945,'1984-W403','1984-11-20 12:56:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',58 from dual union all
select 57946,'1984-W404','1984-11-20 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 57947,'1984-W405','1984-11-20 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',54 from dual union all
select 57948,'1984-W406','1984-11-21 02:04:59','suborbital rocket','success',761,'MT135P-R548','','','','Meteo',1332,'',52 from dual union all
select 57949,'1984-W407','1984-11-21 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',67 from dual union all
select 5795,'1959-F05','1959-07-16 17:37:03','orbital','failure',634,'AM-16','NASA S-1','NASA S-1','','Sat',279,'',0 from dual union all
select 57950,'1984-W408','1984-11-21 16:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',63 from dual union all
select 57951,'1984-W409','1984-11-21 23:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',77 from dual union all
select 57952,'1984-W41','1984-01-27 21:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',55 from dual union all
select 57953,'1984-W410','1984-11-26 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',59 from dual union all
select 57954,'1984-W411','1984-11-28 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 57955,'1984-W412','1984-11-28 21:21:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',60 from dual union all
select 57956,'1984-W413','1984-11-28 22:47:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 57957,'1984-W414','1984-11-29 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',62 from dual union all
select 57958,'1984-W415','1984-11-29 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',67 from dual union all
select 57959,'1984-W416','1984-11-30 18:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 5796,'1959-F06','1959-06-18 08:08:00','orbital','failure',1513,'I1-7','Luna','E-1 No. 5','','Sat',480,'',80 from dual union all
select 57960,'1984-W417','1984-12-04 19:27:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',72 from dual union all
select 57961,'1984-W418','1984-12-04 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',52 from dual union all
select 57962,'1984-W419','1984-12-04 22:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',70 from dual union all
select 57963,'1984-W42','1984-01-28 17:30:00','suborbital rocket','success',1284,'MAP/WINE','M-D 22A','','','Aeron',29,'',74 from dual union all
select 57964,'1984-W420','1984-12-05 02:04:59','suborbital rocket','success',761,'MT135P-R549','','','','Meteo',1332,'',54 from dual union all
select 57965,'1984-W421','1984-12-05 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',68 from dual union all
select 57966,'1984-W422','1984-12-05 14:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 57967,'1984-W423','1984-12-05 22:02:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',64 from dual union all
select 57968,'1984-W424','1984-12-10 15:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',71 from dual union all
select 57969,'1984-W425','1984-12-10 16:41:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',61 from dual;
insert into LAUNCH
select 5797,'1959-F07','1959-08-15 00:31:00','orbital','failure',633,'AM-19B','Beacon 2','Beacon','','Sat',249,'',831 from dual union all
select 57970,'1984-W426','1984-12-10 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',55 from dual union all
select 57971,'1984-W427','1984-12-11 14:17:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 57972,'1984-W428','1984-12-12 02:20:00','suborbital rocket','success',761,'MT135P-R550','','','','Meteo',1332,'',56 from dual union all
select 57973,'1984-W429','1984-12-12 15:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',72 from dual union all
select 57974,'1984-W43','1984-01-31 16:22:00','suborbital rocket','success',1284,'MAP/WINE','M-D 23A','','','Aeron',29,'',61 from dual union all
select 57975,'1984-W430','1984-12-12 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',66 from dual union all
select 57976,'1984-W431','1984-12-12 23:48:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',77 from dual union all
select 57977,'1984-W432','1984-12-14 21:33:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',67 from dual union all
select 57978,'1984-W433','1984-12-14 21:43:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',77 from dual union all
select 57979,'1984-W434','1984-12-15 23:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',66 from dual union all
select 5798,'1959-F08','1959-09-17 14:34:00','orbital','failure',1354,'136','Transit 1A','Transit 1A','','Sat',233,'',640 from dual union all
select 57980,'1984-W435','1984-12-17 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',66 from dual union all
select 57981,'1984-W436','1984-12-18 12:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',61 from dual union all
select 57982,'1984-W437','1984-12-18 20:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',64 from dual union all
select 57983,'1984-W438','1984-12-19 02:00:00','suborbital rocket','success',761,'MT135P-R551','','','','Meteo',1332,'',54 from dual union all
select 57984,'1984-W439','1984-12-19 13:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',54 from dual union all
select 57985,'1984-W44','1984-02-01 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',53 from dual union all
select 57986,'1984-W440','1984-12-19 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 57987,'1984-W441','1984-12-19 21:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',68 from dual union all
select 57988,'1984-W442','1984-12-19 22:54:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',69 from dual union all
select 57989,'1984-W443','1984-12-24 12:56:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',61 from dual union all
select 5799,'1959-F09','1959-11-26 07:26:00','orbital','failure',143,'20D','Able IVB','Able IVB','','Sat',229,'',1000 from dual union all
select 57990,'1984-W444','1984-12-24 12:57:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',58 from dual union all
select 57991,'1984-W445','1984-12-24 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',56 from dual union all
select 57992,'1984-W446','1984-12-31 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',56 from dual union all
select 57993,'1984-W447','1984-08-15 02:20:00','suborbital rocket','success',761,'MT135P-R538','','','','Meteo',1332,'',60 from dual union all
select 57994,'1984-W45','1984-02-02 00:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',76 from dual union all
select 57995,'1984-W46','1984-02-03 13:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',76 from dual union all
select 57996,'1984-W47','1984-02-03 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',276,'',71 from dual union all
select 57997,'1984-W48','1984-02-03 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',276,'',71 from dual union all
select 57998,'1984-W49','1984-02-03 21:18:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',65 from dual union all
select 57999,'1984-W50','1984-02-05 12:38:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',76 from dual union all
select 58,'1944-A22','1944-11-25 11:06:00','military missile','failure',1470,'19835','Ma298','','','Test',661,'',30 from dual union all
select 580,'1945-S56','1945-01-20 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5800,'1959-M01','1959-01-01 00:00:00','suborbital rocket','success',14,'','Model A (F1)','','','Test',383,'F-104',74 from dual union all
select 58000,'1984-W51','1984-02-06 20:00:00','suborbital rocket','success',1284,'MAP/WINE','M-D 25A','','','Aeron',29,'',77 from dual union all
select 58001,'1984-W52','1984-02-06 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 58002,'1984-W53','1984-02-07 12:44:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',74 from dual union all
select 58003,'1984-W54','1984-02-08 02:01:00','suborbital rocket','success',761,'MT135P-R520','','','','Meteo',1332,'',58 from dual union all
select 58004,'1984-W55','1984-02-08 23:08:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',70 from dual union all
select 58005,'1984-W56','1984-02-09 00:52:00','suborbital rocket','success',1284,'MAP/WINE','M-D 26A','','','Aeron',29,'',75 from dual union all
select 58006,'1984-W57','1984-02-09 13:09:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',74 from dual union all
select 58007,'1984-W58','1984-02-09 17:03:59','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',75 from dual union all
select 58008,'1984-W59','1984-02-09 19:22:59','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 58009,'1984-W60','1984-02-09 21:44:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',52 from dual union all
select 5801,'1959-M02','1959-01-01 00:00:00','suborbital rocket','success',14,'','NOTS Viperscan','','','Test',383,'F-104',71 from dual union all
select 58010,'1984-W61','1984-02-10 01:07:00','suborbital rocket','success',1284,'MAP/WINE','M-D 27A','','','Aeron',29,'',75 from dual union all
select 58011,'1984-W62','1984-02-10 12:52:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',65 from dual union all
select 58012,'1984-W63','1984-02-10 16:41:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',59 from dual union all
select 58013,'1984-W64','1984-02-11 07:03:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 58014,'1984-W65','1984-02-11 12:22:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 58015,'1984-W66','1984-02-11 12:24:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',67 from dual union all
select 58016,'1984-W67','1984-02-11 12:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',71 from dual union all
select 58017,'1984-W68','1984-02-11 12:30:00','suborbital rocket','unknown',716,'T 1-0785','','','','Aeron',1821,'',51 from dual union all
select 58018,'1984-W69','1984-02-11 14:25:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',276,'',68 from dual union all
select 58019,'1984-W70','1984-02-11 16:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',71 from dual union all
select 5802,'1959-M03','1959-01-19 15:42:00','military missile','failure',933,'AX-4','','','','Test',244,'',40 from dual union all
select 58020,'1984-W71','1984-02-11 20:00:00','suborbital rocket','success',1284,'MAP/WINE','M-D 28A','','','Aeron',29,'',66 from dual union all
select 58021,'1984-W72','1984-02-13 15:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',69 from dual union all
select 58022,'1984-W73','1984-02-13 20:00:00','suborbital rocket','success',1284,'MAP/WINE','M-D 29A','','','Aeron',29,'',77 from dual union all
select 58023,'1984-W74','1984-02-13 20:36:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 58024,'1984-W75','1984-02-15 02:01:00','suborbital rocket','success',761,'MT135P-R521','','','','Meteo',1332,'',54 from dual union all
select 58025,'1984-W76','1984-02-15 23:35:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',75 from dual union all
select 58026,'1984-W77','1984-02-16 02:28:00','suborbital rocket','success',1284,'MAP/WINE','M-D 30A','','','Aeron',29,'',74 from dual union all
select 58027,'1984-W78','1984-02-16 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',55 from dual union all
select 58028,'1984-W79','1984-02-17 20:35:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',53 from dual union all
select 58029,'1984-W80','1984-02-18 01:49:00','suborbital rocket','success',1284,'MAP/WINE','M-D 31A','','','Aeron',29,'',76 from dual union all
select 5803,'1959-M04','1959-03-23 00:00:00','military missile','success',1162,'21','JPL-500','','','OT?',1895,'',50 from dual union all
select 58030,'1984-W81','1984-02-19 10:26:00','suborbital rocket','success',1284,'MAP/WINE','M-D 32A','','','Aeron',29,'',67 from dual union all
select 58031,'1984-W82','1984-02-19 11:52:00','suborbital rocket','unknown',1284,'M-D 033A','','','','Aeron',29,'',77 from dual union all
select 58032,'1984-W83','1984-02-20 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 58033,'1984-W84','1984-02-21 12:55:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',63 from dual union all
select 58034,'1984-W85','1984-02-21 20:05:00','suborbital rocket','unknown',1284,'M-D 034A','','','','Aeron',29,'',77 from dual union all
select 58035,'1984-W86','1984-02-21 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',53 from dual union all
select 58036,'1984-W87','1984-02-22 02:02:00','suborbital rocket','success',761,'MT135P-R522','','','','Meteo',1332,'',54 from dual union all
select 58037,'1984-W88','1984-02-22 18:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1895,'',62 from dual union all
select 58038,'1984-W89','1984-02-23 00:02:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',78 from dual union all
select 58039,'1984-W90','1984-02-23 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',276,'',61 from dual union all
select 5804,'1959-M05','1959-03-25 00:00:00','military missile','success',1162,'22','','','','OT?',1895,'',50 from dual union all
select 58040,'1984-W91','1984-02-23 20:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',77 from dual union all
select 58041,'1984-W92','1984-02-24 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',70 from dual union all
select 58042,'1984-W93','1984-02-27 14:52:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1895,'',59 from dual union all
select 58043,'1984-W94','1984-02-27 15:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',276,'',56 from dual union all
select 58044,'1984-W95','1984-02-29 15:21:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',62 from dual union all
select 58045,'1984-W96','1984-02-29 16:51:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',63 from dual union all
select 58046,'1984-W97','1984-02-29 20:27:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',74 from dual union all
select 58047,'1984-W98','1984-02-29 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 58048,'1984-W99','1984-02-29 23:59:00','suborbital rocket','unknown',716,'T 1-0863','','','','Aeron',1821,'',59 from dual union all
select 58049,'1984-Y01','1984-01-02 14:30:00','suborbital rocket','success',1502,'MAP/WINE','M-F 13','','','Aeron',29,'',119 from dual union all
select 5805,'1959-M06','1959-03-13 07:30:00','suborbital rocket','success',366,'','RW-120','','','Aeron',1490,'',69 from dual union all
select 58050,'1984-Y02','1984-01-04 17:03:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 58051,'1984-Y03','1984-01-05 20:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 14','','','Aeron',29,'',120 from dual union all
select 58052,'1984-Y04','1984-01-06 20:52:00','suborbital rocket','success',1284,'MAP/WINE','M-C 5','','','Aeron',29,'',96 from dual union all
select 58053,'1984-Y05','1984-01-06 21:24:00','suborbital rocket','success',1502,'MAP/WINE','M-F 15','','','Aeron',29,'',119 from dual union all
select 58054,'1984-Y06','1984-01-06 22:10:00','suborbital rocket','success',1284,'MAP/WINE','M-C 6','','','Aeron',29,'',85 from dual union all
select 58055,'1984-Y07','1984-01-08 20:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 16','','','Aeron',29,'',118 from dual union all
select 58056,'1984-Y08','1984-01-10 19:45:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 58057,'1984-Y09','1984-01-12 17:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 17','','','Aeron',29,'',118 from dual union all
select 58058,'1984-Y10','1984-01-12 18:37:00','suborbital rocket','success',1284,'MAP/WINE','M-D 14A','','','Aeron',29,'',88 from dual;
insert into LAUNCH
select 58059,'1984-Y100','1984-08-30 12:43:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 5806,'1959-M07','1959-03-17 01:35:00','suborbital rocket','success',662,'K-6-RS-3','K-6-14','','','Solar UV',7,'',60 from dual union all
select 58060,'1984-Y101','1984-09-01 13:30:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',85 from dual union all
select 58061,'1984-Y102','1984-09-01 13:38:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',84 from dual union all
select 58062,'1984-Y103','1984-09-01 14:15:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 58063,'1984-Y104','1984-09-03 13:30:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',86 from dual union all
select 58064,'1984-Y105','1984-09-03 13:34:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 58065,'1984-Y106','1984-09-03 13:38:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 58066,'1984-Y107','1984-09-05 07:56:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',88 from dual union all
select 58067,'1984-Y108','1984-09-05 13:24:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 58068,'1984-Y109','1984-09-05 13:25:00','suborbital rocket','success',1284,'','PWN-12A/TI-903','','','Aeron',1821,'',108 from dual union all
select 58069,'1984-Y11','1984-01-12 19:20:00','suborbital rocket','success',1284,'MAP/WINE','M-D 15A','','','Aeron',29,'',84 from dual union all
select 5807,'1959-M08','1959-03-18 02:45:00','suborbital rocket','success',662,'K-6-TW-6','K-6-15','Grenade','','Aeron',7,'',50 from dual union all
select 58070,'1984-Y110','1984-09-05 13:36:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 58071,'1984-Y111','1984-09-05 13:38:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 58072,'1984-Y112','1984-09-05 14:34:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 58073,'1984-Y113','1984-09-20 20:54:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',88 from dual union all
select 58074,'1984-Y114','1984-09-20 21:32:00','suborbital rocket','success',1281,'NASA 15.231UE','','','','Plasma',1821,'',80 from dual union all
select 58075,'1984-Y115','1984-09-26 20:59:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',85 from dual union all
select 58076,'1984-Y116','1984-10-04 13:11:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',86 from dual union all
select 58077,'1984-Y117','1984-10-04 20:18:00','suborbital rocket','unknown',1284,'PWD-11D','PWN-11D','','','Test/Aeron',1821,'',80 from dual union all
select 58078,'1984-Y118','1984-10-05 11:03:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 58079,'1984-Y119','1984-10-05 12:07:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',84 from dual union all
select 5808,'1959-M09','1959-03-19 01:15:00','suborbital rocket','success',662,'K-6-RS-4','K-6-16','','','Solar UV',7,'',50 from dual union all
select 58080,'1984-Y12','1984-01-13 18:57:00','suborbital rocket','success',1502,'MAP/WINE','M-F 19','','','Aeron',29,'',120 from dual union all
select 58081,'1984-Y120','1984-10-13 10:25:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 58082,'1984-Y121','1984-10-13 16:38:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 58083,'1984-Y122','1984-10-13 16:39:00','suborbital rocket','success',1284,'','PWN-12A/TI-963','','','Aeron',1821,'',114 from dual union all
select 58084,'1984-Y123','1984-10-13 16:40:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 58085,'1984-Y124','1984-10-13 16:52:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',85 from dual union all
select 58086,'1984-Y125','1984-10-13 18:07:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 58087,'1984-Y126','1984-11-08 12:14:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 58088,'1984-Y127','1984-11-08 13:14:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',84 from dual union all
select 58089,'1984-Y128','1984-11-16 05:59:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 5809,'1959-M10','1959-03-20 02:50:00','suborbital rocket','success',662,'K-6-TW-7','K-6-17','Grenade','','Aeron',7,'',50 from dual union all
select 58090,'1984-Y129','1984-11-16 11:49:00','suborbital rocket','unknown',1284,'PWN-12A','PWN-12A','','','Test/Aeron',1821,'',113 from dual union all
select 58091,'1984-Y13','1984-01-13 19:31:00','suborbital rocket','success',1502,'MAP/WINE','M-F 20','','','Aeron',29,'',117 from dual union all
select 58092,'1984-Y130','1984-11-16 12:12:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',87 from dual union all
select 58093,'1984-Y131','1984-11-16 12:14:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',84 from dual union all
select 58094,'1984-Y132','1984-12-03 13:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',84 from dual union all
select 58095,'1984-Y133','1984-12-04 20:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',83 from dual union all
select 58096,'1984-Y134','1984-12-14 13:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',85 from dual union all
select 58097,'1984-Y135','1984-12-15 21:49:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',85 from dual union all
select 58098,'1984-Y136','1984-02-08 10:39:00','suborbital rocket','success',725,'','Gerdien probe','','','Aeron',1467,'',85 from dual union all
select 58099,'1984-Y137','1984-01-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 581,'1945-S57','1945-01-20 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5810,'1959-M11','1959-04-16 11:09:00','suborbital rocket','success',366,'','RW-122','','','Aeron',1490,'',66 from dual union all
select 58100,'1984-Y138','1984-01-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58101,'1984-Y139','1984-01-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58102,'1984-Y14','1984-01-13 20:18:00','suborbital rocket','success',1284,'MAP/WINE','M-C 7','','','Aeron',29,'',87 from dual union all
select 58103,'1984-Y140','1984-01-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58104,'1984-Y141','1984-02-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58105,'1984-Y142','1984-02-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58106,'1984-Y143','1984-02-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58107,'1984-Y144','1984-02-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58108,'1984-Y145','1984-03-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58109,'1984-Y146','1984-03-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 5811,'1959-M12','1959-04-16 17:00:00','suborbital rocket','success',366,'','RW-123','','','Aeron',1490,'',79 from dual union all
select 58110,'1984-Y147','1984-03-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58111,'1984-Y148','1984-03-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58112,'1984-Y149','1984-04-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58113,'1984-Y15','1984-01-13 20:50:00','suborbital rocket','success',1502,'MAP/WINE','M-F 21','','','Aeron',29,'',119 from dual union all
select 58114,'1984-Y150','1984-04-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58115,'1984-Y151','1984-04-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58116,'1984-Y152','1984-04-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58117,'1984-Y153','1984-05-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58118,'1984-Y154','1984-05-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58119,'1984-Y155','1984-05-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 5812,'1959-M13','1959-05-14 17:55:00','suborbital rocket','success',89,'NASA 2.01GT','','','','Test',1821,'',68 from dual union all
select 58120,'1984-Y156','1984-05-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58121,'1984-Y157','1984-05-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58122,'1984-Y158','1984-06-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58123,'1984-Y159','1984-06-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58124,'1984-Y16','1984-01-13 21:22:00','suborbital rocket','success',1502,'MAP/WINE','M-F 22','','','Aeron',29,'',116 from dual union all
select 58125,'1984-Y160','1984-06-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58126,'1984-Y161','1984-06-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58127,'1984-Y162','1984-07-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58128,'1984-Y163','1984-07-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58129,'1984-Y164','1984-07-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 5813,'1959-M14','1959-05-15 17:10:00','suborbital rocket','success',89,'NASA 2.02GT','','','','Test',1821,'',60 from dual union all
select 58130,'1984-Y165','1984-08-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58131,'1984-Y166','1984-08-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58132,'1984-Y167','1984-08-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58133,'1984-Y168','1984-08-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58134,'1984-Y169','1984-08-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58135,'1984-Y17','1984-01-13 22:12:00','suborbital rocket','success',1502,'MAP/WINE','M-F 23','','','Aeron',29,'',121 from dual union all
select 58136,'1984-Y170','1984-09-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58137,'1984-Y171','1984-09-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58138,'1984-Y172','1984-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58139,'1984-Y173','1984-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 5814,'1959-M15','1959-05-21 01:00:00','suborbital rocket','success',839,'II6.20F','','IGY II6.20F','','Fields',1821,'',73 from dual union all
select 58140,'1984-Y174','1984-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58141,'1984-Y175','1984-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 58142,'1984-Y18','1984-01-13 22:44:00','suborbital rocket','success',1502,'MAP/WINE','M-F 24','','','Aeron',29,'',115 from dual union all
select 58143,'1984-Y19','1984-01-15 20:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 25','','','Aeron',29,'',118 from dual union all
select 58144,'1984-Y20','1984-01-17 18:03:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 58145,'1984-Y21','1984-01-17 20:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 26','','','Aeron',29,'',116 from dual union all
select 58146,'1984-Y22','1984-01-20 20:18:00','suborbital rocket','success',1502,'MAP/WINE','M-F 27','','','Aeron',29,'',104 from dual union all
select 58147,'1984-Y23','1984-01-21 10:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 28','','','Aeron',29,'',116 from dual union all
select 58148,'1984-Y24','1984-01-21 10:20:00','suborbital rocket','success',1502,'MAP/WINE','M-F 29','','','Aeron',29,'',115 from dual;
insert into LAUNCH
select 58149,'1984-Y25','1984-01-21 11:07:00','suborbital rocket','success',1502,'MAP/WINE','M-F 30','','','Aeron',29,'',110 from dual union all
select 5815,'1959-M16','1959-05-23 14:30:00','suborbital rocket','success',839,'II6.21F','','IGY II6.21F','','Fields',1821,'',73 from dual union all
select 58150,'1984-Y26','1984-01-21 11:31:00','suborbital rocket','success',1284,'MAP/WINE','M-C 8','','','Aeron',29,'',84 from dual union all
select 58151,'1984-Y27','1984-01-21 12:12:00','suborbital rocket','success',1502,'MAP/WINE','M-F 31','','','Aeron',29,'',113 from dual union all
select 58152,'1984-Y28','1984-01-21 12:46:00','suborbital rocket','success',1502,'MAP/WINE','M-F 32','','','Aeron',29,'',112 from dual union all
select 58153,'1984-Y29','1984-01-21 13:08:00','suborbital rocket','success',1284,'MAP/WINE','M-C 9','','','Aeron',29,'',82 from dual union all
select 58154,'1984-Y30','1984-01-21 13:49:00','suborbital rocket','success',1284,'MAP/WINE','M-C 10','','','Aeron',29,'',85 from dual union all
select 58155,'1984-Y31','1984-01-21 14:24:00','suborbital rocket','success',1502,'MAP/WINE','M-F 33','','','Aeron',29,'',115 from dual union all
select 58156,'1984-Y32','1984-01-24 18:30:00','suborbital rocket','success',1502,'MAP/WINE','M-F 34','','','Aeron',29,'',115 from dual union all
select 58157,'1984-Y33','1984-01-24 18:53:00','suborbital rocket','success',1502,'MAP/WINE','M-F 35','','','Aeron',29,'',113 from dual union all
select 58158,'1984-Y34','1984-01-25 16:09:00','suborbital rocket','success',1502,'MAP/WINE','M-F 36','','','Aeron',29,'',112 from dual union all
select 58159,'1984-Y35','1984-01-25 18:13:00','suborbital rocket','success',1502,'MAP/WINE','M-F 37','','','Aeron',29,'',115 from dual union all
select 5816,'1959-M17','1959-06-04 00:00:00','test rocket','success',575,'','High Virgo','','','Interceptor',21,'B-58',51 from dual union all
select 58160,'1984-Y36','1984-01-28 17:01:00','suborbital rocket','success',1502,'MAP/WINE','M-F 38','','','Aeron',29,'',110 from dual union all
select 58161,'1984-Y37','1984-01-30 18:02:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 58162,'1984-Y38','1984-01-31 17:31:00','suborbital rocket','success',1284,'MAP/WINE','M-C 11','','','Aeron',29,'',87 from dual union all
select 58163,'1984-Y39','1984-01-31 18:23:00','suborbital rocket','success',1502,'MAP/WINE','M-F 39','','','Aeron',29,'',118 from dual union all
select 58164,'1984-Y40','1984-01-31 19:20:00','suborbital rocket','success',1284,'MAP/WINE','M-C 12','','','Aeron',29,'',87 from dual union all
select 58165,'1984-Y41','1984-01-31 20:03:00','suborbital rocket','success',1502,'MAP/WINE','M-F 40','','','Aeron',29,'',115 from dual union all
select 58166,'1984-Y42','1984-02-01 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 58167,'1984-Y43','1984-02-03 12:44:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 58168,'1984-Y44','1984-02-03 20:00:00','suborbital rocket','success',1284,'MAP/WINE','M-D 24A','','','Aeron',29,'',80 from dual union all
select 58169,'1984-Y45','1984-02-05 12:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 5817,'1959-M18','1959-06-06 17:39:00','military missile','failure',154,'5D','Mk 2-2A','R&D','','Test',228,'',80 from dual union all
select 58170,'1984-Y46','1984-02-05 20:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 41','','','Aeron',29,'',114 from dual union all
select 58171,'1984-Y47','1984-02-07 12:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 58172,'1984-Y48','1984-02-07 20:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 42','','','Aeron',29,'',121 from dual union all
select 58173,'1984-Y49','1984-02-08 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 58174,'1984-Y50','1984-02-09 02:16:00','suborbital rocket','success',1284,'MAP/WINE','M-C 13','','','Aeron',29,'',90 from dual union all
select 58175,'1984-Y51','1984-02-09 03:04:00','suborbital rocket','success',1502,'MAP/WINE','M-F 43','','','Aeron',29,'',111 from dual union all
select 58176,'1984-Y52','1984-02-09 12:45:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',93 from dual union all
select 58177,'1984-Y53','1984-02-10 01:51:00','suborbital rocket','success',1284,'MAP/WINE','M-C 14','','','Aeron',29,'',85 from dual union all
select 58178,'1984-Y54','1984-02-10 03:44:00','suborbital rocket','success',1502,'MAP/WINE','M-F 44','','','Aeron',29,'',109 from dual union all
select 58179,'1984-Y55','1984-02-10 04:20:00','suborbital rocket','success',1284,'MAP/WINE','M-C 15','','','Aeron',29,'',85 from dual union all
select 5818,'1959-M19','1959-06-18 07:32:00','suborbital rocket','success',366,'','RW-130','','','Aeron',1490,'',80 from dual union all
select 58180,'1984-Y56','1984-02-10 05:29:00','suborbital rocket','success',1502,'MAP/WINE','M-F 45','','','Aeron',29,'',114 from dual union all
select 58181,'1984-Y57','1984-02-10 20:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 46','','','Aeron',29,'',111 from dual union all
select 58182,'1984-Y58','1984-02-11 12:00:00','suborbital rocket','unknown',1284,'T 1-0784','','','','Aeron',1821,'',109 from dual union all
select 58183,'1984-Y59','1984-02-11 13:19:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',276,'',85 from dual union all
select 58184,'1984-Y60','1984-02-12 20:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 47','','','Aeron',29,'',112 from dual union all
select 58185,'1984-Y61','1984-02-14 20:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 48','','','Aeron',29,'',114 from dual union all
select 58186,'1984-Y62','1984-02-16 01:20:00','suborbital rocket','success',1502,'MAP/WINE','M-F 49','','','Aeron',29,'',117 from dual union all
select 58187,'1984-Y63','1984-02-16 01:58:00','suborbital rocket','success',1502,'MAP/WINE','M-F 50','','','Aeron',29,'',114 from dual union all
select 58188,'1984-Y64','1984-02-16 20:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 51','','','Aeron',29,'',100 from dual union all
select 58189,'1984-Y65','1984-02-18 00:21:00','suborbital rocket','success',1502,'MAP/WINE','M-F 52','','','Aeron',29,'',115 from dual union all
select 5819,'1959-M20','1959-06-27 13:08:00','suborbital rocket','success',839,'II6.31F','','IGY II6.31F','','Fields',1895,'',61 from dual union all
select 58190,'1984-Y66','1984-02-18 00:47:00','suborbital rocket','success',1284,'MAP/WINE','M-C 16','','','Aeron',29,'',87 from dual union all
select 58191,'1984-Y67','1984-02-18 02:38:00','suborbital rocket','success',1284,'MAP/WINE','M-C 17','','','Aeron',29,'',87 from dual union all
select 58192,'1984-Y68','1984-02-18 03:14:00','suborbital rocket','success',1502,'MAP/WINE','M-F 53','','','Aeron',29,'',108 from dual union all
select 58193,'1984-Y69','1984-02-18 03:55:00','suborbital rocket','success',1284,'MAP/WINE','M-C 18','','','Aeron',29,'',82 from dual union all
select 58194,'1984-Y70','1984-02-18 04:40:00','suborbital rocket','success',1502,'MAP/WINE','M-F 54','','','Aeron',29,'',118 from dual union all
select 58195,'1984-Y71','1984-02-19 10:00:00','suborbital rocket','success',1502,'MAP/WINE','M-F 55','','','Aeron',29,'',117 from dual union all
select 58196,'1984-Y72','1984-02-19 11:22:00','suborbital rocket','success',1502,'MAP/WINE','M-F 56','','','Aeron',29,'',99 from dual union all
select 58197,'1984-Y73','1984-02-19 12:25:00','suborbital rocket','unknown',1502,'M-F 057','','','','Aeron',29,'',115 from dual union all
select 58198,'1984-Y74','1984-02-20 20:00:00','suborbital rocket','unknown',1502,'M-F 058','','','','Aeron',29,'',115 from dual union all
select 58199,'1984-Y75','1984-02-22 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 582,'1945-S58','1945-01-20 11:10:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5820,'1959-M21','1959-07-01 00:05:00','suborbital rocket','success',1194,'SL19','RAE Test 7','','','Test',1839,'',50 from dual union all
select 58200,'1984-Y76','1984-02-22 20:00:00','suborbital rocket','unknown',1502,'M-F 059','','','','Aeron',29,'',100 from dual union all
select 58201,'1984-Y77','1984-02-23 12:34:00','suborbital rocket','success',1281,'NASA 15.226UE','PSU E','','','Plasma',1821,'',80 from dual union all
select 58202,'1984-Y78','1984-02-23 17:00:00','suborbital rocket','unknown',1284,'M-D 035A','','','','Aeron',29,'',82 from dual union all
select 58203,'1984-Y79','1984-02-29 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 58204,'1984-Y80','1984-03-28 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',98 from dual union all
select 58205,'1984-Y81','1984-04-06 15:27:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 58206,'1984-Y82','1984-04-06 16:57:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 58207,'1984-Y83','1984-04-10 13:11:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 58208,'1984-Y84','1984-04-10 13:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',86 from dual union all
select 58209,'1984-Y85','1984-04-12 12:07:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',84 from dual union all
select 5821,'1959-M22','1959-07-14 21:00:00','suborbital rocket','success',366,'','RW-132','','','Aeron',1490,'',76 from dual union all
select 58210,'1984-Y86','1984-04-13 12:40:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 58211,'1984-Y87','1984-04-13 13:13:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 58212,'1984-Y88','1984-04-13 13:14:00','suborbital rocket','unknown',1284,'T 1-0883','','','','Aeron',1821,'',110 from dual union all
select 58213,'1984-Y89','1984-04-13 13:37:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 58214,'1984-Y90','1984-04-13 14:10:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',80 from dual union all
select 58215,'1984-Y91','1984-06-25 12:43:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',92 from dual union all
select 58216,'1984-Y92','1984-06-26 12:43:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 58217,'1984-Y93','1984-07-20 22:03:00','suborbital rocket','success',1281,'NASA 15.227UE','','','','Plasma',1821,'',80 from dual union all
select 58218,'1984-Y94','1984-07-23 10:52:00','suborbital rocket','success',1281,'NASA 15.228UE','','','','Plasma',1821,'',80 from dual union all
select 58219,'1984-Y95','1984-07-24 10:29:00','suborbital rocket','success',1281,'NASA 15.229UE','','','','Plasma',1821,'',80 from dual union all
select 5822,'1959-M23','1959-07-15 15:34:00','suborbital rocket','success',366,'','RW-133','','','Aeron',1490,'',78 from dual union all
select 58220,'1984-Y96','1984-07-24 11:27:59','suborbital rocket','success',1284,'','AN/DMQ-9- Sphere','','','Meteo',1821,'',93 from dual union all
select 58221,'1984-Y97','1984-07-24 11:56:00','suborbital rocket','success',1281,'NASA 15.230UE','','','','Plasma',1821,'',80 from dual union all
select 58222,'1984-Y98','1984-08-08 21:03:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',86 from dual union all
select 58223,'1984-Y99','1984-08-16 00:00:00','suborbital rocket','success',1284,'','','','','Aeron',1047,'',100 from dual union all
select 58224,'1985-001','1985-01-07 19:26:00','deep space','success',820,'M-3S2-1','Sakigake/KM-P','MS-T5','','Sat',837,'',null from dual union all
select 58225,'1985-002','1985-01-09 10:45:00','orbital','success',1238,'77048  -819','Kosmos-1616','Kobal''t','','Sat',562,'',null from dual union all
select 58226,'1985-003','1985-01-15 14:50:59','orbital','success',1448,'67042-303','Kosmos-1617/1622','Strela-3','','Sat',469,'',null from dual union all
select 58227,'1985-004','1985-01-16 06:22:30','orbital','success',805,'76032  -665','Molniya-3','Molniya-3 No. 36','','Sat',475,'',null from dual union all
select 58228,'1985-005','1985-01-16 08:19:00','orbital','success',1238,'78056  -315','Kosmos-1623','Oblik','','Sat',480,'',null from dual union all
select 58229,'1985-006','1985-01-17 17:46:00','orbital','success',680,'65034-138','Kosmos-1624','Strela-2M','','Sat',456,'',null from dual union all
select 5823,'1959-M24','1959-08-25 00:00:00','military missile','success',1162,'23','','','','OT?',1895,'',50 from dual union all
select 58230,'1985-007','1985-01-18 10:25:00','orbital','success',956,'326-02','Gorizont','Gorizont No. 21L','','Sat',553,'',null from dual union all
select 58231,'1985-008','1985-01-23 19:58:00','orbital','success',1446,'67083-626','Kosmos-1625','US-PM','','Sat',589,'',null from dual union all
select 58232,'1985-009','1985-01-24 16:45:00','orbital','success',1448,'51014-710','Kosmos-1626','Tselina-D','','Sat',470,'',null from dual union all
select 58233,'1985-010','1985-01-24 19:50:00','orbital','success',1243,'STS-20/51-C','USA 8      (ORION 1)','OV-103 IUS deploy','','Sat',921,'MLP1',null from dual union all
select 58234,'1985-011','1985-02-01 19:36:26','orbital','success',680,'47144-445','Kosmos-1627','Parus','','Sat',457,'',null from dual union all
select 58235,'1985-012','1985-02-06 11:00:00','orbital','success',1238,'77046  -313','Kosmos-1628','Oblik','','Sat',462,'',null from dual union all
select 58236,'1985-013','1985-02-06 21:45:00','orbital','success',1448,'51014-704','Meteor-2','Meteor-2 No. 13','','Sat',469,'',null from dual union all
select 58237,'1985-014','1985-02-08 06:10:00','orbital','success',1405,'34B-10 (3B-69)','USA 9        [SDS6]','SDS 6','','Sat',1778,'',null from dual union all
select 58238,'1985-015','1985-02-08 23:22:00','orbital','success',93,'V12 L14','Arab 1A/Brasilsat 1','Arabsat 1A','','Sat',344,'',null from dual;
insert into LAUNCH
select 58239,'1985-016','1985-02-21 07:56:48','orbital','success',956,'327-01','Kosmos-1629','Oko-S F2','','Sat',553,'',null from dual union all
select 5824,'1959-M25','1959-08-07 00:00:00','suborbital rocket','failure',89,'NASA 2.05GT','','','','Test',1821,'',50 from dual union all
select 58240,'1985-017','1985-02-27 11:10:00','orbital','success',1238,'78056  -626','Kosmos-1630','Kobal''t','','Sat',562,'',null from dual union all
select 58241,'1985-018','1985-02-27 12:56:00','orbital','success',680,'47134-139','Kosmos-1631','Vektor','','Sat',456,'',null from dual union all
select 58242,'1985-019','1985-03-01 10:40:00','orbital','success',1238,'76032  -797','Kosmos-1632','Oblik','','Sat',473,'',null from dual union all
select 58243,'1985-020','1985-03-05 15:39:00','orbital','success',1448,'67029-808','Kosmos-1633','Tselina-D','','Sat',470,'',null from dual union all
select 58244,'1985-021','1985-03-13 02:00:00','orbital','success',158,'41E','Geosat','Geosat','','Sat',1776,'',null from dual union all
select 58245,'1985-022','1985-03-14 01:09:00','orbital','success',680,'53744-146','Kosmos-1634','Parus','','Sat',457,'',null from dual union all
select 58246,'1985-023','1985-03-21 00:08:00','orbital','success',680,'53734-134','Kosmos-1635/1642','Strela-1M','','Sat',457,'',null from dual union all
select 58247,'1985-024','1985-03-22 05:00:00','orbital','success',956,'328-01','Ekran','Ekran No. 28L','','Sat',554,'',null from dual union all
select 58248,'1985-025','1985-03-22 23:55:00','orbital','success',169,'AC-63','Intelsat 510','INTELSAT VA F10','','Sat',269,'',null from dual union all
select 58249,'1985-026','1985-03-25 10:00:00','orbital','success',1238,'78056  -204','Kosmos-1643','Terilen','','Sat',480,'',null from dual union all
select 5825,'1959-M26','1959-08-26 19:35:00','suborbital rocket','success',366,'','RW-138','','','Aeron',1490,'',79 from dual union all
select 58250,'1985-027','1985-04-03 08:40:00','orbital','success',1238,'77046  -312','Kosmos-1644','Oblik','','Sat',562,'',null from dual union all
select 58251,'1985-028','1985-04-12 13:59:05','orbital','success',1243,'STS-23/51-D','PAM-D/Leasat Deploy','OV-103 PAM deploy','','Sat',921,'MLP1',null from dual union all
select 58252,'1985-029','1985-04-16 17:15:00','orbital','success',1238,'Sh15000-128','Kosmos-1645','Foton No. 1L','','Sat',473,'',null from dual union all
select 58253,'1985-030','1985-04-18 21:40:43','orbital','success',1446,'51099-801','Kosmos-1646','US-PM1','','Sat',589,'',null from dual union all
select 58254,'1985-031','1985-04-19 14:00:00','orbital','success',1238,'76032  -901','Kosmos-1647','Kobal''t','','Sat',473,'',null from dual union all
select 58255,'1985-032','1985-04-25 09:30:00','orbital','success',1238,'77046  -314','Kosmos-1648','Oblik','','Sat',475,'',null from dual union all
select 58256,'1985-033','1985-04-26 05:48:00','orbital','success',807,'B 15000-252','Prognoz-10-IK','SO-M No. 510','','Sat',562,'',null from dual union all
select 58257,'1985-034','1985-04-29 16:02:18','orbital','success',1243,'STS-21/51-B','Spacelab 3','OV-099 Spacelab','','Sat',921,'MLP2',null from dual union all
select 58258,'1985-035','1985-05-08 01:15:38','orbital','success',93,'V13 L15','Gstar 1/Telecom 1B','Gstar 1A','','Sat',344,'',null from dual union all
select 58259,'1985-036','1985-05-15 12:40:00','orbital','success',1238,'78056  -316','Kosmos-1649','Oblik','','Sat',475,'',null from dual union all
select 5826,'1959-M27','1959-08-27 02:00:00','suborbital rocket','success',366,'','RW-139','','','Aeron',1490,'',80 from dual union all
select 58260,'1985-037','1985-05-17 22:28:41','orbital','success',958,'330-02','Kosmos-1650/52','Uragan No. 20L','','Sat',553,'',null from dual union all
select 58261,'1985-038','1985-05-22 08:35:00','orbital','success',1238,'78056  -152','Kosmos-1653','Resurs-F1 17F41 No. 52','','Sat',473,'',null from dual union all
select 58262,'1985-039','1985-05-23 12:40:00','orbital','success',1238,'78056  -827','Kosmos-1654','Kobal''t','','Sat',562,'',null from dual union all
select 58263,'1985-040','1985-05-29 07:40:44','orbital','success',805,'78056  -676','Molniya-3','Molniya-3 No. 39','','Sat',475,'',null from dual union all
select 58264,'1985-041','1985-05-30 01:14:50','orbital','success',680,'47144-148','Kosmos-1655','Tsikada','','Sat',456,'',null from dual union all
select 58265,'1985-042','1985-05-30 14:59:00','orbital','success',958,'313-02','Kosmos-1656','Tselina-2','','Sat',554,'',null from dual union all
select 58266,'1985-043','1985-06-06 06:39:52','orbital','success',1242,'B 15000-008','Soyuz T-13','Soyuz 7K-ST No. 19L','','Sat',480,'',null from dual union all
select 58267,'1985-044','1985-06-07 07:45:00','orbital','success',1241,'78012  -805','Kosmos-1657','Resurs-F1 17F41 No. 54','','Sat',475,'',null from dual union all
select 58268,'1985-045','1985-06-11 14:26:57','orbital','success',803,'76032  -669','Kosmos-1658','Oko','','Sat',473,'',null from dual union all
select 58269,'1985-046','1985-06-13 12:20:00','orbital','success',1238,'78056  -317','Kosmos-1659','Oblik','','Sat',462,'',null from dual union all
select 5827,'1959-M28','1959-08-27 06:42:00','suborbital rocket','success',366,'','RW-140','','','Aeron',1490,'',80 from dual union all
select 58270,'1985-047','1985-06-14 10:36:00','orbital','success',1448,'45032-307','Kosmos-1660','Musson No. 17','','Sat',469,'',null from dual union all
select 58271,'1985-048','1985-06-17 11:33:00','orbital','success',1243,'STS-25/51-G','PAM-D Deploy','OV-103 PAM deploy/FF','','Sat',921,'MLP1',null from dual union all
select 58272,'1985-049','1985-06-18 00:40:26','orbital','success',803,'76032  -668','Kosmos-1661','Oko','','Sat',462,'',null from dual union all
select 58273,'1985-050','1985-06-19 11:30:00','orbital','success',680,'65044-147','Kosmos-1662','Romb','','Sat',457,'',null from dual union all
select 58274,'1985-051','1985-06-21 00:39:41','orbital','success',1238,'Sc15000-417','Progress-24','Progress 7K-TG No. 125','','Sat',480,'',null from dual union all
select 58275,'1985-052','1985-06-21 07:45:00','orbital','success',1238,'76021  -155','Kosmos-1663','Resurs-F1 17F41 No. 55','','Sat',473,'',null from dual union all
select 58276,'1985-053','1985-06-21 08:29:32','orbital','failure',1537,'51076-105','','EPN No. 03.694','','Sat',567,'',325 from dual union all
select 58277,'1985-054','1985-06-26 12:35:00','orbital','success',1238,'78056  -319','Kosmos-1664','Oblik','','Sat',475,'',null from dual union all
select 58278,'1985-055','1985-06-30 00:44:00','orbital','success',169,'AC-64','Intelsat 511','INTELSAT VA F11','','Sat',269,'',null from dual union all
select 58279,'1985-056','1985-07-02 11:23:13','deep space','success',91,'V14 L10','Giotto','Giotto','','Sat',344,'',null from dual union all
select 5828,'1959-M29','1959-09-26 07:00:00','suborbital rocket','success',366,'','RW-141','','','Aeron',1490,'',72 from dual union all
select 58280,'1985-057','1985-07-03 12:10:00','orbital','success',1238,'78056  -320','Kosmos-1665','Oblik','','Sat',462,'',null from dual union all
select 58281,'1985-058','1985-07-08 23:40:00','orbital','success',1448,'45042-301','Kosmos-1666','Tselina-D','','Sat',470,'',null from dual union all
select 58282,'1985-059','1985-07-10 03:15:00','orbital','success',1238,'B 15000-453','Kosmos-1667','Bion No. 7','','Sat',473,'',null from dual union all
select 58283,'1985-060','1985-07-15 06:30:00','orbital','success',1238,'76021  -325','Kosmos-1668','Oblik','','Sat',562,'',null from dual union all
select 58284,'1985-061','1985-07-17 01:05:00','orbital','success',805,'77024  -658','Molniya-3','Molniya-3 No. 37','','Sat',475,'',null from dual union all
select 58285,'1985-062','1985-07-19 13:05:08','orbital','success',1238,'B 15000-446','Kosmos-1669','Progress 7K-TG No. 126','','Sat',480,'',null from dual union all
select 58286,'1985-063','1985-07-29 21:00:00','orbital','success',1243,'STS-26/51-F','Spacelab 2','OV-099 Spacelab','','Sat',921,'MLP2',null from dual union all
select 58287,'1985-064','1985-08-01 05:36:00','orbital','success',1446,'51099-804','Kosmos-1670','US-A reactor','','Sat',589,'',null from dual union all
select 58288,'1985-065','1985-08-02 11:40:00','orbital','success',1238,'78056  -321','Kosmos-1671','Oblik','','Sat',462,'',null from dual union all
select 58289,'1985-066','1985-08-03 03:31:20','orbital','success',1135,'S209C','SOOS 1','NNS O-24','','Sat',1779,'',null from dual union all
select 5829,'1959-M30','1959-09-26 13:00:00','suborbital rocket','success',366,'','RW-143','','','Aeron',1490,'',77 from dual union all
select 58290,'1985-067','1985-08-07 09:50:00','orbital','success',1238,'77011  -157','Kosmos-1672','Resurs-F1 17F41 No. 57','','Sat',475,'',null from dual union all
select 58291,'1985-068','1985-08-08 10:19:00','orbital','success',1238,'78056  -205','Kosmos-1673','Siluet No. 5','','Sat',480,'',null from dual union all
select 58292,'1985-069','1985-08-08 11:49:00','orbital','success',1448,'67032-306','Kosmos-1674','Tselina-D','','Sat',469,'',null from dual union all
select 58293,'1985-070','1985-08-08 21:01:00','orbital','success',956,'317-02','Raduga','Gran'' No. 26L','','Sat',553,'',null from dual union all
select 58294,'1985-071','1985-08-12 15:09:03','orbital','success',803,'76032  -664','Kosmos-1675','Oko','','Sat',462,'',null from dual union all
select 58295,'1985-072','1985-08-16 15:10:00','orbital','success',1238,'77046  -824','Kosmos-1676','Kobal''t','','Sat',473,'',null from dual union all
select 58296,'1985-073','1985-08-18 23:33:00','deep space','success',820,'M-3S2-2','Suisei/KM-P','PLANET-A','','Sat',837,'',null from dual union all
select 58297,'1985-074','1985-08-22 19:28:00','orbital','success',805,'78056  -679','Molniya-1','Molniya-1!No. 61','','Sat',473,'',null from dual union all
select 58298,'1985-075','1985-08-23 22:33:00','orbital','success',1446,'45099-803','Kosmos-1677','US-A reactor','','Sat',589,'',null from dual union all
select 58299,'1985-076','1985-08-27 10:58:01','orbital','success',1243,'STS-27/51-I','Leasat Rescue/PAM-D','OV-103 PAM deploy/Rend','','Sat',921,'MLP1',null from dual union all
select 583,'1945-S59','1945-01-20 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5830,'1959-M31','1959-09-26 20:45:00','suborbital rocket','success',366,'','RW-144','','','Aeron',1490,'',71 from dual union all
select 58300,'1985-077','1985-08-29 10:15:00','orbital','success',1238,'76021  -153','Kosmos-1678','Resurs-F1 17F41 No. 53','','Sat',473,'',null from dual union all
select 58301,'1985-078','1985-08-29 11:33:00','orbital','success',1238,'78056  -828','Kosmos-1679','Kobal''t','','Sat',562,'',null from dual union all
select 58302,'1985-079','1985-09-04 07:05:00','orbital','success',680,'65011-241','Kosmos-1680','Strela-2M','','Sat',456,'',null from dual union all
select 58303,'1985-080','1985-09-06 10:45:00','orbital','success',1238,'77011  -052','Kosmos-1681','Fram','','Sat',473,'',null from dual union all
select 58304,'1985-081','1985-09-17 12:38:52','orbital','success',1242,'Sc15000-007','Soyuz T-14','Soyuz 7K-ST No. 20L','','Sat',480,'',null from dual union all
select 58305,'1985-082','1985-09-19 01:32:00','orbital','success',1446,'67099-802','Kosmos-1682','US-PM1','','Sat',589,'',null from dual union all
select 58306,'1985-083','1985-09-19 10:10:00','orbital','success',1238,'76021  -100','Kosmos-1683','Oblik','','Sat',473,'',null from dual union all
select 58307,'1985-084','1985-09-24 01:18:10','orbital','success',803,'77046  -674','Kosmos-1684','Oko','','Sat',475,'',null from dual union all
select 58308,'1985-085','1985-09-26 11:15:00','orbital','success',1238,'78056  -322','Kosmos-1685','Oblik','','Sat',462,'',null from dual union all
select 58309,'1985-086','1985-09-27 08:41:42','orbital','success',949,'331-01','Kosmos-1686','TKS-M No. 16501','','Sat',553,'',null from dual union all
select 5831,'1959-M32','1959-10-21 00:00:00','military missile','success',1162,'26','','','','OT?',1895,'',50 from dual union all
select 58310,'1985-087','1985-09-28 23:17:00','orbital','success',169,'AC-65','Intelsat 512','INTELSAT VA F12','','Sat',269,'',null from dual union all
select 58311,'1985-088','1985-09-30 19:23:20','orbital','success',803,'76032  -670','Kosmos-1687','Oko','','Sat',462,'',null from dual union all
select 58312,'1985-089','1985-10-02 09:00:00','orbital','success',680,'65044-153','Kosmos-1688','Romb','','Sat',622,'',null from dual union all
select 58313,'1985-090','1985-10-03 05:48:00','orbital','success',1510,'76053  -344','Kosmos-1689','Resurs-O1 No. 1L','','Sat',562,'',null from dual union all
select 58314,'1985-091','1985-10-03 07:33:00','orbital','success',805,'76021  -685','Molniya-3','Molniya-3 No. 38','','Sat',475,'',null from dual union all
select 58315,'1985-092','1985-10-03 15:15:30','orbital','success',1243,'STS-28/51-J','USA 11/12    [DSCS]','OV-104 IUS deploy','','Sat',921,'MLP2',null from dual union all
select 58316,'1985-093','1985-10-09 02:53:00','orbital','success',159,'55E','USA 10 (GPS 11)','Navstar GPS SVN 11','','Sat',1776,'',null from dual union all
select 58317,'1985-094','1985-10-09 21:35:00','orbital','success',1448,'51076-111','Kosmos-1690/1695','Strela-3','','Sat',469,'',null from dual union all
select 58318,'1985-095','1985-10-16 09:25:00','orbital','success',1238,'76021  -327','Kosmos-1696','Oblik','','Sat',562,'',null from dual union all
select 58319,'1985-096','1985-10-21 05:04:00','orbital','success',301,'Y4       (12)','FSW','FSW No. 8','','Sat',810,'',null from dual union all
select 5832,'1959-M33','1959-10-26 00:00:00','military missile','success',1162,'27','','','','OT?',1895,'',50 from dual union all
select 58320,'1985-097','1985-10-22 07:00:00','orbital','success',1537,'67076-106','Kosmos-1697','EPN No. 03.694','','Sat',567,'',null from dual union all
select 58321,'1985-098','1985-10-22 20:24:07','orbital','success',803,'77046  -675','Kosmos-1698','Oko','','Sat',475,'',null from dual union all
select 58322,'1985-099','1985-10-23 00:42:07','orbital','success',805,'78012  -663','Molniya-1','Molniya-1 11F658 No. 73','','Sat',480,'',null from dual union all
select 58323,'1985-100','1985-10-24 02:30:00','orbital','success',1448,'45061-504','Meteor-3','Meteor-3 No. 2','','Sat',469,'',null from dual union all
select 58324,'1985-101','1985-10-25 14:40:00','orbital','success',1238,'77046  -913','Kosmos-1699','Kobal''t','','Sat',462,'',null from dual union all
select 58325,'1985-102','1985-10-25 15:45:00','orbital','success',958,'332-02','Kosmos-1700','Al''tair No. 11L','','Sat',554,'',null from dual union all
select 58326,'1985-103','1985-10-28 17:24:58','orbital','success',805,'76021  -686','Molniya-1','Molniya-1!No. 56','','Sat',475,'',null from dual union all
select 58327,'1985-104','1985-10-30 17:00:00','orbital','success',1243,'STS-29/61-A','Spacelab D-1','OV-099 Spacelab','','Sat',921,'MLP1',null from dual union all
select 58328,'1985-105','1985-11-09 08:25:27','orbital','success',803,'78056  -681   / 668','Kosmos-1701','Oko','','Sat',473,'',null from dual;
insert into LAUNCH
select 58329,'1985-106','1985-11-13 12:25:00','orbital','success',1238,'77011  -331','Kosmos-1702','Oblik','','Sat',462,'',null from dual union all
select 5833,'1959-M34','1959-10-29 00:00:00','military missile','success',1162,'28','','','','OT?',1895,'',50 from dual union all
select 58330,'1985-107','1985-11-15 14:29:00','orbital','success',956,'326-01','Raduga','Gran'' No. 28L','','Sat',553,'',null from dual union all
select 58331,'1985-108','1985-11-22 22:20:00','orbital','success',1448,'45032-310','Kosmos-1703','Tselina-D','','Sat',470,'',null from dual union all
select 58332,'1985-109','1985-11-27 00:29:00','orbital','success',1243,'STS-30/61-B','PAM-D Deploy','OV-104 PAM deploy','','Sat',921,'MLP2',null from dual union all
select 58333,'1985-110','1985-11-28 13:12:41','orbital','success',680,'47122-346','Kosmos-1704','Parus','','Sat',460,'',null from dual union all
select 58334,'1985-111','1985-12-03 12:15:00','orbital','success',1238,'76021  -154','Kosmos-1705','Oblik','','Sat',462,'',null from dual union all
select 58335,'1985-112','1985-12-11 14:40:00','orbital','success',1238,'76032  -812','Kosmos-1706','Kobal''t','','Sat',462,'',null from dual union all
select 58336,'1985-113','1985-12-12 15:51:00','orbital','success',1448,'51042-302','Kosmos-1707','Tselina-D','','Sat',469,'',null from dual union all
select 58337,'1985-114','1985-12-13 02:35:00','orbital','success',1135,'S207C','USA 13/14 (ITV 1/2)','ITV 1','','Sat',1814,'',null from dual union all
select 58338,'1985-115','1985-12-13 07:45:00','orbital','success',1238,'76021  -156','Kosmos-1708','Resurs-F1 17F41 No. 56','','Sat',475,'',null from dual union all
select 58339,'1985-116','1985-12-19 08:46:55','orbital','success',680,'65022-348','Kosmos-1709','Parus','','Sat',456,'',null from dual union all
select 5834,'1959-M35','1959-10-04 10:00:00','test rocket','success',710,'LJ-6','','','','SC Test',1797,'',59 from dual union all
select 58340,'1985-117','1985-12-24 18:56:00','orbital','success',805,'78056  -680','Molniya-3','Molniya-3 No. 40','','Sat',475,'',null from dual union all
select 58341,'1985-118','1985-12-24 21:43:28','orbital','success',958,'334-02','Kosmos-1710/1712','Uragan No. 22L','','Sat',553,'',null from dual union all
select 58342,'1985-119','1985-12-26 01:50:00','orbital','success',1448,'51032-305','Meteor-2','Meteor-2 No. 14','','Sat',469,'',null from dual union all
select 58343,'1985-120','1985-12-27 17:06:00','orbital','success',1238,'B 15000-468','Kosmos-1713','Efir No. 2','','Sat',473,'',null from dual union all
select 58344,'1985-121','1985-12-28 09:16:30','orbital','success',1537,'51041-502','Kosmos-1714','Tselina-2 No. 3L','','Sat',567,'',null from dual union all
select 58345,'1985-A01','1985-01-24 21:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',47 from dual union all
select 58346,'1985-A02','1985-02-11 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',49 from dual union all
select 58347,'1985-A03','1985-03-03 14:45:00','suborbital rocket','failure',1202,'DLR N-GRC-137','Interzodiak 1','','','EUV Astronomy',330,'',0 from dual union all
select 58348,'1985-A04','1985-05-20 06:00:00','military missile','failure',932,'A3TK','CHEVALINE DASO','','','OT',421,'UK S22',1 from dual union all
select 58349,'1985-A05','1985-08-03 22:00:08','suborbital rocket','failure',227,'NASA 21.94CS','SL2 Coordinated','','','Solar',1873,'',10 from dual union all
select 5835,'1959-M36','1959-10-14 16:30:00','military missile','success',873,'ZW-2/102','2','','','Test',1884,'',40 from dual union all
select 58350,'1985-A06','1985-09-18 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',47 from dual union all
select 58351,'1985-A07','1985-09-28 00:00:00','military missile','failure',614,'','','','','Test',1909,'XIA',0 from dual union all
select 58352,'1985-A08','1985-10-17 19:46:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',49 from dual union all
select 58353,'1985-A09','1985-10-22 00:00:00','military missile','success',500,'1','OT','','','Test',1693,'',20 from dual union all
select 58354,'1985-A10','1985-11-06 02:03:00','suborbital rocket','success',761,'MT135P-R582','','','','Meteo',1332,'',46 from dual union all
select 58355,'1985-A11','1985-11-06 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',46 from dual union all
select 58356,'1985-A12','1985-11-15 14:00:00','suborbital rocket','unknown',716,'LOKI','PWN-8B','','','Test/Aeron',1821,'',49 from dual union all
select 58357,'1985-A13','1985-11-15 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',47 from dual union all
select 58358,'1985-A14','1985-11-25 18:57:00','suborbital rocket','unknown',716,'LOKI','PWN-8B','','','Test/Aeron',1821,'',35 from dual union all
select 58359,'1985-A15','1985-12-01 00:00:00','suborbital rocket','failure',1476,'VLS-R1','','','','Test',330,'',10 from dual union all
select 5836,'1959-M37','1959-11-04 01:23:00','suborbital rocket','success',366,'','RW-146','','','Aeron',1490,'',78 from dual union all
select 58360,'1985-A16','1985-12-04 21:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',48 from dual union all
select 58361,'1985-A17','1985-12-17 02:30:00','suborbital rocket','unknown',716,'LOKI','PWN-8B','','','Test/Aeron',1821,'',34 from dual union all
select 58362,'1985-A18','1985-12-18 00:00:00','suborbital rocket','unknown',716,'LOKI','','','','Test/Aeron',1821,'',48 from dual union all
select 58363,'1985-A19','1985-01-01 00:00:00','atmospheric rocket','success',542,'Sandia','Stores parachute 2','','','Test',1490,'',12 from dual union all
select 58364,'1985-A20','1985-03-01 00:00:00','military missile','failure',1187,'','15Zh66 LKI-1','','','Test',636,'',1 from dual union all
select 58365,'1985-F01','1985-04-13 08:00:00','orbital','failure',1537,'45086-101','','EPN No. 03.694 No. 5102','','Sat',567,'',200 from dual union all
select 58366,'1985-F02','1985-08-28 21:20:00','orbital','failure',1406,'34D-7  (04D-6)','-            [IC1]','Improved CRYSTAL 2101?','','Sat',1777,'',100 from dual union all
select 58367,'1985-F03','1985-09-12 23:26:00','orbital','failure',93,'V15 L16','ECS 3/Spacenet 3','ECS 3','','Sat',344,'',220 from dual union all
select 58368,'1985-F04','1985-10-23 17:24:00','orbital','failure',680,'65034-135','','Parus','','Sat',460,'',80 from dual union all
select 58369,'1985-M01','1985-03-15 17:37:59','suborbital rocket','success',860,'NASA 31.50UU','CWAS 9','','','Aeronomy',428,'',75 from dual union all
select 5837,'1959-M38','1959-11-18 05:03:00','suborbital rocket','success',664,'K-7-1','','','','Test',7,'',53 from dual union all
select 58370,'1985-M02','1985-01-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58371,'1985-M03','1985-01-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58372,'1985-M04','1985-01-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58373,'1985-M05','1985-01-23 13:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58374,'1985-M06','1985-01-23 14:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58375,'1985-M07','1985-02-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58376,'1985-M08','1985-02-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58377,'1985-M09','1985-02-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58378,'1985-M10','1985-02-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58379,'1985-M11','1985-03-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5838,'1959-M39','1959-01-01 00:00:00','suborbital rocket','success',927,'','Pogo Target Test','','','Test',1895,'',60 from dual union all
select 58380,'1985-M12','1985-03-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58381,'1985-M13','1985-03-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58382,'1985-M14','1985-03-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58383,'1985-M15','1985-04-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58384,'1985-M16','1985-04-10 13:30:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58385,'1985-M17','1985-04-10 14:15:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58386,'1985-M18','1985-04-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58387,'1985-M19','1985-05-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58388,'1985-M20','1985-05-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58389,'1985-M21','1985-05-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5839,'1959-M40','1959-01-01 00:00:00','suborbital rocket','success',927,'','Pogo Hi Type IIIC5a','','','Target',1895,'',60 from dual union all
select 58390,'1985-M22','1985-05-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58391,'1985-M23','1985-05-29 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58392,'1985-M24','1985-06-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58393,'1985-M25','1985-06-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58394,'1985-M26','1985-06-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58395,'1985-M27','1985-06-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58396,'1985-M28','1985-07-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58397,'1985-M29','1985-07-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58398,'1985-M30','1985-07-17 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58399,'1985-M31','1985-07-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 584,'1945-S60','1945-01-20 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5840,'1959-M41','1959-06-23 00:00:00','suborbital rocket','success',88,'7','Test','','','Test',1895,'',76 from dual union all
select 58400,'1985-M32','1985-07-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58401,'1985-M33','1985-08-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58402,'1985-M34','1985-01-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58403,'1985-M35','1985-04-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58404,'1985-M36','1985-11-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58405,'1985-M37','1985-09-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58406,'1985-M38','1985-09-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58407,'1985-M39','1985-09-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58408,'1985-M40','1985-09-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58409,'1985-M41','1985-10-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5841,'1959-S01','1959-01-01 00:00:00','suborbital rocket','success',14,'','Model A (F2)','','','Test',383,'F-104',117 from dual union all
select 58410,'1985-M42','1985-10-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58411,'1985-M43','1985-10-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58412,'1985-M44','1985-11-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58413,'1985-M45','1985-11-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58414,'1985-M46','1985-11-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58415,'1985-M47','1985-12-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58416,'1985-M48','1985-12-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58417,'1985-M49','1985-12-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 58418,'1985-M50','1985-12-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual;
insert into LAUNCH
select 58419,'1985-M51','1985-01-15 15:00:00','suborbital rocket','success',1284,'','Datasonde','','','Aeron',1821,'',75 from dual union all
select 5842,'1959-S02','1959-01-01 00:00:00','suborbital rocket','success',26,'','','','','Aeron',1837,'',81 from dual union all
select 58420,'1985-M52','1985-01-15 16:49:00','suborbital rocket','success',1284,'','ROCOZ-A 337','','','Aeron Ozone',1821,'',75 from dual union all
select 58421,'1985-M53','1985-02-06 16:12:00','suborbital rocket','success',1284,'','ROCOZ-A 338','','','Aeron Ozone',1821,'',75 from dual union all
select 58422,'1985-M54','1985-04-25 19:45:00','suborbital rocket','success',1284,'','ROCOZ-A 346','','','Aeron Ozone',1821,'',75 from dual union all
select 58423,'1985-M55','1985-05-20 17:15:00','suborbital rocket','success',1284,'','ROCOZ-A 347','','','Aeron Ozone',1821,'',75 from dual union all
select 58424,'1985-M56','1985-06-19 19:25:00','suborbital rocket','success',1284,'','ROCOZ-A 348','','','Aeron Ozone',1821,'',75 from dual union all
select 58425,'1985-M57','1985-07-30 13:00:00','suborbital rocket','success',1284,'','ROCOZ-A 349','','','Aeron Ozone',1821,'',75 from dual union all
select 58426,'1985-M58','1985-08-14 19:15:00','suborbital rocket','success',1284,'','ROCOZ-A 350','','','Aeron Ozone',1821,'',75 from dual union all
select 58427,'1985-M59','1985-09-18 19:30:00','suborbital rocket','success',1284,'','ROCOZ-A 351','','','Aeron Ozone',1821,'',75 from dual union all
select 58428,'1985-S01','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58429,'1985-S02','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 5843,'1959-S03','1959-01-10 11:00:00','suborbital rocket','success',5,'','','','','Ionos/Aeron',1010,'',100 from dual union all
select 58430,'1985-S03','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58431,'1985-S04','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58432,'1985-S05','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58433,'1985-S06','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58434,'1985-S07','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58435,'1985-S08','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58436,'1985-S09','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58437,'1985-S10','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58438,'1985-S100','1985-07-14 15:05:00','suborbital rocket','success',1337,'Sandia 497-013','ACE I','','','Target',848,'',497 from dual union all
select 58439,'1985-S101','1985-07-15 00:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT110GM','GLORY TRIP 110GM','OT',1705,'',1300 from dual union all
select 5844,'1959-S04','1959-01-16 04:00:00','military missile','failure',147,'13B','Mk 2-1 RV','R&D','','Test',229,'',100 from dual union all
select 58440,'1985-S102','1985-08-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58441,'1985-S103','1985-08-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58442,'1985-S104','1985-08-10 17:00:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 656',500 from dual union all
select 58443,'1985-S105','1985-08-23 16:39:00','military missile','success',907,'FTM-9','Mk 21 RV x 6','','','Test',1703,'',1000 from dual union all
select 58444,'1985-S106','1985-09-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58445,'1985-S107','1985-09-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58446,'1985-S108','1985-09-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58447,'1985-S109','1985-09-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58448,'1985-S11','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58449,'1985-S110','1985-09-02 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 626',500 from dual union all
select 5845,'1959-S05','1959-01-19 20:13:00','military missile','success',1053,'CC-1010','','','','OT 209AG',1854,'',90 from dual union all
select 58450,'1985-S111','1985-09-02 05:03:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 626',500 from dual union all
select 58451,'1985-S112','1985-09-06 15:00:00','suborbital rocket','success',860,'NASA 31.55UU','CWAS 11','','','Aeronomy',1895,'',140 from dual union all
select 58452,'1985-S113','1985-09-07 09:45:00','suborbital rocket','success',891,'NASA 30.26UE','','','','Plasma',1821,'',85 from dual union all
select 58453,'1985-S114','1985-09-10 00:01:00','suborbital rocket','success',891,'NASA 30.28UE','','','','Plasma',1821,'',85 from dual union all
select 58454,'1985-S115','1985-09-10 00:02:10','suborbital rocket','success',1319,'NASA 33.50GE','','','','Plasma',1821,'',200 from dual union all
select 58455,'1985-S116','1985-09-10 00:03:07','suborbital rocket','success',860,'NASA 31.51GE','','','','Plasma',1821,'',140 from dual union all
select 58456,'1985-S117','1985-09-12 14:50:00','suborbital rocket','success',860,'NASA 31.56UU','CWAS 12','','','Aeronomy',1895,'',140 from dual union all
select 58457,'1985-S118','1985-09-13 00:00:00','test rocket','success',20,'','','','','Interceptor',383,'F-15',500 from dual union all
select 58458,'1985-S119','1985-09-14 14:40:00','suborbital rocket','success',668,'K-9M-78','CBR','','','Submm Astron',839,'',297 from dual union all
select 58459,'1985-S12','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 5846,'1959-S06','1959-01-22 00:10:00','military missile','success',638,'CM-21','Tactical Test','','','Test',279,'',500 from dual union all
select 58460,'1985-S120','1985-09-17 19:15:00','suborbital rocket','success',891,'NASA 30.27UU','','','','Aeron',1895,'',85 from dual union all
select 58461,'1985-S121','1985-09-26 08:25:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT111GM','GLORY TRIP 111GM','OT',1699,'',1300 from dual union all
select 58462,'1985-S122','1985-09-27 15:20:00','suborbital rocket','success',1337,'Sandia 497-014','ACE II','','','Target',848,'',497 from dual union all
select 58463,'1985-S123','1985-10-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58464,'1985-S124','1985-10-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58465,'1985-S125','1985-10-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58466,'1985-S126','1985-10-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58467,'1985-S127','1985-10-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58468,'1985-S128','1985-10-10 15:25:00','suborbital rocket','success',1337,'Sandia 497-015','ACE III','','','Target',848,'',479 from dual union all
select 58469,'1985-S129','1985-10-16 10:01:00','suborbital rocket','success',891,'NASA 30.31UE','','','','Plasma',1821,'',85 from dual union all
select 5847,'1959-S07','1959-01-27 17:00:00','suborbital rocket','success',363,'01','','','','Test/Aeron',313,'',127 from dual union all
select 58470,'1985-S13','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58471,'1985-S130','1985-10-25 17:20:00','suborbital rocket','success',211,'NASA 27.90CS','XSST/TRC 4','','','Solar UV/X',1895,'',290 from dual union all
select 58472,'1985-S131','1985-10-16 00:00:00','suborbital rocket','success',1094,'','ASLV Strapon Test','','','Test',1361,'',100 from dual union all
select 58473,'1985-S132','1985-11-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58474,'1985-S133','1985-11-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58475,'1985-S134','1985-11-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58476,'1985-S135','1985-11-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',142,'ZUB',150 from dual union all
select 58477,'1985-S136','1985-11-10 18:56:05','suborbital rocket','success',224,'NASA 36.05IE','Ferdinand 62/MAIMIK','','','Active plas/tether',29,'',381 from dual union all
select 58478,'1985-S137','1985-11-13 17:03:00','military missile','success',907,'FTM-10','Mk 21 RV x 4','','[700 km NE of Guam]','Test',1703,'',1000 from dual union all
select 58479,'1985-S138','1985-11-19 13:00:00','suborbital rocket','success',1220,'PT-02','GL A40.401 (HARP)','','','Test',330,'',667 from dual union all
select 5848,'1959-S08','1959-01-27 20:35:00','suborbital rocket','success',366,'','RW-112','','','Aeron',1490,'',83 from dual union all
select 58480,'1985-S139','1985-11-20 10:19:00','suborbital rocket','success',1318,'NASA 38.09UE','CRRES Test','','','Plasma',1821,'',500 from dual union all
select 58481,'1985-S14','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58482,'1985-S140','1985-12-07 15:30:00','military missile','success',1442,'','DASO','','','OT',421,'SSBN 657',1000 from dual union all
select 58483,'1985-S141','1985-12-11 23:30:00','suborbital rocket','success',1219,'XV-20','','','','Aeron',330,'',516 from dual union all
select 58484,'1985-S142','1985-12-14 07:16:42','suborbital rocket','success',225,'NASA 36.09UE','CHARGE 2 (TPE-4)','','','Plasma/Tether',1873,'',261 from dual union all
select 58485,'1985-S143','1985-12-16 13:03:00','military missile','success',919,'SSO-1 Sys. Shakedn.','P-474 85 Cape 01','','','OT',231,'',320 from dual union all
select 58486,'1985-S144','1985-12-16 19:10:00','suborbital rocket','success',224,'NASA 36.11UL','PACE 1','','','Aeron',1873,'',251 from dual union all
select 58487,'1985-S145','1985-12-17 16:02:00','military missile','success',919,'SSO-2','P-475 85 Cape 02','','','OT',231,'',320 from dual union all
select 58488,'1985-S146','1985-12-17 21:37:00','military missile','success',919,'SSO-3','P-476 85 Cape 03','','','OT',231,'',320 from dual union all
select 58489,'1985-S147','1985-12-06 00:00:00','military missile','success',1434,'64581-451-55','15Zh58 K-9','','','Test',477,'',1000 from dual union all
select 5849,'1959-S09','1959-01-27 23:34:00','military missile','success',150,'4C','Mk 2-2A RV','R&D','','Test',227,'',990 from dual union all
select 58490,'1985-S148','1985-12-10 15:50:00','suborbital rocket','success',1337,'Sandia 497-016','ACE IV','','','Target',848,'',486 from dual union all
select 58491,'1985-S149','1985-03-27 11:46:00','suborbital rocket','success',597,'INTA RP-8501','Test (Ballast)','Rocio-2','','Test',396,'',33 from dual union all
select 58492,'1985-S15','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58493,'1985-S150','1985-06-18 11:49:00','suborbital rocket','failure',597,'INTA RP-8502','Test (Ballast)','Rocio-3','','Test',396,'',1 from dual union all
select 58494,'1985-S151','1985-06-24 11:01:00','suborbital rocket','success',597,'INTA RP-8503','Test (Ballast)','Rocio-4','','Test',396,'',74 from dual union all
select 58495,'1985-S152','1985-01-01 00:00:00','military missile','success',984,'Aerofon','','ZLKI 6','','Test',636,'',86 from dual union all
select 58496,'1985-S153','1985-03-12 00:00:00','military missile','success',1166,'','Kirkuk','','','Weapon',857,'',100 from dual union all
select 58497,'1985-S154','1985-03-14 00:00:00','military missile','success',1166,'','Baghdad','','','Weapon',857,'',100 from dual union all
select 58498,'1985-S155','1985-03-01 00:00:00','military missile','success',1166,'','','','','Weapon',857,'',100 from dual union all
select 58499,'1985-S156','1985-03-01 00:00:00','military missile','success',1166,'','','','','Weapon',857,'',100 from dual union all
select 585,'1945-S61','1945-01-20 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5850,'1959-S10','1959-01-28 00:00:00','suborbital rocket','success',839,'','Beacon Test 4','','','Sat Test',1821,'',122 from dual union all
select 58500,'1985-S157','1985-03-01 00:00:00','military missile','success',1166,'','','','','Weapon',857,'',100 from dual union all
select 58501,'1985-S158','1985-03-01 00:00:00','military missile','success',1166,'','','','','Weapon',857,'',100 from dual union all
select 58502,'1985-S159','1985-03-01 00:00:00','military missile','success',1166,'','','','','Weapon',857,'',100 from dual union all
select 58503,'1985-S16','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58504,'1985-S160','1985-03-01 00:00:00','military missile','success',1166,'','','','','Weapon',857,'',100 from dual union all
select 58505,'1985-S161','1985-03-01 00:00:00','military missile','success',983,'','54 launches vs Iran','','','Weapon',761,'',100 from dual union all
select 58506,'1985-S162','1985-12-01 00:00:00','military missile','success',358,'','R&D range test','','','Test',810,'',500 from dual union all
select 58507,'1985-S163','1985-04-01 00:00:00','military missile','success',983,'Scud B','22 launches vs Iran','','','Weapon',761,'',100 from dual union all
select 58508,'1985-S164','1985-05-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual;
insert into LAUNCH
select 58509,'1985-S165','1985-05-01 00:00:00','military missile','success',983,'Scud B','','','','Weapon',761,'',100 from dual union all
select 5851,'1959-S100','1959-06-29 11:33:00','military missile','success',239,'BK.05','H05','','','Test',1843,'',442 from dual union all
select 58510,'1985-S166','1985-05-01 00:00:00','military missile','success',983,'Scud B','15 launches vs Iran','','','Weapon',761,'',100 from dual union all
select 58511,'1985-S167','1985-08-01 00:00:00','military missile','success',983,'Scud B','Test launch','','','Weapon',761,'',100 from dual union all
select 58512,'1985-S168','1985-01-01 00:00:00','military missile','success',984,'Aerofon','UGCh','ZLKI 7','','Test',636,'',86 from dual union all
select 58513,'1985-S17','1985-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 58514,'1985-S18','1985-04-01 00:00:00','suborbital rocket','success',1268,'SWERVE III','','','','Reentry Test',855,'',200 from dual union all
select 58515,'1985-S19','1985-02-21 00:00:00','military missile','success',1453,'51184-62-651','15A35 UBP-25','','','OT',535,'',1000 from dual union all
select 58516,'1985-S20','1985-03-14 00:00:00','military missile','success',1453,'49282-71-363','15A35 UBP-26','','','OT',508,'',1000 from dual union all
select 58517,'1985-S21','1985-03-15 00:00:00','military missile','success',1453,'49263-91-843','15A35 UBP-27','','','OT',535,'',1000 from dual union all
select 58518,'1985-S22','1985-05-22 00:00:00','military missile','success',1453,'','15A35 UBP-28','','','OT',508,'',1000 from dual union all
select 58519,'1985-S23','1985-11-29 00:00:00','military missile','success',1453,'63046-41-361','15A35 UBP-29','','','OT',596,'',1000 from dual union all
select 5852,'1959-S101','1959-06-30 00:00:00','suborbital rocket','success',839,'ABME6.208','Dumbbell probe','','','Ionos',1821,'',83 from dual union all
select 58520,'1985-S24','1985-12-03 00:00:00','military missile','success',1453,'','15A35 UBP-30','','','OT',596,'',1000 from dual union all
select 58521,'1985-S248','1958-11-04 00:00:00','military missile','success',1013,'M1-4 8K51D','8K51D Fakel Target 2','GCh Fakel NIIP-A Target','','Target',321,'',500 from dual union all
select 58522,'1985-S249','1985-06-11 00:00:00','military missile','failure',1037,'','8K98 UBP-69','','','Test',477,'',0 from dual union all
select 58523,'1985-S25','1985-01-17 15:00:00','suborbital rocket','success',32,'NASA 04.342UG','JHU Airglow 14 (HUT)','','','Airglow',1871,'',169 from dual union all
select 58524,'1985-S250','1985-07-04 00:00:00','military missile','success',1037,'29547-211-47','8K98 UBP-70','','','Test',477,'',1000 from dual union all
select 58525,'1985-S251','1985-08-01 00:00:00','military missile','success',1037,'64647-221-36','8K98 UBP-71','','','Test',477,'',1000 from dual union all
select 58526,'1985-S252','1985-09-21 00:00:00','military missile','success',1037,'64647-221-47','8K98 UBP-72','','','Test',477,'',1000 from dual union all
select 58527,'1985-S253','1985-09-30 00:00:00','military missile','success',1044,'64646-812-52','8K98P UBP-24','','','Test',477,'',1000 from dual union all
select 58528,'1985-S254','1985-01-29 00:00:00','military missile','success',1434,'15L','15Zh58 LKI-15','','','Test',477,'',1000 from dual union all
select 58529,'1985-S255','1985-04-22 00:00:00','military missile','success',1434,'64536-214-45','15Zh58 K-3','','','Test',477,'',1000 from dual union all
select 5853,'1959-S102','1959-07-01 00:00:00','suborbital rocket','success',839,'ABME6.209','Tetrahedron probe','','','Ionos',1821,'',99 from dual union all
select 58530,'1985-S256','1985-06-14 00:00:00','military missile','success',1434,'64551-421-52','15Zh58 K-4','','','Test',477,'',1000 from dual union all
select 58531,'1985-S257','1985-08-06 00:00:00','military missile','success',1434,'64551-431-55','15Zh58 K-5','','','Test',477,'',1000 from dual union all
select 58532,'1985-S258','1985-08-28 00:00:00','military missile','success',1434,'64551-431-58','15Zh58 K-6','','','Test',477,'',1000 from dual union all
select 58533,'1985-S259','1985-10-04 00:00:00','military missile','success',1434,'64581-431-66','15Zh58 K-7','','','Test',477,'',1000 from dual union all
select 58534,'1985-S26','1985-01-23 09:23:00','suborbital rocket','success',1337,'NASA 29.23UE','COPE','','','Plasma',1438,'',500 from dual union all
select 58535,'1985-S260','1985-10-25 00:00:00','military missile','success',1434,'64581-441-52','15Zh58 K-8','','','Test',477,'',1000 from dual union all
select 58536,'1985-S261','1985-02-21 00:00:00','military missile','success',1434,'64536-224-33','15Zh58 K-2','','','Test',477,'',1000 from dual union all
select 58537,'1985-S262','1985-04-15 00:00:00','military missile','success',1040,'9L','15Zh52 LKI-10','','','Test',477,'',1000 from dual union all
select 58538,'1985-S263','1985-04-25 00:00:00','military missile','failure',1042,'2L','15Zh61 LKI-2','','','Test',477,'',0 from dual union all
select 58539,'1985-S264','1985-09-26 00:00:00','military missile','success',1042,'48741-203-4L','15Zh61 LKI-3','','','Test',477,'',1000 from dual union all
select 5854,'1959-S103','1959-07-02 06:40:00','suborbital rocket','success',994,'No. 9','','','','Ionos/Aeron/Bio',636,'',220 from dual union all
select 58540,'1985-S265','1985-10-10 00:00:00','military missile','success',1042,'48741-202-3L','15Zh61 LKI-4','','','Test',477,'',1000 from dual union all
select 58541,'1985-S266','1985-11-04 00:00:00','military missile','success',1042,'48751-204-5L','15Zh61 LKI-5','','','Test',477,'',1000 from dual union all
select 58542,'1985-S267','1985-11-28 00:00:00','military missile','success',1042,'48751-205-6L','15Zh61 LKI-6','','','Test',477,'',1000 from dual union all
select 58543,'1985-S268','1985-12-20 00:00:00','military missile','success',1042,'48751-206-7L','15Zh61 LKI-7','','','Test',477,'',1000 from dual union all
select 58544,'1985-S269','1985-01-01 00:00:00','military missile','success',1450,'','15A20 54 RD UBP-10','','','OT',1468,'',1000 from dual union all
select 58545,'1985-S27','1985-01-23 09:26:32','suborbital rocket','success',231,'NASA 35.009UE','COPE','','','Plasma',1438,'',769 from dual union all
select 58546,'1985-S270','1985-09-20 00:00:00','military missile','success',1006,'','15A18 62 RD UBP-1','','','OT',1646,'',1000 from dual union all
select 58547,'1985-S271','1985-11-13 12:17:00','suborbital rocket','success',597,'INTA RP-8504','Test (Ballast)','Rocio-5','','Test',396,'',110 from dual union all
select 58548,'1985-S272','1985-11-15 11:23:00','suborbital rocket','success',597,'INTA RP-8505','Chaff','Rocio-6','','Aeron',396,'',50 from dual union all
select 58549,'1985-S273','1985-05-30 00:00:00','military missile','success',354,'','','','','Test',1494,'',500 from dual union all
select 5855,'1959-S104','1959-07-07 11:15:00','suborbital rocket','success',624,'JV-1','','','','Test/Particles',1821,'',1045 from dual union all
select 58550,'1985-S278','1985-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-27','Duga-2','','Test',368,'',500 from dual union all
select 58551,'1985-S279','1985-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-28','Duga-2','','Test',368,'',500 from dual union all
select 58552,'1985-S28','1985-01-28 22:10:00','suborbital rocket','success',891,'NASA 30.23UU','','','','Aeron',1047,'',85 from dual union all
select 58553,'1985-S280','1985-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-29','Duga-2','','Test',368,'',500 from dual union all
select 58554,'1985-S281','1985-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-30','Duga-2','','Test',368,'',500 from dual union all
select 58555,'1985-S282','1985-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-31','Duga-2','','Test',368,'',500 from dual union all
select 58556,'1985-S283','1985-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-32','Duga-2','','Test',368,'',500 from dual union all
select 58557,'1985-S284','1985-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-33','Duga-2','','Test',368,'',500 from dual union all
select 58558,'1985-S285','1985-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-34','Duga-2','','Test',368,'',500 from dual union all
select 58559,'1985-S286','1985-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-35','Duga-2','','Test',368,'',500 from dual union all
select 5856,'1959-S105','1959-07-08 00:13:00','suborbital rocket','success',1194,'SL14','LUX-P1/UCL Ion-1','','','Ionos/Solar X',1839,'',93 from dual union all
select 58560,'1985-S287','1985-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-36','Duga-2','','Test',368,'',500 from dual union all
select 58561,'1985-S288','1985-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-37','Duga-2','','Test',368,'',500 from dual union all
select 58562,'1985-S289','1985-01-01 00:00:00','military missile','success',922,'','15Zh45 4 RD UBP-37','Duga-2','','Test',368,'',500 from dual union all
select 58563,'1985-S29','1985-01-30 19:30:00','suborbital rocket','success',1202,'DLR A-GRC-134','CAESAR 2','','','Auroral',29,'',703 from dual union all
select 58564,'1985-S290','1985-01-01 00:00:00','military missile','success',924,'','15Zh57 LKI K-1','','','Test',822,'',500 from dual union all
select 58565,'1985-S291','1985-01-01 00:00:00','military missile','success',924,'','15Zh57 LKI K-2','','','Test',822,'',500 from dual union all
select 58566,'1985-S292','1985-03-07 00:00:00','military missile','success',922,'','15Zh45 50 RD UBP','','','Test',636,'',500 from dual union all
select 58567,'1985-S293','1985-04-25 00:00:00','military missile','success',922,'','15Zh45 19 RD UBP','','','Test',636,'',500 from dual union all
select 58568,'1985-S294','1985-09-21 00:00:00','military missile','success',923,'','15Zh53 32 RD UBP','','','Test',636,'',500 from dual union all
select 58569,'1985-S295','1985-01-01 00:00:00','military missile','success',922,'','15Zh56 LKI-2','Gorn','','Test',636,'',500 from dual union all
select 5857,'1959-S106','1959-07-10 01:01:00','military missile','success',638,'AM-15','R&D','','','Test/Ionos',249,'',500 from dual union all
select 58570,'1985-S296','1985-12-08 07:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',90 from dual union all
select 58571,'1985-S297','1985-12-08 08:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',90 from dual union all
select 58572,'1985-S298','1985-12-14 11:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',90 from dual union all
select 58573,'1985-S299','1985-12-14 12:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',90 from dual union all
select 58574,'1985-S30','1985-02-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',116,'ZUB',150 from dual union all
select 58575,'1985-S31','1985-02-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',116,'ZUB',150 from dual union all
select 58576,'1985-S32','1985-02-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',116,'ZUB',150 from dual union all
select 58577,'1985-S33','1985-02-01 22:15:00','military missile','success',907,'FTM-7','Mk 21 RV x 6','','','Test',1726,'',1000 from dual union all
select 58578,'1985-S34','1985-02-06 13:18:00','military missile','failure',799,'','Mk 12/12A RV x 3','FOT GT106GM','GLORY TRIP 106GM','OT',1699,'',100 from dual union all
select 58579,'1985-S35','1985-02-07 09:00:00','suborbital rocket','success',1073,'S-310-15','','','','Aeron',833,'',213 from dual union all
select 5858,'1959-S107','1959-07-10 04:47:00','suborbital rocket','success',994,'No. 10','','','','Solar UV/Bio',636,'',205 from dual union all
select 58580,'1985-S36','1985-02-07 22:01:00','suborbital rocket','success',891,'NASA 30.24UU','','','','Aeron',1047,'',85 from dual union all
select 58581,'1985-S37','1985-02-08 18:14:00','suborbital rocket','success',1319,'NASA 33.48UU','','','','Aeron',1873,'',200 from dual union all
select 58582,'1985-S38','1985-02-10 02:11:00','suborbital rocket','success',1337,'NASA 29.15UE','COPE/ARCS 3','','','Plasma/Active',1438,'',406 from dual union all
select 58583,'1985-S39','1985-02-10 02:13:00','suborbital rocket','success',231,'NASA 35.012UE','COPE/TOPAZ I','','','Plasma',1438,'',816 from dual union all
select 58584,'1985-S40','1985-02-11 10:30:00','suborbital rocket','success',1074,'S-520-7','','','','Aeron/Ionos',833,'',362 from dual union all
select 58585,'1985-S41','1985-02-15 01:45:15','suborbital rocket','success',225,'AAF-IXB-3','WATERHOLE 4','','','Auroral/Aeron/Ionos',428,'',302 from dual union all
select 58586,'1985-S42','1985-02-15 05:25:11','suborbital rocket','success',232,'AAF-XB-01','MARIE','','','Ionos/Plasma/Fields',428,'',979 from dual union all
select 58587,'1985-S43','1985-02-15 15:15:00','suborbital rocket','success',891,'DLR N-O-135','Fin test','','','Test',330,'',85 from dual union all
select 58588,'1985-S44','1985-02-20 13:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT107GB','GLORY TRIP 107GB','OT',1712,'',1300 from dual union all
select 58589,'1985-S45','1985-02-22 15:20:00','suborbital rocket','success',891,'DLR N-O-136','Fin test','','','Test',330,'',85 from dual union all
select 5859,'1959-S108','1959-07-14 03:40:00','suborbital rocket','success',994,'','','','','Ionos/Aeron',636,'',203 from dual union all
select 58590,'1985-S46','1985-02-27 00:00:00','military missile','success',1042,'48746-218-1L','15Zh61 LKI-1','Rail','','Test',477,'',1000 from dual union all
select 58591,'1985-S47','1985-03-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',116,'ZUB',150 from dual union all
select 58592,'1985-S48','1985-03-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',116,'ZUB',150 from dual union all
select 58593,'1985-S49','1985-03-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',116,'ZUB',150 from dual union all
select 58594,'1985-S50','1985-03-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',116,'ZUB',150 from dual union all
select 58595,'1985-S51','1985-03-05 20:38:00','suborbital rocket','success',1319,'NASA 33.44UE','COPE Cornell E1','','','Plasma',1438,'',200 from dual union all
select 58596,'1985-S52','1985-03-06 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 623',500 from dual union all
select 58597,'1985-S53','1985-03-06 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 623',500 from dual union all
select 58598,'1985-S54','1985-03-06 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 623',500 from dual union all
select 58599,'1985-S55','1985-03-06 03:42:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 623',500 from dual;
insert into LAUNCH
select 586,'1945-S62','1945-01-20 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5860,'1959-S109','1959-07-14 17:45:00','suborbital rocket','success',838,'NN8.65','','','','Solar X',1015,'',200 from dual union all
select 58600,'1985-S56','1985-03-14 12:54:34','suborbital rocket','success',1320,'NASA 34.11UE','','','','Plasma',1047,'',500 from dual union all
select 58601,'1985-S57','1985-03-15 02:05:52','suborbital rocket','success',224,'GL A21.426','COPE/PIIE','','','Plasma',1438,'',429 from dual union all
select 58602,'1985-S58','1985-03-18 00:05:00','suborbital rocket','success',741,'','Plazma','','','Plasma/Active',115,'ZUB',250 from dual union all
select 58603,'1985-S59','1985-03-18 08:50:00','suborbital rocket','success',228,'NASA 21.70UH','MIT WFSXC 3','','','XR Astron',1873,'',213 from dual union all
select 58604,'1985-S60','1985-03-20 22:34:00','suborbital rocket','success',1320,'NASA 34.06UE','COPE Cornell E2','','','Plasma',1438,'',500 from dual union all
select 58605,'1985-S61','1985-03-20 22:44:00','suborbital rocket','success',867,'NASA 18.219UE','COPE','','','Plasma',1438,'',270 from dual union all
select 58606,'1985-S62','1985-03-22 17:33:00','suborbital rocket','success',860,'NASA 31.49UU','CWAS 10','','','Aeronomy',428,'',91 from dual union all
select 58607,'1985-S63','1985-03-27 06:07:11','suborbital rocket','success',211,'NASA 27.98UL','Auroral','','','Aeron',428,'',230 from dual union all
select 58608,'1985-S64','1985-03-29 17:03:20','suborbital rocket','success',860,'NASA 31.48GE','','','','Plasma',1047,'',140 from dual union all
select 58609,'1985-S65','1985-04-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',116,'ZUB',150 from dual union all
select 5861,'1959-S11','1959-01-28 07:28:00','suborbital rocket','success',366,'','RW-114','','','Aeron',1490,'',82 from dual union all
select 58610,'1985-S66','1985-04-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',116,'ZUB',150 from dual union all
select 58611,'1985-S67','1985-04-01 16:57:00','suborbital rocket','success',860,'NASA 31.47GE','','','','Plasma',1047,'',140 from dual union all
select 58612,'1985-S68','1985-04-02 07:21:01','suborbital rocket','success',891,'NASA 30.25UE','','','','Plasma',1047,'',85 from dual union all
select 58613,'1985-S69','1985-04-16 22:30:00','test rocket','success',647,'','BOR-5 No. 502','','','Reentry Test',621,'',210 from dual union all
select 58614,'1985-S70','1985-04-20 07:50:00','suborbital rocket','success',211,'NASA 27.82UG','IMAPS 2','','','FUV Astron',1895,'',265 from dual union all
select 58615,'1985-S71','1985-04-27 04:15:00','suborbital rocket','success',1212,'DLR K-GR-138','DLR TEXUS 11','','','Micrograv',416,'',266 from dual union all
select 58616,'1985-S72','1985-05-20 00:00:00','military missile','success',354,'','','','','Test',1494,'',500 from dual union all
select 58617,'1985-S73','1985-05-06 04:35:00','suborbital rocket','success',1212,'DLR K-GR-139','DLR TEXUS 12','','','Micrograv',416,'',253 from dual union all
select 58618,'1985-S74','1985-05-06 22:45:00','suborbital rocket','success',218,'ARS-3B-70','SMERF 2','','','Aeron',428,'',114 from dual union all
select 58619,'1985-S75','1985-05-14 04:00:00','suborbital rocket','success',891,'DLR K-O-140','DFVLR Strafam 1/5','','','Auroral?',417,'',56 from dual union all
select 5862,'1959-S110','1959-07-15 09:15:00','suborbital rocket','success',366,'','RW-134','','','Aeron',1490,'',81 from dual union all
select 58620,'1985-S76','1985-05-19 00:55:00','military missile','failure',932,'A3TK','CHEVALINE DASO','','','OT',421,'UK S22',1000 from dual union all
select 58621,'1985-S77','1985-05-22 00:00:00','military missile','success',795,'','IPA-2','RSLP IPA-2','','Reentry Test',1698,'',1300 from dual union all
select 58622,'1985-S78','1985-05-22 00:00:00','military missile','success',922,'','','','','Test',636,'',500 from dual union all
select 58623,'1985-S79','1985-05-29 00:59:53','suborbital rocket','success',1073,'S-310JA-11','W-P I','','','Ionos',1458,'',212 from dual union all
select 58624,'1985-S80','1985-06-03 00:00:00','military missile','success',907,'FTM-8','Mk 21 RV x 6','','','Test',1726,'',1000 from dual union all
select 58625,'1985-S81','1985-06-10 08:26:00','military missile','success',932,'A3TK','CHEVALINE DASO','','','OT',421,'UK S22',1000 from dual union all
select 58626,'1985-S82','1985-06-13 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 644',500 from dual union all
select 58627,'1985-S83','1985-06-13 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 644',500 from dual union all
select 58628,'1985-S84','1985-06-13 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 644',500 from dual union all
select 58629,'1985-S85','1985-06-13 06:08:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 644',500 from dual union all
select 5863,'1959-S111','1959-07-15 15:13:00','suborbital rocket','success',366,'','RW-135','','','Aeron',1490,'',83 from dual union all
select 58630,'1985-S86','1985-06-14 00:00:00','suborbital rocket','success',211,'GL A19.250','BERT','','','Active Plasma',1895,'',244 from dual union all
select 58631,'1985-S87','1985-06-14 07:15:00','suborbital rocket','success',227,'NASA 21.81UG','JHU FOT 5 Io','','','UV Astron',1873,'',238 from dual union all
select 58632,'1985-S88','1985-06-16 08:22:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT108GM','GLORY TRIP 108GM','OT',1699,'',1300 from dual union all
select 58633,'1985-S89','1985-06-16 10:15:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT109GB','GLORY TRIP 109GB','OT',1712,'',1300 from dual union all
select 58634,'1985-S90','1985-06-17 04:15:00','suborbital rocket','failure',282,'GL A44.362','BEAM','','','IR Astron',330,'',500 from dual union all
select 58635,'1985-S91','1985-06-18 00:00:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 633',1000 from dual union all
select 58636,'1985-S92','1985-06-18 00:00:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 633',1000 from dual union all
select 58637,'1985-S93','1985-06-18 00:00:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 633',1000 from dual union all
select 58638,'1985-S94','1985-06-18 03:12:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 633',1000 from dual union all
select 58639,'1985-S95','1985-06-18 23:00:00','military missile','success',922,'','Pioner      ','','','Test',636,'',500 from dual union all
select 5864,'1959-S112','1959-07-18 18:15:00','military missile','success',1016,'No. I3-24','8K71 LKI III-8','GCh No. I3-24','','Test',480,'',1350 from dual union all
select 58640,'1985-S96','1985-06-24 17:46:00','suborbital rocket','success',860,'NASA 31.53UE','UIUC Ionos','','','Plasma',1821,'',140 from dual union all
select 58641,'1985-S97','1985-06-26 17:46:00','suborbital rocket','success',860,'NASA 31.54UE','UIUC Ionos','','','Plasma',1821,'',140 from dual union all
select 58642,'1985-S98','1985-07-06 00:55:00','military missile','success',1442,'','DASO','','','OT',421,'SSBN 731',1000 from dual union all
select 58643,'1985-S99','1985-07-12 19:35:39','suborbital rocket','success',1073,'S-310JA-12','W-P II','','','Ionos',1458,'',222 from dual union all
select 58644,'1985-W01','1985-01-02 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',59 from dual union all
select 58645,'1985-W02','1985-01-02 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',57 from dual union all
select 58646,'1985-W03','1985-01-02 17:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',64 from dual union all
select 58647,'1985-W04','1985-01-03 15:31:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 58648,'1985-W05','1985-01-08 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',66 from dual union all
select 58649,'1985-W06','1985-01-09 02:04:59','suborbital rocket','success',761,'MT135P-R552','','','','Meteo',1332,'',53 from dual union all
select 5865,'1959-S113','1959-07-20 00:00:00','suborbital rocket','success',972,'[dup for Jul 21]','','','','Aeron?',636,'',86 from dual union all
select 58650,'1985-W07','1985-01-09 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',61 from dual union all
select 58651,'1985-W08','1985-01-10 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 58652,'1985-W09','1985-01-10 15:56:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 58653,'1985-W10','1985-01-10 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',56 from dual union all
select 58654,'1985-W100','1985-04-10 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 58655,'1985-W101','1985-04-12 14:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',62 from dual union all
select 58656,'1985-W102','1985-04-12 15:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 58657,'1985-W103','1985-04-12 15:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 58658,'1985-W104','1985-04-14 13:58:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',64 from dual union all
select 58659,'1985-W105','1985-04-14 14:08:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 5866,'1959-S114','1959-07-21 00:00:00','suborbital rocket','failure',624,'JV-2','','','','Test',1821,'',100 from dual union all
select 58660,'1985-W106','1985-04-14 14:13:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',63 from dual union all
select 58661,'1985-W107','1985-04-16 12:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',77 from dual union all
select 58662,'1985-W108','1985-04-16 12:17:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 58663,'1985-W109','1985-04-16 14:13:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',63 from dual union all
select 58664,'1985-W11','1985-01-15 12:43:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 58665,'1985-W110','1985-04-17 02:00:00','suborbital rocket','success',761,'MT135P-R563','','','','Meteo',1332,'',55 from dual union all
select 58666,'1985-W111','1985-04-17 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 58667,'1985-W112','1985-04-18 12:12:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',56 from dual union all
select 58668,'1985-W113','1985-04-18 12:19:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',77 from dual union all
select 58669,'1985-W114','1985-04-18 12:46:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',60 from dual union all
select 5867,'1959-S115','1959-07-21 00:00:00','suborbital rocket','success',994,'','','','','Solar X',636,'',105 from dual union all
select 58670,'1985-W115','1985-04-19 06:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',64 from dual union all
select 58671,'1985-W116','1985-04-19 13:53:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',58 from dual union all
select 58672,'1985-W117','1985-04-19 13:53:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',64 from dual union all
select 58673,'1985-W118','1985-04-19 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',55 from dual union all
select 58674,'1985-W119','1985-04-19 14:01:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',54 from dual union all
select 58675,'1985-W12','1985-01-15 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 58676,'1985-W120','1985-04-19 14:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',76 from dual union all
select 58677,'1985-W121','1985-04-19 14:12:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 58678,'1985-W122','1985-04-19 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',50 from dual union all
select 58679,'1985-W123','1985-04-19 15:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',74 from dual union all
select 5868,'1959-S116','1959-07-21 02:00:00','suborbital rocket','success',972,'','','','','Ionos/Aeron/Solar X',636,'',105 from dual union all
select 58680,'1985-W124','1985-04-22 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',55 from dual union all
select 58681,'1985-W125','1985-04-24 02:00:00','suborbital rocket','success',761,'MT135P-R564','','','','Meteo',1332,'',56 from dual union all
select 58682,'1985-W126','1985-04-24 16:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',57 from dual union all
select 58683,'1985-W127','1985-04-25 18:45:00','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 58684,'1985-W128','1985-04-27 15:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 58685,'1985-W129','1985-04-29 16:52:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',67 from dual union all
select 58686,'1985-W13','1985-01-16 02:00:00','suborbital rocket','success',761,'MT135P-R553','','','','Meteo',1332,'',57 from dual union all
select 58687,'1985-W130','1985-04-29 17:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 58688,'1985-W131','1985-04-29 17:38:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',58 from dual union all
select 58689,'1985-W132','1985-04-29 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',53 from dual;
insert into LAUNCH
select 5869,'1959-S117','1959-07-21 05:22:00','military missile','success',150,'8C','RVX-2/AFSWC-5','R&D','','Test/Particles',227,'',900 from dual union all
select 58690,'1985-W133','1985-04-29 20:03:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',73 from dual union all
select 58691,'1985-W134','1985-05-01 16:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 58692,'1985-W135','1985-05-01 16:07:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 58693,'1985-W136','1985-05-03 15:56:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 58694,'1985-W137','1985-05-03 15:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',64 from dual union all
select 58695,'1985-W138','1985-05-03 16:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 58696,'1985-W139','1985-05-03 16:29:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',61 from dual union all
select 58697,'1985-W14','1985-01-16 17:58:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',76 from dual union all
select 58698,'1985-W140','1985-05-03 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 58699,'1985-W141','1985-05-05 15:57:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',72 from dual union all
select 587,'1945-S63','1945-01-20 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5870,'1959-S118','1959-07-21 14:00:00','suborbital rocket','success',49,'NN3.23F','','IGY NN3.23F','','Solar UV/XR Astron',1869,'',197 from dual union all
select 58700,'1985-W142','1985-05-05 15:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',62 from dual union all
select 58701,'1985-W143','1985-05-05 16:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 58702,'1985-W144','1985-05-06 10:42:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',72 from dual union all
select 58703,'1985-W145','1985-05-06 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',67 from dual union all
select 58704,'1985-W146','1985-05-06 16:10:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',69 from dual union all
select 58705,'1985-W147','1985-05-06 16:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',64 from dual union all
select 58706,'1985-W148','1985-05-06 17:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',75 from dual union all
select 58707,'1985-W149','1985-05-06 17:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',77 from dual union all
select 58708,'1985-W15','1985-01-16 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',56 from dual union all
select 58709,'1985-W150','1985-05-06 17:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 5871,'1959-S119','1959-07-21 14:00:00','suborbital rocket','success',972,'','','','','Ionos/Aeron/Solar X',636,'',105 from dual union all
select 58710,'1985-W151','1985-05-08 02:00:00','suborbital rocket','success',761,'MT135P-R565','','','','Meteo',1332,'',55 from dual union all
select 58711,'1985-W152','1985-05-09 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',64 from dual union all
select 58712,'1985-W153','1985-05-15 02:00:00','suborbital rocket','success',761,'MT135P-R566','','','','Meteo',1332,'',56 from dual union all
select 58713,'1985-W154','1985-05-15 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',58 from dual union all
select 58714,'1985-W155','1985-05-15 15:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',72 from dual union all
select 58715,'1985-W156','1985-05-15 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',53 from dual union all
select 58716,'1985-W157','1985-05-16 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',61 from dual union all
select 58717,'1985-W158','1985-05-16 20:32:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',71 from dual union all
select 58718,'1985-W159','1985-05-19 03:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 58719,'1985-W16','1985-01-16 23:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',63 from dual union all
select 5872,'1959-S12','1959-01-30 23:53:00','military missile','success',1367,'154','R&D Series IV','','','Test',234,'',520 from dual union all
select 58720,'1985-W160','1985-05-20 09:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 58721,'1985-W161','1985-05-20 12:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',55 from dual union all
select 58722,'1985-W162','1985-05-20 18:22:59','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',74 from dual union all
select 58723,'1985-W163','1985-05-22 02:00:00','suborbital rocket','success',761,'MT135P-R567','','','','Meteo',1332,'',59 from dual union all
select 58724,'1985-W164','1985-05-22 17:19:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',65 from dual union all
select 58725,'1985-W165','1985-05-22 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',55 from dual union all
select 58726,'1985-W166','1985-05-25 00:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',72 from dual union all
select 58727,'1985-W167','1985-05-28 15:44:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',61 from dual union all
select 58728,'1985-W168','1985-05-29 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 58729,'1985-W169','1985-05-29 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',66 from dual union all
select 5873,'1959-S120','1959-07-22 01:14:00','suborbital rocket','success',972,'[dup]','','','','Ionos/Aeron',636,'',211 from dual union all
select 58730,'1985-W17','1985-01-17 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',71 from dual union all
select 58731,'1985-W170','1985-05-30 02:00:00','suborbital rocket','success',761,'MT135P-R568','','','','Meteo',1332,'',59 from dual union all
select 58732,'1985-W171','1985-05-30 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',64 from dual union all
select 58733,'1985-W172','1985-06-04 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 58734,'1985-W173','1985-06-05 19:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',54 from dual union all
select 58735,'1985-W174','1985-06-05 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',63 from dual union all
select 58736,'1985-W175','1985-06-10 10:25:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 58737,'1985-W176','1985-06-10 10:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',71 from dual union all
select 58738,'1985-W177','1985-06-11 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',67 from dual union all
select 58739,'1985-W178','1985-06-12 02:03:00','suborbital rocket','success',761,'MT135P-R569','','','','Meteo',1332,'',55 from dual union all
select 5874,'1959-S121','1959-07-22 04:02:00','military missile','success',1053,'CC-2003','','','','Test',248,'',90 from dual union all
select 58740,'1985-W179','1985-06-12 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 58741,'1985-W18','1985-01-18 16:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',62 from dual union all
select 58742,'1985-W180','1985-06-17 12:33:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',66 from dual union all
select 58743,'1985-W181','1985-06-17 12:42:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',68 from dual union all
select 58744,'1985-W182','1985-06-17 12:45:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',62 from dual union all
select 58745,'1985-W183','1985-06-17 13:32:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 58746,'1985-W184','1985-06-17 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 58747,'1985-W185','1985-06-18 06:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',67 from dual union all
select 58748,'1985-W186','1985-06-19 03:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',77 from dual union all
select 58749,'1985-W187','1985-06-19 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',64 from dual union all
select 5875,'1959-S122','1959-07-23 01:26:00','suborbital rocket','failure',1193,'SL24','RAE Test 8','','','Test',1839,'',91 from dual union all
select 58750,'1985-W188','1985-06-19 18:26:59','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',69 from dual union all
select 58751,'1985-W189','1985-06-19 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',52 from dual union all
select 58752,'1985-W19','1985-01-18 21:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',68 from dual union all
select 58753,'1985-W190','1985-06-20 02:00:00','suborbital rocket','success',761,'MT135P-R570','','','','Meteo',1332,'',56 from dual union all
select 58754,'1985-W191','1985-06-24 13:07:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',68 from dual union all
select 58755,'1985-W192','1985-06-24 13:08:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',68 from dual union all
select 58756,'1985-W193','1985-06-24 13:35:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',66 from dual union all
select 58757,'1985-W194','1985-06-24 14:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',58 from dual union all
select 58758,'1985-W195','1985-06-24 14:12:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 58759,'1985-W196','1985-06-24 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 5876,'1959-S123','1959-07-24 00:00:00','military missile','success',976,'ID4-20T','8K63 UBP-13','','','Test',636,'',402 from dual union all
select 58760,'1985-W197','1985-06-24 15:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',58 from dual union all
select 58761,'1985-W198','1985-07-03 02:00:00','suborbital rocket','success',761,'MT135P-R571','','','','Meteo',1332,'',57 from dual union all
select 58762,'1985-W199','1985-07-03 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',56 from dual union all
select 58763,'1985-W20','1985-01-21 12:48:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',57 from dual union all
select 58764,'1985-W200','1985-07-04 17:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 58765,'1985-W201','1985-07-04 19:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',77 from dual union all
select 58766,'1985-W202','1985-07-08 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',61 from dual union all
select 58767,'1985-W203','1985-07-10 02:58:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 58768,'1985-W204','1985-07-10 18:27:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 58769,'1985-W205','1985-07-10 20:03:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',52 from dual union all
select 5877,'1959-S124','1959-07-24 12:47:00','military missile','success',1367,'202','R&D Series IV','','','Test',236,'',520 from dual union all
select 58770,'1985-W206','1985-07-10 20:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 58771,'1985-W207','1985-07-11 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 58772,'1985-W208','1985-07-16 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',53 from dual union all
select 58773,'1985-W209','1985-07-17 02:00:00','suborbital rocket','success',761,'MT135P-R572','','','','Meteo',1332,'',61 from dual union all
select 58774,'1985-W21','1985-01-21 20:21:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',65 from dual union all
select 58775,'1985-W210','1985-07-24 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',56 from dual union all
select 58776,'1985-W211','1985-07-26 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 58777,'1985-W212','1985-07-29 21:23:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',53 from dual union all
select 58778,'1985-W213','1985-07-29 21:23:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',60 from dual union all
select 58779,'1985-W214','1985-07-29 21:52:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',63 from dual;
insert into LAUNCH
select 5878,'1959-S125','1959-07-24 16:34:00','suborbital rocket','success',838,'NN8.67','','','','Solar',1015,'',227 from dual union all
select 58780,'1985-W215','1985-07-30 00:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 58781,'1985-W216','1985-07-30 14:10:00','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 58782,'1985-W217','1985-07-30 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',63 from dual union all
select 58783,'1985-W218','1985-07-30 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',63 from dual union all
select 58784,'1985-W219','1985-07-31 02:00:00','suborbital rocket','success',761,'MT135P-R573','','','','Meteo',1332,'',57 from dual union all
select 58785,'1985-W22','1985-01-21 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',55 from dual union all
select 58786,'1985-W220','1985-08-07 15:00:00','suborbital rocket','unknown',716,'LOKI DATASONDE','PWN-8B','','','Test/Aeron',1821,'',57 from dual union all
select 58787,'1985-W221','1985-08-14 08:20:00','suborbital rocket','unknown',738,'','','','','Aeron',912,'KRENKEL''',58 from dual union all
select 58788,'1985-W222','1985-08-16 07:20:00','suborbital rocket','unknown',738,'','','','','Aeron',912,'KRENKEL''',58 from dual union all
select 58789,'1985-W223','1985-08-21 07:45:00','suborbital rocket','unknown',738,'','','','','Aeron',912,'KRENKEL''',59 from dual union all
select 5879,'1959-S126','1959-07-26 00:00:00','military missile','success',976,'I02-20T','8K63 UBP-14','','','Test',636,'',402 from dual union all
select 58790,'1985-W224','1985-08-23 07:13:00','suborbital rocket','unknown',738,'','','','','Aeron',912,'KRENKEL''',60 from dual union all
select 58791,'1985-W225','1985-08-28 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',912,'KRENKEL''',60 from dual union all
select 58792,'1985-W226','1985-08-30 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',912,'KRENKEL''',59 from dual union all
select 58793,'1985-W227','1985-08-30 15:00:00','suborbital rocket','unknown',716,'LOKI DATASONDE','','','','Test/Aeron',1821,'',55 from dual union all
select 58794,'1985-W228','1985-09-03 13:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',64 from dual union all
select 58795,'1985-W229','1985-09-03 13:36:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',59 from dual union all
select 58796,'1985-W23','1985-01-23 02:02:00','suborbital rocket','success',761,'MT135P-R554','','','','Meteo',1332,'',56 from dual union all
select 58797,'1985-W230','1985-09-03 13:54:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',55 from dual union all
select 58798,'1985-W231','1985-09-03 14:18:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 58799,'1985-W232','1985-09-03 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',78 from dual union all
select 588,'1945-S64','1945-01-20 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5880,'1959-S127','1959-07-28 00:00:00','military missile','success',976,'B2-5','8K63 UBP-15','','','Test',636,'',402 from dual union all
select 58800,'1985-W233','1985-09-04 13:54:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',56 from dual union all
select 58801,'1985-W234','1985-09-04 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',77 from dual union all
select 58802,'1985-W235','1985-09-04 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',58 from dual union all
select 58803,'1985-W236','1985-09-05 13:00:00','suborbital rocket','unknown',716,'LOKI DATASONDE','PWN-8B','','','Test/Aeron',1821,'',53 from dual union all
select 58804,'1985-W238','1985-09-10 01:39:00','suborbital rocket','unknown',1284,'','PWN-11D Datasonde','','','Test/Aeron',1821,'',76 from dual union all
select 58805,'1985-W239','1985-09-11 02:01:00','suborbital rocket','success',761,'MT135P-R577','','','','Meteo',1332,'',57 from dual union all
select 58806,'1985-W24','1985-01-23 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',67 from dual union all
select 58807,'1985-W240','1985-09-11 18:55:00','suborbital rocket','unknown',716,'LOKI DATASONDE','PWN-8B','','','Test/Aeron',1821,'',57 from dual union all
select 58808,'1985-W241','1985-09-12 11:55:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',68 from dual union all
select 58809,'1985-W242','1985-09-13 12:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',72 from dual union all
select 5881,'1959-S128','1959-07-28 00:00:00','military missile','success',976,'B2-8','8K63 UBP-16','','','Test',636,'',402 from dual union all
select 58810,'1985-W243','1985-09-13 17:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',61 from dual union all
select 58811,'1985-W244','1985-09-13 19:57:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 58812,'1985-W245','1985-09-13 22:38:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 58813,'1985-W246','1985-09-16 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',912,'KRENKEL''',58 from dual union all
select 58814,'1985-W247','1985-09-18 20:30:00','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',72 from dual union all
select 58815,'1985-W248','1985-09-19 02:00:00','suborbital rocket','success',761,'MT135P-R578','','','','Meteo',1332,'',59 from dual union all
select 58816,'1985-W249','1985-09-20 01:30:00','suborbital rocket','unknown',716,'LOKI DATASONDE','PWN-8B','','','Test/Aeron',1821,'',52 from dual union all
select 58817,'1985-W25','1985-01-24 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',68 from dual union all
select 58818,'1985-W250','1985-09-21 01:19:00','suborbital rocket','unknown',716,'LOKI DATASONDE','PWN-8B','','','Test/Aeron',1821,'',54 from dual union all
select 58819,'1985-W251','1985-09-21 02:19:00','suborbital rocket','success',716,'','PWN-8B','','','Aeron',1821,'',52 from dual union all
select 5882,'1959-S129','1959-07-29 04:10:00','military missile','success',154,'11D','Mk 2-1AE','R&D','','Test',226,'',1800 from dual union all
select 58820,'1985-W252','1985-09-21 03:19:00','suborbital rocket','success',716,'','PWN-8B','','','Aeron',1821,'',50 from dual union all
select 58821,'1985-W253','1985-09-23 20:15:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',58 from dual union all
select 58822,'1985-W254','1985-09-25 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 58823,'1985-W255','1985-09-25 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',57 from dual union all
select 58824,'1985-W256','1985-10-01 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 58825,'1985-W257','1985-10-02 02:00:00','suborbital rocket','success',761,'MT135P-R579','','','','Meteo',1332,'',51 from dual union all
select 58826,'1985-W258','1985-10-02 07:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 58827,'1985-W259','1985-10-02 10:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 58828,'1985-W26','1985-01-24 14:26:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',75 from dual union all
select 58829,'1985-W260','1985-10-03 17:15:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 5883,'1959-S13','1959-02-04 08:01:00','military missile','success',147,'11B','Mk 2-1 RV/AFSWC-4','R&D','','Test',226,'',990 from dual union all
select 58830,'1985-W261','1985-10-09 04:35:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1415,'',60 from dual union all
select 58831,'1985-W262','1985-10-09 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',67 from dual union all
select 58832,'1985-W263','1985-10-09 18:13:00','suborbital rocket','unknown',716,'LOKI','PWN-8B','','','Test/Aeron',1821,'',50 from dual union all
select 58833,'1985-W264','1985-10-11 21:08:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',50 from dual union all
select 58834,'1985-W265','1985-10-16 02:00:00','suborbital rocket','success',761,'MT135P-R580','','','','Meteo',1332,'',54 from dual union all
select 58835,'1985-W266','1985-10-16 13:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',77 from dual union all
select 58836,'1985-W267','1985-10-16 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',65 from dual union all
select 58837,'1985-W268','1985-10-16 19:47:00','suborbital rocket','unknown',716,'LOKI','PWN-8B','','','Test/Aeron',1821,'',51 from dual union all
select 58838,'1985-W269','1985-10-23 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',77 from dual union all
select 58839,'1985-W27','1985-01-26 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 5884,'1959-S130','1959-07-30 04:00:00','military missile','success',1016,'No. 041082','8K71 LKI III-9','GCh','','Test',480,'',1350 from dual union all
select 58840,'1985-W270','1985-10-24 12:39:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 58841,'1985-W271','1985-10-25 21:11:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',71 from dual union all
select 58842,'1985-W272','1985-10-28 07:41:00','suborbital rocket','unknown',738,'','','','','Aeron',981,'MUSSON',60 from dual union all
select 58843,'1985-W273','1985-10-28 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 58844,'1985-W274','1985-10-28 18:45:00','suborbital rocket','unknown',716,'LOKI','PWN-8B','','','Test/Aeron',1821,'',53 from dual union all
select 58845,'1985-W275','1985-10-29 15:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual union all
select 58846,'1985-W276','1985-10-30 07:21:00','suborbital rocket','unknown',738,'','','','','Aeron',981,'MUSSON',60 from dual union all
select 58847,'1985-W277','1985-10-30 13:01:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',75 from dual union all
select 58848,'1985-W278','1985-10-30 16:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',65 from dual union all
select 58849,'1985-W279','1985-10-30 17:01:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',65 from dual union all
select 5885,'1959-S131','1959-07-31 00:00:00','military missile','success',976,'B2-10T','8K63 UBP-17','','','Test',636,'',402 from dual union all
select 58850,'1985-W28','1985-01-26 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',73 from dual union all
select 58851,'1985-W280','1985-10-30 17:22:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',74 from dual union all
select 58852,'1985-W281','1985-10-30 19:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 58853,'1985-W282','1985-10-31 02:00:00','suborbital rocket','success',761,'MT135P-R581','','','','Meteo',1332,'',54 from dual union all
select 58854,'1985-W283','1985-10-31 03:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',55 from dual union all
select 58855,'1985-W284','1985-11-01 07:29:00','suborbital rocket','unknown',738,'','','','','Aeron',981,'MUSSON',59 from dual union all
select 58856,'1985-W285','1985-11-05 14:33:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',79 from dual union all
select 58857,'1985-W286','1985-11-06 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',58 from dual union all
select 58858,'1985-W287','1985-11-06 07:09:00','suborbital rocket','unknown',738,'','','','','Aeron',981,'MUSSON',61 from dual union all
select 58859,'1985-W288','1985-11-06 17:40:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',73 from dual union all
select 5886,'1959-S132','1959-08-03 21:41:00','military missile','success',1367,'175','IWST','','BEAN BALL','Test',1673,'',520 from dual union all
select 58860,'1985-W289','1985-11-06 17:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',63 from dual union all
select 58861,'1985-W29','1985-01-26 14:03:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',68 from dual union all
select 58862,'1985-W290','1985-11-06 17:44:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',68 from dual union all
select 58863,'1985-W291','1985-11-06 17:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',73 from dual union all
select 58864,'1985-W292','1985-11-06 18:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',63 from dual union all
select 58865,'1985-W293','1985-11-06 18:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 58866,'1985-W294','1985-11-06 18:50:00','suborbital rocket','unknown',716,'LOKI','PWN-8B','','','Test/Aeron',1821,'',56 from dual union all
select 58867,'1985-W295','1985-11-13 02:00:00','suborbital rocket','success',761,'MT135P-R583','','','','Meteo',1332,'',58 from dual union all
select 58868,'1985-W296','1985-11-13 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',981,'MUSSON',64 from dual union all
select 58869,'1985-W297','1985-11-16 12:17:00','suborbital rocket','unknown',1284,'PWN-11D','','','','Test/Aeron',1821,'',77 from dual;
insert into LAUNCH
select 5887,'1959-S133','1959-08-06 02:48:00','military missile','success',1367,'208','R&D Series IV','','','Test',234,'',520 from dual union all
select 58870,'1985-W298','1985-11-20 02:00:00','suborbital rocket','success',761,'MT135P-R584','','','','Meteo',1332,'',54 from dual union all
select 58871,'1985-W299','1985-11-20 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',981,'MUSSON',60 from dual union all
select 58872,'1985-W30','1985-01-27 16:31:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',72 from dual union all
select 58873,'1985-W300','1985-11-20 19:00:00','suborbital rocket','unknown',716,'LOKI','PWN-8B','','','Test/Aeron',1821,'',57 from dual union all
select 58874,'1985-W301','1985-11-22 07:30:00','suborbital rocket','unknown',738,'','','','','Aeron',981,'MUSSON',60 from dual union all
select 58875,'1985-W302','1985-11-25 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 58876,'1985-W303','1985-11-27 00:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',64 from dual union all
select 58877,'1985-W304','1985-11-27 01:37:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 58878,'1985-W305','1985-11-27 02:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',73 from dual union all
select 58879,'1985-W306','1985-11-27 14:08:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 5888,'1959-S134','1959-08-05 02:05:00','military missile','success',1053,'CC-2004','','','','Test',248,'',90 from dual union all
select 58880,'1985-W307','1985-11-27 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 58881,'1985-W308','1985-11-28 12:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',71 from dual union all
select 58882,'1985-W309','1985-11-29 07:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',79 from dual union all
select 58883,'1985-W31','1985-01-27 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',54 from dual union all
select 58884,'1985-W310','1985-12-01 11:29:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',77 from dual union all
select 58885,'1985-W311','1985-12-03 15:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',71 from dual union all
select 58886,'1985-W312','1985-12-03 21:32:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',64 from dual union all
select 58887,'1985-W313','1985-12-03 21:33:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 58888,'1985-W314','1985-12-03 21:33:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',63 from dual union all
select 58889,'1985-W315','1985-12-03 21:33:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',59 from dual union all
select 5889,'1959-S135','1959-08-07 00:00:00','suborbital rocket','failure',89,'NASA 2.06GT','','','','Test',1821,'',104 from dual union all
select 58890,'1985-W316','1985-12-03 21:34:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',61 from dual union all
select 58891,'1985-W317','1985-12-03 21:55:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',62 from dual union all
select 58892,'1985-W318','1985-12-03 22:33:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 58893,'1985-W319','1985-12-03 23:34:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 58894,'1985-W32','1985-01-27 21:23:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',66 from dual union all
select 58895,'1985-W320','1985-12-04 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',52 from dual union all
select 58896,'1985-W321','1985-12-05 02:01:00','suborbital rocket','success',761,'MT135P-R585','','','','Meteo',1332,'',55 from dual union all
select 58897,'1985-W322','1985-12-05 19:00:00','suborbital rocket','unknown',716,'LOKI','PWN-8B','','','Test/Aeron',1821,'',55 from dual union all
select 58898,'1985-W323','1985-12-06 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 58899,'1985-W324','1985-12-08 03:15:00','suborbital rocket','success',716,'','PWN-8B','','','Aeron',1821,'',56 from dual union all
select 589,'1945-S65','1945-01-20 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5890,'1959-S136','1959-08-07 00:00:00','military missile','success',976,'I01-11T','8K63 UBP-18','','','Test',636,'',402 from dual union all
select 58900,'1985-W325','1985-12-09 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 58901,'1985-W326','1985-12-10 00:00:00','suborbital rocket','unknown',716,'LOKI','PWN-8B','','','Test/Aeron',1821,'',56 from dual union all
select 58902,'1985-W327','1985-12-10 00:50:00','suborbital rocket','unknown',1284,'PWN-11D','','','','Test/Aeron',1821,'',70 from dual union all
select 58903,'1985-W328','1985-12-11 00:00:00','suborbital rocket','unknown',1284,'PWN-11D','','','','Test/Aeron',1821,'',75 from dual union all
select 58904,'1985-W329','1985-12-11 01:00:00','suborbital rocket','unknown',716,'LOKI','PWN-8B','','','Test/Aeron',1821,'',56 from dual union all
select 58905,'1985-W33','1985-01-27 21:30:00','suborbital rocket','unknown',1284,'DATASONDE','PWN-11A','','','Test/Aeron',1821,'',79 from dual union all
select 58906,'1985-W330','1985-12-11 02:04:59','suborbital rocket','success',761,'MT135P-R586','','','','Meteo',1332,'',55 from dual union all
select 58907,'1985-W331','1985-12-11 21:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 58908,'1985-W332','1985-12-11 23:30:00','suborbital rocket','unknown',716,'LOKI','PWN-8B','','','Test/Aeron',1821,'',57 from dual union all
select 58909,'1985-W333','1985-12-13 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 5891,'1959-S137','1959-08-07 17:05:00','suborbital rocket','success',838,'NN8.68CF','','','','Solar X',1015,'',225 from dual union all
select 58910,'1985-W334','1985-12-16 14:03:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',57 from dual union all
select 58911,'1985-W335','1985-12-16 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',77 from dual union all
select 58912,'1985-W336','1985-12-16 18:00:00','suborbital rocket','unknown',716,'LOKI','PWN-8B','','','Test/Aeron',1821,'',56 from dual union all
select 58913,'1985-W337','1985-12-16 23:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',53 from dual union all
select 58914,'1985-W338','1985-12-17 00:00:00','suborbital rocket','unknown',1284,'PWN-11D','','','','Test/Aeron',1821,'',76 from dual union all
select 58915,'1985-W339','1985-12-17 01:30:00','suborbital rocket','unknown',1284,'PWN-11D','','','','Test/Aeron',1821,'',79 from dual union all
select 58916,'1985-W34','1985-01-27 21:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',52 from dual union all
select 58917,'1985-W340','1985-12-17 23:52:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 58918,'1985-W341','1985-12-18 02:04:59','suborbital rocket','success',761,'MT135P-R587','','','','Meteo',1332,'',57 from dual union all
select 58919,'1985-W342','1985-12-18 02:36:00','suborbital rocket','unknown',716,'PWN-8B','PWN-8B','','','Test/Aeron',1821,'',56 from dual union all
select 5892,'1959-S138','1959-08-11 18:01:00','military missile','success',154,'14D','Mk 2-2A','R&D','','Test',228,'',1800 from dual union all
select 58920,'1985-W343','1985-12-18 20:25:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 58921,'1985-W344','1985-12-23 19:00:00','suborbital rocket','unknown',716,'LOKI    PWN-8B','PWN-8B','','','Test/Aeron',1821,'',54 from dual union all
select 58922,'1985-W345','1985-12-23 19:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',51 from dual union all
select 58923,'1985-W346','1985-12-24 21:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',75 from dual union all
select 58924,'1985-W347','1985-12-25 02:00:00','suborbital rocket','success',761,'MT135P-R588','','','','Meteo',1332,'',59 from dual union all
select 58925,'1985-W348','1985-12-30 15:00:00','suborbital rocket','unknown',716,'LOKI    PWN-8B','Final US MET Loki','','','Test/Aeron',1821,'',55 from dual union all
select 58926,'1985-W349','1985-03-06 02:05:00','suborbital rocket','success',761,'MT135P-R559','','','','Meteo',1332,'',60 from dual union all
select 58927,'1985-W35','1985-01-27 21:51:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',72 from dual union all
select 58928,'1985-W350','1985-04-10 02:00:00','suborbital rocket','success',761,'MT135P-R562','','','','Meteo',1332,'',60 from dual union all
select 58929,'1985-W351','1985-08-14 02:03:00','suborbital rocket','success',761,'MT135P-R574','','','','Meteo',1332,'',60 from dual union all
select 5893,'1959-S139','1959-08-13 23:14:00','military missile','success',1016,'No. I3-25','8K71 LKI III-10','GCh No. I3-25','','Test',480,'',1350 from dual union all
select 58930,'1985-W352','1985-08-21 02:00:00','suborbital rocket','success',761,'MT135P-R575','','','','Meteo',1332,'',60 from dual union all
select 58931,'1985-W353','1985-08-28 02:00:00','suborbital rocket','success',761,'MT135P-R576','','','','Meteo',1332,'',60 from dual union all
select 58932,'1985-W354','1985-01-30 14:00:00','suborbital rocket','unknown',760,'MT135JA-1','','','','Meteo',1458,'',60 from dual union all
select 58933,'1985-W355','1985-03-26 14:30:00','suborbital rocket','success',760,'MT135JA-2','','','','Meteo',1458,'',60 from dual union all
select 58934,'1985-W356','1985-06-28 13:35:00','suborbital rocket','success',760,'MT135JA-3','','','','Meteo',1458,'',61 from dual union all
select 58935,'1985-W357','1985-06-28 16:16:00','suborbital rocket','success',760,'MT135JA-5','','','','Meteo',1458,'',71 from dual union all
select 58936,'1985-W358','1985-06-28 18:10:00','suborbital rocket','success',760,'MT135JA-6','','','','Meteo',1458,'',73 from dual union all
select 58937,'1985-W359','1985-06-28 20:02:00','suborbital rocket','success',760,'MT135JA-7','','','','Meteo',1458,'',69 from dual union all
select 58938,'1985-W36','1985-01-27 22:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 58939,'1985-W360','1985-06-28 21:58:00','suborbital rocket','success',760,'MT135JA-4','','','','Meteo',1458,'',69 from dual union all
select 5894,'1959-S14','1959-02-05 14:45:00','suborbital rocket','success',30,'AA3.100C','Expt 5.2','','','Meteorites',640,'',212 from dual union all
select 58940,'1985-W361','1985-09-25 14:00:00','suborbital rocket','success',760,'MT135JA-9','','','','Meteo',1458,'',71 from dual union all
select 58941,'1985-W362','1985-09-25 16:00:00','suborbital rocket','success',760,'MT135JA-8','','','','Meteo',1458,'',71 from dual union all
select 58942,'1985-W363','1985-09-25 18:00:00','suborbital rocket','success',760,'MT135JA-11','','','','Meteo',1458,'',72 from dual union all
select 58943,'1985-W364','1985-09-25 20:00:00','suborbital rocket','success',760,'MT135JA-10','','','','Meteo',1458,'',73 from dual union all
select 58944,'1985-W365','1985-09-11 02:25:00','suborbital rocket','success',759,'MT-135-44','SONDE','','','Meteo',839,'',63 from dual union all
select 58945,'1985-W37','1985-01-27 22:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',56 from dual union all
select 58946,'1985-W38','1985-01-29 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',72 from dual union all
select 58947,'1985-W39','1985-01-30 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',61 from dual union all
select 58948,'1985-W40','1985-01-30 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 58949,'1985-W41','1985-02-05 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 5895,'1959-S140','1959-08-14 19:30:00','military missile','success',933,'AX-13','SMS test','','','Test',245,'',500 from dual union all
select 58950,'1985-W42','1985-02-06 00:25:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',78 from dual union all
select 58951,'1985-W43','1985-02-06 02:00:00','suborbital rocket','success',761,'MT135P-R555','','','','Meteo',1332,'',55 from dual union all
select 58952,'1985-W44','1985-02-06 15:18:00','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',72 from dual union all
select 58953,'1985-W45','1985-02-07 12:47:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',56 from dual union all
select 58954,'1985-W46','1985-02-07 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',62 from dual union all
select 58955,'1985-W47','1985-02-08 06:40:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',75 from dual union all
select 58956,'1985-W48','1985-02-08 07:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',74 from dual union all
select 58957,'1985-W49','1985-02-08 08:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',58 from dual union all
select 58958,'1985-W50','1985-02-08 20:33:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',53 from dual union all
select 58959,'1985-W51','1985-02-08 21:25:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',65 from dual;
insert into LAUNCH
select 5896,'1959-S141','1959-08-14 09:00:00','military missile','success',1367,'204','R&D Series IV','','','Test',236,'',520 from dual union all
select 58960,'1985-W52','1985-02-08 21:51:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',67 from dual union all
select 58961,'1985-W53','1985-02-12 00:32:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',76 from dual union all
select 58962,'1985-W54','1985-02-13 02:00:00','suborbital rocket','success',761,'MT135P-R556','','','','Meteo',1332,'',54 from dual union all
select 58963,'1985-W55','1985-02-13 12:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',58 from dual union all
select 58964,'1985-W56','1985-02-13 13:54:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',72 from dual union all
select 58965,'1985-W57','1985-02-13 20:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',64 from dual union all
select 58966,'1985-W58','1985-02-14 20:14:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',65 from dual union all
select 58967,'1985-W59','1985-02-14 20:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',63 from dual union all
select 58968,'1985-W60','1985-02-15 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 58969,'1985-W61','1985-02-18 12:42:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',58 from dual union all
select 5897,'1959-S142','1959-08-14 16:00:00','suborbital rocket','success',838,'NN8.69CF','','','','Solar X',1015,'',237 from dual union all
select 58970,'1985-W62','1985-02-18 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',53 from dual union all
select 58971,'1985-W63','1985-02-19 13:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',60 from dual union all
select 58972,'1985-W64','1985-02-19 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',75 from dual union all
select 58973,'1985-W65','1985-02-20 21:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',67 from dual union all
select 58974,'1985-W66','1985-02-21 02:00:00','suborbital rocket','success',761,'MT135P-R557','','','','Meteo',1332,'',58 from dual union all
select 58975,'1985-W67','1985-02-22 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',60 from dual union all
select 58976,'1985-W68','1985-02-25 20:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 58977,'1985-W69','1985-02-27 02:00:00','suborbital rocket','success',761,'MT135P-R558','','','','Meteo',1332,'',56 from dual union all
select 58978,'1985-W70','1985-02-27 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',62 from dual union all
select 58979,'1985-W71','1985-02-27 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 5898,'1959-S143','1959-08-17 09:18:00','suborbital rocket','success',838,'NASA 03.13CA','GCA Release 1 Na','','','Aeronomy',1821,'',236 from dual union all
select 58980,'1985-W72','1985-02-27 16:33:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',64 from dual union all
select 58981,'1985-W73','1985-02-28 21:02:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',54 from dual union all
select 58982,'1985-W74','1985-03-07 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',67 from dual union all
select 58983,'1985-W75','1985-03-08 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',56 from dual union all
select 58984,'1985-W76','1985-03-11 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',65 from dual union all
select 58985,'1985-W77','1985-03-11 18:27:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',73 from dual union all
select 58986,'1985-W78','1985-03-12 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',54 from dual union all
select 58987,'1985-W79','1985-03-13 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',59 from dual union all
select 58988,'1985-W80','1985-03-14 02:00:00','suborbital rocket','success',761,'MT135P-R560','','','','Meteo',1332,'',55 from dual union all
select 58989,'1985-W81','1985-03-15 20:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',50 from dual union all
select 5899,'1959-S144','1959-08-19 00:20:00','suborbital rocket','success',1194,'SL17','UCL LP-2 (Lang.Pr.)','','','Iono',1839,'',144 from dual union all
select 58990,'1985-W82','1985-03-19 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',64 from dual union all
select 58991,'1985-W83','1985-03-20 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 58992,'1985-W84','1985-03-20 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',66 from dual union all
select 58993,'1985-W85','1985-03-21 20:44:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',70 from dual union all
select 58994,'1985-W86','1985-03-26 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',64 from dual union all
select 58995,'1985-W87','1985-03-26 20:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 58996,'1985-W88','1985-03-26 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 58997,'1985-W89','1985-03-27 02:00:00','suborbital rocket','success',761,'MT135P-R561','','','','Meteo',1332,'',57 from dual union all
select 58998,'1985-W90','1985-03-28 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',60 from dual union all
select 58999,'1985-W91','1985-03-29 00:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',70 from dual union all
select 59,'1944-A23','1944-11-27 11:13:00','military missile','failure',1470,'19840','Ma306','','','Test',661,'',0 from dual union all
select 590,'1945-S66','1945-01-20 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5900,'1959-S145','1959-08-22 00:45:00','suborbital rocket','success',30,'AA3.101C','','','','Aeron',1869,'',230 from dual union all
select 59000,'1985-W92','1985-03-29 18:29:00','suborbital rocket','unknown',1284,'DATASONDE','PWN-11D','','','Test/Aeron',1047,'',75 from dual union all
select 59001,'1985-W93','1985-04-01 17:17:00','suborbital rocket','unknown',1284,'DATASONDE','PWN-11D','','','Test/Aeron',1047,'',75 from dual union all
select 59002,'1985-W94','1985-04-01 19:28:00','suborbital rocket','unknown',1284,'DATASONDE','PWN-11D','','','Test/Aeron',1047,'',75 from dual union all
select 59003,'1985-W95','1985-04-03 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 59004,'1985-W96','1985-04-03 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',67 from dual union all
select 59005,'1985-W97','1985-04-08 20:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',52 from dual union all
select 59006,'1985-W98','1985-04-09 17:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',66 from dual union all
select 59007,'1985-W99','1985-04-10 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',62 from dual union all
select 59008,'1985-Y01','1985-01-24 13:57:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 59009,'1985-Y02','1985-01-24 20:49:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',86 from dual union all
select 5901,'1959-S146','1959-08-22 02:30:00','suborbital rocket','success',838,'NN8.70CF','','','','Solar',1015,'',200 from dual union all
select 59010,'1985-Y03','1985-01-27 16:07:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',84 from dual union all
select 59011,'1985-Y04','1985-01-27 21:22:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',84 from dual union all
select 59012,'1985-Y05','1985-01-27 23:15:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 59013,'1985-Y06','1985-03-29 17:04:00','suborbital rocket','success',1281,'NASA 15.232UE','','','','Plasma',1047,'',80 from dual union all
select 59014,'1985-Y07','1985-04-01 18:22:40','suborbital rocket','success',1281,'NASA 15.233UE','','','','Plasma',1047,'',80 from dual union all
select 59015,'1985-Y08','1985-04-12 13:58:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 59016,'1985-Y09','1985-04-12 14:43:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',86 from dual union all
select 59017,'1985-Y10','1985-04-19 06:20:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 59018,'1985-Y100','1985-10-17 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 59019,'1985-Y101','1985-10-22 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',91 from dual union all
select 5902,'1959-S147','1959-08-24 15:53:00','military missile','success',150,'11C','Mk 2-1AE/AFSWC-6','R&D','','Test',227,'',1400 from dual union all
select 59020,'1985-Y102','1985-10-24 12:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 59021,'1985-Y103','1985-10-24 17:49:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 59022,'1985-Y104','1985-10-30 14:26:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 59023,'1985-Y105','1985-10-30 14:26:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',84 from dual union all
select 59024,'1985-Y106','1985-10-30 18:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',80 from dual union all
select 59025,'1985-Y107','1985-11-03 11:52:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 59026,'1985-Y108','1985-11-04 14:38:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',86 from dual union all
select 59027,'1985-Y109','1985-11-06 05:27:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',88 from dual union all
select 59028,'1985-Y11','1985-04-19 13:35:00','suborbital rocket','unknown',1284,'DATASONDE','PWN-11D','','','Test/Aeron',1821,'',80 from dual union all
select 59029,'1985-Y110','1985-11-06 11:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 5903,'1959-S148','1959-08-24 22:47:00','suborbital rocket','success',838,'NN8.71CF','','','','Solar',1015,'',200 from dual union all
select 59030,'1985-Y111','1985-11-06 17:44:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 59031,'1985-Y112','1985-11-08 14:21:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',87 from dual union all
select 59032,'1985-Y113','1985-11-12 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 59033,'1985-Y114','1985-11-13 11:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 59034,'1985-Y115','1985-11-13 11:44:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 59035,'1985-Y116','1985-11-13 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 59036,'1985-Y117','1985-11-15 10:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 59037,'1985-Y118','1985-11-18 17:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 59038,'1985-Y119','1985-11-19 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 59039,'1985-Y12','1985-04-19 13:47:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 5904,'1959-S149','1959-08-26 00:00:00','military missile','success',976,'ID6-20T','8K63 UBP-20','','','Test',636,'',402 from dual union all
select 59040,'1985-Y120','1985-11-19 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 59041,'1985-Y121','1985-11-20 11:54:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 59042,'1985-Y122','1985-11-20 12:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',88 from dual union all
select 59043,'1985-Y123','1985-11-20 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',88 from dual union all
select 59044,'1985-Y124','1985-11-25 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',87 from dual union all
select 59045,'1985-Y125','1985-11-25 15:22:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',86 from dual union all
select 59046,'1985-Y126','1985-11-26 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 59047,'1985-Y127','1985-11-27 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 59048,'1985-Y128','1985-11-27 16:32:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual;
insert into LAUNCH
select 59049,'1985-Y129','1985-11-29 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',84 from dual union all
select 5905,'1959-S15','1959-02-06 21:22:00','military missile','success',1419,'A-3','P200 RV','Dummy St 2','','Test',230,'',200 from dual union all
select 59050,'1985-Y13','1985-04-19 14:37:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 59051,'1985-Y130','1985-11-29 15:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',85 from dual union all
select 59052,'1985-Y131','1985-12-03 09:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',84 from dual union all
select 59053,'1985-Y132','1985-12-03 16:50:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',90 from dual union all
select 59054,'1985-Y133','1985-12-04 11:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 59055,'1985-Y134','1985-12-04 12:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',87 from dual union all
select 59056,'1985-Y135','1985-12-04 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',96 from dual union all
select 59057,'1985-Y136','1985-12-04 15:22:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 59058,'1985-Y137','1985-12-05 10:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',87 from dual union all
select 59059,'1985-Y138','1985-12-05 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',95 from dual union all
select 5906,'1959-S150','1959-08-26 13:58:00','suborbital rocket','success',366,'','RW-137','','','Aeron',1490,'',81 from dual union all
select 59060,'1985-Y139','1985-12-05 16:33:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',85 from dual union all
select 59061,'1985-Y14','1985-04-29 16:02:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 59062,'1985-Y140','1985-12-05 17:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',83 from dual union all
select 59063,'1985-Y141','1985-12-06 13:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',90 from dual union all
select 59064,'1985-Y142','1985-12-10 14:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',93 from dual union all
select 59065,'1985-Y143','1985-12-10 16:15:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',96 from dual union all
select 59066,'1985-Y144','1985-12-10 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 59067,'1985-Y145','1985-12-11 11:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 59068,'1985-Y146','1985-12-11 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 59069,'1985-Y147','1985-12-11 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 5907,'1959-S151','1959-08-27 16:52:00','military missile','success',933,'AX-22','','','','Test',421,'EAG-154',500 from dual union all
select 59070,'1985-Y148','1985-12-11 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 59071,'1985-Y149','1985-12-15 09:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',83 from dual union all
select 59072,'1985-Y15','1985-04-29 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 59073,'1985-Y150','1985-12-15 16:02:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',85 from dual union all
select 59074,'1985-Y151','1985-12-15 17:16:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',89 from dual union all
select 59075,'1985-Y152','1985-12-17 01:00:00','suborbital rocket','unknown',1284,'PWN-12A','','','','Test/Aeron',1821,'',112 from dual union all
select 59076,'1985-Y153','1985-12-17 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 59077,'1985-Y154','1985-12-18 00:55:00','suborbital rocket','unknown',1284,'PWN-12A','','','','Test/Aeron',1821,'',115 from dual union all
select 59078,'1985-Y155','1985-12-18 01:20:00','suborbital rocket','unknown',1284,'PWN-11D','','','','Test/Aeron',1821,'',80 from dual union all
select 59079,'1985-Y156','1985-12-18 11:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 5908,'1959-S152','1959-08-27 12:30:00','military missile','success',1367,'216','R&D Series IV','','','Test',234,'',520 from dual union all
select 59080,'1985-Y157','1985-12-18 12:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 59081,'1985-Y158','1985-12-18 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 59082,'1985-Y159','1985-12-18 14:22:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 59083,'1985-Y16','1985-05-06 10:06:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 59084,'1985-Y160','1985-12-25 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 59085,'1985-Y161','1985-12-25 14:09:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 59086,'1985-Y162','1985-12-25 15:30:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 59087,'1985-Y163','1985-12-26 11:19:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 59088,'1985-Y164','1985-12-30 06:43:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 59089,'1985-Y165','1985-12-30 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 5909,'1959-S153','1959-08-27 01:30:00','military missile','success',638,'AM-19','R&D Short range','','','Test/Ionos',279,'',500 from dual union all
select 59090,'1985-Y166','1985-01-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59091,'1985-Y167','1985-01-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59092,'1985-Y168','1985-01-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59093,'1985-Y169','1985-01-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59094,'1985-Y17','1985-05-06 16:10:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 59095,'1985-Y170','1985-02-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59096,'1985-Y171','1985-02-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59097,'1985-Y172','1985-02-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59098,'1985-Y173','1985-02-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59099,'1985-Y174','1985-04-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 591,'1945-S67','1945-01-20 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5910,'1959-S154','1959-08-28 00:00:00','military missile','success',976,'IP2-5T','8K63 UBP-21','','','Test',636,'',402 from dual union all
select 59100,'1985-Y175','1985-04-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59101,'1985-Y176','1985-04-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59102,'1985-Y177','1985-04-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59103,'1985-Y178','1985-05-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59104,'1985-Y179','1985-05-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59105,'1985-Y18','1985-05-06 16:13:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',93 from dual union all
select 59106,'1985-Y180','1985-05-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59107,'1985-Y181','1985-05-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59108,'1985-Y182','1985-05-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59109,'1985-Y183','1985-06-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 5911,'1959-S155','1959-08-28 16:10:00','suborbital rocket','success',838,'NN8.72CF','','','','Solar X',1015,'',214 from dual union all
select 59110,'1985-Y184','1985-06-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59111,'1985-Y185','1985-06-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59112,'1985-Y186','1985-06-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59113,'1985-Y19','1985-05-22 15:45:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',89 from dual union all
select 59114,'1985-Y20','1985-06-04 00:33:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',80 from dual union all
select 59115,'1985-Y21','1985-06-17 11:33:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 59116,'1985-Y22','1985-06-24 13:11:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 59117,'1985-Y23','1985-07-02 11:43:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',87 from dual union all
select 59118,'1985-Y24','1985-07-02 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 59119,'1985-Y25','1985-07-03 14:22:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',87 from dual union all
select 5912,'1959-S156','1959-08-28 18:45:00','suborbital rocket','success',30,'AA3.122C','EUV/RPA','','','Ionos/Solar EUV',1869,'',235 from dual union all
select 59120,'1985-Y26','1985-07-03 18:21:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 59121,'1985-Y27','1985-07-05 14:22:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59122,'1985-Y28','1985-07-09 20:07:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 59123,'1985-Y29','1985-07-10 11:34:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',88 from dual union all
select 59124,'1985-Y30','1985-07-10 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 59125,'1985-Y31','1985-07-12 14:08:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 59126,'1985-Y32','1985-07-12 20:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 59127,'1985-Y33','1985-07-16 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',91 from dual union all
select 59128,'1985-Y34','1985-07-17 11:34:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 59129,'1985-Y35','1985-07-17 14:09:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 5913,'1959-S157','1959-08-31 22:53:00','suborbital rocket','success',838,'NN8.73CF','Sunflare II','','','Solar X/Solar',1015,'',200 from dual union all
select 59130,'1985-Y36','1985-07-17 15:26:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 59131,'1985-Y37','1985-07-18 11:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 59132,'1985-Y38','1985-07-19 14:10:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 59133,'1985-Y39','1985-07-22 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 59134,'1985-Y40','1985-07-23 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',90 from dual union all
select 59135,'1985-Y41','1985-07-24 11:46:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 59136,'1985-Y42','1985-07-26 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 59137,'1985-Y43','1985-07-26 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59138,'1985-Y44','1985-07-29 11:33:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual;
insert into LAUNCH
select 59139,'1985-Y45','1985-07-29 23:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',86 from dual union all
select 5914,'1959-S158','1959-09-01 01:01:00','suborbital rocket','success',838,'NN8.74CF','','','','Solar',1015,'',200 from dual union all
select 59140,'1985-Y46','1985-07-31 11:41:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 59141,'1985-Y47','1985-08-02 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 59142,'1985-Y48','1985-08-06 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',93 from dual union all
select 59143,'1985-Y49','1985-08-07 11:41:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',87 from dual union all
select 59144,'1985-Y50','1985-08-07 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 59145,'1985-Y51','1985-08-07 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 59146,'1985-Y52','1985-08-07 15:06:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 59147,'1985-Y53','1985-08-07 16:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 59148,'1985-Y54','1985-08-09 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 59149,'1985-Y55','1985-08-13 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 5915,'1959-S159','1959-09-01 19:00:00','suborbital rocket','success',838,'NN8.75CF','','','','Ionos/Plasma/Solar',1015,'',210 from dual union all
select 59150,'1985-Y56','1985-08-14 02:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 59151,'1985-Y57','1985-08-14 11:41:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',87 from dual union all
select 59152,'1985-Y58','1985-08-14 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 59153,'1985-Y59','1985-08-16 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 59154,'1985-Y60','1985-08-20 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 59155,'1985-Y61','1985-08-21 14:22:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 59156,'1985-Y62','1985-08-22 11:42:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 59157,'1985-Y63','1985-08-22 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 59158,'1985-Y64','1985-08-27 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 59159,'1985-Y65','1985-08-28 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 5916,'1959-S16','1959-02-20 05:38:00','military missile','failure',150,'5C','Mk 2-1F','R&D','','Test',227,'',100 from dual union all
select 59160,'1985-Y66','1985-08-28 14:17:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 59161,'1985-Y67','1985-08-29 11:42:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 59162,'1985-Y68','1985-09-03 13:14:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 59163,'1985-Y69','1985-09-03 13:21:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',84 from dual union all
select 59164,'1985-Y70','1985-09-04 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',912,'KRENKEL''',81 from dual union all
select 59165,'1985-Y71','1985-09-04 11:41:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 59166,'1985-Y72','1985-09-04 14:14:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 59167,'1985-Y73','1985-09-10 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 59168,'1985-Y74','1985-09-11 11:31:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 59169,'1985-Y75','1985-09-11 14:31:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 5917,'1959-S160','1959-09-02 00:03:00','suborbital rocket','success',838,'NN8.76CF','','','','Solar',1015,'',200 from dual union all
select 59170,'1985-Y76','1985-09-11 16:15:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 59171,'1985-Y77','1985-09-17 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',92 from dual union all
select 59172,'1985-Y78','1985-09-18 03:55:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 59173,'1985-Y79','1985-09-18 11:25:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 59174,'1985-Y80','1985-09-19 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 59175,'1985-Y81','1985-09-24 20:08:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 59176,'1985-Y82','1985-09-25 06:44:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 59177,'1985-Y83','1985-09-25 10:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 59178,'1985-Y84','1985-09-25 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59179,'1985-Y85','1985-10-01 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',91 from dual union all
select 5918,'1959-S161','1959-09-05 00:00:00','military missile','success',976,'IP2-4T','8K63 UBP-22','','','Test',636,'',402 from dual union all
select 59180,'1985-Y86','1985-10-02 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 59181,'1985-Y87','1985-10-02 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 59182,'1985-Y88','1985-10-03 11:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',87 from dual union all
select 59183,'1985-Y89','1985-10-03 16:15:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',83 from dual union all
select 59184,'1985-Y90','1985-10-08 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 59185,'1985-Y91','1985-10-09 03:00:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1415,'',90 from dual union all
select 59186,'1985-Y92','1985-10-09 03:25:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1415,'',90 from dual union all
select 59187,'1985-Y93','1985-10-09 03:58:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 59188,'1985-Y94','1985-10-09 11:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 59189,'1985-Y95','1985-10-09 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 5919,'1959-S162','1959-09-05 15:27:00','suborbital rocket','success',214,'NRC BB-1-01','PTV-1','','','Test',428,'',100 from dual union all
select 59190,'1985-Y96','1985-10-09 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 59191,'1985-Y97','1985-10-15 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',89 from dual union all
select 59192,'1985-Y98','1985-10-16 12:43:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 59193,'1985-Y99','1985-10-16 15:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 59194,'1986-001','1986-01-08 11:25:00','orbital','success',1238,'78056  -323','Kosmos-1715','Oblik','','Sat',475,'',null from dual union all
select 59195,'1986-002','1986-01-09 02:48:00','orbital','success',680,'47122-337','Kosmos-1716/1723','Strela-1M','','Sat',457,'',null from dual union all
select 59196,'1986-003','1986-01-12 11:55:00','orbital','success',1243,'STS-32/61-C','PAM-D Deploy/MSL-2','OV-102 PAM deploy','','Sat',921,'MLP1',null from dual union all
select 59197,'1986-004','1986-01-15 14:20:00','orbital','success',1238,'76021  -837','Kosmos-1724','Kobal''t','','Sat',473,'',null from dual union all
select 59198,'1986-005','1986-01-16 11:38:19','orbital','success',680,'53744-152','Kosmos-1725','Parus','','Sat',456,'',null from dual union all
select 59199,'1986-006','1986-01-17 07:21:09','orbital','success',1448,'51032-308','Kosmos-1726','Tselina-D','','Sat',469,'',null from dual union all
select 592,'1945-S68','1945-01-22 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5920,'1959-S163','1959-09-05 22:24:00','suborbital rocket','success',214,'NRC BB-1-02','','','','Test',428,'',90 from dual union all
select 59200,'1986-007','1986-01-17 10:20:00','orbital','success',956,'331-02','Raduga','Gran'' No. 29L','','Sat',554,'',null from dual union all
select 59201,'1986-008','1986-01-23 18:52:58','orbital','success',680,'47122-334','Kosmos-1727','Tsikada','','Sat',456,'',null from dual union all
select 59202,'1986-009','1986-01-28 08:35:00','orbital','success',1238,'76021  -330','Kosmos-1728','Oblik','','Sat',480,'',null from dual union all
select 59203,'1986-010','1986-02-01 12:37:00','orbital','success',308,'Y3       (13)','DFH-2','DFH-2','','Sat',1904,'',null from dual union all
select 59204,'1986-011','1986-02-01 18:11:56','orbital','success',803,'78056  -677','Kosmos-1729','Oko','','Sat',462,'',null from dual union all
select 59205,'1986-012','1986-02-04 11:15:00','orbital','success',1238,'76021  -324','Kosmos-1730','Oblik','','Sat',473,'',null from dual union all
select 59206,'1986-013','1986-02-07 08:45:00','orbital','success',1238,'77011  -208','Kosmos-1731','Terilen','','Sat',480,'',null from dual union all
select 59207,'1986-014','1986-02-09 10:06:00','orbital','success',170,'6004H','USA 15-18   [NOSS8]','PARCAE 8','','Sat',1775,'',null from dual union all
select 59208,'1986-015','1986-02-11 06:56:00','orbital','success',1448,'45014-709','Kosmos-1732','Musson No. 16','','Sat',470,'',null from dual union all
select 59209,'1986-016','1986-02-12 07:55:00','orbital','success',825,'N-14(F)','BS 2B','BS 2b','','Sat',1474,'',null from dual union all
select 5921,'1959-S164','1959-09-08 15:14:00','suborbital rocket','success',214,'NRC BB-1-03','','','','Test',428,'',130 from dual union all
select 59210,'1986-017','1986-02-19 21:28:23','orbital','success',949,'337-01','Mir','DOS 7 (17K No. 127-01)','','Sat',553,'',null from dual union all
select 59211,'1986-018','1986-02-19 23:04:00','orbital','success',1448,'45041-507','Kosmos-1733','Tselina-D','','Sat',469,'',null from dual union all
select 59212,'1986-019','1986-02-22 01:44:35','orbital','success',91,'V16 L11','Spot/Viking','SPOT','','Sat',344,'',null from dual union all
select 59213,'1986-020','1986-02-26 13:39:59','orbital','success',1238,'77011  -838','Kosmos-1734','Kobal''t','','Sat',475,'',null from dual union all
select 59214,'1986-021','1986-02-27 01:44:00','orbital','success',1446,'45086-101','Kosmos-1735','US-PM','','Sat',589,'',null from dual union all
select 59215,'1986-022','1986-03-13 12:33:09','orbital','success',1242,'B 15000-012','Soyuz T-15','Soyuz 7K-ST No. 21L','','Sat',480,'',null from dual union all
select 59216,'1986-023','1986-03-19 10:08:25','orbital','success',1242,'B 15000-010','Progress-25','Progress 7K-TG No. 134','','Sat',480,'',null from dual union all
select 59217,'1986-024','1986-03-21 10:05:00','orbital','success',1446,'45029-812','Kosmos-1736','US-A reactor','','Sat',589,'',null from dual union all
select 59218,'1986-025','1986-03-25 19:26:00','orbital','success',1446,'67029-811','Kosmos-1737','US-PM1','','Sat',589,'',null from dual union all
select 59219,'1986-026','1986-03-28 23:30:00','orbital','success',93,'V17 L17','Gstar 2/Brasilsat 2','Gstar 2','','Sat',345,'',null from dual union all
select 5922,'1959-S165','1959-09-09 08:19:00','test rocket','failure',154,'10D','MA Big Joe','','','SC Test',229,'',153 from dual union all
select 59220,'1986-027','1986-04-04 03:45:00','orbital','success',956,'302-01','Kosmos-1738','Geizer No. 13L','','Sat',554,'',null from dual union all
select 59221,'1986-028','1986-04-09 08:00:00','orbital','success',1238,'76021  -836','Kosmos-1739','Kobal''t','','Sat',562,'',null from dual union all
select 59222,'1986-029','1986-04-15 11:40:00','orbital','success',1238,'76021  -332','Kosmos-1740','Oblik','','Sat',462,'',null from dual union all
select 59223,'1986-030','1986-04-17 21:02:00','orbital','success',680,'53726-198','Kosmos-1741','Strela-2M','','Sat',456,'',null from dual union all
select 59224,'1986-031','1986-04-18 19:50:02','orbital','success',805,'77046  -673','Molniya-3','Molniya-3 No. 43','','Sat',473,'',null from dual union all
select 59225,'1986-032','1986-04-23 19:40:05','orbital','success',1242,'B 15000-009','Progress-26','Progress 7K-TG No. 136','','Sat',480,'',null from dual union all
select 59226,'1986-033','1986-05-14 12:40:00','orbital','success',1238,'77011  -336','Kosmos-1742','Oblik','','Sat',462,'',null from dual union all
select 59227,'1986-034','1986-05-15 04:26:00','orbital','success',1448,'45032-304','Kosmos-1743','Tselina-D','','Sat',469,'',null from dual union all
select 59228,'1986-035','1986-05-21 08:21:51','orbital','success',1242,'B 15000-013','Soyuz TM','Soyuz 7K-STM No. 51','','Sat',480,'',null from dual;
insert into LAUNCH
select 59229,'1986-036','1986-05-21 16:30:00','orbital','success',1238,'I 15000-498','Kosmos-1744','Foton No. 2L','','Sat',473,'',null from dual union all
select 5923,'1959-S166','1959-09-09 17:50:42','military missile','success',154,'12D','Mk 2','IOC DEMO','DESERT HEAT','Test',1663,'',1800 from dual union all
select 59230,'1986-037','1986-05-23 12:54:22','orbital','success',680,'53772-362','Kosmos-1745','Parus','','Sat',456,'',null from dual union all
select 59231,'1986-038','1986-05-24 01:42:00','orbital','success',956,'333-01','Ekran','Ekran No. 30L','','Sat',553,'',null from dual union all
select 59232,'1986-039','1986-05-27 09:30:00','orbital','success',1448,'67076-109','Meteor-2','Meteor-2 No. 18','','Sat',469,'',null from dual union all
select 59233,'1986-040','1986-05-28 07:50:01','orbital','success',1238,'76048  -158','Kosmos-1746','Resurs-F1 17F41 No. 58','','Sat',475,'',null from dual union all
select 59234,'1986-041','1986-05-29 09:20:00','orbital','success',1238,'78056  -915','Kosmos-1747','Oblik','','Sat',562,'',null from dual union all
select 59235,'1986-042','1986-06-06 03:57:00','orbital','success',680,'47172-364','Kosmos-1748/1755','Strela-1M','','Sat',456,'',null from dual union all
select 59236,'1986-043','1986-06-06 12:40:00','orbital','success',1238,'77011  -335','Kosmos-1756','Kobal''t','','Sat',562,'',null from dual union all
select 59237,'1986-044','1986-06-10 00:49:00','orbital','success',956,'322-02','Gorizont','Gorizont No. 24L','','Sat',554,'',null from dual union all
select 59238,'1986-045','1986-06-11 07:44:59','orbital','success',1238,'76021  -326','Kosmos-1757','Oblik (Priroda)','','Sat',475,'',null from dual union all
select 59239,'1986-046','1986-06-12 04:43:00','orbital','success',1448,'51086-102','Kosmos-1758','Tselina-D','','Sat',469,'',null from dual union all
select 5924,'1959-S167','1959-09-10 21:48:00','suborbital rocket','success',214,'NRC BB-1-04','','','','Test',428,'',127 from dual union all
select 59240,'1986-047','1986-06-18 20:03:49','orbital','success',680,'53722-341','Kosmos-1759','Parus','','Sat',456,'',null from dual union all
select 59241,'1986-048','1986-06-19 10:30:00','orbital','success',1238,'76048  -348','Kosmos-1760','Oblik','','Sat',480,'',null from dual union all
select 59242,'1986-049','1986-06-19 21:09:00','orbital','success',805,'76054  -653','Molniya-3','Molniya-3 No. 44','','Sat',473,'',null from dual union all
select 59243,'1986-050','1986-07-05 01:16:46','orbital','success',803,'76021  -683','Kosmos-1761','Oko','','Sat',475,'',null from dual union all
select 59244,'1986-051','1986-07-10 08:00:00','orbital','success',1238,'76048  -159','Kosmos-1762','Resurs-F1 14F40 No. 59','','Sat',462,'',null from dual union all
select 59245,'1986-052','1986-07-16 04:21:00','orbital','success',680,'53722-365','Kosmos-1763','Strela-2M','','Sat',460,'',null from dual union all
select 59246,'1986-053','1986-07-17 12:30:00','orbital','success',1238,'77011  -916','Kosmos-1764','Kobal''t','','Sat',562,'',null from dual union all
select 59247,'1986-054','1986-07-24 12:30:01','orbital','success',1238,'77011  -337','Kosmos-1765','Oblik','','Sat',462,'',null from dual union all
select 59248,'1986-055','1986-07-28 21:07:59','orbital','success',1448,'51061-502','Kosmos-1766','Okean-O1 NKhM No. 3','','Sat',470,'',null from dual union all
select 59249,'1986-056','1986-07-30 08:30:00','orbital','success',1537,'45061-501','Kosmos-1767','EPN No. 03.695','','Sat',567,'',null from dual union all
select 5925,'1959-S168','1959-09-11 19:12:00','suborbital rocket','success',30,'NASA 04.08GI','','','','Ionos',428,'',227 from dual union all
select 59250,'1986-057','1986-07-30 15:06:00','orbital','success',805,'76021  -688','Molniya-1','Molniya-1!No. 59','','Sat',475,'',null from dual union all
select 59251,'1986-058','1986-08-02 09:20:00','orbital','success',1238,'78028  -160','Kosmos-1768','Resurs-F1 14F40 No. 60','','Sat',462,'',null from dual union all
select 59252,'1986-059','1986-08-04 05:08:00','orbital','success',1446,'67099-805','Kosmos-1769','US-PM1','','Sat',589,'',null from dual union all
select 59253,'1986-060','1986-08-06 13:30:00','orbital','success',1238,'78056  -445','Kosmos-1770','Terilen','','Sat',480,'',null from dual union all
select 59254,'1986-061','1986-08-12 20:45:00','orbital','success',522,'H-15(F)','EGP','EGP','','Sat',1474,'',null from dual union all
select 59255,'1986-062','1986-08-20 12:58:00','orbital','success',1446,'67099-808','Kosmos-1771','US-A reactor','','Sat',589,'',null from dual union all
select 59256,'1986-063','1986-08-21 11:05:00','orbital','success',1238,'76021  -329','Kosmos-1772','Oblik','','Sat',475,'',null from dual union all
select 59257,'1986-064','1986-08-27 11:40:00','orbital','success',1238,'77011  -917','Kosmos-1773','Kobal''t','','Sat',562,'',null from dual union all
select 59258,'1986-065','1986-08-28 08:02:43','orbital','success',803,'76021  -687','Kosmos-1774','Oko','','Sat',462,'',null from dual union all
select 59259,'1986-066','1986-09-03 07:59:59','orbital','success',1238,'76048  -351','Kosmos-1775','Oblik','','Sat',562,'',null from dual union all
select 5926,'1959-S169','1959-09-12 00:00:00','military missile','success',1367,'217','R&D Series IV','','','Test',236,'',520 from dual union all
select 59260,'1986-067','1986-09-03 09:00:00','orbital','success',680,'65022-342','Kosmos-1776','Romb','','Sat',456,'',null from dual union all
select 59261,'1986-068','1986-09-05 09:12:59','orbital','success',805,'76048  -505','Molniya-1','Molniya-1!No. 57','','Sat',475,'',null from dual union all
select 59262,'1986-069','1986-09-05 15:08:01','orbital','success',391,'D180','USA 19 (SDIO D180)','Payload Adapter System','','Sat',234,'',null from dual union all
select 59263,'1986-070','1986-09-10 01:45:00','orbital','success',680,'47196-164','Kosmos-1777','Strela-2M','','Sat',460,'',null from dual union all
select 59264,'1986-071','1986-09-16 11:38:09','orbital','success',958,'336-01','Kosmos-1778/1780','Uragan No. 24L','','Sat',554,'',null from dual union all
select 59265,'1986-072','1986-09-17 07:59:00','orbital','success',1238,'76048  -349','Kosmos-1781','Oblik','','Sat',562,'',null from dual union all
select 59266,'1986-073','1986-09-17 15:52:00','orbital','success',155,'52E','NOAA 10','NOAA G','','Sat',1776,'',null from dual union all
select 59267,'1986-074','1986-09-30 18:34:00','orbital','success',1448,'67032-309','Kosmos-1782','Tselina-D','','Sat',469,'',null from dual union all
select 59268,'1986-075','1986-10-03 13:05:41','orbital','failure',803,'77011  -694','Kosmos-1783','Oko','','Sat',473,'',null from dual union all
select 59269,'1986-076','1986-10-06 05:40:00','orbital','success',301,'Y5       (14)','FSW','FSW No. 9','','Sat',810,'',null from dual union all
select 5927,'1959-S17','1959-02-25 19:45:00','military missile','success',1419,'A-5','P200 RV','Dummy St 2','','Test',230,'',200 from dual union all
select 59270,'1986-077','1986-10-06 07:40:00','orbital','success',1238,'77011  -447','Kosmos-1784','Siluet No. 6','','Sat',480,'',null from dual union all
select 59271,'1986-078','1986-10-15 09:29:18','orbital','success',803,'76021  -684','Kosmos-1785','Oko','','Sat',473,'',null from dual union all
select 59272,'1986-079','1986-10-20 08:49:00','orbital','success',805,'77049  -645','Molniya-3','Molniya-3 No. 41','','Sat',475,'',null from dual union all
select 59273,'1986-080','1986-10-22 08:00:00','orbital','success',1537,'67041-503','Kosmos-1786','Taifun-1B','','Sat',567,'',null from dual union all
select 59274,'1986-081','1986-10-22 09:00:00','orbital','success',1238,'76048  -353','Kosmos-1787','Oblik','','Sat',480,'',null from dual union all
select 59275,'1986-082','1986-10-25 15:43:00','orbital','success',956,'335-02','Raduga','Gran'' No. 30L','','Sat',554,'',null from dual union all
select 59276,'1986-083','1986-10-27 12:40:00','orbital','success',680,'53748-411','Kosmos-1788','Vektor','','Sat',460,'',null from dual union all
select 59277,'1986-084','1986-10-31 08:00:00','orbital','success',1238,'78028  -161','Kosmos-1789','Resurs-F1 14F40 No. 61','','Sat',462,'',null from dual union all
select 59278,'1986-085','1986-11-04 11:50:00','orbital','success',1238,'76048  -352','Kosmos-1790','Oblik','','Sat',462,'',null from dual union all
select 59279,'1986-086','1986-11-13 06:10:25','orbital','success',680,'53748-414','Kosmos-1791','Tsikada','','Sat',457,'',null from dual union all
select 5928,'1959-S170','1959-09-14 17:27:00','suborbital rocket','success',30,'NASA 04.07GI','','','','Ionos',428,'',218 from dual union all
select 59280,'1986-087','1986-11-13 10:59:59','orbital','success',1238,'77011  -333','Kosmos-1792','Kobal''t','','Sat',562,'',null from dual union all
select 59281,'1986-088','1986-11-14 00:23:00','orbital','success',1135,'S199C','P87-1','NNS O-17','','Sat',1779,'',null from dual union all
select 59282,'1986-089','1986-11-15 21:35:00','orbital','success',805,'76048  -508','Molniya-1','Molniya-1!No. 60','','Sat',473,'',null from dual union all
select 59283,'1986-090','1986-11-18 14:08:03','orbital','success',956,'334-01','Gorizont','Gorizont No. 22L','','Sat',553,'',null from dual union all
select 59284,'1986-091','1986-11-20 12:09:20','orbital','success',803,'77011  -689','Kosmos-1793','Oko','','Sat',462,'',null from dual union all
select 59285,'1986-092','1986-11-21 02:00:00','orbital','success',680,'53796-168','Kosmos-1794/1801','Strela-1M','','Sat',457,'',null from dual union all
select 59286,'1986-093','1986-11-24 21:43:13','orbital','success',680,'47122-349','Kosmos-1802','Parus','','Sat',457,'',null from dual union all
select 59287,'1986-094','1986-12-02 07:00:00','orbital','success',1448,'45086-104','Kosmos-1803','Musson No. 18','','Sat',469,'',null from dual union all
select 59288,'1986-095','1986-12-04 10:10:00','orbital','success',1238,'76048  -851','Kosmos-1804','Oblik','','Sat',480,'',null from dual union all
select 59289,'1986-096','1986-12-05 02:30:01','orbital','success',169,'AC-66','USA 20 (FLTSATCOM 7)','FLTSATCOM F7','','Sat',269,'',null from dual union all
select 5929,'1959-S171','1959-09-15 00:00:00','suborbital rocket','success',28,'','','','','Test',1869,'',200 from dual union all
select 59290,'1986-097','1986-12-10 07:30:00','orbital','success',1448,'67086-103','Kosmos-1805','Tselina-R','','Sat',470,'',null from dual union all
select 59291,'1986-098','1986-12-12 18:35:36','orbital','success',803,'76048  -509','Kosmos-1806','Oko','','Sat',475,'',null from dual union all
select 59292,'1986-099','1986-12-16 14:00:00','orbital','success',1238,'78056  -318','Kosmos-1807','Kobal''t','','Sat',473,'',null from dual union all
select 59293,'1986-100','1986-12-17 17:02:32','orbital','success',680,'53772-353','Kosmos-1808','Parus','','Sat',456,'',null from dual union all
select 59294,'1986-101','1986-12-18 08:00:00','orbital','success',1448,'45061-501','Kosmos-1809','AUOS-Z 501 Ionozond-E','','Sat',470,'',null from dual union all
select 59295,'1986-102','1986-12-26 11:00:00','orbital','success',1238,'78056  -206','Kosmos-1810','Terilen','','Sat',480,'',null from dual union all
select 59296,'1986-103','1986-12-26 15:25:59','orbital','success',805,'78028  -512','Molniya-1','Molniya-1!No. 62','','Sat',474,'',null from dual union all
select 59297,'1986-A01','1986-01-01 09:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',44 from dual union all
select 59298,'1986-A02','1986-01-08 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',40 from dual union all
select 59299,'1986-A03','1986-01-22 15:09:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',47 from dual union all
select 593,'1945-S69','1945-02-13 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5930,'1959-S172','1959-09-15 00:00:00','military missile','success',976,'I1-4T','8K63 UBP-23','','','Test',636,'',402 from dual union all
select 59300,'1986-A04','1986-03-07 21:43:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',49 from dual union all
select 59301,'1986-A05','1986-03-19 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',34 from dual union all
select 59302,'1986-A06','1986-03-23 07:50:00','military missile','failure',1005,'51041-502(1L)','15A18M LKI-1','GCh 15F173 (15F174 x9)','','OT',483,'',0 from dual union all
select 59303,'1986-A07','1986-03-26 18:53:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',49 from dual union all
select 59304,'1986-A08','1986-08-21 03:40:00','military missile','failure',1005,'67041-503(2L)','15A18M LKI-2','GCh 15F173','','OT',487,'',70 from dual union all
select 59305,'1986-A09','1986-08-24 04:40:00','suborbital rocket','failure',117,'NASA 24.09UH','','','','XR Astron',1873,'',23 from dual union all
select 59306,'1986-A10','1986-10-22 13:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',48 from dual union all
select 59307,'1986-A11','1986-11-19 09:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',47 from dual union all
select 59308,'1986-A12','1986-11-27 02:50:00','military missile','failure',1005,'51038-401(3L)','15A18M LKI-3','GCh 15F173','','OT',487,'',100 from dual union all
select 59309,'1986-A13','1986-04-01 00:00:00','atmospheric rocket','success',542,'Sandia','Stores parachute 10','','','Test',1490,'',12 from dual union all
select 5931,'1959-S173','1959-09-15 21:45:00','ballistic missile test','failure',638,'AM-23','Bioflight 3','','','Bio',249,'',500 from dual union all
select 59310,'1986-A14','1986-08-01 00:00:00','military missile','success',1430,'','','','','Test',636,'',26 from dual union all
select 59311,'1986-A15','1986-09-01 00:00:00','military missile','success',702,'PAC-1 Target','','','','Test',1895,'',5 from dual union all
select 59312,'1986-F01','1986-01-28 16:38:00','orbital','failure',1243,'STS-33/51-L','TDRS-B/Spartan','OV-099 IUS deploy','','Sat',922,'MLP2',15 from dual union all
select 59313,'1986-F02','1986-03-26 10:30:00','orbital','failure',1238,'76021  -328','','Oblik??','','Sat',562,'',50 from dual union all
select 59314,'1986-F03','1986-04-18 18:45:01','orbital','failure',1406,'34D-9  (04D-2)','HEXAGON SV20','HEXAGON SV-20','','Sat',1777,'',0 from dual union all
select 59315,'1986-F04','1986-05-03 22:18:00','orbital','failure',390,'D178','GOES G','GOES G','','Sat',233,'',30 from dual union all
select 59316,'1986-F05','1986-05-31 00:53:03','orbital','failure',92,'V18 L18','Intelsat 514','INTELSAT VA F14','','Sat',344,'',250 from dual union all
select 59317,'1986-F06','1986-10-15 05:24:08','orbital','failure',1448,'67061-503','','Strela-3','','Sat',470,'',20 from dual union all
select 59318,'1986-F07','1986-11-29 08:00:00','orbital','failure',949,'338-01','','Mech-K No. 303','','Sat',554,'',50 from dual;
insert into LAUNCH
select 59319,'1986-M01','1986-01-15 00:00:00','suborbital rocket','success',1023,'Eq. Wave Campaign II','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 5932,'1959-S174','1959-09-17 00:12:00','suborbital rocket','success',1194,'SL12','LUX-P2/Bir-1/UCL SUV1','','','Ionos/Solar X/Solar UV',1839,'',132 from dual union all
select 59320,'1986-M02','1986-01-16 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59321,'1986-M03','1986-01-17 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59322,'1986-M04','1986-01-18 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59323,'1986-M05','1986-01-19 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59324,'1986-M06','1986-01-20 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59325,'1986-M07','1986-01-21 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59326,'1986-M08','1986-01-22 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59327,'1986-M09','1986-01-23 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59328,'1986-M10','1986-01-24 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59329,'1986-M100','1986-01-01 00:00:00','military missile','success',592,'','','','','Test',33,'',50 from dual union all
select 5933,'1959-S175','1959-09-17 02:09:00','military missile','failure',154,'17D','Mk 2-2A','R&D','','Test',228,'',1000 from dual union all
select 59330,'1986-M11','1986-01-25 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59331,'1986-M12','1986-01-26 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59332,'1986-M13','1986-01-27 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59333,'1986-M14','1986-01-28 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59334,'1986-M15','1986-01-29 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59335,'1986-M16','1986-01-30 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59336,'1986-M17','1986-01-31 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59337,'1986-M18','1986-02-01 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59338,'1986-M19','1986-02-02 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59339,'1986-M20','1986-02-03 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 5934,'1959-S176','1959-09-17 18:37:00','suborbital rocket','success',30,'NASA 04.02II','DRTE.01','','','Ionos',428,'',256 from dual union all
select 59340,'1986-M21','1986-02-04 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59341,'1986-M22','1986-02-05 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59342,'1986-M23','1986-02-06 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59343,'1986-M24','1986-02-07 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59344,'1986-M25','1986-02-08 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59345,'1986-M26','1986-02-09 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59346,'1986-M27','1986-02-10 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59347,'1986-M28','1986-02-11 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59348,'1986-M29','1986-02-12 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59349,'1986-M30','1986-02-13 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 5935,'1959-S177','1959-09-17 21:09:00','military missile','success',1367,'228','IWST','','GREASE GUN','Test',1674,'',520 from dual union all
select 59350,'1986-M31','1986-02-14 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59351,'1986-M32','1986-02-15 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59352,'1986-M33','1986-02-16 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59353,'1986-M34','1986-02-17 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59354,'1986-M35','1986-02-18 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59355,'1986-M36','1986-02-19 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59356,'1986-M37','1986-02-20 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59357,'1986-M38','1986-02-21 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59358,'1986-M39','1986-02-22 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59359,'1986-M40','1986-02-23 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 5936,'1959-S178','1959-09-18 00:00:00','military missile','success',1053,'CC-1018','Range 134 km','','','OT',1854,'',90 from dual union all
select 59360,'1986-M41','1986-02-24 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59361,'1986-M42','1986-02-25 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59362,'1986-M43','1986-02-26 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59363,'1986-M44','1986-02-27 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59364,'1986-M45','1986-02-28 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1361,'',70 from dual union all
select 59365,'1986-M46','1986-01-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59366,'1986-M47','1986-01-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59367,'1986-M48','1986-01-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59368,'1986-M49','1986-01-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59369,'1986-M50','1986-01-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5937,'1959-S179','1959-09-18 16:02:00','military missile','success',1016,'I1-1T','8K71 LKI III-11','','','Test',480,'',1350 from dual union all
select 59370,'1986-M51','1986-02-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59371,'1986-M52','1986-02-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59372,'1986-M53','1986-02-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59373,'1986-M54','1986-05-07 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59374,'1986-M55','1986-05-14 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59375,'1986-M56','1986-06-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59376,'1986-M57','1986-06-11 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59377,'1986-M58','1986-06-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59378,'1986-M59','1986-06-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59379,'1986-M60','1986-07-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5938,'1959-S18','1959-02-27 15:44:00','suborbital rocket','success',28,'AA3.120C','Expt 6.5','','','Solar EUV',640,'',210 from dual union all
select 59380,'1986-M61','1986-07-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59381,'1986-M62','1986-07-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59382,'1986-M63','1986-07-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59383,'1986-M64','1986-07-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59384,'1986-M65','1986-08-06 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59385,'1986-M66','1986-08-13 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59386,'1986-M67','1986-08-20 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59387,'1986-M68','1986-09-04 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59388,'1986-M69','1986-09-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59389,'1986-M70','1986-09-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5939,'1959-S180','1959-09-18 16:55:00','suborbital rocket','success',839,'AA6.300C','','','','Aeron',394,'',100 from dual union all
select 59390,'1986-M71','1986-09-25 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59391,'1986-M72','1986-10-01 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59392,'1986-M73','1986-10-08 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59393,'1986-M74','1986-10-15 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59394,'1986-M75','1986-10-22 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59395,'1986-M76','1986-10-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59396,'1986-M77','1986-10-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59397,'1986-M78','1986-11-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59398,'1986-M79','1986-11-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59399,'1986-M80','1986-11-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 594,'1945-S70','1945-02-14 19:53:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5940,'1959-S181','1959-09-20 17:35:00','suborbital rocket','failure',30,'NASA 04.03II','DRTE.02','','','Ionos',428,'',100 from dual union all
select 59400,'1986-M81','1986-11-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59401,'1986-M82','1986-12-03 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59402,'1986-M83','1986-12-10 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59403,'1986-M84','1986-12-18 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59404,'1986-M85','1986-12-24 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59405,'1986-M86','1986-12-31 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59406,'1986-M87','1986-02-27 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59407,'1986-M88','1986-03-05 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59408,'1986-M89','1986-03-12 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual;
insert into LAUNCH
select 59409,'1986-M90','1986-03-19 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 5941,'1959-S182','1959-09-21 18:29:00','military missile','success',928,'A1X-1','','','','Test',251,'',500 from dual union all
select 59410,'1986-M91','1986-03-26 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59411,'1986-M92','1986-04-02 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59412,'1986-M93','1986-04-09 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59413,'1986-M94','1986-04-16 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59414,'1986-M95','1986-04-23 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59415,'1986-M96','1986-04-30 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59416,'1986-M97','1986-05-21 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59417,'1986-M98','1986-05-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59418,'1986-M99','1986-08-28 00:00:00','suborbital rocket','success',1023,'','','','','Meteo',320,'',60 from dual union all
select 59419,'1986-S01','1986-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 5942,'1959-S183','1959-09-22 00:00:00','suborbital rocket','success',29,'AA1.260C','','','','Ionos?',1869,'',110 from dual union all
select 59420,'1986-S02','1986-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 59421,'1986-S03','1986-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 59422,'1986-S04','1986-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 59423,'1986-S05','1986-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 59424,'1986-S06','1986-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 59425,'1986-S07','1986-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 59426,'1986-S08','1986-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 59427,'1986-S09','1986-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 59428,'1986-S10','1986-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 59429,'1986-S100','1986-09-30 10:45:00','test rocket','success',20,'','','','','Interceptor',383,'F-15',1000 from dual union all
select 5943,'1959-S184','1959-09-22 18:00:00','military missile','success',1367,'222','R&D Series IV','','','Ionos/Meteorites',234,'',520 from dual union all
select 59430,'1986-S101','1986-09-30 15:20:00','military missile','success',1442,'','DASO','','','Test',421,'SSBN 733',1000 from dual union all
select 59431,'1986-S102','1986-10-15 17:10:00','suborbital rocket','success',211,'NASA 27.75GS','SERTS-2','','','Solar EUV',1895,'',304 from dual union all
select 59432,'1986-S103','1986-10-22 20:00:00','suborbital rocket','success',211,'NASA 27.99US','NIXT','','','Solar',1895,'',270 from dual union all
select 59433,'1986-S104','1986-11-01 01:59:00','suborbital rocket','success',1219,'XV-21','INPE-003/86','','','Aeron',330,'',444 from dual union all
select 59434,'1986-S105','1986-11-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',143,'ZUB',250 from dual union all
select 59435,'1986-S106','1986-11-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',143,'ZUB',250 from dual union all
select 59436,'1986-S107','1986-11-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',143,'ZUB',250 from dual union all
select 59437,'1986-S108','1986-11-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',143,'ZUB',250 from dual union all
select 59438,'1986-S109','1986-11-04 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 644',500 from dual union all
select 59439,'1986-S11','1986-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 5944,'1959-S185','1959-09-24 00:29:00','suborbital rocket','success',1194,'SL15','Bir-2/UCL Ion-2','','','Ionos',1839,'',158 from dual union all
select 59440,'1986-S110','1986-11-04 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 644',500 from dual union all
select 59441,'1986-S111','1986-11-04 20:59:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 644',500 from dual union all
select 59442,'1986-S112','1986-11-04 20:44:01','suborbital rocket','success',1319,'SSC S41H','Aureld-VIP-HIGH','','','Auroral',410,'',206 from dual union all
select 59443,'1986-S113','1986-11-04 20:59:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 644',500 from dual union all
select 59444,'1986-S114','1986-11-20 10:15:00','suborbital rocket','success',891,'NASA 30.33UE','Turbulence','','','Plasma',1821,'',87 from dual union all
select 59445,'1986-S115','1986-12-05 16:00:00','military missile','success',907,'FTM-14','Mk 21 RV x 10 (2 ft dep)','','','Test',1697,'',1000 from dual union all
select 59446,'1986-S116','1986-12-12 20:21:36','suborbital rocket','success',1319,'SSC S41L','Aureld-VIP-LOW','','','Auroral',410,'',185 from dual union all
select 59447,'1986-S117','1986-12-15 19:35:00','suborbital rocket','success',224,'NASA 36.19GU','GSFC Solar UVI 3','','','Solar UV',1873,'',322 from dual union all
select 59448,'1986-S118','1986-12-25 01:00:00','test rocket','success',647,'','BOR-5 No. 503','','','Reentry Test',621,'',210 from dual union all
select 59449,'1986-S119','1986-03-10 04:55:00','suborbital rocket','success',1025,'','D-REG IONIZ. COMP.','','','Aeron',1467,'',110 from dual union all
select 5945,'1959-S186','1959-09-25 00:00:00','military missile','success',976,'BD2-7','8K63 UBP-24','','','Test',636,'',402 from dual union all
select 59450,'1986-S12','1986-02-05 13:00:00','military missile','success',1006,'45087-524-18','15A18 UBP-20 K','GCh 15F183','','OT',485,'',1000 from dual union all
select 59451,'1986-S120','1986-11-06 03:10:00','suborbital rocket','success',1025,'','AEROSOL PRECAMP. II','','','Aeron',1467,'',92 from dual union all
select 59452,'1986-S121','1986-10-03 05:05:00','suborbital rocket','success',1025,'','AEROSOL PRECAMP. II','','','Aeron',1467,'',88 from dual union all
select 59453,'1986-S122','1986-03-07 05:00:00','suborbital rocket','success',287,'','D-REG IONIZ. COMP.','','','Aeron',1467,'',140 from dual union all
select 59454,'1986-S123','1986-04-15 00:00:00','military missile','success',983,'','Lampedusa','','','Weapon',1487,'',100 from dual union all
select 59455,'1986-S124','1986-04-15 00:00:00','military missile','success',983,'','Lampedusa','','','Weapon',1487,'',100 from dual union all
select 59456,'1986-S125','1986-01-01 00:00:00','military missile','success',358,'','R&D range test','','','Test',810,'',500 from dual union all
select 59457,'1986-S126','1986-10-01 00:00:00','military missile','success',359,'','Op test','','','Test',810,'',500 from dual union all
select 59458,'1986-S127','1986-11-10 00:00:00','military missile','unknown',917,'AO-229','P-485 86 Fall 01','','','OT',433,'',180 from dual union all
select 59459,'1986-S128','1986-11-11 01:00:00','military missile','unknown',917,'AO-230','P-486 86 Fall 02','','','OT',433,'',180 from dual union all
select 5946,'1959-S187','1959-09-28 00:00:00','military missile','success',976,'ID7-20T','8K63 UBP-25','','','Test',636,'',402 from dual union all
select 59460,'1986-S129','1986-11-17 00:00:00','military missile','unknown',917,'AO-231','P-487 86 Fall 03','','','OT',433,'',180 from dual union all
select 59461,'1986-S13','1986-04-13 00:00:00','military missile','success',1453,'','15A35 UBP-31','','','OT',535,'',1000 from dual union all
select 59462,'1986-S130','1986-11-17 00:00:00','military missile','unknown',917,'AO-232','P-488 86 Fall 04','','','OT',433,'',180 from dual union all
select 59463,'1986-S131','1986-02-14 00:00:00','military missile','success',918,'','P-477 86 Spring 01','','','OT',433,'',320 from dual union all
select 59464,'1986-S132','1986-02-14 00:00:00','military missile','success',918,'','P-478 86 Spring 01','','','OT',433,'',320 from dual union all
select 59465,'1986-S133','1986-02-14 00:00:00','military missile','success',918,'','P-479 86 Spring 01','','','OT',433,'',320 from dual union all
select 59466,'1986-S134','1986-01-01 00:00:00','military missile','success',644,'','BAMBUK-M','','','Reentry Test',636,'',null from dual union all
select 59467,'1986-S135','1986-01-01 00:00:00','military missile','success',984,'Aerofon','UGCh','ZLKI 8','','Test',636,'',86 from dual union all
select 59468,'1986-S136','1986-03-01 00:00:00','military missile','success',984,'Aerofon','UGCh','LKI III-1','','Test',636,'',86 from dual union all
select 59469,'1986-S137','1986-03-10 00:00:00','suborbital rocket','success',728,'','D-REG IONIZ. COMP.','','','Aeron',1467,'',105 from dual union all
select 5947,'1959-S188','1959-09-29 10:59:00','suborbital rocket','success',839,'AA6.203C','','','','Aeron',394,'',124 from dual union all
select 59470,'1986-S138','1986-10-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 59471,'1986-S139','1986-10-08 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 59472,'1986-S14','1986-05-22 00:00:00','military missile','success',1455,'','15A35S UBP-1 ZP','GCh 15F357','','OT',508,'',1000 from dual union all
select 59473,'1986-S140','1986-10-15 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 59474,'1986-S141','1986-10-16 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 59475,'1986-S142','1986-10-22 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 59476,'1986-S143','1986-10-24 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 59477,'1986-S144','1986-11-05 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 59478,'1986-S145','1986-11-12 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 59479,'1986-S146','1986-11-20 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 5948,'1959-S189','1959-10-01 00:00:00','suborbital rocket','success',814,'','','','','Test',313,'',90 from dual union all
select 59480,'1986-S147','1986-10-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59481,'1986-S148','1986-10-08 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59482,'1986-S149','1986-10-15 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59483,'1986-S15','1986-08-05 00:00:00','military missile','success',1453,'','15A35 UBP-32','GCh 15F355P7','','OT',535,'',1000 from dual union all
select 59484,'1986-S150','1986-10-22 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59485,'1986-S151','1986-10-29 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59486,'1986-S152','1986-11-03 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59487,'1986-S153','1986-11-05 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59488,'1986-S154','1986-11-12 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59489,'1986-S155','1986-11-17 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 5949,'1959-S19','1959-02-27 23:50:00','military missile','success',638,'CM-22','R&D','','','Test',249,'',500 from dual union all
select 59490,'1986-S156','1986-11-19 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59491,'1986-S157','1986-10-07 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 59492,'1986-S158','1986-10-14 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 59493,'1986-S159','1986-10-21 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 59494,'1986-S16','1986-09-17 00:00:00','military missile','success',1453,'','15A35 UBP-33','GCh 15F355P7','','OT',596,'',1000 from dual union all
select 59495,'1986-S160','1986-10-21 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 59496,'1986-S161','1986-10-28 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 59497,'1986-S162','1986-11-04 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 59498,'1986-S163','1986-11-11 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 59499,'1986-S164','1986-11-18 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual;
insert into LAUNCH
select 595,'1945-S71','1945-02-27 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 5950,'1959-S190','1959-10-01 00:00:00','suborbital rocket','success',814,'','','','','Test',313,'',90 from dual union all
select 59500,'1986-S165','1986-12-09 00:00:00','military missile','success',1453,'','15A35 UBP-34','GCh 15F355P7','','OT',596,'',1000 from dual union all
select 59501,'1986-S166','1986-01-24 00:00:00','military missile','success',1037,'64647-221-35','8K98 UBP-73','','','Test',477,'',1000 from dual union all
select 59502,'1986-S167','1986-09-10 00:00:00','military missile','success',1044,'','8K98P UBP-25','','','Test',477,'',1000 from dual union all
select 59503,'1986-S168','1986-04-18 00:00:00','military missile','success',1434,'','15Zh58 K-10','','','Test',477,'',1000 from dual union all
select 59504,'1986-S169','1986-09-20 00:00:00','military missile','success',1434,'','15Zh58 K-11','','','Test',477,'',1000 from dual union all
select 59505,'1986-S17','1986-01-18 18:13:00','suborbital rocket','success',231,'NASA 35.010CE','CENTAUR 2','','','Auroral',29,'',676 from dual union all
select 59506,'1986-S170','1986-12-25 00:00:00','military missile','success',1434,'','15Zh58 K-13','','','Test',477,'',1000 from dual union all
select 59507,'1986-S171','1986-11-29 00:00:00','military missile','success',1434,'','15Zh58 K-12','','','Test',477,'',1000 from dual union all
select 59508,'1986-S172','1986-03-13 00:00:00','military missile','success',1042,'48751-207-9L','15Zh61 LKI-8','','','Test',477,'',1000 from dual union all
select 59509,'1986-S173','1986-07-08 00:00:00','military missile','success',1042,'48751-208-10L','15Zh61 LKI-9','','','Test',477,'',1000 from dual union all
select 5951,'1959-S191','1959-09-30 00:00:00','suborbital rocket','success',1328,'Tarp','UV Scanner','','','Test',1023,'',100 from dual union all
select 59510,'1986-S174','1986-08-12 00:00:00','military missile','success',1042,'48788-213-02','15Zh61 LKI-10','','','Test',477,'',1000 from dual union all
select 59511,'1986-S175','1986-09-02 00:00:00','military missile','success',1042,'48788-213-01','15Zh61 LKI-11','','','Test',477,'',1000 from dual union all
select 59512,'1986-S176','1986-11-12 00:00:00','military missile','success',1042,'48788-213-03','15Zh61 LKI-12','','','Test',477,'',1000 from dual union all
select 59513,'1986-S177','1986-12-11 00:00:00','military missile','success',1042,'48798-213-07','15Zh61 LKI-13','','','Test',477,'',1000 from dual union all
select 59514,'1986-S178','1986-09-04 00:00:00','military missile','success',1041,'48788-303-2L','15Zh60 LKI-2','','','Test',477,'',1000 from dual union all
select 59515,'1986-S179','1986-12-24 00:00:00','military missile','success',1041,'48798-304-3L','15Zh60 LKI-3','','','Test',477,'',1000 from dual union all
select 59516,'1986-S18','1986-01-31 08:00:00','suborbital rocket','success',668,'K-9M-79','','','','Ionos',839,'',319 from dual union all
select 59517,'1986-S183','1986-06-06 00:00:00','military missile','success',1450,'','15A20 52 RD UBP','','','OT',195,'',1000 from dual union all
select 59518,'1986-S184','1986-09-14 00:00:00','military missile','success',1006,'','15A18 62 RD UBP-2','','','OT',1646,'',1000 from dual union all
select 59519,'1986-S185','1986-04-11 00:00:00','military missile','success',1450,'','15A20 36 RD UBP-4','','','OT',898,'',1000 from dual union all
select 5952,'1959-S192','1959-09-30 10:57:00','suborbital rocket','success',839,'AA6.204C','','','','Aeron',394,'',122 from dual union all
select 59520,'1986-S186','1986-01-01 00:00:00','military missile','success',1006,'','15A18 UBP 38 RD','','','OT',363,'',1000 from dual union all
select 59521,'1986-S187','1986-09-16 00:00:00','military missile','success',1044,'','8K98P UBP-26','','','Test',477,'',1000 from dual union all
select 59522,'1986-S188','1986-09-28 00:00:00','military missile','success',1037,'Yu64612-21','8K98 UBP-74','','','Test',477,'',1000 from dual union all
select 59523,'1986-S189','1986-01-01 00:00:00','military missile','success',922,'','15Zh45 33 RD UBP','','','Test',636,'',500 from dual union all
select 59524,'1986-S19','1986-02-01 05:40:00','suborbital rocket','success',211,'NASA 27.103UH','','','','XR Astron',1895,'',228 from dual union all
select 59525,'1986-S190','1986-01-01 00:00:00','military missile','success',922,'','15Zh45 33 RD UBP','','','Test',636,'',500 from dual union all
select 59526,'1986-S191','1986-01-29 00:00:00','military missile','success',922,'','15Zh45 32 RD UBP','','','Test',636,'',500 from dual union all
select 59527,'1986-S192','1986-01-01 00:00:00','military missile','success',923,'','15Zh53 37 RD UBP','','','Test',636,'',500 from dual union all
select 59528,'1986-S193','1986-04-03 00:00:00','military missile','success',922,'','15Zh56 32 RD UBP','Gorn','','Test/Comms',636,'',500 from dual union all
select 59529,'1986-S194','1986-05-01 00:00:00','military missile','success',579,'','','','','Test',1480,'',86 from dual union all
select 5953,'1959-S193','1959-10-01 01:28:00','military missile','success',638,'AM-24','R&D','','','Test',280,'',500 from dual union all
select 59530,'1986-S20','1986-02-01 12:40:00','suborbital rocket','success',1073,'S-310-16','E-region','','','Ionos',833,'',201 from dual union all
select 59531,'1986-S21','1986-02-10 00:00:00','military missile','success',1106,'2607','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 59532,'1986-S22','1986-02-14 00:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT112GB','GLORY TRIP 112GB','OT',1712,'',1300 from dual union all
select 59533,'1986-S23','1986-02-21 17:15:00','military missile','success',1442,'','DASO','','','OT',421,'SSBN 732',1000 from dual union all
select 59534,'1986-S24','1986-02-24 11:53:30','suborbital rocket','success',224,'NASA 36.10DL','NRL UV Schmidt','','','UV Astron/Halley',1873,'',312 from dual union all
select 59535,'1986-S25','1986-02-26 12:02:00','suborbital rocket','success',227,'NASA 21.93UG','JHU FOT 6 Halley','','','UV Astron',1869,'',229 from dual union all
select 59536,'1986-S26','1986-03-01 00:00:00','military missile','success',756,'','Long range test','','','Test',307,'SNLE',800 from dual union all
select 59537,'1986-S27','1986-03-07 21:40:00','military missile','success',907,'FTM-11','Mk 21 RV x 8','','','Test',1703,'',1000 from dual union all
select 59538,'1986-S28','1986-03-07 18:12:39','suborbital rocket','success',211,'NASA 27.104GU','GSFC Solar UVI 2','','','Solar UV',1895,'',319 from dual union all
select 59539,'1986-S29','1986-03-07 19:30:00','suborbital rocket','success',1319,'NASA 33.51UU','','','','Aeron',1873,'',200 from dual union all
select 5954,'1959-S194','1959-10-01 10:49:00','suborbital rocket','success',839,'AA6.210C','','','','Aeron',394,'',138 from dual union all
select 59540,'1986-S30','1986-03-13 10:54:00','suborbital rocket','success',224,'NASA 36.17DL','NRL UV Schmidt','','','UV Astron/Halley',1873,'',312 from dual union all
select 59541,'1986-S31','1986-03-13 11:20:00','suborbital rocket','success',227,'NASA 21.95UG','JHU FOT 7 Halley','','','UV Astron',1873,'',231 from dual union all
select 59542,'1986-S32','1986-03-15 12:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT113GM-1','GLORY TRIP 113GM-1','OT',1705,'',1300 from dual union all
select 59543,'1986-S33','1986-03-15 16:33:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT114GM','GLORY TRIP 114GM','OT',1699,'',1300 from dual union all
select 59544,'1986-S34','1986-03-17 00:00:00','military missile','success',795,'','IPA-3','RSLP IPA-3','','Reentry Test',1698,'',1300 from dual union all
select 59545,'1986-S35','1986-04-01 08:17:00','suborbital rocket','success',231,'NASA 35.015GE','','','','Plasma',1047,'',912 from dual union all
select 59546,'1986-S36','1986-04-03 10:36:00','suborbital rocket','success',1337,'NASA 29.24UE','','','','Plasma',1047,'',500 from dual union all
select 59547,'1986-S37','1986-04-08 09:42:25','suborbital rocket','success',1303,'GL A51.378','SPIRIT I','','','Aeron',1047,'',241 from dual union all
select 59548,'1986-S38','1986-04-13 09:14:01','suborbital rocket','success',231,'NASA 35.016GE','','','','Plasma',1047,'',977 from dual union all
select 59549,'1986-S39','1986-04-18 15:35:00','suborbital rocket','success',860,'NASA 31.59UU','CWAS 13','','','Aeronomy',1895,'',140 from dual union all
select 5955,'1959-S195','1959-10-02 11:00:00','suborbital rocket','success',839,'AA6.211C','','','','Aeron',394,'',143 from dual union all
select 59550,'1986-S40','1986-04-22 07:11:00','suborbital rocket','success',210,'NASA 27.086UG','EUV Spectrometer','','','EUV Astron',1895,'',304 from dual union all
select 59551,'1986-S41','1986-04-22 08:49:00','suborbital rocket','success',231,'NASA 35.013UG','','','','UV Astron?',1821,'',900 from dual union all
select 59552,'1986-S42','1986-04-25 13:30:00','suborbital rocket','failure',860,'NASA 31.58UU','CWAS 14','','','Aeronomy',1895,'',140 from dual union all
select 59553,'1986-S43','1986-04-28 01:58:00','suborbital rocket','success',1318,'NASA 38.05UE','SPINEX 2','','','Plasma',1821,'',420 from dual union all
select 59554,'1986-S44','1986-04-30 04:40:00','suborbital rocket','success',1212,'DLR K-GR-141','DLR TEXUS 13','','','Micrograv',416,'',246 from dual union all
select 59555,'1986-S45','1986-05-12 04:55:00','suborbital rocket','success',1212,'DLR K-GR-142','DLR TEXUS 14A','','','Micrograv',416,'',250 from dual union all
select 59556,'1986-S46','1986-05-13 07:40:00','suborbital rocket','success',231,'NASA 35.014UE','CVE-1 Ba','','','Plasma',1821,'',430 from dual union all
select 59557,'1986-S47','1986-05-13 07:56:00','suborbital rocket','success',1318,'NASA 38.08UE','CRIT I','','','Plasma',1801,'',547 from dual union all
select 59558,'1986-S48','1986-05-20 16:45:00','military missile','success',939,'','DASO','','','OT',421,'SSBN 659',500 from dual union all
select 59559,'1986-S49','1986-05-21 16:00:00','military missile','success',907,'FTM-12','Mk 21 RV x 10','','','Test',1703,'',1000 from dual union all
select 5956,'1959-S196','1959-10-03 11:03:00','suborbital rocket','success',839,'AA6.202C','','','','Aeron',394,'',122 from dual union all
select 59560,'1986-S50','1986-06-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',133,'VISE',150 from dual union all
select 59561,'1986-S51','1986-06-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',133,'VISE',150 from dual union all
select 59562,'1986-S52','1986-06-04 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 620',500 from dual union all
select 59563,'1986-S53','1986-06-04 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 620',500 from dual union all
select 59564,'1986-S54','1986-06-04 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 620',500 from dual union all
select 59565,'1986-S55','1986-06-04 21:34:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 620',500 from dual union all
select 59566,'1986-S56','1986-06-15 08:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT115GB','GLORY TRIP 115GB','OT',1712,'',1300 from dual union all
select 59567,'1986-S57','1986-06-24 13:56:05','military missile','success',919,'DASO-1','P-480 86 Cape 01','','','OT',231,'',320 from dual union all
select 59568,'1986-S58','1986-06-24 15:23:00','military missile','success',919,'DASO-2','P-481 86 Cape 02','','','OT',231,'',320 from dual union all
select 59569,'1986-S59','1986-06-24 16:33:00','military missile','success',919,'DASO-3','P-482 86 Cape 03','','','OT',231,'',320 from dual union all
select 5957,'1959-S197','1959-10-05 11:01:00','suborbital rocket','success',839,'AA6.212C','','','','Aeron',394,'',127 from dual union all
select 59570,'1986-S60','1986-06-26 13:05:00','military missile','success',919,'DASO-4','P-483 86 Cape 04','','','OT',231,'',320 from dual union all
select 59571,'1986-S61','1986-06-26 16:47:00','military missile','success',919,'DASO-5','P-484 86 Cape 05','','','OT',231,'',320 from dual union all
select 59572,'1986-S62','1986-07-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',133,'VISE',150 from dual union all
select 59573,'1986-S63','1986-07-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',133,'VISE',150 from dual union all
select 59574,'1986-S64','1986-07-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',133,'VISE',150 from dual union all
select 59575,'1986-S65','1986-07-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',133,'VISE',150 from dual union all
select 59576,'1986-S66','1986-07-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',133,'VISE',150 from dual union all
select 59577,'1986-S67','1986-07-25 22:30:00','military missile','success',932,'A3TK/A3R','CHEVALINE DASO PEM','','','OT',421,'UK S27',1000 from dual union all
select 59578,'1986-S68','1986-07-26 02:13:00','military missile','success',932,'A3TK/A3R','CHEVALINE DASO PEM','','','OT',421,'UK S27',1000 from dual union all
select 59579,'1986-S69','1986-07-26 22:37:20','suborbital rocket','success',860,'NASA 31.57GE','MAED','','','Plasma',417,'',101 from dual union all
select 5958,'1959-S198','1959-10-06 16:41:00','military missile','success',1367,'235','R&D Series IV','','','Meteorites',236,'',520 from dual union all
select 59580,'1986-S70','1986-07-26 23:03:00','suborbital rocket','success',860,'NASA 31.60UE','MAED','','','Plasma',417,'',99 from dual union all
select 59581,'1986-S71','1986-07-26 23:44:40','suborbital rocket','success',860,'NASA 31.61UE','MAED','','','Plasma',417,'',100 from dual union all
select 59582,'1986-S72','1986-07-28 00:00:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 655',1000 from dual union all
select 59583,'1986-S73','1986-07-28 21:55:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 655',1000 from dual union all
select 59584,'1986-S74','1986-08-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',133,'VISE',150 from dual union all
select 59585,'1986-S75','1986-08-01 00:00:00','suborbital rocket','unknown',740,'D-75MG','','','','Aeron',133,'VISE',150 from dual union all
select 59586,'1986-S76','1986-07-31 12:40:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT116GM-1','GLORY TRIP 116GM-1','OT',1705,'',1300 from dual union all
select 59587,'1986-S77','1986-07-31 00:00:00','military missile','success',1041,'48788-302-1L','15Zh60 LKI-1','','','Test',477,'',1000 from dual union all
select 59588,'1986-S78','1986-08-01 22:15:00','military missile','success',932,'A3TK/A3R','CHEVALINE DASO PEM','','','OT',421,'UK S27',1000 from dual union all
select 59589,'1986-S79','1986-08-02 01:35:00','military missile','success',932,'A3TK/A3R','CHEVALINE DASO PEM','','','OT',421,'UK S27',1000 from dual;
insert into LAUNCH
select 5959,'1959-S199','1959-10-06 05:55:00','military missile','success',154,'18D','Mk 3-1','R&D','','Test',226,'',1800 from dual union all
select 59590,'1986-S80','1986-08-22 00:00:00','test rocket','success',20,'','','','','Interceptor',383,'F-15',1000 from dual union all
select 59591,'1986-S81','1986-08-23 18:25:00','military missile','success',907,'FTM-15','Mk 21 RV x 9','','','Test (SAC Crew)',1697,'',1000 from dual union all
select 59592,'1986-S82','1986-08-28 14:04:00','military missile','failure',799,'','Mk 12/12A RV x 3','FOT GT117GM','GLORY TRIP 117GM','OT',1699,'',1000 from dual union all
select 59593,'1986-S83','1986-09-01 00:00:00','suborbital rocket','unknown',740,'','','','','Aeron',143,'ZUB',150 from dual union all
select 59594,'1986-S84','1986-09-01 00:00:00','suborbital rocket','unknown',740,'','','','','Aeron',143,'ZUB',150 from dual union all
select 59595,'1986-S85','1986-09-01 00:00:00','suborbital rocket','unknown',740,'','','','','Aeron',143,'ZUB',150 from dual union all
select 59596,'1986-S86','1986-09-01 00:00:00','suborbital rocket','unknown',740,'','','','','Aeron',143,'ZUB',150 from dual union all
select 59597,'1986-S87','1986-09-01 00:00:00','military missile','success',1514,'','','','','Test',201,'DELTA2',1000 from dual union all
select 59598,'1986-S88','1986-08-31 00:00:00','military missile','success',1442,'','DASO','','','OT',421,'SSBN 630',1000 from dual union all
select 59599,'1986-S89','1986-09-05 16:30:00','suborbital rocket','success',117,'SDIO G-2','Delta 180 Target','','','Target',1873,'',300 from dual union all
select 596,'1945-S72','1945-03-03 00:00:00','military missile','success',1470,'','London','','','Weapon',1733,'',90 from dual union all
select 5960,'1959-S20','1959-02-28 07:58:00','military missile','success',1355,'131','RVX-1','','','Reentry Test',233,'',1230 from dual union all
select 59600,'1986-S90','1986-09-06 13:00:00','suborbital rocket','success',1073,'S-310-17','Airglow','','','Aeron',833,'',202 from dual union all
select 59601,'1986-S91','1986-09-17 00:00:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 632',1000 from dual union all
select 59602,'1986-S92','1986-09-17 19:04:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 632',1000 from dual union all
select 59603,'1986-S93','1986-09-18 12:30:00','military missile','success',907,'FTM-13','Mk 21 RV x 6','','','Test',1703,'',1000 from dual union all
select 59604,'1986-S94','1986-09-25 08:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT118GB','GLORY TRIP 118GB','OT',1712,'',1300 from dual union all
select 59605,'1986-S95','1986-10-01 00:00:00','suborbital rocket','unknown',740,'','','','','Aeron',143,'ZUB',150 from dual union all
select 59606,'1986-S96','1986-10-01 00:00:00','suborbital rocket','unknown',740,'','','','','Aeron',143,'ZUB',150 from dual union all
select 59607,'1986-S97','1986-10-01 00:00:00','suborbital rocket','unknown',740,'','','','','Aeron',143,'ZUB',150 from dual union all
select 59608,'1986-S98','1986-10-01 00:00:00','suborbital rocket','unknown',740,'','','','','Aeron',143,'ZUB',150 from dual union all
select 59609,'1986-S99','1986-10-01 00:00:00','suborbital rocket','unknown',740,'','','','','Aeron',143,'ZUB',150 from dual union all
select 5961,'1959-S200','1959-10-06 18:26:00','military missile','success',1367,'239','IWST','','FOREIGN TRAVEL','Test',1677,'',520 from dual union all
select 59610,'1986-W01','1986-01-03 16:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 59611,'1986-W02','1986-01-04 16:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',75 from dual union all
select 59612,'1986-W03','1986-01-05 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',78 from dual union all
select 59613,'1986-W04','1986-01-06 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',58 from dual union all
select 59614,'1986-W05','1986-01-08 02:00:00','suborbital rocket','success',761,'MT135P-R589','','','','Meteo',1332,'',57 from dual union all
select 59615,'1986-W06','1986-01-09 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 59616,'1986-W07','1986-01-09 15:11:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',51 from dual union all
select 59617,'1986-W08','1986-01-12 12:33:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',64 from dual union all
select 59618,'1986-W09','1986-01-12 12:45:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',50 from dual union all
select 59619,'1986-W10','1986-01-12 13:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 5962,'1959-S201','1959-10-08 00:00:00','military missile','success',976,'IO3-20T','8K63 UBP-26','','','Test',636,'',402 from dual union all
select 59620,'1986-W100','1986-05-21 16:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 59621,'1986-W101','1986-05-21 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',64 from dual union all
select 59622,'1986-W102','1986-05-22 21:02:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',69 from dual union all
select 59623,'1986-W103','1986-05-28 02:00:00','suborbital rocket','success',761,'MT135P-R604','','','','Meteo',1332,'',56 from dual union all
select 59624,'1986-W104','1986-05-28 18:02:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',77 from dual union all
select 59625,'1986-W105','1986-05-29 19:02:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',56 from dual union all
select 59626,'1986-W106','1986-06-11 02:00:00','suborbital rocket','success',761,'MT135P-R605','','','','Meteo',1332,'',56 from dual union all
select 59627,'1986-W107','1986-06-16 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',72 from dual union all
select 59628,'1986-W108','1986-06-17 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',71 from dual union all
select 59629,'1986-W109','1986-06-17 21:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 5963,'1959-S202','1959-10-09 00:00:00','military missile','success',976,'IP4-20T','8K63 UBP-28','','','Test',636,'',402 from dual union all
select 59630,'1986-W11','1986-01-14 23:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',53 from dual union all
select 59631,'1986-W110','1986-06-18 19:19:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',61 from dual union all
select 59632,'1986-W111','1986-06-20 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',67 from dual union all
select 59633,'1986-W112','1986-06-23 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 59634,'1986-W113','1986-06-24 17:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 59635,'1986-W114','1986-06-25 02:00:00','suborbital rocket','success',761,'MT135P-R606','','','','Meteo',1332,'',52 from dual union all
select 59636,'1986-W115','1986-06-26 18:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 59637,'1986-W116','1986-06-27 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',64 from dual union all
select 59638,'1986-W117','1986-06-27 17:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',60 from dual union all
select 59639,'1986-W118','1986-07-02 02:00:00','suborbital rocket','success',761,'MT135P-R607','','','','Meteo',1332,'',56 from dual union all
select 5964,'1959-S203','1959-10-09 08:40:00','suborbital rocket','success',839,'AA6.209C','','','','Aeron',394,'',143 from dual union all
select 59640,'1986-W119','1986-07-02 15:29:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 59641,'1986-W12','1986-01-15 12:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',76 from dual union all
select 59642,'1986-W120','1986-07-02 17:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',60 from dual union all
select 59643,'1986-W121','1986-07-10 22:15:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',61 from dual union all
select 59644,'1986-W122','1986-07-11 22:13:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',65 from dual union all
select 59645,'1986-W123','1986-07-11 22:41:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',63 from dual union all
select 59646,'1986-W124','1986-07-16 02:01:00','suborbital rocket','success',761,'MT135P-R608','','','','Meteo',1332,'',58 from dual union all
select 59647,'1986-W125','1986-07-17 23:02:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 59648,'1986-W126','1986-07-22 21:51:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',56 from dual union all
select 59649,'1986-W127','1986-07-24 02:00:00','suborbital rocket','success',761,'MT135P-R609','','','','Meteo',1332,'',56 from dual union all
select 5965,'1959-S204','1959-10-09 11:17:00','suborbital rocket','success',839,'AA6.213C','','','','Aeron',394,'',143 from dual union all
select 59650,'1986-W128','1986-07-24 19:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 59651,'1986-W129','1986-07-26 03:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',65 from dual union all
select 59652,'1986-W13','1986-01-16 02:00:00','suborbital rocket','success',761,'MT135P-R590','','','','Meteo',1332,'',56 from dual union all
select 59653,'1986-W130','1986-07-26 04:42:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',57 from dual union all
select 59654,'1986-W131','1986-07-28 22:12:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',64 from dual union all
select 59655,'1986-W132','1986-07-28 22:42:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',64 from dual union all
select 59656,'1986-W133','1986-08-02 03:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',68 from dual union all
select 59657,'1986-W134','1986-08-02 03:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 59658,'1986-W135','1986-08-05 22:27:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 59659,'1986-W136','1986-08-06 02:00:00','suborbital rocket','success',761,'MT135P-R610','','','','Meteo',1332,'',56 from dual union all
select 5966,'1959-S205','1959-10-10 03:10:00','military missile','success',154,'22D','Mk 3-1','R&D','','Test',228,'',1800 from dual union all
select 59660,'1986-W137','1986-08-13 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',77 from dual union all
select 59661,'1986-W138','1986-08-13 17:34:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',65 from dual union all
select 59662,'1986-W139','1986-08-15 19:09:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',65 from dual union all
select 59663,'1986-W14','1986-01-18 13:57:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',61 from dual union all
select 59664,'1986-W140','1986-08-20 02:00:00','suborbital rocket','success',761,'MT135P-R611','','','','Meteo',1332,'',54 from dual union all
select 59665,'1986-W141','1986-08-21 18:01:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',59 from dual union all
select 59666,'1986-W142','1986-08-22 10:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',72 from dual union all
select 59667,'1986-W143','1986-08-22 13:27:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 59668,'1986-W144','1986-08-22 16:10:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',53 from dual union all
select 59669,'1986-W145','1986-08-23 18:25:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',63 from dual union all
select 5967,'1959-S206','1959-10-10 12:00:00','suborbital rocket','success',48,'AA2.102C','','','','Meteorites',1869,'',111 from dual union all
select 59670,'1986-W146','1986-08-23 20:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',65 from dual union all
select 59671,'1986-W147','1986-08-25 22:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',64 from dual union all
select 59672,'1986-W148','1986-08-27 02:00:00','suborbital rocket','success',761,'MT135P-R612','','','','Meteo',1332,'',52 from dual union all
select 59673,'1986-W149','1986-08-28 14:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',60 from dual union all
select 59674,'1986-W15','1986-01-18 13:57:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 59675,'1986-W150','1986-08-31 19:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',69 from dual union all
select 59676,'1986-W151','1986-08-31 21:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',62 from dual union all
select 59677,'1986-W152','1986-09-04 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 59678,'1986-W153','1986-09-07 07:48:00','suborbital rocket','unknown',738,'','','','','Aeron',1639,'USHAKOV',60 from dual union all
select 59679,'1986-W154','1986-09-10 02:00:00','suborbital rocket','success',761,'MT135P-R613','','','','Meteo',1332,'',54 from dual;
insert into LAUNCH
select 5968,'1959-S207','1959-10-12 09:00:00','suborbital rocket','success',839,'AA6.208C','','','','Aeron',394,'',123 from dual union all
select 59680,'1986-W155','1986-09-12 17:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',65 from dual union all
select 59681,'1986-W156','1986-09-12 22:30:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 59682,'1986-W157','1986-09-16 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',79 from dual union all
select 59683,'1986-W158','1986-09-18 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',71 from dual union all
select 59684,'1986-W159','1986-09-19 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1639,'USHAKOV',60 from dual union all
select 59685,'1986-W16','1986-01-18 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',64 from dual union all
select 59686,'1986-W160','1986-09-20 08:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',68 from dual union all
select 59687,'1986-W161','1986-09-20 10:43:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 59688,'1986-W162','1986-09-21 08:04:00','suborbital rocket','unknown',738,'','','','','Aeron',1639,'USHAKOV',60 from dual union all
select 59689,'1986-W163','1986-09-22 08:17:00','suborbital rocket','unknown',738,'','','','','Aeron',1639,'USHAKOV',60 from dual union all
select 5969,'1959-S208','1959-10-12 10:12:00','suborbital rocket','success',839,'AA6.216C','','','','Aeron',394,'',123 from dual union all
select 59690,'1986-W164','1986-09-22 22:23:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',63 from dual union all
select 59691,'1986-W165','1986-09-22 22:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',73 from dual union all
select 59692,'1986-W166','1986-09-24 02:02:00','suborbital rocket','success',761,'MT135P-R614','','','','Meteo',1332,'',54 from dual union all
select 59693,'1986-W167','1986-09-24 07:59:00','suborbital rocket','unknown',738,'','','','','Aeron',1639,'USHAKOV',62 from dual union all
select 59694,'1986-W168','1986-09-24 08:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',67 from dual union all
select 59695,'1986-W169','1986-09-24 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 59696,'1986-W17','1986-01-22 02:00:00','suborbital rocket','success',761,'MT135P-R591','','','','Meteo',1332,'',57 from dual union all
select 59697,'1986-W170','1986-09-24 14:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 59698,'1986-W171','1986-09-25 14:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',66 from dual union all
select 59699,'1986-W172','1986-09-28 07:42:00','suborbital rocket','unknown',738,'','','','','Aeron',1639,'USHAKOV',60 from dual union all
select 597,'1945-S73','1945-03-17 00:00:00','military missile','success',1470,'','Remagen','','','Weapon',1736,'',90 from dual union all
select 5970,'1959-S209','1959-10-13 09:00:00','test rocket','success',254,'','','','','Interceptor',21,'B-47',200 from dual union all
select 59700,'1986-W173','1986-09-30 07:29:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',57 from dual union all
select 59701,'1986-W174','1986-09-30 09:51:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',67 from dual union all
select 59702,'1986-W175','1986-09-30 10:57:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',67 from dual union all
select 59703,'1986-W176','1986-09-30 12:33:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',74 from dual union all
select 59704,'1986-W177','1986-09-30 18:33:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',69 from dual union all
select 59705,'1986-W178','1986-10-01 18:58:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',52 from dual union all
select 59706,'1986-W179','1986-10-03 16:41:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',59 from dual union all
select 59707,'1986-W18','1986-01-22 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 59708,'1986-W180','1986-10-08 12:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',70 from dual union all
select 59709,'1986-W181','1986-10-09 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 5971,'1959-S21','1959-03-03 00:00:00','suborbital rocket','success',1438,'TB I alpha','D58','','','Reentry Test',1821,'',260 from dual union all
select 59710,'1986-W182','1986-10-09 22:13:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',71 from dual union all
select 59711,'1986-W183','1986-10-10 18:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',63 from dual union all
select 59712,'1986-W184','1986-10-14 16:43:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 59713,'1986-W185','1986-10-14 21:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',69 from dual union all
select 59714,'1986-W186','1986-10-15 02:00:00','suborbital rocket','success',761,'MT135P-R616','','','','Meteo',1332,'',54 from dual union all
select 59715,'1986-W187','1986-10-16 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',67 from dual union all
select 59716,'1986-W188','1986-10-17 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',68 from dual union all
select 59717,'1986-W189','1986-10-21 16:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 59718,'1986-W19','1986-01-24 15:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 59719,'1986-W190','1986-10-21 22:07:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',68 from dual union all
select 5972,'1959-S210','1959-10-14 04:15:00','military missile','success',1367,'221','R&D Series IV','','','Test',234,'',520 from dual union all
select 59720,'1986-W191','1986-10-22 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',69 from dual union all
select 59721,'1986-W192','1986-10-22 19:00:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 59722,'1986-W193','1986-10-23 02:00:00','suborbital rocket','success',761,'MT135P-R617','','','','Meteo',1332,'',54 from dual union all
select 59723,'1986-W194','1986-10-28 20:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',65 from dual union all
select 59724,'1986-W195','1986-10-29 02:10:00','suborbital rocket','success',761,'MT135P-R618','','','','Meteo',1332,'',52 from dual union all
select 59725,'1986-W196','1986-10-29 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 59726,'1986-W197','1986-10-30 13:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',69 from dual union all
select 59727,'1986-W198','1986-11-05 13:01:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',67 from dual union all
select 59728,'1986-W199','1986-11-05 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',60 from dual union all
select 59729,'1986-W20','1986-01-27 19:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',74 from dual union all
select 5973,'1959-S211','1959-10-13 11:17:00','suborbital rocket','success',839,'AA6.214C','','','','Aeron',394,'',116 from dual union all
select 59730,'1986-W200','1986-11-06 16:37:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',55 from dual union all
select 59731,'1986-W201','1986-11-07 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',51 from dual union all
select 59732,'1986-W202','1986-11-12 02:00:00','suborbital rocket','success',761,'MT135P-R619','','','','Meteo',1332,'',54 from dual union all
select 59733,'1986-W203','1986-11-12 13:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',66 from dual union all
select 59734,'1986-W204','1986-11-12 16:45:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 59735,'1986-W205','1986-11-13 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',69 from dual union all
select 59736,'1986-W206','1986-11-19 02:00:00','suborbital rocket','success',761,'MT135P-R620','','','','Meteo',1332,'',53 from dual union all
select 59737,'1986-W207','1986-11-19 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',67 from dual union all
select 59738,'1986-W208','1986-11-21 10:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',59 from dual union all
select 59739,'1986-W209','1986-11-26 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',60 from dual union all
select 5974,'1959-S212','1959-10-15 00:00:00','suborbital rocket','success',972,'','','','','Aeron?',1010,'',86 from dual union all
select 59740,'1986-W21','1986-01-28 16:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',64 from dual union all
select 59741,'1986-W210','1986-11-28 13:17:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',50 from dual union all
select 59742,'1986-W211','1986-12-01 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',58 from dual union all
select 59743,'1986-W212','1986-12-01 19:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 59744,'1986-W213','1986-12-03 02:00:00','suborbital rocket','success',761,'MT135P-R621','','','','Meteo',1332,'',54 from dual union all
select 59745,'1986-W214','1986-12-03 14:57:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',62 from dual union all
select 59746,'1986-W215','1986-12-03 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 59747,'1986-W216','1986-12-05 22:54:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',71 from dual union all
select 59748,'1986-W217','1986-12-09 21:53:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',70 from dual union all
select 59749,'1986-W218','1986-12-10 02:00:00','suborbital rocket','success',761,'MT135P-R622','','','','Meteo',1332,'',55 from dual union all
select 5975,'1959-S213','1959-10-15 00:00:00','suborbital rocket','success',972,'','','','','Aeron?',1010,'',86 from dual union all
select 59750,'1986-W219','1986-12-12 19:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',55 from dual union all
select 59751,'1986-W22','1986-01-29 19:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',50 from dual union all
select 59752,'1986-W220','1986-12-15 13:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',54 from dual union all
select 59753,'1986-W221','1986-12-16 16:41:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 59754,'1986-W222','1986-12-17 02:00:00','suborbital rocket','success',761,'MT135P-R623','','','','Meteo',1332,'',56 from dual union all
select 59755,'1986-W223','1986-12-22 19:30:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',55 from dual union all
select 59756,'1986-W224','1986-12-23 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 59757,'1986-W225','1986-12-24 02:00:00','suborbital rocket','success',761,'MT135P-R624','','','','Meteo',1332,'',55 from dual union all
select 59758,'1986-W226','1986-12-24 12:47:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',65 from dual union all
select 59759,'1986-W227','1986-12-29 12:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',59 from dual union all
select 5976,'1959-S214','1959-10-15 11:05:00','suborbital rocket','success',839,'AA6.207C','','','','Aeron',394,'',122 from dual union all
select 59760,'1986-W228','1986-08-20 02:20:00','suborbital rocket','success',759,'MT-135-45','SONDE','','','Meteo',839,'',58 from dual union all
select 59761,'1986-W229','1986-08-20 04:00:00','suborbital rocket','unknown',759,'MT-135-46','SONDE','','','Meteo',839,'',60 from dual union all
select 59762,'1986-W23','1986-01-30 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',79 from dual union all
select 59763,'1986-W230','1986-02-20 02:00:00','suborbital rocket','success',761,'MT135P-R594','','','','Meteo',1332,'',60 from dual union all
select 59764,'1986-W231','1986-04-10 02:00:00','suborbital rocket','success',761,'MT135P-R598','','','','Meteo',1332,'',60 from dual union all
select 59765,'1986-W232','1986-10-08 02:00:00','suborbital rocket','success',761,'MT135P-R615','','','','Meteo',1332,'',60 from dual union all
select 59766,'1986-W24','1986-02-04 23:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',52 from dual union all
select 59767,'1986-W25','1986-02-05 02:00:00','suborbital rocket','success',761,'MT135P-R592','','','','Meteo',1332,'',57 from dual union all
select 59768,'1986-W26','1986-02-07 13:50:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual union all
select 59769,'1986-W27','1986-02-07 17:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual;
insert into LAUNCH
select 5977,'1959-S215','1959-10-16 11:17:00','suborbital rocket','success',839,'AA6.215C','','','','Aeron',394,'',139 from dual union all
select 59770,'1986-W28','1986-02-07 19:10:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 59771,'1986-W29','1986-02-07 19:25:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',53 from dual union all
select 59772,'1986-W30','1986-02-12 02:00:00','suborbital rocket','success',761,'MT135P-R593','','','','Meteo',1332,'',53 from dual union all
select 59773,'1986-W31','1986-02-12 07:23:00','suborbital rocket','unknown',738,'','','','','Aeron',1030,'PASSAT',62 from dual union all
select 59774,'1986-W32','1986-02-12 21:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',51 from dual union all
select 59775,'1986-W33','1986-02-12 23:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',52 from dual union all
select 59776,'1986-W34','1986-02-14 08:42:00','suborbital rocket','unknown',738,'','','','','Aeron',1030,'PASSAT',64 from dual union all
select 59777,'1986-W35','1986-02-17 07:03:00','suborbital rocket','unknown',738,'','','','','Aeron',1030,'PASSAT',61 from dual union all
select 59778,'1986-W36','1986-02-18 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',78 from dual union all
select 59779,'1986-W37','1986-02-18 22:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',79 from dual union all
select 5978,'1959-S216','1959-10-20 14:25:00','suborbital rocket','success',48,'AA2.240C','','','','Ionos',1895,'',113 from dual union all
select 59780,'1986-W38','1986-02-19 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',1030,'PASSAT',62 from dual union all
select 59781,'1986-W39','1986-02-20 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',59 from dual union all
select 59782,'1986-W40','1986-02-20 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 59783,'1986-W41','1986-02-21 02:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 59784,'1986-W42','1986-02-21 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1030,'PASSAT',64 from dual union all
select 59785,'1986-W43','1986-02-21 16:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',54 from dual union all
select 59786,'1986-W44','1986-02-26 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1030,'PASSAT',61 from dual union all
select 59787,'1986-W45','1986-02-26 19:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',55 from dual union all
select 59788,'1986-W46','1986-02-28 07:10:00','suborbital rocket','unknown',738,'','','','','Aeron',1030,'PASSAT',61 from dual union all
select 59789,'1986-W47','1986-03-03 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',1030,'PASSAT',59 from dual union all
select 5979,'1959-S217','1959-10-21 22:57:00','military missile','success',1367,'220','IWST','','STAND FAST','Test',1673,'',520 from dual union all
select 59790,'1986-W48','1986-03-03 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',76 from dual union all
select 59791,'1986-W49','1986-03-05 02:00:00','suborbital rocket','success',761,'MT135P-R595','','','','Meteo',1332,'',52 from dual union all
select 59792,'1986-W50','1986-03-05 07:02:00','suborbital rocket','unknown',738,'','','','','Aeron',1030,'PASSAT',62 from dual union all
select 59793,'1986-W51','1986-03-05 11:37:00','suborbital rocket','unknown',728,'STRATWARM','ISRO 8.862','','','Aeron',1467,'',79 from dual union all
select 59794,'1986-W52','1986-03-05 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1362,'',54 from dual union all
select 59795,'1986-W53','1986-03-07 07:18:00','suborbital rocket','unknown',738,'','','','','Aeron',1030,'PASSAT',55 from dual union all
select 59796,'1986-W54','1986-03-07 22:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',63 from dual union all
select 59797,'1986-W55','1986-03-11 01:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 59798,'1986-W56','1986-03-12 02:00:00','suborbital rocket','success',761,'MT135P-R596','','','','Meteo',1332,'',56 from dual union all
select 59799,'1986-W57','1986-03-13 23:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',50 from dual union all
select 598,'1945-S74','1945-03-17 00:00:00','military missile','success',1470,'','Remagen','','','Weapon',1736,'',90 from dual union all
select 5980,'1959-S218','1959-10-22 00:00:00','military missile','success',883,'002','','','','Reentry Test',313,'',150 from dual union all
select 59800,'1986-W58','1986-03-14 07:39:00','suborbital rocket','unknown',738,'','','','','Aeron',913,'KRENKEL''',65 from dual union all
select 59801,'1986-W59','1986-03-17 07:33:00','suborbital rocket','unknown',738,'','','','','Aeron',913,'KRENKEL''',61 from dual union all
select 59802,'1986-W60','1986-03-18 07:58:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',72 from dual union all
select 59803,'1986-W61','1986-03-18 09:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 59804,'1986-W62','1986-03-18 09:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',55 from dual union all
select 59805,'1986-W63','1986-03-19 07:04:00','suborbital rocket','unknown',738,'','','','','Aeron',913,'KRENKEL''',65 from dual union all
select 59806,'1986-W64','1986-03-19 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 59807,'1986-W65','1986-03-21 07:06:00','suborbital rocket','unknown',738,'','','','','Aeron',913,'KRENKEL''',54 from dual union all
select 59808,'1986-W66','1986-03-26 02:00:00','suborbital rocket','success',761,'MT135P-R597','','','','Meteo',1332,'',56 from dual union all
select 59809,'1986-W67','1986-03-26 07:04:00','suborbital rocket','unknown',738,'','','','','Aeron',913,'KRENKEL''',60 from dual union all
select 5981,'1959-S219','1959-10-22 03:20:00','military missile','success',638,'CM-31','R&D','','','Test',248,'',500 from dual union all
select 59810,'1986-W68','1986-03-26 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',69 from dual union all
select 59811,'1986-W69','1986-03-26 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',63 from dual union all
select 59812,'1986-W70','1986-03-26 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 59813,'1986-W71','1986-03-28 07:13:00','suborbital rocket','unknown',738,'','','','','Aeron',913,'KRENKEL''',59 from dual union all
select 59814,'1986-W72','1986-04-01 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',74 from dual union all
select 59815,'1986-W73','1986-04-02 07:12:00','suborbital rocket','unknown',738,'','','','','Aeron',913,'KRENKEL''',65 from dual union all
select 59816,'1986-W74','1986-04-09 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',913,'KRENKEL''',61 from dual union all
select 59817,'1986-W75','1986-04-11 07:26:00','suborbital rocket','unknown',738,'','','','','Aeron',913,'KRENKEL''',59 from dual union all
select 59818,'1986-W76','1986-04-17 02:04:59','suborbital rocket','success',761,'MT135P-R599','','','','Meteo',1332,'',57 from dual union all
select 59819,'1986-W77','1986-04-21 14:23:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 5982,'1959-S22','1959-03-10 00:00:00','military missile','success',1053,'CC-1016','','','','OT',1854,'',90 from dual union all
select 59820,'1986-W78','1986-04-21 16:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 59821,'1986-W79','1986-04-23 14:46:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',62 from dual union all
select 59822,'1986-W80','1986-04-24 02:00:00','suborbital rocket','success',761,'MT135P-R600','','','','Meteo',1332,'',57 from dual union all
select 59823,'1986-W81','1986-04-24 15:55:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',65 from dual union all
select 59824,'1986-W82','1986-04-25 00:23:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',69 from dual union all
select 59825,'1986-W83','1986-04-27 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',79 from dual union all
select 59826,'1986-W84','1986-04-28 09:37:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',72 from dual union all
select 59827,'1986-W85','1986-04-28 11:19:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 59828,'1986-W86','1986-04-28 11:35:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',64 from dual union all
select 59829,'1986-W87','1986-04-30 03:06:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 5983,'1959-S220','1959-10-22 17:30:00','military missile','success',1016,'No. 267432','8K71 LKI III-12','','','Test',480,'',1350 from dual union all
select 59830,'1986-W88','1986-04-30 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 59831,'1986-W89','1986-05-02 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',61 from dual union all
select 59832,'1986-W90','1986-05-07 11:36:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 59833,'1986-W91','1986-05-07 14:56:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',77 from dual union all
select 59834,'1986-W92','1986-05-07 16:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 59835,'1986-W93','1986-05-08 02:00:00','suborbital rocket','success',761,'MT135P-R601','','','','Meteo',1332,'',59 from dual union all
select 59836,'1986-W94','1986-05-09 19:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',58 from dual union all
select 59837,'1986-W95','1986-05-13 14:03:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',68 from dual union all
select 59838,'1986-W96','1986-05-13 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',63 from dual union all
select 59839,'1986-W97','1986-05-14 02:00:00','suborbital rocket','success',761,'MT135P-R602','','','','Meteo',1332,'',54 from dual union all
select 5984,'1959-S221','1959-10-25 00:00:00','military missile','success',976,'ID8-20T','8K63 UBP-29','','','Test',636,'',402 from dual union all
select 59840,'1986-W98','1986-05-21 02:00:00','suborbital rocket','success',761,'MT135P-R603','','','','Meteo',1332,'',58 from dual union all
select 59841,'1986-W99','1986-05-21 14:31:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 59842,'1986-Y01','1986-01-01 11:16:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 59843,'1986-Y02','1986-01-01 14:22:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 59844,'1986-Y03','1986-01-04 10:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',86 from dual union all
select 59845,'1986-Y04','1986-01-04 18:31:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',80 from dual union all
select 59846,'1986-Y05','1986-01-04 20:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1382,'SHIR',80 from dual union all
select 59847,'1986-Y06','1986-01-05 11:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 59848,'1986-Y07','1986-01-06 11:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 59849,'1986-Y08','1986-01-07 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 5985,'1959-S222','1959-10-25 17:32:00','military missile','success',1016,'No. 267434','8K71 LKI III-13','','','Test',480,'',1350 from dual union all
select 59850,'1986-Y09','1986-01-08 12:07:00','suborbital rocket','success',728,'STRATWARM','ISRO 8.845','','','Aeron',1467,'',85 from dual union all
select 59851,'1986-Y10','1986-01-08 13:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 59852,'1986-Y100','1986-04-29 08:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',80 from dual union all
select 59853,'1986-Y101','1986-04-29 09:46:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',84 from dual union all
select 59854,'1986-Y102','1986-04-29 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',89 from dual union all
select 59855,'1986-Y103','1986-04-30 11:34:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 59856,'1986-Y104','1986-04-30 19:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 59857,'1986-Y105','1986-05-02 07:56:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',87 from dual union all
select 59858,'1986-Y106','1986-05-02 09:06:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',85 from dual union all
select 59859,'1986-Y107','1986-05-04 07:56:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',86 from dual;
insert into LAUNCH
select 5986,'1959-S223','1959-10-25 17:44:00','suborbital rocket','success',42,'AA10.184C','Utah ne probe','','','Ionos',428,'',386 from dual union all
select 59860,'1986-Y108','1986-05-04 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',86 from dual union all
select 59861,'1986-Y109','1986-05-06 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 59862,'1986-Y11','1986-01-08 14:20:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 59863,'1986-Y110','1986-05-13 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 59864,'1986-Y111','1986-05-14 11:34:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 59865,'1986-Y112','1986-05-14 15:30:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 59866,'1986-Y113','1986-05-14 19:15:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 59867,'1986-Y114','1986-05-20 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 59868,'1986-Y115','1986-05-21 11:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',87 from dual union all
select 59869,'1986-Y116','1986-05-22 09:09:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',86 from dual union all
select 5987,'1959-S224','1959-10-29 02:12:00','military missile','success',1367,'230','R&D Series IV','','','Meteorites',236,'',520 from dual union all
select 59870,'1986-Y117','1986-05-22 10:36:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',85 from dual union all
select 59871,'1986-Y118','1986-05-22 11:54:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 59872,'1986-Y119','1986-05-25 08:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',85 from dual union all
select 59873,'1986-Y12','1986-01-12 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 59874,'1986-Y120','1986-05-25 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',80 from dual union all
select 59875,'1986-Y121','1986-05-27 07:20:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',82 from dual union all
select 59876,'1986-Y122','1986-05-27 20:20:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 59877,'1986-Y123','1986-05-28 07:03:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',84 from dual union all
select 59878,'1986-Y124','1986-05-28 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',86 from dual union all
select 59879,'1986-Y125','1986-05-28 15:31:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 5988,'1959-S225','1959-10-28 14:47:00','suborbital rocket','success',42,'AA10.185C','Utah ne probe','','','Ionos',428,'',135 from dual union all
select 59880,'1986-Y126','1986-05-29 08:06:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',81 from dual union all
select 59881,'1986-Y127','1986-05-29 09:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',82 from dual union all
select 59882,'1986-Y128','1986-05-29 11:52:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 59883,'1986-Y129','1986-05-30 06:49:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',80 from dual union all
select 59884,'1986-Y13','1986-01-13 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',88 from dual union all
select 59885,'1986-Y130','1986-06-03 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 59886,'1986-Y131','1986-06-04 11:37:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 59887,'1986-Y132','1986-06-04 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 59888,'1986-Y133','1986-06-04 15:24:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 59889,'1986-Y134','1986-06-09 16:55:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',90 from dual union all
select 5989,'1959-S226','1959-10-28 22:40:00','ballistic missile test','success',1177,'1','E60 Echo Test','','','SC Test',1821,'',400 from dual union all
select 59890,'1986-Y135','1986-06-09 19:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',87 from dual union all
select 59891,'1986-Y136','1986-06-10 20:38:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',91 from dual union all
select 59892,'1986-Y137','1986-06-11 11:34:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 59893,'1986-Y138','1986-06-11 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 59894,'1986-Y139','1986-06-11 16:55:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',90 from dual union all
select 59895,'1986-Y14','1986-01-14 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 59896,'1986-Y140','1986-06-11 19:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 59897,'1986-Y141','1986-06-11 20:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 59898,'1986-Y142','1986-06-17 20:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',89 from dual union all
select 59899,'1986-Y143','1986-06-18 12:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 599,'1945-S75','1945-03-17 00:00:00','military missile','success',1470,'','Remagen','','','Weapon',1736,'',90 from dual union all
select 5990,'1959-S227','1959-10-29 07:20:00','military missile','success',154,'26D','Mk 3-1','R&D','','Test',226,'',1800 from dual union all
select 59900,'1986-Y144','1986-06-18 15:01:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 59901,'1986-Y145','1986-06-18 17:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 59902,'1986-Y146','1986-06-24 20:03:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 59903,'1986-Y147','1986-06-25 11:50:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',90 from dual union all
select 59904,'1986-Y148','1986-06-25 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 59905,'1986-Y149','1986-06-25 14:12:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59906,'1986-Y15','1986-01-15 14:46:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59907,'1986-Y150','1986-07-01 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',90 from dual union all
select 59908,'1986-Y151','1986-07-02 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 59909,'1986-Y152','1986-07-02 14:42:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 5991,'1959-S228','1959-10-30 11:49:00','military missile','success',239,'BK.06','H06','','','Reentry Test/CR',1843,'',732 from dual union all
select 59910,'1986-Y153','1986-07-03 13:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 59911,'1986-Y154','1986-07-07 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 59912,'1986-Y155','1986-07-08 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 59913,'1986-Y156','1986-07-09 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 59914,'1986-Y157','1986-07-09 16:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 59915,'1986-Y158','1986-07-09 17:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 59916,'1986-Y159','1986-07-16 11:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 59917,'1986-Y16','1986-01-15 15:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 59918,'1986-Y160','1986-07-16 14:10:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 59919,'1986-Y161','1986-07-17 15:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 5992,'1959-S229','1959-11-01 00:00:00','military missile','success',978,'','','','','Test',1626,'ZULUV',150 from dual union all
select 59920,'1986-Y162','1986-07-22 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',89 from dual union all
select 59921,'1986-Y163','1986-07-23 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 59922,'1986-Y164','1986-07-23 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 59923,'1986-Y165','1986-07-23 15:22:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 59924,'1986-Y166','1986-07-23 17:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 59925,'1986-Y167','1986-07-25 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 59926,'1986-Y168','1986-07-25 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 59927,'1986-Y169','1986-07-27 17:00:00','suborbital rocket','success',1281,'NASA 15.239UU','','','','Aeron',1047,'',80 from dual union all
select 59928,'1986-Y17','1986-01-19 12:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 59929,'1986-Y170','1986-07-29 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 5993,'1959-S23','1959-03-10 18:38:00','suborbital rocket','success',1490,'AGI17','Na release','','','Aeronomy',668,'',124 from dual union all
select 59930,'1986-Y171','1986-07-29 11:42:00','suborbital rocket','success',1281,'NASA 15.241UU','','','','Aeron',1047,'',80 from dual union all
select 59931,'1986-Y172','1986-07-29 17:58:00','suborbital rocket','success',1281,'NASA 15.242UU','','','','Aeron',1047,'',80 from dual union all
select 59932,'1986-Y173','1986-07-29 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 59933,'1986-Y174','1986-07-30 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 59934,'1986-Y175','1986-08-01 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 59935,'1986-Y176','1986-08-02 12:20:00','suborbital rocket','success',1281,'NASA 15.243UU','','','','Aeron',1047,'',80 from dual union all
select 59936,'1986-Y177','1986-08-02 16:50:00','suborbital rocket','success',1281,'NASA 15.240UU','','','','Aeron',1047,'',80 from dual union all
select 59937,'1986-Y178','1986-08-04 12:26:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 59938,'1986-Y179','1986-08-05 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 59939,'1986-Y18','1986-01-20 12:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 5994,'1959-S230','1959-11-01 21:23:00','military missile','success',1016,'No. 267431','8K71 LKI III-14','','','Test',480,'',1350 from dual union all
select 59940,'1986-Y180','1986-08-06 12:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 59941,'1986-Y181','1986-08-06 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 59942,'1986-Y182','1986-08-06 15:06:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 59943,'1986-Y183','1986-08-08 14:47:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 59944,'1986-Y184','1986-08-09 12:25:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 59945,'1986-Y185','1986-08-12 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 59946,'1986-Y186','1986-08-13 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',89 from dual union all
select 59947,'1986-Y187','1986-08-14 08:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1755,'VOKV',88 from dual union all
select 59948,'1986-Y188','1986-08-14 10:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1755,'VOKV',88 from dual union all
select 59949,'1986-Y189','1986-08-14 13:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1755,'VOKV',88 from dual;
insert into LAUNCH
select 5995,'1959-S231','1959-11-03 00:00:00','military missile','success',1367,'238','R&D Series IV','','','Meteorites',234,'',520 from dual union all
select 59950,'1986-Y19','1986-01-21 22:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 59951,'1986-Y190','1986-08-14 17:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 59952,'1986-Y191','1986-08-17 08:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1755,'VOKV',88 from dual union all
select 59953,'1986-Y192','1986-08-17 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1755,'VOKV',87 from dual union all
select 59954,'1986-Y193','1986-08-19 08:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1755,'VOKV',86 from dual union all
select 59955,'1986-Y194','1986-08-19 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1755,'VOKV',87 from dual union all
select 59956,'1986-Y195','1986-08-19 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1755,'VOKV',88 from dual union all
select 59957,'1986-Y196','1986-08-20 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 59958,'1986-Y197','1986-08-20 14:53:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 59959,'1986-Y198','1986-08-20 15:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 5996,'1959-S232','1959-11-04 21:37:00','military missile','failure',154,'28D','Mk 3-1','R&D','','Test',228,'',1800 from dual union all
select 59960,'1986-Y199','1986-08-20 16:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 59961,'1986-Y20','1986-01-22 14:10:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 59962,'1986-Y200','1986-08-20 16:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 59963,'1986-Y201','1986-08-25 14:32:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 59964,'1986-Y202','1986-08-26 20:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 59965,'1986-Y203','1986-08-27 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 59966,'1986-Y204','1986-08-27 14:36:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 59967,'1986-Y205','1986-08-28 04:30:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 59968,'1986-Y206','1986-08-31 20:20:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 59969,'1986-Y207','1986-09-01 14:32:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',87 from dual union all
select 5997,'1959-S233','1959-11-05 00:38:00','military missile','success',638,'CM-33','R&D','','','Test',280,'',500 from dual union all
select 59970,'1986-Y208','1986-09-03 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 59971,'1986-Y209','1986-09-03 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 59972,'1986-Y21','1986-01-22 11:00:00','suborbital rocket','success',728,'STRATWARM','ISRO 8.848','','','Aeron',1467,'',83 from dual union all
select 59973,'1986-Y210','1986-09-03 14:17:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 59974,'1986-Y211','1986-09-09 20:20:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 59975,'1986-Y212','1986-09-10 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 59976,'1986-Y213','1986-09-10 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 59977,'1986-Y214','1986-09-10 14:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 59978,'1986-Y215','1986-09-17 13:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',87 from dual union all
select 59979,'1986-Y216','1986-09-17 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 5998,'1959-S234','1959-11-05 01:10:00','suborbital rocket','success',366,'','RW-147','','','Aeron',1490,'',84 from dual union all
select 59980,'1986-Y217','1986-09-18 11:36:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 59981,'1986-Y218','1986-09-23 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 59982,'1986-Y219','1986-09-24 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 59983,'1986-Y22','1986-01-23 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 59984,'1986-Y220','1986-09-24 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 59985,'1986-Y221','1986-09-30 16:19:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',94 from dual union all
select 59986,'1986-Y222','1986-09-30 20:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 59987,'1986-Y223','1986-12-05 02:55:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',94 from dual union all
select 59988,'1986-Y224','1986-12-17 16:07:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1895,'',97 from dual union all
select 59989,'1986-Y225','1986-01-15 11:15:00','suborbital rocket','success',728,'STRATWARM','ISRO 8.846','','','Aeron',1467,'',85 from dual union all
select 5999,'1959-S235','1959-11-06 01:13:00','suborbital rocket','success',366,'','RW-148','','','Aeron',1490,'',83 from dual union all
select 59990,'1986-Y226','1986-01-20 11:00:00','suborbital rocket','unknown',728,'STRATWARM','ISRO 8.847','','','Aeron',1467,'',85 from dual union all
select 59991,'1986-Y227','1986-01-24 13:30:00','suborbital rocket','unknown',728,'STRATWARM','ISRO 8.849','','','Aeron',1467,'',85 from dual union all
select 59992,'1986-Y228','1986-01-27 11:44:00','suborbital rocket','unknown',728,'STRATWARM','ISRO 8.850','','','Aeron',1467,'',85 from dual union all
select 59993,'1986-Y229','1986-02-17 11:35:00','suborbital rocket','unknown',728,'STRATWARM','ISRO 8.856','','','Aeron',1467,'',85 from dual union all
select 59994,'1986-Y23','1986-01-23 23:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 59995,'1986-Y230','1986-02-20 12:00:00','suborbital rocket','unknown',728,'STRATWARM','ISRO 8.858','','','Aeron',1467,'',85 from dual union all
select 59996,'1986-Y231','1986-02-24 13:47:00','suborbital rocket','unknown',728,'STRATWARM','ISRO 8.860','','','Aeron',1467,'',85 from dual union all
select 59997,'1986-Y232','1986-10-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59998,'1986-Y233','1986-10-08 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 59999,'1986-Y234','1986-10-22 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 6,'1942-M01','1942-11-09 00:00:00','military missile','success',8,'6','','','','Test',675,'',67 from dual union all
select 60,'1944-A24','1944-12-01 19:52:00','military missile','failure',1470,'19850','Ma275','','','Test',661,'',1 from dual union all
select 600,'1945-S76','1945-03-17 00:00:00','military missile','success',1470,'','Remagen','','','Weapon',1736,'',90 from dual union all
select 6000,'1959-S236','1959-11-06 16:36:00','suborbital rocket','success',42,'AA10.187C','Utah ne probe','','','Ionos',428,'',393 from dual union all
select 60000,'1986-Y235','1986-10-29 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 60001,'1986-Y236','1986-11-03 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 60002,'1986-Y237','1986-11-12 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 60003,'1986-Y238','1986-11-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 60004,'1986-Y239','1986-11-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 60005,'1986-Y24','1986-01-26 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 60006,'1986-Y240','1986-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 60007,'1986-Y241','1986-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 60008,'1986-Y242','1986-12-01 00:00:00','suborbital rocket','unknown',725,'','','','','Aeron',1467,'',80 from dual union all
select 60009,'1986-Y243','1986-07-26 22:14:30','suborbital rocket','success',1284,'PWN-12A Robin','','','','Aeron',409,'',116 from dual union all
select 6001,'1959-S237','1959-11-06 07:25:00','suborbital rocket','success',30,'NN2.78C','Airglow','','','Aeron',1869,'',142 from dual union all
select 60010,'1986-Y244','1986-07-26 23:28:10','suborbital rocket','success',1284,'PWN-12A Robin','','','','Aeron',409,'',110 from dual union all
select 60011,'1986-Y245','1986-07-27 00:06:00','suborbital rocket','success',1284,'PWN-12A Robin','','','','Aeron',409,'',110 from dual union all
select 60012,'1986-Y246','1986-07-27 00:50:00','suborbital rocket','success',1284,'PWN-12A Robin','PWN-12A Robin','','','Aeron',409,'',118 from dual union all
select 60013,'1986-Y247','1986-07-27 22:14:30','suborbital rocket','success',1284,'PWN-12A Robin','PWN-12A Robin','','','Aeron',409,'',120 from dual union all
select 60014,'1986-Y248','1986-07-27 23:16:00','suborbital rocket','success',1284,'PWN-12A Robin','PWN-12A Robin','','','Aeron',409,'',117 from dual union all
select 60015,'1986-Y249','1986-02-03 13:30:00','suborbital rocket','unknown',728,'STRATWARM','ISRO 8.853','','','Aeron',1467,'',85 from dual union all
select 60016,'1986-Y25','1986-01-28 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 60017,'1986-Y26','1986-01-29 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 60018,'1986-Y27','1986-01-29 14:49:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 60019,'1986-Y28','1986-01-30 11:00:00','suborbital rocket','unknown',728,'STRATWARM','ISRO 8.852','','','Aeron',1467,'',82 from dual union all
select 6002,'1959-S238','1959-11-10 12:00:00','sounding rocket','success',1267,'OB11.01','Ion density','','','Test/Ionos',1821,'',1800 from dual union all
select 60020,'1986-Y29','1986-02-02 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 60021,'1986-Y30','1986-02-03 12:27:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',88 from dual union all
select 60022,'1986-Y31','1986-02-04 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 60023,'1986-Y32','1986-02-05 11:06:00','suborbital rocket','unknown',728,'STRATWARM','ISRO 8.854','','','Aeron',1467,'',85 from dual union all
select 60024,'1986-Y33','1986-02-05 12:35:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60025,'1986-Y34','1986-02-05 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 60026,'1986-Y35','1986-02-10 12:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60027,'1986-Y36','1986-02-11 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 60028,'1986-Y37','1986-02-12 12:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 60029,'1986-Y38','1986-02-12 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 6003,'1959-S239','1959-11-12 19:24:00','military missile','success',1367,'181','IWST','','BEACH BUGGY','Test',1674,'',520 from dual union all
select 60030,'1986-Y39','1986-02-13 14:30:00','suborbital rocket','unknown',728,'STRATWARM','ISRO 8.855','','','Aeron',1467,'',84 from dual union all
select 60031,'1986-Y40','1986-02-16 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 60032,'1986-Y41','1986-02-17 12:34:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60033,'1986-Y42','1986-02-19 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 60034,'1986-Y43','1986-02-20 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 60035,'1986-Y44','1986-02-21 13:47:00','suborbital rocket','unknown',728,'STRATWARM','ISRO 8.859','','','Aeron',1467,'',86 from dual union all
select 60036,'1986-Y45','1986-02-23 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 60037,'1986-Y46','1986-02-24 13:18:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual;
insert into LAUNCH
select 60038,'1986-Y47','1986-02-24 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 60039,'1986-Y48','1986-02-26 13:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',87 from dual union all
select 6004,'1959-S24','1959-03-12 05:44:00','suborbital rocket','success',1490,'AGI16','Na release','','','Aeronomy',668,'',174 from dual union all
select 60040,'1986-Y49','1986-02-26 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 60041,'1986-Y50','1986-02-26 18:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 60042,'1986-Y51','1986-02-27 11:36:00','suborbital rocket','unknown',728,'STRATWARM','ISRO 8.861','','','Aeron',1467,'',82 from dual union all
select 60043,'1986-Y52','1986-03-02 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 60044,'1986-Y53','1986-03-04 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 60045,'1986-Y54','1986-03-04 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 60046,'1986-Y55','1986-03-05 13:25:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 60047,'1986-Y56','1986-03-05 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 60048,'1986-Y57','1986-03-10 15:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 60049,'1986-Y58','1986-03-11 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 6005,'1959-S240','1959-11-14 22:38:00','suborbital rocket','success',42,'AA10.165C','GRD sphere','','','Aeron',428,'',138 from dual union all
select 60050,'1986-Y59','1986-03-12 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60051,'1986-Y60','1986-03-12 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 60052,'1986-Y61','1986-03-13 13:26:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 60053,'1986-Y62','1986-03-16 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 60054,'1986-Y63','1986-03-17 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60055,'1986-Y64','1986-03-18 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 60056,'1986-Y65','1986-03-19 11:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 60057,'1986-Y66','1986-03-19 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',89 from dual union all
select 60058,'1986-Y67','1986-03-23 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 60059,'1986-Y68','1986-03-24 13:02:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 6006,'1959-S241','1959-11-18 19:30:00','suborbital rocket','success',838,'AA8.241C','','','','Ionos',394,'',206 from dual union all
select 60060,'1986-Y69','1986-03-25 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 60061,'1986-Y70','1986-03-26 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 60062,'1986-Y71','1986-03-26 14:47:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 60063,'1986-Y72','1986-03-27 11:37:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 60064,'1986-Y73','1986-03-30 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 60065,'1986-Y74','1986-03-31 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',87 from dual union all
select 60066,'1986-Y75','1986-04-02 11:37:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 60067,'1986-Y76','1986-04-02 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60068,'1986-Y77','1986-04-02 16:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',97 from dual union all
select 60069,'1986-Y78','1986-04-02 16:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 6007,'1959-S242','1959-11-18 22:17:00','suborbital rocket','success',838,'NASA 03.15CA','GCA Release 3 Na','','','Aeronomy',1821,'',250 from dual union all
select 60070,'1986-Y79','1986-04-06 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 60071,'1986-Y80','1986-04-07 14:17:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60072,'1986-Y81','1986-04-08 22:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 60073,'1986-Y82','1986-04-09 11:36:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 60074,'1986-Y83','1986-04-09 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 60075,'1986-Y84','1986-04-09 14:31:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',90 from dual union all
select 60076,'1986-Y85','1986-04-09 15:45:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',97 from dual union all
select 60077,'1986-Y86','1986-04-13 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 60078,'1986-Y87','1986-04-14 14:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 60079,'1986-Y88','1986-04-15 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 6008,'1959-S243','1959-11-19 00:00:00','military missile','success',1367,'244','R&D Series IV','','','Test',234,'',520 from dual union all
select 60080,'1986-Y89','1986-04-16 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 60081,'1986-Y90','1986-04-16 14:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 60082,'1986-Y91','1986-04-17 11:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 60083,'1986-Y92','1986-04-21 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 60084,'1986-Y93','1986-04-22 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 60085,'1986-Y94','1986-04-23 14:35:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',89 from dual union all
select 60086,'1986-Y95','1986-04-23 14:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 60087,'1986-Y96','1986-04-24 11:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 60088,'1986-Y97','1986-04-27 10:20:00','suborbital rocket','unknown',728,'','','','','Aeron',1383,'SHIR',82 from dual union all
select 60089,'1986-Y98','1986-04-27 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 6009,'1959-S244','1959-11-19 01:31:00','military missile','success',638,'AM-25','R&D','','','Test',249,'',500 from dual union all
select 60090,'1986-Y99','1986-04-28 14:46:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 60091,'1987-001','1987-01-05 01:20:00','orbital','success',1448,'51086-105','Meteor-2','Meteor-2 No. 17','','Sat',470,'',null from dual union all
select 60092,'1987-002','1987-01-09 12:38:04','orbital','success',1238,'78028  -358','Kosmos-1811','Kobal''t','','Sat',562,'',null from dual union all
select 60093,'1987-003','1987-01-14 09:05:00','orbital','success',1448,'45086-101','Kosmos-1812','Tselina-D','','Sat',470,'',null from dual union all
select 60094,'1987-004','1987-01-15 11:20:00','orbital','success',1238,'77011  -334','Kosmos-1813','Oblik','','Sat',474,'',null from dual union all
select 60095,'1987-005','1987-01-16 06:06:23','orbital','success',1242,'B 15000-011','Progress-27','Progress 7K-TG No. 135','','Sat',480,'',null from dual union all
select 60096,'1987-006','1987-01-21 09:10:00','orbital','success',680,'47172-361','Kosmos-1814','Strela-2M','','Sat',457,'',null from dual union all
select 60097,'1987-007','1987-01-22 07:00:00','orbital','success',680,'65048-415','Kosmos-1815','Romb','','Sat',622,'',null from dual union all
select 60098,'1987-008','1987-01-22 16:06:00','orbital','success',805,'77046  -671','Molniya-3','Molniya-3 No. 42','','Sat',473,'',null from dual union all
select 60099,'1987-009','1987-01-29 06:14:57','orbital','success',680,'65096-172','Kosmos-1816','Tsikada','','Sat',457,'',null from dual union all
select 601,'1945-S77','1945-03-17 00:00:00','military missile','success',1470,'','Remagen','','','Weapon',1736,'',90 from dual union all
select 6010,'1959-S245','1959-11-19 07:44:00','suborbital rocket','failure',30,'AA3.282C','B field profile','','','Ionos',428,'',106 from dual union all
select 60100,'1987-010','1987-01-30 09:19:00','orbital','failure',958,'341-01','Kosmos-1817','Ekran-M No. 11L','','Sat',554,'',254 from dual union all
select 60101,'1987-011','1987-02-01 23:30:00','orbital','success',1446,'51076-105','Kosmos-1818','Plazma-A No. 1','','Sat',589,'',null from dual union all
select 60102,'1987-012','1987-02-05 06:30:00','orbital','success',820,'M-3S2-3','Ginga','ASTRO-C','','Sat',837,'',null from dual union all
select 60103,'1987-013','1987-02-05 21:38:16','orbital','success',1242,'I 15000-014','Soyuz TM-2','Soyuz 7K-STM No. 52','','Sat',480,'',null from dual union all
select 60104,'1987-014','1987-02-07 10:30:00','orbital','success',1238,'76048  -350','Kosmos-1819','Oblik','','Sat',475,'',null from dual union all
select 60105,'1987-015','1987-02-12 06:40:00','orbital','success',1405,'34B-51 (3B-66)','USA 21       [SDS7]','QUASAR 7?','','Sat',1778,'',null from dual union all
select 60106,'1987-016','1987-02-14 08:30:00','orbital','success',1537,'45041-504','Kosmos-1820','EPN No. 03.695','','Sat',567,'',null from dual union all
select 60107,'1987-017','1987-02-18 13:53:20','orbital','success',680,'65072-354','Kosmos-1821','Parus','','Sat',457,'',null from dual union all
select 60108,'1987-018','1987-02-19 01:23:00','orbital','success',824,'N-16(F)','Momo','MOS-1','','Sat',1474,'',null from dual union all
select 60109,'1987-019','1987-02-19 10:15:00','orbital','success',1238,'78028  -357','Kosmos-1822','Oblik','','Sat',462,'',null from dual union all
select 6011,'1959-S246','1959-11-19 10:36:00','suborbital rocket','success',839,'AA6.163C','Diurnal density','','','Aeronomy',428,'',119 from dual union all
select 60110,'1987-020','1987-02-20 04:43:00','orbital','success',1448,'67076-106','Kosmos-1823','Musson No. 19','','Sat',470,'',null from dual union all
select 60111,'1987-021','1987-02-26 13:30:00','orbital','success',1238,'78056  -914','Kosmos-1824','Kobal''t','','Sat',473,'',null from dual union all
select 60112,'1987-022','1987-02-26 23:05:00','orbital','success',394,'D179','GOES 7','GOES H','','Sat',233,'',null from dual union all
select 60113,'1987-023','1987-03-03 11:14:05','orbital','success',1242,'I 15000-016','Progress-28','Progress 7K-TG No. 137','','Sat',480,'',null from dual union all
select 60114,'1987-024','1987-03-03 15:03:00','orbital','success',1448,'51076-108','Kosmos-1825','Tselina-D','','Sat',470,'',null from dual union all
select 60115,'1987-025','1987-03-11 10:25:00','orbital','success',1238,'78028  -359','Kosmos-1826','Oblik','','Sat',462,'',null from dual union all
select 60116,'1987-026','1987-03-13 13:11:56','orbital','success',1448,'67058-408','Kosmos-1827/1832','Strela-3','','Sat',470,'',null from dual union all
select 60117,'1987-027','1987-03-18 08:30:00','orbital','success',1537,'51038-401','Kosmos-1833','EPN No. 03.694','','Sat',567,'',null from dual union all
select 60118,'1987-028','1987-03-19 03:54:00','orbital','success',956,'323-01','Raduga','Gran'' No. 31L','','Sat',554,'',null from dual union all
select 60119,'1987-029','1987-03-20 22:22:00','orbital','success',393,'D182','Palapa B2P','Palapa B2P','','Sat',234,'',null from dual union all
select 6012,'1959-S247','1959-11-19 10:51:00','suborbital rocket','success',838,'NASA 03.16CA','GCA Release 4 Na','','','Aeronomy',1821,'',255 from dual union all
select 60120,'1987-030','1987-03-31 00:06:16','orbital','success',949,'336-02','Kvant','TsM-e 37Ke No. 010','','Sat',553,'',null from dual union all
select 60121,'1987-031','1987-04-08 03:51:21','orbital','success',1446,'45029-809','Kosmos-1834','US-PM','','Sat',589,'',null from dual union all
select 60122,'1987-032','1987-04-09 11:44:00','orbital','success',1238,'77011  -918','Kosmos-1835','Kobal''t','','Sat',480,'',null from dual union all
select 60123,'1987-033','1987-04-16 06:18:00','orbital','success',1238,'77011  -209','Kosmos-1836','Terilen','','Sat',480,'',null from dual union all
select 60124,'1987-034','1987-04-21 15:14:17','orbital','success',1242,'I 15000-015','Progress-29','Progress 7K-TG No. 127','','Sat',480,'',null from dual union all
select 60125,'1987-035','1987-04-22 09:10:01','orbital','success',1238,'78028  -356','Kosmos-1837','Oblik','','Sat',474,'',null from dual union all
select 60126,'1987-036','1987-04-24 12:42:54','orbital','failure',958,'335-01','Kosmos-1838/1840','Uragan No. 30L','','Sat',554,'',17494 from dual union all
select 60127,'1987-037','1987-04-24 16:59:59','orbital','success',1238,'I 15000-510','Kosmos-1841','Foton No. 3L','','Sat',473,'',null from dual;
insert into LAUNCH
select 60128,'1987-038','1987-04-27 00:00:00','orbital','success',1448,'51058-407','Kosmos-1842','Tselina-D','','Sat',470,'',null from dual union all
select 60129,'1987-039','1987-05-05 09:15:00','orbital','success',1238,'78028  -355','Kosmos-1843','Oblik','','Sat',562,'',null from dual union all
select 6013,'1959-S248','1959-11-20 17:11:00','military missile','success',928,'A1X-3','','','','Test',251,'',500 from dual union all
select 60130,'1987-040','1987-05-11 14:45:00','orbital','success',956,'338-02','Gorizont','Gorizont No. 23L','','Sat',553,'',null from dual union all
select 60131,'1987-041','1987-05-13 05:40:00','orbital','success',1537,'45058-403','Kosmos-1844','Tselina-2','','Sat',567,'',null from dual union all
select 60132,'1987-042','1987-05-13 06:00:00','orbital','success',1238,'78028  -361','Kosmos-1845','Oblik','','Sat',562,'',null from dual union all
select 60133,'1987-043','1987-05-15 15:45:00','orbital','success',170,'6005H','USA 22-25   [NOSS9]','PARCAE 9','','Sat',1775,'',null from dual union all
select 60134,'1987-044','1987-05-19 04:02:10','orbital','success',1242,'I 15000-018','Progress-30','Progress 7K-TG No. 128','','Sat',480,'',null from dual union all
select 60135,'1987-045','1987-05-21 07:44:59','orbital','success',1238,'77015  -104','Kosmos-1846','Resurs-F1 14F40 No. 104','','Sat',475,'',null from dual union all
select 60136,'1987-046','1987-05-26 13:39:59','orbital','success',1238,'76048  -929','Kosmos-1847','Kobal''t','','Sat',462,'',null from dual union all
select 60137,'1987-047','1987-05-28 12:45:00','orbital','success',1238,'77015  -364','Kosmos-1848','Oblik','','Sat',475,'',null from dual union all
select 60138,'1987-048','1987-06-04 18:50:23','orbital','success',803,'78028  -511','Kosmos-1849','Oko','','Sat',462,'',null from dual union all
select 60139,'1987-049','1987-06-09 14:45:00','orbital','success',680,'47196-176','Kosmos-1850','Strela-2M','','Sat',456,'',null from dual union all
select 6014,'1959-S249','1959-11-20 10:51:00','suborbital rocket','success',838,'NASA 03.17CA','GCA Release 5 Na','','','Aeronomy',1821,'',236 from dual union all
select 60140,'1987-050','1987-06-12 07:40:28','orbital','success',803,'76048  -507','Kosmos-1851','Oko','','Sat',475,'',null from dual union all
select 60141,'1987-051','1987-06-16 17:51:00','orbital','success',680,'65026-184','Kosmos-1852/1859','Strela-1M','','Sat',456,'',null from dual union all
select 60142,'1987-052','1987-06-18 21:33:00','orbital','success',1446,'51029-810','Kosmos-1860','US-A ROCh','','Sat',589,'',null from dual union all
select 60143,'1987-053','1987-06-20 02:34:00','orbital','success',155,'59E','USA 26 (DMSP F8)','DMSP Block 5D-2 S-9','','Sat',1776,'',null from dual union all
select 60144,'1987-054','1987-06-23 07:37:58','orbital','success',680,'65026-202','Kosmos-1861','Tsikada/RS-10/RS-11','','Sat',456,'',null from dual union all
select 60145,'1987-055','1987-07-01 19:35:00','orbital','success',1448,'51061-505','Kosmos-1862','Tselina-D','','Sat',470,'',null from dual union all
select 60146,'1987-056','1987-07-04 12:25:00','orbital','success',1238,'76048  -354','Kosmos-1863','Oblik','','Sat',473,'',null from dual union all
select 60147,'1987-057','1987-07-06 21:59:27','orbital','success',680,'53796-177','Kosmos-1864','Parus','','Sat',457,'',null from dual union all
select 60148,'1987-058','1987-07-08 10:59:00','orbital','success',1238,'76021  -446','Kosmos-1865','Siluet No. 7','','Sat',480,'',null from dual union all
select 60149,'1987-059','1987-07-09 16:10:00','orbital','success',1238,'78028  -931','Kosmos-1866','Kobal''t','','Sat',462,'',null from dual union all
select 6015,'1959-S25','1959-03-12 10:50:00','military missile','success',239,'BK.03','H02/Proving Trial','','','Test',1843,'',537 from dual union all
select 60150,'1987-060','1987-07-10 15:35:00','orbital','success',1446,'51029-813','Kosmos-1867','Plazma-A No. 2','','Sat',589,'',null from dual union all
select 60151,'1987-061','1987-07-14 14:00:01','orbital','success',680,'47196-161','Kosmos-1868','Taifun-1B','','Sat',457,'',null from dual union all
select 60152,'1987-062','1987-07-16 04:25:00','orbital','success',1448,'67038-402','Kosmos-1869','Okean-O1 NKhM No. 4','','Sat',470,'',null from dual union all
select 60153,'1987-063','1987-07-22 01:59:17','orbital','success',1242,'I 15000-019','Soyuz TM-3','Soyuz 7K-STM No. 53','','Sat',480,'',null from dual union all
select 60154,'1987-064','1987-07-25 09:00:00','orbital','success',949,'347-01','Kosmos-1870','Mech-K No. 304','','Sat',554,'',null from dual union all
select 60155,'1987-065','1987-08-01 03:59:59','orbital','success',1537,'45095-601/11L','Kosmos-1871','EPN No. 03.695','','Sat',567,'',null from dual union all
select 60156,'1987-066','1987-08-03 20:44:11','orbital','success',1242,'I 15000-017','Progress-31','Progress 7K-TG No. 138','','Sat',480,'',null from dual union all
select 60157,'1987-067','1987-08-05 06:39:00','orbital','success',301,'Y6       (15)','FSW','FSW No. 10','','Sat',810,'',null from dual union all
select 60158,'1987-068','1987-08-18 02:27:00','orbital','success',1448,'51041-508','Meteor-2','Meteor-2 No. 21','','Sat',469,'',null from dual union all
select 60159,'1987-069','1987-08-19 06:59:59','orbital','success',1238,'77011  -839','Kosmos-1872','Oblik','','Sat',475,'',null from dual union all
select 6016,'1959-S250','1959-11-20 18:16:00','suborbital rocket','success',839,'AA6.164C','Diurnal density','','','Aeronomy',428,'',119 from dual union all
select 60160,'1987-070','1987-08-27 09:20:00','orbital','success',524,'H-17(F)','Kiku 5','ETS-5','','Sat',1474,'',null from dual union all
select 60161,'1987-071','1987-08-28 08:20:00','orbital','success',1537,'51058-404/10L','Kosmos-1873','EPN No. 03.695','','Sat',567,'',null from dual union all
select 60162,'1987-072','1987-09-03 10:24:59','orbital','success',1238,'76021  -207','Kosmos-1874','Oblik','','Sat',475,'',null from dual union all
select 60163,'1987-073','1987-09-03 19:26:00','orbital','success',956,'337-02','Ekran','Ekran No. 29L','','Sat',553,'',null from dual union all
select 60164,'1987-074','1987-09-07 23:50:09','orbital','success',1448,'67095-603','Kosmos-1875/1880','Strela-3','','Sat',469,'',null from dual union all
select 60165,'1987-075','1987-09-09 07:15:00','orbital','success',301,'Y7       (16)','FSW-1','FSW-1 No. 1','','Sat',810,'',null from dual union all
select 60166,'1987-076','1987-09-11 02:06:00','orbital','success',1238,'78028  -211','Kosmos-1881','Terilen','','Sat',480,'',null from dual union all
select 60167,'1987-077','1987-09-15 10:30:00','orbital','success',1238,'76048  -210','Kosmos-1882','Resurs-F1 14F40 No. 107','','Sat',475,'',null from dual union all
select 60168,'1987-078','1987-09-16 00:45:28','orbital','success',93,'V19 L19','Aussat K3/ECS 4','Aussat K3','','Sat',344,'',null from dual union all
select 60169,'1987-079','1987-09-16 02:53:31','orbital','success',958,'339-02','Kosmos-1883/1885','Uragan No. 33L','','Sat',554,'',null from dual union all
select 6017,'1959-S251','1959-11-20 21:06:00','military missile','success',1016,'I2-1T','8K71 LKI III-15','','','Test',480,'',1350 from dual union all
select 60170,'1987-080','1987-09-16 19:22:00','orbital','success',1135,'S204C','SOOS 2','NNS O-27','','Sat',1779,'',null from dual union all
select 60171,'1987-081','1987-09-17 14:59:59','orbital','success',1238,'78028  -853','Kosmos-1886','Kobal''t','','Sat',473,'',null from dual union all
select 60172,'1987-082','1987-09-23 23:43:54','orbital','success',1242,'L 15000-021','Progress-32','Progress 7K-TG No. 139','','Sat',480,'',null from dual union all
select 60173,'1987-083','1987-09-29 12:49:59','orbital','success',1238,'L 15000-543','Kosmos-1887','Bion No. 8','','Sat',473,'',null from dual union all
select 60174,'1987-084','1987-10-01 17:09:00','orbital','success',958,'328-02','Kosmos-1888','Geizer No. 15L','','Sat',553,'',null from dual union all
select 60175,'1987-085','1987-10-09 08:30:00','orbital','success',1238,'77015  -368','Kosmos-1889','Oblik','','Sat',480,'',null from dual union all
select 60176,'1987-086','1987-10-10 21:48:30','orbital','success',1446,'67029-814','Kosmos-1890','US-PM','','Sat',589,'',null from dual union all
select 60177,'1987-087','1987-10-14 12:35:33','orbital','success',680,'53726-186','Kosmos-1891','Parus','','Sat',460,'',null from dual union all
select 60178,'1987-088','1987-10-20 09:09:00','orbital','success',1448,'51038-401','Kosmos-1892','Tselina-D','','Sat',469,'',null from dual union all
select 60179,'1987-089','1987-10-22 14:24:57','orbital','success',1238,'76035  -945','Kosmos-1893','Kobal''t','','Sat',462,'',null from dual union all
select 6018,'1959-S252','1959-11-24 19:48:00','military missile','success',154,'15D','Mk 3-1','R&D','','Test',228,'',1800 from dual union all
select 60180,'1987-090','1987-10-26 21:32:00','orbital','success',1406,'34D-15 (04D-8)','USA 27       [IC2]','Improved CRYSTAL 2102','','Sat',1777,'',null from dual union all
select 60181,'1987-091','1987-10-28 15:15:00','orbital','success',958,'325-01','Kosmos-1894','Oko-S F3','','Sat',554,'',null from dual union all
select 60182,'1987-092','1987-11-11 09:04:00','orbital','success',1238,'78028  -211','Kosmos-1895','Oblik','','Sat',503,'',null from dual union all
select 60183,'1987-093','1987-11-14 09:29:00','orbital','success',1238,'77015  -459','Kosmos-1896','Kometa No. 8','','Sat',480,'',null from dual union all
select 60184,'1987-094','1987-11-20 23:47:12','orbital','success',1242,'L 15000-022','Progress-33','Progress 7K-TG No. 140','','Sat',480,'',null from dual union all
select 60185,'1987-095','1987-11-21 02:19:00','orbital','success',92,'V20 L20','TV-SAT','TV-SAT','','Sat',345,'',null from dual union all
select 60186,'1987-096','1987-11-26 13:28:00','orbital','success',958,'330-01','Kosmos-1897','Al''tair No. 12L','','Sat',553,'',null from dual union all
select 60187,'1987-097','1987-11-29 03:27:20','orbital','success',1408,'34D-8  (05D-4)','USA 28       [DSP5]','DSP 5R','','Sat',274,'',null from dual union all
select 60188,'1987-098','1987-12-01 14:15:45','orbital','success',680,'65035-621','Kosmos-1898','Strela-2M','','Sat',460,'',null from dual union all
select 60189,'1987-099','1987-12-07 08:50:00','orbital','success',1238,'76035  -370','Kosmos-1899','Oblik','','Sat',562,'',null from dual union all
select 6019,'1959-S253','1959-11-27 01:12:33','military missile','success',1016,'No. 267433','8K71 LKI III-16','GCh No. I3-33','','Test',480,'',1350 from dual union all
select 60190,'1987-100','1987-12-10 11:30:00','orbital','success',958,'343-01','Raduga','Gran'' No. 32L','','Sat',554,'',null from dual union all
select 60191,'1987-101','1987-12-12 05:40:00','orbital','success',1446,'45041-504','Kosmos-1900','US-AM reactor','','Sat',589,'',null from dual union all
select 60192,'1987-102','1987-12-14 11:29:59','orbital','success',1238,'77015  -943','Kosmos-1901','Kobal''t','','Sat',562,'',null from dual union all
select 60193,'1987-103','1987-12-15 13:30:00','orbital','success',680,'53726-183','Kosmos-1902','Vektor','','Sat',460,'',null from dual union all
select 60194,'1987-104','1987-12-21 11:18:03','orbital','success',1242,'I 15000-020','Soyuz TM-4','Soyuz 7K-STM No. 54','','Sat',480,'',null from dual union all
select 60195,'1987-105','1987-12-21 22:35:42','orbital','success',803,'78028  -514','Kosmos-1903','Oko','','Sat',473,'',null from dual union all
select 60196,'1987-106','1987-12-23 20:22:40','orbital','success',680,'53711-281','Kosmos-1904','Parus','','Sat',460,'',null from dual union all
select 60197,'1987-107','1987-12-25 08:44:59','orbital','success',1238,'77015  -369','Kosmos-1905','Oblik','','Sat',562,'',null from dual union all
select 60198,'1987-108','1987-12-26 11:30:01','orbital','success',1238,'L 15000-547','Kosmos-1906','Resurs-F2 17F42 No. 1','','Sat',462,'',null from dual union all
select 60199,'1987-109','1987-12-27 11:25:00','orbital','success',958,'345-01','Ekran','Ekran-M No. 13L','','Sat',553,'',null from dual union all
select 602,'1945-S78','1945-03-17 00:00:00','military missile','success',1470,'','Remagen','','','Weapon',1736,'',90 from dual union all
select 6020,'1959-S254','1959-11-30 00:00:00','military missile','success',976,'ID9-20T','8K63 UBP-31','','','Test',636,'',402 from dual union all
select 60200,'1987-110','1987-12-29 11:40:00','orbital','success',1238,'78028  -362','Kosmos-1907','Oblik','','Sat',475,'',null from dual union all
select 60201,'1987-A01','1987-01-16 13:15:00','military missile','failure',1006,'51026-325-09','15A18 UBP-21 K','GCh 15F183','','OT',489,'',1000 from dual union all
select 60202,'1987-A02','1987-01-21 20:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',46 from dual union all
select 60203,'1987-A03','1987-01-28 15:15:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',43 from dual union all
select 60204,'1987-A04','1987-02-13 17:01:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',43 from dual union all
select 60205,'1987-A05','1987-03-07 06:39:20','suborbital rocket','failure',231,'NASA 35.018UE','','','','Plasma',1047,'',10 from dual union all
select 60206,'1987-A06','1987-04-16 10:54:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',48 from dual union all
select 60207,'1987-A07','1987-04-16 12:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',37 from dual union all
select 60208,'1987-A08','1987-05-10 09:45:00','military missile','failure',932,'A3TK','CHEVALINE CAF','','','OT',421,'UK S23',5 from dual union all
select 60209,'1987-A09','1987-06-05 00:00:00','military missile','failure',1453,'49217-91-568','15A35 60 RD UBP-4','','','OT',1461,'',0 from dual union all
select 6021,'1959-S255','1959-12-11 00:00:00','military missile','success',1013,'BI1-4','8K51 PRO Target C-1','NIIP-A Target','','Target',321,'',500 from dual union all
select 60210,'1987-A10','1987-06-09 02:20:00','military missile','failure',1005,'51058-404(7L)','15A18M LKI-7','GCh 15F173','','OT',491,'',100 from dual union all
select 60211,'1987-A11','1987-07-08 06:59:59','suborbital rocket','success',1061,'','Chaff,Datasonde','','','Meteo',873,'',43 from dual union all
select 60212,'1987-A12','1987-08-28 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',33 from dual union all
select 60213,'1987-A13','1987-08-30 01:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',37 from dual union all
select 60214,'1987-A14','1987-09-03 14:03:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',36 from dual union all
select 60215,'1987-A15','1987-10-01 03:05:00','military missile','failure',1005,'67058-408(11L)','15A18M LKI-11','GCh 15F173','','OT',487,'',0 from dual union all
select 60216,'1987-A16','1987-10-21 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',20 from dual union all
select 60217,'1987-A17','1987-10-26 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',20 from dual;
insert into LAUNCH
select 60218,'1987-A18','1987-10-29 02:06:00','suborbital rocket','success',1061,'','Chaff,Datasonde','','','Meteo',873,'',48 from dual union all
select 60219,'1987-A19','1987-11-02 15:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',44 from dual union all
select 6022,'1959-S256','1959-11-30 09:58:00','suborbital rocket','success',28,'NRL 30','','','','UV Astron',1895,'',140 from dual union all
select 60220,'1987-A20','1987-11-23 09:10:00','suborbital rocket','failure',1212,'DLR K-GR-153','DLR TEXUS 16','','','Micrograv',416,'',0 from dual union all
select 60221,'1987-A21','1987-12-01 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',35 from dual union all
select 60222,'1987-A22','1987-12-17 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',48 from dual union all
select 60223,'1987-A23','1987-06-15 11:57:00','suborbital rocket','failure',1284,'Sphere','MAC-SINE S-C1/H','','','Aeron',29,'',0 from dual union all
select 60224,'1987-A24','1987-02-11 00:00:00','military missile','failure',983,'','Al Hussein R&D 1','','','Weapon/Test',761,'',10 from dual union all
select 60225,'1987-A25','1987-05-14 16:00:00','atmospheric rocket','success',1305,'BRV Flythrough','MISTY PICTURE 1','','','Test',1895,'',12 from dual union all
select 60226,'1987-A26','1987-05-14 16:00:00','atmospheric rocket','success',1305,'BRV Flythrough','MISTY PICTURE 2','','','Test',1895,'',12 from dual union all
select 60227,'1987-A27','1987-05-14 16:00:00','atmospheric rocket','success',1305,'BRV Flythrough','MISTY PICTURE 3','','','Test',1895,'',12 from dual union all
select 60228,'1987-A28','1987-05-14 16:00:00','atmospheric rocket','success',1305,'BRV Flythrough','MISTY PICTURE 4','','','Test',1895,'',12 from dual union all
select 60229,'1987-A29','1987-05-14 16:00:00','atmospheric rocket','success',1502,'MISTY PICTURE 1-20','Dust Collection x20','','','Meteo',1895,'',20 from dual union all
select 6023,'1959-S257','1959-11-30 10:16:00','suborbital rocket','success',1194,'SL16','QUB Na-3','','','Aeronomy',1839,'',151 from dual union all
select 60230,'1987-A30','1987-05-21 00:00:00','military missile','success',702,'FLAGE Target','','','','Test',1895,'',10 from dual union all
select 60231,'1987-F01','1987-03-24 06:39:00','orbital','failure',22,'ASLV-D1','SROSS A','SROSS-A','','Sat',1360,'',10 from dual union all
select 60232,'1987-F02','1987-03-26 21:22:00','orbital','failure',169,'AC-67','FLTSATCOM 6','FLTSATCOM F6','','Sat',269,'',30 from dual union all
select 60233,'1987-F04','1987-06-18 07:24:59','orbital','failure',1238,'77015  -105','','Resurs-F1 14F40 No. 105','','Sat',474,'',0 from dual union all
select 60234,'1987-M01','1987-01-07 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60235,'1987-M02','1987-01-14 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60236,'1987-M03','1987-01-21 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60237,'1987-M04','1987-01-28 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60238,'1987-M05','1987-02-04 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60239,'1987-M06','1987-02-11 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6024,'1959-S258','1959-11-30 12:35:00','suborbital rocket','success',1194,'SL60','UCL GC-7 Gren/Chaff','','','Aeronomy',1839,'',163 from dual union all
select 60240,'1987-M07','1987-02-18 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60241,'1987-M08','1987-02-25 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60242,'1987-M09','1987-03-04 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60243,'1987-M10','1987-03-11 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60244,'1987-M11','1987-03-18 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60245,'1987-M12','1987-03-25 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60246,'1987-M13','1987-04-01 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60247,'1987-M14','1987-04-08 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60248,'1987-M15','1987-04-15 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60249,'1987-M16','1987-04-22 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6025,'1959-S259','1959-12-01 00:00:00','suborbital rocket','success',1438,'TB I gamma','','','','Reentry Test',1821,'',260 from dual union all
select 60250,'1987-M17','1987-04-29 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60251,'1987-M18','1987-05-06 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60252,'1987-M19','1987-05-14 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60253,'1987-M20','1987-05-20 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60254,'1987-M21','1987-05-27 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60255,'1987-M22','1987-06-03 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60256,'1987-M23','1987-06-10 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60257,'1987-M24','1987-06-17 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60258,'1987-M25','1987-06-24 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60259,'1987-M26','1987-07-01 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6026,'1959-S26','1959-03-12 15:44:00','suborbital rocket','success',49,'AA3.121C','Expt 6.5','','','Solar EUV',640,'',210 from dual union all
select 60260,'1987-M27','1987-07-08 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60261,'1987-M28','1987-07-15 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60262,'1987-M29','1987-07-22 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60263,'1987-M30','1987-07-29 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60264,'1987-M31','1987-08-05 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60265,'1987-M32','1987-08-12 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60266,'1987-M33','1987-08-19 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60267,'1987-M34','1987-08-26 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60268,'1987-M35','1987-09-02 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60269,'1987-M36','1987-09-09 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6027,'1959-S260','1959-12-01 17:00:00','military missile','success',1367,'254','R&D Series IV','','','Test',236,'',520 from dual union all
select 60270,'1987-M37','1987-09-16 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60271,'1987-M38','1987-09-23 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60272,'1987-M39','1987-09-30 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60273,'1987-M40','1987-10-07 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60274,'1987-M41','1987-01-01 00:00:00','suborbital rocket','success',1047,'','','','','Test',932,'',70 from dual union all
select 60275,'1987-M42','1987-10-14 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60276,'1987-M43','1987-10-21 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60277,'1987-M44','1987-11-04 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60278,'1987-M45','1987-11-11 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60279,'1987-M46','1987-11-18 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6028,'1959-S261','1959-12-01 11:36:00','suborbital rocket','success',1194,'SL38','UCL GC-8/Bir-3','','','Aeronomy',1839,'',159 from dual union all
select 60280,'1987-M47','1987-11-25 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60281,'1987-M48','1987-12-02 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60282,'1987-M49','1987-12-09 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60283,'1987-M50','1987-12-16 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60284,'1987-M51','1987-12-23 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60285,'1987-M52','1987-12-30 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60286,'1987-M53','1987-10-28 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 60287,'1987-S01','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60288,'1987-S02','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60289,'1987-S03','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 6029,'1959-S262','1959-12-03 00:00:00','suborbital rocket','success',972,'','','','','Aeron?',1010,'',86 from dual union all
select 60290,'1987-S04','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60291,'1987-S05','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60292,'1987-S06','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60293,'1987-S07','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60294,'1987-S08','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60295,'1987-S09','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60296,'1987-S10','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60297,'1987-S100','1987-06-25 13:20:00','military missile','failure',799,'','Mk 12/12A RV x 3','FOT GT122GB-1','GLORY TRIP 122GB-1','OT',1712,'',1000 from dual union all
select 60298,'1987-S101','1987-07-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60299,'1987-S102','1987-07-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 603,'1945-S79','1945-03-17 00:00:00','military missile','success',1470,'','Remagen','','','Weapon',1736,'',90 from dual union all
select 6030,'1959-S263','1959-12-03 00:00:00','suborbital rocket','success',972,'','','','','Aeron?',1010,'',86 from dual union all
select 60300,'1987-S103','1987-07-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60301,'1987-S104','1987-07-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60302,'1987-S105','1987-07-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60303,'1987-S106','1987-07-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60304,'1987-S107','1987-07-08 04:35:00','military missile','success',796,'','Mk 11 RV','FOT GT148M','GLORY TRIP 148M','OT',1699,'',1300 from dual union all
select 60305,'1987-S108','1987-07-12 13:35:00','military missile','failure',799,'','Mk 12/12A RV x 3','FOT GT123GM-1','GLORY TRIP 123GM-1','OT',1706,'',1000 from dual union all
select 60306,'1987-S109','1987-07-15 01:55:00','suborbital rocket','success',1319,'NASA 33.52GE','GSFC Thunderstorm','','','Plasma',1821,'',177 from dual union all
select 60307,'1987-S11','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual;
insert into LAUNCH
select 60308,'1987-S110','1987-07-20 20:45:00','military missile','success',1443,'D5X-6','R&D Mk 5','','','Test',294,'',1000 from dual union all
select 60309,'1987-S111','1987-07-26 23:22:00','suborbital rocket','success',891,'NASA 30.34UE','WIPP/PSU E','','','Plasma',1821,'',90 from dual union all
select 6031,'1959-S264','1959-12-04 16:20:00','test rocket','success',709,'LJ-2','','','','SC Test',1797,'',89 from dual union all
select 60310,'1987-S112','1987-07-27 06:25:00','suborbital rocket','success',1319,'NASA 33.53GE','WIPP/GSFC Thunder.','','','Plasma',1821,'',177 from dual union all
select 60311,'1987-S113','1987-07-27 12:38:07','military missile','success',919,'','P-506 87 Cape 13','','','OT',231,'',320 from dual union all
select 60312,'1987-S114','1987-07-27 12:57:06','military missile','success',919,'','P-507 87 Cape 14','','','OT',231,'',320 from dual union all
select 60313,'1987-S115','1987-07-27 15:20:04','military missile','success',919,'','P-508 87 Cape 15','','','OT',231,'',320 from dual union all
select 60314,'1987-S116','1987-07-27 15:37:05','military missile','success',919,'-        P2-54','P-509 87 Cape 16','','','OT',231,'',320 from dual union all
select 60315,'1987-S117','1987-07-27 17:27:13','suborbital rocket','success',211,'NASA 27.107US','Colo UVS','','','Solar UV',1895,'',340 from dual union all
select 60316,'1987-S118','1987-08-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60317,'1987-S119','1987-08-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60318,'1987-S12','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60319,'1987-S120','1987-08-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 6032,'1959-S265','1959-12-07 20:13:00','military missile','success',928,'A1X-4','','','','Test',251,'',500 from dual union all
select 60320,'1987-S121','1987-08-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60321,'1987-S122','1987-08-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60322,'1987-S123','1987-08-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60323,'1987-S124','1987-07-31 18:00:03','military missile','success',939,'','DASO','','','OT',421,'SSBN 631',500 from dual union all
select 60324,'1987-S125','1987-07-31 03:53:49','suborbital rocket','success',224,'NASA 36.13UE','WIPP','','','Plasma',1821,'',300 from dual union all
select 60325,'1987-S126','1987-08-07 15:02:09','suborbital rocket','success',211,'NASA 27.112UE','ERIC I','','','Plasma',1821,'',268 from dual union all
select 60326,'1987-S127','1987-08-14 16:29:00','suborbital rocket','success',860,'GL A12.629','COLDR','','','Aeron',1821,'',86 from dual union all
select 60327,'1987-S128','1987-08-15 17:25:00','suborbital rocket','success',224,'NASA 36.021CS','AS&E SAIP XRT','','','Solar',1873,'',300 from dual union all
select 60328,'1987-S129','1987-08-24 03:52:00','military missile','success',1442,'','DASO/JANUS','','','Tech',421,'SSBN 629',1000 from dual union all
select 60329,'1987-S13','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 6033,'1959-S266','1959-12-09 00:10:00','military missile','success',154,'31D','Mk 3-1','R&D','','Test',228,'',1800 from dual union all
select 60330,'1987-S130','1987-08-24 16:30:00','suborbital rocket','success',1212,'DLR W-GR-147','Supernova','','','XR Telescope',1834,'',270 from dual union all
select 60331,'1987-S131','1987-08-26 21:20:00','test rocket','success',647,'','BOR-5 No. 504','','','Reentry Test',621,'',210 from dual union all
select 60332,'1987-S132','1987-08-29 00:00:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 630',1000 from dual union all
select 60333,'1987-S133','1987-08-29 00:00:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 630',1000 from dual union all
select 60334,'1987-S134','1987-08-29 00:00:00','military missile','success',1442,'','FOT','','','OT',421,'SSBN 630',1000 from dual union all
select 60335,'1987-S135','1987-08-29 23:43:00','military missile','success',1442,'','FOT          140','','','OT',421,'SSBN 630',1000 from dual union all
select 60336,'1987-S136','1987-09-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60337,'1987-S137','1987-09-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60338,'1987-S138','1987-09-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60339,'1987-S139','1987-09-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 6034,'1959-S267','1959-12-10 00:08:00','military missile','success',638,'AM-32','R&D','','','Test',280,'',500 from dual union all
select 60340,'1987-S14','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60341,'1987-S140','1987-09-01 07:58:00','military missile','success',799,'','Mk 12/12A RV x 3/IPMS','IPMS','','Reentry Test',1701,'',1300 from dual union all
select 60342,'1987-S141','1987-09-08 22:01:00','military missile','success',1443,'D5X-4','R&D Mk 5','','','Test',294,'',1000 from dual union all
select 60343,'1987-S142','1987-09-12 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 622',500 from dual union all
select 60344,'1987-S143','1987-09-12 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 622',500 from dual union all
select 60345,'1987-S144','1987-09-12 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 622',500 from dual union all
select 60346,'1987-S145','1987-09-12 05:28:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 622',500 from dual union all
select 60347,'1987-S146','1987-09-21 00:00:00','military missile','success',795,'','SENT-2','SENT-2','','Reentry Test',1698,'',1300 from dual union all
select 60348,'1987-S147','1987-09-29 10:00:00','suborbital rocket','success',210,'NASA 27.106UG','UCB EUV Tel','','','EUV Astron',1895,'',340 from dual union all
select 60349,'1987-S148','1987-10-14 13:20:00','military missile','success',1006,'51074-317-07','15A18 UBP-24 K','GCh 15F183','','OT',489,'',1000 from dual union all
select 6035,'1959-S268','1959-12-16 04:50:00','military missile','failure',928,'A1X-6','','','','Test',251,'',100 from dual union all
select 60350,'1987-S149','1987-10-04 00:00:00','suborbital rocket','success',547,'','HAVE JEEP IV-D','','','Reentry Test?',871,'',500 from dual union all
select 60351,'1987-S15','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60352,'1987-S150','1987-10-06 17:02:00','military missile','success',1443,'D5X-8','R&D Mk 5','','','Test',294,'',1000 from dual union all
select 60353,'1987-S151','1987-10-15 10:52:00','suborbital rocket','success',860,'DLR A-NO-148','MAC/E-T1 Bugatti IV','','','Aeron',29,'',127 from dual union all
select 60354,'1987-S152','1987-10-15 10:52:20','suborbital rocket','success',891,'NASA 30.38UE','MAC/Epsilon XRG','','','Aeron',29,'',81 from dual union all
select 60355,'1987-S153','1987-10-15 10:52:40','suborbital rocket','success',860,'NASA 31.63UE','MAC/Epsilon NEFS','','','Aeron',29,'',181 from dual union all
select 60356,'1987-S154','1987-10-16 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 654',500 from dual union all
select 60357,'1987-S155','1987-10-16 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 654',500 from dual union all
select 60358,'1987-S156','1987-10-16 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 654',500 from dual union all
select 60359,'1987-S157','1987-10-16 16:17:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 654',500 from dual union all
select 6036,'1959-S269','1959-12-15 00:00:00','military missile','success',883,'003','','','','Reentry Test',313,'',150 from dual union all
select 60360,'1987-S158','1987-10-21 21:33:00','suborbital rocket','success',860,'DLR A-NO-149  F-75','MAC/E-T2 IOMAS II','','','Aeron',29,'',126 from dual union all
select 60361,'1987-S159','1987-10-21 21:33:20','suborbital rocket','success',860,'DLR A-NO-150  F-77','MAC/E-T3 Bugatti IV','','','Aeron',29,'',126 from dual union all
select 60362,'1987-S16','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60363,'1987-S160','1987-10-21 21:33:40','suborbital rocket','success',860,'NASA 31.66GE','MAC/Epsilon EXERT','','','Aeron',29,'',122 from dual union all
select 60364,'1987-S161','1987-10-21 21:34:00','suborbital rocket','success',891,'NASA 30.36UE','MAC/Epsilon MAE','','','Aeron',29,'',89 from dual union all
select 60365,'1987-S162','1987-10-21 21:34:20','suborbital rocket','success',860,'NASA 31.64GE','MAC/Epsilon NEFS','','','Aeron',29,'',179 from dual union all
select 60366,'1987-S163','1987-10-23 18:05:00','suborbital rocket','success',211,'NASA 27.92US','MSSTA 0','','','Solar X',1895,'',340 from dual union all
select 60367,'1987-S164','1987-10-29 01:48:00','military missile','success',796,'','Mk 11 RV','FOT GT150M','GLORY TRIP 150M','OT',1699,'',1300 from dual union all
select 60368,'1987-S165','1987-10-28 00:21:00','suborbital rocket','success',860,'NASA 31.67GE','MAC/Epsilon EXERT','','','Aeron',29,'',119 from dual union all
select 60369,'1987-S166','1987-11-03 08:25:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT124GM','GLORY TRIP 124GM','OT',1705,'',1300 from dual union all
select 6037,'1959-S27','1959-03-12 18:45:00','suborbital rocket','success',366,'','RW-118','','','Aeron',1490,'',85 from dual union all
select 60370,'1987-S167','1987-11-08 00:00:00','suborbital rocket','failure',1220,'PT-03','','','','Test',330,'',130 from dual union all
select 60371,'1987-S168','1987-11-10 02:10:00','military missile','failure',796,'','Mk 11 RV','FOT GT149M','GLORY TRIP 149M','OT',1702,'',1 from dual union all
select 60372,'1987-S169','1987-11-12 00:21:00','suborbital rocket','success',860,'DLR A-NO-151','MAC/E-T4 Ferdinand74','','','Test/Aeron/Ionos',29,'',126 from dual union all
select 60373,'1987-S17','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60374,'1987-S171','1987-11-12 00:21:20','suborbital rocket','success',860,'DLR A-NO-152','MAC/E-T5 F78/Total11','','','Aeron',29,'',124 from dual union all
select 60375,'1987-S172','1987-11-12 00:21:43','suborbital rocket','success',891,'NASA 30.37UE','MAC/Epsilon MAE','','','Aeron',29,'',87 from dual union all
select 60376,'1987-S173','1987-11-12 00:22:00','suborbital rocket','success',860,'NASA 31.65UE','MAC/Epsilon NEFS','','','Aeron',29,'',181 from dual union all
select 60377,'1987-S174','1987-11-14 14:10:00','suborbital rocket','success',224,'NASA 36.030UH','','','','XR Astron',1838,'',300 from dual union all
select 60378,'1987-S175','1987-11-18 00:00:00','suborbital rocket','success',547,'','HAVE JEEP VIII-A','','','Reentry Test?',871,'',500 from dual union all
select 60379,'1987-S176','1987-11-18 13:06:00','suborbital rocket','success',224,'NASA 36.023UG','FUVS 1','','','UV Astron',1838,'',300 from dual union all
select 6038,'1959-S270','1959-12-16 00:00:00','military missile','success',976,'IO4-20T','8K63 UBP-32','','','Test',636,'',402 from dual union all
select 60380,'1987-S177','1987-12-04 17:05:00','suborbital rocket','success',224,'NASA 36.024UH','CAL XR 15','','','XR Astron',1838,'',300 from dual union all
select 60381,'1987-S178','1987-12-11 13:26:00','military missile','success',1443,'D5X-10','R&D Mk 5','','','Test',294,'',1000 from dual union all
select 60382,'1987-S179','1987-12-11 18:15:00','suborbital rocket','success',224,'NASA 36.38CS','AS&E SAIP XRT','','','Solar SX',1873,'',300 from dual union all
select 60383,'1987-S18','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60384,'1987-S180','1987-12-11 18:45:00','suborbital rocket','success',224,'NASA 36.20DS','HRTS 5','','','Solar',1873,'',300 from dual union all
select 60385,'1987-S181','1987-12-14 01:45:00','suborbital rocket','success',231,'GL A22.703','SPEAR I','','','Active Plasma',1801,'',370 from dual union all
select 60386,'1987-S182','1987-12-22 00:00:00','military missile','success',1042,'48751-209-8L','15Zh61 LKI-16','','','Test',477,'',1000 from dual union all
select 60387,'1987-S183','1987-05-04 04:40:00','suborbital rocket','success',1028,'560.30','SPICE-2','','','Ionos',1361,'',280 from dual union all
select 60388,'1987-S184','1987-09-01 00:00:00','military missile','success',354,'','','','','Test',1494,'',500 from dual union all
select 60389,'1987-S185','1987-07-27 00:00:00','suborbital rocket','success',117,'Queen Match','','','','ABM Tech IR',1362,'',300 from dual union all
select 6039,'1959-S271','1959-12-17 00:00:00','military missile','success',1367,'255','R&D Series IV','','','Meteorites',234,'',520 from dual union all
select 60390,'1987-S186','1987-08-03 00:00:00','military missile','success',73,'','Range test, 615 km','','','Weapon/Test',761,'',100 from dual union all
select 60391,'1987-S187','1987-12-28 00:00:00','military missile','success',73,'','Combat valid. test','','','Weapon/Test',761,'',100 from dual union all
select 60392,'1987-S188','1987-04-21 00:00:00','military missile','failure',983,'','Al Hussein R&D 2','','','Weapon/Test',761,'',10 from dual union all
select 60393,'1987-S189','1987-04-28 00:00:00','military missile','failure',983,'','Al Hussein R&D 3','','','Weapon/Test',761,'',10 from dual union all
select 60394,'1987-S19','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60395,'1987-S190','1987-01-01 00:00:00','military missile','success',983,'Scud B','27 launches vs Iran','','','Weapon',761,'',100 from dual union all
select 60396,'1987-S191','1987-03-01 00:00:00','military missile','success',983,'Scud B','9 launches vs Iran','','','Weapon',761,'',100 from dual union all
select 60397,'1987-S192','1987-11-10 00:00:00','military missile','unknown',917,'AO-233  ASP','P-510 87 Fall 01','','','OT',433,'',180 from dual union all
select 60398,'1987-S193','1987-11-10 00:00:00','military missile','unknown',917,'AO-234  ASP','P-511 87 Fall 02','','','OT',433,'',180 from dual;
insert into LAUNCH
select 60399,'1987-S194','1987-11-25 00:00:00','military missile','unknown',917,'AO-235  ASP','P-512 87 Fall 03','','','OT',433,'',180 from dual union all
select 604,'1945-S80','1945-03-17 00:00:00','military missile','success',1470,'','Remagen','','','Weapon',1736,'',90 from dual union all
select 6040,'1959-S272','1959-12-17 00:03:00','military missile','success',638,'AM-26','R&D','','','Test',249,'',500 from dual union all
select 60400,'1987-S195','1987-11-26 00:00:00','military missile','unknown',917,'AO-236  ASP','P-513 87 Fall 04','','','OT',433,'',180 from dual union all
select 60401,'1987-S196','1987-01-30 00:00:00','military missile','success',918,'OT-33','P-489 87 Spring 01','','Site L-1-2','OT',433,'',320 from dual union all
select 60402,'1987-S197','1987-01-30 00:00:00','military missile','success',918,'OT-34','P-490 87 Spring 01','','Site L-1-3','OT',433,'',320 from dual union all
select 60403,'1987-S198','1987-01-30 00:00:00','military missile','success',918,'OT-35','P-491 87 Spring 01','','Site L-2-1','OT',433,'',320 from dual union all
select 60404,'1987-S199','1987-01-30 00:00:00','military missile','success',918,'OT-37','P-492 87 Spring 01','','Site L-2-3','OT',433,'',320 from dual union all
select 60405,'1987-S20','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60406,'1987-S200','1987-01-30 00:00:00','military missile','success',918,'OT-32','P-493 87 Spring 01','','Site L-1-1','OT',433,'',320 from dual union all
select 60407,'1987-S201','1987-01-01 00:00:00','military missile','success',644,'','Pchela-2','','','Reentry Test',636,'',null from dual union all
select 60408,'1987-S202','1987-01-01 00:00:00','military missile','success',886,'','Flight KY 104 (last)','','','Test',636,'',120 from dual union all
select 60409,'1987-S203','1987-01-01 00:00:00','suborbital rocket','success',1279,'','','','','Reentry Test?',843,'',200 from dual union all
select 6041,'1959-S273','1959-12-18 00:00:00','military missile','success',976,'IP5-10T','8K63 UBP-33','','','Test',636,'',402 from dual union all
select 60410,'1987-S204','1987-12-25 04:58:00','military missile','success',1005,'45095-604(14L)','15A18M LKI-14','GCh 15F173','','OT',487,'',1000 from dual union all
select 60411,'1987-S205','1987-12-03 06:10:00','military missile','success',1453,'49217-91-562','15A35 UBP-37','','','OT',535,'',1000 from dual union all
select 60412,'1987-S206','1987-01-29 00:00:00','military missile','success',1037,'','8K98 UBP-75','','','Test',477,'',1000 from dual union all
select 60413,'1987-S207','1987-05-26 00:00:00','military missile','success',1434,'','15Zh58 K-15','','','Test',477,'',1000 from dual union all
select 60414,'1987-S208','1987-02-11 00:00:00','military missile','success',1434,'','15Zh58 K-14','','','Test',477,'',1000 from dual union all
select 60415,'1987-S209','1987-06-30 00:00:00','military missile','success',1434,'','15Zh58 K-16','','','Test',477,'',1000 from dual union all
select 60416,'1987-S21','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60417,'1987-S210','1987-07-31 00:00:00','military missile','success',1434,'','15Zh58 K-17','','','Test',477,'',1000 from dual union all
select 60418,'1987-S211','1987-12-23 00:00:00','military missile','success',1434,'','15Zh58 K-18','','','Test',477,'',1000 from dual union all
select 60419,'1987-S212','1987-07-14 00:00:00','military missile','success',1434,'','15Zh58 UBP-1','','','Test',477,'',1000 from dual union all
select 6042,'1959-S274','1959-12-19 00:48:00','military missile','success',154,'40D','Mk 2-IIJ-3','R&D','','Test',228,'',1800 from dual union all
select 60420,'1987-S213','1987-12-23 00:00:00','military missile','success',1434,'16L','15Zh58 LKI-16','','','Test',477,'',1000 from dual union all
select 60421,'1987-S214','1987-06-17 00:00:00','military missile','success',1042,'48798-213-08','15Zh61 LKI-14','','','Test',477,'',1000 from dual union all
select 60422,'1987-S215','1987-09-09 00:00:00','military missile','success',1042,'48798-213-09','15Zh61 LKI-15','','','Test',477,'',1000 from dual union all
select 60423,'1987-S216','1987-02-03 00:00:00','military missile','failure',1041,'48798-305-4L','15Zh60 LKI-4','','','Test',477,'',0 from dual union all
select 60424,'1987-S217','1987-07-17 00:00:00','military missile','failure',1041,'48765-401-5L','15Zh60 LKI-5','','','Test',477,'',0 from dual union all
select 60425,'1987-S218','1987-08-27 00:00:00','military missile','success',1041,'48775-402-6L','15Zh60 LKI-6','','','Test',477,'',1000 from dual union all
select 60426,'1987-S219','1987-09-18 00:00:00','military missile','success',1041,'48775-403-7L','15Zh60 LKI-7','','','Test',477,'',1000 from dual union all
select 60427,'1987-S22','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60428,'1987-S220','1987-10-05 00:00:00','military missile','success',1041,'48775-404-11L','15Zh60 LKI-8','','','Test',477,'',1000 from dual union all
select 60429,'1987-S221','1987-11-23 00:00:00','military missile','success',1041,'48775-405-12L','15Zh60 LKI-9','','','Test',477,'',1000 from dual union all
select 6043,'1959-S275','1959-12-20 22:00:00','military missile','success',976,'ID3-24Sh','8K63 ShPU Test 2','','','Test',630,'',402 from dual union all
select 60430,'1987-S222','1987-12-04 00:00:00','military missile','success',1041,'48775-406-13L','15Zh60 LKI-10','','','Test',477,'',1000 from dual union all
select 60431,'1987-S223','1987-01-01 00:00:00','military missile','success',1450,'','15A20 54 RD UBP-11','','','OT',1468,'',1000 from dual union all
select 60432,'1987-S224','1987-01-01 00:00:00','military missile','success',1450,'','15A20 54 RD UBP-12','','','OT',1468,'',1000 from dual union all
select 60433,'1987-S225','1987-01-01 00:00:00','military missile','success',1450,'','15A20 54 RD UBP-13','','','OT',1468,'',1000 from dual union all
select 60434,'1987-S226','1987-05-20 00:00:00','military missile','success',744,'51034-41-503','15A15 10 RD UBP-12','','','OT',897,'',1000 from dual union all
select 60435,'1987-S227','1987-05-23 00:00:00','military missile','success',744,'51035-41-502','15A15 10 RD UBP-13','','','OT',897,'',1000 from dual union all
select 60436,'1987-S228','1987-01-21 00:00:00','military missile','success',1450,'','15A20 52 RD UBP','','','OT',195,'',1000 from dual union all
select 60437,'1987-S229','1987-07-23 00:00:00','military missile','success',1006,'','15A18 62 RD UBP-3','','','OT',1646,'',1000 from dual union all
select 60438,'1987-S23','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60439,'1987-S230','1987-07-01 00:00:00','military missile','success',1006,'','15A18 62 RD UBP-4','','','OT',1646,'',1000 from dual union all
select 6044,'1959-S276','1959-12-22 07:56:00','suborbital rocket','success',624,'NASA 8.01GT','X248 test','','','Test/RF Astron',1821,'',901 from dual union all
select 60440,'1987-S231','1987-06-30 14:30:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 60441,'1987-S232','1987-09-16 00:00:00','military missile','unknown',1019,'','','','','Test',596,'',1000 from dual union all
select 60442,'1987-S233','1987-01-01 00:00:00','military missile','success',924,'','15Zh57 LKI K-3','','','Test',822,'',500 from dual union all
select 60443,'1987-S234','1987-01-01 00:00:00','military missile','success',924,'','15Zh57 LKI K-4','','','Test',822,'',500 from dual union all
select 60444,'1987-S235','1987-04-13 00:00:00','military missile','success',923,'','15Zh53 32 RD UBP','','','Test',636,'',500 from dual union all
select 60445,'1987-S24','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60446,'1987-S25','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60447,'1987-S26','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60448,'1987-S27','1987-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 60449,'1987-S28','1987-04-15 12:00:00','military missile','success',1006,'51086-811-03','15A18 UBP-22 K','GCh 15F183','','OT',489,'',1000 from dual union all
select 6045,'1959-S277','1959-12-23 19:05:00','military missile','success',1017,'I1-1','8K74 LKI-1','','','Test',480,'',1350 from dual union all
select 60450,'1987-S29','1987-07-08 13:50:00','military missile','success',1006,'51086-811-02','15A18 UBP-23 K','GCh 15F183','','OT',489,'',1000 from dual union all
select 60451,'1987-S30','1987-03-25 03:10:00','military missile','success',1005,'67038-402(4L)','15A18M LKI-4','GCh 15F173','','OT',491,'',1000 from dual union all
select 60452,'1987-S31','1987-05-28 12:45:00','military missile','success',1005,'45038-403(5L)','15A18M LKI-5','GCh 15F173','','OT',487,'',1000 from dual union all
select 60453,'1987-S32','1987-06-03 01:45:00','military missile','success',1005,'51058-405(6L)','15A18M LKI-6','GCh 15F173','','OT',497,'',1000 from dual union all
select 60454,'1987-S33','1987-09-04 01:05:00','military missile','success',1005,'45058-406(8L)','15A18M LKI-8','GCh 15F173','','OT',512,'',1000 from dual union all
select 60455,'1987-S34','1987-09-10 17:35:00','military missile','success',1005,'51058-407(9L)','15A18M LKI-9','GCh 15F173','','OT',491,'',1000 from dual union all
select 60456,'1987-S35','1987-09-30 00:05:00','military missile','success',1005,'45095-601(10L)','15A18M LKI-10','GCh 15F173','','OT',497,'',1000 from dual union all
select 60457,'1987-S36','1987-11-12 06:10:00','military missile','success',1005,'51095-602(12L)','15A18M LKI-12','GCh 15F173','','OT',512,'',1000 from dual union all
select 60458,'1987-S37','1987-11-25 06:13:00','military missile','success',1005,'67095-603(13L)','15A18M LKI-13','GCh 15F173','','OT',497,'',1000 from dual union all
select 60459,'1987-S38','1987-07-16 00:00:00','military missile','success',1005,'','15A18 UBP 59 RD','Demo','','OT',829,'',1000 from dual union all
select 6046,'1959-S279','1959-12-27 00:00:00','military missile','success',976,'ID3-25Sh','8K63 ShPU Test 3','','','Test',630,'',402 from dual union all
select 60460,'1987-S39','1987-01-01 00:00:00','suborbital rocket','success',1304,'','HAVE JEEP VII-A','','','Target',871,'',500 from dual union all
select 60461,'1987-S40','1987-09-16 08:45:00','military missile','success',1453,'49217-91-564','15A35 UBP-35','','','OT',535,'',1000 from dual union all
select 60462,'1987-S41','1987-09-17 08:05:00','military missile','success',1453,'49217-91-563','15A35 UBP-36','','','OT',508,'',1000 from dual union all
select 60463,'1987-S42','1987-01-15 08:10:00','suborbital rocket','success',1074,'S-520-9','E field','','','Ionos/Aeron',833,'',365 from dual union all
select 60464,'1987-S43','1987-01-15 15:25:00','military missile','success',1443,'D5X-1','R&D Mk 5','','','Test',294,'',1000 from dual union all
select 60465,'1987-S44','1987-01-20 08:35:00','military missile','failure',795,'','SENT-1','SENT-1','','Reentry Test',1698,'',90 from dual union all
select 60466,'1987-S45','1987-01-28 08:00:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT119GM','GLORY TRIP 119GM','OT',1705,'',1300 from dual union all
select 60467,'1987-S46','1987-01-28 19:15:00','suborbital rocket','success',224,'NASA 36.16UL','PACE 2','','','Aeronomy',1873,'',255 from dual union all
select 60468,'1987-S47','1987-01-31 09:32:00','suborbital rocket','success',224,'NASA 36.04UE','SCEX II','','','Plasma',1047,'',211 from dual union all
select 60469,'1987-S48','1987-01-31 19:30:03','military missile','success',939,'','DASO','','','OT',421,'SSBN 628',500 from dual union all
select 6047,'1959-S28','1959-03-13 00:38:00','suborbital rocket','success',366,'','RW-119','','','Aeron',1490,'',84 from dual union all
select 60470,'1987-S49','1987-02-04 09:30:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT120GB','GLORY TRIP 120GB','OT',1712,'',1300 from dual union all
select 60471,'1987-S50','1987-02-12 00:00:00','military missile','success',1453,'','15A35 60 RD UBP-3','Trajectory Test','','OT',1461,'',1000 from dual union all
select 60472,'1987-S51','1987-02-13 16:15:00','military missile','success',907,'FTM-16','Mk 21 RV x 7','','','Test',1703,'',1000 from dual union all
select 60473,'1987-S52','1987-02-17 10:43:00','suborbital rocket','success',891,'DLR K-O-143','Strafam 2/1 DROPMAS3','','','Aeron',417,'',59 from dual union all
select 60474,'1987-S53','1987-02-17 21:55:00','suborbital rocket','success',1319,'NASA 33.56UE','UM UV Glow 2','','','Plasma',1873,'',179 from dual union all
select 60475,'1987-S54','1987-02-21 16:15:00','suborbital rocket','success',1074,'S-520-8','GUV','','','UV Astron',833,'',286 from dual union all
select 60476,'1987-S55','1987-02-22 15:00:00','suborbital rocket','success',668,'K-9M-80','CBR','','','Submm Astron',839,'',317 from dual union all
select 60477,'1987-S56','1987-02-26 23:49:10','suborbital rocket','success',224,'GL A21.628','COPE II/POLAR ARCS','','','Plasma',1438,'',387 from dual union all
select 60478,'1987-S57','1987-02-26 23:43:10','suborbital rocket','success',1337,'NASA 29.25UE','COPE II','','','Plasma',1438,'',500 from dual union all
select 60479,'1987-S58','1987-03-05 06:32:40','suborbital rocket','success',1337,'NASA 29.26UE','COPE II','','','Plasma',1438,'',500 from dual union all
select 6048,'1959-S280','1959-12-28 00:00:00','military missile','success',976,'ID10-20T','8K63 UBP-34','','','Test',636,'',402 from dual union all
select 60480,'1987-S59','1987-03-05 06:38:40','suborbital rocket','success',1318,'NASA 38.12UE','COPE II Super 0','','','Plasma',1438,'',500 from dual union all
select 60481,'1987-S60','1987-03-16 08:22:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT121GM','GLORY TRIP 121GM','OT',1699,'',1300 from dual union all
select 60482,'1987-S61','1987-03-17 17:25:00','military missile','success',1443,'D5X-2','R&D Mk 5','','','Test',294,'',1000 from dual union all
select 60483,'1987-S62','1987-03-19 10:00:00','suborbital rocket','success',225,'MASER 1','SSC MASER 1','','','Micrograv',412,'',295 from dual union all
select 60484,'1987-S63','1987-03-21 15:15:00','military missile','success',907,'FTM-17','Mk 21 RV x 6','','','Test',1703,'',1000 from dual union all
select 60485,'1987-S64','1987-03-21 06:56:00','suborbital rocket','success',211,'GL A19.427','COPE II/POLAR ARCS','','','Plasma Ba',1438,'',354 from dual union all
select 60486,'1987-S65','1987-03-21 07:06:00','suborbital rocket','success',1320,'NASA 34.14UE','COPE II Release','','','Plasma',1438,'',350 from dual union all
select 60487,'1987-S66','1987-03-21 07:21:00','suborbital rocket','success',867,'NASA 18.220UE','COPE II Release','','','Plasma',1438,'',270 from dual union all
select 60488,'1987-S67','1987-03-24 13:33:06','military missile','success',919,'','P-494 87 Cape 01','','','OT',231,'',320 from dual;
insert into LAUNCH
select 60489,'1987-S68','1987-03-24 13:52:07','military missile','success',919,'','P-495 87 Cape 02','','','OT',231,'',320 from dual union all
select 6049,'1959-S281','1959-03-30 22:53:00','military missile','failure',1016,'No. I3-20','8K71 LKI III-4','GCh No. I3-20','','Test',480,'',1000 from dual union all
select 60490,'1987-S69','1987-03-24 14:13:07','military missile','success',919,'','P-496 87 Cape 03','','','OT',231,'',320 from dual union all
select 60491,'1987-S70','1987-03-24 15:34:05','military missile','success',919,'','P-497 87 Cape 04','','','OT',231,'',320 from dual union all
select 60492,'1987-S71','1987-03-24 15:54:05','military missile','success',919,'','P-498 87 Cape 05','','','OT',231,'',320 from dual union all
select 60493,'1987-S72','1987-03-24 16:17:05','military missile','success',919,'','P-499 87 Cape 06','','','OT',231,'',320 from dual union all
select 60494,'1987-S73','1987-03-24 18:30:00','suborbital rocket','success',860,'NASA 31.62UE','UIUC Turb','','','Plasma',1821,'',177 from dual union all
select 60495,'1987-S74','1987-03-31 04:47:00','suborbital rocket','success',224,'NASA 36.14UE','COPE II Main','','','Plasma',1438,'',436 from dual union all
select 60496,'1987-S75','1987-03-31 04:47:05','suborbital rocket','success',1318,'NASA 38.10UE','COPE II Super 1','','','Plasma',1438,'',467 from dual union all
select 60497,'1987-S76','1987-03-31 04:47:11','suborbital rocket','success',1318,'NASA 38.11UE','COPE II Super 2','','','Plasma',1438,'',441 from dual union all
select 60498,'1987-S77','1987-04-02 08:05:00','military missile','success',799,'','Mk 12/12A RV x 3/IPMS','IPMS','','Reentry Test',1701,'',1300 from dual union all
select 60499,'1987-S78','1987-04-16 09:25:00','military missile','success',932,'A3TK','CHEVALINE CAF','','','OT',421,'UK S23',1000 from dual union all
select 605,'1945-S81','1945-03-17 00:00:00','military missile','success',1470,'','Remagen','','','Weapon',1736,'',90 from dual union all
select 6050,'1959-S282','1959-05-09 18:59:00','military missile','failure',1016,'No. I3-21','8K71 LKI III-5','GCh No. 17','','Test',480,'',1000 from dual union all
select 60500,'1987-S79','1987-04-18 03:00:00','military missile','success',932,'A3TK','CHEVALINE CAF','','','OT',421,'UK S23',1000 from dual union all
select 60501,'1987-S80','1987-05-01 00:00:00','military missile','success',627,'','','','','Test',1026,'',300 from dual union all
select 60502,'1987-S81','1987-04-30 20:44:00','military missile','success',1443,'D5X-3','R&D Mk 5','','','Test',294,'',1000 from dual union all
select 60503,'1987-S82','1987-05-03 03:59:59','suborbital rocket','success',1212,'DLR K-GR-144','DLR TEXUS 14B','','','Micrograv',416,'',250 from dual union all
select 60504,'1987-S83','1987-05-06 05:08:00','suborbital rocket','success',891,'DLR K-O-145','RASMUS','','','Aeron',417,'',61 from dual union all
select 60505,'1987-S84','1987-05-09 04:00:00','suborbital rocket','failure',1212,'DLR K-GR-146','DLR TEXUS 15','','','Micrograv',416,'',10 from dual union all
select 60506,'1987-S85','1987-05-10 11:07:00','military missile','success',932,'A3TK','CHEVALINE CAF','','','OT',421,'UK S23',1000 from dual union all
select 60507,'1987-S86','1987-05-21 02:00:07','military missile','success',919,'-         P2-45','P-500  87 Cape 07','','','OT',231,'',320 from dual union all
select 60508,'1987-S87','1987-05-21 02:19:02','military missile','success',919,'','P-501 87 Cape 08','','','OT',231,'',320 from dual union all
select 60509,'1987-S88','1987-05-21 02:40:07','military missile','success',919,'','P-502 87 Cape 09','','','OT',231,'',320 from dual union all
select 6051,'1959-S283','1959-05-30 21:42:00','military missile','failure',1016,'No. I3-22','8K71 LKI III-6','GCh No. I3-22','','Test POR',480,'',1400 from dual union all
select 60510,'1987-S89','1987-05-21 04:00:08','military missile','success',919,'','P-503 87 Cape 10','','','OT',231,'',320 from dual union all
select 60511,'1987-S90','1987-05-21 04:33:08','military missile','success',919,'','P-504 87 Cape 11','','','OT',231,'',320 from dual union all
select 60512,'1987-S91','1987-05-21 04:51:05','military missile','success',919,'','P-505 87 Cape 12','','','OT',231,'',320 from dual union all
select 60513,'1987-S92','1987-06-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60514,'1987-S93','1987-06-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60515,'1987-S94','1987-06-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60516,'1987-S95','1987-06-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60517,'1987-S96','1987-06-01 00:00:00','suborbital rocket','unknown',741,'','','','','Aeron',144,'ZUB',250 from dual union all
select 60518,'1987-S97','1987-06-08 00:00:00','suborbital rocket','success',1026,'','Test','','','Test',1361,'',130 from dual union all
select 60519,'1987-S98','1987-06-10 00:00:00','military missile','success',1106,'2608','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 6052,'1959-S284','1959-06-09 20:34:00','military missile','failure',1016,'No. I3-23','8K71 LKI III-7','GCh No. I3-23','','Test',480,'',1000 from dual union all
select 60520,'1987-S99','1987-06-12 22:45:00','military missile','success',1443,'D5X-5','R&D Mk 5','','','Test',294,'',1000 from dual union all
select 60521,'1987-U01','1987-05-15 17:30:01','orbital','failure',460,'L1506SL','Polyus','Skif-DM TsM','','Sat',561,'',155 from dual union all
select 60522,'1987-W01','1987-01-02 12:20:00','suborbital rocket','unknown',738,'','','','','Aeron',1301,'PRILIV',59 from dual union all
select 60523,'1987-W02','1987-01-02 14:44:00','suborbital rocket','unknown',738,'','','','','Aeron',1301,'PRILIV',58 from dual union all
select 60524,'1987-W03','1987-01-03 15:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',77 from dual union all
select 60525,'1987-W04','1987-01-05 16:24:00','suborbital rocket','unknown',738,'','','','','Aeron',1301,'PRILIV',58 from dual union all
select 60526,'1987-W05','1987-01-05 17:41:00','suborbital rocket','unknown',738,'','','','','Aeron',1301,'PRILIV',60 from dual union all
select 60527,'1987-W06','1987-01-06 14:06:00','suborbital rocket','unknown',738,'','','','','Aeron',1301,'PRILIV',57 from dual union all
select 60528,'1987-W07','1987-01-06 15:25:00','suborbital rocket','unknown',738,'','','','','Aeron',1301,'PRILIV',58 from dual union all
select 60529,'1987-W08','1987-01-07 07:31:00','suborbital rocket','unknown',738,'','','','','Aeron',214,'BUGAEV',57 from dual union all
select 6053,'1959-S285','1959-11-01 00:00:00','suborbital rocket','success',814,'','','','','Test',671,'',90 from dual union all
select 60530,'1987-W09','1987-01-07 11:39:00','suborbital rocket','unknown',738,'','','','','Aeron',1301,'PRILIV',62 from dual union all
select 60531,'1987-W10','1987-01-07 12:48:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',58 from dual union all
select 60532,'1987-W100','1987-04-16 22:02:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',71 from dual union all
select 60533,'1987-W101','1987-04-17 19:19:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',68 from dual union all
select 60534,'1987-W102','1987-04-18 04:35:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 60535,'1987-W103','1987-04-18 05:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',74 from dual union all
select 60536,'1987-W104','1987-04-20 07:50:00','suborbital rocket','unknown',738,'','','','','Aeron',215,'BUGAEV',57 from dual union all
select 60537,'1987-W105','1987-04-22 07:32:00','suborbital rocket','unknown',738,'','','','','Aeron',215,'BUGAEV',59 from dual union all
select 60538,'1987-W106','1987-04-22 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 60539,'1987-W107','1987-04-22 18:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',52 from dual union all
select 6054,'1959-S286','1959-11-01 00:00:00','suborbital rocket','success',814,'','','','','Test',671,'',90 from dual union all
select 60540,'1987-W108','1987-04-22 18:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 60541,'1987-W109','1987-04-23 02:00:00','suborbital rocket','success',761,'MT135P-R635','','','','Meteo',1332,'',56 from dual union all
select 60542,'1987-W11','1987-01-07 16:25:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 60543,'1987-W110','1987-04-23 17:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',75 from dual union all
select 60544,'1987-W111','1987-04-23 18:00:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 60545,'1987-W112','1987-04-24 08:07:00','suborbital rocket','unknown',738,'','','','','Aeron',215,'BUGAEV',60 from dual union all
select 60546,'1987-W113','1987-04-24 14:03:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 60547,'1987-W114','1987-04-24 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',58 from dual union all
select 60548,'1987-W115','1987-04-27 07:53:00','suborbital rocket','unknown',738,'','','','','Aeron',215,'BUGAEV',59 from dual union all
select 60549,'1987-W116','1987-04-28 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 6055,'1959-S287','1959-11-01 00:00:00','suborbital rocket','success',814,'','','','','Test',671,'',90 from dual union all
select 60550,'1987-W117','1987-04-29 02:00:00','suborbital rocket','success',761,'MT135P-R636','','','','Meteo',1332,'',55 from dual union all
select 60551,'1987-W118','1987-04-29 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 60552,'1987-W119','1987-04-30 21:57:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 60553,'1987-W12','1987-01-07 18:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',57 from dual union all
select 60554,'1987-W120','1987-04-30 23:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',69 from dual union all
select 60555,'1987-W121','1987-05-04 21:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',70 from dual union all
select 60556,'1987-W122','1987-05-05 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',79 from dual union all
select 60557,'1987-W123','1987-05-06 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',61 from dual union all
select 60558,'1987-W124','1987-05-08 17:48:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',65 from dual union all
select 60559,'1987-W125','1987-05-10 11:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 6056,'1959-S288','1959-12-01 00:00:00','suborbital rocket','success',814,'','','','','Test',671,'',90 from dual union all
select 60560,'1987-W126','1987-05-10 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',65 from dual union all
select 60561,'1987-W127','1987-05-10 13:36:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',74 from dual union all
select 60562,'1987-W128','1987-05-12 12:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',63 from dual union all
select 60563,'1987-W129','1987-05-13 02:00:00','suborbital rocket','success',761,'MT135P-R637','','','','Meteo',1332,'',57 from dual union all
select 60564,'1987-W13','1987-01-07 18:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',75 from dual union all
select 60565,'1987-W130','1987-05-13 14:35:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 60566,'1987-W131','1987-05-13 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 60567,'1987-W132','1987-05-15 19:07:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',59 from dual union all
select 60568,'1987-W133','1987-05-19 14:55:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 60569,'1987-W134','1987-05-19 15:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',59 from dual union all
select 6057,'1959-S289','1959-12-01 00:00:00','suborbital rocket','success',814,'','','','','Test',671,'',90 from dual union all
select 60570,'1987-W135','1987-05-20 02:00:00','suborbital rocket','success',761,'MT135P-R638','','','','Meteo',1332,'',61 from dual union all
select 60571,'1987-W136','1987-05-20 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 60572,'1987-W137','1987-05-20 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',60 from dual union all
select 60573,'1987-W138','1987-05-21 05:41:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 60574,'1987-W139','1987-05-21 19:03:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',72 from dual union all
select 60575,'1987-W14','1987-01-08 11:05:00','suborbital rocket','unknown',738,'','','','','Aeron',1301,'PRILIV',59 from dual union all
select 60576,'1987-W140','1987-05-27 02:15:00','suborbital rocket','success',761,'MT135P-R639','','','','Meteo',1332,'',57 from dual union all
select 60577,'1987-W141','1987-05-27 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 60578,'1987-W142','1987-05-29 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',54 from dual;
insert into LAUNCH
select 60579,'1987-W143','1987-06-02 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',78 from dual union all
select 6058,'1959-S29','1959-03-13 12:30:00','suborbital rocket','success',366,'','RW-121','','','Aeron',1490,'',83 from dual union all
select 60580,'1987-W144','1987-06-03 12:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',61 from dual union all
select 60581,'1987-W145','1987-06-03 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',53 from dual union all
select 60582,'1987-W146','1987-06-03 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 60583,'1987-W147','1987-06-03 17:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',51 from dual union all
select 60584,'1987-W148','1987-06-03 17:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 60585,'1987-W149','1987-06-05 07:05:00','suborbital rocket','unknown',738,'','','','','Aeron',915,'KRENKEL''',59 from dual union all
select 60586,'1987-W15','1987-01-08 15:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 60587,'1987-W150','1987-06-09 12:38:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',57 from dual union all
select 60588,'1987-W151','1987-06-10 02:30:00','suborbital rocket','success',761,'MT135P-R640','','','','Meteo',1332,'',56 from dual union all
select 60589,'1987-W152','1987-06-10 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',915,'KRENKEL''',57 from dual union all
select 6059,'1959-S290','1959-12-12 00:00:00','military missile','success',983,'BK1-02 (R-11MU)','ZI','','','Test',636,'',150 from dual union all
select 60590,'1987-W153','1987-06-10 15:26:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 60591,'1987-W154','1987-06-10 18:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',74 from dual union all
select 60592,'1987-W155','1987-06-11 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',58 from dual union all
select 60593,'1987-W156','1987-06-12 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',915,'KRENKEL''',58 from dual union all
select 60594,'1987-W157','1987-06-13 00:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 60595,'1987-W158','1987-06-15 07:05:00','suborbital rocket','unknown',738,'','','','','Aeron',915,'KRENKEL''',58 from dual union all
select 60596,'1987-W159','1987-06-15 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 60597,'1987-W16','1987-01-08 22:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',63 from dual union all
select 60598,'1987-W160','1987-06-15 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 60599,'1987-W161','1987-06-17 02:00:00','suborbital rocket','success',761,'MT135P-R641','','','','Meteo',1332,'',56 from dual union all
select 606,'1945-S82','1945-03-17 00:00:00','military missile','success',1470,'','Remagen','','','Weapon',1736,'',90 from dual union all
select 6060,'1959-S291','1959-01-23 05:09:00','military missile','failure',1355,'128','RVX-1','','','Reentry Test',233,'',100 from dual union all
select 60600,'1987-W162','1987-06-17 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',915,'KRENKEL''',60 from dual union all
select 60601,'1987-W163','1987-06-17 12:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',63 from dual union all
select 60602,'1987-W164','1987-06-17 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 60603,'1987-W165','1987-06-19 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',915,'KRENKEL''',59 from dual union all
select 60604,'1987-W166','1987-06-20 02:57:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 60605,'1987-W167','1987-06-22 23:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',67 from dual union all
select 60606,'1987-W168','1987-06-23 13:26:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 60607,'1987-W169','1987-06-24 02:00:00','suborbital rocket','success',761,'MT135P-R642','','','','Meteo',1332,'',57 from dual union all
select 60608,'1987-W17','1987-01-09 08:03:00','suborbital rocket','unknown',738,'','','','','Aeron',214,'BUGAEV',61 from dual union all
select 60609,'1987-W170','1987-06-24 07:03:00','suborbital rocket','unknown',738,'','','','','Aeron',915,'KRENKEL''',60 from dual union all
select 6061,'1959-S292','1959-01-01 00:00:00','suborbital rocket','unknown',720,'LT8','','','','Aeron',1837,'',80 from dual union all
select 60610,'1987-W171','1987-06-24 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 60611,'1987-W172','1987-06-24 15:57:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 60612,'1987-W173','1987-06-25 14:19:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 60613,'1987-W174','1987-06-26 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',915,'KRENKEL''',59 from dual union all
select 60614,'1987-W175','1987-07-01 07:06:00','suborbital rocket','unknown',738,'','','','','Aeron',915,'KRENKEL''',60 from dual union all
select 60615,'1987-W176','1987-07-01 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',77 from dual union all
select 60616,'1987-W177','1987-07-01 23:34:00','suborbital rocket','success',1284,'LIGHT CHAFF','MAC-SINE S-C15/L','','','Aeron',29,'',96 from dual union all
select 60617,'1987-W178','1987-07-08 02:00:00','suborbital rocket','success',761,'MT135P-R643','','','','Meteo',1332,'',56 from dual union all
select 60618,'1987-W179','1987-07-08 04:54:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',68 from dual union all
select 60619,'1987-W18','1987-01-09 10:01:00','suborbital rocket','unknown',738,'','','','','Aeron',1301,'PRILIV',59 from dual union all
select 6062,'1959-S293','1959-01-12 00:00:00','military missile','success',971,'B1-18T','','','','Test',636,'',150 from dual union all
select 60620,'1987-W180','1987-07-08 08:47:00','suborbital rocket','unknown',738,'','','','','Aeron',982,'MUSSON',60 from dual union all
select 60621,'1987-W181','1987-07-08 15:34:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 60622,'1987-W182','1987-07-09 21:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',69 from dual union all
select 60623,'1987-W183','1987-07-10 08:06:00','suborbital rocket','unknown',738,'','','','','Aeron',982,'MUSSON',61 from dual union all
select 60624,'1987-W184','1987-07-12 07:41:00','suborbital rocket','unknown',738,'','','','','Aeron',982,'MUSSON',60 from dual union all
select 60625,'1987-W185','1987-07-15 02:23:00','suborbital rocket','success',1284,'','Datasonde','','','Meteo',1821,'',64 from dual union all
select 60626,'1987-W186','1987-07-15 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 60627,'1987-W187','1987-07-15 15:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 60628,'1987-W188','1987-07-15 22:01:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',73 from dual union all
select 60629,'1987-W189','1987-07-17 07:15:00','suborbital rocket','unknown',738,'','','','','Aeron',982,'MUSSON',58 from dual union all
select 6063,'1959-S294','1959-01-13 00:00:00','military missile','success',971,'B1-2','','','','Test',636,'',150 from dual union all
select 60630,'1987-W19','1987-01-11 12:37:00','suborbital rocket','unknown',738,'','','','','Aeron',1301,'PRILIV',57 from dual union all
select 60631,'1987-W190','1987-07-17 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 60632,'1987-W191','1987-07-19 07:23:00','suborbital rocket','unknown',738,'','','','','Aeron',982,'MUSSON',57 from dual union all
select 60633,'1987-W192','1987-07-20 22:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 60634,'1987-W193','1987-07-22 07:36:00','suborbital rocket','unknown',738,'','','','','Aeron',982,'MUSSON',56 from dual union all
select 60635,'1987-W194','1987-07-23 02:00:00','suborbital rocket','success',761,'MT135P-R644','','','','Meteo',1332,'',55 from dual union all
select 60636,'1987-W195','1987-07-24 07:47:00','suborbital rocket','unknown',738,'','','','','Aeron',982,'MUSSON',56 from dual union all
select 60637,'1987-W196','1987-07-24 14:37:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 60638,'1987-W197','1987-07-26 07:27:00','suborbital rocket','unknown',738,'','','','','Aeron',982,'MUSSON',58 from dual union all
select 60639,'1987-W198','1987-07-27 06:52:00','suborbital rocket','success',1284,'','WIPP/Datasonde','','','Meteo',1821,'',70 from dual union all
select 6064,'1959-S295','1959-01-13 00:00:00','military missile','success',971,'B1-15','','','','Test',636,'',150 from dual union all
select 60640,'1987-W199','1987-07-29 07:06:00','suborbital rocket','unknown',738,'','','','','Aeron',982,'MUSSON',64 from dual union all
select 60641,'1987-W20','1987-01-11 13:47:00','suborbital rocket','unknown',738,'','','','','Aeron',1301,'PRILIV',57 from dual union all
select 60642,'1987-W200','1987-07-29 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',77 from dual union all
select 60643,'1987-W201','1987-07-30 16:45:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',75 from dual union all
select 60644,'1987-W202','1987-08-04 14:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 60645,'1987-W203','1987-08-06 02:00:00','suborbital rocket','success',761,'MT135P-R645','','','','Meteo',1332,'',55 from dual union all
select 60646,'1987-W204','1987-08-06 21:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',69 from dual union all
select 60647,'1987-W205','1987-08-10 19:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',61 from dual union all
select 60648,'1987-W206','1987-08-11 12:45:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',63 from dual union all
select 60649,'1987-W207','1987-08-12 16:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 6065,'1959-S296','1959-01-29 00:00:00','military missile','success',992,'53060204','','','','OT',636,'',170 from dual union all
select 60650,'1987-W208','1987-08-12 21:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',71 from dual union all
select 60651,'1987-W209','1987-08-12 22:23:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',55 from dual union all
select 60652,'1987-W21','1987-01-12 18:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',76 from dual union all
select 60653,'1987-W210','1987-08-13 14:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 60654,'1987-W211','1987-08-17 17:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 60655,'1987-W212','1987-08-19 02:00:00','suborbital rocket','success',761,'MT135P-R646','','','','Meteo',1332,'',55 from dual union all
select 60656,'1987-W213','1987-08-19 14:37:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',77 from dual union all
select 60657,'1987-W214','1987-08-20 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',59 from dual union all
select 60658,'1987-W215','1987-08-21 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',64 from dual union all
select 60659,'1987-W216','1987-08-24 05:46:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 6066,'1959-S297','1959-01-29 00:00:00','military missile','success',969,'D217','','','','OT',636,'',100 from dual union all
select 60660,'1987-W217','1987-08-27 14:15:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 60661,'1987-W218','1987-09-02 02:00:00','suborbital rocket','success',761,'MT135P-R647','','','','Meteo',1332,'',56 from dual union all
select 60662,'1987-W219','1987-09-02 17:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 60663,'1987-W22','1987-01-14 02:00:00','suborbital rocket','success',761,'MT135P-R626','','','','Meteo',1332,'',51 from dual union all
select 60664,'1987-W220','1987-09-04 20:00:00','suborbital rocket','unknown',738,'','','','','Aeron',916,'KRENKEL''',60 from dual union all
select 60665,'1987-W221','1987-09-07 14:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',70 from dual union all
select 60666,'1987-W222','1987-09-08 22:46:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 60667,'1987-W223','1987-09-08 23:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 60668,'1987-W224','1987-09-09 00:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',58 from dual;
insert into LAUNCH
select 60669,'1987-W225','1987-09-09 02:12:00','suborbital rocket','success',761,'MT135P-R648','','','','Meteo',1332,'',55 from dual union all
select 6067,'1959-S298','1959-01-29 00:00:00','military missile','success',975,'B6897','','','','Test',636,'',80 from dual union all
select 60670,'1987-W226','1987-09-11 19:16:00','suborbital rocket','unknown',738,'','','','','Aeron',916,'KRENKEL''',60 from dual union all
select 60671,'1987-W227','1987-09-15 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 60672,'1987-W228','1987-09-16 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',56 from dual union all
select 60673,'1987-W229','1987-09-16 19:18:00','suborbital rocket','unknown',738,'','','','','Aeron',916,'KRENKEL''',60 from dual union all
select 60674,'1987-W23','1987-01-14 07:41:00','suborbital rocket','unknown',738,'','','','','Aeron',214,'BUGAEV',62 from dual union all
select 60675,'1987-W230','1987-09-16 20:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',68 from dual union all
select 60676,'1987-W231','1987-09-18 19:17:00','suborbital rocket','unknown',738,'','','','','Aeron',916,'KRENKEL''',60 from dual union all
select 60677,'1987-W232','1987-09-21 09:13:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',50 from dual union all
select 60678,'1987-W233','1987-09-21 12:18:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',69 from dual union all
select 60679,'1987-W234','1987-09-22 14:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 6068,'1959-S299','1959-01-29 00:00:00','military missile','success',971,'M3-17','','','','Test',636,'',150 from dual union all
select 60680,'1987-W235','1987-09-22 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',63 from dual union all
select 60681,'1987-W236','1987-09-22 22:01:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',69 from dual union all
select 60682,'1987-W237','1987-09-23 13:34:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 60683,'1987-W238','1987-09-23 19:15:00','suborbital rocket','unknown',738,'','','','','Aeron',916,'KRENKEL''',60 from dual union all
select 60684,'1987-W239','1987-09-25 19:05:00','suborbital rocket','unknown',738,'','','','','Aeron',916,'KRENKEL''',60 from dual union all
select 60685,'1987-W24','1987-01-14 13:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 60686,'1987-W240','1987-09-29 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',52 from dual union all
select 60687,'1987-W241','1987-09-29 17:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 60688,'1987-W242','1987-09-30 19:16:00','suborbital rocket','unknown',738,'','','','','Aeron',916,'KRENKEL''',60 from dual union all
select 60689,'1987-W243','1987-10-02 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 6069,'1959-S30','1959-03-13 15:45:00','suborbital rocket','success',49,'NN3.24F','','IGY NN3.24F','','Solar UV/Solar EUV',1869,'',170 from dual union all
select 60690,'1987-W244','1987-10-02 19:19:00','suborbital rocket','unknown',738,'','','','','Aeron',916,'KRENKEL''',56 from dual union all
select 60691,'1987-W245','1987-10-04 10:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',66 from dual union all
select 60692,'1987-W246','1987-10-05 18:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 60693,'1987-W247','1987-10-06 17:18:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',56 from dual union all
select 60694,'1987-W248','1987-10-07 02:00:00','suborbital rocket','success',761,'MT135P-R651','','','','Meteo',1332,'',56 from dual union all
select 60695,'1987-W249','1987-10-07 14:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',63 from dual union all
select 60696,'1987-W25','1987-01-14 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',50 from dual union all
select 60697,'1987-W250','1987-10-09 11:13:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',53 from dual union all
select 60698,'1987-W251','1987-10-13 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',58 from dual union all
select 60699,'1987-W252','1987-10-14 15:30:00','suborbital rocket','unknown',728,'M-100B','','','','Aeron',636,'',77 from dual union all
select 607,'1945-S83','1945-03-21 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 6070,'1959-S300','1959-01-30 00:00:00','military missile','success',975,'B681','','','','Test',636,'',80 from dual union all
select 60700,'1987-W253','1987-10-15 11:33:00','suborbital rocket','success',1284,'Chaff','MAC/EPSILON E-C2/H','','','Aeron',29,'',96 from dual union all
select 60701,'1987-W254','1987-10-15 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 60702,'1987-W255','1987-10-16 13:20:00','suborbital rocket','unknown',728,'M-100B','','','','Aeron',636,'',79 from dual union all
select 60703,'1987-W256','1987-10-21 02:17:00','suborbital rocket','success',761,'MT135P-R652','','','','Meteo',1332,'',57 from dual union all
select 60704,'1987-W257','1987-10-21 15:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 60705,'1987-W258','1987-10-22 12:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',78 from dual union all
select 60706,'1987-W259','1987-10-22 13:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 60707,'1987-W26','1987-01-14 18:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 60708,'1987-W260','1987-10-22 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 60709,'1987-W261','1987-10-23 12:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',78 from dual union all
select 6071,'1959-S301','1959-02-02 00:00:00','military missile','failure',1013,'B1-4 8K52','8K52 Zvezda-4','Zvezda','','Test',636,'',200 from dual union all
select 60710,'1987-W262','1987-10-26 22:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',62 from dual union all
select 60711,'1987-W263','1987-10-28 02:00:00','suborbital rocket','success',761,'MT135P-R653','','','','Meteo',1332,'',55 from dual union all
select 60712,'1987-W264','1987-10-28 11:44:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',77 from dual union all
select 60713,'1987-W265','1987-10-28 16:53:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',76 from dual union all
select 60714,'1987-W266','1987-10-29 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',53 from dual union all
select 60715,'1987-W267','1987-11-03 08:37:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',74 from dual union all
select 60716,'1987-W268','1987-11-04 22:46:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',70 from dual union all
select 60717,'1987-W269','1987-11-05 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',52 from dual union all
select 60718,'1987-W27','1987-01-15 16:54:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 60719,'1987-W270','1987-11-05 14:48:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 6072,'1959-S302','1959-02-13 00:00:00','military missile','failure',1013,'B10-17TA','8K51 UBP-16','Gotovnost-3','','Test',636,'',200 from dual union all
select 60720,'1987-W271','1987-11-09 11:52:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 60721,'1987-W272','1987-11-11 02:00:00','suborbital rocket','success',761,'MT135P-R654','','','','Meteo',1332,'',55 from dual union all
select 60722,'1987-W273','1987-11-11 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 60723,'1987-W274','1987-11-11 14:48:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 60724,'1987-W275','1987-11-13 07:18:00','suborbital rocket','unknown',738,'','','','','Aeron',1640,'USHAKOV',58 from dual union all
select 60725,'1987-W276','1987-11-14 12:34:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',78 from dual union all
select 60726,'1987-W277','1987-11-16 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',77 from dual union all
select 60727,'1987-W278','1987-11-18 02:00:00','suborbital rocket','success',761,'MT135P-R655','','','','Meteo',1332,'',56 from dual union all
select 60728,'1987-W279','1987-11-18 08:56:00','suborbital rocket','unknown',738,'','','','','Aeron',1640,'USHAKOV',60 from dual union all
select 60729,'1987-W28','1987-01-15 18:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',69 from dual union all
select 6073,'1959-S303','1959-03-04 00:00:00','military missile','success',971,'M3-19','','','','Test',636,'',150 from dual union all
select 60730,'1987-W280','1987-11-18 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',76 from dual union all
select 60731,'1987-W281','1987-11-19 06:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',70 from dual union all
select 60732,'1987-W282','1987-11-20 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 60733,'1987-W283','1987-11-21 11:22:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',74 from dual union all
select 60734,'1987-W284','1987-11-24 15:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',74 from dual union all
select 60735,'1987-W285','1987-11-26 02:00:00','suborbital rocket','success',761,'MT135P-R656','','','','Meteo',1332,'',55 from dual union all
select 60736,'1987-W286','1987-11-27 07:22:00','suborbital rocket','unknown',738,'','','','','Aeron',1640,'USHAKOV',61 from dual union all
select 60737,'1987-W287','1987-11-27 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 60738,'1987-W288','1987-11-29 04:18:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 60739,'1987-W289','1987-11-29 09:51:00','suborbital rocket','unknown',738,'','','','','Aeron',1302,'PRILIV',57 from dual union all
select 6074,'1959-S304','1959-03-04 00:00:00','military missile','success',971,'M2-23','','','','Test',636,'',150 from dual union all
select 60740,'1987-W29','1987-01-16 07:40:00','suborbital rocket','unknown',738,'','','','','Aeron',214,'BUGAEV',60 from dual union all
select 60741,'1987-W290','1987-12-01 09:06:00','suborbital rocket','unknown',738,'','','','','Aeron',1302,'PRILIV',58 from dual union all
select 60742,'1987-W291','1987-12-01 20:01:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',62 from dual union all
select 60743,'1987-W292','1987-12-01 23:21:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',59 from dual union all
select 60744,'1987-W293','1987-12-02 07:17:00','suborbital rocket','unknown',738,'','','','','Aeron',1640,'USHAKOV',60 from dual union all
select 60745,'1987-W294','1987-12-02 09:11:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',79 from dual union all
select 60746,'1987-W295','1987-12-02 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 60747,'1987-W296','1987-12-03 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1302,'PRILIV',60 from dual union all
select 60748,'1987-W297','1987-12-03 16:27:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 60749,'1987-W298','1987-12-04 07:40:00','suborbital rocket','unknown',738,'','','','','Aeron',1640,'USHAKOV',62 from dual union all
select 6075,'1959-S305','1959-03-12 00:00:00','military missile','success',971,'I1-15T','','','','Test',636,'',150 from dual union all
select 60750,'1987-W299','1987-12-05 16:15:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',74 from dual union all
select 60751,'1987-W30','1987-01-21 02:00:00','suborbital rocket','success',761,'MT135P-R627','','','','Meteo',1332,'',55 from dual union all
select 60752,'1987-W300','1987-12-07 10:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',76 from dual union all
select 60753,'1987-W301','1987-12-07 14:05:00','suborbital rocket','unknown',738,'','','','','Aeron',1302,'PRILIV',56 from dual union all
select 60754,'1987-W302','1987-12-09 02:00:00','suborbital rocket','success',761,'MT135P-R657','','','','Meteo',1332,'',52 from dual union all
select 60755,'1987-W303','1987-12-09 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1640,'USHAKOV',58 from dual union all
select 60756,'1987-W304','1987-12-09 10:17:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',79 from dual union all
select 60757,'1987-W305','1987-12-09 18:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 60758,'1987-W306','1987-12-11 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1640,'USHAKOV',60 from dual;
insert into LAUNCH
select 60759,'1987-W307','1987-12-11 13:35:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',67 from dual union all
select 6076,'1959-S306','1959-03-04 00:00:00','military missile','failure',1013,'B10-16T','8K51 UBP-17','Gotovnost-1','','Test',636,'',200 from dual union all
select 60760,'1987-W308','1987-12-11 13:55:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 60761,'1987-W309','1987-12-11 14:57:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 60762,'1987-W31','1987-01-21 08:10:00','suborbital rocket','unknown',738,'','','','','Aeron',214,'BUGAEV',58 from dual union all
select 60763,'1987-W310','1987-12-16 02:13:00','suborbital rocket','success',761,'MT135P-R658','','','','Meteo',1332,'',54 from dual union all
select 60764,'1987-W311','1987-12-16 11:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 60765,'1987-W312','1987-09-09 02:00:00','suborbital rocket','success',759,'MT-135-47','SONDE','','','Meteo',839,'',58 from dual union all
select 60766,'1987-W313','1987-09-09 04:00:00','suborbital rocket','success',759,'MT-135-48','SONDE','','','Meteo',839,'',58 from dual union all
select 60767,'1987-W314','1987-01-07 02:00:00','suborbital rocket','success',761,'MT135P-R625','','','','Meteo',1332,'',60 from dual union all
select 60768,'1987-W315','1987-09-23 02:00:00','suborbital rocket','success',761,'MT135P-R649','','','','Meteo',1332,'',60 from dual union all
select 60769,'1987-W316','1987-09-30 02:00:00','suborbital rocket','success',761,'MT135P-R650','','','','Meteo',1332,'',60 from dual union all
select 6077,'1959-S307','1959-03-06 00:00:00','military missile','failure',1013,'B10-18TA','8K51 UBP-18','Gotovnost-1','','Test',636,'',200 from dual union all
select 60770,'1987-W317','1987-12-23 02:00:00','suborbital rocket','success',761,'MT135P-R659','','','','Meteo',1332,'',60 from dual union all
select 60771,'1987-W32','1987-01-21 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',56 from dual union all
select 60772,'1987-W33','1987-01-22 16:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',76 from dual union all
select 60773,'1987-W34','1987-01-23 13:12:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 60774,'1987-W35','1987-01-28 09:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 60775,'1987-W36','1987-01-28 09:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',58 from dual union all
select 60776,'1987-W37','1987-01-29 12:35:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',53 from dual union all
select 60777,'1987-W38','1987-01-30 15:47:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',59 from dual union all
select 60778,'1987-W39','1987-02-03 18:17:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',70 from dual union all
select 60779,'1987-W40','1987-02-04 12:53:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',62 from dual union all
select 6078,'1959-S308','1959-03-06 00:00:00','military missile','failure',1013,'B10-19TA','8K51 UBP-19','Gotovnost-3','','Test',636,'',200 from dual union all
select 60780,'1987-W41','1987-02-04 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',77 from dual union all
select 60781,'1987-W42','1987-02-05 02:00:00','suborbital rocket','success',761,'MT135P-R628','','','','Meteo',1332,'',57 from dual union all
select 60782,'1987-W43','1987-02-05 15:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 60783,'1987-W44','1987-02-11 12:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',50 from dual union all
select 60784,'1987-W45','1987-02-11 15:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 60785,'1987-W46','1987-02-13 18:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',70 from dual union all
select 60786,'1987-W47','1987-02-18 02:00:00','suborbital rocket','success',761,'MT135P-R629','','','','Meteo',1332,'',57 from dual union all
select 60787,'1987-W48','1987-02-18 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',61 from dual union all
select 60788,'1987-W49','1987-02-18 17:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',58 from dual union all
select 60789,'1987-W50','1987-02-23 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 6079,'1959-S309','1959-03-13 00:00:00','military missile','success',971,'I1-6','','','','Test',636,'',150 from dual union all
select 60790,'1987-W51','1987-02-25 02:00:00','suborbital rocket','success',761,'MT135P-R630','','','','Meteo',1332,'',56 from dual union all
select 60791,'1987-W52','1987-02-25 14:04:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',77 from dual union all
select 60792,'1987-W53','1987-02-27 13:21:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',56 from dual union all
select 60793,'1987-W54','1987-02-27 22:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',53 from dual union all
select 60794,'1987-W55','1987-03-02 07:45:00','suborbital rocket','unknown',738,'','','','','Aeron',914,'KRENKEL''',61 from dual union all
select 60795,'1987-W56','1987-03-02 20:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',72 from dual union all
select 60796,'1987-W57','1987-03-04 08:19:00','suborbital rocket','unknown',738,'','','','','Aeron',914,'KRENKEL''',60 from dual union all
select 60797,'1987-W58','1987-03-04 13:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',75 from dual union all
select 60798,'1987-W59','1987-03-04 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 60799,'1987-W60','1987-03-04 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 608,'1945-S84','1945-03-11 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 6080,'1959-S31','1959-03-17 00:00:00','suborbital rocket','success',550,'','20" SM','','','Test',1821,'',1380 from dual union all
select 60800,'1987-W61','1987-03-05 13:37:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',57 from dual union all
select 60801,'1987-W62','1987-03-06 07:04:00','suborbital rocket','unknown',738,'','','','','Aeron',914,'KRENKEL''',60 from dual union all
select 60802,'1987-W63','1987-03-08 11:24:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',78 from dual union all
select 60803,'1987-W64','1987-03-09 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',76 from dual union all
select 60804,'1987-W65','1987-03-11 02:00:00','suborbital rocket','success',761,'MT135P-R631','','','','Meteo',1332,'',56 from dual union all
select 60805,'1987-W66','1987-03-11 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',66 from dual union all
select 60806,'1987-W67','1987-03-17 19:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 60807,'1987-W68','1987-03-18 02:00:00','suborbital rocket','success',761,'MT135P-R632','','','','Meteo',1332,'',56 from dual union all
select 60808,'1987-W69','1987-03-18 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',914,'KRENKEL''',59 from dual union all
select 60809,'1987-W70','1987-03-18 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',65 from dual union all
select 6081,'1959-S310','1959-03-30 00:00:00','military missile','success',975,'I2-6','','','','Test',636,'',80 from dual union all
select 60810,'1987-W71','1987-03-19 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 60811,'1987-W72','1987-03-20 07:23:00','suborbital rocket','unknown',738,'','','','','Aeron',914,'KRENKEL''',58 from dual union all
select 60812,'1987-W73','1987-03-21 15:17:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 60813,'1987-W74','1987-03-23 15:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 60814,'1987-W75','1987-03-24 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 60815,'1987-W76','1987-03-24 17:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 60816,'1987-W77','1987-03-25 02:00:00','suborbital rocket','success',761,'MT135P-R633','','','','Meteo',1332,'',56 from dual union all
select 60817,'1987-W78','1987-03-25 07:17:00','suborbital rocket','unknown',738,'','','','','Aeron',914,'KRENKEL''',61 from dual union all
select 60818,'1987-W79','1987-03-25 15:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 60819,'1987-W80','1987-03-25 16:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 6082,'1959-S311','1959-03-30 00:00:00','military missile','success',975,'I2-15T','','','','Test',636,'',80 from dual union all
select 60820,'1987-W81','1987-03-27 07:05:00','suborbital rocket','unknown',738,'','','','','Aeron',914,'KRENKEL''',58 from dual union all
select 60821,'1987-W82','1987-04-01 08:22:00','suborbital rocket','unknown',738,'','','','','Aeron',215,'BUGAEV',59 from dual union all
select 60822,'1987-W83','1987-04-01 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',74 from dual union all
select 60823,'1987-W84','1987-04-02 10:36:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',59 from dual union all
select 60824,'1987-W85','1987-04-02 11:23:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',72 from dual union all
select 60825,'1987-W86','1987-04-02 12:45:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',50 from dual union all
select 60826,'1987-W87','1987-04-02 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 60827,'1987-W88','1987-04-03 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',215,'BUGAEV',60 from dual union all
select 60828,'1987-W89','1987-04-06 18:09:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',50 from dual union all
select 60829,'1987-W90','1987-04-08 02:00:00','suborbital rocket','success',761,'MT135P-R634','','','','Meteo',1332,'',56 from dual union all
select 6083,'1959-S312','1959-04-20 00:00:00','military missile','success',975,'B2-16T','','','','Test',636,'',80 from dual union all
select 60830,'1987-W91','1987-04-08 08:11:00','suborbital rocket','unknown',738,'','','','','Aeron',215,'BUGAEV',54 from dual union all
select 60831,'1987-W92','1987-04-08 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',55 from dual union all
select 60832,'1987-W93','1987-04-09 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',75 from dual union all
select 60833,'1987-W94','1987-04-09 15:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',59 from dual union all
select 60834,'1987-W95','1987-04-10 08:30:00','suborbital rocket','unknown',738,'','','','','Aeron',215,'BUGAEV',59 from dual union all
select 60835,'1987-W96','1987-04-10 14:37:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',75 from dual union all
select 60836,'1987-W97','1987-04-15 07:58:00','suborbital rocket','unknown',738,'','','','','Aeron',215,'BUGAEV',57 from dual union all
select 60837,'1987-W98','1987-04-16 11:55:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 60838,'1987-W99','1987-04-16 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',68 from dual union all
select 60839,'1987-Y01','1987-01-01 12:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 6084,'1959-S313','1959-05-04 00:00:00','military missile','success',992,'S6-2','','','','OT',636,'',170 from dual union all
select 60840,'1987-Y02','1987-01-04 07:32:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',86 from dual union all
select 60841,'1987-Y03','1987-01-04 14:58:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',84 from dual union all
select 60842,'1987-Y04','1987-01-04 20:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 60843,'1987-Y05','1987-01-06 15:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',84 from dual union all
select 60844,'1987-Y06','1987-01-06 20:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 60845,'1987-Y07','1987-01-07 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 60846,'1987-Y08','1987-01-07 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 60847,'1987-Y09','1987-01-07 14:57:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',85 from dual union all
select 60848,'1987-Y10','1987-01-07 15:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual;
insert into LAUNCH
select 60849,'1987-Y100','1987-03-12 11:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',87 from dual union all
select 6085,'1959-S314','1959-05-13 00:00:00','military missile','success',992,'S9-4','','','','OT',636,'',170 from dual union all
select 60850,'1987-Y101','1987-03-13 11:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',80 from dual union all
select 60851,'1987-Y102','1987-03-15 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 60852,'1987-Y103','1987-03-16 09:06:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 60853,'1987-Y104','1987-03-16 12:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',87 from dual union all
select 60854,'1987-Y105','1987-03-16 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',89 from dual union all
select 60855,'1987-Y106','1987-03-16 14:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 60856,'1987-Y107','1987-03-17 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 60857,'1987-Y108','1987-03-18 12:01:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',89 from dual union all
select 60858,'1987-Y109','1987-03-18 14:15:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',88 from dual union all
select 60859,'1987-Y11','1987-01-11 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 6086,'1959-S315','1959-05-18 00:00:00','military missile','success',992,'S6-1','','','','OT',636,'',170 from dual union all
select 60860,'1987-Y110','1987-03-18 14:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',96 from dual union all
select 60861,'1987-Y111','1987-03-19 12:05:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',90 from dual union all
select 60862,'1987-Y112','1987-03-19 13:11:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',83 from dual union all
select 60863,'1987-Y113','1987-03-20 12:03:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',91 from dual union all
select 60864,'1987-Y114','1987-03-23 14:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60865,'1987-Y115','1987-03-24 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 60866,'1987-Y116','1987-03-25 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 60867,'1987-Y117','1987-03-25 19:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 60868,'1987-Y118','1987-03-29 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 60869,'1987-Y119','1987-03-30 15:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 6087,'1959-S316','1959-05-21 00:00:00','military missile','success',992,'S2-1','','','','OT',636,'',170 from dual union all
select 60870,'1987-Y12','1987-01-12 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',85 from dual union all
select 60871,'1987-Y120','1987-03-31 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 60872,'1987-Y121','1987-04-01 10:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 60873,'1987-Y122','1987-04-02 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',96 from dual union all
select 60874,'1987-Y123','1987-04-04 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 60875,'1987-Y124','1987-04-05 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 60876,'1987-Y125','1987-04-06 20:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 60877,'1987-Y126','1987-04-07 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 60878,'1987-Y127','1987-04-08 08:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 60879,'1987-Y128','1987-04-08 12:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 6088,'1959-S317','1959-05-21 00:00:00','military missile','success',992,'S2-1','','','','OT',636,'',170 from dual union all
select 60880,'1987-Y129','1987-04-12 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 60881,'1987-Y13','1987-01-13 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',90 from dual union all
select 60882,'1987-Y130','1987-04-13 14:26:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60883,'1987-Y131','1987-04-14 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 60884,'1987-Y132','1987-04-15 14:28:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 60885,'1987-Y133','1987-04-16 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 60886,'1987-Y134','1987-04-19 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 60887,'1987-Y135','1987-04-20 13:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60888,'1987-Y136','1987-04-20 14:35:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60889,'1987-Y137','1987-04-20 15:45:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 6089,'1959-S318','1959-05-22 00:00:00','military missile','success',992,'S2-9','','','','OT',636,'',170 from dual union all
select 60890,'1987-Y138','1987-04-21 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 60891,'1987-Y139','1987-04-22 12:02:00','suborbital rocket','unknown',728,'','Ion density','','','Aeron',1467,'',89 from dual union all
select 60892,'1987-Y14','1987-01-13 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 60893,'1987-Y140','1987-04-22 14:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60894,'1987-Y141','1987-04-22 18:13:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',84 from dual union all
select 60895,'1987-Y142','1987-04-23 11:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 60896,'1987-Y143','1987-04-23 17:49:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 60897,'1987-Y144','1987-04-26 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 60898,'1987-Y145','1987-04-27 17:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60899,'1987-Y146','1987-04-28 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 609,'1945-S85','1945-03-16 00:29:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 6090,'1959-S319','1959-04-21 00:00:00','military missile','success',1013,'B1-2','8K51 UBP-21','','','Test',636,'',500 from dual union all
select 60900,'1987-Y147','1987-04-29 10:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 60901,'1987-Y148','1987-04-29 12:00:00','suborbital rocket','unknown',728,'','Ion density','','','Aeron',1467,'',83 from dual union all
select 60902,'1987-Y149','1987-04-29 14:35:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 60903,'1987-Y15','1987-01-13 21:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',81 from dual union all
select 60904,'1987-Y150','1987-04-29 14:47:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 60905,'1987-Y151','1987-04-30 22:17:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',84 from dual union all
select 60906,'1987-Y152','1987-05-03 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 60907,'1987-Y153','1987-05-04 14:54:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 60908,'1987-Y154','1987-05-06 11:38:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 60909,'1987-Y155','1987-05-06 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 6091,'1959-S32','1959-02-17 01:46:00','military missile','success',1016,'No. 041081','8K71 LKI III-2','GCh No. 13','','Test',480,'',1350 from dual union all
select 60910,'1987-Y156','1987-05-06 14:15:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 60911,'1987-Y157','1987-05-08 17:33:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',90 from dual union all
select 60912,'1987-Y158','1987-05-10 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',91 from dual union all
select 60913,'1987-Y159','1987-05-11 15:01:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',88 from dual union all
select 60914,'1987-Y16','1987-01-14 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 60915,'1987-Y160','1987-05-12 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 60916,'1987-Y161','1987-05-13 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 60917,'1987-Y162','1987-05-13 15:07:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 60918,'1987-Y163','1987-05-18 11:54:00','suborbital rocket','unknown',728,'','','','','Aeron',1756,'VOKV',88 from dual union all
select 60919,'1987-Y164','1987-05-19 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1756,'VOKV',87 from dual union all
select 6092,'1959-S320','1959-04-25 00:00:00','military missile','success',1013,'B1-1','8K51 UBP-22','','','Test',636,'',500 from dual union all
select 60920,'1987-Y165','1987-05-19 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 60921,'1987-Y166','1987-05-20 11:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1756,'VOKV',86 from dual union all
select 60922,'1987-Y167','1987-05-20 12:07:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 60923,'1987-Y168','1987-05-20 15:11:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 60924,'1987-Y169','1987-05-21 11:08:00','suborbital rocket','unknown',728,'','','','','Aeron',1756,'VOKV',85 from dual union all
select 60925,'1987-Y17','1987-01-14 17:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',81 from dual union all
select 60926,'1987-Y170','1987-05-24 12:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1756,'VOKV',88 from dual union all
select 60927,'1987-Y171','1987-05-25 11:42:00','suborbital rocket','unknown',728,'','','','','Aeron',1756,'VOKV',87 from dual union all
select 60928,'1987-Y172','1987-05-26 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 60929,'1987-Y173','1987-05-27 11:56:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 6093,'1959-S321','1959-04-29 00:00:00','military missile','success',975,'B2-14','','','','Test',636,'',80 from dual union all
select 60930,'1987-Y174','1987-05-27 15:16:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 60931,'1987-Y175','1987-05-28 11:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1756,'VOKV',87 from dual union all
select 60932,'1987-Y176','1987-06-02 23:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 60933,'1987-Y177','1987-06-03 11:25:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 60934,'1987-Y178','1987-06-03 16:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 60935,'1987-Y179','1987-06-10 11:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 60936,'1987-Y18','1987-01-15 17:53:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',94 from dual union all
select 60937,'1987-Y180','1987-06-10 15:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 60938,'1987-Y181','1987-06-10 17:37:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',87 from dual;
insert into LAUNCH
select 60939,'1987-Y182','1987-06-10 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 6094,'1959-S322','1959-05-15 00:00:00','military missile','success',971,'B2-1','','','','Test',636,'',150 from dual union all
select 60940,'1987-Y183','1987-06-13 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 60941,'1987-Y184','1987-06-17 11:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 60942,'1987-Y185','1987-06-17 16:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60943,'1987-Y186','1987-06-17 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 60944,'1987-Y187','1987-06-20 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 60945,'1987-Y188','1987-06-21 12:09:00','suborbital rocket','unknown',728,'','','','','Aeron',1756,'VOKV',85 from dual union all
select 60946,'1987-Y189','1987-06-22 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1756,'VOKV',80 from dual union all
select 60947,'1987-Y19','1987-01-15 17:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',83 from dual union all
select 60948,'1987-Y190','1987-06-24 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 60949,'1987-Y191','1987-06-24 12:13:00','suborbital rocket','unknown',728,'','','','','Aeron',1756,'VOKV',83 from dual union all
select 6095,'1959-S323','1959-05-19 00:00:00','military missile','success',971,'B2-20','','','','Test',636,'',150 from dual union all
select 60950,'1987-Y192','1987-06-24 14:46:00','suborbital rocket','unknown',728,'','','','','Aeron',1756,'VOKV',81 from dual union all
select 60951,'1987-Y193','1987-06-25 13:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 60952,'1987-Y194','1987-06-25 18:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 60953,'1987-Y195','1987-06-25 20:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 60954,'1987-Y196','1987-06-27 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 60955,'1987-Y197','1987-06-29 11:06:00','suborbital rocket','success',1502,'Sphere','MAC-SINE S-F11','','','Aeron',29,'',112 from dual union all
select 60956,'1987-Y198','1987-07-01 15:34:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 60957,'1987-Y199','1987-07-01 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 60958,'1987-Y20','1987-01-16 15:20:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',96 from dual union all
select 60959,'1987-Y200','1987-07-01 21:08:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F12','','','Aeron',29,'',112 from dual union all
select 6096,'1959-S324','1959-06-01 00:00:00','military missile','success',992,'S2-7','','','','OT',636,'',170 from dual union all
select 60960,'1987-Y201','1987-07-01 21:28:00','suborbital rocket','success',1284,'LIGHT CHAFF','MAC-SINE S-C13/L','','','Aeron',29,'',95 from dual union all
select 60961,'1987-Y202','1987-07-01 22:15:00','suborbital rocket','success',1284,'HEAVY CHAFF','MAC-SINE S-C14/H','','','Aeron',29,'',82 from dual union all
select 60962,'1987-Y203','1987-07-01 23:05:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F13','','','Aeron',29,'',113 from dual union all
select 60963,'1987-Y204','1987-07-01 23:49:00','suborbital rocket','success',1284,'HEAVY CHAFF','MAC-SINE S-C16/H','','','Aeron',29,'',84 from dual union all
select 60964,'1987-Y205','1987-07-02 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 60965,'1987-Y206','1987-07-03 11:00:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F14','','','Aeron',29,'',112 from dual union all
select 60966,'1987-Y207','1987-07-04 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 60967,'1987-Y208','1987-07-06 12:40:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F15','','','Aeron',29,'',106 from dual union all
select 60968,'1987-Y209','1987-07-08 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 60969,'1987-Y21','1987-01-16 17:59:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',81 from dual union all
select 6097,'1959-S325','1959-06-05 00:00:00','military missile','success',992,'S2-4','','','','OT',636,'',170 from dual union all
select 60970,'1987-Y210','1987-07-08 13:00:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F16','','','Aeron',29,'',108 from dual union all
select 60971,'1987-Y211','1987-07-08 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 60972,'1987-Y212','1987-07-08 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 60973,'1987-Y213','1987-07-10 12:12:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F17','','','Aeron',29,'',110 from dual union all
select 60974,'1987-Y214','1987-07-11 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 60975,'1987-Y215','1987-07-13 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 60976,'1987-Y216','1987-07-14 08:00:00','suborbital rocket','success',1281,'NASA 15.244UU','MAC-SINE S-SA1/L','','','Aeron',29,'',95 from dual union all
select 60977,'1987-Y217','1987-07-14 08:11:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F19','','','Aeron',29,'',108 from dual union all
select 60978,'1987-Y218','1987-07-14 08:32:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F20','','','Aeron',29,'',115 from dual union all
select 60979,'1987-Y219','1987-07-14 08:52:00','suborbital rocket','success',1284,'LIGHT CHAFF','MAC-SINE S-C17/L','','','Aeron',29,'',95 from dual union all
select 6098,'1959-S326','1959-06-06 00:00:00','military missile','success',992,'S6-3','','','','OT',636,'',170 from dual union all
select 60980,'1987-Y22','1987-01-17 17:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',82 from dual union all
select 60981,'1987-Y220','1987-07-14 09:29:00','suborbital rocket','success',1281,'NASA 15.245UU','MAC-SINE S-SA2/H','','','Aeron',29,'',96 from dual union all
select 60982,'1987-Y221','1987-07-14 09:43:00','suborbital rocket','success',1284,'LIGHT CHAFF','MAC-SINE S-C18/L','','','Aeron',29,'',96 from dual union all
select 60983,'1987-Y222','1987-07-14 10:19:00','suborbital rocket','success',1284,'HEAVY CHAFF','MAC-SINE S-C19/H','','','Aeron',29,'',80 from dual union all
select 60984,'1987-Y223','1987-07-14 11:02:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F21','','','Aeron',29,'',116 from dual union all
select 60985,'1987-Y224','1987-07-14 11:30:00','suborbital rocket','success',1284,'LIGHT CHAFF','MAC-SINE S-C20/L','','','Aeron',29,'',90 from dual union all
select 60986,'1987-Y225','1987-07-14 12:03:00','suborbital rocket','success',1284,'LIGHT CHAFF','MAC-SINE S-C21/L','','','Aeron',29,'',94 from dual union all
select 60987,'1987-Y226','1987-07-14 12:55:00','suborbital rocket','success',1281,'NASA 15.247UU','MAC-SINE S-SA3/H','','','Aeron',29,'',93 from dual union all
select 60988,'1987-Y227','1987-07-14 13:07:00','suborbital rocket','success',1284,'LIGHT CHAFF','MAC-SINE S-C22/L','','','Aeron',29,'',94 from dual union all
select 60989,'1987-Y228','1987-07-15 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 6099,'1959-S327','1959-06-11 00:00:00','military missile','success',992,'S2-3','','','','OT',636,'',170 from dual union all
select 60990,'1987-Y229','1987-07-15 12:32:00','suborbital rocket','success',1281,'NASA 15.246UU','MAC-SINE S-SA4/L','','','Aeron',29,'',98 from dual union all
select 60991,'1987-Y23','1987-01-18 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 60992,'1987-Y230','1987-07-15 12:46:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F22','','','Aeron',29,'',107 from dual union all
select 60993,'1987-Y231','1987-07-15 13:06:00','suborbital rocket','success',1284,'LIGHT CHAFF','MAC-SINE S-C23/L','','','Aeron',29,'',98 from dual union all
select 60994,'1987-Y232','1987-07-15 15:26:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 60995,'1987-Y233','1987-07-15 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',91 from dual union all
select 60996,'1987-Y234','1987-07-15 20:30:00','suborbital rocket','success',1284,'LIGHT CHAFF','MAC-SINE S-C24/L','','','Aeron',29,'',95 from dual union all
select 60997,'1987-Y235','1987-07-15 21:01:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F23','','','Aeron',29,'',117 from dual union all
select 60998,'1987-Y236','1987-07-15 21:17:00','suborbital rocket','success',1284,'LIGHT CHAFF','MAC-SINE S-C25/L','','','Aeron',29,'',97 from dual union all
select 60999,'1987-Y237','1987-07-15 21:43:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F24','','','Aeron',29,'',113 from dual union all
select 61,'1944-A25','1944-12-06 19:56:00','military missile','failure',1470,'19891','Ma341','','','Test',661,'',0 from dual union all
select 610,'1945-S86','1945-03-17 03:55:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 6100,'1959-S328','1959-06-11 00:00:00','military missile','success',992,'S2-8','','','','OT',636,'',170 from dual union all
select 61000,'1987-Y238','1987-07-15 22:01:00','suborbital rocket','success',1284,'LIGHT CHAFF','MAC-SINE S-C26/L','','','Aeron',29,'',97 from dual union all
select 61001,'1987-Y239','1987-07-15 22:30:00','suborbital rocket','success',1284,'LIGHT CHAFF','MAC-SINE S-C27/L','','','Aeron',29,'',95 from dual union all
select 61002,'1987-Y24','1987-01-20 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',95 from dual union all
select 61003,'1987-Y240','1987-07-15 22:54:00','suborbital rocket','success',1284,'LIGHT CHAFF','MAC-SINE S-C28/L','','','Aeron',29,'',98 from dual union all
select 61004,'1987-Y241','1987-07-17 11:00:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F25','','','Aeron',29,'',112 from dual union all
select 61005,'1987-Y242','1987-07-18 19:07:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61006,'1987-Y243','1987-07-19 11:07:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F26','','','Aeron',29,'',115 from dual union all
select 61007,'1987-Y244','1987-07-19 11:21:00','suborbital rocket','success',1502,'FALLING SPHERE','MAC-SINE S-F27','','','Aeron',29,'',111 from dual union all
select 61008,'1987-Y245','1987-07-21 07:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 61009,'1987-Y246','1987-07-21 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',91 from dual union all
select 6101,'1959-S329','1959-06-06 00:00:00','military missile','success',975,'I1-5T','','','','Test',636,'',80 from dual union all
select 61010,'1987-Y247','1987-07-22 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',86 from dual union all
select 61011,'1987-Y248','1987-07-22 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 61012,'1987-Y249','1987-07-22 15:42:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 61013,'1987-Y25','1987-01-20 15:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',95 from dual union all
select 61014,'1987-Y250','1987-07-22 17:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 61015,'1987-Y251','1987-07-22 18:35:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 61016,'1987-Y252','1987-07-24 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 61017,'1987-Y253','1987-07-27 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 61018,'1987-Y254','1987-07-27 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 61019,'1987-Y255','1987-07-28 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 6102,'1959-S33','1959-03-19 00:59:00','military missile','failure',150,'7C','RVX-2','R&D','','Test',227,'',200 from dual union all
select 61020,'1987-Y256','1987-07-29 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 61021,'1987-Y257','1987-07-31 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 61022,'1987-Y258','1987-08-03 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 61023,'1987-Y259','1987-08-03 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 61024,'1987-Y26','1987-01-21 11:34:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 61025,'1987-Y260','1987-08-04 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 61026,'1987-Y261','1987-08-05 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 61027,'1987-Y262','1987-08-05 15:06:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual;
insert into LAUNCH
select 61028,'1987-Y263','1987-08-05 15:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 61029,'1987-Y264','1987-08-05 16:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',88 from dual union all
select 6103,'1959-S330','1959-06-06 00:00:00','military missile','success',975,'I3-1TS','','','','Test',636,'',80 from dual union all
select 61030,'1987-Y265','1987-08-11 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',90 from dual union all
select 61031,'1987-Y266','1987-08-12 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 61032,'1987-Y267','1987-08-12 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 61033,'1987-Y268','1987-08-18 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',91 from dual union all
select 61034,'1987-Y269','1987-08-19 14:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 61035,'1987-Y27','1987-01-21 18:31:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 61036,'1987-Y270','1987-08-23 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61037,'1987-Y271','1987-08-24 14:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 61038,'1987-Y272','1987-08-25 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61039,'1987-Y273','1987-08-26 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 6104,'1959-S331','1959-06-10 00:00:00','military missile','success',975,'I3-2TS','','','','Test',636,'',80 from dual union all
select 61040,'1987-Y274','1987-08-26 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 61041,'1987-Y275','1987-08-26 14:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 61042,'1987-Y276','1987-08-30 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',89 from dual union all
select 61043,'1987-Y277','1987-08-31 16:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 61044,'1987-Y278','1987-09-01 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 61045,'1987-Y279','1987-09-02 11:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 61046,'1987-Y28','1987-01-21 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 61047,'1987-Y280','1987-09-02 14:37:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 61048,'1987-Y281','1987-09-06 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 61049,'1987-Y282','1987-09-07 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',85 from dual union all
select 6105,'1959-S332','1959-06-13 00:00:00','military missile','success',975,'I1-5TS','','','','Test',636,'',80 from dual union all
select 61050,'1987-Y283','1987-09-08 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 61051,'1987-Y284','1987-09-09 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 61052,'1987-Y285','1987-09-09 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 61053,'1987-Y286','1987-09-13 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',91 from dual union all
select 61054,'1987-Y287','1987-09-13 22:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 61055,'1987-Y288','1987-09-15 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 61056,'1987-Y289','1987-09-16 13:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 61057,'1987-Y29','1987-01-22 17:57:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',85 from dual union all
select 61058,'1987-Y290','1987-09-16 16:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 61059,'1987-Y291','1987-09-23 11:32:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 6106,'1959-S333','1959-06-15 00:00:00','military missile','success',992,'S2-2','','','','OT',636,'',170 from dual union all
select 61060,'1987-Y292','1987-09-23 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',87 from dual union all
select 61061,'1987-Y293','1987-09-29 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 61062,'1987-Y294','1987-09-30 13:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 61063,'1987-Y295','1987-09-30 14:11:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 61064,'1987-Y296','1987-10-06 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 61065,'1987-Y297','1987-10-07 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 61066,'1987-Y298','1987-10-07 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 61067,'1987-Y299','1987-10-12 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 61068,'1987-Y30','1987-01-22 20:27:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',85 from dual union all
select 61069,'1987-Y300','1987-10-14 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 6107,'1959-S334','1959-06-16 00:00:00','military missile','success',992,'S2-6','','','','OT',636,'',170 from dual union all
select 61070,'1987-Y301','1987-10-14 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 61071,'1987-Y302','1987-10-15 10:47:00','suborbital rocket','success',1502,'Sphere','MAC/EPSILON E-F1','','','Aeron',29,'',114 from dual union all
select 61072,'1987-Y303','1987-10-15 10:57:00','suborbital rocket','success',1284,'Light Chaff','MAC/EPSILON E-C1/L','','','Aeron',29,'',87 from dual union all
select 61073,'1987-Y304','1987-10-15 11:32:00','suborbital rocket','success',1284,'FALLING SPHERE','MAC/EPSILON TYL-1697','','','Aeron',29,'',93 from dual union all
select 61074,'1987-Y305','1987-10-15 12:14:00','suborbital rocket','success',1284,'FALLING SPHERE','MAC/EPSILON TY1-1696','','','Aeron',29,'',96 from dual union all
select 61075,'1987-Y306','1987-10-15 13:08:00','suborbital rocket','success',1502,'Chaff','MAC/EPSILON E-C3/SL','','','Aeron',29,'',114 from dual union all
select 61076,'1987-Y307','1987-10-16 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 61077,'1987-Y308','1987-10-19 12:35:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 61078,'1987-Y309','1987-10-19 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 61079,'1987-Y31','1987-01-24 20:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',83 from dual union all
select 6108,'1959-S335','1959-07-03 00:00:00','military missile','success',992,'S3-1','','','','OT',636,'',170 from dual union all
select 61080,'1987-Y310','1987-10-21 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 61081,'1987-Y311','1987-10-21 21:28:00','suborbital rocket','success',1502,'Sphere','MAC/EPSILON E-F2','','','Aeron',29,'',113 from dual union all
select 61082,'1987-Y312','1987-10-21 21:46:00','suborbital rocket','success',1502,'Chaff','MAC/EPSILON E-C6/SL','','','Aeron',29,'',101 from dual union all
select 61083,'1987-Y313','1987-10-21 22:08:00','suborbital rocket','unknown',1284,'FALLING SPHERE','MAC/EPSILON TY1-1698','','','Aeron',29,'',90 from dual union all
select 61084,'1987-Y314','1987-10-21 22:28:00','suborbital rocket','unknown',1284,'FALLING SPHERE','MAC/EPSILON TY1-1699','','','Aeron',29,'',91 from dual union all
select 61085,'1987-Y315','1987-10-21 22:37:00','suborbital rocket','success',1284,'Chaff','MAC/EPSILON E-C7/L','','','Aeron',29,'',82 from dual union all
select 61086,'1987-Y316','1987-10-21 23:21:00','suborbital rocket','success',1502,'Chaff','MAC/EPSILON E-C8/SL','','','Aeron',29,'',102 from dual union all
select 61087,'1987-Y317','1987-10-23 12:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 61088,'1987-Y318','1987-10-23 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61089,'1987-Y319','1987-10-24 11:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',85 from dual union all
select 6109,'1959-S336','1959-07-08 00:00:00','military missile','success',992,'S2-12','','','','OT',636,'',170 from dual union all
select 61090,'1987-Y32','1987-01-25 18:02:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',82 from dual union all
select 61091,'1987-Y320','1987-10-26 10:01:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',83 from dual union all
select 61092,'1987-Y321','1987-10-26 12:22:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 61093,'1987-Y322','1987-10-26 15:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 61094,'1987-Y323','1987-10-27 12:04:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',80 from dual union all
select 61095,'1987-Y324','1987-10-28 00:11:00','suborbital rocket','unknown',1284,'FALLING SPHERE','MAC/EPSILON TY1-1700','','','Aeron',29,'',92 from dual union all
select 61096,'1987-Y325','1987-10-28 00:31:00','suborbital rocket','unknown',1284,'FALLING SPHERE','MAC/EPSILON TY1-1701','','','Aeron',29,'',91 from dual union all
select 61097,'1987-Y326','1987-10-28 11:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 61098,'1987-Y327','1987-10-28 16:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 61099,'1987-Y328','1987-10-28 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 611,'1945-S87','1945-03-17 20:21:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 6110,'1959-S337','1959-07-08 00:00:00','military missile','success',992,'S2-18','','','','OT',636,'',170 from dual union all
select 61100,'1987-Y329','1987-10-30 16:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 61101,'1987-Y33','1987-01-25 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 61102,'1987-Y330','1987-10-30 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61103,'1987-Y331','1987-11-01 14:30:00','suborbital rocket','success',1284,'Chaff Test 2','MAC/EPSILON E-C9/L','','','Aeron',29,'',82 from dual union all
select 61104,'1987-Y332','1987-11-02 12:04:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 61105,'1987-Y333','1987-11-02 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 61106,'1987-Y334','1987-11-04 13:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',87 from dual union all
select 61107,'1987-Y335','1987-11-04 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 61108,'1987-Y336','1987-11-04 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 61109,'1987-Y337','1987-11-06 05:22:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 6111,'1959-S338','1959-07-09 00:00:00','military missile','success',992,'S2-14','','','','OT',636,'',170 from dual union all
select 61110,'1987-Y338','1987-11-06 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 61111,'1987-Y339','1987-11-09 14:16:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',90 from dual union all
select 61112,'1987-Y34','1987-01-26 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',87 from dual union all
select 61113,'1987-Y340','1987-11-09 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61114,'1987-Y341','1987-11-11 13:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 61115,'1987-Y342','1987-11-11 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61116,'1987-Y343','1987-11-11 23:58:00','suborbital rocket','success',1502,'Sphere','MAC/EPSILON E-F3','','','Aeron',29,'',107 from dual union all
select 61117,'1987-Y344','1987-11-12 00:16:00','suborbital rocket','success',1502,'Sphere','MAC/EPSILON E-F4','','','Aeron',29,'',106 from dual;
insert into LAUNCH
select 61118,'1987-Y345','1987-11-12 00:41:00','suborbital rocket','unknown',1284,'FALLING SPHERE','MAC/EPSILON TY1-1702','','','Aeron',29,'',95 from dual union all
select 61119,'1987-Y346','1987-11-12 01:01:00','suborbital rocket','unknown',1284,'FALLING SPHERE','MAC/EPSILON TY1-1703','','','Aeron',29,'',90 from dual union all
select 6112,'1959-S339','1959-06-20 00:00:00','military missile','success',969,'54010808','','','','OT',636,'',100 from dual union all
select 61120,'1987-Y347','1987-11-13 12:58:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',81 from dual union all
select 61121,'1987-Y348','1987-11-13 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 61122,'1987-Y349','1987-11-15 12:18:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',84 from dual union all
select 61123,'1987-Y35','1987-01-26 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',85 from dual union all
select 61124,'1987-Y350','1987-11-16 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',87 from dual union all
select 61125,'1987-Y351','1987-11-16 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 61126,'1987-Y352','1987-11-18 16:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 61127,'1987-Y353','1987-11-18 17:31:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 61128,'1987-Y354','1987-11-20 11:32:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',87 from dual union all
select 61129,'1987-Y355','1987-11-20 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 6113,'1959-S34','1959-03-21 06:19:00','military missile','success',1355,'132','RVX-1','','','Test',233,'',1230 from dual union all
select 61130,'1987-Y356','1987-11-23 10:44:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',85 from dual union all
select 61131,'1987-Y357','1987-11-24 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61132,'1987-Y36','1987-01-27 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61133,'1987-Y360','1987-11-29 09:07:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',83 from dual union all
select 61134,'1987-Y361','1987-11-30 07:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',82 from dual union all
select 61135,'1987-Y362','1987-12-01 09:02:00','suborbital rocket','unknown',728,'','','','','Aeron',1385,'SHIR',83 from dual union all
select 61136,'1987-Y363','1987-12-01 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 61137,'1987-Y364','1987-12-02 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 61138,'1987-Y365','1987-12-03 03:20:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',88 from dual union all
select 61139,'1987-Y366','1987-12-03 14:00:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',84 from dual union all
select 6114,'1959-S340','1959-06-20 00:00:00','military missile','success',969,'54010809','','','','OT',636,'',100 from dual union all
select 61140,'1987-Y367','1987-12-05 03:20:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',85 from dual union all
select 61141,'1987-Y368','1987-12-05 08:20:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',80 from dual union all
select 61142,'1987-Y369','1987-12-05 09:50:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',85 from dual union all
select 61143,'1987-Y37','1987-01-27 19:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',83 from dual union all
select 61144,'1987-Y370','1987-12-05 14:07:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',82 from dual union all
select 61145,'1987-Y371','1987-12-05 17:45:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',85 from dual union all
select 61146,'1987-Y372','1987-12-05 18:53:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',82 from dual union all
select 61147,'1987-Y373','1987-12-05 22:30:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',85 from dual union all
select 61148,'1987-Y374','1987-12-07 09:52:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',85 from dual union all
select 61149,'1987-Y375','1987-12-07 14:00:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',83 from dual union all
select 6115,'1959-S341','1959-06-17 00:00:00','military missile','success',975,'I3-4TS','','','','Test',636,'',80 from dual union all
select 61150,'1987-Y376','1987-12-07 16:00:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',80 from dual union all
select 61151,'1987-Y377','1987-12-07 18:04:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',88 from dual union all
select 61152,'1987-Y378','1987-12-07 22:55:00','suborbital rocket','success',728,'','Ozone Diurnal','','','Aeron',1467,'',84 from dual union all
select 61153,'1987-Y379','1987-12-08 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 61154,'1987-Y38','1987-01-27 20:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 61155,'1987-Y380','1987-12-09 13:56:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 61156,'1987-Y381','1987-12-09 15:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 61157,'1987-Y382','1987-12-15 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 61158,'1987-Y383','1987-12-16 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 61159,'1987-Y384','1987-12-22 19:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 6116,'1959-S342','1959-06-18 00:00:00','military missile','success',975,'I3-3TS','','','','Test',636,'',80 from dual union all
select 61160,'1987-Y385','1987-12-23 05:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 61161,'1987-Y386','1987-12-23 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 61162,'1987-Y387','1987-12-27 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 61163,'1987-Y388','1987-12-29 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 61164,'1987-Y389','1987-12-30 06:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 61165,'1987-Y39','1987-01-27 22:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61166,'1987-Y390','1987-12-30 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 61167,'1987-Y391','1987-06-22 12:30:00','suborbital rocket','success',1284,'Chaff','MAC-SINE S-C02/H','','','Aeron',29,'',84 from dual union all
select 61168,'1987-Y392','1987-06-24 11:20:00','suborbital rocket','success',1284,'Chaff','MAC-SINE S-C04/L','','','Aeron',29,'',93 from dual union all
select 61169,'1987-Y393','1987-06-24 11:53:00','suborbital rocket','success',1284,'Chaff','MAC-SINE S-C05/L','','','Aeron',29,'',95 from dual union all
select 6117,'1959-S343','1959-06-20 00:00:00','military missile','success',975,'I1-14','','','','Test',636,'',80 from dual union all
select 61170,'1987-Y394','1987-06-24 12:40:00','suborbital rocket','success',1284,'Chaff','MAC-SINE S-C07/L','','','Aeron',29,'',97 from dual union all
select 61171,'1987-Y395','1987-06-24 13:33:00','suborbital rocket','success',1284,'Chaff','MAC-SINE S-C08/L','','','Aeron',29,'',99 from dual union all
select 61172,'1987-Y396','1987-06-26 11:27:00','suborbital rocket','success',1284,'Chaff','MAC-SINE S-C09/L','','','Aeron',29,'',95 from dual union all
select 61173,'1987-Y397','1987-06-26 12:44:00','suborbital rocket','success',1284,'Chaff','MAC-SINE S-C11/L','','','Aeron',29,'',96 from dual union all
select 61174,'1987-Y398','1987-06-26 13:36:00','suborbital rocket','success',1284,'Chaff','MAC-SINE S-C12/H','','','Aeron',29,'',86 from dual union all
select 61175,'1987-Y399','1987-06-10 13:40:00','suborbital rocket','success',1502,'Sphere','MAC-SINE S-F1','','','Aeron',29,'',116 from dual union all
select 61176,'1987-Y40','1987-01-28 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 61177,'1987-Y400','1987-06-12 11:00:00','suborbital rocket','success',1502,'Sphere','MAC-SINE S-F2','','','Aeron',29,'',115 from dual union all
select 61178,'1987-Y401','1987-06-15 11:24:00','suborbital rocket','success',1502,'Sphere','MAC-SINE S-F3','','','Aeron',29,'',109 from dual union all
select 61179,'1987-Y402','1987-06-17 11:03:00','suborbital rocket','success',1502,'Sphere','MAC-SINE S-F4','','','Aeron',29,'',115 from dual union all
select 6118,'1959-S344','1959-07-03 00:00:00','military missile','failure',975,'I1-23TS','','','','Test',636,'',80 from dual union all
select 61180,'1987-Y403','1987-06-19 11:05:00','suborbital rocket','success',1502,'Sphere','MAC-SINE S-F5','','','Aeron',29,'',113 from dual union all
select 61181,'1987-Y404','1987-06-22 11:29:00','suborbital rocket','success',1502,'Sphere','MAC-SINE S-F6','','','Aeron',29,'',114 from dual union all
select 61182,'1987-Y405','1987-06-24 11:00:00','suborbital rocket','success',1502,'Sphere','MAC-SINE S-F7','','','Aeron',29,'',105 from dual union all
select 61183,'1987-Y406','1987-06-24 12:25:00','suborbital rocket','success',1284,'Chaff','MAC-SINE S-C06/L','','','Aeron',29,'',83 from dual union all
select 61184,'1987-Y407','1987-06-22 13:11:00','suborbital rocket','failure',1284,'Chaff','MAC-SINE S-C03/H','','','Aeron',29,'',80 from dual union all
select 61185,'1987-Y408','1987-06-24 13:58:00','suborbital rocket','success',1502,'Sphere','MAC-SINE S-F8','','','Aeron',29,'',112 from dual union all
select 61186,'1987-Y409','1987-06-26 11:00:00','suborbital rocket','success',1502,'Sphere','MAC-SINE S-F9','','','Aeron',29,'',115 from dual union all
select 61187,'1987-Y41','1987-01-28 14:04:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 61188,'1987-Y410','1987-06-26 14:12:00','suborbital rocket','success',1502,'Sphere','MAC-SINE S-F10','','','Aeron',29,'',114 from dual union all
select 61189,'1987-Y411','1987-06-26 12:01:00','suborbital rocket','success',1284,'Chaff','MAC-SINE S-C10/H','','','Aeron',29,'',83 from dual union all
select 6119,'1959-S345','1959-07-13 00:00:00','military missile','success',992,'S2-19','','','','OT',636,'',170 from dual union all
select 61190,'1987-Y412','1987-10-15 13:32:00','suborbital rocket','success',1284,'Chaff','MAC/EPSILON E-C4/L','','','Aeron',29,'',84 from dual union all
select 61191,'1987-Y413','1987-10-17 11:38:00','suborbital rocket','success',1502,'Chaff Test 1','MAC/EPSILON E-C5/SL','','','Aeron',29,'',106 from dual union all
select 61192,'1987-Y42','1987-01-28 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 61193,'1987-Y43','1987-01-28 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',82 from dual union all
select 61194,'1987-Y44','1987-01-29 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',81 from dual union all
select 61195,'1987-Y45','1987-01-29 18:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 61196,'1987-Y46','1987-01-30 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',82 from dual union all
select 61197,'1987-Y47','1987-01-30 18:06:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',81 from dual union all
select 61198,'1987-Y48','1987-01-31 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',88 from dual union all
select 61199,'1987-Y49','1987-02-01 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',82 from dual union all
select 612,'1945-S88','1945-03-18 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 6120,'1959-S346','1959-07-13 00:00:00','military missile','success',992,'S3-2','','','','OT',636,'',170 from dual union all
select 61200,'1987-Y50','1987-02-01 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',83 from dual union all
select 61201,'1987-Y51','1987-02-01 20:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',93 from dual union all
select 61202,'1987-Y52','1987-02-02 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',85 from dual union all
select 61203,'1987-Y53','1987-02-02 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',85 from dual union all
select 61204,'1987-Y54','1987-02-03 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',84 from dual union all
select 61205,'1987-Y55','1987-02-03 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61206,'1987-Y56','1987-02-04 11:59:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 61207,'1987-Y57','1987-02-04 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual;
insert into LAUNCH
select 61208,'1987-Y58','1987-02-04 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',82 from dual union all
select 61209,'1987-Y59','1987-02-04 19:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',82 from dual union all
select 6121,'1959-S347','1959-07-14 00:00:00','military missile','success',992,'S2-17','','','','OT',636,'',170 from dual union all
select 61210,'1987-Y60','1987-02-05 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',85 from dual union all
select 61211,'1987-Y61','1987-02-05 19:04:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',84 from dual union all
select 61212,'1987-Y62','1987-02-06 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1384,'SHIR',83 from dual union all
select 61213,'1987-Y63','1987-02-06 18:04:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',83 from dual union all
select 61214,'1987-Y64','1987-02-07 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',87 from dual union all
select 61215,'1987-Y65','1987-02-08 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',88 from dual union all
select 61216,'1987-Y66','1987-02-08 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 61217,'1987-Y67','1987-02-09 12:36:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',89 from dual union all
select 61218,'1987-Y68','1987-02-09 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',82 from dual union all
select 61219,'1987-Y69','1987-02-10 20:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',80 from dual union all
select 6122,'1959-S348','1959-07-14 00:00:00','military missile','success',992,'S3-4','','','','OT',636,'',170 from dual union all
select 61220,'1987-Y70','1987-02-11 12:12:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 61221,'1987-Y71','1987-02-11 14:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 61222,'1987-Y72','1987-02-11 15:23:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 61223,'1987-Y73','1987-02-11 18:05:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',83 from dual union all
select 61224,'1987-Y74','1987-02-12 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',86 from dual union all
select 61225,'1987-Y75','1987-07-13 19:55:00','suborbital rocket','success',1502,'S-F18','MAC-SINE S-F18','','','Aeron',29,'',105 from dual union all
select 61226,'1987-Y76','1987-02-15 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 61227,'1987-Y77','1987-02-16 12:51:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 61228,'1987-Y78','1987-02-17 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61229,'1987-Y79','1987-02-18 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 6123,'1959-S349','1959-07-16 00:00:00','military missile','success',992,'S2-13','','','','OT',636,'',170 from dual union all
select 61230,'1987-Y80','1987-02-18 12:54:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 61231,'1987-Y81','1987-02-18 15:37:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 61232,'1987-Y82','1987-02-22 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 61233,'1987-Y83','1987-02-23 13:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 61234,'1987-Y84','1987-02-23 14:04:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 61235,'1987-Y85','1987-02-24 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 61236,'1987-Y86','1987-02-25 12:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 61237,'1987-Y87','1987-02-27 13:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 61238,'1987-Y88','1987-03-01 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 61239,'1987-Y89','1987-03-02 13:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 6124,'1959-S35','1959-03-22 00:58:00','military missile','success',1367,'158','R&D Series IV','','','Test',236,'',520 from dual union all
select 61240,'1987-Y90','1987-03-02 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 61241,'1987-Y91','1987-03-03 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61242,'1987-Y92','1987-03-04 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 61243,'1987-Y93','1987-03-05 12:26:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',84 from dual union all
select 61244,'1987-Y94','1987-03-08 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61245,'1987-Y95','1987-03-09 10:00:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',84 from dual union all
select 61246,'1987-Y96','1987-03-09 14:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 61247,'1987-Y97','1987-03-10 20:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',91 from dual union all
select 61248,'1987-Y98','1987-03-11 10:06:00','suborbital rocket','unknown',728,'','','','','Aeron',895,'KORO',87 from dual union all
select 61249,'1987-Y99','1987-03-11 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 6125,'1959-S350','1959-07-16 00:00:00','military missile','success',992,'S2-16','','','','OT',636,'',170 from dual union all
select 61250,'1988-001','1988-01-06 07:41:00','orbital','success',1448,'45076-110','Kosmos-1908','Tselina-D','','Sat',470,'',null from dual union all
select 61251,'1988-002','1988-01-15 03:49:21','orbital','success',1448,'67038-405','Kosmos-1909/1914','Strela-3','','Sat',469,'',null from dual union all
select 61252,'1988-003','1988-01-20 22:51:54','orbital','success',1242,'L 15000-025','Progress-34','Progress 7K-TG No. 142','','Sat',480,'',null from dual union all
select 61253,'1988-004','1988-01-26 11:20:00','orbital','success',1238,'77015  -363','Kosmos-1915','Oblik','','Sat',473,'',null from dual union all
select 61254,'1988-005','1988-01-30 10:59:59','orbital','success',1448,'51038-404','Meteor-2','Meteor-2 No. 20','','Sat',469,'',null from dual union all
select 61255,'1988-006','1988-02-03 05:53:00','orbital','success',155,'54E','USA 29 (DMSP F9)','DMSP Block 5D-2 S-8','','Sat',1776,'',null from dual union all
select 61256,'1988-007','1988-02-03 12:14:56','orbital','success',1238,'78028  -932','Kosmos-1916','Kobal''t','','Sat',562,'',null from dual union all
select 61257,'1988-008','1988-02-08 22:07:00','orbital','success',387,'D181','USA 30 (SDIO D181)','Canister Cluster','','Sat',234,'',null from dual union all
select 61258,'1988-009','1988-02-17 00:23:22','orbital','failure',958,'346-02','Kosmos-1917/1919','Uragan No. 36L','','Sat',553,'',314 from dual union all
select 61259,'1988-010','1988-02-18 09:49:59','orbital','success',1238,'76035  -107','Kosmos-1920','Resurs-F1 14F40 No. 106','','Sat',462,'',null from dual union all
select 6126,'1959-S351','1959-07-20 00:00:00','military missile','success',992,'53060119','','','','OT',636,'',170 from dual union all
select 61260,'1988-011','1988-02-19 08:00:00','orbital','success',1238,'76035  -371','Kosmos-1921','Oblik','','Sat',480,'',null from dual union all
select 61261,'1988-012','1988-02-19 10:05:00','orbital','success',524,'H-18(F)','CS 3A','CS-3A','','Sat',1474,'',null from dual union all
select 61262,'1988-013','1988-02-26 09:31:12','orbital','success',803,'77015  -528','Kosmos-1922','Oko','','Sat',473,'',null from dual union all
select 61263,'1988-014','1988-03-07 12:41:00','orbital','success',308,'Y4       (17)','Zhongxing 1','DFH-2A','','Sat',1904,'',null from dual union all
select 61264,'1988-015','1988-03-10 10:30:00','orbital','success',1238,'78028  -360','Kosmos-1923','Oblik','','Sat',475,'',null from dual union all
select 61265,'1988-016','1988-03-11 00:18:00','orbital','success',680,'47161-263','Kosmos-1924/1931','Strela-1M','','Sat',456,'',null from dual union all
select 61266,'1988-017','1988-03-11 06:38:07','orbital','success',805,'78056  -678','Molniya-1','Molniya-1!No. 65','','Sat',480,'',null from dual union all
select 61267,'1988-018','1988-03-11 23:28:00','orbital','success',93,'V21 L21','Tel. 1C/Spacenet 3R','Spacenet 3R','','Sat',344,'',null from dual union all
select 61268,'1988-019','1988-03-14 14:21:00','orbital','success',1446,'67074-702','Kosmos-1932','US-A reactor','','Sat',589,'',null from dual union all
select 61269,'1988-020','1988-03-15 18:50:00','orbital','success',1448,'51095-602','Kosmos-1933','Tselina-D','','Sat',470,'',null from dual union all
select 6127,'1959-S352','1959-07-20 00:00:00','military missile','success',992,'S3-3','','','','OT',636,'',170 from dual union all
select 61270,'1988-021','1988-03-17 06:43:00','orbital','success',1510,'L 15000-79','IRS-1','IRS-1A','','Sat',562,'',null from dual union all
select 61271,'1988-022','1988-03-17 20:55:59','orbital','success',805,'78028  -513','Molniya-1','Molniya-1!No. 64','','Sat',475,'',null from dual union all
select 61272,'1988-023','1988-03-22 14:07:23','orbital','success',680,'47111-221','Kosmos-1934','Parus','','Sat',456,'',null from dual union all
select 61273,'1988-024','1988-03-23 21:05:12','orbital','success',1242,'L 15000-026','Progress-35','Progress 7K-TG No. 143','','Sat',480,'',null from dual union all
select 61274,'1988-025','1988-03-24 14:10:00','orbital','success',1238,'76048  -850','Kosmos-1935','Kobal''t','','Sat',462,'',null from dual union all
select 61275,'1988-026','1988-03-25 19:50:00','orbital','success',1135,'S206C','San Marco 5','San Marco D/L','','Sat',1412,'',null from dual union all
select 61276,'1988-027','1988-03-30 12:00:00','orbital','success',1238,'77015  -365','Kosmos-1936','Terilen','','Sat',480,'',null from dual union all
select 61277,'1988-028','1988-03-31 04:18:00','orbital','success',956,'343-02','Gorizont','Gorizont No. 26L','','Sat',554,'',null from dual union all
select 61278,'1988-029','1988-04-05 14:41:00','orbital','success',680,'47126-188','Kosmos-1937','Strela-2M','','Sat',460,'',null from dual union all
select 61279,'1988-030','1988-04-11 11:15:00','orbital','success',1238,'77015  -367','Kosmos-1938','Oblik','','Sat',462,'',null from dual union all
select 6128,'1959-S353','1959-07-20 00:00:00','military missile','success',992,'S3-6','','','','OT',636,'',170 from dual union all
select 61280,'1988-031','1988-04-14 17:00:00','orbital','success',1238,'B 15000-480','Foton','Foton No. 4L','','Sat',473,'',null from dual union all
select 61281,'1988-032','1988-04-20 05:48:12','orbital','success',1510,'I 15000-78','Kosmos-1939','Resurs-O1 No. 2L','','Sat',562,'',null from dual union all
select 61282,'1988-033','1988-04-26 01:57:00','orbital','success',1135,'S211C','SOOS 3','NNS O-23','','Sat',1779,'',null from dual union all
select 61283,'1988-034','1988-04-26 03:15:10','orbital','success',958,'332-01','Kosmos-1940','Geofizika?','','Sat',553,'',null from dual union all
select 61284,'1988-035','1988-04-27 09:10:00','orbital','success',1238,'76035  -375','Kosmos-1941','Oblik','','Sat',562,'',null from dual union all
select 61285,'1988-036','1988-05-06 02:47:00','orbital','success',956,'349-01','Ekran','Ekran No. 31L','','Sat',553,'',null from dual union all
select 61286,'1988-037','1988-05-12 14:39:59','orbital','success',1238,'77015  -106','Kosmos-1942','Kobal''t','','Sat',475,'',null from dual union all
select 61287,'1988-038','1988-05-13 00:30:25','orbital','success',1242,'L 15000-023','Progress-36','Progress 7K-TG No. 144','','Sat',480,'',null from dual union all
select 61288,'1988-039','1988-05-15 09:20:00','orbital','success',1537,'51065-615','Kosmos-1943','Tselina-2','','Sat',567,'',null from dual union all
select 61289,'1988-040','1988-05-17 23:58:00','orbital','success',92,'V23 L22','Intelsat 513','INTELSAT VA F13','','Sat',344,'',null from dual union all
select 6129,'1959-S354','1959-07-14 00:00:00','military missile','success',975,'I4-7T','','','','Test',636,'',80 from dual union all
select 61290,'1988-041','1988-05-18 10:30:01','orbital','success',1238,'76035  -460','Kosmos-1944','Kometa No. 9','','Sat',480,'',null from dual union all
select 61291,'1988-042','1988-05-19 09:15:00','orbital','success',1238,'76035  -378','Kosmos-1945','Oblik','','Sat',562,'',null from dual union all
select 61292,'1988-043','1988-05-21 17:57:00','orbital','success',958,'348-01','Kosmos-1946/1948','Uragan No. 39L','','Sat',553,'',null from dual union all
select 61293,'1988-044','1988-05-26 15:27:00','orbital','success',805,'77011  -692','Molniya-3','Molniya-3 No. 49','','Sat',475,'',null from dual union all
select 61294,'1988-045','1988-05-28 02:49:00','orbital','success',1446,'45074-703','Kosmos-1949','US-PU','','Sat',589,'',null from dual union all
select 61295,'1988-046','1988-05-30 08:00:00','orbital','success',1448,'45095-601','Kosmos-1950','Musson No. 21','','Sat',469,'',null from dual union all
select 61296,'1988-047','1988-05-31 07:45:00','orbital','success',1238,'78039  -128','Kosmos-1951','Resurs-F1 14F43 No. 28','','Sat',473,'',null from dual union all
select 61297,'1988-048','1988-06-07 14:03:13','orbital','success',1242,'I 15000-027','Soyuz TM-5','Soyuz 7K-STM No. 55','','Sat',480,'',null from dual union all
select 61298,'1988-049','1988-06-11 10:00:00','orbital','success',1238,'76035  -376','Kosmos-1952','Oblik','','Sat',480,'',null from dual;
insert into LAUNCH
select 61299,'1988-050','1988-06-14 03:18:00','orbital','success',1448,'45058-406','Kosmos-1953','Tselina-D','','Sat',470,'',null from dual union all
select 613,'1945-S89','1945-03-21 09:34:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 6130,'1959-S355','1959-07-15 00:00:00','military missile','success',975,'I1-18TS','','','','Test',636,'',80 from dual union all
select 61300,'1988-051','1988-06-15 11:19:01','orbital','success',106,'V22 L401','Meteosat 3/PAS 1','Meteosat P2','','Sat',345,'',null from dual union all
select 61301,'1988-052','1988-06-16 06:54:00','orbital','success',1135,'S213C','Nova 2','Nova 2','','Sat',1779,'',null from dual union all
select 61302,'1988-053','1988-06-21 16:26:00','orbital','success',680,'47126-191','Kosmos-1954','Strela-2M','','Sat',460,'',null from dual union all
select 61303,'1988-054','1988-06-22 13:00:00','orbital','success',1238,'76035  -373','Kosmos-1955','Kobal''t','','Sat',480,'',null from dual union all
select 61304,'1988-055','1988-06-23 07:45:00','orbital','success',1238,'77015  -366','Kosmos-1956','Oblik','','Sat',473,'',null from dual union all
select 61305,'1988-056','1988-07-05 09:45:00','orbital','success',1448,'45065-201','Okean','Okean-O1 NKhM No. 5','','Sat',469,'',null from dual union all
select 61306,'1988-057','1988-07-07 08:05:00','orbital','success',1238,'78039  -129','Kosmos-1957','Resurs-F1 14F43 No. 29','','Sat',462,'',null from dual union all
select 61307,'1988-058','1988-07-07 17:38:04','deep space','success',955,'356-02','Fobos-1','1F No. 101','','Sat',553,'',null from dual union all
select 61308,'1988-059','1988-07-12 17:01:43','deep space','success',955,'356-01','Fobos-2','1F No. 102','','Sat',554,'',null from dual union all
select 61309,'1988-060','1988-07-14 11:40:00','orbital','success',680,'47111-224','Kosmos-1958','Vektor','','Sat',456,'',null from dual union all
select 6131,'1959-S356','1959-07-20 00:00:00','military missile','success',975,'I1-20TS','','','','Test',636,'',80 from dual union all
select 61310,'1988-061','1988-07-18 21:13:09','orbital','success',1242,'L 15000-024','Progress-37','Progress 7K-TG No. 145','','Sat',480,'',null from dual union all
select 61311,'1988-062','1988-07-18 22:28:11','orbital','success',680,'65096-175','Kosmos-1959','Parus','','Sat',460,'',null from dual union all
select 61312,'1988-063','1988-07-21 23:12:00','orbital','success',93,'V24 L23','Insat 1C/ECS 5','Insat 1C','','Sat',344,'',null from dual union all
select 61313,'1988-064','1988-07-26 05:01:00','orbital','success',1448,'67061-506','Meteor-3','Meteor-3 No. 3','','Sat',470,'',null from dual union all
select 61314,'1988-065','1988-07-28 11:20:00','orbital','success',680,'65026-187','Kosmos-1960','Romb','','Sat',456,'',null from dual union all
select 61315,'1988-066','1988-08-01 21:04:00','orbital','success',958,'351-01','Kosmos-1961','Geizer No. 16L','','Sat',553,'',null from dual union all
select 61316,'1988-067','1988-08-05 07:28:35','orbital','success',301,'Y8       (18)','FSW-1','FSW-1 RV','','Sat',810,'',null from dual union all
select 61317,'1988-068','1988-08-08 09:25:00','orbital','success',1238,'76035  -946','Kosmos-1962','Oblik','','Sat',480,'',null from dual union all
select 61318,'1988-069','1988-08-12 12:53:00','orbital','success',805,'76035  -532','Molniya-1','Molniya-1 No. 66','','Sat',473,'',null from dual union all
select 61319,'1988-070','1988-08-16 13:00:00','orbital','success',1238,'76035  -947','Kosmos-1963','Kobal''t','','Sat',480,'',null from dual union all
select 6132,'1959-S357','1959-07-24 00:00:00','military missile','success',975,'I1-24TS','','','','Test',636,'',80 from dual union all
select 61320,'1988-071','1988-08-18 19:52:00','orbital','success',958,'333-02','Gorizont','Gorizont No. 28L','','Sat',554,'',null from dual union all
select 61321,'1988-072','1988-08-23 09:20:00','orbital','success',1238,'78039  -381','Kosmos-1964','Oblik','','Sat',480,'',null from dual union all
select 61322,'1988-073','1988-08-23 11:15:00','orbital','success',1238,'E 15000-569','Kosmos-1965','Resurs-F2 17F42 No. 2','','Sat',473,'',null from dual union all
select 61323,'1988-074','1988-08-25 06:59:00','orbital','success',1135,'S214C','SOOS 4','NNS O-25','','Sat',1779,'',null from dual union all
select 61324,'1988-075','1988-08-29 04:23:11','orbital','success',1242,'L 15000-030','Soyuz TM-6','Soyuz 7K-STM No. 56','','Sat',480,'',null from dual union all
select 61325,'1988-076','1988-08-30 14:14:54','orbital','success',803,'77015  -526','Kosmos-1966','Oko','','Sat',462,'',null from dual union all
select 61326,'1988-077','1988-09-02 12:05:02','orbital','success',1408,'34D-3  (05D-5)','USA 31       [CY12]','VORTEX 5','','Sat',274,'',null from dual union all
select 61327,'1988-078','1988-09-05 09:25:00','orbital','success',1422,'23G-1','USA 32       [T2-1]','P-11 No. 5103','','Sat',1778,'',null from dual union all
select 61328,'1988-079','1988-09-06 07:30:01','orbital','success',1238,'76035  -377','Kosmos-1967','Oblik','','Sat',462,'',null from dual union all
select 61329,'1988-080','1988-09-06 20:30:19','orbital','success',318,'Y1       (19)','Feng Yun 1','feng yun yi jia','','Sat',1492,'',null from dual union all
select 6133,'1959-S358','1959-07-24 00:00:00','military missile','success',975,'I4-1','','','','Test',636,'',80 from dual union all
select 61330,'1988-081','1988-09-08 23:00:00','orbital','success',93,'V25 L24','Gstar 3/SBS 5','Gstar 3','','Sat',345,'',null from dual union all
select 61331,'1988-082','1988-09-09 10:40:00','orbital','success',1238,'78039  -131','Kosmos-1968','Resurs-F1 14F43 No. 31','','Sat',473,'',null from dual union all
select 61332,'1988-083','1988-09-09 23:33:40','orbital','success',1242,'Ye15000-031','Progress-38','Progress 7K-TG No. 146','','Sat 76048-930',480,'',null from dual union all
select 61333,'1988-084','1988-09-15 15:00:00','orbital','success',1238,'76048  -930','Kosmos-1969','Kobal''t','','Sat',473,'',null from dual union all
select 61334,'1988-085','1988-09-16 02:00:47','orbital','success',958,'349-02','Kosmos-1970/1972','Uragan No. 42L','','Sat',553,'',null from dual union all
select 61335,'1988-086','1988-09-16 09:59:00','orbital','success',524,'H-19(F)','CS 3B','CS-3B','','Sat',1474,'',null from dual union all
select 61336,'1988-087','1988-09-19 09:34:00','orbital','success',1173,'1','''Ofeq-1','''Ofeq-1','','Sat',1026,'',null from dual union all
select 61337,'1988-088','1988-09-22 10:20:00','orbital','success',1241,'76035  -379','Kosmos-1973','Oblik','','Sat',462,'',null from dual union all
select 61338,'1988-089','1988-09-24 10:02:00','orbital','success',155,'63E','NOAA 11','NOAA H','','Sat',1776,'',null from dual union all
select 61339,'1988-090','1988-09-29 09:07:00','orbital','success',805,'76048  -510','Molniya-3','Molniya-3 No. 51','','Sat',473,'',null from dual union all
select 6134,'1959-S359','1959-07-27 00:00:00','military missile','success',975,'I10-10T','','','','Test',636,'',80 from dual union all
select 61340,'1988-091','1988-09-29 15:37:00','orbital','success',1243,'STS-26R','TDRS-C','OV-103 IUS deploy','','Sat',922,'MLP2',null from dual union all
select 61341,'1988-092','1988-10-03 22:23:39','orbital','success',803,'76035  -531','Kosmos-1974','Oko','','Sat',473,'',null from dual union all
select 61342,'1988-093','1988-10-11 08:02:00','orbital','success',1448,'45038-403','Kosmos-1975','Tselina-D','','Sat',469,'',null from dual union all
select 61343,'1988-094','1988-10-13 11:19:59','orbital','success',1238,'78039  -387','Kosmos-1976','Oblik','','Sat',462,'',null from dual union all
select 61344,'1988-095','1988-10-20 15:43:00','orbital','success',958,'339-01','Raduga','Gran'' No. 34L','','Sat',553,'',null from dual union all
select 61345,'1988-096','1988-10-25 18:02:31','orbital','success',803,'76035  -534','Kosmos-1977','Oko','','Sat',473,'',null from dual union all
select 61346,'1988-097','1988-10-27 11:31:00','orbital','success',1238,'77059  -390','Kosmos-1978','Oblik','','Sat',462,'',null from dual union all
select 61347,'1988-098','1988-10-28 02:17:00','orbital','success',92,'V26 L25','TDF 1','TDF 1','','Sat',344,'',null from dual union all
select 61348,'1988-099','1988-11-06 18:03:00','orbital','success',1406,'34D-14 (04D-7)','USA 33       [IC3]','Improved CRYSTAL 2103','','Sat',1777,'',null from dual union all
select 61349,'1988-100','1988-11-15 03:00:01','orbital','success',461,'L1501L','Buran','Buran OK-1K No. 711 (1K1','','Sat',498,'',null from dual union all
select 6135,'1959-S36','1959-03-25 05:25:00','military missile','success',1016,'No. I3-18','8K71 LKI III-3','GCh No. 15','','Test',480,'',1350 from dual union all
select 61350,'1988-101','1988-11-18 00:12:28','orbital','success',1446,'45029-815','Kosmos-1979','US-PM','','Sat',589,'',null from dual union all
select 61351,'1988-102','1988-11-23 14:50:58','orbital','success',1537,'51095-612','Kosmos-1980','Tselina-2','','Sat',567,'',null from dual union all
select 61352,'1988-103','1988-11-24 14:50:00','orbital','success',1238,'76035  -372','Kosmos-1981','Oblik','','Sat',473,'',null from dual union all
select 61353,'1988-104','1988-11-26 15:49:34','orbital','success',1242,'Ye15000-028','Soyuz TM-7','Soyuz 7K-STM No. 57','','Sat',480,'',null from dual union all
select 61354,'1988-105','1988-11-30 09:00:00','orbital','success',1238,'76035  -374','Kosmos-1982','Oblik','','Sat',480,'',null from dual union all
select 61355,'1988-106','1988-12-02 14:30:34','orbital','success',1243,'STS-27R','USA 34   (LACROSSE 1)','OV-104 LEO deploy','','Sat',922,'MLP1',null from dual union all
select 61356,'1988-107','1988-12-08 14:50:00','orbital','success',1241,'76035  -380','Kosmos-1983','Oblik','','Sat',474,'',null from dual union all
select 61357,'1988-108','1988-12-10 11:54:00','orbital','success',958,'329-02','Ekran','Ekran-M No. 12L','','Sat',554,'',null from dual union all
select 61358,'1988-109','1988-12-11 00:33:38','orbital','success',106,'V27 L402','Skynet 4B/Astra 1A','Skynet 4B','','Sat',345,'',null from dual union all
select 61359,'1988-110','1988-12-16 19:00:00','orbital','success',1238,'78039  -968','Kosmos-1984','Kobal''t','','Sat',462,'',null from dual union all
select 6136,'1959-S360','1959-07-28 00:00:00','military missile','success',975,'I10-7','','','','Test',636,'',80 from dual union all
select 61360,'1988-111','1988-12-22 12:40:00','orbital','success',308,'Y5       (20)','Zhongxing 2','DFH 2A-2','','Sat',1904,'',null from dual union all
select 61361,'1988-112','1988-12-22 14:15:59','orbital','success',805,'77011  -690','Molniya-3','Molniya-3 No. 52','','Sat',474,'',null from dual union all
select 61362,'1988-113','1988-12-23 07:20:00','orbital','success',1448,'67019-206','Kosmos-1985','ESO','','Sat',469,'',null from dual union all
select 61363,'1988-114','1988-12-25 04:11:37','orbital','success',1242,'Ye15000-029','Progress-39','Progress 7K-TG No. 147','','Sat',480,'',null from dual union all
select 61364,'1988-115','1988-12-28 05:27:59','orbital','success',805,'78039  -559','Molniya-1','Molniya-1 No. 63','','Sat',474,'',null from dual union all
select 61365,'1988-116','1988-12-29 10:00:00','orbital','success',1238,'78028  -458','Kosmos-1986','Kometa No. 10','','Sat',480,'',null from dual union all
select 61366,'1988-A01','1988-01-14 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',46 from dual union all
select 61367,'1988-A02','1988-01-20 15:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',40 from dual union all
select 61368,'1988-A03','1988-02-06 19:55:00','military missile','failure',1442,'','DASO','','','OT',421,'SSBN 641',2 from dual union all
select 61369,'1988-A04','1988-03-18 13:30:00','military missile','failure',1005,'67065-606(15L)','15A18M LKI-16','GCh 15F173','','OT',512,'',0 from dual union all
select 6137,'1959-S361','1959-07-30 00:00:00','military missile','success',975,'I1-6TS','','','','Test',636,'',80 from dual union all
select 61370,'1988-A05','1988-04-16 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',48 from dual union all
select 61371,'1988-A06','1988-07-07 22:38:00','military missile','failure',1443,'D5X-13','R&D Mk 5','','','Test',294,'',20 from dual union all
select 61372,'1988-A07','1988-09-19 17:44:00','military missile','failure',1443,'D5X-15','R&D Mk 5','','','Test',294,'',20 from dual union all
select 61373,'1988-A08','1988-10-21 13:35:00','suborbital rocket','failure',738,'','','','','Aeron',1917,'',1 from dual union all
select 61374,'1988-A09','1988-11-04 13:20:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',43 from dual union all
select 61375,'1988-A10','1988-09-28 00:00:00','atmospheric rocket','success',507,'RSP-101','RSP-101/Genie','','','Test',1490,'',10 from dual union all
select 61376,'1988-A11','1988-08-26 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D01','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61377,'1988-A12','1988-08-26 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D06','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61378,'1988-A13','1988-12-29 00:00:00','military missile','success',923,'189','15Zh53 INF 4RD  D36','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61379,'1988-A14','1988-12-29 00:00:00','military missile','success',923,'190','15Zh53 INF 23RD D36','INF Destruct 72','','INF Treaty Destruct',822,'',10 from dual union all
select 6138,'1959-S362','1959-07-31 00:00:00','military missile','success',975,'I7-13T','','','','Test',636,'',80 from dual union all
select 61380,'1988-A15','1988-09-01 00:00:00','military missile','success',1430,'','','','','Test',636,'',26 from dual union all
select 61381,'1988-A16','1988-09-21 00:00:00','test rocket','failure',541,'RFT-1','HIMES RFT-1','','','Test',839,'',18 from dual union all
select 61382,'1988-A17','1988-08-01 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D19','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61383,'1988-A18','1988-08-01 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D20','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61384,'1988-A19','1988-09-01 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D21','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61385,'1988-A20','1988-09-01 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D22','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61386,'1988-A21','1988-10-01 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D23','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61387,'1988-A23','1988-08-24 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D01','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61388,'1988-A24','1988-08-24 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D02','INF Destruct','','INF Treaty Destruct',822,'',10 from dual;
insert into LAUNCH
select 61389,'1988-A25','1988-08-24 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D03','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 6139,'1959-S363','1959-08-07 00:00:00','military missile','success',975,'I7-11','','','','Test',636,'',80 from dual union all
select 61390,'1988-A26','1988-08-26 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D04','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61391,'1988-A27','1988-08-26 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D05','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61392,'1988-A28','1988-08-27 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D07','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61393,'1988-A29','1988-08-27 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D08','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61394,'1988-A30','1988-08-27 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D09','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61395,'1988-A31','1988-08-29 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D10','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61396,'1988-A32','1988-08-29 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D11','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61397,'1988-A33','1988-08-29 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D12','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61398,'1988-A34','1988-08-29 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D13','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61399,'1988-A35','1988-08-29 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D14','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 614,'1945-S90','1945-03-27 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 6140,'1959-S364','1959-08-17 00:00:00','military missile','success',969,'53081418','','','','OT',636,'',100 from dual union all
select 61400,'1988-A36','1988-08-29 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D15','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61401,'1988-A37','1988-08-31 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D16','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61402,'1988-A38','1988-08-31 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D17','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61403,'1988-A39','1988-08-31 00:00:00','military missile','success',922,'','15Zh45 INF 23RD D18','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61404,'1988-A40','1988-10-25 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D24','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61405,'1988-A41','1988-10-27 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D25','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61406,'1988-A42','1988-10-29 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D26','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61407,'1988-A43','1988-11-24 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D27','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61408,'1988-A44','1988-11-24 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D28','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61409,'1988-A45','1988-11-24 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D29','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 6141,'1959-S365','1959-08-17 00:00:00','military missile','success',975,'B4-32','','','','Test',636,'',80 from dual union all
select 61410,'1988-A46','1988-11-26 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D30','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61411,'1988-A47','1988-11-26 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D31','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61412,'1988-A48','1988-11-26 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D32','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61413,'1988-A49','1988-11-29 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D33','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61414,'1988-A50','1988-11-29 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D34','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61415,'1988-A51','1988-11-29 00:00:00','military missile','success',923,'','15Zh53 INF 23RD D35','INF Destruct','','INF Treaty Destruct',822,'',10 from dual union all
select 61416,'1988-A52','1988-08-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D02','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61417,'1988-A53','1988-08-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D03','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61418,'1988-A54','1988-08-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D04','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61419,'1988-A55','1988-08-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D05','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 6142,'1959-S366','1959-08-19 00:00:00','military missile','success',975,'B4-31','','','','Test',636,'',80 from dual union all
select 61420,'1988-A56','1988-09-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D06','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61421,'1988-A57','1988-09-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D07','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61422,'1988-A58','1988-09-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D08','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61423,'1988-A59','1988-09-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D09','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61424,'1988-A60','1988-09-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D10','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61425,'1988-A61','1988-09-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D11','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61426,'1988-A62','1988-09-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D12','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61427,'1988-A63','1988-10-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D13','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61428,'1988-A64','1988-10-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D14','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61429,'1988-A65','1988-10-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D15','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 6143,'1959-S367','1959-08-22 00:00:00','military missile','success',975,'B4-26','','','','Test',636,'',80 from dual union all
select 61430,'1988-A66','1988-10-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D16','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61431,'1988-A67','1988-10-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D17','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61432,'1988-A68','1988-10-01 00:00:00','military missile','success',922,'','15Zh45 INF 4RD  D18','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61433,'1988-A69','1988-10-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D19','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61434,'1988-A70','1988-10-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D20','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61435,'1988-A71','1988-11-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D21','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61436,'1988-A72','1988-11-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D22','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61437,'1988-A73','1988-11-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D23','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61438,'1988-A74','1988-11-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D24','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61439,'1988-A75','1988-11-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D25','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 6144,'1959-S368','1959-09-03 00:00:00','military missile','success',975,'I11-7','','','','Test',636,'',80 from dual union all
select 61440,'1988-A76','1988-11-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D26','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61441,'1988-A77','1988-11-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D27','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61442,'1988-A78','1988-11-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D28','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61443,'1988-A79','1988-12-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D29','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61444,'1988-A80','1988-12-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D30','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61445,'1988-A81','1988-12-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D31','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61446,'1988-A82','1988-12-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D32','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61447,'1988-A83','1988-12-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D33','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61448,'1988-A84','1988-12-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D34','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 61449,'1988-A85','1988-12-01 00:00:00','military missile','success',923,'','15Zh53 INF 4RD  D35','INF Destruct','','INF Treaty Destruct',368,'',10 from dual union all
select 6145,'1959-S369','1959-09-03 00:00:00','military missile','success',975,'I11-3T','','','','Test',636,'',80 from dual union all
select 61450,'1988-F01','1988-01-18 09:58:00','orbital','failure',958,'341-02','Gorizont','Gorizont No. 25L','','Sat',554,'',200 from dual union all
select 61451,'1988-F02','1988-07-09 13:25:00','orbital','failure',1238,'78028  -212','','Terilen','','Sat',480,'',50 from dual union all
select 61452,'1988-F03','1988-07-13 09:18:00','orbital','failure',22,'ASLV-D2','SROSS B','SROSS-B','','Sat',1360,'',25 from dual union all
select 61453,'1988-F04','1988-07-27 09:05:00','orbital','failure',1238,'78039  -130','','Resurs-F1 14F43 No. 30','','Sat',475,'',0 from dual union all
select 61454,'1988-F05','1988-11-11 10:30:00','orbital','failure',1238,'76035  -713','','Terilen','','Sat',480,'',170 from dual union all
select 61455,'1988-M01','1988-01-21 10:08:00','military missile','failure',1443,'D5X-9','R&D Mk 5','','','Test',294,'',80 from dual union all
select 61456,'1988-M02','1988-09-05 23:00:00','suborbital rocket','success',1295,'TR-1-1','H-II 1/4 Scale','','','Test',1476,'',85 from dual union all
select 61457,'1988-M03','1988-12-19 00:00:00','suborbital rocket','success',1545,'ZN-1','','','','Test',647,'',70 from dual union all
select 61458,'1988-M04','1988-12-20 00:00:00','suborbital rocket','success',1545,'ZN-1','','','','Test',647,'',70 from dual union all
select 61459,'1988-M05','1988-12-22 00:00:00','suborbital rocket','success',1545,'ZN-1','','','','Test',647,'',70 from dual union all
select 6146,'1959-S37','1959-03-27 04:02:00','military missile','success',1367,'162','R&D Series IV','','','Test',234,'',520 from dual union all
select 61460,'1988-M06','1988-12-28 00:00:00','suborbital rocket','success',1545,'ZN-1','','','','Test',647,'',70 from dual union all
select 61461,'1988-M07','1988-05-31 00:00:00','suborbital rocket','success',1023,'Daily to Jul 12','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61462,'1988-M08','1988-06-01 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61463,'1988-M09','1988-06-02 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61464,'1988-M10','1988-06-03 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61465,'1988-M100','1988-12-14 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61466,'1988-M101','1988-12-21 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61467,'1988-M102','1988-12-29 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61468,'1988-M11','1988-06-04 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61469,'1988-M12','1988-06-05 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 6147,'1959-S370','1959-09-07 00:00:00','military missile','success',975,'B4-1T','','','','Test',636,'',80 from dual union all
select 61470,'1988-M13','1988-06-06 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61471,'1988-M14','1988-06-07 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61472,'1988-M15','1988-06-08 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61473,'1988-M16','1988-06-09 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61474,'1988-M17','1988-06-10 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61475,'1988-M18','1988-06-11 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61476,'1988-M19','1988-06-12 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61477,'1988-M20','1988-06-13 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61478,'1988-M21','1988-06-14 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual;
insert into LAUNCH
select 61479,'1988-M22','1988-06-15 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 6148,'1959-S371','1959-09-08 00:00:00','military missile','success',975,'B4-9T','','','','Test',636,'',80 from dual union all
select 61480,'1988-M23','1988-06-16 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61481,'1988-M24','1988-06-17 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61482,'1988-M25','1988-06-18 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61483,'1988-M26','1988-06-19 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61484,'1988-M27','1988-06-20 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61485,'1988-M28','1988-06-21 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61486,'1988-M29','1988-06-22 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61487,'1988-M30','1988-06-23 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61488,'1988-M31','1988-06-24 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61489,'1988-M32','1988-06-25 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 6149,'1959-S372','1959-09-05 00:00:00','military missile','success',1013,'M3-11','8K51 UBP-23','','','Test',636,'',500 from dual union all
select 61490,'1988-M33','1988-06-26 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61491,'1988-M34','1988-06-27 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61492,'1988-M35','1988-06-28 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61493,'1988-M36','1988-06-29 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61494,'1988-M37','1988-06-30 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61495,'1988-M38','1988-07-01 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61496,'1988-M39','1988-07-02 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61497,'1988-M40','1988-07-03 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61498,'1988-M41','1988-07-04 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61499,'1988-M42','1988-07-05 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 615,'1945-S91','1945-03-27 00:00:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 6150,'1959-S373','1959-09-07 00:00:00','military missile','success',1013,'M1-4','8K51 UBP-24','Got.-2','','Test',636,'',500 from dual union all
select 61500,'1988-M43','1988-07-06 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61501,'1988-M44','1988-07-07 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61502,'1988-M45','1988-07-08 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61503,'1988-M46','1988-07-09 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61504,'1988-M47','1988-07-10 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61505,'1988-M48','1988-07-11 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61506,'1988-M49','1988-07-12 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 61507,'1988-M50','1988-10-31 20:17:00','suborbital rocket','success',1284,'','ROCOZ-A 352 PreSTOIC','','','Aeron',1057,'',75 from dual union all
select 61508,'1988-M51','1988-11-01 20:25:00','suborbital rocket','success',1284,'','ROCOZ-A 353 PreSTOIC','','','Aeron',1057,'',75 from dual union all
select 61509,'1988-M52','1988-11-02 22:11:00','suborbital rocket','success',1284,'','ROCOZ-A 354 PreSTOIC','','','Aeron',1057,'',75 from dual union all
select 6151,'1959-S374','1959-09-07 00:00:00','military missile','success',1013,'M3-5','8K51 UBP-25','Got.-3','','Test',636,'',500 from dual union all
select 61510,'1988-M53','1988-11-05 22:37:00','suborbital rocket','success',1284,'','ROCOZ-A 355 PreSTOIC','','','Aeron',1057,'',75 from dual union all
select 61511,'1988-M54','1988-01-06 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61512,'1988-M55','1988-01-13 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61513,'1988-M56','1988-01-20 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61514,'1988-M57','1988-01-28 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61515,'1988-M58','1988-02-03 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61516,'1988-M59','1988-02-10 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61517,'1988-M60','1988-02-17 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61518,'1988-M61','1988-02-24 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61519,'1988-M62','1988-03-02 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6152,'1959-S375','1959-09-08 00:00:00','military missile','success',975,'B1-2','','','','Test',636,'',80 from dual union all
select 61520,'1988-M63','1988-03-09 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61521,'1988-M64','1988-03-16 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61522,'1988-M65','1988-03-23 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61523,'1988-M66','1988-03-30 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61524,'1988-M67','1988-04-06 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61525,'1988-M68','1988-04-13 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61526,'1988-M69','1988-04-20 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61527,'1988-M70','1988-04-27 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61528,'1988-M71','1988-05-11 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61529,'1988-M72','1988-05-25 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6153,'1959-S376','1959-09-18 00:00:00','military missile','success',975,'B4-30','','','','Test',636,'',80 from dual union all
select 61530,'1988-M73','1988-06-01 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61531,'1988-M74','1988-06-08 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61532,'1988-M75','1988-06-15 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61533,'1988-M76','1988-06-22 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61534,'1988-M77','1988-06-29 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61535,'1988-M78','1988-07-07 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61536,'1988-M79','1988-07-13 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61537,'1988-M80','1988-07-20 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61538,'1988-M81','1988-07-27 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61539,'1988-M82','1988-08-03 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6154,'1959-S377','1959-09-22 00:00:00','military missile','success',975,'I5-1','','','','Test',636,'',80 from dual union all
select 61540,'1988-M83','1988-08-10 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61541,'1988-M84','1988-08-17 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61542,'1988-M85','1988-08-26 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61543,'1988-M86','1988-08-31 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61544,'1988-M87','1988-09-07 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61545,'1988-M88','1988-09-14 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61546,'1988-M89','1988-09-21 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61547,'1988-M90','1988-09-28 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61548,'1988-M91','1988-10-05 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61549,'1988-M92','1988-10-12 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6155,'1959-S378','1959-09-22 00:00:00','military missile','success',975,'I5-6T','','','','Test',636,'',80 from dual union all
select 61550,'1988-M93','1988-10-26 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61551,'1988-M94','1988-11-02 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61552,'1988-M95','1988-11-10 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61553,'1988-M96','1988-11-16 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61554,'1988-M97','1988-11-24 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61555,'1988-M98','1988-11-30 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61556,'1988-M99','1988-12-07 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 61557,'1988-S01','1988-06-01 00:00:00','military missile','success',350,'','','','','Test',1494,'',100 from dual union all
select 61558,'1988-S02','1988-04-20 15:10:00','military missile','success',914,'51029-804','15A11 UBP-4','GCh 15B99','','Test',546,'',1000 from dual union all
select 61559,'1988-S03','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 6156,'1959-S379','1959-09-25 00:00:00','military missile','success',975,'B4-28','','','','Test',636,'',80 from dual union all
select 61560,'1988-S04','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61561,'1988-S05','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61562,'1988-S06','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61563,'1988-S07','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61564,'1988-S08','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61565,'1988-S09','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61566,'1988-S10','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61567,'1988-S100','1988-06-23 17:40:00','suborbital rocket','success',224,'NASA 36.35US','NIXT','','','Solar',1873,'',300 from dual union all
select 61568,'1988-S101','1988-07-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual;
insert into LAUNCH
select 61569,'1988-S102','1988-07-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 6157,'1959-S38','1959-03-30 00:00:00','military missile','success',976,'I1-20T','8K63 UBP-2','GCh Gruzomaket','','Test',636,'',402 from dual union all
select 61570,'1988-S103','1988-07-12 11:40:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT126GM','GLORY TRIP 126GM','OT',1705,'',1300 from dual union all
select 61571,'1988-S104','1988-07-12 13:21:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT128GM','GLORY TRIP 128GM','OT',1699,'',1300 from dual union all
select 61572,'1988-S105','1988-07-27 05:47:43','suborbital rocket','success',211,'NASA 27.120UE','TR 2','','','Plasma',1821,'',340 from dual union all
select 61573,'1988-S106','1988-07-27 05:48:03','suborbital rocket','success',1318,'NASA 38.07UE','','','','Plasma',1821,'',500 from dual union all
select 61574,'1988-S107','1988-08-01 00:00:00','suborbital rocket','success',117,'Queen Match','(Recovery failed)','','','ABM Tech IR',1362,'',300 from dual union all
select 61575,'1988-S108','1988-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 61576,'1988-S109','1988-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 61577,'1988-S11','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61578,'1988-S110','1988-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 61579,'1988-S111','1988-08-27 20:04:00','military missile','success',1443,'D5X-14','R&D Mk 5','','','Test',294,'',1000 from dual union all
select 6158,'1959-S380','1959-09-25 00:00:00','military missile','success',975,'B4-27','','','','Test',636,'',80 from dual union all
select 61580,'1988-S112','1988-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 61581,'1988-S113','1988-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 61582,'1988-S114','1988-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 61583,'1988-S115','1988-09-03 14:10:00','suborbital rocket','success',1202,'DLR N-GRC-158','Interzodiak 2','','','EUV Astronomy',330,'',857 from dual union all
select 61584,'1988-S116','1988-09-10 17:10:12','suborbital rocket','success',224,'NASA 36.46UE','ERIC III','','','Plasma',1821,'',300 from dual union all
select 61585,'1988-S117','1988-09-11 07:30:00','suborbital rocket','success',224,'NASA 36.31UG','IMAPS 3','','','UV Astron',1873,'',291 from dual union all
select 61586,'1988-S118','1988-09-15 10:40:00','suborbital rocket','success',211,'NASA 27.110UL','','','','UV Astron',1895,'',340 from dual union all
select 61587,'1988-S119','1988-09-22 00:00:00','military missile','success',1041,'48775-412-14L','15Zh60 LKI-16','','','Test',477,'',1000 from dual union all
select 61588,'1988-S12','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61589,'1988-S120','1988-09-27 00:00:00','military missile','success',614,'','','','','Test',1909,'XIA',500 from dual union all
select 6159,'1959-S381','1959-10-01 00:00:00','military missile','success',975,'I8-20T','','','','Test',636,'',80 from dual union all
select 61590,'1988-S121','1988-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 61591,'1988-S122','1988-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 61592,'1988-S123','1988-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 61593,'1988-S124','1988-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 61594,'1988-S125','1988-09-30 18:10:00','suborbital rocket','success',237,'NASA 12.41WT','BEARS','','','Test/Aeron',1821,'',950 from dual union all
select 61595,'1988-S126','1988-10-04 16:00:00','suborbital rocket','success',1028,'','ESF','','','Ionos',1361,'',348 from dual union all
select 61596,'1988-S127','1988-10-19 00:00:00','suborbital rocket','success',547,'','HAVE JEEP II-C','','','Reentry Test?',871,'',500 from dual union all
select 61597,'1988-S128','1988-10-20 20:29:51','suborbital rocket','success',1319,'NASA 33.49UL','','','','Aeron',1047,'',200 from dual union all
select 61598,'1988-S129','1988-10-24 18:03:01','suborbital rocket','success',224,'NASA 36.41UL','SOLLY','','','UV Astronomy',1873,'',300 from dual union all
select 61599,'1988-S13','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 616,'1945-S92','1945-03-27 05:15:00','military missile','success',1470,'','London','','','Weapon',1627,'',90 from dual union all
select 6160,'1959-S382','1959-10-03 00:00:00','military missile','success',975,'I8-10','','','','Test',636,'',80 from dual union all
select 61600,'1988-S130','1988-10-27 15:28:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT130GM','GLORY TRIP 130GM','OT',1706,'',1300 from dual union all
select 61601,'1988-S131','1988-11-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 61602,'1988-S132','1988-11-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 61603,'1988-S133','1988-11-07 16:30:00','military missile','success',1443,'D5X-18','R&D','','','Test',294,'',1000 from dual union all
select 61604,'1988-S134','1988-11-07 19:02:00','suborbital rocket','success',228,'NASA 21.98IE','NEED I (F-79)','','','Ionos',29,'',322 from dual union all
select 61605,'1988-S135','1988-11-10 19:00:00','suborbital rocket','success',228,'NASA 21.101UU','NCAR/CU-1','','','Aeron/SolarUV',1873,'',250 from dual union all
select 61606,'1988-S136','1988-11-20 18:30:00','suborbital rocket','success',224,'NASA 36.44DS','HRTS 6/ASL-San Marco','','','Test/Solar',1873,'',300 from dual union all
select 61607,'1988-S137','1988-11-22 00:00:00','military missile','success',567,'','','','','Test',307,'',150 from dual union all
select 61608,'1988-S138','1988-11-26 17:00:00','suborbital rocket','success',1204,'DLR A-CR-159','DLR ROSE I','','','Aeron/Ionos/Fields',29,'',120 from dual union all
select 61609,'1988-S139','1988-11-28 08:20:10','suborbital rocket','success',1212,'DLR K-GR-161','DLR TEXUS 19','','','Micrograv',416,'',244 from dual union all
select 6161,'1959-S383','1959-09-24 00:00:00','military missile','success',1013,'M1-5','8K51 UBP-27','','','Test',636,'',500 from dual union all
select 61610,'1988-S14','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61611,'1988-S140','1988-12-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 61612,'1988-S141','1988-12-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',1207,'VISE',250 from dual union all
select 61613,'1988-S142','1988-12-02 10:05:00','suborbital rocket','success',1212,'DLR K-GT-162','DLR TEXUS 20','','','Micrograv',416,'',238 from dual union all
select 61614,'1988-S143','1988-12-05 22:30:00','suborbital rocket','success',1204,'DLR A-CR-160','DLR ROSE II','','','Aeron',29,'',113 from dual union all
select 61615,'1988-S144','1988-12-06 10:02:00','suborbital rocket','success',891,'DLR K-O-163','RASMUS 2','','','Aeron',417,'',65 from dual union all
select 61616,'1988-S145','1988-12-07 04:40:00','suborbital rocket','success',211,'NASA 27.121UH','','','','XR Astron',1895,'',340 from dual union all
select 61617,'1988-S146','1988-12-16 12:49:00','military missile','success',1443,'D5X-17','R&D','','','Test',294,'',1000 from dual union all
select 61618,'1988-S147','1988-12-17 11:01:00','suborbital rocket','success',231,'NASA 35.024UE','CUSP','','','Plasma',29,'',515 from dual union all
select 61619,'1988-S148','1988-01-01 00:00:00','military missile','success',627,'','','','','Test',1026,'',300 from dual union all
select 6162,'1959-S384','1959-10-09 00:00:00','military missile','success',975,'B1-5','','','','Test',636,'',80 from dual union all
select 61620,'1988-S149','1988-02-29 00:00:00','military missile','success',73,'','vs Tehran','','','Weapon',20,'',100 from dual union all
select 61621,'1988-S15','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61622,'1988-S150','1988-02-29 00:00:00','military missile','success',73,'','vs Tehran','','','Weapon',20,'',100 from dual union all
select 61623,'1988-S151','1988-02-29 00:00:00','military missile','success',73,'','vs Tehran','','','Weapon',20,'',100 from dual union all
select 61624,'1988-S152','1988-03-01 00:00:00','military missile','success',73,'','vs Tehran','','','Weapon',20,'',100 from dual union all
select 61625,'1988-S153','1988-03-01 00:00:00','military missile','success',73,'','vs Tehran','','','Weapon',20,'',100 from dual union all
select 61626,'1988-S154','1988-03-01 00:00:00','military missile','success',73,'[total up to 9/2d]','vs Tehran','','','Weapon',20,'',100 from dual union all
select 61627,'1988-S155','1988-04-25 00:00:00','military missile','success',69,'','Test flight','','','Test',757,'',100 from dual union all
select 61628,'1988-S156','1988-06-27 00:00:00','military missile','success',69,'','Test flight','','','Test',757,'',100 from dual union all
select 61629,'1988-S157','1988-06-28 00:00:00','military missile','success',69,'','Test flight','','','Test',757,'',100 from dual union all
select 6163,'1959-S385','1959-10-15 00:00:00','military missile','success',975,'I12-7T','','','','Test',636,'',80 from dual union all
select 61630,'1988-S158','1988-06-29 00:00:00','military missile','success',69,'','Test flight','','','Test',757,'',100 from dual union all
select 61631,'1988-S159','1988-03-01 00:00:00','military missile','success',983,'Scud B','181 launches vs Iran','','','Weapon',761,'',100 from dual union all
select 61632,'1988-S16','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61633,'1988-S160','1987-12-01 00:00:00','military missile','unknown',918,'AO-237','P-514 87 Fall 05','','','OT',433,'',180 from dual union all
select 61634,'1988-S161','1987-12-01 00:00:00','military missile','unknown',918,'AO-238','P-515 87 Fall 06','','','OT',433,'',180 from dual union all
select 61635,'1988-S162','1987-12-01 00:00:00','military missile','unknown',918,'AO-239','P-516 87 Fall 07','','','OT',433,'',180 from dual union all
select 61636,'1988-S163','1988-01-01 00:00:00','military missile','unknown',917,'AO-240','P-517','','','OT',433,'',180 from dual union all
select 61637,'1988-S164','1988-01-01 00:00:00','military missile','unknown',917,'AO-241','P-518','','','OT',433,'',180 from dual union all
select 61638,'1988-S166','1988-10-07 00:00:00','military missile','unknown',917,'AO-243  Cert. Fir.','P-535 88 Fall 01','','','OT',433,'',180 from dual union all
select 61639,'1988-S167','1988-10-07 00:00:00','military missile','unknown',917,'AO-244  Cert. Fir.','P-536 88 Fall 02','','','OT',433,'',180 from dual union all
select 6164,'1959-S386','1959-10-20 00:00:00','military missile','success',975,'I12-3','','','','Test',636,'',80 from dual union all
select 61640,'1988-S168','1988-10-21 00:00:00','military missile','unknown',917,'AO-245  Cert. Fir.','P-537 88 Fall 03','','','OT',433,'',180 from dual union all
select 61641,'1988-S169','1988-10-21 00:00:00','military missile','unknown',917,'AO-246  Cert. Fir.','P-538 88 Fall 04','','','OT',433,'',180 from dual union all
select 61642,'1988-S17','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61643,'1988-S170','1988-05-13 14:30:00','military missile','success',918,'AO','P-531','','','OT',433,'',180 from dual union all
select 61644,'1988-S171','1988-09-14 00:00:00','military missile','success',1434,'','15Zh58 K-19','','','Test',477,'',1000 from dual union all
select 61645,'1988-S172','1988-12-09 00:00:00','military missile','success',1434,'','15Zh58 K-20','','','Test',477,'',1000 from dual union all
select 61646,'1988-S173','1988-04-29 00:00:00','military missile','success',1434,'','15Zh58 UBP-2','','','Test',477,'',1000 from dual union all
select 61647,'1988-S174','1988-08-05 00:00:00','military missile','success',1434,'','15Zh58 UBP-3','','','Test',477,'',1000 from dual union all
select 61648,'1988-S175','1988-10-20 00:00:00','military missile','success',1434,'','15Zh58 UBP-4','','','Test',477,'',1000 from dual union all
select 61649,'1988-S176','1988-02-03 00:00:00','military missile','success',1041,'48775-409-9L','15Zh60 LKI-11','','','Test',477,'',1000 from dual union all
select 6165,'1959-S387','1959-11-09 00:00:00','military missile','success',969,'D21-10','','','','OT',636,'',100 from dual union all
select 61650,'1988-S177','1988-02-17 00:00:00','military missile','success',1041,'48775-411-15L','15Zh60 LKI-12','','','Test',477,'',1000 from dual union all
select 61651,'1988-S178','1988-04-27 00:00:00','military missile','success',1041,'48775-410-10L','15Zh60 LKI-13','','','Test',477,'',1000 from dual union all
select 61652,'1988-S179','1988-05-25 00:00:00','military missile','success',1041,'48719-314-06','15Zh60 LKI-14','','','Test',477,'',1000 from dual union all
select 61653,'1988-S18','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61654,'1988-S180','1988-07-25 00:00:00','military missile','success',1041,'48719-602-16L','15Zh60 LKI-15','','','Test',477,'',1000 from dual union all
select 61655,'1988-S181','1988-07-18 00:00:00','military missile','success',1006,'','15A18 UBP 38 RD','','','OT',363,'',1000 from dual union all
select 61656,'1988-S182','1988-12-19 00:00:00','military missile','success',1006,'','15A18 UBP 38 RD','','','OT',363,'',1000 from dual union all
select 61657,'1988-S183','1988-05-13 14:55:00','military missile','success',918,'AO','P-532','','','OT',433,'',180 from dual union all
select 61658,'1988-S184','1988-05-13 16:15:00','military missile','success',918,'AO','P-533','','','OT',433,'',180 from dual;
insert into LAUNCH
select 61659,'1988-S185','1988-01-01 00:00:00','military missile','unknown',917,'AO-242','P-534','','','OT',433,'',180 from dual union all
select 6166,'1959-S388','1959-11-20 00:00:00','military missile','success',992,'S3-9','','','','OT',636,'',170 from dual union all
select 61660,'1988-S19','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61661,'1988-S20','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61662,'1988-S21','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61663,'1988-S22','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61664,'1988-S23','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61665,'1988-S24','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61666,'1988-S25','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61667,'1988-S26','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61668,'1988-S27','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61669,'1988-S28','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 6167,'1959-S389','1959-11-20 00:00:00','military missile','success',992,'S3-14','','','','OT',636,'',170 from dual union all
select 61670,'1988-S29','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61671,'1988-S30','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61672,'1988-S31','1988-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 61673,'1988-S32','1988-01-01 00:00:00','military missile','success',1548,'','Final OT','','','OT',1011,'PLBR',620 from dual union all
select 61674,'1988-S33','1988-05-05 15:10:00','military missile','success',1006,'51029-748-04','15A18 UBP-25','GCh 15F183','','OT',489,'',1000 from dual union all
select 61675,'1988-S34','1988-09-09 11:50:00','military missile','success',1006,'51074-317-01','15A18 UBP-26','GCh 15F183','','OT',489,'',100 from dual union all
select 61676,'1988-S35','1988-12-22 10:05:00','military missile','success',1006,'45042-613-06','15A18 UBP-27','GCh 15F183','','OT',489,'',1000 from dual union all
select 61677,'1988-S36','1988-02-09 01:45:00','military missile','destroyed before launch',1005,'45065-607(16L)','15A18M LKI-15','GCh 15F173','','OT',497,'',0 from dual union all
select 61678,'1988-S37','1988-04-20 15:00:00','military missile','success',1005,'51065-608(17L)','15A18M LKI-17','GCh 15F173','','OT',491,'',1000 from dual union all
select 61679,'1988-S38','1988-11-18 04:55:00','military missile','success',1005,'45065-216-01(18L)','15A18M LKI-18','GCh 15F175-2 (15F176 x1)','','OT',512,'',1000 from dual union all
select 6168,'1959-S39','1959-03-30 15:01:00','suborbital rocket','success',30,'AA3.140C','Solar EUV Spectrum','','','Solar EUV',640,'',201 from dual union all
select 61680,'1988-S39','1988-11-22 06:30:00','military missile','success',1005,'45079-512-05','15A18M K-1','GCh 15F173','','OT',487,'',1000 from dual union all
select 61681,'1988-S40','1988-12-14 04:40:00','military missile','success',1005,'67019-203','15A18M LKI-19','GCh 15F175','','OT',512,'',1000 from dual union all
select 61682,'1988-S41','1988-01-01 00:00:00','suborbital rocket','success',1304,'','HAVE JEEP VII-C','','','Target',871,'',500 from dual union all
select 61683,'1988-S42','1988-08-23 00:00:00','military missile','success',1453,'49242-83-422','15A35 UBP-38','','','OT',596,'',1000 from dual union all
select 61684,'1988-S43','1988-01-13 14:01:05','military missile','success',919,'AO','P-519 88 Cape 01','','','OT',231,'',320 from dual union all
select 61685,'1988-S44','1988-01-19 02:40:00','military missile','success',795,'','TDMaRV','RSLP TDMaRV','','Reentry Test',1698,'',1300 from dual union all
select 61686,'1988-S45','1988-01-19 08:40:04','suborbital rocket','success',231,'NASA 35.017UE','TOPAZ II','','','Plasma',1047,'',926 from dual union all
select 61687,'1988-S46','1988-01-23 18:30:00','suborbital rocket','success',224,'NASA 36.29GU','','','','Aeron?',1873,'',300 from dual union all
select 61688,'1988-S47','1988-01-25 02:00:00','suborbital rocket','success',668,'K-9M-81','COREX','','','Ionos',839,'',359 from dual union all
select 61689,'1988-S48','1988-01-28 02:00:00','suborbital rocket','success',1073,'S-310-18','D-region','','','Ionos/Aeron',833,'',202 from dual union all
select 6169,'1959-S390','1959-11-26 00:00:00','military missile','success',992,'S3-16','','','','OT',636,'',170 from dual union all
select 61690,'1988-S49','1988-01-29 15:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT125GB-1','GLORY TRIP 125GB-1','OT',1712,'',1300 from dual union all
select 61691,'1988-S50','1988-01-29 19:03:41','suborbital rocket','success',224,'NASA 36.39UE','ERIC II','','','Plasma/Art. Comet',1821,'',300 from dual union all
select 61692,'1988-S51','1988-02-09 00:00:00','suborbital rocket','success',1279,'','','','','Reentry Test?',843,'',200 from dual union all
select 61693,'1988-S52','1988-02-09 08:16:49','suborbital rocket','success',224,'NASA 36.15UE','ECHO VII','','','Plasma/Tether',1047,'',292 from dual union all
select 61694,'1988-S53','1988-02-15 13:03:03','military missile','success',919,'','P-520 88 Cape 02','','','OT',231,'',320 from dual union all
select 61695,'1988-S54','1988-02-15 13:21:05','military missile','success',919,'','P-521 88 Cape 03','','','OT',231,'',320 from dual union all
select 61696,'1988-S55','1988-02-15 13:42:06','military missile','success',919,'','P-522 88 Cape 04','','','OT',231,'',320 from dual union all
select 61697,'1988-S56','1988-02-15 15:16:04','military missile','success',919,'','P-523 88 Cape 05','','','OT',231,'',320 from dual union all
select 61698,'1988-S57','1988-02-15 15:42:06','military missile','success',919,'','P-524 88 Cape 06','','','OT',231,'',320 from dual union all
select 61699,'1988-S58','1988-02-15 16:01:05','military missile','success',919,'-           P2-61','P-525 88 Cape 07','','','OT',231,'',320 from dual union all
select 617,'1945-S93','1945-03-27 14:49:00','military missile','success',1470,'','London/Orpington','','','Weapon',1627,'',90 from dual union all
select 6170,'1959-S391','1959-11-26 00:00:00','military missile','success',992,'S3-12','','','','OT',636,'',170 from dual union all
select 61700,'1988-S59','1988-02-16 13:34:00','suborbital rocket','success',224,'NASA 36.022GH','SXS','','','XR Astronomy',1838,'',300 from dual union all
select 61701,'1988-S60','1988-02-25 00:00:00','military missile','success',941,'1','','','','Test',1361,'',100 from dual union all
select 61702,'1988-S61','1988-02-27 15:27:00','military missile','unknown',1442,'','DASO','','','OT',421,'SSBN 634',1000 from dual union all
select 61703,'1988-S62','1988-02-28 12:10:00','suborbital rocket','success',224,'NASA 36.032UH','SNR 87a','','','XR Astronomy',1838,'',300 from dual union all
select 61704,'1988-S63','1988-02-29 08:34:30','suborbital rocket','success',224,'MASER 2','SSC MASER 2','','','Microgravity',412,'',318 from dual union all
select 61705,'1988-S64','1988-03-04 06:55:00','suborbital rocket','success',231,'NASA 35.023GE','UCB Alaska ''88','','','Plasma',1047,'',1000 from dual union all
select 61706,'1988-S65','1988-03-07 00:00:00','suborbital rocket','failure',1304,'','HAVE JEEP VII-B','','','Target',871,'',500 from dual union all
select 61707,'1988-S66','1988-03-13 13:34:00','suborbital rocket','success',224,'NASA 36.027UG','FUVS   (SNR 87a)','','','UV Astron',1838,'',300 from dual union all
select 61708,'1988-S67','1988-03-17 20:25:00','suborbital rocket','success',224,'NASA 36.36US','EUV-ML 1','','','Solar EUV',1873,'',300 from dual union all
select 61709,'1988-S68','1988-03-21 13:01:05','military missile','success',919,'','P-526 88 Cape 08','','','OT',231,'',320 from dual union all
select 6171,'1959-S392','1959-11-27 00:00:00','military missile','success',992,'S3-13','','','','OT',636,'',170 from dual union all
select 61710,'1988-S69','1988-03-21 13:19:05','military missile','success',919,'','P-527 88 Cape 09','','','OT',231,'',320 from dual union all
select 61711,'1988-S70','1988-03-21 14:13:03','military missile','success',919,'','P-528 88 Cape 10','','','OT',231,'',320 from dual union all
select 61712,'1988-S71','1988-03-21 15:45:05','military missile','success',919,'','P-529 88 Cape 11','','','OT',231,'',320 from dual union all
select 61713,'1988-S72','1988-03-21 16:53:04','military missile','success',919,'','P-530 88 Cape 12','','','OT',231,'',320 from dual union all
select 61714,'1988-S73','1988-04-01 00:00:00','suborbital rocket','success',741,'MR-20','','','','Aeron',636,'',250 from dual union all
select 61715,'1988-S74','1988-04-01 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 628',500 from dual union all
select 61716,'1988-S75','1988-04-01 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 628',500 from dual union all
select 61717,'1988-S76','1988-04-01 00:00:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 628',500 from dual union all
select 61718,'1988-S77','1988-04-01 20:19:00','military missile','success',939,'','FOT','','','OT',421,'SSBN 628',500 from dual union all
select 61719,'1988-S78','1988-04-07 04:59:00','military missile','success',1443,'D5X-11','R&D Mk 5','','','Test',294,'',1000 from dual union all
select 6172,'1959-S393','1959-12-23 00:00:00','military missile','success',992,'S4-6','','','','OT',636,'',170 from dual union all
select 61720,'1988-S79','1988-04-09 00:40:52','suborbital rocket','success',227,'NASA 21.97GE','ERRRIS I','','','Ionos',411,'',128 from dual union all
select 61721,'1988-S80','1988-04-15 00:00:00','military missile','unknown',1442,'','FOT','','','OT',421,'SSBN 658',1000 from dual union all
select 61722,'1988-S81','1988-04-15 00:00:00','military missile','unknown',1442,'','FOT','','','OT',421,'SSBN 658',1000 from dual union all
select 61723,'1988-S82','1988-04-15 05:26:00','military missile','unknown',1442,'','FOT','','','OT',421,'SSBN 658',1000 from dual union all
select 61724,'1988-S83','1988-04-15 04:11:00','military missile','unknown',1442,'','FOT','','','OT',421,'SSBN 658',1000 from dual union all
select 61725,'1988-S84','1988-04-28 04:52:00','military missile','success',1443,'D5X-12','R&D Mk 5','','','Test',294,'',1000 from dual union all
select 61726,'1988-S85','1988-05-01 00:00:00','suborbital rocket','success',741,'MR-20','','','','Aeron',636,'',250 from dual union all
select 61727,'1988-S86','1988-05-01 00:00:00','suborbital rocket','success',741,'MR-20','','','','Aeron',636,'',250 from dual union all
select 61728,'1988-S87','1988-05-01 00:00:00','suborbital rocket','success',741,'MR-20','','','','Aeron',636,'',250 from dual union all
select 61729,'1988-S88','1988-05-01 00:00:00','suborbital rocket','success',741,'MR-20','','','','Aeron',636,'',250 from dual union all
select 6173,'1959-S394','1959-11-26 00:00:00','military missile','success',975,'I13-2T','','','','Test',636,'',80 from dual union all
select 61730,'1988-S89','1988-05-01 00:00:00','suborbital rocket','success',741,'MR-20','','','','Aeron',636,'',250 from dual union all
select 61731,'1988-S90','1988-05-02 04:00:00','suborbital rocket','success',210,'DLR K-NBB5-155','TEXUS 17','','','Micrograv',417,'',285 from dual union all
select 61732,'1988-S91','1988-05-03 15:00:00','suborbital rocket','success',860,'NASA 31.69UU','CWAS 15','','','Aeronomy',1895,'',140 from dual union all
select 61733,'1988-S92','1988-05-06 04:30:00','suborbital rocket','success',210,'DLR K-NBB5-156','TEXUS 18','','','Micrograv',417,'',265 from dual union all
select 61734,'1988-S93','1988-05-09 09:05:00','suborbital rocket','success',1212,'DLR K-GR-157','Testflug (BAE 1','','','Test',416,'',262 from dual union all
select 61735,'1988-S94','1988-05-10 14:30:00','suborbital rocket','success',860,'NASA 31.68UU','CWAS 16','','','Aeronomy',1895,'',140 from dual union all
select 61736,'1988-S95','1988-05-15 03:45:00','military missile','unknown',1442,'','','','','OT',1897,'SSBN 731',1000 from dual union all
select 61737,'1988-S96','1988-05-26 02:07:00','military missile','success',1443,'D5X-7','R&D Mk 4','','','Test',294,'',1000 from dual union all
select 61738,'1988-S97','1988-06-01 00:00:00','suborbital rocket','success',741,'MR-20','','','','Aeron',636,'',250 from dual union all
select 61739,'1988-S98','1988-06-03 13:28:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT127GB','GLORY TRIP 127GB','OT',1712,'',1300 from dual union all
select 6174,'1959-S395','1959-11-27 00:00:00','military missile','success',975,'I13-12','','','','Test',636,'',80 from dual union all
select 61740,'1988-S99','1988-06-21 02:00:00','test rocket','success',647,'','BOR-5 No. 505','','','Reentry Test',621,'',210 from dual union all
select 61741,'1988-W01','1988-01-06 02:24:00','suborbital rocket','success',761,'MT135P-R660','','','','Meteo',1332,'',60 from dual union all
select 61742,'1988-W02','1988-01-12 16:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 61743,'1988-W03','1988-01-13 17:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 61744,'1988-W04','1988-01-18 22:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',77 from dual union all
select 61745,'1988-W05','1988-01-19 04:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',77 from dual union all
select 61746,'1988-W06','1988-01-19 10:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',74 from dual union all
select 61747,'1988-W07','1988-01-19 13:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',76 from dual union all
select 61748,'1988-W08','1988-01-19 16:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',79 from dual;
insert into LAUNCH
select 61749,'1988-W09','1988-01-20 02:00:00','suborbital rocket','success',761,'MT135P-R662','','','','Meteo',1332,'',56 from dual union all
select 6175,'1959-S396','1959-12-12 00:00:00','military missile','success',975,'I6-11','','','','Test',636,'',80 from dual union all
select 61750,'1988-W10','1988-01-20 15:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 61751,'1988-W100','1988-04-27 16:09:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',79 from dual union all
select 61752,'1988-W101','1988-05-02 14:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',77 from dual union all
select 61753,'1988-W102','1988-05-03 13:58:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',78 from dual union all
select 61754,'1988-W103','1988-05-04 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',77 from dual union all
select 61755,'1988-W104','1988-05-06 13:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 61756,'1988-W105','1988-05-11 02:03:00','suborbital rocket','success',761,'MT135P-R673','','','','Meteo',1332,'',58 from dual union all
select 61757,'1988-W106','1988-05-11 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 61758,'1988-W107','1988-05-18 08:36:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',79 from dual union all
select 61759,'1988-W108','1988-05-18 15:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',77 from dual union all
select 6176,'1959-S397','1959-12-12 00:00:00','military missile','success',975,'I6-6T','','','','Test',636,'',80 from dual union all
select 61760,'1988-W109','1988-05-24 13:09:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',66 from dual union all
select 61761,'1988-W11','1988-01-21 04:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',77 from dual union all
select 61762,'1988-W110','1988-05-24 13:36:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 61763,'1988-W111','1988-05-25 02:00:00','suborbital rocket','success',761,'MT135P-R674','','','','Meteo',1332,'',53 from dual union all
select 61764,'1988-W112','1988-05-25 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1642,'USHAKOV',60 from dual union all
select 61765,'1988-W113','1988-05-27 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1642,'USHAKOV',60 from dual union all
select 61766,'1988-W114','1988-05-29 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1642,'USHAKOV',60 from dual union all
select 61767,'1988-W115','1988-06-01 02:00:00','suborbital rocket','success',761,'MT135P-R675','','','','Meteo',1332,'',52 from dual union all
select 61768,'1988-W116','1988-06-01 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1642,'USHAKOV',60 from dual union all
select 61769,'1988-W117','1988-06-03 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',1642,'USHAKOV',60 from dual union all
select 6177,'1959-S398','1959-12-17 00:00:00','military missile','success',975,'I14-1T','','','','Test',636,'',80 from dual union all
select 61770,'1988-W118','1988-06-03 15:46:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',54 from dual union all
select 61771,'1988-W119','1988-06-06 13:52:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',64 from dual union all
select 61772,'1988-W12','1988-01-21 07:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 61773,'1988-W120','1988-06-07 21:45:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',68 from dual union all
select 61774,'1988-W121','1988-06-08 02:00:00','suborbital rocket','success',761,'MT135P-R676','','','','Meteo',1332,'',55 from dual union all
select 61775,'1988-W122','1988-06-08 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1642,'USHAKOV',58 from dual union all
select 61776,'1988-W123','1988-06-10 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1642,'USHAKOV',59 from dual union all
select 61777,'1988-W124','1988-06-15 20:02:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',62 from dual union all
select 61778,'1988-W125','1988-06-16 22:09:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',66 from dual union all
select 61779,'1988-W126','1988-06-17 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1642,'USHAKOV',58 from dual union all
select 6178,'1959-S399','1959-12-19 00:00:00','military missile','success',975,'I14-0','','','','Test',636,'',80 from dual union all
select 61780,'1988-W127','1988-06-17 15:17:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',64 from dual union all
select 61781,'1988-W128','1988-06-22 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1642,'USHAKOV',59 from dual union all
select 61782,'1988-W129','1988-06-22 13:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',56 from dual union all
select 61783,'1988-W13','1988-01-21 11:37:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',56 from dual union all
select 61784,'1988-W130','1988-06-24 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1642,'USHAKOV',59 from dual union all
select 61785,'1988-W131','1988-06-27 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',75 from dual union all
select 61786,'1988-W132','1988-06-29 02:00:00','suborbital rocket','success',761,'MT135P-R678','','','','Meteo',1332,'',57 from dual union all
select 61787,'1988-W133','1988-06-29 17:44:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 61788,'1988-W134','1988-06-30 18:21:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',65 from dual union all
select 61789,'1988-W135','1988-07-12 13:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',71 from dual union all
select 6179,'1959-S40','1959-03-30 16:45:00','suborbital rocket','success',30,'NRL 52','','','','Aeron',1869,'',238 from dual union all
select 61790,'1988-W136','1988-07-13 02:14:00','suborbital rocket','success',761,'MT135P-R679','','','','Meteo',1332,'',56 from dual union all
select 61791,'1988-W137','1988-07-14 21:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',70 from dual union all
select 61792,'1988-W138','1988-07-27 02:00:00','suborbital rocket','success',761,'MT135P-R680','','','','Meteo',1332,'',56 from dual union all
select 61793,'1988-W139','1988-08-01 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',75 from dual union all
select 61794,'1988-W14','1988-01-21 12:45:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 61795,'1988-W140','1988-08-03 20:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 61796,'1988-W141','1988-08-04 02:00:00','suborbital rocket','success',761,'MT135P-R681','','','','Meteo',1332,'',53 from dual union all
select 61797,'1988-W142','1988-08-11 17:37:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',63 from dual union all
select 61798,'1988-W143','1988-08-15 14:20:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 61799,'1988-W144','1988-08-17 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',68 from dual union all
select 618,'1946-A01','1946-03-19 00:00:00','suborbital rocket','failure',611,'','','','','Test',636,'',1 from dual union all
select 6180,'1959-S400','1959-12-24 00:00:00','military missile','success',971,'B1-25','','','','Test',636,'',150 from dual union all
select 61800,'1988-W145','1988-08-17 19:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',75 from dual union all
select 61801,'1988-W146','1988-08-18 02:00:00','suborbital rocket','success',761,'MT135P-R682','','','','Meteo',1332,'',55 from dual union all
select 61802,'1988-W147','1988-08-24 12:17:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',68 from dual union all
select 61803,'1988-W148','1988-08-24 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 61804,'1988-W149','1988-08-24 20:53:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',65 from dual union all
select 61805,'1988-W15','1988-01-21 13:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 61806,'1988-W150','1988-08-25 16:30:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',64 from dual union all
select 61807,'1988-W151','1988-08-28 06:06:00','suborbital rocket','unknown',738,'','','','','Aeron',1643,'USHAKOV',58 from dual union all
select 61808,'1988-W152','1988-08-31 02:00:00','suborbital rocket','success',761,'MT135P-R683','','','','Meteo',1332,'',54 from dual union all
select 61809,'1988-W153','1988-08-31 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',1643,'USHAKOV',59 from dual union all
select 6181,'1959-S401','1959-12-24 00:00:00','military missile','success',971,'B1-16','','','','Test',636,'',150 from dual union all
select 61810,'1988-W154','1988-08-31 12:17:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 61811,'1988-W155','1988-09-02 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',1643,'USHAKOV',60 from dual union all
select 61812,'1988-W156','1988-09-02 22:09:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',64 from dual union all
select 61813,'1988-W157','1988-09-09 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',1643,'USHAKOV',59 from dual union all
select 61814,'1988-W158','1988-09-11 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1643,'USHAKOV',58 from dual union all
select 61815,'1988-W159','1988-09-14 07:02:00','suborbital rocket','unknown',738,'','','','','Aeron',1643,'USHAKOV',58 from dual union all
select 61816,'1988-W16','1988-01-21 16:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 61817,'1988-W160','1988-09-14 13:55:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 61818,'1988-W161','1988-09-14 16:42:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',58 from dual union all
select 61819,'1988-W162','1988-09-15 22:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',68 from dual union all
select 6182,'1959-S402','1959-12-27 00:00:00','military missile','success',971,'B1-22','','','','Test',636,'',150 from dual union all
select 61820,'1988-W163','1988-09-16 07:05:00','suborbital rocket','unknown',738,'','','','','Aeron',1643,'USHAKOV',59 from dual union all
select 61821,'1988-W164','1988-09-21 07:07:00','suborbital rocket','unknown',738,'','','','','Aeron',1643,'USHAKOV',58 from dual union all
select 61822,'1988-W165','1988-09-21 13:11:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',60 from dual union all
select 61823,'1988-W166','1988-09-21 14:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 61824,'1988-W167','1988-09-22 22:29:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',70 from dual union all
select 61825,'1988-W168','1988-09-23 07:07:00','suborbital rocket','unknown',738,'','','','','Aeron',1643,'USHAKOV',59 from dual union all
select 61826,'1988-W169','1988-09-24 10:09:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',67 from dual union all
select 61827,'1988-W17','1988-01-22 23:41:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',72 from dual union all
select 61828,'1988-W170','1988-09-28 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 61829,'1988-W171','1988-09-28 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 6183,'1959-S403','1959-12-28 00:00:00','military missile','success',971,'B1-24','','','','Test',636,'',150 from dual union all
select 61830,'1988-W172','1988-09-29 02:00:00','suborbital rocket','success',761,'MT135P-R685','','','','Meteo',1332,'',55 from dual union all
select 61831,'1988-W173','1988-09-29 12:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',68 from dual union all
select 61832,'1988-W174','1988-09-29 13:29:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',63 from dual union all
select 61833,'1988-W175','1988-09-29 13:33:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',60 from dual union all
select 61834,'1988-W176','1988-09-29 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',69 from dual union all
select 61835,'1988-W177','1988-09-29 15:27:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 61836,'1988-W178','1988-10-01 14:52:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',62 from dual union all
select 61837,'1988-W179','1988-10-03 13:29:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 61838,'1988-W18','1988-01-25 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual;
insert into LAUNCH
select 61839,'1988-W180','1988-10-03 17:08:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',61 from dual union all
select 6184,'1959-S404','1959-12-29 00:00:00','military missile','success',971,'B1-19','','','','Test',636,'',150 from dual union all
select 61840,'1988-W181','1988-10-05 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',52 from dual union all
select 61841,'1988-W182','1988-10-12 02:02:00','suborbital rocket','success',761,'MT135P-R686','','','','Meteo',1332,'',55 from dual union all
select 61842,'1988-W183','1988-10-20 09:56:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',68 from dual union all
select 61843,'1988-W184','1988-10-26 02:00:00','suborbital rocket','success',761,'MT135P-R687','','','','Meteo',1332,'',56 from dual union all
select 61844,'1988-W185','1988-10-26 16:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',61 from dual union all
select 61845,'1988-W186','1988-10-28 22:37:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',69 from dual union all
select 61846,'1988-W187','1988-11-01 13:01:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',67 from dual union all
select 61847,'1988-W188','1988-11-02 07:07:00','suborbital rocket','unknown',738,'','','','','Aeron',1032,'PASSAT',62 from dual union all
select 61848,'1988-W189','1988-11-02 11:10:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',63 from dual union all
select 61849,'1988-W19','1988-01-27 13:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',74 from dual union all
select 6185,'1959-S405','1959-12-29 00:00:00','military missile','success',971,'B1-23','','','','Test',636,'',150 from dual union all
select 61850,'1988-W190','1988-11-08 13:11:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',78 from dual union all
select 61851,'1988-W191','1988-11-09 02:00:00','suborbital rocket','success',761,'MT135P-R688','','','','Meteo',1332,'',58 from dual union all
select 61852,'1988-W192','1988-11-09 07:07:00','suborbital rocket','unknown',738,'','','','','Aeron',1032,'PASSAT',63 from dual union all
select 61853,'1988-W193','1988-11-09 09:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',63 from dual union all
select 61854,'1988-W194','1988-11-11 07:03:00','suborbital rocket','unknown',738,'','','','','Aeron',1032,'PASSAT',62 from dual union all
select 61855,'1988-W195','1988-11-14 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1032,'PASSAT',62 from dual union all
select 61856,'1988-W196','1988-11-15 04:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 61857,'1988-W197','1988-11-16 02:00:00','suborbital rocket','success',761,'MT135P-R689','','','','Meteo',1332,'',54 from dual union all
select 61858,'1988-W198','1988-11-16 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',1032,'PASSAT',60 from dual union all
select 61859,'1988-W199','1988-11-18 07:02:00','suborbital rocket','unknown',738,'','','','','Aeron',1032,'PASSAT',62 from dual union all
select 6186,'1959-S406','1959-12-25 00:00:00','military missile','success',975,'I15-10','','','','Test',636,'',80 from dual union all
select 61860,'1988-W20','1988-01-27 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 61861,'1988-W200','1988-11-22 13:20:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',53 from dual union all
select 61862,'1988-W201','1988-11-23 07:16:00','suborbital rocket','unknown',738,'','','','','Aeron',1032,'PASSAT',62 from dual union all
select 61863,'1988-W202','1988-11-23 11:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',78 from dual union all
select 61864,'1988-W203','1988-11-23 13:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',76 from dual union all
select 61865,'1988-W204','1988-11-28 07:05:00','suborbital rocket','unknown',738,'','','','','Aeron',1032,'PASSAT',63 from dual union all
select 61866,'1988-W205','1988-11-29 22:18:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',69 from dual union all
select 61867,'1988-W206','1988-11-30 02:02:00','suborbital rocket','success',761,'MT135P-R691','','','','Meteo',1332,'',54 from dual union all
select 61868,'1988-W207','1988-12-02 16:13:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',67 from dual union all
select 61869,'1988-W208','1988-12-02 16:57:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',72 from dual union all
select 6187,'1959-S407','1959-12-25 00:00:00','military missile','success',975,'I15-2T','','','','Test',636,'',80 from dual union all
select 61870,'1988-W209','1988-12-04 17:15:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 61871,'1988-W21','1988-01-28 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',56 from dual union all
select 61872,'1988-W210','1988-12-04 18:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 61873,'1988-W211','1988-12-06 15:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',54 from dual union all
select 61874,'1988-W212','1988-12-06 16:22:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',63 from dual union all
select 61875,'1988-W213','1988-12-07 00:31:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 61876,'1988-W214','1988-12-07 02:00:00','suborbital rocket','success',761,'MT135P-R692','','','','Meteo',1332,'',52 from dual union all
select 61877,'1988-W215','1988-12-07 11:15:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',75 from dual union all
select 61878,'1988-W216','1988-12-14 02:00:00','suborbital rocket','success',761,'MT135P-R693','','','','Meteo',1332,'',56 from dual union all
select 61879,'1988-W217','1988-12-14 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 6188,'1959-S408','1959-12-26 00:00:00','military missile','success',975,'I9-15','','','','Test',636,'',80 from dual union all
select 61880,'1988-W218','1988-12-14 19:57:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',72 from dual union all
select 61881,'1988-W219','1988-12-15 11:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',61 from dual union all
select 61882,'1988-W22','1988-02-01 12:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',76 from dual union all
select 61883,'1988-W220','1988-12-18 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',59 from dual union all
select 61884,'1988-W221','1988-12-20 08:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',58 from dual union all
select 61885,'1988-W222','1988-12-21 02:00:00','suborbital rocket','success',761,'MT135P-R694','','','','Meteo',1332,'',59 from dual union all
select 61886,'1988-W223','1988-12-21 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',55 from dual union all
select 61887,'1988-W224','1988-12-22 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',60 from dual union all
select 61888,'1988-W225','1988-12-25 10:59:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',57 from dual union all
select 61889,'1988-W226','1988-12-27 13:32:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',59 from dual union all
select 6189,'1959-S409','1959-12-27 00:00:00','military missile','success',975,'I9-6T','','','','Test',636,'',80 from dual union all
select 61890,'1988-W227','1988-12-27 15:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual union all
select 61891,'1988-W228','1988-12-28 02:12:00','suborbital rocket','success',761,'MT135P-R695','','','','Meteo',1332,'',55 from dual union all
select 61892,'1988-W229','1988-12-28 11:09:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',59 from dual union all
select 61893,'1988-W23','1988-02-02 14:02:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 61894,'1988-W230','1988-12-28 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',79 from dual union all
select 61895,'1988-W231','1988-12-28 15:10:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual union all
select 61896,'1988-W232','1988-09-11 02:25:00','suborbital rocket','success',759,'MT-135-49','NNP','','','Meteo',839,'',58 from dual union all
select 61897,'1988-W233','1988-01-13 02:00:00','suborbital rocket','success',761,'MT135P-R661','','','','Meteo',1332,'',60 from dual union all
select 61898,'1988-W234','1988-03-03 02:00:00','suborbital rocket','success',761,'MT135P-R666','','','','Meteo',1332,'',60 from dual union all
select 61899,'1988-W235','1988-03-09 02:05:00','suborbital rocket','success',761,'MT135P-R667','','','','Meteo',1332,'',60 from dual union all
select 619,'1946-A02','1946-04-16 21:47:00','suborbital rocket','failure',1470,'2','','','','Test/CR',1863,'',8 from dual union all
select 6190,'1959-S41','1959-04-03 17:11:00','military missile','failure',1419,'A-4','P200 RV','Dummy St 2','','Test',230,'',200 from dual union all
select 61900,'1988-W236','1988-06-15 02:00:00','suborbital rocket','success',761,'MT135P-R677','','','','Meteo',1332,'',60 from dual union all
select 61901,'1988-W237','1988-09-14 02:00:00','suborbital rocket','success',761,'MT135P-R684','','','','Meteo',1332,'',60 from dual union all
select 61902,'1988-W238','1988-11-23 02:00:00','suborbital rocket','success',761,'MT135P-R690','','','','Meteo',1332,'',60 from dual union all
select 61903,'1988-W24','1988-02-02 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',77 from dual union all
select 61904,'1988-W25','1988-02-06 20:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 61905,'1988-W26','1988-02-08 22:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',68 from dual union all
select 61906,'1988-W27','1988-02-08 23:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 61907,'1988-W28','1988-02-10 02:01:00','suborbital rocket','success',761,'MT135P-R663','','','','Meteo',1332,'',55 from dual union all
select 61908,'1988-W29','1988-02-12 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 61909,'1988-W30','1988-02-15 07:10:00','suborbital rocket','unknown',738,'','','','','Aeron',1641,'USHAKOV',63 from dual union all
select 6191,'1959-S410','1959-12-24 00:00:00','military missile','success',983,'BK1-04 (R-11MU)','ZI','','','Test',636,'',150 from dual union all
select 61910,'1988-W31','1988-02-15 17:01:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 61911,'1988-W32','1988-02-17 02:00:00','suborbital rocket','success',761,'MT135P-R664','','','','Meteo',1332,'',55 from dual union all
select 61912,'1988-W33','1988-02-17 07:02:00','suborbital rocket','unknown',738,'','','','','Aeron',1641,'USHAKOV',60 from dual union all
select 61913,'1988-W34','1988-02-19 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1641,'USHAKOV',57 from dual union all
select 61914,'1988-W35','1988-02-23 10:18:00','suborbital rocket','unknown',738,'','','','','Aeron',1303,'PRILIV',59 from dual union all
select 61915,'1988-W36','1988-02-24 02:00:00','suborbital rocket','success',761,'MT135P-R665','','','','Meteo',1332,'',55 from dual union all
select 61916,'1988-W37','1988-02-24 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1641,'USHAKOV',60 from dual union all
select 61917,'1988-W38','1988-02-24 11:05:00','suborbital rocket','unknown',738,'','','','','Aeron',1303,'PRILIV',56 from dual union all
select 61918,'1988-W39','1988-02-24 17:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',67 from dual union all
select 61919,'1988-W40','1988-02-24 19:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',79 from dual union all
select 6192,'1959-S411','1959-12-24 00:00:00','military missile','success',992,'S4-7','','','','OT',636,'',170 from dual union all
select 61920,'1988-W41','1988-02-25 12:31:00','suborbital rocket','unknown',738,'','','','','Aeron',1303,'PRILIV',58 from dual union all
select 61921,'1988-W42','1988-02-25 15:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 61922,'1988-W43','1988-02-26 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1641,'USHAKOV',60 from dual union all
select 61923,'1988-W44','1988-02-26 11:08:00','suborbital rocket','unknown',738,'','','','','Aeron',1303,'PRILIV',57 from dual union all
select 61924,'1988-W45','1988-02-27 11:05:00','suborbital rocket','unknown',738,'','','','','Aeron',1303,'PRILIV',59 from dual union all
select 61925,'1988-W46','1988-02-27 17:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',64 from dual union all
select 61926,'1988-W47','1988-02-28 11:13:00','suborbital rocket','unknown',738,'','','','','Aeron',1303,'PRILIV',60 from dual union all
select 61927,'1988-W48','1988-02-29 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1641,'USHAKOV',60 from dual union all
select 61928,'1988-W49','1988-03-02 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1641,'USHAKOV',59 from dual;
insert into LAUNCH
select 61929,'1988-W50','1988-03-02 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',76 from dual union all
select 6193,'1959-S412','1959-12-24 00:00:00','military missile','success',992,'S4-13','','','','OT',636,'',170 from dual union all
select 61930,'1988-W51','1988-03-04 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1641,'USHAKOV',60 from dual union all
select 61931,'1988-W52','1988-03-08 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 61932,'1988-W53','1988-03-09 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1641,'USHAKOV',60 from dual union all
select 61933,'1988-W54','1988-03-09 13:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',77 from dual union all
select 61934,'1988-W55','1988-03-10 14:32:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',56 from dual union all
select 61935,'1988-W56','1988-03-11 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1641,'USHAKOV',61 from dual union all
select 61936,'1988-W57','1988-03-16 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',51 from dual union all
select 61937,'1988-W58','1988-03-17 02:00:00','suborbital rocket','success',761,'MT135P-R668','','','','Meteo',1332,'',57 from dual union all
select 61938,'1988-W59','1988-03-17 11:37:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',77 from dual union all
select 61939,'1988-W60','1988-03-18 07:42:00','suborbital rocket','unknown',738,'','','','','Aeron',1031,'PASSAT',63 from dual union all
select 6194,'1959-S413','1959-12-28 00:00:00','military missile','success',992,'S4-12','','','','OT',636,'',170 from dual union all
select 61940,'1988-W61','1988-03-18 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',51 from dual union all
select 61941,'1988-W62','1988-03-20 16:24:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',74 from dual union all
select 61942,'1988-W63','1988-03-21 17:53:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 61943,'1988-W64','1988-03-23 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1031,'PASSAT',62 from dual union all
select 61944,'1988-W65','1988-03-25 07:16:00','suborbital rocket','unknown',738,'','','','','Aeron',1031,'PASSAT',61 from dual union all
select 61945,'1988-W66','1988-03-25 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',57 from dual union all
select 61946,'1988-W67','1988-03-30 02:00:00','suborbital rocket','success',761,'MT135P-R669','','','','Meteo',1332,'',57 from dual union all
select 61947,'1988-W68','1988-03-30 06:09:00','suborbital rocket','unknown',738,'','','','','Aeron',1031,'PASSAT',59 from dual union all
select 61948,'1988-W69','1988-03-30 13:09:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',72 from dual union all
select 61949,'1988-W70','1988-04-01 06:48:00','suborbital rocket','unknown',738,'','','','','Aeron',1031,'PASSAT',60 from dual union all
select 6195,'1959-S414','1959-12-28 00:00:00','military missile','success',992,'S4-9','','','','OT',636,'',170 from dual union all
select 61950,'1988-W71','1988-04-04 06:03:00','suborbital rocket','unknown',738,'','','','','Aeron',1031,'PASSAT',60 from dual union all
select 61951,'1988-W72','1988-04-06 06:05:00','suborbital rocket','unknown',738,'','','','','Aeron',1031,'PASSAT',60 from dual union all
select 61952,'1988-W73','1988-04-06 11:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 61953,'1988-W74','1988-04-07 12:10:00','suborbital rocket','unknown',738,'','','','','Aeron',1304,'PRILIV',59 from dual union all
select 61954,'1988-W75','1988-04-07 16:04:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',77 from dual union all
select 61955,'1988-W76','1988-04-08 13:09:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 61956,'1988-W77','1988-04-10 10:06:00','suborbital rocket','unknown',738,'','','','','Aeron',1304,'PRILIV',57 from dual union all
select 61957,'1988-W78','1988-04-10 14:45:00','suborbital rocket','unknown',738,'','','','','Aeron',1304,'PRILIV',58 from dual union all
select 61958,'1988-W79','1988-04-11 15:54:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',76 from dual union all
select 61959,'1988-W80','1988-04-11 20:37:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',66 from dual union all
select 6196,'1959-S415','1959-04-29 00:00:00','military missile','success',975,'B2-20','','','','Test',636,'',80 from dual union all
select 61960,'1988-W81','1988-04-13 07:02:00','suborbital rocket','unknown',738,'','','','','Aeron',1031,'PASSAT',61 from dual union all
select 61961,'1988-W82','1988-04-13 11:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',78 from dual union all
select 61962,'1988-W83','1988-04-13 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 61963,'1988-W84','1988-04-13 14:27:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',77 from dual union all
select 61964,'1988-W85','1988-04-14 01:00:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',66 from dual union all
select 61965,'1988-W86','1988-04-14 02:00:00','suborbital rocket','success',761,'MT135P-R670','','','','Meteo',1332,'',55 from dual union all
select 61966,'1988-W87','1988-04-14 15:01:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',73 from dual union all
select 61967,'1988-W88','1988-04-15 18:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',67 from dual union all
select 61968,'1988-W89','1988-04-15 20:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',65 from dual union all
select 61969,'1988-W90','1988-04-16 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1304,'PRILIV',58 from dual union all
select 6197,'1959-S416','1959-04-28 00:00:00','military missile','success',971,'I2-9','','','','Test',636,'',150 from dual union all
select 61970,'1988-W91','1988-04-17 10:25:00','suborbital rocket','unknown',738,'','','','','Aeron',1304,'PRILIV',59 from dual union all
select 61971,'1988-W92','1988-04-18 10:25:00','suborbital rocket','unknown',738,'','','','','Aeron',1304,'PRILIV',55 from dual union all
select 61972,'1988-W93','1988-04-18 13:12:00','suborbital rocket','unknown',738,'','','','','Aeron',1304,'PRILIV',60 from dual union all
select 61973,'1988-W94','1988-04-20 02:00:00','suborbital rocket','success',761,'MT135P-R671','','','','Meteo',1332,'',55 from dual union all
select 61974,'1988-W95','1988-04-20 16:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 61975,'1988-W96','1988-04-26 20:15:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',56 from dual union all
select 61976,'1988-W97','1988-04-27 02:02:00','suborbital rocket','success',761,'MT135P-R672','','','','Meteo',1332,'',54 from dual union all
select 61977,'1988-W98','1988-04-27 13:41:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',72 from dual union all
select 61978,'1988-W99','1988-04-27 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 61979,'1988-Y01','1988-01-06 10:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 6198,'1959-S417','1959-04-23 00:00:00','military missile','success',975,'B2-12T','','','','Test',636,'',80 from dual union all
select 61980,'1988-Y02','1988-01-06 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 61981,'1988-Y03','1988-01-06 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 61982,'1988-Y04','1988-01-11 11:37:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 61983,'1988-Y05','1988-01-11 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 61984,'1988-Y06','1988-01-13 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 61985,'1988-Y07','1988-01-13 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 61986,'1988-Y08','1988-01-13 14:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',87 from dual union all
select 61987,'1988-Y09','1988-01-13 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61988,'1988-Y10','1988-01-13 21:25:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 61989,'1988-Y100','1988-04-20 14:35:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 6199,'1959-S418','1959-04-24 00:00:00','military missile','success',975,'B2-6','','','','Test',636,'',80 from dual union all
select 61990,'1988-Y101','1988-04-20 16:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 61991,'1988-Y102','1988-04-20 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',89 from dual union all
select 61992,'1988-Y103','1988-04-20 21:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 61993,'1988-Y104','1988-04-21 11:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 61994,'1988-Y105','1988-04-22 16:45:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',89 from dual union all
select 61995,'1988-Y106','1988-04-27 11:34:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 61996,'1988-Y107','1988-04-27 15:22:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 61997,'1988-Y108','1988-04-27 16:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 61998,'1988-Y109','1988-04-27 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 61999,'1988-Y11','1988-01-14 11:17:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',85 from dual union all
select 62,'1944-A26','1944-12-12 13:07:00','military missile','failure',1470,'19855','Ma323','','','Test',661,'',0 from dual union all
select 620,'1946-A03','1946-05-01 00:00:00','atmospheric rocket','success',471,'','RM-5 Model','','','Test',1821,'',5 from dual union all
select 6200,'1959-S419','1959-04-28 00:00:00','military missile','success',971,'I2-3T','','','','Test',636,'',150 from dual union all
select 62000,'1988-Y110','1988-04-30 12:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',86 from dual union all
select 62001,'1988-Y111','1988-05-04 08:42:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 62002,'1988-Y112','1988-05-04 11:34:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 62003,'1988-Y113','1988-05-04 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 62004,'1988-Y114','1988-05-04 18:03:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',88 from dual union all
select 62005,'1988-Y115','1988-05-11 11:09:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 62006,'1988-Y116','1988-05-11 11:42:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 62007,'1988-Y117','1988-05-11 15:25:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 62008,'1988-Y118','1988-05-11 19:15:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 62009,'1988-Y119','1988-05-18 10:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 6201,'1959-S42','1959-04-04 00:00:00','test rocket','success',254,'','','','','Interceptor',21,'B-47',200 from dual union all
select 62010,'1988-Y12','1988-01-18 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 62011,'1988-Y120','1988-05-18 18:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 62012,'1988-Y121','1988-05-19 11:51:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 62013,'1988-Y122','1988-05-25 11:04:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 62014,'1988-Y123','1988-05-25 11:38:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 62015,'1988-Y124','1988-05-25 18:31:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',89 from dual union all
select 62016,'1988-Y125','1988-05-25 19:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 62017,'1988-Y126','1988-05-25 20:31:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual;
insert into LAUNCH
select 62018,'1988-Y127','1988-06-01 08:31:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 62019,'1988-Y128','1988-06-01 11:36:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 6202,'1959-S420','1959-04-07 00:00:00','military missile','success',1013,'I1-20TS','8K51 UBP-20','','','Test',636,'',200 from dual union all
select 62020,'1988-Y129','1988-06-01 13:54:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62021,'1988-Y13','1988-01-18 22:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',89 from dual union all
select 62022,'1988-Y130','1988-06-01 16:01:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 62023,'1988-Y131','1988-06-06 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 62024,'1988-Y132','1988-06-08 08:41:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 62025,'1988-Y133','1988-06-08 10:59:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 62026,'1988-Y134','1988-06-08 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 62027,'1988-Y135','1988-06-08 16:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 62028,'1988-Y136','1988-06-15 08:27:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 62029,'1988-Y137','1988-06-15 11:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 6203,'1959-S421','1959-04-09 00:00:00','military missile','success',1013,'B1-8 8K52','8K52 Zvezda-5','Zvezda','','Test',636,'',500 from dual union all
select 62030,'1988-Y138','1988-06-15 16:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 62031,'1988-Y139','1988-06-15 17:49:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62032,'1988-Y14','1988-01-19 01:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 62033,'1988-Y140','1988-06-17 14:37:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 62034,'1988-Y141','1988-06-20 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 62035,'1988-Y142','1988-06-22 10:54:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 62036,'1988-Y143','1988-06-22 11:48:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62037,'1988-Y144','1988-06-22 17:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 62038,'1988-Y145','1988-06-22 18:36:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62039,'1988-Y146','1988-06-29 11:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 6204,'1959-S422','1959-09-25 12:17:00','suborbital rocket','success',839,'HUGO 6','Weather photo','','','Aeron',276,'',100 from dual union all
select 62040,'1988-Y147','1988-06-29 11:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 62041,'1988-Y148','1988-06-29 16:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 62042,'1988-Y149','1988-07-04 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 62043,'1988-Y15','1988-01-19 01:35:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 62044,'1988-Y150','1988-07-06 11:17:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 62045,'1988-Y151','1988-07-06 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62046,'1988-Y152','1988-07-06 15:03:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 62047,'1988-Y153','1988-07-06 18:32:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',91 from dual union all
select 62048,'1988-Y154','1988-07-11 14:37:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62049,'1988-Y155','1988-07-11 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 6205,'1959-S423','1959-04-14 00:00:00','military missile','success',1013,'B1-6 8K52','8K52 Zvezda-6','Zvezda','','Test',636,'',500 from dual union all
select 62050,'1988-Y156','1988-07-11 21:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 62051,'1988-Y157','1988-07-11 22:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 62052,'1988-Y158','1988-07-12 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 62053,'1988-Y159','1988-07-12 01:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 62054,'1988-Y16','1988-01-19 04:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 62055,'1988-Y160','1988-07-12 03:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 62056,'1988-Y161','1988-07-12 06:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 62057,'1988-Y162','1988-07-12 07:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 62058,'1988-Y163','1988-07-12 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 62059,'1988-Y164','1988-07-12 10:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 6206,'1959-S424','1959-09-19 00:00:00','military missile','success',1013,'','8K51 UBP-26','','','Test',636,'',500 from dual union all
select 62060,'1988-Y165','1988-07-12 13:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 62061,'1988-Y166','1988-07-12 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 62062,'1988-Y167','1988-07-12 15:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 62063,'1988-Y168','1988-07-12 16:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 62064,'1988-Y169','1988-07-12 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 62065,'1988-Y17','1988-01-19 07:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 62066,'1988-Y170','1988-07-12 21:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 62067,'1988-Y171','1988-07-13 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 62068,'1988-Y172','1988-07-13 03:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 62069,'1988-Y173','1988-07-13 05:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 6207,'1959-S43','1959-04-04 00:34:00','military missile','success',638,'CM-22A','R&D','','','Test',249,'',500 from dual union all
select 62070,'1988-Y174','1988-07-13 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 62071,'1988-Y175','1988-07-13 11:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62072,'1988-Y176','1988-07-13 12:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 62073,'1988-Y177','1988-07-13 15:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 62074,'1988-Y178','1988-07-13 17:39:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62075,'1988-Y179','1988-07-13 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 62076,'1988-Y18','1988-01-19 07:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 62077,'1988-Y180','1988-07-13 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 62078,'1988-Y181','1988-07-13 22:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 62079,'1988-Y182','1988-07-14 01:03:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 6208,'1959-S44','1959-04-07 14:46:00','suborbital rocket','failure',30,'AA3.160C','','','','Aeron',1869,'',317 from dual union all
select 62080,'1988-Y183','1988-07-14 04:28:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 62081,'1988-Y184','1988-07-14 07:06:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 62082,'1988-Y185','1988-07-14 10:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 62083,'1988-Y186','1988-07-14 15:07:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 62084,'1988-Y187','1988-07-18 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',89 from dual union all
select 62085,'1988-Y188','1988-07-20 11:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62086,'1988-Y189','1988-07-20 15:20:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62087,'1988-Y19','1988-01-19 10:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 62088,'1988-Y190','1988-07-20 15:24:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 62089,'1988-Y191','1988-07-20 16:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 6209,'1959-S45','1959-04-08 06:35:00','military missile','success',1355,'133','RVX-1','','','Reentry Test/Particles',233,'',1230 from dual union all
select 62090,'1988-Y192','1988-07-25 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',88 from dual union all
select 62091,'1988-Y193','1988-07-27 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 62092,'1988-Y194','1988-07-27 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 62093,'1988-Y195','1988-07-27 15:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62094,'1988-Y196','1988-07-27 15:16:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 62095,'1988-Y197','1988-08-03 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 62096,'1988-Y198','1988-08-03 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 62097,'1988-Y199','1988-08-03 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 62098,'1988-Y20','1988-01-19 13:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 62099,'1988-Y200','1988-08-10 11:59:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 621,'1946-A04','1946-05-10 00:00:00','suborbital rocket','success',1516,'12','Demo round','','','Test',1852,'',20 from dual union all
select 6210,'1959-S46','1959-04-11 00:00:00','military missile','success',976,'I1-5T','8K63 UBP-3','','','Test',636,'',402 from dual union all
select 62100,'1988-Y201','1988-08-10 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 62101,'1988-Y202','1988-08-10 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 62102,'1988-Y203','1988-08-17 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 62103,'1988-Y204','1988-08-17 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 62104,'1988-Y205','1988-08-17 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62105,'1988-Y206','1988-08-24 12:25:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 62106,'1988-Y207','1988-08-24 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 62107,'1988-Y208','1988-08-24 15:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual;
insert into LAUNCH
select 62108,'1988-Y21','1988-01-19 16:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 62109,'1988-Y210','1988-08-31 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 6211,'1959-S47','1959-04-14 00:00:00','military missile','success',976,'I1-3T','8K63 UBP-4','','','Test',636,'',402 from dual union all
select 62110,'1988-Y211','1988-08-31 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 62111,'1988-Y212','1988-08-31 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 62112,'1988-Y213','1988-08-31 14:21:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 62113,'1988-Y214','1988-09-07 11:41:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',86 from dual union all
select 62114,'1988-Y215','1988-09-07 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 62115,'1988-Y216','1988-09-07 14:09:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 62116,'1988-Y217','1988-09-07 14:20:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',87 from dual union all
select 62117,'1988-Y218','1988-09-10 02:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',87 from dual union all
select 62118,'1988-Y219','1988-09-10 12:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',87 from dual union all
select 62119,'1988-Y22','1988-01-19 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 6212,'1959-S48','1959-04-15 14:46:00','suborbital rocket','success',30,'AA3.180C','','','','Ionos',640,'',100 from dual union all
select 62120,'1988-Y220','1988-09-10 13:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 62121,'1988-Y221','1988-09-14 11:28:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62122,'1988-Y222','1988-09-14 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 62123,'1988-Y223','1988-09-14 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 62124,'1988-Y224','1988-09-15 21:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 62125,'1988-Y225','1988-09-21 11:27:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 62126,'1988-Y226','1988-09-21 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 62127,'1988-Y227','1988-09-21 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62128,'1988-Y228','1988-09-28 11:39:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 62129,'1988-Y229','1988-09-28 15:14:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 6213,'1959-S49','1959-04-16 20:46:00','military missile','success',1367,'161','IWST','','LIONS ROAR','Test',1677,'',520 from dual union all
select 62130,'1988-Y23','1988-01-19 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 62131,'1988-Y230','1988-09-28 15:36:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 62132,'1988-Y231','1988-09-29 12:20:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 62133,'1988-Y232','1988-09-29 14:15:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',84 from dual union all
select 62134,'1988-Y233','1988-10-01 14:15:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 62135,'1988-Y234','1988-10-03 13:00:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 62136,'1988-Y235','1988-10-05 11:17:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 62137,'1988-Y236','1988-10-05 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 62138,'1988-Y237','1988-10-05 15:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 62139,'1988-Y238','1988-10-05 20:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',88 from dual union all
select 6214,'1959-S50','1959-04-16 23:10:00','suborbital rocket','success',366,'','RW-124','','','Aeron',1490,'',83 from dual union all
select 62140,'1988-Y239','1988-10-12 11:11:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 62141,'1988-Y24','1988-01-20 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',88 from dual union all
select 62142,'1988-Y240','1988-10-12 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',88 from dual union all
select 62143,'1988-Y241','1988-10-12 14:37:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 62144,'1988-Y242','1988-10-12 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 62145,'1988-Y243','1988-10-18 17:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',88 from dual union all
select 62146,'1988-Y244','1988-10-18 19:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 62147,'1988-Y245','1988-10-19 11:07:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 62148,'1988-Y246','1988-10-19 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62149,'1988-Y247','1988-10-19 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 6215,'1959-S51','1959-04-17 05:00:00','suborbital rocket','success',366,'','RW-125','','','Aeron',1490,'',82 from dual union all
select 62150,'1988-Y248','1988-10-26 11:03:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 62151,'1988-Y249','1988-10-26 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62152,'1988-Y25','1988-01-20 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',89 from dual union all
select 62153,'1988-Y250','1988-10-26 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 62154,'1988-Y251','1988-11-01 23:52:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 62155,'1988-Y252','1988-11-02 11:01:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62156,'1988-Y253','1988-11-02 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 62157,'1988-Y254','1988-11-02 16:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 62158,'1988-Y255','1988-11-09 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 62159,'1988-Y256','1988-11-09 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 6216,'1959-S52','1959-04-20 15:34:00','military missile','success',933,'AX-6','','','','Test',244,'',200 from dual union all
select 62160,'1988-Y257','1988-11-10 10:59:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62161,'1988-Y258','1988-11-15 05:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 62162,'1988-Y259','1988-11-15 06:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 62163,'1988-Y26','1988-01-20 22:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 62164,'1988-Y260','1988-11-16 10:57:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 62165,'1988-Y261','1988-11-16 14:14:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 62166,'1988-Y262','1988-11-16 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 62167,'1988-Y263','1988-11-17 16:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',87 from dual union all
select 62168,'1988-Y264','1988-11-17 17:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 62169,'1988-Y265','1988-11-22 17:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 6217,'1959-S53','1959-04-21 01:05:00','suborbital rocket','success',30,'AA3.181C','','','','Ionos',640,'',235 from dual union all
select 62170,'1988-Y266','1988-11-22 19:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 62171,'1988-Y267','1988-11-23 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 62172,'1988-Y268','1988-11-23 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 62173,'1988-Y269','1988-11-23 20:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',89 from dual union all
select 62174,'1988-Y27','1988-01-20 22:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 62175,'1988-Y270','1988-11-30 10:57:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62176,'1988-Y271','1988-11-30 11:15:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 62177,'1988-Y272','1988-11-30 14:12:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 62178,'1988-Y273','1988-11-30 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 62179,'1988-Y274','1988-12-02 14:33:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',88 from dual union all
select 6218,'1959-S54','1959-04-22 19:29:00','suborbital rocket','failure',839,'AA6.182C','','','','Ionos',646,'',100 from dual union all
select 62180,'1988-Y275','1988-12-02 15:03:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',84 from dual union all
select 62181,'1988-Y276','1988-12-02 15:34:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 62182,'1988-Y277','1988-12-02 17:02:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 62183,'1988-Y278','1988-12-04 14:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',93 from dual union all
select 62184,'1988-Y279','1988-12-06 14:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 62185,'1988-Y28','1988-01-21 01:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 62186,'1988-Y280','1988-12-06 14:42:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',91 from dual union all
select 62187,'1988-Y281','1988-12-07 10:01:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 62188,'1988-Y282','1988-12-07 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',89 from dual union all
select 62189,'1988-Y283','1988-12-14 11:01:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 6219,'1959-S55','1959-04-23 05:30:00','military missile','success',1367,'176','R&D Series IV','','','Test',234,'',520 from dual union all
select 62190,'1988-Y284','1988-12-14 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 62191,'1988-Y285','1988-12-21 11:01:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 62192,'1988-Y286','1988-12-21 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 62193,'1988-Y287','1988-12-21 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 62194,'1988-Y288','1988-12-27 11:34:00','suborbital rocket','unknown',728,'','','','','Aeron',896,'KORO',82 from dual union all
select 62195,'1988-Y289','1988-12-28 11:08:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 62196,'1988-Y29','1988-01-21 01:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 62197,'1988-Y290','1988-12-28 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62198,'1988-Y291','1988-06-24 20:28:00','suborbital rocket','success',1502,'Chaff NC01 EL Na','Sodium-88 M-N01','','','Aeron',29,'',98 from dual;
insert into LAUNCH
select 62199,'1988-Y292','1988-06-24 21:35:00','suborbital rocket','success',1502,'Chaff NC02 EL Na','Sodium-88 M-N02','','','Aeron',29,'',98 from dual union all
select 622,'1946-A05','1946-06-01 00:00:00','suborbital rocket','failure',611,'','','','','Test',636,'',0 from dual union all
select 6220,'1959-S56','1959-04-25 05:00:00','military missile','success',1367,'164','R&D Series IV','','','Test',236,'',520 from dual union all
select 62200,'1988-Y293','1988-06-24 22:10:00','suborbital rocket','success',1502,'Chaff NC03 EL Na','Sodium-88 M-N03','','','Aeron',29,'',98 from dual union all
select 62201,'1988-Y294','1988-06-24 22:22:00','suborbital rocket','success',1502,'Chaff NC04 EL Na','Sodium-88 M-N04','','','Aeron',29,'',98 from dual union all
select 62202,'1988-Y295','1988-06-24 22:56:00','suborbital rocket','success',1502,'Chaff NC05 EL Na','Sodium-88 M-N05','','','Aeron',29,'',98 from dual union all
select 62203,'1988-Y296','1988-06-29 21:43:00','suborbital rocket','success',1502,'Chaff NC06 EL Na','Sodium-88 M-N06','','','Aeron',29,'',98 from dual union all
select 62204,'1988-Y297','1988-07-04 22:27:00','suborbital rocket','success',1502,'Chaff NC07 EL Na','Sodium-88 M-N07','','','Aeron',29,'',98 from dual union all
select 62205,'1988-Y298','1988-07-12 22:20:00','suborbital rocket','success',1502,'Chaff NC08 EL Na','Sodium-88 M-N08','','','Aeron',29,'',98 from dual union all
select 62206,'1988-Y299','1988-07-12 23:13:00','suborbital rocket','success',1502,'Chaff NC09 EL Na','Sodium-88 M-N09','','','Aeron',29,'',98 from dual union all
select 62207,'1988-Y30','1988-01-21 04:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 62208,'1988-Y300','1988-07-13 00:03:00','suborbital rocket','success',1502,'Chaff NC10 EL Na','Sodium-88 M-N10','','','Aeron',29,'',98 from dual union all
select 62209,'1988-Y31','1988-01-21 07:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 6221,'1959-S57','1959-04-29 00:00:00','military missile','success',976,'I2-1','8K63 UBP-5','','','Test',636,'',402 from dual union all
select 62210,'1988-Y32','1988-01-21 10:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 62211,'1988-Y33','1988-01-21 13:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 62212,'1988-Y34','1988-01-21 16:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 62213,'1988-Y35','1988-01-21 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 62214,'1988-Y36','1988-01-21 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 62215,'1988-Y37','1988-01-25 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 62216,'1988-Y38','1988-01-27 11:25:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',88 from dual union all
select 62217,'1988-Y39','1988-01-27 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 62218,'1988-Y40','1988-01-27 22:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 62219,'1988-Y41','1988-01-29 16:17:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',873,'',85 from dual union all
select 6222,'1959-S58','1959-05-01 00:00:00','suborbital rocket','success',720,'LT7','','','','Test/Aeron',1837,'',100 from dual union all
select 62220,'1988-Y42','1988-02-03 12:22:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',87 from dual union all
select 62221,'1988-Y43','1988-02-03 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 62222,'1988-Y44','1988-02-03 21:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 62223,'1988-Y45','1988-02-05 14:10:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',81 from dual union all
select 62224,'1988-Y46','1988-02-08 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 62225,'1988-Y47','1988-02-10 11:38:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 62226,'1988-Y48','1988-02-10 13:45:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',88 from dual union all
select 62227,'1988-Y49','1988-02-10 17:12:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 62228,'1988-Y50','1988-02-10 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 62229,'1988-Y51','1988-02-15 13:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',87 from dual union all
select 6223,'1959-S59','1959-05-02 00:00:00','military missile','success',883,'001','','','','Reentry Test',313,'',150 from dual union all
select 62230,'1988-Y52','1988-02-15 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 62231,'1988-Y53','1988-02-17 12:53:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 62232,'1988-Y54','1988-02-17 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 62233,'1988-Y55','1988-02-17 23:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 62234,'1988-Y56','1988-02-22 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 62235,'1988-Y57','1988-02-22 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 62236,'1988-Y58','1988-02-24 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',91 from dual union all
select 62237,'1988-Y59','1988-02-24 22:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62238,'1988-Y60','1988-02-26 13:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',85 from dual union all
select 62239,'1988-Y61','1988-02-27 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',84 from dual union all
select 6224,'1959-S60','1959-05-04 18:30:00','military missile','success',1419,'A-6','P200 RV','Dummy St 2,Sep test','','Test',230,'',200 from dual union all
select 62240,'1988-Y62','1988-02-29 13:27:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',87 from dual union all
select 62241,'1988-Y63','1988-02-29 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 62242,'1988-Y64','1988-03-02 13:31:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 62243,'1988-Y65','1988-03-02 21:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 62244,'1988-Y66','1988-03-03 11:56:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',87 from dual union all
select 62245,'1988-Y67','1988-03-08 19:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 62246,'1988-Y68','1988-03-09 11:37:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62247,'1988-Y69','1988-03-09 21:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 62248,'1988-Y70','1988-03-15 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 62249,'1988-Y71','1988-03-16 13:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 6225,'1959-S61','1959-05-05 01:45:00','suborbital rocket','success',48,'AA2.141C','','','','Aeron',1869,'',132 from dual union all
select 62250,'1988-Y72','1988-03-16 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 62251,'1988-Y73','1988-03-18 15:19:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',84 from dual union all
select 62252,'1988-Y74','1988-03-18 16:34:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',80 from dual union all
select 62253,'1988-Y75','1988-03-22 15:41:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',85 from dual union all
select 62254,'1988-Y76','1988-03-23 11:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62255,'1988-Y77','1988-03-23 13:56:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 62256,'1988-Y78','1988-03-23 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 62257,'1988-Y79','1988-03-23 15:51:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',85 from dual union all
select 62258,'1988-Y80','1988-03-23 20:36:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 62259,'1988-Y81','1988-03-30 11:37:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 6226,'1959-S62','1959-05-05 08:55:00','suborbital rocket','success',366,'','RW-126','','','Aeron',1490,'',81 from dual union all
select 62260,'1988-Y82','1988-03-30 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62261,'1988-Y83','1988-03-30 14:17:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 62262,'1988-Y84','1988-03-30 15:25:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',91 from dual union all
select 62263,'1988-Y85','1988-03-30 17:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',89 from dual union all
select 62264,'1988-Y86','1988-03-30 20:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 62265,'1988-Y87','1988-03-31 16:51:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',82 from dual union all
select 62266,'1988-Y88','1988-04-05 16:20:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',80 from dual union all
select 62267,'1988-Y89','1988-04-06 09:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',89 from dual union all
select 62268,'1988-Y90','1988-04-06 11:37:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 62269,'1988-Y91','1988-04-06 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 6227,'1959-S63','1959-05-05 23:20:00','suborbital rocket','success',366,'','RW-127','','','Aeron',1490,'',82 from dual union all
select 62270,'1988-Y92','1988-04-06 15:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 62271,'1988-Y93','1988-04-06 16:02:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',80 from dual union all
select 62272,'1988-Y94','1988-04-06 16:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 62273,'1988-Y95','1988-04-06 19:53:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 62274,'1988-Y96','1988-04-12 16:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1386,'SHIR',81 from dual union all
select 62275,'1988-Y97','1988-04-13 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 62276,'1988-Y98','1988-04-13 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 62277,'1988-Y99','1988-04-20 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 62278,'1989-001','1989-01-10 02:05:25','orbital','success',958,'350-02','Kosmos-1987/1989','Uragan No. 27L','','Sat',553,'',null from dual union all
select 62279,'1989-002','1989-01-12 11:29:59','orbital','success',1238,'Ye15000-580','Kosmos-1990','Resurs-F2 17F42 No. 3','','Sat',462,'',null from dual union all
select 6228,'1959-S64','1959-05-07 01:47:00','military missile','success',638,'AM-12','R&D','','','Test/Ionos',249,'',500 from dual union all
select 62280,'1989-003','1989-01-18 08:20:00','orbital','success',1238,'77059  -388','Kosmos-1991','Oblik','','Sat',480,'',null from dual union all
select 62281,'1989-004','1989-01-26 09:16:00','orbital','success',958,'351-02','Gorizont','Gorizont No. 29L','','Sat',554,'',null from dual union all
select 62282,'1989-005','1989-01-26 15:36:00','orbital','success',680,'65079-812','Kosmos-1992','Strela-2M','','Sat',457,'',null from dual union all
select 62283,'1989-006','1989-01-27 01:21:00','orbital','success',92,'V28 L26','Intelsat 515','INTELSAT VA F15','','Sat',344,'',null from dual union all
select 62284,'1989-007','1989-01-28 12:30:00','orbital','success',1238,'77015  -944','Kosmos-1993','Kobal''t','','Sat',480,'',null from dual union all
select 62285,'1989-008','1989-02-10 08:53:52','orbital','success',1242,'Ye15000-032','Progress-40','Progress 7K-TG No. 148','','Sat',480,'',null from dual union all
select 62286,'1989-009','1989-02-10 15:13:56','orbital','success',1448,'67065-606','Kosmos-1994/1999','Strela-3','','Sat',469,'',null from dual union all
select 62287,'1989-010','1989-02-10 16:55:00','orbital','success',1238,'77059  -974','Kosmos-2000','Oblik','','Sat',473,'',null from dual union all
select 62288,'1989-011','1989-02-14 04:21:11','orbital','success',803,'78039  -557','Kosmos-2001','Oko','','Sat',474,'',null from dual;
insert into LAUNCH
select 62289,'1989-012','1989-02-14 17:00:00','orbital','success',680,'53726-192','Kosmos-2002','Duga-K','','Sat',457,'',null from dual union all
select 6229,'1959-S65','1959-05-08 19:28:00','military missile','success',933,'AX-8','','','','Test',244,'',500 from dual union all
select 62290,'1989-013','1989-02-14 18:30:00','orbital','success',408,'D184','USA 35 (GPS II-1)','Navstar GPS SVN 14','','Sat',233,'',null from dual union all
select 62291,'1989-014','1989-02-15 11:00:00','orbital','success',805,'77011  -691','Molniya-1','Molniya-1 11F658 No. 84','','Sat',480,'',null from dual union all
select 62292,'1989-015','1989-02-17 14:59:59','orbital','success',1238,'78039  -384','Kosmos-2003','Oblik','','Sat',474,'',null from dual union all
select 62293,'1989-016','1989-02-21 23:30:00','orbital','success',820,'M-3S2-4','Akebono/KM-D','EXOS-D','','Sat',837,'',null from dual union all
select 62294,'1989-017','1989-02-22 03:28:49','orbital','success',680,'47111-236','Kosmos-2004','Parus','','Sat',457,'',null from dual union all
select 62295,'1989-018','1989-02-28 04:05:00','orbital','success',1448,'45065-607','Meteor-2','Meteor-2 No. 22','','Sat',470,'',null from dual union all
select 62296,'1989-019','1989-03-02 18:59:59','orbital','success',1238,'78039  -969','Kosmos-2005','Kobal''t','','Sat',474,'',null from dual union all
select 62297,'1989-020','1989-03-06 23:29:00','orbital','success',106,'V29 L403','JCSAT 1/Meteosat 4','JCSAT 1','','Sat',345,'',null from dual union all
select 62298,'1989-021','1989-03-13 14:57:00','orbital','success',1243,'STS-29R','TDRS-D','OV-103 IUS deploy','','Sat',922,'MLP2',null from dual union all
select 62299,'1989-022','1989-03-16 14:59:59','orbital','success',1241,'78039  -382','Kosmos-2006','Oblik','','Sat',462,'',null from dual union all
select 623,'1946-A06','1946-06-01 00:00:00','suborbital rocket','failure',611,'','','','','Test',636,'',0 from dual union all
select 6230,'1959-S66','1959-05-12 00:00:00','military missile','success',976,'I01-08T','8K63 UBP-6','','','Test',636,'',402 from dual union all
select 62300,'1989-023','1989-03-16 18:54:15','orbital','success',1242,'T 15000-034','Progress-41','Progress 7K-TG No. 149','','Sat',480,'',null from dual union all
select 62301,'1989-024','1989-03-23 12:25:00','orbital','success',1238,'77015  -875','Kosmos-2007','Terilen','','Sat',480,'',null from dual union all
select 62302,'1989-025','1989-03-24 13:38:00','orbital','success',680,'65011-232','Kosmos-2008/2015','Strela-1M','','Sat',457,'',null from dual union all
select 62303,'1989-026','1989-03-24 21:50:49','orbital','success',392,'D183','USA 36 (Delta Star)','Delta Star','','Sat',234,'',null from dual union all
select 62304,'1989-027','1989-04-02 02:28:00','orbital','success',92,'V30 L27','Tele-X','Tele-X','','Sat',344,'',null from dual union all
select 62305,'1989-028','1989-04-04 18:36:52','orbital','success',680,'47126-194','Kosmos-2016','Parus','','Sat',457,'',null from dual union all
select 62306,'1989-029','1989-04-06 14:00:00','orbital','success',1241,'78039  -383','Kosmos-2017','Oblik','','Sat',474,'',null from dual union all
select 62307,'1989-030','1989-04-14 04:08:00','orbital','success',958,'359-02','Raduga','Gran'' No. 33L','','Sat',553,'',null from dual union all
select 62308,'1989-031','1989-04-20 18:29:59','orbital','success',1238,'77059  -391','Kosmos-2018','Kobal''t','','Sat',474,'',null from dual union all
select 62309,'1989-032','1989-04-26 17:00:00','orbital','success',1238,'T 15000-593','Foton','Foton No. 5L','','Sat',473,'',null from dual union all
select 6231,'1959-S67','1959-05-12 00:00:00','military missile','success',1053,'CC-1013','','','','OT',1854,'',90 from dual union all
select 62310,'1989-033','1989-05-04 18:47:00','deep space','success',1243,'STS-30R','Magellan','OV-104 IUS deploy','','Sat',922,'MLP1',null from dual union all
select 62311,'1989-034','1989-05-05 13:00:00','orbital','success',1241,'77059  -394','Kosmos-2019','Oblik','','Sat',462,'',null from dual union all
select 62312,'1989-035','1989-05-10 19:47:01','orbital','success',1408,'34D-16 (05D-6)','USA 37       [CY13]','VORTEX 6','','Sat',274,'',null from dual union all
select 62313,'1989-036','1989-05-17 13:00:00','orbital','success',1238,'76035  -876','Kosmos-2020','Kobal''t','','Sat',480,'',null from dual union all
select 62314,'1989-037','1989-05-24 10:30:00','orbital','success',1238,'77059  -471','Kosmos-2021','Kometa No. 11','','Sat',480,'',null from dual union all
select 62315,'1989-038','1989-05-25 08:50:00','orbital','success',1241,'76047  -145','Resurs-F','Resurs-F1 14F43 No. 45','','Sat',474,'',null from dual union all
select 62316,'1989-039','1989-05-31 08:31:59','orbital','success',958,'352-02','Kosmos-2022/2024','Uragan No. 28L','','Sat',554,'',null from dual union all
select 62317,'1989-040','1989-06-01 12:59:59','orbital','success',1238,'77059  -389','Kosmos-2025','Oblik','','Sat',475,'',null from dual union all
select 62318,'1989-041','1989-06-05 22:37:18','orbital','success',103,'V31 L404','Superbird A/DFS 1','Superbird A','','Sat',345,'',null from dual union all
select 62319,'1989-042','1989-06-07 05:12:35','orbital','success',680,'53711-225','Kosmos-2026','Parus','','Sat',457,'',null from dual union all
select 6232,'1959-S68','1959-05-12 17:35:00','military missile','success',1367,'187','R&D Series IV','','','Test',234,'',520 from dual union all
select 62320,'1989-043','1989-06-08 17:09:59','orbital','success',805,'76048  -506','Molniya-3','Molniya-3 No. 45','','Sat',474,'',null from dual union all
select 62321,'1989-044','1989-06-10 22:30:00','orbital','success',408,'D185','USA 38 (GPS II-2)','Navstar GPS SVN 13','','Sat',233,'',null from dual union all
select 62322,'1989-045','1989-06-14 12:30:00','orbital','success',680,'53761-249','Kosmos-2027','Vektor','','Sat',460,'',null from dual union all
select 62323,'1989-046','1989-06-14 13:18:01','orbital','success',1411,'K-1    (45D-1)','USA 39       [DSP14]','DSP 14','','Sat',275,'',null from dual union all
select 62324,'1989-047','1989-06-16 09:30:00','orbital','success',1238,'77059  -398','Kosmos-2028','Oblik','','Sat',480,'',null from dual union all
select 62325,'1989-048','1989-06-21 23:35:00','orbital','success',958,'355-02','Raduga-1','Globus No. 11','','Sat',553,'',null from dual union all
select 62326,'1989-049','1989-06-27 08:04:59','orbital','success',1238,'76047  -146','Resurs-F','Resurs-F1 14F43 No. 46','','Sat',462,'',null from dual union all
select 62327,'1989-050','1989-07-04 15:21:36','orbital','success',680,'65061-250','Nadezhda','Nadehzda 17F118 No. 403','','Sat',460,'',null from dual union all
select 62328,'1989-051','1989-07-05 08:00:00','orbital','success',1241,'77059  -395','Kosmos-2029','Oblik','','Sat',475,'',null from dual union all
select 62329,'1989-052','1989-07-05 22:45:00','orbital','success',958,'340-02','Gorizont','Gorizont No. 27L','','Sat',554,'',null from dual union all
select 6233,'1959-S69','1959-05-12 11:39:00','suborbital rocket','success',30,'AA3.200C','','','','Aeron',640,'',145 from dual union all
select 62330,'1989-053','1989-07-12 00:14:00','orbital','success',93,'V32 L28','Olympus','Olympus','','Sat',344,'',null from dual union all
select 62331,'1989-054','1989-07-12 15:00:00','orbital','success',1238,'78039  -970','Kosmos-2030','Kobal''t','','Sat',473,'',null from dual union all
select 62332,'1989-055','1989-07-18 09:44:59','orbital','success',1238,'76047  -147','Resurs-F','Resurs-F1 14F43 No. 47','','Sat',462,'',null from dual union all
select 62333,'1989-056','1989-07-18 12:10:00','orbital','success',1238,'T 15000-594','Kosmos-2031','Orlets','','Sat',480,'',null from dual union all
select 62334,'1989-057','1989-07-20 08:59:59','orbital','success',1238,'77059  -393','Kosmos-2032','Oblik','','Sat',474,'',null from dual union all
select 62335,'1989-058','1989-07-24 00:00:59','orbital','success',1446,'67032-306','Kosmos-2033','US-PM','','Sat',589,'',null from dual union all
select 62336,'1989-059','1989-07-25 07:48:44','orbital','success',680,'65061-256','Kosmos-2034','Parus','','Sat',460,'',null from dual union all
select 62337,'1989-060','1989-08-02 11:29:59','orbital','success',1238,'77059  -396','Kosmos-2035','Oblik','','Sat',462,'',null from dual union all
select 62338,'1989-061','1989-08-08 12:37:00','orbital','success',1243,'STS-28R','USA 40/41 (QUASAR 8)','OV-102 Medium deploy','','Sat',922,'MLP2',null from dual union all
select 62339,'1989-062','1989-08-08 23:25:53','orbital','success',106,'V33 L405','TV-SAT 2/Hipparcos','TV-SAT 2','','Sat',345,'',null from dual union all
select 6234,'1959-S70','1959-05-14 00:00:00','military missile','success',976,'I2-20T','8K63 UBP-7','','','Test',636,'',402 from dual union all
select 62340,'1989-063','1989-08-15 10:29:59','orbital','success',1238,'77017  -254','Resurs-F','Resurs-F2 17F42 No. 4','','Sat',475,'',null from dual union all
select 62341,'1989-064','1989-08-18 05:57:59','orbital','success',408,'D186','USA 42 (GPS II-3)','Navstar GPS SVN 16','','Sat',233,'',null from dual union all
select 62342,'1989-065','1989-08-22 12:59:59','orbital','success',1238,'76047  -809','Kosmos-2036','Oblik','','Sat',473,'',null from dual union all
select 62343,'1989-066','1989-08-23 03:09:32','orbital','success',1242,'T 15000-037','Progress M','Progress 7K-TGM No. 201','','Sat',480,'',null from dual union all
select 62344,'1989-067','1989-08-27 22:59:00','orbital','success',395,'D187','Marcopolo 1','BSB R1','','Sat',234,'',null from dual union all
select 62345,'1989-068','1989-08-28 00:14:00','orbital','success',1448,'51065-608','Kosmos-2037','Musson No. 20','','Sat',470,'',null from dual union all
select 62346,'1989-069','1989-09-04 05:54:01','orbital','success',1408,'34D-2  (05D-7)','USA 43/44 (DSCS)','DSCS II E-15','','Sat',274,'',null from dual union all
select 62347,'1989-070','1989-09-05 19:11:00','orbital','success',523,'H-20(F)','GMS 4','GMS 4','','Sat',1474,'',null from dual union all
select 62348,'1989-071','1989-09-05 21:38:03','orbital','success',1242,'T 15000-036','Soyuz TM-8','Soyuz 7K-STM No. 58','','Sat',480,'',null from dual union all
select 62349,'1989-072','1989-09-06 01:48:00','orbital','success',1422,'23G-2','USA 45       [T2-2]','P-11 No. 5104','','Sat',1778,'',null from dual union all
select 6235,'1959-S71','1959-05-14 05:52:00','military missile','success',638,'AM-17','R&D','','','Test/Ionos',279,'',500 from dual union all
select 62350,'1989-073','1989-09-06 10:49:59','orbital','success',1241,'76047  -810','Resurs-F','Resurs-F1 14F43 No. 48','','Sat',474,'',null from dual union all
select 62351,'1989-074','1989-09-14 09:49:06','orbital','success',1448,'51079-202','Kosmos-2038/2043','Strela-3','','Sat',469,'',null from dual union all
select 62352,'1989-075','1989-09-15 06:30:02','orbital','success',1238,'T 15000-599','Kosmos-2044','Bion No. 9','','Sat',473,'',null from dual union all
select 62353,'1989-076','1989-09-22 08:00:00','orbital','success',1238,'77059  -392','Kosmos-2045','Oblik','','Sat',480,'',null from dual union all
select 62354,'1989-077','1989-09-25 08:56:02','orbital','success',169,'AC-68','USA 46 (FLTSATCOM 8)','FLTSATCOM F8','','Sat',269,'',null from dual union all
select 62355,'1989-078','1989-09-27 14:38:00','orbital','success',805,'78039  -556','Molniya-1','Molniya-1 No. 69','','Sat',475,'',null from dual union all
select 62356,'1989-079','1989-09-27 16:20:00','orbital','success',1446,'45032-307','Kosmos-2046','US-PU','','Sat',589,'',null from dual union all
select 62357,'1989-080','1989-09-28 00:04:59','orbital','success',1448,'67057-904','Interkosmos-24','AUOS-Z No. 201','','Sat',470,'',null from dual union all
select 62358,'1989-081','1989-09-28 17:05:00','orbital','success',958,'346-01','Gorizont','Gorizont No. 31L','','Sat',554,'',null from dual union all
select 62359,'1989-082','1989-10-03 14:59:59','orbital','success',1238,'78039  -386','Kosmos-2047','Kobal''t','','Sat',474,'',null from dual union all
select 6236,'1959-S72','1959-05-15 00:00:00','military missile','success',976,'I2-2','8K63 UBP-8','','','Test',636,'',402 from dual union all
select 62360,'1989-083','1989-10-17 13:00:00','orbital','success',1238,'77059  -397','Kosmos-2048','Oblik','','Sat',475,'',null from dual union all
select 62361,'1989-084','1989-10-18 16:53:40','deep space','success',1243,'STS-34R','Galileo','OV-104 IUS deploy','','Sat',922,'MLP1',null from dual union all
select 62362,'1989-085','1989-10-21 09:31:01','orbital','success',408,'D188','USA 47 (GPS II-4)','Navstar GPS SVN 19','','Sat',233,'',null from dual union all
select 62363,'1989-086','1989-10-24 21:35:00','orbital','success',1448,'67057-901','Meteor-3','Meteor-3 No. 4','','Sat',469,'',null from dual union all
select 62364,'1989-087','1989-10-27 23:05:00','orbital','success',103,'V34 L406','Intelsat 602','INTELSAT VI F-2','','Sat',345,'',null from dual union all
select 62365,'1989-088','1989-11-17 10:50:00','orbital','success',1238,'78039  -725','Kosmos-2049','Terilen','','Sat',480,'',null from dual union all
select 62366,'1989-089','1989-11-18 14:34:00','orbital','success',405,'D189','COBE','COBE','','Sat',1724,'',null from dual union all
select 62367,'1989-090','1989-11-23 00:23:30','orbital','success',1243,'STS-33R','USA 48    (ORION 2)','OV-103 IUS deploy','','Sat',922,'MLP2',null from dual union all
select 62368,'1989-091','1989-11-23 20:35:44','orbital','success',803,'77059  -563','Kosmos-2050','Oko','','Sat',462,'',null from dual union all
select 62369,'1989-092','1989-11-24 23:22:00','orbital','failure',1446,'51086-102','Kosmos-2051','US-PM','','Sat',589,'',null from dual union all
select 6237,'1959-S73','1959-05-21 06:40:00','military missile','success',1355,'135','RVX-1','','','Reentry Test',233,'',1200 from dual union all
select 62370,'1989-093','1989-11-26 13:01:41','orbital','success',949,'354-01','Kvant-2','TsM-D','','Sat',553,'',null from dual union all
select 62371,'1989-094','1989-11-28 10:02:00','orbital','success',805,'78039  -555','Molniya-3','Molniya-3 No. 46','','Sat',474,'',null from dual union all
select 62372,'1989-095','1989-11-30 15:00:01','orbital','success',1238,'78039  -971','Kosmos-2052','Kobal''t','','Sat',462,'',null from dual union all
select 62373,'1989-096','1989-12-01 20:20:57','orbital','success',954,'352-01','Granat','1AS No. 1','','Sat',554,'',null from dual union all
select 62374,'1989-097','1989-12-11 18:10:01','orbital','success',408,'D190','USA 49 (GPS II-5)','Navstar GPS SVN 17','','Sat',234,'',null from dual union all
select 62375,'1989-098','1989-12-15 11:30:00','orbital','success',958,'344-01','Raduga','Gran'' No. 36L','','Sat',587,'',null from dual union all
select 62376,'1989-099','1989-12-20 03:30:50','orbital','success',1242,'T 15000-039','Progress M-2','Progress 7K-TGM No. 202','','Sat',480,'',null from dual union all
select 62377,'1989-100','1989-12-27 00:00:00','orbital','success',1448,'45076-107','Kosmos-2053','Kol''tso','','Sat',470,'',null from dual union all
select 62378,'1989-101','1989-12-27 11:10:00','orbital','success',958,'347-02','Kosmos-2054','Al''tair No. 14L','','Sat',553,'',null from dual;
insert into LAUNCH
select 62379,'1989-A01','1989-01-09 08:10:00','suborbital rocket','failure',224,'NASA 36.45UG','JHU FOT 8 (Io)','','','UV Astron',1873,'',10 from dual union all
select 6238,'1959-S74','1959-05-23 02:42:00','military missile','success',1367,'184','R&D Series IV','','','Test',236,'',520 from dual union all
select 62380,'1989-A02','1989-03-21 16:20:00','military missile','failure',1443,'','PEM-1','','','OT',419,'SSBN 734',0 from dual union all
select 62381,'1989-A03','1989-04-19 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',25 from dual union all
select 62382,'1989-A04','1989-05-15 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',49 from dual union all
select 62383,'1989-A05','1989-08-10 14:27:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',0 from dual union all
select 62384,'1989-A06','1989-08-15 21:10:00','military missile','failure',1443,'','PEM-4','','','OT',421,'SSBN 734',0 from dual union all
select 62385,'1989-A07','1989-09-06 02:23:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',19 from dual union all
select 62386,'1989-A08','1989-10-05 17:28:00','suborbital rocket','failure',16,'SET-1','','','','Test',1688,'',0 from dual union all
select 62387,'1989-A09','1989-10-23 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',49 from dual union all
select 62388,'1989-A10','1989-11-01 13:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',35 from dual union all
select 62389,'1989-A11','1989-11-15 16:45:00','suborbital rocket','failure',213,'Starfire 1/F2','CONSORT 2','','','Micrograv',1879,'',10 from dual union all
select 6239,'1959-S75','1959-05-22 11:15:00','suborbital rocket','success',30,'AA3.201C','','','','Aeron',640,'',100 from dual union all
select 62390,'1989-A12','1989-12-04 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',38 from dual union all
select 62391,'1989-A13','1989-12-08 23:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',35 from dual union all
select 62392,'1989-A14','1989-12-21 20:06:00','suborbital rocket','success',1312,'NASA 12.43WT','','','','Test',1821,'',10 from dual union all
select 62393,'1989-A15','1989-03-16 00:00:00','military missile','success',684,'1      -','15Zh59 B-1','Mobile launch test','','Test',477,'',1 from dual union all
select 62394,'1989-A16','1989-08-19 00:00:00','military missile','success',684,'2      -','15Zh59 B-2','Mobile launch test','','Test',477,'',1 from dual union all
select 62395,'1989-A17','1989-06-01 00:00:00','military missile','success',702,'Test firing','','','','Test',984,'',10 from dual union all
select 62396,'1989-F01','1989-06-09 10:10:00','orbital','failure',1448,'45019-210','Okean','Okean-O1 No. 4 (NKhM No.','','Sat',470,'',180 from dual union all
select 62397,'1989-M01','1989-01-26 23:00:00','suborbital rocket','success',1295,'TR-1-2','H-II 1/4 Scale','','','Test',1476,'',80 from dual union all
select 62398,'1989-M02','1989-05-11 18:00:00','military missile','failure',1082,'FTM-1','','','','Test',1726,'',80 from dual union all
select 62399,'1989-M03','1989-05-18 00:00:00','suborbital rocket','success',1477,'VLS-R2','','','','Test',330,'',40 from dual union all
select 624,'1946-A07','1946-06-01 00:00:00','suborbital rocket','success',611,'','','','','Test',636,'',35 from dual union all
select 6240,'1959-S76','1959-05-27 04:51:00','suborbital rocket','success',839,'AA6.161C','','','','Aeronomy',1895,'',169 from dual union all
select 62400,'1989-M04','1989-08-19 23:00:00','suborbital rocket','success',1295,'TR-1-3','H-II 1/4 Scale','','','Test',1476,'',85 from dual union all
select 62401,'1989-M05','1989-12-05 00:00:00','military missile','failure',70,'','','','','Test',22,'',25 from dual union all
select 62402,'1989-M06','1989-07-20 19:39:00','suborbital rocket','success',1284,'','ROCOZ-A 356 STOIC','','','Aeron',1415,'',75 from dual union all
select 62403,'1989-M07','1989-07-24 19:22:00','suborbital rocket','success',1284,'','ROCOZ-A 357 STOIC','','','Aeron',1415,'',75 from dual union all
select 62404,'1989-M08','1989-07-26 20:53:00','suborbital rocket','success',1284,'','ROCOZ-A 358 STOIC','','','Aeron',1415,'',75 from dual union all
select 62405,'1989-M09','1989-07-27 18:49:00','suborbital rocket','success',1284,'','ROCOZ-A 359 STOIC','','','Aeron',1415,'',75 from dual union all
select 62406,'1989-M10','1989-07-28 18:27:00','suborbital rocket','success',1284,'','ROCOZ-A 360 STOIC','','','Aeron',1415,'',75 from dual union all
select 62407,'1989-M11','1989-08-02 17:38:00','suborbital rocket','success',1284,'','ROCOZ-A 361 STOIC','','','Aeron',1415,'',75 from dual union all
select 62408,'1989-M12','1989-02-01 00:00:00','military missile','success',568,'','','','','Test',1419,'',70 from dual union all
select 62409,'1989-M13','1989-01-04 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6241,'1959-S77','1959-05-27 19:55:00','suborbital rocket','success',839,'AA6.162C','','','','Aeronomy',1895,'',177 from dual union all
select 62410,'1989-M14','1989-01-18 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62411,'1989-M15','1989-01-25 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62412,'1989-M16','1989-02-01 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62413,'1989-M17','1989-02-08 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62414,'1989-M18','1989-02-15 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62415,'1989-M19','1989-02-22 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62416,'1989-M20','1989-03-01 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62417,'1989-M21','1989-03-08 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62418,'1989-M22','1989-03-15 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62419,'1989-M23','1989-03-22 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6242,'1959-S78','1959-05-28 07:35:02','ballistic missile test','success',638,'AM-18','Bioflight 2','','','Test/Ionos',249,'',483 from dual union all
select 62420,'1989-M24','1989-03-29 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62421,'1989-M25','1989-04-05 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62422,'1989-M26','1989-04-13 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62423,'1989-M27','1989-04-19 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62424,'1989-M28','1989-04-26 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62425,'1989-M29','1989-05-03 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62426,'1989-M30','1989-05-17 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62427,'1989-M31','1989-05-25 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62428,'1989-M32','1989-05-31 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62429,'1989-M33','1989-06-07 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6243,'1959-S79','1959-05-30 00:00:00','military missile','success',976,'I3-20T','8K63 UBP-9','','','Test',636,'',402 from dual union all
select 62430,'1989-M34','1989-06-14 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62431,'1989-M35','1989-06-21 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62432,'1989-M36','1989-06-28 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62433,'1989-M37','1989-07-05 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62434,'1989-M38','1989-07-19 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62435,'1989-M39','1989-08-02 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62436,'1989-M40','1989-08-16 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62437,'1989-M41','1989-08-23 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62438,'1989-M42','1989-08-30 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62439,'1989-M43','1989-09-06 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6244,'1959-S80','1959-06-01 00:00:00','military missile','success',978,'','','','','Test',636,'',150 from dual union all
select 62440,'1989-M44','1989-09-13 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62441,'1989-M45','1989-09-20 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62442,'1989-M46','1989-09-27 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62443,'1989-M47','1989-10-04 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62444,'1989-M48','1989-10-11 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62445,'1989-M49','1989-10-18 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62446,'1989-M50','1989-10-25 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62447,'1989-M51','1989-11-01 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62448,'1989-M52','1989-11-15 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62449,'1989-M53','1989-11-23 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6245,'1959-S81','1959-06-02 00:00:00','military missile','success',976,'I01-20T','8K63 UBP-10','','','Test',636,'',402 from dual union all
select 62450,'1989-M54','1989-11-29 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62451,'1989-M55','1989-12-07 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62452,'1989-M56','1989-12-14 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62453,'1989-M57','1989-12-21 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62454,'1989-M58','1989-12-27 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 62455,'1989-S01','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62456,'1989-S02','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62457,'1989-S03','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62458,'1989-S04','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62459,'1989-S05','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 6246,'1959-S82','1959-06-05 04:15:00','suborbital rocket','success',1438,'TB I beta','','','','Reentry Test',1821,'',260 from dual union all
select 62460,'1989-S06','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62461,'1989-S07','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62462,'1989-S08','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62463,'1989-S09','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62464,'1989-S10','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62465,'1989-S100','1989-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62466,'1989-S101','1989-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62467,'1989-S102','1989-10-23 01:58:00','suborbital rocket','success',224,'NASA 36.37DE','NICARE 1','','','Plasma',1821,'',310 from dual union all
select 62468,'1989-S103','1989-11-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual;
insert into LAUNCH
select 62469,'1989-S104','1989-11-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 6247,'1959-S83','1959-06-08 00:00:00','test rocket','success',253,'','','','','Interceptor',21,'B-47',100 from dual union all
select 62470,'1989-S105','1989-11-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62471,'1989-S106','1989-11-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62472,'1989-S107','1989-11-07 16:17:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT136GM','GLORY TRIP 136GM','OT',1706,'',1300 from dual union all
select 62473,'1989-S108','1989-11-20 02:30:00','suborbital rocket','success',210,'NASA 27.123UG','FUVS 5','','','UV Astron',1895,'',340 from dual union all
select 62474,'1989-S109','1989-11-25 08:05:00','suborbital rocket','success',1212,'DLR K-GR-170','DLR TEXUS 23','','','Micrograv',416,'',249 from dual union all
select 62475,'1989-S11','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62476,'1989-S110','1989-12-04 15:40:03','military missile','unknown',1443,'','PEM-5','','','OT',419,'SSBN 734',1000 from dual union all
select 62477,'1989-S111','1989-12-05 19:21:00','suborbital rocket','success',237,'NASA 12.42WT','','','','Test/Aeron',1821,'',1459 from dual union all
select 62478,'1989-S112','1989-12-06 08:11:08','suborbital rocket','success',1212,'DLR K-GR-171','DLR TEXUS 24','','','Micrograv',416,'',244 from dual union all
select 62479,'1989-S113','1989-12-13 14:15:03','military missile','success',1443,'','PEM-11','','','OT',419,'SSBN 734',1000 from dual union all
select 6248,'1959-S84','1959-06-10 16:40:00','suborbital rocket','success',30,'AA3.280C','','','','Ionos',640,'',225 from dual union all
select 62480,'1989-S114','1989-12-15 14:15:00','military missile','success',1443,'','PEM-3','','','OT',421,'SSBN 734',1000 from dual union all
select 62481,'1989-S115','1989-01-01 00:00:00','suborbital rocket','success',1279,'','THORNY MERIT','','','Reentry Test?',843,'',200 from dual union all
select 62482,'1989-S116','1989-07-01 00:00:00','suborbital rocket','success',117,'Queen Match','','','','ABM Tech IR',1362,'',300 from dual union all
select 62483,'1989-S117','1989-03-21 00:00:00','military missile','success',1106,'','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 62484,'1989-S118','1989-02-12 00:00:00','military missile','success',69,'','Test flight','','','Test',757,'',100 from dual union all
select 62485,'1989-S119','1989-02-12 00:00:00','military missile','success',69,'','Test flight','','','Test',757,'',100 from dual union all
select 62486,'1989-S12','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62487,'1989-S120','1989-08-21 00:00:00','military missile','success',69,'','Test flight','','','Test',757,'',100 from dual union all
select 62488,'1989-S121','1989-07-27 00:00:00','military missile','success',73,'','Launch trailer test','','','Test',761,'',100 from dual union all
select 62489,'1989-S122','1989-02-01 00:00:00','military missile','success',569,'','','','','Test',1419,'',100 from dual union all
select 6249,'1959-S85','1959-06-11 06:44:00','military missile','success',1355,'137','RVX-1','','','Reentry Test',233,'',1200 from dual union all
select 62490,'1989-S123','1989-09-01 00:00:00','military missile','success',984,'Aerofon','UGCh','LKI III-22','','Test',636,'',86 from dual union all
select 62491,'1989-S124','1989-02-07 00:00:00','military missile','success',1434,'','15Zh58 UBP-5','','','Test',477,'',1000 from dual union all
select 62492,'1989-S125','1989-03-21 00:00:00','military missile','success',1434,'','15Zh58 UBP-6','','','Test',477,'',1000 from dual union all
select 62493,'1989-S126','1989-06-15 00:00:00','military missile','success',1434,'','15Zh58 K-21','','','Test',477,'',1000 from dual union all
select 62494,'1989-S127','1989-09-20 00:00:00','military missile','success',1434,'','15Zh58 K-22','','','Test',477,'',1000 from dual union all
select 62495,'1989-S13','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62496,'1989-S130','1989-10-26 00:00:00','military missile','success',1434,'','15Zh58 K-23','','','Test',477,'',1000 from dual union all
select 62497,'1989-S131','1989-09-20 00:00:00','military missile','success',1042,'48737-415-03','15Zh61 K-1','','','Test',477,'',1000 from dual union all
select 62498,'1989-S132','1989-02-22 00:00:00','military missile','success',1450,'','15A20 52 RD UBP','','','OT',195,'',1000 from dual union all
select 62499,'1989-S133','1989-09-15 00:00:00','military missile','success',1450,'','15A20 52 RD UBP','','','OT',195,'',1000 from dual union all
select 625,'1946-A08','1946-07-01 00:00:00','atmospheric rocket','success',471,'','RM-5 Model','','','Test',1821,'',5 from dual union all
select 6250,'1959-S86','1959-06-11 13:03:00','military missile','success',239,'BK.04','H03','','','Test',1843,'',803 from dual union all
select 62500,'1989-S134','1989-01-01 00:00:00','military missile','success',436,'','','','','Test',1494,'',1000 from dual union all
select 62501,'1989-S135','1989-11-01 00:00:00','military missile','success',1041,'48737-501-8L','15Zh60 K-1','','','Test',477,'',1000 from dual union all
select 62502,'1989-S136','1989-10-28 00:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',100 from dual union all
select 62503,'1989-S14','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62504,'1989-S15','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62505,'1989-S16','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62506,'1989-S17','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62507,'1989-S18','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62508,'1989-S19','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 62509,'1989-S20','1989-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 6251,'1959-S87','1959-06-05 00:00:00','military missile','success',976,'I2-3','8K63 UBP-11','','','Test',636,'',402 from dual union all
select 62510,'1989-S21','1989-01-18 04:15:00','military missile','success',1005,'45019-201','15A18M LKI-20','GCh 15F175','','Test',497,'',1000 from dual union all
select 62511,'1989-S22','1989-04-11 04:26:00','military missile','success',1005,'67019-206','15A18M LKI-21','GCh 15F175','','Test',487,'',1000 from dual union all
select 62512,'1989-S23','1989-08-01 13:40:00','military missile','success',1005,'67057-319-06T','15A18M K-2','GCh 15F173','','Test',487,'',1000 from dual union all
select 62513,'1989-S24','1989-08-11 21:59:00','military missile','success',1005,'51057-902(23L)','15A18M LKI-22','GCh 15F175','','Test',497,'',1000 from dual union all
select 62514,'1989-S25','1989-09-02 05:30:00','military missile','success',1005,'51019-202','15A18M LKI-23','GCh 15F175','','OT',491,'',1000 from dual union all
select 62515,'1989-S26','1989-12-26 04:40:00','military missile','success',1005,'67057-319-10T','15A18M K-3 ZP','GCh 15F173','','OT',512,'',1000 from dual union all
select 62516,'1989-S27','1989-01-09 10:52:00','military missile','success',1443,'D5X-19','R&D','','','Test',294,'',1000 from dual union all
select 62517,'1989-S28','1989-01-17 19:00:03','suborbital rocket','success',224,'NASA 36.47GU','','','','Aeron?',1873,'',300 from dual union all
select 62518,'1989-S29','1989-01-23 10:40:05','military missile','unknown',1442,'','DASO-31','','','OT',421,'SSBN 640',1000 from dual union all
select 62519,'1989-S30','1989-01-25 13:57:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT131GB','GLORY TRIP 131GB','OT',1712,'',1300 from dual union all
select 6252,'1959-S88','1959-06-17 00:00:00','military missile','success',976,'I3-10T','8K63 UBP-12','','','Test',636,'',402 from dual union all
select 62520,'1989-S31','1989-01-26 09:07:00','military missile','success',1443,'D5X-20','R&D','','','Test',294,'',1000 from dual union all
select 62521,'1989-S32','1989-01-30 08:30:00','suborbital rocket','success',891,'DLR K-O-166','Strafam 2/2','','','Aeron',417,'',66 from dual union all
select 62522,'1989-S33','1989-01-30 21:14:40','suborbital rocket','success',232,'AAF-XB-02','OEDIPUS A','','','Plasma/tether',29,'',512 from dual union all
select 62523,'1989-S34','1989-02-01 12:00:00','suborbital rocket','success',1073,'S-310-19','','','','Aeron',833,'',230 from dual union all
select 62524,'1989-S35','1989-02-04 03:03:04','military missile','success',1442,'','DASO-32/JANUS','','','Tech',421,'SSBN 643',1000 from dual union all
select 62525,'1989-S36','1989-02-07 23:36:30','suborbital rocket','success',1204,'DLR K-CR-164','DLR ROSE III','','','Aeron',416,'',124 from dual union all
select 62526,'1989-S37','1989-02-09 23:42:00','suborbital rocket','success',1204,'DLR K-CR-165','DLR ROSE IV','','','Aeron',416,'',125 from dual union all
select 62527,'1989-S38','1989-02-22 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT132GM-1','GLORY TRIP 132GM-1','OT',1699,'',1300 from dual union all
select 62528,'1989-S39','1989-03-03 18:17:14','suborbital rocket','success',227,'NASA 21.100GE','ERRRIS II','','','Aeron',411,'',124 from dual union all
select 62529,'1989-S40','1989-03-04 01:02:47','suborbital rocket','success',228,'NASA 21.96GE','ERRRIS II','','','Auroral',411,'',175 from dual union all
select 6253,'1959-S89','1959-06-17 17:00:00','suborbital rocket','success',366,'','RW-128','','','Aeron',1490,'',81 from dual union all
select 62530,'1989-S41','1989-03-07 08:26:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT133GM','GLORY TRIP 133GM','OT',1706,'',1300 from dual union all
select 62531,'1989-S42','1989-03-07 22:00:00','suborbital rocket','success',1319,'NASA 33.57UL','','','','Aeron',1047,'',200 from dual union all
select 62532,'1989-S43','1989-03-08 00:00:00','military missile','success',567,'','','','','Test',307,'',150 from dual union all
select 62533,'1989-S44','1989-03-09 21:33:02','military missile','success',939,'','DASO','','','OT',421,'SSBN 645',500 from dual union all
select 62534,'1989-S45','1989-03-18 16:00:00','suborbital rocket','success',860,'NASA 31.73UU','CWAS 17','','','Aeronomy',428,'',140 from dual union all
select 62535,'1989-S46','1989-03-19 17:45:00','military missile','success',907,'FTM-18','Mk 21 RV x 7','','','Test',1703,'',1000 from dual union all
select 62536,'1989-S47','1989-03-22 16:01:31','suborbital rocket','success',860,'NASA 31.74UU','CWAS 18','','','Aeronomy',428,'',140 from dual union all
select 62537,'1989-S48','1989-03-28 00:00:00','suborbital rocket','success',1220,'PT-04','','','','Test',330,'',821 from dual union all
select 62538,'1989-S49','1989-03-29 15:41:00','suborbital rocket','success',213,'Starfire 1/F1','CONSORT 1','','','Micrograv',1879,'',308 from dual union all
select 62539,'1989-S50','1989-04-03 02:27:00','suborbital rocket','success',1337,'NASA 29.27UE','RED AIR 1','','','Plasma',1821,'',380 from dual union all
select 6254,'1959-S90','1959-06-18 01:00:00','suborbital rocket','success',366,'','RW-129','','','Aeron',1490,'',81 from dual union all
select 62540,'1989-S51','1989-04-09 04:37:39','suborbital rocket','success',232,'NASA 35.022GE','Ba+','','','Plasma',428,'',700 from dual union all
select 62541,'1989-S52','1989-04-10 08:21:00','suborbital rocket','success',224,'MASER 3','SSC MASER 3','','','Micrograv',412,'',297 from dual union all
select 62542,'1989-S53','1989-04-11 03:36:00','suborbital rocket','success',232,'NASA 35.021GE','Ba+','','','Plasma',428,'',900 from dual union all
select 62543,'1989-S54','1989-04-30 03:48:00','suborbital rocket','success',1212,'DLR K-GR-167','DLR TEXUS 21','','','Micrograv',416,'',268 from dual union all
select 62544,'1989-S55','1989-05-03 03:30:00','suborbital rocket','success',1212,'DLR K-GR-168','DLR TEXUS 22','','','Micrograv',416,'',223 from dual union all
select 62545,'1989-S56','1989-05-04 01:42:10','suborbital rocket','success',231,'NASA 35.019UE','CVE-2 (CRIT-2)','','','Plasma/Ba',1821,'',430 from dual union all
select 62546,'1989-S57','1989-05-05 17:45:00','suborbital rocket','success',224,'NASA 36.25GS','SERTS-89 (SERTS-3)','','','Solar',1873,'',320 from dual union all
select 62547,'1989-S58','1989-05-22 01:40:00','military missile','success',68,'01','','','','Test',320,'',300 from dual union all
select 62548,'1989-S59','1989-05-31 00:00:00','suborbital rocket','success',228,'','(Site CE)','','','Test?',304,'',250 from dual union all
select 62549,'1989-S61','1989-06-18 00:00:00','military missile','success',1106,'2609','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 6255,'1959-S91','1959-06-18 13:00:00','suborbital rocket','success',366,'','RW-131','','','Aeron',1490,'',82 from dual union all
select 62550,'1989-S62','1989-06-20 19:00:00','suborbital rocket','success',224,'NASA 36.54US','EUV-ML 2/NCAR/CU-2','','','Solar UV/EUV',1873,'',300 from dual union all
select 62551,'1989-S63','1989-06-27 06:00:01','suborbital rocket','success',224,'NASA 36.43GG','MAMA UV Telescope','','','UV Astron',1873,'',300 from dual union all
select 62552,'1989-S64','1989-07-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62553,'1989-S65','1989-07-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62554,'1989-S66','1989-07-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62555,'1989-S67','1989-07-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62556,'1989-S68','1989-07-05 00:00:00','military missile','success',1034,'2','','','','Test',1014,'',300 from dual union all
select 62557,'1989-S69','1989-07-06 13:16:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT129GB-2','GLORY TRIP 129GB-2','OT',1712,'',1300 from dual union all
select 62558,'1989-S70','1989-07-11 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT134GM','GLORY TRIP 134GM','OT',1705,'',1300 from dual;
insert into LAUNCH
select 62559,'1989-S71','1989-07-13 08:30:00','suborbital rocket','success',117,'GL A24.602','BEAR','','','Particle Beam',1873,'',195 from dual union all
select 6256,'1959-S92','1959-06-19 14:30:00','test rocket','success',253,'','','','','Interceptor',293,'B-47',100 from dual union all
select 62560,'1989-S72','1989-07-17 19:22:26','suborbital rocket','success',211,'NASA 27.122UE','','','','Plasma',1895,'',340 from dual union all
select 62561,'1989-S73','1989-07-24 08:39:59','suborbital rocket','success',211,'SDIO Black Star 1','SP-4 DRO-0401','CHEER','','Target?',1047,'',340 from dual union all
select 62562,'1989-S74','1989-07-31 00:00:00','suborbital rocket','success',228,'SDIO Black Star 5','','','','Target',1821,'',250 from dual union all
select 62563,'1989-S75','1989-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62564,'1989-S76','1989-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62565,'1989-S77','1989-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62566,'1989-S78','1989-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62567,'1989-S79','1989-07-31 00:00:00','suborbital rocket','success',231,'SDIO Black Star 3','','','','Target',1821,'',900 from dual union all
select 62568,'1989-S80','1989-08-23 10:42:00','suborbital rocket','success',231,'SDIO Black Star 4','','','','Target',1821,'',900 from dual union all
select 62569,'1989-S81','1989-08-02 15:00:03','military missile','success',1443,'','PEM-2','','','OT',421,'SSBN 734',1000 from dual union all
select 6257,'1959-S93','1959-06-21 00:00:00','suborbital rocket','success',1011,'','R-5A Geophysical','','','Solar X',636,'',400 from dual union all
select 62570,'1989-S82','1989-08-13 09:00:00','suborbital rocket','success',211,'SDIO Black Star 2','SP-4 DRO-0402','CHEER','','Target?',1047,'',340 from dual union all
select 62571,'1989-S83','1989-08-23 18:05:00','suborbital rocket','success',224,'NASA 36.59UL','EUV Spectrometer','','','Aeron/Solar EUV',1873,'',333 from dual union all
select 62572,'1989-S84','1989-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62573,'1989-S85','1989-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62574,'1989-S86','1989-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62575,'1989-S87','1989-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62576,'1989-S88','1989-09-04 00:00:00','suborbital rocket','success',1337,'SDIO Starmate','','','','Target',848,'',724 from dual union all
select 62577,'1989-S89','1989-09-04 16:15:00','suborbital rocket','success',1074,'S-520-10','CIR  Cosmic IR','','','IR Astron',833,'',288 from dual union all
select 62578,'1989-S90','1989-09-11 00:00:00','suborbital rocket','success',1337,'SDIO Starmate','','','','Target',848,'',724 from dual union all
select 62579,'1989-S91','1989-09-11 16:36:00','suborbital rocket','success',224,'NASA 36.52US','Corona Tel 1','','','Solar EUV/SX',1873,'',300 from dual union all
select 6258,'1959-S94','1959-06-22 14:32:00','suborbital rocket','success',28,'AA1.260C','','','','Solar IR',640,'',110 from dual union all
select 62580,'1989-S92','1989-09-14 00:00:00','military missile','success',627,'','','','','Test',1026,'',300 from dual union all
select 62581,'1989-S93','1989-09-14 19:00:00','military missile','failure',907,'01PA 75/46/58/76','Mk 21 RV x 9','GT 01PA','','OT',1697,'',200 from dual union all
select 62582,'1989-S94','1989-09-15 00:00:00','suborbital rocket','success',228,'SDIO Black Star 6','','','','Target',1821,'',250 from dual union all
select 62583,'1989-S95','1989-09-26 13:14:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT135GB','GLORY TRIP 135GB','OT',1712,'',1300 from dual union all
select 62584,'1989-S96','1989-09-26 05:10:00','military missile','success',941,'2','SS-520','','','Test',1361,'',100 from dual union all
select 62585,'1989-S97','1989-09-29 18:04:02','military missile','success',939,'','DASO','','','OT',421,'SSBN 617',500 from dual union all
select 62586,'1989-S98','1989-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62587,'1989-S99','1989-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',145,'ZUB',250 from dual union all
select 62588,'1989-W01','1989-01-01 17:11:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',60 from dual union all
select 62589,'1989-W02','1989-01-02 09:24:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',60 from dual union all
select 6259,'1959-S95','1959-06-24 12:10:00','suborbital rocket','success',30,'AA3.220C','','','','Ionos',640,'',200 from dual union all
select 62590,'1989-W03','1989-01-03 11:14:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',55 from dual union all
select 62591,'1989-W04','1989-01-04 08:10:00','suborbital rocket','unknown',738,'','','','','Aeron',216,'BUGAEV',60 from dual union all
select 62592,'1989-W05','1989-01-04 10:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',62 from dual union all
select 62593,'1989-W06','1989-01-04 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 62594,'1989-W07','1989-01-04 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',79 from dual union all
select 62595,'1989-W08','1989-01-04 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 62596,'1989-W09','1989-01-05 10:46:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',58 from dual union all
select 62597,'1989-W10','1989-01-06 07:04:00','suborbital rocket','unknown',738,'','','','','Aeron',216,'BUGAEV',56 from dual union all
select 62598,'1989-W100','1989-03-22 15:22:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',65 from dual union all
select 62599,'1989-W101','1989-03-23 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 626,'1946-A09','1946-07-19 19:11:00','suborbital rocket','failure',1470,'8','','','','Ionos',1863,'',5 from dual union all
select 6260,'1959-S96','1959-06-26 00:00:00','military missile','success',1367,'198','R&D Series IV','','','Test',236,'',520 from dual union all
select 62600,'1989-W102','1989-03-29 15:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',70 from dual union all
select 62601,'1989-W103','1989-04-03 13:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',63 from dual union all
select 62602,'1989-W104','1989-04-04 19:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',63 from dual union all
select 62603,'1989-W105','1989-04-05 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',79 from dual union all
select 62604,'1989-W106','1989-04-07 13:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',72 from dual union all
select 62605,'1989-W107','1989-04-11 14:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 62606,'1989-W108','1989-04-11 15:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',59 from dual union all
select 62607,'1989-W109','1989-04-12 13:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',76 from dual union all
select 62608,'1989-W11','1989-01-06 10:49:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',58 from dual union all
select 62609,'1989-W110','1989-04-19 09:14:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',79 from dual union all
select 6261,'1959-S97','1959-06-26 00:00:00','suborbital rocket','success',1327,'Terasca','Radar beacon','','','Test',1023,'',100 from dual union all
select 62610,'1989-W111','1989-04-19 09:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',64 from dual union all
select 62611,'1989-W112','1989-04-19 12:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',57 from dual union all
select 62612,'1989-W113','1989-04-19 13:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual union all
select 62613,'1989-W114','1989-04-19 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 62614,'1989-W115','1989-04-19 14:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',68 from dual union all
select 62615,'1989-W116','1989-04-26 12:40:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',75 from dual union all
select 62616,'1989-W117','1989-04-26 17:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',74 from dual union all
select 62617,'1989-W118','1989-04-27 18:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',57 from dual union all
select 62618,'1989-W119','1989-04-28 16:38:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',65 from dual union all
select 62619,'1989-W12','1989-01-07 10:13:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',62 from dual union all
select 6262,'1959-S98','1959-06-29 15:42:00','military missile','success',933,'AX-9','','','','Test',244,'',500 from dual union all
select 62620,'1989-W120','1989-05-02 21:25:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 62621,'1989-W121','1989-05-03 07:15:00','suborbital rocket','unknown',738,'','','','','Aeron',1034,'PASSAT',63 from dual union all
select 62622,'1989-W122','1989-05-03 13:10:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',69 from dual union all
select 62623,'1989-W123','1989-05-03 17:13:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 62624,'1989-W124','1989-05-04 11:33:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',77 from dual union all
select 62625,'1989-W125','1989-05-04 20:13:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 62626,'1989-W126','1989-05-04 21:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',69 from dual union all
select 62627,'1989-W127','1989-05-05 16:10:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 62628,'1989-W128','1989-05-05 17:22:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',64 from dual union all
select 62629,'1989-W129','1989-05-08 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',70 from dual union all
select 6263,'1959-S99','1959-06-30 02:37:00','military missile','success',1367,'194','R&D Series IV Lofted','','','Test',234,'',1000 from dual union all
select 62630,'1989-W13','1989-01-07 12:51:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',56 from dual union all
select 62631,'1989-W130','1989-05-08 15:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',58 from dual union all
select 62632,'1989-W131','1989-05-08 16:33:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',63 from dual union all
select 62633,'1989-W132','1989-05-08 19:43:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',57 from dual union all
select 62634,'1989-W133','1989-05-08 20:14:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 62635,'1989-W134','1989-05-08 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',70 from dual union all
select 62636,'1989-W135','1989-05-10 07:11:00','suborbital rocket','unknown',738,'','','','','Aeron',1034,'PASSAT',62 from dual union all
select 62637,'1989-W136','1989-05-10 12:10:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',74 from dual union all
select 62638,'1989-W137','1989-05-10 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 62639,'1989-W138','1989-05-11 13:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',72 from dual union all
select 6264,'1959-U01','1959-06-25 22:47:45','orbital','failure',1360,'179/FTV 1023','Discoverer 4','CORONA C-1','LONG ROAD','Sat TAG4',1679,'',177 from dual union all
select 62640,'1989-W139','1989-05-12 07:03:00','suborbital rocket','unknown',738,'','','','','Aeron',1034,'PASSAT',65 from dual union all
select 62641,'1989-W14','1989-01-08 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 62642,'1989-W140','1989-05-15 17:03:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 62643,'1989-W141','1989-05-17 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',1034,'PASSAT',63 from dual union all
select 62644,'1989-W142','1989-05-18 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 62645,'1989-W143','1989-05-18 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 62646,'1989-W144','1989-05-19 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',1034,'PASSAT',65 from dual union all
select 62647,'1989-W145','1989-05-19 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 62648,'1989-W146','1989-05-22 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual;
insert into LAUNCH
select 62649,'1989-W147','1989-05-24 07:15:00','suborbital rocket','unknown',738,'','','','','Aeron',1034,'PASSAT',62 from dual union all
select 6265,'1959-W01','1959-01-01 00:00:00','suborbital rocket','success',88,'(1 of 25)','','','','Aeron',276,'',61 from dual union all
select 62650,'1989-W148','1989-05-24 09:54:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',79 from dual union all
select 62651,'1989-W149','1989-05-24 12:10:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',55 from dual union all
select 62652,'1989-W15','1989-01-09 07:05:00','suborbital rocket','unknown',738,'','','','','Aeron',216,'BUGAEV',60 from dual union all
select 62653,'1989-W150','1989-05-24 14:13:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 62654,'1989-W151','1989-05-24 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',64 from dual union all
select 62655,'1989-W152','1989-05-26 07:03:00','suborbital rocket','unknown',738,'','','','','Aeron',1034,'PASSAT',63 from dual union all
select 62656,'1989-W153','1989-05-31 16:21:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',76 from dual union all
select 62657,'1989-W154','1989-06-01 13:06:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 62658,'1989-W155','1989-06-02 07:03:00','suborbital rocket','unknown',738,'','','','','Aeron',983,'MUSSON',57 from dual union all
select 62659,'1989-W156','1989-06-04 07:02:00','suborbital rocket','unknown',738,'','','','','Aeron',983,'MUSSON',60 from dual union all
select 6266,'1959-W02','1959-01-23 15:15:00','suborbital rocket','success',88,'4','Test','','','Test',1895,'',54 from dual union all
select 62660,'1989-W157','1989-06-06 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 62661,'1989-W158','1989-06-07 07:08:00','suborbital rocket','unknown',738,'','','','','Aeron',983,'MUSSON',64 from dual union all
select 62662,'1989-W159','1989-06-07 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 62663,'1989-W16','1989-01-09 11:42:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 62664,'1989-W160','1989-06-08 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 62665,'1989-W161','1989-06-09 07:05:00','suborbital rocket','unknown',738,'','','','','Aeron',983,'MUSSON',61 from dual union all
select 62666,'1989-W162','1989-06-13 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 62667,'1989-W163','1989-06-14 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 62668,'1989-W164','1989-06-14 14:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',62 from dual union all
select 62669,'1989-W165','1989-06-14 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',77 from dual union all
select 6267,'1959-W03','1959-02-11 23:00:00','suborbital rocket','success',716,'','','','','Aeron',1895,'',50 from dual union all
select 62670,'1989-W166','1989-06-14 16:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 62671,'1989-W167','1989-06-16 07:03:00','suborbital rocket','unknown',738,'','','','','Aeron',983,'MUSSON',62 from dual union all
select 62672,'1989-W168','1989-06-16 14:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 62673,'1989-W169','1989-06-19 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 62674,'1989-W17','1989-01-09 12:46:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',67 from dual union all
select 62675,'1989-W170','1989-06-20 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 62676,'1989-W171','1989-06-21 07:04:00','suborbital rocket','unknown',738,'','','','','Aeron',983,'MUSSON',61 from dual union all
select 62677,'1989-W172','1989-06-21 10:08:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',78 from dual union all
select 62678,'1989-W173','1989-06-21 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',77 from dual union all
select 62679,'1989-W174','1989-06-21 12:30:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',77 from dual union all
select 6268,'1959-W04','1959-08-07 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1821,'',55 from dual union all
select 62680,'1989-W175','1989-06-21 18:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',70 from dual union all
select 62681,'1989-W176','1989-06-22 22:52:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 62682,'1989-W177','1989-06-23 07:17:00','suborbital rocket','unknown',738,'','','','','Aeron',983,'MUSSON',61 from dual union all
select 62683,'1989-W178','1989-06-26 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 62684,'1989-W179','1989-06-28 07:14:00','suborbital rocket','unknown',738,'','','','','Aeron',983,'MUSSON',62 from dual union all
select 62685,'1989-W18','1989-01-09 12:52:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',55 from dual union all
select 62686,'1989-W180','1989-06-28 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 62687,'1989-W181','1989-06-28 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',66 from dual union all
select 62688,'1989-W182','1989-06-28 16:33:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 62689,'1989-W183','1989-07-12 15:15:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',77 from dual union all
select 6269,'1959-W05','1959-10-13 00:00:00','suborbital rocket','success',715,'Loki II','','','','Aeron',1057,'',70 from dual union all
select 62690,'1989-W184','1989-07-12 16:28:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',77 from dual union all
select 62691,'1989-W185','1989-07-26 16:28:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',72 from dual union all
select 62692,'1989-W186','1989-08-02 11:22:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',72 from dual union all
select 62693,'1989-W187','1989-08-02 16:09:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 62694,'1989-W188','1989-08-03 09:49:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',79 from dual union all
select 62695,'1989-W189','1989-08-07 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 62696,'1989-W19','1989-01-11 02:02:00','suborbital rocket','success',761,'MT135P-R696','','','','Meteo',1332,'',55 from dual union all
select 62697,'1989-W190','1989-08-08 14:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 62698,'1989-W191','1989-08-08 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 62699,'1989-W192','1989-08-09 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',63 from dual union all
select 627,'1946-A10','1946-08-15 18:00:00','suborbital rocket','failure',1470,'10','','','','CR/Ionos',1863,'',3 from dual union all
select 6270,'1959-W06','1959-10-14 00:00:00','suborbital rocket','success',715,'Loki II','','','','Aeron',1057,'',69 from dual union all
select 62700,'1989-W193','1989-08-09 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',76 from dual union all
select 62701,'1989-W194','1989-08-09 18:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 62702,'1989-W195','1989-08-10 12:33:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',62 from dual union all
select 62703,'1989-W196','1989-08-10 13:22:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',65 from dual union all
select 62704,'1989-W197','1989-08-11 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 62705,'1989-W198','1989-08-13 10:57:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 62706,'1989-W199','1989-08-13 11:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',60 from dual union all
select 62707,'1989-W20','1989-01-11 08:30:00','suborbital rocket','unknown',738,'','','','','Aeron',216,'BUGAEV',60 from dual union all
select 62708,'1989-W200','1989-08-13 11:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',60 from dual union all
select 62709,'1989-W201','1989-08-13 11:55:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',62 from dual union all
select 6271,'1959-W07','1959-10-20 00:00:00','suborbital rocket','success',715,'Loki II','','','','Aeron',1057,'',70 from dual union all
select 62710,'1989-W202','1989-08-13 13:36:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',63 from dual union all
select 62711,'1989-W203','1989-08-13 14:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',59 from dual union all
select 62712,'1989-W204','1989-08-13 15:15:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',64 from dual union all
select 62713,'1989-W205','1989-08-16 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',58 from dual union all
select 62714,'1989-W206','1989-08-16 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',78 from dual union all
select 62715,'1989-W207','1989-08-21 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 62716,'1989-W208','1989-08-24 16:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',61 from dual union all
select 62717,'1989-W209','1989-08-25 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 62718,'1989-W21','1989-01-11 11:16:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',77 from dual union all
select 62719,'1989-W210','1989-08-28 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 6272,'1959-W08','1959-10-21 00:00:00','suborbital rocket','success',715,'Loki II','','','','Aeron',1057,'',64 from dual union all
select 62720,'1989-W211','1989-08-29 16:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',69 from dual union all
select 62721,'1989-W212','1989-08-30 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 62722,'1989-W213','1989-09-01 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 62723,'1989-W214','1989-09-01 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',76 from dual union all
select 62724,'1989-W215','1989-09-01 17:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',76 from dual union all
select 62725,'1989-W216','1989-09-01 18:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',62 from dual union all
select 62726,'1989-W217','1989-09-05 18:51:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',78 from dual union all
select 62727,'1989-W218','1989-09-06 02:00:00','suborbital rocket','success',759,'MT-135-50','NNP-O3','','','Meteo',839,'',54 from dual union all
select 62728,'1989-W219','1989-09-06 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 62729,'1989-W22','1989-01-11 14:44:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',57 from dual union all
select 6273,'1959-W09','1959-10-26 00:00:00','suborbital rocket','success',715,'Loki II','','','','Aeron',1057,'',70 from dual union all
select 62730,'1989-W220','1989-09-06 12:38:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',74 from dual union all
select 62731,'1989-W221','1989-09-08 14:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',73 from dual union all
select 62732,'1989-W222','1989-09-11 20:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',71 from dual union all
select 62733,'1989-W223','1989-09-13 11:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',76 from dual union all
select 62734,'1989-W224','1989-09-15 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',74 from dual union all
select 62735,'1989-W225','1989-09-22 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 62736,'1989-W226','1989-09-26 17:25:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 62737,'1989-W227','1989-09-28 16:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',66 from dual union all
select 62738,'1989-W228','1989-10-04 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',78 from dual;
insert into LAUNCH
select 62739,'1989-W229','1989-10-04 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 6274,'1959-W10','1959-10-27 00:00:00','suborbital rocket','success',715,'Loki II','','','','Aeron',1057,'',70 from dual union all
select 62740,'1989-W23','1989-01-11 15:09:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',69 from dual union all
select 62741,'1989-W230','1989-10-09 16:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',64 from dual union all
select 62742,'1989-W231','1989-10-09 16:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',64 from dual union all
select 62743,'1989-W232','1989-10-10 13:10:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',74 from dual union all
select 62744,'1989-W233','1989-10-11 07:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',76 from dual union all
select 62745,'1989-W234','1989-10-11 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',78 from dual union all
select 62746,'1989-W235','1989-10-11 16:25:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 62747,'1989-W236','1989-10-15 16:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',74 from dual union all
select 62748,'1989-W237','1989-10-16 07:17:00','suborbital rocket','unknown',728,'','','','','Aeron',1644,'USHAKOV',65 from dual union all
select 62749,'1989-W238','1989-10-16 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 6275,'1959-W11','1959-11-12 00:00:00','suborbital rocket','success',715,'Loki II','','','','Aeron',1057,'',67 from dual union all
select 62750,'1989-W239','1989-10-17 14:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 62751,'1989-W24','1989-01-12 10:30:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',61 from dual union all
select 62752,'1989-W240','1989-10-18 08:20:00','suborbital rocket','unknown',738,'','','','','Aeron',1644,'USHAKOV',66 from dual union all
select 62753,'1989-W241','1989-10-18 12:37:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',77 from dual union all
select 62754,'1989-W242','1989-10-18 13:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',74 from dual union all
select 62755,'1989-W243','1989-10-18 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 62756,'1989-W244','1989-10-18 21:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',58 from dual union all
select 62757,'1989-W245','1989-10-19 14:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',66 from dual union all
select 62758,'1989-W246','1989-10-20 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 62759,'1989-W247','1989-10-22 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1644,'USHAKOV',64 from dual union all
select 6276,'1959-W12','1959-11-18 00:00:00','suborbital rocket','success',715,'Loki II','','','','Aeron',1057,'',66 from dual union all
select 62760,'1989-W248','1989-10-22 08:09:00','suborbital rocket','unknown',738,'','','','','Aeron',1644,'USHAKOV',64 from dual union all
select 62761,'1989-W249','1989-10-23 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1644,'USHAKOV',62 from dual union all
select 62762,'1989-W25','1989-01-13 08:57:00','suborbital rocket','unknown',738,'','','','','Aeron',216,'BUGAEV',60 from dual union all
select 62763,'1989-W250','1989-10-23 08:32:00','suborbital rocket','unknown',738,'','','','','Aeron',1644,'USHAKOV',56 from dual union all
select 62764,'1989-W251','1989-10-23 17:12:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 62765,'1989-W252','1989-10-23 18:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',58 from dual union all
select 62766,'1989-W253','1989-10-23 19:48:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',69 from dual union all
select 62767,'1989-W254','1989-10-25 07:15:00','suborbital rocket','unknown',738,'','','','','Aeron',1644,'USHAKOV',64 from dual union all
select 62768,'1989-W255','1989-10-25 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',70 from dual union all
select 62769,'1989-W256','1989-10-25 14:03:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',68 from dual union all
select 6277,'1959-W13','1959-10-28 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1057,'',51 from dual union all
select 62770,'1989-W257','1989-10-26 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',78 from dual union all
select 62771,'1989-W258','1989-10-30 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',69 from dual union all
select 62772,'1989-W259','1989-10-31 17:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',69 from dual union all
select 62773,'1989-W26','1989-01-13 11:51:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',56 from dual union all
select 62774,'1989-W260','1989-11-01 14:03:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 62775,'1989-W261','1989-11-02 17:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',61 from dual union all
select 62776,'1989-W262','1989-11-04 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',917,'KRENKEL''',60 from dual union all
select 62777,'1989-W263','1989-11-07 17:31:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',66 from dual union all
select 62778,'1989-W264','1989-11-08 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',917,'KRENKEL''',65 from dual union all
select 62779,'1989-W265','1989-11-08 11:18:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',77 from dual union all
select 6278,'1959-W14','1959-11-04 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1057,'',59 from dual union all
select 62780,'1989-W266','1989-11-08 13:10:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',73 from dual union all
select 62781,'1989-W267','1989-11-08 15:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 62782,'1989-W268','1989-11-10 07:08:00','suborbital rocket','unknown',738,'','','','','Aeron',917,'KRENKEL''',65 from dual union all
select 62783,'1989-W269','1989-11-15 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',917,'KRENKEL''',64 from dual union all
select 62784,'1989-W27','1989-01-15 07:02:00','suborbital rocket','unknown',738,'','','','','Aeron',216,'BUGAEV',65 from dual union all
select 62785,'1989-W270','1989-11-15 22:36:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',67 from dual union all
select 62786,'1989-W271','1989-11-17 07:16:00','suborbital rocket','unknown',738,'','','','','Aeron',917,'KRENKEL''',63 from dual union all
select 62787,'1989-W272','1989-11-21 14:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 62788,'1989-W273','1989-11-21 17:58:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',67 from dual union all
select 62789,'1989-W274','1989-11-22 07:04:00','suborbital rocket','unknown',738,'','','','','Aeron',917,'KRENKEL''',64 from dual union all
select 6279,'1959-W15','1959-11-10 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1057,'',61 from dual union all
select 62790,'1989-W275','1989-11-22 13:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',77 from dual union all
select 62791,'1989-W276','1989-11-22 14:01:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',67 from dual union all
select 62792,'1989-W277','1989-11-22 16:03:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',63 from dual union all
select 62793,'1989-W278','1989-11-22 20:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 62794,'1989-W279','1989-11-23 02:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 62795,'1989-W28','1989-01-18 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',216,'BUGAEV',63 from dual union all
select 62796,'1989-W280','1989-11-26 14:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',66 from dual union all
select 62797,'1989-W281','1989-11-26 15:09:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',65 from dual union all
select 62798,'1989-W282','1989-11-27 07:02:00','suborbital rocket','unknown',738,'','','','','Aeron',917,'KRENKEL''',63 from dual union all
select 62799,'1989-W283','1989-11-28 00:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',61 from dual union all
select 628,'1946-A11','1946-08-22 17:15:00','suborbital rocket','failure',1470,'11','UM Aeron','','','Aeron',1863,'',0 from dual union all
select 6280,'1959-W16','1959-11-16 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1057,'',58 from dual union all
select 62800,'1989-W284','1989-11-28 00:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',68 from dual union all
select 62801,'1989-W285','1989-11-28 01:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',66 from dual union all
select 62802,'1989-W286','1989-11-28 02:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 62803,'1989-W287','1989-11-28 02:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',54 from dual union all
select 62804,'1989-W288','1989-11-29 08:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',79 from dual union all
select 62805,'1989-W289','1989-11-29 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 62806,'1989-W29','1989-01-18 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',59 from dual union all
select 62807,'1989-W290','1989-11-29 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',72 from dual union all
select 62808,'1989-W291','1989-11-29 18:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 62809,'1989-W292','1989-12-06 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 6281,'1959-W18','1959-06-05 00:00:00','suborbital rocket','success',88,'6','Test','','','Test',1895,'',66 from dual union all
select 62810,'1989-W293','1989-12-06 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',58 from dual union all
select 62811,'1989-W294','1989-12-08 13:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',75 from dual union all
select 62812,'1989-W295','1989-12-13 15:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 62813,'1989-W296','1989-12-13 16:19:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 62814,'1989-W297','1989-12-13 16:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',68 from dual union all
select 62815,'1989-W298','1989-12-15 14:37:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 62816,'1989-W299','1989-12-15 15:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 62817,'1989-W30','1989-01-19 15:29:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',67 from dual union all
select 62818,'1989-W300','1989-12-15 17:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',57 from dual union all
select 62819,'1989-W301','1989-12-17 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',58 from dual union all
select 6282,'1959-W19','1959-08-07 00:00:00','suborbital rocket','success',88,'','Robin','','','Test',394,'',80 from dual union all
select 62820,'1989-W302','1989-12-18 22:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 62821,'1989-W303','1989-12-20 13:40:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',78 from dual union all
select 62822,'1989-W304','1989-12-20 15:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 62823,'1989-W305','1989-12-26 18:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',60 from dual union all
select 62824,'1989-W306','1989-12-28 22:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',60 from dual union all
select 62825,'1989-W307','1989-01-18 01:50:00','suborbital rocket','success',761,'MT135P-R697','','','','Meteo',1332,'',60 from dual union all
select 62826,'1989-W308','1989-01-25 02:00:00','suborbital rocket','success',761,'MT135P-R698','','','','Meteo',1332,'',60 from dual union all
select 62827,'1989-W309','1989-02-01 02:00:00','suborbital rocket','success',761,'MT135P-R699','','','','Meteo',1332,'',60 from dual union all
select 62828,'1989-W31','1989-01-20 07:02:00','suborbital rocket','unknown',738,'','','','','Aeron',216,'BUGAEV',61 from dual;
insert into LAUNCH
select 62829,'1989-W310','1989-02-15 02:00:00','suborbital rocket','success',761,'MT135P-R700','','','','Meteo',1332,'',60 from dual union all
select 6283,'1959-W20','1959-01-30 16:55:00','suborbital rocket','success',88,'5','Test','','','Test',1895,'',52 from dual union all
select 62830,'1989-W311','1989-02-22 02:06:00','suborbital rocket','success',761,'MT135P-R701','','','','Meteo',1332,'',60 from dual union all
select 62831,'1989-W312','1989-03-02 02:00:00','suborbital rocket','success',761,'MT135P-R702','','','','Meteo',1332,'',60 from dual union all
select 62832,'1989-W313','1989-03-08 02:00:00','suborbital rocket','success',761,'MT135P-R703','','','','Meteo',1332,'',60 from dual union all
select 62833,'1989-W314','1989-03-15 02:00:00','suborbital rocket','success',761,'MT135P-R704','','','','Meteo',1332,'',60 from dual union all
select 62834,'1989-W315','1989-03-29 02:00:00','suborbital rocket','success',761,'MT135P-R705','','','','Meteo',1332,'',60 from dual union all
select 62835,'1989-W316','1989-04-13 02:02:00','suborbital rocket','success',761,'MT135P-R706','','','','Meteo',1332,'',60 from dual union all
select 62836,'1989-W317','1989-04-19 02:00:00','suborbital rocket','success',761,'MT135P-R707','','','','Meteo',1332,'',60 from dual union all
select 62837,'1989-W318','1989-04-26 02:04:00','suborbital rocket','success',761,'MT135P-R708','','','','Meteo',1332,'',60 from dual union all
select 62838,'1989-W319','1989-05-10 02:00:00','suborbital rocket','success',761,'MT135P-R709','','','','Meteo',1332,'',60 from dual union all
select 62839,'1989-W32','1989-01-23 13:32:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',67 from dual union all
select 6284,'1959-Y01','1959-01-03 00:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 62840,'1989-W320','1989-05-17 02:00:00','suborbital rocket','success',761,'MT135P-R710','','','','Meteo',1332,'',60 from dual union all
select 62841,'1989-W321','1989-05-31 02:03:00','suborbital rocket','success',761,'MT135P-R711','','','','Meteo',1332,'',60 from dual union all
select 62842,'1989-W322','1989-06-14 02:00:00','suborbital rocket','success',761,'MT135P-R712','','','','Meteo',1332,'',60 from dual union all
select 62843,'1989-W323','1989-06-21 02:00:00','suborbital rocket','success',761,'MT135P-R713','','','','Meteo',1332,'',60 from dual union all
select 62844,'1989-W324','1989-06-28 02:00:00','suborbital rocket','success',761,'MT135P-R714','','','','Meteo',1332,'',60 from dual union all
select 62845,'1989-W325','1989-07-13 02:00:00','suborbital rocket','success',761,'MT135P-R715','','','','Meteo',1332,'',60 from dual union all
select 62846,'1989-W326','1989-08-02 02:00:00','suborbital rocket','success',761,'MT135P-R716','','','','Meteo',1332,'',60 from dual union all
select 62847,'1989-W327','1989-08-16 02:00:00','suborbital rocket','success',761,'MT135P-R717','','','','Meteo',1332,'',60 from dual union all
select 62848,'1989-W328','1989-08-30 02:00:00','suborbital rocket','success',761,'MT135P-R718','','','','Meteo',1332,'',60 from dual union all
select 62849,'1989-W329','1989-09-13 02:00:00','suborbital rocket','success',761,'MT135P-R719','','','','Meteo',1332,'',60 from dual union all
select 6285,'1959-Y02','1959-01-10 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 62850,'1989-W33','1989-01-24 10:00:00','suborbital rocket','unknown',728,'','','','','Aeron',896,'KORO',77 from dual union all
select 62851,'1989-W330','1989-09-20 02:01:00','suborbital rocket','success',761,'MT135P-R720','','','','Meteo',1332,'',60 from dual union all
select 62852,'1989-W331','1989-10-04 02:00:00','suborbital rocket','success',761,'MT135P-R721','','','','Meteo',1332,'',60 from dual union all
select 62853,'1989-W332','1989-10-18 02:04:00','suborbital rocket','success',761,'MT135P-R722','','','','Meteo',1332,'',60 from dual union all
select 62854,'1989-W333','1989-11-01 02:00:00','suborbital rocket','success',761,'MT135P-R723','','','','Meteo',1332,'',60 from dual union all
select 62855,'1989-W334','1989-11-15 02:00:00','suborbital rocket','success',761,'MT135P-R724','','','','Meteo',1332,'',60 from dual union all
select 62856,'1989-W335','1989-11-22 02:00:00','suborbital rocket','success',761,'MT135P-R725','','','','Meteo',1332,'',60 from dual union all
select 62857,'1989-W336','1989-12-06 02:00:00','suborbital rocket','success',761,'MT135P-R726','','','','Meteo',1332,'',60 from dual union all
select 62858,'1989-W337','1989-12-13 02:02:00','suborbital rocket','success',761,'MT135P-R727','','','','Meteo',1332,'',60 from dual union all
select 62859,'1989-W338','1989-12-21 02:10:00','suborbital rocket','success',761,'MT135P-R728','','','','Meteo',1332,'',60 from dual union all
select 6286,'1959-Y03','1959-02-12 00:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 62860,'1989-W339','1989-12-28 02:00:00','suborbital rocket','success',761,'MT135P-R729','','','','Meteo',1332,'',60 from dual union all
select 62861,'1989-W34','1989-01-25 07:21:00','suborbital rocket','unknown',738,'','','','','Aeron',216,'BUGAEV',61 from dual union all
select 62862,'1989-W35','1989-01-25 13:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',63 from dual union all
select 62863,'1989-W36','1989-01-25 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 62864,'1989-W37','1989-01-25 14:31:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',66 from dual union all
select 62865,'1989-W38','1989-01-25 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',79 from dual union all
select 62866,'1989-W39','1989-01-26 09:58:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 62867,'1989-W40','1989-01-26 11:00:00','suborbital rocket','unknown',728,'','','','','Aeron',896,'KORO',78 from dual union all
select 62868,'1989-W41','1989-01-26 17:20:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',60 from dual union all
select 62869,'1989-W42','1989-01-27 00:11:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',64 from dual union all
select 6287,'1959-Y04','1959-03-12 11:09:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 62870,'1989-W43','1989-01-27 08:39:00','suborbital rocket','unknown',738,'','','','','Aeron',216,'BUGAEV',63 from dual union all
select 62871,'1989-W44','1989-01-30 13:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',59 from dual union all
select 62872,'1989-W45','1989-02-01 07:22:00','suborbital rocket','unknown',738,'','','','','Aeron',1033,'PASSAT',53 from dual union all
select 62873,'1989-W46','1989-02-01 19:42:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',68 from dual union all
select 62874,'1989-W47','1989-02-03 07:09:00','suborbital rocket','unknown',738,'','','','','Aeron',1033,'PASSAT',57 from dual union all
select 62875,'1989-W48','1989-02-03 15:00:00','suborbital rocket','unknown',728,'','','','','Aeron',896,'KORO',67 from dual union all
select 62876,'1989-W49','1989-02-05 07:06:00','suborbital rocket','unknown',738,'','','','','Aeron',1033,'PASSAT',60 from dual union all
select 62877,'1989-W50','1989-02-08 07:04:00','suborbital rocket','unknown',738,'','','','','Aeron',1033,'PASSAT',61 from dual union all
select 62878,'1989-W51','1989-02-08 12:34:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 62879,'1989-W52','1989-02-09 08:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',77 from dual union all
select 6288,'1959-Y05','1959-03-12 15:40:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 62880,'1989-W53','1989-02-09 15:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 62881,'1989-W54','1989-02-10 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',79 from dual union all
select 62882,'1989-W55','1989-02-10 20:12:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',64 from dual union all
select 62883,'1989-W56','1989-02-11 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 62884,'1989-W57','1989-02-12 07:04:00','suborbital rocket','unknown',738,'','','','','Aeron',1033,'PASSAT',60 from dual union all
select 62885,'1989-W58','1989-02-15 15:09:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 62886,'1989-W59','1989-02-17 07:03:00','suborbital rocket','unknown',738,'','','','','Aeron',1033,'PASSAT',67 from dual union all
select 62887,'1989-W60','1989-02-22 07:01:00','suborbital rocket','unknown',738,'','','','','Aeron',1033,'PASSAT',60 from dual union all
select 62888,'1989-W61','1989-02-22 12:44:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 62889,'1989-W62','1989-02-22 14:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',76 from dual union all
select 6289,'1959-Y06','1959-03-13 15:10:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 62890,'1989-W63','1989-02-22 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 62891,'1989-W64','1989-02-22 15:25:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',74 from dual union all
select 62892,'1989-W65','1989-02-23 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',63 from dual union all
select 62893,'1989-W66','1989-02-24 14:00:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',76 from dual union all
select 62894,'1989-W67','1989-02-24 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',76 from dual union all
select 62895,'1989-W68','1989-02-26 11:46:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',65 from dual union all
select 62896,'1989-W69','1989-02-27 15:29:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',56 from dual union all
select 62897,'1989-W70','1989-03-01 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',67 from dual union all
select 62898,'1989-W71','1989-03-01 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 62899,'1989-W72','1989-03-01 18:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 629,'1946-A12','1946-09-24 17:15:00','atmospheric rocket','success',875,'46-A','Dummy Sustainer','','','Test',1863,'',9 from dual union all
select 6290,'1959-Y07','1959-04-02 08:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 62900,'1989-W73','1989-03-08 14:43:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',65 from dual union all
select 62901,'1989-W74','1989-03-08 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',51 from dual union all
select 62902,'1989-W75','1989-03-10 15:14:00','suborbital rocket','unknown',728,'','','','','Aeron',896,'KORO',78 from dual union all
select 62903,'1989-W76','1989-03-12 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 62904,'1989-W77','1989-03-13 12:25:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',63 from dual union all
select 62905,'1989-W78','1989-03-13 13:29:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 62906,'1989-W79','1989-03-13 16:05:00','suborbital rocket','unknown',728,'','','','','Aeron',896,'KORO',78 from dual union all
select 62907,'1989-W80','1989-03-14 15:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 62908,'1989-W81','1989-03-15 10:33:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',76 from dual union all
select 62909,'1989-W82','1989-03-15 11:02:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',70 from dual union all
select 6291,'1959-Y08','1959-05-05 12:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 62910,'1989-W83','1989-03-15 12:01:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',65 from dual union all
select 62911,'1989-W84','1989-03-15 13:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 62912,'1989-W85','1989-03-15 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',74 from dual union all
select 62913,'1989-W86','1989-03-15 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',68 from dual union all
select 62914,'1989-W87','1989-03-15 15:45:00','suborbital rocket','unknown',728,'','','','','Aeron',896,'KORO',79 from dual union all
select 62915,'1989-W88','1989-03-16 12:47:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',57 from dual union all
select 62916,'1989-W89','1989-03-16 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 62917,'1989-W90','1989-03-17 11:55:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',59 from dual union all
select 62918,'1989-W91','1989-03-17 11:57:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',54 from dual;
insert into LAUNCH
select 62919,'1989-W92','1989-03-17 14:32:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',59 from dual union all
select 6292,'1959-Y09','1959-07-02 03:41:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 62920,'1989-W93','1989-03-18 10:45:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',68 from dual union all
select 62921,'1989-W94','1989-03-18 11:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',64 from dual union all
select 62922,'1989-W95','1989-03-18 11:48:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',67 from dual union all
select 62923,'1989-W96','1989-03-18 15:09:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 62924,'1989-W97','1989-03-18 15:19:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',75 from dual union all
select 62925,'1989-W98','1989-03-19 10:30:00','suborbital rocket','unknown',738,'','','','','Aeron',1305,'PRILIV',62 from dual union all
select 62926,'1989-W99','1989-03-20 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 62927,'1989-Y01','1989-01-04 06:38:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',82 from dual union all
select 62928,'1989-Y02','1989-01-04 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 62929,'1989-Y03','1989-01-09 11:25:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 6293,'1959-Y10','1959-07-03 03:45:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 62930,'1989-Y04','1989-01-11 06:50:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',86 from dual union all
select 62931,'1989-Y05','1989-01-11 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 62932,'1989-Y06','1989-01-16 11:41:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',88 from dual union all
select 62933,'1989-Y07','1989-01-16 11:52:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62934,'1989-Y08','1989-01-16 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 62935,'1989-Y09','1989-01-17 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',896,'KORO',80 from dual union all
select 62936,'1989-Y10','1989-01-18 07:04:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',84 from dual union all
select 62937,'1989-Y100','1989-04-27 13:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 62938,'1989-Y101','1989-04-28 15:18:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 62939,'1989-Y102','1989-05-02 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',95 from dual union all
select 6294,'1959-Y11','1959-09-07 12:45:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 62940,'1989-Y103','1989-05-03 09:27:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',82 from dual union all
select 62941,'1989-Y104','1989-05-03 14:52:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 62942,'1989-Y105','1989-05-03 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 62943,'1989-Y106','1989-05-04 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',80 from dual union all
select 62944,'1989-Y107','1989-05-04 19:35:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',86 from dual union all
select 62945,'1989-Y108','1989-05-04 21:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 62946,'1989-Y109','1989-05-04 21:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',95 from dual union all
select 62947,'1989-Y11','1989-01-18 11:27:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62948,'1989-Y110','1989-05-05 15:31:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',85 from dual union all
select 62949,'1989-Y111','1989-05-08 14:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',88 from dual union all
select 6295,'1959-Y12','1959-09-10 00:20:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 62950,'1989-Y112','1989-05-08 14:37:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 62951,'1989-Y113','1989-05-08 15:03:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 62952,'1989-Y114','1989-05-08 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',88 from dual union all
select 62953,'1989-Y115','1989-05-08 19:43:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 62954,'1989-Y116','1989-05-10 11:20:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 62955,'1989-Y117','1989-05-10 15:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 62956,'1989-Y118','1989-05-10 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',92 from dual union all
select 62957,'1989-Y119','1989-05-11 10:21:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',85 from dual union all
select 62958,'1989-Y12','1989-01-18 13:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 62959,'1989-Y120','1989-05-17 10:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',83 from dual union all
select 6296,'1959-Y13','1959-09-13 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 62960,'1989-Y121','1989-05-17 11:02:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 62961,'1989-Y122','1989-05-17 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 62962,'1989-Y123','1989-05-17 17:45:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 62963,'1989-Y124','1989-05-24 11:52:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62964,'1989-Y125','1989-05-24 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',89 from dual union all
select 62965,'1989-Y126','1989-05-24 16:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 62966,'1989-Y127','1989-05-31 10:27:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',83 from dual union all
select 62967,'1989-Y128','1989-05-31 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62968,'1989-Y129','1989-05-31 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 62969,'1989-Y13','1989-01-18 14:37:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 6297,'1959-Y14','1959-09-18 19:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 62970,'1989-Y130','1989-06-07 09:58:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',85 from dual union all
select 62971,'1989-Y131','1989-06-07 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 62972,'1989-Y132','1989-06-07 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 62973,'1989-Y133','1989-06-09 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 62974,'1989-Y134','1989-06-14 10:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',85 from dual union all
select 62975,'1989-Y135','1989-06-21 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',87 from dual union all
select 62976,'1989-Y136','1989-06-21 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 62977,'1989-Y137','1989-06-21 16:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 62978,'1989-Y138','1989-06-26 09:20:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',81 from dual union all
select 62979,'1989-Y139','1989-06-28 10:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',82 from dual union all
select 6298,'1959-Y15','1959-09-24 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 62980,'1989-Y14','1989-01-18 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 62981,'1989-Y140','1989-06-28 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 62982,'1989-Y141','1989-07-05 10:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',80 from dual union all
select 62983,'1989-Y142','1989-07-05 11:26:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 62984,'1989-Y143','1989-07-05 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',87 from dual union all
select 62985,'1989-Y144','1989-07-05 16:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 62986,'1989-Y145','1989-07-05 18:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 62987,'1989-Y146','1989-07-10 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',90 from dual union all
select 62988,'1989-Y147','1989-07-10 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',90 from dual union all
select 62989,'1989-Y148','1989-07-10 19:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 6299,'1959-Y16','1959-10-08 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 62990,'1989-Y149','1989-07-11 13:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 62991,'1989-Y15','1989-01-18 17:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',89 from dual union all
select 62992,'1989-Y150','1989-07-12 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',86 from dual union all
select 62993,'1989-Y151','1989-07-12 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 62994,'1989-Y152','1989-07-13 10:09:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',86 from dual union all
select 62995,'1989-Y153','1989-07-17 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 62996,'1989-Y154','1989-07-19 14:01:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 62997,'1989-Y155','1989-07-19 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 62998,'1989-Y156','1989-07-19 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 62999,'1989-Y157','1989-07-19 16:27:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 63,'1944-A27','1944-12-13 13:20:00','military missile','failure',1470,'19857','Ma324','','','Test',661,'',40 from dual union all
select 630,'1946-A13','1946-09-27 17:00:00','atmospheric rocket','success',875,'46-B','Dummy Sustainer','','','Test',1863,'',13 from dual union all
select 6300,'1959-Y17','1959-10-15 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 63000,'1989-Y158','1989-07-20 13:23:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 63001,'1989-Y159','1989-07-26 10:02:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',82 from dual union all
select 63002,'1989-Y16','1989-01-20 13:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 63003,'1989-Y160','1989-07-26 11:36:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 63004,'1989-Y161','1989-07-26 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 63005,'1989-Y162','1989-07-26 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 63006,'1989-Y163','1989-08-02 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 63007,'1989-Y164','1989-08-02 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual;
insert into LAUNCH
select 63008,'1989-Y165','1989-08-02 16:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 63009,'1989-Y166','1989-08-08 12:56:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',91 from dual union all
select 6301,'1959-Y18','1959-10-20 13:15:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 63010,'1989-Y167','1989-08-08 14:20:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 63011,'1989-Y168','1989-08-09 09:37:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',84 from dual union all
select 63012,'1989-Y169','1989-08-09 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 63013,'1989-Y17','1989-01-21 04:00:00','suborbital rocket','unknown',728,'','','','','Aeron',896,'KORO',81 from dual union all
select 63014,'1989-Y170','1989-08-09 14:25:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 63015,'1989-Y171','1989-08-09 16:11:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 63016,'1989-Y172','1989-08-13 10:20:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',94 from dual union all
select 63017,'1989-Y173','1989-08-13 13:36:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 63018,'1989-Y174','1989-08-16 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 63019,'1989-Y175','1989-08-16 15:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 6302,'1959-Y19','1959-10-22 03:56:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 63020,'1989-Y176','1989-08-16 17:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 63021,'1989-Y177','1989-08-16 20:40:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 63022,'1989-Y178','1989-08-23 10:47:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',80 from dual union all
select 63023,'1989-Y179','1989-08-23 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 63024,'1989-Y18','1989-01-23 11:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 63025,'1989-Y180','1989-08-23 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 63026,'1989-Y181','1989-08-23 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 63027,'1989-Y182','1989-08-23 15:37:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 63028,'1989-Y183','1989-08-26 17:01:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',83 from dual union all
select 63029,'1989-Y184','1989-08-27 06:50:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',83 from dual union all
select 6303,'1959-Y20','1959-10-22 06:29:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 63030,'1989-Y185','1989-08-29 18:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',87 from dual union all
select 63031,'1989-Y186','1989-08-30 10:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 63032,'1989-Y187','1989-08-30 14:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',82 from dual union all
select 63033,'1989-Y188','1989-08-30 15:25:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 63034,'1989-Y189','1989-08-30 15:53:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 63035,'1989-Y19','1989-01-23 12:38:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 63036,'1989-Y190','1989-08-31 16:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',81 from dual union all
select 63037,'1989-Y191','1989-09-01 07:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',82 from dual union all
select 63038,'1989-Y192','1989-09-01 16:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',81 from dual union all
select 63039,'1989-Y193','1989-09-01 17:27:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',82 from dual union all
select 6304,'1959-Y21','1959-10-22 09:55:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 63040,'1989-Y194','1989-09-01 19:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',81 from dual union all
select 63041,'1989-Y195','1989-09-01 19:40:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 63042,'1989-Y196','1989-09-04 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 63043,'1989-Y197','1989-09-06 10:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',81 from dual union all
select 63044,'1989-Y198','1989-09-06 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 63045,'1989-Y199','1989-09-06 15:11:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 63046,'1989-Y20','1989-01-23 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 63047,'1989-Y200','1989-09-06 15:53:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 63048,'1989-Y201','1989-09-08 16:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',91 from dual union all
select 63049,'1989-Y202','1989-09-11 06:20:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',82 from dual union all
select 6305,'1959-Y22','1959-10-22 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 63050,'1989-Y203','1989-09-11 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 63051,'1989-Y204','1989-09-11 16:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',84 from dual union all
select 63052,'1989-Y205','1989-09-11 17:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',83 from dual union all
select 63053,'1989-Y206','1989-09-11 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 63054,'1989-Y207','1989-09-11 20:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 63055,'1989-Y208','1989-09-13 08:24:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',80 from dual union all
select 63056,'1989-Y209','1989-09-13 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 63057,'1989-Y21','1989-01-25 07:19:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',81 from dual union all
select 63058,'1989-Y210','1989-09-13 14:57:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 63059,'1989-Y211','1989-09-13 22:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 6306,'1959-Y23','1959-10-29 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 63060,'1989-Y212','1989-09-18 14:15:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 63061,'1989-Y213','1989-09-20 09:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',84 from dual union all
select 63062,'1989-Y214','1989-09-20 11:20:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 63063,'1989-Y215','1989-09-20 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 63064,'1989-Y216','1989-09-20 14:43:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 63065,'1989-Y217','1989-09-20 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 63066,'1989-Y218','1989-09-25 15:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',88 from dual union all
select 63067,'1989-Y219','1989-09-27 11:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 63068,'1989-Y22','1989-01-25 11:46:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',84 from dual union all
select 63069,'1989-Y220','1989-09-27 13:25:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 6307,'1959-Y24','1959-11-12 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 63070,'1989-Y221','1989-09-27 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 63071,'1989-Y222','1989-09-28 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 63072,'1989-Y223','1989-10-02 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',88 from dual union all
select 63073,'1989-Y224','1989-10-04 08:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',81 from dual union all
select 63074,'1989-Y225','1989-10-04 13:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 63075,'1989-Y226','1989-10-04 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 63076,'1989-Y227','1989-10-04 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 63077,'1989-Y228','1989-10-05 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 63078,'1989-Y229','1989-10-05 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',85 from dual union all
select 63079,'1989-Y23','1989-01-25 13:16:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',81 from dual union all
select 6308,'1959-Y25','1959-11-19 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 63080,'1989-Y230','1989-10-05 18:02:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',85 from dual union all
select 63081,'1989-Y231','1989-10-09 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 63082,'1989-Y232','1989-10-09 18:14:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',82 from dual union all
select 63083,'1989-Y233','1989-10-11 07:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',95 from dual union all
select 63084,'1989-Y234','1989-10-11 07:01:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',95 from dual union all
select 63085,'1989-Y235','1989-10-11 09:34:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',80 from dual union all
select 63086,'1989-Y236','1989-10-11 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 63087,'1989-Y237','1989-10-11 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 63088,'1989-Y238','1989-10-11 17:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 63089,'1989-Y239','1989-10-11 17:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',84 from dual union all
select 6309,'1959-Y26','1959-12-03 08:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 63090,'1989-Y24','1989-01-26 10:46:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',85 from dual union all
select 63091,'1989-Y240','1989-10-11 20:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 63092,'1989-Y241','1989-10-12 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',93 from dual union all
select 63093,'1989-Y242','1989-10-12 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',93 from dual union all
select 63094,'1989-Y243','1989-10-12 17:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',82 from dual union all
select 63095,'1989-Y244','1989-10-12 18:07:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',86 from dual union all
select 63096,'1989-Y245','1989-10-15 18:36:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',84 from dual union all
select 63097,'1989-Y246','1989-10-16 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 63098,'1989-Y247','1989-10-18 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual;
insert into LAUNCH
select 63099,'1989-Y248','1989-10-18 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 631,'1946-A14','1946-10-01 17:29:00','atmospheric rocket','success',875,'46-C','Dummy Sustainer','','','Test',1863,'',13 from dual union all
select 6310,'1959-Y27','1959-12-03 10:04:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 63100,'1989-Y249','1989-10-18 17:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 63101,'1989-Y25','1989-01-30 15:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 63102,'1989-Y250','1989-10-18 17:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',81 from dual union all
select 63103,'1989-Y251','1989-10-18 18:26:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',84 from dual union all
select 63104,'1989-Y252','1989-10-18 21:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 63105,'1989-Y253','1989-10-19 08:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',81 from dual union all
select 63106,'1989-Y254','1989-10-20 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 63107,'1989-Y255','1989-10-20 18:02:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',83 from dual union all
select 63108,'1989-Y256','1989-10-20 20:50:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',88 from dual union all
select 63109,'1989-Y257','1989-10-20 20:50:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',88 from dual union all
select 6311,'1959-Y28','1959-12-03 12:28:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 63110,'1989-Y258','1989-10-20 21:26:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 63111,'1989-Y259','1989-10-21 19:28:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',82 from dual union all
select 63112,'1989-Y26','1989-02-01 07:33:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',85 from dual union all
select 63113,'1989-Y260','1989-10-21 19:28:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',87 from dual union all
select 63114,'1989-Y261','1989-10-23 05:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',90 from dual union all
select 63115,'1989-Y262','1989-10-23 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 63116,'1989-Y263','1989-10-23 16:52:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',84 from dual union all
select 63117,'1989-Y264','1989-10-23 18:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1387,'SHIR',90 from dual union all
select 63118,'1989-Y265','1989-10-25 08:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',88 from dual union all
select 63119,'1989-Y266','1989-10-25 09:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',83 from dual union all
select 6312,'1959-Y29','1959-12-03 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 63120,'1989-Y267','1989-10-25 11:18:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 63121,'1989-Y268','1989-10-25 12:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 63122,'1989-Y269','1989-10-25 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 63123,'1989-Y27','1989-02-01 11:33:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 63124,'1989-Y270','1989-11-01 11:01:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',81 from dual union all
select 63125,'1989-Y271','1989-11-01 12:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 63126,'1989-Y272','1989-11-01 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',84 from dual union all
select 63127,'1989-Y273','1989-11-01 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 63128,'1989-Y274','1989-11-01 17:09:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 63129,'1989-Y275','1989-11-08 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',80 from dual union all
select 6313,'1959-Y30','1959-12-04 05:07:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 63130,'1989-Y276','1989-11-08 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',87 from dual union all
select 63131,'1989-Y277','1989-11-09 11:49:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 63132,'1989-Y278','1989-11-09 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 63133,'1989-Y279','1989-11-13 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 63134,'1989-Y28','1989-02-01 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 63135,'1989-Y280','1989-11-15 08:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',81 from dual union all
select 63136,'1989-Y281','1989-11-15 11:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 63137,'1989-Y282','1989-11-15 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 63138,'1989-Y283','1989-11-15 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 63139,'1989-Y284','1989-11-20 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',88 from dual union all
select 6314,'1959-Y31','1959-12-04 07:00:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 63140,'1989-Y285','1989-11-22 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',82 from dual union all
select 63141,'1989-Y286','1989-11-22 13:37:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 63142,'1989-Y287','1989-11-22 14:52:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',90 from dual union all
select 63143,'1989-Y288','1989-11-22 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 63144,'1989-Y289','1989-11-23 01:02:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 63145,'1989-Y29','1989-02-01 14:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 63146,'1989-Y290','1989-11-23 10:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',82 from dual union all
select 63147,'1989-Y291','1989-11-26 14:24:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 63148,'1989-Y292','1989-11-26 15:04:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',92 from dual union all
select 63149,'1989-Y293','1989-11-27 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 6315,'1959-Y32','1959-12-04 10:25:00','suborbital rocket','success',725,'','','','','Aeron',636,'',85 from dual union all
select 63150,'1989-Y294','1989-11-28 00:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',84 from dual union all
select 63151,'1989-Y295','1989-11-28 01:06:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',85 from dual union all
select 63152,'1989-Y296','1989-11-28 01:58:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 63153,'1989-Y297','1989-11-29 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 63154,'1989-Y298','1989-11-29 14:08:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 63155,'1989-Y299','1989-12-04 17:35:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',84 from dual union all
select 63156,'1989-Y30','1989-02-01 14:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 63157,'1989-Y300','1989-12-06 11:18:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 63158,'1989-Y301','1989-12-06 14:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 63159,'1989-Y302','1989-12-06 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 6316,'1959-Y33','1959-12-24 21:00:00','suborbital rocket','success',725,'','','','','Aeron',1010,'',85 from dual union all
select 63160,'1989-Y303','1989-12-13 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',82 from dual union all
select 63161,'1989-Y304','1989-12-13 13:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 63162,'1989-Y305','1989-12-13 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 63163,'1989-Y306','1989-12-13 14:35:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 63164,'1989-Y307','1989-12-13 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 63165,'1989-Y308','1989-12-13 15:26:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',85 from dual union all
select 63166,'1989-Y309','1989-12-15 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',83 from dual union all
select 63167,'1989-Y31','1989-02-01 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 63168,'1989-Y310','1989-12-20 09:18:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',82 from dual union all
select 63169,'1989-Y311','1989-12-20 11:18:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 6317,'1960 ALP','1960-03-11 13:00:07','deep space','success',1357,'219','Pioneer 5','P-2','','Sat',233,'',null from dual union all
select 63170,'1989-Y312','1989-12-20 16:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 63171,'1989-Y313','1989-12-27 10:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',83 from dual union all
select 63172,'1989-Y314','1989-12-27 13:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 63173,'1989-Y315','1989-12-27 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 63174,'1989-Y316','1989-12-27 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 63175,'1989-Y32','1989-02-01 20:00:00','suborbital rocket','unknown',728,'','','','','Aeron',896,'KORO',83 from dual union all
select 63176,'1989-Y33','1989-02-03 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 63177,'1989-Y34','1989-02-06 11:31:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 63178,'1989-Y35','1989-02-06 12:29:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 63179,'1989-Y36','1989-02-06 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 6318,'1960 BET','1960-04-01 11:40:09','orbital','success',1354,'148','Tiros 1','A-1','','Sat',233,'',null from dual union all
select 63180,'1989-Y37','1989-02-08 07:47:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',83 from dual union all
select 63181,'1989-Y38','1989-02-08 11:31:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 63182,'1989-Y39','1989-02-08 15:27:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 63183,'1989-Y40','1989-02-08 17:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 63184,'1989-Y41','1989-02-08 20:20:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 63185,'1989-Y42','1989-02-08 21:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 63186,'1989-Y43','1989-02-10 12:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',84 from dual union all
select 63187,'1989-Y44','1989-02-13 11:33:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 63188,'1989-Y45','1989-02-13 12:47:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual;
insert into LAUNCH
select 63189,'1989-Y46','1989-02-13 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 6319,'1960 DEL','1960-04-15 20:30:37','orbital','success',1360,'234/FTV 1055','Discoverer 11','CORONA C-8','RAM HORN','Sat TAG11',1679,'',null from dual union all
select 63190,'1989-Y47','1989-02-15 11:41:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',83 from dual union all
select 63191,'1989-Y48','1989-02-15 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 63192,'1989-Y49','1989-02-15 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',91 from dual union all
select 63193,'1989-Y50','1989-02-15 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 63194,'1989-Y51','1989-02-15 17:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 63195,'1989-Y52','1989-02-15 18:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 63196,'1989-Y53','1989-02-16 08:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',84 from dual union all
select 63197,'1989-Y54','1989-02-17 12:54:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 63198,'1989-Y55','1989-02-17 15:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 63199,'1989-Y56','1989-02-20 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 632,'1946-A15','1946-10-08 18:53:00','atmospheric rocket','success',875,'46-2','Nike-46 1R','','','Test',1863,'',42 from dual union all
select 6320,'1960 EPS','1960-05-15 00:00:05','orbital','success',1511,'L1-11','Korabl''-Sputnik','Vostok-1P','','Sat',480,'',null from dual union all
select 63200,'1989-Y57','1989-02-20 15:23:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 63201,'1989-Y58','1989-02-20 17:20:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 63202,'1989-Y59','1989-02-20 19:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 63203,'1989-Y60','1989-02-22 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 63204,'1989-Y61','1989-02-22 16:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 63205,'1989-Y62','1989-02-24 09:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',90 from dual union all
select 63206,'1989-Y63','1989-02-27 15:32:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',89 from dual union all
select 63207,'1989-Y64','1989-02-27 16:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 63208,'1989-Y65','1989-03-01 08:58:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',81 from dual union all
select 63209,'1989-Y66','1989-03-01 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 6321,'1960 ETA','1960-06-22 05:54:00','orbital','success',1359,'281/AB003','Transit 2A/GRAB','Transit 2A','','Sat',234,'',null from dual union all
select 63210,'1989-Y67','1989-03-01 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 63211,'1989-Y68','1989-03-01 16:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 63212,'1989-Y69','1989-03-02 14:17:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 63213,'1989-Y70','1989-03-03 08:25:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',95 from dual union all
select 63214,'1989-Y71','1989-03-03 19:03:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 63215,'1989-Y72','1989-03-07 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 63216,'1989-Y73','1989-03-08 09:40:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',83 from dual union all
select 63217,'1989-Y74','1989-03-09 14:15:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 63218,'1989-Y75','1989-03-13 11:03:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',95 from dual union all
select 63219,'1989-Y76','1989-03-13 11:54:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',90 from dual union all
select 6322,'1960 GAM','1960-04-13 12:02:36','orbital','success',1359,'257/AB002','Transit 1B','Transit 1B','','Sat',234,'',null from dual union all
select 63220,'1989-Y77','1989-03-15 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',80 from dual union all
select 63221,'1989-Y78','1989-03-16 09:31:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',81 from dual union all
select 63222,'1989-Y79','1989-03-17 11:20:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',81 from dual union all
select 63223,'1989-Y80','1989-03-17 13:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',89 from dual union all
select 63224,'1989-Y81','1989-03-18 09:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',87 from dual union all
select 63225,'1989-Y82','1989-03-18 09:56:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1057,'',84 from dual union all
select 63226,'1989-Y83','1989-03-18 14:34:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',86 from dual union all
select 63227,'1989-Y84','1989-03-22 09:46:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',81 from dual union all
select 63228,'1989-Y85','1989-03-22 13:54:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 63229,'1989-Y86','1989-03-29 08:57:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',85 from dual union all
select 6323,'1960 IOT','1960-08-12 09:39:43','orbital','success',1375,'270/D2','Echo 1','A-11','','Sat',233,'',null from dual union all
select 63230,'1989-Y87','1989-03-29 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',87 from dual union all
select 63231,'1989-Y88','1989-03-29 14:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',88 from dual union all
select 63232,'1989-Y89','1989-03-29 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',88 from dual union all
select 63233,'1989-Y90','1989-04-03 12:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 63234,'1989-Y91','1989-04-05 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',84 from dual union all
select 63235,'1989-Y92','1989-04-10 16:00:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 63236,'1989-Y93','1989-04-12 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 63237,'1989-Y94','1989-04-12 14:36:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 63238,'1989-Y95','1989-04-12 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 63239,'1989-Y96','1989-04-13 11:23:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',82 from dual union all
select 6324,'1960 KAP','1960-08-18 19:57:08','orbital','success',1360,'237/FTV 1056','Discoverer 14','CORONA C-9','LIMBER LEG','Sat TAG14',1678,'',null from dual union all
select 63240,'1989-Y97','1989-04-19 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 63241,'1989-Y98','1989-04-26 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 63242,'1989-Y99','1989-04-26 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 63243,'1990-001','1990-01-01 00:07:00','orbital','success',335,'CT-1','Skynet/JCSAT','Skynet 4A','','Sat',274,'',null from dual union all
select 63244,'1990-002','1990-01-09 12:35:00','orbital','success',1243,'STS-32R','LDEF Retrieve','OV-102 Medium dep/Rend','','Sat',921,'MLP3',null from dual union all
select 63245,'1990-003','1990-01-17 14:45:00','orbital','success',1238,'78047  -148','Kosmos-2055','Oblik','','Sat',474,'',null from dual union all
select 63246,'1990-004','1990-01-18 12:52:00','orbital','success',680,'65061-247','Kosmos-2056','Strela-2M','','Sat',460,'',null from dual union all
select 63247,'1990-005','1990-01-22 01:35:27','orbital','success',94,'V35 L408','SPOT 2/ASAP-1','SPOT 2','','Sat',345,'',null from dual union all
select 63248,'1990-006','1990-01-23 02:51:59','orbital','success',805,'77015  -527','Molniya-3','Molniya-3 No. 53','','Sat',475,'',null from dual union all
select 63249,'1990-007','1990-01-24 11:46:00','deep space','success',820,'M-3S2-5','Hiten/KM-M','MUSES-A','','Sat',837,'',null from dual union all
select 6325,'1960 LAM','1960-08-19 08:44:06','orbital','success',1511,'L1-12','Korabl''-Sputnik-2','Vostok-1 No. 2','','Sat',480,'',null from dual union all
select 63250,'1990-008','1990-01-24 22:55:01','orbital','success',408,'D191','USA 50 (GPS II-6)','Navstar GPS SVN 18','','Sat',233,'',null from dual union all
select 63251,'1990-009','1990-01-25 17:15:00','orbital','success',1238,'77059  -971','Kosmos-2057','Kobal''t','','Sat',462,'',null from dual union all
select 63252,'1990-010','1990-01-30 11:20:00','orbital','success',1448,'45095-604','Kosmos-2058','Tselina-R','','Sat',469,'',null from dual union all
select 63253,'1990-011','1990-02-04 12:27:03','orbital','success',308,'Y6       (21)','Zhongxing 3','DFH 2A','','Sat',1904,'',null from dual union all
select 63254,'1990-012','1990-02-06 16:30:00','orbital','success',680,'53711-237','Kosmos-2059','Duga-K','','Sat',457,'',null from dual union all
select 63255,'1990-013','1990-02-07 01:33:00','orbital','success',522,'H-21(F)','Momo-1b','Mos-1b','','Sat',1474,'',null from dual union all
select 63256,'1990-014','1990-02-11 06:16:00','orbital','success',1242,'T 15000-038','Soyuz TM-9','Soyuz 7K-STM No. 60','','Sat',480,'',null from dual union all
select 63257,'1990-015','1990-02-14 16:15:01','orbital','success',407,'D192','USA 51/52 (LACE/RME)','LACE','','Sat',234,'',null from dual union all
select 63258,'1990-016','1990-02-15 07:52:00','orbital','success',958,'363-02','Raduga','Gran'' No. 35L','','Sat',587,'',null from dual union all
select 63259,'1990-017','1990-02-27 20:59:42','orbital','success',680,'53711-228','Nadezhda','Nadezhda 17F118 No. 405','','Sat',457,'',null from dual union all
select 6326,'1960 MU','1960-09-13 22:13:39','orbital','success',1360,'246/FTV 1058','Discoverer 15','CORONA C-10','COFFEE CALL','Sat TAG15',1679,'',null from dual union all
select 63260,'1990-018','1990-02-28 00:55:00','orbital','success',1448,'45027-909','Okean','Okean-O1 No. 5/NKhM No.','','Sat',470,'',null from dual union all
select 63261,'1990-019','1990-02-28 07:50:22','orbital','success',1243,'STS-36R','USA 53       [MST1]','OV-104 LEO deploy','','Sat',921,'MLP1',null from dual union all
select 63262,'1990-020','1990-02-28 23:10:57','orbital','success',1242,'T 15000-040','Progress M-3','Progress 7K-TGM No. 203','','Sat',480,'',null from dual union all
select 63263,'1990-021','1990-03-14 11:52:31','orbital','success',335,'CT-2','Intelsat 603','INTELSAT VI F-3','','Sat',274,'',null from dual union all
select 63264,'1990-022','1990-03-14 15:27:00','orbital','success',1446,'67076-103','Kosmos-2060','US-PM','','Sat',589,'',null from dual union all
select 63265,'1990-023','1990-03-20 00:25:22','orbital','success',680,'47161-245','Kosmos-2061','Parus','','Sat',460,'',null from dual union all
select 63266,'1990-024','1990-03-22 07:20:00','orbital','success',1238,'78039  -385','Kosmos-2062','Oblik','','Sat',475,'',null from dual union all
select 63267,'1990-025','1990-03-26 02:45:01','orbital','success',408,'D193','USA 54 (GPS II-7)','Navstar GPS SVN 20','','Sat',233,'',null from dual union all
select 63268,'1990-026','1990-03-27 16:40:08','orbital','success',803,'77059  -566','Kosmos-2063','Oko','','Sat',474,'',null from dual union all
select 63269,'1990-027','1990-04-03 12:02:00','orbital','success',1173,'2','''Ofeq-2','''Ofeq-2','','Sat',1026,'',null from dual union all
select 6327,'1960 NU','1960-10-04 17:50:00','orbital','success',1359,'293/AB005','Courier 1B','Courier 1B','','Sat',234,'',null from dual union all
select 63270,'1990-028','1990-04-05 19:10:17','orbital','success',909,'001/F1','Pegsat/USA 55 GLOMR','Pegsat','','Sat',379,'NB-52 008',null from dual union all
select 63271,'1990-029','1990-04-06 03:13:00','orbital','success',680,'65061-259','Kosmos-2064/2071','Strela-1M','','Sat',460,'',null from dual union all
select 63272,'1990-030','1990-04-07 13:30:02','orbital','success',308,'Y7       (22)','Asiasat 1','Asiasat 1','','Sat',1904,'',null from dual union all
select 63273,'1990-031','1990-04-11 15:00:00','orbital','success',156,'28E','USA 56-58 (P87-2)','POGS','','Sat',1776,'',null from dual union all
select 63274,'1990-032','1990-04-11 17:00:00','orbital','success',1238,'G 15000-590','Foton','Foton No. 6L','','Sat',474,'',null from dual union all
select 63275,'1990-033','1990-04-13 18:53:16','orbital','success',1238,'76035  -714','Kosmos-2072','Terilen','','Sat',480,'',null from dual union all
select 63276,'1990-034','1990-04-13 22:28:00','orbital','success',409,'D194','Palapa B2R','Palapa B2R','','Sat',234,'',null from dual union all
select 63277,'1990-035','1990-04-17 08:00:00','orbital','success',1238,'77053  -814','Kosmos-2073','Oblik','','Sat',475,'',null from dual union all
select 63278,'1990-036','1990-04-20 18:41:08','orbital','success',680,'53726-195','Kosmos-2074','Parus','','Sat',460,'',null from dual;
insert into LAUNCH
select 63279,'1990-037','1990-04-24 12:33:52','orbital','success',1243,'STS-31R','HST','OV-103 HST mission','','Sat',922,'MLP2',null from dual union all
select 6328,'1960 OMI','1960-11-12 20:42:32','orbital','success',1361,'297/FTV 1062','Discoverer 17','CORONA C''-2','BOXING GLOVE','Sat TAG17',1679,'',null from dual union all
select 63280,'1990-038','1990-04-25 13:00:00','orbital','success',680,'65061-253','Kosmos-2075','Romb','','Sat',457,'',null from dual union all
select 63281,'1990-039','1990-04-26 01:37:00','orbital','success',805,'77015  -525','Molniya-1','Molniya-1 No. 71','','Sat',475,'',null from dual union all
select 63282,'1990-040','1990-04-28 11:37:02','orbital','success',803,'76047  -570','Kosmos-2076','Oko','','Sat',462,'',null from dual union all
select 63283,'1990-041','1990-05-05 20:44:01','orbital','success',1242,'T 15000-041','Progress-42','Progress 7K-TG No. 150','','Sat',480,'',null from dual union all
select 63284,'1990-042','1990-05-07 18:39:59','orbital','success',1238,'77059  -973','Kosmos-2077','Kobal''t','','Sat',462,'',null from dual union all
select 63285,'1990-043','1990-05-09 17:50:00','orbital','success',1135,'S212C','MACSAT 1/2','MACSAT 1','','Sat',1779,'',null from dual union all
select 63286,'1990-044','1990-05-15 09:55:00','orbital','success',1238,'77059  -472','Kosmos-2078','Kometa No. 12','','Sat',480,'',null from dual union all
select 63287,'1990-045','1990-05-19 08:32:33','orbital','success',958,'350-01','Kosmos-2079/2081','Uragan No. 46L','','Sat',554,'',null from dual union all
select 63288,'1990-046','1990-05-22 05:14:02','orbital','success',1537,'67065-613','Kosmos-2082','Tselina-2','','Sat',568,'',null from dual union all
select 63289,'1990-047','1990-05-29 07:19:59','orbital','success',1238,'77053  -149','Resurs-F','Resurs-F1 14F43 No. 50','','Sat',475,'',null from dual union all
select 6329,'1960 PI','1960-11-23 11:13:03','orbital','success',1375,'245/D3','Tiros 2','NASA A-2','','Sat',233,'',null from dual union all
select 63290,'1990-048','1990-05-31 10:33:20','orbital','success',949,'360-01','Kristall','TsM-T 77KST No. 17201','','Sat',553,'',null from dual union all
select 63291,'1990-049','1990-06-01 21:48:00','orbital','success',406,'D195','ROSAT','ROSAT','','Sat',233,'',null from dual union all
select 63292,'1990-050','1990-06-08 05:21:41','orbital','success',1417,'K-4    (45H-4)','USA 59-62   [NOSS10]','SLDCOM 1/INTRUDER 1','','Sat',275,'',null from dual union all
select 63293,'1990-051','1990-06-12 05:52:00','orbital','success',395,'D196','Insat 1D','Insat 1D','','Sat',234,'',null from dual union all
select 63294,'1990-052','1990-06-13 01:07:00','orbital','success',805,'77015  -530','Molniya-3','Molniya-3 No. 47','','Sat',474,'',null from dual union all
select 63295,'1990-053','1990-06-19 08:45:00','orbital','success',1238,'77053  -815','Kosmos-2083','Oblik','','Sat',462,'',null from dual union all
select 63296,'1990-054','1990-06-20 23:36:00','orbital','success',956,'342-02','Gorizont','Gorizont No. 30L','','Sat',554,'',null from dual union all
select 63297,'1990-055','1990-06-21 20:45:52','orbital','failure',803,'78039  -560','Kosmos-2084','Oko','','Sat',474,'',null from dual union all
select 63298,'1990-056','1990-06-23 11:19:00','orbital','success',335,'CT-3','Intelsat 604','INTELSAT VI F-4','','Sat',274,'',null from dual union all
select 63299,'1990-057','1990-06-27 22:30:00','orbital','success',1448,'51095-605','Meteor-2','Meteor-2 No. 23','','Sat',469,'',null from dual union all
select 633,'1946-A16','1946-10-11 16:50:00','atmospheric rocket','success',875,'46-3','Nike-46 2R','','','Test',1863,'',33 from dual union all
select 6330,'1960 RHO','1960-12-01 07:30:04','orbital','success',1511,'L1-13','Korabl''-Sputnik-3','Vostok-1 No. 3','','Sat',480,'',null from dual union all
select 63300,'1990-058','1990-07-11 10:00:00','orbital','success',1242,'G 15000-035','Gamma','Gamma No. 1L','','Sat',480,'',null from dual union all
select 63301,'1990-059','1990-07-16 00:40:00','orbital','success',305,'Y1       (23)','HS-601 Model','BADR','','Sat',1903,'',null from dual union all
select 63302,'1990-060','1990-07-17 09:29:59','orbital','success',1241,'78023  -265','Resurs-F','Resurs-F2 17F42 No. 5','','Sat',474,'',null from dual union all
select 63303,'1990-061','1990-07-18 21:46:00','orbital','success',958,'340-01','Kosmos-2085','Geizer No. 17L','','Sat',553,'',null from dual union all
select 63304,'1990-062','1990-07-20 08:40:00','orbital','success',1238,'78023  -816','Kosmos-2086','Oblik','','Sat',474,'',null from dual union all
select 63305,'1990-063','1990-07-24 22:25:00','orbital','success',103,'V37 L409','TDF 2/DFS 2','TDF 2','','Sat',345,'',null from dual union all
select 63306,'1990-064','1990-07-25 18:13:56','orbital','success',810,'77017  -572','Kosmos-2087','Oko','','Sat',462,'',null from dual union all
select 63307,'1990-065','1990-07-25 19:21:00','orbital','success',171,'AC-69','CRRES','CRRES','','Sat',269,'',null from dual union all
select 63308,'1990-066','1990-07-30 00:06:00','orbital','success',1448,'45079-204','Kosmos-2088','Musson No. 22','','Sat',469,'',null from dual union all
select 63309,'1990-067','1990-08-01 09:32:21','orbital','success',1242,'G 15000-043','Soyuz TM-10','Soyuz 7K-STM No. 61A','','Sat',480,'',null from dual union all
select 6331,'1960 SIG','1960-12-07 20:20:58','orbital','success',1361,'296/FTV 1103','Discoverer 18','CORONA C''-3','POWER TRACTOR','Sat TAG18',1678,'',null from dual union all
select 63310,'1990-068','1990-08-02 05:39:00','orbital','success',408,'D197','USA 63 (GPS II-8)','Navstar GPS SVN 21','','Sat',233,'',null from dual union all
select 63311,'1990-069','1990-08-03 19:45:00','orbital','success',1241,'76047  -976','Kosmos-2089','Kobal''t','','Sat',462,'',null from dual union all
select 63312,'1990-070','1990-08-08 04:15:07','orbital','success',1448,'67079-203','Kosmos-2090/2095','Strela-3','','Sat',470,'',null from dual union all
select 63313,'1990-071','1990-08-10 20:18:58','orbital','success',805,'77011  -693','Molniya-1','Molniya-1T No. 68','','Sat',475,'',null from dual union all
select 63314,'1990-072','1990-08-15 04:00:41','orbital','success',1242,'G 15000-042','Progress M-4','Progress 7K-TGM No. 204','','Sat',480,'',null from dual union all
select 63315,'1990-073','1990-08-16 09:54:59','orbital','success',1241,'77053  -150','Resurs-F','Resurs-F1 14F43 No. 49','','Sat',475,'',null from dual union all
select 63316,'1990-074','1990-08-18 00:42:00','orbital','success',408,'D198','Marcopolo 2','BSB R2','','Sat',234,'',null from dual union all
select 63317,'1990-075','1990-08-23 16:17:00','orbital','success',1446,'45076-104','Kosmos-2096','US-PM','','Sat',589,'',null from dual union all
select 63318,'1990-076','1990-08-28 07:49:13','orbital','success',810,'77017  -575','Kosmos-2097','Oko','','Sat',475,'',null from dual union all
select 63319,'1990-077','1990-08-28 09:05:00','orbital','success',524,'H-22(F)','BS 3A','BS 3A','','Sat',1474,'',null from dual union all
select 6332,'1960 TAU','1960-12-20 20:32:00','orbital','success',1361,'258/FTV 1101','Discoverer 19','CORONA RM-1','TEE BIRD','Sat TAG19',1679,'',null from dual union all
select 63320,'1990-078','1990-08-28 15:45:00','orbital','success',680,'53711-240','Kosmos-2098','Vektor','','Sat',460,'',null from dual union all
select 63321,'1990-079','1990-08-30 22:46:00','orbital','success',106,'V38 L410b','Eut. 2F1/Skynet 4C','Skynet 4C','','Sat',345,'',null from dual union all
select 63322,'1990-080','1990-08-31 08:00:00','orbital','success',1238,'77053  -811','Kosmos-2099','Oblik','','Sat',475,'',null from dual union all
select 63323,'1990-081','1990-09-03 00:53:00','orbital','success',318,'Y2       (24)','Feng Yun 1','feng yun yi yi','','Sat',1492,'',null from dual union all
select 63324,'1990-082','1990-09-07 11:59:59','orbital','success',1241,'78023  -151','Resurs-F','Resurs-F1 14F43 No. 51','','Sat',462,'',null from dual union all
select 63325,'1990-083','1990-09-14 05:59:07','orbital','success',680,'65048-403','Kosmos-2100','Parus','','Sat',460,'',null from dual union all
select 63326,'1990-084','1990-09-20 20:16:59','orbital','success',805,'77059  -562','Molniya-3','Molniya-3 No. 54','','Sat',475,'',null from dual union all
select 63327,'1990-085','1990-09-27 10:37:42','orbital','success',1242,'G 15000-044','Progress M-5','Progress 7K-TGM No. 206','','Sat',480,'',null from dual union all
select 63328,'1990-086','1990-09-28 07:30:00','orbital','success',1448,'51065-611','Meteor-2','Meteor-2 No. 25','','Sat',469,'',null from dual union all
select 63329,'1990-087','1990-10-01 11:00:00','orbital','success',1242,'G 15000-047','Kosmos-2101','Orlets','','Sat',480,'',null from dual union all
select 6333,'1960 THE','1960-08-10 20:37:54','orbital','success',1360,'231/FTV 1057','Discoverer 13','CORONA Diagnostic 2','FROGGY BOTTOM','Sat TAG13',1679,'',null from dual union all
select 63330,'1990-088','1990-10-01 21:56:00','orbital','success',408,'D199','USA 64 (GPS II-9)','Navstar GPS SVN 15','','Sat',233,'',null from dual union all
select 63331,'1990-089','1990-10-05 06:14:00','orbital','success',301,'Y9       (25)','FSW-1','FSW-1 No. 3','','Sat',810,'',null from dual union all
select 63332,'1990-090','1990-10-06 11:47:16','deep space','success',1243,'STS-41','Ulysses','OV-103 IUS deploy','','Sat',922,'MLP2',null from dual union all
select 63333,'1990-091','1990-10-12 22:58:18','orbital','success',103,'V39 L411b','SBS 6/Galaxy 6','SBS 6','','Sat',345,'',null from dual union all
select 63334,'1990-092','1990-10-16 19:00:00','orbital','success',1238,'76047  -977','Kosmos-2102','Kobal''t','','Sat',462,'',null from dual union all
select 63335,'1990-093','1990-10-30 23:16:00','orbital','success',408,'D200','Inmarsat II F-1','INMARSAT II F-1','','Sat',234,'',null from dual union all
select 63336,'1990-094','1990-11-03 14:40:00','orbital','success',958,'370-01','Gorizont','Gorizont No. 32L','','Sat',587,'',null from dual union all
select 63337,'1990-095','1990-11-13 00:37:02','orbital','success',1411,'K-6    (45D-2)','USA 65       [DSP15]','DSP 15','','Sat',275,'',null from dual union all
select 63338,'1990-096','1990-11-14 06:33:40','orbital','success',1446,'67061-503','Kosmos-2103','US-PM','','Sat',589,'',null from dual union all
select 63339,'1990-097','1990-11-15 23:48:15','orbital','success',1243,'STS-38','USA 67   (QUASAR  9)','OV-104 Medium dep','','Sat',921,'MLP1',null from dual union all
select 6334,'1960 XI','1960-11-03 05:23:10','orbital','success',634,'AM-19D','Explorer 8','NASA S-30','','Sat',249,'',null from dual union all
select 63340,'1990-098','1990-11-16 16:30:00','orbital','success',1238,'77053  -812','Kosmos-2104','Oblik','','Sat',475,'',null from dual union all
select 63341,'1990-099','1990-11-20 02:33:14','orbital','success',810,'76047  -568','Kosmos-2105','Oko','','Sat',462,'',null from dual union all
select 63342,'1990-100','1990-11-20 23:11:00','orbital','success',100,'V40 L412b','Satcom C1/Gstar 4','Satcom C1','','Sat',345,'',null from dual union all
select 63343,'1990-101','1990-11-23 03:51:00','orbital','success',805,'77015  -529','Molniya-1','Molniya-1T No. 70','','Sat',474,'',null from dual union all
select 63344,'1990-102','1990-11-23 13:22:00','orbital','success',958,'348-02','Gorizont','Gorizont No. 33L','','Sat',553,'',null from dual union all
select 63345,'1990-103','1990-11-26 21:39:01','orbital','success',422,'D201','USA 66 (GPS IIA-10)','Navstar GPS SVN 23','','Sat',233,'',null from dual union all
select 63346,'1990-104','1990-11-28 16:33:58','orbital','success',1448,'45013-302','Kosmos-2106','Kol''tso','','Sat',470,'',null from dual union all
select 63347,'1990-105','1990-12-01 15:57:00','orbital','success',155,'61E','USA 68 (DMSP F10)','DMSP Block 5D-2 S-10','','Sat',1776,'',null from dual union all
select 63348,'1990-106','1990-12-02 06:49:00','orbital','success',1243,'STS-35R','Spacelab Astro 1','OV-102 Spacelab','','Sat',922,'MLP3',null from dual union all
select 63349,'1990-107','1990-12-02 08:13:32','orbital','success',1242,'T 15000-046','Soyuz TM-11','Soyuz 7K-STM No. 61','','Sat',480,'',null from dual union all
select 6335,'1960 ZET','1960-05-24 17:36:46','orbital','success',144,'45D','Midas 2','Midas ETS-I F2','','Sat',229,'',null from dual union all
select 63350,'1990-108','1990-12-04 00:48:31','orbital','success',1446,'45061-501','Kosmos-2107','US-PM','','Sat',589,'',null from dual union all
select 63351,'1990-109','1990-12-04 18:30:00','orbital','success',1238,'76047  -978','Kosmos-2108','Kobal''t','','Sat',475,'',null from dual union all
select 63352,'1990-110','1990-12-08 02:43:00','orbital','success',958,'366-02','Kosmos-2109/2111','Uragan No. 47L','','Sat',554,'',null from dual union all
select 63353,'1990-111','1990-12-10 07:54:59','orbital','success',680,'47126-203','Kosmos-2112','Strela-2M','','Sat',460,'',null from dual union all
select 63354,'1990-112','1990-12-20 11:35:00','orbital','success',958,'361-01','Raduga','Gran'' No. 37L','','Sat',587,'',null from dual union all
select 63355,'1990-113','1990-12-21 06:20:11','orbital','success',1238,'77017  -739','Kosmos-2113','Terilen','','Sat',480,'',null from dual union all
select 63356,'1990-114','1990-12-22 07:28:53','orbital','success',1448,'67065-609','Kosmos-2114/2119','Strela-3','','Sat',470,'',null from dual union all
select 63357,'1990-115','1990-12-26 11:10:00','orbital','success',1241,'77053  -813','Kosmos-2120','Oblik','','Sat',462,'',null from dual union all
select 63358,'1990-116','1990-12-27 11:08:00','orbital','success',958,'342-01','Raduga-1','Globus No. 12','','Sat',553,'',null from dual union all
select 63359,'1990-A01','1990-01-26 16:38:00','military missile','success',539,'','KITE-1','','','Test',1883,'',10 from dual union all
select 6336,'1960-A01','1960-10-13 00:00:00','atmospheric rocket','success',1065,'R1J','','','','Test',1832,'',20 from dual union all
select 63360,'1990-A02','1990-02-26 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM13','','','Aeron',1010,'',40 from dual union all
select 63361,'1990-A03','1990-07-26 05:51:00','suborbital rocket','failure',117,'SPEAR-2','','','','Tech',1873,'',2 from dual union all
select 63362,'1990-A04','1990-08-09 00:00:00','military missile','failure',118,'','','','','Test',1024,'SHIP-ISR',0 from dual union all
select 63363,'1990-A05','1990-08-22 09:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',46 from dual union all
select 63364,'1990-A06','1990-09-19 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',43 from dual union all
select 63365,'1990-A07','1990-10-05 13:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',38 from dual union all
select 63366,'1990-A08','1990-11-02 13:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',42 from dual union all
select 63367,'1990-A09','1990-06-22 00:00:00','suborbital rocket','success',606,'','','','','Test',768,'',34 from dual union all
select 63368,'1990-A10','1990-07-16 00:00:00','suborbital rocket','success',606,'','','','','Test',768,'',33 from dual;
insert into LAUNCH
select 63369,'1990-A11','1990-02-14 20:26:00','suborbital rocket','unknown',1502,'Chaff','DYANA DAC38','','','Aeron',29,'',0 from dual union all
select 6337,'1960-A02','1960-01-01 00:00:00','military missile','success',1469,'','LKI','','','Test',182,'',20 from dual union all
select 63370,'1990-A12','1990-02-28 20:46:00','suborbital rocket','unknown',1502,'Sphere','DYANA DAS54','','','Aeron',29,'',0 from dual union all
select 63371,'1990-A13','1990-03-01 20:15:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD56','','','Aeron',29,'',0 from dual union all
select 63372,'1990-A14','1990-03-11 20:10:00','suborbital rocket','failure',1502,'Sphere','DYANA DAS82','','','Aeron',29,'',6 from dual union all
select 63373,'1990-A15','1990-06-15 00:00:00','military missile','success',684,'3      -','15Zh59 B-3','Mobile launch test','','Test',477,'',1 from dual union all
select 63374,'1990-F01','1990-02-22 23:17:00','orbital','failure',103,'V36 L407','Superbird B/BS 2X','Superbird B','','Sat',345,'',9 from dual union all
select 63375,'1990-F02','1990-04-03 18:00:00','orbital','failure',1238,'76047  -975','','Kobal''t No. 527','','Sat',475,'',195 from dual union all
select 63376,'1990-F03','1990-07-03 19:19:58','orbital','failure',1238,'77015  -874','','Kobal''t No. 531','','Sat',462,'',70 from dual union all
select 63377,'1990-F04','1990-08-09 20:18:59','orbital','failure',958,'345-02','','Ekran-M No. 14L','','Sat',553,'',58 from dual union all
select 63378,'1990-F05','1990-10-04 04:28:00','orbital','failure',1537,'45079-216/15L','','Tselina-2','','Sat',568,'',0 from dual union all
select 63379,'1990-M01','1990-01-29 13:00:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',76 from dual union all
select 6338,'1960-A03','1960-01-01 00:00:00','military missile','success',1469,'','LKI','','','Test',182,'',20 from dual union all
select 63380,'1990-M02','1990-01-31 13:10:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',78 from dual union all
select 63381,'1990-M03','1990-02-02 13:00:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',75 from dual union all
select 63382,'1990-M04','1990-02-05 13:00:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',78 from dual union all
select 63383,'1990-M05','1990-02-07 13:20:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',52 from dual union all
select 63384,'1990-M06','1990-02-09 13:00:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',73 from dual union all
select 63385,'1990-M07','1990-02-12 13:07:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',73 from dual union all
select 63386,'1990-M08','1990-02-14 13:05:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',77 from dual union all
select 63387,'1990-M09','1990-02-16 13:23:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',77 from dual union all
select 63388,'1990-M10','1990-02-19 13:08:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',57 from dual union all
select 63389,'1990-M11','1990-02-23 13:50:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',77 from dual union all
select 6339,'1960-A04','1960-01-01 00:00:00','military missile','success',1469,'','LKI','','','Test',182,'',20 from dual union all
select 63390,'1990-M12','1990-02-28 13:05:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',77 from dual union all
select 63391,'1990-M13','1990-03-02 13:05:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',72 from dual union all
select 63392,'1990-M14','1990-03-05 13:05:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',76 from dual union all
select 63393,'1990-M15','1990-03-07 13:00:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',79 from dual union all
select 63394,'1990-M16','1990-03-09 13:15:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',77 from dual union all
select 63395,'1990-M17','1990-09-24 00:00:00','suborbital rocket','success',606,'','','','','Test',768,'',62 from dual union all
select 63396,'1990-M18','1990-01-03 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63397,'1990-M19','1990-01-10 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63398,'1990-M20','1990-01-17 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63399,'1990-M21','1990-01-24 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 634,'1946-A17','1946-10-15 21:23:00','atmospheric rocket','failure',875,'46-5','Nike-46 3R','','','Test',1863,'',18 from dual union all
select 6340,'1960-A05','1960-01-01 00:00:00','military missile','success',1469,'','LKI','','','Test',182,'',20 from dual union all
select 63400,'1990-M22','1990-03-14 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63401,'1990-M23','1990-03-21 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63402,'1990-M24','1990-03-29 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63403,'1990-M25','1990-04-04 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63404,'1990-M26','1990-04-12 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63405,'1990-M27','1990-04-18 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63406,'1990-M28','1990-04-25 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63407,'1990-M29','1990-05-02 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63408,'1990-M30','1990-05-10 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63409,'1990-M31','1990-05-16 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6341,'1960-A06','1960-01-01 00:00:00','military missile','success',1469,'','LKI','','','Test',182,'',20 from dual union all
select 63410,'1990-M32','1990-05-23 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63411,'1990-M33','1990-05-30 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63412,'1990-M34','1990-06-06 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63413,'1990-M35','1990-06-13 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63414,'1990-M36','1990-06-20 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63415,'1990-M37','1990-06-27 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63416,'1990-M38','1990-07-05 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63417,'1990-M39','1990-07-11 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63418,'1990-M40','1990-07-18 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63419,'1990-M41','1990-07-25 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 6342,'1960-A07','1960-01-01 00:00:00','military missile','success',1469,'','LKI','','','Test',182,'',20 from dual union all
select 63420,'1990-M42','1990-08-01 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63421,'1990-M43','1990-08-08 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',320,'',70 from dual union all
select 63422,'1990-S01','1990-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',150 from dual union all
select 63423,'1990-S02','1990-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',150 from dual union all
select 63424,'1990-S03','1990-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',150 from dual union all
select 63425,'1990-S04','1990-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',150 from dual union all
select 63426,'1990-S05','1990-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',150 from dual union all
select 63427,'1990-S06','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63428,'1990-S07','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63429,'1990-S08','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 6343,'1960-A08','1960-01-01 00:00:00','military missile','success',1469,'','LKI','','','Test',182,'',20 from dual union all
select 63430,'1990-S09','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63431,'1990-S10','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63432,'1990-S100','1990-06-01 14:25:00','suborbital rocket','success',224,'NASA 36.69UL','','','','Aeron',1873,'',300 from dual union all
select 63433,'1990-S101','1990-06-12 03:30:00','military missile','success',1005,'51013-713-03T','15A18M K-4 ZP','GCh 15F173','','OT',512,'',1000 from dual union all
select 63434,'1990-S102','1990-06-26 00:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT140GB','GLORY TRIP 140GB','OT',1712,'',1300 from dual union all
select 63435,'1990-S103','1990-07-12 00:00:00','suborbital rocket','success',228,'','','','','Test?',304,'',250 from dual union all
select 63436,'1990-S104','1990-07-16 07:00:00','suborbital rocket','success',211,'NASA 27.124UG','SISSR','','','UV Astron',1895,'',340 from dual union all
select 63437,'1990-S105','1990-07-26 23:05:00','suborbital rocket','success',1210,'DLR K-GRVI-181','DLR SISSI 2','','','Aeron',416,'',168 from dual union all
select 63438,'1990-S106','1990-07-30 09:48:44','suborbital rocket','success',1337,'NASA 29.28UE','EQUIS Spread F','','','Plasma',871,'',480 from dual union all
select 63439,'1990-S107','1990-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 6344,'1960-A09','1960-01-01 00:00:00','military missile','success',1469,'','LKI','','','Test',182,'',20 from dual union all
select 63440,'1990-S108','1990-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63441,'1990-S109','1990-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63442,'1990-S11','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63443,'1990-S110','1990-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63444,'1990-S111','1990-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63445,'1990-S112','1990-07-31 12:55:00','military missile','success',1434,'','15Zh58 UBP-9','','','Test',477,'',1000 from dual union all
select 63446,'1990-S113','1990-08-02 10:07:09','suborbital rocket','success',1337,'NASA 29.29UE','EQUIS Spread F','','','Plasma',871,'',480 from dual union all
select 63447,'1990-S114','1990-08-02 22:40:00','suborbital rocket','success',1212,'DLR K-GR-182','DLR SISSI 3','','','Aeron',416,'',239 from dual union all
select 63448,'1990-S115','1990-08-03 11:50:00','military missile','success',915,'1L','15Zh75 LKI-1','','','Test',477,'',1000 from dual union all
select 63449,'1990-S116','1990-08-11 08:10:00','suborbital rocket','success',1318,'NASA 38.17UE','CRRES AA-5','','','Plasma',871,'',375 from dual union all
select 6345,'1960-A10','1960-01-01 00:00:00','military missile','success',1469,'','LKI','','','Test',182,'',20 from dual union all
select 63450,'1990-S117','1990-08-15 00:00:00','military missile','success',73,'','','','','Weapon',758,'',100 from dual union all
select 63451,'1990-S118','1990-08-15 08:29:00','suborbital rocket','success',1318,'NASA 38.15UE','CRRES AA-6','','','Plasma',871,'',428 from dual union all
select 63452,'1990-S119','1990-08-15 08:43:56','suborbital rocket','success',1318,'NASA 38.16UE','CRRES','','','Plasma',871,'',440 from dual union all
select 63453,'1990-S12','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63454,'1990-S120','1990-08-16 07:20:00','military missile','success',1434,'','15Zh58 K-25','','','Test',477,'',1000 from dual union all
select 63455,'1990-S121','1990-08-17 10:00:00','military missile','success',1006,'51086-811-05','15A18 UBP-29 K','GCh 15F183','','Test',489,'',1000 from dual union all
select 63456,'1990-S122','1990-08-22 08:16:00','suborbital rocket','success',224,'NASA 36.56DE','NICARE 2','','','Plasma',871,'',350 from dual union all
select 63457,'1990-S123','1990-08-22 08:40:10','suborbital rocket','success',1318,'NASA 38.18UE','CRRES','','','Plasma',871,'',440 from dual union all
select 63458,'1990-S124','1990-08-25 07:04:00','suborbital rocket','success',209,'SDI BP-1','SPFE 1 (LACE Target)','','','Tech',1821,'',200 from dual;
insert into LAUNCH
select 63459,'1990-S125','1990-08-29 05:00:00','military missile','success',1005,'45057-903(34L)','15A18M LKI-24','GCh 15F177(174x6,178x4)','','OT',487,'',1000 from dual union all
select 6346,'1960-A100','1960-04-18 23:09:00','ballistic missile test','failure',1137,'SX-1','Cub Scout','','','Test',1812,'',48 from dual union all
select 63460,'1990-S126','1990-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63461,'1990-S127','1990-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63462,'1990-S128','1990-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63463,'1990-S129','1990-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63464,'1990-S13','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63465,'1990-S130','1990-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63466,'1990-S131','1990-09-04 19:04:00','suborbital rocket','success',224,'NASA 36.72UL','EUV Spectrometer','','','Aeron/Solar EUV',1873,'',326 from dual union all
select 63467,'1990-S132','1990-09-13 00:00:00','military missile','success',907,'04PA (85-163)','','GT 04PA','','OT',1700,'',1000 from dual union all
select 63468,'1990-S133','1990-09-19 08:40:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT141GM/AOA Target','GLORY TRIP 141GM','OT',1705,'',1300 from dual union all
select 63469,'1990-S134','1990-09-26 15:55:00','military missile','success',1443,'','DASO-4','','','OT',420,'SSBN 735',1000 from dual union all
select 6347,'1960-A101','1960-04-19 16:51:44','suborbital spaceplane','success',1520,'1-5-10','','','','Test',379,'NB-52 003',18 from dual union all
select 63470,'1990-S135','1990-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63471,'1990-S136','1990-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63472,'1990-S137','1990-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63473,'1990-S138','1990-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63474,'1990-S139','1990-10-12 20:28:38','suborbital rocket','success',228,'NASA 21.102IE','NEED II (F-80)','','','Auroral',29,'',304 from dual union all
select 63475,'1990-S14','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63476,'1990-S140','1990-10-15 00:00:00','suborbital rocket','success',228,'','','','','Test?',304,'',250 from dual union all
select 63477,'1990-S141','1990-10-17 13:50:00','military missile','success',915,'2L','15Zh75 LKI-2','','','Test',477,'',1000 from dual union all
select 63478,'1990-S142','1990-10-20 00:00:00','suborbital rocket','success',1337,'SDIO','Firefly 2','','','Target',1821,'',456 from dual union all
select 63479,'1990-S143','1990-11-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 6348,'1960-A102','1960-04-26 00:00:00','suborbital rocket','failure',546,'','Porous RV','','','Reentry Test',1821,'',43 from dual union all
select 63480,'1990-S144','1990-11-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63481,'1990-S145','1990-11-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63482,'1990-S146','1990-11-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63483,'1990-S147','1990-11-01 14:30:00','military missile','success',1434,'','15Zh58 UBP-10','','','Test',477,'',1000 from dual union all
select 63484,'1990-S148','1990-11-03 11:53:00','military missile','success',1443,'','CET','','','OT',420,'SSBN 734',1000 from dual union all
select 63485,'1990-S149','1990-11-03 12:38:11','military missile','success',1443,'','CET','','','OT',420,'SSBN 734',1000 from dual union all
select 63486,'1990-S15','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63487,'1990-S150','1990-11-03 13:37:00','military missile','success',1443,'','CET','','','OT',420,'SSBN 734',1000 from dual union all
select 63488,'1990-S151','1990-11-03 13:37:00','military missile','success',1443,'','CET','','','OT',420,'SSBN 734',1000 from dual union all
select 63489,'1990-S152','1990-11-06 00:00:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT142GM','GLORY TRIP 142GM','OT',1699,'',1300 from dual union all
select 6349,'1960-A103','1960-04-27 00:00:00','military missile','success',764,'58-1408','','','','Test',243,'',12 from dual union all
select 63490,'1990-S153','1990-11-15 07:05:00','suborbital rocket','success',1212,'DLR K-GR-183','DLR TEXUS 27','','','Micrograv',416,'',249 from dual union all
select 63491,'1990-S154','1990-11-19 00:00:00','military missile','success',1034,'3','','','','Test',1014,'',300 from dual union all
select 63492,'1990-S155','1990-11-20 04:02:00','test rocket','success',1063,'No. 1L/511-1992-002','Briz test','Naryad-V GVM?','','Test/ASAT?',505,'',900 from dual union all
select 63493,'1990-S156','1990-11-21 18:15:00','suborbital rocket','success',224,'NASA 36.058DS','AS&E SIAP XRT','','','Solar',1873,'',300 from dual union all
select 63494,'1990-S157','1990-11-21 18:40:00','suborbital rocket','success',224,'NASA 36.060DS','HRTS 7','','','Solar',1873,'',300 from dual union all
select 63495,'1990-S158','1990-11-25 05:27:00','suborbital rocket','success',1202,'DLR K-GRC-184','SSC Maxus Test','','','Test',416,'',534 from dual union all
select 63496,'1990-S159','1990-11-26 00:00:00','suborbital rocket','success',1216,'XV-54','Manival','','','Ionos?',13,'',91 from dual union all
select 63497,'1990-S16','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63498,'1990-S160','1990-11-27 09:00:00','military missile','success',915,'3L','15Zh75 LKI-3','','','Test',477,'',1000 from dual union all
select 63499,'1990-S161','1990-11-28 20:00:03','military missile','success',1443,'','DASO-5','','','OT',420,'SSBN 736',1000 from dual union all
select 635,'1946-A18','1946-10-18 17:50:00','atmospheric rocket','failure',875,'46-1','Nike-46 4R','','','Test',1863,'',11 from dual union all
select 6350,'1960-A104','1960-04-27 21:56:00','suborbital rocket','failure',838,'NASA 03.03GS','','','','Solar UV?',1821,'',24 from dual union all
select 63500,'1990-S162','1990-12-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63501,'1990-S163','1990-12-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63502,'1990-S164','1990-12-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',146,'ZUB',250 from dual union all
select 63503,'1990-S165','1990-12-02 00:00:00','military missile','success',73,'','','','','Test',761,'',100 from dual union all
select 63504,'1990-S166','1990-12-02 00:00:00','military missile','success',73,'','','','','Test',761,'',100 from dual union all
select 63505,'1990-S167','1990-12-02 00:00:00','military missile','success',73,'','','','','Test',761,'',100 from dual union all
select 63506,'1990-S168','1990-11-30 00:00:00','suborbital rocket','success',1219,'XV-22','','','','Ionos?',330,'',500 from dual union all
select 63507,'1990-S169','1990-12-11 07:40:00','military missile','success',1005,'51013-713-10T','15A18M K-5','GCh 15F173','','OT',512,'',1000 from dual union all
select 63508,'1990-S17','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63509,'1990-S170','1990-12-18 03:37:25','suborbital rocket','success',1258,'','LACE UVPI Target','','','Target',239,'',100 from dual union all
select 6351,'1960-A105','1960-04-29 00:00:00','atmospheric rocket','success',705,'L1E','','','','Test',2,'',20 from dual union all
select 63510,'1990-S171','1990-12-21 00:00:00','military missile','success',629,'','','','','OT',988,'',100 from dual union all
select 63511,'1990-S172','1990-12-21 08:05:00','military missile','success',1453,'49256-42-368','15A35 UBP-39','','','OT',508,'',1000 from dual union all
select 63512,'1990-S173','1990-12-25 14:40:00','military missile','success',915,'4L','15Zh75 LKI-4','','','Test',477,'',1000 from dual union all
select 63513,'1990-S174','1990-12-25 15:49:00','military missile','success',1434,'','15Zh58 K-26','','','Test',477,'',1000 from dual union all
select 63514,'1990-S175','1990-10-17 11:33:00','suborbital rocket','success',597,'INTA MZ-9001','Test','M0','','Test',396,'',1 from dual union all
select 63515,'1990-S176','1990-10-21 12:42:00','suborbital rocket','success',597,'INTA MZ-9002','Test','M1','','Test',396,'',4 from dual union all
select 63516,'1990-S177','1990-10-22 11:58:00','suborbital rocket','success',597,'INTA MZ-9003','Test','M2','','Test',396,'',4 from dual union all
select 63517,'1990-S178','1990-01-02 00:00:00','military missile','success',1176,'','Test flight','','','Test',761,'',100 from dual union all
select 63518,'1990-S179','1990-01-02 00:00:00','military missile','success',73,'','Test flight','','','Test',761,'',100 from dual union all
select 63519,'1990-S18','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 6352,'1960-A106','1960-04-29 00:00:00','atmospheric rocket','success',705,'L1F','','','','Test',2,'',20 from dual union all
select 63520,'1990-S180','1990-01-02 00:00:00','military missile','success',1176,'','Test flight','','','Test',761,'',100 from dual union all
select 63521,'1990-S181','1990-01-02 00:00:00','military missile','success',73,'','Test flight','','','Test',761,'',100 from dual union all
select 63522,'1990-S182','1990-04-08 00:00:00','military missile','success',73,'','Chemical warhead','','','Test',761,'',100 from dual union all
select 63523,'1990-S183','1990-04-08 00:00:00','military missile','success',1176,'','Test flight','','','Test',761,'',100 from dual union all
select 63524,'1990-S184','1990-04-18 00:00:00','military missile','success',72,'','Test flight','','','Test',761,'',100 from dual union all
select 63525,'1990-S185','1990-12-28 00:00:00','military missile','success',69,'To Ar Rutba','Separating RV test','','','Test',758,'',100 from dual union all
select 63526,'1990-S186','1990-12-28 00:00:00','military missile','success',69,'','Separating RV test','','','Test',758,'',100 from dual union all
select 63527,'1990-S187','1990-12-26 00:00:00','military missile','success',983,'','Launch trailer test','','','Test',746,'',100 from dual union all
select 63528,'1990-S188','1990-01-01 00:00:00','military missile','success',348,'','Test launch','','','Test',1494,'',100 from dual union all
select 63529,'1990-S189','1990-02-14 20:26:00','suborbital rocket','success',1502,'Chaff','DYANA DAC39','','','Aeron',29,'',107 from dual union all
select 6353,'1960-A107','1960-05-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 63530,'1990-S19','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63531,'1990-S190','1990-06-27 00:00:00','military missile','success',73,'H4-1','Separating RV test','','','Test',757,'',100 from dual union all
select 63532,'1990-S191','1990-07-17 00:00:00','military missile','success',1450,'','15A20 52 RD UBP','','','OT',195,'',1000 from dual union all
select 63533,'1990-S192','1990-01-23 21:17:00','suborbital rocket','unknown',1284,'Datasonde','DYANA DED4','INTA DYSLD-9004','','Aeron',396,'',90 from dual union all
select 63534,'1990-S193','1990-02-05 21:34:00','suborbital rocket','unknown',1284,'Datasonde','DYANA DED9','INTA DYSLD-9009','','Aeron',396,'',90 from dual union all
select 63535,'1990-S194','1990-03-01 21:18:00','suborbital rocket','unknown',1284,'Datasonde','DYANA DED17','INTA DYSLD-9017','','Aeron',396,'',90 from dual union all
select 63536,'1990-S195','1990-05-24 00:00:00','military missile','success',1434,'','15Zh58 K-24','','','Test',477,'',1000 from dual union all
select 63537,'1990-S20','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63538,'1990-S21','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63539,'1990-S22','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 6354,'1960-A108','1960-05-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 63540,'1990-S23','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63541,'1990-S24','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63542,'1990-S25','1990-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 63543,'1990-S26','1990-01-15 16:45:07','military missile','success',1443,'','PEM-7','','','OT',420,'SSBN 734',1000 from dual union all
select 63544,'1990-S27','1990-01-16 16:00:03','military missile','success',1443,'','PEM-6','','','OT',420,'SSBN 734',1000 from dual union all
select 63545,'1990-S28','1990-01-21 06:05:00','suborbital rocket','success',211,'','COBRA','','','Submm Astron',1895,'',250 from dual union all
select 63546,'1990-S29','1990-01-22 10:20:00','suborbital rocket','success',860,'Ferdinand 81','DYANA DAT13 TURBO R.','','','Aeron',29,'',132 from dual union all
select 63547,'1990-S30','1990-01-23 19:40:00','suborbital rocket','success',224,'NASA 36.33UL','PACE','','','Aeronomy',1873,'',300 from dual union all
select 63548,'1990-S31','1990-01-25 08:50:00','suborbital rocket','success',891,'DLR K-O-172','Strafam 2/3','','','Aeron',417,'',0 from dual;
insert into LAUNCH
select 63549,'1990-S32','1990-01-26 13:25:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',83 from dual union all
select 6355,'1960-A109','1960-05-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 63550,'1990-S33','1990-01-27 19:30:00','suborbital rocket','success',1073,'S-310-20','','','','Aeron',833,'',185 from dual union all
select 63551,'1990-S34','1990-01-30 21:22:00','suborbital rocket','success',561,'GL A26.804','TM2-1','','','Test',1821,'',422 from dual union all
select 63552,'1990-S35','1990-02-01 12:07:00','suborbital rocket','success',234,'NASA 39.002UE','SCEX III','','','Active Plasma',1047,'',375 from dual union all
select 63553,'1990-S36','1990-02-06 08:01:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT137GB','GLORY TRIP 137GB','OT',1712,'',1300 from dual union all
select 63554,'1990-S37','1990-02-12 14:10:00','military missile','success',1443,'','DASO-1','','','OT',420,'SSBN 734',1000 from dual union all
select 63555,'1990-S38','1990-02-12 14:10:20','military missile','success',1443,'','PEM-8','','','OT',420,'SSBN 734',1000 from dual union all
select 63556,'1990-S39','1990-02-14 00:00:00','military missile','success',795,'','MAST','RSLP MAST (OSC CFE)','','Reentry Test',1698,'',1300 from dual union all
select 63557,'1990-S40','1990-02-20 04:54:00','suborbital rocket','success',860,'DLR C-NO-175','DYANA DBN1 TURBO','','','Aeron',307,'',129 from dual union all
select 63558,'1990-S41','1990-02-20 09:41:00','suborbital rocket','success',860,'SSC RSM/ANODE','DYANA Nitric oxide','','','Aeron',412,'',138 from dual union all
select 63559,'1990-S42','1990-02-21 00:00:00','suborbital rocket','success',1216,'XV-53','Alcantara','','','Ionos?',13,'',101 from dual union all
select 6356,'1960-A11','1960-01-01 00:00:00','military missile','success',1469,'','LKI','','','Test',182,'',20 from dual union all
select 63560,'1990-S43','1990-02-21 13:59:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',80 from dual union all
select 63561,'1990-S44','1990-02-21 16:00:00','suborbital rocket','success',1074,'S-520-11','CIR','','','IR Astron',833,'',317 from dual union all
select 63562,'1990-S45','1990-02-23 07:31:25','suborbital rocket','success',209,'NASA 35.20UE','ARCS 4','','','Plasma',1047,'',586 from dual union all
select 63563,'1990-S46','1990-02-25 19:20:00','suborbital rocket','success',860,'Ferdinand 82','DYANA DAT50 TURBO','','','Aeron',29,'',128 from dual union all
select 63564,'1990-S47','1990-02-25 23:21:20','suborbital rocket','success',860,'SSC RSG/ATOMIC 1','DYANA Atomic oxygen','','','Aeron',412,'',133 from dual union all
select 63565,'1990-S48','1990-02-26 02:06:00','suborbital rocket','success',1074,'S-520-12','A-1','','','Auroral',29,'',369 from dual union all
select 63566,'1990-S49','1990-02-26 13:55:00','suborbital rocket','success',1023,'','DYANA Chaff','','','Meteo',320,'',80 from dual union all
select 63567,'1990-S50','1990-02-28 00:51:59','suborbital rocket','success',860,'SSC RSG/ATOMIC 2','DYANA Atomic oxygen','','','Aeron',412,'',139 from dual union all
select 63568,'1990-S51','1990-02-28 06:09:00','military missile','success',1454,'51145-93-001P','15A35P LKI-1','15Yu70-010/102E','','OT',535,'',1000 from dual union all
select 63569,'1990-S52','1990-02-28 14:30:00','military missile','success',1041,'48737-415-08','15Zh60 K-2','','','Test',477,'',1000 from dual union all
select 6357,'1960-A110','1960-05-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 63570,'1990-S53','1990-03-06 02:41:00','suborbital rocket','success',860,'Ferdinand 83','DYANA DAT62 TURBO','','','Aeron',29,'',133 from dual union all
select 63571,'1990-S54','1990-03-06 05:18:30','suborbital rocket','success',860,'DLR C-NO-176','DYANA DBN2  TURBO','','','Aeron',307,'',128 from dual union all
select 63572,'1990-S55','1990-03-06 04:41:00','suborbital rocket','success',1210,'DLR K-GRVI-173','DLR SISSI 1','','','Aeron',416,'',178 from dual union all
select 63573,'1990-S56','1990-03-06 08:30:00','suborbital rocket','success',605,'DLR K-O-174','RASMUS 3','','','Aeron',417,'',79 from dual union all
select 63574,'1990-S57','1990-03-08 00:00:00','military missile','success',907,'02PA (85-161)','','GT 02PA','','OT',1700,'',1000 from dual union all
select 63575,'1990-S58','1990-03-08 22:53:00','suborbital rocket','success',860,'Ferdinand 84','DYANA DAT73 TURBO','','','Aeron',29,'',119 from dual union all
select 63576,'1990-S59','1990-03-09 00:25:00','suborbital rocket','success',860,'Ferdinand 85','DYANA DAT76 TURBO','','','Aeron',29,'',130 from dual union all
select 63577,'1990-S60','1990-03-09 18:00:00','suborbital rocket','success',224,'NASA 36.66GU','','','','Aeron?',1873,'',300 from dual union all
select 63578,'1990-S61','1990-03-11 20:42:00','suborbital rocket','success',860,'Ferdinand 86','DYANA DAT84 TURBO','','','Aeron',29,'',131 from dual union all
select 63579,'1990-S62','1990-03-12 03:45:03','military missile','success',1443,'','DASO-3','','','OT',420,'SSBN 735',1000 from dual union all
select 6358,'1960-A111','1960-05-01 00:00:00','military missile','success',993,'K-1.1','','','','Test',326,'',1 from dual union all
select 63580,'1990-S63','1990-03-13 04:21:00','suborbital rocket','success',860,'DLR C-NO-177','DYANA DBN3  TURBO','','','Aeron',307,'',128 from dual union all
select 63581,'1990-S64','1990-03-17 04:30:00','suborbital rocket','success',224,'NASA 36.34UH','','','','XR Astron',1873,'',300 from dual union all
select 63582,'1990-S65','1990-03-21 00:00:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT139GM','GLORY TRIP 139GM','OT',1705,'',1300 from dual union all
select 63583,'1990-S66','1990-03-22 07:38:26','suborbital rocket','success',237,'NASA 40.002UE','','','','Auroral?',1047,'',1400 from dual union all
select 63584,'1990-S67','1990-03-22 10:03:23','suborbital rocket','success',860,'NASA 31.70UE','','','','Plasma',1047,'',140 from dual union all
select 63585,'1990-S68','1990-03-24 00:00:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT138GM-1','GLORY TRIP 138GM-1','OT',1699,'',1300 from dual union all
select 63586,'1990-S69','1990-03-24 04:16:00','military missile','unknown',1442,'','CET              .','','','OT',420,'SSBN 629',1000 from dual union all
select 63587,'1990-S70','1990-03-24 00:00:00','military missile','unknown',1442,'','CET','','','OT',420,'SSBN 629',1000 from dual union all
select 63588,'1990-S71','1990-03-24 00:00:00','military missile','unknown',1442,'','CET','','','OT',420,'SSBN 629',1000 from dual union all
select 63589,'1990-S72','1990-03-24 00:00:00','military missile','unknown',1442,'','CET','','','OT',420,'SSBN 629',1000 from dual union all
select 6359,'1960-A112','1960-05-02 00:00:00','military missile','success',1080,'F89','','','','Test',671,'',20 from dual union all
select 63590,'1990-S73','1990-03-29 09:26:00','suborbital rocket','success',1337,'SDIO Laser Target','Firefly 1','','','Target',1821,'',462 from dual union all
select 63591,'1990-S74','1990-03-29 06:45:00','military missile','success',1454,'51145-93-002P','15A35P LKI-2','15Yu70/102E','','OT',535,'',1000 from dual union all
select 63592,'1990-S75','1990-03-29 07:45:00','military missile','success',1434,'','15Zh58 UBP-7','','','Test',477,'',1000 from dual union all
select 63593,'1990-S76','1990-03-29 08:07:00','suborbital rocket','success',224,'MASER 4','SSC MASER 4','','','Micrograv',412,'',317 from dual union all
select 63594,'1990-S77','1990-03-30 17:00:00','suborbital rocket','success',224,'NASA 36.53DE','','','','Plasma',1873,'',300 from dual union all
select 63595,'1990-S78','1990-04-05 13:47:00','suborbital rocket','success',231,'NASA 35.28CE','WISP 1','','','Plasma',1821,'',586 from dual union all
select 63596,'1990-S79','1990-04-09 08:50:00','suborbital rocket','success',224,'NASA 36.67UG','FUVS','','','UV Astron',1873,'',300 from dual union all
select 63597,'1990-S80','1990-04-17 06:00:00','suborbital rocket','success',224,'NASA 36.063UG','DUVE','','','UV Astron',1873,'',300 from dual union all
select 63598,'1990-S81','1990-04-21 10:15:00','suborbital rocket','success',224,'NASA 36.73UG','JHU FOT 9 (C/Austin)','','','UV Astron',1873,'',300 from dual union all
select 63599,'1990-S82','1990-04-25 05:32:00','suborbital rocket','success',1337,'SDIO BSUV','Bow Shock 1','','','Reentry Test',1821,'',721 from dual union all
select 636,'1946-A19','1946-11-12 21:33:00','atmospheric rocket','failure',875,'46-6','Nike-46 5R','','','Test',1863,'',10 from dual union all
select 6360,'1960-A113','1960-05-05 00:00:00','atmospheric rocket','success',705,'L1G','','','','Test',2,'',20 from dual union all
select 63600,'1990-S83','1990-04-27 07:00:59','suborbital rocket','success',117,'GL A24.570','EXCEDE-3','','','Active Plasma',1892,'',116 from dual union all
select 63601,'1990-S84','1990-04-28 09:45:00','suborbital rocket','success',224,'NASA 36.74UG','FUVS (C/Austin)','','','UV Astron',1873,'',300 from dual union all
select 63602,'1990-S85','1990-05-01 00:00:00','military missile','failure',586,'','[may be spurious]','','','Test',1480,'',0 from dual union all
select 63603,'1990-S86','1990-04-30 17:50:00','military missile','success',939,'','DASO-61','','','OT',421,'SSBN 642',500 from dual union all
select 63604,'1990-S87','1990-05-13 03:52:00','suborbital rocket','success',1212,'DLR K-GR-178','DLR TEXUS 25','','','Micrograv',416,'',234 from dual union all
select 63605,'1990-S88','1990-05-13 21:29:02','suborbital rocket','success',1319,'NASA 33.59GE','','','','Plasma',1047,'',130 from dual union all
select 63606,'1990-S89','1990-05-13 21:37:40','suborbital rocket','success',891,'NASA 30.35UE','','','','Plasma',1047,'',85 from dual union all
select 63607,'1990-S90','1990-05-14 20:26:00','suborbital rocket','success',1319,'NASA 33.60GE','','','','Plasma',1047,'',130 from dual union all
select 63608,'1990-S91','1990-05-15 02:45:00','suborbital rocket','success',1212,'DLR K-GR-179','DLR TEXUS 26','','','Micrograv',416,'',235 from dual union all
select 63609,'1990-S92','1990-05-15 13:10:00','military missile','success',1041,'48747-415-13','15Zh60 K-3','','','Test',477,'',1000 from dual union all
select 6361,'1960-A114','1960-05-05 00:00:00','military missile','success',1080,'F90','','','','Test',671,'',20 from dual union all
select 63610,'1990-S93','1990-05-16 18:01:00','military missile','success',907,'03PA (86-450)','','GT 03PA','','OT',1703,'',1000 from dual union all
select 63611,'1990-S94','1990-05-16 13:40:00','suborbital rocket','success',213,'Starfire 1/F3','CONSORT 3','','','Micrograv',1879,'',304 from dual union all
select 63612,'1990-S95','1990-05-17 12:05:00','military missile','success',1006,'45042-613-07','15A18 UBP-28','GCh 15F183','','OT',489,'',1000 from dual union all
select 63613,'1990-S96','1990-05-21 19:20:00','military missile','success',1434,'','15Zh58 UBP-8','','','Test',477,'',1000 from dual union all
select 63614,'1990-S97','1990-07-24 00:00:00','military missile','success',1042,'48743-711-43','15Zh61 K-2','','','Test',477,'',1000 from dual union all
select 63615,'1990-S98','1990-05-29 05:09:00','suborbital rocket','success',860,'','SSC SPINRAC Test','','','Test',417,'',107 from dual union all
select 63616,'1990-S99','1990-06-01 00:00:00','military missile','success',580,'','','','','Test',1480,'',86 from dual union all
select 63617,'1990-W01','1990-01-01 13:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',55 from dual union all
select 63618,'1990-W02','1990-01-03 08:15:00','suborbital rocket','unknown',738,'','','','','Aeron',1035,'PASSAT',62 from dual union all
select 63619,'1990-W03','1990-01-03 08:52:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',75 from dual union all
select 6362,'1960-A115','1960-05-05 00:00:00','suborbital spaceplane','failure',1520,'2-A-17','','','','Test',379,'NB-52 008',14 from dual union all
select 63620,'1990-W04','1990-01-03 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',55 from dual union all
select 63621,'1990-W05','1990-01-05 07:10:00','suborbital rocket','unknown',738,'','','','','Aeron',1035,'PASSAT',65 from dual union all
select 63622,'1990-W06','1990-01-05 07:25:00','suborbital rocket','unknown',738,'','','','','Aeron',1645,'USHAKOV',60 from dual union all
select 63623,'1990-W07','1990-01-05 14:30:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',60 from dual union all
select 63624,'1990-W08','1990-01-06 13:17:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',55 from dual union all
select 63625,'1990-W09','1990-01-08 07:29:00','suborbital rocket','unknown',738,'','','','','Aeron',1035,'PASSAT',60 from dual union all
select 63626,'1990-W10','1990-01-09 20:10:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',79 from dual union all
select 63627,'1990-W100','1990-02-28 09:25:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',60 from dual union all
select 63628,'1990-W101','1990-02-28 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM14','','','Aeron',1010,'',77 from dual union all
select 63629,'1990-W102','1990-02-28 15:30:00','suborbital rocket','unknown',728,'','DYANA DVM14','','','Aeron',636,'',77 from dual union all
select 6363,'1960-A116','1960-05-06 16:47:02','military missile','failure',154,'23D','Mk 2-4','IOC ORT','LUCKY DRAGON','Test',1666,'',2 from dual union all
select 63630,'1990-W103','1990-03-02 07:40:00','suborbital rocket','unknown',738,'','','','','Aeron',918,'KRENKEL''',60 from dual union all
select 63631,'1990-W104','1990-03-04 15:47:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 63632,'1990-W105','1990-03-04 16:46:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',63 from dual union all
select 63633,'1990-W106','1990-03-05 14:35:00','suborbital rocket','unknown',728,'','DYANA DVM15','','','Aeron',636,'',75 from dual union all
select 63634,'1990-W107','1990-03-05 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM15','','','Aeron',1010,'',73 from dual union all
select 63635,'1990-W108','1990-03-06 19:20:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',77 from dual union all
select 63636,'1990-W109','1990-03-07 02:00:00','suborbital rocket','success',761,'MT-135P-R738','DYANA','','','Aeron',1332,'',55 from dual union all
select 63637,'1990-W11','1990-01-09 20:30:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',76 from dual union all
select 63638,'1990-W110','1990-03-07 07:02:00','suborbital rocket','unknown',738,'','','','','Aeron',918,'KRENKEL''',60 from dual;
insert into LAUNCH
select 63639,'1990-W111','1990-03-07 11:10:00','suborbital rocket','unknown',728,'','DYANA DTM21','','','Aeron',1467,'',79 from dual union all
select 6364,'1960-A117','1960-05-06 17:53:19','suborbital spaceplane','success',1520,'1-6-11','','','','Test',379,'NB-52 003',18 from dual union all
select 63640,'1990-W112','1990-03-07 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',75 from dual union all
select 63641,'1990-W113','1990-03-07 14:37:00','suborbital rocket','unknown',728,'','DYANA DVM16','','','Aeron',636,'',77 from dual union all
select 63642,'1990-W114','1990-03-07 22:37:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',61 from dual union all
select 63643,'1990-W115','1990-03-09 07:54:00','suborbital rocket','unknown',738,'','','','','Aeron',918,'KRENKEL''',60 from dual union all
select 63644,'1990-W116','1990-03-11 03:03:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',55 from dual union all
select 63645,'1990-W117','1990-03-11 04:09:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',60 from dual union all
select 63646,'1990-W118','1990-03-11 07:03:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',77 from dual union all
select 63647,'1990-W119','1990-03-11 10:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',76 from dual union all
select 63648,'1990-W12','1990-01-09 23:10:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',76 from dual union all
select 63649,'1990-W120','1990-03-11 10:58:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',55 from dual union all
select 6365,'1960-A118','1960-05-07 02:33:00','military missile','success',793,'','Silo Test','','','Test',381,'',0 from dual union all
select 63650,'1990-W121','1990-03-11 14:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',77 from dual union all
select 63651,'1990-W122','1990-03-11 18:57:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',72 from dual union all
select 63652,'1990-W123','1990-03-11 22:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',75 from dual union all
select 63653,'1990-W124','1990-03-12 14:52:00','suborbital rocket','unknown',728,'','DYANA DVM17','','','Aeron',636,'',73 from dual union all
select 63654,'1990-W125','1990-03-13 20:00:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',77 from dual union all
select 63655,'1990-W126','1990-03-14 02:00:00','suborbital rocket','success',761,'MT-135P-R739','DYANA','','','Aeron',1332,'',55 from dual union all
select 63656,'1990-W127','1990-03-14 12:56:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',55 from dual union all
select 63657,'1990-W128','1990-03-14 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM18','','','Aeron',1010,'',77 from dual union all
select 63658,'1990-W129','1990-03-21 09:25:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',60 from dual union all
select 63659,'1990-W13','1990-01-17 02:00:00','suborbital rocket','success',761,'MT-135P-R731','DYANA','','','Aeron',1332,'',59 from dual union all
select 6366,'1960-A119','1960-05-12 00:00:00','military missile','success',771,'GM-54-62','','','','Test',222,'',12 from dual union all
select 63660,'1990-W130','1990-03-21 13:20:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',65 from dual union all
select 63661,'1990-W131','1990-03-21 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 63662,'1990-W132','1990-03-21 14:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',75 from dual union all
select 63663,'1990-W133','1990-03-21 15:25:00','suborbital rocket','unknown',738,'','','','','Aeron',636,'',75 from dual union all
select 63664,'1990-W134','1990-03-28 10:25:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',60 from dual union all
select 63665,'1990-W135','1990-03-28 14:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',74 from dual union all
select 63666,'1990-W136','1990-04-29 10:41:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',66 from dual union all
select 63667,'1990-W137','1990-04-29 11:37:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',66 from dual union all
select 63668,'1990-W138','1990-05-08 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',60 from dual union all
select 63669,'1990-W139','1990-05-09 00:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',67 from dual union all
select 6367,'1960-A12','1960-01-02 00:00:00','atmospheric rocket','failure',705,'L1B','','','','Test',2,'',0 from dual union all
select 63670,'1990-W14','1990-01-10 07:19:00','suborbital rocket','unknown',738,'','','','','Aeron',1645,'USHAKOV',60 from dual union all
select 63671,'1990-W140','1990-05-09 12:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',74 from dual union all
select 63672,'1990-W141','1990-05-11 12:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',74 from dual union all
select 63673,'1990-W142','1990-05-30 12:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',66 from dual union all
select 63674,'1990-W143','1990-06-08 12:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',76 from dual union all
select 63675,'1990-W144','1990-06-13 12:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',77 from dual union all
select 63676,'1990-W145','1990-06-20 12:10:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',77 from dual union all
select 63677,'1990-W146','1990-06-26 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',72 from dual union all
select 63678,'1990-W147','1990-07-17 15:37:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',71 from dual union all
select 63679,'1990-W148','1990-07-19 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 6368,'1960-A120','1960-05-12 00:00:00','military missile','success',771,'GM-54-60','','','','Test',222,'',12 from dual union all
select 63680,'1990-W149','1990-07-23 12:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 63681,'1990-W15','1990-01-10 13:00:00','suborbital rocket','success',738,'','DYANA DZD1','','','Aeron',1917,'',75 from dual union all
select 63682,'1990-W150','1990-07-31 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 63683,'1990-W151','1990-08-07 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 63684,'1990-W152','1990-08-16 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 63685,'1990-W153','1990-08-21 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',63 from dual union all
select 63686,'1990-W154','1990-08-23 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 63687,'1990-W155','1990-08-27 02:10:00','suborbital rocket','success',759,'MT-135-52','NNP-O3','','','Aeron',839,'',57 from dual union all
select 63688,'1990-W156','1990-08-30 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 63689,'1990-W157','1990-09-12 12:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',77 from dual union all
select 6369,'1960-A121','1960-05-12 00:00:00','military missile','failure',1469,'','LKI','','','Test',182,'',10 from dual union all
select 63690,'1990-W158','1990-09-17 13:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',51 from dual union all
select 63691,'1990-W159','1990-09-19 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 63692,'1990-W16','1990-01-10 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',75 from dual union all
select 63693,'1990-W160','1990-09-21 17:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 63694,'1990-W161','1990-09-26 12:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',79 from dual union all
select 63695,'1990-W162','1990-09-26 18:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',69 from dual union all
select 63696,'1990-W163','1990-09-27 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 63697,'1990-W164','1990-10-01 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 63698,'1990-W165','1990-10-05 13:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 63699,'1990-W166','1990-10-06 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 637,'1946-A20','1946-11-15 21:34:00','atmospheric rocket','failure',875,'46-7','Nike-46 6R','','','Test',1863,'',0 from dual union all
select 6370,'1960-A122','1960-05-12 16:47:37','suborbital spaceplane','success',1520,'1-7-12','','','','Test',379,'NB-52 003',23 from dual union all
select 63700,'1990-W167','1990-10-10 09:53:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',59 from dual union all
select 63701,'1990-W168','1990-10-10 10:33:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',55 from dual union all
select 63702,'1990-W169','1990-10-10 10:42:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',59 from dual union all
select 63703,'1990-W17','1990-01-10 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',78 from dual union all
select 63704,'1990-W170','1990-10-10 11:37:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',62 from dual union all
select 63705,'1990-W171','1990-10-12 13:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',53 from dual union all
select 63706,'1990-W172','1990-10-17 13:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',77 from dual union all
select 63707,'1990-W173','1990-10-17 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 63708,'1990-W174','1990-10-17 13:40:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',79 from dual union all
select 63709,'1990-W175','1990-10-29 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 6371,'1960-A123','1960-05-16 00:00:00','military missile','success',1214,'SM-62A N-3444','','','','OT',224,'',10 from dual union all
select 63710,'1990-W176','1990-11-02 08:01:00','suborbital rocket','unknown',738,'','','','','Aeron',1036,'PASSAT',63 from dual union all
select 63711,'1990-W177','1990-11-02 13:40:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',74 from dual union all
select 63712,'1990-W178','1990-11-05 08:14:00','suborbital rocket','unknown',738,'','','','','Aeron',1036,'PASSAT',64 from dual union all
select 63713,'1990-W179','1990-11-07 07:26:00','suborbital rocket','unknown',738,'','','','','Aeron',1036,'PASSAT',64 from dual union all
select 63714,'1990-W18','1990-01-10 15:22:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',78 from dual union all
select 63715,'1990-W180','1990-11-21 20:00:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',79 from dual union all
select 63716,'1990-W181','1990-11-23 13:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',68 from dual union all
select 63717,'1990-W182','1990-12-04 21:15:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',72 from dual union all
select 63718,'1990-W183','1990-12-04 22:20:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',72 from dual union all
select 63719,'1990-W184','1990-12-05 13:30:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',75 from dual union all
select 6372,'1960-A124','1960-05-19 16:46:47','suborbital spaceplane','success',1520,'1-8-13','Alt Buildup','','','Test',379,'NB-52 003',33 from dual union all
select 63720,'1990-W185','1990-12-14 13:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',72 from dual union all
select 63721,'1990-W186','1990-12-19 13:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',66 from dual union all
select 63722,'1990-W187','1990-12-19 13:30:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',71 from dual union all
select 63723,'1990-W188','1990-12-20 09:05:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',78 from dual union all
select 63724,'1990-W189','1990-12-31 07:06:00','suborbital rocket','unknown',738,'','','','','Aeron',217,'BUGAEV',62 from dual union all
select 63725,'1990-W19','1990-01-11 08:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',70 from dual union all
select 63726,'1990-W190','1990-01-20 12:10:00','suborbital rocket','success',1284,'SL-3','','','','Aeron',839,'',77 from dual union all
select 63727,'1990-W191','1990-01-06 02:00:00','suborbital rocket','success',761,'MT135P-R730','','','','Meteo',1332,'',60 from dual union all
select 63728,'1990-W192','1990-03-21 02:00:00','suborbital rocket','success',761,'MT135P-R740','','','','Meteo',1332,'',60 from dual;
insert into LAUNCH
select 63729,'1990-W193','1990-03-28 02:00:00','suborbital rocket','success',761,'MT135P-R741','','','','Meteo',1332,'',60 from dual union all
select 6373,'1960-A125','1960-05-24 00:00:00','military missile','success',771,'GM-56-1928','','','','Test',222,'',12 from dual union all
select 63730,'1990-W194','1990-04-11 02:00:00','suborbital rocket','success',761,'MT135P-R742','','','','Meteo',1332,'',60 from dual union all
select 63731,'1990-W195','1990-04-18 02:00:00','suborbital rocket','success',761,'MT135P-R743','','','','Meteo',1332,'',60 from dual union all
select 63732,'1990-W196','1990-04-25 02:00:00','suborbital rocket','success',761,'MT135P-R744','','','','Meteo',1332,'',60 from dual union all
select 63733,'1990-W197','1990-05-10 02:00:00','suborbital rocket','success',761,'MT135P-R745','','','','Meteo',1332,'',60 from dual union all
select 63734,'1990-W198','1990-05-16 02:00:00','suborbital rocket','success',761,'MT135P-R746','','','','Meteo',1332,'',60 from dual union all
select 63735,'1990-W199','1990-05-23 02:00:00','suborbital rocket','success',761,'MT135P-R747','','','','Meteo',1332,'',60 from dual union all
select 63736,'1990-W20','1990-01-11 11:15:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',55 from dual union all
select 63737,'1990-W200','1990-06-06 02:00:00','suborbital rocket','success',761,'MT135P-R748','','','','Meteo',1332,'',60 from dual union all
select 63738,'1990-W201','1990-06-20 02:00:00','suborbital rocket','success',761,'MT135P-R749','','','','Meteo',1332,'',60 from dual union all
select 63739,'1990-W202','1990-06-28 02:00:00','suborbital rocket','success',761,'MT135P-R750','','','','Meteo',1332,'',60 from dual union all
select 6374,'1960-A126','1960-05-24 00:00:00','military missile','success',771,'GM-54-67','','','','Test',222,'',12 from dual union all
select 63740,'1990-W203','1990-07-11 02:00:00','suborbital rocket','success',761,'MT135P-R751','','','','Meteo',1332,'',60 from dual union all
select 63741,'1990-W204','1990-07-25 02:05:00','suborbital rocket','success',761,'MT135P-R752','','','','Meteo',1332,'',60 from dual union all
select 63742,'1990-W205','1990-08-08 02:00:00','suborbital rocket','success',761,'MT135P-R753','','','','Meteo',1332,'',60 from dual union all
select 63743,'1990-W206','1990-08-23 02:00:00','suborbital rocket','success',761,'MT135P-R754','','','','Meteo',1332,'',60 from dual union all
select 63744,'1990-W207','1990-09-05 02:01:00','suborbital rocket','success',761,'MT135P-R755','','','','Meteo',1332,'',60 from dual union all
select 63745,'1990-W208','1990-09-19 02:01:00','suborbital rocket','success',761,'MT135P-R756','','','','Meteo',1332,'',60 from dual union all
select 63746,'1990-W209','1990-10-03 02:00:00','suborbital rocket','success',761,'MT135P-R757','','','','Meteo',1332,'',60 from dual union all
select 63747,'1990-W21','1990-01-12 07:05:00','suborbital rocket','unknown',738,'','','','','Aeron',1645,'USHAKOV',57 from dual union all
select 63748,'1990-W210','1990-10-17 02:05:00','suborbital rocket','success',761,'MT135P-R758','','','','Meteo',1332,'',60 from dual union all
select 63749,'1990-W211','1990-11-01 02:00:00','suborbital rocket','success',761,'MT135P-R759','','','','Meteo',1332,'',60 from dual union all
select 6375,'1960-A127','1960-05-24 08:57:00','suborbital rocket','failure',838,'NASA 03.23CA','GCA Release 6 Na','','','Aeronomy',1821,'',24 from dual union all
select 63750,'1990-W212','1990-11-07 02:00:00','suborbital rocket','success',761,'MT135P-R760','','','','Meteo',1332,'',60 from dual union all
select 63751,'1990-W213','1990-11-14 02:01:00','suborbital rocket','success',761,'MT135P-R761','','','','Meteo',1332,'',60 from dual union all
select 63752,'1990-W214','1990-11-28 02:00:00','suborbital rocket','success',761,'MT135P-R762','','','','Meteo',1332,'',60 from dual union all
select 63753,'1990-W215','1990-12-05 02:00:00','suborbital rocket','success',761,'MT135P-R763','','','','Meteo',1332,'',60 from dual union all
select 63754,'1990-W216','1990-12-12 02:00:00','suborbital rocket','success',761,'MT135P-R764','','','','Meteo',1332,'',60 from dual union all
select 63755,'1990-W217','1990-12-19 02:02:00','suborbital rocket','success',761,'MT135P-R765','','','','Meteo',1332,'',60 from dual union all
select 63756,'1990-W218','1990-12-27 02:01:00','suborbital rocket','success',761,'MT135P-R766','','','','Meteo',1332,'',60 from dual union all
select 63757,'1990-W219','1990-08-02 23:46:00','suborbital rocket','success',1284,'Robin Dart','SISSI So5','','','Aeron',409,'',39 from dual union all
select 63758,'1990-W22','1990-01-12 08:24:00','suborbital rocket','unknown',738,'','','','','Aeron',1035,'PASSAT',61 from dual union all
select 63759,'1990-W23','1990-01-13 10:10:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',58 from dual union all
select 6376,'1960-A128','1960-05-26 17:08:36','suborbital spaceplane','success',1520,'2-9-18','SAS, High alpha, BCS','','','Test',379,'NB-52 008',15 from dual union all
select 63760,'1990-W24','1990-01-16 23:15:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',76 from dual union all
select 63761,'1990-W25','1990-01-17 07:04:00','suborbital rocket','unknown',738,'','','','','Aeron',1645,'USHAKOV',55 from dual union all
select 63762,'1990-W26','1990-01-17 09:08:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',55 from dual union all
select 63763,'1990-W27','1990-01-17 10:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',70 from dual union all
select 63764,'1990-W28','1990-01-17 12:40:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 63765,'1990-W29','1990-01-17 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',75 from dual union all
select 63766,'1990-W30','1990-01-17 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 63767,'1990-W31','1990-01-17 15:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 63768,'1990-W32','1990-01-19 05:57:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',66 from dual union all
select 63769,'1990-W33','1990-01-19 07:04:00','suborbital rocket','unknown',738,'','','','','Aeron',1645,'USHAKOV',60 from dual union all
select 6377,'1960-A129','1960-05-27 00:00:00','atmospheric rocket','unknown',705,'L3','','','','Test',2,'',20 from dual union all
select 63770,'1990-W34','1990-01-19 07:37:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',54 from dual union all
select 63771,'1990-W35','1990-01-19 13:00:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',55 from dual union all
select 63772,'1990-W36','1990-01-20 04:01:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',61 from dual union all
select 63773,'1990-W37','1990-01-20 04:58:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',57 from dual union all
select 63774,'1990-W38','1990-01-21 10:50:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',55 from dual union all
select 63775,'1990-W39','1990-01-22 10:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',57 from dual union all
select 63776,'1990-W40','1990-01-22 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM3','','','Aeron',1010,'',77 from dual union all
select 63777,'1990-W41','1990-01-23 10:09:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',55 from dual union all
select 63778,'1990-W42','1990-01-23 23:10:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',76 from dual union all
select 63779,'1990-W43','1990-01-24 02:00:00','suborbital rocket','success',761,'MT-135P-R732','DYANA','','','Aeron',1332,'',58 from dual union all
select 6378,'1960-A13','1960-01-07 00:00:00','atmospheric rocket','failure',705,'L1C','','','','Test',2,'',0 from dual union all
select 63780,'1990-W44','1990-01-24 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1645,'USHAKOV',60 from dual union all
select 63781,'1990-W45','1990-01-24 08:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',77 from dual union all
select 63782,'1990-W46','1990-01-24 10:23:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',55 from dual union all
select 63783,'1990-W47','1990-01-24 11:10:00','suborbital rocket','unknown',728,'','DYANA DTM5','','','Aeron',1467,'',77 from dual union all
select 63784,'1990-W48','1990-01-24 16:05:00','suborbital rocket','unknown',728,'','DYANA DHM4','','','Aeron',1010,'',65 from dual union all
select 63785,'1990-W49','1990-01-24 22:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 63786,'1990-W50','1990-01-25 10:44:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',55 from dual union all
select 63787,'1990-W51','1990-01-26 07:00:00','suborbital rocket','unknown',738,'','','','','Aeron',1645,'USHAKOV',60 from dual union all
select 63788,'1990-W52','1990-01-26 11:10:00','suborbital rocket','unknown',728,'','DYANA DTM6','','','Aeron',1467,'',77 from dual union all
select 63789,'1990-W53','1990-01-26 13:13:00','suborbital rocket','unknown',738,'','','','','Aeron',1306,'PRILIV',50 from dual union all
select 6379,'1960-A130','1960-05-27 00:00:00','suborbital spaceplane','failure',1520,'1-A-14','','','','Test',379,'NB-52 003',14 from dual union all
select 63790,'1990-W54','1990-01-29 07:39:00','suborbital rocket','unknown',738,'','','','','Aeron',1035,'PASSAT',60 from dual union all
select 63791,'1990-W55','1990-01-31 02:00:00','suborbital rocket','success',761,'MT-135P-R733','DYANA','','','Aeron',1332,'',58 from dual union all
select 63792,'1990-W56','1990-01-31 04:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',66 from dual union all
select 63793,'1990-W57','1990-01-31 07:07:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',60 from dual union all
select 63794,'1990-W58','1990-01-31 07:08:00','suborbital rocket','unknown',738,'','','','','Aeron',1035,'PASSAT',60 from dual union all
select 63795,'1990-W59','1990-01-31 07:11:00','suborbital rocket','unknown',738,'','','','','Aeron',1645,'USHAKOV',60 from dual union all
select 63796,'1990-W60','1990-01-31 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',78 from dual union all
select 63797,'1990-W61','1990-01-31 11:25:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',60 from dual union all
select 63798,'1990-W62','1990-01-31 14:53:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',55 from dual union all
select 63799,'1990-W63','1990-01-31 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 638,'1946-A21','1946-12-02 00:00:00','suborbital rocket','failure',1517,'21','Booster test','','','Test',1852,'',4 from dual union all
select 6380,'1960-A131','1960-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 63800,'1990-W64','1990-01-31 15:10:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',79 from dual union all
select 63801,'1990-W65','1990-01-31 18:53:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',76 from dual union all
select 63802,'1990-W66','1990-01-31 23:20:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',60 from dual union all
select 63803,'1990-W67','1990-02-02 13:05:00','suborbital rocket','unknown',728,'','DYANA DTM8','','','Aeron',1467,'',75 from dual union all
select 63804,'1990-W68','1990-02-05 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM7','','','Aeron',1010,'',78 from dual union all
select 63805,'1990-W69','1990-02-07 02:00:00','suborbital rocket','success',761,'MT-135P-R734','DYANA','','','Aeron',1332,'',58 from dual union all
select 63806,'1990-W70','1990-02-07 05:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',77 from dual union all
select 63807,'1990-W71','1990-02-07 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',76 from dual union all
select 63808,'1990-W72','1990-02-07 15:20:00','suborbital rocket','unknown',728,'','DYANA DVM8','','','Aeron',636,'',78 from dual union all
select 63809,'1990-W73','1990-02-07 22:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',70 from dual union all
select 6381,'1960-A132','1960-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 63810,'1990-W74','1990-02-09 07:09:00','suborbital rocket','unknown',738,'','','','','Aeron',918,'KRENKEL''',57 from dual union all
select 63811,'1990-W75','1990-02-12 07:06:00','suborbital rocket','unknown',738,'','','','','Aeron',918,'KRENKEL''',62 from dual union all
select 63812,'1990-W76','1990-02-14 02:00:00','suborbital rocket','success',761,'MT-135P-R735','DYANA','','','Aeron',1332,'',58 from dual union all
select 63813,'1990-W77','1990-02-14 07:03:00','suborbital rocket','unknown',738,'','','','','Aeron',918,'KRENKEL''',55 from dual union all
select 63814,'1990-W78','1990-02-14 09:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',77 from dual union all
select 63815,'1990-W79','1990-02-14 12:12:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',67 from dual union all
select 63816,'1990-W80','1990-02-14 13:00:00','suborbital rocket','success',738,'','DYANA DZD2','','','Aeron',1917,'',79 from dual union all
select 63817,'1990-W81','1990-02-14 14:31:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',79 from dual union all
select 63818,'1990-W82','1990-02-14 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',70 from dual;
insert into LAUNCH
select 63819,'1990-W83','1990-02-16 07:03:00','suborbital rocket','unknown',738,'','','','','Aeron',918,'KRENKEL''',60 from dual union all
select 6382,'1960-A133','1960-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 63820,'1990-W84','1990-02-20 23:10:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',77 from dual union all
select 63821,'1990-W85','1990-02-21 02:00:00','suborbital rocket','success',761,'MT-135P-R736','DYANA','','','Aeron',1332,'',58 from dual union all
select 63822,'1990-W86','1990-02-21 02:00:00','suborbital rocket','success',759,'MT-135-51','DYANA parachute','','','Aeron',839,'',58 from dual union all
select 63823,'1990-W87','1990-02-21 02:54:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',75 from dual union all
select 63824,'1990-W88','1990-02-21 06:52:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',67 from dual union all
select 63825,'1990-W89','1990-02-21 07:03:00','suborbital rocket','unknown',738,'','','','','Aeron',918,'KRENKEL''',60 from dual union all
select 63826,'1990-W90','1990-02-21 10:53:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',70 from dual union all
select 63827,'1990-W91','1990-02-21 11:18:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',75 from dual union all
select 63828,'1990-W92','1990-02-21 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',77 from dual union all
select 63829,'1990-W93','1990-02-21 14:57:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',67 from dual union all
select 6383,'1960-A134','1960-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 63830,'1990-W94','1990-02-21 18:55:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',72 from dual union all
select 63831,'1990-W95','1990-02-22 00:32:00','suborbital rocket','unknown',728,'','','','','Aeron',1388,'SHIR',72 from dual union all
select 63832,'1990-W96','1990-02-23 07:03:00','suborbital rocket','unknown',738,'','','','','Aeron',918,'KRENKEL''',60 from dual union all
select 63833,'1990-W97','1990-02-26 14:40:00','suborbital rocket','unknown',728,'','DYANA DVM13','','','Aeron',636,'',75 from dual union all
select 63834,'1990-W98','1990-02-28 02:00:00','suborbital rocket','success',761,'MT-135P-R737','DYANA','','','Aeron',1332,'',55 from dual union all
select 63835,'1990-W99','1990-02-28 07:05:00','suborbital rocket','unknown',738,'','','','','Aeron',918,'KRENKEL''',56 from dual union all
select 63836,'1990-Y01','1990-01-02 19:57:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',84 from dual union all
select 63837,'1990-Y02','1990-01-02 20:27:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',80 from dual union all
select 63838,'1990-Y03','1990-01-02 23:00:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',84 from dual union all
select 63839,'1990-Y04','1990-01-03 00:15:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',84 from dual union all
select 6384,'1960-A135','1960-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 63840,'1990-Y05','1990-01-03 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 63841,'1990-Y06','1990-01-03 13:16:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 63842,'1990-Y07','1990-01-03 13:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 63843,'1990-Y08','1990-01-03 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 63844,'1990-Y09','1990-01-03 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 63845,'1990-Y10','1990-01-03 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',91 from dual union all
select 63846,'1990-Y100','1990-02-01 19:17:00','suborbital rocket','success',1502,'Chaff','DYANA DBC7','','','Aeron',307,'',99 from dual union all
select 63847,'1990-Y101','1990-02-01 19:47:00','suborbital rocket','success',1502,'Sphere','DYANA DBS8','','','Aeron',307,'',95 from dual union all
select 63848,'1990-Y102','1990-02-01 20:02:30','suborbital rocket','success',1284,'Datasonde','DYANA DAD27','','','Aeron',29,'',90 from dual union all
select 63849,'1990-Y103','1990-02-01 21:25:00','suborbital rocket','success',1284,'Datasonde','DYANA DED8','INTA DYSLD-9008','','Aeron',396,'',90 from dual union all
select 6385,'1960-A136','1960-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 63850,'1990-Y104','1990-02-02 04:19:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD6','','','Aeron',331,'',90 from dual union all
select 63851,'1990-Y105','1990-02-02 20:00:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD28','','','Aeron',29,'',78 from dual union all
select 63852,'1990-Y106','1990-02-05 11:45:00','suborbital rocket','unknown',728,'','DYANA DTM9','','','Aeron',1467,'',83 from dual union all
select 63853,'1990-Y107','1990-02-05 12:00:00','suborbital rocket','success',1284,'Datasonde SL-6','DYANA DUD5','','','Aeron',839,'',80 from dual union all
select 63854,'1990-Y108','1990-02-05 13:55:00','suborbital rocket','unknown',728,'','DYANA DVM7','','','Aeron',636,'',83 from dual union all
select 63855,'1990-Y109','1990-02-05 19:05:00','suborbital rocket','success',1502,'Chaff','DYANA DBC9','','','Aeron',307,'',96 from dual union all
select 63856,'1990-Y11','1990-01-03 16:00:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 63857,'1990-Y110','1990-02-05 19:45:00','suborbital rocket','success',1502,'Sphere','DYANA DBS10','','','Aeron',307,'',96 from dual union all
select 63858,'1990-Y111','1990-02-05 20:20:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD29','','','Aeron',29,'',81 from dual union all
select 63859,'1990-Y112','1990-02-05 22:14:00','suborbital rocket','success',1284,'Datasonde','DYANA DED10','INTA DYSLD-9010','','Aeron',396,'',90 from dual union all
select 6386,'1960-A137','1960-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 63860,'1990-Y113','1990-02-06 03:46:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD7','','','Aeron',331,'',90 from dual union all
select 63861,'1990-Y114','1990-02-06 20:00:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',80 from dual union all
select 63862,'1990-Y115','1990-02-06 23:00:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',88 from dual union all
select 63863,'1990-Y116','1990-02-07 00:15:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',84 from dual union all
select 63864,'1990-Y117','1990-02-07 14:08:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 63865,'1990-Y118','1990-02-07 14:10:00','suborbital rocket','unknown',728,'','DYANA DTM10','','','Aeron',1467,'',80 from dual union all
select 63866,'1990-Y119','1990-02-07 19:05:00','suborbital rocket','unknown',728,'','DYANA DHM8','','','Aeron',1010,'',82 from dual union all
select 63867,'1990-Y12','1990-01-03 20:45:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 63868,'1990-Y120','1990-02-07 19:56:30','suborbital rocket','success',1502,'Sphere','DYANA DAS30','','','Aeron',29,'',118 from dual union all
select 63869,'1990-Y121','1990-02-07 20:22:00','suborbital rocket','success',1502,'Chaff','DYANA DAC31','','','Aeron',29,'',105 from dual union all
select 6387,'1960-A138','1960-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 63870,'1990-Y122','1990-02-08 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 63871,'1990-Y123','1990-02-08 20:00:00','suborbital rocket','success',1284,'Sphere','DYANA DAS32','','','Aeron',29,'',84 from dual union all
select 63872,'1990-Y124','1990-02-08 20:26:00','suborbital rocket','success',1502,'Chaff','DYANA DBC11','','','Aeron',307,'',90 from dual union all
select 63873,'1990-Y125','1990-02-08 20:51:00','suborbital rocket','success',1502,'Sphere','DYANA DBS12','','','Aeron',307,'',90 from dual union all
select 63874,'1990-Y126','1990-02-08 21:12:00','suborbital rocket','success',1284,'Datasonde','DYANA DED11','INTA DYSLD-9011','','Aeron',396,'',90 from dual union all
select 63875,'1990-Y127','1990-02-08 21:17:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD34','','','Aeron',29,'',82 from dual union all
select 63876,'1990-Y128','1990-02-09 04:05:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD8','','','Aeron',331,'',90 from dual union all
select 63877,'1990-Y129','1990-02-09 11:10:00','suborbital rocket','unknown',728,'','DYANA DTM11','','','Aeron',1467,'',80 from dual union all
select 63878,'1990-Y13','1990-01-04 08:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',82 from dual union all
select 63879,'1990-Y130','1990-02-10 21:03:00','suborbital rocket','success',1502,'Sphere','DYANA DAS35','','','Aeron',29,'',106 from dual union all
select 6388,'1960-A139','1960-06-03 00:00:00','military missile','success',843,'','Corporal intercept','','','Test',1895,'',10 from dual union all
select 63880,'1990-Y131','1990-02-12 12:50:00','suborbital rocket','unknown',728,'','DYANA DTM12','','','Aeron',1467,'',80 from dual union all
select 63881,'1990-Y132','1990-02-12 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM9','','','Aeron',1010,'',80 from dual union all
select 63882,'1990-Y133','1990-02-12 15:10:00','suborbital rocket','unknown',728,'','DYANA DVM9','','','Aeron',636,'',81 from dual union all
select 63883,'1990-Y134','1990-02-12 20:00:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD36','','','Aeron',29,'',86 from dual union all
select 63884,'1990-Y135','1990-02-12 21:00:00','suborbital rocket','success',1284,'Datasonde','DYANA DED12','INTA DYSLD-9012','','Aeron',396,'',90 from dual union all
select 63885,'1990-Y136','1990-02-13 03:45:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD9','','','Aeron',331,'',90 from dual union all
select 63886,'1990-Y137','1990-02-13 20:00:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',80 from dual union all
select 63887,'1990-Y138','1990-02-13 23:10:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',87 from dual union all
select 63888,'1990-Y139','1990-02-14 11:45:00','suborbital rocket','unknown',728,'','DYANA DTM13','','','Aeron',1467,'',85 from dual union all
select 63889,'1990-Y14','1990-01-09 23:00:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',84 from dual union all
select 6389,'1960-A14','1960-01-08 00:00:00','atmospheric rocket','success',1163,'','Mercury model','','','Reentry Test',1821,'',34 from dual union all
select 63890,'1990-Y140','1990-02-14 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM10','','','Aeron',1010,'',80 from dual union all
select 63891,'1990-Y141','1990-02-14 15:12:00','suborbital rocket','unknown',728,'','DYANA DVM10','','','Aeron',636,'',82 from dual union all
select 63892,'1990-Y142','1990-02-14 20:00:00','suborbital rocket','success',1502,'Sphere','DYANA DAS37','','','Aeron',29,'',114 from dual union all
select 63893,'1990-Y143','1990-02-15 20:02:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD40','','','Aeron',29,'',69 from dual union all
select 63894,'1990-Y144','1990-02-15 21:34:00','suborbital rocket','success',1284,'Datasonde','DYANA DED13','INTA DYSLD-9013','','Aeron',396,'',90 from dual union all
select 63895,'1990-Y145','1990-02-16 03:55:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD10','','','Aeron',331,'',90 from dual union all
select 63896,'1990-Y146','1990-02-16 11:10:00','suborbital rocket','unknown',728,'','DYANA DTM14','','','Aeron',1467,'',87 from dual union all
select 63897,'1990-Y147','1990-02-17 12:00:00','suborbital rocket','success',1502,'Sphere VP-5','DYANA DUS8','','','Aeron',839,'',112 from dual union all
select 63898,'1990-Y148','1990-02-17 20:00:00','suborbital rocket','success',1284,'Sphere','DYANA DAS41','','','Aeron',29,'',85 from dual union all
select 63899,'1990-Y149','1990-02-17 20:24:48','suborbital rocket','success',1502,'Chaff','DYANA DAC42','','','Aeron',29,'',90 from dual union all
select 639,'1946-A22','1946-12-03 00:00:00','suborbital rocket','success',1517,'22','Modified WAC A','','','Test',1852,'',20 from dual union all
select 6390,'1960-A140','1960-06-03 00:00:00','military missile','success',764,'58-1391','','','','Test',243,'',12 from dual union all
select 63900,'1990-Y15','1990-01-10 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 63901,'1990-Y150','1990-02-17 21:02:00','suborbital rocket','success',1284,'Sphere','DYANA DAS43','','','Aeron',29,'',83 from dual union all
select 63902,'1990-Y151','1990-02-19 11:40:00','suborbital rocket','unknown',728,'','DYANA DTM15','','','Aeron',1467,'',81 from dual union all
select 63903,'1990-Y152','1990-02-19 14:16:00','suborbital rocket','unknown',728,'','DYANA DVM11','','','Aeron',636,'',85 from dual union all
select 63904,'1990-Y153','1990-02-19 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM11','','','Aeron',1010,'',82 from dual union all
select 63905,'1990-Y154','1990-02-19 20:03:35','suborbital rocket','success',1284,'Datasonde','DYANA DAD44','','','Aeron',29,'',77 from dual union all
select 63906,'1990-Y155','1990-02-19 21:35:00','suborbital rocket','success',1284,'Datasonde','DYANA DED14','INTA DYSLD-9014','','Aeron',396,'',90 from dual union all
select 63907,'1990-Y156','1990-02-20 03:00:00','suborbital rocket','success',1502,'Chaff','DYANA DBC13','','','Aeron',307,'',90 from dual union all
select 63908,'1990-Y157','1990-02-20 03:25:00','suborbital rocket','success',1502,'Sphere','DYANA DBS14','','','Aeron',307,'',90 from dual;
insert into LAUNCH
select 63909,'1990-Y158','1990-02-20 03:45:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD11','','','Aeron',331,'',90 from dual union all
select 6391,'1960-A141','1960-06-03 00:00:00','suborbital spaceplane','failure',1520,'1-A-15','','','','Test',379,'NB-52 003',14 from dual union all
select 63910,'1990-Y159','1990-02-20 03:50:00','suborbital rocket','success',1502,'Chaff','DYANA DBC15','','','Aeron',307,'',90 from dual union all
select 63911,'1990-Y16','1990-01-10 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',84 from dual union all
select 63912,'1990-Y160','1990-02-20 05:14:00','suborbital rocket','success',1502,'Sphere','DYANA DBS16','','','Aeron',307,'',90 from dual union all
select 63913,'1990-Y161','1990-02-20 05:39:00','suborbital rocket','success',1502,'Chaff','DYANA DBC17','','','Aeron',307,'',90 from dual union all
select 63914,'1990-Y162','1990-02-20 06:04:00','suborbital rocket','success',1502,'Chaff','DYANA DBC18','','','Aeron',307,'',89 from dual union all
select 63915,'1990-Y163','1990-02-20 06:29:00','suborbital rocket','success',1502,'Sphere','DYANA DBS19','','','Aeron',307,'',90 from dual union all
select 63916,'1990-Y164','1990-02-20 06:54:00','suborbital rocket','success',1502,'Chaff','DYANA DBC20','','','Aeron',307,'',90 from dual union all
select 63917,'1990-Y165','1990-02-20 20:16:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',80 from dual union all
select 63918,'1990-Y166','1990-02-21 11:22:00','suborbital rocket','unknown',728,'','DYANA DTM16','','','Aeron',1467,'',86 from dual union all
select 63919,'1990-Y167','1990-02-21 14:20:00','suborbital rocket','unknown',728,'','DYANA DVM12','','','Aeron',636,'',82 from dual union all
select 6392,'1960-A142','1960-06-08 00:00:00','military missile','success',771,'GM-56-1927','','','','Test',222,'',12 from dual union all
select 63920,'1990-Y168','1990-02-21 16:10:00','suborbital rocket','unknown',728,'','DYANA DHM12','','','Aeron',1010,'',80 from dual union all
select 63921,'1990-Y169','1990-02-21 20:05:42','suborbital rocket','success',1502,'Sphere','DYANA DAS45','','','Aeron',29,'',117 from dual union all
select 63922,'1990-Y17','1990-01-10 11:34:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 63923,'1990-Y170','1990-02-22 19:06:00','suborbital rocket','success',1502,'Chaff','DYANA DBC21','','','Aeron',307,'',90 from dual union all
select 63924,'1990-Y171','1990-02-22 19:30:00','suborbital rocket','success',1502,'Sphere','DYANA DBS22','','','Aeron',307,'',90 from dual union all
select 63925,'1990-Y172','1990-02-22 20:15:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD46','','','Aeron',29,'',80 from dual union all
select 63926,'1990-Y173','1990-02-22 21:13:00','suborbital rocket','success',1284,'Datasonde','DYANA DED15','INTA DYSLD-9015','','Aeron',396,'',90 from dual union all
select 63927,'1990-Y174','1990-02-23 03:46:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD12','','','Aeron',331,'',90 from dual union all
select 63928,'1990-Y175','1990-02-23 11:10:00','suborbital rocket','unknown',728,'','DYANA DTM17','','','Aeron',1467,'',85 from dual union all
select 63929,'1990-Y176','1990-02-24 12:00:00','suborbital rocket','success',1502,'Sphere VP-7','DYANA DUS9','','','Aeron',839,'',112 from dual union all
select 6393,'1960-A143','1960-06-08 00:00:00','military missile','success',771,'GM-55-427','','','','Test',222,'',12 from dual union all
select 63930,'1990-Y177','1990-02-24 20:48:55','suborbital rocket','success',1502,'Sphere','DYANA DAS47','','','Aeron',29,'',105 from dual union all
select 63931,'1990-Y178','1990-02-25 18:25:00','suborbital rocket','success',1502,'Chaff','DYANA DAC48','','','Aeron',29,'',95 from dual union all
select 63932,'1990-Y179','1990-02-25 19:04:00','suborbital rocket','success',1502,'Sphere','DYANA DAS49','','','Aeron',29,'',111 from dual union all
select 63933,'1990-Y18','1990-01-10 13:20:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',86 from dual union all
select 63934,'1990-Y180','1990-02-25 19:40:00','suborbital rocket','success',1502,'Chaff','DYANA DAC51','','','Aeron',29,'',95 from dual union all
select 63935,'1990-Y181','1990-02-25 20:04:00','suborbital rocket','success',1502,'Sphere','DYANA DAS52','','','Aeron',29,'',117 from dual union all
select 63936,'1990-Y182','1990-02-26 11:35:00','suborbital rocket','unknown',728,'','DYANA DTM18','','','Aeron',1467,'',85 from dual union all
select 63937,'1990-Y183','1990-02-26 12:00:00','suborbital rocket','success',1502,'Sphere VP-8','DYANA DUS10','','','Aeron',839,'',112 from dual union all
select 63938,'1990-Y184','1990-02-26 20:00:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD53','','','Aeron',29,'',84 from dual union all
select 63939,'1990-Y185','1990-02-26 21:30:00','suborbital rocket','success',1284,'Datasonde','DYANA DED16','INTA DYSLD-9016','','Aeron',396,'',90 from dual union all
select 6394,'1960-A144','1960-06-08 00:00:00','military missile','success',771,'GM-54-63','','','','Test',222,'',12 from dual union all
select 63940,'1990-Y186','1990-02-27 03:45:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD13','','','Aeron',331,'',90 from dual union all
select 63941,'1990-Y187','1990-02-27 20:04:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',81 from dual union all
select 63942,'1990-Y188','1990-02-27 23:00:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',88 from dual union all
select 63943,'1990-Y189','1990-02-28 11:10:00','suborbital rocket','unknown',728,'','DYANA DTM19','','','Aeron',1467,'',80 from dual union all
select 63944,'1990-Y19','1990-01-10 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',85 from dual union all
select 63945,'1990-Y190','1990-02-28 14:09:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 63946,'1990-Y191','1990-02-28 21:09:00','suborbital rocket','success',1502,'Sphere','DYANA DAS55','','','Aeron',29,'',117 from dual union all
select 63947,'1990-Y192','1990-03-01 19:30:00','suborbital rocket','success',1502,'Chaff','DYANA DBC23','','','Aeron',307,'',90 from dual union all
select 63948,'1990-Y193','1990-03-01 20:00:00','suborbital rocket','success',1502,'Sphere','DYANA DBS24','','','Aeron',307,'',90 from dual union all
select 63949,'1990-Y194','1990-03-01 20:42:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD57','','','Aeron',29,'',83 from dual union all
select 6395,'1960-A145','1960-06-08 00:00:00','suborbital spaceplane','failure',1520,'1-A-16','','','','Test',379,'NB-52 003',14 from dual union all
select 63950,'1990-Y195','1990-03-01 22:03:00','suborbital rocket','success',1284,'Datasonde','DYANA DED18','INTA DYSLD-9018','','Aeron',396,'',90 from dual union all
select 63951,'1990-Y196','1990-03-02 03:52:16','suborbital rocket','success',1284,'Datasonde','DYANA DCD14','','','Aeron',331,'',90 from dual union all
select 63952,'1990-Y197','1990-03-02 11:35:00','suborbital rocket','unknown',728,'','DYANA DTM20','','','Aeron',1467,'',85 from dual union all
select 63953,'1990-Y198','1990-03-02 19:31:00','suborbital rocket','success',1502,'Sphere','DYANA DAS58','','','Aeron',29,'',101 from dual union all
select 63954,'1990-Y199','1990-03-04 15:27:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 63955,'1990-Y20','1990-01-11 21:21:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD1','','','Aeron',29,'',90 from dual union all
select 63956,'1990-Y200','1990-03-05 19:01:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD59','','','Aeron',29,'',84 from dual union all
select 63957,'1990-Y201','1990-03-05 22:20:00','suborbital rocket','success',1284,'Datasonde','DYANA DED19','INTA DYSLD-9019','','Aeron',396,'',90 from dual union all
select 63958,'1990-Y202','1990-03-06 02:03:00','suborbital rocket','success',1502,'Chaff','DYANA DAC60','','','Aeron',29,'',98 from dual union all
select 63959,'1990-Y203','1990-03-06 02:19:00','suborbital rocket','success',1502,'Sphere','DYANA DAS61','','','Aeron',29,'',117 from dual union all
select 6396,'1960-A146','1960-06-09 00:00:00','military missile','success',771,'GM-55-426','','','','Test',222,'',12 from dual union all
select 63960,'1990-Y204','1990-03-06 03:01:00','suborbital rocket','success',1502,'Chaff','DYANA DAC63','','','Aeron',29,'',99 from dual union all
select 63961,'1990-Y205','1990-03-06 03:31:00','suborbital rocket','success',1502,'Sphere','DYANA DAS64','','','Aeron',29,'',115 from dual union all
select 63962,'1990-Y206','1990-03-06 03:45:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD15','','','Aeron',331,'',90 from dual union all
select 63963,'1990-Y207','1990-03-06 05:03:30','suborbital rocket','success',1502,'Sphere','DYANA DBS26','','','Aeron',307,'',110 from dual union all
select 63964,'1990-Y208','1990-03-06 05:38:00','suborbital rocket','success',1502,'Chaff','DYANA DBC27','','','Aeron',307,'',90 from dual union all
select 63965,'1990-Y209','1990-03-06 05:58:30','suborbital rocket','success',1502,'Sphere','DYANA DBS28','','','Aeron',307,'',109 from dual union all
select 63966,'1990-Y21','1990-01-11 22:39:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD2','','','Aeron',29,'',66 from dual union all
select 63967,'1990-Y210','1990-03-06 06:20:00','suborbital rocket','success',1502,'Sphere','DYANA DBS29','','','Aeron',307,'',109 from dual union all
select 63968,'1990-Y211','1990-03-06 06:40:00','suborbital rocket','success',1502,'Chaff','DYANA DBC30','','','Aeron',307,'',90 from dual union all
select 63969,'1990-Y212','1990-03-06 07:00:00','suborbital rocket','success',1502,'Chaff','DYANA DBC31','','','Aeron',307,'',90 from dual union all
select 6397,'1960-A147','1960-06-10 00:00:00','military missile','success',771,'GM-54-124','','','','Test',222,'',12 from dual union all
select 63970,'1990-Y213','1990-03-06 23:00:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',86 from dual union all
select 63971,'1990-Y214','1990-03-07 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM16','','','Aeron',1010,'',85 from dual union all
select 63972,'1990-Y215','1990-03-07 19:32:00','suborbital rocket','success',1502,'Sphere','DYANA DAS65','','','Aeron',29,'',117 from dual union all
select 63973,'1990-Y216','1990-03-07 19:59:00','suborbital rocket','success',1502,'Chaff','DYANA DAC66','','','Aeron',29,'',99 from dual union all
select 63974,'1990-Y217','1990-03-07 20:30:00','suborbital rocket','success',1502,'Chaff','DYANA DAC67','','','Aeron',29,'',101 from dual union all
select 63975,'1990-Y218','1990-03-07 21:14:00','suborbital rocket','success',1502,'Sphere','DYANA DAS68','','','Aeron',29,'',115 from dual union all
select 63976,'1990-Y219','1990-03-07 22:52:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 63977,'1990-Y22','1990-01-12 21:00:00','suborbital rocket','success',1502,'Sphere','DYANA DAS3','','','Aeron',29,'',113 from dual union all
select 63978,'1990-Y220','1990-03-08 19:00:00','suborbital rocket','success',1502,'Chaff','DYANA DBC32','','','Aeron',307,'',90 from dual union all
select 63979,'1990-Y221','1990-03-08 19:04:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD69','','','Aeron',29,'',80 from dual union all
select 6398,'1960-A148','1960-06-10 00:00:00','military missile','success',771,'GM-54-125','','','','Test',222,'',12 from dual union all
select 63980,'1990-Y222','1990-03-08 19:30:00','suborbital rocket','success',1502,'Chaff','DYANA DBC33','','','Aeron',307,'',90 from dual union all
select 63981,'1990-Y223','1990-03-08 21:15:00','suborbital rocket','success',1284,'Datasonde','DYANA DED20','INTA DYSLD-9020','','Aeron',396,'',90 from dual union all
select 63982,'1990-Y224','1990-03-08 21:41:00','suborbital rocket','success',1502,'Chaff','DYANA DAC70','','','Aeron',29,'',99 from dual union all
select 63983,'1990-Y225','1990-03-08 22:10:00','suborbital rocket','success',1502,'Chaff','DYANA DAC71','','','Aeron',29,'',95 from dual union all
select 63984,'1990-Y226','1990-03-08 22:37:00','suborbital rocket','success',1502,'Sphere','DYANA DAS72','','','Aeron',29,'',119 from dual union all
select 63985,'1990-Y227','1990-03-08 23:14:00','suborbital rocket','success',1502,'Chaff','DYANA DAC74','','','Aeron',29,'',97 from dual union all
select 63986,'1990-Y228','1990-03-08 23:45:00','suborbital rocket','success',1502,'Chaff','DYANA DAC75','','','Aeron',29,'',90 from dual union all
select 63987,'1990-Y229','1990-03-09 00:45:00','suborbital rocket','success',1502,'Sphere','DYANA DAS77','','','Aeron',29,'',114 from dual union all
select 63988,'1990-Y23','1990-01-12 21:48:05','suborbital rocket','success',1502,'Sphere','DYANA DAS4','','','Aeron',29,'',102 from dual union all
select 63989,'1990-Y230','1990-03-09 01:05:00','suborbital rocket','success',1502,'Chaff','DYANA DAC78','','','Aeron',29,'',101 from dual union all
select 6399,'1960-A149','1960-06-13 00:00:00','military missile','success',771,'GM-54-61','','','','Test',222,'',12 from dual union all
select 63990,'1990-Y231','1990-03-09 05:02:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD17','','','Aeron',331,'',90 from dual union all
select 63991,'1990-Y232','1990-03-09 20:19:00','suborbital rocket','success',1502,'Sphere','DYANA DAS79','','','Aeron',29,'',106 from dual union all
select 63992,'1990-Y233','1990-03-11 19:21:00','suborbital rocket','success',1502,'Sphere','DYANA DAS80','','','Aeron',29,'',113 from dual union all
select 63993,'1990-Y234','1990-03-11 19:41:00','suborbital rocket','success',1502,'Chaff','DYANA DAC81','','','Aeron',29,'',103 from dual union all
select 63994,'1990-Y235','1990-03-11 20:24:00','suborbital rocket','success',1502,'Sphere','DYANA DAS83','','','Aeron',29,'',112 from dual union all
select 63995,'1990-Y236','1990-03-11 21:03:00','suborbital rocket','success',1502,'Chaff','DYANA DAC85','','','Aeron',29,'',97 from dual union all
select 63996,'1990-Y237','1990-03-11 21:32:00','suborbital rocket','success',1502,'Sphere','DYANA DAS86','','','Aeron',29,'',114 from dual union all
select 63997,'1990-Y238','1990-03-11 21:52:00','suborbital rocket','success',1502,'Chaff','DYANA DAC87','','',' Aeron',29,'',96 from dual union all
select 63998,'1990-Y239','1990-03-11 22:24:00','suborbital rocket','success',1502,'Chaff','DYANA DAC88','','','Aeron',29,'',97 from dual union all
select 63999,'1990-Y24','1990-01-13 21:34:00','suborbital rocket','success',1502,'Sphere','DYANA DAS5','','','Aeron',29,'',92 from dual;
insert into LAUNCH
select 64,'1944-A28','1944-12-14 15:12:00','military missile','failure',1470,'19865','Ma325','','','Test',661,'',1 from dual union all
select 640,'1946-A23','1946-12-06 00:00:00','suborbital rocket','success',1517,'23','','','','Test',1852,'',27 from dual union all
select 6400,'1960-A15','1960-01-13 00:00:00','military missile','success',771,'GM-54-54','','','','Test',222,'',12 from dual union all
select 64000,'1990-Y240','1990-03-12 14:00:00','suborbital rocket','unknown',728,'','DYANA DTM22','','','Aeron',1467,'',80 from dual union all
select 64001,'1990-Y241','1990-03-12 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM17','','','Aeron',1010,'',81 from dual union all
select 64002,'1990-Y242','1990-03-12 20:00:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD89','','','Aeron',29,'',79 from dual union all
select 64003,'1990-Y243','1990-03-12 21:29:00','suborbital rocket','success',1284,'Datasonde','DYANA DED21','INTA DYSLD-9021','','Aeron',396,'',90 from dual union all
select 64004,'1990-Y244','1990-03-13 02:00:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',84 from dual union all
select 64005,'1990-Y245','1990-03-13 02:45:00','suborbital rocket','success',1502,'Sphere','DYANA DBS34','','','Aeron',307,'',93 from dual union all
select 64006,'1990-Y246','1990-03-13 03:10:00','suborbital rocket','success',1502,'Chaff','DYANA DBC35','','','Aeron',307,'',98 from dual union all
select 64007,'1990-Y247','1990-03-13 03:40:00','suborbital rocket','success',1502,'Sphere','DYANA DBS36','','','Aeron',307,'',95 from dual union all
select 64008,'1990-Y248','1990-03-13 03:45:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD18','','','Aeron',331,'',90 from dual union all
select 64009,'1990-Y249','1990-03-13 04:04:00','suborbital rocket','success',1502,'Chaff','DYANA DBC37','','','Aeron',307,'',104 from dual union all
select 6401,'1960-A150','1960-06-13 00:00:00','military missile','success',771,'GM-56-1918','','','','Test',222,'',12 from dual union all
select 64010,'1990-Y25','1990-01-15 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 64011,'1990-Y250','1990-03-13 04:40:00','suborbital rocket','success',1502,'Sphere','DYANA DBS38','','','Aeron',307,'',90 from dual union all
select 64012,'1990-Y251','1990-03-13 05:00:00','suborbital rocket','success',1502,'Chaff','DYANA DBC39','','','Aeron',307,'',102 from dual union all
select 64013,'1990-Y252','1990-03-13 05:20:00','suborbital rocket','success',1502,'Sphere','DYANA DBS40','','','Aeron',307,'',95 from dual union all
select 64014,'1990-Y253','1990-03-14 14:53:00','suborbital rocket','unknown',728,'','DYANA DVM18','','','Aeron',636,'',85 from dual union all
select 64015,'1990-Y254','1990-03-14 19:30:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD90','','','Aeron',29,'',74 from dual union all
select 64016,'1990-Y255','1990-03-15 19:21:00','suborbital rocket','success',1502,'Sphere','DYANA DBS41','','','Aeron',307,'',90 from dual union all
select 64017,'1990-Y256','1990-03-15 21:14:00','suborbital rocket','success',1284,'Datasonde','DYANA DED22','INTA DYSLD-9022','','Aeron',396,'',90 from dual union all
select 64018,'1990-Y257','1990-03-16 03:45:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD19','','','Aeron',331,'',90 from dual union all
select 64019,'1990-Y258','1990-03-20 19:40:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',80 from dual union all
select 6402,'1960-A151','1960-06-15 00:00:00','military missile','success',771,'GM-56-1919','','','','Test',222,'',12 from dual union all
select 64020,'1990-Y259','1990-03-20 23:00:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',83 from dual union all
select 64021,'1990-Y26','1990-01-15 12:40:00','suborbital rocket','success',1502,'Chaff','DYANA DBC1','','','Aeron',307,'',92 from dual union all
select 64022,'1990-Y260','1990-03-27 19:55:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',80 from dual union all
select 64023,'1990-Y261','1990-03-27 23:00:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',86 from dual union all
select 64024,'1990-Y262','1990-03-28 11:32:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 64025,'1990-Y263','1990-03-28 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',80 from dual union all
select 64026,'1990-Y264','1990-03-28 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 64027,'1990-Y265','1990-03-28 15:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 64028,'1990-Y266','1990-04-11 23:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',95 from dual union all
select 64029,'1990-Y267','1990-04-25 21:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 6403,'1960-A152','1960-06-16 00:00:00','military missile','success',1214,'SM-62A N-3445','','','','OT',224,'',10 from dual union all
select 64030,'1990-Y268','1990-05-08 21:52:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 64031,'1990-Y269','1990-05-08 22:33:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',92 from dual union all
select 64032,'1990-Y27','1990-01-15 14:32:00','suborbital rocket','unknown',728,'','DYANA DVM1','','','Aeron',636,'',84 from dual union all
select 64033,'1990-Y270','1990-05-23 21:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 64034,'1990-Y271','1990-09-01 12:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 64035,'1990-Y272','1990-09-19 21:26:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 64036,'1990-Y273','1990-09-25 21:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 64037,'1990-Y274','1990-10-06 12:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',83 from dual union all
select 64038,'1990-Y275','1990-10-10 08:33:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 64039,'1990-Y276','1990-10-10 09:31:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',86 from dual union all
select 6404,'1960-A153','1960-06-16 05:29:00','suborbital rocket','failure',839,'NASA 10.03GA','GSFC Grenade 1','','','Aeronomy',1821,'',10 from dual union all
select 64040,'1990-Y277','1990-10-17 21:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 64041,'1990-Y278','1990-10-24 21:35:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 64042,'1990-Y279','1990-10-31 22:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 64043,'1990-Y28','1990-01-15 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM1','','','Aeron',1010,'',80 from dual union all
select 64044,'1990-Y280','1990-11-02 17:53:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',84 from dual union all
select 64045,'1990-Y281','1990-11-06 11:15:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',80 from dual union all
select 64046,'1990-Y282','1990-11-06 14:25:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',83 from dual union all
select 64047,'1990-Y283','1990-11-06 23:00:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',83 from dual union all
select 64048,'1990-Y284','1990-11-07 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',81 from dual union all
select 64049,'1990-Y285','1990-11-07 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 6405,'1960-A154','1960-06-24 00:00:00','military missile','success',764,'56-2898','','','','Test',243,'',12 from dual union all
select 64050,'1990-Y286','1990-11-09 17:00:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',82 from dual union all
select 64051,'1990-Y287','1990-11-13 23:00:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',83 from dual union all
select 64052,'1990-Y288','1990-11-14 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',80 from dual union all
select 64053,'1990-Y289','1990-11-14 11:45:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 64054,'1990-Y29','1990-01-15 19:15:00','suborbital rocket','success',1502,'Sphere','DYANA DBS2','','','Aeron',307,'',92 from dual union all
select 64055,'1990-Y290','1990-11-14 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 64056,'1990-Y291','1990-11-14 16:10:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',81 from dual union all
select 64057,'1990-Y292','1990-11-14 18:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',86 from dual union all
select 64058,'1990-Y293','1990-11-20 22:35:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',87 from dual union all
select 64059,'1990-Y294','1990-11-21 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',80 from dual union all
select 6406,'1960-A155','1960-06-28 00:00:00','military missile','success',771,'GM-56-1914','','','','Test',222,'',12 from dual union all
select 64060,'1990-Y295','1990-11-21 14:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 64061,'1990-Y296','1990-11-21 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',86 from dual union all
select 64062,'1990-Y297','1990-11-21 18:37:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 64063,'1990-Y298','1990-11-27 23:00:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',84 from dual union all
select 64064,'1990-Y299','1990-11-28 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 64065,'1990-Y30','1990-01-15 20:00:00','suborbital rocket','success',1502,'Chaff','DYANA DBC3','','','Aeron',307,'',92 from dual union all
select 64066,'1990-Y300','1990-11-28 16:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',83 from dual union all
select 64067,'1990-Y301','1990-11-28 18:46:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',84 from dual union all
select 64068,'1990-Y302','1990-11-28 21:30:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',80 from dual union all
select 64069,'1990-Y303','1990-11-29 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',83 from dual union all
select 6407,'1960-A156','1960-06-28 00:00:00','military missile','success',771,'GM-56-1915','','','','Test',222,'',12 from dual union all
select 64070,'1990-Y304','1990-12-04 19:55:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',83 from dual union all
select 64071,'1990-Y305','1990-12-04 23:40:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',85 from dual union all
select 64072,'1990-Y306','1990-12-05 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',81 from dual union all
select 64073,'1990-Y307','1990-12-05 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',82 from dual union all
select 64074,'1990-Y308','1990-12-05 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',81 from dual union all
select 64075,'1990-Y309','1990-12-05 15:45:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual union all
select 64076,'1990-Y31','1990-01-15 20:05:00','suborbital rocket','success',1502,'Sphere','DYANA DAS6','','','Aeron',29,'',109 from dual union all
select 64077,'1990-Y310','1990-12-10 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',84 from dual union all
select 64078,'1990-Y311','1990-12-11 19:55:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',83 from dual union all
select 64079,'1990-Y312','1990-12-11 23:00:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',85 from dual union all
select 6408,'1960-A157','1960-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64080,'1990-Y313','1990-12-12 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 64081,'1990-Y314','1990-12-12 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 64082,'1990-Y315','1990-12-12 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',81 from dual union all
select 64083,'1990-Y316','1990-12-14 12:50:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',82 from dual union all
select 64084,'1990-Y317','1990-12-18 19:55:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',82 from dual union all
select 64085,'1990-Y318','1990-12-19 14:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',84 from dual union all
select 64086,'1990-Y319','1990-12-19 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 64087,'1990-Y32','1990-01-15 21:40:00','suborbital rocket','failure',1284,'Datasonde','DYANA DED1','INTA DYSLD-9001','','Aeron',396,'',80 from dual union all
select 64088,'1990-Y320','1990-12-19 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',82 from dual;
insert into LAUNCH
select 64089,'1990-Y321','1990-12-25 20:40:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',82 from dual union all
select 6409,'1960-A158','1960-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64090,'1990-Y322','1990-12-25 23:00:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',82 from dual union all
select 64091,'1990-Y323','1990-12-26 11:30:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',84 from dual union all
select 64092,'1990-Y324','1990-12-26 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',85 from dual union all
select 64093,'1990-Y325','1990-12-26 14:21:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',83 from dual union all
select 64094,'1990-Y326','1990-12-26 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 64095,'1990-Y327','1990-02-15 12:00:00','suborbital rocket','success',1502,'VP-4','','','','Aeron',839,'',112 from dual union all
select 64096,'1990-Y328','1990-02-15 12:00:00','suborbital rocket','success',1502,'VP-3','','','','Aeron',839,'',112 from dual union all
select 64097,'1990-Y329','1990-03-06 03:26:00','suborbital rocket','failure',1284,'','SISSI So1','','','Aeron',409,'',0 from dual union all
select 64098,'1990-Y33','1990-01-15 22:56:00','suborbital rocket','failure',1284,'Datasonde','DYANA DED2','INTA DYSLD-9002','','Aeron',396,'',80 from dual union all
select 64099,'1990-Y330','1990-07-25 21:00:00','suborbital rocket','success',1284,'Robin Dart','SISSI So2','','','Aeron',409,'',81 from dual union all
select 641,'1946-A24','1946-12-12 00:00:00','suborbital rocket','success',1517,'24','','','','Test',1852,'',32 from dual union all
select 6410,'1960-A159','1960-06-30 00:00:00','military missile','success',928,'A1X','','','','Test',1338,'',2 from dual union all
select 64100,'1990-Y331','1990-07-26 22:05:00','suborbital rocket','success',1284,'Robin Dart','SISSI So3','','','Aeron',409,'',81 from dual union all
select 64101,'1990-Y332','1990-07-26 23:25:00','suborbital rocket','success',1502,'Robin Sphere','SISSI RD1','','','Aeron',417,'',115 from dual union all
select 64102,'1990-Y333','1990-07-26 23:55:00','suborbital rocket','success',1284,'Robin Dart','SISSI So4','','','Aeron',409,'',84 from dual union all
select 64103,'1990-Y334','1990-08-02 23:01:00','suborbital rocket','success',1502,'Robin Sphere','SISSI RD2','','','Aeron',417,'',118 from dual union all
select 64104,'1990-Y34','1990-01-16 04:01:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD1','','','Aeron',331,'',90 from dual union all
select 64105,'1990-Y35','1990-01-16 20:12:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',80 from dual union all
select 64106,'1990-Y36','1990-01-16 23:01:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',83 from dual union all
select 64107,'1990-Y37','1990-01-17 11:10:00','suborbital rocket','unknown',728,'','DYANA DTM2','','','Aeron',1467,'',80 from dual union all
select 64108,'1990-Y38','1990-01-17 11:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',80 from dual union all
select 64109,'1990-Y39','1990-01-17 13:10:00','suborbital rocket','success',1284,'Datasonde SL-2','DYANA DUD1','','','Aeron',839,'',80 from dual union all
select 6411,'1960-A16','1960-01-13 00:00:00','military missile','success',771,'GM-54-55','','','','Test',222,'',12 from dual union all
select 64110,'1990-Y40','1990-01-17 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM2','','','Aeron',1010,'',82 from dual union all
select 64111,'1990-Y41','1990-01-17 16:30:00','suborbital rocket','unknown',728,'','DYANA DVM2','','','Aeron',636,'',83 from dual union all
select 64112,'1990-Y42','1990-01-17 20:00:00','suborbital rocket','success',1502,'Sphere','DYANA DAS7','','','Aeron',29,'',119 from dual union all
select 64113,'1990-Y43','1990-01-17 20:42:35','suborbital rocket','success',1502,'Chaff','DYANA DAC8','','','Aeron',29,'',111 from dual union all
select 64114,'1990-Y44','1990-01-18 19:35:00','suborbital rocket','success',1502,'Chaff','DYANA DBC4','','','Aeron',307,'',90 from dual union all
select 64115,'1990-Y45','1990-01-18 20:04:00','suborbital rocket','success',1502,'Sphere','DYANA DBS5','','','Aeron',307,'',90 from dual union all
select 64116,'1990-Y46','1990-01-18 20:29:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD9','','','Aeron',29,'',90 from dual union all
select 64117,'1990-Y47','1990-01-18 21:00:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',82 from dual union all
select 64118,'1990-Y48','1990-01-18 21:09:00','suborbital rocket','success',1284,'Datasonde','DYANA DED3','INTA DYSLD-9003','','Aeron',396,'',90 from dual union all
select 64119,'1990-Y49','1990-01-18 21:15:40','suborbital rocket','success',1284,'Datasonde','DYANA DAD10','','','Aeron',29,'',70 from dual union all
select 6412,'1960-A160','1960-07-01 17:29:00','military missile','failure',1419,'J-2','Mk 4-2 RV','','','Test',239,'',0 from dual union all
select 64120,'1990-Y50','1990-01-19 04:07:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD2','','','Aeron',331,'',90 from dual union all
select 64121,'1990-Y51','1990-01-19 06:04:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 64122,'1990-Y52','1990-01-19 06:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',94 from dual union all
select 64123,'1990-Y53','1990-01-19 11:10:00','suborbital rocket','unknown',728,'','DYANA DTM3','','','Aeron',1467,'',80 from dual union all
select 64124,'1990-Y54','1990-01-19 20:00:00','suborbital rocket','success',1502,'Sphere','DYANA DAS11','','','Aeron',29,'',114 from dual union all
select 64125,'1990-Y55','1990-01-22 09:55:00','suborbital rocket','success',1502,'Sphere','DYANA DAS12','','','Aeron',29,'',111 from dual union all
select 64126,'1990-Y56','1990-01-22 11:10:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',80 from dual union all
select 64127,'1990-Y57','1990-01-22 11:21:00','suborbital rocket','success',1502,'Chaff','DYANA DAC14','','','Aeron',29,'',103 from dual union all
select 64128,'1990-Y58','1990-01-22 11:45:00','suborbital rocket','success',1502,'Sphere','DYANA DAS15','','','Aeron',29,'',113 from dual union all
select 64129,'1990-Y59','1990-01-22 13:32:00','suborbital rocket','unknown',728,'','DYANA DVM3','','','Aeron',636,'',85 from dual union all
select 6413,'1960-A161','1960-07-06 15:55:00','military missile','failure',928,'A1X-40','','','','Test',244,'',10 from dual union all
select 64130,'1990-Y60','1990-01-22 19:15:00','suborbital rocket','success',1502,'Sphere','DYANA DBS6','','','Aeron',307,'',90 from dual union all
select 64131,'1990-Y61','1990-01-22 20:00:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD16','','','Aeron',29,'',52 from dual union all
select 64132,'1990-Y62','1990-01-22 20:56:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD17','','','Aeron',29,'',81 from dual union all
select 64133,'1990-Y63','1990-01-23 03:45:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD3','','','Aeron',331,'',90 from dual union all
select 64134,'1990-Y64','1990-01-23 20:54:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',80 from dual union all
select 64135,'1990-Y65','1990-01-23 22:09:00','suborbital rocket','success',1284,'Datasonde','DYANA DED5','INTA DYSLD-9005','','Aeron',396,'',90 from dual union all
select 64136,'1990-Y66','1990-01-23 23:01:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',80 from dual union all
select 64137,'1990-Y67','1990-01-24 04:21:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',108 from dual union all
select 64138,'1990-Y68','1990-01-24 11:35:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',83 from dual union all
select 64139,'1990-Y69','1990-01-24 13:30:00','suborbital rocket','unknown',728,'','','','','Aeron',636,'',85 from dual union all
select 6414,'1960-A162','1960-07-07 00:00:00','military missile','failure',928,'A1X-39','','','','Test',421,'EAG-154',0 from dual union all
select 64140,'1990-Y70','1990-01-24 13:37:00','suborbital rocket','unknown',728,'','DYANA DVM4','','','Aeron',636,'',80 from dual union all
select 64141,'1990-Y71','1990-01-24 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',82 from dual union all
select 64142,'1990-Y72','1990-01-24 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',88 from dual union all
select 64143,'1990-Y73','1990-01-24 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',87 from dual union all
select 64144,'1990-Y74','1990-01-24 20:01:30','suborbital rocket','success',1502,'Sphere','DYANA DAS18','','','Aeron',29,'',117 from dual union all
select 64145,'1990-Y75','1990-01-24 23:02:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',91 from dual union all
select 64146,'1990-Y76','1990-01-25 20:00:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD19','','','Aeron',29,'',87 from dual union all
select 64147,'1990-Y77','1990-01-25 20:20:00','suborbital rocket','unknown',728,'','','','','Aeron',183,'',82 from dual union all
select 64148,'1990-Y78','1990-01-25 21:32:00','suborbital rocket','success',1284,'Datasonde','DYANA DED6','INTA DYSLD-9006','','Aeron',396,'',90 from dual union all
select 64149,'1990-Y79','1990-01-26 03:46:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD4','','','Aeron',331,'',90 from dual union all
select 6415,'1960-A163','1960-07-08 00:00:00','military missile','success',1214,'SM-62A N-3446','Cat IIE-6, Sta12','','','OT',224,'',10 from dual union all
select 64150,'1990-Y80','1990-01-28 20:00:00','suborbital rocket','success',1502,'Sphere','DYANA DAS20','','','Aeron',29,'',90 from dual union all
select 64151,'1990-Y81','1990-01-28 20:50:58','suborbital rocket','success',1502,'Chaff','DYANA DAC22','','','Aeron',29,'',90 from dual union all
select 64152,'1990-Y82','1990-01-28 21:11:00','suborbital rocket','success',1284,'Datasonde','DYANA DAD23','','','Aeron',29,'',80 from dual union all
select 64153,'1990-Y83','1990-01-29 12:25:00','suborbital rocket','success',1284,'Datasonde SL-4','DYANA DUD3','','','Aeron',839,'',80 from dual union all
select 64154,'1990-Y84','1990-01-29 13:42:00','suborbital rocket','unknown',728,'','DYANA DVM5','','','Aeron',636,'',85 from dual union all
select 64155,'1990-Y85','1990-01-29 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM5','','','Aeron',1010,'',82 from dual union all
select 64156,'1990-Y86','1990-01-29 20:07:11','suborbital rocket','success',1284,'Datasonde','DYANA DAD24','','','Aeron',29,'',78 from dual union all
select 64157,'1990-Y87','1990-01-30 03:46:00','suborbital rocket','success',1284,'Datasonde','DYANA DCD5','','','Aeron',331,'',90 from dual union all
select 64158,'1990-Y88','1990-01-30 21:15:00','suborbital rocket','success',1284,'Datasonde','DYANA DED7','INTA DYSLD-9007','','Aeron',396,'',90 from dual union all
select 64159,'1990-Y89','1990-01-30 23:00:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',86 from dual union all
select 6416,'1960-A164','1960-07-11 00:00:00','military missile','success',764,'58-1417  (20)','','','','Test',240,'',12 from dual union all
select 64160,'1990-Y90','1990-01-30 23:50:00','suborbital rocket','unknown',728,'','','','','Aeron',6,'',82 from dual union all
select 64161,'1990-Y91','1990-01-31 11:15:00','suborbital rocket','unknown',728,'','','','','Aeron',1334,'',84 from dual union all
select 64162,'1990-Y92','1990-01-31 12:25:00','suborbital rocket','success',1284,'Datasonde SL-5','DYANA DUD4','','','Aeron',839,'',80 from dual union all
select 64163,'1990-Y93','1990-01-31 13:47:00','suborbital rocket','unknown',728,'','DYANA DVM6','','','Aeron',636,'',83 from dual union all
select 64164,'1990-Y94','1990-01-31 14:07:00','suborbital rocket','unknown',728,'','','','','Aeron',961,'',84 from dual union all
select 64165,'1990-Y95','1990-01-31 15:05:00','suborbital rocket','unknown',728,'','DYANA DHM6','','','Aeron',1010,'',80 from dual union all
select 64166,'1990-Y96','1990-01-31 15:05:00','suborbital rocket','unknown',728,'','','','','Aeron',1010,'',84 from dual union all
select 64167,'1990-Y97','1990-01-31 20:00:00','suborbital rocket','success',1502,'Sphere','DYANA DAS25','','','Aeron',29,'',112 from dual union all
select 64168,'1990-Y98','1990-01-31 20:24:00','suborbital rocket','success',1502,'Chaff','DYANA DAC26','','','Aeron',29,'',99 from dual union all
select 64169,'1990-Y99','1990-02-01 12:05:00','suborbital rocket','unknown',728,'','DYANA DTM7','','','Aeron',1467,'',80 from dual union all
select 6417,'1960-A165','1960-07-12 00:00:00','atmospheric rocket','unknown',705,'L4','','','','Test',2,'',20 from dual union all
select 64170,'1991-001','1991-01-08 00:53:01','orbital','success',422,'D202','NATO 4A','NATO 4A','','Sat',234,'',null from dual union all
select 64171,'1991-002','1991-01-14 14:50:27','orbital','success',1242,'G 15000-045','Progress M-6','Progress 7K-TGM No. 205','','Sat',480,'',null from dual union all
select 64172,'1991-003','1991-01-15 23:10:49','orbital','success',103,'V41 L413a','Italsat 1/Eut. 2F2','Italsat 1','','Sat',345,'',null from dual union all
select 64173,'1991-004','1991-01-17 10:30:01','orbital','success',1241,'78023  -817','Kosmos-2121','Oblik','','Sat',462,'',null from dual union all
select 64174,'1991-005','1991-01-18 11:34:40','orbital','success',1446,'51061-502','Kosmos-2122','US-PU','','Sat',589,'',null from dual union all
select 64175,'1991-006','1991-01-29 11:59:58','orbital','success',680,'53744-158','Informator-1','Informator No. 1','','Sat',460,'',null from dual union all
select 64176,'1991-007','1991-02-05 02:36:45','orbital','success',680,'47148-407','Kosmos-2123','Tsikada/RS-12/RS-13','','Sat',460,'',null from dual union all
select 64177,'1991-008','1991-02-07 18:15:00','orbital','success',1238,'77017  -980','Kosmos-2124','Kobal''t','','Sat',462,'',null from dual union all
select 64178,'1991-009','1991-02-12 02:44:00','orbital','success',680,'47187-204','Kosmos-2125/2132','Strela-1M','','Sat',460,'',null from dual;
insert into LAUNCH
select 64179,'1991-010','1991-02-14 08:31:56','orbital','success',958,'344-02','Kosmos-2133','Oko-1 No. 7120','','Sat',553,'',null from dual union all
select 6418,'1960-A166','1960-07-13 00:00:00','military missile','success',771,'GM-54-88','','','','Test',222,'',12 from dual union all
select 64180,'1991-011','1991-02-15 09:30:00','orbital','success',1238,'77053  -483','Kosmos-2134','Kometa No. 13','','Sat',480,'',null from dual union all
select 64181,'1991-012','1991-02-15 15:19:00','orbital','success',805,'77017  -571','Molniya-1','Molniya-1T No. 73?','','Sat',474,'',null from dual union all
select 64182,'1991-013','1991-02-26 04:53:06','orbital','success',680,'47168-254','Kosmos-2135','Parus','','Sat',460,'',null from dual union all
select 64183,'1991-014','1991-02-28 05:30:00','orbital','success',958,'360-02','Raduga','Gran'' No. 38L','','Sat',587,'',null from dual union all
select 64184,'1991-015','1991-03-02 23:36:00','orbital','success',106,'V42 L414','Astra 1B/Meteosat 5','Astra 1B','','Sat',345,'',null from dual union all
select 64185,'1991-016','1991-03-06 15:30:00','orbital','success',1241,'78023  -819','Kosmos-2136','Oblik','','Sat',462,'',null from dual union all
select 64186,'1991-017','1991-03-08 12:03:00','orbital','success',1413,'K-5    (45F-1)','USA 69       [LAC2]','LACROSSE 2','','Sat',1777,'',null from dual union all
select 64187,'1991-018','1991-03-08 23:03:00','orbital','success',408,'D203','Inmarsat II F-2','INMARSAT II F-2','','Sat',234,'',null from dual union all
select 64188,'1991-019','1991-03-12 19:29:04','orbital','success',680,'65048-406','Nadezhda','Nadezhda 17F118 No. 409','','Sat',460,'',null from dual union all
select 64189,'1991-020','1991-03-19 13:05:15','orbital','success',1242,'R 15000-049','Progress M-7','Progress 7K-TGM No. 208','','Sat',480,'',null from dual union all
select 6419,'1960-A167','1960-07-14 00:00:00','military missile','success',771,'GM-54-65','','','','Test',222,'',12 from dual union all
select 64190,'1991-021','1991-03-19 14:30:00','orbital','success',680,'53448-417','Kosmos-2137','Taifun-1B','','Sat',456,'',null from dual union all
select 64191,'1991-022','1991-03-22 12:19:59','orbital','success',805,'77017  -574','Molniya-3','Molniya-3 No. 55','','Sat',474,'',null from dual union all
select 64192,'1991-023','1991-03-26 13:45:00','orbital','success',1238,'77053  -982','Kosmos-2138','Kobal''t','','Sat',462,'',null from dual union all
select 64193,'1991-024','1991-03-31 15:12:00','orbital','success',949,'365-01','Almaz-1','Mech-KU No. 305','','Sat',554,'',null from dual union all
select 64194,'1991-025','1991-04-04 10:47:12','orbital','success',958,'354-02','Kosmos-2139/2141','Uragan No. 50L','','Sat',553,'',null from dual union all
select 64195,'1991-026','1991-04-04 23:33:00','orbital','success',109,'V43 L415a','Anik E2','Anik E2','','Sat',345,'',null from dual union all
select 64196,'1991-027','1991-04-05 14:22:45','orbital','success',1243,'STS-37R','GRO','OV-104 LEO deploy','','Sat',922,'MLP1',null from dual union all
select 64197,'1991-028','1991-04-13 00:09:00','orbital','success',422,'D204','Spacenet 4','ASC 2','','Sat',234,'',null from dual union all
select 64198,'1991-029','1991-04-16 07:21:42','orbital','success',680,'47148-401','Kosmos-2142','Parus','','Sat',456,'',null from dual union all
select 64199,'1991-030','1991-04-24 01:37:00','orbital','success',1448,'67027-907','Meteor-3','Meteor-3 No. 6','','Sat',470,'',null from dual union all
select 642,'1946-A25','1946-12-12 00:00:00','suborbital rocket','success',1517,'25','','','','Test',1852,'',49 from dual union all
select 6420,'1960-A168','1960-07-14 00:00:00','atmospheric rocket','unknown',1065,'R1H','','','','Test',2,'',20 from dual union all
select 64200,'1991-031','1991-04-28 11:33:14','orbital','success',1243,'STS-39','SDIO/SPAS','OV-103 Research/FF','','Sat',921,'MLP2',null from dual union all
select 64201,'1991-032','1991-05-14 15:52:03','orbital','success',155,'50E','NOAA 12','NOAA D','','Sat',1776,'',null from dual union all
select 64202,'1991-033','1991-05-16 21:40:54','orbital','success',1448,'51019-205','Kosmos-2143/2148','Strela-3','','Sat',470,'',null from dual union all
select 64203,'1991-034','1991-05-18 12:50:28','orbital','success',1242,'R 15000-051','Soyuz TM-12','Soyuz 7K-STM No. 62','','Sat',480,'',null from dual union all
select 64204,'1991-035','1991-05-21 09:00:00','orbital','success',1241,'78023  -266','Resurs-F','Resurs-F2 17F42 No. 6','','Sat',475,'',null from dual union all
select 64205,'1991-036','1991-05-24 15:29:59','orbital','success',1238,'77017  -981','Kosmos-2149','Kobal''t','','Sat',474,'',null from dual union all
select 64206,'1991-037','1991-05-29 22:55:00','orbital','success',422,'D205','Aurora 2','Satcom C-5','','Sat',234,'',null from dual union all
select 64207,'1991-038','1991-05-30 08:04:03','orbital','success',1242,'R 15000-050','Progress M-8','Progress 7K-TGM No. 207','','Sat',480,'',null from dual union all
select 64208,'1991-039','1991-06-04 08:10:00','orbital','success',1448,'45033-305','Okean','Okean-O1 No. 6/NKhM No.','','Sat',470,'',null from dual union all
select 64209,'1991-040','1991-06-05 13:24:51','orbital','success',1243,'STS-40','Spacelab SLS-1','OV-102 Spacelab','','Sat',922,'MLP3',null from dual union all
select 6421,'1960-A169','1960-07-14 00:44:00','suborbital rocket','failure',839,'NASA 10.01GA','GSFC Grenade 3','','','Aeronomy',1821,'',28 from dual union all
select 64210,'1991-041','1991-06-11 05:41:59','orbital','success',680,'47159-819','Kosmos-2150','Strela-2M','','Sat',460,'',null from dual union all
select 64211,'1991-042','1991-06-13 15:41:00','orbital','success',1448,'45019-207','Kosmos-2151','Tselina-R','','Sat',470,'',null from dual union all
select 64212,'1991-043','1991-06-18 09:09:00','orbital','success',805,'77053  -606','Molniya-1','Molniya-1T No. 74','','Sat',475,'',null from dual union all
select 64213,'1991-044','1991-06-28 08:09:59','orbital','success',1241,'78023  -153','Resurs-F','Resurs-F1 14F43 No. 52','','Sat',474,'',null from dual union all
select 64214,'1991-045','1991-06-29 14:00:00','orbital','success',1135,'S216C','REX','REX','','Sat',1779,'',null from dual union all
select 64215,'1991-046','1991-07-01 21:53:00','orbital','success',958,'373-01','Gorizont','Gorizont No. 34L','','Sat',553,'',null from dual union all
select 64216,'1991-047','1991-07-04 02:32:00','orbital','success',422,'D206','USA 71 (GPS IIA-11)','Navstar GPS SVN 24','','Sat',233,'',null from dual union all
select 64217,'1991-048','1991-07-09 09:40:00','orbital','success',1241,'78023  -818','Kosmos-2152','Oblik','','Sat',475,'',null from dual union all
select 64218,'1991-049','1991-07-10 14:00:00','orbital','success',1238,'77059  -726','Kosmos-2153','Yantar''-4KS1M','','Sat',562,'',null from dual union all
select 64219,'1991-050','1991-07-17 01:46:31','orbital','success',94,'V44 L416a','ERS 1/ASAP 2','ERS-1','','Sat',345,'',null from dual union all
select 6422,'1960-A17','1960-01-21 14:23:00','test rocket','success',707,'LJ-1B','','','','SC Test',1797,'',14 from dual union all
select 64220,'1991-051','1991-07-17 17:33:53','orbital','failure',913,'002/F2','Microsat','Microsat 1','','Sat',379,'NB-52 008',null from dual union all
select 64221,'1991-052','1991-07-23 09:05:00','orbital','success',1238,'78012  -152','Resurs-F','Resurs-F1 14F43 No. 53','','Sat',474,'',null from dual union all
select 64222,'1991-053','1991-08-01 11:53:00','orbital','success',805,'76035  -533','Molniya-1','Molniya-1T No. 76','','Sat',475,'',null from dual union all
select 64223,'1991-054','1991-08-02 15:02:00','orbital','success',1243,'STS-43','TDRS-E','OV-104 IUS deploy','','Sat',921,'MLP1',null from dual union all
select 64224,'1991-055','1991-08-14 23:15:13','orbital','success',103,'V45 L417','Intelsat 605','INTELSAT VI F-5','','Sat',345,'',null from dual union all
select 64225,'1991-056','1991-08-15 09:15:00','orbital','success',1448,'67082-803','Meteor-3','Meteor-3 No. 5/TOMS','','Sat',470,'',null from dual union all
select 64226,'1991-057','1991-08-20 22:54:10','orbital','success',1242,'R 15000-052','Progress M-9','Progress 7K-TGM No. 210','','Sat',480,'',null from dual union all
select 64227,'1991-058','1991-08-21 10:50:00','orbital','success',1241,'78012  -269','Resurs-F','Resurs-F2 17F42 No. 7','','Sat',474,'',null from dual union all
select 64228,'1991-059','1991-08-22 12:35:46','orbital','success',680,'53778-429','Kosmos-2154','Parus','','Sat',456,'',null from dual union all
select 64229,'1991-060','1991-08-25 08:40:00','orbital','success',524,'H-23(F)','BS 3B','BS 3B','','Sat',1474,'',null from dual union all
select 6423,'1960-A170','1960-07-22 23:46:16','military missile','failure',154,'74D','Mk 3-3B','IOC ORT','TIGER SKIN','Test',1665,'',20 from dual union all
select 64230,'1991-061','1991-08-29 06:48:43','orbital','success',1510,'I 15000-079','IRS-1B','IRS-1B','','Sat',562,'',null from dual union all
select 64231,'1991-062','1991-08-30 02:30:00','orbital','success',820,'M-3S2-6','Yohkoh','SOLAR-A','','Sat',837,'',null from dual union all
select 64232,'1991-063','1991-09-12 23:11:04','orbital','success',1243,'STS-48','UARS','OV-103 LEO deploy','','Sat',921,'MLP3',null from dual union all
select 64233,'1991-064','1991-09-13 17:51:02','orbital','success',958,'353-01','Kosmos-2155','Oko-S F4','','Sat',587,'',null from dual union all
select 64234,'1991-065','1991-09-17 20:01:59','orbital','success',805,'76047  -569','Molniya-3','Molniya-3 No. 48','','Sat',475,'',null from dual union all
select 64235,'1991-066','1991-09-19 16:20:00','orbital','success',1238,'77053  -983','Kosmos-2156','Kobal''t','','Sat',474,'',null from dual union all
select 64236,'1991-067','1991-09-26 23:43:00','orbital','success',109,'V46 L418a','Anik E1','Anik E1','','Sat',345,'',null from dual union all
select 64237,'1991-068','1991-09-28 07:05:55','orbital','success',1448,'51057-902','Kosmos-2157/2162','Strela-3','','Sat',470,'',null from dual union all
select 64238,'1991-069','1991-10-02 05:59:38','orbital','success',1242,'R 15000-054','Soyuz TM-13','Soyuz 7K-STM No. 63','','Sat',480,'',null from dual union all
select 64239,'1991-070','1991-10-04 18:10:00','orbital','success',1241,'R 15000-026','Foton','Foton No. 7L','','Sat',475,'',null from dual union all
select 6424,'1960-A171','1960-07-27 20:03:00','suborbital rocket','failure',42,'TATTLETALE PCC 4','AF-49','','','Sigint Test?',385,'',5 from dual union all
select 64240,'1991-071','1991-10-09 13:15:00','orbital','success',1242,'G 15000-048','Kosmos-2163','Orlets','','Sat',480,'',null from dual union all
select 64241,'1991-072','1991-10-10 14:00:00','orbital','success',680,'53778-420','Kosmos-2164','Taifun-1B','','Sat',456,'',null from dual union all
select 64242,'1991-073','1991-10-17 00:05:25','orbital','success',1242,'R 15000-055','Progress M-10','Progress 7K-TGM No. 211','','Sat',480,'',null from dual union all
select 64243,'1991-074','1991-10-23 15:25:00','orbital','success',958,'362-02','Gorizont','Gorizont No. 35L','','Sat',553,'',null from dual union all
select 64244,'1991-075','1991-10-29 23:08:08','orbital','success',103,'V47 L419','Intelsat 601','INTELSAT VI F-1','','Sat',345,'',null from dual union all
select 64245,'1991-076','1991-11-08 07:07:00','orbital','success',1413,'K-8    (45F-2)','USA 72-77   [NOSS11]','SLDCOM 2/INTRUDER 2','','Sat',1777,'',null from dual union all
select 64246,'1991-077','1991-11-12 20:09:33','orbital','success',1448,'45057-903','Kosmos-2165/2170','Strela-3','','Sat',469,'',null from dual union all
select 64247,'1991-078','1991-11-20 19:15:00','orbital','success',1238,'77053  -984','Kosmos-2171','Kobal''t','','Sat',474,'',null from dual union all
select 64248,'1991-079','1991-11-22 13:27:00','orbital','success',958,'353-02','Kosmos-2172','Geizer No. 18L','','Sat',587,'',null from dual union all
select 64249,'1991-080','1991-11-24 23:44:00','orbital','success',1243,'STS-44','USA 75 (DSP F-16)','OV-104 IUS deploy','','Sat',921,'MLP1',null from dual union all
select 6425,'1960-A172','1960-07-28 00:00:00','atmospheric rocket','success',274,'NOTS EV-II','TV-1','','','Test',1023,'F4D-1 747',20 from dual union all
select 64250,'1991-081','1991-11-27 03:30:26','orbital','success',680,'65048-409','Kosmos-2173','Parus','','Sat',460,'',null from dual union all
select 64251,'1991-082','1991-11-28 13:23:00','orbital','success',155,'53E','USA 73 (DMSP F11)','DMSP Block 5D-2 S-12','','Sat',1776,'',null from dual union all
select 64252,'1991-083','1991-12-07 22:47:00','orbital','success',172,'AC-102','Eutelsat IIF3','Eutelsat II F-3','','Sat',269,'',null from dual union all
select 64253,'1991-084','1991-12-16 23:19:48','orbital','success',103,'V48 L420a','Inmarsat 2F3/Tel. 2A','Telecom 2A','','Sat',345,'',null from dual union all
select 64254,'1991-085','1991-12-17 11:00:00','orbital','success',1238,'76032  -494','Kosmos-2174','Kometa No. 14','','Sat',562,'',null from dual union all
select 64255,'1991-086','1991-12-18 03:54:00','orbital','success',1448,'51033-304','Interkosmos-25','AUOS-Z No. 301','','Sat',470,'',null from dual union all
select 64256,'1991-087','1991-12-19 11:41:00','orbital','success',958,'355-01','Raduga','Gran'' No. 39L','','Sat',587,'',null from dual union all
select 64257,'1991-088','1991-12-28 12:00:00','orbital','success',308,'Y9       (26)','Zhongxing 4','DFH 2A-4','','Sat',1904,'',null from dual union all
select 64258,'1991-A01','1991-03-05 19:19:00','suborbital rocket','success',1325,'NASA 12.44WT','','','','Test',1821,'',20 from dual union all
select 64259,'1991-A02','1991-03-25 00:00:00','military missile','failure',118,'','','','','Test',1024,'SHIP-ISR',20 from dual union all
select 6426,'1960-A173','1960-07-29 13:13:03','test rocket','failure',154,'50D','MA-1','','','SC Test',229,'',13 from dual union all
select 64260,'1991-A03','1991-06-18 11:34:00','suborbital rocket','failure',948,'JOUST 1','','','','Micrograv',239,'',2 from dual union all
select 64261,'1991-A04','1991-08-20 09:45:48','suborbital rocket','failure',117,'Red Tigress 1A','','','','Target',239,'',2 from dual union all
select 64262,'1991-A05','1991-09-23 15:40:00','military missile','failure',539,'','KITE-2','','','Test',1883,'',0 from dual union all
select 64263,'1991-A06','1991-10-30 00:00:00','military missile','failure',118,'','','','','Test',1024,'SHIP-ISR',20 from dual union all
select 64264,'1991-A07','1991-09-30 21:19:46','suborbital rocket','failure',1502,'Sphere','METAL/L-S15','','','Aeron',29,'',8 from dual union all
select 64265,'1991-A08','1991-05-30 00:00:00','military missile','success',684,'4      -','15Zh59 B-4','Mobile launch test','','Test',477,'',1 from dual union all
select 64266,'1991-F01','1991-04-18 23:30:00','orbital','failure',171,'AC-70','BS-3H','BS-3H','','Sat',269,'',175 from dual union all
select 64267,'1991-F02','1991-06-25 13:20:00','orbital','failure',680,'65061-262','','Romb','','Sat',456,'',100 from dual union all
select 64268,'1991-F03','1991-08-30 08:58:01','orbital','failure',1537,'51019-217/16L','','Tselina-2','','Sat',567,'',150 from dual;
insert into LAUNCH
select 64269,'1991-M01','1991-07-04 08:22:00','military missile','success',941,'4','','','','Test',1361,'',50 from dual union all
select 6427,'1960-A174','1960-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64270,'1991-M02','1991-08-07 05:28:00','military missile','success',941,'5','','','','Test',1361,'',50 from dual union all
select 64271,'1991-M03','1991-01-01 00:00:00','military missile','success',608,'1','','','','Test',636,'',50 from dual union all
select 64272,'1991-M04','1991-01-01 00:00:00','military missile','success',608,'2','','','','Test',636,'',50 from dual union all
select 64273,'1991-M05','1991-01-01 00:00:00','military missile','success',608,'3','','','','Test',636,'',50 from dual union all
select 64274,'1991-M06','1991-01-01 00:00:00','military missile','success',608,'4','','','','Test',636,'',50 from dual union all
select 64275,'1991-M07','1991-01-01 00:00:00','military missile','success',608,'5','','','','Test',636,'',50 from dual union all
select 64276,'1991-S01','1991-01-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',636,'',250 from dual union all
select 64277,'1991-S02','1991-01-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',636,'',250 from dual union all
select 64278,'1991-S03','1991-01-01 00:00:00','military missile','unknown',646,'','Layma','','','Reentry Test',621,'',675 from dual union all
select 64279,'1991-S04','1991-01-01 00:00:00','military missile','unknown',646,'','Igla-3 penetrator','','','Reentry Test',621,'',675 from dual union all
select 6428,'1960-A175','1960-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64280,'1991-S05','1991-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64281,'1991-S06','1991-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64282,'1991-S07','1991-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64283,'1991-S08','1991-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64284,'1991-S09','1991-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64285,'1991-S10','1991-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64286,'1991-S100','1991-04-29 00:00:00','suborbital rocket','success',1219,'XV-23','','','','Ionos?',330,'',441 from dual union all
select 64287,'1991-S101','1991-05-01 00:00:00','military missile','success',1167,'','','','','Test',1311,'',125 from dual union all
select 64288,'1991-S102','1991-04-30 19:19:00','suborbital rocket','failure',860,'NASA 31.80UU','CWAS 20','','','Aeronomy',1047,'',10 from dual union all
select 64289,'1991-S103','1991-05-04 04:10:00','suborbital rocket','success',224,'NASA 36.062UL','UM JASPR I','','','UV Astron',1873,'',300 from dual union all
select 6429,'1960-A176','1960-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64290,'1991-S104','1991-05-07 18:05:00','suborbital rocket','success',224,'NASA 36.086GS','SERTS-91 (04)','','','Solar',1873,'',300 from dual union all
select 64291,'1991-S105','1991-05-08 03:54:00','suborbital rocket','failure',772,'MAXUS 1','SSC MAXUS 1','SSC MAXUS 1','','Micrograv',407,'',157 from dual union all
select 64292,'1991-S106','1991-05-10 00:00:00','military missile','unknown',1442,'FCET-39','FCET','','','OT',420,'SSBN 658',1000 from dual union all
select 64293,'1991-S107','1991-05-10 00:00:00','military missile','unknown',1442,'FCET-39','FCET','','','OT',420,'SSBN 658',1000 from dual union all
select 64294,'1991-S108','1991-05-11 20:40:00','military missile','success',795,'','ERIS Target','GBI-2 ERIS Target','','Target',1698,'',1300 from dual union all
select 64295,'1991-S109','1991-05-13 19:03:00','suborbital rocket','success',224,'NASA 36.049US','MSSTA I','','','Solar X',1873,'',230 from dual union all
select 64296,'1991-S11','1991-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64297,'1991-S110','1991-05-20 07:30:00','suborbital rocket','success',117,'NASA 24.11CH','SXT/XOGS','','','XR Astron',1873,'',300 from dual union all
select 64298,'1991-S111','1991-05-28 12:05:00','military missile','success',1005,'67027-319-14(36L)','15A18M LKI-25','GCh 15F177(174x5,178x3)','','OT',491,'',1000 from dual union all
select 64299,'1991-S112','1991-06-11 17:47:00','military missile','success',907,'06PA (84-546)','','GT 06PA','','OT',1703,'',1000 from dual union all
select 643,'1946-A26','1946-11-07 20:31:00','suborbital rocket','failure',1470,'14','','','','CR',1863,'',0 from dual union all
select 6430,'1960-A177','1960-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64300,'1991-S113','1991-06-21 03:01:00','military missile','success',795,'','AST','RSLP AST','','Reentry Test',1698,'',1300 from dual union all
select 64301,'1991-S114','1991-06-24 00:00:00','military missile','success',1106,'26010','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 64302,'1991-S115','1991-06-26 02:39:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT144GB','GLORY TRIP 144GB','OT',1712,'',1300 from dual union all
select 64303,'1991-S116','1991-06-25 17:00:00','military missile','success',1434,'','15Zh58 K-27','','','Test',477,'',1000 from dual union all
select 64304,'1991-S117','1991-07-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64305,'1991-S118','1991-07-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64306,'1991-S119','1991-07-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64307,'1991-S12','1991-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64308,'1991-S120','1991-07-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64309,'1991-S121','1991-07-01 00:00:00','military missile','success',580,'','','','','Test',324,'',125 from dual union all
select 6431,'1960-A178','1960-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64310,'1991-S122','1991-07-02 10:17:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT143GM','GLORY TRIP 143GM','OT',1706,'',1300 from dual union all
select 64311,'1991-S123','1991-07-11 17:25:00','suborbital rocket','success',224,'NASA 36.087US','Corona Tel 3','','','Solar EUV/Eclipse',1873,'',300 from dual union all
select 64312,'1991-S124','1991-07-29 00:00:00','military missile','success',1443,'','CET','','','OT',420,'SSBN 735',1000 from dual union all
select 64313,'1991-S125','1991-07-29 00:00:00','military missile','success',1443,'','CET','','','OT',420,'SSBN 735',1000 from dual union all
select 64314,'1991-S126','1991-07-29 00:00:00','military missile','success',1443,'','CET','','','OT',420,'SSBN 735',1000 from dual union all
select 64315,'1991-S127','1991-07-29 00:00:00','military missile','success',1443,'','CET','','','OT',420,'SSBN 735',1000 from dual union all
select 64316,'1991-S128','1991-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64317,'1991-S129','1991-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64318,'1991-S13','1991-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64319,'1991-S130','1991-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 6432,'1960-A179','1960-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64320,'1991-S131','1991-08-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64321,'1991-S132','1991-08-01 01:28:20','suborbital rocket','success',227,'NASA 21.103GE NLC-91','N-B-E03 EFIELD-B','','','Ionos',417,'',99 from dual union all
select 64322,'1991-S133','1991-08-01 01:39:00','suborbital rocket','success',860,'NASA 31.77UE  NLC-91','N-B-P04 PEP B','','','Aeron',417,'',93 from dual union all
select 64323,'1991-S134','1991-08-09 23:15:00','suborbital rocket','success',860,'DLR K-NO-189  NLC-91','N-A-T13 TURBO A','','','Aeron',417,'',117 from dual union all
select 64324,'1991-S135','1991-08-09 23:30:20','suborbital rocket','success',227,'NASA 21.104GE NLC-91','N-B-E16 EFIELD-A','','','Ionos',417,'',101 from dual union all
select 64325,'1991-S136','1991-08-15 12:05:00','military missile','success',1453,'63022-82-421','15A35 UBP-40','','','OT',505,'',1000 from dual union all
select 64326,'1991-S137','1991-08-19 22:55:00','military missile','success',1434,'','15Zh58 K-28','','','Test',477,'',1000 from dual union all
select 64327,'1991-S138','1991-08-30 23:32:03','military missile','success',1443,'','DASO-7','','','OT',420,'SSBN 737',1000 from dual union all
select 64328,'1991-S139','1991-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64329,'1991-S14','1991-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 6433,'1960-A18','1960-01-24 00:17:05','suborbital spaceplane','success',1520,'1-2-7','SAS Eval','','','Test',379,'NB-52 008',20 from dual union all
select 64330,'1991-S140','1991-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64331,'1991-S141','1991-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64332,'1991-S142','1991-09-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64333,'1991-S143','1991-09-03 00:00:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT145GM','GLORY TRIP 145GM','OT',1699,'',1300 from dual union all
select 64334,'1991-S144','1991-09-04 05:50:00','suborbital rocket','success',1303,'ZEST-1','Tungsten dust cloud','','','Target/Release',843,'',330 from dual union all
select 64335,'1991-S145','1991-09-06 21:50:00','suborbital rocket','success',211,'NASA 27.129UE','','','','Plasma',1895,'',148 from dual union all
select 64336,'1991-S146','1991-09-12 05:41:00','suborbital rocket','success',1303,'ZEST-2','','','','Target/Release',843,'',330 from dual union all
select 64337,'1991-S147','1991-09-12 12:45:00','military missile','success',1005,'67027-903(35L)','15A18M LKI-26','GCh 15F177(174x?,178x3)','','OT',487,'',1000 from dual union all
select 64338,'1991-S148','1991-09-15 23:00:00','suborbital rocket','success',1296,'TR-1A-1','','','','Micrograv',1476,'',270 from dual union all
select 64339,'1991-S149','1991-09-17 00:00:00','military missile','success',907,'07PA (86-442)','','GT 07PA','','OT',1697,'',1000 from dual union all
select 6434,'1960-A180','1960-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64340,'1991-S15','1991-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64341,'1991-S150','1991-09-17 10:20:00','military missile','success',1006,'51074-317-08','15A18 UBP-30','GCh 15F183','','Test',489,'',1000 from dual union all
select 64342,'1991-S151','1991-09-17 23:43:00','suborbital rocket','success',860,'DLR A-NO-190','Turbo-Metal L-T1 F95','','','Aeron',29,'',140 from dual union all
select 64343,'1991-S152','1991-09-20 20:48:00','suborbital rocket','success',860,'DLR A-NO-191','Turbo-Metal L-T6 F96','','','Aeron',29,'',140 from dual union all
select 64344,'1991-S153','1991-09-20 22:40:00','suborbital rocket','success',860,'DLR A-NO-192','Turbo-Metal L-I9 F93','','','Aeron',29,'',140 from dual union all
select 64345,'1991-S154','1991-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64346,'1991-S155','1991-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64347,'1991-S156','1991-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64348,'1991-S157','1991-10-01 00:00:00','suborbital rocket','success',741,'','','','','Aeron',147,'ZUB',250 from dual union all
select 64349,'1991-S158','1991-09-30 20:55:15','suborbital rocket','success',860,'DLR A-NO-193','Turbo-MetalL-T13 F97','','','Aeron',29,'',140 from dual union all
select 6435,'1960-A181','1960-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64350,'1991-S159','1991-10-02 15:50:00','military missile','success',1434,'','15Zh58 K-29','','','Test',477,'',1000 from dual union all
select 64351,'1991-S16','1991-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64352,'1991-S160','1991-10-03 22:27:30','suborbital rocket','success',860,'DLR A-NO-194','Turbo-MetalL-T17 F98','','','Aeron',29,'',140 from dual union all
select 64353,'1991-S161','1991-10-04 00:08:00','suborbital rocket','success',860,'DLR A-NO-195','Turbo-MetalL-T21 F99','','','Aeron',29,'',140 from dual union all
select 64354,'1991-S162','1991-10-10 11:05:00','military missile','success',1005,'45082-418-05T','15A18M K-6','GCh 15F173','','Test',512,'',1000 from dual union all
select 64355,'1991-S163','1991-10-14 10:17:20','suborbital rocket','success',117,'Red Tigress 1B','','','','Target',239,'',320 from dual union all
select 64356,'1991-S164','1991-10-30 00:00:00','military missile','success',629,'','','','','OT',988,'',100 from dual union all
select 64357,'1991-S165','1991-11-04 20:42:00','military missile','success',1443,'','DASO-8','','','OT',419,'SSBN 737',1000 from dual union all
select 64358,'1991-S166','1991-11-11 10:19:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT146GM','GLORY TRIP 146GM','OT',1705,'',1300 from dual;
insert into LAUNCH
select 64359,'1991-S167','1991-11-11 00:00:00','military missile','success',1443,'','CET','','','OT',420,'SSBN 735',1000 from dual union all
select 6436,'1960-A182','1960-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64360,'1991-S168','1991-11-11 00:00:00','military missile','success',1443,'','CET','','','OT',420,'SSBN 735',1000 from dual union all
select 64361,'1991-S169','1991-11-11 00:00:00','military missile','success',1443,'','CET','','','OT',420,'SSBN 735',1000 from dual union all
select 64362,'1991-S17','1991-01-17 00:00:00','military missile','success',73,'','Israel','','','Weapon',762,'',100 from dual union all
select 64363,'1991-S170','1991-11-11 00:00:00','military missile','success',1443,'','CET','','','OT',420,'SSBN 735',1000 from dual union all
select 64364,'1991-S171','1991-11-16 15:30:00','suborbital rocket','success',213,'Starfire 1/F4','CONSORT 4','','','Micrograv',1879,'',298 from dual union all
select 64365,'1991-S172','1991-11-20 08:05:00','military missile','success',1006,'67048-332-15','15A18 UBP-31','GCh 15F183','','OT',489,'',1000 from dual union all
select 64366,'1991-S173','1991-11-23 09:40:00','suborbital rocket','success',1212,'DLR K-GR-196','DLR TEXUS 28','','','Micrograv',416,'',239 from dual union all
select 64367,'1991-S174','1991-11-26 02:30:00','military missile','success',1454,'51145-93-003P','15A35P LKI-3','15Yu70','','Test',535,'',1000 from dual union all
select 64368,'1991-S175','1991-10-28 07:10:00','military missile','success',1005,'45092-418-10T','15A18M K-7','GCh 15F173','','OT',512,'',1000 from dual union all
select 64369,'1991-S176','1991-12-01 00:00:00','military missile','success',1548,'','Sprint','','','Tech',1626,'PLBR',620 from dual union all
select 6437,'1960-A183','1960-08-01 17:10:00','military missile','failure',928,'A1E-3 (A1EP-2)','','','','Test',421,'SSBN 598',1 from dual union all
select 64370,'1991-S177','1991-12-01 00:00:00','military missile','success',1507,'VMF','','','','OT',1626,'KALMAR',1000 from dual union all
select 64371,'1991-S178','1991-12-06 00:54:00','suborbital rocket','success',1318,'NASA 38.19UE','RED AIR 2','','','Plasma',1821,'',359 from dual union all
select 64372,'1991-S179','1991-12-06 01:19:00','suborbital rocket','success',1318,'NASA 38.20UE','RED AIR 2','','','Plasma',1821,'',361 from dual union all
select 64373,'1991-S18','1991-01-19 00:00:00','military missile','success',73,'','Israel','','','Weapon',762,'',100 from dual union all
select 64374,'1991-S180','1991-12-09 21:05:00','suborbital rocket','success',1216,'XV-55','Aguas Belas','','','Aeron',13,'',88 from dual union all
select 64375,'1991-S181','1991-12-20 20:30:00','test rocket','success',1063,'No. 3L/511-1992-003','Briz test ''Grand Prix''','GVM','','Test',537,'',1000 from dual union all
select 64376,'1991-S182','1991-08-09 23:15:15','suborbital rocket','success',860,'NLC-91 N-A-D14','SSC DECIMALS A','','','Aeronomy',417,'',117 from dual union all
select 64377,'1991-S183','1991-08-10 01:37:00','suborbital rocket','success',860,'NLC-91 N-A-D20','SSC DECIMALS B','','','Aeronomy',417,'',118 from dual union all
select 64378,'1991-S184','1991-08-01 01:40:00','suborbital rocket','success',860,'DLR K-NO-188  NLC-91','N-B-T05 TURBO B','','','Aeron',417,'',130 from dual union all
select 64379,'1991-S185','1991-08-09 23:40:00','suborbital rocket','success',860,'NASA 31.78UE  NLC-91','N-A-P17 PEP A','','','Aeronomy',417,'',117 from dual union all
select 6438,'1960-A184','1960-08-01 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1234,'SHIP-SOV',42 from dual union all
select 64380,'1991-S186','1991-01-18 12:13:00','suborbital rocket','success',597,'INTA RP-9101','Test','Rocio-7','','Test',396,'',118 from dual union all
select 64381,'1991-S187','1991-01-18 14:00:00','suborbital rocket','failure',597,'INTA RP-9102','Meteo','Rocio-8','','Aeron',396,'',22 from dual union all
select 64382,'1991-S188','1991-10-14 12:15:00','suborbital rocket','success',597,'INTA RP-9103','Test','Rocio-9','','Test',396,'',100 from dual union all
select 64383,'1991-S189','1991-05-10 00:00:00','military missile','unknown',1442,'','FCET','','','OT',420,'SSBN 658',1000 from dual union all
select 64384,'1991-S19','1991-01-19 00:00:00','military missile','success',73,'','Israel','','','Weapon',762,'',100 from dual union all
select 64385,'1991-S190','1991-05-10 00:00:00','military missile','unknown',1442,'','FCET','','','OT',420,'SSBN 658',1000 from dual union all
select 64386,'1991-S191','1991-06-19 18:00:00','military missile','unknown',1442,'','-                 .','','','OT',421,'SSBN 657',1000 from dual union all
select 64387,'1991-S192','1991-02-25 00:00:00','military missile','success',72,'','Israel-Negev','','','Weapon',762,'',100 from dual union all
select 64388,'1991-S193','1991-02-25 00:00:00','military missile','success',72,'','Israel-Negev','','','Weapon',762,'',100 from dual union all
select 64389,'1991-S194','1991-02-25 00:00:00','military missile','success',72,'','Israel-Negev','','','Weapon',762,'',100 from dual union all
select 6439,'1960-A185','1960-08-03 00:58:00','military missile','success',928,'A1X-37','','','','Test',245,'',500 from dual union all
select 64390,'1991-S195','1991-02-25 00:00:00','military missile','success',72,'','Israel-Negev','','','Weapon',762,'',100 from dual union all
select 64391,'1991-S196','1991-02-25 00:00:00','military missile','success',72,'','Israel-Negev','','','Weapon',762,'',100 from dual union all
select 64392,'1991-S197','1991-09-18 00:07:00','suborbital rocket','unknown',1502,'Chaff','METAL/L-C2','','','Aeron',29,'',90 from dual union all
select 64393,'1991-S198','1991-09-18 00:48:13','suborbital rocket','unknown',1502,'Chaff','METAL/L-C3','','','Aeron',29,'',90 from dual union all
select 64394,'1991-S199','1991-09-18 01:06:00','suborbital rocket','success',1502,'Sphere','METAL/L-S4','','','Aeron',29,'',108 from dual union all
select 64395,'1991-S20','1991-01-19 00:00:00','military missile','success',73,'','Israel','','','Weapon',762,'',100 from dual union all
select 64396,'1991-S200','1991-09-18 01:28:30','suborbital rocket','success',1502,'Chaff','METAL/L-C5','','','Aeron',29,'',100 from dual union all
select 64397,'1991-S201','1991-09-20 21:08:00','suborbital rocket','success',1502,'Chaff','METAL/L-C7','','','Aeron',29,'',100 from dual union all
select 64398,'1991-S202','1991-09-20 21:30:40','suborbital rocket','success',1502,'Sphere','METAL/L-S8','','','Aeron',29,'',109 from dual union all
select 64399,'1991-S203','1991-09-20 23:00:00','suborbital rocket','unknown',1502,'Chaff','METAL/L-C10','','','Aeron',29,'',100 from dual union all
select 644,'1946-A27','1946-05-07 00:00:00','atmospheric rocket','failure',1401,'11','','','','Test',1852,'',4 from dual union all
select 6440,'1960-A186','1960-08-02 00:00:00','military missile','failure',976,'','8K63 UBP-63','','','Test',636,'',0 from dual union all
select 64400,'1991-S204','1991-09-20 23:23:30','suborbital rocket','success',1502,'Sphere','METAL/L-S11','','','Aeron',29,'',109 from dual union all
select 64401,'1991-S205','1991-09-27 23:18:00','suborbital rocket','success',1284,'Sphere','METAL/L-L12','','','Aeron',29,'',83 from dual union all
select 64402,'1991-S206','1991-09-30 21:08:11','suborbital rocket','unknown',1502,'Chaff','METAL/L-C14','','','Aeron',29,'',100 from dual union all
select 64403,'1991-S207','1991-09-30 21:30:09','suborbital rocket','success',1502,'Sphere','METAL/L-S16','','','Aeron',29,'',114 from dual union all
select 64404,'1991-S208','1991-10-03 22:42:00','suborbital rocket','unknown',1502,'Chaff','METAL/L-C18','','','Aeron',29,'',100 from dual union all
select 64405,'1991-S209','1991-10-03 23:02:30','suborbital rocket','unknown',1502,'Sphere','METAL/L-S19','','','Aeron',29,'',100 from dual union all
select 64406,'1991-S21','1991-01-20 18:43:00','military missile','success',73,'','Dhahran','','','Weapon',758,'',100 from dual union all
select 64407,'1991-S210','1991-10-03 23:16:30','suborbital rocket','success',1502,'Sphere','METAL/L-S20','','','Aeron',29,'',112 from dual union all
select 64408,'1991-S211','1991-10-04 00:23:30','suborbital rocket','success',1502,'Chaff','METAL/L-C22','','','Aeron',29,'',102 from dual union all
select 64409,'1991-S212','1991-10-04 00:44:00','suborbital rocket','unknown',1502,'Sphere','METAL/L-S23','','','Aeron',29,'',100 from dual union all
select 6441,'1960-A187','1960-08-02 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1235,'SHIP-SOV',44 from dual union all
select 64410,'1991-S213','1991-10-04 01:02:30','suborbital rocket','unknown',1502,'Sphere','METAL/L-S24','','','Aeron',29,'',100 from dual union all
select 64411,'1991-S214','1991-10-04 01:16:30','suborbital rocket','success',1284,'Sphere','METAL/L-L25','','','Aeron',29,'',83 from dual union all
select 64412,'1991-S215','1991-10-15 12:30:00','suborbital rocket','success',597,'INTA RP-9104','Test','Rocio-10','','Test',396,'',73 from dual union all
select 64413,'1991-S216','1991-07-26 00:00:00','military missile','success',674,'1      -','79M6 test flight','','','ASAT Test',183,'MiG31D-72',200 from dual union all
select 64414,'1991-S22','1991-01-20 18:43:00','military missile','success',73,'','Dhahran','','','Weapon',758,'',100 from dual union all
select 64415,'1991-S23','1991-01-21 08:29:00','military missile','success',73,'','Dhahran','','','Weapon',758,'',100 from dual union all
select 64416,'1991-S24','1991-01-21 08:29:00','military missile','success',73,'','Dhahran','','','Weapon',758,'',100 from dual union all
select 64417,'1991-S25','1991-01-21 09:42:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64418,'1991-S26','1991-01-21 09:42:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64419,'1991-S27','1991-01-21 09:42:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 6442,'1960-A188','1960-08-03 00:00:00','atmospheric rocket','success',551,'','Glide Rocket','','','Reentry Test',1821,'',27 from dual union all
select 64420,'1991-S28','1991-01-21 09:42:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64421,'1991-S29','1991-01-21 19:18:00','military missile','success',73,'','Al Jubayl','','','Weapon',758,'',100 from dual union all
select 64422,'1991-S30','1991-01-22 00:00:00','military missile','success',73,'','Israel','','','Weapon',762,'',100 from dual union all
select 64423,'1991-S31','1991-01-22 00:00:00','suborbital rocket','success',1545,'ZN-3','','','','Test',647,'',120 from dual union all
select 64424,'1991-S32','1991-01-22 00:41:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64425,'1991-S33','1991-01-22 00:41:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64426,'1991-S34','1991-01-22 00:41:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64427,'1991-S35','1991-01-22 04:10:00','military missile','success',73,'','Dhahran','','','Weapon',758,'',100 from dual union all
select 64428,'1991-S36','1991-01-22 04:10:00','military missile','success',73,'','Dhahran','','','Weapon',758,'',100 from dual union all
select 64429,'1991-S37','1991-01-22 04:10:00','military missile','success',73,'','Dhahran','','','Weapon',758,'',100 from dual union all
select 6443,'1960-A189','1960-08-03 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1236,'SHIP-SOV',40 from dual union all
select 64430,'1991-S38','1991-01-23 19:54:00','military missile','success',73,'','Dhahran','','','Weapon',758,'',100 from dual union all
select 64431,'1991-S39','1991-01-23 19:54:00','military missile','success',73,'','Dhahran','','','Weapon',758,'',100 from dual union all
select 64432,'1991-S40','1991-01-23 19:54:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64433,'1991-S41','1991-01-23 19:54:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64434,'1991-S42','1991-01-23 19:54:00','military missile','success',73,'','Israel','','','Weapon',762,'',100 from dual union all
select 64435,'1991-S43','1991-01-25 00:00:00','military missile','success',73,'','Israel','','','Weapon',762,'',100 from dual union all
select 64436,'1991-S44','1991-01-25 19:23:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64437,'1991-S45','1991-01-25 19:23:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64438,'1991-S46','1991-01-26 00:00:00','military missile','success',73,'','Israel','','','Weapon',762,'',100 from dual union all
select 64439,'1991-S47','1991-01-26 00:28:00','military missile','success',73,'','Dhahran','','','Weapon',758,'',100 from dual union all
select 6444,'1960-A19','1960-01-24 06:59:59','suborbital rocket','success',725,'','','','','Meteo',1211,'VOKV',44 from dual union all
select 64440,'1991-S48','1991-01-26 06:25:00','suborbital rocket','success',224,'NASA 36.057UG','JHU FOT 10 (Io)','','','UV Astron',1873,'',300 from dual union all
select 64441,'1991-S49','1991-01-26 19:46:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64442,'1991-S50','1991-01-28 17:55:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64443,'1991-S51','1991-01-28 18:03:00','military missile','success',795,'','ERIS Target','GBI-1 ERIS Target','','Target',1698,'',1300 from dual union all
select 64444,'1991-S52','1991-01-28 18:24:24','military missile','success',453,'ERIS 1','','','','Interceptor',868,'',270 from dual union all
select 64445,'1991-S53','1991-02-01 00:00:00','military missile','success',73,'','Israel','','','Weapon',762,'',100 from dual union all
select 64446,'1991-S54','1991-02-01 00:00:00','military missile','success',73,'','Israel','','','Weapon',762,'',100 from dual union all
select 64447,'1991-S55','1991-02-01 00:00:00','military missile','success',73,'','Israel','','','Weapon',762,'',100 from dual union all
select 64448,'1991-S56','1991-02-02 00:00:00','military missile','success',73,'','Israel','','','Weapon',762,'',100 from dual;
insert into LAUNCH
select 64449,'1991-S57','1991-02-02 00:00:00','military missile','success',73,'','Israel','','','Weapon',762,'',100 from dual union all
select 6445,'1960-A190','1960-08-04 16:59:13','suborbital spaceplane','success',1520,'1-9-17','Aero Heating','','','Test',379,'NB-52 003',23 from dual union all
select 64450,'1991-S58','1991-02-02 21:41:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64451,'1991-S59','1991-02-03 11:30:00','suborbital rocket','success',891,'DLR K-O-185','Strafam 3/2','','','Aeron',417,'',58 from dual union all
select 64452,'1991-S60','1991-02-06 07:29:21','suborbital rocket','success',686,'','','','','Target/Active Pl.',1821,'',510 from dual union all
select 64453,'1991-S61','1991-02-06 17:30:00','suborbital rocket','success',740,'MR-12','E field','','','Plasma/Active',636,'',150 from dual union all
select 64454,'1991-S62','1991-02-07 08:15:00','military missile','success',1434,'','15Zh58 UBP-11','','','Test',477,'',1000 from dual union all
select 64455,'1991-S63','1991-02-07 22:54:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64456,'1991-S64','1991-02-11 06:45:00','military missile','success',941,'3','','','','Test',1361,'',100 from dual union all
select 64457,'1991-S65','1991-02-11 19:20:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64458,'1991-S66','1991-02-12 03:29:00','suborbital rocket','success',1074,'S-520-14','A-2','','','Auroral',29,'',353 from dual union all
select 64459,'1991-S67','1991-02-12 09:11:54','suborbital rocket','success',237,'NASA 40.001UE','TOPAZ 3','','','Active Plasma',1047,'',1000 from dual union all
select 6446,'1960-A191','1960-08-05 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1237,'SHIP-SOV',43 from dual union all
select 64460,'1991-S68','1991-02-14 08:45:00','military missile','success',73,'','Hafir Al Batin','','','Weapon',758,'',100 from dual union all
select 64461,'1991-S69','1991-02-14 08:45:00','military missile','success',73,'','Hafir Al Batin','','','Weapon',758,'',100 from dual union all
select 64462,'1991-S70','1991-02-15 23:01:00','military missile','success',73,'','Al Jubayl','','','Weapon',758,'',100 from dual union all
select 64463,'1991-S71','1991-02-16 13:00:00','suborbital rocket','success',1074,'S-520-13','EUV Telescopes','','','EUV Astron',833,'',337 from dual union all
select 64464,'1991-S72','1991-02-18 14:18:35','suborbital rocket','success',1279,'LACE','Bow Shock 2','','','Reentry Test',843,'',120 from dual union all
select 64465,'1991-S73','1991-02-21 14:06:00','military missile','success',73,'','KK Mil City','','','Weapon',758,'',100 from dual union all
select 64466,'1991-S74','1991-02-21 14:06:00','military missile','success',73,'','KK Mil City','','','Weapon',758,'',100 from dual union all
select 64467,'1991-S75','1991-02-21 18:00:00','military missile','success',73,'','KK Mil City','','','Weapon',758,'',100 from dual union all
select 64468,'1991-S76','1991-02-21 23:31:00','military missile','success',73,'','Bahrain','','','Weapon',758,'',100 from dual union all
select 64469,'1991-S77','1991-02-22 19:46:00','suborbital rocket','success',224,'NASA 36.070US','Corona Tel 2','','','Solar',1873,'',300 from dual union all
select 6447,'1960-A192','1960-08-06 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1238,'SHIP-SOV',44 from dual union all
select 64470,'1991-S78','1991-02-23 01:59:00','military missile','success',73,'','Dhahran','','','Weapon',758,'',100 from dual union all
select 64471,'1991-S79','1991-02-23 01:59:00','military missile','success',73,'','Dhahran','','','Weapon',758,'',100 from dual union all
select 64472,'1991-S80','1991-02-24 00:00:00','military missile','success',73,'','Israel','','','Weapon',762,'',100 from dual union all
select 64473,'1991-S81','1991-02-24 01:32:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64474,'1991-S82','1991-02-24 06:23:00','military missile','success',73,'','Riyadh','','','Weapon',758,'',100 from dual union all
select 64475,'1991-S83','1991-02-24 09:17:00','military missile','success',73,'','KK Mil City','','','Weapon',758,'',100 from dual union all
select 64476,'1991-S84','1991-02-25 17:32:00','military missile','success',73,'','Dhahran','','','Weapon',758,'',100 from dual union all
select 64477,'1991-S85','1991-02-25 22:26:00','military missile','success',73,'','Qatar','','','Weapon',758,'',100 from dual union all
select 64478,'1991-S86','1991-03-12 10:23:00','military missile','success',907,'05PA (85-176)','','GT 05PA','','OT',1697,'',1000 from dual union all
select 64479,'1991-S87','1991-03-18 10:15:00','suborbital rocket','success',224,'NASA 36.077UH','Colorado SXT','','','XR Astron',1873,'',300 from dual union all
select 6448,'1960-A193','1960-08-09 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1239,'SHIP-SOV',43 from dual union all
select 64480,'1991-S88','1991-03-19 00:00:00','military missile','unknown',1442,'','','','','OT',1897,'SSBN UNK',1000 from dual union all
select 64481,'1991-S89','1991-03-19 00:00:00','military missile','unknown',1442,'','','','','OT',1897,'SSBN UNK',1000 from dual union all
select 64482,'1991-S90','1991-03-23 07:00:00','suborbital rocket','success',224,'NASA 36.068GG','MAMA UV Telescope','','','UV Astron',1873,'',300 from dual union all
select 64483,'1991-S91','1991-03-30 02:27:00','suborbital rocket','success',224,'NASA 36.078UL','','','','Aeron',1873,'',300 from dual union all
select 64484,'1991-S92','1991-04-05 14:05:00','military missile','success',1434,'','15Zh58 UBP-12','','','Test',477,'',1000 from dual union all
select 64485,'1991-S93','1991-04-09 03:41:00','suborbital rocket','success',891,'DLR K-O-186','RASMUS 4','','','Aeron',417,'',65 from dual union all
select 64486,'1991-S94','1991-04-09 18:49:59','suborbital rocket','success',1212,'DLR K-GR-187','DLR SISSI 4','','','Aeron',416,'',245 from dual union all
select 64487,'1991-S95','1991-04-12 00:00:00','suborbital rocket','success',561,'Firebird 1','','','','Target/Decoys',1821,'',400 from dual union all
select 64488,'1991-S96','1991-04-15 22:10:03','military missile','success',1443,'','DASO-6','','','OT',420,'SSBN 736',1000 from dual union all
select 64489,'1991-S97','1991-04-17 07:40:00','suborbital rocket','success',209,'SDI BP-2','SPFE 2 (LACE Target)','','','Tech',1821,'',300 from dual union all
select 6449,'1960-A194','1960-08-09 20:10:00','suborbital rocket','success',838,'NASA 03.28GT','','','','Test',1821,'',26 from dual union all
select 64490,'1991-S98','1991-04-18 18:33:00','military missile','success',1082,'FTM-2A','KMR','','','Test',1726,'',1000 from dual union all
select 64491,'1991-S99','1991-04-25 18:00:00','suborbital rocket','success',860,'NASA 31.79UU','CWAS 19','','','Aeronomy',1047,'',140 from dual union all
select 64492,'1991-W01','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64493,'1991-W02','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64494,'1991-W03','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64495,'1991-W04','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64496,'1991-W05','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64497,'1991-W06','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64498,'1991-W07','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64499,'1991-W08','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 645,'1946-A28','1946-05-20 00:00:00','atmospheric rocket','success',1401,'13','','','','Test',1852,'',4 from dual union all
select 6450,'1960-A195','1960-08-10 00:00:00','military missile','success',771,'GM-54-90','','','','Test',222,'',12 from dual union all
select 64500,'1991-W09','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64501,'1991-W10','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64502,'1991-W100','1991-08-19 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 64503,'1991-W101','1991-08-28 23:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',65 from dual union all
select 64504,'1991-W102','1991-08-31 02:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',67 from dual union all
select 64505,'1991-W103','1991-09-01 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 64506,'1991-W104','1991-09-11 02:00:00','suborbital rocket','success',759,'MT-135-55','NNP-O3','','','Aeron',839,'',56 from dual union all
select 64507,'1991-W105','1991-09-11 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 64508,'1991-W106','1991-09-11 17:22:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',74 from dual union all
select 64509,'1991-W107','1991-09-11 22:33:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',70 from dual union all
select 6451,'1960-A196','1960-08-10 01:31:00','military missile','failure',1053,'CC-2023','','','','Test',280,'',10 from dual union all
select 64510,'1991-W108','1991-09-12 02:00:00','suborbital rocket','success',759,'MT-135-56','NNP-O3','','','Aeron',839,'',57 from dual union all
select 64511,'1991-W109','1991-09-13 00:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 64512,'1991-W11','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64513,'1991-W110','1991-09-24 09:22:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',69 from dual union all
select 64514,'1991-W111','1991-10-02 20:47:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 64515,'1991-W112','1991-10-09 17:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 64516,'1991-W113','1991-10-09 22:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',63 from dual union all
select 64517,'1991-W114','1991-10-13 14:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 64518,'1991-W115','1991-10-16 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',72 from dual union all
select 64519,'1991-W116','1991-10-17 00:08:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',70 from dual union all
select 6452,'1960-A197','1960-08-11 00:00:00','suborbital spaceplane','failure',1520,'1-A-18','','','','Test',379,'NB-52 003',14 from dual union all
select 64520,'1991-W117','1991-10-23 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',66 from dual union all
select 64521,'1991-W118','1991-10-30 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',67 from dual union all
select 64522,'1991-W119','1991-10-30 22:32:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',61 from dual union all
select 64523,'1991-W12','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64524,'1991-W120','1991-11-04 22:09:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',57 from dual union all
select 64525,'1991-W121','1991-11-06 15:54:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 64526,'1991-W122','1991-11-11 13:13:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',70 from dual union all
select 64527,'1991-W123','1991-11-11 20:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',51 from dual union all
select 64528,'1991-W124','1991-11-13 15:21:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 64529,'1991-W125','1991-11-18 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 6453,'1960-A198','1960-08-12 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1240,'SHIP-SOV',39 from dual union all
select 64530,'1991-W126','1991-11-20 15:11:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 64531,'1991-W127','1991-11-22 15:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 64532,'1991-W128','1991-11-27 15:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 64533,'1991-W129','1991-12-05 22:03:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',68 from dual union all
select 64534,'1991-W13','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64535,'1991-W130','1991-12-06 18:44:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 64536,'1991-W131','1991-12-18 22:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',66 from dual union all
select 64537,'1991-W132','1991-01-09 02:05:00','suborbital rocket','success',761,'MT135P-R767','','','','Meteo',1332,'',60 from dual union all
select 64538,'1991-W133','1991-01-16 02:03:00','suborbital rocket','success',761,'MT135P-R768','','','','Meteo',1332,'',60 from dual;
insert into LAUNCH
select 64539,'1991-W134','1991-01-23 02:01:00','suborbital rocket','success',761,'MT135P-R769','','','','Meteo',1332,'',60 from dual union all
select 6454,'1960-A199','1960-08-12 16:48:43','suborbital spaceplane','success',1520,'1-10-19','High Alt','','','Test',379,'NB-52 003',41 from dual union all
select 64540,'1991-W135','1991-01-30 02:08:00','suborbital rocket','success',761,'MT135P-R770','','','','Meteo',1332,'',60 from dual union all
select 64541,'1991-W136','1991-02-06 02:25:00','suborbital rocket','success',761,'MT135P-R771','','','','Meteo',1332,'',60 from dual union all
select 64542,'1991-W137','1991-02-13 02:02:00','suborbital rocket','success',761,'MT135P-R772','','','','Meteo',1332,'',60 from dual union all
select 64543,'1991-W138','1991-02-20 02:20:00','suborbital rocket','success',761,'MT135P-R773','','','','Meteo',1332,'',60 from dual union all
select 64544,'1991-W139','1991-02-27 02:04:00','suborbital rocket','success',761,'MT135P-R774','','','','Meteo',1332,'',60 from dual union all
select 64545,'1991-W14','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64546,'1991-W140','1991-03-13 02:13:00','suborbital rocket','success',761,'MT135P-R775','','','','Meteo',1332,'',60 from dual union all
select 64547,'1991-W141','1991-03-20 02:00:00','suborbital rocket','success',761,'MT135P-R776','','','','Meteo',1332,'',60 from dual union all
select 64548,'1991-W142','1991-03-27 02:00:00','suborbital rocket','success',761,'MT135P-R777','','','','Meteo',1332,'',60 from dual union all
select 64549,'1991-W143','1991-04-10 02:03:00','suborbital rocket','success',761,'MT135P-R778','','','','Meteo',1332,'',60 from dual union all
select 6455,'1960-A20','1960-01-24 16:15:00','military missile','failure',1017,'I1-3','8K74 LKI-3','','','Test',480,'',20 from dual union all
select 64550,'1991-W144','1991-04-17 02:05:00','suborbital rocket','success',761,'MT135P-R779','','','','Meteo',1332,'',60 from dual union all
select 64551,'1991-W145','1991-04-24 02:00:00','suborbital rocket','success',761,'MT135P-R780','','','','Meteo',1332,'',60 from dual union all
select 64552,'1991-W146','1991-05-08 02:02:00','suborbital rocket','success',761,'MT135P-R781','','','','Meteo',1332,'',60 from dual union all
select 64553,'1991-W147','1991-05-15 02:00:00','suborbital rocket','success',761,'MT135P-R782','','','','Meteo',1332,'',60 from dual union all
select 64554,'1991-W148','1991-05-22 02:00:00','suborbital rocket','success',761,'MT135P-R783','','','','Meteo',1332,'',60 from dual union all
select 64555,'1991-W149','1991-06-05 02:25:00','suborbital rocket','success',761,'MT135P-R784','','','','Meteo',1332,'',60 from dual union all
select 64556,'1991-W15','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64557,'1991-W150','1991-06-19 02:10:00','suborbital rocket','success',761,'MT135P-R785','','','','Meteo',1332,'',60 from dual union all
select 64558,'1991-W151','1991-06-26 02:00:00','suborbital rocket','success',761,'MT135P-R786','','','','Meteo',1332,'',60 from dual union all
select 64559,'1991-W152','1991-07-11 02:00:00','suborbital rocket','success',761,'MT135P-R787','','','','Meteo',1332,'',60 from dual union all
select 6456,'1960-A200','1960-08-13 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1241,'SHIP-SOV',33 from dual union all
select 64560,'1991-W153','1991-07-17 02:00:00','suborbital rocket','success',761,'MT135P-R788','','','','Meteo',1332,'',60 from dual union all
select 64561,'1991-W154','1991-08-01 02:00:00','suborbital rocket','success',761,'MT135P-R789','','','','Meteo',1332,'',60 from dual union all
select 64562,'1991-W155','1991-08-14 02:00:00','suborbital rocket','success',761,'MT135P-R790','','','','Meteo',1332,'',60 from dual union all
select 64563,'1991-W156','1991-08-28 02:05:00','suborbital rocket','success',761,'MT135P-R791','','','','Meteo',1332,'',60 from dual union all
select 64564,'1991-W157','1991-09-04 02:00:00','suborbital rocket','success',761,'MT135P-R792','','','','Meteo',1332,'',60 from dual union all
select 64565,'1991-W158','1991-09-12 02:00:00','suborbital rocket','success',761,'MT135P-R793','','','','Meteo',1332,'',60 from dual union all
select 64566,'1991-W159','1991-09-25 02:10:00','suborbital rocket','success',761,'MT135P-R794','','','','Meteo',1332,'',60 from dual union all
select 64567,'1991-W16','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64568,'1991-W160','1991-10-03 02:00:00','suborbital rocket','success',761,'MT135P-R795','','','','Meteo',1332,'',60 from dual union all
select 64569,'1991-W161','1991-10-16 02:00:00','suborbital rocket','success',761,'MT135P-R796','','','','Meteo',1332,'',60 from dual union all
select 6457,'1960-A201','1960-08-14 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1242,'SHIP-SOV',47 from dual union all
select 64570,'1991-W162','1991-10-23 02:03:00','suborbital rocket','success',761,'MT135P-R797','','','','Meteo',1332,'',60 from dual union all
select 64571,'1991-W163','1991-11-06 02:14:00','suborbital rocket','success',761,'MT135P-R798','','','','Meteo',1332,'',60 from dual union all
select 64572,'1991-W164','1991-11-13 02:00:00','suborbital rocket','success',761,'MT135P-R799','','','','Meteo',1332,'',60 from dual union all
select 64573,'1991-W165','1991-11-27 02:00:00','suborbital rocket','success',761,'MT135P-R800','','','','Meteo',1332,'',60 from dual union all
select 64574,'1991-W166','1991-12-04 02:00:00','suborbital rocket','success',761,'MT135P-R801','','','','Meteo',1332,'',60 from dual union all
select 64575,'1991-W167','1991-12-11 02:03:00','suborbital rocket','success',761,'MT135P-R802','','','','Meteo',1332,'',60 from dual union all
select 64576,'1991-W168','1991-12-18 02:00:00','suborbital rocket','success',761,'MT135P-R803','','','','Meteo',1332,'',60 from dual union all
select 64577,'1991-W169','1991-12-25 02:00:00','suborbital rocket','success',761,'MT135P-R804','','','','Meteo',1332,'',60 from dual union all
select 64578,'1991-W17','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64579,'1991-W170','1991-04-09 17:38:00','suborbital rocket','success',1284,'SISSI So5','SISSI So5','','','Aeron',409,'',72 from dual union all
select 6458,'1960-A202','1960-08-15 23:00:00','suborbital rocket','success',725,'','','','','Meteo',1243,'SHIP-SOV',44 from dual union all
select 64580,'1991-W18','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64581,'1991-W19','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64582,'1991-W20','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64583,'1991-W21','1991-01-01 00:00:00','suborbital rocket','unknown',728,'','','','','Aeron',1467,'',79 from dual union all
select 64584,'1991-W22','1991-01-06 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',55 from dual union all
select 64585,'1991-W23','1991-01-16 17:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',73 from dual union all
select 64586,'1991-W24','1991-01-17 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 64587,'1991-W25','1991-01-29 09:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',77 from dual union all
select 64588,'1991-W26','1991-01-30 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 64589,'1991-W27','1991-02-05 09:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',63 from dual union all
select 6459,'1960-A203','1960-08-17 00:00:00','atmospheric rocket','success',620,'G1','Dummy St 2/St 3','','','Test',2,'',20 from dual union all
select 64590,'1991-W28','1991-02-05 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',60 from dual union all
select 64591,'1991-W29','1991-02-09 02:00:00','suborbital rocket','success',759,'MT-135-53','NNP-O3','','','Aeron',839,'',57 from dual union all
select 64592,'1991-W30','1991-02-11 02:10:00','suborbital rocket','success',759,'MT-135-54','NNP-O3','','','Aeron',839,'',62 from dual union all
select 64593,'1991-W31','1991-02-13 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 64594,'1991-W32','1991-02-13 17:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',76 from dual union all
select 64595,'1991-W33','1991-02-19 09:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',53 from dual union all
select 64596,'1991-W34','1991-02-27 15:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 64597,'1991-W35','1991-03-06 17:36:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',70 from dual union all
select 64598,'1991-W36','1991-03-07 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 64599,'1991-W37','1991-03-13 17:31:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',77 from dual union all
select 646,'1946-A29','1946-05-23 00:00:00','atmospheric rocket','success',1401,'14','','','','Test',1852,'',4 from dual union all
select 6460,'1960-A204','1960-08-18 00:00:00','suborbital spaceplane','failure',1520,'1- A-20','','','','Test',379,'NB-52 003',14 from dual union all
select 64600,'1991-W38','1991-03-13 21:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',62 from dual union all
select 64601,'1991-W39','1991-03-20 08:32:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',67 from dual union all
select 64602,'1991-W40','1991-03-27 17:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',62 from dual union all
select 64603,'1991-W41','1991-03-27 18:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 64604,'1991-W42','1991-04-02 09:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',67 from dual union all
select 64605,'1991-W43','1991-04-02 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',57 from dual union all
select 64606,'1991-W44','1991-04-03 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',71 from dual union all
select 64607,'1991-W45','1991-04-03 16:03:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',70 from dual union all
select 64608,'1991-W46','1991-04-04 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 64609,'1991-W47','1991-04-05 17:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 6461,'1960-A205','1960-08-19 16:34:22','suborbital spaceplane','success',1520,'1-11-21','Aero Heating','','','Test',379,'NB-52 003',23 from dual union all
select 64610,'1991-W48','1991-04-10 12:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',62 from dual union all
select 64611,'1991-W49','1991-04-10 13:01:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',64 from dual union all
select 64612,'1991-W50','1991-04-10 13:26:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',66 from dual union all
select 64613,'1991-W51','1991-04-11 11:02:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',64 from dual union all
select 64614,'1991-W52','1991-04-11 12:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',56 from dual union all
select 64615,'1991-W53','1991-04-11 12:11:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',60 from dual union all
select 64616,'1991-W54','1991-04-11 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',58 from dual union all
select 64617,'1991-W55','1991-04-11 23:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',64 from dual union all
select 64618,'1991-W56','1991-04-16 01:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',66 from dual union all
select 64619,'1991-W57','1991-04-17 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 6462,'1960-A206','1960-08-22 17:40:57','suborbital rocket','failure',838,'NASA 03.12CI','GCA Ionos 1','','','Ionos',1821,'',23 from dual union all
select 64620,'1991-W58','1991-04-22 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 64621,'1991-W59','1991-04-26 15:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 64622,'1991-W60','1991-04-28 13:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 64623,'1991-W61','1991-05-03 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 64624,'1991-W62','1991-05-06 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',70 from dual union all
select 64625,'1991-W63','1991-05-06 15:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',69 from dual union all
select 64626,'1991-W64','1991-05-06 16:02:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',66 from dual union all
select 64627,'1991-W65','1991-05-06 18:27:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',71 from dual union all
select 64628,'1991-W66','1991-05-06 20:23:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual;
insert into LAUNCH
select 64629,'1991-W67','1991-05-15 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',72 from dual union all
select 6463,'1960-A207','1960-08-23 00:00:00','military missile','success',771,'GM-54-64','','','','Test',222,'',12 from dual union all
select 64630,'1991-W68','1991-05-27 16:32:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',79 from dual union all
select 64631,'1991-W69','1991-05-28 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 64632,'1991-W70','1991-05-29 05:49:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',873,'',70 from dual union all
select 64633,'1991-W71','1991-05-31 13:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',52 from dual union all
select 64634,'1991-W72','1991-06-04 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 64635,'1991-W73','1991-06-05 15:00:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 64636,'1991-W74','1991-06-07 13:33:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 64637,'1991-W75','1991-06-14 12:00:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',66 from dual union all
select 64638,'1991-W76','1991-06-14 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',69 from dual union all
select 64639,'1991-W77','1991-06-14 13:18:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',58 from dual union all
select 6464,'1960-A208','1960-08-23 00:00:00','military missile','success',771,'GM-54-66','','','','Test',222,'',12 from dual union all
select 64640,'1991-W78','1991-06-14 14:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',67 from dual union all
select 64641,'1991-W79','1991-06-17 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 64642,'1991-W80','1991-06-26 16:43:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',70 from dual union all
select 64643,'1991-W81','1991-06-28 01:38:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',67 from dual union all
select 64644,'1991-W82','1991-06-28 19:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',65 from dual union all
select 64645,'1991-W83','1991-07-01 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 64646,'1991-W84','1991-07-01 18:29:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',63 from dual union all
select 64647,'1991-W85','1991-07-17 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 64648,'1991-W86','1991-07-17 17:02:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',50 from dual union all
select 64649,'1991-W87','1991-07-23 14:55:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 6465,'1960-A209','1960-08-24 20:43:00','military missile','success',928,'A1','','','','Test',1338,'',2 from dual union all
select 64650,'1991-W88','1991-07-29 22:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',67 from dual union all
select 64651,'1991-W89','1991-07-31 14:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 64652,'1991-W90','1991-08-02 16:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 64653,'1991-W91','1991-08-11 08:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',66 from dual union all
select 64654,'1991-W92','1991-08-11 09:33:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',69 from dual union all
select 64655,'1991-W93','1991-08-11 10:21:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',50 from dual union all
select 64656,'1991-W94','1991-08-11 10:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',66 from dual union all
select 64657,'1991-W95','1991-08-11 11:52:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',63 from dual union all
select 64658,'1991-W96','1991-08-11 12:37:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',75 from dual union all
select 64659,'1991-W97','1991-08-11 13:12:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 6466,'1960-A21','1960-01-27 00:00:00','military missile','success',793,'','Silo Test','','','Test',381,'',0 from dual union all
select 64660,'1991-W98','1991-08-11 13:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',58 from dual union all
select 64661,'1991-W99','1991-08-17 01:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',61 from dual union all
select 64662,'1991-Y01','1991-01-09 22:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 64663,'1991-Y02','1991-01-23 22:20:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',95 from dual union all
select 64664,'1991-Y03','1991-02-20 22:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 64665,'1991-Y04','1991-02-27 22:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 64666,'1991-Y05','1991-03-20 22:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 64667,'1991-Y06','1991-03-20 22:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 64668,'1991-Y07','1991-03-27 22:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',95 from dual union all
select 64669,'1991-Y08','1991-04-05 15:10:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 6467,'1960-A210','1960-08-26 00:00:00','military missile','success',1214,'SM-62A N-3448','Cat IIE-7, Sta12','','','OT',224,'',10 from dual union all
select 64670,'1991-Y09','1991-04-10 11:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',93 from dual union all
select 64671,'1991-Y10','1991-04-10 11:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',91 from dual union all
select 64672,'1991-Y11','1991-04-10 21:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 64673,'1991-Y12','1991-04-11 10:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',90 from dual union all
select 64674,'1991-Y13','1991-04-11 11:35:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',89 from dual union all
select 64675,'1991-Y14','1991-04-17 21:22:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 64676,'1991-Y15','1991-04-24 21:23:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 64677,'1991-Y16','1991-04-28 12:18:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 64678,'1991-Y17','1991-04-29 16:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1307,'',81 from dual union all
select 64679,'1991-Y18','1991-05-06 15:21:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',95 from dual union all
select 6468,'1960-A211','1960-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64680,'1991-Y19','1991-05-06 19:09:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',86 from dual union all
select 64681,'1991-Y20','1991-06-05 14:25:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',86 from dual union all
select 64682,'1991-Y21','1991-07-11 00:00:00','suborbital rocket','success',1502,'','Imaging Radiometer','','','Aeron/Eclipse',765,'',90 from dual union all
select 64683,'1991-Y22','1991-08-01 01:28:00','suborbital rocket','success',1281,'NASA 15.249UU NLC-91','N-B-M02 MISTI B','','','Ionos',417,'',89 from dual union all
select 64684,'1991-Y23','1991-08-01 01:54:00','suborbital rocket','success',1502,'Sphere        NLC-91','N-B-S06 PMSE','','','Aeron',417,'',111 from dual union all
select 64685,'1991-Y24','1991-08-01 02:24:00','suborbital rocket','success',1502,'Chaff         NLC-91','N-B-C07 PMSE (VCH28)','','','Aeron',417,'',115 from dual union all
select 64686,'1991-Y25','1991-08-01 02:39:00','suborbital rocket','success',1502,'Sphere        NLC-91','N-B-S08 PMSE','','','Aeron',417,'',112 from dual union all
select 64687,'1991-Y26','1991-08-02 16:02:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',86 from dual union all
select 64688,'1991-Y27','1991-08-05 23:21:00','suborbital rocket','success',1281,'NASA 15.250UE NLC-91','N-C-M09 MISTI C','','','Plasma',417,'',88 from dual union all
select 64689,'1991-Y28','1991-08-09 22:53:00','suborbital rocket','success',1502,'Sphere        NLC-91','N-A-S12 NLC','','','Aeron',417,'',115 from dual union all
select 6469,'1960-A212','1960-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64690,'1991-Y29','1991-08-09 23:30:00','suborbital rocket','success',1281,'NASA 15.251UE NLC-91','N-A-M15 MISTI A','','','Plasma',417,'',80 from dual union all
select 64691,'1991-Y30','1991-08-10 00:06:00','suborbital rocket','success',1502,'Sphere        NLC-91','N-A-S18 NLC','','','Aeron',417,'',101 from dual union all
select 64692,'1991-Y31','1991-08-10 00:24:00','suborbital rocket','success',1502,'Chaff         NLC-91','N-A-C19 NLC (VCH42)','','','Aeron',417,'',115 from dual union all
select 64693,'1991-Y32','1991-08-11 07:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',92 from dual union all
select 64694,'1991-Y33','1991-08-11 08:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',855,'',95 from dual union all
select 64695,'1991-Y34','1991-08-31 00:41:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',84 from dual union all
select 64696,'1991-Y35','1991-09-13 00:10:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 64697,'1991-Y36','1991-10-30 21:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',873,'',81 from dual union all
select 64698,'1991-Y37','1991-11-01 09:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',86 from dual union all
select 64699,'1991-Y38','1991-11-06 15:14:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',83 from dual union all
select 647,'1946-A30','1946-05-23 00:00:00','atmospheric rocket','success',1401,'15','','','','Test',1852,'',4 from dual union all
select 6470,'1960-A213','1960-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64700,'1991-Y39','1991-12-14 17:55:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',92 from dual union all
select 64701,'1991-Y40','1991-12-17 14:04:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',83 from dual union all
select 64702,'1991-Y41','1991-12-21 16:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',92 from dual union all
select 64703,'1991-Y42','1991-12-23 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 64704,'1991-Y43','1991-12-23 15:45:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 64705,'1991-Y44','1991-08-01 01:03:00','suborbital rocket','success',1502,'Sphere        NLC-91','N-B-S01 PMSE','','','Aeron',417,'',112 from dual union all
select 64706,'1991-Y45','1991-08-05 23:32:00','suborbital rocket','success',1502,'Sphere        NLC-91','N-C-S10 Salvo C','','','Aeron',417,'',113 from dual union all
select 64707,'1991-Y46','1991-08-09 22:39:00','suborbital rocket','failure',1502,'Sphere        NLC-91','N-A-S11 NLC','','','Aeron',417,'',5 from dual union all
select 64708,'1991-Y47','1991-08-10 01:42:00','suborbital rocket','success',1502,'Sphere        NLC-91','N-A-S21 NLC','','','Aeron',417,'',117 from dual union all
select 64709,'1991-Y48','1991-04-09 18:18:00','suborbital rocket','success',1502,'Robin Sphere','SISSI RD1','','','Aeron',417,'',110 from dual union all
select 6471,'1960-A214','1960-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64710,'1991-Y49','1991-04-09 19:20:00','suborbital rocket','success',1502,'Robin Sphere','SISSI RD2','','','Aeron',417,'',110 from dual union all
select 64711,'1991-Y50','1991-04-09 19:50:00','suborbital rocket','success',1284,'SISSI So6','SISSI So6','','','Aeron',409,'',80 from dual union all
select 64712,'1991-Y51','1991-04-09 20:41:00','suborbital rocket','success',1284,'SISSI So7','SISSI So7','','','Aeron',409,'',87 from dual union all
select 64713,'1992-001','1992-01-21 15:00:00','orbital','success',1238,'77053  -985','Kosmos-2175','Kobal''t','','Sat',475,'',null from dual union all
select 64714,'1992-002','1992-01-22 14:52:33','orbital','success',1243,'STS-42','Spacelab IML-1','OV-103 Spacelab','','Sat',921,'MLP3',null from dual union all
select 64715,'1992-003','1992-01-24 01:18:01','orbital','success',803,'78023  -608','Kosmos-2176','Oko','','Sat',474,'',null from dual union all
select 64716,'1992-004','1992-01-25 07:50:17','orbital','success',1242,'R 15000-058','Progress M-11','Progress 7K-TGM No. 212','','Sat',480,'',null from dual union all
select 64717,'1992-005','1992-01-29 22:19:12','orbital','success',958,'372-02','Kosmos-2177/2179','Uragan No. 68L','','Sat',587,'',null from dual union all
select 64718,'1992-006','1992-02-11 00:41:02','orbital','success',172,'AC-101','USA 78 (DSCS 3B-14)','DSCS III B-14','','Sat',268,'',null from dual;
insert into LAUNCH
select 64719,'1992-007','1992-02-11 01:50:00','orbital','success',522,'H-24(F)','Fuyo','ERS-1','','Sat',1474,'',null from dual union all
select 6472,'1960-A215','1960-09-02 00:00:00','suborbital spaceplane','failure',1520,'1- A-22','','','','Test',379,'NB-52',14 from dual union all
select 64720,'1992-008','1992-02-17 22:05:08','orbital','success',680,'65078-418','Kosmos-2180','Parus','','Sat',460,'',null from dual union all
select 64721,'1992-009','1992-02-23 22:29:00','orbital','success',422,'D207','USA 79 (GPS IIA-12)','Navstar GPS SVN 25','','Sat',234,'',null from dual union all
select 64722,'1992-010','1992-02-26 23:58:10','orbital','success',103,'V49 L421b','Superbird B1/Arab 1C','Superbird B1','','Sat',345,'',null from dual union all
select 64723,'1992-011','1992-03-04 04:27:00','orbital','success',805,'76047  -567','Molniya-1','Molniya-1T!No. 75','','Sat',475,'',null from dual union all
select 64724,'1992-012','1992-03-09 22:35:59','orbital','success',680,'53778-423','Kosmos-2181','Tsikada','','Sat',456,'',null from dual union all
select 64725,'1992-013','1992-03-14 00:00:00','orbital','success',171,'AC-72','Galaxy 5','Galaxy V','','Sat',269,'',null from dual union all
select 64726,'1992-014','1992-03-17 10:54:30','orbital','success',1242,'R 15000-056','Soyuz TM-14','Soyuz 7K-STM No. 64','','Sat',480,'',null from dual union all
select 64727,'1992-015','1992-03-24 13:13:39','orbital','success',1243,'STS-45','Spacelab Atlas 1','OV-104 Spacelab','','Sat',921,'MLP1',null from dual union all
select 64728,'1992-016','1992-04-01 14:18:00','orbital','success',1241,'78023  -988','Kosmos-2182','Kobal''t','','Sat',462,'',null from dual union all
select 64729,'1992-017','1992-04-02 01:50:00','orbital','success',958,'369-01','Gorizont','Gorizont No. 36L','','Sat',587,'',null from dual union all
select 6473,'1960-A216','1960-09-07 00:00:00','military missile','success',771,'GM-56-1929','','','','Test',222,'',12 from dual union all
select 64730,'1992-018','1992-04-08 12:20:00','orbital','success',1238,'77053  -750','Kosmos-2183','Neman','','Sat',562,'',null from dual union all
select 64731,'1992-019','1992-04-10 03:20:00','orbital','success',422,'D208','USA 80 (GPS IIA-13)','Navstar GPS SVN 28','','Sat',234,'',null from dual union all
select 64732,'1992-020','1992-04-15 07:17:43','orbital','success',680,'47178-419','Kosmos-2184','Parus','','Sat',456,'',null from dual union all
select 64733,'1992-021','1992-04-15 23:25:27','orbital','success',104,'V50 L422a','Inmarsat 2F4/Tel. 2B','Telecom 2B','','Sat',345,'',null from dual union all
select 64734,'1992-022','1992-04-19 21:29:25','orbital','success',1242,'R 15000-059','Progress M-12','Progress 7K-TGM No. 213','','Sat',480,'',null from dual union all
select 64735,'1992-023','1992-04-25 08:53:00','orbital','success',1422,'23G-3','USA 81       [T2-3]','P-11 No. 5105','','Sat',1778,'',null from dual union all
select 64736,'1992-024','1992-04-29 09:00:00','orbital','success',1238,'78012  -267','Resurs-F','Resurs-F2 17F42 No. 8','','Sat',475,'',null from dual union all
select 64737,'1992-025','1992-04-29 10:10:00','orbital','success',1238,'77017  -738','Kosmos-2185','Kometa No. 15','','Sat',480,'',null from dual union all
select 64738,'1992-026','1992-05-07 23:40:00','orbital','success',1243,'STS-49','Intelsat Rescue','OV-105 Rendezvous','','Sat',922,'MLP2',null from dual union all
select 64739,'1992-027','1992-05-14 00:40:00','orbital','success',422,'D209','Palapa B4','Palapa B4','','Sat',234,'',null from dual union all
select 6474,'1960-A217','1960-09-08 18:30:00','atmospheric rocket','success',835,'Smoke 005','','','','Aeronomy',1821,'',9 from dual union all
select 64740,'1992-028','1992-05-20 00:30:00','orbital','success',22,'ASLV-D3','SROSS C','SROSS-C','','Sat',1360,'',null from dual union all
select 64741,'1992-029','1992-05-28 19:09:59','orbital','success',1238,'78023  -986','Kosmos-2186','Kobal''t','','Sat',462,'',null from dual union all
select 64742,'1992-030','1992-06-03 00:50:30','orbital','success',680,'53742-114','Kosmos-2187/2194','Strela-1M','','Sat',460,'',null from dual union all
select 64743,'1992-031','1992-06-07 16:40:00','orbital','success',406,'D210','EUVE','EUVE','','Sat',233,'',null from dual union all
select 64744,'1992-032','1992-06-10 00:00:00','orbital','success',173,'AC-105','Intelsat K','INTELSAT K','','Sat',269,'',null from dual union all
select 64745,'1992-033','1992-06-23 08:00:00','orbital','success',1241,'78012  -154','Resurs-F','Resurs-F1 14F43 No. 55','','Sat',474,'',null from dual union all
select 64746,'1992-034','1992-06-25 16:12:22','orbital','success',1243,'STS-50','Spacelab USML-1','OV-102 Spacelab','','Sat',921,'MLP3',null from dual union all
select 64747,'1992-035','1992-06-30 16:43:13','orbital','success',1242,'U 15000-062','Progress M-13','Progress 7K-TGM No. 214','','Sat',562,'',null from dual union all
select 64748,'1992-036','1992-07-01 20:16:22','orbital','success',680,'65072-427','Kosmos-2195','Parus','','Sat',460,'',null from dual union all
select 64749,'1992-037','1992-07-02 21:54:01','orbital','success',172,'AC-103','USA 82 (DSCS 3B-12)','DSCS III B-12','','Sat',268,'',null from dual union all
select 6475,'1960-A218','1960-09-10 19:45:10','suborbital spaceplane','success',1520,'1-12-23','Stability/Control','','','Test',379,'NB-52 008',24 from dual union all
select 64750,'1992-038','1992-07-03 14:19:00','orbital','success',1135,'S215C','SAMPEX','SAMPEX','','Sat',1779,'',null from dual union all
select 64751,'1992-039','1992-07-07 09:20:01','orbital','success',422,'D211','USA 83 (GPS IIA-14)','Navstar GPS SVN 26','','Sat',234,'',null from dual union all
select 64752,'1992-040','1992-07-08 09:53:14','orbital','success',810,'77053  -605','Kosmos-2196','Oko','','Sat',474,'',null from dual union all
select 64753,'1992-041','1992-07-09 22:42:19','orbital','success',103,'V51 L423c','Insat 2A/Eutel. 2F4','Insat 2A','','Sat',345,'',null from dual union all
select 64754,'1992-042','1992-07-13 17:41:40','orbital','success',1448,'51019-208','Kosmos-2197/2202','Strela-3','','Sat',469,'',null from dual union all
select 64755,'1992-043','1992-07-14 22:02:00','orbital','success',958,'371-02','Gorizont','Gorizont No. 37L','','Sat',587,'',null from dual union all
select 64756,'1992-044','1992-07-24 14:26:00','deep space','success',408,'D212','Geotail','Geotail','','Sat',233,'',null from dual union all
select 64757,'1992-045','1992-07-24 19:40:00','orbital','success',1238,'78012  -989','Kosmos-2203','Kobal''t','','Sat',474,'',null from dual union all
select 64758,'1992-046','1992-07-27 06:08:42','orbital','success',1242,'R 15000-053','Soyuz TM-15','Soyuz 7K-STM No. 65','','Sat',480,'',null from dual union all
select 64759,'1992-047','1992-07-30 01:59:01','orbital','success',958,'376-01','Kosmos-2204/2206','Uragan No. 56L','','Sat',587,'',null from dual union all
select 6476,'1960-A219','1960-09-13 17:55:00','military missile','failure',928,'A1XE-5 (A1XP-7)','','','','Test',421,'SSBN 599',0 from dual union all
select 64760,'1992-048','1992-07-30 11:00:00','orbital','success',1241,'76032  -832','Kosmos-2207','Oblik','','Sat',475,'',null from dual union all
select 64761,'1992-049','1992-07-31 13:56:48','orbital','success',1243,'STS-46','TSS-1/Eureca','OV-104 Research/FF','','Sat',922,'MLP1',null from dual union all
select 64762,'1992-050','1992-08-06 19:30:59','orbital','success',805,'78039  -558','Molniya-1','Molniya-1T!No. 72','','Sat',474,'',null from dual union all
select 64763,'1992-051','1992-08-09 08:00:00','orbital','success',304,'Y1       (27)','FSW-2','FSW-2 No. 1','','Sat',810,'',null from dual union all
select 64764,'1992-052','1992-08-10 23:08:07','orbital','success',100,'V52 L424b','Topex/ASAP 3','Topex-Poseidon','','Sat',345,'',null from dual union all
select 64765,'1992-053','1992-08-12 05:44:01','orbital','success',680,'65078-421','Kosmos-2208','Strela-2M','','Sat',456,'',null from dual union all
select 64766,'1992-054','1992-08-13 23:00:00','orbital','success',305,'Y3       (28)','Optus B1','Optus B1','','Sat',1903,'',null from dual union all
select 64767,'1992-055','1992-08-15 22:18:32','orbital','success',1242,'U 15000-064','Progress M-14','Progress 7K-TGM No. 209','','Sat',562,'',null from dual union all
select 64768,'1992-056','1992-08-19 10:20:00','orbital','success',1238,'76024  -157','Resurs-F','Resurs-F1 14F43 No. 54','','Sat',462,'',null from dual union all
select 64769,'1992-057','1992-08-31 10:40:59','orbital','success',422,'D213','Satcom C4','Satcom C-4','','Sat',234,'',null from dual union all
select 6477,'1960-A22','1960-01-29 00:00:00','military missile','success',256,'631-6','','','','OT',272,'',20 from dual union all
select 64770,'1992-058','1992-09-09 08:57:00','orbital','success',422,'D214','USA 84 (GPS IIA-15)','Navstar GPS SVN 27','','Sat',233,'',null from dual union all
select 64771,'1992-059','1992-09-10 18:01:18','orbital','success',958,'363-01','Kosmos-2209','Oko-S F5','','Sat',587,'',null from dual union all
select 64772,'1992-060','1992-09-10 23:04:00','orbital','success',107,'V53 L425a','Hisp. 1A/Satcom C3','Hispasat 1A','','Sat',345,'',null from dual union all
select 64773,'1992-061','1992-09-12 14:23:00','orbital','success',1243,'STS-47','Spacelab J','OV-105 Spacelab','','Sat',922,'MLP2',null from dual union all
select 64774,'1992-062','1992-09-22 16:10:00','orbital','success',1238,'78023  -987','Kosmos-2210','Kobal''t','','Sat',462,'',null from dual union all
select 64775,'1992-063','1992-09-25 17:05:01','deep space','success',335,'CT-4','Mars Observer','Mars Observer','','Sat',274,'',null from dual union all
select 64776,'1992-064','1992-10-06 06:20:05','orbital','success',301,'Y10      (29)','FSW-1/Freja','Freja','','Sat',810,'',null from dual union all
select 64777,'1992-065','1992-10-08 19:00:00','orbital','success',1238,'U 15000-623','Foton','Foton No. 8L','','Sat',475,'',null from dual union all
select 64778,'1992-066','1992-10-12 09:47:00','orbital','success',422,'D215','DFS 3','DFS 3','','Sat',234,'',null from dual union all
select 64779,'1992-067','1992-10-14 19:58:00','orbital','success',811,'78012  -641','Molniya-3','Molniya-3 No. 50','','Sat',474,'',null from dual union all
select 6478,'1960-A220','1960-09-17 02:47:00','suborbital rocket','success',662,'K-6-TW-8','K-6-18','Grenade','','Aeron',7,'',46 from dual union all
select 64780,'1992-068','1992-10-20 12:58:12','orbital','success',1448,'67019-209','Kosmos-2211/2216','Strela-3','','Sat',469,'',null from dual union all
select 64781,'1992-069','1992-10-21 10:21:22','orbital','success',810,'77053  -604','Kosmos-2217','Oko','','Sat',462,'',null from dual union all
select 64782,'1992-070','1992-10-22 17:09:40','orbital','success',1243,'STS-52','Lageos 2/USMP-1','OV-102 Research/Med dep','','Sat',922,'MLP3',null from dual union all
select 64783,'1992-071','1992-10-27 17:19:41','orbital','success',1242,'U 15000-061','Progress M-15','Progress 7K-TGM No. 215','','Sat',562,'',null from dual union all
select 64784,'1992-072','1992-10-28 00:15:00','orbital','success',101,'V54 L426a','Galaxy 7','Galaxy VII','','Sat',345,'',null from dual union all
select 64785,'1992-073','1992-10-29 10:40:33','orbital','success',680,'65078-424','Kosmos-2218','Parus','','Sat',460,'',null from dual union all
select 64786,'1992-074','1992-10-30 14:59:00','orbital','success',958,'372-01','Ekran','Ekran-M No. 15L','','Sat',587,'',null from dual union all
select 64787,'1992-075','1992-11-15 21:45:01','orbital','success',1241,'U 15000-037','Resurs-500','Resurs-500 No. 1','','Sat',462,'',null from dual union all
select 64788,'1992-076','1992-11-17 07:47:01','orbital','success',1537,'67092-802','Kosmos-2219','Tselina-2','','Sat',567,'',null from dual union all
select 64789,'1992-077','1992-11-20 15:29:59','orbital','success',1238,'78012  -990','Kosmos-2220','Kobal''t','','Sat',475,'',null from dual union all
select 6479,'1960-A221','1960-09-20 00:00:00','suborbital spaceplane','failure',1520,'1- A-24','','','','Test',379,'NB-52',14 from dual union all
select 64790,'1992-078','1992-11-21 13:45:00','orbital','success',1135,'S210C','MSTI 1','MSTI-1','','Sat',1779,'',null from dual union all
select 64791,'1992-079','1992-11-22 23:54:00','orbital','success',422,'D216','USA 85 (GPS IIA-16)','Navstar GPS SVN 32','','Sat',233,'',null from dual union all
select 64792,'1992-080','1992-11-24 04:09:59','orbital','success',1448,'51027-905','Kosmos-2221','Tselina-D','','Sat',470,'',null from dual union all
select 64793,'1992-081','1992-11-25 12:18:54','orbital','success',810,'77059  -565','Kosmos-2222','Oko','','Sat',474,'',null from dual union all
select 64794,'1992-082','1992-11-27 13:10:00','orbital','success',958,'364-01','Gorizont','Gorizont No. 38L','','Sat',587,'',null from dual union all
select 64795,'1992-083','1992-11-28 21:34:00','orbital','success',1415,'K-3    (45J-1)','USA 86       [IC4]','Improved CRYSTAL 2104','','Sat',1777,'',null from dual union all
select 64796,'1992-084','1992-12-01 22:48:00','orbital','success',101,'V55 L427a','Superbird A1','Superbird A1','','Sat',345,'',null from dual union all
select 64797,'1992-085','1992-12-02 01:57:00','orbital','success',811,'78023  -607','Molniya-3','Molniya-3 No. 56','','Sat',474,'',null from dual union all
select 64798,'1992-086','1992-12-02 13:24:00','orbital','success',1243,'STS-53','USA 89   (QUASAR 10)','OV-103 Medium deploy','','Sat',921,'MLP1',null from dual union all
select 64799,'1992-087','1992-12-09 11:25:00','orbital','success',1238,'76032  -495','Kosmos-2223','Neman','','Sat',480,'',null from dual union all
select 648,'1946-A31','1946-05-24 00:00:00','atmospheric rocket','success',1401,'16','','','','Test',1852,'',4 from dual union all
select 6480,'1960-A222','1960-09-20 00:16:00','suborbital rocket','failure',839,'AA6.166','Sphere','','','Aeronomy',428,'',0 from dual union all
select 64800,'1992-088','1992-12-17 12:45:00','orbital','success',958,'357-02','Kosmos-2224','Oko-1 No. 7121','','Sat',553,'',null from dual union all
select 64801,'1992-089','1992-12-18 22:16:00','orbital','success',422,'D217','USA 87 (GPS IIA-17)','Navstar GPS SVN 29','','Sat',234,'',null from dual union all
select 64802,'1992-090','1992-12-21 11:21:00','orbital','failure',305,'Y4       (30)','Optus B2','Optus B2','','Sat',1903,'',1032 from dual union all
select 64803,'1992-091','1992-12-22 12:00:00','orbital','success',1238,'76032  -314','Kosmos-2225','Don','','Sat',562,'',null from dual union all
select 64804,'1992-092','1992-12-22 12:36:00','orbital','success',1448,'67027-910','Kosmos-2226','Musson No. 23','','Sat',470,'',null from dual union all
select 64805,'1992-093','1992-12-25 05:56:00','orbital','success',1537,'67033-306','Kosmos-2227','Tselina-2','','Sat',567,'',null from dual union all
select 64806,'1992-094','1992-12-25 20:07:59','orbital','success',1448,'45027-906','Kosmos-2228','Tselina-D','','Sat',470,'',null from dual union all
select 64807,'1992-095','1992-12-29 13:30:01','orbital','success',1241,'U 15000-033','Kosmos-2229','Bion No. 10','','Sat',474,'',null from dual union all
select 64808,'1992-A01','1992-02-13 00:00:00','military missile','failure',941,'6','','','','Test',1361,'',0 from dual;
insert into LAUNCH
select 64809,'1992-A02','1992-03-03 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',35,'',49 from dual union all
select 6481,'1960-A223','1960-09-20 20:18:00','suborbital rocket','success',839,'NASA 10.05GA','GSFC Grenade 4','','','Aeronomy',1821,'',100 from dual union all
select 64810,'1992-A03','1992-03-06 07:32:00','suborbital rocket','failure',738,'','','','','Aeron',1917,'',49 from dual union all
select 64811,'1992-A04','1992-03-18 00:00:00','suborbital rocket','failure',1136,'','','','','Test',1341,'',1 from dual union all
select 64812,'1992-A05','1992-03-20 07:40:00','suborbital rocket','failure',738,'','','','','Aeron',1917,'',37 from dual union all
select 64813,'1992-A06','1992-04-02 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',49 from dual union all
select 64814,'1992-A07','1992-04-03 06:30:00','suborbital rocket','failure',738,'','','','','Aeron',1917,'',20 from dual union all
select 64815,'1992-A08','1992-04-29 00:00:00','military missile','failure',354,'DF-21A test','','','','Test',1494,'',0 from dual union all
select 64816,'1992-A09','1992-11-01 00:00:00','military missile','failure',354,'DF-21A test','','','','Test',1494,'',0 from dual union all
select 64817,'1992-A10','1992-05-28 00:00:00','military missile','failure',68,'02','','','','Test',320,'',10 from dual union all
select 64818,'1992-A11','1992-06-01 00:00:00','military missile','failure',586,'','','','','Test',1480,'',0 from dual union all
select 64819,'1992-A12','1992-08-26 14:31:00','military missile','success',539,'','KITE-2A','','','Test',1883,'',12 from dual union all
select 6482,'1960-A224','1960-09-21 00:00:00','military missile','success',764,'58-1226   (21)','','','','Test',240,'',12 from dual union all
select 64820,'1992-A13','1992-09-23 00:00:00','military missile','success',118,'','','','','Test',1024,'SHIP-ISR',20 from dual union all
select 64821,'1992-A14','1992-09-24 00:00:00','suborbital rocket','success',1335,'Rockwell LEAP','FTV-1 (Dummy LEAP)','','','Interceptor',1057,'CG-20',30 from dual union all
select 64822,'1992-A15','1992-10-22 00:00:00','suborbital rocket','failure',117,'BP-3 (ALV)','SPFE-3/Orbus 1','','','Target',1821,'',20 from dual union all
select 64823,'1992-A16','1992-10-31 00:00:00','suborbital rocket','success',1216,'XV-56','Ponta de Areia','','','Ionos?',13,'',32 from dual union all
select 64824,'1992-A17','1992-12-01 00:00:00','military missile','success',1088,'','Block IV Demo','','','Test',1895,'',10 from dual union all
select 64825,'1992-A18','1992-12-01 08:25:00','military missile','failure',1453,'63022-82-422','15A35 UBP-41','GCh 15F355P7','','OT',535,'',0 from dual union all
select 64826,'1992-A19','1992-12-15 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',47 from dual union all
select 64827,'1992-F01','1992-02-05 18:14:00','orbital','failure',1537,'45082-801/17L','','Tselina-2','','Sat',567,'',150 from dual union all
select 64828,'1992-F02','1992-08-22 22:40:00','orbital','failure',171,'AC-71','Galaxy 1R','Galaxy IR','','Sat',269,'',164 from dual union all
select 64829,'1992-M01','1992-02-14 23:00:00','test rocket','success',541,'RFT-2','HIMES RFT-2','','','Test',839,'',67 from dual union all
select 6483,'1960-A225','1960-09-21 23:20:00','suborbital rocket','failure',839,'OB6.05','H2O/ne','','','Aeron',428,'',1 from dual union all
select 64830,'1992-M02','1992-09-21 18:32:00','suborbital rocket','success',891,'NASA 30.40UP','CSOAR','','','Aeron/Education',1821,'',54 from dual union all
select 64831,'1992-M03','1992-10-14 00:00:00','suborbital rocket','success',1023,'','ABR 200 (I)','','','Test',1361,'',70 from dual union all
select 64832,'1992-M04','1992-10-19 00:00:00','suborbital rocket','success',1023,'','ABR 200 (I)','','','Test',1361,'',70 from dual union all
select 64833,'1992-M05','1992-01-01 00:00:00','military missile','success',608,'II-1','','','','Test',636,'',50 from dual union all
select 64834,'1992-S01','1992-01-01 00:00:00','military missile','success',567,'','','','','Test',307,'',150 from dual union all
select 64835,'1992-S02','1992-01-01 00:00:00','suborbital rocket','success',740,'','','','','Aeron',636,'',150 from dual union all
select 64836,'1992-S03','1992-01-01 00:00:00','military missile','unknown',646,'','Oval (VMF RV)','','','Reentry Test',621,'',675 from dual union all
select 64837,'1992-S04','1992-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64838,'1992-S05','1992-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64839,'1992-S06','1992-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 6484,'1960-A226','1960-09-22 22:05:00','military missile','failure',928,'A1XE-7 (A1FP-3)','','','','Test',421,'SSBN 599',0 from dual union all
select 64840,'1992-S07','1992-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64841,'1992-S08','1992-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64842,'1992-S09','1992-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 64843,'1992-S10','1992-01-01 00:00:00','military missile','success',1507,'VMF RF','','','','OT',1626,'KALMAR',1000 from dual union all
select 64844,'1992-S100','1992-12-09 00:00:00','suborbital rocket','success',213,'','VERT  (POIVRE)','','','Test/Imaging',304,'',300 from dual union all
select 64845,'1992-S101','1992-12-11 22:45:00','suborbital rocket','success',860,'NASA 31.95UU','CWAS 28','','','Aeronomy',1895,'',140 from dual union all
select 64846,'1992-S102','1992-12-15 03:00:00','suborbital rocket','success',224,'NASA 36.085UG','JHU FOT 11 (HUT Cal)','','','UV Astron',1873,'',302 from dual union all
select 64847,'1992-S103','1992-03-19 16:10:00','suborbital rocket','success',224,'NASA 36.088DE','HIRAAS 2','','','Airglow',1873,'',300 from dual union all
select 64848,'1992-S104','1992-01-01 00:00:00','military missile','success',359,'','Training','','','Test',810,'',500 from dual union all
select 64849,'1992-S105','1992-08-29 05:13:00','military missile','success',941,'9','','','','Test',320,'',100 from dual union all
select 6485,'1960-A227','1960-09-23 01:05:00','military missile','failure',928,'A1XE-8 (A1FP-4)','','','','Test',421,'SSBN 599',0 from dual union all
select 64850,'1992-S106','1992-04-07 10:25:00','suborbital rocket','success',597,'INTA RP-9201','Tech Payload','Rocio-11','','Tech',396,'',100 from dual union all
select 64851,'1992-S107','1992-04-08 12:01:00','suborbital rocket','success',597,'INTA RP-9202','Tech Payload','Rocio-12','','Tech',396,'',100 from dual union all
select 64852,'1992-S11','1992-01-11 03:40:00','suborbital rocket','success',224,'NASA 36.079UG','EUVS','','','EUV Astron',1873,'',248 from dual union all
select 64853,'1992-S12','1992-01-17 00:00:00','military missile','success',1262,'S1','BTTV-1 Validation','','','Test',1892,'',200 from dual union all
select 64854,'1992-S13','1992-01-23 19:19:00','suborbital rocket','success',224,'NASA 36.075UE','','','','Plasma',1873,'',300 from dual union all
select 64855,'1992-S14','1992-01-28 12:00:00','suborbital rocket','success',1073,'S-310-21','','','','Aeron',833,'',223 from dual union all
select 64856,'1992-S15','1992-01-31 11:55:00','suborbital rocket','success',224,'NASA 36.096UH','Colorado SXT','','','XR Astron',1873,'',300 from dual union all
select 64857,'1992-S16','1992-02-01 16:00:00','suborbital rocket','success',1074,'S-520-15','CIR','','','IR Astron',833,'',338 from dual union all
select 64858,'1992-S17','1992-02-18 00:00:00','suborbital rocket','success',117,'LEAP 1','','','','Tech',1873,'',320 from dual union all
select 64859,'1992-S18','1992-02-22 00:00:00','military missile','success',1443,'','CET','','','OT',420,'SSBN 736',1000 from dual union all
select 6486,'1960-A228','1960-09-23 16:10:00','atmospheric rocket','success',835,'Smoke 006','','','','Aeronomy',1821,'',13 from dual union all
select 64860,'1992-S19','1992-02-22 00:00:00','military missile','success',1443,'','CET','','','OT',421,'SSBN 736',1000 from dual union all
select 64861,'1992-S20','1992-02-22 00:00:00','military missile','success',1443,'','CET','','','OT',421,'SSBN 736',1000 from dual union all
select 64862,'1992-S21','1992-02-22 00:00:00','military missile','success',1443,'','CET','','','OT',421,'SSBN 736',1000 from dual union all
select 64863,'1992-S22','1992-02-22 03:15:00','suborbital rocket','success',224,'NASA 36.089UG','','','','UV Astron',1873,'',300 from dual union all
select 64864,'1992-S23','1992-03-03 13:57:30','suborbital rocket','success',867,'NASA 18.221UE','Aria-1','','','Plasma',1047,'',295 from dual union all
select 64865,'1992-S24','1992-03-03 14:06:48','suborbital rocket','success',211,'NASA 27.130CE','Aria-1','','','Plasma',1047,'',289 from dual union all
select 64866,'1992-S25','1992-03-03 14:57:00','suborbital rocket','success',867,'NASA 18.222UE','Aria-1','','','Plasma',1047,'',270 from dual union all
select 64867,'1992-S26','1992-03-04 14:45:00','military missile','success',907,'08PA (85-173)','','GT 08PA','','OT',1700,'',1000 from dual union all
select 64868,'1992-S27','1992-03-06 13:57:30','suborbital rocket','success',867,'NASA 18.223UE','','','','Plasma',1047,'',270 from dual union all
select 64869,'1992-S28','1992-03-12 22:42:00','suborbital rocket','success',860,'NASA 31.83UU','CWAS 21','','','Aeronomy',1895,'',140 from dual union all
select 6487,'1960-A229','1960-09-23 17:52:06','suborbital spaceplane','success',1520,'1-13-25','','','','Test',379,'NB-52 008',16 from dual union all
select 64870,'1992-S29','1992-03-13 18:15:00','military missile','success',795,'','ERIS Target','GBI-2B ERIS Target','','Target',1698,'',1300 from dual union all
select 64871,'1992-S30','1992-03-13 18:36:27','military missile','success',453,'ERIS 2','','','','Interceptor',868,'',290 from dual union all
select 64872,'1992-S31','1992-03-15 22:46:00','suborbital rocket','success',860,'NASA 31.82UU','CWAS 22','','','Aeronomy',1895,'',140 from dual union all
select 64873,'1992-S32','1992-03-19 16:05:00','suborbital rocket','success',224,'CSAR 1','CSAR 1','','','Micrograv',1873,'',226 from dual union all
select 64874,'1992-S33','1992-03-29 07:27:00','suborbital rocket','success',234,'NASA 39.003UE','CHARGE-2B','','','Ionos/Tether',1047,'',267 from dual union all
select 64875,'1992-S34','1992-03-29 08:07:45','suborbital rocket','success',561,'GL A26.780','SPIRIT-2','','','IR Background',1047,'',326 from dual union all
select 64876,'1992-S35','1992-04-08 00:00:00','military missile','unknown',1442,'','FCET-40','','','OT',420,'SSBN 655',1000 from dual union all
select 64877,'1992-S36','1992-04-08 00:00:00','military missile','unknown',1442,'','FCET-40','','','OT',420,'SSBN 655',1000 from dual union all
select 64878,'1992-S37','1992-04-09 10:40:00','suborbital rocket','success',212,'MASER 5','SSC MASER 5','','','Micrograv',416,'',317 from dual union all
select 64879,'1992-S38','1992-04-15 09:09:00','suborbital rocket','success',561,'Firebird 1B','','','','Target/Decoys',1821,'',450 from dual union all
select 6488,'1960-A23','1960-01-30 11:54:00','suborbital rocket','success',725,'','','','','Meteo',1212,'VOKV',42 from dual union all
select 64880,'1992-S39','1992-05-05 00:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT147GM-1','GLORY TRIP 147GM-1','OT',1706,'',1300 from dual union all
select 64881,'1992-S40','1992-05-05 00:00:00','military missile','success',941,'7','','','','Test',320,'',100 from dual union all
select 64882,'1992-S41','1992-05-12 14:26:00','suborbital rocket','success',224,'NASA 36.048CS','SPARTAN SPDE','','','Solar',1873,'',300 from dual union all
select 64883,'1992-S42','1992-05-23 00:00:00','suborbital rocket','success',860,'NASA 31.84UU','CWAS 23','','','Aeronomy',1895,'',140 from dual union all
select 64884,'1992-S43','1992-05-24 00:00:00','suborbital rocket','success',1337,'','CDX (LWIS)','','','Target',848,'',290 from dual union all
select 64885,'1992-S44','1992-05-25 23:52:00','suborbital rocket','success',867,'NASA 18.224UE','EL COQUI AA-3A','','','Plasma',1484,'',292 from dual union all
select 64886,'1992-S45','1992-05-27 12:27:00','suborbital rocket','success',860,'NASA 31.85UU','CWAS 24','','','Aeronomy',1895,'',140 from dual union all
select 64887,'1992-S46','1992-05-30 08:11:00','suborbital rocket','success',224,'NASA 36.65DE','EL COQUI AA-4/IFH','','','Plasma',1484,'',385 from dual union all
select 64888,'1992-S47','1992-06-01 02:52:00','suborbital rocket','success',1219,'XV-24','MULTIFOT','','','Aeron',13,'',282 from dual union all
select 64889,'1992-S48','1992-06-02 11:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT148GB','GLORY TRIP 148GB','OT',1712,'',1300 from dual union all
select 6489,'1960-A230','1960-09-26 00:00:00','military missile','success',1214,'SM-62A N-3452','Cat IIE-8, Sta12','','','OT',224,'',10 from dual union all
select 64890,'1992-S49','1992-06-06 08:37:31','suborbital rocket','success',224,'NASA 36.64CE','EL COQUI AA-3B','','','Plasma',1484,'',368 from dual union all
select 64891,'1992-S50','1992-06-09 05:32:00','suborbital rocket','success',224,'NASA 36.71UE','EL COQUI NC-1','','','Plasma',1484,'',359 from dual union all
select 64892,'1992-S51','1992-06-16 00:00:00','military missile','failure',1262,'S2','BTTV-2 Validation','','','Test',1892,'',200 from dual union all
select 64893,'1992-S52','1992-06-18 00:00:00','military missile','success',1443,'CET','CET','','','OT',420,'SSBN 736',1000 from dual union all
select 64894,'1992-S53','1992-06-18 00:00:00','military missile','success',1443,'CET','CET','','','OT',420,'SSBN 736',1000 from dual union all
select 64895,'1992-S54','1992-06-18 00:00:00','military missile','success',1443,'CET','CET','','','OT',420,'SSBN 736',1000 from dual union all
select 64896,'1992-S55','1992-06-18 00:00:00','military missile','success',1443,'CET','CET','','','OT',420,'SSBN 736',1000 from dual union all
select 64897,'1992-S56','1992-06-19 11:01:00','suborbital rocket','success',117,'LEAP 2','','','','Tech',1873,'',330 from dual union all
select 64898,'1992-S57','1992-06-23 00:24:00','suborbital rocket','success',228,'NASA 21.105GE','EL COQUI NC-2','','','Ionos',1484,'',129 from dual union all
select 64899,'1992-S58','1992-07-01 00:00:00','military missile','success',580,'','','','','Test',1457,'',125 from dual;
insert into LAUNCH
select 649,'1946-A32','1946-05-24 00:00:00','atmospheric rocket','success',1401,'17','','','','Test',1852,'',4 from dual union all
select 6490,'1960-A231','1960-09-28 00:00:00','military missile','success',771,'GM-56-1937','','','','Test',222,'',12 from dual union all
select 64900,'1992-S59','1992-07-01 00:00:00','military missile','success',580,'','','','','Test',1457,'',125 from dual union all
select 64901,'1992-S60','1992-07-01 02:20:00','military missile','success',907,'09PA (84-549)','','GT 09PA','','OT',1697,'',1000 from dual union all
select 64902,'1992-S61','1992-07-02 09:01:15','suborbital rocket','success',224,'NASA 36.82DE','EL COQUI AA-1','','','Plasma',1484,'',431 from dual union all
select 64903,'1992-S62','1992-07-03 08:33:00','suborbital rocket','success',1028,'','','','','Ionos?',1361,'',320 from dual union all
select 64904,'1992-S63','1992-07-04 08:58:00','suborbital rocket','success',224,'NASA 36.83DE','EL COQUI AA-7','','','Plasma',1484,'',421 from dual union all
select 64905,'1992-S64','1992-07-12 09:02:00','suborbital rocket','success',224,'NASA 36.81CE','EL COQUI AA-2','','','Plasma',1484,'',368 from dual union all
select 64906,'1992-S65','1992-07-24 13:36:00','suborbital rocket','success',211,'NASA 27.133UE','','','','Plasma',1047,'',340 from dual union all
select 64907,'1992-S66','1992-07-28 17:40:00','military missile','success',1454,'51165-93-004P','15A35P LKI-4','15Yu70','','OT',508,'',1000 from dual union all
select 64908,'1992-S67','1992-07-29 00:00:00','military missile','success',1443,'','DASO-9','','','OT',419,'SSBN 738',1000 from dual union all
select 64909,'1992-S68','1992-08-04 00:00:00','military missile','success',1443,'CET','CET','','','OT',420,'SSBN 734',1000 from dual union all
select 6491,'1960-A232','1960-09-28 00:00:00','military missile','success',771,'GM-56-1932','','','','Test',222,'',12 from dual union all
select 64910,'1992-S69','1992-08-04 00:00:00','military missile','success',1443,'CET','CET','','','OT',420,'SSBN 734',1000 from dual union all
select 64911,'1992-S70','1992-08-04 00:00:00','military missile','success',1443,'CET','CET','','','OT',420,'SSBN 734',1000 from dual union all
select 64912,'1992-S71','1992-08-04 00:00:00','military missile','success',1443,'CET','CET','','','OT',420,'SSBN 734',1000 from dual union all
select 64913,'1992-S72','1992-08-18 04:00:00','military missile','success',941,'8','','','','Test',320,'',100 from dual union all
select 64914,'1992-S73','1992-08-18 09:20:00','suborbital rocket','success',847,'DLR K-NO2-198','Testflug','','','Test',417,'',92 from dual union all
select 64915,'1992-S74','1992-08-19 23:30:00','suborbital rocket','success',1296,'TR-1A-2','','','','Micrograv',1476,'',290 from dual union all
select 64916,'1992-S75','1992-08-21 17:46:00','suborbital rocket','success',860,'NASA 31.81UU','','','','Aeron',1895,'',140 from dual union all
select 64917,'1992-S76','1992-08-24 16:30:00','suborbital rocket','success',224,'NASA 36.090DS','HRTS 8','','','Solar',1873,'',249 from dual union all
select 64918,'1992-S77','1992-08-26 15:10:00','suborbital rocket','success',860,'NASA 31.93UU','CWAS 25','','','Aeronomy',1895,'',140 from dual union all
select 64919,'1992-S78','1992-08-27 18:30:00','suborbital rocket','success',211,'NASA 12.045WT','','','','Test',1821,'',198 from dual union all
select 6492,'1960-A233','1960-10-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64920,'1992-S79','1992-09-01 01:00:00','suborbital rocket','success',860,'NASA 31.92UU','CWAS 26','','','Aeronomy',1895,'',140 from dual union all
select 64921,'1992-S80','1992-09-03 00:00:00','military missile','success',1443,'CET','CET','','','OT',420,'SSBN 737',1000 from dual union all
select 64922,'1992-S81','1992-09-03 00:00:00','military missile','success',1443,'CET','CET','','','OT',420,'SSBN 737',1000 from dual union all
select 64923,'1992-S82','1992-09-03 00:00:00','military missile','success',1443,'CET','CET','','','OT',420,'SSBN 737',1000 from dual union all
select 64924,'1992-S83','1992-09-03 00:00:00','military missile','success',1443,'CET','CET','','','OT',420,'SSBN 737',1000 from dual union all
select 64925,'1992-S84','1992-09-10 14:30:00','suborbital rocket','failure',213,'Starfire 1/F5','CONSORT 5','','','Micrograv',1879,'',235 from dual union all
select 64926,'1992-S85','1992-09-16 03:15:00','military missile','success',907,'10PA (87-146)','','GT 10PA','','OT',1700,'',1000 from dual union all
select 64927,'1992-S86','1992-09-28 00:00:00','military missile','success',799,'','Mk 12  RV x 3','FOT GT149GB','GLORY TRIP 149GB','OT',1712,'',1300 from dual union all
select 64928,'1992-S87','1992-10-16 00:00:00','suborbital rocket','failure',280,'SPFE 6/Orbus 1','','','','Reentry Test',1784,'',500 from dual union all
select 64929,'1992-S88','1992-10-19 00:00:00','military missile','success',1443,'DASO-10','DASO-10','','','OT',419,'SSBN 738',1000 from dual union all
select 6493,'1960-A234','1960-10-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64930,'1992-S89','1992-10-25 01:06:00','military missile','failure',795,'','AST-DT 2','RSLP AST-DT 2','','Target',1698,'',100 from dual union all
select 64931,'1992-S90','1992-10-27 18:30:00','suborbital rocket','success',224,'NASA 36.098UE','NCAR/CU-3','','','Solar UV',1873,'',298 from dual union all
select 64932,'1992-S91','1992-11-04 02:00:00','military missile','failure',799,'','Mk 12A RV x 3','FOT GT150GM','GLORY TRIP 150GM','OT',1699,'',100 from dual union all
select 64933,'1992-S92','1992-11-08 10:25:00','suborbital rocket','success',772,'MAXUS 1B','SSC MAXUS 1B','SSC MAXUS 1B','','Micrograv',407,'',717 from dual union all
select 64934,'1992-S93','1992-11-10 00:00:00','military missile','success',1443,'CET','','','','OT',420,'SSBN 734',1000 from dual union all
select 64935,'1992-S94','1992-11-10 00:00:00','military missile','success',1443,'CET','','','','OT',420,'SSBN 734',1000 from dual union all
select 64936,'1992-S95','1992-11-10 00:00:00','military missile','success',1443,'CET','','','','OT',420,'SSBN 734',1000 from dual union all
select 64937,'1992-S96','1992-11-10 00:00:00','military missile','success',1443,'CET','','','','OT',420,'SSBN 734',1000 from dual union all
select 64938,'1992-S97','1992-11-22 09:22:00','suborbital rocket','success',1212,'DLR K-GR-199','DLR TEXUS 29','','','Micrograv',416,'',230 from dual union all
select 64939,'1992-S98','1992-12-06 16:00:00','suborbital rocket','success',860,'NASA 31.94UU','CWAS 27','','','Aeronomy',1895,'',140 from dual union all
select 6494,'1960-A235','1960-10-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64940,'1992-S99','1992-12-09 00:00:00','military missile','success',1548,'','Efir','','','Tech',1062,'PLBR',620 from dual union all
select 64941,'1992-W01','1992-01-02 20:08:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 64942,'1992-W02','1992-01-15 02:00:00','suborbital rocket','success',761,'MT135P-R806','','','','Meteo',1332,'',57 from dual union all
select 64943,'1992-W03','1992-01-21 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 64944,'1992-W04','1992-01-22 02:00:00','suborbital rocket','success',761,'MT135P-R807','','','','Meteo',1332,'',58 from dual union all
select 64945,'1992-W05','1992-01-22 23:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',74 from dual union all
select 64946,'1992-W06','1992-02-05 02:03:00','suborbital rocket','success',761,'MT135P-R808','','','','Meteo',1332,'',54 from dual union all
select 64947,'1992-W07','1992-02-05 18:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',57 from dual union all
select 64948,'1992-W08','1992-02-07 18:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 64949,'1992-W09','1992-02-11 19:38:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',66 from dual union all
select 6495,'1960-A236','1960-10-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64950,'1992-W10','1992-02-13 22:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',68 from dual union all
select 64951,'1992-W100','1992-10-19 21:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',64 from dual union all
select 64952,'1992-W101','1992-10-21 13:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 64953,'1992-W102','1992-10-28 02:00:00','suborbital rocket','success',761,'MT135P-R833','','','','Meteo',1332,'',54 from dual union all
select 64954,'1992-W103','1992-11-05 02:00:00','suborbital rocket','success',761,'MT135P-R834','','','','Meteo',1332,'',55 from dual union all
select 64955,'1992-W104','1992-11-05 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 64956,'1992-W105','1992-11-10 19:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',68 from dual union all
select 64957,'1992-W106','1992-11-18 02:00:00','suborbital rocket','success',761,'MT135P-R835','','','','Meteo',1332,'',53 from dual union all
select 64958,'1992-W107','1992-11-20 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',54 from dual union all
select 64959,'1992-W108','1992-11-25 02:00:00','suborbital rocket','success',761,'MT135P-R836','','','','Meteo',1332,'',54 from dual union all
select 6496,'1960-A237','1960-10-01 00:00:00','atmospheric rocket','success',546,'','E48 Stability Test','','','Reentry Test',1821,'',10 from dual union all
select 64960,'1992-W109','1992-12-01 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 64961,'1992-W11','1992-02-14 07:50:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',70 from dual union all
select 64962,'1992-W110','1992-12-02 02:20:00','suborbital rocket','success',761,'MT135P-R837','','','','Meteo',1332,'',52 from dual union all
select 64963,'1992-W111','1992-12-16 02:09:00','suborbital rocket','success',761,'MT135P-R838','','','','Meteo',1332,'',56 from dual union all
select 64964,'1992-W112','1992-12-18 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',59 from dual union all
select 64965,'1992-W113','1992-12-21 15:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',53 from dual union all
select 64966,'1992-W114','1992-12-31 02:03:00','suborbital rocket','success',761,'MT135P-R840','','','','Meteo',1332,'',54 from dual union all
select 64967,'1992-W115','1992-01-09 02:00:00','suborbital rocket','success',761,'MT135P-R805','','','','Meteo',1332,'',60 from dual union all
select 64968,'1992-W116','1992-03-11 02:00:00','suborbital rocket','success',761,'MT135P-R811','','','','Meteo',1332,'',60 from dual union all
select 64969,'1992-W117','1992-04-08 02:00:00','suborbital rocket','success',761,'MT135P-R814','','','','Meteo',1332,'',60 from dual union all
select 6497,'1960-A238','1960-10-01 00:00:00','military missile','failure',976,'','8K63 UBP-82','','','Test',636,'',0 from dual union all
select 64970,'1992-W118','1992-05-13 02:00:00','suborbital rocket','success',761,'MT135P-R818','','','','Meteo',1332,'',60 from dual union all
select 64971,'1992-W119','1992-05-20 02:00:00','suborbital rocket','success',761,'MT135P-R819','','','','Meteo',1332,'',60 from dual union all
select 64972,'1992-W12','1992-02-14 18:17:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',68 from dual union all
select 64973,'1992-W120','1992-09-02 02:00:00','suborbital rocket','success',761,'MT135P-R828','','','','Meteo',1332,'',60 from dual union all
select 64974,'1992-W121','1992-10-22 02:00:00','suborbital rocket','success',761,'MT135P-R832','','','','Meteo',1332,'',60 from dual union all
select 64975,'1992-W122','1992-12-23 02:00:00','suborbital rocket','success',761,'MT135P-R839','','','','Meteo',1332,'',60 from dual union all
select 64976,'1992-W123','1992-09-23 02:00:00','suborbital rocket','success',761,'MT135P-R830','','','','Meteo',1332,'',60 from dual union all
select 64977,'1992-W13','1992-02-17 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 64978,'1992-W14','1992-02-19 02:11:00','suborbital rocket','success',761,'MT135P-R809','','','','Meteo',1332,'',56 from dual union all
select 64979,'1992-W15','1992-02-19 07:45:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',73 from dual union all
select 6498,'1960-A239','1960-10-07 00:00:00','military missile','success',764,'58-1427    (22)','','','','Test',240,'',12 from dual union all
select 64980,'1992-W16','1992-02-21 07:30:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',77 from dual union all
select 64981,'1992-W17','1992-02-21 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 64982,'1992-W18','1992-02-22 04:44:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',68 from dual union all
select 64983,'1992-W19','1992-02-26 02:08:00','suborbital rocket','success',761,'MT135P-R810','','','','Meteo',1332,'',56 from dual union all
select 64984,'1992-W20','1992-02-26 07:30:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',55 from dual union all
select 64985,'1992-W21','1992-02-26 08:50:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',54 from dual union all
select 64986,'1992-W22','1992-02-28 07:31:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',70 from dual union all
select 64987,'1992-W23','1992-03-06 08:24:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',79 from dual union all
select 64988,'1992-W24','1992-03-11 07:32:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',76 from dual union all
select 64989,'1992-W25','1992-03-11 08:27:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',74 from dual;
insert into LAUNCH
select 6499,'1960-A24','1960-02-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 64990,'1992-W26','1992-03-11 15:34:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 64991,'1992-W27','1992-03-12 09:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',70 from dual union all
select 64992,'1992-W28','1992-03-18 02:00:00','suborbital rocket','success',761,'MT135P-R812','','','','Meteo',1332,'',54 from dual union all
select 64993,'1992-W29','1992-03-18 23:25:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',70 from dual union all
select 64994,'1992-W30','1992-03-20 08:58:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',75 from dual union all
select 64995,'1992-W31','1992-03-21 18:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 64996,'1992-W32','1992-03-25 02:00:00','suborbital rocket','success',761,'MT135P-R813','','','','Meteo',1332,'',51 from dual union all
select 64997,'1992-W33','1992-03-25 07:32:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',51 from dual union all
select 64998,'1992-W34','1992-03-25 08:25:00','suborbital rocket','failure',738,'','','','','Aeron',1917,'',51 from dual union all
select 64999,'1992-W35','1992-03-27 07:33:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',73 from dual union all
select 65,'1944-A29','1944-12-19 15:20:00','military missile','failure',1470,'19828','Ma315','','','Test',661,'',30 from dual union all
select 650,'1946-A33','1946-05-26 00:00:00','atmospheric rocket','success',1401,'18','','','','Test',1852,'',4 from dual union all
select 6500,'1960-A240','1960-10-11 00:00:00','atmospheric rocket','success',545,'','Glide Rocket','','','Reentry Test',1821,'',10 from dual union all
select 65000,'1992-W36','1992-04-01 06:33:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',52 from dual union all
select 65001,'1992-W37','1992-04-01 22:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',63 from dual union all
select 65002,'1992-W38','1992-04-02 18:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 65003,'1992-W39','1992-04-03 07:20:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',75 from dual union all
select 65004,'1992-W40','1992-04-06 14:07:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',54 from dual union all
select 65005,'1992-W41','1992-04-08 03:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',65 from dual union all
select 65006,'1992-W42','1992-04-08 18:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 65007,'1992-W43','1992-04-10 06:30:00','suborbital rocket','success',738,'','','','','Aeron',1917,'',67 from dual union all
select 65008,'1992-W44','1992-04-15 02:04:59','suborbital rocket','success',761,'MT135P-R815','','','','Meteo',1332,'',58 from dual union all
select 65009,'1992-W45','1992-04-22 02:02:00','suborbital rocket','success',761,'MT135P-R816','','','','Meteo',1332,'',58 from dual union all
select 6501,'1960-A241','1960-10-11 00:00:00','suborbital spaceplane','failure',1520,'1- A-26','','','','Test',379,'NB-52 008',14 from dual union all
select 65010,'1992-W46','1992-04-23 00:00:00','suborbital rocket','success',1502,'Aerosole','','','','Aeron',29,'',75 from dual union all
select 65011,'1992-W47','1992-04-30 02:00:00','suborbital rocket','success',761,'MT135P-R817','','','','Meteo',1332,'',56 from dual union all
select 65012,'1992-W48','1992-05-06 01:56:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',72 from dual union all
select 65013,'1992-W49','1992-05-06 17:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',72 from dual union all
select 65014,'1992-W50','1992-05-12 13:27:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 65015,'1992-W51','1992-05-19 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 65016,'1992-W52','1992-05-27 02:00:00','suborbital rocket','success',761,'MT135P-R820','','','','Meteo',1332,'',59 from dual union all
select 65017,'1992-W53','1992-05-28 14:08:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 65018,'1992-W54','1992-06-03 00:00:00','suborbital rocket','success',1502,'Aerosole','','','','Aeron',29,'',75 from dual union all
select 65019,'1992-W55','1992-06-05 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 6502,'1960-A242','1960-10-12 00:00:00','military missile','success',771,'GM-56-1940','','','','Test',222,'',12 from dual union all
select 65020,'1992-W56','1992-06-08 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',55 from dual union all
select 65021,'1992-W57','1992-06-10 02:00:00','suborbital rocket','success',761,'MT135P-R821','','','','Meteo',1332,'',58 from dual union all
select 65022,'1992-W58','1992-06-17 02:10:00','suborbital rocket','success',761,'MT135P-R822','','','','Meteo',1332,'',53 from dual union all
select 65023,'1992-W59','1992-06-17 09:32:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',69 from dual union all
select 65024,'1992-W60','1992-06-18 20:34:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',68 from dual union all
select 65025,'1992-W61','1992-06-19 15:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 65026,'1992-W62','1992-06-23 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 65027,'1992-W63','1992-06-24 02:00:00','suborbital rocket','success',761,'MT135P-R823','','','','Meteo',1332,'',55 from dual union all
select 65028,'1992-W64','1992-06-24 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 65029,'1992-W65','1992-06-26 18:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 6503,'1960-A243','1960-10-13 00:00:00','suborbital spaceplane','failure',1520,'2-A-19','','','','Test',379,'NB-52',14 from dual union all
select 65030,'1992-W66','1992-06-29 14:03:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 65031,'1992-W67','1992-06-30 09:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',51 from dual union all
select 65032,'1992-W68','1992-07-02 00:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',71 from dual union all
select 65033,'1992-W69','1992-07-02 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 65034,'1992-W70','1992-07-02 18:27:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',57 from dual union all
select 65035,'1992-W71','1992-07-06 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual union all
select 65036,'1992-W72','1992-07-06 17:33:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1057,'',64 from dual union all
select 65037,'1992-W73','1992-07-09 02:00:00','suborbital rocket','success',761,'MT135P-R824','','','','Meteo',1332,'',58 from dual union all
select 65038,'1992-W74','1992-07-15 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',63 from dual union all
select 65039,'1992-W75','1992-07-15 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',61 from dual union all
select 6504,'1960-A244','1960-10-13 04:53:49','military missile','failure',154,'81D','Mk 3-3','IOC DASO','DIAMOND JUBILEE','Test',1665,'',20 from dual union all
select 65040,'1992-W76','1992-07-22 02:00:00','suborbital rocket','success',761,'MT135P-R825','','','','Meteo',1332,'',58 from dual union all
select 65041,'1992-W77','1992-07-24 13:01:20','suborbital rocket','success',1502,'','','','','Aeron',1047,'',75 from dual union all
select 65042,'1992-W78','1992-07-25 12:46:00','suborbital rocket','success',1502,'','','','','Aeron',1047,'',75 from dual union all
select 65043,'1992-W79','1992-07-28 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',69 from dual union all
select 65044,'1992-W80','1992-07-29 22:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',51 from dual union all
select 65045,'1992-W81','1992-07-30 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 65046,'1992-W82','1992-08-04 21:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',66 from dual union all
select 65047,'1992-W83','1992-08-05 02:00:00','suborbital rocket','success',761,'MT135P-R826','','','','Meteo',1332,'',55 from dual union all
select 65048,'1992-W84','1992-08-07 17:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',855,'',67 from dual union all
select 65049,'1992-W85','1992-08-19 02:30:00','suborbital rocket','success',761,'MT135P-R827','','','','Meteo',1332,'',55 from dual union all
select 6505,'1960-A245','1960-10-14 00:00:00','military missile','success',1214,'SM-62A N-3454','Cat IIE-9','','','OT',224,'',10 from dual union all
select 65050,'1992-W86','1992-08-19 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 65051,'1992-W87','1992-08-30 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 65052,'1992-W88','1992-09-03 20:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',69 from dual union all
select 65053,'1992-W89','1992-09-08 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 65054,'1992-W90','1992-09-09 02:00:00','suborbital rocket','success',761,'MT135P-R829','','','','Meteo',1332,'',55 from dual union all
select 65055,'1992-W91','1992-09-09 02:00:00','suborbital rocket','success',759,'MT-135-57','NNP-O3','','','Aeron',839,'',57 from dual union all
select 65056,'1992-W92','1992-09-11 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 65057,'1992-W94','1992-09-24 14:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 65058,'1992-W95','1992-10-07 02:10:00','suborbital rocket','success',761,'MT135P-R831','','','','Meteo',1332,'',57 from dual union all
select 65059,'1992-W96','1992-10-09 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 6506,'1960-A246','1960-10-19 00:00:00','atmospheric rocket','success',546,'','E48 Stability Test','','','Reentry Test',1821,'',10 from dual union all
select 65060,'1992-W97','1992-10-14 00:00:00','suborbital rocket','success',1284,'','Student Launch Prog','','','Education',295,'',75 from dual union all
select 65061,'1992-W98','1992-10-14 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',58 from dual union all
select 65062,'1992-W99','1992-10-16 13:42:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',66 from dual union all
select 65063,'1992-Y01','1992-01-22 22:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',873,'',86 from dual union all
select 65064,'1992-Y02','1992-02-11 19:14:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',95 from dual union all
select 65065,'1992-Y03','1992-02-13 23:35:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',873,'',89 from dual union all
select 65066,'1992-Y04','1992-02-14 17:55:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 65067,'1992-Y05','1992-02-26 23:20:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',873,'',90 from dual union all
select 65068,'1992-Y06','1992-03-17 13:49:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65069,'1992-Y07','1992-03-18 13:54:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',95 from dual union all
select 6507,'1960-A247','1960-10-20 17:30:27','suborbital spaceplane','success',1520,'1-14-27','Stability/Control','','','Test',379,'NB-52 008',16 from dual union all
select 65070,'1992-Y08','1992-03-18 21:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',873,'',90 from dual union all
select 65071,'1992-Y09','1992-03-19 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65072,'1992-Y10','1992-03-25 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',92 from dual union all
select 65073,'1992-Y11','1992-03-30 14:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65074,'1992-Y12','1992-04-01 21:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',873,'',85 from dual union all
select 65075,'1992-Y13','1992-04-08 21:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65076,'1992-Y14','1992-04-15 21:36:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65077,'1992-Y15','1992-04-16 14:03:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',83 from dual union all
select 65078,'1992-Y16','1992-04-22 19:02:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual;
insert into LAUNCH
select 65079,'1992-Y17','1992-04-23 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',84 from dual union all
select 6508,'1960-A248','1960-10-21 00:00:00','military missile','success',764,'59-4976   (23)','','','','Test',240,'',12 from dual union all
select 65080,'1992-Y18','1992-04-28 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 65081,'1992-Y19','1992-04-28 17:40:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65082,'1992-Y20','1992-05-06 01:32:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',873,'',90 from dual union all
select 65083,'1992-Y21','1992-06-02 13:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65084,'1992-Y22','1992-06-17 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 65085,'1992-Y23','1992-06-29 21:15:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 65086,'1992-Y24','1992-06-30 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',82 from dual union all
select 65087,'1992-Y25','1992-07-10 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 65088,'1992-Y26','1992-07-22 13:40:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65089,'1992-Y27','1992-07-28 19:12:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 6509,'1960-A249','1960-10-24 00:00:00','atmospheric rocket','failure',274,'NOTS EV-II','TV-2','','','Test',1023,'F4D-1 747',20 from dual union all
select 65090,'1992-Y28','1992-08-05 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',84 from dual union all
select 65091,'1992-Y29','1992-08-06 14:49:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',92 from dual union all
select 65092,'1992-Y30','1992-08-10 18:20:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65093,'1992-Y31','1992-08-12 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',86 from dual union all
select 65094,'1992-Y32','1992-08-19 15:44:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 65095,'1992-Y33','1992-08-22 00:00:00','suborbital rocket','success',782,'','(Viper 3)','','','Aeron',339,'',82 from dual union all
select 65096,'1992-Y34','1992-08-24 14:09:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 65097,'1992-Y35','1992-10-07 15:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',96 from dual union all
select 65098,'1992-Y36','1992-10-20 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',86 from dual union all
select 65099,'1992-Y37','1992-11-06 13:09:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 651,'1946-A34','1946-05-26 00:00:00','atmospheric rocket','success',1401,'19','','','','Test',1852,'',4 from dual union all
select 6510,'1960-A25','1960-02-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 65100,'1992-Y38','1992-12-01 00:00:00','suborbital rocket','success',737,'','','','','Test',636,'',85 from dual union all
select 65101,'1992-Y39','1992-12-01 00:00:00','suborbital rocket','success',737,'','','','','Test',636,'',85 from dual union all
select 65102,'1992-Y40','1992-12-16 16:09:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 65103,'1993-001','1993-01-12 11:10:17','orbital','success',680,'53778-426','Kosmos-2230','Tsikada','','Sat',460,'',null from dual union all
select 65104,'1993-002','1993-01-13 01:49:00','orbital','success',805,'78012  -643','Molniya-1','Molniya-1T!No. 78','','Sat',475,'',null from dual union all
select 65105,'1993-003','1993-01-13 13:59:30','orbital','success',1243,'STS-54','TDRS-F','OV-105 IUS deploy','','Sat',922,'MLP2',null from dual union all
select 65106,'1993-004','1993-01-19 14:49:01','orbital','success',1238,'77017  -979','Kosmos-2231','Kobal''t','','Sat',474,'',null from dual union all
select 65107,'1993-005','1993-01-24 05:58:05','orbital','success',1242,'U 15000-060','Soyuz TM-16','Soyuz 7K-STM No. 101','','Sat',480,'',null from dual union all
select 65108,'1993-006','1993-01-26 15:55:26','orbital','success',803,'78023  -609','Kosmos-2232','Oko','','Sat',462,'',null from dual union all
select 65109,'1993-007','1993-02-03 02:55:00','orbital','success',422,'D218','USA 88 (GPS IIA-18)','Navstar GPS SVN 22','','Sat',233,'',null from dual union all
select 6511,'1960-A251','1960-10-26 00:00:00','military missile','success',771,'GM-56-1935','','','','Test',222,'',12 from dual union all
select 65110,'1993-008','1993-02-09 02:56:56','orbital','success',680,'47178-428','Kosmos-2233','Parus','','Sat',460,'',null from dual union all
select 65111,'1993-009','1993-02-09 14:30:34','orbital','success',909,'003/F3','SCD-1','Orbcomm OXP','','Sat',923,'NB-52 008',null from dual union all
select 65112,'1993-010','1993-02-17 20:09:47','orbital','success',958,'362-01','Kosmos-2234/2236','Uragan No. 73L','','Sat',587,'',null from dual union all
select 65113,'1993-011','1993-02-20 02:20:00','orbital','success',820,'M-3S2-7','Asuka','ASTRO-D','','Sat',837,'',null from dual union all
select 65114,'1993-012','1993-02-21 18:32:33','orbital','success',1242,'U 15000-068','Progress M-16','Progress 7K-TGM No. 216','','Sat',480,'',null from dual union all
select 65115,'1993-013','1993-03-25 02:28:00','orbital','success',958,'358-01','Raduga','Gran'' No. 42L','','Sat',587,'',null from dual union all
select 65116,'1993-014','1993-03-25 13:15:27','orbital','success',1260,'','EKA','EKA-1','','Sat',605,'',null from dual union all
select 65117,'1993-015','1993-03-25 21:38:02','orbital','success',171,'AC-74','UHF 1','UFO F1','','Sat',269,'',null from dual union all
select 65118,'1993-016','1993-03-26 02:21:00','orbital','success',1537,'45065-614','Kosmos-2237','Tselina-2','','Sat',567,'',null from dual union all
select 65119,'1993-017','1993-03-30 03:09:00','orbital','success',422,'D219','USA 90 (GPS IIA-19)','Navstar GPS SVN 31','','Sat',233,'',null from dual union all
select 6512,'1960-A252','1960-10-26 00:00:00','military missile','success',771,'GM-56-1939','','','','Test',222,'',12 from dual union all
select 65120,'1993-018','1993-03-30 12:00:00','orbital','success',1446,'67042-303','Kosmos-2238','US-PM','','Sat',591,'',null from dual union all
select 65121,'1993-019','1993-03-31 03:34:13','orbital','success',1242,'N 15000-069','Progress M-17','Progress 7K-TGM No. 217','','Sat',480,'',null from dual union all
select 65122,'1993-020','1993-04-01 18:57:26','orbital','success',680,'47178-431','Kosmos-2239','Parus','','Sat',460,'',null from dual union all
select 65123,'1993-021','1993-04-02 14:30:01','orbital','success',1238,'78012  -991','Kosmos-2240','Kobal''t','','Sat',462,'',null from dual union all
select 65124,'1993-022','1993-04-06 19:07:27','orbital','success',803,'78012  -644','Kosmos-2241','Oko','','Sat',475,'',null from dual union all
select 65125,'1993-023','1993-04-08 05:29:00','orbital','success',1243,'STS-56','Spacelab Atlas 2','OV-103 Spacelab','','Sat',922,'MLP1',null from dual union all
select 65126,'1993-024','1993-04-16 07:49:00','orbital','success',1448,'51027-908','Kosmos-2242','Tselina-R','','Sat',469,'',null from dual union all
select 65127,'1993-025','1993-04-21 00:23:00','orbital','success',805,'76032  -646','Molniya-3','Molniya-3 No. 57','','Sat',475,'',null from dual union all
select 65128,'1993-026','1993-04-25 13:56:00','orbital','success',909,'004/F4','Alexis','Alexis','','Sat',379,'NB-52 008',null from dual union all
select 65129,'1993-027','1993-04-26 14:50:00','orbital','success',1243,'STS-55','Spacelab D-2','OV-102 Spacelab','','Sat',921,'MLP3',null from dual union all
select 6513,'1960-A253','1960-10-28 17:43:56','suborbital spaceplane','success',1520,'1-15-28','Stability/Control','','','Test',379,'NB-52 008',15 from dual union all
select 65130,'1993-028','1993-04-27 10:35:00','orbital','success',1238,'76047  -737','Kosmos-2243','Kometa No. 16','','Sat',562,'',null from dual union all
select 65131,'1993-029','1993-04-28 03:39:20','orbital','success',1446,'45042-304','Kosmos-2244','US-PM','','Sat',591,'',null from dual union all
select 65132,'1993-030','1993-05-11 14:56:01','orbital','success',1448,'51013-301','Kosmos-2245/2250','Strela-3','','Sat',469,'',null from dual union all
select 65133,'1993-031','1993-05-12 00:56:32','orbital','success',97,'V56 L428c','Astra 1C/Arsene','Astra 1C','','Sat',345,'',null from dual union all
select 65134,'1993-032','1993-05-13 00:07:00','orbital','success',422,'D220','USA 91 (GPS IIA-20)','Navstar GPS SVN 37','','Sat',233,'',null from dual union all
select 65135,'1993-033','1993-05-21 09:15:01','orbital','success',1238,'78012  -830','Resurs-F','Resurs-F2 17F42 No. 9','','Sat',462,'',null from dual union all
select 65136,'1993-034','1993-05-22 06:41:47','orbital','success',1242,'U 15000-063 (Kero)','Progress M-18','Progress 7K-TGM No. 218','','Sat',480,'',null from dual union all
select 65137,'1993-035','1993-05-26 03:23:59','orbital','success',805,'77044  -660','Molniya-1','Molniya-1T No. 81','','Sat',475,'',null from dual union all
select 65138,'1993-036','1993-06-16 04:17:00','orbital','success',680,'47135-601','Kosmos-2251','Strela-2M','','Sat',456,'',null from dual union all
select 65139,'1993-037','1993-06-21 13:07:22','orbital','success',1243,'STS-57','SH-01/SHOOT','OV-105 Research/Rendez','','Sat',922,'MLP2',null from dual union all
select 6514,'1960-A254','1960-11-01 00:00:00','suborbital rocket','failure',88,'','','','','Aeron',428,'',0 from dual union all
select 65140,'1993-038','1993-06-24 04:12:41','orbital','success',1448,'45082-801','Kosmos-2252/2257','Strela-3','','Sat',469,'',null from dual union all
select 65141,'1993-039','1993-06-25 00:18:00','orbital','success',101,'V57 L430b','Galaxy 4H','Galaxy IVH','','Sat',345,'',null from dual union all
select 65142,'1993-040','1993-06-25 08:20:00','orbital','success',1241,'76024  -156','Resurs-F','Resurs-F1 14F43 No. 57','','Sat',462,'',null from dual union all
select 65143,'1993-041','1993-06-25 23:30:00','orbital','success',1135,'S217C','RADCAL','RADCAL','','Sat',1779,'',null from dual union all
select 65144,'1993-042','1993-06-26 13:27:00','orbital','success',422,'D221','USA 92 (GPS IIA-21)','Navstar GPS SVN 39','','Sat',233,'',null from dual union all
select 65145,'1993-043','1993-07-01 14:32:58','orbital','success',1242,'N 15000-070','Soyuz TM-17','Soyuz 7K-STM No. 66','','Sat',480,'',null from dual union all
select 65146,'1993-044','1993-07-07 07:15:00','orbital','success',1446,'51032-305','Kosmos-2258','US-PM','','Sat',591,'',null from dual union all
select 65147,'1993-045','1993-07-14 16:40:00','orbital','success',1241,'76024  -994','Kosmos-2259','Kobal''t','','Sat',474,'',null from dual union all
select 65148,'1993-046','1993-07-19 22:04:02','orbital','success',172,'AC-104','USA 93 (DSCS 3B-9)','DSCS III B-9','','Sat',268,'',null from dual union all
select 65149,'1993-047','1993-07-22 08:45:00','orbital','success',1241,'76024  -269','Kosmos-2260','Resurs-T','','Sat',474,'',null from dual union all
select 6515,'1960-A255','1960-11-02 00:00:00','military missile','success',771,'GM-55-40','','','','Test',222,'',12 from dual union all
select 65150,'1993-048','1993-07-22 22:58:55','orbital','success',104,'V58 L429c','Hisp. 1B/Insat 2B','Hispasat 1B','','Sat',345,'',null from dual union all
select 65151,'1993-049','1993-08-04 00:52:00','orbital','success',805,'76024  -659','Molniya-3','Molniya-3 No. 58','','Sat',474,'',null from dual union all
select 65152,'1993-050','1993-08-09 10:02:00','orbital','success',155,'34E','NOAA 13','NOAA I','','Sat',1776,'',null from dual union all
select 65153,'1993-051','1993-08-10 14:53:45','orbital','success',803,'78023  -610','Kosmos-2261','Oko','','Sat',462,'',null from dual union all
select 65154,'1993-052','1993-08-10 22:23:45','orbital','success',1238,'N 15000-634','Progress M-19','Progress 7K-TGM No. 219','','Sat',480,'',null from dual union all
select 65155,'1993-053','1993-08-24 10:45:00','orbital','success',1241,'78012  -155','Resurs-F','Resurs-F1 14F43 No. 56','','Sat',462,'',null from dual union all
select 65156,'1993-054','1993-08-30 12:38:00','orbital','success',422,'D222','USA 94 (GPS IIA-22)','Navstar GPS SVN 35','','Sat',234,'',null from dual union all
select 65157,'1993-055','1993-08-31 04:40:00','orbital','success',1448,'45092-804','Meteor-2','Meteor-2 No. 24','','Sat',469,'',null from dual union all
select 65158,'1993-056','1993-09-03 11:17:02','orbital','success',171,'AC-75','USA 95 (UHF F2)','UFO F2','','Sat',269,'',null from dual union all
select 65159,'1993-057','1993-09-07 13:25:00','orbital','success',1242,'77044  -356','Kosmos-2262','Don','','Sat',562,'',null from dual union all
select 6516,'1960-A256','1960-11-04 00:00:00','suborbital spaceplane','failure',1520,'2-A-20','','','','Test',379,'NB-52',14 from dual union all
select 65160,'1993-058','1993-09-12 11:45:00','orbital','success',1243,'STS-51','ACTS/Orfeus','OV-103 IUS deploy/FF','','Sat',922,'MLP3',null from dual union all
select 65161,'1993-059','1993-09-16 07:36:19','orbital','success',1537,'67019-218','Kosmos-2263','Tselina-2','','Sat',567,'',null from dual union all
select 65162,'1993-060','1993-09-17 00:43:10','orbital','success',1446,'67057-901','Kosmos-2264','US-PM','','Sat',591,'',null from dual union all
select 65163,'1993-061','1993-09-26 01:45:00','orbital','success',94,'V59 L431','SPOT 3/ASAP 4/Stella','SPOT 3','','Sat',345,'',null from dual union all
select 65164,'1993-062','1993-09-30 17:05:59','orbital','success',958,'359-01','Raduga','Gran'' No. 41L','','Sat',587,'',null from dual union all
select 65165,'1993-063','1993-10-08 08:00:00','orbital','success',301,'Y11      (31)','FSW-1','FSW-1 No. 5 (14)','','Sat',810,'',null from dual union all
select 65166,'1993-064','1993-10-11 21:33:19','orbital','success',1238,'77044  -270','Progress M-20','Progress 7K-TGM No. 220','','Sat',480,'',null from dual union all
select 65167,'1993-065','1993-10-18 14:53:10','orbital','success',1243,'STS-58','Spacelab SLS 2','OV-102 Spacelab','','Sat',922,'MLP1',null from dual union all
select 65168,'1993-066','1993-10-22 06:46:00','orbital','success',106,'V60 L425a','Intelsat 701','INTELSAT 701','','Sat',345,'',null from dual;
insert into LAUNCH
select 65169,'1993-067','1993-10-26 13:00:04','orbital','success',680,'65065-624','Kosmos-2265','Taifun-1B','','Sat',456,'',null from dual union all
select 6517,'1960-A257','1960-11-04 20:43:33','suborbital spaceplane','success',1520,'1-16-29','Stability/Control','','','Test',379,'NB-52 008',14 from dual union all
select 65170,'1993-068','1993-10-26 17:04:00','orbital','success',422,'D223','USA 96 (GPS IIA-23)','Navstar GPS SVN 34','','Sat',234,'',null from dual union all
select 65171,'1993-069','1993-10-28 15:17:00','orbital','success',958,'368-01','Gorizont','Gorizont No. 40L','','Sat',587,'',null from dual union all
select 65172,'1993-070','1993-11-02 12:10:09','orbital','success',680,'53735-608','Kosmos-2266','Parus','','Sat',456,'',null from dual union all
select 65173,'1993-071','1993-11-05 08:25:00','orbital','success',1238,'76024  -496','Kosmos-2267','Neman','','Sat',480,'',null from dual union all
select 65174,'1993-072','1993-11-18 13:54:59','orbital','success',958,'367-01','Gorizont','Gorizont No. 41L','','Sat',587,'',null from dual union all
select 65175,'1993-073','1993-11-20 01:17:00','orbital','success',107,'V61 L433a','Solid. 1/Meteosat 6','Solidaridad 1','','Sat',345,'',null from dual union all
select 65176,'1993-074','1993-11-28 23:40:02','orbital','success',172,'AC-106','USA 97 (DSCS 3B-10)','DSCS III B-10','','Sat',268,'',null from dual union all
select 65177,'1993-075','1993-12-02 09:27:00','orbital','success',1243,'STS-61','HST SM-1','OV-105 HST mission','','Sat',922,'MLP2',null from dual union all
select 65178,'1993-076','1993-12-08 00:48:00','orbital','success',422,'D224','USA 98 (NATO 4B)','NATO 4B','','Sat',233,'',null from dual union all
select 65179,'1993-077','1993-12-16 00:40:02','orbital','success',174,'AC-108','Telstar 401','Telstar 401','','Sat',269,'',null from dual union all
select 6518,'1960-A258','1960-11-08 15:18:23','test rocket','failure',708,'LJ-5','','','','SC Test',1797,'',16 from dual union all
select 65180,'1993-078','1993-12-18 01:27:00','orbital','success',104,'V62 L434a','DBS 1/Thaicom 1','DBS 1','','Sat',345,'',null from dual union all
select 65181,'1993-079','1993-12-22 20:37:16','orbital','success',805,'78024  -658','Molniya-1T','Molniya-1T','','Sat',474,'',null from dual union all
select 65182,'1993-A01','1993-01-01 00:00:00','military missile','failure',354,'','','','','Test',1494,'',0 from dual union all
select 65183,'1993-A02','1993-01-27 10:43:41','suborbital rocket','failure',237,'NASA 40.003UE','PHAZE','','','Ionos',1047,'',10 from dual union all
select 65184,'1993-A03','1993-02-11 00:00:00','suborbital rocket','failure',561,'TCMP 1A','','','','Reentry Test',1784,'',2 from dual union all
select 65185,'1993-A04','1993-02-28 00:00:00','military missile','success',118,'','','','','Test',1024,'SHIP-ISR',20 from dual union all
select 65186,'1993-A05','1993-02-28 00:00:00','military missile','success',118,'','','','','Test',1024,'SHIP-ISR',20 from dual union all
select 65187,'1993-A06','1993-06-04 00:00:00','suborbital rocket','success',649,'KSR-420S-1','','','','Aeronomy',33,'',39 from dual union all
select 65188,'1993-A07','1993-06-15 17:30:00','military missile','failure',795,'','TDT-1','RSLP TDT-1','','Target',1698,'',1 from dual union all
select 65189,'1993-A08','1993-07-14 00:00:00','military missile','success',118,'','','','','Test',1024,'SHIP-ISR',20 from dual union all
select 6519,'1960-A259','1960-11-09 00:00:00','military missile','success',771,'GM-56-1938','','','','Test',222,'',12 from dual union all
select 65190,'1993-A09','1993-08-18 22:43:56','test rocket','success',345,'','','','','Test',1895,'',0 from dual union all
select 65191,'1993-A10','1993-08-28 09:45:00','suborbital rocket','failure',117,'NASA 24.17CH','SXT/XOGS','','','XR Astron',1873,'',8 from dual union all
select 65192,'1993-A11','1993-09-01 00:00:00','suborbital rocket','success',649,'KSR-420S-2','','','','Aeronomy',33,'',49 from dual union all
select 65193,'1993-A12','1993-09-11 17:12:03','test rocket','success',345,'','','','','Test',1895,'',0 from dual union all
select 65194,'1993-A13','1993-09-27 00:00:00','suborbital rocket','success',1335,'Rockwell LEAP','FTV-2 LEAP Mockup','','','Interceptor',1057,'CG-29',30 from dual union all
select 65195,'1993-A14','1993-09-30 16:28:00','test rocket','success',345,'','','','','Test',1895,'',0 from dual union all
select 65196,'1993-A15','1993-10-14 00:00:00','military missile','success',118,'','','','','Test',1024,'SHIP-ISR',20 from dual union all
select 65197,'1993-A16','1993-10-14 00:00:00','military missile','success',118,'','','','','Test',1024,'SHIP-ISR',20 from dual union all
select 65198,'1993-A17','1993-01-01 00:00:00','military missile','success',3,'','Test','','','ABM Test',183,'',40 from dual union all
select 65199,'1993-A18','1993-06-01 00:00:00','military missile','success',702,'ERINT Target','','','','Test',1895,'',10 from dual union all
select 652,'1946-A35','1946-05-29 00:00:00','atmospheric rocket','success',1401,'20','','','','Test',1852,'',4 from dual union all
select 6520,'1960-A26','1960-02-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 65200,'1993-F01','1993-05-27 01:22:00','orbital','failure',958,'364-02','Gorizont','Gorizont No. 39L','','Sat',587,'',150 from dual union all
select 65201,'1993-F02','1993-08-02 19:59:00','orbital','failure',1413,'K-11   (45F-9)','-           [NOSS12]','SLDCOM 3/INTRUDER 3','','Sat',1777,'',33 from dual union all
select 65202,'1993-F03','1993-09-20 05:12:00','orbital','failure',901,'PSLV-D1','IRS-1E','IRS-1E','','Sat',1357,'',349 from dual union all
select 65203,'1993-F04','1993-10-05 17:56:00','orbital','failure',1422,'23G-5','Landsat 6','Landsat 6','','Sat',1778,'',724 from dual union all
select 65204,'1993-M01','1993-01-01 00:00:00','suborbital rocket','success',1023,'','Chaff','','','Meteo',320,'',70 from dual union all
select 65205,'1993-M02','1993-01-01 00:00:00','suborbital rocket','success',1023,'','Chaff','','','Meteo',320,'',70 from dual union all
select 65206,'1993-M03','1993-01-01 00:00:00','suborbital rocket','success',1023,'','Chaff','','','Meteo',320,'',70 from dual union all
select 65207,'1993-M04','1993-01-01 00:00:00','suborbital rocket','success',1023,'','Chaff','','','Meteo',320,'',70 from dual union all
select 65208,'1993-M05','1993-01-01 00:00:00','suborbital rocket','success',1023,'','Chaff','','','Meteo',320,'',70 from dual union all
select 65209,'1993-M06','1993-01-01 00:00:00','suborbital rocket','success',1023,'','Chaff','','','Meteo',320,'',70 from dual union all
select 6521,'1960-A260','1960-11-13 11:04:59','suborbital rocket','success',725,'','','','','Meteo',1245,'SHIP-SOV',44 from dual union all
select 65210,'1993-M07','1993-11-30 06:55:00','military missile','success',941,'12','250 kg RV','','','Test',320,'',100 from dual union all
select 65211,'1993-S01','1993-01-01 00:00:00','military missile','success',6,'','LKI','','','Test',183,'',300 from dual union all
select 65212,'1993-S02','1993-01-01 00:00:00','suborbital rocket','success',1025,'','','','','Ionos?',1466,'',100 from dual union all
select 65213,'1993-S03','1993-01-01 00:00:00','suborbital rocket','success',1025,'','','','','Ionos?',1466,'',100 from dual union all
select 65214,'1993-S04','1993-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 65215,'1993-S05','1993-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 65216,'1993-S06','1993-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 65217,'1993-S07','1993-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 65218,'1993-S08','1993-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 65219,'1993-S09','1993-08-25 00:00:00','military missile','success',1453,'49242-83-423','15A35 UBP-42','GCh 15F355P7','','OT',596,'',1000 from dual union all
select 6522,'1960-A261','1960-11-14 14:25:00','military missile','success',1214,'SM-62A N-3460','Cat IIE-10','','','OT',224,'',10 from dual union all
select 65220,'1993-S10','1993-01-28 00:00:00','suborbital rocket','success',561,'TCMP 1B','','','','Reentry Test',1784,'',400 from dual union all
select 65221,'1993-S11','1993-02-05 16:24:00','military missile','success',1262,'S3','BTTV-3 Validation','','','Test',1892,'',200 from dual union all
select 65222,'1993-S12','1993-02-07 08:20:00','military missile','success',941,'10','','','','Test',320,'',100 from dual union all
select 65223,'1993-S13','1993-02-08 20:00:00','suborbital rocket','success',224,'NASA 36.100NS','','','','Solar',1873,'',289 from dual union all
select 65224,'1993-S14','1993-02-18 07:00:00','suborbital rocket','success',1074,'S-520-16','METS','','','Ionos/Active plasma',833,'',272 from dual union all
select 65225,'1993-S15','1993-02-19 18:00:00','suborbital rocket','success',213,'Starfire 1/F6','CONSORT 6','','','Micrograv',1879,'',301 from dual union all
select 65226,'1993-S16','1993-02-25 13:40:00','military missile','success',1434,'','15Zh58 K-30','','','Test',608,'',1000 from dual union all
select 65227,'1993-S17','1993-02-26 20:45:00','military missile','success',1111,'FTU-1','FTU-1','','','Test',849,'',900 from dual union all
select 65228,'1993-S18','1993-02-27 00:00:00','suborbital rocket','success',891,'DLR K-O-200','Strafam 3/21','','','Aeron',417,'',85 from dual union all
select 65229,'1993-S19','1993-03-04 08:19:00','military missile','success',907,'11PA (87-147)','Mk 21 RV x 3','GT 11PA','5Tx3','OT',1697,'',1000 from dual union all
select 6523,'1960-A262','1960-11-15 00:00:00','military missile','success',764,'59-4980  (27)','','','','Test',240,'',12 from dual union all
select 65230,'1993-S20','1993-03-08 00:15:00','suborbital rocket','success',860,'NASA 31.97UU','CWAS 29','','','Aeronomy',1895,'',140 from dual union all
select 65231,'1993-S21','1993-03-10 01:54:00','military missile','success',799,'','Mk 12  RV x 3','FOT GT151GB','GLORY TRIP 151GB','OT',1712,'',1300 from dual union all
select 65232,'1993-S22','1993-03-10 13:38:00','suborbital rocket','success',860,'NASA 31.96UU','CWAS 30','','','Aeronomy',1895,'',140 from dual union all
select 65233,'1993-S23','1993-03-16 02:12:41','suborbital rocket','success',213,'','SPEAR 3','','','Active plasma',1801,'',290 from dual union all
select 65234,'1993-S24','1993-03-22 00:00:00','suborbital rocket','success',1216,'XV-57','Maruda','','','Ionos?',13,'',102 from dual union all
select 65235,'1993-S25','1993-04-02 10:09:00','suborbital rocket','success',237,'NASA 40.004UE','UCB Alaska ''93','','','Ionos',1047,'',1425 from dual union all
select 65236,'1993-S26','1993-04-02 10:34:00','suborbital rocket','success',1482,'PT-01','','','','Test',13,'',950 from dual union all
select 65237,'1993-S27','1993-04-12 17:18:00','suborbital rocket','success',224,'NASA 36.099US','NIXT','','','Solar',1873,'',226 from dual union all
select 65238,'1993-S28','1993-04-17 09:15:00','suborbital rocket','success',224,'NASA 36.095UH','Colorado SXT','','','XR Astron',1873,'',254 from dual union all
select 65239,'1993-S29','1993-04-19 05:50:00','suborbital rocket','success',1028,'No. 36','SPICE-3','','','Ionos',1361,'',323 from dual union all
select 6524,'1960-A263','1960-11-15 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1245,'SHIP-SOV',40 from dual union all
select 65240,'1993-S30','1993-05-01 05:35:00','suborbital rocket','success',1212,'DLR K-GR-201','DLR TEXUS 30','','','Micrograv',416,'',234 from dual union all
select 65241,'1993-S31','1993-05-06 15:38:00','suborbital rocket','success',211,'NASA 27.136UE','','','','Plasma',1047,'',271 from dual union all
select 65242,'1993-S32','1993-05-14 01:10:00','suborbital rocket','success',860,'NASA 31.99UU','CWAS 31','','','Aeronomy',1895,'',140 from dual union all
select 65243,'1993-S33','1993-05-19 00:47:00','suborbital rocket','success',860,'NASA 31.98UU','CWAS 32','','','Aeronomy',1895,'',140 from dual union all
select 65244,'1993-S34','1993-05-23 09:17:00','suborbital rocket','success',686,'Red Tigress 2A','','','','Target',239,'',378 from dual union all
select 65245,'1993-S35','1993-05-26 09:43:00','suborbital rocket','success',860,'DLR C-NO-202','Aerospatiale','','','Test?',307,'',140 from dual union all
select 65246,'1993-S36','1993-05-28 08:34:00','suborbital rocket','success',686,'Red Tigress 2B','','','','Target',239,'',390 from dual union all
select 65247,'1993-S37','1993-05-29 00:00:00','military missile','success',586,'','','','','Test',1480,'',150 from dual union all
select 65248,'1993-S38','1993-05-29 00:00:00','military missile','success',580,'','','','','Test',1480,'',125 from dual union all
select 65249,'1993-S39','1993-05-29 00:00:00','military missile','success',580,'','','','','Test',1480,'',125 from dual union all
select 6525,'1960-A264','1960-11-15 17:59:00','suborbital spaceplane','success',1520,'2-10-21','XLR-99 Checkout','','','Test',379,'NB-52 003',24 from dual union all
select 65250,'1993-S40','1993-05-30 00:00:00','military missile','success',580,'','','','','Test',1480,'',125 from dual union all
select 65251,'1993-S41','1993-06-12 00:00:00','military missile','success',941,'11','','','','Test',320,'',100 from dual union all
select 65252,'1993-S42','1993-06-16 04:39:00','suborbital rocket','success',224,'NASA 36.101UL','','','','Aeron',1873,'',253 from dual union all
select 65253,'1993-S43','1993-06-22 00:00:00','suborbital rocket','success',117,'LEAP 3','','','','Tech',1873,'',300 from dual union all
select 65254,'1993-S44','1993-07-03 00:01:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT152GM','GLORY TRIP 152GM','OT',1705,'',1300 from dual union all
select 65255,'1993-S45','1993-07-07 00:00:00','military missile','success',1443,'FCET-9','','','','OT',420,'SSBN 735',1000 from dual union all
select 65256,'1993-S46','1993-07-07 00:00:00','military missile','success',1443,'FCET-9','','','','OT',420,'SSBN 735',1000 from dual union all
select 65257,'1993-S47','1993-07-14 03:19:00','military missile','success',907,'12PA (84-542)','Mk 21 RV x 8','GT 12PA','5Fx4/5Tx2','OT',1700,'',1000 from dual union all
select 65258,'1993-S48','1993-07-22 08:25:00','suborbital rocket','success',231,'NASA 35.29UE','WISP 2','','','Plasma',1821,'',900 from dual;
insert into LAUNCH
select 65259,'1993-S49','1993-07-23 08:22:00','military missile','success',1434,'','15Zh58 K-31','','','Test',608,'',1000 from dual union all
select 6526,'1960-A265','1960-11-17 19:00:00','military missile','failure',928,'A1X-52','','','','Test',244,'',10 from dual union all
select 65260,'1993-S50','1993-07-27 00:00:00','suborbital rocket','success',213,'','BLANC','','','Imaging',304,'',300 from dual union all
select 65261,'1993-S51','1993-08-01 01:46:00','suborbital rocket','success',860,'','SCT-06 (CONE)','','','Aeron',29,'',140 from dual union all
select 65262,'1993-S52','1993-08-02 00:23:00','suborbital rocket','success',860,'DLR K-NO-203  NLC-93','SSC DECIMALS A','','','Aeron',417,'',98 from dual union all
select 65263,'1993-S53','1993-08-02 01:02:00','suborbital rocket','success',860,'DLR K-NO-204  NLC-93','SSC DECIMALS B','','','Aeron',417,'',105 from dual union all
select 65264,'1993-S54','1993-08-17 18:00:00','suborbital rocket','success',224,'NASA 36.105GS','SERTS-93 (05)','','','Solar',1873,'',312 from dual union all
select 65265,'1993-S55','1993-08-20 18:27:00','military missile','success',1443,'DASO-11','','','','OT',419,'SSBN 739',1000 from dual union all
select 65266,'1993-S56','1993-08-25 10:00:00','military missile','success',1111,'M-1','Zodiac Beauchamp','','','Target',849,'',900 from dual union all
select 65267,'1993-S57','1993-08-31 00:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT153GB','GLORY TRIP 153GB','OT',1712,'',1300 from dual union all
select 65268,'1993-S58','1993-09-10 16:00:00','suborbital rocket','success',860,'NASA 31.100UU','CWAS 33','','','Aeronomy',1895,'',140 from dual union all
select 65269,'1993-S59','1993-09-13 18:00:00','suborbital rocket','success',860,'NASA 31.101UU','CWAS 34','','','Aeronomy',1895,'',140 from dual union all
select 6527,'1960-A266','1960-11-17 00:00:00','atmospheric rocket','success',1065,'R1I','','','','Test',2,'',20 from dual union all
select 65270,'1993-S60','1993-09-15 00:00:00','military missile','success',907,'13PA (88-336)','Mk 21 RV x 3','GT 13PA','','OT',1697,'',1000 from dual union all
select 65271,'1993-S61','1993-09-16 23:00:00','suborbital rocket','success',1296,'TR-1A-3','','','','Micrograv',1476,'',264 from dual union all
select 65272,'1993-S62','1993-10-04 17:45:00','suborbital rocket','success',224,'NASA 36.107UE','NCAR/CU-4','','','Solar UV',1873,'',300 from dual union all
select 65273,'1993-S63','1993-10-04 19:35:00','suborbital rocket','success',1319,'NASA 33.062UE','','','','Plasma',1895,'',200 from dual union all
select 65274,'1993-S64','1993-10-08 00:00:00','military missile','success',1262,'','BTTV-4 PAC-2 Target','','','Target',1892,'',200 from dual union all
select 65275,'1993-S65','1993-10-21 01:46:00','suborbital rocket','success',600,'IAA','FEIROX/FEIROH','','','Aeron',396,'',154 from dual union all
select 65276,'1993-S66','1993-10-26 00:00:00','military missile','success',1262,'','BTTV-5 ERINT Mutimode','','','Target',1873,'',200 from dual union all
select 65277,'1993-S67','1993-11-03 00:00:00','military missile','success',1106,'26011','Tir d''exercice','','','OT',301,'',1000 from dual union all
select 65278,'1993-S68','1993-11-04 07:07:00','suborbital rocket','success',1212,'MASER 6','SSC MASER 6','','','Micrograv',416,'',243 from dual union all
select 65279,'1993-S69','1993-11-17 00:00:00','suborbital rocket','success',1219,'XV-25','','','','Ionos?',330,'',555 from dual union all
select 6528,'1960-A267','1960-11-17 11:04:59','suborbital rocket','success',725,'','','','','Meteo',1245,'SHIP-SOV',35 from dual union all
select 65280,'1993-S70','1993-11-18 00:00:00','military missile','success',1443,'DASO-12','PGW Test','','','OT',419,'SSBN 739',1000 from dual union all
select 65281,'1993-S71','1993-11-26 11:00:00','suborbital rocket','success',1212,'DLR K-GR-205','DLR TEXUS 31','','','Micrograv',416,'',257 from dual union all
select 65282,'1993-S72','1993-11-29 09:30:00','suborbital rocket','success',847,'DLR K-NO2-206','Mini-Texus 1','','','Micrograv',417,'',146 from dual union all
select 65283,'1993-S73','1993-12-01 00:00:00','military missile','success',1548,'','Efir','','','Tech',1062,'PLBR',620 from dual union all
select 65284,'1993-S74','1993-12-01 00:00:00','military missile','success',1507,'VMF RF','','','','OT',1626,'KALMAR',1000 from dual union all
select 65285,'1993-S75','1993-11-30 00:00:00','military missile','success',1262,'','BTTV-6 ERINT GTF-2','','','Target',1873,'',200 from dual union all
select 65286,'1993-S76','1993-12-17 00:00:00','military missile','success',1262,'','MTTV-1 ERINT Demo','','','Target',1892,'',200 from dual union all
select 65287,'1993-S77','1993-02-19 13:15:00','suborbital rocket','success',1028,'','Ba release','','','Ionos/Release',1361,'',300 from dual union all
select 65288,'1993-S78','1993-02-19 13:45:00','suborbital rocket','success',1028,'','Ba release','','','Ionos/Release',1361,'',290 from dual union all
select 65289,'1993-S79','1993-01-01 00:00:00','military missile','success',437,'-        -','','','','Test',810,'',1000 from dual union all
select 6529,'1960-A268','1960-11-17 20:43:07','suborbital spaceplane','success',1520,'1-17-30','Aero Heating','','','Test',379,'NB-52 003',16 from dual union all
select 65290,'1993-W01','1993-01-06 02:04:59','suborbital rocket','success',761,'MT135P-R841','','','','Meteo',1332,'',56 from dual union all
select 65291,'1993-W02','1993-01-12 18:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',53 from dual union all
select 65292,'1993-W03','1993-01-20 02:01:00','suborbital rocket','success',761,'MT135P-R842','','','','Meteo',1332,'',58 from dual union all
select 65293,'1993-W04','1993-01-27 02:00:00','suborbital rocket','success',761,'MT135P-R843','','','','Meteo',1332,'',55 from dual union all
select 65294,'1993-W05','1993-02-01 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 65295,'1993-W06','1993-02-03 02:00:00','suborbital rocket','success',761,'MT135P-R844','','','','Meteo',1332,'',57 from dual union all
select 65296,'1993-W07','1993-02-18 02:00:00','suborbital rocket','success',761,'MT135P-R845','','','','Meteo',1332,'',56 from dual union all
select 65297,'1993-W08','1993-03-03 02:00:00','suborbital rocket','success',761,'MT135P-R847','','','','Meteo',1332,'',56 from dual union all
select 65298,'1993-W09','1993-03-17 02:00:00','suborbital rocket','success',761,'MT135P-R848','','','','Meteo',1332,'',57 from dual union all
select 65299,'1993-W10','1993-03-17 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 653,'1946-M01','1946-12-13 00:00:00','suborbital rocket','success',1517,'26','','','','Test',1852,'',53 from dual union all
select 6530,'1960-A269','1960-11-19 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1245,'SHIP-SOV',45 from dual union all
select 65300,'1993-W11','1993-03-21 14:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 65301,'1993-W12','1993-03-24 02:07:00','suborbital rocket','success',761,'MT135P-R849','','','','Meteo',1332,'',56 from dual union all
select 65302,'1993-W13','1993-03-24 15:19:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',67 from dual union all
select 65303,'1993-W14','1993-03-24 15:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 65304,'1993-W15','1993-03-26 15:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 65305,'1993-W16','1993-04-04 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',79 from dual union all
select 65306,'1993-W17','1993-04-07 02:07:00','suborbital rocket','success',761,'MT135P-R850','','','','Meteo',1332,'',55 from dual union all
select 65307,'1993-W18','1993-04-21 02:04:59','suborbital rocket','success',761,'MT135P-R851','','','','Meteo',1332,'',56 from dual union all
select 65308,'1993-W19','1993-04-28 02:02:00','suborbital rocket','success',761,'MT135P-R852','','','','Meteo',1332,'',58 from dual union all
select 65309,'1993-W20','1993-05-07 02:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',873,'',71 from dual union all
select 6531,'1960-A27','1960-02-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 65310,'1993-W21','1993-05-12 02:02:00','suborbital rocket','success',761,'MT135P-R853','','','','Meteo',1332,'',57 from dual union all
select 65311,'1993-W22','1993-05-13 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',70 from dual union all
select 65312,'1993-W23','1993-05-19 02:04:59','suborbital rocket','success',761,'MT135P-R854','','','','Meteo',1332,'',56 from dual union all
select 65313,'1993-W24','1993-05-21 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',57 from dual union all
select 65314,'1993-W25','1993-05-26 02:10:00','suborbital rocket','success',761,'MT135P-R855','','','','Meteo',1332,'',59 from dual union all
select 65315,'1993-W26','1993-05-26 11:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',54 from dual union all
select 65316,'1993-W27','1993-05-27 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 65317,'1993-W28','1993-06-09 02:26:59','suborbital rocket','success',761,'MT135P-R856','','','','Meteo',1332,'',57 from dual union all
select 65318,'1993-W29','1993-06-16 02:04:59','suborbital rocket','success',761,'MT135P-R857','','','','Meteo',1332,'',57 from dual union all
select 65319,'1993-W30','1993-06-18 14:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 6532,'1960-A270','1960-11-21 11:07:00','suborbital rocket','success',725,'','','','','Meteo',1245,'SHIP-SOV',44 from dual union all
select 65320,'1993-W31','1993-06-25 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',71 from dual union all
select 65321,'1993-W32','1993-06-30 02:21:00','suborbital rocket','success',761,'MT135P-R858','','','','Meteo',1332,'',57 from dual union all
select 65322,'1993-W33','1993-07-07 22:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',70 from dual union all
select 65323,'1993-W34','1993-07-12 19:25:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',73 from dual union all
select 65324,'1993-W35','1993-07-15 02:00:00','suborbital rocket','success',761,'MT135P-R859','','','','Meteo',1332,'',56 from dual union all
select 65325,'1993-W36','1993-07-16 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',68 from dual union all
select 65326,'1993-W37','1993-07-23 14:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',69 from dual union all
select 65327,'1993-W38','1993-08-04 02:10:00','suborbital rocket','success',761,'MT135P-R861','','','','Meteo',1332,'',56 from dual union all
select 65328,'1993-W39','1993-08-11 15:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',70 from dual union all
select 65329,'1993-W40','1993-08-18 02:00:00','suborbital rocket','success',761,'MT135P-R862','','','','Meteo',1332,'',57 from dual union all
select 6533,'1960-A271','1960-11-21 14:00:00','test rocket','failure',1054,'MRLV-1','MR-1','','','Test',279,'',0 from dual union all
select 65330,'1993-W41','1993-08-26 19:19:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',72 from dual union all
select 65331,'1993-W42','1993-09-01 02:03:00','suborbital rocket','success',761,'MT135P-R863','','','','Meteo',1332,'',57 from dual union all
select 65332,'1993-W43','1993-09-01 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 65333,'1993-W44','1993-09-10 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',74 from dual union all
select 65334,'1993-W45','1993-09-15 02:00:00','suborbital rocket','success',761,'MT135P-R864','','','','Meteo',1332,'',55 from dual union all
select 65335,'1993-W46','1993-09-22 02:00:00','suborbital rocket','success',761,'MT135P-R865','','','','Meteo',1332,'',54 from dual union all
select 65336,'1993-W47','1993-09-24 02:00:00','suborbital rocket','success',759,'MT-135-59','NNP-O3','','','Aeron',839,'',56 from dual union all
select 65337,'1993-W48','1993-10-13 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',66 from dual union all
select 65338,'1993-W49','1993-10-14 02:00:00','suborbital rocket','success',761,'MT135P-R867','','','','Meteo',1332,'',53 from dual union all
select 65339,'1993-W50','1993-10-17 13:38:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',56 from dual union all
select 6534,'1960-A272','1960-11-22 21:25:55','suborbital spaceplane','success',1520,'2-11-22','XLR-99, BCS','','','Test',379,'NB-52 003',18 from dual union all
select 65340,'1993-W51','1993-10-20 02:10:00','suborbital rocket','success',761,'MT135P-R868','','','','Meteo',1332,'',57 from dual union all
select 65341,'1993-W52','1993-10-27 02:00:00','suborbital rocket','success',761,'MT135P-R869','','','','Meteo',1332,'',54 from dual union all
select 65342,'1993-W53','1993-11-10 02:00:00','suborbital rocket','success',761,'MT135P-R870','','','','Meteo',1332,'',54 from dual union all
select 65343,'1993-W54','1993-11-17 02:00:00','suborbital rocket','success',761,'MT135P-R871','','','','Meteo',1332,'',52 from dual union all
select 65344,'1993-W55','1993-11-18 23:55:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 65345,'1993-W56','1993-11-29 15:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',54 from dual union all
select 65346,'1993-W57','1993-12-02 02:00:00','suborbital rocket','success',761,'MT135P-R872','','','','Meteo',1332,'',54 from dual union all
select 65347,'1993-W58','1993-12-08 02:00:00','suborbital rocket','success',761,'MT135P-R873','','','','Meteo',1332,'',55 from dual union all
select 65348,'1993-W59','1993-12-15 02:15:00','suborbital rocket','success',761,'MT135P-R874','','','','Meteo',1332,'',53 from dual;
insert into LAUNCH
select 65349,'1993-W60','1993-12-22 02:03:00','suborbital rocket','success',761,'MT135P-R875','','','','Meteo',1332,'',56 from dual union all
select 6535,'1960-A273','1960-11-24 00:00:00','military missile','success',1469,'','LKI','','','ABM Test',182,'',25 from dual union all
select 65350,'1993-W61','1993-09-23 02:00:00','suborbital rocket','success',759,'MT-135-58','NNP-O3','','','Aeron',839,'',57 from dual union all
select 65351,'1993-W62','1993-02-24 02:00:00','suborbital rocket','success',761,'MT135P-R846','','','','Meteo',1332,'',60 from dual union all
select 65352,'1993-W63','1993-07-21 02:00:00','suborbital rocket','success',761,'MT135P-R860','','','','Meteo',1332,'',60 from dual union all
select 65353,'1993-W64','1993-09-29 02:00:00','suborbital rocket','success',761,'MT135P-R866','','','','Meteo',1332,'',60 from dual union all
select 65354,'1993-Y01','1993-01-05 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',88 from dual union all
select 65355,'1993-Y02','1993-01-15 09:24:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',91 from dual union all
select 65356,'1993-Y03','1993-02-17 15:40:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65357,'1993-Y04','1993-02-24 13:01:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',88 from dual union all
select 65358,'1993-Y05','1993-03-10 05:40:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',873,'',90 from dual union all
select 65359,'1993-Y06','1993-03-29 18:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 6536,'1960-A274','1960-11-26 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1246,'SHIP-SOV',44 from dual union all
select 65360,'1993-Y07','1993-04-06 20:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 65361,'1993-Y08','1993-04-23 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',86 from dual union all
select 65362,'1993-Y09','1993-04-25 13:14:59','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',84 from dual union all
select 65363,'1993-Y10','1993-04-29 16:00:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65364,'1993-Y11','1993-05-11 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',82 from dual union all
select 65365,'1993-Y12','1993-05-11 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 65366,'1993-Y13','1993-05-11 19:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 65367,'1993-Y14','1993-05-20 13:44:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',84 from dual union all
select 65368,'1993-Y15','1993-06-01 11:23:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 65369,'1993-Y16','1993-07-02 14:37:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',85 from dual union all
select 6537,'1960-A275','1960-11-28 11:42:00','suborbital rocket','success',725,'','','','','Meteo',1246,'SHIP-SOV',42 from dual union all
select 65370,'1993-Y17','1993-07-03 02:43:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',873,'',85 from dual union all
select 65371,'1993-Y18','1993-07-06 20:45:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 65372,'1993-Y19','1993-07-14 14:37:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65373,'1993-Y20','1993-08-03 18:59:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65374,'1993-Y21','1993-08-20 13:24:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 65375,'1993-Y22','1993-08-25 11:11:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',873,'',90 from dual union all
select 65376,'1993-Y23','1993-09-01 00:00:00','suborbital rocket','success',728,'','TERLS M100 #1162','','','Aeron',1467,'',80 from dual union all
select 65377,'1993-Y24','1993-09-15 17:35:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 65378,'1993-Y25','1993-09-16 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 65379,'1993-Y26','1993-10-01 17:13:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 6538,'1960-A276','1960-12-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 65380,'1993-Y27','1993-10-13 14:35:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 65381,'1993-Y28','1993-11-18 22:19:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',176,'',84 from dual union all
select 65382,'1993-Y29','1993-12-14 16:04:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',93 from dual union all
select 65383,'1993-Y30','1993-12-17 18:12:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65384,'1993-Y31','1993-12-20 16:34:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 65385,'1993-Y32','1993-01-01 00:00:00','suborbital rocket','unknown',728,'Meteo','','','','Aeron',1467,'',79 from dual union all
select 65386,'1993-Y33','1993-01-01 00:00:00','suborbital rocket','unknown',728,'Meteo','','','','Aeron',1467,'',79 from dual union all
select 65387,'1993-Y34','1993-01-01 00:00:00','suborbital rocket','unknown',728,'Meteo','','','','Aeron',1467,'',79 from dual union all
select 65388,'1993-Y35','1993-01-01 00:00:00','suborbital rocket','unknown',728,'Meteo','','','','Aeron',1467,'',79 from dual union all
select 65389,'1993-Y36','1993-01-01 00:00:00','suborbital rocket','unknown',728,'Meteo','','','','Aeron',1467,'',79 from dual union all
select 6539,'1960-A277','1960-12-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 65390,'1993-Y37','1993-01-01 00:00:00','suborbital rocket','unknown',728,'Meteo','','','','Aeron',1467,'',79 from dual union all
select 65391,'1993-Y38','1993-08-02 00:37:00','suborbital rocket','success',1502,'Sphere        NLC-93','','','','Aeron',417,'',110 from dual union all
select 65392,'1993-Y39','1993-08-02 01:24:00','suborbital rocket','success',1502,'Sphere        NLC-93','','','','Aeron',417,'',105 from dual union all
select 65393,'1993-Y40','1993-08-02 03:24:00','suborbital rocket','success',1502,'Sphere        NLC-93','','','','Aeron',417,'',106 from dual union all
select 65394,'1993-Y41','1993-08-02 05:54:00','suborbital rocket','success',1502,'Sphere        NLC-93','','','','Aeron',417,'',107 from dual union all
select 65395,'1993-Y42','1993-07-28 05:43:00','suborbital rocket','success',1502,'Sphere        NLC-93','','','','Aeron',417,'',116 from dual union all
select 65396,'1993-Y43','1993-07-30 06:19:00','suborbital rocket','success',1502,'Sphere        NLC-93','','','','Aeron',417,'',116 from dual union all
select 65397,'1994-001','1994-01-08 10:05:34','orbital','success',1242,'N 15000-071','Soyuz TM-18','Soyuz 7K-STM No. 67','','Sat',480,'',null from dual union all
select 65398,'1994-002','1994-01-20 09:49:00','orbital','success',959,'358-02','Gals 1','Gals No. 11L','','Sat',587,'',null from dual union all
select 65399,'1994-003','1994-01-25 00:25:00','orbital','success',1448,'67051-701','Meteor-3','Meteor-3 No. 7','','Sat',469,'',null from dual union all
select 654,'1946-S01','1946-05-10 21:15:00','suborbital rocket','success',1470,'3','','','','CR',1863,'',112 from dual union all
select 6540,'1960-A278','1960-12-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 65400,'1994-004','1994-01-25 16:34:00','deep space','success',1422,'23G-11','DSPSE','DSPSE','','Sat',1778,'',null from dual union all
select 65401,'1994-005','1994-01-28 02:12:10','orbital','success',1238,'N 15000-635','Progress M-21','Progress 7K-TGM No. 221','','Sat',480,'',null from dual union all
select 65402,'1994-006','1994-02-03 12:10:00','orbital','success',1243,'STS-60','SH-02/WSF','OV-103 Research/FF','','Sat',921,'MLP3',null from dual union all
select 65403,'1994-007','1994-02-03 22:20:00','orbital','success',526,'H-II-1F','OREX','OREX','','Sat',1477,'',null from dual union all
select 65404,'1994-008','1994-02-05 08:46:00','orbital','success',958,'375-02','Raduga-1','Globus!No. 13L','','Sat',587,'',null from dual union all
select 65405,'1994-009','1994-02-07 21:47:01','orbital','success',1409,'K-10/TC-12  (45E-3)','USA 99  (Milstar 1)','Milstar DFS-1','','Sat',274,'',null from dual union all
select 65406,'1994-010','1994-02-08 08:33:53','orbital','success',309,'Y1       (32)','KF-1/SJ-4','shijian sihao','','Sat',1903,'',null from dual union all
select 65407,'1994-011','1994-02-12 08:54:13','orbital','success',1448,'67013-303','Kosmos-2268/2273','Strela-3','','Sat',469,'',null from dual union all
select 65408,'1994-012','1994-02-18 07:56:00','orbital','success',958,'376-02','Raduga','Gran'' No. 40L','','Sat',587,'',null from dual union all
select 65409,'1994-013','1994-02-19 23:45:00','orbital','success',426,'D225','Galaxy 1R','Galaxy 1RR','','Sat',234,'',null from dual union all
select 6541,'1960-A279','1960-12-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 65410,'1994-014','1994-03-02 03:25:00','orbital','success',1448,'51024-504','Koronas-I','AUOS-SM-KI-IK!No. 801','','Sat',469,'',null from dual union all
select 65411,'1994-015','1994-03-04 13:53:00','orbital','success',1243,'STS-62','USMP-2/OAST-2','OV-102 Research','','Sat',922,'MLP1',null from dual union all
select 65412,'1994-016','1994-03-10 03:40:01','orbital','success',422,'D226','USA 100 (GPS IIA-24)','Navstar GPS SVN 36','','Sat',233,'',null from dual union all
select 65413,'1994-017','1994-03-13 22:32:00','orbital','success',18,'T1   1110','STEP 0','P90-5','','Sat',1670,'',null from dual union all
select 65414,'1994-018','1994-03-17 16:30:00','orbital','success',1241,'76032  -993','Kosmos-2274','Kobal''t','','Sat',474,'',null from dual union all
select 65415,'1994-019','1994-03-22 04:54:12','orbital','success',1238,'76032  -992','Progress M-22','Progress 7K-TGM No. 222','','Sat',480,'',null from dual union all
select 65416,'1994-020','1994-04-09 11:05:01','orbital','success',1243,'STS-59','SRL-1','OV-105 Spacelab','','Sat',921,'MLP2',null from dual union all
select 65417,'1994-021','1994-04-11 07:49:22','orbital','success',958,'377-01','Kosmos-2275/2277','Uragan No. 58L','','Sat',587,'',null from dual union all
select 65418,'1994-022','1994-04-13 06:04:02','orbital','success',171,'AC-73','GOES 8','GOES I','','Sat',269,'',null from dual union all
select 65419,'1994-023','1994-04-23 08:01:59','orbital','success',1537,'45019-219','Kosmos-2278','Tselina-2','','Sat',567,'',null from dual union all
select 6542,'1960-A28','1960-02-01 10:30:00','suborbital rocket','success',725,'','','','','Meteo',1213,'VOKV',45 from dual union all
select 65420,'1994-024','1994-04-26 02:14:15','orbital','success',680,'53735-614','Kosmos-2279','Parus','','Sat',460,'',null from dual union all
select 65421,'1994-025','1994-04-28 17:14:00','orbital','success',1238,'78023  -751','Kosmos-2280','Neman','','Sat',562,'',null from dual union all
select 65422,'1994-026','1994-05-03 15:55:01','orbital','success',1409,'K-7/TC-10   (45E-1)','USA 103      [JS8]','TRUMPET 1','','Sat',275,'',null from dual union all
select 65423,'1994-027','1994-05-04 00:00:00','orbital','success',22,'ASLV-D4','SROSS C2','SROSS-C2','','Sat',1360,'',null from dual union all
select 65424,'1994-028','1994-05-09 02:47:00','orbital','success',1135,'S218C','MSTI 2','MSTI-2','','Sat',1779,'',null from dual union all
select 65425,'1994-029','1994-05-19 17:03:00','orbital','success',913,'005/F5','STEP 2','STEP 2','','Sat',379,'NB-52 008',null from dual union all
select 65426,'1994-030','1994-05-20 02:01:00','orbital','success',958,'357-01','Gorizont','Gorizont No. 42L','','Sat',587,'',null from dual union all
select 65427,'1994-031','1994-05-22 04:30:04','orbital','success',1242,'76024  -355 (ex-U2)','Progress M-23','Progress 7K-TGM No. 223','','Sat',480,'',null from dual union all
select 65428,'1994-032','1994-06-07 07:20:00','orbital','success',1241,'78012  -831','Kosmos-2281','Oblik (Priroda)','','Sat',462,'',null from dual union all
select 65429,'1994-033','1994-06-14 16:05:00','orbital','success',1241,'N 15000-040','Foton','Foton No. 9','','Sat',474,'',null from dual union all
select 6543,'1960-A280','1960-12-01 00:00:00','suborbital rocket','success',642,'SP-150-1','K-9L 1/1.6 Model','','','Test',7,'',5 from dual union all
select 65430,'1994-034','1994-06-17 07:07:19','orbital','success',107,'V64 L436','Intelsat 702/ASAP 5','INTELSAT 702','','Sat',345,'',null from dual union all
select 65431,'1994-035','1994-06-24 13:50:02','orbital','success',171,'AC-76','USA 104 (UHF F3)','UFO F3','','Sat',269,'',null from dual union all
select 65432,'1994-036','1994-07-01 12:24:50','orbital','success',1242,'N 15000-072','Soyuz TM-19','Soyuz 7K-STM No. 68','','Sat',480,'',null from dual union all
select 65433,'1994-037','1994-07-03 08:00:00','orbital','success',304,'Y2       (33)','FSW-2','FSW-2 No. 2','','Sat',810,'',null from dual union all
select 65434,'1994-038','1994-07-06 23:58:51','orbital','success',958,'365-02','Kosmos-2282','Oko-1 No. 7123','','Sat',587,'',null from dual union all
select 65435,'1994-039','1994-07-08 16:43:00','orbital','success',1243,'STS-65','Spacelab IML-2','OV-102 Spacelab','','Sat',921,'MLP3',null from dual union all
select 65436,'1994-040','1994-07-08 23:05:32','orbital','success',104,'V65 L437b','BS-3N/PAS 2','Panamsat K1','','Sat',345,'',null from dual union all
select 65437,'1994-041','1994-07-14 05:13:30','orbital','success',680,'65035-612','Nadezhda','Nadezhda 17F118 No. 104','','Sat',460,'',null from dual union all
select 65438,'1994-042','1994-07-20 17:35:00','orbital','success',1241,'77044  -995','Kosmos-2283','Kobal''t','','Sat',474,'',null from dual;
insert into LAUNCH
select 65439,'1994-043','1994-07-21 10:55:00','orbital','success',308,'Y8       (34)','Apstar 1','APSTAR 1','','Sat',1904,'',null from dual union all
select 6544,'1960-A281','1960-12-01 00:00:00','suborbital rocket','success',642,'SP-150-2','K-9L 1/1.6 Model','','','Test',7,'',5 from dual union all
select 65440,'1994-044','1994-07-29 09:30:00','orbital','success',1238,'77031  -497','Kosmos-2284','Kometa No. 17','','Sat',562,'',null from dual union all
select 65441,'1994-045','1994-08-02 20:00:01','orbital','success',680,'47135-613','Kosmos-2285','Obzor No. 1','','Sat',456,'',null from dual union all
select 65442,'1994-046','1994-08-03 14:38:00','orbital','success',909,'F7','APEX','APEX','','Sat',379,'NB-52 008',null from dual union all
select 65443,'1994-047','1994-08-03 23:57:02','orbital','success',173,'AC-107','DBS 2','DBS 2','','Sat',268,'',null from dual union all
select 65444,'1994-048','1994-08-05 01:12:21','orbital','success',803,'78012  -642','Kosmos-2286','Oko','','Sat',462,'',null from dual union all
select 65445,'1994-049','1994-08-10 23:05:24','orbital','success',107,'V66 L438a','Turksat 1B/Bras. B1','Brasilsat B1','','Sat',345,'',null from dual union all
select 65446,'1994-050','1994-08-11 15:27:46','orbital','success',958,'367-02','Kosmos-2287/2289','Uragan No. 67L','','Sat',587,'',null from dual union all
select 65447,'1994-051','1994-08-23 14:30:59','orbital','success',805,'77044  -661','Molniya-3','Molniya-3 No. 60','','Sat',475,'',null from dual union all
select 65448,'1994-052','1994-08-25 14:25:12','orbital','success',1238,'N 15000-636','Progress M-24','Progress 7K-TGM No. 224','','Sat',480,'',null from dual union all
select 65449,'1994-053','1994-08-26 12:00:00','orbital','success',1537,'67057-901','Kosmos-2290','Orlets-2','','Sat',567,'',null from dual union all
select 6545,'1960-A282','1960-11-30 11:48:00','suborbital rocket','success',725,'','','','','Meteo',1246,'SHIP-SOV',46 from dual union all
select 65450,'1994-054','1994-08-27 08:58:01','orbital','success',1409,'K-9/TC-11   (45E-2)','USA 105      [CY14]','MERCURY 1','','Sat',275,'',null from dual union all
select 65451,'1994-055','1994-08-27 23:10:00','orbital','success',305,'Y5       (35)','Optus B3','Optus B3','','Sat',1903,'',null from dual union all
select 65452,'1994-056','1994-08-28 07:50:00','orbital','success',526,'H-II-2F','ETS-6/LAPS','ETS-6','','Sat',1477,'',null from dual union all
select 65453,'1994-057','1994-08-29 17:38:00','orbital','success',155,'20E','USA 106 (DMSP)','DMSP Block 5D-2 S-11','','Sat',1776,'',null from dual union all
select 65454,'1994-058','1994-09-09 00:29:44','orbital','success',98,'V67 L429a','Telstar 402','Telstar 402','','Sat',345,'',null from dual union all
select 65455,'1994-059','1994-09-09 22:22:53','orbital','success',1243,'STS-64','Spartan/LITE','OV-103 Research/FF','','Sat',922,'MLP2',null from dual union all
select 65456,'1994-060','1994-09-21 17:53:00','orbital','success',958,'381-02','Kosmos-2291','Geizer No. 19L','','Sat',553,'',null from dual union all
select 65457,'1994-061','1994-09-27 14:00:00','orbital','success',680,'47135-616','Kosmos-2292','Vektor','','Sat',456,'',null from dual union all
select 65458,'1994-062','1994-09-30 11:16:00','orbital','success',1243,'STS-68','SRL-2','OV-105 Spacelab','','Sat',921,'MLP1',null from dual union all
select 65459,'1994-063','1994-10-03 22:42:30','orbital','success',1242,'Ya15000-074','Soyuz TM-20','Soyuz 7K-STM No. 69','','Sat',480,'',null from dual union all
select 6546,'1960-A283','1960-11-30 18:42:43','suborbital spaceplane','success',1520,'1-18-31','','','','Test',379,'NB-52 008',14 from dual union all
select 65460,'1994-064','1994-10-06 06:35:02','orbital','success',174,'AC-111','Intelsat 703','INTELSAT 703','','Sat',269,'',null from dual union all
select 65461,'1994-065','1994-10-08 01:07:00','orbital','success',104,'V68 L440','Thaicom 2/Solid. 2','Solidaridad 2','','Sat',345,'',null from dual union all
select 65462,'1994-066','1994-10-11 14:30:00','orbital','success',1448,'67033-306','Okean','Okean-O1 No. 7/NKhM No.','','Sat',470,'',null from dual union all
select 65463,'1994-067','1994-10-13 16:19:00','orbital','success',959,'377-02','Ekspress 11','Ekspress No. 11L','','Sat',553,'',null from dual union all
select 65464,'1994-068','1994-10-15 05:05:00','orbital','success',901,'PSLV-D2','IRS-P2','IRS-P2','','Sat',1357,'',null from dual union all
select 65465,'1994-069','1994-10-31 14:30:56','orbital','success',958,'361-02','Elektro','Elektro No. 1L','','Sat',587,'',null from dual union all
select 65466,'1994-070','1994-11-01 00:37:00','orbital','success',101,'V69 L441','Astra 1D','Astra 1D','','Sat',345,'',null from dual union all
select 65467,'1994-071','1994-11-01 09:31:00','deep space','success',423,'D227','Wind','WIND','','Sat',234,'',null from dual union all
select 65468,'1994-072','1994-11-02 01:04:00','orbital','success',1446,'51013-301','Kosmos-2293','US-PM','','Sat',591,'',null from dual union all
select 65469,'1994-073','1994-11-03 16:59:43','orbital','success',1243,'STS-66','Spacelab Atlas 3','OV-104 Spacelab','','Sat',922,'MLP3',null from dual union all
select 6547,'1960-A284','1960-12-01 00:00:00','suborbital rocket','success',552,'','','','','Reentry Test',1821,'',26 from dual union all
select 65470,'1994-074','1994-11-04 05:47:00','orbital','success',1537,'51057-902','Resurs-O1','Resurs-O1 No. 3L','','Sat',567,'',null from dual union all
select 65471,'1994-075','1994-11-11 07:21:58','orbital','success',1238,'Ya15000-638','Progress M-25','Progress 7K-TGM No. 225','','Sat',480,'',null from dual union all
select 65472,'1994-076','1994-11-20 00:39:37','orbital','success',958,'371-01','Kosmos-2294/2296','Uragan No. 62L','','Sat',553,'',null from dual union all
select 65473,'1994-077','1994-11-24 09:15:59','orbital','success',1537,'45033-305','Kosmos-2297','Tselina-2','','Sat',567,'',null from dual union all
select 65474,'1994-078','1994-11-29 02:54:00','orbital','success',1448,'51092-805','Geo-IK','Musson No. 24','','Sat',470,'',null from dual union all
select 65475,'1994-079','1994-11-29 10:21:02','orbital','success',173,'AC-110','Orion 1','Orion 1','','Sat',268,'',null from dual union all
select 65476,'1994-080','1994-11-29 17:02:00','orbital','success',309,'Y2       (36)','Zhongxing 6','DFH-3','','Sat',1903,'',null from dual union all
select 65477,'1994-081','1994-12-14 14:21:00','orbital','success',805,'77031  -672','Molniya-1T','Molniya-1T','','Sat',475,'',null from dual union all
select 65478,'1994-082','1994-12-16 12:00:00','orbital','success',958,'373-02','Luch','Al''tair No. 13L','','Sat',587,'',null from dual union all
select 65479,'1994-083','1994-12-20 05:11:01','orbital','success',680,'53735-617','Kosmos-2298','Strela-2M','','Sat',456,'',null from dual union all
select 6548,'1960-A285','1960-12-01 11:04:59','suborbital rocket','success',725,'','','','','Meteo',1247,'SHIP-SOV',41 from dual union all
select 65480,'1994-084','1994-12-22 22:19:02','orbital','success',1411,'K-14        (45D-3)','USA 107      [DSP17]','DSP 17','','Sat',274,'',null from dual union all
select 65481,'1994-085','1994-12-26 03:01:16','orbital','success',1063,'No. 4L/511-1992-004','Radio-ROSTO','Radio-ROSTO','','Sat',537,'',null from dual union all
select 65482,'1994-086','1994-12-26 22:26:58','orbital','success',1448,'67034-502','Kosmos-2299/2304','Strela-3','','Sat',470,'',null from dual union all
select 65483,'1994-087','1994-12-28 11:31:00','orbital','success',958,'366-01','Raduga','Gran'' No. 43L','','Sat',587,'',null from dual union all
select 65484,'1994-088','1994-12-29 11:30:00','orbital','success',1238,'76032  -762','Kosmos-2305','Neman','','Sat',562,'',null from dual union all
select 65485,'1994-089','1994-12-30 10:02:00','orbital','success',155,'11E','NOAA 14','NOAA J','','Sat',1776,'',null from dual union all
select 65486,'1994-A01','1994-01-01 00:00:00','military missile','success',1088,'','Block IVA Demo','','','Test',1895,'',10 from dual union all
select 65487,'1994-A02','1994-01-06 02:00:00','suborbital rocket','success',761,'MT135P-R876','','','','Meteo',1332,'',49 from dual union all
select 65488,'1994-A03','1994-02-04 00:00:00','test rocket','success',128,'','Pumped Prop''n Test','','','Test',1704,'',2 from dual union all
select 65489,'1994-A04','1994-03-01 00:00:00','military missile','success',118,'','','','','Test',1024,'SHIP-ISR',20 from dual union all
select 6549,'1960-A286','1960-12-02 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1248,'SHIP-SOV',39 from dual union all
select 65490,'1994-A05','1994-06-12 00:00:00','military missile','success',118,'','','','','Test',1024,'SHIP-ISR',20 from dual union all
select 65491,'1994-A06','1994-06-20 14:42:00','test rocket','success',345,'','','','','Test',1895,'',1 from dual union all
select 65492,'1994-A07','1994-06-27 14:36:00','test rocket','success',345,'','','','','Test',1895,'',0 from dual union all
select 65493,'1994-F01','1994-01-24 21:37:00','orbital','failure',107,'V63 L435b','Eut. 2F5/Turksat 1A','Eutelsat 2 F5','','Sat',345,'',220 from dual union all
select 65494,'1994-F02','1994-05-25 10:15:00','orbital','failure',1448,'51082-802','','Tselina-D','','Sat',470,'',221 from dual union all
select 65495,'1994-F03','1994-06-27 21:15:00','orbital','failure',911,'F6','STEP 1','STEP 1','','Sat',1718,'L-1011',38 from dual union all
select 65496,'1994-F04','1994-12-01 22:57:51','orbital','failure',102,'V70 L442a','PAS 3','Panamsat K2','','Sat',345,'',100 from dual union all
select 65497,'1994-M01','1994-12-05 00:48:00','suborbital rocket','success',605,'SSC HYGROSOND','SSC Hygrosond','','','Aeron',417,'',77 from dual union all
select 65498,'1994-S01','1994-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 65499,'1994-S02','1994-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 655,'1946-S02','1946-05-29 21:12:00','suborbital rocket','success',1470,'4','','','','CR',1863,'',112 from dual union all
select 6550,'1960-A287','1960-12-04 05:35:00','military missile','destroyed before launch',1419,'V-2','Dummy RV','','','Test',1714,'',0 from dual union all
select 65500,'1994-S03','1994-11-24 00:00:00','military missile','success',1453,'51122-81-434','15A35 UBP-43','GCh 15F355P7','','OT',596,'',1000 from dual union all
select 65501,'1994-S04','1994-01-20 01:03:52','military missile','success',1443,'FCET-10','','','','OT',420,'SSBN 738',1000 from dual union all
select 65502,'1994-S05','1994-01-20 01:04:09','military missile','success',1443,'FCET-10','','','','OT',420,'SSBN 738',1000 from dual union all
select 65503,'1994-S06','1994-01-20 02:21:06','military missile','success',1443,'FCET-10','','','','OT',420,'SSBN 738',1000 from dual union all
select 65504,'1994-S07','1994-01-20 02:22:23','military missile','success',1443,'FCET-10','','','','OT',420,'SSBN 738',1000 from dual union all
select 65505,'1994-S08','1994-01-30 22:00:00','suborbital rocket','success',211,'NASA 27.137UE','','','','Plasma',1047,'',340 from dual union all
select 65506,'1994-S09','1994-02-02 00:00:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT154GB','GLORY TRIP 154GB','OT',1712,'',1300 from dual union all
select 65507,'1994-S10','1994-02-09 23:43:00','suborbital rocket','success',224,'NASA 36.110IE','PULSAUR-2 (F-92)','','','Auroral',29,'',300 from dual union all
select 65508,'1994-S100','1994-11-06 00:00:00','military missile','success',1166,'','vs Camp Ashraf','','','Weapon',857,'',100 from dual union all
select 65509,'1994-S101','1994-11-06 00:00:00','military missile','success',1166,'','vs Camp Ashraf','','','Weapon',857,'',100 from dual union all
select 6551,'1960-A288','1960-12-04 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1249,'SHIP-SOV',41 from dual union all
select 65510,'1994-S102','1994-11-06 00:00:00','military missile','success',1166,'','vs Camp Ashraf','','','Weapon',857,'',100 from dual union all
select 65511,'1994-S11','1994-02-10 05:35:00','suborbital rocket','success',213,'NASA 36.097IE','GEMINI','','','Plasma/Aeron',1895,'',245 from dual union all
select 65512,'1994-S12','1994-02-12 13:08:00','suborbital rocket','success',867,'NASA 18.233UE','Aria-II','','','Plasma',1047,'',270 from dual union all
select 65513,'1994-S13','1994-02-12 13:12:00','suborbital rocket','success',867,'NASA 18.232UE','Aria-II','','','Plasma',1047,'',270 from dual union all
select 65514,'1994-S14','1994-02-12 13:16:00','suborbital rocket','success',211,'NASA 27.131CE','Aria-II','','','Plasma',1047,'',242 from dual union all
select 65515,'1994-S15','1994-02-15 16:18:00','military missile','success',1262,'','BTTV-7 ERINT GTF-3','','','Target',1892,'',200 from dual union all
select 65516,'1994-S16','1994-02-16 07:52:00','suborbital rocket','success',1073,'S-310-22','NNP/AIR/NID/NEL','','','Aeron',29,'',192 from dual union all
select 65517,'1994-S17','1994-02-19 09:27:00','suborbital rocket','failure',891,'DLR K-O-207','RASMUS 5','','','Aeron',417,'',3 from dual union all
select 65518,'1994-S18','1994-02-19 12:13:00','military missile','success',68,'04','Agni RV Mk 1','','','Test',320,'',300 from dual union all
select 65519,'1994-S19','1994-03-01 00:00:00','suborbital rocket','success',213,'','NOIR','','','Imaging',304,'',300 from dual union all
select 6552,'1960-A289','1960-12-05 00:00:00','military missile','success',1214,'SM-62A N-3458','Cat IIE-11, Sta12','','','OT',224,'',10 from dual union all
select 65520,'1994-S20','1994-03-06 08:21:00','suborbital rocket','success',237,'NASA 40.005UE','Auroral Turbulence 1','','','Ionos',1047,'',455 from dual union all
select 65521,'1994-S21','1994-03-07 09:11:47','suborbital rocket','failure',860,'NASA 31.071UE','','','','Plasma',1047,'',140 from dual union all
select 65522,'1994-S22','1994-03-08 03:39:00','military missile','success',907,'14PA (84-559)','Mk 21 RV x 8','GT 14PA','5F/5Tx2/SPO 5Tx5','OT',1700,'',1000 from dual union all
select 65523,'1994-S23','1994-03-11 06:50:42','suborbital rocket','success',224,'NASA 36.114DE','HIRAAS 3','','','Auroral',1047,'',308 from dual union all
select 65524,'1994-S24','1994-03-28 18:06:55','military missile','unknown',1442,'FCET-44','-        181','','','OT',420,'SSBN 729',1000 from dual union all
select 65525,'1994-S25','1994-03-28 18:07:00','military missile','unknown',1442,'FCET-44','','','','OT',420,'SSBN 729',1000 from dual union all
select 65526,'1994-S26','1994-03-28 18:07:00','military missile','unknown',1442,'FCET-44','','','','OT',420,'SSBN 729',1000 from dual union all
select 65527,'1994-S27','1994-03-28 18:07:00','military missile','unknown',1442,'FCET-44','','','','OT',420,'SSBN 729',1000 from dual union all
select 65528,'1994-S28','1994-04-05 18:22:00','suborbital rocket','success',1334,'NASA 12.046WT','','','','Test',1821,'',127 from dual;
insert into LAUNCH
select 65529,'1994-S29','1994-04-16 01:22:51','suborbital rocket','success',600,'IAA','O2-INTA300','','','Aeron',396,'',156 from dual union all
select 6553,'1960-A29','1960-02-02 10:37:59','suborbital rocket','success',725,'','','','','Meteo',1214,'VOKV',46 from dual union all
select 65530,'1994-S30','1994-04-18 04:30:00','suborbital rocket','success',224,'NASA 36.109UG','JHU FOT 11R','','','UV Astron',1873,'',300 from dual union all
select 65531,'1994-S31','1994-04-25 20:45:00','suborbital rocket','success',224,'NASA 36.123CS','','','','Solar',1873,'',300 from dual union all
select 65532,'1994-S32','1994-05-03 07:40:00','suborbital rocket','success',847,'DLR K-NO2-208','Mini-Texus 2','','','Micrograv',417,'',139 from dual union all
select 65533,'1994-S33','1994-05-05 04:15:00','suborbital rocket','success',1212,'DLR K-GR-209','DLR TEXUS 32','','','Micrograv',416,'',235 from dual union all
select 65534,'1994-S34','1994-05-17 00:00:00','military missile','success',907,'15PA (84-550)','Mk 21 RV x 8','GT 15PA','5F/5Tx2/SPO 5Tx5','OT',1697,'',1000 from dual union all
select 65535,'1994-S35','1994-05-26 22:15:03','military missile','success',1443,'DASO-1','','','','OT',419,'UK S28',1000 from dual union all
select 65536,'1994-S36','1994-06-04 06:10:00','military missile','success',941,'13','','','','Test',320,'',100 from dual union all
select 65537,'1994-S37','1994-06-06 00:00:00','military missile','success',941,'14','','Army user trial','','Test',320,'',100 from dual union all
select 65538,'1994-S38','1994-06-08 00:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT155GM','GLORY TRIP 155GM','OT',1699,'',1300 from dual union all
select 65539,'1994-S39','1994-06-20 03:00:03','military missile','success',1443,'DASO-2','','','','OT',419,'UK S28',1000 from dual union all
select 6554,'1960-A290','1960-12-06 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1251,'SHIP-SOV',39 from dual union all
select 65540,'1994-S40','1994-06-22 00:00:00','suborbital rocket','success',1216,'XV-58','Oituia','','','Ionos?',330,'',93 from dual union all
select 65541,'1994-S41','1994-06-22 02:20:00','suborbital rocket','success',860,'NASA 31.087UE','','','','Plasma',1821,'',140 from dual union all
select 65542,'1994-S42','1994-06-22 08:22:00','military missile','success',1434,'','15Zh58 UBP-13','','','Test',608,'',1000 from dual union all
select 65543,'1994-S43','1994-06-27 22:05:00','suborbital rocket','success',1319,'NASA 33.063UE','','','','Plasma',1047,'',200 from dual union all
select 65544,'1994-S44','1994-07-01 00:00:00','military missile','success',580,'Scud C','','','','Test',1457,'',125 from dual union all
select 65545,'1994-S45','1994-07-06 00:00:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT156GM','GLORY TRIP 156GM','OT',1705,'',1300 from dual union all
select 65546,'1994-S46','1994-07-08 00:00:00','suborbital rocket','success',1162,'','MSTI-2 Target','','','Target',1821,'',100 from dual union all
select 65547,'1994-S47','1994-07-16 00:23:00','suborbital rocket','success',860,'NASA 31.086UE','','','','Ionos',1821,'',140 from dual union all
select 65548,'1994-S48','1994-07-16 00:23:00','suborbital rocket','success',891,'NASA 30.39UE','','','','Ionos',1821,'',85 from dual union all
select 65549,'1994-S49','1994-07-20 05:53:02','suborbital rocket','success',224,'NASA 36.121CL','EUVS Jupiter','','','UV Astron',1873,'',163 from dual union all
select 6555,'1960-A291','1960-12-06 23:29:30','suborbital spaceplane','success',1520,'2-12-23','XLR-99, BCS','','','Test',379,'NB-52 003',16 from dual union all
select 65550,'1994-S50','1994-07-22 00:00:00','military missile','success',1111,'M-2 STARS II','ODES','','','Target',849,'',1000 from dual union all
select 65551,'1994-S51','1994-07-28 22:39:00','suborbital rocket','success',860,'DLR A-NO-210','Echo 94 F-102 DUSTY','','','Ionos',29,'',140 from dual union all
select 65552,'1994-S52','1994-07-31 00:50:33','suborbital rocket','success',860,'DLR A-NO-211','Echo 94 F-103 DUSTY','','','Ionos',29,'',140 from dual union all
select 65553,'1994-S53','1994-08-08 00:00:00','suborbital rocket','success',1162,'','CEC Target','','','Target',1821,'',100 from dual union all
select 65554,'1994-S54','1994-08-12 00:53:00','suborbital rocket','success',860,'DLR A-NO-212','Echo 94 F-101 CONE','','','Ionos',29,'',140 from dual union all
select 65555,'1994-S55','1994-08-16 03:25:02','suborbital rocket','success',224,'NASA 36.117CL','EUVS Venus','','','UV Astron',1873,'',300 from dual union all
select 65556,'1994-S56','1994-08-18 00:00:00','suborbital rocket','success',1162,'','JTF-95 MSTI Target','','','Target',1821,'',100 from dual union all
select 65557,'1994-S57','1994-08-19 15:08:00','suborbital rocket','success',860,'NASA 31.102GE','GUARA MALTED/CADRE','','','Ionos',13,'',125 from dual union all
select 65558,'1994-S58','1994-08-20 02:18:00','suborbital rocket','success',860,'NASA 31.103GE','GUARA MALTED/CADRE','','','Ionos',13,'',125 from dual union all
select 65559,'1994-S59','1994-08-22 19:38:00','suborbital rocket','success',891,'NASA 30.41UP','CSHARP','','','Plasma/Education',1821,'',85 from dual union all
select 6556,'1960-A292','1960-12-07 00:00:00','military missile','success',771,'GM-56-1930','','','','Test',222,'',12 from dual union all
select 65560,'1994-S60','1994-08-24 13:39:00','suborbital rocket','success',860,'NASA 31.105GE','GUARA MALTED/CADRE','','','Ionos',13,'',127 from dual union all
select 65561,'1994-S61','1994-08-25 01:43:30','suborbital rocket','success',860,'NASA 31.104GE','GUARA MALTED/CADRE','','','Ionos',13,'',126 from dual union all
select 65562,'1994-S62','1994-09-07 14:41:00','military missile','success',907,'16PA (84-551)','Mk 21 RV x 8','GT 16PA','5F/5Tx2/SPO 5Tx5','OT',1700,'',1000 from dual union all
select 65563,'1994-S63','1994-09-09 13:33:50','suborbital rocket','success',228,'NASA 21.110GE','GUARA Electrojet','','','Ionos',13,'',120 from dual union all
select 65564,'1994-S64','1994-09-12 15:34:00','suborbital rocket','success',860,'NASA 31.107UU','CWAS 35','','','Aeronomy',1895,'',140 from dual union all
select 65565,'1994-S65','1994-09-15 00:00:00','military missile','failure',627,'3','''Ofeq','','','Sat?',1026,'',300 from dual union all
select 65566,'1994-S66','1994-09-15 15:00:01','suborbital rocket','success',860,'NASA 31.108UU','CWAS 36','','','Aeronomy',1895,'',140 from dual union all
select 65567,'1994-S67','1994-09-16 04:00:00','suborbital rocket','success',1109,'ST-735-2','FIH Test','','','Test',833,'',118 from dual union all
select 65568,'1994-S68','1994-09-21 13:56:00','suborbital rocket','success',228,'NASA 21.111GE','GUARA Electrojet','','','Ionos',13,'',120 from dual union all
select 65569,'1994-S69','1994-09-23 15:15:00','military missile','success',1434,'','15Zh58 K-32','','','Test',608,'',1000 from dual union all
select 6557,'1960-A293','1960-12-07 00:00:00','military missile','success',771,'GM-54-89','','','','Test',222,'',12 from dual union all
select 65570,'1994-S70','1994-09-23 21:15:00','suborbital rocket','success',867,'NASA 18.226UE','GUARA Sunset Elecdyn','','','Ionos',13,'',270 from dual union all
select 65571,'1994-S71','1994-09-23 21:17:00','suborbital rocket','success',867,'NASA 18.225UE','GUARA Sunset Elecdyn','','','Ionos',13,'',270 from dual union all
select 65572,'1994-S72','1994-09-24 21:15:00','suborbital rocket','success',867,'NASA 18.228UE','GUARA Sunset Elecdyn','','','Ionos',13,'',270 from dual union all
select 65573,'1994-S73','1994-09-24 21:17:00','suborbital rocket','success',867,'NASA 18.227UE','GUARA Sunset Elecdyn','','','Ionos',13,'',270 from dual union all
select 65574,'1994-S74','1994-10-04 18:58:30','suborbital rocket','success',224,'NASA 36.084UE','PIMS','','','Ionos',1873,'',300 from dual union all
select 65575,'1994-S75','1994-10-05 08:00:00','military missile','success',799,'','Mk 12/12A RV','FOT GT157GM','GLORY TRIP 157GM','OT',1699,'',1300 from dual union all
select 65576,'1994-S76','1994-10-06 13:22:00','suborbital rocket','failure',228,'NASA 21.112GE','GUARA Electrojet','','','Ionos',13,'',120 from dual union all
select 65577,'1994-S77','1994-10-14 22:55:00','suborbital rocket','success',231,'NASA 35.030IE','GUARA H.Alt Spread F','','','Ionos',13,'',956 from dual union all
select 65578,'1994-S78','1994-10-15 13:41:31','suborbital rocket','success',228,'NASA 21.113GE','GUARA Electrojet','','','Ionos',13,'',120 from dual union all
select 65579,'1994-S79','1994-10-22 20:09:03','military missile','unknown',1442,'DASO-33','','','','OT',419,'SSBN 726',1000 from dual union all
select 6558,'1960-A294','1960-12-07 00:20:00','suborbital rocket','failure',1194,'SL50','LUX-2/UCW-2','','','Ionos/Solar X',1839,'',35 from dual union all
select 65580,'1994-S80','1994-11-03 18:45:00','suborbital rocket','success',224,'NASA 36.124AE','NCAR/CU-5','','','Ionos/Solar UV',1873,'',231 from dual union all
select 65581,'1994-S81','1994-11-03 19:15:00','suborbital rocket','success',224,'NASA 36.091US','MSSTA II','','','Solar X',1873,'',272 from dual union all
select 65582,'1994-S82','1994-11-10 07:30:00','military missile','success',1434,'','15Zh58 UBP-14','','','Test',608,'',1000 from dual union all
select 65583,'1994-S83','1994-11-22 15:40:00','suborbital rocket','success',224,'NASA 36.103NP','','','','Test?',1873,'',300 from dual union all
select 65584,'1994-S84','1994-11-24 10:20:00','suborbital rocket','success',1073,'S-310-23','NNP/AIR/NID/NEL','','','Aeron',29,'',200 from dual union all
select 65585,'1994-S85','1994-11-30 10:30:00','suborbital rocket','success',1212,'DLR K-GR-213','DLR TEXUS 33','','','Micrograv',416,'',267 from dual union all
select 65586,'1994-S86','1994-12-01 00:45:03','military missile','success',1443,'DASO-13','','','','OT',419,'SSBN 740',1000 from dual union all
select 65587,'1994-S87','1994-12-01 21:39:00','suborbital rocket','success',1074,'S-520-21','','','','Auroral',29,'',344 from dual union all
select 65588,'1994-S88','1994-12-03 07:45:00','suborbital rocket','success',224,'NASA 36.050UG','WISP 1','','','UV Astronomy',1873,'',300 from dual union all
select 65589,'1994-S89','1994-12-07 16:11:40','military missile','success',1442,'FCET-45','','','','OT',420,'SSBN 730',1000 from dual union all
select 6559,'1960-A295','1960-12-09 19:52:40','suborbital spaceplane','success',1520,'1-19-32','Ball nose, Stabil.','','','Test',379,'NB-52 008',15 from dual union all
select 65590,'1994-S90','1994-12-07 16:11:57','military missile','success',1442,'FCET-45','','','','OT',420,'SSBN 730',1000 from dual union all
select 65591,'1994-S91','1994-12-08 00:00:00','suborbital rocket','success',213,'Starfire 1/F7','CSAR-2','','','Micrograv',1874,'',254 from dual union all
select 65592,'1994-S92','1994-12-17 02:55:00','suborbital rocket','success',224,'NASA 36.102UG','','','','UV Astronomy',1873,'',300 from dual union all
select 65593,'1994-S93','1994-12-20 08:50:00','military missile','success',1436,'1L','15Zh65 LKI-1','','','Test',607,'',1000 from dual union all
select 65594,'1994-S94','1994-05-12 00:00:00','military missile','success',580,'','Sanaa (total 30)','','','Test',1910,'',100 from dual union all
select 65595,'1994-S95','1994-06-01 00:00:00','military missile','success',580,'','Sanaa','','','Test',1910,'',100 from dual union all
select 65596,'1994-S96','1994-07-01 00:00:00','military missile','success',580,'','Sanaa','','','Test',1910,'',100 from dual union all
select 65597,'1994-S97','1994-12-07 16:11:57','military missile','success',1442,'FCET-45','','','','OT',420,'SSBN 730',1000 from dual union all
select 65598,'1994-S98','1994-12-07 16:11:57','military missile','success',1442,'FCET-45','','','','OT',420,'SSBN 730',1000 from dual union all
select 65599,'1994-S99','1994-02-25 00:00:00','military missile','success',117,'LEAP Target','LEAP FTV-TD','','','Target',1821,'',300 from dual union all
select 656,'1946-S03','1946-06-13 23:40:00','suborbital rocket','success',1470,'5','UVS/Ionos prop.','','','Solar UV/Ionos',1863,'',117 from dual union all
select 6560,'1960-A296','1960-12-09 22:15:00','suborbital rocket','failure',839,'NASA 10.11CA','GCA Release 9 Na','','','Aeronomy',1821,'',24 from dual union all
select 65600,'1994-W01','1994-01-13 02:00:00','suborbital rocket','success',761,'MT135P-R877','','','','Meteo',1332,'',50 from dual union all
select 65601,'1994-W02','1994-09-06 02:00:00','suborbital rocket','success',759,'MT-135-60','NNP-O3','','','Aeron',839,'',56 from dual union all
select 65602,'1994-W03','1994-09-07 02:00:00','suborbital rocket','success',759,'MT-135-61','NNP-O3','','','Aeron',839,'',56 from dual union all
select 65603,'1994-W04','1994-09-15 02:00:00','suborbital rocket','success',759,'MT-135-62','NNP-O3','','','Aeron',839,'',55 from dual union all
select 65604,'1994-W05','1994-09-18 02:00:00','suborbital rocket','success',759,'MT-135-63','NNP-O3','','','Aeron',839,'',53 from dual union all
select 65605,'1994-W06','1994-01-26 02:00:00','suborbital rocket','success',761,'MT135P-R878','','','','Meteo',1332,'',60 from dual union all
select 65606,'1994-W07','1994-02-03 02:00:00','suborbital rocket','success',761,'MT135P-R879','','','','Meteo',1332,'',60 from dual union all
select 65607,'1994-W08','1994-02-16 02:07:00','suborbital rocket','success',761,'MT135P-R880','','','','Meteo',1332,'',60 from dual union all
select 65608,'1994-W09','1994-03-02 02:20:00','suborbital rocket','success',761,'MT135P-R881','','','','Meteo',1332,'',60 from dual union all
select 65609,'1994-W10','1994-03-09 02:00:00','suborbital rocket','success',761,'MT135P-R882','','','','Meteo',1332,'',60 from dual union all
select 6561,'1960-A297','1960-12-10 00:00:00','military missile','failure',1469,'','','','','ABM Test',182,'',10 from dual union all
select 65610,'1994-W11','1994-03-16 02:00:00','suborbital rocket','success',761,'MT135P-R883','','','','Meteo',1332,'',60 from dual union all
select 65611,'1994-W12','1994-03-24 02:10:00','suborbital rocket','success',761,'MT135P-R884','','','','Meteo',1332,'',60 from dual union all
select 65612,'1994-W13','1994-03-30 02:03:00','suborbital rocket','success',761,'MT135P-R885','','','','Meteo',1332,'',60 from dual union all
select 65613,'1994-W14','1994-04-13 02:28:00','suborbital rocket','success',761,'MT135P-R886','','','','Meteo',1332,'',60 from dual union all
select 65614,'1994-W15','1994-04-20 02:10:00','suborbital rocket','success',761,'MT135P-R887','','','','Meteo',1332,'',60 from dual union all
select 65615,'1994-W16','1994-04-27 02:10:00','suborbital rocket','success',761,'MT135P-R888','','','','Meteo',1332,'',60 from dual union all
select 65616,'1994-W17','1994-05-11 02:03:00','suborbital rocket','success',761,'MT135P-R889','','','','Meteo',1332,'',60 from dual union all
select 65617,'1994-W18','1994-05-25 02:15:00','suborbital rocket','success',761,'MT135P-R890','','','','Meteo',1332,'',60 from dual union all
select 65618,'1994-W19','1994-06-08 02:03:00','suborbital rocket','success',761,'MT135P-R891','','','','Meteo',1332,'',60 from dual;
insert into LAUNCH
select 65619,'1994-W20','1994-06-15 02:20:00','suborbital rocket','success',761,'MT135P-R892','','','','Meteo',1332,'',60 from dual union all
select 6562,'1960-A298','1960-12-11 10:30:00','suborbital rocket','success',725,'','','','','Meteo',1254,'SHIP-SOV',46 from dual union all
select 65620,'1994-W21','1994-06-29 02:00:00','suborbital rocket','success',761,'MT135P-R893','','','','Meteo',1332,'',60 from dual union all
select 65621,'1994-W22','1994-07-13 02:20:00','suborbital rocket','success',761,'MT135P-R894','','','','Meteo',1332,'',60 from dual union all
select 65622,'1994-W23','1994-07-20 02:08:00','suborbital rocket','success',761,'MT135P-R895','','','','Meteo',1332,'',60 from dual union all
select 65623,'1994-W24','1994-08-03 02:05:00','suborbital rocket','success',761,'MT135P-R896','','','','Meteo',1332,'',60 from dual union all
select 65624,'1994-W25','1994-08-17 02:05:00','suborbital rocket','success',761,'MT135P-R897','','','','Meteo',1332,'',60 from dual union all
select 65625,'1994-W26','1994-08-24 02:03:00','suborbital rocket','success',761,'MT135P-R898','','','','Meteo',1332,'',60 from dual union all
select 65626,'1994-W27','1994-09-08 02:03:00','suborbital rocket','success',761,'MT135P-R899','','','','Meteo',1332,'',60 from dual union all
select 65627,'1994-W28','1994-09-21 02:00:00','suborbital rocket','success',761,'MT135P-R900','','','','Meteo',1332,'',60 from dual union all
select 65628,'1994-W29','1994-09-28 02:10:00','suborbital rocket','success',761,'MT135P-R901','','','','Meteo',1332,'',60 from dual union all
select 65629,'1994-W30','1994-10-05 02:03:00','suborbital rocket','success',761,'MT135P-R902','','','','Meteo',1332,'',60 from dual union all
select 6563,'1960-A299','1960-12-12 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1255,'SHIP-SOV',48 from dual union all
select 65630,'1994-W31','1994-10-12 02:00:00','suborbital rocket','success',761,'MT135P-R903','','','','Meteo',1332,'',60 from dual union all
select 65631,'1994-W32','1994-10-19 02:03:00','suborbital rocket','success',761,'MT135P-R904','','','','Meteo',1332,'',60 from dual union all
select 65632,'1994-W33','1994-10-26 02:00:00','suborbital rocket','success',761,'MT135P-R905','','','','Meteo',1332,'',60 from dual union all
select 65633,'1994-W34','1994-11-09 02:00:00','suborbital rocket','success',761,'MT135P-R906','','','','Meteo',1332,'',60 from dual union all
select 65634,'1994-W35','1994-11-16 02:15:00','suborbital rocket','success',761,'MT135P-R907','','','','Meteo',1332,'',60 from dual union all
select 65635,'1994-W36','1994-11-23 02:00:00','suborbital rocket','success',761,'MT135P-R908','','','','Meteo',1332,'',60 from dual union all
select 65636,'1994-W37','1994-12-07 02:00:00','suborbital rocket','success',761,'MT135P-R909','','','','Meteo',1332,'',60 from dual union all
select 65637,'1994-W38','1994-12-15 02:00:00','suborbital rocket','success',761,'MT135P-R910','','','','Meteo',1332,'',60 from dual union all
select 65638,'1994-W39','1994-12-22 02:00:00','suborbital rocket','success',761,'MT135P-R911','','','','Meteo',1332,'',60 from dual union all
select 65639,'1994-W40','1994-12-28 02:00:00','suborbital rocket','success',761,'MT135P-R912','','','','Meteo',1332,'',60 from dual union all
select 6564,'1960-A30','1960-02-03 11:21:00','suborbital rocket','success',725,'','','','','Meteo',1215,'VOKV',43 from dual union all
select 65640,'1994-Y01','1994-11-09 14:45:00','suborbital rocket','success',1284,'','CWS-34','','','Aeron',1821,'',100 from dual union all
select 65641,'1994-Y02','1994-11-25 23:00:00','suborbital rocket','success',1284,'','CWS-58','','','Aeron',1821,'',100 from dual union all
select 65642,'1994-Y03','1994-08-16 14:22:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1874','','','Aeron',13,'',90 from dual union all
select 65643,'1994-Y04','1994-08-17 15:40:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1876','','','Aeron',13,'',90 from dual union all
select 65644,'1994-Y05','1994-08-18 14:08:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1877','','','Aeron',13,'',90 from dual union all
select 65645,'1994-Y06','1994-08-19 14:24:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1878','','','Aeron',13,'',90 from dual union all
select 65646,'1994-Y07','1994-08-19 20:00:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1879','','','Aeron',13,'',90 from dual union all
select 65647,'1994-Y08','1994-08-20 03:56:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1881','','','Aeron',13,'',90 from dual union all
select 65648,'1994-Y09','1994-08-20 08:00:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1882','','','Aeron',13,'',90 from dual union all
select 65649,'1994-Y10','1994-08-20 08:35:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1883','','','Aeron',13,'',90 from dual union all
select 6565,'1960-A300','1960-12-15 00:00:00','atmospheric rocket','success',620,'J1','','','','Test',1836,'',24 from dual union all
select 65650,'1994-Y11','1994-08-20 14:32:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1892','','','Aeron',13,'',90 from dual union all
select 65651,'1994-Y12','1994-08-21 14:39:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1894','','','Aeron',13,'',90 from dual union all
select 65652,'1994-Y13','1994-08-22 16:27:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1896','','','Aeron',13,'',90 from dual union all
select 65653,'1994-Y14','1994-08-23 13:46:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1897','','','Aeron',13,'',90 from dual union all
select 65654,'1994-Y15','1994-08-24 13:52:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1898','','','Aeron',13,'',90 from dual union all
select 65655,'1994-Y16','1994-08-24 20:02:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1899','','','Aeron',13,'',90 from dual union all
select 65656,'1994-Y17','1994-08-25 02:44:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1901','','','Aeron',13,'',90 from dual union all
select 65657,'1994-Y18','1994-08-25 09:26:00','suborbital rocket','success',1502,'Sphere','GUARA MET NRO1902','','','Aeron',13,'',90 from dual union all
select 65658,'1994-Y19','1994-08-17 00:00:00','suborbital rocket','failure',1502,'Sphere','GUARA MET NRO1875','','','Aeron',13,'',90 from dual union all
select 65659,'1994-Y20','1994-08-20 00:00:00','suborbital rocket','failure',1502,'Sphere','GUARA MET NRO1880','','','Aeron',13,'',90 from dual union all
select 6566,'1960-A301','1960-12-15 00:00:00','suborbital spaceplane','failure',1520,'1- A-33','','','','Test',379,'NB-52 008',14 from dual union all
select 65660,'1994-Y21','1994-08-22 00:00:00','suborbital rocket','failure',1502,'Sphere','GUARA MET NRO1895','','','Aeron',13,'',90 from dual union all
select 65661,'1994-Y22','1994-08-25 00:00:00','suborbital rocket','failure',1502,'Sphere','GUARA MET NRO1900','','','Aeron',13,'',90 from dual union all
select 65662,'1995-001','1995-01-10 06:18:02','orbital','success',174,'AC-113','Intelsat 704','INTELSAT 704','','Sat',269,'',null from dual union all
select 65663,'1995-002','1995-01-24 03:54:22','orbital','success',680,'53735-620','Tsikada/Astrid','Tsikada','','Sat',456,'',null from dual union all
select 65664,'1995-003','1995-01-29 01:25:01','orbital','success',172,'AC-112','USA 108 (UHF F4)','UFO F4 EHF','','Sat',268,'',null from dual union all
select 65665,'1995-004','1995-02-03 05:22:04','orbital','success',1243,'STS-63','SH-03/Mir Rendezvous','OV-103 Shuttle/Mir','','Sat',922,'MLP2',null from dual union all
select 65666,'1995-005','1995-02-15 16:48:28','orbital','success',1238,'Ya15000-641','Progress M-26','Progress 7K-TGM No. 226','','Sat',480,'',null from dual union all
select 65667,'1995-006','1995-02-16 17:39:59','orbital','success',1238,'Ya15000-051','Foton','Foton No. 10','','Sat',475,'',null from dual union all
select 65668,'1995-007','1995-03-02 06:38:13','orbital','success',1243,'STS-67','Spacelab Astro 2','OV-105 Spacelab','','Sat',921,'MLP1',null from dual union all
select 65669,'1995-008','1995-03-02 13:00:00','orbital','success',680,'65065-627','Kosmos-2306','Romb','','Sat',456,'',null from dual union all
select 6567,'1960-A302','1960-12-16 00:00:00','military missile','success',268,'10-05','','','','Test',1752,'',15 from dual union all
select 65670,'1995-009','1995-03-07 09:23:44','orbital','success',958,'370-02','Kosmos-2307/2309','Uragan No. 65L','','Sat',553,'',null from dual union all
select 65671,'1995-010','1995-03-14 06:11:34','orbital','success',1242,'V 15000-075','Soyuz TM-21','Soyuz 7K-STM No. 70','','Sat',480,'',null from dual union all
select 65672,'1995-011','1995-03-18 08:01:00','orbital','success',525,'H-II-3F','SFU/GMS-5','SFU','','Sat',1477,'',null from dual union all
select 65673,'1995-012','1995-03-22 04:09:03','orbital','success',680,'47135-622','Kosmos-2310','Parus','','Sat',456,'',null from dual union all
select 65674,'1995-013','1995-03-22 06:18:01','orbital','success',174,'AC-115','Intelsat 705','INTELSAT 705','','Sat',269,'',null from dual union all
select 65675,'1995-014','1995-03-22 16:44:59','orbital','success',1241,'77031  -507','Kosmos-2311','Kobal''t','','Sat',474,'',null from dual union all
select 65676,'1995-015','1995-03-24 14:05:00','orbital','success',155,'45E','USA 109 (DMSP)','DMSP Block 5D-2 S-13','','Sat',1776,'',null from dual union all
select 65677,'1995-016','1995-03-28 23:14:19','orbital','success',107,'V71 L443a','Hot Bird 1/Bras. B2','Brasilsat B2','','Sat',345,'',null from dual union all
select 65678,'1995-017','1995-04-03 13:48:00','orbital','success',910,'F8','Orbcomm 1/2','Orbcomm FM1','','Sat',1718,'L-1011',null from dual union all
select 65679,'1995-018','1995-04-05 11:16:00','orbital','success',1174,'3','''Ofeq-3','''Ofeq-3','','Sat',1026,'',null from dual union all
select 6568,'1960-A303','1960-12-16 00:00:00','military missile','success',764,'56-2889 (8)','','','','Test',240,'',12 from dual union all
select 65680,'1995-019','1995-04-07 23:47:00','orbital','success',173,'AC-114','AMSC 1','AMSC 1','','Sat',268,'',null from dual union all
select 65681,'1995-020','1995-04-09 19:34:12','orbital','success',1238,'V 15000-646','Progress M-27','Progress 7K-TGM No. 227','','Sat',480,'',null from dual union all
select 65682,'1995-021','1995-04-21 01:44:02','orbital','success',95,'V72 L444','ERS 2','ERS-2','','Sat',345,'',null from dual union all
select 65683,'1995-022','1995-05-14 13:45:01','orbital','success',1409,'K-23/TC-17  (45E-8)','USA 110      [RH7]','ORION 3','','Sat',274,'',null from dual union all
select 65684,'1995-023','1995-05-17 06:34:00','orbital','success',108,'V73 L445','Intelsat 706','INTELSAT 706','','Sat',345,'',null from dual union all
select 65685,'1995-024','1995-05-20 03:33:22','orbital','success',949,'378-02','Spektr','TsM-O No. 17301','','Sat',587,'',null from dual union all
select 65686,'1995-025','1995-05-23 05:52:02','orbital','success',171,'AC-77','GOES 9','GOES J','','Sat',269,'',null from dual union all
select 65687,'1995-026','1995-05-24 20:10:09','orbital','success',803,'77017  -573','Kosmos-2312','Oko','','Sat',462,'',null from dual union all
select 65688,'1995-027','1995-05-31 15:27:01','orbital','success',172,'AC-116','USA 111 (UHF F5)','UHF F/O F5-EHF','','Sat',268,'',null from dual union all
select 65689,'1995-028','1995-06-08 04:43:00','orbital','success',1446,'51041-505','Kosmos-2313','US-PM','','Sat',591,'',null from dual union all
select 6569,'1960-A304','1960-12-17 00:00:00','atmospheric rocket','success',1473,'','','','','Test',664,'',20 from dual union all
select 65690,'1995-029','1995-06-10 00:24:00','orbital','success',102,'V74 L446b','DBS 3','DBS 3','','Sat',345,'',null from dual union all
select 65691,'1995-030','1995-06-27 19:32:18','orbital','success',1243,'STS-71','Shuttle/Mir-1','OV-104 Shuttle/Mir','','Sat',921,'MLP3',null from dual union all
select 65692,'1995-031','1995-06-28 18:25:00','orbital','success',1241,'77031  -506','Kosmos-2314','Kobal''t','','Sat',474,'',null from dual union all
select 65693,'1995-032','1995-07-05 03:09:03','orbital','success',680,'53765-638','Kosmos-2315','Nadezhda 17F118','','Sat',456,'',null from dual union all
select 65694,'1995-033','1995-07-07 16:23:34','orbital','success',96,'V75 L447','Helios 1A/ASAP 6','Helios 1A','','Sat',345,'',null from dual union all
select 65695,'1995-034','1995-07-10 12:38:03','orbital','success',1409,'K-19/TC-8   (45E-5)','USA 112      [JS9]','TRUMPET 2','','Sat',275,'',null from dual union all
select 65696,'1995-035','1995-07-13 13:41:55','orbital','success',1243,'STS-70','TDRS-G','OV-103 IUS deploy','','Sat',922,'MLP2',null from dual union all
select 65697,'1995-036','1995-07-20 03:04:41','orbital','success',1238,'V 15000-643','Progress M-28','Progress 7K-TGM No. 228','','Sat',480,'',null from dual union all
select 65698,'1995-037','1995-07-24 15:52:10','orbital','success',958,'374-01','Kosmos-2316/2318','Uragan No. 80L','','Sat',553,'',null from dual union all
select 65699,'1995-038','1995-07-31 23:30:02','orbital','success',173,'AC-118','USA 113 (DSCS 3B-7)','DSCS III B-7','','Sat',268,'',null from dual union all
select 657,'1946-S04','1946-06-28 19:25:00','suborbital rocket','success',1470,'6','UVS/Ionos prop.','','','Solar UV',1863,'',108 from dual union all
select 6570,'1960-A305','1960-12-19 00:00:00','atmospheric rocket','success',1473,'','','','','Test',664,'',20 from dual union all
select 65700,'1995-039','1995-08-02 23:59:11','orbital','success',807,'N 15000-294 294','Interbol-1','SO-M2 No. 511','','Sat',474,'',null from dual union all
select 65701,'1995-040','1995-08-03 22:58:00','orbital','success',99,'V76 L448b','PAS 4','Panamsat K3','','Sat',345,'',null from dual union all
select 65702,'1995-041','1995-08-05 11:10:00','orbital','failure',422,'D228','Koreasat 1','Koreasat 1','','Sat',234,'',null from dual union all
select 65703,'1995-042','1995-08-09 02:21:00','orbital','success',805,'77031  -674','Molniya-3','Molniya-3 No. 59','','Sat',474,'',null from dual union all
select 65704,'1995-043','1995-08-29 00:53:02','orbital','success',174,'AC-117','JCSAT 3','JCSAT 3','','Sat',269,'',null from dual union all
select 65705,'1995-044','1995-08-29 06:41:00','orbital','success',110,'V77 L449b','N-Star a','N-STAR a','','Sat',345,'',null from dual union all
select 65706,'1995-045','1995-08-30 19:33:00','orbital','success',958,'369-02','Kosmos-2319','Geizer No. 20L','','Sat',553,'',null from dual union all
select 65707,'1995-046','1995-08-31 06:49:59','orbital','success',1448,'45025-801','Sich','Okean-O1 No. 8/NKhM 10','','Sat',470,'',null from dual union all
select 65708,'1995-047','1995-09-03 09:00:23','orbital','success',1242,'V 15000-076','Soyuz TM-22','Soyuz 7K-STM No. 71','','Sat',480,'',null from dual;
insert into LAUNCH
select 65709,'1995-048','1995-09-07 15:09:00','orbital','success',1243,'STS-69','WSF/Spartan/IEH-1','OV-105 Research/FF','','Sat',921,'MLP1',null from dual union all
select 6571,'1960-A306','1960-12-20 00:00:00','military missile','failure',1419,'J-9','Mk 4-4 RV','','','Test',239,'',100 from dual union all
select 65710,'1995-049','1995-09-24 00:06:00','orbital','success',99,'V78 L450b','Telstar 402R','Telstar 402R','','Sat',345,'',null from dual union all
select 65711,'1995-050','1995-09-26 11:20:00','orbital','success',1238,'V 15000-053','Resurs-F2','Resurs-F2 No. 10','','Sat',475,'',null from dual union all
select 65712,'1995-051','1995-09-29 04:25:00','orbital','success',1238,'Ya15000-642','Kosmos-2320','Neman','','Sat',562,'',null from dual union all
select 65713,'1995-052','1995-10-06 03:23:10','orbital','failure',680,'47165-625','Kosmos-2321','Parus','','Sat',456,'',null from dual union all
select 65714,'1995-053','1995-10-08 18:50:40','orbital','success',1238,'V 15000-645','Progress M-29','Progress 7K-TGM No. 229','','Sat',480,'',null from dual union all
select 65715,'1995-054','1995-10-11 16:26:00','orbital','success',958,'386-01','Luch-1','Gelios No. 12L','','Sat',587,'',null from dual union all
select 65716,'1995-055','1995-10-19 00:38:00','orbital','success',99,'V79 L451c','Astra 1E','Astra 1E','','Sat',345,'',null from dual union all
select 65717,'1995-056','1995-10-20 13:53:00','orbital','success',1243,'STS-73','Spacelab USML-2','OV-102 Spacelab','','Sat',922,'MLP3',null from dual union all
select 65718,'1995-057','1995-10-22 08:00:02','orbital','success',172,'AC-119','USA 114 (UHF F6)','UFO F6-EHF','','Sat',268,'',null from dual union all
select 65719,'1995-058','1995-10-31 20:19:00','orbital','success',1537,'45057-903','Kosmos-2322','Tselina-2','','Sat',567,'',null from dual union all
select 6572,'1960-A307','1960-12-23 00:00:00','military missile','failure',1469,'','','','','ABM Test',182,'',10 from dual union all
select 65720,'1995-059','1995-11-04 14:22:00','orbital','success',417,'D229','Radarsat','Radarsat','','Sat',1724,'',null from dual union all
select 65721,'1995-060','1995-11-06 05:15:01','orbital','success',1409,'K-21/TC-13  (45E-7)','USA 115 (Milstar 2)','Milstar DFS-2','','Sat',274,'',null from dual union all
select 65722,'1995-061','1995-11-12 12:30:43','orbital','success',1243,'STS-74','Shuttle/Mir-2','OV-104 Shuttle/Mir','','Sat',921,'MLP2',null from dual union all
select 65723,'1995-062','1995-11-17 01:20:00','orbital','success',110,'V80 L452b','ISO','ISO','','Sat',345,'',null from dual union all
select 65724,'1995-063','1995-11-17 14:25:00','orbital','success',958,'384-01','Gals 2','Gals No. 12L','','Sat',553,'',null from dual union all
select 65725,'1995-064','1995-11-28 11:30:05','orbital','success',305,'Y7       (38)','Asiasat 2','Asiasat 2','','Sat',1903,'',null from dual union all
select 65726,'1995-065','1995-12-02 08:08:01','deep space','success',174,'AC-121','SOHO','SOHO','','Sat',269,'',null from dual union all
select 65727,'1995-066','1995-12-05 21:18:00','orbital','success',1415,'K-15        (45J-3)','USA 116      [IC5]','Improved CRYSTAL 2105','','Sat',1777,'',null from dual union all
select 65728,'1995-067','1995-12-06 23:23:16','orbital','success',105,'V81 L453','Insat 2C/Telecom 2C','Telecom 2C','','Sat',345,'',null from dual union all
select 65729,'1995-068','1995-12-14 06:10:31','orbital','success',958,'378-01','Kosmos-2323/2325','Uragan No. 82L','','Sat',553,'',null from dual union all
select 6573,'1960-A308','1960-12-31 00:00:00','military missile','failure',1469,'','','','','ABM Test',182,'',10 from dual union all
select 65730,'1995-069','1995-12-15 00:23:01','orbital','success',173,'AC-120','Galaxy 3R','Galaxy 3R','','Sat',268,'',null from dual union all
select 65731,'1995-070','1995-12-18 14:31:35','orbital','success',1238,'V 15000-647','Progress M-30','Progress 7K-TGM No. 230','','Sat',480,'',null from dual union all
select 65732,'1995-071','1995-12-20 00:52:14','orbital','success',1446,'67041-506','Kosmos-2326','US-PU/Konus-A','','Sat',591,'',null from dual union all
select 65733,'1995-072','1995-12-28 06:45:18','orbital','success',803,'V 15000-040','IRS-1C','IRS-1C','','Sat',562,'',null from dual union all
select 65734,'1995-073','1995-12-28 11:50:00','orbital','success',305,'Y8       (39)','Echostar 1','Echostar 1','','Sat',1903,'',null from dual union all
select 65735,'1995-074','1995-12-30 13:48:00','orbital','success',417,'D230','XTE','XTE','','Sat',233,'',null from dual union all
select 65736,'1995-A01','1995-03-04 10:20:00','suborbital rocket','success',1335,'Hughes LEAP','FTV-3','','','Interceptor',114,'CG-20',30 from dual union all
select 65737,'1995-A02','1995-03-28 00:00:00','suborbital rocket','success',1335,'Hughes LEAP','FTV-4','','','Interceptor',114,'CG-20',30 from dual union all
select 65738,'1995-A03','1995-05-16 15:40:02','test rocket','success',345,'','','','','Test',1895,'',1 from dual union all
select 65739,'1995-A04','1995-06-12 14:38:00','test rocket','success',345,'','','','','Test',1895,'',1 from dual union all
select 6574,'1960-A309','1960-03-28 00:00:00','suborbital rocket','success',665,'K-8D-1','','','','Test',7,'',20 from dual union all
select 65740,'1995-A05','1995-07-07 13:02:00','test rocket','success',345,'','','','','Test',1895,'',2 from dual union all
select 65741,'1995-A06','1995-07-30 00:00:00','military missile','success',119,'F1','','','','Test',1026,'',20 from dual union all
select 65742,'1995-A07','1995-09-17 07:30:00','suborbital rocket','failure',1074,'S-520-20','Lunar Penetrator test','','','Tech',833,'',1 from dual union all
select 65743,'1995-F01','1995-01-25 22:40:00','orbital','failure',305,'Y6       (37)','Apstar 2','Apstar 2','','Sat',1903,'',1 from dual union all
select 65744,'1995-F02','1995-03-28 10:00:00','orbital','failure',1259,'','Techsat/Unamsat','Techsat 1','','Sat',605,'',670 from dual union all
select 65745,'1995-F03','1995-06-22 19:58:00','orbital','failure',911,'F9','STEP 3','STEP 3','','Sat',1718,'L-1011',144 from dual union all
select 65746,'1995-F04','1995-08-15 22:30:00','orbital','failure',688,'DLV','Gemstar','Vitasat 1','','Sat',1780,'',147 from dual union all
select 65747,'1995-F05','1995-10-23 22:03:00','orbital','failure',336,'F1','Meteor 1','Meteor SM','','Sat',1796,'',10 from dual union all
select 65748,'1995-M01','1995-01-01 00:00:00','suborbital rocket','success',1047,'','','','','Test',932,'',70 from dual union all
select 65749,'1995-M02','1995-05-12 21:00:00','suborbital rocket','success',1284,'Dart','High School Payload','','','Education',1821,'',73 from dual union all
select 6575,'1960-A31','1960-02-04 00:00:00','suborbital spaceplane','failure',1520,'2-A-10','','','','Test',379,'NB-52',14 from dual union all
select 65750,'1995-S01','1995-01-01 00:00:00','military missile','success',350,'','','','','Test',1296,'',100 from dual union all
select 65751,'1995-S02','1995-01-19 22:48:56','military missile','success',1443,'FCET-12','','','','OT',420,'SSBN 739',1000 from dual union all
select 65752,'1995-S03','1995-01-19 22:49:12','military missile','success',1443,'FCET-12','','','','OT',420,'SSBN 739',1000 from dual union all
select 65753,'1995-S04','1995-01-19 08:01:00','military missile','success',907,'17PA (84-548)','Mk 21 RV x 3','GT 17PA','5F/5Tx2','OT',1697,'',1000 from dual union all
select 65754,'1995-S05','1995-01-23 12:30:00','suborbital rocket','success',1074,'S-520-17','','','','Submm Astron',833,'',346 from dual union all
select 65755,'1995-S06','1995-01-25 06:24:48','suborbital rocket','success',237,'NASA 40.006IE','SCIFER','','','Ionos',28,'',1453 from dual union all
select 65756,'1995-S07','1995-01-28 16:00:00','suborbital rocket','success',1074,'S-520-19','EUV Telescopes','','','EUV Astron',833,'',330 from dual union all
select 65757,'1995-S08','1995-02-01 00:00:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT158GM','GLORY TRIP 158GM','OT',1705,'',1300 from dual union all
select 65758,'1995-S09','1995-02-02 15:27:20','suborbital rocket','success',211,'NASA 27.138CE','Aria-II','','','Ionos',1047,'',340 from dual union all
select 65759,'1995-S10','1995-02-02 15:51:41','suborbital rocket','success',867,'NASA 18.230UE','','','','Ionos',1047,'',270 from dual union all
select 6576,'1960-A310','1960-01-15 00:00:00','suborbital rocket','success',715,'Loki I','','','','Aeron',1057,'',37 from dual union all
select 65760,'1995-S11','1995-02-07 00:00:00','military missile','success',1262,'S9','BTTV-8 THAAD Demo','','','Target',1892,'',200 from dual union all
select 65761,'1995-S12','1995-02-12 00:00:00','military missile','success',117,'LEAP Target','','','','Target',1821,'',300 from dual union all
select 65762,'1995-S13','1995-02-14 00:00:00','military missile','success',757,'1','','','','Test',206,'S616',800 from dual union all
select 65763,'1995-S14','1995-02-24 10:21:46','suborbital rocket','success',237,'NASA 40.007UE','AMICIST','','','Auroral',1047,'',900 from dual union all
select 65764,'1995-S15','1995-03-04 10:15:00','military missile','success',117,'','LEAP FTV-3 Target','','','Target',1821,'',300 from dual union all
select 65765,'1995-S16','1995-03-06 20:07:32','military missile','success',1442,'FCET-46','-    190','','','OT',421,'SSBN 728',1000 from dual union all
select 65766,'1995-S17','1995-03-06 20:07:49','military missile','success',1442,'FCET-46','','','','OT',421,'SSBN 728',1000 from dual union all
select 65767,'1995-S18','1995-03-06 21:03:25','military missile','success',1442,'FCET-46','','','','OT',421,'SSBN 728',1000 from dual union all
select 65768,'1995-S19','1995-03-14 16:50:01','military missile','success',1442,'FCET-46A','','','','OT',421,'SSBN 728',1000 from dual union all
select 65769,'1995-S20','1995-03-14 16:50:20','military missile','success',1442,'FCET-46A','','','','OT',421,'SSBN 728',1000 from dual union all
select 6577,'1960-A311','1960-01-21 00:00:00','suborbital rocket','success',715,'Loki I','','','','Aeron',1057,'',33 from dual union all
select 65770,'1995-S21','1995-03-15 20:21:00','atmospheric rocket','success',860,'NASA 31.106NP','LIO/Destruct test','','','Test',1821,'',6 from dual union all
select 65771,'1995-S22','1995-03-17 00:00:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT159GM','GLORY TRIP 159GM','OT',1699,'',1300 from dual union all
select 65772,'1995-S23','1995-03-21 15:00:00','suborbital rocket','success',860,'NASA 31.112UU','CWAS 37','','','Aeronomy',1895,'',140 from dual union all
select 65773,'1995-S24','1995-03-21 19:11:31','suborbital rocket','success',224,'NASA 36.120UE','PIMS','','','Ionos',1873,'',153 from dual union all
select 65774,'1995-S25','1995-03-25 08:55:01','suborbital rocket','success',209,'NASA 35.031UE','','','','Ionos/release',1047,'',900 from dual union all
select 65775,'1995-S26','1995-03-27 15:40:00','suborbital rocket','success',860,'NASA 31.113UU','CWAS 38','','','Aeronomy',1895,'',140 from dual union all
select 65776,'1995-S27','1995-03-28 00:00:00','military missile','success',117,'','LEAP FTV-4 Target','','','Target',1821,'',300 from dual union all
select 65777,'1995-S28','1995-04-01 09:33:00','suborbital rocket','success',224,'NASA 36.104UL','','','','Aeronomy?',1873,'',156 from dual union all
select 65778,'1995-S29','1995-04-09 20:20:00','military missile','success',1443,'FCET-13','','','','OT',420,'SSBN 734',1000 from dual union all
select 65779,'1995-S30','1995-04-09 20:20:00','military missile','success',1443,'FCET-13','','','','OT',420,'SSBN 734',1000 from dual union all
select 6578,'1960-A312','1960-01-26 00:00:00','suborbital rocket','success',715,'Loki I','','','','Aeron',1057,'',39 from dual union all
select 65780,'1995-S31','1995-04-14 11:30:00','military missile','success',1434,'','15Zh58 UBP-15','','','Test',608,'',1000 from dual union all
select 65781,'1995-S32','1995-04-15 10:07:42','suborbital rocket','success',224,'NASA 36.137CL','EUVS Spica Occ.','','','UV Astron',1873,'',260 from dual union all
select 65782,'1995-S33','1995-04-18 18:00:00','suborbital rocket','success',224,'NASA 36.108DS','','','','Solar',1873,'',247 from dual union all
select 65783,'1995-S34','1995-04-21 15:04:00','military missile','success',1293,'FT-01','','','','Test',1895,'',100 from dual union all
select 65784,'1995-S35','1995-04-24 00:00:00','military missile','success',570,'1','BRV/Propulsion Test','','','Target',1860,'',275 from dual union all
select 65785,'1995-S36','1995-04-29 05:55:00','suborbital rocket','success',847,'DLR K-NO2-214','Mini-Texus 4','','','Micrograv',417,'',149 from dual union all
select 65786,'1995-S37','1995-05-02 05:35:00','suborbital rocket','success',847,'DLR K-NO2-215','Mini-Texus 3','','','Micrograv',417,'',125 from dual union all
select 65787,'1995-S38','1995-05-15 18:00:00','suborbital rocket','success',224,'NASA 36.123GS','SERTS-95 (06)','','','Solar',1873,'',191 from dual union all
select 65788,'1995-S39','1995-05-22 07:05:03','suborbital rocket','success',224,'NASA 36.092UH','','','','XR Astronomy',1873,'',300 from dual union all
select 65789,'1995-S40','1995-05-29 00:00:00','military missile','success',295,'','','','','Test',1494,'',1000 from dual union all
select 6579,'1960-A313','1960-01-28 00:00:00','suborbital rocket','success',715,'Loki I','','','','Aeron',1057,'',34 from dual union all
select 65790,'1995-S41','1995-06-06 22:00:00','suborbital rocket','success',1319,'NASA 33.064UE','','','','Ionos',1047,'',200 from dual union all
select 65791,'1995-S42','1995-06-06 22:10:00','military missile','success',1507,'VMF RF','Volan','','','Test?',201,'KALMAR',1270 from dual union all
select 65792,'1995-S43','1995-06-08 12:45:00','military missile','success',1453,'49242-83-430','15A35 UBP-44','GCh 15F355P7','','OT',596,'',1000 from dual union all
select 65793,'1995-S44','1995-06-14 00:00:00','military missile','success',907,'18PA (84-0544)','Mk 21 RV x 3','GT 18PA','5F/5Tx2','OT',1700,'',1000 from dual union all
select 65794,'1995-S45','1995-06-26 00:00:00','suborbital rocket','failure',1275,'','ETCE-11','','','Target',855,'',200 from dual union all
select 65795,'1995-S46','1995-06-29 00:00:00','suborbital rocket','success',1275,'','ETCE-12','','','Target',855,'',200 from dual union all
select 65796,'1995-S47','1995-07-01 00:00:00','military missile','success',354,'DF-21A','','','','Test',1494,'',500 from dual union all
select 65797,'1995-S48','1995-06-30 18:00:02','suborbital rocket','success',1334,'NASA 12.47WT','','','','Test',1821,'',300 from dual union all
select 65798,'1995-S49','1995-07-14 07:50:00','suborbital rocket','failure',605,'DLR K-O/I-216','DLR Test Orion 1','','','Test',417,'',7 from dual union all
select 65799,'1995-S50','1995-07-24 22:30:03','military missile','success',1443,'DASO-3','','','','OT',421,'UK S29',1000 from dual;
insert into LAUNCH
select 658,'1946-S05','1946-07-09 19:25:00','suborbital rocket','success',1470,'7','','','','CR/Ionos',1863,'',134 from dual union all
select 6580,'1960-A314','1960-02-02 00:00:00','suborbital rocket','success',715,'Loki I','','','','Aeron',1057,'',35 from dual union all
select 65800,'1995-S51','1995-07-26 09:33:31','suborbital rocket','success',860,'NASA 31.109UE','','','','Ionos',1047,'',140 from dual union all
select 65801,'1995-S52','1995-07-31 00:00:00','military missile','success',1293,'FT-02','','','','Test',1895,'',100 from dual union all
select 65802,'1995-S53','1995-08-08 08:20:35','suborbital rocket','success',860,'NASA 31.110UE','','','','Ionos',1047,'',140 from dual union all
select 65803,'1995-S54','1995-08-16 00:00:00','military missile','success',1262,'S10','MTTV-2 MTD-1(GPS)','','','Target',1873,'',200 from dual union all
select 65804,'1995-S55','1995-08-16 00:00:00','suborbital rocket','success',1029,'','','','','Test',1361,'',420 from dual union all
select 65805,'1995-S56','1995-08-22 22:25:04','military missile','success',1443,'DASO-4','','','','OT',421,'UK S29',1000 from dual union all
select 65806,'1995-S57','1995-08-24 22:00:00','suborbital rocket','success',1296,'TR-1A-4','','','','Micrograv',1476,'',270 from dual union all
select 65807,'1995-S58','1995-08-25 00:00:00','military missile','success',1057,'','','','','Test',1001,'TK-20',1000 from dual union all
select 65808,'1995-S59','1995-08-28 17:30:02','suborbital rocket','success',224,'NASA 36.138NP','','','','Microgravity',1873,'',300 from dual union all
select 65809,'1995-S60','1995-08-30 00:00:00','military missile','success',907,'19PA (90-0394)','Mk 21 RV x 3','GT 19PA','','OT',1697,'',1000 from dual union all
select 6581,'1960-A315','1960-02-11 00:00:00','suborbital rocket','success',715,'Loki I','','','','Aeron',1057,'',35 from dual union all
select 65810,'1995-S61','1995-09-02 01:13:00','suborbital rocket','success',224,'NASA 36.111UE','Thunderstorm III','','','Ionos',1821,'',300 from dual union all
select 65811,'1995-S62','1995-09-05 07:50:00','military missile','success',1436,'2L','15Zh65 LKI-2','','','Test',608,'',1000 from dual union all
select 65812,'1995-S63','1995-09-12 18:05:00','suborbital rocket','success',213,'NASA 36.131US','','','','Solar',1873,'',197 from dual union all
select 65813,'1995-S64','1995-10-02 00:00:00','military missile','success',570,'2','BRV/THAAD Demo','','','Target',1857,'',300 from dual union all
select 65814,'1995-S65','1995-10-08 10:36:00','suborbital rocket','success',605,'DLR K-O/I-217','DLR Test Orion 2','','','Test',417,'',74 from dual union all
select 65815,'1995-S66','1995-10-10 00:00:00','military missile','success',1434,'','15Zh58 UBP-16','','','Test',608,'',1000 from dual union all
select 65816,'1995-S67','1995-10-13 14:40:00','military missile','success',1262,'S11','BTTV-9 THAAD Target','','','Target',1892,'',200 from dual union all
select 65817,'1995-S68','1995-10-13 14:41:00','military missile','success',1293,'FT-03','','','','Test',1895,'',100 from dual union all
select 65818,'1995-S69','1995-10-25 13:13:00','suborbital rocket','success',224,'NASA 36.106UH','','','','XR Astronomy',1838,'',320 from dual union all
select 65819,'1995-S70','1995-10-28 18:00:00','suborbital rocket','success',224,'NASA 36.132UG','JHU FOT 12 (30 Dor)','','','UV Astronomy',1838,'',300 from dual union all
select 6582,'1960-A316','1960-02-01 00:00:00','suborbital rocket','success',88,'','PMR package','','','Aeron',1057,'',46 from dual union all
select 65820,'1995-S71','1995-10-29 21:00:03','military missile','success',1442,'DASO-34','','','','OT',421,'SSBN 727',1000 from dual union all
select 65821,'1995-S72','1995-11-02 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 65822,'1995-S73','1995-11-05 17:13:00','suborbital rocket','success',224,'NASA 36.116UG','','','','UV Astronomy',1838,'',300 from dual union all
select 65823,'1995-S74','1995-11-07 06:38:18','suborbital rocket','success',237,'NASA 40.009IE','OEDIPUS C','','','Ionos/Tether',1047,'',824 from dual union all
select 65824,'1995-S75','1995-11-10 00:00:00','military missile','success',354,'DF-21A','','','','Test',1494,'',1000 from dual union all
select 65825,'1995-S76','1995-11-14 17:04:00','suborbital rocket','success',224,'NASA 36.127UG','','','','UV Astronomy',1838,'',300 from dual union all
select 65826,'1995-S77','1995-11-19 15:30:00','suborbital rocket','success',224,'NASA 36.128UG','WISP 2','','','UV Astronomy',1838,'',300 from dual union all
select 65827,'1995-S78','1995-11-20 17:00:00','suborbital rocket','success',224,'NASA 36.126UG','','','','UV Astronomy',1838,'',300 from dual union all
select 65828,'1995-S79','1995-11-24 14:00:00','suborbital rocket','success',867,'NASA 18.231UE','','','','Ionos',1047,'',270 from dual union all
select 65829,'1995-S80','1995-11-27 08:03:00','suborbital rocket','success',867,'NASA 18.229UE','','','','Ionos',1047,'',270 from dual union all
select 6583,'1960-A317','1960-02-05 00:00:00','suborbital rocket','success',88,'','PMR package','','','Aeron',1057,'',45 from dual union all
select 65830,'1995-S81','1995-11-27 08:07:24','suborbital rocket','success',211,'NASA 27.139CE','','','','Ionos',1047,'',340 from dual union all
select 65831,'1995-S82','1995-11-28 09:42:00','suborbital rocket','success',772,'MAXUS 2','SSC MAXUS 2','','','Micrograv',407,'',705 from dual union all
select 65832,'1995-S83','1995-12-04 10:23:00','suborbital rocket','success',605,'SSC REXUS','SSC REXUS 1','','','Test',417,'',89 from dual union all
select 65833,'1995-S84','1995-12-04 11:20:01','suborbital rocket','success',211,'NASA 27.132UH','XQC F1','','','XR Astronomy',1874,'',220 from dual union all
select 65834,'1995-S85','1995-12-07 20:47:00','military missile','success',1443,'DASO-14','ENTB RV','','','OT',420,'SSBN 741',1000 from dual union all
select 65835,'1995-S86','1995-12-13 12:00:00','military missile','success',1262,'','BTTV-10 THAAD Target','','','Target',1892,'',200 from dual union all
select 65836,'1995-S87','1995-12-13 12:05:00','military missile','success',1293,'FT-04','','','','ABM Test',1895,'',100 from dual union all
select 65837,'1995-S88','1995-12-19 01:17:00','suborbital rocket','success',1219,'XV-26','','','','Ionos',13,'',557 from dual union all
select 65838,'1995-S89','1995-03-06 21:03:42','military missile','success',1442,'FCET-46','','','','OT',421,'SSBN 728',1000 from dual union all
select 65839,'1995-S90','1995-11-10 00:00:00','military missile','success',1434,'','15Zh58 UBP-17','','','Test',608,'',1000 from dual union all
select 6584,'1960-A318','1960-05-01 05:53:00','military missile','success',1471,'','U-2 intercept','','','Intercept',1454,'',20 from dual union all
select 65840,'1995-S91','1995-01-01 00:00:00','military missile','success',437,'-        -','','','','Test',1494,'',1000 from dual union all
select 65841,'1995-S92','1995-01-01 00:00:00','military missile','success',354,'DF-21A','','','','Test',1494,'',500 from dual union all
select 65842,'1995-U01','1995-01-15 13:45:00','orbital','failure',820,'M-3S2-8','EXPRESS/KM-M','EXPRESS','','Sat',837,'',null from dual union all
select 65843,'1995-W01','1995-01-12 02:03:00','suborbital rocket','success',761,'MT135P-R913','','','','Meteo',1332,'',60 from dual union all
select 65844,'1995-W02','1995-01-18 02:00:00','suborbital rocket','success',761,'MT135P-R914','','','','Meteo',1332,'',60 from dual union all
select 65845,'1995-W03','1995-01-25 02:10:00','suborbital rocket','success',761,'MT135P-R915','','','','Meteo',1332,'',60 from dual union all
select 65846,'1995-W04','1995-03-20 00:00:00','suborbital rocket','success',1502,'Aerosole','','','','Aeron',29,'',75 from dual union all
select 65847,'1995-W05','1995-04-11 00:00:00','suborbital rocket','success',1502,'Aerosole','','','','Aeron',29,'',75 from dual union all
select 65848,'1995-W06','1995-05-02 00:00:00','suborbital rocket','success',1502,'Aerosole','','','','Aeron',29,'',75 from dual union all
select 65849,'1995-W07','1995-05-31 00:00:00','suborbital rocket','success',1502,'Aerosole','','','','Aeron',29,'',75 from dual union all
select 6585,'1960-A319','1960-08-22 17:06:00','atmospheric rocket','success',852,'GAC-1','Cree II','','','Parachute test',394,'',46 from dual union all
select 65850,'1995-W08','1995-07-25 10:41:00','suborbital rocket','success',1284,'','','','','Aeron',1047,'',75 from dual union all
select 65851,'1995-W09','1995-08-08 09:10:00','suborbital rocket','success',1284,'','','','','Aeron',1047,'',75 from dual union all
select 65852,'1995-W10','1995-11-20 14:00:00','suborbital rocket','success',1284,'DLR KO-SL-218','','','','Aeron',350,'',75 from dual union all
select 65853,'1995-W11','1995-11-23 14:00:00','suborbital rocket','success',1284,'DLR KO-SL-219','','','','Aeron',350,'',75 from dual union all
select 65854,'1995-W12','1995-12-14 02:00:00','suborbital rocket','success',761,'MT135P-R946','','','','Meteo',1332,'',51 from dual union all
select 65855,'1995-W13','1995-12-20 02:00:00','suborbital rocket','success',761,'MT135P-R947','','','','Meteo',1332,'',53 from dual union all
select 65856,'1995-W14','1995-12-27 02:00:00','suborbital rocket','success',761,'MT135P-R948','','','','Meteo',1332,'',52 from dual union all
select 65857,'1995-W15','1995-03-22 02:00:00','suborbital rocket','success',761,'MT135P-R920','','','','Meteo',1332,'',60 from dual union all
select 65858,'1995-W16','1995-03-29 02:00:00','suborbital rocket','success',761,'MT135P-R921','','','','Meteo',1332,'',60 from dual union all
select 65859,'1995-W17','1995-04-26 02:20:00','suborbital rocket','success',761,'MT135P-R922','','','','Meteo',1332,'',60 from dual union all
select 6586,'1960-A32','1960-02-04 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1216,'VOKV',44 from dual union all
select 65860,'1995-W18','1995-02-09 02:03:00','suborbital rocket','success',761,'MT135P-R916','','','','Meteo',1332,'',60 from dual union all
select 65861,'1995-W19','1995-02-15 02:03:00','suborbital rocket','success',761,'MT135P-R917','','','','Meteo',1332,'',60 from dual union all
select 65862,'1995-W20','1995-02-22 02:00:00','suborbital rocket','success',761,'MT135P-R918','','','','Meteo',1332,'',60 from dual union all
select 65863,'1995-W21','1995-03-08 02:00:00','suborbital rocket','success',761,'MT135P-R919','','','','Meteo',1332,'',60 from dual union all
select 65864,'1995-W22','1995-05-10 02:00:00','suborbital rocket','success',761,'MT135P-R923','','','','Meteo',1332,'',60 from dual union all
select 65865,'1995-W23','1995-05-17 02:08:00','suborbital rocket','success',761,'MT135P-R924','','','','Meteo',1332,'',60 from dual union all
select 65866,'1995-W24','1995-05-24 02:17:00','suborbital rocket','success',761,'MT135P-R925','','','','Meteo',1332,'',60 from dual union all
select 65867,'1995-W25','1995-05-31 02:05:00','suborbital rocket','success',761,'MT135P-R926','','','','Meteo',1332,'',60 from dual union all
select 65868,'1995-W26','1995-06-07 02:03:00','suborbital rocket','success',761,'MT135P-R927','','','','Meteo',1332,'',60 from dual union all
select 65869,'1995-W27','1995-06-21 02:07:00','suborbital rocket','success',761,'MT135P-R928','','','','Meteo',1332,'',60 from dual union all
select 6587,'1960-A320','1960-11-21 17:00:00','atmospheric rocket','success',852,'GAC-2','Cree II','','','Parachute test',394,'',33 from dual union all
select 65870,'1995-W28','1995-06-28 02:00:00','suborbital rocket','success',761,'MT135P-R929','','','','Meteo',1332,'',60 from dual union all
select 65871,'1995-W29','1995-07-05 02:00:00','suborbital rocket','success',761,'MT135P-R930','','','','Meteo',1332,'',60 from dual union all
select 65872,'1995-W30','1995-07-12 02:00:00','suborbital rocket','success',761,'MT135P-R931','','','','Meteo',1332,'',60 from dual union all
select 65873,'1995-W31','1995-07-19 02:00:00','suborbital rocket','success',761,'MT135P-R932','','','','Meteo',1332,'',60 from dual union all
select 65874,'1995-W32','1995-08-02 02:00:00','suborbital rocket','success',761,'MT135P-R933','','','','Meteo',1332,'',60 from dual union all
select 65875,'1995-W33','1995-08-09 02:03:00','suborbital rocket','success',761,'MT135P-R934','','','','Meteo',1332,'',60 from dual union all
select 65876,'1995-W34','1995-08-23 02:05:00','suborbital rocket','success',761,'MT135P-R935','','','','Meteo',1332,'',60 from dual union all
select 65877,'1995-W35','1995-09-06 02:00:00','suborbital rocket','success',761,'MT135P-R936','','','','Meteo',1332,'',60 from dual union all
select 65878,'1995-W36','1995-09-20 02:03:00','suborbital rocket','success',761,'MT135P-R937','','','','Meteo',1332,'',60 from dual union all
select 65879,'1995-W37','1995-10-04 02:03:00','suborbital rocket','success',761,'MT135P-R938','','','','Meteo',1332,'',60 from dual union all
select 6588,'1960-A321','1960-04-20 00:00:00','suborbital rocket','failure',88,'253 Z2.4XZ','','','','Aeron',421,'T-AGM-12',40 from dual union all
select 65880,'1995-W38','1995-10-11 02:00:00','suborbital rocket','success',761,'MT135P-R939','','','','Meteo',1332,'',60 from dual union all
select 65881,'1995-W39','1995-10-18 02:00:00','suborbital rocket','success',761,'MT135P-R940','','','','Meteo',1332,'',60 from dual union all
select 65882,'1995-W40','1995-10-26 02:00:00','suborbital rocket','success',761,'MT135P-R941','','','','Meteo',1332,'',60 from dual union all
select 65883,'1995-W41','1995-11-09 02:15:00','suborbital rocket','success',761,'MT135P-R942','','','','Meteo',1332,'',60 from dual union all
select 65884,'1995-W42','1995-11-16 02:00:00','suborbital rocket','success',761,'MT135P-R943','','','','Meteo',1332,'',60 from dual union all
select 65885,'1995-W43','1995-11-22 02:03:00','suborbital rocket','success',761,'MT135P-R944','','','','Meteo',1332,'',60 from dual union all
select 65886,'1995-W44','1995-11-29 02:05:00','suborbital rocket','success',761,'MT135P-R945','','','','Meteo',1332,'',60 from dual union all
select 65887,'1995-Y01','1995-01-24 16:09:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',99 from dual union all
select 65888,'1995-Y02','1995-12-07 23:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',99 from dual union all
select 65889,'1996-001','1996-01-11 09:41:00','orbital','success',1243,'STS-72','Spartan/SLA','OV-105 Research/FF','','Sat',922,'MLP1',null from dual;
insert into LAUNCH
select 6589,'1960-A322','1960-07-19 00:00:00','suborbital rocket','failure',88,'337 A210.4GA','','','','Aeron',1821,'',20 from dual union all
select 65890,'1996-002','1996-01-12 23:10:00','orbital','success',105,'V82 L454','Measat 1/PAS 3R','Panamsat K4','','Sat',345,'',null from dual union all
select 65891,'1996-003','1996-01-14 11:10:00','orbital','success',422,'D231','Koreasat 2','Koreasat 2','','Sat',234,'',null from dual union all
select 65892,'1996-004','1996-01-16 15:33:45','orbital','success',680,'47187-201','Kosmos-2327','Parus','','Sat',456,'',null from dual union all
select 65893,'1996-005','1996-01-25 09:56:00','orbital','success',958,'374-02','Gorizont','Gorizont No. 43L','','Sat',553,'',null from dual union all
select 65894,'1996-006','1996-02-01 01:15:01','orbital','success',174,'AC-126','Palapa C1','Palapa C1','','Sat',269,'',null from dual union all
select 65895,'1996-007','1996-02-05 07:19:38','orbital','success',110,'V83 L455','N-Star b','N-Star b','','Sat',345,'',null from dual union all
select 65896,'1996-008','1996-02-17 20:43:27','deep space','success',426,'D232','NEAR','NEAR','','Sat',234,'',null from dual union all
select 65897,'1996-009','1996-02-19 00:58:25','orbital','success',1448,'51034-501','Gonets/Kosmos','Gonets-D1 No. 1 (13)','','Sat',469,'',null from dual union all
select 65898,'1996-010','1996-02-19 08:19:00','orbital','success',958,'383-02','Raduga','Gran'' No. 44L','','Sat',553,'',null from dual union all
select 65899,'1996-011','1996-02-21 12:34:05','orbital','success',1238,'V 15000-651','Soyuz TM-23','Soyuz 7K-STM No. 72','','Sat',480,'',null from dual union all
select 659,'1946-S06','1946-07-30 19:36:00','suborbital rocket','success',1470,'9','','','','CR/Ionos',1863,'',167 from dual union all
select 6590,'1960-A323','1960-08-10 00:00:00','suborbital rocket','failure',88,'359 N62.4MU','','','','Aeron',1057,'',20 from dual union all
select 65900,'1996-012','1996-02-22 20:18:00','orbital','success',1243,'STS-75','TSS-1R','OV-102 Research/FF','','Sat',922,'MLP3',null from dual union all
select 65901,'1996-013','1996-02-24 11:24:00','orbital','success',423,'D233','Polar','Polar','','Sat',1724,'',null from dual union all
select 65902,'1996-014','1996-03-09 01:33:00','orbital','success',911,'F10','REX II','REX II','','Sat',1718,'L-1011',null from dual union all
select 65903,'1996-015','1996-03-14 07:11:01','orbital','success',108,'V84 L456','Intelsat 707','INTELSAT 707','','Sat',345,'',null from dual union all
select 65904,'1996-016','1996-03-14 17:40:00','orbital','success',1241,'77044  -764','Kosmos-2331','Kobal''t','','Sat',475,'',null from dual union all
select 65905,'1996-017','1996-03-21 04:53:00','orbital','success',901,'PSLV-D3','IRS-P3','IRS-P3','','Sat',1357,'',null from dual union all
select 65906,'1996-018','1996-03-22 08:13:04','orbital','success',1243,'STS-76','Shuttle/Mir-3','OV-104 Shuttle/Mir','','Sat',922,'MLP2',null from dual union all
select 65907,'1996-019','1996-03-28 00:21:00','orbital','success',422,'D234','USA 117 (GPS IIA-25)','Navstar GPS SVN 33','','Sat',234,'',null from dual union all
select 65908,'1996-020','1996-04-03 23:01:01','orbital','success',173,'AC-122','Inmarsat 3F1','INMARSAT III F-1','','Sat',268,'',null from dual union all
select 65909,'1996-021','1996-04-08 23:09:01','orbital','success',960,'390-01','Astra 1F','Astra 1F','','Sat',587,'',null from dual union all
select 6591,'1960-A324','1960-09-16 00:00:00','suborbital rocket','failure',88,'402 A250AA','','','','Aeron',1895,'',20 from dual union all
select 65910,'1996-022','1996-04-20 22:36:00','orbital','success',102,'V85 L457','MSAT-1','MSAT-1','','Sat',345,'',null from dual union all
select 65911,'1996-023','1996-04-23 11:48:50','orbital','success',949,'385-01','Priroda','TsM-I 77KSI No. 17401','','Sat',587,'',null from dual union all
select 65912,'1996-024','1996-04-24 12:27:40','orbital','success',417,'D235','MSX','MSX','','Sat',1724,'',null from dual union all
select 65913,'1996-025','1996-04-24 13:00:01','orbital','success',680,'47187-213','Kosmos-2332','Taifun-1B','','Sat',456,'',null from dual union all
select 65914,'1996-026','1996-04-24 23:37:02','orbital','success',1409,'K-16/TC-15  (45E-4)','USA 118      [CY15]','MERCURY 2','','Sat',275,'',null from dual union all
select 65915,'1996-027','1996-04-30 04:31:01','orbital','success',171,'AC-78','SAX','SAX','','Sat',269,'',null from dual union all
select 65916,'1996-028','1996-05-05 07:04:18','orbital','success',1238,'Kh15000-648','Progress M-31','Progress 7K-TGM No. 231','','Sat',480,'',null from dual union all
select 65917,'1996-029','1996-05-12 21:32:00','orbital','success',1413,'K-22   (45F-11)','USA 119-124 [NOSS13]','SLDCOM 4/INTRUDER 4','','Sat',1777,'',null from dual union all
select 65918,'1996-030','1996-05-16 01:56:29','orbital','success',105,'V86 L458a','Palapa C2/Amos 1','Palapa C2','','Sat',345,'',null from dual union all
select 65919,'1996-031','1996-05-17 02:44:00','orbital','success',910,'F11','MSTI 3','MSTI 3','','Sat',1718,'L-1011',null from dual union all
select 6592,'1960-A325','1960-10-19 00:00:00','suborbital rocket','success',88,'460 A276.4AA','','','','Test',1895,'',38 from dual union all
select 65920,'1996-032','1996-05-19 10:30:00','orbital','success',1243,'STS-77','SH-04/Spartan/TEAMS','OV-105 Research/FF','','Sat',922,'MLP1',null from dual union all
select 65921,'1996-033','1996-05-24 01:10:00','orbital','success',422,'D236','Galaxy 9','Galaxy 9','','Sat',234,'',null from dual union all
select 65922,'1996-034','1996-05-25 02:05:00','orbital','success',958,'379-01','Gorizont','Gorizont No. 44L','','Sat',553,'',null from dual union all
select 65923,'1996-035','1996-06-15 06:55:09','orbital','success',110,'V87 L459a','Intelsat 709','INTELSAT 709','','Sat',345,'',null from dual union all
select 65924,'1996-036','1996-06-20 14:49:00','orbital','success',1243,'STS-78','Spacelab LMS','OV-102 Spacelab','','Sat',922,'MLP3',null from dual union all
select 65925,'1996-037','1996-07-02 07:48:00','orbital','success',911,'F12','TOMS-EP','TOMS-EP','','Sat',1718,'L-1011',null from dual union all
select 65926,'1996-038','1996-07-03 00:30:02','orbital','success',1417,'K-2    (45H-1)','USA 125      [SDS11]','QUASAR 11?','','Sat',274,'',null from dual union all
select 65927,'1996-039','1996-07-03 10:47:00','orbital','success',308,'Y10A     (41)','Apstar 1A','Apstar 1A','','Sat',1904,'',null from dual union all
select 65928,'1996-040','1996-07-09 22:24:55','orbital','success',105,'V89 L460a','Turksat 1C/Arab. 2A','Arabsat 2A','','Sat',345,'',null from dual union all
select 65929,'1996-041','1996-07-16 00:50:00','orbital','success',422,'D237','USA 126 (GPS IIA-26)','Navstar GPS SVN 40','','Sat',233,'',null from dual union all
select 6593,'1960-A326','1960-04-21 00:00:00','atmospheric rocket','success',252,'FJ 3','Flight 1','','','Test',2,'',10 from dual union all
select 65930,'1996-042','1996-07-25 12:41:01','orbital','success',172,'AC-125','USA 127 (UHF F7)','UFO F7-EHF','','Sat',268,'',null from dual union all
select 65931,'1996-043','1996-07-31 20:00:06','orbital','success',1238,'Kh15000-049','Progress M-32','Progress 7K-TGM No. 232','','Sat',480,'',null from dual union all
select 65932,'1996-044','1996-08-08 22:49:00','orbital','success',105,'V90 L461a','Italsat 2/Telecom 2D','Italsat F2','','Sat',345,'',null from dual union all
select 65933,'1996-045','1996-08-14 22:20:59','orbital','success',805,'77031  -675','Molniya-1T','Molniya-1T','','Sat',474,'',null from dual union all
select 65934,'1996-046','1996-08-17 01:53:00','orbital','success',526,'H-II-4F','ADEOS','ADEOS','','Sat',1477,'',null from dual union all
select 65935,'1996-047','1996-08-17 13:18:03','orbital','success',1238,'Kh15000-077','Soyuz TM-24','Soyuz 7K-STM No. 73','','Sat',480,'',null from dual union all
select 65936,'1996-048','1996-08-18 10:27:00','orbital','failure',308,'Y14      (42)','Zhongxing 7','Zhongxing 7','','Sat',1904,'',null from dual union all
select 65937,'1996-049','1996-08-21 09:47:26','orbital','success',911,'F13','FAST','FAST','','Sat',1718,'L-1011',null from dual union all
select 65938,'1996-050','1996-08-29 05:22:00','orbital','success',807,'76047  -680','Interbol-2','Microsatelite','','Sat',474,'',null from dual union all
select 65939,'1996-051','1996-09-04 09:01:00','orbital','success',1537,'51027-905','Kosmos-2333','Tselina-2','','Sat',567,'',null from dual union all
select 6594,'1960-A327','1960-04-26 00:00:00','atmospheric rocket','success',252,'','','','','Test',2,'',10 from dual union all
select 65940,'1996-052','1996-09-05 12:47:39','orbital','success',680,'53759-826','Kosmos-2334/UNAMSAT','Parus','','Sat',456,'',null from dual union all
select 65941,'1996-053','1996-09-06 17:37:39','orbital','success',957,'375-01','Inmarsat 3 F2','INMARSAT III F-2','','Sat',587,'',null from dual union all
select 65942,'1996-054','1996-09-08 21:49:01','orbital','success',173,'AC-123','GE 1','GE 1','','Sat',269,'',null from dual union all
select 65943,'1996-055','1996-09-11 00:00:59','orbital','success',102,'V91 L462','Echostar 2','Echostar II','','Sat',345,'',null from dual union all
select 65944,'1996-056','1996-09-12 08:49:00','orbital','success',422,'D238','USA 128 (GPS IIA-27)','Navstar GPS SVN 30','','Sat',233,'',null from dual union all
select 65945,'1996-057','1996-09-16 08:54:49','orbital','success',1243,'STS-79','Shuttle/Mir-4','OV-104 Shuttle/Mir','','Sat',921,'MLP1',null from dual union all
select 65946,'1996-058','1996-09-26 17:50:53','orbital','success',959,'379-02','Ekspress 12','Ekspress No. 12L','','Sat',553,'',null from dual union all
select 65947,'1996-059','1996-10-20 07:20:00','orbital','success',304,'Y3       (43)','FSW-2','FSW-2 No. 3','','Sat',810,'',null from dual union all
select 65948,'1996-060','1996-10-24 11:37:00','orbital','success',803,'78012  -645','Molniya-3','Molniya-3 No. 62','','Sat',475,'',null from dual union all
select 65949,'1996-061','1996-11-04 17:08:56','orbital','failure',911,'F14','SAC-B/HETE','HETE','','Sat',1819,'L-1011',null from dual union all
select 6595,'1960-A328','1960-07-20 00:00:00','atmospheric rocket','success',1527,'','','','','Test',1895,'B-29',10 from dual union all
select 65950,'1996-062','1996-11-07 17:00:49','deep space','success',422,'D239','MGS','MGS','','Sat',233,'',null from dual union all
select 65951,'1996-063','1996-11-13 22:40:00','orbital','success',105,'V92 L463a','Arab. 2B/Measat 2','Arabsat 2B','','Sat',345,'',null from dual union all
select 65952,'1996-064','1996-11-16 20:48:53','orbital','success',955,'392-02','Mars-96','M1 No. 520 (Mars-96)','','Sat',553,'',null from dual union all
select 65953,'1996-065','1996-11-19 19:55:47','orbital','success',1243,'STS-80','ORFEUS/WSF','OV-102 Research/FF','','Sat',922,'MLP3',null from dual union all
select 65954,'1996-066','1996-11-19 23:20:38','orbital','success',1238,'Kh15000-639','Progress M-33','Progress 7K-TGM No. 233','','Sat',480,'',null from dual union all
select 65955,'1996-067','1996-11-21 20:47:01','orbital','success',173,'AC-124','Hot Bird 2','Eutelsat HB2','','Sat',269,'',null from dual union all
select 65956,'1996-068','1996-12-04 06:58:07','deep space','success',422,'D240','MPF','MPF Lander','','Sat',234,'',null from dual union all
select 65957,'1996-069','1996-12-11 12:00:00','orbital','success',1446,'45013-302','Kosmos-2335','US-PU','','Sat',590,'',null from dual union all
select 65958,'1996-070','1996-12-18 01:57:01','orbital','success',173,'AC-129','Inmarsat 3F3','INMARSAT III F3','','Sat',268,'',null from dual union all
select 65959,'1996-071','1996-12-20 06:43:58','orbital','success',680,'65065-633','Kosmos-2336','Parus','','Sat',456,'',null from dual union all
select 6596,'1960-A329','1960-03-15 00:00:00','military missile','success',843,'','Last 1959-60 ASF','','','Test',443,'',3 from dual union all
select 65960,'1996-072','1996-12-20 18:04:00','orbital','success',1415,'K-13   (45J-5)','USA 129      [IC6]','Improved CRYSTAL 2106','','Sat',1777,'',null from dual union all
select 65961,'1996-073','1996-12-24 13:50:00','orbital','success',1238,'V 15000-050','Bion No. 11','Bion No. 11','','Sat',475,'',null from dual union all
select 65962,'1996-A01','1996-01-01 00:00:00','military missile','failure',518,'RSM-52V','','','','Test',989,'',0 from dual union all
select 65963,'1996-A02','1996-05-18 14:20:00','test rocket','success',346,'','','','','Test',1895,'',0 from dual union all
select 65964,'1996-A03','1996-06-07 16:15:00','test rocket','success',346,'','','','','Test',1895,'',0 from dual union all
select 65965,'1996-A04','1996-06-08 18:17:00','test rocket','success',346,'','','','','Test',1895,'',3 from dual union all
select 65966,'1996-A05','1996-07-31 19:15:00','test rocket','failure',346,'','','','','Test',1895,'',1 from dual union all
select 65967,'1996-A06','1996-08-02 00:00:00','military missile','success',1088,'Aegis ETR-1A','NTW','','','Test',1869,'',10 from dual union all
select 65968,'1996-A07','1996-10-23 00:00:00','military missile','success',1057,'','Arms control','','','Destruct/Test',201,'TK-20',3 from dual union all
select 65969,'1996-A08','1996-11-15 00:00:00','suborbital rocket','success',591,'FPDP 1','LA-1','','','Test',1821,'',8 from dual union all
select 6597,'1960-A33','1960-02-05 21:46:00','military missile','failure',1419,'C-4','RVX-3','','','Test',231,'',0 from dual union all
select 65970,'1996-A09','1996-11-15 00:00:00','suborbital rocket','success',591,'FPDP 2','LA-2','','','Test',1821,'',8 from dual union all
select 65971,'1996-A10','1996-07-27 00:00:00','suborbital rocket','failure',1502,'','Mini-DUSTY MD-01','','','Aeron',29,'',40 from dual union all
select 65972,'1996-A11','1996-07-30 00:00:00','suborbital rocket','failure',1502,'','Mini-DUSTY MD-02','','','Aeron',29,'',40 from dual union all
select 65973,'1996-A12','1996-08-22 00:00:00','military missile','success',702,'Hawk Target','','','','Test',1895,'',20 from dual union all
select 65974,'1996-A13','1996-09-19 00:00:00','military missile','failure',757,'2','','','','Test',206,'S616',10 from dual union all
select 65975,'1996-A14','1996-08-28 00:00:00','military missile','success',702,'Hawk Target','','','','Test',1895,'',20 from dual union all
select 65976,'1996-A15','1996-12-07 00:00:00','military missile','success',702,'Hawk Target','','','','Test',1895,'',20 from dual union all
select 65977,'1996-F01','1996-02-14 19:01:00','orbital','failure',313,'Y1       (40)','Intelsat 708','INTELSAT 708','','Sat',1903,'',1 from dual union all
select 65978,'1996-F02','1996-05-14 08:55:00','orbital','failure',1238,'78051  -368','','Kometa No. 18','','Sat',562,'',70 from dual union all
select 65979,'1996-F03','1996-06-04 12:34:06','orbital','failure',114,'V88  (501)','Cluster','Cluster F1-F4','','Sat',346,'',4 from dual;
insert into LAUNCH
select 6598,'1960-A330','1960-11-20 23:14:00','military missile','success',843,'','ASF','','','Test',401,'',12 from dual union all
select 65980,'1996-F04','1996-06-20 18:44:59','orbital','failure',1241,'77044  -843','','Kobal''t','','Sat',462,'',10 from dual union all
select 65981,'1996-M01','1996-02-20 00:00:00','military missile','success',119,'F2','','','','Test',1026,'',60 from dual union all
select 65982,'1996-M02','1996-08-20 00:00:00','military missile','success',119,'F3','','','','ATBM Test',1026,'',60 from dual union all
select 65983,'1996-M03','1996-08-20 00:00:00','military missile','success',118,'','','','','Target',1024,'SHIP-ISR',80 from dual union all
select 65984,'1996-M04','1996-01-01 00:00:00','military missile','success',610,'1','','','','Test',636,'',50 from dual union all
select 65985,'1996-S01','1996-01-01 00:00:00','military missile','success',942,'2-2','','','','Test',320,'',100 from dual union all
select 65986,'1996-S02','1996-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 65987,'1996-S03','1996-01-10 00:00:00','military missile','success',354,'','','','','Test',1494,'',1000 from dual union all
select 65988,'1996-S04','1996-01-27 08:52:00','military missile','success',942,'2-1 (15)','','','','Test',320,'',100 from dual union all
select 65989,'1996-S05','1996-02-02 00:00:00','military missile','success',570,'3','BRV/Piledriver Demo','','','Target',1857,'',89 from dual union all
select 6599,'1960-A331','1960-11-29 00:00:00','military missile','failure',843,'B310','Last elevator launch','','','Test',1895,'',1 from dual union all
select 65990,'1996-S06','1996-02-11 11:00:00','suborbital rocket','success',1073,'S-310-24','NTV-1','','','Aeron',833,'',199 from dual union all
select 65991,'1996-S07','1996-02-11 23:00:00','test rocket','success',612,'J-1-1F','Hyflex','','','SC Test',1474,'',110 from dual union all
select 65992,'1996-S08','1996-02-23 19:17:45','suborbital rocket','success',224,'NASA 36.143UE','','','','Ionos',1873,'',300 from dual union all
select 65993,'1996-S09','1996-02-23 19:44:00','suborbital rocket','success',224,'NASA 36.145NP','','','','Microgravity',1873,'',300 from dual union all
select 65994,'1996-S10','1996-03-02 10:00:00','suborbital rocket','success',1212,'DLR K-GR-220','DLR TEXUS 34','','','Micrograv',416,'',232 from dual union all
select 65995,'1996-S11','1996-03-06 03:35:00','suborbital rocket','success',224,'NASA 36.122UG','EUVS','','','EUV Astron',1873,'',300 from dual union all
select 65996,'1996-S12','1996-03-06 20:01:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT161GM','GLORY TRIP 161GM','OT',1705,'',1300 from dual union all
select 65997,'1996-S13','1996-03-22 00:00:00','military missile','success',570,'4','BRV/THAAD Target','','','Target',1857,'',300 from dual union all
select 65998,'1996-S14','1996-03-22 00:00:00','military missile','success',1293,'FT-05','','','','ABM Test',1895,'',100 from dual union all
select 65999,'1996-S15','1996-04-03 15:59:00','suborbital rocket','success',213,'Starfire 1/F8','CONQUEST 1','','','Aeron?',1873,'',281 from dual union all
select 66,'1944-A30','1944-12-20 08:47:00','military missile','failure',1470,'19845','Ma272','','','Test',661,'',1 from dual union all
select 660,'1946-S07','1946-10-10 18:02:00','suborbital rocket','success',1470,'12','','','','Solar UV',1863,'',164 from dual union all
select 6600,'1960-A332','1960-03-18 00:00:00','military missile','success',843,'','ZURF Test','','','Test',1894,'',20 from dual union all
select 66000,'1996-S16','1996-04-17 07:00:00','military missile','success',1434,'','15Zh58 UBP-18','','','Test',608,'',1000 from dual union all
select 66001,'1996-S17','1996-04-21 21:36:06','military missile','success',1443,'FCET-14','','','','OT',421,'SSBN 738',1000 from dual union all
select 66002,'1996-S18','1996-04-21 21:36:23','military missile','success',1443,'FCET-14','','','','OT',421,'SSBN 738',1000 from dual union all
select 66003,'1996-S19','1996-05-03 04:16:00','suborbital rocket','success',1212,'DLR K-GR-221','SSC MASER 7','','','Micrograv',416,'',252 from dual union all
select 66004,'1996-S20','1996-05-08 08:01:00','military missile','success',907,'20PA (87-0144)','Mk 21 RV x 5','GT 20PA','','OT',1700,'',1000 from dual union all
select 66005,'1996-S21','1996-05-16 21:48:55','military missile','success',1442,'FCET-47','','','','OT',421,'SSBN 726',1000 from dual union all
select 66006,'1996-S22','1996-05-16 21:49:12','military missile','success',1442,'FCET-47','','','','OT',421,'SSBN 726',1000 from dual union all
select 66007,'1996-S23','1996-05-16 22:50:13','military missile','success',1442,'FCET-47','','','','OT',421,'SSBN 726',1000 from dual union all
select 66008,'1996-S24','1996-05-16 22:50:31','military missile','success',1442,'FCET-47','-    200','','','OT',421,'SSBN 726',1000 from dual union all
select 66009,'1996-S25','1996-05-30 08:01:00','military missile','success',907,'21PA (86-440)','Mk 21 RV x 5','GT 21PA','','OT',1697,'',1000 from dual union all
select 6601,'1960-A333','1960-04-08 00:00:00','military missile','success',843,'','ZURF Target','','','Test',1894,'',20 from dual union all
select 66010,'1996-S26','1996-06-04 05:06:00','suborbital rocket','success',211,'NASA 27.140UH','XQC F2','','','XR Astronomy',1881,'',220 from dual union all
select 66011,'1996-S28','1996-06-06 00:00:00','military missile','success',1453,'51122-81-428','15A35 UBP-45','GCh 15F355P7','','OT',596,'',1000 from dual union all
select 66012,'1996-S29','1996-06-17 05:50:01','suborbital rocket','success',224,'NASA 36.134UR','','','','Radio Astronomy',1873,'',300 from dual union all
select 66013,'1996-S30','1996-06-20 14:40:01','suborbital rocket','success',224,'NASA 36.118NP','DARTFire 1','','','Microgravity',1873,'',300 from dual union all
select 66014,'1996-S31','1996-06-26 12:17:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT162GM','GLORY TRIP 162GM','OT',1706,'',1300 from dual union all
select 66015,'1996-S32','1996-06-26 14:59:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT160GM-1','GLORY TRIP 160GM-1','OT',1699,'',1300 from dual union all
select 66016,'1996-S33','1996-06-26 19:00:00','suborbital rocket','success',224,'NASA 36.147US','USC-1 SOHO Cal','','','Solar EUV',1873,'',309 from dual union all
select 66017,'1996-S34','1996-06-28 00:00:00','military missile','success',1514,'','','','','OT',1011,'PLBR',1000 from dual union all
select 66018,'1996-S35','1996-06-28 00:00:00','military missile','success',1514,'','','','','OT',1011,'PLBR',1000 from dual union all
select 66019,'1996-S36','1996-06-28 00:00:00','military missile','success',1514,'','','','','OT',1011,'PLBR',1000 from dual union all
select 6602,'1960-A334','1960-04-14 00:00:00','military missile','success',843,'','Improved NH Test','','','Test',1895,'',20 from dual union all
select 66020,'1996-S37','1996-07-09 00:00:00','military missile','success',117,'ABIE-TLV','TBM tracking test','','','Target',1821,'',300 from dual union all
select 66021,'1996-S38','1996-07-14 07:15:01','suborbital rocket','success',213,'NASA 36.113UG','NUVIEWS 1','','','UV Astronomy',1873,'',300 from dual union all
select 66022,'1996-S39','1996-07-15 00:00:00','military missile','success',1514,'','','','','OT',201,'DELTA3',1000 from dual union all
select 66023,'1996-S40','1996-07-15 00:00:00','military missile','success',1179,'','','','','OT',201,'DELTA4',1000 from dual union all
select 66024,'1996-S41','1996-07-15 11:28:00','military missile','success',570,'5','BRV/THAAD Target','','','Target',1857,'',300 from dual union all
select 66025,'1996-S42','1996-07-15 11:31:00','military missile','success',1293,'FT-06','','','','ABM Test',1895,'',100 from dual union all
select 66026,'1996-S43','1996-07-15 18:21:00','suborbital rocket','success',281,'TCMP IIB','IRLV','','','Reentry Test',1784,'',317 from dual union all
select 66027,'1996-S44','1996-07-19 20:01:01','military missile','success',1443,'FCET-15','','','','OT',421,'SSBN 735',1000 from dual union all
select 66028,'1996-S45','1996-07-19 20:01:17','military missile','success',1443,'FCET-15','','','','OT',421,'SSBN 735',1000 from dual union all
select 66029,'1996-S46','1996-07-25 00:00:00','military missile','success',1436,'3L','15Zh65 LKI-3','','','Test',608,'',1000 from dual union all
select 6603,'1960-A335','1960-06-01 00:00:00','military missile','success',843,'','Improved NH Test','','','Test',1895,'',20 from dual union all
select 66030,'1996-S47','1996-07-26 09:40:00','suborbital rocket','success',213,'NASA 36.148CL','EUVS Venus','','','UV Astronomy',1873,'',300 from dual union all
select 66031,'1996-S48','1996-08-12 18:58:00','suborbital rocket','success',860,'NASA 31.114UP','HOMER','','','Aeron/Education',1895,'',96 from dual union all
select 66032,'1996-S49','1996-08-20 15:30:00','suborbital rocket','success',1073,'S-310-26','SEEK-1','','','Plasma',833,'',177 from dual union all
select 66033,'1996-S50','1996-08-20 16:00:00','suborbital rocket','success',213,'NASA 36.130NS','','','','Solar',1873,'',300 from dual union all
select 66034,'1996-S51','1996-08-26 14:00:00','suborbital rocket','success',1073,'S-310-25','SEEK-1','','','Plasma',833,'',180 from dual union all
select 66035,'1996-S52','1996-08-28 00:00:00','suborbital rocket','success',1216,'','','','','Ionos?',330,'',87 from dual union all
select 66036,'1996-S53','1996-08-29 17:38:00','suborbital rocket','success',228,'','','','','Target?',1821,'',100 from dual union all
select 66037,'1996-S54','1996-08-31 15:41:49','military missile','success',1111,'M-3 STARS II','ODES/MSX MDT-2','','','Target',849,'',909 from dual union all
select 66038,'1996-S55','1996-09-11 08:01:00','military missile','success',907,'22PA (86-165)','Mk 21 RV x 7','GT 22PA','','OT',1700,'',1000 from dual union all
select 66039,'1996-S56','1996-09-24 22:00:00','suborbital rocket','success',1296,'TR-1A-5','','','','Micrograv',1476,'',264 from dual union all
select 6604,'1960-A336','1960-09-05 00:00:00','military missile','success',843,'','ATBM Target','','','Test',1894,'',20 from dual union all
select 66040,'1996-S57','1996-09-27 16:51:00','military missile','success',796,'MSLS IFT-Demo','MSLS IFT-Demo','MSLS','','Target',1698,'',1600 from dual union all
select 66041,'1996-S58','1996-10-02 22:30:03','military missile','success',1442,'DASO-35','','','','OT',421,'SSBN 728',1000 from dual union all
select 66042,'1996-S59','1996-10-03 00:00:00','military missile','success',1179,'','','','','OT',201,'PLBR',1000 from dual union all
select 66043,'1996-S60','1996-10-03 00:00:00','military missile','success',1434,'','15Zh58 UBP-19','','','Test',608,'',1000 from dual union all
select 66044,'1996-S61','1996-10-09 12:10:00','military missile','success',570,'6','BRV/THAAD RST-1','','','Target',1857,'',309 from dual union all
select 66045,'1996-S62','1996-10-16 10:41:00','suborbital rocket','success',686,'Red Tigress 3','','','','Target',1821,'',400 from dual union all
select 66046,'1996-S63','1996-10-16 16:02:00','suborbital rocket','success',224,'NASA 36.154NP','DARTFire 2','','','Microgravity',1873,'',300 from dual union all
select 66047,'1996-S64','1996-10-21 03:00:00','suborbital rocket','success',224,'NASA 36.115UG','JHU FPT 1 (Jupiter)','','','UV Astron',1873,'',300 from dual union all
select 66048,'1996-S65','1996-10-28 19:30:20','military missile','success',1443,'FCET-16','','','','OT',420,'SSBN 734',1000 from dual union all
select 66049,'1996-S66','1996-10-28 19:30:37','military missile','success',1443,'FCET-16','','','','OT',420,'SSBN 734',1000 from dual union all
select 6605,'1960-A337','1960-06-02 00:00:00','atmospheric rocket','success',512,'','Jaguar R&D','','','Test',1849,'',20 from dual union all
select 66050,'1996-S67','1996-10-29 03:30:00','suborbital rocket','success',224,'NASA 36.149UL','','','','Aeron',1873,'',300 from dual union all
select 66051,'1996-S68','1996-11-06 13:06:00','military missile','success',907,'23PA (85-169)','Mk 21 RV x 7','GT 23PA','','OT',1697,'',1000 from dual union all
select 66052,'1996-S69','1996-11-05 00:00:00','military missile','success',1434,'52','15Zh58 UBP-20','','','Test',608,'',1000 from dual union all
select 66053,'1996-S70','1996-11-13 18:30:00','suborbital rocket','success',224,'NASA 36.142GS','SERTS-96 (07)','','','Solar EUV',1873,'',300 from dual union all
select 66054,'1996-S71','1996-11-17 13:30:00','suborbital rocket','success',1334,'NASA 41.005GP','','','','Test?',1873,'',150 from dual union all
select 66055,'1996-S72','1996-11-24 11:05:00','suborbital rocket','success',1212,'DLR K-GR-222','DLR TEXUS 35','','','Micrograv',416,'',267 from dual union all
select 66056,'1996-S73','1996-11-28 00:00:00','military missile','success',1042,'15Zh61','15Zh61 UBP-1','Rail','','Test',477,'',1000 from dual union all
select 66057,'1996-S74','1996-12-28 00:00:00','military missile','success',354,'','','','','Test',1494,'',1000 from dual union all
select 66058,'1996-S75','1996-06-13 00:00:00','suborbital rocket','success',228,'','LAADEX','','','Target',855,'',180 from dual union all
select 66059,'1996-S76','1996-03-07 16:00:00','military missile','success',350,'','Near Taiwan','','','Test',1296,'',100 from dual union all
select 6606,'1960-A338','1960-08-10 00:00:00','atmospheric rocket','success',512,'','Jaguar R&D','','','Test',1849,'',20 from dual union all
select 66060,'1996-S77','1996-03-07 17:00:00','military missile','success',350,'','Near Taiwan','','','Test',1296,'',100 from dual union all
select 66061,'1996-S78','1996-03-07 18:00:00','military missile','success',350,'','Near Taiwan','','','Test',1296,'',100 from dual union all
select 66062,'1996-S79','1996-03-13 00:00:00','military missile','success',350,'','Near Taiwan','','','Test',1296,'',100 from dual union all
select 66063,'1996-W01','1996-01-09 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',73 from dual union all
select 66064,'1996-W02','1996-01-11 02:15:00','suborbital rocket','success',761,'MT135P-R949','','','','Meteo',1332,'',56 from dual union all
select 66065,'1996-W03','1996-01-17 02:00:00','suborbital rocket','success',761,'MT135P-R950','','','','Meteo',1332,'',55 from dual union all
select 66066,'1996-W04','1996-01-18 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',71 from dual union all
select 66067,'1996-W05','1996-01-24 02:00:00','suborbital rocket','success',761,'MT135P-R951','','','','Meteo',1332,'',54 from dual union all
select 66068,'1996-W06','1996-01-30 18:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',61 from dual;
insert into LAUNCH
select 66069,'1996-W07','1996-01-31 02:04:59','suborbital rocket','success',761,'MT135P-R952','','','','Meteo',1332,'',54 from dual union all
select 6607,'1960-A339','1960-09-22 00:00:00','atmospheric rocket','success',512,'','Jaguar R&D','','','Test',1849,'',20 from dual union all
select 66070,'1996-W08','1996-09-15 02:30:00','suborbital rocket','success',759,'MT-135-64','NNP-O3','','','Aeron',839,'',56 from dual union all
select 66071,'1996-W09','1996-12-04 02:03:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',52 from dual union all
select 66072,'1996-W10','1996-12-11 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66073,'1996-W11','1996-12-18 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66074,'1996-W12','1996-12-19 15:20:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',53 from dual union all
select 66075,'1996-W13','1996-12-25 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66076,'1996-W14','1996-02-14 02:00:00','suborbital rocket','success',761,'MT135P-R953','','','','Meteo',1332,'',60 from dual union all
select 66077,'1996-Y01','1996-06-10 00:00:00','suborbital rocket','failure',1502,'NASA 12.48WT','','','','Aeron',1821,'',100 from dual union all
select 66078,'1996-Y02','1996-12-10 18:31:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',92 from dual union all
select 66079,'1996-Y03','1996-02-08 00:00:00','suborbital rocket','success',1502,'Wind','','','','Aeron',839,'',110 from dual union all
select 6608,'1960-A34','1960-02-05 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1217,'VOKV',38 from dual union all
select 66080,'1996-Y04','1996-11-13 00:00:00','suborbital rocket','success',1502,'','Mini-DUSTY MD-03','','','Aeron',29,'',100 from dual union all
select 66081,'1997-001','1997-01-12 09:27:23','orbital','success',1243,'STS-81','Shuttle/Mir-5','OV-104 Shuttle/Mir','','Sat',922,'MLP2',null from dual union all
select 66082,'1997-002','1997-01-30 22:04:00','orbital','success',105,'V93 L465b','Nahuel 1A/GE-2','GE 2','','Sat',345,'',null from dual union all
select 66083,'1997-003','1997-02-10 14:09:30','orbital','success',1238,'Kh15000-650','Soyuz TM-25','Soyuz 7K-STM No. 74','','Sat',480,'',null from dual union all
select 66084,'1997-004','1997-02-11 08:55:17','orbital','success',1243,'STS-82','HST SM-2','OV-103 HST mission','','Sat',921,'MLP1',null from dual union all
select 66085,'1997-005','1997-02-12 04:50:00','orbital','success',731,'M-V-1','VSOP/KM','MUSES-B','','Sat',836,'',null from dual union all
select 66086,'1997-006','1997-02-14 03:47:22','orbital','success',1448,'51091-703','Gonets/Kosmos','Strela-3','','Sat',469,'',null from dual union all
select 66087,'1997-007','1997-02-17 01:42:02','orbital','success',174,'AC-127','JCSAT 4','JCSAT 4','','Sat',269,'',null from dual union all
select 66088,'1997-008','1997-02-23 20:20:01','orbital','success',1412,'4B-24 (K-24, 45D-4)','USA 130      [DSP18]','DSP 20','','Sat',274,'',null from dual union all
select 66089,'1997-009','1997-03-01 01:07:42','orbital','success',110,'V94 L464c','Intelsat 801','INTELSAT 801','','Sat',345,'',null from dual union all
select 6609,'1960-A35','1960-02-06 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1218,'VOKV',38 from dual union all
select 66090,'1997-010','1997-03-04 02:00:02','orbital','success',1261,'','Zeya','Mozhaets 2','','Sat',478,'',null from dual union all
select 66091,'1997-011','1997-03-08 06:01:01','orbital','success',173,'AC-128','Tempo 2','Tempo 2','','Sat',268,'',null from dual union all
select 66092,'1997-012','1997-04-04 16:47:00','orbital','success',1422,'23G-6','USA 131 (DMSP)','DMSP Block 5D-2 S-14','','Sat',1778,'',null from dual union all
select 66093,'1997-013','1997-04-04 19:20:32','orbital','success',1243,'STS-83','Spacelab MSL-1','OV-102 Spacelab','','Sat',921,'MLP3',null from dual union all
select 66094,'1997-014','1997-04-06 16:04:05','orbital','success',1238,'Kh15000-652','Progress M-34','Progress 7K-TGM No. 234','','Sat',480,'',null from dual union all
select 66095,'1997-015','1997-04-09 08:58:44','orbital','success',803,'76032  -647','Kosmos-2340','Oko','','Sat',462,'',null from dual union all
select 66096,'1997-016','1997-04-16 23:08:44','orbital','success',108,'V95 L467d','Thaicom 3/BSAT 1a','Thaicom 3','','Sat',345,'',null from dual union all
select 66097,'1997-017','1997-04-17 13:03:21','orbital','success',680,'53765-635','Kosmos-2341','Parus','','Sat',456,'',null from dual union all
select 66098,'1997-018','1997-04-21 11:59:06','orbital','success',911,'F15','Minisat-01','Minisat-01','','Sat',451,'L-1011',null from dual union all
select 66099,'1997-019','1997-04-25 05:49:01','orbital','success',171,'AC-79','GOES 10','GOES K','','Sat',269,'',null from dual union all
select 661,'1946-S08','1946-10-24 19:15:00','suborbital rocket','success',1470,'13','','','','Solar UV/CR',1863,'',105 from dual union all
select 6610,'1960-A36','1960-02-07 11:03:59','suborbital rocket','success',725,'','','','','Meteo',1219,'VOKV',44 from dual union all
select 66100,'1997-020','1997-05-05 14:55:28','orbital','success',418,'D242','Iridium MS-1A','Iridium SV008','','Sat',1724,'',null from dual union all
select 66101,'1997-021','1997-05-11 16:17:00','orbital','success',309,'Y3      (44)','Zhongxing 6','DFH-3','','Sat',1903,'',null from dual union all
select 66102,'1997-022','1997-05-14 00:33:57','orbital','success',803,'77031  -673','Kosmos-2342','Oko','','Sat',475,'',null from dual union all
select 66103,'1997-023','1997-05-15 08:07:48','orbital','success',1243,'STS-84','Shuttle/Mir-6','OV-104 Shuttle/Mir','','Sat',921,'MLP2',null from dual union all
select 66104,'1997-024','1997-05-15 12:10:00','orbital','success',1238,'77031  -168','Kosmos-2343','Don','','Sat',562,'',null from dual union all
select 66105,'1997-025','1997-05-20 22:39:00','orbital','success',422,'D243','Thor II','Thor 2A','','Sat',233,'',null from dual union all
select 66106,'1997-026','1997-05-24 17:00:00','orbital','success',961,'380-02','Telstar 5','Telstar 5','','Sat',587,'',null from dual union all
select 66107,'1997-027','1997-06-03 23:20:06','orbital','success',105,'V97 L468b','Inmarsat 3/Insat 2D','INMARSAT III F4','','Sat',345,'',null from dual union all
select 66108,'1997-028','1997-06-06 16:56:54','orbital','success',950,'380-01','Kosmos-2344','Arkon 11F664 No. 1','','Sat',553,'',null from dual union all
select 66109,'1997-029','1997-06-10 12:01:00','orbital','success',308,'Y11      (45)','Feng Yun 2','Feng Yun Er Jia','','Sat',1904,'',null from dual union all
select 6611,'1960-A37','1960-02-08 11:02:00','suborbital rocket','success',725,'','','','','Meteo',1220,'VOKV',43 from dual union all
select 66110,'1997-030','1997-06-18 14:02:45','orbital','success',951,'390-02','Iridium','Iridium SV014','','Sat',587,'',null from dual union all
select 66111,'1997-031','1997-06-25 23:44:00','orbital','success',110,'V96 L466b','Intelsat 802','INTELSAT 802','','Sat',345,'',null from dual union all
select 66112,'1997-032','1997-07-01 18:02:00','orbital','success',1243,'STS-94','Spacelab MSL-1R','OV-102 Spacelab','','Sat',921,'MLP1',null from dual union all
select 66113,'1997-033','1997-07-05 04:11:54','orbital','success',1238,'S 15000-653','Progress M-35','Progress 7K-TGM No. 235','','Sat',480,'',null from dual union all
select 66114,'1997-034','1997-07-09 13:04:00','orbital','success',418,'D244','Iridium MS-2','Iridium SV015','','Sat',1724,'',null from dual union all
select 66115,'1997-035','1997-07-23 03:43:01','orbital','success',422,'D245','USA 132 (GPS IIR-2)','Navstar GPS SVN 43','','Sat',233,'',null from dual union all
select 66116,'1997-036','1997-07-28 01:15:01','orbital','success',174,'AC-133','Superbird C','Superbird C','','Sat',269,'',null from dual union all
select 66117,'1997-037','1997-08-01 20:20:00','orbital','success',911,'F16','OrbView-2','Seastar','','Sat',1718,'L-1011',null from dual union all
select 66118,'1997-038','1997-08-05 15:35:53','orbital','success',1238,'S 15000-655','Soyuz TM-26','Soyuz 7K-STM No. 75','','Sat',480,'',null from dual union all
select 66119,'1997-039','1997-08-07 14:41:00','orbital','success',1243,'STS-85','CRISTA/MFD/IEH','OV-103 Research/FF','','Sat',921,'MLP3',null from dual union all
select 6612,'1960-A38','1960-02-09 11:07:00','suborbital rocket','success',725,'','','','','Meteo',1221,'VOKV',42 from dual union all
select 66120,'1997-040','1997-08-08 06:46:00','orbital','success',110,'V98 L469e','PAS 6','PAS 6','','Sat',345,'',null from dual union all
select 66121,'1997-041','1997-08-14 20:49:14','orbital','success',958,'381-01','Kosmos-2345','Oko-S F6','','Sat',553,'',null from dual union all
select 66122,'1997-042','1997-08-19 17:50:00','orbital','success',313,'Y2       (46)','Agila 2','Mabuhay','','Sat',1903,'',null from dual union all
select 66123,'1997-043','1997-08-21 00:38:40','orbital','success',418,'D246','Iridium MS-3','Iridium SV026','','Sat',1724,'',null from dual union all
select 66124,'1997-044','1997-08-23 06:51:01','orbital','success',690,'LM-002','Lewis','SSTI/Lewis','','Sat',1780,'',null from dual union all
select 66125,'1997-045','1997-08-25 14:39:00','deep space','success',420,'D247','ACE','Advanced Composition Exp','','Sat',233,'',null from dual union all
select 66126,'1997-046','1997-08-28 00:33:30','orbital','success',960,'387-02','PAS 5','PAS 5','','Sat',587,'',null from dual union all
select 66127,'1997-047','1997-08-29 15:02:22','orbital','success',911,'019/F17','FORTE','FORTE','','Sat',1718,'L-1011',null from dual union all
select 66128,'1997-048','1997-09-01 14:00:15','orbital','success',302,'Y1       (47)','Iridium MFS','Iridium MFS 1','','Sat',1492,'',null from dual union all
select 66129,'1997-049','1997-09-02 22:21:07','orbital','success',108,'V99 L470g','Hot Bird 3/MTP','Eutelsat HB3','','Sat',345,'',null from dual union all
select 6613,'1960-A39','1960-02-10 00:00:00','military missile','success',771,'GM-54-56','','','','Test',222,'',12 from dual union all
select 66130,'1997-050','1997-09-04 12:03:00','orbital','success',174,'AC-146','GE 3','GE 3','','Sat',268,'',null from dual union all
select 66131,'1997-051','1997-09-14 01:36:54','orbital','success',951,'391-01','Iridium','Iridium SV029','','Sat',587,'',null from dual union all
select 66132,'1997-052','1997-09-23 16:44:51','orbital','success',680,'47165-637','Kosmos-2346/FAISAT','Parus','','Sat',456,'',null from dual union all
select 66133,'1997-053','1997-09-23 23:58:00','orbital','success',99,'V100 L471e','Intelsat 803','INTELSAT 803','','Sat',345,'',null from dual union all
select 66134,'1997-054','1997-09-24 21:30:59','orbital','success',805,'78051  -678','Molniya-1T','Molniya-1T No. 98','','Sat',474,'',null from dual union all
select 66135,'1997-055','1997-09-26 02:34:19','orbital','success',1243,'STS-86','Shuttle/Mir-7','OV-104 Shuttle/Mir','','Sat',921,'MLP2',null from dual union all
select 66136,'1997-056','1997-09-27 01:23:36','orbital','success',418,'D248','Iridium MS-4','Iridium SV019','','Sat',1724,'',null from dual union all
select 66137,'1997-057','1997-09-29 04:47:00','orbital','failure',899,'PSLV-C1','IRS-1D','IRS-1D','','Sat',1357,'',null from dual union all
select 66138,'1997-058','1997-10-05 15:08:57','orbital','success',1238,'S 15000-064','Progress M-36','RS-17','','Sat',480,'',null from dual union all
select 66139,'1997-059','1997-10-05 21:01:01','orbital','success',174,'AC-135','Echostar 3','Echostar 3','','Sat',269,'',null from dual union all
select 6614,'1960-A40','1960-02-10 12:00:00','suborbital rocket','success',725,'','','','','Meteo',1222,'VOKV',42 from dual union all
select 66140,'1997-060','1997-10-09 17:59:59','orbital','success',1238,'S 15000-057','Foton No. 11','Foton No. 11','','Sat',474,'',null from dual union all
select 66141,'1997-061','1997-10-15 08:43:01','deep space','success',1410,'4B-33/TC-21 (45E-13)','Cassini','Cassini','','Sat',274,'',null from dual union all
select 66142,'1997-062','1997-10-16 19:13:00','orbital','success',313,'Y3       (48)','Apstar 2R','Apstar 2R','','Sat',1903,'',null from dual union all
select 66143,'1997-063','1997-10-22 13:13:00','orbital','success',911,'F18','STEP 4','STEP M4','','Sat',1819,'L-1011',null from dual union all
select 66144,'1997-064','1997-10-24 02:32:00','orbital','success',1413,'4A-18 (K-18, 45F-3)','USA 133      [LAC3]','LACROSSE 3','','Sat',1777,'',null from dual union all
select 66145,'1997-065','1997-10-25 00:46:00','orbital','success',173,'AC-131','USA 135 (DSCS 3B-13)','DSCS III B-13','','Sat',268,'',null from dual union all
select 66146,'1997-066','1997-10-30 13:43:00','orbital','failure',114,'V101 (502)','Maqsat','TEAMSAT','','Sat',346,'',null from dual union all
select 66147,'1997-067','1997-11-06 00:30:00','orbital','success',422,'D249','USA 134 (GPS IIA-28)','Navstar GPS SVN 38','','Sat',233,'',null from dual union all
select 66148,'1997-068','1997-11-08 02:05:01','orbital','success',1409,'4A-17/TC-16 (45E-)','USA 136      [JS10]','TRUMPET 3','','Sat',275,'',null from dual union all
select 66149,'1997-069','1997-11-09 01:34:26','orbital','success',418,'D250','Iridium MS-5','Iridium SV043','','Sat',1724,'',null from dual union all
select 6615,'1960-A41','1960-02-11 00:00:00','military missile','success',764,'57-2445','','','','Test',243,'',12 from dual union all
select 66150,'1997-070','1997-11-12 17:00:00','orbital','success',959,'382-01','Kupon','K95K','','Sat',553,'',null from dual union all
select 66151,'1997-071','1997-11-12 21:48:00','orbital','success',105,'V102 L472d','Sirius 2/Cakrawarta','Sirius 2','','Sat',345,'',null from dual union all
select 66152,'1997-072','1997-11-18 11:14:59','orbital','success',1238,'S 15000-054','Resurs F-1M','Resurs F-1M','','Sat',474,'',null from dual union all
select 66153,'1997-073','1997-11-19 19:46:00','orbital','success',1243,'STS-87','Spartan 201/USMP-4','OV-102 Research/FF','','Sat',922,'MLP1',null from dual union all
select 66154,'1997-074','1997-11-27 21:27:00','orbital','success',526,'H-II-6F','TRMM/ETS-7','TRMM','','Sat',1477,'',null from dual union all
select 66155,'1997-075','1997-12-02 22:52:00','orbital','success',110,'V103 L473','JCSAT-5/Equator-S','JCSAT 5','','Sat',345,'',null from dual union all
select 66156,'1997-076','1997-12-02 23:10:37','orbital','success',960,'382-02','Astra 1G','Astra 1G','','Sat',587,'',null from dual union all
select 66157,'1997-077','1997-12-08 07:16:49','orbital','success',302,'Y2       (49)','Iridium 42/44','Iridium SV042','','Sat',1492,'',null from dual union all
select 66158,'1997-078','1997-12-08 23:52:01','orbital','success',174,'AC-149','Galaxy 8i','Galaxy 8-i','','Sat',269,'',null from dual;
insert into LAUNCH
select 66159,'1997-079','1997-12-09 07:17:20','orbital','success',1446,'670-33-  303','Kosmos-2347','US-PU','','Sat',590,'',null from dual union all
select 6616,'1960-A42','1960-02-11 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1223,'VOKV',42 from dual union all
select 66160,'1997-080','1997-12-15 15:40:00','orbital','success',1241,'76024  -763','Kosmos-2348','Kobal''t','','Sat',462,'',null from dual union all
select 66161,'1997-081','1997-12-20 08:45:02','orbital','success',1238,'S 15000-654','Progress M-37','Progress 7K-TGM No 236','','Sat',480,'',null from dual union all
select 66162,'1997-082','1997-12-20 13:16:00','orbital','success',418,'D251','Iridium MS-6','Iridium SV045','','Sat',1724,'',null from dual union all
select 66163,'1997-083','1997-12-22 00:16:00','orbital','success',99,'V104 L474b','Intelsat 804','INTELSAT 804','','Sat',345,'',null from dual union all
select 66164,'1997-084','1997-12-23 19:11:42','orbital','success',912,'F19','Orbcomm A1-A8','Orbcomm FM5','','Sat',1819,'L-1011',null from dual union all
select 66165,'1997-085','1997-12-24 13:32:52','orbital','success',1260,'','EarlyBird','EarlyBird','','Sat',478,'',1000 from dual union all
select 66166,'1997-086','1997-12-24 23:19:00','orbital','failure',960,'394-01','Asiasat 3','Asiasat 3','','Sat',587,'',null from dual union all
select 66167,'1997-A01','1997-01-01 00:00:00','military missile','failure',518,'','','','','Test',989,'',0 from dual union all
select 66168,'1997-A02','1997-01-08 00:00:00','suborbital rocket','success',591,'FPDP 3','HA-1','','','Test',1821,'',36 from dual union all
select 66169,'1997-A03','1997-01-24 15:32:00','military missile','success',1088,'Aegis DTR-1A','','','','Test',1869,'',10 from dual union all
select 6617,'1960-A43','1960-02-11 18:15:04','suborbital spaceplane','success',1520,'2-4-11','','','','Test',379,'NB-52 008',26 from dual union all
select 66170,'1997-A04','1997-01-29 00:00:00','military missile','failure',1263,'','MTTV-3/MTD-2 PAC Tgt','','','Target',1860,'',10 from dual union all
select 66171,'1997-A05','1997-01-30 21:55:00','military missile','failure',78,'','AltAir (Have Gold)','','','Test',1057,'C-130',15 from dual union all
select 66172,'1997-A06','1997-02-20 23:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',49 from dual union all
select 66173,'1997-A07','1997-03-27 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66174,'1997-A08','1997-03-27 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66175,'1997-A09','1997-03-27 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66176,'1997-A10','1997-03-27 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66177,'1997-A11','1997-03-27 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66178,'1997-A12','1997-03-27 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66179,'1997-A13','1997-03-27 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 6618,'1960-A44','1960-02-12 00:00:00','military missile','success',1214,'SM-62A N-3425','','','','OT',223,'',10 from dual union all
select 66180,'1997-A14','1997-03-27 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66181,'1997-A15','1997-03-27 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66182,'1997-A16','1997-03-27 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66183,'1997-A17','1997-03-28 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66184,'1997-A18','1997-03-28 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66185,'1997-A19','1997-03-28 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66186,'1997-A20','1997-03-28 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66187,'1997-A21','1997-03-28 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66188,'1997-A22','1997-03-28 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66189,'1997-A23','1997-03-28 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 6619,'1960-A45','1960-02-12 10:54:59','suborbital rocket','success',725,'','','','','Meteo',1224,'VOKV',35 from dual union all
select 66190,'1997-A24','1997-03-28 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66191,'1997-A25','1997-03-28 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66192,'1997-A26','1997-03-28 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66193,'1997-A27','1997-03-28 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',3 from dual union all
select 66194,'1997-A28','1997-04-10 13:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',45 from dual union all
select 66195,'1997-A29','1997-04-25 00:00:00','suborbital rocket','success',591,'FPDP 4','HA-2','','','Test',1821,'',33 from dual union all
select 66196,'1997-A30','1997-07-03 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',42 from dual union all
select 66197,'1997-A31','1997-08-08 13:24:30','suborbital rocket','failure',1334,'NASA 41.11UE','COORS/ROCKI (CRISTA)','','','Ionos/Aeron',1821,'',20 from dual union all
select 66198,'1997-A32','1997-08-12 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',33 from dual union all
select 66199,'1997-A33','1997-09-26 00:00:00','military missile','failure',1088,'Aegis SM-2-IV RRF','','','','Test',855,'',10 from dual union all
select 662,'1946-S10','1946-11-21 16:55:00','suborbital rocket','success',1470,'15','UM Aeron','','','Aeron/Ionos',1863,'',100 from dual union all
select 6620,'1960-A46','1960-02-13 00:00:00','military missile','failure',976,'','8K63 UBP-37','','','Test',636,'',0 from dual union all
select 66200,'1997-A34','1997-11-19 00:00:00','military missile','failure',518,'RSM-52V','','','','Test',989,'',0 from dual union all
select 66201,'1997-A35','1997-11-19 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',42 from dual union all
select 66202,'1997-A36','1997-12-03 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66203,'1997-A37','1997-12-03 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66204,'1997-A38','1997-12-03 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66205,'1997-A39','1997-12-03 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66206,'1997-A40','1997-12-03 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66207,'1997-A41','1997-12-03 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66208,'1997-A42','1997-12-03 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66209,'1997-A43','1997-12-03 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 6621,'1960-A47','1960-02-13 08:55:00','suborbital rocket','success',725,'','','','','Meteo',1225,'VOKV',42 from dual union all
select 66210,'1997-A44','1997-12-03 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66211,'1997-A45','1997-12-03 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66212,'1997-A46','1997-12-03 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66213,'1997-A47','1997-12-03 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66214,'1997-A48','1997-12-04 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66215,'1997-A49','1997-12-04 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66216,'1997-A50','1997-12-04 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66217,'1997-A51','1997-12-04 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66218,'1997-A52','1997-12-04 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66219,'1997-A53','1997-12-04 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 6622,'1960-A48','1960-02-16 20:48:00','suborbital rocket','failure',30,'NASA 04.01GT','','','','Test',1821,'',3 from dual union all
select 66220,'1997-A54','1997-12-04 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66221,'1997-A55','1997-12-04 00:00:00','military missile','success',1057,'','Arms control','','','Destruct',201,'TK-20',10 from dual union all
select 66222,'1997-A56','1997-12-18 23:15:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',49 from dual union all
select 66223,'1997-A57','1997-01-24 15:30:00','military missile','success',702,'Aegis Target','','','','Test',1895,'',40 from dual union all
select 66224,'1997-F01','1997-01-17 16:28:01','orbital','failure',422,'D241','GPS IIR-1','Navstar GPS SVN 42','','Sat',233,'',1 from dual union all
select 66225,'1997-F02','1997-05-20 07:07:00','orbital','failure',1537,'45013-302','','Tselina-2','','Sat',567,'',10 from dual union all
select 66226,'1997-F03','1997-11-02 12:25:00','orbital','failure',1475,'V01','SCD-2A','SCD-2A','','Sat',12,'',3 from dual union all
select 66227,'1997-M01','1997-03-11 00:00:00','military missile','success',119,'F4','','','','ATBM Test',1026,'',60 from dual union all
select 66228,'1997-M02','1997-08-19 08:20:00','military missile','success',118,'','','','','Target',1024,'SHIP-ISR',80 from dual union all
select 66229,'1997-M03','1997-08-19 08:29:00','military missile','failure',119,'F5','','','','ATBM Test',1026,'',1 from dual union all
select 6623,'1960-A49','1960-02-17 17:41:32','suborbital spaceplane','success',1520,'2-5-12','SAS Eval','','','Test',379,'NB-52 008',16 from dual union all
select 66230,'1997-M04','1997-11-17 13:10:00','military missile','failure',570,'8','MTV (Moving Target)','','','Target',436,'',70 from dual union all
select 66231,'1997-M05','1997-11-20 17:30:00','suborbital rocket','failure',1026,'F-110','ISBJORN-1','','','Ionos',1453,'',71 from dual union all
select 66232,'1997-M06','1997-10-24 00:00:00','military missile','success',75,'','Test 5b','','','Test',760,'',50 from dual union all
select 66233,'1997-M07','1997-05-11 12:25:00','suborbital rocket','success',537,'SL-1','Rockoon','','','Test',649,'Balloon',67 from dual union all
select 66234,'1997-M08','1997-01-01 00:00:00','military missile','success',608,'II-10','','','','Test',636,'',50 from dual union all
select 66235,'1997-S01','1997-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 66236,'1997-S02','1997-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 66237,'1997-S03','1997-01-15 00:00:00','military missile','success',1507,'','','','','OT',201,'KALMAR',1000 from dual union all
select 66238,'1997-S04','1997-01-17 01:17:00','military missile','success',796,'MSLS IFT-1','MSLS IFT-1','IFT-1 EKV Target','','Target',1698,'',1300 from dual union all
select 66239,'1997-S05','1997-01-30 07:30:00','suborbital rocket','success',1074,'S-520-18','Lunar Penetrator test','','','Tech',833,'',309 from dual union all
select 6624,'1960-A50','1960-02-19 00:00:00','suborbital rocket','success',1292,'','','','','Test',931,'',8 from dual union all
select 66240,'1997-S06','1997-01-30 08:42:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT163GB','GLORY TRIP 163GB','OT',1712,'',1300 from dual union all
select 66241,'1997-S07','1997-01-31 01:55:08','suborbital rocket','success',740,'AGRE (D75MG)','Flaksus-1 (Fluxus)','','','Plasma/Active',636,'',160 from dual union all
select 66242,'1997-S08','1997-02-05 02:16:07','suborbital rocket','success',740,'AGRE (D75MG)','Flaksus-2','','','Plasma/Active',636,'',160 from dual union all
select 66243,'1997-S09','1997-02-07 00:00:00','military missile','success',1151,'FMA','Willow Dune','','','Target',178,'',125 from dual union all
select 66244,'1997-S10','1997-02-10 05:17:15','suborbital rocket','success',237,'NASA 40.010UE','PHAZE 2','','','Ionos',1047,'',945 from dual union all
select 66245,'1997-S11','1997-02-10 06:30:00','suborbital rocket','success',1280,'RBIFF 1','','','','Test',1004,'',220 from dual union all
select 66246,'1997-S12','1997-02-11 08:36:41','suborbital rocket','success',237,'NASA 40.011UE','Auroral Turbulence 2','','','Ionos',1047,'',546 from dual union all
select 66247,'1997-S13','1997-02-12 21:00:00','suborbital rocket','success',686,'MSX CEP 1','MDT IV','','','Target',1821,'',400 from dual union all
select 66248,'1997-S14','1997-02-20 20:11:31','military missile','success',1442,'FCET-48','','','','OT',420,'SSBN 732',1000 from dual;
insert into LAUNCH
select 66249,'1997-S15','1997-02-20 20:11:49','military missile','success',1442,'FCET-48','','','','OT',420,'SSBN 732',1000 from dual union all
select 6625,'1960-A51','1960-02-19 17:18:00','suborbital rocket','failure',470,'CRL AA13.320C','','','','CR',394,'',37 from dual union all
select 66250,'1997-S16','1997-02-20 21:24:47','military missile','success',1442,'FCET-48','','','','OT',420,'SSBN 732',1000 from dual union all
select 66251,'1997-S17','1997-02-20 21:25:04','military missile','success',1442,'FCET-48','','','','OT',420,'SSBN 732',1000 from dual union all
select 66252,'1997-S18','1997-02-22 00:00:00','suborbital rocket','success',281,'TCMP II','IRLV','','','Reentry Test',1784,'',300 from dual union all
select 66253,'1997-S19','1997-02-23 06:22:00','military missile','success',942,'2-3','','','','Test',320,'',100 from dual union all
select 66254,'1997-S20','1997-02-23 10:06:00','suborbital rocket','success',686,'MSX CEP 2','MDT III','','','Target',1821,'',400 from dual union all
select 66255,'1997-S21','1997-02-26 20:00:00','suborbital rocket','success',224,'NASA 36.161NP','DARTFire 3','','','Microgravity',1873,'',300 from dual union all
select 66256,'1997-S22','1997-03-01 00:00:00','suborbital rocket','success',281,'TCMP IIC','IRLV','','','Reentry Test',1784,'',300 from dual union all
select 66257,'1997-S23','1997-03-06 12:40:00','military missile','success',570,'7','BRV/THAAD Target','','','Target',1857,'',300 from dual union all
select 66258,'1997-S24','1997-03-06 12:41:00','military missile','success',1293,'FT-07','','','','ABM Test',1895,'',100 from dual union all
select 66259,'1997-S25','1997-03-13 10:20:31','suborbital rocket','success',224,'NASA 36.139UE','','','','Ionos',1047,'',300 from dual union all
select 6626,'1960-A52','1960-02-20 00:00:00','military missile','success',268,'10-02','','','','Test',1752,'',15 from dual union all
select 66260,'1997-S26','1997-03-17 22:26:28','military missile','success',1443,'FCET-17','','','','OT',420,'SSBN 736',1000 from dual union all
select 66261,'1997-S27','1997-03-18 00:16:00','military missile','success',1443,'FCET-17','','','','OT',420,'SSBN 736',1000 from dual union all
select 66262,'1997-S28','1997-03-20 00:00:00','military missile','success',1151,'FMA','Willow Dune','','','Target',178,'',125 from dual union all
select 66263,'1997-S29','1997-03-25 03:15:00','suborbital rocket','success',224,'NASA 36.158UL','Hale-Bopp','','','UV Astronomy',1873,'',282 from dual union all
select 66264,'1997-S30','1997-03-30 03:25:00','suborbital rocket','success',224,'NASA 36.155CL','EUVS','','','UV Astronomy',1873,'',300 from dual union all
select 66265,'1997-S31','1997-04-06 03:45:00','suborbital rocket','success',213,'NASA 36.156UG','JHU FOT 13','','','UV Astronomy',1873,'',300 from dual union all
select 66266,'1997-S32','1997-04-08 03:55:00','suborbital rocket','success',213,'NASA 36.157UL','WISP 3','','','UV Astronomy',1873,'',375 from dual union all
select 66267,'1997-S33','1997-04-17 12:30:19','military missile','success',1006,'670-4833-207','15A18-01 UBP-32','GCh 15F183-3 (10 RV)','','OT',497,'',1000 from dual union all
select 66268,'1997-S34','1997-04-28 16:42:00','suborbital rocket','success',1478,'XV-01','DLR AL-VS30-223','','','Test',13,'',128 from dual union all
select 66269,'1997-S35','1997-04-06 00:00:00','suborbital rocket','success',1029,'','','','','Test',1361,'',420 from dual union all
select 6627,'1960-A53','1960-02-23 00:00:00','atmospheric rocket','success',705,'L1D','','','','Test',2,'',20 from dual union all
select 66270,'1997-S36','1997-05-02 07:30:00','suborbital rocket','success',224,'NASA 36.093UH','','','','XR Astronomy',1873,'',300 from dual union all
select 66271,'1997-S37','1997-05-08 07:10:00','suborbital rocket','success',224,'NASA 36.133UG','SPINR 1','','','UV Astronomy',1873,'',300 from dual union all
select 66272,'1997-S38','1997-05-08 08:01:00','military missile','success',907,'24PA (84-555)','Mk 21 RV x 9','GT 24PA','','OT',1700,'',1000 from dual union all
select 66273,'1997-S39','1997-05-15 19:16:00','suborbital rocket','success',224,'NASA 36.135AE','NCAR/CU-6 MXUVI','','','Solar UV',1873,'',300 from dual union all
select 66274,'1997-S40','1997-05-21 08:27:00','military missile','success',799,'','Mk 12/12A RV x 2/SHARP','FOT GT164GM/SHARP','GLORY TRIP 164GM','OT/Reentry Test',1699,'',1300 from dual union all
select 66275,'1997-S41','1997-05-23 21:44:00','suborbital rocket','success',1345,'NASA 41.13DT','MSX','','','Target',1821,'',150 from dual union all
select 66276,'1997-S417','1997-09-04 00:00:00','suborbital rocket','success',1029,'','','','','Test',1361,'',420 from dual union all
select 66277,'1997-S42','1997-05-29 04:56:00','suborbital rocket','success',224,'NASA 36.163UR','NITE 1','','','IR Astron',1873,'',345 from dual union all
select 66278,'1997-S43','1997-05-30 22:30:04','military missile','success',1443,'DASO-15','','','','OT',419,'SSBN 742',1000 from dual union all
select 66279,'1997-S44','1997-06-04 09:00:00','suborbital rocket','success',605,'SSC GYROTEST','SSC GYROTEST','','','Test',417,'',94 from dual union all
select 6628,'1960-A54','1960-02-23 00:00:00','suborbital rocket','failure',1490,'AGI23','','','','Aeronomy',668,'',50 from dual union all
select 66280,'1997-S45','1997-06-10 07:20:00','military missile','success',1453,'49238-21-118','15A35 UBP-46','GCh 15F355P7','','OT',508,'',1000 from dual union all
select 66281,'1997-S46','1997-06-18 10:01:00','military missile','success',799,'','Mk 12 RV x 3','FOT GT165GM','GLORY TRIP 165GM','OT',1706,'',1300 from dual union all
select 66282,'1997-S47','1997-06-24 03:39:00','military missile','success',796,'MSLS IFT-1A','MSLS IFT-1A','MSLS IFT-1A','','Target',1698,'',1600 from dual union all
select 66283,'1997-S48','1997-06-24 04:00:00','military missile','success',797,'PLV','IFT-1A EKV','','','ABM Test',868,'',200 from dual union all
select 66284,'1997-S49','1997-07-08 12:25:00','military missile','success',1436,'4L','15Zh65 LKI-4','','','Test',608,'',1000 from dual union all
select 66285,'1997-S50','1997-07-09 00:00:00','suborbital rocket','success',650,'KSR-II-1','','','','XR Astron/Ionos',33,'',150 from dual union all
select 66286,'1997-S51','1997-07-23 00:00:00','military missile','success',1507,'','','','','OT',1011,'PLBR',1000 from dual union all
select 66287,'1997-S52','1997-08-08 16:29:00','suborbital rocket','success',228,'NASA 21.120NE','METEORS','','','Ionos/Aeron',1895,'',250 from dual union all
select 66288,'1997-S53','1997-08-11 18:18:00','suborbital rocket','success',224,'NASA 36.164US','USC-2 SOHO Cal','','','Solar EUV',1895,'',307 from dual union all
select 66289,'1997-S54','1997-09-02 07:00:00','suborbital rocket','success',1334,'NASA 41.14DT','DUNDEE 1','','','Target',34,'',300 from dual union all
select 6629,'1960-A55','1960-02-26 00:00:00','military missile','failure',928,'A1X-13','','','','Test',251,'',10 from dual union all
select 66290,'1997-S55','1997-09-05 04:00:00','suborbital rocket','success',1334,'NASA 41.15DT','DUNDEE 2','','','Target',34,'',300 from dual union all
select 66291,'1997-S56','1997-09-10 02:00:00','suborbital rocket','success',1334,'NASA 41.16DT','DUNDEE 3','','','Target',34,'',300 from dual union all
select 66292,'1997-S57','1997-09-10 14:40:00','suborbital rocket','success',224,'NASA 36.165NP','','','','Microgravity',1895,'',300 from dual union all
select 66293,'1997-S58','1997-09-10 15:10:00','suborbital rocket','success',224,'NASA 36.169NM','DARTFire 4','','','Microgravity',1895,'',300 from dual union all
select 66294,'1997-S59','1997-09-11 20:15:30','suborbital rocket','success',1334,'NASA 41.17DT','DUNDEE 4 (MSX)','','','Target',34,'',300 from dual union all
select 66295,'1997-S60','1997-09-17 08:01:00','military missile','success',907,'25PA (86-448)','Mk 21 RV x 7','FDEP GT 25PA','','OT',1700,'',1000 from dual union all
select 66296,'1997-S61','1997-09-19 13:00:00','suborbital rocket','success',224,'NASA 36.144UE','','','','Ionos',1895,'',300 from dual union all
select 66297,'1997-S62','1997-09-24 00:00:00','military missile','success',1263,'','MTTV-4/Patriot Targ','','','Target',1860,'',110 from dual union all
select 66298,'1997-S63','1997-09-25 00:30:00','suborbital rocket','success',1296,'TR-1A-6','','','','Micrograv',1476,'',264 from dual union all
select 66299,'1997-S64','1997-09-30 20:10:00','suborbital rocket','success',224,'NASA 36.140DS','HRTS 10','','','Solar',1873,'',300 from dual union all
select 663,'1946-S11','1946-12-05 20:08:00','suborbital rocket','success',1470,'16','','','','Solar UV',1863,'',167 from dual union all
select 6630,'1960-A56','1960-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 66300,'1997-S65','1997-10-03 07:07:00','military missile','success',1434,'','15Zh58 UBP-21','','','Test',608,'',1000 from dual union all
select 66301,'1997-S66','1997-10-04 18:30:00','suborbital rocket','failure',1280,'RBIFF 2','','','','Test',1004,'',100 from dual union all
select 66302,'1997-S67','1997-10-10 20:05:03','military missile','success',1443,'DASO-5','','','','OT',420,'UK S30',1000 from dual union all
select 66303,'1997-S68','1997-10-10 00:00:00','military missile','success',1443,'DASO-6','','','','OT',420,'UK S30',1000 from dual union all
select 66304,'1997-S69','1997-10-12 01:19:00','suborbital rocket','success',1478,'XV-02 RONALD','DLR AL-VS30-226','','','Aeron',29,'',120 from dual union all
select 66305,'1997-S70','1997-10-16 19:00:00','suborbital rocket','success',224,'NASA 36.129DS','EIT-CALROC','','','Solar',1895,'',350 from dual union all
select 66306,'1997-S71','1997-11-01 00:00:00','military missile','success',295,'','','','','Test',1494,'',1000 from dual union all
select 66307,'1997-S72','1997-11-05 20:02:00','suborbital rocket','success',1334,'NASA 41.04IE','','','','Ionos',29,'',120 from dual union all
select 66308,'1997-S73','1997-11-05 21:01:00','military missile','success',907,'26PA (86-443)','Mk 21 RV x 8','FDEP GT 26PA','','OT',1697,'',1000 from dual union all
select 66309,'1997-S74','1997-11-14 05:00:00','suborbital rocket','success',224,'NASA 36.080UG','SCARI 1','','','UV Astronomy',1895,'',300 from dual union all
select 6631,'1960-A57','1960-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 66310,'1997-S75','1997-11-18 19:35:00','suborbital rocket','success',224,'NASA 36.167GS','SERTS-97 (08)','','','Solar EUV',1873,'',300 from dual union all
select 66311,'1997-S76','1997-12-02 08:42:00','suborbital rocket','success',224,'NASA 36.153IE','','','','Plasma',1453,'',433 from dual union all
select 66312,'1997-S77','1997-12-03 09:06:00','suborbital rocket','success',224,'NASA 36.152IE','','','','Plasma',1453,'',446 from dual union all
select 66313,'1997-S78','1997-12-18 00:00:00','military missile','success',1443,'DASO-16','-   (77ss)','','','OT',421,'SSBN 743',1000 from dual union all
select 66314,'1997-S79','1997-01-01 00:00:00','military missile','success',437,'-        -','','','','Test',1494,'',1000 from dual union all
select 66315,'1997-S80','1997-10-04 00:00:00','military missile','success',1514,'','Jubilee launch','','','OT',201,'PLBR',1000 from dual union all
select 66316,'1997-W01','1997-01-08 02:04:59','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66317,'1997-W02','1997-01-10 15:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 66318,'1997-W03','1997-01-13 02:00:00','suborbital rocket','success',759,'MT-135-65','NNP-O3','','','Aeron',839,'',55 from dual union all
select 66319,'1997-W04','1997-01-15 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 6632,'1960-A58','1960-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 66320,'1997-W05','1997-01-22 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66321,'1997-W06','1997-01-29 02:04:59','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66322,'1997-W07','1997-02-12 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66323,'1997-W08','1997-02-19 02:10:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66324,'1997-W09','1997-02-27 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 66325,'1997-W10','1997-03-05 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66326,'1997-W11','1997-03-19 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66327,'1997-W12','1997-03-26 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66328,'1997-W13','1997-04-02 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',60 from dual union all
select 66329,'1997-W14','1997-04-08 21:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',67 from dual union all
select 6633,'1960-A59','1960-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 66330,'1997-W15','1997-04-09 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',57 from dual union all
select 66331,'1997-W16','1997-04-11 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',60 from dual union all
select 66332,'1997-W17','1997-04-16 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66333,'1997-W18','1997-04-22 18:15:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',59 from dual union all
select 66334,'1997-W19','1997-04-30 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',53 from dual union all
select 66335,'1997-W20','1997-05-13 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 66336,'1997-W21','1997-05-22 02:03:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66337,'1997-W22','1997-05-28 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66338,'1997-W23','1997-05-29 14:40:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',58 from dual;
insert into LAUNCH
select 66339,'1997-W24','1997-06-11 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 6634,'1960-A60','1960-03-03 00:00:00','military missile','success',1214,'SM-62A N-3426','','','','OT',238,'',10 from dual union all
select 66340,'1997-W25','1997-06-18 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66341,'1997-W26','1997-06-19 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',65 from dual union all
select 66342,'1997-W27','1997-06-25 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66343,'1997-W28','1997-06-30 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',55 from dual union all
select 66344,'1997-W29','1997-07-10 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',53 from dual union all
select 66345,'1997-W30','1997-07-11 17:44:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',63 from dual union all
select 66346,'1997-W31','1997-07-15 00:00:00','suborbital rocket','success',1502,'','','','','Aeron',29,'',75 from dual union all
select 66347,'1997-W32','1997-07-15 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',54 from dual union all
select 66348,'1997-W33','1997-07-17 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',57 from dual union all
select 66349,'1997-W34','1997-07-23 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 6635,'1960-A61','1960-03-04 00:00:00','military missile','success',793,'','Silo Test','','','Test',382,'',0 from dual union all
select 66350,'1997-W35','1997-07-31 22:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',59 from dual union all
select 66351,'1997-W36','1997-08-04 22:55:00','suborbital rocket','success',1284,'Datasonde 11D','CRISTA','','','Aeron',1821,'',76 from dual union all
select 66352,'1997-W37','1997-08-04 23:28:00','suborbital rocket','success',1502,'Falling Sphere','AND-22 TTS07','','','Aeron',29,'',75 from dual union all
select 66353,'1997-W38','1997-08-05 22:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',60 from dual union all
select 66354,'1997-W39','1997-08-06 00:00:00','suborbital rocket','success',1284,'Datasonde 10D','CRISTA GMD-5000','','','Aeron',1821,'',75 from dual union all
select 66355,'1997-W40','1997-08-06 00:00:00','suborbital rocket','success',1284,'Datasonde 10D','CRISTA GMD-5000','','','Aeron',1821,'',75 from dual union all
select 66356,'1997-W41','1997-08-06 00:00:00','suborbital rocket','success',1284,'Datasonde 10D','CRISTA GMD-5000','','','Aeron',1821,'',75 from dual union all
select 66357,'1997-W42','1997-08-06 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',53 from dual union all
select 66358,'1997-W43','1997-08-06 13:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 66359,'1997-W44','1997-08-06 22:04:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',66 from dual union all
select 6636,'1960-A62','1960-03-06 00:00:00','military missile','success',268,'10-04','','','','Test',1752,'',15 from dual union all
select 66360,'1997-W45','1997-08-06 22:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',57 from dual union all
select 66361,'1997-W46','1997-08-07 23:02:00','suborbital rocket','success',1502,'Falling Sphere 6','AND-22 TTS08/TRAMP','','','Aeron',29,'',75 from dual union all
select 66362,'1997-W47','1997-08-08 16:00:00','suborbital rocket','success',1284,'Falling Sphere','METEORS/CRISTA','','','Aeron',1895,'',75 from dual union all
select 66363,'1997-W48','1997-08-08 16:00:00','suborbital rocket','success',1284,'Falling Sphere','METEORS/CRISTA','','','Aeron',1895,'',75 from dual union all
select 66364,'1997-W49','1997-08-09 23:12:00','suborbital rocket','success',1284,'Datasonde','CRISTA','','','Aeron',1821,'',75 from dual union all
select 66365,'1997-W50','1997-08-10 21:21:20','suborbital rocket','success',1284,'Datasonde','CRISTA','','','Aeron',1821,'',75 from dual union all
select 66366,'1997-W51','1997-08-10 21:22:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',70 from dual union all
select 66367,'1997-W52','1997-08-10 23:51:00','suborbital rocket','success',1502,'Falling Sphere 7','AND-22 TTS09/TRAMP','','','Aeron',29,'',75 from dual union all
select 66368,'1997-W53','1997-08-11 00:16:00','suborbital rocket','success',1502,'Chaff 3','AND-22 TTC10/TRAMP','','','Aeron',29,'',75 from dual union all
select 66369,'1997-W54','1997-08-11 21:10:00','suborbital rocket','success',1284,'Datasonde','CRISTA','','','Aeron',1821,'',78 from dual union all
select 6637,'1960-A63','1960-03-08 18:00:00','military missile','failure',1419,'C-1','P13 RV','','','Test',231,'',100 from dual union all
select 66370,'1997-W55','1997-08-11 21:19:00','suborbital rocket','success',1284,'Datasonde','CRISTA','','','Aeron',1821,'',75 from dual union all
select 66371,'1997-W56','1997-08-12 21:30:00','suborbital rocket','success',1284,'Datasonde','CRISTA','','','Aeron',1821,'',77 from dual union all
select 66372,'1997-W57','1997-08-13 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',52 from dual union all
select 66373,'1997-W58','1997-08-14 20:00:00','suborbital rocket','success',1502,'Sphere 8','AND-22 CRISTA TTS11','','','Aeron',29,'',75 from dual union all
select 66374,'1997-W59','1997-08-14 20:22:00','suborbital rocket','success',1502,'Chaff 4','AND-22 CRISTA TTC12','','','Aeron',29,'',101 from dual union all
select 66375,'1997-W60','1997-08-14 21:27:10','suborbital rocket','success',1284,'Datasonde','CRISTA','','','Aeron',1821,'',75 from dual union all
select 66376,'1997-W61','1997-08-15 10:53:00','suborbital rocket','failure',1284,'','CRISTA','','','Aeron',1821,'',40 from dual union all
select 66377,'1997-W62','1997-08-18 19:30:00','suborbital rocket','success',1284,'','CRISTA','','','Aeron',1821,'',78 from dual union all
select 66378,'1997-W63','1997-08-18 19:40:00','suborbital rocket','success',1284,'','CRISTA','','','Aeron',1821,'',74 from dual union all
select 66379,'1997-W64','1997-08-22 20:10:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',70 from dual union all
select 6638,'1960-A64','1960-03-09 00:00:00','military missile','success',771,'GM-54-58','','','','Test',222,'',12 from dual union all
select 66380,'1997-W65','1997-08-27 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66381,'1997-W66','1997-08-27 19:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',1821,'',54 from dual union all
select 66382,'1997-W67','1997-09-08 00:00:00','suborbital rocket','success',1502,'','','','','Aeron',29,'',75 from dual union all
select 66383,'1997-W68','1997-09-08 02:00:00','suborbital rocket','success',759,'MT-135-66','','','','Aeron',839,'',50 from dual union all
select 66384,'1997-W69','1997-09-09 02:00:00','suborbital rocket','success',759,'MT-135-67','','','','Aeron',839,'',52 from dual union all
select 66385,'1997-W70','1997-09-10 02:10:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66386,'1997-W71','1997-09-12 13:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',64 from dual union all
select 66387,'1997-W72','1997-09-18 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66388,'1997-W73','1997-10-01 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',52 from dual union all
select 66389,'1997-W74','1997-10-08 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 6639,'1960-A65','1960-03-11 00:36:00','military missile','failure',154,'51D','Mk 3-1A','R&D','','Test',228,'',0 from dual union all
select 66390,'1997-W75','1997-10-10 13:06:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',59 from dual union all
select 66391,'1997-W76','1997-10-22 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66392,'1997-W77','1997-10-30 15:24:07','suborbital rocket','success',1284,'DLR KO-SL-227','','','','Aeron',350,'',75 from dual union all
select 66393,'1997-W78','1997-10-30 17:00:00','suborbital rocket','success',1284,'DLR KO-SL-228','','','','Aeron',350,'',75 from dual union all
select 66394,'1997-W79','1997-11-05 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66395,'1997-W80','1997-11-12 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66396,'1997-W81','1997-12-03 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',53 from dual union all
select 66397,'1997-W82','1997-12-10 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66398,'1997-W83','1997-12-17 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66399,'1997-W84','1997-12-24 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 664,'1946-S12','1946-12-18 05:12:49','suborbital rocket','success',1470,'17','CR/Grenades','','','CR/Meteor test',1863,'',187 from dual union all
select 6640,'1960-A66','1960-03-12 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1226,'VOKV',41 from dual union all
select 66400,'1997-Y01','1997-01-14 12:00:00','suborbital rocket','success',1502,'','Chaff','','','Aeron',839,'',100 from dual union all
select 66401,'1997-Y02','1997-01-14 13:15:00','suborbital rocket','success',1502,'','Chaff','','','Aeron',839,'',100 from dual union all
select 66402,'1997-Y03','1997-02-09 14:49:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',99 from dual union all
select 66403,'1997-Y04','1997-03-19 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',99 from dual union all
select 66404,'1997-Y05','1997-06-06 14:03:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',99 from dual union all
select 66405,'1997-Y06','1997-07-29 12:59:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',99 from dual union all
select 66406,'1997-Y07','1997-08-08 13:18:59','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',96 from dual union all
select 66407,'1997-Y08','1997-08-08 13:19:30','suborbital rocket','success',1284,'Falling Sphere','CRISTA','','','Aeron',1821,'',120 from dual union all
select 66408,'1997-Y09','1997-08-09 13:29:00','suborbital rocket','success',1284,'Falling Sphere','CRISTA','','','Aeron',1821,'',105 from dual union all
select 66409,'1997-Y10','1997-08-09 22:57:00','suborbital rocket','success',1284,'Falling Sphere','CRISTA','','','Aeron',1821,'',110 from dual union all
select 6641,'1960-A67','1960-03-15 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1227,'VOKV',42 from dual union all
select 66410,'1997-Y11','1997-08-10 09:35:59','suborbital rocket','success',1061,'','','','','Meteo',1821,'',95 from dual union all
select 66411,'1997-Y12','1997-08-10 13:36:00','suborbital rocket','success',1502,'Falling Sphere','CRISTA','','','Aeron',1821,'',123 from dual union all
select 66412,'1997-Y13','1997-08-10 21:30:10','suborbital rocket','success',1284,'Falling Sphere','CRISTA','','','Aeron',1821,'',111 from dual union all
select 66413,'1997-Y14','1997-08-10 21:31:00','suborbital rocket','success',1061,'','AN/DMQ-9-','','','Meteo',1821,'',94 from dual union all
select 66414,'1997-Y15','1997-08-11 13:38:00','suborbital rocket','success',1284,'Falling Sphere','CRISTA','','','Aeron',1821,'',107 from dual union all
select 66415,'1997-Y16','1997-08-11 13:43:00','suborbital rocket','success',1502,'Falling Sphere','CRISTA','','','Aeron',1821,'',119 from dual union all
select 66416,'1997-Y17','1997-08-11 21:34:00','suborbital rocket','success',1284,'Falling Sphere','CRISTA','','','Aeron',1821,'',106 from dual union all
select 66417,'1997-Y18','1997-08-12 12:10:00','suborbital rocket','success',1284,'Falling Sphere','CRISTA','','','Aeron',1821,'',111 from dual union all
select 66418,'1997-Y19','1997-08-12 12:15:00','suborbital rocket','success',1502,'Falling Sphere','CRISTA','','','Aeron',1821,'',124 from dual union all
select 66419,'1997-Y20','1997-08-12 12:26:06','suborbital rocket','success',1284,'Falling Sphere','CRISTA','','','Aeron',1821,'',118 from dual union all
select 6642,'1960-A68','1960-03-17 01:52:00','suborbital rocket','failure',838,'NASA 03.10UI','Dumbbell/Ion sphere','','','Ionos',428,'',0 from dual union all
select 66420,'1997-Y21','1997-08-12 21:41:00','suborbital rocket','success',1284,'Falling Sphere','CRISTA','','','Aeron',1821,'',112 from dual union all
select 66421,'1997-Y22','1997-08-13 13:50:00','suborbital rocket','success',1284,'Sphere','CRISTA','','','Aeron',1821,'',117 from dual union all
select 66422,'1997-Y23','1997-08-13 13:55:00','suborbital rocket','success',1284,'Sphere','CRISTA','','','Aeron',1821,'',121 from dual union all
select 66423,'1997-Y24','1997-08-13 21:23:00','suborbital rocket','success',1284,'Datasonde','CRISTA','','','Aeron',1821,'',80 from dual union all
select 66424,'1997-Y25','1997-08-13 21:40:00','suborbital rocket','success',1284,'Sphere','CRISTA','','','Aeron',1821,'',121 from dual union all
select 66425,'1997-Y26','1997-08-14 12:22:00','suborbital rocket','success',1284,'Sphere','CRISTA','','','Aeron',1821,'',115 from dual union all
select 66426,'1997-Y27','1997-08-14 12:27:00','suborbital rocket','success',1502,'Sphere','CRISTA','','','Aeron',1821,'',125 from dual union all
select 66427,'1997-Y28','1997-08-14 21:53:00','suborbital rocket','success',1502,'Sphere','CRISTA','','','Aeron',1821,'',124 from dual union all
select 66428,'1997-Y29','1997-08-15 10:58:00','suborbital rocket','success',1502,'','CRISTA','','','Aeron',1821,'',115 from dual;
insert into LAUNCH
select 66429,'1997-Y30','1997-08-15 20:25:00','suborbital rocket','success',1284,'','CRISTA','','','Aeron',1821,'',121 from dual union all
select 6643,'1960-A69','1960-03-17 16:31:25','suborbital spaceplane','success',1520,'2-6-13','SAS Eval/Stability','','','Test',379,'NB-52 008',16 from dual union all
select 66430,'1997-Y31','1997-09-24 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',99 from dual union all
select 66431,'1997-Y32','1997-10-09 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',99 from dual union all
select 66432,'1997-Y33','1997-10-10 22:24:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',99 from dual union all
select 66433,'1997-Y34','1997-10-21 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',83 from dual union all
select 66434,'1997-Y35','1997-11-18 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',99 from dual union all
select 66435,'1997-Y36','1997-11-20 19:34:00','suborbital rocket','success',1502,'F-111 ISBJORN-2','Mini-DUSTY MD-05','','','Aeron',1453,'',97 from dual union all
select 66436,'1997-Y37','1997-05-22 00:00:00','suborbital rocket','success',1502,'','Mini-DUSTY MD-04','','','Aeron',29,'',100 from dual union all
select 66437,'1998-001','1998-01-07 02:28:44','deep space','success',138,'LM-004','Lunar Prospector','LP','','Sat',1425,'',null from dual union all
select 66438,'1998-002','1998-01-10 00:32:01','orbital','success',427,'D252','Skynet 4D','Skynet 4D','','Sat',234,'',null from dual union all
select 66439,'1998-003','1998-01-23 02:48:15','orbital','success',1243,'STS-89','Shuttle/Mir-8','OV-105 Shuttle/Mir','','Sat',921,'MLP3',null from dual union all
select 6644,'1960-A70','1960-03-18 00:00:00','suborbital spaceplane','failure',1520,'2-A-14','','','','Test',379,'NB-52 008',14 from dual union all
select 66440,'1998-004','1998-01-29 16:33:42','orbital','success',1241,'S 15000-657','Soyuz TM-27','Soyuz 7K-STM No. 76','','Sat',480,'',null from dual union all
select 66441,'1998-005','1998-01-29 18:37:00','orbital','success',173,'AC-109','USA 137      [SDS12]','QUASAR 12?','','Sat',288,'',null from dual union all
select 66442,'1998-006','1998-02-04 23:29:00','orbital','success',108,'V105 L475d','Brasil B3/Inmar 3F5','Brasilsat B3','','Sat',345,'',null from dual union all
select 66443,'1998-007','1998-02-10 13:20:00','orbital','success',1315,'T2','GFO/Orbcomm','GFO','','Sat',1670,'',null from dual union all
select 66444,'1998-008','1998-02-14 14:34:00','orbital','success',413,'D253','Globalstar 1-4','Globalstar FM1','','Sat',233,'',null from dual union all
select 66445,'1998-009','1998-02-17 10:35:00','orbital','success',1241,'A 15000-046','Kosmos-2349','Kometa No. 19','','Sat',562,'',null from dual union all
select 66446,'1998-010','1998-02-18 13:58:09','orbital','success',418,'D254','Iridium MS-7','Iridium SV052','','Sat',1724,'',null from dual union all
select 66447,'1998-011','1998-02-21 07:55:00','orbital','success',526,'H-II-5F','COMETS','COMETS','','Sat',1477,'',null from dual union all
select 66448,'1998-012','1998-02-26 07:07:00','orbital','success',911,'F20','SNOE/T1','SNOE','','Sat',1717,'L-1011',null from dual union all
select 66449,'1998-013','1998-02-27 22:38:00','orbital','success',102,'V106 L476c','Hot Bird 4','Eutelsat HB4','','Sat',345,'',null from dual union all
select 6645,'1960-A71','1960-03-18 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1228,'VOKV',45 from dual union all
select 66450,'1998-014','1998-02-28 00:21:01','orbital','success',174,'AC-151','Intelsat 806','INTELSAT 806','','Sat',289,'',null from dual union all
select 66451,'1998-015','1998-03-14 22:45:55','orbital','success',1241,'S 15000-656','Progress M-38','Progress 7K-TGM No 240','','Sat',480,'',null from dual union all
select 66452,'1998-016','1998-03-16 21:32:00','orbital','success',172,'AC-132','USA 138 (UHF F8)','UFO F8','','Sat',288,'',null from dual union all
select 66453,'1998-017','1998-03-24 01:46:00','orbital','success',96,'V107 L477b','SPOT 4','SPOT-4','','Sat',345,'',null from dual union all
select 66454,'1998-018','1998-03-25 17:01:06','orbital','success',302,'Y3       (50)','Iridium 51/61','Iridium SV051','','Sat',1492,'',null from dual union all
select 66455,'1998-019','1998-03-30 06:02:46','orbital','success',418,'D255','Iridium MS-8','Iridium SV055','','Sat',1724,'',null from dual union all
select 66456,'1998-020','1998-04-02 02:42:39','orbital','success',911,'F21','TRACE','TRACE','','Sat',1718,'L-1011',null from dual union all
select 66457,'1998-021','1998-04-07 02:13:03','orbital','success',951,'391-02','Iridium','Iridium SV062','','Sat',587,'',null from dual union all
select 66458,'1998-022','1998-04-17 18:19:00','orbital','success',1243,'STS-90','Neurolab','OV-102 Spacelab','','Sat',922,'MLP2',null from dual union all
select 66459,'1998-023','1998-04-24 22:38:34','orbital','success',413,'D256','Globalstar 5-8','Globalstar M014','','Sat',233,'',null from dual union all
select 6646,'1960-A72','1960-03-22 00:00:00','suborbital rocket','failure',131,'Geodetic flares','','','','Test',394,'',0 from dual union all
select 66460,'1998-024','1998-04-28 22:53:00','orbital','success',110,'V108 L478b','Nilesat 101/BSAT 1b','Nilesat 101','','Sat',345,'',null from dual union all
select 66461,'1998-025','1998-04-29 04:36:54','orbital','success',958,'384-02','Kosmos-2350','Oko-1 No. 7122','','Sat',553,'',null from dual union all
select 66462,'1998-026','1998-05-02 09:16:53','orbital','success',302,'Y4       (51)','Iridium 69/71','Iridium SV069','','Sat',1492,'',null from dual union all
select 66463,'1998-027','1998-05-07 08:53:22','orbital','success',805,'78051  -676','Kosmos-2351','Oko','','Sat',462,'',null from dual union all
select 66464,'1998-028','1998-05-07 23:45:00','orbital','success',960,'393-02','Echostar 4','Echostar 4','','Sat',587,'',null from dual union all
select 66465,'1998-029','1998-05-09 01:38:01','orbital','success',1410,'4B-25/TC-18 (K-25)','USA 139      [RH8]','ORION 4','','Sat',274,'',null from dual union all
select 66466,'1998-030','1998-05-13 15:52:04','orbital','success',1422,'23G-12','NOAA 15','NOAA K','','Sat',1778,'',null from dual union all
select 66467,'1998-031','1998-05-14 22:12:59','orbital','success',1241,'M 15000-661','Progress M-39','Progress 7K-TGM No 238','','Sat',480,'',null from dual union all
select 66468,'1998-032','1998-05-17 21:16:56','orbital','success',418,'D257','Iridium MS-9','Iridium SV070','','Sat',1724,'',null from dual union all
select 66469,'1998-033','1998-05-30 10:00:04','orbital','success',313,'Y5       (52)','Zhongwei 1','Chinastar 1','','Sat',1903,'',null from dual union all
select 6647,'1960-A73','1960-03-23 00:00:00','military missile','success',268,'10-03','','','','Test',1752,'',25 from dual union all
select 66470,'1998-034','1998-06-02 22:06:24','orbital','success',1243,'STS-91','Shuttle/Mir-9','OV-103 Shuttle/Mir','','Sat',921,'MLP1',null from dual union all
select 66471,'1998-035','1998-06-10 00:35:01','orbital','success',427,'D258','Thor III','Thor 3','','Sat',233,'',null from dual union all
select 66472,'1998-036','1998-06-15 22:58:05','orbital','success',1448,'450-91-  702','Kosmos-2352/2357','Strela-3','','Sat',469,'',null from dual union all
select 66473,'1998-037','1998-06-18 22:48:01','orbital','success',174,'AC-153','Intelsat 805','INTELSAT 805','','Sat',288,'',null from dual union all
select 66474,'1998-038','1998-06-24 18:29:58','orbital','success',1241,'Kh15000-043','Kosmos-2358','Kobal''t','','Sat',474,'',null from dual union all
select 66475,'1998-039','1998-06-25 14:00:00','orbital','success',1241,'A 15000-499','Kosmos-2359','Neman','','Sat',562,'',null from dual union all
select 66476,'1998-040','1998-07-01 00:48:01','orbital','success',805,'77017  -682','Molniya-3','Molniya-3 No. 61','','Sat',474,'',null from dual union all
select 66477,'1998-041','1998-07-03 18:12:00','deep space','success',731,'M-V-3','PLANET B/KM','Planet B','','Sat',836,'',null from dual union all
select 66478,'1998-042','1998-07-07 03:15:00','orbital','success',1180,'','TUBSAT-N','Tubsat-N','','Sat',200,'K-407',null from dual union all
select 66479,'1998-043','1998-07-10 06:30:00','orbital','success',1537,'67021-803','Resurs-O1 No. 4','Resurs-O1 No. 4L','','Sat',567,'',null from dual union all
select 6648,'1960-A74','1960-03-24 00:00:00','atmospheric rocket','success',1163,'','Mercury model','','','Reentry Test',1821,'',34 from dual union all
select 66480,'1998-044','1998-07-18 09:20:00','orbital','success',313,'Y4       (53)','Sinosat','xinnuo yihao','','Sat',1903,'',null from dual union all
select 66481,'1998-045','1998-07-28 09:15:00','orbital','success',1537,'45092-803','Kosmos-2360','Tselina-2','','Sat',567,'',null from dual union all
select 66482,'1998-046','1998-08-02 16:24:00','orbital','success',912,'F22','Orbcomm B1-B8','Orbcomm FM17','','Sat',1819,'L-1011',null from dual union all
select 66483,'1998-047','1998-08-13 09:43:11','orbital','success',1241,'M 15000-658','Soyuz TM-28','Soyuz 7K-STM No. 77','','Sat',480,'',null from dual union all
select 66484,'1998-048','1998-08-19 23:01:46','orbital','success',302,'Y5       (54)','Iridium 78/76','Iridium SV078','','Sat',1492,'',null from dual union all
select 66485,'1998-049','1998-08-25 23:07:00','orbital','success',110,'V109 L479c','ST-1','Singapore-Taiwan 1','','Sat',345,'',null from dual union all
select 66486,'1998-050','1998-08-30 00:31:00','orbital','success',960,'383-01','Astra 2A','Astra 2A','','Sat',587,'',null from dual union all
select 66487,'1998-051','1998-09-08 21:13:00','orbital','success',418,'D260','Iridium MS-10','Iridium SV082','','Sat',1724,'',null from dual union all
select 66488,'1998-052','1998-09-16 06:31:00','orbital','success',108,'V110 L480f','PAS 7','PAS 7','','Sat',345,'',null from dual union all
select 66489,'1998-053','1998-09-23 05:06:00','orbital','success',912,'F23','Orbcomm C1-C8','Orbcomm FM21','','Sat',1819,'L-1011',null from dual union all
select 6649,'1960-A75','1960-03-24 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1229,'VOKV',43 from dual union all
select 66490,'1998-054','1998-09-28 23:41:27','orbital','success',805,'Kh15000-030','Molniya-1T','Molniya-1T No. 99','','Sat',474,'',null from dual union all
select 66491,'1998-055','1998-10-03 10:04:49','orbital','success',18,'T3 1110','STEX/USA 141','STEX','','Sat',1670,'',null from dual union all
select 66492,'1998-056','1998-10-05 22:51:00','orbital','success',105,'V111 L481f','Eutelsat W2/Sirius 3','Eutelsat W2','','Sat',345,'',null from dual union all
select 66493,'1998-057','1998-10-09 22:50:01','orbital','success',173,'AC-134','Hot Bird 5','Eutelsat HB5','','Sat',289,'',null from dual union all
select 66494,'1998-058','1998-10-20 07:19:02','orbital','success',173,'AC-130','USA 140 (UHF F9)','UFO F9','','Sat',288,'',null from dual union all
select 66495,'1998-059','1998-10-21 16:37:21','orbital','success',114,'V112 (503)','ARD/Maqsat-3','Maqsat-3','','Sat',346,'',null from dual union all
select 66496,'1998-060','1998-10-23 00:02:00','orbital','success',910,'F24/P-33','SCD-2','SCD-2','','Sat',284,'L-1011',null from dual union all
select 66497,'1998-061','1998-10-24 12:08:01','deep space','success',412,'D261','Deep Space 1','Deep Space 1','','Sat',285,'',null from dual union all
select 66498,'1998-062','1998-10-25 04:14:57','orbital','success',1241,'M 15000-660','Progress M-40','Progress 7K-TGM No 239','','Sat',480,'',null from dual union all
select 66499,'1998-063','1998-10-28 22:15:00','orbital','success',105,'V113 L482d','Afristar/GE 5','Afristar','','Sat',345,'',null from dual union all
select 665,'1947-A01','1947-01-10 22:30:00','atmospheric rocket','failure',875,'46-8','Nike-46 7R','','','Test',1863,'',0 from dual union all
select 6650,'1960-A76','1960-03-25 23:43:23','suborbital spaceplane','success',1520,'1-3-8','','','','Test',379,'NB-52 008',14 from dual union all
select 66500,'1998-064','1998-10-29 19:19:34','orbital','success',1243,'STS-95','Spartan 201/Spacehab','OV-103 Research/FF','','Sat',922,'MLP2',null from dual union all
select 66501,'1998-065','1998-11-04 05:12:00','orbital','success',960,'395-02','PAS 8','PAS 8','','Sat',587,'',null from dual union all
select 66502,'1998-066','1998-11-06 13:37:52','orbital','success',418,'D262','Iridium MS-11','Iridium SV087','','Sat',1724,'',null from dual union all
select 66503,'1998-067','1998-11-20 06:40:00','orbital','success',949,'395-01','Zarya','77KM No. 17501','','Sat ISS 1A',587,'',null from dual union all
select 66504,'1998-068','1998-11-22 23:54:00','orbital','success',427,'D263','BONUM-1','BONUM-1','','Sat',286,'',null from dual union all
select 66505,'1998-069','1998-12-04 08:35:34','orbital','success',1243,'STS-88','ISS 2A (Unity)','OV-105 ISS','','Sat',921,'MLP3',null from dual union all
select 66506,'1998-070','1998-12-06 00:43:00','orbital','success',99,'V114 L483c','Satmex 5','Satmex 5','','Sat',345,'',null from dual union all
select 66507,'1998-071','1998-12-06 00:57:54','orbital','success',911,'F25','SWAS','SWAS','','Sat',1718,'L-1011',null from dual union all
select 66508,'1998-072','1998-12-10 11:57:07','orbital','success',680,'53713-320','Nadezhda/Astrid-2','Nadezhda 17F118','','Sat',456,'',null from dual union all
select 66509,'1998-073','1998-12-11 18:45:52','deep space','success',415,'D264','Mars Climate Orbiter','MCO','','Sat',285,'',null from dual union all
select 6651,'1960-A77','1960-03-27 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1230,'VOKV',45 from dual union all
select 66510,'1998-074','1998-12-19 11:39:44','orbital','success',302,'Y6       (55)','Iridium 88/89','Iridium SV088?','','Sat',1492,'',null from dual union all
select 66511,'1998-075','1998-12-22 01:08:00','orbital','success',99,'V115 L484d','PAS 6B','PAS 6B','','Sat',345,'',null from dual union all
select 66512,'1998-076','1998-12-24 20:02:19','orbital','success',680,'47164-504','Kosmos-2361','Parus','','Sat',456,'',null from dual union all
select 66513,'1998-077','1998-12-30 18:35:46','orbital','success',958,'385-02','Kosmos-2362/2364','Uragan No. 86L','','Sat',553,'',null from dual union all
select 66514,'1998-A01','1998-05-12 11:20:00','military missile','failure',1293,'FT-08','','','','ABM Test',1895,'',0 from dual union all
select 66515,'1998-A02','1998-07-22 00:00:00','military missile','failure',1168,'','','','','Test',1356,'',50 from dual union all
select 66516,'1998-A03','1998-10-22 12:53:00','military missile','failure',1436,'00354-591-01(5L)','15Zh65 LKI-5','','','Test',608,'',0 from dual union all
select 66517,'1998-A04','1998-01-25 05:00:00','suborbital rocket','failure',1502,'','Chaff','','','Aeron',839,'',10 from dual union all
select 66518,'1998-A05','1998-01-25 07:00:00','suborbital rocket','failure',1502,'','Chaff','','','Aeron',839,'',10 from dual;
insert into LAUNCH
select 66519,'1998-A06','1998-02-21 10:00:00','military missile','failure',75,'','Test 11','','','Test',760,'',0 from dual union all
select 6652,'1960-A78','1960-03-29 17:59:28','suborbital spaceplane','success',1520,'2-7-15','','','','Test',379,'NB-52 008',15 from dual union all
select 66520,'1998-A07','1998-10-20 12:50:00','military missile','failure',75,'','Test 47','','','Test',760,'',0 from dual union all
select 66521,'1998-A08','1998-10-07 13:35:00','atmospheric rocket','success',471,'','','','','Test',417,'',5 from dual union all
select 66522,'1998-F01','1998-01-22 12:56:00','orbital','failure',1174,'4','''Ofeq-4','EROS A','','Sat',1026,'',150 from dual union all
select 66523,'1998-F02','1998-08-12 11:30:01','orbital','failure',1409,'4A-20/TC-9 (K-17)','-            [CY16]','MERCURY 3','','Sat',275,'',5 from dual union all
select 66524,'1998-F03','1998-08-27 01:17:00','orbital','failure',429,'D259','Galaxy X','Galaxy X','','Sat',234,'',40 from dual union all
select 66525,'1998-F04','1998-08-31 03:07:00','orbital','failure',904,'','Kwangmyongsong-1','','','Sat',1480,'',200 from dual union all
select 66526,'1998-F05','1998-09-09 20:29:00','orbital','failure',1538,'67047-801/22D','Globalstar','Globalstar FM5','','Sat',567,'',200 from dual union all
select 66527,'1998-M01','1998-03-02 13:01:00','military missile','success',570,'9','MBRV-3','','','Target',436,'',100 from dual union all
select 66528,'1998-M02','1998-09-14 12:41:00','military missile','success',119,'F6','','','','ATBM Test',1026,'',60 from dual union all
select 66529,'1998-M03','1998-12-14 00:00:00','military missile','failure',570,'11','PAC-3 SCF','','','Target',436,'',70 from dual union all
select 6653,'1960-A79','1960-03-30 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1231,'VOKV',45 from dual union all
select 66530,'1998-M04','1998-02-20 13:23:00','military missile','success',75,'','Test 20','','','Test',760,'',50 from dual union all
select 66531,'1998-M05','1998-03-30 00:00:00','military missile','success',75,'','Test 10','','','Test',760,'',50 from dual union all
select 66532,'1998-M06','1998-08-11 09:40:00','military missile','success',75,'','Test 44','','','Test',760,'',50 from dual union all
select 66533,'1998-S01','1998-01-01 00:00:00','military missile','unknown',646,'','','','','Reentry Test',621,'',675 from dual union all
select 66534,'1998-S02','1998-01-16 03:25:00','military missile','success',796,'MSLS IFT-2','MSLS IFT-2','MSLS IFT-2','','Target',1698,'',1600 from dual union all
select 66535,'1998-S03','1998-01-16 03:46:00','military missile','success',797,'PLV','IFT-2 EKV','','','ABM Test',868,'',300 from dual union all
select 66536,'1998-S04','1998-01-25 08:35:00','suborbital rocket','success',1073,'S-310-27','SEEK Ozone','','','Aeron',833,'',183 from dual union all
select 66537,'1998-S05','1998-01-26 12:26:00','suborbital rocket','success',847,'DLR K-NO2-230','SSC MERMAID','','','Microgravity/Education',417,'',132 from dual union all
select 66538,'1998-S06','1998-01-31 04:30:00','suborbital rocket','success',1074,'S-520-22','SOLAR-B XDT','','','Solar XUV',833,'',270 from dual union all
select 66539,'1998-S07','1998-01-31 23:43:00','suborbital rocket','success',1478,'XV-03 RONALD 2','DLR AL-VS30-229','','','Aeron',29,'',120 from dual union all
select 6654,'1960-A80','1960-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 66540,'1998-S08','1998-02-05 08:30:00','suborbital rocket','success',1101,'SS-520-1','ENA/EPS/LAP','','','Test/Plasma',833,'',750 from dual union all
select 66541,'1998-S09','1998-02-07 07:40:00','suborbital rocket','success',1212,'DLR K-GR-231','DLR TEXUS 36','','','Micrograv',416,'',238 from dual union all
select 66542,'1998-S10','1998-02-10 21:33:09','military missile','success',1442,'FCET-49','','','','OT',420,'SSBN 733',1000 from dual union all
select 66543,'1998-S11','1998-02-10 21:33:26','military missile','success',1442,'FCET-49','','','','OT',420,'SSBN 733',1000 from dual union all
select 66544,'1998-S12','1998-02-10 22:36:47','military missile','success',1442,'FCET-49','','','','OT',420,'SSBN 733',1000 from dual union all
select 66545,'1998-S13','1998-02-10 22:37:04','military missile','success',1442,'FCET-49','','','','OT',420,'SSBN 733',1000 from dual union all
select 66546,'1998-S14','1998-02-11 09:42:00','suborbital rocket','success',847,'DLR K-NO2-232','DLR Mini-Texus 5','','','Microgravity',417,'',140 from dual union all
select 66547,'1998-S15','1998-02-19 00:00:00','military missile','success',1179,'','','','','OT',201,'DELTA4',1000 from dual union all
select 66548,'1998-S16','1998-02-19 00:00:00','military missile','success',1179,'','','','','OT',201,'DELTA4',1000 from dual union all
select 66549,'1998-S17','1998-02-20 00:09:02','suborbital rocket','success',228,'NASA 21.117UE','Coqui Dos/SAL','','','Ionos',1484,'',115 from dual union all
select 6655,'1960-A81','1960-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 66550,'1998-S18','1998-02-20 00:37:43','suborbital rocket','success',228,'NASA 21.119UE','Coqui Dos','','','Ionos/Release',1484,'',150 from dual union all
select 66551,'1998-S19','1998-02-20 08:18:00','military missile','failure',799,'','Mk 12A RV x 2','FOT GT166GM','GLORY TRIP 166GM','Op Test',1699,'',1000 from dual union all
select 66552,'1998-S20','1998-02-25 03:17:00','suborbital rocket','success',228,'NASA 21.118UE','Coqui Dos','','','Ionos/Release',1484,'',150 from dual union all
select 66553,'1998-S21','1998-02-25 07:43:00','suborbital rocket','success',1319,'NASA 33.067UE','','','','Ionos',1484,'',200 from dual union all
select 66554,'1998-S22','1998-02-25 07:50:00','suborbital rocket','success',1334,'NASA 41.08UE','EDDY','','','Ionos',1484,'',128 from dual union all
select 66555,'1998-S23','1998-03-03 22:33:00','suborbital rocket','success',847,'DLR K-NO2-233','SSC NLTE-1 Atomic 2A','','','Ionos',417,'',134 from dual union all
select 66556,'1998-S24','1998-03-06 21:26:00','suborbital rocket','success',847,'DLR K-NO2-234','SSC NLTE-2 Atomic 2B','','','Ionos',417,'',134 from dual union all
select 66557,'1998-S25','1998-03-07 23:42:31','military missile','success',1443,'FCET-18','-    78','','','OT',420,'SSBN 736',1000 from dual union all
select 66558,'1998-S26','1998-03-07 23:42:47','military missile','success',1443,'FCET-18','-    79','','','OT',420,'SSBN 736',1000 from dual union all
select 66559,'1998-S27','1998-03-07 01:33:07','suborbital rocket','success',228,'NASA 21.114UE','Coqui Dos','','','Ionos?',1484,'',250 from dual union all
select 6656,'1960-A82','1960-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 66560,'1998-S28','1998-03-10 23:33:10','military missile','success',1443,'FCET-19','-    80','','','OT',420,'SSBN 736',1000 from dual union all
select 66561,'1998-S29','1998-03-11 01:15:58','military missile','success',1443,'FCET-19','-    81','','','OT',420,'SSBN 736',1000 from dual union all
select 66562,'1998-S30','1998-03-11 23:39:00','suborbital rocket','success',224,'NASA 36.159UE','Coqui Dos LaTuR','','','Ionos',1484,'',375 from dual union all
select 66563,'1998-S31','1998-03-21 00:00:00','suborbital rocket','success',1482,'V02','VAP-1','','','Test',13,'',900 from dual union all
select 66564,'1998-S32','1998-03-25 01:45:21','suborbital rocket','success',228,'NASA 21.115UE','Coqui Dos','','','Ionos?',1484,'',250 from dual union all
select 66565,'1998-S33','1998-04-06 00:00:00','military missile','success',509,'Hatf V','','','','Test',1473,'',150 from dual union all
select 66566,'1998-S35','1998-04-15 00:00:00','military missile','success',1005,'','Dnepr Demo','','','Test',596,'',1000 from dual union all
select 66567,'1998-S36','1998-04-17 17:05:00','suborbital rocket','success',1303,'','Red Crow Flight Expt','','','Tech',855,'',100 from dual union all
select 66568,'1998-S37','1998-04-18 04:00:00','suborbital rocket','success',224,'NASA 36.177UG','SCARI 2','','','UV Astron',1895,'',300 from dual union all
select 66569,'1998-S38','1998-04-18 07:30:00','suborbital rocket','success',224,'NASA 36.160UG','','','','UV Astron',1895,'',300 from dual union all
select 6657,'1960-A83','1960-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 66570,'1998-S39','1998-04-19 13:51:00','suborbital rocket','success',1029,'','DEOS F05','','','Ionos',1361,'',433 from dual union all
select 66571,'1998-S40','1998-04-28 12:10:00','suborbital rocket','success',225,'','ACTIVE','','','Ionos',428,'',315 from dual union all
select 66572,'1998-S41','1998-05-06 17:55:58','suborbital rocket','success',891,'NASA 30.042NP','','','','Education?',1821,'',85 from dual union all
select 66573,'1998-S42','1998-05-07 10:30:00','military missile','success',907,'27PA (87-0049)','Mk 21 RV x 8','FDEP GT 27PA','','OT',1700,'',1000 from dual union all
select 66574,'1998-S43','1998-05-12 11:22:00','military missile','success',570,'10','BRV/THAAD Target','','','Target',1857,'',300 from dual union all
select 66575,'1998-S44','1998-05-22 06:22:00','suborbital rocket','success',224,'NASA 36.175UR','NITE 2','','','IR Astron',1895,'',337 from dual union all
select 66576,'1998-S45','1998-06-03 19:57:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT167GB','GLORY TRIP 167GB','Op Test',1712,'',1300 from dual union all
select 66577,'1998-S46','1998-06-11 01:00:00','suborbital rocket','success',650,'KSR-II-2','','','','XR Astron/Ionos',33,'',137 from dual union all
select 66578,'1998-S47','1998-06-16 14:19:00','suborbital rocket','success',860,'NASA 31.111UP','Cincinatti/OhioSGC','','','Aeron/Education',1821,'',117 from dual union all
select 66579,'1998-S48','1998-06-24 08:01:00','military missile','success',799,'','Mk 12A RV x 3','GRP-IDF-1','GRP-IDF-1','Test',1705,'',1300 from dual union all
select 6658,'1960-A84','1960-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 66580,'1998-S49','1998-06-24 12:46:00','military missile','success',799,'','Mk 12/12A RV x 3','FOT GT168GM (GB?)','GLORY TRIP 168GM','Op Test',1706,'',1300 from dual union all
select 66581,'1998-S50','1998-08-15 05:30:00','suborbital rocket','success',224,'NASA 36.176UH','EEV CCD','','','XR Astronomy',1895,'',300 from dual union all
select 66582,'1998-S51','1998-08-21 00:00:00','military missile','success',1507,'','','','','OT',201,'PLBR',1000 from dual union all
select 66583,'1998-S52','1998-09-16 11:10:00','military missile','success',1434,'64536-214-43','15Zh58 UBP-22','','','Test',608,'',1000 from dual union all
select 66584,'1998-S53','1998-09-18 08:01:00','military missile','success',799,'','Mk 12/12A RV x 3','GRP-IDF-2','GRP-IDF-2','Test',1712,'',1300 from dual union all
select 66585,'1998-S54','1998-09-18 15:00:00','suborbital rocket','success',224,'NASA 36.150NP','SOAREX-1 Hypersonic','','','Test',1873,'',314 from dual union all
select 66586,'1998-S55','1998-09-21 14:51:00','suborbital rocket','success',1029,'','DEOS F07','','','Ionos',1361,'',425 from dual union all
select 66587,'1998-S56','1998-09-24 12:50:00','military missile','success',1263,'','MTTV-5/MTD-3','','','Target',1860,'',180 from dual union all
select 66588,'1998-S57','1998-09-28 15:11:00','suborbital rocket','success',1029,'','DEOS F06','','','Ionos',1361,'',422 from dual union all
select 66589,'1998-S58','1998-10-07 11:35:00','suborbital rocket','success',891,'DLR K-O-236','DLR Maxus Test','','','Test',417,'',56 from dual union all
select 6659,'1960-A85','1960-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 66590,'1998-S59','1998-10-07 12:00:00','military missile','success',1453,'','15A35 UBP-47','GCh 15F355P7','','OT',535,'',1000 from dual union all
select 66591,'1998-S60','1998-11-02 18:20:00','suborbital rocket','success',224,'NASA 36.171CS','SOPHIE (NCAR/CU-7)','','','Solar UV',1895,'',350 from dual union all
select 66592,'1998-S61','1998-11-06 01:32:00','military missile','success',632,'','ait-1','','','Test',862,'',715 from dual union all
select 66593,'1998-S62','1998-11-18 15:40:00','suborbital rocket','success',224,'NASA 36.178NM','','','','Micrograv',1895,'',300 from dual union all
select 66594,'1998-S63','1998-11-18 23:00:00','suborbital rocket','success',1296,'TR-1A-7','','','','Micrograv',1476,'',266 from dual union all
select 66595,'1998-S64','1998-11-20 12:30:00','military missile','success',732,'Aegis ALI Target','Aegis TTV-1','','','Target',855,'',322 from dual union all
select 66596,'1998-S65','1998-11-24 09:53:00','suborbital rocket','success',772,'MAXUS 3','SSC MAXUS 3','','','Bio/Physics',407,'',713 from dual union all
select 66597,'1998-S66','1998-12-03 11:04:00','suborbital rocket','success',847,'DLR K-NO2-237','DLR Mini-Texus 6','','','Microgravity',417,'',140 from dual union all
select 66598,'1998-S67','1998-12-08 11:25:00','military missile','success',1436,'00354-251-01(6L)','15Zh65 LKI-6','','','Test',608,'',1000 from dual union all
select 66599,'1998-S68','1998-12-09 10:20:00','military missile','success',1042,'48792-213-06','15Zh61 UBP-2','','','Test',477,'',1000 from dual union all
select 666,'1947-A02','1947-01-24 23:07:00','atmospheric rocket','success',875,'46-9','Nike-46 8R','','','Test',1863,'',31 from dual union all
select 6660,'1960-A86','1960-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 66600,'1998-S70','1998-12-15 00:00:00','suborbital rocket','success',1221,'1','Test flight','','','Test',808,'',250 from dual union all
select 66601,'1998-S71','1998-07-01 00:00:00','military missile','success',1087,'Aegis TMT-1','RIMPAC 98','','','Target',855,'',80 from dual union all
select 66602,'1998-S72','1998-11-18 00:00:00','military missile','success',1087,'Aegis TMT-2','LINEBACKER','','','Target',855,'',87 from dual union all
select 66603,'1998-W01','1998-01-08 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66604,'1998-W02','1998-01-14 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66605,'1998-W03','1998-01-21 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66606,'1998-W04','1998-02-04 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66607,'1998-W05','1998-02-10 23:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',60 from dual union all
select 66608,'1998-W06','1998-02-11 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',51 from dual;
insert into LAUNCH
select 66609,'1998-W07','1998-02-18 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 6661,'1960-A87','1960-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 66610,'1998-W08','1998-02-24 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',62 from dual union all
select 66611,'1998-W09','1998-03-04 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66612,'1998-W10','1998-03-18 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66613,'1998-W11','1998-03-25 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66614,'1998-W12','1998-04-08 02:07:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66615,'1998-W13','1998-04-15 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',58 from dual union all
select 66616,'1998-W14','1998-04-16 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66617,'1998-W15','1998-04-29 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66618,'1998-W16','1998-05-07 00:00:00','suborbital rocket','success',1284,'','THAAD prep','','','Aeron',1895,'',75 from dual union all
select 66619,'1998-W17','1998-05-20 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',51 from dual union all
select 6662,'1960-A88','1960-03-31 00:00:00','military missile','success',764,'57-2452','','','','Test',243,'',12 from dual union all
select 66620,'1998-W18','1998-05-27 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',57 from dual union all
select 66621,'1998-W19','1998-06-01 14:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',67 from dual union all
select 66622,'1998-W20','1998-06-11 02:04:59','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66623,'1998-W21','1998-06-17 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',57 from dual union all
select 66624,'1998-W22','1998-06-24 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',57 from dual union all
select 66625,'1998-W23','1998-07-09 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66626,'1998-W24','1998-07-15 02:03:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66627,'1998-W25','1998-07-29 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66628,'1998-W26','1998-08-05 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',53 from dual union all
select 66629,'1998-W27','1998-08-20 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 6663,'1960-A89','1960-03-31 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1232,'VOKV',44 from dual union all
select 66630,'1998-W28','1998-09-07 02:00:00','suborbital rocket','success',759,'MT-135-68','Ozone','','','Aeron',839,'',57 from dual union all
select 66631,'1998-W29','1998-09-09 02:00:00','suborbital rocket','success',759,'MT-135-69','','','','Aeron',839,'',57 from dual union all
select 66632,'1998-W30','1998-09-09 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66633,'1998-W31','1998-09-17 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66634,'1998-W32','1998-09-23 02:03:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 66635,'1998-W33','1998-10-07 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',52 from dual union all
select 66636,'1998-W34','1998-10-14 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66637,'1998-W35','1998-10-15 14:12:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',64 from dual union all
select 66638,'1998-W36','1998-10-21 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66639,'1998-W37','1998-11-04 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 6664,'1960-A90','1960-03-31 16:42:05','suborbital spaceplane','success',1520,'2-8-16','SAS Gains','','','Test',379,'NB-52 008',15 from dual union all
select 66640,'1998-W38','1998-11-11 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66641,'1998-W39','1998-11-18 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 66642,'1998-W40','1998-12-02 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 66643,'1998-W41','1998-12-23 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',57 from dual union all
select 66644,'1998-Y01','1998-01-04 19:10:00','suborbital rocket','success',1502,'','TPS01','','','Aeron',1330,'',100 from dual union all
select 66645,'1998-Y02','1998-01-05 22:00:00','suborbital rocket','success',1502,'','TPS02','','','Aeron',1330,'',100 from dual union all
select 66646,'1998-Y03','1998-01-08 18:15:00','suborbital rocket','success',1502,'','TPS03','','','Aeron',1330,'',100 from dual union all
select 66647,'1998-Y04','1998-01-08 21:15:00','suborbital rocket','success',1502,'','TPS04','','','Aeron',1330,'',100 from dual union all
select 66648,'1998-Y05','1998-01-10 15:00:00','suborbital rocket','success',1502,'','TPS05','','','Aeron',1330,'',100 from dual union all
select 66649,'1998-Y06','1998-01-14 17:40:00','suborbital rocket','success',1502,'','TPS06','','','Aeron',1330,'',100 from dual union all
select 6665,'1960-A91','1960-04-05 00:00:00','military missile','success',793,'','Silo Test','','','Test',381,'',0 from dual union all
select 66650,'1998-Y07','1998-01-16 16:35:00','suborbital rocket','success',1502,'','TPS07','','','Aeron',1330,'',100 from dual union all
select 66651,'1998-Y08','1998-01-19 17:00:00','suborbital rocket','success',1502,'','TPS08','','','Aeron',1330,'',100 from dual union all
select 66652,'1998-Y09','1998-01-21 13:05:00','suborbital rocket','success',1502,'','TPS09','','','Aeron',1330,'',100 from dual union all
select 66653,'1998-Y10','1998-01-21 14:08:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',99 from dual union all
select 66654,'1998-Y11','1998-01-23 17:12:00','suborbital rocket','success',1502,'','TPS10','','','Aeron',1330,'',100 from dual union all
select 66655,'1998-Y12','1998-01-26 17:03:00','suborbital rocket','success',1502,'','TPS11','','','Aeron',1330,'',100 from dual union all
select 66656,'1998-Y13','1998-01-28 17:05:00','suborbital rocket','success',1502,'','TPS12','','','Aeron',1330,'',100 from dual union all
select 66657,'1998-Y14','1998-01-30 17:00:00','suborbital rocket','success',1502,'','TPS13','','','Aeron',1330,'',100 from dual union all
select 66658,'1998-Y15','1998-02-01 18:15:00','suborbital rocket','success',1502,'','TPS14','','','Aeron',1330,'',100 from dual union all
select 66659,'1998-Y16','1998-02-04 18:30:00','suborbital rocket','success',1502,'','TPS15','','','Aeron',1330,'',100 from dual union all
select 6666,'1960-A92','1960-04-06 00:00:00','military missile','success',1214,'SM-62A N-3427','','','','OT',238,'',10 from dual union all
select 66660,'1998-Y17','1998-02-06 17:18:00','suborbital rocket','success',1502,'','TPS16','','','Aeron',1330,'',100 from dual union all
select 66661,'1998-Y18','1998-02-07 16:10:00','suborbital rocket','success',1502,'','TPS17','','','Aeron',1330,'',100 from dual union all
select 66662,'1998-Y19','1998-02-09 18:10:00','suborbital rocket','success',1502,'','TPS18','','','Aeron',1330,'',100 from dual union all
select 66663,'1998-Y20','1998-02-11 18:00:00','suborbital rocket','success',1502,'','TPS19','','','Aeron',1330,'',100 from dual union all
select 66664,'1998-Y21','1998-02-13 17:00:00','suborbital rocket','success',1502,'','TPS20','','','Aeron',1330,'',100 from dual union all
select 66665,'1998-Y22','1998-02-17 13:34:00','suborbital rocket','success',1502,'','TPS21','','','Aeron',1330,'',100 from dual union all
select 66666,'1998-Y23','1998-02-17 16:47:00','suborbital rocket','success',1502,'','TPS22','','','Aeron',1330,'',100 from dual union all
select 66667,'1998-Y24','1998-02-19 17:00:00','suborbital rocket','success',1502,'','TPS23','','','Aeron',1330,'',100 from dual union all
select 66668,'1998-Y25','1998-02-23 20:15:00','suborbital rocket','success',1502,'','TPS24','','','Aeron',1330,'',100 from dual union all
select 66669,'1998-Y26','1998-02-25 16:05:00','suborbital rocket','success',1502,'','TPS25','','','Aeron',1330,'',100 from dual union all
select 6667,'1960-A93','1960-04-07 21:25:00','suborbital rocket','success',839,'Flight 7914','Launcher test','','','Test',428,'',2 from dual union all
select 66670,'1998-Y27','1998-02-27 21:00:00','suborbital rocket','success',1502,'','TPS26','','','Aeron',1330,'',100 from dual union all
select 66671,'1998-Y28','1998-03-05 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',84 from dual union all
select 66672,'1998-Y29','1998-03-27 15:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',82 from dual union all
select 66673,'1998-Y30','1998-05-05 14:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 66674,'1998-Y31','1998-07-30 14:00:00','suborbital rocket','success',1061,'','Met Sphere','','','Meteo',295,'',84 from dual union all
select 66675,'1998-Y32','1998-10-22 21:21:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',99 from dual union all
select 66676,'1998-Y33','1998-10-23 01:01:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',99 from dual union all
select 66677,'1998-Y34','1998-12-21 16:30:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',85 from dual union all
select 66678,'1998-Y35','1998-10-22 23:30:00','suborbital rocket','success',1284,'','PHYSX support','','','Aeron',295,'',90 from dual union all
select 66679,'1998-Y36','1998-10-23 00:30:00','suborbital rocket','success',1284,'','PHYSX support','','','Aeron',295,'',90 from dual union all
select 6668,'1960-A94','1960-04-08 02:06:00','military missile','failure',154,'48D','Mk 3-1A-216','R&D D/AIG guid.test OL2','','Test',226,'',0 from dual union all
select 66680,'1998-Y37','1998-03-07 00:33:00','suborbital rocket','success',1284,'Chaff','','','','Aeron',409,'',100 from dual union all
select 66681,'1998-Y38','1998-03-07 01:24:00','suborbital rocket','success',1284,'Chaff','','','','Aeron',409,'',100 from dual union all
select 66682,'1999-001','1999-01-03 20:21:10','deep space','success',415,'D265','Mars Polar Lander','MPL','','Sat',286,'',null from dual union all
select 66683,'1999-002','1999-01-27 00:34:02','orbital','success',137,'LM-006','ROCSAT','ROCSAT-1','','Sat',1425,'',null from dual union all
select 66684,'1999-003','1999-02-07 21:04:15','deep space','success',416,'D266','Stardust','Stardust','','Sat',285,'',null from dual union all
select 66685,'1999-004','1999-02-09 03:53:59','orbital','success',1240,'S 15000-058     ST01','Globalstar S-1','Globalstar FM23','','Sat',480,'',null from dual union all
select 66686,'1999-005','1999-02-15 05:12:00','orbital','success',960,'396-01','Telstar 6','Telstar 6','','Sat',587,'',null from dual union all
select 66687,'1999-006','1999-02-16 01:45:26','orbital','success',174,'AC-152','JCSAT 6','JCSAT 6','','Sat',288,'',null from dual union all
select 66688,'1999-007','1999-02-20 04:18:01','orbital','success',1238,'M 15000-662','Soyuz TM-29','Soyuz 7K-STM No. 78','','Sat',480,'',null from dual union all
select 66689,'1999-008','1999-02-23 10:29:55','orbital','success',417,'D267','ARGOS','P91-1','','Sat',1724,'',null from dual union all
select 6669,'1960-A95','1960-04-13 00:00:00','military missile','success',771,'GM-54-57','','','','Test',222,'',12 from dual union all
select 66690,'1999-009','1999-02-26 22:44:00','orbital','success',105,'V116 L485','Arabsat 3A/Skynet 4E','Arabsat 3A','','Sat',345,'',null from dual union all
select 66691,'1999-010','1999-02-28 04:00:00','orbital','success',958,'387-01','Raduga-1','Globus No. 15L','','Sat',587,'',null from dual union all
select 66692,'1999-011','1999-03-05 02:56:00','orbital','success',911,'F26/M-22','WIRE','WIRE','','Sat',1718,'L-1011',null from dual union all
select 66693,'1999-012','1999-03-15 03:06:00','orbital','success',1240,'S 15000-059     ST02','Globalstar S-2','Globalstar FM22','','Sat',480,'',null from dual union all
select 66694,'1999-013','1999-03-21 00:09:30','orbital','success',960,'388-01','Asiasat 3S','Asiasat 3S','','Sat',587,'',null from dual union all
select 66695,'1999-014','1999-03-28 01:29:59','orbital','success',1542,'1L/DM-SL-1TL','SL1 DemoSat','DemoSat','','Sat',860,'ODYSSEY',null from dual union all
select 66696,'1999-015','1999-04-02 11:28:43','orbital','success',1241,'M 15000-663','Progress M-41','Progress 7K-TGM No 241','','Sat',480,'',null from dual union all
select 66697,'1999-016','1999-04-02 22:03:00','orbital','success',102,'V117 L486b','Insat 2E','Insat 2E','','Sat',345,'',null from dual union all
select 66698,'1999-017','1999-04-09 17:01:00','orbital','failure',1412,'4B-27 (K-32)','USA 142      [DSP19]','DSP 19?','','Sat',275,'',null from dual union all
select 66699,'1999-018','1999-04-12 22:50:01','orbital','success',174,'AC-154','Eutelsat W3','Eutelsat W3','','Sat',288,'',null from dual;
insert into LAUNCH
select 667,'1947-A03','1947-01-28 21:05:00','atmospheric rocket','failure',875,'46-4','Nike-46 9R','','','Test',1863,'',0 from dual union all
select 6670,'1960-A96','1960-04-13 00:00:00','military missile','success',771,'GM-56-1925','','','','Test',222,'',12 from dual union all
select 66700,'1999-019','1999-04-15 00:46:00','orbital','success',1240,'S 15000-060     ST03','Globalstar S-3','Globalstar FM45','','Sat',480,'',null from dual union all
select 66701,'1999-020','1999-04-15 18:32:00','orbital','success',417,'D268','Landsat 7','Landsat 7','','Sat',1724,'',null from dual union all
select 66702,'1999-021','1999-04-21 04:59:12','orbital','success',434,'670-3542-509','UoSAT-12','UoSAT-12','','Sat',497,'',null from dual union all
select 66703,'1999-022','1999-04-28 20:30:00','orbital','success',680,'65036-413','ABRIXAS/Megsat-0','ABRIXAS','','Sat',621,'',null from dual union all
select 66704,'1999-023','1999-04-30 16:30:00','orbital','failure',1410,'4B-32/TC-14 (K-26)','USA 143 (Milstar 3)','Milstar DFS-3','','Sat',274,'',null from dual union all
select 66705,'1999-024','1999-05-05 01:00:00','orbital','failure',429,'D269','Orion 3','Orion 3','','Sat',286,'',null from dual union all
select 66706,'1999-025','1999-05-10 01:33:00','orbital','success',319,'Y2       (56)','FY-1C/SJ-5','Feng Yun Yi Beng','','Sat',1492,'',null from dual union all
select 66707,'1999-026','1999-05-18 05:09:36','orbital','success',912,'F27','TERRIERS/MUBLCOM','TERRIERS','','Sat',1718,'L-1011',null from dual union all
select 66708,'1999-027','1999-05-20 22:30:00','orbital','success',960,'396-02','Nimiq','Telesat DTH-1','','Sat',587,'',null from dual union all
select 66709,'1999-028','1999-05-22 09:36:00','orbital','success',1416,'4B-12','USA 144      [MIST2]','MISTY 2?','','Sat',1777,'',null from dual union all
select 6671,'1960-A97','1960-04-13 17:15:11','suborbital spaceplane','success',1520,'1-4-9','','','','Test',379,'NB-52 003',14 from dual union all
select 66710,'1999-029','1999-05-26 06:22:00','orbital','success',899,'PSLV-C2','IRS-P4','KITSAT-3','','Sat',1357,'',null from dual union all
select 66711,'1999-030','1999-05-27 10:49:42','orbital','success',1243,'STS-96','ISS 2A.1 (Spacehab)','OV-103 ISS','','Sat',922,'MLP2',null from dual union all
select 66712,'1999-031','1999-06-10 13:48:43','orbital','success',413,'D270','Globalstar-3','Globalstar M025','','Sat',286,'',null from dual union all
select 66713,'1999-032','1999-06-11 17:15:33','orbital','success',302,'Y7       (57)','Iridium 14A/21A','Iridium SV092','','Sat',1492,'',null from dual union all
select 66714,'1999-033','1999-06-18 01:49:30','orbital','success',960,'397-02','Astra 1H','Astra 1H','','Sat',587,'',null from dual union all
select 66715,'1999-034','1999-06-20 02:15:00','orbital','success',1422,'23G-7','Quikscat','QuikScat','','Sat',1778,'',null from dual union all
select 66716,'1999-035','1999-06-24 15:44:00','orbital','success',410,'D271','FUSE','Far UV Spec. Explorer','','Sat',285,'',null from dual union all
select 66717,'1999-036','1999-07-08 08:45:06','orbital','success',805,'78051  -677','Molniya-3','Molniya-3 No. 63?','','Sat',474,'',null from dual union all
select 66718,'1999-037','1999-07-10 08:45:37','orbital','success',413,'D272','Globalstar-4','Globalstar M035','','Sat',286,'',null from dual union all
select 66719,'1999-038','1999-07-16 16:37:33','orbital','success',1241,'M 15000-667','Progress M-42','Progress 7K-TGM No 242','','Sat',480,'',null from dual union all
select 6672,'1960-A98','1960-04-14 22:21:00','military missile','success',928,'STV-2A-2','Underwater test','','','Test',1338,'',1 from dual union all
select 66720,'1999-039','1999-07-17 05:38:00','orbital','success',1537,'51025-802/17L','Okean-O No. 1L','Okean-O No. 1','','Sat',567,'',null from dual union all
select 66721,'1999-040','1999-07-23 04:31:00','orbital','success',1243,'STS-93','Chandra','OV-102 IUS deploy','','Sat',922,'MLP1',null from dual union all
select 66722,'1999-041','1999-07-25 07:46:03','orbital','success',413,'D273','Globalstar-5','Globalstar M048','','Sat',285,'',null from dual union all
select 66723,'1999-042','1999-08-12 22:52:00','orbital','success',102,'V118 L487f','Telkom 1','Telkom 1','','Sat',345,'',null from dual union all
select 66724,'1999-043','1999-08-17 04:37:41','orbital','success',413,'D274','Globalstar-6','Globalstar M024','','Sat',286,'',null from dual union all
select 66725,'1999-044','1999-08-18 18:00:01','orbital','success',1241,'77017  -785','Kosmos-2365','Kobal''t','','Sat',474,'',null from dual union all
select 66726,'1999-045','1999-08-26 12:02:15','orbital','success',680,'65021-703','Kosmos-2366','Parus','','Sat',456,'',null from dual union all
select 66727,'1999-046','1999-09-04 22:34:00','orbital','success',102,'V120 L488d','Koreasat 3','Koreasat 3','','Sat',345,'',null from dual union all
select 66728,'1999-047','1999-09-06 16:36:00','orbital','success',959,'388-02','Yamal-100 No. 1/2','Yamal-100 No. 2','','Sat',587,'',null from dual union all
select 66729,'1999-048','1999-09-09 18:00:00','orbital','success',1241,'M 15000-082','Foton No. 12','Foton No. 12','','Sat',475,'',null from dual union all
select 6673,'1960-A99','1960-04-15 14:50:00','military missile','success',256,'631-8','','','','OT',272,'',20 from dual union all
select 66730,'1999-049','1999-09-22 14:33:00','orbital','success',1240,'S 15000-061     ST04','Globalstar S-4','Globalstar FM58','','Sat',480,'',null from dual union all
select 66731,'1999-050','1999-09-23 06:02:02','orbital','success',174,'AC-155','Echostar V','Echostar 5','','Sat',288,'',null from dual union all
select 66732,'1999-051','1999-09-24 18:21:08','orbital','success',138,'LM-007','Ikonos-2','Ikonos 2','','Sat',1780,'',null from dual union all
select 66733,'1999-052','1999-09-25 06:29:00','orbital','success',108,'V121 L489d','Telstar 7','Telstar 7','','Sat',345,'',null from dual union all
select 66734,'1999-053','1999-09-26 22:30:00','orbital','success',960,'398-02','LMI-1','LMI 1','','Sat',587,'',null from dual union all
select 66735,'1999-054','1999-09-28 11:00:05','orbital','success',1241,'M 15000-045','Resurs F-1M','Resurs F-1M','','Sat',475,'',null from dual union all
select 66736,'1999-055','1999-10-07 12:51:01','orbital','success',427,'D275','USA 145 (GPS IIR-3)','Navstar GPS SVN 46','','Sat',285,'',null from dual union all
select 66737,'1999-056','1999-10-10 03:28:00','orbital','success',1542,'3L/DM-SL-3L','SL2 DirecTV 1-R','DirecTV-1R','','Sat',860,'ODYSSEY',null from dual union all
select 66738,'1999-057','1999-10-14 03:15:00','orbital','success',319,'Y1       (58)','ZY-1/SACI-1','ZY-1/CBERS 1','','Sat',1492,'',null from dual union all
select 66739,'1999-058','1999-10-18 13:32:00','orbital','success',1240,'M 15000-062     ST05','Globalstar S-5','Globalstar M057','','Sat',480,'',null from dual union all
select 6674,'1960-E01','1960-10-24 15:45:00','military missile','destroyed before launch',981,'LD1-3T','8K64 LKI','','','Test',565,'',0 from dual union all
select 66740,'1999-059','1999-10-19 06:22:00','orbital','success',108,'V122 L490c','Orion 2','Orion 2','','Sat',345,'',null from dual union all
select 66741,'1999-060','1999-11-13 22:54:00','orbital','success',108,'V123 L491a','GE 4','GE 4','','Sat',345,'',null from dual union all
select 66742,'1999-061','1999-11-19 22:30:04','orbital','success',306,'Y1       (59)','Shenzhou','Shenzhou 1 Fanhui Cang','','Sat',814,'',null from dual union all
select 66743,'1999-062','1999-11-22 16:20:00','orbital','success',1240,'M 15000-063     ST06','Globalstar S-6','Globalstar M039','','Sat',480,'',null from dual union all
select 66744,'1999-063','1999-11-23 04:06:01','orbital','success',173,'AC-136','USA 146 (UHF F10)','UFO 10','','Sat',289,'',null from dual union all
select 66745,'1999-064','1999-12-03 16:22:46','orbital','success',96,'V124 L492','Helios 1B/Clementine','Helios 1B','','Sat',345,'',null from dual union all
select 66746,'1999-065','1999-12-04 18:53:00','orbital','success',912,'F28','Orbcomm D1-D7','Orbcomm FM30','','Sat',1820,'L-1011',null from dual union all
select 66747,'1999-066','1999-12-10 14:32:07','orbital','success',114,'V119 (504a)','XMM','XMM','','Sat',346,'',null from dual union all
select 66748,'1999-067','1999-12-12 17:38:01','orbital','success',1422,'23G-8','USA 147 (DMSP F15)','DMSP Block 5D-3 S-15','','Sat',1778,'',null from dual union all
select 66749,'1999-068','1999-12-18 18:57:39','orbital','success',174,'AC-141','Terra','EOS AM-1','','Sat',1775,'',null from dual union all
select 6675,'1960-F01','1960-02-04 18:51:45','orbital','failure',1360,'218/FTV 1052','Discoverer 9','CORONA C-6','HUNGRY EYE','Sat TAG9',1678,'',200 from dual union all
select 66750,'1999-069','1999-12-20 00:50:00','orbital','success',1243,'STS-103','HST SM-3A','OV-103 HST mission','','Sat',922,'MLP2',null from dual union all
select 66751,'1999-070','1999-12-21 07:13:00','orbital','success',1314,'T4','KOMPSAT/ACRIMSAT','KOMPSAT','','Sat',1670,'',null from dual union all
select 66752,'1999-071','1999-12-22 00:50:00','orbital','success',105,'V125 L493','Galaxy 11','Galaxy 11','','Sat',345,'',null from dual union all
select 66753,'1999-072','1999-12-26 08:00:00','orbital','success',1446,'450-82-  801','Kosmos-2367','US-PU/Konus-A','','Sat',591,'',null from dual union all
select 66754,'1999-073','1999-12-27 19:12:44','orbital','success',803,'77017  -681','Kosmos-2368','Oko','','Sat',462,'',null from dual union all
select 66755,'1999-A01','1999-01-15 00:00:00','military missile','failure',1169,'','','','','Test',1419,'',0 from dual union all
select 66756,'1999-A02','1999-08-05 13:59:00','military missile','success',1088,'','','','','Test',1869,'',10 from dual union all
select 66757,'1999-A03','1999-08-05 18:11:05','suborbital rocket','success',891,'NASA 30.043NO','Sub-SEM','','','Education',1821,'',40 from dual union all
select 66758,'1999-A04','1999-10-01 00:00:00','military missile','failure',1507,'','','','','OT',201,'PLBR',10 from dual union all
select 66759,'1999-A05','1999-11-02 14:51:00','military missile','success',3,'','Test','','','Op Test',183,'',40 from dual union all
select 6676,'1960-F02','1960-02-19 20:15:14','orbital','failure',1360,'223/FTV 1054','Discoverer 10','CORONA C-7','DERBY DAY','Sat TAG10',1679,'',6 from dual union all
select 66760,'1999-A06','1999-02-12 00:00:00','military missile','success',2,'S-300','48N6E test','','','Test',636,'',40 from dual union all
select 66761,'1999-F01','1999-04-27 18:22:01','orbital','failure',138,'LM-005','Ikonos-1','Ikonos 1','','Sat',1780,'',200 from dual union all
select 66762,'1999-F02','1999-07-05 13:32:00','orbital','failure',952,'389-01/88501','Raduga','Gran'' No. 45','','Sat',588,'',30 from dual union all
select 66763,'1999-F03','1999-10-27 16:16:00','orbital','failure',958,'386-02','Ekspress A No. 1','Ekspress-A No. R 001','','Sat',553,'',120 from dual union all
select 66764,'1999-F04','1999-11-15 07:29:00','orbital','failure',526,'H-II-8F','MTSAT','MTSAT','','Sat',1477,'',100 from dual union all
select 66765,'1999-F05','1999-12-11 18:25:00','orbital','failure',1475,'V02','SACI-2','SACI-2','','Sat',12,'',10 from dual union all
select 66766,'1999-M01','1999-07-01 00:00:00','military missile','success',1087,'Aegis TMT-3','Slugger','','','Test',855,'',72 from dual union all
select 66767,'1999-M02','1999-11-01 09:42:00','military missile','success',119,'F7','','','','ATBM Test',1026,'',60 from dual union all
select 66768,'1999-M03','1999-11-01 09:42:00','military missile','success',118,'TM-91','','','','Target',1024,'SHIP-ISR',80 from dual union all
select 66769,'1999-M04','1999-05-01 00:00:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 6677,'1960-F03','1960-02-26 17:25:30','orbital','failure',144,'29D','Midas 1','Midas ETS-I F1','','Sat',229,'',90 from dual union all
select 66770,'1999-M05','1999-08-31 00:00:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 66771,'1999-M06','1999-09-05 04:15:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 66772,'1999-M07','1999-12-23 00:00:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 66773,'1999-M08','1999-12-28 13:15:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 66774,'1999-S01','1999-01-21 06:13:30','suborbital rocket','success',237,'NASA 40.012IE','CAPER','','','Ionos',29,'',1360 from dual union all
select 66775,'1999-S02','1999-01-22 13:57:03','suborbital rocket','success',237,'NASA 40.013DP','North Star (APEX)','','','Active plasma',1047,'',364 from dual union all
select 66776,'1999-S03','1999-01-22 15:20:17','suborbital rocket','success',227,'NASA 21.121CE','','','','Ionos?',1047,'',250 from dual union all
select 66777,'1999-S04','1999-01-22 15:31:00','suborbital rocket','success',1319,'NASA 33.068UE','','','','Ionos',1047,'',200 from dual union all
select 66778,'1999-S05','1999-02-02 01:30:00','suborbital rocket','success',1073,'S-310-28','SELENE RF/XR test','','','Ionos/XR',833,'',184 from dual union all
select 66779,'1999-S06','1999-02-09 20:10:00','military missile','success',1443,'FCET-20','-    to ASC C-25','','','OT',420,'SSBN 740',1000 from dual union all
select 6678,'1960-F04','1960-03-23 13:35:11','orbital','failure',634,'AM-19C','NASA S-46','NASA S-46','','Sat',249,'',401 from dual union all
select 66780,'1999-S07','1999-02-09 20:10:15','military missile','success',1443,'FCET-20','-    to ASC C-25','','','OT',420,'SSBN 740',1000 from dual union all
select 66781,'1999-S08','1999-02-10 08:06:00','military missile','success',799,'','Mk 12/12A RV','FDEP GT169GM','GLORY TRIP 169GM','Op Test',1699,'',1300 from dual union all
select 66782,'1999-S09','1999-02-11 06:45:31','suborbital rocket','success',231,'NASA 35.032UE','ENSTROPHY','','','Ionos',1047,'',1071 from dual union all
select 66783,'1999-S10','1999-02-20 04:45:00','suborbital rocket','success',224,'NASA 36.180UG','SPINR 2','','','UV Astron',1895,'',300 from dual union all
select 66784,'1999-S11','1999-03-10 08:01:00','military missile','success',907,'28PA (88-0335)','Mk 21 RV x 8','FDEP GT 28PA','','OT',1697,'',1000 from dual union all
select 66785,'1999-S12','1999-03-15 09:52:00','suborbital rocket','success',1478,'XV-04','Operacao San Marcos','','','Micrograv',13,'',128 from dual union all
select 66786,'1999-S13','1999-03-15 13:49:00','military missile','success',570,'12','BRV/PAC-3 SCF-2','','','Target',436,'',94 from dual union all
select 66787,'1999-S14','1999-03-23 21:46:10','military missile','success',1442,'FCET-50','','','','OT',420,'SSBN 730',1000 from dual union all
select 66788,'1999-S15','1999-03-23 21:46:28','military missile','success',1442,'FCET-50','','','','OT',420,'SSBN 730',1000 from dual union all
select 66789,'1999-S16','1999-03-25 00:17:05','military missile','success',1442,'FCET-50','','','','OT',420,'SSBN 730',1000 from dual;
insert into LAUNCH
select 6679,'1960-F06','1960-04-16 16:07:41','orbital','failure',1513,'L1-9A','','E-3 No. 2','','Sat',480,'',0 from dual union all
select 66790,'1999-S17','1999-03-25 00:17:22','military missile','success',1442,'FCET-50','','','','OT',420,'SSBN 730',1000 from dual union all
select 66791,'1999-S18','1999-03-28 09:00:00','suborbital rocket','success',211,'NASA 27.141UH','XQC F3','','','XR Astronomy',1895,'',220 from dual union all
select 66792,'1999-S19','1999-03-29 12:13:00','military missile','success',570,'13','BRV/THAAD Targ 4','','','Target',1857,'',300 from dual union all
select 66793,'1999-S20','1999-03-29 12:20:00','military missile','success',1293,'FT-09','','','','ABM Test',1895,'',100 from dual union all
select 66794,'1999-S21','1999-03-31 00:15:00','military missile','success',1096,'','SRALT/MBRV-3','','','Test',850,'C-130',99 from dual union all
select 66795,'1999-S22','1999-04-01 00:00:00','military missile','success',1507,'','','','','OT',201,'PLBR',1000 from dual union all
select 66796,'1999-S23','1999-04-11 04:17:00','military missile','success',63,'','Agni RV Mk 2','','','Test',316,'',1000 from dual union all
select 66797,'1999-S24','1999-04-12 06:35:00','suborbital rocket','success',213,'NASA 36.172UG','WISP 4','','','UV Astron?',1895,'',375 from dual union all
select 66798,'1999-S25','1999-04-14 05:35:00','military missile','success',509,'Hatf-V/Ghauri II','','','','Test',1473,'',150 from dual union all
select 66799,'1999-S26','1999-04-15 04:58:00','military missile','success',1169,'','','','','Test',1419,'',100 from dual union all
select 668,'1947-A04','1947-02-11 00:00:00','atmospheric rocket','success',471,'','RM-10 Model','','','Test',1821,'',5 from dual union all
select 6680,'1960-F07','1960-05-13 09:16:05','orbital','failure',1375,'144/D1','Echo','A-10 Echo','','Sat',233,'',1668 from dual union all
select 66800,'1999-S27','1999-04-26 19:47:31','military missile','success',1443,'FCET-21','-    to ASC C-17','','','OT',420,'SSBN 738',1000 from dual union all
select 66801,'1999-S28','1999-04-26 19:47:47','military missile','success',1443,'FCET-21','-    to ASC C-17','','','OT',420,'SSBN 738',1000 from dual union all
select 66802,'1999-S29','1999-05-04 00:00:00','military missile','success',757,'3','','','','Test',206,'S617',800 from dual union all
select 66803,'1999-S30','1999-05-04 00:00:00','military missile','success',756,'','Veuglaire','','','Test',206,'S615',800 from dual union all
select 66804,'1999-S31','1999-05-07 20:00:06','suborbital rocket','success',224,'NASA 36.141DS','VAULT I','','','Solar UV',1895,'',300 from dual union all
select 66805,'1999-S32','1999-05-14 11:33:00','suborbital rocket','success',1212,'MASER 8','SSC MASER 8','','','Micrograv',416,'',260 from dual union all
select 66806,'1999-S33','1999-05-25 11:14:00','military missile','failure',570,'14','BRV/THAAD Target 5','','','Target',436,'',92 from dual union all
select 66807,'1999-S34','1999-06-03 14:20:00','military missile','success',1436,'00354-331-01(7L)','15Zh65 LKI-7','','','Test',608,'',1000 from dual union all
select 66808,'1999-S35','1999-06-10 11:07:00','military missile','success',570,'15','BRV/THAAD T6/ FT-10','','','Target',1857,'',310 from dual union all
select 66809,'1999-S36','1999-06-10 11:14:00','military missile','success',1293,'FT-10','High Endo','','','ABM Test',1895,'',90 from dual union all
select 6681,'1960-F08','1960-06-29 22:00:44','orbital','failure',1360,'160/FTV 1053','Discoverer 12','CORONA Diagnostic 1','RED GARTER','Sat TAG12',1678,'',205 from dual union all
select 66810,'1999-S37','1999-06-14 07:40:00','suborbital rocket','success',213,'NASA 36.136UG','JHU FOT 14','','','UV Astron',1895,'',300 from dual union all
select 66811,'1999-S38','1999-06-24 17:00:00','suborbital rocket','success',224,'NASA 36.182GS','SERTS-99 (09)','','','Solar UV',1895,'',300 from dual union all
select 66812,'1999-S39','1999-06-29 00:00:00','military missile','success',469,'','TBM Target','','','Target',433,'',100 from dual union all
select 66813,'1999-S40','1999-06-30 00:00:00','military missile','success',469,'','TBM Target','','','Target',433,'',100 from dual union all
select 66814,'1999-S41','1999-06-30 00:00:00','military missile','success',469,'','TBM Target','','','Target',433,'',100 from dual union all
select 66815,'1999-S42','1999-07-01 03:00:00','suborbital rocket','success',765,'TMBD Target','Malemute II Slugger','','','Target',855,'',120 from dual union all
select 66816,'1999-S43','1999-07-05 03:57:57','suborbital rocket','success',228,'NASA 21.116GE','','','','Ionos?',1821,'',250 from dual union all
select 66817,'1999-S44','1999-07-05 04:02:00','suborbital rocket','success',1319,'NASA 33.065UE','','','','Ionos',1821,'',200 from dual union all
select 66818,'1999-S45','1999-07-05 23:36:30','suborbital rocket','success',226,'NASA 21.123GE','MD-DR-04 DROPPS-1','','','Aeron',29,'',117 from dual union all
select 66819,'1999-S46','1999-07-06 00:06:00','suborbital rocket','success',605,'MD-MI-05','MIDAS','','','Aeron',29,'',85 from dual union all
select 6682,'1960-F09','1960-07-28 09:31:00','orbital','failure',1511,'L1-10','','Vostok-1 No. 1','','Sat',480,'',10 from dual union all
select 66820,'1999-S47','1999-07-14 03:28:45','suborbital rocket','success',226,'NASA 21.124GE','MD-DR-13 DROPPS-2','','','Aeron',29,'',118 from dual union all
select 66821,'1999-S48','1999-07-18 03:41:00','suborbital rocket','success',1319,'NASA 33.066UE','','','','Ionos',1821,'',200 from dual union all
select 66822,'1999-S49','1999-07-27 21:47:50','suborbital rocket','success',227,'NASA 21.122GE','ICON','','','Ionos',417,'',104 from dual union all
select 66823,'1999-S50','1999-08-02 00:00:00','military missile','success',357,'','','','','Test',1494,'',1000 from dual union all
select 66824,'1999-S51','1999-08-02 11:37:00','military missile','success',570,'16','BRV/THAAD T7/ FT-11','','','Target',1857,'',270 from dual union all
select 66825,'1999-S52','1999-08-02 11:44:00','military missile','success',1293,'FT-11','Exo','','','ABM Test',1895,'',100 from dual union all
select 66826,'1999-S53','1999-08-18 18:05:00','suborbital rocket','success',213,'NASA 36.181US','USC-3 (SOHO Cal)','','','Solar EUV',1873,'',314 from dual union all
select 66827,'1999-S54','1999-08-20 08:45:59','military missile','success',799,'','Mk 12/12A RV x 2/RCT-1','RCT-1 GT170GM-1/RRF-6','GLORY TRIP 170GM-1','Op Test',1706,'',1300 from dual union all
select 66828,'1999-S55','1999-08-20 11:27:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT171GM','GLORY TRIP 171GM','Op Test',1705,'',1300 from dual union all
select 66829,'1999-S56','1999-08-24 07:07:02','suborbital rocket','success',224,'NASA 36.119UG','','','','UV Astron?',1895,'',300 from dual union all
select 6683,'1960-F10','1960-08-18 19:58:00','orbital','failure',1359,'262/AB004','Courier 1A','Courier 1A','','Sat',234,'',24 from dual union all
select 66830,'1999-S57','1999-08-27 00:00:00','military missile','failure',646,'','','','','Reentry Test',621,'',675 from dual union all
select 66831,'1999-S58','1999-09-03 11:44:00','military missile','success',1436,'00354-331-02(8L)','15Zh65 LKI-8','','','Test',608,'',1000 from dual union all
select 66832,'1999-S59','1999-09-06 00:00:00','military missile','success',1097,'','TCMP-3A','','','Reentry Test',1784,'',300 from dual union all
select 66833,'1999-S60','1999-09-15 21:00:00','military missile','success',283,'ait-2','','','','Test',862,'',1060 from dual union all
select 66834,'1999-S61','1999-09-16 13:26:00','military missile','success',570,'17','BRV/PAC-3 DT-3','','','Target',436,'',95 from dual union all
select 66835,'1999-S62','1999-09-24 00:00:00','military missile','success',1090,'Aegis CTV-1A','Stellar Phoenix','','','Test',851,'CG-67',107 from dual union all
select 66836,'1999-S63','1999-09-27 10:00:00','suborbital rocket','success',224,'NASA 36.168UG','EOR','','','EUV Astron',1895,'',329 from dual union all
select 66837,'1999-S64','1999-10-01 00:00:00','military missile','success',1507,'','','','','OT',1011,'K-211',1000 from dual union all
select 66838,'1999-S65','1999-10-01 08:42:00','military missile','success',1434,'64536-214-47','15Zh58 UBP-23 (57)','','','Test',608,'',1000 from dual union all
select 66839,'1999-S66','1999-10-02 00:00:00','military missile','success',1507,'','','','','OT',1011,'K-211',1000 from dual union all
select 6684,'1960-F11','1960-09-25 15:13:17','orbital','failure',143,'80D','Able VA','Able VA','','Sat',227,'',1290 from dual union all
select 66840,'1999-S67','1999-10-03 02:01:00','military missile','success',796,'MSLS IFT-3','MSLS IFT-3','MSLS IFT-3','','Target',1698,'',1600 from dual union all
select 66841,'1999-S68','1999-10-03 02:22:00','military missile','success',797,'PLV','IFT-3 EKV','','','ABM Test',868,'',230 from dual union all
select 66842,'1999-S69','1999-10-20 10:00:00','military missile','success',1453,'51178-22-124','15A35 UBP-48','GCh 15F355P7 476673334','','OT',533,'',1000 from dual union all
select 66843,'1999-S70','1999-11-01 00:00:00','military missile','success',6,'','LKI','','','Test',183,'',300 from dual union all
select 66844,'1999-S71','1999-11-01 00:00:00','suborbital rocket','success',1025,'','EWS','','','Ionosphere',1361,'',135 from dual union all
select 66845,'1999-S72','1999-11-10 20:23:56','military missile','success',1443,'FCET-22','','','','OT',420,'SSBN 737',1000 from dual union all
select 66846,'1999-S73','1999-11-10 21:50:00','military missile','success',1443,'FCET-22','-    (87ss)','','','OT',420,'SSBN 737',1000 from dual union all
select 66847,'1999-S74','1999-11-13 08:19:00','military missile','success',799,'SPO/AFSPC','Mk 12/12A RV x 3','PRP FTM-01','PRP FTM-01','Test',1712,'',1300 from dual union all
select 66848,'1999-S75','1999-11-17 00:00:00','military missile','success',1057,'','(Kura)','','','Test',201,'TK-20',1000 from dual union all
select 66849,'1999-S76','1999-11-17 00:00:00','military missile','success',1057,'','(Kura)','','','Test',201,'TK-20',1000 from dual union all
select 6685,'1960-F12','1960-10-10 14:27:49','orbital','failure',802,'L1-4M','','1M No. 1','','Sat',480,'',200 from dual union all
select 66850,'1999-S77','1999-11-18 01:55:00','suborbital rocket','success',1026,'','LMS','','','Ionosphere/Meteor',1361,'',135 from dual union all
select 66851,'1999-S78','1999-11-20 01:35:00','suborbital rocket','success',1026,'','LMS','','','Ionosphere/Meteor',1361,'',135 from dual union all
select 66852,'1999-S79','1999-12-14 08:05:00','military missile','success',1436,'00354-431-03(9L)','15Zh65 LKI-9','','','Test',608,'',1000 from dual union all
select 66853,'1999-S80','1999-12-17 17:00:00','suborbital rocket','success',1334,'NASA 41.020NM','MG. Pool Boiling','','','Micrograv',1821,'',168 from dual union all
select 66854,'1999-S81','1999-06-10 00:00:00','military missile','success',1166,'','vs Camp Ashraf','','','Weapon',857,'',100 from dual union all
select 66855,'1999-S82','1999-06-10 00:00:00','military missile','success',1166,'','vs Camp Ashraf','','','Weapon',857,'',100 from dual union all
select 66856,'1999-S83','1999-06-10 00:00:00','military missile','success',1166,'','vs Camp Ashraf','','','Weapon',857,'',100 from dual union all
select 66857,'1999-S84','1999-11-02 00:00:00','military missile','success',1166,'','vs Camp Habib','','','Weapon',857,'',100 from dual union all
select 66858,'1999-S85','1999-11-02 00:00:00','military missile','success',1166,'','vs Camp Habib','','','Weapon',857,'',100 from dual union all
select 66859,'1999-S86','1999-11-02 00:00:00','military missile','success',1166,'','vs Camp Habib','','','Weapon',857,'',100 from dual union all
select 6686,'1960-F13','1960-10-11 20:33:28','orbital','failure',144,'57D','Samos 1','SAMOS E-1','GIBSON GIRL','Sat',1019,'',200 from dual union all
select 66860,'1999-W01','1999-01-15 14:30:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',55 from dual union all
select 66861,'1999-W02','1999-02-09 22:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',72 from dual union all
select 66862,'1999-W03','1999-03-25 02:45:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',69 from dual union all
select 66863,'1999-W04','1999-07-04 23:38:00','suborbital rocket','success',1284,'MD-CH-01','MIDAS chaff','','','Aeron',29,'',90 from dual union all
select 66864,'1999-W05','1999-07-05 00:04:00','suborbital rocket','success',1502,'MD-FS-02','MIDAS/DROPPS','','','Aeron',29,'',120 from dual union all
select 66865,'1999-W06','1999-07-05 23:13:00','suborbital rocket','success',1502,'MD-FS-03','MIDAS/DROPPS','','','Aeron',29,'',120 from dual union all
select 66866,'1999-W07','1999-07-06 00:46:00','suborbital rocket','success',1502,'MD-MD-06','Mini-DUSTY MD-06','','','Aeron',29,'',120 from dual union all
select 66867,'1999-W08','1999-07-06 01:11:00','suborbital rocket','success',1502,'MD-MM-07','Mini-Midas','','','Aeron',29,'',120 from dual union all
select 66868,'1999-W09','1999-07-06 01:28:00','suborbital rocket','success',1502,'MD-FS-08','MIDAS/DROPPS','','','Aeron',29,'',120 from dual union all
select 66869,'1999-W10','1999-07-06 01:49:00','suborbital rocket','success',1502,'MD-FS-09','MIDAS/DROPPS','','','Aeron',29,'',120 from dual union all
select 6687,'1960-F14','1960-10-14 13:51:03','orbital','failure',802,'L1-5M','','1M No. 2','','Sat',480,'',200 from dual union all
select 66870,'1999-W11','1999-07-06 02:09:30','suborbital rocket','success',1502,'MD-FS-10','MIDAS/DROPPS','','','Aeron',29,'',120 from dual union all
select 66871,'1999-W12','1999-07-06 02:29:00','suborbital rocket','success',1284,'MD-CH-11','MIDAS/DROPPS','','','Aeron',29,'',90 from dual union all
select 66872,'1999-W13','1999-07-14 02:26:00','suborbital rocket','success',1502,'MD-FS-12','MIDAS/DROPPS','','','Aeron',29,'',120 from dual union all
select 66873,'1999-W14','1999-07-14 03:51:00','suborbital rocket','success',1502,'MD-MD-14','Mini-DUSTY MD-07','','','Aeron',29,'',90 from dual union all
select 66874,'1999-W15','1999-07-14 04:10:30','suborbital rocket','success',1502,'MD-FS-15','MIDAS/DROPPS','','','Aeron',29,'',120 from dual union all
select 66875,'1999-W16','1999-07-14 04:38:00','suborbital rocket','success',1502,'MD-CH-16','MIDAS/DROPPS','','','Aeron',29,'',120 from dual union all
select 66876,'1999-W17','1999-07-14 05:04:00','suborbital rocket','success',1502,'MD-FS-17','MIDAS/DROPPS','','','Aeron',29,'',120 from dual union all
select 66877,'1999-W18','1999-07-16 09:04:00','suborbital rocket','success',1502,'MD-FS-18','MIDAS/DROPPS','','','Aeron',29,'',120 from dual union all
select 66878,'1999-W19','1999-07-16 09:30:00','suborbital rocket','success',1284,'MD-CH-19','MIDAS/DROPPS','','','Aeron',29,'',90 from dual union all
select 66879,'1999-W20','1999-07-16 10:12:00','suborbital rocket','success',1502,'MD-FS-20','MIDAS/DROPPS','','','Aeron',29,'',120 from dual;
insert into LAUNCH
select 6688,'1960-F15','1960-10-26 20:26:09','orbital','failure',1361,'253/FTV 1061','Discoverer 16','CORONA C''-1','SOUP SPOON','Sat TAG16',1678,'',213 from dual union all
select 66880,'1999-W21','1999-08-16 12:45:00','suborbital rocket','success',1502,'TR-S-01 Sphere','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66881,'1999-W22','1999-08-17 09:44:00','suborbital rocket','success',1502,'TR-S-02 Sphere','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66882,'1999-W23','1999-08-18 10:23:00','suborbital rocket','success',1502,'TR-S-03 Sphere','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66883,'1999-W24','1999-08-18 10:39:00','suborbital rocket','success',1502,'TR-C-04 Chaff','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66884,'1999-W25','1999-08-18 11:11:25','suborbital rocket','success',1502,'TR-C-05 Chaff','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66885,'1999-W26','1999-08-23 10:30:00','suborbital rocket','success',1502,'TR-S-06 Sphere','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66886,'1999-W27','1999-08-26 10:00:00','suborbital rocket','success',1502,'TR-S-07 Sphere','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66887,'1999-W28','1999-08-26 10:17:00','suborbital rocket','success',1502,'TR-C-08 Chaff','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66888,'1999-W29','1999-08-30 10:00:00','suborbital rocket','success',1502,'TR-S-09 Sphere','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66889,'1999-W30','1999-09-06 10:03:00','suborbital rocket','success',1502,'TR-S-10 Sphere','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 6689,'1960-F16','1960-11-30 19:50:00','orbital','failure',1359,'283/AB006','Transit 3A/GRAB','Transit 3A','','Sat',234,'',100 from dual union all
select 66890,'1999-W31','1999-09-07 02:00:00','suborbital rocket','success',759,'MT-135-70','','','','Aeron',839,'',55 from dual union all
select 66891,'1999-W32','1999-09-09 08:00:00','suborbital rocket','success',1502,'TR-S-11 Sphere','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66892,'1999-W33','1999-09-11 02:00:00','suborbital rocket','success',759,'MT-135-71','','','','Aeron',839,'',55 from dual union all
select 66893,'1999-W34','1999-09-14 10:00:00','suborbital rocket','success',1502,'TR-S-12 Sphere','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66894,'1999-W35','1999-09-14 11:06:00','suborbital rocket','success',1502,'TR-S-13 Sphere','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66895,'1999-W36','1999-09-16 10:00:00','suborbital rocket','success',1502,'TR-S-14 Sphere','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66896,'1999-W37','1999-09-18 10:00:00','suborbital rocket','success',1502,'TR-S-15 Sphere','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66897,'1999-W38','1999-09-20 10:04:00','suborbital rocket','success',1502,'TR-S-16 Sphere','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66898,'1999-W39','1999-09-22 10:03:00','suborbital rocket','success',1502,'TR-S-17 Sphere','TRAMP-99','','','Aeron',29,'',75 from dual union all
select 66899,'1999-Y01','1999-06-16 17:00:00','suborbital rocket','success',1061,'','AN/DMQ-9','','','Meteo',295,'',84 from dual union all
select 669,'1947-A05','1947-02-18 00:00:00','suborbital rocket','failure',1517,'27B','Booster test','','','Test',1852,'',4 from dual union all
select 6690,'1960-F17','1960-12-04 21:14:00','orbital','failure',1138,'ST-3','S-56','NASA S-56','','Sat',1812,'',40 from dual union all
select 66900,'1999-Y02','1999-10-14 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',99 from dual union all
select 66901,'1999-Y03','1999-11-18 17:45:00','suborbital rocket','success',1502,'','Comet Chaser','','','Aeron/Meteorites',1316,'',90 from dual union all
select 66902,'1999-Y04','1999-12-09 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',99 from dual union all
select 66903,'2000-001','2000-01-21 01:03:01','orbital','success',173,'AC-138','USA 148 (DSCS 3B-8)','DSCS III B-8','','Sat',288,'',null from dual union all
select 66904,'2000-002','2000-01-25 01:04:00','orbital','success',99,'V126 L494','Galaxy 10R','Galaxy XR','','Sat',345,'',null from dual union all
select 66905,'2000-003','2000-01-25 16:45:05','orbital','success',309,'Y4       (60)','Zhongxing-22','Feng Huo 1','','Sat',1903,'',null from dual union all
select 66906,'2000-004','2000-01-27 03:03:06','orbital','success',783,'1','P98-1 JAWSAT','JAWSAT','','Sat',1781,'',null from dual union all
select 66907,'2000-005','2000-02-01 06:47:23','orbital','success',1241,'A 15000-669 669','Progress M1-1','Progress 7K-TGM No. 250','','Sat',480,'',null from dual union all
select 66908,'2000-006','2000-02-03 09:26:00','orbital','success',1537,'45025-801','Kosmos-2369','Tselina-2','','Sat',567,'',null from dual union all
select 66909,'2000-007','2000-02-03 23:30:01','orbital','success',174,'AC-158','Hispasat 1C','Hispasat 1C','','Sat',289,'',null from dual union all
select 6691,'1960-F18','1960-12-15 09:10:00','orbital','failure',143,'91D','Able VB','Able VB','','Sat',227,'',13 from dual union all
select 66910,'2000-008','2000-02-08 21:24:00','orbital','success',413,'D276','Globalstar-7','Globalstar 63','','Sat',286,'',null from dual union all
select 66911,'2000-009','2000-02-08 23:20:00','orbital','success',1239,'A 15000-079 079 ST07','IRDT/GVM','Dummy satellite','','Sat',562,'',null from dual union all
select 66912,'2000-010','2000-02-11 17:43:40','orbital','success',1243,'STS-99','SRTM (SRL-3)','OV-105 Spacelab','','Sat',921,'MLP3',null from dual union all
select 66913,'2000-011','2000-02-12 09:10:54','orbital','success',960,'399-02','Garuda 1','Garuda 1','','Sat',587,'',null from dual union all
select 66914,'2000-012','2000-02-18 01:04:00','orbital','success',108,'V127 L495','Superbird 4','Superbird 4','','Sat',345,'',null from dual union all
select 66915,'2000-013','2000-03-12 04:07:00','orbital','success',959,'399-01','Ekspress A No. 2','Ekspress A No. 2','','Sat',553,'',null from dual union all
select 66916,'2000-014','2000-03-12 09:29:00','orbital','success',1313,'T5','MTI','MTI','','Sat',1670,'',null from dual union all
select 66917,'2000-015','2000-03-20 18:28:30','orbital','success',1239,'A 15000-068 068 ST08','Dumsat','Dumsat','','Sat',562,'',null from dual union all
select 66918,'2000-016','2000-03-21 23:28:19','orbital','success',114,'V128 (505)','Asiastar/Insat 3B','Asiastar','','Sat',346,'',null from dual union all
select 66919,'2000-017','2000-03-25 20:34:43','orbital','success',412,'D277','IMAGE','IMAGE','','Sat',1724,'',null from dual union all
select 6692,'1960-F19','1960-12-22 07:45:19','orbital','failure',1512,'L1-13A','','Vostok-1 No. 4','','Sat',480,'',214 from dual union all
select 66920,'2000-018','2000-04-04 05:01:29','orbital','success',1241,'A 15000-665 665','Soyuz TM-30','Soyuz 7K-STM No. 204','','Sat',480,'',null from dual union all
select 66921,'2000-019','2000-04-17 21:06:00','orbital','success',959,'397-01','Sesat','SESAT','','Sat',553,'',null from dual union all
select 66922,'2000-020','2000-04-19 00:29:00','orbital','success',99,'V129 L496','Galaxy 4R','Galaxy IVR','','Sat',345,'',null from dual union all
select 66923,'2000-021','2000-04-25 20:08:02','orbital','success',1241,'A 15000-664 664','Progress M1-2','Progress M1 No. 252','','Sat',480,'',null from dual union all
select 66924,'2000-022','2000-05-03 07:07:03','orbital','success',173,'AC-137','GOES L','GOES L','','Sat',288,'',null from dual union all
select 66925,'2000-023','2000-05-03 13:25:00','orbital','success',1241,'A 15000-649 649','Kosmos-2370','Neman','','Sat',480,'',null from dual union all
select 66926,'2000-024','2000-05-08 16:01:00','orbital','success',1412,'4B-29','USA 149      [DSP20]','DSP 18?','','Sat',274,'',null from dual union all
select 66927,'2000-025','2000-05-11 01:48:00','orbital','success',427,'D278','USA 150 (GPS IIR-4)','Navstar GPS SVN 51','','Sat',285,'',null from dual union all
select 66928,'2000-026','2000-05-16 08:27:41','orbital','success',1064,'630-6992-011P/72501','Simsat-1/2','EPN 813IP/003 (Iridium)','','Sat',460,'',1000 from dual union all
select 66929,'2000-027','2000-05-19 10:11:10','orbital','success',1243,'STS-101','ISS 2A.2a','OV-104 ISS','','Sat',921,'MLP1',null from dual union all
select 6693,'1960-M01','1960-02-03 21:02:00','military missile','success',873,'ZW-4/105','4','','','Test',1857,'',40 from dual union all
select 66930,'2000-028','2000-05-24 23:10:05','orbital','success',139,'AC-201','Eutelsat W4','Eutelsat W4','','Sat',289,'',null from dual union all
select 66931,'2000-029','2000-06-06 02:59:00','orbital','success',952,'392-01/88502','Gorizont No. 45','Gorizont No. 45L','','Sat',588,'',null from dual union all
select 66932,'2000-030','2000-06-07 13:19:30','orbital','success',911,'F29','TSX-5','TSX-5','','Sat',1718,'L-1011',null from dual union all
select 66933,'2000-031','2000-06-24 00:28:00','orbital','success',959,'394-02','Ekspress A No. 3','Ekspress-A No. 3','','Sat',553,'',null from dual union all
select 66934,'2000-032','2000-06-25 11:50:00','orbital','success',308,'Y12      (61)','Feng Yun 2','Feng Yun Er Yi','','Sat',1904,'',null from dual union all
select 66935,'2000-033','2000-06-28 10:37:42','orbital','success',680,'53752-105','Nadezhda/Tsinghua','Nadezhda 17F118 No. 701','','Sat',456,'',null from dual union all
select 66936,'2000-034','2000-06-30 12:56:01','orbital','success',173,'AC-139','TDRS 8','TDRS H','','Sat',288,'',null from dual union all
select 66937,'2000-035','2000-06-30 22:08:47','orbital','success',960,'400-01','Sirius CD Radio 1','Sirius CD Radio 1','','Sat',588,'',null from dual union all
select 66938,'2000-036','2000-07-04 23:44:00','orbital','success',958,'389-02','Kosmos-2371','Geizer No. 22L','','Sat',553,'',null from dual union all
select 66939,'2000-037','2000-07-12 04:56:36','orbital','success',949,'398-01','Zvezda','77KSM No. 128-01','','Sat ISS 1R',587,'',null from dual union all
select 6694,'1960-M02','1960-02-04 20:11:00','suborbital rocket','success',868,'','','','','Aeron/Tech',1016,'',400 from dual union all
select 66940,'2000-038','2000-07-14 05:21:01','orbital','success',174,'AC-161','Echostar VI','Echostar VI','','Sat',289,'',null from dual union all
select 66941,'2000-039','2000-07-15 12:00:00','orbital','success',680,'47136-414','CHAMP/BIRD/MITA','MITA-NINA','','Sat',456,'',null from dual union all
select 66942,'2000-040','2000-07-16 09:17:00','orbital','success',427,'D279','USA 151 (GPS IIR-5)','Navstar GPS SVN 44','','Sat',285,'',null from dual union all
select 66943,'2000-041','2000-07-16 12:39:34','orbital','success',1239,'A 15000-069 069 ST09','Cluster FM6/FM7','Cluster FM7 (C3)','','Sat',562,'',null from dual union all
select 66944,'2000-042','2000-07-19 20:09:00','orbital','success',783,'2','Mightysat 2.1','Mightysat 2.1','','Sat',1781,'',null from dual union all
select 66945,'2000-043','2000-07-28 22:42:00','orbital','success',1542,'5/DM-SL-4L','SL4 PAS 9','PAS 9','','Sat',860,'ODYSSEY',null from dual union all
select 66946,'2000-044','2000-08-06 18:26:42','orbital','success',1241,'K 15000-668 668','Progress M1-3','Progress M1 No. 251','','Sat ISS 1P',480,'',null from dual union all
select 66947,'2000-045','2000-08-09 11:13:35','orbital','success',1239,'A 15000-070 070/ST10','Cluster FM5/FM8','Cluster FM5 (C1)','','Sat',562,'',null from dual union all
select 66948,'2000-046','2000-08-17 23:16:00','orbital','success',108,'V131 L4097','Brasil B4/Nilest 102','Brasilsat B4','','Sat',345,'',null from dual union all
select 66949,'2000-047','2000-08-17 23:45:01','orbital','success',1414,'4B-28','USA 152      [LAC4]','ONYX 4','','Sat',1777,'',null from dual union all
select 6695,'1960-M03','1960-03-09 00:00:00','military missile','success',873,'ZW-5/106','5','','','Test',1884,'',40 from dual union all
select 66950,'2000-048','2000-08-23 11:05:02','orbital','success',429,'D280','DM-F3','DM-F3','','Sat',286,'',null from dual union all
select 66951,'2000-049','2000-08-28 20:08:00','orbital','success',958,'401-02','Raduga-1','Globus No. 16L','','Sat',588,'',null from dual union all
select 66952,'2000-050','2000-09-01 03:25:00','orbital','success',319,'Y3       (62)','ZY-2','ziyuan erhao 01 xing','','Sat',1492,'',null from dual union all
select 66953,'2000-051','2000-09-05 09:43:58','orbital','success',960,'400-02','Sirius CD Radio 2','Sirius CD Radio 2','','Sat',587,'',null from dual union all
select 66954,'2000-052','2000-09-06 22:33:00','orbital','success',110,'V132 L4098','Eutelsat W1','Eutelsat W1','','Sat',345,'',null from dual union all
select 66955,'2000-053','2000-09-08 12:45:47','orbital','success',1243,'STS-106','ISS 2A.2b','OV-104 ISS','','Sat',922,'MLP2',null from dual union all
select 66956,'2000-054','2000-09-14 22:54:07','orbital','success',114,'V130 (506)','Astra 2B/GE 7','Astra 2B','','Sat',346,'',null from dual union all
select 66957,'2000-055','2000-09-21 10:22:00','orbital','success',1422,'23G-13','NOAA 16','NOAA L','','Sat',1778,'',null from dual union all
select 66958,'2000-056','2000-09-25 10:10:00','orbital','success',1537,'67013-303','Kosmos-2372','Yenisey','','Sat',567,'',null from dual union all
select 66959,'2000-057','2000-09-26 10:05:00','orbital','success',434,'450-8752-403','Tiungsat/Megsat-1/','GCh 15F183','','Sat',497,'',null from dual union all
select 6696,'1960-M05','1960-04-18 00:00:00','military missile','failure',928,'A1X-19','','','','Test',421,'EAG-154',50 from dual union all
select 66960,'2000-058','2000-09-29 09:30:15','orbital','success',1241,'76023  -462 075','Kosmos-2373','Kometa No. 20?','','Sat',562,'',null from dual union all
select 66961,'2000-059','2000-10-01 22:00:00','orbital','success',960,'401-01','GE-1A','GE-1A','','Sat',587,'',null from dual union all
select 66962,'2000-060','2000-10-06 23:00:00','orbital','success',99,'V133 L4099','N-SAT-110','Superbird 5','','Sat',345,'',null from dual union all
select 66963,'2000-061','2000-10-09 05:38:18','orbital','success',910,'F30/P-35','HETE 2','HETE 2','','Sat',872,'L-1011',null from dual union all
select 66964,'2000-062','2000-10-11 23:17:00','orbital','success',1243,'STS-92','ISS 3A','OV-103 ISS','','Sat',921,'MLP3',null from dual union all
select 66965,'2000-063','2000-10-13 14:12:45','orbital','success',958,'393-01','Kosmos-2374/2376','Uragan No. 83L','','Sat',588,'',null from dual union all
select 66966,'2000-064','2000-10-16 21:27:06','orbital','success',1241,'K 15000-085 085','Progress M-43','Progress 7K-TGM No. 243','','Sat',480,'',null from dual union all
select 66967,'2000-065','2000-10-20 00:40:02','orbital','success',173,'AC-140 MLV-9','USA 153 (DSCS3B-11)','DSCS III B-11','','Sat',288,'',null from dual union all
select 66968,'2000-066','2000-10-21 05:52:00','orbital','success',1542,'6L/DM-SL-6L','SL5 Thuraya','Thuraya 1','','Sat',860,'ODYSSEY',null from dual union all
select 66969,'2000-067','2000-10-21 22:00:00','orbital','success',960,'402-01','GE-6','GE 6','','Sat',587,'',null from dual;
insert into LAUNCH
select 6697,'1960-M06','1960-04-20 00:00:00','suborbital rocket','failure',14,'','Model A (F4)','','','Test',383,'F-104',70 from dual union all
select 66970,'2000-068','2000-10-29 05:59:00','orbital','success',108,'V134 L4100','Europe*Star F1','EuropeStar FM1','','Sat',345,'',null from dual union all
select 66971,'2000-069','2000-10-30 16:02:00','orbital','success',309,'Y5      (63)','Beidou','yi ke beidou daohang s.w','','Sat',1903,'',null from dual union all
select 66972,'2000-070','2000-10-31 07:52:47','orbital','success',1241,'A 15000-666 666','Soyuz TM-31','Soyuz 7K-STM No. 205','','Sat ISS 1S',480,'',null from dual union all
select 66973,'2000-071','2000-11-10 17:14:02','orbital','success',427,'D281','USA 154 (GPS IIR-6)','Navstar GPS SVN 41','','Sat',285,'',null from dual union all
select 66974,'2000-072','2000-11-16 01:07:07','orbital','success',114,'V135 (507)','PAS 1R/AMSAT P3D','PAS 1R','','Sat',346,'',null from dual union all
select 66975,'2000-073','2000-11-16 01:32:36','orbital','success',1241,'K 15000-671 671','Progress M1-4','Progress M1 No. 253','','Sat ISS 2P',480,'',null from dual union all
select 66976,'2000-074','2000-11-20 23:00:00','orbital','failure',680,'47165-631','QuickBird 1','QuickBird 1','','Sat',456,'',614 from dual union all
select 66977,'2000-075','2000-11-21 18:24:25','orbital','success',410,'D282','EO-1/SAC-C','Earth Orbiter 1','','Sat',1724,'',null from dual union all
select 66978,'2000-076','2000-11-21 23:56:00','orbital','success',105,'V136 L4102','Anik F1','Anik F1','','Sat',345,'',null from dual union all
select 66979,'2000-077','2000-11-30 19:59:47','orbital','success',960,'402-02','Sirius CD Radio 3','Sirius CD Radio 3','','Sat',587,'',null from dual union all
select 6698,'1960-M07','1960-04-29 00:00:00','suborbital rocket','success',868,'','','','','Aeron?',1016,'',80 from dual union all
select 66980,'2000-078','2000-12-01 03:06:01','orbital','success',1243,'STS-97','ISS 4A','OV-105 ISS','','Sat',922,'MLP1',null from dual union all
select 66981,'2000-079','2000-12-05 12:32:00','orbital','success',1260,'434','EROS A1','EROS A1','','Sat',478,'',null from dual union all
select 66982,'2000-080','2000-12-06 02:47:01','orbital','success',174,'AC-157 MLV-11','USA 155      [SDS13]','QUASAR 13?','','Sat',288,'',null from dual union all
select 66983,'2000-081','2000-12-20 00:26:00','orbital','success',114,'V138 (508)','Astra 2D/GE 8/LDREX','Astra 2D','','Sat',346,'',null from dual union all
select 66984,'2000-082','2000-12-20 16:20:00','orbital','success',309,'Y6      (64)','Beidou','er ke beidou daohang s.w','','Sat',1903,'',null from dual union all
select 66985,'2000-A01','2000-02-26 04:59:00','suborbital rocket','failure',224,'NASA 36.162DG','J-PEX 1','','','UV Astron',1895,'',10 from dual union all
select 66986,'2000-A02','2000-04-11 08:05:00','military missile','failure',430,'','Test','','','Test',320,'',1 from dual union all
select 66987,'2000-A03','2000-06-08 00:00:00','atmospheric rocket','success',1087,'Aegis TMT','PACBLITZ 00','','','Target',855,'',10 from dual union all
select 66988,'2000-A04','2000-06-08 00:00:00','atmospheric rocket','success',1087,'Aegis TMT','PACBLITZ 00','','','Target',855,'',10 from dual union all
select 66989,'2000-A05','2000-08-24 14:50:00','military missile','success',1088,'Aegis CTV-2','Block IVA','','','Test',1869,'',10 from dual union all
select 6699,'1960-M08','1960-04-30 00:00:00','suborbital rocket','success',14,'','Model A (F5)','','','Test',383,'F-104',62 from dual union all
select 66990,'2000-A06','2000-09-14 09:00:00','military missile','success',244,'','','','','Test',1026,'F-15D',40 from dual union all
select 66991,'2000-A07','2000-09-29 15:40:00','suborbital rocket','failure',271,'','','','','Test',208,'',14 from dual union all
select 66992,'2000-A08','2000-09-01 00:00:00','military missile','failure',71,'F6','Test','','','Test',761,'',1 from dual union all
select 66993,'2000-A09','2000-10-23 00:00:00','military missile','success',71,'F8','Test','','','Test',761,'',20 from dual union all
select 66994,'2000-A10','2000-09-21 00:00:00','military missile','failure',1168,'Shahab-3D','','','','Test',1348,'',1 from dual union all
select 66995,'2000-F02','2000-03-12 14:49:15','orbital','failure',1542,'2L/DM-SL-2L','SL3 ICO F-1','ICO F1','','Sat',860,'ODYSSEY',200 from dual union all
select 66996,'2000-F03','2000-12-27 09:56:31','orbital','failure',1448,'450-34-  503','Gonets/Kosmos','Gonets-D1 No. 7','','Sat',469,'',200 from dual union all
select 66997,'2000-M01','2000-01-30 10:25:01','suborbital rocket','success',891,'NASA 30.044UP','ASRP SRP-3','','','Education',1047,'',79 from dual union all
select 66998,'2000-M02','2000-02-05 14:04:00','military missile','success',570,'18','MBRV-3/PAC-3 DT-5','','','Target',436,'',60 from dual union all
select 66999,'2000-M03','2000-02-21 00:00:00','suborbital rocket','success',1023,'','Atm Waves (1 of 40)','','','Aeron',1361,'',70 from dual union all
select 67,'1944-A31','1944-12-27 00:00:00','military missile','failure',9,'','','','','Test',677,'',0 from dual union all
select 670,'1947-A06','1947-02-19 00:00:00','suborbital rocket','success',1517,'28B','','','','Test',1852,'',44 from dual union all
select 6700,'1960-M09','1960-05-19 00:00:00','military missile','success',1162,'30','JPL-500','','','OT?',1895,'',50 from dual union all
select 67000,'2000-M04','2000-02-22 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67001,'2000-M05','2000-02-23 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67002,'2000-M06','2000-02-24 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67003,'2000-M07','2000-02-25 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67004,'2000-M08','2000-02-26 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67005,'2000-M09','2000-02-27 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67006,'2000-M10','2000-02-28 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67007,'2000-M11','2000-03-01 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67008,'2000-M12','2000-03-02 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67009,'2000-M13','2000-03-03 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 6701,'1960-M10','1960-05-20 00:00:00','military missile','success',1162,'31','JPL-500','','','OT?',1895,'',50 from dual union all
select 67010,'2000-M14','2000-03-04 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67011,'2000-M15','2000-03-05 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67012,'2000-M16','2000-03-06 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67013,'2000-M17','2000-03-07 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67014,'2000-M18','2000-03-08 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67015,'2000-M19','2000-03-09 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67016,'2000-M20','2000-03-10 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67017,'2000-M21','2000-03-11 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67018,'2000-M22','2000-03-12 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67019,'2000-M23','2000-03-13 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 6702,'1960-M11','1960-05-23 00:00:00','military missile','success',1162,'33','','','','OT?',1895,'',50 from dual union all
select 67020,'2000-M24','2000-03-14 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67021,'2000-M25','2000-03-15 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67022,'2000-M26','2000-03-16 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67023,'2000-M27','2000-03-17 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67024,'2000-M28','2000-03-18 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67025,'2000-M29','2000-03-19 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67026,'2000-M30','2000-03-20 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67027,'2000-M31','2000-03-21 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67028,'2000-M32','2000-03-22 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67029,'2000-M33','2000-03-23 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 6703,'1960-M12','1960-05-31 00:00:00','military missile','success',1162,'32','','','','OT?',1895,'',50 from dual union all
select 67030,'2000-M34','2000-03-24 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67031,'2000-M35','2000-03-25 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67032,'2000-M36','2000-03-26 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67033,'2000-M37','2000-03-27 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67034,'2000-M38','2000-03-28 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67035,'2000-M39','2000-03-29 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67036,'2000-M40','2000-03-30 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67037,'2000-M41','2000-03-31 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67038,'2000-M42','2000-04-01 00:00:00','suborbital rocket','success',1023,'','Atm Waves','','','Aeron',1361,'',70 from dual union all
select 67039,'2000-M43','2000-09-14 09:05:00','military missile','success',119,'F8','AST-5 Black Sparrow','','','ATBM Test',1026,'',60 from dual union all
select 6704,'1960-M13','1960-05-25 00:00:00','military missile','success',873,'ZW-7','7','','','Test',1884,'',40 from dual union all
select 67040,'2000-M44','2000-02-17 09:38:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67041,'2000-M45','2000-03-12 13:15:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67042,'2000-M46','2000-04-25 12:01:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67043,'2000-M47','2000-05-23 10:30:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67044,'2000-M48','2000-11-13 00:00:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67045,'2000-M49','2000-11-15 00:00:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67046,'2000-M50','2000-11-20 12:15:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67047,'2000-M51','2000-12-12 08:30:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67048,'2000-M52','2000-11-18 00:00:00','military missile','success',71,'F9','Test','','','Test',761,'',60 from dual union all
select 67049,'2000-M53','2000-02-01 00:00:00','military missile','success',568,'Hatf-IA','','','','Test',1419,'',70 from dual union all
select 6705,'1960-M14','1960-06-01 00:00:00','suborbital rocket','success',86,'','HIGHBALL test','','','Test',1895,'',60 from dual union all
select 67050,'2000-S01','2000-01-09 20:50:00','suborbital rocket','success',1073,'S-310-29','WAVE2000','','','Aeron',833,'',179 from dual union all
select 67051,'2000-S02','2000-01-19 02:19:00','military missile','success',796,'MSLS IFT-4','MSLS IFT-4','MSLS IFT-4','','Target',1698,'',1600 from dual union all
select 67052,'2000-S03','2000-01-19 02:40:00','military missile','success',797,'PLV','IFT-4 EKV','','','ABM Test',868,'',230 from dual union all
select 67053,'2000-S04','2000-02-06 16:37:00','suborbital rocket','success',1478,'XV-05','Lencois Maranhenses','','','Micrograv',13,'',148 from dual union all
select 67054,'2000-S05','2000-02-09 09:59:00','military missile','success',1436,'10L','15Zh65 LKI-10','','','Test',608,'',1000 from dual union all
select 67055,'2000-S06','2000-02-12 03:27:00','suborbital rocket','success',213,'NASA 36.186UG','JHU FOT 15','','','UV Astron',1895,'',300 from dual union all
select 67056,'2000-S07','2000-02-25 07:45:10','suborbital rocket','success',237,'NASA 40.008UE','','','','Auroral',1043,'',1170 from dual union all
select 67057,'2000-S08','2000-02-26 20:00:00','military missile','success',1442,'FCET-51','','','','OT',420,'SSBN 730',1000 from dual union all
select 67058,'2000-S09','2000-02-26 20:00:00','military missile','success',1442,'FCET-51','','','','OT',420,'SSBN 730',1000 from dual;
insert into LAUNCH
select 67059,'2000-S10','2000-02-26 22:43:00','military missile','success',1442,'FCET-51','','','','OT',420,'SSBN 730',1000 from dual union all
select 6706,'1960-M15','1960-06-07 00:00:00','military missile','failure',928,'A1X-27','','','','Test',244,'',50 from dual union all
select 67060,'2000-S11','2000-02-26 22:43:00','military missile','success',1442,'FCET-51','','','','OT',420,'SSBN 730',1000 from dual union all
select 67061,'2000-S12','2000-02-26 09:19:11','suborbital rocket','success',237,'NASA 40.015IE','GEODESIC','','','Ionos',1044,'',990 from dual union all
select 67062,'2000-S13','2000-03-08 09:40:30','military missile','success',907,'29PA (84-0553)','Mk 21 RV x 8','FDEP GT 29PA','','OT',1700,'',1000 from dual union all
select 67063,'2000-S14','2000-03-27 06:00:00','military missile','success',1179,'','','','','OT',201,'K-18',1000 from dual union all
select 67064,'2000-S15','2000-03-27 09:00:00','military missile','success',1179,'','','','','OT',201,'K-18',1000 from dual union all
select 67065,'2000-S16','2000-03-27 11:17:00','suborbital rocket','success',1212,'TEXUS 37','DLR TEXUS 37','','','Micrograv',416,'',243 from dual union all
select 67066,'2000-S17','2000-04-02 07:34:00','suborbital rocket','success',1212,'TEXUS 38','DLR TEXUS 38','','','Micrograv',416,'',250 from dual union all
select 67067,'2000-S18','2000-04-06 00:00:00','military missile','success',629,'','','','','OT',988,'',100 from dual union all
select 67068,'2000-S19','2000-04-06 05:00:00','military missile','success',1507,'','','','','OT',201,'K-496',1000 from dual union all
select 67069,'2000-S20','2000-04-30 19:40:32','military missile','success',1443,'FCET-23','-    (88ss)','','','OT',420,'SSBN 742',1000 from dual union all
select 6707,'1960-M16','1960-06-27 00:00:00','suborbital rocket','success',868,'','','','','Aeron?',1016,'',80 from dual union all
select 67070,'2000-S21','2000-04-30 00:00:00','military missile','success',1443,'FCET-23','-    (89ss)','','','OT',421,'SSBN 742',1000 from dual union all
select 67071,'2000-S22','2000-05-03 00:00:00','military missile','success',469,'','TBM Target','','','Target',433,'',90 from dual union all
select 67072,'2000-S23','2000-05-04 00:00:00','military missile','success',469,'','TBM Target','','','Target',433,'',90 from dual union all
select 67073,'2000-S24','2000-05-04 00:00:00','military missile','success',469,'','TBM Target','','','Target',433,'',90 from dual union all
select 67074,'2000-S25','2000-05-04 00:00:00','military missile','success',469,'','TBM Target','','','Target',433,'',90 from dual union all
select 67075,'2000-S26','2000-05-04 00:00:00','military missile','success',469,'','TBM Target','','','Target',433,'',90 from dual union all
select 67076,'2000-S27','2000-05-06 06:34:59','suborbital rocket','failure',224,'NASA 36.183UG','','','','UV Astron?',1895,'',273 from dual union all
select 67077,'2000-S28','2000-05-06 17:08:00','suborbital rocket','success',605,'MS-MI-03','MIDAS Ferdinand-115','','','Aeron',29,'',85 from dual union all
select 67078,'2000-S29','2000-05-15 00:46:00','suborbital rocket','success',605,'MS-MI-05','MIDAS Ferdinand-116','','','Aeron',29,'',85 from dual union all
select 67079,'2000-S30','2000-05-17 10:18:39','suborbital rocket','success',860,'NASA 31.122UP','SPIRIT 1','','','Aeron',1821,'',140 from dual union all
select 6708,'1960-M17','1960-07-26 00:00:00','military missile','success',1162,'36','','','','OT?',1895,'',50 from dual union all
select 67080,'2000-S31','2000-05-24 08:01:00','military missile','failure',799,'SPO/AFSPC','Mk 12/12A RV x 3','PRP FTM-02','PRP FTM-02','Test',1705,'',1300 from dual union all
select 67081,'2000-S32','2000-05-28 20:00:00','military missile','success',784,'TLV Demo','TLV CFE','OSPTLV 1','','Target',1701,'',1300 from dual union all
select 67082,'2000-S33','2000-06-09 08:01:00','military missile','success',799,'','Mk 12A RV x 3','FOT GT172GM','GLORY TRIP 172GM','Op Test',1706,'',1300 from dual union all
select 67083,'2000-S34','2000-06-13 16:06:00','suborbital rocket','success',860,'NASA 31.121UP','','','','Aeron?',1821,'',140 from dual union all
select 67084,'2000-S35','2000-06-16 08:45:00','military missile','success',941,'17','','','','Test',320,'',100 from dual union all
select 67085,'2000-S36','2000-06-18 11:03:00','suborbital rocket','success',891,'NASA 30.045NO','','','','Education',1821,'',85 from dual union all
select 67086,'2000-S37','2000-06-21 10:43:00','suborbital rocket','success',860,'NASA 31.120UP','','','','Aeron?',1821,'',140 from dual union all
select 67087,'2000-S38','2000-07-14 00:00:00','military missile','failure',1090,'Aegis FTR-1','Stellar Archer','','','Test',851,'CG-70',100 from dual union all
select 67088,'2000-S39','2000-07-06 14:41:00','suborbital rocket','success',224,'NASA 36.166UM','','','','Micrograv',1895,'',300 from dual union all
select 67089,'2000-S40','2000-07-07 10:25:00','suborbital rocket','success',1344,'','','','','Test',1821,'',368 from dual union all
select 6709,'1960-M18','1960-06-30 20:25:00','suborbital rocket','failure',624,'NASA 8.07GE','Magnetic fields','','','Magnetospheric',1821,'',65 from dual union all
select 67090,'2000-S41','2000-07-08 04:19:00','military missile','success',796,'MSLS IFT-5','MSLS IFT-5','MSLS IFT-5','','Target',1698,'',1300 from dual union all
select 67091,'2000-S42','2000-07-08 04:40:00','military missile','success',797,'PLV IFT-5','IFT-5 EKV','','','ABM Test',868,'',230 from dual union all
select 67092,'2000-S43','2000-07-15 00:00:00','military missile','success',1168,'','','','','Test',947,'',150 from dual union all
select 67093,'2000-S44','2000-07-17 06:17:00','suborbital rocket','success',211,'NASA 27.143UE','','','','Ionos',1821,'',340 from dual union all
select 67094,'2000-S45','2000-07-26 18:06:00','suborbital rocket','success',224,'NASA 36.185GS','SERTS-00 (10)','','','Solar UV',1873,'',300 from dual union all
select 67095,'2000-S46','2000-08-05 05:00:02','suborbital rocket','success',224,'NASA 36.189UG','SPINR 3','','','UV Astron',1895,'',406 from dual union all
select 67096,'2000-S47','2000-08-21 20:14:00','suborbital rocket','success',1479,'XV-01','Baronesa','','','Micrograv',13,'',315 from dual union all
select 67097,'2000-S48','2000-09-01 00:00:00','military missile','success',1153,'','','','','Test',1457,'',150 from dual union all
select 67098,'2000-S49','2000-09-05 18:02:00','suborbital rocket','success',224,'NASA 36.151US','','','','Solar X',1895,'',300 from dual union all
select 67099,'2000-S50','2000-09-12 00:00:00','military missile','success',1179,'VMF RF','','','','OT',201,'K-18',1000 from dual union all
select 671,'1947-A07','1947-04-01 00:00:00','atmospheric rocket','success',365,'','Test','','','Test',1821,'',10 from dual union all
select 6710,'1960-M19','1960-07-01 00:00:00','suborbital rocket','unknown',366,'','','','','Aeron?',1016,'',80 from dual union all
select 67100,'2000-S52','2000-09-21 20:50:00','military missile','success',1443,'DASO-7','-    (90ss)','','','OT',421,'UK S31',1000 from dual union all
select 67101,'2000-S53','2000-09-23 00:00:00','military missile','success',1153,'','','','','Test',1457,'',150 from dual union all
select 67102,'2000-S54','2000-09-26 11:00:00','military missile','success',1436,'00354-242-11(11L)','15Zh65 K-1','','','Test',608,'',1000 from dual union all
select 67103,'2000-S55','2000-09-27 09:50:00','military missile','success',1437,'00354-431-04(1L)','15Zh55 LKI-1','','','Test',606,'',1000 from dual union all
select 67104,'2000-S56','2000-09-28 08:01:00','military missile','success',799,'','Mk 12A RV x2/SHARP','GT173GM/SHARP-B2','GLORY TRIP 173GM','Op Test',1705,'',1300 from dual union all
select 67105,'2000-S57','2000-09-28 10:01:00','military missile','success',799,'','Mk 12A RV x2/RCT-2','GT174GM/RCT-2 FOT','GLORY TRIP 174GM','Op Test/Target',1699,'',1300 from dual union all
select 67106,'2000-S58','2000-10-11 09:30:00','military missile','success',1434,'64536-214-44','15Zh58 UBP-24 (58)','','','OT',608,'',1000 from dual union all
select 67107,'2000-S59','2000-10-13 14:45:02','suborbital rocket','success',1334,'NASA 41.021DP','','','','Test?',1895,'',100 from dual union all
select 67108,'2000-S60','2000-10-14 13:30:00','military missile','success',1263,'PAC-3 DT-6','MTTV-6','','','Target',436,'',104 from dual union all
select 67109,'2000-S61','2000-10-26 09:57:00','suborbital rocket','success',228,'NASA 21.126CE','TOMEX','','','Plasma',1895,'',143 from dual union all
select 6711,'1960-M20','1960-07-08 00:00:00','suborbital rocket','unknown',366,'','','','','Aeron?',1016,'',80 from dual union all
select 67110,'2000-S62','2000-11-01 10:00:00','military missile','success',1453,'51178-22-133 (87)','15A35 UBP-49','GCh 15F355P7 52459212','','OT',538,'',1000 from dual union all
select 67111,'2000-S63','2000-11-04 00:00:00','military missile','success',357,'','','','','Test',1494,'',1000 from dual union all
select 67112,'2000-S64','2000-12-04 09:16:00','suborbital rocket','success',1101,'SS-520-2','Ion Outflow','','','Ionos',1453,'',1108 from dual union all
select 67113,'2000-S65','2000-12-09 12:20:00','suborbital rocket','success',1219,'XV-30','Alecrim','','','Micrograv',330,'',430 from dual union all
select 67114,'2000-S66','2000-12-16 00:00:00','military missile','success',357,'','','','','Test',1494,'',1000 from dual union all
select 67115,'2000-S67','2000-12-19 11:57:00','suborbital rocket','success',1336,'NASA 12.50DP','NAWC','','','Test',1821,'',300 from dual union all
select 67116,'2000-S68','2000-12-27 08:00:00','military missile','success',1179,'','','','','OT',201,'K-407',1000 from dual union all
select 67117,'2000-U01','2000-02-10 01:30:00','orbital','failure',731,'M-V-4','ASTRO E','ASTRO E','','Sat',836,'',200 from dual union all
select 67118,'2000-W01','2000-01-12 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',53 from dual union all
select 67119,'2000-W02','2000-01-19 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 6712,'1960-M21','1960-07-14 00:00:00','suborbital rocket','unknown',366,'','','','','Aeron?',1016,'',80 from dual union all
select 67120,'2000-W03','2000-01-21 16:22:00','suborbital rocket','success',1284,'LiteStar','','','','Education',239,'',75 from dual union all
select 67121,'2000-W04','2000-01-26 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 67122,'2000-W05','2000-02-09 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',57 from dual union all
select 67123,'2000-W06','2000-02-24 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 67124,'2000-W07','2000-03-15 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 67125,'2000-W08','2000-04-12 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 67126,'2000-W09','2000-04-19 02:10:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 67127,'2000-W10','2000-05-06 16:21:00','suborbital rocket','success',1502,'MS-FS-01','MIDAS Sphere','','','Aeron',29,'',90 from dual union all
select 67128,'2000-W11','2000-05-06 16:41:00','suborbital rocket','success',1502,'MS-FS-02','MIDAS Sphere','','','Aeron',29,'',95 from dual union all
select 67129,'2000-W12','2000-05-15 00:32:00','suborbital rocket','success',1502,'MS-FS-04','MIDAS Sphere','','','Aeron',29,'',92 from dual union all
select 6713,'1960-M22','1960-08-12 00:00:00','military missile','success',1162,'35','','','','OT?',1895,'',50 from dual union all
select 67130,'2000-W13','2000-05-15 01:12:00','suborbital rocket','success',1502,'MS-MM-06','Ferd. 117/Mini-MIDAS','','','Aeron',29,'',75 from dual union all
select 67131,'2000-W14','2000-05-25 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 67132,'2000-W15','2000-05-31 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 67133,'2000-W16','2000-06-21 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',56 from dual union all
select 67134,'2000-W17','2000-06-29 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 67135,'2000-W18','2000-07-06 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 67136,'2000-W19','2000-07-19 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 67137,'2000-W20','2000-08-03 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',50 from dual union all
select 67138,'2000-W21','2000-08-23 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',53 from dual union all
select 67139,'2000-W22','2000-09-07 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',53 from dual union all
select 6714,'1960-M23','1960-08-13 10:35:00','suborbital rocket','success',545,'AA5.423','Firefly MARGIE','','','Aeronomy',394,'',75 from dual union all
select 67140,'2000-W23','2000-09-18 02:00:00','suborbital rocket','success',759,'MT-135-72','Ozone','','','Aeron',839,'',55 from dual union all
select 67141,'2000-W24','2000-09-19 02:00:00','suborbital rocket','success',759,'MT-135-73','Ozone','','','Aeron',839,'',53 from dual union all
select 67142,'2000-W25','2000-09-20 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',54 from dual union all
select 67143,'2000-W26','2000-10-04 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',55 from dual union all
select 67144,'2000-W27','2000-10-19 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',51 from dual union all
select 67145,'2000-W28','2000-11-01 02:15:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',53 from dual union all
select 67146,'2000-W29','2000-11-15 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',52 from dual union all
select 67147,'2000-W30','2000-11-22 02:15:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',52 from dual union all
select 67148,'2000-W31','2000-12-07 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',53 from dual;
insert into LAUNCH
select 67149,'2000-W32','2000-12-12 20:00:00','suborbital rocket','success',1284,'LiteStar','','','','Test',239,'',75 from dual union all
select 6715,'1960-M24','1960-09-15 00:00:00','military missile','success',1162,'34','','','','OT?',1895,'',50 from dual union all
select 67150,'2000-W33','2000-12-13 16:00:00','suborbital rocket','success',1284,'LiteStar','','','','Test',239,'',75 from dual union all
select 67151,'2000-W34','2000-12-20 02:00:00','suborbital rocket','success',761,'','','','','Meteo',1332,'',52 from dual union all
select 67152,'2000-Y01','2000-02-27 00:14:59','suborbital rocket','success',1061,'','Datasonde','','','Meteo',176,'',99 from dual union all
select 67153,'2000-Y02','2000-06-15 15:00:00','suborbital rocket','success',1061,'','Datasonde','','','Meteo',295,'',99 from dual union all
select 67154,'2001-001','2001-01-09 17:00:03','orbital','success',306,'Y2       (65)','Shenzhou 2','Shenzhou 2 Fanhui Cang','','Sat',814,'',null from dual union all
select 67155,'2001-002','2001-01-10 22:09:00','orbital','success',110,'V137 L4101','Turksat 2A','Eurasiasat 1','','Sat',345,'',null from dual union all
select 67156,'2001-003','2001-01-24 04:28:42','orbital','success',1241,'K 15000-673 673','Progress M1-5','Progress M1 No. 254','','Sat',480,'',null from dual union all
select 67157,'2001-004','2001-01-30 07:55:01','orbital','success',427,'D283','USA 156 (GPS IIR-7)','Navstar GPS SVN 54','','Sat',285,'',null from dual union all
select 67158,'2001-005','2001-02-07 23:05:10','orbital','success',105,'V139 L4103','Sicral/Skynet 4F','SICRAL 1A','','Sat',345,'',null from dual union all
select 67159,'2001-006','2001-02-07 23:13:02','orbital','success',1243,'STS-98','ISS 5A','OV-104 ISS','','Sat',921,'MLP2',null from dual union all
select 6716,'1960-M25','1960-09-21 00:00:00','military missile','success',1162,'37','','','','OT?',1895,'',50 from dual union all
select 67160,'2001-007','2001-02-20 08:48:27','orbital','success',1260,'168','Odin','Odin','','Sat',478,'',null from dual union all
select 67161,'2001-008','2001-02-26 08:09:35','orbital','success',1241,'K 15000-670 670','Progress M-44','Progress 7K-TGM No. 244','','Sat ISS 3P',480,'',null from dual union all
select 67162,'2001-009','2001-02-27 21:20:00','orbital','success',1410,'4B-41/TC-22 (K-30)','USA 157 (Milstar 4)','Milstar DFS-4','','Sat',291,'',null from dual union all
select 67163,'2001-010','2001-03-08 11:42:09','orbital','success',1243,'STS-102','ISS 5A.1','OV-103 ISS','','Sat',922,'MLP3',null from dual union all
select 67164,'2001-011','2001-03-08 22:51:00','orbital','success',114,'V140 (509)','Eurobird/BSAT-2a','Eurobird','','Sat',346,'',null from dual union all
select 67165,'2001-012','2001-03-18 22:33:30','orbital','success',1542,'7/DM-SL-5L','SL7 XM Radio-2','XM Radio-2','','Sat',860,'ODYSSEY',null from dual union all
select 67166,'2001-013','2001-04-07 15:02:22','deep space','success',427,'D284','2001 Mars Odyssey','2001 Mars Odyssey','','Sat',285,'',null from dual union all
select 67167,'2001-014','2001-04-07 03:47:00','orbital','success',962,'535-01/88503','Ekran-M No. 18','Ekran-M No. 18L','','Sat',588,'',null from dual union all
select 67168,'2001-015','2001-04-18 10:13:00','orbital','success',501,'GSLV-D1','GSAT-1','GSAT-1','','Sat',1357,'',null from dual union all
select 67169,'2001-016','2001-04-19 18:40:42','orbital','success',1243,'STS-100','ISS 6A','OV-105 ISS','','Sat',921,'MLP1',null from dual union all
select 6717,'1960-M26','1960-09-23 00:00:00','military missile','success',1162,'39','','','','OT?',1895,'',50 from dual union all
select 67170,'2001-017','2001-04-28 07:37:20','orbital','success',1241,'K 15000-674 674','Soyuz TM-32','Soyuz 7K-STM No. 206','','Sat ISS 2S',480,'',null from dual union all
select 67171,'2001-018','2001-05-08 22:10:29','orbital','success',1542,'9/DM-SL-7L','SL6 XM Radio-1','XM Radio-1','','Sat',860,'ODYSSEY',null from dual union all
select 67172,'2001-019','2001-05-15 01:11:30','orbital','success',960,'403-01','PAS 10','PAS 10','','Sat',587,'',null from dual union all
select 67173,'2001-020','2001-05-18 17:45:01','orbital','success',427,'D285','USA 158 (GeoLITE)','GeoLITE','','Sat',286,'',null from dual union all
select 67174,'2001-021','2001-05-20 22:32:40','orbital','success',1233,'K 15000-001 001','Progress M1-6','Progress M1 No. 255','','Sat ISS 4P',480,'',null from dual union all
select 67175,'2001-022','2001-05-29 17:55:00','orbital','success',1241,'77023  -841 076','Kosmos-2377','Kobal''t','','Sat',475,'',null from dual union all
select 67176,'2001-023','2001-06-08 15:08:42','orbital','success',680,'53706-412','Kosmos-2378','Parus','','Sat',456,'',null from dual union all
select 67177,'2001-024','2001-06-09 06:45:00','orbital','success',105,'V141 L4104','Intelsat 901','IS-901','','Sat',345,'',null from dual union all
select 67178,'2001-025','2001-06-16 01:49:00','orbital','success',960,'403-02','Astra 2C','Astra 2C','','Sat',587,'',null from dual union all
select 67179,'2001-026','2001-06-19 04:41:02','orbital','success',174,'AC-156','ICO 2','ICO 2','','Sat',289,'',null from dual union all
select 6718,'1960-M27','1960-09-15 18:09:00','suborbital rocket','success',29,'NASA 01.03GP','AMPP','','','Remote sensing',428,'',76 from dual union all
select 67180,'2001-027','2001-06-30 19:46:46','deep space','success',414,'D286','MAP','Microwave Anisotropy Pro','','Sat',286,'',null from dual union all
select 67181,'2001-028','2001-07-12 09:03:59','orbital','success',1243,'STS-104','ISS 7A','OV-104 ISS','','Sat',922,'',null from dual union all
select 67182,'2001-029','2001-07-12 21:58:00','orbital','failure',114,'V142 (510)','Artemis/BSAT-2b','Artemis','','Sat',346,'',null from dual union all
select 67183,'2001-030','2001-07-20 00:17:00','orbital','success',805,'77045  -686 043','Molniya-3','Molniya-3K No. 11?','','Sat',475,'',null from dual union all
select 67184,'2001-031','2001-07-23 07:23:01','orbital','success',173,'AC-142','GOES M','GOES M','','Sat',288,'',null from dual union all
select 67185,'2001-032','2001-07-31 08:00:00','orbital','success',1448,'510-25-  802','Koronas-F','AUOS-SM-KF','','Sat',470,'',null from dual union all
select 67186,'2001-033','2001-08-06 07:28:00','orbital','success',1412,'4B-31','USA 159      [DSP21]','DSP','','Sat',291,'',null from dual union all
select 67187,'2001-034','2001-08-08 16:13:40','deep space','success',412,'D287','Genesis','Genesis','','Sat',285,'',null from dual union all
select 67188,'2001-035','2001-08-10 21:10:14','orbital','success',1243,'STS-105','ISS 7A.1','OV-103 ISS','','Sat',921,'',null from dual union all
select 67189,'2001-036','2001-08-21 09:23:54','orbital','success',1241,'M 15000-065 065','Progress M-45','Progress M No. 245','','Sat ISS 5P',480,'',null from dual union all
select 6719,'1960-M28','1960-09-24 18:13:00','suborbital rocket','success',29,'NASA 01.05GP','AMPP','','','Remote sensing',428,'',76 from dual union all
select 67190,'2001-037','2001-08-24 20:34:50','orbital','success',958,'404-01','Kosmos-2379','Oko-1 No. 7124','','Sat',588,'',null from dual union all
select 67191,'2001-038','2001-08-29 07:00:00','orbital','success',528,'H-IIA-1F','VEP-2/LRE','LRE','','Sat',1477,'',null from dual union all
select 67192,'2001-039','2001-08-30 06:46:00','orbital','success',105,'V143 L4105','Intelsat 902','INTELSAT 902','','Sat',345,'',null from dual union all
select 67193,'2001-040','2001-09-08 15:25:05','orbital','success',174,'AC-160 MLV-10','USA 160     [NOSS14]','INTRUDER 5?','','Sat',1775,'',null from dual union all
select 67194,'2001-041','2001-09-14 23:34:55','orbital','success',1241,'F 15000-677 677','Pirs','Progress No. 301','','Sat',480,'',null from dual union all
select 67195,'2001-042','2001-09-25 23:21:00','orbital','success',110,'V144 L4106','Atlantic Bird 2','Atlantic Bird 2','','Sat',345,'',null from dual union all
select 67196,'2001-043','2001-09-30 02:40:00','orbital','success',137,'LM-001','Kodiak Star','Starshine 3','','Sat',861,'',null from dual union all
select 67197,'2001-044','2001-10-05 21:21:01','orbital','success',1416,'4B-34','USA 161      [IC7]','Improved CRYSTAL 2107?','','Sat',1777,'',null from dual union all
select 67198,'2001-045','2001-10-06 16:45:00','orbital','success',958,'405-01','Raduga-1','Globus No. 14L','','Sat',588,'',null from dual union all
select 67199,'2001-046','2001-10-11 02:32:00','orbital','success',174,'AC-162 MLV-12','USA 162      [SDS14]','QUASAR 14?','','Sat',289,'',null from dual union all
select 672,'1947-A08','1947-04-24 00:00:00','atmospheric rocket','success',565,'','RM-10 Model','','','Test',1821,'',5 from dual union all
select 6720,'1960-M29','1960-09-29 02:46:00','suborbital rocket','success',663,'K-6H-TW-9','K-6H-1','Grenade','','Aeron',7,'',70 from dual union all
select 67200,'2001-047','2001-10-18 18:51:26','orbital','success',410,'D288','QuickBird 2','QuickBird','','Sat',1724,'',null from dual union all
select 67201,'2001-048','2001-10-21 08:59:35','orbital','success',1241,'F 15000-672 672','Soyuz TM-33','Soyuz TM No. 207','','Sat ISS 3S',480,'',null from dual union all
select 67202,'2001-049','2001-10-22 04:53:00','orbital','success',899,'PSLV-C3','TES','TES','','Sat',1357,'',null from dual union all
select 67203,'2001-050','2001-10-25 11:34:13','orbital','success',805,'77013  -687 044','Molniya-3','Molniya-3 No. 64','','Sat',474,'',null from dual union all
select 67204,'2001-051','2001-11-26 18:24:12','orbital','success',1233,'F 15000-002 002','Progress M1-7','Progress M1 No. 256','','Sat ISS 6P',480,'',null from dual union all
select 67205,'2001-052','2001-11-27 00:35:00','orbital','success',108,'V146 L4107','DirecTV-4S','DirecTV-4S','','Sat',345,'',null from dual union all
select 67206,'2001-053','2001-12-01 18:04:44','orbital','success',958,'405-02','Kosmos-2380/2382','Uragan-M No. 711','','Sat',588,'',null from dual union all
select 67207,'2001-054','2001-12-05 22:19:28','orbital','success',1243,'STS-108','ISS UF-1','OV-105 ISS','','Sat',922,'MLP1',null from dual union all
select 67208,'2001-055','2001-12-07 15:07:35','orbital','success',417,'D289','Jason/TIMED','Jason','','Sat',1724,'',null from dual union all
select 67209,'2001-056','2001-12-10 17:18:57','orbital','success',1537,'45021-101/19L','Meteor-3M','Meteor-3M 17F45 No. 101','','Sat',567,'',null from dual union all
select 6721,'1960-M30','1960-09-30 00:00:00','suborbital rocket','success',868,'','','','','Aeron?',1016,'',80 from dual union all
select 67210,'2001-057','2001-12-21 04:00:00','orbital','success',1446,'510-82-  802','Kosmos-2383','US-PU','','Sat',591,'',null from dual union all
select 67211,'2001-058','2001-12-28 03:24:24','orbital','success',1448,'670-21-  803','Gonets/Kosmos','Strela-3','','Sat',469,'',null from dual union all
select 67212,'2001-A01','2001-06-02 20:43:31','atmospheric rocket','failure',590,'1','X-43A','','','Test',379,'NB-52 008',7 from dual union all
select 67213,'2001-A02','2001-10-24 16:00:00','suborbital rocket','failure',1221,'2','TMA','','','Aeron',808,'',2 from dual union all
select 67214,'2001-A03','2001-11-09 18:12:00','military missile','failure',1111,'M-4 WCRRF','BMDO Radar Target','','','Target',862,'',10 from dual union all
select 67215,'2001-A04','2001-12-13 18:15:00','military missile','failure',498,'BV-3','GMDS BVT-3','','','Test',1707,'',5 from dual union all
select 67216,'2001-A05','2001-04-27 10:35:05','suborbital rocket','success',891,'NASA 30.046UO','','','','Education',1821,'',47 from dual union all
select 67217,'2001-A06','2001-08-27 00:00:00','military missile','success',244,'','','','','Test',1026,'F-15D',40 from dual union all
select 67218,'2001-A07','2001-01-21 11:25:00','military missile','failure',75,'','Test','','','Test',760,'',0 from dual union all
select 67219,'2001-A08','2001-02-12 09:07:00','military missile','success',75,'','Test','','','Test',760,'',20 from dual union all
select 6722,'1960-M31','1960-12-01 00:00:00','military missile','success',1162,'40','','','','OT?',1895,'',50 from dual union all
select 67220,'2001-A09','2001-03-28 00:00:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67221,'2001-A10','2001-03-29 00:00:00','military missile','success',75,'','Test','','','Test',760,'',20 from dual union all
select 67222,'2001-A11','2001-05-28 00:00:00','military missile','failure',75,'','Test','','','Test',760,'',0 from dual union all
select 67223,'2001-A12','2001-07-12 03:40:00','military missile','failure',75,'','Test','','','Test',760,'',0 from dual union all
select 67224,'2001-A13','2001-08-20 03:59:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67225,'2001-A14','2001-08-21 00:00:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67226,'2001-A15','2001-08-21 00:00:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67227,'2001-A16','2001-09-24 00:00:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67228,'2001-A17','2001-09-26 13:32:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67229,'2001-A18','2001-12-29 00:00:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 6723,'1960-M32','1960-12-12 00:00:00','military missile','success',1162,'42','','','','OT?',1895,'',50 from dual union all
select 67230,'2001-A19','2001-04-29 00:00:00','military missile','failure',71,'F13','Test','','','Test',761,'',20 from dual union all
select 67231,'2001-A20','2001-05-30 00:00:00','military missile','failure',71,'F14','Test','','','Test',761,'',5 from dual union all
select 67232,'2001-A21','2001-08-08 00:00:00','military missile','failure',71,'F18','Test','','','Test',761,'',2 from dual union all
select 67233,'2001-A22','2001-08-22 00:00:00','military missile','failure',71,'F19','Test','','','Test',761,'',2 from dual union all
select 67234,'2001-A23','2001-11-03 00:00:00','military missile','success',71,'F21','Test','','','Test',761,'',60 from dual union all
select 67235,'2001-A24','2001-08-23 00:00:00','military missile','success',76,'','Test 1','','','Test',745,'',45 from dual union all
select 67236,'2001-A25','2001-10-11 00:00:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67237,'2001-A26','2001-11-07 06:30:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67238,'2001-M01','2001-01-01 00:00:00','suborbital rocket','success',1023,'','Meteo','','','Aeron',320,'',70 from dual;
insert into LAUNCH
select 67239,'2001-M02','2001-01-01 00:00:00','suborbital rocket','success',1023,'','Meteo','','','Aeron',320,'',70 from dual union all
select 6724,'1960-M33','1960-12-13 00:00:00','military missile','success',1162,'41','','','','OT?',1895,'',50 from dual union all
select 67240,'2001-M03','2001-01-01 00:00:00','suborbital rocket','success',1023,'','Meteo','','','Aeron',320,'',70 from dual union all
select 67241,'2001-M04','2001-09-21 00:00:00','military missile','success',430,'','Test','','','Test',320,'INS-P51',50 from dual union all
select 67242,'2001-M05','2001-01-09 00:00:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67243,'2001-M06','2001-01-13 08:02:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67244,'2001-M07','2001-01-14 10:39:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67245,'2001-M08','2001-02-16 08:46:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67246,'2001-M09','2001-03-20 08:42:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67247,'2001-M10','2001-05-29 00:00:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67248,'2001-M11','2001-07-11 03:14:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 67249,'2001-M12','2001-07-17 00:00:00','military missile','success',75,'','Test','','','Test',760,'',50 from dual union all
select 6725,'1960-M34','1960-10-18 00:00:00','suborbital rocket','success',88,'459 A275.AA','','','','Test',1895,'',77 from dual union all
select 67250,'2001-M16','2001-03-17 00:00:00','military missile','success',71,'F11','Test','','','Test',761,'',60 from dual union all
select 67251,'2001-M17','2001-03-27 00:00:00','military missile','success',71,'F12','Test','','','Test',761,'',60 from dual union all
select 67252,'2001-M18','2001-08-08 00:00:00','military missile','success',71,'F17','Test','','','Test',761,'',60 from dual union all
select 67253,'2001-M19','2001-09-06 00:00:00','military missile','success',71,'F20','Test','','','Test',761,'',60 from dual union all
select 67254,'2001-M20','2001-11-22 00:00:00','military missile','success',71,'F23','Test','','','Test',761,'',60 from dual union all
select 67255,'2001-M21','2001-12-05 00:00:00','military missile','success',71,'F22','Test','','','Test',761,'',60 from dual union all
select 67256,'2001-M22','2001-12-15 00:00:00','military missile','success',71,'F24','Test','','','Test',761,'',60 from dual union all
select 67257,'2001-M23','2001-12-18 00:00:00','military missile','success',71,'F25','Test','','','Test',761,'',60 from dual union all
select 67258,'2001-M24','2001-12-20 00:00:00','military missile','success',71,'F26','Test','','','Test',761,'',60 from dual union all
select 67259,'2001-M25','2001-05-31 00:00:00','military missile','success',489,'','R&D','','','Test',759,'',60 from dual union all
select 6726,'1960-M35','1960-10-20 00:00:00','suborbital rocket','success',88,'461 A277.4AA','','','','Test',1895,'',62 from dual union all
select 67260,'2001-S01','2001-01-01 00:00:00','suborbital rocket','success',1024,'RH-200SV-1','','','','Test',1361,'',110 from dual union all
select 67261,'2001-S02','2001-01-01 00:00:00','suborbital rocket','success',1024,'RH-200SV-2','','','','Test',1361,'',110 from dual union all
select 67262,'2001-S03','2001-01-17 04:31:00','military missile','success',63,'','Agni RV Mk 2','','','Test',317,'',1000 from dual union all
select 67263,'2001-S04','2001-01-26 03:57:00','military missile','success',732,'Aegis FTR-1A Target','Aegis TTV-2','','','Target',855,'',300 from dual union all
select 67264,'2001-S05','2001-01-26 04:03:00','military missile','success',1090,'Aegis FTR-1A','Stellar Gemini','','','Test',851,'CG-70',100 from dual union all
select 67265,'2001-S06','2001-02-07 09:28:00','military missile','success',799,'','Mk 12/12A RV x3','FDEP GT175GM','GLORY TRIP 175GM','Op Test',1706,'',1300 from dual union all
select 67266,'2001-S07','2001-02-09 04:00:06','suborbital rocket','success',213,'NASA 36.198UG','JHU FOT 16','','','UV Astron',1895,'',300 from dual union all
select 67267,'2001-S08','2001-02-12 16:28:00','suborbital rocket','success',224,'NASA 36.187NM','','','','Micrograv',1895,'',300 from dual union all
select 67268,'2001-S09','2001-02-16 10:28:00','military missile','success',1179,'VMF RF','','','','OT',201,'PLBR',1000 from dual union all
select 67269,'2001-S10','2001-02-16 10:43:00','military missile','success',1434,'64536-224-34','15Zh58 UBP-25','','','OT',608,'',1000 from dual union all
select 6727,'1960-M36','1960-11-03 00:00:00','suborbital rocket','success',88,'','','','','Weather',855,'',65 from dual union all
select 67270,'2001-S11','2001-02-19 06:02:00','suborbital rocket','success',605,'DLR K-O-','DLR Test Maxus-4','','','Test',417,'',81 from dual union all
select 67271,'2001-S12','2001-02-20 18:58:00','military missile','success',1097,'','TCMP-3B','','','Sensors',1784,'',300 from dual union all
select 67272,'2001-S13','2001-02-22 04:55:00','suborbital rocket','success',224,'NASA 36.195DG','J-PEX 2','','','UV Astron',1895,'',300 from dual union all
select 67273,'2001-S14','2001-03-22 22:15:00','military missile','success',732,'QRLV-1','','','','Target',862,'',163 from dual union all
select 67274,'2001-S15','2001-03-31 00:00:00','military missile','success',570,'19/MBRV-3','MBRV-3/PAC-3 DT-8a','','','Target',436,'',114 from dual union all
select 67275,'2001-S16','2001-03-31 06:00:00','military missile','success',941,'18','Solid fuel test','','','Test',320,'',100 from dual union all
select 67276,'2001-S17','2001-04-17 22:00:00','military missile','success',757,'4','M51 R&D (to CSG)','','','Test',206,'S615',800 from dual union all
select 67277,'2001-S19','2001-04-29 11:28:00','suborbital rocket','success',772,'MAXUS 4','SSC MAXUS 4','','','Micrograv',407,'',705 from dual union all
select 67278,'2001-S20','2001-05-08 09:55:00','suborbital rocket','success',1212,'TEXUS 39','DLR TEXUS 39','','','Micrograv',416,'',248 from dual union all
select 67279,'2001-S21','2001-05-14 00:00:00','military missile','success',1443,'FCET-24','-    (91ss)','','','OT',421,'SSBN 737',1000 from dual union all
select 6728,'1960-M37','1960-11-01 00:00:00','suborbital rocket','success',88,'','Aerialrecovery','','','Weather',855,'',65 from dual union all
select 67280,'2001-S22','2001-05-15 01:57:00','suborbital rocket','success',605,'HiN-2','Hotel Dart test','','','Test',29,'',115 from dual union all
select 67281,'2001-S23','2001-06-05 11:32:00','military missile','success',1179,'VMF RF','','','','OT',201,'PLBR',1000 from dual union all
select 67282,'2001-S24','2001-06-06 10:50:00','suborbital rocket','success',891,'NASA 30.048NO','','','','Education',1821,'',85 from dual union all
select 67283,'2001-S25','2001-06-17 00:05:00','suborbital rocket','success',605,'SO-MI-05','MIDAS','','','Aeron/Ionos',29,'',85 from dual union all
select 67284,'2001-S26','2001-06-21 17:32:00','suborbital rocket','success',224,'NASA 36.199US','TXI','','','Solar X',1873,'',272 from dual union all
select 67285,'2001-S27','2001-06-24 21:21:15','suborbital rocket','success',605,'SO-MI-11','MIDAS','','','Aeron/Ionos',29,'',85 from dual union all
select 67286,'2001-S28','2001-06-25 23:16:00','military missile','success',1443,'FCET-25','','','','OT',420,'SSBN 743',1000 from dual union all
select 67287,'2001-S29','2001-06-25 23:16:00','military missile','success',1443,'FCET-25','','','','OT',420,'SSBN 743',1000 from dual union all
select 67288,'2001-S30','2001-06-26 01:13:00','military missile','success',1443,'FCET-25','','','','OT',420,'SSBN 743',1000 from dual union all
select 67289,'2001-S31','2001-06-27 00:00:00','military missile','success',627,'','','','','OT',1026,'',100 from dual union all
select 6729,'1960-M38','1960-10-01 00:00:00','suborbital rocket','success',86,'','HIGHBALL IA test','','','Test',1895,'',60 from dual union all
select 67290,'2001-S32','2001-06-27 04:35:00','military missile','success',1454,'63078-23-114','15A35P LKI-5','15Yu70/102E','','OT',508,'',1000 from dual union all
select 67291,'2001-S33','2001-06-29 04:44:01','suborbital rocket','success',228,'NASA 21.125GE','','','','Plasma',1821,'',250 from dual union all
select 67292,'2001-S34','2001-07-09 00:00:00','military missile','success',570,'20/MBRV-1E','MBRV-1E/PAC DT/OT-9a','','','Target',436,'',145 from dual union all
select 67293,'2001-S35','2001-07-15 02:40:00','military missile','success',796,'MSLS IFT-6','MSLS IFT-6','GMDS','','Target',1698,'',1600 from dual union all
select 67294,'2001-S36','2001-07-15 03:01:00','military missile','success',797,'PLV IFT-6','IFT-6 EKV','','','ABM Test',868,'',225 from dual union all
select 67295,'2001-S37','2001-07-20 00:31:00','suborbital rocket','success',1507,'','Cosmos 1 Solar Sail','','','SC Test',201,'K-496',412 from dual union all
select 67296,'2001-S38','2001-07-27 08:01:00','military missile','failure',907,'30PA (85-0162)','Mk 21 RV x 9','FDEP GT 30PA','','OT',1697,'',100 from dual union all
select 67297,'2001-S39','2001-08-27 10:50:00','military missile','success',119,'','AST-6 Black Sparrow','','','ATBM Test',1026,'',100 from dual union all
select 67298,'2001-S40','2001-08-31 20:00:00','military missile','success',498,'BV-2','GMDS BVT-2','','','Test',1707,'',200 from dual union all
select 67299,'2001-S41','2001-09-18 00:00:00','military missile','success',1507,'VMF RF','','','','OT',1011,'K-223',1000 from dual union all
select 673,'1947-A09','1947-04-28 00:00:00','atmospheric rocket','success',443,'','RM-5 Model','','','Test',1821,'',5 from dual union all
select 6730,'1960-M39','1960-10-01 00:00:00','suborbital rocket','success',86,'','HIGHBALL IB test','','','Test',1895,'',60 from dual union all
select 67300,'2001-S42','2001-09-27 00:00:00','military missile','success',1179,'VMF RF','','','','OT',201,'TK-20',1000 from dual union all
select 67301,'2001-S43','2001-09-27 00:00:00','military missile','success',1179,'VMF RF','','','','OT',201,'TK-20',1000 from dual union all
select 67302,'2001-S44','2001-09-27 16:10:00','suborbital rocket','success',1336,'NASA 42.01DP','','','','Test',855,'',300 from dual union all
select 67303,'2001-S45','2001-10-03 09:00:00','military missile','success',1434,'','15Zh58 UBP-26','','','OT',608,'',1000 from dual union all
select 67304,'2001-S46','2001-10-18 08:00:00','military missile','success',1179,'VMF RF','','','','OT',190,'PLBR',1000 from dual union all
select 67305,'2001-S47','2001-10-26 12:33:00','military missile','success',1453,'-       -119','15A35 UBP-50','GCh 15F355P7','','OT',535,'',1000 from dual union all
select 67306,'2001-S48','2001-10-30 02:31:00','suborbital rocket','failure',1341,'T-Orion 5A','Hyshot 1 ''Hilary''','','','Tech',1849,'',100 from dual union all
select 67307,'2001-S49','2001-11-01 15:20:00','military missile','success',1434,'','15Zh58 UBP-27','MRV Test','','OT',608,'',1000 from dual union all
select 67308,'2001-S50','2001-11-07 08:15:00','military missile','success',799,'','Mk 12A RV x 3','FDEP GT176GM','GLORY TRIP 176GM','Op Test',1699,'',1300 from dual union all
select 67309,'2001-S51','2001-12-04 02:59:00','military missile','success',784,'TLV-1','MSLS IFT-7','GMDS IFT-7','','Target',1701,'',1600 from dual union all
select 6731,'1960-M40','1960-10-01 00:00:00','suborbital rocket','success',86,'','HIGHBALL IC test','','','Test',1895,'',60 from dual union all
select 67310,'2001-S52','2001-12-04 03:19:00','military missile','success',797,'PLV IFT-7','IFT-7 EKV','','','ABM Test',868,'',225 from dual union all
select 67311,'2001-S53','2001-12-06 03:20:00','suborbital rocket','success',224,'NASA 36.188UL','WISP 5 comet study','','','UV Astron',1873,'',300 from dual union all
select 67312,'2001-S54','2001-12-09 20:11:00','military missile','success',1442,'FCET-52','OT-III (218)','','','OT',420,'SSBN 726',1000 from dual union all
select 67313,'2001-S55','2001-12-09 20:11:00','military missile','success',1442,'FCET-52','OT-III','','','OT',420,'SSBN 726',1000 from dual union all
select 67314,'2001-S56','2001-12-09 21:36:00','military missile','success',1442,'FCET-52','OT-III','','','OT',420,'SSBN 726',1000 from dual union all
select 67315,'2001-S57','2001-12-09 21:36:00','military missile','success',1442,'FCET-52','OT-III (221)','','','OT',420,'SSBN 726',1000 from dual union all
select 67316,'2001-S58','2001-12-12 05:11:00','military missile','success',941,'','Prithvi-2','','','Test',320,'',100 from dual union all
select 67317,'2001-S59','2001-12-16 05:12:00','suborbital rocket','success',605,'SSC Hygrosond-2','SSC Hygrosond-2','','','Aeron',417,'',90 from dual union all
select 67318,'2001-S60','2001-12-18 20:11:00','military missile','success',1442,'FCET-53','OT-III (223)','','','OT',420,'SSBN 726',1000 from dual union all
select 67319,'2001-S61','2001-12-18 20:11:00','military missile','failure',1442,'FCET-53','OT-III (224)','','','OT',420,'SSBN 726',1000 from dual union all
select 6732,'1960-M41','1960-11-28 00:00:00','suborbital rocket','success',88,'','Robin','','','Aeron',394,'',60 from dual union all
select 67320,'2001-S62','2001-12-18 22:04:00','military missile','success',1442,'FCET-53','OT-III (225)','','','OT',420,'SSBN 726',1000 from dual union all
select 67321,'2001-S63','2001-12-18 22:04:00','military missile','success',1442,'FCET-53','OT-III','','','OT',420,'SSBN 726',1000 from dual union all
select 67322,'2001-S64','2001-07-01 00:00:00','military missile','success',1152,'','','','','Test',960,'',100 from dual union all
select 67323,'2001-S65','2001-08-21 00:00:00','military missile','success',356,'','','','','Opt Test',819,'',500 from dual union all
select 67324,'2001-S66','2001-04-18 00:00:00','military missile','success',1166,'','vs MKO camps','','','Weapon',857,'',100 from dual union all
select 67325,'2001-S67','2001-04-18 00:00:00','military missile','success',1166,'','vs MKO camps','','','Weapon',857,'',100 from dual union all
select 67326,'2001-S68','2001-04-18 00:00:00','military missile','success',1166,'[many launches]','vs MKO camps','','','Weapon',857,'',100 from dual union all
select 67327,'2001-U01','2001-09-21 18:49:30','orbital','failure',1314,'T6','Orbview-4/QuikTOMS','Warfighter-1','','Sat',1670,'',427 from dual union all
select 67328,'2001-W01','2001-03-21 00:00:00','suborbital rocket','success',759,'MT-135 R1119','Last flight','','','Aeron',839,'',55 from dual;
insert into LAUNCH
select 67329,'2001-W02','2001-05-16 13:41:00','suborbital rocket','success',1502,'NASA 12.049UT','','','','Test',1821,'',75 from dual union all
select 6733,'1960-S01','1960-01-01 00:00:00','suborbital rocket','success',26,'','','','','Aeron',1837,'',81 from dual union all
select 67330,'2001-W03','2001-06-08 14:32:00','suborbital rocket','success',1502,'SO-FS-01','MIDAS','','','Aeron',29,'',75 from dual union all
select 67331,'2001-W04','2001-06-12 01:10:30','suborbital rocket','success',1502,'SO-FS-02','MIDAS','','','Aeron',29,'',75 from dual union all
select 67332,'2001-W05','2001-06-14 01:34:30','suborbital rocket','success',1502,'SO-FS-03','MIDAS','','','Aeron',29,'',75 from dual union all
select 67333,'2001-W06','2001-06-16 23:51:00','suborbital rocket','success',1502,'SO-FS-04','MIDAS','','','Aeron',29,'',75 from dual union all
select 67334,'2001-W07','2001-06-17 00:36:00','suborbital rocket','success',1502,'SO-CH-06','MIDAS','','','Aeron',29,'',75 from dual union all
select 67335,'2001-W08','2001-06-17 00:59:00','suborbital rocket','success',1502,'SO-FS-07','MIDAS','','','Aeron',29,'',75 from dual union all
select 67336,'2001-W09','2001-06-17 01:19:00','suborbital rocket','success',1502,'SO-CH-08','MIDAS','','','Aeron',29,'',75 from dual union all
select 67337,'2001-W10','2001-06-21 01:44:00','suborbital rocket','success',1502,'SO-CH-09','MIDAS','','','Aeron',29,'',75 from dual union all
select 67338,'2001-W11','2001-06-24 21:04:00','suborbital rocket','success',1502,'SO-FS-10','MIDAS','','','Aeron',29,'',75 from dual union all
select 67339,'2001-W12','2001-06-24 21:51:00','suborbital rocket','success',1502,'SO-CH-12','MIDAS','','','Aeron',29,'',75 from dual union all
select 6734,'1960-S02','1960-01-01 00:00:00','suborbital rocket','success',26,'','','','','Aeron',1837,'',81 from dual union all
select 67340,'2001-W13','2001-06-24 22:20:10','suborbital rocket','success',1502,'SO-FS-13','MIDAS','','','Aeron',29,'',75 from dual union all
select 67341,'2001-W14','2001-06-24 22:45:15','suborbital rocket','success',1502,'SO-CH-14','MIDAS','','','Aeron',29,'',75 from dual union all
select 67342,'2001-W15','2001-12-15 10:20:30','suborbital rocket','success',1284,'Wind','Hygrosonde II Met','','','Aeron',409,'',75 from dual union all
select 67343,'2001-W16','2001-12-16 03:11:30','suborbital rocket','success',1284,'','Hygrosonde II Met','','','Aeron',409,'',75 from dual union all
select 67344,'2001-W17','2001-12-16 05:43:00','suborbital rocket','success',1284,'Falling Sphere','Hygrosonde II Met','','','Aeron',409,'',75 from dual union all
select 67345,'2001-W18','2001-12-16 06:18:00','suborbital rocket','success',1284,'Falling Sphere','Hygrosonde II Met','','','Aeron',409,'',75 from dual union all
select 67346,'2001-Y01','2001-07-16 11:25:00','suborbital rocket','success',1502,'ROFS-01','ROMA','','','Aeron',1452,'',108 from dual union all
select 67347,'2001-Y02','2001-07-19 11:17:20','suborbital rocket','success',1502,'ROFS-02','ROMA','','','Aeron',1452,'',105 from dual union all
select 67348,'2001-Y03','2001-07-22 12:20:00','suborbital rocket','success',1502,'ROFS-03','ROMA','','','Aeron',1452,'',100 from dual union all
select 67349,'2001-Y04','2001-07-22 12:55:00','suborbital rocket','success',1502,'ROCH-04','ROMA','','','Aeron',1452,'',100 from dual union all
select 6735,'1960-S03','1960-01-01 00:00:00','suborbital rocket','success',856,'','Teflon nose','','','Reentry Test',1821,'',120 from dual union all
select 67350,'2001-Y05','2001-07-25 10:00:00','suborbital rocket','success',1502,'ROFS-05','ROMA','','','Aeron',1452,'',100 from dual union all
select 67351,'2001-Y06','2001-07-28 10:07:00','suborbital rocket','success',1502,'ROFS-06','ROMA','','','Aeron',1452,'',100 from dual union all
select 67352,'2001-Y07','2001-07-31 09:00:00','suborbital rocket','success',1502,'ROFS-07','ROMA','','','Aeron',1452,'',100 from dual union all
select 67353,'2001-Y08','2001-07-31 09:50:00','suborbital rocket','success',1502,'ROCH-08','ROMA','','','Aeron',1452,'',100 from dual union all
select 67354,'2001-Y09','2001-08-02 18:00:00','suborbital rocket','success',1502,'ROFS-09','ROMA','','','Aeron',1452,'',100 from dual union all
select 67355,'2001-Y10','2001-08-06 09:38:00','suborbital rocket','success',1502,'ROFS-10','ROMA','','','Aeron',1452,'',100 from dual union all
select 67356,'2001-Y11','2001-08-09 10:13:00','suborbital rocket','success',1502,'ROFS-11','ROMA','','','Aeron',1452,'',100 from dual union all
select 67357,'2001-Y12','2001-08-12 10:16:00','suborbital rocket','success',1502,'ROFS-12','ROMA','','','Aeron',1452,'',100 from dual union all
select 67358,'2001-Y13','2001-08-17 11:36:00','suborbital rocket','success',1502,'ROFS-13','ROMA','','','Aeron',1452,'',100 from dual union all
select 67359,'2001-Y14','2001-08-17 12:19:00','suborbital rocket','success',1502,'ROCH-14','ROMA','','','Aeron',1452,'',100 from dual union all
select 6736,'1960-S04','1960-01-01 00:00:00','suborbital rocket','success',839,'','','','','Aeron?',1895,'',100 from dual union all
select 67360,'2001-Y15','2001-08-20 10:25:00','suborbital rocket','success',1502,'ROFS-15','ROMA','','','Aeron',1452,'',100 from dual union all
select 67361,'2001-Y16','2001-08-20 19:48:00','suborbital rocket','success',1502,'ROFS-16','ROMA','','','Aeron',1452,'',100 from dual union all
select 67362,'2001-Y17','2001-08-23 10:09:00','suborbital rocket','success',1502,'ROFS-17','ROMA','','','Aeron',1452,'',100 from dual union all
select 67363,'2001-Y18','2001-08-27 10:45:00','suborbital rocket','success',1502,'ROFS-18','ROMA','','','Aeron',1452,'',100 from dual union all
select 67364,'2001-Y19','2001-08-28 21:48:39','suborbital rocket','success',1502,'ROFS-19','ROMA','','','Aeron',1452,'',100 from dual union all
select 67365,'2001-Y20','2001-08-28 22:15:00','suborbital rocket','success',1502,'ROCH-20','ROMA','','','Aeron',1452,'',100 from dual union all
select 67366,'2001-Y21','2001-08-29 10:11:00','suborbital rocket','success',1502,'ROFS-21','ROMA','','','Aeron',1452,'',100 from dual union all
select 67367,'2001-Y22','2001-09-01 10:35:00','suborbital rocket','success',1502,'ROFS-22','ROMA','','','Aeron',1452,'',100 from dual union all
select 67368,'2001-Y23','2001-09-05 20:05:00','suborbital rocket','success',1502,'ROFS-23','ROMA','','','Aeron',1452,'',100 from dual union all
select 67369,'2001-Y24','2001-09-06 09:43:01','suborbital rocket','success',1502,'ROFS-24','ROMA','','','Aeron',1452,'',100 from dual union all
select 6737,'1960-S05','1960-01-01 00:00:00','suborbital rocket','success',619,'','','','','Test/Aeron',1895,'B-57A',800 from dual union all
select 67370,'2001-Y25','2001-09-08 10:24:00','suborbital rocket','success',1502,'ROFS-25','ROMA','','','Aeron',1452,'',100 from dual union all
select 67371,'2001-Y26','2001-09-11 09:22:00','suborbital rocket','success',1502,'ROFS-26','ROMA','','','Aeron',1452,'',100 from dual union all
select 67372,'2001-Y27','2001-09-11 09:51:00','suborbital rocket','success',1502,'ROFS-27','ROMA','','','Aeron',1452,'',100 from dual union all
select 67373,'2001-Y28','2001-09-14 09:16:00','suborbital rocket','success',1502,'ROFS-28','ROMA','','','Aeron',1452,'',100 from dual union all
select 67374,'2001-Y29','2001-09-14 10:16:00','suborbital rocket','success',1502,'ROFS-29','ROMA','','','Aeron',1452,'',100 from dual union all
select 67375,'2001-Y30','2001-09-14 10:44:01','suborbital rocket','success',1502,'ROCH-30','ROMA','','','Aeron',1452,'',100 from dual union all
select 67376,'2001-Y31','2001-12-15 11:53:30','suborbital rocket','success',1284,'Wind','Hygrosonde II Met','','','Aeron',409,'',110 from dual union all
select 67377,'2002-001','2002-01-16 00:30:01','orbital','success',1410,'4B-38/TC-19','USA 164 (Milstar 5)','Milstar DFS-5','','Sat',291,'',null from dual union all
select 67378,'2002-002','2002-01-23 23:46:57','orbital','success',99,'V147 L4108','Insat 3C','Insat 3C','','Sat',345,'',null from dual union all
select 67379,'2002-003','2002-02-04 02:45:00','orbital','success',531,'H-IIA-2F','VEP-3/MDS-1/DASH','MDS-1','','Sat',1477,'',null from dual union all
select 6738,'1960-S06','1960-01-01 00:00:00','suborbital rocket','failure',1247,'','','','','Test',1057,'F-3B',100 from dual union all
select 67380,'2002-004','2002-02-05 20:58:12','orbital','success',911,'F31','HESSI','HESSI','','Sat',284,'L-1011',null from dual union all
select 67381,'2002-005','2002-02-11 17:43:44','orbital','success',418,'D290','Iridium IS-1','Iridium SV091','','Sat',1724,'',null from dual union all
select 67382,'2002-006','2002-02-21 12:43:01','orbital','success',140,'AC-204','Echostar 7','Echostar 7','','Sat',289,'',null from dual union all
select 67383,'2002-007','2002-02-23 06:59:00','orbital','success',105,'V148 L4109','Intelsat 904','INTELSAT 904','','Sat',345,'',null from dual union all
select 67384,'2002-008','2002-02-25 17:26:21','orbital','success',1241,'77082  -842 083','Kosmos-2387','Kobal''t','','Sat',474,'',null from dual union all
select 67385,'2002-009','2002-03-01 01:07:59','orbital','success',114,'V145 (511)','Envisat','Envisat PPF','','Sat',346,'',null from dual union all
select 67386,'2002-010','2002-03-01 11:22:02','orbital','success',1243,'STS-109','HST SM-3B','OV-102 HST mission','','Sat',921,'',null from dual union all
select 67387,'2002-011','2002-03-08 22:59:01','orbital','success',173,'AC-143','TDRS 9','TDRS I','','Sat',288,'',null from dual union all
select 67388,'2002-012','2002-03-17 09:21:27','orbital','success',1064,'492-6992-013P/72505','GRACE 1/2','GRACE 1','','Sat',460,'',null from dual union all
select 67389,'2002-013','2002-03-21 20:13:39','orbital','success',1241,'F 15000-678 678','Progress M1-8','Progress M1 No. 257','','Sat ISS 7P',480,'',null from dual union all
select 6739,'1960-S07','1960-01-08 02:52:00','military missile','success',928,'A1X-7','Inertial test','','','Test',251,'',500 from dual union all
select 67390,'2002-014','2002-03-25 14:15:04','orbital','success',306,'Y3       (66)','Shenzhou 3','Shenzhou 3 Fanhui Cang','','Sat',814,'',null from dual union all
select 67391,'2002-015','2002-03-29 01:29:00','orbital','success',105,'V149 L4110','Astra 3A/JCSAT-8','JCSAT-8','','Sat',345,'',null from dual union all
select 67392,'2002-016','2002-03-30 17:25:00','orbital','success',960,'406-01','Intelsat 903','INTELSAT 903','','Sat',587,'',null from dual union all
select 67393,'2002-017','2002-04-01 22:06:45','orbital','success',803,'78054  -688 045','Kosmos-2388','Oko US-KS','','Sat',462,'',null from dual union all
select 67394,'2002-018','2002-04-08 20:44:19','orbital','success',1243,'STS-110','ISS 8A','OV-104 ISS','','Sat',922,'',null from dual union all
select 67395,'2002-019','2002-04-16 23:02:00','orbital','success',105,'V150 L4111','NSS 7','NSS 7','','Sat',345,'',null from dual union all
select 67396,'2002-020','2002-04-25 06:26:35','orbital','success',1241,'F 15000-675 675','Soyuz TM-34','Soyuz TM No. 208','','Sat ISS 4S',480,'',null from dual union all
select 67397,'2002-021','2002-05-04 01:31:46','orbital','success',102,'V151 L4112','SPOT 5','SPOT 5','','Sat',345,'',null from dual union all
select 67398,'2002-022','2002-05-04 09:54:58','orbital','success',419,'D291','Aqua','EOS PM-1','','Sat',1724,'',null from dual union all
select 67399,'2002-023','2002-05-07 17:00:00','orbital','success',960,'404-02','DirecTV-5','DirecTV-5','','Sat',588,'',null from dual union all
select 674,'1947-A10','1947-04-28 00:00:00','atmospheric rocket','success',471,'','XF-91 Model','','','Test',1821,'',5 from dual union all
select 6740,'1960-S08','1960-01-07 01:40:00','military missile','success',154,'43D','Mk 3-IX','R&D','','Test',228,'',1800 from dual union all
select 67400,'2002-024','2002-05-15 01:50:00','orbital','success',319,'Y5       (67)','FY-1D/HY-1','haiyang yihao A','','Sat',1492,'',null from dual union all
select 67401,'2002-025','2002-05-28 15:25:00','orbital','success',1174,'5','''Ofeq-5','''Ofeq-5','','Sat',1026,'',null from dual union all
select 67402,'2002-026','2002-05-28 18:14:41','orbital','success',680,'53721-702','Kosmos-2389','Parus?','','Sat',456,'',null from dual union all
select 67403,'2002-027','2002-06-05 06:44:00','orbital','success',105,'V152 L4113','Intelsat 905','INTELSAT 905','','Sat',345,'',null from dual union all
select 67404,'2002-028','2002-06-05 21:22:49','orbital','success',1243,'STS-111','ISS UF-2','OV-105 ISS','','Sat',921,'MLP1',null from dual union all
select 67405,'2002-029','2002-06-10 01:14:00','orbital','success',959,'407-01','Ekspress A No. 4','Ekspress-A No. 4','','Sat',553,'',null from dual union all
select 67406,'2002-030','2002-06-15 22:39:30','orbital','success',1542,'SL8/DM-SL-8L','SL8 Galaxy 3C','Galaxy III-C','','Sat',860,'ODYSSEY',null from dual union all
select 67407,'2002-031','2002-06-20 09:33:46','orbital','success',1064,'492-5882-034/72502','Iridium 97/98','Iridium SV097','','Sat',460,'',null from dual union all
select 67408,'2002-032','2002-06-24 18:23:04','orbital','success',1422,'23G-14','NOAA 17','NOAA M','','Sat',1778,'',null from dual union all
select 67409,'2002-033','2002-06-26 05:36:30','orbital','success',1241,'E 15000-679 679','Progress M-46','Progress M No. 246','','Sat ISS 8P',480,'',null from dual union all
select 6741,'1960-S09','1960-01-13 21:07:00','military missile','success',928,'A1X-8','','','','Test',251,'',500 from dual union all
select 67410,'2002-034','2002-07-03 06:47:41','deep space','success',415,'D292','CONTOUR','Contour','','Sat',285,'',null from dual union all
select 67411,'2002-035','2002-07-05 23:22:00','orbital','success',114,'V153 (512)','Stellat/NStar c','Stellat 5','','Sat',346,'',null from dual union all
select 67412,'2002-036','2002-07-08 06:35:41','orbital','success',680,'47121-704','Kosmos-2390/2391','Strela-3','','Sat',456,'',null from dual union all
select 67413,'2002-037','2002-07-25 15:13:21','orbital','success',950,'408-01','Kosmos-2392','Arkon 11F664 No. 2?','','Sat',588,'',null from dual union all
select 67414,'2002-038','2002-08-21 22:05:00','orbital','success',182,'AV-001','Hot Bird 6','Eutelsat HB6','','Sat',292,'',null from dual union all
select 67415,'2002-039','2002-08-22 05:15:00','orbital','success',960,'406-02','Echostar VIII','Echostar 8','','Sat',587,'',null from dual union all
select 67416,'2002-040','2002-08-28 22:45:00','orbital','success',114,'V155 (513)','Atl. Bird 1/MSG 1','Atlantic Bird 1','','Sat',346,'',null from dual union all
select 67417,'2002-041','2002-09-06 06:44:00','orbital','success',105,'V154 L4114','Intelsat 906','INTELSAT 906','','Sat',345,'',null from dual union all
select 67418,'2002-042','2002-09-10 08:20:00','orbital','success',531,'H-IIA-3F','USERS/DRTS','USERS SEM','','Sat',1477,'',null from dual;
insert into LAUNCH
select 67419,'2002-043','2002-09-12 10:23:40','orbital','success',899,'PSLV-C4','METSAT','METSAT-1','','Sat',1357,'',null from dual union all
select 6742,'1960-S10','1960-01-14 00:00:00','suborbital rocket','success',42,'TATTLETALE PCC 1','','','','Sigint Test?',394,'',100 from dual union all
select 67420,'2002-044','2002-09-18 22:04:01','orbital','success',174,'AC-159','Hispasat 1D','Hispasat 1D','','Sat',288,'',null from dual union all
select 67421,'2002-045','2002-09-25 16:58:24','orbital','success',1233,'E 15000-003 003','Progress M1-9','Progress M1 No. 258','','Sat ISS 9P',480,'',null from dual union all
select 67422,'2002-046','2002-09-26 14:27:14','orbital','success',680,'65021-706','Nadezhda-M','Nadezhda-M 17F118M No. 4','','Sat',456,'',null from dual union all
select 67423,'2002-047','2002-10-07 19:45:51','orbital','success',1243,'STS-112','ISS 9A','OV-104 ISS','','Sat',922,'',null from dual union all
select 67424,'2002-048','2002-10-17 04:41:00','orbital','success',951,'409-01','Integral','Integral','','Sat',553,'',null from dual union all
select 67425,'2002-049','2002-10-27 03:17:00','orbital','success',319,'Y6       (68)','ZY-2','ziyuan erhao 02 xing','','Sat',1492,'',null from dual union all
select 67426,'2002-050','2002-10-30 03:11:11','orbital','success',1233,'E 15000-004 004','Soyuz TMA-1','Soyuz 11F732 No. 211','','Sat ISS 5S',480,'',null from dual union all
select 67427,'2002-051','2002-11-20 22:39:00','orbital','success',400,'D4-1 (293) 4240','Eutelsat W5','Eutelsat W5','','Sat',290,'',null from dual union all
select 67428,'2002-052','2002-11-24 00:49:47','orbital','success',1243,'STS-113','ISS 11A','OV-105 ISS','','Sat',921,'',null from dual union all
select 67429,'2002-053','2002-11-25 23:04:23','orbital','failure',960,'408-02','Astra 1K','Astra 1K','','Sat',587,'',352 from dual union all
select 6743,'1960-S100','1960-05-20 15:00:27','military missile','success',154,'56D','Mk 3-1B-220/LMSC rad','D/RIG Long range test','','Test',227,'',1765 from dual union all
select 67430,'2002-054','2002-11-28 06:07:07','orbital','success',680,'53779-802','DMC-1/Mozhaets','DMC Alsat-1','','Sat',456,'',null from dual union all
select 67431,'2002-055','2002-12-05 02:42:01','orbital','success',173,'AC-144','TDRS 10','TDRS J','','Sat',288,'',null from dual union all
select 67432,'2002-056','2002-12-14 01:31:00','orbital','success',528,'H-IIA-4F','ADEOS-2','ADEOS-2','','Sat',1477,'',null from dual union all
select 67433,'2002-057','2002-12-17 23:04:00','orbital','success',105,'V156 L4115','NSS 6','NSS 6','','Sat',345,'',null from dual union all
select 67434,'2002-058','2002-12-20 17:00:00','orbital','success',434,'510-3541-505','TB/Latinsat/','Rubin-2/GCh 15F183','','Sat',497,'',null from dual union all
select 67435,'2002-059','2002-12-24 12:20:13','orbital','success',803,'77045  -685 042','Kosmos-2393','Oko US-KS','','Sat',462,'',null from dual union all
select 67436,'2002-060','2002-12-25 07:37:58','orbital','success',959,'409-02','Kosmos-2394/2396','Uragan No. 791','','Sat',587,'',null from dual union all
select 67437,'2002-061','2002-12-29 16:40:04','orbital','success',306,'Y4       (69)','Shenzhou 4','Shenzhou 4 Fanhui Cang','','Sat',814,'',null from dual union all
select 67438,'2002-062','2002-12-29 23:16:40','orbital','success',962,'535-02/88504','Nimiq 2','Nimiq 2','','Sat',588,'',null from dual union all
select 67439,'2002-A01','2002-07-14 01:34:00','atmospheric rocket','failure',829,'','NEXST-1','','','Test',1831,'',0 from dual union all
select 6744,'1960-S101','1960-05-23 00:00:00','military missile','success',928,'A1X-17','','','','Test',421,'EAG-154',500 from dual union all
select 67440,'2002-A02','2002-11-14 19:25:00','military missile','success',1152,'BLUE VELVET','GMDS','','','Target',1713,'',46 from dual union all
select 67441,'2002-A03','2002-11-28 05:52:26','suborbital rocket','success',651,'KSR-I-1','','','','Aeronomy',33,'',43 from dual union all
select 67442,'2002-A04','2002-09-19 16:20:00','suborbital rocket','failure',271,'','','','','Test',208,'',0 from dual union all
select 67443,'2002-A05','2002-08-25 03:45:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67444,'2002-A06','2002-08-25 00:00:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67445,'2002-A07','2002-10-30 06:04:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67446,'2002-A08','2002-08-18 00:00:00','military missile','failure',71,'F75','Test','','','Test',761,'',2 from dual union all
select 67447,'2002-A09','2002-01-09 09:49:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67448,'2002-A10','2002-01-10 00:00:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67449,'2002-A11','2002-01-31 12:29:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 6745,'1960-S102','1960-05-24 11:30:00','military missile','failure',240,'BK.08','H-201','','','Reentry Test',1843,'',563 from dual union all
select 67450,'2002-A12','2002-03-14 00:00:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67451,'2002-A13','2002-04-11 00:00:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67452,'2002-A14','2002-05-10 00:00:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67453,'2002-A15','2002-06-06 00:00:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67454,'2002-A16','2002-06-17 03:07:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67455,'2002-A17','2002-06-18 03:12:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67456,'2002-A18','2002-07-18 03:54:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67457,'2002-A19','2002-07-23 02:56:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67458,'2002-A20','2002-10-30 08:12:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 67459,'2002-A21','2002-11-16 00:00:00','military missile','success',76,'','Test 2','','','Test',745,'',45 from dual union all
select 6746,'1960-S103','1960-05-24 16:40:00','suborbital rocket','success',30,'AA3.183C','','','','Ionos/Solar EUV',394,'',269 from dual union all
select 67460,'2002-A22','2002-01-24 15:30:00','military missile','success',1088,'','Lance intercept','','','Interceptor',1869,'',10 from dual union all
select 67461,'2002-F01','2002-09-15 10:30:00','orbital','failure',652,'001201Y','PS-1','HTSTL-1','','Test',1494,'',100 from dual union all
select 67462,'2002-F02','2002-10-15 18:20:00','orbital','failure',1241,'A 15000-066 066','Foton-M No. 1','Foton-M No. 1','','Sat',474,'',0 from dual union all
select 67463,'2002-F03','2002-12-11 22:22:00','orbital','failure',111,'V157 (517)','Hot Bird 7/Stentor','Hot Bird 7','','Sat',346,'',150 from dual union all
select 67464,'2002-M01','2002-08-12 13:48:00','suborbital rocket','success',1023,'','Middle Atmosphere','','','Aeron',1467,'',70 from dual union all
select 67465,'2002-M02','2002-11-21 00:00:00','suborbital rocket','success',1023,'','Middle Atmosphere','','','Aeron',1467,'',70 from dual union all
select 67466,'2002-M03','2002-12-18 11:15:00','suborbital rocket','success',566,'','SOAREX-2','','','Tech',1821,'',70 from dual union all
select 67467,'2002-M04','2002-01-09 00:00:00','suborbital rocket','success',520,'','ANSR','','','Test',1790,'',50 from dual union all
select 67468,'2002-M05','2002-01-11 00:00:00','suborbital rocket','success',520,'','Barrage Round','','','Test',1790,'',50 from dual union all
select 67469,'2002-M19','2002-01-26 00:00:00','military missile','success',71,'F37','Test','','','Test',761,'',60 from dual union all
select 6747,'1960-S104','1960-05-24 17:44:00','suborbital rocket','success',214,'NRC BB-1-06','','','','Test',428,'',171 from dual union all
select 67470,'2002-M20','2002-03-14 00:00:00','military missile','success',71,'F29','Test','','','Test',761,'',60 from dual union all
select 67471,'2002-M21','2002-04-22 00:00:00','military missile','success',71,'F46','Test','','','Test',761,'',60 from dual union all
select 67472,'2002-M22','2002-06-09 00:00:00','military missile','success',71,'F59','Test','','','Test',761,'',60 from dual union all
select 67473,'2002-M23','2002-06-09 00:00:00','military missile','success',71,'F60','Test','','','Test',761,'',60 from dual union all
select 67474,'2002-M24','2002-07-22 00:00:00','military missile','success',71,'F67','Test','','','Test',761,'',60 from dual union all
select 67475,'2002-M25','2002-08-22 00:00:00','military missile','success',71,'F76','Test','','','Test',761,'',60 from dual union all
select 67476,'2002-M26','2002-08-24 00:00:00','military missile','success',71,'F74','Test','','','Test',761,'',60 from dual union all
select 67477,'2002-M27','2002-09-30 00:00:00','military missile','success',71,'F70','Test','','','Test',761,'',60 from dual union all
select 67478,'2002-M28','2002-09-30 00:00:00','military missile','success',71,'F79','Test','','','Test',761,'',60 from dual union all
select 67479,'2002-M29','2002-09-30 00:00:00','military missile','success',71,'F80','Test','','','Test',761,'',60 from dual union all
select 6748,'1960-S105','1960-05-25 00:48:00','suborbital rocket','success',838,'NASA 03.24CA','GCA Release 7 Na','','','Aeronomy',1821,'',199 from dual union all
select 67480,'2002-M30','2002-10-28 00:00:00','military missile','success',71,'F84','Test','','','Test',761,'',60 from dual union all
select 67481,'2002-M31','2002-10-28 00:00:00','military missile','success',71,'F78','Test','','','Test',761,'',60 from dual union all
select 67482,'2002-M32','2002-11-25 00:00:00','military missile','success',71,'F43','Test','','','Test',761,'',60 from dual union all
select 67483,'2002-M33','2002-11-25 00:00:00','military missile','success',71,'F44','Test','','','Test',761,'',60 from dual union all
select 67484,'2002-M34','2002-05-26 00:00:00','military missile','success',510,'Hatf III','','','','Test',1419,'',50 from dual union all
select 67485,'2002-M35','2002-05-26 00:00:00','military missile','success',569,'Abdali','','','','Test',1419,'',70 from dual union all
select 67486,'2002-M36','2002-09-06 00:00:00','military missile','success',489,'','R&D','','','Test',759,'',60 from dual union all
select 67487,'2002-S01','2002-01-03 12:15:00','military missile','failure',359,'','DF-31 RV test','','','Test',1494,'',100 from dual union all
select 67488,'2002-S02','2002-01-14 08:23:05','suborbital rocket','success',237,'NASA 40.014UE','SIERRA','','','Ionos/Auroral',1044,'',735 from dual union all
select 67489,'2002-S03','2002-01-25 03:15:00','military missile','success',62,'','Agni RV Mk 2','','','Test',320,'',300 from dual union all
select 6749,'1960-S106','1960-05-25 22:00:00','suborbital rocket','success',838,'NASA 03.04GS','','','','Solar UV?',1821,'',193 from dual union all
select 67490,'2002-S04','2002-01-26 02:00:00','military missile','success',732,'Aegis FM-2 Target','Aegis TTV-3','','','Target',855,'',300 from dual union all
select 67491,'2002-S05','2002-01-26 02:08:00','military missile','success',1090,'Aegis FM-2','Stellar Eagle','','','Interceptor',851,'CG-70',160 from dual union all
select 67492,'2002-S06','2002-02-06 09:38:15','suborbital rocket','success',237,'NASA 40.016UE','RACE','','','Ionos',1047,'',922 from dual union all
select 67493,'2002-S07','2002-02-08 18:41:30','suborbital rocket','success',224,'NASA 36.192UE','NCAR/CU-8','','','Solar EUV',1873,'',300 from dual union all
select 67494,'2002-S08','2002-02-06 10:30:00','suborbital rocket','success',1073,'S-310-30','NTV-2 Thermosphere','','','Aeron',833,'',150 from dual union all
select 67495,'2002-S09','2002-02-21 09:55:01','suborbital rocket','success',227,'NASA 21.128UE','CODA','','','Ionos',1047,'',250 from dual union all
select 67496,'2002-S10','2002-02-21 09:57:01','suborbital rocket','success',1319,'NASA 33.069UE','','','','Ionos',1047,'',200 from dual union all
select 67497,'2002-S11','2002-02-21 09:59:01','suborbital rocket','success',1319,'NASA 33.070UE','','','','Ionos',1047,'',200 from dual union all
select 67498,'2002-S12','2002-02-21 10:01:01','suborbital rocket','success',1319,'NASA 33.073UE','','','','Ionos',1047,'',200 from dual union all
select 67499,'2002-S13','2002-03-07 09:07:51','suborbital rocket','success',605,'NASA 30.49UE','','','','Ionos',1047,'',85 from dual union all
select 675,'1947-A11','1947-05-01 00:00:00','atmospheric rocket','success',564,'','FR-2 Model','','','Test',1821,'',5 from dual union all
select 6750,'1960-S107','1960-05-27 17:20:00','military missile','success',1419,'G-9','Mk 4-1A RV','','','Test',231,'',1000 from dual union all
select 67500,'2002-S14','2002-03-15 11:15:41','suborbital rocket','success',605,'NASA 30.52UE','','','','Aeron?',1047,'',85 from dual union all
select 67501,'2002-S15','2002-03-15 13:03:51','suborbital rocket','success',605,'NASA 30.50UE','','','','Aeron?',1047,'',85 from dual union all
select 67502,'2002-S16','2002-03-15 13:42:31','suborbital rocket','success',605,'NASA 30.51UE','','','','Aeron?',1047,'',85 from dual union all
select 67503,'2002-S17','2002-03-16 21:50:00','military missile','success',1443,'DASO-17','-    (95ss)','','','OT',419,'SSBN 732',1000 from dual union all
select 67504,'2002-S18','2002-03-16 02:11:00','military missile','success',784,'TLV-2','MSLS IFT-8','GMDS IFT-8','','Target',1701,'',1600 from dual union all
select 67505,'2002-S19','2002-03-16 02:32:00','military missile','success',797,'PLV IFT-8','IFT-8 EKV','','','ABM Test',868,'',225 from dual union all
select 67506,'2002-S20','2002-03-16 09:37:00','suborbital rocket','success',1212,'MASER 9','SSC MASER 9','','','Micrograv',416,'',260 from dual union all
select 67507,'2002-S21','2002-03-18 21:07:01','suborbital rocket','success',605,'NASA 30.47UO','ASRP SRP-4','','','Education/Ionos',1047,'',89 from dual union all
select 67508,'2002-S22','2002-03-21 14:09:00','military missile','success',570,'21/MBRV-3','PAC-3 OT/DT-1','','','Target',436,'',114 from dual;
insert into LAUNCH
select 67509,'2002-S23','2002-04-08 09:29:00','military missile','success',799,'','Mk 12/12A RV x 3','FDEP GT178GM','GLORY TRIP 178GM','Op Test',1706,'',1300 from dual union all
select 6751,'1960-S108','1960-05-27 04:05:00','suborbital rocket','success',30,'NRL 59','(NRL 49)','','','EUV Astron',1869,'',193 from dual union all
select 67510,'2002-S24','2002-04-24 22:00:00','military missile','success',732,'QRLV-2','','','','Target',862,'',160 from dual union all
select 67511,'2002-S25','2002-04-24 20:23:00','military missile','success',1443,'FCET-26','','','','OT',420,'SSBN 743',1000 from dual union all
select 67512,'2002-S26','2002-04-24 00:00:00','military missile','success',1443,'FCET-26','','','','OT',420,'SSBN 743',1000 from dual union all
select 67513,'2002-S27','2002-04-25 00:00:00','military missile','success',1443,'FCET-27','','','','OT',420,'SSBN 743',1000 from dual union all
select 67514,'2002-S28','2002-04-25 13:00:00','military missile','success',1263,'','PAC-3 OT/DT-4','','','Target',1895,'',100 from dual union all
select 67515,'2002-S29','2002-04-30 18:20:10','suborbital rocket','success',224,'NASA 36.194US','MSSTA III','','','Solar EUV',1873,'',270 from dual union all
select 67516,'2002-S30','2002-05-05 00:00:00','military missile','success',1168,'','','','','Test',1356,'',150 from dual union all
select 67517,'2002-S31','2002-05-10 06:32:00','suborbital rocket','success',860,'NASA 31.124UE','','','','Aeron',1821,'',93 from dual union all
select 67518,'2002-S32','2002-05-12 04:47:00','suborbital rocket','success',1219,'XV-31','Parnamirim F2Glow-2','','','Ionos',330,'',593 from dual union all
select 67519,'2002-S33','2002-05-16 06:26:00','suborbital rocket','success',860,'NASA 31.125UE','','','','Aeron',1821,'',93 from dual union all
select 6752,'1960-S109','1960-05-27 05:30:00','suborbital rocket','success',35,'NASA 04.05GG','UV star photometer','','','UV Astron',1821,'',215 from dual union all
select 67520,'2002-S34','2002-05-25 03:30:00','military missile','success',509,'Hatf-V/Ghauri II','','','','Test',1473,'',150 from dual union all
select 67521,'2002-S35','2002-05-30 02:00:00','military missile','success',1097,'MBRV-2','PAC-3 OT-2a','','','Target',1784,'',270 from dual union all
select 67522,'2002-S36','2002-06-03 08:01:00','military missile','success',907,'31PA (84-0543)','Mk 21 RV x 9','FDEP GT 31PA','','OT',1697,'',1000 from dual union all
select 67523,'2002-S37','2002-06-05 10:43:00','suborbital rocket','success',1345,'NASA 41.023UE','SAMPLER','','','Plasma?',1895,'',130 from dual union all
select 67524,'2002-S38','2002-06-05 11:13:00','suborbital rocket','success',1345,'NASA 41.022UE','SAMPLER','','','Plasma?',1895,'',130 from dual union all
select 67525,'2002-S39','2002-06-06 00:00:00','military missile','success',1437,'','15Zh55 LKI-2','','','OT',608,'',1000 from dual union all
select 67526,'2002-S40','2002-06-06 10:41:00','suborbital rocket','success',891,'NASA 30.53NO','','','','Education',1821,'',85 from dual union all
select 67527,'2002-S41','2002-06-07 08:24:00','military missile','success',799,'','Mk 12/12A RV x 3','FDEP GT179GM','GLORY TRIP 179GM','Op Test',1712,'',1300 from dual union all
select 67528,'2002-S42','2002-06-11 07:00:00','suborbital rocket','success',224,'NASA 36.112UG','NUVIEWS 2','','','UV Astron',1873,'',300 from dual union all
select 67529,'2002-S43','2002-06-14 01:23:00','military missile','success',732,'Aegis FM-3 Target','Aegis TTV-4','','','Target',855,'',300 from dual union all
select 6753,'1960-S11','1960-01-14 16:35:00','military missile','success',1368,'256','Special Test','','','Test/Wire',236,'',520 from dual union all
select 67530,'2002-S44','2002-06-14 01:29:00','military missile','success',1090,'Aegis FM-3','Stellar Impact','','','Test',851,'CG-70',100 from dual union all
select 67531,'2002-S45','2002-06-14 18:10:00','suborbital rocket','success',224,'NASA 36.196DS','VAULT II (LyA loops)','','','Solar UV',1873,'',273 from dual union all
select 67532,'2002-S46','2002-07-01 23:56:00','suborbital rocket','success',1345,'NASA 41.032GE','MM-MW-08 MacWAVE','','','Plasma',29,'',130 from dual union all
select 67533,'2002-S47','2002-07-02 01:44:00','suborbital rocket','success',605,'MIDAS','MM-MI-12','','','Aeron,Fields',29,'',106 from dual union all
select 67534,'2002-S48','2002-07-02 21:45:00','suborbital rocket','success',1336,'NASA 42.02DP','PACBLITZ 02','','','Target',855,'',300 from dual union all
select 67535,'2002-S49','2002-07-03 21:05:00','suborbital rocket','success',1336,'NASA 42.03DP','PACBLITZ 02','','','Target',855,'',300 from dual union all
select 67536,'2002-S50','2002-07-05 00:47:00','suborbital rocket','success',1345,'NASA 41.033GE','MM-MW-23 MacWAVE','','','Plasma',29,'',132 from dual union all
select 67537,'2002-S51','2002-07-05 01:10:00','suborbital rocket','success',605,'MIDAS','MM-MI-24','','','Aeron',29,'',85 from dual union all
select 67538,'2002-S52','2002-07-05 01:42:00','suborbital rocket','success',605,'MIDAS','MM-MI-25','','','Aeron',29,'',85 from dual union all
select 67539,'2002-S53','2002-07-12 00:58:31','suborbital rocket','success',1507,'','IRDT-2','','','Reentry Test',201,'K-44',200 from dual union all
select 6754,'1960-S110','1960-05-28 18:27:00','suborbital rocket','success',214,'NRC BB-1-07','','','','Test',428,'',177 from dual union all
select 67540,'2002-S54','2002-07-17 08:03:00','military missile','success',799,'','Mk 12/12A RV x 2','FDEP GT177GM','GLORY TRIP 177GM','Op Test',1705,'',1300 from dual union all
select 67541,'2002-S55','2002-07-30 02:05:00','suborbital rocket','success',1341,'','Hyshot 2 ''Hans''','','','Tech',1849,'',314 from dual union all
select 67542,'2002-S56','2002-08-03 14:24:00','suborbital rocket','success',1073,'S-310-31','SEEK-2','','','Aeron',833,'',152 from dual union all
select 67543,'2002-S57','2002-08-03 14:39:00','suborbital rocket','success',1073,'S-310-32','SEEK-2','','','Aeron',833,'',117 from dual union all
select 67544,'2002-S58','2002-08-06 18:30:00','suborbital rocket','success',224,'NASA 36.202US','USC-4 (SOHO SEM)','','','Solar EUV',1895,'',300 from dual union all
select 67545,'2002-S59','2002-09-20 02:31:00','military missile','success',799,'','Mk 12/12A RV x 2','FDEP GT180GM','GLORY TRIP 180GM','Op Test',1699,'',1300 from dual union all
select 67546,'2002-S60','2002-10-02 00:00:00','military missile','success',10,'','51T6','','','Test',183,'',100 from dual union all
select 67547,'2002-S61','2002-10-04 02:30:00','military missile','success',1169,'Hatf IV','','','','Test',1419,'',100 from dual union all
select 67548,'2002-S63','2002-10-12 09:12:00','military missile','success',1507,'VMF RF','','','','OT',1011,'PLBR',1000 from dual union all
select 67549,'2002-S64','2002-10-12 09:52:00','military missile','success',1434,'','15Zh58 UBP-28','','','OT',608,'',1000 from dual union all
select 6755,'1960-S111','1960-05-30 00:00:00','military missile','success',976,'','8K63 UBP-48','','','Test',636,'',402 from dual union all
select 67550,'2002-S65','2002-10-12 09:55:00','military missile','success',1179,'VMF RF','','','','OT',201,'TK-20',1000 from dual union all
select 67551,'2002-S66','2002-10-15 02:01:00','military missile','success',784,'TLV-3','MSLS IFT-9','GMDS IFT-9','','Target',1701,'',1600 from dual union all
select 67552,'2002-S67','2002-10-15 02:23:00','military missile','success',797,'PLV IFT-9','IFT-9 EKV','','','Target',868,'',225 from dual union all
select 67553,'2002-S68','2002-10-22 21:31:00','military missile','success',1443,'FCET-28','','','','OT',420,'SSBN 742',1000 from dual union all
select 67554,'2002-S69','2002-11-18 02:32:00','suborbital rocket','success',224,'NASA 36.197UG','ISIS','','','UV Astron',1895,'',300 from dual union all
select 67555,'2002-S70','2002-11-20 01:33:00','suborbital rocket','success',1345,'NASA 41.039DP','VALPE-1','','','Test',1821,'',135 from dual union all
select 67556,'2002-S71','2002-11-22 00:30:00','military missile','success',732,'Aegis FM-4 Target','Aegis TTV-5','','','Target',855,'',325 from dual union all
select 67557,'2002-S72','2002-11-22 00:32:00','military missile','success',1090,'Aegis FM-4','Stellar Viper (LEAP)','','','Test',851,'CG-70',152 from dual union all
select 67558,'2002-S73','2002-11-23 22:18:00','suborbital rocket','success',1479,'XV-02','Piraperna','','','Ionos',13,'',434 from dual union all
select 67559,'2002-S74','2002-11-25 09:30:00','military missile','success',1152,'BLUE VELVET','GMDS','','','Target',1713,'',84 from dual union all
select 6756,'1960-S112','1960-06-01 00:00:00','military missile','success',976,'','8K63 UBP-49','','','Test',636,'',402 from dual union all
select 67560,'2002-S75','2002-12-01 12:33:00','suborbital rocket','failure',1478,'XV-06','Cuma','','','Micrograv',13,'',60 from dual union all
select 67561,'2002-S76','2002-12-10 05:15:57','military missile','success',1453,'-       -118','15A35 UBP-51','GCh 15F355P7 (6 RV)','','OT',508,'',1000 from dual union all
select 67562,'2002-S77','2002-12-11 19:50:00','military missile','success',1443,'DASO-18','','','','OT',419,'SSBN 733',1000 from dual union all
select 67563,'2002-S78','2002-12-11 08:26:00','military missile','success',784,'TLV-4','MSLS IFT-10','GMDS IFT-10','','Target',1701,'',1600 from dual union all
select 67564,'2002-S79','2002-12-11 08:46:00','military missile','success',797,'PLV IFT-10','IFT-10 EKV','','','Target',868,'',225 from dual union all
select 67565,'2002-S80','2002-12-14 11:16:48','suborbital rocket','success',209,'NASA 35.033GE','CUSP 2002','','','Plasma',1453,'',766 from dual union all
select 67566,'2002-S81','2002-11-23 00:00:00','military missile','success',357,'','','','','Test',1494,'',1000 from dual union all
select 67567,'2002-S82','2002-07-01 00:00:00','military missile','success',354,'Decoy test','','','','Test',1494,'',1000 from dual union all
select 67568,'2002-S83','2002-12-01 00:00:00','military missile','success',354,'Decoy test','','','','Test',1494,'',1000 from dual union all
select 67569,'2002-S84','2002-07-04 00:00:00','military missile','success',1168,'','','','','Test',1356,'',150 from dual union all
select 6757,'1960-S113','1960-06-01 00:00:00','military missile','success',976,'','8K63 UBP-50','','','Test',636,'',402 from dual union all
select 67570,'2002-S85','2002-01-01 00:00:00','military missile','success',437,'-        -','','','','Test',1494,'',1000 from dual union all
select 67571,'2002-S86','2002-01-01 00:00:00','military missile','failure',646,'65013-318','','','','Reentry Test',621,'',675 from dual union all
select 67572,'2002-W01','2002-05-10 06:14:20','suborbital rocket','success',1502,'NASA 37.12UE','','','','Aeron',1821,'',75 from dual union all
select 67573,'2002-W02','2002-05-16 06:00:00','suborbital rocket','success',1502,'NASA 37.13UE','','','','Aeron',1821,'',75 from dual union all
select 67574,'2002-W03','2002-05-16 07:00:00','suborbital rocket','success',1502,'NASA 37.14UE','','','','Aeron',1821,'',75 from dual union all
select 67575,'2002-W04','2002-06-13 04:40:00','suborbital rocket','success',1502,'NASA 12.51UE','','','','Aeron/Test',1821,'',75 from dual union all
select 67576,'2002-W05','2002-06-28 17:10:00','suborbital rocket','success',1284,'','MM-SFS-01','','','Aeron',29,'',75 from dual union all
select 67577,'2002-W06','2002-07-01 17:20:00','suborbital rocket','success',1284,'','MM-LFS-02','','','Aeron',29,'',75 from dual union all
select 67578,'2002-W07','2002-07-01 19:20:00','suborbital rocket','success',1284,'','MM-LFS-03','','','Aeron',29,'',75 from dual union all
select 67579,'2002-W08','2002-07-01 20:20:00','suborbital rocket','success',1502,'','MM-VFS-04','','','Aeron',29,'',75 from dual union all
select 6758,'1960-S114','1960-06-25 00:00:00','military missile','success',979,'','8K65 LKI-2','','','Test',636,'',675 from dual union all
select 67580,'2002-W09','2002-07-01 21:20:00','suborbital rocket','success',1502,'','MM-VFS-05','','','Aeron',29,'',75 from dual union all
select 67581,'2002-W10','2002-07-01 22:20:00','suborbital rocket','success',1502,'','MM-VFS-06','','','Aeron',29,'',75 from dual union all
select 67582,'2002-W11','2002-07-01 23:20:00','suborbital rocket','success',1502,'','MM-VFS-07','','','Aeron',29,'',75 from dual union all
select 67583,'2002-W12','2002-07-02 00:20:00','suborbital rocket','success',1502,'','MM-VFS-09','','','Aeron',29,'',75 from dual union all
select 67584,'2002-W13','2002-07-02 00:40:00','suborbital rocket','success',1284,'','MM-LFS-10','','','Aeron',29,'',75 from dual union all
select 67585,'2002-W14','2002-07-02 01:20:00','suborbital rocket','success',1502,'','MM-VFS-11','','','Aeron',29,'',75 from dual union all
select 67586,'2002-W15','2002-07-02 02:20:00','suborbital rocket','success',1284,'','MM-LFS-13','','','Aeron',29,'',75 from dual union all
select 67587,'2002-W16','2002-07-02 03:20:00','suborbital rocket','success',1284,'','MM-LFS-14','','','Aeron',29,'',75 from dual union all
select 67588,'2002-W17','2002-07-04 19:18:00','suborbital rocket','success',1284,'','MM-LFS-15','','','Aeron',29,'',75 from dual union all
select 67589,'2002-W18','2002-07-04 20:14:00','suborbital rocket','success',1502,'','MM-VFS-16','','','Aeron',29,'',75 from dual union all
select 6759,'1960-S115','1960-05-31 00:00:00','ballistic missile test','success',1177,'5','E60 Echo Test','','','SC Test',1821,'',400 from dual union all
select 67590,'2002-W19','2002-07-04 20:45:00','suborbital rocket','success',1284,'','MM-LFS-17','','','Aeron',29,'',75 from dual union all
select 67591,'2002-W20','2002-07-04 21:30:00','suborbital rocket','success',1502,'','MM-VFS-18','','','Aeron',29,'',75 from dual union all
select 67592,'2002-W21','2002-07-04 22:15:00','suborbital rocket','success',1284,'','MM-LFS-19','','','Aeron',29,'',75 from dual union all
select 67593,'2002-W22','2002-07-04 23:00:00','suborbital rocket','success',1502,'','MM-VFS-20','','','Aeron',29,'',75 from dual union all
select 67594,'2002-W23','2002-07-04 23:20:00','suborbital rocket','success',1502,'','MM-VFS-21','','','Aeron',29,'',75 from dual union all
select 67595,'2002-W24','2002-07-05 00:05:00','suborbital rocket','success',1502,'','MM-VFS-22','','','Aeron',29,'',75 from dual union all
select 67596,'2002-W25','2002-07-05 02:19:00','suborbital rocket','success',1502,'','MM-VFS-26','','','Aeron',29,'',75 from dual union all
select 67597,'2002-W26','2002-07-05 03:00:00','suborbital rocket','success',1502,'','MM-VFS-27','','','Aeron',29,'',75 from dual union all
select 67598,'2002-W27','2002-07-05 04:00:00','suborbital rocket','success',1284,'','MM-LFS-28','','','Aeron',29,'',75 from dual union all
select 67599,'2002-W28','2002-07-05 05:00:00','suborbital rocket','success',1284,'','MM-LFS-29','','','Aeron',29,'',75 from dual;
insert into LAUNCH
select 676,'1947-A12','1947-05-22 00:00:00','military missile','success',340,'1','31D','','','Test',1863,'',39 from dual union all
select 6760,'1960-S116','1960-06-22 00:00:00','military missile','failure',873,'ZW-8','8','','','Test',1884,'',0 from dual union all
select 67600,'2002-W29','2002-07-05 06:00:00','suborbital rocket','success',1284,'','MM-LFS-30','','','Aeron',29,'',75 from dual union all
select 67601,'2002-W30','2002-07-05 06:45:00','suborbital rocket','success',1502,'','MM-VFS-31','','','Aeron',29,'',75 from dual union all
select 67602,'2002-W31','2002-08-21 10:23:00','suborbital rocket','success',1502,'','RO-A-FS-01 Roma-02','','','Aeron',29,'',75 from dual union all
select 67603,'2002-W32','2002-08-29 10:00:00','suborbital rocket','success',1502,'','RO-A-FS-04','','','Aeron',29,'',75 from dual union all
select 67604,'2002-W33','2002-09-04 10:00:00','suborbital rocket','success',1502,'','RO-A-FS-05','','','Aeron',29,'',75 from dual union all
select 67605,'2002-W34','2002-09-06 10:00:00','suborbital rocket','success',1502,'','RO-A-FS-06','','','Aeron',29,'',75 from dual union all
select 67606,'2002-W35','2002-09-10 10:00:00','suborbital rocket','success',1502,'','RO-A-FS-07','','','Aeron',29,'',75 from dual union all
select 67607,'2002-W36','2002-09-13 10:50:00','suborbital rocket','success',1502,'','RO-A-FS-08','','','Aeron',29,'',75 from dual union all
select 67608,'2002-W37','2002-09-14 10:27:00','suborbital rocket','success',1502,'','RO-A-FS-09','','','Aeron',29,'',75 from dual union all
select 67609,'2002-W38','2002-09-17 10:00:00','suborbital rocket','success',1502,'','RO-A-FS-10','','','Aeron',29,'',75 from dual union all
select 6761,'1960-S117','1960-06-02 00:00:00','military missile','success',1053,'CC-2022','-            174 km','','','OT 46AG',1854,'',75 from dual union all
select 67610,'2002-W39','2002-09-20 10:00:00','suborbital rocket','success',1502,'','RO-A-FS-11','','','Aeron',29,'',75 from dual union all
select 67611,'2002-W40','2002-09-20 10:35:30','suborbital rocket','success',1502,'','RO-A-FS-12','','','Aeron',29,'',75 from dual union all
select 67612,'2002-W41','2002-09-22 10:00:00','suborbital rocket','success',1502,'','RO-A-FS-13','','','Aeron',29,'',75 from dual union all
select 67613,'2002-W42','2002-09-26 10:00:00','suborbital rocket','success',1502,'','RO-A-FS-14','','','Aeron',29,'',75 from dual union all
select 67614,'2002-W43','2002-09-29 10:00:00','suborbital rocket','success',1502,'','RO-A-FS-15','','','Aeron',29,'',75 from dual union all
select 67615,'2002-W44','2002-09-29 10:19:00','suborbital rocket','success',1502,'','RO-A-FS-16','','','Aeron',29,'',75 from dual union all
select 67616,'2002-W45','2002-09-30 23:05:00','suborbital rocket','success',1502,'','RO-A-FS-17','','','Aeron',29,'',75 from dual union all
select 67617,'2002-W46','2002-10-04 22:00:00','suborbital rocket','success',1502,'','RO-A-FS-18','','','Aeron',29,'',75 from dual union all
select 67618,'2002-W47','2002-10-07 10:00:00','suborbital rocket','success',1502,'','RO-A-FS-19','','','Aeron',29,'',75 from dual union all
select 67619,'2002-W48','2002-10-07 10:25:00','suborbital rocket','success',1502,'','RO-A-FS-20','','','Aeron',29,'',75 from dual union all
select 6762,'1960-S118','1960-06-04 15:49:00','military missile','success',1016,'L1-9','8K74 UBP (Readiness)','','','OT',480,'',1350 from dual union all
select 67620,'2002-W49','2002-10-07 10:48:00','suborbital rocket','success',1502,'','RO-A-CH-21','','','Aeron',29,'',75 from dual union all
select 67621,'2002-W50','2002-10-10 10:00:00','suborbital rocket','success',1502,'','RO-A-FS-22','','','Aeron',29,'',75 from dual union all
select 67622,'2002-W51','2002-10-10 10:29:00','suborbital rocket','success',1502,'','RO-A-FS-23','','','Aeron',29,'',75 from dual union all
select 67623,'2002-Y01','2002-08-21 10:49:00','suborbital rocket','success',1502,'','RO-A-FS-02','','','Aeron',29,'',106 from dual union all
select 67624,'2002-Y02','2002-08-24 10:00:00','suborbital rocket','success',1502,'','RO-A-FS-03','','','Aeron',29,'',108 from dual union all
select 67625,'2003-001','2003-01-06 14:19:00','orbital','success',1422,'23G-4','Coriolis','P98-2 Coriolis','','Sat',1778,'',null from dual union all
select 67626,'2003-002','2003-01-13 00:45:00','orbital','success',410,'D294','ICESAT/Chipsat','Icesat','','Sat',1724,'',null from dual union all
select 67627,'2003-003','2003-01-16 15:39:00','orbital','success',1243,'STS-107','Spacehab','OV-102 Research','','Sat',921,'',null from dual union all
select 67628,'2003-004','2003-01-25 20:13:35','orbital','success',911,'F32','SORCE','SORCE','','Sat',284,'L-1011',null from dual union all
select 67629,'2003-005','2003-01-29 18:06:00','orbital','success',427,'D295','USA 166 (GPS IIR-8)','Navstar GPS SVN 56','','Sat',286,'',null from dual union all
select 6763,'1960-S119','1960-06-06 00:00:00','military missile','success',979,'','8K65 LKI-1','','','Test',636,'',675 from dual union all
select 67630,'2003-006','2003-02-02 12:59:40','orbital','success',1241,'E 15000-680 680','Progress M-47','Progress M No. 247','','Sat ISS 10P',480,'',null from dual union all
select 67631,'2003-007','2003-02-15 07:00:00','orbital','success',105,'V159 L4116','Intelsat 907','INTELSAT 907','','Sat',345,'',null from dual union all
select 67632,'2003-008','2003-03-11 00:59:00','orbital','success',398,'D4-2 (296) 4040','USA 167 (DSCS 3A-3)','DSCS III A-3/IABS','','Sat',290,'',null from dual union all
select 67633,'2003-009','2003-03-28 01:27:00','orbital','success',531,'H-IIA-5F','IGS O-1/IGS R-1','IGS Optical-1','','Sat',1477,'',null from dual union all
select 67634,'2003-010','2003-03-31 22:09:01','orbital','success',427,'D297','USA 168 (GPS IIR-9)','Navstar GPS SVN 45','','Sat',285,'',null from dual union all
select 67635,'2003-011','2003-04-02 01:53:01','orbital','success',803,'78029  -684 041','Molniya-1T','Molniya-1T','','Sat',462,'',null from dual union all
select 67636,'2003-012','2003-04-08 13:43:00','orbital','success',1410,'4B-35/TC-23','USA 169 (Milstar 6)','Milstar DFS-6','','Sat',291,'',null from dual union all
select 67637,'2003-013','2003-04-09 22:52:19','orbital','success',114,'V160 (514)','Galaxy 12/Insat 3A','Insat 3A','','Sat',346,'',null from dual union all
select 67638,'2003-014','2003-04-12 00:47:01','orbital','success',141,'AC-205','Asiasat 4','Asiasat 4','','Sat',289,'',null from dual union all
select 67639,'2003-015','2003-04-24 04:23:17','orbital','success',958,'410-02','Kosmos-2397','Oko-1 No. 7126','','Sat',588,'',null from dual union all
select 6764,'1960-S12','1960-01-15 04:52:00','suborbital rocket','success',624,'JV-3','','','','Test/LyAlpha',1821,'',1200 from dual union all
select 67640,'2003-016','2003-04-26 03:53:52','orbital','success',1233,'E 15000-006 006','Soyuz TMA-2','Soyuz 11F732 No. 212','','Sat ISS 6S',480,'',null from dual union all
select 67641,'2003-017','2003-04-28 11:59:57','orbital','success',911,'F33','GALEX','GALEX','','Sat',284,'L-1011',null from dual union all
select 67642,'2003-018','2003-05-08 11:28:00','orbital','success',501,'GSLV-D2','GSAT-2','GSAT-2','','Sat',1357,'',null from dual union all
select 67643,'2003-019','2003-05-09 04:29:25','deep space','success',730,'M-V-5','MUSES C/KM','MUSES C','','Sat',836,'',null from dual union all
select 67644,'2003-020','2003-05-13 22:10:00','orbital','success',182,'AV-002','Hellas Sat 2','Hellas Sat 2','','Sat',292,'',null from dual union all
select 67645,'2003-021','2003-05-24 16:34:04','orbital','success',309,'Y7       (70)','Beidou','san ke beidou daohang s.','','Sat',1903,'',null from dual union all
select 67646,'2003-022','2003-06-02 17:45:26','deep space','success',1232,'E 15000-005 005/ST11','Mars Express','Mars Express','','Sat',562,'',null from dual union all
select 67647,'2003-023','2003-06-04 19:23:52','orbital','success',680,'53721-705','Kosmos-2398','Parus?','','Sat',456,'',null from dual union all
select 67648,'2003-024','2003-06-06 22:15:15','orbital','success',952,'410-01/88505','AMC-9','Americom 9','','Sat',553,'',null from dual union all
select 67649,'2003-025','2003-06-08 10:34:19','orbital','success',1241,'D 15000-681 681','Progress M1-10','Progress M1 No. 259','','Sat ISS 11P',480,'',null from dual union all
select 6765,'1960-S120','1960-06-06 17:10:00','suborbital rocket','success',839,'NASA 10.17GE','SBE Solar Beam Expt','','','Magnetospheric',428,'',122 from dual union all
select 67650,'2003-026','2003-06-10 13:55:59','orbital','success',1542,'SL16/DM-SL-18L','SL9 Thuraya 2','Thuraya 2','','Sat',860,'ODYSSEY',null from dual union all
select 67651,'2003-027','2003-06-10 17:58:47','deep space','success',427,'D298','MER-A','Mars Exploration Rover 2','','Sat',285,'',null from dual union all
select 67652,'2003-028','2003-06-11 22:38:15','orbital','success',114,'V161 (515)','Optus C1-D/BSAT-2c','BSAT-2c','','Sat',346,'',null from dual union all
select 67653,'2003-029','2003-06-19 20:00:35','orbital','success',803,'77082  -689 046','Molniya-3','Molniya-3 No. 65?','','Sat',462,'',null from dual union all
select 67654,'2003-030','2003-06-26 18:53:00','orbital','success',911,'F34','Orbview-3','Orbview-3','','Sat',1718,'L-1011',null from dual union all
select 67655,'2003-031','2003-06-30 14:15:26','orbital','success',1064,'511-1992-008P/72503','MOM','Mimosa','','Sat',460,'',null from dual union all
select 67656,'2003-032','2003-07-08 03:18:15','deep space','success',428,'D299','MER-B','Mars Exploration Rover 1','','Sat',286,'',null from dual union all
select 67657,'2003-033','2003-07-17 23:45:01','orbital','success',191,'AV-003','Rainbow 1','Rainbow 1','','Sat',292,'',null from dual union all
select 67658,'2003-034','2003-08-08 03:30:55','orbital','success',1542,'SL10/DM-SL-10L','SL10 Echostar IX','Echostar IX','','Sat',860,'ODYSSEY',null from dual union all
select 67659,'2003-035','2003-08-12 14:20:00','orbital','success',1241,'77082  -575 081','Kosmos-2399','Don','','Sat',562,'',null from dual union all
select 6766,'1960-S121','1960-06-09 16:19:00','military missile','success',916,'108','P-04','','','Test',256,'',180 from dual union all
select 67660,'2003-036','2003-08-13 02:09:33','orbital','success',911,'F35','Scisat-1','SCISAT-1','','Sat',1718,'L-1011',null from dual union all
select 67661,'2003-037','2003-08-19 10:50:45','orbital','success',680,'47179-804','Kosmos-2400/2401','Strela-3','','Sat',456,'',null from dual union all
select 67662,'2003-038','2003-08-25 05:35:39','deep space','success',421,'D300','SIRTF','SIRTF','','Sat',286,'',null from dual union all
select 67663,'2003-039','2003-08-29 01:47:59','orbital','success',1241,'D 15000-682 682','Progress M-48','Progress M No. 248','','Sat ISS 12P',480,'',null from dual union all
select 67664,'2003-040','2003-08-29 23:13:00','orbital','success',398,'D4-3 (301) 4040','USA 170 (DSCS 3B-6)','DSCS III B-6/IABS','','Sat',290,'',null from dual union all
select 67665,'2003-041','2003-09-09 04:29:00','orbital','success',1410,'4B-36/TC-20','USA 171      [RH9]','ORION 5','','Sat',291,'',null from dual union all
select 67666,'2003-042','2003-09-27 06:11:44','orbital','success',680,'65052-103','STSat-1/DMC-2/Mozh.','Mozhaets 4','','Sat',456,'',null from dual union all
select 67667,'2003-043','2003-09-27 23:14:46','deep space','success',114,'V162 (516)','Insat 3E/e-Bird','e-Bird','','Sat',346,'',null from dual union all
select 67668,'2003-044','2003-10-01 04:02:59','orbital','success',1542,'SL13/DM-SL-12L','SL11 Galaxy 13/Hor.1','Galaxy XIII/Horizons 1','','Sat',860,'ODYSSEY',null from dual union all
select 67669,'2003-045','2003-10-15 01:00:03','orbital','success',306,'Y5       (71)','Shenzhou 5','Shenzhou 5 Fanhui Cang','','Sat',814,'',null from dual union all
select 6767,'1960-S122','1960-06-10 00:00:00','military missile','success',1013,'','8K51 PRO Target C-2','NIIP-A Target','','Target',321,'',500 from dual union all
select 67670,'2003-046','2003-10-17 04:52:08','orbital','success',899,'PSLV-C5','IRS-P6','IRS-P6','','Sat',1357,'',null from dual union all
select 67671,'2003-047','2003-10-18 05:38:03','orbital','success',1233,'D 15000-007 007','Soyuz TMA-3','Soyuz 11F732 No. 213','','Sat ISS 7S',480,'',null from dual union all
select 67672,'2003-048','2003-10-18 16:17:00','orbital','success',1422,'23G-9','USA 172 (DMSP F16)','DMSP Block 5D-3 S-20','','Sat',1778,'',null from dual union all
select 67673,'2003-049','2003-10-21 03:16:00','orbital','success',319,'Y4       (72)','CBERS-2','ZY-1-2','','Sat',1492,'',null from dual union all
select 67674,'2003-050','2003-10-30 13:43:42','orbital','success',1064,'492-1921-121/72506','SERVIS-1','SERVIS-1','','Sat',460,'',null from dual union all
select 67675,'2003-051','2003-11-03 07:20:04','orbital','success',303,'Y4       (73)','JB-4','FSW Service Module','','Sat',813,'',null from dual union all
select 67676,'2003-052','2003-11-14 16:01:00','orbital','success',309,'Y8       (74)','Zhongxing-20','Shentong yi hao 01 xing','','Sat',1903,'',null from dual union all
select 67677,'2003-053','2003-11-24 06:22:00','orbital','success',959,'407-02','Yamal-200','Yamal-201/202','','Sat',587,'',null from dual union all
select 67678,'2003-054','2003-12-02 10:04:00','orbital','success',174,'AC-164','USA 173     [NOSS15]','INTRUDER 6?','','Sat',1775,'',null from dual union all
select 67679,'2003-055','2003-12-05 06:00:00','orbital','success',1265,'511-7822-123','Gruzomaket','Kondor-E GVM 14S135','','Sat',535,'',null from dual union all
select 6768,'1960-S123','1960-06-11 06:30:18','military missile','success',154,'54D','Mk 3-1B-221','D/AIG guidance test CL1','','Test',226,'',994 from dual union all
select 67680,'2003-056','2003-12-10 17:42:12','orbital','success',952,'410-03/88506','Kosmos-2402/2404','Uragan 794','','Sat',588,'',null from dual union all
select 67681,'2003-057','2003-12-18 02:30:00','orbital','success',141,'AC-203','USA 174 (UHF F11)','UHF F/O F11','','Sat',289,'',null from dual union all
select 67682,'2003-058','2003-12-21 08:05:00','orbital','success',427,'D302','USA 175 (GPS IIR-10)','Navstar GPS SVN 47','','Sat',285,'',null from dual union all
select 67683,'2003-059','2003-12-27 21:30:00','orbital','success',1232,'D 15000-008 084/ST12','Amos-2','Amos-2','','Sat',562,'',null from dual union all
select 67684,'2003-060','2003-12-28 23:00:00','orbital','success',959,'410-04','Ekspress AM-22','Sesat-2','','Sat',553,'',null from dual union all
select 67685,'2003-061','2003-12-29 19:06:18','orbital','success',297,'Y1       (75)','TC-1','Double Star DSP-E','','Sat',1902,'',null from dual union all
select 67686,'2003-A01','2003-01-05 14:37:00','military missile','success',119,'F10','AST-8','','','ATBM Test',1026,'',30 from dual union all
select 67687,'2003-A02','2003-03-27 12:10:01','suborbital rocket','failure',1334,'NASA 41.028UE','JOULE','','','Auroral/Release',1045,'',10 from dual union all
select 67688,'2003-A03','2003-05-20 00:00:00','suborbital spaceplane','success',1245,'24C/01C','','','','Test',955,'WK1',14 from dual union all
select 67689,'2003-A04','2003-06-06 10:01:00','suborbital rocket','success',891,'NASA 30.55NO','Student rocket','','','Education',1821,'',40 from dual;
insert into LAUNCH
select 6769,'1960-S124','1960-06-13 00:00:00','suborbital rocket','success',1490,'AGI20','Na release','','','Aeronomy',668,'',176 from dual union all
select 67690,'2003-A05','2003-07-29 00:00:00','suborbital spaceplane','success',1245,'29C/02C','','','','Test',955,'WK1',14 from dual union all
select 67691,'2003-A06','2003-08-07 14:37:00','suborbital spaceplane','success',1245,'30L/03G','','','','Test',955,'WK1',14 from dual union all
select 67692,'2003-A07','2003-08-27 00:00:00','suborbital spaceplane','success',1245,'31LC/04GC','','','','Test',955,'WK1',14 from dual union all
select 67693,'2003-A08','2003-08-27 00:00:00','suborbital spaceplane','success',1245,'32L/05G','','','','Test',955,'WK1',14 from dual union all
select 67694,'2003-A09','2003-09-23 00:00:00','suborbital spaceplane','success',1245,'37L/06G','','','','Test',955,'WK1',14 from dual union all
select 67695,'2003-A10','2003-10-17 00:00:00','suborbital spaceplane','success',1245,'38L/07G','','','','Test',955,'WK1',14 from dual union all
select 67696,'2003-A11','2003-10-18 19:00:00','suborbital rocket','success',1341,'FASTT SPV-1','Dummy Scramjet','','','Test',1821,'',20 from dual union all
select 67697,'2003-A12','2003-11-14 00:00:00','suborbital spaceplane','success',1245,'40L/08G','','','','Test',955,'WK1',14 from dual union all
select 67698,'2003-A13','2003-11-19 00:00:00','suborbital spaceplane','success',1245,'41L/09G','','','','Test',955,'WK1',14 from dual union all
select 67699,'2003-A14','2003-12-04 00:00:00','suborbital spaceplane','success',1245,'42L/10G','','','','Test',955,'WK1',14 from dual union all
select 677,'1947-A13','1947-07-10 19:18:00','suborbital rocket','failure',1470,'29','','','','Aeron',1863,'',16 from dual union all
select 6770,'1960-S126','1960-06-15 02:43:00','suborbital rocket','success',994,'','','','','Aeron/Ionos/Solar/Bio',636,'',200 from dual union all
select 67700,'2003-A15','2003-12-16 00:00:00','military missile','success',119,'F11','AST-9 Black Sparrow','','','ATBM Test',1026,'',30 from dual union all
select 67701,'2003-A16','2003-12-17 16:15:00','suborbital spaceplane','success',1245,'43L/11P','','','','Test',955,'WK1',20 from dual union all
select 67702,'2003-A17','2003-11-28 12:05:30','suborbital rocket','failure',847,'','ICI-1','','','Plasma',1453,'',12 from dual union all
select 67703,'2003-A18','2003-12-16 00:00:00','military missile','success',244,'','','','','Test',1026,'F-15D',40 from dual union all
select 67704,'2003-A19','2003-03-20 22:08:00','military missile','success',76,'','Kuwait','','','Weapon',747,'',45 from dual union all
select 67705,'2003-A20','2003-03-23 21:59:00','military missile','success',76,'','Camp NJ','','','Weapon',749,'',45 from dual union all
select 67706,'2003-A21','2003-03-24 10:35:00','military missile','success',76,'','Camp VA/Camp NJ','','','Weapon',748,'',45 from dual union all
select 67707,'2003-A22','2003-03-26 16:58:00','military missile','failure',71,'','Camp Doha','','','Weapon',747,'',2 from dual union all
select 67708,'2003-A23','2003-04-01 06:03:00','military missile','success',76,'','US Forces Iraq','','','Weapon',755,'',45 from dual union all
select 67709,'2003-A24','2003-01-01 00:00:00','military missile','success',702,'RRF','MLP Test (FY03)','','','Test',1057,'',40 from dual union all
select 6771,'1960-S129','1960-06-15 15:45:00','suborbital rocket','success',30,'NC3.107','NRL-57','','','Aeron',1895,'',224 from dual union all
select 67710,'2003-A25','2003-01-01 00:00:00','military missile','success',702,'RRF','MLP Test (FY03)','','','Test',1057,'',40 from dual union all
select 67711,'2003-E01','2003-08-22 16:30:00','orbital','destroyed before launch',1475,'V03','SATEC/UNOSAT','','','Sat',12,'',0 from dual union all
select 67712,'2003-F02','2003-11-29 04:33:00','orbital','failure',531,'H-IIA-6F','IGS O-2a/IGS R-2a','IGS Optical-2','','Sat',1477,'',422 from dual union all
select 67713,'2003-M01','2003-10-09 11:37:00','suborbital rocket','success',605,'NASA 30.54UO','Student rocket','','','Education',1821,'',64 from dual union all
select 67714,'2003-M02','2003-11-21 10:30:00','suborbital rocket','success',1023,'','Anniversary','','','Aeron',1467,'',70 from dual union all
select 67715,'2003-M03','2003-07-10 00:00:00','suborbital rocket','success',520,'','ANSR','','','Test',1790,'',50 from dual union all
select 67716,'2003-M04','2003-09-24 00:00:00','suborbital rocket','success',520,'','ANSR','','','Test',1790,'',50 from dual union all
select 67717,'2003-M05','2003-03-20 09:24:00','military missile','success',71,'','TAA Thunder, Kuwait','','','Weapon',750,'',60 from dual union all
select 67718,'2003-M06','2003-03-20 10:30:00','military missile','success',71,'','Camp Doha','','','Weapon',750,'',60 from dual union all
select 67719,'2003-M07','2003-03-20 21:30:00','military missile','success',71,'','Persian Gulf','','','Weapon',751,'',60 from dual union all
select 6772,'1960-S13','1960-01-16 22:35:00','ballistic missile test','success',1177,'2','E60 Echo Test','','','SC Test',1821,'',413 from dual union all
select 67720,'2003-M08','2003-03-20 23:20:00','military missile','success',71,'','Camp Udari','','','Weapon',751,'',60 from dual union all
select 67721,'2003-M09','2003-03-21 10:01:00','military missile','success',71,'','Al Jahra/Camp Doha','','','Weapon',750,'',60 from dual union all
select 67722,'2003-M10','2003-03-24 13:42:00','military missile','success',71,'','Camp Doha','','','Weapon',747,'',60 from dual union all
select 67723,'2003-M11','2003-03-25 12:48:00','military missile','success',71,'','Camp Commando,N Kuw','','','Weapon',751,'',60 from dual union all
select 67724,'2003-M12','2003-03-27 08:31:00','military missile','success',71,'','Camp Doha','','','Weapon',747,'',60 from dual union all
select 67725,'2003-M13','2003-10-03 03:42:00','military missile','success',510,'Hatf III','','','','Test',1419,'',50 from dual union all
select 67726,'2003-M14','2003-03-27 20:56:00','military missile','success',71,'','','','','Weapon',754,'',60 from dual union all
select 67727,'2003-M15','2003-03-26 00:00:00','military missile','success',569,'','','','','Test',1419,'',70 from dual union all
select 67728,'2003-S01','2003-01-09 03:17:00','military missile','success',62,'','Agni RV Mk 2','','','Test',316,'',300 from dual union all
select 67729,'2003-S02','2003-01-24 20:50:00','suborbital rocket','success',1345,'NASA 41.030GE','MacWAVE','','','Plasma',417,'',131 from dual union all
select 6773,'1960-S130','1960-06-15 22:56:00','suborbital rocket','success',42,'NASA 06.02UI','UM ionos probe','','','Ionos',428,'',321 from dual union all
select 67730,'2003-S03','2003-01-28 07:50:02','suborbital rocket','success',224,'NASA 36.200UE','HIBAR','','','Auroral',1047,'',385 from dual union all
select 67731,'2003-S04','2003-01-28 18:32:00','suborbital rocket','success',1345,'NASA 41.031GE','MacWAVE','','','Plasma/Aeron',417,'',130 from dual union all
select 67732,'2003-S05','2003-02-06 07:30:00','suborbital rocket','success',224,'NASA 36.204UE','NUVIEWS 3','','','UV Astron',1895,'',300 from dual union all
select 67733,'2003-S06','2003-02-06 21:00:00','military missile','success',497,'1','Taurus Lite','','','Test',1670,'',1810 from dual union all
select 67734,'2003-S07','2003-03-12 09:29:00','military missile','success',907,'32PA (85-0174)','Mk 21 RV x 8','FDEP GT 32PA','','OT',1697,'',1000 from dual union all
select 67735,'2003-S08','2003-03-25 09:50:01','suborbital rocket','success',209,'NASA 35.034UE','HEX H','','','Auroral/Release',1042,'',160 from dual union all
select 67736,'2003-S09','2003-03-25 10:09:01','suborbital rocket','success',1334,'NASA 41.034UE','HEX V','','','Auroral/Release',1043,'',180 from dual union all
select 67737,'2003-S10','2003-03-26 06:00:00','military missile','success',941,'','Prithvi-2','','','Test',320,'',100 from dual union all
select 67738,'2003-S11','2003-03-27 09:27:30','military missile','success',1434,'','15Zh58 UBP-29','','','OT',608,'',1000 from dual union all
select 67739,'2003-S12','2003-03-27 12:09:01','suborbital rocket','success',224,'NASA 36.206UE','JOULE','','','Auroral/Release',1047,'',193 from dual union all
select 6774,'1960-S131','1960-06-16 00:00:00','suborbital rocket','success',1490,'AGI19','Na release','','','Aeronomy',668,'',180 from dual union all
select 67740,'2003-S13','2003-03-27 12:12:01','suborbital rocket','success',228,'NASA 21.131UE','JOULE','','','Ionos',1041,'',199 from dual union all
select 67741,'2003-S14','2003-03-27 12:13:01','suborbital rocket','success',1334,'NASA 41.029UE','JOULE','','','Auroral/Release',1042,'',180 from dual union all
select 67742,'2003-S15','2003-04-01 06:00:00','suborbital rocket','success',772,'MAXUS 5','SSC MAXUS 5','','','Micrograv',407,'',701 from dual union all
select 67743,'2003-S16','2003-04-08 08:20:00','suborbital rocket','success',1212,'TEXUS 40','DLR TEXUS 40','','','Micrograv',416,'',246 from dual union all
select 67744,'2003-S17','2003-04-17 03:01:54','suborbital rocket','success',224,'NASA 36.201UL','BDPR','','','UV Astron',1895,'',300 from dual union all
select 67745,'2003-S18','2003-04-23 18:41:00','military missile','success',1443,'FCET-29','','','','OT',420,'SSBN 741',1000 from dual union all
select 67746,'2003-S19','2003-04-23 20:43:00','military missile','success',1443,'FCET-29','','','','OT',420,'SSBN 741',1000 from dual union all
select 67747,'2003-S20','2003-04-29 05:50:00','military missile','success',941,'','Prithvi-2','','','Test',320,'',100 from dual union all
select 67748,'2003-S21','2003-05-30 09:35:00','suborbital rocket','success',234,'NASA 39.005DP','RED DOG','','','Target',1821,'',300 from dual union all
select 67749,'2003-S22','2003-06-02 09:28:00','suborbital rocket','success',234,'NASA 39.006DP','RED DOG','','','Target',1821,'',300 from dual union all
select 6775,'1960-S132','1960-06-16 12:04:00','suborbital rocket','success',1194,'SL10','QUB Na-4-MM','','','Aeronomy',1839,'',129 from dual union all
select 67750,'2003-S23','2003-06-10 06:31:00','suborbital rocket','success',1337,'NASA 29.035UP','DEBI (Bowshock)','','','Tech',1821,'',800 from dual union all
select 67751,'2003-S24','2003-06-11 08:01:00','military missile','success',799,'','Mk 12/12A RV x 3','FDEP GT182GM','GLORY TRIP 182GM','Op Test',1699,'',1300 from dual union all
select 67752,'2003-S25','2003-06-18 23:15:00','military missile','success',732,'Aegis FM-5 Target','Aegis TTV-6','','','Target',843,'',300 from dual union all
select 67753,'2003-S26','2003-06-18 23:17:00','military missile','success',1090,'Aegis FM-5','Stellar Hammer','','','Interceptor',851,'CG-70',150 from dual union all
select 67754,'2003-S27','2003-06-30 03:19:00','suborbital rocket','success',1334,'NASA 41.036UE','Release M/Dtr','','','Aeron/release',1821,'',202 from dual union all
select 67755,'2003-S28','2003-06-30 05:41:00','suborbital rocket','success',1334,'NASA 41.037UE','Release M/Dtr','','','Aeron/release',1821,'',206 from dual union all
select 67756,'2003-S29','2003-06-30 06:50:00','suborbital rocket','success',210,'NASA 27.144UE','Intermediate Layers','','','Aeron/Ionos',1821,'',220 from dual union all
select 67757,'2003-S30','2003-06-30 07:07:00','suborbital rocket','success',1334,'NASA 41.038UE','Release M/Dtr','','','Aeron/release',1821,'',204 from dual union all
select 67758,'2003-S31','2003-07-01 09:23:00','suborbital rocket','success',605,'ROMA RO-MI-01','ROMA 2003','','','Aeron',1453,'',107 from dual union all
select 67759,'2003-S32','2003-07-04 08:20:00','suborbital rocket','success',605,'ROMA RO-MI-02','ROMA 2003','','','Aeron',1453,'',106 from dual union all
select 6776,'1960-S133','1960-06-18 00:00:00','suborbital rocket','success',1490,'AGI25','','','','Aeronomy',668,'',152 from dual union all
select 67760,'2003-S33','2003-07-06 08:27:00','suborbital rocket','success',605,'ROMA RO-MI-03','ROMA 2003','','','Aeron',1453,'',106 from dual union all
select 67761,'2003-S34','2003-07-15 07:02:00','suborbital rocket','success',1334,'NASA 41.025UE','SAMPLER (CWAS)','','','Aeron',1895,'',128 from dual union all
select 67762,'2003-S35','2003-07-15 07:32:00','suborbital rocket','success',1334,'NASA 41.024UE','SAMPLER (CWAS)','','','Aeron',1895,'',130 from dual union all
select 67763,'2003-S36','2003-08-01 10:25:03','suborbital rocket','success',1341,'NASA 12.052GT','NMACS Test','','','Test',1821,'',204 from dual union all
select 67764,'2003-S37','2003-08-06 09:06:00','military missile','success',799,'','Mk 12/12A RV x 3','FDEP GT183GB','GLORY TRIP 183GB','Op Test',1712,'',1300 from dual union all
select 67765,'2003-S38','2003-08-12 18:23:30','suborbital rocket','success',224,'NASA 36.205UE','LASP/CU-9 (SEE)','','','Plasma',1895,'',300 from dual union all
select 67766,'2003-S39','2003-08-16 18:00:00','military missile','success',497,'2','BV-6','','','Test',1709,'',1874 from dual union all
select 67767,'2003-S40','2003-08-20 11:05:00','suborbital rocket','success',1345,'NASA 41.040DP','VALPE-2','','','Test',1821,'',135 from dual union all
select 67768,'2003-S41','2003-09-10 11:31:00','military missile','success',799,'','Mk 12/12A RV x 3','FDEP GT181GM','GLORY TRIP 181GM','Op Test',1706,'',1300 from dual union all
select 67769,'2003-S42','2003-10-03 14:30:00','suborbital rocket','success',1334,'NASA 41.035UE','SPIRIT 2','','','Aeron/release',1821,'',130 from dual union all
select 6777,'1960-S134','1960-06-21 00:00:00','suborbital rocket','success',673,'','PHOENIX 1','','','Test',1057,'',300 from dual union all
select 67770,'2003-S43','2003-10-24 10:45:17','suborbital rocket','success',1341,'NASA 12.053GT','NMACS Test','SOAREX-3','','Test',1821,'',156 from dual union all
select 67771,'2003-S44','2003-11-05 21:10:02','military missile','success',1443,'FCET-30','','','','OT',420,'SSBN 738',1000 from dual union all
select 67772,'2003-S45','2003-11-05 23:22:43','military missile','success',1443,'FCET-30','-    (104ss)','','','OT',420,'SSBN 738',1000 from dual union all
select 67773,'2003-S47','2003-12-05 19:30:00','suborbital rocket','success',224,'NASA 36.211US','USC-5 (SOHO SEM)','','','Solar EUV',1895,'',300 from dual union all
select 67774,'2003-S48','2003-12-05 20:00:00','suborbital rocket','success',224,'NASA 36.184DS','','','','Solar EUV?',1895,'',300 from dual union all
select 67775,'2003-S49','2003-12-11 00:00:00','military missile','success',518,'','','','','Test',190,'TK-208',1000 from dual union all
select 67776,'2003-S50','2003-12-11 18:10:00','military missile','success',732,'Aegis FM-6 Target','Aegis TTV-7','','','Target',843,'',300 from dual union all
select 67777,'2003-S51','2003-12-11 18:12:00','military missile','success',1090,'Aegis FM-6','Stellar Defender','','','Interceptor',851,'CG-70',137 from dual union all
select 67778,'2003-S52','2003-12-16 03:00:00','suborbital rocket','success',224,'NASA 36.208UG','JHU FOT 17/LIDOS','','','UV Astron',1895,'',300 from dual union all
select 67779,'2003-S53','2003-12-24 15:25:00','suborbital rocket','success',1221,'3','TMA','','','Aeron',808,'',265 from dual;
insert into LAUNCH
select 6778,'1960-S135','1960-06-21 10:05:00','military missile','success',240,'BK.09','H-202','','','Reentry Test/Ionos',1843,'',484 from dual union all
select 67780,'2003-S54','2003-01-01 00:00:00','suborbital rocket','success',1024,'RH-200SV-6','','','','Test',1361,'',110 from dual union all
select 67781,'2003-S55','2003-09-02 00:00:00','military missile','success',1507,'VMF RF','','','','OT',1011,'K-223',1000 from dual union all
select 67782,'2003-S56','2003-06-01 00:00:00','military missile','success',1168,'','','','','Test',1356,'',150 from dual union all
select 67783,'2003-S57','2003-10-08 03:13:00','military missile','success',1169,'Hatf IV','','','','Test',1419,'',100 from dual union all
select 67784,'2003-S58','2003-10-15 10:03:00','military missile','success',1179,'','','','','OT',190,'K-18',1000 from dual union all
select 67785,'2003-S59','2003-10-14 00:00:00','military missile','success',1169,'Hatf IV','','','','Test',1419,'',100 from dual union all
select 67786,'2003-S60','2003-02-07 00:00:00','military missile','success',1151,'FMA Demo','','','','Test',1895,'',80 from dual union all
select 67787,'2003-S61','2003-12-24 00:00:00','military missile','failure',646,'65013-321','','','','Reentry Test',621,'',675 from dual union all
select 67788,'2003-U01','2003-09-16 10:30:00','orbital','failure',652,'','PS-2','KT-1 Test Payload','','Test',1494,'',100 from dual union all
select 67789,'2003-W01','2003-11-07 00:00:00','suborbital rocket','success',1284,'','Demo','','','Edu',295,'',60 from dual union all
select 6779,'1960-S136','1960-06-23 02:22:00','military missile','success',928,'A1X-32','','','','Test',421,'EAG-154',500 from dual union all
select 67790,'2003-Y01','2003-01-15 13:02:00','suborbital rocket','success',1284,'','SAGE-3 1','','','Aeron',409,'',110 from dual union all
select 67791,'2003-Y02','2003-01-16 13:00:00','suborbital rocket','success',1284,'','SAGE-3 2','','','Aeron',409,'',116 from dual union all
select 67792,'2003-Y03','2003-01-17 13:00:00','suborbital rocket','success',1284,'','SAGE-3 3','','','Aeron',409,'',112 from dual union all
select 67793,'2003-Y04','2003-01-21 13:00:00','suborbital rocket','success',1284,'','MM-LFS Wind 6','','','Aeron',409,'',113 from dual union all
select 67794,'2003-Y05','2003-01-24 13:11:00','suborbital rocket','success',1284,'ES03SL13','MM-LFS-','','','Aeron',409,'',107 from dual union all
select 67795,'2003-Y06','2003-01-24 14:03:00','suborbital rocket','success',1284,'ES03SL14','MM-LFS-','','','Aeron',409,'',103 from dual union all
select 67796,'2003-Y07','2003-01-24 15:05:00','suborbital rocket','success',1284,'ES03SL15','MM-LFS-','','','Aeron',409,'',110 from dual union all
select 67797,'2003-Y08','2003-01-24 16:00:00','suborbital rocket','success',1284,'ES03SL16','MM-LFS-','','','Aeron',409,'',106 from dual union all
select 67798,'2003-Y09','2003-01-24 17:00:00','suborbital rocket','success',1284,'ES03SL17','MM-LFS-','','','Aeron',409,'',114 from dual union all
select 67799,'2003-Y10','2003-01-24 18:00:00','suborbital rocket','success',1284,'ES03SL18','MM-LFS-','','','Aeron',409,'',115 from dual union all
select 678,'1947-A14','1947-07-17 17:30:00','military missile','failure',340,'2','33E','','','Test',1863,'',0 from dual union all
select 6780,'1960-S137','1960-06-22 00:00:00','suborbital rocket','failure',1490,'AGI26','','','','Aeronomy',668,'',60 from dual union all
select 67800,'2003-Y11','2003-01-24 19:01:00','suborbital rocket','success',1284,'ES03SL19','MM-LFS-','','','Aeron',409,'',106 from dual union all
select 67801,'2003-Y12','2003-01-24 20:00:00','suborbital rocket','success',1284,'ES03SL20','MM-LFS-','','','Aeron',409,'',109 from dual union all
select 67802,'2003-Y13','2003-01-24 21:02:00','suborbital rocket','success',1284,'ES03SL21','MM-LFS-','','','Aeron',409,'',113 from dual union all
select 67803,'2003-Y14','2003-01-24 22:00:00','suborbital rocket','success',1284,'ES03SL22','MM-LFS-','','','Aeron',409,'',106 from dual union all
select 67804,'2003-Y15','2003-01-24 23:00:00','suborbital rocket','success',1284,'ES03SL24','MM-LFS-','','','Aeron',409,'',91 from dual union all
select 67805,'2003-Y16','2003-01-28 13:00:00','suborbital rocket','success',1284,'ES03SL34','MM-LFS-','','','Aeron',409,'',115 from dual union all
select 67806,'2003-Y17','2003-01-28 15:04:00','suborbital rocket','success',1284,'ES03SL35','MM-LFS-','','','Aeron',409,'',115 from dual union all
select 67807,'2003-Y18','2003-01-28 15:57:00','suborbital rocket','success',1284,'ES03SL36','MM-LFS-','','','Aeron',409,'',116 from dual union all
select 67808,'2003-Y19','2003-01-28 17:05:00','suborbital rocket','success',1284,'ES03SL38','MM-LFS-','','','Aeron',409,'',115 from dual union all
select 67809,'2003-Y20','2003-01-28 17:57:00','suborbital rocket','success',1284,'ES03SL39','MM-LFS-','','','Aeron',409,'',117 from dual union all
select 6781,'1960-S138','1960-06-22 14:49:33','military missile','success',154,'62D','Mk 3-2B-219/R&D Decoy','D/RIG test','','Test',229,'',1231 from dual union all
select 67810,'2003-Y21','2003-01-28 18:42:00','suborbital rocket','success',1284,'ES03SL41','MM-LFS-','','','Aeron',409,'',108 from dual union all
select 67811,'2003-Y22','2003-01-28 19:42:00','suborbital rocket','success',1284,'ES03SL42','MM-LFS-','','','Aeron',409,'',114 from dual union all
select 67812,'2003-Y23','2003-01-25 00:00:00','suborbital rocket','success',1284,'ES03SL25','MM-LFS-','','','Aeron',409,'',117 from dual union all
select 67813,'2003-Y24','2003-01-25 00:58:00','suborbital rocket','success',1284,'ES03SL26','MM-LFS-','','','Aeron',409,'',114 from dual union all
select 67814,'2003-Y25','2003-01-29 13:00:00','suborbital rocket','success',1284,'','MM-LFS-','','','Aeron',409,'',113 from dual union all
select 67815,'2003-Y26','2003-01-30 13:00:00','suborbital rocket','success',1284,'','MM-LFS-','','','Aeron',409,'',117 from dual union all
select 67816,'2003-Y27','2003-01-30 13:12:00','suborbital rocket','success',1284,'','SAGE-3','','','Aeron',409,'',112 from dual union all
select 67817,'2003-Y28','2003-01-30 21:10:00','suborbital rocket','success',1284,'','SAGE-3','','','Aeron',409,'',107 from dual union all
select 67818,'2003-Y29','2003-01-30 21:55:00','suborbital rocket','success',1284,'','SAGE-3','','','Aeron',409,'',114 from dual union all
select 67819,'2003-Y30','2003-01-28 16:56:00','suborbital rocket','success',1284,'ES03SL33','MM-LFS-','','','Aeron',409,'',120 from dual union all
select 6782,'1960-S139','1960-06-22 23:26:00','military missile','success',1367,'233','CTL','','CLAN CHATTAN','OT',1676,'',520 from dual union all
select 67820,'2003-Y31','2003-01-30 22:03:00','suborbital rocket','success',1284,'','SAGE-3','','','Aeron',409,'',96 from dual union all
select 67821,'2003-Y32','2003-06-11 16:18:45','suborbital rocket','success',469,'','Mini-DUSTY MD-08','','','Aeron',29,'',90 from dual union all
select 67822,'2003-Y33','2003-06-13 15:30:20','suborbital rocket','success',469,'','Mini-DUSTY MD-09','','','Aeron',29,'',90 from dual union all
select 67823,'2003-Y34','2003-08-08 14:30:00','suborbital rocket','success',1165,'1','Shadow','','','Edu',295,'',110 from dual union all
select 67824,'2003-Y35','2003-08-08 15:30:00','suborbital rocket','success',1165,'2','Shadow','','','Edu',295,'',110 from dual union all
select 67825,'2003-Y36','2003-08-12 00:00:00','suborbital rocket','success',1284,'','Irish School Expt','','','Edu',295,'',110 from dual union all
select 67826,'2003-Y37','2003-01-13 13:08:00','suborbital rocket','success',1284,'','SAGE-3 Test 1','','','Aeron',409,'',110 from dual union all
select 67827,'2003-Y38','2003-01-13 14:49:00','suborbital rocket','success',1284,'','SAGE-3 Test 2','','','Aeron',409,'',110 from dual union all
select 67828,'2003-Y39','2003-01-14 13:00:00','suborbital rocket','success',1284,'','SAGE-3 Test 3','','','Aeron',409,'',110 from dual union all
select 67829,'2003-Y40','2003-01-14 14:00:00','suborbital rocket','success',1284,'','SAGE-3 Test 4','','','Aeron',409,'',110 from dual union all
select 6783,'1960-S14','1960-01-18 00:00:00','military missile','success',976,'','8K63 UBP-35','','','Test',636,'',402 from dual union all
select 67830,'2003-Y41','2003-01-18 13:00:00','suborbital rocket','success',1284,'','SAGE-3 Wind 4','','','Aeron',409,'',117 from dual union all
select 67831,'2003-Y42','2003-01-19 13:00:00','suborbital rocket','success',1284,'','SAGE-3 Wind 5','','','Aeron',409,'',118 from dual union all
select 67832,'2003-Y43','2003-01-22 13:00:00','suborbital rocket','success',1284,'','SAGE-3 Wind 7','','','Aeron',409,'',112 from dual union all
select 67833,'2003-Y44','2003-01-22 13:35:00','suborbital rocket','success',1284,'','SAGE-3 Wind 8','','','Aeron',409,'',115 from dual union all
select 67834,'2003-Y45','2003-01-23 13:00:00','suborbital rocket','success',1284,'','SAGE-3 Wind 9','','','Aeron',409,'',113 from dual union all
select 67835,'2003-Y46','2003-01-23 13:15:00','suborbital rocket','success',1284,'','SAGE-3 Wind 10','','','Aeron',409,'',115 from dual union all
select 67836,'2003-Y47','2003-01-23 13:50:00','suborbital rocket','success',1284,'','SAGE-3 Wind 11','','','Aeron',409,'',89 from dual union all
select 67837,'2003-Y48','2003-01-24 13:00:00','suborbital rocket','success',1284,'','SAGE-3 Wind 12','','','Aeron',409,'',115 from dual union all
select 67838,'2003-Y49','2003-01-26 13:00:00','suborbital rocket','success',1284,'ES03SL26','MM-LFS-','','','Aeron',409,'',111 from dual union all
select 67839,'2003-Y50','2003-01-27 13:00:00','suborbital rocket','success',1284,'ES03SL27','MM-LFS-','','','Aeron',409,'',120 from dual union all
select 6784,'1960-S140','1960-06-23 05:52:00','military missile','success',928,'A1X-34','','','','Test',245,'',500 from dual union all
select 67840,'2003-Y51','2003-01-27 15:05:00','suborbital rocket','success',1284,'ES03SL28','MM-LFS-','','','Aeron',409,'',99 from dual union all
select 67841,'2003-Y52','2003-01-27 15:45:00','suborbital rocket','success',1284,'ES03SL29','MM-LFS-','','','Aeron',409,'',115 from dual union all
select 67842,'2003-Y53','2003-01-27 16:33:00','suborbital rocket','success',1284,'ES03SL30','MM-LFS-','','','Aeron',409,'',115 from dual union all
select 67843,'2003-Y54','2003-01-27 19:26:00','suborbital rocket','success',1284,'ES03SL31','MM-LFS-','','','Aeron',409,'',117 from dual union all
select 67844,'2003-Y55','2003-01-27 19:38:00','suborbital rocket','success',1284,'ES03SL32','MM-LFS-','','','Aeron',409,'',115 from dual union all
select 67845,'2004-001','2004-01-11 04:12:59','orbital','success',1542,'SL12/DM-SL-9L','SL12 Telstar 14','Telstar 14','','Sat',860,'ODYSSEY',null from dual union all
select 67846,'2004-002','2004-01-29 11:58:08','orbital','success',1241,'D 15000-683 683','Progress M1-11','Progress M1 No. 260','','Sat ISS 13P',480,'',null from dual union all
select 67847,'2004-003','2004-02-05 23:46:02','orbital','success',174,'AC-165','AMC-10','AMC-10','','Sat',288,'',null from dual union all
select 67848,'2004-004','2004-02-14 18:50:00','orbital','success',1412,'4B-39','USA 176      [DSP22]','DSP F22','','Sat',291,'',null from dual union all
select 67849,'2004-005','2004-02-18 07:05:55','orbital','success',805,'76092  -690 047','Molniya-1T','Molniya-1T?','','Sat',462,'',null from dual union all
select 6785,'1960-S141','1960-06-23 05:00:00','suborbital rocket','success',972,'','','','','Aeron/Ionos',636,'',200 from dual union all
select 67850,'2004-006','2004-03-02 07:17:44','deep space','success',115,'V158 (518)','Rosetta','Rosetta','','Sat',346,'',null from dual union all
select 67851,'2004-007','2004-03-13 05:40:01','orbital','success',139,'AC-202','MBSAT','MBSAT','','Sat',289,'',null from dual union all
select 67852,'2004-008','2004-03-15 23:06:00','orbital','success',962,'535-03/88507','Eutelsat W3A','Eutelsat W3A','','Sat',588,'',null from dual union all
select 67853,'2004-009','2004-03-20 17:53:00','orbital','success',427,'D303','USA 177 (GPS IIR-11)','Navstar GPS SVN 59','','Sat',286,'',null from dual union all
select 67854,'2004-010','2004-03-27 03:30:00','orbital','success',958,'410-05','Raduga-1','Globus No. 17L','','Sat',587,'',null from dual union all
select 67855,'2004-011','2004-04-16 00:45:00','orbital','success',174,'AC-163','Superbird 6','Superbird 6','','Sat',288,'',null from dual union all
select 67856,'2004-012','2004-04-18 15:59:04','orbital','success',301,'Y14      (76)','Shiyan-1/NX-1','Shiyan 1','','Sat',1902,'',null from dual union all
select 67857,'2004-013','2004-04-19 03:19:00','orbital','success',1233,'Zh15000-009 009','Soyuz TMA-4','Soyuz 11F732 No. 214','','Sat ISS 8S',480,'',null from dual union all
select 67858,'2004-014','2004-04-20 16:57:24','orbital','success',418,'D304','GP-B','GP-B','','Sat',1724,'',null from dual union all
select 67859,'2004-015','2004-04-26 20:37:00','orbital','success',959,'410-06','Ekspress AM-11','Ekspress AM-11','','Sat',553,'',null from dual union all
select 6786,'1960-S142','1960-06-23 23:00:00','suborbital rocket','success',994,'No. 13','','','','Ionos/Aeron/Bio',636,'',200 from dual union all
select 67860,'2004-016','2004-05-04 12:42:00','orbital','success',1542,'SL14/DM-SL-16L','SL13 DirecTV 7S','DirecTV 7S','','Sat',860,'ODYSSEY',null from dual union all
select 67861,'2004-017','2004-05-19 22:22:01','orbital','success',174,'AC-166','AMC-11','Americom 11','','Sat',289,'',null from dual union all
select 67862,'2004-018','2004-05-20 17:47:03','orbital','success',1317,'T7 (XL)','ROCSAT-2','ROCSAT-2','','Sat',1670,'',null from dual union all
select 67863,'2004-019','2004-05-25 12:34:23','orbital','success',1241,'D 15000-684 684','Progress M-49','Progress M No. 249','','Sat ISS 14P',480,'',null from dual union all
select 67864,'2004-020','2004-05-28 06:00:00','orbital','success',1446,'670-92-  803','Kosmos-2405','US-PU','','Sat',591,'',null from dual union all
select 67865,'2004-021','2004-06-10 01:28:00','orbital','success',1537,'70121-402/1-95','Kosmos-2406','Tselina-2','','Sat',567,'',null from dual union all
select 67866,'2004-022','2004-06-16 22:27:00','orbital','success',962,'535-06/88509','Intelsat 10-02','INTELSAT 10-02','','Sat',553,'',null from dual union all
select 67867,'2004-023','2004-06-23 22:54:00','orbital','success',427,'D305','USA 178 (GPS IIR-12)','Navstar GPS SVN 60','','Sat',286,'',null from dual union all
select 67868,'2004-024','2004-06-29 03:58:59','orbital','success',1542,'SL11/DM-SL-11L','SL20 Telstar 18','Apstar 5','','Sat',860,'ODYSSEY',null from dual union all
select 67869,'2004-025','2004-06-29 06:30:06','orbital','success',434,'450-3341-631','Demeter/Latinsat/','Latinsat D','','Sat SSO',497,'',null from dual;
insert into LAUNCH
select 6787,'1960-S143','1960-06-24 00:00:00','military missile','success',976,'','8K63 UBP-51','','','Test',636,'',402 from dual union all
select 67870,'2004-026','2004-07-15 10:01:59','orbital','success',419,'D306','Aura','EOS Chem-1','','Sat',1724,'',null from dual union all
select 67871,'2004-027','2004-07-18 00:44:00','orbital','success',115,'V163 (519f)','Anik F2','Anik F2','','Sat',346,'',null from dual union all
select 67872,'2004-028','2004-07-22 17:46:28','orbital','success',680,'53779-805','Kosmos-2407','Parus','','Sat',456,'',null from dual union all
select 67873,'2004-029','2004-07-25 07:05:00','orbital','success',297,'Y2       (77)','TC-2','Double Star DSP-P','','Sat',1492,'',null from dual union all
select 67874,'2004-030','2004-08-03 06:15:57','deep space','success',428,'D307','Messenger','MESSENGER','','Sat',286,'',null from dual union all
select 67875,'2004-031','2004-08-04 22:32:00','orbital','success',962,'535-07/88508','Amazonas','Amazonas','','Sat',553,'',null from dual union all
select 67876,'2004-032','2004-08-11 05:03:07','orbital','success',1238,'D 15000-685 685','Progress M-50','Progress 7K-TGM No. 350','','Sat ISS 15P',480,'',null from dual union all
select 67877,'2004-033','2004-08-29 07:50:05','orbital','success',301,'Y12      (78)','FSW 19','FSW-3?','','Sat',813,'',null from dual union all
select 67878,'2004-034','2004-08-31 23:17:00','orbital','success',174,'AC-167','USA 179      [SDS15]','QUASAR 15?','','Sat',288,'',null from dual union all
select 67879,'2004-035','2004-09-08 23:14:00','orbital','success',319,'Y7       (79)','SJ-6A/SJ-6B','shijian liuhao B xing','','Sat',1492,'',null from dual union all
select 6788,'1960-S144','1960-06-25 00:04:00','military missile','success',1419,'G-10','Mk 4-1 RV','','','Test',230,'',1000 from dual union all
select 67880,'2004-036','2004-09-20 10:31:00','orbital','success',502,'GSLV-F01','GSAT-3','GSAT-3','','Sat',1357,'',null from dual union all
select 67881,'2004-037','2004-09-23 15:07:36','orbital','success',680,'53779-811','Kosmos-2408/2409','Strela-3','','Sat',456,'',null from dual union all
select 67882,'2004-038','2004-09-24 16:50:00','orbital','success',1241,'78036  -844 087','Kosmos-2410','Kobal''t-M','','Sat',462,'',null from dual union all
select 67883,'2004-039','2004-09-27 08:00:04','orbital','success',303,'Y5       (80)','FSW 20','FSW-3?','','Sat',813,'',null from dual union all
select 67884,'2004-040','2004-10-14 03:06:26','orbital','success',1233,'Zh15000-012 012','Soyuz TMA-5','Soyuz 11F732 No. 215','','Sat ISS 9S',480,'',null from dual union all
select 67885,'2004-041','2004-10-14 21:23:00','orbital','success',962,'535-08/88510','AMC-15','AMC-15','','Sat',553,'',null from dual union all
select 67886,'2004-042','2004-10-19 01:20:05','orbital','success',309,'Y9       (81)','FY-2C','FY-2 O4','','Sat',1903,'',null from dual union all
select 67887,'2004-043','2004-10-29 22:11:00','orbital','success',959,'410-08','Ekspress AM-1','Ekspress AM-1','','Sat',553,'',null from dual union all
select 67888,'2004-044','2004-11-06 03:10:00','orbital','success',319,'Y8       (82)','ZY-2C','ziyuan erhao 03 xing','','Sat',1492,'',null from dual union all
select 67889,'2004-045','2004-11-06 05:39:00','orbital','success',427,'D308','USA 180 (GPS IIR-13)','Navstar GPS SVN 61','','Sat',286,'',null from dual union all
select 6789,'1960-S145','1960-06-24 06:25:00','suborbital rocket','success',35,'NASA 04.06GG','UV star photometer','','','UV Astron',1821,'',211 from dual union all
select 67890,'2004-046','2004-11-18 10:45:00','orbital','success',301,'Y15      (83)','SW-2','Shiyan 2','','Sat',1902,'',null from dual union all
select 67891,'2004-047','2004-11-20 17:16:01','orbital','success',411,'D309','Swift','Swift GRB Explorer','','Sat',285,'',null from dual union all
select 67892,'2004-048','2004-12-17 12:07:00','orbital','success',191,'AV-005','AMC 16','AMC 16','','Sat',292,'',null from dual union all
select 67893,'2004-049','2004-12-18 16:26:00','orbital','success',115,'V165 (520g)','Helios-2A/ASAP','Helios 2A','','Sat',346,'',null from dual union all
select 67894,'2004-050','2004-12-21 21:50:00','orbital','failure',397,'D4-4 (310) 4050H','Demosat','HLV-OLDSP','','Sat',290,'',null from dual union all
select 67895,'2004-051','2004-12-23 22:19:34','orbital','success',1241,'Zh15000-092 092','Progress M-51','Progress 7K-TGM No. 351','','Sat ISS 16P',480,'',null from dual union all
select 67896,'2004-052','2004-12-24 11:20:00','orbital','failure',1448,'450-43-  701','Sich-1M','Okean-O1 No. 9','','Sat',470,'',null from dual union all
select 67897,'2004-053','2004-12-26 13:53:31','orbital','success',958,'410-09','Kosmos-2411/2413','Uragan No. 796','','Sat',553,'',null from dual union all
select 67898,'2004-A01','2004-02-18 09:30:00','military missile','failure',1185,'VMF RF','','','','OT',201,'K-18',10 from dual union all
select 67899,'2004-A02','2004-03-11 00:00:00','suborbital spaceplane','success',1245,'49L/12G','','','','Test',955,'WK1',14 from dual union all
select 679,'1947-A15','1947-08-01 00:00:00','atmospheric rocket','success',365,'','Test','','','Test',1821,'',10 from dual union all
select 6790,'1960-S146','1960-06-26 06:17:00','suborbital rocket','success',1438,'TB Ib','D58 Trailblazer','','','Reentry Test',1821,'',294 from dual union all
select 67900,'2004-A03','2004-03-27 21:59:57','atmospheric rocket','success',590,'2','X-43A','','','Test',379,'NB-52 008',29 from dual union all
select 67901,'2004-A04','2004-04-08 15:11:00','suborbital spaceplane','success',1245,'53L/13P','','','','Test',955,'WK1',32 from dual union all
select 67902,'2004-A05','2004-05-18 00:00:00','atmospheric rocket','success',342,'','GQM-163A Test','','','Test',1057,'',1 from dual union all
select 67903,'2004-A06','2004-06-09 10:35:00','suborbital rocket','success',891,'NASA 30.60NO','Student rocket','','','Education',1821,'',43 from dual union all
select 67904,'2004-A07','2004-07-26 00:00:00','military missile','failure',615,'','Julang-2','','','Test',1909,'XIA',10 from dual union all
select 67905,'2004-A08','2004-07-29 17:25:00','military missile','success',119,'F12','USFT-1','','','ATBM Test',1415,'',40 from dual union all
select 67906,'2004-A09','2004-08-26 17:30:00','military missile','success',119,'F13','USFT-2','','','ATBM Test',1415,'',40 from dual union all
select 67907,'2004-A10','2004-08-27 00:00:00','atmospheric rocket','success',342,'','GQM-163A Test','','','Test',1057,'',1 from dual union all
select 67908,'2004-A11','2004-09-23 00:00:00','military missile','success',261,'','Popup test','','','Test',201,'TK-208',1 from dual union all
select 67909,'2004-A12','2004-11-16 22:35:00','atmospheric rocket','success',590,'3','X-43A','','','Test',379,'NB-52 008',33 from dual union all
select 6791,'1960-S147','1960-06-27 00:00:00','suborbital rocket','success',838,'','','','','Ionos',293,'',150 from dual union all
select 67910,'2004-A13','2004-11-24 03:00:00','suborbital rocket','failure',1345,'NASA 12.061GT','','','','Test',1895,'',20 from dual union all
select 67911,'2004-A15','2004-12-14 00:00:00','atmospheric rocket','success',342,'','GQM-163A Test','','','Test',1057,'',1 from dual union all
select 67912,'2004-A16','2004-04-16 16:56:00','suborbital rocket','success',1345,'FASTT SPV-2','Dummy Scramjet','','','Test',1821,'',20 from dual union all
select 67913,'2004-A17','2004-04-30 00:00:00','military missile','success',2,'S-300','48N6DM','','','Test',636,'',40 from dual union all
select 67914,'2004-F01','2004-09-06 10:53:00','orbital','failure',1174,'6','''Ofeq-6','''Ofeq-6','','Sat',1026,'',260 from dual union all
select 67915,'2004-M01','2004-05-13 15:31:00','suborbital spaceplane','success',1245,'56L/14P','','','','Test',955,'WK1',64 from dual union all
select 67916,'2004-M02','2004-07-29 17:22:00','military missile','success',1151,'FMA','Arrow USFT-1 Target','','','Target',1057,'MLP',80 from dual union all
select 67917,'2004-M03','2004-09-10 11:00:00','suborbital rocket','success',1023,'','Middle Atmosphere','','','Aeron',1467,'',70 from dual union all
select 67918,'2004-M04','2004-09-24 00:00:00','suborbital rocket','success',1023,'','Middle Atmosphere','','','Aeron',1467,'',70 from dual union all
select 67919,'2004-M05','2004-09-29 00:50:00','suborbital rocket','success',1047,'','','','','Aeron',932,'',70 from dual union all
select 6792,'1960-S148','1960-06-28 05:10:00','suborbital rocket','success',838,'AA8.242','NAsp-II','','','Ionos',394,'',195 from dual union all
select 67920,'2004-M06','2004-09-29 02:25:00','suborbital rocket','success',1047,'','','','','Aeron',932,'',70 from dual union all
select 67921,'2004-M07','2004-10-06 00:00:00','suborbital rocket','success',1023,'','Middle Atmosphere','','','Aeron',1467,'',74 from dual union all
select 67922,'2004-M08','2004-11-07 06:50:00','military missile','success',430,'3','Ship test','','','Test',320,'INS-P51',50 from dual union all
select 67923,'2004-M09','2004-07-23 06:45:00','suborbital rocket','success',1023,'','Meso Turb Chaff','','','Aeron',1361,'',76 from dual union all
select 67924,'2004-M10','2004-03-19 04:40:00','military missile','success',941,'','Prithvi-3','','','Test',320,'',50 from dual union all
select 67925,'2004-M11','2004-10-27 05:59:00','military missile','success',941,'','Prithvi-3 (Dhanush)','','','Test',315,'',50 from dual union all
select 67926,'2004-M12','2004-07-10 00:00:00','suborbital rocket','success',520,'','ANSR Boost Test','','','Test',1850,'',50 from dual union all
select 67927,'2004-M13','2004-11-29 00:00:00','military missile','success',510,'Hatf III','','','','Test',1419,'',50 from dual union all
select 67928,'2004-S01','2004-01-09 18:40:00','military missile','success',499,'BV-5+','GMDS BVT-5 ''BV+''','','','Test',1707,'',200 from dual union all
select 67929,'2004-S02','2004-01-16 12:29:22','suborbital rocket','success',224,'NASA 36.210UL','BDPR/VULCAM','','','UV Astron',1895,'',263 from dual union all
select 6793,'1960-S149','1960-06-27 13:15:00','suborbital rocket','success',30,'NC3.108','NRL-58','','','Aeron',1869,'',222 from dual union all
select 67930,'2004-S03','2004-01-17 15:30:00','suborbital rocket','success',1073,'S-310-33','WAVE2004','','','Aeron',833,'',141 from dual union all
select 67931,'2004-S04','2004-01-22 08:57:00','suborbital rocket','success',209,'NASA 35.035UE','SERSIO','','','Plasma',1453,'',782 from dual union all
select 67932,'2004-S05','2004-01-23 09:09:00','military missile','success',941,'23','','','','Test',320,'',100 from dual union all
select 67933,'2004-S06','2004-01-27 02:23:00','military missile','success',497,'3','IFT-13b','','','Test',869,'',272 from dual union all
select 67934,'2004-S07','2004-02-18 09:00:00','military missile','success',1454,'511-7822-121','15A35P LKI-6','Yu-70/102E','','OT',596,'',1000 from dual union all
select 67935,'2004-S08','2004-02-18 10:28:00','military missile','success',1434,'','15Zh58 UBP-30','Bezopasnost''-2004','','OT/R&D?',608,'',1000 from dual union all
select 67936,'2004-S09','2004-02-26 00:31:20','military missile','success',1443,'FCET-31','','','','OT',420,'SSBN 739',1000 from dual union all
select 67937,'2004-S10','2004-02-26 01:33:51','military missile','success',1443,'FCET-31','','','','OT',420,'SSBN 739',1000 from dual union all
select 67938,'2004-S11','2004-03-09 00:00:00','military missile','success',1171,'','','','','Test',1419,'',300 from dual union all
select 67939,'2004-S12','2004-03-17 12:08:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-407',1000 from dual union all
select 6794,'1960-S15','1960-01-19 18:18:00','suborbital rocket','success',30,'AA3.124C','EUV monochromator','','','Solar EUV',1869,'',210 from dual union all
select 67940,'2004-S13','2004-03-17 14:00:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-407',1000 from dual union all
select 67941,'2004-S14','2004-04-13 03:30:00','suborbital rocket','success',1341,'NASA 12.54GT','NMACS Test','','','Test',1895,'',122 from dual union all
select 67942,'2004-S15','2004-04-20 17:30:00','military missile','success',1437,'','15Zh55 LKI-3','','','OT',608,'',1000 from dual union all
select 67943,'2004-S16','2004-05-03 04:06:00','military missile','success',691,'','LRALT Demo','','','Test',958,'C-17',300 from dual union all
select 67944,'2004-S17','2004-05-17 18:12:00','suborbital rocket','success',271,'','GoFast','','','Test',208,'',116 from dual union all
select 67945,'2004-S18','2004-05-29 00:00:00','military missile','success',509,'Hatf-V','','','','Test',1473,'',150 from dual union all
select 67946,'2004-S19','2004-06-01 07:50:00','suborbital rocket','success',224,'NASA 36.209UG','ISIS (Cyg Loop)','','','UV Astron',1895,'',300 from dual union all
select 67947,'2004-S20','2004-06-01 22:01:00','military missile','success',757,'5','ME-34 Teutates','','','Test',206,'S618',800 from dual union all
select 67948,'2004-S21','2004-06-04 00:00:00','military missile','success',509,'Hatf-V','','','','Test',1473,'',150 from dual union all
select 67949,'2004-S22','2004-06-21 14:50:00','suborbital spaceplane','success',1245,'60L/15P','','','','Test',956,'WK1',100 from dual union all
select 6795,'1960-S150','1960-06-27 19:10:00','suborbital rocket','failure',838,'AA8.243','NAsp-III [Not ASE!]','','','Ionos',394,'',170 from dual union all
select 67950,'2004-S23','2004-06-23 08:32:00','military missile','success',799,'','Mk 12/12A RV','FDEP GT185GM','GLORY TRIP 185GM','Op Test',1706,'',1300 from dual union all
select 67951,'2004-S24','2004-06-29 07:25:00','military missile','success',1179,'','','','','OT',201,'K-84',1000 from dual union all
select 67952,'2004-S25','2004-07-04 04:40:00','military missile','success',62,'','Agni A-1','','','Test',316,'',300 from dual union all
select 67953,'2004-S26','2004-07-21 08:01:00','military missile','success',907,'33PA (84-0554)','Mk 21 RV x 8','FDEP GT 33PA','','OT',1700,'',1000 from dual union all
select 67954,'2004-S27','2004-07-23 08:37:00','military missile','success',799,'','Mk 12/12A RV','FDEP GT184GM-1','GLORY TRIP 184GM-1','Op Test',1705,'',1300 from dual union all
select 67955,'2004-S28','2004-08-07 08:52:55','suborbital rocket','success',1337,'NASA 29.037UE','EQUIS II','','','Ionos',873,'',450 from dual union all
select 67956,'2004-S29','2004-08-07 09:07:30','suborbital rocket','success',1334,'NASA 41.043UE','EQUIS II','','','Ionos',873,'',130 from dual union all
select 67957,'2004-S30','2004-08-07 09:11:30','suborbital rocket','success',1334,'NASA 41.045UE','EQUIS II','','','Ionos',873,'',130 from dual union all
select 67958,'2004-S31','2004-08-09 08:15:00','suborbital rocket','success',1073,'S-310-34','Solar Sails','','','Tech',833,'',170 from dual union all
select 67959,'2004-S32','2004-08-11 00:00:00','military missile','success',508,'Improved Shahab 3','','','','Test',1356,'',150 from dual;
insert into LAUNCH
select 6796,'1960-S151','1960-06-28 02:30:54','military missile','success',154,'27D','Mk 3-2B-213B/AFCRC rad','D/RIG test/radiometer','','Ionos',227,'',1152 from dual union all
select 67960,'2004-S33','2004-08-11 10:25:00','military missile','success',1453,'492-3821-128','15A35 UBP-52','','','OT',596,'',1000 from dual union all
select 67961,'2004-S34','2004-08-15 08:21:53','suborbital rocket','success',1337,'NASA 29.036UE','EQUIS II','','','Ionos',873,'',450 from dual union all
select 67962,'2004-S35','2004-08-15 08:47:00','suborbital rocket','success',1334,'NASA 41.044UE','EQUIS II','','','Ionos',873,'',130 from dual union all
select 67963,'2004-S36','2004-08-15 08:51:00','suborbital rocket','failure',1334,'NASA 41.046UE','EQUIS II','','','Ionos',873,'',100 from dual union all
select 67964,'2004-S37','2004-08-27 22:42:43','suborbital rocket','success',1334,'NASA 41.041UE','EQUIS II LEMMA 1','','','Ionos',873,'',130 from dual union all
select 67965,'2004-S38','2004-08-29 07:26:00','military missile','success',63,'','OT','','','Test',316,'',1000 from dual union all
select 67966,'2004-S39','2004-09-03 20:55:10','suborbital rocket','success',211,'NASA 27.145UE','EQUIS II','','','Ionos',873,'',340 from dual union all
select 67967,'2004-S40','2004-09-07 11:45:53','suborbital rocket','success',228,'NASA 21.132GE','EQUIS II','','','Ionos',873,'',114 from dual union all
select 67968,'2004-S41','2004-09-07 23:56:00','suborbital rocket','success',1334,'NASA 41.048UE','EQUIS II','','','Ionos',873,'',130 from dual union all
select 67969,'2004-S42','2004-09-08 05:38:00','military missile','success',1179,'','','','','OT',201,'K-84',1000 from dual union all
select 6797,'1960-S152','1960-06-29 00:00:00','military missile','success',976,'','8K63 UBP-52','','','Test',636,'',402 from dual union all
select 67970,'2004-S43','2004-09-08 07:38:00','military missile','success',1507,'','','','','OT',201,'K-496',1000 from dual union all
select 67971,'2004-S44','2004-09-15 08:01:00','military missile','success',799,'','Mk 12/12A RV x 3','FDEP GT186GB','GLORY TRIP 186GB','Op Test',1712,'',1300 from dual union all
select 67972,'2004-S45','2004-09-17 08:00:45','suborbital rocket','success',228,'NASA 21.133GE','EQUIS II','','','Ionos',873,'',117 from dual union all
select 67973,'2004-S46','2004-09-17 08:12:00','suborbital rocket','success',1334,'NASA 41.047UE','EQUIS II','','','Ionos',873,'',130 from dual union all
select 67974,'2004-S47','2004-09-17 09:02:01','suborbital rocket','success',211,'NASA 27.146UE','EQUIS II','','','Ionos',873,'',340 from dual union all
select 67975,'2004-S48','2004-09-19 23:30:00','suborbital rocket','success',1334,'NASA 41.042UE','EQUIS II LEMMA 2','','','Ionos',873,'',131 from dual union all
select 67976,'2004-S49','2004-09-23 03:00:00','suborbital rocket','failure',1341,'NASA 12.56GT','NMACS Test','','','Test',1895,'',120 from dual union all
select 67977,'2004-S51','2004-09-29 00:00:00','military missile','success',1097,'','CMP-4A (TCMP)','','','Target',1784,'',200 from dual union all
select 67978,'2004-S53','2004-09-29 15:09:53','suborbital spaceplane','success',1245,'65L/16P','','','','Test',956,'WK1',103 from dual union all
select 67979,'2004-S54','2004-09-29 22:00:00','suborbital rocket','success',1047,'','[unconfirmed]','','','Ionos?',932,'',100 from dual union all
select 6798,'1960-S153','1960-07-01 00:00:00','suborbital rocket','success',673,'','PHOENIX 5','','','Test',1057,'',325 from dual union all
select 67980,'2004-S55','2004-09-30 03:00:00','suborbital rocket','success',1047,'','[unconfirmed]','','','Ionos?',932,'',100 from dual union all
select 67981,'2004-S56','2004-10-04 14:49:11','suborbital spaceplane','success',1245,'66L/17P','','','','Test',956,'WK1',112 from dual union all
select 67982,'2004-S57','2004-10-06 00:00:00','military missile','success',1097,'','CMP-4B (TCMP)','','','Target',1784,'',200 from dual union all
select 67983,'2004-S58','2004-10-12 00:00:00','military missile','success',509,'Hatf-V','','','','Test',1473,'',150 from dual union all
select 67984,'2004-S59','2004-10-15 17:28:33','suborbital rocket','success',224,'NASA 36.217UE','LASP/CU-10 (SEE)','','','Solar EUV',1895,'',300 from dual union all
select 67985,'2004-S60','2004-10-20 00:00:00','military missile','success',508,'','','','','Test',1356,'',150 from dual union all
select 67986,'2004-S61','2004-10-23 16:51:00','suborbital rocket','success',1484,'XV-01','Cajuana','','','Test',13,'',240 from dual union all
select 67987,'2004-S62','2004-10-28 19:49:00','suborbital rocket','success',605,'SSC REXUS II','EURO REXUS-2/ECOMA','','','Education/Meteor',412,'',91 from dual union all
select 67988,'2004-S63','2004-11-02 09:25:00','military missile','success',1507,'','','','','OT',1011,'K-433',1000 from dual union all
select 67989,'2004-S64','2004-11-02 09:52:00','military missile','success',1434,'','15Zh58 UBP-31','','','OT',608,'',1000 from dual union all
select 6799,'1960-S154','1960-07-01 00:00:00','military missile','success',976,'','8K63 UBP-53','','','Test',636,'',402 from dual union all
select 67990,'2004-S65','2004-11-18 00:00:00','military missile','success',1263,'PAC-3 DT/OT-12','MTTV-7','','','Target',437,'',100 from dual union all
select 67991,'2004-S66','2004-11-22 08:35:00','suborbital rocket','success',772,'MAXUS 6','SSC MAXUS 6','','','Micrograv',407,'',706 from dual union all
select 67992,'2004-S67','2004-11-29 08:00:00','military missile','success',3,'','Test','','','Op Test',183,'',40 from dual union all
select 67993,'2004-S68','2004-12-02 10:30:00','suborbital rocket','success',1212,'TEXUS 41','DLR TEXUS 41','','','Micrograv',416,'',227 from dual union all
select 67994,'2004-S69','2004-12-13 00:33:00','suborbital rocket','success',1073,'S-310-35','DELTA 2004','','','Aeron',28,'',140 from dual union all
select 67995,'2004-S70','2004-12-14 14:30:00','suborbital rocket','success',1221,'4','Space science','','','Ionos/Aeron',808,'',265 from dual union all
select 67996,'2004-S71','2004-12-15 05:45:00','military missile','success',1111,'M-5','IFT-13c Target','','','Target',862,'',1000 from dual union all
select 67997,'2004-S72','2004-12-22 08:30:20','military missile','success',1005,'450-7951-210','15A18M UBP 13 RD','','','Test',1908,'',1000 from dual union all
select 67998,'2004-S73','2004-12-24 09:39:00','military missile','success',1437,'00354-416-30(3L)','15Zh55 LKI-4','','','OT',608,'',1000 from dual union all
select 67999,'2004-S74','2004-07-23 06:12:00','suborbital rocket','success',1026,'','MTS Meso. Turbulence','','','Aeron',1361,'',109 from dual union all
select 68,'1944-A32','1944-12-28 11:37:00','military missile','failure',1470,'20848','Ma427','','','Test',661,'',20 from dual union all
select 680,'1947-A16','1947-09-06 15:53:00','military missile','failure',1470,'','Operation Sandy','','','Test',36,'CV-41',1 from dual union all
select 6800,'1960-S155','1960-07-01 00:00:00','military missile','success',976,'','8K63 UBP-54','','','Test',636,'',402 from dual union all
select 68000,'2004-S75','2004-01-01 00:00:00','military missile','success',354,'','','','','Test',1494,'',1000 from dual union all
select 68001,'2004-S76','2004-11-04 00:00:00','military missile','success',1443,'FCET-32','-    (107ss)','','','OT',1897,'SSBN 733',1000 from dual union all
select 68002,'2004-S77','2004-11-04 00:00:00','military missile','success',1443,'FCET-32','-    (108ss)','','','OT',1897,'SSBN 733',1000 from dual union all
select 68003,'2004-S78','2004-07-16 00:00:00','military missile','success',17,'Pacific Explorer III','ARAV-ER/Star 20','','','Target',855,'',200 from dual union all
select 68004,'2004-S79','2004-09-19 22:50:00','suborbital rocket','success',1502,'','Sphere','','','Aeron',873,'',85 from dual union all
select 68005,'2004-S80','2004-09-20 00:01:00','suborbital rocket','success',1502,'','Sphere','','','Aeron',873,'',85 from dual union all
select 68006,'2004-S81','2004-09-20 02:00:00','suborbital rocket','success',1502,'','Sphere','','','Aeron',873,'',85 from dual union all
select 68007,'2004-S82','2004-12-08 00:00:00','military missile','success',1169,'Hatf IV','','','','Test',1419,'',100 from dual union all
select 68008,'2004-U01','2004-11-08 18:29:58','orbital','success',1227,'F 15000-001 001','Oblik 17F116ML','Oblik 17F116ML No. 103','','Sat',475,'',null from dual union all
select 68009,'2004-Y01','2004-09-01 14:30:00','suborbital rocket','success',1165,'Shadow 1B','PWN-10X','','','Edu',295,'',110 from dual union all
select 6801,'1960-S156','1960-07-14 00:00:00','military missile','success',976,'','8K63 UBP-57 29 RD','','','Test',636,'',402 from dual union all
select 68010,'2005-001','2005-01-12 18:47:08','deep space','success',427,'D311','Deep Impact','Deep Impact','','Sat',286,'',null from dual union all
select 68011,'2005-002','2005-01-20 03:00:07','orbital','success',680,'47106-411','Kosmos-2414','Parus?','','Sat',456,'',null from dual union all
select 68012,'2005-003','2005-02-03 02:27:32','orbital','success',962,'535-09/88511','AMC-12','Americom 12','','Sat',588,'',null from dual union all
select 68013,'2005-004','2005-02-03 07:41:01','orbital','success',141,'AC-206','USA 181     [NOSS16]','INTRUDER 7','','Sat',289,'',null from dual union all
select 68014,'2005-005','2005-02-12 21:03:01','orbital','success',111,'V164 (521)','XTAR-EUR/Maqsat-B2','XTAR-EUR','','Sat',346,'',null from dual union all
select 68015,'2005-006','2005-02-26 09:25:00','orbital','success',529,'H-IIA-7F','MTSAT-1R','MTSAT-1R','','Sat',1477,'',null from dual union all
select 68016,'2005-007','2005-02-28 19:09:18','orbital','success',1241,'Zh15000-093 093','Progress M-52','TEKh-42','','Sat ISS 17P',480,'',null from dual union all
select 68017,'2005-008','2005-03-01 03:50:59','orbital','success',1542,'SL17/DM-SL-17L','SL18 XM Radio-3','XM Radio-3','','Sat',860,'ODYSSEY',null from dual union all
select 68018,'2005-009','2005-03-11 21:42:00','orbital','success',188,'AV-004','Inmarsat 4 F-1','INMARSAT 4 F1','','Sat',292,'',null from dual union all
select 68019,'2005-010','2005-03-29 21:31:00','orbital','success',959,'410-10','Ekspress AM-2','Ekspress AM-2','','Sat',553,'',null from dual union all
select 6802,'1960-S157','1960-07-01 00:00:00','military missile','success',976,'','8K63 UBP-58','','','Test',636,'',402 from dual union all
select 68020,'2005-011','2005-04-11 13:35:00','orbital','success',783,'3','XSS-11','USA 165 (XSS-11)','','Sat',1781,'',null from dual union all
select 68021,'2005-012','2005-04-12 12:00:00','orbital','success',313,'Y6       (84)','Apstar 6','Apstar 6','','Sat',1903,'',null from dual union all
select 68022,'2005-013','2005-04-15 00:46:25','orbital','success',1233,'Zh15000-014 014','Soyuz TMA-6','Soyuz 11F732 No. 216','','Sat ISS 10S',480,'',null from dual union all
select 68023,'2005-014','2005-04-15 17:26:50','orbital','success',912,'F36','DART','DART/HAPS','','Sat',1718,'L-1011',null from dual union all
select 68024,'2005-015','2005-04-26 07:31:29','orbital','success',1542,'SL18/DM-SL-19L','SL15 Spaceway 1','Spaceway 1','','Sat',860,'ODYSSEY',null from dual union all
select 68025,'2005-016','2005-04-30 00:50:00','orbital','success',1418,'4B-30','USA 182      [LAC5]','ONYX 5','','Sat',291,'',null from dual union all
select 68026,'2005-017','2005-05-05 04:45:00','orbital','success',899,'PSLV-C6','Cartosat','Cartosat-1','','Sat',1359,'',null from dual union all
select 68027,'2005-018','2005-05-20 10:22:01','orbital','success',411,'D312','NOAA N','NOAA N','','Sat',1724,'',null from dual union all
select 68028,'2005-019','2005-05-22 17:59:08','orbital','success',962,'535-10/88512','DirecTV-8','DirecTV-8','','Sat',553,'',null from dual union all
select 68029,'2005-020','2005-05-31 12:00:00','orbital','success',1241,'Zh15000-091 091','Foton-M No. 2','Foton-M No. 2','','Sat',480,'',null from dual union all
select 6803,'1960-S158','1960-07-01 00:00:00','military missile','success',1013,'','8K51 PRO Target C-3','NIIP-A Target','','Target',321,'',500 from dual union all
select 68030,'2005-021','2005-06-16 23:09:34','orbital','success',1241,'Zh15000-094 094','Progress M-53','Progress 7K-TGM No. 353/','','Sat ISS 18P',480,'',null from dual union all
select 68031,'2005-022','2005-06-23 14:03:00','orbital','success',1542,'SL4/DM-SL-13L','SL14 IA-8','Intelsat Americas 8','','Sat',860,'ODYSSEY',null from dual union all
select 68032,'2005-023','2005-06-24 19:41:00','orbital','success',958,'410-07','Ekspress AM-3','Ekspress AM-3','','Sat',553,'',null from dual union all
select 68033,'2005-024','2005-07-05 22:40:00','orbital','success',303,'Y6       (85)','SJ-7','Tansuo-2','','Sat',813,'',null from dual union all
select 68034,'2005-025','2005-07-10 03:30:00','orbital','success',730,'M-V-6','ASTRO E2','ASTRO-E2','','Sat',836,'',null from dual union all
select 68035,'2005-026','2005-07-26 14:39:00','orbital','success',1243,'STS-114','ISS LF-1','OV-103 ISS','','Sat',922,'',null from dual union all
select 68036,'2005-027','2005-08-02 07:30:03','orbital','success',301,'Y13      (86)','FSW 21','FSW-3?','','Sat',813,'',null from dual union all
select 68037,'2005-028','2005-08-11 08:20:44','orbital','success',116,'V166 (523g)','Thaicom 4','iPSTAR 1','','Sat',346,'',null from dual union all
select 68038,'2005-029','2005-08-12 11:43:00','deep space','success',182,'AV-007','MRO','MRO','','Sat',292,'',null from dual union all
select 68039,'2005-030','2005-08-13 23:28:27','orbital','success',1232,'Zh15000-011 011/ST13','Galaxy 14','Galaxy 14','','Sat',562,'',null from dual union all
select 6804,'1960-S159','1960-06-30 16:00:00','military missile','success',916,'109','P-05','','','Test',256,'',180 from dual union all
select 68040,'2005-031','2005-08-23 21:09:59','orbital','success',434,'510-3342-621','OICETS/INDEX','OICETS','','Sat SSO',497,'',null from dual union all
select 68041,'2005-032','2005-08-26 18:34:28','orbital','success',1064,'630-7823-115/72507','Monitor-E','98M','','Sat',460,'',null from dual union all
select 68042,'2005-033','2005-08-29 08:45:04','orbital','success',303,'Y7       (87)','FSW 22','FSW-3?','','Sat',813,'',null from dual union all
select 68043,'2005-034','2005-09-02 09:50:00','orbital','success',1241,'78036  -576 086','Kosmos-2415','Kometa No. 21?','','Sat',562,'',null from dual union all
select 68044,'2005-035','2005-09-08 13:07:54','orbital','success',1241,'P 15000-095 095','Progress M-54','Progress 7K-TGM No. 354/','','Sat ISS 19P',480,'',null from dual union all
select 68045,'2005-036','2005-09-08 21:53:40','orbital','success',962,'535-12/88513','Anik F1R','Anik F1R','','Sat',553,'',null from dual union all
select 68046,'2005-037','2005-09-23 02:24:29','orbital','success',783,'4','Streak','Streak (STP-R1)','','Sat',1781,'',null from dual union all
select 68047,'2005-038','2005-09-26 03:37:00','orbital','success',427,'D313','USA 183 (GPS IIRM-1)','Navstar GPS SVN 53','','Sat',285,'',null from dual union all
select 68048,'2005-039','2005-10-01 03:54:53','orbital','success',1233,'Zh15000-017 017','Soyuz TMA-7','Soyuz 11F732 No. 217','','Sat ISS 11S',480,'',null from dual;
insert into LAUNCH
select 68049,'2005-040','2005-10-12 01:00:00','orbital','success',306,'Y6       (88)','Shenzhou 6','Shenzhou 6 Fanhui Cang','','Sat',814,'',null from dual union all
select 6805,'1960-S16','1960-01-20 20:34:00','military missile','success',928,'A1X-9','','','','Test',251,'',500 from dual union all
select 68050,'2005-041','2005-10-13 22:32:00','orbital','success',116,'V168 (524g)','Syracuse 3A/Gal. 15','Galaxy 15','','Sat',346,'',null from dual union all
select 68051,'2005-042','2005-10-19 18:05:00','orbital','success',1416,'4B-26','USA 186      [IC8]','Improved CRYSTAL 2108?','','Sat',1777,'',null from dual union all
select 68052,'2005-043','2005-10-27 06:52:26','orbital','success',680,'47152-104','Topsat/DMC-3/Mozh.','China-DMC','','Sat',456,'',null from dual union all
select 68053,'2005-044','2005-11-08 14:06:59','orbital','success',1542,'SL19/DM-SL-20L','SL23 Inmarsat 4F2','INMARSAT 4 F2','','Sat',860,'ODYSSEY',null from dual union all
select 68054,'2005-045','2005-11-09 03:33:34','deep space','success',1232,'Zh15000-010 010/ST14','Venus Express','Venus Express','','Sat',562,'',null from dual union all
select 68055,'2005-046','2005-11-16 23:46:00','orbital','success',111,'V167 (522a)','Spaceway 2/Telkom 2','Telkom 2','','Sat',346,'',null from dual union all
select 68056,'2005-047','2005-12-21 18:38:20','orbital','success',1241,'F 15000-080 080','Progress M-55','Progress 7K-TGM No. 355/','','Sat ISS 20P',480,'',null from dual union all
select 68057,'2005-048','2005-12-21 19:34:20','orbital','success',680,'53727-232','Gonets/Kosmos-2416','Rodnik','','Sat',456,'',null from dual union all
select 68058,'2005-049','2005-12-21 22:33:00','orbital','success',116,'V169 (525)','Insat 4A/MSG 2','Insat 4A','','Sat',346,'',null from dual union all
select 68059,'2005-050','2005-12-25 05:07:10','orbital','success',958,'410-11','Kosmos-2417/2419','Uragan No. 798','','Sat',588,'',null from dual union all
select 6806,'1960-S160','1960-07-01 14:35:00','suborbital rocket','success',30,'AA3.107','','','','Meteorites',1869,'',162 from dual union all
select 68060,'2005-051','2005-12-28 05:19:08','orbital','success',1232,'P 15000-015 015/ST15','GIOVE A','GSTB-v2/A','','Sat',562,'',null from dual union all
select 68061,'2005-052','2005-12-29 02:28:40','orbital','success',962,'535-13/88514','AMC-23','Americom 23','','Sat',553,'',null from dual union all
select 68062,'2005-A01','2005-03-06 10:31:17','suborbital rocket','failure',237,'NASA 40.017UE','CASCADES','','','Auroral',1044,'',29 from dual union all
select 68063,'2005-A02','2005-03-24 00:00:00','atmospheric rocket','success',342,'','GQM-163A Test','','','Test',1057,'',1 from dual union all
select 68064,'2005-A03','2005-04-22 00:00:00','atmospheric rocket','success',342,'','GQM-163A Test','','','Test',1057,'',1 from dual union all
select 68065,'2005-A04','2005-06-09 10:32:00','suborbital rocket','success',605,'NASA 30.61NO','Student rocket','','','Education',1821,'',49 from dual union all
select 68066,'2005-A05','2005-10-09 21:36:00','atmospheric rocket','success',829,'','NEXST-1 F2','','','Test',1831,'',20 from dual union all
select 68067,'2005-A06','2005-12-02 08:28:00','military missile','success',119,'F14 BLK-3','AST  Black Sparrow','','','ATBM Test',1026,'',40 from dual union all
select 68068,'2005-A07','2005-12-10 11:30:00','suborbital rocket','success',1345,'','FASTT','','','Scramjet',1821,'',19 from dual union all
select 68069,'2005-A08','2005-12-02 08:27:00','military missile','success',244,'','','','','Test',1026,'F-15D',40 from dual union all
select 6807,'1960-S161','1960-07-02 00:04:00','ballistic missile test','failure',1138,'ST-1','','','','Test',1812,'',1380 from dual union all
select 68070,'2005-F01','2005-06-21 00:48:37','orbital','failure',805,'77046  -694 051','Molniya-3K','Molniya-3K No. 12L','','Sat',462,'',195 from dual union all
select 68071,'2005-F02','2005-06-21 19:46:09','orbital','failure',1507,'M-1491-035','Cosmos-1/Dem-2R','TPS Solar Sail','','Sat',201,'K-496',850 from dual union all
select 68072,'2005-F03','2005-10-08 15:02:00','orbital','failure',1064,'492-5882-030/72508','Cryosat','Cryosat','','Sat',460,'',700 from dual union all
select 68073,'2005-F04','2005-06-09 00:00:00','orbital','failure',652,'UNCONFIRMED','MS-1','MS-1?','','Test',1494,'',300 from dual union all
select 68074,'2005-M01','2005-06-01 01:00:00','suborbital rocket','success',1047,'','','','','Test',932,'',70 from dual union all
select 68075,'2005-M02','2005-09-09 02:45:00','suborbital rocket','success',605,'NASA 30.062DR','','Acoustic monitoring','','Calib',1895,'',50 from dual union all
select 68076,'2005-M03','2005-09-09 06:14:00','suborbital rocket','success',605,'NASA 30.063DR','','Acoustic monitoring','','Calib',1895,'',50 from dual union all
select 68077,'2005-M04','2005-05-12 07:34:00','military missile','success',941,'','Prithvi-3','','','Test',320,'',50 from dual union all
select 68078,'2005-M05','2005-12-28 08:28:00','military missile','success',430,'4','INS Rajput','','','Test',320,'INS-D15',50 from dual union all
select 68079,'2005-M06','2005-10-31 00:00:00','suborbital rocket','success',520,'','ANSR Boost Test','','','Test',1850,'',50 from dual union all
select 6808,'1960-S162','1960-07-02 06:58:22','military missile','success',154,'60D','Mk 3-1B-223','D/AIG guidance test CL2','','Test',226,'',901 from dual union all
select 68080,'2005-M07','2005-03-31 00:00:00','military missile','success',569,'','','','','Test',1419,'',70 from dual union all
select 68081,'2005-M08','2005-04-08 06:30:00','suborbital rocket','success',1023,'','Meso Turb Chaff','','','Aeron',1361,'',75 from dual union all
select 68082,'2005-S01','2005-12-01 09:04:00','suborbital rocket','success',1484,'V02','DLR TEXUS 42 (EML-1)','','','Micrograv',416,'',263 from dual union all
select 68083,'2005-S02','2005-01-10 04:37:29','suborbital rocket','success',605,'SSC MAGIC','SSC MAGIC 1','','','Meteorite dust',412,'',95 from dual union all
select 68084,'2005-S03','2005-01-20 16:00:00','suborbital rocket','success',847,'NASA 12.062GT','NIACS','','','Test',1895,'',90 from dual union all
select 68085,'2005-S04','2005-02-01 00:00:00','military missile','success',757,'6','','','','Test',206,'S616',800 from dual union all
select 68086,'2005-S05','2005-02-02 20:57:00','suborbital rocket','success',1345,'NASA 41.58DR','','','','Target?',855,'',130 from dual union all
select 68087,'2005-S06','2005-02-14 06:22:00','military missile','success',1111,'M-6','IFT-14 Target','','','Target',862,'',1000 from dual union all
select 68088,'2005-S07','2005-02-24 21:03:00','military missile','success',732,'Aegis FTM 04-1 Targ.','Aegis TTV-8','','','Target',855,'',150 from dual union all
select 68089,'2005-S08','2005-02-24 21:04:00','military missile','success',1090,'Aegis FTM 04-1 (FM7)','Stellar Dragon','','','Interceptor',851,'CG-70',150 from dual union all
select 6809,'1960-S163','1960-07-04 00:00:00','military missile','success',994,'','NIIP-A target','','','Target',634,'',100 from dual union all
select 68090,'2005-S09','2005-03-01 23:13:00','suborbital rocket','success',1345,'NASA 41.59DR','ARAV-A FTM-7 TRACKEX','','','Target?',855,'',130 from dual union all
select 68091,'2005-S10','2005-03-02 04:00:13','military missile','success',1443,'FCET-33','Short range test','','','OT',420,'SSBN 734',1000 from dual union all
select 68092,'2005-S11','2005-03-02 05:09:16','military missile','success',1443,'FCET-33','','','','OT',420,'SSBN 734',1000 from dual union all
select 68093,'2005-S12','2005-03-02 21:11:00','suborbital rocket','success',1345,'NASA 41.60DR','ARAV-A FTM-7 TRACKEX','','','Target?',855,'',130 from dual union all
select 68094,'2005-S13','2005-03-03 22:05:00','suborbital rocket','success',1344,'NASA 45.01DR','ARAV-B1 FTM-7 TRACKEX','','','Target?',855,'',300 from dual union all
select 68095,'2005-S14','2005-03-15 05:45:00','suborbital rocket','success',605,'NASA 30.58UE','DUST','','','Aeron/Dust',1043,'',105 from dual union all
select 68096,'2005-S15','2005-03-15 07:45:00','suborbital rocket','success',605,'NASA 30.59UE','DUST','','','Aeron/Dust',1042,'',105 from dual union all
select 68097,'2005-S16','2005-03-19 00:00:00','military missile','success',1171,'','','','','Test',1419,'',300 from dual union all
select 68098,'2005-S17','2005-04-08 17:30:00','military missile','success',748,'MRT','MRT-1 RRF','','','Test',1208,'C-17',500 from dual union all
select 68099,'2005-S18','2005-05-02 05:00:00','suborbital rocket','success',1212,'MASER 10','SSC MASER 10','','','Micrograv',416,'',252 from dual union all
select 681,'1947-A17','1947-09-22 00:00:00','atmospheric rocket','success',876,'47-E','Nike-47 Round D','','','Test',1863,'',10 from dual union all
select 6810,'1960-S164','1960-07-05 15:56:00','military missile','success',1017,'L1-7','8K74 LKI-7','','','Test',480,'',1350 from dual union all
select 68100,'2005-S19','2005-05-05 09:35:00','suborbital rocket','success',1334,'NASA 41.057DP','MCAFT-1/IBSi','','','Missile def/Bio',1821,'',156 from dual union all
select 68101,'2005-S20','2005-05-17 11:14:00','suborbital rocket','success',605,'NASA 30.56UO','MAGIC 2','','','Aeron',1821,'',90 from dual union all
select 68102,'2005-S21','2005-05-27 00:00:00','military missile','success',1153,'','','','','Test',960,'',100 from dual union all
select 68103,'2005-S22','2005-05-27 00:00:00','military missile','failure',1153,'','(Hit Turkey)','','','Test',960,'',100 from dual union all
select 68104,'2005-S23','2005-05-27 00:00:00','military missile','success',1152,'','','','','Test',960,'',100 from dual union all
select 68105,'2005-S24','2005-06-12 00:00:00','military missile','success',615,'','','','','Test',1909,'XIA',1000 from dual union all
select 68106,'2005-S25','2005-06-17 00:00:00','military missile','success',891,'','Target','','','Target',1895,'',85 from dual union all
select 68107,'2005-S26','2005-06-17 00:00:00','military missile','success',891,'','Target','','','Target',1895,'',85 from dual union all
select 68108,'2005-S27','2005-06-28 22:54:00','suborbital rocket','success',1326,'NASA 12.060GT','Test flight','','','Test',1821,'',275 from dual union all
select 68109,'2005-S28','2005-07-04 08:41:00','suborbital rocket','success',605,'HP-200 Hotel','IMEF','','','Aeron/Ionos',29,'',100 from dual union all
select 6811,'1960-S165','1960-07-06 00:00:00','military missile','success',1053,'CC-1019','Range 328 km','','','OT',437,'',90 from dual union all
select 68110,'2005-S29','2005-07-07 16:20:00','suborbital rocket','success',224,'NASA 36.222DS','VAULT III','','','Solar UV',1895,'',300 from dual union all
select 68111,'2005-S30','2005-07-21 08:01:00','military missile','success',799,'','Mk 21 RV','SERV-1-1','','Test',1706,'',1300 from dual union all
select 68112,'2005-S31','2005-08-03 18:45:00','suborbital rocket','success',224,'NASA 36.227US','USC-6 (SOHO SEM)','','','Solar EUV',1895,'',300 from dual union all
select 68113,'2005-S32','2005-08-04 00:00:00','military missile','success',281,'CMCM-1','','','','Test',855,'',400 from dual union all
select 68114,'2005-S33','2005-08-17 07:06:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-84',1000 from dual union all
select 68115,'2005-S34','2005-08-18 15:40:00','military missile','success',281,'CMCM-1','','','','Test',855,'',400 from dual union all
select 68116,'2005-S35','2005-08-25 08:01:00','military missile','success',799,'','Mk 21 RV','SERV-2','','Op Test',1712,'',1300 from dual union all
select 68117,'2005-S36','2005-09-07 08:53:00','military missile','success',799,'','Mk 12A RV x 2','FDEP GT187GM-1','GLORY TRIP 187GM-1','Op Test',1699,'',1300 from dual union all
select 68118,'2005-S37','2005-09-14 08:01:00','military missile','success',799,'','Mk 12A RV','FDEP GT189GM','GLORY TRIP 189GM','Op Test',1705,'',1300 from dual union all
select 68119,'2005-S38','2005-09-26 00:00:00','military missile','success',691,'','COBRA DANE Test','','','Target',959,'C-17',300 from dual union all
select 6812,'1960-S166','1960-07-07 00:00:00','military missile','success',994,'','NIIP-A target','','','Target',634,'',100 from dual union all
select 68120,'2005-S39','2005-09-27 13:22:00','military missile','success',261,'R-30 01','','','','OT',190,'TK-208',1000 from dual union all
select 68121,'2005-S40','2005-09-29 09:15:40','suborbital rocket','success',605,'NASA 30.070DR','Telemetry Test','','','Test',1895,'',90 from dual union all
select 68122,'2005-S41','2005-09-30 08:22:00','military missile','success',1507,'VMF RF','','','','OT',1011,'K-433',1000 from dual union all
select 68123,'2005-S42','2005-10-06 21:30:04','suborbital rocket','success',1507,'P-1491-205','IRDT-2R','','','Reentry Test',201,'K-496',200 from dual union all
select 68124,'2005-S43','2005-10-07 21:30:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-496',1000 from dual union all
select 68125,'2005-S44','2005-10-10 21:10:08','military missile','success',1443,'DASO-8','-    (111ss)','','','OT',419,'UK S28',1000 from dual union all
select 68126,'2005-S45','2005-10-20 07:30:00','military missile','success',1453,'492-3821-111','15A35 UBP-53','','','OT',533,'',1000 from dual union all
select 68127,'2005-S46','2005-10-27 13:45:00','suborbital rocket','success',1479,'V03','DLR SHEFEX','','','Reentry Test',29,'',211 from dual union all
select 68128,'2005-S47','2005-11-01 17:10:00','military missile','success',1435,'15Zh58E 6457351128','15Zh58E GCh-1','','','OT',636,'',1000 from dual union all
select 68129,'2005-S48','2005-11-02 22:34:00','suborbital rocket','success',1340,'NASA 45.02DR','ARAV-B2 TRACKEX FTM-8','','','Target',855,'',300 from dual union all
select 6813,'1960-S167','1960-07-07 15:27:00','military missile','success',1017,'L1-8','8K74 LKI-8','','','Test',480,'',1350 from dual union all
select 68130,'2005-S49','2005-11-09 15:40:00','suborbital rocket','success',605,'NASA 30.071DR','','','','Target',1895,'',90 from dual union all
select 68131,'2005-S50','2005-11-14 20:30:00','suborbital rocket','success',1334,'NASA 41.66DR','','','','Target',1895,'',130 from dual union all
select 68132,'2005-S51','2005-11-17 18:12:00','military missile','success',748,'Aegis FTM-04-2 Targ.','MRT-2 (Sep MRT)','','','Target',855,'',150 from dual union all
select 68133,'2005-S52','2005-11-17 18:16:00','military missile','success',1090,'Aegis FTM 04-2 (FM8)','Stellar Valkyrie','','','Interceptor',851,'CG-70',150 from dual union all
select 68134,'2005-S53','2005-11-18 21:13:00','suborbital rocket','success',1334,'NASA 41.67DR','','','','Target',1895,'',130 from dual union all
select 68135,'2005-S54','2005-11-22 00:00:00','military missile','success',1294,'THAAD FTT-01','Block 04 Demo FT-12','','','Test',1895,'',100 from dual union all
select 68136,'2005-S55','2005-11-29 07:44:00','military missile','success',1434,'64581-451-61','15Zh58 UBP-32','81','','OT',608,'',1000 from dual union all
select 68137,'2005-S56','2005-12-14 03:04:00','military missile','success',497,'4','FT-1','','','Test',868,'',1800 from dual union all
select 68138,'2005-S57','2005-12-20 17:25:47','suborbital rocket','success',1345,'NASA 12.63GT','Water recovery test','','','Test',1821,'',191 from dual;
insert into LAUNCH
select 68139,'2005-S58','2005-12-21 05:19:00','military missile','success',261,'R-30 02','','','','OT',190,'TK-208',1000 from dual union all
select 6814,'1960-S168','1960-07-09 04:00:00','suborbital rocket','success',839,'NASA 10.04GA','GSFC Grenade 2','','','Aeronomy',1821,'',105 from dual union all
select 68140,'2005-S59','2005-01-01 00:00:00','suborbital rocket','success',1026,'','Range test','','','Test',1361,'',130 from dual union all
select 68141,'2005-S60','2005-04-08 05:56:00','suborbital rocket','success',1026,'','MTS','','','Aeron',1361,'',130 from dual union all
select 68142,'2005-S61','2005-01-01 00:00:00','military missile','success',1443,'','','','','OT',1897,'SSBN UNK',1000 from dual union all
select 68143,'2005-S62','2005-12-09 19:02:42','military missile','success',1443,'FCET-34','','','','OT',420,'SSBN 740',1000 from dual union all
select 68144,'2005-S63','2005-07-07 00:00:00','military missile','success',354,'ASAT 1','','','','ASAT',1905,'',850 from dual union all
select 68145,'2005-S64','2005-01-01 00:00:00','military missile','failure',646,'','','','','Reentry Test',621,'',675 from dual union all
select 68146,'2005-Y01','2005-01-10 03:29:00','suborbital rocket','success',1284,'','SSC MAGIC support','','','Aeron',409,'',85 from dual union all
select 68147,'2005-Y02','2005-01-10 06:00:00','suborbital rocket','success',1284,'','SSC MAGIC support','','','Aeron',409,'',77 from dual union all
select 68148,'2005-Y03','2005-01-10 07:03:00','suborbital rocket','success',1284,'','SSC MAGIC support','','','Aeron',409,'',80 from dual union all
select 68149,'2005-Y04','2005-01-18 09:18:00','suborbital rocket','success',1502,'','ROMA 2005 Chaff Test','','','Aeron',29,'',90 from dual union all
select 6815,'1960-S169','1960-07-11 04:24:00','suborbital rocket','success',665,'K-8-1','','','','Test',7,'',150 from dual union all
select 68150,'2005-Y05','2005-01-18 11:36:30','suborbital rocket','success',1502,'Mini-MIDAS RPMM-01','ROMA 2005 RWMM01','','','Aeron',29,'',78 from dual union all
select 68151,'2005-Y06','2005-01-18 12:06:00','suborbital rocket','success',1502,'Mini-MIDAS RPMM-02','ROMA 2005 RWMM02','','','Aeron',29,'',87 from dual union all
select 68152,'2005-Y07','2005-01-18 12:36:00','suborbital rocket','success',1502,'ROMA/Winter (RW)','ROMA 2005 RWFS03','','','Aeron',29,'',90 from dual union all
select 68153,'2005-Y08','2005-01-18 13:15:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS04','','','Aeron',29,'',90 from dual union all
select 68154,'2005-Y09','2005-01-18 13:58:00','suborbital rocket','success',1284,'','ROMA 2005 RWCH05','','','Aeron',29,'',100 from dual union all
select 68155,'2005-Y10','2005-01-18 14:42:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS06','','','Aeron',29,'',90 from dual union all
select 68156,'2005-Y11','2005-01-18 15:31:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS07','','','Aeron',29,'',90 from dual union all
select 68157,'2005-Y12','2005-01-18 16:07:00','suborbital rocket','success',1284,'','ROMA 2005 RWCH08','','','Aeron',29,'',100 from dual union all
select 68158,'2005-Y13','2005-01-18 16:54:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS09','','','Aeron',29,'',90 from dual union all
select 68159,'2005-Y14','2005-01-18 17:24:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS10','','','Aeron',29,'',90 from dual union all
select 6816,'1960-S17','1960-01-20 16:24:00','military missile','success',1017,'I1-2','8K74 LKI-2','','','Test',480,'',1000 from dual union all
select 68160,'2005-Y15','2005-01-18 17:57:00','suborbital rocket','success',1284,'','ROMA 2005 RWCH11','','','Aeron',29,'',100 from dual union all
select 68161,'2005-Y16','2005-01-18 18:35:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS12','','','Aeron',29,'',90 from dual union all
select 68162,'2005-Y17','2005-01-20 08:46:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS13','','','Aeron',29,'',90 from dual union all
select 68163,'2005-Y18','2005-01-20 09:16:00','suborbital rocket','success',1284,'','ROMA 2005 RWCH14','','','Aeron',29,'',100 from dual union all
select 68164,'2005-Y19','2005-01-20 11:00:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS15','','','Aeron',29,'',90 from dual union all
select 68165,'2005-Y20','2005-01-20 11:30:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS16','','','Aeron',29,'',90 from dual union all
select 68166,'2005-Y21','2005-01-20 12:19:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS17','','','Aeron',29,'',90 from dual union all
select 68167,'2005-Y22','2005-01-21 13:06:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS18','','','Aeron',29,'',90 from dual union all
select 68168,'2005-Y23','2005-01-25 01:30:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS19','','','Aeron',29,'',90 from dual union all
select 68169,'2005-Y24','2005-01-25 03:30:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS20','','','Aeron',29,'',90 from dual union all
select 6817,'1960-S170','1960-07-12 00:00:00','military missile','success',976,'','8K63 UBP-55','','','Test',636,'',402 from dual union all
select 68170,'2005-Y25','2005-01-25 05:30:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS21','','','Aeron',29,'',90 from dual union all
select 68171,'2005-Y26','2005-01-25 07:30:00','suborbital rocket','success',1502,'','ROMA 2005 RWFS22','','','Aeron',29,'',90 from dual union all
select 68172,'2005-Y27','2005-07-04 22:32:00','suborbital rocket','success',469,'2','Mini-DUSTY MD-11','','','Aeron',29,'',90 from dual union all
select 68173,'2006-001','2006-01-19 19:00:00','deep space','success',196,'AV-010','New Horizons/Star 48','New Horizons','','Sat',292,'',null from dual union all
select 68174,'2006-002','2006-01-24 01:33:00','orbital','success',529,'H-IIA-8F','ALOS','ALOS','','Sat',1477,'',null from dual union all
select 68175,'2006-003','2006-02-15 23:34:55','orbital','success',1542,'SL15/DM-SL-14L','SL24 Echostar 10','Echostar 10','','Sat',860,'ODYSSEY',null from dual union all
select 68176,'2006-004','2006-02-18 06:27:00','orbital','success',531,'H-IIA-9F','MTSAT-2','MTSAT-2','','Sat',1477,'',null from dual union all
select 68177,'2006-005','2006-02-21 21:28:00','orbital','success',730,'M-V-8','ASTRO F','ASTRO-F','','Sat',836,'',null from dual union all
select 68178,'2006-006','2006-02-28 20:10:00','orbital','failure',962,'535-11/88515','Arabsat 4A','Arabsat 4A','','Sat',553,'',null from dual union all
select 68179,'2006-007','2006-03-11 22:33:00','orbital','success',111,'V170 (527)','Hot Bird 7A/Spainsat','Spainsat','','Sat',346,'',null from dual union all
select 6818,'1960-S171','1960-07-13 00:00:00','military missile','success',976,'','8K63 UBP-56','','','Test',636,'',402 from dual union all
select 68180,'2006-008','2006-03-22 14:03:45','orbital','success',911,'F37','ST5','Space Technology 5','','Sat',1718,'L-1011',null from dual union all
select 68181,'2006-009','2006-03-30 02:30:20','orbital','success',1233,'P 15000-018 018','Soyuz TMA-8','Soyuz 11F732 No. 218','','Sat ISS 12S',480,'',null from dual union all
select 68182,'2006-010','2006-04-12 23:29:59','orbital','success',1542,'SL21/DM-SL-22L','SL31 JCSAT-9','JCSAT-9','','Sat',860,'ODYSSEY',null from dual union all
select 68183,'2006-011','2006-04-15 01:40:00','orbital','success',783,'5','FORMOSAT 3 FM6','COSMIC','','Sat',1781,'',null from dual union all
select 68184,'2006-012','2006-04-20 20:27:00','orbital','success',184,'AV-008','Astra 1KR','Astra 1KR','','Sat',292,'',null from dual union all
select 68185,'2006-013','2006-04-24 16:03:25','orbital','success',1241,'P 15000-100 100','Progress M-56','Progress 7K-TGM No. 356','','Sat ISS 21P',480,'',null from dual union all
select 68186,'2006-014','2006-04-25 16:47:16','orbital','success',1260,'64536-214-41','EROS B','EROS B','','Sat',478,'',null from dual union all
select 68187,'2006-015','2006-04-26 22:48:00','orbital','success',320,'Y1       (89)','Yaogan 1','yaogan weixing yihao','','Sat',1492,'',null from dual union all
select 68188,'2006-016','2006-04-28 10:02:16','orbital','success',413,'D314','Calipso/Cloudsat','Cloudsat','','Sat',1724,'',null from dual union all
select 68189,'2006-017','2006-05-03 17:38:01','orbital','success',1241,'78031  -800 090','Kosmos-2420','Kobal''t-M','','Sat',462,'',null from dual union all
select 6819,'1960-S172','1960-07-13 00:23:00','suborbital rocket','success',839,'AA6.403','','','','Aeronomy',394,'',170 from dual union all
select 68190,'2006-018','2006-05-24 22:11:00','orbital','success',400,'D4-5 (315) 4240','GOES N','GOES N','','Sat',290,'',null from dual union all
select 68191,'2006-019','2006-05-26 18:50:00','orbital','success',1180,'','Kompas-2','Vulkan-Kompas-2','','Sat',201,'K-84',null from dual union all
select 68192,'2006-020','2006-05-27 21:09:00','orbital','success',111,'V171 (529)','Satmex 6/Thaicom 5','Satmex 6','','Sat',346,'',null from dual union all
select 68193,'2006-021','2006-06-15 08:00:00','orbital','success',1241,'P 15000-096 096','Resurs DK-1','Resurs-DK No. 1','','Sat',480,'',null from dual union all
select 68194,'2006-022','2006-06-17 22:44:05','orbital','success',960,'410-12','Kazsat','Kazsat 78KS','','Sat',553,'',null from dual union all
select 68195,'2006-023','2006-06-18 07:50:00','orbital','success',1542,'SL20/DM-SL-21L','SL29 Galaxy 16','Galaxy 16','','Sat',860,'ODYSSEY',null from dual union all
select 68196,'2006-024','2006-06-21 22:15:00','orbital','success',427,'D316','USA 187-189 (MITEX)','MITEX OSC satellite','','Sat',285,'',null from dual union all
select 68197,'2006-025','2006-06-24 15:08:18','orbital','success',1241,'Zh15000-101 101','Progress M-57','Progress 7K-TGM No. 357','','Sat ISS 22P',480,'',null from dual union all
select 68198,'2006-026','2006-06-25 04:00:00','orbital','success',1446,'450-92-  804','Kosmos-2421','US-PU/Konus-A','','Sat',591,'',null from dual union all
select 68199,'2006-027','2006-06-28 03:33:00','orbital','success',400,'D4-6 (317) 4240','USA 184       [JS11]','Adv TRUMPET 1','','Sat',1780,'',null from dual union all
select 682,'1947-A18','1947-09-25 00:00:00','suborbital rocket','success',58,'A1','Dummy','','','Test',1869,'',1 from dual union all
select 6820,'1960-S173','1960-07-13 14:47:00','suborbital rocket','failure',1267,'OB11.03','','','','Ionos',1821,'',640 from dual union all
select 68200,'2006-028','2006-07-04 18:37:55','orbital','success',1243,'STS-121','ISS ULF-1.1','OV-103 ISS','','Sat',922,'',null from dual union all
select 68201,'2006-029','2006-07-12 14:53:36','orbital','success',434,'450-2973-806','Genesis-1','Bigelow Genesis-1','','Sat',1906,'',null from dual union all
select 68202,'2006-030','2006-07-21 04:20:03','orbital','success',803,'78036  -691 048','Kosmos-2422','Oko US-KS?','','Sat',462,'',null from dual union all
select 68203,'2006-031','2006-07-28 07:05:43','orbital','success',1064,'492-5882-032/72504','Kompsat-2','KOMPSAT-2','','Sat',460,'',null from dual union all
select 68204,'2006-032','2006-08-04 21:48:00','orbital','success',962,'535-14/88516','Hot Bird 8','Hot Bird 8','','Sat',553,'',null from dual union all
select 68205,'2006-033','2006-08-11 22:15:00','orbital','success',111,'V172 (531)','JCSAT-10/Syracuse 3B','JCSAT 10','','Sat',346,'',null from dual union all
select 68206,'2006-034','2006-08-22 03:27:01','orbital','success',1542,'SL23/DM-SL-23L','SL25 Koreasat-5','Koreasat 5','','Sat',860,'ODYSSEY',null from dual union all
select 68207,'2006-035','2006-09-09 07:00:04','orbital','success',301,'Y16      (90)','SJ-8','shijian bahao yuzhong we','','Sat',813,'',null from dual union all
select 68208,'2006-036','2006-09-09 15:14:55','orbital','success',1243,'STS-115','ISS 12A','OV-104 ISS','','Sat',922,'',null from dual union all
select 68209,'2006-037','2006-09-11 04:35:00','orbital','success',528,'H-IIA-10','IGS O-2','IGS Optical-2','','Sat',1477,'',null from dual union all
select 6821,'1960-S174','1960-07-14 02:43:00','suborbital rocket','failure',1267,'OB11.04','','','','Ionos',1821,'',691 from dual union all
select 68210,'2006-038','2006-09-12 16:02:00','orbital','success',309,'Y10      (91)','ZX-22A','Feng Huo 2?','','Sat',1903,'',null from dual union all
select 68211,'2006-039','2006-09-14 13:41:00','orbital','success',1241,'78041  -490 088','Kosmos-2423','Don','','Sat',562,'',null from dual union all
select 68212,'2006-040','2006-09-18 04:08:42','orbital','success',1233,'Ts15000-023 023','Soyuz TMA-9','Soyuz 11F732 No. 219','','Sat ISS 13S',480,'',null from dual union all
select 68213,'2006-041','2006-09-22 21:36:00','orbital','success',730,'M-V-7','SOLAR B','SOLAR-B','','Sat',836,'',null from dual union all
select 68214,'2006-042','2006-09-25 18:50:00','orbital','success',427,'D318','USA 190 (GPS IIRM-2)','Navstar GPS SVN 52','','Sat',285,'',null from dual union all
select 68215,'2006-043','2006-10-13 20:56:00','orbital','success',111,'V173 (533)','DirecTV-9S/Optus D1','DirecTV 9S','','Sat',346,'',null from dual union all
select 68216,'2006-044','2006-10-19 16:28:13','orbital','success',1226,'Zh15000-003 003/ST16','Metop 2','METOP A','','Sat',562,'',null from dual union all
select 68217,'2006-045','2006-10-23 13:40:36','orbital','success',1241,'Ts15000-102 102','Progress M-58','Progress 7K-TGM No. 358','','Sat ISS 23P',480,'',null from dual union all
select 68218,'2006-046','2006-10-23 23:34:00','orbital','success',319,'Y16      (92)','SJ-6-2A/SJ-6-2B','shijian liuhao er zu B x','','Sat',1492,'',null from dual union all
select 68219,'2006-047','2006-10-26 00:52:00','deep space','success',425,'D319','STEREO A/B','STEREO Ahead','','Sat',286,'',null from dual union all
select 6822,'1960-S175','1960-07-14 10:05:00','suborbital rocket','success',839,'AA6.409','Firefly FRANCES','','','Aeronomy',394,'',148 from dual union all
select 68220,'2006-048','2006-10-28 16:20:00','orbital','success',313,'Y7       (93)','Xinnuo-2','xinnuo erhao','','Sat',1903,'',null from dual union all
select 68221,'2006-049','2006-10-30 23:48:59','orbital','success',1542,'SL22/DM-SL-24L','SL22 XM Radio-4','XM Radio-4','','Sat',860,'ODYSSEY',null from dual union all
select 68222,'2006-050','2006-11-04 13:53:00','orbital','success',398,'D4-7 (320) 4040','USA 191 (DMSP 17)','DMSP Block 5D-3 S-17','','Sat',1780,'',null from dual union all
select 68223,'2006-051','2006-11-08 20:01:00','orbital','success',962,'535-15/88517','Arabsat 4A','Arabsat 4B','','Sat',553,'',null from dual union all
select 68224,'2006-052','2006-11-17 19:12:00','orbital','success',427,'D321','USA 192 (GPS IIRM-3)','Navstar GPS SVN 58','','Sat',285,'',null from dual union all
select 68225,'2006-053','2006-12-08 00:53:00','orbital','success',309,'Y11      (94)','FY-2D','fengyun erhao D xing','','Sat',1903,'',null from dual union all
select 68226,'2006-054','2006-12-08 22:08:00','orbital','success',111,'V174 (534)','WildBlue 1/AMC 18','WildBlue 1','','Sat',346,'',null from dual union all
select 68227,'2006-055','2006-12-10 01:47:35','orbital','success',1243,'STS-116','ISS 12A.1','OV-103 ISS','','Sat',922,'',null from dual union all
select 68228,'2006-056','2006-12-11 23:28:43','orbital','success',962,'535-21/88518','Measat 3','Measat','','Sat',553,'',null from dual;
insert into LAUNCH
select 68229,'2006-057','2006-12-14 21:00:00','orbital','success',418,'D322','USA 193 (NROL-21)','NROL-21','','Sat',1724,'',null from dual union all
select 6823,'1960-S176','1960-07-15 06:10:00','military missile','success',928,'A1X-42','','','','Test',245,'',500 from dual union all
select 68230,'2006-058','2006-12-16 12:00:00','orbital','success',783,'6','Tacsat 2','Tacsat-2','','Sat',945,'',null from dual union all
select 68231,'2006-059','2006-12-18 06:32:00','orbital','success',533,'H-IIA-11','ETS-8','ETS-8','','Sat',1477,'',null from dual union all
select 68232,'2006-060','2006-12-19 14:00:19','orbital','success',680,'53727-226 SL','SAR-Lupe 1','SAR-Lupe 1','','Sat',456,'',null from dual union all
select 68233,'2006-061','2006-12-24 08:34:44','orbital','success',1226,'77033  -135 002','Meridian 1','Meridian No. 11L','','Sat',475,'',null from dual union all
select 68234,'2006-062','2006-12-25 20:18:12','orbital','success',958,'410-15','Kosmos-2424/2426','Uragan-M No. 715','','Sat',588,'',null from dual union all
select 68235,'2006-063','2006-12-27 14:23:38','orbital','success',1228,'P 15000-001 001/ST17','COROT','COROT','','Sat',562,'',null from dual union all
select 68236,'2006-A01','2006-05-17 14:00:00','suborbital rocket','success',1284,'Genesis test','','','','Test',295,'',47 from dual union all
select 68237,'2006-A02','2006-05-24 20:36:00','military missile','success',1088,'Aegis SBTT','Pacific Phoenix','','','Test',851,'CG-70',10 from dual union all
select 68238,'2006-A03','2006-07-01 07:05:00','suborbital rocket','failure',605,'','ARR HotPay-1','','','Aeron',29,'',40 from dual union all
select 68239,'2006-A04','2006-07-04 20:01:00','military missile','failure',1461,'1','','','','Test',1480,'',40 from dual union all
select 6824,'1960-S177','1960-07-15 00:00:00','suborbital rocket','success',972,'','','','','Ionos/Aeron',636,'',206 from dual union all
select 68240,'2006-A05','2006-09-07 15:50:00','military missile','failure',261,'R-30 03','','','','Test',190,'TK-208',1 from dual union all
select 68241,'2006-A06','2006-09-25 20:14:00','suborbital rocket','failure',1244,'SpaceLoft SL-1','','','','Test',1422,'',12 from dual union all
select 68242,'2006-A07','2006-12-24 00:00:00','military missile','failure',261,'R-30 05','','','','Test',190,'TK-208',1 from dual union all
select 68243,'2006-A08','2006-12-05 08:00:00','military missile','success',3,'','Test','','','Op Test',183,'',40 from dual union all
select 68244,'2006-A09','2006-05-24 00:00:00','military missile','success',702,'Aegis SBT Target','Unitary SRT','','','Target',855,'',40 from dual union all
select 68245,'2006-A10','2006-02-16 00:00:00','military missile','success',1088,'','Subsonic target','','','Test',1895,'',10 from dual union all
select 68246,'2006-A11','2006-11-01 00:00:00','suborbital rocket','success',670,'Kavoshgar-1','Kavoshgar Class A','','','Test',1356,'',10 from dual union all
select 68247,'2006-F01','2006-03-24 22:30:00','orbital','failure',475,'F001','Falconsat-2','Falconsat 2','','Sat',870,'',5 from dual union all
select 68248,'2006-F02','2006-07-10 12:08:00','orbital','failure',502,'GSLV-F02','Insat 4C','Insat 4C','','Sat',1359,'',40 from dual union all
select 68249,'2006-F03','2006-07-26 19:43:05','orbital','failure',434,'450-2973-804','BelKA/P-PODs','BelKA','','Sat',497,'',40 from dual union all
select 6825,'1960-S178','1960-07-15 10:02:00','suborbital rocket','success',839,'AA6.401','','','','Aeronomy',394,'',120 from dual union all
select 68250,'2006-M01','2006-06-08 11:11:00','suborbital rocket','success',891,'NASA 30.072NO','Student (SUBSEM)','','','Education',1821,'',51 from dual union all
select 68251,'2006-M02','2006-07-12 11:20:00','military missile','success',1294,'THAAD FTT-03','High Endo','','','ABM Test',1895,'',60 from dual union all
select 68252,'2006-M03','2006-06-11 08:10:00','military missile','success',941,'','User Trial','','','Test',320,'',50 from dual union all
select 68253,'2006-M04','2006-02-21 00:00:00','suborbital rocket','success',520,'','ANSR','','','Test',1850,'',50 from dual union all
select 68254,'2006-M05','2006-06-01 00:00:00','military missile','success',510,'Hatf III','','','','Test',1419,'',50 from dual union all
select 68255,'2006-M06','2006-12-09 00:00:00','military missile','success',510,'Hatf III','','','','Test',1419,'',50 from dual union all
select 68256,'2006-M07','2006-02-08 08:00:00','suborbital rocket','success',1023,'','CAWSES-India','','','Aeron',1467,'',70 from dual union all
select 68257,'2006-M08','2006-02-22 12:00:00','suborbital rocket','success',1023,'','CAWSES-India','','','Aeron',1467,'',70 from dual union all
select 68258,'2006-M09','2006-03-08 04:05:00','suborbital rocket','success',1023,'','CAWSES-India','','','Aeron',1467,'',70 from dual union all
select 68259,'2006-M10','2006-03-22 00:12:00','suborbital rocket','success',1023,'','CAWSES-India','','','Aeron',1467,'',70 from dual union all
select 6826,'1960-S179','1960-07-17 04:11:00','suborbital rocket','success',665,'K-8-2','ID-1','','','Test',7,'',182 from dual union all
select 68260,'2006-M11','2006-04-05 16:00:00','suborbital rocket','success',1023,'','CAWSES-India','','','Aeron',1467,'',70 from dual union all
select 68261,'2006-M12','2006-04-17 20:00:00','suborbital rocket','success',1023,'','CAWSES-India','','','Aeron',1467,'',70 from dual union all
select 68262,'2006-M13','2006-11-02 00:00:00','military missile','success',489,'','','','','Test',759,'',60 from dual union all
select 68263,'2006-S01','2006-01-18 11:48:00','suborbital rocket','success',1221,'5','Space science','','','Ionos/Aeron',808,'',282 from dual union all
select 68264,'2006-S02','2006-01-22 04:00:00','suborbital rocket','success',1073,'S-310-36','Furoshiki','','','Tech',833,'',130 from dual union all
select 68265,'2006-S03','2006-02-08 18:47:00','suborbital rocket','success',224,'NASA 36.193','MOSES','','','Solar EUV',1895,'',282 from dual union all
select 68266,'2006-S04','2006-02-16 08:01:00','military missile','success',799,'','Mk 21 RV','SERV-3','','Test',1706,'',1300 from dual union all
select 68267,'2006-S05','2006-02-23 16:09:00','military missile','success',1111,'M-7 FT04-1','GMD FT04-1 Target','','','Target',862,'',1000 from dual union all
select 68268,'2006-S06','2006-03-08 20:45:00','military missile','success',1090,'Aegis JCTV-1','Stellar Tsuru','','','Test',852,'CG-70',150 from dual union all
select 68269,'2006-S07','2006-03-25 03:15:00','suborbital rocket','success',1341,'','Hyshot 3','','','Tech',1849,'',325 from dual union all
select 6827,'1960-S18','1960-01-21 20:10:00','military missile','success',1367,'215','IWST','','RED CABOOSE','Test',1674,'',520 from dual union all
select 68270,'2006-S08','2006-03-25 09:00:00','suborbital rocket','success',891,'NASA 30.064DR','','','','Target',1895,'',90 from dual union all
select 68271,'2006-S09','2006-03-25 13:07:21','suborbital rocket','success',891,'NASA 30.065DR','','','','Target',1895,'',90 from dual union all
select 68272,'2006-S10','2006-03-30 02:40:00','suborbital rocket','success',1341,'','Hyshot 4 (JAXA)','','','Tech',1849,'',290 from dual union all
select 68273,'2006-S11','2006-04-05 05:56:00','suborbital rocket','success',605,'SSC REXUS 3','EURO REXUS 3','','','Education',417,'',95 from dual union all
select 68274,'2006-S12','2006-04-05 16:00:00','suborbital rocket','success',1334,'NASA 41.068NT','Descent imagery/RSO','','','Planetary test',1873,'',125 from dual union all
select 68275,'2006-S13','2006-04-07 13:00:00','military missile','success',799,'','Mk 12 RV','FDEP GT190GB','GLORY TRIP 190GB','Op Test (Guam)',1712,'',1300 from dual union all
select 68276,'2006-S14','2006-04-12 18:10:00','suborbital rocket','success',224,'NASA 36.203GS','EUNIS-06','','','Solar EUV',1895,'',318 from dual union all
select 68277,'2006-S15','2006-04-13 00:00:00','military missile','success',1097,'MBRV-3','CMCP FTC-02B (TCMP)','','','Target',855,'',200 from dual union all
select 68278,'2006-S16','2006-04-22 15:40:00','military missile','success',646,'65087-215','PBS test 2','','','Reentry Test',621,'',675 from dual union all
select 68279,'2006-S17','2006-04-28 00:00:00','military missile','success',1097,'MBRV-3','CMCP FTC-02  (TCMP)','','','Target',855,'',200 from dual union all
select 6828,'1960-S180','1960-07-18 10:01:00','suborbital rocket','success',839,'AA6.405','Firefly CARRY','','','Aeronomy',394,'',130 from dual union all
select 68280,'2006-S18','2006-04-29 00:00:00','military missile','success',1171,'','','','','Test',1419,'',300 from dual union all
select 68281,'2006-S19','2006-05-02 06:16:00','suborbital rocket','success',772,'MAXUS 7','SSC MAXUS 7','','','Micrograv',407,'',702 from dual union all
select 68282,'2006-S20','2006-05-11 08:12:00','suborbital rocket','success',1484,'V03','DLR TEXUS 43','','','Micrograv',416,'',237 from dual union all
select 68283,'2006-S21','2006-05-11 00:00:00','military missile','success',1294,'THAAD FTT-02','Virtual Target FT-13','','','Test',1895,'',100 from dual union all
select 68284,'2006-S22','2006-05-23 00:00:00','military missile','success',508,'','','','','Test',1356,'',150 from dual union all
select 68285,'2006-S23','2006-05-22 09:30:00','suborbital rocket','success',1334,'NASA 12.58GT','ACS test','','','Test',1895,'',150 from dual union all
select 68286,'2006-S24','2006-06-05 16:05:00','suborbital rocket','success',1334,'NASA 41.073DR','','','','Target',1895,'',130 from dual union all
select 68287,'2006-S25','2006-06-07 22:00:00','suborbital rocket','success',1334,'NASA 41.071DR','','','','Target',855,'',130 from dual union all
select 68288,'2006-S26','2006-06-08 16:00:00','suborbital rocket','success',1334,'NASA 41.074DR','DUNDEE target','','','Target',1895,'',130 from dual union all
select 68289,'2006-S27','2006-06-14 08:22:00','military missile','success',799,'','Mk 12 RV','FDEP GT191GM','GLORY TRIP 191GM','Test',1699,'',1300 from dual union all
select 6829,'1960-S181','1960-07-19 16:00:00','military missile','success',928,'A1X-41','','','','Test',244,'',500 from dual union all
select 68290,'2006-S28','2006-06-19 00:45:56','suborbital rocket','failure',1024,'','Mini-DUSTY MD-12','','','Aeron/Test',29,'',80 from dual union all
select 68291,'2006-S29','2006-06-22 22:00:00','military missile','success',748,'Aegis FTM-10 Target','MRT-3 (Sep MRT)','','','Target',855,'',150 from dual union all
select 68292,'2006-S30','2006-06-22 22:04:00','military missile','success',1090,'Aegis FTM-10','Stellar Predator','','','Test',851,'CG-67',150 from dual union all
select 68293,'2006-S31','2006-06-23 23:02:00','suborbital rocket','success',1334,'NASA 41.072DR Aegis','ARAV-A FTM-10 Targ','','','Target ARAV-A',855,'',130 from dual union all
select 68294,'2006-S32','2006-06-30 06:25:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-114',1000 from dual union all
select 68295,'2006-S33','2006-07-01 06:39:00','suborbital rocket','success',1334,'NASA 41.056UO','ESPRIT (SPIRIT 3)','','','Ionos/Plasma',29,'',169 from dual union all
select 68296,'2006-S34','2006-07-04 18:32:00','military missile','success',580,'','','','','Test',859,'',150 from dual union all
select 68297,'2006-S35','2006-07-04 19:04:00','military missile','success',586,'','','','','Test',859,'',150 from dual union all
select 68298,'2006-S36','2006-07-04 22:12:00','military missile','success',580,'','','','','Test',859,'',150 from dual union all
select 68299,'2006-S37','2006-07-04 22:31:00','military missile','success',586,'','','','','Test',859,'',150 from dual union all
select 683,'1947-A19','1947-09-26 00:00:00','atmospheric rocket','success',876,'47-F','Nike-47 Round E','','','Test',1863,'',10 from dual union all
select 6830,'1960-S182','1960-07-20 00:00:00','suborbital rocket','unknown',839,'AEC probe','','','','Aeron?',1016,'',120 from dual union all
select 68300,'2006-S38','2006-07-04 22:32:00','military missile','success',586,'','','','','Test',859,'',150 from dual union all
select 68301,'2006-S39','2006-07-05 08:20:00','military missile','success',586,'','','','','Test',859,'',150 from dual union all
select 68302,'2006-S40','2006-07-09 05:33:00','military missile','failure',65,'01','','','','Test',316,'',100 from dual union all
select 68303,'2006-S41','2006-07-12 11:17:00','military missile','success',570,'THAAD FTT-03 Target','BRV','','','Target',1857,'',300 from dual union all
select 68304,'2006-S42','2006-07-21 09:04:00','suborbital rocket','success',605,'NASA 30.066DR','Target','','','Target',1895,'',90 from dual union all
select 68305,'2006-S43','2006-07-20 10:14:00','military missile','success',799,'','Mk 12A RV x 3','FDEP GT192GM','GLORY TRIP 192GM','Test',1705,'',1300 from dual union all
select 68306,'2006-S44','2006-07-21 13:05:00','suborbital rocket','success',605,'NASA 30.067DR','Target','','','Target',1895,'',90 from dual union all
select 68307,'2006-S45','2006-08-03 10:38:00','military missile','success',1434,'64588-343-65','15Zh58 UBP-33','','','OT',608,'',1000 from dual union all
select 68308,'2006-S46','2006-08-21 14:30:00','suborbital rocket','success',224,'NASA 36.238DR','THAAD Target','','','Target',1895,'',380 from dual union all
select 68309,'2006-S47','2006-08-25 14:30:00','suborbital rocket','success',224,'NASA 36.237DR','THAAD Target','','','Target',1895,'',382 from dual union all
select 6831,'1960-S183','1960-07-20 00:00:00','military missile','success',1013,'','8K51 PRO Target C-4','NIIP-A Target','','Target',321,'',500 from dual union all
select 68310,'2006-S48','2006-09-01 17:22:00','military missile','success',1111,'M-8','GMD FTG-02 Target','','','Target',862,'',1000 from dual union all
select 68311,'2006-S49','2006-09-01 17:39:00','military missile','success',497,'5','GMD FTG-02','','','Test',1709,'',1000 from dual union all
select 68312,'2006-S50','2006-09-04 00:00:00','military missile','success',357,'','','','','Test',1494,'',1000 from dual union all
select 68313,'2006-S51','2006-09-09 11:20:00','military missile','success',1185,'VMF RF','3x RV','','','OT',1001,'K-84',1000 from dual union all
select 68314,'2006-S52','2006-09-08 22:17:00','suborbital rocket','success',847,'','ECOMA 1','','','Meteors/Dust',29,'',130 from dual union all
select 68315,'2006-S53','2006-09-10 14:50:00','military missile','success',1507,'VMF RF','2x RV','','','OT',1405,'K-433',1000 from dual union all
select 68316,'2006-S54','2006-09-13 10:30:00','military missile','failure',570,'THAAD FTT-04 Target','BRV Sep','','','Target',1857,'',100 from dual union all
select 68317,'2006-S55','2006-09-17 21:06:46','suborbital rocket','success',847,'','ECOMA 2','','','Meteors/Dust',29,'',129 from dual union all
select 68318,'2006-S56','2006-10-25 13:05:00','military missile','failure',261,'R-30 04','','','','Test',190,'TK-208',100 from dual;
insert into LAUNCH
select 68319,'2006-S57','2006-09-23 15:17:54','suborbital rocket','success',233,'NASA 12.064GT','Igniter test','','','Test',1821,'',300 from dual union all
select 6832,'1960-S184','1960-07-20 17:39:00','military missile','success',928,'A1E-1 (A1XP-5)','','','','Test',421,'SSBN 598',500 from dual union all
select 68320,'2006-S58','2006-10-28 17:58:00','suborbital rocket','success',224,'NASA 36.233US','LASP/CU-11 (SEE/EVE)','','','Solar EUV',1895,'',300 from dual union all
select 68321,'2006-S59','2006-11-02 00:00:00','military missile','success',508,'','','','','Test',1356,'',150 from dual union all
select 68322,'2006-S60','2006-11-09 08:51:00','military missile','success',758,'1','','','','Test',307,'',1000 from dual union all
select 68323,'2006-S61','2006-11-09 11:35:00','military missile','success',1453,'492-3821-114','15A35 UBP-54','','','OT',533,'',1000 from dual union all
select 68324,'2006-S62','2006-11-07 19:30:00','suborbital rocket','success',224,'NASA 36.236US','USC-7 (SOHO SEM)','','','Solar EUV',1895,'',300 from dual union all
select 68325,'2006-S63','2006-11-21 02:00:00','suborbital rocket','success',224,'NASA 36.224UH','CyXESS','','','XR Astron',1895,'',300 from dual union all
select 68326,'2006-S64','2006-11-16 00:00:00','military missile','success',509,'Hatf-V','','','','Test',1473,'',400 from dual union all
select 68327,'2006-S65','2006-11-19 04:25:00','military missile','success',941,'','Test','','','Test',315,'',100 from dual union all
select 68328,'2006-S66','2006-11-27 04:45:00','military missile','success',941,'','PAD Target (P2)','','','Target',315,'',90 from dual union all
select 68329,'2006-S67','2006-11-27 04:46:00','military missile','success',941,'','PAD Intercept (P2)','','','ABM Test',316,'',48 from dual union all
select 6833,'1960-S185','1960-07-20 20:32:00','military missile','success',928,'A1E-2 (A1XP-8)','','','','Test',421,'SSBN 598',500 from dual union all
select 68330,'2006-S68','2006-11-29 00:00:00','military missile','success',1169,'Hatf IV','','','','Test',1419,'',100 from dual union all
select 68331,'2006-S69','2006-12-07 00:00:00','military missile','success',732,'Aegis FTM-11 Target','Aegis TTV-9','','','Target',855,'',300 from dual union all
select 68332,'2006-S70','2006-05-09 00:00:00','military missile','success',1443,'FCET-35','W76 LEP','','','OT',1897,'SSBN 732',1000 from dual union all
select 68333,'2006-S71','2006-05-09 00:00:00','military missile','success',1443,'FCET-35','W76 LEP','','','OT',1897,'SSBN 732',1000 from dual union all
select 68334,'2006-S72','2006-11-21 00:00:00','military missile','success',1443,'FCET-36','W76  (116ss)','','','OT',421,'SSBN 738',1000 from dual union all
select 68335,'2006-S73','2006-11-21 00:00:00','military missile','success',1443,'FCET-36','W76  (117ss)','','','OT',421,'SSBN 738',1000 from dual union all
select 68336,'2006-S74','2006-12-01 00:00:00','suborbital rocket','success',508,'Safir test','Kavesh','','','Test',1347,'',150 from dual union all
select 68337,'2006-S75','2006-02-06 00:00:00','military missile','success',354,'ASAT 2','','','','ASAT',1905,'',850 from dual union all
select 68338,'2006-S76','2006-11-16 00:00:00','military missile','success',1344,'FTM-11 TRACKEX','ARAV-B3 FTM-11 (sep)','','','Target',855,'',150 from dual union all
select 68339,'2006-S77','2006-12-21 08:20:00','military missile','success',1005,'450-7951-213','15A18M UBP 13 RD','','','OT',1908,'',1000 from dual union all
select 6834,'1960-S186','1960-07-21 01:32:00','suborbital rocket','success',839,'AA6.413','Firefly IDA','','','Aeronomy',394,'',130 from dual union all
select 68340,'2006-S78','2006-01-17 00:00:00','military missile','success',1168,'','','','','Test',1356,'',150 from dual union all
select 68341,'2006-S79','2006-01-01 00:00:00','military missile','success',593,'','','','','Test',30,'',150 from dual union all
select 68342,'2006-Y01','2006-06-08 10:20:00','suborbital rocket','success',1284,'','Langmuir probe dart','','','Aeron/Test',1821,'',80 from dual union all
select 68343,'2007-001','2007-01-10 03:54:00','orbital','success',899,'PSLV-C7','Cartosat-2/SRE','LAPAN-TUBSAT','','Sat',1357,'',null from dual union all
select 68344,'2007-002','2007-01-18 02:12:13','orbital','success',1241,'Ts15000-107 107','Progress M-59','Progress 7K-TGM No. 359','','Sat ISS 24P',480,'',null from dual union all
select 68345,'2007-003','2007-02-02 16:28:00','orbital','success',309,'Y12      (95)','Beidou 4','di si ke beidou daohang','','Sat',1903,'',null from dual union all
select 68346,'2007-004','2007-02-17 23:01:00','deep space','success',424,'D323','THEMIS','THEMIS P5','','Sat',286,'',null from dual union all
select 68347,'2007-005','2007-02-24 04:41:00','orbital','success',531,'H-IIA-12','IGS R-2/O-3V','IGS Radar-2','','Sat',1477,'',null from dual union all
select 68348,'2007-006','2007-03-09 03:10:00','orbital','success',182,'AV-013','STP-1','Orbital Express','','Sat',292,'',null from dual union all
select 68349,'2007-007','2007-03-11 22:03:00','orbital','success',111,'V175 (535)','Skynet 5A/Insat 4B','Insat 4B','','Sat',346,'',null from dual union all
select 6835,'1960-S187','1960-07-21 10:02:00','suborbital rocket','success',839,'AA6.406','Firefly LILY','','','Aeronomy',394,'',151 from dual union all
select 68350,'2007-008','2007-04-07 17:31:14','orbital','success',1233,'Ts15000-019 019','Soyuz TMA-10','Soyuz 11F732 No. 220','','Sat ISS 14S',480,'',null from dual union all
select 68351,'2007-009','2007-04-09 22:54:00','orbital','success',962,'535-16/88521','Anik F3','Anik F3','','Sat',553,'',null from dual union all
select 68352,'2007-010','2007-04-11 03:27:00','orbital','success',301,'Y18      (96)','HY-1B','haiyang yihao B','','Sat',1492,'',null from dual union all
select 68353,'2007-011','2007-04-13 20:11:00','orbital','success',309,'Y13      (97)','Beidou 2A','yi ke beidou daohang wei','','Sat',1904,'',null from dual union all
select 68354,'2007-012','2007-04-17 06:46:39','orbital','success',434,'510-9971-806','Misr/Saudisat-3/','Egyptsat 1','','Sat SSO',497,'',null from dual union all
select 68355,'2007-013','2007-04-23 10:00:00','orbital','success',900,'PSLV-C8','AGILE/AAM','AGILE','','Sat',1359,'',null from dual union all
select 68356,'2007-014','2007-04-24 06:48:00','orbital','success',783,'7','NFIRE','NFIRE','','Sat',945,'',null from dual union all
select 68357,'2007-015','2007-04-25 20:26:03','orbital','success',911,'F38','AIM','AIM','','Sat',1718,'L-1011',null from dual union all
select 68358,'2007-016','2007-05-04 22:29:00','orbital','success',111,'V176 (536)','Astra 1L/Galaxy 17','Astra 1L','','Sat',346,'',null from dual union all
select 68359,'2007-017','2007-05-12 03:25:38','orbital','success',1241,'Ts15000-104 104','Progress M-60','Progress 7K-TGM No. 360','','Sat ISS 25P',480,'',null from dual union all
select 6836,'1960-S188','1960-07-22 04:53:00','suborbital rocket','success',839,'NASA 10.18GE','SBE-2','','','Auroral',428,'',129 from dual union all
select 68360,'2007-018','2007-05-13 16:01:02','orbital','success',311,'Y9       (98)','Nigcomsat 1','Nigcomsat 1','','Sat',1903,'',null from dual union all
select 68361,'2007-019','2007-05-25 07:12:00','orbital','success',303,'Y8       (99)','Yaogan 2','yaogan weixing erhao','','Sat',813,'',null from dual union all
select 68362,'2007-020','2007-05-29 20:31:30','orbital','success',1232,'Ts15000-021 021','Globalstar','Globalstar','','Sat',562,'',null from dual union all
select 68363,'2007-021','2007-05-31 16:08:00','orbital','success',309,'Y15      (100)','Xinnuo 3','xinnuo sanhao','','Sat',1904,'',null from dual union all
select 68364,'2007-022','2007-06-07 18:00:01','orbital','success',1241,'76033  -137 099','Kosmos-2427','Kobal''t-M','','Sat',462,'',null from dual union all
select 68365,'2007-023','2007-06-08 02:34:00','orbital','success',413,'D324','COSMO 1','COSMO-Skymed','','Sat',1724,'',null from dual union all
select 68366,'2007-024','2007-06-08 23:38:04','orbital','success',1243,'STS-117','ISS 13A','OV-104 ISS','','Sat',921,'',null from dual union all
select 68367,'2007-025','2007-06-10 23:40:00','orbital','success',1174,'7','''Ofeq-7','''Ofeq-7','','Sat',1026,'',null from dual union all
select 68368,'2007-026','2007-06-15 02:14:00','orbital','success',434,'670-8343-630','TerraSAR-X','TerraSAR-X','','Sat SSO',497,'',null from dual union all
select 68369,'2007-027','2007-06-15 15:12:00','orbital','failure',182,'AV-009','USA 194     [NOSS17]','INTRUDER 8?','','Sat',292,'',null from dual union all
select 6837,'1960-S189','1960-07-22 10:11:00','suborbital rocket','success',839,'AA6.402','Firefly HEDY','','','Aeronomy',394,'',108 from dual union all
select 68370,'2007-028','2007-06-28 15:02:20','orbital','success',434,'670-8343-634','Genesis-2','Bigelow Genesis-2','','Sat',1906,'',null from dual union all
select 68371,'2007-029','2007-06-29 10:00:00','orbital','success',1539,'70141-301/1-2005','Kosmos-2428','Tselina-2','','Sat',567,'',null from dual union all
select 68372,'2007-030','2007-07-02 19:38:41','orbital','success',680,'47163-313 SL','SAR-Lupe 2','SAR-Lupe 2','','Sat',456,'',null from dual union all
select 68373,'2007-031','2007-07-05 12:08:00','orbital','success',313,'Y10      (101)','Zhongxing 6B','zhongxing liu B weixing','','Sat',1903,'',null from dual union all
select 68374,'2007-032','2007-07-07 01:16:00','orbital','success',962,'535-20/88520','DirecTV-10','DirecTV 10','','Sat',553,'',null from dual union all
select 68375,'2007-033','2007-08-02 17:33:47','orbital','success',1241,'Ts15000-108 108','Progress M-61','Progress 7K-TGM No. 361','','Sat ISS 26P',480,'',null from dual union all
select 68376,'2007-034','2007-08-04 09:26:34','deep space','success',427,'D325','Mars Phoenix','Phoenix Lander','','Sat',285,'',null from dual union all
select 68377,'2007-035','2007-08-08 22:36:42','orbital','success',1243,'STS-118','ISS 13A.1','OV-105 ISS','','Sat',921,'',null from dual union all
select 68378,'2007-036','2007-08-14 23:44:00','orbital','success',111,'V177 (537)','Spaceway 3/BSat 3A','Spaceway 3','','Sat',346,'',null from dual union all
select 68379,'2007-037','2007-09-02 12:51:00','orbital','failure',502,'GSLV-F04','Insat 4CR','INSAT 4CR','','Sat',1359,'',null from dual union all
select 6838,'1960-S19','1960-01-26 00:00:00','military missile','success',1053,'CC-2011','Capsule     /127 km','Bat B','','OT',1854,'',90 from dual union all
select 68380,'2007-038','2007-09-11 13:05:06','orbital','success',680,'53752-108','Kosmos-2429','Parus?','','Sat',456,'',null from dual union all
select 68381,'2007-039','2007-09-14 01:31:01','deep space','success',529,'H-IIA-13','SELENE','SELENE','','Sat',1477,'',null from dual union all
select 68382,'2007-040','2007-09-14 11:00:00','orbital','success',1241,'Ts15000-098 098','Foton-M No. 3','Foton-M No. 3','','Sat',480,'',null from dual union all
select 68383,'2007-041','2007-09-18 18:35:00','orbital','success',418,'D326','WorldView-1','WorldView-1','','Sat',1724,'',null from dual union all
select 68384,'2007-042','2007-09-19 03:26:13','orbital','success',319,'Y17      (102)','CBERS-2B','ziyuan yihao 02B weixing','','Sat',1492,'',null from dual union all
select 68385,'2007-043','2007-09-27 11:34:00','deep space','success',428,'D327','Dawn','Dawn','','Sat',286,'',null from dual union all
select 68386,'2007-044','2007-10-05 22:02:00','orbital','success',116,'V178 (526)','IS-11/Optus D2','Optus D2','','Sat',346,'',null from dual union all
select 68387,'2007-045','2007-10-10 13:22:39','orbital','success',1233,'Ts15000-020 020','Soyuz TMA-11','Soyuz 11F732 No. 221','','Sat',480,'',null from dual union all
select 68388,'2007-046','2007-10-11 00:22:00','orbital','success',186,'AV-011','USA 195 (WGS SV-1)','Wideband Global Satcom 1','','Sat',292,'',null from dual union all
select 68389,'2007-047','2007-10-17 12:23:00','orbital','success',427,'D328','USA 196 (GPS IIRM-4)','Navstar GPS SVN 55','','Sat',285,'',null from dual union all
select 6839,'1960-S190','1960-07-22 12:05:00','suborbital rocket','success',607,'NASA 5.01GT','','','','Aeronomy/Test',1797,'',224 from dual union all
select 68390,'2007-048','2007-10-20 20:12:25','orbital','success',1232,'Ts15000-022 022','Globalstar','Globalstar','','Sat ISS 15S',562,'',null from dual union all
select 68391,'2007-049','2007-10-23 04:39:17','orbital','success',803,'77046  -692 049','Kosmos-2430','Oko US-KS','','Sat',462,'',null from dual union all
select 68392,'2007-050','2007-10-23 15:38:19','orbital','success',1243,'STS-120','ISS 10A','OV-103 ISS','','Sat',921,'',null from dual union all
select 68393,'2007-051','2007-10-24 10:05:04','deep space','success',309,'Y14      (103)','Chang''e-1','chang''e yihao weixing','','Sat',1904,'',null from dual union all
select 68394,'2007-052','2007-10-26 07:35:24','orbital','success',958,'410-17','Kosmos-2431/2433','Uragan-M No. 718','','Sat',588,'',null from dual union all
select 68395,'2007-053','2007-11-01 00:51:44','orbital','success',680,'53765-632 SL','SAR-Lupe 3','SAR-Lupe 3','','Sat',456,'',null from dual union all
select 68396,'2007-054','2007-11-11 01:50:00','orbital','success',397,'D4-8 (329) 4050H','USA 197 (DSP 23)','DSP F23','','Sat',290,'',null from dual union all
select 68397,'2007-055','2007-11-11 22:48:00','orbital','success',320,'Y3       (104)','Yaogan 3','yaogan weixing sanhao','','Sat',1492,'',null from dual union all
select 68398,'2007-056','2007-11-14 22:06:00','orbital','success',111,'V179 (538)','Skynet 5B/StarOne C1','Star One C1','','Sat',346,'',null from dual union all
select 68399,'2007-057','2007-11-17 22:39:47','orbital','success',962,'535-23/88523','Sirius 4','Sirius 4','','Sat',553,'',null from dual union all
select 684,'1947-A20','1947-10-02 00:00:00','suborbital rocket','success',58,'A2','Dummy','','','Test',1869,'',1 from dual union all
select 6840,'1960-S191','1960-07-23 00:00:00','military missile','success',994,'','NIIP-A target','','','Target',634,'',100 from dual union all
select 68400,'2007-058','2007-12-09 00:16:00','orbital','success',962,'535-26/88526','Raduga-1M  (Globus)','Globus-1M No. 11L','','Sat',588,'',null from dual union all
select 68401,'2007-059','2007-12-09 02:31:42','orbital','success',413,'D330','COSMO 2','COSMO-Skymed 2','','Sat',1724,'',null from dual union all
select 68402,'2007-060','2007-12-10 22:05:00','orbital','success',182,'AV-015','USA 198      [SDS16]','QUASAR 15?','','Sat',292,'',null from dual union all
select 68403,'2007-061','2007-12-14 13:17:34','orbital','success',1232,'Ts15000-025 025','Radarsat-2','Radarsat-2','','Sat',562,'',null from dual union all
select 68404,'2007-062','2007-12-20 20:04:00','orbital','success',427,'D331','USA 199 (GPS IIRM-5)','Navstar GPS SVN 57','','Sat',285,'',null from dual union all
select 68405,'2007-063','2007-12-21 21:41:55','orbital','success',116,'V180 (530)','Rascom 1/Horizons 2','Rascom 1','','Sat',346,'',null from dual union all
select 68406,'2007-064','2007-12-23 07:12:41','orbital','success',1241,'Sh15000-109 109','Progress M-62','Progress 7K-TGM No.362','','Sat ISS 27P',480,'',null from dual union all
select 68407,'2007-065','2007-12-25 19:32:34','orbital','success',963,'535-28','Kosmos-2434/2436','Uragan-M No. 721','','Sat',588,'',null from dual union all
select 68408,'2007-A01','2007-02-11 18:18:00','military missile','success',119,'M4','AST   Black Sparrow','','','ATBM Test',1026,'',40 from dual;
insert into LAUNCH
select 68409,'2007-A02','2007-03-26 00:00:00','military missile','success',119,'','','','','Test',1026,'',40 from dual union all
select 6841,'1960-S192','1960-07-24 00:00:00','military missile','success',976,'','8K63 UBP-59','','','Test',636,'',402 from dual union all
select 68410,'2007-A03','2007-05-25 14:15:00','military missile','failure',1111,'M-9','GMD FTG-03 Target','','','Target',862,'',40 from dual union all
select 68411,'2007-A04','2007-05-03 12:50:56','atmospheric rocket','success',587,'','HTR','','','Tech',28,'',13 from dual union all
select 68412,'2007-A05','2007-06-26 00:00:00','military missile','success',1294,'THAAD FTT-05','Low Endo','','','Test',1895,'',40 from dual union all
select 68413,'2007-A06','2007-10-11 08:01:00','military missile','success',3,'','Test','','','Op Test',183,'',40 from dual union all
select 68414,'2007-A07','2007-10-30 08:00:00','military missile','success',3,'','Test','','','Op Test',183,'',40 from dual union all
select 68415,'2007-A08','2007-12-03 16:07:00','military missile','success',891,'','NCADE Target','','','Target',1895,'',20 from dual union all
select 68416,'2007-A09','2007-11-10 00:00:00','military missile','failure',261,'R-30 07','','','','R&D',190,'TK-208',1 from dual union all
select 68417,'2007-A10','2007-02-11 00:00:00','military missile','success',244,'','','','','Test',1026,'F-15D',40 from dual union all
select 68418,'2007-A11','2007-04-26 21:32:00','military missile','success',1089,'Aegis FTM-11 (2-3A)','','','','Test',851,'CG-70',10 from dual union all
select 68419,'2007-A12','2007-12-06 05:35:00','military missile','success',11,'02','AAD Interceptor','','','Test',316,'',15 from dual union all
select 6842,'1960-S193','1960-07-25 08:30:00','suborbital rocket','success',839,'AA6.420','','','','Aeronomy',394,'',120 from dual union all
select 68420,'2007-A13','2007-12-02 05:14:00','military missile','success',11,'01','AAD Interceptor','','','Test',316,'',15 from dual union all
select 68421,'2007-A14','2007-12-05 16:06:25','suborbital rocket','failure',1165,'Shadow 1B','Space Express','','','Tech',1895,'',40 from dual union all
select 68422,'2007-F01','2007-01-30 23:22:00','orbital','failure',1542,'SL24/DM-SL-25L','SL16 NSS 8','NSS 8','','Sat',860,'ODYSSEY',0 from dual union all
select 68423,'2007-F02','2007-03-21 01:10:00','orbital','failure',475,'F002','Demosat','AFSS/LCT2','','Sat',870,'',330 from dual union all
select 68424,'2007-F03','2007-09-05 22:43:10','orbital','failure',962,'535-22/88522','JCSAT 11','JCSAT 11','','Sat',553,'',74 from dual union all
select 68425,'2007-M01','2007-03-22 10:15:00','suborbital rocket','success',602,'1','ERAU/Icarus','','','Test',1821,'',60 from dual union all
select 68426,'2007-M02','2007-04-06 06:45:00','military missile','success',1294,'THAAD FTT-07','Mid Endo','','','ABM Test',855,'',60 from dual union all
select 68427,'2007-M03','2007-03-01 22:28:29','suborbital rocket','success',1024,'','Mini-DUSTY MD-13','','','Test/Aeron',29,'',75 from dual union all
select 68428,'2007-M04','2007-06-19 00:00:00','suborbital rocket','success',1047,'','','','','Test',932,'',50 from dual union all
select 68429,'2007-S01','2007-01-11 22:28:00','military missile','success',354,'ASAT 3','','','','ASAT',1905,'',850 from dual union all
select 6843,'1960-S194','1960-07-25 11:54:00','military missile','success',239,'BK.07','H11','','','Reentry/CR/Ionos',1843,'',531 from dual union all
select 68430,'2007-S02','2007-01-19 12:45:01','suborbital rocket','success',228,'NASA 21.138UE','JOULE II','','','Auroral',1047,'',217 from dual union all
select 68431,'2007-S03','2007-01-19 12:29:01','suborbital rocket','success',224,'NASA 36.234UE','JOULE II','','','Auroral',1047,'',190 from dual union all
select 68432,'2007-S04','2007-01-19 12:30:01','suborbital rocket','success',1334,'NASA 41.064UE','JOULE II','','','Auroral',1047,'',200 from dual union all
select 68433,'2007-S05','2007-01-19 12:46:01','suborbital rocket','success',1334,'NASA 41.065UE','JOULE II','','','Auroral',1047,'',200 from dual union all
select 68434,'2007-S06','2007-01-27 05:18:00','military missile','success',1151,'THAAD FTT-06 Target','FMA Target','','','Target',855,'MLP',100 from dual union all
select 68435,'2007-S07','2007-01-27 05:20:00','military missile','success',1294,'THAAD FTT-06','High Endo','','','ABM Test',855,'',90 from dual union all
select 68436,'2007-S08','2007-01-28 00:00:00','military missile','success',1153,'Long range test','','','','Test',960,'',150 from dual union all
select 68437,'2007-S09','2007-02-07 08:14:00','military missile','success',799,'','Mk 21 RV','GT193GM','GLORY TRIP 193GM','Op Test',1706,'',1300 from dual union all
select 68438,'2007-S10','2007-02-12 12:45:04','suborbital rocket','success',237,'NASA 40.020UE','ROPA','','','Auroral',1047,'',748 from dual union all
select 68439,'2007-S11','2007-02-14 09:22:01','suborbital rocket','success',1345,'NASA 41.061','HEX-II V-1','','','Aeron',1047,'',200 from dual union all
select 6844,'1960-S195','1960-07-26 00:00:00','military missile','success',976,'','8K63 UBP-60','','','Test',636,'',402 from dual union all
select 68440,'2007-S12','2007-02-14 09:27:11','suborbital rocket','success',231,'NASA 35.037','HEX-II H','','','Aeron',1047,'',150 from dual union all
select 68441,'2007-S13','2007-02-14 09:36:11','suborbital rocket','success',1345,'NASA 41.063','HEX-II V-2','','','Aeron',1047,'',200 from dual union all
select 68442,'2007-S14','2007-02-14 09:38:11','suborbital rocket','success',1345,'NASA 41.062','HEX-II V-3','','','Aeron',1047,'',200 from dual union all
select 68443,'2007-S15','2007-02-28 08:39:41','suborbital rocket','success',237,'NASA 40.019UE','CHARM','','','Auroral',1047,'',733 from dual union all
select 68444,'2007-S16','2007-03-06 00:30:00','military missile','success',1098,'','SRALT','','','Target',850,'C-17',100 from dual union all
select 68445,'2007-S17','2007-03-21 04:27:00','military missile','success',784,'TLV-5','TLV Target','FTX-02 SBR Target','','Target',1701,'',1300 from dual union all
select 68446,'2007-S18','2007-04-06 06:42:00','military missile','success',1151,'THAAD FTT-07 Target','FMA Target','','','Target',855,'MLP',100 from dual union all
select 68447,'2007-S19','2007-04-28 14:56:00','suborbital rocket','success',1244,'SpaceLoft SL-2','Celestis Legacy','','','Test/Burial',1422,'',117 from dual union all
select 68448,'2007-S20','2007-04-26 21:31:00','military missile','success',1345,'Aegis FTM-11E4 Tgt','ARAV-A','','','Target',855,'',160 from dual union all
select 68449,'2007-S21','2007-04-26 21:32:00','military missile','success',1090,'Aegis FTM-11E4','Stellar Hunters','','','Test',851,'CG-70',160 from dual union all
select 6845,'1960-S196','1960-07-26 10:17:00','suborbital rocket','success',839,'AA6.421','Firefly JANET','','','Aeronomy',394,'',119 from dual union all
select 68450,'2007-S22','2007-04-12 05:22:00','military missile','success',65,'','','','','Test',316,'',400 from dual union all
select 68451,'2007-S23','2007-05-29 10:20:00','military missile','success',1530,'1L','15Zh55M LKI-1','RS-24','','R&D',608,'',1000 from dual union all
select 68452,'2007-S24','2007-06-15 02:45:00','suborbital rocket','success',1303,'','HYCAUSE','','','Test',1849,'',454 from dual union all
select 68453,'2007-S25','2007-06-21 15:14:00','military missile','success',758,'2','','','','Test',307,'',1000 from dual union all
select 68454,'2007-S26','2007-06-21 05:38:00','suborbital rocket','success',1334,'NASA 12.059GT','Star tracker/ACS','','','Test',1895,'',130 from dual union all
select 68455,'2007-S27','2007-06-23 02:40:00','military missile','success',748,'Aegis FTM-12-3 Targ','MRT-4 MBRV-3','','','Target',855,'',150 from dual union all
select 68456,'2007-S28','2007-06-23 02:44:00','military missile','success',1090,'Aegis FTM-12','Stellar Athena','','','Test',851,'DDG-73',160 from dual union all
select 68457,'2007-S29','2007-07-19 15:13:00','suborbital rocket','success',1484,'V04','Cuma II','','','Micrograv',13,'',242 from dual union all
select 68458,'2007-S30','2007-08-03 22:51:20','suborbital rocket','success',1334,'NASA 41.069','MASS 1 (NLCs)','','','Aeron',29,'',130 from dual union all
select 68459,'2007-S31','2007-08-03 23:22:00','suborbital rocket','success',847,'','ECOMA 3','','','Meteors/Dust',29,'',127 from dual union all
select 6846,'1960-S197','1960-07-26 16:00:00','military missile','success',916,'110','P-06','','','Test',257,'',180 from dual union all
select 68460,'2007-S32','2007-08-06 22:56:00','suborbital rocket','success',1334,'NASA 41.070','MASS 2 (NLCs)','','','Aeron',29,'',130 from dual union all
select 68461,'2007-S33','2007-08-13 05:45:00','suborbital rocket','success',224,'NASA 36.220UG','JHU FOT 18/LIDOS 2','','','UV Astron',1895,'',278 from dual union all
select 68462,'2007-S34','2007-08-23 08:31:00','military missile','success',784,'TLV-7','TLV Target','NFIRE 2a','','Target',1701,'',1300 from dual union all
select 68463,'2007-S35','2007-06-15 00:00:00','military missile','success',1345,'Aegis FTM-12-2 Targ','ARAV-A TRACKEX','','','Target',855,'',160 from dual union all
select 68464,'2007-S36','2007-06-15 00:00:00','military missile','success',1345,'Aegis FTM-12-2 Targ','ARAV-A TRACKEX','','','Target',855,'',160 from dual union all
select 68465,'2007-S37','2007-01-16 02:20:00','suborbital rocket','success',1073,'S-310-37','','','','Ionos',833,'',138 from dual union all
select 68466,'2007-S38','2007-05-15 00:00:00','military missile','success',1443,'FCET-37','','','','OT',421,'SSBN 734',1000 from dual union all
select 68467,'2007-S39','2007-05-15 00:00:00','military missile','success',1443,'FCET-37','','','','OT',421,'SSBN 734',1000 from dual union all
select 68468,'2007-S40','2007-08-07 00:00:00','military missile','success',1507,'VMF RF','1x RV ','','','OT',1011,'K-211',1000 from dual union all
select 68469,'2007-S41','2007-06-28 00:00:00','military missile','success',261,'R-30 06','','','','R&D',190,'TK-208',1000 from dual union all
select 6847,'1960-S198','1960-07-27 10:19:00','suborbital rocket','success',839,'AA6.425','Firefly DOLLY','','','Aeronomy',394,'',115 from dual union all
select 68470,'2007-S42','2007-09-28 20:01:00','military missile','success',1111,'','GMD FTG-03a Target','','','Target',862,'',1000 from dual union all
select 68471,'2007-S43','2007-09-28 20:16:00','military missile','success',497,'6','GMD FTG-03a','','','Test',1709,'',1000 from dual union all
select 68472,'2007-S44','2007-10-18 05:10:00','military missile','success',1434,'64588-343-64','15Zh58 UBP-34','','','OT',608,'',1000 from dual union all
select 68473,'2007-S45','2007-09-06 21:09:00','suborbital rocket','success',1334,'NASA 41.055NP','IRVE','','','Test?',1821,'',130 from dual union all
select 68474,'2007-S46','2007-02-23 00:00:00','military missile','success',1171,'','','','','Test',1419,'',300 from dual union all
select 68475,'2007-S47','2007-03-03 00:00:00','military missile','success',569,'','','','','Test',1419,'',70 from dual union all
select 68476,'2007-S48','2007-03-30 09:05:00','military missile','success',430,'5','Ship test','','','Test',320,'INS-P51',50 from dual union all
select 68477,'2007-S49','2007-05-09 05:10:00','military missile','success',941,'','','','','Test',315,'',100 from dual union all
select 68478,'2007-S50','2007-10-05 05:07:00','military missile','success',62,'','Agni RV','','','Test',316,'',300 from dual union all
select 68479,'2007-S51','2007-10-24 00:00:00','military missile','success',62,'','Agni RV','','','Test',316,'',300 from dual union all
select 6848,'1960-S199','1960-07-28 00:00:00','military missile','success',976,'','8K63 UBP-61','','','Test',636,'',402 from dual union all
select 68480,'2007-S52','2007-10-27 07:15:00','military missile','success',1151,'THAAD FTT-08 Target','FMA Target','','','Target',855,'MLP',100 from dual union all
select 68481,'2007-S53','2007-10-27 07:18:00','military missile','success',1294,'THAAD FTT-08','Exo','','','ABM Test',855,'',100 from dual union all
select 68482,'2007-S54','2007-10-29 09:00:00','military missile','success',1453,'492-3821-113','15A35 UBP-55','','','OT',533,'',1000 from dual union all
select 68483,'2007-S55','2007-10-30 04:12:52','suborbital rocket','success',224,'NASA 36.218UE','STORMS (MidLat SpF)','','','Plasma',1821,'',395 from dual union all
select 68484,'2007-S56','2007-11-06 00:00:00','military missile','success',1110,'Ballistic Target','PAC-2 target','','','Test',984,'',85 from dual union all
select 68485,'2007-S57','2007-11-07 04:14:00','military missile','success',1090,'Aegis FTM-13a','Stellar Gryphon','','','Test',851,'CG-70',160 from dual union all
select 68486,'2007-S58','2007-11-07 04:14:00','military missile','success',1090,'Aegis FTM-13b','','','','Test',851,'CG-70',160 from dual union all
select 68487,'2007-S59','2007-11-07 04:12:00','military missile','success',1345,'Aegis FTM-13 Target','ARAV-A','','','Target',855,'',150 from dual union all
select 68488,'2007-S60','2007-11-07 04:13:00','military missile','success',1345,'Aegis FTM-13 Target','ARAV-A','','','Target',855,'',150 from dual union all
select 68489,'2007-S61','2007-11-06 18:00:00','suborbital rocket','success',224,'NASA 36.241GS','EUNIS-07','','','Solar EUV',1895,'',300 from dual union all
select 6849,'1960-S20','1960-01-26 00:48:00','military missile','success',638,'AM-28','R&D','','','Test',249,'',500 from dual union all
select 68490,'2007-S62','2007-11-15 00:00:00','military missile','success',748,'Aegis JFTM Target','MBRV-3','','','Target',855,'',160 from dual union all
select 68491,'2007-S63','2007-12-16 09:15:38','suborbital rocket','success',1478,'V07','Angicos','','','Tech/GPS',330,'',120 from dual union all
select 68492,'2007-S64','2007-12-17 00:00:00','military missile','success',1179,'VMF RF','','','','OT',201,'K-114',1000 from dual union all
select 68493,'2007-S65','2007-12-25 10:00:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-114',1000 from dual union all
select 68494,'2007-S66','2007-12-08 14:43:00','military missile','success',1435,'','15Zh58E GCh-2','','','OT',636,'',1000 from dual union all
select 68495,'2007-S67','2007-12-25 13:10:00','military missile','success',1530,'2L RS-24','15Zh55M LKI-2','','','R&D',608,'',1000 from dual union all
select 68496,'2007-S68','2007-12-17 22:05:00','military missile','success',748,'Aegis JFTM-1 Target','MRT-5 MBRV-3 Sep','','','Target',855,'',150 from dual union all
select 68497,'2007-S69','2007-12-17 22:08:00','military missile','success',1090,'Aegis JFTM-1','Stellar Kiji','','','Test',851,'DDG-173',160 from dual union all
select 68498,'2007-S70','2007-12-10 09:00:00','suborbital rocket','success',237,'NASA 40.018UE','TRICE High','','','Plasma',28,'',1150 from dual union all
select 68499,'2007-S71','2007-12-10 09:02:00','suborbital rocket','success',237,'NASA 40.022UE','TRICE Low','','','Plasma',23,'',630 from dual;
insert into LAUNCH
select 685,'1947-A21','1947-10-07 00:00:00','atmospheric rocket','success',876,'47-G','Nike-47 Round F','','','Test',1863,'',10 from dual union all
select 6850,'1960-S200','1960-07-28 21:38:00','military missile','failure',1419,'J-4','Mk 4-1 RV','','','Test',239,'',100 from dual union all
select 68500,'2007-S72','2007-11-29 00:00:00','military missile','success',1443,'DASO-19','-    (120ss)','','','OT',1897,'SSBN 730',1000 from dual union all
select 68501,'2007-S73','2007-09-13 05:50:00','suborbital rocket','success',1221,'6','Materials/Recovery','','','Micrograv/Aeron',808,'',280 from dual union all
select 68502,'2007-S74','2007-09-02 10:20:00','suborbital rocket','success',1074,'S-520-23','WIND','','','Ionos',833,'',279 from dual union all
select 68503,'2007-S75','2007-01-01 00:00:00','suborbital rocket','success',728,'Ionos','[Restart launches]','','','Aeron/Ionos',1010,'',85 from dual union all
select 68504,'2007-S76','2007-11-20 00:00:00','military missile','success',121,'R&D','','','','Test',1348,'',300 from dual union all
select 68505,'2007-S77','2007-12-06 05:30:00','military missile','success',941,'','AAD Target','','','Target',315,'',110 from dual union all
select 68506,'2007-S78','2007-08-31 00:00:00','military missile','success',1019,'Aegis FTM-11a Target','','','','Target',855,'',160 from dual union all
select 68507,'2007-S79','2007-08-31 00:00:00','military missile','success',1090,'Aegis FTM-11a','','','','Test',851,'AEGIS',160 from dual union all
select 68508,'2007-S80','2007-12-17 00:00:00','military missile','success',1344,'JFTM-1 TRACKEX','ARAV-B4 TRACKEX','','','Target',855,'',150 from dual union all
select 68509,'2008-001','2008-01-15 11:48:59','orbital','success',1542,'SL25/DM-SL-15L','SL36 Thuraya 3','Thuraya 3','','Sat',860,'ODYSSEY',null from dual union all
select 6851,'1960-S201','1960-07-28 08:30:00','suborbital rocket','unknown',839,'AA6.419','Firefly AMY','','','Aeron',394,'',111 from dual union all
select 68510,'2008-002','2008-01-21 03:45:00','orbital','success',900,'PSLV-C10','Tecsar','TecSAR-1','','Sat',1357,'',null from dual union all
select 68511,'2008-003','2008-01-28 00:18:00','orbital','success',962,'535-27/88530','Ekspress AM-33','Ekspress AM-33','','Sat',553,'',null from dual union all
select 68512,'2008-004','2008-02-05 13:02:57','orbital','success',1241,'Ts15000-106 106','Progress M-63','Progress 7K-TGM No.363','','Sat ISS 28P',480,'',null from dual union all
select 68513,'2008-005','2008-02-07 19:45:30','orbital','success',1243,'STS-122','ISS 1E','OV-104 ISS','','Sat',921,'',null from dual union all
select 68514,'2008-006','2008-02-11 11:34:00','orbital','success',962,'535-24/88524','Thor 5','Thor 5','','Sat',553,'',null from dual union all
select 68515,'2008-007','2008-02-23 08:55:00','orbital','success',530,'H-IIA-14','WINDS','WINDS','','Sat',1477,'',null from dual union all
select 68516,'2008-008','2008-03-09 04:03:11','orbital','success',113,'V181 (528)','ATV 1','ATV 1','','Sat',346,'',null from dual union all
select 68517,'2008-009','2008-03-11 06:28:14','orbital','success',1243,'STS-123','ISS 1J/A','OV-105 ISS','','Sat',921,'',null from dual union all
select 68518,'2008-010','2008-03-13 10:02:00','orbital','success',184,'AV-006','USA 200       [JS12]','Adv TRUMPET 2','','Sat',1775,'',null from dual union all
select 68519,'2008-011','2008-03-14 23:18:55','orbital','failure',962,'535-25/88525','AMC 14','Americom 14','','Sat',553,'',null from dual union all
select 6852,'1960-S202','1960-07-28 19:36:00','suborbital rocket','success',720,'LT12','','','','Aeron',1837,'',126 from dual union all
select 68520,'2008-012','2008-03-15 06:10:00','orbital','success',427,'D332','USA 201 (GPS IIRM-6)','Navstar GPS SVN 48','','Sat',285,'',null from dual union all
select 68521,'2008-013','2008-03-19 22:47:59','orbital','success',1542,'SL26/DM-SL-26L','SL32 DirecTV 11','DirecTV-11','','Sat',860,'ODYSSEY',null from dual union all
select 68522,'2008-014','2008-03-27 17:16:18','orbital','success',680,'53713-326 SL','SAR-Lupe 4','SAR-Lupe 4','','Sat',456,'',null from dual union all
select 68523,'2008-015','2008-04-08 11:16:39','orbital','success',1233,'Sh15000-024 024','Soyuz TMA-12','Soyuz 11F732 No. 222','','Sat ISS 16S',480,'',null from dual union all
select 68524,'2008-016','2008-04-14 20:12:00','orbital','success',186,'AV-014','ICO G1','ICO G1','','Sat',292,'',null from dual union all
select 68525,'2008-017','2008-04-16 17:02:48','orbital','success',911,'F39','C/NOFS','C/NOFS','','Sat',872,'L-1011',null from dual union all
select 68526,'2008-018','2008-04-18 22:17:00','orbital','success',111,'V182 (539)','Star One C2/Vinasat','Vinasat','','Sat',346,'',null from dual union all
select 68527,'2008-019','2008-04-25 15:35:11','orbital','success',316,'Y1      (105)','Tian Lian 1','tianlian yihao 01 xing','','Sat',1903,'',null from dual union all
select 68528,'2008-020','2008-04-26 22:16:02','orbital','success',1232,'P 15000-016 016','GIOVE B','Giove B','','Sat',562,'',null from dual union all
select 68529,'2008-021','2008-04-28 03:53:51','orbital','success',900,'PSLV-C9','Cartosat-2A/IMS-1','Cartosat-2A','','Sat',1359,'',null from dual union all
select 6853,'1960-S203','1960-07-29 00:00:00','military missile','success',976,'','8K63 UBP-62','','','Test',636,'',402 from dual union all
select 68530,'2008-022','2008-04-28 05:00:00','orbital','success',1543,'SB60.1/SLB 1TL','AMOS 3','Amos 3','','Sat',567,'',null from dual union all
select 68531,'2008-023','2008-05-14 20:22:56','orbital','success',1241,'Sh15000-110 110','Progress M-64','Progress 7K-TGM No.364','','Sat ISS 29P',480,'',null from dual union all
select 68532,'2008-024','2008-05-21 09:43:59','orbital','success',1542,'SL27/DM-SL-27L','SL30 Galaxy 18','Galaxy 18','','Sat',860,'ODYSSEY',null from dual union all
select 68533,'2008-025','2008-05-23 15:20:09','orbital','success',1064,'511-3922-114/72509','Kosmos-2437-2439/Yub','Yubileyniy','','Sat',460,'',null from dual union all
select 68534,'2008-026','2008-05-27 03:02:33','orbital','success',320,'Y2       (106)','FY-3A','fengyun sanhao 01 xing','','Sat',1492,'',null from dual union all
select 68535,'2008-027','2008-05-31 21:02:12','orbital','success',1243,'STS-124','ISS IJ','OV-103 ISS','','Sat',921,'',null from dual union all
select 68536,'2008-028','2008-06-09 12:15:04','orbital','success',313,'Y11      (107)','Zhongxing-9','zhongxing jiuhao','','Sat',1903,'',null from dual union all
select 68537,'2008-029','2008-06-11 16:05:01','orbital','success',421,'D333','GLAST','GLAST','','Sat',286,'',null from dual union all
select 68538,'2008-030','2008-06-12 22:05:09','orbital','success',111,'V183 (540)','Skynet 5C/Turksat 3A','Skynet 5C','','Sat',346,'',null from dual union all
select 68539,'2008-031','2008-06-19 06:36:46','orbital','success',680,'53781-708 KL','Orbcomm CDS/R1-R5','Orbcomm FM38','','Sat',621,'',null from dual union all
select 6854,'1960-S204','1960-07-29 08:30:00','suborbital rocket','success',839,'AA6.422','Firefly CATHY','','','Aeronomy',394,'',94 from dual union all
select 68540,'2008-032','2008-06-20 07:46:25','orbital','success',411,'D334','Jason-2','Jason-2','','Sat',1724,'',null from dual union all
select 68541,'2008-033','2008-06-26 23:59:43','orbital','success',958,'410-14/111L','Kosmos-2440','Oko-1 No. 7127','','Sat',588,'',null from dual union all
select 68542,'2008-034','2008-07-07 21:47:00','orbital','success',111,'V184 (541)','Protostar 1/Badr 6','Protostar 1','','Sat',346,'',null from dual union all
select 68543,'2008-035','2008-07-16 05:20:59','orbital','success',1542,'SL28/DM-SL-28L','SL34 Echostar 11','Echostar 11','','Sat',860,'ODYSSEY',null from dual union all
select 68544,'2008-036','2008-07-22 02:40:09','orbital','success',680,'53752-101 SL','SAR-Lupe 5','SAR-Lupe 5','','Sat',456,'',null from dual union all
select 68545,'2008-037','2008-07-26 18:31:36','orbital','success',1229,'77057  -145 003','Kosmos-2441','Persona No. 1','','Sat',475,'',null from dual union all
select 68546,'2008-038','2008-08-14 20:44:00','orbital','success',111,'V185 (542)','Superbird 7/AMC 21','Superbird 7','','Sat',346,'',null from dual union all
select 68547,'2008-039','2008-08-18 22:43:00','orbital','success',962,'935-02/99502','Inmarsat 4 F3','INMARSAT 4 F3','','Sat',553,'',null from dual union all
select 68548,'2008-040','2008-08-29 07:15:58','orbital','success',434,'670-8343-638','RapidEye 1-5','RapidEye 2','','Sat SSO',497,'',null from dual union all
select 68549,'2008-041','2008-09-06 03:25:00','orbital','success',298,'Y1       (108)','HJ-1A/HJ-1B','huanjing jian zai A xing','','Sat',1492,'',null from dual union all
select 6855,'1960-S205','1960-07-29 10:14:00','suborbital rocket','success',839,'AA6.407','Firefly MAVIS','','','Aeronomy',394,'',94 from dual union all
select 68550,'2008-042','2008-09-06 18:50:57','orbital','success',413,'D335','GeoEye-1','GeoEye-1','','Sat',1724,'',null from dual union all
select 68551,'2008-043','2008-09-10 19:50:02','orbital','success',1241,'Sh15000-111 111','Progress M-65','Progress 7K-TGM No.365','','Sat ISS 30P',480,'',null from dual union all
select 68552,'2008-044','2008-09-19 21:48:00','orbital','success',962,'535-29/88528','Nimiq 4','Nimiq 4','','Sat',553,'',null from dual union all
select 68553,'2008-045','2008-09-24 09:27:59','orbital','success',1542,'SL29/DM-SL-29L','SL35 Galaxy 19','Galaxy 19','','Sat',860,'ODYSSEY',null from dual union all
select 68554,'2008-046','2008-09-25 08:49:37','orbital','success',963,'535-31/112L','Kosmos-2442/2444','Uragan-M No. 724','','Sat',587,'',null from dual union all
select 68555,'2008-047','2008-09-25 13:10:00','orbital','success',306,'Y7       (109)','Shenzhou 7','shenzhou qihao fanhui ca','','Sat',814,'',null from dual union all
select 68556,'2008-048','2008-09-28 23:15:00','orbital','success',475,'F004','Ratsat','Mass simulator','','Sat',870,'',null from dual union all
select 68557,'2008-049','2008-10-01 06:37:21','orbital','success',434,'670-8343-637','THEOS','THEOS','','Sat',1907,'',null from dual union all
select 68558,'2008-050','2008-10-12 07:01:33','orbital','success',1233,'Sh15000-026 026','Soyuz TMA-13','Soyuz 11F732A17 No. 223','','Sat ISS 17S',480,'',null from dual union all
select 68559,'2008-051','2008-10-19 17:47:23','deep space','success',911,'F40','IBEX','IBEX','','Sat',872,'L-1011',null from dual union all
select 6856,'1960-S206','1960-07-30 20:20:00','military missile','success',928,'A1E-4 (A1XP-4)','','','','Test',421,'SSBN 598',500 from dual union all
select 68560,'2008-052','2008-10-22 00:52:11','deep space','success',902,'PSLV-C11','Chandraayan-1','Chandrayaan-1','','Sat',1359,'',null from dual union all
select 68561,'2008-053','2008-10-25 01:15:00','orbital','success',319,'Y22      (110)','SJ-6-3A/SJ-6-3B','shijian liuhao san zu B','','Sat',1493,'',null from dual union all
select 68562,'2008-054','2008-10-25 02:28:25','orbital','success',413,'D336','COSMO 3','COSMO-Skymed 3','','Sat',1724,'',null from dual union all
select 68563,'2008-055','2008-10-29 16:53:43','orbital','success',311,'Y12      (111)','Venesat-1','Venesat','','Sat',1903,'',null from dual union all
select 68564,'2008-056','2008-11-05 00:15:04','orbital','success',303,'Y12      (112)','SW-3/CX-1-02','shiyan weixing sanhao','','Sat',813,'',null from dual union all
select 68565,'2008-057','2008-11-05 20:44:20','orbital','success',962,'535-33/88531','Astra 1M','Astra 1M','','Sat',553,'',null from dual union all
select 68566,'2008-058','2008-11-14 15:50:00','orbital','success',1241,'77057  -146 103','Kosmos-2445','Kobal''t-M','','Sat',462,'',null from dual union all
select 68567,'2008-059','2008-11-15 00:55:39','orbital','success',1243,'STS-126','ISS ULF-2','OV-105 ISS','','Sat',921,'',null from dual union all
select 68568,'2008-060','2008-11-26 12:38:38','orbital','success',1241,'Sh15000-114 114','Progress M-01M','Progress 7K-TGM No.401','','Sat ISS 31P',480,'',null from dual union all
select 68569,'2008-061','2008-12-01 04:42:00','orbital','success',303,'Y9       (113)','Yaogan 4','yaogan weixing sihao','','Sat',813,'',null from dual union all
select 6857,'1960-S207','1960-08-01 00:00:00','military missile','success',976,'','8K63 UBP-67','','','Test',636,'',402 from dual union all
select 68570,'2008-062','2008-12-02 04:59:59','orbital','success',803,'77046  -693 050','Kosmos-2446','Oko US-KS?','','Sat',462,'',null from dual union all
select 68571,'2008-063','2008-12-10 13:43:00','orbital','success',962,'935-03/99503','Ciel 2','Ciel 2','','Sat',553,'',null from dual union all
select 68572,'2008-064','2008-12-15 03:22:00','orbital','success',319,'Y20      (114)','Yaogan 5','yaogan weixing wuhao','','Sat',1493,'',null from dual union all
select 68573,'2008-065','2008-12-20 22:35:14','orbital','success',111,'V186 (543)','Hot Bird 9/Eut. W2M','Hot Bird 9','','Sat',346,'',null from dual union all
select 68574,'2008-066','2008-12-23 00:54:00','orbital','success',309,'Y20      (115)','FY-2-06','fengyun erhao 06 xing','','Sat',1904,'',null from dual union all
select 68575,'2008-067','2008-12-25 10:43:42','orbital','success',963,'535-34/114L','Kosmos-2447/2449','Uragan-M No. 727','','Sat',588,'',null from dual union all
select 68576,'2008-A01','2008-05-07 19:18:00','suborbital rocket','failure',774,'NASA 12.065GT','','','','Test',1801,'',15 from dual union all
select 68577,'2008-A02','2008-05-18 23:15:00','suborbital rocket','success',1048,'','RX-3228.02.01','','','Test',932,'',42 from dual union all
select 68578,'2008-A03','2008-05-23 04:56:00','military missile','success',942,'2','','','','Training',320,'',44 from dual union all
select 68579,'2008-A04','2008-06-05 18:17:00','military missile','success',1088,'Aegis FTM-14','Stellar Scorpion','','','Test Sea-Based Terminal',851,'CG-70',19 from dual union all
select 6858,'1960-S208','1960-08-01 00:00:00','military missile','success',976,'','8K63 UBP-71','','','Test',636,'',402 from dual union all
select 68580,'2008-A05','2008-06-05 18:17:00','military missile','success',1088,'Aegis FTM-14','Stellar Scorpion','','','Test Sea-Based Terminal',851,'CG-70',19 from dual union all
select 68581,'2008-A06','2008-08-22 09:10:00','suborbital rocket','failure',15,'X-1','HyBolt/SOAREX-6','','','Test/Reentry Test',945,'',10 from dual union all
select 68582,'2008-A07','2008-09-17 13:55:00','military missile','success',1019,'TBM Target','Japan PAC-3 target','','','Target',1895,'',40 from dual union all
select 68583,'2008-A08','2008-09-11 10:26:00','suborbital rocket','failure',728,'Ionos','[Accidental launch]','','','Aeron/Ionos',1010,'',10 from dual union all
select 68584,'2008-A09','2008-07-02 00:00:00','suborbital rocket','success',1048,'','','','','Test',932,'',40 from dual union all
select 68585,'2008-F01','2008-08-03 03:34:00','orbital','failure',475,'F003','Trailblazer','ORS Trailblazer','','Sat',870,'',40 from dual union all
select 68586,'2008-F02','2008-08-16 19:32:00','orbital','failure',1114,'OES.0001','Omid','','','Test',1347,'',250 from dual union all
select 68587,'2008-M01','2008-03-28 00:00:00','suborbital rocket','success',1024,'','Mini-DUSTY MD-14','','','Aeron',29,'',75 from dual union all
select 68588,'2008-M02','2008-06-26 02:22:00','military missile','success',1294,'THAAD FTT-09','Mid Endo','','','ABM Test',855,'',60 from dual union all
select 68589,'2008-M03','2008-06-27 10:36:00','suborbital rocket','success',891,'NASA 30.074NO','RockOn','','','Education',1821,'',67 from dual;
insert into LAUNCH
select 6859,'1960-S209','1960-08-01 00:00:00','military missile','success',978,'','','','','Test',1626,'ZULUV',150 from dual union all
select 68590,'2008-M04','2008-04-10 00:00:00','suborbital rocket','success',520,'','AGMD Round','','','Test',1792,'',50 from dual union all
select 68591,'2008-M05','2008-04-10 00:00:00','suborbital rocket','success',520,'','AGMD Round','','','Test',1792,'',50 from dual union all
select 68592,'2008-M06','2008-04-10 00:00:00','suborbital rocket','success',520,'','AGMD Round','','','Test',1792,'',50 from dual union all
select 68593,'2008-M07','2008-04-10 00:00:00','suborbital rocket','success',520,'','AGMD Round (Inst)','','','Test',1792,'',50 from dual union all
select 68594,'2008-M08','2008-04-10 00:00:00','suborbital rocket','success',520,'','AGMD Round (Inst)','','','Test',1792,'',50 from dual union all
select 68595,'2008-M09','2008-02-13 00:00:00','military missile','success',510,'Hatf III','','','','Test',1419,'',50 from dual union all
select 68596,'2008-M10','2008-08-08 00:00:00','military missile','success',610,'','S Ossetia war','','','Test',995,'',50 from dual union all
select 68597,'2008-S01','2008-01-11 05:23:00','suborbital rocket','success',221,'NASA 36.243UG','JHU FOT 19/LIDOS 3','','','UV Astron',1873,'',315 from dual union all
select 68598,'2008-S02','2008-01-17 06:45:00','military missile','success',628,'M1','','','','Test',1026,'',300 from dual union all
select 68599,'2008-S03','2008-01-18 07:30:00','suborbital rocket','success',237,'NASA 40.021UE','SCIFER-2','','','Plasma',29,'',1460 from dual union all
select 686,'1947-A22','1947-10-16 00:00:00','atmospheric rocket','success',876,'47-H','Nike-47 Round G','','','Test',1863,'',10 from dual union all
select 6860,'1960-S21','1960-01-26 05:42:00','suborbital rocket','success',624,'NASA 8.02GT','X248 test','','','Test',1821,'',951 from dual union all
select 68600,'2008-S04','2008-01-25 00:00:00','military missile','success',1169,'Hatf IV','','','','Test',1419,'',100 from dual union all
select 68601,'2008-S05','2008-01-31 19:14:00','suborbital rocket','success',1479,'V04','ARR HotPay-2','','','Aeron/Auroral',29,'',381 from dual union all
select 68602,'2008-S07','2008-02-06 09:14:40','suborbital rocket','success',1073,'S-310-38','VLF/Chaff','','','Ionos',833,'',157 from dual union all
select 68603,'2008-S08','2008-02-07 11:30:00','suborbital rocket','success',1484,'V05','DLR TEXUS 44 (EML-2)','','','Micrograv',416,'',264 from dual union all
select 68604,'2008-S09','2008-02-21 06:15:00','suborbital rocket','success',1484,'V06','DLR TEXUS 45','','','Micrograv/Bio',416,'',270 from dual union all
select 68605,'2008-S10','2008-02-21 03:26:00','military missile','success',1090,'Op. Burnt Frost','USA 193 intercept','','','Test',851,'CG-70',250 from dual union all
select 68606,'2008-S11','2008-02-26 07:28:00','military missile','success',639,'1','Sagarika','','','Test',1750,'',300 from dual union all
select 68607,'2008-S12','2008-03-23 04:45:00','military missile','success',62,'','Agni RV','','','Test',316,'',300 from dual union all
select 68608,'2008-S13','2008-04-02 08:01:00','military missile','success',799,'','Mk 21 RV','GT196GM','GLORY TRIP 196GM','Op Test',1705,'',1300 from dual union all
select 68609,'2008-S14','2008-04-14 16:58:00','suborbital rocket','success',224,'NASA 36.240UE','LASP/CU-12 (SEE/EVE)','','','Solar EUV',1895,'',300 from dual union all
select 6861,'1960-S210','1960-08-03 00:00:00','military missile','success',979,'','8K65 LKI-3','','','Test',636,'',675 from dual union all
select 68610,'2008-S15','2008-04-15 00:00:00','military missile','success',250,'1','','','','Test',951,'F-15D',100 from dual union all
select 68611,'2008-S16','2008-04-19 00:00:00','military missile','success',1171,'Hatf-VI','','','','Test',1419,'',300 from dual union all
select 68612,'2008-S17','2008-04-21 00:00:00','military missile','success',1171,'Hatf-VI','','','','Test',1419,'',300 from dual union all
select 68613,'2008-S18','2008-05-01 05:30:00','suborbital rocket','success',224,'NASA 36.223UH','XQC F4','','','XR Astron',1895,'',300 from dual union all
select 68614,'2008-S19','2008-05-07 04:26:00','military missile','success',65,'','','','','Test',316,'',350 from dual union all
select 68615,'2008-S20','2008-05-06 18:08:00','suborbital rocket','success',774,'NASA 12.066GT','','','','Test',1801,'',85 from dual union all
select 68616,'2008-S21','2008-05-08 00:00:00','military missile','success',1443,'FCET-38','','','','OT',1897,'SSBN 739',1000 from dual union all
select 68617,'2008-S22','2008-05-15 04:00:00','suborbital rocket','success',1484,'V08','SSC MASER 11','','','Micrograv',416,'',252 from dual union all
select 68618,'2008-S23','2008-05-08 00:00:00','military missile','success',1443,'FCET-38','-    (122ss)','','','OT',1897,'SSBN 739',1000 from dual union all
select 68619,'2008-S24','2008-05-22 10:04:00','military missile','success',799,'','Mk 21 RV','GT197GM','GLORY TRIP 197GM','Op Test (Guam)',1706,'',1300 from dual union all
select 6862,'1960-S211','1960-08-01 00:00:00','military missile','success',979,'','8K65 LKI-4','','','Test',636,'',675 from dual union all
select 68620,'2008-S25','2008-05-29 00:00:00','military missile','success',615,'','','','','Test',1909,'XIA',1000 from dual union all
select 68621,'2008-S26','2008-06-05 18:13:00','military missile','success',1151,'Aegis FTM-14 Target','FMA Target','','','Target',855,'MLP',150 from dual union all
select 68622,'2008-S27','2008-06-13 00:00:00','military missile','success',1344,'FTM-14 TRACKEX','ARAV-B5 TRACKEX','','','Target',855,'',150 from dual union all
select 68623,'2008-S28','2008-06-13 00:00:00','military missile','success',1344,'FTM-14 TRACKEX','ARAV-B6 TRACKEX','','','Target',855,'',150 from dual union all
select 68624,'2008-S29','2008-06-26 02:16:00','military missile','success',748,'THAAD FTT-09 Target','MRT-6 (Sep)','','','Target',1208,'C-17',500 from dual union all
select 68625,'2008-S30','2008-06-26 19:57:00','suborbital rocket','success',234,'NASA 39.008DR','NGSP-1','','','Sensor test/Decoys',1797,'',300 from dual union all
select 68626,'2008-S31','2008-06-30 13:22:00','suborbital rocket','success',847,'','ECOMA 4','','','Meteors/Dust',29,'',130 from dual union all
select 68627,'2008-S32','2008-07-08 03:30:00','military missile','success',508,'','','','','Test',1356,'',500 from dual union all
select 68628,'2008-S33','2008-07-07 21:24:00','suborbital rocket','success',847,'','ECOMA 5','','','Meteors/Dust',29,'',127 from dual union all
select 68629,'2008-S34','2008-07-12 10:46:00','suborbital rocket','success',847,'','ECOMA 6','','','Meteors/Dust',29,'',127 from dual union all
select 6863,'1960-S212','1960-08-01 00:00:00','military missile','success',979,'','8K65 LKI-5','','','Test',636,'',675 from dual union all
select 68630,'2008-S35','2008-07-14 10:10:00','suborbital rocket','success',1341,'NASA 41.075NP (GT)','SubTec-II','','','Test',1821,'',130 from dual union all
select 68631,'2008-S36','2008-08-01 00:00:00','military missile','success',1507,'VMF RF','','','','OT',201,'K-44',1000 from dual union all
select 68632,'2008-S37','2008-08-02 08:30:00','suborbital rocket','success',1074,'S-520-24','Microgravity','','','Microgravity',833,'',293 from dual union all
select 68633,'2008-S38','2008-08-13 08:01:00','military missile','success',799,'','2 x Mk12A,1 x Mk 21','GT194GM-1','GLORY TRIP 194GM-1','Op Test',1699,'',1300 from dual union all
select 68634,'2008-S40','2008-08-25 00:00:00','military missile','success',1443,'FCET-39','-    (123ss)','','','OT',1897,'SSBN 743',1000 from dual union all
select 68635,'2008-S41','2008-08-25 00:00:00','military missile','success',1443,'FCET-39','-    (124ss)','','','OT',1897,'SSBN 743',1000 from dual union all
select 68636,'2008-S42','2008-08-28 10:36:00','military missile','success',1435,'35435-122-48','15Zh58E LKI-P1','','','OT',608,'',1000 from dual union all
select 68637,'2008-S43','2008-09-18 14:45:00','military missile','success',261,'R-30 08','','','','R&D?',190,'TK-208',1000 from dual union all
select 68638,'2008-S44','2008-09-18 02:05:00','military missile','failure',1151,'THAAD FTT-10 Target','','','','Target',855,'MLP',80 from dual union all
select 68639,'2008-S45','2008-09-24 06:57:00','military missile','success',784,'TLV-8 Chimera','TLV Target','NFIRE 2b','','Target',1701,'',1300 from dual union all
select 6864,'1960-S213','1960-07-31 00:00:00','military missile','success',994,'','NIIP-A target','','','Target',634,'',100 from dual union all
select 68640,'2008-S46','2008-10-11 00:00:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-114',1000 from dual union all
select 68641,'2008-S47','2008-10-12 07:24:00','military missile','success',1434,'','15Zh58E LKI-P2','Stabil''nost-2008','','OT',608,'',1000 from dual union all
select 68642,'2008-S48','2008-10-12 00:00:00','military missile','success',1179,'VMF RF','','','','OT',201,'K-84',1000 from dual union all
select 68643,'2008-S49','2008-10-12 00:00:00','military missile','success',1507,'VMF RF','','','','OT',1011,'K-506',1000 from dual union all
select 68644,'2008-S50','2008-10-20 08:39:01','suborbital rocket','success',224,'NASA 36.207DG','J-PEX','','','EUV Astron',1895,'',300 from dual union all
select 68645,'2008-S51','2008-10-22 09:10:00','military missile','success',1453,'-        236','15A35 UBP-56','','','OT',533,'',1000 from dual union all
select 68646,'2008-S52','2008-10-27 07:04:08','suborbital rocket','success',605,'AEB VO1','Parelhas','','','Test?',330,'',95 from dual union all
select 68647,'2008-S53','2008-10-22 12:30:00','suborbital rocket','success',847,'','EURO REXUS 4','MIRIAM 1','','Tech',417,'',175 from dual union all
select 68648,'2008-S54','2008-10-30 00:00:00','military missile','success',748,'Aegis target','MRT-7 Japan SM-3 Tgt','','','Target',855,'',150 from dual union all
select 68649,'2008-S56','2008-11-01 00:00:00','military missile','success',1090,'Aegis','PacBlitz08','','','Interceptor',851,'DDG-60',150 from dual union all
select 6865,'1960-S214','1960-08-01 08:30:00','suborbital rocket','success',839,'AA6.408','Firefly RUTHY','','','Aeronomy',394,'',113 from dual union all
select 68650,'2008-S58','2008-11-01 00:00:00','military missile','success',1090,'Aegis','PacBlitz08','','','Interceptor',851,'DDG-70',150 from dual union all
select 68651,'2008-S59','2008-11-05 09:00:00','military missile','success',799,'','Mk 21 RV','GT198GM','GLORY TRIP 198GM','Op Test',1705,'',1300 from dual union all
select 68652,'2008-S60','2008-11-12 05:55:00','military missile','success',1178,'1','R&D','','','Test',315,'',200 from dual union all
select 68653,'2008-S61','2008-11-11 00:00:00','military missile','success',1156,'R&D','Sejjil-1','','','Test',1348,'',500 from dual union all
select 68654,'2008-S62','2008-11-13 09:06:00','military missile','success',758,'3','','','','Test',301,'',1000 from dual union all
select 68655,'2008-S64','2008-01-01 00:00:00','suborbital rocket','success',728,'Ionos','[Restart launches]','','','Aeron/Ionos',1010,'',85 from dual union all
select 68656,'2008-S65','2008-11-20 02:21:00','military missile','success',748,'JFTM-2 TRACKEX','MRT-8 TRACKEX','[Or ARAV-B?]','','Target',855,'',150 from dual union all
select 68657,'2008-S66','2008-11-20 02:24:00','military missile','success',1090,'Aegis JFTM-2','Stellar Hayabusa','','','Interceptor',851,'DDG-176',150 from dual union all
select 68658,'2008-S67','2008-11-26 00:00:00','suborbital rocket','success',833,'Kavoshgar-2','Kavoshgar Class B','','','Test',1356,'',40 from dual union all
select 68659,'2008-S68','2008-11-26 13:20:00','military missile','success',1530,'RS-24','15Zh55M LKI-3','3? RVs road-mobile','','OT',608,'',1000 from dual union all
select 6866,'1960-S215','1960-08-01 10:23:00','suborbital rocket','success',839,'AA6.410','Firefly HILDA','','','Aeronomy',394,'',126 from dual union all
select 68660,'2008-S69','2008-12-05 10:35:10','suborbital rocket','success',1479,'V05','ICI-2','','','Auroral',1453,'',330 from dual union all
select 68661,'2008-S70','2008-12-05 20:04:00','military missile','success',1111,'','GMD FTG-05 Target','','','Target',862,'',1000 from dual union all
select 68662,'2008-S71','2008-12-05 20:21:00','military missile','success',497,'','GMD FTG-05','','','Test',1709,'',1000 from dual union all
select 68663,'2008-S72','2008-11-28 00:00:00','military missile','success',261,'R-30 09','','','','R&D?',190,'TK-208',1000 from dual union all
select 68664,'2008-S73','2008-09-22 13:30:00','suborbital rocket','success',1284,'','PWN-12A','','','Test',295,'',50 from dual union all
select 68665,'2008-S74','2008-12-23 03:00:00','military missile','failure',261,'R-30 10','','','','R&D?',190,'TK-208',100 from dual union all
select 68666,'2008-S75','2008-12-01 00:00:00','military missile','success',1443,'FCET-40','-    (125ss)','','','OT',1897,'SSBN UNK',1000 from dual union all
select 68667,'2008-S76','2008-03-29 00:00:00','military missile','success',1110,'Ballistic Target','PAC-2 target','','','Test',354,'',85 from dual union all
select 68668,'2008-S77','2008-03-29 00:00:00','military missile','success',1110,'Ballistic Target','PAC-2 target','','','Test',354,'',85 from dual union all
select 68669,'2008-S78','2008-03-29 00:00:00','military missile','success',1110,'Ballistic Target','PAC-2 target','','','Test',354,'',85 from dual union all
select 6867,'1960-S216','1960-08-02 03:14:00','suborbital rocket','failure',1267,'OB11.05','','','','Ionos',1821,'',550 from dual union all
select 68670,'2008-S79','2008-03-29 00:00:00','military missile','success',1110,'Ballistic Target','PAC-2 target','','','Test',354,'',85 from dual union all
select 68671,'2008-S80','2008-11-04 00:00:00','military missile','success',1110,'Ballistic Target','PAC-2 target','','','Test',354,'',85 from dual union all
select 68672,'2008-S81','2008-11-04 00:00:00','military missile','success',1110,'Ballistic Target','PAC-2 target','','','Test',354,'',85 from dual union all
select 68673,'2008-S82','2008-11-04 00:00:00','military missile','success',1110,'Ballistic Target','PAC-2 target','','','Test',354,'',85 from dual union all
select 68674,'2008-S83','2008-11-05 00:00:00','military missile','success',1110,'Ballistic Target','PAC-2 target','','','Test',354,'',85 from dual union all
select 68675,'2008-S84','2008-11-05 00:00:00','military missile','success',1110,'Ballistic Target','PAC-2 target','','','Test',354,'',85 from dual union all
select 68676,'2008-S85','2008-11-05 00:00:00','military missile','success',1110,'Ballistic Target','PAC-2 target','','','Test',354,'',85 from dual union all
select 68677,'2008-S86','2008-04-01 12:20:00','suborbital rocket','success',605,'SSC April One','SSC APR ONE','','','Test',417,'',101 from dual union all
select 68678,'2008-S87','2008-02-01 00:00:00','military missile','success',509,'Hatf-V','','','','Test',1473,'',400 from dual union all
select 68679,'2008-S88','2008-07-18 21:47:00','military missile','success',1111,'FTX-03','GMD FTX-03 Target','','','Target',862,'',1000 from dual;
insert into LAUNCH
select 6868,'1960-S217','1960-08-02 13:30:00','suborbital rocket','success',30,'NN2.32','','','','Ionos?',1869,'',148 from dual union all
select 68680,'2008-S89','2008-02-04 00:00:00','suborbital rocket','success',1114,'HUS.0001','Test satellite','','','Test',1347,'',250 from dual union all
select 68681,'2008-S90','2008-01-29 15:49:00','suborbital rocket','success',1026,'','Langmuir probe','','','Aeron',1467,'',118 from dual union all
select 68682,'2009-001','2009-01-18 02:47:00','orbital','success',397,'D4-9 (337) 4050H','USA 202       [RH10]','ORION 6','','Sat',290,'',null from dual union all
select 68683,'2009-002','2009-01-23 03:54:00','orbital','success',527,'H-IIA-15','GOSAT','GOSAT','','Sat',1477,'',null from dual union all
select 68684,'2009-003','2009-01-30 13:30:00','orbital','success',1448,'450-84-  701','Koronas-Foton','Koronas-Foton','','Sat',470,'',null from dual union all
select 68685,'2009-004','2009-02-02 18:34:00','orbital','success',1114,'GBS.0092','Omid/Safir-2','','','Sat',1347,'',null from dual union all
select 68686,'2009-005','2009-02-06 10:22:00','orbital','success',411,'D338','NOAA N''','NOAA N''','','Sat',1724,'',null from dual union all
select 68687,'2009-006','2009-02-10 05:49:46','orbital','success',1241,'Yu15000-115 115','Progress M-66','Progress 7K-TGM No.366','','Sat ISS 32P',562,'',null from dual union all
select 68688,'2009-007','2009-02-11 00:03:00','orbital','success',962,'935-01/99501','Ekspress AM44/MD1','Ekspress AM-44','','Sat',553,'',null from dual union all
select 68689,'2009-008','2009-02-12 22:09:00','orbital','success',111,'V187 (545)','Hot Bird 10/NSS 9','NSS 9','','Sat',346,'',null from dual union all
select 6869,'1960-S218','1960-08-03 16:26:00','suborbital rocket','success',43,'NASA 06.03UI','UM ionospheric','','','Ionos',1821,'',415 from dual union all
select 68690,'2009-009','2009-02-26 18:29:55','orbital','success',1543,'SB60.2/SLB 2','Telstar 11N','Telstar 11N','','Sat',567,'',null from dual union all
select 68691,'2009-010','2009-02-28 04:10:00','orbital','success',958,'410-16/107L','Raduga-1','Globus No. 18L','','Sat',588,'',null from dual union all
select 68692,'2009-011','2009-03-07 03:49:57','deep space','success',425,'D339','Kepler','Kepler','','Sat',286,'',null from dual union all
select 68693,'2009-012','2009-03-15 23:43:44','orbital','success',1243,'STS-119','ISS 15A','OV-103 ISS','','Sat',921,'',null from dual union all
select 68694,'2009-013','2009-03-17 14:21:14','orbital','success',1064,'511-6992-012P/72511','GOCE','GOCE','','Sat',460,'',null from dual union all
select 68695,'2009-014','2009-03-24 08:34:00','orbital','success',427,'D340','USA 203 (GPS IIRM-7)','Navstar GPS SVN 49','','Sat',285,'',null from dual union all
select 68696,'2009-015','2009-03-26 11:49:18','orbital','success',1233,'Yu15000-027 027','Soyuz TMA-14','Soyuz 11F732A17 No. 224','','Sat ISS 18S',480,'',null from dual union all
select 68697,'2009-016','2009-04-03 16:24:00','orbital','success',962,'935-04/99504','Eutelsat W2A','Eutelsat W2A','','Sat',553,'',null from dual union all
select 68698,'2009-017','2009-04-04 00:31:00','orbital','success',186,'AV-016','USA (WGS SV-2)','Wideband Global Satcom 2','','Sat',292,'',null from dual union all
select 68699,'2009-018','2009-04-14 16:16:04','orbital','success',316,'Y3       (116)','Beidou DW2','er ke beidou daohang wei','','Sat',1903,'',null from dual union all
select 687,'1947-A23','1947-10-23 00:00:00','atmospheric rocket','success',876,'47-I','Nike-47 Round H','','','Test',1863,'',10 from dual union all
select 6870,'1960-S219','1960-08-04 00:00:00','military missile','success',928,'A1X-44','','','','Test',244,'',500 from dual union all
select 68700,'2009-019','2009-04-20 01:15:00','orbital','success',900,'PSLV-C12','RISAT-2/Anusat','TecSAR-2','','Sat',1359,'',null from dual union all
select 68701,'2009-020','2009-04-20 08:15:59','orbital','success',1542,'SL30/DM-SL-30L','SL42 SICRAL 1B','SICRAL 1B','','Sat',860,'ODYSSEY',null from dual union all
select 68702,'2009-021','2009-04-22 02:55:04','orbital','success',301,'Y19      (117)','Yaogan 6','yaogan weixing liuhao','','Sat',1492,'',null from dual union all
select 68703,'2009-022','2009-04-29 16:58:00','orbital','success',1241,'78075  -152 105','Kosmos-2450','Kobal''t-M','','Sat',462,'',null from dual union all
select 68704,'2009-023','2009-05-05 20:24:26','orbital','success',418,'D341','USA 205','STSS Adv Tech Risk Reduc','','Sat',1724,'',null from dual union all
select 68705,'2009-024','2009-05-07 18:37:09','orbital','success',1241,'Sh15000-113 113','Progress M-02M','Progress 7K-TGM No.402','','Sat ISS 33P',480,'',null from dual union all
select 68706,'2009-025','2009-05-11 18:01:56','orbital','success',1243,'STS-125','HST SM-4','OV-104 HST mission','','Sat',921,'',null from dual union all
select 68707,'2009-026','2009-05-14 13:12:00','deep space','success',111,'V188 (546)','Herschel/Planck','Herschel','','Sat',346,'',null from dual union all
select 68708,'2009-027','2009-05-16 00:57:38','orbital','success',962,'935-05/99505','Protostar 2','Indostar II/Protostar II','','Sat',553,'',null from dual union all
select 68709,'2009-028','2009-05-19 23:55:00','orbital','success',783,'8','Tacsat 3','Tacsat 3','','Sat',945,'',null from dual union all
select 6871,'1960-S22','1960-01-26 23:43:05','military missile','success',154,'6D','Mk 2-4','IOC DEMO','DUAL EXHAUST','Test',1664,'',1800 from dual union all
select 68710,'2009-029','2009-05-21 21:53:33','orbital','failure',1226,'77067  -162 005','Meridian No. 12','Meridian No. 12','','Sat',475,'',null from dual union all
select 68711,'2009-030','2009-05-27 10:34:53','orbital','success',1233,'Yu15000-030 030','Soyuz TMA-15','Soyuz 11F732A17 No. 225','','Sat ISS 19S',480,'',null from dual union all
select 68712,'2009-031','2009-06-18 21:32:00','deep space','success',182,'AV-020','LRO/LCROSS','LRO','','Sat',292,'',null from dual union all
select 68713,'2009-032','2009-06-21 21:50:00','orbital','success',1543,'SB60.3/SLB 3L','Measat-3A','Measat 3A','','Sat',567,'',null from dual union all
select 68714,'2009-033','2009-06-27 22:51:00','orbital','success',400,'D4-10 (342) 4240','GOES O','GOES O','','Sat',290,'',null from dual union all
select 68715,'2009-034','2009-06-30 19:10:00','orbital','success',962,'935-06/99506','Sirius FM5','Sirius CD Radio 5','','Sat',553,'',null from dual union all
select 68716,'2009-035','2009-07-01 17:52:00','orbital','success',111,'V189 (547)','Terrestar 1','Terrestar 1','','Sat',346,'',null from dual union all
select 68717,'2009-036','2009-07-06 01:26:34','orbital','success',1064,'492-1791-573/72510','Kosmos-2451-2453','Srela-3','','Sat',460,'',null from dual union all
select 68718,'2009-037','2009-07-14 03:36:00','orbital','success',475,'F005','Razaksat','MACSat','','Sat',870,'',null from dual union all
select 68719,'2009-038','2009-07-15 22:03:10','orbital','success',1243,'STS-127','ISS 2J/A','OV-105 ISS','','Sat',921,'',null from dual union all
select 6872,'1960-S220','1960-08-04 00:00:00','military missile','success',976,'','8K63 UBP-64','','','Test',636,'',402 from dual union all
select 68720,'2009-039','2009-07-21 03:57:43','orbital','success',680,'47181-710','Kosmos-2454/Sterkh','Parus','','Sat',456,'',null from dual union all
select 68721,'2009-040','2009-07-24 10:56:56','orbital','success',1241,'Yu15000-112 112','Progress M-67','Progress 7K-TGM No.367','','Sat ISS 34P',480,'',null from dual union all
select 68722,'2009-041','2009-07-29 18:46:29','orbital','success',434,'510-7681-113','Dubaisat/','Deimos-1 DMC','','Sat',497,'',null from dual union all
select 68723,'2009-042','2009-08-11 19:47:33','orbital','success',962,'935-07/99507','Asiasat 5','Asiasat 5','','Sat',553,'',null from dual union all
select 68724,'2009-043','2009-08-17 10:35:00','orbital','success',427,'D343','USA 206 (GPS IIRM8)','Navstar GPS SVN 50','','Sat',285,'',null from dual union all
select 68725,'2009-044','2009-08-21 22:09:00','orbital','success',111,'V190 (548)','JCSAT 12/Optus D3','JCSAT 12','','Sat',346,'',null from dual union all
select 68726,'2009-045','2009-08-29 03:59:37','orbital','success',1243,'STS-128','ISS 17A','OV-103 ISS','','Sat',921,'',null from dual union all
select 68727,'2009-046','2009-08-31 09:28:00','orbital','failure',313,'Y8       (118)','Palapa D','Palapa D','','Sat',1903,'',null from dual union all
select 68728,'2009-047','2009-09-08 21:35:00','orbital','success',182,'AV-018','PAN','NEMESIS 1','','Sat',292,'',null from dual union all
select 68729,'2009-048','2009-09-10 17:01:46','orbital','success',534,'H-IIB-TF1','HTV-1','HTV 1 Gijutso jissho-ki','','Sat',1478,'',null from dual union all
select 6873,'1960-S221','1960-08-04 00:00:00','military missile','success',976,'','8K63 UBP-65','','','Test',636,'',402 from dual union all
select 68730,'2009-049','2009-09-17 15:55:07','orbital','success',1228,'Ts15000-002 002','Meteor-M','Meteor-M No. 1','','Sat',562,'',null from dual union all
select 68731,'2009-050','2009-09-17 19:19:19','orbital','success',962,'935-08/99508','Nimiq 5','Nimiq 5','','Sat',553,'',null from dual union all
select 68732,'2009-051','2009-09-23 06:21:00','orbital','success',900,'PSLV-C14','Oceansat-2','IRS Oceansat-2','','Sat',1357,'',null from dual union all
select 68733,'2009-052','2009-09-25 12:20:00','orbital','success',418,'D344','USA 208/USA 209','STSS 1','','Sat',286,'',null from dual union all
select 68734,'2009-053','2009-09-30 07:14:45','orbital','success',1233,'B 15000-029 029','Soyuz TMA-16','Soyuz 11F732A17 No. 226','','Sat ISS 20S',480,'',null from dual union all
select 68735,'2009-054','2009-10-01 21:59:00','orbital','success',111,'V191 (549)','Amazonas 2/COMSATBw1','Amazonas 2','','Sat',346,'',null from dual union all
select 68736,'2009-055','2009-10-08 18:51:01','orbital','success',418,'D345','WorldView-2','WorldView-2','','Sat',1724,'',null from dual union all
select 68737,'2009-056','2009-10-15 01:14:37','orbital','success',1241,'Yu15000-120 120','Progress M-03M','Progress 7K-TGM No.403','','Sat ISS 35P',480,'',null from dual union all
select 68738,'2009-057','2009-10-18 16:12:01','deep space','success',182,'AV-017','DMSP 5D-3 F18','DMSP Block 5D-3 S-18','','Sat',1775,'',null from dual union all
select 68739,'2009-058','2009-10-29 20:00:00','orbital','success',111,'V192 (550)','NSS 12/Thor 6','NSS 12','','Sat',346,'',null from dual union all
select 6874,'1960-S222','1960-08-06 09:00:00','suborbital rocket','success',839,'AA6.411','Firefly GERTA','','','Aeronomy',394,'',138 from dual union all
select 68740,'2009-059','2009-11-02 01:50:51','orbital','success',1064,'492-5882-033/72513','SMOS/Proba-2','SMOS','','Sat',460,'',null from dual union all
select 68741,'2009-060','2009-11-10 14:22:04','orbital','success',1241,'B 15000-121 121','Poisk','Maliy Issled. Modul-2','','Sat ISS 5R',480,'',null from dual union all
select 68742,'2009-061','2009-11-12 02:45:00','orbital','success',301,'Y21      (119)','SJ-11-01','shijian shi yihao 01 xin','','Sat',813,'',null from dual union all
select 68743,'2009-062','2009-11-16 19:28:10','orbital','success',1243,'STS-129','ISS ULF-3','OV-104 ISS','','Sat',921,'',null from dual union all
select 68744,'2009-063','2009-11-20 10:44:00','orbital','success',1241,'76043  -811 097','Kosmos-2455','Lotos-S No.  801','','Sat',462,'',null from dual union all
select 68745,'2009-064','2009-11-23 06:55:00','orbital','success',188,'AV-024','Intelsat IS-14','IS-14','','Sat',292,'',null from dual union all
select 68746,'2009-065','2009-11-24 14:19:10','orbital','success',962,'935-09/99509','Eutelsat W7','Eutelsat W7','','Sat',553,'',null from dual union all
select 68747,'2009-066','2009-11-28 01:21:00','orbital','success',528,'H-IIA-16','IGS O-3','IGS Optical-3','','Sat',1477,'',null from dual union all
select 68748,'2009-067','2009-11-30 21:00:00','orbital','success',1543,'SB60.4/SLB 4L','Intelsat IS-15','IS-15','','Sat',567,'',null from dual union all
select 68749,'2009-068','2009-12-06 01:47:00','orbital','success',404,'D4-11 (346) 4450','WGS 3','Wideband Global Satcom 3','','Sat',290,'',null from dual union all
select 6875,'1960-S223','1960-08-08 00:00:00','military missile','success',976,'','8K63 UBP-66','','','Test',636,'',402 from dual union all
select 68750,'2009-069','2009-12-09 08:42:00','orbital','success',303,'Y10      (120)','Yaogan 7','yaogan weixing qihao','','Sat',813,'',null from dual union all
select 68751,'2009-070','2009-12-14 10:38:27','orbital','success',963,'535-38/115L','Kosmos-2456/2458','Uragan-M No. 730','','Sat',588,'',null from dual union all
select 68752,'2009-071','2009-12-14 14:09:33','orbital','success',411,'D347','WISE','Wide Field IR Explorer','','Sat',1724,'',null from dual union all
select 68753,'2009-072','2009-12-15 02:31:05','orbital','success',320,'Y4       (121)','Yaogan 8','yaogan weixing bahao','','Sat',1494,'',null from dual union all
select 68754,'2009-073','2009-12-18 16:26:00','orbital','success',116,'V193 (532)','Helios IIB','Helios 2B','','Sat',346,'',null from dual union all
select 68755,'2009-074','2009-12-20 21:52:00','orbital','success',1233,'B 15000-031 031','Soyuz TMA-17','Soyuz 11F732A17 No. 227','','Sat ISS 21S',480,'',null from dual union all
select 68756,'2009-075','2009-12-29 00:22:00','orbital','success',962,'935-10/99510','DirecTV 12','DirecTV-12','','Sat',553,'',null from dual union all
select 68757,'2009-A01','2009-04-07 00:00:00','military missile','success',119,'','AST   Black Sparrow','','','Test',1026,'',40 from dual union all
select 68758,'2009-A02','2009-03-26 19:20:00','military missile','success',1088,'Aegis BFI','Stellar Daggers','','','Interceptor',1271,'DDG-65',10 from dual union all
select 68759,'2009-A03','2009-05-02 14:00:00','suborbital rocket','failure',1244,'SpaceLoft SL-3','Education Launch','','','Test',1422,'',10 from dual union all
select 6876,'1960-S224','1960-08-08 10:15:00','suborbital rocket','success',839,'AA6.412','Firefly BETSEY','','','Aeronomy',394,'',109 from dual union all
select 68760,'2009-A04','2009-03-18 00:30:00','military missile','success',1294,'THAAD FTT-10a Salvo','High Endo','','','ABM Test',855,'',60 from dual union all
select 68761,'2009-A05','2009-07-15 00:00:00','military missile','failure',261,'R-30 11 3M30','','','','R&D?',190,'TK-208',10 from dual union all
select 68762,'2009-A06','2009-10-28 15:30:00','suborbital rocket','success',90,'Ares I-X','USS/CMLAS','','','Test',922,'MLP1',null from dual union all
select 68763,'2009-A07','2009-10-29 00:00:00','military missile','success',3,'','Test','','','Op Test',183,'',40 from dual union all
select 68764,'2009-A08','2009-11-23 14:19:00','military missile','failure',63,'','Agni RV','','','Test',316,'',20 from dual union all
select 68765,'2009-A09','2009-12-11 00:00:00','military missile','success',1098,'THAAD FTT-11 Target','MBRV','','','Target',958,'C-17',10 from dual union all
select 68766,'2009-A10','2009-12-17 11:53:00','suborbital rocket','success',514,'','','','','Test',1352,'',40 from dual union all
select 68767,'2009-A11','2009-08-10 18:18:00','atmospheric rocket','success',472,'1','','','','Test',13,'',32 from dual union all
select 68768,'2009-A12','2009-08-13 14:21:00','atmospheric rocket','success',472,'2','','','','Test',13,'',31 from dual union all
select 68769,'2009-A13','2009-10-20 20:16:00','atmospheric rocket','success',472,'3','','','','Test',330,'',32 from dual;
insert into LAUNCH
select 6877,'1960-S225','1960-08-09 10:37:00','suborbital rocket','success',839,'AA6.414','Firefly MARIE','','','Aeronomy',394,'',81 from dual union all
select 68770,'2009-A14','2009-10-21 20:30:00','atmospheric rocket','success',472,'4','','','','Test',330,'',32 from dual union all
select 68771,'2009-F01','2009-02-24 09:55:30','orbital','failure',1316,'T8 (XL)','OCO','Orbiting Carbon Observat','','Sat',1670,'',500 from dual union all
select 68772,'2009-F02','2009-04-05 02:30:00','orbital','failure',1462,'2','Kwangmyongsong-2','','','Sat',1480,'',490 from dual union all
select 68773,'2009-F03','2009-08-25 08:00:33','orbital','failure',831,'74501','STSAT-2A','STSat-2','','Sat',987,'',387 from dual union all
select 68774,'2009-M01','2009-03-06 10:57:00','military missile','success',941,'','PAD Interceptor 3','','','ABM Test',316,'',75 from dual union all
select 68775,'2009-M02','2009-03-26 19:20:00','military missile','success',702,'Aegis SM-2 target','SRT','','','Target',1415,'',50 from dual union all
select 68776,'2009-M03','2009-04-15 04:50:00','military missile','success',942,'2','','','','Test',320,'',50 from dual union all
select 68777,'2009-M04','2009-07-02 01:00:00','suborbital rocket','success',1049,'','','','','Test',932,'',53 from dual union all
select 68778,'2009-M05','2009-03-18 00:30:00','military missile','success',1294,'THAAD FTT-10a Salvo','High Endo','','','ABM Test',855,'',60 from dual union all
select 68779,'2009-M06','2009-10-12 04:58:00','military missile','success',942,'2','Prithvi-2','','','Test',320,'',50 from dual union all
select 6878,'1960-S226','1960-08-09 18:09:33','military missile','success',154,'32D','Mk 3-1B-224/AFCRC','D/RIG Long range test','','Aeron',227,'',1531 from dual union all
select 68780,'2009-M07','2009-10-12 05:03:00','military missile','success',942,'2','Prithvi-2','','','Test',320,'',50 from dual union all
select 68781,'2009-M08','2009-12-13 06:01:00','military missile','success',430,'','','F6','','Test',320,'INS-P51',50 from dual union all
select 68782,'2009-S01','2009-01-10 23:17:00','suborbital rocket','success',605,'NASA 30.073UO','ASRP SRP-5 (ISIS)','','','Plasma',1047,'',92 from dual union all
select 68783,'2009-S02','2009-01-26 00:15:00','suborbital rocket','success',1073,'S-310-39','DELTA-2','','','Aeron',28,'',140 from dual union all
select 68784,'2009-S03','2009-01-29 09:49:41','suborbital rocket','success',224,'NASA 36.242UE','ACES I','','','Auroral',1047,'',364 from dual union all
select 68785,'2009-S04','2009-01-29 09:51:11','suborbital rocket','success',228,'NASA 21.139UE','ACES II','','','Auroral',1047,'',133 from dual union all
select 68786,'2009-S05','2009-02-18 09:52:00','suborbital rocket','success',1334,'NASA 41.076','Turbopause','','','Aeron',1047,'',140 from dual union all
select 68787,'2009-S06','2009-02-18 10:29:00','suborbital rocket','success',1334,'NASA 41.078','Turbopause','','','Aeron',1047,'',140 from dual union all
select 68788,'2009-S07','2009-02-18 10:59:00','suborbital rocket','success',1334,'NASA 41.079','Turbopause','','','Aeron',1047,'',140 from dual union all
select 68789,'2009-S08','2009-02-18 11:49:00','suborbital rocket','success',1334,'NASA 41.077','Turbopause','','','Aeron',1047,'',140 from dual union all
select 6879,'1960-S227','1960-08-10 00:00:00','suborbital rocket','success',42,'TATTLETALE PCC 5','','','','Sigint Test?',394,'',383 from dual union all
select 68790,'2009-S09','2009-12-19 00:00:00','military missile','success',1443,'DASO-21','-    (130ss)','','','OT',421,'SSBN 732',1000 from dual union all
select 68791,'2009-S10','2009-02-13 00:00:00','military missile','success',1443,'DASO-20','-    (126ss)','','','OT',1897,'SSBN 731',1000 from dual union all
select 68792,'2009-S11','2009-02-25 10:45:00','suborbital rocket','success',224,'NASA 36.226UG','CIBER-I','','','IR Astron',1895,'',330 from dual union all
select 68793,'2009-S12','2009-03-06 10:54:00','military missile','success',430,'','PAD Target','','','Target',320,'INS-P51',120 from dual union all
select 68794,'2009-S13','2009-03-12 10:08:00','suborbital rocket','success',605,'SSC REXUS 6','EURO REXUS 6','','','Ionos/Tech',417,'',88 from dual union all
select 68795,'2009-S14','2009-03-13 06:00:00','suborbital rocket','success',605,'SSC REXUS 5','EURO REXUS 5','','','Aeron/Tech',417,'',87 from dual union all
select 68796,'2009-S15','2009-03-18 00:25:00','military missile','success',1151,'THAAD FTT-10a Target','RV (Sep Target)','','','Target',855,'MLP',100 from dual union all
select 68797,'2009-S18','2009-03-20 11:04:01','suborbital rocket','success',237,'NASA 40.023UE','CASCADES II','','','Auroral',1043,'',564 from dual union all
select 68798,'2009-S19','2009-03-25 13:25:00','military missile','success',570,'','PAC-3 Target','','','Target',436,'',100 from dual union all
select 68799,'2009-S20','2009-04-10 09:10:00','military missile','success',1434,'35435-112-53','15Zh58 UBP-35','','','OT',608,'',1000 from dual union all
select 688,'1947-A24','1947-10-23 14:05:00','military missile','failure',8,'08T','','','','Test',636,'',14 from dual union all
select 6880,'1960-S228','1960-08-10 00:00:00','military missile','success',976,'','8K63 UBP-68','','','Test',636,'',402 from dual union all
select 68800,'2009-S21','2009-04-17 11:18:00','suborbital rocket','success',485,'FalconLaunch 7','','','','Test',1895,'',108 from dual union all
select 68801,'2009-S22','2009-05-20 00:00:00','military missile','success',1157,'R&D','Sejjil-2','','','Test',1348,'',800 from dual union all
select 68802,'2009-S23','2009-05-07 02:42:00','suborbital rocket','success',1345,'','HiFire HF0','','','Hypersonic',1849,'',314 from dual union all
select 68803,'2009-S24','2009-05-22 10:32:00','suborbital rocket','success',847,'','DLR MAPHEUS','','','Microgravity',417,'',140 from dual union all
select 68804,'2009-S25','2009-05-28 16:52:00','suborbital rocket','success',1345,'NASA 41.080NR','SOAREX-7 TDRV','','','Hypersonic',1821,'',130 from dual union all
select 68805,'2009-S26','2009-05-29 19:25:00','suborbital rocket','success',605,'AEB V02','Maracati 1','','','Test',13,'',93 from dual union all
select 68806,'2009-S27','2009-05-19 04:36:00','military missile','success',63,'','Agni RV','','','Test',316,'',300 from dual union all
select 68807,'2009-S28','2009-06-06 00:00:00','military missile','success',1336,'','ABL Target','','','Target',1415,'',80 from dual union all
select 68808,'2009-S29','2009-06-13 00:00:00','military missile','success',1336,'','ABL Target','','','Target',1415,'',80 from dual union all
select 68809,'2009-S30','2009-06-26 09:30:00','suborbital rocket','success',1345,'NASA 41.083UO','RockOn','','','Education',1821,'',117 from dual union all
select 6881,'1960-S229','1960-08-10 00:00:00','military missile','success',872,'ZW-9/20009','9 First Tac','','','Test',1884,'',40 from dual union all
select 68810,'2009-S31','2009-06-29 10:01:00','military missile','success',799,'','Mk 12A RV x 3','GT199GM','GLORY TRIP 199GM','Op Test',1699,'',1300 from dual union all
select 68811,'2009-S32','2009-06-27 07:30:00','suborbital rocket','success',224,'NASA 36.244UG','DICE','','','UV Astron',1895,'',300 from dual union all
select 68812,'2009-S33','2009-07-13 01:20:00','military missile','success',1179,'VMF RF','','','','OT',196,'K-84',1000 from dual union all
select 68813,'2009-S34','2009-07-13 23:50:00','military missile','success',1179,'VMF RF','','','','OT',1001,'K-117',1500 from dual union all
select 68814,'2009-S35','2009-07-22 19:45:00','military missile','success',691,'Arrow Target','CARAVAN','','','Target',1271,'C-17',500 from dual union all
select 68815,'2009-S36','2009-05-26 00:00:00','military missile','success',1443,'DASO-9','-    (127ss)','','','OT',421,'UK S29',1000 from dual union all
select 68816,'2009-S37','2009-07-31 03:40:00','military missile','success',1345,'Aegis FTM-17 target','ARAV-A','','','Target',855,'',100 from dual union all
select 68817,'2009-S38','2009-07-31 03:42:00','military missile','success',1090,'Aegis FTM-17','Stellar Avenger','','','Interceptor',851,'DDG-70',160 from dual union all
select 68818,'2009-S39','2009-07-31 04:00:00','military missile','success',1345,'FTX-06E1 TRACKEX','ARAV-A','','','Target',855,'',100 from dual union all
select 68819,'2009-S40','2009-08-11 04:50:32','military missile','success',224,'NASA 36.229DR','MARTI ABL Target','','','Target',1415,'',100 from dual union all
select 6882,'1960-S23','1960-01-27 00:00:00','military missile','success',928,'A1X-10','','','','Test',251,'',500 from dual union all
select 68820,'2009-S41','2009-08-17 11:52:00','suborbital rocket','success',1502,'','IVRE Support 1','','','Aeron',1821,'',90 from dual union all
select 68821,'2009-S42','2009-08-17 12:52:00','suborbital rocket','success',224,'NASA 36.254NR','IRVE-II','','','Reentry test',1801,'',210 from dual union all
select 68822,'2009-S43','2009-08-23 16:01:00','military missile','success',799,'','Mk 12/21 RV','GT195GM-2','GLORY TRIP 195GM-2','Op Test',1705,'',1300 from dual union all
select 68823,'2009-S44','2009-09-14 17:40:00','suborbital rocket','success',224,'NASA 36.221DS','HERSCHEL HEIT/SCORE','','','Solar EUV',1895,'',300 from dual union all
select 68824,'2009-S45','2009-09-19 23:46:00','suborbital rocket','success',234,'NASA 39.009DR','CARE','','','Aeron',1797,'',240 from dual union all
select 68825,'2009-S46','2009-09-27 00:00:00','military missile','success',1166,'','','','','Test',1356,'',100 from dual union all
select 68826,'2009-S47','2009-09-27 00:00:00','military missile','success',1167,'Shahab 2','','','','Test',1356,'',125 from dual union all
select 68827,'2009-S48','2009-09-28 00:00:00','military missile','success',508,'','','','','Test',759,'',500 from dual union all
select 68828,'2009-S49','2009-09-28 00:00:00','military missile','success',1157,'R&D','Sejjil-2','','','Test',1348,'',800 from dual union all
select 68829,'2009-S50','2009-10-06 00:00:00','military missile','success',1507,'VMF RF','','','','OT',1011,'K-433',1000 from dual union all
select 6883,'1960-S230','1960-08-10 22:46:00','military missile','success',1419,'J-7','Mk 4-2 RV','','','Test',237,'',1000 from dual union all
select 68830,'2009-S51','2009-10-07 00:00:00','military missile','success',1507,'VMF RF','','','','OT',1011,'K-44',1000 from dual union all
select 68831,'2009-S52','2009-09-03 00:00:00','military missile','success',1443,'FCET-41','-    (128ss)','','','OT',421,'SSBN 736',1000 from dual union all
select 68832,'2009-S53','2009-09-04 00:00:00','military missile','success',1443,'FCET-41','-    (129ss)','','','OT',421,'SSBN 736',1000 from dual union all
select 68833,'2009-S54','2009-11-01 00:00:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-117',1000 from dual union all
select 68834,'2009-S55','2009-09-16 14:40:00','military missile','success',570,'JASDF Target','PAC-3 Target','','','Target',436,'',100 from dual union all
select 68835,'2009-S56','2009-10-28 04:00:00','military missile','success',748,'Aegis JFTM-3 target','MRT-9 (Sep MRT)','','','Target',855,'',150 from dual union all
select 68836,'2009-S57','2009-10-28 04:04:00','military missile','success',1090,'Aegis JFTM-3','Stellar Raicho','','','Interceptor',851,'DDG-175',150 from dual union all
select 68837,'2009-S58','2009-10-16 00:00:00','military missile','success',1344,'FTX-06E2','ARAV-B10','','','Target',855,'',100 from dual union all
select 68838,'2009-S59','2009-10-16 00:00:00','military missile','success',1344,'FTX-06E3','ARAV-B11','','','Target',855,'',100 from dual union all
select 68839,'2009-S60','2009-11-05 00:00:00','military missile','success',1303,'Aegis FTX-06E4 Tgt','ARAV-C','','','Target',855,'',300 from dual union all
select 6884,'1960-S231','1960-08-10 10:35:00','suborbital rocket','success',839,'AA6.418','Firefly JEANNIE','','','Aeronomy',394,'',104 from dual union all
select 68840,'2009-S61','2009-11-14 02:30:00','suborbital rocket','success',224,'NASA 36.252UH','EXOS/CyXESS','','','XR Astron',1895,'',300 from dual union all
select 68841,'2009-S62','2009-11-22 11:15:00','suborbital rocket','success',1484,'V09','DLR TEXUS 46','','','Micrograv',416,'',252 from dual union all
select 68842,'2009-S63','2009-11-29 09:00:00','suborbital rocket','success',1484,'V10','DLR TEXUS 47','','','Micrograv',416,'',264 from dual union all
select 68843,'2009-S64','2009-11-30 01:28:00','suborbital rocket','success',134,'1','Manu Karere','','','Test',601,'',120 from dual union all
select 68844,'2009-S65','2009-12-09 06:45:00','military missile','failure',261,'R-30 12','','','','R&D?',190,'TK-208',500 from dual union all
select 68845,'2009-S66','2009-12-10 11:35:00','military missile','success',1435,'','15Zh58E GCh-3','','','OT',636,'',1000 from dual union all
select 68846,'2009-S67','2009-10-15 12:29:00','military missile','success',570,'ATM-48 Target','PAC-2 Target','','','Target',436,'',100 from dual union all
select 68847,'2009-S68','2009-12-16 00:00:00','military missile','success',1157,'R&D','Sejjil-2','','','Test',1348,'',800 from dual union all
select 68848,'2009-S69','2009-12-16 13:46:00','suborbital rocket','success',774,'NASA 12.068GT','','','','Test',1801,'',90 from dual union all
select 68849,'2009-S70','2009-12-17 03:25:00','suborbital rocket','success',1345,'NASA 41.086UO','HAROH','','','Aeron',1895,'',130 from dual union all
select 6885,'1960-S232','1960-08-10 10:35:00','suborbital rocket','success',1194,'SL61','UCL GC-10','','','Test',1839,'',171 from dual union all
select 68850,'2009-S71','2009-12-24 06:30:00','military missile','success',1005,'450-7951-202','15A18M UBP 13 RD','','','OT',1908,'',1000 from dual union all
select 68851,'2009-S72','2009-10-13 00:00:00','military missile','success',1344,'JFTM-3 TRACKEX','ARAV-B9','','','Target',855,'',100 from dual union all
select 68852,'2009-S73','2009-10-13 00:00:00','military missile','success',1344,'JFTM-3 TRACKEX','ARAV-B8','','','Target',855,'',100 from dual union all
select 68853,'2009-S74','2009-08-17 00:00:00','suborbital rocket','success',1502,'','IVRE Support 2','','','Aeron',1821,'',90 from dual union all
select 68854,'2009-S75','2009-07-03 23:00:00','military missile','success',586,'','[or Hwasong 6]','','','Test',859,'',150 from dual union all
select 68855,'2009-S76','2009-07-03 23:30:00','military missile','success',586,'','[or Hwasong 6]','','','Test',859,'',150 from dual union all
select 68856,'2009-S77','2009-07-04 01:45:00','military missile','success',580,'','','','','Test',859,'',150 from dual union all
select 68857,'2009-S78','2009-07-04 03:00:00','military missile','success',580,'','','','','Test',859,'',150 from dual union all
select 68858,'2009-S79','2009-07-04 05:50:00','military missile','success',580,'','','','','Test',859,'',150 from dual union all
select 68859,'2009-S80','2009-07-04 07:10:00','military missile','success',580,'','','','','Test',859,'',150 from dual;
insert into LAUNCH
select 6886,'1960-S233','1960-08-11 00:00:00','military missile','success',976,'','8K63 UBP-69','','','Test',636,'',402 from dual union all
select 68860,'2009-S81','2009-07-04 08:40:00','military missile','success',580,'','','','','Test',859,'',150 from dual union all
select 68861,'2010-001','2010-01-16 16:12:04','orbital','success',316,'Y2       (122)','Beidou DW 3','di san ke beidou daohang','','Sat',1903,'',null from dual union all
select 68862,'2010-002','2010-01-28 00:18:00','orbital','success',962,'535-35/88527','Raduga-1M','Globus-1M No. 12L?','','Sat',588,'',null from dual union all
select 68863,'2010-003','2010-02-03 03:45:29','orbital','success',1241,'B 15000-117 117','Progress M-04M','Progress 7K-TGM No.404','','Sat ISS 36P',480,'',null from dual union all
select 68864,'2010-004','2010-02-08 09:14:08','orbital','success',1243,'STS-130','ISS 20A','OV-105 ISS','','Sat',921,'',null from dual union all
select 68865,'2010-005','2010-02-11 15:23:00','orbital','success',182,'AV-021','SDO','SDO','','Sat',292,'',null from dual union all
select 68866,'2010-006','2010-02-12 00:39:40','orbital','success',962,'535-32/99511','Intelsat IS-16','IS-16','','Sat',553,'',null from dual union all
select 68867,'2010-007','2010-03-01 21:19:44','orbital','success',963,'535-40/116L','Kosmos-2459/2461','Uragan-M No. 731','','Sat',588,'',null from dual union all
select 68868,'2010-008','2010-03-04 23:57:00','orbital','success',400,'D4-12 (348) 4240','GOES P','GOES P','','Sat',290,'',null from dual union all
select 68869,'2010-009','2010-03-05 04:55:05','orbital','success',320,'Y5       (123)','Yaogan 9','yaogan weixing jiuhao','','Sat',813,'',null from dual union all
select 6887,'1960-S234','1960-08-12 00:00:00','suborbital rocket','unknown',673,'','','','','Aeron?',1016,'',300 from dual union all
select 68870,'2010-010','2010-03-20 18:26:57','orbital','success',962,'935-14/99514','Echostar XIV','Echostar 14','','Sat',553,'',null from dual union all
select 68871,'2010-011','2010-04-02 04:04:33','orbital','success',1233,'Yu15000-028 028','Soyuz TMA-18','Soyuz 11F732A17 No. 228','','Sat ISS 22S',480,'',null from dual union all
select 68872,'2010-012','2010-04-05 10:21:25','orbital','success',1243,'STS-131','ISS 19A','OV-103 ISS','','Sat',921,'',null from dual union all
select 68873,'2010-013','2010-04-08 13:57:05','orbital','success',434,'450-4261-304','CryoSat-2','Cryosat 2','','Sat P',497,'',null from dual union all
select 68874,'2010-014','2010-04-16 15:00:00','orbital','success',1241,'76029  -169 124','Kosmos-2462','Kobal''t-M','','Sat',462,'',null from dual union all
select 68875,'2010-015','2010-04-22 23:52:00','deep space','success',190,'AV-012','X-37B OTV FLT-1','X-37B No. 1','','Sat',292,'',null from dual union all
select 68876,'2010-016','2010-04-24 11:19:00','orbital','success',962,'935-11/99512','SES-1','SES-1','','Sat',553,'',null from dual union all
select 68877,'2010-017','2010-04-27 01:05:09','orbital','success',680,'47181-707','Kosmos-2463','Parus?','','Sat',456,'',null from dual union all
select 68878,'2010-018','2010-04-28 17:15:09','orbital','success',1241,'B 15000-118 118','Progress M-05M','Progress 7K-TGM No.405','','Sat ISS 37P',480,'',null from dual union all
select 68879,'2010-019','2010-05-14 18:20:09','orbital','success',1243,'STS-132','ISS ULF-4','OV-104 ISS','','Sat',921,'',null from dual union all
select 6888,'1960-S235','1960-08-12 18:28:00','military missile','success',928,'A1X-47','','','','Test',244,'',500 from dual union all
select 68880,'2010-020','2010-05-20 21:58:22','deep space','success',527,'H-IIA-17','PLANET-C','K-SAT','','Sat',1477,'',null from dual union all
select 68881,'2010-021','2010-05-21 22:01:00','orbital','success',111,'V194 (551)','Astra 3B/COMSATBw-2','Astra 3B','','Sat',346,'',null from dual union all
select 68882,'2010-022','2010-05-28 03:00:00','orbital','success',400,'D4-13 (349) 4240','GPS IIF SV-1','GPS SVN 62','','Sat',290,'',null from dual union all
select 68883,'2010-023','2010-06-02 01:59:12','orbital','success',1064,'511-1992-007P/72516','SERVIS-2','SERVIS-2','','Sat',460,'',null from dual union all
select 68884,'2010-024','2010-06-02 15:53:04','orbital','success',316,'Y4       (124)','Beidou DW4','di si ke beidou daohang','','Sat',1903,'',null from dual union all
select 68885,'2010-025','2010-06-03 22:00:08','orbital','success',962,'935-12/99513','Arabsat 5B','Arabsat 5B','','Sat',553,'',null from dual union all
select 68886,'2010-026','2010-06-04 18:45:00','orbital','success',482,'001','Dragon Qual Unit','Dragon Spacecraft','','Sat',274,'',null from dual union all
select 68887,'2010-027','2010-06-15 01:39:04','orbital','success',303,'Y15      (125)','SJ-12','shijian shi erhao weixin','','Sat',813,'',null from dual union all
select 68888,'2010-028','2010-06-15 14:42:21','orbital','success',434,'450-3261-316','PRISMA/Picard','Picard','','Sat',1907,'',null from dual union all
select 68889,'2010-029','2010-06-15 21:35:19','orbital','success',1233,'B 15000-032 032','Soyuz TMA-19','Soyuz 11F732A17 No. 229','','Sat ISS 23S',480,'',null from dual union all
select 6889,'1960-S236','1960-08-12 00:00:00','military missile','success',976,'','8K63 UBP-70','','','Test',636,'',402 from dual union all
select 68890,'2010-030','2010-06-21 02:14:08','orbital','success',434,'450-3261-314','Tandem-X','TDX','','Sat SSO',497,'',null from dual union all
select 68891,'2010-031','2010-06-22 19:00:00','orbital','success',1174,'8','''Ofeq-9','''Ofeq-9','','Sat',1026,'',null from dual union all
select 68892,'2010-032','2010-06-26 21:41:00','orbital','success',111,'V195 (552)','Arabsat5A/COMS-1','COMS-1','','Sat',346,'',null from dual union all
select 68893,'2010-033','2010-06-30 15:35:14','orbital','success',1241,'B 15000-119 119','Progress M-06M','Progress 7K-TGM No.406','','Sat ISS 38P',480,'',null from dual union all
select 68894,'2010-034','2010-07-10 18:40:36','orbital','success',962,'935-15/99515','Echostar XV','','','Sat',553,'',null from dual union all
select 68895,'2010-035','2010-07-12 03:52:00','orbital','success',900,'PSLV-C15','Cartosat-2B','Cartosat-2B','','Sat',1357,'',null from dual union all
select 68896,'2010-036','2010-07-31 21:30:04','orbital','success',309,'Y16      (126)','Beidou DW5','diwu ke beidou daohang w','','Sat',1904,'',null from dual union all
select 68897,'2010-037','2010-08-04 20:59:00','orbital','success',111,'V196 (554)','Nilesat 201/Rascom','Nilesat 201','','Sat',346,'',null from dual union all
select 68898,'2010-038','2010-08-09 22:49:05','orbital','success',320,'Y6','Yaogan 10','yaogan weixing shihao','','Sat',1494,'',null from dual union all
select 68899,'2010-039','2010-08-14 11:07:00','orbital','success',192,'AV-019','AEHF 1','AEHF SV-1','','Sat',292,'',null from dual union all
select 689,'1947-A25','1947-10-28 00:00:00','atmospheric rocket','success',876,'47-12','Nike-47 10R','','','Test',1863,'',35 from dual union all
select 6890,'1960-S237','1960-08-12 13:00:10','military missile','success',154,'66D','RVX-2A S/N 421','D/AIG guidance test CL3','','Test',226,'',965 from dual union all
select 68900,'2010-040','2010-08-24 07:10:04','orbital','success',303,'Y14','Tianhui 1-01','tianhui yi hao weixing','','Sat',813,'',null from dual union all
select 68901,'2010-041','2010-09-02 00:53:50','orbital','success',963,'535-30/118L','Kosmos-2464/2466','Uragan-M No. 736','','Sat',588,'',null from dual union all
select 68902,'2010-042','2010-09-04 16:14:04','orbital','success',311,'Y13','Zhongxing 6A','zhongxing liu A weixing','','Sat',1903,'',null from dual union all
select 68903,'2010-043','2010-09-08 03:30:18','orbital','success',1064,'630-9793-567/72514','Gonets-M/Kosmos','Gonets-M No. 12','','Sat',460,'',null from dual union all
select 68904,'2010-044','2010-09-10 10:22:57','orbital','success',1241,'B 15000-122 122','Progress M-07M','Progress 7K-TGM No.407','','Sat ISS 39P',562,'',null from dual union all
select 68905,'2010-045','2010-09-11 11:17:00','orbital','success',527,'H-IIA-18','QZSS','QZSS','','Sat',1477,'',null from dual union all
select 68906,'2010-046','2010-09-21 04:03:30','orbital','success',190,'AV-025','USA 215   (FIAR-1)','TOPAZ 1','','Sat',1775,'',null from dual union all
select 68907,'2010-047','2010-09-22 02:42:04','orbital','success',303,'Y11','Yaogan 11','yaogan weixing shi yihao','','Sat',813,'',null from dual union all
select 68908,'2010-048','2010-09-26 04:41:00','orbital','success',788,'2','SBSS','SBSS Block 10','','Sat',1781,'',null from dual union all
select 68909,'2010-049','2010-09-30 17:01:15','orbital','success',803,'78041  -699 052','Kosmos-2469','Oko US-KS?','','Sat',462,'',null from dual union all
select 6891,'1960-S238','1960-08-15 10:35:00','suborbital rocket','success',545,'AA5.424','Firefly LOLA','','','Aeronomy',394,'',84 from dual union all
select 68910,'2010-050','2010-10-01 10:59:57','deep space','success',316,'Y7','Chang''e-2','chang''e erhao weixing','','Sat',1903,'',null from dual union all
select 68911,'2010-051','2010-10-06 00:49:05','orbital','success',319,'Y23','SJ-6-4A/B','shijian liuhao 4 zu B xi','','Sat',1494,'',null from dual union all
select 68912,'2010-052','2010-10-07 23:10:55','orbital','success',1233,'B 15000-035 035','Soyuz TMA-01M','Soyuz 11F732A47 No. 701','','Sat ISS 24S',480,'',null from dual union all
select 68913,'2010-053','2010-10-14 18:53:21','orbital','success',962,'935-16/99517','Sirius XM-5','','','Sat',588,'',null from dual union all
select 68914,'2010-054','2010-10-19 17:10:59','orbital','success',1226,'B 15000-009 009/ST22','Globalstar','Globalstar-2 FM8?','','Sat',562,'',null from dual union all
select 68915,'2010-055','2010-10-27 15:11:50','orbital','success',1241,'I 15000-123 123','Progress M-08M','Progress 7K-TGM No.408','','Sat ISS 40P',480,'',null from dual union all
select 68916,'2010-056','2010-10-28 21:51:00','orbital','success',111,'V197 (555)','Eutelsat W3B/BSAT 3B','Eutelsat W3B','','Sat',346,'',null from dual union all
select 68917,'2010-057','2010-10-31 16:26:04','orbital','success',316,'Y5','Beidou DW6','di liu ke beidou daohang','','Sat',1903,'',null from dual union all
select 68918,'2010-058','2010-11-02 00:58:39','orbital','success',1226,'760-12 -167 006','Meridian No. 13','Meridian No. 13L','','Sat',475,'',null from dual union all
select 68919,'2010-059','2010-11-04 18:37:12','orbital','success',320,'Y7','FY-3B','fengyun sanhao 01 B xing','','Sat',1494,'',null from dual union all
select 6892,'1960-S239','1960-08-16 01:42:00','suborbital rocket','success',838,'AA8.404','Firefly ARLENE','','','Aeronomy',394,'',104 from dual union all
select 68920,'2010-060','2010-11-06 02:20:03','orbital','success',413,'D350','COSMO 4','COSMO-Skymed 4','','Sat',1724,'',null from dual union all
select 68921,'2010-061','2010-11-14 17:29:20','orbital','success',962,'935-13/99516','SkyTerra 1','MSV 1','','Sat',553,'',null from dual union all
select 68922,'2010-062','2010-11-20 01:25:00','orbital','success',787,'3','STP S26','STP-SIV','','Sat',861,'',null from dual union all
select 68923,'2010-063','2010-11-21 22:58:00','orbital','success',397,'D4-14 (351)','USA 223      [RH11]','ORION 7','','Sat',290,'',null from dual union all
select 68924,'2010-064','2010-11-24 16:09:04','orbital','success',309,'Y21','Zhongxing 20A','Shentong yi hao 02 xing','','Sat',1904,'',null from dual union all
select 68925,'2010-065','2010-11-26 18:39:00','orbital','success',111,'V198 (556)','IS-17/Hylas 1','Hylas','','Sat',346,'',null from dual union all
select 68926,'2010-066','2010-12-08 15:43:00','orbital','success',482,'002','Dragon C1','Dragon C101','','Sat',274,'',null from dual union all
select 68927,'2010-067','2010-12-15 19:09:25','orbital','success',1233,'B 15000-034 034','Soyuz TMA-20','Soyuz 11F732A17 No. 230','','Sat ISS 25S',480,'',null from dual union all
select 68928,'2010-068','2010-12-17 20:20:04','orbital','success',309,'Y18','Beidou DW7','diqi ke beidou daohang w','','Sat',1904,'',null from dual union all
select 68929,'2010-069','2010-12-26 21:51:00','orbital','success',962,'935-17/99518','KA-Sat','KA-Sat','','Sat',553,'',null from dual union all
select 6893,'1960-S24','1960-01-27 01:31:00','military missile','success',154,'44D','RVX-4A-2','R&D','','Test',228,'',1800 from dual union all
select 68930,'2010-070','2010-12-29 21:27:00','orbital','success',111,'V199 (557)','Hisp 1E/Koreasat 6','Hispasat 1E','','Sat',346,'',null from dual union all
select 68931,'2010-A01','2010-03-03 03:00:00','suborbital rocket','success',24,'ATV-D01','SPEX Scramjet','','','Test/Tech',1361,'',46 from dual union all
select 68932,'2010-A02','2010-05-26 17:10:00','atmospheric rocket','success',1525,'1','','','','Hypersonic Scramjet',379,'B-52H',19 from dual union all
select 68933,'2010-A03','2010-06-29 07:37:00','military missile','success',1294,'THAAD FTT-14','Low Endo','','','ABM Test',855,'',20 from dual union all
select 68934,'2010-A04','2010-07-26 04:40:00','military missile','success',11,'03','AAD Interceptor','','','Interceptor',316,'',15 from dual union all
select 68935,'2010-A05','2010-12-10 04:35:00','military missile','failure',64,'Agni II Prime','Agni RV','','','Test',316,'',20 from dual union all
select 68936,'2010-A06','2010-10-10 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-41/GF01','','','Glide 13:00',955,'WK2',14 from dual union all
select 68937,'2010-A07','2010-10-28 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-44/GF02','','','Glide 10:51',955,'WK2',14 from dual union all
select 68938,'2010-A08','2010-11-17 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-45/GF03','','','Glide 11:39',955,'WK2',14 from dual union all
select 68939,'2010-A09','2010-03-22 00:00:00','suborbital spaceplane','success',1246,'','WK2-25/CC01','','','Captive carry',957,'WK2',10 from dual union all
select 6894,'1960-S240','1960-08-16 10:41:00','suborbital rocket','success',839,'AA6.426','Firefly PEGGY','','','Aeronomy',394,'',103 from dual union all
select 68940,'2010-A10','2010-05-16 00:00:00','suborbital spaceplane','success',1246,'','WK2-27/CC02','','','Captive carry',957,'WK2',15 from dual union all
select 68941,'2010-A11','2010-07-15 00:00:00','suborbital spaceplane','success',1246,'','WK2-33/CC03','','','Captive carry',957,'WK2',10 from dual union all
select 68942,'2010-A12','2010-09-30 00:00:00','suborbital spaceplane','success',1246,'','WK2-39/CC04','','','Captive carry',957,'WK2',10 from dual union all
select 68943,'2010-A13','2010-10-22 00:00:00','suborbital spaceplane','success',1246,'','WK2-42/CC05','','','To Spaceport America',957,'WK2',10 from dual union all
select 68944,'2010-A14','2010-10-22 00:00:00','suborbital spaceplane','success',1246,'','WK2-43/CC06','','','Spaceport Am. to MHV',1424,'WK2',10 from dual union all
select 68945,'2010-A15','2010-02-25 18:30:00','atmospheric rocket','success',472,'5','','','','Test',330,'',32 from dual union all
select 68946,'2010-A16','2010-04-27 18:00:00','atmospheric rocket','success',472,'6','','','','Test',13,'',32 from dual union all
select 68947,'2010-A17','2010-04-29 18:00:00','atmospheric rocket','success',472,'7','','','','Test',330,'',32 from dual union all
select 68948,'2010-A18','2010-07-26 17:30:00','atmospheric rocket','success',472,'8','','','','Test',13,'',32 from dual union all
select 68949,'2010-A19','2010-08-03 15:25:00','atmospheric rocket','success',473,'1','','','','Test',330,'',38 from dual;
insert into LAUNCH
select 6895,'1960-S241','1960-08-17 10:42:00','suborbital rocket','success',839,'AA6.427','Firefly SUSAN','','','Aeronomy',394,'',114 from dual union all
select 68950,'2010-A20','2010-09-20 18:10:00','atmospheric rocket','success',472,'9','','','','Test',13,'',32 from dual union all
select 68951,'2010-A21','2010-09-30 19:00:00','atmospheric rocket','success',473,'2','','','','Test',13,'',32 from dual union all
select 68952,'2010-A22','2010-05-06 13:00:00','test rocket','success',892,'PA-1','','','','Test',1861,'',1 from dual union all
select 68953,'2010-F01','2010-04-15 10:57:00','orbital','failure',504,'GSLV-D3','GSAT-4','GSAT-4','','Sat',1359,'',137 from dual union all
select 68954,'2010-F02','2010-06-10 08:01:00','orbital','failure',831,'74502','STSAT-2B','STSat-2B','','Sat',987,'',70 from dual union all
select 68955,'2010-F03','2010-12-05 10:25:19','orbital','failure',964,'535-37/01L','Glonass-M','Uragan-M No. 739','','Sat',588,'',169 from dual union all
select 68956,'2010-F04','2010-12-25 10:34:00','orbital','failure',503,'GSLV-F06','GSAT-5P','GSAT-5 Prime','','Sat',1359,'',15 from dual union all
select 68957,'2010-M01','2010-01-14 07:37:00','suborbital rocket','success',1023,'','Eclipse','','','Eclipse',1467,'',70 from dual union all
select 68958,'2010-M02','2010-01-14 09:30:00','suborbital rocket','success',1023,'','Eclipse','','','Eclipse',1467,'',70 from dual union all
select 68959,'2010-M03','2010-01-15 07:30:00','suborbital rocket','success',1023,'','Eclipse','','','Eclipse',1467,'',70 from dual union all
select 6896,'1960-S242','1960-08-19 03:20:00','military missile','success',928,'A1X-43','','','','Test',244,'',500 from dual union all
select 68960,'2010-M04','2010-01-15 09:30:00','suborbital rocket','success',1023,'','Eclipse','','','Eclipse',1467,'',70 from dual union all
select 68961,'2010-M05','2010-03-15 04:32:00','military missile','failure',941,'','AAD Target','','','Target',320,'',70 from dual union all
select 68962,'2010-M06','2010-07-07 10:20:00','suborbital rocket','success',1023,'TDF (RH-200-395)','Tech Demo Flight','','','Test/Edu',1467,'',60 from dual union all
select 68963,'2010-M07','2010-12-22 02:45:00','military missile','success',941,'','Test','','','Test',315,'',50 from dual union all
select 68964,'2010-M08','2010-12-22 03:45:00','military missile','success',941,'','Test','','','Test',315,'',50 from dual union all
select 68965,'2010-M10','2010-05-08 00:00:00','military missile','success',1169,'Hatf IV','','','','Test',1419,'',100 from dual union all
select 68966,'2010-M11','2010-05-08 00:00:00','military missile','success',510,'Hatf III','','','','Test',1419,'',50 from dual union all
select 68967,'2010-M12','2010-08-24 00:00:00','military missile','success',489,'','','','','Test',759,'',60 from dual union all
select 68968,'2010-S01','2010-01-11 11:50:00','military missile','success',198,'ABM Target','CSS-X-11','','','ABM',819,'',250 from dual union all
select 68969,'2010-S02','2010-01-11 11:52:42','military missile','success',354,'ABM Interceptor','','','','ABM',1629,'',250 from dual union all
select 6897,'1960-S243','1960-08-18 01:35:00','suborbital rocket','success',838,'AA8.415','','','','Aeronomy',394,'',150 from dual union all
select 68970,'2010-S03','2010-01-14 06:50:00','suborbital rocket','success',1026,'','Eclipse','','','Eclipse',1467,'',116 from dual union all
select 68971,'2010-S04','2010-01-14 07:35:00','suborbital rocket','success',1026,'','Eclipse','','','Eclipse',1467,'',116 from dual union all
select 68972,'2010-S05','2010-01-14 00:00:00','suborbital rocket','success',1029,'','Eclipse','','','Eclipse',1361,'',548 from dual union all
select 68973,'2010-S06','2010-01-15 07:00:00','suborbital rocket','success',1026,'','Eclipse','','','Eclipse',1467,'',116 from dual union all
select 68974,'2010-S07','2010-01-15 07:30:00','suborbital rocket','success',1026,'','Eclipse','','','Eclipse',1467,'',116 from dual union all
select 68975,'2010-S08','2010-01-15 10:30:00','suborbital rocket','success',1026,'','Eclipse','','','Eclipse',1467,'',116 from dual union all
select 68976,'2010-S09','2010-01-15 07:45:00','suborbital rocket','success',1029,'No. 9','Eclipse','','','Eclipse',1361,'',523 from dual union all
select 68977,'2010-S10','2010-01-10 23:00:00','military missile','success',224,'NASA 36.247DR','MARTI ABL Target','','','Target',1415,'',100 from dual union all
select 68978,'2010-S11','2010-01-27 08:25:00','military missile','success',758,'4','N Atlantic','','','Test',207,'S612',1000 from dual union all
select 68979,'2010-S12','2010-01-31 23:39:00','military missile','success',695,'','GMD FTG-06 Target','','','Target',868,'',1000 from dual union all
select 6898,'1960-S244','1960-08-18 02:42:00','suborbital rocket','success',839,'AA6.428','Firefly LINDA','','','Aeronomy',394,'',133 from dual union all
select 68980,'2010-S13','2010-01-31 23:45:05','military missile','success',497,'','GMD FTG-06','','','Test',1709,'',1000 from dual union all
select 68981,'2010-S14','2010-02-03 00:00:00','suborbital rocket','success',833,'Kavoshgar-3','Kavoshgar Class B','','','Bio',1356,'',55 from dual union all
select 68982,'2010-S15','2010-02-07 05:20:00','military missile','success',65,'','','','','Test',316,'',350 from dual union all
select 68983,'2010-S16','2010-02-09 09:01:00','suborbital rocket','success',1334,'NASA 41.084UE','Ampules','','','Aeron TMA',1047,'',160 from dual union all
select 68984,'2010-S17','2010-02-12 04:44:00','military missile','success',1151,'FMA','ALTB Target','','','Target',1271,'MLP',100 from dual union all
select 68985,'2010-S18','2010-02-12 05:31:20','military missile','success',224,'NASA 36.251DR','MARTI ABL Target','','','Target',1415,'',150 from dual union all
select 68986,'2010-S19','2010-02-16 09:49:11','suborbital rocket','success',237,'NASA 40.025UE','CHARM II','','','Auroral',1047,'',803 from dual union all
select 68987,'2010-S20','2010-02-04 08:03:07','military missile','success',224,'NASA 36.250DR','MARTI ABL Target','','','Target',1415,'',100 from dual union all
select 68988,'2010-S21','2010-03-04 04:50:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-114',1000 from dual union all
select 68989,'2010-S22','2010-03-02 07:25:00','suborbital rocket','success',605,'SSC REXUS 7','EURO REXUS 7','','','Aeron/Tech',417,'',83 from dual union all
select 6899,'1960-S245','1960-08-18 10:15:00','suborbital rocket','success',838,'AA8.416','','','','Aeronomy',394,'',150 from dual union all
select 68990,'2010-S23','2010-03-04 10:15:00','suborbital rocket','success',605,'SSC REXUS 8','EURO REXUS 8','','','Tech',417,'',88 from dual union all
select 68991,'2010-S24','2010-03-26 13:43:00','suborbital rocket','success',772,'MAXUS 8','SSC MAXUS 8','','','Micrograv',407,'',700 from dual union all
select 68992,'2010-S25','2010-03-27 00:14:00','military missile','success',430,'','Test','','','Target',320,'INS-P51',100 from dual union all
select 68993,'2010-S26','2010-03-27 00:18:00','military missile','success',941,'','Test','','','Target',315,'',100 from dual union all
select 68994,'2010-S27','2010-03-27 14:09:56','suborbital rocket','success',1332,'NASA 12.067GT','Test/ADAMASat/CP1U','','','Test',1798,'',270 from dual union all
select 68995,'2010-S28','2010-03-28 07:37:00','military missile','success',62,'','Agni RV','','','Test',316,'',300 from dual union all
select 68996,'2010-S29','2010-03-22 00:45:00','suborbital rocket','success',1334,'','HiFire HF1','','','Hypersonic',1849,'',288 from dual union all
select 68997,'2010-S30','2010-04-22 23:00:00','military missile','success',789,'1','HTV-2a','','','Test',1781,'',100 from dual union all
select 68998,'2010-S31','2010-05-03 18:32:00','suborbital rocket','success',224,'NASA 36.258UE','SDO EVE Cal','','','Solar EUV',1895,'',280 from dual union all
select 68999,'2010-S32','2010-05-04 12:41:02','suborbital rocket','success',1244,'SpaceLoft SL-4','','','','Edu/Burial',1422,'',113 from dual union all
select 69,'1944-A33','1944-03-17 11:00:00','military missile','failure',8,'17.047','','','','Test',675,'',1 from dual union all
select 690,'1947-A26','1947-10-30 00:00:00','atmospheric rocket','success',876,'47-13','Nike-47 11R','','','Test',1863,'',10 from dual union all
select 6900,'1960-S246','1960-08-19 10:40:00','suborbital rocket','success',839,'AA6.429','Firefly OLIVE','','','Aeronomy',394,'',106 from dual union all
select 69000,'2010-S33','2010-05-05 11:50:00','suborbital rocket','success',1221,'7','','','','Ionos',808,'',287 from dual union all
select 69001,'2010-S34','2010-05-17 03:48:00','military missile','success',63,'Agni A-2','Agni RV','','','Test',316,'',300 from dual union all
select 69002,'2010-S35','2010-05-03 09:47:00','military missile','success',224,'NASA 36.248DR','MARTI ABL Target','','','Target',1415,'',150 from dual union all
select 69003,'2010-S36','2010-05-21 09:00:00','suborbital rocket','success',224,'NASA 36.270UG','DICE','','','UV Astron',1895,'',325 from dual union all
select 69004,'2010-S37','2010-06-06 22:25:00','military missile','success',496,'','BVT-1','','','Test',1710,'',300 from dual union all
select 69005,'2010-S38','2010-06-08 00:00:00','military missile','success',1443,'FCET-42','-    (131ss)','','','OT',421,'SSBN 738',1000 from dual union all
select 69006,'2010-S39','2010-06-08 00:00:00','military missile','success',1443,'FCET-42','-    (132ss)','','','OT',421,'SSBN 738',1000 from dual union all
select 69007,'2010-S40','2010-06-09 00:00:00','military missile','success',1443,'FCET-43','-    (133ss)','','','OT',421,'SSBN 738',1000 from dual union all
select 69008,'2010-S41','2010-06-09 00:00:00','military missile','success',1443,'FCET-43','-    (134ss)','','','OT',421,'SSBN 738',1000 from dual union all
select 69009,'2010-S42','2010-06-16 10:01:00','military missile','success',799,'','Mk 21 RV','GT200GM-1','GLORY TRIP 200GM-1','Op Test',1706,'',1300 from dual union all
select 6901,'1960-S247','1960-08-19 19:56:00','suborbital rocket','success',839,'AA6.430','Firefly RENA  (SF6)','','','Aeronomy',394,'',105 from dual union all
select 69010,'2010-S43','2010-06-24 11:17:00','suborbital rocket','success',1334,'NASA 41.088UO','RockOn','','','Education',1809,'',117 from dual union all
select 69011,'2010-S44','2010-06-30 10:40:01','military missile','success',799,'','Mk 21 RV','GT201GM','GLORY TRIP 201GM','Op Test',1699,'',1300 from dual union all
select 69012,'2010-S45','2010-07-11 04:50:00','suborbital rocket','success',224,'NASA 36.265UG','CIBER-I F2','','','IR Astron',1895,'',320 from dual union all
select 69013,'2010-S46','2010-06-29 07:32:00','military missile','success',1151,'THAAD FTT-14 target','FMA','','','Target',855,'MLP',80 from dual union all
select 69014,'2010-S47','2010-07-10 07:32:00','military missile','success',758,'5','N Atlantic','','','Test',207,'S612',1000 from dual union all
select 69015,'2010-S48','2010-07-26 04:35:00','military missile','success',941,'','AAD Target','','','Target',315,'',100 from dual union all
select 69016,'2010-S49','2010-07-30 18:21:00','suborbital rocket','success',224,'NASA 36.213NS','SUMI','','','Solar EUV',1869,'',246 from dual union all
select 69017,'2010-S50','2010-08-04 09:15:03','suborbital rocket','success',231,'NASA 12.073GT','Motor test','','','Test',1797,'',691 from dual union all
select 69018,'2010-S51','2010-08-06 15:15:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-114',1000 from dual union all
select 69019,'2010-S52','2010-08-06 15:15:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-114',1000 from dual union all
select 6902,'1960-S248','1960-08-20 00:00:00','military missile','success',976,'','8K63 UBP-72','','','Test',636,'',402 from dual union all
select 69020,'2010-S53','2010-08-23 17:57:00','suborbital rocket','success',224,'NASA 36.219US','RAISE','','','Solar UV',1895,'',295 from dual union all
select 69021,'2010-S54','2010-08-20 00:00:00','military missile','success',967,'Qiam-1','','','','Test',1356,'',126 from dual union all
select 69022,'2010-S55','2010-08-30 20:00:00','suborbital rocket','success',1074,'S-520-25','T-REX','','','Tech/Ionos',833,'',309 from dual union all
select 69023,'2010-S56','2010-09-17 10:03:00','military missile','success',799,'','Mk 21 RV','GT202GM','GLORY TRIP 220GM','Op Test (Guam)',1705,'',1300 from dual union all
select 69024,'2010-S57','2010-09-01 00:00:00','military missile','success',1151,'','ALTB FEL-01b Target','','','Target',1057,'',100 from dual union all
select 69025,'2010-S58','2010-09-21 13:07:30','suborbital rocket','success',1334,'NASA 41.082NP','SUBTEC III','','','Tech',1801,'',130 from dual union all
select 69026,'2010-S59','2010-10-07 00:00:00','military missile','success',261,'R-30 13','','','','R&D?',190,'TK-208',1000 from dual union all
select 69027,'2010-S60','2010-10-27 10:15:00','suborbital rocket','success',847,'','DLR MAPHEUS-2','','','Microgravity',417,'',135 from dual union all
select 69028,'2010-S61','2010-10-21 17:00:00','military missile','success',224,'NASA 36.230DR','MARTI ABL Target','','','Target',1415,'',100 from dual union all
select 69029,'2010-S62','2010-10-28 09:59:00','military missile','success',1434,'','15Zh58 UBP-36','','','OT',608,'',1000 from dual union all
select 6903,'1960-S249','1960-08-23 17:01:00','suborbital rocket','success',35,'NASA 04.16UE','','','','Cosmic rays',1821,'',190 from dual union all
select 69030,'2010-S63','2010-10-28 10:30:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-117',1000 from dual union all
select 69031,'2010-S64','2010-10-28 14:00:00','military missile','success',1507,'VMF RF','','','','OT',1011,'K-433',1000 from dual union all
select 69032,'2010-S65','2010-10-29 01:10:00','military missile','success',261,'R-30 14','','','','R&D?',190,'TK-208',1000 from dual union all
select 69033,'2010-S66','2010-10-29 03:06:00','military missile','success',748,'Aegis JFTM-4 target','MRT-10 (Sep MRT)','','','Target',855,'',150 from dual union all
select 69034,'2010-S67','2010-10-29 03:09:00','military missile','success',1090,'Aegis JFTM-4','Stellar Taka','','','Interceptor',851,'DDG-174',150 from dual union all
select 69035,'2010-S68','2010-12-05 19:11:00','military missile','success',1435,'','15Zh58E GCh-4','','','OT',636,'',1000 from dual union all
select 69036,'2010-S69','2010-12-06 16:45:00','suborbital rocket','success',1334,'NASA 41.087NT','TRaINED','','','Tech',1895,'',120 from dual union all
select 69037,'2010-S70','2010-11-25 04:40:00','military missile','success',62,'','Agni RV','','','Test',316,'',300 from dual union all
select 69038,'2010-S71','2010-12-06 17:19:00','suborbital rocket','success',605,'AEB V03','Maracati 2','','','Test',13,'',103 from dual;
insert into LAUNCH
select 69039,'2010-S72','2010-12-12 15:35:00','suborbital rocket','success',1484,'V07','Maracati 2/MICROG 1A','','','Micrograv',13,'',242 from dual union all
select 6904,'1960-S25','1960-01-29 00:00:00','military missile','success',976,'','8K63 UBP-36','','','Test',636,'',402 from dual union all
select 69040,'2010-S73','2010-12-12 06:38:00','suborbital rocket','success',237,'NASA 40.026UE','RENU','','','Auroral',29,'',340 from dual union all
select 69041,'2010-S74','2010-12-04 04:21:00','suborbital rocket','success',847,'','ECOMA 7','','','Meteors/Dust',29,'',135 from dual union all
select 69042,'2010-S75','2010-12-13 03:24:30','suborbital rocket','success',847,'','ECOMA 8','','','Meteors/Dust',29,'',138 from dual union all
select 69043,'2010-S76','2010-12-15 19:57:00','military missile','success',695,'','GMD FTG-06A Target','','','Target',868,'',1000 from dual union all
select 69044,'2010-S77','2010-12-15 20:03:00','military missile','success',497,'','GMD FTG-06a','','','Test',1709,'',1000 from dual union all
select 69045,'2010-S78','2010-12-19 02:36:00','suborbital rocket','success',847,'','ECOMA 9','','','Meteors/Dust',29,'',135 from dual union all
select 69046,'2010-S80','2010-12-21 00:00:00','military missile','success',509,'Hatf-5','','','','Test',1473,'',400 from dual union all
select 69047,'2010-S81','2010-02-17 00:00:00','military missile','success',632,'1/MBRV-3','PAC-3 Target','','','Target',436,'',100 from dual union all
select 69048,'2010-S82','2010-10-01 00:00:00','military missile','success',1157,'R&D','Sejjil-2','','','Test',1348,'',800 from dual union all
select 69049,'2010-S83','2010-11-12 00:00:00','military missile','success',1344,'Aegis Target 16E2B','ARAV-B14','','','Target',855,'',100 from dual union all
select 6905,'1960-S250','1960-08-23 17:10:00','suborbital rocket','success',30,'AA3.128','','','','Solar EUV',1869,'',237 from dual union all
select 69050,'2010-S84','2010-10-06 00:00:00','military missile','success',1344,'Aegis Target','JFTM-4 E1','','','Target',855,'',100 from dual union all
select 69051,'2010-S85','2010-10-06 00:00:00','military missile','success',1344,'Aegis Target','JFTM-4 E1','','','Target',855,'',100 from dual union all
select 69052,'2010-S86','2010-06-18 01:20:00','military missile','success',941,'','Test','','','Test',315,'',100 from dual union all
select 69053,'2011-001','2011-01-20 12:29:02','orbital','success',1544,'SB80.1-2007/1/2001','Elektro-L','Elektro-L','','Sat',567,'',null from dual union all
select 69054,'2011-002','2011-01-20 21:10:30','orbital','success',397,'D4-15 (352)','USA 224      [IC]','Improved CRYSTAL 2109?','','Sat',1780,'',null from dual union all
select 69055,'2011-003','2011-01-22 05:37:57','orbital','success',534,'H-IIB-F2','HTV-2','HTV 2','','Sat',1478,'',null from dual union all
select 69056,'2011-004','2011-01-28 01:31:39','orbital','success',1241,'I 15000-126 126','Progress M-09M','Progress 7K-TGM No.409','','Sat ISS 41P',480,'',null from dual union all
select 69057,'2011-005','2011-02-01 14:00:14','orbital','failure',1064,'630-9793-568/72517','Geo-IK-2','Musson-2 No. 11','','Sat',460,'',null from dual union all
select 69058,'2011-006','2011-02-06 12:26:00','orbital','success',783,'','RPP','USA 225','','Sat',1781,'',null from dual union all
select 69059,'2011-007','2011-02-16 21:51:02','orbital','success',113,'V200 (544)','ATV 2','ATV 2','','Sat',346,'',null from dual union all
select 6906,'1960-S251','1960-08-25 01:04:00','suborbital rocket','success',838,'AA8.244','Firefly','','','Aeronomy',394,'',200 from dual union all
select 69060,'2011-008','2011-02-24 21:53:24','orbital','success',1243,'STS-133','ISS ULF-5','OV-103 ISS','','Sat',921,'',null from dual union all
select 69061,'2011-009','2011-02-26 03:07:15','orbital','success',1228,'770-24 -208 017','Glonass-K1','Uragan-K1 No. 11L','','Sat',475,'',null from dual union all
select 69062,'2011-010','2011-03-05 22:46:00','orbital','success',190,'AV-026','X-37B OTV FLT-2','X-37B No. 2','','Sat',292,'',null from dual union all
select 69063,'2011-011','2011-03-11 23:38:00','orbital','success',400,'D4-16 (353)','USA 227      [SDS17]','QUASAR 17?','','Sat',290,'',null from dual union all
select 69064,'2011-012','2011-04-04 22:18:20','orbital','success',1233,'I 15000-036 036','Soyuz TMA-21','Soyuz 11F732A17 No. 231','','Sat ISS 26S',480,'',null from dual union all
select 69065,'2011-013','2011-04-09 20:47:04','orbital','success',309,'Y19','Beidou DW8','diba ke beidou daohang w','','Sat',1904,'',null from dual union all
select 69066,'2011-014','2011-04-15 04:24:10','orbital','success',184,'AV-027','USA 229','INTRUDER 9','','Sat',1775,'',null from dual union all
select 69067,'2011-015','2011-04-20 04:42:00','orbital','success',899,'PSLV-C16','Resourcesat-2','Resourcesat-2','','Sat',1357,'',null from dual union all
select 69068,'2011-016','2011-04-22 21:37:00','orbital','success',111,'VA201 (558)','Yahsat 1A/New Dawn','Intelsat New Dawn','','Sat',346,'',null from dual union all
select 69069,'2011-017','2011-04-27 13:05:21','orbital','success',1241,'Yu15000-116 116','Progress M-10M','Progress 7K-TGM No.410','','Sat ISS 42P',480,'',null from dual union all
select 6907,'1960-S252','1960-08-25 14:28:00','suborbital rocket','success',545,'AA5.431','Firefly ZELDA','','','Aeronomy',394,'',102 from dual union all
select 69070,'2011-018','2011-05-04 17:41:33','orbital','success',1226,'760-12 -230 015','Meridian No. 14','Meridian No. 14L','','Sat',475,'',null from dual union all
select 69071,'2011-019','2011-05-07 18:10:00','orbital','success',182,'AV-022','SBIRS GEO-1','SBIRS GEO-1','','Sat',292,'',null from dual union all
select 69072,'2011-020','2011-05-16 12:56:28','orbital','success',1243,'STS-134','ISS ULF-6','OV-105 ISS','','Sat',921,'',null from dual union all
select 69073,'2011-021','2011-05-20 19:15:19','orbital','success',962,'935-19/99520','Telstar 14R','Telstar 14R','','Sat',553,'',null from dual union all
select 69074,'2011-022','2011-05-20 20:38:00','orbital','success',111,'VA202 (559)','ST-2/GSAT-8','INSAT-4G','','Sat',346,'',null from dual union all
select 69075,'2011-023','2011-06-07 20:12:45','orbital','success',1233,'I 15000-037 037','Soyuz TMA-02M','Soyuz 11F732A47 No. 702','','Sat ISS 27S',480,'',null from dual union all
select 69076,'2011-024','2011-06-10 14:20:13','orbital','success',411,'D354','SAC-D','SAC-D','','Sat',1724,'',null from dual union all
select 69077,'2011-025','2011-06-15 09:15:03','orbital','success',1114,'UIS.0001','Rasad  ','','','Sat',1347,'',null from dual union all
select 69078,'2011-026','2011-06-20 16:13:04','orbital','success',311,'Y20','Zhongxing 10','zhongxing shi hao weixin','','Sat',1903,'',null from dual union all
select 69079,'2011-027','2011-06-21 14:38:15','orbital','success',1241,'I 15000-128 128','Progress M-11M','Progress 7K-TGM No.411','','Sat ISS 43P',480,'',null from dual union all
select 6908,'1960-S253','1960-08-26 01:03:00','suborbital rocket','success',838,'AA8.417','Firefly','','','Aeronomy',394,'',150 from dual union all
select 69080,'2011-028','2011-06-27 16:00:00','orbital','success',1241,'760-12 -222 125','Kosmos-2472','Kobal''t-M','','Sat',462,'',null from dual union all
select 69081,'2011-029','2011-06-30 03:09:00','orbital','success',783,'','ORS-1','ORS-1','','Sat',945,'',null from dual union all
select 69082,'2011-030','2011-07-06 04:28:04','orbital','success',301,'Y25','SJ-11-03','shijian shi yihao 03 xin','','Sat',813,'',null from dual union all
select 69083,'2011-031','2011-07-08 15:29:04','orbital','success',1243,'STS-135','ISS ULF-7','OV-104 ISS','','Sat',921,'',null from dual union all
select 69084,'2011-032','2011-07-11 15:41:04','orbital','success',316,'Y8','Tian Lian 1-02','tianlian yihao 02 xing','','Sat',1903,'',null from dual union all
select 69085,'2011-033','2011-07-13 02:27:04','orbital','success',1226,'Yu15000-008 008/ST23','Globalstar','Globalstar-2 FM7?','','Sat',562,'',null from dual union all
select 69086,'2011-034','2011-07-15 11:18:00','orbital','success',902,'PSLV-C17','GSAT-12','GSAT-12','','Sat',1359,'',null from dual union all
select 69087,'2011-035','2011-07-15 23:16:10','orbital','success',962,'935-18/99519','SES-3/Kazsat-2','SES-3','','Sat',553,'',null from dual union all
select 69088,'2011-036','2011-07-16 06:41:00','orbital','success',400,'D4-17 (355) 4240','GPS IIF SV-2','GPS SVN 63','','Sat',290,'',null from dual union all
select 69089,'2011-037','2011-07-18 02:31:18','orbital','success',1544,'70181-401/SLB80.3','Spektr-R','RadioAstron','','Sat',567,'',null from dual union all
select 6909,'1960-S254','1960-08-26 11:00:00','suborbital rocket','success',1194,'SL13','QUB Na-5-MM','','','Aeron',1839,'',109 from dual union all
select 69090,'2011-038','2011-07-26 21:44:28','orbital','success',309,'Y17','Beidou DW9','di jiu ke beidou daohang','','Sat',1904,'',null from dual union all
select 69091,'2011-039','2011-07-29 07:42:03','orbital','success',301,'Y24','SJ-11-02','shijian shi yihao 02 xin','','Sat',813,'',null from dual union all
select 69092,'2011-040','2011-08-05 16:25:00','deep space','success',196,'AV-029','Juno','Juno','','Sat',292,'',null from dual union all
select 69093,'2011-041','2011-08-06 22:52:37','orbital','success',111,'VA203 (560)','Astra 1N/BSAT-3c','Astra 1N','','Sat',346,'',null from dual union all
select 69094,'2011-042','2011-08-11 16:15:04','orbital','success',311,'Y19','Paksat-1R','Paksat 1R','','Sat',1903,'',null from dual union all
select 69095,'2011-043','2011-08-15 22:57:19','orbital','success',319,'Y14','Haiyang-2','Haiyang er hao','','Sat',1494,'',null from dual union all
select 69096,'2011-044','2011-08-17 07:12:20','orbital','success',434,'450-3261-319','Sich-2/','Edusat-ASI','','Sat',1907,'',null from dual union all
select 69097,'2011-045','2011-08-17 21:25:01','orbital','failure',962,'935-21/99522','Ekspress AM-4','Ekspress AM-4','','Sat',553,'',null from dual union all
select 69098,'2011-046','2011-09-10 13:08:52','deep space','success',421,'D356','GRAIL-A/GRAIL-B','GRAIL A','','Sat',286,'',null from dual union all
select 69099,'2011-047','2011-09-18 16:33:03','orbital','success',313,'Y16','Zhongxing 1A','Fenghuo 2','','Sat',1903,'',null from dual union all
select 691,'1947-A27','1947-10-31 00:00:00','suborbital rocket','success',58,'A3','Dummy','','','Test',1869,'',1 from dual union all
select 6910,'1960-S255','1960-08-29 03:38:00','suborbital rocket','success',1438,'TB Ic','D58 Trailblazer','','','Reentry Test',1821,'',260 from dual union all
select 69100,'2011-048','2011-09-20 22:47:00','orbital','success',962,'535-42/88529','Kosmos-2473','Garpun No. 1','','Sat',588,'',null from dual union all
select 69101,'2011-049','2011-09-21 21:38:00','orbital','success',111,'VA204 (561)','Arabsat 5C/SES-2','SES-2','','Sat',346,'',null from dual union all
select 69102,'2011-050','2011-09-23 04:36:50','orbital','success',528,'H-IIA-19','IGS O-4','IGS Optical-4','','Sat',1477,'',null from dual union all
select 69103,'2011-051','2011-09-24 20:17:59','orbital','success',1542,'SL31/DM-SL-31L','SL39 Atlantic Bird 7','Atlantic Bird 7','','Sat',860,'ODYSSEY',null from dual union all
select 69104,'2011-052','2011-09-27 15:49:00','orbital','success',790,'5','Tacsat-4','TacSat-4','','Sat',861,'',null from dual union all
select 69105,'2011-053','2011-09-29 13:16:03','orbital','success',307,'Y1','Tiangong 1','Tiangong 1','','Sat',814,'',null from dual union all
select 69106,'2011-054','2011-09-29 18:32:00','orbital','success',962,'935-22/99524','QuetzSat-1','QuetzSat-1','','Sat',553,'',null from dual union all
select 69107,'2011-055','2011-10-02 20:15:14','orbital','success',1228,'770-24 -209 018/1045','Glonass-M42','Uragan-M No. 742','','Sat',475,'',null from dual union all
select 69108,'2011-056','2011-10-05 21:00:02','orbital','success',1543,'SB60.5/SLB 5L','Intelsat IS-18','IS-18','','Sat',567,'',null from dual union all
select 69109,'2011-057','2011-10-07 08:21:04','orbital','success',311,'Y18','Eutelsat W3C','Eutelsat W3C','','Sat',1903,'',null from dual union all
select 6911,'1960-S256','1960-08-29 06:13:00','suborbital rocket','success',1438,'TB Id','D58 Trailblazer','','','Reentry Test',1821,'',260 from dual union all
select 69110,'2011-058','2011-10-12 05:31:00','orbital','success',900,'PSLV-C18','Megha-Tropiques','Megha-Tropiques','','Sat',1357,'',null from dual union all
select 69111,'2011-059','2011-10-19 18:48:58','orbital','success',962,'935-20/99521','ViaSat-1','ViaSat-1','','Sat',553,'',null from dual union all
select 69112,'2011-060','2011-10-21 10:30:26','orbital','success',1236,'B 15000-001 001','Galileo IOV-1/2','Galileo PFM','','Sat VS01/SZ08 1030MT',347,'',null from dual union all
select 69113,'2011-061','2011-10-28 09:48:02','orbital','success',418,'D357','NPP','NPOESS Prep Project','','Sat',1724,'',null from dual union all
select 69114,'2011-062','2011-10-30 10:11:12','orbital','success',1241,'I 15000-129 129','Progress M-13M','Progress 7K-TGM No.413','','Sat ISS 45P',480,'',null from dual union all
select 69115,'2011-063','2011-10-31 21:58:10','orbital','success',306,'Y8','Shenzhou 8','shenzhou ba fanhui cang','','Sat',814,'',null from dual union all
select 69116,'2011-064','2011-11-04 12:51:41','orbital','success',962,'535-39/99523','Kosmos-2475/2477','Uragan-M No. 743','','Sat',588,'',null from dual union all
select 69117,'2011-065','2011-11-08 20:16:03','orbital','success',1540,'2SB41.1,2FG','Fobos-Grunt','Pereletniy modul'' FG','','Sat',567,'',null from dual union all
select 69118,'2011-066','2011-11-09 03:21:05','orbital','success',319,'Y21','Yaogan 12','tianxun yihao','','Sat',1494,'',null from dual union all
select 69119,'2011-067','2011-11-14 04:14:03','orbital','success',1233,'I 15000-038 038','Soyuz TMA-22','Soyuz 11F732A17 No. 232','','Sat ISS 28S',480,'',null from dual union all
select 6912,'1960-S257','1960-08-30 00:00:00','suborbital rocket','failure',838,'','Firefly','','','Ionos',394,'',150 from dual union all
select 69120,'2011-068','2011-11-20 00:15:04','orbital','success',303,'Y19','SW-4/CX-1-03','chuangxin yihao 03 xing','','Sat',813,'',null from dual union all
select 69121,'2011-069','2011-11-25 19:10:34','orbital','success',962,'935-25/99527','Asiasat-7','Asiasat 7','','Sat',553,'',null from dual union all
select 69122,'2011-070','2011-11-26 15:02:00','deep space','success',194,'AV-028','MSL','Mars Science Lab Rover','','Sat',292,'',null from dual union all
select 69123,'2011-071','2011-11-28 08:25:58','orbital','success',1228,'780-31 -219 019/1046','Glonass-M46','Uragan-M No. 746','','Sat',475,'',null from dual union all
select 69124,'2011-072','2011-11-29 18:50:04','orbital','success',301,'Y20','Yaogan 13','yaogan weixing shisan ha','','Sat',1493,'',null from dual union all
select 69125,'2011-073','2011-12-01 21:07:04','orbital','success',309,'Y23','Beidou DW10','di shi ke beidou daohang','','Sat',1904,'',null from dual union all
select 69126,'2011-074','2011-12-11 11:17:00','orbital','success',962,'935-23/99525','Luch-5A/Amos-5','Amos 5','','Sat',588,'',null from dual union all
select 69127,'2011-075','2011-12-12 01:21:00','orbital','success',528,'H-IIA-20','IGS R-3','IGS Radar-3','','Sat',1477,'',null from dual union all
select 69128,'2011-076','2011-12-17 02:03:48','orbital','success',1234,'Sh15000-002 002/1024','Pleiades/Elisa','Elisa 1','','Sat VS02/SZ02',347,'',null from dual;
insert into LAUNCH
select 69129,'2011-077','2011-12-19 16:41:03','orbital','success',311,'Y21','Nigcomsat 1R','Nigcomsat 1R','','Sat',1903,'',null from dual union all
select 6913,'1960-S258','1960-08-30 00:00:00','military missile','success',1419,'J-5','Mk 4-1 RV','','','Test',239,'',1000 from dual union all
select 69130,'2011-078','2011-12-21 13:16:15','orbital','success',1233,'L 15000-039 039','Soyuz TMA-03M','Soyuz 11F732A47 No. 703','','Sat ISS 29S B',480,'',null from dual union all
select 69131,'2011-079','2011-12-22 03:26:14','orbital','success',319,'Y15','ZY-1 02C','ziyuan yihao 02C weixing','','Sat',1494,'',null from dual union all
select 69132,'2011-080','2011-12-28 17:09:02','orbital','success',1226,'B 15000-007 007/1027','Globalstar','Globalstar-2 FM13?','','Sat ST24',562,'',null from dual union all
select 69133,'2011-A01','2011-02-23 06:30:00','military missile','success',1019,'','Arrow 2 Target','','','ATBM Test',1057,'MLP',40 from dual union all
select 69134,'2011-A02','2011-02-23 06:32:00','military missile','success',119,'','','','','ATBM Test',1415,'',40 from dual union all
select 69135,'2011-A03','2011-03-06 04:07:00','military missile','success',11,'04','AAD Interceptor','','','Interceptor',316,'',16 from dual union all
select 69136,'2011-A04','2011-07-21 02:50:00','military missile','success',940,'','Prahaar','','','Test',315,'',35 from dual union all
select 69137,'2011-A05','2011-12-20 08:01:00','military missile','success',3,'','Test','','','Op Test',183,'',40 from dual union all
select 69138,'2011-A06','2011-02-15 00:00:00','suborbital spaceplane','success',1246,'','WK2-49/05GC-CC08','','','WX abort, 1.8hr',955,'WK2',10 from dual union all
select 69139,'2011-A07','2011-04-06 00:00:00','suborbital spaceplane','success',1246,'','WK2-52/CC09','','','Mojave to SFO T2 display',955,'WK2',10 from dual union all
select 6914,'1960-S259','1960-08-31 00:00:00','suborbital rocket','success',42,'TATTLETALE PCC 6','','','','Sigint Test?',394,'',100 from dual union all
select 69140,'2011-A08','2011-04-07 00:00:00','suborbital spaceplane','success',1246,'','WK2-53/CC10','','','SFO T2 to Mojave',1353,'WK2',10 from dual union all
select 69141,'2011-A09','2011-04-14 00:00:00','suborbital spaceplane','success',1246,'','WK2-55/CC11','','','Flt 3.7h, checkouts',955,'WK2',14 from dual union all
select 69142,'2011-A10','2011-04-22 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-56/GF05','','','Glide 14:31',955,'WK2',14 from dual union all
select 69143,'2011-A11','2011-04-27 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-57/GF06','','','Glide 16:07',955,'WK2',14 from dual union all
select 69144,'2011-A12','2011-05-04 14:28:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-58/GF07','','','Glide 11:05',955,'WK2',15 from dual union all
select 69145,'2011-A13','2011-05-10 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-59/GF08','','','Glide 13:02 Feather',955,'WK2',15 from dual union all
select 69146,'2011-A14','2011-05-19 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-60/GF09','','','Glide 11:32',955,'WK2',15 from dual union all
select 69147,'2011-A15','2011-05-25 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-61/GF10','','','Glide 10:14 Feather',955,'WK2',15 from dual union all
select 69148,'2011-A16','2011-06-09 00:00:00','suborbital spaceplane','success',1246,'','WK2-62/CC12','','','Flt 1.4h, abort drop',955,'WK2',15 from dual union all
select 69149,'2011-A17','2011-06-14 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-64/GF11','','','Glide 13:18',955,'WK2',15 from dual union all
select 6915,'1960-S26','1960-01-29 18:51:00','suborbital rocket','success',30,'AA3.125C','EUV monochromator','','','Solar EUV',1869,'',226 from dual union all
select 69150,'2011-A18','2011-06-15 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-65/GF12','','','Glide 10:32',955,'WK2',15 from dual union all
select 69151,'2011-A19','2011-06-21 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-66/GF13','','','Glide  8:55',955,'WK2',15 from dual union all
select 69152,'2011-A20','2011-06-23 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-67/GF14','','','Glide  7:33',955,'WK2',15 from dual union all
select 69153,'2011-A21','2011-06-27 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-68/GF15','','','Glide  7:39',955,'WK2',15 from dual union all
select 69154,'2011-A22','2011-06-13 00:00:00','atmospheric rocket','success',1525,'2','','','','Hypersonic Scramjet',379,'B-52H',16 from dual union all
select 69155,'2011-A23','2011-09-08 13:45:00','suborbital rocket','success',1325,'NASA 12.076GT','Terrier Mk12 Test Flight','','','Test',1809,'',10 from dual union all
select 69156,'2011-A24','2011-09-24 09:00:00','military missile','success',1178,'2','R&D','','','Test',315,'',40 from dual union all
select 69157,'2011-A25','2011-09-27 07:08:00','military missile','failure',1067,'2L','15Zh67 LKI-1','','','R&D',608,'',5 from dual union all
select 69158,'2011-A26','2011-09-07 00:00:00','suborbital rocket','failure',669,'Kavoshgar-5','Kavoshgar Class C','','','Bio',1356,'',1 from dual union all
select 69159,'2011-A27','2011-12-04 18:15:00','suborbital rocket','success',1112,'I','','','','Test',1422,'',43 from dual union all
select 6916,'1960-S260','1960-09-01 00:00:00','military missile','success',979,'','8K65 LKI-6','','','Test',636,'',675 from dual union all
select 69160,'2011-A28','2011-09-29 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-73/GF16','','','Glide  7:15',955,'WK2',15 from dual union all
select 69161,'2011-A29','2011-01-13 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-47/GF04','','','Glide 11:34',955,'WK2',14 from dual union all
select 69162,'2011-A30','2011-02-14 00:00:00','suborbital spaceplane','success',1246,'','WK2-48/05GC-CC07','','','WX abort, 1.4hr',955,'WK2',10 from dual union all
select 69163,'2011-A31','2011-04-27 18:00:00','atmospheric rocket','success',472,'10','','','','Test',330,'',32 from dual union all
select 69164,'2011-A34','2011-06-16 17:30:00','atmospheric rocket','success',472,'11','','','','Test',13,'',32 from dual union all
select 69165,'2011-A35','2011-06-17 17:00:00','atmospheric rocket','success',472,'12','','','','Test',13,'',32 from dual union all
select 69166,'2011-A36','2011-08-31 00:00:00','atmospheric rocket','failure',473,'5','','','','Test',13,'',39 from dual union all
select 69167,'2011-A37','2011-10-20 00:00:00','atmospheric rocket','success',472,'13','','','','Test',330,'',32 from dual union all
select 69168,'2011-A38','2011-10-01 00:00:00','atmospheric rocket','success',120,'F0','Canister eject test','','','Test',1026,'',0 from dual union all
select 69169,'2011-A39','2011-11-15 00:00:00','military missile','success',244,'','Aster 30 Target','','','Test',307,'F-15D',40 from dual union all
select 6917,'1960-S261','1960-09-01 00:00:00','military missile','success',979,'','8K65 LKI-7','','','Test',636,'',675 from dual union all
select 69170,'2011-A40','2011-01-01 00:00:00','military missile','failure',1443,'FCET-44','-    -','','','OT',1897,'SSBN UNK',0 from dual union all
select 69171,'2011-F01','2011-03-04 10:09:43','orbital','failure',1316,'T9 (XL)','Glory','Glory','','Sat',1670,'',550 from dual union all
select 69172,'2011-F02','2011-08-18 09:28:03','orbital','failure',301,'Y26','SJ-11-04','shijian shi yihao 04 xin','','Sat',813,'',150 from dual union all
select 69173,'2011-F03','2011-08-24 13:00:08','orbital','failure',1241,'L 15000-132 132','Progress M-12M','Progress 7K-TGM No.412','','Sat ISS 44P',480,'',190 from dual union all
select 69174,'2011-F04','2011-12-23 12:08:10','orbital','failure',1228,'780-75 -164 006/1042','Meridian No. 15','Meridian No. 15L?','','Sat',475,'',200 from dual union all
select 69175,'2011-M01','2011-03-06 04:02:00','military missile','success',941,'','AAD Target','','','Target',315,'',50 from dual union all
select 69176,'2011-M02','2011-06-09 03:35:00','military missile','success',942,'','Test','','','Test',315,'',50 from dual union all
select 69177,'2011-M03','2011-05-18 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 69178,'2011-M04','2011-03-11 04:33:00','military missile','success',430,'','','','','Test',320,'INS-P51',50 from dual union all
select 69179,'2011-M05','2011-03-11 05:30:00','military missile','success',942,'2','','','','Test',320,'',50 from dual union all
select 6918,'1960-S262','1960-09-01 00:00:00','military missile','success',992,'','','','','Test',811,'',100 from dual union all
select 69180,'2011-M06','2011-05-26 17:00:00','atmospheric rocket','success',473,'3','','','','Test',13,'',56 from dual union all
select 69181,'2011-M07','2011-05-27 17:10:00','atmospheric rocket','success',473,'4','','','','Test',13,'',65 from dual union all
select 69182,'2011-M08','2011-09-02 00:00:00','atmospheric rocket','success',473,'6','','','','Test',13,'',52 from dual union all
select 69183,'2011-S01','2011-01-22 06:10:00','military missile','success',1344,'Aegis Target','ARAV-B','Atlantic Trident','','Target',1809,'',100 from dual union all
select 69184,'2011-S02','2011-01-28 10:46:00','suborbital rocket','success',224,'NASA 36.257UG','FIRE','','','UV Astron',1043,'',240 from dual union all
select 69185,'2011-S03','2011-02-05 08:11:11','suborbital rocket','success',224,'NASA 36.256UE','POLAR NOX','','','Aeron',1043,'',260 from dual union all
select 69186,'2011-S04','2011-02-22 12:56:00','suborbital rocket','success',605,'SSC REXUS 9','EURO REXUS 9','','','Tech',417,'',81 from dual union all
select 69187,'2011-S05','2011-02-23 10:00:00','suborbital rocket','success',605,'SSC REXUS 10','EURO REXUS 10','','','Tech',417,'',82 from dual union all
select 69188,'2011-S06','2011-03-02 13:40:00','military missile','success',632,'2/MBRV-3','PAC-3 Target','','','Target',436,'',100 from dual union all
select 69189,'2011-S07','2011-03-01 21:00:00','military missile','success',1443,'DASO-22','-    (135ss)','','','OT',1898,'SSBN 733',1000 from dual union all
select 6919,'1960-S263','1960-09-02 00:00:00','military missile','success',928,'A1X-45','Sandia warhead','','','Test',244,'',500 from dual union all
select 69190,'2011-S08','2011-03-15 03:00:00','suborbital rocket','success',669,'Kavoshgar-4','Kavoshgar Class C','','','Test',1356,'',120 from dual union all
select 69191,'2011-S09','2011-03-16 00:00:00','military missile','success',1344,'SSTS Target','ARAV-B','','','Target',855,'',150 from dual union all
select 69192,'2011-S10','2011-03-29 06:01:00','suborbital rocket','success',1484,'V15','DLR TEXUS 49','','','Micrograv',416,'',268 from dual union all
select 69193,'2011-S11','2011-03-23 18:50:00','suborbital rocket','success',224,'NASA 36.275UE','SDO EVE Cal','','','Solar EUV',1895,'',280 from dual union all
select 69194,'2011-S12','2011-04-15 06:52:00','military missile','success',695,'','Aegis FTM-15 Target','','','Target',868,'',1000 from dual union all
select 69195,'2011-S13','2011-04-15 07:03:00','military missile','success',1090,'Aegis FTM-15','Stellar Charon','','','Test',854,'DDG-77',160 from dual union all
select 69196,'2011-S14','2011-04-26 00:00:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-84',1000 from dual union all
select 69197,'2011-S15','2011-04-27 08:00:00','suborbital rocket','success',224,'NASA 36.278GT','SRPO DIagnostic','','','Test',1043,'',339 from dual union all
select 69198,'2011-S16','2011-05-06 23:02:00','suborbital rocket','success',1397,'Kunpeng-1','','','','Ionos',647,'',197 from dual union all
select 69199,'2011-S17','2011-05-20 14:50:00','military missile','success',704,'VMF RF/LAYNER','','','','R&D',201,'K-84',1000 from dual union all
select 692,'1947-A28','1947-10-31 13:41:00','military missile','failure',8,'06T','','','','Test',636,'',0 from dual union all
select 6920,'1960-S264','1960-09-03 14:08:00','suborbital rocket','success',839,'NASA 10.19GE','SBE 3 flare part.','','','Magnetospheric',428,'',122 from dual union all
select 69200,'2011-S18','2011-05-20 13:21:00','suborbital rocket','success',1244,'SpaceLoft SL-5','','','','Edu/Burial',1422,'',118 from dual union all
select 69201,'2011-S19','2011-06-10 11:16:00','suborbital rocket','success',1334,'NASA 41.096GT','SUBTEC IV/SMART','','','Tech',1809,'',118 from dual union all
select 69202,'2011-S20','2011-02-01 00:00:00','military missile','success',1157,'R&D','Sejjil-2','','','Test',1348,'',800 from dual union all
select 69203,'2011-S21','2011-02-01 00:00:00','military missile','success',1168,'','','','','Test',1348,'',150 from dual union all
select 69204,'2011-S22','2011-06-23 10:17:59','suborbital rocket','success',1338,'NASA 41.095UO','RockOn IV/Rocksat-C','','','Tech',1798,'',119 from dual union all
select 69205,'2011-S23','2011-06-22 13:35:00','military missile','success',799,'','Mk 12/21 RV','GT204GM','GLORY TRIP 204GM','Op Test',1706,'',1300 from dual union all
select 69206,'2011-S24','2011-06-28 00:00:00','military missile','success',1166,'','','','','Test',1356,'',100 from dual union all
select 69207,'2011-S25','2011-06-28 00:00:00','military missile','success',1167,'','','','','Test',1356,'',125 from dual union all
select 69208,'2011-S26','2011-06-28 00:00:00','military missile','success',508,'Ghadr-F','','','','Test',1348,'',150 from dual union all
select 69209,'2011-S27','2011-06-28 11:55:00','military missile','success',261,'R-30 15','','','','R&D?',190,'K-535',1000 from dual union all
select 6921,'1960-S265','1960-09-03 17:29:00','suborbital rocket','success',839,'NASA 10.20GE','SBE 4','','','Magnetospheric',428,'',122 from dual union all
select 69210,'2011-S28','2011-07-10 14:00:00','suborbital rocket','success',228,'NASA 21.141GE','Daytime Dynamo','','','Ionos',1804,'',158 from dual union all
select 69211,'2011-S29','2011-07-10 14:00:15','suborbital rocket','success',1334,'NASA 41.091GE','Daytime Dynamo','','','Ionos',1809,'',158 from dual union all
select 69212,'2011-S30','2011-07-11 15:35:00','suborbital rocket','success',515,'','','','','Test',310,'',100 from dual union all
select 69213,'2011-S31','2011-07-09 02:04:00','military missile','success',1098,'','FTX-17','','','Target',1057,'C-17',200 from dual union all
select 69214,'2011-S32','2011-07-21 11:58:00','suborbital rocket','success',1334,'NASA 41.092GO','RockSat-X 1','','','Edu/Tech',1797,'',121 from dual union all
select 69215,'2011-S33','2011-07-27 10:01:00','military missile','failure',799,'','Mk 12/21 RV','GT205GM','GLORY TRIP 205GM','Op Test',1699,'',500 from dual union all
select 69216,'2011-S34','2011-07-27 00:00:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-84',1000 from dual union all
select 69217,'2011-S35','2011-03-09 00:00:00','military missile','success',1303,'Aegis FTM-16/SSTS','ARAV-C++','','','Target',855,'',150 from dual union all
select 69218,'2011-S36','2011-08-11 14:45:00','military missile','success',789,'4','HTV-2b','','','Test',1781,'',100 from dual;
insert into LAUNCH
select 69219,'2011-S37','2011-08-27 03:20:00','military missile','success',261,'R-30 16','','','','R&D?',190,'K-535',1000 from dual union all
select 6922,'1960-S266','1960-09-05 00:00:00','military missile','success',976,'','8K63 UBP-73','','','Test',636,'',402 from dual union all
select 69220,'2011-S38','2011-09-01 13:53:00','military missile','success',1344,'Aegis Target','ARAV-B','','','Target',855,'',150 from dual union all
select 69221,'2011-S39','2011-09-01 13:54:00','military missile','success',1091,'Aegis FTM-16 E2','','','','Interceptor',851,'CG-70',150 from dual union all
select 69222,'2011-S40','2011-09-03 09:46:00','military missile','success',1434,'146','15Zh58 UBP-37','','','R&D',608,'',1000 from dual union all
select 69223,'2011-S41','2011-07-21 07:01:00','suborbital rocket','success',847,'','SSC PHOCUS','','','Aeron',412,'',107 from dual union all
select 69224,'2011-S42','2011-09-26 03:20:00','military missile','success',942,'','Test','','','Target',315,'',100 from dual union all
select 69225,'2011-S43','2011-09-29 00:00:00','military missile','success',704,'VMF RF','','','','R&D',201,'K-114',1000 from dual union all
select 69226,'2011-S44','2011-09-30 04:00:00','military missile','success',63,'','Agni RV','','','Test',316,'',220 from dual union all
select 69227,'2011-S45','2011-10-05 05:56:00','military missile','success',1098,'THAAD FTT-12 Target','','','','Target',855,'C-17',100 from dual union all
select 69228,'2011-S46','2011-10-05 05:56:00','military missile','success',1151,'THAAD FTT-12 Target','','','','Target',855,'MLP',100 from dual union all
select 69229,'2011-S47','2011-10-05 06:00:00','military missile','success',1294,'THAAD FTT-12','','','','ABM Test',855,'',100 from dual union all
select 6923,'1960-S267','1960-09-10 00:00:00','military missile','success',974,'','','','','Test',190,'B-67',86 from dual union all
select 69230,'2011-S48','2011-10-05 06:00:00','military missile','success',1294,'THAAD FTT-12','','','','ABM Test',855,'',100 from dual union all
select 69231,'2011-S49','2011-10-11 21:15:00','suborbital rocket','success',1334,'NASA 41.094UE','CHAMPS','','','Meteor dust',23,'',130 from dual union all
select 69232,'2011-S50','2011-10-13 13:50:00','suborbital rocket','success',1334,'NASA 41.093UE','CHAMPS','','','Meteor dust',28,'',130 from dual union all
select 69233,'2011-S51','2011-11-02 07:50:00','military missile','success',628,'','','','','Test',1026,'',300 from dual union all
select 69234,'2011-S52','2011-10-28 03:40:00','military missile','success',261,'','','','','R&D?',190,'K-535',1000 from dual union all
select 69235,'2011-S53','2011-11-03 06:45:00','military missile','success',1434,'3543511241','15Zh58 UBP-38','','','OT',608,'',1000 from dual union all
select 69236,'2011-S54','2011-11-06 07:00:00','suborbital rocket','success',224,'NASA 36.264UH','XQC F5','','','XR Astron',1895,'',250 from dual union all
select 69237,'2011-S55','2011-11-15 03:30:00','military missile','success',66,'Agni II Prime','Agni RV','','','Test',316,'',900 from dual union all
select 69238,'2011-S56','2011-10-08 10:25:01','suborbital rocket','success',224,'NASA 36.225UG','PICTURE','','','Opt Astron',1895,'',250 from dual union all
select 69239,'2011-S57','2011-11-17 11:30:00','military missile','success',1111,'AHW FT1A','AHW/HGB','','','Hypersonic',849,'',1000 from dual union all
select 6924,'1960-S268','1960-09-12 20:38:38','military missile','failure',154,'47D','Mk 3-3','IOC DASO','GOLDEN JOURNEY','Test',1667,'',1000 from dual union all
select 69240,'2011-S58','2011-06-28 00:00:00','military missile','success',1166,'','','','','Test',1356,'',100 from dual union all
select 69241,'2011-S59','2011-06-28 00:00:00','military missile','success',1167,'','','','','Test',1356,'',125 from dual union all
select 69242,'2011-S60','2011-11-27 09:10:00','suborbital rocket','success',1484,'V14','DLR TEXUS 48','','','Micrograv',416,'',258 from dual union all
select 69243,'2011-S61','2011-05-11 18:00:00','suborbital rocket','success',605,'AEB V04','Camurupim','','','Tech',330,'',100 from dual union all
select 69244,'2011-S62','2011-11-25 23:00:00','suborbital rocket','success',605,'AEB V05','Brasil-Alemanha','','','Tech',330,'',100 from dual union all
select 69245,'2011-S63','2011-12-02 22:00:00','suborbital rocket','success',1478,'V08','Brasil-Alemanha','','','Ionos/Test',330,'',173 from dual union all
select 69246,'2011-S64','2011-12-03 07:21:31','suborbital rocket','success',1479,'V08','ICI-3','','','Auroral',1453,'',354 from dual union all
select 69247,'2011-S65','2011-12-19 00:00:00','suborbital rocket','success',743,'','R&D','','','Test',636,'',150 from dual union all
select 69248,'2011-S66','2011-12-23 15:50:00','military missile','success',261,'','','','','R&D?',190,'K-535',1000 from dual union all
select 69249,'2011-S67','2011-12-23 15:50:00','military missile','success',261,'','','','','R&D?',190,'K-535',1000 from dual union all
select 6925,'1960-S269','1960-09-15 21:28:00','military missile','success',928,'A1XE-6 (A1FP-1)','','','','Test',421,'SSBN 599',500 from dual union all
select 69250,'2011-S68','2011-12-27 12:00:00','military missile','success',1454,'511-6552-249','15A35P LKI-7','Yu-70/102E Proekt 4202','','OT',508,'',1000 from dual union all
select 69251,'2011-S69','2011-12-20 08:48:00','suborbital rocket','success',1073,'S-310-40','','','','Aeron',833,'',180 from dual union all
select 69252,'2011-S70','2011-12-10 10:30:00','suborbital rocket','success',224,'NASA.36.274UH','CODEX','','','XR Astron',1895,'',250 from dual union all
select 69253,'2011-S71','2011-01-01 00:00:00','military missile','success',1443,'FCET-44','-    (136s)','','','OT',1897,'SSBN UNK',1000 from dual union all
select 69254,'2011-S73','2011-12-01 04:00:00','military missile','success',62,'','Agni RV','','','Test',316,'',300 from dual union all
select 69255,'2011-S74','2011-12-15 00:00:00','military missile','success',1344,'Aegis Target','FTX-16 Event 3','','','Target',855,'',100 from dual union all
select 69256,'2012-001','2012-01-09 03:17:10','orbital','success',319,'Y26','ZY-3','ziyuan san hao','','Sat',1494,'',null from dual union all
select 69257,'2012-002','2012-01-13 00:56:04','orbital','success',309,'Y22','FY-2F','fengyun erhao 07 xing','','Sat',1904,'',null from dual union all
select 69258,'2012-003','2012-01-20 00:38:00','orbital','success',404,'D4-18 (358)','WGS 4','Wideband Global Satcom 4','','Sat',290,'',null from dual union all
select 69259,'2012-004','2012-01-25 23:06:40','orbital','success',1241,'I 15000-127 127','Progress M-14M','Progress 7K-TGM No.414','','Sat ISS 46P',480,'',null from dual union all
select 6926,'1960-S27','1960-02-01 00:00:00','suborbital rocket','success',814,'','','','','Test',671,'',90 from dual union all
select 69260,'2012-005','2012-02-03 00:04:00','orbital','success',1114,'ERS.2002','Navid','Navid','','Sat',1347,'',null from dual union all
select 69261,'2012-006','2012-02-13 10:00:00','orbital','success',1488,'VV01','LARES/AlmaSat','LARES','','Sat',349,'',null from dual union all
select 69262,'2012-007','2012-02-14 19:36:37','orbital','success',962,'935-24/99526','SES-4','SES-4','','Sat',553,'',null from dual union all
select 69263,'2012-008','2012-02-24 16:12:04','orbital','success',316,'Y6','Beidou DW11','di shiyi ke beidou d.w.','','Sat',1903,'',null from dual union all
select 69264,'2012-009','2012-02-24 22:15:00','orbital','success',196,'AV-030','MUOS 1','MUOS 1','','Sat',292,'',null from dual union all
select 69265,'2012-010','2012-03-23 04:34:12','orbital','success',113,'VA205 (553)','ATV 3','ATV 3','','Sat',346,'',null from dual union all
select 69266,'2012-011','2012-03-25 12:10:32','orbital','success',962,'935-28/99537','Intelsat IS-22','Intelsat IS-22','','Sat',553,'',null from dual union all
select 69267,'2012-012','2012-03-30 05:49:32','orbital','success',958,'410-18/117L','Kosmos-2479','Oko-1 No. 7128','','Sat',588,'',null from dual union all
select 69268,'2012-013','2012-03-31 10:27:04','orbital','success',311,'Y22','Apstar 7','Apstar7','','Sat',1903,'',null from dual union all
select 69269,'2012-014','2012-04-03 23:12:57','orbital','success',402,'D4-19 (359)','USA 234','TOPAZ 2?','','Sat',1780,'',null from dual union all
select 6927,'1960-S270','1960-09-15 00:00:00','military missile','success',976,'','8K63 UBP-74','','','Test',636,'',402 from dual union all
select 69270,'2012-015','2012-04-20 12:50:24','orbital','success',1241,'L 15000-135 135','Progress M-15M','Progress 7K-TGM No.415','','Sat ISS 47P',562,'',null from dual union all
select 69271,'2012-016','2012-04-23 22:18:13','orbital','success',962,'935-27/99529','Yahsat-1B','Yahsat 1B','','Sat',553,'',null from dual union all
select 69272,'2012-017','2012-04-26 00:17:00','orbital','success',902,'PSLV-C19','RISAT-1','RISAT-1','','Sat',1357,'',null from dual union all
select 69273,'2012-018','2012-04-29 20:50:03','orbital','success',310,'Y14','Beidou DW12/13','di shier ke beidou-2 w.','','Sat',1903,'',null from dual union all
select 69274,'2012-019','2012-05-04 18:42:00','orbital','success',192,'AV-031','AEHF 2','AEHF SV-2','','Sat',292,'',null from dual union all
select 69275,'2012-020','2012-05-06 07:10:05','orbital','success',303,'Y17','Tianhui 1-02','tianhui yi hao 02 xing','','Sat',813,'',null from dual union all
select 69276,'2012-021','2012-05-10 07:06:04','orbital','success',319,'Y12','Yaogan 14','yaogan weixing shisi hao','','Sat',1493,'',null from dual union all
select 69277,'2012-022','2012-05-15 03:01:23','orbital','success',1233,'L 15000-041 041','Soyuz TMA-04M','Soyuz 11F732A47 No. 705','','Sat ISS 30S A',480,'',null from dual union all
select 69278,'2012-023','2012-05-15 22:13:00','orbital','success',111,'VA206 (562)','JCSAT 13/Vinasat 2','JCSAT 13','','Sat',346,'',null from dual union all
select 69279,'2012-024','2012-05-17 14:05:00','orbital','success',1241,'780-31 -229 131','Kosmos-2480','Kobal''t-M','','Sat',462,'',null from dual union all
select 6928,'1960-S271','1960-09-15 14:01:00','suborbital rocket','success',29,'AA1.126','Solar Constant','','','Solar UV',1869,'',102 from dual union all
select 69280,'2012-025','2012-05-17 16:39:00','orbital','success',527,'H-IIA-21','GCOM-W1/KOMPSAT-3','GCOM-W1','','Sat',1477,'',null from dual union all
select 69281,'2012-026','2012-05-17 19:12:14','orbital','success',962,'935-29/99530','Nimiq 6','Nimiq 6','','Sat',588,'',null from dual union all
select 69282,'2012-027','2012-05-22 07:44:38','orbital','success',482,'003','Dragon C2','Dragon C102','','Sat',274,'',null from dual union all
select 69283,'2012-028','2012-05-26 15:56:04','orbital','success',312,'Y17','Zhongxing 2A','Shentong-2?','','Sat',1903,'',null from dual union all
select 69284,'2012-029','2012-05-29 07:31:05','orbital','success',320,'Y10','Yaogan 15','yaogan weixing shiwuhao','','Sat',1493,'',null from dual union all
select 69285,'2012-030','2012-06-01 05:22:59','orbital','success',1542,'SL33/DM SL-32L','SL45 Intelsat IS-19','Intelsat IS-19','','Sat',860,'ODYSSEY',null from dual union all
select 69286,'2012-031','2012-06-13 16:00:42','orbital','success',911,'F41','NuSTAR','NuSTAR','','Sat',872,'L-1011',null from dual union all
select 69287,'2012-032','2012-06-16 10:37:25','orbital','success',306,'Y9','Shenzhou 9','shenzhou jiu fanhui cang','','Sat',814,'',null from dual union all
select 69288,'2012-033','2012-06-20 12:28:00','orbital','success',182,'AV-023','USA 236','QUASAR 17?','','Sat',292,'',null from dual union all
select 69289,'2012-034','2012-06-29 13:15:00','orbital','success',396,'D4-20 (360)','USA 237      [RH12]','ORION 8','','Sat',290,'',null from dual union all
select 6929,'1960-S272','1960-09-16 00:00:00','suborbital rocket','success',994,'','','','','Ionos/Aeron/Bio',636,'',210 from dual union all
select 69290,'2012-035','2012-07-05 21:36:00','orbital','success',111,'VA207 (563)','Echostar 17/MSG 3','Jupiter 1','','Sat',346,'',null from dual union all
select 69291,'2012-036','2012-07-09 18:38:30','orbital','success',962,'935-30/99531','SES-5','Astra 4B ?','','Sat',588,'',null from dual union all
select 69292,'2012-037','2012-07-15 02:40:03','orbital','success',1233,'L 15000-042 042','Soyuz TMA-05M','Soyuz 11F732A47 No. 706','','Sat ISS 31S B',480,'',null from dual union all
select 69293,'2012-038','2012-07-21 02:06:18','orbital','success',534,'H-IIB-F3','HTV-3','HTV 3','','Sat',1478,'',null from dual union all
select 69294,'2012-039','2012-07-22 06:41:39','orbital','success',1232,'B 15000-033 033','Kanopus/BelKA','Kanopus-V','','Sat',562,'',null from dual union all
select 69295,'2012-040','2012-07-25 15:43:04','orbital','success',316,'Y9','Tian Lian-1-03','tianlian yihao 03 xing','','Sat',1903,'',null from dual union all
select 69296,'2012-041','2012-07-28 01:35:34','orbital','success',1064,'492-6391-835/72515','Gonets-M/Kosmos','Strela-3M','','Sat',460,'',null from dual union all
select 69297,'2012-042','2012-08-01 19:35:13','orbital','success',1241,'L 15000-134 134','Progress M-16M','Progress 7K-TGM No.416','','Sat ISS 48P',480,'',null from dual union all
select 69298,'2012-043','2012-08-02 20:54:00','orbital','success',111,'VA208 (564)','IS-20/Hylas 2','Intelsat IS-20','','Sat',346,'',null from dual union all
select 69299,'2012-044','2012-08-06 19:31:00','orbital','failure',962,'935-31/99532','Telkom-3/EkspressMD2','Telkom-3','','Sat',588,'',null from dual union all
select 693,'1947-A29','1947-11-03 12:05:00','military missile','failure',8,'30N','','','','Test',636,'',0 from dual union all
select 6930,'1960-S273','1960-09-17 00:00:00','military missile','success',976,'','8K63 UBP-75','','','Test',636,'',402 from dual union all
select 69300,'2012-045','2012-08-19 06:54:59','orbital','success',1542,'SL34/DM SL-33L','SL49 Intelsat IS-21','Intelsat IS-21','','Sat',860,'ODYSSEY',null from dual union all
select 69301,'2012-046','2012-08-30 08:05:27','orbital','success',182,'AV-032','RBSP A/B','RBSP-A','','Sat',292,'',null from dual union all
select 69302,'2012-047','2012-09-09 04:23:00','orbital','success',900,'PSLV-C21','SPOT-6','SPOT 6','','Sat',1357,'',null from dual union all
select 69303,'2012-048','2012-09-13 21:39:00','orbital','success',182,'AV-033','USA 238/OUTSAT','INTRUDER 10','','Sat',1775,'',null from dual union all
select 69304,'2012-049','2012-09-17 16:28:40','orbital','success',1226,'L 15000-012 012/1037','Metop B','METOP B','','Sat ST25',562,'',null from dual union all
select 69305,'2012-050','2012-09-18 19:10:04','orbital','success',310,'Y15','Beidou DW14/15','14 ke beidou daohang wei','','Sat',1903,'',null from dual union all
select 69306,'2012-051','2012-09-28 21:18:07','orbital','success',111,'VA209 (565)','Astra 2F/GSAT-10','Astra 2F','','Sat',346,'',null from dual union all
select 69307,'2012-052','2012-09-29 04:12:05','orbital','success',303,'Y16','VRSS-1','VRSS-1','','Sat',813,'',null from dual union all
select 69308,'2012-053','2012-10-04 12:10:00','orbital','success',400,'D4-21 (361)','GPS IIF SV-3','GPS SVN 65','','Sat',290,'',null from dual;
insert into LAUNCH
select 69309,'2012-054','2012-10-08 00:35:07','orbital','success',482,'004','Dragon CRS-1','Dragon C103','','Sat',274,'',null from dual union all
select 6931,'1960-S274','1960-09-16 09:58:06','military missile','success',154,'76D','RVX-2A S/N 422','D/AIG guidance test CL4','','CR/Tech',226,'',943 from dual union all
select 69310,'2012-055','2012-10-12 18:15:00','orbital','success',1236,'B 15000-002 002/1031','Galileo IOV-3/4','Galileo FM3','','Sat VS03/SZ09',347,'',null from dual union all
select 69311,'2012-056','2012-10-14 03:25:05','orbital','success',298,'Y2','SJ-9A/B','shi jian liuhao A weixin','','Sat',1494,'',null from dual union all
select 69312,'2012-057','2012-10-14 08:37:00','orbital','success',962,'935-26/99534','Intelsat IS-23','IS-23','','Sat',588,'',null from dual union all
select 69313,'2012-058','2012-10-23 10:51:11','orbital','success',1233,'L 15000-044 044','Soyuz TMA-06M','Soyuz 11F732A47 No. 707','','Sat ISS 32S A',562,'',null from dual union all
select 69314,'2012-059','2012-10-25 15:33:04','orbital','success',316,'Y10','Beidou DW16','di 16 ke beidou d.weixin','','Sat',1903,'',null from dual union all
select 69315,'2012-060','2012-10-31 07:41:18','orbital','success',1241,'L 15000-136 136','Progress M-17M','Progress 7K-TGM No. 417','','Sat ISS 49P',480,'',null from dual union all
select 69316,'2012-061','2012-11-02 21:04:00','orbital','success',962,'935-32/99533','Luch-5B/Yamal-300K','Luch-5B','','Sat',588,'',null from dual union all
select 69317,'2012-062','2012-11-10 21:05:07','orbital','success',111,'VA210 (566)','Star One C3/Eut21B','Star One C3','','Sat',346,'',null from dual union all
select 69318,'2012-063','2012-11-14 11:42:46','orbital','success',1226,'770-46 -267 016/1034','Meridian No. 16L','Meridian No. 16L','','Sat',475,'',null from dual union all
select 69319,'2012-064','2012-11-18 22:53:05','orbital','success',301,'Y17','HJ-1C','huan jing yihao C weixin','','Sat',1493,'',null from dual union all
select 6932,'1960-S275','1960-09-19 15:35:00','sounding rocket','success',630,'NASA 11.01GE','NERV I','','','Cosmic rays/Bio',1015,'',1884 from dual union all
select 69320,'2012-065','2012-11-20 18:31:00','orbital','success',962,'935-33/99528','Echostar 16','Echostar 16','','Sat',553,'',null from dual union all
select 69321,'2012-066','2012-11-25 04:06:04','orbital','success',320,'Y9','Yaogan 16','yaogan weixing shiliu ha','','Sat',813,'',null from dual union all
select 69322,'2012-067','2012-11-27 10:13:03','orbital','success',311,'Y24','Zhongxing 12','zhongxing shier hao weix','','Sat',1903,'',null from dual union all
select 69323,'2012-068','2012-12-02 02:02:50','orbital','success',1234,'Sh15000-001 001/1020','Pleiades 1B','Pleiades HR-2','','Sat VS04',347,'',null from dual union all
select 69324,'2012-069','2012-12-03 20:43:59','orbital','success',1542,'SL35/DM SL-34L','SL50 Eutelsat 70B','EUTELSAT 70B','','Sat',860,'ODYSSEY',null from dual union all
select 69325,'2012-070','2012-12-08 13:13:43','orbital','failure',962,'935-34/99535','Yamal 402','Yamal 402','','Sat',553,'',null from dual union all
select 69326,'2012-071','2012-12-11 18:03:00','orbital','success',190,'AV-034','X-37B OTV FLT-3','X-37B No. 1','','Sat',292,'',null from dual union all
select 69327,'2012-072','2012-12-12 00:49:46','orbital','success',1463,'4','Kwangmyongsong-3/F2','KMS-3 Unit 2','','Sat',1416,'',120 from dual union all
select 69328,'2012-073','2012-12-18 16:13:05','orbital','success',304,'Y22','Gokturk-2','GK-2','','Sat',813,'',null from dual union all
select 69329,'2012-074','2012-12-19 12:12:35','orbital','success',1233,'L 15000-040 040','Soyuz TMA-07M','Soyuz 11F732A47 No. 704A','','Sat ISS 33S',480,'',null from dual union all
select 6933,'1960-S276','1960-09-19 18:31:27','military missile','success',154,'79D','Mk 3-1B-226/LMSC spec.','D/RIG Long range test','','Test',229,'',1691 from dual union all
select 69330,'2012-075','2012-12-19 21:49:07','orbital','success',111,'VA211 (567)','Skynet 5D/Mexsat-B','Skynet 5D','','Sat',346,'',null from dual union all
select 69331,'2012-A01','2012-02-10 04:44:00','military missile','success',11,'05','AAD Interceptor','','','Interceptor',316,'',15 from dual union all
select 69332,'2012-A02','2012-05-01 00:00:00','atmospheric rocket','success',1348,'','HiFire 2','','','Hypersonic',855,'',43 from dual union all
select 69333,'2012-A03','2012-06-26 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-87/GF17','','','Glide 11:22',955,'WK2',15 from dual union all
select 69334,'2012-A04','2012-06-29 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-88/GF18','','','Glide 13:00',955,'WK2',15 from dual union all
select 69335,'2012-A05','2012-07-18 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-90/GF19','','','Glide 10:39',955,'WK2',15 from dual union all
select 69336,'2012-A06','2012-08-02 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-91/GF20','','','Glide  8:00',955,'WK2',15 from dual union all
select 69337,'2012-A07','2012-08-14 18:35:00','atmospheric rocket','failure',1525,'3','','','','Hypersonic Scramjet',379,'B-52H',16 from dual union all
select 69338,'2012-A08','2012-08-07 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-92/GF21','','','Glide  9:52',955,'WK2',15 from dual union all
select 69339,'2012-A09','2012-08-11 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-93/GF22','','','Glide  8:02',955,'WK2',15 from dual union all
select 6934,'1960-S277','1960-09-20 12:55:00','suborbital rocket','success',30,'AA3.142','Thermal/CR','','','Aeron',1869,'',195 from dual union all
select 69340,'2012-A10','2012-09-21 00:00:00','atmospheric rocket','success',517,'','','','','Test',1428,'',0 from dual union all
select 69341,'2012-A11','2012-03-14 18:39:00','atmospheric rocket','success',472,'14','','','','Test',13,'',32 from dual union all
select 69342,'2012-A12','2012-03-29 00:00:00','atmospheric rocket','success',472,'15','','','','Test',330,'',32 from dual union all
select 69343,'2012-A13','2012-05-21 17:00:00','atmospheric rocket','success',472,'16','','','','Test',13,'',32 from dual union all
select 69344,'2012-A14','2012-05-29 16:50:00','atmospheric rocket','success',472,'17','','','','Test',13,'',32 from dual union all
select 69345,'2012-A15','2012-06-21 18:00:00','atmospheric rocket','success',472,'18','','','','Test',330,'',32 from dual union all
select 69346,'2012-A16','2012-08-08 19:59:00','atmospheric rocket','success',472,'19','','','','Test',13,'',32 from dual union all
select 69347,'2012-A17','2012-09-19 00:00:00','atmospheric rocket','success',472,'20','','','','Test',330,'',32 from dual union all
select 69348,'2012-A18','2012-09-27 17:11:00','atmospheric rocket','success',472,'21','','','','Test',13,'',32 from dual union all
select 69349,'2012-A19','2012-10-16 08:01:00','military missile','success',3,'','Test','','','Op Test',183,'',40 from dual union all
select 6935,'1960-S278','1960-09-21 00:00:00','suborbital rocket','success',972,'','','','','Aeron?',636,'',210 from dual union all
select 69350,'2012-A20','2012-04-23 09:00:00','suborbital rocket','failure',1479,'V06','HiFire-5','','','Hypersonic',29,'',10 from dual union all
select 69351,'2012-A21','2012-10-31 17:03:00','atmospheric rocket','success',472,'22','','','','Test',13,'',32 from dual union all
select 69352,'2012-A22','2012-11-01 00:00:00','atmospheric rocket','success',517,'','','','','Test',1428,'',0 from dual union all
select 69353,'2012-A23','2012-11-07 17:00:00','atmospheric rocket','success',472,'23','','','','Test',13,'',32 from dual union all
select 69354,'2012-A24','2012-11-23 07:25:00','military missile','success',11,'','AAD Interceptor','','','Interceptor',316,'',15 from dual union all
select 69355,'2012-A25','2012-12-17 00:00:00','atmospheric rocket','success',517,'','','','','Test',1428,'',0 from dual union all
select 69356,'2012-A26','2012-12-19 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-109/GF23','','','Glide  13:24',955,'WK2',15 from dual union all
select 69357,'2012-A27','2012-10-06 00:00:00','suborbital rocket','failure',1113,'1','','','','Test',1422,'',1 from dual union all
select 69358,'2012-A28','2012-11-03 16:26:00','suborbital rocket','failure',1113,'2','','','','Test',1422,'',1 from dual union all
select 69359,'2012-A29','2012-09-08 04:30:00','suborbital rocket','failure',669,'Kavoshgar-6','Kavoshgar Class C','','','Bio',1356,'',1 from dual union all
select 6936,'1960-S279','1960-09-22 00:00:00','military missile','success',976,'','8K63 UBP-76','','','Test',636,'',402 from dual union all
select 69360,'2012-F01','2012-04-12 22:38:55','orbital','failure',1463,'3','Kwangmyongsong-3','','','Sat',1416,'',120 from dual union all
select 69361,'2012-F02','2012-05-23 00:00:00','orbital','failure',1114,'','Fajr','Fajr','','Sat',1347,'',40 from dual union all
select 69362,'2012-F03','2012-09-22 00:00:00','orbital','failure',1114,'','Fajr','Fajr','','Sat',1347,'',0 from dual union all
select 69363,'2012-M01','2012-11-16 10:45:00','suborbital rocket','success',605,'SSC REXUS 11','EURO REXUS 11','','','Tech',417,'',79 from dual union all
select 69364,'2012-M02','2012-11-23 07:22:00','military missile','success',941,'','AAD Target','','','Target',315,'',50 from dual union all
select 69365,'2012-M03','2012-11-29 17:52:00','atmospheric rocket','success',473,'8','','','','Test',13,'',52 from dual union all
select 69366,'2012-M04','2012-07-01 00:00:00','military missile','success',489,'','R&D','','','Test',759,'',60 from dual union all
select 69367,'2012-M05','2012-12-25 00:00:00','military missile','success',489,'','','','','Weapon',14,'',60 from dual union all
select 69368,'2012-M06','2012-11-13 19:00:00','atmospheric rocket','success',473,'7','','','','Test',13,'',53 from dual union all
select 69369,'2012-S01','2012-01-11 13:25:00','suborbital rocket','success',1332,'NASA 12.074GT','','','','Test',1821,'',261 from dual union all
select 6937,'1960-S28','1960-01-31 16:17:00','military missile','success',1017,'I1-4','8K74 LKI-4','','','Test',480,'',1000 from dual union all
select 69370,'2012-S02','2012-01-12 14:51:00','suborbital rocket','success',1074,'S-520-26','Li release','','','Aeron',833,'',298 from dual union all
select 69371,'2012-S03','2012-01-28 18:15:00','suborbital rocket','success',1112,'III','','','','Test',1422,'',82 from dual union all
select 69372,'2012-S04','2012-02-10 04:40:00','military missile','success',941,'','AAD Target','','','Target',315,'',100 from dual union all
select 69373,'2012-S05','2012-02-13 09:32:00','suborbital rocket','success',1484,'V16','SSC MASER 12','','','Micrograv',416,'',260 from dual union all
select 69374,'2012-S06','2012-02-19 05:41:07','suborbital rocket','success',224,'NASA 36.273UE','MICA','','','Auroral',1043,'',325 from dual union all
select 69375,'2012-S07','2012-02-25 10:46:00','military missile','success',799,'','Mk 12/21 RV','GT203GM','GLORY TRIP 203GM','Op Test',1705,'',1300 from dual union all
select 69376,'2012-S08','2012-02-22 00:00:00','military missile','success',1443,'DASO-23','-    (137s)','','','OT',421,'SSBN 734',1000 from dual union all
select 69377,'2012-S09','2012-03-19 14:05:00','suborbital rocket','success',605,'SSC REXUS 12','EURO REXUS 12','','','Tech',417,'',82 from dual union all
select 69378,'2012-S10','2012-03-22 09:00:00','suborbital rocket','success',224,'NASA 36.277UG','CIBER-I F3','','','IR Astron',1895,'',320 from dual union all
select 69379,'2012-S11','2012-03-27 08:58:00','suborbital rocket','success',1340,'NASA 45.004UE','ATREX 1','','','Aeron',1804,'',375 from dual union all
select 6938,'1960-S280','1960-09-22 06:32:00','suborbital rocket','success',665,'K-8-3','ID-2/CR-3','','','Ionos',7,'',200 from dual union all
select 69380,'2012-S12','2012-03-27 08:59:20','suborbital rocket','success',1331,'NASA 46.002UE','ATREX 2','','','Aeron',1809,'',200 from dual union all
select 69381,'2012-S13','2012-03-27 09:00:40','suborbital rocket','success',1338,'NASA 41.097UE','ATREX 3','','','Aeron',1805,'',157 from dual union all
select 69382,'2012-S14','2012-03-27 09:02:00','suborbital rocket','success',1332,'NASA 46.003UE','ATREX 4','','','Aeron',1798,'',254 from dual union all
select 69383,'2012-S15','2012-03-27 09:03:20','suborbital rocket','success',1339,'NASA 41.098UE','ATREX 5','','','Aeron',1805,'',169 from dual union all
select 69384,'2012-S16','2012-04-05 14:18:00','suborbital rocket','success',1244,'SpaceLoft SL-6','','','','Tech',1422,'',117 from dual union all
select 69385,'2012-S17','2012-03-29 00:00:00','military missile','success',1019,'PAC target','','','','Target',1895,'',80 from dual union all
select 69386,'2012-S18','2012-04-19 02:37:00','military missile','success',67,'','','','','Test',316,'',800 from dual union all
select 69387,'2012-S19','2012-04-25 00:00:00','military missile','success',1170,'Hatf IV','','','','Test',1419,'',100 from dual union all
select 69388,'2012-S20','2012-05-10 06:18:00','military missile','success',1334,'Aegis FTM16E2a Targ','ARAV-A','','','Target',855,'',100 from dual union all
select 69389,'2012-S21','2012-05-10 06:20:00','military missile','success',1091,'Aegis FTM-16 E2A','Stellar Vengeance','','','Interceptor',851,'CG-70',100 from dual union all
select 6939,'1960-S281','1960-09-22 20:56:00','suborbital rocket','success',994,'','','','','Ionos/Aeron/Bio?',636,'',210 from dual union all
select 69390,'2012-S22','2012-05-23 06:15:00','military missile','success',1067,'2TL','15Zh67 LKI-2','Avangard? RS-26','','R&D',608,'',1000 from dual union all
select 69391,'2012-S23','2012-03-01 00:00:00','military missile','success',1443,'FCET-45','-    (138s)','','','OT',1897,'SSBN UNK',1000 from dual union all
select 69392,'2012-S24','2012-04-14 00:00:00','military missile','success',1443,'FCET-46','-    (139s)','','','OT',421,'SSBN 738',1000 from dual union all
select 69393,'2012-S25','2012-04-14 00:00:00','military missile','success',1443,'FCET-46','-    (140s)','','','OT',421,'SSBN 738',1000 from dual union all
select 69394,'2012-S26','2012-04-16 00:00:00','military missile','success',1443,'FCET-47','-    (141s)','','','OT',421,'SSBN 738',1000 from dual union all
select 69395,'2012-S27','2012-04-16 00:00:00','military missile','success',1443,'FCET-47','-    (142s)','','','OT',421,'SSBN 738',1000 from dual union all
select 69396,'2012-S28','2012-06-07 17:39:00','military missile','success',1435,'','15Zh58E GCh-5','','','OT',636,'',1000 from dual union all
select 69397,'2012-S29','2012-06-21 10:40:00','suborbital rocket','success',1338,'NASA 41.101UO','RockOn','','','Tech',1809,'',117 from dual union all
select 69398,'2012-S30','2012-06-22 19:18:00','suborbital rocket','success',1483,'','SHEFEX II','','','Hypersonic',28,'',177 from dual union all
select 69399,'2012-S31','2012-06-23 19:30:00','suborbital rocket','success',224,'NASA 36.286UE','SDO EVE Cal','','','Solar EUV',1895,'',300 from dual;
insert into LAUNCH
select 694,'1947-A30','1947-11-04 23:05:00','military missile','success',340,'3','','','','Test',1863,'',20 from dual union all
select 6940,'1960-S282','1960-09-24 02:02:00','military missile','success',928,'A1X-48','[Grand Turk]','','','Test',244,'',500 from dual union all
select 69400,'2012-S32','2012-06-27 09:15:00','military missile','success',748,'Aegis target','MRT-11 Sep Target','','','Target',855,'',150 from dual union all
select 69401,'2012-S33','2012-06-27 09:20:00','military missile','success',1091,'Aegis FTM-18','Stellar Minotaur','','','Interceptor',851,'CG-70',150 from dual union all
select 69402,'2012-S34','2012-07-05 18:49:59','suborbital rocket','success',224,'NASA 36.284NS','SUMI 2','','','Solar EUV',1895,'',275 from dual union all
select 69403,'2012-S35','2012-07-03 00:00:00','military missile','success',1168,'','','','','Test',1348,'',150 from dual union all
select 69404,'2012-S36','2012-07-03 00:00:00','military missile','success',1166,'','','','','Test',1356,'',100 from dual union all
select 69405,'2012-S37','2012-07-03 00:00:00','military missile','success',1167,'','','','','Test',1356,'',125 from dual union all
select 69406,'2012-S38','2012-07-11 18:50:00','suborbital rocket','success',224,'NASA 36.272NS','Hi-C','','','Solar EUV',1895,'',268 from dual union all
select 69407,'2012-S39','2012-07-23 11:01:00','suborbital rocket','success',234,'NASA 39.011NR','IRVE-III','','','Reentry test',1804,'',458 from dual union all
select 69408,'2012-S40','2012-07-24 19:17:00','suborbital rocket','success',224,'NASA 36.263US','DFS','','','Solar EUV',1895,'',324 from dual union all
select 69409,'2012-S41','2012-08-07 07:30:00','suborbital rocket','success',1073,'S-310-41','MAAC','','','Reentry test',833,'',150 from dual union all
select 6941,'1960-S283','1960-09-23 00:00:00','suborbital rocket','success',972,'','','','','Ionos/Aeron',636,'',86 from dual union all
select 69410,'2012-S42','2012-07-13 04:36:00','military missile','success',62,'','Agni RV','','','Test',316,'',300 from dual union all
select 69411,'2012-S43','2012-08-09 03:16:00','military missile','success',63,'','Agni RV','','','Test',316,'',220 from dual union all
select 69412,'2012-S44','2012-07-24 00:00:00','military missile','success',360,'DF-41','','','','Test',1494,'',1000 from dual union all
select 69413,'2012-S45','2012-02-10 09:00:00','military missile','success',250,'2','Radar target','','','Test',951,'F-15D',100 from dual union all
select 69414,'2012-S46','2012-09-13 12:30:00','military missile','success',632,'3/MBRV-3','PAC-3 Target','','','Target',436,'',100 from dual union all
select 69415,'2012-S47','2012-09-12 04:45:00','military missile','success',1336,'','Shark+ 01/Star 13C','','','Target',1804,'',300 from dual union all
select 69416,'2012-S48','2012-09-19 06:18:00','military missile','success',66,'Agni II Prime','Agni RV','','','Test',316,'',900 from dual union all
select 69417,'2012-S49','2012-08-16 00:00:00','military missile','success',615,'','','','','Test',205,'T094',1000 from dual union all
select 69418,'2012-S50','2012-08-20 00:00:00','military missile','success',437,'','','','','Test',1494,'',1000 from dual union all
select 69419,'2012-S51','2012-08-30 00:00:00','military missile','success',357,'DF-31A','','','','Test',1494,'',1000 from dual union all
select 6942,'1960-S284','1960-09-23 06:40:00','suborbital rocket','success',214,'NRC BB-1-08','NO2 seeding','','','Aeron',428,'',100 from dual union all
select 69420,'2012-S52','2012-09-21 11:16:30','suborbital rocket','success',1332,'NASA 46.004GO','RockSat-X 2','','','Edu/Tech',1804,'',153 from dual union all
select 69421,'2012-S53','2012-09-22 11:00:00','suborbital rocket','success',1306,'NASA 12.075GT','TATEOR/Dummy Nihka','SRPO Diagnostic/APRA','','Test',1798,'',269 from dual union all
select 69422,'2012-S54','2012-09-13 00:00:00','suborbital rocket','success',1479,'V07','HiFire-3','','','Hypersonic',29,'',345 from dual union all
select 69423,'2012-S55','2012-10-04 03:37:00','military missile','success',942,'','Test','','','Test',315,'',100 from dual union all
select 69424,'2012-S56','2012-10-05 05:55:00','military missile','success',430,'','Test','','','Test',320,'INS-P51',100 from dual union all
select 69425,'2012-S57','2012-09-20 19:50:00','military missile','success',65,'','','','','Test',316,'',500 from dual union all
select 69426,'2012-S58','2012-10-19 00:00:00','military missile','success',1507,'VMF RF','','','','OT',1011,'K-433',1000 from dual union all
select 69427,'2012-S59','2012-10-19 09:12:00','military missile','success',1434,'110','15Zh58 UBP-39','','','OT',608,'',1000 from dual union all
select 69428,'2012-S60','2012-10-25 00:00:00','military missile','success',451,'FTI-01','FTI-01 THAAD Target','','','Target',1272,'C-17',300 from dual union all
select 69429,'2012-S61','2012-10-25 00:00:00','military missile','success',1344,'FTI-01 Aegis Target','ARAV-B','','','Target',1784,'',100 from dual union all
select 6943,'1960-S285','1960-09-24 00:00:00','military missile','success',976,'','8K63 UBP-77','','','Test',636,'',402 from dual union all
select 69430,'2012-S62','2012-10-25 00:00:00','military missile','success',1090,'Aegis FTI-01','','','','ABM Test',864,'DDG-62',150 from dual union all
select 69431,'2012-S63','2012-10-25 00:00:00','military missile','success',1151,'FMA','FTI-01 PAC-3 Target','','','ABM Test',865,'MLP',150 from dual union all
select 69432,'2012-S64','2012-10-24 18:28:00','military missile','success',1067,'Avangard','15Zh67 LKI-3','','','R&D',636,'',1000 from dual union all
select 69433,'2012-S65','2012-11-02 17:55:00','suborbital rocket','success',224,'NASA 36.255US','FOXSI','','','Solar X',1895,'',321 from dual union all
select 69434,'2012-S66','2012-11-14 11:07:01','military missile','success',799,'','Mk 12/21 RV','GT206GM','GLORY TRIP 206GM','Op Test',1706,'',1300 from dual union all
select 69435,'2012-S67','2012-10-25 00:00:00','military missile','success',1294,'THAAD FTI-01','','','','ABM Test',868,'',100 from dual union all
select 69436,'2012-S68','2012-11-21 10:55:13','suborbital rocket','success',224,'NASA 36.260UG','IMAGER','','','UV Astron',1895,'',291 from dual union all
select 69437,'2012-S69','2012-12-13 05:20:00','suborbital rocket','success',224,'NASA 36.283UH','DXL/STORM','','','XR Astron',1895,'',257 from dual union all
select 69438,'2012-S70','2012-11-25 11:20:00','suborbital rocket','success',847,'','DLR MAPHEUS-3','','','Microgravity',417,'',145 from dual union all
select 69439,'2012-S71','2012-11-28 00:00:00','military missile','success',509,'Hatf-V','[1300 km] Arab.Sea','','','Test',1473,'',400 from dual union all
select 6944,'1960-S286','1960-09-26 00:00:00','military missile','success',883,'006','','','','Reentry Test',313,'',150 from dual union all
select 69440,'2012-S72','2012-12-08 21:00:00','suborbital rocket','success',1479,'V10','Iguaiba','','','Ionos',13,'',428 from dual union all
select 69441,'2012-S73','2012-12-17 07:00:00','suborbital rocket','success',1074,'S-520-28','Microgravity/Dust nuc','','','Microgravity/Physics',833,'',312 from dual union all
select 69442,'2012-S74','2012-12-20 03:51:00','military missile','success',942,'','Test','','','Test',315,'',100 from dual union all
select 69443,'2012-S75','2012-10-23 00:00:00','military missile','success',1443,'DASO-10','-    (143s)','','','OT',421,'UK S30',1000 from dual union all
select 69444,'2012-S76','2012-08-25 05:34:00','military missile','success',942,'','Test','','','Test',315,'',100 from dual union all
select 69445,'2012-S78','2012-12-10 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69446,'2012-S79','2012-12-10 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69447,'2012-S80','2012-12-20 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69448,'2012-S81','2012-12-20 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69449,'2012-S82','2012-12-20 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 6945,'1960-S287','1960-09-26 11:25:00','suborbital rocket','success',665,'K-8-4','ID-3/CR-4','','','Ionos',7,'',185 from dual union all
select 69450,'2012-S83','2012-12-20 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69451,'2012-S84','2012-12-10 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69452,'2012-S85','2012-12-10 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69453,'2012-S86','2012-12-10 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69454,'2012-S87','2012-12-10 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69455,'2012-S88','2012-11-30 00:00:00','military missile','success',357,'DF-31A','','','','Test',1494,'',1000 from dual union all
select 69456,'2012-S89','2012-03-17 00:00:00','suborbital rocket','unknown',683,'UNCONFIRMED','Test payload','','','Test',818,'',200 from dual union all
select 69457,'2012-S90','2012-05-31 00:00:00','suborbital rocket','success',743,'','R&D','','','Test',636,'',300 from dual union all
select 69458,'2012-S91','2012-07-23 00:00:00','suborbital rocket','success',1502,'','IVRE Support 1','','','Aeron',1821,'',90 from dual union all
select 69459,'2012-S92','2012-07-23 00:00:00','suborbital rocket','success',1502,'','IVRE Support 2','','','Aeron',1821,'',90 from dual union all
select 6946,'1960-S288','1960-09-27 14:44:00','suborbital rocket','success',839,'NASA 10.21GE','SBE 5','','','Magnetospheric',428,'',129 from dual union all
select 69460,'2012-S93','2012-02-15 00:00:00','military missile','success',605,'ET-1','Economical Target 1','','','Target',394,'',80 from dual union all
select 69461,'2012-S94','2012-07-12 00:00:00','military missile','success',1153,'','','','','Test',15,'',150 from dual union all
select 69462,'2012-S95','2012-07-12 00:00:00','military missile','success',1151,'','','','','Test',15,'',150 from dual union all
select 69463,'2012-S96','2012-01-01 00:00:00','military missile','success',593,'','','','','Test',30,'',150 from dual union all
select 69464,'2013-001','2013-01-15 16:24:59','orbital','failure',1064,'492-6391-831/72518','Kosmos-2482/3/4','Strela-3M','','Sat',460,'',null from dual union all
select 69465,'2013-002','2013-01-27 04:40:00','orbital','success',527,'H-IIA-22','IGS R-4/Demo','IGS Radar-4','','Sat',1477,'',null from dual union all
select 69466,'2013-003','2013-01-30 07:00:00','orbital','success',831,'74503','STSAT-2C','STSat-2C','','Sat',987,'',null from dual union all
select 69467,'2013-004','2013-01-31 01:48:00','orbital','success',182,'AV-036','TDRS 11','TDRS K','','Sat',292,'',null from dual union all
select 69468,'2013-005','2013-02-06 16:04:24','orbital','success',1226,'I 15000-011 011/1029','Globalstar','G lobalstar-2','','Sat ST26',562,'',null from dual union all
select 69469,'2013-006','2013-02-07 21:36:00','orbital','success',111,'VA212 (568)','Amazonas 3/Azerspace','Amazonas 3/Azerspace','','Sat',346,'',null from dual union all
select 6947,'1960-S289','1960-09-27 22:10:00','suborbital rocket','success',30,'AA3.102','CR/Airglow','','','Aeron',1869,'',100 from dual union all
select 69470,'2013-007','2013-02-11 14:41:47','orbital','success',1241,'L 15000-137 137','Progress M-18M','Progress 7K-TGM No. 418','','Sat ISS 50P',480,'',null from dual union all
select 69471,'2013-008','2013-02-11 18:02:00','orbital','success',182,'AV-035','Landsat 8','Landsat LDCM','','Sat',1775,'',null from dual union all
select 69472,'2013-009','2013-02-25 12:31:00','orbital','success',900,'PSLV-C20','SARAL/Sapphire','SARAL/Sapphire','','Sat',1357,'',null from dual union all
select 69473,'2013-010','2013-03-01 15:10:13','orbital','success',482,'005','Dragon CRS-2','Dragon C104','','Sat',274,'',null from dual union all
select 69474,'2013-011','2013-03-19 21:21:00','orbital','success',182,'AV-037','SBIRS GEO-2','SBIRS GEO-2','','Sat',292,'',null from dual union all
select 69475,'2013-012','2013-03-26 19:06:48','orbital','success',962,'935-36/99536','Satmex 8','Satmex 8','','Sat',553,'',null from dual union all
select 69476,'2013-013','2013-03-28 20:43:20','orbital','success',1233,'E 15000-043 043','Soyuz TMA-08M','Soyuz 11F732A47 No. 708','','Sat ISS 34S',480,'',null from dual union all
select 69477,'2013-014','2013-04-15 18:36:00','orbital','success',962,'935-37/99538','Anik G-1','Anik G1','','Sat',553,'',null from dual union all
select 69478,'2013-015','2013-04-19 10:00:00','orbital','success',1227,'L 15000-014 014','Bion-M No. 1','Bion-M 12KSM No. 1','','Sat',562,'',null from dual union all
select 69479,'2013-016','2013-04-21 21:00:00','orbital','success',82,'A-1/70101-402','Cygnus Mass Sim','A-ONE','','Sat',944,'',null from dual union all
select 6948,'1960-S29','1960-02-02 18:08:00','military missile','success',1419,'B-7A','P13 RV','B-6 Stage 2','','Test',237,'',1000 from dual union all
select 69480,'2013-017','2013-04-24 10:12:16','orbital','success',1241,'L 15000-138 138','Progress M-19M','Progress 7K-TGM No. 419','','Sat ISS 51P',480,'',null from dual union all
select 69481,'2013-018','2013-04-26 04:13:00','orbital','success',304,'Y18','GaoFen 1','GF-1','','Sat',813,'',null from dual union all
select 69482,'2013-019','2013-04-26 05:23:41','orbital','success',1228,'780-31- 220 021/1047','Glonass-M47','Uragan-M No. 747','','Sat',475,'',null from dual union all
select 69483,'2013-020','2013-05-01 16:06:04','orbital','success',311,'Y25','Zhongxing-11','ZX-11','','Sat',1903,'',null from dual union all
select 69484,'2013-021','2013-05-07 02:06:31','orbital','success',1488,'VV02','PROBA-V/VRNREDSAT','PROBA-V','','Sat',349,'',null from dual union all
select 69485,'2013-022','2013-05-14 16:02:00','orbital','success',962,'935-38/99539','Eutelsat 3D','Eutelsat 3D','','Sat',553,'',null from dual union all
select 69486,'2013-023','2013-05-15 21:38:00','orbital','success',182,'AV-039','GPS II F-4','GPS SVN 66','','Sat',292,'',null from dual union all
select 69487,'2013-024','2013-05-25 00:27:00','orbital','success',404,'D4-22 (362)','WGS 5','Wideband Global Satcom 5','','Sat',290,'',null from dual union all
select 69488,'2013-025','2013-05-28 20:31:24','orbital','success',1233,'E 15000-045 045','Soyuz TMA-09M','Soyuz 11F732A47 No. 709?','','Sat ISS 35S',480,'',null from dual union all
select 69489,'2013-026','2013-06-03 09:18:31','orbital','success',962,'935-40/99541','SES-6','SES-6','','Sat',553,'',null from dual;
insert into LAUNCH
select 6949,'1960-S290','1960-09-27 22:10:00','suborbital rocket','success',838,'AA8.350','AS&E-1 Lunar X-ray','','','XR Astron',394,'',233 from dual union all
select 69490,'2013-027','2013-06-05 21:52:18','orbital','success',113,'VA213 (592)','ATV 4','ATV 4','','Sat',346,'',null from dual union all
select 69491,'2013-028','2013-06-07 18:37:59','orbital','success',1229,'780-31- 215 007','Kosmos-2486','Persona No. 2','','Sat',475,'',null from dual union all
select 69492,'2013-029','2013-06-11 09:38:03','orbital','success',306,'Y10','Shenzhou 10','','','Sat',814,'',null from dual union all
select 69493,'2013-030','2013-06-25 17:28:48','orbital','success',1229,'I 15000-013 013','Resurs-P No. 1','Resurs-P No. 1','','Sat',562,'',null from dual union all
select 69494,'2013-031','2013-06-25 19:27:03','orbital','success',1236,'E 15000-003 003/1041','O3b Mission 1','O3b SV1/SV2/SV4/SV5','','Sat VS05',347,'',null from dual union all
select 69495,'2013-032','2013-06-27 16:53:00','orbital','success',1265,'492-6391-832','Kosmos-2487','Kondor No. 202','','Sat',538,'',null from dual union all
select 69496,'2013-033','2013-06-28 02:27:46','orbital','success',911,'F42','IRIS','IRIS','','Sat',1718,'L-1011',null from dual union all
select 69497,'2013-034','2013-07-01 18:11:00','orbital','success',902,'PSLV-C22','IRNSS-1A','IRNSS-1A','','Sat',1357,'',null from dual union all
select 69498,'2013-035','2013-07-15 09:27:03','orbital','success',301,'Y23','SJ-11-05','SJ11-05','','Sat',813,'',null from dual union all
select 69499,'2013-036','2013-07-19 13:00:00','orbital','success',196,'AV-040','MUOS 2','MUOS 2','','Sat',292,'',null from dual union all
select 695,'1947-A31','1947-11-10 09:39:00','military missile','failure',8,'21N','','','','Test',636,'',11 from dual union all
select 6950,'1960-S291','1960-09-28 00:00:00','military missile','success',976,'','8K63 UBP-78','','','Test',636,'',402 from dual union all
select 69500,'2013-037','2013-07-19 23:37:56','orbital','success',320,'Y11','SJ-15/CX-3/SY-7','SJ-15/CX-3/SY-7','','Sat',1493,'',null from dual union all
select 69501,'2013-038','2013-07-25 19:54:07','orbital','success',111,'VA214 (569)','Alphasat/Insat-3D','Alphasat/Insat-3D','','Sat',346,'',null from dual union all
select 69502,'2013-039','2013-07-27 20:45:08','orbital','success',1241,'L 15000-130 130','Progress M-20M','Progress 7K-TGM No. 420','','Sat ISS 52P',562,'',null from dual union all
select 69503,'2013-040','2013-08-03 19:48:46','orbital','success',534,'H-IIB-F4','HTV-4','HTV 4','','Sat',1478,'',null from dual union all
select 69504,'2013-041','2013-08-08 00:29:00','orbital','success',404,'D4-23 (363)','WGS 6','Wideband Global Satcom 6','','Sat',290,'',null from dual union all
select 69505,'2013-042','2013-08-22 14:39:20','orbital','success',434,'510-7681-112','Arirang-5','Kompsat-5','','Sat',1907,'',null from dual union all
select 69506,'2013-043','2013-08-28 18:03:00','orbital','success',397,'D4-24 (364)','USA 245      [IC]','EIS CRYSTAL','','Sat',1780,'',null from dual union all
select 69507,'2013-044','2013-08-29 20:30:00','orbital','success',111,'VA215 (570)','Eutelsat 25B/GSAT-7','Es''hail/GSAT-7','','Sat',346,'',null from dual union all
select 69508,'2013-045','2013-08-31 20:05:00','orbital','success',1543,'SB60.6/SLB 6L','Amos 4','Amos 4','','Sat',567,'',null from dual union all
select 69509,'2013-046','2013-09-01 19:16:00','orbital','success',320,'Y13','Yaogan 17','YW-17','','Sat',813,'',null from dual union all
select 6951,'1960-S292','1960-09-28 13:32:00','military missile','success',1419,'J-8','Mk 4-2 RV','','','Test',237,'',1000 from dual union all
select 69510,'2013-047','2013-09-07 03:27:00','orbital','success',791,'1','LADEE','LADEE','','Sat',945,'',null from dual union all
select 69511,'2013-048','2013-09-11 23:23:04','orbital','success',1064,'492-6391-838/72519','Gonets-M  Blok 12','Gonets-M No. 14/16/17','','Sat',460,'',null from dual union all
select 69512,'2013-049','2013-09-14 05:00:00','orbital','success',465,'E-1','SPRINT-A','','','Sat',832,'',null from dual union all
select 69513,'2013-050','2013-09-18 08:10:00','orbital','success',192,'AV-041','AEHF 3','AEHF 3','','Sat',292,'',null from dual union all
select 69514,'2013-051','2013-09-18 14:58:02','orbital','success',82,'2/2TRS2S1.3','Orb-D','Cygnus 1','','Sat',944,'',null from dual union all
select 69515,'2013-052','2013-09-23 03:07:17','orbital','success',320,'Y12','Fengyun 3-3','san ke fengyun san hao','','Sat',1493,'',null from dual union all
select 69516,'2013-053','2013-09-25 04:37:00','orbital','success',683,'','Kuaizhou-1','','','Sat',818,'',null from dual union all
select 69517,'2013-054','2013-09-25 20:58:50','orbital','success',1233,'E 15000-046 046','Soyuz TMA-10M','Soyuz 11F732A47 No. 710','','Sat ISS 36S',480,'',null from dual union all
select 69518,'2013-055','2013-09-29 16:00:13','orbital','success',476,'006','Cassiope/Demo','Cassiope','','Sat',1777,'',null from dual union all
select 69519,'2013-056','2013-09-29 21:38:10','orbital','success',962,'935-39/99540','Astra 2E','Astra 2E','','Sat',553,'',null from dual union all
select 6952,'1960-S293','1960-09-28 19:38:00','military missile','success',916,'205','P-07','','','Test',257,'',180 from dual union all
select 69520,'2013-057','2013-10-25 03:50:00','orbital','success',319,'Y25','SJ-16','SJ-16','','Sat',813,'',null from dual union all
select 69521,'2013-058','2013-10-25 18:08:54','orbital','success',962,'935-35/99542','Sirius FM6','Sirius FM6','','Sat',553,'',null from dual union all
select 69522,'2013-059','2013-10-29 02:50:00','orbital','success',301,'Y28','Yaogan 18','YW18','','Sat',1493,'',null from dual union all
select 69523,'2013-060','2013-11-05 09:08:00','orbital','success',902,'PSLV-C25','MOM','Mars Orbiter Spacecraft','','Sat',1357,'',null from dual union all
select 69524,'2013-061','2013-11-07 04:14:15','orbital','success',1233,'T 15000-048 048','Soyuz TMA-11M','Soyuz 11F732A47 No. 711','','Sat ISS 37S',480,'',null from dual union all
select 69525,'2013-062','2013-11-11 23:46:00','orbital','success',962,'535-41/88532','Raduga-1M','Globus-1M No. 13L','','Sat',588,'',null from dual union all
select 69526,'2013-063','2013-11-18 18:28:00','orbital','success',182,'AV-038','MAVEN','MAVEN','','Sat',292,'',null from dual union all
select 69527,'2013-064','2013-11-20 01:15:00','orbital','success',783,'','ORS-3/STPSAT-3','ORS-3','','Sat',945,'',null from dual union all
select 69528,'2013-065','2013-11-20 03:31:05','orbital','success',320,'Y14','Yaogan 19','yaogan weixing 19','','Sat',1493,'',null from dual union all
select 69529,'2013-066','2013-11-21 07:10:17','orbital','success',434,'510-7681-111','Dubaisat-2','Dubiasat-2/Cubesats','','Sat',1907,'',null from dual union all
select 6953,'1960-S294','1960-09-28 22:45:00','suborbital rocket','success',839,'OB6.07','H2O/ne','','','Aeron',428,'',105 from dual union all
select 69530,'2013-067','2013-11-22 12:02:29','orbital','success',1064,'492-5882-031/72524','SWARM','SWARM 1,2,3','','Sat',460,'',null from dual union all
select 69531,'2013-068','2013-11-25 02:12:00','orbital','success',304,'Y23','Shiyan 5','SW5','','Sat',813,'',null from dual union all
select 69532,'2013-069','2013-11-25 20:53:06','orbital','success',1241,'E 15000-133 133','Progress M-21M','Progress 7K-TGM No. 421','','Sat ISS 53P',562,'',null from dual union all
select 69533,'2013-070','2013-12-01 17:30:00','deep space','success',312,'Y23/G3Z','Chang''e 3','CE-3','','Sat',1903,'',null from dual union all
select 69534,'2013-071','2013-12-03 22:41:00','orbital','success',476,'007','SES-8','SES-8','','Sat',274,'',null from dual union all
select 69535,'2013-072','2013-12-06 07:14:30','orbital','success',190,'AV-042','USA 247','TOPAZ 3','','Sat',1775,'',null from dual union all
select 69536,'2013-073','2013-12-08 12:12:00','orbital','success',962,'935-44/99546','Inmarsat 5F1','Inmarsat 5F1','','Sat',553,'',null from dual union all
select 69537,'2013-074','2013-12-19 09:12:18','orbital','success',1236,'E 15000-004 004/1040','Gaia','Gaia','','Sat VS06',347,'',null from dual union all
select 69538,'2013-075','2013-12-20 16:42:04','orbital','success',311,'Y27','Tupac Katari','TKSat-1','','Sat',1903,'',null from dual union all
select 69539,'2013-076','2013-12-25 00:31:55','orbital','success',1064,'492-3931-130/72520','Kosmos-2488/89/90/91','Rodnik','','Sat',460,'',null from dual union all
select 6954,'1960-S295','1960-09-29 00:00:00','military missile','success',976,'','8K63 UBP-79','','','Test',636,'',402 from dual union all
select 69540,'2013-077','2013-12-26 10:49:56','orbital','success',962,'935-41/99543','Ekspress AM-5','Ekspress AM-5','','Sat',588,'',null from dual union all
select 69541,'2013-078','2013-12-28 12:30:00','orbital','success',1230,'760-58- 002 002/001','Aist-1/SKRL-756','Aist-1','','Sat',475,'',null from dual union all
select 69542,'2013-A01','2013-01-05 18:41:00','suborbital rocket','failure',1113,'3','','','','Test',1422,'',5 from dual union all
select 69543,'2013-A02','2013-03-07 00:00:00','atmospheric rocket','success',517,'','','','','Test',1428,'',0 from dual union all
select 69544,'2013-A03','2013-03-13 12:00:00','atmospheric rocket','success',472,'24','','','','Test',330,'',32 from dual union all
select 69545,'2013-A04','2013-04-03 15:31:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-113/GF24','','','Glide   9',955,'WK2',15 from dual union all
select 69546,'2013-A05','2013-04-12 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-114/CF01','Cold Flow','','Glide  10.8',955,'WK2',15 from dual union all
select 69547,'2013-A06','2013-04-19 00:00:00','atmospheric rocket','success',517,'','','','','Test',1428,'',0 from dual union all
select 69548,'2013-A07','2013-04-29 14:47:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-115/PF01','','','       13       /0:16',955,'WK2',17 from dual union all
select 69549,'2013-A08','2013-05-01 17:55:00','atmospheric rocket','success',1525,'4','','','','Hypersonic Scramjet',379,'B-52H',16 from dual union all
select 6955,'1960-S296','1960-09-29 14:20:00','military missile','success',1419,'G-8','RVX-3B','','','Test',230,'',1000 from dual union all
select 69550,'2013-A09','2013-05-05 07:30:00','military missile','failure',758,'','','','','Test',207,'S618',1 from dual union all
select 69551,'2013-A10','2013-05-23 00:00:00','atmospheric rocket','success',472,'25','','','','Test',13,'',32 from dual union all
select 69552,'2013-A11','2013-06-14 00:00:00','atmospheric rocket','success',517,'','','','','Test',1428,'',0 from dual union all
select 69553,'2013-A12','2013-08-13 00:00:00','atmospheric rocket','success',517,'','','','','Test',1428,'',0 from dual union all
select 69554,'2013-A13','2013-07-25 00:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-130/GF25','','','Glide  11:52',955,'WK2',15 from dual union all
select 69555,'2013-A14','2013-08-08 14:50:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-131/GF26','','','Glide  10',955,'WK2',15 from dual union all
select 69556,'2013-A15','2013-09-05 16:11:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-132/PF02','','','       14      /0:20',955,'WK2',21 from dual union all
select 69557,'2013-A16','2013-08-15 00:00:00','military missile','success',1019,'PAAT','PAC-3 Target','','','Target',1895,'',40 from dual union all
select 69558,'2013-A17','2013-08-08 17:51:00','atmospheric rocket','success',472,'26','','','','Test',13,'',32 from dual union all
select 69559,'2013-A18','2013-09-18 13:18:00','suborbital rocket','failure',1479,'V09','SCRAMSPACE 1','','','Hypersonic',29,'',5 from dual union all
select 6956,'1960-S297','1960-09-29 20:31:11','military missile','failure',154,'33D','Mk 3-3','IOC DASO','HIGH ARROW','Test',1666,'',500 from dual union all
select 69560,'2013-A19','2013-10-07 00:00:00','atmospheric rocket','success',517,'','','','','Test',1428,'',1 from dual union all
select 69561,'2013-A20','2013-10-30 00:00:00','military missile','success',3,'','Test','','','Op Test',181,'',40 from dual union all
select 69562,'2013-A21','2013-10-30 00:00:00','military missile','success',1430,'','','','','Test',636,'',26 from dual union all
select 69563,'2013-A22','2013-10-30 00:00:00','military missile','success',1430,'','','','','Test',636,'',26 from dual union all
select 69564,'2013-A23','2013-10-30 00:00:00','military missile','success',1430,'','','','','Test',636,'',26 from dual union all
select 69565,'2013-A24','2013-10-30 00:00:00','military missile','success',2,'S-300','','','','OT',636,'',40 from dual union all
select 69566,'2013-A25','2013-10-30 00:00:00','military missile','success',2,'S-400','','','','OT',636,'',40 from dual union all
select 69567,'2013-A26','2013-12-11 17:45:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-145/GF27','','','Glide  10',955,'WK2',15 from dual union all
select 69568,'2013-A27','2013-12-18 10:42:00','suborbital rocket','success',292,'','Experiencia Centenario','','','Micrograv/Tech',310,'',49 from dual union all
select 69569,'2013-A28','2013-02-10 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 6957,'1960-S298','1960-09-30 00:00:00','military missile','success',976,'','8K63 UBP-80','','','Test',636,'',402 from dual union all
select 69570,'2013-A29','2013-03-15 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69571,'2013-A30','2013-05-18 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69572,'2013-A31','2013-05-19 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69573,'2013-A32','2013-05-20 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69574,'2013-A33','2013-07-01 00:00:00','military missile','success',702,'MEADS Target','','','','Target',1895,'',10 from dual union all
select 69575,'2013-A34','2013-07-01 00:00:00','military missile','success',702,'MEADS Target','','','','Target',1895,'',10 from dual union all
select 69576,'2013-A35','2013-07-01 00:00:00','military missile','success',702,'MEADS Target','','','','Target',1895,'',10 from dual union all
select 69577,'2013-A36','2013-08-01 00:00:00','military missile','success',702,'MEADS Target','','','','Target',1895,'',10 from dual union all
select 69578,'2013-A37','2013-08-01 00:00:00','military missile','success',702,'MEADS Target','','','','Target',1895,'',10 from dual union all
select 69579,'2013-A38','2013-09-01 00:00:00','military missile','success',702,'MEADS Target','','','','Target',1895,'',10 from dual;
insert into LAUNCH
select 6958,'1960-S299','1960-09-30 00:00:00','military missile','success',976,'','8K63 UBP-81','','','Test',636,'',402 from dual union all
select 69580,'2013-A39','2013-09-01 00:00:00','military missile','success',702,'MEADS Target','','','','Target',1895,'',10 from dual union all
select 69581,'2013-A40','2013-10-01 00:00:00','military missile','success',702,'MEADS Target','','','','Target',1895,'',10 from dual union all
select 69582,'2013-A41','2013-10-01 00:00:00','military missile','success',702,'MEADS Target','','','','Target',1895,'',10 from dual union all
select 69583,'2013-A42','2013-11-06 00:00:00','military missile','success',702,'MEADS Target','','','','Target',1895,'',10 from dual union all
select 69584,'2013-F01','2013-02-01 06:56:00','orbital','failure',1542,'SL36/DM-SL-35L','SL48 Intelsat IS-27','Intelsat IS-27','','Sat',860,'ODYSSEY',1 from dual union all
select 69585,'2013-F02','2013-07-02 02:38:22','orbital','failure',964,'535-43/2L','Glonass-M','Glonass-M 48,49,50','','Sat',588,'',1 from dual union all
select 69586,'2013-F03','2013-12-09 03:26:00','orbital','failure',319,'Y10','CBERS 3','CBERS 3','','Sat',1493,'',720 from dual union all
select 69587,'2013-M01','2013-01-02 00:00:00','military missile','success',489,'','','','','Weapon',14,'',60 from dual union all
select 69588,'2013-M02','2013-01-03 00:00:00','military missile','success',489,'','','','','Weapon',14,'',60 from dual union all
select 69589,'2013-M03','2013-01-09 00:00:00','military missile','success',489,'','','','','Weapon',14,'',60 from dual union all
select 6959,'1960-S30','1960-02-04 22:19:00','military missile','success',928,'A1X-12','','','','Test',251,'',500 from dual union all
select 69590,'2013-M04','2013-06-13 18:14:00','atmospheric rocket','success',473,'9','','','','Test',13,'',52 from dual union all
select 69591,'2013-M05','2013-06-26 18:00:00','atmospheric rocket','success',473,'10','','','','Test',330,'',52 from dual union all
select 69592,'2013-M06','2013-10-30 00:00:00','military missile','success',610,'','','','','Test',636,'',50 from dual union all
select 69593,'2013-M07','2013-11-21 12:55:00','suborbital rocket','success',1023,'','Jubilee Launch','','','Meteo',1467,'',70 from dual union all
select 69594,'2013-S01','2013-01-26 22:00:00','military missile','success',497,'','GMD CTV-01','','','Test',1709,'',1000 from dual union all
select 69595,'2013-S02','2013-01-27 08:10:00','military missile','success',639,'14','BO-5','','','Test',1750,'',300 from dual union all
select 69596,'2013-S03','2013-01-27 12:10:00','military missile','success',198,'ABM Target','','','','ABM',819,'',100 from dual union all
select 69597,'2013-S04','2013-01-27 12:10:00','military missile','success',354,'ABM Interceptor','','','','ABM',1629,'',250 from dual union all
select 69598,'2013-S05','2013-01-28 00:00:00','suborbital rocket','success',669,'Kavoshgar Pishgam','Kavoshgar Class C','Pioneer Probe','','Bio',1356,'',120 from dual union all
select 69599,'2013-S06','2013-01-29 22:50:00','suborbital rocket','success',1339,'NASA 41.107GT','Li release test','','','Aeron/Tech',1809,'',130 from dual union all
select 696,'1947-A32','1947-11-14 00:00:00','atmospheric rocket','success',443,'','D-558-II Model','','','Test',1821,'',5 from dual union all
select 6960,'1960-S300','1960-10-01 00:00:00','military missile','success',976,'','8K63 UBP-83','','','Test',636,'',402 from dual union all
select 69600,'2013-S07','2013-02-07 08:21:00','suborbital rocket','success',890,'NASA 49.001UE','VISIONS','','','Auroral',1044,'',754 from dual union all
select 69601,'2013-S08','2013-02-13 09:10:00','military missile','success',747,'Aegis FTM-20 target','MBRM-T3 Sep Target','[or ARAV-B]','','Target',855,'',150 from dual union all
select 69602,'2013-S09','2013-02-13 09:15:00','military missile','success',1090,'Aegis FTM-20','Stellar Eyes','','','Interceptor',851,'CG-70',150 from dual union all
select 69603,'2013-S10','2013-02-15 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69604,'2013-S100','2013-11-27 01:53:00','suborbital rocket','success',224,'NASA 36.261UG','VeSpR','','','UV Astron',1895,'',280 from dual union all
select 69605,'2013-S101','2013-12-14 00:00:00','suborbital rocket','success',1166,'Kavoshgar Pazhuhesh','Kavoshgar Type D','','','Bio',1347,'',120 from dual union all
select 69606,'2013-S102','2013-09-27 00:00:00','military missile','failure',1454,'492-6391-837','15A35-71 LKI-1','Yu-71 Hypersonic RV','','Test',1908,'',1000 from dual union all
select 69607,'2013-S103','2013-12-17 12:36:00','military missile','success',799,'','Mk 12/21 RV','GT210GM','GLORY TRIP 210GM','Op Test',1699,'',1300 from dual union all
select 69608,'2013-S104','2013-12-24 07:00:00','military missile','success',1531,'','15Zh65M LKI-1','RV x 4? Silo','','OT',608,'',1000 from dual union all
select 69609,'2013-S105','2013-12-27 17:30:00','military missile','success',1435,'','15Zh58E GCh-7','RV Test 2','','R&D',636,'',1000 from dual union all
select 6961,'1960-S301','1960-10-01 00:00:00','military missile','success',976,'','8K63 UBP-85','','','Test',636,'',402 from dual union all
select 69610,'2013-S106','2013-12-23 11:28:00','military missile','success',65,'','','','','Test',316,'',350 from dual union all
select 69611,'2013-S107','2013-05-07 04:00:00','suborbital rocket','success',605,'SSC REXUS 14','EURO REXUS 14','','','Tech',417,'',81 from dual union all
select 69612,'2013-S108','2013-05-09 04:00:00','suborbital rocket','success',605,'SSC REXUS 13','EURO REXUS 13','','','Tech',417,'',81 from dual union all
select 69613,'2013-S109','2013-06-05 07:00:00','suborbital rocket','success',1221,'8','H2O2 test/Recovery','','','Tech',808,'',279 from dual union all
select 69614,'2013-S11','2013-02-15 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69615,'2013-S110','2013-12-13 00:00:00','military missile','success',360,'','','','','Test',1494,'',1000 from dual union all
select 69616,'2013-S112','2013-12-18 00:00:00','military missile','success',1067,'','15Zh67 LKI-5','','','R&D',636,'',1000 from dual union all
select 69617,'2013-S12','2013-02-16 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69618,'2013-S13','2013-02-16 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69619,'2013-S14','2013-02-17 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 6962,'1960-S302','1960-10-01 00:00:00','military missile','success',979,'','8K65 LKI-8','','','Test',636,'',675 from dual union all
select 69620,'2013-S15','2013-02-17 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69621,'2013-S16','2013-02-18 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69622,'2013-S17','2013-02-19 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69623,'2013-S18','2013-02-22 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69624,'2013-S19','2013-02-22 00:00:00','military missile','success',1151,'','','','','Weapon',14,'',80 from dual union all
select 69625,'2013-S20','2013-02-25 06:00:00','military missile','success',120,'','Hetz-3','','','Test',1026,'',100 from dual union all
select 69626,'2013-S21','2013-03-11 12:09:00','military missile','success',1336,'','Shark+ 02/Star 13C','','','Target',1809,'',300 from dual union all
select 69627,'2013-S22','2013-04-12 04:25:00','suborbital rocket','success',1484,'V17','DLR TEXUS 50','','','Micrograv',416,'',261 from dual union all
select 69628,'2013-S23','2013-04-10 00:00:00','military missile','success',1169,'Hatf IV','','','','Test',1419,'',100 from dual union all
select 69629,'2013-S24','2013-04-04 21:55:00','suborbital rocket','success',1398,'Kunpeng-1','','','','Ionos',647,'',191 from dual union all
select 6963,'1960-S303','1960-10-01 00:00:00','military missile','success',979,'','8K65 LKI-9','','','Test',636,'',675 from dual union all
select 69630,'2013-S25','2013-04-07 04:55:00','military missile','success',63,'','Agni RV','','','Test',316,'',220 from dual union all
select 69631,'2013-S26','2013-04-21 08:00:00','suborbital rocket','success',224,'NASA 36.271UG','SLICE','','','UV Astron',1895,'',318 from dual union all
select 69632,'2013-S27','2013-04-23 17:30:00','suborbital rocket','success',224,'NASA 36.269GS','EUNIS 3','','','Solar EUV',1895,'',320 from dual union all
select 69633,'2013-S28','2013-02-15 18:34:31','suborbital rocket','success',1334,'NASA 41.104GT','','','','Tech',1895,'',130 from dual union all
select 69634,'2013-S29','2013-05-01 07:38:00','suborbital rocket','success',1334,'NASA 41.100DR','MOSC 1A','','','Active plasma  Sm',871,'',185 from dual union all
select 69635,'2013-S30','2013-05-11 05:00:00','suborbital rocket','success',224,'NASA 36.268UG','FORTIS 1','','','UV Astron',1895,'',280 from dual union all
select 69636,'2013-S31','2013-05-07 07:39:00','suborbital rocket','success',1344,'NASA 45.005UE','EVEX','','','Aeron/release',871,'',350 from dual union all
select 69637,'2013-S32','2013-05-07 07:40:30','suborbital rocket','success',1333,'NASA 46.001UE','EVEX','','','Aeron/release',871,'',190 from dual union all
select 69638,'2013-S34','2013-05-09 07:23:00','suborbital rocket','success',1334,'NASA 41.102DR','MOSC 1B','','','Active plasma  Sm',871,'',188 from dual union all
select 69639,'2013-S35','2013-05-16 03:25:00','military missile','success',1303,'Aegis FTM-19 Tgt','ARAV-C','','','Target',855,'',300 from dual union all
select 6964,'1960-S304','1960-09-04 00:00:00','military missile','success',872,'ZW-10','','','','Test',1884,'',40 from dual union all
select 69640,'2013-S36','2013-05-16 03:30:00','military missile','success',1091,'Aegis FTM-19','Stellar Hecate','','','Interceptor',853,'CG-70',150 from dual union all
select 69641,'2013-S37','2013-05-22 13:27:00','military missile','success',799,'','Mk 12/21 RV','GT207GM','GLORY TRIP 207GM','Op Test',1699,'',1300 from dual union all
select 69642,'2013-S38','2013-06-06 03:05:00','suborbital rocket','success',237,'NASA 40.030UG','CIBER-I F4','','','IR Astron',1798,'',577 from dual union all
select 69643,'2013-S39','2013-06-06 17:45:00','military missile','success',1067,'Avangard','15Zh67 LKI-4','','','R&D',636,'',1000 from dual union all
select 69644,'2013-S40','2013-06-20 09:30:00','suborbital rocket','success',1338,'NASA 41.106UO','RockOn 2013','','','Edu',1798,'',118 from dual union all
select 69645,'2013-S41','2013-06-21 13:57:00','suborbital rocket','success',1244,'SpaceLoft SL-7','NASA FOP-1','Celestis Centennial','','Tech',1422,'',119 from dual union all
select 69646,'2013-S42','2013-06-20 00:00:00','military missile','success',1153,'Scud D','','','','Weapon',15,'',100 from dual union all
select 69647,'2013-S43','2013-03-29 00:00:00','military missile','success',1153,'Scud D','','','','Weapon',15,'',100 from dual union all
select 69648,'2013-S44','2013-01-05 00:00:00','military missile','success',1151,'Scud','','','','Weapon',15,'',100 from dual union all
select 69649,'2013-S45','2013-07-04 14:31:25','suborbital rocket','success',228,'NASA 21.140GE','Daytime Dynamo','','','Ionos',1804,'',135 from dual union all
select 6965,'1960-S305','1960-09-30 00:00:00','military missile','success',883,'007','','','','Reentry Test',313,'',150 from dual union all
select 69650,'2013-S46','2013-07-04 14:31:40','suborbital rocket','success',1339,'NASA 41.090GE','Daytime Dynamo','','','Ionos',1809,'',160 from dual union all
select 69651,'2013-S47','2013-07-05 18:30:00','military missile','success',695,'GBI Target','GMD FTG-07 Target','','','Target',868,'',1000 from dual union all
select 69652,'2013-S48','2013-07-05 18:35:00','military missile','failure',497,'GBI','GMD FTG-07','CE-I EKV','','Interceptor',1709,'',1000 from dual union all
select 69653,'2013-S49','2013-07-15 05:53:00','suborbital rocket','success',1478,'V10','MAPHEUS 4','','','Micrograv',416,'',151 from dual union all
select 69654,'2013-S50','2013-07-20 02:00:00','suborbital rocket','success',1073,'S-310-42','','','','Aeron',833,'',139 from dual union all
select 69655,'2013-S51','2013-07-20 02:57:00','suborbital rocket','success',1074,'S-520-27','','','','Aeron',833,'',316 from dual union all
select 69656,'2013-S52','2013-07-12 00:00:00','military missile','success',628,'','','','','Test',1026,'',300 from dual union all
select 69657,'2013-S53','2013-08-08 18:10:00','suborbital rocket','success',224,'NASA 36.239DS','VERIS','','','Solar UV',1895,'',280 from dual union all
select 69658,'2013-S54','2013-08-12 03:45:00','military missile','success',942,'','Test','','','Test',315,'',100 from dual union all
select 69659,'2013-S55','2013-08-13 10:00:00','suborbital rocket','success',1331,'NASA 46.005UO','Rocksat-X 3','','','Edu',1804,'',151 from dual union all
select 6966,'1960-S306','1960-09-30 06:17:00','suborbital rocket','success',214,'NRC BB-1-09','NO2 seeding','','','Aeron',428,'',100 from dual union all
select 69660,'2013-S56','2013-09-10 00:00:00','military missile','success',1552,'MBRV','FTO-01 Aegis Target','','','Target',1784,'',300 from dual union all
select 69661,'2013-S57','2013-09-03 06:16:00','military missile','success',1183,'','Radar target','Ankor','','Test',951,'F-15D',150 from dual union all
select 69662,'2013-S58','2013-09-10 00:00:00','military missile','success',1551,'MBRV','FTO-01 THAAD Target','','','Target',1272,'C-17',300 from dual union all
select 69663,'2013-S59','2013-09-10 00:00:00','military missile','success',1090,'Aegis FTO-01','','','','Interceptor',864,'DDG-73',150 from dual union all
select 69664,'2013-S60','2013-09-10 00:00:00','military missile','success',1294,'THAAD FTO-01','eMBRM intercept','','','ABM Test',868,'',100 from dual union all
select 69665,'2013-S61','2013-09-10 00:00:00','military missile','success',1294,'THAAD FTO-01','Aegis Backup','','','ABM Test',868,'',100 from dual union all
select 69666,'2013-S62','2013-09-06 05:20:00','military missile','failure',261,'','','','','Test',190,'K-550',100 from dual union all
select 69667,'2013-S63','2013-09-15 03:13:00','military missile','success',67,'','','','','Test',316,'',800 from dual union all
select 69668,'2013-S64','2013-09-19 00:30:00','military missile','success',1303,'Aegis FTM-21 Tgt','ARAV-C++','','','Target',855,'',300 from dual union all
select 69669,'2013-S65','2013-09-19 00:32:00','military missile','success',1091,'Aegis FTM-21','Stellar Ninja','','','Interceptor',851,'CG-70',300 from dual;
insert into LAUNCH
select 6967,'1960-S307','1960-10-05 16:04:00','military missile','success',928,'A1X-49','','','','Test',244,'',500 from dual union all
select 69670,'2013-S66','2013-09-19 00:32:00','military missile','success',1091,'Aegis FTM-21','Stellar Ninja','','','Interceptor',851,'CG-70',300 from dual union all
select 69671,'2013-S67','2013-06-26 00:00:00','suborbital rocket','success',1284,'','WADIS SL1','','','Aeron',29,'',100 from dual union all
select 69672,'2013-S68','2013-06-27 23:52:30','suborbital rocket','success',1478,'V09','WADIS 1','','','Aeron',29,'',115 from dual union all
select 69673,'2013-S69','2013-06-27 00:00:00','suborbital rocket','success',1284,'','WADIS SL2','','','Aeron',29,'',100 from dual union all
select 69674,'2013-S70','2013-06-27 00:00:00','suborbital rocket','success',1284,'','WADIS SL3','','','Aeron',29,'',100 from dual union all
select 69675,'2013-S71','2013-06-27 00:00:00','suborbital rocket','success',1284,'','WADIS SL4','','','Aeron',29,'',100 from dual union all
select 69676,'2013-S72','2013-06-27 00:00:00','suborbital rocket','success',1284,'','WADIS SL5','','','Aeron',29,'',100 from dual union all
select 69677,'2013-S73','2013-06-28 00:00:00','suborbital rocket','success',1284,'','WADIS SL6','','','Aeron',29,'',100 from dual union all
select 69678,'2013-S74','2013-06-29 00:00:00','suborbital rocket','success',1284,'','WADIS SL7','','','Aeron',29,'',100 from dual union all
select 69679,'2013-S75','2013-06-30 00:00:00','suborbital rocket','success',1284,'','WADIS SL8','','','Aeron',29,'',100 from dual union all
select 6968,'1960-S308','1960-10-05 19:52:00','suborbital rocket','success',30,'NASA 04.43GP','AMPP','','','Remote sensing',428,'',225 from dual union all
select 69680,'2013-S76','2013-07-01 00:00:00','suborbital rocket','success',1284,'','WADIS SL9','','','Aeron',29,'',100 from dual union all
select 69681,'2013-S77','2013-07-02 00:00:00','suborbital rocket','success',1284,'','WADIS SL10','','','Aeron',29,'',100 from dual union all
select 69682,'2013-S78','2013-07-03 00:00:00','suborbital rocket','success',1284,'','WADIS SL11','','','Aeron',29,'',100 from dual union all
select 69683,'2013-S79','2013-09-22 10:01:00','military missile','success',799,'','Mk 12/21 RV','GT209GM','GLORY TRIP 209GM','Op Test',1706,'',1300 from dual union all
select 69684,'2013-S80','2013-04-01 00:00:00','military missile','success',1443,'DASO-24','-    (144s)','','','OT',1897,'SSBN 735',1000 from dual union all
select 69685,'2013-S81','2013-09-10 21:10:00','military missile','success',1443,'FCET-48','-    (145s)','','','OT',421,'SSBN 734',1650 from dual union all
select 69686,'2013-S82','2013-09-10 21:10:00','military missile','success',1443,'FCET-48','-    (146s)','','','OT',421,'SSBN 734',1000 from dual union all
select 69687,'2013-S83','2013-09-12 00:00:00','military missile','success',1443,'FCET-49','-    (147s)','','','OT',421,'SSBN 734',1000 from dual union all
select 69688,'2013-S84','2013-09-12 00:00:00','military missile','success',1443,'FCET-49','-    (148s)','','','OT',421,'SSBN 734',1000 from dual union all
select 69689,'2013-S85','2013-07-24 00:00:00','military missile','success',357,'DF-31A','','','','Test',1494,'',1000 from dual union all
select 6969,'1960-S309','1960-10-06 03:44:00','military missile','success',1053,'CC-2037','','','','Test',280,'',90 from dual union all
select 69690,'2013-S86','2013-09-26 10:33:00','military missile','success',799,'','Mk 12/21 RV','GT208GM-1','GLORY TRIP 208GM-1','Op Test',1705,'',1300 from dual union all
select 69691,'2013-S87','2013-10-07 03:45:00','military missile','success',942,'','Test','','','Test',315,'',100 from dual union all
select 69692,'2013-S88','2013-10-10 13:39:00','military missile','success',1435,'','15Zh58E GCh-6','RV Test 1','','R&D',636,'',1000 from dual union all
select 69693,'2013-S89','2013-10-04 05:33:00','military missile','success',1348,'Aegis FTM-22 target','ARAV-ER','','','Target',855,'',300 from dual union all
select 69694,'2013-S90','2013-10-04 05:37:00','military missile','success',1091,'Aegis FTM-22','Stellar Raven','','','Interceptor',851,'CG-70',200 from dual union all
select 69695,'2013-S91','2013-10-08 06:50:00','military missile','success',942,'','Test','','','Test',315,'',100 from dual union all
select 69696,'2013-S92','2013-10-21 18:00:00','suborbital rocket','success',224,'NASA 36.290US','SDO EVE Cal','','','Solar UV',1895,'',278 from dual union all
select 69697,'2013-S93','2013-10-30 00:00:00','military missile','success',1434,'','15Zh58 UBP-40','','','OT',608,'',1000 from dual union all
select 69698,'2013-S94','2013-10-30 00:00:00','military missile','success',1005,'','15A18M UBP 13 RD','','','OT',1908,'',1000 from dual union all
select 69699,'2013-S95','2013-10-30 00:00:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-117',1000 from dual union all
select 697,'1947-A33','1947-11-20 23:47:00','military missile','failure',1470,'Special','G.E. Special','','','Test',1863,'',29 from dual union all
select 6970,'1960-S31','1960-02-05 00:19:00','military missile','success',638,'AM-30','R&D','','','Test',280,'',500 from dual union all
select 69700,'2013-S96','2013-10-30 00:00:00','military missile','success',1507,'VMF RF','','','','OT',1011,'K-433',1000 from dual union all
select 69701,'2013-S97','2013-11-03 09:25:00','suborbital rocket','success',224,'NASA 36.294UH','XQC F6','','','XR Astron',1895,'',280 from dual union all
select 69702,'2013-S98','2013-11-12 16:12:00','suborbital rocket','success',1244,'SpaceLoft SL-8','NASA FOP-2','','','Tech',1422,'',116 from dual union all
select 69703,'2013-S99','2013-11-20 11:40:01','suborbital rocket','success',224,'NASA 36.296UG','FORTIS 2 (ISON)','','','UV Astron',1895,'',277 from dual union all
select 69704,'2013-U01','2013-05-13 12:58:00','suborbital rocket','success',354,'Kunpeng-7','','','','Magnetosphere',1905,'',10000 from dual union all
select 69705,'2014-001','2014-01-05 10:48:00','orbital','success',504,'GSLV-D5','GSAT-14','GSAT-14','','Sat',1359,'',null from dual union all
select 69706,'2014-002','2014-01-06 22:06:00','orbital','success',476,'008','Thaicom  6','','','Sat',274,'',null from dual union all
select 69707,'2014-003','2014-01-09 18:07:08','orbital','success',83,'3/70101-401','Orb-1','Cygnus 2','','Sat',944,'',null from dual union all
select 69708,'2014-004','2014-01-24 02:33:00','orbital','success',182,'AV-043','TDRS 12','TDRS L','','Sat',292,'',null from dual union all
select 69709,'2014-005','2014-02-05 16:23:33','orbital','success',1241,'E 15000-139 139','Progress M-22M','Progress 7K-TGM No. 422','','Sat ISS 54P',480,'',null from dual union all
select 6971,'1960-S310','1960-10-07 00:00:00','military missile','success',976,'','8K63 UBP-84','','','Test',636,'',402 from dual union all
select 69710,'2014-006','2014-02-06 21:30:00','orbital','success',111,'VA217 (572)','ABS-2/Athena-Fidus','ABS-2/Athena-Fidus','','Sat',346,'',null from dual union all
select 69711,'2014-007','2014-02-14 21:09:03','orbital','success',962,'935-43/99544','Turksat 4A','Turksat 4A','','Sat',588,'',null from dual union all
select 69712,'2014-008','2014-02-21 01:59:00','orbital','success',400,'D4-25 (365)','GPS IIF SV-4','GPS SVN 64','','Sat',290,'',null from dual union all
select 69713,'2014-009','2014-02-27 18:37:00','orbital','success',527,'H-IIA-23','GPM-C','GPM-C','','Sat',1477,'',null from dual union all
select 69714,'2014-010','2014-03-15 23:08:00','orbital','success',962,'935-42/99545','Ekspress AT1/AT2','Ekspress AT1/Exp. AT2','','Sat',588,'',null from dual union all
select 69715,'2014-011','2014-03-22 22:04:00','orbital','success',111,'VA216 (571)','Astra 5B/Amazonas 4A','Astra 5B/Amazonas 4A','','Sat',346,'',null from dual union all
select 69716,'2014-012','2014-03-23 22:54:03','orbital','success',1228,'780-85- 168 010','Glonass-M54','Uragan-M No. 754','','Sat',475,'',null from dual union all
select 69717,'2014-013','2014-03-25 21:17:23','orbital','success',1233,'T 15000-047 047','Soyuz TMA-12M','Soyuz 11F732A47 No. 712','','Sat ISS 38S',480,'',null from dual union all
select 69718,'2014-014','2014-03-31 02:58:00','orbital','success',301,'Y27','SJ-11-06','SJ-11-06','','Sat',813,'',null from dual union all
select 69719,'2014-015','2014-04-03 14:46:30','orbital','success',182,'AV-044','DMSP 5D-13 F19','DMSP 5D-13 F19','','Sat',1775,'',null from dual union all
select 6972,'1960-S311','1960-10-07 15:50:00','military missile','success',1419,'J-3','Mk 4-2 RV','','','Test',239,'',1000 from dual union all
select 69720,'2014-016','2014-04-03 21:02:26','orbital','success',1234,'B 15000-004 004/1038','Sentinel 1A','Sentinel 1A','','Sat VS07',347,'',null from dual union all
select 69721,'2014-017','2014-04-04 11:44:00','orbital','success',902,'PSLV-C24','IRNSS-1B','IRNSS-1B','','Sat',1357,'',null from dual union all
select 69722,'2014-018','2014-04-09 15:26:27','orbital','success',1241,'T 15000-141 141','Progress M-23M','Progress 7K-TGM No. 427','','Sat ISS 55P',480,'',null from dual union all
select 69723,'2014-019','2014-04-09 19:06:02','orbital','success',1174,'9','''Ofeq-10','''Ofeq-10','','Sat',1026,'',null from dual union all
select 69724,'2014-020','2014-04-10 17:45:00','orbital','success',194,'AV-045','USA 250','ORION 9?','','Sat',292,'',null from dual union all
select 69725,'2014-021','2014-04-16 16:20:00','orbital','success',1241,'E 15000-143 143','Egyptsat-2','Egyptsat-2','','Sat',562,'',null from dual union all
select 69726,'2014-022','2014-04-18 19:25:22','orbital','success',476,'009','Dragon CRS-3','Dragon C105','','Sat',274,'',null from dual union all
select 69727,'2014-023','2014-04-28 04:25:00','orbital','success',962,'935-46/99548','Luch-5V/Kazsat-3','Luch-5V/Kazsat-3','','Sat',588,'',null from dual union all
select 69728,'2014-024','2014-04-30 01:35:15','orbital','success',1488,'VV03','DZZ-HR','KazEOSat-1','','Sat',349,'',null from dual union all
select 69729,'2014-025','2014-05-06 13:49:35','orbital','success',1227,'760-58- 163 019','Kosmos-2495','Kobal''t-M No. 564','','Sat',475,'',null from dual union all
select 6973,'1960-S312','1960-10-08 14:45:00','suborbital rocket','success',214,'BB-1-11','DRTE-04 Type II aur.','','','Auroral',428,'',116 from dual union all
select 69730,'2014-026','2014-05-17 00:03:00','orbital','success',400,'D4-26 (366)','GPS IIF-6','GPS SVN 67','','Sat',290,'',null from dual union all
select 69731,'2014-027','2014-05-22 13:09:00','orbital','success',182,'AV-046','USA 252','QUASAR 18?','','Sat',292,'',null from dual union all
select 69732,'2014-028','2014-05-23 05:27:54','orbital','success',1064,'630-4393-851/72521','Kosmos-2496/97/98/99','Rodnik','','Sat',460,'',null from dual union all
select 69733,'2014-029','2014-05-24 03:05:14','orbital','success',527,'H-IIA-24','ALOS-2','ALOS-2','','Sat',1477,'',null from dual union all
select 69734,'2014-030','2014-05-26 21:09:59','orbital','success',1542,'SL37/DM-SL-36L','Eutelsat 3B','EUTELSAT 3B','','Sat',860,'ODYSSEY',null from dual union all
select 69735,'2014-031','2014-05-28 19:57:41','orbital','success',1233,'T 15000-049 049','Soyuz TMA-13M','Soyuz 11F732A47 No. 713','','Sat ISS 39S',480,'',null from dual union all
select 69736,'2014-032','2014-06-14 17:16:48','orbital','success',1228,'T 15000-023 023/','Glonass-M55','Uragan-M No. 755','','Sat',475,'',null from dual union all
select 69737,'2014-033','2014-06-19 19:11:11','orbital','success',434,'510-7681-108','Deimos-2','Deimos-2 Cluster','','Sat',1907,'',null from dual union all
select 69738,'2014-034','2014-06-30 04:22:00','orbital','success',900,'PSLV-C23','SPOT-7','SPOT-7','','Sat',1357,'',null from dual union all
select 69739,'2014-035','2014-07-02 09:56:23','orbital','success',411,'D367','OCO-2','OCO-2','','Sat',1724,'',null from dual union all
select 6974,'1960-S313','1960-10-10 00:00:00','military missile','success',928,'A1X-53','','','','Test',244,'',500 from dual union all
select 69740,'2014-036','2014-07-03 12:43:52','orbital','success',1064,'492-9794-555/72522','Gonets-M','Gonets-M No.18/19/20','','Sat',460,'',null from dual union all
select 69741,'2014-037','2014-07-08 15:58:28','orbital','success',1228,'L 15000-011 011/1025','Meteor-M No. 2','Meteor-M No. 2','','Sat',562,'',null from dual union all
select 69742,'2014-038','2014-07-10 18:55:56','orbital','success',1236,'E 15000-005 005/1032','O3b Mission 2','O3b SV3/SV6/SV7/SV8','','Sat VS08',347,'',null from dual union all
select 69743,'2014-039','2014-07-13 16:52:14','orbital','success',83,'4/70102-401','Orb-2','Cygnus 3','','Sat',944,'',null from dual union all
select 69744,'2014-040','2014-07-14 15:15:00','orbital','success',476,'010','Orbcomm OG2 Mission 1','Orbcomm OG2','','Sat',274,'',null from dual union all
select 69745,'2014-041','2014-07-18 20:50:00','orbital','success',1227,'T 15000-020 020','Foton-M No. 4','Foton-M No. 4','','Sat',562,'',null from dual union all
select 69746,'2014-042','2014-07-23 21:44:44','orbital','success',1241,'T 15000-140 140','Progress M-24M','Progress 7K-TGM No. 423','','Sat ISS 56P',480,'',null from dual union all
select 69747,'2014-043','2014-07-28 23:28:00','orbital','success',400,'D4-27 (368)','AFSPC-4','GSSAP-1/2/ANGELS','','Sat',290,'',null from dual union all
select 69748,'2014-044','2014-07-29 23:47:38','orbital','success',113,'VA219 (593)','ATV 5','ATV 5','','Sat',346,'',null from dual union all
select 69749,'2014-045','2014-08-02 03:23:00','orbital','success',182,'AV-048','GPS II F-7','GPS SVN 68','','Sat',292,'',null from dual union all
select 6975,'1960-S314','1960-10-11 19:15:00','military missile','failure',155,'3E','Mk 4-1','R&D','','CR',228,'',100 from dual union all
select 69750,'2014-046','2014-08-05 08:00:00','orbital','success',476,'011','Asiasat 8','Asiasat 8','','Sat',274,'',null from dual union all
select 69751,'2014-047','2014-08-09 05:45:03','orbital','success',320,'Y14','Yaogan 20','YW20','','Sat',813,'',null from dual union all
select 69752,'2014-048','2014-08-13 18:30:30','orbital','success',182,'AV-047','Worldview-3','Worldview-3','','Sat',1775,'',null from dual union all
select 69753,'2014-049','2014-08-19 03:15:04','orbital','success',319,'Y27','Gaofen 2','GF-2','','Sat',1493,'',null from dual union all
select 69754,'2014-050','2014-08-22 12:27:11','orbital','failure',1236,'E 15000-006 006/1039','Galileo FOC 1/2','GalileoSat 5-6','','Sat VS09',347,'',null from dual union all
select 69755,'2014-051','2014-09-04 00:15:04','orbital','success',304,'Y25','CX-1-04/LQ','CX-1-04/LQ','','Sat',813,'',null from dual union all
select 69756,'2014-052','2014-09-07 05:00:00','orbital','success',476,'013','Asiasat 6','Asiasat 6/Thaicom','','Sat',274,'',null from dual union all
select 69757,'2014-053','2014-09-08 03:22:05','orbital','success',319,'Y28','Yaogan 21','YG-21/TT-1 No. 2','','Sat',1493,'',null from dual union all
select 69758,'2014-054','2014-09-11 22:05:00','orbital','success',111,'VA218 (573)','Measat 3B/Optus 10','Measat 3B/Optus 10','','Sat',346,'',null from dual union all
select 69759,'2014-055','2014-09-17 00:10:00','orbital','success',182,'AV-049','CLIO','CLIO','','Sat',292,'',null from dual;
insert into LAUNCH
select 6976,'1960-S315','1960-10-11 21:53:00','military missile','success',1367,'186','CTL','','LEFT RUDDER','OT',1677,'',520 from dual union all
select 69760,'2014-056','2014-09-21 05:52:03','orbital','success',476,'012','Dragon CRS-4','Dragon C106','','Sat',274,'',null from dual union all
select 69761,'2014-057','2014-09-25 20:25:00','orbital','success',1233,'T 15000-050 050','Soyuz TMA-14M','Soyuz 11F732A47 No. 714','','Sat ISS 40S',480,'',null from dual union all
select 69762,'2014-058','2014-09-27 20:23:00','orbital','success',962,'935-47/99549','Luch','Olymp-K','','Sat',588,'',null from dual union all
select 69763,'2014-059','2014-09-28 05:13:00','orbital','success',301,'Y31','SJ-11-07','SJ-11-07','','Sat',813,'',null from dual union all
select 69764,'2014-060','2014-10-07 05:16:00','orbital','success',527,'H-IIA-25','Himawari 8','Himawari 8','','Sat',1477,'',null from dual union all
select 69765,'2014-061','2014-10-15 20:02:00','orbital','success',902,'PSLV-C26','IRNSS-1C','IRNSS-1C','','Sat',1357,'',null from dual union all
select 69766,'2014-062','2014-10-16 21:43:52','orbital','success',111,'VA220 (574)','IS-30/Arsat-1','IS-30/Arsat 1','','Sat',346,'',null from dual union all
select 69767,'2014-063','2014-10-20 06:31:00','orbital','success',320,'Y16','Yaogan 22','YG-22','','Sat',1493,'',null from dual union all
select 69768,'2014-064','2014-10-21 15:09:32','orbital','success',962,'935-48/99550','Ekspress AM-6','Ekspress AM-6','','Sat',588,'',null from dual union all
select 69769,'2014-065','2014-10-23 18:00:04','deep space','success',315,'Y12','CE-5T1','CE5-T1','','Sat',1903,'',null from dual union all
select 6977,'1960-S316','1960-10-12 00:00:00','suborbital rocket','unknown',673,'','','','','Aeron?',1016,'',300 from dual union all
select 69770,'2014-066','2014-10-27 06:59:03','orbital','success',301,'Y32','SJ-11-08','SJ-11-08','','Sat',813,'',null from dual union all
select 69771,'2014-067','2014-10-29 07:09:43','orbital','success',1227,'G 15000-021 021','Progress M-25M','Progress 7K-TGM No. 424','','Sat ISS 57P',480,'',null from dual union all
select 69772,'2014-068','2014-10-29 17:21:00','orbital','success',182,'AV-050','GPS II F-8','GPS SVN 69','','Sat',292,'',null from dual union all
select 69773,'2014-069','2014-10-30 01:42:52','orbital','success',1226,'760-58- 174 013/1026','Meridian No. 17L','Meridian No. 17L','','Sat',475,'',null from dual union all
select 69774,'2014-070','2014-11-06 07:35:55','orbital','success',434,'510-7681-109','ASNARO-1','ANSARO-1','','Sat',1907,'',null from dual union all
select 69775,'2014-071','2014-11-14 18:53:05','orbital','success',301,'Y35','Yaogan 23','YG-23','','Sat',1493,'',null from dual union all
select 69776,'2014-072','2014-11-20 07:12:04','orbital','success',304,'Y24','Yaogan 24','YG-24','','Sat',813,'',null from dual union all
select 69777,'2014-073','2014-11-21 06:37:08','orbital','success',683,'','Kuaizhou-2','','','Sat',818,'',null from dual union all
select 69778,'2014-074','2014-11-23 21:01:13','orbital','success',1233,'T 15000-051 051','Soyuz TMA-15M','Soyuz 11F732A47 No. 715','','Sat ISS 41S',562,'',null from dual union all
select 69779,'2014-075','2014-11-30 21:52:26','orbital','success',1228,'G 15000-026 026/1044','Kosmos-2501','Glonass-K No. 2','','Sat',475,'',null from dual union all
select 6978,'1960-S317','1960-10-12 16:37:00','suborbital rocket','success',215,'CC-2A-13','','','','Test',428,'',196 from dual union all
select 69780,'2014-076','2014-12-03 04:22:04','deep space','success',527,'H-IIA-26','Hayabusa-2','Hayabusa-2','','Sat',1477,'',null from dual union all
select 69781,'2014-077','2014-12-05 12:05:00','orbital','success',397,'D4-28 (369)','Orion EFT-1','Orion EFT-1','','Sat',290,'',null from dual union all
select 69782,'2014-078','2014-12-06 20:40:00','orbital','success',111,'VA221 (575)','DirectTV-14/GSAT-16','DirectTV-14/GSAT-16','','Sat',346,'',null from dual union all
select 69783,'2014-079','2014-12-07 03:26:04','orbital','success',319,'Y32','CBERS 4','ZY1-4','','Sat',1493,'',null from dual union all
select 69784,'2014-080','2014-12-10 19:33:04','orbital','success',320,'Y17','Yaogan 25','YG-25','','Sat',813,'',null from dual union all
select 69785,'2014-081','2014-12-13 03:19:29','orbital','success',193,'AV-051','USA 259','Adv TRUMPET 3?','','Sat',1775,'',null from dual union all
select 69786,'2014-082','2014-12-15 00:16:00','orbital','success',962,'935-50/99551','Yamal-401','Yamal-401','','Sat',588,'',null from dual union all
select 69787,'2014-083','2014-12-18 18:37:00','orbital','success',1236,'T 15000-007 007','O3b Mission 3','O3b SV9/SV10/SV11/SV12','','Sat VS10',347,'',null from dual union all
select 69788,'2014-084','2014-12-19 04:43:32','orbital','success',1265,'630-9793-560','Kondor-E No. 2','Kondor-E No. 2','','Sat',538,'',null from dual union all
select 69789,'2014-085','2014-12-23 05:57:00','orbital','success',79,'71751 (1LM)/85501','IPM','','','Sat',472,'',null from dual union all
select 6979,'1960-S318','1960-10-12 21:27:00','suborbital rocket','success',215,'CC-2A-14','','','','Test',428,'',216 from dual union all
select 69790,'2014-086','2014-12-25 03:01:13','orbital','success',1229,'780-31- 199 005','Kosmos-2502','Lotos-S No. 802','','Sat',475,'',null from dual union all
select 69791,'2014-087','2014-12-26 18:55:50','orbital','success',1229,'L 15000-009 009','Resurs-P No. 2','Resurs-P No. 2','','Sat',562,'',null from dual union all
select 69792,'2014-088','2014-12-27 03:22:04','orbital','success',319,'Y29','Yaogan 26','YG-26','','Sat',1493,'',null from dual union all
select 69793,'2014-089','2014-12-27 21:37:49','orbital','success',962,'935-49/99552','Astra 2G','Astra 2G','','Sat',553,'',null from dual union all
select 69794,'2014-090','2014-12-31 01:02:04','orbital','success',309,'Y24','FY-2-08','FY-2-08','','Sat',1903,'',null from dual union all
select 69795,'2014-A01','2014-01-10 16:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-147/PF03','','','       12:43   /0:20',955,'WK2',22 from dual union all
select 69796,'2014-A02','2014-01-07 04:18:00','military missile','success',942,'','Test','','','Test',315,'',44 from dual union all
select 69797,'2014-A03','2014-01-17 17:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-149/GF28  51','','','Glide  10',955,'WK2',15 from dual union all
select 69798,'2014-A04','2014-01-09 00:00:00','military missile','success',301,'Hypersonic test','[WU-14]','Hypersonic RV','','Test',1494,'',40 from dual union all
select 69799,'2014-A05','2014-03-12 16:00:00','atmospheric rocket','success',472,'27','','','','Test',13,'',32 from dual union all
select 698,'1947-A34','1947-12-09 00:00:00','atmospheric rocket','success',876,'47-16','Nike-47 13R','','','Test',1863,'',10 from dual union all
select 6980,'1960-S319','1960-10-13 09:34:38','military missile','success',154,'71D','RVX-2A S/N 423','D/AIG guidance test CL5','','Test  Range 8119',226,'',936 from dual union all
select 69800,'2014-A06','2014-04-17 00:00:00','atmospheric rocket','success',483,'001','F9R Dev1 F001','','','Test',1428,'',0 from dual union all
select 69801,'2014-A07','2014-05-01 00:00:00','atmospheric rocket','success',483,'002','F9R Dev1 F002','','','Test',1428,'',1 from dual union all
select 69802,'2014-A08','2014-05-08 00:00:00','military missile','success',3,'','Test','','','Op Test',183,'',40 from dual union all
select 69803,'2014-A09','2014-06-17 22:04:00','atmospheric rocket','success',483,'003','F9R Dev1 F003 Fin','','','Test',1428,'',1 from dual union all
select 69804,'2014-A10','2014-06-28 21:05:00','atmospheric rocket','success',687,'LDSD SFDT-1','SIAD-R/SSDS','','','Test',855,'Balloon',54 from dual union all
select 69805,'2014-A11','2014-07-02 08:36:00','suborbital rocket','failure',1331,'NASA 46.007GP','SubTec-6','','','Tech',1804,'',8 from dual union all
select 69806,'2014-A12','2014-08-07 00:00:00','military missile','failure',301,'Hypersonic test','[WU-14]','Hypersonic RV','','Test',1494,'',40 from dual union all
select 69807,'2014-A13','2014-08-22 22:45:00','atmospheric rocket','failure',483,'004','F9R Dev1 F004','3 engine test','','Test',1428,'',1 from dual union all
select 69808,'2014-A14','2014-08-25 08:25:00','military missile','failure',1111,'AHW FT2','AHW/HTT','','','Hypersonic',862,'',1 from dual union all
select 69809,'2014-A15','2014-08-28 00:00:00','atmospheric rocket','success',472,'28','','','','Test',330,'',32 from dual union all
select 6981,'1960-S32','1960-02-09 17:11:00','military missile','success',1368,'259','Special Test','','','Test',236,'',520 from dual union all
select 69810,'2014-A16','2014-06-26 08:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69811,'2014-A17','2014-06-26 08:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69812,'2014-A18','2014-06-26 08:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69813,'2014-A19','2014-07-02 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69814,'2014-A20','2014-07-02 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69815,'2014-A21','2014-07-30 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69816,'2014-A22','2014-07-30 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69817,'2014-A23','2014-07-30 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69818,'2014-A24','2014-07-30 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69819,'2014-A25','2014-08-14 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 6982,'1960-S320','1960-10-15 00:00:00','military missile','success',928,'A1FP-5 (A1E-9)','Op. Test','','','Test',418,'SSBN 599',500 from dual union all
select 69820,'2014-A26','2014-08-14 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69821,'2014-A27','2014-08-14 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69822,'2014-A28','2014-08-14 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69823,'2014-A29','2014-08-14 00:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 69824,'2014-A30','2014-08-21 16:58:00','atmospheric rocket','success',473,'12','','','','Test',13,'',49 from dual union all
select 69825,'2014-A31','2014-07-29 20:49:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-156/GF29','','','Glide  12',955,'WK2',15 from dual union all
select 69826,'2014-A32','2014-08-28 14:53:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-164/CF02','Cold Flow','','Glide  13',955,'WK2',15 from dual union all
select 69827,'2014-A33','2014-09-09 00:00:00','military missile','success',119,'','AST-14','','','Test',1026,'',40 from dual union all
select 69828,'2014-A34','2014-10-07 16:00:00','suborbital spaceplane','success',1246,'VSS Enterprise','WK2-170/GF30','','','Glide  10:30',955,'WK2',15 from dual union all
select 69829,'2014-A35','2014-10-02 12:35:00','atmospheric rocket','success',472,'29','','','','Test',330,'',32 from dual union all
select 6983,'1960-S321','1960-10-15 00:00:00','military missile','success',928,'A1FP-6 (A1E-10)','','','','Test',418,'SSBN 599',500 from dual union all
select 69830,'2014-A36','2014-10-31 17:09:00','suborbital spaceplane','failure',1246,'VSS Enterprise','WK2-173/PF04','','','Accident       /0:09?',955,'WK2',15 from dual union all
select 69831,'2014-A37','2014-11-06 22:06:00','atmospheric rocket','success',1089,'Aegis FTM-25','2-3A','','','Interceptor',851,'DDG-53',10 from dual union all
select 69832,'2014-A38','2014-11-06 22:06:00','atmospheric rocket','success',1089,'Aegis FTM-25','2-3A','','','Interceptor',851,'DDG-53',10 from dual union all
select 69833,'2014-A39','2014-11-14 05:10:00','military missile','success',942,'','Test','','','Test',315,'',45 from dual union all
select 69834,'2014-A40','2014-12-02 00:00:00','military missile','success',301,'Hypersonic test','[WU-14]','Hypersonic RV','','Test',1494,'',40 from dual union all
select 69835,'2014-A41','2014-12-10 13:00:00','atmospheric rocket','success',472,'30','','','','Test',330,'',32 from dual union all
select 69836,'2014-A42','2014-12-17 03:30:00','atmospheric rocket','success',1310,'NASA 12.082CR','Sprint','','','Tech',1895,'',2 from dual union all
select 69837,'2014-A43','2014-11-11 00:00:00','military missile','failure',1435,'Topol''-E','15Zh58E GCh-10','RV Test 5?','','R&D',636,'',10 from dual union all
select 69838,'2014-A44','2014-06-18 00:00:00','military missile','success',1093,'Test','','','','Test',1415,'DDG-53',20 from dual union all
select 69839,'2014-A45','2014-06-19 00:00:00','military missile','success',1093,'Test','','','','Test',1415,'DDG-53',20 from dual union all
select 6984,'1960-S322','1960-10-16 00:00:00','military missile','success',928,'A1FP-7 (A1E-11)','','','','Test',418,'SSBN 599',500 from dual union all
select 69840,'2014-A46','2014-06-20 00:00:00','military missile','success',1093,'Test','','','','Test',1415,'DDG-53',20 from dual union all
select 69841,'2014-A47','2014-06-20 00:00:00','military missile','success',1093,'Test','','','','Test',1415,'DDG-53',20 from dual union all
select 69842,'2014-A48','2014-08-14 00:00:00','military missile','success',1093,'Test FOT&E','','','','Test',1895,'',20 from dual union all
select 69843,'2014-A49','2014-08-12 00:00:00','military missile','failure',882,'','14Ts033 LKI-1','','','Test',608,'',0 from dual union all
select 69844,'2014-F01','2014-05-15 21:42:00','orbital','failure',962,'935-45/99547','Ekspress AM-4R','Ekspress AM-4R','','Sat',553,'',160 from dual union all
select 69845,'2014-F02','2014-10-28 22:22:30','orbital','failure',84,'5/2TRS2S1.5','Orb-3','Cygnus 4','','Sat',944,'',0 from dual union all
select 69846,'2014-M01','2014-05-09 18:08:00','atmospheric rocket','success',473,'11','','Op. Aguia I','','Test',13,'',52 from dual union all
select 69847,'2014-M02','2014-04-22 00:00:00','military missile','success',510,'Hatf III','','','','Test',1419,'',50 from dual union all
select 69848,'2014-M03','2014-05-08 00:00:00','military missile','success',510,'Hatf III','','','','Test',1419,'',50 from dual union all
select 69849,'2014-M04','2014-11-14 14:10:00','military missile','success',430,'','Test','','','Test',320,'INS-OPV',50 from dual;
insert into LAUNCH
select 6985,'1960-S323','1960-10-17 00:00:00','military missile','success',976,'','8K63 UBP-86','','','Test',636,'',402 from dual union all
select 69850,'2014-M05','2014-10-07 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 69851,'2014-M06','2014-10-08 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 69852,'2014-M07','2014-10-09 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 69853,'2014-M08','2014-10-10 00:00:00','suborbital rocket','success',1023,'','Meteo Chaff','','','Meteo',1467,'',70 from dual union all
select 69854,'2014-S01','2014-01-03 06:00:00','military missile','success',120,'A3FO-2','Hetz-3','','','Test',1026,'',100 from dual union all
select 69855,'2014-S02','2014-01-15 09:09:00','suborbital rocket','success',1334,'FTX-18 Target','ARAV-A','','','Target',1809,'',130 from dual union all
select 69856,'2014-S03','2014-01-15 09:09:20','suborbital rocket','success',1334,'FTX-18 Target','ARAV-A','','','Target',1805,'',130 from dual union all
select 69857,'2014-S04','2014-01-15 09:09:40','suborbital rocket','success',1334,'FTX-18 Target','ARAV-A','','','Target',1804,'',130 from dual union all
select 69858,'2014-S05','2014-01-20 05:22:00','military missile','success',66,'Agni II Prime','Agni RV','','','Test',316,'',900 from dual union all
select 69859,'2014-S06','2014-03-03 11:09:51','suborbital rocket','success',222,'NASA 36.287UE','GREECE','','','Aurora',1043,'',335 from dual union all
select 6986,'1960-S324','1960-10-17 21:04:00','suborbital rocket','success',839,'AA6.340','UM density/pressure','','','Aeron',428,'',132 from dual union all
select 69860,'2014-S07','2014-03-04 18:10:00','military missile','success',1435,'Topol''-E','15Zh58E GCh-8','RV Test 3','','R&D',636,'',1000 from dual union all
select 69861,'2014-S08','2014-02-27 08:42:00','military missile','success',579,'','','','','Test',859,'',80 from dual union all
select 69862,'2014-S09','2014-02-27 08:42:00','military missile','success',579,'','','','','Test',859,'',80 from dual union all
select 69863,'2014-S10','2014-02-27 08:42:00','military missile','success',579,'','','','','Test',859,'',80 from dual union all
select 69864,'2014-S11','2014-02-27 08:42:00','military missile','success',579,'','','','','Test',859,'',80 from dual union all
select 69865,'2014-S12','2014-03-02 21:19:00','military missile','success',580,'Scud C','','','','Test',1830,'',150 from dual union all
select 69866,'2014-S13','2014-03-02 21:19:00','military missile','success',580,'Scud C','','','','Test',1830,'',150 from dual union all
select 69867,'2014-S14','2014-03-25 17:35:00','military missile','success',586,'Rodong','','','','Test',1448,'',150 from dual union all
select 69868,'2014-S15','2014-03-25 17:42:00','military missile','success',586,'Rodong','','','','Test',1448,'',150 from dual union all
select 69869,'2014-S16','2014-03-23 00:00:00','military missile','success',594,'','','','','Test',30,'',150 from dual union all
select 6987,'1960-S325','1960-10-18 00:00:00','military missile','success',928,'A1FP-8 (A1E-12)','','','','Test',418,'SSBN 599',500 from dual union all
select 69870,'2014-S17','2014-04-14 06:40:00','military missile','success',1530,'','15Zh55M K-1','RV x 4?','','OT',608,'',1000 from dual union all
select 69871,'2014-S18','2014-03-26 22:25:00','suborbital rocket','failure',739,'MN-300','','','','Test',636,'',300 from dual union all
select 69872,'2014-S19','2014-04-27 03:37:00','military missile','success',430,'','Target','','','Target',315,'INS-OPV',120 from dual union all
select 69873,'2014-S20','2014-04-27 03:40:00','military missile','success',942,'','Prithvi Defense Vehicle','PDV','','Interceptor',320,'',120 from dual union all
select 69874,'2014-S21','2014-05-08 00:00:00','military missile','success',1434,'','15Zh58 UBP-41','','','OT',608,'',1000 from dual union all
select 69875,'2014-S22','2014-05-08 00:00:00','military missile','success',1179,'VMF RF','','','','OT',201,'K-114',1000 from dual union all
select 69876,'2014-S23','2014-05-08 00:00:00','military missile','success',1507,'VMF RF','LAYNER','','','OT',1011,'K-223',1000 from dual union all
select 69877,'2014-S24','2014-05-03 08:00:01','suborbital rocket','success',224,'NASA 36.235US','HYPE','','','UV Astron',1895,'',278 from dual union all
select 69878,'2014-S25','2014-05-20 17:08:00','military missile','success',1435,'Topol''-E','15Zh58E GCh-9','RV Test 4','','R&D',636,'',1000 from dual union all
select 69879,'2014-S26','2014-05-21 05:35:00','military missile','success',1091,'Aegis AA CTV-01','','','','Interceptor',841,'',150 from dual union all
select 6988,'1960-S326','1960-10-18 00:00:00','military missile','success',976,'','8K63 UBP-87','','','Test',636,'',402 from dual union all
select 69880,'2014-S27','2014-05-28 11:30:00','suborbital rocket','success',605,'SSC REXUS 16','EURO REXUS 16','','','Tech',417,'',87 from dual union all
select 69881,'2014-S28','2014-05-29 10:00:00','suborbital rocket','success',605,'SSC REXUS 15','EURO REXUS 15','','','Tech',417,'',80 from dual union all
select 69882,'2014-S29','2014-06-02 00:00:00','military missile','success',1443,'DASO-25 D5LE','-    (149s)','','','OT',421,'SSBN 736',1000 from dual union all
select 69883,'2014-S30','2014-06-02 00:00:00','military missile','success',1443,'DASO-25 D5LE','-    (150s)','','','OT',421,'SSBN 736',1000 from dual union all
select 69884,'2014-S31','2014-05-24 07:35:00','suborbital rocket','success',224,'NASA 36.285UG','CHESS','','','UV Astron',1895,'',278 from dual union all
select 69885,'2014-S32','2014-06-22 18:49:00','military missile','success',695,'GBI Target','GMD FTG-06b Target','','','Target',868,'',1000 from dual union all
select 69886,'2014-S33','2014-06-22 18:55:00','military missile','success',497,'GBI','GMD FTG-06b','CE-II EKV','','Interceptor',1709,'',1000 from dual union all
select 69887,'2014-S34','2014-06-26 11:21:00','suborbital rocket','success',1338,'NASA 41.110UO','RockOn 2014','','','Edu',1809,'',118 from dual union all
select 69888,'2014-S35','2014-06-28 19:50:00','military missile','success',580,'Scud C','','','','Test',1828,'',150 from dual union all
select 69889,'2014-S36','2014-06-28 19:58:00','military missile','success',580,'Scud C','','','','Test',1828,'',150 from dual union all
select 6989,'1960-S327','1960-10-18 00:00:00','military missile','success',976,'','8K63 UBP-88','','','Test',636,'',402 from dual union all
select 69890,'2014-S37','2014-07-09 12:00:00','suborbital rocket','success',80,'71601      /82501','Maketa','','','Test',472,'',189 from dual union all
select 69891,'2014-S38','2014-07-12 16:20:00','military missile','success',580,'Scud C','','','','Test',821,'',150 from dual union all
select 69892,'2014-S39','2014-07-12 16:30:00','military missile','success',580,'Scud C','','','','Test',821,'',150 from dual union all
select 69893,'2014-S40','2014-07-23 00:00:00','military missile','success',198,'ABM Target','','','','ABM',819,'',100 from dual union all
select 69894,'2014-S41','2014-07-23 00:00:00','military missile','success',354,'ABM Interceptor','','','','ABM',1629,'',250 from dual union all
select 69895,'2014-S42','2014-07-22 19:10:00','suborbital rocket','success',222,'NASA 36.289US','DFSSP','','','Solar EUV',1895,'',320 from dual union all
select 69896,'2014-S43','2014-08-04 14:00:00','suborbital rocket','success',1073,'S-310-43','','Cryo tank flow test','','Tech',833,'',117 from dual union all
select 69897,'2014-S44','2014-08-17 10:10:00','suborbital rocket','success',1074,'S-520-29','Sporadic E study','','','Ionos',833,'',243 from dual union all
select 69898,'2014-S45','2014-08-23 13:13:00','suborbital rocket','success',1336,'','MDA Shark No. 3','','','Target',1801,'',150 from dual union all
select 69899,'2014-S46','2014-07-08 19:00:00','military missile','success',580,'Scud C','','','','Test',678,'',150 from dual union all
select 699,'1947-A35','1947-12-09 00:00:00','atmospheric rocket','success',876,'47-15','Nike-47 12R','','','Test',1863,'',35 from dual union all
select 6990,'1960-S328','1960-10-18 14:17:00','suborbital rocket','success',607,'NASA 5.02GT','','','','Aeronomy/Test',1797,'',225 from dual union all
select 69900,'2014-S47','2014-07-08 19:20:00','military missile','success',580,'Scud C','','','','Test',678,'',150 from dual union all
select 69901,'2014-S48','2014-08-28 09:00:30','suborbital rocket','success',224,'NASA 36.308GT','Release test','','','Aeron/Tech',1809,'',350 from dual union all
select 69902,'2014-S49','2014-09-02 02:02:00','suborbital rocket','success',1478,'V13','EPL-ME','','','Tech',13,'',130 from dual union all
select 69903,'2014-S50','2014-09-10 00:00:00','military missile','success',261,'','','','','R&D?',190,'K-551',1000 from dual union all
select 69904,'2014-S51','2014-04-11 17:33:00','military missile','success',62,'','Agni RV','','','Test',316,'',300 from dual union all
select 69905,'2014-S52','2014-09-11 05:41:00','military missile','success',62,'','Agni RV','','','Test',316,'',300 from dual union all
select 69906,'2014-S53','2014-09-12 13:00:00','military missile','success',1443,'FCET-50','-    (151s)','','','OT',1338,'SSBN 743',1000 from dual union all
select 69907,'2014-S54','2014-09-12 13:00:00','military missile','success',1443,'FCET-50','-    (152s)','','','OT',1338,'SSBN 743',1000 from dual union all
select 69908,'2014-S55','2014-09-12 13:00:00','military missile','success',1443,'FCET-50','-    (153s)','','','OT',1338,'SSBN 743',1000 from dual union all
select 69909,'2014-S56','2014-09-09 00:00:00','military missile','success',250,'3','Target','','','Test',951,'F-15D',100 from dual union all
select 6991,'1960-S329','1960-10-19 00:00:00','military missile','success',976,'','8K63 UBP-89','','','Test',636,'',402 from dual union all
select 69910,'2014-S57','2014-09-23 14:45:00','military missile','success',799,'','Mk 12/21 RV','GT211GM','GLORY TRIP 211GM','Op Test',1705,'',1300 from dual union all
select 69911,'2014-S58','2014-09-30 18:07:00','suborbital rocket','success',224,'NASA 36.288DS','VAULT 2.0','','','Solar UV',1895,'',290 from dual union all
select 69912,'2014-S59','2014-09-25 00:00:00','military missile','success',357,'DF-31B','','','','Test',1494,'',1000 from dual union all
select 69913,'2014-S60','2014-10-12 04:27:00','suborbital rocket','success',1336,'','MDA Shark No. 4','','','Target',1801,'',150 from dual union all
select 69914,'2014-S61','2014-03-26 13:34:00','suborbital rocket','success',1221,'9','','','','Ionos',808,'',286 from dual union all
select 69915,'2014-S62','2014-10-07 03:10:00','suborbital rocket','success',1221,'10','','','','Ionos',808,'',286 from dual union all
select 69916,'2014-S63','2014-10-17 07:08:00','military missile','success',747,'Aegis FTX-20 Target','Unknown','Pacific Wildcat','','Target',855,'',300 from dual union all
select 69917,'2014-S64','2014-10-23 13:33:00','suborbital rocket','success',1244,'SpaceLoft SL-9','NASA FOP-3','Celestis Conestoga','','Tech',1422,'',124 from dual union all
select 69918,'2014-S65','2014-07-14 14:32:00','suborbital rocket','success',271,'','GoFast','','','Test',208,'',118 from dual union all
select 69919,'2014-S66','2014-10-29 17:27:00','military missile','success',261,'','','','','R&D?',201,'K-535',1000 from dual union all
select 6992,'1960-S33','1960-02-10 00:00:00','military missile','success',928,'A1X-11','','','','Test',251,'',500 from dual union all
select 69920,'2014-S67','2014-11-01 06:20:00','military missile','success',1436,'','15Zh65 UBP-1','','','OT',608,'',1000 from dual union all
select 69921,'2014-S68','2014-11-05 00:00:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-114',1000 from dual union all
select 69922,'2014-S69','2014-11-06 22:03:00','military missile','success',1344,'Aegis FTM-25 Target','ARAV-B','','','Target',855,'',150 from dual union all
select 69923,'2014-S70','2014-11-06 22:06:00','military missile','success',1091,'Aegis FTM-25','Stellar Wyvern','','','Interceptor',851,'DDG-53',150 from dual union all
select 69924,'2014-S71','2014-11-06 19:07:00','suborbital rocket','success',224,'NASA 36.253US','RAISE 2','','','Solar UV',1895,'',300 from dual union all
select 69925,'2014-S72','2014-11-13 00:00:00','military missile','success',1171,'Hatf-VI','','','','Test',1419,'',300 from dual union all
select 69926,'2014-S73','2014-11-17 00:00:00','military missile','success',1170,'Hatf IV','','','','Test',1419,'',100 from dual union all
select 69927,'2014-S74','2014-11-28 00:00:00','military missile','success',261,'','','','','R&D?',201,'K-550',1000 from dual union all
select 69928,'2014-S75','2014-11-09 04:10:00','military missile','success',63,'','Agni RV','','','Test',316,'',300 from dual union all
select 69929,'2014-S76','2014-12-02 04:49:00','military missile','success',66,'','Agni RV','','','Test',316,'',500 from dual union all
select 6993,'1960-S330','1960-10-20 00:00:00','military missile','success',976,'','8K63 UBP-90','','','Test',636,'',402 from dual union all
select 69930,'2014-S77','2014-12-11 19:11:00','suborbital rocket','success',224,'NASA 36.295US','FOXSI','','','Solar X',1895,'',338 from dual union all
select 69931,'2014-S78','2014-11-24 08:05:00','suborbital rocket','success',238,'NASA 52.001UE','C-REX','','','Plasma',28,'',486 from dual union all
select 69932,'2014-S79','2014-12-18 04:00:00','suborbital rocket','success',505,'LVM3-X','CARE','CARE/LVM3-X','','Sat',1359,'',125 from dual union all
select 69933,'2014-S80','2014-12-16 00:00:00','military missile','success',1183,'','Target','','','Test',951,'F-15D',100 from dual union all
select 69934,'2014-S81','2014-12-13 00:00:00','military missile','success',360,'','','','','Test',1494,'',1000 from dual union all
select 69935,'2014-S82','2014-12-26 08:02:00','military missile','success',1530,'','15Zh55M K-2','RV x 4? Road-Mobile','','OT',608,'',1000 from dual union all
select 69936,'2014-S83','2014-12-17 04:00:00','suborbital rocket','success',605,'NASA 30.076DR','CTREX','','','Tech',1895,'',90 from dual union all
select 69937,'2014-S86','2014-07-26 12:40:00','military missile','success',580,'Scud C','','','','Test',678,'',150 from dual union all
select 69938,'2014-S87','2014-05-24 00:00:00','suborbital rocket','success',1348,'','Polar Bear','','','Target?',855,'',200 from dual union all
select 69939,'2015-001','2015-01-10 09:47:10','orbital','success',476,'014','Dragon CRS-5','Dragon C107','','Sat',274,'',null from dual;
insert into LAUNCH
select 6994,'1960-S331','1960-10-20 16:02:00','military missile','success',638,'CM-217','LST','','','Test',248,'',500 from dual union all
select 69940,'2015-002','2015-01-21 01:04:00','orbital','success',195,'AV-052','MUOS 3','MUOS 3','','Sat',292,'',null from dual union all
select 69941,'2015-003','2015-01-31 14:22:00','orbital','success',411,'D370','SMAP','SMAP','','Sat',1724,'',null from dual union all
select 69942,'2015-004','2015-02-01 01:21:00','orbital','success',527,'H-IIA-27','IGS R-Spare','IGS Radar-Spare','','Sat',1477,'',null from dual union all
select 69943,'2015-005','2015-02-01 12:31:00','orbital','success',962,'935-51/99554','Inmarsat 5F2','Inmarsat 5F2','','Sat',553,'',null from dual union all
select 69944,'2015-006','2015-02-02 08:52:00','orbital','success',1114,'LBS.2001','Fajr','Fajr','','Sat',1347,'',null from dual union all
select 69945,'2015-007','2015-02-11 23:03:42','deep space','success',476,'015','DSCOVR','DSCOVR','','Sat',274,'',null from dual union all
select 69946,'2015-008','2015-02-17 11:00:17','orbital','success',1241,'T 15000-144 144','Progress M-26M','Progress 7K-TGM No. 425','','Sat ISS 58P',480,'',null from dual union all
select 69947,'2015-009','2015-02-27 11:01:36','orbital','success',1227,'770-46- 243 010','Kosmos-2503','Bars-M No. 1L','','Sat',475,'',null from dual union all
select 69948,'2015-010','2015-03-02 03:50:00','orbital','success',476,'016','ABS-3A/Eutelsat115WB','ABS-3A/Eutelsat115WB','','Sat',274,'',null from dual union all
select 69949,'2015-011','2015-03-13 02:44:00','orbital','success',186,'AV-053','MMS','MMS 1-4','','Sat',292,'',null from dual union all
select 6995,'1960-S332','1960-10-20 19:50:00','suborbital rocket','success',214,'BB-1-10','DRTE-03','','','Ionos/Plasma',428,'',137 from dual union all
select 69950,'2015-012','2015-03-18 22:05:00','orbital','success',962,'935-52/99553','Ekspress AM-7','Ekspress AM-7','','Sat',553,'',null from dual union all
select 69951,'2015-013','2015-03-25 18:36:00','orbital','success',400,'D4-29 (371)','GPS IIF-7','GPS SVN 71','','Sat',290,'',null from dual union all
select 69952,'2015-014','2015-03-25 22:08:53','orbital','success',434,'510-8681-101','Arirang-3A','Kompsat-3A','','Sat',1907,'',null from dual union all
select 69953,'2015-015','2015-03-26 01:21:00','orbital','success',527,'H-IIA-28','IGS O-5','IGS Optical-5','','Sat',1477,'',null from dual union all
select 69954,'2015-016','2015-03-27 19:42:57','orbital','success',1233,'G 15000-053 053','Soyuz TMA-16M','Soyuz 11F732A47 No.716','','Sat ISS 42S',480,'',null from dual union all
select 69955,'2015-017','2015-03-27 21:46:18','orbital','success',1236,'T 15000-008 008','Galileo FOC 3/4','GalileoSat 7-8','','Sat VS11',347,'',null from dual union all
select 69956,'2015-018','2015-03-28 11:49:00','orbital','success',902,'PSLV-C27','IRNSS-1D','IRNSS-1D','','Sat',1357,'',null from dual union all
select 69957,'2015-019','2015-03-30 13:52:31','orbital','success',317,'Y11','Beidou DW17','Beidou I1-S','','Sat',1903,'',null from dual union all
select 69958,'2015-020','2015-03-31 13:47:56','orbital','success',1064,'492-9794-554/72526','Gonets-M/ Kosmos-2504','Gonets-M','','Sat',460,'',null from dual union all
select 69959,'2015-021','2015-04-14 20:10:41','orbital','success',476,'017','Dragon CRS-6','Dragon C108','','Sat',274,'',null from dual union all
select 6996,'1960-S333','1960-10-21 00:00:00','military missile','success',976,'','8K63 UBP-91','','','Test',636,'',402 from dual union all
select 69960,'2015-022','2015-04-26 20:00:07','orbital','success',111,'VA222 (576)','Sicral 2/Thor 7','Sicral 2/Thor 7','','Sat',346,'',null from dual union all
select 69961,'2015-023','2015-04-27 23:03:00','orbital','success',476,'018','TurkmenAlem52E/MonacoSat','Turkmensat','','Sat',274,'',null from dual union all
select 69962,'2015-024','2015-04-28 07:09:50','orbital','failure',1227,'G 15000-022 022','Progress M-27M','Progress 7K-TGM No. 426','','Sat ISS 59P',562,'',null from dual union all
select 69963,'2015-025','2015-05-20 15:05:00','orbital','success',189,'AV-054','X-37B OTV FLT-4','X-37B No. 2?','','Sat',292,'',null from dual union all
select 69964,'2015-026','2015-05-27 21:16:07','orbital','success',111,'VA223 (577)','DirecTV-15/SKYM-1','DirecTV-15/SKYM-1','','Sat',346,'',null from dual union all
select 69965,'2015-027','2015-06-05 15:23:54','orbital','success',1227,'780-72- 171 024','Kosmos-2505','Kobal''t-M No. 565','','Sat',475,'',null from dual union all
select 69966,'2015-028','2015-06-23 01:51:58','orbital','success',1487,'VV05','Sentinel-2A','Sentinel-2A','','Sat',349,'',null from dual union all
select 69967,'2015-029','2015-06-23 16:44:00','orbital','success',1229,'760-58- 156 012','Kosmos-2506','Persona No. 3','','Sat',475,'',null from dual union all
select 69968,'2015-030','2015-06-26 06:22:04','orbital','success',319,'Y30','Gaofen 8','GF-8','','Sat',1493,'',null from dual union all
select 69969,'2015-031','2015-07-03 04:55:45','orbital','success',1241,'T 15000-142 142','Progress M-28M','Progress 7K-TGM No. 428','','Sat ISS 60P',480,'',null from dual union all
select 6997,'1960-S334','1960-10-22 03:09:00','suborbital rocket','success',1438,'TB Ie','D58 Trailblazer','','','Reentry Test',1821,'',260 from dual union all
select 69970,'2015-032','2015-07-10 16:28:00','orbital','success',902,'PSLV-C28','DMC3','DMC3','','Sat',1357,'',null from dual union all
select 69971,'2015-033','2015-07-15 15:36:00','orbital','success',181,'AV-055','GPS II F-10','GPS SVN 72','','Sat',292,'',null from dual union all
select 69972,'2015-034','2015-07-15 21:42:00','orbital','success',111,'VA224 (578)','Star One C4/MSG 4','Star One C4/MSG 4','','Sat',346,'',null from dual union all
select 69973,'2015-035','2015-07-22 21:02:45','orbital','success',1233,'G 15000-052 052','Soyuz TMA-17M','Soyuz 11F732A47 No. 717','','Sat ISS 43S',480,'',null from dual union all
select 69974,'2015-036','2015-07-24 00:07:00','orbital','success',403,'D4-30 (372)','WGS 7','Wideband Global Satcom 7','','Sat',290,'',null from dual union all
select 69975,'2015-037','2015-07-25 12:29:00','orbital','success',314,'Y26','Beidou 18/19','Beidou M1S?M2S','','Sat',1903,'',null from dual union all
select 69976,'2015-038','2015-08-19 11:50:49','orbital','success',534,'H-IIB-F5','HTV-5','HTV 5','','Sat',1478,'',null from dual union all
select 69977,'2015-039','2015-08-20 20:34:08','orbital','success',111,'VA225 (579)','Eutelsat 8WB/IS-34','Eutelsat 8WB/IS-34','','Sat',346,'',null from dual union all
select 69978,'2015-040','2015-08-27 02:31:35','orbital','success',320,'Y18','Yaogan 27','YG-27','','Sat',1493,'',null from dual union all
select 69979,'2015-041','2015-08-27 11:22:00','orbital','success',504,'GSLV-D6','GSAT-6','GSAT-6','','Sat',1359,'',null from dual union all
select 6998,'1960-S335','1960-10-22 05:13:20','military missile','success',154,'55D','Mk 3-2B-222, LMSC radm','D/RIG Long range test','','Test',229,'',1190 from dual union all
select 69980,'2015-042','2015-08-28 11:44:00','orbital','success',962,'935-55/99556','Inmarsat 5F3','Inmarsat 5F3','','Sat',553,'',null from dual union all
select 69981,'2015-043','2015-09-02 04:37:43','orbital','success',1233,'G 15000-054 054','Soyuz TMA-18M','Soyuz 11F732A47 No. 718','','Sat ISS 44S',480,'',null from dual union all
select 69982,'2015-044','2015-09-02 10:18:00','orbital','success',195,'AV-056','MUOS 4','MUOS 4','','Sat',292,'',null from dual union all
select 69983,'2015-045','2015-09-11 02:08:10','orbital','success',1236,'T 15000-009 009','Galileo FOC 5/6','GalileoSat 9-10','','Sat VS12',347,'',null from dual union all
select 69984,'2015-046','2015-09-12 15:42:04','orbital','success',311,'Y32','Tongxing Jisshu Shiyan 1','TXJSSY-1','','Sat',1903,'',null from dual union all
select 69985,'2015-047','2015-09-14 04:42:00','orbital','success',304,'Y21','Gaofen 9','GF-9','','Sat',813,'',null from dual union all
select 69986,'2015-048','2015-09-14 19:00:00','orbital','success',964,'935-53/5L','Ekspress AM-8','Ekspress AM-8','','Sat',588,'',null from dual union all
select 69987,'2015-049','2015-09-19 23:01:14','orbital','success',324,'Y1','Cluster','','','Sat',1491,'',null from dual union all
select 69988,'2015-050','2015-09-23 21:59:38','orbital','success',1064,'511-2361-274/72512','Kosmos-2507/08/09','Rodnik','','Sat',460,'',null from dual union all
select 69989,'2015-051','2015-09-25 01:41:40','orbital','success',294,'Y1','TW-1','TW-1?','','Sat',818,'',null from dual union all
select 6999,'1960-S336','1960-10-24 23:16:00','military missile','success',1419,'J-6','Mk 4-1 RV','','','Test',237,'',1000 from dual union all
select 69990,'2015-052','2015-09-28 04:30:00','orbital','success',902,'PSLV-C30','ASTROSAT','ASTROSAT','','Sat',1357,'',null from dual union all
select 69991,'2015-053','2015-09-29 23:13:00','orbital','success',311,'Y33','Beidou DW20','Beidou I2-S','','Sat',1904,'',null from dual union all
select 69992,'2015-054','2015-09-30 16:30:07','orbital','success',111,'VA226 (580)','Sky Muster/ARSAT-2','NBN Co.1/ARSAT 2','','Sat',346,'',null from dual union all
select 69993,'2015-055','2015-10-01 16:49:40','orbital','success',1241,'G 15000-146 146','Progress M-29M','Progress 7K-TGM No. 429','','Sat ISS 61P',480,'',null from dual union all
select 69994,'2015-056','2015-10-02 10:28:30','orbital','success',185,'AV-059','Morelos 3','Morelos 3','','Sat',292,'',null from dual union all
select 69995,'2015-057','2015-10-07 04:13:05','orbital','success',304,'Y37','Jilin-1','Jilin-1','','Sat',813,'',null from dual union all
select 69996,'2015-058','2015-10-08 12:49:00','orbital','success',181,'AV-058','NROL-55','INTRUDER','','Sat',1775,'',null from dual union all
select 69997,'2015-059','2015-10-16 16:16:00','orbital','success',311,'Y36','Apstar-9','Apstar-9','','Sat',1903,'',null from dual union all
select 69998,'2015-060','2015-10-16 20:40:11','orbital','success',962,'935-56/99557','Turksat 4B','Turksat 4B','','Sat',553,'',null from dual union all
select 69999,'2015-061','2015-10-26 07:10:00','orbital','success',304,'Y26','Tianhui 1-03','tianhui yi hao 03 xing','','Sat',813,'',null from dual union all
select 7,'1942-S01','1942-10-03 14:58:00','military missile','success',8,'4','','','','Test',675,'',85 from dual union all
select 70,'1944-A34','1944-12-01 00:00:00','military missile','success',943,'1','','','','Test',764,'',5 from dual union all
select 700,'1947-M01','1947-01-24 00:22:00','suborbital rocket','success',1470,'19','','','','Test',1863,'',50 from dual union all
select 7000,'1960-S337','1960-10-27 00:00:00','suborbital rocket','unknown',673,'','','','','Aeron?',1016,'',300 from dual union all
select 70000,'2015-062','2015-10-31 16:13:00','orbital','success',181,'AV-060','GPS II F-11','GPS SVN 73','','Sat',292,'',null from dual union all
select 70001,'2015-063','2015-11-03 16:25:04','orbital','success',312,'Y34','Zhongxing 2C','zhongxing er hao C xing','','Sat',1904,'',null from dual union all
select 70002,'2015-064','2015-11-08 07:06:04','orbital','success',319,'Y24','Yaogan 28','YG-28','','Sat',1493,'',null from dual union all
select 70003,'2015-065','2015-11-10 21:34:00','orbital','success',111,'VA227 (581)','Arabsat 6B/GSAT 15','Badr 7/GSAT 15','','Sat',346,'',null from dual union all
select 70004,'2015-066','2015-11-17 06:33:41','orbital','success',1228,'760-12- 227 015/1033','Kosmos-2510','EKS No. 1','','Sat',475,'',null from dual union all
select 70005,'2015-067','2015-11-20 16:07:04','orbital','success',311,'Y38','Laosat 1','Laosat 1','','Sat',1903,'',null from dual union all
select 70006,'2015-068','2015-11-24 06:50:00','orbital','success',532,'H-IIA-29','Telstar 12V','Telstar 12V','','Sat',1477,'',null from dual union all
select 70007,'2015-069','2015-11-26 21:24:05','orbital','success',320,'Y8','Yaogan 29','YG-29','','Sat',1493,'',null from dual union all
select 70008,'2015-070','2015-12-03 04:04:01','deep space','success',1487,'VV06','LISA Pathfinder','LPF','','Sat',349,'',null from dual union all
select 70009,'2015-071','2015-12-05 14:08:33','orbital','failure',1230,'780-31- 001 001/002','Kosmos-2511/2512','Kanopus-ST/KYuA-1','','Sat',475,'',null from dual union all
select 7001,'1960-S338','1960-10-27 00:00:00','military missile','success',976,'','8K63 UBP-92','','','Test',636,'',402 from dual union all
select 70010,'2015-072','2015-12-06 21:44:57','orbital','success',181,'AV-061','Cygnus OA-4','Cygnus OA-4','','Sat',292,'',null from dual union all
select 70011,'2015-073','2015-12-09 16:46:00','orbital','success',311,'Y31','Zhongxing 1C','zhongxing yi hao C xing','','Sat',1904,'',null from dual union all
select 70012,'2015-074','2015-12-11 13:45:32','orbital','success',1544,'SB80.3/402/2004','Elektro-L No. 2','Elektro-L No. 2','','Sat',567,'',null from dual union all
select 70013,'2015-075','2015-12-13 00:19:00','orbital','success',962,'535-44/88533','Kosmos-2513','Garpun No. 12L','','Sat',588,'',null from dual union all
select 70014,'2015-076','2015-12-15 11:03:10','orbital','success',1233,'G 15000-055 055','Soyuz TMA-19M','Soyuz 11F732A47 No. 719','','Sat ISS 45S',480,'',null from dual union all
select 70015,'2015-077','2015-12-16 12:30:00','orbital','success',900,'PSLV-C29','TeLEOS-1','TeLEOS-1','','Sat',1357,'',null from dual union all
select 70016,'2015-078','2015-12-17 00:12:04','orbital','success',304,'Y31','Wukong','DAMPE','','Sat',813,'',null from dual union all
select 70017,'2015-079','2015-12-17 11:51:56','orbital','success',1236,'R 15000-010 010','Galileo FOC 8/9','GalileoSat 11-12','','Sat VS13',347,'',null from dual union all
select 70018,'2015-080','2015-12-21 08:44:39','orbital','success',1227,'R 15000-026 026','Progress MS-01','Progress 7K-TGM No. 431','','Sat ISS 62P',562,'',null from dual union all
select 70019,'2015-081','2015-12-22 01:29:00','orbital','success',481,'021/B1019','Orbcomm OG2 Mission 2','Orbcomm OG2','','Sat',274,'',null from dual union all
select 7002,'1960-S339','1960-10-28 12:21:00','suborbital rocket','success',214,'BB-1-12','DRTE-05','','','Ionos/Plasma',428,'',150 from dual union all
select 70020,'2015-082','2015-12-24 21:31:19','orbital','success',962,'935-57/99559','Ekspress-AMU1','Ekspress-AMU1','','Sat',553,'',null from dual union all
select 70021,'2015-083','2015-12-28 16:04:00','orbital','success',311,'Y37','Gaofen 4','gaofen si yao weixing','','Sat',1903,'',null from dual union all
select 70022,'2015-A01','2015-02-19 03:45:00','military missile','success',942,'','Test','','','Test',315,'',50 from dual union all
select 70023,'2015-A02','2015-02-26 08:00:00','military missile','failure',1454,'511-3922-119','15A35-71 LKI-2','Yu-71 / Proekt 4202','','Test',1908,'',10 from dual union all
select 70024,'2015-A03','2015-05-06 13:00:00','atmospheric rocket','success',449,'1','Pad Abort','','','Test',291,'',1 from dual union all
select 70025,'2015-A04','2015-05-08 00:00:00','atmospheric rocket','success',259,'','Bukgeukseong','','','Test',1406,'KPN PLAT',1 from dual union all
select 70026,'2015-A05','2015-05-21 19:15:00','suborbital rocket','failure',224,'NASA 36.300US','SDO EVE Cal','','','Solar UV',1873,'',5 from dual union all
select 70027,'2015-A06','2015-06-08 21:35:00','atmospheric rocket','success',687,'LDSD SFDT-2','SIAD-R/SSRS','','','Test',855,'Balloon',54 from dual;
insert into LAUNCH
select 70028,'2015-A07','2015-06-09 08:32:00','military missile','success',3,'','Test','','','Op Test',183,'',40 from dual union all
select 70029,'2015-A08','2015-06-07 00:00:00','military missile','success',301,'Hypersonic test','[WU-14]','Hypersonic RV','','Test',1494,'',40 from dual union all
select 7003,'1960-S34','1960-02-12 04:11:00','military missile','success',154,'49D','Mk 3-1A','R&D','','Test',228,'',1800 from dual union all
select 70030,'2015-A09','2015-06-26 00:00:00','military missile','failure',601,'Aegis FTO-02E1','Aegis Ashore Target','MBRV-7?','','Target',1275,'C-17',0 from dual union all
select 70031,'2015-A10','2015-07-29 08:32:00','military missile','success',1093,'Aegis MMW E1','','','','Interceptor',851,'DDG-53',20 from dual union all
select 70032,'2015-A11','2015-07-30 06:17:00','military missile','success',1088,'Aegis MMW E2','','','','Interceptor',851,'DDG-53',20 from dual union all
select 70033,'2015-A12','2015-08-01 00:32:00','military missile','success',1088,'Aegis MMW E3','Cruise missile intercept','','','Interceptor',851,'DDG-53',10 from dual union all
select 70034,'2015-A13','2015-08-02 01:47:00','military missile','success',1093,'Aegis MMW E4','Cruise missile intercept','','','Interceptor',851,'DDG-53',10 from dual union all
select 70035,'2015-A14','2015-08-20 00:00:00','military missile','success',301,'Hypersonic test','[WU-14]','Hypersonic RV','','Test',1494,'',40 from dual union all
select 70036,'2015-A15','2015-06-19 00:00:00','atmospheric rocket','success',472,'31','','','','Test',330,'',32 from dual union all
select 70037,'2015-A16','2015-08-27 14:45:00','atmospheric rocket','success',472,'32','','','','Test',13,'',32 from dual union all
select 70038,'2015-A17','2015-01-23 00:00:00','military missile','success',259,'KN-11','Ejection test','','','Test',1406,'',1 from dual union all
select 70039,'2015-A18','2015-10-16 14:33:00','atmospheric rocket','success',1264,'1','','','','Test',396,'',21 from dual union all
select 7004,'1960-S340','1960-11-01 00:00:00','military missile','success',979,'','8K65 LKI-10','','','Test',636,'',675 from dual union all
select 70040,'2015-A19','2015-11-01 03:12:00','military missile','failure',1091,'Aegis FTO-02 E2a','eMRBM intercept','','','Interceptor',1784,'DDG-53',10 from dual union all
select 70041,'2015-A20','2015-10-20 00:00:00','military missile','success',1088,'MTMD Forum ASD15','Aegis','','','Interceptor',1447,'DDG-68',10 from dual union all
select 70042,'2015-A21','2015-11-23 01:00:00','military missile','success',301,'Hypersonic test','[WU-14]','Hypersonic RV','','Test',1494,'',40 from dual union all
select 70043,'2015-A22','2015-12-26 20:00:00','military missile','success',965,'Qahir-1','','','','Weapon',1333,'',20 from dual union all
select 70044,'2015-A23','2015-12-21 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70045,'2015-A24','2015-12-30 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70046,'2015-A25','2015-12-13 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70047,'2015-A26','2015-12-14 00:00:00','military missile','success',1429,'','','','','Weapon',1333,'',40 from dual union all
select 70048,'2015-A27','2015-12-20 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70049,'2015-A28','2015-12-27 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 7005,'1960-S341','1960-11-01 00:00:00','military missile','success',979,'','8K65 LKI-11','','','Test',636,'',675 from dual union all
select 70050,'2015-A29','2015-09-04 00:00:00','military missile','success',1429,'','','','','Weapon',1333,'',40 from dual union all
select 70051,'2015-A30','2015-12-18 00:00:00','military missile','success',1429,'','','','','Weapon',1333,'',40 from dual union all
select 70052,'2015-A31','2015-12-23 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70053,'2015-A32','2015-12-29 00:00:00','military missile','success',965,'','','Aramco target','','Weapon',1333,'',20 from dual union all
select 70054,'2015-A33','2015-12-22 00:00:00','military missile','success',965,'','','Aramco target','','Weapon',1333,'',20 from dual union all
select 70055,'2015-A34','2015-12-20 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70056,'2015-A35','2015-12-24 00:00:00','military missile','success',1429,'','','','','Weapon',1333,'',40 from dual union all
select 70057,'2015-A36','2015-12-18 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70058,'2015-A37','2015-12-14 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70059,'2015-A38','2015-04-22 00:00:00','military missile','failure',882,'','14Ts033 LKI-2','','','Test',608,'',0 from dual union all
select 7006,'1960-S342','1960-11-01 00:00:00','military missile','success',976,'','8K63 UBP-93','','','Test',636,'',402 from dual union all
select 70060,'2015-A39','2015-10-04 00:00:00','military missile','failure',1551,'FTO-02 E2','FTO-02E2 Target','MBRV-7?','','Target',1273,'C-17',10 from dual union all
select 70061,'2015-A40','2015-04-22 00:00:00','military missile','success',259,'KN-11','Ejection test','','','Test',1406,'',1 from dual union all
select 70062,'2015-A41','2015-11-28 00:00:00','military missile','failure',259,'KN-11','Ejection test','','','Test',1406,'KPN PLAT',0 from dual union all
select 70063,'2015-A42','2015-12-21 00:00:00','military missile','failure',259,'KN-11','Ejection test','','','Test',1406,'KPN PLAT',0 from dual union all
select 70064,'2015-A43','2015-01-01 00:00:00','atmospheric rocket','success',641,'','Target','','','Test',33,'',40 from dual union all
select 70065,'2015-A44','2015-02-08 07:10:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 70066,'2015-A45','2015-02-08 07:30:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 70067,'2015-A46','2015-02-08 07:30:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 70068,'2015-A47','2015-02-08 07:30:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 70069,'2015-A48','2015-02-08 08:10:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 7007,'1960-S343','1960-11-02 22:25:00','suborbital rocket','success',839,'NASA 10.09UA','Composition','','','Aeronomy',1821,'',109 from dual union all
select 70070,'2015-A49','2015-04-02 01:30:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 70071,'2015-A50','2015-04-03 07:14:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 70072,'2015-A51','2015-04-03 07:30:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 70073,'2015-A52','2015-04-03 07:30:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 70074,'2015-A53','2015-04-03 08:00:00','military missile','success',648,'','','','','OT',1829,'',50 from dual union all
select 70075,'2015-A54','2015-03-10 00:00:00','military missile','success',702,'Risk Reduction','','','','Target',1895,'',10 from dual union all
select 70076,'2015-E01','2015-11-13 17:20:00','suborbital rocket','destroyed before launch',1483,'V03','SARA Suborbital 1','','','Reentry test',13,'',0 from dual union all
select 70077,'2015-F01','2015-05-16 05:47:39','orbital','failure',962,'935-54/99555','Mexsat-1','Mexsat-1','','Sat',553,'',160 from dual union all
select 70078,'2015-F02','2015-06-28 14:21:11','orbital','failure',476,'020','Dragon CRS-7','Dragon C109','','Sat',274,'',45 from dual union all
select 70079,'2015-F03','2015-11-04 03:45:00','orbital','failure',1285,'1','ORS-4','Hiakasat','','Sat',842,'',30 from dual union all
select 7008,'1960-S344','1960-11-03 21:19:00','suborbital rocket','success',838,'NASA 03.29GT','','','','Test',1821,'',193 from dual union all
select 70080,'2015-M01','2015-03-01 21:33:00','military missile','success',580,'Scud C','','','','Test',985,'',134 from dual union all
select 70081,'2015-M02','2015-03-01 21:41:00','military missile','success',580,'Scud C','','','','Test',985,'',134 from dual union all
select 70082,'2015-M03','2015-04-09 05:32:00','military missile','success',430,'','Test','','','Test',320,'INS-OPV',50 from dual union all
select 70083,'2015-M04','2015-07-15 00:00:00','suborbital rocket','success',1023,'No. 1545','','','','Meteo',1467,'',70 from dual union all
select 70084,'2015-S01','2015-01-26 09:13:01','suborbital rocket','success',1331,'NASA 46.009UE','M-TeX 1','','','Aeron',1041,'',160 from dual union all
select 70085,'2015-S02','2015-01-26 09:14:00','suborbital rocket','success',1334,'NASA 41.111UE','MIST 1','','','Aeron',1042,'',130 from dual union all
select 70086,'2015-S03','2015-01-26 09:46:01','suborbital rocket','success',1331,'NASA 46.010UE','M-TeX 2','','','Aeron',1043,'',160 from dual union all
select 70087,'2015-S04','2015-01-26 09:47:00','suborbital rocket','success',1334,'NASA 41.112UE','MIST 2','','','Aeron',1045,'',130 from dual union all
select 70088,'2015-S05','2015-01-28 10:41:00','suborbital rocket','success',890,'NASA 49.002UE','ASSP','','','Auroral',1044,'',590 from dual union all
select 70089,'2015-S06','2015-01-31 02:36:00','military missile','success',67,'','','','','Test',316,'',800 from dual union all
select 7009,'1960-S345','1960-11-04 00:00:00','military missile','success',976,'','8K63 UBP-94','','','Test',636,'',402 from dual union all
select 70090,'2015-S07','2015-01-23 00:00:00','military missile','success',615,'','','','','Test',205,'XIA',1000 from dual union all
select 70091,'2015-S09','2015-02-24 03:30:00','suborbital rocket','success',1344,'FTX-19 Target','ARAV-B','','','Target',1809,'',150 from dual union all
select 70092,'2015-S10','2015-02-24 03:30:08','suborbital rocket','success',1344,'FTX-19 Target','ARAV-B','','','Target',1804,'',150 from dual union all
select 70093,'2015-S11','2015-02-24 03:30:20','suborbital rocket','success',1344,'FTX-19 Target','ARAV-B','','','Target',1798,'',150 from dual union all
select 70094,'2015-S12','2015-02-25 12:26:00','suborbital rocket','success',224,'NASA 36.299DR','MOSC 2','','','Active plasma',1873,'',200 from dual union all
select 70095,'2015-S13','2015-02-22 00:00:00','military missile','success',1443,'FCET-51','-    (154s)','','','OT',1338,'SSBN 735',1000 from dual union all
select 70096,'2015-S14','2015-02-22 00:00:00','military missile','success',1443,'FCET-51','-    (155s)','','','OT',1338,'SSBN 735',1000 from dual union all
select 70097,'2015-S15','2015-02-19 22:06:00','suborbital rocket','success',1479,'V11','ICI-4','','','Auroral',29,'',365 from dual union all
select 70098,'2015-S16','2015-02-22 07:52:00','suborbital rocket','success',1484,'V20','Cryofenix','','','Tech',416,'',265 from dual union all
select 70099,'2015-S17','2015-03-05 01:44:00','suborbital rocket','success',1478,'V11','WADIS 2','','','Aeron',29,'',115 from dual union all
select 701,'1947-M02','1947-02-24 00:00:00','suborbital rocket','success',1517,'29B','','','','Test',1852,'',73 from dual union all
select 7010,'1960-S346','1960-11-04 00:00:00','military missile','success',976,'','8K63 UBP-95','','','Test',636,'',402 from dual union all
select 70100,'2015-S18','2015-03-09 00:00:00','military missile','success',1172,'','','','','Test',1419,'',500 from dual union all
select 70101,'2015-S19','2015-03-23 10:36:00','military missile','success',799,'','Mk 12/21 RV','GT214GM','GLORY TRIP 214GM','Op Test',1706,'',1300 from dual union all
select 70102,'2015-S20','2015-03-18 00:00:00','military missile','success',1067,'Avangard','15Zh67 LKI-6','','','R&D',636,'',1000 from dual union all
select 70103,'2015-S21','2015-03-17 09:15:00','suborbital rocket','success',605,'SSC REXUS 17','EURO REXUS 17','','','Tech',417,'',81 from dual union all
select 70104,'2015-S22','2015-03-18 13:30:00','suborbital rocket','success',605,'SSC REXUS 18','EURO REXUS 18','','','Tech',417,'',81 from dual union all
select 70105,'2015-S23','2015-03-27 10:53:59','military missile','success',799,'','Mk 12/21 RV','GT215GM','GLORY TRIP 215GM','Op Test',1699,'',1300 from dual union all
select 70106,'2015-S24','2015-04-15 00:00:00','military missile','success',509,'Hatf-5','','','','Test',1473,'',400 from dual union all
select 70107,'2015-S25','2015-04-18 11:01:00','suborbital rocket','success',1331,'NASA 46.008UO','RockSat-X 4','','','Edu',1804,'',154 from dual union all
select 70108,'2015-S26','2015-04-16 04:22:00','military missile','success',65,'','','','','Test',316,'',350 from dual union all
select 70109,'2015-S27','2015-04-23 07:35:00','suborbital rocket','success',1484,'V18','DLR TEXUS 51','','','Micrograv',416,'',261 from dual union all
select 7011,'1960-S347','1960-11-04 00:00:00','military missile','success',976,'','8K63 UBP-96','','','Test',636,'',402 from dual union all
select 70110,'2015-S28','2015-04-27 04:55:00','suborbital rocket','success',1484,'V21','DLR TEXUS 52','','','Micrograv',416,'',255 from dual union all
select 70111,'2015-S29','2015-04-29 15:00:00','suborbital rocket','success',834,'1','New Shepard M1','New Shepard CC','','Test',1896,'',93 from dual union all
select 70112,'2015-S30','2015-05-02 08:30:01','suborbital rocket','success',224,'NASA 36.292UH','OGRESS','','','XR Astron',1873,'',272 from dual union all
select 70113,'2015-S31','2015-05-20 10:37:00','military missile','success',799,'','Mk 12/21 RV','GT212GM','GLORY TRIP 212GM','Op Test',1705,'',1300 from dual union all
select 70114,'2015-S32','2015-06-06 00:00:00','military missile','success',580,'Scud C','','','','Weapon',1333,'',80 from dual union all
select 70115,'2015-S34','2015-06-06 00:00:00','military missile','success',1092,'Aegis SCD CTV-01','','','','Test',1415,'',150 from dual union all
select 70116,'2015-S35','2015-06-25 10:00:00','suborbital rocket','success',1338,'NASA 41.113UO','RockOn 8/RockSat-C 7','','','Edu',1809,'',115 from dual union all
select 70117,'2015-S36','2015-06-30 00:00:00','military missile','success',580,'Scud C','','','','Weapon',1333,'',80 from dual;
insert into LAUNCH
select 70118,'2015-S37','2015-06-03 00:00:00','military missile','success',594,'','','','','Test',30,'',150 from dual union all
select 70119,'2015-S38','2015-07-07 10:15:00','suborbital rocket','success',221,'NASA 36.313NP','SOAREX-8/RCHS','','','Tech',1804,'',331 from dual union all
select 7012,'1960-S348','1960-11-04 20:52:00','suborbital rocket','success',839,'AA6.800','Antenna Breakdown Ex','','','Test',394,'',100 from dual union all
select 70120,'2015-S39','2015-03-30 15:26:00','suborbital rocket','success',1484,'V13','HiFire-7','','','Hypersonic',29,'',308 from dual union all
select 70121,'2015-S40','2015-06-30 04:55:00','suborbital rocket','success',1484,'V24','MAPHEUS 5','','','Micrograv',416,'',256 from dual union all
select 70122,'2015-S41','2015-08-12 10:04:00','suborbital rocket','success',1331,'NASA 46.012UO','RockSat-X 5','','','Edu',1804,'',156 from dual union all
select 70123,'2015-S42','2015-07-29 08:30:00','military missile','success',702,'Aegis MMW E1 Target','Stellar War Ax','Multi-Mission Warfare','','Target',855,'',100 from dual union all
select 70124,'2015-S43','2015-07-30 06:15:00','military missile','success',702,'Aegis MMW E2 Target','','','','Target',855,'',100 from dual union all
select 70125,'2015-S44','2015-08-19 10:03:00','military missile','success',799,'','Mk 12/21 RV','GT213GM','GLORY TRIP 213GM','Op Test',1706,'',1300 from dual union all
select 70126,'2015-S45','2015-08-22 15:13:00','military missile','success',1435,'Topol''-E','15Zh58E GCh-11','RV Test 6?','','R&D',636,'',1000 from dual union all
select 70127,'2015-S46','2015-08-26 00:00:00','military missile','success',580,'Scud C','','','','Weapon',1333,'',80 from dual union all
select 70128,'2015-S47','2015-08-27 17:45:00','suborbital rocket','success',224,'NASA 36.282US','MOSES-II','','','Solar EUV',1873,'',250 from dual union all
select 70129,'2015-S48','2015-09-03 17:01:00','suborbital rocket','success',224,'NASA 36.291US','CLASP','','','Solar UV',1873,'',287 from dual union all
select 7013,'1960-S349','1960-11-05 00:00:00','military missile','failure',1009,'V-1000 target','8A62 PRO Target C-1','NIIP-A Target','','Target',321,'',300 from dual union all
select 70130,'2015-S49','2015-09-11 11:00:00','suborbital rocket','success',1074,'S-520-30','Dust nucleation','','','Microgravity/Physics',833,'',312 from dual union all
select 70131,'2015-S50','2015-09-16 19:06:00','suborbital rocket','success',234,'NASA 39.012DR','CARE-II','','','Ionos/Cal',28,'',299 from dual union all
select 70132,'2015-S51','2015-09-30 08:28:00','military missile','success',758,'','','','','Test',307,'',1000 from dual union all
select 70133,'2015-S52','2015-10-02 05:39:00','suborbital rocket','success',1481,'V01','SSC O-STATES 1','','','Aeron',410,'',246 from dual union all
select 70134,'2015-S53','2015-10-07 23:07:00','suborbital rocket','success',223,'NASA 36.310GT','Mk4 Motor Test','','','Tech/Aeron',1821,'',164 from dual union all
select 70135,'2015-S54','2015-10-11 00:00:00','military missile','success',457,'','','','','Test',1348,'',200 from dual union all
select 70136,'2015-S55','2015-10-21 12:45:00','military missile','success',799,'','Mk 12/21 RV','GT216GM','GLORY TRIP 216GM','Op Test',1699,'',1300 from dual union all
select 70137,'2015-S56','2015-10-19 14:09:00','suborbital rocket','success',1481,'V02','SSC O-STATES 2','','','Aeron',410,'',244 from dual union all
select 70138,'2015-S57','2015-10-28 11:30:00','military missile','success',1531,'','15Zh65M LKI-2','RV x 4? Silo','','OT',608,'',1000 from dual union all
select 70139,'2015-S58','2015-10-30 00:00:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-117',1000 from dual union all
select 7014,'1960-S35','1960-02-13 00:43:00','suborbital rocket','success',48,'AA2.143C','UV airglow spectra','','','Aeron',1869,'',143 from dual union all
select 70140,'2015-S59','2015-10-30 00:00:00','military missile','success',1507,'VMF RF','','','','OT',1011,'K-223',1000 from dual union all
select 70141,'2015-S60','2015-10-30 00:00:00','military missile','success',1434,'','15Zh58 UBP-42','','','OT',608,'',1000 from dual union all
select 70142,'2015-S61','2015-11-01 03:05:00','military missile','success',1098,'FTO-02 E2a','FTO-02E2a Target','','','Target',1273,'C-17',300 from dual union all
select 70143,'2015-S62','2015-11-01 03:07:00','military missile','success',1294,'THAAD FTO-02 E2a','SRALT intercept','','','ABM Test',1783,'',100 from dual union all
select 70144,'2015-S63','2015-11-01 03:09:25','military missile','success',1551,'MBRV','FTO-02E2a Target','MBRV-7?','','Target',1274,'C-17',300 from dual union all
select 70145,'2015-S64','2015-11-01 03:12:00','military missile','success',1294,'THAAD FTO-02 E2a','eMRBM intercept','','','ABM Test',1783,'',100 from dual union all
select 70146,'2015-S65','2015-10-31 23:00:00','military missile','success',198,'ABM Target','','','','ABM',819,'',100 from dual union all
select 70147,'2015-S66','2015-10-31 23:00:00','military missile','success',354,'ABM Interceptor','','','','ABM',1629,'',250 from dual union all
select 70148,'2015-S67','2015-10-20 00:00:00','military missile','success',1345,'ASD-15 E2','Aegis Target','','','Target',1447,'',100 from dual union all
select 70149,'2015-S68','2015-10-20 00:00:00','military missile','success',1090,'MTMD Forum ASD15','','','','Interceptor',1447,'DDG-71',100 from dual union all
select 7015,'1960-S350','1960-11-05 01:00:00','military missile','success',347,'','','','','Test',811,'',100 from dual union all
select 70150,'2015-S69','2015-11-08 02:00:00','military missile','success',1443,'DASO-26','-    (156s)','','','OT',1338,'SSBN 737',1000 from dual union all
select 70151,'2015-S70','2015-11-09 20:00:00','military missile','success',1443,'DASO-26','-    (157s)','','','OT',1338,'SSBN 737',1000 from dual union all
select 70152,'2015-S71','2015-11-09 04:15:00','military missile','success',66,'','Agni RV','','','Test',316,'',600 from dual union all
select 70153,'2015-S72','2015-11-06 15:01:00','suborbital rocket','success',1244,'SpaceLoft SL-10','NASA FOP-4/Maraia','Celestis Tribute','','Tech',1422,'',121 from dual union all
select 70154,'2015-S73','2015-11-14 00:00:00','military missile','success',261,'','','','','OT?',201,'K-551',40 from dual union all
select 70155,'2015-S74','2015-11-14 00:00:00','military missile','success',261,'','','','','OT?',201,'K-551',1000 from dual union all
select 70156,'2015-S75','2015-11-17 12:12:00','military missile','success',1435,'Topol''-E','15Zh58E GCh-12','RV Test 7?','','R&D',636,'',1000 from dual union all
select 70157,'2015-S76','2015-11-23 17:21:00','suborbital rocket','success',834,'2-1','New Shepard M2','New Shepard CC','','Test',1896,'',101 from dual union all
select 70158,'2015-S77','2015-11-25 04:17:00','suborbital rocket','success',224,'NASA 36.293UG','PICTURE-B','','','Opt Astron',1873,'',217 from dual union all
select 70159,'2015-S79','2015-11-18 00:00:00','military missile','success',882,'','14Ts033 LKI-3','A-235','','Test',608,'',100 from dual union all
select 7016,'1960-S351','1960-11-07 19:00:00','military missile','success',928,'A1X-46','','','','Test',244,'',500 from dual union all
select 70160,'2015-S80','2015-11-30 07:25:00','suborbital rocket','failure',890,'NASA 49.003UE','CAPER','','','Auroral',29,'',100 from dual union all
select 70161,'2015-S81','2015-12-05 06:00:00','suborbital rocket','success',224,'NASA 36.305UH','DXL-2','','','XR Astron',1895,'',238 from dual union all
select 70162,'2015-S82','2015-11-21 00:00:00','military missile','success',508,'Ghadr-110','','','','Test',1348,'',150 from dual union all
select 70163,'2015-S83','2015-12-01 05:00:00','suborbital rocket','success',1484,'V22','SSC MASER 13','','','Micrograv',416,'',260 from dual union all
select 70164,'2015-S84','2015-12-08 00:00:00','military missile','success',1092,'Aegis SCD CTV-02','Stellar Kirin','','','Test',1415,'',150 from dual union all
select 70165,'2015-S85','2015-12-10 06:12:00','military missile','success',1183,'','Target','','','Test',951,'F-15D',100 from dual union all
select 70166,'2015-S86','2015-12-10 06:15:00','military missile','success',120,'','Hetz-3','','','Interceptor',1026,'',100 from dual union all
select 70167,'2015-S87','2015-12-10 00:00:00','military missile','success',601,'Aegis FTO-02E1a','FTO-02E1a Target','MBRV-7?','','Target',1275,'C-17',300 from dual union all
select 70168,'2015-S88','2015-12-10 00:00:00','military missile','success',1091,'Aegis FTO-02 E1a','Aegis Ashore','','','Interceptor',841,'',150 from dual union all
select 70169,'2015-S89','2015-12-10 13:55:00','military missile','success',632,'5/MBRV-3','PAC-3 P8-4 Target','','','Target',436,'',100 from dual union all
select 7017,'1960-S352','1960-11-08 13:18:00','ballistic missile test','failure',247,'D-2','HETS','','','Magnetospheric',235,'',200 from dual union all
select 70170,'2015-S90','2015-12-12 00:00:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-51',1000 from dual union all
select 70171,'2015-S91','2015-12-11 00:00:00','military missile','success',1172,'','','','','Test',1419,'',500 from dual union all
select 70172,'2015-S92','2015-12-15 00:00:00','military missile','success',1170,'Hatf IV','','','','Test',1419,'',100 from dual union all
select 70173,'2015-S93','2015-12-13 07:34:00','suborbital rocket','success',238,'NASA 52.002UE','RENU 2','','','Auroral',29,'',447 from dual union all
select 70174,'2015-S94','2015-12-24 17:55:00','military missile','success',1435,'Topol''-E','15Zh58E GCh-13','RV Test 8?','','R&D',636,'',1000 from dual union all
select 70175,'2015-S95','2015-12-18 06:52:00','suborbital rocket','success',224,'NASA 36.312UG','FORTIS 3','','','UV Astron',1895,'',282 from dual union all
select 70176,'2015-S96','2015-09-03 13:23:00','suborbital rocket','failure',739,'MN-300','','','','Test',1472,'',300 from dual union all
select 70177,'2015-S97','2015-08-06 00:00:00','military missile','success',360,'','','','','Test',1494,'',1000 from dual union all
select 70178,'2015-U01','2015-02-11 13:40:00','orbital','success',1488,'VV04','IXV','IXV','','Sat',349,'',null from dual union all
select 70179,'2016-001','2016-01-15 16:57:00','orbital','success',311,'Y29','Belintersat-1','Belintersat-1','','Sat',1904,'',null from dual union all
select 7018,'1960-S353','1960-11-10 16:22:00','military missile','success',929,'A2X-1','','','','Test',244,'',1000 from dual union all
select 70180,'2016-002','2016-01-17 18:42:18','orbital','success',476,'019','Jason-3','Jason-3','','Sat',1777,'',null from dual union all
select 70181,'2016-003','2016-01-20 04:01:00','orbital','success',902,'PSLV-C31','IRNSS-1E','IRNSS-1E','','Sat',1359,'',null from dual union all
select 70182,'2016-004','2016-01-27 23:20:07','orbital','success',111,'VA228 (583)','Intelsat 29e','Intelsat 29e','','Sat',346,'',null from dual union all
select 70183,'2016-005','2016-01-29 22:20:09','orbital','success',962,'935-58/99558','Eutelsat 9B','Eutelsat 9B','','Sat',553,'',null from dual union all
select 70184,'2016-006','2016-02-01 07:29:00','orbital','success',317,'Y14','Beidou DW21','Beidou M3-S','','Sat',1903,'',null from dual union all
select 70185,'2016-007','2016-02-05 13:38:00','orbital','success',181,'AV-057','GPS II F-12','GPS SVN 70','','Sat',292,'',null from dual union all
select 70186,'2016-008','2016-02-07 00:21:00','orbital','success',1228,'G 15000-030 030/003','Kosmos-2513','Uragan-M','','Sat',475,'',null from dual union all
select 70187,'2016-009','2016-02-07 00:30:00','orbital','success',685,'5','Kwangmyongsong-4','KMS-4 Unit 1','','Sat',1416,'',120 from dual union all
select 70188,'2016-010','2016-02-10 11:40:32','orbital','success',401,'D4-31 (373)','USA 267','TOPAZ 4','','Sat',1780,'',null from dual union all
select 70189,'2016-011','2016-02-16 17:57:45','orbital','success',1064,'492-9794-553/72525','Sentinel-3A','Sentinel-3A','','Sat',460,'',null from dual union all
select 7019,'1960-S354','1960-11-10 01:44:00','suborbital rocket','success',624,'NASA 8.04CA','Ionosphere','','','Aeronomy',1821,'',975 from dual union all
select 70190,'2016-012','2016-02-17 08:45:00','orbital','success',527,'H-IIA-30','Hitomi','ASTRO-H','','Sat',1477,'',null from dual union all
select 70191,'2016-013','2016-03-04 23:35:00','orbital','success',481,'022/B1020','SES-9','SES-9','','Sat',274,'',null from dual union all
select 70192,'2016-014','2016-03-09 05:20:07','orbital','success',111,'VA229 (582)','Eutelsat 65 West A','Eutelsat 65 West A','','Sat',346,'',null from dual union all
select 70193,'2016-015','2016-03-10 10:31:00','orbital','success',902,'PSLV-C32','IRNSS-1F','IRNSS-1F','','Sat',1359,'',null from dual union all
select 70194,'2016-016','2016-03-13 18:56:00','orbital','success',1229,'T 15000-016 016','Resurs-P No. 3','Resurs-P No. 3','','Sat',562,'',null from dual union all
select 70195,'2016-017','2016-03-14 09:31:42','deep space','success',962,'935-60/99560','ExoMars TGO/EDM','ExoMars TGO/EDM','','Sat',553,'',null from dual union all
select 70196,'2016-018','2016-03-18 21:26:39','orbital','success',1233,'R 15000-057 057','Soyuz TMA-20M','Soyuz 11F732A47 No. 720','','Sat ISS 46S',480,'',null from dual union all
select 70197,'2016-019','2016-03-23 03:05:52','orbital','success',181,'AV-064','Cygnus OA-6','Cygnus OA-6','','Sat',292,'',null from dual union all
select 70198,'2016-020','2016-03-24 09:42:00','orbital','success',1227,'760-58- 161','Kosmos-2515','Bars-M No. 2','','Sat',475,'',null from dual union all
select 70199,'2016-021','2016-03-29 20:11:00','orbital','success',309,'Y26','Beidou DW22','Beidou I3-S','','Sat',1903,'',null from dual union all
select 702,'1947-M03','1947-03-03 00:00:00','suborbital rocket','success',1517,'30B','','','','Test',1852,'',63 from dual union all
select 7020,'1960-S355','1960-11-10 02:35:00','suborbital rocket','success',30,'ND3.101','','','','Aeron',1869,'',211 from dual union all
select 70200,'2016-022','2016-03-31 16:23:57','orbital','success',1227,'R 15000-023 023','Progress MS-02','Progress 7K-TGM No. 432','','Sat ISS 63P',562,'',null from dual union all
select 70201,'2016-023','2016-04-05 17:38:00','orbital','success',304,'Y36','Shi Jian 10','SJ-10','','Sat',813,'',null from dual union all
select 70202,'2016-024','2016-04-08 20:43:31','orbital','success',481,'023/B1021','Dragon CRS-8','Dragon C110','','Sat',274,'',null from dual union all
select 70203,'2016-025','2016-04-25 21:02:13','orbital','success',1234,'R 15000-005 005/008','Sentinel 1B','Sentinel 1B/Microscope','','Sat VS14',347,'',null from dual union all
select 70204,'2016-026','2016-04-28 02:01:21','orbital','success',1230,'R 15000-001 001/006','Lomonosov','Lomonosov/Aist-2D','','Sat',1764,'',null from dual union all
select 70205,'2016-027','2016-04-28 07:20:00','orbital','success',902,'PSLV-C33','IRNSS-1G','IRNSS-1G','','Sat',1357,'',null from dual union all
select 70206,'2016-028','2016-05-06 05:21:00','orbital','success',481,'024/B1022','JCSAT 2B','JCSAT 14','','Sat',274,'',null from dual union all
select 70207,'2016-029','2016-05-15 02:43:00','orbital','success',304,'Y27','Yaogan 30','YG30','','Sat',813,'',null from dual;
insert into LAUNCH
select 70208,'2016-030','2016-05-24 08:48:43','orbital','success',1236,'R 15000-011 011/005','Galileo FOC 10/11','GalileoSat 13-14','','Sat VS15',347,'',null from dual union all
select 70209,'2016-031','2016-05-27 21:39:00','orbital','success',481,'025/B1023','Thaicom 8','Thaicom 8','','Sat',274,'',null from dual union all
select 7021,'1960-S356','1960-11-11 11:22:00','suborbital rocket','success',839,'NASA 10.22GE','SBE 6','','','Magnetospheric',428,'',129 from dual union all
select 70210,'2016-032','2016-05-29 08:44:35','orbital','success',1228,'G 15000-027 027','Kosmos-2516','Uragan-M No. 753','','Sat',475,'',null from dual union all
select 70211,'2016-033','2016-05-30 03:17:04','orbital','success',319,'Y33','Ziyuan-3 02','ZY-3-02/Nusat-1,2','','Sat',1493,'',null from dual union all
select 70212,'2016-034','2016-06-04 14:00:14','orbital','success',1064,'492-6391-834/72527','Geo-IK-2 No. 12','Musson-2 No. 12','','Sat',460,'',null from dual union all
select 70213,'2016-035','2016-06-09 07:10:00','orbital','success',962,'937-01/99570','Intelsat IS-31','Intelsat IS-31/DLA-2','','Sat',588,'',null from dual union all
select 70214,'2016-036','2016-06-11 17:51:00','orbital','success',396,'D4-32 (374)','USA 268','ORION 10','','Sat',290,'',null from dual union all
select 70215,'2016-037','2016-06-12 15:30:00','orbital','success',316,'Y15','Beidou DW23','Beidou G7','','Sat',1904,'',null from dual union all
select 70216,'2016-038','2016-06-15 14:29:00','orbital','success',481,'026/B1024','ABS-2/Eutelsat 117WB','ABS-2/Eutelsat 117WB','','Sat',274,'',null from dual union all
select 70217,'2016-039','2016-06-18 21:38:39','orbital','success',111,'VA230 (584)','Echostar 18/BRISAT','Echostar 18/BRISAT','','Sat',346,'',null from dual union all
select 70218,'2016-040','2016-06-22 03:55:00','orbital','success',902,'PSLV-C34','Cartosat-2(S)','Cartosat-2C','','Sat',1359,'',null from dual union all
select 70219,'2016-041','2016-06-24 14:30:00','orbital','success',195,'AV-063','MUOS 5','MUOS 5','','Sat',292,'',null from dual union all
select 7022,'1960-S357','1960-11-11 12:11:00','suborbital rocket','success',839,'NASA 10.23GE','SBE 7','','','Magnetospheric',428,'',129 from dual union all
select 70220,'2016-042','2016-06-25 12:00:00','orbital','success',326,'Y1','Fanhui Cang','FC/Aoxiang','','Sat',1788,'',null from dual union all
select 70221,'2016-043','2016-06-29 03:21:00','orbital','success',319,'Y35','SJ-16-02','SJ-16-02','','Sat',813,'',null from dual union all
select 70222,'2016-044','2016-07-07 01:36:40','orbital','success',1233,'R 15000-056 056','Soyuz MS-01','Soyuz 11F732A48 No. 731','','Sat ISS 47S',480,'',null from dual union all
select 70223,'2016-045','2016-07-16 21:41:46','orbital','success',1241,'G 15000-147 147','Progress MS-03','Progress 7K-TGM No. 433','','Sat ISS 64P',562,'',null from dual union all
select 70224,'2016-046','2016-07-18 04:45:29','orbital','success',481,'027/B1025','Dragon CRS-9','Dragon C111','','Sat',274,'',null from dual union all
select 70225,'2016-047','2016-07-28 12:37:00','orbital','success',185,'AV-065','USA 269','QUASAR 19?','','Sat',292,'',null from dual union all
select 70226,'2016-048','2016-08-05 16:22:00','orbital','success',311,'Y35','Tiantong-1','Tiantong-1','','Sat',1904,'',null from dual union all
select 70227,'2016-049','2016-08-09 22:55:00','orbital','success',320,'Y19','Gaofen 3','Gaofen 3','','Sat',1493,'',null from dual union all
select 70228,'2016-050','2016-08-14 05:26:00','orbital','success',481,'028/B1026','JCSAT 16','JCSAT 16','','Sat',274,'',null from dual union all
select 70229,'2016-051','2016-08-15 17:40:04','orbital','success',304,'Y32','Mozi hao','QUESS','','Sat',813,'',null from dual union all
select 7023,'1960-S358','1960-11-12 00:00:00','military missile','success',976,'','8K63 UBP-97','','','Test',636,'',402 from dual union all
select 70230,'2016-052','2016-08-19 04:52:00','orbital','success',399,'D4-33 (375)','AFSPC-6','GSSAP 3/4','','Sat',290,'',null from dual union all
select 70231,'2016-053','2016-08-24 22:16:01','orbital','success',111,'VA232 (586)','IS-33e/IS-36','IS-33e/IS-36','','Sat',346,'',null from dual union all
select 70232,'2016-054','2016-09-08 11:20:00','orbital','success',504,'GSLV-F05','Insat-3DR','Insat-3DR','','Sat',1359,'',null from dual union all
select 70233,'2016-055','2016-09-08 23:05:00','orbital','success',183,'AV-067','OSIRIS-REx','OSIRIS-REx','','Sat',292,'',null from dual union all
select 70234,'2016-056','2016-09-13 14:38:00','orbital','success',1174,'10','''Ofeq-11','''Ofeq-11','','Sat',1026,'',null from dual union all
select 70235,'2016-057','2016-09-15 14:04:09','orbital','success',306,'T2','Tiangong 2','Tiangong 2','','Sat',814,'',null from dual union all
select 70236,'2016-058','2016-09-16 01:43:35','orbital','success',1487,'VV07','Skysat 4-7/PeruSat','PeruSAT-1/Skysat 4-7','','Sat',349,'',null from dual union all
select 70237,'2016-059','2016-09-26 03:42:00','orbital','success',899,'PSLV-C35','Scatsat-1','Scatsat-1','','Sat',1357,'',null from dual union all
select 70238,'2016-060','2016-10-05 20:30:00','orbital','success',111,'VA231 (585)','Sky Muster 2/GSAT-18','Sky Muster/GSAT','','Sat',346,'',null from dual union all
select 70239,'2016-061','2016-10-16 23:30:31','orbital','success',306,'Y11','Shenzhou 11','Shenzhou 11','','Sat',814,'',null from dual union all
select 7024,'1960-S359','1960-11-12 18:40:00','suborbital rocket','success',839,'NASA 10.24GE','SBE 8 (LE SCR)','','','Magnetospheric',428,'',129 from dual union all
select 70240,'2016-062','2016-10-17 23:45:40','orbital','success',85,'0000.7','Cygnus OA-5','Cygnus OA-5','','Sat',944,'',null from dual union all
select 70241,'2016-063','2016-10-19 08:05:14','orbital','success',1233,'R 15000-059 059','Soyuz MS-02','Soyuz 11F732A48 No. 732','','Sat ISS 48S',562,'',null from dual union all
select 70242,'2016-064','2016-11-02 06:20:00','orbital','success',527,'H-IIA-31','Himawari 9','Himawari 9','','Sat',1477,'',null from dual union all
select 70243,'2016-065','2016-11-03 12:43:14','orbital','success',323,'Y1','Shi Jian 17','SJ-17','','Sat',1787,'',null from dual union all
select 70244,'2016-066','2016-11-09 23:42:00','orbital','success',294,'Y2','MXSW','MXSW','','Sat',818,'',null from dual union all
select 70245,'2016-067','2016-11-11 18:30:33','orbital','success',181,'AV-062','WorldView-4','WorldView-4','','Sat',1775,'',null from dual union all
select 70246,'2016-068','2016-11-11 23:14:00','orbital','success',304,'Y34','Yunhai-1 01','Yunhai-1 01','','Sat',813,'',null from dual union all
select 70247,'2016-069','2016-11-17 13:06:55','orbital','success',112,'VA233 (594)','Galileo FOC-M6','GalileoSat 15-18','','Sat',346,'',null from dual union all
select 70248,'2016-070','2016-11-17 20:20:13','orbital','success',1233,'R 15000-060 060','Soyuz MS-03','Soyuz 11F732A48 No. 733','','Sat ISS 49S',480,'',null from dual union all
select 70249,'2016-071','2016-11-19 23:42:00','orbital','success',193,'AV-069','GOES R','GOES R','','Sat',292,'',null from dual union all
select 7025,'1960-S36','1960-02-16 00:00:00','suborbital rocket','success',42,'TATTLETALE PCC 2','','','','Sigint Test?',394,'',100 from dual union all
select 70250,'2016-072','2016-11-22 15:24:04','orbital','success',315,'Y13','Tian Lian-1 04','tianlian yihao 04 xing','','Sat',1903,'',null from dual union all
select 70251,'2016-073','2016-12-05 13:51:00','orbital','success',1487,'VV08','Gokturk-1','Gokturk-1','','Sat',349,'',null from dual union all
select 70252,'2016-074','2016-12-07 04:54:00','orbital','success',902,'PSLV-C36','Resourcesat-2A','Resourcesat-2A','','Sat',1357,'',null from dual union all
select 70253,'2016-075','2016-12-07 23:52:00','orbital','success',403,'D4-34 (376)','WGS 8','Wideband Global Satcom 8','','Sat',290,'',null from dual union all
select 70254,'2016-076','2016-12-09 13:26:47','orbital','success',534,'H-IIB-F6','HTV-6','HTV 6','','Sat',1478,'',null from dual union all
select 70255,'2016-077','2016-12-10 16:11:00','orbital','success',311,'Y42','Fengyun 4-01','fengyun 4 01 xing','','Sat',1904,'',null from dual union all
select 70256,'2016-078','2016-12-15 13:37:00','orbital','success',911,'F43','CYGNSS','CYGNSS','','Sat',293,'L-1011',null from dual union all
select 70257,'2016-079','2016-12-18 19:13:00','orbital','success',187,'AV-071','Echostar 19','Echostar 19 (Jupiter 2)','','Sat',292,'',null from dual union all
select 70258,'2016-080','2016-12-20 11:00:00','orbital','success',465,'E-2','ERG','','','Sat',832,'',null from dual union all
select 70259,'2016-081','2016-12-21 19:22:04','orbital','success',304,'Y33','Tan Weixing','TanSat','','Sat',813,'',null from dual union all
select 7026,'1960-S360','1960-11-12 23:32:00','suborbital rocket','success',839,'NASA 10.15GE','SBE 9','','','Magnetospheric',428,'',129 from dual union all
select 70260,'2016-082','2016-12-21 20:30:00','orbital','success',111,'VA234 (587)','JCSAT-15/Star One D1','JCSAT-15/Star One D1','','Sat',346,'',null from dual union all
select 70261,'2016-083','2016-12-28 03:23:00','orbital','success',304,'Y','Gaojing 1/2','Gaojing 1/2','','Sat',1493,'',null from dual union all
select 70262,'2016-A01','2016-01-01 00:00:00','military missile','success',1093,'FOT&E','','','','Test',851,'DDG-53',20 from dual union all
select 70263,'2016-A02','2016-01-01 00:00:00','military missile','success',1093,'FOT&E','','','','Test',851,'DDG-53',20 from dual union all
select 70264,'2016-A03','2016-01-01 00:00:00','military missile','success',1093,'FOT&E','','','','Test',851,'DDG-53',20 from dual union all
select 70265,'2016-A04','2016-01-01 00:00:00','military missile','success',1093,'FOT&E','','','','Test',851,'DDG-53',20 from dual union all
select 70266,'2016-A05','2016-01-01 00:00:00','military missile','success',1093,'FOT&E','','','','Test',851,'DDG-53',20 from dual union all
select 70267,'2016-A06','2016-03-08 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70268,'2016-A07','2016-03-09 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70269,'2016-A08','2016-03-10 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 7027,'1960-S361','1960-11-13 16:03:00','suborbital rocket','success',839,'NASA 10.16GE','SBE 10 (LE SCR)','','','Magnetospheric',428,'',129 from dual union all
select 70270,'2016-A09','2016-03-04 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70271,'2016-A10','2016-03-05 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70272,'2016-A11','2016-03-02 00:00:00','military missile','success',1429,'','','','','Weapon',1333,'',40 from dual union all
select 70273,'2016-A12','2016-03-03 00:00:00','military missile','success',1429,'','','','','Weapon',1333,'',40 from dual union all
select 70274,'2016-A13','2016-02-21 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70275,'2016-A14','2016-02-24 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70276,'2016-A15','2016-02-08 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70277,'2016-A16','2016-02-08 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70278,'2016-A17','2016-02-09 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70279,'2016-A18','2016-02-12 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 7028,'1960-S362','1960-11-15 05:54:07','military missile','success',154,'83D','Mk 4-1-11/LMSC specm','D/RIG, R&D decoy','','Test',227,'',1096 from dual union all
select 70280,'2016-A19','2016-01-01 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70281,'2016-A20','2016-02-05 00:00:00','military missile','success',1429,'','','','','Weapon',1333,'',40 from dual union all
select 70282,'2016-A21','2016-01-18 00:00:00','military missile','success',1429,'','','','','Weapon',1333,'',40 from dual union all
select 70283,'2016-A22','2016-01-30 00:00:00','military missile','success',1429,'','','','','Weapon',1333,'',40 from dual union all
select 70284,'2016-A23','2016-01-17 00:00:00','military missile','success',1429,'','','','','Weapon',1333,'',40 from dual union all
select 70285,'2016-A24','2016-02-13 18:45:00','military missile','success',965,'','','[Alleged Scud]','','Weapon',1333,'',20 from dual union all
select 70286,'2016-A25','2016-03-17 21:00:00','military missile','failure',586,'Rodong','','','','Test',1448,'',0 from dual union all
select 70287,'2016-A26','2016-04-14 20:33:00','military missile','failure',582,'','BM-25 Musudan','','','Test',1830,'',0 from dual union all
select 70288,'2016-A27','2016-04-23 09:30:00','military missile','success',259,'','Bukgeukseong','','','Test',1406,'KPN UNK',10 from dual union all
select 70289,'2016-A28','2016-04-27 00:00:00','military missile','failure',582,'','BM-25 Musudan','','','Test',1830,'',1 from dual union all
select 7029,'1960-S363','1960-11-15 16:41:00','suborbital rocket','success',35,'NASA 04.14GA','Composition','','','Aeronomy',1821,'',227 from dual union all
select 70290,'2016-A29','2016-04-27 21:40:00','military missile','failure',582,'','BM-25 Musudan','','','Test',1830,'',10 from dual union all
select 70291,'2016-A30','2016-04-04 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70292,'2016-A31','2016-04-06 00:00:00','military missile','success',1429,'','','','','Weapon',1333,'',40 from dual union all
select 70293,'2016-A32','2016-04-05 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70294,'2016-A33','2016-04-07 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70295,'2016-A34','2016-05-30 20:20:00','military missile','failure',582,'','BM-25 Musudan','','','Test',1830,'',0 from dual union all
select 70296,'2016-A35','2016-05-30 00:00:00','military missile','success',965,'','','','','Weapon',1333,'',20 from dual union all
select 70297,'2016-A36','2016-07-09 02:30:00','military missile','failure',259,'','Bukgeukseong','','','Test',1406,'KPN UNK',40 from dual union all
select 70298,'2016-A37','2016-06-21 04:00:00','military missile','success',3,'','Test','','','Op Test',183,'',40 from dual;
insert into LAUNCH
select 70299,'2016-A38','2016-08-02 23:00:00','military missile','failure',586,'Rodong','','','','Test',422,'',0 from dual union all
select 703,'1947-M04','1947-05-30 00:00:00','military missile','failure',574,'B-1 No. 0','Hermes II Dummy Ram','','','Test',1863,'',50 from dual union all
select 7030,'1960-S364','1960-11-16 11:02:00','suborbital rocket','success',1194,'SL33','RAE Test 10','','','Test',1839,'',105 from dual union all
select 70300,'2016-A39','2016-09-08 19:04:00','suborbital spaceplane','success',1246,'VSS Unity','WK2-/CC2-01','','','Captive Carry 3h43',955,'WK2',10 from dual union all
select 70301,'2016-A40','2016-09-27 00:00:00','military missile','success',261,'','','','','OT',190,'K-535',40 from dual union all
select 70302,'2016-A41','2016-10-15 03:33:00','military missile','failure',583,'','','','','Test',925,'',0 from dual union all
select 70303,'2016-A42','2016-10-19 22:00:00','military missile','failure',583,'','','','','Test',925,'',0 from dual union all
select 70304,'2016-A43','2016-11-01 13:30:00','suborbital spaceplane','success',1246,'VSS Unity','WK2-215/CC2-02','','','Glide abort',955,'WK2',15 from dual union all
select 70305,'2016-A44','2016-11-03 13:30:00','suborbital spaceplane','success',1246,'VSS Unity','WK2-216/CC2-03','','','Glide abort',955,'WK2',15 from dual union all
select 70306,'2016-A45','2016-11-30 22:07:00','suborbital spaceplane','success',1246,'VSS Unity','WK2-217/CC2-04','','','Captive Carry',955,'WK2',15 from dual union all
select 70307,'2016-A46','2016-12-03 15:40:00','suborbital spaceplane','success',1246,'VSS Unity','WK2-218/GF2-05','','','Glide  10',955,'WK2',15 from dual union all
select 70308,'2016-A47','2016-12-15 00:00:00','military missile','success',1093,'Aegis FTM-27','Stellar Crossbow','','','Test',851,'DDG-53',20 from dual union all
select 70309,'2016-A48','2016-12-15 00:00:00','military missile','success',1093,'Aegis FTM-27','Stellar Crossbow','','','Test',851,'DDG-53',20 from dual union all
select 7031,'1960-S365','1960-11-16 17:37:00','suborbital rocket','success',839,'NASA 10.10UA','Composition','','','Aeronomy',1821,'',152 from dual union all
select 70310,'2016-A49','2016-12-22 22:00:00','suborbital spaceplane','success',1246,'VSS Unity','WK2-221/GF2-06','','','Glide  10',955,'WK2',15 from dual union all
select 70311,'2016-A50','2016-06-01 00:00:00','military missile','failure',1443,'DASO-11','-    -','','','OT',421,'UK S31',10 from dual union all
select 70312,'2016-E01','2016-09-01 13:07:00','orbital','destroyed before launch',481,'029/B1028','Amos 6','Amos 6','','Sat',274,'',0 from dual union all
select 70313,'2016-F01','2016-08-31 18:55:25','orbital','failure',320,'Y21','Gaofen 10','Gaofen 10','','Sat',1493,'',650 from dual union all
select 70314,'2016-F02','2016-12-01 14:51:52','orbital','failure',1241,'P 15000-148 148','Progress MS-04','Progress 7K-TGM No. 434','','Sat ISS 65P',480,'',null from dual union all
select 70315,'2016-M01','2016-05-23 01:30:00','suborbital rocket','success',562,'RLV-TD','HEX-01','','','Reentry Test',1357,'',65 from dual union all
select 70316,'2016-M02','2016-08-28 00:30:00','suborbital rocket','success',24,'ATV-D02','SPEX Scramjet','','','Test/Tech',1361,'',70 from dual union all
select 70317,'2016-M03','2016-08-18 00:00:00','military missile','success',610,'','','','','Test',384,'',50 from dual union all
select 70318,'2016-M04','2016-11-21 04:07:00','military missile','success',942,'','Test','','','Test',315,'',50 from dual union all
select 70319,'2016-M05','2016-11-21 04:08:00','military missile','success',942,'','Test','','','Test',315,'',50 from dual union all
select 7032,'1960-S366','1960-11-16 18:25:00','military missile','success',916,'206','P-08','','','Test',257,'',180 from dual union all
select 70320,'2016-S01','2016-01-15 03:00:00','suborbital rocket','success',1073,'S-310-44','','Plasma heating','','Ionos',833,'',161 from dual union all
select 70321,'2016-S02','2016-01-22 13:00:00','suborbital rocket','success',834,'2-2','New Shepard M3','New Shepard CC','','Test',1896,'',101 from dual union all
select 70322,'2016-S03','2016-01-23 08:30:00','suborbital rocket','success',1484,'V23','DLR TEXUS 53','','','Micrograv',416,'',253 from dual union all
select 70323,'2016-S04','2016-01-28 21:55:00','military missile','success',601,'GMD Target','GMD CTV-02+ Target','','','Target',1275,'C-17',300 from dual union all
select 70324,'2016-S05','2016-01-28 21:57:00','military missile','success',497,'GBI','GMD CTV-02+','CE-II EKV','','Interceptor',1709,'',1000 from dual union all
select 70325,'2016-S06','2016-02-02 21:09:00','suborbital rocket','success',1478,'','SSC SPIDER/LEEWAVES','','','Auroral',416,'',138 from dual union all
select 70326,'2016-S07','2016-02-21 07:34:02','military missile','success',799,'','Mk 12/21 RV','GT217GM','GLORY TRIP 217GM','Op Test',1705,'',1300 from dual union all
select 70327,'2016-S08','2016-02-26 07:01:00','military missile','success',799,'','Mk 12/21 RV','GT218GM','GLORY TRIP 218GM','Op Test',1706,'',1300 from dual union all
select 70328,'2016-S09','2016-02-22 04:15:00','suborbital rocket','success',224,'NASA 36.297UG','CHESS 2','','','UV Astron',1895,'',309 from dual union all
select 70329,'2016-S10','2016-03-01 14:50:00','suborbital rocket','success',1333,'NASA 46.011GP','MUSIC','','','Tech',1821,'',185 from dual union all
select 7033,'1960-S367','1960-11-16 19:51:00','suborbital rocket','success',839,'NASA 10.13GE','SBE 11','','','Magnetospheric',428,'',129 from dual union all
select 70330,'2016-S11','2016-03-07 12:05:00','suborbital rocket','success',1334,'NASA 41.114NP','FOP/SOAREX-9','','','Tech',1821,'',160 from dual union all
select 70331,'2016-S12','2016-03-08 00:00:00','military missile','success',1166,'','','','','Test',1311,'',100 from dual union all
select 70332,'2016-S13','2016-03-08 00:00:00','military missile','success',1167,'','','','','Test',1311,'',125 from dual union all
select 70333,'2016-S14','2016-03-08 00:00:00','military missile','success',967,'Qiam-1','','','','Test',1311,'',125 from dual union all
select 70334,'2016-S15','2016-03-09 00:00:00','military missile','success',508,'Qadr-F','','','','Test',1348,'',150 from dual union all
select 70335,'2016-S16','2016-03-09 00:00:00','military missile','success',508,'Qadr-H','','','','Test',1348,'',150 from dual union all
select 70336,'2016-S17','2016-03-09 20:20:00','military missile','success',580,'Scud C','','','','Test',678,'',150 from dual union all
select 70337,'2016-S18','2016-03-09 20:20:00','military missile','success',580,'Scud C','','','','Test',678,'',150 from dual union all
select 70338,'2016-S19','2016-03-17 20:55:00','military missile','success',586,'Rodong','','','','Test',1448,'',150 from dual union all
select 70339,'2016-S20','2016-03-14 00:00:00','military missile','success',1443,'FCET-52','-    (158s)','','','OT',421,'SSBN UNK',1000 from dual union all
select 7034,'1960-S368','1960-11-17 00:00:00','military missile','success',1013,'','8K51 PRO Target C-5','NIIP-A Target','','Target',321,'',500 from dual union all
select 70340,'2016-S21','2016-03-15 00:00:00','military missile','success',1443,'FCET-52','-    (159s)','','','OT',421,'SSBN UNK',1000 from dual union all
select 70341,'2016-S22','2016-03-16 00:00:00','military missile','success',1443,'FCET-52','-    (160s)','','','OT',421,'SSBN UNK',1000 from dual union all
select 70342,'2016-S23','2016-04-02 15:18:00','suborbital rocket','success',834,'2-3','New Shepard M4','New Shepard CC','','Test',1896,'',101 from dual union all
select 70343,'2016-S24','2016-04-19 06:41:55','military missile','success',1454,'492-9152-331','15A35-71 LKI-3','Yu-71 / Proekt 4202','','Test',1908,'',1000 from dual union all
select 70344,'2016-S25','2016-04-19 09:33:00','test rocket','success',1184,'1','','','','Test',1348,'',200 from dual union all
select 70345,'2016-S26','2016-04-12 00:00:00','military missile','success',360,'RV x 2','','','','Test',1494,'',1000 from dual union all
select 70346,'2016-S27','2016-03-31 00:00:00','military missile','success',640,'1','','','','Test',1750,'INS-S73',500 from dual union all
select 70347,'2016-S28','2016-04-26 17:00:00','suborbital rocket','success',1399,'Kunpeng-1B','','','','Ionos',647,'',316 from dual union all
select 70348,'2016-S29','2016-05-18 07:02:00','military missile','success',746,'Aegis FTX-21 Target','','','','Target',855,'',300 from dual union all
select 70349,'2016-S30','2016-05-18 00:45:00','suborbital rocket','success',1479,'','HiFire-5B','','','Hypersonic',1849,'',278 from dual union all
select 7035,'1960-S369','1960-11-17 06:00:00','suborbital rocket','success',839,'NASA 10.14GE','SBE 12','','','Magnetospheric',428,'',129 from dual union all
select 70350,'2016-S31','2016-05-25 00:00:00','military missile','success',882,'','14Ts033 LKI-4','A-235','','Test',608,'',100 from dual union all
select 70351,'2016-S32','2016-05-25 00:00:00','military missile','success',1091,'Aegis TU CTV-01a','','','','Test',851,'DDG-70',100 from dual union all
select 70352,'2016-S33','2016-05-26 00:00:00','military missile','success',1091,'Aegis TU CTV-02','','','','Test',851,'DDG-70',100 from dual union all
select 70353,'2016-S34','2016-03-15 05:30:00','suborbital rocket','success',605,'SSC REXUS 19','EURO REXUS 19','','','Tech/Aeron',417,'',81 from dual union all
select 70354,'2016-S35','2016-03-18 05:10:00','suborbital rocket','success',605,'SSC REXUS 20','EURO REXUS 20','','','Tech',417,'',81 from dual union all
select 70355,'2016-S36','2016-06-01 19:00:00','suborbital rocket','success',224,'NASA 36.318UE','SDO EVE','','','Solar UV',1895,'',290 from dual union all
select 70356,'2016-S37','2016-06-19 14:36:00','suborbital rocket','success',834,'2-4','New Shepard M5','New Shepard CC','','Test',1896,'',101 from dual union all
select 70357,'2016-S38','2016-06-21 20:56:00','military missile','failure',582,'','BM-25 Musudan','','','Test',1830,'',100 from dual union all
select 70358,'2016-S39','2016-06-21 23:03:00','military missile','success',582,'','BM-25 Musudan','','','Test',1830,'',1414 from dual union all
select 70359,'2016-S40','2016-06-24 10:06:00','suborbital rocket','success',1334,'NASA 41.116UO','RockOn 9/RockSat-C 8','','','Edu',1821,'',119 from dual union all
select 7036,'1960-S37','1960-02-17 01:00:00','suborbital rocket','success',839,'AA6.103C','','','','Meteorites',394,'',120 from dual union all
select 70360,'2016-S41','2016-06-30 09:43:18','suborbital rocket','success',604,'MaxiDusty 1','ASC MaxiDusty 1','','','Aeron',28,'',115 from dual union all
select 70361,'2016-S42','2016-07-01 07:18:00','military missile','success',758,'','','','','Test',207,'S616',1000 from dual union all
select 70362,'2016-S43','2016-07-18 20:45:00','military missile','success',580,'Scud C','','','','Test',678,'',150 from dual union all
select 70363,'2016-S44','2016-07-18 20:50:00','military missile','success',580,'Scud C','','','','Test',678,'',150 from dual union all
select 70364,'2016-S45','2016-07-18 21:40:00','military missile','success',586,'Scud C','','','','Test',422,'',150 from dual union all
select 70365,'2016-S46','2016-07-19 04:05:00','suborbital rocket','success',1334,'ROTEX-T','DLR ROTEX-T','','','Hypersonic test',417,'',182 from dual union all
select 70366,'2016-S47','2016-07-27 18:26:00','suborbital rocket','success',224,'NASA 36.314NS','Hi-C 2','','','Solar EUV',1895,'',260 from dual union all
select 70367,'2016-S48','2016-08-02 22:53:00','military missile','success',586,'Rodong','','','','Test',422,'',150 from dual union all
select 70368,'2016-S49','2016-08-17 11:33:30','suborbital rocket','success',1333,'NASA 46.014UO','RockSat-X 6','','','Edu',1801,'',153 from dual union all
select 70369,'2016-S50','2016-07-08 13:01:00','suborbital rocket','success',604,'MaxiDusty 1b','ASC MaxiDusty 1b','','','Aeron',28,'',117 from dual union all
select 7037,'1960-S370','1960-11-17 10:06:00','suborbital rocket','success',839,'','','','','Aeron',1895,'',152 from dual union all
select 70370,'2016-S51','2016-08-23 20:30:00','military missile','success',259,'','Bukgeukseong','','','Test',1406,'KPN UNK',500 from dual union all
select 70371,'2016-S52','2016-09-05 03:14:00','military missile','success',581,'Scud ER','','','','Test',678,'',150 from dual union all
select 70372,'2016-S53','2016-09-05 03:14:00','military missile','success',581,'Scud ER','','','','Test',678,'',150 from dual union all
select 70373,'2016-S54','2016-09-05 03:14:00','military missile','success',581,'Scud ER','','','','Test',678,'',150 from dual union all
select 70374,'2016-S55','2016-09-05 09:10:00','military missile','success',799,'','Mk 12/21 RV','GT219GM','GLORY TRIP 219GM','Op Test',1699,'',1300 from dual union all
select 70375,'2016-S56','2016-08-25 08:53:00','military missile','failure',1530,'','','','','OT',608,'',100 from dual union all
select 70376,'2016-S57','2016-09-09 00:00:00','military missile','success',1434,'','15Zh58 UBP-43','','','OT',608,'',1000 from dual union all
select 70377,'2016-S58','2016-08-31 00:00:00','military missile','success',1443,'DASO-27','-    (161s)','','','OT',421,'SSBN 738',1000 from dual union all
select 70378,'2016-S59','2016-09-27 00:00:00','military missile','success',261,'','','','','OT',190,'K-535',1000 from dual union all
select 70379,'2016-S60','2016-10-05 15:36:00','suborbital rocket','success',834,'2-5','New Shepard M6','New Shepard CC Abort','','Test',1896,'',94 from dual union all
select 7038,'1960-S371','1960-11-17 10:40:00','suborbital rocket','success',1195,'SL67','UCL GC-11/QUB Na-6','','','Aeronomy',1839,'',247 from dual union all
select 70380,'2016-S61','2016-10-12 00:00:00','military missile','success',1507,'VMF RF','','','','OT',1011,'K-433',1000 from dual union all
select 70381,'2016-S62','2016-10-12 00:00:00','military missile','success',1185,'VMF RF','','','','OT',201,'K-407',1000 from dual union all
select 70382,'2016-S63','2016-10-12 00:00:00','military missile','success',1434,'','15Zh58 UBP-44','','','OT',608,'',1000 from dual union all
select 70383,'2016-S64','2016-10-25 08:58:00','military missile','success',1454,'',' Possible 15A35-71','','','Test',1908,'',1000 from dual union all
select 70384,'2016-S65','2016-12-15 16:15:00','suborbital rocket','success',1546,'NASA 12.080DR','Zombie Pathfinder','','','Test',1895,'',80 from dual union all
select 70385,'2016-S66','2016-12-15 00:00:00','military missile','success',746,'Aegis FTM-27 Target','Stellar Crossbow','','','Target',855,'',200 from dual union all
select 70386,'2016-S67','2016-12-16 00:00:00','military missile','success',882,'','','','','Test',608,'',100 from dual union all
select 70387,'2016-S68','2016-12-26 05:35:00','military missile','success',67,'','','Reduced range test','','Test',316,'',800 from dual union all
select 70388,'2016-S69','2016-07-12 00:00:00','military missile','success',671,'','','','','Test',1348,'',300 from dual;
insert into LAUNCH
select 70389,'2016-S70','2016-12-21 02:45:00','military missile','success',198,'','','Possible target','','ABM',819,'',100 from dual union all
select 7039,'1960-S372','1960-11-17 13:40:00','suborbital rocket','success',30,'AA3.144','UV detector','','','Aeron',1869,'',217 from dual union all
select 70390,'2016-S71','2016-12-12 00:00:00','military missile','success',350,'DF-15C','','','','Test',819,'',100 from dual union all
select 70391,'2017-001','2017-01-05 15:18:00','orbital','success',311,'Y39','TJS 2','TJS 2','','Sat',1903,'',null from dual union all
select 70392,'2017-002','2017-01-09 04:11:12','orbital','success',683,'Jilin-1 Shipin 03','Jilin-1','','','Sat',818,'',null from dual union all
select 70393,'2017-003','2017-01-14 17:54:39','orbital','success',481,'030/B1029','Iridium-1','Iridium Next SV1-10','','Sat',1777,'',null from dual union all
select 70394,'2017-004','2017-01-21 00:42:00','orbital','success',181,'AV-066','SBRIS GEO-3','SBIRS GEO-3','','Sat',292,'',null from dual union all
select 70395,'2017-005','2017-01-24 07:44:00','orbital','success',532,'H-IIA-32','DSN 2','DSN 2','','Sat',1477,'',null from dual union all
select 70396,'2017-006','2017-01-28 01:03:34','orbital','success',1236,'R 15000-012 012/006','Hispasat 36W-1','Hispasat 36W-1','','Sat VS16',347,'',null from dual union all
select 70397,'2017-007','2017-02-14 21:39:00','orbital','success',111,'VA235 (588)','SkyBrasil-1/Telkom-3S','SkyBrasil-1/Telkom-3S','','Sat',346,'',null from dual union all
select 70398,'2017-008','2017-02-15 03:58:00','orbital','success',902,'PSLV-C37','Cartosat-2(S)/Flock-3p','Cartosat-2D','','Sat',1357,'',null from dual union all
select 70399,'2017-009','2017-02-19 14:39:00','orbital','success',481,'032/B1031','Dragon CRS-10','Dragon C112','','Sat',921,'',null from dual union all
select 704,'1947-M05','1947-06-12 00:00:00','suborbital rocket','success',1517,'29B','','','','Test',1852,'',60 from dual union all
select 7040,'1960-S373','1960-11-18 03:39:00','suborbital rocket','success',839,'NASA 10.26GE','SBE 13','','','Magnetospheric',428,'',128 from dual union all
select 70400,'2017-010','2017-02-22 05:58:33','orbital','success',1241,'T 15000-145 145','Progress MS-05','Progress 7K-TGM No. 435','','Sat ISS 66P',480,'',null from dual union all
select 70401,'2017-011','2017-03-01 17:49:51','orbital','success',181,'AV-068','USA 274','INTRUDER','','Sat',1775,'',null from dual union all
select 70402,'2017-012','2017-03-02 23:53:00','orbital','success',653,'Tiankun-1','Tiankun-1','','','Sat',819,'',null from dual union all
select 70403,'2017-013','2017-03-07 01:49:24','orbital','success',1487,'VV09','Sentinel-2B','Sentinel-2B','','Sat',349,'',null from dual union all
select 70404,'2017-014','2017-03-16 06:00:00','orbital','success',481,'031/B1030','Echostar 23','Echostar 23','','Sat',921,'',null from dual union all
select 70405,'2017-015','2017-03-17 01:20:00','orbital','success',527,'H-IIA-33','IGS R-5','IGS R-5','','Sat',1477,'',null from dual union all
select 70406,'2017-016','2017-03-19 00:18:00','orbital','success',403,'D4-35 (377)','WGS 9','Wideband Global Satcom 9','','Sat',290,'',null from dual union all
select 70407,'2017-017','2017-03-30 22:27:00','orbital','success',481,'033/B1021','SES-10','SES-10','','Sat',921,'',null from dual union all
select 70408,'2017-018','2017-04-12 11:04:00','orbital','success',311,'Y43','SJ-13','Shi Jian 13','','Sat',1903,'',null from dual union all
select 70409,'2017-019','2017-04-18 15:11:26','orbital','success',181,'AV-070','Cygnus OA-7','Cygnus OA-7','','Sat',292,'',null from dual union all
select 7041,'1960-S374','1960-11-18 23:38:37','suborbital rocket','success',839,'NASA 10.27GE','SBE 14','','','Magnetospheric',428,'',129 from dual union all
select 70410,'2017-020','2017-04-20 07:13:44','orbital','success',1233,'U 15000-065 065','Soyuz MS-04','Soyuz 11F732A48 No. 735','','Sat ISS 50S',480,'',null from dual union all
select 70411,'2017-021','2017-04-20 11:41:35','orbital','success',325,'Y2','Tianzhou 1','Tianzhou 1','','Sat',1788,'',null from dual union all
select 70412,'2017-022','2017-05-01 11:15:00','orbital','success',481,'035/B1032','USA 276','NROL-76','','Sat',921,'',null from dual union all
select 70413,'2017-023','2017-05-04 21:50:00','orbital','success',111,'VA236 (589)','SDGC/Koreasat 7','SDGC/Koreasat 7','','Sat',346,'',null from dual union all
select 70414,'2017-024','2017-05-05 11:27:00','orbital','success',504,'GSLV-F09','GSAT-9','GSAT-9','','Sat',1359,'',null from dual union all
select 70415,'2017-025','2017-05-15 23:21:00','orbital','success',481,'034/B1034','Inmarsat 5F4','Inmarsat 5F4','','Sat',921,'',null from dual union all
select 70416,'2017-026','2017-05-18 11:54:53','orbital','success',1234,'R 15000-007 007/009','SES-15','SES-15','','Sat VS17',347,'',null from dual union all
select 70417,'2017-027','2017-05-25 06:33:41','orbital','success',1228,'780-72- 183/111-301','Kosmos-2518','EKS No. 2','','Sat',475,'',null from dual union all
select 70418,'2017-028','2017-06-01 00:17:36','orbital','success',527,'H-IIA-34','Michibiki 2','QZSS 2','','Sat',1477,'',null from dual union all
select 70419,'2017-029','2017-06-01 23:45:07','orbital','success',111,'VA237 (590)','Viasat 2/Eutelsat 172B','Viasat 2/Eutelsat 172B','','Sat',346,'',null from dual union all
select 7042,'1960-S375','1960-11-21 00:00:00','suborbital rocket','unknown',839,'','','','','Aeron?',1016,'',100 from dual union all
select 70420,'2017-030','2017-06-03 21:07:38','orbital','success',481,'036/B1035','Dragon CRS-11','Dragon C106.2','','Sat',921,'',null from dual union all
select 70421,'2017-031','2017-06-05 11:58:00','orbital','success',505,'D1','GSAT-19','GSAT-19','','Sat',1359,'',null from dual union all
select 70422,'2017-032','2017-06-08 03:45:47','orbital','success',962,'937-02/99571','Echostar 21','Echostar 21','','Sat',588,'',null from dual union all
select 70423,'2017-033','2017-06-14 09:20:13','orbital','success',1227,'U 15000-028','Progress MS-06','Progress 7K-TGM No. 436','','Sat ISS 67P',562,'',null from dual union all
select 70424,'2017-034','2017-06-15 03:00:00','orbital','success',319,'Y31','Huiyan','HXMT','','Sat',813,'',null from dual union all
select 70425,'2017-035','2017-06-18 16:11:00','orbital','failure',311,'','Zhongxing-9A','Zhongxing-9A','','Sat',1903,'',null from dual union all
select 70426,'2017-036','2017-06-23 03:59:00','orbital','success',902,'PSLV-C38','Cartosat-2(S)/Cubesats','Cartosat-2E','','Sat',1357,'',null from dual union all
select 70427,'2017-037','2017-06-23 18:04:39','orbital','success',1230,'780-72- 003','Kosmos-2519','Napryazheniye No. 1?','','Sat',475,'',null from dual union all
select 70428,'2017-038','2017-06-23 19:10:00','orbital','success',481,'037/B1029','BulgariaSat-1','BulgariaSat-1','','Sat',921,'',null from dual union all
select 70429,'2017-039','2017-06-25 20:25:14','orbital','success',481,'038/B1036','Iridium-2','Iridium Next SV11-20','','Sat',1777,'',null from dual union all
select 7043,'1960-S376','1960-11-22 00:00:00','suborbital rocket','unknown',839,'','','','','Aeron?',1016,'',100 from dual union all
select 70430,'2017-040','2017-06-28 21:15:00','orbital','success',111,'VA238 (591)','HellasSat 3/GSAT 17','HellasSat 3/GSAT 17','','Sat',346,'',null from dual union all
select 70431,'2017-041','2017-07-05 23:38:00','orbital','success',481,'039/B1037','Intelsat IS-35e','Intelsat IS-35e','','Sat',921,'',null from dual union all
select 70432,'2017-042','2017-07-14 06:36:49','orbital','success',1226,'T 15000-018/122-02','Kanopus-V-IK/Cubesats','Kanopus-V-IK/Cubesats','','Sat',562,'',null from dual union all
select 70433,'2017-043','2017-07-28 15:41:12','orbital','success',1233,'R 15000-058','Soyuz MS-05','Soyuz 11F732A48 No. 736','','Sat ISS 51S',480,'',null from dual union all
select 70434,'2017-044','2017-08-02 01:58:33','orbital','success',1487,'VV10','Optsat-3000/Venmus','Optsat-3000/Venmus','','Sat',349,'',null from dual union all
select 70435,'2017-045','2017-08-14 16:31:37','orbital','success',481,'040/B1039','Dragon CRS-12','Dragon C113','','Sat',921,'',null from dual union all
select 70436,'2017-046','2017-08-16 22:07:00','orbital','success',962,'937-03/99572','Blagovest 11L','Blagovest 11L','','Sat',588,'',null from dual union all
select 70437,'2017-047','2017-08-18 12:29:00','orbital','success',181,'AV-071','TDRS 13','TDRS M','','Sat',292,'',null from dual union all
select 70438,'2017-048','2017-08-19 05:29:00','orbital','success',532,'H-IIA-35','Michibiki 3','QZSS 3','','Sat',1477,'',null from dual union all
select 70439,'2017-049','2017-08-24 18:51:00','orbital','success',481,'041/B1038','Formosat-5','Formosat-5','','Sat',1777,'',null from dual union all
select 7044,'1960-S377','1960-11-22 08:42:00','suborbital rocket','success',35,'NASA 04.11GG','UV Spectrograph','','','UV Astron',1821,'',183 from dual union all
select 70440,'2017-050','2017-08-26 06:04:00','orbital','success',786,'6','ORS-5','ORS-5','','Sat',1425,'',null from dual union all
select 70441,'2017-051','2017-08-31 13:30:00','orbital','failure',902,'PSLV-C39','IRNSS-1H','IRNSS-1H','','Sat',1359,'',null from dual union all
select 70442,'2017-A01','2017-01-01 00:00:00','military missile','success',244,'','DST-5 Target','David''s Sling Target','','Test',1026,'F-15D',20 from dual union all
select 70443,'2017-A02','2017-02-24 18:10:00','suborbital spaceplane','success',1246,'VSS Unity','WK2-226/GF2-07','','','Glide  10',955,'WK2',15 from dual union all
select 70444,'2017-A03','2017-03-21 22:00:00','military missile','failure',582,'','BM-25 Musudan','','','Test',1830,'',1 from dual union all
select 70445,'2017-A04','2017-04-15 21:20:00','military missile','failure',583,'KN-17','','','','Test',1406,'',1 from dual union all
select 70446,'2017-A05','2017-05-01 17:38:00','suborbital spaceplane','success',1246,'VSS Unity','WK2-227/GF2-08','Feather test','','Glide  10',955,'WK2',15 from dual union all
select 70447,'2017-A06','2017-06-01 17:55:00','suborbital spaceplane','success',1246,'VSS Unity','WK2-230/GF2-09','Water ballast test','','Glide  10',955,'WK2',15 from dual union all
select 70448,'2017-A07','2017-06-16 00:00:00','military missile','success',3,'','Test','','','Op Test',183,'',40 from dual union all
select 70449,'2017-A08','2017-07-30 07:32:00','suborbital rocket','failure',812,'','TF-1','','','-',1459,'',40 from dual union all
select 7045,'1960-S378','1960-11-23 10:19:00','suborbital rocket','success',29,'NASA 01.01GI','OkSU quiet night','','','Ionos',428,'',91 from dual union all
select 70450,'2017-A09','2017-08-04 00:00:00','suborbital spaceplane','success',1246,'VSS Unity','WK2-234/GF2-10','','','Glide  10',955,'WK2',15 from dual union all
select 70451,'2017-A10','2017-08-25 22:07:00','military missile','failure',579,'','','','','Test',859,'',0 from dual union all
select 70452,'2017-A11','2017-08-29 00:00:00','military missile','success',1093,'Aegis FTM-27E2','','','','Test',851,'DDG-53',20 from dual union all
select 70453,'2017-F01','2017-01-14 23:33:00','orbital','failure',1100,'SS-520-4','TRICOM 1','','','Sat',833,'',220 from dual union all
select 70454,'2017-F02','2017-05-25 04:20:00','orbital','failure',456,'1','It''s A Test','','','Sat',940,'',224 from dual union all
select 70455,'2017-F03','2017-07-02 11:23:23','orbital','failure',322,'Y2','Shi Jian 18','SJ-18','','Sat',1787,'',170 from dual union all
select 70456,'2017-M01','2017-04-28 20:33:00','military missile','failure',583,'KN-17','','','','Test',1309,'',71 from dual union all
select 70457,'2017-M02','2017-06-02 04:20:00','military missile','success',942,'','Test','','','Test',315,'',50 from dual union all
select 70458,'2017-M03','2017-09-02 00:00:00','military missile','success',610,'','','','','Test',177,'',50 from dual union all
select 70459,'2017-M04','2017-09-02 00:00:00','military missile','success',610,'','','','','Test',177,'',50 from dual union all
select 7046,'1960-S379','1960-11-23 23:55:00','suborbital rocket','success',1194,'SL49','LUX-1/UCL Ion3/UCW-1','','','Ionos/Solar X',1839,'',160 from dual union all
select 70460,'2017-S01','2017-01-16 00:00:00','military missile','success',1436,'','15Zh65 UBP-2','','','OT',608,'',1000 from dual union all
select 70461,'2017-S02','2017-01-23 02:30:00','suborbital rocket','success',1484,'V19','MAIUS-1','','','Physics',416,'',238 from dual union all
select 70462,'2017-S03','2017-01-24 00:00:00','military missile','success',25,'','','','','Test',1419,'',300 from dual union all
select 70463,'2017-S04','2017-01-27 13:45:00','suborbital rocket','success',224,'NASA 36.302UE','PolarNOx','','','Aeron',1043,'',283 from dual union all
select 70464,'2017-S05','2017-01-15 00:00:00','military missile','success',438,'','RV x 10','','','Test',1494,'',1000 from dual union all
select 70465,'2017-S06','2017-01-29 00:00:00','military missile','success',671,'','','','','Test',1348,'',300 from dual union all
select 70466,'2017-S07','2017-02-04 08:30:00','military missile','success',1348,'Aegis SFTM-01 Target','MRBM-T4E','','','Target',855,'',200 from dual union all
select 70467,'2017-S08','2017-02-04 08:33:00','military missile','success',1092,'Aegis SFTM-01','','','','Interceptor',851,'DDG-53',150 from dual union all
select 70468,'2017-S09','2017-02-09 07:38:59','military missile','success',799,'','Mk 12A RV x 3','GT221GM','GLORY TRIP 221GM','Op Test',1706,'',1300 from dual union all
select 70469,'2017-S10','2017-02-11 22:55:00','military missile','success',260,'KN-15','','','','Test',926,'',550 from dual union all
select 7047,'1960-S38','1960-02-24 16:03:00','military missile','success',1419,'G-4','RVX-4A','','','Test',230,'',1000 from dual union all
select 70470,'2017-S11','2017-02-14 11:30:00','military missile','success',1443,'FCET-53','-    (162s)','','','OT',1899,'SSBN UNK',1000 from dual union all
select 70471,'2017-S12','2017-02-14 14:20:00','military missile','success',1443,'FCET-53','-    (163s)','','','OT',1899,'SSBN UNK',1000 from dual union all
select 70472,'2017-S13','2017-02-15 00:00:00','military missile','success',1443,'FCET-53','-    (164s)','','','OT',1899,'SSBN UNK',1000 from dual union all
select 70473,'2017-S14','2017-02-16 00:00:00','military missile','success',1443,'FCET-53','-    (165s)','','','OT',1899,'SSBN UNK',1000 from dual union all
select 70474,'2017-S15','2017-02-22 10:14:00','suborbital rocket','success',224,'NASA 36.303UE','ISINGLASS 1','','','Auroral',1041,'',362 from dual union all
select 70475,'2017-S16','2017-03-02 05:41:00','suborbital rocket','success',224,'NASA 36.301GE','NJAA 1','Neutral Jets in Aur Arcs','','Auroral',1043,'',330 from dual union all
select 70476,'2017-S17','2017-03-02 05:42:30','suborbital rocket','success',224,'NASA 36.306GE','NJAA 2','Neutral Jets in Aur Arcs','','Auroral',1044,'',190 from dual union all
select 70477,'2017-S18','2017-03-02 07:50:00','suborbital rocket','success',224,'NASA 36.304UE','ISINGLASS 2','','','Auroral',1042,'',365 from dual union all
select 70478,'2017-S19','2017-03-05 22:34:00','military missile','success',581,'Scud ER','','','','Test',1417,'',260 from dual;
insert into LAUNCH
select 70479,'2017-S20','2017-03-05 22:34:00','military missile','success',581,'Scud ER','','','','Test',1417,'',260 from dual union all
select 7048,'1960-S380','1960-11-24 00:00:00','military missile','failure',1013,'','8K51 PRO Target C-6','NIIP-A Target','','Target',321,'',0 from dual union all
select 70480,'2017-S21','2017-03-05 22:34:00','military missile','success',581,'Scud ER','','','','Test',1417,'',260 from dual union all
select 70481,'2017-S22','2017-03-05 22:34:00','military missile','success',581,'Scud ER','','','','Test',1417,'',260 from dual union all
select 70482,'2017-S23','2017-03-15 10:43:00','suborbital rocket','success',605,'SSC REXUS 21','EURO REXUS 21','','','Tech',417,'',86 from dual union all
select 70483,'2017-S24','2017-03-16 13:00:00','suborbital rocket','success',605,'SSC REXUS 22','EURO REXUS 22','','','Tech',417,'',84 from dual union all
select 70484,'2017-S25','2017-04-04 21:42:00','military missile','failure',583,'KN-17','','','','Test',1406,'',189 from dual union all
select 70485,'2017-S26','2017-03-01 00:00:00','military missile','success',595,'','','','','Test',33,'SIR',150 from dual union all
select 70486,'2017-S27','2017-04-07 09:30:00','suborbital rocket','success',772,'MAXUS 9','SSC MAXUS 9','','','Micrograv',407,'',678 from dual union all
select 70487,'2017-S28','2017-04-26 07:03:06','military missile','success',799,'','Mk 12/21 RV','GT220GM','GLORY TRIP 220GM','Op Test',1705,'',1300 from dual union all
select 70488,'2017-S29','2017-05-05 18:24:58','suborbital rocket','success',224,'NASA 36.309US','RAISE 3','','','Solar EUV',1895,'',296 from dual union all
select 70489,'2017-S30','2017-05-13 09:20:00','suborbital rocket','success',1484,'','DLR MAPHEUS 6','','','Micrograv',416,'',254 from dual union all
select 7049,'1960-S381','1960-11-25 00:00:00','military missile','success',81,'008','','','','Reentry Test',313,'',150 from dual union all
select 70490,'2017-S31','2017-05-13 20:28:00','military missile','success',583,'KN-17','11831851','','','Test',927,'',2111 from dual union all
select 70491,'2017-S32','2017-05-03 07:01:59','military missile','success',799,'','Mk 12/21 RV','GT222GM','GLORY TRIP 222GM','Op Test',1699,'',1300 from dual union all
select 70492,'2017-S33','2017-05-16 09:45:00','suborbital rocket','success',224,'NASA 36.317GP','SubTec-7','','','Tech',1821,'',248 from dual union all
select 70493,'2017-S34','2017-05-21 07:59:00','military missile','success',260,'KN-15','','','','Test',1309,'',560 from dual union all
select 70494,'2017-S35','2017-05-28 20:39:00','military missile','success',581,'KN-18','','','','Test',1830,'',150 from dual union all
select 70495,'2017-S36','2017-05-30 19:10:00','military missile','success',596,'','FTG-15 Target','FTG-15 Target','FTG-15','Target',868,'',1300 from dual union all
select 70496,'2017-S37','2017-05-30 19:28:04','military missile','success',497,'','GMD FTG-15 OBV','FTG-15','FTG-15','Interceptor',1709,'',1000 from dual union all
select 70497,'2017-S38','2017-04-27 03:42:00','military missile','success',65,'','','','','Test',316,'',500 from dual union all
select 70498,'2017-S39','2017-06-18 00:00:00','military missile','success',1547,'Laylat-al-Qadr','','','','Weapon',857,'',200 from dual union all
select 70499,'2017-S40','2017-06-18 00:00:00','military missile','success',1547,'','','','','Weapon',763,'',200 from dual union all
select 705,'1947-M06','1947-11-24 17:20:00','suborbital rocket','failure',58,'A4','Particles','','','CR',1869,'',56 from dual union all
select 7050,'1960-S382','1960-11-27 17:48:00','suborbital rocket','success',29,'NASA 01.02GI','OkSU quiet day','','','Ionos',428,'',85 from dual union all
select 70500,'2017-S41','2017-06-18 00:00:00','military missile','success',1547,'','','','','Weapon',857,'',200 from dual union all
select 70501,'2017-S42','2017-06-18 00:00:00','military missile','success',1547,'','','','','Weapon',857,'',200 from dual union all
select 70502,'2017-S43','2017-06-18 00:00:00','military missile','success',1547,'','','','','Weapon',857,'',200 from dual union all
select 70503,'2017-S44','2017-06-18 00:00:00','military missile','success',1547,'','','','','Weapon',857,'',200 from dual union all
select 70504,'2017-S45','2017-06-22 05:20:00','military missile','success',1348,'Aegis SFTM-02 Target','MRBM-T4E','','','Target',855,'',200 from dual union all
select 70505,'2017-S46','2017-06-22 09:30:00','suborbital rocket','success',1334,'NASA 41.121UO','Rock On 10/RockSat-C 9','','','Edu',1821,'',116 from dual union all
select 70506,'2017-S47','2017-06-23 01:30:00','military missile','success',595,'','','','','Test',33,'SIR',150 from dual union all
select 70507,'2017-S48','2017-06-22 05:23:00','military missile','failure',1092,'Aegis SFTM-02','','','','Target',855,'DDG-53',200 from dual union all
select 70508,'2017-S49','2017-06-26 00:00:00','military missile','success',261,'','','','','OT',201,'K-535',1000 from dual union all
select 70509,'2017-S50','2017-06-29 08:25:00','suborbital rocket','success',1333,'NASA 46.015GT','Ampoule Test','','','Tech/Aeron',1821,'',190 from dual union all
select 7051,'1960-S383','1960-11-29 11:16:00','suborbital rocket','success',30,'NB3.103','NRL NB3.103','','','UV Astron',1869,'',197 from dual union all
select 70510,'2017-S51','2017-03-15 00:00:00','military missile','success',1344,'AMDR Target','','','','Target',855,'',200 from dual union all
select 70511,'2017-S52','2017-06-27 05:10:00','suborbital rocket','success',224,'NASA 36.323UG','CHESS 3','','','UV Astron',1895,'',300 from dual union all
select 70512,'2017-S53','2017-07-04 00:40:00','military missile','success',584,'KN-20','36311771','','','Test',928,'',2802 from dual union all
select 70513,'2017-S54','2017-07-04 22:00:00','suborbital rocket','success',23,'','','','','Test',611,'',80 from dual union all
select 70514,'2017-S55','2017-07-04 22:00:00','military missile','success',594,'','','','','Test',611,'',150 from dual union all
select 70515,'2017-S56','2017-07-11 00:00:00','military missile','success',601,'','THAAD FTT-18 Target','MBRV-8?','','Target',1275,'C-17',300 from dual union all
select 70516,'2017-S57','2017-07-11 00:00:00','military missile','success',1294,'THAAD FTT-18','IRBM intercept','','','ABM Test',1308,'',100 from dual union all
select 70517,'2017-S58','2017-06-30 00:00:00','suborbital rocket','success',1484,'V12','HiFire-4','','','Hypersonic',1849,'',292 from dual union all
select 70518,'2017-S59','2017-07-23 14:00:00','military missile','success',198,'ABM Target','','','','ABM',819,'',100 from dual union all
select 70519,'2017-S60','2017-07-23 14:00:00','military missile','success',354,'ABM Interceptor','','','','ABM',1629,'',250 from dual union all
select 7052,'1960-S384','1960-12-01 00:00:00','suborbital rocket','success',856,'','Teflon nose','','','Reentry Test',1821,'',120 from dual union all
select 70520,'2017-S61','2017-07-27 09:30:00','test rocket','success',1184,'2','','','','Test',1348,'',200 from dual union all
select 70521,'2017-S62','2017-07-28 14:41:00','military missile','success',584,'KN-20','36311672','','','Test',970,'',3725 from dual union all
select 70522,'2017-S63','2017-07-28 22:00:00','suborbital rocket','success',23,'','','','','Test',611,'',80 from dual union all
select 70523,'2017-S64','2017-07-28 22:00:00','military missile','success',594,'','','','','Test',611,'',150 from dual union all
select 70524,'2017-S65','2017-07-30 00:00:00','military missile','success',1551,'THAAD FET-1 Target','FET-1 Target','','','Target',1276,'C-17',300 from dual union all
select 70525,'2017-S66','2017-07-30 00:00:00','military missile','success',1294,'THAAD FET-1','MRBM intercept','','','ABM Test',1308,'',100 from dual union all
select 70526,'2017-S67','2017-08-02 09:10:00','military missile','success',799,'','Mk 12/21 RV','GT223GM','GLORY TRIP 223GM','Op Test',1706,'',1300 from dual union all
select 70527,'2017-S68','2017-08-23 00:00:00','military missile','success',359,'','','','','Test',1494,'',100 from dual union all
select 70528,'2017-S69','2017-08-25 21:49:00','military missile','success',579,'','','','','Test',859,'',100 from dual union all
select 70529,'2017-S70','2017-08-25 22:19:00','military missile','success',579,'','','','','Test',859,'',100 from dual union all
select 7053,'1960-S385','1960-11-30 00:00:00','military missile','success',976,'','8K63 UBP-98','','','Test',636,'',402 from dual union all
select 70530,'2017-S71','2017-08-23 00:00:00','military missile','success',595,'','','','','Test',30,'',150 from dual union all
select 70531,'2017-S72','2017-08-23 00:00:00','military missile','success',595,'','','','','Test',31,'',150 from dual union all
select 70532,'2017-S73','2017-08-25 00:00:00','military missile','success',594,'','','','','Test',32,'',150 from dual union all
select 70533,'2017-S74','2017-08-28 20:58:00','military missile','success',583,'KN-17','11831805','','','Test',1449,'',550 from dual union all
select 70534,'2017-S75','2017-08-29 00:00:00','military missile','success',747,'Aegis FTM-27E2 Targ','','MRBM Target','','Target',855,'',200 from dual union all
select 70535,'2017-S76','2017-09-03 21:00:00','military missile','success',593,'','','','','Test',611,'',150 from dual union all
select 7054,'1960-S386','1960-11-01 00:00:00','military missile','success',872,'ZW-11','','','','Test',1884,'',40 from dual union all
select 7055,'1960-S387','1960-11-30 00:00:00','military missile','success',81,'009','','','','Reentry Test',313,'',150 from dual union all
select 7056,'1960-S388','1960-12-01 00:00:00','military missile','success',347,'','','','','Test',811,'',100 from dual union all
select 7057,'1960-S389','1960-12-01 00:00:00','military missile','success',347,'','','','','Test',811,'',100 from dual union all
select 7058,'1960-S39','1960-02-25 18:02:00','military missile','success',916,'105','P-01','','','Test',256,'',180 from dual union all
select 7059,'1960-S390','1960-11-30 01:12:00','military missile','failure',155,'4E','Mk 3-1B','R&D','','Test',228,'',100 from dual union all
select 706,'1947-S01','1947-01-10 21:13:00','suborbital rocket','success',1470,'18','','','','CR',1863,'',116 from dual union all
select 7060,'1960-S391','1960-11-30 05:01:00','suborbital rocket','success',30,'AA3.284','MHD waves','','','Ionos',428,'',241 from dual union all
select 7061,'1960-S392','1960-12-01 00:00:00','military missile','success',872,'ZW-12/20013','Guidance test','','','Test',1885,'',40 from dual union all
select 7062,'1960-S393','1960-12-05 00:00:00','military missile','success',929,'A2X-2','','','','Test',244,'',1000 from dual union all
select 7063,'1960-S394','1960-12-06 18:44:00','suborbital rocket','success',673,'','PHOENIX','','','Ionos',1016,'',367 from dual union all
select 7064,'1960-S395','1960-12-08 00:00:00','military missile','failure',1009,'V-1000 target','8A62 PRO Target C-2','V-1000 Target','','Target',321,'',300 from dual union all
select 7065,'1960-S396','1960-12-08 16:52:09','suborbital rocket','success',839,'NASA 10.25CI','GCA Ionos 2','','','Ionos',1821,'',152 from dual union all
select 7066,'1960-S397','1960-12-09 11:20:00','suborbital rocket','success',839,'NASA 10.12CA','GCA Release 8 Na','','','Aeronomy',1821,'',146 from dual union all
select 7067,'1960-S398','1960-12-10 00:00:00','military missile','success',1009,'V-1000 target','8A62 PRO Target C-3','V-1000 Target','','Target',321,'',300 from dual union all
select 7068,'1960-S399','1960-12-10 22:30:00','suborbital rocket','success',624,'NASA 8.05CA','Na release','','','Aeronomy',1821,'',716 from dual union all
select 7069,'1960-S40','1960-02-27 06:53:00','suborbital rocket','success',49,'AA4.360C','JHU Aurora 1','','','Aurora',428,'',117 from dual union all
select 707,'1947-S02','1947-02-20 18:16:00','suborbital rocket','success',1470,'20','Blossom I/UM Aeron','','','Ionos/Bio',1863,'',109 from dual union all
select 7070,'1960-S400','1960-12-11 00:00:00','military missile','success',976,'','8K63 UBP-99','','','Test',636,'',402 from dual union all
select 7071,'1960-S401','1960-12-12 00:00:00','military missile','success',976,'','8K63 UBP-100','','','Test',636,'',402 from dual union all
select 7072,'1960-S402','1960-12-12 18:42:00','military missile','success',916,'207','P-09','','','Test',257,'',180 from dual union all
select 7073,'1960-S403','1960-12-12 22:36:00','suborbital rocket','success',624,'NASA 8.08GE','Magnetic fields','','','Magnetospheric',1821,'',1140 from dual union all
select 7074,'1960-S404','1960-12-13 20:08:00','military missile','success',1367,'267','CTL','','ACTION TOWN?','OT',1677,'',520 from dual union all
select 7075,'1960-S405','1960-12-14 00:00:00','suborbital rocket','unknown',673,'','','','','Aeron?',1016,'',300 from dual union all
select 7076,'1960-S406','1960-12-14 00:00:00','suborbital rocket','unknown',673,'','','','','Aeron?',1016,'',300 from dual union all
select 7077,'1960-S407','1960-12-14 16:52:00','suborbital rocket','success',839,'NASA 10.06GA','GSFC Grenade 5','','','Aeronomy',1821,'',110 from dual union all
select 7078,'1960-S408','1960-12-15 00:00:00','military missile','success',976,'','8K63 UBP-101','','','Test',636,'',402 from dual union all
select 7079,'1960-S409','1960-12-15 00:00:00','military missile','success',976,'','8K63 UBP-102','','','Test',636,'',402 from dual union all
select 708,'1947-S03','1947-03-07 18:23:00','suborbital rocket','success',1470,'21','','','','Solar UV/CR/Ionos',1863,'',161 from dual union all
select 7080,'1960-S41','1960-02-27 23:20:00','ballistic missile test','success',1177,'3','E60 Echo Test','','','SC Test',1821,'',362 from dual union all
select 7081,'1960-S410','1960-12-15 00:00:00','military missile','success',976,'','8K63 UBP-103','','','Test',636,'',402 from dual union all
select 7082,'1960-S411','1960-12-15 00:00:00','military missile','success',976,'','8K63 UBP-104','','','Test',636,'',402 from dual union all
select 7083,'1960-S412','1960-12-16 00:00:00','suborbital rocket','unknown',673,'','','','','Aeron?',1016,'',300 from dual union all
select 7084,'1960-S413','1960-12-16 20:35:33','military missile','success',154,'99D','Mk 3-3','R&D','HOT SHOT','Test',1667,'',1800 from dual union all
select 7085,'1960-S414','1960-12-17 00:00:00','military missile','success',1009,'V-1000 target','8A62 PRO Target C-4','V-1000 Target','','Target',321,'',300 from dual union all
select 7086,'1960-S415','1960-12-19 16:15:00','test rocket','success',1054,'MRLV-3','MR-1A','','','Test',279,'',210 from dual;
insert into LAUNCH
select 7087,'1960-S416','1960-12-20 00:00:00','military missile','success',976,'','8K63 UBP-105','','','Test',636,'',402 from dual union all
select 7088,'1960-S417','1960-12-22 18:34:00','military missile','success',928,'A1E-13 (A1P-19)','','','','OT',421,'SSBN 601',500 from dual union all
select 7089,'1960-S418','1960-12-22 00:00:00','military missile','success',976,'','8K63 UBP-106','','','Test',636,'',402 from dual union all
select 709,'1947-S04','1947-04-01 20:10:00','suborbital rocket','success',1470,'22','','','','Solar UV/CR',1863,'',129 from dual union all
select 7090,'1960-S419','1960-12-22 00:00:00','military missile','success',976,'','8K63 UBP-107','','','Test',636,'',402 from dual union all
select 7091,'1960-S42','1960-03-01 00:00:00','military missile','success',978,'','','','','Test',636,'',150 from dual union all
select 7092,'1960-S420','1960-12-22 00:00:00','military missile','success',1009,'V-1000 target','8A62 PRO Target C-5','V-1000 Target','','Target',321,'',300 from dual union all
select 7093,'1960-S421','1960-12-23 00:00:00','military missile','success',976,'','8K63 UBP-108','','','Test',636,'',402 from dual union all
select 7094,'1960-S422','1960-12-23 00:00:00','military missile','success',1009,'V-1000 target','8A62 PRO Target C-6','V-1000 Target','','Target',321,'',300 from dual union all
select 7095,'1960-S423','1960-12-24 00:00:00','military missile','success',976,'','8K63 UBP-109','','','Test',636,'',402 from dual union all
select 7096,'1960-S424','1960-12-24 00:00:00','military missile','success',979,'','8K65 LKI-12','','','Test',636,'',675 from dual union all
select 7097,'1960-S425','1960-12-24 00:00:00','military missile','success',1009,'[Spurious]','','V-1000 Target','','Target',321,'',300 from dual union all
select 7098,'1960-S426','1960-12-26 00:00:00','military missile','success',976,'','8K63 UBP-110','','','Test',636,'',402 from dual union all
select 7099,'1960-S427','1960-12-29 00:00:00','military missile','success',976,'','8K63 UBP-111','','','Test',636,'',402 from dual union all
select 71,'1944-A35','1944-12-01 00:00:00','military missile','success',943,'2','','','','Test',764,'',5 from dual union all
select 710,'1947-S05','1947-04-09 00:10:00','suborbital rocket','success',1470,'23','','','','Solar UV/CR',1863,'',103 from dual union all
select 7100,'1960-S428','1960-12-31 00:00:00','military missile','success',979,'','8K65 LKI-13','','','Test',636,'',675 from dual union all
select 7101,'1960-S429','1960-12-31 00:00:00','military missile','success',1009,'V-1000 target','8A62 PRO Target C-7','V-1000 Target','','Target',321,'',300 from dual union all
select 7102,'1960-S43','1960-02-29 00:00:00','military missile','success',1368,'263','Special Test','','','Test',236,'',520 from dual union all
select 7103,'1960-S430','1960-01-01 00:00:00','suborbital rocket','success',1505,'V-A','','','','Aeron?',1490,'',100 from dual union all
select 7104,'1960-S431','1960-01-01 00:00:00','suborbital rocket','success',538,'HAT01','','','','Aeron',1837,'',80 from dual union all
select 7105,'1960-S432','1960-01-01 00:00:00','suborbital rocket','unknown',720,'LT9','','','','Aeron',1837,'',80 from dual union all
select 7106,'1960-S433','1960-01-01 00:00:00','suborbital rocket','unknown',720,'LT10','','','','Aeron',1837,'',80 from dual union all
select 7107,'1960-S434','1960-01-01 00:00:00','suborbital rocket','unknown',720,'LT11','','','','Aeron',1837,'',80 from dual union all
select 7108,'1960-S435','1960-02-01 00:00:00','military missile','success',983,'LKI-7  (End)','','','','Test',636,'',150 from dual union all
select 7109,'1960-S436','1960-06-21 00:00:00','military missile','success',1013,'','8K51 UBP-28','','','Test',636,'',500 from dual union all
select 711,'1947-S06','1947-04-17 23:22:00','suborbital rocket','success',1470,'24','Grenades','','','Test/Aeron',1863,'',140 from dual union all
select 7110,'1960-S437','1960-07-21 00:00:00','military missile','success',1013,'','8K51 UBP-29','','','Test',636,'',500 from dual union all
select 7111,'1960-S438','1960-06-01 00:00:00','military missile','success',1009,'','8A62 UBP','','','Test',636,'',500 from dual union all
select 7112,'1960-S44','1960-03-01 22:11:00','suborbital rocket','success',838,'NASA 03.01GS','','','','Solar UV?',1821,'',212 from dual union all
select 7113,'1960-S45','1960-03-02 18:32:00','suborbital rocket','success',1490,'AGI22','Na release','','','Aeronomy',668,'',188 from dual union all
select 7114,'1960-S46','1960-03-02 20:06:00','military missile','success',1367,'272','CTL','','CENTER BOARD','OT',1677,'',520 from dual union all
select 7115,'1960-S47','1960-03-03 22:50:00','suborbital rocket','success',838,'NASA 03.02GS','','','','Solar UV?',1821,'',212 from dual union all
select 7116,'1960-S48','1960-03-05 00:00:00','suborbital rocket','success',1490,'AGI21','Na release','','','Aeronomy',668,'',187 from dual union all
select 7117,'1960-S49','1960-03-08 13:10:00','military missile','success',154,'42D','Mk 3-IX-211','R&D D/AIG guid.test OL1','','Test',226,'',1800 from dual union all
select 7118,'1960-S50','1960-03-09 00:00:00','military missile','success',928,'A1X-14','','','','Test',244,'',500 from dual union all
select 7119,'1960-S51','1960-03-15 00:00:00','military missile','success',1053,'CC-2014','T-1 Capsule /263 km','','','OT',1854,'',90 from dual union all
select 712,'1947-S07','1947-05-15 23:08:00','suborbital rocket','failure',1470,'26','','','','Solar UV',1863,'',122 from dual union all
select 7120,'1960-S52','1960-03-16 00:00:00','suborbital rocket','success',42,'TATTLETALE PCC 3','','','','Sigint Test?',394,'',100 from dual union all
select 7121,'1960-S53','1960-03-16 21:25:00','suborbital rocket','success',42,'NASA 06.01UI','UM ionos probe','','','Ionos',428,'',337 from dual union all
select 7122,'1960-S54','1960-03-17 23:55:00','military missile','success',1017,'L1-5','8K74 LKI-5','','','Test',480,'',1350 from dual union all
select 7123,'1960-S55','1960-03-18 00:00:00','military missile','success',928,'A1X-15','','','','Test',245,'',500 from dual union all
select 7124,'1960-S56','1960-03-20 00:00:00','military missile','success',976,'','8K63 UBP-38','','','Test',636,'',402 from dual union all
select 7125,'1960-S57','1960-03-22 00:00:00','military missile','success',1419,'G-5','RVX-4A','?','','Test',230,'',1000 from dual union all
select 7126,'1960-S58','1960-03-22 01:22:00','military missile','success',1053,'CC-2020','-            322 km','','','Test',280,'',90 from dual union all
select 7127,'1960-S59','1960-03-23 11:20:00','suborbital rocket','success',26,'','AL5','','','Aeron',1837,'',81 from dual union all
select 7128,'1960-S60','1960-03-24 00:00:00','military missile','success',976,'','8K63 UBP-39','','','Test',636,'',402 from dual union all
select 7129,'1960-S61','1960-03-24 02:06:00','military missile','success',1017,'L1-6','8K74 LKI-6','','','Test',480,'',1350 from dual union all
select 713,'1947-S08','1947-07-29 12:55:00','suborbital rocket','success',1470,'30','','','','CR/Solar UV',1863,'',159 from dual union all
select 7130,'1960-S62','1960-03-25 00:00:00','military missile','success',928,'A1X-16','','','','Test',245,'',500 from dual union all
select 7131,'1960-S63','1960-03-25 00:00:00','military missile','success',976,'','8K63 UBP-40','','','Test',636,'',402 from dual union all
select 7132,'1960-S64','1960-03-25 18:40:00','suborbital rocket','success',35,'NASA 04.12GT','','','','Test/Meteorites',1821,'',212 from dual union all
select 7133,'1960-S65','1960-03-28 00:00:00','military missile','success',976,'','8K63 UBP-41','','','Test',636,'',402 from dual union all
select 7134,'1960-S66','1960-03-29 00:00:00','suborbital rocket','success',1438,'TB Ia','D58 Trailblazer','','','Reentry Test',1821,'',260 from dual union all
select 7135,'1960-S67','1960-03-29 00:00:00','military missile','failure',928,'A1X-18','Integrated test','','','Test',421,'EAG-154',300 from dual union all
select 7136,'1960-S68','1960-03-29 00:00:00','military missile','success',976,'','8K63 UBP-42','','','Test',636,'',402 from dual union all
select 7137,'1960-S69','1960-03-31 00:00:00','military missile','success',976,'','8K63 UBP-43','','','Test',636,'',402 from dual union all
select 7138,'1960-S70','1960-04-01 23:55:00','ballistic missile test','success',1177,'4','E60 Echo Test','','','SC Test',1821,'',320 from dual union all
select 7139,'1960-S71','1960-04-07 02:06:00','suborbital rocket','success',30,'AA3.221','IR studies','','','Solar IR',394,'',236 from dual union all
select 714,'1947-S09','1947-10-09 19:15:00','suborbital rocket','success',1470,'27','','','','Solar UV/Aeron',1863,'',156 from dual union all
select 7140,'1960-S72','1960-04-08 18:11:00','military missile','failure',1419,'C-5','RVX-3','','','Test',231,'',1000 from dual union all
select 7141,'1960-S73','1960-04-08 12:04:00','suborbital rocket','success',839,'OB6.03','','','','Aeron/Ionos',428,'',100 from dual union all
select 7142,'1960-S74','1960-04-12 03:18:00','suborbital rocket','success',1195,'SL51','RAE Test 9','','','Test',1839,'',218 from dual union all
select 7143,'1960-S75','1960-04-19 22:30:00','military missile','success',1053,'CC-2021','T-1 Capsule  104 km','','','OT 46AG',1854,'',90 from dual union all
select 7144,'1960-S76','1960-04-19 14:15:00','suborbital rocket','success',49,'NN3.60','NN3.60 Pinhole cam','','','Solar UV/Solar EUV',1895,'',222 from dual union all
select 7145,'1960-S77','1960-04-20 18:30:00','military missile','success',916,'106','P-02','','','Test',256,'',180 from dual union all
select 7146,'1960-S78','1960-04-21 20:55:00','military missile','success',1419,'G-6','RVX-4A','','','Test',230,'',1000 from dual union all
select 7147,'1960-S79','1960-04-21 10:35:00','suborbital rocket','success',1194,'SL62','UCL GC-9','','','Aeronomy',1839,'',141 from dual union all
select 7148,'1960-S80','1960-04-21 15:00:00','suborbital rocket','success',30,'AA3.283C','Magnetometer','','','Fields',394,'',245 from dual union all
select 7149,'1960-S81','1960-04-22 19:39:01','military missile','success',154,'25D','Mk 2-4','IOC ORT','QUICK START','Test',1666,'',1800 from dual union all
select 715,'1947-S10','1947-10-18 07:47:00','military missile','success',8,'010T','','','','Test',636,'',86 from dual union all
select 7150,'1960-S82','1960-04-23 21:45:00','suborbital rocket','success',35,'NASA 04.10GT','','','','Test',1821,'',246 from dual union all
select 7151,'1960-S83','1960-04-26 01:13:00','military missile','success',928,'A1X-22','','','','Test',251,'',500 from dual union all
select 7152,'1960-S84','1960-04-27 04:18:00','suborbital rocket','success',30,'NASA 04.04GG','UV star photometer','','','UV Astron',1821,'',200 from dual union all
select 7153,'1960-S85','1960-04-28 14:50:00','military missile','success',873,'ZW-6/107','6 First Silo','','','Test',1884,'',40 from dual union all
select 7154,'1960-S86','1960-04-28 20:18:00','military missile','success',1419,'C-6','P13 RV','','','Test',231,'',1000 from dual union all
select 7155,'1960-S87','1960-04-29 23:45:00','military missile','success',928,'A1X-23','','','','Test',251,'',500 from dual union all
select 7156,'1960-S88','1960-04-30 03:45:00','military missile','success',928,'A1X-25','','','','Test',244,'',500 from dual union all
select 7157,'1960-S89','1960-04-29 00:00:00','military missile','success',976,'','8K63 UBP-44','','','Test',636,'',402 from dual union all
select 7158,'1960-S90','1960-04-29 15:47:00','suborbital rocket','success',35,'NASA 04.09GA','Composition','','','Aeronomy',1821,'',242 from dual union all
select 7159,'1960-S91','1960-05-01 00:00:00','military missile','success',976,'','8K63 UBP-47','','','Test',636,'',402 from dual union all
select 716,'1947-S11','1947-10-20 08:14:00','military missile','success',8,'04T','','','','Test',636,'',85 from dual union all
select 7160,'1960-S92','1960-05-02 00:00:00','military missile','success',883,'004','','','','Reentry Test',313,'',150 from dual union all
select 7161,'1960-S93','1960-05-05 00:00:00','military missile','success',976,'','8K63 UBP-45','','','Test',636,'',402 from dual union all
select 7162,'1960-S94','1960-05-05 00:00:00','military missile','success',883,'005','(or May 19)','','','Reentry Test',313,'',150 from dual union all
select 7163,'1960-S95','1960-05-06 00:00:00','military missile','success',976,'','8K63 UBP-46','','','Test',636,'',402 from dual union all
select 7164,'1960-S96','1960-05-10 16:00:00','military missile','success',916,'107','P-03','','','Test',256,'',180 from dual union all
select 7165,'1960-S97','1960-05-13 21:25:00','military missile','success',1419,'G-7','Mk 4-1 RV','','','Test',230,'',1000 from dual union all
select 7166,'1960-S98','1960-05-18 00:00:00','military missile','success',928,'A1X-30','','','','Test',245,'',500 from dual union all
select 7167,'1960-S99','1960-05-18 21:04:00','suborbital rocket','failure',214,'NRC BB-1-05','','','','Test',428,'',89 from dual union all
select 7168,'1960-U01','1960-04-15 15:06:45','orbital','failure',1513,'L1-9','','E-3 No. 1','','Sat',480,'',200000 from dual union all
select 7169,'1960-U02','1960-09-21 13:01:53','ballistic missile test','success',247,'D-1','HETS','HETS','','Magnetospheric',235,'',26700 from dual union all
select 717,'1947-S12','1947-10-28 13:47:00','military missile','success',8,'03T','','','','Test',636,'',87 from dual union all
select 7170,'1960-U03','1960-10-04 15:23:00','ballistic missile test','success',1138,'ST-2','Radiation Probe','ST-2','','Plasma',1812,'',5600 from dual union all
select 7171,'1960-W01','1960-02-15 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1057,'',61 from dual union all
select 7172,'1960-W02','1960-02-17 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1057,'',61 from dual union all
select 7173,'1960-W03','1960-03-02 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1057,'',65 from dual union all
select 7174,'1960-W04','1960-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7175,'1960-W05','1960-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7176,'1960-W06','1960-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual;
insert into LAUNCH
select 7177,'1960-W07','1960-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7178,'1960-W08','1960-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7179,'1960-W09','1960-07-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 718,'1947-S13','1947-11-02 15:14:00','military missile','success',8,'14N','','','','Test',636,'',88 from dual union all
select 7180,'1960-W10','1960-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7181,'1960-W11','1960-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7182,'1960-W12','1960-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7183,'1960-W13','1960-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7184,'1960-W14','1960-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7185,'1960-W15','1960-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7186,'1960-W16','1960-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7187,'1960-W17','1960-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7188,'1960-W18','1960-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7189,'1960-W19','1960-08-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 719,'1947-S14','1947-11-04 15:02:00','military missile','success',8,'01T','','','','Test',636,'',89 from dual union all
select 7190,'1960-W20','1960-09-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7191,'1960-W21','1960-09-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7192,'1960-W22','1960-09-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7193,'1960-W23','1960-09-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7194,'1960-W24','1960-09-01 00:00:00','suborbital rocket','success',1287,'','','','','Test',1364,'',60 from dual union all
select 7195,'1960-W25','1960-10-12 00:15:00','suborbital rocket','success',716,'','','','','Aeron',394,'',75 from dual union all
select 7196,'1960-W26','1960-10-13 00:45:00','suborbital rocket','success',716,'','','','','Aeron',1895,'',62 from dual union all
select 7197,'1960-W27','1960-10-25 23:42:00','suborbital rocket','success',716,'','','','','Aeron',394,'',65 from dual union all
select 7198,'1960-W28','1960-10-27 23:05:00','suborbital rocket','success',716,'','','','','Aeron',394,'',69 from dual union all
select 7199,'1960-W29','1960-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 72,'1944-A36','1944-12-01 00:00:00','military missile','success',943,'3','','','','Test',764,'',5 from dual union all
select 720,'1947-S15','1947-11-13 08:30:00','military missile','success',8,'22N','','','','Test',636,'',89 from dual union all
select 7200,'1960-W30','1960-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7201,'1960-W31','1960-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7202,'1960-W32','1960-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7203,'1960-W33','1960-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7204,'1960-W34','1960-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7205,'1960-W35','1960-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7206,'1960-W36','1960-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7207,'1960-W37','1960-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7208,'1960-W38','1960-11-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 7209,'1960-W39','1960-10-31 23:00:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',69 from dual union all
select 721,'1947-S16','1947-11-13 14:00:00','military missile','success',8,'19N','','','','Test',636,'',89 from dual union all
select 7210,'1960-W40','1960-11-08 14:04:59','suborbital rocket','success',725,'','','','','Meteo',1244,'SHIP-SOV',50 from dual union all
select 7211,'1960-W41','1960-11-17 22:12:00','suborbital rocket','success',716,'','','','','Aeron',394,'',60 from dual union all
select 7212,'1960-W42','1960-11-21 10:50:00','suborbital rocket','success',716,'','','','','Aeron',394,'',65 from dual union all
select 7213,'1960-W43','1960-12-05 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1250,'SHIP-SOV',55 from dual union all
select 7214,'1960-W44','1960-12-08 11:00:00','suborbital rocket','success',725,'','','','','Meteo',1252,'SHIP-SOV',50 from dual union all
select 7215,'1960-W45','1960-12-09 10:54:59','suborbital rocket','success',725,'','','','','Meteo',1253,'SHIP-SOV',52 from dual union all
select 7216,'1960-W46','1960-07-13 18:39:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7217,'1960-W47','1960-07-13 19:15:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7218,'1960-W48','1960-08-08 17:01:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7219,'1960-W49','1960-08-08 17:20:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 722,'1947-S17','1947-12-08 21:42:00','suborbital rocket','success',1470,'28','Blossom II/UM Aeron','','','Solar X/Aeron',1863,'',105 from dual union all
select 7220,'1960-W50','1960-08-11 15:00:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7221,'1960-W51','1960-08-11 16:00:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7222,'1960-W52','1960-08-12 13:30:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7223,'1960-W53','1960-08-12 14:30:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7224,'1960-W54','1960-08-12 15:30:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7225,'1960-W55','1960-08-12 16:45:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7226,'1960-W56','1960-08-15 18:00:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7227,'1960-W57','1960-08-15 18:20:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7228,'1960-W58','1960-08-19 17:00:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7229,'1960-W59','1960-08-19 17:20:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 723,'1948-A01','1948-01-01 00:00:00','atmospheric rocket','success',365,'','E15 Model Test','','','Test',1821,'',10 from dual union all
select 7230,'1960-W60','1960-08-22 16:26:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7231,'1960-W61','1960-08-22 16:50:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7232,'1960-W62','1960-08-24 18:12:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7233,'1960-W63','1960-08-24 18:56:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7234,'1960-W64','1960-08-25 15:03:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7235,'1960-W65','1960-08-25 15:20:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7236,'1960-W66','1960-08-25 15:40:00','suborbital rocket','success',715,'','','','','Aeron',276,'',60 from dual union all
select 7237,'1960-W67','1960-01-22 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1057,'',57 from dual union all
select 7238,'1960-W68','1960-01-25 00:00:00','suborbital rocket','success',715,'Loki II','','','','Aeron',1057,'',70 from dual union all
select 7239,'1960-W69','1960-01-18 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1057,'',59 from dual union all
select 724,'1948-A02','1948-03-19 23:10:00','suborbital rocket','failure',1470,'39','Blossom IIA','','','Aeron/B field',1863,'',5 from dual union all
select 7240,'1960-W70','1960-02-03 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1057,'',51 from dual union all
select 7241,'1960-W71','1960-02-04 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1057,'',56 from dual union all
select 7242,'1960-W72','1960-02-08 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1057,'',55 from dual union all
select 7243,'1960-W73','1960-02-16 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1057,'',50 from dual union all
select 7244,'1960-W74','1960-11-01 00:00:00','suborbital rocket','success',88,'','Robin','','','Aeron',646,'',50 from dual union all
select 7245,'1960-Y01','1960-05-18 00:00:00','suborbital rocket','success',126,'','Test','','','Test',1821,'',80 from dual union all
select 7246,'1960-Y02','1960-05-19 00:00:00','suborbital rocket','success',126,'','Test','','','Test',1821,'',80 from dual union all
select 7247,'1960-Y03','1960-08-13 07:19:00','suborbital rocket','success',1503,'','TP Annie','','','Aeron',394,'',120 from dual union all
select 7248,'1960-Y04','1960-08-13 09:47:00','suborbital rocket','success',1503,'','TP Norma','','','Aeron',394,'',97 from dual union all
select 7249,'1961 A ALP','1961-09-13 14:04:00','orbital','success',154,'88D','MA-4','Mercury SC8A','','Sat',229,'',null from dual union all
select 725,'1948-A03','1948-05-26 00:00:00','military missile','failure',830,'','','','','Test',643,'',1 from dual union all
select 7250,'1961 A BET','1961-09-17 21:00:00','orbital','success',1361,'324/FTV 1114','Discoverer 31','CORONA C''''''-3','CANE POLE','Sat TAG31',1673,'',null from dual union all
select 7251,'1961 A DEL','1961-10-21 13:53:03','orbital','success',145,'105D','Midas 4','Midas ETS-II F2','BIG TOWN','Sat',1020,'',null from dual union all
select 7252,'1961 A EPS','1961-11-05 20:00:30','orbital','success',1361,'330/FTV 1117','Discoverer 34','CORONA C''-9','FOG CUTTER','Sat TAG34',1673,'',null from dual union all
select 7253,'1961 A ETA','1961-11-15 22:26:00','orbital','success',1359,'305/AB009','Transit 4B','Transit 4B','','Sat',234,'',null from dual union all
select 7254,'1961 A GAM','1961-10-13 19:22:00','orbital','success',1361,'328/FTV 1115','Discoverer 32','CORONA C''''''-4','CAP PISTOL','Sat TAG32',1678,'',null from dual union all
select 7255,'1961 A IOT','1961-11-29 15:07:57','orbital','success',154,'93D','MA-5','Mercury SC9','','Sat',229,'',null from dual union all
select 7256,'1961 A KAP','1961-12-12 20:40:21','orbital','success',1361,'325/FTV 1119','Discoverer 36','CORONA C''''''-5','SILVER STRIP','Sat TAG36',1678,'',null from dual union all
select 7257,'1961 A LAM','1961-12-22 19:12:33','orbital','success',145,'114D','Samos 5','Samos E-5','OCEAN WAY','Sat',1020,'',null from dual union all
select 7258,'1961 A THE','1961-11-18 08:12:21','orbital','success',145,'117D (AA2)','Ranger 2','NASA P-33','','Sat',227,'',null from dual union all
select 7259,'1961 A ZET','1961-11-15 21:22:46','orbital','success',1361,'326/FTV 1118','Discoverer 35','CORONA C''-10','CAT FIGHT','Sat TAG35',1678,'',null from dual union all
select 726,'1948-A04','1948-06-01 00:00:00','atmospheric rocket','success',565,'','Rascal Model','','','Test',1821,'',5 from dual union all
select 7260,'1961 ALP','1961-01-31 20:21:19','orbital','success',144,'70D','Samos 2','SAMOS F-1','JAYHAWK JAMBOREE','Sat',1019,'',null from dual union all
select 7261,'1961 BET','1961-02-04 01:18:03','orbital','failure',802,'L1-7V','Tyazholiy Sputnik','1VA No. 1','','Sat',480,'',318 from dual union all
select 7262,'1961 CHI','1961-08-25 18:29:44','orbital','success',1138,'ST-6','Explorer 13','NASA S-55A','','Sat',1812,'',null from dual union all
select 7263,'1961 DEL','1961-02-16 13:05:00','orbital','success',1138,'ST-4','Explorer 9','NASA S-56A','','Sat',1812,'',null from dual union all
select 7264,'1961 EPS','1961-02-17 20:25:02','orbital','success',1361,'298/FTV 1104','Discoverer 20','ARGON 1','SPIRIT LEVEL','Sat TAG20',1678,'',null from dual union all
select 7265,'1961 ETA','1961-02-22 03:45:00','orbital','success',1359,'313/AB007','Transit 3B','Transit 3B','','Sat',234,'',null from dual union all
select 7266,'1961 GAM','1961-02-12 00:34:36','deep space','success',802,'L1-6V','Venera','1VA No. 2','','Sat',480,'',null from dual;
insert into LAUNCH
select 7267,'1961 IOT','1961-03-25 05:54:00','orbital','success',1512,'E103-15','Korabl''-Sputnik-5','Vostok-3 No. 2','','Sat',480,'',null from dual union all
select 7268,'1961 KAP','1961-03-25 15:17:04','orbital','success',1375,'295/D4','Explorer 10','P-14','','Sat',233,'',null from dual union all
select 7269,'1961 LAM','1961-04-08 19:21:08','orbital','success',1361,'307/FTV 1106','Discoverer 23','ARGON 2','RUNNING BOARD','Sat TAG23',1679,'',null from dual union all
select 727,'1948-A05','1948-06-15 22:30:00','atmospheric rocket','success',878,'47-J','Nike 48-1 I dummy','','','Test',1863,'',10 from dual union all
select 7270,'1961 MU','1961-04-12 06:07:00','orbital','success',1512,'E103-16','Vostok','Vostok-3A No. 3','','Sat',480,'',null from dual union all
select 7271,'1961 NU','1961-04-27 14:16:38','orbital','success',634,'AM-19E','Explorer 11','NASA S-15','','Sat',249,'',null from dual union all
select 7272,'1961 OME','1961-09-12 19:59:00','orbital','success',1361,'310/FTV 1113','Discoverer 30','CORONA C''''''-2','TWISTED BRAIDS','Sat TAG30',1679,'',null from dual union all
select 7273,'1961 OMI','1961-06-29 04:23:00','orbital','success',1359,'315/AB008','Transit 4A/GRAB','Transit 4A','','Sat',234,'',null from dual union all
select 7274,'1961 PHI','1961-08-23 10:04:10','orbital','success',145,'111D (AA1)','Ranger 1','NASA P-32','','Sat',227,'',null from dual union all
select 7275,'1961 PI','1961-07-07 23:29:48','orbital','success',1361,'308/FTV 1109','Discoverer 26','CORONA C''-6','HIGH WING','Sat TAG26',1679,'',null from dual union all
select 7276,'1961 PSI','1961-08-30 20:00:00','orbital','success',1361,'323/FTV 1112','Discoverer 29','CORONA C''''''-1','FULL BLOWER','Sat TAG29',1678,'',null from dual union all
select 7277,'1961 RHO','1961-07-12 10:25:06','orbital','success',1375,'286/D5','Tiros 3','NASA A-3','','Sat',233,'',null from dual union all
select 7278,'1961 SIG','1961-07-12 15:11:46','orbital','success',145,'97D','Midas 3','Midas ETS-II F1','POLAR ORBIT','Sat',1020,'',null from dual union all
select 7279,'1961 TAU','1961-08-06 06:00:00','orbital','success',1512,'E103-17','Vostok-2','Vostok-3A No. 4','','Sat',480,'',null from dual union all
select 728,'1948-A06','1948-06-17 22:44:00','atmospheric rocket','success',877,'47-N','Nike 48-0 J dummy','','','Test',1863,'',1 from dual union all
select 7280,'1961 THE','1961-03-09 06:29:00','orbital','success',1512,'E103-14','Korabl''-Sputnik-4','Vostok-3 No. 1','','Sat',480,'',null from dual union all
select 7281,'1961 UPS','1961-08-16 03:21:05','orbital','success',1375,'312/D6','Explorer 12','NASA S-3','','Sat',233,'',null from dual union all
select 7282,'1961 XI','1961-06-16 23:02:52','orbital','success',1361,'303/FTV 1107','Discoverer 25','CORONA C''-5','MARKED CARD','Sat TAG25',1673,'',null from dual union all
select 7283,'1961 ZET','1961-02-18 22:57:58','orbital','success',1361,'261/FTV 1102','Discoverer 21','CORONA RM-2','BENCH WARRANT','Sat TAG21',1679,'',null from dual union all
select 7284,'1961-A01','1961-01-01 00:00:00','suborbital rocket','failure',88,'','','','','Aeron',428,'',0 from dual union all
select 7285,'1961-A02','1961-01-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7286,'1961-A03','1961-01-01 00:00:00','atmospheric rocket','success',925,'','Target test','','','Test',1057,'',25 from dual union all
select 7287,'1961-A04','1961-01-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7288,'1961-A05','1961-01-01 00:00:00','suborbital rocket','failure',716,'','','','','Aeron',428,'',0 from dual union all
select 7289,'1961-A06','1961-01-01 00:00:00','suborbital rocket','failure',716,'','','','','Aeron',428,'',0 from dual union all
select 729,'1948-A07','1948-06-24 16:36:00','atmospheric rocket','success',878,'47-K','Nike 48-1 K dummy','','','Test',1863,'',10 from dual union all
select 7290,'1961-A07','1961-01-01 00:00:00','military missile','success',993,'K-1.1','','','','Test',327,'S-229',1 from dual union all
select 7291,'1961-A08','1961-01-01 00:00:00','atmospheric rocket','success',1489,'','Ramjet','','','Test',671,'',25 from dual union all
select 7292,'1961-A09','1961-01-01 00:00:00','military missile','success',1469,'','','','','ABM Test',182,'',25 from dual union all
select 7293,'1961-A10','1961-01-01 00:00:00','military missile','success',1469,'','','','','ABM Test',182,'',25 from dual union all
select 7294,'1961-A100','1961-04-07 00:00:00','atmospheric rocket','success',861,'','Reentry Stability','','','Reentry Test',1821,'',10 from dual union all
select 7295,'1961-A101','1961-04-13 00:00:00','military missile','success',771,'GM-56-1931','','','','Test',222,'',12 from dual union all
select 7296,'1961-A102','1961-04-14 19:24:00','military missile','failure',1017,'E 15001- 05','8K74 UBP-5','','','Test',562,'',1000 from dual union all
select 7297,'1961-A103','1961-04-14 15:30:00','atmospheric rocket','success',835,'Smoke 010','Smoke','','','Aeronomy',1821,'',10 from dual union all
select 7298,'1961-A104','1961-04-14 19:35:00','atmospheric rocket','success',835,'Smoke 012','Smoke','','','Aeronomy',1821,'',12 from dual union all
select 7299,'1961-A105','1961-04-15 13:58:29','military missile','failure',981,'7L-12T','8K64 LKI-4','','','Test',563,'',0 from dual union all
select 73,'1944-A37','1944-12-01 00:00:00','military missile','success',943,'4','','','','Test',764,'',5 from dual union all
select 730,'1948-A08','1948-06-29 16:30:00','atmospheric rocket','success',877,'47-17','Nike 48-0 14R','','','Test',1863,'',29 from dual union all
select 7300,'1961-A106','1961-04-21 14:59:00','military missile','failure',981,'6L-11T','8K64 LKI-5','','','Test',563,'',0 from dual union all
select 7301,'1961-A107','1961-04-21 00:00:00','atmospheric rocket','success',620,'J2','','','','Test',1836,'',29 from dual union all
select 7302,'1961-A108','1961-04-21 18:05:17','suborbital spaceplane','success',1520,'2-15-29','Mach 4, Aero','','','Test',379,'NB-52 003',32 from dual union all
select 7303,'1961-A109','1961-04-25 02:26:00','military missile','failure',1018,'E 15001-02T','8K75 LKI-3','','','Test',569,'',0 from dual union all
select 7304,'1961-A11','1961-01-01 00:00:00','military missile','success',1469,'','','','','ABM Test',182,'',25 from dual union all
select 7305,'1961-A110','1961-04-26 00:00:00','atmospheric rocket','success',1065,'R1K','','','','Test',2,'',20 from dual union all
select 7306,'1961-A111','1961-04-27 22:32:00','suborbital rocket','failure',839,'NASA 10.34GA','GSFC Grenade 9','','','Aeronomy',1821,'',18 from dual union all
select 7307,'1961-A112','1961-04-28 00:00:00','military missile','success',764,'59-4874  (46)','','','','Test',240,'',12 from dual union all
select 7308,'1961-A113','1961-04-28 14:03:00','test rocket','success',709,'LJ-5B','LJ-VII','','','SC Test',1816,'',4 from dual union all
select 7309,'1961-A114','1961-05-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 731,'1948-A09','1948-09-10 00:00:00','military missile','success',830,'','','','','Test',643,'',15 from dual union all
select 7310,'1961-A115','1961-05-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7311,'1961-A116','1961-05-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7312,'1961-A117','1961-05-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7313,'1961-A118','1961-05-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7314,'1961-A119','1961-05-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7315,'1961-A12','1961-01-01 00:00:00','military missile','success',1469,'','','','','ABM Test',182,'',25 from dual union all
select 7316,'1961-A120','1961-05-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7317,'1961-A121','1961-05-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7318,'1961-A122','1961-05-02 21:15:00','suborbital rocket','success',835,'NASA 12.01GT','Cone test','','','Test',1821,'',14 from dual union all
select 7319,'1961-A123','1961-05-09 16:00:22','ballistic missile test','failure',245,'D-6','HETS A1-2','','','Plasma',236,'',10 from dual union all
select 732,'1948-A10','1948-07-01 16:39:00','atmospheric rocket','success',877,'47-18','Nike 48-0 15R','','','Test',1863,'',10 from dual union all
select 7320,'1961-A124','1961-05-11 00:00:00','military missile','success',771,'GM-56-1942','','','','Test',222,'',12 from dual union all
select 7321,'1961-A125','1961-05-11 00:00:00','military missile','success',771,'GM-56-1946','','','','Test',222,'',12 from dual union all
select 7322,'1961-A126','1961-05-19 00:00:00','suborbital spaceplane','failure',1520,'2- A-30','','','','Test',379,'NB-52 003',14 from dual union all
select 7323,'1961-A127','1961-05-19 18:03:00','suborbital rocket','success',29,'NASA 01.06GP','AMPP FU5','','','Remote sensing',428,'',23 from dual union all
select 7324,'1961-A128','1961-05-24 03:18:06','military missile','failure',981,'9L-9T','8K64 LKI-7','','','Test',566,'',0 from dual union all
select 7325,'1961-A129','1961-05-25 19:53:00','military missile','failure',929,'A2X-9','','','','Test',421,'EAG-154',0 from dual union all
select 7326,'1961-A13','1961-01-01 00:00:00','military missile','success',1469,'','','','','ABM Test',182,'',25 from dual union all
select 7327,'1961-A130','1961-05-25 20:16:35','suborbital spaceplane','success',1520,'2-16-31','Mach 5, SAS, Aero','','','Test',379,'NB-52 003',32 from dual union all
select 7328,'1961-A131','1961-05-29 01:44:23','military missile','failure',1018,'E 103   10','8K75 LKI-4','','','Test',569,'',0 from dual union all
select 7329,'1961-A132','1961-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 733,'1948-A11','1948-07-01 22:00:00','atmospheric rocket','success',878,'47-M','Nike 48-1 L dummy','','','Test',1863,'',10 from dual union all
select 7330,'1961-A133','1961-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7331,'1961-A134','1961-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7332,'1961-A135','1961-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7333,'1961-A136','1961-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7334,'1961-A137','1961-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7335,'1961-A138','1961-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7336,'1961-A139','1961-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7337,'1961-A14','1961-01-01 00:00:00','military missile','success',1469,'','','','','ABM Test',182,'',25 from dual union all
select 7338,'1961-A140','1961-06-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7339,'1961-A141','1961-06-01 00:00:00','military missile','success',771,'GM-56-1949','','','','Test',222,'',12 from dual union all
select 734,'1948-A12','1948-07-08 00:00:00','atmospheric rocket','success',565,'','Rascal Model','','','Test',1821,'',5 from dual union all
select 7340,'1961-A142','1961-06-01 00:00:00','military missile','success',771,'GM-56-1948','','','','Test',222,'',12 from dual union all
select 7341,'1961-A143','1961-06-01 00:00:00','military missile','success',771,'GM-56-1947','','','','Test',222,'',12 from dual union all
select 7342,'1961-A144','1961-06-01 00:00:00','military missile','success',771,'GM-56-1944','','','','Test',222,'',12 from dual union all
select 7343,'1961-A145','1961-06-02 00:00:00','military missile','success',764,'59-4876  (48)','','','','Test',240,'',12 from dual union all
select 7344,'1961-A146','1961-06-02 05:12:14','military missile','failure',1018,'E 15001-01T','8K75 LKI-5','','','Test',569,'',0 from dual union all
select 7345,'1961-A147','1961-06-03 00:00:00','test rocket','success',61,'','','','','Test',664,'',20 from dual union all
select 7346,'1961-A148','1961-06-06 18:00:00','military missile','failure',981,'11L-13T','8K64 LKI-9','','','Test',566,'',0 from dual union all
select 7347,'1961-A149','1961-06-07 00:00:00','test rocket','success',61,'','','','','Test',664,'',20 from dual union all
select 7348,'1961-A15','1961-01-01 00:00:00','military missile','success',1469,'','','','','ABM Test',182,'',25 from dual union all
select 7349,'1961-A150','1961-06-07 21:37:31','military missile','failure',155,'27E','Mk 4-5','R&D Cat II','SURE SHOT','Test',1715,'',0 from dual union all
select 735,'1948-A13','1948-07-08 16:30:00','atmospheric rocket','success',877,'47-11','Nike 48-0 16R','','','Test',1863,'',27 from dual union all
select 7350,'1961-A151','1961-06-09 00:00:00','military missile','success',1469,'','','','','ABM Test',182,'',25 from dual union all
select 7351,'1961-A152','1961-06-16 00:00:00','military missile','success',764,'59-4879  (51)','','','','Test',240,'',12 from dual union all
select 7352,'1961-A153','1961-06-16 07:47:23','military missile','failure',981,'14L-16T','8K64 LKI-10','','','Test',566,'',0 from dual union all
select 7353,'1961-A154','1961-06-20 00:00:00','suborbital spaceplane','failure',1520,'2- A-32','','','','Test',379,'NB-52 008',14 from dual union all
select 7354,'1961-A155','1961-06-21 00:00:00','military missile','success',764,'59-4883  (55)','','','','Test',240,'',12 from dual union all
select 7355,'1961-A156','1961-06-24 03:28:00','military missile','failure',1419,'M-1','Mk 4-4 RV','','','Test',237,'',100 from dual union all
select 7356,'1961-A157','1961-06-23 03:00:00','military missile','failure',155,'17E','Mk 3-1B','R&D','','Test',226,'',10 from dual;
insert into LAUNCH
select 7357,'1961-A158','1961-06-23 22:00:05','suborbital spaceplane','success',1520,'2-17-33','Mach 5, Aero, Stab.','','','Test',379,'NB-52 003',32 from dual union all
select 7358,'1961-A159','1961-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7359,'1961-A16','1961-01-01 00:00:00','military missile','success',1469,'','','','','ABM Test',182,'',25 from dual union all
select 736,'1948-A14','1948-07-13 16:31:00','atmospheric rocket','success',877,'47-14','Nike 48-0 17R','','','Test',1863,'',7 from dual union all
select 7360,'1961-A160','1961-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7361,'1961-A161','1961-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7362,'1961-A162','1961-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7363,'1961-A163','1961-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7364,'1961-A164','1961-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7365,'1961-A165','1961-06-30 17:20:00','suborbital rocket','failure',130,'CRL AA15.303','','','','Test',394,'',29 from dual union all
select 7366,'1961-A166','1961-07-14 16:00:00','atmospheric rocket','success',835,'Smoke 014','','','','Aeronomy',1821,'',14 from dual union all
select 7367,'1961-A167','1961-07-14 21:20:00','atmospheric rocket','success',835,'Smoke 015','','','','Aeronomy',1821,'',14 from dual union all
select 7368,'1961-A168','1961-07-14 21:20:00','atmospheric rocket','success',835,'Smoke 016','','','','Aeronomy',1821,'',14 from dual union all
select 7369,'1961-A169','1961-07-18 00:00:00','military missile','success',764,'60-5397','','','','Test',240,'',12 from dual union all
select 737,'1948-A15','1948-07-14 01:05:00','military missile','failure',1045,'MX-774','','','','Test',1863,'',1 from dual union all
select 7370,'1961-A17','1961-01-06 00:10:00','military missile','failure',916,'208','P-10','','','Test',255,'',0 from dual union all
select 7371,'1961-A170','1961-07-20 00:00:00','atmospheric rocket','success',705,'L5','','','','Test',2,'',20 from dual union all
select 7372,'1961-A171','1961-07-22 07:55:00','military missile','failure',981,'13L-15T','8K64 LKI-11','','','Test',566,'',0 from dual union all
select 7373,'1961-A172','1961-07-26 00:00:00','atmospheric rocket','failure',620,'J3','','','','Test',1836,'',10 from dual union all
select 7374,'1961-A173','1961-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7375,'1961-A174','1961-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7376,'1961-A175','1961-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7377,'1961-A176','1961-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7378,'1961-A177','1961-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7379,'1961-A178','1961-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 738,'1948-A16','1948-07-16 17:40:00','atmospheric rocket','success',878,'48-20','Nike 48-1 18R','','','Test',1863,'',10 from dual union all
select 7380,'1961-A179','1961-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7381,'1961-A18','1961-01-11 18:40:00','military missile','failure',928,'A1E-15 (A1P-23)','','','','OT',421,'SSBN 601',20 from dual union all
select 7382,'1961-A180','1961-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7383,'1961-A181','1961-08-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7384,'1961-A182','1961-08-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7385,'1961-A183','1961-08-01 20:18:00','sounding rocket','failure',129,'AA16.304','','','','Test/Geodetic',1015,'',10 from dual union all
select 7386,'1961-A184','1961-08-03 16:23:00','military missile','failure',1018,'E 103   13','8K75 LKI-8','','','Test',569,'',0 from dual union all
select 7387,'1961-A185','1961-08-04 00:00:00','military missile','success',764,'60-5399','','','','Test',240,'',12 from dual union all
select 7388,'1961-A186','1961-08-11 01:49:00','military missile','failure',916,'315','P-20','','','Test',256,'',5 from dual union all
select 7389,'1961-A187','1961-08-10 18:27:05','suborbital spaceplane','success',1520,'1-22-37','XLR-99, beta-dot','','','Test',379,'NB-52 003',23 from dual union all
select 739,'1948-A17','1948-07-29 18:10:00','atmospheric rocket','success',878,'48-21','Nike 48-1 19R','','','Test',1863,'',10 from dual union all
select 7390,'1961-A188','1961-08-13 04:00:00','military missile','failure',981,'21L','8K64 LKI-13','','','Test',566,'',0 from dual union all
select 7391,'1961-A189','1961-08-18 00:00:00','military missile','failure',929,'A2X-22','','','','Test',421,'EAG-154',5 from dual union all
select 7392,'1961-A19','1961-01-11 00:00:00','suborbital spaceplane','failure',1520,'1- A-34','','','','Test',379,'NB-52 008',14 from dual union all
select 7393,'1961-A190','1961-08-24 00:00:00','atmospheric rocket','success',1065,'Ra23','Ranger 23','','','Test',2,'',20 from dual union all
select 7394,'1961-A191','1961-08-24 00:00:00','atmospheric rocket','success',1065,'','Ranger','','','Test',2,'',20 from dual union all
select 7395,'1961-A192','1961-08-30 17:30:00','military missile','failure',794,'404','Mk 5 RV','R&D','','Test',266,'',1 from dual union all
select 7396,'1961-A193','1961-09-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7397,'1961-A194','1961-09-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7398,'1961-A195','1961-09-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7399,'1961-A196','1961-09-01 00:00:00','military missile','success',764,'60-5398','','','','Test',240,'',12 from dual union all
select 74,'1944-A38','1944-12-01 00:00:00','military missile','success',943,'5','','','','Test',764,'',5 from dual union all
select 740,'1948-A18','1948-07-30 23:10:00','atmospheric rocket','success',878,'48-22','Nike 48-1 20R','','','Test',1863,'',10 from dual union all
select 7400,'1961-A197','1961-09-09 20:02:00','military missile','failure',872,'ZM-1  20025','ZM-1','','','Test',1057,'',1 from dual union all
select 7401,'1961-A198','1961-09-10 18:10:00','military missile','failure',1018,'E 103   11','8K75 LKI-9','','','Test',569,'',0 from dual union all
select 7402,'1961-A199','1961-09-12 22:40:17','suborbital spaceplane','success',1520,'2-18-34','Mach 5, Aero, Stab.','','','Test',379,'NB-52 008',34 from dual union all
select 7403,'1961-A20','1961-01-13 00:00:00','military missile','failure',1469,'','','','','ABM Test',182,'',10 from dual union all
select 7404,'1961-A200','1961-09-15 00:00:00','military missile','success',872,'ZW-25','','','','Test',1884,'',40 from dual union all
select 7405,'1961-A201','1961-09-16 00:00:00','suborbital rocket','success',327,'','','','','Test',353,'',35 from dual union all
select 7406,'1961-A202','1961-09-16 10:02:00','suborbital rocket','failure',838,'NASA 03.09CA','GCA Release 14 Na','','','Aeronomy',1821,'',27 from dual union all
select 7407,'1961-A203','1961-09-19 16:55:06','military missile','failure',1018,'E 103   14','8K75 LKI-10','','','Test',569,'',0 from dual union all
select 7408,'1961-A204','1961-09-22 00:00:00','military missile','success',764,'60-5400','','','','Test',240,'',12 from dual union all
select 7409,'1961-A205','1961-09-22 15:45:00','military missile','failure',981,'16L-18T','8K64 LKI-17','','','Test',566,'',80 from dual union all
select 741,'1948-A19','1948-08-10 16:35:00','atmospheric rocket','success',879,'482-P','Nike 48-2 M dummy','','','Test',1863,'',11 from dual union all
select 7410,'1961-A206','1961-09-28 17:50:25','suborbital spaceplane','success',1520,'2-19-35','Mach 5, Heat Trans.','','','Test',379,'NB-52 008',31 from dual union all
select 7411,'1961-A207','1961-09-29 00:00:00','suborbital spaceplane','failure',1520,'1- A-38','','','','Test',379,'NB-52 003',14 from dual union all
select 7412,'1961-A208','1961-10-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7413,'1961-A209','1961-10-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7414,'1961-A21','1961-01-14 00:00:00','military missile','failure',928,'A1E-14 (A1P-24)','','','','OT',421,'SSBN 601',20 from dual union all
select 7415,'1961-A210','1961-10-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7416,'1961-A211','1961-10-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7417,'1961-A212','1961-10-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7418,'1961-A213','1961-10-01 00:00:00','military missile','success',993,'K-1.1','','','','Test',327,'S-229',1 from dual union all
select 7419,'1961-A214','1961-10-01 00:00:00','military missile','unknown',274,'SIP','','','','Test',1415,'',20 from dual union all
select 742,'1948-A20','1948-08-17 21:00:00','atmospheric rocket','success',878,'48-23','Nike 48-1 21R','','','Test',1863,'',10 from dual union all
select 7420,'1961-A215','1961-10-03 18:40:50','suborbital spaceplane','success',1520,'1-23-39','VO Stability','','','Test',379,'NB-52 003',23 from dual union all
select 7421,'1961-A216','1961-10-05 19:10:00','suborbital rocket','failure',274,'NC17.116','','','','Test/Aeron',1023,'F-4H',10 from dual union all
select 7422,'1961-A217','1961-10-07 18:03:00','military missile','failure',872,'ZM-2','ZM-2','','','Test',1057,'',3 from dual union all
select 7423,'1961-A218','1961-10-08 15:44:00','military missile','failure',1018,'E 15001-22','8K75 LKI-13','','','Test',569,'',0 from dual union all
select 7424,'1961-A219','1961-10-11 15:40:00','military missile','failure',982,'1LU','8K64U LKI-2','','','Test',566,'',0 from dual union all
select 7425,'1961-A22','1961-01-14 00:00:00','military missile','success',1469,'','','','','ABM Test',182,'',25 from dual union all
select 7426,'1961-A220','1961-10-17 18:57:33','suborbital spaceplane','success',1520,'1-24-40','Mach 5, Aero, Stab','','','Test',379,'NB-52 003',33 from dual union all
select 7427,'1961-A221','1961-10-23 00:00:00','military missile','failure',979,'','8K65 UBP-8 29 RD','','','Test',636,'',0 from dual union all
select 7428,'1961-A222','1961-10-24 16:10:00','atmospheric rocket','success',835,'Smoke 018','','','','Aeronomy',1821,'',12 from dual union all
select 7429,'1961-A223','1961-10-24 00:00:00','atmospheric rocket','success',1154,'SEABEE','Hydra','','','Test',1057,'',1 from dual union all
select 743,'1948-A21','1948-08-19 00:00:00','atmospheric rocket','success',365,'','Sparrow Model','','','Test',1821,'',10 from dual union all
select 7430,'1961-A224','1961-10-26 05:54:53','military missile','failure',1018,'E 15001-24','8K75 LKI-15','','','Test',569,'',0 from dual union all
select 7431,'1961-A225','1961-10-27 00:00:00','military missile','success',1469,'','Op K-1','','','ABM Test',182,'',25 from dual union all
select 7432,'1961-A226','1961-10-27 00:00:00','military missile','success',1469,'','Op K-2','','','ABM Test',182,'',25 from dual union all
select 7433,'1961-A227','1961-10-27 00:00:00','suborbital spaceplane','failure',1520,'1- A-41','','','','Test',379,'NB-52',14 from dual union all
select 7434,'1961-A228','1961-11-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7435,'1961-A229','1961-11-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7436,'1961-A23','1961-01-17 18:04:00','suborbital rocket','success',716,'','','','','Aeron',1057,'',37 from dual union all
select 7437,'1961-A230','1961-11-01 00:00:00','atmospheric rocket','success',620,'J4','','','','Test',1836,'',24 from dual union all
select 7438,'1961-A231','1961-11-02 00:00:00','suborbital spaceplane','failure',1520,'1- A-42','','','','Test',379,'NB-52',14 from dual union all
select 7439,'1961-A232','1961-11-03 00:00:00','suborbital spaceplane','failure',1520,'1- A-43','','','','Test',379,'NB-52',14 from dual union all
select 744,'1948-A22','1948-08-19 21:30:00','atmospheric rocket','success',879,'482-Q','Nike 48-2 N dummy','','','Test',1863,'',11 from dual union all
select 7440,'1961-A233','1961-11-09 17:57:17','suborbital spaceplane','success',1520,'2-21-37','Mach 6, Aero, Stab','','','Test',379,'NB-52 008',31 from dual union all
select 7441,'1961-A234','1961-11-10 14:55:00','ballistic missile test','failure',155,'32E','RVX-2A S/N 424A/Pod 13','R&D','','Test/CR/Tech',228,'',40 from dual union all
select 7442,'1961-A235','1961-11-13 00:00:00','test rocket','success',61,'','','','','Test',664,'',20 from dual union all
select 7443,'1961-A236','1961-11-14 00:00:00','military missile','success',764,'60-5396','','','','Test',240,'',12 from dual union all
select 7444,'1961-A237','1961-11-17 00:00:00','test rocket','success',61,'','','','','Test',664,'',20 from dual union all
select 7445,'1961-A238','1961-11-22 15:30:00','atmospheric rocket','success',835,'Smoke 019','','','','Aeronomy',1821,'',12 from dual union all
select 7446,'1961-A239','1961-11-21 00:00:00','suborbital rocket','failure',1197,'SL83','LUX-5/UCL I-7/UCW-3','','','Ionos/Solar X',1839,'',0 from dual;
insert into LAUNCH
select 7447,'1961-A24','1961-01-18 00:00:00','military missile','success',771,'GM-56-1936','','','','Test',222,'',12 from dual union all
select 7448,'1961-A240','1961-12-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7449,'1961-A241','1961-12-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 745,'1948-A23','1948-08-19 14:45:00','suborbital rocket','failure',265,'Bumper 2','','','','Test',1863,'',13 from dual union all
select 7450,'1961-A242','1961-12-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7451,'1961-A243','1961-12-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7452,'1961-A244','1961-12-05 00:00:00','military missile','failure',843,'INH-19','Redstone intercept','','','Test',1895,'',20 from dual union all
select 7453,'1961-A245','1961-12-01 23:29:00','military missile','destroyed before launch',981,'E 226   01-01T','8K64 UBP','Pad explosion','','Test',565,'',0 from dual union all
select 7454,'1961-A246','1961-12-05 00:00:00','military missile','success',764,'59-4887','','','','Test',240,'',12 from dual union all
select 7455,'1961-A247','1961-12-14 00:00:00','military missile','failure',979,'','8K65 UBP-15','','','Test',636,'',0 from dual union all
select 7456,'1961-A248','1961-12-14 21:00:00','military missile','success',872,'ZW-34 20041','Hercules intercept','','','Test',1884,'',23 from dual union all
select 7457,'1961-A249','1961-12-19 00:00:00','suborbital spaceplane','failure',1520,'3-A-1','','','','Test',379,'NB-52 003',14 from dual union all
select 7458,'1961-A25','1961-01-18 00:00:00','military missile','success',771,'GM-56-1943','','','','Test',222,'',12 from dual union all
select 7459,'1961-A250','1961-12-20 22:45:50','suborbital spaceplane','success',1520,'3-1-2','MH-96','','','Test',379,'NB-52 003',24 from dual union all
select 746,'1948-A24','1948-08-20 16:45:00','atmospheric rocket','success',879,'482-R','Nike 48-2 P dummy','','','Test',1863,'',11 from dual union all
select 7460,'1961-A251','1961-12-26 00:00:00','military missile','failure',979,'','8K65 UBP-16','','','Test',636,'',0 from dual union all
select 7461,'1961-A252','1961-06-01 00:00:00','suborbital rocket','failure',870,'Test Flight 1','Sphere','','','Test/Aeron',1821,'',10 from dual union all
select 7462,'1961-A253','1961-07-26 00:00:00','suborbital rocket','failure',870,'Test Flight 2','Sphere','','','Test/Aeron',1821,'',10 from dual union all
select 7463,'1961-A254','1961-10-26 02:43:00','suborbital rocket','success',642,'FN-150','Four Nozzle test','','','Test',7,'',5 from dual union all
select 7464,'1961-A255','1961-01-31 00:00:00','military missile','failure',872,'ZW-14/20016','','','','Test',1884,'',5 from dual union all
select 7465,'1961-A256','1961-04-26 00:00:00','military missile','failure',872,'ZW-15','Zeus A','','','Test',1884,'',5 from dual union all
select 7466,'1961-A257','1961-07-01 00:00:00','military missile','unknown',872,'ZW-18','','','','Test',1884,'',40 from dual union all
select 7467,'1961-A258','1961-07-01 00:00:00','military missile','unknown',872,'ZW-20','','','','Test',1884,'',40 from dual union all
select 7468,'1961-A259','1961-09-06 00:00:00','military missile','unknown',872,'ZW-24','','','','Test',1884,'',40 from dual union all
select 7469,'1961-A26','1961-01-18 02:31:00','suborbital rocket','success',716,'','','','','Aeron',276,'',43 from dual union all
select 747,'1948-A25','1948-09-01 00:00:00','atmospheric rocket','success',565,'','Rascal Model','','','Test',1821,'',5 from dual union all
select 7470,'1961-A260','1961-09-21 00:00:00','military missile','unknown',872,'ZW-26 20026','','','','Test',1884,'',40 from dual union all
select 7471,'1961-A261','1961-11-07 00:00:00','military missile','unknown',872,'ZW-27','','','','Test',1884,'',40 from dual union all
select 7472,'1961-A262','1961-12-04 00:00:00','military missile','unknown',872,'ZW-31','','','','Test',1884,'',40 from dual union all
select 7473,'1961-A263','1961-12-08 00:00:00','military missile','unknown',872,'ZW-32','','','','Test',1884,'',40 from dual union all
select 7474,'1961-A264','1961-10-01 00:00:00','military missile','unknown',872,'ZW-33','','','','Test',1884,'',40 from dual union all
select 7475,'1961-A265','1961-11-15 00:00:00','military missile','success',272,'HIGHBALL','HIGHBALL II (Zeus)','','','Target',1884,'',20 from dual union all
select 7476,'1961-A266','1961-10-30 00:00:00','atmospheric rocket','success',607,'3-Fin Test','Dummy Iris','','','Test',394,'',1 from dual union all
select 7477,'1961-A267','1961-11-02 00:00:00','atmospheric rocket','success',1154,'SEABEE','Hydra','','','Test',1057,'',1 from dual union all
select 7478,'1961-A268','1961-04-14 00:00:00','military missile','success',1321,'','9K71 first launch','','','Test',636,'',10 from dual union all
select 7479,'1961-A269','1961-04-01 00:00:00','military missile','success',1321,'','9K71 No. 2','','','Test',636,'',10 from dual union all
select 748,'1948-A26','1948-09-09 22:55:00','atmospheric rocket','success',878,'48-24','Nike 48-1 22R','','','Test',1863,'',10 from dual union all
select 7480,'1961-A27','1961-01-20 20:53:00','military missile','failure',1419,'AJ-10','Mk 4-1 RV','','','Test',237,'',100 from dual union all
select 7481,'1961-A270','1961-08-15 00:00:00','military missile','success',1321,'','9K71 No. 3','','','Test',636,'',10 from dual union all
select 7482,'1961-A271','1961-10-01 00:00:00','military missile','failure',1321,'','9K71 Phase 2 No. 1','','','Test',636,'',10 from dual union all
select 7483,'1961-A272','1961-12-20 06:05:00','suborbital rocket','success',642,'RT-150','Radar test','','','Test',7,'',5 from dual union all
select 7484,'1961-A28','1961-01-26 01:57:00','suborbital rocket','success',716,'','','','','Aeron',276,'',35 from dual union all
select 7485,'1961-A29','1961-01-26 23:20:00','suborbital rocket','success',716,'','','','','Aeron',1057,'',34 from dual union all
select 7486,'1961-A30','1961-02-01 00:00:00','suborbital rocket','failure',88,'','','','','Aeron',428,'',0 from dual union all
select 7487,'1961-A31','1961-02-01 00:00:00','suborbital rocket','failure',88,'','','','','Aeron',428,'',0 from dual union all
select 7488,'1961-A32','1961-02-01 00:00:00','suborbital rocket','failure',88,'','','','','Aeron',428,'',0 from dual union all
select 7489,'1961-A33','1961-02-01 00:00:00','suborbital rocket','failure',88,'','','','','Aeron',428,'',0 from dual union all
select 749,'1948-A27','1948-09-14 17:36:00','atmospheric rocket','success',878,'47-L','Nike 48-1 Q dummy','','','Test',1863,'',10 from dual union all
select 7490,'1961-A34','1961-02-01 00:00:00','suborbital rocket','failure',88,'','','','','Aeron',428,'',0 from dual union all
select 7491,'1961-A35','1961-02-01 00:00:00','suborbital rocket','failure',88,'','','','','Aeron',428,'',0 from dual union all
select 7492,'1961-A36','1961-02-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7493,'1961-A37','1961-02-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7494,'1961-A38','1961-02-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7495,'1961-A39','1961-02-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7496,'1961-A40','1961-02-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7497,'1961-A41','1961-02-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7498,'1961-A42','1961-02-01 18:47:32','suborbital spaceplane','success',1520,'1-20-35','Ball/side controller','','','Test',379,'NB-52 008',15 from dual union all
select 7499,'1961-A43','1961-02-02 19:00:00','military missile','failure',981,'L1-5T 3L','8K64 LKI-1','','','Test',565,'',1000 from dual union all
select 75,'1944-A39','1944-12-01 00:00:00','military missile','success',943,'6','','','','Test',764,'',5 from dual union all
select 750,'1948-A28','1948-09-15 00:00:00','atmospheric rocket','success',878,'48-26','Nike 48-1 23R','','','Test',1863,'',10 from dual union all
select 7500,'1961-A44','1961-02-06 21:40:00','military missile','failure',929,'A2X-6','','','','Test',244,'',0 from dual union all
select 7501,'1961-A45','1961-02-07 20:56:10','suborbital spaceplane','success',1520,'1-21-36','Stab./Control','','','Test',379,'NB-52 008',23 from dual union all
select 7502,'1961-A46','1961-02-08 00:00:00','military missile','success',771,'GM-56-1941','','','','Test',222,'',12 from dual union all
select 7503,'1961-A47','1961-02-13 00:00:00','suborbital rocket','failure',1490,'AGI28','','','','Aeronomy',668,'',0 from dual union all
select 7504,'1961-A48','1961-02-15 18:00:00','atmospheric rocket','success',835,'Smoke 007','Smoke','','','Aeronomy',1821,'',10 from dual union all
select 7505,'1961-A49','1961-02-18 00:00:00','military missile','success',1469,'','','','','ABM Test',182,'',25 from dual union all
select 7506,'1961-A50','1961-02-18 00:00:00','suborbital rocket','failure',1490,'AGI30','','','','Aeronomy',668,'',16 from dual union all
select 7507,'1961-A51','1961-02-21 00:00:00','suborbital spaceplane','failure',1520,'2- A-24','','','','Test',379,'NB-52',14 from dual union all
select 7508,'1961-A52','1961-02-22 00:00:00','military missile','success',1469,'','','','','ABM Test',182,'',25 from dual union all
select 7509,'1961-A53','1961-02-24 00:00:00','suborbital spaceplane','failure',1520,'2- A-25','','','','Test',379,'NB-52',14 from dual union all
select 751,'1948-A29','1948-09-17 16:32:00','atmospheric rocket','success',878,'48-25','Nike 48-1 24R','','','Test',1863,'',35 from dual union all
select 7510,'1961-A54','1961-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7511,'1961-A55','1961-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7512,'1961-A56','1961-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7513,'1961-A57','1961-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7514,'1961-A58','1961-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7515,'1961-A59','1961-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7516,'1961-A60','1961-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7517,'1961-A61','1961-03-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7518,'1961-A62','1961-03-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7519,'1961-A63','1961-03-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 752,'1948-A30','1948-09-17 00:00:00','military missile','failure',969,'I-4','','','','Test',636,'',1 from dual union all
select 7520,'1961-A64','1961-03-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7521,'1961-A65','1961-03-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7522,'1961-A66','1961-03-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7523,'1961-A67','1961-03-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7524,'1961-A68','1961-03-01 00:00:00','military missile','failure',976,'','8K63 UBP-119','','','Test',636,'',0 from dual union all
select 7525,'1961-A69','1961-03-02 00:00:00','military missile','success',1469,'','','','','ABM Test',182,'',25 from dual union all
select 7526,'1961-A70','1961-03-03 01:45:00','military missile','failure',1419,'AJ-12','Mk 4-1 RV','','','Test',239,'',200 from dual union all
select 7527,'1961-A71','1961-03-04 00:00:00','military missile','success',1469,'','R-12 Intercept','','','ABM Test',182,'',25 from dual union all
select 7528,'1961-A72','1961-03-07 00:00:00','military missile','success',764,'59-4979 (26)','','','','Test',240,'',12 from dual union all
select 7529,'1961-A73','1961-03-07 18:28:33','suborbital spaceplane','success',1520,'2-13-26','Mach 4 flight','','','Test',379,'NB-52 008',23 from dual union all
select 753,'1948-A31','1948-09-21 22:15:00','atmospheric rocket','success',878,'48-29','Nike 48-1 25R','','','Test',1863,'',10 from dual union all
select 7530,'1961-A74','1961-03-08 00:00:00','military missile','success',771,'GM-56-1945','','','','Test',222,'',12 from dual union all
select 7531,'1961-A75','1961-03-16 00:00:00','military missile','success',764,'59-4977 (24)','','','','Test',240,'',12 from dual union all
select 7532,'1961-A76','1961-03-16 00:00:00','atmospheric rocket','success',1473,'','','','','Test',664,'',20 from dual union all
select 7533,'1961-A77','1961-03-18 16:49:00','test rocket','success',709,'LJ-5A','LJ-VI','','','SC Test',1797,'',12 from dual union all
select 7534,'1961-A78','1961-03-21 00:00:00','atmospheric rocket','success',1473,'','','','','Test',664,'',20 from dual union all
select 7535,'1961-A79','1961-03-21 00:00:00','suborbital spaceplane','failure',1520,'2- A-27','','','','Test',379,'NB-52 008',14 from dual union all
select 7536,'1961-A80','1961-03-23 16:25:00','military missile','failure',928,'A1E-16','','','','OT',421,'SSBN 600',10 from dual;
insert into LAUNCH
select 7537,'1961-A81','1961-03-23 22:41:00','military missile','failure',928,'A1E-18 (A1P-39)','','','','OT',421,'SSBN 600',10 from dual union all
select 7538,'1961-A82','1961-03-26 00:00:00','military missile','success',1469,'','','','','ABM Test',182,'',25 from dual union all
select 7539,'1961-A83','1961-03-28 00:00:00','military missile','failure',764,'59-4978 (25)','','','','Test',240,'',0 from dual union all
select 754,'1948-A32','1948-09-24 00:00:00','atmospheric rocket','success',365,'','Convair XF-92 Model','','','Test',1821,'',10 from dual union all
select 7540,'1961-A84','1961-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7541,'1961-A85','1961-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7542,'1961-A86','1961-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7543,'1961-A87','1961-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7544,'1961-A88','1961-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7545,'1961-A89','1961-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 7546,'1961-A90','1961-04-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7547,'1961-A91','1961-04-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7548,'1961-A92','1961-04-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7549,'1961-A93','1961-04-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 755,'1948-A33','1948-09-27 00:00:00','military missile','failure',1045,'MX-774','','','','Test',1863,'',47 from dual union all
select 7550,'1961-A94','1961-04-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7551,'1961-A95','1961-04-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7552,'1961-A96','1961-04-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7553,'1961-A97','1961-04-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 7554,'1961-A98','1961-03-31 20:44:00','military missile','failure',1419,'AJ-15','Mk 3-2 RV','','','Test',239,'',0 from dual union all
select 7555,'1961-A99','1961-04-06 19:35:00','atmospheric rocket','success',835,'Smoke 009','Smoke','','','Aeronomy',1821,'',12 from dual union all
select 7556,'1961-F01','1961-02-25 00:13:16','orbital','failure',634,'AM-19F','NASA S-45','NASA S-45','','Sat',249,'',395 from dual union all
select 7557,'1961-F02','1961-03-30 20:34:43','orbital','failure',1361,'300/FTV 1105','Discoverer 22','CORONA C''-4','FEATHER CUT','Sat TAG22',1678,'',200 from dual union all
select 7558,'1961-F03','1961-04-25 16:15:00','orbital','failure',154,'100D','MA-3','Mercury SC8','','Sat',229,'',7 from dual union all
select 7559,'1961-F04','1961-05-24 19:48:05','orbital','failure',634,'AM-19G','NASA S-45A','NASA S-45A','','Sat',249,'',444 from dual union all
select 756,'1948-A34','1948-09-27 23:15:00','atmospheric rocket','success',879,'482-S','Nike 48-2 R dummy','','','Test',1863,'',11 from dual union all
select 7560,'1961-F05','1961-06-08 21:16:00','orbital','failure',1361,'302/FTV 1108','Discoverer 24','ARGON 3','ISLAND QUEEN','Sat TAG24',1678,'',200 from dual union all
select 7561,'1961-F06','1961-06-30 17:09:00','orbital','failure',1138,'ST-5','S-55','NASA S-55','','Sat',1812,'',172 from dual union all
select 7562,'1961-F07','1961-07-21 22:35:00','orbital','failure',1361,'322/FTV 1110','Discoverer 27','ARGON 4','STACKED DECK','Sat TAG27',1678,'',10 from dual union all
select 7563,'1961-F08','1961-08-04 00:01:00','orbital','failure',1361,'309/FTV 1111','Discoverer 28','CORONA C''-7','CRISP BACON','Sat TAG28',1673,'',200 from dual union all
select 7564,'1961-F09','1961-09-09 19:28:27','orbital','failure',145,'106D','Samos 3','SAMOS E-2','FIRST MOTION','Sat',1019,'',0 from dual union all
select 7565,'1961-F10','1961-10-23 19:23:00','orbital','failure',1361,'329/FTV 1116','Discoverer 33','CORONA C''-8','DEAD HEAT','Sat TAG33',1679,'',420 from dual union all
select 7566,'1961-F11','1961-10-27 16:30:00','orbital','failure',681,'1LK','','DS-1 No. 1','','Sat',630,'',50 from dual union all
select 7567,'1961-F12','1961-11-01 15:32:00','orbital','failure',246,'D-8','Mercury-Scout 1','Mercury MS-1','','Sat',236,'',5 from dual union all
select 7568,'1961-F13','1961-11-22 20:45:47','orbital','failure',145,'108D','Samos 4','Samos 4 (E-5)','ROUND TRIP','Sat',1019,'',200 from dual union all
select 7569,'1961-F14','1961-12-11 09:39:02','orbital','failure',1512,'E103-21','','Zenit-2 No. 1','','Sat',480,'',200 from dual union all
select 757,'1948-A35','1948-10-01 00:00:00','atmospheric rocket','success',564,'','E14 Model','','','Test',1821,'',5 from dual union all
select 7570,'1961-F15','1961-12-21 12:30:00','orbital','failure',681,'2LK','','DS-1 No. 2','','Sat',630,'',200 from dual union all
select 7571,'1961-M01','1961-01-10 00:00:00','military missile','success',1162,'43','','','','OT?',1895,'',50 from dual union all
select 7572,'1961-M02','1961-01-26 00:00:00','military missile','success',1162,'45','','','','OT?',1895,'',50 from dual union all
select 7573,'1961-M03','1961-03-10 00:00:00','military missile','success',1162,'49','','','','OT?',1895,'',50 from dual union all
select 7574,'1961-M04','1961-03-07 00:00:00','military missile','success',1080,'F91','','','','Test',671,'',65 from dual union all
select 7575,'1961-M05','1961-03-10 00:00:00','military missile','success',1080,'F92','','','','Test',671,'',67 from dual union all
select 7576,'1961-M06','1961-03-27 00:00:00','suborbital rocket','unknown',366,'','','','','Aeron?',1016,'',80 from dual union all
select 7577,'1961-M07','1961-03-30 18:05:00','suborbital spaceplane','success',1520,'2-14-28','Alt., BCS, Aero','','','Test',379,'NB-52 008',51 from dual union all
select 7578,'1961-M08','1961-03-31 02:50:00','suborbital rocket','success',35,'NASA 04.34GG','UV star photometer','','','UV Astron',1821,'',72 from dual union all
select 7579,'1961-M09','1961-04-11 15:40:00','military missile','failure',929,'A2X-12','','','','Test',251,'',50 from dual union all
select 758,'1948-A36','1948-09-30 22:15:00','atmospheric rocket','success',878,'48-28','Nike 48-1 26R','','','Test',1863,'',15 from dual union all
select 7580,'1961-M10','1961-04-24 02:53:00','suborbital rocket','success',535,'HAD 101 (HAT 201)','','','','Aeron',1835,'',58 from dual union all
select 7581,'1961-M11','1961-05-04 00:00:00','military missile','success',1162,'50','','','','OT?',1895,'',50 from dual union all
select 7582,'1961-M12','1961-05-15 00:00:00','military missile','success',1162,'54','','','','OT?',1895,'',50 from dual union all
select 7583,'1961-M13','1961-05-16 00:00:00','military missile','success',1162,'53','','','','OT?',1895,'',50 from dual union all
select 7584,'1961-M14','1961-05-17 00:00:00','military missile','success',1162,'51','','','','OT?',1895,'',50 from dual union all
select 7585,'1961-M15','1961-05-17 19:22:00','suborbital rocket','success',29,'NASA 01.04GP','AMPP FU6','','','Remote sensing',428,'',76 from dual union all
select 7586,'1961-M16','1961-05-24 00:00:00','suborbital rocket','success',203,'B05','','','','Test',664,'',60 from dual union all
select 7587,'1961-M17','1961-05-25 00:00:00','suborbital rocket','success',203,'B06','','','','Test',664,'',60 from dual union all
select 7588,'1961-M18','1961-07-04 10:23:00','suborbital rocket','success',1194,'SL65','UCL GC-15','','','Aeronomy',1839,'',70 from dual union all
select 7589,'1961-M19','1961-07-21 03:50:00','suborbital rocket','success',839,'NASA 10.35GA','GSFC Grenade 16','','','Aeronomy',1821,'',68 from dual union all
select 759,'1948-A37','1948-10-15 18:21:00','atmospheric rocket','success',878,'48-30','Nike 48-1 27R','','','Test',1863,'',13 from dual union all
select 7590,'1961-M20','1961-07-27 11:45:00','sounding rocket','failure',1266,'CRL AA11.501','CWAS','','','Aeron',1895,'',75 from dual union all
select 7591,'1961-M21','1961-09-20 00:00:00','military missile','success',1162,'55','','','','OT?',1895,'',50 from dual union all
select 7592,'1961-M22','1961-09-26 00:00:00','military missile','success',1162,'56','','','','OT?',1895,'',50 from dual union all
select 7593,'1961-M23','1961-09-28 00:00:00','military missile','success',1162,'58','','','','OT?',1895,'',50 from dual union all
select 7594,'1961-M24','1961-09-28 00:00:00','military missile','success',1162,'59','','','','OT?',1895,'',50 from dual union all
select 7595,'1961-M25','1961-09-16 00:00:00','suborbital rocket','success',328,'','','','','Test',353,'',50 from dual union all
select 7596,'1961-M26','1961-09-23 18:27:00','suborbital rocket','success',29,'NASA 01.08GA','Air sampler','','','Aeronomy',428,'',78 from dual union all
select 7597,'1961-M27','1961-09-27 12:30:00','suborbital rocket','success',30,'AA2.502','CWAS','','','Aeron',1869,'',59 from dual union all
select 7598,'1961-M28','1961-09-30 20:42:00','suborbital rocket','success',29,'NASA 01.09GA','Air sampler','','','Aeronomy',428,'',73 from dual union all
select 7599,'1961-M29','1961-10-11 20:20:00','suborbital spaceplane','success',1520,'2-20-36','Alt, Aero, Low q','','','Test',379,'NB-52 003',66 from dual union all
select 76,'1944-A40','1944-12-01 00:00:00','military missile','success',943,'7','','','','Test',764,'',5 from dual union all
select 760,'1948-A38','1948-10-15 00:00:00','military missile','success',830,'','','','','Test',643,'',15 from dual union all
select 7600,'1961-M30','1961-10-15 14:33:00','suborbital rocket','success',29,'NASA 01.10GA','Air sampler','','','Aeronomy',428,'',76 from dual union all
select 7601,'1961-M31','1961-10-17 20:11:00','suborbital rocket','success',29,'NASA 01.07GA','Air sampler','','','Aeronomy',428,'',65 from dual union all
select 7602,'1961-M32','1961-11-02 17:24:00','suborbital rocket','success',29,'NASA 01.11GA','Air sampler','','','Aeronomy',428,'',68 from dual union all
select 7603,'1961-M33','1961-11-05 20:11:00','suborbital rocket','success',29,'NASA 01.12GA','Air sampler','','','Aeronomy',428,'',71 from dual union all
select 7604,'1961-M36','1961-02-01 00:00:00','suborbital rocket','success',86,'','HIGHBALL III test','','','Test',1895,'',60 from dual union all
select 7605,'1961-M37','1961-05-01 00:00:00','suborbital rocket','success',272,'','HIGHBALL II test','','','Test',1895,'',60 from dual union all
select 7606,'1961-M38','1961-12-27 00:00:00','military missile','success',723,'','First launch','','','Test',636,'',70 from dual union all
select 7607,'1961-M39','1961-06-15 00:00:00','suborbital rocket','success',88,'','','','','Aeron',394,'',60 from dual union all
select 7608,'1961-M84','1961-01-19 00:00:00','suborbital rocket','success',88,'666 A361.4AA','','','','Test',1895,'',75 from dual union all
select 7609,'1961-M85','1961-02-10 00:00:00','suborbital rocket','success',88,'689 A374.4AA','','','','Test',1895,'',68 from dual union all
select 761,'1948-A39','1948-10-22 16:44:00','atmospheric rocket','success',878,'48-33','Nike 48-1 28R','','','Test',1863,'',22 from dual union all
select 7610,'1961-M86','1961-01-04 00:00:00','suborbital rocket','success',852,'','Cree','','','Parachute test',385,'',51 from dual union all
select 7611,'1961-S01','1961-01-01 00:00:00','suborbital rocket','success',672,'','','','','Test',1490,'',100 from dual union all
select 7612,'1961-S02','1961-01-01 00:00:00','suborbital rocket','success',26,'','','','','Aeron',1837,'',81 from dual union all
select 7613,'1961-S03','1961-01-01 00:00:00','suborbital rocket','success',26,'','','','','Aeron',1837,'',81 from dual union all
select 7614,'1961-S04','1961-01-01 00:00:00','military missile','success',979,'','8K65 LKI-14','','','Test',636,'',675 from dual union all
select 7615,'1961-S05','1961-01-01 00:00:00','military missile','success',979,'','8K65 LKI-15','','','Test',636,'',675 from dual union all
select 7616,'1961-S06','1961-01-01 00:00:00','military missile','success',979,'','8K65 LKI-16','','','Test',636,'',675 from dual union all
select 7617,'1961-S07','1961-01-01 00:00:00','military missile','success',979,'','8K65 LKI-17','','','Test',636,'',675 from dual union all
select 7618,'1961-S08','1961-01-01 00:00:00','military missile','success',979,'','8K65 LKI-18','','','Test',636,'',675 from dual union all
select 7619,'1961-S09','1961-01-01 00:00:00','suborbital rocket','success',619,'','','','','Test/Aeron',1895,'B-57A',800 from dual union all
select 762,'1948-A40','1948-11-05 00:00:00','military missile','failure',830,'','','','','Test',643,'',0 from dual union all
select 7620,'1961-S10','1961-01-01 00:00:00','suborbital rocket','unknown',1248,'','','','','Test',1057,'F-3B',100 from dual union all
select 7621,'1961-S100','1961-04-02 14:06:00','military missile','success',981,'5L-7T','8K64 LKI-3','','','Test',563,'',1210 from dual union all
select 7622,'1961-S101','1961-04-03 00:00:00','military missile','success',976,'','8K63 UBP-126','','','Test',636,'',402 from dual union all
select 7623,'1961-S102','1961-04-04 00:00:00','military missile','success',976,'','8K63 UBP-127','','','Test',636,'',402 from dual union all
select 7624,'1961-S103','1961-04-05 10:56:00','suborbital rocket','success',1194,'SL64','UCL GC-14','','','Aeronomy',1839,'',158 from dual union all
select 7625,'1961-S104','1961-04-05 12:57:00','suborbital rocket','success',839,'NASA 10.33GA','GSFC Grenade 8','','','Aeronomy',1821,'',116 from dual union all
select 7626,'1961-S105','1961-04-06 00:00:00','military missile','success',928,'A1E-19 (A1P-65)','','','','OT',421,'SSBN 600',500 from dual;
insert into LAUNCH
select 7627,'1961-S106','1961-04-07 00:00:00','military missile','success',976,'','8K63 UBP-128 40 RD','','','Test',636,'',402 from dual union all
select 7628,'1961-S107','1961-04-09 09:16:00','military missile','success',1018,'E 103   08','8K75 LKI-1','','','Test',569,'',800 from dual union all
select 7629,'1961-S108','1961-04-10 00:00:00','military missile','success',976,'','8K63 UBP-129 29 RD','','','Test',636,'',402 from dual union all
select 763,'1948-A41','1948-11-01 14:24:38','suborbital rocket','failure',265,'Bumper 4','','','','Test/Solar SX/Bio',1863,'',8 from dual union all
select 7630,'1961-S109','1961-04-12 06:07:00','ballistic missile test','success',246,'D-5','HETS A2-2','','','Plasma',236,'',1931 from dual union all
select 7631,'1961-S11','1961-01-16 00:00:00','military missile','failure',872,'ZW-13','','','','Test',1884,'',5 from dual union all
select 7632,'1961-S110','1961-04-14 17:15:00','suborbital rocket','success',35,'NASA 04.19GT','Attitude control','','','Test',1821,'',206 from dual union all
select 7633,'1961-S111','1961-04-18 04:40:00','suborbital rocket','success',30,'NB3.104','NRL NB3.104','','','UV Astron',1869,'',242 from dual union all
select 7634,'1961-S112','1961-04-18 12:27:00','suborbital rocket','success',665,'K-8-6','','','','Ionos',7,'',150 from dual union all
select 7635,'1961-S113','1961-04-19 22:04:00','military missile','success',928,'A1E-20','','','','OT',421,'SSBN 601',500 from dual union all
select 7636,'1961-S114','1961-04-19 00:00:00','military missile','success',976,'','8K63 UBP-131','','','Test',636,'',402 from dual union all
select 7637,'1961-S115','1961-04-19 09:36:00','suborbital rocket','success',838,'NASA 03.05CA','GCA Release 10 Na','','','Aeronomy',1821,'',164 from dual union all
select 7638,'1961-S116','1961-04-19 18:37:00','suborbital rocket','success',838,'ISRC-RS-02','','','','Aeron',1343,'',200 from dual union all
select 7639,'1961-S117','1961-04-20 03:46:00','suborbital rocket','success',838,'ISRC-RS-03','','','','Aeron',1343,'',190 from dual union all
select 764,'1948-A42','1948-11-09 17:03:00','atmospheric rocket','success',878,'48-27','Nike 48-1 29R','','','Test',1863,'',18 from dual union all
select 7640,'1961-S118','1961-04-20 18:39:00','suborbital rocket','success',838,'ISRC-RS-04','','','','Aeron',1343,'',188 from dual union all
select 7641,'1961-S119','1961-04-20 18:43:00','suborbital rocket','success',838,'ISRC-RS-05','','','','Aeron',1343,'',190 from dual union all
select 7642,'1961-S12','1961-01-07 17:33:42','ballistic missile test','success',245,'D-3','HETS A1-1','','','Plasma/RF Astron',236,'',1600 from dual union all
select 7643,'1961-S120','1961-04-21 00:12:00','suborbital rocket','failure',838,'NASA 03.06CA','GCA Release 11 Na','','','Aeronomy',1821,'',180 from dual union all
select 7644,'1961-S121','1961-04-21 02:26:14','military missile','success',1018,'E 103   09','8K75 LKI-2','','','Test',569,'',1160 from dual union all
select 7645,'1961-S122','1961-04-21 09:34:00','suborbital rocket','failure',838,'NASA 03.07CA','GCA Release 12 Na','','','Aeronomy',1821,'',200 from dual union all
select 7646,'1961-S123','1961-04-21 09:39:00','suborbital rocket','success',838,'NASA 03.08CA','GCA Release 13 Na','','','Aeronomy',1821,'',165 from dual union all
select 7647,'1961-S124','1961-04-21 12:00:00','suborbital rocket','success',35,'NASA 04.39NP','LeRC LH2 test','','','Tech',1821,'',154 from dual union all
select 7648,'1961-S125','1961-04-21 15:53:00','military missile','success',916,'308','P-15','','','Test',256,'',180 from dual union all
select 7649,'1961-S126','1961-04-22 14:07:00','military missile','success',638,'CM-209','CTL','','','OT',248,'',500 from dual union all
select 765,'1948-A43','1948-11-12 17:00:00','atmospheric rocket','success',878,'48-31','Nike 48-1 30R','','','Test',1863,'',20 from dual union all
select 7650,'1961-S127','1961-04-27 00:00:00','military missile','success',976,'','8K63 UBP-132','','','Test',636,'',402 from dual union all
select 7651,'1961-S128','1961-04-27 20:02:00','suborbital rocket','success',624,'NASA 8.10GI','P-21 program','','','Ionos',1821,'',724 from dual union all
select 7652,'1961-S129','1961-04-28 00:00:00','military missile','success',976,'','8K63 UBP-133 35 RD','','','Test',636,'',402 from dual union all
select 7653,'1961-S13','1961-01-10 17:15:00','military missile','success',929,'A2X-3','','','','Test',251,'',1000 from dual union all
select 7654,'1961-S130','1961-05-01 00:00:00','suborbital rocket','success',673,'','PHOENIX Chaff','','','Aeron',1821,'',273 from dual union all
select 7655,'1961-S131','1961-05-01 17:00:00','military missile','success',929,'A2X-16','','','','Test',251,'',1000 from dual union all
select 7656,'1961-S132','1961-05-01 12:10:00','suborbital rocket','success',1194,'SL43','UCL UV-1','','','UV Astron',1839,'',155 from dual union all
select 7657,'1961-S133','1961-05-03 17:00:00','military missile','success',928,'A1E-22','','','','OT',421,'SSBN 602',500 from dual union all
select 7658,'1961-S134','1961-05-03 20:10:44','military missile','success',1419,'VS-1','Mk 4-6B RV No. 001','Dummy St2','SILVER SADDLE','Test',1725,'',116 from dual union all
select 7659,'1961-S135','1961-05-05 14:34:00','test rocket','success',1054,'MRLV-7','MR-3','','','Test/Bio',279,'',187 from dual union all
select 766,'1948-A44','1948-12-01 00:00:00','military missile','success',771,'','','','','Test',645,'',12 from dual union all
select 7660,'1961-S136','1961-05-05 23:00:00','suborbital rocket','success',839,'NASA 10.02GA','GSFC Grenade 10','','','Aeronomy',1821,'',117 from dual union all
select 7661,'1961-S137','1961-05-06 04:54:00','suborbital rocket','success',839,'NASA 10.28GA','GSFC Grenade 11','','','Aeronomy',1821,'',117 from dual union all
select 7662,'1961-S138','1961-05-08 17:10:00','military missile','success',929,'A2X-19','','','','Test',251,'',1000 from dual union all
select 7663,'1961-S139','1961-05-09 11:53:00','suborbital rocket','success',839,'NASA 10.29GA','GSFC Grenade 12','','','Aeronomy',1821,'',117 from dual union all
select 7664,'1961-S14','1961-01-10 00:00:00','military missile','success',976,'','8K63 UBP-112','','','Test',636,'',402 from dual union all
select 7665,'1961-S140','1961-05-09 12:07:00','military missile','failure',240,'BK.14','H-204','','','Reentry/CR/Ionos',1843,'',415 from dual union all
select 7666,'1961-S141','1961-05-10 00:00:00','military missile','success',976,'','8K63 UBP-136','','','Test',636,'',402 from dual union all
select 7667,'1961-S142','1961-05-13 00:00:00','atmospheric rocket','success',81,'013','','','','Ionospheric?',671,'',280 from dual union all
select 7668,'1961-S143','1961-05-13 02:00:00','military missile','success',155,'12E','Mk 5-1','R&D','','Test',226,'',1500 from dual union all
select 7669,'1961-S144','1961-05-16 15:45:00','suborbital rocket','success',30,'AA3.145','','','','Solar UV',1869,'',294 from dual union all
select 767,'1948-A45','1948-12-23 00:00:00','atmospheric rocket','failure',365,'','D3 Model','','','Test',1821,'',0 from dual union all
select 7670,'1961-S145','1961-05-16 17:00:00','suborbital rocket','success',30,'AA3.127','','','','Solar EUV',1869,'',312 from dual union all
select 7671,'1961-S146','1961-05-16 19:23:27','military missile','success',981,'8L-8T','8K64 LKI-6','','','Test',563,'',1210 from dual union all
select 7672,'1961-S147','1961-05-17 00:00:00','military missile','failure',928,'A1E-26 (A1P-70)','','','','OT',421,'SSBN 602',500 from dual union all
select 7673,'1961-S148','1961-05-17 00:00:00','military missile','failure',928,'A1E-28 (A1P-56)','','','','OT',421,'SSBN 602',500 from dual union all
select 7674,'1961-S149','1961-05-17 00:00:00','military missile','success',928,'A1E-33 (A1P-66)','','','','OT',421,'SSBN 602',500 from dual union all
select 7675,'1961-S15','1961-01-12 16:54:00','suborbital rocket','success',839,'ISRC-RS-01','','','','Aeronomy Na',1343,'',160 from dual union all
select 7676,'1961-S150','1961-05-17 00:00:00','military missile','success',928,'A1E-27 (A1P-55)','','','','OT',421,'SSBN 602',500 from dual union all
select 7677,'1961-S151','1961-05-17 00:00:00','military missile','success',928,'A1E-29','','','','OT',421,'SSBN 602',500 from dual union all
select 7678,'1961-S152','1961-05-18 02:00:00','military missile','success',1053,'CC-2042','','','','Test',280,'',90 from dual union all
select 7679,'1961-S153','1961-05-18 06:10:00','suborbital rocket','success',1438,'TB Ih','','','','Reentry Test',1821,'',260 from dual union all
select 768,'1948-M01','1948-04-19 19:54:00','suborbital rocket','failure',1470,'38','','','','Solar UV/Ionos',1863,'',56 from dual union all
select 7680,'1961-S154','1961-05-19 14:18:00','military missile','failure',794,'402','Mk 5 RV','R&D','','Test',261,'',100 from dual union all
select 7681,'1961-S155','1961-05-19 02:00:00','military missile','success',916,'310','P-16','','','Test',255,'',180 from dual union all
select 7682,'1961-S156','1961-05-24 03:27:00','military missile','success',1419,'AJ-16','Mk 4-2b RV','','','Test',239,'',1000 from dual union all
select 7683,'1961-S157','1961-05-24 21:50:38','military missile','success',154,'95D','Mk 3','R&D','LITTLE SATIN','Test',1666,'',1800 from dual union all
select 7684,'1961-S158','1961-05-25 00:00:00','suborbital rocket','success',30,'','','','','Solar EUV',1869,'',200 from dual union all
select 7685,'1961-S159','1961-05-25 17:03:00','suborbital rocket','success',837,'NASA 14.01GT','','','','Test',1821,'',116 from dual union all
select 7686,'1961-S16','1961-01-13 00:00:00','military missile','success',976,'','8K63 UBP-113','','','Test',636,'',402 from dual union all
select 7687,'1961-S160','1961-05-26 02:26:00','military missile','success',155,'18E','Mk 4-4','R&D','','Test',228,'',1500 from dual union all
select 7688,'1961-S161','1961-05-27 00:00:00','suborbital rocket','success',287,'C03','','','','Test',664,'',130 from dual union all
select 7689,'1961-S162','1961-05-29 00:00:00','military missile','success',976,'','8K63 UBP-137','','','Test',636,'',402 from dual union all
select 769,'1948-M02','1948-06-11 10:22:00','suborbital rocket','success',1470,'37','Blossom III (Albert)','','','Solar X/Ionos/Bio',1863,'',63 from dual union all
select 7690,'1961-S163','1961-05-29 00:00:00','suborbital rocket','success',287,'C04','','','','Test',664,'',130 from dual union all
select 7691,'1961-S164','1961-05-30 05:15:00','suborbital rocket','success',1194,'SL25','RAE Test 11','','','Test',1839,'',149 from dual union all
select 7692,'1961-S165','1961-06-02 17:24:30','military missile','success',981,'10L-10T','8K64 LKI-8','','','Test',566,'',1210 from dual union all
select 7693,'1961-S166','1961-06-04 00:00:00','military missile','success',979,'','8K65 UBP-4 43 RD','','','Test',636,'',675 from dual union all
select 7694,'1961-S167','1961-06-06 12:31:00','suborbital rocket','success',30,'AA3.108','Venus Flytrap','','','Meteorites',1869,'',165 from dual union all
select 7695,'1961-S168','1961-06-06 21:48:00','suborbital rocket','success',839,'NASA 10.50UA','Sphere/Inf Sphere 1','','','Aeronomy',1821,'',150 from dual union all
select 7696,'1961-S169','1961-06-07 00:40:00','suborbital rocket','success',1194,'SL26','RAE Test 12','','','Test',1839,'',184 from dual union all
select 7697,'1961-S17','1961-01-13 00:00:00','military missile','success',1009,'V-1000 target','8A62 PRO Target C-8','V-1000 Target','','Target',321,'',300 from dual union all
select 7698,'1961-S170','1961-06-07 11:20:00','suborbital rocket','failure',240,'BK.17  UK-1 probe','H-203','','','Reentry/RF Astron/Ionos',1844,'',583 from dual union all
select 7699,'1961-S171','1961-06-09 00:00:00','military missile','success',976,'','8K63 UBP-138','','','Test',636,'',402 from dual union all
select 77,'1944-A41','1944-12-01 00:00:00','military missile','success',943,'8','','','','Test',764,'',5 from dual union all
select 770,'1948-M03','1948-12-02 22:01:00','military missile','success',1045,'MX-774','','','','Test',1863,'',50 from dual union all
select 7700,'1961-S172','1961-06-09 00:00:00','military missile','success',976,'','8K63 PRO-3','V-1000 target','','Target',942,'',402 from dual union all
select 7701,'1961-S173','1961-06-09 05:11:00','military missile','success',916,'311','P-17','','','Test',255,'',180 from dual union all
select 7702,'1961-S174','1961-06-09 18:02:00','suborbital rocket','success',839,'NASA 10.56UA','Composition','','','Aeronomy',1821,'',142 from dual union all
select 7703,'1961-S175','1961-06-10 00:00:00','suborbital rocket','success',1490,'AGI31','','','','Aeronomy',668,'',100 from dual union all
select 7704,'1961-S176','1961-06-12 15:53:00','military missile','success',929,'A2X-5','','','','Test',244,'',1000 from dual union all
select 7705,'1961-S177','1961-06-13 05:12:00','suborbital rocket','success',624,'NASA 8.09GI','P-21 program','','','Ionos',1821,'',867 from dual union all
select 7706,'1961-S178','1961-06-15 00:00:00','military missile','success',976,'','8K63 UBP-139','','','Test',636,'',402 from dual union all
select 7707,'1961-S179','1961-06-15 00:02:59','suborbital rocket','success',624,'NASA 8.13II','Antenna deploy test','','','Ionos',1821,'',901 from dual union all
select 7708,'1961-S18','1961-01-14 00:00:00','military missile','success',976,'','8K63 UBP-114','','','Test',636,'',402 from dual union all
select 7709,'1961-S180','1961-06-15 05:45:00','military missile','success',1017,'E 15001- 06','8K74 UBP-6','','','Test',562,'',1350 from dual union all
select 771,'1948-S01','1948-01-22 20:12:00','suborbital rocket','success',1470,'34','','','','CR/Aeron',1863,'',159 from dual union all
select 7710,'1961-S181','1961-06-17 07:29:00','suborbital rocket','success',30,'ND3.112','NRL ND3.112','','','Aeron',1869,'',197 from dual union all
select 7711,'1961-S182','1961-06-18 04:35:00','suborbital rocket','success',660,'K-4-2','Sigma 4 Rockoon','','','CR',1007,'',105 from dual union all
select 7712,'1961-S183','1961-06-20 23:54:00','military missile','success',1367,'276','CTL','','WHITE BISHOP','OT',1676,'',520 from dual union all
select 7713,'1961-S184','1961-06-21 14:45:00','suborbital rocket','success',30,'NE3.102','NRL NE3.102','','','Solar EUV',1869,'',271 from dual union all
select 7714,'1961-S185','1961-06-23 15:00:00','suborbital rocket','success',839,'AA6.104','Micrometeors','','','Meteorites',394,'',157 from dual union all
select 7715,'1961-S186','1961-06-24 00:00:00','military missile','success',976,'','8K63 PRO-4','V-1000 target','','Target',942,'',402 from dual union all
select 7716,'1961-S187','1961-06-24 23:17:00','suborbital rocket','success',624,'NASA 8.15AI','S-48 Topside Sounder','','','Ionos',1821,'',1018 from dual;
insert into LAUNCH
select 7717,'1961-S188','1961-06-26 00:00:00','military missile','failure',929,'A2X-11','','','','Test',421,'EAG-154',500 from dual union all
select 7718,'1961-S189','1961-06-26 15:15:00','suborbital rocket','success',35,'NASA 04.20GT','Attitude control','','','Test',1821,'',187 from dual union all
select 7719,'1961-S19','1961-01-14 00:00:00','military missile','success',1009,'V-1000 target','8A62 PRO Target C-9','V-1000 Target','','Target',321,'',300 from dual union all
select 772,'1948-S02','1948-02-06 17:17:00','suborbital rocket','success',1470,'36','','','','Test',1863,'',113 from dual union all
select 7720,'1961-S190','1961-06-27 02:20:00','military missile','success',1053,'CC-2043','','','','Test',280,'',90 from dual union all
select 7721,'1961-S191','1961-06-29 00:00:00','military missile','success',976,'','8K63 UBP-140','','','Test',636,'',402 from dual union all
select 7722,'1961-S193','1961-06-30 00:00:00','military missile','success',976,'','8K63 PRO-5','V-1000 target','','Target',942,'',402 from dual union all
select 7723,'1961-S194','1961-05-26 00:30:00','military missile','success',872,'ZW-16/20021','','','','Test',1884,'',40 from dual union all
select 7724,'1961-S195','1961-07-01 00:00:00','military missile','success',872,'ZW-19','','','','Test',1884,'',40 from dual union all
select 7725,'1961-S196','1961-08-25 20:34:00','military missile','success',872,'ZW-22 20019','ZW-22/DM-15B','','','Test',1884,'',10 from dual union all
select 7726,'1961-S197','1961-07-26 00:00:00','military missile','success',872,'ZW-21','','','','Test',1884,'',40 from dual union all
select 7727,'1961-S198','1961-08-31 00:00:00','military missile','success',872,'ZW-23','New mainstage test','','','Test',1884,'',40 from dual union all
select 7728,'1961-S199','1961-07-01 02:00:00','military missile','success',916,'312','P-18','','','Test',255,'',180 from dual union all
select 7729,'1961-S20','1961-01-14 01:19:04','military missile','success',1017,'L1-1T','8K74 UBP-2','','','OT',562,'',1350 from dual union all
select 773,'1948-S03','1948-03-05 22:51:00','suborbital rocket','success',58,'A5','Particles','','','CR',1869,'',118 from dual union all
select 7730,'1961-S200','1961-07-04 04:00:00','military missile','success',1017,'L2-4','8K74 UBP-7','Airburst','','Test',562,'',1350 from dual union all
select 7731,'1961-S201','1961-07-04 20:20:00','military missile','success',1017,'L2-2','8K74 UBP-8','Airburst','','Test',562,'',1350 from dual union all
select 7732,'1961-S202','1961-07-06 00:00:00','military missile','success',1053,'CC-1005','','','','OT',437,'',90 from dual union all
select 7733,'1961-S203','1961-07-07 04:51:00','military missile','success',155,'22E','Mk 3-2B','R&D','','Test',228,'',1500 from dual union all
select 7734,'1961-S204','1961-07-13 23:30:00','military missile','success',929,'A2X-20','Grand Turk','','','Test',251,'',1000 from dual union all
select 7735,'1961-S205','1961-07-13 22:07:00','suborbital rocket','success',839,'NASA 10.30GA','GSFC Grenade 13','','','Aeronomy',1821,'',119 from dual union all
select 7736,'1961-S206','1961-07-14 02:57:00','suborbital rocket','success',837,'NASA 14.03UE','Magnetic Fields','','','Magnetospheric',1821,'',196 from dual union all
select 7737,'1961-S207','1961-07-14 15:00:00','suborbital rocket','success',837,'NASA 14.04UE','Magnetic Fields','','','Magnetospheric',1821,'',200 from dual union all
select 7738,'1961-S208','1961-07-14 16:02:00','suborbital rocket','success',839,'NASA 10.31GA','GSFC Grenade 14','','','Aeronomy',1821,'',101 from dual union all
select 7739,'1961-S209','1961-07-19 00:00:00','military missile','success',1009,'UNK','8A62 D04B','','','Target',636,'',500 from dual union all
select 774,'1948-S04','1948-04-02 13:47:00','suborbital rocket','success',1470,'25','UM-1','','','Aeron/Ionos/Solar UV',1863,'',144 from dual union all
select 7740,'1961-S21','1961-01-17 21:25:00','suborbital rocket','success',838,'NASA 03.36GT','','','','Test',1821,'',209 from dual union all
select 7741,'1961-S210','1961-07-19 00:00:00','military missile','success',1009,'UNK','8A62 D04B','','','Target',636,'',500 from dual union all
select 7742,'1961-S211','1961-07-21 02:00:00','military missile','success',1419,'AJ-18','Mk 4-2a RV/Mk I Decoy','','','Test',239,'',1000 from dual union all
select 7743,'1961-S212','1961-07-20 02:00:00','military missile','success',916,'313','P-19','','','Test',256,'',180 from dual union all
select 7744,'1961-S213','1961-07-20 10:30:00','suborbital rocket','success',839,'NASA 10.32GA','GSFC Grenade 15','','','Aeronomy',1821,'',124 from dual union all
select 7745,'1961-S214','1961-07-20 13:12:00','suborbital rocket','success',837,'NASA 14.05UE','Magnetic Fields','','','Magnetospheric',1821,'',193 from dual union all
select 7746,'1961-S215','1961-07-21 02:42:00','suborbital rocket','success',665,'K-8-7','TW-10/ID-6','','','Aeron/Ionos',7,'',158 from dual union all
select 7747,'1961-S216','1961-07-21 12:20:00','test rocket','success',1054,'MRLV-8','MR-4','','','Test',279,'',189 from dual union all
select 7748,'1961-S217','1961-07-22 00:00:00','military missile','success',976,'','8K63 UBP-141','','','Test',636,'',402 from dual union all
select 7749,'1961-S218','1961-07-25 00:00:00','military missile','success',976,'','8K63 UBP-142','','','Test',636,'',402 from dual union all
select 775,'1948-S05','1948-04-13 21:41:00','suborbital rocket','success',58,'A6','B field','','','Magnetospheric',1869,'',114 from dual union all
select 7750,'1961-S219','1961-07-25 15:57:00','military missile','success',1018,'E 103   12','8K75 LKI-6','','','Test',569,'',1160 from dual union all
select 7751,'1961-S22','1961-01-18 04:00:00','suborbital rocket','success',1438,'TB If','D58 Trailblazer','','','Reentry Test',1821,'',260 from dual union all
select 7752,'1961-S220','1961-07-25 19:05:00','military missile','success',1419,'M-2','Mk 4-4 RV','','','Test',237,'',1000 from dual union all
select 7753,'1961-S221','1961-07-26 19:22:00','suborbital rocket','success',839,'NASA 10.57UA','Composition','','','Aeronomy',1821,'',143 from dual union all
select 7754,'1961-S222','1961-07-27 15:09:00','military missile','success',794,'403','Mk 5 RV','R&D','','Test',261,'',1300 from dual union all
select 7755,'1961-S223','1961-07-27 00:00:00','military missile','success',976,'','8K63 UBP-143','','','Test',636,'',402 from dual union all
select 7756,'1961-S224','1961-07-30 16:00:09','military missile','success',1018,'E 15001-25','8K75 LKI-7','','','Test',569,'',1160 from dual union all
select 7757,'1961-S225','1961-08-15 00:00:00','military missile','success',1053,'CC-1009','','','','OT 46AG',437,'',90 from dual union all
select 7758,'1961-S226','1961-07-31 21:32:00','military missile','success',155,'21E','Mk 5-1','R&D','','Test',226,'',1500 from dual union all
select 7759,'1961-S227','1961-08-01 11:02:00','suborbital rocket','success',1194,'SL34','UCL UV-2','','','UV Astron',1839,'',124 from dual union all
select 776,'1948-S06','1948-05-13 13:43:00','suborbital rocket','success',265,'Bumper 1','','','','Test',1863,'',127 from dual union all
select 7760,'1961-S228','1961-08-02 00:00:00','military missile','success',929,'A2X-21','Antigua','','','Test',251,'',1000 from dual union all
select 7761,'1961-S229','1961-08-04 01:40:00','military missile','success',1419,'AJ-19','Mk 4-4 RV/Mk II Decoy','','','Test',239,'',1000 from dual union all
select 7762,'1961-S23','1961-01-19 12:42:00','suborbital rocket','failure',607,'NASA 5.03GT','','','','Aeronomy/Test',1797,'',138 from dual union all
select 7763,'1961-S230','1961-08-05 00:19:00','military missile','success',638,'CM-218','CTL','','','OT',248,'',500 from dual union all
select 7764,'1961-S231','1961-08-09 04:05:00','military missile','success',981,'12L-14T','8K64 LKI-12','','','Test',566,'',1210 from dual union all
select 7765,'1961-S232','1961-08-09 04:31:00','military missile','success',162,'2F','Mk 5-1','R&D','','Test',228,'',1400 from dual union all
select 7766,'1961-S233','1961-08-04 00:00:00','military missile','success',1013,'','8K51 UBP-30 (39 RD)','','','Test',636,'',500 from dual union all
select 7767,'1961-S234','1961-08-13 00:00:00','military missile','success',1013,'','8K51 UBP-31 (39 RD)','','','Test',636,'',500 from dual union all
select 7768,'1961-S235','1961-08-11 14:30:00','suborbital rocket','success',470,'CRL AA13.194','Ion density','','','Test/Ionos',394,'',114 from dual union all
select 7769,'1961-S236','1961-08-12 00:00:00','military missile','success',928,'A1E-42 (A1P-9)','','','','OT',421,'SSBN 602',500 from dual union all
select 777,'1948-S07','1948-05-27 14:15:00','suborbital rocket','success',1470,'35','','','','Solar UV/CR',1863,'',140 from dual union all
select 7770,'1961-S237','1961-08-12 00:00:00','military missile','success',928,'A1E-43 (A1P-97)','','','','OT',421,'SSBN 602',500 from dual union all
select 7771,'1961-S238','1961-08-12 00:00:00','military missile','success',928,'A1E-34 (A1P-103)','','','','OT',421,'SSBN 602',500 from dual union all
select 7772,'1961-S239','1961-08-12 00:00:00','military missile','success',928,'A1E-35 (A1P-58)','','','','OT',421,'SSBN 602',500 from dual union all
select 7773,'1961-S24','1961-01-22 02:04:00','military missile','success',1053,'CC-2038','','','','Test',280,'',90 from dual union all
select 7774,'1961-S240','1961-08-12 00:00:00','military missile','success',928,'A1E-36 (A1P-105)','','','','OT',421,'SSBN 602',500 from dual union all
select 7775,'1961-S241','1961-08-12 00:00:00','military missile','success',928,'A1E-37 (A1P-98)','','','','OT',421,'SSBN 602',500 from dual union all
select 7776,'1961-S242','1961-08-12 10:57:00','suborbital rocket','success',35,'NASA 04.42NP','LeRC LH2 test','','','Tech',1821,'',153 from dual union all
select 7777,'1961-S243','1961-08-15 00:00:00','military missile','success',976,'','8K63 UBP-144','','','Test',636,'',402 from dual union all
select 7778,'1961-S244','1961-08-15 17:21:00','suborbital rocket','success',30,'AA3.130','EUV Monochromator','','','Solar EUV',1869,'',271 from dual union all
select 7779,'1961-S245','1961-08-16 20:27:00','suborbital rocket','success',837,'NASA 14.02GT','','','','Test',1821,'',182 from dual union all
select 778,'1948-S08','1948-07-26 16:47:00','suborbital rocket','success',58,'A7','','','','Earth imaging',1869,'',113 from dual union all
select 7780,'1961-S246','1961-08-16 00:00:00','military missile','success',1013,'','8K51 UBP-32','','','Test',636,'',500 from dual union all
select 7781,'1961-S247','1961-08-17 00:00:00','military missile','success',1009,'UNK','8A62 D04B','','','Target',636,'',500 from dual union all
select 7782,'1961-S248','1961-08-18 03:06:00','suborbital rocket','success',839,'NASA 10.51CI','GCA Ionos 3','','','Ionos',1821,'',141 from dual union all
select 7783,'1961-S249','1961-08-23 01:55:00','military missile','success',916,'316','P-21','','','Test',256,'',180 from dual union all
select 7784,'1961-S25','1961-01-23 21:02:05','military missile','success',154,'90D','Mk 3-1B-218A/LMSC specm','D/RIG, Mod 1 Decoy','','Test',227,'',896 from dual union all
select 7785,'1961-S250','1961-08-22 00:00:00','military missile','failure',1009,'UNK','8A62 D04B','','','Target',636,'',100 from dual union all
select 7786,'1961-S251','1961-08-22 00:00:00','military missile','success',1009,'UNK','8A62 D04B','','','Target',636,'',500 from dual union all
select 7787,'1961-S252','1961-08-23 01:16:00','military missile','success',154,'101D','Mk 3-3B','R&D','NEW NICKEL','Test',1667,'',1800 from dual union all
select 7788,'1961-S253','1961-08-23 17:02:00','suborbital rocket','success',30,'','EUV Mon/XR counter','','','Aeron/Solar EUV',1869,'',225 from dual union all
select 7789,'1961-S254','1961-08-24 00:00:00','military missile','success',1009,'UNK','8A62 D04B','','','Target',636,'',500 from dual union all
select 779,'1948-S09','1948-07-26 18:03:00','suborbital rocket','success',1470,'40','','','','CR/Aeron',1863,'',97 from dual union all
select 7790,'1961-S255','1961-08-25 00:00:00','military missile','success',1009,'UNK','8A62 D04B','','','Target',636,'',500 from dual union all
select 7791,'1961-S256','1961-08-25 00:00:00','military missile','success',1009,'UNK','8A62 D04B','','','Target',636,'',500 from dual union all
select 7792,'1961-S257','1961-08-28 17:41:00','suborbital rocket','success',838,'','ELF D-region','','','Ionos',394,'',200 from dual union all
select 7793,'1961-S258','1961-08-29 14:35:00','suborbital rocket','success',30,'NE3.110','NRL UV Echelle','','','Solar UV',1895,'',260 from dual union all
select 7794,'1961-S259','1961-08-30 00:00:00','military missile','success',976,'','8K63 UBP-145','','','Test',636,'',402 from dual union all
select 7795,'1961-S26','1961-01-24 21:55:00','military missile','failure',155,'8E','Mk 4-4','R&D','','Test',228,'',500 from dual union all
select 7796,'1961-S260','1961-08-30 12:34:00','suborbital rocket','success',1020,'RAM A-1','RAM A-1','','','Reentry Test',1821,'',1280 from dual union all
select 7797,'1961-S261','1961-08-31 00:00:00','military missile','success',976,'','8K63 UBP-146','','','Test',636,'',402 from dual union all
select 7798,'1961-S262','1961-09-01 00:00:00','military missile','success',976,'','8K63 UBP-149','','','Test',636,'',402 from dual union all
select 7799,'1961-S263','1961-09-01 00:00:00','military missile','success',979,'','8K65 UBP-5 29 RD','','','Test',636,'',675 from dual union all
select 78,'1944-A42','1944-12-01 00:00:00','military missile','success',943,'9','','','','Test',764,'',5 from dual union all
select 780,'1948-S10','1948-08-05 12:07:00','suborbital rocket','success',1470,'43','UV Spec/Solar X','','','Test/Solar UV,X',1863,'',166 from dual union all
select 7800,'1961-S264','1961-09-01 00:00:00','military missile','success',979,'','8K65 UBP-6 29 RD','','','Test',636,'',675 from dual union all
select 7801,'1961-S265','1961-09-21 00:00:00','military missile','success',1053,'CC-1006','','Bat B','','OT 40AG Bat B',437,'',90 from dual union all
select 7802,'1961-S266','1961-09-04 02:46:00','military missile','success',981,'23L','8K64 LKI-14','','','Test',566,'',1210 from dual union all
select 7803,'1961-S267','1961-09-06 00:00:00','military missile','success',1013,'','8K51 Nuclear Test','','','Nuclear Test',636,'',500 from dual union all
select 7804,'1961-S268','1961-09-07 00:00:00','military missile','success',1419,'AJ-17','Mk 4-2a RV/Capsule','','','Test',239,'',1000 from dual union all
select 7805,'1961-S269','1961-09-06 22:30:00','military missile','success',1367,'165','CTL','','SKYE BOAT','OT',1695,'',520 from dual union all
select 7806,'1961-S27','1961-01-26 00:30:00','military missile','success',916,'209','P-11','','','Test',256,'',180 from dual;
insert into LAUNCH
select 7807,'1961-S270','1961-09-08 01:30:00','military missile','success',1419,'M-3','Mk 4-4 RV','','','Test',237,'',1000 from dual union all
select 7808,'1961-S271','1961-09-07 18:17:00','suborbital rocket','success',838,'ISRC-RS-06','','','','Aeron',1343,'',200 from dual union all
select 7809,'1961-S272','1961-09-08 04:06:00','suborbital rocket','success',839,'ISRC-RS-07','','','','Aeron',1343,'',100 from dual union all
select 781,'1948-S11','1948-08-06 01:37:00','suborbital rocket','success',58,'NRL 1','Aerobee Venus 1','','','Aeron/Solar UV',1869,'',98 from dual union all
select 7810,'1961-S273','1961-09-08 04:11:00','suborbital rocket','success',839,'ISRC-RS-08','','','','Aeron',1343,'',145 from dual union all
select 7811,'1961-S274','1961-09-09 01:42:27','military missile','failure',155,'26E','Mk 4-2A-4 L-25599','R&D','','Test',228,'',100 from dual union all
select 7812,'1961-S275','1961-09-09 23:45:00','military missile','success',981,'22L','8K64 K (Kontrol''niy)','','','Test',566,'',1210 from dual union all
select 7813,'1961-S276','1961-09-10 05:50:00','military missile','success',976,'','8K63','Nuclear test','','Nuclear Test',1763,'',100 from dual union all
select 7814,'1961-S277','1961-09-11 00:00:00','military missile','success',976,'','8K63 UBP-147','','','Test',636,'',402 from dual union all
select 7815,'1961-S278','1961-09-12 05:00:00','military missile','success',976,'','8K63 Salekhard-3','Nuclear test','','Nuclear Test',1335,'',100 from dual union all
select 7816,'1961-S279','1961-09-13 00:00:00','military missile','success',916,'318','P-22','','','Test',257,'',180 from dual union all
select 7817,'1961-S28','1961-01-30 23:20:00','suborbital rocket','success',30,'AA3.370','Mass spectrometer','','','Aeron',394,'',204 from dual union all
select 7818,'1961-S280','1961-09-13 00:00:00','military missile','success',976,'','8K63 UBP-148','','','Test',636,'',402 from dual union all
select 7819,'1961-S281','1961-09-13 15:44:00','military missile','success',981,'17L-19T','8K64 LKI-15','','','Test',566,'',1210 from dual union all
select 782,'1948-S12','1948-09-03 01:00:00','suborbital rocket','success',1470,'33','UM-2 Grenades','','','Aeron',1863,'',151 from dual union all
select 7820,'1961-S282','1961-09-13 09:32:00','suborbital rocket','success',624,'NASA 8.06CA','Na release','','','Aeronomy',1821,'',331 from dual union all
select 7821,'1961-S283','1961-09-13 23:53:00','suborbital rocket','success',624,'NASA 8.22CA','Na release','','','Aeronomy',1821,'',431 from dual union all
select 7822,'1961-S284','1961-09-14 04:45:00','military missile','success',978,'','','','','Nuclear Test',190,'ZULUV',150 from dual union all
select 7823,'1961-S285','1961-09-16 04:00:00','military missile','success',978,'','','','','Nuclear Test',190,'ZULUV',150 from dual union all
select 7824,'1961-S286','1961-09-16 23:39:00','suborbital rocket','success',838,'NASA 03.18CA','GCA Release 15 Na','','','Aeronomy',1821,'',208 from dual union all
select 7825,'1961-S287','1961-09-16 23:55:00','suborbital rocket','failure',839,'NASA 10.36GA','GSFC Gren-Sphere 1','','','Aeronomy',1821,'',88 from dual union all
select 7826,'1961-S288','1961-09-17 15:45:00','military missile','success',981,'15L-17T','8K64 LKI-16','','','Test',566,'',1210 from dual union all
select 7827,'1961-S289','1961-09-17 04:47:00','suborbital rocket','success',1438,'TB Ii','','','','Reentry Test',1821,'',260 from dual union all
select 7828,'1961-S29','1961-02-01 00:00:00','military missile','success',979,'','8K65 UBP-1 43 RD','','','Test',636,'',675 from dual union all
select 7829,'1961-S290','1961-09-17 10:03:00','suborbital rocket','success',838,'NASA 03.19CA','GCA Release 16 Na','','','Aeronomy',1821,'',273 from dual union all
select 783,'1948-S13','1948-09-30 15:30:09','suborbital rocket','failure',265,'Bumper 3','','','','Test/Solar SX',1863,'',150 from dual union all
select 7830,'1961-S291','1961-09-17 10:13:00','suborbital rocket','success',839,'NASA 10.37GA','GSFC Gren-Sphere 2','','','Aeronomy',1821,'',92 from dual union all
select 7831,'1961-S292','1961-09-18 12:15:00','suborbital rocket','success',1195,'NASA 9.01GG','Southern Sky Survey','','','UV Astron',1839,'',193 from dual union all
select 7832,'1961-S293','1961-09-21 16:30:00','military missile','success',1017,'E 15003- 03','8K74 UBP-9','Max Range Test','','Test',562,'',1350 from dual union all
select 7833,'1961-S294','1961-09-23 00:00:00','suborbital rocket','success',972,'','','','','Aeron?',636,'',86 from dual union all
select 7834,'1961-S295','1961-09-23 20:36:00','military missile','success',1419,'SM-68-2','Mk 4-5H RV','R&D','BIG SAM','Test',1654,'',1000 from dual union all
select 7835,'1961-S296','1961-09-27 00:30:00','military missile','success',916,'319','P-23','','','Test',256,'',180 from dual union all
select 7836,'1961-S297','1961-09-26 00:00:00','military missile','success',976,'','8K63 UBP-150','','','Test',636,'',402 from dual union all
select 7837,'1961-S298','1961-09-26 00:57:00','military missile','success',1018,'E 103   15','8K75 LKI-11','','','Test',569,'',1160 from dual union all
select 7838,'1961-S299','1961-09-26 21:56:00','suborbital rocket','success',839,'OB6.16','','','','Aeron',1821,'',128 from dual union all
select 7839,'1961-S30','1961-02-15 00:00:00','military missile','success',979,'','8K65 LKI-22','','','Test',636,'',675 from dual union all
select 784,'1948-S14','1948-10-10 00:00:00','military missile','success',969,'I-1','','','','Test',636,'',100 from dual union all
select 7840,'1961-S300','1961-09-27 00:25:00','suborbital rocket','success',1194,'SL40','LUX-3/IC-1/UCL I-4','','','Ionos/Solar X',1839,'',153 from dual union all
select 7841,'1961-S301','1961-09-29 01:52:00','military missile','success',1419,'AJ-20','Mk 4-4 RV','','','Test',239,'',1000 from dual union all
select 7842,'1961-S302','1961-09-29 14:00:00','military missile','success',928,'A1X-50','FITVC test','','','R&D',244,'',1000 from dual union all
select 7843,'1961-S303','1961-09-30 00:00:00','military missile','success',976,'','8K63 UBP-151','','','Test',636,'',402 from dual union all
select 7844,'1961-S304','1961-10-01 00:00:00','military missile','success',976,'','8K63 UBP-152 29 RD','','','Test',636,'',402 from dual union all
select 7845,'1961-S305','1961-10-01 00:00:00','military missile','success',976,'','8K63 UBP-153','','','Test',636,'',402 from dual union all
select 7846,'1961-S306','1961-10-01 00:00:00','military missile','success',976,'','8K63 UBP-154','','','Test',636,'',402 from dual union all
select 7847,'1961-S307','1961-10-01 00:00:00','military missile','success',976,'','8K63 UBP-155','','','Test',636,'',402 from dual union all
select 7848,'1961-S308','1961-10-01 00:00:00','military missile','success',976,'','8K63 UBP-156','','','Test',636,'',402 from dual union all
select 7849,'1961-S309','1961-10-01 00:00:00','military missile','success',976,'','8K63 UBP-157','','','Test',636,'',402 from dual union all
select 785,'1948-S15','1948-10-13 00:00:00','military missile','success',969,'I-9','','','','Test',636,'',100 from dual union all
select 7850,'1961-S31','1961-01-31 16:54:51','test rocket','success',1054,'MRLV-2','MR-2','','','Test/Bio',279,'',251 from dual union all
select 7851,'1961-S310','1961-10-01 00:00:00','military missile','success',976,'','8K63 UBP-158','','','Test',636,'',402 from dual union all
select 7852,'1961-S311','1961-10-01 00:00:00','military missile','success',976,'','8K63 UBP-159','','','Test',636,'',402 from dual union all
select 7853,'1961-S312','1961-10-01 00:00:00','military missile','success',976,'','8K63 UBP-160','','','Test',636,'',402 from dual union all
select 7854,'1961-S313','1961-10-01 00:00:00','military missile','success',976,'','8K63 UBP-161','','','Test',636,'',402 from dual union all
select 7855,'1961-S314','1961-10-01 00:00:00','military missile','success',976,'','8K63 UBP-162','','','Test',636,'',402 from dual union all
select 7856,'1961-S315','1961-10-26 00:00:00','military missile','success',1053,'CC-1012','','','','OT 40AG',437,'',90 from dual union all
select 7857,'1961-S316','1961-09-30 14:30:00','suborbital rocket','success',30,'NASA 04.25US','OSO spectrometer','','','Solar X',1821,'',223 from dual union all
select 7858,'1961-S317','1961-10-02 18:23:00','ballistic missile test','success',155,'25E','Mk 5-1/Pod 7','R&D','','Test',226,'',1500 from dual union all
select 7859,'1961-S318','1961-10-04 14:22:00','military missile','success',929,'A2X-13','(A2X-15)','','','Test',421,'EAG-154',1000 from dual union all
select 786,'1948-S16','1948-10-21 00:00:00','military missile','success',969,'I-6','','','','Test',636,'',100 from dual union all
select 7860,'1961-S319','1961-10-04 00:00:00','military missile','success',1009,'','8A62 PRO Target C-15','NIIP-A Target','','Target',321,'',500 from dual union all
select 7861,'1961-S32','1961-02-01 00:00:00','military missile','success',976,'','8K63 UBP-115','','','Test',636,'',402 from dual union all
select 7862,'1961-S320','1961-10-04 11:19:00','suborbital rocket','success',1195,'NASA 9.02GG','Southern Sky Survey','','','UV Astron',1839,'',195 from dual union all
select 7863,'1961-S321','1961-10-05 00:00:00','suborbital rocket','success',363,'03','','','','Test/Aeron',313,'',130 from dual union all
select 7864,'1961-S322','1961-10-05 13:42:37','military missile','success',155,'30E','Mk 4-1B','R&D','','Test',228,'',1500 from dual union all
select 7865,'1961-S323','1961-10-06 00:00:00','military missile','success',1013,'','8K51 Nuclear Test','GROM','','Nuclear Test',636,'',500 from dual union all
select 7866,'1961-S324','1961-10-06 20:57:00','military missile','success',1018,'E 15001-21','8K75 LKI-12','','','Test',569,'',1160 from dual union all
select 7867,'1961-S325','1961-10-07 01:30:00','military missile','success',1419,'M-4','Mk 4-4 RV','','','Test',237,'',1000 from dual union all
select 7868,'1961-S326','1961-10-09 00:00:00','suborbital rocket','success',363,'02','','','','Test/Aeron',313,'',130 from dual union all
select 7869,'1961-S327','1961-10-09 20:47:00','military missile','success',982,'2LU','8K64U LKI-1','','','Test',566,'',1210 from dual union all
select 787,'1948-S17','1948-10-23 00:00:00','military missile','success',969,'I-10','','','','Test',636,'',100 from dual union all
select 7870,'1961-S328','1961-10-11 00:34:00','military missile','success',916,'320','P-24','','','Test',256,'',180 from dual union all
select 7871,'1961-S329','1961-10-10 17:40:00','sounding rocket','success',624,'NASA 8.23GA','','','','Aeronomy',1821,'',1544 from dual union all
select 7872,'1961-S33','1961-02-01 15:59:00','military missile','success',794,'401','Mk 5 RV','R&D','','Test',261,'',1300 from dual union all
select 7873,'1961-S330','1961-10-12 15:35:18','military missile','success',1018,'E 15001-23','8K75 LKI-14','','','Test',569,'',1160 from dual union all
select 7874,'1961-S331','1961-10-12 13:21:00','suborbital rocket','success',30,'AA3.146','UV absorption','','','Aeron',1869,'',192 from dual union all
select 7875,'1961-S332','1961-10-14 00:00:00','suborbital rocket','success',30,'[Spurious]','Lunar X-ray','','','XR Astron',1869,'',200 from dual union all
select 7876,'1961-S333','1961-10-14 04:26:00','suborbital rocket','success',624,'NASA 8.17AI','Topside Sounder','','','Ionos',1821,'',1055 from dual union all
select 7877,'1961-S334','1961-10-15 17:00:00','military missile','success',982,'4LU','8K64U LKI-3','','','Test',563,'',1210 from dual union all
select 7878,'1961-S335','1961-10-16 14:54:00','military missile','success',928,'A1E-38 (A1P-100)','','','','OT',421,'SSBN 608',500 from dual union all
select 7879,'1961-S336','1961-10-18 13:25:00','suborbital rocket','success',35,'NASA 04.40NP','LeRC LH2 test','','','Tech',1821,'',152 from dual union all
select 788,'1948-S18','1948-11-01 00:00:00','military missile','success',969,'I-3','','','','Test',636,'',100 from dual union all
select 7880,'1961-S337','1961-10-19 00:00:00','military missile','success',979,'','8K65 UBP-7 29 RD','','','Test',636,'',675 from dual union all
select 7881,'1961-S338','1961-10-20 00:00:00','military missile','success',978,'Nuclear Test','','','','Nuclear Test',190,'ZULUV',150 from dual union all
select 7882,'1961-S339','1961-10-21 00:00:00','suborbital rocket','success',1011,'','R-5A Geophysical','','','Aeron?',636,'',500 from dual union all
select 7883,'1961-S34','1961-02-02 00:00:00','suborbital rocket','success',972,'','','','','Aeron?',636,'',86 from dual union all
select 7884,'1961-S340','1961-10-21 00:00:00','suborbital rocket','success',1011,'','R-5A Geophysical','','','Aeron?',636,'',500 from dual union all
select 7885,'1961-S341','1961-10-22 17:00:00','military missile','success',982,'3LU','8K64U LKI-4','','','Test',563,'',1210 from dual union all
select 7886,'1961-S342','1961-10-22 00:00:00','military missile','success',1009,'UNK','8A62 D04B','','','Target',636,'',500 from dual union all
select 7887,'1961-S343','1961-10-23 16:38:40','military missile','success',929,'A2E-X1 (A2X-4)','','','','Test',421,'SSBN 608',1000 from dual union all
select 7888,'1961-S344','1961-10-23 00:00:00','military missile','success',1009,'','8A62 PRO Target C-16','NIIP-A Target','','Target',321,'',500 from dual union all
select 7889,'1961-S345','1961-10-24 00:00:00','military missile','success',1009,'UNK','8A62 D04B','','','Target',636,'',500 from dual union all
select 789,'1948-S19','1948-11-02 00:15:00','suborbital rocket','success',58,'A8','CR/Solar','','','Particles/Solar X',1869,'',91 from dual union all
select 7890,'1961-S346','1961-10-24 23:28:00','military missile','success',1419,'AJ-21','Mk 4-1 RV','Midas target','','Test',239,'',1000 from dual union all
select 7891,'1961-S347','1961-10-24 00:23:00','suborbital rocket','success',1194,'SL37','LUX-4/UCL I-5/JB-1','','','Solar X/Aeron/Meteorites',1839,'',142 from dual union all
select 7892,'1961-S348','1961-10-24 03:59:00','suborbital rocket','success',665,'K-8-8','','','','Ionos',7,'',200 from dual union all
select 7893,'1961-S349','1961-10-25 00:00:00','military missile','failure',1009,'UNK','8A62 D04B','','','Target',636,'',100 from dual union all
select 7894,'1961-S35','1961-02-02 00:00:00','suborbital rocket','success',972,'','','','','Aeron?',636,'',86 from dual union all
select 7895,'1961-S350','1961-10-25 06:59:00','suborbital rocket','success',30,'AB3.351','AS&E-2','','','XR Astron',1869,'',232 from dual union all
select 7896,'1961-S351','1961-10-26 00:00:00','military missile','failure',1009,'UNK','8A62 D04B','','','Target',636,'',100 from dual union all
select 7897,'1961-S352','1961-10-27 00:00:00','suborbital rocket','success',972,'','','','','Aeron?',636,'',86 from dual;
insert into LAUNCH
select 7898,'1961-S353','1961-10-27 00:00:00','suborbital rocket','success',972,'','','','','Aeron?',636,'',86 from dual union all
select 7899,'1961-S354','1961-10-27 00:00:00','military missile','success',976,'','8K63 K-1','Nuclear Test','','Nuclear Test',942,'',180 from dual union all
select 79,'1944-A43','1944-12-01 00:00:00','military missile','success',943,'10','','','','Test',764,'',5 from dual union all
select 790,'1948-S20','1948-11-03 00:00:00','military missile','success',969,'I-12','','','','Test',636,'',100 from dual union all
select 7900,'1961-S355','1961-10-27 00:00:00','military missile','success',976,'','8K63 K-2','Nuclear Test','','Nuclear test',942,'',300 from dual union all
select 7901,'1961-S356','1961-10-27 09:35:00','suborbital rocket','success',839,'NASA 10.52CI','GCA Ionos 4','','','Ionospheric',1821,'',148 from dual union all
select 7902,'1961-S357','1961-10-27 15:06:04','test rocket','success',1118,'SA-1','','','','Test',267,'',136 from dual union all
select 7903,'1961-S358','1961-10-28 15:40:00','military missile','success',982,'5LU','8K64U LKI-5','','','Test',563,'',1210 from dual union all
select 7904,'1961-S359','1961-10-28 00:00:00','military missile','success',1009,'UNK','8A62 D04B','','','Target',636,'',500 from dual union all
select 7905,'1961-S36','1961-02-05 13:48:00','suborbital rocket','success',35,'NASA 04.38NP','LeRC LH2 test','','','Tech',1821,'',152 from dual union all
select 7906,'1961-S361','1961-10-30 11:13:00','suborbital rocket','success',665,'K-8-9','','','','Ionos',7,'',175 from dual union all
select 7907,'1961-S362','1961-10-31 00:00:00','military missile','success',977,'','8K63U LKI-1 ShPU','','','Test',636,'',402 from dual union all
select 7908,'1961-S363','1961-10-31 00:00:00','military missile','success',979,'','8K65 UBP-9','','','Test',636,'',675 from dual union all
select 7909,'1961-S364','1961-11-01 03:00:00','suborbital rocket','success',30,'ND3.122','','','','Aeron',1869,'',212 from dual union all
select 791,'1948-S21','1948-11-04 00:00:00','military missile','success',969,'I-2','','','','Test',636,'',100 from dual union all
select 7910,'1961-S365','1961-11-01 17:56:00','suborbital rocket','success',1195,'NASA 9.03GG','Southern Sky Survey','','','UV Astron',1839,'',193 from dual union all
select 7911,'1961-S366','1961-11-03 01:00:00','military missile','success',916,'322','P-25','','','Test',256,'',180 from dual union all
select 7912,'1961-S367','1961-11-03 14:01:32','military missile','success',928,'A1E-39','','','','OT',421,'SSBN 608',500 from dual union all
select 7913,'1961-S368','1961-11-03 15:30:00','military missile','success',928,'A1E-40','','','','OT',421,'SSBN 608',500 from dual union all
select 7914,'1961-S369','1961-11-03 17:00:00','military missile','success',928,'A1E-41 (A1P-12)','','','','OT',421,'SSBN 608',500 from dual union all
select 7915,'1961-S37','1961-02-07 12:46:00','military missile','success',239,'BK.13','H14','','','Reentry/RF Astron/Ionos',1843,'',687 from dual union all
select 7916,'1961-S370','1961-11-08 16:10:00','military missile','success',929,'A2E-X2 (A2X-15)','','','','Test',421,'SSBN 608',1000 from dual union all
select 7917,'1961-S371','1961-11-08 09:00:00','suborbital rocket','success',1194,'SL35','MO-1/UCL Ion-6/JB-2','','','Aeronomy/Iono',1839,'',152 from dual union all
select 7918,'1961-S372','1961-11-09 00:00:00','military missile','failure',929,'A2X-14 (A2G-1)','Mk 2 guidance','','','R&D',251,'',40 from dual union all
select 7919,'1961-S373','1961-11-15 00:00:00','suborbital rocket','success',837,'','Speedball','','','Test',871,'',100 from dual union all
select 792,'1948-S22','1948-11-05 00:00:00','military missile','success',969,'I-11','','','','Test',636,'',100 from dual union all
select 7920,'1961-S374','1961-11-16 02:35:00','military missile','success',916,'323','P-26','','','Test R=480',256,'',180 from dual union all
select 7921,'1961-S375','1961-11-15 00:00:00','military missile','success',872,'ZW-28 20030','Intercept Highball','','','Test',1884,'',20 from dual union all
select 7922,'1961-S376','1961-11-15 00:00:00','military missile','success',872,'ZW-29','','','','Test',1884,'',50 from dual union all
select 7923,'1961-S377','1961-11-15 13:00:00','suborbital rocket','success',1011,'','8A62 Geophysical','','','Aeron/Ionos/Fields',636,'',430 from dual union all
select 7924,'1961-S378','1961-11-15 13:43:00','sounding rocket','success',630,'NASA 11.04GB','BIOS I','','','Biology     ARC/',1015,'',1057 from dual union all
select 7925,'1961-S379','1961-11-17 15:15:00','military missile','success',794,'405','Mk 5 RV','R&D','','Test',266,'',900 from dual union all
select 7926,'1961-S38','1961-02-08 00:00:00','military missile','success',979,'','8K65 UBP-2 43 RD','','','Test',636,'',675 from dual union all
select 7927,'1961-S380','1961-11-16 20:37:00','military missile','success',872,'ZM-3','PM-3 Dummy St 3','','','Test',1057,'',50 from dual union all
select 7928,'1961-S381','1961-11-17 16:06:00','suborbital rocket','success',624,'','','','','Aeron',1821,'',610 from dual union all
select 7929,'1961-S382','1961-11-18 06:30:00','suborbital rocket','success',839,'NASA 10.72NA','Airglow','','','Aeronomy',1821,'',142 from dual union all
select 793,'1948-S23','1948-11-18 22:35:00','suborbital rocket','success',1470,'44','Hermes B ramjet','','','Solar X/Biology',1863,'',145 from dual union all
select 7930,'1961-S383','1961-11-18 13:42:00','sounding rocket','success',630,'NASA 11.05GB','BIOS II','','','Biology',1015,'',1057 from dual union all
select 7931,'1961-S384','1961-11-20 18:04:00','suborbital rocket','success',1195,'NASA 9.04GG','Southern Sky Survey','','','UV Astron',1839,'',209 from dual union all
select 7932,'1961-S385','1961-11-21 19:00:00','military missile','success',872,'ZM-4','PM-4 Dummy St 3','','','Test',1057,'',50 from dual union all
select 7933,'1961-S386','1961-11-22 00:30:00','military missile','success',1419,'AJ-22','Nike TVX-3','NTMP','','Test',239,'',1000 from dual union all
select 7934,'1961-S387','1961-11-22 21:04:00','ballistic missile test','success',162,'4F','Mk 5-2/Pod 3/Pod 22','R&D','','Test/Ionos/Aeron',226,'',1400 from dual union all
select 7935,'1961-S388','1961-11-25 00:00:00','military missile','success',976,'','8K63 UBP-164 43 RD','','','Test',636,'',402 from dual union all
select 7936,'1961-S389','1961-11-25 00:00:00','military missile','success',979,'','8K65 UBP-10','','','Test',636,'',675 from dual union all
select 7937,'1961-S39','1961-02-10 00:00:00','military missile','success',976,'','8K63 UBP-116','','','Test',636,'',402 from dual union all
select 7938,'1961-S390','1961-11-29 00:00:00','military missile','success',979,'','8K65 UBP-11','','','Test',636,'',675 from dual union all
select 7939,'1961-S391','1961-11-29 05:59:03','military missile','success',981,'E 225   01-01T','8K64 UBP-1','','','Test',563,'',1210 from dual union all
select 794,'1948-S24','1948-12-09 16:08:00','suborbital rocket','success',1470,'42','','','','Aeron/Solar X/Bio',1863,'',108 from dual union all
select 7940,'1961-S392','1961-11-29 08:15:00','military missile','success',1017,'L2-3','8K74 UBP-10','GCh E15000-13','','OT',562,'',1350 from dual union all
select 7941,'1961-S393','1961-11-30 01:40:00','military missile','success',1419,'M-5','Mk 4-4 RV','','','Test',237,'',1000 from dual union all
select 7942,'1961-S394','1961-11-29 23:01:49','military missile','success',154,'53D','Mk 3-3B','OT','BIG PUSH','Test',1666,'',1800 from dual union all
select 7943,'1961-S395','1961-12-01 00:00:00','military missile','success',976,'','8K63 UBP-166','','','Test',636,'',402 from dual union all
select 7944,'1961-S396','1961-12-01 00:00:00','military missile','success',976,'','8K63 UBP-168','','','Test',636,'',402 from dual union all
select 7945,'1961-S397','1961-12-05 00:00:00','military missile','success',1053,'CC-1008','INH-19 target','Operation HARDNOSE','','OT 333AG/209AG  Bat B',1894,'',90 from dual union all
select 7946,'1961-S398','1961-11-30 00:00:00','military missile','success',871,'ZW-30','First suc 3-stage','','','Test',1884,'',50 from dual union all
select 7947,'1961-S399','1961-11-30 17:10:00','suborbital rocket','success',30,'NE3.123','','','','Solar EUV',1869,'',238 from dual union all
select 7948,'1961-S40','1961-02-10 00:00:00','military missile','success',976,'','8K63 UBP-117','','','Test',636,'',402 from dual union all
select 7949,'1961-S400','1961-12-02 01:00:00','military missile','success',916,'324','P-27','','','Test',256,'',180 from dual union all
select 795,'1948-S25','1948-12-09 22:38:00','suborbital rocket','success',59,'SC 1','Air sampling','','','Aeronomy',1869,'',91 from dual union all
select 7950,'1961-S401','1961-12-01 00:00:00','military missile','success',929,'A2X-23','','','','Test',251,'',1000 from dual union all
select 7951,'1961-S402','1961-12-01 00:00:00','military missile','success',979,'','8K65 UBP-12','','','Test',636,'',675 from dual union all
select 7952,'1961-S403','1961-12-01 20:40:00','ballistic missile test','success',155,'35E','Mk 4-4/Pod 23','R&D','','Test',228,'',1600 from dual union all
select 7953,'1961-S404','1961-12-02 00:00:00','military missile','success',976,'','8K63 UBP-165','','','Test',636,'',402 from dual union all
select 7954,'1961-S405','1961-12-04 00:00:00','military missile','success',979,'','8K65 UBP-13 68 RB','','','Test',636,'',675 from dual union all
select 7955,'1961-S406','1961-12-05 00:00:00','military missile','success',928,'A1X-51','FITVC test','','','R&D',244,'',500 from dual union all
select 7956,'1961-S407','1961-12-05 23:26:00','suborbital rocket','success',1195,'SL42','LUX-6/UCL I-8/Bir-4','','','Ionos/Solar X',1839,'',226 from dual union all
select 7957,'1961-S408','1961-12-06 00:00:00','suborbital rocket','success',287,'C02','','','','Aeronomy',1314,'',130 from dual union all
select 7958,'1961-S409','1961-12-06 00:00:00','suborbital rocket','success',287,'C06','','','','Aeronomy',664,'',130 from dual union all
select 7959,'1961-S41','1961-02-10 05:55:00','military missile','success',1419,'AJ-11','Mk 3-2b RV','','','Test',239,'',1000 from dual union all
select 796,'1949-A01','1949-01-11 00:00:00','atmospheric rocket','success',564,'','D13 Model','','','Test',1821,'',5 from dual union all
select 7960,'1961-S410','1961-12-06 01:30:00','military missile','success',1367,'214','CTL','','PIPERS DELIGHT','OT',1696,'',520 from dual union all
select 7961,'1961-S411','1961-12-06 22:37:00','military missile','success',638,'CM-115','CTL','','','OT',248,'',500 from dual union all
select 7962,'1961-S412','1961-12-07 21:18:36','military missile','success',154,'82D','Mk 3-3B','OT','BIG CHIEF','Test',1667,'',1800 from dual union all
select 7963,'1961-S413','1961-12-07 23:15:00','suborbital rocket','success',30,'AB3.344','Sphere','','','Aeron/Ionos',394,'',234 from dual union all
select 7964,'1961-S414','1961-12-08 00:00:00','military missile','success',979,'','8K65 UBP-14','','','Test',636,'',675 from dual union all
select 7965,'1961-S415','1961-12-08 08:15:00','sounding rocket','success',129,'','Project CAMBRIDGE','','','Geodetic',1015,'',2190 from dual union all
select 7966,'1961-S416','1961-12-09 00:00:00','suborbital rocket','success',287,'C05','','','','Aeronomy',1314,'',130 from dual union all
select 7967,'1961-S417','1961-12-09 06:05:00','suborbital rocket','success',287,'C07','','','','Aeronomy',664,'',130 from dual union all
select 7968,'1961-S418','1961-12-09 00:00:00','suborbital rocket','success',287,'C08','','','','Aeronomy',664,'',130 from dual union all
select 7969,'1961-S419','1961-12-10 17:01:00','suborbital rocket','success',839,'NASA 10.76GE','Auroral radiation','','','Cosmic rays',428,'',132 from dual union all
select 797,'1949-A02','1949-01-13 20:26:00','military missile','failure',574,'B-1 No. 1','Hermes II RTV-G-3','','','Test',1863,'',1 from dual union all
select 7970,'1961-S42','1961-02-11 10:41:00','suborbital rocket','success',1490,'AGI27','Airglow','','','Aeronomy',668,'',91 from dual union all
select 7971,'1961-S420','1961-12-12 20:16:00','ballistic missile test','failure',162,'5F','Mk 4-2A/Pod 24','R&D','','Test',226,'',1000 from dual union all
select 7972,'1961-S421','1961-12-14 00:05:00','military missile','success',1419,'AJ-23','Nike TVX-3','NTMP TV','','Test',239,'',1000 from dual union all
select 7973,'1961-S422','1961-12-13 12:00:00','suborbital rocket','success',30,'AB3.110','Micrometorites','','','Test/Meteorites',1869,'',246 from dual union all
select 7974,'1961-S423','1961-12-14 20:26:00','military missile','success',872,'ZM-6  20043','PM-6','','','Test',1057,'',80 from dual union all
select 7975,'1961-S424','1961-12-14 20:00:00','military missile','failure',871,'ZK-1  20047','KMR-1','','','Test',873,'',10 from dual union all
select 7976,'1961-S425','1961-12-14 07:09:48','suborbital rocket','success',1440,'TB IIa','Trailblazer IIa','','','Reentry Test',1821,'',157 from dual union all
select 7977,'1961-S426','1961-12-14 10:29:00','suborbital rocket','success',535,'HAD 103','','','','Aeron',1835,'',130 from dual union all
select 7978,'1961-S427','1961-12-16 01:20:00','military missile','failure',1419,'M-6','Mk 4-4 RV','','','Test',237,'',100 from dual union all
select 7979,'1961-S428','1961-12-15 17:00:00','suborbital rocket','success',30,'AB3.140','HR Solar Spectra','','','Solar UV',1869,'',169 from dual union all
select 798,'1949-A04','1949-02-01 18:38:00','suborbital rocket','failure',58,'NRL 3','Venus 3','','','Solar UV,X',1869,'',0 from dual union all
select 7980,'1961-S429','1961-12-18 13:04:00','military missile','success',794,'406','Mk 5 RV','R&D','','Test',264,'',900 from dual union all
select 7981,'1961-S43','1961-02-13 04:39:00','military missile','success',1017,'L1-3T','8K74 UBP-3','','','OT',562,'',1350 from dual union all
select 7982,'1961-S430','1961-12-20 03:32:00','ballistic missile test','success',155,'36E','Mk 5-1A/Pod 6/Pod 8','R&D','','Test/CR',228,'',1800 from dual union all
select 7983,'1961-S431','1961-12-21 03:35:00','ballistic missile test','failure',162,'6F','Mk 5-2/Pod 15/Pod 25','R&D','','Test/CR/Ionos/Meteorites',226,'',500 from dual union all
select 7984,'1961-S432','1961-12-21 19:54:00','suborbital rocket','success',839,'NASA 10.64GA','Structure','','','Aeronomy',1821,'',132 from dual union all
select 7985,'1961-S433','1961-12-21 21:31:00','suborbital rocket','success',839,'NASA 10.74GI','','','','Ionos',1821,'',99 from dual union all
select 7986,'1961-S434','1961-12-22 04:24:00','suborbital rocket','success',43,'NASA 06.05UI','UM Ionospheric','','','Ionos',1821,'',365 from dual union all
select 7987,'1961-S435','1961-12-23 00:00:00','military missile','success',976,'','8K63 UBP-167','','','Test',636,'',402 from dual;
insert into LAUNCH
select 7988,'1961-S436','1961-12-23 00:00:00','military missile','success',1009,'','8A62 PRO Target C-17','NIIP-A Target','','Target',321,'',500 from dual union all
select 7989,'1961-S437','1961-12-26 00:00:00','military missile','success',1009,'','8A62 PRO Target C-18','NIIP-A Target','','Target',321,'',500 from dual union all
select 799,'1949-A05','1949-02-02 00:00:00','atmospheric rocket','success',365,'','E15 Model','','','Test',1821,'',10 from dual union all
select 7990,'1961-S438','1961-12-26 00:00:00','military missile','success',1009,'','8A62 PRO Target C-19','NIIP-A Target','','Target',321,'',500 from dual union all
select 7991,'1961-S439','1961-12-26 05:05:00','suborbital rocket','success',667,'K-9L-2','','','','Test',7,'',343 from dual union all
select 7992,'1961-S44','1961-02-13 13:58:00','suborbital rocket','success',1194,'SL36','UCL GC-12','','','Aeronomy',1839,'',146 from dual union all
select 7993,'1961-S440','1961-12-27 00:00:00','test rocket','success',976,'','MP-1','','','Reentry Test',1013,'',405 from dual union all
select 7994,'1961-S441','1961-12-29 00:30:00','military missile','success',916,'327','P-28','','','Test',256,'',180 from dual union all
select 7995,'1961-S442','1961-12-29 00:00:00','military missile','success',976,'','8K63 UBP-169','','','Test',636,'',402 from dual union all
select 7996,'1961-S443','1961-12-29 00:00:00','military missile','success',1009,'','8A62 PRO Target C-20','NIIP-A Target','','Target',321,'',500 from dual union all
select 7997,'1961-S444','1961-12-29 00:00:00','military missile','success',1009,'','8A62 PRO Target C-21','NIIP-A Target','','Target',321,'',500 from dual union all
select 7998,'1961-S445','1961-12-30 00:00:00','military missile','success',976,'','8K63 UBP-170','','','Test',636,'',402 from dual union all
select 7999,'1961-S446','1961-12-30 00:00:00','military missile','success',1009,'','8A62 PRO Target C-22','NIIP-A Target','','Target',321,'',500 from dual union all
select 8,'1943-A01','1943-01-07 00:00:00','military missile','failure',8,'10','','','','Test',675,'',0 from dual union all
select 80,'1944-A44','1944-12-01 00:00:00','military missile','success',943,'11','','','','Test',764,'',5 from dual union all
select 800,'1949-A06','1949-03-01 00:00:00','suborbital rocket','success',58,'A9','Ship launch test','','','Test',1869,'',1 from dual union all
select 8000,'1961-S447','1961-09-01 00:00:00','military missile','success',983,'','Last LKI','','','Test',636,'',150 from dual union all
select 8001,'1961-S448','1961-01-01 00:00:00','suborbital rocket','unknown',1248,'','Flight 8','','','Test',1057,'F-3B',100 from dual union all
select 8002,'1961-S449','1961-01-18 00:00:00','suborbital rocket','success',88,'668 A363.4AA','','','','Test',1895,'',90 from dual union all
select 8003,'1961-S45','1961-02-14 00:00:00','sounding rocket','failure',1267,'E22-H1-1','Geodetic Flares','','','Test',1821,'',100 from dual union all
select 8004,'1961-S450','1961-01-20 00:00:00','suborbital rocket','success',88,'667 A362.4AA','','','','Test',1895,'',88 from dual union all
select 8005,'1961-S451','1961-02-02 00:00:00','suborbital rocket','success',88,'690 A375.4AA','','','','Test',1895,'',87 from dual union all
select 8006,'1961-S452','1961-01-01 00:00:00','suborbital rocket','success',257,'','Test','','','Test',1895,'',90 from dual union all
select 8007,'1961-S453','1961-02-01 00:00:00','suborbital rocket','success',1505,'V-A 23','','','','Aeron?',1490,'',100 from dual union all
select 8008,'1961-S454','1961-03-01 00:00:00','suborbital rocket','success',837,'','SPEEDBALL','','','Target',1869,'',160 from dual union all
select 8009,'1961-S455','1961-06-27 15:00:00','suborbital rocket','success',578,'Hugo III S/N-D2','KB-14A Camera','','','Imaging',1869,'',161 from dual union all
select 801,'1949-A07','1949-03-24 15:14:00','suborbital rocket','failure',58,'A12','','','','Ionospheric',1258,'AVM1',5 from dual union all
select 8010,'1961-S456','1961-06-01 00:00:00','military missile','success',976,'','8K63','Op. Rosa (to NZEM)','','Test',1763,'',400 from dual union all
select 8011,'1961-S457','1961-10-19 00:00:00','military missile','success',978,'Nuclear Test Demo','','','','Test',190,'ZULUV',150 from dual union all
select 8012,'1961-S458','1961-01-01 00:00:00','suborbital rocket','unknown',720,'LT13','','','','Aeron',1837,'',80 from dual union all
select 8013,'1961-S459','1961-09-01 00:00:00','military missile','success',992,'','R-16 training','','','Test',636,'',100 from dual union all
select 8014,'1961-S46','1961-02-14 23:50:00','suborbital rocket','success',839,'NASA 10.07GA','GSFC Grenade 6','','','Aeronomy',1821,'',130 from dual union all
select 8015,'1961-S460','1961-01-01 00:00:00','military missile','failure',1321,'','9K71 Phase 2 No. 2','','','Test',636,'',100 from dual union all
select 8016,'1961-S461','1961-10-27 00:00:00','military missile','success',976,'','8K63 K-1','Rocket 2? PRO?','','Nuclear Test support',942,'',180 from dual union all
select 8017,'1961-S462','1961-10-27 00:00:00','military missile','success',976,'','8K63 K-2','Rocket 2? PRO?','','Nuclear Test support',942,'',300 from dual union all
select 8018,'1961-S463','1961-09-03 00:00:00','military missile','success',976,'','8K63 Salekhard-1','','','Nuclear test support',1335,'',400 from dual union all
select 8019,'1961-S464','1961-09-04 00:00:00','military missile','success',976,'','8K63 Salekhard-2','','','Nuclear test support',1335,'',400 from dual union all
select 802,'1949-A08','1949-03-28 00:00:00','atmospheric rocket','success',365,'','XF-91 Model','','','Test',1821,'',10 from dual union all
select 8020,'1961-S465','1961-09-16 00:00:00','military missile','success',976,'','8K63 Salekhard-4','','','Nuclear test support',1335,'',400 from dual union all
select 8021,'1961-S466','1961-07-01 00:00:00','military missile','success',976,'','8K63 60 RD UBP-1','','','Test',1354,'',402 from dual union all
select 8022,'1961-S467','1961-04-08 00:00:00','military missile','success',1009,'V-1000 target','8A62 PRO Target C-13','V-1000 Target','','Target',321,'',300 from dual union all
select 8023,'1961-S468','1961-04-12 00:00:00','military missile','success',1009,'V-1000 target','8A62 PRO Target C-14','V-1000 Target','','Target',321,'',300 from dual union all
select 8024,'1961-S469','1961-08-11 00:00:00','military missile','success',1009,'UNK','8A62 D04B','','','Target',636,'',500 from dual union all
select 8025,'1961-S47','1961-02-15 08:20:00','suborbital rocket','success',972,'','UV ozone spectra','','','Aeron/Eclipse',636,'',96 from dual union all
select 8026,'1961-S470','1961-08-11 00:00:00','military missile','success',1009,'UNK','8A62 D04B','','','Target',636,'',500 from dual union all
select 8027,'1961-S471','1961-08-17 00:00:00','military missile','success',1009,'UNK','8A62 D04B','','','Target',636,'',500 from dual union all
select 8028,'1961-S472','1961-02-24 00:00:00','military missile','success',976,'','8K63 UBP-118 19 RD','','','Test',636,'',402 from dual union all
select 8029,'1961-S473','1961-04-13 00:00:00','military missile','success',976,'','8K63 UBP-130 40 RD','','','Test',636,'',402 from dual union all
select 803,'1949-A09','1949-03-29 00:00:00','atmospheric rocket','failure',365,'','D3 Model','','','Test',1821,'',10 from dual union all
select 8030,'1961-S474','1961-04-28 00:00:00','military missile','success',976,'','8K63 UBP-134 35 RD','','','Test',636,'',402 from dual union all
select 8031,'1961-S475','1961-04-28 00:00:00','military missile','success',976,'','8K63 UBP-135 29 RD','','','Test',636,'',402 from dual union all
select 8032,'1961-S477','1961-12-30 00:00:00','military missile','success',977,'','8K63U LKI-1 NPU','','','Test',636,'',402 from dual union all
select 8033,'1961-S478','1961-04-01 00:00:00','military missile','success',979,'','8K65 UBP-3 43 RD','','','Test',636,'',675 from dual union all
select 8034,'1961-S479','1961-11-01 00:00:00','military missile','success',976,'','8K63 UBP-163 32 RD','','','Test',636,'',402 from dual union all
select 8035,'1961-S48','1961-02-15 22:01:00','suborbital rocket','success',1490,'AGI29','','','','Aeronomy',668,'',150 from dual union all
select 8036,'1961-S49','1961-02-16 01:27:00','military missile','success',916,'210','P-12','','','Test',255,'',180 from dual union all
select 8037,'1961-S50','1961-02-17 02:26:00','suborbital rocket','success',839,'NASA 10.08CA','GSFC Grenade 7','','','Aeronomy',1821,'',85 from dual union all
select 8038,'1961-S51','1961-02-17 07:17:00','suborbital rocket','success',837,'AA5.391','','','','Test',1821,'',200 from dual union all
select 8039,'1961-S52','1961-02-17 19:04:00','suborbital rocket','success',30,'AA3.129','EUV Monochromator','','','Solar EUV',394,'',275 from dual union all
select 804,'1949-A10','1949-03-30 00:00:00','atmospheric rocket','success',365,'','D8 Model','','','Test',1821,'',10 from dual union all
select 8040,'1961-S53','1961-02-18 00:00:00','sounding rocket','success',1267,'E22-H1-2','Geodetic Flares','','','Test',1821,'',2027 from dual union all
select 8041,'1961-S54','1961-02-18 00:00:00','military missile','success',1009,'V-1000 target','8A62 PRO Target C-10','V-1000 Target','','Target',321,'',300 from dual union all
select 8042,'1961-S55','1961-02-20 00:00:00','military missile','success',1419,'AJ-13','Mk 4-4 RV','','','Test',237,'',1000 from dual union all
select 8043,'1961-S56','1961-02-21 14:10:00','test rocket','success',154,'67D','MA-2','','','Test',229,'',182 from dual union all
select 8044,'1961-S57','1961-02-22 00:00:00','military missile','success',1009,'V-1000 target','8A62 PRO Target C-11','V-1000 Target','','Target',321,'',300 from dual union all
select 8045,'1961-S58','1961-02-22 07:00:00','suborbital rocket','success',1490,'AGI24','Bio (Rat)','','','Life Sci',668,'',110 from dual union all
select 8046,'1961-S59','1961-02-24 00:19:00','suborbital rocket','success',839,'AA6.170','Sphere','','','Aeron',394,'',133 from dual union all
select 8047,'1961-S60','1961-02-24 18:29:00','military missile','success',155,'9E','Mk 3-2B','R&D','','Test',228,'',1500 from dual union all
select 8048,'1961-S61','1961-02-27 00:52:03','military missile','success',1017,'L2-1','8K74 UBP-4','','','OT',562,'',1350 from dual union all
select 8049,'1961-S62','1961-03-01 15:42:00','suborbital rocket','success',838,'AA8.820','VLF Propagation','','','Solar EUV',394,'',200 from dual union all
select 805,'1949-A11','1949-04-01 00:00:00','atmospheric rocket','success',365,'','D4','','','Test',1821,'',10 from dual union all
select 8050,'1961-S63','1961-03-01 18:19:00','military missile','success',929,'A2X-7','','','','Test',421,'EAG-154',1000 from dual union all
select 8051,'1961-S64','1961-03-02 00:00:00','military missile','success',976,'','8K63 PRO-1','V-1000 target','','Target',942,'',402 from dual union all
select 8052,'1961-S65','1961-03-02 05:07:00','military missile','success',916,'211','P-13','','','Test',256,'',180 from dual union all
select 8053,'1961-S66','1961-03-03 16:02:00','ballistic missile test','success',246,'D-4','HETS A2-1','','','Plasma',236,'',2540 from dual union all
select 8054,'1961-S67','1961-03-03 00:00:00','military missile','success',81,'010','','','','Reentry Test',313,'',150 from dual union all
select 8055,'1961-S68','1961-03-03 03:39:00','suborbital rocket','failure',839,'AA6.189','Electron density','','','Ionos',394,'',100 from dual union all
select 8056,'1961-S69','1961-03-03 13:59:19','military missile','success',981,'4L-6T','8K64 LKI-2','','','Test',563,'',1210 from dual union all
select 8057,'1961-S70','1961-03-04 00:00:00','military missile','success',976,'','8K63 PRO-2','V-1000 target','','Target',942,'',460 from dual union all
select 8058,'1961-S71','1961-03-06 00:00:00','military missile','success',81,'011','','','','Reentry Test',313,'',150 from dual union all
select 8059,'1961-S72','1961-03-06 09:56:00','suborbital rocket','success',1195,'SL63','UCL GC-13/QUB Na-7','','','Aeronomy',1839,'',231 from dual union all
select 806,'1949-A12','1949-04-01 00:00:00','atmospheric rocket','success',365,'','D4','','','Test',1821,'',10 from dual union all
select 8060,'1961-S73','1961-03-07 00:00:00','military missile','success',81,'012','','','','Reentry Test',313,'',150 from dual union all
select 8061,'1961-S74','1961-03-08 16:53:00','suborbital rocket','success',42,'AA10.192','Ne/RPA','','','Ionos',394,'',400 from dual union all
select 8062,'1961-S75','1961-03-08 17:53:00','suborbital rocket','success',129,'AE16.305','','','','Ionos',394,'',431 from dual union all
select 8063,'1961-S76','1961-03-09 00:00:00','military missile','success',929,'A2X-8','','','','Test',421,'EAG-154',1000 from dual union all
select 8064,'1961-S77','1961-03-09 02:30:00','military missile','success',1053,'CC-2040','','','','Test',280,'',90 from dual union all
select 8065,'1961-S78','1961-03-10 00:00:00','military missile','success',976,'','8K63 UBP-120','','','Test',636,'',402 from dual union all
select 8066,'1961-S79','1961-03-11 00:00:00','military missile','success',976,'','8K63 UBP-121','','','Test',636,'',402 from dual union all
select 8067,'1961-S80','1961-03-14 04:17:00','military missile','failure',155,'13E','Mk 3-1B','R&D','','Test',228,'',500 from dual union all
select 8068,'1961-S81','1961-03-14 21:52:00','suborbital rocket','success',839,'AA6.801','Antenna Breakdown Ex','','','Ionos',394,'',118 from dual union all
select 8069,'1961-S82','1961-03-15 17:30:00','military missile','success',929,'A2X-10','','','','Test',251,'',1000 from dual union all
select 807,'1949-A13','1949-04-21 00:00:00','atmospheric rocket','success',365,'','E17 Drag Model','','','Test',1821,'',10 from dual union all
select 8070,'1961-S83','1961-03-15 17:47:00','suborbital rocket','success',839,'NASA 10.49GT','Cajun fin test','','','Test',1821,'',130 from dual union all
select 8071,'1961-S84','1961-03-16 01:19:00','military missile','success',916,'212','P-14','','','Test',256,'',180 from dual union all
select 8072,'1961-S85','1961-03-17 00:00:00','military missile','success',976,'','8K63 UBP-122','','','Test',636,'',402 from dual union all
select 8073,'1961-S86','1961-03-23 19:25:00','military missile','success',928,'A1E-17','','','','OT',421,'SSBN 600',500 from dual union all
select 8074,'1961-S87','1961-03-24 17:30:00','test rocket','success',1054,'MRLV-5','MR-BD','','','Test',279,'',181 from dual union all
select 8075,'1961-S88','1961-03-24 19:17:00','suborbital rocket','success',839,'AA6.802','Antenna Breakdown Ex','','','Ionos',394,'',100 from dual union all
select 8076,'1961-S89','1961-03-25 00:00:00','military missile','success',976,'','8K63 UBP-123','','','Test',636,'',402 from dual;
insert into LAUNCH
select 8077,'1961-S90','1961-03-25 01:49:00','military missile','failure',155,'16E','Mk 3-2B','R&D','','Test',228,'',1000 from dual union all
select 8078,'1961-S91','1961-03-26 00:00:00','military missile','success',1009,'V-1000 target','8A62 PRO Target C-12','V-1000 Target','','Target',321,'',300 from dual union all
select 8079,'1961-S92','1961-03-26 16:54:00','suborbital rocket','success',43,'NASA 06.04UI','UM ionospheric','','','Ionos/Gamma Astron',1821,'',386 from dual union all
select 808,'1949-A14','1949-05-05 15:15:00','suborbital rocket','failure',1470,'46','Hermes B','','','Solar EUV/X',1863,'',8 from dual union all
select 8080,'1961-S93','1961-03-27 04:08:00','suborbital rocket','success',665,'K-8-5','ID-4/AG-1','','','Ionos',7,'',172 from dual union all
select 8081,'1961-S94','1961-03-27 00:00:00','military missile','success',976,'','8K63 UBP-124 29 RD','','','Test',636,'',402 from dual union all
select 8082,'1961-S95','1961-03-28 00:00:00','military missile','success',976,'','8K63 UBP-125','','','Test',636,'',402 from dual union all
select 8083,'1961-S96','1961-03-28 00:00:00','military missile','success',1419,'AJ-14','Mk 4-2a RV','','','Test',237,'',1000 from dual union all
select 8084,'1961-S97','1961-03-30 05:13:43','military missile','success',1367,'243','CTL','','SHEPHERDS BUSH','OT',1676,'',520 from dual union all
select 8085,'1961-S98','1961-06-23 23:40:00','military missile','success',872,'ZW-17','','','','Test',1884,'',40 from dual union all
select 8086,'1961-S99','1961-04-01 03:25:00','suborbital rocket','success',667,'K-9L-1','','','','Test',7,'',350 from dual union all
select 8087,'1961-U01','1961-08-17 14:29:00','ballistic missile test','success',247,'O-1','HETS','HETS','','Magnetospheric',235,'',225000 from dual union all
select 8088,'1961-U02','1961-04-21 05:56:20','suborbital rocket','success',1438,'TB Ig','','Meteor','','Reentry Test',1821,'',280 from dual union all
select 8089,'1961-U03','1961-10-19 17:38:00','ballistic missile test','success',1138,'ST-7','P-21','ST-7','','Plasma/Aeron',1812,'',6855 from dual union all
select 809,'1949-A15','1949-05-09 00:00:00','suborbital rocket','unknown',1517,'9B','','','','Test',1852,'',40 from dual union all
select 8090,'1961-U04','1961-12-04 04:00:16','ballistic missile test','success',247,'O-2','','HETS','','Magnetospheric',1015,'',44400 from dual union all
select 8091,'1961-W01','1961-01-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 8092,'1961-W02','1961-01-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 8093,'1961-W03','1961-01-17 17:15:00','suborbital rocket','success',716,'','','','','Aeron',1895,'',57 from dual union all
select 8094,'1961-W04','1961-02-01 00:00:00','suborbital rocket','success',88,'','','','','Aeron',428,'',60 from dual union all
select 8095,'1961-W05','1961-02-01 12:19:00','suborbital rocket','success',716,'','','','','Aeron',1895,'',62 from dual union all
select 8096,'1961-W06','1961-02-16 18:38:00','suborbital rocket','success',716,'','','','','Aeron',1895,'',51 from dual union all
select 8097,'1961-W07','1961-03-17 14:45:00','suborbital rocket','success',716,'','','','','Aeron',1895,'',69 from dual union all
select 8098,'1961-W08','1961-03-31 14:43:00','suborbital rocket','success',716,'','','','','Aeron',1895,'',60 from dual union all
select 8099,'1961-W09','1961-04-10 22:10:00','suborbital rocket','success',716,'','','','','Aeron',1895,'',68 from dual union all
select 81,'1944-A45','1944-12-01 00:00:00','military missile','success',943,'12','','','','Test',764,'',5 from dual union all
select 810,'1949-A16','1949-05-12 00:00:00','atmospheric rocket','success',565,'','F25 Drag Model','','','Test',1821,'',5 from dual union all
select 8100,'1961-W10','1961-04-19 23:45:00','suborbital rocket','success',716,'','','','','Aeron',1895,'',72 from dual union all
select 8101,'1961-W11','1961-04-20 14:30:00','suborbital rocket','success',716,'','','','','Aeron',1895,'',66 from dual union all
select 8102,'1961-W12','1961-05-04 21:47:00','suborbital rocket','success',716,'','Instrument test','','','Aeron',1895,'',55 from dual union all
select 8103,'1961-W13','1961-05-04 23:25:00','suborbital rocket','success',716,'','','','','Aeron',394,'',60 from dual union all
select 8104,'1961-W14','1961-05-09 23:30:00','suborbital rocket','success',88,'','Robin E1','','','Aeron',394,'',64 from dual union all
select 8105,'1961-W15','1961-05-10 01:00:00','suborbital rocket','success',88,'','Falling sphere','','','Aeron',394,'',64 from dual union all
select 8106,'1961-W16','1961-05-10 01:30:00','suborbital rocket','success',88,'','Robin E2','','','Aeron',394,'',64 from dual union all
select 8107,'1961-W17','1961-05-10 02:30:00','suborbital rocket','success',88,'','Robin E3','','','Aeron',394,'',64 from dual union all
select 8108,'1961-W18','1961-05-10 03:30:00','suborbital rocket','success',88,'','Robin E4','','','Aeron',394,'',64 from dual union all
select 8109,'1961-W19','1961-05-10 04:30:00','suborbital rocket','success',88,'','Robin E6','','','Aeron',394,'',64 from dual union all
select 811,'1949-A17','1949-05-13 17:35:00','atmospheric rocket','success',878,'48-19','Nike 48-1 31R','','','Test',1863,'',20 from dual union all
select 8110,'1961-W20','1961-05-10 05:30:00','suborbital rocket','success',88,'','Robin E7','','','Aeron',394,'',64 from dual union all
select 8111,'1961-W21','1961-05-10 06:30:00','suborbital rocket','success',88,'','Robin E8','','','Aeron',394,'',64 from dual union all
select 8112,'1961-W22','1961-05-10 07:30:00','suborbital rocket','success',88,'','Robin E9','','','Aeron',394,'',64 from dual union all
select 8113,'1961-W23','1961-05-10 08:30:00','suborbital rocket','success',88,'','Robin E10','','','Aeron',394,'',64 from dual union all
select 8114,'1961-W24','1961-05-10 09:30:00','suborbital rocket','success',88,'','Robin E11','','','Aeron',394,'',64 from dual union all
select 8115,'1961-W25','1961-05-10 10:15:00','suborbital rocket','success',88,'','Falling sphere','','','Aeron',394,'',64 from dual union all
select 8116,'1961-W26','1961-05-10 11:15:00','suborbital rocket','success',88,'','Falling sphere','','','Aeron',394,'',64 from dual union all
select 8117,'1961-W27','1961-05-10 12:15:00','suborbital rocket','success',88,'','Robin E12','','','Aeron',394,'',64 from dual union all
select 8118,'1961-W28','1961-05-10 13:15:00','suborbital rocket','success',88,'','Robin E13','','','Aeron',394,'',64 from dual union all
select 8119,'1961-W29','1961-05-10 14:15:00','suborbital rocket','success',88,'','Robin E14','','','Aeron',394,'',64 from dual union all
select 812,'1949-A18','1949-05-17 17:33:00','atmospheric rocket','success',878,'48-32','Nike 48-1 32R','','','Test',1863,'',11 from dual union all
select 8120,'1961-W30','1961-05-10 15:15:00','suborbital rocket','success',88,'','Robin E15','','','Aeron',394,'',64 from dual union all
select 8121,'1961-W31','1961-05-10 16:15:00','suborbital rocket','success',88,'','Robin E16','','','Aeron',394,'',64 from dual union all
select 8122,'1961-W32','1961-05-10 17:15:00','suborbital rocket','success',88,'','Robin E17','','','Aeron',394,'',64 from dual union all
select 8123,'1961-W33','1961-05-10 18:15:00','suborbital rocket','success',88,'','Robin E18','','','Aeron',394,'',64 from dual union all
select 8124,'1961-W34','1961-05-10 19:15:00','suborbital rocket','success',88,'','Robin E19','','','Aeron',394,'',64 from dual union all
select 8125,'1961-W35','1961-05-10 20:00:00','suborbital rocket','success',88,'','Falling sphere','','','Aeron',394,'',64 from dual union all
select 8126,'1961-W36','1961-05-10 21:15:00','suborbital rocket','success',88,'','Robin E20','','','Aeron',394,'',64 from dual union all
select 8127,'1961-W37','1961-05-10 22:15:00','suborbital rocket','success',88,'','Robin E21','','','Aeron',394,'',64 from dual union all
select 8128,'1961-W38','1961-05-16 19:46:00','suborbital rocket','success',88,'','Falling sphere','','','Aeron',394,'',64 from dual union all
select 8129,'1961-W39','1961-06-01 00:00:00','suborbital rocket','success',88,'','HYDRA Arcas','','','Test',1209,'LSD-31',60 from dual union all
select 813,'1949-A19','1949-05-20 17:36:00','atmospheric rocket','success',878,'48-34','Nike 48-1 33R','','','Test',1863,'',20 from dual union all
select 8130,'1961-W40','1961-06-16 20:18:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',66 from dual union all
select 8131,'1961-W41','1961-06-16 21:53:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',68 from dual union all
select 8132,'1961-W42','1961-08-30 00:00:00','suborbital rocket','success',88,'','Arcas Robin','','','Aeron',1821,'',60 from dual union all
select 8133,'1961-W43','1961-08-30 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',1821,'',60 from dual union all
select 8134,'1961-Y01','1961-08-14 21:05:00','suborbital rocket','success',88,'RTG K61-1','Nattpip','','','Aeron',919,'',91 from dual union all
select 8135,'1961-Y02','1961-09-01 00:00:00','suborbital rocket','success',88,'','Robin','','','Aeron',646,'',98 from dual union all
select 8136,'1962 A ALP','1962-06-19 12:19:01','orbital','success',1375,'321/D10','Tiros 5','Tiros E','','Sat',233,'',null from dual union all
select 8137,'1962 A BET','1962-06-23 00:30:47','orbital','success',1361,'339/FTV 1129','CORONA 44/9037','CORONA M-6','TIGHT SHIRT','Sat TAG46',1678,'',null from dual union all
select 8138,'1962 A CHI','1962-09-17 23:46:18','orbital','success',1361,'350/FTV 1133','CORONA 51/9043','CORONA M-12','BIG FLIGHT','Sat TAG53',1678,'',null from dual union all
select 8139,'1962 A DEL','1962-06-30 16:00:00','orbital','success',681,'','Kosmos-6','DS-P1 No. 1','','Sat',630,'',null from dual union all
select 814,'1949-A20','1949-05-24 00:00:00','atmospheric rocket','success',365,'','D4','','','Test',1821,'',10 from dual union all
select 8140,'1962 A EPS','1962-07-10 08:35:05','orbital','success',1375,'316/D11','Telstar 1','Telstar 1','','Sat',234,'',null from dual union all
select 8141,'1962 A ETA','1962-07-21 00:56:40','orbital','success',1361,'342/FTV 1130','CORONA 46/9039','CORONA M-8','ADOBE HOME','Sat TAG48',1679,'',null from dual union all
select 8142,'1962 A GAM','1962-06-28 01:09:00','orbital','success',1362,'340/FTV 1151','CORONA 45/9038','CORONA M-7','TRIAL TRACK','Sat TAG47',1673,'',null from dual union all
select 8143,'1962 A IOT','1962-07-28 09:18:31','orbital','success',1509,'T 15000- 07','Kosmos-7','Zenit-2 No. 4','','Sat',480,'',null from dual union all
select 8144,'1962 A KAP','1962-08-02 00:17:00','orbital','success',1362,'344/FTV 1152','CORONA 48/9041','CORONA M-10','APPLE GREEN','Sat TAG50',1673,'',null from dual union all
select 8145,'1962 A LAM','1962-08-05 17:58:59','orbital','success',145,'124D','FTV 2404','AFP-201 PVP 854','AIR SCOUT','Sat',1019,'',null from dual union all
select 8146,'1962 A MU','1962-08-11 08:30:00','orbital','success',1512,'E103-23','Vostok-3','Vostok-3A No. 5','','Sat',480,'',null from dual union all
select 8147,'1962 A NU','1962-08-12 08:02:17','orbital','success',1512,'E103-22','Vostok-4','Vostok-3A No. 6','','Sat',480,'',null from dual union all
select 8148,'1962 A OME','1962-09-27 09:39:51','orbital','success',1509,'T 15000- 06','Kosmos-9','Zenit-2 No. 7','','Sat',480,'',null from dual union all
select 8149,'1962 A OMI','1962-08-23 11:33:00','orbital','success',1143,'S117','FTV 3502 (AF-2)','P35-2','BLUE SCOUT III','Sat',1018,'',null from dual union all
select 815,'1949-A21','1949-05-27 00:00:00','suborbital rocket','unknown',1517,'10B','','','','Test',1852,'',40 from dual union all
select 8150,'1962 A PHI','1962-09-12 00:59:13','orbital','failure',802,'T103-14','','2MV-2 No. 1','','Sat',480,'',213 from dual union all
select 8151,'1962 A PI','1962-08-25 02:18:45','orbital','failure',802,'T103-12','','2MV-1 No. 1','','Sat',480,'',252 from dual union all
select 8152,'1962 A PSI','1962-09-18 08:53:08','orbital','success',1375,'318/D12','Tiros 6','Tiros F2','','Sat',233,'',null from dual union all
select 8153,'1962 A RHO','1962-08-27 06:53:14','deep space','success',145,'179D (AA6)','Mariner 2','Mariner R-2','','Sat',227,'',null from dual union all
select 8154,'1962 A SIG','1962-08-29 01:00:36','orbital','success',1362,'349/FTV 1153','CORONA 49/9044','CORONA M-11','APPLE RIND','Sat TAG51',1674,'',null from dual union all
select 8155,'1962 A TAU','1962-09-01 02:12:30','orbital','failure',802,'T103-13','','2MV-1 No. 2','','Sat',480,'',310 from dual union all
select 8156,'1962 A THE','1962-07-28 00:30:09','orbital','success',1361,'347/FTV 1131','CORONA 47/9040','CORONA M-9','ANCHOR ROPE','Sat TAG49',1678,'',null from dual union all
select 8157,'1962 A UPS','1962-09-01 20:39:09','orbital','success',1361,'348/FTV 1132','CORONA 50/9042A','ARGON 10','BEADY EYE','Sat TAG52',1679,'',null from dual union all
select 8158,'1962 A XI','1962-08-18 15:00:00','orbital','success',681,'','Kosmos-8','DS-K-8 No. 1','','Sat',630,'',null from dual union all
select 8159,'1962 A ZET','1962-07-18 20:51:20','orbital','success',145,'120D','FTV 2403','AFP-201 PVP 853','ARMORED CAR','Sat',1019,'',null from dual union all
select 816,'1949-A22','1949-06-07 00:00:00','military missile','failure',340,'4E','4E','','','Test',1863,'',20 from dual union all
select 8160,'1962 ALP','1962-01-26 20:30:11','deep space','success',145,'121D (AA3)','Ranger 3','NASA P-34','','Sat',227,'',null from dual union all
select 8161,'1962 B ALP','1962-09-29 06:05:00','orbital','success',1361,'341/FTV 6101 (TA1)','Alouette 1','Alouette 1','BIG GAME','Sat TAG54',1673,'',null from dual union all
select 8162,'1962 B BET','1962-09-29 23:34:50','orbital','success',1362,'351/FTV 1154','CORONA 52/9045','CORONA M-13','ARCTIC ZONE','Sat TAG55',1674,'',null from dual union all
select 8163,'1962 B CHI','1962-12-16 14:33:04','orbital','success',1144,'S115','Explorer 16','NASA S-55B','','Sat',1812,'',null from dual union all
select 8164,'1962 B DEL','1962-10-03 12:15:11','orbital','success',154,'113D','MA-8','Mercury SC16','','Sat',229,'',null from dual union all
select 8165,'1962 B EPS','1962-10-09 18:35:39','orbital','success',1361,'352/FTV 1134','CORONA 53/9046A','ARGON 9','CALL BOARD','Sat TAG56',1678,'',null from dual union all
select 8166,'1962 B ETA','1962-10-18 16:59:00','deep space','success',145,'215D (AA7)','Ranger 5','Ranger 5','','Sat',227,'',null from dual;
insert into LAUNCH
select 8167,'1962 B GAM','1962-10-02 22:11:30','orbital','success',1376,'345/D13','Explorer 14','EPE B','','Sat',234,'',null from dual union all
select 8168,'1962 B IOT','1962-10-24 17:55:04','orbital','failure',802,'T103-15','','2MV-4 No. 1','','Sat',480,'',485 from dual union all
select 8169,'1962 B KAP','1962-10-26 16:14:39','orbital','success',1362,'353/FTV 1401','CORONA 54/STARAD','CORONA R&D FTV 1401','ANCHOR BUOY','Sat TAG57',1674,'',null from dual union all
select 817,'1949-A23','1949-08-23 00:00:00','atmospheric rocket','success',365,'','D3 Model','','','Test',1821,'',10 from dual union all
select 8170,'1962 B LAM','1962-10-27 23:15:01','orbital','success',1376,'346/D14','Explorer 15','EPE C','','Sat',234,'',null from dual union all
select 8171,'1962 B MU','1962-10-31 08:08:00','orbital','success',1359,'319/AB012','ANNA 1B','ANNA 1B','','Sat',233,'',null from dual union all
select 8172,'1962 B NU','1962-11-01 16:14:16','deep space','success',802,'T103-16','Mars-1','2MV-4 No. 2','','Sat',480,'',null from dual union all
select 8173,'1962 B OME','1962-12-22 09:23:00','orbital','success',1509,'T 15000- 10','Kosmos-12','Zenit-2 No. 6','','Sat',480,'',null from dual union all
select 8174,'1962 B OMI','1962-11-05 22:04:32','orbital','success',1361,'356/FTV 1136','CORONA 55/9047','CORONA M-14','BAIL OUT','Sat TAG58',1678,'',null from dual union all
select 8175,'1962 B PHI','1962-12-14 21:26:07','orbital','success',1362,'368/FTV 1156','CORONA 58/9050','CORONA M-17','BABY DOLL','Sat TAG62',1679,'',null from dual union all
select 8176,'1962 B PI','1962-11-11 20:17:02','orbital','success',145,'128D','FTV 2405','AFP-201 PVP 855','AFTER DECK','Sat',1019,'',null from dual union all
select 8177,'1962 B PSI','1962-12-19 01:25:45','orbital','success',1144,'S118','Transit VA-1 (N-1)','Transit VA-1','BLUE SCOUT IV','Sat',1018,'',null from dual union all
select 8178,'1962 B RHO','1962-11-24 22:00:54','orbital','success',1361,'367/FTV 1135','CORONA 56/9048','CORONA M-15','GOLDEN RUSH','Sat TAG59',1678,'',null from dual union all
select 8179,'1962 B SIG','1962-12-04 21:30:35','orbital','success',1362,'361/FTV 1155','CORONA 57/9049','CORONA M-16','CALAMITY JANE','Sat TAG60',1674,'',null from dual union all
select 818,'1949-A24','1949-09-16 23:19:00','suborbital rocket','failure',1470,'32','Blossom IVC (Alb 3)','','','Solar X/Ionos/Biology',1863,'',5 from dual union all
select 8180,'1962 B TAU','1962-12-13 04:07:00','orbital','success',1362,'365/FTV 2351','POPPY 1/NRL Comp 2','POPPY 1 SV1','CORN FIELD','Sat TAG61',1673,'',null from dual union all
select 8181,'1962 B THE','1962-10-20 04:00:00','orbital','success',681,'','Kosmos-11','DS-A1 No. 1','','Sat',630,'',null from dual union all
select 8182,'1962 B UPS','1962-12-13 23:30:01','orbital','success',1377,'355/D15','Relay 1','Relay A','','Sat',233,'',null from dual union all
select 8183,'1962 B XI','1962-11-04 15:35:15','orbital','failure',802,'T103-17','','2MV-3 No. 1','','Sat',480,'',250 from dual union all
select 8184,'1962 B ZET','1962-10-17 09:00:00','orbital','success',1509,'T 15000- 03','Kosmos-10','Zenit-2 No. 5','','Sat',480,'',null from dual union all
select 8185,'1962 BET','1962-02-08 12:43:45','orbital','success',1375,'317/D7','Tiros 4','NASA A-9','','Sat',233,'',null from dual union all
select 8186,'1962 CHI','1962-06-02 00:31:28','orbital','success',1361,'335/FTV 1127','CORONA 43/9036','CORONA M-5','KNOTTY PINE','Sat TAG44',1678,'',null from dual union all
select 8187,'1962 DEL','1962-02-21 18:44:00','orbital','success',1361,'332/FTV 2301','FTV 2301','Ferret 1','CABLE SPLICE','Sat TAG38',1679,'',null from dual union all
select 8188,'1962 EPS','1962-02-27 19:39:20','orbital','success',1361,'241/FTV 1123','Discoverer 38','CORONA M-1','CAREER GIRL','Sat TAG39',1678,'',null from dual union all
select 8189,'1962 ETA','1962-03-07 22:10:31','orbital','success',145,'112D','Samos 6','Samos E-5','LOOSE TOOTH','Sat',1020,'',null from dual union all
select 819,'1949-A25','1949-09-21 00:00:00','atmospheric rocket','success',365,'','X-3 Model','','','Test',1821,'',10 from dual union all
select 8190,'1962 GAM','1962-02-20 14:47:39','orbital','success',154,'109D','MA-6','Mercury SC13','','Sat',229,'',null from dual union all
select 8191,'1962 IOT','1962-04-06 17:15:00','orbital','success',681,'5LK','Kosmos-2','1MS No. 1','','Sat',630,'',null from dual union all
select 8192,'1962 KAP','1962-04-09 15:04:48','orbital','success',145,'110D','Midas 5','Midas ETS-II F3','NIGHT HUNT','Sat',1020,'',null from dual union all
select 8193,'1962 LAM','1962-04-18 00:53:47','orbital','success',1361,'331/FTV 1124','Discoverer 39','CORONA M-2','LONG SLICE','Sat TAG40',1679,'',null from dual union all
select 8194,'1962 MU','1962-04-23 20:50:00','deep space','success',145,'133D (AA4)','Ranger 4','Ranger 4','','Sat',227,'',null from dual union all
select 8195,'1962 NU','1962-04-24 04:00:00','orbital','success',681,'4LK','Kosmos-3','2MS No. 1','','Sat',630,'',null from dual union all
select 8196,'1962 OME','1962-06-18 20:20:00','orbital','success',1361,'343/FTV 2312','FTV 2312','Ferret 2','TASTY TREAT','Sat TAG45',1679,'',null from dual union all
select 8197,'1962 OMI','1962-04-26 18:00:16','orbital','success',1375,'320/D9','Ariel 1','UK 1','','Sat',233,'',null from dual union all
select 8198,'1962 PHI','1962-05-30 01:00:04','orbital','success',1361,'336/FTV 1128','CORONA 42/9035','CORONA M-4','LEAK PROOF','Sat TAG43',1673,'',null from dual union all
select 8199,'1962 PI','1962-04-26 18:56:08','orbital','success',145,'118D','FTV 2401','AFP-201 PVP 851','DAINTY DOLL','Sat',1019,'',null from dual union all
select 82,'1944-A46','1944-12-01 00:00:00','military missile','success',943,'13','','','','Test',764,'',5 from dual union all
select 820,'1949-A26','1949-10-06 00:00:00','suborbital rocket','failure',574,'B-1 No. 2','Hermes II RTV-G-3','','','Test',1863,'',4 from dual union all
select 8200,'1962 PSI','1962-06-17 18:14:18','orbital','success',145,'115D','FTV 2402','AFP-201 PVP 852','RUBBER GUN','Sat',1019,'',null from dual union all
select 8201,'1962 RHO','1962-04-29 00:30:12','orbital','success',1361,'333/FTV 1125','CORONA 40/9033','CORONA M-3','TOTAL TIME','Sat TAG41',1678,'',null from dual union all
select 8202,'1962 SIG','1962-05-15 19:36:00','orbital','success',1361,'334/FTV 1126','CORONA 41/9034A','ARGON 5','HOLE PUNCH','Sat TAG42',1679,'',null from dual union all
select 8203,'1962 TAU','1962-05-24 12:45:16','orbital','success',154,'107D','MA-7','Mercury SC18','','Sat',229,'',null from dual union all
select 8204,'1962 THE','1962-03-16 11:59:00','orbital','success',681,'6LK','Kosmos-1','DS-2 No. 1','','Sat',630,'',null from dual union all
select 8205,'1962 UPS','1962-05-28 03:00:00','orbital','success',681,'3LK','Kosmos-5','2MS No. 2','','Sat',630,'',null from dual union all
select 8206,'1962 XI','1962-04-26 10:02:00','orbital','success',1512,'E103-20','Kosmos-4','Zenit-2 No. 2','','Sat',480,'',null from dual union all
select 8207,'1962 ZET','1962-03-07 16:06:18','orbital','success',1375,'301/D8','OSO 1','NASA S-16','','Sat',233,'',null from dual union all
select 8208,'1962-A01','1962-01-01 00:00:00','military missile','success',7,'5TYa','F2','','','Test',182,'',10 from dual union all
select 8209,'1962-A02','1962-01-01 00:00:00','military missile','success',7,'5TYa','F3','','','Test',182,'',10 from dual union all
select 821,'1949-A27','1949-11-16 00:00:00','atmospheric rocket','failure',442,'','F23 Ramjet','','','Test',1821,'',0 from dual union all
select 8210,'1962-A03','1962-01-01 00:00:00','military missile','success',7,'5TYa','F4','','','Test',182,'',10 from dual union all
select 8211,'1962-A04','1962-01-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 8212,'1962-A05','1962-01-10 20:28:16','suborbital spaceplane','failure',1520,'1-25-44','High alpha, Aero','','','Test',379,'NB-52 003',13 from dual union all
select 8213,'1962-A06','1962-01-12 00:00:00','military missile','success',764,'60-5401','','','','Test',240,'',12 from dual union all
select 8214,'1962-A07','1962-01-12 19:08:02','military missile','success',764,'60-5405','','','','Test',240,'',12 from dual union all
select 8215,'1962-A08','1962-01-16 21:10:00','atmospheric rocket','success',835,'Smoke 020','','','','Aeronomy',1821,'',16 from dual union all
select 8216,'1962-A09','1962-01-17 20:00:34','suborbital spaceplane','success',1520,'3-2-3','MH-96. Mach 5','','','Test',379,'NB-52 003',40 from dual union all
select 8217,'1962-A10','1962-01-19 00:00:00','atmospheric rocket','success',1065,'Ra26','Ranger 26','','','Test',2,'',20 from dual union all
select 8218,'1962-A100','1962-06-26 11:30:00','military missile','failure',871,'ZK-6','K-1 Atlas Intercept','','','Test',873,'',10 from dual union all
select 8219,'1962-A101','1962-06-27 21:08:10','suborbital spaceplane','success',1520,'1-30-51','High alpha, Mach 6','','','Test',379,'NB-52 003',37 from dual union all
select 822,'1949-A28','1949-12-15 17:10:00','suborbital rocket','failure',50,'USAF 2','MX-1011','','','Solar X/Aeron',640,'',0 from dual union all
select 8220,'1962-A102','1962-06-28 00:00:00','suborbital rocket','failure',217,'AA-3-4','BB3 04','','','Test',1821,'',13 from dual union all
select 8221,'1962-A103','1962-06-29 00:00:00','military missile','failure',1192,'20026','PL.2','','','Test',21,'B-52',12 from dual union all
select 8222,'1962-A104','1962-06-29 18:41:47','suborbital spaceplane','success',1520,'2-24-44','Heating rates, notch','','','Test',379,'NB-52 008',25 from dual union all
select 8223,'1962-A105','1962-07-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8224,'1962-A106','1962-07-09 08:52:19','suborbital rocket','failure',129,'','STARFISH DIag','','','Magnetospheric?',1016,'',1 from dual union all
select 8225,'1962-A107','1962-07-09 08:58:30','suborbital rocket','failure',545,'Project 6.3/X-3','STARFISH Rad X-3','','','Ionos',797,'',0 from dual union all
select 8226,'1962-A108','1962-07-10 00:00:00','suborbital spaceplane','failure',1520,'3-A-11','','','','Test',379,'NB-52 003',14 from dual union all
select 8227,'1962-A109','1962-07-11 00:00:00','suborbital spaceplane','failure',1520,'3-A-12','','','','Test',379,'NB-52 003',14 from dual union all
select 8228,'1962-A11','1962-01-23 17:05:37','military missile','failure',929,'A2X-17 (A2G-2)','Mk 2 guidance','','','R&D',251,'',150 from dual union all
select 8229,'1962-A110','1962-07-13 03:03:10','military missile','failure',794,'417','Mk 5 RV','R&D','','Test',264,'',1 from dual union all
select 823,'1949-A29','1949-12-23 00:00:00','atmospheric rocket','success',442,'','D4 Model','','','Test',1821,'',10 from dual union all
select 8230,'1962-A111','1962-07-13 14:24:59','military missile','failure',982,'10PU','8K64U LKI-10','Silo test','','Test',572,'',0 from dual union all
select 8231,'1962-A112','1962-07-16 00:00:00','suborbital spaceplane','failure',1520,'3-A-13','','','','Test',379,'NB-52 003',14 from dual union all
select 8232,'1962-A113','1962-07-16 22:09:25','suborbital spaceplane','success',1520,'1-31-52','Notch/ASAS/Aero drag','','','Test',379,'NB-52 008',32 from dual union all
select 8233,'1962-A114','1962-07-19 17:53:45','suborbital spaceplane','success',1520,'2-25-45','Heating rates, drag','','','Test',379,'NB-52 008',26 from dual union all
select 8234,'1962-A115','1962-07-26 09:13:53','military missile','failure',1370,'180','BLUEGILL PRIME','','BLUEGILL PRIME','Nuclear Test',772,'',0 from dual union all
select 8235,'1962-A116','1962-07-26 19:22:30','suborbital spaceplane','success',1520,'1-32-53','Aero stab. and drag','','','Test',379,'NB-52 003',30 from dual union all
select 8236,'1962-A117','1962-07-28 00:00:00','military missile','failure',976,'','8K63 UBP-197','','','Test',636,'',0 from dual union all
select 8237,'1962-A118','1962-07-29 04:16:00','military missile','failure',1018,'T 103   32','8K75 LKI 2-12','','','OT',583,'',200 from dual union all
select 8238,'1962-A119','1962-08-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8239,'1962-A12','1962-01-24 17:02:36','military missile','success',764,'59-4885','','','','Test',240,'',12 from dual union all
select 824,'1949-M01','1949-01-28 17:20:00','suborbital rocket','failure',1470,'45','Blossom','','','Solar X/Ionos/Aeron/Bio',1863,'',60 from dual union all
select 8240,'1962-A120','1962-08-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 8241,'1962-A121','1962-08-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 8242,'1962-A122','1962-08-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 8243,'1962-A123','1962-08-01 00:00:00','military missile','failure',843,'','Certification','','','Test',803,'',5 from dual union all
select 8244,'1962-A124','1962-08-22 04:00:00','suborbital rocket','success',642,'AT-150','Antenna Test','','','Test',838,'',13 from dual union all
select 8245,'1962-A125','1962-08-24 00:00:00','atmospheric rocket','success',620,'J7','HRV 913','','','Test',1836,'',24 from dual union all
select 8246,'1962-A126','1962-08-01 00:00:00','suborbital spaceplane','failure',1520,'3-A-15','','','','Test',379,'NB-52',14 from dual union all
select 8247,'1962-A127','1962-08-01 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',44 from dual union all
select 8248,'1962-A128','1962-08-02 17:56:15','suborbital spaceplane','success',1520,'3-8-16','MH-96 fixed gain','','','Test',379,'NB-52 003',44 from dual union all
select 8249,'1962-A129','1962-08-05 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',46 from dual union all
select 825,'1949-M02','1949-04-22 00:17:16','suborbital rocket','failure',265,'Bumper 6','','','','Test/CR/Solar SX',1863,'',50 from dual union all
select 8250,'1962-A13','1962-01-26 14:02:04','military missile','success',764,'59-2468','','','','Test',240,'',12 from dual union all
select 8251,'1962-A130','1962-08-07 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',44 from dual union all
select 8252,'1962-A131','1962-08-08 18:08:35','suborbital spaceplane','success',1520,'2-26-46','Heating, aero drag','','','Test',379,'NB-52 008',27 from dual union all
select 8253,'1962-A132','1962-08-08 19:15:00','atmospheric rocket','success',835,'Smoke 303','','','','Aeronomy',293,'',16 from dual union all
select 8254,'1962-A133','1962-08-09 15:05:02','military missile','failure',794,'418','Mk 5 RV','R&D','','Test',266,'',1 from dual union all
select 8255,'1962-A134','1962-08-09 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',41 from dual union all
select 8256,'1962-A135','1962-08-10 00:00:00','suborbital spaceplane','failure',1520,'3-A-17','','','','Test',379,'NB-52 003',14 from dual;
insert into LAUNCH
select 8257,'1962-A136','1962-08-10 21:11:21','military missile','failure',162,'57F','Mk 4-5B','R&D','CRASH TRUCK','Test',1716,'',10 from dual union all
select 8258,'1962-A137','1962-08-13 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',47 from dual union all
select 8259,'1962-A138','1962-08-15 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',45 from dual union all
select 826,'1949-M03','1949-06-02 13:10:00','suborbital rocket','success',59,'SC 2','Air Sampling','','','Aeronomy',1869,'',78 from dual union all
select 8260,'1962-A139','1962-08-16 15:39:00','suborbital rocket','success',88,'','McMurdo 3','','','Aeron',950,'',41 from dual union all
select 8261,'1962-A14','1962-02-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 8262,'1962-A140','1962-08-17 16:34:10','military missile','failure',929,'A2TF-3','','','','Test',244,'',1 from dual union all
select 8263,'1962-A141','1962-08-19 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',46 from dual union all
select 8264,'1962-A142','1962-08-20 18:08:40','suborbital spaceplane','success',1520,'2-27-47','Heating, drag, ASAS','','','Test',379,'NB-52 008',27 from dual union all
select 8265,'1962-A143','1962-08-22 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',31 from dual union all
select 8266,'1962-A144','1962-08-23 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',39 from dual union all
select 8267,'1962-A145','1962-08-29 18:36:03','suborbital spaceplane','success',1520,'2-28-48','Heating','','','Test',379,'NB-52 008',29 from dual union all
select 8268,'1962-A146','1962-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8269,'1962-A147','1962-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 827,'1949-M04','1949-07-21 16:01:00','suborbital rocket','success',59,'SC 4','Air Sampling','','','Aeronomy',1869,'',76 from dual union all
select 8270,'1962-A148','1962-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8271,'1962-A149','1962-09-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8272,'1962-A15','1962-02-01 16:09:00','military missile','success',764,'60-5403','','','','Test',240,'',12 from dual union all
select 8273,'1962-A150','1962-09-01 00:00:00','military missile','failure',843,'','Certification','','','Test',803,'',5 from dual union all
select 8274,'1962-A151','1962-09-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 8275,'1962-A152','1962-09-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 8276,'1962-A153','1962-09-01 00:00:00','atmospheric rocket','success',1065,'Bo1','Bonorong 1','','','Test',1836,'',20 from dual union all
select 8277,'1962-A154','1962-09-13 00:00:00','military missile','failure',1192,'20027','PL.3','','','Test',21,'B-52',10 from dual union all
select 8278,'1962-A155','1962-09-20 00:00:00','military missile','failure',976,'','8K63 UBP-203','','','Test',636,'',0 from dual union all
select 8279,'1962-A156','1962-09-21 00:00:00','suborbital rocket','failure',1021,'RAM B-1','RAM B-1','','','Reentry Test',1821,'',20 from dual union all
select 828,'1949-M05','1949-09-06 16:57:00','suborbital rocket','failure',1499,'2','Viking 2','','','Aeron/Photo',1852,'',57 from dual union all
select 8280,'1962-A158','1962-09-25 20:28:00','atmospheric rocket','success',835,'Smoke 304','','','','Aeronomy',293,'',11 from dual union all
select 8281,'1962-A159','1962-09-27 00:00:00','suborbital spaceplane','failure',1520,'2- A-49','','','','Test',379,'NB-52 008',14 from dual union all
select 8282,'1962-A16','1962-02-05 20:59:00','suborbital rocket','success',716,'','','','','Aeron',394,'',44 from dual union all
select 8283,'1962-A160','1962-09-27 08:08:00','suborbital rocket','unknown',835,'','','','','Aeron',394,'',10 from dual union all
select 8284,'1962-A161','1962-09-28 18:04:55','suborbital spaceplane','success',1520,'2-29-50','Heating, VO Stab','','','Test',379,'NB-52 008',20 from dual union all
select 8285,'1962-A162','1962-10-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8286,'1962-A163','1962-10-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8287,'1962-A164','1962-10-03 20:30:00','atmospheric rocket','success',835,'Smoke 302','','','','Aeronomy',293,'',15 from dual union all
select 8288,'1962-A165','1962-10-04 18:10:11','suborbital spaceplane','success',1520,'3-10-19','VO Stability','','','Test',379,'NB-52 008',34 from dual union all
select 8289,'1962-A166','1962-10-05 15:30:00','military missile','failure',932,'A3X-3','[No NF sep]','','','Test',251,'',50 from dual union all
select 829,'1949-M06','1949-09-20 17:03:00','suborbital rocket','success',59,'SC 5','Air Sampling','','','Aeronomy',1869,'',58 from dual union all
select 8290,'1962-A167','1962-10-08 19:00:00','atmospheric rocket','success',835,'Smoke 305','','','','Aeronomy',293,'',15 from dual union all
select 8291,'1962-A168','1962-10-09 18:58:32','suborbital spaceplane','success',1520,'2-30-51','VO Stability','','','Test',379,'NB-52 003',39 from dual union all
select 8292,'1962-A169','1962-10-10 20:30:00','atmospheric rocket','success',835,'Smoke 306','','','','Aeronomy',293,'',16 from dual union all
select 8293,'1962-A17','1962-02-06 19:00:00','atmospheric rocket','success',835,'Smoke 021','','','','Aeronomy',1821,'',17 from dual union all
select 8294,'1962-A170','1962-10-14 00:00:00','military missile','failure',928,'A1T-7','','','','Test',421,'EAG-154',10 from dual union all
select 8295,'1962-A171','1962-10-16 09:14:38','military missile','failure',1370,'156','BLUEGILL DOUBLE PRIME','','BLUEGILL DOUBLE PRIME','Nuclear Test',773,'',10 from dual union all
select 8296,'1962-A172','1962-10-18 00:51:54','military missile','failure',794,'422','Mk 5 RV','R&D','','Test',264,'',1 from dual union all
select 8297,'1962-A173','1962-10-18 16:34:00','suborbital rocket','success',88,'','McMurdo 8','','','Aeron',950,'',31 from dual union all
select 8298,'1962-A174','1962-10-19 19:53:00','atmospheric rocket','success',835,'Smoke 307','','','','Aeronomy',293,'',16 from dual union all
select 8299,'1962-A175','1962-10-22 03:36:13','military missile','failure',1018,'T 15001-01','8K75 LKI 2-13','8K75N','','OT',583,'',5 from dual union all
select 83,'1944-A47','1944-12-01 00:00:00','military missile','success',943,'14','','','','Test',764,'',5 from dual union all
select 830,'1949-M07','1949-09-29 16:58:00','suborbital rocket','success',1470,'49','NRL Ionos 1','','','Solar X/Ionos/Meteorites',1863,'',51 from dual union all
select 8300,'1962-A176','1962-10-22 00:00:00','military missile','success',1469,'','Op K-3','','','ABM Test',182,'',25 from dual union all
select 8301,'1962-A177','1962-10-23 19:30:40','suborbital spaceplane','success',1520,'3-11-20','VO Stability','','','Test',379,'NB-52 008',41 from dual union all
select 8302,'1962-A178','1962-10-28 06:36:00','military missile','failure',1018,'T 15001-05','8K75 LKI 2-14','Operation K-4','','OT',569,'',0 from dual union all
select 8303,'1962-A179','1962-10-28 00:00:00','military missile','success',1469,'','Op K-4','','','ABM Test',182,'',25 from dual union all
select 8304,'1962-A18','1962-02-07 04:38:00','suborbital rocket','failure',35,'NASA 04.35GG','UV Spectrograph','','','UV Astron',1821,'',4 from dual union all
select 8305,'1962-A180','1962-10-29 20:13:00','atmospheric rocket','success',835,'Smoke 308','','','','Aeronomy',293,'',13 from dual union all
select 8306,'1962-A181','1962-11-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8307,'1962-A182','1962-11-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8308,'1962-A183','1962-11-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8309,'1962-A184','1962-10-31 17:08:07','military missile','success',764,'58-1443','','','','Test',241,'',12 from dual union all
select 831,'1949-M08','1949-12-06 18:32:00','suborbital rocket','success',59,'SC 3','Air Sampling','','','Aeronomy',1869,'',65 from dual union all
select 8310,'1962-A185','1962-11-01 00:00:00','military missile','success',1469,'','Op K-5','','','ABM Test',182,'',25 from dual union all
select 8311,'1962-A186','1962-11-01 20:02:00','atmospheric rocket','success',835,'Smoke 309','','','','Aeronomy',293,'',13 from dual union all
select 8312,'1962-A187','1962-11-04 07:15:00','military missile','success',843,'','Radar transmitter','','','Test',803,'',24 from dual union all
select 8313,'1962-A188','1962-11-04 07:15:00','military missile','success',843,'','DOMINIC TIGHTROPE','','','Nuclear Test',803,'',24 from dual union all
select 8314,'1962-A189','1962-11-05 15:29:59','military missile','failure',932,'A3X-4','','','','Test',251,'',10 from dual union all
select 8315,'1962-A19','1962-02-08 18:00:01','military missile','success',764,'60-5406','','','','Test',240,'',12 from dual union all
select 8316,'1962-A190','1962-11-05 21:03:00','atmospheric rocket','success',835,'Smoke 310','','','','Aeronomy',293,'',21 from dual union all
select 8317,'1962-A191','1962-11-06 20:49:00','atmospheric rocket','success',835,'Smoke 311','','','','Aeronomy',293,'',16 from dual union all
select 8318,'1962-A192','1962-11-09 00:58:01','military missile','success',764,'59-4891','','','','Test',241,'',12 from dual union all
select 8319,'1962-A193','1962-11-09 18:23:07','suborbital spaceplane','failure',1520,'2-31-52','VO Stab, Bound.Layer','','','Test',379,'NB-52 008',16 from dual union all
select 832,'1949-M09','1949-12-07 00:16:00','suborbital rocket','success',59,'SC 7','Air Sampling','','','Aeronomy',1869,'',60 from dual union all
select 8320,'1962-A194','1962-11-13 19:30:00','atmospheric rocket','success',835,'Smoke 312','','','','Aeronomy',293,'',15 from dual union all
select 8321,'1962-A195','1962-11-16 04:22:00','suborbital rocket','failure',35,'NASA 04.79II','CSIRO VLF 1','','','Ionos',1821,'',29 from dual union all
select 8322,'1962-A196','1962-11-16 05:59:00','suborbital rocket','failure',839,'NASA 10.65GA','GSFC Grenade 22','','','Aeronomy',428,'',16 from dual union all
select 8323,'1962-A197','1962-11-16 17:45:00','atmospheric rocket','success',835,'Smoke 368','','','','Aeronomy',293,'',18 from dual union all
select 8324,'1962-A198','1962-11-16 20:23:00','atmospheric rocket','success',835,'Smoke 313','','','','Aeronomy',293,'',18 from dual union all
select 8325,'1962-A199','1962-11-19 00:00:00','suborbital rocket','success',1158,'','','','','Test',353,'',40 from dual union all
select 8326,'1962-A20','1962-02-13 05:52:00','military missile','failure',872,'ZM-8','PM-8 Dummy St 3','','','Test',1057,'',10 from dual union all
select 8327,'1962-A200','1962-11-19 00:00:00','suborbital rocket','success',1158,'','','','','Test',353,'',40 from dual union all
select 8328,'1962-A201','1962-11-19 00:00:00','suborbital rocket','success',1158,'','','','','Test',353,'',40 from dual union all
select 8329,'1962-A202','1962-11-20 16:55:00','suborbital rocket','failure',29,'NASA 01.14NA','UV Airglow','','','Aeronomy',1869,'',20 from dual union all
select 833,'1949-S01','1949-01-29 06:17:00','suborbital rocket','success',58,'NRL 2','Venus 2','','','Ionos/Solar X',1869,'',96 from dual union all
select 8330,'1962-A203','1962-11-25 17:45:00','military missile','failure',982,'T 22500-16LU','8K64U UBP-9','8F116','','Test',566,'',0 from dual union all
select 8331,'1962-A204','1962-11-28 18:45:00','military missile','failure',1192,'20031','GL.1','','','Test',21,'B-52',10 from dual union all
select 8332,'1962-A205','1962-11-29 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',38 from dual union all
select 8333,'1962-A206','1962-12-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8334,'1962-A207','1962-11-30 00:00:00','atmospheric rocket','success',705,'L6','Leopard 6','','','Test',2,'',20 from dual union all
select 8335,'1962-A208','1962-12-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 8336,'1962-A209','1962-12-01 00:00:00','suborbital rocket','failure',287,'C27','','','','Aeronomy',664,'',40 from dual union all
select 8337,'1962-A21','1962-02-15 21:13:47','military missile','success',764,'59-4886','','','','Test',240,'',12 from dual union all
select 8338,'1962-A210','1962-12-02 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',43 from dual union all
select 8339,'1962-A211','1962-12-04 21:47:37','military missile','success',764,'59-4892','','','','Test',242,'',12 from dual union all
select 834,'1949-S02','1949-02-17 17:00:00','suborbital rocket','success',1470,'48','','','','CR/Solar EUV/Bio',1863,'',100 from dual union all
select 8340,'1962-A212','1962-12-04 20:07:00','atmospheric rocket','success',835,'Smoke 314','','','','Aeronomy',293,'',16 from dual union all
select 8341,'1962-A213','1962-12-06 06:47:00','suborbital rocket','failure',88,'','Arcasonde 10.66 sup.','','','Aeron',428,'',0 from dual union all
select 8342,'1962-A214','1962-12-06 21:20:00','atmospheric rocket','success',835,'Smoke 315','','','','Aeronomy',293,'',14 from dual union all
select 8343,'1962-A215','1962-12-08 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',45 from dual union all
select 8344,'1962-A216','1962-12-10 20:00:00','military missile','failure',794,'502','Mk 5 RV','R&D Cat I','AMERICAN BEAUTY','Test',1650,'',20 from dual union all
select 8345,'1962-A217','1962-12-10 17:30:00','atmospheric rocket','success',835,'Smoke 316','','','','Aeronomy',293,'',16 from dual union all
select 8346,'1962-A218','1962-12-10 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',45 from dual;
insert into LAUNCH
select 8347,'1962-A219','1962-12-11 19:45:34','military missile','success',764,'59-4890','','','','Test',242,'',12 from dual union all
select 8348,'1962-A22','1962-02-21 20:23:34','military missile','success',764,'59-4889','','','','Test',240,'',12 from dual union all
select 8349,'1962-A220','1962-12-11 20:47:00','atmospheric rocket','success',835,'Smoke 317','','','','Aeronomy',293,'',14 from dual union all
select 835,'1949-S03','1949-02-24 22:22:44','suborbital rocket','success',265,'Bumper 5','','','','Test',1863,'',393 from dual union all
select 8350,'1962-A221','1962-12-12 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',41 from dual union all
select 8351,'1962-A222','1962-12-13 00:00:00','suborbital spaceplane','failure',1520,'3- A-21','VO Stab, UVP','','','Test/UV Astron',379,'NB-52 008',14 from dual union all
select 8352,'1962-A223','1962-12-13 17:23:00','suborbital rocket','failure',35,'NASA 04.74UA','JHU Airglow 3','','','Aeronomy',1821,'',0 from dual union all
select 8353,'1962-A224','1962-12-13 20:00:00','atmospheric rocket','success',835,'Smoke 318','','','','Aeronomy',293,'',19 from dual union all
select 8354,'1962-A225','1962-12-14 18:44:07','suborbital spaceplane','success',1520,'3-12-22','VO Stab, UVP','','','Test/UV Astron',379,'NB-52 008',43 from dual union all
select 8355,'1962-A226','1962-12-14 19:36:00','atmospheric rocket','success',835,'Smoke 319','','','','Aeronomy',293,'',18 from dual union all
select 8356,'1962-A227','1962-12-14 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',40 from dual union all
select 8357,'1962-A228','1962-12-17 20:26:00','atmospheric rocket','success',835,'Smoke 320','','','','Aeronomy',293,'',17 from dual union all
select 8358,'1962-A229','1962-12-18 17:03:00','atmospheric rocket','success',835,'Smoke 321','','','','Aeronomy',293,'',14 from dual union all
select 8359,'1962-A23','1962-02-25 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',43 from dual union all
select 836,'1949-S04','1949-03-17 23:20:00','suborbital rocket','success',58,'A10','','','','Ionospheric',1257,'AVM1',105 from dual union all
select 8360,'1962-A230','1962-12-18 17:26:55','military missile','failure',155,'64E','Nike TV-33','NTMP K-4','OAK TREE','ABM Test',1715,'',10 from dual union all
select 8361,'1962-A231','1962-12-19 17:00:00','atmospheric rocket','success',835,'Smoke 322','','','','Aeronomy',293,'',18 from dual union all
select 8362,'1962-A232','1962-12-20 19:25:04','suborbital spaceplane','success',1520,'3-13-23','VO Stab, MH-96','','','Test',379,'NB-52 008',48 from dual union all
select 8363,'1962-A233','1962-12-20 19:47:00','atmospheric rocket','success',835,'Smoke 323','','','','Aeronomy',293,'',16 from dual union all
select 8364,'1962-A234','1962-12-20 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',48 from dual union all
select 8365,'1962-A235','1962-12-21 17:26:00','atmospheric rocket','success',835,'Smoke 324','','','','Aeronomy',293,'',15 from dual union all
select 8366,'1962-A237','1962-12-24 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',48 from dual union all
select 8367,'1962-A238','1962-12-28 18:07:00','atmospheric rocket','success',835,'Smoke 325','','','','Aeronomy',293,'',21 from dual union all
select 8368,'1962-A239','1962-03-01 00:00:00','military missile','failure',1035,'','','','','Test',636,'',10 from dual union all
select 8369,'1962-A24','1962-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 837,'1949-S05','1949-03-22 06:43:00','suborbital rocket','success',1470,'41','Blossom IVA','','','Ionos/Aeron/Solar X',1863,'',129 from dual union all
select 8370,'1962-A241','1962-12-12 12:10:00','military missile','failure',871,'ZK-9 20077','K-6 Salvo','','','Test',873,'',10 from dual union all
select 8371,'1962-A242','1962-12-22 10:00:00','military missile','failure',871,'ZK-11 20100','K-7 Salvo','','','Test',873,'',10 from dual union all
select 8372,'1962-A243','1962-06-07 00:00:00','atmospheric rocket','success',252,'','Flight 15','','','Test',2,'',10 from dual union all
select 8373,'1962-A245','1962-01-01 00:00:00','military missile','unknown',871,'ZK-3','','','','Test',873,'',80 from dual union all
select 8374,'1962-A246','1962-04-01 00:00:00','military missile','unknown',871,'ZK-4','','','','Test',873,'',80 from dual union all
select 8375,'1962-A247','1962-04-01 00:00:00','military missile','unknown',871,'ZK-5','','','','Test',873,'',80 from dual union all
select 8376,'1962-A248','1962-05-12 00:00:00','military missile','success',871,'ZW-45','','','','Test',1884,'',50 from dual union all
select 8377,'1962-A249','1962-07-31 00:00:00','military missile','success',871,'ZW-48','','','','Test',1884,'',50 from dual union all
select 8378,'1962-A25','1962-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8379,'1962-A250','1962-02-23 00:00:00','military missile','success',871,'ZW-38','','','','Test',1884,'',50 from dual union all
select 838,'1949-S06','1949-03-22 17:20:00','suborbital rocket','success',58,'A11','','','','Ionospheric',1257,'AVM1',105 from dual union all
select 8380,'1962-A251','1962-02-19 00:00:00','suborbital rocket','failure',1350,'HITAB-BWSS-01','','','','IR background',394,'',10 from dual union all
select 8381,'1962-A252','1962-11-15 00:00:00','suborbital rocket','success',506,'','Dummy St 2','','','Meteo',310,'',5 from dual union all
select 8382,'1962-A253','1962-11-15 00:00:00','suborbital rocket','success',506,'','Dummy St 2','','','Meteo',310,'',5 from dual union all
select 8383,'1962-A254','1962-11-15 00:00:00','suborbital rocket','success',506,'','Dummy St 2','','','Meteo',310,'',5 from dual union all
select 8384,'1962-A255','1962-12-06 00:00:00','suborbital rocket','success',506,'','','','','Meteo',310,'',35 from dual union all
select 8385,'1962-A256','1962-11-21 06:09:00','suborbital rocket','success',642,'SP-150-3','Spin Test','','','Test',839,'',6 from dual union all
select 8386,'1962-A257','1962-03-29 02:34:00','suborbital rocket','success',642,'HT-150-1','Heating Test','K-6S Prototype','','Test',7,'',20 from dual union all
select 8387,'1962-A258','1962-11-21 02:09:00','suborbital rocket','success',642,'LT-150-1','Launcher Test','','','Test',839,'',5 from dual union all
select 8388,'1962-A259','1962-11-25 06:11:00','suborbital rocket','success',642,'SP-150-4','Spin Test','','','Test',839,'',10 from dual union all
select 8389,'1962-A26','1962-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 839,'1949-S07','1949-04-11 22:05:00','suborbital rocket','unknown',1470,'50','UM-3','','','Aeron/Solar X/Bio',1863,'',85 from dual union all
select 8390,'1962-A260','1962-12-20 03:11:00','suborbital rocket','success',642,'SO-150-1','Operational Reliability','Electronics test','','Test',839,'',12 from dual union all
select 8391,'1962-A27','1962-03-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8392,'1962-A28','1962-03-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 8393,'1962-A29','1962-03-01 16:00:05','military missile','success',764,'58-1425','','','','Test',240,'',12 from dual union all
select 8394,'1962-A30','1962-03-01 23:30:00','suborbital rocket','failure',839,'NASA 10.69GT','Water test','','','Test',1821,'',16 from dual union all
select 8395,'1962-A31','1962-03-02 16:00:00','military missile','success',764,'59-4888','','','','Test',240,'',12 from dual union all
select 8396,'1962-A32','1962-03-09 00:16:00','military missile','failure',871,'ZM-9','PM-9','','','Test',1057,'',10 from dual union all
select 8397,'1962-A33','1962-03-19 00:00:00','test rocket','success',61,'','','','','Test',664,'',20 from dual union all
select 8398,'1962-A34','1962-03-20 21:14:05','military missile','failure',1018,'E 103   17','8K75 LKI 2-1','','','OT',569,'',70 from dual union all
select 8399,'1962-A35','1962-03-21 00:00:00','military missile','failure',353,'','','','','Test',811,'',0 from dual union all
select 84,'1944-A48','1944-12-01 00:00:00','military missile','success',943,'15','','','','Test',764,'',5 from dual union all
select 840,'1949-S08','1949-05-03 16:14:00','suborbital rocket','success',1499,'1','Viking 1','','','Aeron/Photo',1852,'',83 from dual union all
select 8400,'1962-A36','1962-03-23 00:00:00','test rocket','success',61,'','','','','Test',664,'',20 from dual union all
select 8401,'1962-A37','1962-03-26 19:03:00','suborbital rocket','failure',274,'NC17.121','','','','Test/Aeron',1023,'F-4H',21 from dual union all
select 8402,'1962-A38','1962-03-28 00:00:00','military missile','success',871,'ZW-41','Hercules intercept','','','Test',1884,'',40 from dual union all
select 8403,'1962-A39','1962-03-29 00:00:00','suborbital spaceplane','failure',1520,'3-A-4','','','','Test',379,'NB-52 003',14 from dual union all
select 8404,'1962-A40','1962-03-30 21:28:00','military missile','failure',929,'A2X-28 (A2M-3)','NPRB Nylon Ph. RB','','','R&Dt',244,'',0 from dual union all
select 8405,'1962-A41','1962-03-30 00:00:00','suborbital spaceplane','failure',1520,'3-A-5','','','','Test',379,'NB-52 003',14 from dual union all
select 8406,'1962-A42','1962-04-01 00:00:00','military missile','success',7,'5TYa','F1','','','Test',182,'',10 from dual union all
select 8407,'1962-A43','1962-04-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8408,'1962-A44','1962-03-31 00:00:00','suborbital spaceplane','failure',1520,'3-A-6','','','','Test',379,'NB-52 003',14 from dual union all
select 8409,'1962-A45','1962-04-04 00:00:00','atmospheric rocket','success',620,'J5','','','','Test',1836,'',24 from dual union all
select 841,'1949-S09','1949-05-07 03:12:00','military missile','success',985,'1','','','','Test',636,'',100 from dual union all
select 8410,'1962-A46','1962-04-04 20:50:00','atmospheric rocket','success',835,'Smoke 022','','','','Aeronomy',1821,'',11 from dual union all
select 8411,'1962-A47','1962-04-09 21:07:40','military missile','failure',162,'11F','Mk 4-1B','R&D','','Test',226,'',0 from dual union all
select 8412,'1962-A48','1962-04-10 18:20:00','suborbital rocket','failure',1194,'SL162','UCL GC-20','','','Aeronomy',1839,'',0 from dual union all
select 8413,'1962-A49','1962-04-11 00:00:00','military missile','failure',979,'','8K65 UBP-20','','','Test',636,'',0 from dual union all
select 8414,'1962-A50','1962-04-12 16:00:25','test rocket','failure',245,'D-7','Reentry Test','','','Reentry Test',236,'',30 from dual union all
select 8415,'1962-A51','1962-04-16 05:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',44 from dual union all
select 8416,'1962-A52','1962-04-17 12:59:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',42 from dual union all
select 8417,'1962-A53','1962-04-18 00:00:00','suborbital spaceplane','failure',1520,'1- A-45','','','','Test',379,'NB-52',14 from dual union all
select 8418,'1962-A54','1962-04-19 21:03:00','military missile','failure',1192,'20025','PL.1','','','Test',21,'B-52G',78 from dual union all
select 8419,'1962-A55','1962-04-19 00:00:00','military missile','failure',872,'ZM-11','ZM-11 Sim target','','','Test',1057,'',10 from dual union all
select 842,'1949-S10','1949-05-10 15:57:00','military missile','success',985,'2','','','','Test',636,'',100 from dual union all
select 8420,'1962-A56','1962-04-19 18:02:20','suborbital spaceplane','success',1520,'1-26-46','ASAS, alpha 20, M=5','','','Test',379,'NB-52 003',46 from dual union all
select 8421,'1962-A57','1962-04-20 20:32:00','military missile','failure',1018,'E 15001-19','8K75 LKI 2-4','','','OT',569,'',5 from dual union all
select 8422,'1962-A58','1962-04-25 01:59:45','military missile','failure',794,'412','Mk 5 RV','R&D','','Test',266,'',20 from dual union all
select 8423,'1962-A59','1962-04-24 17:00:00','military missile','failure',916,'337','P-34','','','Test',256,'',10 from dual union all
select 8424,'1962-A60','1962-04-25 00:00:00','suborbital spaceplane','failure',1520,'2- A-38','','','','Test',379,'NB-52',14 from dual union all
select 8425,'1962-A61','1962-04-26 00:00:00','suborbital spaceplane','failure',1520,'2- A-39','','','','Test',379,'NB-52',14 from dual union all
select 8426,'1962-A62','1962-04-27 00:00:00','suborbital spaceplane','failure',1520,'1- A-47','','','','Test',379,'NB-52',14 from dual union all
select 8427,'1962-A63','1962-05-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8428,'1962-A64','1962-05-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 8429,'1962-A65','1962-05-02 00:00:00','suborbital spaceplane','success',1520,'(3-C)','Eglin Demo','','','Demo',379,'NB-52 003',14 from dual union all
select 843,'1949-S11','1949-05-15 02:48:00','military missile','success',985,'3','','','','Test',636,'',100 from dual union all
select 8430,'1962-A66','1962-05-04 20:30:00','atmospheric rocket','success',835,'Smoke 023','','','','Aeronomy',1821,'',9 from dual union all
select 8431,'1962-A67','1962-05-04 20:30:00','atmospheric rocket','success',835,'Smoke 024','','','','Aeronomy',1821,'',12 from dual union all
select 8432,'1962-A68','1962-05-04 20:30:00','atmospheric rocket','success',835,'Smoke 025','','','','Aeronomy',1821,'',18 from dual union all
select 8433,'1962-A69','1962-05-05 00:00:00','military missile','unknown',274,'SIP','','','','Test',1415,'',20 from dual union all
select 8434,'1962-A70','1962-05-05 00:00:00','suborbital spaceplane','success',1520,'(3-C)','Eglin Demo','','','Demo',394,'NB-52 003',14 from dual union all
select 8435,'1962-A71','1962-05-05 00:00:00','suborbital spaceplane','success',1520,'(3-C)','Eglin Demo','','','Demo',19,'NB-52 003',14 from dual union all
select 8436,'1962-A72','1962-05-05 18:40:00','atmospheric rocket','success',835,'Smoke 026','','','','Aeronomy',1821,'',15 from dual;
insert into LAUNCH
select 8437,'1962-A73','1962-05-08 18:01:28','suborbital spaceplane','success',1520,'2-22-40','Heat T/ASAS/High al.','','','Test',379,'NB-52 008',21 from dual union all
select 8438,'1962-A74','1962-05-08 19:49:00','test rocket','failure',152,'F-1','Centaur R&D','R&D','','Test',268,'',6 from dual union all
select 8439,'1962-A75','1962-05-09 00:00:00','suborbital rocket','failure',203,'B02','','','','Test',313,'',0 from dual union all
select 844,'1949-S12','1949-05-16 21:55:00','military missile','success',985,'4','','','','Test',636,'',100 from dual union all
select 8440,'1962-A76','1962-05-09 13:29:59','military missile','failure',916,'330','P-36','','','Test',256,'',10 from dual union all
select 8441,'1962-A77','1962-05-09 22:15:24','military missile','failure',929,'A2PE-3','SDAP (DASO)','','','OT',421,'SSBN 609',10 from dual union all
select 8442,'1962-A78','1962-05-15 00:00:00','suborbital rocket','failure',203,'B03','','','','Test',313,'',0 from dual union all
select 8443,'1962-A79','1962-05-19 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',47 from dual union all
select 8444,'1962-A80','1962-05-22 18:04:46','suborbital spaceplane','success',1520,'1-28-49','Local flow','','','Test',379,'NB-52 003',30 from dual union all
select 8445,'1962-A81','1962-05-22 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',47 from dual union all
select 8446,'1962-A82','1962-05-23 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',37 from dual union all
select 8447,'1962-A83','1962-05-24 10:50:00','suborbital rocket','failure',665,'K-8-10','','','','Ionos',7,'',0 from dual union all
select 8448,'1962-A84','1962-05-25 00:00:00','suborbital spaceplane','failure',1520,'2- A-41','','','','Test',379,'NB-52',14 from dual union all
select 8449,'1962-A85','1962-05-25 17:40:00','atmospheric rocket','success',835,'Smoke 027','','','','Aeronomy',1821,'',15 from dual union all
select 845,'1949-S13','1949-05-24 01:40:00','military missile','success',985,'5','FIAR-1','','','Test/Aeron',636,'',100 from dual union all
select 8450,'1962-A86','1962-05-27 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',41 from dual union all
select 8451,'1962-A87','1962-05-29 00:00:00','suborbital spaceplane','failure',1520,'2- A-42','','','','Test',379,'NB-52',14 from dual union all
select 8452,'1962-A88','1962-06-01 00:00:00','military missile','unknown',339,'','','','','Test',1895,'',40 from dual union all
select 8453,'1962-A89','1962-06-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 8454,'1962-A90','1962-06-01 18:51:15','suborbital spaceplane','success',1520,'2-23-43','ASAS, 23 alpha, M=5','','','Test',379,'NB-52 008',40 from dual union all
select 8455,'1962-A91','1962-06-04 14:00:00','military missile','failure',929,'A2X-30 (A2MG-2)','Mk 2 guid./NPRB','','','Test',244,'',10 from dual union all
select 8456,'1962-A92','1962-06-06 00:00:00','suborbital rocket','failure',203,'B08','Radioactivity','','','Aeronomy',664,'',0 from dual union all
select 8457,'1962-A93','1962-06-07 08:52:00','suborbital rocket','failure',838,'NASA 03.22CA','GCA Release 23 Na','','','Aeronomy',1821,'',34 from dual union all
select 8458,'1962-A94','1962-06-07 18:29:20','suborbital spaceplane','success',1520,'1-29-50','Local flow, M=5','','','Test',379,'NB-52 003',31 from dual union all
select 8459,'1962-A95','1962-06-14 00:00:00','suborbital rocket','success',122,'','Pershing meteo','','','Aeron',293,'',30 from dual union all
select 846,'1949-S14','1949-05-28 01:50:00','military missile','success',985,'6','','','','Test',636,'',100 from dual union all
select 8460,'1962-A96','1962-06-15 02:00:01','military missile','failure',916,'342','P-38','','','Test',256,'',40 from dual union all
select 8461,'1962-A97','1962-06-19 16:33:00','suborbital rocket','success',88,'','McMurdo 1','','','Aeron',950,'',32 from dual union all
select 8462,'1962-A98','1962-06-20 08:46:16','military missile','failure',1370,'193','STARFISH','','STARFISH','Nuclear Test',772,'',10 from dual union all
select 8463,'1962-A99','1962-06-26 00:00:00','atmospheric rocket','success',620,'J6','','','','Test',1836,'',24 from dual union all
select 8464,'1962-F01','1962-01-13 21:41:02','orbital','failure',1361,'327/FTV 1120','Discoverer 37','CORONA C''''''-6','CANDY WRAPPER','Sat TAG37',1678,'',200 from dual union all
select 8465,'1962-F02','1962-01-24 09:30:00','orbital','failure',1359,'311/AB010','NRL Composite 1/GRAB','GRAB SV4','','Sat',234,'',150 from dual union all
select 8466,'1962-F03','1962-04-26 10:49:00','orbital','failure',1141,'S111','GRAB/SR 4B','GRAB SV5','BLUE SCOUT I','Sat',1018,'',100 from dual union all
select 8467,'1962-F04','1962-05-10 12:06:00','orbital','failure',1359,'314/AB011','ANNA 1A','ANNA 1A','','Sat',234,'',200 from dual union all
select 8468,'1962-F05','1962-05-24 00:00:00','orbital','failure',1143,'S112','FTV 3501 (AF-1)','P35-1','BLUE SCOUT II','Sat',1018,'',40 from dual union all
select 8469,'1962-F06','1962-06-01 09:38:00','orbital','failure',1509,'E 15000- 01','','Zenit-2 No. 3','','Sat',480,'',0 from dual union all
select 847,'1949-S15','1949-06-14 22:35:00','suborbital rocket','success',1470,'47','Blossom IVB (Alb 2)','','','Ionos/Solar X/Biology',1863,'',134 from dual union all
select 8470,'1962-F07','1962-07-22 09:21:23','orbital','failure',145,'145D (AA5)','Mariner 1','Mariner R-1','','Sat',227,'',90 from dual union all
select 8471,'1962-F08','1962-10-25 07:00:00','orbital','failure',681,'7LK','','1MS No. 2','','Sat',630,'',20 from dual union all
select 8472,'1962-F09','1962-12-17 20:36:33','orbital','failure',145,'131D','Midas 6','Midas ETS-III F1','BARGAIN COUNTER','Sat',1020,'',10 from dual union all
select 8473,'1962-M01','1962-02-23 00:00:00','military missile','failure',1419,'SM-68-18','Mk 4 RV','R&D Cat II','BLUE GANDER','OT',1654,'',50 from dual union all
select 8474,'1962-M02','1962-03-19 15:30:00','suborbital rocket','failure',35,'NASA 04.18GA','Structure','','','Aeronomy',1821,'',79 from dual union all
select 8475,'1962-M03','1962-04-05 04:31:00','suborbital rocket','success',839,'NASA 10.79NA','Ozone release','','','Aeronomy',1821,'',70 from dual union all
select 8476,'1962-M04','1962-04-05 18:04:25','suborbital spaceplane','success',1520,'3-3-7','MH-96','','','Test',379,'NB-52 003',54 from dual union all
select 8477,'1962-M05','1962-04-20 19:26:58','suborbital spaceplane','success',1520,'3-4-8','MH-96, M=5','','','Test',379,'NB-52 008',63 from dual union all
select 8478,'1962-M06','1962-04-30 18:23:20','suborbital spaceplane','success',1520,'1-27-48','Alt, Low q, Aero','','','Test',379,'NB-52 008',75 from dual union all
select 8479,'1962-M07','1962-05-09 22:08:00','military missile','failure',871,'ZM-12','ZM-12','','','Test',1057,'',50 from dual union all
select 848,'1949-S16','1949-06-15 02:03:00','suborbital rocket','success',58,'NRL 5','Ozone spectrograph','','','Solar UV',1869,'',109 from dual union all
select 8480,'1962-M08','1962-05-16 00:00:00','suborbital rocket','success',203,'B04','','','','Test',313,'',60 from dual union all
select 8481,'1962-M09','1962-05-29 00:00:00','suborbital rocket','success',203,'B07','Radioactivity','','','Aeronomy',664,'',60 from dual union all
select 8482,'1962-M10','1962-06-12 20:04:00','suborbital spaceplane','success',1520,'3-5-9','RCS check, M=5','','','Test',379,'NB-52 008',56 from dual union all
select 8483,'1962-M11','1962-06-21 17:47:05','suborbital spaceplane','success',1520,'3-6-10','MH-96 Demo, M=5','','','Test',379,'NB-52 008',75 from dual union all
select 8484,'1962-M12','1962-07-01 00:00:00','military missile','success',1162,'','Eng-Service Test','','','OT?',1895,'',50 from dual union all
select 8485,'1962-M13','1962-07-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 8486,'1962-M14','1962-07-20 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 8487,'1962-M15','1962-07-21 00:00:00','suborbital rocket','success',74,'','','','','Test',767,'',50 from dual union all
select 8488,'1962-M16','1962-07-21 00:00:00','suborbital rocket','success',74,'','','','','Test',767,'',50 from dual union all
select 8489,'1962-M17','1962-08-07 22:02:00','suborbital rocket','success',88,'RTG K62-2','Air sample','','','Aeron',920,'',70 from dual union all
select 849,'1949-S17','1949-06-17 11:50:00','suborbital rocket','success',58,'A13','Aerodynamics test','','','Test',1869,'',88 from dual union all
select 8490,'1962-M18','1962-08-14 18:41:35','suborbital spaceplane','success',1520,'3-9-18','Const Theta Entry','','','Test',379,'NB-52 003',59 from dual union all
select 8491,'1962-M19','1962-09-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 8492,'1962-M20','1962-09-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 8493,'1962-M21','1962-09-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 8494,'1962-M22','1962-09-06 14:30:00','suborbital rocket','success',29,'NASA 01.13NA','UV Airglow','','','Aeronomy',1869,'',74 from dual union all
select 8495,'1962-M23','1962-09-28 21:01:00','military missile','failure',794,'503','Mk 5 RV','R&D Cat I','BIG GAME','Test',1649,'',50 from dual union all
select 8496,'1962-M24','1962-10-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 8497,'1962-M25','1962-10-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 8498,'1962-M26','1962-10-11 20:33:00','suborbital rocket','unknown',839,'','','','','Aeron',394,'',67 from dual union all
select 8499,'1962-M27','1962-10-24 00:00:00','military missile','success',1162,'','Tactical Field Fir.','','','Test',1895,'',50 from dual union all
select 85,'1944-A49','1944-12-01 00:00:00','military missile','success',943,'16','','','','Test',764,'',5 from dual union all
select 850,'1949-S18','1949-06-23 23:21:00','suborbital rocket','success',58,'A14','CR/Solar','','','Particles/Solar X/Aeron',1869,'',88 from dual union all
select 8500,'1962-M28','1962-10-24 19:00:00','military missile','failure',871,'ZM-15 (F44)','ZM-15','','','Test',1057,'',50 from dual union all
select 8501,'1962-M30','1962-11-02 00:00:00','military missile','success',1162,'','Tactical Field Fir.','','','Test',1895,'',50 from dual union all
select 8502,'1962-M31','1962-11-17 17:06:17','military missile','failure',932,'A3X-5','','','','Test',251,'',50 from dual union all
select 8503,'1962-M32','1962-11-21 00:00:00','suborbital rocket','success',203,'B45','','','','Aeronomy',313,'',70 from dual union all
select 8504,'1962-M33','1962-11-22 00:00:00','suborbital rocket','success',203,'B41','','','','Aeronomy',313,'',60 from dual union all
select 8505,'1962-M34','1962-11-25 02:01:00','suborbital rocket','failure',668,'K-9M-1','','','','Ionos',839,'',58 from dual union all
select 8506,'1962-M35','1962-12-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 8507,'1962-M36','1962-12-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 8508,'1962-M37','1962-12-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 8509,'1962-M38','1962-12-04 07:19:00','suborbital rocket','success',839,'NASA 10.46GA','GSFC Grenade 26','','','Aeronomy',1821,'',50 from dual union all
select 851,'1949-S19','1949-09-10 00:00:00','military missile','success',969,'II-1','','','','Test',636,'',100 from dual union all
select 8510,'1962-M39','1962-12-06 14:19:04','military missile','failure',932,'A3X-6','','','','Test',251,'',100 from dual union all
select 8511,'1962-M40','1962-12-11 01:36:00','suborbital rocket','failure',35,'NASA 04.80II','CSIRO VLF 2','','','Ionos',1821,'',52 from dual union all
select 8512,'1962-M41','1962-12-19 00:00:00','test rocket','success',1432,'C3','','','','Test',669,'',20 from dual union all
select 8513,'1962-M42','1962-12-21 00:35:15','military missile','failure',794,'426','Mk 5 RV','R&D','','Test',266,'',50 from dual union all
select 8514,'1962-M43','1962-02-06 00:00:00','military missile','success',272,'HIGHBALL 59','HIGHBALL IIM SN 45','','','Target',1895,'',60 from dual union all
select 8515,'1962-M44','1962-03-02 00:00:00','military missile','success',86,'HIGHBALL 60','HIGHBALL IH SN 9','','','Target',1895,'',60 from dual union all
select 8516,'1962-M45','1962-03-20 00:00:00','military missile','success',272,'HIGHBALL 61','HIGHBALL IIK SN 46','','','Target',1895,'',60 from dual union all
select 8517,'1962-M46','1962-03-20 00:00:00','military missile','success',86,'HIGHBALL 62','HIGHBALL IH SN 10','','','Target',1895,'',60 from dual union all
select 8518,'1962-M47','1962-03-21 00:00:00','military missile','success',86,'HIGHBALL 63','HIGHBALL IE SN11','','','Target',1895,'',60 from dual union all
select 8519,'1962-M48','1962-03-22 00:00:00','military missile','success',86,'HIGHBALL 64','HIGHBALL IH SN12','','','Target',1895,'',60 from dual union all
select 852,'1949-S20','1949-09-11 00:00:00','military missile','success',969,'II-2','','','','Test',636,'',100 from dual union all
select 8520,'1962-M49','1962-03-26 00:00:00','military missile','success',86,'HIGHBALL 65','HIGHBALL IH SN13','','','Target',1895,'',60 from dual union all
select 8521,'1962-M50','1962-03-26 00:00:00','military missile','success',272,'HIGHBALL 66','HIGHBALL IIM SN47','','','Target',1895,'',60 from dual union all
select 8522,'1962-M51','1962-03-28 00:00:00','military missile','success',86,'HIGHBALL 67','HIGHBALL IH SN14','','','Target',1895,'',60 from dual union all
select 8523,'1962-M52','1962-03-01 00:00:00','suborbital rocket','unknown',88,'','Robin','','','Aeron',394,'',68 from dual union all
select 8524,'1962-M53','1962-03-01 00:00:00','suborbital rocket','unknown',88,'','Robin','','','Aeron',394,'',68 from dual union all
select 8525,'1962-S01','1962-01-01 00:00:00','military missile','success',973,'','MV parachute tests','','','Test',636,'',86 from dual union all
select 8526,'1962-S02','1962-01-01 00:00:00','military missile','success',973,'','','','','Test',636,'',86 from dual;
insert into LAUNCH
select 8527,'1962-S03','1962-01-01 00:00:00','military missile','success',973,'','','','','Test',636,'',86 from dual union all
select 8528,'1962-S04','1962-01-01 00:00:00','military missile','success',973,'','','','','Test',636,'',86 from dual union all
select 8529,'1962-S05','1962-01-01 00:00:00','military missile','success',973,'','','','','Test',636,'',86 from dual union all
select 853,'1949-S21','1949-09-13 00:00:00','military missile','success',969,'II-11','','','','Test',636,'',100 from dual union all
select 8530,'1962-S06','1961-12-09 19:00:00','military missile','success',871,'ZM-5','Low altitude test','','','Test',1057,'',40 from dual union all
select 8531,'1962-S07','1962-01-18 00:00:00','military missile','success',872,'ZW-35','','','','Test',1895,'',50 from dual union all
select 8532,'1962-S08','1962-01-04 11:15:00','military missile','success',981,'E 225   02-12T','8K64 UBP-2','','','Test',566,'',1210 from dual union all
select 8533,'1962-S09','1962-01-05 15:00:02','military missile','success',794,'407','Mk 5 RV','R&D','','Test',264,'',900 from dual union all
select 8534,'1962-S10','1962-01-12 00:00:00','military missile','success',980,'','8K65U LKI-1 NPU','','','Test',636,'',675 from dual union all
select 8535,'1962-S100','1962-04-17 09:28:00','suborbital rocket','success',839,'NASA 10.42GA','GSFC Grenade 20','','','Aeronomy',1821,'',119 from dual union all
select 8536,'1962-S101','1962-04-17 09:43:00','suborbital rocket','success',838,'NASA 03.20CA','GCA Release 21 Na','','','Aeronomy',1821,'',201 from dual union all
select 8537,'1962-S102','1962-04-17 17:20:00','suborbital rocket','success',30,'NA3.114F','','','','Aeron',1895,'',286 from dual union all
select 8538,'1962-S103','1962-04-18 19:29:00','military missile','success',1018,'E 15001-20','8K75 LKI 2-3','','','OT',569,'',1160 from dual union all
select 8539,'1962-S104','1962-04-18 18:17:54','military missile','failure',638,'CM-114','CTL','','','OT',248,'',500 from dual union all
select 854,'1949-S22','1949-09-14 00:00:00','military missile','success',969,'II-4','','','','Test',636,'',100 from dual union all
select 8540,'1962-S105','1962-04-18 19:32:00','suborbital rocket','success',130,'CRL AA15.285','Mid-latitude B field','','','Ionos',394,'',221 from dual union all
select 8541,'1962-S106','1962-04-23 19:30:00','military missile','success',1018,'E 15001-18','8K75 LKI 2-5','','','OT',569,'',1160 from dual union all
select 8542,'1962-S107','1962-04-25 16:50:26','military missile','success',929,'A2PE-1','','','','Test',421,'SSBN 609',1000 from dual union all
select 8543,'1962-S108','1962-04-25 14:00:34','test rocket','success',1118,'SA-2','','','','Test',267,'',145 from dual union all
select 8544,'1962-S109','1962-04-26 00:00:00','military missile','success',976,'','8K63 UBP-183','','','Test',636,'',402 from dual union all
select 8545,'1962-S11','1962-01-11 00:00:00','military missile','success',979,'','8K65 UBP-17 68 RB','','','Test',636,'',675 from dual union all
select 8546,'1962-S110','1962-04-26 16:00:00','suborbital rocket','success',839,'NASA 10.110GI','Japan/US Ionos-1','','','Ionos',1821,'',128 from dual union all
select 8547,'1962-S111','1962-04-27 17:24:45','military missile','success',916,'329','P-35','','','Test',256,'',180 from dual union all
select 8548,'1962-S112','1962-04-27 00:00:00','military missile','success',871,'ZW-43','','','','Test',1884,'',50 from dual union all
select 8549,'1962-S113','1962-04-27 23:24:00','military missile','success',154,'140D','Mk 3-3B','OT DASO','BLUE BALL','OT',1666,'',1800 from dual union all
select 855,'1949-S23','1949-09-17 00:00:00','military missile','success',969,'II-8','','','','Test',636,'',100 from dual union all
select 8550,'1962-S114','1962-04-28 00:00:00','military missile','success',1035,'','','','','Test',636,'',500 from dual union all
select 8551,'1962-S115','1962-05-01 00:00:00','suborbital rocket','success',740,'D-75M LKI','','','','Aeron',636,'',150 from dual union all
select 8552,'1962-S116','1962-04-30 00:00:00','military missile','success',871,'ZW-44','Sim target','','','Test',1884,'',50 from dual union all
select 8553,'1962-S117','1962-04-30 18:35:00','suborbital rocket','success',130,'CRL AA15.196','','','','Ionos',394,'',255 from dual union all
select 8554,'1962-S118','1962-05-01 13:13:00','suborbital rocket','success',239,'BK.15','H16','','','Reentry Test/Ionos',1844,'',795 from dual union all
select 8555,'1962-S119','1962-05-01 19:18:00','suborbital rocket','success',29,'AA1.193','Gyro interaction','','','Ionos?',394,'',94 from dual union all
select 8556,'1962-S12','1962-01-12 22:00:00','military missile','success',982,'6LU','8K64U LKI-6','','','Test',566,'',1210 from dual union all
select 8557,'1962-S120','1962-05-02 22:53:00','suborbital rocket','success',30,'AA3.361','Day airglow','','','Aeron',1869,'',275 from dual union all
select 8558,'1962-S121','1962-05-02 23:44:53','military missile','success',1370,'177','TIGERFISH','','TIGERFISH','Test',772,'',500 from dual union all
select 8559,'1962-S122','1962-05-04 01:41:02','military missile','success',929,'A2X-18 (A2MG-1)','Mk 2 guid./NPRB','','','R&D',244,'',1000 from dual union all
select 856,'1949-S24','1949-09-19 00:00:00','military missile','success',969,'II-5','','','','Test',636,'',100 from dual union all
select 8560,'1962-S123','1962-05-03 18:03:00','suborbital rocket','success',607,'NASA 5.04GA','Structure','','','Aeronomy',1797,'',113 from dual union all
select 8561,'1962-S124','1962-05-03 20:00:00','suborbital rocket','success',624,'NASA 8.21GI','ELF electron trap','','','Ionos',1821,'',845 from dual union all
select 8562,'1962-S125','1962-05-04 21:43:00','military missile','success',1419,'SM-34','Mk 4 RV','R&D Cat II','SILVER TOP','Test',1654,'',1000 from dual union all
select 8563,'1962-S126','1962-05-06 23:17:49','military missile','success',928,'A1P','FRIGATE BIRD','','','Nuclear Test',367,'SSBN 608',500 from dual union all
select 8564,'1962-S127','1962-05-08 15:05:00','suborbital rocket','success',30,'NASA 04.46NP','JPL Rocket Radar 1','','','Tech',1869,'',196 from dual union all
select 8565,'1962-S128','1962-05-11 15:00:02','military missile','success',794,'413','Mk 5 RV','R&D','','Test',266,'',900 from dual union all
select 8566,'1962-S129','1962-05-11 18:26:04','military missile','success',929,'A2PE-4','','','','OT',421,'SSBN 609',1000 from dual union all
select 8567,'1962-S13','1962-01-13 16:15:00','suborbital rocket','success',35,'NASA 04.68GT','Attitude control','','','Test',1821,'',209 from dual union all
select 8568,'1962-S130','1962-05-12 00:31:48','military missile','success',154,'127D','Mk 3-3B','OT DASO','CANNONBALL FLYER','OT',1667,'',1800 from dual union all
select 8569,'1962-S131','1962-05-16 17:03:00','suborbital rocket','success',839,'NASA 10.112GI','Japan/US Ionos-2','','','Ionos',1821,'',121 from dual union all
select 857,'1949-S25','1949-09-20 00:00:00','military missile','failure',969,'II-9','','','','Test',636,'',0 from dual union all
select 8570,'1962-S132','1962-05-17 00:00:00','military missile','success',976,'','8K63 UBP-184','','','Test',636,'',402 from dual union all
select 8571,'1962-S133','1962-05-17 02:04:00','suborbital rocket','success',839,'NASA 10.111GI','Japan/US Ionos-3','','','Ionos',1821,'',127 from dual union all
select 8572,'1962-S134','1962-05-19 01:59:59','military missile','success',794,'414','Mk 5 RV','R&D','','Test',266,'',900 from dual union all
select 8573,'1962-S135','1962-05-18 00:00:00','suborbital rocket','success',287,'C10','','','','Test',313,'',130 from dual union all
select 8574,'1962-S136','1962-05-18 18:01:00','suborbital rocket','success',839,'NASA 10.91UA','Composition','','','Aeronomy',1821,'',135 from dual union all
select 8575,'1962-S137','1962-05-21 00:00:00','military missile','success',994,'','NIIP-A target','','','Target',634,'',100 from dual union all
select 8576,'1962-S138','1962-05-22 00:00:00','military missile','success',976,'','8K63 UBP-185','','','Test',636,'',402 from dual union all
select 8577,'1962-S139','1962-05-23 18:01:36','military missile','success',929,'A2PE-5','','','','OT',421,'SSBN 610',1000 from dual union all
select 8578,'1962-S14','1962-01-15 11:07:01','ballistic missile test','success',1369,'337','AVT-1','','','Sat Test',233,'',1505 from dual union all
select 8579,'1962-S140','1962-05-23 19:48:00','military missile','success',929,'A2PE-6','','','','OT',421,'SSBN 610',1000 from dual union all
select 858,'1949-S26','1949-09-23 00:00:00','military missile','failure',969,'II-15','','','','Test',636,'',0 from dual union all
select 8580,'1962-S141','1962-05-24 19:40:00','suborbital rocket','success',1490,'AGI39','','','','Aeronomy',668,'',168 from dual union all
select 8581,'1962-S142','1962-05-25 12:43:00','suborbital rocket','success',35,'NASA 04.48GT','Sea recovery test','','','Test/Ionos',1821,'',200 from dual union all
select 8582,'1962-S143','1962-05-26 18:04:00','military missile','success',871,'ZM-13','ZM-13 Altitude test','','','Test',1057,'',100 from dual union all
select 8583,'1962-S144','1962-05-27 00:00:00','military missile','success',976,'','8K63 UBP-186','','','Test',636,'',402 from dual union all
select 8584,'1962-S145','1962-05-29 00:00:00','suborbital rocket','success',287,'C12','','','','Aeronomy',313,'',130 from dual union all
select 8585,'1962-S146','1962-05-29 00:00:00','suborbital rocket','success',287,'C15','','','','Aeronomy',1314,'',130 from dual union all
select 8586,'1962-S147','1962-05-29 00:00:00','suborbital rocket','success',287,'C18','','','','Aeronomy',664,'',130 from dual union all
select 8587,'1962-S148','1962-05-30 00:00:00','military missile','success',976,'','8K63 UBP-187','','','Test',636,'',402 from dual union all
select 8588,'1962-S149','1962-06-26 00:00:00','military missile','success',1053,'CC-1014','','','','OT',437,'',90 from dual union all
select 8589,'1962-S15','1962-01-16 00:00:00','suborbital rocket','success',30,'','EUV Mon/XR counter','','','Solar EUV',1869,'',200 from dual union all
select 859,'1949-S27','1949-09-25 11:16:00','military missile','success',996,'1','2RE ZI','','','Test',636,'',100 from dual union all
select 8590,'1962-S150','1962-05-31 00:00:00','suborbital rocket','success',1490,'AGI38','','','','Aeronomy',668,'',100 from dual union all
select 8591,'1962-S151','1962-05-31 03:01:00','suborbital rocket','success',720,'LT14','Weather imaging','','','Aeron',1837,'',137 from dual union all
select 8592,'1962-S152','1962-05-31 17:08:44','test rocket','success',249,'102','ERCS/279L 1','','','Comms',1015,'',830 from dual union all
select 8593,'1962-S153','1962-06-01 00:00:00','suborbital rocket','success',839,'FB-1','','','','Aeronomy',803,'',100 from dual union all
select 8594,'1962-S154','1962-06-01 02:10:50','military missile','success',982,'T 225   02-20T','8K64U K 2 (Kontrol''niy)','','','Test',566,'',1210 from dual union all
select 8595,'1962-S155','1962-06-01 00:00:00','suborbital rocket','success',1490,'AGI41','','','','Aeronomy',668,'',100 from dual union all
select 8596,'1962-S156','1962-06-02 19:16:01','military missile','success',929,'A2PE-7','','','','OT',421,'SSBN 610',1000 from dual union all
select 8597,'1962-S157','1962-06-02 22:34:57','military missile','success',929,'A2PE-8','','','','OT',421,'SSBN 610',1000 from dual union all
select 8598,'1962-S158','1962-06-05 04:06:21','military missile','success',916,'339','P-37','','','Test',256,'',180 from dual union all
select 8599,'1962-S159','1962-06-04 00:00:00','military missile','success',979,'','8K65 UBP-21 43 RD','','','Test',636,'',675 from dual union all
select 86,'1944-A50','1944-12-01 00:00:00','military missile','success',943,'17','','','','Test',764,'',5 from dual union all
select 860,'1949-S28','1949-09-28 00:00:00','military missile','success',969,'II-10','','','','Test',636,'',100 from dual union all
select 8600,'1962-S16','1962-01-16 18:59:59','military missile','success',916,'334','P-29','','','Test',256,'',180 from dual union all
select 8601,'1962-S160','1962-06-04 00:00:00','suborbital rocket','failure',1490,'AGI42','','','','Aeronomy',1314,'',100 from dual union all
select 8602,'1962-S161','1962-06-04 09:44:17','military missile','success',1370,'199','BLUEGILL','','BLUEGILL','Nuclear Test',772,'',500 from dual union all
select 8603,'1962-S162','1962-06-05 00:00:00','suborbital rocket','success',287,'C14','','','','Aeronomy',313,'',130 from dual union all
select 8604,'1962-S163','1962-06-05 00:00:00','suborbital rocket','success',287,'C16','','','','Aeronomy',1314,'',130 from dual union all
select 8605,'1962-S164','1962-06-05 00:00:00','suborbital rocket','success',287,'C17','','','','Aeronomy',1314,'',130 from dual union all
select 8606,'1962-S165','1962-06-05 00:00:00','suborbital rocket','success',287,'C19','','','','Aeronomy',664,'',130 from dual union all
select 8607,'1962-S166','1962-06-05 00:00:00','suborbital rocket','success',287,'C09','','','','Aeronomy',664,'',130 from dual union all
select 8608,'1962-S167','1962-06-05 12:45:00','suborbital rocket','success',30,'AB3.133','EUV Monochromator','','','Aeron/Solar EUV',1869,'',216 from dual union all
select 8609,'1962-S168','1962-06-06 00:00:00','military missile','success',979,'','8K65 UBP-22','','','Test',636,'',675 from dual union all
select 861,'1949-S29','1949-09-30 11:49:00','military missile','success',996,'2','2RE ZI','','','Test',636,'',100 from dual union all
select 8610,'1962-S169','1962-06-06 19:47:00','suborbital rocket','success',1490,'AGI43','','','','Aeronomy',1314,'',100 from dual union all
select 8611,'1962-S17','1962-01-16 12:05:00','military missile','success',982,'7LU','8K64U LKI-7','','','Test',566,'',1210 from dual union all
select 8612,'1962-S170','1962-06-06 03:00:00','suborbital rocket','success',1197,'SL35C','RAE Test 14','','','Test',1839,'',240 from dual union all
select 8613,'1962-S171','1962-06-06 23:40:00','suborbital rocket','success',837,'NASA 14.19UA','UM Pitot 1','','','Aeronomy',1821,'',125 from dual union all
select 8614,'1962-S172','1962-06-07 00:00:00','military missile','success',976,'','8K63 UBP-188','','','Test',636,'',402 from dual union all
select 8615,'1962-S173','1962-06-07 16:00:00','military missile','success',982,'T 225   03-20T','8K64U K 3 (Kontrol''niy)','','','Test',566,'',1210 from dual union all
select 8616,'1962-S174','1962-06-07 18:21:37','military missile','success',1420,'N-1','Mk 6 RV     [DST]','','','Test',230,'',1300 from dual;
insert into LAUNCH
select 8617,'1962-S175','1962-06-07 00:05:00','suborbital rocket','success',839,'NASA 10.43GA','GSFC Gren-Sphere 4','','','Aeronomy',1821,'',120 from dual union all
select 8618,'1962-S176','1962-06-07 00:56:00','suborbital rocket','success',838,'NASA 03.21CA','GCA Release 22 Na','','','Aeronomy',1821,'',162 from dual union all
select 8619,'1962-S177','1962-06-07 14:45:00','suborbital rocket','success',839,'Rehbar 1','Na release','','','Aeronomy',1419,'',126 from dual union all
select 862,'1949-S30','1949-10-02 11:00:00','military missile','failure',996,'3','2RE ZI','','','Test',636,'',100 from dual union all
select 8620,'1962-S178','1962-06-08 17:35:01','military missile','success',794,'415','Mk 5 RV','R&D','','Test',266,'',900 from dual union all
select 8621,'1962-S179','1962-06-08 00:00:00','military missile','success',979,'','8K65 UBP-23','','','Test',636,'',675 from dual union all
select 8622,'1962-S18','1962-01-17 12:00:00','military missile','success',982,'8LU','8K64U LKI-8','','','Test',566,'',1210 from dual union all
select 8623,'1962-S180','1962-06-08 00:53:00','suborbital rocket','success',839,'NASA 10.44GA','GSFC Grenade 21','','','Aeronomy',1821,'',117 from dual union all
select 8624,'1962-S181','1962-06-09 02:59:39','military missile','success',1018,'E 103   29','8K75 LKI 2-6','','','OT',569,'',1160 from dual union all
select 8625,'1962-S182','1962-06-11 14:50:00','suborbital rocket','success',839,'Rehbar 2','Na release','','','Aeronomy',1419,'',116 from dual union all
select 8626,'1962-S183','1962-06-12 21:30:00','suborbital rocket','success',30,'AB3.134','EUV Monochromator','','','Solar EUV',1869,'',299 from dual union all
select 8627,'1962-S184','1962-06-14 04:30:00','military missile','success',1018,'E 103   28','8K75 LKI 2-7','','','OT',569,'',1160 from dual union all
select 8628,'1962-S185','1962-06-15 20:14:00','suborbital rocket','failure',217,'AA-3-1','BB3 01','','','Test',1821,'',98 from dual union all
select 8629,'1962-S186','1962-06-15 23:01:00','suborbital rocket','failure',217,'AA-3-2','BB3 02','','','Test',1821,'',92 from dual union all
select 863,'1949-S31','1949-10-03 00:00:00','military missile','success',969,'II-14','','','','Test',636,'',100 from dual union all
select 8630,'1962-S187','1962-06-15 00:00:00','military missile','success',976,'','8K63 UBP-189','','','Test',636,'',402 from dual union all
select 8631,'1962-S188','1962-06-15 16:48:00','suborbital rocket','success',837,'NASA 14.12GI','','','','Ionos',1821,'',143 from dual union all
select 8632,'1962-S189','1962-06-19 00:00:00','suborbital rocket','failure',217,'AA-3-3','BB3 03','','','Test',1821,'',100 from dual union all
select 8633,'1962-S19','1962-01-17 21:02:26','military missile','success',154,'123D','Mk 3','OT','BLUE FIN','OT',1666,'',1800 from dual union all
select 8634,'1962-S190','1962-06-19 00:00:00','suborbital rocket','success',839,'FB-2','','','','Aeronomy',803,'',100 from dual union all
select 8635,'1962-S191','1962-06-19 00:00:00','military missile','success',976,'','8K63 UBP-190','','','Test',636,'',402 from dual union all
select 8636,'1962-S192','1962-06-19 08:40:35','military missile','success',1018,'E 15001-27','8K75 LKI 2-8','8K75N','','OT',583,'',1160 from dual union all
select 8637,'1962-S193','1962-06-19 00:30:00','military missile','success',1367,'269','CTL','','BLAZING CINDERS','OT',1696,'',520 from dual union all
select 8638,'1962-S194','1962-06-19 06:59:00','suborbital rocket','success',30,'AB3.352','AS&E-3  (Sco X-1)','','','XR Astron',1869,'',224 from dual union all
select 8639,'1962-S195','1962-06-20 05:20:00','suborbital rocket','success',1197,'SL45','LUX-7/UCL SUV-2GS-2I','','','Aeron/Solar X',1839,'',227 from dual union all
select 864,'1949-S32','1949-10-08 00:00:00','military missile','success',969,'II-16','','','','Test',636,'',100 from dual union all
select 8640,'1962-S196','1962-06-20 13:29:00','suborbital rocket','success',35,'NASA 04.26NP','LeRC LH2 test','','','Tech',1821,'',158 from dual union all
select 8641,'1962-S197','1962-06-21 03:27:00','military missile','success',1018,'E 15001-26','8K75 LKI 2-9','8K75N','','OT',583,'',1160 from dual union all
select 8642,'1962-S198','1962-06-22 00:00:00','military missile','success',976,'','8K63 UBP-191','','','Test',636,'',402 from dual union all
select 8643,'1962-S199','1962-06-26 00:00:00','military missile','success',976,'','8K63 UBP-192','','','Test',636,'',402 from dual union all
select 8644,'1962-S20','1962-01-21 00:57:00','military missile','failure',1419,'SM-68-4','Mk 4-5B RV','SAC Demo','DOUBLE MARTINI','Test',1656,'',100 from dual union all
select 8645,'1962-S200','1962-06-26 08:38:00','suborbital rocket','success',535,'HAD 108','Sphere','','','Aeron',1835,'',116 from dual union all
select 8646,'1962-S201','1962-06-26 10:57:42','military missile','success',154,'21D','Nike TV-33','NTMP K-1','ALL JAZZ','Target',1667,'',1800 from dual union all
select 8647,'1962-S202','1962-06-27 00:00:00','military missile','success',976,'','8K63 UBP-193','','','Test',636,'',402 from dual union all
select 8648,'1962-S203','1962-06-27 00:00:00','military missile','success',206,'001','','','','Reentry Test',313,'',270 from dual union all
select 8649,'1962-S204','1962-06-27 14:10:00','suborbital rocket','unknown',838,'','','','','Ionos',1821,'',200 from dual union all
select 865,'1949-S33','1949-10-08 06:05:00','military missile','success',996,'4','2RE ZI','','','Test',636,'',100 from dual union all
select 8650,'1962-S205','1962-06-28 00:00:00','military missile','success',976,'','8K63 UBP-194','','','Test',636,'',402 from dual union all
select 8651,'1962-S206','1962-06-28 00:10:00','suborbital rocket','unknown',838,'','','','','Ionos',1821,'',200 from dual union all
select 8652,'1962-S207','1962-06-29 05:40:00','military missile','success',794,'416','Mk 5 RV','R&D','','Test',266,'',900 from dual union all
select 8653,'1962-S208','1962-06-29 14:05:58','military missile','success',929,'A2X-29 (A2MG-4)','Mk 2 guid./NPRB','','','Test',244,'',1000 from dual union all
select 8654,'1962-S209','1962-06-29 00:00:00','military missile','success',976,'','8K63 UBP-195','','','Test',636,'',402 from dual union all
select 8655,'1962-S21','1962-01-22 00:00:00','military missile','success',979,'','','','','Test',636,'',675 from dual union all
select 8656,'1962-S210','1962-06-29 21:00:00','suborbital rocket','success',35,'NASA 04.72UP','JHU Airglow 2','','','Aeronomy',1821,'',210 from dual union all
select 8657,'1962-S211','1962-07-01 00:00:00','military missile','success',979,'','8K65 UBP-24','','','Test',636,'',675 from dual union all
select 8658,'1962-S212','1962-07-01 00:00:00','military missile','success',979,'','8K65 UBP-25','','','Test',636,'',675 from dual union all
select 8659,'1962-S213','1962-07-01 00:00:00','military missile','success',979,'','8K65 UBP-26','','','Test',636,'',675 from dual union all
select 866,'1949-S34','1949-10-10 00:00:00','military missile','success',969,'II-12','','','','Test',636,'',100 from dual union all
select 8660,'1962-S214','1962-07-17 00:00:00','military missile','success',1053,'CC-1017','','','','OT 40AG Bat A',437,'',90 from dual union all
select 8661,'1962-S215','1962-07-01 00:00:00','military missile','success',206,'003','','','','Reentry Test',313,'',270 from dual union all
select 8662,'1962-S216','1962-06-29 04:55:00','suborbital rocket','success',35,'NASA 04.71UA/UP','JHU Airglow 1','','','Aeronomy',1821,'',208 from dual union all
select 8663,'1962-S217','1962-07-01 00:00:00','suborbital rocket','success',839,'','7-inch Sphere','','','Aeronomy',803,'',100 from dual union all
select 8664,'1962-S218','1962-07-02 16:36:00','military missile','success',1017,'T 15001- 03','8K74 UBP-11','','','OT',562,'',1350 from dual union all
select 8665,'1962-S219','1962-07-04 00:00:00','military missile','success',976,'','8K63 PRO Target','SSATC Target','','Test',636,'',402 from dual union all
select 8666,'1962-S22','1962-01-23 21:28:27','military missile','success',154,'132D','Mk 3-3B','OT','BLUE MOSS','OT',1667,'',1800 from dual union all
select 8667,'1962-S220','1962-07-04 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8668,'1962-S221','1962-07-05 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8669,'1962-S222','1962-07-05 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 867,'1949-S35','1949-10-11 12:45:00','military missile','failure',996,'5','2RE ZI','','','Test',636,'',10 from dual union all
select 8670,'1962-S223','1962-07-05 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8671,'1962-S224','1962-07-05 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8672,'1962-S225','1962-07-05 03:46:00','suborbital rocket','success',1196,'SL164','UCL GC-21','','','Aeronomy',1839,'',111 from dual union all
select 8673,'1962-S226','1962-07-06 00:00:00','military missile','success',206,'002','','','','Reentry Test',313,'',270 from dual union all
select 8674,'1962-S227','1962-07-08 00:00:00','suborbital rocket','success',839,'FB-3','7-inch Sphere','','','Aeronomy',803,'',100 from dual union all
select 8675,'1962-S228','1962-07-09 08:50:00','suborbital rocket','success',624,'Project 6.4/J1','STARFISH Diag','','','Ionos',784,'',325 from dual union all
select 8676,'1962-S229','1962-07-09 09:07:00','suborbital rocket','success',624,'Project 6.4/J2','STARFISH Diag.','','','Ionos',788,'',500 from dual union all
select 8677,'1962-S23','1962-01-25 03:26:00','military missile','success',916,'332','P-30','','','Test',256,'',180 from dual union all
select 8678,'1962-S230','1962-07-09 06:15:00','suborbital rocket','success',839,'Project 9.1 Na','STARFISH Na','','','Aeronomy',787,'',137 from dual union all
select 8679,'1962-S231','1962-07-09 08:30:00','suborbital rocket','success',839,'Project 9.1 Na','STARFISH Na','','','Aeronomy',786,'',137 from dual union all
select 868,'1949-S36','1949-10-12 00:00:00','military missile','success',969,'II-7','','','','Test',636,'',100 from dual union all
select 8680,'1962-S232','1962-07-09 08:59:00','suborbital rocket','success',839,'Project 9.1 Sphere','STARFISH Mass spec','','','Aeronomy',781,'',118 from dual union all
select 8681,'1962-S233','1962-07-09 09:08:00','suborbital rocket','success',839,'Project 9.1 Sphere','STARFISH Mass spec','','','Aeronomy',780,'',118 from dual union all
select 8682,'1962-S234','1962-07-09 08:52:09','suborbital rocket','failure',630,'','STARFISH Diag','','','Magnetospheric?',1015,'',100 from dual union all
select 8683,'1962-S235','1962-07-09 08:46:28','military missile','success',1370,'195','STARFISH PRIME','','STARFISH PRIME','Nuclear Test',772,'',400 from dual union all
select 8684,'1962-S236','1962-07-09 09:07:00','suborbital rocket','success',545,'Project 6.3/X-4','STARFISH Rad X-4','','','Ionos',783,'',89 from dual union all
select 8685,'1962-S237','1962-07-10 15:00:00','suborbital rocket','success',30,'NASA 04.47NP','JPL Rocket Radar 2','','','Tech',1869,'',200 from dual union all
select 8686,'1962-S238','1962-07-11 18:51:54','military missile','success',1420,'N-6','Mk 6 RV','','','Test',230,'',1300 from dual union all
select 8687,'1962-S239','1962-07-12 18:31:57','military missile','success',929,'A2PE-11','','','','OT',421,'SSBN 611',1000 from dual union all
select 8688,'1962-S24','1962-01-25 14:59:48','military missile','success',794,'408','Mk 5 RV','R&D','','Test',264,'',900 from dual union all
select 8689,'1962-S240','1962-07-12 20:05:13','military missile','success',929,'A2PE-10','+80 min','','','OT',421,'SSBN 611',1000 from dual union all
select 869,'1949-S37','1949-10-13 00:00:00','military missile','success',969,'II-13','','','','Test',636,'',100 from dual union all
select 8690,'1962-S241','1962-07-12 16:57:57','military missile','success',154,'141D','Mk 3-3B','OT DASO','LONG LADY','Target',1666,'',1800 from dual union all
select 8691,'1962-S242','1962-07-13 21:11:33','military missile','failure',155,'67E','Mk 4-5B','R&D Cat II','EXTRA BONUS','Test',1715,'',1800 from dual union all
select 8692,'1962-S243','1962-07-16 00:00:00','military missile','success',871,'ZW-46','Requal test','','','Test',1884,'',50 from dual union all
select 8693,'1962-S244','1962-07-17 17:31:10','suborbital spaceplane','success',1520,'3-7-14','MH-96 Demo','','','Test',379,'NB-52 003',95 from dual union all
select 8694,'1962-S245','1962-07-18 09:30:12','ballistic missile test','success',1369,'338','AVT 2','','','Sat Test',233,'',1484 from dual union all
select 8695,'1962-S246','1962-07-19 08:40:00','suborbital rocket','success',839,'FB-5','7-inch Sphere','','','Aeronomy',803,'',100 from dual union all
select 8696,'1962-S247','1962-07-19 11:05:27','military missile','success',154,'13D','Nike TV-33','NTMP K-2','FIRST TRY','Test',1665,'',1800 from dual union all
select 8697,'1962-S248','1962-07-19 11:35:00','military missile','success',871,'ZK-7  20071','K-2 Atlas Intercept','','','Test',873,'',70 from dual union all
select 8698,'1962-S249','1962-07-21 20:49:54','military missile','success',929,'A2P-12','','','','OT',421,'SSBN 611',1000 from dual union all
select 8699,'1962-S25','1962-01-25 13:00:00','military missile','success',982,'9LU','8K64U LKI-9','','','Test',566,'',1210 from dual union all
select 87,'1944-A51','1944-12-01 00:00:00','military missile','success',943,'18','No booster','','','Test',764,'',1 from dual union all
select 870,'1949-S38','1949-10-13 00:00:00','military missile','success',969,'II-17','','','','Test',636,'',100 from dual union all
select 8700,'1962-S250','1962-07-21 22:19:15','military missile','success',929,'A2P-13','','','','OT',421,'SSBN 611',1000 from dual union all
select 8701,'1962-S251','1962-07-21 23:49:14','military missile','success',929,'A2P-2','','','','OT',421,'SSBN 611',1000 from dual union all
select 8702,'1962-S252','1962-07-21 04:21:00','military missile','success',982,'T 225   04-20T','8K64U K 4','','','Test',566,'',1210 from dual union all
select 8703,'1962-S253','1962-07-21 17:01:00','military missile','success',1018,'T 103   30','8K75 LKI 2-10','','','OT',583,'',1160 from dual union all
select 8704,'1962-S254','1962-07-23 00:00:00','suborbital rocket','success',624,'','','','','Aeron',803,'',500 from dual union all
select 8705,'1962-S255','1962-07-24 17:05:00','military missile','success',1018,'T 103   31','8K75 LKI 2-11','','','OT',583,'',1160 from dual union all
select 8706,'1962-S256','1962-07-24 12:09:00','suborbital rocket','unknown',35,'','','','','Aeron',394,'',100 from dual;
insert into LAUNCH
select 8707,'1962-S257','1962-07-24 17:29:12','test rocket','success',249,'101','ERCS/279L 2','','','Comms',1015,'',1000 from dual union all
select 8708,'1962-S258','1962-07-24 21:41:00','suborbital rocket','success',30,'NASA 04.23US','Ly alpha test','','','Solar UV',1821,'',208 from dual union all
select 8709,'1962-S259','1962-07-23 00:00:00','suborbital rocket','success',839,'FB-4','Sphere/Mass spec','','','Aeronomy',803,'',100 from dual union all
select 871,'1949-S39','1949-10-15 00:00:00','military missile','success',969,'II-19','','','','Test',636,'',100 from dual union all
select 8710,'1962-S26','1962-01-25 22:02:00','military missile','success',872,'ZM-7','PM-7 Sim target','','','Test',1057,'',80 from dual union all
select 8711,'1962-S260','1962-07-25 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8712,'1962-S261','1962-07-25 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8713,'1962-S262','1962-07-25 15:41:00','suborbital rocket','success',274,'NC17.117','','','','Test/Aeron',1023,'F-4H',1166 from dual union all
select 8714,'1962-S263','1962-07-25 16:17:20','military missile','success',1420,'N-4','Mk 6 RV','','','Test',231,'',1300 from dual union all
select 8715,'1962-S264','1962-07-26 17:50:02','military missile','success',929,'A2TF-1','','','','Test',244,'',1000 from dual union all
select 8716,'1962-S265','1962-07-27 00:00:00','military missile','success',928,'A1P-1','ORRT','','','OT',421,'SSBN 598',500 from dual union all
select 8717,'1962-S266','1962-07-27 00:00:00','military missile','success',928,'A1P-2','ORRT','','','OT',421,'SSBN 598',500 from dual union all
select 8718,'1962-S267','1962-07-27 00:00:00','military missile','success',928,'A1P-3','ORRT','','','OT',421,'SSBN 598',500 from dual union all
select 8719,'1962-S268','1962-07-27 00:00:00','military missile','success',928,'A1P-4','ORRT','','','OT',421,'SSBN 598',500 from dual union all
select 872,'1949-S40','1949-10-18 00:00:00','military missile','success',969,'II-23','','','','Test',636,'',100 from dual union all
select 8720,'1962-S269','1962-07-27 00:00:00','military missile','success',976,'','8K63 UBP-196','','','Test',636,'',402 from dual union all
select 8721,'1962-S27','1962-01-27 00:00:00','military missile','success',976,'','8K63 UBP-171','','','Test',636,'',402 from dual union all
select 8722,'1962-S270','1962-07-28 02:50:00','suborbital rocket','success',1438,'TB Ik','','','','Reentry Test',1821,'',280 from dual union all
select 8723,'1962-S271','1962-07-30 00:00:00','military missile','success',976,'','8K63 UBP-198','','','Test',636,'',402 from dual union all
select 8724,'1962-S272','1962-07-31 14:10:00','military missile','success',982,'11LU','8K64U LKI-11','','','Test',572,'',1210 from dual union all
select 8725,'1962-S273','1962-08-07 00:00:00','military missile','success',1053,'CC-1007','','','','OT 40AG-Bat A',437,'',90 from dual union all
select 8726,'1962-S274','1962-08-30 00:00:00','military missile','success',1053,'CC-1015','','Bat B','','OT 40AG Bat B',437,'',90 from dual union all
select 8727,'1962-S275','1962-08-01 18:40:40','military missile','success',638,'CM-111','CTL','','','OT',248,'',500 from dual union all
select 8728,'1962-S276','1962-08-01 21:07:10','military missile','success',162,'15F','Mk 4-5B','R&D','HIS NIBS','Test',1670,'',1400 from dual union all
select 8729,'1962-S277','1962-08-03 17:46:00','suborbital rocket','success',470,'','Bipolar Probe','','','Ionos',394,'',365 from dual union all
select 873,'1949-S41','1949-10-19 00:00:00','military missile','success',969,'II-22','','','','Test',636,'',100 from dual union all
select 8730,'1962-S278','1962-08-05 00:00:00','military missile','success',979,'','8K65 UBP-27 29 RD','','','Test',636,'',675 from dual union all
select 8731,'1962-S279','1962-08-05 00:00:00','suborbital rocket','unknown',673,'','','','','Aeron?',1016,'',300 from dual union all
select 8732,'1962-S28','1962-01-27 00:00:00','military missile','success',979,'','','','','Test',636,'',675 from dual union all
select 8733,'1962-S280','1962-08-07 19:20:05','military missile','failure',932,'A3X-1','','','','Test',251,'',200 from dual union all
select 8734,'1962-S281','1962-08-07 00:47:08','suborbital rocket','success',839,'RTG K62-1','Air sample','','','Aeronomy',920,'',105 from dual union all
select 8735,'1962-S282','1962-08-08 00:00:00','military missile','success',976,'','8K63 PRO Target','SSATC Target','','Test',636,'',402 from dual union all
select 8736,'1962-S283','1962-08-08 16:00:00','military missile','success',982,'T 226   01-20T','8K64U K 5','','','Test',566,'',1210 from dual union all
select 8737,'1962-S284','1962-08-08 16:55:00','suborbital rocket','success',30,'NASA 04.60GT','Attitude control','','','Test',1821,'',149 from dual union all
select 8738,'1962-S285','1962-08-09 22:51:08','military missile','success',154,'8D','Mk 3-3D','OT DASO','PEG BOARD','Test',1667,'',1800 from dual union all
select 8739,'1962-S286','1962-08-09 23:05:19','military missile','success',154,'87D','Mk 3-2D','OT DASO','PEG BOARD II','Test',1666,'',1800 from dual union all
select 874,'1949-S42','1949-10-22 00:00:00','military missile','success',969,'II-20','','','','Test',636,'',100 from dual union all
select 8740,'1962-S287','1962-08-10 21:27:00','military missile','success',929,'A2TF-2','PX-1 Penaids','','','Test',244,'',1000 from dual union all
select 8741,'1962-S288','1962-08-10 00:00:00','military missile','success',976,'','8K63 PRO Target','SSATC Target','','Test',636,'',402 from dual union all
select 8742,'1962-S289','1962-08-10 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8743,'1962-S29','1962-01-29 23:30:56','military missile','success',1419,'M-7','Nike TVX-3','NTMP','','Test',237,'',1000 from dual union all
select 8744,'1962-S290','1962-08-10 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8745,'1962-S291','1962-08-10 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8746,'1962-S292','1962-08-10 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8747,'1962-S293','1962-08-11 01:48:11','suborbital rocket','success',839,'RTG K62-3','Air sample','','','Aeronomy',920,'',105 from dual union all
select 8748,'1962-S294','1962-08-13 22:00:00','ballistic missile test','success',162,'7F','Mk 4-1B/Pod 16','R&D','','Test',226,'',1400 from dual union all
select 8749,'1962-S295','1962-08-14 15:20:00','suborbital rocket','success',1194,'SL109','UCL SUV-3I/RSHF-1','','','Ionos',1839,'',111 from dual union all
select 875,'1949-S43','1949-10-23 00:00:00','military missile','success',969,'II-3','','','','Test',636,'',100 from dual union all
select 8750,'1962-S296','1962-08-15 00:00:00','military missile','success',976,'','8K63 UBP-199','','','Test',636,'',402 from dual union all
select 8751,'1962-S297','1962-08-18 07:09:00','suborbital rocket','success',839,'Ferdinand 1','Ferdinand 1','','','Ionos',29,'',100 from dual union all
select 8752,'1962-S298','1962-08-19 00:57:39','suborbital rocket','success',839,'RTG K62-4','Air sample','','','Aeronomy',920,'',109 from dual union all
select 8753,'1962-S299','1962-08-22 01:30:00','military missile','success',916,'338','P-39','','','Test',256,'',180 from dual union all
select 8754,'1962-S30','1962-01-29 20:00:00','suborbital rocket','success',130,'CRL AA15.109','Day Airglow','','','Test/Aeron',394,'',250 from dual union all
select 8755,'1962-S300','1962-08-21 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8756,'1962-S301','1962-08-21 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8757,'1962-S302','1962-08-21 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8758,'1962-S303','1962-08-21 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8759,'1962-S304','1962-08-21 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 876,'1949-S44','1949-11-18 16:03:00','suborbital rocket','success',1470,'56','UM Sampling,Grenades','','','Aeron/CR',1863,'',124 from dual union all
select 8760,'1962-S305','1962-08-21 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8761,'1962-S306','1962-08-22 15:30:00','suborbital rocket','success',30,'NE3.128','','','','Solar EUV/XR',1869,'',234 from dual union all
select 8762,'1962-S307','1962-08-23 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8763,'1962-S308','1962-08-23 07:15:00','suborbital rocket','success',666,'K-8L-1','','','','Test',838,'',173 from dual union all
select 8764,'1962-S309','1962-08-23 17:10:00','suborbital rocket','success',837,'NASA 14.30CA','Structure','','','Aeronomy',1821,'',129 from dual union all
select 8765,'1962-S31','1962-01-30 00:00:00','military missile','success',976,'','8K63 UBP-172','','','Test',636,'',402 from dual union all
select 8766,'1962-S310','1962-08-25 02:07:00','military missile','failure',871,'ZM-14','ZM-14','','','Test',1057,'',50 from dual union all
select 8767,'1962-S311','1962-08-24 11:38:00','suborbital rocket','success',242,'BK.16','H-205','','','Reentry Test/Ionos',1843,'',573 from dual union all
select 8768,'1962-S312','1962-08-27 15:02:03','military missile','success',929,'A2TF-4','','','','Test',244,'',1000 from dual union all
select 8769,'1962-S313','1962-08-27 09:08:00','suborbital rocket','success',535,'HAD 107','Sphere','','','Aeron',1835,'',114 from dual union all
select 877,'1949-S45','1949-12-02 22:20:00','suborbital rocket','success',50,'USAF 1','MX-1011','','','Solar X/Aeron/imaging',640,'',96 from dual union all
select 8770,'1962-S314','1962-08-29 04:50:00','military missile','success',871,'ZW-47 //','','','','Test',1884,'',50 from dual union all
select 8771,'1962-S315','1962-08-28 19:34:00','suborbital rocket','success',839,'AC6.829','EM Absorption','','','Ionos',394,'',148 from dual union all
select 8772,'1962-S316','1962-08-29 01:01:00','suborbital rocket','success',1194,'SL28','RAE Test 15','','','Test',1839,'',109 from dual union all
select 8773,'1962-S317','1962-08-29 19:00:00','suborbital rocket','success',839,'AC6.830','EM Absorption','','','Ionos',394,'',100 from dual union all
select 8774,'1962-S318','1962-09-25 00:00:00','military missile','success',979,'','8K65 UBP-28 29 RD','','','Test',636,'',675 from dual union all
select 8775,'1962-S32','1962-02-01 00:00:00','military missile','success',871,'ZW-36','3 stage control test','','','Test',1884,'',50 from dual union all
select 8776,'1962-S320','1962-08-31 00:56:07','suborbital rocket','success',839,'RTG K62-5','Air sample','','','Aeronomy',920,'',111 from dual union all
select 8777,'1962-S321','1962-09-04 00:00:00','military missile','success',976,'','8K63 PRO Target','SSATC Target','','Test',636,'',402 from dual union all
select 8778,'1962-S322','1962-09-04 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8779,'1962-S323','1962-09-04 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 878,'1949-S46','1949-12-08 19:15:00','suborbital rocket','success',1470,'31','Blossom IVD (Alb 4)','','','Solar X/Ionos',1863,'',127 from dual union all
select 8780,'1962-S324','1962-09-04 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8781,'1962-S325','1962-09-04 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8782,'1962-S326','1962-09-04 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8783,'1962-S327','1962-09-06 16:00:00','military missile','failure',932,'A3X-2','[St 2 fail]','','','Test',251,'',80 from dual union all
select 8784,'1962-S328','1962-09-07 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8785,'1962-S329','1962-09-07 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8786,'1962-S33','1962-02-05 20:58:00','suborbital rocket','success',1197,'SL35B','RAE Test 13','','','Test',1839,'',215 from dual union all
select 8787,'1962-S330','1962-09-11 13:30:00','suborbital rocket','success',1195,'SL44','UCL I-10 Ionos.','','','Iono',1839,'',232 from dual union all
select 8788,'1962-S331','1962-09-11 23:24:00','suborbital rocket','success',30,'AC3.322','Day Airglow','','','Aeron',1869,'',306 from dual union all
select 8789,'1962-S332','1962-09-12 15:50:00','military missile','success',1420,'N-5','Mk 6 RV','','','Test',230,'',1300 from dual union all
select 879,'1950-A01','1950-01-11 00:00:00','atmospheric rocket','success',365,'','F26 Model','','','Test',1821,'',10 from dual union all
select 8790,'1962-S333','1962-09-14 08:09:06','military missile','success',928,'A1T-6','','','','Test',421,'EAG-154',500 from dual union all
select 8791,'1962-S334','1962-09-15 20:37:10','military missile','failure',928,'A1T-5','[No RV sep]','','','Test',421,'EAG-154',500 from dual union all
select 8792,'1962-S335','1962-09-15 00:00:00','military missile','success',976,'','8K63 UBP-200','','','Test',636,'',402 from dual union all
select 8793,'1962-S336','1962-09-19 03:06:22','military missile','success',794,'421','Mk 5 RV','R&D','','Test',264,'',900 from dual union all
select 8794,'1962-S337','1962-09-20 03:45:41','military missile','success',794,'420','Mk 5 RV','R&D','','Test',264,'',900 from dual union all
select 8795,'1962-S338','1962-09-19 00:00:00','military missile','success',976,'','8K63 UBP-201','','','Test',636,'',402 from dual union all
select 8796,'1962-S339','1962-09-19 19:30:00','ballistic missile test','success',162,'8F','Mk 4-1B/Pod 17','R&D','','Test',226,'',1400 from dual union all
select 8797,'1962-S34','1962-02-07 00:00:00','military missile','success',872,'ZW-37','','','','Test',1884,'',50 from dual;
insert into LAUNCH
select 8798,'1962-S340','1962-09-20 00:00:00','military missile','success',976,'','8K63 UBP-202','','','Test',636,'',402 from dual union all
select 8799,'1962-S341','1962-09-20 04:21:00','suborbital rocket','success',1194,'SL108','RSHF-2','','','Ionos',1839,'',113 from dual union all
select 88,'1944-A52','1944-12-01 00:00:00','military missile','success',943,'19','','','','Test',764,'',5 from dual union all
select 880,'1950-A02','1950-01-25 00:00:00','atmospheric rocket','success',365,'','X-3 Model','','','Test',1821,'',10 from dual union all
select 8800,'1962-S342','1962-09-22 02:24:00','suborbital rocket','success',35,'NASA 04.36GG','UV Photometer','','','UV Astron',1821,'',187 from dual union all
select 8801,'1962-S343','1962-09-22 06:45:00','sounding rocket','success',630,'NASA 11.02UR','','','','RF Astron',1821,'',1691 from dual union all
select 8802,'1962-S344','1962-09-27 02:14:56','military missile','success',916,'347','P-40','','','Test',255,'',180 from dual union all
select 8803,'1962-S345','1962-09-27 14:19:18','military missile','success',982,'T 225   04','8K64U K 6','','','Test',566,'',1210 from dual union all
select 8804,'1962-S346','1962-09-27 00:00:00','military missile','success',871,'ZW-49','Sim target','','','Test',1884,'',50 from dual union all
select 8805,'1962-S347','1962-09-27 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8806,'1962-S348','1962-09-27 18:00:00','suborbital rocket','success',30,'AB3.246','Electric structure','','','Ionos',1869,'',260 from dual union all
select 8807,'1962-S349','1962-09-28 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8808,'1962-S35','1962-02-07 11:50:00','suborbital rocket','success',624,'NASA 8.16AI','','','','Ionos',1821,'',960 from dual union all
select 8809,'1962-S350','1962-10-01 00:00:00','suborbital rocket','success',740,'D-75M','','','','Nuc test',183,'',150 from dual union all
select 881,'1950-A03','1950-01-31 00:00:00','atmospheric rocket','success',880,'484-38','Nike 484 35R','','','Test',1863,'',10 from dual union all
select 8810,'1962-S351','1962-10-01 00:00:00','suborbital rocket','success',740,'D-75M','','','','Nuc test',183,'',150 from dual union all
select 8811,'1962-S352','1962-10-01 00:00:00','suborbital rocket','success',740,'D-75M','','','','Nuc test',183,'',150 from dual union all
select 8812,'1962-S353','1962-10-01 00:00:00','suborbital rocket','success',740,'D-75M','','','','Nuc test',183,'',150 from dual union all
select 8813,'1962-S354','1962-10-01 00:00:00','suborbital rocket','success',740,'D-75M','','','','Nuc test',183,'',150 from dual union all
select 8814,'1962-S355','1962-10-01 00:00:00','suborbital rocket','success',740,'D-75M','','','','Nuc test',183,'',150 from dual union all
select 8815,'1962-S356','1962-10-01 00:00:00','suborbital rocket','success',740,'D-75M','','','','Nuc test',183,'',150 from dual union all
select 8816,'1962-S357','1962-10-01 00:00:00','suborbital rocket','success',740,'D-75M','','','','Nuc test',183,'',150 from dual union all
select 8817,'1962-S358','1962-10-01 00:00:00','military missile','success',929,'A2','','','','OT',1897,'SSBN 608',1000 from dual union all
select 8818,'1962-S359','1962-10-01 00:00:00','military missile','success',929,'A2','','','','OT',1897,'SSBN 608',1000 from dual union all
select 8819,'1962-S36','1962-02-11 00:00:00','military missile','success',980,'','8K65U LKI-1 ShPU','','','Test',635,'',675 from dual union all
select 882,'1950-A04','1950-02-02 00:00:00','atmospheric rocket','success',880,'484-37','Nike 484 36R','','','Test',1895,'',10 from dual union all
select 8820,'1962-S360','1962-10-01 00:00:00','military missile','success',929,'A2','','','','OT',1897,'SSBN 608',1000 from dual union all
select 8821,'1962-S361','1962-10-01 00:00:00','military missile','success',929,'A2','','','','OT',1897,'SSBN 608',1000 from dual union all
select 8822,'1962-S362','1962-10-01 00:00:00','military missile','success',976,'','8K63 UBP-204','','','Test',636,'',402 from dual union all
select 8823,'1962-S363','1962-10-01 00:00:00','military missile','success',976,'','8K63 UBP-205','','','Test',636,'',402 from dual union all
select 8824,'1962-S364','1962-10-01 00:00:00','military missile','success',976,'','8K63 UBP-206','','','Test',636,'',402 from dual union all
select 8825,'1962-S365','1962-10-01 00:00:00','military missile','success',976,'','8K63 UBP-207','','','Test',636,'',402 from dual union all
select 8826,'1962-S366','1962-10-01 00:00:00','military missile','success',976,'','8K63 UBP-208','','','Test',636,'',402 from dual union all
select 8827,'1962-S367','1962-10-01 00:00:00','military missile','success',976,'','8K63 UBP-209','','','Test',636,'',402 from dual union all
select 8828,'1962-S368','1962-10-01 00:00:00','military missile','success',976,'','8K63 UBP-210','','','Test',636,'',402 from dual union all
select 8829,'1962-S369','1962-10-01 00:00:00','military missile','success',976,'','8K63 UBP-211','','','Test',636,'',402 from dual union all
select 883,'1950-A05','1950-02-09 00:00:00','atmospheric rocket','success',880,'484-39','Nike 484 37R','','','Test',1895,'',10 from dual union all
select 8830,'1962-S37','1962-02-13 20:55:00','military missile','success',155,'40E','Mk 4-2A','R&D','','Test',228,'',1600 from dual union all
select 8831,'1962-S370','1962-10-01 00:00:00','military missile','success',976,'','8K63 UBP-212','','','Test',636,'',402 from dual union all
select 8832,'1962-S371','1962-10-01 00:00:00','military missile','success',976,'','8K63 UBP-213','','','Test',636,'',402 from dual union all
select 8833,'1962-S372','1962-10-01 00:00:00','military missile','success',976,'','8K63 UBP-214','','','Test',636,'',402 from dual union all
select 8834,'1962-S373','1962-10-01 00:00:00','military missile','success',979,'','8K65 UBP-29','','','Test',636,'',675 from dual union all
select 8835,'1962-S374','1962-10-01 00:00:00','military missile','success',979,'','8K65 UBP-30','','','Test',636,'',675 from dual union all
select 8836,'1962-S375','1962-10-01 00:00:00','military missile','success',979,'','8K65 UBP-31','','','Test',636,'',675 from dual union all
select 8837,'1962-S376','1962-10-29 00:00:00','military missile','success',979,'','8K65 UBP-32 32 RD','','','Test',636,'',675 from dual union all
select 8838,'1962-S377','1962-09-30 06:02:00','suborbital rocket','success',35,'NASA 04.69CG','Lockheed XR','','','XR Astron',1821,'',171 from dual union all
select 8839,'1962-S378','1962-10-02 11:46:25','military missile','failure',154,'4D','Nike TV-33','NTMP K-3','BRIAR STREET','Target',1666,'',500 from dual union all
select 884,'1950-A06','1950-02-14 00:00:00','atmospheric rocket','success',880,'484-40','Nike 484 38R','','','Test',1895,'',10 from dual union all
select 8840,'1962-S379','1962-10-05 15:11:33','military missile','success',982,'12LU','8K64U LKI-12','','','Test',572,'',1210 from dual union all
select 8841,'1962-S38','1962-02-15 15:35:53','military missile','success',794,'410','Mk 5 RV','R&D','','Test',264,'',900 from dual union all
select 8842,'1962-S380','1962-10-05 15:29:00','military missile','success',982,'T 225   05-20T','8K64U UBP-1','Salvo','','Test',570,'',1210 from dual union all
select 8843,'1962-S381','1962-10-06 00:00:00','military missile','success',1419,'SM-35','Mk 4 RV','R&D Cat II','TIGHT BRACELET','Test',1654,'',1000 from dual union all
select 8844,'1962-S382','1962-10-07 00:00:00','suborbital rocket','success',839,'','','','','Aeron/Ionos',394,'',100 from dual union all
select 8845,'1962-S383','1962-10-11 00:00:00','suborbital rocket','unknown',545,'','','','','Aeron',855,'',100 from dual union all
select 8846,'1962-S384','1962-10-11 01:11:38','military missile','success',982,'T 226   01LU','8K64U UBP-2','8F115','','Test',566,'',1210 from dual union all
select 8847,'1962-S385','1962-10-11 13:43:00','military missile','success',982,'T 226   02LU','8K64U UBP-3','8F115','','Test',566,'',1210 from dual union all
select 8848,'1962-S386','1962-10-11 00:30:00','suborbital rocket','success',839,'','','','','Aeron/Ionos',394,'',100 from dual union all
select 8849,'1962-S387','1962-10-12 16:24:03','military missile','success',1420,'N-9','Mk 6 RV','','','Test',231,'',1300 from dual union all
select 885,'1950-A07','1950-02-22 00:54:00','suborbital rocket','success',59,'SC 9','Air Sampling','','','Aeronomy',1869,'',49 from dual union all
select 8850,'1962-S388','1962-10-13 06:59:00','suborbital rocket','success',30,'AC3.353','AS&E-4','','','XR Astron',1869,'',317 from dual union all
select 8851,'1962-S389','1962-10-14 00:00:00','military missile','success',928,'A1T-8','','','','Test',421,'EAG-154',500 from dual union all
select 8852,'1962-S39','1962-02-16 23:04:00','military missile','success',154,'137D','Mk 3-3B','OT','BIG JOHN','OT',1666,'',1800 from dual union all
select 8853,'1962-S390','1962-10-15 08:39:00','suborbital rocket','success',1490,'AGI37','Bio (Rat)','','','Life Sci',668,'',120 from dual union all
select 8854,'1962-S391','1962-10-15 11:15:00','suborbital rocket','success',839,'AC6.441','Firefly III BONNIE','','','Aeron',392,'',91 from dual union all
select 8855,'1962-S392','1962-10-15 16:17:00','suborbital rocket','success',1194,'SL82','MRAO-1/UCL I-11','','','Ionos',1839,'',154 from dual union all
select 8856,'1962-S393','1962-10-16 15:20:35','military missile','success',982,'E 22502-18LU','8K64U UBP-4','8F115','','Test',566,'',1210 from dual union all
select 8857,'1962-S394','1962-10-16 11:15:00','suborbital rocket','success',839,'AC6.434','Firefly III ALICE','','','Aeron',392,'',90 from dual union all
select 8858,'1962-S395','1962-10-16 15:06:00','suborbital rocket','success',837,'NASA 14.31GI','GCA Ionos 5/CWPE','','','Ionos',1821,'',166 from dual union all
select 8859,'1962-S396','1962-10-17 15:35:19','military missile','success',982,'T 22502-19LU','8K64U UBP-5','8F115','','Test',566,'',1210 from dual union all
select 886,'1950-A08','1950-02-23 00:00:00','atmospheric rocket','success',880,'484-41','Nike 484 39R','','','Test',1895,'',10 from dual union all
select 8860,'1962-S397','1962-10-17 00:40:00','suborbital rocket','success',839,'AC6.443','Firefly III DAGMAR','','','Aeron',392,'',89 from dual union all
select 8861,'1962-S398','1962-10-17 11:16:00','suborbital rocket','success',839,'AC6.442','Firefly III CINDY','','','Aeron',392,'',86 from dual union all
select 8862,'1962-S399','1962-10-17 15:52:00','suborbital rocket','success',839,'AC6.832','VLF Propagation','','','Ionos',1821,'',141 from dual union all
select 8863,'1962-S40','1962-02-17 00:00:00','military missile','success',976,'','8K63 UBP-173','','','Test',636,'',402 from dual union all
select 8864,'1962-S400','1962-10-18 00:00:00','suborbital rocket','success',1012,'','R-5B Geophysical','','','Ionos',636,'',508 from dual union all
select 8865,'1962-S401','1962-10-18 08:31:00','suborbital rocket','success',1490,'AGI36','Bio (Rat)','','','Life Sci',668,'',110 from dual union all
select 8866,'1962-S402','1962-10-18 01:00:00','suborbital rocket','failure',839,'AC6.435','Firefly III BRENDA','','','Aeron',392,'',86 from dual union all
select 8867,'1962-S403','1962-10-19 00:00:00','suborbital rocket','success',1490,'AGI44','','','','Test',668,'',135 from dual union all
select 8868,'1962-S404','1962-10-19 11:17:00','suborbital rocket','success',839,'AC6.438','Firefly III QUEENIE','','','Aeron',392,'',91 from dual union all
select 8869,'1962-S405','1962-10-19 18:15:00','military missile','success',162,'14F','Mk 4-1B S/N 14','R&D','','Test',226,'',1400 from dual union all
select 887,'1950-A09','1950-03-02 00:00:00','atmospheric rocket','success',880,'484-42','Nike 484 40R','','','Test',1895,'',10 from dual union all
select 8870,'1962-S406','1962-10-20 08:30:00','test rocket','success',1268,'','CHECKMATE','','','Nuclear Test',803,'',147 from dual union all
select 8871,'1962-S407','1962-10-20 09:00:00','suborbital rocket','success',839,'FB-7','Sphere','','','Aeronomy',803,'',100 from dual union all
select 8872,'1962-S408','1962-10-21 00:00:00','suborbital rocket','unknown',545,'','','','','Aeron',855,'',100 from dual union all
select 8873,'1962-S409','1962-10-23 03:44:24','military missile','success',916,'351','P-41','','','Test',257,'',180 from dual union all
select 8874,'1962-S41','1962-02-17 00:00:00','military missile','success',976,'','8K63 UBP-174','','','Test',636,'',402 from dual union all
select 8875,'1962-S410','1962-10-22 00:00:00','suborbital rocket','success',1012,'','8A62 Geophysical','RAO? K-3','','Aeron?',636,'',500 from dual union all
select 8876,'1962-S411','1962-10-22 03:35:00','military missile','success',976,'Burst 0340Z','8K63 K-3','Nuclear test K-3','','Nuclear test',942,'',400 from dual union all
select 8877,'1962-S412','1962-10-22 00:00:00','suborbital rocket','success',1490,'AGI46','','','','Test',668,'',120 from dual union all
select 8878,'1962-S413','1962-10-23 11:01:00','suborbital rocket','success',1490,'AGI32','Airglow','','','Aeronomy/Ionos',668,'',175 from dual union all
select 8879,'1962-S414','1962-10-22 11:19:00','suborbital rocket','success',839,'AC6.448','Firefly III LOUISE','','','Aeron',392,'',100 from dual union all
select 888,'1950-A10','1950-03-14 00:00:00','atmospheric rocket','success',880,'484-43','Nike 484 41R','','','Test',1895,'',10 from dual union all
select 8880,'1962-S415','1962-10-23 11:19:00','suborbital rocket','success',839,'AC6.446','Firefly III KITTY','','','Aeron',393,'',100 from dual union all
select 8881,'1962-S417','1962-10-23 19:10:00','suborbital rocket','success',549,'AC20.463','Firefly III ETHEL','','','Ionos',391,'',159 from dual union all
select 8882,'1962-S418','1962-10-24 09:46:00','suborbital rocket','success',535,'HAD 111','Sphere','','','Aeron',1835,'',133 from dual union all
select 8883,'1962-S419','1962-10-25 00:00:00','suborbital rocket','success',839,'','Mass spectrometer','','','Ionos',803,'',100 from dual union all
select 8884,'1962-S42','1962-02-17 19:43:00','suborbital rocket','success',30,'NASA 04.41NP','LeRC LH2 test','','','Tech',1821,'',157 from dual union all
select 8885,'1962-S420','1962-10-25 06:00:00','suborbital rocket','success',470,'CRL AA13.195','','','','Ionos',394,'',669 from dual union all
select 8886,'1962-S421','1962-10-25 06:30:00','suborbital rocket','success',839,'AC6.444','Firefly III ENID','','','Aeron',393,'',137 from dual union all
select 8887,'1962-S423','1962-10-25 11:21:00','suborbital rocket','unknown',839,'AC6.437','Firefly III PAULA','','','Aeron',393,'',95 from dual;
insert into LAUNCH
select 8888,'1962-S424','1962-10-25 23:28:00','suborbital rocket','success',30,'AC3.138','EUV Mon/XR/RPA','','','Solar UV',1869,'',213 from dual union all
select 8889,'1962-S425','1962-10-26 00:00:00','suborbital rocket','success',624,'','Mass spectrometer','','','Ionos',803,'',500 from dual union all
select 889,'1950-A11','1950-03-14 20:43:00','suborbital rocket','failure',50,'USAF 3','Solar X/Airglow','','','Solar X/UV?',640,'',3 from dual union all
select 8890,'1962-S426','1962-10-26 00:00:00','suborbital rocket','success',839,'','Ionos winds','','','Ionos',803,'',100 from dual union all
select 8891,'1962-S427','1962-10-26 00:00:00','suborbital rocket','success',839,'','Sphere','','','Aeronomy',803,'',100 from dual union all
select 8892,'1962-S428','1962-10-26 09:44:05','military missile','success',1370,'141','BLUEGILL TRIPLE PRIME','','BLUEGILL TRIPLE PRIME','Nuclear Test',772,'',500 from dual union all
select 8893,'1962-S429','1962-10-26 09:58:00','suborbital rocket','success',545,'X-11','','','','Ionos',803,'',100 from dual union all
select 8894,'1962-S43','1962-02-20 01:00:01','military missile','success',916,'335','P-31','','','Test',256,'',180 from dual union all
select 8895,'1962-S430','1962-10-26 10:05:00','suborbital rocket','success',545,'X-12','','','','Ionos',803,'',100 from dual union all
select 8896,'1962-S431','1962-10-26 10:11:00','suborbital rocket','success',545,'X-14','','','','Ionos',803,'',100 from dual union all
select 8897,'1962-S432','1962-10-26 10:15:00','suborbital rocket','success',839,'FB-6','Mass spectrometer','','','Aeronomy',803,'',100 from dual union all
select 8898,'1962-S433','1962-10-26 10:22:00','suborbital rocket','success',545,'X-17','','','','Ionos',803,'',100 from dual union all
select 8899,'1962-S434','1962-10-26 10:59:25','ballistic missile test','success',154,'159D','Nike TV-33/Pod 20','NTMP K-5','CLOSED CIRCUITS','Target',1662,'',1800 from dual union all
select 89,'1944-A53','1944-12-01 00:00:00','military missile','success',943,'20','','','','Test',764,'',5 from dual union all
select 890,'1950-A12','1950-03-21 00:00:00','atmospheric rocket','success',880,'484-45','Nike 484 42R','','','Test',1895,'',10 from dual union all
select 8900,'1962-S435','1962-10-26 17:05:34','military missile','success',1420,'N-12','Mk 6 RV','','','Test',230,'',1300 from dual union all
select 8901,'1962-S436','1962-10-27 00:00:00','suborbital rocket','success',839,'','Sphere','','','Aeronomy',803,'',100 from dual union all
select 8902,'1962-S437','1962-10-28 04:10:00','military missile','success',976,'','8K63 K-4','Nuclear test K-4','','Nuclear test',942,'',400 from dual union all
select 8903,'1962-S438','1962-10-29 00:00:00','suborbital rocket','unknown',545,'','','','','Aeron',855,'',100 from dual union all
select 8904,'1962-S439','1962-10-29 23:21:00','suborbital rocket','success',1490,'AGI34','','','','Aeronomy/Ionos',668,'',180 from dual union all
select 8905,'1962-S44','1962-02-20 13:28:00','suborbital rocket','success',839,'NASA 10.90UA','Composition','','','Aeronomy',1821,'',132 from dual union all
select 8906,'1962-S440','1962-10-30 01:50:00','suborbital rocket','success',35,'NASA 04.54UG','OAO sensors','','','UV Astron',1821,'',188 from dual union all
select 8907,'1962-S441','1962-10-31 00:00:00','military missile','success',976,'','8K63 PRO Target','SSATC Target','','Test',636,'',402 from dual union all
select 8908,'1962-S442','1962-11-01 00:00:00','military missile','success',976,'','8K63 UBP-215','','','Test',636,'',402 from dual union all
select 8909,'1962-S443','1962-11-01 00:00:00','military missile','success',976,'','8K63 UBP-216','','','Test',636,'',402 from dual union all
select 891,'1950-A13','1950-03-23 00:00:00','atmospheric rocket','success',880,'484-44','Nike 484 43R','','','Test',1895,'',10 from dual union all
select 8910,'1962-S445','1962-11-01 00:00:00','military missile','success',979,'','8K65 UBP-33','','','Test',636,'',675 from dual union all
select 8911,'1962-S446','1962-11-01 00:00:00','military missile','success',979,'','8K65 UBP-34','','','Test',636,'',675 from dual union all
select 8912,'1962-S447','1962-11-04 00:00:00','military missile','success',979,'','8K65 UBP-35 32 RD','','','Test',636,'',675 from dual union all
select 8913,'1962-S448','1962-11-26 00:00:00','military missile','success',979,'','8K65 UBP-36 32 RD','','','Test',636,'',675 from dual union all
select 8914,'1962-S449','1962-11-01 00:00:00','military missile','success',979,'','','','','Test',636,'',675 from dual union all
select 8915,'1962-S45','1962-02-21 00:00:00','military missile','success',976,'','8K63 UBP-175','','','Test',636,'',402 from dual union all
select 8916,'1962-S450','1962-11-01 00:00:00','military missile','success',206,'004','','','','Reentry Test',313,'',270 from dual union all
select 8917,'1962-S451','1962-10-31 23:32:00','suborbital rocket','failure',549,'AC20.461','Firefly III HAZEL','','','Aeron',391,'',100 from dual union all
select 8918,'1962-S452','1962-11-01 11:25:00','suborbital rocket','success',839,'AC6.439','Firefly III RUBY','','','Aeron',392,'',100 from dual union all
select 8919,'1962-S453','1962-11-01 11:54:48','military missile','success',1370,'226','KINGFISH','','KINGFISH','Nuclear Test',773,'',500 from dual union all
select 892,'1950-A14','1950-03-24 00:00:00','atmospheric rocket','success',442,'','F23 Ramjet','','','Test',1821,'',17 from dual union all
select 8920,'1962-S454','1962-11-01 12:08:00','suborbital rocket','success',545,'X-20','Electron density','','','Ionos',803,'',96 from dual union all
select 8921,'1962-S455','1962-11-01 12:16:00','suborbital rocket','success',545,'X-24','Electron density','','','Ionos',803,'',100 from dual union all
select 8922,'1962-S456','1962-11-01 12:20:00','suborbital rocket','success',839,'FB-8','Sphere','','','Aeronomy',803,'',100 from dual union all
select 8923,'1962-S457','1962-11-01 12:24:00','suborbital rocket','success',545,'X-27','Electron density','','','Ionos',803,'',100 from dual union all
select 8924,'1962-S458','1962-11-01 12:40:00','suborbital rocket','success',545,'X-28','Electron density','','','Ionos',803,'',100 from dual union all
select 8925,'1962-S459','1962-11-02 04:27:59','suborbital rocket','success',549,'AC20.462','Firefly III LISA','','','Aeron',391,'',170 from dual union all
select 8926,'1962-S46','1962-02-21 12:30:00','suborbital rocket','success',1020,'RAM A-2','RAM A-2','','','Reentry Test',1821,'',1000 from dual union all
select 8927,'1962-S460','1962-11-02 07:34:00','suborbital rocket','failure',839,'FB-9','Ionos winds','','','Aeronomy',803,'',100 from dual union all
select 8928,'1962-S461','1962-11-02 11:25:00','suborbital rocket','failure',839,'AC6.436','Firefly III CAROL','','','Aeron',392,'',10 from dual union all
select 8929,'1962-S462','1962-11-03 00:00:00','suborbital rocket','success',839,'','Ionos winds','','','Aeronomy',803,'',100 from dual union all
select 893,'1950-A15','1950-03-28 00:00:00','atmospheric rocket','success',880,'484-47','Nike 484 44R','','','Test',1895,'',10 from dual union all
select 8930,'1962-S463','1962-11-03 02:53:00','military missile','success',982,'T 22602-20T','8K64U UBP-6','','','OT',566,'',1210 from dual union all
select 8931,'1962-S464','1962-11-04 00:00:00','suborbital rocket','success',839,'','Ionos winds','','','Aeronomy',803,'',100 from dual union all
select 8932,'1962-S465','1962-11-04 00:00:00','suborbital rocket','success',839,'','Ionos winds','','','Aeronomy',803,'',100 from dual union all
select 8933,'1962-S466','1962-11-05 11:29:00','suborbital rocket','success',839,'AC6.440','Firefly III SALLY','','','Aeron',392,'',97 from dual union all
select 8934,'1962-S467','1962-11-06 08:00:00','suborbital rocket','success',839,'AC6.445','Firefly III BEVERLY','','','Aeron',392,'',89 from dual union all
select 8935,'1962-S468','1962-11-06 19:45:00','suborbital rocket','success',30,'AC3.148','HR solar spectrum','','','Solar EUV',1869,'',308 from dual union all
select 8936,'1962-S469','1962-11-07 10:25:00','suborbital rocket','success',839,'NASA 10.99CI','GCA Ionos 6','','','Ionos',1821,'',132 from dual union all
select 8937,'1962-S47','1962-02-21 22:30:00','military missile','failure',154,'52D','Mk 3-2B','OT','CHAIN SMOKE','OT',1667,'',1800 from dual union all
select 8938,'1962-S470','1962-11-07 10:53:00','suborbital rocket','success',837,'NASA 14.16CA','GCA Release 24 Na','','','Aeronomy',1821,'',165 from dual union all
select 8939,'1962-S471','1962-11-07 19:43:00','military missile','success',162,'16F','Mk 4-1B','R&D','','Test',226,'',1400 from dual union all
select 894,'1950-A16','1950-03-31 00:00:00','atmospheric rocket','success',880,'484-46','Nike 484 45R','','','Test',1895,'',10 from dual union all
select 8940,'1962-S472','1962-11-09 03:12:00','military missile','success',871,'ZM-16','ZM-16  sim target','','','Test',1057,'',80 from dual union all
select 8941,'1962-S473','1962-11-08 00:00:00','military missile','success',871,'ZW-50','','','','Test',1884,'',50 from dual union all
select 8942,'1962-S474','1962-11-11 01:03:00','suborbital rocket','success',30,'AC3.452','Firefly III FANNY','','','Aeron/Ionos',390,'',260 from dual union all
select 8943,'1962-S475','1962-11-13 10:10:00','suborbital rocket','success',1199,'SL114','LUX-8/JB-3/MO-2','','','Solar X/Aeron/Meteorites',1839,'',206 from dual union all
select 8944,'1962-S476','1962-11-13 22:30:00','suborbital rocket','success',30,'AC3.453','Firefly III GILDA','','','Aeron',390,'',253 from dual union all
select 8945,'1962-S477','1962-11-14 22:36:02','military missile','failure',162,'13F','Mk 4-5B','R&D','ACTION TIME','Test',1716,'',300 from dual union all
select 8946,'1962-S478','1962-11-16 00:52:22','military missile','success',916,'348','P-42','','','Test',256,'',180 from dual union all
select 8947,'1962-S479','1962-11-16 01:26:00','suborbital rocket','success',30,'AC3.449  (SF6)','Firefly III KAREN','','','Aeron',390,'',234 from dual union all
select 8948,'1962-S48','1962-02-24 00:00:00','military missile','success',993,'','','','','Test',1626,'HOTEL2',293 from dual union all
select 8949,'1962-S480','1962-11-16 17:45:02','test rocket','success',1118,'SA-3','','','','Test',267,'',167 from dual union all
select 895,'1950-A17','1950-04-07 00:00:00','atmospheric rocket','success',880,'484-48','Nike 484 46R','','','Test',1895,'',10 from dual union all
select 8950,'1962-S481','1962-11-18 04:57:00','suborbital rocket','success',35,'NASA 04.27NP','LeRC LH2 test','','','Tech',1821,'',206 from dual union all
select 8951,'1962-S482','1962-11-19 17:00:11','military missile','success',794,'421A','Mk 5 RV','R&D','','Test',264,'',900 from dual union all
select 8952,'1962-S483','1962-11-19 00:00:00','military missile','success',976,'','8K63 UBP-217','','','Test',636,'',402 from dual union all
select 8953,'1962-S484','1962-11-20 21:41:30','suborbital rocket','success',43,'NASA 06.06GA','Thermosphere Probe 1','','','Aeronomy',1821,'',345 from dual union all
select 8954,'1962-S485','1962-11-21 18:20:35','test rocket','success',249,'201','ERCS/279L 3','','','Comms',1015,'',1000 from dual union all
select 8955,'1962-S486','1962-11-22 19:43:06','military missile','success',982,'T 226   03LU','8K64U UBP-7','8F116','','Test',566,'',1210 from dual union all
select 8956,'1962-S487','1962-11-22 01:11:00','suborbital rocket','success',1198,'SL27','RAE Test 16','','','Test',1839,'',114 from dual union all
select 8957,'1962-S488','1962-11-24 00:00:00','military missile','success',976,'','8K63 UBP-218','','','Test',636,'',402 from dual union all
select 8958,'1962-S489','1962-11-24 17:45:13','military missile','success',982,'T 22500-15LU','8K64U UBP-8','8F116','','Test',566,'',1210 from dual union all
select 8959,'1962-S49','1962-03-01 19:45:01','military missile','success',929,'A2X-27 (A2G-5)','Mk 2 guidance','','','R&D',251,'',1000 from dual union all
select 896,'1950-A18','1950-04-11 00:00:00','atmospheric rocket','success',880,'484-36','Nike 484 47R','','','Test',1895,'',10 from dual union all
select 8960,'1962-S490','1962-11-26 00:00:00','suborbital rocket','success',287,'C20','','','','Aeronomy',664,'',120 from dual union all
select 8961,'1962-S491','1962-11-27 00:00:00','suborbital rocket','success',624,'','RPA','','','Ionos',803,'',500 from dual union all
select 8962,'1962-S492','1962-11-28 02:12:08','military missile','success',916,'354','P-43','','','Test',256,'',180 from dual union all
select 8963,'1962-S493','1962-11-27 23:37:00','suborbital rocket','success',287,'C30','','','','Aeronomy',310,'',180 from dual union all
select 8964,'1962-S494','1962-11-27 10:17:00','suborbital rocket','success',535,'HAD 112','Sphere','','','Aeron',1835,'',113 from dual union all
select 8965,'1962-S495','1962-11-27 18:00:00','suborbital rocket','success',30,'NASA 04.21US','','','','Solar EUV',1869,'',203 from dual union all
select 8966,'1962-S496','1962-11-28 00:00:00','suborbital rocket','success',549,'AC20.454','Firefly III MABEL','','','Aeron',391,'',121 from dual union all
select 8967,'1962-S497','1962-11-28 23:25:00','military missile','success',871,'ZM-17','ZM-17  St3 guidance','','','Test',1057,'',80 from dual union all
select 8968,'1962-S498','1962-11-28 17:36:00','suborbital rocket','success',287,'C32','LangP/Na','','','Ionos',664,'',160 from dual union all
select 8969,'1962-S499','1962-11-28 10:17:00','suborbital rocket','success',1199,'SL165','UCL GC-22','','','Aeronomy',1839,'',230 from dual union all
select 897,'1950-A19','1950-04-14 00:00:00','atmospheric rocket','success',880,'484-49','Nike 484 48R','','','Test',1895,'',10 from dual union all
select 8970,'1962-S50','1962-03-01 00:14:09','military missile','success',155,'66E','Mk 4-5B','R&D Cat II','SILVER SPUR','Test',1715,'',1600 from dual union all
select 8971,'1962-S500','1962-11-29 00:00:00','military missile','success',976,'','8K63 UBP-219','','','Test',636,'',402 from dual union all
select 8972,'1962-S501','1962-11-29 00:00:00','military missile','success',871,'ZW-51','Sim target','','','Test',1884,'',50 from dual union all
select 8973,'1962-S502','1962-11-29 16:33:00','military missile','success',242,'BK.18','H-209','','','Reentry Test/Ionos',1843,'',576 from dual union all
select 8974,'1962-S503','1962-12-01 00:00:00','military missile','success',837,'BMRS','','','','Reentry test',1895,'',100 from dual union all
select 8975,'1962-S504','1962-12-01 00:00:00','military missile','success',837,'BMRS','','','','Reentry test',1895,'',100 from dual union all
select 8976,'1962-S505','1962-12-01 00:00:00','military missile','success',837,'BMRS','','','','Reentry test',1895,'',100 from dual union all
select 8977,'1962-S506','1962-12-09 00:00:00','military missile','success',979,'','8K65 UBP-37 43 RD','','','Test',636,'',675 from dual;
insert into LAUNCH
select 8978,'1962-S507','1962-12-17 00:00:00','military missile','success',874,'ZW-52 MUDFLAP 1','DM-15S Test 1','','','ASAT Test MUDFLAP',1884,'',180 from dual union all
select 8979,'1962-S508','1962-11-30 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 898,'1950-A20','1950-04-20 00:00:00','atmospheric rocket','success',880,'484-50','Nike 484 49R','','','Test',1895,'',10 from dual union all
select 8980,'1962-S509','1962-11-30 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8981,'1962-S51','1962-03-01 05:07:00','test rocket','success',1139,'ST-8','Reentry 1','','','Reentry Test',1812,'',214 from dual union all
select 8982,'1962-S510','1962-11-30 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 8983,'1962-S511','1962-11-30 17:36:00','suborbital rocket','success',287,'C11','LangP/Na','','','Ionos',664,'',170 from dual union all
select 8984,'1962-S512','1962-11-30 00:00:00','suborbital rocket','success',287,'C26','','','','Aeronomy',664,'',130 from dual union all
select 8985,'1962-S513','1962-11-30 08:17:00','suborbital rocket','success',287,'C29','','','','Aeronomy',310,'',180 from dual union all
select 8986,'1962-S514','1962-11-30 00:00:00','suborbital rocket','success',287,'C37','','','','Aeronomy',313,'',60 from dual union all
select 8987,'1962-S515','1962-11-30 10:57:00','suborbital rocket','success',839,'NASA 10.108CI','GCA Ionos 7','','','Ionos',1821,'',122 from dual union all
select 8988,'1962-S516','1962-11-30 11:15:00','suborbital rocket','success',837,'NASA 14.17CA','GCA Ionos 8/Rel Na','','','Aeronomy',1821,'',170 from dual union all
select 8989,'1962-S517','1962-12-01 18:06:00','suborbital rocket','success',837,'NASA 14.32GI','','','','Ionos',1821,'',169 from dual union all
select 899,'1950-A21','1950-05-12 00:00:00','atmospheric rocket','success',365,'','X-2 Model','','','Test',1821,'',10 from dual union all
select 8990,'1962-S518','1962-12-01 20:34:00','suborbital rocket','success',837,'NASA 14.20UA','UM Pitot 2','','','Aeronomy',1821,'',131 from dual union all
select 8991,'1962-S519','1962-12-01 21:25:00','suborbital rocket','success',839,'NASA 10.45GA','GSFC Grenade 23','','','Aeronomy',1821,'',126 from dual union all
select 8992,'1962-S52','1962-03-01 17:07:00','suborbital rocket','success',839,'AA6.167','7-inch Sphere','','','Aeronomy',394,'',144 from dual union all
select 8993,'1962-S520','1962-12-01 21:34:00','suborbital rocket','success',839,'NASA 10.68GA','GSFC Grenade 24','','','Aeronomy',428,'',108 from dual union all
select 8994,'1962-S521','1962-12-01 23:20:00','suborbital rocket','success',837,'NASA 14.45AA/AC5.465','Firefly III TERRY','','','Aeronomy Na',392,'',222 from dual union all
select 8995,'1962-S522','1962-12-03 23:20:00','suborbital rocket','success',837,'NASA 14.46AA/AC5.468','Firefly III SHARON','','','Aeronomy Na',392,'',222 from dual union all
select 8996,'1962-S523','1962-12-04 00:00:00','suborbital rocket','success',287,'C21','','','','Aeronomy',671,'',60 from dual union all
select 8997,'1962-S524','1962-12-04 00:01:00','suborbital rocket','success',837,'AC5.466','Firefly III IVY','','','Aeron',393,'',150 from dual union all
select 8998,'1962-S525','1962-12-04 03:45:00','suborbital rocket','success',837,'AC5.467','Firefly III ESTHER','','','Aeron',393,'',135 from dual union all
select 8999,'1962-S526','1962-12-04 04:45:00','suborbital rocket','success',549,'AC20.455','Firefly III DINAH','','','Aeron',391,'',115 from dual union all
select 9,'1943-A02','1943-01-25 00:00:00','military missile','failure',8,'11','','','','Test',675,'',40 from dual union all
select 90,'1944-A54','1944-12-01 00:00:00','military missile','success',943,'21','','','','Test',764,'',5 from dual union all
select 900,'1950-A22','1950-05-18 00:00:00','atmospheric rocket','success',564,'','E2 Model','','','Test',1821,'',5 from dual union all
select 9000,'1962-S527','1962-12-04 07:06:00','suborbital rocket','success',839,'NASA 10.67GA','GSFC Grenade 25','','','Aeronomy',428,'',114 from dual union all
select 9001,'1962-S528','1962-12-04 07:56:00','suborbital rocket','success',1196,'SL166','UCL GC-23','','','Aeronomy',1839,'',105 from dual union all
select 9002,'1962-S529','1962-12-04 10:28:00','suborbital rocket','success',535,'HAD 113','Sphere','','','Aeron',1835,'',113 from dual union all
select 9003,'1962-S53','1962-03-01 23:23:00','suborbital rocket','success',839,'NASA 10.100CA','GCA Release 17 Na','','','Aeronomy',1821,'',135 from dual union all
select 9004,'1962-S530','1962-12-05 00:00:00','suborbital rocket','failure',446,'D-03','','','','Test',664,'',80 from dual union all
select 9005,'1962-S531','1962-12-05 00:00:00','military missile','success',1419,'SM-11','Mk 4 RV','R&D Cat II','YELLOW JACKET','Test',1654,'',1000 from dual union all
select 9006,'1962-S532','1962-12-05 01:51:00','suborbital rocket','success',549,'AC20.460','Firefly III PATSY','','','Aeron',391,'',150 from dual union all
select 9007,'1962-S533','1962-12-05 21:25:00','military missile','success',162,'21F','Mk 4-1B','R&D','','Test',226,'',1400 from dual union all
select 9008,'1962-S534','1962-12-05 22:00:00','suborbital rocket','success',839,'NASA 10.109CI','GCA Ionos 9','','','Ionos',1821,'',128 from dual union all
select 9009,'1962-S535','1962-12-06 00:00:00','military missile','success',976,'','8K63 PRO-6 Target','','','Test',942,'',402 from dual union all
select 901,'1950-A23','1950-05-19 00:00:00','military missile','failure',571,'1','','','','Test',1863,'',0 from dual union all
select 9010,'1962-S536','1962-12-05 22:16:00','suborbital rocket','success',837,'NASA 14.18CA','GCA Ionos 10/Rel Na','','','Aeron',1821,'',100 from dual union all
select 9011,'1962-S537','1962-12-06 05:32:00','suborbital rocket','success',839,'NASA 10.47GA','GSFC Grenade 27','','','Aeronomy',1821,'',119 from dual union all
select 9012,'1962-S538','1962-12-06 05:43:00','suborbital rocket','success',839,'NASA 10.66GA','GSFC Grenade 28','','','Aeronomy',428,'',122 from dual union all
select 9013,'1962-S539','1962-12-06 10:21:00','suborbital rocket','success',549,'AC20.459','Firefly III EVA','','','Aeron',391,'',89 from dual union all
select 9014,'1962-S54','1962-03-02 00:01:00','suborbital rocket','success',839,'NASA 10.38GA','GSFC Grenade 17','','','Aeronomy',1821,'',113 from dual union all
select 9015,'1962-S540','1962-12-06 20:31:37','military missile','failure',1420,'N-11','Mk 4 RV','','','Test',231,'',500 from dual union all
select 9016,'1962-S541','1962-12-07 15:00:01','military missile','success',795,'424','Mk 11 RV','R&D','','Test',266,'',900 from dual union all
select 9017,'1962-S542','1962-12-07 00:00:00','military missile','success',976,'','8K63 PRO Target','SSATC Target?','','Test',636,'',402 from dual union all
select 9018,'1962-S543','1962-12-07 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 9019,'1962-S544','1962-12-07 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 902,'1950-A24','1950-05-23 00:00:00','atmospheric rocket','success',365,'','F26 Model','','','Test',1821,'',10 from dual union all
select 9020,'1962-S545','1962-12-08 00:00:00','suborbital rocket','success',287,'C28','','','','Aeronomy',310,'',180 from dual union all
select 9021,'1962-S546','1962-12-08 01:08:00','suborbital rocket','success',30,'AC3.450','Firefly III LAURA','','','Aeron',390,'',220 from dual union all
select 9022,'1962-S547','1962-12-09 00:00:00','suborbital rocket','success',287,'C31','','','','Aeronomy',310,'',180 from dual union all
select 9023,'1962-S548','1962-12-11 02:55:29','military missile','success',916,'353','P-44','','','Test',256,'',180 from dual union all
select 9024,'1962-S549','1962-12-11 00:00:00','suborbital rocket','success',839,'ISRC-RS-','','','','Aeron?',1343,'',100 from dual union all
select 9025,'1962-S55','1962-03-02 10:47:00','suborbital rocket','success',839,'NASA 10.70GT','Water test','','','Test',1821,'',145 from dual union all
select 9026,'1962-S550','1962-12-11 00:00:00','military missile','success',976,'','8K63 PRO-7','','','Test',942,'',402 from dual union all
select 9027,'1962-S551','1962-12-11 01:30:00','suborbital rocket','success',839,'AC6.447','Firefly III DANA','','','Aeron',393,'',108 from dual union all
select 9028,'1962-S552','1962-12-11 03:27:00','suborbital rocket','success',839,'Ferdinand 3','Ferdinand 3','','','Ionos',29,'',121 from dual union all
select 9029,'1962-S553','1962-12-12 11:58:35','military missile','success',982,'T 22500-13LU','8K64U UBP-10','8F116','','Test',566,'',1210 from dual union all
select 903,'1950-A25','1950-06-02 17:07:00','suborbital rocket','success',50,'USAF 5','Airglow','','','Opt Astronomy',640,'',24 from dual union all
select 9030,'1962-S554','1962-12-12 00:00:00','suborbital rocket','success',287,'C24','','','','Aeronomy',313,'',130 from dual union all
select 9031,'1962-S555','1962-12-12 00:00:00','suborbital rocket','success',287,'C25','','','','Aeronomy',313,'',130 from dual union all
select 9032,'1962-S556','1962-12-12 11:38:03','military missile','success',154,'161D','Nike TV-23','NTMP K-6','DEER PARK','Target',1664,'',1800 from dual union all
select 9033,'1962-S557','1962-12-12 12:10:00','military missile','success',871,'ZK-8 20066','K-6 Atlas Intercept','','','Test',873,'',70 from dual union all
select 9034,'1962-S558','1962-12-12 23:46:00','suborbital rocket','success',549,'AC20.456','Firefly III NETTY','','','Aeron',391,'',101 from dual union all
select 9035,'1962-S559','1962-12-13 15:20:00','suborbital rocket','success',217,'AA-3-5','BB3 05','','','Test',1821,'',98 from dual union all
select 9036,'1962-S56','1962-03-02 10:54:00','suborbital rocket','success',839,'NASA 10.101CA','GCA Release 18 Na','','','Aeronomy',1821,'',134 from dual union all
select 9037,'1962-S560','1962-12-13 19:45:00','suborbital rocket','success',217,'AA-3-6','BB3 06','','','Test',1821,'',90 from dual union all
select 9038,'1962-S561','1962-12-13 11:14:32','military missile','success',982,'T 22500-14LU','8K64U UBP-11','8F116','','Test',566,'',1210 from dual union all
select 9039,'1962-S562','1962-12-13 00:00:00','suborbital rocket','unknown',1329,'','Vista-300','','','Test',1016,'',150 from dual union all
select 904,'1950-A26','1950-06-06 00:00:00','atmospheric rocket','success',442,'','F23 Ramjet','','','Test',1821,'',48 from dual union all
select 9040,'1962-S563','1962-12-15 03:00:10','military missile','success',795,'423','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 9041,'1962-S564','1962-12-14 00:00:00','military missile','success',976,'','8K63 PRO-8','','','Test',942,'',402 from dual union all
select 9042,'1962-S565','1962-12-14 20:52:00','suborbital rocket','success',839,'Ferdinand 2','Ferdinand 2','','','Ionos',29,'',123 from dual union all
select 9043,'1962-S566','1962-12-14 23:52:00','suborbital rocket','success',549,'AC20.458','Firefly III OLGA','','','Aeron/Ionos',391,'',90 from dual union all
select 9044,'1962-S567','1962-12-15 17:30:00','suborbital rocket','success',30,'AC3.451  (SF6)','Firefly III MARTHA','','','Ionos',390,'',222 from dual union all
select 9045,'1962-S568','1962-12-16 23:30:26','military missile','success',982,'T 22500-17LU','8K64U UBP-12','8F116K','','Test',566,'',1210 from dual union all
select 9046,'1962-S569','1962-12-18 05:03:00','suborbital rocket','success',665,'K-8-11','','','','Cosmic rays',839,'',202 from dual union all
select 9047,'1962-S57','1962-03-02 11:15:00','suborbital rocket','success',839,'NASA 10.39GA','GSFC Grenade 18','','','Aeronomy',1821,'',115 from dual union all
select 9048,'1962-S570','1962-12-18 19:45:00','suborbital rocket','success',30,'AC3.139','EUV Mon/RPA','','','Solar EUV',1869,'',224 from dual union all
select 9049,'1962-S571','1962-12-19 07:33:44','military missile','success',929,'A2X-24 (A2G-4)','Mk 2 guidance','','','Test',244,'',1000 from dual union all
select 905,'1950-A27','1950-07-18 00:00:00','atmospheric rocket','success',565,'','F25 Model','','','Test',1821,'',5 from dual union all
select 9050,'1962-S572','1962-12-19 00:00:00','military missile','success',976,'','8K63 PRO-9','','','Test',942,'',402 from dual union all
select 9051,'1962-S573','1962-12-19 00:00:00','ballistic missile test','success',248,'21-1','Ion Test A (SWC-16)','','','Tech',1015,'',2000 from dual union all
select 9052,'1962-S574','1962-12-19 21:51:00','military missile','success',871,'ZM-19','ZM-19/Eniwetok','','','Test',1057,'',80 from dual union all
select 9053,'1962-S575','1962-12-19 20:08:28','military missile','success',1420,'N-13','Mk 6 RV','','','Test',230,'',1300 from dual union all
select 9054,'1962-S576','1962-12-21 00:00:00','military missile','success',976,'','8K63 PRO-10','','','Test',942,'',402 from dual union all
select 9055,'1962-S577','1962-12-22 09:29:26','ballistic missile test','success',154,'160D','Nike TV-33/Pod 4','NTMP K-7','FLY HIGH','ABM Test/Plume',1662,'',2259 from dual union all
select 9056,'1962-S578','1962-12-22 10:00:00','military missile','success',871,'ZK-10 20068','K-7 Atlas Intercept','','','Test',873,'',70 from dual union all
select 9057,'1962-S579','1962-12-25 00:00:00','military missile','success',976,'','8K63 PRO-11','','','Test',942,'',402 from dual union all
select 9058,'1962-S58','1962-03-03 00:00:00','military missile','success',982,'T 225   01-15T','8K64U K 1 (Kontrol''niy)','','','Test',566,'',1210 from dual union all
select 9059,'1962-S580','1962-12-26 00:00:00','military missile','success',980,'','8K65U UBP-1 68 RB ShPU','','','Test',636,'',675 from dual union all
select 906,'1950-A28','1950-08-01 00:00:00','atmospheric rocket','success',367,'','Hermes Ramjet Model','','','Test',1821,'',10 from dual union all
select 9060,'1962-S583','1962-12-22 14:15:00','military missile','success',1192,'20032','GL.2','','','Test',21,'B-52',580 from dual union all
select 9061,'1962-S584','1962-07-09 08:51:39','suborbital rocket','success',1286,'Project 6.7 SWIK','STARFISH Diag P-4','','','Diagnostic',791,'',820 from dual union all
select 9062,'1962-S585','1962-07-09 08:52:27','suborbital rocket','success',1286,'Project 6.7 SWIK','STARFISH Diag P-2','','','Diagnostic',774,'',1100 from dual union all
select 9063,'1962-S586','1962-07-09 08:55:43','suborbital rocket','success',1286,'Project 6.7 SWIK','STARFISH Diag P-7','','','Diagnostic',785,'',1200 from dual union all
select 9064,'1962-S587','1962-07-09 08:57:29','suborbital rocket','success',1286,'Project 6.7 SWIK','STARFISH Diag P-6','','','Diagnostic',790,'',1030 from dual union all
select 9065,'1962-S588','1962-07-09 08:57:49','suborbital rocket','success',1286,'Project 6.7 SWIK','STARFISH Diag P-3','','','Diagnostic',789,'',900 from dual union all
select 9066,'1962-S589','1962-10-20 08:31:00','suborbital rocket','success',1286,'Project 6.7 SWIK','CHECKMATE Diag P-1','','','Diagnostic',803,'',300 from dual;
insert into LAUNCH
select 9067,'1962-S59','1962-03-05 00:00:00','military missile','success',871,'ZK-2','KMR-2 Sim target','','','Test',873,'',80 from dual union all
select 9068,'1962-S590','1962-10-20 08:32:00','suborbital rocket','success',1286,'Project 6.7 SWIK','CHECKMATE Diag P-5','','','Diagnostic',803,'',300 from dual union all
select 9069,'1962-S591','1962-07-09 08:56:44','suborbital rocket','success',837,'SKI-153','STARFISH X-ray','','','Diagnostic',855,'',167 from dual union all
select 907,'1950-A29','1950-08-01 00:00:00','atmospheric rocket','failure',367,'','Hermes Ramjet Model','','','Test',1821,'',10 from dual union all
select 9070,'1962-S592','1962-07-09 08:56:44','suborbital rocket','success',837,'SKI-111','STARFISH Neutron','','','Diagnostic',855,'',157 from dual union all
select 9071,'1962-S593','1962-07-09 08:56:44','suborbital rocket','success',837,'SKI-112','STARFISH Neutron','','','Diagnostic',855,'',157 from dual union all
select 9072,'1962-S594','1962-07-09 08:56:44','suborbital rocket','success',837,'SKI-115','STARFISH Alpha','','','Diagnostic',855,'',156 from dual union all
select 9073,'1962-S595','1962-07-09 08:56:44','suborbital rocket','success',837,'SKI-131','STARFISH Neutron','','','Diagnostic',855,'',157 from dual union all
select 9074,'1962-S596','1962-07-09 08:56:44','suborbital rocket','success',837,'SKI-137','STARFISH X-ray','','','Diagnostic',855,'',156 from dual union all
select 9075,'1962-S597','1962-07-09 08:56:44','suborbital rocket','success',837,'SKI-151','STARFISH Timing','','','Diagnostic',855,'',157 from dual union all
select 9076,'1962-S598','1962-07-09 08:56:44','suborbital rocket','success',837,'SKI-154','STARFISH Neut-Gamma','','','Diagnostic',855,'',159 from dual union all
select 9077,'1962-S599','1962-07-09 08:56:54','suborbital rocket','success',837,'SKI-113','STARFISH X-gamma','','','Diagnostic',855,'',159 from dual union all
select 9078,'1962-S60','1962-03-05 09:55:00','suborbital rocket','success',1194,'SL66','UCL GC-16/QUB Na-8','','','Aeronomy',1839,'',233 from dual union all
select 9079,'1962-S600','1962-07-09 08:56:54','suborbital rocket','success',837,'SKI-114','STARFISH X-gamma','','','Diagnostic',855,'',159 from dual union all
select 908,'1950-A30','1950-08-02 00:00:00','suborbital rocket','failure',1496,'R1','','','','Test',1439,'',2 from dual union all
select 9080,'1962-S601','1962-07-09 08:56:54','suborbital rocket','success',837,'SKI-133','STARFISH X-ray','','','Diagnostic',855,'',155 from dual union all
select 9081,'1962-S602','1962-07-09 08:56:54','suborbital rocket','success',837,'SKI-138','STARFISH X-ray','','','Diagnostic',855,'',155 from dual union all
select 9082,'1962-S603','1962-07-09 08:57:00','suborbital rocket','success',837,'SKI-116','STARFISH Alpha','','','Diagnostic',855,'',157 from dual union all
select 9083,'1962-S604','1962-07-09 08:57:00','suborbital rocket','success',837,'SKI-134','STARFISH X-ray','','','Diagnostic',855,'',155 from dual union all
select 9084,'1962-S605','1962-07-09 08:57:00','suborbital rocket','success',837,'SKI-152','STARFISH Timing','','','Diagnostic',855,'',157 from dual union all
select 9085,'1962-S606','1962-07-09 08:57:05','suborbital rocket','success',837,'SKI-155','STARFISH Neut-Gamma','','','Diagnostic',855,'',144 from dual union all
select 9086,'1962-S607','1962-07-09 08:57:05','suborbital rocket','success',837,'SKI-157','STARFISH X-ray','','','Diagnostic',855,'',143 from dual union all
select 9087,'1962-S608','1962-07-09 08:57:14','suborbital rocket','success',837,'SKI-156','STARFISH Gamma','','','Diagnostic',855,'',139 from dual union all
select 9088,'1962-S609','1962-07-09 08:59:20','suborbital rocket','success',837,'SKI-132','STARFISH Neutron','','','Diagnostic',855,'',157 from dual union all
select 9089,'1962-S61','1962-03-06 00:00:00','military missile','success',976,'','8K63 UBP-176','','','Test',636,'',402 from dual union all
select 909,'1950-A31','1950-08-03 23:52:00','suborbital rocket','failure',54,'NRL 6','Sunfollower/SX','','','Solar UV,X',1869,'',6 from dual union all
select 9090,'1962-S610','1962-07-09 08:59:20','suborbital rocket','success',837,'SKI-135','STARFISH Gamma','','','Diagnostic',855,'',159 from dual union all
select 9091,'1962-S611','1962-07-09 08:59:20','suborbital rocket','success',837,'SKI-136','STARFISH Gamma','','','Diagnostic',855,'',159 from dual union all
select 9092,'1962-S612','1962-07-10 15:49:00','suborbital rocket','success',839,'Project 9.1 Na','STARFISH Na','','','Aeronomy',787,'',137 from dual union all
select 9093,'1962-S613','1962-07-09 09:03:40','suborbital rocket','success',837,'Project 6.13','STARFISH Speedball','','','Diagnostic',794,'',230 from dual union all
select 9094,'1962-S614','1962-07-09 09:11:50','suborbital rocket','success',837,'Project 6.13','STARFISH Speedball','','','Diagnostic',792,'',230 from dual union all
select 9095,'1962-S615','1962-07-09 09:31:00','suborbital rocket','success',837,'Project 6.13','STARFISH Speedball','','','Diagnostic',793,'',230 from dual union all
select 9096,'1962-S616','1962-07-09 08:56:34','suborbital rocket','success',837,'SJI-152','STARFISH X-ray','','','Diagnostic',779,'',172 from dual union all
select 9097,'1962-S617','1962-07-09 08:56:39','suborbital rocket','success',837,'SJI-152','STARFISH X-ray/Beta','','','Diagnostic',775,'',159 from dual union all
select 9098,'1962-S618','1962-07-09 08:56:40','suborbital rocket','success',837,'SJI-111','STARFISH X-ray/RCS','','','Diagnostic',798,'',159 from dual union all
select 9099,'1962-S619','1962-07-09 08:56:41','suborbital rocket','success',837,'SJI-151','STARFISH X-ray/RCS','','','Diagnostic',800,'',159 from dual union all
select 91,'1944-A55','1944-12-01 00:00:00','military missile','success',943,'22','','','','Test',764,'',5 from dual union all
select 910,'1950-A32','1950-08-04 00:00:00','suborbital rocket','success',1496,'R2','','','','Test',1439,'',0 from dual union all
select 9100,'1962-S62','1962-03-06 12:35:00','suborbital rocket','success',1194,'SL160','UCL GC-17','','','Aeronomy',1839,'',173 from dual union all
select 9101,'1962-S620','1962-07-09 08:56:42','suborbital rocket','success',837,'SJI-112','STARFISH X-ray/RCS','','','Diagnostic',799,'',159 from dual union all
select 9102,'1962-S621','1962-07-09 08:57:48','suborbital rocket','success',837,'SJS-151','STARFISH RCS','','','Diagnostic',778,'',176 from dual union all
select 9103,'1962-S622','1962-07-09 09:59:00','suborbital rocket','success',837,'SJS-152','STARFISH RCS','','','Diagnostic',776,'',137 from dual union all
select 9104,'1962-S623','1962-07-09 09:16:00','suborbital rocket','success',624,'Project 6.4','STARFISH Diag.','','','Ionos',782,'',500 from dual union all
select 9105,'1962-S624','1962-07-09 09:20:12','suborbital rocket','success',624,'Project 6.2','STARFISH Diag.','','','Ionos',796,'',640 from dual union all
select 9106,'1962-S625','1962-07-09 09:40:00','suborbital rocket','success',624,'Project 6.2','STARFISH Diag.','','','Ionos',795,'',640 from dual union all
select 9107,'1962-S626','1962-09-25 14:09:00','military missile','failure',1192,'20028','PL.4','','','Test',21,'B-52',100 from dual union all
select 9108,'1962-S627','1962-02-01 00:00:00','military missile','success',837,'','SPEEDBALL II 11','','','Target',1895,'',150 from dual union all
select 9109,'1962-S628','1962-02-06 00:00:00','military missile','success',837,'','SPEEDBALL SK-6','','','Target',871,'',150 from dual union all
select 911,'1950-A33','1950-08-18 00:00:00','atmospheric rocket','success',367,'','D4','','','Test',1821,'',10 from dual union all
select 9110,'1962-S629','1962-03-02 00:00:00','military missile','success',837,'','SPEEDBALL SK-7','','','Target',871,'',150 from dual union all
select 9111,'1962-S63','1962-03-06 13:30:00','suborbital rocket','success',30,'AB3.150','UV Absorption','','','Aeron',1869,'',323 from dual union all
select 9112,'1962-S630','1962-03-12 00:00:00','military missile','success',837,'','SPEEDBALL SK-8','','','Target',871,'',150 from dual union all
select 9113,'1962-S631','1962-03-12 00:00:00','military missile','success',837,'','SPEEDBALL SK-9','','','Target',871,'',150 from dual union all
select 9114,'1962-S632','1962-03-15 00:00:00','military missile','success',837,'','SPEEDBALL SK-10','','','Target',871,'',150 from dual union all
select 9115,'1962-S633','1962-03-20 00:00:00','military missile','success',837,'','SPEEDBALL SK-11','','','Target',871,'',150 from dual union all
select 9116,'1962-S634','1962-03-21 00:00:00','military missile','success',837,'','SPEEDBALL SK-12','','','Target',871,'',150 from dual union all
select 9117,'1962-S635','1962-11-01 11:15:00','military missile','success',976,'','8K63 K-5','Nuclear Test K-5','','Nuclear Test',942,'',400 from dual union all
select 9118,'1962-S636','1962-11-21 00:00:00','suborbital rocket','success',285,'Cedar 3','','','','Test',356,'',40 from dual union all
select 9119,'1962-S637','1962-12-08 21:20:00','military missile','failure',871,'ZM-18','ZM-18','','','Test',1057,'',50 from dual union all
select 912,'1950-A34','1950-08-24 00:00:00','atmospheric rocket','success',564,'','D28 Model','','','Test',1821,'',5 from dual union all
select 9120,'1962-S638','1962-06-23 00:00:00','suborbital rocket','unknown',1350,'HITAB-BWSS-02','','','','IR background',394,'',140 from dual union all
select 9121,'1962-S639','1962-09-22 00:00:00','suborbital rocket','success',1268,'Cert Round','','','','Test',803,'',100 from dual union all
select 9122,'1962-S64','1962-03-08 15:23:51','military missile','success',794,'409','Mk 5 RV','R&D','','Test',264,'',900 from dual union all
select 9123,'1962-S640','1962-09-30 00:00:00','suborbital rocket','success',1268,'Test Round','','','','Test',803,'',100 from dual union all
select 9124,'1962-S641','1962-10-01 00:00:00','suborbital rocket','success',1268,'Test Round','','','','Test',803,'',100 from dual union all
select 9125,'1962-S642','1962-11-15 00:00:00','military missile','success',979,'','8K65 68 RB','Climate test','','Test',3,'',675 from dual union all
select 9126,'1962-S643','1962-09-05 00:00:00','military missile','success',979,'','8K65U 29 RD','Tyul''pan','','Test',1911,'',675 from dual union all
select 9127,'1962-S644','1962-09-05 00:00:00','military missile','success',979,'','8K65U 29 RD','Tyul''pan','','Test',1911,'',675 from dual union all
select 9128,'1962-S645','1962-09-08 00:00:00','military missile','success',979,'','8K65U 29 RD','Tyul''pan','','Test',1911,'',675 from dual union all
select 9129,'1962-S646','1962-10-22 00:00:00','military missile','success',976,'','8K63 K-3 PRO','','','Target?',942,'',400 from dual union all
select 913,'1950-A35','1950-08-31 00:00:00','atmospheric rocket','success',442,'','XF-92 Model','','','Test',1821,'',10 from dual union all
select 9130,'1962-S647','1962-10-28 00:00:00','military missile','success',976,'','8K63 K-4 PRO','','','Target?',942,'',400 from dual union all
select 9131,'1962-S648','1962-09-01 00:00:00','military missile','success',976,'','8K63 60 RD UBP-2','','','Test',1354,'',402 from dual union all
select 9132,'1962-S649','1962-09-01 00:00:00','military missile','success',976,'','8K63 60 RD UBP-3','','','Test',1354,'',402 from dual union all
select 9133,'1962-S65','1962-03-13 00:00:00','military missile','success',871,'ZW-39','Sim target','','','Test',1884,'',50 from dual union all
select 9134,'1962-S650','1962-12-27 00:00:00','military missile','success',977,'','8K63U UBP 51 RB','','','Test',635,'',402 from dual union all
select 9135,'1962-S651','1962-12-29 00:00:00','military missile','success',977,'','8K63U UBP 51 RB','','','Test',635,'',402 from dual union all
select 9136,'1962-S652','1962-03-01 00:00:00','military missile','success',979,'','8K65 UBP-18 43 RD','','','Test',636,'',675 from dual union all
select 9137,'1962-S653','1962-03-01 00:00:00','military missile','success',979,'','8K65 UBP-19 43 RD','','','Test',636,'',675 from dual union all
select 9138,'1962-S654','1962-11-01 00:00:00','military missile','success',976,'','8K63 K-5 PRO','','','Target?',942,'',400 from dual union all
select 9139,'1962-S66','1962-03-13 00:00:00','military missile','success',871,'ZW-40','Sim target','','','Test',1884,'',50 from dual union all
select 914,'1950-A36','1950-09-08 00:00:00','atmospheric rocket','success',365,'','E17 Drag Model','','','Test',1821,'',10 from dual union all
select 9140,'1962-S67','1962-03-14 15:42:00','suborbital rocket','success',1329,'','Vista-300','','','Test/Comms',1016,'',150 from dual union all
select 9141,'1962-S68','1962-03-15 23:31:50','military missile','success',916,'328','P-32','','','Test',256,'',180 from dual union all
select 9142,'1962-S69','1962-03-16 00:00:00','suborbital rocket','success',839,'','IR atmosphere','','','Aeron?',394,'',100 from dual union all
select 9143,'1962-S70','1962-03-16 18:08:58','military missile','success',1420,'N-2','Mk 6 RV','','','Test',231,'',1300 from dual union all
select 9144,'1962-S71','1962-03-19 11:04:00','suborbital rocket','success',1194,'SL161','UCL GC-18','','','Aeronomy',1839,'',140 from dual union all
select 9145,'1962-S72','1962-03-19 23:28:00','military missile','success',1367,'229','CTL','','BLACK KNIFE','OT',1695,'',520 from dual union all
select 9146,'1962-S73','1962-03-22 20:26:12','military missile','success',1018,'E 103   16','8K75 LKI 2-2','','','OT',569,'',1160 from dual union all
select 9147,'1962-S74','1962-03-23 01:30:02','military missile','success',794,'411','Mk 5 RV','R&D','','Test',266,'',900 from dual union all
select 9148,'1962-S75','1962-03-23 23:44:00','suborbital rocket','success',839,'NASA 10.102CA','GCA Release 19 Na','','','Aeronomy',1821,'',142 from dual union all
select 9149,'1962-S76','1962-03-23 23:54:00','suborbital rocket','success',839,'NASA 10.40GA','GSFC Gren-Sphere 3','','','Aeronomy',1821,'',121 from dual union all
select 915,'1950-A37','1950-09-14 00:00:00','military missile','failure',571,'2','','','','Test',1863,'',0 from dual union all
select 9150,'1962-S77','1962-03-24 00:39:32','military missile','success',154,'134D','Mk 3-3B','OT DASO/R&D Cat III','CURRY COMB I','OT',1666,'',1800 from dual union all
select 9151,'1962-S78','1962-03-27 23:48:00','suborbital rocket','success',839,'NASA 10.103CA','GCA Release 20 Na','','','Aeronomy',1821,'',117 from dual union all
select 9152,'1962-S79','1962-03-28 00:00:00','military missile','success',1009,'','8A62 PRO Target C-23','NIIP-A Target','','Target',321,'',500 from dual union all
select 9153,'1962-S80','1962-03-28 00:03:00','suborbital rocket','success',839,'NASA 10.41GA','GSFC Grenade 19','','','Aeronomy',1821,'',123 from dual union all
select 9154,'1962-S81','1962-03-29 09:28:00','suborbital rocket','success',535,'HAD 105','Sphere','','','Aeron',1835,'',125 from dual union all
select 9155,'1962-S82','1962-04-01 00:00:00','suborbital rocket','success',839,'','','','','Aeronomy',1895,'',100 from dual union all
select 9156,'1962-S83','1962-04-01 00:00:00','suborbital rocket','success',839,'','','','','Aeronomy',1895,'',100 from dual;
insert into LAUNCH
select 9157,'1962-S84','1962-04-01 00:00:00','suborbital rocket','success',839,'','','','','Aeronomy',1895,'',100 from dual union all
select 9158,'1962-S85','1962-04-01 00:00:00','military missile','success',976,'','8K63 UBP-177','','','Test',636,'',402 from dual union all
select 9159,'1962-S86','1962-04-03 04:57:00','suborbital rocket','success',1438,'TB Ij','','','','Reentry Test',1821,'',260 from dual union all
select 916,'1950-A38','1950-09-15 00:00:00','atmospheric rocket','success',564,'','D27 Model','','','Test',1821,'',5 from dual union all
select 9160,'1962-S87','1962-04-04 19:45:45','military missile','success',916,'326','P-33','','','Test',256,'',180 from dual union all
select 9161,'1962-S88','1962-04-05 00:00:00','military missile','success',871,'ZW-42','Sim target','','','Test',1884,'',50 from dual union all
select 9162,'1962-S89','1962-04-05 15:47:00','suborbital rocket','success',1194,'SL163','UCL GC-19','','','Aeronomy',1839,'',238 from dual union all
select 9163,'1962-S90','1962-04-06 00:00:00','military missile','success',976,'','8K63 UBP-178','','','Test',636,'',402 from dual union all
select 9164,'1962-S91','1962-04-08 00:00:00','military missile','success',1009,'[Spurious]','NIIP-A Target','','','Target',321,'',500 from dual union all
select 9165,'1962-S92','1962-04-10 00:00:00','military missile','success',976,'','8K63 UBP-179','','','Test',636,'',402 from dual union all
select 9166,'1962-S93','1962-04-11 00:00:00','military missile','success',1009,'','8A62 PRO Target C-24','NIIP-A Target','','Target',321,'',500 from dual union all
select 9167,'1962-S94','1962-04-12 00:00:00','military missile','success',1009,'[Spurious]','NIIP-A Target','','','Target',321,'',500 from dual union all
select 9168,'1962-S95','1962-04-12 20:09:00','military missile','success',871,'ZM-10','ZM-10 3-stage test','','','Test',1057,'',80 from dual union all
select 9169,'1962-S96','1962-04-12 01:57:26','military missile','success',154,'129D','Mk 3-3B','OT DASO','CURRY COMB II','OT',1666,'',1800 from dual union all
select 917,'1950-A39','1950-10-02 00:00:00','atmospheric rocket','success',365,'','D8 Model','','','Test',1821,'',10 from dual union all
select 9170,'1962-S97','1962-04-13 00:00:00','military missile','success',976,'','8K63 UBP-180','','','Test',636,'',402 from dual union all
select 9171,'1962-S98','1962-04-13 00:00:00','military missile','success',976,'','8K63 UBP-181','','','Test',636,'',402 from dual union all
select 9172,'1962-S99','1962-04-14 00:00:00','military missile','success',976,'','8K63 UBP-182','','','Test',636,'',402 from dual union all
select 9173,'1962-U01','1962-03-29 07:27:00','ballistic missile test','success',1140,'ST-9','P-21A','ST-9','','Plasma/Aeron',1812,'',6291 from dual union all
select 9174,'1962-U02','1962-05-06 05:41:00','suborbital rocket','success',1439,'TB IIb','Meteor 1','Meteor 1 pellet','','Reentry Test',1821,'',280 from dual union all
select 9175,'1962-U03','1962-08-31 16:25:00','test rocket','success',1145,'S114','Reentry 2','Reentry 2','','Reentry Test',1812,'',217 from dual union all
select 9176,'1962-W01','1962-01-01 00:00:00','suborbital rocket','success',1188,'','Test','','','Test/Aeron',1447,'',70 from dual union all
select 9177,'1962-W02','1962-01-21 00:00:00','suborbital rocket','success',88,'','','','','Aeron',194,'',60 from dual union all
select 9178,'1962-W03','1962-01-21 00:00:00','suborbital rocket','success',88,'','','','','Aeron',194,'',60 from dual union all
select 9179,'1962-W04','1962-03-23 21:25:00','suborbital rocket','unknown',88,'','Robin','','','Aeron',1821,'',60 from dual union all
select 918,'1950-A40','1950-10-02 00:00:00','atmospheric rocket','failure',564,'','D35 Model','','','Test',1821,'',5 from dual union all
select 9180,'1962-W05','1962-03-23 22:55:00','suborbital rocket','unknown',88,'','Arcasonde','','','Aeron',1821,'',60 from dual union all
select 9181,'1962-W06','1962-04-16 12:59:59','suborbital rocket','success',725,'','','','','Meteo',1010,'',50 from dual union all
select 9182,'1962-W07','1962-06-07 02:41:00','suborbital rocket','unknown',88,'','Arcasonde','','','Aeron',1821,'',60 from dual union all
select 9183,'1962-W08','1962-06-07 03:29:00','suborbital rocket','unknown',88,'','Robin','','','Aeron',1821,'',60 from dual union all
select 9184,'1962-W09','1962-06-30 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',70 from dual union all
select 9185,'1962-W10','1962-07-02 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',70 from dual union all
select 9186,'1962-W11','1962-07-05 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',70 from dual union all
select 9187,'1962-W12','1962-07-06 00:00:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',70 from dual union all
select 9188,'1962-W13','1962-07-21 05:55:00','suborbital rocket','success',88,'','McMurdo 2','','','Aeron',950,'',51 from dual union all
select 9189,'1962-W14','1962-07-26 03:03:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',66 from dual union all
select 919,'1950-A41','1950-10-10 00:00:00','atmospheric rocket','success',565,'','F25 Drag Model','','','Test',1821,'',5 from dual union all
select 9190,'1962-W15','1962-07-26 05:09:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',61 from dual union all
select 9191,'1962-W16','1962-07-26 06:09:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',59 from dual union all
select 9192,'1962-W17','1962-07-27 09:11:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',66 from dual union all
select 9193,'1962-W18','1962-07-27 09:39:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',66 from dual union all
select 9194,'1962-W19','1962-07-27 10:37:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',68 from dual union all
select 9195,'1962-W20','1962-07-27 11:45:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',66 from dual union all
select 9196,'1962-W21','1962-07-30 11:43:00','suborbital rocket','success',88,'','','','','Aeron',394,'',67 from dual union all
select 9197,'1962-W22','1962-07-30 13:41:00','suborbital rocket','success',88,'','','','','Aeron',394,'',67 from dual union all
select 9198,'1962-W23','1962-08-06 17:44:00','suborbital rocket','success',88,'','','','','Aeron',394,'',60 from dual union all
select 9199,'1962-W24','1962-08-06 18:21:00','suborbital rocket','success',88,'','','','','Aeron',394,'',62 from dual union all
select 92,'1944-A56','1944-12-01 00:00:00','military missile','success',943,'23','','','','Test',764,'',5 from dual union all
select 920,'1950-A42','1950-10-17 00:00:00','atmospheric rocket','success',869,'490-57','Nike 490A 50R','','','Test',1895,'',10 from dual union all
select 9200,'1962-W25','1962-08-06 20:20:00','suborbital rocket','success',88,'','','','','Aeron',394,'',60 from dual union all
select 9201,'1962-W26','1962-09-11 17:32:00','suborbital rocket','success',88,'','McMurdo 4','','','Aeron',950,'',63 from dual union all
select 9202,'1962-W27','1962-09-21 03:14:00','suborbital rocket','success',88,'','McMurdo 5','','','Aeron',950,'',61 from dual union all
select 9203,'1962-W28','1962-09-24 17:11:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',69 from dual union all
select 9204,'1962-W29','1962-09-24 17:50:00','suborbital rocket','success',88,'','','','','Aeron',1895,'',69 from dual union all
select 9205,'1962-W30','1962-09-27 04:19:00','suborbital rocket','success',88,'','McMurdo 6','','','Aeron',950,'',69 from dual union all
select 9206,'1962-W31','1962-10-30 00:00:00','suborbital rocket','success',88,'Range test','Robin','','','Aeron',428,'',64 from dual union all
select 9207,'1962-W32','1962-10-30 00:00:00','suborbital rocket','success',88,'','Robin','','','Aeron',428,'',64 from dual union all
select 9208,'1962-W33','1962-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 9209,'1962-W34','1962-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 921,'1950-A43','1950-10-24 00:00:00','atmospheric rocket','success',869,'490-59','Nike 490A 51R','','','Test',1895,'',10 from dual union all
select 9210,'1962-W35','1962-11-01 00:00:00','suborbital rocket','success',88,'','Arcasonde','','','Aeron',428,'',60 from dual union all
select 9211,'1962-W36','1962-10-03 03:34:00','suborbital rocket','success',88,'','McMurdo 7','','','Aeron',950,'',66 from dual union all
select 9212,'1962-W37','1962-10-12 21:01:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',67 from dual union all
select 9213,'1962-W38','1962-10-12 21:32:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',61 from dual union all
select 9214,'1962-W39','1962-10-12 22:38:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',65 from dual union all
select 9215,'1962-W40','1962-10-12 23:09:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',61 from dual union all
select 9216,'1962-W41','1962-10-13 00:13:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',60 from dual union all
select 9217,'1962-W42','1962-10-13 01:31:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',68 from dual union all
select 9218,'1962-W43','1962-10-13 01:36:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',65 from dual union all
select 9219,'1962-W44','1962-10-13 02:14:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',65 from dual union all
select 922,'1950-A44','1950-10-25 00:00:00','military missile','success',703,'225','','','','Test',222,'',8 from dual union all
select 9220,'1962-W45','1962-10-13 02:44:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',63 from dual union all
select 9221,'1962-W46','1962-10-13 03:35:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',64 from dual union all
select 9222,'1962-W47','1962-10-13 03:52:00','suborbital rocket','unknown',716,'','','','','Aeron',394,'',62 from dual union all
select 9223,'1962-W48','1962-11-07 16:35:00','suborbital rocket','success',88,'','McMurdo 10 (MCM 30)','','','Aeron',950,'',60 from dual union all
select 9224,'1962-W49','1962-11-28 03:14:00','suborbital rocket','success',88,'','McMurdo 12','','','Aeron',950,'',60 from dual union all
select 9225,'1962-W50','1962-12-05 05:30:00','suborbital rocket','success',88,'','McMurdo 13','','','Aeron',950,'',60 from dual union all
select 9226,'1962-W51','1962-12-07 00:00:00','suborbital rocket','failure',734,'MD-01/5 (105)','','','','Aeron',671,'',21 from dual union all
select 9227,'1962-W52','1962-12-07 00:00:00','suborbital rocket','failure',734,'MD-01/6 (106)','','','','Aeron',671,'',17 from dual union all
select 9228,'1962-W53','1962-12-12 04:17:00','suborbital rocket','success',88,'','McMurdo 14','','','Aeron',950,'',60 from dual union all
select 9229,'1962-W54','1962-12-21 03:49:00','suborbital rocket','success',88,'','McMurdo 15','','','Aeron',950,'',60 from dual union all
select 923,'1950-A45','1950-10-26 00:00:00','military missile','failure',703,'226','','','','Test',222,'',0 from dual union all
select 9230,'1962-W55','1962-11-03 04:20:00','suborbital rocket','success',88,'','McMurdo 9','','','Aeron',950,'',60 from dual union all
select 9231,'1962-W56','1962-11-15 05:47:00','suborbital rocket','success',88,'','McMurdo 11','','','Aeron',950,'',60 from dual union all
select 9232,'1963-001','1963-01-04 08:49:00','orbital','failure',800,'T103-09','','E-6 No. 2','','Sat',480,'',226 from dual union all
select 9233,'1963-002','1963-01-07 21:09:49','orbital','success',1362,'369/FTV 1157','CORONA 59/9051','CORONA M-18','CANDY KISSES','Sat TAG63',1673,'',null from dual union all
select 9234,'1963-003','1963-01-16 21:59:00','orbital','success',1361,'363/FTV 2313','OPS 0180','Ferret 3','CIRCUS BOY','Sat TAG64',1679,'',null from dual union all
select 9235,'1963-004','1963-02-14 05:35:08','orbital','success',1377,'358/D16','Syncom 1','Syncom 1','','Sat',234,'',null from dual union all
select 9236,'1963-005','1963-02-19 16:52:00','orbital','success',1147,'S126','OPS 0240 (AF-3)','P35-3','BLUE SCOUT V','Sat',1018,'',null from dual union all
select 9237,'1963-006','1963-03-21 08:30:01','orbital','success',1509,'T 15000- 01','Kosmos-13','Zenit-2 No. 9','','Sat',480,'',null from dual union all
select 9238,'1963-007','1963-04-01 23:01:00','orbital','success',1362,'376/FTV 1160','CORONA 62/9053','CORONA M-19','NICKEL STEEL','Sat TAG67',1679,'',null from dual union all
select 9239,'1963-008','1963-04-02 08:16:37','deep space','success',800,'G103-11','Luna-4','E-6 No. 4','','Sat',480,'',null from dual union all
select 924,'1950-A46','1950-10-26 23:02:00','suborbital rocket','failure',1470,'61','','','','Ionos/Meteorites',1863,'',8 from dual union all
select 9240,'1963-009','1963-04-03 02:00:02','orbital','success',1377,'357/D17','Explorer 17','AE-A','','Sat',233,'',null from dual union all
select 9241,'1963-010','1963-04-13 11:00:00','orbital','success',681,'','Kosmos-14','Omega-1 No. 1','','Sat',630,'',null from dual union all
select 9242,'1963-011','1963-04-22 08:30:00','orbital','success',1509,'T 15000- 08','Kosmos-15','Zenit-2 No. 8','','Sat',480,'',null from dual union all
select 9243,'1963-012','1963-04-28 08:49:56','orbital','success',1509,'E 15000- 02','Kosmos-16','Zenit-2 No. 10','','Sat',480,'',null from dual union all
select 9244,'1963-013','1963-05-07 11:38:03','orbital','success',1377,'366/D18','Telstar 2','Telstar 2','','Sat',234,'',null from dual union all
select 9245,'1963-014','1963-05-09 20:06:16','orbital','success',145,'119D','Midas 7','Midas ETS-III F2','DAMP CLAY','Sat',1020,'',null from dual union all
select 9246,'1963-015','1963-05-15 13:04:13','orbital','success',154,'130D','MA-9','Mercury SC20','','Sat',229,'',null from dual;
insert into LAUNCH
select 9247,'1963-016','1963-05-18 22:21:36','orbital','success',1394,'364/FTV 1165','CORONA 64/LANYARD 2','LANYARD 2','GATE LATCH','Sat TAG69',1679,'',null from dual union all
select 9248,'1963-017','1963-05-22 03:00:00','orbital','success',681,'','Kosmos-17','DS-A1 No. 2','','Sat',630,'',null from dual union all
select 9249,'1963-018','1963-05-24 10:34:06','orbital','success',1509,'E 15000- 12','Kosmos-18','Zenit-2 No. 11','','Sat',480,'',null from dual union all
select 925,'1950-A47','1950-10-27 00:00:00','atmospheric rocket','success',869,'490-60','Nike 490A 52R','','','Test',1895,'',10 from dual union all
select 9250,'1963-019','1963-06-12 23:58:38','orbital','success',1394,'362/FTV 1161','CORONA 65/9054','CORONA M-21','GREEN CASTLE','Sat TAG70',1678,'',null from dual union all
select 9251,'1963-020','1963-06-14 11:58:58','orbital','success',1512,'E103-24','Vostok-5','Vostok-3A No. 7','','Sat',480,'',null from dual union all
select 9252,'1963-021','1963-06-15 14:29:00','orbital','failure',1362,'378/FTV 2353','POPPY 2/NRL Comp 3','SURCAL','BUSY FLY','Sat TAG71',1673,'',null from dual union all
select 9253,'1963-022','1963-06-16 01:49:52','orbital','success',1144,'S120','Transit VA-3 (N-3)','Transit VA-3','BLUE SCOUT VIII','Sat',1018,'',null from dual union all
select 9254,'1963-023','1963-06-16 09:29:52','orbital','success',1512,'E103-25','Vostok-6','Vostok-3A No. 8','','Sat',480,'',null from dual union all
select 9255,'1963-024','1963-06-19 09:50:01','orbital','success',1377,'359/D19','Tiros 7','Tiros G','','Sat',234,'',null from dual union all
select 9256,'1963-025','1963-06-27 00:37:26','orbital','success',1394,'381/FTV 1166','CORONA 66/9056','CORONA M-22','CALICO MISS','Sat TAG72',1674,'',null from dual union all
select 9257,'1963-026','1963-06-28 21:19:22','orbital','success',1148,'S113','GRS (PL-19, CRL-1)','AFCRL A','','Sat',1812,'',null from dual union all
select 9258,'1963-027','1963-06-29 22:30:00','orbital','success',1393,'380/FTV 2314','OPS 1440','Ferret 4','BABY BLUE','Sat TAG73',1679,'',null from dual union all
select 9259,'1963-028','1963-07-12 20:45:59','orbital','success',146,'201D','AFP-206','GAMBIT SV 951','FISH POOL','Sat',1021,'',null from dual union all
select 926,'1950-A48','1950-11-02 00:00:00','atmospheric rocket','success',368,'','Sparrow Model','','','Test',1821,'',10 from dual union all
select 9260,'1963-029','1963-07-19 00:00:10','orbital','success',1362,'388/FTV 1412','CORONA 67/9057','CORONA M-23','CHILI WILLIE','Sat TAG74',1673,'',null from dual union all
select 9261,'1963-030','1963-07-19 03:51:18','orbital','success',145,'75D','Midas 9','Midas ETS-III F4','DAMP DUCK','Sat',1020,'',null from dual union all
select 9262,'1963-031','1963-07-26 14:33:00','orbital','success',1377,'370/D20','Syncom 2','Syncom 2','','Sat',233,'',null from dual union all
select 9263,'1963-032','1963-07-31 00:00:26','orbital','success',1394,'382/FTV 1167','CORONA 68/LANYARD 3','LANYARD 3','BIG TALK','Sat TAG75',1674,'',null from dual union all
select 9264,'1963-033','1963-08-06 06:00:00','orbital','success',681,'','Kosmos-19','DS-P1 No. 3','','Sat',630,'',null from dual union all
select 9265,'1963-034','1963-08-25 00:29:58','orbital','success',1394,'377/FTV 1162','CORONA 69/1001','CORONA J-1','GHOST DANCE','Sat TAG76',1678,'',null from dual union all
select 9266,'1963-035','1963-08-29 20:31:04','orbital','success',1392,'394/FTV 1169','CORONA 70/9058A','ARGON 11','PELICAN PETE','Sat TAG77',1679,'',null from dual union all
select 9267,'1963-036','1963-09-06 19:30:18','orbital','success',146,'212D','AFP-206','GAMBIT SV 952','FIXED FEE','Sat',1021,'',null from dual union all
select 9268,'1963-037','1963-09-23 23:00:00','orbital','success',1394,'383/FTV 1163','CORONA 71/1002','CORONA J-2','FELLOW KING','Sat TAG78',1674,'',null from dual union all
select 9269,'1963-038','1963-09-28 20:22:00','orbital','success',1358,'375/AB013','Transit VBN-1','Transit VBN-1','DUCK TAIL','Sat',1673,'',null from dual union all
select 927,'1950-A49','1950-11-02 00:00:00','atmospheric rocket','success',869,'490-61','Nike 490A 53R','','','Test',1895,'',10 from dual union all
select 9270,'1963-039','1963-10-17 02:37:00','orbital','success',146,'197D','Vela 1A/1B','Vela 1A','','Sat',228,'',null from dual union all
select 9271,'1963-040','1963-10-18 09:29:58','orbital','success',1509,'G 15001- 01','Kosmos-20','Zenit-2 No. 13','','Sat',480,'',null from dual union all
select 9272,'1963-041','1963-10-25 18:59:27','orbital','success',146,'224D','AFP-206','GAMBIT SV 953','HAY BAILER','Sat',1021,'',null from dual union all
select 9273,'1963-042','1963-10-29 21:19:04','orbital','success',1394,'386/FTV 1601','CORONA 72/9059A','ARGON 6','MARK DOWN','Sat TAG79',1678,'',null from dual union all
select 9274,'1963-043','1963-11-01 08:56:38','orbital','success',1253,'E 15003- 02A','Polyot','I-2B No. 102','','Sat',562,'',null from dual union all
select 9275,'1963-044','1963-11-11 06:23:35','orbital','failure',802,'G 15000- 17/G10318','Kosmos-21','3MV-1 No. 1','','Sat',480,'',232 from dual union all
select 9276,'1963-045','1963-11-16 10:34:25','orbital','success',1508,'G 15000- 06','Kosmos-22','Zenit-4 No. 1','','Sat',480,'',null from dual union all
select 9277,'1963-046','1963-11-27 02:30:01','orbital','success',1378,'387/D21','Explorer 18','IMP A','','Sat',234,'',null from dual union all
select 9278,'1963-047','1963-11-27 19:03:23','orbital','success',152,'AC-2','Centaur R&D','AC-2','','Sat',268,'',null from dual union all
select 9279,'1963-048','1963-11-27 21:15:40','orbital','success',1392,'406/FTV 1172','CORONA 74/9061','CORONA M-25','DRY DUNE','Sat TAG81',1019,'',null from dual union all
select 928,'1950-A50','1950-11-07 00:00:00','atmospheric rocket','success',869,'490-58','Nike 490A 54R','','','Test',1895,'',10 from dual union all
select 9280,'1963-049','1963-12-05 21:51:00','orbital','success',1358,'385/AB015','Transit VBN-2','Transit VBN-2','LIMIT LINE','Sat',1673,'',null from dual union all
select 9281,'1963-050','1963-12-13 14:15:24','orbital','success',681,'','Kosmos-23','Omega-1 No. 2','','Sat',630,'',null from dual union all
select 9282,'1963-051','1963-12-18 21:45:30','orbital','success',146,'227D','AFP-206','GAMBIT SV 954','REST EASY','Sat',1021,'',null from dual union all
select 9283,'1963-052','1963-12-19 09:28:58','orbital','success',1509,'G 15001- 03','Kosmos-24','Zenit-2 No. 15','','Sat',480,'',null from dual union all
select 9284,'1963-053','1963-12-19 18:49:25','orbital','success',1148,'S122R','Explorer 19','AD-A','BLUE SCOUT X','Sat',1018,'',null from dual union all
select 9285,'1963-054','1963-12-21 09:30:00','orbital','success',1377,'371/D22','Tiros 8','Tiros H','','Sat',234,'',null from dual union all
select 9286,'1963-055','1963-12-21 21:45:42','orbital','success',1394,'398/FTV 1168','CORONA 75/9062','CORONA M-26','WATER SPOUT','Sat TAG82',1674,'',null from dual union all
select 9287,'1963-A01','1963-01-01 00:00:00','military missile','success',7,'5TYa','F5','','','Test',182,'',10 from dual union all
select 9288,'1963-A02','1963-01-01 00:00:00','atmospheric rocket','success',557,'','Cork reentry test','','','Test',1821,'',22 from dual union all
select 9289,'1963-A03','1963-01-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 929,'1950-A51','1950-11-09 00:00:00','atmospheric rocket','success',565,'','F25 Drag Model','','','Test',1821,'',5 from dual union all
select 9290,'1963-A04','1963-01-01 00:00:00','suborbital rocket','success',776,'','','','','Test',933,'',36 from dual union all
select 9291,'1963-A05','1963-01-03 19:58:00','atmospheric rocket','success',835,'Smoke 326','','','','Aeronomy',293,'',14 from dual union all
select 9292,'1963-A06','1963-01-03 21:00:00','atmospheric rocket','success',835,'Smoke 327','','','','Aeronomy',293,'',12 from dual union all
select 9293,'1963-A07','1963-01-04 20:00:00','atmospheric rocket','success',835,'Smoke 328','','','','Aeronomy',293,'',15 from dual union all
select 9294,'1963-A08','1963-01-07 00:00:00','military missile','unknown',19,'','HS201','','','Test',1895,'',30 from dual union all
select 9295,'1963-A09','1963-01-09 18:00:00','atmospheric rocket','success',835,'Smoke 330','','','','Aeronomy',293,'',15 from dual union all
select 9296,'1963-A10','1963-01-10 19:30:00','atmospheric rocket','success',835,'Smoke 331','','','','Aeronomy',293,'',21 from dual union all
select 9297,'1963-A100','1963-06-30 00:00:00','suborbital rocket','failure',1431,'IQSY Tomahawk','','','','Test',1057,'',10 from dual union all
select 9298,'1963-A101','1963-07-01 18:00:00','atmospheric rocket','success',835,'Smoke 363','','','','Aeronomy',293,'',17 from dual union all
select 9299,'1963-A102','1963-07-03 00:00:00','suborbital rocket','failure',866,'Sandia 152-4','Nitehawk-9','','','Test',855,'',20 from dual union all
select 93,'1944-A57','1944-12-16 00:00:00','military missile','success',943,'24','','','','Test',764,'',5 from dual union all
select 930,'1950-A52','1950-11-15 00:00:00','atmospheric rocket','success',365,'','Snark Model','','','Test',1821,'',10 from dual union all
select 9300,'1963-A103','1963-07-03 00:00:00','suborbital spaceplane','failure',1520,'1- A-58','','','','Tech/Test',379,'NB-52',14 from dual union all
select 9301,'1963-A104','1963-07-17 02:30:02','military missile','failure',795,'430','Mk 11 RV','R&D','','Test',266,'',10 from dual union all
select 9302,'1963-A105','1963-07-17 13:59:59','military missile','success',764,'59-4981','','','','Test',240,'',12 from dual union all
select 9303,'1963-A106','1963-07-17 00:00:00','suborbital spaceplane','failure',1520,'1- A-60','','','','Test',379,'NB-52',14 from dual union all
select 9304,'1963-A107','1963-07-17 19:30:00','atmospheric rocket','success',835,'Smoke 364','','','','Aeronomy',293,'',14 from dual union all
select 9305,'1963-A108','1963-07-18 18:07:20','suborbital spaceplane','success',1520,'1-38-61','VO Stability','','','Test',379,'NB-52 003',31 from dual union all
select 9306,'1963-A109','1963-07-20 05:44:08','test rocket','failure',1145,'S110','Reentry 3','','','Reentry Test',1812,'',2 from dual union all
select 9307,'1963-A11','1963-01-11 19:42:00','atmospheric rocket','success',835,'Smoke 332','','','','Aeronomy',293,'',16 from dual union all
select 9308,'1963-A110','1963-07-20 20:00:02','suborbital rocket','failure',837,'NASA 14.89CI','GCA Probe High 2','','','Ionos/Eclipse',428,'',5 from dual union all
select 9309,'1963-A111','1963-07-20 20:30:00','suborbital rocket','failure',837,'NASA 14.90CI','GCA Probe High 3','','','Ionos/Eclipse',428,'',5 from dual union all
select 931,'1950-A53','1950-11-16 00:00:00','atmospheric rocket','success',365,'','X-3 Model','','','Test',1821,'',10 from dual union all
select 9310,'1963-A112','1963-07-24 00:00:00','military missile','failure',1018,'T 15001-06','8K75 LKI 3-8','','','OT',584,'',0 from dual union all
select 9311,'1963-A113','1963-07-25 04:59:00','suborbital rocket','failure',1198,'SL29','RAE Test 17','','','Test',1839,'',0 from dual union all
select 9312,'1963-A114','1963-07-26 19:30:00','atmospheric rocket','success',835,'Smoke 365','','','','Aeronomy',293,'',20 from dual union all
select 9313,'1963-A115','1963-07-31 19:30:00','atmospheric rocket','success',835,'Smoke 366','','','','Aeronomy',293,'',17 from dual union all
select 9314,'1963-A116','1963-08-02 02:30:00','suborbital rocket','failure',1197,'SL86','RAE Test 18','','','Test',1839,'',0 from dual union all
select 9315,'1963-A117','1963-08-06 00:00:00','suborbital spaceplane','failure',1520,'3- A-33','','','','Test/Aeron',379,'NB-52 008',14 from dual union all
select 9316,'1963-A118','1963-08-07 19:30:00','atmospheric rocket','success',835,'Smoke 367','','','','Aeronomy',293,'',19 from dual union all
select 9317,'1963-A119','1963-08-13 00:00:00','suborbital spaceplane','failure',1520,'3- A-34','','','','Test/Aeron',379,'NB-52 008',14 from dual union all
select 9318,'1963-A12','1963-01-17 21:17:00','atmospheric rocket','success',835,'Smoke 333','','','','Aeronomy',293,'',15 from dual union all
select 9319,'1963-A120','1963-08-15 00:00:00','suborbital spaceplane','failure',1520,'3- A-35','','','','Test/Aeron',379,'NB-52 008',14 from dual union all
select 932,'1950-A54','1950-11-16 00:00:00','atmospheric rocket','success',869,'490-67','Nike 490A 55R','','','Test',1895,'',10 from dual union all
select 9320,'1963-A121','1963-08-15 23:11:00','atmospheric rocket','success',835,'Smoke 031','','','','Aeronomy',1821,'',20 from dual union all
select 9321,'1963-A122','1963-08-22 00:00:00','suborbital rocket','failure',577,'F11','','','','Test',1821,'',1 from dual union all
select 9322,'1963-A123','1963-08-28 16:00:02','test rocket','success',714,'LJII-1 (12-50-1)','QTV','','','SC Test',1873,'',8 from dual union all
select 9323,'1963-A124','1963-09-12 00:00:00','atmospheric rocket','success',545,'Cree 13','Parachute test','','','Test',394,'',30 from dual union all
select 9324,'1963-A125','1963-09-19 20:14:00','atmospheric rocket','success',835,'Smoke 032','','','','Aeronomy',1821,'',20 from dual union all
select 9325,'1963-A126','1963-09-19 21:48:00','atmospheric rocket','success',835,'Smoke 033','','','','Aeronomy',1821,'',20 from dual union all
select 9326,'1963-A127','1963-09-23 18:00:00','atmospheric rocket','success',835,'Smoke 034','','','','Aeronomy',1821,'',19 from dual union all
select 9327,'1963-A128','1963-09-28 06:00:00','military missile','destroyed before launch',999,'G 22500-02L','8K67 LKI-0','GCh 8F674 No. 2L','','Test',573,'',0 from dual union all
select 9328,'1963-A129','1963-10-11 00:00:00','military missile','success',843,'ATBM-12','','','','Test',1895,'',20 from dual union all
select 9329,'1963-A13','1963-01-18 22:17:00','atmospheric rocket','success',835,'Smoke 334','','','','Aeronomy',293,'',15 from dual union all
select 933,'1950-A55','1950-11-20 00:00:00','atmospheric rocket','success',869,'490-62','Nike 490A 56R','','','Test',1895,'',10 from dual union all
select 9330,'1963-A130','1963-10-24 00:00:00','military missile','success',843,'ATBM-13','','','','Test',1895,'',20 from dual union all
select 9331,'1963-A131','1963-10-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 9332,'1963-A132','1963-10-05 00:00:00','military missile','success',843,'ATBM-11','T-45 warhead','','','Test',1895,'',12 from dual union all
select 9333,'1963-A133','1963-10-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 9334,'1963-A134','1963-09-30 19:30:00','atmospheric rocket','success',835,'Smoke 369','','','','Aeronomy',293,'',20 from dual union all
select 9335,'1963-A135','1963-10-01 20:12:00','atmospheric rocket','success',835,'Smoke 370','','','','Aeronomy',293,'',20 from dual union all
select 9336,'1963-A136','1963-10-01 21:34:00','atmospheric rocket','success',835,'Smoke 035','','','','Aeronomy',1821,'',20 from dual;
insert into LAUNCH
select 9337,'1963-A137','1963-10-02 00:00:00','suborbital rocket','failure',866,'Sandia 152-6','Ballistic','','','Test',855,'',10 from dual union all
select 9338,'1963-A138','1963-10-04 00:00:00','suborbital spaceplane','failure',1520,'1- A-62','','','','Test/Tech',379,'NB-52',14 from dual union all
select 9339,'1963-A139','1963-10-04 05:17:49','military missile','failure',162,'45F','Mk 4-5B','R&D','HOT RUM','Test',1672,'',0 from dual union all
select 934,'1950-A56','1950-11-21 00:00:00','military missile','success',703,'227','','','','Test',222,'',8 from dual union all
select 9340,'1963-A14','1963-01-22 19:46:00','atmospheric rocket','success',835,'Smoke 335','','','','Aeronomy',293,'',15 from dual union all
select 9341,'1963-A140','1963-10-04 18:14:00','atmospheric rocket','success',835,'Smoke 036','','','','Aeronomy',1821,'',20 from dual union all
select 9342,'1963-A141','1963-10-07 20:22:56','suborbital spaceplane','success',1520,'1-39-63','Checkout/Opt Deg.','','','Test/Tech',379,'NB-52 008',23 from dual union all
select 9343,'1963-A142','1963-10-07 21:31:00','military missile','failure',154,'163D','Mk 11-5B','ST COOL WATER V','COOL WATER V','Test',1667,'',10 from dual union all
select 9344,'1963-A143','1963-10-09 20:30:00','atmospheric rocket','success',835,'Smoke 371','','','','Aeronomy',293,'',13 from dual union all
select 9345,'1963-A144','1963-10-10 10:33:00','suborbital rocket','failure',30,'NASA 04.31GG','UV geocorona','','','UV Astron',1869,'',2 from dual union all
select 9346,'1963-A145','1963-10-11 18:09:00','atmospheric rocket','success',835,'Smoke 037','','','','Aeronomy',1821,'',22 from dual union all
select 9347,'1963-A146','1963-10-14 00:00:00','suborbital spaceplane','failure',1520,'3- A-37','','','','Test/Tech',379,'NB-52',14 from dual union all
select 9348,'1963-A147','1963-10-15 19:38:00','atmospheric rocket','success',835,'Smoke 038','','','','Aeronomy',1821,'',16 from dual union all
select 9349,'1963-A148','1963-10-18 00:00:00','military missile','failure',1018,'','8K75 LKI 3-12','','','OT',579,'',0 from dual union all
select 935,'1950-A57','1950-12-01 00:00:00','atmospheric rocket','success',564,'','Rascal Model','','','Test',1821,'',5 from dual union all
select 9350,'1963-A149','1963-10-22 19:17:00','atmospheric rocket','success',835,'Smoke 372','','','','Aeronomy',293,'',16 from dual union all
select 9351,'1963-A15','1963-01-28 18:00:00','atmospheric rocket','success',835,'Smoke 336','','','','Aeronomy',293,'',16 from dual union all
select 9352,'1963-A150','1963-10-23 00:00:00','military missile','failure',916,'','P-68 63 Fall 12/W-5','','','OT',437,'',0 from dual union all
select 9353,'1963-A151','1963-10-25 00:00:00','suborbital spaceplane','failure',1520,'3- A-38','Sharp Upper Vert.','','','Test',379,'NB-52',14 from dual union all
select 9354,'1963-A152','1963-10-29 20:42:34','suborbital spaceplane','success',1520,'1-40-64','Checkout/Opt Deg.','','','Test/Tech',379,'NB-52 008',22 from dual union all
select 9355,'1963-A153','1963-10-30 19:30:00','atmospheric rocket','success',835,'Smoke 374','','','','Aeronomy',293,'',18 from dual union all
select 9356,'1963-A154','1963-11-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 9357,'1963-A155','1963-11-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 9358,'1963-A156','1963-12-09 00:00:00','military missile','success',843,'ATBM-14','Sergeant intercept','','','Test',1895,'',20 from dual union all
select 9359,'1963-A157','1963-12-18 00:00:00','military missile','success',843,'ATBM-15','Sergeant intercept','','','Test',1895,'',20 from dual union all
select 936,'1950-A58','1950-12-04 00:00:00','atmospheric rocket','success',869,'490-69','Nike 490A 57R','','','Test',1895,'',10 from dual union all
select 9360,'1963-A158','1963-11-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 9361,'1963-A159','1963-11-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 9362,'1963-A16','1963-01-29 16:27:00','suborbital rocket','failure',35,'NASA 04.73UA','JHU Airglow 4','','','Aeronomy',1821,'',4 from dual union all
select 9363,'1963-A160','1963-11-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 9364,'1963-A161','1963-10-31 20:30:00','atmospheric rocket','success',835,'Smoke 373','','','','Aeronomy',293,'',19 from dual union all
select 9365,'1963-A162','1963-11-05 10:24:25','military missile','failure',1457,'G10701-301','8K81 LKI-1','GCh 8F114','','Test',590,'',0 from dual union all
select 9366,'1963-A163','1963-11-06 19:30:00','atmospheric rocket','success',835,'Smoke 375','','','','Aeronomy',293,'',19 from dual union all
select 9367,'1963-A164','1963-11-07 17:00:02','military missile','failure',795,'434','Mk 11 RV','R&D','','Test',266,'',1 from dual union all
select 9368,'1963-A165','1963-11-07 16:00:01','test rocket','success',87,'PA-1','','','','Test',1873,'',2 from dual union all
select 9369,'1963-A166','1963-11-07 18:11:14','suborbital spaceplane','success',1520,'3-23-39','Sharp Upper Vert.','','','Test',379,'NB-52 008',25 from dual union all
select 937,'1950-A59','1950-12-07 00:00:00','atmospheric rocket','success',442,'','X-3 Model','','','Test',1821,'',10 from dual union all
select 9370,'1963-A167','1963-11-12 20:20:00','atmospheric rocket','success',835,'Smoke 039','','','','Aeronomy',1821,'',19 from dual union all
select 9371,'1963-A168','1963-11-13 20:11:00','atmospheric rocket','success',835,'Smoke 376','','','','Aeronomy',293,'',20 from dual union all
select 9372,'1963-A169','1963-11-14 19:19:21','suborbital spaceplane','success',1520,'1-41-65','CO/Opt Deg/Systems','','','Test/Tech',379,'NB-52 008',27 from dual union all
select 9373,'1963-A17','1963-02-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 9374,'1963-A170','1963-11-15 19:33:00','atmospheric rocket','success',835,'Smoke 377','','','','Aeronomy',293,'',20 from dual union all
select 9375,'1963-A171','1963-11-19 00:00:00','test rocket','failure',61,'R','','','','Test',313,'',0 from dual union all
select 9376,'1963-A172','1963-11-19 00:00:00','military missile','failure',979,'','8K65 UBP-47','','','Test',636,'',0 from dual union all
select 9377,'1963-A173','1963-11-19 00:00:00','suborbital spaceplane','failure',1520,'3- A-40','Checkout','','','Test',379,'NB-52',14 from dual union all
select 9378,'1963-A174','1963-11-19 00:05:00','suborbital rocket','failure',1199,'SL103','LUX-15/UCL I-16SUV7','','','Iono/Solar X/UCW-6/Bir-7',1839,'',0 from dual union all
select 9379,'1963-A175','1963-11-21 19:30:00','atmospheric rocket','success',835,'Smoke 378','','','','Aeronomy',293,'',19 from dual union all
select 938,'1950-A60','1950-12-07 00:00:00','atmospheric rocket','success',869,'490-63','Nike 490A 58R','','','Test',1895,'',10 from dual union all
select 9380,'1963-A176','1963-11-27 20:17:40','suborbital spaceplane','success',1520,'3-24-41','Checkout','','','Test',379,'NB-52 008',27 from dual union all
select 9381,'1963-A177','1963-11-28 00:00:00','test rocket','success',61,'R','','','','Test',313,'',20 from dual union all
select 9382,'1963-A178','1963-12-03 00:00:00','suborbital spaceplane','failure',1520,'1- A-66','','','','Test',379,'NB-52',14 from dual union all
select 9383,'1963-A179','1963-12-04 00:00:00','military missile','failure',1018,'G 15000-04 (8TS)','8K75 LKI 3-14','','','OT',584,'',0 from dual union all
select 9384,'1963-A18','1963-01-31 20:00:00','atmospheric rocket','success',835,'Smoke 337','','','','Aeronomy',293,'',15 from dual union all
select 9385,'1963-A180','1963-12-05 00:00:00','suborbital rocket','success',1158,'','','','','Test',353,'',30 from dual union all
select 9386,'1963-A181','1963-12-05 19:04:36','suborbital spaceplane','success',1520,'1-42-67','Opt Deg/Sys/Mach 6','','','Tech/Test',379,'NB-52 008',30 from dual union all
select 9387,'1963-A182','1963-12-05 20:25:00','atmospheric rocket','success',835,'Smoke 379','','','','Aeronomy',293,'',15 from dual union all
select 9388,'1963-A183','1963-12-08 05:00:00','military missile','failure',1018,'G 15001-07 (9TS)','8K75 LKI 3-15','','','OT',584,'',0 from dual union all
select 9389,'1963-A184','1963-12-12 00:00:00','military missile','failure',976,'','8K63 KY63-38','','','Test',636,'',0 from dual union all
select 939,'1950-A61','1950-12-11 00:00:00','atmospheric rocket','success',869,'490-64','Nike 490A 59R','','','Test',1895,'',10 from dual union all
select 9390,'1963-A185','1963-12-13 03:00:00','military missile','destroyed before launch',999,'G 22500-03L','8K67 LKI','','','Test',573,'',0 from dual union all
select 9391,'1963-A186','1963-12-18 00:00:00','suborbital spaceplane','failure',1520,'1- A-68','Opt Deg','','','Tech',379,'NB-52',14 from dual union all
select 9392,'1963-A187','1963-12-20 17:30:00','atmospheric rocket','success',835,'Smoke 380','','','','Aeronomy',293,'',16 from dual union all
select 9393,'1963-A188','1963-02-26 00:00:00','suborbital rocket','success',459,'Emma 02','','','','Aeron',671,'',32 from dual union all
select 9394,'1963-A189','1963-08-01 00:00:00','suborbital rocket','success',506,'','','','','Meteo',310,'',35 from dual union all
select 9395,'1963-A19','1963-01-31 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',49 from dual union all
select 9396,'1963-A190','1963-01-01 00:00:00','suborbital rocket','success',329,'','LRL Sampling','','','Aeron',1057,'B-57B',40 from dual union all
select 9397,'1963-A191','1963-03-18 00:00:00','atmospheric rocket','success',557,'','Hypervelocity test','','','Test',1821,'',24 from dual union all
select 9398,'1963-A192','1963-05-07 00:00:00','atmospheric rocket','success',557,'','Hypervelocity test','','','Test',1821,'',24 from dual union all
select 9399,'1963-A193','1963-11-18 00:00:00','military missile','success',843,'','ASF','','','Test',401,'',10 from dual union all
select 94,'1944-A58','1945-01-08 00:00:00','military missile','failure',9,'1','','','','Test',675,'',1 from dual union all
select 940,'1950-A62','1950-12-11 17:04:00','suborbital rocket','failure',60,'SC 15','Probe','','','Aeronomy',1869,'',1 from dual union all
select 9400,'1963-A196','1963-12-10 00:00:00','military missile','success',843,'','ASF','','','Test',401,'',10 from dual union all
select 9401,'1963-A198','1963-04-01 00:00:00','military missile','success',871,'ZW-59','','','','Test',1884,'',50 from dual union all
select 9402,'1963-A199','1963-04-01 00:00:00','military missile','success',871,'ZW-60','','','','Test',1884,'',50 from dual union all
select 9403,'1963-A20','1963-02-03 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',44 from dual union all
select 9404,'1963-A200','1963-04-01 00:00:00','military missile','success',871,'ZW-61','','','','Test',1884,'',50 from dual union all
select 9405,'1963-A201','1963-10-11 00:00:00','military missile','success',871,'ZW-68','','','','Test',1884,'',50 from dual union all
select 9406,'1963-A202','1963-01-01 00:00:00','suborbital rocket','failure',1050,'14','','','','Weather',1895,'',10 from dual union all
select 9407,'1963-A203','1963-01-01 00:00:00','suborbital rocket','failure',1050,'1','','','','Weather',1895,'',20 from dual union all
select 9408,'1963-A204','1963-01-01 00:00:00','suborbital rocket','failure',1050,'2','','','','Weather',1895,'',15 from dual union all
select 9409,'1963-A205','1963-01-01 00:00:00','suborbital rocket','failure',1050,'3','','','','Weather',1895,'',22 from dual union all
select 941,'1950-A63','1950-12-19 00:00:00','atmospheric rocket','success',564,'','D35 Model','','','Test',1821,'',5 from dual union all
select 9410,'1963-A206','1963-01-01 00:00:00','suborbital rocket','failure',1050,'4','','','','Weather',1895,'',40 from dual union all
select 9411,'1963-A207','1963-01-01 00:00:00','suborbital rocket','failure',1050,'11','','','','Weather',1895,'',9 from dual union all
select 9412,'1963-A208','1963-01-01 00:00:00','suborbital rocket','failure',1050,'12','','','','Weather',1895,'',12 from dual union all
select 9413,'1963-A209','1963-01-01 00:00:00','suborbital rocket','failure',1050,'13','','','','Weather',1895,'',10 from dual union all
select 9414,'1963-A21','1963-02-07 17:45:00','atmospheric rocket','success',835,'Smoke 338','','','','Aeronomy',293,'',22 from dual union all
select 9415,'1963-A210','1963-01-01 00:00:00','military missile','failure',1321,'','9K71 Phase 2 No. 11','','','Test',636,'',10 from dual union all
select 9416,'1963-A211','1963-08-01 00:00:00','suborbital rocket','success',506,'','','','','Meteo',310,'',35 from dual union all
select 9417,'1963-A212','1963-08-01 00:00:00','suborbital rocket','success',506,'','','','','Meteo',310,'',35 from dual union all
select 9418,'1963-A22','1963-02-13 18:15:00','atmospheric rocket','success',835,'Smoke 339','','','','Aeronomy',293,'',16 from dual union all
select 9419,'1963-A23','1963-02-14 00:00:00','military missile','failure',1018,'T 15000-10','8K75 LKI 3-2','','','OT',582,'',0 from dual union all
select 942,'1950-A64','1950-12-21 00:00:00','military missile','failure',763,'','','','','Test',644,'',0 from dual union all
select 9420,'1963-A24','1963-02-16 21:45:00','military missile','failure',1420,'N-7','Mk 6 RV','R&D','AWFUL TIRED','Test',1658,'',5 from dual union all
select 9421,'1963-A25','1963-02-18 14:00:02','military missile','failure',932,'A3X-9','','','','Test',251,'',10 from dual union all
select 9422,'1963-A26','1963-02-18 23:14:00','suborbital rocket','failure',838,'NASA 03.11CA','GCA Release 25 Na','','','Aeronomy',1821,'',25 from dual union all
select 9423,'1963-A27','1963-02-20 19:30:00','atmospheric rocket','success',835,'Smoke 340','','','','Aeronomy',293,'',18 from dual union all
select 9424,'1963-A28','1963-02-27 19:30:00','atmospheric rocket','success',835,'Smoke 341','','','','Aeronomy',293,'',14 from dual union all
select 9425,'1963-A29','1963-03-01 00:00:00','military missile','unknown',843,'','','','','Test',1895,'',20 from dual union all
select 9426,'1963-A30','1963-02-28 00:00:00','suborbital rocket','failure',577,'F1','','','','Test',1821,'',1 from dual;
insert into LAUNCH
select 9427,'1963-A31','1963-03-05 00:00:00','atmospheric rocket','success',1474,'','','','','Test',669,'',20 from dual union all
select 9428,'1963-A32','1963-03-07 17:48:15','military missile','failure',929,'A2P-116','','','','OT',421,'SSBN 618',1 from dual union all
select 9429,'1963-A33','1963-03-08 19:20:00','atmospheric rocket','success',835,'Smoke 342','','','','Aeronomy',293,'',20 from dual union all
select 943,'1950-A65','1950-01-23 00:00:00','atmospheric rocket','success',880,'484-35','Nike 484 34R','','','Test',1863,'',10 from dual union all
select 9430,'1963-A34','1963-03-10 02:42:00','military missile','failure',154,'102D','Mk 3 RV','OT','TALL TREE 3','OT',1667,'',0 from dual union all
select 9431,'1963-A35','1963-03-11 00:00:00','military missile','failure',1018,'E 15001-34','8K75 LKI 3-4','','','OT',584,'',0 from dual union all
select 9432,'1963-A36','1963-03-11 21:30:00','atmospheric rocket','success',835,'Smoke 343','','','','Aeronomy',293,'',14 from dual union all
select 9433,'1963-A37','1963-03-12 19:25:00','atmospheric rocket','success',835,'Smoke 344','','','','Aeronomy',293,'',14 from dual union all
select 9434,'1963-A38','1963-03-12 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',44 from dual union all
select 9435,'1963-A40','1963-03-18 19:30:00','atmospheric rocket','success',835,'Smoke 345','','','','Aeronomy',293,'',15 from dual union all
select 9436,'1963-A41','1963-03-21 19:10:00','atmospheric rocket','success',835,'Smoke 346','','','','Aeronomy',293,'',15 from dual union all
select 9437,'1963-A42','1963-03-22 18:30:00','atmospheric rocket','success',835,'Smoke 347','','','','Aeronomy',293,'',16 from dual union all
select 9438,'1963-A43','1963-03-24 00:29:14','military missile','failure',162,'52F','Mk 4 RV','OT TALL TREE 4','TALL TREE 4','OT',1670,'',10 from dual union all
select 9439,'1963-A44','1963-03-27 00:00:00','suborbital rocket','failure',577,'F2','','','','Test',1821,'',1 from dual union all
select 944,'1950-M01','1950-01-15 23:45:00','suborbital rocket','success',54,'A15','','','','Particles',191,'AVM1',72 from dual union all
select 9440,'1963-A45','1963-03-27 00:00:00','atmospheric rocket','success',620,'J8','HRV 915','','','Test',1836,'',24 from dual union all
select 9441,'1963-A46','1963-03-28 00:00:00','military missile','failure',979,'','8K65 UBP-40','','','Test',636,'',0 from dual union all
select 9442,'1963-A47','1963-03-28 18:02:00','atmospheric rocket','success',835,'Smoke 348','','','','Aeronomy',293,'',21 from dual union all
select 9443,'1963-A48','1963-03-28 20:10:00','atmospheric rocket','success',835,'Smoke 408','','','','Aeronomy',293,'',19 from dual union all
select 9444,'1963-A49','1963-04-02 19:23:00','atmospheric rocket','success',835,'Smoke 349','','','','Aeronomy',293,'',13 from dual union all
select 9445,'1963-A50','1963-04-03 19:20:00','atmospheric rocket','success',835,'Smoke 350','','','','Aeronomy',293,'',14 from dual union all
select 9446,'1963-A51','1963-04-05 19:03:00','atmospheric rocket','success',835,'Smoke 351','','','','Aeronomy',293,'',15 from dual union all
select 9447,'1963-A52','1963-04-10 20:33:07','military missile','failure',932,'A3X-14','','','','Test',421,'EAG-154',1 from dual union all
select 9448,'1963-A53','1963-04-11 18:03:20','suborbital spaceplane','success',1520,'1-33-54','KC-1/APU','','','Test/Imaging',379,'NB-52 008',22 from dual union all
select 9449,'1963-A54','1963-04-11 20:35:00','atmospheric rocket','success',835,'Smoke 352','','','','Aeronomy',293,'',15 from dual union all
select 945,'1950-M02','1950-01-18 23:17:00','suborbital rocket','success',54,'A16','','','','Particles',192,'AVM1',80 from dual union all
select 9450,'1963-A55','1963-04-12 19:25:00','atmospheric rocket','success',835,'Smoke 353','','','','Aeronomy',293,'',12 from dual union all
select 9451,'1963-A56','1963-04-12 21:00:00','suborbital rocket','success',725,'','','','','Meteo',1010,'',35 from dual union all
select 9452,'1963-A57','1963-04-15 19:30:00','atmospheric rocket','success',835,'Smoke 354','','','','Aeronomy',293,'',16 from dual union all
select 9453,'1963-A58','1963-04-16 19:30:00','atmospheric rocket','success',835,'Smoke 355','','','','Aeronomy',293,'',21 from dual union all
select 9454,'1963-A59','1963-04-17 19:30:00','atmospheric rocket','success',835,'Smoke 356','','','','Aeronomy',293,'',16 from dual union all
select 9455,'1963-A60','1963-04-18 20:16:17','suborbital spaceplane','success',1520,'3-15-25','HT,Local flow,IR','','','Test/Tech',379,'NB-52 008',28 from dual union all
select 9456,'1963-A61','1963-04-22 19:30:00','atmospheric rocket','success',835,'Smoke 357','','','','Aeronomy',293,'',12 from dual union all
select 9457,'1963-A62','1963-04-25 00:00:00','suborbital rocket','failure',577,'F3','','','','Test',1821,'',1 from dual union all
select 9458,'1963-A63','1963-04-25 19:58:00','atmospheric rocket','success',835,'Smoke 028','','','','Aeronomy',1821,'',20 from dual union all
select 9459,'1963-A64','1963-04-25 19:58:00','atmospheric rocket','success',835,'Smoke 029','','','','Aeronomy',1821,'',11 from dual union all
select 946,'1950-M03','1950-02-09 21:44:00','suborbital rocket','success',1499,'3','Viking 3','','','Solar UV,X/Photo',1852,'',80 from dual union all
select 9460,'1963-A65','1963-04-25 19:58:00','atmospheric rocket','success',835,'Smoke 030','','','','Aeronomy',1821,'',17 from dual union all
select 9461,'1963-A66','1963-04-25 22:04:19','suborbital spaceplane','success',1520,'1-34-55','KC-1','','','Imaging',379,'NB-52 008',32 from dual union all
select 9462,'1963-A67','1963-05-01 00:00:00','military missile','success',7,'5TYa','F6','','','Test',182,'',10 from dual union all
select 9463,'1963-A68','1963-05-01 10:15:00','military missile','failure',1419,'V-4','Mk 4 RV','R&D','MARES TAIL','Test',1654,'',0 from dual union all
select 9464,'1963-A69','1963-05-07 20:30:00','atmospheric rocket','success',835,'Smoke 358','','','','Aeronomy',293,'',15 from dual union all
select 9465,'1963-A70','1963-05-08 19:30:00','atmospheric rocket','success',835,'Smoke 359','','','','Aeronomy',293,'',14 from dual union all
select 9466,'1963-A71','1963-05-09 00:00:00','suborbital rocket','success',577,'F4','','','','Test',1821,'',22 from dual union all
select 9467,'1963-A72','1963-05-10 00:00:00','military missile','failure',871,'ZW-58','','','','Test',1884,'',20 from dual union all
select 9468,'1963-A73','1963-05-10 00:00:00','suborbital spaceplane','failure',1520,'3- A-27','','','','Test/UV Ast/Tech',379,'NB-52 008',14 from dual union all
select 9469,'1963-A74','1963-05-14 20:11:56','suborbital spaceplane','success',1520,'3-17-28','Heating,UVP,IR','','','Test/UV Ast/Tech',379,'NB-52 008',29 from dual union all
select 947,'1950-M04','1950-03-04 00:36:00','suborbital rocket','success',59,'SC 6','Grenades','','','Aeronomy',1869,'',72 from dual union all
select 9470,'1963-A75','1963-05-14 21:35:00','atmospheric rocket','success',835,'Smoke 360','','','','Aeronomy',293,'',17 from dual union all
select 9471,'1963-A76','1963-05-15 18:50:46','suborbital spaceplane','success',1520,'1-35-56','Opt Deg/Trav Probe','','','Test/Tech',379,'NB-52 003',37 from dual union all
select 9472,'1963-A77','1963-05-18 00:00:00','test rocket','success',61,'','','','','Test',669,'',20 from dual union all
select 9473,'1963-A78','1963-05-18 05:14:00','military missile','failure',795,'425B','Mk 11 RV','R&D','','Test',266,'',10 from dual union all
select 9474,'1963-A79','1963-05-18 14:55:00','military missile','failure',982,'G 22500-02TS','8K64U UBP-17','8F115','','OT',571,'',0 from dual union all
select 9475,'1963-A81','1963-05-21 00:00:00','test rocket','success',61,'','','','','Test',669,'',20 from dual union all
select 9476,'1963-A82','1963-05-29 16:56:24','military missile','failure',1420,'N-20','Mk 6 RV','R&D','','Test',231,'',6 from dual union all
select 9477,'1963-A83','1963-05-29 18:43:22','suborbital spaceplane','success',1520,'3-18-29','Aero heating/VO Stab','','','Test',379,'NB-52 008',28 from dual union all
select 9478,'1963-A84','1963-06-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 9479,'1963-A85','1963-06-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 948,'1950-M05','1950-05-26 19:43:00','suborbital rocket','success',50,'USAF 4','Insolation','','','Solar UV?',640,'',67 from dual union all
select 9480,'1963-A86','1963-06-02 00:26:00','military missile','destroyed before launch',162,'77F','Mk 4 RV','','','Err',1823,'',0 from dual union all
select 9481,'1963-A87','1963-06-05 16:10:00','military missile','success',764,'58-1423','','','','Test',241,'',12 from dual union all
select 9482,'1963-A88','1963-06-07 19:00:00','atmospheric rocket','success',835,'Smoke 361','','','','Aeronomy',293,'',13 from dual union all
select 9483,'1963-A89','1963-06-12 17:23:26','military missile','success',764,'58-1435','','','','Test',241,'',12 from dual union all
select 9484,'1963-A90','1963-06-13 16:30:02','military missile','failure',929,'A2P-121','','','','OT',421,'SSBN 616',100 from dual union all
select 9485,'1963-A91','1963-06-13 00:00:00','military missile','failure',1018,'G 15001-05','8K75 LKI 3-7','','','OT',584,'',0 from dual union all
select 9486,'1963-A92','1963-06-19 00:00:00','suborbital rocket','failure',1490,'AGI40','','','','Ionos',668,'',38 from dual union all
select 9487,'1963-A93','1963-06-19 20:45:00','atmospheric rocket','success',835,'Smoke 362','','','','Aeronomy',293,'',16 from dual union all
select 9488,'1963-A94','1963-06-21 04:13:00','military missile','failure',982,'T 22601-15','8K64U UBP-21 BD','','','OT',565,'',0 from dual union all
select 9489,'1963-A95','1963-06-25 17:53:50','suborbital spaceplane','success',1520,'1-36-57','Opt Deg/Trav Probe','','','Tech',379,'NB-52 003',34 from dual union all
select 949,'1950-M06','1950-07-11 00:00:00','military missile','success',340,'5E','','','','Test',1863,'',50 from dual union all
select 9490,'1963-A96','1963-06-28 14:20:40','military missile','success',764,'59-2484','','','','Test',242,'',12 from dual union all
select 9491,'1963-A97','1963-07-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 9492,'1963-A98','1963-07-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 9493,'1963-A99','1963-07-01 00:00:00','military missile','unknown',19,'','','','','Test',1895,'',30 from dual union all
select 9494,'1963-E01','1963-07-10 00:00:00','orbital','destroyed before launch',1509,'E 15000- 04','','Zenit-2 No. 12','','Sat',480,'',0 from dual union all
select 9495,'1963-E02','1963-10-24 00:00:00','military missile','destroyed before launch',1018,'','8K75','Fuelling accident','','OT',579,'',0 from dual union all
select 9496,'1963-F01','1963-02-03 09:29:14','orbital','failure',800,'G103-10','','E-6 No. 3','','Sat',480,'',220 from dual union all
select 9497,'1963-F02','1963-02-28 21:48:30','orbital','failure',1394,'354/FTV 1159','CORONA 60/9052','CORONA M-20','FARM COUNTRY','Sat TAG65',1679,'',43 from dual union all
select 9498,'1963-F03','1963-03-18 21:13:01','orbital','failure',1394,'360/FTV 1164','CORONA 61/LANYARD 1','LANYARD 1','CAMP OUT','Sat TAG66',1678,'',183 from dual union all
select 9499,'1963-F04','1963-04-05 03:01:43','orbital','failure',1144,'S119','Transit VA-2 (N-2)','Transit VA-2','BLUE SCOUT VI','Sat',1018,'',700 from dual union all
select 95,'1944-A59','1944-09-26 00:00:00','military missile','failure',1470,'','London','','','Weapon',1741,'',10 from dual union all
select 950,'1950-M07','1950-07-14 08:39:00','suborbital rocket','success',59,'SC 8','Grenades','','','Aeronomy',1869,'',69 from dual union all
select 9500,'1963-F05','1963-04-06 03:01:00','orbital','failure',681,'','','DS-P1 No. 2','','Sat',630,'',20 from dual union all
select 9501,'1963-F06','1963-04-26 00:00:00','orbital','failure',1143,'S121','OPS 1298 (AF-4)','P35-4','BLUE SCOUT VII','Sat',1018,'',100 from dual union all
select 9502,'1963-F07','1963-04-26 20:12:57','orbital','failure',1362,'372/FTV 1411','CORONA 63/9055A','ARGON 12','FALL HARVEST','Sat TAG68',1673,'',618 from dual union all
select 9503,'1963-F08','1963-06-01 02:50:00','orbital','failure',681,'','','DS-MT No. 1','','Sat',630,'',0 from dual union all
select 9504,'1963-F09','1963-06-12 00:00:00','orbital','failure',145,'139D','Midas 8','Midas ETS-III F3','BIG FOUR','Sat',1020,'',20 from dual union all
select 9505,'1963-F11','1963-08-22 06:00:00','orbital','failure',681,'','','DS-A1 No. 3','','Sat',630,'',40 from dual union all
select 9506,'1963-F12','1963-09-27 11:17:49','orbital','failure',1142,'S132','OPS 1610 (AF-5)','P35-5','BLUE SCOUT IX','Sat',1018,'',730 from dual union all
select 9507,'1963-F13','1963-10-24 04:30:00','orbital','failure',681,'','','DS-A1 No. 4','','Sat',630,'',250 from dual union all
select 9508,'1963-F14','1963-11-09 20:27:54','orbital','failure',1392,'400/FTV 1171','CORONA 73/9060','CORONA M-24','JUMP SUIT','Sat TAG80',1674,'',50 from dual union all
select 9509,'1963-F15','1963-11-28 09:19:59','orbital','failure',1509,'G 15001- 02','','Zenit-2 No. 14','','Sat',480,'',206 from dual union all
select 951,'1950-M08','1950-07-24 14:29:00','suborbital rocket','failure',265,'Bumper 8','','','','Test',253,'',20 from dual union all
select 9510,'1963-M01','1963-06-01 00:00:00','suborbital rocket','success',767,'','2A II-4','','','Test',185,'',70 from dual union all
select 9511,'1963-M02','1963-01-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 9512,'1963-M03','1963-01-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 9513,'1963-M04','1963-01-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 9514,'1963-M05','1963-01-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 9515,'1963-M06','1963-01-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 9516,'1963-M07','1963-01-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual;
insert into LAUNCH
select 9517,'1963-M08','1963-01-17 01:44:00','suborbital rocket','success',839,'NASA 10.80NA','Ozone release','','','Aeronomy',1821,'',74 from dual union all
select 9518,'1963-M09','1963-02-05 00:00:00','military missile','success',1162,'C-11','','','','OT?',1895,'',50 from dual union all
select 9519,'1963-M10','1963-02-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 952,'1950-M09','1950-07-29 11:44:00','suborbital rocket','success',265,'Bumper 7','','','','Test',253,'',50 from dual union all
select 9520,'1963-M11','1963-02-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 9521,'1963-M12','1963-02-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 9522,'1963-M13','1963-02-19 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 9523,'1963-M14','1963-02-19 00:00:00','military missile','success',1162,'','','','','OT?',1855,'',50 from dual union all
select 9524,'1963-M15','1963-02-07 00:00:00','suborbital rocket','success',1158,'','','','','Test',353,'',52 from dual union all
select 9525,'1963-M16','1963-02-07 00:00:00','suborbital rocket','success',1159,'','','','','Test',353,'',80 from dual union all
select 9526,'1963-M17','1963-02-07 00:00:00','suborbital rocket','success',1159,'','','','','Test',353,'',80 from dual union all
select 9527,'1963-M18','1963-03-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 9528,'1963-M19','1963-03-18 00:00:00','military missile','success',1162,'','Deployability test','','','Op Test',1415,'',50 from dual union all
select 9529,'1963-M20','1963-03-20 00:00:00','military missile','success',1162,'','Deployability test','','','Op Test',1415,'',50 from dual union all
select 953,'1950-M10','1950-10-17 04:00:00','suborbital rocket','success',60,'SC 10','Grenades','','','Aeronomy',1869,'',80 from dual union all
select 9530,'1963-M21','1963-03-21 00:00:00','military missile','success',1162,'','Deployability test','','','Op Test',1415,'',50 from dual union all
select 9531,'1963-M22','1963-03-22 00:00:00','test rocket','success',1432,'C4','','','','Test',669,'',80 from dual union all
select 9532,'1963-M23','1963-03-28 00:00:00','test rocket','success',1432,'C7','','','','Test',669,'',80 from dual union all
select 9533,'1963-M24','1963-04-08 11:26:00','sounding rocket','failure',129,'NASA 16.01GT','','','','Test',1811,'',72 from dual union all
select 9534,'1963-M25','1963-05-01 00:00:00','suborbital rocket','success',74,'','Meteo','','','Test',767,'',50 from dual union all
select 9535,'1963-M26','1963-05-16 00:00:00','military missile','success',1162,'','Off Range Test','','','Test',355,'',50 from dual union all
select 9536,'1963-M27','1963-05-02 00:00:00','suborbital rocket','success',1158,'','','','','Test',353,'',50 from dual union all
select 9537,'1963-M28','1963-05-02 17:59:54','suborbital spaceplane','success',1520,'3-16-26','APU,UVP,IR,flow','','','Test/UV Ast/Tech',379,'NB-52 008',63 from dual union all
select 9538,'1963-M29','1963-05-06 00:00:00','suborbital rocket','success',203,'B01','Radioactivity','','','Aeronomy',664,'',94 from dual union all
select 9539,'1963-M30','1963-06-05 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 954,'1950-M11','1950-10-27 13:30:00','suborbital rocket','success',60,'SC 13','Sampling','','','Aeronomy',1869,'',80 from dual union all
select 9540,'1963-M31','1963-06-25 00:00:00','military missile','success',1162,'','','350 Btn','','OT?',1895,'',50 from dual union all
select 9541,'1963-M32','1963-06-26 00:00:00','military missile','success',1162,'','','350 Btn','','OT?',1895,'',50 from dual union all
select 9542,'1963-M33','1963-06-18 18:34:21','suborbital spaceplane','success',1520,'3-19-30','Alt, UVP, Tail p','','','Test/UV Ast',379,'NB-52 008',68 from dual union all
select 9543,'1963-M34','1963-06-21 00:00:00','test rocket','success',1432,'C9','','','','Test',669,'',80 from dual union all
select 9544,'1963-M35','1963-06-27 00:00:00','test rocket','success',1432,'C8','','','','Test',669,'',80 from dual union all
select 9545,'1963-M36','1963-07-09 20:12:12','suborbital spaceplane','success',1520,'1-37-59','Opt Deg/TravP/RAS','','','Tech/Test',379,'NB-52 008',69 from dual union all
select 9546,'1963-M37','1963-07-20 21:06:00','suborbital rocket','failure',30,'NASA 04.75UA','JHU Eclipse Airglow','','','Aeronomy/Eclipse',428,'',78 from dual union all
select 9547,'1963-M38','1963-07-24 00:00:00','suborbital rocket','success',203,'B73','','','','Aeronomy',313,'',60 from dual union all
select 9548,'1963-M39','1963-08-10 03:17:00','suborbital rocket','failure',692,'LS-A Sustainer','','','','Test',990,'',10 from dual union all
select 9549,'1963-M40','1963-08-10 04:58:00','suborbital rocket','success',1075,'S-A1','','','','Test',990,'',5 from dual union all
select 955,'1950-M12','1950-11-02 00:00:00','military missile','success',340,'6E','','','','Test',1863,'',50 from dual union all
select 9550,'1963-M41','1963-08-10 07:20:00','suborbital rocket','success',1075,'S-A2','','','','Test',990,'',5 from dual union all
select 9551,'1963-M42','1963-08-10 08:58:00','suborbital rocket','success',1075,'S-A3','','','','Test',990,'',5 from dual union all
select 9552,'1963-M43','1963-08-24 02:00:00','suborbital rocket','failure',696,'L-2-1','','','','Ionos/Test',834,'',51 from dual union all
select 9553,'1963-M44','1963-09-18 09:39:00','test rocket','success',1371,'232','ASSET 1','','','Reentry Test',234,'',56 from dual union all
select 9554,'1963-M45','1963-07-26 00:00:00','military missile','success',1162,'','','5thBtn 73rdArt.','','OT?',1895,'',50 from dual union all
select 9555,'1963-M46','1963-07-26 00:00:00','military missile','success',1162,'','','5thBtn 73rdArt.','','OT?',1895,'',50 from dual union all
select 9556,'1963-M47','1963-10-04 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 9557,'1963-M48','1963-10-24 00:00:00','test rocket','success',1432,'C10','','','','Test',669,'',80 from dual union all
select 9558,'1963-M49','1963-10-24 00:00:00','suborbital rocket','unknown',344,'','USN Eval 2','','','Aeron?',1015,'',50 from dual union all
select 9559,'1963-M50','1963-12-01 00:00:00','military missile','success',1162,'','','','','OT?',1895,'',50 from dual union all
select 956,'1950-M13','1950-12-12 07:04:00','suborbital rocket','failure',1499,'6','Viking 6','','','Iono-solar',1852,'',64 from dual union all
select 9560,'1963-M51','1963-12-05 23:42:00','suborbital rocket','success',535,'HAD 301','UV absorption','','','Solar EUV/Aeron',1835,'',54 from dual union all
select 9561,'1963-M52','1963-12-10 00:00:00','suborbital rocket','unknown',344,'','USN Eval 4','','','Aeron?',1015,'',50 from dual union all
select 9562,'1963-M53','1963-12-13 22:32:00','suborbital rocket','failure',837,'NASA 14.37GI','','','','Solar X/Ionos',1895,'',73 from dual union all
select 9563,'1963-M54','1963-12-20 00:00:00','suborbital rocket','unknown',1155,'','','','','Aeron?',1015,'',50 from dual union all
select 9564,'1963-M55','1963-04-26 00:00:00','suborbital rocket','failure',446,'D-04','','','','Test',664,'',58 from dual union all
select 9565,'1963-M56','1963-05-13 17:25:00','military missile','success',1162,'','Off Range Test','','','Test',355,'',50 from dual union all
select 9566,'1963-M57','1963-05-15 00:00:00','military missile','success',1162,'','Off Range Test','','','Test',355,'',50 from dual union all
select 9567,'1963-M58','1963-01-21 00:00:00','suborbital rocket','success',766,'','1A I','','','Test',185,'',26 from dual union all
select 9568,'1963-M59','1963-02-01 00:00:00','suborbital rocket','success',766,'','1B I','','','Test',185,'',27 from dual union all
select 9569,'1963-M60','1963-04-19 00:00:00','military missile','success',874,'ZK-17','DM-15S Sim Target','','','Test',873,'',70 from dual union all
select 957,'1950-M14','1950-12-12 09:10:00','suborbital rocket','success',60,'SC 16','Grenades','','','Aeronomy',1869,'',77 from dual union all
select 9570,'1963-M61','1963-04-01 00:00:00','military missile','success',871,'ZK-18','','','','Test',873,'',70 from dual union all
select 9571,'1963-M62','1963-04-01 00:00:00','military missile','success',871,'ZK-19','','','','Test',873,'',70 from dual union all
select 9572,'1963-M63','1963-11-27 00:00:00','military missile','success',871,'ZK-30','','','','Interceptor',873,'',80 from dual union all
select 9573,'1963-M64','1963-10-08 00:00:00','military missile','success',871,'ZK-26','','','','Interceptor',873,'',80 from dual union all
select 9574,'1963-M65','1963-11-12 00:00:00','military missile','success',871,'ZK-27','','','','Interceptor',873,'',80 from dual union all
select 9575,'1963-M66','1963-09-20 00:00:00','military missile','success',871,'ZK-25','','','','Interceptor',873,'',80 from dual union all
select 9576,'1963-M67','1963-11-20 00:00:00','military missile','success',871,'ZK-29','','','','Interceptor',873,'',80 from dual union all
select 9577,'1963-M68','1963-01-01 00:00:00','military missile','success',871,'ZK-31','','','','Interceptor',873,'',80 from dual union all
select 9578,'1963-M69','1963-01-01 00:00:00','military missile','success',871,'ZK-32','','','','Interceptor',873,'',80 from dual union all
select 9579,'1963-M70','1963-08-24 00:00:00','suborbital rocket','success',947,'1','','','','Test',310,'',50 from dual union all
select 958,'1950-S01','1950-02-14 23:14:00','suborbital rocket','success',58,'NRL 4','Cosmic gamma','','','Ionospheric',1869,'',87 from dual union all
select 9580,'1963-M71','1963-08-25 00:00:00','suborbital rocket','success',947,'2','','','','Test',310,'',50 from dual union all
select 9581,'1963-S01','1963-01-01 00:00:00','military missile','success',7,'6TYa','F2','','','Test',182,'',100 from dual union all
select 9582,'1963-S02','1963-01-01 00:00:00','military missile','success',837,'BMRS','','','','Reentry test',1895,'',100 from dual union all
select 9583,'1963-S03','1963-01-01 00:00:00','military missile','success',837,'BMRS','','','','Reentry test',1895,'',100 from dual union all
select 9584,'1963-S04','1963-01-01 00:00:00','military missile','success',979,'','8K65 UBP-38','','','Test',636,'',675 from dual union all
select 9585,'1963-S05','1963-01-01 00:00:00','military missile','success',993,'','','','','Test',1626,'HOTEL2',293 from dual union all
select 9586,'1963-S06','1963-09-27 00:00:00','military missile','success',1018,'','8K75 LKI 3-11','Silo test  8K75V','','OT',579,'',1160 from dual union all
select 9587,'1963-S07','1963-06-01 00:00:00','military missile','success',1035,'','','','','Test',636,'',500 from dual union all
select 9588,'1963-S08','1963-01-01 00:00:00','suborbital rocket','success',1439,'TB IId','Trailblazer IId','','','Reentry Test',1821,'',300 from dual union all
select 9589,'1963-S09','1963-01-07 22:30:10','military missile','failure',795,'423A','Mk 11 RV','R&D','','Test',264,'',100 from dual union all
select 959,'1950-S02','1950-02-17 18:01:00','suborbital rocket','success',1470,'53','','','','Solar X/CR/Aeron',1863,'',148 from dual union all
select 9590,'1963-S10','1963-01-10 05:31:02','military missile','failure',1420,'N-15','Mk 6 RV','','','Test',231,'',500 from dual union all
select 9591,'1963-S100','1963-03-16 02:05:40','military missile','success',162,'63F','Mk 4-5B','OT TALL TREE 5','TALL TREE 5','OT',1669,'',1400 from dual union all
select 9592,'1963-S101','1963-03-16 02:19:00','suborbital rocket','success',35,'NASA 04.70CG','Lockheed XR 2','','','XR Astron',1821,'',201 from dual union all
select 9593,'1963-S102','1963-03-16 08:32:22','ballistic missile test','failure',154,'193D','Nike TV-23/NC20.145','NTMP K-14','LEADING EDGE','Target/Aeron',1662,'',300 from dual union all
select 9594,'1963-S103','1963-03-19 01:29:59','military missile','failure',795,'425','Mk 11 RV','R&D','','Test',264,'',100 from dual union all
select 9595,'1963-S104','1963-03-19 16:00:02','military missile','failure',932,'A3X-11','','','','Test',251,'',100 from dual union all
select 9596,'1963-S105','1963-03-19 00:00:00','military missile','success',976,'','8K63 PRO-20','','','Test',942,'',402 from dual union all
select 9597,'1963-S106','1963-03-19 18:00:00','suborbital rocket','success',30,'AC3.149','UV absorption','','','Aeron',1869,'',99 from dual union all
select 9598,'1963-S107','1963-03-22 01:15:00','military missile','success',916,'410','P-51','','','Test',256,'',180 from dual union all
select 9599,'1963-S108','1963-03-21 15:22:48','military missile','success',1420,'N-18','Mk 6 RV','','','Test',230,'',1300 from dual union all
select 96,'1944-A60','1944-10-27 12:00:00','military missile','failure',1470,'','','','','Weapon',1744,'',0 from dual union all
select 960,'1950-S03','1950-04-26 01:11:00','suborbital rocket','success',60,'SC 11','Air Sampling','','','Aeronomy',1869,'',100 from dual union all
select 9600,'1963-S109','1963-03-21 03:53:00','suborbital rocket','success',214,'CC-1-28','NO2 seeding','','','Aeron',428,'',80 from dual union all
select 9601,'1963-S11','1963-01-16 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 9602,'1963-S110','1963-03-21 11:40:00','test rocket','success',976,'','MP-2 (M-12)','','','Reentry Test',1013,'',405 from dual union all
select 9603,'1963-S111','1963-03-21 21:10:08','military missile','success',162,'83F','Mk 4-5B','R&D','KENDALL GREEN','Test',1716,'',1400 from dual union all
select 9604,'1963-S112','1963-03-25 00:00:00','military missile','success',976,'','8K63 KY63-5','','','Test',636,'',402 from dual union all
select 9605,'1963-S113','1963-03-25 03:17:00','suborbital rocket','success',214,'CC-1-38','NO2 seeding','','','Aeron',428,'',100 from dual union all
select 9606,'1963-S114','1963-03-28 01:30:00','military missile','success',795,'419A','Mk 11 RV','R&D','','Test',266,'',900 from dual;
insert into LAUNCH
select 9607,'1963-S115','1963-03-28 00:00:00','military missile','success',976,'','8K63 PRO-21','','','Test',942,'',402 from dual union all
select 9608,'1963-S116','1963-03-28 00:15:00','suborbital rocket','success',30,'NASA 04.30GG','UV geocorona','','','UV Astron',1869,'',300 from dual union all
select 9609,'1963-S117','1963-03-28 07:54:59','suborbital rocket','success',837,'NASA 14.08UA','UM Composition 1','','','Aeronomy',1821,'',190 from dual union all
select 961,'1950-S04','1950-05-12 03:08:00','suborbital rocket','success',1499,'4','Viking 4','','','Ionos/Aeron',1284,'AVM1',171 from dual union all
select 9610,'1963-S118','1963-03-21 09:30:00','suborbital rocket','success',535,'HAD 118','Sphere','','','Aeron',1835,'',133 from dual union all
select 9611,'1963-S119','1963-03-28 19:06:00','suborbital rocket','success',837,'NASA 14.09UA','UM Composition 2','','','Aeronomy',1821,'',190 from dual union all
select 9612,'1963-S12','1963-01-16 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 9613,'1963-S120','1963-03-28 20:06:00','suborbital rocket','success',837,'NASA 14.87CI','GCA Ionos 12','','','Ionos/Eclipse test',1821,'',161 from dual union all
select 9614,'1963-S121','1963-03-28 20:11:55','test rocket','success',1120,'SA-4','','','','Test',267,'',129 from dual union all
select 9615,'1963-S122','1963-03-26 01:30:00','suborbital rocket','success',839,'Falling Sphere 1','Three-sphere payload','','','Aeronomy',871,'',93 from dual union all
select 9616,'1963-S123','1963-03-29 02:57:00','suborbital rocket','success',839,'Falling Sphere 2','','','','Aeronomy',871,'',177 from dual union all
select 9617,'1963-S124','1963-03-30 03:30:00','military missile','success',982,'G 226   01-20T','8K64U UBP-14 K','8F17.3000','','OT',566,'',1210 from dual union all
select 9618,'1963-S125','1963-03-30 00:00:00','military missile','success',871,'ZK-15 20114','K-17 Titan Intercept','','','Test',873,'',80 from dual union all
select 9619,'1963-S126','1963-03-30 08:45:00','military missile','success',1419,'SM-3','Nike TVX-32','NTMP K-17','YOUNG BLOOD','Target',1655,'',1000 from dual union all
select 962,'1950-S05','1950-05-12 12:30:00','suborbital rocket','success',54,'A17','','','','Particles',1869,'',88 from dual union all
select 9620,'1963-S129','1963-04-01 00:00:00','military missile','success',979,'','8K65 UBP-41','','','Test',636,'',675 from dual union all
select 9621,'1963-S13','1963-01-18 03:32:50','military missile','success',916,'358','P-45','','','Test',256,'',180 from dual union all
select 9622,'1963-S130','1963-04-01 00:00:00','suborbital rocket','success',287,'','Spurious','','','Ionos',856,'',130 from dual union all
select 9623,'1963-S131','1963-04-04 03:25:00','military missile','success',916,'408','P-52 Last R&D','','','Test',255,'',180 from dual union all
select 9624,'1963-S132','1963-04-03 16:46:00','suborbital rocket','success',35,'NASA 04.58UI','EOGO VLF test','','','Ionos',1821,'',237 from dual union all
select 9625,'1963-S133','1963-04-05 04:52:37','military missile','success',1419,'V-1','Mk 4 RV','','HALF MOON','Test',1654,'',1000 from dual union all
select 9626,'1963-S134','1963-04-04 15:00:00','suborbital rocket','success',30,'NB3.118','Solar XR Spectrum 1','','','Solar X',1869,'',205 from dual union all
select 9627,'1963-S135','1963-04-06 00:44:55','military missile','success',916,'511','P-53 IST/ Orlando','','','Test',255,'',180 from dual union all
select 9628,'1963-S136','1963-04-06 04:03:00','suborbital rocket','success',214,'AD-1-26','Ionos/fields','','','Auroral/Aeron/Ionos',428,'',144 from dual union all
select 9629,'1963-S137','1963-04-06 07:25:00','suborbital rocket','success',214,'AD-1-23','','','','Auroral/Ionos/Field',428,'',140 from dual union all
select 963,'1950-S06','1950-06-20 15:38:00','suborbital rocket','success',50,'USAF 6','UM T/p','','','Aeronomy',640,'',93 from dual union all
select 9630,'1963-S138','1963-04-08 16:00:02','military missile','success',932,'A3X-18','[No RB sep]','','','Test',251,'',1000 from dual union all
select 9631,'1963-S139','1963-04-09 00:00:00','military missile','success',976,'','8K63 PRO-22','','','Test',942,'',402 from dual union all
select 9632,'1963-S14','1963-01-17 18:59:37','suborbital spaceplane','success',1520,'3-14-24','VO Alt, IR expt','','','Test/Tech',379,'NB-52 008',82 from dual union all
select 9633,'1963-S140','1963-04-09 20:28:00','suborbital rocket','success',837,'NASA 14.108GI','D-region','','','Ionos',1821,'',163 from dual union all
select 9634,'1963-S141','1963-04-11 02:08:59','military missile','success',795,'425A','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 9635,'1963-S142','1963-04-10 00:00:00','military missile','success',976,'','8K63 KY63-6','','','Test',636,'',402 from dual union all
select 9636,'1963-S143','1963-04-11 20:30:00','military missile','success',794,'534','Mk 5 RV','R&D','AFGHAN RUG','Test',1651,'',900 from dual union all
select 9637,'1963-S144','1963-04-11 00:00:00','military missile','success',979,'','8K65 UBP-42','','','Test',636,'',675 from dual union all
select 9638,'1963-S145','1963-04-12 00:00:00','military missile','success',794,'565','Mk 5 RV','R&D','VELVET TOUCH','Test',1647,'',900 from dual union all
select 9639,'1963-S146','1963-04-12 19:29:21','military missile','success',916,'515','P-54','','','Test',255,'',180 from dual union all
select 964,'1950-S07','1950-08-17 15:45:00','suborbital rocket','success',54,'A18','Mass spectrometer','','','Aeronomy',1869,'',101 from dual union all
select 9640,'1963-S147','1963-04-12 05:30:00','suborbital rocket','success',35,'NASA 04.96II','CSIRO VLF 3','','','Ionos',1821,'',198 from dual union all
select 9641,'1963-S148','1963-04-12 14:00:00','suborbital rocket','failure',30,'KP3.1','Visible Dayglow','','','Aeron',1869,'',200 from dual union all
select 9642,'1963-S149','1963-04-13 00:00:00','military missile','success',871,'ZK-16','K-21 Titan Intercept','','','Test',873,'',70 from dual union all
select 9643,'1963-S15','1963-01-23 00:27:37','military missile','success',638,'CM-106','CTL','','','OT',248,'',500 from dual union all
select 9644,'1963-S150','1963-04-13 00:00:00','military missile','success',1419,'SM-1','Nike TVX-33','NTMP K-21','RAMP ROOSTER','Target',1656,'',1000 from dual union all
select 9645,'1963-S151','1963-04-17 16:39:00','military missile','success',916,'520','P-55','','','Test',260,'',180 from dual union all
select 9646,'1963-S152','1963-04-17 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 608',1000 from dual union all
select 9647,'1963-S153','1963-04-17 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 608',1000 from dual union all
select 9648,'1963-S154','1963-04-17 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 608',1000 from dual union all
select 9649,'1963-S155','1963-04-17 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 965,'1950-S08','1950-08-31 17:09:00','suborbital rocket','success',1470,'51','Blossom IVG (Mouse)','','','Ionos/Solar X/Bio',1863,'',137 from dual union all
select 9650,'1963-S156','1963-04-17 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 9651,'1963-S157','1963-04-17 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 9652,'1963-S158','1963-04-17 09:23:00','suborbital rocket','success',1199,'SL167','UCL GS-2/QUB Na-9','','','Aeronomy',1839,'',236 from dual union all
select 9653,'1963-S159','1963-04-18 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 608',1000 from dual union all
select 9654,'1963-S16','1963-01-22 00:00:00','military missile','success',976,'','8K63 PRO-12','','','Test',942,'',402 from dual union all
select 9655,'1963-S160','1963-04-18 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 608',1000 from dual union all
select 9656,'1963-S161','1963-04-18 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 608',1000 from dual union all
select 9657,'1963-S162','1963-04-18 00:00:00','military missile','success',871,'ZW-56','','','','Test',1884,'',50 from dual union all
select 9658,'1963-S163','1963-04-18 21:00:14','suborbital rocket','success',43,'NASA 06.07GA','Thermo Probe 2(AE-A)','','','Aeronomy',1821,'',342 from dual union all
select 9659,'1963-S164','1963-04-19 20:00:11','military missile','failure',1420,'N-21','Mk 6 RV','R&D','','Test',230,'',500 from dual union all
select 966,'1950-S09','1950-10-26 00:00:00','military missile','success',992,'2','8Zh38 EDI-1-2','','','Test',636,'',150 from dual union all
select 9660,'1963-S165','1963-04-21 02:16:00','ballistic missile test','success',1177,'6','San Marco','','','Sat Test',1821,'',426 from dual union all
select 9661,'1963-S166','1963-04-20 10:34:00','suborbital rocket','success',1490,'AGI35','Airglow','','','Aeronomy/Ionos',668,'',175 from dual union all
select 9662,'1963-S167','1963-04-22 12:32:06','military missile','success',1017,'L1-4','8K74 UBP-12','','','OT',562,'',1350 from dual union all
select 9663,'1963-S168','1963-04-23 00:00:00','military missile','success',794,'585','Mk 5 RV','R&D','PAT HAND','Test',1650,'',900 from dual union all
select 9664,'1963-S169','1963-04-23 22:04:00','suborbital rocket','success',1490,'AGI49','','','','Aeronomy/Ionos',668,'',140 from dual union all
select 9665,'1963-S17','1963-01-23 00:00:00','suborbital rocket','success',366,'','Tracking Flare Test','','','Test',1490,'',90 from dual union all
select 9666,'1963-S170','1963-04-23 20:48:00','suborbital rocket','success',35,'NASA 04.44GI','Electron density','','','Ionos',1821,'',199 from dual union all
select 9667,'1963-S171','1963-04-24 20:40:49','military missile','success',916,'521','P-56','','','Test',255,'',180 from dual union all
select 9668,'1963-S172','1963-04-24 00:00:00','military missile','success',871,'ZW-57','','','','Test',1884,'',50 from dual union all
select 9669,'1963-S173','1963-04-24 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 967,'1950-S10','1950-10-27 00:00:00','military missile','success',992,'3','8Zh38 EDI-1-3','','','Test',636,'',150 from dual union all
select 9670,'1963-S174','1963-04-24 20:59:36','military missile','success',155,'65E','Mk 4-5B','R&D','BLACK BUCK','Test',1715,'',1600 from dual union all
select 9671,'1963-S175','1963-04-25 00:00:00','military missile','success',976,'','8K63 KY63-7','','','Test',636,'',402 from dual union all
select 9672,'1963-S176','1963-04-26 18:31:23','military missile','failure',932,'A3X-20','','','','Test',421,'EAG-154',80 from dual union all
select 9673,'1963-S177','1963-04-26 03:00:00','military missile','success',982,'G 225   02-20T','8K64U UBP-15 K','','','OT',566,'',1210 from dual union all
select 9674,'1963-S179','1963-04-27 00:00:00','military missile','success',1420,'N-8','Mk 6 RV','R&D','DINNER PARTY','Test',1658,'',1300 from dual union all
select 9675,'1963-S18','1963-01-23 00:00:00','suborbital rocket','success',366,'','Tracking Flare Test','','','Test',1490,'',90 from dual union all
select 9676,'1963-S180','1963-04-27 02:03:00','military missile','success',162,'135F','ABRES REX-II/Decoy','','','Reentry Test',226,'',1400 from dual union all
select 9677,'1963-S181','1963-04-28 00:00:00','military missile','success',976,'','8K63 KY63-8','','','Test',636,'',402 from dual union all
select 9678,'1963-S182','1963-04-30 06:13:00','suborbital rocket','success',369,'','RFD-1 Support','','','Aeron',194,'',90 from dual union all
select 9679,'1963-S183','1963-04-30 09:21:00','suborbital rocket','success',369,'','RFD-1 Support','','','Aeron',194,'',90 from dual union all
select 968,'1950-S11','1950-10-12 19:36:00','suborbital rocket','success',50,'USAF 7','Photo/Temp','','','Imaging',640,'',87 from dual union all
select 9680,'1963-S184','1963-04-30 00:00:00','military missile','success',794,'595','Mk 5 RV','R&D','BOLD JOURNEY','Test',1648,'',900 from dual union all
select 9681,'1963-S185','1963-04-30 04:00:00','suborbital rocket','success',30,'NB3.130','UV Phot/NRL X-ray','','','UV Astron/XR Astron',1869,'',197 from dual union all
select 9682,'1963-S186','1963-05-01 16:59:00','suborbital rocket','success',1490,'AGI45','','','','Aeronomy/Ionos',668,'',160 from dual union all
select 9683,'1963-S187','1963-05-02 15:03:00','suborbital rocket','success',1160,'','','','','Test',353,'',120 from dual union all
select 9684,'1963-S188','1963-05-02 17:03:00','suborbital rocket','success',30,'AC3.510','EUV Mon 2/XR/RPA','','','Solar UV',1869,'',232 from dual union all
select 9685,'1963-S189','1963-05-07 20:30:00','suborbital rocket','success',215,'BB-2A-22','','','','Ionos',428,'',139 from dual union all
select 9686,'1963-S19','1963-01-23 18:10:13','military missile','success',795,'419','Mk 11 RV','R&D','','Test',266,'',900 from dual union all
select 9687,'1963-S190','1963-05-07 21:11:00','suborbital rocket','success',35,'NASA 04.98UA','JHU Airglow 5','','','Aeronomy',1821,'',223 from dual union all
select 9688,'1963-S191','1963-05-08 00:00:00','military missile','success',794,'604','Mk 5 RV','R&D','FINE SHOW','Test',1651,'',900 from dual union all
select 9689,'1963-S192','1963-05-08 17:23:00','suborbital rocket','success',837,'NASA 14.110CA','','','','Aeronomy',1821,'',166 from dual union all
select 969,'1950-S12','1950-10-18 04:30:00','suborbital rocket','success',60,'SC 12','Grenades','','','Aeronomy',1869,'',85 from dual union all
select 9690,'1963-S193','1963-05-09 16:08:32','military missile','failure',1420,'N-14','Mk 6 RV','R&D','','Test',231,'',1000 from dual union all
select 9691,'1963-S194','1963-05-09 18:02:00','suborbital rocket','success',35,'NASA 04.97II','CSIRO VLF 4','','','Ionos',1821,'',203 from dual union all
select 9692,'1963-S195','1963-05-10 17:44:04','military missile','success',932,'A3X-12','A,B,PX-2(D)','','','Test',244,'',1000 from dual union all
select 9693,'1963-S196','1963-05-10 00:00:00','suborbital rocket','success',1490,'AGI48','Coronagraph','','','Solar UV',668,'',135 from dual union all
select 9694,'1963-S197','1963-05-10 14:45:00','suborbital rocket','success',30,'NE3.140R','','','','Solar EUV',1869,'',214 from dual union all
select 9695,'1963-S198','1963-05-11 00:00:00','suborbital rocket','unknown',673,'','','','','Aeron?',1016,'',300 from dual union all
select 9696,'1963-S199','1963-05-13 00:00:00','military missile','success',1420,'N-19','Mk 6 RV','R&D','FLYING FROG','Test',1659,'',1300 from dual union all
select 9697,'1963-S20','1963-01-25 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual;
insert into LAUNCH
select 9698,'1963-S200','1963-05-14 00:00:00','suborbital rocket','success',287,'C51','','','','Aeronomy',664,'',155 from dual union all
select 9699,'1963-S201','1963-05-14 15:45:00','suborbital rocket','success',30,'NASA 04.66NP','Paraglider','','','Tech',1869,'',185 from dual union all
select 97,'1944-A61','1944-10-30 00:00:00','military missile','success',1519,'','','','','Test',677,'',17 from dual union all
select 970,'1950-S13','1950-10-21 00:00:00','military missile','failure',992,'1','8Zh38 EDI-1-1','','','Test',636,'',150 from dual union all
select 9700,'1963-S202','1963-05-15 18:22:00','suborbital rocket','success',287,'C42','MPE Ba-3','','','Aeronomy',664,'',160 from dual union all
select 9701,'1963-S203','1963-05-16 04:00:00','suborbital rocket','success',839,'AC6.472','Firefly IV CLAIRE','','','Aeron',394,'',156 from dual union all
select 9702,'1963-S205','1963-05-16 14:38:00','suborbital rocket','success',839,'NASA 10.77IA','Rehbar 3  (Na)','','','Aeronomy',1419,'',146 from dual union all
select 9703,'1963-S206','1963-05-17 16:14:59','military missile','success',932,'A3X-15','No B,C sep','','','Test',244,'',1000 from dual union all
select 9704,'1963-S207','1963-05-17 23:00:18','test rocket','success',249,'301','ERCS 279L','','','Comms',1015,'',1000 from dual union all
select 9705,'1963-S209','1963-05-18 14:50:00','military missile','success',982,'G 22500-01TS','8K64U UBP-16','8F115','','OT',570,'',1210 from dual union all
select 9706,'1963-S21','1963-01-25 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 9707,'1963-S210','1963-05-18 15:00:00','military missile','success',982,'G 22500-03TS','8K64U UBP-18','8F115','','OT',572,'',1210 from dual union all
select 9708,'1963-S211','1963-05-18 07:41:00','military missile','success',1017,'L1-6','8K74 UBP-13','','','OT',562,'',1350 from dual union all
select 9709,'1963-S212','1963-05-18 01:06:00','suborbital rocket','success',837,'NASA14.141DA AC5.469','Firefly IV SARAH','','','Aeronomy Na',394,'',185 from dual union all
select 971,'1950-S14','1950-10-30 00:00:00','military missile','success',992,'4','8Zh38 EDI-1-4','','','Test',636,'',150 from dual union all
select 9710,'1963-S213','1963-05-18 04:18:00','suborbital rocket','success',837,'AC5.470','Firefly IV STELLA','','','Aeron',394,'',94 from dual union all
select 9711,'1963-S214','1963-05-18 08:55:00','suborbital rocket','success',839,'AC6.473','Firefly IV DORIS','','','Aeron',394,'',167 from dual union all
select 9712,'1963-S215','1963-05-18 10:06:00','suborbital rocket','success',837,'NASA14.140DA AC5.471','Firefly IV BLANCHE','','','Aeronomy Na',394,'',187 from dual union all
select 9713,'1963-S216','1963-05-20 00:00:00','suborbital rocket','success',837,'NASA 14.137IA','Na','','','Aeronomy',1343,'',200 from dual union all
select 9714,'1963-S217','1963-05-20 00:00:00','military missile','success',1018,'T 103   44','8K75 LKI 3-6','','','OT',584,'',1160 from dual union all
select 9715,'1963-S218','1963-05-20 02:09:00','suborbital rocket','success',668,'K-9M-2','','','','Ionos',839,'',345 from dual union all
select 9716,'1963-S219','1963-05-21 00:00:00','suborbital rocket','success',837,'NASA 14.139IA','Na','','','Aeronomy',1343,'',200 from dual union all
select 9717,'1963-S22','1963-01-25 10:44:07','military missile','failure',154,'39D','Nike TV-33','NTMP K-9/OT','BIG SUE','ABM Test',1666,'',100 from dual union all
select 9718,'1963-S220','1963-05-21 00:00:00','suborbital rocket','success',837,'NASA 14.138IA','Na','','','Aeronomy',1343,'',200 from dual union all
select 9719,'1963-S221','1963-05-21 00:00:00','suborbital rocket','success',287,'C45','','','','Aeronomy',1314,'',170 from dual union all
select 972,'1950-S15','1950-11-08 00:00:00','military missile','failure',992,'5','8Zh38 EDI-1-5','','','Test',636,'',150 from dual union all
select 9720,'1963-S222','1963-05-21 00:00:00','suborbital rocket','success',287,'C48','','','','Aeronomy',313,'',170 from dual union all
select 9721,'1963-S223','1963-05-21 18:25:00','suborbital rocket','success',287,'C43','MPE Ba-4','','','Aeronomy',664,'',125 from dual union all
select 9722,'1963-S224','1963-05-21 05:30:00','suborbital rocket','success',30,'NB3.119','FUV Telescope','','','FUV Astron',1869,'',192 from dual union all
select 9723,'1963-S225','1963-05-22 01:10:00','suborbital rocket','success',839,'NASA 10.130DA','Firefly IV IRENE','','','Aeronomy',394,'',135 from dual union all
select 9724,'1963-S226','1963-05-22 04:38:00','test rocket','success',1144,'S116','RFD-1','','','Reentry Test',1812,'',145 from dual union all
select 9725,'1963-S227','1963-05-22 05:10:00','suborbital rocket','success',837,'NASA 14.13CA','GCA-CNRS Na-Li 1','','','Aeronomy',428,'',186 from dual union all
select 9726,'1963-S228','1963-05-22 08:51:00','suborbital rocket','success',837,'NASA 14.14CA','GCA-CNRS Na-Li 2','','','Aeronomy',428,'',184 from dual union all
select 9727,'1963-S229','1963-05-24 00:00:00','military missile','success',874,'ZK-20 20138 (100)','ZK-20 DM-15S Agena','','','ASAT Test',873,'',200 from dual union all
select 9728,'1963-S23','1963-01-27 00:00:00','suborbital rocket','success',972,'','','','','Ionos?',636,'',86 from dual union all
select 9729,'1963-S230','1963-05-23 05:13:00','suborbital rocket','success',837,'NASA 14.15CA','GCA-CNRS Na-Li 3','','','Aeronomy',428,'',200 from dual union all
select 973,'1950-S16','1950-11-14 00:00:00','military missile','success',992,'6','8Zh38 EDI-1-6','','','Test',636,'',150 from dual union all
select 9730,'1963-S231','1963-05-23 08:46:00','suborbital rocket','success',1199,'SL115','LUX-11/JB-4/MO-3','','','Solar X/Aeronomy',1839,'',201 from dual union all
select 9731,'1963-S232','1963-05-25 05:50:00','military missile','success',795,'658','Mk 11 RV','R&D','HEY DAY','Test',1652,'',900 from dual union all
select 9732,'1963-S233','1963-05-24 17:33:05','military missile','success',1420,'N-17','Mk 6 RV','R&D','','Test',230,'',1300 from dual union all
select 9733,'1963-S234','1963-05-24 00:00:00','suborbital rocket','failure',287,'C41','','','','Aeronomy',664,'',15 from dual union all
select 9734,'1963-S235','1963-05-24 00:45:00','suborbital rocket','success',837,'NASA 14.40CA','GCA Release 26 Na','','','Aeronomy',1821,'',205 from dual union all
select 9735,'1963-S236','1963-05-24 09:00:00','suborbital rocket','success',837,'NASA 14.41CA','GCA Release 27 Na','','','Aeronomy',1821,'',204 from dual union all
select 9736,'1963-S237','1963-05-25 15:00:00','military missile','success',982,'G 22500-04TS','8K64U UBP-19','','','OT',564,'',1210 from dual union all
select 9737,'1963-S238','1963-05-25 00:00:00','suborbital rocket','success',287,'C35','Na cloud','','','Aeronomy',310,'',189 from dual union all
select 9738,'1963-S239','1963-05-25 00:47:00','suborbital rocket','success',837,'NASA 14.42CA','GCA Release 28 Na','','','Aeronomy',1821,'',197 from dual union all
select 9739,'1963-S24','1963-01-27 01:00:00','suborbital rocket','success',5,'','','','','Test',636,'',100 from dual union all
select 974,'1950-S17','1950-11-18 00:00:00','military missile','failure',992,'7','8Zh38 EDI-1-7','','','Test',636,'',150 from dual union all
select 9740,'1963-S240','1963-05-26 05:34:00','suborbital rocket','unknown',545,'','','','','Aeron',855,'',103 from dual union all
select 9741,'1963-S241','1963-05-28 18:59:09','suborbital rocket','success',1021,'RAM B-2','RAM B-2','','','Reentry Test',1821,'',88 from dual union all
select 9742,'1963-S242','1963-05-29 04:48:14','military missile','success',795,'428','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 9743,'1963-S243','1963-05-29 00:00:00','suborbital rocket','success',850,'TRUMP 1','TRUMP 1','','','Test',394,'',131 from dual union all
select 9744,'1963-S244','1963-05-29 04:35:00','suborbital rocket','success',1199,'SL127','LUX-12/UCL I-12/RRS-1','','','Ionos/Solar X',1839,'',209 from dual union all
select 9745,'1963-S245','1963-05-30 00:00:00','suborbital rocket','success',287,'C36','','','','Aeron?',310,'',150 from dual union all
select 9746,'1963-S246','1963-05-30 00:00:00','suborbital rocket','success',287,'C39','','','','Aeronomy',310,'',150 from dual union all
select 9747,'1963-S247','1963-06-01 00:00:00','military missile','success',837,'BMRS','','','','Reentry test',1895,'',100 from dual union all
select 9748,'1963-S248','1963-06-01 00:00:00','military missile','success',837,'Capache','','','','-',1895,'',100 from dual union all
select 9749,'1963-S249','1963-06-01 00:00:00','military missile','success',837,'Capache','','','','-',1895,'',100 from dual union all
select 975,'1950-S18','1950-11-26 00:00:00','military missile','failure',992,'8','8Zh38 EDI-1-8','','','Test',636,'',1 from dual union all
select 9750,'1963-S25','1963-01-29 00:00:00','military missile','success',1419,'SM-8','Mk 4 RV','R&D Cat II','TEN MEN','Test',1654,'',1000 from dual union all
select 9751,'1963-S250','1963-06-01 00:00:00','military missile','success',837,'Capache','','','','-',1895,'',100 from dual union all
select 9752,'1963-S251','1963-06-01 03:00:00','military missile','failure',982,'E 22502-15','8K64U UBP-20','','','OT',564,'',1210 from dual union all
select 9753,'1963-S252','1963-06-04 00:00:00','military missile','success',929,'A2','ORRT','','','OT',421,'SSBN 601',1000 from dual union all
select 9754,'1963-S253','1963-06-04 00:00:00','military missile','success',929,'A2','ORRT','','','OT',421,'SSBN 601',1000 from dual union all
select 9755,'1963-S254','1963-06-04 00:00:00','military missile','success',929,'A2','ORRT','','','OT',421,'SSBN 601',1000 from dual union all
select 9756,'1963-S255','1963-06-04 00:00:00','military missile','failure',929,'A2','ORRT','','','OT',421,'SSBN 601',1000 from dual union all
select 9757,'1963-S256','1963-06-04 00:00:00','military missile','success',929,'A2','ORRT','','','OT',421,'SSBN 601',1000 from dual union all
select 9758,'1963-S257','1963-06-04 00:00:00','military missile','success',929,'A2','ORRT','','','OT',421,'SSBN 601',1000 from dual union all
select 9759,'1963-S258','1963-06-04 20:12:18','military missile','success',155,'62E','Mk 4-5','R&D','DOCK HAND','Test',1715,'',1600 from dual union all
select 976,'1950-S19','1950-11-02 16:29:00','suborbital rocket','success',50,'USAF 8','Airglow','','','Aeronomy',640,'',92 from dual union all
select 9760,'1963-S259','1963-06-05 00:00:00','military missile','success',871,'ZW-63','JFK Demo','','','Test',1884,'',50 from dual union all
select 9761,'1963-S26','1963-01-31 00:30:05','military missile','success',916,'363','P-46','','','Test',256,'',180 from dual union all
select 9762,'1963-S260','1963-06-06 04:14:59','military missile','success',795,'427','Mk 11 RV','R&D','','Test',266,'',900 from dual union all
select 9763,'1963-S261','1963-06-06 16:16:59','military missile','success',932,'A3X-25','','','','Test',244,'',1000 from dual union all
select 9764,'1963-S262','1963-06-06 00:00:00','military missile','success',976,'','8K63 KY63-9','','','Test',636,'',402 from dual union all
select 9765,'1963-S263','1963-06-06 00:30:00','suborbital rocket','success',1012,'','R-5B Geophysical','VAO','','Ionos/Plasma',636,'',500 from dual union all
select 9766,'1963-S264','1963-06-06 14:30:00','suborbital rocket','success',30,'NC3.115F','','','','Aeron',1869,'',208 from dual union all
select 9767,'1963-S265','1963-06-08 00:00:00','military missile','success',976,'','8K63 KY63-10','','','Test',636,'',402 from dual union all
select 9768,'1963-S266','1963-06-08 00:00:00','military missile','success',976,'','8K63 KY63-11','','','Test',636,'',402 from dual union all
select 9769,'1963-S267','1963-06-11 05:06:00','suborbital rocket','success',215,'BB-2A-21','','','','Ionos',428,'',135 from dual union all
select 977,'1950-S20','1950-11-09 00:00:00','military missile','success',574,'B-1 No. 2A','Hermes II RTV-G-3','','','Test',1863,'',150 from dual union all
select 9770,'1963-S268','1963-06-11 05:15:00','suborbital rocket','success',30,'AC3.354','ASE/Lunar X-ray','','','Solar X',1869,'',302 from dual union all
select 9771,'1963-S269','1963-06-12 00:00:00','suborbital rocket','success',1431,'Sandia 152-1','Seagull (IQSY)','','','Test',1490,'',105 from dual union all
select 9772,'1963-S27','1963-01-30 10:32:00','suborbital rocket','success',535,'HAD 115','Sphere','','','Aeron',1835,'',94 from dual union all
select 9773,'1963-S270','1963-06-12 18:39:00','suborbital rocket','success',288,'C54','LangP/Na','','','Ionos',664,'',145 from dual union all
select 9774,'1963-S271','1963-06-12 09:03:01','military missile','success',154,'198D','Nike TV-23','NTMP K-15','HARPOON GUN','Target',1664,'',1800 from dual union all
select 9775,'1963-S272','1963-06-12 09:30:00','military missile','success',871,'ZK-21','K-15 Atlas Intercept','','','Test',873,'',80 from dual union all
select 9776,'1963-S273','1963-06-13 17:05:34','military missile','success',929,'A2P-154','','','','OT',421,'SSBN 616',1000 from dual union all
select 9777,'1963-S274','1963-06-13 00:00:00','suborbital rocket','success',288,'C55','','','','Aeronomy',664,'',188 from dual union all
select 9778,'1963-S275','1963-06-14 00:00:00','military missile','success',976,'','8K63 KY63-12','','','Test',636,'',402 from dual union all
select 9779,'1963-S276','1963-06-14 00:00:00','suborbital rocket','success',288,'C53','','','','Aeronomy',664,'',185 from dual union all
select 978,'1950-S21','1950-11-21 17:18:00','suborbital rocket','success',1499,'5','Viking 5','','','Iono-solar',1852,'',174 from dual union all
select 9780,'1963-S277','1963-06-17 16:14:02','military missile','success',932,'A3X-24','','','','Test',421,'EAG-154',1000 from dual union all
select 9781,'1963-S278','1963-06-17 18:00:00','suborbital rocket','success',30,'NASA 04.87GT','Attitude control','','','Test',1869,'',182 from dual union all
select 9782,'1963-S279','1963-06-18 04:30:00','suborbital rocket','success',1199,'SL126','LUX-13/UCL I-13/RRS-2','','','Ionos/Solar X',1839,'',196 from dual union all
select 9783,'1963-S28','1963-02-01 00:00:00','military missile','success',837,'BMRS','','','','Reentry test',1895,'',100 from dual union all
select 9784,'1963-S280','1963-06-18 00:00:00','suborbital rocket','success',767,'','2A II-2','','','Test',185,'',92 from dual union all
select 9785,'1963-S281','1963-06-18 00:00:00','military missile','success',794,'521','Mk 5 RV','DASO','WAR AXE','Test',1649,'',900 from dual union all
select 9786,'1963-S282','1963-06-18 00:00:00','suborbital rocket','failure',1490,'AGI33','','','','Ionos',668,'',160 from dual union all
select 9787,'1963-S283','1963-06-18 00:30:00','suborbital rocket','success',1012,'','R-5B Geophysical','VAO','','Aeron/Ionos',636,'',500 from dual;
insert into LAUNCH
select 9788,'1963-S284','1963-06-18 03:28:00','suborbital rocket','success',839,'Falling Sphere 3','','','','Aeronomy',871,'',177 from dual union all
select 9789,'1963-S285','1963-06-19 17:33:00','suborbital rocket','success',30,'NASA 04.28NP','LeRC LH2 test','','','Tech',1821,'',189 from dual union all
select 979,'1950-S22','1950-12-11 00:00:00','military missile','success',992,'9','8Zh38 EDI-1-9','','','Test',636,'',150 from dual union all
select 9790,'1963-S286','1963-06-20 00:00:00','military missile','success',1420,'N-22','Mk 6 RV','R&D','THREAD NEEDLE','Test',1658,'',1300 from dual union all
select 9791,'1963-S287','1963-06-20 03:00:00','suborbital rocket','success',839,'Falling Sphere 4','','','','Aeronomy',871,'',180 from dual union all
select 9792,'1963-S288','1963-06-20 05:20:00','suborbital rocket','success',1199,'SL46','LUX-14/UCL I-14SUV6','','','Ionos/Solar X/Solar UV',1839,'',227 from dual union all
select 9793,'1963-S289','1963-06-20 14:25:00','suborbital rocket','success',30,'NASA 04.61AS','NRL Coronagraph','','','Test',1869,'',195 from dual union all
select 9794,'1963-S29','1963-03-21 00:00:00','military missile','success',874,'ZK-14/MUDFLAP 3','DM-15S Test 3','','','ASAT Test',873,'',207 from dual union all
select 9795,'1963-S290','1963-06-21 15:44:03','military missile','success',932,'A3X-22','','','','Test',421,'EAG-154',1000 from dual union all
select 9796,'1963-S291','1963-06-24 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 610',1000 from dual union all
select 9797,'1963-S292','1963-06-24 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 610',1000 from dual union all
select 9798,'1963-S293','1963-06-24 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 610',1000 from dual union all
select 9799,'1963-S294','1963-06-24 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 610',1000 from dual union all
select 98,'1944-A62','1944-12-31 00:00:00','military missile','failure',1470,'','London','','','Weapon',1627,'',0 from dual union all
select 980,'1950-S23','1950-12-12 00:00:00','military missile','success',992,'10','8Zh38 EDI-1-10','','','Test',636,'',150 from dual union all
select 9800,'1963-S295','1963-06-24 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 610',1000 from dual union all
select 9801,'1963-S296','1963-06-24 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 610',1000 from dual union all
select 9802,'1963-S297','1963-06-24 19:00:00','suborbital rocket','success',215,'CC-2A-18','','','','Test',428,'',171 from dual union all
select 9803,'1963-S298','1963-06-24 23:00:00','suborbital rocket','success',215,'CC-2A-17','','','','Test',428,'',162 from dual union all
select 9804,'1963-S299','1963-06-25 00:00:00','suborbital rocket','success',866,'Sandia 152-2','Nitehawk-9','','','Test',855,'',250 from dual union all
select 9805,'1963-S30','1963-01-31 08:51:51','military missile','success',154,'176D','Nike TV-22','NTMP K-12','FAINT CLICK','ABM Test',1664,'',1800 from dual union all
select 9806,'1963-S300','1963-06-25 13:20:00','suborbital rocket','success',30,'KP3.2','Visible Dayglow','','','Aeron/Auroral',1869,'',213 from dual union all
select 9807,'1963-S301','1963-06-26 00:00:00','military missile','success',976,'','8K63 PRO-23','','','Test',942,'',402 from dual union all
select 9808,'1963-S302','1963-06-28 02:34:12','military missile','success',795,'429','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 9809,'1963-S303','1963-06-27 00:00:00','military missile','success',206,'005','','','','Reentry Test',313,'',270 from dual union all
select 981,'1950-S24','1950-12-16 00:00:00','military missile','success',992,'11','8Zh38 EDI-1-11','','','Test',636,'',150 from dual union all
select 9810,'1963-S304','1963-06-27 17:56:03','suborbital spaceplane','success',1520,'3-20-31','Alt/UVP/IR/HorScan','','','Test/UV Ast/Tech',379,'NB-52 008',86 from dual union all
select 9811,'1963-S306','1963-06-28 00:00:00','military missile','success',794,'546','Mk 5 RV','DASO','TRIM CHIEF','Test',1647,'',900 from dual union all
select 9812,'1963-S307','1963-06-28 15:30:00','military missile','success',982,'G 22500-05TS','8K64U UBP-22','','','OT',565,'',1210 from dual union all
select 9813,'1963-S308','1963-06-28 14:30:00','suborbital rocket','success',30,'NASA 04.62AS','NRL NE3.129','','','Solar/Solar EUV',1869,'',204 from dual union all
select 9814,'1963-S309','1963-06-29 00:00:00','military missile','success',979,'','8K65 UBP-43','','','Test',636,'',675 from dual union all
select 9815,'1963-S31','1963-02-01 00:00:00','military missile','success',976,'','8K63 KY63-1','','','Test',636,'',402 from dual union all
select 9816,'1963-S310','1963-06-29 00:00:00','military missile','success',979,'','8K65 UBP-44','','','Test',636,'',675 from dual union all
select 9817,'1963-S311','1963-06-29 16:46:00','suborbital rocket','success',30,'AC3.607','Gyro-resonance','','','Ionos',394,'',156 from dual union all
select 9818,'1963-S312','1963-07-01 00:00:00','military missile','success',837,'Capache','','','','-',1895,'',100 from dual union all
select 9819,'1963-S313','1963-07-01 00:00:00','military missile','success',837,'Capache','','','','-',1895,'',100 from dual union all
select 982,'1950-S25','1950-12-12 04:06:00','suborbital rocket','success',60,'SC 14','Grenades','','','Aeronomy',1869,'',84 from dual union all
select 9820,'1963-S314','1963-06-30 01:00:00','military missile','success',982,'G 225   03-20T','8K64U UBP-23 K','','','OT',571,'',1210 from dual union all
select 9821,'1963-S315','1963-07-01 00:00:00','suborbital rocket','success',217,'AA-3-7','USN Eval 1','','','Test',1015,'',100 from dual union all
select 9822,'1963-S316','1963-07-01 21:43:55','military missile','success',795,'431','Mk 11 RV','R&D','','Test',266,'',900 from dual union all
select 9823,'1963-S317','1963-07-01 00:00:00','suborbital rocket','success',866,'Sandia 152-3','Nitehawk-9','','','Test',855,'',250 from dual union all
select 9824,'1963-S318','1963-07-01 00:00:00','military missile','success',976,'','8K63 KY63-13','','','Test',636,'',402 from dual union all
select 9825,'1963-S319','1963-07-01 00:00:00','military missile','success',871,'ZW-64  (104)','','','','Test',1884,'',50 from dual union all
select 9826,'1963-S32','1963-02-02 06:56:29','test rocket','success',249,'202','ERCS 279L','','','Comms',1015,'',1000 from dual union all
select 9827,'1963-S320','1963-07-02 16:00:04','military missile','success',929,'A2P-118','','','','OT',421,'SSBN 616',1000 from dual union all
select 9828,'1963-S321','1963-07-02 16:30:03','military missile','success',929,'A2P-167','','','','OT',421,'SSBN 616',1000 from dual union all
select 9829,'1963-S322','1963-07-02 14:18:00','suborbital rocket','success',624,'NASA 8.14GI','Topside sounder','','','Ionos',1821,'',934 from dual union all
select 983,'1950-S26','1950-12-12 18:26:00','suborbital rocket','success',50,'USAF 9','UM T/p','','','Aeronomy',640,'',106 from dual union all
select 9830,'1963-S323','1963-07-03 17:48:03','military missile','success',932,'A3X-26','','','','Test',244,'',1000 from dual union all
select 9831,'1963-S324','1963-07-03 00:00:00','military missile','success',871,'ZK-22','K-23 Atlas Intercept','','','Test',873,'',80 from dual union all
select 9832,'1963-S325','1963-07-03 00:00:00','military missile','success',871,'ZW-65','','','','Test',1884,'',50 from dual union all
select 9833,'1963-S326','1963-07-03 00:00:00','military missile','success',206,'006','','','','Reentry Test',313,'',270 from dual union all
select 9834,'1963-S327','1963-07-03 09:00:00','suborbital rocket','success',30,'AB3.247','Structure','','','Test/Ionos',394,'',199 from dual union all
select 9835,'1963-S328','1963-07-03 21:13:57','military missile','success',155,'69E','Nike TV-32','OT DASO/NTMP','GO BOY','Test',1668,'',1600 from dual union all
select 9836,'1963-S329','1963-07-05 21:00:00','military missile','success',794,'518','Mk 5 RV','R&D','GRAND TOUR','Test',1650,'',900 from dual union all
select 9837,'1963-S33','1963-02-04 00:00:00','military missile','failure',929,'A2P-108','ORRT','','','OT',421,'SSBN 599',1000 from dual union all
select 9838,'1963-S330','1963-07-05 00:00:00','military missile','success',976,'','8K63 PRO-24','','','Test',942,'',402 from dual union all
select 9839,'1963-S331','1963-07-06 00:36:00','suborbital rocket','success',1012,'','R-5B Geophysical','','','Aeron',636,'',400 from dual union all
select 984,'1950-S27','1950-12-19 18:52:00','suborbital rocket','success',60,'SC 17','Sampling','','','Aeronomy',1869,'',82 from dual union all
select 9840,'1963-S332','1963-07-08 15:30:00','military missile','success',982,'G 22500-06TS','8K64U UBP-24','','','OT',565,'',1210 from dual union all
select 9841,'1963-S333','1963-07-09 15:30:00','military missile','success',982,'G 22500-07TS','8K64U UBP-25','','','OT',565,'',1210 from dual union all
select 9842,'1963-S334','1963-07-10 02:46:00','suborbital rocket','success',35,'NASA 04.59UI','EOGO VLF test','','','Ionos',1821,'',204 from dual union all
select 9843,'1963-S335','1963-07-10 17:02:00','suborbital rocket','success',30,'AC3.511','EUV Mon/RPA','','','Solar UV',1869,'',225 from dual union all
select 9844,'1963-S336','1963-07-11 23:00:00','military missile','success',794,'650','Mk 5 RV','DASO','TRIPLE PLAY','Test',1648,'',900 from dual union all
select 9845,'1963-S337','1963-07-11 00:00:00','military missile','success',976,'','8K63 KY63-14','','','Test',636,'',402 from dual union all
select 9846,'1963-S339','1963-07-14 21:03:28','suborbital rocket','success',837,'NASA 14.88CI','GCA Probe High 1','','','Ionos/Eclipse test',428,'',172 from dual union all
select 9847,'1963-S34','1963-02-04 00:00:00','military missile','failure',929,'A2P-109','ORRT','','','OT',421,'SSBN 599',1000 from dual union all
select 9848,'1963-S340','1963-07-16 00:00:00','military missile','success',1419,'SM-24','Mk 4 RV','DASO','SILVER SPUR','OT',1655,'',1000 from dual union all
select 9849,'1963-S341','1963-07-19 01:00:00','military missile','success',932,'A3X-27','PX-2(P)','','','Test',244,'',1000 from dual union all
select 985,'1950-S28','1950-12-20 00:00:00','military missile','failure',992,'12','8Zh38 EDI-1-12','','','Test',636,'',1 from dual union all
select 9850,'1963-S342','1963-07-19 00:00:00','military missile','success',976,'','8K63 PRO-25','','','Test',942,'',402 from dual union all
select 9851,'1963-S343','1963-07-19 05:30:00','suborbital rocket','success',35,'NASA 04.37GG','UV spectrograph','','','UV Astron',1821,'',185 from dual union all
select 9852,'1963-S344','1963-07-19 18:20:05','suborbital spaceplane','success',1520,'3-21-32','VO,UVP,IR,HS,Balloon','','','Test/Tech/UV Ast',379,'NB-52 008',106 from dual union all
select 9853,'1963-S345','1963-07-20 15:18:00','suborbital rocket','success',30,'NASA 04.77GS','OSO B spectrometer','','','Solar UV/Eclipse',1869,'',213 from dual union all
select 9854,'1963-S346','1963-07-20 21:03:00','suborbital rocket','success',837,'NASA 14.91CI','GCA Probe High 4','','','Ionos/Eclipse',428,'',196 from dual union all
select 9855,'1963-S347','1963-07-20 21:05:00','suborbital rocket','success',215,'AB17.199','Opn Probe High','','','Eclipse',428,'',143 from dual union all
select 9856,'1963-S348','1963-07-20 21:13:00','suborbital rocket','success',837,'NASA 14.92CI','GCA Probe High 5','','','Ionos/Eclipse/Solar X',428,'',199 from dual union all
select 9857,'1963-S349','1963-07-20 21:40:00','suborbital rocket','success',837,'NASA 14.93CI','GCA Probe High 6','','','Ionos/Eclipse/Solar X',428,'',200 from dual union all
select 9858,'1963-S35','1963-02-05 00:00:00','military missile','success',929,'A2P-110','ORRT','','','OT',421,'SSBN 599',1000 from dual union all
select 9859,'1963-S350','1963-07-20 21:54:45','suborbital rocket','success',43,'NASA 06.08GA','Thermosphere Probe 3','','','Aeronomy/Eclipse',1821,'',338 from dual union all
select 986,'1951-A01','1951-01-18 20:14:00','suborbital rocket','failure',1470,'54','','','','Solar UV,X/CR',1863,'',2 from dual union all
select 9860,'1963-S351','1963-07-20 22:10:00','suborbital rocket','success',837,'NASA 14.94CI','GCA Probe High 7','','','Ionos/Eclipse/Solar X',428,'',192 from dual union all
select 9861,'1963-S352','1963-07-23 06:00:00','suborbital rocket','success',35,'NASA 04.29GG','UV spectrograph','','','UV Astron',1821,'',177 from dual union all
select 9862,'1963-S353','1963-07-25 03:30:13','military missile','success',795,'432','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 9863,'1963-S354','1963-07-25 00:00:00','military missile','success',977,'','8K63U KY63-16','','','OT',636,'',402 from dual union all
select 9864,'1963-S355','1963-07-25 22:38:00','suborbital rocket','success',470,'CRL AC13.831','Refractive Index','','','Ionos',394,'',623 from dual union all
select 9865,'1963-S356','1963-07-27 04:55:01','military missile','success',932,'A3X-33','PX-2(P)','','','Test',251,'',1000 from dual union all
select 9866,'1963-S357','1963-07-26 00:00:00','military missile','success',976,'','8K63 KY63-17','','','Test',636,'',402 from dual union all
select 9867,'1963-S358','1963-07-26 00:00:00','military missile','success',976,'','8K63 KY63-18','','','Test',636,'',402 from dual union all
select 9868,'1963-S359','1963-07-26 00:00:00','military missile','success',976,'','8K63 KY63-19','','','Test',636,'',402 from dual union all
select 9869,'1963-S36','1963-02-05 00:00:00','military missile','failure',929,'A2P-111','ORRT','','','OT',421,'SSBN 599',1000 from dual union all
select 987,'1951-A02','1951-02-08 00:00:00','military missile','success',571,'4','','','','Test',1863,'',24 from dual union all
select 9870,'1963-S360','1963-07-26 00:00:00','military missile','success',871,'ZW-66','Nike X R&D','','','Test',1884,'',40 from dual union all
select 9871,'1963-S361','1963-07-26 19:19:00','military missile','failure',155,'24E','Mk 4-5B','OT DASO','SILVER DOLL','Test',1715,'',300 from dual union all
select 9872,'1963-S362','1963-07-26 21:19:00','suborbital rocket','success',215,'AC17.61','D region e density','','','Auroral/Ionos',428,'',143 from dual union all
select 9873,'1963-S363','1963-07-27 00:00:00','military missile','success',794,'514','Mk 5 RV','DASO','DIAL RIGHT','Test',1651,'',900 from dual union all
select 9874,'1963-S364','1963-07-27 00:00:00','military missile','success',976,'','8K63 KY63-20','','','Test',636,'',402 from dual union all
select 9875,'1963-S365','1963-07-27 03:00:00','military missile','success',982,'G 226   02-20T','8K64U UBP-26 K','','','OT',565,'',1210 from dual union all
select 9876,'1963-S366','1963-07-27 00:10:00','suborbital rocket','success',839,'RTG K63-1','MISU Grenades 1','','','Aeronomy',920,'',126 from dual union all
select 9877,'1963-S367','1963-07-25 14:15:00','suborbital rocket','success',30,'NB3.124','Solar XR Spectrum 2','','','Solar X',1869,'',200 from dual;
insert into LAUNCH
select 9878,'1963-S368','1963-07-29 23:28:00','suborbital rocket','success',839,'RTG K63-2','MISU Grenades 2','','','Aeronomy',920,'',128 from dual union all
select 9879,'1963-S369','1963-07-30 18:36:49','military missile','success',155,'70E','Mk 4-5B','OT DASO','BIG FLIGHT','Test',1668,'',1600 from dual union all
select 988,'1951-A03','1951-02-06 00:00:00','atmospheric rocket','success',442,'','RM-10 Model','','','Test',1821,'',10 from dual union all
select 9880,'1963-S37','1963-02-05 00:00:00','military missile','success',976,'','8K63 PRO-13','','','Test',942,'',402 from dual union all
select 9881,'1963-S370','1963-08-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 9882,'1963-S371','1963-08-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 9883,'1963-S372','1963-07-31 20:52:00','military missile','success',154,'143D','Mk 5-5B','ST COOL WATER I','COOL WATER I','Test',1665,'',1000 from dual union all
select 9884,'1963-S373','1963-08-01 08:06:00','suborbital rocket','success',30,'NC3.','Bennett Mass Spec 2','','','Aeron',1895,'',232 from dual union all
select 9885,'1963-S374','1963-08-01 23:27:00','suborbital rocket','success',839,'RTG K63-3','MISU Grenades 3','','','Aeronomy',920,'',126 from dual union all
select 9886,'1963-S375','1963-08-02 08:00:00','suborbital rocket','success',1247,'','NITE OWL 1','','','UV Astron',1057,'F-3B',107 from dual union all
select 9887,'1963-S376','1963-08-02 21:43:00','ballistic missile test','success',1177,'7','San Marco','','','Sat Test',1821,'',294 from dual union all
select 9888,'1963-S377','1963-08-02 23:33:00','suborbital rocket','success',839,'NASA 10.75UA','Density (SM Support)','','','Aeronomy',1821,'',151 from dual union all
select 9889,'1963-S378','1963-08-05 17:30:15','military missile','failure',795,'433','Mk 11 RV','R&D','','Test',266,'',100 from dual union all
select 989,'1951-A04','1951-03-01 00:00:00','atmospheric rocket','failure',565,'','F25 Drag Model','','','Test',1821,'',5 from dual union all
select 9890,'1963-S379','1963-08-05 00:00:00','military missile','success',1053,'CC-2033','','','','OT 333-2',437,'',90 from dual union all
select 9891,'1963-S38','1963-02-06 17:58:24','military missile','success',1420,'N-16','Mk 6 RV','','','Test',230,'',1300 from dual union all
select 9892,'1963-S380','1963-08-07 22:29:00','suborbital rocket','success',839,'RTG K63-4','MISU Grenades 4','','','Aeronomy',920,'',130 from dual union all
select 9893,'1963-S381','1963-08-08 00:00:00','military missile','success',794,'520','Mk 5 RV','DASO','WELL DONE','Test',1647,'',900 from dual union all
select 9894,'1963-S382','1963-08-08 00:00:00','military missile','success',871,'ZW-67','','','','Test',1884,'',50 from dual union all
select 9895,'1963-S383','1963-08-09 17:14:03','military missile','success',929,'A2P-210','','','','OT',421,'SSBN 617',1000 from dual union all
select 9896,'1963-S384','1963-08-09 18:19:43','military missile','success',929,'A2P-209','','','','OT',421,'SSBN 617',1000 from dual union all
select 9897,'1963-S385','1963-08-09 00:00:00','military missile','success',976,'','8K63 60 RD UBP-4','','','Test',1354,'',402 from dual union all
select 9898,'1963-S386','1963-08-14 03:56:05','military missile','success',932,'A3X-36','PX-2(P)','','','Test',244,'',1000 from dual union all
select 9899,'1963-S387','1963-08-13 04:59:47','military missile','success',982,'G 22500-08TS','8K64U UBP-27','','','OT',565,'',757 from dual union all
select 99,'1944-M01','1944-06-13 15:03:00','military missile','failure',8,'V89','Kalmar impact','','','Test',675,'',50 from dual union all
select 990,'1951-A05','1951-03-08 00:00:00','military missile','success',763,'','','','','Test',644,'',10 from dual union all
select 9900,'1963-S388','1963-08-15 00:00:00','military missile','success',871,'ZK-23','K-26 Titan Intercept','','','Test',873,'',80 from dual union all
select 9901,'1963-S389','1963-08-15 00:00:00','military missile','success',1419,'SM-7','Nike TVX-13','DASO/NTMP K-26','HIGH RIVER','OT',1654,'',1000 from dual union all
select 9902,'1963-S39','1963-02-07 15:59:59','military missile','success',932,'A3X-8','[No NF sep]','','','Test',251,'',1000 from dual union all
select 9903,'1963-S390','1963-08-17 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 611',1000 from dual union all
select 9904,'1963-S391','1963-08-17 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 611',1000 from dual union all
select 9905,'1963-S392','1963-08-17 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 611',1000 from dual union all
select 9906,'1963-S393','1963-08-17 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 611',1000 from dual union all
select 9907,'1963-S394','1963-08-17 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 611',1000 from dual union all
select 9908,'1963-S395','1963-08-17 00:00:00','military missile','success',929,'A2','','','','OT',421,'SSBN 611',1000 from dual union all
select 9909,'1963-S396','1963-08-20 04:00:03','military missile','success',932,'A3X-37','PX-2(P)','','','Test',251,'',1000 from dual union all
select 991,'1951-A06','1951-03-09 03:16:00','suborbital rocket','failure',1470,'57','Blossom IVE','','','Ionos/Solar X/Aeron/IR',1863,'',3 from dual union all
select 9910,'1963-S397','1963-08-19 00:00:00','military missile','success',1053,'CC-2008','','','','OT 333-2',437,'',90 from dual union all
select 9911,'1963-S398','1963-08-20 00:00:00','military missile','success',916,'','P-57 63 Fall 01','','','OT',435,'',180 from dual union all
select 9912,'1963-S399','1963-08-20 07:30:00','military missile','success',982,'G 22501-02T','8K64U UBP-28','','','OT',571,'',1210 from dual union all
select 9913,'1963-S40','1963-02-08 00:00:00','military missile','success',976,'','8K63 PRO-14','','','Test',942,'',402 from dual union all
select 9914,'1963-S400','1963-08-21 21:23:55','ballistic missile test','success',1420,'N-24','Mk 6 RV/Pod T-202','R&D','','Test/Plume study',230,'',1300 from dual union all
select 9915,'1963-S401','1963-08-22 18:05:57','suborbital spaceplane','success',1520,'3-22-36','VO,Spectrom,Photom','','','Test/Aeron',379,'NB-52 003',107 from dual union all
select 9916,'1963-S402','1963-08-23 17:01:06','military missile','success',929,'A2P-208','','','','OT',421,'SSBN 617',1000 from dual union all
select 9917,'1963-S403','1963-08-23 17:41:04','military missile','success',929,'A2P-211','','','','OT',421,'SSBN 617',1000 from dual union all
select 9918,'1963-S404','1963-08-23 00:00:00','military missile','success',976,'','8K63 60 RD UBP-5','','','Test',1354,'',402 from dual union all
select 9919,'1963-S405','1963-08-24 09:50:57','military missile','success',155,'72E','Nike TVX-13','OT DASO/NTMP K-28','PIPE DREAM','Test',1671,'',1600 from dual union all
select 992,'1951-A07','1951-03-15 00:00:00','military missile','success',571,'5','','','','Test',1863,'',24 from dual union all
select 9920,'1963-S406','1963-08-24 00:00:00','military missile','success',871,'ZK-24','K-28 Atlas Intercept','','','Test',873,'',80 from dual union all
select 9921,'1963-S407','1963-08-27 17:02:23','military missile','success',795,'435','Mk 11 RV','R&D','','Test',264,'',900 from dual union all
select 9922,'1963-S408','1963-08-27 00:00:00','military missile','success',916,'','P-58 63 Fall 02','','','OT',435,'',180 from dual union all
select 9923,'1963-S409','1963-08-28 23:10:00','military missile','success',154,'142D','Mk 11-5B','ST COOL WATER II','COOL WATER II','Test',1667,'',1800 from dual union all
select 9924,'1963-S41','1963-02-11 15:30:00','military missile','success',932,'A3X-7','','','','Test',244,'',1000 from dual union all
select 9925,'1963-S410','1963-08-30 00:00:00','military missile','success',795,'625','Mk 11 RV','R&D','GLASS WAND','Test',1652,'',900 from dual union all
select 9926,'1963-S411','1963-08-31 01:35:04','military missile','success',932,'A3X-38','PX-2(F)/No A,C sep','','','Test',244,'',1000 from dual union all
select 9927,'1963-S412','1963-08-30 00:00:00','military missile','success',976,'','8K63 KY63-21','','','Test',636,'',402 from dual union all
select 9928,'1963-S413','1963-08-30 00:00:00','military missile','success',979,'','8K65 UBP-46','','','Test',636,'',675 from dual union all
select 9929,'1963-S414','1963-08-30 00:00:00','military missile','success',1419,'SM-56','Mk 4 RV','DASO','POLAR ROUTE','OT',1656,'',1000 from dual union all
select 993,'1951-A08','1951-03-23 00:00:00','atmospheric rocket','success',365,'','XF4D-1 Model','','','Test',1821,'',10 from dual union all
select 9930,'1963-S415','1963-08-30 06:30:26','military missile','success',982,'G 22500-09TS','8K64U UBP-29','','','OT',565,'',754 from dual union all
select 9931,'1963-S416','1963-09-01 00:00:00','military missile','success',7,'6TYa','F1','','','Test',182,'',100 from dual union all
select 9932,'1963-S417','1963-09-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 9933,'1963-S418','1963-09-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 9934,'1963-S419','1963-08-31 00:00:00','military missile','success',976,'','8K63 KY63-22','','','Test',636,'',402 from dual union all
select 9935,'1963-S42','1963-02-11 00:00:00','military missile','success',1018,'E 15001-33','8K75 LKI 3-1','Military test 1','','OT',582,'',1160 from dual union all
select 9936,'1963-S420','1963-09-01 08:03:00','suborbital rocket','success',839,'Ferdinand 5','Ferdinand 5','','','Aeron/Ionos/Solar',29,'',100 from dual union all
select 9937,'1963-S421','1963-09-03 22:36:00','suborbital rocket','success',30,'AD3.374','Mass spectrometer','','','Aeron',1869,'',288 from dual union all
select 9938,'1963-S422','1963-09-04 00:00:00','military missile','success',928,'A1','ORRT','','','OT',421,'SSBN 602',500 from dual union all
select 9939,'1963-S423','1963-09-04 00:00:00','military missile','success',928,'A1','ORRT','','','OT',421,'SSBN 602',500 from dual union all
select 994,'1951-A09','1951-04-01 00:00:00','suborbital rocket','success',1494,'P2','','','','Test',1748,'',0 from dual union all
select 9940,'1963-S424','1963-09-04 00:00:00','military missile','failure',928,'A1','ORRT','','','OT',421,'SSBN 602',10 from dual union all
select 9941,'1963-S425','1963-09-04 00:00:00','military missile','failure',928,'A1','ORRT','','','OT',421,'SSBN 602',100 from dual union all
select 9942,'1963-S426','1963-09-04 00:00:00','military missile','failure',928,'A1','ORRT','','','OT',421,'SSBN 602',10 from dual union all
select 9943,'1963-S427','1963-09-04 00:00:00','military missile','success',928,'A1','ORRT','','','OT',421,'SSBN 602',500 from dual union all
select 9944,'1963-S428','1963-09-04 19:20:00','military missile','success',982,'G 22500-10TS','8K64U UBP-30','','','OT',565,'',1210 from dual union all
select 9945,'1963-S429','1963-09-04 13:11:00','suborbital rocket','success',30,'NASA 04.91GE','CR nuclei','','','Cosmic rays',428,'',228 from dual union all
select 9946,'1963-S43','1963-02-12 01:47:00','sounding rocket','success',630,'NASA 11.06UE','Electron Spectrometer','','','Magnetospheric',1015,'',1675 from dual union all
select 9947,'1963-S430','1963-09-05 00:00:00','suborbital rocket','success',769,'','3A III-1','','','Test',185,'',100 from dual union all
select 9948,'1963-S431','1963-09-05 03:41:00','suborbital rocket','success',1197,'SL87','RAE Test 19','','','Test',1839,'',214 from dual union all
select 9949,'1963-S432','1963-09-06 00:00:00','military missile','success',976,'','8K63 KY63-23','','','Test',636,'',402 from dual union all
select 995,'1951-A10','1951-04-11 00:00:00','military missile','failure',703,'231','','','','Test',222,'',0 from dual union all
select 9950,'1963-S433','1963-09-06 21:30:00','suborbital rocket','success',30,'NASA 04.22US','OSO B experiment','','','Solar',1869,'',222 from dual union all
select 9951,'1963-S434','1963-09-06 21:59:00','military missile','failure',154,'63D','Mk 11-5B','ST COOL WATER III','COOL WATER III','Test',1666,'',200 from dual union all
select 9952,'1963-S435','1963-09-09 00:00:00','military missile','success',1019,'','SSATC Target','','','Test',636,'',200 from dual union all
select 9953,'1963-S436','1963-09-09 16:02:00','suborbital rocket','success',837,'NASA 14.06UE','Electrojet','','','Magnetospheric',1821,'',169 from dual union all
select 9954,'1963-S437','1963-09-10 03:33:00','military missile','success',982,'T 22602-18TB','8K64U UBP-31','','','OT',565,'',1210 from dual union all
select 9955,'1963-S438','1963-09-10 00:00:00','military missile','success',1053,'CC-2015','','','','OT 333-1 Bat B',437,'',90 from dual union all
select 9956,'1963-S439','1963-09-11 14:24:00','suborbital rocket','success',35,'NASA 04.32NP','LeRC LH2 test','','','Tech',1821,'',165 from dual union all
select 9957,'1963-S44','1963-02-13 00:00:00','military missile','success',976,'','8K63 KY63-2','','','Test',636,'',402 from dual union all
select 9958,'1963-S440','1963-09-11 21:00:00','military missile','failure',154,'84D','Mk 11-5B','ST COOL WATER IV','COOL WATER IV','Test',1665,'',1700 from dual union all
select 9959,'1963-S441','1963-09-12 00:00:00','military missile','success',1018,'T 15001-03','8K75 LKI 3-9','','','OT',584,'',1160 from dual union all
select 996,'1951-A100','1951-12-19 00:00:00','atmospheric rocket','success',365,'','XF10F-1 Model','','','Test',1821,'',10 from dual union all
select 9960,'1963-S442','1963-09-12 06:20:00','suborbital rocket','success',837,'Ferdinand 4','Ferdinand 4','','','Ionos/Plasma/Solar',29,'',136 from dual union all
select 9961,'1963-S443','1963-09-17 00:00:00','military missile','success',1018,'T 15001-07','8K75 LKI 3-10','','','OT',584,'',1160 from dual union all
select 9962,'1963-S444','1963-09-17 00:00:00','military missile','success',1419,'SM-83','Mk 4 RV','DASO','DAILY MAIL','OT',1655,'',1000 from dual union all
select 9963,'1963-S445','1963-09-19 00:00:00','military missile','success',976,'','8K63 PRO-26','','','Test',942,'',402 from dual union all
select 9964,'1963-S446','1963-09-20 00:00:00','military missile','success',976,'','8K63 KY63-24','','','Test',636,'',402 from dual union all
select 9965,'1963-S447','1963-09-20 15:45:00','suborbital rocket','success',30,'NE3.151','SX solar spectrum','','','Solar EUV',1869,'',208 from dual union all
select 9966,'1963-S448','1963-09-21 00:00:00','military missile','success',976,'','8K63 KY63-25','','','Test',636,'',402 from dual union all
select 9967,'1963-S449','1963-09-23 00:00:00','military missile','success',1053,'CC-2044','ATBM target','','','OT 333-1',437,'',90 from dual;
insert into LAUNCH
select 9968,'1963-S45','1963-02-13 00:00:00','military missile','success',979,'','8K65 UBP-39','','','Test',636,'',675 from dual union all
select 9969,'1963-S450','1963-09-23 00:00:00','military missile','success',1420,'N-23','Mk 6 RV','R&D','TAR TOP','Test',1659,'',1300 from dual union all
select 997,'1951-A101','1951-12-19 00:00:00','atmospheric rocket','success',442,'','Falcon Missile','','','Test',1821,'',10 from dual union all
select 9970,'1963-S451','1963-09-24 17:02:00','military missile','success',916,'','P-59 63 Fall 03','','','OT',204,'',180 from dual union all
select 9971,'1963-S452','1963-09-25 13:40:00','military missile','success',916,'','P-60 63 Fall 04','','','OT',204,'',180 from dual union all
select 9972,'1963-S453','1963-09-24 04:15:00','suborbital rocket','success',837,'AC5.475','Firefly IV IRMA','','','Aeron/Ionos',394,'',158 from dual union all
select 9973,'1963-S454','1963-09-25 23:05:00','military missile','success',916,'','P-61 63 Fall 05','','','OT',204,'',180 from dual union all
select 9974,'1963-S455','1963-09-25 03:57:00','suborbital rocket','success',839,'NASA 10.92NA','Release','','','Aeronomy',1821,'',143 from dual union all
select 9975,'1963-S456','1963-09-25 04:34:00','suborbital rocket','success',839,'NASA 10.93NA','Release','','','Aeronomy',1821,'',151 from dual union all
select 9976,'1963-S457','1963-09-25 07:09:00','suborbital rocket','success',35,'NASA 04.65GI','Japan-US Ionos 4','','','Ionos',1821,'',224 from dual union all
select 9977,'1963-S458','1963-09-25 11:04:41','military missile','failure',155,'71E','Nike TV-32','OT DASO/NTMP','FILTER TIP','OT',1668,'',100 from dual union all
select 9978,'1963-S459','1963-09-27 01:15:00','military missile','success',795,'770','Mk 11 RV','R&D','STATE PARK','Test',1653,'',900 from dual union all
select 9979,'1963-S46','1963-02-13 11:55:05','ballistic missile test','success',154,'182D','Nike TV-23/NC20.133','NTMP K-8','FLAG RACE','ABM Test',1662,'',1800 from dual union all
select 998,'1951-A102','1951-04-26 00:00:00','atmospheric rocket','failure',1526,'1','','','','Test',1895,'B-29',10 from dual union all
select 9980,'1963-S460','1963-09-27 03:25:00','military missile','success',932,'A3X-39','Steam eject, PX-2','','','Test',421,'EAG-154',1000 from dual union all
select 9981,'1963-S461','1963-09-26 04:00:00','military missile','success',982,'T 22602-19TB','8K64U UBP-32','','','OT',565,'',1210 from dual union all
select 9982,'1963-S462','1963-09-26 04:37:00','suborbital rocket','success',837,'AC5.476','Firefly IV WANDA','','','Aeron',394,'',170 from dual union all
select 9983,'1963-S463','1963-09-26 05:30:00','suborbital rocket','success',837,'AC5.478','Firefly IV VIOLET','','','Aeron',394,'',170 from dual union all
select 9984,'1963-S464','1963-09-26 18:30:00','suborbital rocket','success',30,'AC3.151','UV absorption','','','Aeron',1869,'',98 from dual union all
select 9985,'1963-S465','1963-09-27 13:40:00','military missile','success',916,'','P-62 63 Fall 06','','','OT R=560',204,'',180 from dual union all
select 9986,'1963-S466','1963-09-27 20:40:00','military missile','success',916,'','P-63 63 Fall 07','','','OT',204,'',180 from dual union all
select 9987,'1963-S467','1963-09-28 14:43:00','suborbital rocket','success',35,'NASA 04.64GI','Japan-US Ionos 5','','','Ionos',1821,'',226 from dual union all
select 9988,'1963-S468','1963-09-29 02:37:00','suborbital rocket','success',624,'NASA 8.18GI','Topside sounder','','','Ionos',1821,'',1037 from dual union all
select 9989,'1963-S469','1963-10-01 00:00:00','military missile','success',837,'BMTS','','','','Target',1895,'',100 from dual union all
select 999,'1951-A103','1951-11-01 00:00:00','atmospheric rocket','failure',1526,'2','','','','Test',1895,'B-29',10 from dual union all
select 9990,'1963-S47','1963-02-13 12:30:00','military missile','success',871,'ZK-12','K-8 Atlas Intercept','','','Test',873,'',70 from dual union all
select 9991,'1963-S470','1963-10-01 19:39:05','military missile','success',929,'A2P-222','PY-1 DASO','','','OT',421,'SSBN 619',1000 from dual union all
select 9992,'1963-S471','1963-10-01 00:00:00','military missile','success',976,'','8K63 PRO-27','','','Test',942,'',402 from dual union all
select 9993,'1963-S472','1963-10-01 03:26:00','military missile','success',982,'G 22603-20T','8K64U UBP-33','','','OT',565,'',1210 from dual union all
select 9994,'1963-S473','1963-10-01 10:47:00','suborbital rocket','success',837,'AC5.477','Firefly IV BECKY','','','Aeron',394,'',182 from dual union all
select 9995,'1963-S474','1963-10-01 17:00:00','suborbital rocket','success',30,'NASA 04.78GS','OSO B spectrometer','','','Solar',1869,'',219 from dual union all
select 9996,'1963-S475','1963-10-02 15:10:00','suborbital rocket','success',1198,'SL81A','MRAO-2/UCL I-15','','','Ionos',1839,'',148 from dual union all
select 9997,'1963-S476','1963-10-04 00:00:00','military missile','success',795,'695','Mk 11 RV','R&D','GOLD DUKE','Test',1652,'',900 from dual union all
select 9998,'1963-S477','1963-10-04 05:18:07','military missile','success',932,'A3X-41','PX-2','','','Test',421,'EAG-154',1000 from dual union all
select 9999,'1963-S478','1963-10-04 00:00:00','military missile','success',976,'','8K63 PRO-28','','','Test',942,'',402 from dual;


--------------------------------------------------------------------------------
-- LAUNCH_PAYLOAD_ORG
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing LAUNCH_PAYLOAD_ORG...');

CREATE TABLE "LAUNCH_PAYLOAD_ORG" 
 (	"LAUNCH_ID" NUMBER, 
	"PAYLOAD_ORG_CODE" VARCHAR2(4000), 
	 CONSTRAINT "LAUNCH_PAYLOAD_ORG_PK" PRIMARY KEY ("LAUNCH_ID", "PAYLOAD_ORG_CODE")
USING INDEX  ENABLE, 
	 CONSTRAINT "LAUNCH_PAYLOAD_ORG_LAUNCH_FK" FOREIGN KEY ("LAUNCH_ID")
	  REFERENCES "LAUNCH" ("LAUNCH_ID") ENABLE, 
	 CONSTRAINT "LAUNCH_PAYLOAD_ORG_ORG_FK" FOREIGN KEY ("PAYLOAD_ORG_CODE")
	  REFERENCES "ORGANIZATION" ("ORG_CODE") ENABLE
 ) ;

insert into LAUNCH_PAYLOAD_ORG
select 10002,'BRL' from dual union all
select 10003,'LERC' from dual union all
select 10005,'LERC' from dual union all
select 1001,'UCO' from dual union all
select 10010,'DRTE' from dual union all
select 10013,'QUB' from dual union all
select 10013,'UCL' from dual union all
select 10014,'ICST' from dual union all
select 10014,'UCL' from dual union all
select 10015,'ICST' from dual union all
select 10015,'UCL' from dual union all
select 10016,'ASE' from dual union all
select 10016,'GSFC' from dual union all
select 10017,'QUB' from dual union all
select 10017,'UCL' from dual union all
select 10022,'CNET' from dual union all
select 10025,'CERMA' from dual union all
select 10026,'CARDE' from dual union all
select 10034,'AFCRL' from dual union all
select 10039,'CERMA' from dual union all
select 10040,'AN' from dual union all
select 10043,'DRTE' from dual union all
select 10047,'AFCRL' from dual union all
select 10049,'CARDE' from dual union all
select 10050,'DMA' from dual union all
select 10051,'RAE' from dual union all
select 10051,'WRE' from dual union all
select 10053,'CNET' from dual union all
select 10054,'AFCRL' from dual union all
select 10055,'GSFC' from dual union all
select 10056,'NRL' from dual union all
select 10060,'AFCRL' from dual union all
select 10063,'AFCRL' from dual union all
select 10064,'UMI' from dual union all
select 10065,'UMI' from dual union all
select 10066,'KPNO' from dual union all
select 10067,'LARC' from dual union all
select 10068,'AFCRL' from dual union all
select 10071,'AFCRL' from dual union all
select 10072,'USN' from dual union all
select 10074,'UMI' from dual union all
select 10076,'JHU' from dual union all
select 10078,'NRL' from dual union all
select 10080,'AFCRL' from dual union all
select 10084,'UMI' from dual union all
select 10087,'JPL' from dual union all
select 10088,'LRL' from dual union all
select 10092,'SAND' from dual union all
select 10093,'PRL' from dual union all
select 10095,'SAND' from dual union all
select 10097,'UMI' from dual union all
select 10098,'UMI' from dual union all
select 10102,'NRL' from dual union all
select 10108,'SIMED' from dual union all
select 10109,'RAE' from dual union all
select 10109,'WRE' from dual union all
select 10113,'BIR' from dual union all
select 10113,'LUX' from dual union all
select 10113,'UCW' from dual union all
select 10116,'AFCRL' from dual union all
select 10118,'GSFC' from dual union all
select 10119,'UMI' from dual union all
select 10121,'GCA' from dual union all
select 10122,'ASE' from dual union all
select 10126,'KYO' from dual union all
select 10126,'TOK' from dual union all
select 10128,'ISAS' from dual union all
select 10129,'AFCRL' from dual union all
select 1013,'BU' from dual union all
select 10130,'WRE' from dual union all
select 10131,'AFCRL' from dual union all
select 10132,'GSFC' from dual union all
select 10136,'AFCRL' from dual union all
select 10138,'SAC' from dual union all
select 10140,'UMI' from dual union all
select 10141,'UMI' from dual union all
select 10143,'GSFC' from dual union all
select 10149,'AN' from dual union all
select 10156,'HARP' from dual union all
select 10157,'HARP' from dual union all
select 10158,'HARP' from dual union all
select 10159,'HARP' from dual union all
select 10160,'HARP' from dual union all
select 10161,'HARP' from dual union all
select 10162,'HARP' from dual union all
select 10163,'NMC' from dual union all
select 10164,'NMC' from dual union all
select 10165,'GCA' from dual union all
select 10169,'NOTS' from dual union all
select 1017,'UMI' from dual union all
select 10170,'APGC' from dual union all
select 10171,'NMC' from dual union all
select 10172,'NOTS' from dual union all
select 10174,'APGC' from dual union all
select 10175,'APGC' from dual union all
select 10177,'APGC' from dual union all
select 1018,'NRL' from dual union all
select 10187,'AFCRL' from dual union all
select 10193,'GCA' from dual union all
select 10198,'GSFC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 10199,'GSFC' from dual union all
select 10201,'BIR' from dual union all
select 10201,'LUX' from dual union all
select 10201,'UCL' from dual union all
select 10202,'UNK' from dual union all
select 10205,'AFCRL' from dual union all
select 10208,'GSFC' from dual union all
select 10209,'GSFC' from dual union all
select 10210,'GSFC' from dual union all
select 10211,'BIR' from dual union all
select 10211,'LUX' from dual union all
select 10211,'UCL' from dual union all
select 10214,'WRE' from dual union all
select 10220,'USAF' from dual union all
select 10224,'AFCRL' from dual union all
select 10224,'HCO' from dual union all
select 10225,'LARC' from dual union all
select 10226,'LARC' from dual union all
select 10227,'LARC' from dual union all
select 10228,'TXWC' from dual union all
select 10229,'TXWC' from dual union all
select 10230,'NASA' from dual union all
select 10232,'MNF' from dual union all
select 10233,'MNF' from dual union all
select 10234,'MNF' from dual union all
select 10235,'USAF' from dual union all
select 10236,'ONR' from dual union all
select 10237,'ONR' from dual union all
select 10238,'ONR' from dual union all
select 10239,'MNF' from dual union all
select 10240,'MNF' from dual union all
select 10241,'MSFC' from dual union all
select 10242,'TXWC' from dual union all
select 10243,'UCAN' from dual union all
select 10244,'SUD' from dual union all
select 10245,'SUD' from dual union all
select 10246,'4WG' from dual union all
select 10246,'NASA' from dual union all
select 10247,'MSFC' from dual union all
select 10248,'USAF' from dual union all
select 10249,'TXWC' from dual union all
select 1025,'AFCRL' from dual union all
select 1025,'UCO' from dual union all
select 10250,'TXWC' from dual union all
select 10251,'USAF' from dual union all
select 10252,'USAF' from dual union all
select 10253,'USAF' from dual union all
select 10254,'TXWC' from dual union all
select 10255,'TXWC' from dual union all
select 10256,'USAF' from dual union all
select 10257,'LARC' from dual union all
select 10258,'TXWC' from dual union all
select 10259,'USAF' from dual union all
select 10260,'USAF' from dual union all
select 10261,'MNF' from dual union all
select 10262,'MNF' from dual union all
select 10263,'USAF' from dual union all
select 10264,'USAF' from dual union all
select 10265,'TXWC' from dual union all
select 10266,'TXWC' from dual union all
select 10267,'TXWC' from dual union all
select 10268,'TXWC' from dual union all
select 10269,'TXWC' from dual union all
select 10270,'ONR' from dual union all
select 10271,'ONR' from dual union all
select 10272,'ONR' from dual union all
select 10273,'ONR' from dual union all
select 10274,'WRE' from dual union all
select 10275,'MSFC' from dual union all
select 10276,'MSFC' from dual union all
select 10277,'MSFC' from dual union all
select 10278,'MSFC' from dual union all
select 10279,'UIO' from dual union all
select 10280,'UIO' from dual union all
select 10281,'WRE' from dual union all
select 10282,'WRE' from dual union all
select 10283,'MSFC' from dual union all
select 10284,'MSFC' from dual union all
select 10285,'MSFC' from dual union all
select 10286,'UIO' from dual union all
select 10377,'GCA' from dual union all
select 10380,'LARC' from dual union all
select 10381,'LARC' from dual union all
select 10382,'LRL' from dual union all
select 10383,'GSFC' from dual union all
select 10385,'LARC' from dual union all
select 10387,'SAND' from dual union all
select 10388,'DASA' from dual union all
select 10389,'LARC' from dual union all
select 10391,'JMA' from dual union all
select 10392,'AERAR' from dual union all
select 10393,'LARC' from dual union all
select 10395,'ISAS' from dual union all
select 10396,'LARC' from dual union all
select 10397,'LARC' from dual union all
select 10399,'LARC' from dual union all
select 10400,'DASA' from dual union all
select 10401,'GSFC' from dual union all
select 10402,'DASA' from dual union all
select 10407,'LARC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 10408,'LARC' from dual union all
select 10409,'SAND' from dual union all
select 10412,'WFF' from dual union all
select 10414,'LARC' from dual union all
select 10416,'LARC' from dual union all
select 10417,'ALB' from dual union all
select 10418,'LARC' from dual union all
select 10419,'LARC' from dual union all
select 10420,'DASA' from dual union all
select 10423,'LARC' from dual union all
select 10424,'WFF' from dual union all
select 10436,'LARC' from dual union all
select 10437,'DASA' from dual union all
select 10438,'LARC' from dual union all
select 10440,'MSFC' from dual union all
select 10441,'SAND' from dual union all
select 10443,'SAND' from dual union all
select 10444,'LARC' from dual union all
select 10447,'GSFC' from dual union all
select 10448,'LARC' from dual union all
select 10457,'DASA' from dual union all
select 10458,'LARC' from dual union all
select 10460,'LARC' from dual union all
select 10462,'UMI' from dual union all
select 10463,'APGC' from dual union all
select 10464,'LARC' from dual union all
select 10467,'LARC' from dual union all
select 10471,'LARC' from dual union all
select 10475,'GSFC' from dual union all
select 10477,'ONERA' from dual union all
select 10480,'LRL' from dual union all
select 10482,'UKMO' from dual union all
select 10483,'ONR' from dual union all
select 10484,'ONR' from dual union all
select 10485,'ONR' from dual union all
select 10486,'ONR' from dual union all
select 10487,'ONR' from dual union all
select 10488,'USA' from dual union all
select 10489,'USA' from dual union all
select 10490,'WRE' from dual union all
select 10492,'WRE' from dual union all
select 10493,'LARCN' from dual union all
select 10494,'AEC' from dual union all
select 10494,'LRL' from dual union all
select 10495,'AEC' from dual union all
select 10495,'LRL' from dual union all
select 10496,'NOTS' from dual union all
select 10497,'PMR' from dual union all
select 10500,'WRE' from dual union all
select 10501,'WRE' from dual union all
select 10503,'AFCRL' from dual union all
select 10508,'LARC' from dual union all
select 10509,'LARC' from dual union all
select 10511,'LARC' from dual union all
select 10517,'LARC' from dual union all
select 10518,'LARC' from dual union all
select 10520,'LARC' from dual union all
select 10521,'LARC' from dual union all
select 10522,'LARC' from dual union all
select 10524,'LRL' from dual union all
select 10527,'LARC' from dual union all
select 10529,'LARC' from dual union all
select 10530,'LARC' from dual union all
select 10531,'LARC' from dual union all
select 10533,'LARC' from dual union all
select 10535,'LARC' from dual union all
select 10536,'JPL' from dual union all
select 10537,'LARC' from dual union all
select 10539,'GSFC' from dual union all
select 10539,'LUX' from dual union all
select 10540,'LARC' from dual union all
select 10544,'LARC' from dual union all
select 10545,'LARC' from dual union all
select 10546,'LARC' from dual union all
select 10547,'LARC' from dual union all
select 1055,'LRBA' from dual union all
select 10552,'LARC' from dual union all
select 10553,'LARC' from dual union all
select 10555,'LARC' from dual union all
select 10556,'LARC' from dual union all
select 10557,'LARC' from dual union all
select 10558,'LARC' from dual union all
select 10559,'AFCRL' from dual union all
select 10563,'LARC' from dual union all
select 10564,'LARC' from dual union all
select 10566,'LARC' from dual union all
select 10567,'LARC' from dual union all
select 1057,'LRBA' from dual union all
select 10570,'LARC' from dual union all
select 10572,'LARC' from dual union all
select 10575,'WRE' from dual union all
select 10576,'WRE' from dual union all
select 1058,'LRBA' from dual union all
select 10593,'HARP' from dual union all
select 10594,'SASK' from dual union all
select 10597,'WRE' from dual union all
select 10601,'GSFC' from dual union all
select 10603,'IFAMR' from dual union all
select 10604,'IFAMR' from dual union all
select 10605,'IFAMR' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 10606,'IFAMR' from dual union all
select 10607,'IFAMR' from dual union all
select 10608,'IFAMR' from dual union all
select 10609,'IFAMR' from dual union all
select 10610,'IFAMR' from dual union all
select 10611,'IFAMR' from dual union all
select 10612,'IFAMR' from dual union all
select 10614,'IFAMR' from dual union all
select 10615,'IFAMR' from dual union all
select 10616,'IFAMR' from dual union all
select 10617,'IFAMR' from dual union all
select 10618,'IFAMR' from dual union all
select 10619,'IFAMR' from dual union all
select 10620,'IFAMR' from dual union all
select 10621,'IFAMR' from dual union all
select 10622,'IFAMR' from dual union all
select 10623,'IFAMR' from dual union all
select 10625,'IFAMR' from dual union all
select 10626,'IFAMR' from dual union all
select 10627,'IFAMR' from dual union all
select 10628,'IFAMR' from dual union all
select 10629,'IFAMR' from dual union all
select 10630,'IFAMR' from dual union all
select 10631,'IFAMR' from dual union all
select 10632,'IFAMR' from dual union all
select 10633,'IFAMR' from dual union all
select 10634,'LARCN' from dual union all
select 10653,'NOTS' from dual union all
select 10654,'NOTS' from dual union all
select 10655,'NOTS' from dual union all
select 10657,'NOL' from dual union all
select 10658,'NOTS' from dual union all
select 10659,'PMR' from dual union all
select 10660,'SAND' from dual union all
select 10663,'STA' from dual union all
select 10664,'SIMED' from dual union all
select 10665,'STA' from dual union all
select 10667,'STA' from dual union all
select 10673,'LPA' from dual union all
select 10674,'AFCRL' from dual union all
select 10681,'IFAMR' from dual union all
select 10682,'IFAMR' from dual union all
select 10683,'IFAMR' from dual union all
select 10684,'IFAMR' from dual union all
select 10685,'IFAMR' from dual union all
select 10686,'IFAMR' from dual union all
select 10687,'IFAMR' from dual union all
select 10688,'IFAMR' from dual union all
select 10689,'IFAMR' from dual union all
select 10690,'IFAMR' from dual union all
select 10691,'IFAMR' from dual union all
select 10692,'IFAMR' from dual union all
select 10693,'IFAMR' from dual union all
select 10694,'IFAMR' from dual union all
select 10695,'IFAMR' from dual union all
select 10696,'IFAMR' from dual union all
select 10707,'NRL' from dual union all
select 10709,'AFCRL' from dual union all
select 10711,'AFCRL' from dual union all
select 10716,'RAE' from dual union all
select 10716,'WRE' from dual union all
select 10717,'GSFC' from dual union all
select 10723,'UNK' from dual union all
select 10726,'JHU' from dual union all
select 10732,'GSFC' from dual union all
select 10738,'WRE' from dual union all
select 10740,'GSFC' from dual union all
select 10742,'SAND' from dual union all
select 10743,'ICST' from dual union all
select 10743,'LUX' from dual union all
select 10743,'SOTON' from dual union all
select 10749,'AFCRL' from dual union all
select 10750,'ICST' from dual union all
select 10750,'LUX' from dual union all
select 10750,'SOTON' from dual union all
select 10751,'GSFC' from dual union all
select 10751,'NDRE' from dual union all
select 10753,'AFCRL' from dual union all
select 10755,'UMI' from dual union all
select 10756,'LRL' from dual union all
select 10759,'BERG' from dual union all
select 10759,'GSFC' from dual union all
select 10763,'NOTS' from dual union all
select 10764,'SAND' from dual union all
select 10765,'RICE' from dual union all
select 10766,'BERG' from dual union all
select 10766,'NDRE' from dual union all
select 10769,'RICE' from dual union all
select 10772,'RICE' from dual union all
select 10774,'AFCRL' from dual union all
select 10775,'GSFC' from dual union all
select 10776,'CRA' from dual union all
select 10777,'GSFC' from dual union all
select 10781,'GSFC' from dual union all
select 10783,'APGC' from dual union all
select 10784,'CRA' from dual union all
select 10785,'AFCRL' from dual union all
select 10787,'DASA' from dual union all
select 10789,'NOTS' from dual union all
select 10790,'ISAS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 10791,'AFCRL' from dual union all
select 10792,'CRA' from dual union all
select 10795,'GSFC' from dual union all
select 10797,'PRL' from dual union all
select 10799,'NRCC' from dual union all
select 10800,'KPNO' from dual union all
select 10801,'SASK' from dual union all
select 10803,'AFCRL' from dual union all
select 10804,'SUPA' from dual union all
select 10805,'LARC' from dual union all
select 10806,'UAF' from dual union all
select 10807,'CNRS' from dual union all
select 10809,'CNRS' from dual union all
select 10810,'UKMO' from dual union all
select 10812,'CNRS' from dual union all
select 10813,'CNRS' from dual union all
select 10814,'CNRS' from dual union all
select 10815,'UAF' from dual union all
select 10816,'WRE' from dual union all
select 10817,'UMI' from dual union all
select 10818,'UMI' from dual union all
select 10820,'GSFC' from dual union all
select 10821,'SASK' from dual union all
select 10822,'UIUC' from dual union all
select 10823,'BAJ' from dual union all
select 10824,'UMI' from dual union all
select 10825,'GSFC' from dual union all
select 10826,'GSFC' from dual union all
select 10830,'ALB' from dual union all
select 10832,'BAJ' from dual union all
select 10833,'UAF' from dual union all
select 10839,'DASA' from dual union all
select 10841,'DASA' from dual union all
select 10842,'CNET' from dual union all
select 10843,'NRL' from dual union all
select 10845,'USN' from dual union all
select 10846,'CNET' from dual union all
select 10847,'GRI' from dual union all
select 10848,'CNRS' from dual union all
select 10850,'SAND' from dual union all
select 10852,'UMI' from dual union all
select 10853,'SAND' from dual union all
select 10854,'GRI' from dual union all
select 10855,'RAE' from dual union all
select 10855,'WRE' from dual union all
select 10856,'WRE' from dual union all
select 10857,'SAND' from dual union all
select 10861,'PRL' from dual union all
select 10862,'SAND' from dual union all
select 10863,'AFCRL' from dual union all
select 10864,'AFCRL' from dual union all
select 10865,'AFCRL' from dual union all
select 10866,'AFCRL' from dual union all
select 10867,'AFCRL' from dual union all
select 10868,'AFCRL' from dual union all
select 10869,'SAND' from dual union all
select 10873,'SAND' from dual union all
select 10876,'LASL' from dual union all
select 10879,'LASL' from dual union all
select 10880,'AFCRL' from dual union all
select 10880,'ASE' from dual union all
select 10881,'NOTS' from dual union all
select 10888,'AFCRL' from dual union all
select 10889,'DASA' from dual union all
select 10892,'GSFC' from dual union all
select 10894,'SAND' from dual union all
select 10898,'LRBA' from dual union all
select 10901,'AFCRL' from dual union all
select 10902,'LERC' from dual union all
select 10903,'GSFC' from dual union all
select 10907,'WRE' from dual union all
select 10909,'LRBA' from dual union all
select 1091,'AFCRL' from dual union all
select 10911,'NRL' from dual union all
select 10912,'UMI' from dual union all
select 10914,'UMI' from dual union all
select 10915,'ALB' from dual union all
select 10916,'AFCRL' from dual union all
select 10917,'UMI' from dual union all
select 10918,'USN' from dual union all
select 10919,'KPNO' from dual union all
select 1092,'HAM' from dual union all
select 10920,'USN' from dual union all
select 10926,'AFCRL' from dual union all
select 10927,'GSFC' from dual union all
select 10928,'GSFC' from dual union all
select 1093,'SCEL' from dual union all
select 10931,'SAND' from dual union all
select 10935,'RICE' from dual union all
select 10936,'SAND' from dual union all
select 10939,'AFCRL' from dual union all
select 10940,'SAND' from dual union all
select 10941,'LIEGE' from dual union all
select 10941,'MPE' from dual union all
select 10944,'NRL' from dual union all
select 10945,'GCA' from dual union all
select 1095,'AFCRL' from dual union all
select 10950,'LIEGE' from dual union all
select 10950,'MPE' from dual union all
select 10954,'RICE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 10955,'WRE' from dual union all
select 10959,'TOKAI' from dual union all
select 1096,'AFCRL' from dual union all
select 1096,'HADC' from dual union all
select 10961,'WRE' from dual union all
select 10963,'WRE' from dual union all
select 10964,'RAE' from dual union all
select 10964,'WRE' from dual union all
select 10965,'GCA' from dual union all
select 10966,'WRE' from dual union all
select 10968,'GCA' from dual union all
select 10969,'UIUC' from dual union all
select 1097,'UMI' from dual union all
select 10970,'GCA' from dual union all
select 10971,'GCA' from dual union all
select 10972,'UIUC' from dual union all
select 10973,'UIUC' from dual union all
select 10974,'LRL' from dual union all
select 10977,'WRE' from dual union all
select 10979,'WRE' from dual union all
select 1098,'HAM' from dual union all
select 10980,'GSFC' from dual union all
select 10981,'NOTS' from dual union all
select 10985,'RAE' from dual union all
select 10985,'WRE' from dual union all
select 10986,'WRE' from dual union all
select 10987,'AFCRL' from dual union all
select 10989,'GCA' from dual union all
select 1099,'UMI' from dual union all
select 10990,'AFCRL' from dual union all
select 10990,'NEU' from dual union all
select 10991,'LERC' from dual union all
select 10992,'LRL' from dual union all
select 10993,'LASL' from dual union all
select 10994,'LASL' from dual union all
select 10996,'STA' from dual union all
select 10997,'STA' from dual union all
select 10998,'GSFC' from dual union all
select 10999,'GCA' from dual union all
select 1100,'SCEL' from dual union all
select 11000,'GSFC' from dual union all
select 11001,'ISAS' from dual union all
select 11002,'TAO' from dual union all
select 11005,'AFCRL' from dual union all
select 11006,'GSFC' from dual union all
select 11006,'UMI' from dual union all
select 11008,'AFCRL' from dual union all
select 11009,'ISAS' from dual union all
select 1101,'APL' from dual union all
select 11010,'NRL' from dual union all
select 11011,'SAND' from dual union all
select 11019,'CNRS' from dual union all
select 1102,'APL' from dual union all
select 11023,'LASL' from dual union all
select 11028,'AFCRL' from dual union all
select 11028,'MISU' from dual union all
select 11031,'CNRS' from dual union all
select 11032,'CARDE' from dual union all
select 11033,'GSFC' from dual union all
select 11033,'MISU' from dual union all
select 11034,'GSFC' from dual union all
select 11036,'LASL' from dual union all
select 11037,'GSFC' from dual union all
select 11038,'LRL' from dual union all
select 11040,'CULH' from dual union all
select 11040,'LUX' from dual union all
select 11041,'GSFC' from dual union all
select 11042,'CARDE' from dual union all
select 11043,'AFCRL' from dual union all
select 11043,'MISU' from dual union all
select 11044,'GSFC' from dual union all
select 11044,'MISU' from dual union all
select 11045,'GSFC' from dual union all
select 11046,'GSFC' from dual union all
select 11049,'AFCRL' from dual union all
select 11049,'MISU' from dual union all
select 11050,'GSFC' from dual union all
select 11050,'MISU' from dual union all
select 11051,'GSFC' from dual union all
select 11052,'GSFC' from dual union all
select 11055,'AFCRL' from dual union all
select 11055,'MISU' from dual union all
select 11056,'GSFC' from dual union all
select 11056,'MISU' from dual union all
select 11057,'GSFC' from dual union all
select 11058,'GSFC' from dual union all
select 11059,'GSFC' from dual union all
select 11062,'LUX' from dual union all
select 11062,'UCW' from dual union all
select 11063,'WRE' from dual union all
select 11066,'GSFC' from dual union all
select 11068,'AFCRL' from dual union all
select 11070,'GSFC' from dual union all
select 11072,'ASE' from dual union all
select 11072,'MIT' from dual union all
select 11073,'AFAPL' from dual union all
select 11073,'EOS' from dual union all
select 11077,'DASA' from dual union all
select 11078,'DASA' from dual union all
select 11079,'DASA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 1108,'APL' from dual union all
select 11082,'UKMO' from dual union all
select 11083,'UWI' from dual union all
select 11084,'RAE' from dual union all
select 11084,'WRE' from dual union all
select 1109,'AFCRL' from dual union all
select 11092,'FIAN' from dual union all
select 11094,'SASK' from dual union all
select 11095,'KPNO' from dual union all
select 11096,'AFCRL' from dual union all
select 11099,'LUX' from dual union all
select 11099,'UCW' from dual union all
select 11100,'WRE' from dual union all
select 11101,'JPL' from dual union all
select 11102,'AFCRL' from dual union all
select 11108,'UMI' from dual union all
select 11110,'LUX' from dual union all
select 11110,'RSRS' from dual union all
select 11113,'SAND' from dual union all
select 11116,'AN' from dual union all
select 11117,'GSFC' from dual union all
select 11119,'GSFC' from dual union all
select 11122,'LUX' from dual union all
select 11122,'RSRS' from dual union all
select 11124,'NMSU' from dual union all
select 11130,'LPARL' from dual union all
select 11132,'CNET' from dual union all
select 11134,'CARDE' from dual union all
select 11135,'CNET' from dual union all
select 11136,'CARDE' from dual union all
select 11137,'GCA' from dual union all
select 11138,'SAND' from dual union all
select 11139,'LPARL' from dual union all
select 11140,'UNH' from dual union all
select 11141,'GCA' from dual union all
select 11142,'GSFC' from dual union all
select 11149,'NOTS' from dual union all
select 11150,'SAND' from dual union all
select 11151,'WRE' from dual union all
select 11154,'AFCRL' from dual union all
select 11155,'AFCRL' from dual union all
select 11160,'GSFC' from dual union all
select 11160,'RRL' from dual union all
select 11163,'GSFC' from dual union all
select 11164,'WRE' from dual union all
select 11165,'WRE' from dual union all
select 11166,'GSFC' from dual union all
select 11167,'SAND' from dual union all
select 11168,'ASE' from dual union all
select 11168,'MIT' from dual union all
select 11169,'UNH' from dual union all
select 11170,'LUX' from dual union all
select 11170,'UCL' from dual union all
select 11171,'AFCRL' from dual union all
select 11172,'AFCRL' from dual union all
select 11173,'SAND' from dual union all
select 11182,'LPA' from dual union all
select 11183,'GSFC' from dual union all
select 11184,'DASA' from dual union all
select 11185,'DASA' from dual union all
select 11186,'DASA' from dual union all
select 11187,'DASA' from dual union all
select 11188,'DASA' from dual union all
select 11189,'DASA' from dual union all
select 11191,'SAND' from dual union all
select 11192,'GCA' from dual union all
select 11192,'GSFC' from dual union all
select 11193,'SAND' from dual union all
select 11194,'SAND' from dual union all
select 11197,'TAO' from dual union all
select 11198,'PRIN' from dual union all
select 11199,'AFCRL' from dual union all
select 11200,'SAND' from dual union all
select 11201,'GSFC' from dual union all
select 11202,'GSFC' from dual union all
select 11205,'ONUSJ' from dual union all
select 11206,'CNRS' from dual union all
select 11208,'CNRS' from dual union all
select 11209,'CNRS' from dual union all
select 11210,'CNRS' from dual union all
select 11211,'CNRS' from dual union all
select 11213,'CNRS' from dual union all
select 11214,'ISAS' from dual union all
select 11215,'GSFC' from dual union all
select 11216,'JHU' from dual union all
select 11217,'BRL' from dual union all
select 11218,'BRL' from dual union all
select 11219,'ONUSJ' from dual union all
select 11222,'SAND' from dual union all
select 11223,'GSFC' from dual union all
select 11223,'UMI' from dual union all
select 11224,'CNRS' from dual union all
select 11225,'CNRS' from dual union all
select 11226,'GSFC' from dual union all
select 11227,'PRL' from dual union all
select 11228,'GSFC' from dual union all
select 11229,'ONUSJ' from dual union all
select 11230,'GSFC' from dual union all
select 11231,'UCL' from dual union all
select 11233,'BRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 11234,'GSFC' from dual union all
select 11235,'BRL' from dual union all
select 11235,'UMI' from dual union all
select 11236,'GSFC' from dual union all
select 11237,'CNRS' from dual union all
select 11239,'SAND' from dual union all
select 11240,'PRL' from dual union all
select 11241,'USAF' from dual union all
select 11242,'USAF' from dual union all
select 11243,'PRL' from dual union all
select 11244,'OSAKA' from dual union all
select 11245,'GSFC' from dual union all
select 11246,'UIUC' from dual union all
select 11247,'GCA' from dual union all
select 11248,'GCA' from dual union all
select 11249,'AFCRL' from dual union all
select 11250,'WRE' from dual union all
select 11251,'GCA' from dual union all
select 11252,'GCA' from dual union all
select 11253,'OSAKA' from dual union all
select 11254,'WRE' from dual union all
select 11255,'GCA' from dual union all
select 11256,'UNH' from dual union all
select 11257,'GCA' from dual union all
select 11258,'NRL' from dual union all
select 11259,'SAND' from dual union all
select 11261,'GSFC' from dual union all
select 11263,'GSFC' from dual union all
select 11264,'ARC' from dual union all
select 11266,'UMI' from dual union all
select 11267,'GSFC' from dual union all
select 11268,'UMI' from dual union all
select 11269,'SAND' from dual union all
select 11272,'AFCRL' from dual union all
select 11276,'AFCRL' from dual union all
select 11277,'NRL' from dual union all
select 11279,'GSFC' from dual union all
select 11280,'UIUC' from dual union all
select 11281,'UIUC' from dual union all
select 11282,'AFCRL' from dual union all
select 11283,'SAND' from dual union all
select 11284,'AFCRL' from dual union all
select 11285,'GSFC' from dual union all
select 11286,'SAND' from dual union all
select 11287,'NRL' from dual union all
select 11288,'NRCC' from dual union all
select 11290,'AFCRL' from dual union all
select 11291,'MPE' from dual union all
select 11293,'LRL' from dual union all
select 11294,'SAND' from dual union all
select 11296,'DASA' from dual union all
select 11297,'DASA' from dual union all
select 11298,'SUPA' from dual union all
select 11300,'MPE' from dual union all
select 11301,'SAND' from dual union all
select 11302,'SUPA' from dual union all
select 11303,'TUC' from dual union all
select 11304,'GSFC' from dual union all
select 11307,'MPE' from dual union all
select 11309,'TUC' from dual union all
select 11310,'UNH' from dual union all
select 11312,'UNH' from dual union all
select 11314,'WRE' from dual union all
select 11315,'AFCRL' from dual union all
select 11318,'WRE' from dual union all
select 11319,'NOTS' from dual union all
select 11325,'NMC' from dual union all
select 11326,'RAE' from dual union all
select 11326,'WRE' from dual union all
select 11327,'GSFC' from dual union all
select 11328,'GSFC' from dual union all
select 11329,'NMC' from dual union all
select 1133,'IAM' from dual union all
select 11330,'CULH' from dual union all
select 11330,'LUX' from dual union all
select 11331,'JHU' from dual union all
select 11333,'USN' from dual union all
select 11338,'AFAPL' from dual union all
select 11338,'EOS' from dual union all
select 11348,'BAJ' from dual union all
select 11349,'BAJ' from dual union all
select 1135,'FIAN' from dual union all
select 1135,'IAM' from dual union all
select 11352,'HARP' from dual union all
select 11354,'HARP' from dual union all
select 11355,'HARP' from dual union all
select 11356,'HARP' from dual union all
select 11357,'HARP' from dual union all
select 11358,'HARP' from dual union all
select 11359,'HARP' from dual union all
select 1136,'BU' from dual union all
select 11360,'HARP' from dual union all
select 11361,'HARP' from dual union all
select 11362,'HARP' from dual union all
select 11363,'HARP' from dual union all
select 11365,'HARP' from dual union all
select 11366,'HARP' from dual union all
select 11367,'HARP' from dual union all
select 11368,'HARP' from dual union all
select 11369,'HARP' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 1137,'NRL' from dual union all
select 11370,'HARP' from dual union all
select 11371,'HARP' from dual union all
select 11372,'HARP' from dual union all
select 11373,'HARP' from dual union all
select 11374,'HARP' from dual union all
select 11375,'GSFC' from dual union all
select 11376,'HARP' from dual union all
select 11377,'HARP' from dual union all
select 11378,'HARP' from dual union all
select 11379,'HARP' from dual union all
select 1138,'FIAN' from dual union all
select 1138,'IAM' from dual union all
select 11380,'HARP' from dual union all
select 11381,'HARP' from dual union all
select 11382,'HARP' from dual union all
select 11383,'HARP' from dual union all
select 11384,'HARP' from dual union all
select 11385,'HARP' from dual union all
select 11387,'HARP' from dual union all
select 11388,'HARP' from dual union all
select 11389,'HARP' from dual union all
select 1139,'IAM' from dual union all
select 11390,'HARP' from dual union all
select 11391,'HARP' from dual union all
select 11392,'HARP' from dual union all
select 11393,'HARP' from dual union all
select 11394,'HARP' from dual union all
select 11395,'HARP' from dual union all
select 11396,'HARP' from dual union all
select 11397,'RAE' from dual union all
select 11397,'WRE' from dual union all
select 11398,'HARP' from dual union all
select 11399,'HARP' from dual union all
select 1140,'SCEL' from dual union all
select 11400,'HARP' from dual union all
select 11401,'HARP' from dual union all
select 11402,'HARP' from dual union all
select 11403,'HARP' from dual union all
select 11404,'HARP' from dual union all
select 11405,'HARP' from dual union all
select 11406,'HARP' from dual union all
select 11407,'HARP' from dual union all
select 11408,'UMI' from dual union all
select 11409,'HARP' from dual union all
select 1141,'FIAN' from dual union all
select 1141,'IAM' from dual union all
select 11410,'HARP' from dual union all
select 11411,'HARP' from dual union all
select 11412,'HARP' from dual union all
select 11413,'HARP' from dual union all
select 11414,'HARP' from dual union all
select 11415,'HARP' from dual union all
select 11416,'HARP' from dual union all
select 11417,'HARP' from dual union all
select 11418,'HARP' from dual union all
select 11419,'GSFC' from dual union all
select 1142,'FIAN' from dual union all
select 1142,'IAM' from dual union all
select 11420,'HARP' from dual union all
select 11421,'HARP' from dual union all
select 11422,'HARP' from dual union all
select 11423,'HARP' from dual union all
select 11424,'HARP' from dual union all
select 11425,'HARP' from dual union all
select 11426,'GSFC' from dual union all
select 11427,'NMC' from dual union all
select 11428,'NMC' from dual union all
select 11429,'NMC' from dual union all
select 1143,'BU' from dual union all
select 11430,'GSFC' from dual union all
select 11431,'NMC' from dual union all
select 11432,'NMC' from dual union all
select 11433,'NMC' from dual union all
select 11434,'NMC' from dual union all
select 11435,'NMC' from dual union all
select 11436,'NMC' from dual union all
select 11437,'NMC' from dual union all
select 11438,'NMSU' from dual union all
select 11439,'WRE' from dual union all
select 1144,'SCEL' from dual union all
select 1144,'UMI' from dual union all
select 11440,'NMSU' from dual union all
select 11442,'LRS' from dual union all
select 11443,'LRS' from dual union all
select 11444,'LRS' from dual union all
select 11449,'NMSU' from dual union all
select 1145,'SCEL' from dual union all
select 11450,'NMSU' from dual union all
select 11454,'PMR' from dual union all
select 11455,'NMC' from dual union all
select 11456,'PMR' from dual union all
select 11457,'PMR' from dual union all
select 11458,'NMC' from dual union all
select 11459,'USN' from dual union all
select 11460,'PMR' from dual union all
select 11461,'PMR' from dual union all
select 11463,'CNRS' from dual union all
select 11474,'UUT' from dual union all
select 11475,'AFCRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 11476,'DMA' from dual union all
select 11479,'CNRS' from dual union all
select 11480,'CNRS' from dual union all
select 11481,'CNRS' from dual union all
select 11482,'GSFC' from dual union all
select 11484,'CNRS' from dual union all
select 11485,'CNRS' from dual union all
select 11486,'GSFC' from dual union all
select 11487,'GSFC' from dual union all
select 11488,'GSFC' from dual union all
select 11490,'MPE' from dual union all
select 11491,'DAC' from dual union all
select 11492,'MPE' from dual union all
select 11493,'LARC' from dual union all
select 11494,'LARC' from dual union all
select 11496,'LARC' from dual union all
select 11497,'LARC' from dual union all
select 11498,'USAF' from dual union all
select 11499,'USAF' from dual union all
select 11500,'USAF' from dual union all
select 11501,'USAF' from dual union all
select 11502,'GSFC' from dual union all
select 11503,'SAMSO' from dual union all
select 11504,'SAMSO' from dual union all
select 11505,'GSFC' from dual union all
select 11506,'USAF' from dual union all
select 11507,'USAF' from dual union all
select 11508,'USAF' from dual union all
select 11509,'USAF' from dual union all
select 11510,'USAF' from dual union all
select 11511,'USAF' from dual union all
select 11512,'USAF' from dual union all
select 11513,'WSMR' from dual union all
select 11514,'JMA' from dual union all
select 11515,'JMA' from dual union all
select 11516,'USAF' from dual union all
select 11517,'USAF' from dual union all
select 11518,'USAF' from dual union all
select 11519,'USAF' from dual union all
select 11520,'USAF' from dual union all
select 11521,'USAF' from dual union all
select 11522,'USAF' from dual union all
select 11523,'USAF' from dual union all
select 11524,'USAF' from dual union all
select 11525,'UCAN' from dual union all
select 11526,'WSMR' from dual union all
select 11527,'DASA' from dual union all
select 11528,'ISRO' from dual union all
select 11529,'USAF' from dual union all
select 11530,'KSC' from dual union all
select 11531,'KSC' from dual union all
select 11532,'PSU' from dual union all
select 11533,'ISRO' from dual union all
select 11534,'ISRO' from dual union all
select 11535,'AFCRL' from dual union all
select 11535,'ATLR' from dual union all
select 11536,'DASA' from dual union all
select 11536,'PSU' from dual union all
select 11537,'DASA' from dual union all
select 11538,'ONERA' from dual union all
select 11539,'SUPA' from dual union all
select 11540,'USAF' from dual union all
select 11541,'SUPA' from dual union all
select 11542,'SUPA' from dual union all
select 11543,'SUPA' from dual union all
select 11544,'UKMO' from dual union all
select 11545,'UKMO' from dual union all
select 11546,'UKMO' from dual union all
select 11547,'ONR' from dual union all
select 11548,'ONR' from dual union all
select 11549,'ONR' from dual union all
select 11550,'ONR' from dual union all
select 11551,'USAF' from dual union all
select 11552,'ONR' from dual union all
select 11553,'PSU' from dual union all
select 11554,'USAF' from dual union all
select 11555,'ISAS' from dual union all
select 11556,'USAF' from dual union all
select 11557,'USAF' from dual union all
select 11558,'USAF' from dual union all
select 11559,'USAF' from dual union all
select 11560,'USAF' from dual union all
select 11561,'USAF' from dual union all
select 11562,'USAF' from dual union all
select 11563,'USAF' from dual union all
select 11564,'USAF' from dual union all
select 11565,'USAF' from dual union all
select 11566,'USAF' from dual union all
select 11567,'USAF' from dual union all
select 11568,'USAF' from dual union all
select 11569,'USAF' from dual union all
select 11570,'USAF' from dual union all
select 11571,'USAF' from dual union all
select 11572,'USAF' from dual union all
select 11573,'USAF' from dual union all
select 11574,'WFF' from dual union all
select 11575,'USAF' from dual union all
select 11576,'USAF' from dual union all
select 11577,'USAF' from dual union all
select 11578,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 11579,'WFF' from dual union all
select 11580,'ISRO' from dual union all
select 11581,'ISRO' from dual union all
select 11582,'USAF' from dual union all
select 11583,'ISRO' from dual union all
select 11584,'WFF' from dual union all
select 11585,'WFF' from dual union all
select 11586,'USAF' from dual union all
select 11587,'USAF' from dual union all
select 11588,'USAF' from dual union all
select 11589,'USAF' from dual union all
select 11590,'USAF' from dual union all
select 11591,'UMI' from dual union all
select 11592,'USAF' from dual union all
select 11593,'UMI' from dual union all
select 11594,'ETAC' from dual union all
select 11595,'ETAC' from dual union all
select 11596,'ETAC' from dual union all
select 11597,'LARC' from dual union all
select 11598,'LARC' from dual union all
select 11599,'ISRO' from dual union all
select 11600,'ETAC' from dual union all
select 11601,'MSFC' from dual union all
select 11602,'WSMR' from dual union all
select 11603,'WSMR' from dual union all
select 11604,'MSFC' from dual union all
select 11605,'MSFC' from dual union all
select 11606,'MSFC' from dual union all
select 11607,'USAF' from dual union all
select 11608,'USAF' from dual union all
select 11609,'USAF' from dual union all
select 11610,'UMI' from dual union all
select 11611,'UMI' from dual union all
select 11612,'USAF' from dual union all
select 11613,'UMI' from dual union all
select 11614,'WSMR' from dual union all
select 11615,'DASA' from dual union all
select 11616,'USAF' from dual union all
select 11617,'USAF' from dual union all
select 11618,'USAF' from dual union all
select 11619,'USAF' from dual union all
select 11620,'USAF' from dual union all
select 11621,'USAF' from dual union all
select 11622,'USAF' from dual union all
select 11623,'USAF' from dual union all
select 11624,'MSFC' from dual union all
select 11625,'USAF' from dual union all
select 11626,'USAF' from dual union all
select 11627,'USAF' from dual union all
select 11628,'DASA' from dual union all
select 11629,'USAF' from dual union all
select 11630,'USAF' from dual union all
select 11631,'USAF' from dual union all
select 11632,'USAF' from dual union all
select 11633,'MSFC' from dual union all
select 11634,'USAF' from dual union all
select 11635,'USAF' from dual union all
select 11636,'USAF' from dual union all
select 11637,'USAF' from dual union all
select 11638,'USAF' from dual union all
select 11639,'ERDL' from dual union all
select 11640,'ERDL' from dual union all
select 11645,'MSFC' from dual union all
select 11646,'MSFC' from dual union all
select 11647,'MSFC' from dual union all
select 11648,'UIO' from dual union all
select 11659,'SUPA' from dual union all
select 1166,'LRBA' from dual union all
select 11660,'MSFC' from dual union all
select 11661,'MSFC' from dual union all
select 11662,'MSFC' from dual union all
select 11741,'MSFC' from dual union all
select 11742,'MSFC' from dual union all
select 1177,'LRBA' from dual union all
select 11828,'AFCRL' from dual union all
select 11828,'NRL' from dual union all
select 11863,'PIHM' from dual union all
select 11864,'PIHM' from dual union all
select 11865,'PIHM' from dual union all
select 11866,'PIHM' from dual union all
select 11868,'LARC' from dual union all
select 11869,'LRL' from dual union all
select 11872,'LARC' from dual union all
select 11873,'AFCRL' from dual union all
select 11874,'LARC' from dual union all
select 11876,'AFCRL' from dual union all
select 11879,'ISAS' from dual union all
select 11880,'ISAS' from dual union all
select 11881,'LARC' from dual union all
select 11883,'DASA' from dual union all
select 11885,'DASA' from dual union all
select 11889,'LUX' from dual union all
select 11889,'UCL' from dual union all
select 11894,'LARC' from dual union all
select 11895,'LARC' from dual union all
select 11901,'ISAS' from dual union all
select 11902,'ISAS' from dual union all
select 11905,'BAJ' from dual union all
select 11907,'UCB' from dual union all
select 11909,'UCB' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 11911,'LARC' from dual union all
select 11913,'JMA' from dual union all
select 11917,'LARC' from dual union all
select 11918,'GCA' from dual union all
select 11918,'GSFC' from dual union all
select 11920,'PSU' from dual union all
select 11923,'LARC' from dual union all
select 11924,'NMC' from dual union all
select 11926,'LARC' from dual union all
select 11930,'LARC' from dual union all
select 11933,'ISAS' from dual union all
select 11936,'UMI' from dual union all
select 11939,'STA' from dual union all
select 11940,'STA' from dual union all
select 11941,'LARC' from dual union all
select 11942,'LARC' from dual union all
select 11944,'LRL' from dual union all
select 11945,'LARC' from dual union all
select 11950,'LARC' from dual union all
select 11951,'LARC' from dual union all
select 11956,'SAND' from dual union all
select 11957,'HARP' from dual union all
select 11958,'HARP' from dual union all
select 11960,'HARP' from dual union all
select 11961,'HARP' from dual union all
select 11962,'BRL' from dual union all
select 11963,'IIAE' from dual union all
select 11964,'IIAE' from dual union all
select 11965,'IIAE' from dual union all
select 11966,'IIAE' from dual union all
select 11967,'AFCRL' from dual union all
select 11968,'AFCRL' from dual union all
select 11969,'AFCRL' from dual union all
select 11970,'USN' from dual union all
select 11974,'LARC' from dual union all
select 11984,'WRE' from dual union all
select 11986,'WRE' from dual union all
select 11988,'WFF' from dual union all
select 1199,'LRBA' from dual union all
select 11993,'AFCRL' from dual union all
select 11994,'AFCRL' from dual union all
select 11995,'AFCRL' from dual union all
select 11996,'LARC' from dual union all
select 11997,'AEC' from dual union all
select 11997,'SAND' from dual union all
select 11998,'AEC' from dual union all
select 11998,'SAND' from dual union all
select 11999,'AFCRL' from dual union all
select 12000,'WFF' from dual union all
select 12001,'WRE' from dual union all
select 12003,'IIAE' from dual union all
select 12004,'IIAE' from dual union all
select 1201,'SUI' from dual union all
select 12016,'AFCRL' from dual union all
select 12017,'GCA' from dual union all
select 12018,'GCA' from dual union all
select 12019,'AFCRL' from dual union all
select 1202,'SUI' from dual union all
select 12020,'AFCRL' from dual union all
select 12022,'GSFC' from dual union all
select 12022,'NDRE' from dual union all
select 12023,'LARC' from dual union all
select 12024,'GSFC' from dual union all
select 12024,'ILC' from dual union all
select 12025,'ROE' from dual union all
select 12025,'SHEF' from dual union all
select 12025,'UCL' from dual union all
select 12026,'KGO' from dual union all
select 12026,'NICP' from dual union all
select 12029,'PIHM' from dual union all
select 1203,'AFCRL' from dual union all
select 1203,'UUT' from dual union all
select 12030,'BREI' from dual union all
select 12030,'UCL' from dual union all
select 12031,'LARC' from dual union all
select 12037,'SUPA' from dual union all
select 12038,'LARC' from dual union all
select 12040,'NRL' from dual union all
select 12042,'WRE' from dual union all
select 12045,'LARC' from dual union all
select 12046,'LARC' from dual union all
select 12048,'LARC' from dual union all
select 12051,'DRTE' from dual union all
select 12054,'LARC' from dual union all
select 12059,'PIHM' from dual union all
select 12061,'STA' from dual union all
select 12069,'LARC' from dual union all
select 12071,'NMC' from dual union all
select 12073,'DASA' from dual union all
select 12092,'AFCRL' from dual union all
select 12093,'GCA' from dual union all
select 12094,'SAND' from dual union all
select 12095,'AFCRL' from dual union all
select 12097,'CNRS' from dual union all
select 1210,'LRBA' from dual union all
select 12102,'SIMED' from dual union all
select 12103,'SIMED' from dual union all
select 12104,'STA' from dual union all
select 12105,'STA' from dual union all
select 12107,'STA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 12115,'HARP' from dual union all
select 12116,'HARP' from dual union all
select 12117,'HARP' from dual union all
select 12121,'STA' from dual union all
select 12122,'STA' from dual union all
select 12123,'ONERA' from dual union all
select 12124,'ONERA' from dual union all
select 12125,'ONERA' from dual union all
select 12126,'BRL' from dual union all
select 12127,'BRL' from dual union all
select 12128,'BRL' from dual union all
select 12129,'BRL' from dual union all
select 12130,'BRL' from dual union all
select 12131,'BRL' from dual union all
select 12132,'BRL' from dual union all
select 12133,'BRL' from dual union all
select 12134,'BRL' from dual union all
select 12135,'BRL' from dual union all
select 12136,'BRL' from dual union all
select 12137,'BRL' from dual union all
select 12138,'BRL' from dual union all
select 12139,'BRL' from dual union all
select 12140,'BRL' from dual union all
select 12141,'BRL' from dual union all
select 12142,'BRL' from dual union all
select 12143,'BRL' from dual union all
select 12144,'BRL' from dual union all
select 12145,'BRL' from dual union all
select 12146,'BRL' from dual union all
select 12147,'BRL' from dual union all
select 12148,'BRL' from dual union all
select 12149,'BRL' from dual union all
select 12150,'BRL' from dual union all
select 12151,'BRL' from dual union all
select 12152,'BRL' from dual union all
select 12153,'BRL' from dual union all
select 12154,'BRL' from dual union all
select 12155,'BRL' from dual union all
select 12156,'BRL' from dual union all
select 12157,'BRL' from dual union all
select 12158,'BRL' from dual union all
select 12159,'BRL' from dual union all
select 12160,'LRL' from dual union all
select 12161,'LRL' from dual union all
select 12163,'SAND' from dual union all
select 12170,'LASL' from dual union all
select 12171,'UMI' from dual union all
select 12172,'AFCRL' from dual union all
select 12173,'AFCRL' from dual union all
select 12175,'AFCRL' from dual union all
select 12178,'AFCRL' from dual union all
select 12179,'UMI' from dual union all
select 12181,'AFCRL' from dual union all
select 12186,'GCA' from dual union all
select 12187,'AFCRL' from dual union all
select 12188,'AFCRL' from dual union all
select 12189,'AFCRL' from dual union all
select 12190,'AFCRL' from dual union all
select 12194,'UIO' from dual union all
select 12195,'LASL' from dual union all
select 12196,'SAND' from dual union all
select 12198,'SAND' from dual union all
select 12199,'AFCRL' from dual union all
select 12200,'RAE' from dual union all
select 12200,'WRE' from dual union all
select 12201,'AFCRL' from dual union all
select 12204,'CNRS' from dual union all
select 12205,'LRL' from dual union all
select 12208,'AFCRL' from dual union all
select 12210,'UIO' from dual union all
select 12211,'LASL' from dual union all
select 12214,'PRIN' from dual union all
select 12215,'AFCRL' from dual union all
select 12215,'UUT' from dual union all
select 12217,'GSFC' from dual union all
select 12218,'UMI' from dual union all
select 12219,'UMI' from dual union all
select 12220,'UNH' from dual union all
select 12222,'UNH' from dual union all
select 12223,'PMR' from dual union all
select 12223,'UMI' from dual union all
select 12225,'UMI' from dual union all
select 12226,'PMR' from dual union all
select 12226,'UMI' from dual union all
select 12227,'SAND' from dual union all
select 12228,'UNH' from dual union all
select 12229,'UNH' from dual union all
select 1223,'SCEL' from dual union all
select 1223,'UMI' from dual union all
select 12231,'GSFC' from dual union all
select 12232,'AFCRL' from dual union all
select 12233,'AFCRL' from dual union all
select 12233,'UUT' from dual union all
select 12240,'GSFC' from dual union all
select 12241,'GSFC' from dual union all
select 12242,'ASE' from dual union all
select 12243,'UTOR' from dual union all
select 12244,'GSFC' from dual union all
select 12245,'GSFC' from dual union all
select 12247,'NAG' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 12247,'TOK' from dual union all
select 12248,'GSFC' from dual union all
select 12249,'GSFC' from dual union all
select 12250,'UTD' from dual union all
select 12251,'GSFC' from dual union all
select 12252,'SASK' from dual union all
select 12253,'GSFC' from dual union all
select 12253,'UMI' from dual union all
select 12254,'UMI' from dual union all
select 12255,'UIUC' from dual union all
select 12256,'AFCRL' from dual union all
select 12258,'AFCRL' from dual union all
select 12261,'AFCRL' from dual union all
select 12262,'AFCRL' from dual union all
select 12263,'UIUC' from dual union all
select 12264,'NRCC' from dual union all
select 12265,'GSFC' from dual union all
select 12266,'GSFC' from dual union all
select 12269,'ROE' from dual union all
select 12269,'SHEF' from dual union all
select 12269,'UCL' from dual union all
select 12271,'NRCC' from dual union all
select 12272,'TAO' from dual union all
select 12273,'AFCRL' from dual union all
select 12276,'GSFC' from dual union all
select 12277,'GSFC' from dual union all
select 12278,'GSFC' from dual union all
select 12280,'TOK' from dual union all
select 12281,'GSFC' from dual union all
select 12282,'KYO' from dual union all
select 12283,'AFCRL' from dual union all
select 12285,'GSFC' from dual union all
select 12286,'GSFC' from dual union all
select 12288,'NRL' from dual union all
select 12289,'WRE' from dual union all
select 12292,'LERC' from dual union all
select 12293,'UNH' from dual union all
select 12294,'BREI' from dual union all
select 12294,'UCL' from dual union all
select 12295,'RICE' from dual union all
select 12296,'UMI' from dual union all
select 12298,'UIUC' from dual union all
select 12299,'UNH' from dual union all
select 12302,'UMI' from dual union all
select 12303,'CULH' from dual union all
select 12303,'LUX' from dual union all
select 12304,'UIUC' from dual union all
select 12305,'UMI' from dual union all
select 12308,'GSFC' from dual union all
select 12309,'UIUC' from dual union all
select 12310,'AFCRL' from dual union all
select 12314,'UMI' from dual union all
select 12315,'UMI' from dual union all
select 12316,'UNH' from dual union all
select 12317,'UMI' from dual union all
select 12318,'UMN' from dual union all
select 12319,'UMN' from dual union all
select 12320,'UMI' from dual union all
select 12321,'UMI' from dual union all
select 12326,'NRL' from dual union all
select 12327,'UCL' from dual union all
select 12329,'NRCC' from dual union all
select 12330,'LARC' from dual union all
select 12331,'LARC' from dual union all
select 12335,'GSFC' from dual union all
select 12340,'NRL' from dual union all
select 12341,'GSFC' from dual union all
select 12342,'PRL' from dual union all
select 12343,'ICST' from dual union all
select 12343,'QUB' from dual union all
select 12343,'UCL' from dual union all
select 12345,'UCL' from dual union all
select 12346,'UCL' from dual union all
select 12347,'UCL' from dual union all
select 12348,'UCL' from dual union all
select 12349,'SUPA' from dual union all
select 12350,'UCL' from dual union all
select 12351,'UCL' from dual union all
select 12353,'DASA' from dual union all
select 12356,'LARC' from dual union all
select 12358,'PRL' from dual union all
select 12359,'GSFC' from dual union all
select 12360,'GSFC' from dual union all
select 12365,'WRE' from dual union all
select 12367,'WRE' from dual union all
select 12368,'WRE' from dual union all
select 12369,'WRE' from dual union all
select 12370,'WRE' from dual union all
select 12371,'WRE' from dual union all
select 12372,'WRE' from dual union all
select 12373,'WRE' from dual union all
select 12374,'QUB' from dual union all
select 12374,'UCL' from dual union all
select 12377,'LERC' from dual union all
select 12378,'RSRS' from dual union all
select 12385,'LUX' from dual union all
select 12385,'RSRS' from dual union all
select 12388,'PSU' from dual union all
select 12391,'UMI' from dual union all
select 12393,'UMI' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 12395,'WRE' from dual union all
select 12399,'AFCRL' from dual union all
select 12405,'GSFC' from dual union all
select 12406,'AFCRL' from dual union all
select 12408,'LASL' from dual union all
select 12410,'LASL' from dual union all
select 12411,'LASL' from dual union all
select 12412,'LASL' from dual union all
select 12413,'LASL' from dual union all
select 12415,'DASA' from dual union all
select 12426,'PRIN' from dual union all
select 12427,'AFCRL' from dual union all
select 12428,'AFCRL' from dual union all
select 12429,'DRTE' from dual union all
select 12431,'KPNO' from dual union all
select 12433,'BRL' from dual union all
select 12434,'CNES' from dual union all
select 12435,'BRL' from dual union all
select 12436,'BRL' from dual union all
select 12437,'DASA' from dual union all
select 12439,'WRE' from dual union all
select 12441,'PSU' from dual union all
select 12442,'WRE' from dual union all
select 12445,'LRL' from dual union all
select 12446,'BAJ' from dual union all
select 12447,'UIUC' from dual union all
select 12451,'ONERA' from dual union all
select 12452,'JPL' from dual union all
select 12455,'GSFC' from dual union all
select 12456,'UIUC' from dual union all
select 12457,'DASA' from dual union all
select 12458,'BAJ' from dual union all
select 12459,'NBS' from dual union all
select 12459,'NOAA' from dual union all
select 12460,'GSFC' from dual union all
select 12461,'CNRS' from dual union all
select 12464,'DASA' from dual union all
select 12467,'CNRS' from dual union all
select 12468,'CNRS' from dual union all
select 12469,'CNRS' from dual union all
select 12470,'CNRS' from dual union all
select 12471,'GCA' from dual union all
select 12472,'GCA' from dual union all
select 12473,'GCA' from dual union all
select 12475,'GCA' from dual union all
select 12476,'GCA' from dual union all
select 12477,'KPNO' from dual union all
select 12478,'GSFC' from dual union all
select 12479,'CNRS' from dual union all
select 12480,'CNRS' from dual union all
select 12481,'CNRS' from dual union all
select 12482,'CNRS' from dual union all
select 12483,'DASA' from dual union all
select 12486,'AFCRL' from dual union all
select 12487,'DASA' from dual union all
select 12489,'JPL' from dual union all
select 12490,'HARP' from dual union all
select 12507,'SAND' from dual union all
select 12510,'UMI' from dual union all
select 12512,'JPL' from dual union all
select 12513,'LUX' from dual union all
select 12513,'UCL' from dual union all
select 12518,'NRL' from dual union all
select 12519,'AFCRL' from dual union all
select 12520,'WRE' from dual union all
select 12522,'UCL' from dual union all
select 12523,'UMN' from dual union all
select 12525,'TOK' from dual union all
select 12527,'AFCRL' from dual union all
select 12529,'ISAS' from dual union all
select 12530,'AFCRL' from dual union all
select 12531,'GSFC' from dual union all
select 12532,'AFCRL' from dual union all
select 12533,'SAND' from dual union all
select 12535,'ISAS' from dual union all
select 12535,'TAO' from dual union all
select 12537,'TOKAI' from dual union all
select 12538,'DASA' from dual union all
select 1254,'LRBA' from dual union all
select 12540,'BAJ' from dual union all
select 12542,'WRE' from dual union all
select 12546,'ISAS' from dual union all
select 12549,'GSFC' from dual union all
select 12550,'GSFC' from dual union all
select 12552,'UMI' from dual union all
select 12553,'GSFC' from dual union all
select 12554,'GSFC' from dual union all
select 12555,'GSFC' from dual union all
select 12556,'GSFC' from dual union all
select 12557,'GSFC' from dual union all
select 12558,'GSFC' from dual union all
select 12559,'UMI' from dual union all
select 12560,'GSFC' from dual union all
select 12561,'GSFC' from dual union all
select 12563,'GSFC' from dual union all
select 12565,'ROE' from dual union all
select 12567,'LAPAN' from dual union all
select 12568,'DUD' from dual union all
select 12569,'AFCRL' from dual union all
select 12572,'LAPAN' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 12573,'BAJ' from dual union all
select 12575,'LERC' from dual union all
select 12578,'CNRS' from dual union all
select 12580,'GSFC' from dual union all
select 12580,'INPE' from dual union all
select 12582,'PITT' from dual union all
select 12584,'BAJ' from dual union all
select 12586,'AFCRL' from dual union all
select 12586,'UUT' from dual union all
select 12588,'ISAS' from dual union all
select 12591,'UTD' from dual union all
select 12592,'CNRS' from dual union all
select 12593,'UTD' from dual union all
select 12595,'NMC' from dual union all
select 12597,'RAE' from dual union all
select 12597,'WRE' from dual union all
select 12598,'GSFC' from dual union all
select 12601,'GCA' from dual union all
select 12601,'UIUC' from dual union all
select 12602,'AFCRL' from dual union all
select 12603,'UCB' from dual union all
select 12605,'CNES' from dual union all
select 12606,'SAND' from dual union all
select 12608,'UTR' from dual union all
select 12610,'AN' from dual union all
select 12611,'UCB' from dual union all
select 12613,'UTR' from dual union all
select 12614,'ASE' from dual union all
select 12615,'GSFC' from dual union all
select 12616,'GSFC' from dual union all
select 12618,'UTR' from dual union all
select 12620,'AFCRL' from dual union all
select 12621,'CNES' from dual union all
select 12623,'UTR' from dual union all
select 12624,'GSFC' from dual union all
select 12627,'GSFC' from dual union all
select 12628,'CORN' from dual union all
select 12628,'NRL' from dual union all
select 12630,'CNIE' from dual union all
select 12636,'MPE' from dual union all
select 12636,'UCL' from dual union all
select 12637,'CNRS' from dual union all
select 12637,'OPM' from dual union all
select 12641,'LPARL' from dual union all
select 12642,'AN' from dual union all
select 12643,'MPE' from dual union all
select 12643,'UCL' from dual union all
select 12644,'KYO' from dual union all
select 12645,'AFCRL' from dual union all
select 12646,'PSU' from dual union all
select 12647,'NRL' from dual union all
select 12648,'WRE' from dual union all
select 12649,'GCA' from dual union all
select 12649,'GSFC' from dual union all
select 1265,'AFCRL' from dual union all
select 1265,'UUT' from dual union all
select 12651,'TOKAI' from dual union all
select 12655,'AN' from dual union all
select 12656,'GSFC' from dual union all
select 12657,'PRIN' from dual union all
select 12658,'GSFC' from dual union all
select 12659,'GSFC' from dual union all
select 12660,'GSFC' from dual union all
select 12663,'LRBA' from dual union all
select 12664,'AFCRL' from dual union all
select 12665,'QUB' from dual union all
select 12665,'UCL' from dual union all
select 12666,'GSFC' from dual union all
select 12668,'GSFC' from dual union all
select 12669,'UCO' from dual union all
select 12670,'GSFC' from dual union all
select 12671,'CULH' from dual union all
select 12671,'LUX' from dual union all
select 12673,'NRL' from dual union all
select 12675,'JHU' from dual union all
select 12676,'AFCRL' from dual union all
select 12678,'BREI' from dual union all
select 12679,'NRL' from dual union all
select 12680,'GSFC' from dual union all
select 12681,'GSFC' from dual union all
select 12682,'GSFC' from dual union all
select 12683,'LRBA' from dual union all
select 12685,'GSFC' from dual union all
select 12686,'GSFC' from dual union all
select 12687,'GSFC' from dual union all
select 12688,'NRL' from dual union all
select 12689,'GSFC' from dual union all
select 12690,'BREI' from dual union all
select 12691,'LRL' from dual union all
select 12693,'CORN' from dual union all
select 12693,'NRL' from dual union all
select 12695,'NMSA' from dual union all
select 12696,'DASA' from dual union all
select 12697,'DASA' from dual union all
select 12698,'ISAS' from dual union all
select 12701,'AFCRL' from dual union all
select 12702,'AFCRL' from dual union all
select 12703,'AFCRL' from dual union all
select 12704,'AFCRL' from dual union all
select 12712,'ISAS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 12713,'AFCRL' from dual union all
select 12715,'WRE' from dual union all
select 12716,'UMI' from dual union all
select 12717,'AFCRL' from dual union all
select 12718,'GSFC' from dual union all
select 12718,'UMI' from dual union all
select 12719,'AFCRL' from dual union all
select 12720,'GSFC' from dual union all
select 12720,'UMI' from dual union all
select 12722,'AFCRL' from dual union all
select 12723,'AFCRL' from dual union all
select 12724,'AFCRL' from dual union all
select 12725,'SAND' from dual union all
select 12729,'ARC' from dual union all
select 12730,'AFCRL' from dual union all
select 12731,'LRL' from dual union all
select 12732,'DASA' from dual union all
select 12733,'UAF' from dual union all
select 12734,'AFCRL' from dual union all
select 12735,'AFCRL' from dual union all
select 12736,'AFCRL' from dual union all
select 12737,'AFCRL' from dual union all
select 12738,'AFCRL' from dual union all
select 12739,'AFCRL' from dual union all
select 12740,'AFCRL' from dual union all
select 12741,'AFCRL' from dual union all
select 12742,'DUD' from dual union all
select 12744,'MPE' from dual union all
select 12745,'ICST' from dual union all
select 12745,'LUX' from dual union all
select 12745,'UCL' from dual union all
select 12746,'MPE' from dual union all
select 12747,'GSFC' from dual union all
select 12747,'NDRE' from dual union all
select 12748,'UAF' from dual union all
select 12749,'KPNO' from dual union all
select 12750,'MPE' from dual union all
select 12751,'SAND' from dual union all
select 12752,'SAND' from dual union all
select 12753,'STA' from dual union all
select 12755,'BRL' from dual union all
select 12756,'AFCRL' from dual union all
select 12757,'BRL' from dual union all
select 12758,'BRL' from dual union all
select 12759,'BRL' from dual union all
select 12760,'BRL' from dual union all
select 12761,'UAF' from dual union all
select 12762,'RICE' from dual union all
select 12763,'ONUSJ' from dual union all
select 12764,'ONUSJ' from dual union all
select 12765,'OSAKA' from dual union all
select 12767,'ONUSJ' from dual union all
select 12775,'GSFC' from dual union all
select 12778,'GSFC' from dual union all
select 12781,'WRE' from dual union all
select 12783,'BONN' from dual union all
select 12785,'NRL' from dual union all
select 12786,'ADEL' from dual union all
select 12788,'AFCRL' from dual union all
select 12790,'AFCRL' from dual union all
select 12791,'USAF' from dual union all
select 12792,'BONN' from dual union all
select 12793,'AFCRL' from dual union all
select 12794,'AFCRL' from dual union all
select 12795,'OSAKA' from dual union all
select 12796,'ICST' from dual union all
select 12796,'LUX' from dual union all
select 12796,'UCL' from dual union all
select 12799,'LUX' from dual union all
select 12799,'UCL' from dual union all
select 12799,'UTR' from dual union all
select 12802,'UIUC' from dual union all
select 12803,'GSFC' from dual union all
select 12803,'INPE' from dual union all
select 12804,'GRI' from dual union all
select 12805,'GRI' from dual union all
select 12806,'CNRS' from dual union all
select 12807,'GSFC' from dual union all
select 12807,'INPE' from dual union all
select 12808,'DOSH' from dual union all
select 12808,'NAG' from dual union all
select 12809,'DASA' from dual union all
select 1281,'LRBA' from dual union all
select 12811,'GRI' from dual union all
select 1282,'LRBA' from dual union all
select 12820,'SAND' from dual union all
select 12823,'ADEL' from dual union all
select 12825,'SAND' from dual union all
select 12826,'HARP' from dual union all
select 12827,'HARP' from dual union all
select 12828,'HARP' from dual union all
select 12829,'HARP' from dual union all
select 12830,'HARP' from dual union all
select 12831,'WRE' from dual union all
select 12832,'HARP' from dual union all
select 12833,'HARP' from dual union all
select 12834,'HARP' from dual union all
select 12835,'HARP' from dual union all
select 12836,'HARP' from dual union all
select 12837,'HARP' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 12838,'HARP' from dual union all
select 12839,'HARP' from dual union all
select 12840,'HARP' from dual union all
select 12841,'HARP' from dual union all
select 12842,'SAND' from dual union all
select 12843,'HARP' from dual union all
select 12844,'HARP' from dual union all
select 12845,'HARP' from dual union all
select 12846,'HARP' from dual union all
select 12847,'HARP' from dual union all
select 12848,'HARP' from dual union all
select 12849,'HARP' from dual union all
select 12850,'HARP' from dual union all
select 12851,'HARP' from dual union all
select 12852,'HARP' from dual union all
select 12853,'UNH' from dual union all
select 12854,'HARP' from dual union all
select 12855,'HARP' from dual union all
select 12856,'HARP' from dual union all
select 12857,'HARP' from dual union all
select 12858,'HARP' from dual union all
select 12859,'HARP' from dual union all
select 12860,'HARP' from dual union all
select 12861,'HARP' from dual union all
select 12862,'HARP' from dual union all
select 12863,'HARP' from dual union all
select 12864,'GSFC' from dual union all
select 12865,'HARP' from dual union all
select 12866,'HARP' from dual union all
select 12867,'HARP' from dual union all
select 12868,'HARP' from dual union all
select 12869,'HARP' from dual union all
select 12870,'HARP' from dual union all
select 12871,'HARP' from dual union all
select 12872,'BRL' from dual union all
select 12873,'BRL' from dual union all
select 12874,'GSFC' from dual union all
select 12875,'BRL' from dual union all
select 12876,'BRL' from dual union all
select 12877,'BRL' from dual union all
select 12878,'BRL' from dual union all
select 12879,'BRL' from dual union all
select 12880,'BRL' from dual union all
select 12881,'BRL' from dual union all
select 12882,'BRL' from dual union all
select 12883,'BRL' from dual union all
select 12884,'HARP' from dual union all
select 12885,'GSFC' from dual union all
select 12886,'HARP' from dual union all
select 12887,'HARP' from dual union all
select 12888,'HARP' from dual union all
select 12889,'HARP' from dual union all
select 1289,'LRBA' from dual union all
select 12890,'HARP' from dual union all
select 12891,'HARP' from dual union all
select 12892,'HARP' from dual union all
select 12893,'HARP' from dual union all
select 12894,'HARP' from dual union all
select 12896,'HARP' from dual union all
select 12897,'BRL' from dual union all
select 12899,'NMSU' from dual union all
select 12900,'NMSU' from dual union all
select 12901,'NMSU' from dual union all
select 12902,'NMSU' from dual union all
select 12903,'NMSU' from dual union all
select 12904,'NMSU' from dual union all
select 12905,'NMSU' from dual union all
select 12907,'NMSU' from dual union all
select 12908,'NMSU' from dual union all
select 12909,'NMSU' from dual union all
select 1291,'LRBA' from dual union all
select 12910,'LARC' from dual union all
select 12910,'SAO' from dual union all
select 12919,'USN' from dual union all
select 12920,'AEC' from dual union all
select 12920,'SAND' from dual union all
select 12921,'AEC' from dual union all
select 12921,'SAND' from dual union all
select 12929,'WRE' from dual union all
select 12938,'OSAKA' from dual union all
select 12939,'TAO' from dual union all
select 12940,'AFCRL' from dual union all
select 12943,'GSFC' from dual union all
select 12944,'GSFC' from dual union all
select 12945,'GSFC' from dual union all
select 12947,'AFCRL' from dual union all
select 1295,'LRBA' from dual union all
select 12951,'CNRS' from dual union all
select 12954,'UMI' from dual union all
select 12955,'AFCRL' from dual union all
select 12956,'JHU' from dual union all
select 12959,'LARC' from dual union all
select 12960,'LARC' from dual union all
select 12961,'AFCRL' from dual union all
select 12961,'ASE' from dual union all
select 12962,'AFWL' from dual union all
select 12963,'AFCRL' from dual union all
select 12964,'AFWL' from dual union all
select 12965,'USAF' from dual union all
select 12966,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 12967,'USAF' from dual union all
select 12968,'USAF' from dual union all
select 12969,'USAF' from dual union all
select 12970,'USAF' from dual union all
select 12971,'ISRO' from dual union all
select 12972,'ISRO' from dual union all
select 12973,'DASA' from dual union all
select 12974,'USAF' from dual union all
select 12975,'ISRO' from dual union all
select 12976,'MSFC' from dual union all
select 12977,'MSFC' from dual union all
select 12978,'MSFC' from dual union all
select 12979,'USAF' from dual union all
select 12980,'USAF' from dual union all
select 12981,'USAF' from dual union all
select 12982,'USAF' from dual union all
select 12983,'USAF' from dual union all
select 12984,'USAF' from dual union all
select 12985,'USAF' from dual union all
select 12986,'USAF' from dual union all
select 12987,'USAF' from dual union all
select 12988,'USAF' from dual union all
select 12989,'GSFC' from dual union all
select 1299,'LRBA' from dual union all
select 12990,'GSFC' from dual union all
select 12991,'GSFC' from dual union all
select 12992,'GSFC' from dual union all
select 12993,'MSFC' from dual union all
select 12994,'ISRO' from dual union all
select 12995,'MSFC' from dual union all
select 12996,'USAF' from dual union all
select 12997,'MSFC' from dual union all
select 12998,'USAF' from dual union all
select 12999,'USAF' from dual union all
select 13000,'USAF' from dual union all
select 13001,'USAF' from dual union all
select 13002,'USAF' from dual union all
select 13003,'USAF' from dual union all
select 13004,'USAF' from dual union all
select 13005,'MSFC' from dual union all
select 13006,'LARC' from dual union all
select 13007,'USAF' from dual union all
select 13008,'LARC' from dual union all
select 13009,'MSFC' from dual union all
select 13010,'ISRO' from dual union all
select 13011,'MSFC' from dual union all
select 13012,'MSFC' from dual union all
select 13013,'JMA' from dual union all
select 13014,'MSFC' from dual union all
select 13015,'USAF' from dual union all
select 13016,'USAF' from dual union all
select 13017,'USAF' from dual union all
select 13018,'USAF' from dual union all
select 13019,'USAF' from dual union all
select 13020,'USAF' from dual union all
select 13021,'USAF' from dual union all
select 13022,'USAF' from dual union all
select 13023,'USAF' from dual union all
select 13024,'USAF' from dual union all
select 13025,'MSFC' from dual union all
select 13026,'MSFC' from dual union all
select 13027,'ISRO' from dual union all
select 13028,'USAF' from dual union all
select 13029,'MSFC' from dual union all
select 13030,'US' from dual union all
select 13031,'MSFC' from dual union all
select 13032,'DASA' from dual union all
select 13033,'USAF' from dual union all
select 13034,'USAF' from dual union all
select 13035,'USAF' from dual union all
select 13036,'USAF' from dual union all
select 13037,'USAF' from dual union all
select 13038,'USAF' from dual union all
select 13039,'USAF' from dual union all
select 13040,'USAF' from dual union all
select 13041,'MSFC' from dual union all
select 13042,'MSFC' from dual union all
select 13043,'ISRO' from dual union all
select 13044,'MSFC' from dual union all
select 13045,'AFCRL' from dual union all
select 13046,'AFCRL' from dual union all
select 13047,'AFCRL' from dual union all
select 13048,'AFCRL' from dual union all
select 13049,'AFCRL' from dual union all
select 13050,'ISRO' from dual union all
select 13051,'AFCRL' from dual union all
select 13052,'ISRO' from dual union all
select 13053,'WSMR' from dual union all
select 13054,'MSFC' from dual union all
select 13055,'MSFC' from dual union all
select 13056,'USAF' from dual union all
select 13057,'USAF' from dual union all
select 13058,'USAF' from dual union all
select 13059,'USAF' from dual union all
select 13060,'USAF' from dual union all
select 13061,'MSFC' from dual union all
select 13062,'USAF' from dual union all
select 13063,'USAF' from dual union all
select 13064,'USAF' from dual union all
select 13065,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 13066,'USAF' from dual union all
select 13067,'USAF' from dual union all
select 13068,'USAF' from dual union all
select 13069,'USAF' from dual union all
select 13070,'USAF' from dual union all
select 13071,'USAF' from dual union all
select 13072,'MSFC' from dual union all
select 13073,'USAF' from dual union all
select 13074,'USAF' from dual union all
select 13075,'USAF' from dual union all
select 13076,'USAF' from dual union all
select 13077,'USAF' from dual union all
select 13078,'USAF' from dual union all
select 13079,'USAF' from dual union all
select 13080,'USAF' from dual union all
select 13081,'MSFC' from dual union all
select 13082,'USAF' from dual union all
select 13083,'MSFC' from dual union all
select 13084,'USAF' from dual union all
select 13085,'ISRO' from dual union all
select 13086,'ISRO' from dual union all
select 13087,'MSFC' from dual union all
select 13088,'ISRO' from dual union all
select 13089,'MSFC' from dual union all
select 13090,'PSU' from dual union all
select 13091,'SUPA' from dual union all
select 13092,'SUPA' from dual union all
select 13093,'SUPA' from dual union all
select 13094,'USAF' from dual union all
select 13095,'KSC' from dual union all
select 13096,'KSC' from dual union all
select 13097,'KSC' from dual union all
select 13098,'KSC' from dual union all
select 13099,'WFF' from dual union all
select 13100,'WFF' from dual union all
select 13101,'WFF' from dual union all
select 13102,'WFF' from dual union all
select 13103,'WFF' from dual union all
select 13104,'WFF' from dual union all
select 13105,'USAF' from dual union all
select 13106,'WFF' from dual union all
select 13107,'WFF' from dual union all
select 13108,'WFF' from dual union all
select 13109,'WFF' from dual union all
select 13110,'WFF' from dual union all
select 13111,'WFF' from dual union all
select 13112,'WFF' from dual union all
select 13113,'WFF' from dual union all
select 13114,'WFF' from dual union all
select 13115,'NOL' from dual union all
select 13115,'WFF' from dual union all
select 13116,'USAF' from dual union all
select 13117,'NOL' from dual union all
select 13117,'WFF' from dual union all
select 13118,'NOL' from dual union all
select 13118,'WFF' from dual union all
select 13119,'NOL' from dual union all
select 13119,'WFF' from dual union all
select 13120,'NOL' from dual union all
select 13120,'WFF' from dual union all
select 13121,'NOL' from dual union all
select 13121,'WFF' from dual union all
select 13122,'NOL' from dual union all
select 13122,'WFF' from dual union all
select 13123,'NOL' from dual union all
select 13123,'WFF' from dual union all
select 13124,'NOL' from dual union all
select 13124,'WFF' from dual union all
select 13125,'NOL' from dual union all
select 13125,'WFF' from dual union all
select 13126,'USAF' from dual union all
select 13127,'NOL' from dual union all
select 13127,'WFF' from dual union all
select 13128,'NOL' from dual union all
select 13128,'WFF' from dual union all
select 13129,'NOL' from dual union all
select 13129,'WFF' from dual union all
select 13130,'ESSA' from dual union all
select 13131,'ESSA' from dual union all
select 13132,'ESSA' from dual union all
select 13133,'ESSA' from dual union all
select 13134,'ESSA' from dual union all
select 13135,'ESSA' from dual union all
select 13136,'ESSA' from dual union all
select 13137,'USAF' from dual union all
select 13138,'ESSA' from dual union all
select 13139,'ESSA' from dual union all
select 13140,'ESSA' from dual union all
select 13141,'ESSA' from dual union all
select 13142,'ESSA' from dual union all
select 13143,'ESSA' from dual union all
select 13144,'ESSA' from dual union all
select 13145,'ESSA' from dual union all
select 13146,'ESSA' from dual union all
select 13147,'ESSA' from dual union all
select 13148,'USAF' from dual union all
select 13149,'ESSA' from dual union all
select 13150,'ESSA' from dual union all
select 13151,'ESSA' from dual union all
select 13152,'ESSA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 13153,'ESSA' from dual union all
select 13154,'ESSA' from dual union all
select 13155,'NOL' from dual union all
select 13155,'WFF' from dual union all
select 13156,'WFF' from dual union all
select 13157,'WFF' from dual union all
select 13158,'WFF' from dual union all
select 13159,'USAF' from dual union all
select 13160,'WFF' from dual union all
select 13161,'WFF' from dual union all
select 13162,'WFF' from dual union all
select 13163,'WFF' from dual union all
select 13164,'WFF' from dual union all
select 13165,'WFF' from dual union all
select 13166,'WFF' from dual union all
select 13167,'WFF' from dual union all
select 13168,'WFF' from dual union all
select 13169,'WFF' from dual union all
select 13170,'USAF' from dual union all
select 13171,'SUPA' from dual union all
select 13172,'SUPA' from dual union all
select 13173,'NOL' from dual union all
select 13173,'WFF' from dual union all
select 13174,'USAF' from dual union all
select 13175,'USAF' from dual union all
select 13176,'USAF' from dual union all
select 13177,'MSFC' from dual union all
select 13178,'WSMR' from dual union all
select 13179,'ESSA' from dual union all
select 13180,'JMA' from dual union all
select 13181,'ESSA' from dual union all
select 13182,'MSFC' from dual union all
select 13183,'MSFC' from dual union all
select 13184,'JMA' from dual union all
select 13185,'JMA' from dual union all
select 13186,'ESSA' from dual union all
select 13187,'GSFC' from dual union all
select 13188,'ISRO' from dual union all
select 13189,'MSFC' from dual union all
select 13190,'PSU' from dual union all
select 13191,'GSFC' from dual union all
select 13192,'MSFC' from dual union all
select 13193,'USAF' from dual union all
select 13194,'USAF' from dual union all
select 13195,'USAF' from dual union all
select 13196,'USAF' from dual union all
select 13197,'DASA' from dual union all
select 13198,'INPE' from dual union all
select 13199,'MSFC' from dual union all
select 13200,'MSFC' from dual union all
select 13201,'MSFC' from dual union all
select 13202,'MSFC' from dual union all
select 13203,'MSFC' from dual union all
select 13204,'MSFC' from dual union all
select 13205,'USAF' from dual union all
select 13206,'USAF' from dual union all
select 13207,'USAF' from dual union all
select 13208,'USAF' from dual union all
select 13209,'USAF' from dual union all
select 13210,'USAF' from dual union all
select 13211,'USAF' from dual union all
select 13213,'MSFC' from dual union all
select 13214,'IMDP' from dual union all
select 13215,'USAF' from dual union all
select 13216,'USAF' from dual union all
select 13217,'MSFC' from dual union all
select 13218,'USAF' from dual union all
select 13219,'USAF' from dual union all
select 13220,'USAF' from dual union all
select 13221,'USAF' from dual union all
select 13222,'USAF' from dual union all
select 13223,'USAF' from dual union all
select 13224,'MSFC' from dual union all
select 13225,'MSFC' from dual union all
select 13226,'IMDP' from dual union all
select 13227,'MSFC' from dual union all
select 13228,'MSFC' from dual union all
select 13229,'USAF' from dual union all
select 13230,'USAF' from dual union all
select 13231,'USAF' from dual union all
select 13232,'USAF' from dual union all
select 13233,'USAF' from dual union all
select 13234,'USAF' from dual union all
select 13235,'USAF' from dual union all
select 13236,'USAF' from dual union all
select 13237,'MSFC' from dual union all
select 13238,'MSFC' from dual union all
select 13239,'JMA' from dual union all
select 13240,'JMA' from dual union all
select 13241,'JMA' from dual union all
select 13242,'PSU' from dual union all
select 13243,'MSFC' from dual union all
select 13244,'PSU' from dual union all
select 13245,'CAST' from dual union all
select 13309,'UIO' from dual union all
select 13319,'ILC' from dual union all
select 13319,'NDRE' from dual union all
select 13320,'UIO' from dual union all
select 13321,'GSFC' from dual union all
select 13382,'GSFC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 13389,'GSFC' from dual union all
select 13390,'GSFC' from dual union all
select 13391,'GSFC' from dual union all
select 13392,'GSFC' from dual union all
select 13393,'GSFC' from dual union all
select 13394,'GSFC' from dual union all
select 13432,'AFCRL' from dual union all
select 13432,'NRL' from dual union all
select 13470,'AFCRL' from dual union all
select 13470,'NRL' from dual union all
select 13527,'PIHM' from dual union all
select 13528,'PIHM' from dual union all
select 13529,'PIHM' from dual union all
select 13530,'PIHM' from dual union all
select 13531,'PIHM' from dual union all
select 13532,'PIHM' from dual union all
select 13533,'PIHM' from dual union all
select 13534,'PIHM' from dual union all
select 13535,'PIHM' from dual union all
select 13541,'SAND' from dual union all
select 13543,'GSFC' from dual union all
select 13545,'LARC' from dual union all
select 13546,'PIHM' from dual union all
select 13554,'LARC' from dual union all
select 13555,'AFCRL' from dual union all
select 13556,'LARC' from dual union all
select 13557,'PIHM' from dual union all
select 13558,'JMA' from dual union all
select 13562,'INTA' from dual union all
select 13563,'LARC' from dual union all
select 13564,'LRL' from dual union all
select 13565,'LARC' from dual union all
select 13568,'PIHM' from dual union all
select 1357,'SUI' from dual union all
select 13574,'LARC' from dual union all
select 13575,'CNIE' from dual union all
select 13576,'USAF' from dual union all
select 13579,'LARC' from dual union all
select 13580,'LARC' from dual union all
select 13583,'LARC' from dual union all
select 13584,'BOL' from dual union all
select 13584,'ROE' from dual union all
select 13588,'CULH' from dual union all
select 13588,'LUX' from dual union all
select 1359,'ORDRSA' from dual union all
select 13591,'LARC' from dual union all
select 13592,'LARC' from dual union all
select 13593,'NRCC' from dual union all
select 13594,'KPNO' from dual union all
select 13603,'SAND' from dual union all
select 13604,'SRCI' from dual union all
select 13605,'SRCI' from dual union all
select 13606,'SRCI' from dual union all
select 13607,'SRCI' from dual union all
select 13608,'SRCI' from dual union all
select 13609,'SRCI' from dual union all
select 13610,'SRCI' from dual union all
select 13611,'SRCI' from dual union all
select 13612,'JMA' from dual union all
select 13613,'SRCI' from dual union all
select 13614,'SRCI' from dual union all
select 13615,'SRCI' from dual union all
select 13616,'SRCI' from dual union all
select 13617,'SRCI' from dual union all
select 13618,'SRCI' from dual union all
select 13619,'SRCI' from dual union all
select 13620,'SRCI' from dual union all
select 13621,'SRCI' from dual union all
select 13622,'SRCI' from dual union all
select 13624,'SRCI' from dual union all
select 13625,'SRCI' from dual union all
select 13626,'SRCI' from dual union all
select 13627,'SRCI' from dual union all
select 13632,'ISAS' from dual union all
select 13634,'USAF' from dual union all
select 13635,'SRCI' from dual union all
select 13636,'USAF' from dual union all
select 13637,'USAF' from dual union all
select 13638,'USAF' from dual union all
select 13639,'NASA' from dual union all
select 13641,'USAF' from dual union all
select 13642,'NASA' from dual union all
select 13643,'USAF' from dual union all
select 13644,'SHEF' from dual union all
select 13644,'UCL' from dual union all
select 13645,'SRCI' from dual union all
select 13646,'USAF' from dual union all
select 13647,'USAF' from dual union all
select 13648,'AFCRL' from dual union all
select 13649,'LARC' from dual union all
select 1365,'AFCRL' from dual union all
select 13650,'USAF' from dual union all
select 13651,'UMI' from dual union all
select 13653,'LARC' from dual union all
select 13654,'LARC' from dual union all
select 13655,'AFSWC' from dual union all
select 13657,'CNES' from dual union all
select 13658,'SAND' from dual union all
select 13667,'ISAS' from dual union all
select 13678,'ISAS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 13683,'WRE' from dual union all
select 13684,'WRE' from dual union all
select 13685,'LRS' from dual union all
select 13686,'SAND' from dual union all
select 13688,'IIAE' from dual union all
select 13700,'GSFC' from dual union all
select 13702,'RSRS' from dual union all
select 13702,'UCL' from dual union all
select 13705,'NMC' from dual union all
select 13712,'AFCRL' from dual union all
select 13720,'ISRO' from dual union all
select 13723,'LARC' from dual union all
select 13726,'ISAS' from dual union all
select 13727,'ISAS' from dual union all
select 13729,'SAND' from dual union all
select 13731,'ISAS' from dual union all
select 13748,'LRL' from dual union all
select 13749,'KPNO' from dual union all
select 13750,'AFCRL' from dual union all
select 13751,'BRL' from dual union all
select 13752,'SRCI' from dual union all
select 13753,'SRCI' from dual union all
select 13755,'SRCI' from dual union all
select 13756,'SRCI' from dual union all
select 13757,'AFCRL' from dual union all
select 13758,'AFCRL' from dual union all
select 13759,'AFCRL' from dual union all
select 13760,'AFCRL' from dual union all
select 13761,'AFCRL' from dual union all
select 13762,'AFCRL' from dual union all
select 13763,'AFCRL' from dual union all
select 13764,'AFCRL' from dual union all
select 13765,'AFCRL' from dual union all
select 13766,'AFCRL' from dual union all
select 13767,'SRCI' from dual union all
select 13768,'AFCRL' from dual union all
select 13769,'AFCRL' from dual union all
select 13770,'AFCRL' from dual union all
select 13771,'AFCRL' from dual union all
select 13772,'AFCRL' from dual union all
select 13773,'AFCRL' from dual union all
select 13774,'AFCRL' from dual union all
select 13775,'AFCRL' from dual union all
select 13776,'AFCRL' from dual union all
select 13777,'AFCRL' from dual union all
select 13778,'SRCI' from dual union all
select 13779,'AFCRL' from dual union all
select 13780,'AFCRL' from dual union all
select 13781,'AFCRL' from dual union all
select 13782,'AFCRL' from dual union all
select 13783,'AFCRL' from dual union all
select 13784,'AFCRL' from dual union all
select 13785,'AFCRL' from dual union all
select 13786,'AFCRL' from dual union all
select 13787,'AFCRL' from dual union all
select 13788,'MRN' from dual union all
select 13789,'SRCI' from dual union all
select 13790,'MRN' from dual union all
select 13791,'MRN' from dual union all
select 13792,'INTA' from dual union all
select 13792,'NASA' from dual union all
select 13793,'INTA' from dual union all
select 13793,'NASA' from dual union all
select 13794,'INTA' from dual union all
select 13794,'NASA' from dual union all
select 13795,'INTA' from dual union all
select 13795,'NASA' from dual union all
select 13796,'INTA' from dual union all
select 13796,'NASA' from dual union all
select 13797,'INTA' from dual union all
select 13797,'NASA' from dual union all
select 13798,'INTA' from dual union all
select 13798,'NASA' from dual union all
select 13799,'INTA' from dual union all
select 13799,'NASA' from dual union all
select 13800,'SRCI' from dual union all
select 13801,'INTA' from dual union all
select 13801,'NASA' from dual union all
select 13802,'PSU' from dual union all
select 13803,'PSU' from dual union all
select 13804,'MSFC' from dual union all
select 13805,'MSFC' from dual union all
select 13807,'SRCI' from dual union all
select 13808,'AFCRL' from dual union all
select 13809,'ISAS' from dual union all
select 13810,'LRL' from dual union all
select 13816,'GRI' from dual union all
select 13818,'MILAN' from dual union all
select 13820,'MILAN' from dual union all
select 13828,'AFCRL' from dual union all
select 13830,'AFCRL' from dual union all
select 13831,'PSU' from dual union all
select 13833,'PSU' from dual union all
select 13834,'PSU' from dual union all
select 13835,'PSU' from dual union all
select 13836,'PSU' from dual union all
select 13839,'SRCI' from dual union all
select 13840,'SRCI' from dual union all
select 13841,'SRCI' from dual union all
select 13842,'SRCI' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 13843,'SRCI' from dual union all
select 13844,'SRCI' from dual union all
select 13845,'SRCI' from dual union all
select 13846,'SRCI' from dual union all
select 13847,'SRCI' from dual union all
select 13848,'SRCI' from dual union all
select 13849,'SRCI' from dual union all
select 13850,'SRCI' from dual union all
select 13851,'SRCI' from dual union all
select 13852,'SRCI' from dual union all
select 13853,'SRCI' from dual union all
select 13854,'SRCI' from dual union all
select 13855,'SRCI' from dual union all
select 13856,'SRCI' from dual union all
select 13857,'SRCI' from dual union all
select 13858,'SRCI' from dual union all
select 13859,'SRCI' from dual union all
select 13860,'SRCI' from dual union all
select 13861,'SRCI' from dual union all
select 13862,'SRCI' from dual union all
select 13863,'SRCI' from dual union all
select 13864,'SRCI' from dual union all
select 13865,'SRCI' from dual union all
select 13866,'SRCI' from dual union all
select 13867,'SRCI' from dual union all
select 13868,'SRCI' from dual union all
select 13869,'SRCI' from dual union all
select 13870,'SRCI' from dual union all
select 13871,'SRCI' from dual union all
select 13872,'SRCI' from dual union all
select 13873,'SRCI' from dual union all
select 13874,'SRCI' from dual union all
select 13875,'SRCI' from dual union all
select 13876,'SRCI' from dual union all
select 13877,'SRCI' from dual union all
select 13878,'SRCI' from dual union all
select 13879,'SRCI' from dual union all
select 13880,'SRCI' from dual union all
select 13881,'SRCI' from dual union all
select 13882,'SRCI' from dual union all
select 13883,'SRCI' from dual union all
select 13884,'SRCI' from dual union all
select 13885,'SRCI' from dual union all
select 13886,'SRCI' from dual union all
select 13887,'SRCI' from dual union all
select 13888,'SRCI' from dual union all
select 13889,'SRCI' from dual union all
select 13890,'SRCI' from dual union all
select 13891,'SRCI' from dual union all
select 13902,'GSFC' from dual union all
select 13903,'GSFC' from dual union all
select 13904,'GSFC' from dual union all
select 13905,'GSFC' from dual union all
select 13906,'AFCRL' from dual union all
select 13907,'GSFC' from dual union all
select 13908,'GSFC' from dual union all
select 13909,'GSFC' from dual union all
select 13919,'RICE' from dual union all
select 13920,'GSFC' from dual union all
select 13921,'AFCRL' from dual union all
select 13924,'AFCRL' from dual union all
select 13925,'JHU' from dual union all
select 13926,'AFCRL' from dual union all
select 13927,'NRL' from dual union all
select 13928,'NRL' from dual union all
select 13929,'NRL' from dual union all
select 13930,'KGO' from dual union all
select 13930,'UCL' from dual union all
select 13931,'WRE' from dual union all
select 13932,'KGO' from dual union all
select 13932,'UCL' from dual union all
select 13939,'SUPA' from dual union all
select 13941,'SUPA' from dual union all
select 13942,'UMI' from dual union all
select 13948,'USN' from dual union all
select 13951,'AFCRL' from dual union all
select 13952,'GSFC' from dual union all
select 13954,'AFCRL' from dual union all
select 13955,'CNRS' from dual union all
select 13958,'ISAS' from dual union all
select 13960,'ASE' from dual union all
select 13960,'MIT' from dual union all
select 13961,'PRIN' from dual union all
select 13962,'BREI' from dual union all
select 13962,'RSRS' from dual union all
select 13962,'UCL' from dual union all
select 13963,'DASA' from dual union all
select 13965,'AFCRL' from dual union all
select 13966,'BREI' from dual union all
select 13966,'RSRS' from dual union all
select 13966,'UCL' from dual union all
select 13967,'ONERA' from dual union all
select 13971,'GSFC' from dual union all
select 13977,'RAE' from dual union all
select 13977,'WRE' from dual union all
select 13985,'NOTS' from dual union all
select 13986,'NAG' from dual union all
select 13986,'RRL' from dual union all
select 13986,'TAO' from dual union all
select 13988,'GSFC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 13990,'WRE' from dual union all
select 13991,'GSFC' from dual union all
select 13993,'PRL' from dual union all
select 13994,'SUPA' from dual union all
select 13994,'UCL' from dual union all
select 13995,'PRL' from dual union all
select 13996,'CNES' from dual union all
select 13997,'PRL' from dual union all
select 13999,'PRL' from dual union all
select 14002,'NOTS' from dual union all
select 14003,'NRCC' from dual union all
select 14004,'SUPA' from dual union all
select 14004,'UCL' from dual union all
select 14007,'NRCC' from dual union all
select 14008,'AFCRL' from dual union all
select 14021,'NBS' from dual union all
select 14025,'CNRS' from dual union all
select 14028,'UTD' from dual union all
select 14031,'GSFC' from dual union all
select 14032,'UCO' from dual union all
select 14034,'UCO' from dual union all
select 14037,'RAE' from dual union all
select 14037,'WRE' from dual union all
select 14039,'TOK' from dual union all
select 14043,'RAE' from dual union all
select 14043,'WRE' from dual union all
select 14045,'SAND' from dual union all
select 14046,'JHU' from dual union all
select 14048,'MPE' from dual union all
select 14051,'SUPA' from dual union all
select 14051,'UCL' from dual union all
select 14052,'NRL' from dual union all
select 14053,'AFCRL' from dual union all
select 14059,'AFCRL' from dual union all
select 14060,'GSFC' from dual union all
select 14062,'GSFC' from dual union all
select 14063,'GSFC' from dual union all
select 14064,'GSFC' from dual union all
select 14067,'GSFC' from dual union all
select 14068,'GSFC' from dual union all
select 14069,'GSFC' from dual union all
select 14070,'CNRS' from dual union all
select 14071,'GSFC' from dual union all
select 14071,'INPE' from dual union all
select 14072,'LUX' from dual union all
select 14072,'RAE' from dual union all
select 14072,'UCL' from dual union all
select 14074,'JPL' from dual union all
select 14075,'SAND' from dual union all
select 14078,'CNRS' from dual union all
select 14080,'LUX' from dual union all
select 14080,'UTR' from dual union all
select 14084,'WRE' from dual union all
select 14085,'UWI' from dual union all
select 14086,'MPE' from dual union all
select 14086,'UCL' from dual union all
select 14086,'UTR' from dual union all
select 14088,'MPE' from dual union all
select 14088,'UCL' from dual union all
select 14088,'UTR' from dual union all
select 14089,'GSFC' from dual union all
select 14090,'GSFC' from dual union all
select 14091,'GSFC' from dual union all
select 14092,'SAND' from dual union all
select 14093,'CNRS' from dual union all
select 14095,'SAND' from dual union all
select 14097,'PRIN' from dual union all
select 14099,'SAND' from dual union all
select 14101,'GRI' from dual union all
select 14114,'LRL' from dual union all
select 14115,'MSSL' from dual union all
select 14115,'SHEF' from dual union all
select 14116,'CNRS' from dual union all
select 14118,'UCW' from dual union all
select 14118,'UKMO' from dual union all
select 14121,'LERC' from dual union all
select 14123,'SAND' from dual union all
select 14124,'SAND' from dual union all
select 14125,'UCW' from dual union all
select 14125,'UKMO' from dual union all
select 14132,'SAND' from dual union all
select 14133,'SAND' from dual union all
select 14134,'SAND' from dual union all
select 14135,'SAND' from dual union all
select 14136,'CNRS' from dual union all
select 14137,'UIUC' from dual union all
select 14138,'CNRS' from dual union all
select 14139,'WRE' from dual union all
select 1414,'LRBA' from dual union all
select 14140,'CNRS' from dual union all
select 14141,'CNRS' from dual union all
select 14143,'SAND' from dual union all
select 14144,'SAND' from dual union all
select 14145,'MPE' from dual union all
select 14146,'GSFC' from dual union all
select 14147,'GSFC' from dual union all
select 14148,'AFCRL' from dual union all
select 14149,'MPE' from dual union all
select 1415,'LRBA' from dual union all
select 14150,'MPE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 14150,'UKMO' from dual union all
select 14156,'MSSL' from dual union all
select 14156,'SHEF' from dual union all
select 14157,'GSFC' from dual union all
select 14158,'GSFC' from dual union all
select 14159,'AFCRL' from dual union all
select 14162,'AFCRL' from dual union all
select 14163,'AFCRL' from dual union all
select 14165,'AFCRL' from dual union all
select 14166,'AFCRL' from dual union all
select 14169,'AFCRL' from dual union all
select 14169,'DSRI' from dual union all
select 14169,'GSFC' from dual union all
select 14170,'CNES' from dual union all
select 14171,'MPE' from dual union all
select 14171,'UKMO' from dual union all
select 14172,'AFCRL' from dual union all
select 14172,'UUT' from dual union all
select 14173,'CNIE' from dual union all
select 14187,'PRL' from dual union all
select 14188,'LIEGE' from dual union all
select 14192,'UMI' from dual union all
select 14194,'PITT' from dual union all
select 14196,'LIEGE' from dual union all
select 14199,'SIMED' from dual union all
select 14200,'BREI' from dual union all
select 14201,'GSFC' from dual union all
select 14202,'GCA' from dual union all
select 14203,'TOK' from dual union all
select 14204,'GCA' from dual union all
select 14205,'GCA' from dual union all
select 14206,'GCA' from dual union all
select 14207,'GCA' from dual union all
select 14209,'NRL' from dual union all
select 14210,'CALG' from dual union all
select 14211,'WRE' from dual union all
select 14212,'GSFC' from dual union all
select 14213,'CALG' from dual union all
select 14215,'AFCRL' from dual union all
select 14216,'OSAKA' from dual union all
select 14216,'RRL' from dual union all
select 14227,'NRL' from dual union all
select 14228,'SIMED' from dual union all
select 14230,'RAE' from dual union all
select 14230,'WRE' from dual union all
select 14231,'LRL' from dual union all
select 14232,'LRL' from dual union all
select 14233,'AFCRL' from dual union all
select 14238,'AFCRL' from dual union all
select 14239,'AFCRL' from dual union all
select 14241,'SAND' from dual union all
select 14242,'SAND' from dual union all
select 14243,'NMC' from dual union all
select 14244,'ISAS' from dual union all
select 14245,'SAND' from dual union all
select 14246,'SAND' from dual union all
select 14247,'SAND' from dual union all
select 14249,'AFCRL' from dual union all
select 14251,'AFCRL' from dual union all
select 14252,'AFCRL' from dual union all
select 14253,'GSFC' from dual union all
select 14254,'GSFC' from dual union all
select 14255,'ISRO' from dual union all
select 14256,'GSFC' from dual union all
select 14257,'UMI' from dual union all
select 14258,'GSFC' from dual union all
select 14259,'GSFC' from dual union all
select 1426,'NRL' from dual union all
select 14260,'GSFC' from dual union all
select 14263,'AFCRL' from dual union all
select 14265,'KYO' from dual union all
select 14266,'SAND' from dual union all
select 14267,'TOK' from dual union all
select 14268,'AFCRL' from dual union all
select 14269,'CNIE' from dual union all
select 14270,'GSFC' from dual union all
select 14271,'GSFC' from dual union all
select 14273,'NRL' from dual union all
select 14274,'DUD' from dual union all
select 14277,'LARC' from dual union all
select 14278,'MPE' from dual union all
select 14279,'GSFC' from dual union all
select 14282,'MPE' from dual union all
select 14285,'UNH' from dual union all
select 14286,'UIUC' from dual union all
select 14288,'UIUC' from dual union all
select 14290,'AFCRL' from dual union all
select 14291,'GSFC' from dual union all
select 14291,'UMI' from dual union all
select 14292,'GSFC' from dual union all
select 14292,'UMI' from dual union all
select 14294,'UMI' from dual union all
select 14295,'NRCC' from dual union all
select 14296,'UMI' from dual union all
select 14297,'GSFC' from dual union all
select 14297,'UMI' from dual union all
select 14298,'WRE' from dual union all
select 14300,'GSFC' from dual union all
select 14301,'ADEL' from dual union all
select 14302,'CNIE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 14309,'SAND' from dual union all
select 14312,'UCB' from dual union all
select 14314,'GSFC' from dual union all
select 14316,'GSFC' from dual union all
select 14317,'NRL' from dual union all
select 14318,'GSFC' from dual union all
select 14319,'UCB' from dual union all
select 14320,'UCB' from dual union all
select 14321,'GCA' from dual union all
select 14322,'GCA' from dual union all
select 14325,'GCA' from dual union all
select 14327,'GCA' from dual union all
select 14328,'GCA' from dual union all
select 14329,'UMD' from dual union all
select 1433,'UMI' from dual union all
select 14330,'UCB' from dual union all
select 14331,'SAND' from dual union all
select 14335,'PRIN' from dual union all
select 14336,'LRL' from dual union all
select 1434,'HAM' from dual union all
select 14340,'LRL' from dual union all
select 14341,'MPE' from dual union all
select 14343,'MPE' from dual union all
select 14345,'WRE' from dual union all
select 14349,'AFCRL' from dual union all
select 14349,'UUT' from dual union all
select 1435,'LRBA' from dual union all
select 14350,'NMSU' from dual union all
select 14351,'NMSU' from dual union all
select 14354,'GSFC' from dual union all
select 14355,'GSFC' from dual union all
select 14357,'GSFC' from dual union all
select 14358,'GSFC' from dual union all
select 1436,'NRL' from dual union all
select 14360,'AN' from dual union all
select 14361,'GSFC' from dual union all
select 14362,'GSFC' from dual union all
select 14363,'GSFC' from dual union all
select 14364,'GSFC' from dual union all
select 14365,'GSFC' from dual union all
select 14367,'CNES' from dual union all
select 14368,'NRL' from dual union all
select 1437,'SUI' from dual union all
select 14373,'GSFC' from dual union all
select 14375,'CNES' from dual union all
select 14378,'AFCRL' from dual union all
select 14379,'KPNO' from dual union all
select 1438,'SUI' from dual union all
select 14380,'ASE' from dual union all
select 14383,'AN' from dual union all
select 14386,'AFCRL' from dual union all
select 14387,'CNRS' from dual union all
select 14388,'TOK' from dual union all
select 1439,'SUI' from dual union all
select 14391,'TOKAI' from dual union all
select 14393,'KYO' from dual union all
select 14393,'TOK' from dual union all
select 1440,'SUI' from dual union all
select 14400,'ARC' from dual union all
select 14402,'BIR' from dual union all
select 14404,'BIR' from dual union all
select 14405,'GSFC' from dual union all
select 14406,'ESSA' from dual union all
select 14407,'AFAL' from dual union all
select 14407,'OSU' from dual union all
select 1441,'AFCRL' from dual union all
select 14411,'BIR' from dual union all
select 14412,'BIR' from dual union all
select 14413,'BIR' from dual union all
select 1442,'NRL' from dual union all
select 14421,'AFCRL' from dual union all
select 14423,'WRE' from dual union all
select 14428,'CNIE' from dual union all
select 1443,'AFCRL' from dual union all
select 1443,'UUT' from dual union all
select 14430,'ISAS' from dual union all
select 14433,'AFCRL' from dual union all
select 14433,'UUT' from dual union all
select 14436,'PRL' from dual union all
select 14437,'GSFC' from dual union all
select 14438,'CNIE' from dual union all
select 1444,'BU' from dual union all
select 14441,'WRE' from dual union all
select 14443,'MPE' from dual union all
select 14444,'CNIE' from dual union all
select 14445,'CNIE' from dual union all
select 14446,'CNIE' from dual union all
select 14447,'UIUC' from dual union all
select 14448,'CNRS' from dual union all
select 14449,'NRL' from dual union all
select 1445,'NRL' from dual union all
select 14450,'SAND' from dual union all
select 14451,'CNRS' from dual union all
select 14452,'SAND' from dual union all
select 14453,'GCA' from dual union all
select 14454,'UIUC' from dual union all
select 14455,'UIUC' from dual union all
select 14456,'UIUC' from dual union all
select 14457,'SAND' from dual union all
select 14459,'AFCRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 1446,'NRL' from dual union all
select 14460,'AFCRL' from dual union all
select 14460,'UUT' from dual union all
select 14461,'AFCRL' from dual union all
select 14461,'UUT' from dual union all
select 14462,'AFCRL' from dual union all
select 14462,'UUT' from dual union all
select 14463,'UIUC' from dual union all
select 14464,'GSFC' from dual union all
select 14467,'CNRS' from dual union all
select 14468,'CNRS' from dual union all
select 1447,'AFCRL' from dual union all
select 1447,'RIU' from dual union all
select 14471,'AFCRL' from dual union all
select 14472,'AFSWC' from dual union all
select 14472,'AFTAC' from dual union all
select 14474,'AFCRL' from dual union all
select 14477,'SRCI' from dual union all
select 14478,'SRCI' from dual union all
select 14479,'SRCI' from dual union all
select 1448,'NRL' from dual union all
select 14480,'GRI' from dual union all
select 14481,'SRCI' from dual union all
select 14482,'SRCI' from dual union all
select 14483,'RAE' from dual union all
select 14483,'WRE' from dual union all
select 14484,'DUD' from dual union all
select 14485,'SASK' from dual union all
select 14486,'SRCI' from dual union all
select 14487,'SRCI' from dual union all
select 14488,'SRCI' from dual union all
select 14489,'SRCI' from dual union all
select 1449,'SCEL' from dual union all
select 14490,'SRCI' from dual union all
select 14491,'GCA' from dual union all
select 14492,'SRCI' from dual union all
select 14493,'SRCI' from dual union all
select 14494,'LIEGE' from dual union all
select 14494,'UTR' from dual union all
select 14495,'SRCI' from dual union all
select 14496,'SRCI' from dual union all
select 14497,'SRCI' from dual union all
select 14498,'SRCI' from dual union all
select 1450,'SCEL' from dual union all
select 1450,'UMI' from dual union all
select 14500,'GSFC' from dual union all
select 14503,'AN' from dual union all
select 14506,'CNES' from dual union all
select 14508,'AFCRL' from dual union all
select 1451,'DENV' from dual union all
select 14512,'BREI' from dual union all
select 14512,'MSSL' from dual union all
select 14513,'GCA' from dual union all
select 14514,'NRCC' from dual union all
select 14515,'NRCC' from dual union all
select 14517,'UMN' from dual union all
select 14518,'UNH' from dual union all
select 14519,'BREI' from dual union all
select 14519,'MSSL' from dual union all
select 1452,'AFCRL' from dual union all
select 14521,'UMN' from dual union all
select 14522,'ISAS' from dual union all
select 14522,'KYO' from dual union all
select 14523,'AFCRL' from dual union all
select 14524,'GCA' from dual union all
select 14528,'CNET' from dual union all
select 14529,'GSFC' from dual union all
select 14530,'ISAS' from dual union all
select 14531,'LARC' from dual union all
select 14532,'TAO' from dual union all
select 14534,'BONN' from dual union all
select 14535,'GSFC' from dual union all
select 14538,'UCB' from dual union all
select 14540,'RAE' from dual union all
select 14540,'WRE' from dual union all
select 14541,'AFCRL' from dual union all
select 14541,'UUT' from dual union all
select 14542,'AFCRL' from dual union all
select 14543,'NRL' from dual union all
select 14545,'BAL' from dual union all
select 14546,'GCA' from dual union all
select 14547,'AN' from dual union all
select 14548,'BAL' from dual union all
select 14557,'GCA' from dual union all
select 14558,'CNIE' from dual union all
select 14560,'AFAL' from dual union all
select 14561,'SAND' from dual union all
select 14562,'SAND' from dual union all
select 14563,'SAND' from dual union all
select 14564,'SRCI' from dual union all
select 14565,'SRCI' from dual union all
select 14566,'SRCI' from dual union all
select 14567,'SRCI' from dual union all
select 14569,'SRCI' from dual union all
select 14570,'SRCI' from dual union all
select 14571,'SRCI' from dual union all
select 14572,'SRCI' from dual union all
select 14573,'SRCI' from dual union all
select 14574,'SRCI' from dual union all
select 14575,'SRCI' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 14576,'SRCI' from dual union all
select 14577,'SRCI' from dual union all
select 14578,'SRCI' from dual union all
select 14580,'SRCI' from dual union all
select 14581,'SRCI' from dual union all
select 14582,'SRCI' from dual union all
select 14583,'SRCI' from dual union all
select 14584,'SRCI' from dual union all
select 14585,'SRCI' from dual union all
select 14586,'SRCI' from dual union all
select 14589,'WSMR' from dual union all
select 14590,'CNRS' from dual union all
select 14591,'WSMR' from dual union all
select 14592,'WSMR' from dual union all
select 14593,'WSMR' from dual union all
select 14594,'WSMR' from dual union all
select 14595,'WSMR' from dual union all
select 14596,'NMSU' from dual union all
select 14607,'LRS' from dual union all
select 14612,'AFCRL' from dual union all
select 14613,'WSMR' from dual union all
select 14615,'NMSU' from dual union all
select 14634,'GSFC' from dual union all
select 14636,'AFCRL' from dual union all
select 14637,'AFCRL' from dual union all
select 1464,'SUI' from dual union all
select 14645,'UMI' from dual union all
select 14656,'WRE' from dual union all
select 14663,'SAND' from dual union all
select 14664,'SAND' from dual union all
select 14666,'NRL' from dual union all
select 14671,'NRL' from dual union all
select 14672,'GSFC' from dual union all
select 14673,'GSFC' from dual union all
select 14674,'GSFC' from dual union all
select 14675,'PRIN' from dual union all
select 14676,'CULH' from dual union all
select 14676,'LUX' from dual union all
select 14679,'UMI' from dual union all
select 14680,'UMI' from dual union all
select 14681,'SHEF' from dual union all
select 14681,'UCL' from dual union all
select 14682,'SAND' from dual union all
select 14685,'LARC' from dual union all
select 14687,'LARC' from dual union all
select 14689,'LARC' from dual union all
select 14691,'INPE' from dual union all
select 14692,'USAF' from dual union all
select 14693,'USAF' from dual union all
select 14694,'USAF' from dual union all
select 14695,'USAF' from dual union all
select 14696,'USAF' from dual union all
select 14697,'USAF' from dual union all
select 14699,'MSFC' from dual union all
select 1470,'NRL' from dual union all
select 14700,'PSU' from dual union all
select 14701,'USAF' from dual union all
select 14702,'USAF' from dual union all
select 14703,'USAF' from dual union all
select 14704,'USAF' from dual union all
select 14705,'USAF' from dual union all
select 14706,'USAF' from dual union all
select 14707,'USAF' from dual union all
select 14708,'USAF' from dual union all
select 14709,'USAF' from dual union all
select 14710,'USAF' from dual union all
select 14711,'PSU' from dual union all
select 14712,'USAF' from dual union all
select 14713,'USAF' from dual union all
select 14714,'USAF' from dual union all
select 14715,'UKMO' from dual union all
select 14716,'USAF' from dual union all
select 14717,'USAF' from dual union all
select 14718,'USAF' from dual union all
select 14719,'USAF' from dual union all
select 1472,'SCEL' from dual union all
select 14720,'USAF' from dual union all
select 14721,'USAF' from dual union all
select 14723,'USAF' from dual union all
select 14724,'UKMO' from dual union all
select 14725,'USAF' from dual union all
select 14726,'USAF' from dual union all
select 14727,'USAF' from dual union all
select 14728,'ISRO' from dual union all
select 14729,'SUPA' from dual union all
select 1473,'AFCRL' from dual union all
select 1473,'DENV' from dual union all
select 14730,'SUPA' from dual union all
select 14731,'USAF' from dual union all
select 14732,'USAF' from dual union all
select 14733,'MSFC' from dual union all
select 14734,'USAF' from dual union all
select 14735,'USAF' from dual union all
select 14736,'USAF' from dual union all
select 14737,'USAF' from dual union all
select 14738,'USAF' from dual union all
select 14739,'USAF' from dual union all
select 1474,'SCEL' from dual union all
select 14740,'USAF' from dual union all
select 14741,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 14742,'USAF' from dual union all
select 14743,'USAF' from dual union all
select 14744,'WFF' from dual union all
select 14745,'USAF' from dual union all
select 14746,'USAF' from dual union all
select 14747,'USAF' from dual union all
select 14748,'USAF' from dual union all
select 14749,'USAF' from dual union all
select 1475,'UMI' from dual union all
select 14750,'USAF' from dual union all
select 14751,'USAF' from dual union all
select 14752,'USAF' from dual union all
select 14753,'USAF' from dual union all
select 14754,'USAF' from dual union all
select 14755,'SUPA' from dual union all
select 14756,'UKMO' from dual union all
select 14757,'USAF' from dual union all
select 14758,'MSFC' from dual union all
select 14759,'USAF' from dual union all
select 14760,'USAF' from dual union all
select 14761,'USAF' from dual union all
select 14762,'USAF' from dual union all
select 14763,'USAF' from dual union all
select 14764,'USAF' from dual union all
select 14765,'USAF' from dual union all
select 14766,'INPE' from dual union all
select 14767,'USAF' from dual union all
select 14768,'USAF' from dual union all
select 14769,'USAF' from dual union all
select 14770,'USAF' from dual union all
select 14771,'UKMO' from dual union all
select 14772,'MSFC' from dual union all
select 14773,'USAF' from dual union all
select 14774,'USAF' from dual union all
select 14775,'USAF' from dual union all
select 14776,'USAF' from dual union all
select 14777,'USAF' from dual union all
select 14778,'USAF' from dual union all
select 14779,'USAF' from dual union all
select 14780,'USAF' from dual union all
select 14781,'USAF' from dual union all
select 14782,'USAF' from dual union all
select 14783,'USAF' from dual union all
select 14784,'USAF' from dual union all
select 14785,'USAF' from dual union all
select 14786,'UKMO' from dual union all
select 14787,'USAF' from dual union all
select 14789,'USAF' from dual union all
select 14790,'USAF' from dual union all
select 14791,'USAF' from dual union all
select 14792,'USAF' from dual union all
select 14793,'USAF' from dual union all
select 14794,'USAF' from dual union all
select 14795,'USAF' from dual union all
select 14796,'USAF' from dual union all
select 14797,'MSFC' from dual union all
select 14798,'USAF' from dual union all
select 14799,'JMA' from dual union all
select 1480,'UMI' from dual union all
select 14800,'USAF' from dual union all
select 14801,'USAF' from dual union all
select 14802,'USAF' from dual union all
select 14803,'USAF' from dual union all
select 14804,'KSC' from dual union all
select 14805,'MSFC' from dual union all
select 14806,'USAF' from dual union all
select 14807,'USAF' from dual union all
select 14808,'UKMO' from dual union all
select 14809,'USAF' from dual union all
select 1481,'UCO' from dual union all
select 14810,'MSFC' from dual union all
select 14811,'USAF' from dual union all
select 14812,'USAF' from dual union all
select 14813,'USAF' from dual union all
select 14814,'USAF' from dual union all
select 14815,'USAF' from dual union all
select 14816,'USAF' from dual union all
select 14817,'USAF' from dual union all
select 14818,'USAF' from dual union all
select 14819,'USAF' from dual union all
select 1482,'NRL' from dual union all
select 14820,'USAF' from dual union all
select 14821,'JMA' from dual union all
select 14822,'USAF' from dual union all
select 14823,'USAF' from dual union all
select 14824,'USAF' from dual union all
select 14825,'USAF' from dual union all
select 14826,'USAF' from dual union all
select 14827,'UKMO' from dual union all
select 14828,'USAF' from dual union all
select 14829,'INPE' from dual union all
select 14830,'UKMO' from dual union all
select 14831,'UKMO' from dual union all
select 14832,'ISRO' from dual union all
select 14833,'UKMO' from dual union all
select 14835,'UKMO' from dual union all
select 14837,'ISRO' from dual union all
select 14838,'INPE' from dual union all
select 14839,'SUPA' from dual union all
select 14840,'SUPA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 14841,'AFCRL' from dual union all
select 14841,'USU' from dual union all
select 14842,'USAF' from dual union all
select 14844,'USAF' from dual union all
select 14845,'USAF' from dual union all
select 14846,'ISRO' from dual union all
select 14847,'SUPA' from dual union all
select 14848,'DRTE' from dual union all
select 14850,'UKMO' from dual union all
select 14852,'CNIE' from dual union all
select 14853,'USAF' from dual union all
select 14854,'USAF' from dual union all
select 14855,'USAF' from dual union all
select 14856,'USAF' from dual union all
select 14857,'USAF' from dual union all
select 14858,'USAF' from dual union all
select 14859,'USAF' from dual union all
select 14860,'USAF' from dual union all
select 14861,'JMA' from dual union all
select 14862,'JMA' from dual union all
select 14863,'INPE' from dual union all
select 14864,'JMA' from dual union all
select 14866,'GSFC' from dual union all
select 14867,'GSFC' from dual union all
select 14868,'WFF' from dual union all
select 14869,'WFF' from dual union all
select 14870,'WFF' from dual union all
select 14871,'WFF' from dual union all
select 14873,'SUPA' from dual union all
select 14874,'INPE' from dual union all
select 14875,'CONAE' from dual union all
select 14876,'SUPA' from dual union all
select 14877,'CONAE' from dual union all
select 14878,'WSMR' from dual union all
select 14879,'USAF' from dual union all
select 14880,'USAF' from dual union all
select 14881,'USAF' from dual union all
select 14882,'USAF' from dual union all
select 14883,'USAF' from dual union all
select 14884,'USAF' from dual union all
select 14885,'USAF' from dual union all
select 14886,'USAF' from dual union all
select 14887,'USAF' from dual union all
select 14888,'USAF' from dual union all
select 14889,'INPE' from dual union all
select 14890,'UKMO' from dual union all
select 14893,'SUPA' from dual union all
select 14894,'CONAE' from dual union all
select 14895,'GSFC' from dual union all
select 14896,'AFCRL' from dual union all
select 14897,'GSFC' from dual union all
select 14899,'USAF' from dual union all
select 14900,'USAF' from dual union all
select 14901,'USAF' from dual union all
select 14902,'USAF' from dual union all
select 14903,'USAF' from dual union all
select 14904,'USAF' from dual union all
select 14905,'USAF' from dual union all
select 14906,'USAF' from dual union all
select 14907,'MSFC' from dual union all
select 14908,'MSFC' from dual union all
select 14909,'MSFC' from dual union all
select 14910,'SUPA' from dual union all
select 14911,'CONAE' from dual union all
select 14912,'CONAE' from dual union all
select 14913,'CONAE' from dual union all
select 14915,'USAF' from dual union all
select 14916,'USAF' from dual union all
select 14917,'USAF' from dual union all
select 14918,'USAF' from dual union all
select 14919,'USAF' from dual union all
select 14921,'USAF' from dual union all
select 14922,'USAF' from dual union all
select 14923,'USAF' from dual union all
select 14924,'USAF' from dual union all
select 14925,'GSFC' from dual union all
select 14926,'GSFC' from dual union all
select 14927,'SUPA' from dual union all
select 14928,'JMA' from dual union all
select 14929,'MSFC' from dual union all
select 14930,'MSFC' from dual union all
select 14931,'USAF' from dual union all
select 14933,'USAF' from dual union all
select 14934,'USAF' from dual union all
select 14935,'USAF' from dual union all
select 14936,'USAF' from dual union all
select 14937,'USAF' from dual union all
select 14938,'USAF' from dual union all
select 14939,'USAF' from dual union all
select 14940,'USAF' from dual union all
select 14941,'USAF' from dual union all
select 14943,'USAF' from dual union all
select 14944,'USAF' from dual union all
select 14945,'USAF' from dual union all
select 14946,'USAF' from dual union all
select 14947,'USAF' from dual union all
select 14948,'USAF' from dual union all
select 14949,'USAF' from dual union all
select 14950,'USAF' from dual union all
select 14951,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 14952,'USAF' from dual union all
select 14953,'DASA' from dual union all
select 14954,'USAF' from dual union all
select 14955,'USAF' from dual union all
select 14956,'USAF' from dual union all
select 14957,'USAF' from dual union all
select 14958,'CONAE' from dual union all
select 14959,'CONAE' from dual union all
select 14960,'ISAS' from dual union all
select 14961,'CONAE' from dual union all
select 14962,'UKMO' from dual union all
select 14963,'USAF' from dual union all
select 14964,'NASA' from dual union all
select 14965,'AFCRL' from dual union all
select 14965,'USU' from dual union all
select 14966,'USAF' from dual union all
select 14967,'USAF' from dual union all
select 14968,'USAF' from dual union all
select 14969,'USAF' from dual union all
select 14970,'USAF' from dual union all
select 14971,'USAF' from dual union all
select 14972,'USAF' from dual union all
select 14973,'USAF' from dual union all
select 14974,'USAF' from dual union all
select 14975,'GSFC' from dual union all
select 14976,'SUPA' from dual union all
select 14977,'JMA' from dual union all
select 14978,'CONAE' from dual union all
select 14980,'USAF' from dual union all
select 14981,'USAF' from dual union all
select 14982,'USAF' from dual union all
select 14983,'USAF' from dual union all
select 14984,'USAF' from dual union all
select 14985,'USAF' from dual union all
select 14986,'WSMR' from dual union all
select 14987,'USAF' from dual union all
select 14988,'USAF' from dual union all
select 14990,'INPE' from dual union all
select 14991,'INPE' from dual union all
select 14992,'INPE' from dual union all
select 14993,'INPE' from dual union all
select 14994,'INPE' from dual union all
select 14995,'INPE' from dual union all
select 14996,'INPE' from dual union all
select 14998,'INPE' from dual union all
select 14999,'SUPA' from dual union all
select 15000,'CONAE' from dual union all
select 15001,'ONERA' from dual union all
select 15002,'ONERA' from dual union all
select 15003,'ONERA' from dual union all
select 15004,'USAF' from dual union all
select 15005,'USAF' from dual union all
select 15006,'USAF' from dual union all
select 15007,'USAF' from dual union all
select 15008,'USAF' from dual union all
select 15009,'USAF' from dual union all
select 15010,'USAF' from dual union all
select 15011,'USAF' from dual union all
select 15012,'USAF' from dual union all
select 15013,'LARC' from dual union all
select 15014,'USAF' from dual union all
select 15015,'LARC' from dual union all
select 15016,'LARC' from dual union all
select 15017,'USAF' from dual union all
select 15018,'LARC' from dual union all
select 15019,'USAF' from dual union all
select 15020,'USAF' from dual union all
select 15021,'USAF' from dual union all
select 15022,'LARC' from dual union all
select 15024,'CONAE' from dual union all
select 15025,'CONAE' from dual union all
select 15030,'WSMR' from dual union all
select 15038,'PSU' from dual union all
select 15039,'AFCRL' from dual union all
select 15040,'AFCRL' from dual union all
select 15041,'USAF' from dual union all
select 15042,'AFCRL' from dual union all
select 15043,'AFCRL' from dual union all
select 15044,'AFCRL' from dual union all
select 15045,'AFCRL' from dual union all
select 15046,'AFCRL' from dual union all
select 15047,'AFCRL' from dual union all
select 15048,'AFCRL' from dual union all
select 15049,'AFCRL' from dual union all
select 15050,'AFCRL' from dual union all
select 15051,'AFCRL' from dual union all
select 15052,'USN' from dual union all
select 15053,'AFCRL' from dual union all
select 15054,'AFCRL' from dual union all
select 15055,'AFCRL' from dual union all
select 15056,'AFCRL' from dual union all
select 15057,'AFCRL' from dual union all
select 15058,'AFCRL' from dual union all
select 15059,'AFCRL' from dual union all
select 15060,'AFCRL' from dual union all
select 15061,'AFCRL' from dual union all
select 15062,'AFCRL' from dual union all
select 15063,'USAF' from dual union all
select 15064,'AFCRL' from dual union all
select 15065,'AFCRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 15066,'KSC' from dual union all
select 15067,'KSC' from dual union all
select 15068,'WFF' from dual union all
select 15069,'WFF' from dual union all
select 15070,'WFF' from dual union all
select 15071,'WFF' from dual union all
select 15072,'WFF' from dual union all
select 15073,'WFF' from dual union all
select 15074,'USAF' from dual union all
select 15075,'WFF' from dual union all
select 15076,'WFF' from dual union all
select 15077,'WFF' from dual union all
select 15078,'WFF' from dual union all
select 15079,'WFF' from dual union all
select 15080,'WFF' from dual union all
select 15081,'WFF' from dual union all
select 15082,'WFF' from dual union all
select 15083,'WFF' from dual union all
select 15084,'WFF' from dual union all
select 15085,'USAF' from dual union all
select 15086,'WFF' from dual union all
select 15087,'WFF' from dual union all
select 15088,'WFF' from dual union all
select 15089,'WFF' from dual union all
select 15090,'WFF' from dual union all
select 15091,'WFF' from dual union all
select 15092,'WFF' from dual union all
select 15093,'WFF' from dual union all
select 15094,'WFF' from dual union all
select 15095,'WFF' from dual union all
select 15096,'USAF' from dual union all
select 15097,'WFF' from dual union all
select 15098,'AFCRL' from dual union all
select 15098,'KSC' from dual union all
select 15099,'AFCRL' from dual union all
select 15099,'KSC' from dual union all
select 15100,'INTA' from dual union all
select 15101,'INTA' from dual union all
select 15102,'INTA' from dual union all
select 15102,'NASA' from dual union all
select 15103,'ASL' from dual union all
select 15104,'ASL' from dual union all
select 15105,'ASL' from dual union all
select 15107,'ASL' from dual union all
select 15108,'ASL' from dual union all
select 15109,'ASL' from dual union all
select 15110,'ASL' from dual union all
select 15111,'ASL' from dual union all
select 15112,'ASL' from dual union all
select 15113,'ASL' from dual union all
select 15114,'ASL' from dual union all
select 15115,'ASL' from dual union all
select 15116,'ASL' from dual union all
select 15117,'DASA' from dual union all
select 15118,'ASL' from dual union all
select 15119,'ASL' from dual union all
select 15120,'CONAE' from dual union all
select 15121,'CONAE' from dual union all
select 15122,'INPE' from dual union all
select 15123,'INPE' from dual union all
select 15124,'INPE' from dual union all
select 15125,'INPE' from dual union all
select 15126,'INPE' from dual union all
select 15127,'INPE' from dual union all
select 15128,'USAF' from dual union all
select 15129,'INPE' from dual union all
select 15130,'INPE' from dual union all
select 15131,'INPE' from dual union all
select 15132,'WFF' from dual union all
select 15133,'WFF' from dual union all
select 15134,'WFF' from dual union all
select 15135,'WFF' from dual union all
select 15136,'WFF' from dual union all
select 15137,'WFF' from dual union all
select 15138,'WFF' from dual union all
select 15139,'WSMR' from dual union all
select 15140,'WFF' from dual union all
select 15141,'WFF' from dual union all
select 15142,'WFF' from dual union all
select 15143,'WFF' from dual union all
select 15144,'USAF' from dual union all
select 15145,'USAF' from dual union all
select 15146,'USAF' from dual union all
select 15147,'WSMR' from dual union all
select 15148,'USAF' from dual union all
select 15150,'USAF' from dual union all
select 15151,'USAF' from dual union all
select 15152,'USAF' from dual union all
select 15153,'USAF' from dual union all
select 15154,'USAF' from dual union all
select 15155,'USAF' from dual union all
select 15156,'USAF' from dual union all
select 15157,'USAF' from dual union all
select 15158,'USAF' from dual union all
select 15159,'USAF' from dual union all
select 15160,'USAF' from dual union all
select 15161,'USAF' from dual union all
select 15162,'USAF' from dual union all
select 15163,'USAF' from dual union all
select 15164,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 15165,'USAF' from dual union all
select 15166,'USAF' from dual union all
select 15167,'USAF' from dual union all
select 15168,'USAF' from dual union all
select 15169,'UKMO' from dual union all
select 15170,'NASA' from dual union all
select 15171,'USAF' from dual union all
select 15173,'USAF' from dual union all
select 15174,'USAF' from dual union all
select 15175,'USAF' from dual union all
select 15176,'USAF' from dual union all
select 15177,'WSMR' from dual union all
select 15178,'USN' from dual union all
select 15179,'USAF' from dual union all
select 15180,'USAF' from dual union all
select 15181,'USAF' from dual union all
select 15182,'USAF' from dual union all
select 15183,'USAF' from dual union all
select 15184,'GSFC' from dual union all
select 15185,'USAF' from dual union all
select 15186,'USAF' from dual union all
select 15187,'USAF' from dual union all
select 15188,'GSFC' from dual union all
select 15189,'USAF' from dual union all
select 15190,'USAF' from dual union all
select 15191,'USAF' from dual union all
select 15192,'UKMO' from dual union all
select 15200,'NOL' from dual union all
select 15203,'NOL' from dual union all
select 15204,'KOBN' from dual union all
select 15204,'RSRS' from dual union all
select 15204,'UCL' from dual union all
select 15206,'GSFC' from dual union all
select 15207,'RSRS' from dual union all
select 15207,'UCL' from dual union all
select 15208,'GSFC' from dual union all
select 15209,'RSRS' from dual union all
select 15209,'UCL' from dual union all
select 15210,'GSFC' from dual union all
select 15211,'RSRS' from dual union all
select 15211,'UCL' from dual union all
select 15212,'GSFC' from dual union all
select 15213,'RSRS' from dual union all
select 15213,'UCL' from dual union all
select 15214,'GSFC' from dual union all
select 15216,'GSFC' from dual union all
select 15217,'NOL' from dual union all
select 15219,'NOL' from dual union all
select 15224,'WSMR' from dual union all
select 15226,'CONAE' from dual union all
select 15229,'GSFC' from dual union all
select 15230,'GSFC' from dual union all
select 15232,'PSU' from dual union all
select 15236,'INTA' from dual union all
select 15237,'CONAE' from dual union all
select 15237,'WSMR' from dual union all
select 15238,'CONAE' from dual union all
select 15238,'WSMR' from dual union all
select 15239,'CONAE' from dual union all
select 15239,'WSMR' from dual union all
select 15240,'CONAE' from dual union all
select 15240,'WSMR' from dual union all
select 15241,'CONAE' from dual union all
select 15241,'WSMR' from dual union all
select 15242,'CONAE' from dual union all
select 15242,'WSMR' from dual union all
select 15243,'CONAE' from dual union all
select 15243,'WSMR' from dual union all
select 15244,'CONAE' from dual union all
select 15244,'WSMR' from dual union all
select 15245,'CONAE' from dual union all
select 15245,'WSMR' from dual union all
select 15246,'CONAE' from dual union all
select 15246,'WSMR' from dual union all
select 15247,'CONAE' from dual union all
select 15247,'WSMR' from dual union all
select 15248,'CONAE' from dual union all
select 15248,'WSMR' from dual union all
select 15251,'AN' from dual union all
select 15384,'PIHM' from dual union all
select 15385,'PIHM' from dual union all
select 15386,'PIHM' from dual union all
select 15387,'PIHM' from dual union all
select 15388,'PIHM' from dual union all
select 15389,'PIHM' from dual union all
select 15390,'PIHM' from dual union all
select 15391,'PIHM' from dual union all
select 15392,'PIHM' from dual union all
select 15397,'WFF' from dual union all
select 15400,'NDRE' from dual union all
select 15401,'GSFC' from dual union all
select 15403,'PIHM' from dual union all
select 15407,'LARC' from dual union all
select 15408,'DASA' from dual union all
select 15410,'UIUC' from dual union all
select 15411,'GSFC' from dual union all
select 15412,'UMI' from dual union all
select 15414,'PIHM' from dual union all
select 15416,'LARC' from dual union all
select 15418,'GSFC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 15420,'LARC' from dual union all
select 15421,'LARC' from dual union all
select 15423,'BONN' from dual union all
select 15423,'UTR' from dual union all
select 15425,'PIHM' from dual union all
select 15426,'AFCRL' from dual union all
select 15428,'DASA' from dual union all
select 15432,'ISRO' from dual union all
select 15433,'ISRO' from dual union all
select 15434,'CEA' from dual union all
select 15435,'ISRO' from dual union all
select 15436,'PIHM' from dual union all
select 15438,'AFCRL' from dual union all
select 15442,'IAFE' from dual union all
select 15444,'CNES' from dual union all
select 15445,'ONERA' from dual union all
select 15446,'ONERA' from dual union all
select 15447,'PIHM' from dual union all
select 15448,'ONERA' from dual union all
select 15449,'SAND' from dual union all
select 15450,'SAND' from dual union all
select 15451,'AFCRL' from dual union all
select 15451,'SDCO' from dual union all
select 15452,'AFCRL' from dual union all
select 15452,'SDCO' from dual union all
select 15453,'AFCRL' from dual union all
select 15453,'SDCO' from dual union all
select 15454,'AFCRL' from dual union all
select 15454,'SDCO' from dual union all
select 15457,'BERN' from dual union all
select 15458,'PIHM' from dual union all
select 15459,'IIAE' from dual union all
select 15463,'PIHM' from dual union all
select 15464,'PIHM' from dual union all
select 15465,'PIHM' from dual union all
select 15466,'PIHM' from dual union all
select 15467,'PIHM' from dual union all
select 15468,'PIHM' from dual union all
select 15469,'PIHM' from dual union all
select 15470,'PIHM' from dual union all
select 15471,'PIHM' from dual union all
select 15472,'PIHM' from dual union all
select 15473,'PIHM' from dual union all
select 15474,'PIHM' from dual union all
select 15475,'PIHM' from dual union all
select 15476,'PIHM' from dual union all
select 15477,'PIHM' from dual union all
select 15478,'PIHM' from dual union all
select 15479,'PIHM' from dual union all
select 15480,'PIHM' from dual union all
select 15481,'PIHM' from dual union all
select 15482,'PIHM' from dual union all
select 15483,'PIHM' from dual union all
select 15484,'PIHM' from dual union all
select 15485,'PIHM' from dual union all
select 15486,'PIHM' from dual union all
select 15487,'PIHM' from dual union all
select 15489,'STA' from dual union all
select 15494,'AFCRL' from dual union all
select 15496,'AFCRL' from dual union all
select 15500,'LARC' from dual union all
select 15501,'UMI' from dual union all
select 15507,'ISAS' from dual union all
select 15509,'CONAE' from dual union all
select 15510,'CNRS' from dual union all
select 15513,'PIHM' from dual union all
select 15515,'AFCRL' from dual union all
select 15517,'GSFC' from dual union all
select 15524,'LARC' from dual union all
select 15527,'PIHM' from dual union all
select 15529,'LRL' from dual union all
select 15535,'DUD' from dual union all
select 15538,'LARC' from dual union all
select 15562,'AFAPL' from dual union all
select 15562,'GASL' from dual union all
select 15564,'AFCRL' from dual union all
select 15567,'GSFC' from dual union all
select 15571,'AFCRL' from dual union all
select 15572,'AFCRL' from dual union all
select 15573,'AFCRL' from dual union all
select 15574,'AFCRL' from dual union all
select 15575,'AFCRL' from dual union all
select 15576,'AFCRL' from dual union all
select 15578,'AFCRL' from dual union all
select 15579,'AFCRL' from dual union all
select 15580,'AFCRL' from dual union all
select 15581,'AFCRL' from dual union all
select 15582,'AFCRL' from dual union all
select 15583,'USA' from dual union all
select 15584,'LRL' from dual union all
select 15585,'LRL' from dual union all
select 15586,'LRL' from dual union all
select 15587,'LRL' from dual union all
select 15589,'LRL' from dual union all
select 15590,'LRL' from dual union all
select 15591,'LRL' from dual union all
select 15592,'LRL' from dual union all
select 15593,'IIAE' from dual union all
select 15594,'IIAE' from dual union all
select 15595,'IIAE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 15596,'AFCRL' from dual union all
select 15597,'AFCRL' from dual union all
select 15598,'SAND' from dual union all
select 15603,'IAFE' from dual union all
select 15604,'AFCRL' from dual union all
select 15605,'AFCRL' from dual union all
select 15606,'AFCRL' from dual union all
select 15607,'AFCRL' from dual union all
select 15608,'AFCRL' from dual union all
select 15609,'AFCRL' from dual union all
select 15610,'AFCRL' from dual union all
select 15611,'AFCRL' from dual union all
select 15612,'AFCRL' from dual union all
select 15613,'AFCRL' from dual union all
select 15614,'AFCRL' from dual union all
select 15615,'AFCRL' from dual union all
select 15621,'ONERA' from dual union all
select 15622,'ONERA' from dual union all
select 15623,'SECT' from dual union all
select 15626,'UMI' from dual union all
select 15627,'GCA' from dual union all
select 15628,'GSFC' from dual union all
select 15629,'UMI' from dual union all
select 15630,'GCA' from dual union all
select 15631,'GSFC' from dual union all
select 15632,'UMI' from dual union all
select 15633,'GCA' from dual union all
select 15634,'GSFC' from dual union all
select 15635,'GCA' from dual union all
select 15637,'UMI' from dual union all
select 15638,'GSFC' from dual union all
select 15640,'AN' from dual union all
select 15641,'GSFC' from dual union all
select 15642,'NRCC' from dual union all
select 15643,'LIEGE' from dual union all
select 15643,'UTR' from dual union all
select 15644,'ISAS' from dual union all
select 15644,'NAG' from dual union all
select 15646,'RICE' from dual union all
select 15649,'BREI' from dual union all
select 15649,'MSSL' from dual union all
select 15650,'JHU' from dual union all
select 15653,'BREI' from dual union all
select 15653,'MSSL' from dual union all
select 15655,'CNES' from dual union all
select 15657,'HCO' from dual union all
select 15659,'WSMR' from dual union all
select 15661,'LASL' from dual union all
select 15664,'GSFC' from dual union all
select 15665,'CORN' from dual union all
select 15666,'GSFC' from dual union all
select 15666,'NDRE' from dual union all
select 15669,'AFCRL' from dual union all
select 15671,'AFCRL' from dual union all
select 15672,'PRL' from dual union all
select 15674,'CERMA' from dual union all
select 15675,'NAG' from dual union all
select 15675,'TAO' from dual union all
select 15676,'GSFC' from dual union all
select 15677,'PRL' from dual union all
select 15678,'PRL' from dual union all
select 15679,'PRL' from dual union all
select 15682,'CERMA' from dual union all
select 15683,'PRL' from dual union all
select 15684,'CULH' from dual union all
select 15684,'LUX' from dual union all
select 15685,'AFCRL' from dual union all
select 15686,'NDRE' from dual union all
select 15686,'NICP' from dual union all
select 15689,'PRL' from dual union all
select 15693,'NRL' from dual union all
select 15694,'RICE' from dual union all
select 15696,'AFCRL' from dual union all
select 15697,'BREI' from dual union all
select 15699,'UNH' from dual union all
select 15703,'CNRS' from dual union all
select 15704,'BREI' from dual union all
select 15707,'GCA' from dual union all
select 15707,'UMD' from dual union all
select 15709,'SECT' from dual union all
select 15713,'GSFC' from dual union all
select 15714,'GCA' from dual union all
select 15716,'GSFC' from dual union all
select 15717,'GSFC' from dual union all
select 15720,'CEA' from dual union all
select 15720,'LAS' from dual union all
select 15721,'GSFC' from dual union all
select 15722,'ADEL' from dual union all
select 15722,'UCW' from dual union all
select 15723,'LPARL' from dual union all
select 15727,'PRIN' from dual union all
select 15728,'AFCRL' from dual union all
select 15729,'MPE' from dual union all
select 15731,'MPE' from dual union all
select 15732,'MPE' from dual union all
select 15733,'LUX' from dual union all
select 15733,'QUB' from dual union all
select 15734,'MPE' from dual union all
select 15735,'GSFC' from dual union all
select 15736,'GSFC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 15737,'MPE' from dual union all
select 15740,'GSFC' from dual union all
select 15742,'LUX' from dual union all
select 15742,'QUB' from dual union all
select 15743,'AFCRL' from dual union all
select 15744,'AFCRL' from dual union all
select 15745,'AFCRL' from dual union all
select 15746,'AFCRL' from dual union all
select 15747,'AFCRL' from dual union all
select 15748,'AFCRL' from dual union all
select 15750,'AFCRL' from dual union all
select 15751,'AFCRL' from dual union all
select 15753,'AFCRL' from dual union all
select 15754,'GSFC' from dual union all
select 15755,'AESPC' from dual union all
select 15755,'ESTEC' from dual union all
select 15756,'PRL' from dual union all
select 15757,'AFCRL' from dual union all
select 15759,'GSFC' from dual union all
select 15760,'ADEL' from dual union all
select 15760,'UCW' from dual union all
select 15764,'GSFC' from dual union all
select 15765,'BAL' from dual union all
select 15768,'BAL' from dual union all
select 15769,'RAE' from dual union all
select 15769,'WRE' from dual union all
select 15771,'AFCRL' from dual union all
select 15772,'GSFC' from dual union all
select 15775,'SAMSO' from dual union all
select 15776,'GSFC' from dual union all
select 15777,'CNRS' from dual union all
select 15777,'SUPA' from dual union all
select 15778,'GSFC' from dual union all
select 15781,'GSFC' from dual union all
select 15782,'SASK' from dual union all
select 15783,'PRIN' from dual union all
select 15784,'SUPA' from dual union all
select 15786,'UTD' from dual union all
select 15787,'CNRS' from dual union all
select 15787,'SUPA' from dual union all
select 15789,'GSFC' from dual union all
select 15790,'NRL' from dual union all
select 15792,'SAND' from dual union all
select 15793,'SAND' from dual union all
select 15794,'GSFC' from dual union all
select 15795,'GSFC' from dual union all
select 15797,'NRL' from dual union all
select 15799,'PITT' from dual union all
select 15800,'LRL' from dual union all
select 15804,'SAND' from dual union all
select 15806,'BOL' from dual union all
select 15806,'ROE' from dual union all
select 15810,'ROE' from dual union all
select 15812,'SAMSO' from dual union all
select 15813,'BAJ' from dual union all
select 15814,'BAJ' from dual union all
select 15819,'SAND' from dual union all
select 1582,'LRBA' from dual union all
select 15820,'SAND' from dual union all
select 15821,'SAND' from dual union all
select 15822,'SAND' from dual union all
select 15823,'SAND' from dual union all
select 15824,'SAND' from dual union all
select 15827,'GSFC' from dual union all
select 15829,'SAND' from dual union all
select 15830,'SAND' from dual union all
select 15832,'ARC' from dual union all
select 15833,'UNH' from dual union all
select 15834,'AFCRL' from dual union all
select 15837,'PRIN' from dual union all
select 15841,'ASE' from dual union all
select 15844,'MPAE' from dual union all
select 15845,'MPAE' from dual union all
select 15851,'UTD' from dual union all
select 15852,'UMN' from dual union all
select 15862,'GSFC' from dual union all
select 15863,'GSFC' from dual union all
select 15864,'GSFC' from dual union all
select 15866,'CNES' from dual union all
select 15867,'AFCRL' from dual union all
select 15870,'NMSU' from dual union all
select 15871,'SAMSO' from dual union all
select 15873,'BAJ' from dual union all
select 15875,'MPE' from dual union all
select 15877,'MPE' from dual union all
select 15878,'OPM' from dual union all
select 15879,'AFCRL' from dual union all
select 15884,'MIT' from dual union all
select 15887,'SASK' from dual union all
select 15888,'SASK' from dual union all
select 1589,'LRBA' from dual union all
select 15896,'UMN' from dual union all
select 15897,'UMN' from dual union all
select 15899,'GCA' from dual union all
select 15902,'SAND' from dual union all
select 15905,'AFCRL' from dual union all
select 15906,'SAMSO' from dual union all
select 15908,'RTG' from dual union all
select 15909,'LUND' from dual union all
select 15909,'MPIK' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 15910,'MPE' from dual union all
select 15911,'UMI' from dual union all
select 15913,'UWI' from dual union all
select 15914,'GSFC' from dual union all
select 15916,'UMI' from dual union all
select 15917,'MPE' from dual union all
select 15920,'LUX' from dual union all
select 15920,'MSSL' from dual union all
select 15921,'UMI' from dual union all
select 15922,'AFCRL' from dual union all
select 15923,'UIUC' from dual union all
select 15924,'GCA' from dual union all
select 15926,'GCA' from dual union all
select 15927,'GCA' from dual union all
select 15928,'GCA' from dual union all
select 15929,'GCA' from dual union all
select 15930,'ARC' from dual union all
select 15932,'NRL' from dual union all
select 15933,'AFCRL' from dual union all
select 15934,'RAE' from dual union all
select 15934,'WRE' from dual union all
select 15940,'OX' from dual union all
select 15942,'NDRE' from dual union all
select 15943,'LPARL' from dual union all
select 15944,'AFCRL' from dual union all
select 15945,'GSFC' from dual union all
select 15946,'GSFC' from dual union all
select 15948,'GSFC' from dual union all
select 15953,'OX' from dual union all
select 15954,'GSFC' from dual union all
select 15955,'TUC' from dual union all
select 15957,'SAMSO' from dual union all
select 15961,'GSFC' from dual union all
select 15962,'GSFC' from dual union all
select 15963,'TUC' from dual union all
select 15964,'GSFC' from dual union all
select 15965,'GSFC' from dual union all
select 15969,'ONUSJ' from dual union all
select 15970,'AFCRL' from dual union all
select 15971,'ONUSJ' from dual union all
select 15972,'ONUSJ' from dual union all
select 15973,'UIUC' from dual union all
select 15974,'LRL' from dual union all
select 15978,'UIUC' from dual union all
select 15980,'NRL' from dual union all
select 15982,'NMSU' from dual union all
select 15985,'GSFC' from dual union all
select 15986,'GSFC' from dual union all
select 15989,'GSFC' from dual union all
select 15989,'UMI' from dual union all
select 15991,'NMSU' from dual union all
select 15992,'NMSU' from dual union all
select 15994,'LASL' from dual union all
select 15995,'SAMSO' from dual union all
select 15997,'SAND' from dual union all
select 15998,'UIA' from dual union all
select 15999,'NMSU' from dual union all
select 16000,'NMSU' from dual union all
select 16002,'NMSU' from dual union all
select 16004,'NMSU' from dual union all
select 16005,'NMSU' from dual union all
select 16006,'SAND' from dual union all
select 16007,'BIR' from dual union all
select 16007,'RSRS' from dual union all
select 16008,'BAL' from dual union all
select 16008,'NRCC' from dual union all
select 16009,'LRL' from dual union all
select 16014,'AFCRL' from dual union all
select 16017,'LRL' from dual union all
select 16018,'AFCRL' from dual union all
select 16019,'LASL' from dual union all
select 16021,'SAND' from dual union all
select 16022,'GSFC' from dual union all
select 16024,'SAND' from dual union all
select 16025,'AFCRL' from dual union all
select 16026,'BONN' from dual union all
select 16026,'UTR' from dual union all
select 16027,'BONN' from dual union all
select 16027,'LPA' from dual union all
select 16027,'MSSL' from dual union all
select 16028,'AFCRL' from dual union all
select 16029,'BAL' from dual union all
select 16029,'NRCC' from dual union all
select 16030,'NRL' from dual union all
select 16031,'MPE' from dual union all
select 16031,'UTR' from dual union all
select 16032,'BIR' from dual union all
select 16032,'RSRS' from dual union all
select 16035,'BERG' from dual union all
select 16035,'NDRE' from dual union all
select 16036,'CNRS' from dual union all
select 16037,'BONN' from dual union all
select 16037,'MSSL' from dual union all
select 16038,'CNRS' from dual union all
select 16039,'SAMSO' from dual union all
select 16043,'SAMSO' from dual union all
select 16047,'BERG' from dual union all
select 16047,'NDRE' from dual union all
select 16049,'GSFC' from dual union all
select 16050,'GSFC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 16051,'GSFC' from dual union all
select 16053,'UCO' from dual union all
select 16054,'CALG' from dual union all
select 16057,'CALG' from dual union all
select 16058,'MPE' from dual union all
select 16059,'AFCRL' from dual union all
select 16060,'AFCRL' from dual union all
select 16062,'BERN' from dual union all
select 16063,'GSFC' from dual union all
select 16064,'AFCRL' from dual union all
select 16065,'AFCRL' from dual union all
select 16075,'CULH' from dual union all
select 16075,'LUX' from dual union all
select 16076,'PRIN' from dual union all
select 16078,'IEM' from dual union all
select 16084,'CNIE' from dual union all
select 16086,'AFCRL' from dual union all
select 16087,'MEWTA' from dual union all
select 16092,'ICST' from dual union all
select 16092,'RSRS' from dual union all
select 16093,'NRL' from dual union all
select 16095,'AFCRL' from dual union all
select 16097,'UMD' from dual union all
select 16098,'ICST' from dual union all
select 16098,'RSRS' from dual union all
select 16099,'CEA' from dual union all
select 16099,'LAS' from dual union all
select 16101,'AFCRL' from dual union all
select 16102,'ASE' from dual union all
select 16104,'AFCRL' from dual union all
select 16105,'UCB' from dual union all
select 16106,'DTU' from dual union all
select 16106,'MSSL' from dual union all
select 16106,'RSRS' from dual union all
select 16110,'CNRS' from dual union all
select 16112,'BREI' from dual union all
select 16112,'DSRI' from dual union all
select 16113,'SUPA' from dual union all
select 16114,'AFCRL' from dual union all
select 16115,'AFCRL' from dual union all
select 16118,'ADEL' from dual union all
select 16119,'AFCRL' from dual union all
select 16120,'AFCRL' from dual union all
select 16121,'AFCRL' from dual union all
select 16122,'KPNO' from dual union all
select 16122,'UFL' from dual union all
select 16124,'AFCRL' from dual union all
select 16125,'AFCRL' from dual union all
select 16126,'UAF' from dual union all
select 16127,'JHU' from dual union all
select 16128,'WFF' from dual union all
select 16129,'BRSW' from dual union all
select 16130,'GSFC' from dual union all
select 16131,'LRL' from dual union all
select 16132,'AFCRL' from dual union all
select 16133,'AFCRL' from dual union all
select 16134,'AFCRL' from dual union all
select 16135,'BREI' from dual union all
select 16135,'DSRI' from dual union all
select 16136,'UTD' from dual union all
select 16139,'ARC' from dual union all
select 16140,'USAF' from dual union all
select 16141,'UCO' from dual union all
select 16142,'GSFC' from dual union all
select 16143,'AFCRL' from dual union all
select 16144,'AFCRL' from dual union all
select 16145,'AFCRL' from dual union all
select 16146,'AFCRL' from dual union all
select 16147,'AFCRL' from dual union all
select 16148,'AFCRL' from dual union all
select 16149,'AFCRL' from dual union all
select 16150,'AFCRL' from dual union all
select 16152,'GSFC' from dual union all
select 16153,'GSFC' from dual union all
select 16154,'SAND' from dual union all
select 16155,'GSFC' from dual union all
select 16162,'SECT' from dual union all
select 16164,'AFCRL' from dual union all
select 16164,'WSMR' from dual union all
select 16165,'LAS' from dual union all
select 16166,'AN' from dual union all
select 16168,'AFAL' from dual union all
select 16168,'OSU' from dual union all
select 16169,'SAND' from dual union all
select 16170,'SAND' from dual union all
select 16171,'SAND' from dual union all
select 16172,'SAND' from dual union all
select 16173,'SRCI' from dual union all
select 16174,'BRL' from dual union all
select 16175,'BRL' from dual union all
select 16176,'AFCRL' from dual union all
select 16177,'BRL' from dual union all
select 16178,'SRCI' from dual union all
select 16179,'SRCI' from dual union all
select 16180,'SRCI' from dual union all
select 16181,'AFCRL' from dual union all
select 16182,'AFCRL' from dual union all
select 16183,'AFCRL' from dual union all
select 16184,'AFCRL' from dual union all
select 16185,'AFCRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 16186,'AFCRL' from dual union all
select 16188,'AFCRL' from dual union all
select 16189,'AFCRL' from dual union all
select 16190,'AFCRL' from dual union all
select 16191,'AFCRL' from dual union all
select 16192,'IIAE' from dual union all
select 16193,'WSMR' from dual union all
select 16194,'WSMR' from dual union all
select 16195,'WSMR' from dual union all
select 16196,'WSMR' from dual union all
select 16201,'NMSU' from dual union all
select 16202,'NMSU' from dual union all
select 16203,'NMSU' from dual union all
select 16204,'NMSU' from dual union all
select 16205,'NMSU' from dual union all
select 16206,'NMSU' from dual union all
select 16207,'NMSU' from dual union all
select 16208,'NMSU' from dual union all
select 16210,'MEWTA' from dual union all
select 16211,'WRE' from dual union all
select 16213,'CNIE' from dual union all
select 16214,'CNIE' from dual union all
select 16220,'AFCRL' from dual union all
select 16225,'AEC' from dual union all
select 16225,'SAND' from dual union all
select 16226,'AEC' from dual union all
select 16226,'SAND' from dual union all
select 16227,'AEC' from dual union all
select 16227,'SAND' from dual union all
select 16228,'AEC' from dual union all
select 16228,'SAND' from dual union all
select 16229,'AEC' from dual union all
select 16229,'SAND' from dual union all
select 16230,'AEC' from dual union all
select 16230,'SAND' from dual union all
select 16231,'NRCC' from dual union all
select 16232,'AEC' from dual union all
select 16232,'SAND' from dual union all
select 16233,'AEC' from dual union all
select 16233,'SAND' from dual union all
select 16234,'AEC' from dual union all
select 16234,'SAND' from dual union all
select 16235,'AEC' from dual union all
select 16235,'SAND' from dual union all
select 16236,'NMSU' from dual union all
select 16237,'NMSU' from dual union all
select 16253,'NRCC' from dual union all
select 16264,'AFCRL' from dual union all
select 16264,'WSMR' from dual union all
select 16265,'AFCRL' from dual union all
select 16265,'WSMR' from dual union all
select 16266,'AFCRL' from dual union all
select 16266,'WSMR' from dual union all
select 16267,'AFCRL' from dual union all
select 16267,'WSMR' from dual union all
select 16279,'JMA' from dual union all
select 16282,'BERN' from dual union all
select 16284,'IIAE' from dual union all
select 16290,'MSFC' from dual union all
select 16290,'UMI' from dual union all
select 16301,'MSFC' from dual union all
select 16301,'UMI' from dual union all
select 16306,'MSFC' from dual union all
select 16306,'UMI' from dual union all
select 16307,'MSFC' from dual union all
select 16307,'UMI' from dual union all
select 16308,'MSFC' from dual union all
select 16308,'UMI' from dual union all
select 16309,'MSFC' from dual union all
select 16309,'UMI' from dual union all
select 16310,'MSFC' from dual union all
select 16310,'UMI' from dual union all
select 16311,'MSFC' from dual union all
select 16311,'UMI' from dual union all
select 16312,'GRI' from dual union all
select 16314,'ISAS' from dual union all
select 16315,'GSFC' from dual union all
select 16316,'GRI' from dual union all
select 16317,'GRI' from dual union all
select 16318,'GRI' from dual union all
select 16320,'NAG' from dual union all
select 16321,'UIUC' from dual union all
select 16322,'GSFC' from dual union all
select 16323,'AN' from dual union all
select 16324,'UMI' from dual union all
select 16325,'GCA' from dual union all
select 16326,'GSFC' from dual union all
select 16327,'GCA' from dual union all
select 16328,'GSFC' from dual union all
select 16329,'LARC' from dual union all
select 16332,'LARC' from dual union all
select 16333,'USAF' from dual union all
select 16334,'USAF' from dual union all
select 16335,'USAF' from dual union all
select 16336,'USAF' from dual union all
select 16337,'USAF' from dual union all
select 16338,'USAF' from dual union all
select 16339,'USAF' from dual union all
select 16340,'USAF' from dual union all
select 16341,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 16342,'USAF' from dual union all
select 16348,'USAF' from dual union all
select 16349,'USAF' from dual union all
select 16350,'USAF' from dual union all
select 16351,'USAF' from dual union all
select 16352,'USAF' from dual union all
select 16353,'USAF' from dual union all
select 16354,'USAF' from dual union all
select 16355,'USAF' from dual union all
select 16356,'USAF' from dual union all
select 16362,'CONAE' from dual union all
select 16363,'USAF' from dual union all
select 16364,'USAF' from dual union all
select 16365,'USAF' from dual union all
select 16366,'USAF' from dual union all
select 16367,'USAF' from dual union all
select 16368,'USAF' from dual union all
select 16369,'USAF' from dual union all
select 16370,'USAF' from dual union all
select 16371,'USAF' from dual union all
select 16372,'USAF' from dual union all
select 16373,'USAF' from dual union all
select 16374,'AFCRL' from dual union all
select 16375,'USAF' from dual union all
select 16376,'CONAE' from dual union all
select 16378,'DUD' from dual union all
select 16384,'USAF' from dual union all
select 16385,'USAF' from dual union all
select 16386,'USAF' from dual union all
select 16387,'USAF' from dual union all
select 16388,'USAF' from dual union all
select 16389,'USAF' from dual union all
select 16390,'USAF' from dual union all
select 16391,'USAF' from dual union all
select 16392,'USAF' from dual union all
select 16393,'USAF' from dual union all
select 16394,'UCW' from dual union all
select 16397,'USAF' from dual union all
select 16399,'CONAE' from dual union all
select 16404,'LARC' from dual union all
select 16405,'USAF' from dual union all
select 16406,'USAF' from dual union all
select 16407,'USAF' from dual union all
select 16408,'USAF' from dual union all
select 16409,'USAF' from dual union all
select 16410,'USAF' from dual union all
select 16411,'USAF' from dual union all
select 16412,'USAF' from dual union all
select 16413,'USAF' from dual union all
select 16414,'USAF' from dual union all
select 16415,'NOTS' from dual union all
select 16416,'CONAE' from dual union all
select 16419,'USAF' from dual union all
select 16420,'NOTS' from dual union all
select 16421,'WSMR' from dual union all
select 16424,'USAF' from dual union all
select 16425,'USAF' from dual union all
select 16426,'USAF' from dual union all
select 16428,'CONAE' from dual union all
select 16429,'NOTS' from dual union all
select 16430,'USAF' from dual union all
select 16432,'NOTS' from dual union all
select 16435,'WSMR' from dual union all
select 16436,'WSMR' from dual union all
select 16437,'NOTS' from dual union all
select 16438,'WSMR' from dual union all
select 16440,'USAF' from dual union all
select 16442,'USAF' from dual union all
select 16443,'USAF' from dual union all
select 16444,'USAF' from dual union all
select 16445,'WSMR' from dual union all
select 16446,'NOTS' from dual union all
select 16447,'CONAE' from dual union all
select 16448,'WSMR' from dual union all
select 16449,'NOTS' from dual union all
select 16450,'WSMR' from dual union all
select 16453,'NOTS' from dual union all
select 16455,'GSFC' from dual union all
select 16456,'GSFC' from dual union all
select 16457,'NOTS' from dual union all
select 16459,'USAF' from dual union all
select 16460,'USAF' from dual union all
select 16461,'USAF' from dual union all
select 16462,'USAF' from dual union all
select 16464,'USAF' from dual union all
select 16465,'WSMR' from dual union all
select 16466,'WSMR' from dual union all
select 16467,'WSMR' from dual union all
select 16468,'MSFC' from dual union all
select 16469,'MSFC' from dual union all
select 16471,'WSMR' from dual union all
select 16472,'WSMR' from dual union all
select 16473,'CONAE' from dual union all
select 16475,'ISRO' from dual union all
select 16481,'SUPA' from dual union all
select 16483,'USAF' from dual union all
select 16484,'USAF' from dual union all
select 16486,'USAF' from dual union all
select 16487,'USAF' from dual union all
select 16488,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 16489,'USAF' from dual union all
select 16490,'USAF' from dual union all
select 16492,'SUPA' from dual union all
select 16493,'CONAE' from dual union all
select 16495,'ISRO' from dual union all
select 16497,'WFF' from dual union all
select 16498,'WFF' from dual union all
select 16499,'WFF' from dual union all
select 16500,'WFF' from dual union all
select 16501,'WFF' from dual union all
select 16502,'WFF' from dual union all
select 16503,'WFF' from dual union all
select 16504,'WFF' from dual union all
select 16505,'CONAE' from dual union all
select 16506,'WFF' from dual union all
select 16507,'WFF' from dual union all
select 16508,'WFF' from dual union all
select 16509,'WFF' from dual union all
select 16510,'WFF' from dual union all
select 16511,'WFF' from dual union all
select 16512,'WFF' from dual union all
select 16513,'WFF' from dual union all
select 16514,'WFF' from dual union all
select 16515,'WFF' from dual union all
select 16516,'JMA' from dual union all
select 16517,'WFF' from dual union all
select 16518,'WFF' from dual union all
select 16519,'WFF' from dual union all
select 16520,'WFF' from dual union all
select 16521,'WFF' from dual union all
select 16522,'WFF' from dual union all
select 16523,'WFF' from dual union all
select 16524,'WFF' from dual union all
select 16525,'WFF' from dual union all
select 16526,'WFF' from dual union all
select 16528,'WFF' from dual union all
select 16529,'WFF' from dual union all
select 16530,'WFF' from dual union all
select 16531,'WFF' from dual union all
select 16532,'WFF' from dual union all
select 16533,'WFF' from dual union all
select 16534,'WFF' from dual union all
select 16535,'WFF' from dual union all
select 16536,'WFF' from dual union all
select 16537,'WFF' from dual union all
select 16539,'WFF' from dual union all
select 16540,'WFF' from dual union all
select 16541,'WFF' from dual union all
select 16542,'WFF' from dual union all
select 16543,'WFF' from dual union all
select 16544,'WFF' from dual union all
select 16545,'WFF' from dual union all
select 16546,'WFF' from dual union all
select 16547,'WFF' from dual union all
select 16548,'WFF' from dual union all
select 16549,'JMA' from dual union all
select 16550,'WFF' from dual union all
select 16551,'WFF' from dual union all
select 16552,'WFF' from dual union all
select 16553,'WFF' from dual union all
select 16554,'AFCRL' from dual union all
select 16554,'WSMR' from dual union all
select 16555,'AFCRL' from dual union all
select 16555,'WSMR' from dual union all
select 16556,'AFCRL' from dual union all
select 16556,'WSMR' from dual union all
select 16557,'AFCRL' from dual union all
select 16557,'WSMR' from dual union all
select 16558,'AFCRL' from dual union all
select 16558,'WSMR' from dual union all
select 16559,'WSMR' from dual union all
select 16560,'JMA' from dual union all
select 16561,'INPE' from dual union all
select 16562,'INPE' from dual union all
select 16563,'INPE' from dual union all
select 16564,'INPE' from dual union all
select 16565,'INPE' from dual union all
select 16566,'INPE' from dual union all
select 16567,'INPE' from dual union all
select 16568,'INPE' from dual union all
select 16569,'INPE' from dual union all
select 16570,'INPE' from dual union all
select 16571,'JMA' from dual union all
select 16572,'INPE' from dual union all
select 16573,'INPE' from dual union all
select 16574,'INPE' from dual union all
select 16575,'INPE' from dual union all
select 16576,'INPE' from dual union all
select 16577,'INPE' from dual union all
select 16578,'WFF' from dual union all
select 16579,'WFF' from dual union all
select 16580,'WFF' from dual union all
select 16581,'WFF' from dual union all
select 16583,'WFF' from dual union all
select 16584,'WFF' from dual union all
select 16585,'WFF' from dual union all
select 16586,'WFF' from dual union all
select 16587,'WFF' from dual union all
select 16588,'WFF' from dual union all
select 16589,'AFCRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 16589,'SDCO' from dual union all
select 16590,'AFCRL' from dual union all
select 16590,'SDCO' from dual union all
select 16591,'AFCRL' from dual union all
select 16591,'SDCO' from dual union all
select 16592,'AFCRL' from dual union all
select 16592,'SDCO' from dual union all
select 16593,'JMA' from dual union all
select 16594,'AFCRL' from dual union all
select 16594,'SDCO' from dual union all
select 16595,'AFCRL' from dual union all
select 16595,'SDCO' from dual union all
select 16596,'AFCRL' from dual union all
select 16596,'SDCO' from dual union all
select 16597,'AFCRL' from dual union all
select 16597,'SDCO' from dual union all
select 16598,'AFCRL' from dual union all
select 16598,'SDCO' from dual union all
select 16599,'AFCRL' from dual union all
select 16599,'SDCO' from dual union all
select 16600,'AFCRL' from dual union all
select 16600,'SDCO' from dual union all
select 16601,'AFCRL' from dual union all
select 16602,'AFCRL' from dual union all
select 16603,'AFCRL' from dual union all
select 16604,'MSFC' from dual union all
select 16605,'AFCRL' from dual union all
select 16606,'AFCRL' from dual union all
select 16610,'USAF' from dual union all
select 16611,'USAF' from dual union all
select 16612,'USAF' from dual union all
select 16613,'USAF' from dual union all
select 16614,'USAF' from dual union all
select 16615,'USAF' from dual union all
select 16616,'USAF' from dual union all
select 16617,'USAF' from dual union all
select 16620,'CONAE' from dual union all
select 16621,'USAF' from dual union all
select 16622,'USAF' from dual union all
select 16623,'USAF' from dual union all
select 16624,'USAF' from dual union all
select 16625,'USAF' from dual union all
select 16626,'USAF' from dual union all
select 16627,'USAF' from dual union all
select 16628,'USAF' from dual union all
select 16629,'USAF' from dual union all
select 16630,'USAF' from dual union all
select 16631,'MSFC' from dual union all
select 16634,'USAF' from dual union all
select 16635,'USAF' from dual union all
select 16636,'USAF' from dual union all
select 16637,'USAF' from dual union all
select 16638,'USAF' from dual union all
select 16639,'USAF' from dual union all
select 16640,'USAF' from dual union all
select 16641,'USAF' from dual union all
select 16642,'USAF' from dual union all
select 16643,'USAF' from dual union all
select 16644,'USAF' from dual union all
select 16645,'USAF' from dual union all
select 16646,'JMA' from dual union all
select 16647,'ESSA' from dual union all
select 16648,'JMA' from dual union all
select 16649,'ESSA' from dual union all
select 16650,'JMA' from dual union all
select 16651,'ESSA' from dual union all
select 16652,'JMA' from dual union all
select 16653,'ESSA' from dual union all
select 16654,'JMA' from dual union all
select 16655,'JMA' from dual union all
select 16656,'ESSA' from dual union all
select 16657,'ESSA' from dual union all
select 16658,'JMA' from dual union all
select 16659,'ESSA' from dual union all
select 16660,'JMA' from dual union all
select 16661,'ESSA' from dual union all
select 16662,'JMA' from dual union all
select 16663,'ESSA' from dual union all
select 16664,'JMA' from dual union all
select 16665,'ESSA' from dual union all
select 16666,'AFCRL' from dual union all
select 16669,'CONAE' from dual union all
select 16689,'WSMR' from dual union all
select 16698,'ISAS' from dual union all
select 16699,'ISAS' from dual union all
select 16700,'ISAS' from dual union all
select 16701,'ISAS' from dual union all
select 16702,'ISAS' from dual union all
select 16703,'ISAS' from dual union all
select 16704,'ISAS' from dual union all
select 16705,'ISAS' from dual union all
select 16706,'ISAS' from dual union all
select 16707,'ISAS' from dual union all
select 16712,'UCW' from dual union all
select 16847,'WRE' from dual union all
select 16848,'PIHM' from dual union all
select 16849,'PIHM' from dual union all
select 16850,'PIHM' from dual union all
select 16851,'PIHM' from dual union all
select 16852,'PIHM' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 16853,'PIHM' from dual union all
select 16854,'PIHM' from dual union all
select 16855,'PIHM' from dual union all
select 16856,'PIHM' from dual union all
select 16857,'GSFC' from dual union all
select 16858,'DASA' from dual union all
select 16859,'DASA' from dual union all
select 16865,'AFCRL' from dual union all
select 16866,'PIHM' from dual union all
select 16870,'MPE' from dual union all
select 16870,'NDRE' from dual union all
select 16874,'CNES' from dual union all
select 16875,'SAND' from dual union all
select 16876,'LARC' from dual union all
select 16877,'PIHM' from dual union all
select 16878,'AFCRL' from dual union all
select 16879,'PMR' from dual union all
select 16880,'PMR' from dual union all
select 16881,'PMR' from dual union all
select 16882,'PMR' from dual union all
select 16883,'PMR' from dual union all
select 16884,'PMR' from dual union all
select 16885,'PMR' from dual union all
select 16886,'PMR' from dual union all
select 16887,'PMR' from dual union all
select 16888,'PIHM' from dual union all
select 16889,'PMR' from dual union all
select 16890,'PMR' from dual union all
select 16892,'PIHM' from dual union all
select 16893,'PIHM' from dual union all
select 16894,'PIHM' from dual union all
select 16895,'PIHM' from dual union all
select 16896,'PIHM' from dual union all
select 16897,'PIHM' from dual union all
select 16898,'PIHM' from dual union all
select 16899,'PIHM' from dual union all
select 16900,'PIHM' from dual union all
select 16901,'PIHM' from dual union all
select 16902,'PIHM' from dual union all
select 16903,'PIHM' from dual union all
select 16904,'PIHM' from dual union all
select 16905,'PIHM' from dual union all
select 16906,'PIHM' from dual union all
select 16907,'PIHM' from dual union all
select 16908,'PIHM' from dual union all
select 16909,'PIHM' from dual union all
select 16910,'PIHM' from dual union all
select 16911,'PIHM' from dual union all
select 16912,'PIHM' from dual union all
select 16913,'PIHM' from dual union all
select 16914,'PIHM' from dual union all
select 16915,'PIHM' from dual union all
select 16916,'PIHM' from dual union all
select 16917,'PIHM' from dual union all
select 16918,'PIHM' from dual union all
select 16919,'PIHM' from dual union all
select 16920,'PIHM' from dual union all
select 16921,'PIHM' from dual union all
select 16922,'PIHM' from dual union all
select 16923,'PIHM' from dual union all
select 16924,'PIHM' from dual union all
select 16925,'ISRO' from dual union all
select 16926,'ISRO' from dual union all
select 16927,'ISRO' from dual union all
select 16932,'ISRO' from dual union all
select 16933,'ISRO' from dual union all
select 16934,'GSFC' from dual union all
select 16940,'GSFC' from dual union all
select 16943,'DASA' from dual union all
select 16945,'CNES' from dual union all
select 16950,'GSFC' from dual union all
select 16952,'YORK' from dual union all
select 16953,'UMI' from dual union all
select 16955,'CAL' from dual union all
select 16959,'MSSL' from dual union all
select 16967,'CONAE' from dual union all
select 16970,'NRCC' from dual union all
select 16972,'GSFC' from dual union all
select 16973,'ISRO' from dual union all
select 16975,'AFCRL' from dual union all
select 16976,'STA' from dual union all
select 16977,'GSFC' from dual union all
select 16988,'LARC' from dual union all
select 16990,'GSFC' from dual union all
select 16991,'YORK' from dual union all
select 16995,'YORK' from dual union all
select 16996,'YORK' from dual union all
select 16997,'MPAE' from dual union all
select 16998,'MPAE' from dual union all
select 17004,'NRL' from dual union all
select 17005,'ISAS' from dual union all
select 17006,'ISAS' from dual union all
select 17007,'ISAS' from dual union all
select 17009,'GSFC' from dual union all
select 17011,'LARC' from dual union all
select 17013,'BERG' from dual union all
select 17013,'NDRE' from dual union all
select 17015,'LARC' from dual union all
select 17016,'CBMET' from dual union all
select 17016,'WRE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 17017,'CBMET' from dual union all
select 17017,'WRE' from dual union all
select 17018,'CBMET' from dual union all
select 17018,'WRE' from dual union all
select 17019,'CBMET' from dual union all
select 17019,'WRE' from dual union all
select 17020,'CBMET' from dual union all
select 17020,'WRE' from dual union all
select 17021,'CBMET' from dual union all
select 17021,'WRE' from dual union all
select 17022,'CBMET' from dual union all
select 17022,'WRE' from dual union all
select 17023,'CBMET' from dual union all
select 17023,'WRE' from dual union all
select 17024,'CBMET' from dual union all
select 17024,'WRE' from dual union all
select 17025,'CBMET' from dual union all
select 17025,'WRE' from dual union all
select 17026,'CBMET' from dual union all
select 17026,'WRE' from dual union all
select 17027,'AFCRL' from dual union all
select 17028,'CBMET' from dual union all
select 17028,'WRE' from dual union all
select 17029,'CBMET' from dual union all
select 17029,'WRE' from dual union all
select 17030,'CBMET' from dual union all
select 17030,'WRE' from dual union all
select 17031,'AFCRL' from dual union all
select 17032,'INTA' from dual union all
select 17032,'NASA' from dual union all
select 17033,'INTA' from dual union all
select 17033,'NASA' from dual union all
select 17034,'INTA' from dual union all
select 17034,'NASA' from dual union all
select 17035,'INTA' from dual union all
select 17035,'NASA' from dual union all
select 17036,'INTA' from dual union all
select 17036,'NASA' from dual union all
select 17037,'INTA' from dual union all
select 17037,'NASA' from dual union all
select 17038,'INTA' from dual union all
select 17038,'NASA' from dual union all
select 17039,'INTA' from dual union all
select 17039,'NASA' from dual union all
select 17040,'INTA' from dual union all
select 17040,'NASA' from dual union all
select 17041,'INTA' from dual union all
select 17041,'NASA' from dual union all
select 17042,'INTA' from dual union all
select 17042,'NASA' from dual union all
select 17043,'INTA' from dual union all
select 17043,'NASA' from dual union all
select 17044,'INTA' from dual union all
select 17044,'NASA' from dual union all
select 17045,'INTA' from dual union all
select 17045,'NASA' from dual union all
select 17046,'INTA' from dual union all
select 17046,'NASA' from dual union all
select 17047,'IIAE' from dual union all
select 17048,'IIAE' from dual union all
select 17049,'IIAE' from dual union all
select 17058,'SECT' from dual union all
select 17059,'SECT' from dual union all
select 17060,'SECT' from dual union all
select 17065,'GSFC' from dual union all
select 17066,'NRCC' from dual union all
select 17070,'UCL' from dual union all
select 17071,'GSFC' from dual union all
select 17072,'GSFC' from dual union all
select 17073,'GSFC' from dual union all
select 17075,'GSFC' from dual union all
select 17076,'GSFC' from dual union all
select 17079,'WSMR' from dual union all
select 17080,'ASE' from dual union all
select 17081,'GSFC' from dual union all
select 17082,'PRL' from dual union all
select 17083,'PRL' from dual union all
select 17084,'RSRS' from dual union all
select 17086,'UCL' from dual union all
select 17087,'GSFC' from dual union all
select 17088,'GSFC' from dual union all
select 17089,'UNH' from dual union all
select 17090,'LASL' from dual union all
select 17092,'RAE' from dual union all
select 17092,'WRE' from dual union all
select 17093,'LASL' from dual union all
select 17094,'JHU' from dual union all
select 17097,'CNRS' from dual union all
select 17099,'MPAE' from dual union all
select 17100,'MPAE' from dual union all
select 17101,'KPNO' from dual union all
select 17102,'MPAE' from dual union all
select 17103,'AFCRL' from dual union all
select 17104,'LPARL' from dual union all
select 17105,'GMS' from dual union all
select 17105,'PRL' from dual union all
select 17107,'GCA' from dual union all
select 17108,'GSFC' from dual union all
select 17109,'GCA' from dual union all
select 17110,'GCA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 17111,'GCA' from dual union all
select 17112,'GCA' from dual union all
select 17113,'GCA' from dual union all
select 17114,'GMS' from dual union all
select 17114,'PRL' from dual union all
select 17115,'UCB' from dual union all
select 17117,'UAF' from dual union all
select 17119,'USAF' from dual union all
select 17120,'GCA' from dual union all
select 17123,'UAF' from dual union all
select 17124,'CNRS' from dual union all
select 17129,'RSRS' from dual union all
select 17130,'CORN' from dual union all
select 17131,'RICE' from dual union all
select 17132,'UCB' from dual union all
select 17137,'BIR' from dual union all
select 17137,'SOTON' from dual union all
select 17139,'MSSL' from dual union all
select 1714,'SUI' from dual union all
select 17140,'CNES' from dual union all
select 17141,'SAND' from dual union all
select 17142,'SAND' from dual union all
select 17143,'SAND' from dual union all
select 17144,'SECT' from dual union all
select 17145,'GRI' from dual union all
select 17146,'AFCRL' from dual union all
select 17147,'ASE' from dual union all
select 17148,'GSFC' from dual union all
select 1715,'SUI' from dual union all
select 17150,'GSFC' from dual union all
select 17151,'CNRS' from dual union all
select 17152,'GSFC' from dual union all
select 17152,'UMI' from dual union all
select 17153,'UMI' from dual union all
select 17154,'GSFC' from dual union all
select 17154,'UMI' from dual union all
select 17155,'UMI' from dual union all
select 17156,'GSFC' from dual union all
select 17156,'UMI' from dual union all
select 17157,'UMI' from dual union all
select 17158,'UCB' from dual union all
select 1716,'SUI' from dual union all
select 17162,'ARC' from dual union all
select 17164,'GMS' from dual union all
select 17165,'CULH' from dual union all
select 17165,'LUX' from dual union all
select 17166,'MPE' from dual union all
select 17167,'LRL' from dual union all
select 17167,'NMC' from dual union all
select 17168,'CNRS' from dual union all
select 17170,'SU' from dual union all
select 17173,'UMD' from dual union all
select 17175,'GSFC' from dual union all
select 17176,'MPE' from dual union all
select 17177,'CNET' from dual union all
select 17177,'SUPA' from dual union all
select 17178,'IEM' from dual union all
select 17179,'GSFC' from dual union all
select 1718,'SUI' from dual union all
select 17180,'IEM' from dual union all
select 17181,'GMS' from dual union all
select 17183,'GSFC' from dual union all
select 17184,'GSFC' from dual union all
select 17186,'BIR' from dual union all
select 17186,'SOTON' from dual union all
select 17188,'ICST' from dual union all
select 17188,'MSSL' from dual union all
select 17189,'GRI' from dual union all
select 17190,'NRCC' from dual union all
select 17196,'CNES' from dual union all
select 17197,'MPE' from dual union all
select 17197,'PRL' from dual union all
select 17198,'GSFC' from dual union all
select 17201,'NRL' from dual union all
select 17202,'AFCRL' from dual union all
select 17203,'CNET' from dual union all
select 17203,'SUPA' from dual union all
select 17206,'GMS' from dual union all
select 17207,'MPE' from dual union all
select 17207,'PRL' from dual union all
select 17209,'AFSWC' from dual union all
select 17210,'AFSWC' from dual union all
select 17222,'MPE' from dual union all
select 17222,'PRL' from dual union all
select 17223,'MPE' from dual union all
select 17223,'PRL' from dual union all
select 17224,'GRI' from dual union all
select 17227,'CNET' from dual union all
select 17227,'SUPA' from dual union all
select 17228,'NRCC' from dual union all
select 17229,'GSFC' from dual union all
select 1723,'WFF' from dual union all
select 17230,'GSFC' from dual union all
select 17231,'MPE' from dual union all
select 17232,'CALG' from dual union all
select 17233,'NRL' from dual union all
select 17235,'CNES' from dual union all
select 17237,'PITT' from dual union all
select 17237,'USCLA' from dual union all
select 1724,'SUI' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 17243,'CALG' from dual union all
select 17246,'CALG' from dual union all
select 17248,'CALG' from dual union all
select 17249,'UMN' from dual union all
select 17250,'ISAS' from dual union all
select 17250,'PRL' from dual union all
select 17252,'YORK' from dual union all
select 17253,'UMN' from dual union all
select 17253,'UNH' from dual union all
select 17254,'MPAE' from dual union all
select 17255,'UCO' from dual union all
select 17256,'MPAE' from dual union all
select 17257,'MPAE' from dual union all
select 17259,'ISAS' from dual union all
select 17259,'PRL' from dual union all
select 17260,'NMSU' from dual union all
select 17261,'TRW' from dual union all
select 17262,'UMN' from dual union all
select 17262,'UNH' from dual union all
select 17268,'LARC' from dual union all
select 1727,'SUI' from dual union all
select 17270,'NRL' from dual union all
select 17273,'NRL' from dual union all
select 17274,'YORK' from dual union all
select 17275,'AFSWC' from dual union all
select 17276,'AFSWC' from dual union all
select 17278,'UNH' from dual union all
select 17279,'AFCRL' from dual union all
select 17281,'AFCRL' from dual union all
select 17282,'AFCRL' from dual union all
select 17283,'AFCRL' from dual union all
select 17285,'UNH' from dual union all
select 17286,'PRIN' from dual union all
select 17287,'DRTE' from dual union all
select 17288,'SAND' from dual union all
select 17289,'USAF' from dual union all
select 17290,'LRL' from dual union all
select 17290,'SAND' from dual union all
select 17296,'AFCRL' from dual union all
select 17297,'GMS' from dual union all
select 17298,'AFCRL' from dual union all
select 17299,'CNES' from dual union all
select 17301,'ONERA' from dual union all
select 17303,'LRL' from dual union all
select 17305,'KPNO' from dual union all
select 17306,'GSFC' from dual union all
select 17307,'AFCRL' from dual union all
select 17308,'SAND' from dual union all
select 17310,'AFCRL' from dual union all
select 17311,'SAND' from dual union all
select 17312,'SAND' from dual union all
select 17315,'LRL' from dual union all
select 17316,'GSFC' from dual union all
select 17317,'LUND' from dual union all
select 17317,'MPIK' from dual union all
select 17318,'GMS' from dual union all
select 17322,'GMS' from dual union all
select 17323,'MPIK' from dual union all
select 17323,'UTR' from dual union all
select 17327,'AFCRL' from dual union all
select 17328,'SAND' from dual union all
select 17329,'AFCRL' from dual union all
select 17330,'SAND' from dual union all
select 17331,'SAND' from dual union all
select 17332,'SAND' from dual union all
select 17333,'SAND' from dual union all
select 17334,'UIA' from dual union all
select 17336,'UWI' from dual union all
select 17337,'SHEF' from dual union all
select 17340,'UCL' from dual union all
select 17341,'UCL' from dual union all
select 17343,'SHEF' from dual union all
select 17344,'LASL' from dual union all
select 17344,'UAF' from dual union all
select 17345,'GSFC' from dual union all
select 17350,'LUND' from dual union all
select 17350,'MPIK' from dual union all
select 17351,'ASE' from dual union all
select 17352,'NRL' from dual union all
select 17353,'LUND' from dual union all
select 17353,'MPIK' from dual union all
select 17354,'GSFC' from dual union all
select 17355,'AFCRL' from dual union all
select 17356,'GSFC' from dual union all
select 17357,'DUD' from dual union all
select 17359,'MPIK' from dual union all
select 1736,'SUI' from dual union all
select 17360,'MSC' from dual union all
select 17362,'LASL' from dual union all
select 17362,'UAF' from dual union all
select 17363,'LUX' from dual union all
select 17364,'MPIK' from dual union all
select 1737,'NRL' from dual union all
select 17370,'WFF' from dual union all
select 17381,'LUX' from dual union all
select 17385,'IEM' from dual union all
select 17385,'NIRFI' from dual union all
select 17386,'AFCRL' from dual union all
select 17389,'AFCRL' from dual union all
select 17390,'AFCRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 17391,'CNES' from dual union all
select 17392,'AFCRL' from dual union all
select 17393,'AFCRL' from dual union all
select 17394,'AFCRL' from dual union all
select 17395,'AFCRL' from dual union all
select 17396,'UCO' from dual union all
select 17397,'GSFC' from dual union all
select 17398,'UCW' from dual union all
select 17399,'UMI' from dual union all
select 17400,'UCO' from dual union all
select 17401,'UIUC' from dual union all
select 17402,'GSFC' from dual union all
select 17403,'UIUC' from dual union all
select 17404,'UIUC' from dual union all
select 17405,'UMI' from dual union all
select 17406,'UIUC' from dual union all
select 17408,'GSFC' from dual union all
select 17409,'CEA' from dual union all
select 17409,'LAS' from dual union all
select 17410,'IEM' from dual union all
select 17410,'NIRFI' from dual union all
select 17413,'CAL' from dual union all
select 17414,'MIT' from dual union all
select 17417,'RTG' from dual union all
select 17423,'SU' from dual union all
select 17424,'SU' from dual union all
select 17425,'JHU' from dual union all
select 17426,'SU' from dual union all
select 17427,'SU' from dual union all
select 17428,'ARC' from dual union all
select 17429,'RSRS' from dual union all
select 17430,'SASK' from dual union all
select 17431,'UCW' from dual union all
select 17432,'AFCRL' from dual union all
select 17433,'RICE' from dual union all
select 17434,'ARMT' from dual union all
select 17435,'AFCRL' from dual union all
select 17436,'GSFC' from dual union all
select 17436,'UMI' from dual union all
select 17437,'UMI' from dual union all
select 17438,'GSFC' from dual union all
select 17438,'UMI' from dual union all
select 17440,'UMI' from dual union all
select 17441,'AFCRL' from dual union all
select 17442,'USAF' from dual union all
select 17443,'AFCRL' from dual union all
select 17444,'DUD' from dual union all
select 17447,'AFCRL' from dual union all
select 17448,'AFCRL' from dual union all
select 17452,'AFCRL' from dual union all
select 17453,'ARMT' from dual union all
select 17454,'AFCRL' from dual union all
select 17455,'AFCRL' from dual union all
select 17456,'DUD' from dual union all
select 17459,'GSFC' from dual union all
select 17460,'GSFC' from dual union all
select 17462,'GSFC' from dual union all
select 17463,'UMD' from dual union all
select 17465,'LRL' from dual union all
select 17466,'AFCRL' from dual union all
select 17467,'AFCRL' from dual union all
select 17474,'PRL' from dual union all
select 17475,'PRL' from dual union all
select 17476,'PRL' from dual union all
select 17477,'BIR' from dual union all
select 17477,'MSSL' from dual union all
select 17479,'USAF' from dual union all
select 17480,'AFCRL' from dual union all
select 17481,'AFCRL' from dual union all
select 17482,'NCAR' from dual union all
select 17484,'USAF' from dual union all
select 17485,'USAF' from dual union all
select 17489,'ISAS' from dual union all
select 17490,'ISAS' from dual union all
select 17491,'GSFC' from dual union all
select 17492,'GSFC' from dual union all
select 17493,'STA' from dual union all
select 17495,'STA' from dual union all
select 17497,'GSFC' from dual union all
select 17497,'UAF' from dual union all
select 17498,'NDRE' from dual union all
select 17498,'RSRS' from dual union all
select 17499,'UWI' from dual union all
select 17500,'GSFC' from dual union all
select 17501,'NRL' from dual union all
select 17502,'GSFC' from dual union all
select 17503,'HCO' from dual union all
select 17509,'GSFC' from dual union all
select 17511,'GSFC' from dual union all
select 17512,'UCO' from dual union all
select 17513,'NDRE' from dual union all
select 17513,'RSRS' from dual union all
select 17518,'ROE' from dual union all
select 17521,'CALG' from dual union all
select 17522,'BOL' from dual union all
select 17522,'ESTEC' from dual union all
select 17523,'CALG' from dual union all
select 17525,'RAE' from dual union all
select 17525,'WRE' from dual union all
select 17527,'AN' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 17528,'NRL' from dual union all
select 17529,'CNES' from dual union all
select 17530,'GSFC' from dual union all
select 17531,'GSFC' from dual union all
select 17532,'CNRS' from dual union all
select 17532,'UCL' from dual union all
select 17533,'GSFC' from dual union all
select 17534,'UMI' from dual union all
select 17535,'GSFC' from dual union all
select 17538,'LPARL' from dual union all
select 17539,'USAF' from dual union all
select 17545,'SU' from dual union all
select 17547,'USAF' from dual union all
select 17550,'USAF' from dual union all
select 17553,'KPNO' from dual union all
select 17555,'LRL' from dual union all
select 17557,'AN' from dual union all
select 17558,'AFSWC' from dual union all
select 17560,'AFSWC' from dual union all
select 17561,'SAND' from dual union all
select 17562,'SASK' from dual union all
select 17563,'SASK' from dual union all
select 17564,'CNRS' from dual union all
select 17564,'UCL' from dual union all
select 17565,'CNRS' from dual union all
select 17565,'ESRO' from dual union all
select 17566,'ISRO' from dual union all
select 17566,'PRL' from dual union all
select 17567,'LRL' from dual union all
select 17571,'MPAE' from dual union all
select 17572,'PRL' from dual union all
select 17573,'LRL' from dual union all
select 17574,'GSFC' from dual union all
select 17575,'MPAE' from dual union all
select 17577,'AFCRL' from dual union all
select 17578,'BIR' from dual union all
select 17578,'SOTON' from dual union all
select 17579,'MPAE' from dual union all
select 17582,'MPAE' from dual union all
select 17586,'PRIN' from dual union all
select 17587,'MPAE' from dual union all
select 17589,'MPAE' from dual union all
select 1759,'SUI' from dual union all
select 17591,'BRSW' from dual union all
select 17593,'SASK' from dual union all
select 17594,'AFCRL' from dual union all
select 17595,'GSFC' from dual union all
select 17596,'GSFC' from dual union all
select 17597,'UMI' from dual union all
select 17599,'GSFC' from dual union all
select 1760,'SUI' from dual union all
select 17600,'GSFC' from dual union all
select 17601,'MPAE' from dual union all
select 17604,'AFCRL' from dual union all
select 17605,'UCO' from dual union all
select 17606,'UKY' from dual union all
select 17606,'WFF' from dual union all
select 17607,'GSFC' from dual union all
select 17608,'GSFC' from dual union all
select 17609,'MPAE' from dual union all
select 17610,'LUX' from dual union all
select 17610,'UTR' from dual union all
select 17612,'BIR' from dual union all
select 17612,'BREI' from dual union all
select 17612,'LIEGE' from dual union all
select 17614,'MPAE' from dual union all
select 17616,'MPAE' from dual union all
select 17617,'WRE' from dual union all
select 17618,'MPAE' from dual union all
select 17619,'RSRS' from dual union all
select 17620,'BONN' from dual union all
select 17621,'ROE' from dual union all
select 17622,'BRSW' from dual union all
select 17625,'MPE' from dual union all
select 17625,'NDRE' from dual union all
select 17626,'BONN' from dual union all
select 17627,'DUD' from dual union all
select 17628,'BONN' from dual union all
select 17629,'BONN' from dual union all
select 17630,'BRSW' from dual union all
select 17633,'CULH' from dual union all
select 17633,'QUB' from dual union all
select 17635,'RSRS' from dual union all
select 17638,'BIR' from dual union all
select 17638,'MSSL' from dual union all
select 17638,'RSRS' from dual union all
select 17639,'ASE' from dual union all
select 17640,'UCW' from dual union all
select 17641,'AFCRL' from dual union all
select 17642,'AFCRL' from dual union all
select 17644,'PITT' from dual union all
select 17646,'AFCRL' from dual union all
select 17647,'AFCRL' from dual union all
select 17648,'WRE' from dual union all
select 17649,'GSFC' from dual union all
select 17650,'GSFC' from dual union all
select 17652,'GSFC' from dual union all
select 17653,'GSFC' from dual union all
select 17654,'GSFC' from dual union all
select 17655,'GSFC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 17657,'NRCC' from dual union all
select 17658,'DUD' from dual union all
select 17660,'LAS' from dual union all
select 17661,'CALG' from dual union all
select 17663,'CORN' from dual union all
select 17665,'CEA' from dual union all
select 17665,'LAS' from dual union all
select 17666,'UMI' from dual union all
select 17668,'UMI' from dual union all
select 17670,'AFAL' from dual union all
select 17670,'OSU' from dual union all
select 17671,'CBMET' from dual union all
select 17671,'WRE' from dual union all
select 17672,'CBMET' from dual union all
select 17672,'WRE' from dual union all
select 17673,'WRE' from dual union all
select 17674,'AFCRL' from dual union all
select 17675,'AFCRL' from dual union all
select 17676,'AFCRL' from dual union all
select 17678,'AFCRL' from dual union all
select 17679,'AFCRL' from dual union all
select 17680,'AFCRL' from dual union all
select 17681,'AFCRL' from dual union all
select 17682,'AFCRL' from dual union all
select 17683,'AFCRL' from dual union all
select 17684,'AFCRL' from dual union all
select 17685,'AFCRL' from dual union all
select 17686,'AFCRL' from dual union all
select 17687,'BRL' from dual union all
select 17689,'BRL' from dual union all
select 17690,'BRL' from dual union all
select 17691,'BRL' from dual union all
select 17692,'BRL' from dual union all
select 17694,'USAF' from dual union all
select 17695,'BAJ' from dual union all
select 17696,'SAND' from dual union all
select 17697,'AN' from dual union all
select 17708,'AEC' from dual union all
select 17708,'SAND' from dual union all
select 17709,'AEC' from dual union all
select 17709,'SAND' from dual union all
select 17710,'LASL' from dual union all
select 17784,'GSFC' from dual union all
select 17785,'GSFC' from dual union all
select 17786,'AFCRL' from dual union all
select 17788,'NRCC' from dual union all
select 17789,'AFCRL' from dual union all
select 17790,'SASK' from dual union all
select 17791,'GSFC' from dual union all
select 17792,'AFCRL' from dual union all
select 17793,'AFCRL' from dual union all
select 17794,'NRL' from dual union all
select 17796,'UAF' from dual union all
select 17797,'SASK' from dual union all
select 17799,'TUC' from dual union all
select 17802,'LARC' from dual union all
select 17803,'USAF' from dual union all
select 17804,'USAF' from dual union all
select 17805,'USAF' from dual union all
select 17806,'USAF' from dual union all
select 17807,'USAF' from dual union all
select 17808,'USAF' from dual union all
select 17809,'USAF' from dual union all
select 1781,'AFCRL' from dual union all
select 17810,'USAF' from dual union all
select 17811,'USAF' from dual union all
select 17812,'USAF' from dual union all
select 17813,'MSFC' from dual union all
select 17816,'CONAE' from dual union all
select 17818,'SAND' from dual union all
select 17819,'SAND' from dual union all
select 17820,'SAND' from dual union all
select 17821,'MSFC' from dual union all
select 17822,'MSFC' from dual union all
select 17823,'USAF' from dual union all
select 17824,'SAND' from dual union all
select 17825,'CONAE' from dual union all
select 17826,'USAF' from dual union all
select 17827,'USAF' from dual union all
select 17828,'USAF' from dual union all
select 17829,'USAF' from dual union all
select 17830,'USAF' from dual union all
select 17831,'USAF' from dual union all
select 17832,'USAF' from dual union all
select 17833,'USAF' from dual union all
select 17834,'USAF' from dual union all
select 17835,'USAF' from dual union all
select 17836,'USAF' from dual union all
select 17837,'USAF' from dual union all
select 17840,'CONAE' from dual union all
select 17843,'CONAE' from dual union all
select 17844,'USAF' from dual union all
select 17845,'USAF' from dual union all
select 17846,'USAF' from dual union all
select 17847,'USAF' from dual union all
select 17848,'USAF' from dual union all
select 17849,'USAF' from dual union all
select 17850,'USAF' from dual union all
select 17851,'USAF' from dual union all
select 17852,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 17853,'USAF' from dual union all
select 17854,'USAF' from dual union all
select 17855,'USAF' from dual union all
select 17856,'USAF' from dual union all
select 17857,'USAF' from dual union all
select 17858,'USAF' from dual union all
select 17859,'USAF' from dual union all
select 17860,'USAF' from dual union all
select 17861,'USAF' from dual union all
select 17862,'USAF' from dual union all
select 17867,'USAF' from dual union all
select 17870,'CONAE' from dual union all
select 17872,'NWC' from dual union all
select 17873,'NWC' from dual union all
select 17874,'USAF' from dual union all
select 17875,'USAF' from dual union all
select 17876,'USAF' from dual union all
select 17877,'USAF' from dual union all
select 17878,'USAF' from dual union all
select 17879,'USAF' from dual union all
select 17880,'USAF' from dual union all
select 17882,'CONAE' from dual union all
select 17889,'USAF' from dual union all
select 17890,'CONAE' from dual union all
select 17891,'USAF' from dual union all
select 17892,'USAF' from dual union all
select 17893,'USAF' from dual union all
select 17894,'USAF' from dual union all
select 17895,'USAF' from dual union all
select 17896,'USAF' from dual union all
select 17897,'USAF' from dual union all
select 17898,'USAF' from dual union all
select 17900,'USAF' from dual union all
select 17905,'JMA' from dual union all
select 17906,'JMA' from dual union all
select 17908,'NOTS' from dual union all
select 17910,'PSU' from dual union all
select 17911,'USAF' from dual union all
select 17912,'USAF' from dual union all
select 17913,'USAF' from dual union all
select 17914,'USAF' from dual union all
select 17915,'USAF' from dual union all
select 17916,'USAF' from dual union all
select 17917,'USAF' from dual union all
select 17918,'USAF' from dual union all
select 17919,'USAF' from dual union all
select 17920,'USAF' from dual union all
select 17921,'USAF' from dual union all
select 17922,'USAF' from dual union all
select 17923,'USAF' from dual union all
select 17924,'USAF' from dual union all
select 17925,'USAF' from dual union all
select 17926,'USAF' from dual union all
select 17927,'USAF' from dual union all
select 17928,'USAF' from dual union all
select 17929,'USAF' from dual union all
select 17930,'USAF' from dual union all
select 17931,'USAF' from dual union all
select 17932,'USAF' from dual union all
select 17933,'USAF' from dual union all
select 17934,'USAF' from dual union all
select 17935,'USAF' from dual union all
select 17936,'USAF' from dual union all
select 17937,'USAF' from dual union all
select 17938,'USAF' from dual union all
select 17939,'USAF' from dual union all
select 17940,'USAF' from dual union all
select 17941,'USAF' from dual union all
select 17942,'USAF' from dual union all
select 17943,'USAF' from dual union all
select 17944,'USAF' from dual union all
select 17945,'USAF' from dual union all
select 17946,'USAF' from dual union all
select 17947,'USAF' from dual union all
select 17948,'USAF' from dual union all
select 17949,'USAF' from dual union all
select 17950,'PIHM' from dual union all
select 17951,'AN' from dual union all
select 17952,'WSMR' from dual union all
select 17953,'MSFC' from dual union all
select 17955,'PIHM' from dual union all
select 17957,'USAF' from dual union all
select 17958,'USAF' from dual union all
select 17959,'USAF' from dual union all
select 17960,'USAF' from dual union all
select 17961,'USAF' from dual union all
select 17962,'USAF' from dual union all
select 17963,'USAF' from dual union all
select 17964,'USAF' from dual union all
select 17965,'USAF' from dual union all
select 17966,'PIHM' from dual union all
select 17967,'USAF' from dual union all
select 17968,'USAF' from dual union all
select 17969,'USAF' from dual union all
select 17972,'MSFC' from dual union all
select 17973,'MSFC' from dual union all
select 17978,'USAF' from dual union all
select 17979,'USAF' from dual union all
select 17980,'USAF' from dual union all
select 17981,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 17982,'USAF' from dual union all
select 17983,'USAF' from dual union all
select 17984,'USAF' from dual union all
select 17985,'CONAE' from dual union all
select 17986,'INPE' from dual union all
select 17987,'USAF' from dual union all
select 17989,'USAF' from dual union all
select 17990,'USAF' from dual union all
select 17991,'USAF' from dual union all
select 17992,'USAF' from dual union all
select 17994,'PSU' from dual union all
select 17995,'PSU' from dual union all
select 17996,'USAF' from dual union all
select 17997,'USAF' from dual union all
select 17998,'USAF' from dual union all
select 18000,'USAF' from dual union all
select 18001,'USAF' from dual union all
select 18002,'MSFC' from dual union all
select 18003,'MSFC' from dual union all
select 18008,'WSMR' from dual union all
select 18009,'WSMR' from dual union all
select 18011,'WSMR' from dual union all
select 18012,'WSMR' from dual union all
select 18013,'WSMR' from dual union all
select 18014,'WSMR' from dual union all
select 18015,'USN' from dual union all
select 18016,'USAF' from dual union all
select 18017,'USAF' from dual union all
select 18018,'USAF' from dual union all
select 18019,'PSU' from dual union all
select 18020,'CONAE' from dual union all
select 18021,'AN' from dual union all
select 18022,'AN' from dual union all
select 18023,'LARC' from dual union all
select 18024,'CONAE' from dual union all
select 18025,'CONAE' from dual union all
select 18026,'CONAE' from dual union all
select 18027,'CONAE' from dual union all
select 18028,'CONAE' from dual union all
select 18029,'CONAE' from dual union all
select 18030,'INPE' from dual union all
select 18031,'MSFC' from dual union all
select 18032,'INPE' from dual union all
select 18033,'INPE' from dual union all
select 18034,'INPE' from dual union all
select 18035,'INPE' from dual union all
select 18036,'INPE' from dual union all
select 18037,'INPE' from dual union all
select 18038,'INPE' from dual union all
select 18039,'INPE' from dual union all
select 18040,'INPE' from dual union all
select 18042,'INPE' from dual union all
select 18043,'WFF' from dual union all
select 18044,'WFF' from dual union all
select 18045,'WFF' from dual union all
select 18046,'WFF' from dual union all
select 18047,'WFF' from dual union all
select 18048,'WFF' from dual union all
select 18049,'WFF' from dual union all
select 18050,'WFF' from dual union all
select 18052,'WFF' from dual union all
select 18053,'WFF' from dual union all
select 18054,'WFF' from dual union all
select 18055,'WFF' from dual union all
select 18056,'WFF' from dual union all
select 18057,'WFF' from dual union all
select 18058,'WFF' from dual union all
select 18059,'WFF' from dual union all
select 18060,'WFF' from dual union all
select 18061,'WFF' from dual union all
select 18062,'USAF' from dual union all
select 18063,'WFF' from dual union all
select 18064,'WFF' from dual union all
select 18065,'WFF' from dual union all
select 18066,'WFF' from dual union all
select 18067,'WFF' from dual union all
select 18068,'WFF' from dual union all
select 18069,'WFF' from dual union all
select 18070,'WFF' from dual union all
select 18071,'WFF' from dual union all
select 18072,'WFF' from dual union all
select 18073,'USAF' from dual union all
select 18074,'WFF' from dual union all
select 18075,'WFF' from dual union all
select 18076,'WFF' from dual union all
select 18077,'WFF' from dual union all
select 18078,'WFF' from dual union all
select 18079,'WFF' from dual union all
select 18080,'WFF' from dual union all
select 18081,'WFF' from dual union all
select 18082,'WFF' from dual union all
select 18083,'WFF' from dual union all
select 18084,'USAF' from dual union all
select 18085,'WFF' from dual union all
select 18086,'WFF' from dual union all
select 18087,'WFF' from dual union all
select 18088,'WFF' from dual union all
select 18089,'WFF' from dual union all
select 18090,'WFF' from dual union all
select 18091,'WFF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 18092,'WFF' from dual union all
select 18093,'WFF' from dual union all
select 18094,'WFF' from dual union all
select 18095,'USAF' from dual union all
select 18096,'USAF' from dual union all
select 18097,'USAF' from dual union all
select 18098,'USAF' from dual union all
select 18099,'USAF' from dual union all
select 18100,'USAF' from dual union all
select 18101,'USAF' from dual union all
select 18102,'USAF' from dual union all
select 18103,'ISRO' from dual union all
select 18107,'CONAE' from dual union all
select 18110,'USAF' from dual union all
select 18111,'USAF' from dual union all
select 18112,'USAF' from dual union all
select 18113,'USAF' from dual union all
select 18114,'USAF' from dual union all
select 18115,'USAF' from dual union all
select 18116,'USAF' from dual union all
select 18117,'USAF' from dual union all
select 18118,'USAF' from dual union all
select 18119,'USAF' from dual union all
select 18120,'USAF' from dual union all
select 18121,'USAF' from dual union all
select 18122,'USAF' from dual union all
select 18123,'USAF' from dual union all
select 18124,'USAF' from dual union all
select 18125,'USAF' from dual union all
select 18127,'MNF' from dual union all
select 18128,'CONAE' from dual union all
select 18129,'CONAE' from dual union all
select 18130,'CONAE' from dual union all
select 18131,'USAF' from dual union all
select 18132,'USAF' from dual union all
select 18133,'USAF' from dual union all
select 18134,'USAF' from dual union all
select 18135,'USAF' from dual union all
select 18136,'USAF' from dual union all
select 18137,'USAF' from dual union all
select 18138,'USAF' from dual union all
select 18139,'USAF' from dual union all
select 18140,'CONAE' from dual union all
select 18141,'MSFC' from dual union all
select 18143,'GSFC' from dual union all
select 18144,'CONAE' from dual union all
select 18145,'USAF' from dual union all
select 18146,'USAF' from dual union all
select 18147,'CONAE' from dual union all
select 18149,'USAF' from dual union all
select 18150,'USAF' from dual union all
select 18151,'USAF' from dual union all
select 18152,'USAF' from dual union all
select 18153,'USAF' from dual union all
select 18154,'USAF' from dual union all
select 18155,'USAF' from dual union all
select 18157,'MSFC' from dual union all
select 18165,'PSU' from dual union all
select 18167,'AFCRL' from dual union all
select 18168,'UCW' from dual union all
select 18170,'AN' from dual union all
select 18171,'USAF' from dual union all
select 18172,'USAF' from dual union all
select 18178,'AFCRL' from dual union all
select 18180,'AFCRL' from dual union all
select 18184,'PSU' from dual union all
select 18185,'GSFC' from dual union all
select 18185,'ISRO' from dual union all
select 18186,'GSFC' from dual union all
select 18186,'ISRO' from dual union all
select 18189,'UCW' from dual union all
select 18190,'AFCRL' from dual union all
select 18191,'MSFC' from dual union all
select 18192,'ESSA' from dual union all
select 18193,'MSFC' from dual union all
select 18195,'MSFC' from dual union all
select 18196,'MSFC' from dual union all
select 18197,'MSFC' from dual union all
select 18198,'PSU' from dual union all
select 18203,'ESSA' from dual union all
select 18205,'MSSL' from dual union all
select 18208,'MSSL' from dual union all
select 18215,'MSSL' from dual union all
select 18217,'SU' from dual union all
select 18222,'GSFC' from dual union all
select 18223,'GSFC' from dual union all
select 18224,'UCW' from dual union all
select 18226,'DSRI' from dual union all
select 18226,'MSSL' from dual union all
select 18237,'DSRI' from dual union all
select 18237,'MSSL' from dual union all
select 18253,'AN' from dual union all
select 18257,'UIO' from dual union all
select 18259,'ESSA' from dual union all
select 18260,'ESSA' from dual union all
select 18261,'UCW' from dual union all
select 18262,'AN' from dual union all
select 18263,'UCW' from dual union all
select 18265,'MSSL' from dual union all
select 18266,'AN' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 18267,'MSSL' from dual union all
select 18382,'JMA' from dual union all
select 18385,'ISAS' from dual union all
select 18398,'DASA' from dual union all
select 18399,'DASA' from dual union all
select 18418,'INTA' from dual union all
select 18421,'ISRO' from dual union all
select 18440,'NRL' from dual union all
select 18442,'ISAS' from dual union all
select 18448,'STA' from dual union all
select 18456,'UKMO' from dual union all
select 18457,'JMA' from dual union all
select 18458,'RAE' from dual union all
select 18461,'PIHM' from dual union all
select 18462,'PIHM' from dual union all
select 18464,'PIHM' from dual union all
select 18480,'AFCRL' from dual union all
select 18481,'PSU' from dual union all
select 18485,'UIO' from dual union all
select 18501,'INTA' from dual union all
select 18503,'KPNO' from dual union all
select 18505,'UKMO' from dual union all
select 18509,'LARC' from dual union all
select 18510,'LARC' from dual union all
select 18511,'AFCRL' from dual union all
select 18512,'AFCRL' from dual union all
select 18513,'AFCRL' from dual union all
select 18514,'AFCRL' from dual union all
select 18518,'USAF' from dual union all
select 18524,'ADEL' from dual union all
select 18532,'GSFC' from dual union all
select 18533,'UKMO' from dual union all
select 18537,'USAF' from dual union all
select 18538,'GSFC' from dual union all
select 18541,'SUPA' from dual union all
select 18552,'LARC' from dual union all
select 18553,'LARC' from dual union all
select 18556,'SASK' from dual union all
select 18557,'BREI' from dual union all
select 18557,'KGO' from dual union all
select 18557,'TLS' from dual union all
select 18563,'WSMR' from dual union all
select 18565,'WSMR' from dual union all
select 18566,'WSMR' from dual union all
select 18576,'USAF' from dual union all
select 18579,'LARC' from dual union all
select 18592,'LARC' from dual union all
select 18609,'KYO' from dual union all
select 18612,'MISU' from dual union all
select 18612,'UCL' from dual union all
select 18613,'STA' from dual union all
select 18614,'STA' from dual union all
select 18616,'SUPA' from dual union all
select 18617,'DSRI' from dual union all
select 18617,'UCL' from dual union all
select 18618,'DSRI' from dual union all
select 18618,'UCL' from dual union all
select 18619,'DSRI' from dual union all
select 18619,'UCL' from dual union all
select 18620,'DSRI' from dual union all
select 18620,'UCL' from dual union all
select 18621,'DSRI' from dual union all
select 18621,'UCL' from dual union all
select 18622,'UKMO' from dual union all
select 18623,'UKMO' from dual union all
select 18624,'CNES' from dual union all
select 18624,'CSG' from dual union all
select 18625,'UKMO' from dual union all
select 18626,'UKMO' from dual union all
select 18627,'UKMO' from dual union all
select 18628,'UKMO' from dual union all
select 18629,'UKMO' from dual union all
select 18630,'UKMO' from dual union all
select 18631,'UKMO' from dual union all
select 18632,'UKMO' from dual union all
select 18636,'CNES' from dual union all
select 18636,'CSG' from dual union all
select 18637,'ISAS' from dual union all
select 18638,'STA' from dual union all
select 18639,'UKMO' from dual union all
select 18640,'ONERA' from dual union all
select 18644,'WRE' from dual union all
select 18645,'INTA' from dual union all
select 18646,'AFCRL' from dual union all
select 18647,'TSAO' from dual union all
select 18648,'TSAO' from dual union all
select 18649,'INTA' from dual union all
select 18649,'NASA' from dual union all
select 18650,'INTA' from dual union all
select 18650,'NASA' from dual union all
select 18651,'INTA' from dual union all
select 18651,'NASA' from dual union all
select 18652,'ASL' from dual union all
select 18653,'IIAE' from dual union all
select 18654,'IIAE' from dual union all
select 18655,'IIAE' from dual union all
select 18657,'SECT' from dual union all
select 18667,'AFCRL' from dual union all
select 18668,'UCO' from dual union all
select 18669,'GSFC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 18670,'AFCRL' from dual union all
select 18672,'RSRS' from dual union all
select 18673,'USAF' from dual union all
select 18674,'USAF' from dual union all
select 18675,'UMN' from dual union all
select 18676,'GSFC' from dual union all
select 18678,'AN' from dual union all
select 18678,'CNET' from dual union all
select 18679,'STA' from dual union all
select 18680,'GSFC' from dual union all
select 18681,'UMN' from dual union all
select 18682,'GCA' from dual union all
select 18682,'UIUC' from dual union all
select 18683,'UCO' from dual union all
select 18684,'GSFC' from dual union all
select 18685,'AN' from dual union all
select 18685,'CNET' from dual union all
select 18686,'AN' from dual union all
select 18686,'CNET' from dual union all
select 18687,'AN' from dual union all
select 18687,'CNET' from dual union all
select 18689,'STA' from dual union all
select 18690,'STA' from dual union all
select 18691,'JHU' from dual union all
select 18692,'UTD' from dual union all
select 18693,'STA' from dual union all
select 18694,'RSRS' from dual union all
select 18695,'UTD' from dual union all
select 18696,'JHU' from dual union all
select 18697,'CRR' from dual union all
select 18698,'NRL' from dual union all
select 18700,'ISAS' from dual union all
select 18702,'OSAKA' from dual union all
select 18703,'GCA' from dual union all
select 18704,'GCA' from dual union all
select 18705,'GCA' from dual union all
select 18706,'GCA' from dual union all
select 18707,'GCA' from dual union all
select 18708,'GCA' from dual union all
select 18709,'GCA' from dual union all
select 18711,'GCA' from dual union all
select 18712,'SHEF' from dual union all
select 18713,'SHEF' from dual union all
select 18714,'UTD' from dual union all
select 18716,'YORK' from dual union all
select 18719,'CNET' from dual union all
select 18720,'UTD' from dual union all
select 18722,'DSRI' from dual union all
select 18722,'UCL' from dual union all
select 18723,'KIEL' from dual union all
select 18723,'UTR' from dual union all
select 18724,'KIEL' from dual union all
select 18724,'UTR' from dual union all
select 18725,'PRL' from dual union all
select 18726,'USAF' from dual union all
select 18727,'RICE' from dual union all
select 18728,'CFTH' from dual union all
select 18729,'AFSWC' from dual union all
select 18731,'GSFC' from dual union all
select 18734,'GSFC' from dual union all
select 18735,'UAF' from dual union all
select 18736,'UAF' from dual union all
select 18737,'AN' from dual union all
select 18738,'CNES' from dual union all
select 18739,'CRC' from dual union all
select 18741,'CAL' from dual union all
select 18742,'GSFC' from dual union all
select 18744,'AFGL' from dual union all
select 18744,'USU' from dual union all
select 18745,'GSFC' from dual union all
select 18747,'CNRS' from dual union all
select 18748,'GSFC' from dual union all
select 18749,'CNRS' from dual union all
select 18750,'CALG' from dual union all
select 18751,'CNRS' from dual union all
select 18752,'CNRS' from dual union all
select 18753,'NRCC' from dual union all
select 18755,'UCO' from dual union all
select 18756,'LASL' from dual union all
select 18756,'UAF' from dual union all
select 18758,'AN' from dual union all
select 18760,'CNRS' from dual union all
select 18761,'AN' from dual union all
select 18762,'CNRS' from dual union all
select 18763,'NRL' from dual union all
select 18764,'GSFC' from dual union all
select 18766,'AN' from dual union all
select 18768,'UAF' from dual union all
select 18769,'MPE' from dual union all
select 18770,'BIR' from dual union all
select 18770,'BREI' from dual union all
select 18770,'LIEGE' from dual union all
select 18771,'LIEGE' from dual union all
select 18772,'BOCH' from dual union all
select 18773,'SOTON' from dual union all
select 18774,'ORIAS' from dual union all
select 18775,'UAF' from dual union all
select 18777,'MPE' from dual union all
select 18778,'BIR' from dual union all
select 18780,'UAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 18781,'SOTON' from dual union all
select 18785,'LASL' from dual union all
select 18785,'UAF' from dual union all
select 18788,'GMS' from dual union all
select 18790,'AFGL' from dual union all
select 18791,'IEM' from dual union all
select 18799,'AN' from dual union all
select 18809,'SAND' from dual union all
select 18810,'GSFC' from dual union all
select 18810,'INTA' from dual union all
select 18811,'NRL' from dual union all
select 18812,'GSFC' from dual union all
select 18814,'GSFC' from dual union all
select 18814,'INTA' from dual union all
select 18815,'SASK' from dual union all
select 18816,'GSFC' from dual union all
select 18817,'SASK' from dual union all
select 18818,'GMS' from dual union all
select 18819,'SHEF' from dual union all
select 18821,'AFGL' from dual union all
select 18824,'CNES' from dual union all
select 18824,'SUPA' from dual union all
select 18825,'DRTE' from dual union all
select 18826,'LUX' from dual union all
select 18827,'AN' from dual union all
select 18830,'AN' from dual union all
select 18831,'BIR' from dual union all
select 18831,'MSSL' from dual union all
select 18833,'AFGL' from dual union all
select 18834,'AFGL' from dual union all
select 18835,'ASE' from dual union all
select 18838,'MPE' from dual union all
select 18838,'NDRE' from dual union all
select 18841,'CNES' from dual union all
select 18842,'GSFC' from dual union all
select 18843,'ESRO' from dual union all
select 18844,'GSFC' from dual union all
select 18846,'GSFC' from dual union all
select 18847,'GSFC' from dual union all
select 18851,'BREI' from dual union all
select 18851,'KGO' from dual union all
select 18851,'TLS' from dual union all
select 18853,'IIAE' from dual union all
select 18854,'NRL' from dual union all
select 18855,'UIUC' from dual union all
select 18856,'CULH' from dual union all
select 18856,'MO' from dual union all
select 18857,'TRW' from dual union all
select 18858,'NRL' from dual union all
select 18859,'CNRS' from dual union all
select 18859,'MPE' from dual union all
select 18863,'AFCRL' from dual union all
select 18864,'AFCRL' from dual union all
select 18865,'IEM' from dual union all
select 18866,'UNK' from dual union all
select 18867,'UNK' from dual union all
select 18868,'IEM' from dual union all
select 18869,'UNK' from dual union all
select 18870,'GMS' from dual union all
select 18871,'UNK' from dual union all
select 18874,'CULH' from dual union all
select 18877,'CNES' from dual union all
select 18879,'UNK' from dual union all
select 18880,'321SMW' from dual union all
select 18884,'CNES' from dual union all
select 18885,'ISAS' from dual union all
select 18885,'PRL' from dual union all
select 18886,'MIT' from dual union all
select 18887,'ISAS' from dual union all
select 18887,'PRL' from dual union all
select 18889,'AFSWC' from dual union all
select 18890,'AFSWC' from dual union all
select 18891,'AFSWC' from dual union all
select 18892,'AFSWC' from dual union all
select 18896,'SAND' from dual union all
select 18897,'UWO' from dual union all
select 18898,'GSFC' from dual union all
select 18900,'UWO' from dual union all
select 18901,'INTA' from dual union all
select 18902,'GSFC' from dual union all
select 18903,'INTA' from dual union all
select 18905,'GSFC' from dual union all
select 18906,'GSFC' from dual union all
select 18907,'UMI' from dual union all
select 18908,'AFCRL' from dual union all
select 18909,'AFCRL' from dual union all
select 18910,'CULH' from dual union all
select 18910,'LUX' from dual union all
select 18911,'AFCRL' from dual union all
select 18912,'AFCRL' from dual union all
select 18913,'AFCRL' from dual union all
select 18914,'UKY' from dual union all
select 18916,'AFCRL' from dual union all
select 18917,'GSFC' from dual union all
select 18918,'LRL' from dual union all
select 18920,'321SMW' from dual union all
select 18923,'AFCRL' from dual union all
select 18924,'AFCRL' from dual union all
select 18925,'AFCRL' from dual union all
select 18927,'AFCRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 18929,'LRL' from dual union all
select 18931,'GRI' from dual union all
select 18932,'AFCRL' from dual union all
select 18933,'CNIE' from dual union all
select 18934,'321SMW' from dual union all
select 18935,'DUD' from dual union all
select 18938,'WSMR' from dual union all
select 18940,'AFCRL' from dual union all
select 18941,'CAST' from dual union all
select 18942,'WSMR' from dual union all
select 18944,'BOL' from dual union all
select 18944,'ESTEC' from dual union all
select 18946,'DUD' from dual union all
select 18948,'321SMW' from dual union all
select 18949,'UNK' from dual union all
select 18950,'KPNO' from dual union all
select 18951,'UCO' from dual union all
select 18952,'UCB' from dual union all
select 18954,'AFCRL' from dual union all
select 18956,'AFCRL' from dual union all
select 18957,'RSRS' from dual union all
select 18959,'RSRS' from dual union all
select 18963,'PRIN' from dual union all
select 18965,'UCB' from dual union all
select 18967,'AFCRL' from dual union all
select 18972,'UTD' from dual union all
select 18974,'UCO' from dual union all
select 18976,'CAST' from dual union all
select 18977,'WSMR' from dual union all
select 18978,'GMS' from dual union all
select 18979,'AFCRL' from dual union all
select 18982,'UNK' from dual union all
select 18983,'IASB' from dual union all
select 18983,'LIEGE' from dual union all
select 18984,'IASB' from dual union all
select 18986,'GMS' from dual union all
select 18989,'WSMR' from dual union all
select 18990,'WSMR' from dual union all
select 18992,'IEM' from dual union all
select 18992,'NIRFI' from dual union all
select 18993,'CNRS' from dual union all
select 18994,'CNRS' from dual union all
select 18997,'IASB' from dual union all
select 18997,'LIEGE' from dual union all
select 18998,'IASB' from dual union all
select 18999,'SHEF' from dual union all
select 18999,'UCW' from dual union all
select 19004,'UWI' from dual union all
select 19005,'MSSL' from dual union all
select 19005,'QUB' from dual union all
select 19005,'UCW' from dual union all
select 19006,'90MW' from dual union all
select 19014,'AN' from dual union all
select 19016,'SHEF' from dual union all
select 19016,'UCW' from dual union all
select 19023,'SHEF' from dual union all
select 19025,'MSSL' from dual union all
select 19025,'QUB' from dual union all
select 19025,'UCW' from dual union all
select 19026,'WSMR' from dual union all
select 19027,'WSMR' from dual union all
select 19030,'SHEF' from dual union all
select 19038,'BRL' from dual union all
select 19039,'ISAS' from dual union all
select 19040,'AFCRL' from dual union all
select 19041,'AFCRL' from dual union all
select 19042,'AFCRL' from dual union all
select 19043,'KYO' from dual union all
select 19044,'ISAS' from dual union all
select 19045,'DFF' from dual union all
select 19045,'MISU' from dual union all
select 19048,'AFCRL' from dual union all
select 19049,'DUD' from dual union all
select 19050,'MPIK' from dual union all
select 19051,'AFCRL' from dual union all
select 19056,'LUX' from dual union all
select 19058,'UMI' from dual union all
select 19059,'GSFC' from dual union all
select 19059,'UMI' from dual union all
select 19060,'CNES' from dual union all
select 19062,'DUD' from dual union all
select 19063,'ISAS' from dual union all
select 19064,'NAG' from dual union all
select 19068,'CNIE' from dual union all
select 19069,'CNIE' from dual union all
select 19070,'AFSWC' from dual union all
select 19071,'AFSWC' from dual union all
select 19072,'AFSWC' from dual union all
select 19079,'ISAS' from dual union all
select 19080,'LASL' from dual union all
select 19080,'UAF' from dual union all
select 19083,'LASL' from dual union all
select 19083,'UAF' from dual union all
select 19084,'TOK' from dual union all
select 19085,'AFCRL' from dual union all
select 19090,'CORN' from dual union all
select 19091,'STA' from dual union all
select 19092,'GCA' from dual union all
select 19092,'UIUC' from dual union all
select 19093,'HCO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 19094,'NRCC' from dual union all
select 19095,'LRL' from dual union all
select 19096,'NRCC' from dual union all
select 19097,'UIUC' from dual union all
select 19098,'IIAE' from dual union all
select 19101,'LASL' from dual union all
select 19101,'UAF' from dual union all
select 19102,'NASDA' from dual union all
select 19103,'RSRS' from dual union all
select 19105,'NASDA' from dual union all
select 19107,'AFCRL' from dual union all
select 19109,'MSC' from dual union all
select 19109,'UCB' from dual union all
select 19112,'STA' from dual union all
select 19113,'GMS' from dual union all
select 19113,'PRL' from dual union all
select 19114,'NRL' from dual union all
select 19116,'UHI' from dual union all
select 19119,'WSMR' from dual union all
select 19120,'UCO' from dual union all
select 19123,'ISAS' from dual union all
select 19124,'AFSWC' from dual union all
select 19127,'RSRS' from dual union all
select 19129,'RSRS' from dual union all
select 19130,'RSRS' from dual union all
select 19131,'AFCRL' from dual union all
select 19133,'MIT' from dual union all
select 19134,'CIT' from dual union all
select 19135,'QUB' from dual union all
select 19135,'RSRS' from dual union all
select 19136,'BIR' from dual union all
select 19136,'SOTON' from dual union all
select 19137,'LIEGE' from dual union all
select 19138,'AFCRL' from dual union all
select 19140,'SOTON' from dual union all
select 19141,'AFCRL' from dual union all
select 19144,'NRL' from dual union all
select 19146,'MSSL' from dual union all
select 19149,'UCL' from dual union all
select 19151,'GSFC' from dual union all
select 19152,'ISAS' from dual union all
select 19153,'GSFC' from dual union all
select 19154,'BIR' from dual union all
select 19154,'SOTON' from dual union all
select 19155,'UCL' from dual union all
select 19157,'QUB' from dual union all
select 19157,'RSRS' from dual union all
select 19163,'TOK' from dual union all
select 19168,'RAE' from dual union all
select 19168,'WRE' from dual union all
select 19171,'WSMR' from dual union all
select 19172,'MSSL' from dual union all
select 19174,'TOKAI' from dual union all
select 19176,'IIAE' from dual union all
select 19177,'AFCRL' from dual union all
select 19179,'GMS' from dual union all
select 19180,'BOL' from dual union all
select 19180,'ESTEC' from dual union all
select 19182,'CALG' from dual union all
select 19183,'CALG' from dual union all
select 19185,'JHU' from dual union all
select 19195,'AFCRL' from dual union all
select 19196,'GSFC' from dual union all
select 19197,'BRL' from dual union all
select 19198,'AFCRL' from dual union all
select 19199,'AFCRL' from dual union all
select 19200,'AFCRL' from dual union all
select 19201,'AFCRL' from dual union all
select 19202,'AFCRL' from dual union all
select 19203,'NRCC' from dual union all
select 19204,'AFCRL' from dual union all
select 19205,'BRL' from dual union all
select 19206,'BRL' from dual union all
select 19207,'GSFC' from dual union all
select 19208,'AFCRL' from dual union all
select 19210,'AFCRL' from dual union all
select 19211,'AFCRL' from dual union all
select 19212,'AFCRL' from dual union all
select 19213,'BRL' from dual union all
select 19214,'AFCRL' from dual union all
select 19215,'AFCRL' from dual union all
select 19217,'NRL' from dual union all
select 19218,'AFCRL' from dual union all
select 19219,'ASE' from dual union all
select 19220,'BRL' from dual union all
select 19221,'AFCRL' from dual union all
select 19222,'BRL' from dual union all
select 19223,'AFCRL' from dual union all
select 19224,'AFCRL' from dual union all
select 19227,'NRCC' from dual union all
select 19228,'AFCRL' from dual union all
select 19229,'ISAS' from dual union all
select 19229,'TOK' from dual union all
select 19232,'AFCRL' from dual union all
select 19233,'LAS' from dual union all
select 19233,'ONERA' from dual union all
select 19234,'NRL' from dual union all
select 19236,'BREI' from dual union all
select 19236,'SHEF' from dual union all
select 19238,'MSSL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 19239,'GRI' from dual union all
select 19240,'AFCRL' from dual union all
select 19241,'LRL' from dual union all
select 19243,'CULH' from dual union all
select 19243,'UKMO' from dual union all
select 19246,'MPE' from dual union all
select 19246,'NDRE' from dual union all
select 19248,'CULH' from dual union all
select 19248,'QUB' from dual union all
select 19249,'BREI' from dual union all
select 19249,'MPAE' from dual union all
select 19250,'BREI' from dual union all
select 19250,'MPAE' from dual union all
select 19251,'GSFC' from dual union all
select 19252,'NDRE' from dual union all
select 19256,'YORK' from dual union all
select 19257,'YORK' from dual union all
select 19258,'UWI' from dual union all
select 19260,'DUD' from dual union all
select 19261,'AFCRL' from dual union all
select 19262,'NRCC' from dual union all
select 19263,'AFCRL' from dual union all
select 19264,'AFCRL' from dual union all
select 19265,'PRL' from dual union all
select 19266,'MPAE' from dual union all
select 19267,'MPAE' from dual union all
select 19269,'AFCRL' from dual union all
select 19271,'MPAE' from dual union all
select 19272,'MPAE' from dual union all
select 19273,'NAG' from dual union all
select 19277,'AFCRL' from dual union all
select 19279,'AFCRL' from dual union all
select 19280,'MPAE' from dual union all
select 19281,'MPAE' from dual union all
select 19282,'GSFC' from dual union all
select 19283,'AN' from dual union all
select 19284,'MPAE' from dual union all
select 19285,'GCA' from dual union all
select 19286,'GCA' from dual union all
select 19288,'NRCC' from dual union all
select 19289,'MPAE' from dual union all
select 19292,'MPAE' from dual union all
select 19295,'MPAE' from dual union all
select 19296,'AFCRL' from dual union all
select 19299,'MPAE' from dual union all
select 19300,'MPAE' from dual union all
select 19304,'ISAS' from dual union all
select 19305,'IIAE' from dual union all
select 19306,'IIAE' from dual union all
select 19307,'IIAE' from dual union all
select 19308,'GMS' from dual union all
select 19310,'BRL' from dual union all
select 19311,'BRL' from dual union all
select 19312,'BRL' from dual union all
select 19313,'BRL' from dual union all
select 19314,'BRL' from dual union all
select 19316,'BRL' from dual union all
select 19317,'WFF' from dual union all
select 19318,'AFCRL' from dual union all
select 19319,'AFCRL' from dual union all
select 19320,'AFCRL' from dual union all
select 19321,'AFCRL' from dual union all
select 19322,'AFCRL' from dual union all
select 19323,'AFCRL' from dual union all
select 19324,'NRL' from dual union all
select 19325,'NRL' from dual union all
select 19326,'MISU' from dual union all
select 19326,'UCL' from dual union all
select 19327,'BRL' from dual union all
select 19328,'BRL' from dual union all
select 19329,'BRL' from dual union all
select 19330,'BRL' from dual union all
select 19334,'WRE' from dual union all
select 19335,'WRE' from dual union all
select 19336,'WRE' from dual union all
select 19337,'GSFC' from dual union all
select 19338,'WRE' from dual union all
select 19339,'AN' from dual union all
select 19340,'AN' from dual union all
select 19341,'AFCRL' from dual union all
select 19342,'NRL' from dual union all
select 19343,'LASL' from dual union all
select 19348,'GSFC' from dual union all
select 19355,'TAO' from dual union all
select 19366,'GSFC' from dual union all
select 19371,'AFCRL' from dual union all
select 19372,'AFCRL' from dual union all
select 19388,'IEM' from dual union all
select 19388,'NIRFI' from dual union all
select 19405,'AFCRL' from dual union all
select 19406,'AFCRL' from dual union all
select 19407,'GSFC' from dual union all
select 19409,'IEM' from dual union all
select 19409,'NIRFI' from dual union all
select 19410,'BIR' from dual union all
select 19410,'MSSL' from dual union all
select 19410,'RSRS' from dual union all
select 19411,'MISU' from dual union all
select 19411,'UCL' from dual union all
select 19412,'GSFC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 19413,'MISU' from dual union all
select 19413,'UCL' from dual union all
select 19414,'GSFC' from dual union all
select 19415,'MSC' from dual union all
select 19416,'UWI' from dual union all
select 19417,'UTOR' from dual union all
select 19421,'GSFC' from dual union all
select 19422,'NRL' from dual union all
select 19423,'GSFC' from dual union all
select 19424,'GSFC' from dual union all
select 19425,'GCA' from dual union all
select 19425,'UIUC' from dual union all
select 19426,'AFCRL' from dual union all
select 19427,'GSFC' from dual union all
select 19431,'USAF' from dual union all
select 19432,'USAF' from dual union all
select 19433,'USAF' from dual union all
select 19434,'USAF' from dual union all
select 19435,'PIHM' from dual union all
select 19436,'PIHM' from dual union all
select 19437,'PIHM' from dual union all
select 19441,'UKMO' from dual union all
select 19452,'INTA' from dual union all
select 19454,'USAF' from dual union all
select 19466,'SUPA' from dual union all
select 19469,'USAF' from dual union all
select 19480,'USAF' from dual union all
select 19492,'PSU' from dual union all
select 19493,'PRL' from dual union all
select 19496,'USAF' from dual union all
select 19503,'USAF' from dual union all
select 19518,'USAF' from dual union all
select 19528,'USAF' from dual union all
select 19533,'SUPA' from dual union all
select 19536,'USAF' from dual union all
select 19540,'USAF' from dual union all
select 19547,'USAF' from dual union all
select 19558,'PRL' from dual union all
select 19563,'USAF' from dual union all
select 19574,'USAF' from dual union all
select 19582,'USAF' from dual union all
select 19586,'USAF' from dual union all
select 19605,'USAF' from dual union all
select 19617,'USAF' from dual union all
select 19618,'USAF' from dual union all
select 19622,'USAF' from dual union all
select 19623,'USAF' from dual union all
select 19624,'USAF' from dual union all
select 19629,'INTA' from dual union all
select 19634,'USAF' from dual union all
select 19640,'USAF' from dual union all
select 19651,'USAF' from dual union all
select 19654,'JMA' from dual union all
select 19656,'JMA' from dual union all
select 19657,'JMA' from dual union all
select 19675,'SUPA' from dual union all
select 19681,'USAF' from dual union all
select 19687,'USAF' from dual union all
select 19695,'USAF' from dual union all
select 19707,'USAF' from dual union all
select 19709,'USAF' from dual union all
select 19717,'PSU' from dual union all
select 19724,'USAF' from dual union all
select 19743,'USAF' from dual union all
select 19749,'PSU' from dual union all
select 19751,'USAF' from dual union all
select 19756,'USAF' from dual union all
select 19762,'USAF' from dual union all
select 19772,'USAF' from dual union all
select 19785,'USAF' from dual union all
select 19791,'JMA' from dual union all
select 19797,'USAF' from dual union all
select 19802,'USAF' from dual union all
select 19810,'USAF' from dual union all
select 19829,'INTA' from dual union all
select 19839,'USAF' from dual union all
select 19859,'USAF' from dual union all
select 19873,'USAF' from dual union all
select 19874,'INTA' from dual union all
select 19887,'USAF' from dual union all
select 19905,'USAF' from dual union all
select 19911,'USAF' from dual union all
select 19917,'SUPA' from dual union all
select 19926,'USAF' from dual union all
select 19933,'USAF' from dual union all
select 19941,'USAF' from dual union all
select 19952,'USAF' from dual union all
select 1996,'NRL' from dual union all
select 1997,'NRL' from dual union all
select 19970,'USAF' from dual union all
select 19971,'USAF' from dual union all
select 19972,'PSU' from dual union all
select 19974,'PSU' from dual union all
select 19975,'PSU' from dual union all
select 19977,'USAF' from dual union all
select 19978,'PSU' from dual union all
select 19979,'PSU' from dual union all
select 1998,'NRL' from dual union all
select 19985,'USAF' from dual union all
select 19986,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 1999,'NRL' from dual union all
select 19991,'PSU' from dual union all
select 19992,'PSU' from dual union all
select 19993,'PSU' from dual union all
select 2000,'NRL' from dual union all
select 20002,'USAF' from dual union all
select 20007,'USAF' from dual union all
select 20008,'USAF' from dual union all
select 2001,'UMI' from dual union all
select 20018,'USAF' from dual union all
select 2002,'NRL' from dual union all
select 20020,'INTA' from dual union all
select 20028,'USAF' from dual union all
select 2003,'NRL' from dual union all
select 20037,'USAF' from dual union all
select 2004,'SCEL' from dual union all
select 20044,'USAF' from dual union all
select 2005,'AFCRL' from dual union all
select 20051,'INTA' from dual union all
select 20054,'USAF' from dual union all
select 20055,'SUPA' from dual union all
select 20061,'USAF' from dual union all
select 20067,'SUPA' from dual union all
select 20080,'USAF' from dual union all
select 20083,'USAF' from dual union all
select 20091,'INTA' from dual union all
select 20094,'USAF' from dual union all
select 20097,'UKMO' from dual union all
select 20102,'USAF' from dual union all
select 20105,'UKMO' from dual union all
select 20107,'USAF' from dual union all
select 20108,'USAF' from dual union all
select 2011,'AFCRL' from dual union all
select 20116,'USAF' from dual union all
select 20117,'UKMO' from dual union all
select 2013,'SCEL' from dual union all
select 20131,'USAF' from dual union all
select 20132,'UKMO' from dual union all
select 20137,'USAF' from dual union all
select 2014,'UMI' from dual union all
select 20142,'PSU' from dual union all
select 20148,'UKMO' from dual union all
select 20155,'USAF' from dual union all
select 20164,'USAF' from dual union all
select 20167,'UKMO' from dual union all
select 20169,'PSU' from dual union all
select 20170,'USAF' from dual union all
select 20171,'UCW' from dual union all
select 20181,'USAF' from dual union all
select 20189,'USAF' from dual union all
select 2019,'AFCRL' from dual union all
select 20198,'USAF' from dual union all
select 2020,'AFCRL' from dual union all
select 20201,'SUPA' from dual union all
select 20208,'USAF' from dual union all
select 20214,'USAF' from dual union all
select 20216,'PSU' from dual union all
select 20223,'UKMO' from dual union all
select 20224,'USAF' from dual union all
select 2023,'AFCRL' from dual union all
select 2023,'UUT' from dual union all
select 20230,'MRN' from dual union all
select 20236,'UKMO' from dual union all
select 2024,'AFCRL' from dual union all
select 2024,'UUT' from dual union all
select 20240,'USAF' from dual union all
select 20242,'MRN' from dual union all
select 20248,'UKMO' from dual union all
select 20252,'UKMO' from dual union all
select 20256,'UKMO' from dual union all
select 2026,'AFCRL' from dual union all
select 2026,'RIU' from dual union all
select 20260,'SUPA' from dual union all
select 20262,'USAF' from dual union all
select 20264,'MRN' from dual union all
select 20269,'AN' from dual union all
select 2027,'AFCRL' from dual union all
select 20270,'LARC' from dual union all
select 20272,'USAF' from dual union all
select 20273,'USAF' from dual union all
select 20276,'JMA' from dual union all
select 2028,'SUI' from dual union all
select 20280,'NOTS' from dual union all
select 20287,'JMA' from dual union all
select 2029,'SUI' from dual union all
select 20298,'JMA' from dual union all
select 2030,'SUI' from dual union all
select 20309,'UKMO' from dual union all
select 2031,'SUI' from dual union all
select 20315,'USAF' from dual union all
select 20317,'NOTS' from dual union all
select 2032,'SCEL' from dual union all
select 2033,'SUI' from dual union all
select 20332,'USAF' from dual union all
select 2034,'SUI' from dual union all
select 2035,'SUI' from dual union all
select 2036,'SCEL' from dual union all
select 20374,'USAF' from dual union all
select 20387,'USAF' from dual union all
select 2039,'AFCRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 2039,'UCO' from dual union all
select 20396,'USAF' from dual union all
select 20397,'USAF' from dual union all
select 20404,'USAF' from dual union all
select 20419,'UKMO' from dual union all
select 20420,'USAF' from dual union all
select 20434,'USAF' from dual union all
select 20446,'MRN' from dual union all
select 20459,'USAF' from dual union all
select 20462,'USAF' from dual union all
select 20484,'MRN' from dual union all
select 20485,'USAF' from dual union all
select 20496,'UKMO' from dual union all
select 20501,'MRN' from dual union all
select 20506,'USAF' from dual union all
select 20517,'USAF' from dual union all
select 20518,'USAF' from dual union all
select 2054,'AFCRL' from dual union all
select 2054,'UUT' from dual union all
select 20542,'WSMR' from dual union all
select 20549,'USAF' from dual union all
select 2055,'AFCRL' from dual union all
select 2055,'UUT' from dual union all
select 20556,'TSAO' from dual union all
select 20565,'USAF' from dual union all
select 20577,'USAF' from dual union all
select 2058,'NRL' from dual union all
select 20581,'USAF' from dual union all
select 20595,'UKMO' from dual union all
select 20596,'USAF' from dual union all
select 20598,'USAF' from dual union all
select 20599,'USAF' from dual union all
select 2060,'NRL' from dual union all
select 20601,'USAF' from dual union all
select 2062,'NRL' from dual union all
select 20649,'USAF' from dual union all
select 20665,'MRN' from dual union all
select 20666,'USAF' from dual union all
select 20676,'USAF' from dual union all
select 20680,'INTA' from dual union all
select 20705,'USAF' from dual union all
select 20706,'YORK' from dual union all
select 20719,'MRN' from dual union all
select 2073,'PARD' from dual union all
select 20732,'USAF' from dual union all
select 20742,'USAF' from dual union all
select 20760,'UKMO' from dual union all
select 20763,'INTA' from dual union all
select 20783,'MRN' from dual union all
select 20806,'USAF' from dual union all
select 20814,'INTA' from dual union all
select 20821,'USAF' from dual union all
select 20823,'WSMR' from dual union all
select 20827,'INTA' from dual union all
select 20841,'USAF' from dual union all
select 20850,'USAF' from dual union all
select 20853,'INTA' from dual union all
select 20859,'WSMR' from dual union all
select 20871,'USAF' from dual union all
select 20895,'USAF' from dual union all
select 20908,'USAF' from dual union all
select 20919,'USAF' from dual union all
select 20936,'UKMO' from dual union all
select 20940,'USAF' from dual union all
select 20941,'USAF' from dual union all
select 20952,'USAF' from dual union all
select 20962,'USAF' from dual union all
select 20976,'PRL' from dual union all
select 20984,'USAF' from dual union all
select 20987,'GSFC' from dual union all
select 21006,'USAF' from dual union all
select 21019,'SUPA' from dual union all
select 21052,'PRL' from dual union all
select 21070,'USAF' from dual union all
select 21093,'USAF' from dual union all
select 21116,'USAF' from dual union all
select 21127,'USAF' from dual union all
select 21130,'USAF' from dual union all
select 21134,'USAF' from dual union all
select 21140,'PRL' from dual union all
select 21141,'SUPA' from dual union all
select 21143,'USAF' from dual union all
select 21147,'USAF' from dual union all
select 21166,'PRL' from dual union all
select 21178,'CAST' from dual union all
select 21189,'WSMR' from dual union all
select 21194,'TSAO' from dual union all
select 21195,'TSAO' from dual union all
select 21196,'TSAO' from dual union all
select 21197,'ESSA' from dual union all
select 21199,'ESSA' from dual union all
select 21200,'ESSA' from dual union all
select 21201,'AN' from dual union all
select 21204,'TSAO' from dual union all
select 21205,'TSAO' from dual union all
select 21206,'ESSA' from dual union all
select 21207,'YORK' from dual union all
select 21208,'TSAO' from dual union all
select 21210,'TSAO' from dual union all
select 21211,'WSMR' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 21213,'TSAO' from dual union all
select 21214,'TSAO' from dual union all
select 21215,'TSAO' from dual union all
select 21217,'TSAO' from dual union all
select 21218,'TSAO' from dual union all
select 21219,'AN' from dual union all
select 21221,'WSMR' from dual union all
select 21222,'TSAO' from dual union all
select 21223,'AN' from dual union all
select 21224,'TSAO' from dual union all
select 21225,'WSMR' from dual union all
select 21226,'WSMR' from dual union all
select 21227,'WSMR' from dual union all
select 21236,'TSAO' from dual union all
select 21279,'AN' from dual union all
select 21282,'MSFC' from dual union all
select 21288,'USAF' from dual union all
select 21291,'USAF' from dual union all
select 21350,'GSFC' from dual union all
select 21352,'WSMR' from dual union all
select 21380,'PSU' from dual union all
select 21385,'WSMR' from dual union all
select 21398,'UCW' from dual union all
select 21410,'UIO' from dual union all
select 21411,'AN' from dual union all
select 21412,'USAF' from dual union all
select 21413,'USAF' from dual union all
select 21415,'USAF' from dual union all
select 21418,'USAF' from dual union all
select 21423,'USAF' from dual union all
select 21427,'USAF' from dual union all
select 21428,'WSMR' from dual union all
select 21429,'AN' from dual union all
select 21433,'AN' from dual union all
select 21436,'USAF' from dual union all
select 21438,'GSFC' from dual union all
select 21440,'WSMR' from dual union all
select 21452,'AN' from dual union all
select 21467,'AN' from dual union all
select 21477,'AN' from dual union all
select 2150,'NRL' from dual union all
select 21592,'WRE' from dual union all
select 21593,'USAF' from dual union all
select 21598,'GCA' from dual union all
select 21601,'GSFC' from dual union all
select 21615,'INTA' from dual union all
select 21615,'NASA' from dual union all
select 21628,'RSRS' from dual union all
select 21634,'USAF' from dual union all
select 21635,'BAL' from dual union all
select 21637,'BAL' from dual union all
select 21638,'BAL' from dual union all
select 21640,'BAL' from dual union all
select 21648,'JMA' from dual union all
select 21649,'JMA' from dual union all
select 21650,'JMA' from dual union all
select 21651,'JMA' from dual union all
select 21652,'JMA' from dual union all
select 21653,'JMA' from dual union all
select 21654,'JMA' from dual union all
select 21655,'ISRO' from dual union all
select 21657,'ISRO' from dual union all
select 21658,'IIAE' from dual union all
select 21659,'WFF' from dual union all
select 21660,'WFF' from dual union all
select 21661,'WFF' from dual union all
select 21662,'WFF' from dual union all
select 21663,'WFF' from dual union all
select 21664,'USA' from dual union all
select 21675,'WRE' from dual union all
select 21676,'WRE' from dual union all
select 21677,'SAND' from dual union all
select 21679,'SAND' from dual union all
select 21680,'WSMR' from dual union all
select 21682,'SAND' from dual union all
select 21701,'WRE' from dual union all
select 21706,'BONN' from dual union all
select 21728,'INTA' from dual union all
select 21742,'LARC' from dual union all
select 21743,'BOL' from dual union all
select 21743,'ESTEC' from dual union all
select 21768,'STA' from dual union all
select 21777,'MPAE' from dual union all
select 21778,'MRI' from dual union all
select 21779,'GSFC' from dual union all
select 21780,'CNES' from dual union all
select 21780,'CSG' from dual union all
select 21781,'GSFC' from dual union all
select 21782,'AFCRL' from dual union all
select 21783,'AFCRL' from dual union all
select 21784,'GSFC' from dual union all
select 21785,'UKMO' from dual union all
select 21786,'WRE' from dual union all
select 21787,'UKMO' from dual union all
select 21788,'UKMO' from dual union all
select 21789,'UKMO' from dual union all
select 21790,'UKMO' from dual union all
select 21791,'UKMO' from dual union all
select 21792,'UKMO' from dual union all
select 21793,'MSC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 21794,'UKMO' from dual union all
select 21795,'UKMO' from dual union all
select 21796,'AFCRL' from dual union all
select 21797,'UTOR' from dual union all
select 21800,'AFCRL' from dual union all
select 21801,'PIHM' from dual union all
select 21802,'CNES' from dual union all
select 21802,'CSG' from dual union all
select 21803,'UAF' from dual union all
select 21805,'UAF' from dual union all
select 21806,'INTA' from dual union all
select 21806,'NASA' from dual union all
select 21807,'MSFC' from dual union all
select 21808,'MSFC' from dual union all
select 21809,'MSFC' from dual union all
select 21810,'MSFC' from dual union all
select 21811,'MSFC' from dual union all
select 21812,'MSFC' from dual union all
select 21813,'MSFC' from dual union all
select 21824,'UCL' from dual union all
select 21825,'UAF' from dual union all
select 21827,'GSFC' from dual union all
select 21828,'UAF' from dual union all
select 21830,'MPE' from dual union all
select 21831,'ASE' from dual union all
select 21832,'AFCRL' from dual union all
select 21835,'UAF' from dual union all
select 21836,'UTD' from dual union all
select 21837,'NIPR' from dual union all
select 21838,'UCL' from dual union all
select 21841,'GSOC' from dual union all
select 21841,'KIEL' from dual union all
select 21841,'MPE' from dual union all
select 21842,'GSOC' from dual union all
select 21842,'KIEL' from dual union all
select 21842,'MPE' from dual union all
select 21843,'CAL' from dual union all
select 21845,'MPE' from dual union all
select 21845,'UCL' from dual union all
select 21846,'MPAE' from dual union all
select 21847,'MPAE' from dual union all
select 21848,'NIPR' from dual union all
select 21850,'UCB' from dual union all
select 21851,'MPAE' from dual union all
select 21853,'GSFC' from dual union all
select 21854,'MPAE' from dual union all
select 21857,'MPAE' from dual union all
select 21858,'MPAE' from dual union all
select 21859,'BIR' from dual union all
select 21860,'MPAE' from dual union all
select 21861,'GMS' from dual union all
select 21862,'MPE' from dual union all
select 21863,'INTA' from dual union all
select 21863,'MPAE' from dual union all
select 21864,'INTA' from dual union all
select 21864,'MPAE' from dual union all
select 21865,'INTA' from dual union all
select 21865,'MPAE' from dual union all
select 21866,'NRCC' from dual union all
select 21867,'INTA' from dual union all
select 21867,'MPAE' from dual union all
select 21868,'INTA' from dual union all
select 21868,'MPAE' from dual union all
select 21869,'BIR' from dual union all
select 21869,'DSRI' from dual union all
select 21870,'MPAE' from dual union all
select 21872,'INTA' from dual union all
select 21872,'MPAE' from dual union all
select 21874,'INTA' from dual union all
select 21874,'MPAE' from dual union all
select 21875,'INTA' from dual union all
select 21875,'MPAE' from dual union all
select 21877,'INTA' from dual union all
select 21877,'MPAE' from dual union all
select 21878,'INTA' from dual union all
select 21878,'MPAE' from dual union all
select 21879,'INTA' from dual union all
select 21879,'MPAE' from dual union all
select 21880,'BONN' from dual union all
select 21880,'UKMO' from dual union all
select 21881,'MPAE' from dual union all
select 21882,'INTA' from dual union all
select 21882,'MPAE' from dual union all
select 21883,'INTA' from dual union all
select 21883,'MPAE' from dual union all
select 21884,'INTA' from dual union all
select 21884,'MPAE' from dual union all
select 21885,'BONN' from dual union all
select 21885,'UKMO' from dual union all
select 21886,'INTA' from dual union all
select 21886,'MPAE' from dual union all
select 21888,'GMS' from dual union all
select 21889,'GMS' from dual union all
select 21890,'MPAE' from dual union all
select 21891,'INTA' from dual union all
select 21891,'MPAE' from dual union all
select 21892,'INTA' from dual union all
select 21892,'MPAE' from dual union all
select 21893,'INTA' from dual union all
select 21893,'MPAE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 21894,'INTA' from dual union all
select 21894,'MPAE' from dual union all
select 21895,'INTA' from dual union all
select 21895,'MPAE' from dual union all
select 21896,'MPAE' from dual union all
select 21897,'INTA' from dual union all
select 21897,'MPAE' from dual union all
select 21899,'AN' from dual union all
select 21900,'AN' from dual union all
select 21902,'INTA' from dual union all
select 21902,'MPAE' from dual union all
select 21903,'GSFC' from dual union all
select 21904,'INTA' from dual union all
select 21904,'MPAE' from dual union all
select 21905,'INTA' from dual union all
select 21905,'MPAE' from dual union all
select 21906,'INTA' from dual union all
select 21906,'MPAE' from dual union all
select 21908,'GMS' from dual union all
select 21910,'NRL' from dual union all
select 21913,'GSFC' from dual union all
select 21914,'GSFC' from dual union all
select 21916,'GSFC' from dual union all
select 21919,'GSFC' from dual union all
select 21921,'GSFC' from dual union all
select 21922,'INTA' from dual union all
select 21922,'MPAE' from dual union all
select 21923,'INTA' from dual union all
select 21923,'MPAE' from dual union all
select 21924,'GSFC' from dual union all
select 21926,'GSFC' from dual union all
select 21927,'UMI' from dual union all
select 21928,'GSOC' from dual union all
select 21928,'KIEL' from dual union all
select 21928,'MPE' from dual union all
select 21929,'WRE' from dual union all
select 21930,'GCA' from dual union all
select 21930,'LPARL' from dual union all
select 21933,'UMI' from dual union all
select 21935,'UIUC' from dual union all
select 21936,'AFCRL' from dual union all
select 21937,'NRCC' from dual union all
select 21937,'RSRS' from dual union all
select 21938,'UMI' from dual union all
select 21939,'GSFC' from dual union all
select 21939,'UMI' from dual union all
select 21940,'AFCRL' from dual union all
select 21941,'UMI' from dual union all
select 21943,'GSFC' from dual union all
select 21943,'UMI' from dual union all
select 21944,'NRL' from dual union all
select 21945,'GCA' from dual union all
select 21946,'CULH' from dual union all
select 21946,'HCO' from dual union all
select 21946,'ICST' from dual union all
select 21947,'GSFC' from dual union all
select 21947,'UMD' from dual union all
select 21948,'UIUC' from dual union all
select 21949,'UIUC' from dual union all
select 21950,'AFCRL' from dual union all
select 21951,'UIUC' from dual union all
select 21952,'AFCRL' from dual union all
select 21954,'TRW' from dual union all
select 21955,'ASE' from dual union all
select 21956,'NRCC' from dual union all
select 21956,'RSRS' from dual union all
select 21957,'NRCC' from dual union all
select 21957,'RSRS' from dual union all
select 21958,'NRL' from dual union all
select 21959,'NRCC' from dual union all
select 21959,'RSRS' from dual union all
select 21960,'AFCRL' from dual union all
select 21961,'NRL' from dual union all
select 21962,'AFCRL' from dual union all
select 21963,'GSFC' from dual union all
select 21965,'UMI' from dual union all
select 21966,'NDRE' from dual union all
select 21967,'GSOC' from dual union all
select 21967,'KIEL' from dual union all
select 21967,'MPE' from dual union all
select 21969,'AFCRL' from dual union all
select 21970,'AFCRL' from dual union all
select 21971,'GSFC' from dual union all
select 21971,'UCB' from dual union all
select 21972,'GSFC' from dual union all
select 21973,'GSFC' from dual union all
select 21976,'WRE' from dual union all
select 21978,'MSSL' from dual union all
select 21979,'WSMR' from dual union all
select 21981,'AFCRL' from dual union all
select 21982,'NRL' from dual union all
select 21983,'BAL' from dual union all
select 21983,'MPE' from dual union all
select 21983,'NDRE' from dual union all
select 21984,'BAL' from dual union all
select 21984,'MPE' from dual union all
select 21984,'NDRE' from dual union all
select 21985,'LUX' from dual union all
select 21987,'RTG' from dual union all
select 21988,'GSFC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 21989,'RTG' from dual union all
select 21990,'RTG' from dual union all
select 21991,'RTG' from dual union all
select 21992,'GSFC' from dual union all
select 21993,'RTG' from dual union all
select 21994,'GSFC' from dual union all
select 21995,'ROE' from dual union all
select 21998,'MPE' from dual union all
select 21998,'NDRE' from dual union all
select 22000,'LUX' from dual union all
select 22001,'AFCRL' from dual union all
select 22003,'AFCRL' from dual union all
select 22004,'JHU' from dual union all
select 22005,'UCO' from dual union all
select 22006,'PRL' from dual union all
select 22007,'SUPA' from dual union all
select 22010,'SUPA' from dual union all
select 22013,'AFSWC' from dual union all
select 22017,'MPE' from dual union all
select 22017,'UCL' from dual union all
select 22018,'UCL' from dual union all
select 22020,'UCB' from dual union all
select 22021,'UAF' from dual union all
select 22023,'AFCRL' from dual union all
select 22024,'MPE' from dual union all
select 22024,'QUB' from dual union all
select 22025,'CULH' from dual union all
select 22036,'UMN' from dual union all
select 22036,'UNH' from dual union all
select 22038,'UMI' from dual union all
select 22040,'GSFC' from dual union all
select 22045,'WRE' from dual union all
select 22046,'UTD' from dual union all
select 22049,'ADEL' from dual union all
select 22049,'UKMO' from dual union all
select 22050,'WRE' from dual union all
select 22065,'SOTON' from dual union all
select 22073,'YORK' from dual union all
select 22076,'CAL' from dual union all
select 22080,'AN' from dual union all
select 22081,'ARC' from dual union all
select 22083,'WSMR' from dual union all
select 22084,'WSMR' from dual union all
select 22086,'ARC' from dual union all
select 22087,'TUC' from dual union all
select 22089,'LRL' from dual union all
select 22090,'TUC' from dual union all
select 22091,'AFCRL' from dual union all
select 22092,'AFCRL' from dual union all
select 22094,'MIT' from dual union all
select 22102,'UMN' from dual union all
select 22103,'LRL' from dual union all
select 22105,'AN' from dual union all
select 22106,'LASL' from dual union all
select 22106,'UAF' from dual union all
select 22109,'AFCRL' from dual union all
select 22110,'WRE' from dual union all
select 22111,'ADEL' from dual union all
select 22112,'AFCRL' from dual union all
select 22113,'AFCRL' from dual union all
select 22115,'AFCRL' from dual union all
select 22116,'NRL' from dual union all
select 22117,'ASE' from dual union all
select 22117,'MSC' from dual union all
select 22119,'WRE' from dual union all
select 22120,'LASL' from dual union all
select 22120,'UAF' from dual union all
select 22121,'WRE' from dual union all
select 22124,'UWI' from dual union all
select 22131,'SAND' from dual union all
select 22133,'UWI' from dual union all
select 22134,'JHU' from dual union all
select 22135,'PRIN' from dual union all
select 22136,'LASL' from dual union all
select 22136,'UAF' from dual union all
select 22137,'MSSL' from dual union all
select 22137,'UTR' from dual union all
select 22138,'AFCRL' from dual union all
select 22138,'GSFC' from dual union all
select 22140,'AFCRL' from dual union all
select 22140,'GSFC' from dual union all
select 22144,'KPNO' from dual union all
select 22144,'UCB' from dual union all
select 22151,'UHI' from dual union all
select 22152,'GSFC' from dual union all
select 22156,'GSFC' from dual union all
select 22159,'ASE' from dual union all
select 22160,'CIT' from dual union all
select 22162,'SHEF' from dual union all
select 22165,'GMS' from dual union all
select 22166,'UWI' from dual union all
select 22168,'MSSL' from dual union all
select 22168,'RSRS' from dual union all
select 22169,'GMS' from dual union all
select 22170,'GMS' from dual union all
select 22171,'GMS' from dual union all
select 22174,'MSSL' from dual union all
select 22174,'RSRS' from dual union all
select 22178,'ADEL' from dual union all
select 22178,'MSSL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 22179,'PIHM' from dual union all
select 22182,'WRE' from dual union all
select 22184,'MSSL' from dual union all
select 22188,'CULH' from dual union all
select 22188,'MSSL' from dual union all
select 22189,'UIUC' from dual union all
select 22191,'PIHM' from dual union all
select 22195,'LUX' from dual union all
select 22195,'RSRS' from dual union all
select 22196,'BREI' from dual union all
select 22196,'RSRS' from dual union all
select 22204,'BREI' from dual union all
select 22204,'RSRS' from dual union all
select 22206,'GSFC' from dual union all
select 22207,'AFCRL' from dual union all
select 22211,'UNH' from dual union all
select 22213,'SAMSO' from dual union all
select 22219,'UMI' from dual union all
select 22222,'NDRE' from dual union all
select 22230,'DUD' from dual union all
select 22231,'AFCRL' from dual union all
select 22232,'DUD' from dual union all
select 22233,'AFCRL' from dual union all
select 22234,'MPIK' from dual union all
select 22234,'UIO' from dual union all
select 22235,'RTG' from dual union all
select 22240,'AFCRL' from dual union all
select 22241,'AFCRL' from dual union all
select 22242,'UMN' from dual union all
select 22243,'NRL' from dual union all
select 22245,'SASK' from dual union all
select 22248,'MSSL' from dual union all
select 22248,'UTR' from dual union all
select 22249,'NPL' from dual union all
select 22251,'NRL' from dual union all
select 22255,'SAMSO' from dual union all
select 22257,'CIT' from dual union all
select 22260,'GSFC' from dual union all
select 22262,'PITT' from dual union all
select 22264,'ISAS' from dual union all
select 22264,'TOK' from dual union all
select 22265,'MSSL' from dual union all
select 22267,'MSSL' from dual union all
select 22271,'GSFC' from dual union all
select 22276,'STA' from dual union all
select 22277,'MSSL' from dual union all
select 22279,'ISAS' from dual union all
select 22282,'GSFC' from dual union all
select 22285,'GSFC' from dual union all
select 22286,'UMI' from dual union all
select 22288,'UCW' from dual union all
select 22289,'NAG' from dual union all
select 22291,'UMI' from dual union all
select 22292,'UCO' from dual union all
select 22293,'GSFC' from dual union all
select 22294,'GSFC' from dual union all
select 22296,'LRL' from dual union all
select 22296,'SAND' from dual union all
select 22299,'MSC' from dual union all
select 22300,'ISAS' from dual union all
select 22301,'ISAS' from dual union all
select 22302,'ASE' from dual union all
select 22303,'MSC' from dual union all
select 22304,'MPAE' from dual union all
select 22309,'SAND' from dual union all
select 22310,'LARC' from dual union all
select 22311,'BRSW' from dual union all
select 22315,'LRL' from dual union all
select 22316,'AN' from dual union all
select 22318,'MPE' from dual union all
select 22319,'OPM' from dual union all
select 22320,'BRSW' from dual union all
select 22323,'LARC' from dual union all
select 22324,'PIHM' from dual union all
select 22326,'GSFC' from dual union all
select 22327,'LUX' from dual union all
select 22328,'BRSW' from dual union all
select 22329,'BRSW' from dual union all
select 22330,'LARC' from dual union all
select 22331,'BRSW' from dual union all
select 22332,'BONN' from dual union all
select 22334,'MSSL' from dual union all
select 22335,'ESSA' from dual union all
select 22335,'UMI' from dual union all
select 22336,'BIR' from dual union all
select 22336,'UTR' from dual union all
select 22337,'UMI' from dual union all
select 22338,'CULH' from dual union all
select 22340,'WRE' from dual union all
select 22346,'OPM' from dual union all
select 22347,'BRSW' from dual union all
select 22348,'GSFC' from dual union all
select 22352,'RSRS' from dual union all
select 22353,'NCAR' from dual union all
select 22355,'RSRS' from dual union all
select 22358,'GSFC' from dual union all
select 22359,'GSFC' from dual union all
select 22367,'WRE' from dual union all
select 22370,'GSFC' from dual union all
select 22371,'UMN' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 22371,'UNH' from dual union all
select 22372,'MSSL' from dual union all
select 22373,'ADEL' from dual union all
select 22374,'UCW' from dual union all
select 22376,'YORK' from dual union all
select 22377,'KGO' from dual union all
select 22377,'NDRE' from dual union all
select 22378,'SOTON' from dual union all
select 22379,'UMN' from dual union all
select 22379,'UNH' from dual union all
select 22380,'SHEF' from dual union all
select 22380,'SOTON' from dual union all
select 22381,'GCA' from dual union all
select 22382,'LUX' from dual union all
select 22383,'PITT' from dual union all
select 22384,'AFCRL' from dual union all
select 22385,'AFCRL' from dual union all
select 22386,'AFCRL' from dual union all
select 22387,'AFCRL' from dual union all
select 22388,'AFCRL' from dual union all
select 22389,'AFCRL' from dual union all
select 22390,'UMI' from dual union all
select 22391,'AFCRL' from dual union all
select 22392,'GSFC' from dual union all
select 22397,'ASE' from dual union all
select 22398,'LUX' from dual union all
select 22399,'NRL' from dual union all
select 22400,'GMS' from dual union all
select 22402,'AN' from dual union all
select 22403,'GCA' from dual union all
select 22404,'PRIN' from dual union all
select 22405,'AFCRL' from dual union all
select 22405,'CORN' from dual union all
select 22407,'LUX' from dual union all
select 22409,'CALG' from dual union all
select 22410,'INTA' from dual union all
select 22410,'MPAE' from dual union all
select 22411,'FRANK' from dual union all
select 22413,'UMD' from dual union all
select 22414,'INTA' from dual union all
select 22414,'MPAE' from dual union all
select 22416,'INTA' from dual union all
select 22416,'MPAE' from dual union all
select 22417,'NRCC' from dual union all
select 22419,'GCA' from dual union all
select 22420,'GCA' from dual union all
select 22421,'GCA' from dual union all
select 22422,'GCA' from dual union all
select 22423,'GCA' from dual union all
select 22424,'NRCC' from dual union all
select 22426,'FRANK' from dual union all
select 22428,'UMN' from dual union all
select 22429,'GMS' from dual union all
select 22430,'CIT' from dual union all
select 22432,'INTA' from dual union all
select 22434,'AN' from dual union all
select 22435,'GCA' from dual union all
select 22438,'AN' from dual union all
select 22440,'UMI' from dual union all
select 22441,'UIO' from dual union all
select 22442,'LASL' from dual union all
select 22445,'AFCRL' from dual union all
select 22446,'GCA' from dual union all
select 22447,'AFCRL' from dual union all
select 22448,'AFCRL' from dual union all
select 22449,'WRE' from dual union all
select 22450,'WRE' from dual union all
select 22451,'WRE' from dual union all
select 22452,'WRE' from dual union all
select 22453,'WRE' from dual union all
select 22454,'AN' from dual union all
select 22455,'WSMR' from dual union all
select 22456,'AFCRL' from dual union all
select 22457,'GCA' from dual union all
select 22458,'WSMR' from dual union all
select 22459,'WSMR' from dual union all
select 22460,'AFCRL' from dual union all
select 22461,'SAND' from dual union all
select 22462,'SAND' from dual union all
select 22463,'SAND' from dual union all
select 22464,'SAND' from dual union all
select 22465,'SAND' from dual union all
select 22466,'NRL' from dual union all
select 22467,'SAND' from dual union all
select 22468,'GCA' from dual union all
select 22469,'SAND' from dual union all
select 22483,'GSFC' from dual union all
select 22493,'MPAE' from dual union all
select 22498,'WRE' from dual union all
select 22499,'WRE' from dual union all
select 22500,'AFCRL' from dual union all
select 22515,'ISAS' from dual union all
select 22524,'AFCRL' from dual union all
select 22525,'SAND' from dual union all
select 22526,'UMD' from dual union all
select 22527,'MPAE' from dual union all
select 22529,'ISAS' from dual union all
select 22530,'MPAE' from dual union all
select 22532,'MPIK' from dual union all
select 22532,'RSRS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 22533,'JHU' from dual union all
select 22533,'NRL' from dual union all
select 22534,'PRIN' from dual union all
select 22535,'ISAS' from dual union all
select 22535,'RRL' from dual union all
select 22536,'GSOC' from dual union all
select 22537,'ISAS' from dual union all
select 22537,'KYO' from dual union all
select 22538,'GSFC' from dual union all
select 22542,'GSOC' from dual union all
select 22543,'NASDA' from dual union all
select 22544,'GSOC' from dual union all
select 22544,'KIEL' from dual union all
select 22544,'MPE' from dual union all
select 22545,'GSFC' from dual union all
select 22546,'GSFC' from dual union all
select 22547,'STA' from dual union all
select 22548,'GSFC' from dual union all
select 22548,'NDRE' from dual union all
select 22549,'NASDA' from dual union all
select 22550,'UTD' from dual union all
select 22554,'AFCRL' from dual union all
select 22555,'AFCRL' from dual union all
select 22556,'AFCRL' from dual union all
select 22557,'AFCRL' from dual union all
select 22558,'AFCRL' from dual union all
select 22596,'SUPA' from dual union all
select 22601,'INTA' from dual union all
select 22647,'USAF' from dual union all
select 22682,'INTA' from dual union all
select 22686,'USAF' from dual union all
select 22693,'USAF' from dual union all
select 22708,'UCL' from dual union all
select 22719,'INTA' from dual union all
select 22724,'JMA' from dual union all
select 22730,'UCL' from dual union all
select 22737,'USAF' from dual union all
select 22751,'USAF' from dual union all
select 22760,'USAF' from dual union all
select 22778,'USAF' from dual union all
select 22826,'USAF' from dual union all
select 22830,'USAF' from dual union all
select 22836,'PIHM' from dual union all
select 22844,'USAF' from dual union all
select 22850,'USAF' from dual union all
select 22865,'USAF' from dual union all
select 22871,'USAF' from dual union all
select 22874,'USAF' from dual union all
select 22884,'USAF' from dual union all
select 22885,'INTA' from dual union all
select 22888,'PIHM' from dual union all
select 22896,'PIHM' from dual union all
select 22903,'PIHM' from dual union all
select 22908,'UCL' from dual union all
select 22912,'INTA' from dual union all
select 22912,'NASA' from dual union all
select 22916,'USAF' from dual union all
select 22922,'PIHM' from dual union all
select 22930,'MRN' from dual union all
select 22942,'USAF' from dual union all
select 22960,'INTA' from dual union all
select 22960,'NASA' from dual union all
select 22964,'USAF' from dual union all
select 22978,'NASA' from dual union all
select 22979,'NASA' from dual union all
select 22986,'USAF' from dual union all
select 22992,'USAF' from dual union all
select 22995,'USAF' from dual union all
select 22999,'NASA' from dual union all
select 23000,'USAF' from dual union all
select 23019,'USAF' from dual union all
select 23021,'USAF' from dual union all
select 23030,'JMA' from dual union all
select 23031,'INTA' from dual union all
select 23031,'NASA' from dual union all
select 23050,'INTA' from dual union all
select 23050,'NASA' from dual union all
select 23054,'USAF' from dual union all
select 23063,'JMA' from dual union all
select 23069,'USAF' from dual union all
select 23074,'UCL' from dual union all
select 23080,'AFCRL' from dual union all
select 23083,'INTA' from dual union all
select 23083,'NASA' from dual union all
select 23090,'AFCRL' from dual union all
select 23095,'AFCRL' from dual union all
select 23097,'AFCRL' from dual union all
select 23098,'AFCRL' from dual union all
select 23108,'USAF' from dual union all
select 23116,'GSFC' from dual union all
select 23134,'USAF' from dual union all
select 23151,'USAF' from dual union all
select 23161,'GSFC' from dual union all
select 23178,'SUPA' from dual union all
select 23196,'UCL' from dual union all
select 23210,'INTA' from dual union all
select 23210,'NASA' from dual union all
select 23215,'USAF' from dual union all
select 23237,'USAF' from dual union all
select 23241,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 23249,'USAF' from dual union all
select 23257,'USAF' from dual union all
select 23263,'INTA' from dual union all
select 23266,'USAF' from dual union all
select 23278,'USAF' from dual union all
select 23279,'GSFC' from dual union all
select 23288,'USAF' from dual union all
select 23297,'USAF' from dual union all
select 23301,'USAF' from dual union all
select 23306,'GSFC' from dual union all
select 23307,'UCL' from dual union all
select 23312,'USAF' from dual union all
select 23316,'USAF' from dual union all
select 23323,'USAF' from dual union all
select 23330,'USAF' from dual union all
select 23338,'USAF' from dual union all
select 23348,'USAF' from dual union all
select 23359,'USAF' from dual union all
select 23366,'USAF' from dual union all
select 23369,'SUPA' from dual union all
select 23379,'USAF' from dual union all
select 23395,'USAF' from dual union all
select 23396,'USAF' from dual union all
select 23405,'USAF' from dual union all
select 23412,'USAF' from dual union all
select 23417,'USAF' from dual union all
select 23425,'USAF' from dual union all
select 23429,'UCL' from dual union all
select 23439,'USAF' from dual union all
select 23447,'USAF' from dual union all
select 23454,'USAF' from dual union all
select 23464,'USAF' from dual union all
select 23481,'INTA' from dual union all
select 23495,'USAF' from dual union all
select 23503,'USAF' from dual union all
select 23512,'USAF' from dual union all
select 23519,'USAF' from dual union all
select 23525,'USAF' from dual union all
select 23532,'USAF' from dual union all
select 23535,'USAF' from dual union all
select 23538,'USAF' from dual union all
select 23545,'USAF' from dual union all
select 23550,'SUPA' from dual union all
select 23552,'USAF' from dual union all
select 23557,'USAF' from dual union all
select 23560,'JMA' from dual union all
select 23561,'JMA' from dual union all
select 23564,'JMA' from dual union all
select 23565,'UKMO' from dual union all
select 23566,'UKMO' from dual union all
select 23567,'UKMO' from dual union all
select 23568,'UKMO' from dual union all
select 23569,'UKMO' from dual union all
select 23570,'UKMO' from dual union all
select 23571,'UKMO' from dual union all
select 23572,'UKMO' from dual union all
select 23573,'UKMO' from dual union all
select 23575,'UKMO' from dual union all
select 23576,'UKMO' from dual union all
select 23577,'UKMO' from dual union all
select 23583,'USAF' from dual union all
select 23586,'USAF' from dual union all
select 23593,'USAF' from dual union all
select 23604,'USAF' from dual union all
select 23619,'USAF' from dual union all
select 23623,'USAF' from dual union all
select 23626,'USAF' from dual union all
select 23628,'INTA' from dual union all
select 23630,'USAF' from dual union all
select 23631,'SUPA' from dual union all
select 23638,'USAF' from dual union all
select 23641,'USAF' from dual union all
select 23644,'USAF' from dual union all
select 23659,'USAF' from dual union all
select 23665,'USAF' from dual union all
select 23674,'USAF' from dual union all
select 23677,'USAF' from dual union all
select 23684,'HOUS' from dual union all
select 23685,'USAF' from dual union all
select 23691,'INTA' from dual union all
select 23695,'INTA' from dual union all
select 23703,'USAF' from dual union all
select 23710,'USAF' from dual union all
select 23720,'USAF' from dual union all
select 23723,'USAF' from dual union all
select 23724,'HOUS' from dual union all
select 23729,'USAF' from dual union all
select 23736,'INTA' from dual union all
select 23740,'UCL' from dual union all
select 23750,'LARC' from dual union all
select 23752,'PRL' from dual union all
select 23758,'LARC' from dual union all
select 23759,'LARC' from dual union all
select 23760,'LARC' from dual union all
select 23762,'LARC' from dual union all
select 23763,'LARC' from dual union all
select 23764,'LARC' from dual union all
select 23765,'LARC' from dual union all
select 23766,'LARC' from dual union all
select 23767,'PRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 23772,'USAF' from dual union all
select 23775,'USAF' from dual union all
select 23777,'PRL' from dual union all
select 23782,'SUPA' from dual union all
select 23785,'SUPA' from dual union all
select 23791,'USAF' from dual union all
select 23792,'PRL' from dual union all
select 23794,'USAF' from dual union all
select 23804,'USAF' from dual union all
select 23806,'PRL' from dual union all
select 23807,'PRL' from dual union all
select 23813,'PRL' from dual union all
select 23815,'USAF' from dual union all
select 23818,'PRL' from dual union all
select 23825,'USAF' from dual union all
select 23828,'MSFC' from dual union all
select 23830,'PRL' from dual union all
select 23833,'MSFC' from dual union all
select 23835,'PRL' from dual union all
select 23838,'USAF' from dual union all
select 23840,'MSFC' from dual union all
select 23854,'MSFC' from dual union all
select 23860,'USAF' from dual union all
select 23864,'GSFC' from dual union all
select 23868,'MSFC' from dual union all
select 2387,'WFF' from dual union all
select 23872,'SUPA' from dual union all
select 23878,'MSFC' from dual union all
select 23891,'MRN' from dual union all
select 23904,'USAF' from dual union all
select 23908,'MSFC' from dual union all
select 23914,'MSFC' from dual union all
select 23915,'UCL' from dual union all
select 23920,'MSFC' from dual union all
select 23929,'MSFC' from dual union all
select 23960,'SUPA' from dual union all
select 23962,'SUPA' from dual union all
select 23964,'MSFC' from dual union all
select 23973,'HOUS' from dual union all
select 23975,'MSFC' from dual union all
select 23982,'MSFC' from dual union all
select 23984,'MRN' from dual union all
select 23993,'MSFC' from dual union all
select 23999,'MRN' from dual union all
select 24003,'USAF' from dual union all
select 24041,'MRN' from dual union all
select 24058,'UCL' from dual union all
select 24078,'INTA' from dual union all
select 24098,'USAF' from dual union all
select 24114,'INTA' from dual union all
select 24122,'USAF' from dual union all
select 24129,'GSFC' from dual union all
select 24142,'USAF' from dual union all
select 24146,'INTA' from dual union all
select 24161,'INTA' from dual union all
select 24168,'UCL' from dual union all
select 24171,'USAF' from dual union all
select 24187,'USAF' from dual union all
select 24193,'WSMR' from dual union all
select 24202,'WSMR' from dual union all
select 24206,'WSMR' from dual union all
select 24207,'WSMR' from dual union all
select 24210,'WSMR' from dual union all
select 24213,'WSMR' from dual union all
select 24217,'INTA' from dual union all
select 24248,'INTA' from dual union all
select 2425,'SUI' from dual union all
select 24254,'USAF' from dual union all
select 24256,'USAF' from dual union all
select 24267,'INTA' from dual union all
select 24278,'SUPA' from dual union all
select 2428,'NRL' from dual union all
select 24289,'SUPA' from dual union all
select 2429,'SUI' from dual union all
select 24292,'INTA' from dual union all
select 24301,'USAF' from dual union all
select 24311,'SUPA' from dual union all
select 24332,'INTA' from dual union all
select 24335,'SUPA' from dual union all
select 24341,'USAF' from dual union all
select 24358,'GSFC' from dual union all
select 24377,'GSFC' from dual union all
select 24385,'INTA' from dual union all
select 24399,'USAF' from dual union all
select 2440,'SUI' from dual union all
select 24423,'INTA' from dual union all
select 24454,'GSFC' from dual union all
select 24464,'INTA' from dual union all
select 24473,'UCW' from dual union all
select 2448,'SUI' from dual union all
select 24487,'CISIR' from dual union all
select 24487,'GSFC' from dual union all
select 24488,'CISIR' from dual union all
select 24488,'GSFC' from dual union all
select 24489,'CISIR' from dual union all
select 24489,'GSFC' from dual union all
select 24514,'UCW' from dual union all
select 24515,'UCW' from dual union all
select 24517,'MSSL' from dual union all
select 24518,'UCW' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 24519,'MSSL' from dual union all
select 24523,'MSSL' from dual union all
select 24527,'MSSL' from dual union all
select 24532,'MSSL' from dual union all
select 24540,'UCL' from dual union all
select 2456,'NRL' from dual union all
select 24568,'UCW' from dual union all
select 24571,'UCW' from dual union all
select 24648,'PSU' from dual union all
select 24753,'GMS' from dual union all
select 24788,'MSSL' from dual union all
select 24788,'PRL' from dual union all
select 24940,'ISRO' from dual union all
select 24940,'PRL' from dual union all
select 24946,'DLR' from dual union all
select 24948,'UCL' from dual union all
select 24961,'USAF' from dual union all
select 24963,'JMA' from dual union all
select 24964,'JMA' from dual union all
select 24965,'JMA' from dual union all
select 24966,'BERN' from dual union all
select 24966,'LUND' from dual union all
select 24967,'USA' from dual union all
select 24968,'EGAO' from dual union all
select 24969,'UCL' from dual union all
select 24970,'UKMO' from dual union all
select 24971,'UCL' from dual union all
select 24973,'WRE' from dual union all
select 24975,'SAND' from dual union all
select 24976,'SAND' from dual union all
select 24978,'WRE' from dual union all
select 24979,'WRE' from dual union all
select 24980,'WRE' from dual union all
select 24981,'WRE' from dual union all
select 24982,'WRE' from dual union all
select 24983,'WRE' from dual union all
select 25029,'LASL' from dual union all
select 25038,'BAL' from dual union all
select 25040,'BAL' from dual union all
select 25041,'BAL' from dual union all
select 25042,'BAL' from dual union all
select 25043,'BAL' from dual union all
select 25045,'BAL' from dual union all
select 25046,'BAL' from dual union all
select 25047,'BAL' from dual union all
select 25048,'BAL' from dual union all
select 25049,'BAL' from dual union all
select 25050,'BAL' from dual union all
select 25051,'BAL' from dual union all
select 25063,'INTA' from dual union all
select 25065,'USAF' from dual union all
select 25071,'ISAS' from dual union all
select 25074,'USAF' from dual union all
select 25076,'USAF' from dual union all
select 25078,'RSRS' from dual union all
select 25082,'INTA' from dual union all
select 25084,'ISRO' from dual union all
select 25093,'USAF' from dual union all
select 25098,'MPE' from dual union all
select 25114,'STA' from dual union all
select 25115,'STA' from dual union all
select 25116,'GSFC' from dual union all
select 25117,'GSFC' from dual union all
select 25118,'IASB' from dual union all
select 25119,'IASB' from dual union all
select 25120,'IASB' from dual union all
select 25121,'IASB' from dual union all
select 25122,'IASB' from dual union all
select 25124,'DENV' from dual union all
select 25125,'STA' from dual union all
select 25126,'STA' from dual union all
select 25127,'ISAS' from dual union all
select 25130,'RSRS' from dual union all
select 25131,'INTA' from dual union all
select 25132,'INTA' from dual union all
select 25133,'INTA' from dual union all
select 25134,'INTA' from dual union all
select 25134,'NASA' from dual union all
select 25135,'INTA' from dual union all
select 25135,'NASA' from dual union all
select 25136,'INTA' from dual union all
select 25136,'NASA' from dual union all
select 25137,'INTA' from dual union all
select 25137,'NASA' from dual union all
select 25138,'INTA' from dual union all
select 25138,'NASA' from dual union all
select 25139,'INTA' from dual union all
select 25139,'NASA' from dual union all
select 25140,'INTA' from dual union all
select 25140,'NASA' from dual union all
select 25141,'IIAE' from dual union all
select 25142,'CBMET' from dual union all
select 25143,'CBMET' from dual union all
select 25144,'WRE' from dual union all
select 25145,'WRE' from dual union all
select 25146,'WRE' from dual union all
select 25147,'WRE' from dual union all
select 25148,'WRE' from dual union all
select 25149,'WRE' from dual union all
select 25150,'WRE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 25151,'WRE' from dual union all
select 25152,'WRE' from dual union all
select 25156,'USN' from dual union all
select 25157,'USN' from dual union all
select 25163,'MPE' from dual union all
select 25164,'CESR' from dual union all
select 25164,'FREIB' from dual union all
select 25165,'UAF' from dual union all
select 25166,'MSSL' from dual union all
select 25167,'GSFC' from dual union all
select 25168,'LASL' from dual union all
select 25168,'UAF' from dual union all
select 25169,'GSFC' from dual union all
select 25170,'NASDA' from dual union all
select 25171,'UAF' from dual union all
select 25174,'UAF' from dual union all
select 25175,'UAF' from dual union all
select 25179,'CALG' from dual union all
select 25180,'BIR' from dual union all
select 25180,'BONN' from dual union all
select 25180,'RSRS' from dual union all
select 25181,'BIR' from dual union all
select 25181,'BONN' from dual union all
select 25181,'RSRS' from dual union all
select 25182,'BIR' from dual union all
select 25182,'BONN' from dual union all
select 25182,'RSRS' from dual union all
select 25185,'UMI' from dual union all
select 25187,'RICE' from dual union all
select 25190,'CALG' from dual union all
select 25190,'NRCC' from dual union all
select 25191,'CAL' from dual union all
select 25192,'MSSL' from dual union all
select 25194,'ONERA' from dual union all
select 25197,'ARC' from dual union all
select 25198,'NRL' from dual union all
select 25199,'MSSL' from dual union all
select 25201,'USN' from dual union all
select 25202,'USN' from dual union all
select 25203,'UBC' from dual union all
select 25204,'UWI' from dual union all
select 25205,'KPNO' from dual union all
select 25208,'MPE' from dual union all
select 25209,'AN' from dual union all
select 25210,'MSSL' from dual union all
select 25210,'RSRS' from dual union all
select 25210,'UCL' from dual union all
select 25211,'WRE' from dual union all
select 25212,'WRE' from dual union all
select 25213,'NRCC' from dual union all
select 25214,'GSFC' from dual union all
select 25215,'GSFC' from dual union all
select 25216,'UMI' from dual union all
select 25218,'UMI' from dual union all
select 25219,'UMI' from dual union all
select 25220,'GSFC' from dual union all
select 25222,'LUX' from dual union all
select 25222,'MPE' from dual union all
select 25222,'TUBI' from dual union all
select 25223,'LASL' from dual union all
select 25224,'AFCRL' from dual union all
select 25225,'AFCRL' from dual union all
select 25226,'MSSL' from dual union all
select 25227,'RSRS' from dual union all
select 25229,'UCO' from dual union all
select 25231,'GMS' from dual union all
select 25232,'AN' from dual union all
select 25233,'GSFC' from dual union all
select 25234,'GSFC' from dual union all
select 25236,'KTH' from dual union all
select 25237,'AN' from dual union all
select 25240,'UAF' from dual union all
select 25241,'AFCRL' from dual union all
select 25243,'UAF' from dual union all
select 25244,'GMS' from dual union all
select 25245,'GSFC' from dual union all
select 25246,'GMS' from dual union all
select 25247,'GSFC' from dual union all
select 25248,'GSFC' from dual union all
select 25249,'GSFC' from dual union all
select 25257,'UAF' from dual union all
select 25258,'UCSD' from dual union all
select 25259,'GSFC' from dual union all
select 25260,'UWI' from dual union all
select 25262,'AFCRL' from dual union all
select 25263,'AFCRL' from dual union all
select 25264,'YORK' from dual union all
select 25265,'UAF' from dual union all
select 25266,'GSFC' from dual union all
select 25267,'AFCRL' from dual union all
select 25267,'USU' from dual union all
select 25268,'GSFC' from dual union all
select 25277,'AFCRL' from dual union all
select 25278,'KTH' from dual union all
select 25279,'BONN' from dual union all
select 25279,'LUX' from dual union all
select 25279,'RSRS' from dual union all
select 25281,'NRL' from dual union all
select 25282,'UCSD' from dual union all
select 25284,'AN' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 25285,'AN' from dual union all
select 25297,'TRW' from dual union all
select 25298,'AN' from dual union all
select 25299,'AFCRL' from dual union all
select 25302,'SHEF' from dual union all
select 25303,'AFCRL' from dual union all
select 25303,'USU' from dual union all
select 25305,'AFCRL' from dual union all
select 25305,'USU' from dual union all
select 25307,'SOTON' from dual union all
select 25309,'UCL' from dual union all
select 25310,'SOTON' from dual union all
select 25312,'PRL' from dual union all
select 25315,'AFCRL' from dual union all
select 25316,'BIR' from dual union all
select 25317,'UCL' from dual union all
select 25318,'SOTON' from dual union all
select 25318,'SUSX' from dual union all
select 25322,'UCO' from dual union all
select 25323,'DUD' from dual union all
select 25323,'MPIK' from dual union all
select 25324,'SOTON' from dual union all
select 25324,'SUSX' from dual union all
select 25325,'ISAS' from dual union all
select 25325,'PRL' from dual union all
select 25326,'AFCRL' from dual union all
select 25327,'PRL' from dual union all
select 25329,'INTA' from dual union all
select 25330,'INTA' from dual union all
select 25331,'INTA' from dual union all
select 25332,'CNRS' from dual union all
select 25332,'INTA' from dual union all
select 25333,'INTA' from dual union all
select 25337,'AFCRL' from dual union all
select 25337,'USU' from dual union all
select 25339,'LPARL' from dual union all
select 25340,'ISAS' from dual union all
select 25340,'PRL' from dual union all
select 25341,'NIPR' from dual union all
select 25343,'GSFC' from dual union all
select 25344,'MIT' from dual union all
select 25345,'BIR' from dual union all
select 25346,'AFCRL' from dual union all
select 25349,'AFCRL' from dual union all
select 25350,'LUX' from dual union all
select 25357,'MSC' from dual union all
select 25358,'MSC' from dual union all
select 25360,'LUX' from dual union all
select 25360,'RSRS' from dual union all
select 25362,'CNES' from dual union all
select 25363,'AFCRL' from dual union all
select 25364,'AFCRL' from dual union all
select 25366,'AFCRL' from dual union all
select 25367,'AFCRL' from dual union all
select 25368,'AFCRL' from dual union all
select 25369,'AFCRL' from dual union all
select 25371,'AFCRL' from dual union all
select 25372,'AFCRL' from dual union all
select 25373,'WRE' from dual union all
select 25374,'AFCRL' from dual union all
select 25375,'AFCRL' from dual union all
select 25376,'AFCRL' from dual union all
select 25377,'MSC' from dual union all
select 25379,'AFCRL' from dual union all
select 25380,'INTA' from dual union all
select 25383,'LRL' from dual union all
select 25385,'LRL' from dual union all
select 25386,'LEID' from dual union all
select 25386,'NAG' from dual union all
select 25387,'CNRS' from dual union all
select 25387,'INTA' from dual union all
select 25388,'INTA' from dual union all
select 25390,'INTA' from dual union all
select 25391,'CNRS' from dual union all
select 25391,'INTA' from dual union all
select 25393,'CNRS' from dual union all
select 25393,'INTA' from dual union all
select 25394,'INTA' from dual union all
select 25395,'INTA' from dual union all
select 25397,'INTA' from dual union all
select 25400,'PRIN' from dual union all
select 25402,'LASL' from dual union all
select 25407,'AFCRL' from dual union all
select 25408,'APW' from dual union all
select 25408,'BREI' from dual union all
select 25410,'JHU' from dual union all
select 25413,'APW' from dual union all
select 25413,'BREI' from dual union all
select 25415,'UHI' from dual union all
select 25419,'ARC' from dual union all
select 25420,'MSC' from dual union all
select 25422,'NIPR' from dual union all
select 25423,'LPARL' from dual union all
select 25424,'GSFC' from dual union all
select 25426,'GSFC' from dual union all
select 25427,'NOAA' from dual union all
select 25428,'UCW' from dual union all
select 25432,'RSRS' from dual union all
select 25433,'GSFC' from dual union all
select 25434,'RSRS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 25435,'AFCRL' from dual union all
select 25438,'AFCRL' from dual union all
select 25439,'GSFC' from dual union all
select 25440,'MPIH' from dual union all
select 25441,'GSFC' from dual union all
select 25442,'GSFC' from dual union all
select 25444,'GSFC' from dual union all
select 25445,'GSFC' from dual union all
select 25448,'GSFC' from dual union all
select 25450,'GSFC' from dual union all
select 25451,'GSFC' from dual union all
select 25452,'GSFC' from dual union all
select 25453,'UTD' from dual union all
select 25454,'UTD' from dual union all
select 25455,'SOTON' from dual union all
select 25456,'SOTON' from dual union all
select 25457,'CORN' from dual union all
select 25460,'WRE' from dual union all
select 25461,'GSFC' from dual union all
select 25462,'NIPR' from dual union all
select 25463,'UMI' from dual union all
select 25465,'RTG' from dual union all
select 25466,'DUD' from dual union all
select 25466,'MPIK' from dual union all
select 25467,'DUD' from dual union all
select 25467,'MPIK' from dual union all
select 25469,'RTG' from dual union all
select 25472,'CULH' from dual union all
select 25474,'NDRE' from dual union all
select 25475,'NIPR' from dual union all
select 25476,'NRL' from dual union all
select 25477,'GSFC' from dual union all
select 25478,'NRL' from dual union all
select 25483,'ISAS' from dual union all
select 25483,'OSAKA' from dual union all
select 25484,'GSFC' from dual union all
select 25485,'ISAS' from dual union all
select 25487,'AN' from dual union all
select 25488,'ESSA' from dual union all
select 25488,'UMI' from dual union all
select 25489,'GSFC' from dual union all
select 25491,'UIUC' from dual union all
select 25492,'UIUC' from dual union all
select 25493,'UIUC' from dual union all
select 25494,'ISAS' from dual union all
select 25495,'ESSA' from dual union all
select 25495,'UMI' from dual union all
select 25496,'CORN' from dual union all
select 25496,'DMI' from dual union all
select 25500,'CORN' from dual union all
select 25500,'DMI' from dual union all
select 25502,'WRE' from dual union all
select 25503,'ISAS' from dual union all
select 25517,'TAO' from dual union all
select 25524,'GSFC' from dual union all
select 25526,'ISAS' from dual union all
select 25526,'NAG' from dual union all
select 25527,'UCB' from dual union all
select 25528,'UCB' from dual union all
select 25529,'GCA' from dual union all
select 25535,'STA' from dual union all
select 25536,'CNRS' from dual union all
select 25537,'NIPR' from dual union all
select 25538,'LPSP' from dual union all
select 25543,'SAMSO' from dual union all
select 25544,'GCA' from dual union all
select 25546,'NASDA' from dual union all
select 25547,'GSFC' from dual union all
select 25548,'CNRS' from dual union all
select 25549,'CNRS' from dual union all
select 25550,'GSFC' from dual union all
select 25551,'GSFC' from dual union all
select 25552,'WRE' from dual union all
select 25553,'GSFC' from dual union all
select 25554,'GSFC' from dual union all
select 25555,'GSFC' from dual union all
select 25557,'GSFC' from dual union all
select 25558,'GSFC' from dual union all
select 25559,'GSFC' from dual union all
select 25560,'AFCRL' from dual union all
select 25561,'GSFC' from dual union all
select 25562,'GSFC' from dual union all
select 25563,'GSFC' from dual union all
select 25564,'GSFC' from dual union all
select 25565,'RSRS' from dual union all
select 25566,'AN' from dual union all
select 25568,'AN' from dual union all
select 25571,'RSRS' from dual union all
select 25572,'NIPR' from dual union all
select 25575,'LUX' from dual union all
select 25576,'AFCRL' from dual union all
select 25579,'UCW' from dual union all
select 25580,'ROE' from dual union all
select 25584,'AN' from dual union all
select 25591,'AFCRL' from dual union all
select 25593,'AFCRL' from dual union all
select 25594,'AFCRL' from dual union all
select 25595,'AFCRL' from dual union all
select 25596,'AFCRL' from dual union all
select 25597,'AFCRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 25599,'AFCRL' from dual union all
select 25600,'GSFC' from dual union all
select 25601,'AFCRL' from dual union all
select 25603,'AFCRL' from dual union all
select 25604,'AFCRL' from dual union all
select 25608,'AN' from dual union all
select 25609,'MSSL' from dual union all
select 25611,'GSFC' from dual union all
select 25614,'NRL' from dual union all
select 25615,'AN' from dual union all
select 25618,'AN' from dual union all
select 25621,'GSFC' from dual union all
select 25622,'FRASC' from dual union all
select 25622,'ROSAP' from dual union all
select 25623,'GSFC' from dual union all
select 25624,'GSFC' from dual union all
select 25627,'LAS' from dual union all
select 25628,'AFCRL' from dual union all
select 25630,'LASL' from dual union all
select 25630,'UAF' from dual union all
select 25631,'WRE' from dual union all
select 25632,'AFCRL' from dual union all
select 25633,'ISRO' from dual union all
select 25634,'LASL' from dual union all
select 25634,'UAF' from dual union all
select 25635,'GSFC' from dual union all
select 25635,'MSFC' from dual union all
select 25638,'LUX' from dual union all
select 25640,'CIT' from dual union all
select 25641,'LRL' from dual union all
select 25642,'LASL' from dual union all
select 25642,'UAF' from dual union all
select 25644,'LASL' from dual union all
select 25644,'UAF' from dual union all
select 25645,'MSSL' from dual union all
select 25646,'GSFC' from dual union all
select 25647,'AFCRL' from dual union all
select 25652,'AFCRL' from dual union all
select 25654,'AFCRL' from dual union all
select 25657,'LASL' from dual union all
select 25657,'UAF' from dual union all
select 25658,'LASL' from dual union all
select 25658,'UAF' from dual union all
select 25659,'WRE' from dual union all
select 25660,'AFCRL' from dual union all
select 25661,'AFCRL' from dual union all
select 25662,'NRL' from dual union all
select 25664,'KPNO' from dual union all
select 25666,'AFCRL' from dual union all
select 25668,'GSFC' from dual union all
select 25669,'SUPA' from dual union all
select 25669,'UCL' from dual union all
select 25670,'LRL' from dual union all
select 25671,'GSFC' from dual union all
select 25672,'GSFC' from dual union all
select 25674,'LIEGE' from dual union all
select 25674,'RSRS' from dual union all
select 25677,'AFCRL' from dual union all
select 25680,'SUPA' from dual union all
select 25680,'UCL' from dual union all
select 25681,'BIR' from dual union all
select 25681,'RSRS' from dual union all
select 25682,'LUX' from dual union all
select 25684,'RSRS' from dual union all
select 25688,'NIPR' from dual union all
select 25689,'RSRS' from dual union all
select 25690,'GSFC' from dual union all
select 25690,'UMI' from dual union all
select 25691,'GSFC' from dual union all
select 25692,'GSFC' from dual union all
select 25692,'UMI' from dual union all
select 25693,'GSFC' from dual union all
select 25693,'UMI' from dual union all
select 25694,'GSFC' from dual union all
select 25694,'UMI' from dual union all
select 25695,'GSFC' from dual union all
select 25696,'GSFC' from dual union all
select 25697,'GSFC' from dual union all
select 25698,'GSFC' from dual union all
select 25700,'RSRS' from dual union all
select 25704,'RSRS' from dual union all
select 25708,'MSSL' from dual union all
select 25709,'GSFC' from dual union all
select 25710,'BIR' from dual union all
select 25711,'AFCRL' from dual union all
select 25712,'AFCRL' from dual union all
select 25713,'AFCRL' from dual union all
select 25715,'BERN' from dual union all
select 25715,'LUND' from dual union all
select 25719,'WRE' from dual union all
select 25720,'MSC' from dual union all
select 25723,'AN' from dual union all
select 25723,'CNRS' from dual union all
select 25726,'CNIE' from dual union all
select 25728,'AN' from dual union all
select 25731,'GSFC' from dual union all
select 25737,'INTA' from dual union all
select 25738,'GMS' from dual union all
select 25739,'AFCRL' from dual union all
select 25739,'USU' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 25740,'AFCRL' from dual union all
select 25740,'USU' from dual union all
select 25741,'AFCRL' from dual union all
select 25741,'USU' from dual union all
select 25742,'SASK' from dual union all
select 25743,'AFCRL' from dual union all
select 25743,'USU' from dual union all
select 25744,'AFCRL' from dual union all
select 25745,'LASL' from dual union all
select 25746,'UAF' from dual union all
select 25746,'UCB' from dual union all
select 25747,'SAND' from dual union all
select 25750,'WRE' from dual union all
select 25751,'WRE' from dual union all
select 25752,'WRE' from dual union all
select 25753,'MSFC' from dual union all
select 25753,'UMI' from dual union all
select 25754,'WRE' from dual union all
select 25755,'WRE' from dual union all
select 25756,'WRE' from dual union all
select 25757,'WRE' from dual union all
select 25758,'WRE' from dual union all
select 25759,'WRE' from dual union all
select 25760,'WRE' from dual union all
select 25761,'WRE' from dual union all
select 25762,'AN' from dual union all
select 25764,'MSFC' from dual union all
select 25764,'UMI' from dual union all
select 25765,'AFCRL' from dual union all
select 25766,'AFCRL' from dual union all
select 25767,'AFCRL' from dual union all
select 25768,'AFCRL' from dual union all
select 25769,'WSMR' from dual union all
select 25770,'SAND' from dual union all
select 25771,'LASL' from dual union all
select 25772,'BRL' from dual union all
select 25773,'BRL' from dual union all
select 25774,'BRL' from dual union all
select 25775,'ISAS' from dual union all
select 25776,'BRL' from dual union all
select 25777,'BRL' from dual union all
select 25778,'BRL' from dual union all
select 25785,'ISAS' from dual union all
select 25794,'AFCRL' from dual union all
select 25795,'AFCRL' from dual union all
select 25796,'LASL' from dual union all
select 25796,'UAF' from dual union all
select 25797,'ISRO' from dual union all
select 25807,'MPE' from dual union all
select 25818,'MPE' from dual union all
select 25821,'SAND' from dual union all
select 25822,'AFCRL' from dual union all
select 25823,'LASL' from dual union all
select 25823,'UAF' from dual union all
select 25824,'CALG' from dual union all
select 25825,'AFCRL' from dual union all
select 25826,'AFCRL' from dual union all
select 25827,'AFCRL' from dual union all
select 25828,'AFCRL' from dual union all
select 25829,'AFCRL' from dual union all
select 25830,'GSFC' from dual union all
select 25830,'NDRE' from dual union all
select 25831,'GSFC' from dual union all
select 25832,'SASK' from dual union all
select 25833,'KYO' from dual union all
select 25835,'ISAS' from dual union all
select 25836,'GSFC' from dual union all
select 25837,'JHU' from dual union all
select 25838,'KIEL' from dual union all
select 25839,'JHU' from dual union all
select 25840,'GSFC' from dual union all
select 25841,'KIEL' from dual union all
select 25842,'MSSL' from dual union all
select 25843,'YORK' from dual union all
select 25844,'UAF' from dual union all
select 25845,'LASL' from dual union all
select 25845,'UAF' from dual union all
select 25847,'MPAE' from dual union all
select 25849,'PRL' from dual union all
select 25850,'PRL' from dual union all
select 25853,'MPE' from dual union all
select 25854,'USAF' from dual union all
select 25858,'USAF' from dual union all
select 25861,'USAF' from dual union all
select 25869,'PSU' from dual union all
select 25869,'WSMR' from dual union all
select 25873,'USAF' from dual union all
select 25879,'INTA' from dual union all
select 25895,'USAF' from dual union all
select 25897,'USAF' from dual union all
select 25911,'PSU' from dual union all
select 25917,'USAF' from dual union all
select 25933,'USAF' from dual union all
select 25942,'GSFC' from dual union all
select 25948,'GSFC' from dual union all
select 25952,'GSFC' from dual union all
select 25956,'USAF' from dual union all
select 25982,'GSFC' from dual union all
select 25985,'USAF' from dual union all
select 25996,'UCL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 26026,'USAF' from dual union all
select 26042,'USAF' from dual union all
select 2605,'BREI' from dual union all
select 26054,'USAF' from dual union all
select 26065,'USAF' from dual union all
select 26070,'USAF' from dual union all
select 26073,'UKMO' from dual union all
select 26074,'USAF' from dual union all
select 26085,'UKMO' from dual union all
select 26086,'USAF' from dual union all
select 26096,'USAF' from dual union all
select 26110,'INTA' from dual union all
select 26126,'USAF' from dual union all
select 26129,'PSU' from dual union all
select 26129,'WSMR' from dual union all
select 26134,'USAF' from dual union all
select 26140,'WSMR' from dual union all
select 26141,'INTA' from dual union all
select 26144,'USAF' from dual union all
select 26159,'USAF' from dual union all
select 26162,'UKMO' from dual union all
select 26164,'WSMR' from dual union all
select 26168,'USAF' from dual union all
select 26180,'USAF' from dual union all
select 26184,'USAF' from dual union all
select 26193,'INTA' from dual union all
select 26196,'UCL' from dual union all
select 26199,'USAF' from dual union all
select 26204,'INTA' from dual union all
select 26207,'UKMO' from dual union all
select 26214,'INTA' from dual union all
select 26215,'USAF' from dual union all
select 26225,'USAF' from dual union all
select 26229,'UCL' from dual union all
select 26233,'USAF' from dual union all
select 26244,'USAF' from dual union all
select 26250,'INTA' from dual union all
select 26265,'USAF' from dual union all
select 26274,'USAF' from dual union all
select 26278,'PSU' from dual union all
select 26281,'GSFC' from dual union all
select 26286,'INTA' from dual union all
select 26293,'USAF' from dual union all
select 26299,'USAF' from dual union all
select 26309,'USAF' from dual union all
select 26310,'INTA' from dual union all
select 26318,'USAF' from dual union all
select 26321,'USAF' from dual union all
select 26327,'USAF' from dual union all
select 26329,'UKMO' from dual union all
select 26337,'USAF' from dual union all
select 26338,'INTA' from dual union all
select 26342,'USAF' from dual union all
select 26349,'USAF' from dual union all
select 26357,'USAF' from dual union all
select 26361,'USAF' from dual union all
select 26362,'USAF' from dual union all
select 26363,'WSMR' from dual union all
select 26367,'USAF' from dual union all
select 26371,'INTA' from dual union all
select 26373,'INTA' from dual union all
select 26376,'USAF' from dual union all
select 26383,'PSU' from dual union all
select 26383,'WSMR' from dual union all
select 26388,'USAF' from dual union all
select 26401,'USAF' from dual union all
select 26412,'USAF' from dual union all
select 26416,'USAF' from dual union all
select 26418,'USAF' from dual union all
select 26426,'USAF' from dual union all
select 26432,'INTA' from dual union all
select 26435,'USAF' from dual union all
select 26439,'USAF' from dual union all
select 26441,'USAF' from dual union all
select 26452,'USAF' from dual union all
select 26456,'USAF' from dual union all
select 26468,'USAF' from dual union all
select 26476,'USAF' from dual union all
select 26481,'USAF' from dual union all
select 26488,'USAF' from dual union all
select 26495,'USAF' from dual union all
select 26504,'USAF' from dual union all
select 26510,'USAF' from dual union all
select 26515,'USAF' from dual union all
select 26516,'UKMO' from dual union all
select 26521,'USAF' from dual union all
select 26525,'USAF' from dual union all
select 26532,'USAF' from dual union all
select 26536,'INTA' from dual union all
select 26542,'USAF' from dual union all
select 26544,'USAF' from dual union all
select 26549,'USAF' from dual union all
select 26553,'USAF' from dual union all
select 26554,'USAF' from dual union all
select 26557,'USAF' from dual union all
select 26569,'USAF' from dual union all
select 26575,'INTA' from dual union all
select 26577,'USAF' from dual union all
select 26584,'USAF' from dual union all
select 26590,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 26595,'USAF' from dual union all
select 26598,'INTA' from dual union all
select 26601,'USAF' from dual union all
select 26604,'INTA' from dual union all
select 26606,'USAF' from dual union all
select 26608,'USAF' from dual union all
select 26612,'USAF' from dual union all
select 26618,'USAF' from dual union all
select 26620,'USAF' from dual union all
select 26624,'USAF' from dual union all
select 26626,'USAF' from dual union all
select 26629,'USAF' from dual union all
select 26630,'JMA' from dual union all
select 26631,'JMA' from dual union all
select 26632,'JMA' from dual union all
select 26633,'JMA' from dual union all
select 26634,'JMA' from dual union all
select 26635,'JMA' from dual union all
select 26636,'JMA' from dual union all
select 26637,'JMA' from dual union all
select 26638,'USAF' from dual union all
select 26640,'JMA' from dual union all
select 26641,'JMA' from dual union all
select 26642,'JMA' from dual union all
select 26645,'UCL' from dual union all
select 26649,'UCL' from dual union all
select 26651,'UKMO' from dual union all
select 26654,'USAF' from dual union all
select 26660,'PSU' from dual union all
select 26660,'WSMR' from dual union all
select 26668,'UKMO' from dual union all
select 26672,'USAF' from dual union all
select 26676,'UCL' from dual union all
select 26681,'UKMO' from dual union all
select 26685,'UCL' from dual union all
select 26689,'UKMO' from dual union all
select 26698,'USAF' from dual union all
select 26699,'UKMO' from dual union all
select 26705,'USAF' from dual union all
select 26711,'UCL' from dual union all
select 26714,'UKMO' from dual union all
select 26715,'UKMO' from dual union all
select 26724,'USAF' from dual union all
select 26730,'USAF' from dual union all
select 26731,'UCL' from dual union all
select 26737,'USAF' from dual union all
select 26738,'UCL' from dual union all
select 26744,'USAF' from dual union all
select 26747,'USAF' from dual union all
select 26750,'INTA' from dual union all
select 26762,'WSMR' from dual union all
select 26768,'HOUS' from dual union all
select 26769,'USAF' from dual union all
select 26773,'USAF' from dual union all
select 26781,'INTA' from dual union all
select 26784,'WSMR' from dual union all
select 26790,'GSFC' from dual union all
select 26813,'HOUS' from dual union all
select 26814,'INTA' from dual union all
select 26819,'USAF' from dual union all
select 26821,'GSFC' from dual union all
select 26823,'USAF' from dual union all
select 26832,'USAF' from dual union all
select 26835,'GSFC' from dual union all
select 26846,'USAF' from dual union all
select 26849,'USAF' from dual union all
select 26858,'WSMR' from dual union all
select 26875,'INTA' from dual union all
select 26883,'USAF' from dual union all
select 26890,'UNK' from dual union all
select 26891,'UNK' from dual union all
select 26899,'USAF' from dual union all
select 26900,'USAF' from dual union all
select 26901,'INTA' from dual union all
select 26911,'USAF' from dual union all
select 26914,'WSMR' from dual union all
select 26916,'WSMR' from dual union all
select 26919,'USAF' from dual union all
select 26924,'USAF' from dual union all
select 26936,'USAF' from dual union all
select 26947,'USAF' from dual union all
select 26949,'USAF' from dual union all
select 26963,'USAF' from dual union all
select 26973,'INTA' from dual union all
select 26993,'INTA' from dual union all
select 27014,'USAF' from dual union all
select 27029,'USAF' from dual union all
select 27048,'USAF' from dual union all
select 27057,'USAF' from dual union all
select 27072,'USAF' from dual union all
select 27081,'USAF' from dual union all
select 27083,'USAF' from dual union all
select 27099,'INTA' from dual union all
select 27106,'USAF' from dual union all
select 27113,'USAF' from dual union all
select 27116,'USAF' from dual union all
select 27124,'USAF' from dual union all
select 27125,'USAF' from dual union all
select 27135,'INTA' from dual union all
select 27152,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 27161,'USAF' from dual union all
select 27176,'USAF' from dual union all
select 27193,'USAF' from dual union all
select 27209,'USAF' from dual union all
select 27218,'INTA' from dual union all
select 27223,'USAF' from dual union all
select 27235,'UCL' from dual union all
select 27257,'USAF' from dual union all
select 27261,'USAF' from dual union all
select 27282,'USAF' from dual union all
select 27286,'USAF' from dual union all
select 27287,'USAF' from dual union all
select 27290,'USAF' from dual union all
select 27298,'USAF' from dual union all
select 27337,'PSU' from dual union all
select 27337,'WSMR' from dual union all
select 27338,'USAF' from dual union all
select 27369,'INTA' from dual union all
select 27381,'ARC' from dual union all
select 27383,'ARC' from dual union all
select 2739,'LRBA' from dual union all
select 27396,'USAF' from dual union all
select 27403,'USAF' from dual union all
select 27427,'USAF' from dual union all
select 27433,'USAF' from dual union all
select 27439,'INTA' from dual union all
select 27443,'USAF' from dual union all
select 27457,'USAF' from dual union all
select 27475,'USAF' from dual union all
select 27477,'UCL' from dual union all
select 27492,'INTA' from dual union all
select 27512,'USAF' from dual union all
select 2752,'NMSU' from dual union all
select 27521,'UKMO' from dual union all
select 27523,'USAF' from dual union all
select 27543,'INTA' from dual union all
select 27581,'PRL' from dual union all
select 2760,'NMSU' from dual union all
select 2762,'NMSU' from dual union all
select 2763,'NMSU' from dual union all
select 2764,'NMSU' from dual union all
select 2765,'NMSU' from dual union all
select 27659,'ARC' from dual union all
select 2766,'NMSU' from dual union all
select 27660,'ARC' from dual union all
select 27675,'UCW' from dual union all
select 27692,'UCW' from dual union all
select 27708,'UCW' from dual union all
select 27733,'GSFC' from dual union all
select 27757,'UCW' from dual union all
select 27761,'UKMO' from dual union all
select 27800,'UCW' from dual union all
select 27831,'UCW' from dual union all
select 27838,'UKMO' from dual union all
select 27879,'GSFC' from dual union all
select 28039,'WRE' from dual union all
select 28041,'PRL' from dual union all
select 28041,'SUSX' from dual union all
select 28044,'PRL' from dual union all
select 28044,'SUSX' from dual union all
select 28045,'PRL' from dual union all
select 28045,'SOTON' from dual union all
select 28048,'BAL' from dual union all
select 28053,'ERNO' from dual union all
select 28055,'LASL' from dual union all
select 28055,'UAF' from dual union all
select 28056,'GCA' from dual union all
select 28058,'ERNO' from dual union all
select 28059,'BAL' from dual union all
select 28066,'AFCRL' from dual union all
select 28070,'BAL' from dual union all
select 28074,'SAND' from dual union all
select 28075,'SAND' from dual union all
select 28076,'WFF' from dual union all
select 28077,'SAND' from dual union all
select 28078,'WFF' from dual union all
select 28080,'BAL' from dual union all
select 28081,'SDCO' from dual union all
select 28082,'WRE' from dual union all
select 28083,'SAND' from dual union all
select 28084,'WRE' from dual union all
select 28085,'WRE' from dual union all
select 2809,'SUI' from dual union all
select 28090,'UMN' from dual union all
select 28092,'BIR' from dual union all
select 28096,'LARC' from dual union all
select 2810,'PARD' from dual union all
select 28102,'MPE' from dual union all
select 28102,'NDRE' from dual union all
select 28103,'DENV' from dual union all
select 28108,'INTA' from dual union all
select 28115,'INTA' from dual union all
select 2812,'NRL' from dual union all
select 2813,'LRBA' from dual union all
select 28142,'AFCRL' from dual union all
select 28148,'BAL' from dual union all
select 28149,'BAL' from dual union all
select 28150,'BAL' from dual union all
select 28153,'AFCRL' from dual union all
select 28156,'RAE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 28161,'GSFC' from dual union all
select 28162,'WFF' from dual union all
select 28170,'USAF' from dual union all
select 2818,'NRL' from dual union all
select 28183,'PRL' from dual union all
select 28183,'SUSX' from dual union all
select 28184,'MPE' from dual union all
select 28184,'PRL' from dual union all
select 28184,'UCB' from dual union all
select 28185,'MPE' from dual union all
select 28185,'PRL' from dual union all
select 28185,'UCB' from dual union all
select 28186,'DENV' from dual union all
select 28193,'AFCRL' from dual union all
select 28194,'AFCRL' from dual union all
select 28196,'CNES' from dual union all
select 28198,'CNES' from dual union all
select 28200,'INTA' from dual union all
select 28200,'NASA' from dual union all
select 28201,'INTA' from dual union all
select 28201,'NASA' from dual union all
select 28202,'INTA' from dual union all
select 28203,'INTA' from dual union all
select 28204,'INTA' from dual union all
select 28205,'INTA' from dual union all
select 28206,'GSFC' from dual union all
select 28207,'JMA' from dual union all
select 28208,'JMA' from dual union all
select 28220,'AFCRL' from dual union all
select 28221,'GSFC' from dual union all
select 28222,'LASL' from dual union all
select 28222,'UAF' from dual union all
select 28223,'GSFC' from dual union all
select 28224,'MPE' from dual union all
select 28225,'GMS' from dual union all
select 28226,'AFCRL' from dual union all
select 28227,'UNH' from dual union all
select 28228,'GSFC' from dual union all
select 28230,'CRA' from dual union all
select 28232,'CRA' from dual union all
select 28233,'CRA' from dual union all
select 28235,'MSSL' from dual union all
select 28235,'UCL' from dual union all
select 28236,'LASL' from dual union all
select 28237,'AFCRL' from dual union all
select 28237,'USU' from dual union all
select 28238,'CRA' from dual union all
select 28239,'CULH' from dual union all
select 28241,'JHU' from dual union all
select 28242,'LASL' from dual union all
select 28245,'GMS' from dual union all
select 28248,'ONERA' from dual union all
select 28249,'USU' from dual union all
select 28253,'YORK' from dual union all
select 28254,'CAL' from dual union all
select 28255,'CRA' from dual union all
select 28256,'WRE' from dual union all
select 28257,'UTOR' from dual union all
select 28258,'BONN' from dual union all
select 28259,'NRL' from dual union all
select 28260,'RAE' from dual union all
select 28261,'NRL' from dual union all
select 28262,'AFCRL' from dual union all
select 28263,'NRL' from dual union all
select 28265,'BONN' from dual union all
select 28267,'UMI' from dual union all
select 28268,'NRL' from dual union all
select 28269,'ASE' from dual union all
select 28270,'NRL' from dual union all
select 28271,'NRL' from dual union all
select 28272,'AFCRL' from dual union all
select 28273,'AFCRL' from dual union all
select 28274,'GMS' from dual union all
select 28275,'AFCRL' from dual union all
select 28276,'PRL' from dual union all
select 28277,'SUPA' from dual union all
select 28278,'SUPA' from dual union all
select 28279,'AFCRL' from dual union all
select 28281,'AFCRL' from dual union all
select 28283,'AFCRL' from dual union all
select 28284,'LAS' from dual union all
select 28285,'AFCRL' from dual union all
select 28286,'AFCRL' from dual union all
select 28287,'AFCRL' from dual union all
select 28288,'AFCRL' from dual union all
select 28289,'AFCRL' from dual union all
select 28290,'AFCRL' from dual union all
select 28291,'AFCRL' from dual union all
select 28292,'MPAE' from dual union all
select 28293,'AFCRL' from dual union all
select 28294,'AFCRL' from dual union all
select 28296,'AFCRL' from dual union all
select 28297,'NIPR' from dual union all
select 28300,'WRE' from dual union all
select 28301,'MPAE' from dual union all
select 28302,'MPAE' from dual union all
select 28303,'NCAR' from dual union all
select 28305,'LPARL' from dual union all
select 28307,'SUPA' from dual union all
select 28308,'MPAE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 28313,'AFCRL' from dual union all
select 28314,'AFCRL' from dual union all
select 28315,'MPAE' from dual union all
select 28318,'MPAE' from dual union all
select 28319,'MPAE' from dual union all
select 28321,'NRL' from dual union all
select 28322,'NIPR' from dual union all
select 28323,'NIPR' from dual union all
select 28324,'LASL' from dual union all
select 28325,'NRCC' from dual union all
select 28326,'GSFC' from dual union all
select 28327,'MPAE' from dual union all
select 28329,'GSFC' from dual union all
select 28330,'UMI' from dual union all
select 28331,'GSFC' from dual union all
select 28332,'WRE' from dual union all
select 28333,'MIT' from dual union all
select 28334,'GSFC' from dual union all
select 28335,'LRL' from dual union all
select 28336,'LEID' from dual union all
select 28336,'NAG' from dual union all
select 28338,'MPAE' from dual union all
select 28341,'ADEL' from dual union all
select 28343,'WSMR' from dual union all
select 28358,'NDRE' from dual union all
select 28358,'UMD' from dual union all
select 28359,'NDRE' from dual union all
select 28359,'UMD' from dual union all
select 28360,'MPAE' from dual union all
select 28362,'UCO' from dual union all
select 28363,'LRL' from dual union all
select 28365,'MPAE' from dual union all
select 28366,'UCB' from dual union all
select 28367,'GSFC' from dual union all
select 28368,'CIT' from dual union all
select 28371,'CHIC' from dual union all
select 28373,'MPAE' from dual union all
select 28374,'WRE' from dual union all
select 28378,'MPAE' from dual union all
select 28379,'MSC' from dual union all
select 28382,'MSSL' from dual union all
select 28384,'CAL' from dual union all
select 28385,'GSFC' from dual union all
select 28385,'INTA' from dual union all
select 28386,'RAE' from dual union all
select 28387,'MPAE' from dual union all
select 28389,'GSFC' from dual union all
select 28389,'INTA' from dual union all
select 28390,'SASK' from dual union all
select 28391,'LASL' from dual union all
select 28392,'SASK' from dual union all
select 28393,'SASK' from dual union all
select 28395,'SASK' from dual union all
select 28396,'CRC' from dual union all
select 28396,'RSRS' from dual union all
select 28397,'CRC' from dual union all
select 28397,'RSRS' from dual union all
select 28398,'CRC' from dual union all
select 28398,'RSRS' from dual union all
select 28399,'CRC' from dual union all
select 28399,'RSRS' from dual union all
select 28400,'NRL' from dual union all
select 28401,'ASE' from dual union all
select 28402,'MPAE' from dual union all
select 28404,'UCO' from dual union all
select 28407,'CORN' from dual union all
select 28409,'HCO' from dual union all
select 28410,'AFCRL' from dual union all
select 28414,'GSFC' from dual union all
select 28415,'ASE' from dual union all
select 28417,'NIPR' from dual union all
select 28419,'MIT' from dual union all
select 28420,'UWI' from dual union all
select 28421,'NIPR' from dual union all
select 28423,'PRL' from dual union all
select 28424,'PRL' from dual union all
select 28430,'AFCRL' from dual union all
select 28431,'AFCRL' from dual union all
select 28433,'BIR' from dual union all
select 28434,'RSRS' from dual union all
select 28435,'RSRS' from dual union all
select 28439,'RSRS' from dual union all
select 28440,'RSRS' from dual union all
select 28441,'JHU' from dual union all
select 28442,'GSFC' from dual union all
select 28443,'ISAS' from dual union all
select 28446,'MIT' from dual union all
select 28451,'ISAS' from dual union all
select 28452,'AFCRL' from dual union all
select 28453,'RSRS' from dual union all
select 28455,'UMN' from dual union all
select 28456,'STA' from dual union all
select 28459,'AFCRL' from dual union all
select 28460,'PRL' from dual union all
select 28460,'SUSX' from dual union all
select 28462,'GSFC' from dual union all
select 2847,'NRL' from dual union all
select 28473,'BIR' from dual union all
select 28473,'SOTON' from dual union all
select 28474,'BIR' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 28474,'SOTON' from dual union all
select 28475,'DUD' from dual union all
select 28475,'SSC' from dual union all
select 28476,'LASL' from dual union all
select 28476,'UAF' from dual union all
select 28477,'DUD' from dual union all
select 28477,'SSC' from dual union all
select 28478,'SOTON' from dual union all
select 2848,'AFCRL' from dual union all
select 28482,'PRL' from dual union all
select 28482,'SUSX' from dual union all
select 28483,'NRL' from dual union all
select 28484,'AFCRL' from dual union all
select 28485,'SUSX' from dual union all
select 28486,'DUD' from dual union all
select 28486,'SSC' from dual union all
select 28487,'MSSL' from dual union all
select 28488,'PRL' from dual union all
select 28489,'GCA' from dual union all
select 28490,'GCA' from dual union all
select 28491,'PRL' from dual union all
select 28492,'NOAA' from dual union all
select 28493,'PRL' from dual union all
select 28493,'SUSX' from dual union all
select 28495,'QUB' from dual union all
select 28495,'SHEF' from dual union all
select 28496,'MPAE' from dual union all
select 28498,'MPAE' from dual union all
select 28499,'LRL' from dual union all
select 28500,'LRL' from dual union all
select 28501,'UMN' from dual union all
select 28502,'BIR' from dual union all
select 28502,'SOTON' from dual union all
select 28503,'MO' from dual union all
select 28503,'MSSL' from dual union all
select 28504,'NOAA' from dual union all
select 28505,'SUSX' from dual union all
select 28506,'MSSL' from dual union all
select 28507,'GSFC' from dual union all
select 28507,'HCO' from dual union all
select 28508,'MPAE' from dual union all
select 2851,'NRL' from dual union all
select 28512,'MSSL' from dual union all
select 28515,'NDRE' from dual union all
select 28517,'SUSX' from dual union all
select 28518,'CHIC' from dual union all
select 28519,'GMS' from dual union all
select 2852,'FIAN' from dual union all
select 28521,'AFCRL' from dual union all
select 28522,'AFCRL' from dual union all
select 28523,'AFCRL' from dual union all
select 28524,'AFCRL' from dual union all
select 28526,'PRL' from dual union all
select 28526,'SUSX' from dual union all
select 28527,'UIUC' from dual union all
select 28528,'NRL' from dual union all
select 28529,'AFCRL' from dual union all
select 28534,'UTD' from dual union all
select 28535,'CEA' from dual union all
select 28535,'UTR' from dual union all
select 28536,'MPE' from dual union all
select 28537,'NRCC' from dual union all
select 28539,'SOTON' from dual union all
select 2854,'AFCRL' from dual union all
select 2854,'UCO' from dual union all
select 28540,'UCO' from dual union all
select 28541,'MPE' from dual union all
select 28542,'AFCRL' from dual union all
select 28543,'GSFC' from dual union all
select 28544,'BIR' from dual union all
select 28544,'SOTON' from dual union all
select 28545,'MPE' from dual union all
select 28546,'GSFC' from dual union all
select 28547,'NRCC' from dual union all
select 28549,'GSFC' from dual union all
select 28553,'CRA' from dual union all
select 28556,'WSMR' from dual union all
select 28557,'UBC' from dual union all
select 28558,'NRCC' from dual union all
select 28561,'GMS' from dual union all
select 28563,'AFCRL' from dual union all
select 28564,'AFCRL' from dual union all
select 28565,'UIUC' from dual union all
select 28566,'LUX' from dual union all
select 28567,'AFCRL' from dual union all
select 28568,'AFCRL' from dual union all
select 28569,'RSRS' from dual union all
select 28571,'UWI' from dual union all
select 28572,'ADEL' from dual union all
select 28572,'QUB' from dual union all
select 28572,'SHEF' from dual union all
select 28577,'AFCRL' from dual union all
select 28578,'AFCRL' from dual union all
select 28579,'AFCRL' from dual union all
select 2858,'FIAN' from dual union all
select 2858,'IAM' from dual union all
select 28580,'JHU' from dual union all
select 28583,'LPSP' from dual union all
select 28584,'BERN' from dual union all
select 28584,'FREIB' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 28585,'UCO' from dual union all
select 28586,'NIPR' from dual union all
select 2859,'FIAN' from dual union all
select 2859,'IAM' from dual union all
select 28599,'AFCRL' from dual union all
select 2860,'FIAN' from dual union all
select 2860,'IAM' from dual union all
select 28600,'TSAO' from dual union all
select 28601,'ISAS' from dual union all
select 28602,'GMS' from dual union all
select 28603,'GMS' from dual union all
select 28607,'SDCO' from dual union all
select 28608,'SDCO' from dual union all
select 28609,'SDCO' from dual union all
select 2861,'UMI' from dual union all
select 28610,'SDCO' from dual union all
select 28611,'SDCO' from dual union all
select 28612,'KGO' from dual union all
select 28612,'RSRS' from dual union all
select 28613,'SDCO' from dual union all
select 28614,'SDCO' from dual union all
select 28616,'ADEL' from dual union all
select 28617,'ADEL' from dual union all
select 28618,'WRE' from dual union all
select 28619,'WRE' from dual union all
select 2862,'SUI' from dual union all
select 28620,'WRE' from dual union all
select 28621,'WRE' from dual union all
select 28622,'WRE' from dual union all
select 28624,'WRE' from dual union all
select 28625,'WRE' from dual union all
select 28626,'WRE' from dual union all
select 28627,'WRE' from dual union all
select 28628,'LPSP' from dual union all
select 28629,'CNES' from dual union all
select 2863,'NRL' from dual union all
select 28630,'CNES' from dual union all
select 28631,'CNES' from dual union all
select 28632,'CNES' from dual union all
select 28633,'CNES' from dual union all
select 28634,'KGO' from dual union all
select 28634,'RSRS' from dual union all
select 2864,'SUI' from dual union all
select 28645,'RSRS' from dual union all
select 2865,'NRL' from dual union all
select 28652,'ISAS' from dual union all
select 28654,'GSFC' from dual union all
select 28654,'MSFC' from dual union all
select 28655,'AERO' from dual union all
select 28656,'ISAS' from dual union all
select 28657,'AERO' from dual union all
select 28659,'UIUC' from dual union all
select 2866,'SUI' from dual union all
select 28660,'AFCRL' from dual union all
select 28661,'AFCRL' from dual union all
select 28662,'PRL' from dual union all
select 28662,'RSRS' from dual union all
select 28663,'RICE' from dual union all
select 28664,'WRE' from dual union all
select 28665,'MPE' from dual union all
select 28668,'PRL' from dual union all
select 28668,'RSRS' from dual union all
select 28669,'NASDA' from dual union all
select 2867,'NRL' from dual union all
select 28670,'AFCRL' from dual union all
select 28671,'RTG' from dual union all
select 28672,'SASK' from dual union all
select 28674,'TRW' from dual union all
select 28677,'NIPR' from dual union all
select 2868,'SUI' from dual union all
select 28681,'MPAE' from dual union all
select 28682,'MPAE' from dual union all
select 28683,'WRE' from dual union all
select 28684,'PRL' from dual union all
select 28684,'RSRS' from dual union all
select 2869,'SUI' from dual union all
select 28693,'NDRE' from dual union all
select 28695,'SASK' from dual union all
select 28696,'PITT' from dual union all
select 28697,'ISAS' from dual union all
select 28698,'UCW' from dual union all
select 28699,'GMS' from dual union all
select 2870,'SUI' from dual union all
select 28700,'ISAS' from dual union all
select 28701,'UWI' from dual union all
select 28702,'ISAS' from dual union all
select 28704,'GMS' from dual union all
select 28705,'ISAS' from dual union all
select 28705,'TOK' from dual union all
select 28706,'WRE' from dual union all
select 28707,'RICE' from dual union all
select 28708,'IASB' from dual union all
select 28709,'IASB' from dual union all
select 2871,'SUI' from dual union all
select 28711,'MPE' from dual union all
select 28711,'NRL' from dual union all
select 28711,'TUB' from dual union all
select 28712,'KGO' from dual union all
select 28712,'RSRS' from dual union all
select 28714,'GMS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 28715,'GMS' from dual union all
select 28719,'AFCRL' from dual union all
select 2872,'SUI' from dual union all
select 28720,'AFCRL' from dual union all
select 28721,'AFCRL' from dual union all
select 28722,'AFCRL' from dual union all
select 28729,'INTA' from dual union all
select 2873,'SUI' from dual union all
select 2874,'NRL' from dual union all
select 28747,'USAF' from dual union all
select 2875,'NRL' from dual union all
select 28757,'USAF' from dual union all
select 28762,'USAF' from dual union all
select 28774,'USAF' from dual union all
select 28782,'USAF' from dual union all
select 28785,'INTA' from dual union all
select 2879,'AFCRL' from dual union all
select 2879,'UUT' from dual union all
select 28793,'USAF' from dual union all
select 28802,'USAF' from dual union all
select 28810,'USAF' from dual union all
select 28817,'USAF' from dual union all
select 28820,'USAF' from dual union all
select 28830,'USAF' from dual union all
select 28837,'NASDA' from dual union all
select 28838,'NASDA' from dual union all
select 28839,'USAF' from dual union all
select 28845,'USAF' from dual union all
select 28847,'INTA' from dual union all
select 28852,'USAF' from dual union all
select 2887,'AFCRL' from dual union all
select 2887,'RIU' from dual union all
select 28871,'USAF' from dual union all
select 28888,'USAF' from dual union all
select 28900,'USAF' from dual union all
select 28908,'USAF' from dual union all
select 28920,'USAF' from dual union all
select 28924,'INTA' from dual union all
select 2893,'NRL' from dual union all
select 28934,'USAF' from dual union all
select 28939,'INTA' from dual union all
select 2894,'PARD' from dual union all
select 28940,'USAF' from dual union all
select 28949,'INTA' from dual union all
select 28956,'USAF' from dual union all
select 2896,'LPA' from dual union all
select 28961,'USAF' from dual union all
select 28970,'USAF' from dual union all
select 28984,'USAF' from dual union all
select 28990,'USAF' from dual union all
select 28999,'USAF' from dual union all
select 29011,'WSMR' from dual union all
select 29014,'USAF' from dual union all
select 29022,'USAF' from dual union all
select 29025,'USAF' from dual union all
select 29035,'USAF' from dual union all
select 29051,'USAF' from dual union all
select 29056,'USAF' from dual union all
select 29064,'USAF' from dual union all
select 29081,'USAF' from dual union all
select 29083,'INTA' from dual union all
select 29095,'USAF' from dual union all
select 29096,'USAF' from dual union all
select 29100,'PSU' from dual union all
select 29112,'HOUS' from dual union all
select 29113,'USAF' from dual union all
select 29119,'USAF' from dual union all
select 29128,'USAF' from dual union all
select 29133,'USAF' from dual union all
select 29138,'USAF' from dual union all
select 29139,'WSMR' from dual union all
select 29148,'USAF' from dual union all
select 29150,'USAF' from dual union all
select 29157,'USAF' from dual union all
select 29160,'WSMR' from dual union all
select 29173,'USAF' from dual union all
select 29176,'USAF' from dual union all
select 29184,'USAF' from dual union all
select 29191,'USAF' from dual union all
select 29201,'USAF' from dual union all
select 29209,'USAF' from dual union all
select 29215,'USAF' from dual union all
select 29221,'USAF' from dual union all
select 29222,'USAF' from dual union all
select 29232,'USAF' from dual union all
select 29233,'USAF' from dual union all
select 29234,'INTA' from dual union all
select 29242,'USAF' from dual union all
select 29243,'USAF' from dual union all
select 29251,'USAF' from dual union all
select 29258,'USAF' from dual union all
select 29259,'USAF' from dual union all
select 29264,'USAF' from dual union all
select 29265,'INTA' from dual union all
select 29273,'USAF' from dual union all
select 29274,'USAF' from dual union all
select 29276,'USAF' from dual union all
select 29279,'USAF' from dual union all
select 29287,'USAF' from dual union all
select 29288,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 29289,'USAF' from dual union all
select 29290,'USAF' from dual union all
select 29300,'USAF' from dual union all
select 29306,'USAF' from dual union all
select 29311,'USAF' from dual union all
select 29315,'USAF' from dual union all
select 29323,'USAF' from dual union all
select 29331,'USAF' from dual union all
select 29336,'USAF' from dual union all
select 29339,'USAF' from dual union all
select 29344,'USAF' from dual union all
select 29346,'USAF' from dual union all
select 29352,'USAF' from dual union all
select 29359,'USAF' from dual union all
select 29361,'USAF' from dual union all
select 29365,'UKMO' from dual union all
select 29368,'USAF' from dual union all
select 29372,'USAF' from dual union all
select 29374,'WSMR' from dual union all
select 29377,'WSMR' from dual union all
select 29380,'NASDA' from dual union all
select 29381,'INTA' from dual union all
select 29384,'USAF' from dual union all
select 29389,'USAF' from dual union all
select 29394,'USAF' from dual union all
select 29402,'NASDA' from dual union all
select 29403,'USAF' from dual union all
select 29413,'USAF' from dual union all
select 29417,'INTA' from dual union all
select 29430,'INTA' from dual union all
select 29431,'USAF' from dual union all
select 29436,'USAF' from dual union all
select 29437,'INTA' from dual union all
select 29444,'WSMR' from dual union all
select 29450,'USAF' from dual union all
select 29456,'USAF' from dual union all
select 29459,'INTA' from dual union all
select 29460,'USAF' from dual union all
select 29463,'USAF' from dual union all
select 29465,'INTA' from dual union all
select 29467,'PSU' from dual union all
select 29472,'USAF' from dual union all
select 29476,'USAF' from dual union all
select 29480,'INTA' from dual union all
select 29484,'USAF' from dual union all
select 29491,'USAF' from dual union all
select 29495,'INTA' from dual union all
select 29498,'USAF' from dual union all
select 29506,'USAF' from dual union all
select 29531,'USAF' from dual union all
select 29537,'USAF' from dual union all
select 29542,'USAF' from dual union all
select 29551,'WSMR' from dual union all
select 29559,'USAF' from dual union all
select 29568,'USAF' from dual union all
select 29579,'USAF' from dual union all
select 29585,'INTA' from dual union all
select 29590,'WFF' from dual union all
select 29594,'CNES' from dual union all
select 29598,'USAF' from dual union all
select 29602,'USAF' from dual union all
select 29618,'USAF' from dual union all
select 29630,'WFF' from dual union all
select 29632,'WFF' from dual union all
select 29633,'USAF' from dual union all
select 29634,'INTA' from dual union all
select 29646,'USAF' from dual union all
select 29656,'USAF' from dual union all
select 29666,'USAF' from dual union all
select 29676,'USAF' from dual union all
select 29680,'USAF' from dual union all
select 29699,'INTA' from dual union all
select 29709,'USAF' from dual union all
select 29718,'USAF' from dual union all
select 29731,'USAF' from dual union all
select 29732,'INTA' from dual union all
select 29740,'USAF' from dual union all
select 29749,'USAF' from dual union all
select 29773,'USAF' from dual union all
select 29782,'USAF' from dual union all
select 29793,'USAF' from dual union all
select 29806,'USAF' from dual union all
select 29832,'INTA' from dual union all
select 29835,'USAF' from dual union all
select 29840,'INTA' from dual union all
select 29846,'GSFC' from dual union all
select 29847,'INTA' from dual union all
select 29858,'USAF' from dual union all
select 29867,'USAF' from dual union all
select 29876,'USAF' from dual union all
select 29881,'USAF' from dual union all
select 29893,'USAF' from dual union all
select 29902,'USAF' from dual union all
select 29910,'GSFC' from dual union all
select 29911,'INTA' from dual union all
select 29919,'USAF' from dual union all
select 29929,'USAF' from dual union all
select 29933,'USAF' from dual union all
select 29945,'GSFC' from dual union all
select 29947,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 29955,'USAF' from dual union all
select 29960,'USAF' from dual union all
select 29967,'USAF' from dual union all
select 29972,'USAF' from dual union all
select 29973,'USAF' from dual union all
select 29974,'USAF' from dual union all
select 29975,'USAF' from dual union all
select 29976,'USAF' from dual union all
select 29983,'HOUS' from dual union all
select 29983,'NRCC' from dual union all
select 29984,'USAF' from dual union all
select 29994,'HOUS' from dual union all
select 30049,'USAF' from dual union all
select 30115,'USAF' from dual union all
select 30118,'LARC' from dual union all
select 30123,'LARC' from dual union all
select 30124,'LARC' from dual union all
select 30128,'LARC' from dual union all
select 30141,'USAF' from dual union all
select 30178,'BAL' from dual union all
select 30179,'BAL' from dual union all
select 30182,'BAL' from dual union all
select 30183,'BAL' from dual union all
select 30185,'BAL' from dual union all
select 30186,'BAL' from dual union all
select 30187,'BAL' from dual union all
select 30191,'BAL' from dual union all
select 30194,'USAF' from dual union all
select 30203,'USAF' from dual union all
select 30224,'USAF' from dual union all
select 30236,'WSMR' from dual union all
select 30250,'USAF' from dual union all
select 30258,'PSU' from dual union all
select 30271,'USAF' from dual union all
select 30281,'USAF' from dual union all
select 30296,'AN' from dual union all
select 30296,'ISRO' from dual union all
select 30332,'UCW' from dual union all
select 30342,'UCW' from dual union all
select 30360,'AN' from dual union all
select 30360,'ISRO' from dual union all
select 30375,'AN' from dual union all
select 30375,'ISRO' from dual union all
select 30420,'AN' from dual union all
select 30420,'ISRO' from dual union all
select 30437,'AN' from dual union all
select 30437,'ISRO' from dual union all
select 30449,'AN' from dual union all
select 30449,'ISRO' from dual union all
select 30465,'UCW' from dual union all
select 30474,'GMS' from dual union all
select 30479,'GMS' from dual union all
select 30502,'PSU' from dual union all
select 30513,'AN' from dual union all
select 30513,'ISRO' from dual union all
select 30528,'AN' from dual union all
select 30528,'ISRO' from dual union all
select 30529,'AN' from dual union all
select 30529,'ISRO' from dual union all
select 30530,'AN' from dual union all
select 30530,'ISRO' from dual union all
select 30531,'AN' from dual union all
select 30531,'ISRO' from dual union all
select 30532,'AN' from dual union all
select 30532,'ISRO' from dual union all
select 30533,'AN' from dual union all
select 30533,'ISRO' from dual union all
select 30534,'AN' from dual union all
select 30534,'ISRO' from dual union all
select 30541,'AN' from dual union all
select 30541,'ISRO' from dual union all
select 30547,'PSU' from dual union all
select 30571,'AN' from dual union all
select 30571,'ISRO' from dual union all
select 30736,'PIHM' from dual union all
select 30737,'PIHM' from dual union all
select 30738,'PIHM' from dual union all
select 30739,'PIHM' from dual union all
select 30740,'PIHM' from dual union all
select 30748,'GSFC' from dual union all
select 30759,'PIHM' from dual union all
select 30760,'PIHM' from dual union all
select 30761,'PIHM' from dual union all
select 30763,'PIHM' from dual union all
select 30764,'GSFC' from dual union all
select 30772,'GSFC' from dual union all
select 30773,'AFCRL' from dual union all
select 30788,'AFCRL' from dual union all
select 30789,'SDCO' from dual union all
select 30792,'EGAO' from dual union all
select 30793,'EGAO' from dual union all
select 30796,'SAND' from dual union all
select 30797,'SAND' from dual union all
select 30798,'SAND' from dual union all
select 30799,'SAND' from dual union all
select 30800,'WFF' from dual union all
select 30801,'SAND' from dual union all
select 30808,'MPAE' from dual union all
select 30810,'MPAE' from dual union all
select 30811,'SUPA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 30812,'AFCRL' from dual union all
select 30813,'MPAE' from dual union all
select 30814,'AFCRL' from dual union all
select 30815,'SUPA' from dual union all
select 30818,'GSFC' from dual union all
select 30819,'AFCRL' from dual union all
select 30820,'SOTON' from dual union all
select 30822,'CNES' from dual union all
select 30823,'SDCO' from dual union all
select 30824,'SDCO' from dual union all
select 30825,'SDCO' from dual union all
select 30826,'SDCO' from dual union all
select 30827,'SDCO' from dual union all
select 30828,'SDCO' from dual union all
select 30829,'SDCO' from dual union all
select 30830,'SDCO' from dual union all
select 30832,'CNES' from dual union all
select 30833,'CNES' from dual union all
select 30834,'CNES' from dual union all
select 30835,'CNES' from dual union all
select 30836,'CNES' from dual union all
select 30837,'GSFC' from dual union all
select 30838,'CNES' from dual union all
select 30839,'CNES' from dual union all
select 30840,'CNES' from dual union all
select 30841,'CNES' from dual union all
select 30842,'CNES' from dual union all
select 30843,'CNES' from dual union all
select 30844,'CNES' from dual union all
select 30845,'INTA' from dual union all
select 30845,'NASA' from dual union all
select 30846,'INTA' from dual union all
select 30846,'NASA' from dual union all
select 30847,'INTA' from dual union all
select 30847,'NASA' from dual union all
select 30848,'INTA' from dual union all
select 30848,'NASA' from dual union all
select 30850,'GSFC' from dual union all
select 30851,'GSFC' from dual union all
select 30852,'GSFC' from dual union all
select 30853,'GSFC' from dual union all
select 30854,'GSFC' from dual union all
select 30855,'GSFC' from dual union all
select 30856,'GSFC' from dual union all
select 30857,'GSFC' from dual union all
select 30866,'AFCRL' from dual union all
select 30871,'CULH' from dual union all
select 30872,'SUPA' from dual union all
select 30874,'UMN' from dual union all
select 30878,'MPAE' from dual union all
select 30880,'BIR' from dual union all
select 30880,'SOTON' from dual union all
select 30881,'CULH' from dual union all
select 30882,'GSFC' from dual union all
select 30883,'GSFC' from dual union all
select 30884,'AFCRL' from dual union all
select 30885,'RAE' from dual union all
select 30886,'MPAE' from dual union all
select 30887,'CIT' from dual union all
select 30888,'AFCRL' from dual union all
select 30890,'NIPR' from dual union all
select 30891,'AFCRL' from dual union all
select 30892,'AFCRL' from dual union all
select 30893,'GSFC' from dual union all
select 30894,'RAE' from dual union all
select 30895,'MPAE' from dual union all
select 30898,'MIT' from dual union all
select 30899,'GSFC' from dual union all
select 30903,'BIR' from dual union all
select 30903,'SOTON' from dual union all
select 30904,'GSFC' from dual union all
select 30905,'RSRS' from dual union all
select 30906,'RSRS' from dual union all
select 30907,'RSRS' from dual union all
select 30908,'RSRS' from dual union all
select 30909,'MPAE' from dual union all
select 30910,'RSRS' from dual union all
select 30911,'UCB' from dual union all
select 30912,'RSRS' from dual union all
select 30913,'NRL' from dual union all
select 30914,'RSRS' from dual union all
select 30917,'MPAE' from dual union all
select 30919,'NRL' from dual union all
select 30920,'ISRO' from dual union all
select 30921,'AFCRL' from dual union all
select 30923,'AFCRL' from dual union all
select 30925,'CNRS' from dual union all
select 30926,'LUX' from dual union all
select 30927,'NIPR' from dual union all
select 30931,'LRL' from dual union all
select 30932,'AFCRL' from dual union all
select 30933,'ISAS' from dual union all
select 30936,'GMS' from dual union all
select 30941,'BONN' from dual union all
select 30941,'MPE' from dual union all
select 30942,'NRCC' from dual union all
select 30943,'UCO' from dual union all
select 30944,'UIUC' from dual union all
select 30945,'BRIS' from dual union all
select 30948,'NCAR' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 30949,'PRIN' from dual union all
select 30950,'AFCRL' from dual union all
select 30951,'CESR' from dual union all
select 30951,'IKI' from dual union all
select 30951,'IZM' from dual union all
select 30952,'GMS' from dual union all
select 30954,'WSMR' from dual union all
select 30955,'NOAA' from dual union all
select 30956,'WSMR' from dual union all
select 30957,'WSMR' from dual union all
select 30958,'WSMR' from dual union all
select 30959,'WSMR' from dual union all
select 30960,'WSMR' from dual union all
select 30961,'WSMR' from dual union all
select 30962,'WSMR' from dual union all
select 30963,'WSMR' from dual union all
select 30965,'LPARL' from dual union all
select 30969,'ASE' from dual union all
select 30970,'HCO' from dual union all
select 30972,'NIPR' from dual union all
select 30973,'NIPR' from dual union all
select 30974,'LPARL' from dual union all
select 30975,'AFCRL' from dual union all
select 30976,'KYO' from dual union all
select 30977,'AFCRL' from dual union all
select 30978,'NRL' from dual union all
select 30980,'RSRS' from dual union all
select 30980,'UCL' from dual union all
select 30982,'CALG' from dual union all
select 30985,'LRL' from dual union all
select 30987,'MPAE' from dual union all
select 30991,'UCW' from dual union all
select 30997,'GMS' from dual union all
select 30998,'TIFR' from dual union all
select 30999,'CULH' from dual union all
select 30999,'KPNO' from dual union all
select 31000,'CRA' from dual union all
select 31001,'GSFC' from dual union all
select 31002,'GSFC' from dual union all
select 31003,'GMS' from dual union all
select 31004,'CAL' from dual union all
select 31005,'UCW' from dual union all
select 31007,'PRL' from dual union all
select 31008,'BONN' from dual union all
select 31008,'FREIB' from dual union all
select 31009,'MPAE' from dual union all
select 31013,'NIPR' from dual union all
select 31014,'RAE' from dual union all
select 31017,'GSFC' from dual union all
select 31018,'GSFC' from dual union all
select 31019,'PITT' from dual union all
select 31020,'GSFC' from dual union all
select 31021,'GSFC' from dual union all
select 31022,'GSFC' from dual union all
select 31023,'GMS' from dual union all
select 31027,'BRIS' from dual union all
select 31029,'GSFC' from dual union all
select 31030,'351SMW' from dual union all
select 31032,'UCO' from dual union all
select 31034,'GMS' from dual union all
select 31045,'DUD' from dual union all
select 31045,'MISU' from dual union all
select 31046,'DUD' from dual union all
select 31046,'MISU' from dual union all
select 31048,'GSFC' from dual union all
select 31049,'UIUC' from dual union all
select 31050,'DFVLR' from dual union all
select 31050,'RAE' from dual union all
select 31050,'SSC' from dual union all
select 31051,'GSFC' from dual union all
select 31052,'DUD' from dual union all
select 31052,'MISU' from dual union all
select 31054,'GSFC' from dual union all
select 31056,'HCO' from dual union all
select 31057,'AFCRL' from dual union all
select 31058,'AFCRL' from dual union all
select 31059,'UIUC' from dual union all
select 31060,'WRE' from dual union all
select 31062,'PITT' from dual union all
select 31065,'ISAS' from dual union all
select 31065,'TOK' from dual union all
select 31066,'ISAS' from dual union all
select 31067,'NIPR' from dual union all
select 31069,'GSFC' from dual union all
select 31071,'NAG' from dual union all
select 31073,'UCO' from dual union all
select 31079,'QUB' from dual union all
select 31080,'QUB' from dual union all
select 31081,'NRL' from dual union all
select 31083,'UCW' from dual union all
select 31085,'NASDA' from dual union all
select 31086,'MPAE' from dual union all
select 31087,'UCL' from dual union all
select 31088,'UCL' from dual union all
select 31089,'AFCRL' from dual union all
select 31090,'GSFC' from dual union all
select 31092,'SHEF' from dual union all
select 31092,'SOTON' from dual union all
select 31093,'GMS' from dual union all
select 31096,'AFCRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 31098,'AFCRL' from dual union all
select 31099,'WRE' from dual union all
select 31100,'GSFC' from dual union all
select 31101,'ISAS' from dual union all
select 31101,'TOK' from dual union all
select 31105,'AFCRL' from dual union all
select 31106,'AFCRL' from dual union all
select 31107,'AFCRL' from dual union all
select 31108,'WRE' from dual union all
select 31109,'GMS' from dual union all
select 31111,'WRE' from dual union all
select 31112,'WSMR' from dual union all
select 31116,'MPAE' from dual union all
select 31124,'ESRO' from dual union all
select 31124,'MPE' from dual union all
select 31125,'AFCRL' from dual union all
select 31126,'MPAE' from dual union all
select 31127,'AFCRL' from dual union all
select 31128,'GSFC' from dual union all
select 31130,'MPE' from dual union all
select 31132,'UCL' from dual union all
select 31133,'SOTON' from dual union all
select 31134,'MSSL' from dual union all
select 31134,'RSRS' from dual union all
select 31134,'SOTON' from dual union all
select 31135,'WRE' from dual union all
select 31136,'UCL' from dual union all
select 31137,'NDRE' from dual union all
select 31138,'GMS' from dual union all
select 31139,'NRL' from dual union all
select 31142,'AFCRL' from dual union all
select 31143,'AFCRL' from dual union all
select 31144,'AFCRL' from dual union all
select 31145,'AFCRL' from dual union all
select 31146,'AFCRL' from dual union all
select 31147,'MPAE' from dual union all
select 31148,'ISAS' from dual union all
select 31149,'AFCRL' from dual union all
select 31150,'AFCRL' from dual union all
select 31151,'AFCRL' from dual union all
select 31152,'WRE' from dual union all
select 31153,'GMS' from dual union all
select 31154,'AFCRL' from dual union all
select 31155,'JHU' from dual union all
select 31156,'AFCRL' from dual union all
select 31157,'GMS' from dual union all
select 31158,'MSSL' from dual union all
select 31158,'RSRS' from dual union all
select 31158,'SOTON' from dual union all
select 31159,'NRCC' from dual union all
select 31160,'RSRS' from dual union all
select 31161,'RSRS' from dual union all
select 31167,'UWI' from dual union all
select 31168,'CIT' from dual union all
select 31170,'MPAE' from dual union all
select 31171,'AFCRL' from dual union all
select 31174,'MIT' from dual union all
select 31175,'CIT' from dual union all
select 31176,'MIT' from dual union all
select 31177,'UWI' from dual union all
select 31178,'NRL' from dual union all
select 31179,'BIR' from dual union all
select 31179,'UCL' from dual union all
select 31181,'BIR' from dual union all
select 31182,'UCL' from dual union all
select 31183,'BIR' from dual union all
select 31184,'UCL' from dual union all
select 31185,'BIR' from dual union all
select 31186,'UCB' from dual union all
select 31187,'IIAE' from dual union all
select 31187,'MPE' from dual union all
select 31188,'IIAE' from dual union all
select 31188,'MPE' from dual union all
select 31190,'WRE' from dual union all
select 31191,'MSSL' from dual union all
select 31193,'MPE' from dual union all
select 31194,'DLR' from dual union all
select 31195,'UCB' from dual union all
select 31196,'UHI' from dual union all
select 31197,'YORK' from dual union all
select 31198,'LUX' from dual union all
select 31199,'MPE' from dual union all
select 31200,'UAF' from dual union all
select 31201,'WSMR' from dual union all
select 31206,'MPAE' from dual union all
select 31207,'SUSX' from dual union all
select 31208,'MPAE' from dual union all
select 31209,'YORK' from dual union all
select 31210,'UAF' from dual union all
select 31211,'YORK' from dual union all
select 31212,'WSMR' from dual union all
select 31213,'YORK' from dual union all
select 31214,'YORK' from dual union all
select 31215,'MPAE' from dual union all
select 31216,'YORK' from dual union all
select 31217,'SHEF' from dual union all
select 31217,'SOTON' from dual union all
select 31220,'SUSX' from dual union all
select 31221,'SHEF' from dual union all
select 31221,'SOTON' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 31222,'AFCRL' from dual union all
select 31223,'WSMR' from dual union all
select 31224,'GMS' from dual union all
select 31225,'MPAE' from dual union all
select 31227,'HCO' from dual union all
select 31228,'AFCRL' from dual union all
select 31230,'MPAE' from dual union all
select 31232,'MPAE' from dual union all
select 31235,'MPAE' from dual union all
select 31236,'MPAE' from dual union all
select 31238,'LPARL' from dual union all
select 31242,'GMS' from dual union all
select 31247,'AFCRL' from dual union all
select 31248,'AFCRL' from dual union all
select 31249,'AFCRL' from dual union all
select 31250,'GMS' from dual union all
select 31251,'GMS' from dual union all
select 31252,'GMS' from dual union all
select 31253,'GMS' from dual union all
select 31254,'GMS' from dual union all
select 31255,'GMS' from dual union all
select 31257,'GMS' from dual union all
select 31258,'GMS' from dual union all
select 31261,'SDCO' from dual union all
select 31262,'SDCO' from dual union all
select 31263,'SDCO' from dual union all
select 31264,'SDCO' from dual union all
select 31265,'SDCO' from dual union all
select 31266,'SDCO' from dual union all
select 31268,'SDCO' from dual union all
select 31269,'SDCO' from dual union all
select 31270,'SDCO' from dual union all
select 31271,'SDCO' from dual union all
select 31272,'SDCO' from dual union all
select 31273,'SDCO' from dual union all
select 31274,'SDCO' from dual union all
select 31280,'WRE' from dual union all
select 31283,'ISAS' from dual union all
select 31284,'TSAO' from dual union all
select 31333,'DLR' from dual union all
select 31344,'NRCC' from dual union all
select 31347,'MPAE' from dual union all
select 31348,'WRE' from dual union all
select 31349,'WRE' from dual union all
select 31350,'WRE' from dual union all
select 31351,'WRE' from dual union all
select 31352,'WRE' from dual union all
select 31353,'WRE' from dual union all
select 31355,'NRCC' from dual union all
select 31366,'PRL' from dual union all
select 31377,'DLR' from dual union all
select 31382,'LASL' from dual union all
select 31382,'UAF' from dual union all
select 31383,'LASL' from dual union all
select 31384,'NDRE' from dual union all
select 31385,'NASDA' from dual union all
select 31386,'DLR' from dual union all
select 31387,'KGO' from dual union all
select 31387,'KTH' from dual union all
select 31388,'NRCC' from dual union all
select 31389,'UCB' from dual union all
select 31392,'NIPR' from dual union all
select 31393,'DLR' from dual union all
select 31393,'INPE' from dual union all
select 31394,'OPM' from dual union all
select 31395,'TAO' from dual union all
select 31396,'UCW' from dual union all
select 31402,'NRCC' from dual union all
select 31403,'UCW' from dual union all
select 31404,'ISAS' from dual union all
select 31405,'UCW' from dual union all
select 31406,'NRCC' from dual union all
select 31407,'SASK' from dual union all
select 31408,'UMI' from dual union all
select 31409,'LUX' from dual union all
select 31414,'PITT' from dual union all
select 31415,'PRL' from dual union all
select 31416,'PRL' from dual union all
select 31417,'UTD' from dual union all
select 31418,'HCO' from dual union all
select 31422,'ASE' from dual union all
select 31425,'USAF' from dual union all
select 31426,'USAF' from dual union all
select 31427,'USAF' from dual union all
select 31428,'USAF' from dual union all
select 31429,'USAF' from dual union all
select 31436,'USAF' from dual union all
select 31450,'USAF' from dual union all
select 31465,'USAF' from dual union all
select 31474,'USAF' from dual union all
select 31475,'GSFC' from dual union all
select 31478,'GSFC' from dual union all
select 31488,'USAF' from dual union all
select 31498,'USAF' from dual union all
select 31523,'USAF' from dual union all
select 31525,'USAF' from dual union all
select 31544,'USAF' from dual union all
select 31552,'USAF' from dual union all
select 31556,'USAF' from dual union all
select 31557,'WSMR' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 31558,'USAF' from dual union all
select 31559,'USAF' from dual union all
select 31569,'GSFC' from dual union all
select 31571,'NASDA' from dual union all
select 31572,'NASDA' from dual union all
select 31602,'USAF' from dual union all
select 31609,'USAF' from dual union all
select 31618,'USAF' from dual union all
select 31637,'USAF' from dual union all
select 31643,'USAF' from dual union all
select 31651,'USAF' from dual union all
select 31656,'USAF' from dual union all
select 31676,'USAF' from dual union all
select 31679,'USAF' from dual union all
select 31681,'USAF' from dual union all
select 31688,'USAF' from dual union all
select 31697,'USAF' from dual union all
select 31698,'USAF' from dual union all
select 31699,'USAF' from dual union all
select 31700,'USAF' from dual union all
select 31702,'USAF' from dual union all
select 31703,'USAF' from dual union all
select 31704,'USAF' from dual union all
select 31705,'USAF' from dual union all
select 31712,'USAF' from dual union all
select 31790,'USAF' from dual union all
select 31833,'USAF' from dual union all
select 31835,'USAF' from dual union all
select 31836,'USAF' from dual union all
select 31837,'USAF' from dual union all
select 31838,'USAF' from dual union all
select 31839,'USAF' from dual union all
select 31840,'USAF' from dual union all
select 31841,'USAF' from dual union all
select 31879,'USAF' from dual union all
select 31946,'GSFC' from dual union all
select 31967,'USAF' from dual union all
select 31970,'USAF' from dual union all
select 31971,'USAF' from dual union all
select 31972,'USAF' from dual union all
select 31973,'USAF' from dual union all
select 32103,'CNES' from dual union all
select 32104,'CNES' from dual union all
select 32105,'CNES' from dual union all
select 32106,'CNES' from dual union all
select 32107,'CNES' from dual union all
select 32108,'CNES' from dual union all
select 32109,'CNES' from dual union all
select 32110,'CNES' from dual union all
select 32111,'CNES' from dual union all
select 32113,'CNES' from dual union all
select 32114,'CNES' from dual union all
select 32115,'CNES' from dual union all
select 32116,'GMS' from dual union all
select 32117,'GMS' from dual union all
select 32118,'GMS' from dual union all
select 32119,'GMS' from dual union all
select 32120,'GMS' from dual union all
select 32121,'GMS' from dual union all
select 32122,'GMS' from dual union all
select 32124,'GMS' from dual union all
select 32125,'UKMO' from dual union all
select 32126,'UKMO' from dual union all
select 32127,'UKMO' from dual union all
select 32128,'UKMO' from dual union all
select 32129,'UKMO' from dual union all
select 32130,'UKMO' from dual union all
select 32131,'UKMO' from dual union all
select 32132,'UKMO' from dual union all
select 32133,'UKMO' from dual union all
select 32135,'UKMO' from dual union all
select 32136,'UKMO' from dual union all
select 32137,'NOAA' from dual union all
select 32138,'NOAA' from dual union all
select 32139,'NOAA' from dual union all
select 32140,'NOAA' from dual union all
select 32141,'NOAA' from dual union all
select 32142,'NOAA' from dual union all
select 32143,'NOAA' from dual union all
select 32145,'NOAA' from dual union all
select 32146,'NOAA' from dual union all
select 32147,'NOAA' from dual union all
select 32148,'NOAA' from dual union all
select 32149,'NOAA' from dual union all
select 32150,'IAE' from dual union all
select 32151,'IAE' from dual union all
select 32152,'IAE' from dual union all
select 32153,'IAE' from dual union all
select 32154,'IAE' from dual union all
select 32156,'IAE' from dual union all
select 32157,'IAE' from dual union all
select 32158,'IAE' from dual union all
select 32159,'WSMR' from dual union all
select 32160,'NOAA' from dual union all
select 32162,'USAF' from dual union all
select 32167,'USAF' from dual union all
select 32168,'HOUS' from dual union all
select 32177,'USAF' from dual union all
select 32178,'NASDA' from dual union all
select 32185,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 32187,'USAF' from dual union all
select 32191,'USAF' from dual union all
select 32192,'INTA' from dual union all
select 32202,'NASDA' from dual union all
select 32203,'USAF' from dual union all
select 32210,'USAF' from dual union all
select 32214,'USAF' from dual union all
select 32227,'USAF' from dual union all
select 32230,'GSFC' from dual union all
select 32232,'USAF' from dual union all
select 32241,'USAF' from dual union all
select 32242,'USAF' from dual union all
select 32250,'USAF' from dual union all
select 32257,'USAF' from dual union all
select 32261,'USAF' from dual union all
select 32265,'GSFC' from dual union all
select 32269,'USAF' from dual union all
select 32279,'USAF' from dual union all
select 32283,'USAF' from dual union all
select 32293,'USAF' from dual union all
select 32313,'USAF' from dual union all
select 32315,'USAF' from dual union all
select 32323,'USAF' from dual union all
select 32330,'USAF' from dual union all
select 32331,'USAF' from dual union all
select 32337,'USAF' from dual union all
select 32341,'USAF' from dual union all
select 32343,'INTA' from dual union all
select 32347,'USAF' from dual union all
select 32357,'USAF' from dual union all
select 32360,'USAF' from dual union all
select 32370,'USAF' from dual union all
select 32374,'USAF' from dual union all
select 32378,'USAF' from dual union all
select 32381,'USAF' from dual union all
select 32393,'USAF' from dual union all
select 32398,'USAF' from dual union all
select 32405,'USAF' from dual union all
select 32415,'USAF' from dual union all
select 32422,'USAF' from dual union all
select 32428,'USAF' from dual union all
select 32436,'USAF' from dual union all
select 32447,'USAF' from dual union all
select 32456,'USAF' from dual union all
select 32465,'USAF' from dual union all
select 32474,'USAF' from dual union all
select 32479,'USAF' from dual union all
select 32484,'USAF' from dual union all
select 3249,'CNRS' from dual union all
select 3249,'PARIS' from dual union all
select 32499,'USAF' from dual union all
select 3250,'CNRS' from dual union all
select 3250,'PARIS' from dual union all
select 32502,'USAF' from dual union all
select 3251,'CNRS' from dual union all
select 3251,'PARIS' from dual union all
select 32513,'USAF' from dual union all
select 32529,'USAF' from dual union all
select 32532,'INTA' from dual union all
select 32535,'USAF' from dual union all
select 32539,'USAF' from dual union all
select 32549,'USAF' from dual union all
select 32563,'USAF' from dual union all
select 32569,'USAF' from dual union all
select 32593,'USAF' from dual union all
select 32600,'USAF' from dual union all
select 32611,'USAF' from dual union all
select 32619,'USAF' from dual union all
select 32628,'USAF' from dual union all
select 32638,'USAF' from dual union all
select 32643,'USAF' from dual union all
select 32657,'USAF' from dual union all
select 32659,'INTA' from dual union all
select 32662,'USAF' from dual union all
select 32676,'USAF' from dual union all
select 32684,'USAF' from dual union all
select 32695,'NRCC' from dual union all
select 32702,'USAF' from dual union all
select 32734,'USAF' from dual union all
select 32744,'USAF' from dual union all
select 32748,'USAF' from dual union all
select 32751,'USAF' from dual union all
select 32764,'USAF' from dual union all
select 32788,'GSFC' from dual union all
select 32793,'INTA' from dual union all
select 32799,'USAF' from dual union all
select 32831,'GMS' from dual union all
select 32838,'USAF' from dual union all
select 32872,'USAF' from dual union all
select 32898,'USAF' from dual union all
select 32902,'USAF' from dual union all
select 32908,'USAF' from dual union all
select 32909,'USAF' from dual union all
select 32910,'USAF' from dual union all
select 32920,'USAF' from dual union all
select 32935,'USAF' from dual union all
select 32940,'USAF' from dual union all
select 32945,'WSMR' from dual union all
select 32963,'WSMR' from dual union all
select 32964,'WFF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 32965,'WFF' from dual union all
select 32966,'WFF' from dual union all
select 32967,'WFF' from dual union all
select 32969,'WFF' from dual union all
select 32973,'WFF' from dual union all
select 32974,'WFF' from dual union all
select 32975,'WFF' from dual union all
select 32976,'WFF' from dual union all
select 32977,'WFF' from dual union all
select 32983,'USAF' from dual union all
select 33005,'USAF' from dual union all
select 33033,'USAF' from dual union all
select 33035,'USAF' from dual union all
select 33036,'USAF' from dual union all
select 33037,'USAF' from dual union all
select 33047,'USAF' from dual union all
select 33050,'USAF' from dual union all
select 33056,'USAF' from dual union all
select 33070,'USAF' from dual union all
select 33078,'CNES' from dual union all
select 33079,'CNES' from dual union all
select 33080,'CNES' from dual union all
select 33081,'CNES' from dual union all
select 33082,'CNES' from dual union all
select 33083,'CNES' from dual union all
select 33084,'CNES' from dual union all
select 33085,'CNES' from dual union all
select 33086,'CNES' from dual union all
select 33087,'CNES' from dual union all
select 33098,'CNES' from dual union all
select 33109,'CNES' from dual union all
select 33174,'GMS' from dual union all
select 33185,'GMS' from dual union all
select 33188,'GMS' from dual union all
select 3320,'ONR' from dual union all
select 3320,'UMD' from dual union all
select 3322,'NRL' from dual union all
select 3323,'LMSC' from dual union all
select 33242,'GMS' from dual union all
select 33243,'GMS' from dual union all
select 33244,'GMS' from dual union all
select 33245,'GMS' from dual union all
select 33246,'GMS' from dual union all
select 33248,'GMS' from dual union all
select 33251,'GMS' from dual union all
select 3329,'LMSC' from dual union all
select 33299,'WFF' from dual union all
select 3330,'SUI' from dual union all
select 33300,'WSMR' from dual union all
select 33301,'WFF' from dual union all
select 33302,'WSMR' from dual union all
select 33303,'UNK' from dual union all
select 33304,'UNK' from dual union all
select 33305,'WFF' from dual union all
select 33306,'WSMR' from dual union all
select 33318,'PSU' from dual union all
select 3332,'NRL' from dual union all
select 3333,'NRL' from dual union all
select 3337,'NRL' from dual union all
select 3341,'SUI' from dual union all
select 3342,'SUI' from dual union all
select 3350,'NRL' from dual union all
select 3350,'SBRC' from dual union all
select 3353,'NRL' from dual union all
select 33551,'GMS' from dual union all
select 33552,'GMS' from dual union all
select 33553,'GMS' from dual union all
select 33554,'GMS' from dual union all
select 33555,'GMS' from dual union all
select 33556,'GMS' from dual union all
select 33564,'HOUS' from dual union all
select 33572,'GMS' from dual union all
select 3359,'NRL' from dual union all
select 3359,'SBRC' from dual union all
select 3360,'CNRS' from dual union all
select 3360,'PARIS' from dual union all
select 33710,'SAND' from dual union all
select 33713,'LRL' from dual union all
select 33718,'AFCRL' from dual union all
select 33720,'PITT' from dual union all
select 33723,'GSFC' from dual union all
select 33732,'WFF' from dual union all
select 33736,'AFCRL' from dual union all
select 33737,'AFCRL' from dual union all
select 33738,'AFCRL' from dual union all
select 33747,'SAND' from dual union all
select 33760,'SAND' from dual union all
select 33766,'MPE' from dual union all
select 33767,'NRL' from dual union all
select 33769,'LRL' from dual union all
select 33770,'ADEL' from dual union all
select 33770,'NRL' from dual union all
select 33779,'SAND' from dual union all
select 33780,'SAND' from dual union all
select 33781,'SAND' from dual union all
select 33797,'WFF' from dual union all
select 33798,'MISU' from dual union all
select 33798,'WFF' from dual union all
select 33799,'CNES' from dual union all
select 33800,'CNES' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 33801,'CNES' from dual union all
select 33802,'CNES' from dual union all
select 33803,'CNES' from dual union all
select 33804,'CNES' from dual union all
select 33805,'CNES' from dual union all
select 33806,'CNES' from dual union all
select 33807,'CNES' from dual union all
select 33808,'CNES' from dual union all
select 33809,'CNES' from dual union all
select 33810,'CNES' from dual union all
select 33811,'CNES' from dual union all
select 33812,'CNES' from dual union all
select 33813,'CNES' from dual union all
select 33814,'CNES' from dual union all
select 33815,'CNES' from dual union all
select 33816,'CNES' from dual union all
select 33817,'CNES' from dual union all
select 33818,'CNES' from dual union all
select 33819,'CNES' from dual union all
select 33820,'CNES' from dual union all
select 33821,'CNES' from dual union all
select 33822,'INTA' from dual union all
select 33823,'INTA' from dual union all
select 33824,'INTA' from dual union all
select 33825,'INTA' from dual union all
select 33826,'INTA' from dual union all
select 33827,'INTA' from dual union all
select 33828,'INTA' from dual union all
select 33829,'INTA' from dual union all
select 33830,'INTA' from dual union all
select 33832,'GSFC' from dual union all
select 33833,'IIAE' from dual union all
select 33834,'IIAE' from dual union all
select 33836,'WRE' from dual union all
select 33837,'WRE' from dual union all
select 33846,'NRCC' from dual union all
select 33847,'AFCRL' from dual union all
select 33849,'AFCRL' from dual union all
select 33850,'AFCRL' from dual union all
select 33852,'AFCRL' from dual union all
select 33853,'STO' from dual union all
select 33854,'GSFC' from dual union all
select 33854,'KGO' from dual union all
select 33854,'KTH' from dual union all
select 33855,'STO' from dual union all
select 33857,'GSFC' from dual union all
select 33857,'KGO' from dual union all
select 33857,'KTH' from dual union all
select 33860,'AFCRL' from dual union all
select 33861,'SASK' from dual union all
select 33862,'PITT' from dual union all
select 33865,'AFCRL' from dual union all
select 33868,'LPSP' from dual union all
select 33868,'UCO' from dual union all
select 33869,'CALG' from dual union all
select 33872,'PITT' from dual union all
select 33874,'GSFC' from dual union all
select 33877,'RICE' from dual union all
select 33883,'GMS' from dual union all
select 33885,'GSFC' from dual union all
select 33886,'HOUS' from dual union all
select 33886,'LASL' from dual union all
select 33886,'UAF' from dual union all
select 33887,'LASL' from dual union all
select 33887,'SHEF' from dual union all
select 33888,'NRL' from dual union all
select 33888,'UMD' from dual union all
select 33892,'UMI' from dual union all
select 33893,'GSFC' from dual union all
select 33893,'MISU' from dual union all
select 33894,'GSFC' from dual union all
select 33894,'MISU' from dual union all
select 33895,'GMS' from dual union all
select 33896,'GSFC' from dual union all
select 33897,'GSFC' from dual union all
select 33899,'LASL' from dual union all
select 33899,'UAF' from dual union all
select 33903,'AFCRL' from dual union all
select 33904,'AERO' from dual union all
select 33905,'ESTEC' from dual union all
select 33905,'GMS' from dual union all
select 33905,'ORL' from dual union all
select 33907,'RSRS' from dual union all
select 33908,'UCW' from dual union all
select 33912,'AFCRL' from dual union all
select 33913,'RSRS' from dual union all
select 33914,'UCW' from dual union all
select 33915,'GMS' from dual union all
select 33916,'AFCRL' from dual union all
select 33918,'GMS' from dual union all
select 33919,'LPARL' from dual union all
select 33920,'AFCRL' from dual union all
select 33921,'LASL' from dual union all
select 33921,'UAF' from dual union all
select 33923,'AFCRL' from dual union all
select 33925,'AFCRL' from dual union all
select 33928,'UMN' from dual union all
select 33929,'AFCRL' from dual union all
select 33930,'UIUC' from dual union all
select 33931,'AFCRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 33932,'ADEL' from dual union all
select 33932,'UCL' from dual union all
select 33934,'ISRO' from dual union all
select 33938,'WSMR' from dual union all
select 33939,'SHEF' from dual union all
select 33942,'ERNO' from dual union all
select 33948,'GSFC' from dual union all
select 33949,'GSFC' from dual union all
select 33950,'PSU' from dual union all
select 33951,'HCO' from dual union all
select 33963,'WSMR' from dual union all
select 33976,'UCB' from dual union all
select 33978,'DASA' from dual union all
select 33980,'MSSL' from dual union all
select 33981,'NRL' from dual union all
select 33982,'UHI' from dual union all
select 33983,'SAND' from dual union all
select 33983,'UCO' from dual union all
select 33984,'SAND' from dual union all
select 33984,'UCO' from dual union all
select 33985,'GMS' from dual union all
select 33986,'GMS' from dual union all
select 33990,'GSFC' from dual union all
select 33993,'ASE' from dual union all
select 33995,'AFCRL' from dual union all
select 33996,'AFCRL' from dual union all
select 33997,'AFCRL' from dual union all
select 33999,'MPE' from dual union all
select 34000,'AFCRL' from dual union all
select 34001,'AFCRL' from dual union all
select 34002,'AFCRL' from dual union all
select 34003,'WSMR' from dual union all
select 34005,'AFCRL' from dual union all
select 34006,'UIUC' from dual union all
select 34007,'AFCRL' from dual union all
select 34008,'UIUC' from dual union all
select 34010,'PITT' from dual union all
select 34011,'AFCRL' from dual union all
select 34012,'AFCRL' from dual union all
select 34013,'AFCRL' from dual union all
select 34014,'AFCRL' from dual union all
select 34015,'AFCRL' from dual union all
select 34016,'UMI' from dual union all
select 34017,'UIUC' from dual union all
select 34018,'AFCRL' from dual union all
select 34019,'LRL' from dual union all
select 34021,'AFCRL' from dual union all
select 34022,'AFCRL' from dual union all
select 34023,'DMI' from dual union all
select 34025,'GSFC' from dual union all
select 34026,'GSFC' from dual union all
select 34027,'DMI' from dual union all
select 34034,'SAMSO' from dual union all
select 34035,'UWI' from dual union all
select 34036,'RAE' from dual union all
select 34041,'DUD' from dual union all
select 34044,'AFCRL' from dual union all
select 34045,'MPE' from dual union all
select 34045,'NDRE' from dual union all
select 34047,'ISAS' from dual union all
select 34051,'ISAS' from dual union all
select 34054,'SOTON' from dual union all
select 34055,'AFCRL' from dual union all
select 34056,'SOTON' from dual union all
select 34057,'NRL' from dual union all
select 34060,'MSSL' from dual union all
select 34061,'AFCRL' from dual union all
select 34062,'IASB' from dual union all
select 34063,'AFCRL' from dual union all
select 34064,'IASB' from dual union all
select 34065,'IASB' from dual union all
select 34067,'ISAS' from dual union all
select 34068,'AFCRL' from dual union all
select 34069,'HCO' from dual union all
select 34070,'ISAS' from dual union all
select 34072,'GMS' from dual union all
select 34078,'GMS' from dual union all
select 34080,'GMS' from dual union all
select 34082,'ISAS' from dual union all
select 34090,'TSAO' from dual union all
select 34091,'GSFC' from dual union all
select 34092,'QUB' from dual union all
select 34094,'QUB' from dual union all
select 34097,'UCW' from dual union all
select 34102,'GSFC' from dual union all
select 34104,'MSSL' from dual union all
select 34105,'GSFC' from dual union all
select 34109,'LUX' from dual union all
select 34109,'MPE' from dual union all
select 34109,'TUBI' from dual union all
select 34110,'INTA' from dual union all
select 34111,'RAE' from dual union all
select 34112,'PSU' from dual union all
select 34126,'AFCRL' from dual union all
select 34127,'LASL' from dual union all
select 34142,'KTH' from dual union all
select 34142,'RSRS' from dual union all
select 34143,'LRL' from dual union all
select 34144,'CAL' from dual union all
select 34145,'LEID' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 34145,'NAG' from dual union all
select 34147,'GMS' from dual union all
select 34148,'GMS' from dual union all
select 34150,'NRL' from dual union all
select 34152,'MIT' from dual union all
select 34153,'LAS' from dual union all
select 34159,'UCO' from dual union all
select 34160,'UCB' from dual union all
select 34161,'UWI' from dual union all
select 34164,'USCLA' from dual union all
select 34165,'CULH' from dual union all
select 34166,'MPE' from dual union all
select 34167,'RSRS' from dual union all
select 34168,'UCW' from dual union all
select 34172,'MSSL' from dual union all
select 34175,'UCO' from dual union all
select 34176,'ISAS' from dual union all
select 34176,'NRCC' from dual union all
select 34177,'MITLL' from dual union all
select 34178,'MPE' from dual union all
select 34179,'UCW' from dual union all
select 34180,'RSRS' from dual union all
select 34181,'QUB' from dual union all
select 34183,'NRCC' from dual union all
select 34187,'GSFC' from dual union all
select 34187,'NDRE' from dual union all
select 34188,'AFCRL' from dual union all
select 34193,'MPE' from dual union all
select 34194,'UCL' from dual union all
select 34196,'UCL' from dual union all
select 34198,'UCL' from dual union all
select 34199,'UCL' from dual union all
select 34202,'GMS' from dual union all
select 34204,'CAL' from dual union all
select 34205,'NRL' from dual union all
select 34215,'ADEL' from dual union all
select 34216,'ADEL' from dual union all
select 34218,'ADEL' from dual union all
select 34219,'AFCRL' from dual union all
select 34220,'MITLL' from dual union all
select 34221,'MITLL' from dual union all
select 3423,'NMSU' from dual union all
select 34291,'WRE' from dual union all
select 34293,'JHU' from dual union all
select 34304,'NRL' from dual union all
select 34315,'MPAE' from dual union all
select 34324,'MPAE' from dual union all
select 34326,'MPE' from dual union all
select 34327,'UCO' from dual union all
select 34328,'MPAE' from dual union all
select 34329,'NRL' from dual union all
select 34330,'ASE' from dual union all
select 34331,'ISAS' from dual union all
select 34332,'MPAE' from dual union all
select 34333,'UIUC' from dual union all
select 34334,'GSFC' from dual union all
select 34335,'LAS' from dual union all
select 34336,'MPAE' from dual union all
select 34338,'GSFC' from dual union all
select 34339,'MPAE' from dual union all
select 34341,'GSFC' from dual union all
select 34342,'ISAS' from dual union all
select 34343,'AERO' from dual union all
select 34344,'YORK' from dual union all
select 34345,'MPAE' from dual union all
select 34347,'MPAE' from dual union all
select 34350,'GSFC' from dual union all
select 34350,'NDRE' from dual union all
select 34355,'NASDA' from dual union all
select 34357,'MSSL' from dual union all
select 34357,'UCB' from dual union all
select 34363,'ESTEC' from dual union all
select 34363,'MPE' from dual union all
select 34363,'NDRE' from dual union all
select 34364,'AFCRL' from dual union all
select 34365,'CAL' from dual union all
select 34366,'STA' from dual union all
select 34367,'UCO' from dual union all
select 34369,'GMS' from dual union all
select 34370,'USAF' from dual union all
select 34371,'USAF' from dual union all
select 34372,'USAF' from dual union all
select 34373,'USAF' from dual union all
select 34374,'USAF' from dual union all
select 34375,'USAF' from dual union all
select 34376,'USAF' from dual union all
select 34506,'USAF' from dual union all
select 34507,'USAF' from dual union all
select 34508,'USAF' from dual union all
select 34509,'USAF' from dual union all
select 34510,'USAF' from dual union all
select 34512,'NASDA' from dual union all
select 34514,'NASDA' from dual union all
select 34648,'USAF' from dual union all
select 34649,'USAF' from dual union all
select 34650,'USAF' from dual union all
select 34651,'USAF' from dual union all
select 34652,'USAF' from dual union all
select 34653,'USAF' from dual union all
select 34654,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 34655,'USAF' from dual union all
select 34656,'USAF' from dual union all
select 34799,'USAF' from dual union all
select 34800,'USAF' from dual union all
select 34802,'USAF' from dual union all
select 34803,'USAF' from dual union all
select 34804,'USAF' from dual union all
select 34868,'USAF' from dual union all
select 34879,'USAF' from dual union all
select 34890,'USAF' from dual union all
select 34901,'USAF' from dual union all
select 34923,'GSFC' from dual union all
select 34927,'USAF' from dual union all
select 34928,'USAF' from dual union all
select 34929,'USAF' from dual union all
select 34930,'USAF' from dual union all
select 34931,'USAF' from dual union all
select 34932,'USAF' from dual union all
select 35034,'NASDA' from dual union all
select 35075,'JMA' from dual union all
select 35076,'JMA' from dual union all
select 35077,'JMA' from dual union all
select 35078,'JMA' from dual union all
select 35080,'JMA' from dual union all
select 35081,'JMA' from dual union all
select 35082,'JMA' from dual union all
select 35083,'JMA' from dual union all
select 35084,'JMA' from dual union all
select 35085,'JMA' from dual union all
select 35086,'JMA' from dual union all
select 35087,'JMA' from dual union all
select 35088,'JMA' from dual union all
select 35089,'JMA' from dual union all
select 35091,'JMA' from dual union all
select 35092,'JMA' from dual union all
select 35093,'JMA' from dual union all
select 35094,'JMA' from dual union all
select 35095,'JMA' from dual union all
select 35096,'JMA' from dual union all
select 35097,'JMA' from dual union all
select 35098,'JMA' from dual union all
select 35099,'JMA' from dual union all
select 35100,'JMA' from dual union all
select 35102,'JMA' from dual union all
select 35103,'JMA' from dual union all
select 35104,'JMA' from dual union all
select 35105,'JMA' from dual union all
select 35106,'JMA' from dual union all
select 35107,'JMA' from dual union all
select 35108,'JMA' from dual union all
select 35109,'JMA' from dual union all
select 35110,'JMA' from dual union all
select 35111,'JMA' from dual union all
select 35113,'JMA' from dual union all
select 35114,'JMA' from dual union all
select 35115,'JMA' from dual union all
select 35116,'JMA' from dual union all
select 35117,'JMA' from dual union all
select 35119,'ASL' from dual union all
select 35120,'ASL' from dual union all
select 35121,'ASL' from dual union all
select 35122,'ASL' from dual union all
select 35124,'ASL' from dual union all
select 35125,'ASL' from dual union all
select 35126,'ASL' from dual union all
select 35127,'ASL' from dual union all
select 35128,'ASL' from dual union all
select 35129,'ASL' from dual union all
select 35130,'ASL' from dual union all
select 35131,'ASL' from dual union all
select 35132,'ASL' from dual union all
select 35133,'ASL' from dual union all
select 35135,'ASL' from dual union all
select 35136,'ASL' from dual union all
select 35137,'ASL' from dual union all
select 35138,'ASL' from dual union all
select 35139,'ASL' from dual union all
select 35140,'ASL' from dual union all
select 35141,'ASL' from dual union all
select 35142,'ASL' from dual union all
select 35143,'ASL' from dual union all
select 35144,'ASL' from dual union all
select 35146,'ASL' from dual union all
select 35147,'ASL' from dual union all
select 35163,'NASDA' from dual union all
select 35219,'WFF' from dual union all
select 35253,'USAF' from dual union all
select 35254,'USAF' from dual union all
select 35255,'USAF' from dual union all
select 35256,'USAF' from dual union all
select 35257,'USAF' from dual union all
select 35351,'WFF' from dual union all
select 35352,'WFF' from dual union all
select 35353,'WFF' from dual union all
select 35354,'WFF' from dual union all
select 35355,'WFF' from dual union all
select 35356,'WFF' from dual union all
select 35358,'WFF' from dual union all
select 35359,'WFF' from dual union all
select 35360,'WFF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 35361,'WFF' from dual union all
select 35362,'WFF' from dual union all
select 35363,'WFF' from dual union all
select 35364,'WFF' from dual union all
select 35365,'WFF' from dual union all
select 35366,'WFF' from dual union all
select 35369,'WFF' from dual union all
select 35370,'WFF' from dual union all
select 35371,'WFF' from dual union all
select 35372,'WFF' from dual union all
select 35373,'WFF' from dual union all
select 35374,'WFF' from dual union all
select 35375,'WFF' from dual union all
select 35376,'WFF' from dual union all
select 35377,'WFF' from dual union all
select 35378,'WFF' from dual union all
select 35380,'WFF' from dual union all
select 35381,'WFF' from dual union all
select 35382,'WFF' from dual union all
select 35383,'WFF' from dual union all
select 35384,'WFF' from dual union all
select 35385,'WFF' from dual union all
select 35386,'WFF' from dual union all
select 35387,'WFF' from dual union all
select 35388,'WFF' from dual union all
select 35389,'WFF' from dual union all
select 35391,'WFF' from dual union all
select 35392,'WFF' from dual union all
select 35393,'WFF' from dual union all
select 35394,'WFF' from dual union all
select 35395,'WFF' from dual union all
select 35396,'WFF' from dual union all
select 35397,'WFF' from dual union all
select 35398,'WFF' from dual union all
select 35399,'WFF' from dual union all
select 35400,'WFF' from dual union all
select 35402,'WFF' from dual union all
select 35403,'WFF' from dual union all
select 35404,'WFF' from dual union all
select 35405,'WFF' from dual union all
select 35407,'WFF' from dual union all
select 35408,'WFF' from dual union all
select 35409,'WFF' from dual union all
select 35410,'WFF' from dual union all
select 35411,'WFF' from dual union all
select 35413,'WFF' from dual union all
select 35414,'WFF' from dual union all
select 35415,'WFF' from dual union all
select 35416,'WFF' from dual union all
select 35417,'WFF' from dual union all
select 35418,'WFF' from dual union all
select 35419,'WFF' from dual union all
select 35420,'WFF' from dual union all
select 35421,'WFF' from dual union all
select 35422,'WFF' from dual union all
select 35424,'WFF' from dual union all
select 35426,'WFF' from dual union all
select 35427,'WFF' from dual union all
select 35428,'WFF' from dual union all
select 35429,'WFF' from dual union all
select 35430,'WFF' from dual union all
select 35440,'NRCC' from dual union all
select 35445,'PSU' from dual union all
select 35572,'CALG' from dual union all
select 35607,'USAF' from dual union all
select 35608,'USAF' from dual union all
select 35609,'USAF' from dual union all
select 35611,'USAF' from dual union all
select 3575,'AFCRL' from dual union all
select 3575,'UUT' from dual union all
select 35754,'USAF' from dual union all
select 35755,'USAF' from dual union all
select 35756,'USAF' from dual union all
select 35757,'USAF' from dual union all
select 3576,'AFCRL' from dual union all
select 3577,'NRL' from dual union all
select 3578,'ONR' from dual union all
select 3578,'UMD' from dual union all
select 35787,'PIHM' from dual union all
select 3579,'ONR' from dual union all
select 3579,'UMD' from dual union all
select 3580,'AFCRL' from dual union all
select 3580,'UCO' from dual union all
select 3580,'UUT' from dual union all
select 3581,'NRL' from dual union all
select 3582,'NRL' from dual union all
select 3583,'ONR' from dual union all
select 3583,'UMD' from dual union all
select 3584,'ONR' from dual union all
select 3584,'UMD' from dual union all
select 35887,'AFCRL' from dual union all
select 35888,'AFCRL' from dual union all
select 35888,'WFF' from dual union all
select 35889,'WFF' from dual union all
select 35890,'WFF' from dual union all
select 35891,'WSMR' from dual union all
select 35892,'WFF' from dual union all
select 35893,'AFCRL' from dual union all
select 35893,'WFF' from dual union all
select 35894,'WFF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 35896,'WFF' from dual union all
select 35897,'USAF' from dual union all
select 35898,'USAF' from dual union all
select 35899,'USAF' from dual union all
select 35900,'USAF' from dual union all
select 35901,'USAF' from dual union all
select 35902,'USAF' from dual union all
select 35903,'USAF' from dual union all
select 35904,'USAF' from dual union all
select 35905,'USAF' from dual union all
select 35907,'AFCRL' from dual union all
select 35907,'WFF' from dual union all
select 35908,'WFF' from dual union all
select 35909,'PSU' from dual union all
select 35910,'WFF' from dual union all
select 35911,'WFF' from dual union all
select 35912,'AFCRL' from dual union all
select 35912,'WFF' from dual union all
select 35913,'WFF' from dual union all
select 35914,'AFCRL' from dual union all
select 35914,'WFF' from dual union all
select 35915,'WFF' from dual union all
select 35916,'AFCRL' from dual union all
select 35916,'WFF' from dual union all
select 35918,'WFF' from dual union all
select 35919,'AFCRL' from dual union all
select 35919,'WFF' from dual union all
select 3592,'AFCRL' from dual union all
select 35920,'AFCRL' from dual union all
select 35920,'WFF' from dual union all
select 35921,'WFF' from dual union all
select 35922,'AFCRL' from dual union all
select 35923,'AFCRL' from dual union all
select 35923,'WFF' from dual union all
select 35924,'WFF' from dual union all
select 3593,'FIAN' from dual union all
select 3593,'IAM' from dual union all
select 3597,'FIAN' from dual union all
select 3597,'IAM' from dual union all
select 3598,'NRL' from dual union all
select 3599,'NRL' from dual union all
select 3601,'NRL' from dual union all
select 36018,'HOUS' from dual union all
select 3602,'LMSC' from dual union all
select 36030,'WFF' from dual union all
select 36032,'WFF' from dual union all
select 36033,'WFF' from dual union all
select 36035,'USAF' from dual union all
select 36036,'USAF' from dual union all
select 36037,'USAF' from dual union all
select 36039,'USAF' from dual union all
select 3604,'NRL' from dual union all
select 36040,'USAF' from dual union all
select 36041,'USAF' from dual union all
select 36042,'USAF' from dual union all
select 3605,'AFCRL' from dual union all
select 3605,'UCO' from dual union all
select 3605,'UUT' from dual union all
select 36093,'GSFC' from dual union all
select 36100,'NRCC' from dual union all
select 36135,'INTA' from dual union all
select 36139,'VSSC' from dual union all
select 36179,'PSU' from dual union all
select 36208,'PSU' from dual union all
select 36209,'PSU' from dual union all
select 36210,'WSMR' from dual union all
select 36274,'GMS' from dual union all
select 36284,'GMS' from dual union all
select 36286,'GMS' from dual union all
select 36287,'GMS' from dual union all
select 36288,'GMS' from dual union all
select 36290,'GMS' from dual union all
select 36291,'GMS' from dual union all
select 36294,'GMS' from dual union all
select 36296,'GMS' from dual union all
select 36298,'GMS' from dual union all
select 3638,'NRL' from dual union all
select 3649,'FIAN' from dual union all
select 3649,'IAM' from dual union all
select 36588,'SAND' from dual union all
select 36590,'GSFC' from dual union all
select 36594,'SAND' from dual union all
select 36606,'GSFC' from dual union all
select 36607,'SAND' from dual union all
select 36609,'UCO' from dual union all
select 36610,'SDCO' from dual union all
select 36621,'SAND' from dual union all
select 36630,'SAND' from dual union all
select 36631,'SAND' from dual union all
select 36636,'GSFC' from dual union all
select 36648,'WFF' from dual union all
select 36651,'NPL' from dual union all
select 36652,'ASL' from dual union all
select 36662,'PSU' from dual union all
select 36663,'CNES' from dual union all
select 36664,'CNES' from dual union all
select 36665,'CNES' from dual union all
select 36666,'CNES' from dual union all
select 36667,'CNES' from dual union all
select 36668,'CNES' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 36669,'CNES' from dual union all
select 36670,'CNES' from dual union all
select 36671,'CNES' from dual union all
select 36672,'CNES' from dual union all
select 36673,'CNES' from dual union all
select 36674,'CNES' from dual union all
select 36675,'CNES' from dual union all
select 36676,'CNES' from dual union all
select 36677,'CNES' from dual union all
select 36678,'CNES' from dual union all
select 36679,'CNES' from dual union all
select 3668,'AFCRL' from dual union all
select 3668,'UUT' from dual union all
select 3669,'NRL' from dual union all
select 36692,'GSFC' from dual union all
select 36693,'AFCRL' from dual union all
select 36694,'SSC' from dual union all
select 36695,'MPE' from dual union all
select 36695,'TUBI' from dual union all
select 36696,'SAO' from dual union all
select 36697,'JHU' from dual union all
select 36698,'PRL' from dual union all
select 36698,'SOTON' from dual union all
select 36699,'PRL' from dual union all
select 36699,'SOTON' from dual union all
select 36701,'UMN' from dual union all
select 36701,'UNH' from dual union all
select 36703,'PRL' from dual union all
select 36703,'SOTON' from dual union all
select 36704,'UMN' from dual union all
select 36704,'UNH' from dual union all
select 36705,'PRL' from dual union all
select 36708,'UMI' from dual union all
select 36709,'GSFC' from dual union all
select 36710,'AFCRL' from dual union all
select 36711,'AFCRL' from dual union all
select 36712,'AFCRL' from dual union all
select 36715,'GSFC' from dual union all
select 36716,'AFCRL' from dual union all
select 36717,'GSFC' from dual union all
select 36718,'GSFC' from dual union all
select 3672,'NRL' from dual union all
select 36722,'PRL' from dual union all
select 36723,'AFCRL' from dual union all
select 36725,'AFCRL' from dual union all
select 36726,'AFCRL' from dual union all
select 36727,'PRL' from dual union all
select 3673,'AFCRL' from dual union all
select 3673,'UCO' from dual union all
select 36731,'AFCRL' from dual union all
select 36732,'AFCRL' from dual union all
select 36733,'AFCRL' from dual union all
select 36738,'UCW' from dual union all
select 36739,'MPAE' from dual union all
select 36740,'MPAE' from dual union all
select 36742,'MPAE' from dual union all
select 36743,'MPAE' from dual union all
select 36745,'RAE' from dual union all
select 36747,'SAND' from dual union all
select 36748,'SAND' from dual union all
select 36753,'GSFC' from dual union all
select 36753,'JPL' from dual union all
select 36756,'GSFC' from dual union all
select 36758,'GSFC' from dual union all
select 36759,'DUD' from dual union all
select 36760,'DUD' from dual union all
select 36764,'UIUC' from dual union all
select 36767,'PITT' from dual union all
select 36768,'ISAS' from dual union all
select 36769,'UIUC' from dual union all
select 3677,'PARD' from dual union all
select 3677,'UMI' from dual union all
select 36779,'USAF' from dual union all
select 36785,'WSMR' from dual union all
select 36786,'WSMR' from dual union all
select 36787,'WSMR' from dual union all
select 36788,'LAS' from dual union all
select 36789,'UIUC' from dual union all
select 36791,'GCA' from dual union all
select 36794,'GSFC' from dual union all
select 36795,'GCA' from dual union all
select 36796,'AFCRL' from dual union all
select 36797,'MPE' from dual union all
select 36797,'NDRE' from dual union all
select 36815,'LUX' from dual union all
select 36819,'GSFC' from dual union all
select 3682,'AFCRL' from dual union all
select 36827,'RAE' from dual union all
select 36828,'GSFC' from dual union all
select 36829,'GMS' from dual union all
select 36829,'IKI' from dual union all
select 36830,'NRL' from dual union all
select 36835,'GSFC' from dual union all
select 36838,'UCO' from dual union all
select 36842,'USAF' from dual union all
select 36844,'AFCRL' from dual union all
select 36846,'GSFC' from dual union all
select 36850,'AFCRL' from dual union all
select 36852,'MIT' from dual union all
select 36853,'NASDA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 36854,'GSFC' from dual union all
select 36862,'ISAS' from dual union all
select 36864,'UCO' from dual union all
select 36870,'ISAS' from dual union all
select 36871,'CLSR' from dual union all
select 36871,'IKI' from dual union all
select 36872,'ISAS' from dual union all
select 36873,'LRL' from dual union all
select 36876,'RSRS' from dual union all
select 36877,'QUB' from dual union all
select 36879,'UCW' from dual union all
select 36880,'WFF' from dual union all
select 36882,'IZM' from dual union all
select 36884,'SAND' from dual union all
select 36885,'AFCRL' from dual union all
select 36887,'AFCRL' from dual union all
select 36888,'44SMW' from dual union all
select 36890,'LRL' from dual union all
select 36891,'ISAS' from dual union all
select 36892,'ISAS' from dual union all
select 36893,'UTD' from dual union all
select 36901,'MPE' from dual union all
select 36904,'MPAE' from dual union all
select 36908,'MPE' from dual union all
select 36909,'NRL' from dual union all
select 36910,'MPIH' from dual union all
select 36912,'UCO' from dual union all
select 36914,'AFCRL' from dual union all
select 36914,'CORN' from dual union all
select 36916,'AFCRL' from dual union all
select 36917,'MIT' from dual union all
select 36918,'INTA' from dual union all
select 36920,'GSFC' from dual union all
select 36922,'LASL' from dual union all
select 36922,'UAF' from dual union all
select 36923,'SDCO' from dual union all
select 36930,'UWI' from dual union all
select 36933,'USAF' from dual union all
select 36935,'WFF' from dual union all
select 36936,'GSFC' from dual union all
select 36937,'GSFC' from dual union all
select 36938,'SSC' from dual union all
select 3694,'HADC' from dual union all
select 36940,'CEA' from dual union all
select 36940,'LUX' from dual union all
select 36941,'LASL' from dual union all
select 36942,'UCW' from dual union all
select 36943,'RSRS' from dual union all
select 36944,'LASL' from dual union all
select 36944,'UAF' from dual union all
select 36945,'LASL' from dual union all
select 36950,'AFCRL' from dual union all
select 36951,'AFCRL' from dual union all
select 36952,'AFCRL' from dual union all
select 36953,'AFCRL' from dual union all
select 36954,'AFCRL' from dual union all
select 36956,'AFCRL' from dual union all
select 36957,'GSFC' from dual union all
select 36958,'BAL' from dual union all
select 36959,'USAF' from dual union all
select 36961,'AFCRL' from dual union all
select 36962,'GSFC' from dual union all
select 36963,'WRE' from dual union all
select 36964,'SAO' from dual union all
select 36965,'NRL' from dual union all
select 36966,'MPE' from dual union all
select 36967,'MSFC' from dual union all
select 36968,'AFCRL' from dual union all
select 36969,'GSFC' from dual union all
select 36971,'MPAE' from dual union all
select 36974,'ISRO' from dual union all
select 36976,'ADEL' from dual union all
select 36977,'AFCRL' from dual union all
select 36978,'ISAS' from dual union all
select 36982,'WRE' from dual union all
select 36988,'AFCRL' from dual union all
select 3699,'UMI' from dual union all
select 36999,'ISAS' from dual union all
select 37010,'AFCRL' from dual union all
select 37016,'INTA' from dual union all
select 37017,'INTA' from dual union all
select 37018,'INTA' from dual union all
select 37019,'INTA' from dual union all
select 37020,'AFCRL' from dual union all
select 37021,'INTA' from dual union all
select 37022,'INTA' from dual union all
select 37023,'WRE' from dual union all
select 37024,'WRE' from dual union all
select 37025,'UDAYT' from dual union all
select 37031,'AFCRL' from dual union all
select 37037,'LPSP' from dual union all
select 37037,'UCO' from dual union all
select 37038,'NAG' from dual union all
select 37039,'MPAE' from dual union all
select 37040,'MPAE' from dual union all
select 37041,'CESR' from dual union all
select 37041,'IKI' from dual union all
select 37041,'IZM' from dual union all
select 37043,'LASL' from dual union all
select 37046,'MPAE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 37047,'MPAE' from dual union all
select 37048,'LASL' from dual union all
select 37049,'GSFC' from dual union all
select 3705,'NRL' from dual union all
select 37050,'LPARL' from dual union all
select 37053,'LASL' from dual union all
select 37055,'MPAE' from dual union all
select 37056,'UCL' from dual union all
select 37057,'UCL' from dual union all
select 37058,'PRL' from dual union all
select 37058,'UCL' from dual union all
select 37059,'BIR' from dual union all
select 37059,'PRL' from dual union all
select 37060,'PRL' from dual union all
select 37060,'UCL' from dual union all
select 37061,'PRL' from dual union all
select 37061,'UCL' from dual union all
select 37062,'PITT' from dual union all
select 37063,'UWI' from dual union all
select 37064,'PITT' from dual union all
select 37065,'UMI' from dual union all
select 37066,'CESR' from dual union all
select 37066,'IKI' from dual union all
select 37066,'IZM' from dual union all
select 37067,'PRL' from dual union all
select 37067,'UCL' from dual union all
select 37068,'UCL' from dual union all
select 37069,'BIR' from dual union all
select 37070,'PRL' from dual union all
select 37070,'UCL' from dual union all
select 37071,'UCL' from dual union all
select 37072,'CULH' from dual union all
select 37073,'PITT' from dual union all
select 37075,'AFCRL' from dual union all
select 37077,'RAE' from dual union all
select 37079,'AFCRL' from dual union all
select 37080,'RICE' from dual union all
select 37081,'DNA' from dual union all
select 37082,'MISU' from dual union all
select 37082,'SSC' from dual union all
select 37084,'DNA' from dual union all
select 37085,'DNA' from dual union all
select 37086,'MISU' from dual union all
select 37087,'UWI' from dual union all
select 37088,'DNA' from dual union all
select 37089,'JHU' from dual union all
select 37090,'SAND' from dual union all
select 37091,'AFCRL' from dual union all
select 37092,'AFCRL' from dual union all
select 37093,'USAF' from dual union all
select 37094,'USAF' from dual union all
select 37095,'USAF' from dual union all
select 37096,'USAF' from dual union all
select 37097,'USAF' from dual union all
select 37113,'NASDA' from dual union all
select 37189,'UTEP' from dual union all
select 37189,'WSMR' from dual union all
select 37203,'USAF' from dual union all
select 37204,'USAF' from dual union all
select 37205,'USAF' from dual union all
select 37206,'USAF' from dual union all
select 37207,'USAF' from dual union all
select 37208,'USAF' from dual union all
select 37209,'USAF' from dual union all
select 37210,'USAF' from dual union all
select 3722,'AFCRL' from dual union all
select 3722,'UMI' from dual union all
select 37225,'NASDA' from dual union all
select 3723,'SUI' from dual union all
select 3724,'SUI' from dual union all
select 3725,'SUI' from dual union all
select 3726,'NRL' from dual union all
select 37269,'USAF' from dual union all
select 3727,'SUI' from dual union all
select 3728,'SUI' from dual union all
select 37280,'USAF' from dual union all
select 3729,'NRL' from dual union all
select 3730,'SUI' from dual union all
select 3731,'SUI' from dual union all
select 3732,'SUI' from dual union all
select 3733,'SUI' from dual union all
select 3734,'SUI' from dual union all
select 37346,'CALG' from dual union all
select 3737,'SUI' from dual union all
select 3738,'SUI' from dual union all
select 3740,'SUI' from dual union all
select 3742,'NRL' from dual union all
select 37428,'WFF' from dual union all
select 3743,'AFCRL' from dual union all
select 3744,'SUI' from dual union all
select 3745,'SUI' from dual union all
select 3746,'NRL' from dual union all
select 3747,'NRL' from dual union all
select 3750,'NRL' from dual union all
select 3750,'SBRC' from dual union all
select 3751,'NRL' from dual union all
select 3751,'SBRC' from dual union all
select 3752,'CNRS' from dual union all
select 3752,'PARIS' from dual union all
select 37534,'GSFC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 37563,'GSFC' from dual union all
select 37585,'WFF' from dual union all
select 37586,'WFF' from dual union all
select 37587,'WFF' from dual union all
select 37588,'WFF' from dual union all
select 37589,'WFF' from dual union all
select 37590,'WFF' from dual union all
select 37592,'WFF' from dual union all
select 37593,'WFF' from dual union all
select 37594,'WFF' from dual union all
select 37595,'WFF' from dual union all
select 37596,'WFF' from dual union all
select 37597,'WFF' from dual union all
select 37598,'WFF' from dual union all
select 37599,'WFF' from dual union all
select 37600,'WFF' from dual union all
select 37601,'WFF' from dual union all
select 37603,'WFF' from dual union all
select 37604,'WFF' from dual union all
select 37605,'WFF' from dual union all
select 37606,'ASL' from dual union all
select 37607,'ASL' from dual union all
select 37608,'ASL' from dual union all
select 37609,'ASL' from dual union all
select 37610,'ASL' from dual union all
select 37611,'ASL' from dual union all
select 37612,'ASL' from dual union all
select 37613,'NASDA' from dual union all
select 37614,'ASL' from dual union all
select 37615,'ASL' from dual union all
select 37616,'ASL' from dual union all
select 37617,'ASL' from dual union all
select 37618,'ASL' from dual union all
select 37619,'ASL' from dual union all
select 37620,'ASL' from dual union all
select 37621,'ASL' from dual union all
select 37622,'ASL' from dual union all
select 3766,'NRL' from dual union all
select 37680,'WFF' from dual union all
select 37683,'WFF' from dual union all
select 37694,'WFF' from dual union all
select 37724,'USAF' from dual union all
select 37726,'USAF' from dual union all
select 37727,'USAF' from dual union all
select 37728,'USAF' from dual union all
select 37729,'USAF' from dual union all
select 3776,'ISAS' from dual union all
select 3779,'ISAS' from dual union all
select 3780,'NRL' from dual union all
select 3780,'SBRC' from dual union all
select 37803,'GSFC' from dual union all
select 3781,'ISAS' from dual union all
select 3782,'AFCRL' from dual union all
select 3783,'ISAS' from dual union all
select 3784,'AFCRL' from dual union all
select 3785,'AFCRL' from dual union all
select 3786,'AFCRL' from dual union all
select 37883,'USAF' from dual union all
select 37884,'USAF' from dual union all
select 37885,'USAF' from dual union all
select 37922,'WFF' from dual union all
select 37978,'WFF' from dual union all
select 37989,'WFF' from dual union all
select 38013,'USAF' from dual union all
select 38014,'USAF' from dual union all
select 38015,'USAF' from dual union all
select 38016,'USAF' from dual union all
select 38017,'USAF' from dual union all
select 38128,'GSFC' from dual union all
select 38129,'GSFC' from dual union all
select 38130,'WFF' from dual union all
select 38131,'WFF' from dual union all
select 38132,'WFF' from dual union all
select 38133,'WFF' from dual union all
select 38166,'USAF' from dual union all
select 38167,'USAF' from dual union all
select 38168,'USAF' from dual union all
select 38169,'USAF' from dual union all
select 38170,'USAF' from dual union all
select 38171,'USAF' from dual union all
select 38172,'USAF' from dual union all
select 38173,'USAF' from dual union all
select 38174,'USAF' from dual union all
select 38175,'USAF' from dual union all
select 38177,'USAF' from dual union all
select 38178,'USAF' from dual union all
select 38179,'USAF' from dual union all
select 38209,'WFF' from dual union all
select 38220,'WFF' from dual union all
select 3824,'NRL' from dual union all
select 3824,'SBRC' from dual union all
select 38285,'USAF' from dual union all
select 38287,'USAF' from dual union all
select 38288,'USAF' from dual union all
select 38289,'USAF' from dual union all
select 38290,'USAF' from dual union all
select 38291,'USAF' from dual union all
select 3835,'NRL' from dual union all
select 3835,'SBRC' from dual union all
select 38392,'WFF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 38394,'GSFC' from dual union all
select 38404,'USAF' from dual union all
select 38405,'USAF' from dual union all
select 38406,'USAF' from dual union all
select 38408,'USAF' from dual union all
select 38456,'WFF' from dual union all
select 3846,'NRL' from dual union all
select 3846,'SBRC' from dual union all
select 38538,'USAF' from dual union all
select 38540,'USAF' from dual union all
select 38541,'USAF' from dual union all
select 38542,'USAF' from dual union all
select 3857,'NRL' from dual union all
select 3857,'SBRC' from dual union all
select 38637,'PSU' from dual union all
select 38638,'PSU' from dual union all
select 38639,'GSFC' from dual union all
select 38649,'PSU' from dual union all
select 38650,'PSU' from dual union all
select 38707,'WFF' from dual union all
select 38717,'GSFC' from dual union all
select 38718,'GSFC' from dual union all
select 38773,'TSAO' from dual union all
select 38779,'HOUS' from dual union all
select 3878,'NMSU' from dual union all
select 38790,'HOUS' from dual union all
select 3880,'NMSU' from dual union all
select 3881,'NMSU' from dual union all
select 3882,'NMSU' from dual union all
select 3883,'NMSU' from dual union all
select 3884,'NMSU' from dual union all
select 38849,'PRL' from dual union all
select 3885,'NMSU' from dual union all
select 3886,'NMSU' from dual union all
select 3887,'NMSU' from dual union all
select 38870,'GSFC' from dual union all
select 3888,'NMSU' from dual union all
select 38893,'HOUS' from dual union all
select 38901,'HOUS' from dual union all
select 38902,'HOUS' from dual union all
select 38948,'GMS' from dual union all
select 39083,'MPAE' from dual union all
select 39086,'KOLN' from dual union all
select 39090,'UAF' from dual union all
select 39091,'INPE' from dual union all
select 39092,'PITT' from dual union all
select 39096,'BONN' from dual union all
select 39104,'AFCRL' from dual union all
select 39124,'GSFC' from dual union all
select 39132,'AFCRL' from dual union all
select 39143,'PSU' from dual union all
select 39147,'ISRO' from dual union all
select 39163,'SAND' from dual union all
select 39164,'WFF' from dual union all
select 39165,'DNA' from dual union all
select 39166,'DNA' from dual union all
select 39167,'DNA' from dual union all
select 39168,'DNA' from dual union all
select 39169,'WRE' from dual union all
select 39173,'PSU' from dual union all
select 39174,'INTA' from dual union all
select 39174,'MPAE' from dual union all
select 39175,'MPAE' from dual union all
select 39176,'INTA' from dual union all
select 39176,'MPAE' from dual union all
select 39177,'SDCO' from dual union all
select 39178,'PSU' from dual union all
select 39179,'NRCC' from dual union all
select 39180,'PRL' from dual union all
select 39181,'GSFC' from dual union all
select 39182,'CNES' from dual union all
select 39183,'CNES' from dual union all
select 39184,'UDAYT' from dual union all
select 39185,'CNES' from dual union all
select 39186,'CNES' from dual union all
select 39187,'CNES' from dual union all
select 39188,'CNES' from dual union all
select 39189,'CNES' from dual union all
select 39190,'CNES' from dual union all
select 39191,'CNES' from dual union all
select 39192,'CNES' from dual union all
select 39193,'CNES' from dual union all
select 39194,'CNES' from dual union all
select 39195,'CNES' from dual union all
select 39196,'UDAYT' from dual union all
select 39197,'CNES' from dual union all
select 39198,'UDAYT' from dual union all
select 39199,'UDAYT' from dual union all
select 39210,'KYO' from dual union all
select 39210,'NIPR' from dual union all
select 39213,'RICE' from dual union all
select 39214,'UCO' from dual union all
select 39215,'351SMW' from dual union all
select 39216,'RAE' from dual union all
select 39217,'MPIK' from dual union all
select 39217,'NDRE' from dual union all
select 39217,'UIO' from dual union all
select 39218,'AFCRL' from dual union all
select 39221,'AFCRL' from dual union all
select 39222,'WRE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 39225,'AFCRL' from dual union all
select 39226,'UTD' from dual union all
select 39227,'GMS' from dual union all
select 39228,'AN' from dual union all
select 39230,'RICE' from dual union all
select 39232,'UCL' from dual union all
select 39232,'UIO' from dual union all
select 39233,'GMS' from dual union all
select 39234,'AFCRL' from dual union all
select 39236,'GMS' from dual union all
select 39237,'UCO' from dual union all
select 39238,'JHU' from dual union all
select 39240,'KGI' from dual union all
select 39240,'UIO' from dual union all
select 39241,'GMS' from dual union all
select 39243,'GMS' from dual union all
select 39246,'GSFC' from dual union all
select 39252,'UMI' from dual union all
select 39254,'UMI' from dual union all
select 39255,'GSFC' from dual union all
select 39256,'ISAS' from dual union all
select 39256,'ISRO' from dual union all
select 39257,'ISAS' from dual union all
select 39257,'ISRO' from dual union all
select 39258,'JHU' from dual union all
select 39259,'AFCRL' from dual union all
select 39260,'ERDA' from dual union all
select 39262,'GSFC' from dual union all
select 39263,'GSFC' from dual union all
select 39265,'GSFC' from dual union all
select 39267,'GSFC' from dual union all
select 39268,'UAF' from dual union all
select 39269,'ARC' from dual union all
select 39269,'LPARL' from dual union all
select 39270,'MPE' from dual union all
select 39271,'UTD' from dual union all
select 39272,'AFCRL' from dual union all
select 39273,'SDCO' from dual union all
select 39277,'ISRO' from dual union all
select 39280,'NRL' from dual union all
select 39284,'WRE' from dual union all
select 39285,'WSMR' from dual union all
select 39291,'AFCRL' from dual union all
select 39292,'MPAE' from dual union all
select 39293,'MSSL' from dual union all
select 39294,'UCW' from dual union all
select 39295,'CIT' from dual union all
select 39295,'GSFC' from dual union all
select 39296,'AERO' from dual union all
select 39298,'BIR' from dual union all
select 39298,'MSFC' from dual union all
select 39298,'MSSL' from dual union all
select 39299,'UAF' from dual union all
select 39302,'MSFC' from dual union all
select 39303,'AFCRL' from dual union all
select 39304,'JPL' from dual union all
select 39306,'BAC' from dual union all
select 39306,'UCL' from dual union all
select 39307,'LEID' from dual union all
select 39307,'NAG' from dual union all
select 39314,'SHEF' from dual union all
select 39315,'SHEF' from dual union all
select 39316,'LASL' from dual union all
select 39320,'MSSL' from dual union all
select 39321,'WFF' from dual union all
select 39323,'SAO' from dual union all
select 39324,'BIR' from dual union all
select 39329,'NIPR' from dual union all
select 39335,'BONN' from dual union all
select 39339,'GSFC' from dual union all
select 39340,'GSFC' from dual union all
select 39342,'WRE' from dual union all
select 39344,'WSMR' from dual union all
select 39346,'RSRS' from dual union all
select 39347,'GSFC' from dual union all
select 39349,'MPE' from dual union all
select 39349,'MSSL' from dual union all
select 39350,'SAND' from dual union all
select 39352,'NIPR' from dual union all
select 39353,'GSFC' from dual union all
select 39354,'ISRO' from dual union all
select 39355,'MITLL' from dual union all
select 39366,'AFCRL' from dual union all
select 39367,'UHI' from dual union all
select 39368,'PSU' from dual union all
select 39369,'GSFC' from dual union all
select 39371,'UIUC' from dual union all
select 39372,'NIPR' from dual union all
select 39374,'ISAS' from dual union all
select 39375,'GMS' from dual union all
select 39393,'ISAS' from dual union all
select 39394,'DMI' from dual union all
select 39394,'DSRI' from dual union all
select 39397,'DSRI' from dual union all
select 39400,'ISAS' from dual union all
select 39410,'AFCRL' from dual union all
select 39411,'NIPR' from dual union all
select 39418,'NIPR' from dual union all
select 39419,'ISAS' from dual union all
select 39420,'ASE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 39421,'ISAS' from dual union all
select 39423,'INPE' from dual union all
select 39424,'GSFC' from dual union all
select 39425,'GSFC' from dual union all
select 39426,'NASDA' from dual union all
select 39431,'ISRO' from dual union all
select 39432,'ISRO' from dual union all
select 39434,'NRL' from dual union all
select 39442,'TIFR' from dual union all
select 39444,'UCO' from dual union all
select 39445,'NRL' from dual union all
select 39448,'LUX' from dual union all
select 39452,'NRCC' from dual union all
select 39454,'DNA' from dual union all
select 39455,'SDCO' from dual union all
select 39456,'ASE' from dual union all
select 39457,'ISRO' from dual union all
select 39458,'AFCRL' from dual union all
select 39458,'DNA' from dual union all
select 39459,'UCB' from dual union all
select 39465,'MSSL' from dual union all
select 39465,'RSRS' from dual union all
select 39466,'RSRS' from dual union all
select 39469,'DNA' from dual union all
select 39470,'NDRE' from dual union all
select 39470,'OSLO' from dual union all
select 39471,'NDRE' from dual union all
select 39471,'OSLO' from dual union all
select 39474,'RSRS' from dual union all
select 39478,'NDRE' from dual union all
select 39478,'RSRS' from dual union all
select 39478,'UCL' from dual union all
select 39479,'UCL' from dual union all
select 39481,'MSFC' from dual union all
select 39485,'NRL' from dual union all
select 39491,'ISAS' from dual union all
select 39495,'LASL' from dual union all
select 39495,'SAND' from dual union all
select 39496,'WRE' from dual union all
select 39497,'WRE' from dual union all
select 39498,'ADEL' from dual union all
select 39499,'ADEL' from dual union all
select 39500,'DNA' from dual union all
select 39502,'WRE' from dual union all
select 39531,'INTA' from dual union all
select 39531,'MPAE' from dual union all
select 39532,'INTA' from dual union all
select 39532,'MPAE' from dual union all
select 39533,'INTA' from dual union all
select 39534,'INTA' from dual union all
select 39535,'INTA' from dual union all
select 39536,'INTA' from dual union all
select 39537,'INTA' from dual union all
select 39539,'INTA' from dual union all
select 39540,'INTA' from dual union all
select 39541,'INTA' from dual union all
select 39542,'INTA' from dual union all
select 39543,'INTA' from dual union all
select 39544,'INTA' from dual union all
select 39545,'INTA' from dual union all
select 39546,'INTA' from dual union all
select 39547,'INTA' from dual union all
select 39548,'INTA' from dual union all
select 39550,'INTA' from dual union all
select 39551,'INTA' from dual union all
select 39552,'MPAE' from dual union all
select 39553,'MPAE' from dual union all
select 39554,'MPAE' from dual union all
select 39555,'MPAE' from dual union all
select 39556,'ADEL' from dual union all
select 39557,'WRE' from dual union all
select 39559,'WRE' from dual union all
select 39561,'WRE' from dual union all
select 39562,'UDAYT' from dual union all
select 39578,'MPAE' from dual union all
select 39583,'AN' from dual union all
select 39584,'INTA' from dual union all
select 39584,'MPAE' from dual union all
select 39585,'MPE' from dual union all
select 39586,'BONN' from dual union all
select 39587,'MPAE' from dual union all
select 39588,'UCL' from dual union all
select 39589,'MPAE' from dual union all
select 39590,'MPAE' from dual union all
select 39593,'MPAE' from dual union all
select 39594,'GSFC' from dual union all
select 39594,'SSC' from dual union all
select 39595,'MPAE' from dual union all
select 39596,'MPAE' from dual union all
select 39597,'GSFC' from dual union all
select 39598,'ISAS' from dual union all
select 39599,'UIUC' from dual union all
select 39600,'ISAS' from dual union all
select 39603,'AFCRL' from dual union all
select 39604,'MPE' from dual union all
select 39605,'BONN' from dual union all
select 39606,'UCL' from dual union all
select 39607,'UCL' from dual union all
select 39608,'MPAE' from dual union all
select 39609,'MPAE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 39610,'KGI' from dual union all
select 39610,'UIO' from dual union all
select 39612,'MPAE' from dual union all
select 39613,'AFCRL' from dual union all
select 39614,'AFCRL' from dual union all
select 39617,'NIPR' from dual union all
select 39617,'TOK' from dual union all
select 39618,'MPAE' from dual union all
select 39619,'NASDA' from dual union all
select 39620,'MPAE' from dual union all
select 39623,'LPARL' from dual union all
select 39623,'MSSL' from dual union all
select 39626,'UMN' from dual union all
select 39627,'NDRE' from dual union all
select 39631,'NRCC' from dual union all
select 39632,'MPAE' from dual union all
select 39633,'MPAE' from dual union all
select 39635,'INTA' from dual union all
select 39635,'MPAE' from dual union all
select 39636,'MPAE' from dual union all
select 39639,'SAO' from dual union all
select 39710,'USAF' from dual union all
select 39711,'USAF' from dual union all
select 39712,'USAF' from dual union all
select 39713,'USAF' from dual union all
select 39714,'USAF' from dual union all
select 39715,'USAF' from dual union all
select 39717,'USAF' from dual union all
select 39782,'GMS' from dual union all
select 39829,'VSSC' from dual union all
select 39853,'PSU' from dual union all
select 39861,'PSU' from dual union all
select 39868,'PSU' from dual union all
select 39873,'PSU' from dual union all
select 39881,'GMS' from dual union all
select 39890,'USAF' from dual union all
select 39891,'USAF' from dual union all
select 39892,'USAF' from dual union all
select 39893,'USAF' from dual union all
select 39895,'USAF' from dual union all
select 39896,'USAF' from dual union all
select 39897,'USAF' from dual union all
select 39928,'GMS' from dual union all
select 39943,'GMS' from dual union all
select 39957,'GMS' from dual union all
select 39959,'PSU' from dual union all
select 39963,'PSU' from dual union all
select 39980,'GMS' from dual union all
select 40017,'GMS' from dual union all
select 40019,'GSFC' from dual union all
select 40026,'WFF' from dual union all
select 40034,'NASDA' from dual union all
select 40052,'NASDA' from dual union all
select 40070,'WFF' from dual union all
select 40075,'GMS' from dual union all
select 40086,'USAF' from dual union all
select 40087,'USAF' from dual union all
select 40088,'USAF' from dual union all
select 40089,'USAF' from dual union all
select 40090,'USAF' from dual union all
select 40091,'USAF' from dual union all
select 40092,'USAF' from dual union all
select 40095,'USAF' from dual union all
select 40109,'WFF' from dual union all
select 40123,'GMS' from dual union all
select 40157,'GMS' from dual union all
select 40186,'GMS' from dual union all
select 40194,'GMS' from dual union all
select 40211,'WFF' from dual union all
select 40238,'USAF' from dual union all
select 40249,'USAF' from dual union all
select 40260,'USAF' from dual union all
select 40264,'USAF' from dual union all
select 40265,'USAF' from dual union all
select 40266,'USAF' from dual union all
select 40267,'USAF' from dual union all
select 40268,'USAF' from dual union all
select 40269,'USAF' from dual union all
select 40270,'USAF' from dual union all
select 40271,'USAF' from dual union all
select 40272,'USAF' from dual union all
select 40273,'USAF' from dual union all
select 40274,'USAF' from dual union all
select 40275,'USAF' from dual union all
select 40276,'USAF' from dual union all
select 40277,'USAF' from dual union all
select 40278,'USAF' from dual union all
select 40282,'USAF' from dual union all
select 40293,'USAF' from dual union all
select 40304,'USAF' from dual union all
select 40316,'USAF' from dual union all
select 40327,'USAF' from dual union all
select 40329,'WFF' from dual union all
select 40338,'USAF' from dual union all
select 40345,'WFF' from dual union all
select 40347,'WFF' from dual union all
select 40362,'GMS' from dual union all
select 40378,'WFF' from dual union all
select 40379,'WFF' from dual union all
select 40388,'GMS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 4043,'SCEL' from dual union all
select 4045,'SCEL' from dual union all
select 4046,'SCEL' from dual union all
select 40471,'WFF' from dual union all
select 40484,'WFF' from dual union all
select 40486,'WFF' from dual union all
select 40515,'WFF' from dual union all
select 40598,'GMS' from dual union all
select 40608,'WFF' from dual union all
select 40644,'GMS' from dual union all
select 40645,'GMS' from dual union all
select 40647,'PSU' from dual union all
select 40650,'PSU' from dual union all
select 40651,'PSU' from dual union all
select 40652,'INTA' from dual union all
select 40653,'WFF' from dual union all
select 40656,'WFF' from dual union all
select 40671,'GMS' from dual union all
select 40685,'GMS' from dual union all
select 40696,'GMS' from dual union all
select 4070,'NRL' from dual union all
select 4070,'SBRC' from dual union all
select 40707,'GMS' from dual union all
select 4071,'NRL' from dual union all
select 4071,'SBRC' from dual union all
select 40711,'GMS' from dual union all
select 4072,'NRL' from dual union all
select 40776,'GMS' from dual union all
select 40789,'USAF' from dual union all
select 4079,'NRL' from dual union all
select 4079,'SBRC' from dual union all
select 40790,'USAF' from dual union all
select 40791,'USAF' from dual union all
select 40792,'USAF' from dual union all
select 40793,'USAF' from dual union all
select 40795,'USAF' from dual union all
select 40796,'USAF' from dual union all
select 40797,'USAF' from dual union all
select 4080,'USN' from dual union all
select 4081,'USN' from dual union all
select 40814,'GMS' from dual union all
select 40837,'GMS' from dual union all
select 40846,'GMS' from dual union all
select 40847,'GMS' from dual union all
select 4086,'NRL' from dual union all
select 40873,'GMS' from dual union all
select 40874,'GMS' from dual union all
select 4088,'NRL' from dual union all
select 4088,'SBRC' from dual union all
select 40888,'GMS' from dual union all
select 40889,'GMS' from dual union all
select 4089,'USN' from dual union all
select 4090,'USN' from dual union all
select 40908,'WFF' from dual union all
select 4095,'NRL' from dual union all
select 4095,'SBRC' from dual union all
select 4096,'NRL' from dual union all
select 4097,'NRL' from dual union all
select 40985,'HOUS' from dual union all
select 40985,'UTD' from dual union all
select 4099,'LMSC' from dual union all
select 4100,'AFCRL' from dual union all
select 4101,'LMSC' from dual union all
select 41061,'WFF' from dual union all
select 41090,'GMS' from dual union all
select 4111,'UMD' from dual union all
select 4112,'ISAS' from dual union all
select 41127,'GMS' from dual union all
select 41147,'USAF' from dual union all
select 41148,'USAF' from dual union all
select 41149,'USAF' from dual union all
select 41150,'USAF' from dual union all
select 41151,'USAF' from dual union all
select 41152,'USAF' from dual union all
select 41153,'USAF' from dual union all
select 41154,'USAF' from dual union all
select 4116,'ISAS' from dual union all
select 41166,'GMS' from dual union all
select 4117,'ISAS' from dual union all
select 41225,'NRCC' from dual union all
select 41227,'GMS' from dual union all
select 4125,'NRL' from dual union all
select 4126,'PARD' from dual union all
select 41269,'GMS' from dual union all
select 4127,'AFCRL' from dual union all
select 41276,'WFF' from dual union all
select 41282,'PSU' from dual union all
select 41287,'WFF' from dual union all
select 4129,'SCEL' from dual union all
select 4129,'UMI' from dual union all
select 4130,'SCEL' from dual union all
select 41308,'GMS' from dual union all
select 4131,'SCEL' from dual union all
select 4132,'SCEL' from dual union all
select 41323,'GMS' from dual union all
select 4133,'SCEL' from dual union all
select 41336,'USAF' from dual union all
select 41337,'USAF' from dual union all
select 41338,'USAF' from dual union all
select 41339,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 4134,'SCEL' from dual union all
select 4135,'SCEL' from dual union all
select 4136,'SCEL' from dual union all
select 4137,'SCEL' from dual union all
select 4138,'SCEL' from dual union all
select 41380,'GMS' from dual union all
select 4139,'SCEL' from dual union all
select 4140,'SCEL' from dual union all
select 41405,'GMS' from dual union all
select 4141,'SCEL' from dual union all
select 41418,'GMS' from dual union all
select 4142,'SCEL' from dual union all
select 4143,'SCEL' from dual union all
select 4144,'SCEL' from dual union all
select 41441,'WFF' from dual union all
select 4145,'SCEL' from dual union all
select 41459,'PSU' from dual union all
select 4146,'SCEL' from dual union all
select 41465,'WFF' from dual union all
select 4147,'SCEL' from dual union all
select 4148,'SCEL' from dual union all
select 41522,'WFF' from dual union all
select 4155,'LMSC' from dual union all
select 41552,'GSFC' from dual union all
select 41560,'GSFC' from dual union all
select 4159,'NRL' from dual union all
select 4160,'LMSC' from dual union all
select 4161,'NRL' from dual union all
select 4162,'NRL' from dual union all
select 41628,'PRL' from dual union all
select 4163,'LMSC' from dual union all
select 4165,'LMSC' from dual union all
select 4166,'NRL' from dual union all
select 4167,'LMSC' from dual union all
select 4168,'AFCRL' from dual union all
select 4171,'FIAN' from dual union all
select 41738,'PRL' from dual union all
select 41804,'PRL' from dual union all
select 41832,'VSSC' from dual union all
select 41835,'VSSC' from dual union all
select 41840,'VSSC' from dual union all
select 41842,'VSSC' from dual union all
select 41846,'VSSC' from dual union all
select 41850,'VSSC' from dual union all
select 41854,'VSSC' from dual union all
select 41857,'VSSC' from dual union all
select 41870,'UTD' from dual union all
select 41917,'GSFC' from dual union all
select 41924,'GSFC' from dual union all
select 41969,'PRL' from dual union all
select 4219,'LMSC' from dual union all
select 4220,'AFCRL' from dual union all
select 4221,'AFCRL' from dual union all
select 4225,'AFCRL' from dual union all
select 42257,'UWI' from dual union all
select 42261,'MPE' from dual union all
select 4227,'LMSC' from dual union all
select 42273,'WFF' from dual union all
select 4228,'CNRS' from dual union all
select 4228,'PARIS' from dual union all
select 42281,'OTR' from dual union all
select 42282,'NRL' from dual union all
select 4229,'CNRS' from dual union all
select 4229,'PARIS' from dual union all
select 4230,'AFCRL' from dual union all
select 4230,'UCO' from dual union all
select 42300,'UAF' from dual union all
select 42302,'YORK' from dual union all
select 42305,'IITM' from dual union all
select 42306,'WRE' from dual union all
select 42307,'WRE' from dual union all
select 42308,'WRE' from dual union all
select 42309,'WRE' from dual union all
select 4231,'NRL' from dual union all
select 42317,'GSFC' from dual union all
select 42318,'WFF' from dual union all
select 42319,'WFF' from dual union all
select 4232,'FIAN' from dual union all
select 4232,'IAM' from dual union all
select 42320,'WFF' from dual union all
select 42321,'WFF' from dual union all
select 42322,'WFF' from dual union all
select 42323,'WFF' from dual union all
select 42324,'WFF' from dual union all
select 42325,'WFF' from dual union all
select 42326,'WFF' from dual union all
select 42327,'WFF' from dual union all
select 42328,'WFF' from dual union all
select 42329,'WFF' from dual union all
select 42330,'WFF' from dual union all
select 42331,'GMS' from dual union all
select 42332,'WFF' from dual union all
select 42333,'WFF' from dual union all
select 42334,'WFF' from dual union all
select 42335,'WFF' from dual union all
select 42336,'WFF' from dual union all
select 4235,'AFCRL' from dual union all
select 42353,'GMS' from dual union all
select 42354,'DNA' from dual union all
select 42356,'NRCC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 42357,'PITT' from dual union all
select 42358,'UCO' from dual union all
select 42359,'UCO' from dual union all
select 42360,'DNA' from dual union all
select 42362,'DNA' from dual union all
select 42364,'DFVLR' from dual union all
select 42367,'GMS' from dual union all
select 42368,'FREIB' from dual union all
select 42368,'TUB' from dual union all
select 4237,'FIAN' from dual union all
select 4237,'IAM' from dual union all
select 42375,'UMI' from dual union all
select 42378,'MPE' from dual union all
select 42379,'UBC' from dual union all
select 4238,'PARD' from dual union all
select 42380,'UWI' from dual union all
select 42386,'NIPR' from dual union all
select 4239,'FIAN' from dual union all
select 4239,'IAM' from dual union all
select 42390,'GMS' from dual union all
select 42391,'GMS' from dual union all
select 42395,'WSMR' from dual union all
select 42397,'WSMR' from dual union all
select 42398,'INPE' from dual union all
select 4240,'AFCRL' from dual union all
select 4240,'UUT' from dual union all
select 42401,'GSFC' from dual union all
select 42402,'NIPR' from dual union all
select 42405,'JHU' from dual union all
select 42406,'AFCRL' from dual union all
select 42408,'MSSL' from dual union all
select 42412,'WSMR' from dual union all
select 4242,'AFCRL' from dual union all
select 4242,'UUT' from dual union all
select 42424,'AFCRL' from dual union all
select 42426,'UWI' from dual union all
select 4243,'AFCRL' from dual union all
select 4243,'UUT' from dual union all
select 4244,'LMSC' from dual union all
select 42446,'DNA' from dual union all
select 42447,'RAE' from dual union all
select 42448,'SAO' from dual union all
select 42449,'DORN' from dual union all
select 4245,'AFCRL' from dual union all
select 4245,'UUT' from dual union all
select 42457,'MSFC' from dual union all
select 42459,'RSRS' from dual union all
select 4246,'NRL' from dual union all
select 42460,'UCW' from dual union all
select 42467,'UCO' from dual union all
select 42468,'NIPR' from dual union all
select 42470,'WFF' from dual union all
select 42471,'GSFC' from dual union all
select 42472,'NIPR' from dual union all
select 42473,'SAO' from dual union all
select 42474,'ISRO' from dual union all
select 42475,'LUX' from dual union all
select 42475,'MIT' from dual union all
select 42477,'MITLL' from dual union all
select 42481,'AFCRL' from dual union all
select 42482,'91MW' from dual union all
select 42483,'UIUC' from dual union all
select 42484,'NIPR' from dual union all
select 42486,'ISAS' from dual union all
select 42487,'GSFC' from dual union all
select 42489,'UTD' from dual union all
select 4249,'NRL' from dual union all
select 42491,'NASDA' from dual union all
select 42492,'AN' from dual union all
select 42493,'WSMR' from dual union all
select 42494,'LASL' from dual union all
select 42497,'ISAS' from dual union all
select 4250,'UMI' from dual union all
select 42500,'SAND' from dual union all
select 42501,'LASL' from dual union all
select 42506,'ISAS' from dual union all
select 42508,'ISAS' from dual union all
select 42509,'ISAS' from dual union all
select 4251,'LMSC' from dual union all
select 42510,'AFCRL' from dual union all
select 42511,'AFCRL' from dual union all
select 42512,'AFCRL' from dual union all
select 42513,'USCLA' from dual union all
select 42514,'FRASC' from dual union all
select 42514,'SAO' from dual union all
select 42516,'FREIB' from dual union all
select 42516,'TUB' from dual union all
select 42517,'MSSL' from dual union all
select 42517,'RSRS' from dual union all
select 42519,'RSRS' from dual union all
select 42519,'UCL' from dual union all
select 4252,'NRL' from dual union all
select 42520,'UCL' from dual union all
select 42521,'INPE' from dual union all
select 42523,'SSC' from dual union all
select 42525,'CLSR' from dual union all
select 42525,'GICZ' from dual union all
select 42525,'IKI' from dual union all
select 4253,'NRL' from dual union all
select 42536,'FREIB' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 42536,'TUB' from dual union all
select 42539,'AFCRL' from dual union all
select 42542,'AFCRL' from dual union all
select 42543,'SRI' from dual union all
select 42545,'MSSL' from dual union all
select 42545,'RSRS' from dual union all
select 42546,'RSRS' from dual union all
select 4255,'NRL' from dual union all
select 42553,'WFF' from dual union all
select 42554,'SHEF' from dual union all
select 42557,'YORK' from dual union all
select 42558,'GSFC' from dual union all
select 42559,'GSFC' from dual union all
select 42561,'GSFC' from dual union all
select 42562,'CIFTQ' from dual union all
select 42562,'INTA' from dual union all
select 42563,'AFCRL' from dual union all
select 42564,'ERNO' from dual union all
select 42564,'SSC' from dual union all
select 42565,'PITT' from dual union all
select 42566,'CIFTQ' from dual union all
select 42566,'INTA' from dual union all
select 4257,'NRL' from dual union all
select 42570,'PRL' from dual union all
select 42572,'AFGL' from dual union all
select 42573,'AFGL' from dual union all
select 42574,'AFGL' from dual union all
select 42575,'AFGL' from dual union all
select 42576,'AFGL' from dual union all
select 42577,'NRL' from dual union all
select 4258,'NRL' from dual union all
select 42587,'USU' from dual union all
select 42588,'USU' from dual union all
select 42589,'USU' from dual union all
select 4259,'NMSU' from dual union all
select 42590,'USU' from dual union all
select 42591,'USU' from dual union all
select 42592,'USU' from dual union all
select 42597,'GMS' from dual union all
select 42598,'GMS' from dual union all
select 42599,'GMS' from dual union all
select 42600,'GMS' from dual union all
select 42601,'GMS' from dual union all
select 4261,'NRL' from dual union all
select 42614,'UTOR' from dual union all
select 42625,'UIUC' from dual union all
select 42627,'INTA' from dual union all
select 42628,'INTA' from dual union all
select 42629,'INTA' from dual union all
select 4263,'NRL' from dual union all
select 42630,'INTA' from dual union all
select 42631,'INTA' from dual union all
select 42632,'INTA' from dual union all
select 42633,'INTA' from dual union all
select 42634,'INTA' from dual union all
select 42635,'INTA' from dual union all
select 42636,'PSU' from dual union all
select 42637,'INTA' from dual union all
select 42638,'INTA' from dual union all
select 42639,'INTA' from dual union all
select 4264,'NRL' from dual union all
select 42640,'INTA' from dual union all
select 42641,'INTA' from dual union all
select 42642,'INTA' from dual union all
select 42643,'INTA' from dual union all
select 42644,'INTA' from dual union all
select 42645,'INTA' from dual union all
select 42646,'INTA' from dual union all
select 42647,'SASK' from dual union all
select 42648,'INTA' from dual union all
select 42649,'INTA' from dual union all
select 42650,'INTA' from dual union all
select 42651,'INTA' from dual union all
select 42652,'INTA' from dual union all
select 42653,'INTA' from dual union all
select 42654,'INTA' from dual union all
select 42655,'INTA' from dual union all
select 42656,'INTA' from dual union all
select 42657,'INTA' from dual union all
select 42658,'NOAA' from dual union all
select 42659,'INTA' from dual union all
select 42660,'INTA' from dual union all
select 42661,'INTA' from dual union all
select 42662,'INTA' from dual union all
select 42663,'INTA' from dual union all
select 42664,'INTA' from dual union all
select 42665,'INTA' from dual union all
select 42666,'INTA' from dual union all
select 42667,'INTA' from dual union all
select 42668,'INTA' from dual union all
select 42669,'MPE' from dual union all
select 42669,'NDRE' from dual union all
select 42669,'UIO' from dual union all
select 42670,'INTA' from dual union all
select 42671,'INTA' from dual union all
select 42672,'INTA' from dual union all
select 42673,'INTA' from dual union all
select 42674,'INTA' from dual union all
select 42675,'INTA' from dual union all
select 42676,'INTA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 42677,'INTA' from dual union all
select 42678,'INTA' from dual union all
select 42679,'INTA' from dual union all
select 4268,'AFCRL' from dual union all
select 42680,'UCO' from dual union all
select 42681,'INTA' from dual union all
select 42682,'INTA' from dual union all
select 42683,'INTA' from dual union all
select 42684,'INTA' from dual union all
select 42685,'INTA' from dual union all
select 42686,'INTA' from dual union all
select 42687,'INTA' from dual union all
select 42690,'IIAE' from dual union all
select 42691,'UWI' from dual union all
select 42692,'GMS' from dual union all
select 42693,'GMS' from dual union all
select 42694,'GMS' from dual union all
select 42695,'WFF' from dual union all
select 42696,'GMS' from dual union all
select 42697,'GMS' from dual union all
select 42698,'GMS' from dual union all
select 42699,'GMS' from dual union all
select 4270,'UMI' from dual union all
select 42700,'GMS' from dual union all
select 42701,'GMS' from dual union all
select 42702,'NAG' from dual union all
select 42703,'GMS' from dual union all
select 42704,'GMS' from dual union all
select 42705,'GMS' from dual union all
select 42706,'GMS' from dual union all
select 42707,'GMS' from dual union all
select 42708,'WFF' from dual union all
select 42709,'WFF' from dual union all
select 4271,'UMI' from dual union all
select 42710,'WFF' from dual union all
select 42711,'WFF' from dual union all
select 42712,'WFF' from dual union all
select 4272,'BRL' from dual union all
select 42726,'FREIB' from dual union all
select 42726,'TUB' from dual union all
select 42727,'UMN' from dual union all
select 42728,'NASDA' from dual union all
select 42729,'MPE' from dual union all
select 42729,'NRL' from dual union all
select 4273,'UMI' from dual union all
select 42739,'UMN' from dual union all
select 42740,'UAF' from dual union all
select 42741,'UCW' from dual union all
select 42741,'UKMO' from dual union all
select 42742,'RSRS' from dual union all
select 42743,'UCW' from dual union all
select 42744,'LIEGE' from dual union all
select 42745,'FREIB' from dual union all
select 42745,'TUB' from dual union all
select 42747,'NIPR' from dual union all
select 42749,'UAF' from dual union all
select 4275,'LMSC' from dual union all
select 42750,'UCW' from dual union all
select 42751,'UCW' from dual union all
select 42752,'RSRS' from dual union all
select 42753,'UIO' from dual union all
select 42754,'GMS' from dual union all
select 42755,'PITT' from dual union all
select 42756,'CAL' from dual union all
select 42757,'NRL' from dual union all
select 42758,'UCL' from dual union all
select 42758,'UIO' from dual union all
select 4276,'LMSC' from dual union all
select 42761,'GSFC' from dual union all
select 42762,'GSFC' from dual union all
select 42762,'NDRE' from dual union all
select 42763,'JHU' from dual union all
select 42764,'GSFC' from dual union all
select 42766,'ASE' from dual union all
select 42766,'LUX' from dual union all
select 42767,'FREIB' from dual union all
select 42767,'TUB' from dual union all
select 42769,'JHU' from dual union all
select 4277,'LMSC' from dual union all
select 42770,'ADEL' from dual union all
select 42771,'NRL' from dual union all
select 42773,'DNA' from dual union all
select 42774,'PITT' from dual union all
select 42775,'DNA' from dual union all
select 42779,'NRCC' from dual union all
select 42780,'USAF' from dual union all
select 42781,'USAF' from dual union all
select 42782,'USAF' from dual union all
select 42783,'USAF' from dual union all
select 42784,'USAF' from dual union all
select 42785,'USAF' from dual union all
select 42786,'USAF' from dual union all
select 42787,'USAF' from dual union all
select 42788,'USAF' from dual union all
select 42789,'USAF' from dual union all
select 4279,'UMD' from dual union all
select 42791,'GMS' from dual union all
select 4280,'LMSC' from dual union all
select 42803,'WFF' from dual union all
select 4281,'LMSC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 42817,'GMS' from dual union all
select 4282,'LMSC' from dual union all
select 4283,'LMSC' from dual union all
select 42838,'GMS' from dual union all
select 42843,'GMS' from dual union all
select 4285,'UMI' from dual union all
select 42850,'GMS' from dual union all
select 42851,'WFF' from dual union all
select 42853,'WFF' from dual union all
select 4286,'UMI' from dual union all
select 4287,'LMSC' from dual union all
select 42872,'GMS' from dual union all
select 4288,'UMI' from dual union all
select 42888,'GMS' from dual union all
select 42890,'GMS' from dual union all
select 42900,'USAF' from dual union all
select 42903,'USAF' from dual union all
select 42904,'USAF' from dual union all
select 4291,'AFCRL' from dual union all
select 4292,'UMI' from dual union all
select 4293,'UMI' from dual union all
select 4294,'LMSC' from dual union all
select 4295,'UMI' from dual union all
select 4296,'UMI' from dual union all
select 42962,'GMS' from dual union all
select 4298,'USN' from dual union all
select 4299,'USN' from dual union all
select 42999,'WFF' from dual union all
select 4300,'USN' from dual union all
select 43007,'WFF' from dual union all
select 43009,'GMS' from dual union all
select 4301,'USN' from dual union all
select 43011,'GMS' from dual union all
select 43014,'WFF' from dual union all
select 43017,'GMS' from dual union all
select 43019,'WFF' from dual union all
select 4302,'USN' from dual union all
select 43022,'WFF' from dual union all
select 43025,'WFF' from dual union all
select 43026,'WFF' from dual union all
select 4303,'SCEL' from dual union all
select 43030,'WFF' from dual union all
select 43032,'WFF' from dual union all
select 43035,'WFF' from dual union all
select 43038,'WFF' from dual union all
select 4304,'SCEL' from dual union all
select 43045,'WFF' from dual union all
select 43047,'GSFC' from dual union all
select 43048,'GSFC' from dual union all
select 43069,'GMS' from dual union all
select 43075,'GMS' from dual union all
select 43077,'GMS' from dual union all
select 43092,'GMS' from dual union all
select 43094,'GMS' from dual union all
select 43104,'USAF' from dual union all
select 43105,'USAF' from dual union all
select 43106,'USAF' from dual union all
select 43107,'USAF' from dual union all
select 4313,'LMSC' from dual union all
select 43133,'GMS' from dual union all
select 43148,'GMS' from dual union all
select 4317,'NRL' from dual union all
select 43174,'GMS' from dual union all
select 43183,'GMS' from dual union all
select 4319,'ISAS' from dual union all
select 43193,'WFF' from dual union all
select 43197,'WFF' from dual union all
select 4320,'ISAS' from dual union all
select 43201,'WFF' from dual union all
select 43204,'GMS' from dual union all
select 43211,'GMS' from dual union all
select 43222,'NASDA' from dual union all
select 43236,'NASDA' from dual union all
select 43242,'GMS' from dual union all
select 43256,'USAF' from dual union all
select 4326,'NRL' from dual union all
select 43263,'GMS' from dual union all
select 4327,'RAE' from dual union all
select 4327,'WRE' from dual union all
select 43279,'GMS' from dual union all
select 4329,'LRL' from dual union all
select 4330,'LRL' from dual union all
select 43307,'WFF' from dual union all
select 43337,'WFF' from dual union all
select 43383,'USAF' from dual union all
select 43384,'USAF' from dual union all
select 4340,'SUI' from dual union all
select 43420,'GMS' from dual union all
select 43422,'GMS' from dual union all
select 43433,'WFF' from dual union all
select 43438,'GMS' from dual union all
select 43446,'GMS' from dual union all
select 43447,'GMS' from dual union all
select 43468,'GMS' from dual union all
select 43470,'GMS' from dual union all
select 43479,'WFF' from dual union all
select 43482,'GMS' from dual union all
select 43500,'GMS' from dual union all
select 43509,'USAF' from dual union all
select 43511,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 43512,'USAF' from dual union all
select 43513,'USAF' from dual union all
select 43514,'USAF' from dual union all
select 4357,'SCEL' from dual union all
select 43603,'WFF' from dual union all
select 43604,'WFF' from dual union all
select 4362,'AFCRL' from dual union all
select 4362,'UMI' from dual union all
select 4365,'NMSU' from dual union all
select 43662,'USAF' from dual union all
select 43663,'USAF' from dual union all
select 43664,'USAF' from dual union all
select 43666,'USAF' from dual union all
select 43667,'USAF' from dual union all
select 43668,'USAF' from dual union all
select 43669,'USAF' from dual union all
select 43684,'GMS' from dual union all
select 43705,'GMS' from dual union all
select 43738,'GMS' from dual union all
select 43746,'WFF' from dual union all
select 43753,'WFF' from dual union all
select 43761,'WFF' from dual union all
select 43771,'GMS' from dual union all
select 43793,'GMS' from dual union all
select 43800,'WFF' from dual union all
select 43801,'GMS' from dual union all
select 43802,'GMS' from dual union all
select 43803,'GMS' from dual union all
select 43804,'GMS' from dual union all
select 43805,'GMS' from dual union all
select 43806,'GMS' from dual union all
select 43807,'GMS' from dual union all
select 43808,'GMS' from dual union all
select 43809,'GMS' from dual union all
select 43811,'GMS' from dual union all
select 43812,'GMS' from dual union all
select 43813,'GMS' from dual union all
select 43814,'GMS' from dual union all
select 43850,'USAF' from dual union all
select 43851,'USAF' from dual union all
select 43852,'USAF' from dual union all
select 43853,'USAF' from dual union all
select 43854,'USAF' from dual union all
select 43856,'USAF' from dual union all
select 43857,'USAF' from dual union all
select 43858,'USAF' from dual union all
select 43859,'USAF' from dual union all
select 43860,'USAF' from dual union all
select 43888,'GMS' from dual union all
select 43917,'GMS' from dual union all
select 43978,'GMS' from dual union all
select 44001,'WFF' from dual union all
select 44002,'WFF' from dual union all
select 44003,'WFF' from dual union all
select 44035,'WFF' from dual union all
select 44043,'NASDA' from dual union all
select 44058,'GMS' from dual union all
select 44059,'USAF' from dual union all
select 44060,'USAF' from dual union all
select 44061,'USAF' from dual union all
select 44062,'USAF' from dual union all
select 44063,'USAF' from dual union all
select 44065,'USAF' from dual union all
select 44066,'USAF' from dual union all
select 44067,'USAF' from dual union all
select 44068,'USAF' from dual union all
select 44069,'USAF' from dual union all
select 4408,'LRL' from dual union all
select 44080,'GMS' from dual union all
select 4409,'LRL' from dual union all
select 4410,'ISAS' from dual union all
select 44101,'GMS' from dual union all
select 4411,'ISAS' from dual union all
select 44111,'CALG' from dual union all
select 44122,'CALG' from dual union all
select 44125,'GMS' from dual union all
select 44140,'GMS' from dual union all
select 44175,'WFF' from dual union all
select 44177,'WFF' from dual union all
select 44180,'GMS' from dual union all
select 44181,'GMS' from dual union all
select 44187,'GMS' from dual union all
select 44208,'GMS' from dual union all
select 44209,'GMS' from dual union all
select 4421,'LRL' from dual union all
select 4422,'LRL' from dual union all
select 44224,'GMS' from dual union all
select 44227,'GMS' from dual union all
select 4423,'LRL' from dual union all
select 4424,'LRL' from dual union all
select 44253,'GMS' from dual union all
select 44265,'GMS' from dual union all
select 44266,'GMS' from dual union all
select 44270,'WFF' from dual union all
select 44282,'GMS' from dual union all
select 44283,'GMS' from dual union all
select 4430,'ISAS' from dual union all
select 4431,'ISAS' from dual union all
select 44325,'GMS' from dual union all
select 4434,'LRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 44342,'GMS' from dual union all
select 4435,'LRL' from dual union all
select 4436,'LRL' from dual union all
select 44363,'GMS' from dual union all
select 44365,'GMS' from dual union all
select 4437,'NRL' from dual union all
select 44375,'WFF' from dual union all
select 44381,'GMS' from dual union all
select 4439,'NRL' from dual union all
select 44395,'GMS' from dual union all
select 44403,'GMS' from dual union all
select 4442,'SAND' from dual union all
select 44422,'GMS' from dual union all
select 4443,'UMI' from dual union all
select 44434,'USAF' from dual union all
select 44435,'GMS' from dual union all
select 44436,'GMS' from dual union all
select 4444,'LRL' from dual union all
select 4445,'LRL' from dual union all
select 44450,'GMS' from dual union all
select 44453,'GMS' from dual union all
select 44469,'GMS' from dual union all
select 4448,'NRL' from dual union all
select 44496,'GMS' from dual union all
select 4450,'SUI' from dual union all
select 4451,'SUI' from dual union all
select 4452,'SUI' from dual union all
select 44533,'GMS' from dual union all
select 4454,'SUI' from dual union all
select 44545,'WFF' from dual union all
select 44549,'WFF' from dual union all
select 44574,'GMS' from dual union all
select 4459,'SCEL' from dual union all
select 4459,'UMI' from dual union all
select 4460,'SCEL' from dual union all
select 4462,'NRL' from dual union all
select 44623,'GMS' from dual union all
select 44639,'GMS' from dual union all
select 44642,'GMS' from dual union all
select 44651,'GMS' from dual union all
select 4466,'LRL' from dual union all
select 44660,'GMS' from dual union all
select 44661,'GMS' from dual union all
select 4467,'LRL' from dual union all
select 44701,'GMS' from dual union all
select 4472,'NRL' from dual union all
select 4473,'NRL' from dual union all
select 44733,'PSU' from dual union all
select 4478,'SUI' from dual union all
select 4479,'SUI' from dual union all
select 4480,'SUI' from dual union all
select 4481,'SAND' from dual union all
select 4483,'AFCRL' from dual union all
select 4484,'RAE' from dual union all
select 4484,'WRE' from dual union all
select 4485,'SUI' from dual union all
select 4486,'SCEL' from dual union all
select 4486,'UMI' from dual union all
select 4487,'SCEL' from dual union all
select 4487,'UMI' from dual union all
select 4488,'NRL' from dual union all
select 4489,'NRL' from dual union all
select 44900,'GSFC' from dual union all
select 4492,'UMD' from dual union all
select 4493,'UMD' from dual union all
select 4494,'UMD' from dual union all
select 4495,'UMD' from dual union all
select 4496,'SAND' from dual union all
select 4497,'LMSC' from dual union all
select 44979,'WFF' from dual union all
select 4498,'LMSC' from dual union all
select 44981,'PRL' from dual union all
select 4501,'CNRS' from dual union all
select 4501,'PARIS' from dual union all
select 4502,'NRL' from dual union all
select 4504,'SUI' from dual union all
select 4505,'IAM' from dual union all
select 4506,'AFCRL' from dual union all
select 4506,'UMI' from dual union all
select 4510,'FIAN' from dual union all
select 4512,'CNRS' from dual union all
select 4512,'PARIS' from dual union all
select 4513,'AFCRL' from dual union all
select 4515,'AFCRL' from dual union all
select 45157,'HOUS' from dual union all
select 45159,'HOUS' from dual union all
select 4516,'SUI' from dual union all
select 4517,'SUI' from dual union all
select 45176,'UWO' from dual union all
select 4518,'WRE' from dual union all
select 4521,'SUI' from dual union all
select 4523,'AFCRL' from dual union all
select 4525,'SUI' from dual union all
select 4526,'AFCRL' from dual union all
select 4527,'SUI' from dual union all
select 4528,'SUI' from dual union all
select 4529,'SUI' from dual union all
select 4530,'SUI' from dual union all
select 4531,'SUI' from dual union all
select 4532,'SUI' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 45320,'HOUS' from dual union all
select 45324,'UWI' from dual union all
select 4533,'LMSC' from dual union all
select 45333,'AFCRL' from dual union all
select 4534,'SUI' from dual union all
select 45340,'OTR' from dual union all
select 45341,'RAE' from dual union all
select 45346,'OTR' from dual union all
select 45348,'INTA' from dual union all
select 4535,'SUI' from dual union all
select 4536,'SUI' from dual union all
select 45364,'AFCRL' from dual union all
select 45365,'AFCRL' from dual union all
select 4538,'SUI' from dual union all
select 4539,'SUI' from dual union all
select 45391,'DFVLR' from dual union all
select 45392,'AFCRL' from dual union all
select 45393,'AFCRL' from dual union all
select 45394,'SDCO' from dual union all
select 4540,'SUI' from dual union all
select 45407,'AFCRL' from dual union all
select 45409,'NOAA' from dual union all
select 45410,'UCB' from dual union all
select 45416,'SASK' from dual union all
select 45419,'JHU' from dual union all
select 45419,'UIO' from dual union all
select 4542,'SUI' from dual union all
select 45420,'DFVLR' from dual union all
select 45420,'ISRO' from dual union all
select 4543,'AFCRL' from dual union all
select 45433,'INPE' from dual union all
select 45439,'UWI' from dual union all
select 4544,'LMSC' from dual union all
select 45440,'CULH' from dual union all
select 45440,'MSSL' from dual union all
select 45445,'NRL' from dual union all
select 45447,'AFCRL' from dual union all
select 45463,'CORN' from dual union all
select 45465,'MPE' from dual union all
select 45467,'UTD' from dual union all
select 4547,'NRL' from dual union all
select 45471,'UIUC' from dual union all
select 45476,'AERO' from dual union all
select 45496,'SAO' from dual union all
select 45498,'AFCRL' from dual union all
select 4550,'AFCRL' from dual union all
select 45505,'MISU' from dual union all
select 45506,'RSRS' from dual union all
select 45506,'SSC' from dual union all
select 45507,'SSC' from dual union all
select 4551,'SUI' from dual union all
select 45510,'RSRS' from dual union all
select 45510,'SSC' from dual union all
select 45511,'NRL' from dual union all
select 45512,'NAG' from dual union all
select 45515,'NASDA' from dual union all
select 4552,'SUI' from dual union all
select 45521,'MSFC' from dual union all
select 45523,'DFVLR' from dual union all
select 45524,'RAE' from dual union all
select 45526,'AFCRL' from dual union all
select 45527,'AFCRL' from dual union all
select 45528,'UCO' from dual union all
select 4553,'SUI' from dual union all
select 45530,'GSFC' from dual union all
select 45531,'UIUC' from dual union all
select 45532,'INPE' from dual union all
select 45535,'AFCRL' from dual union all
select 45537,'AFCRL' from dual union all
select 45538,'AFCRL' from dual union all
select 4554,'SUI' from dual union all
select 45540,'AFCRL' from dual union all
select 45540,'DNA' from dual union all
select 45541,'WFF' from dual union all
select 45542,'CNES' from dual union all
select 45543,'UMI' from dual union all
select 45545,'RSRS' from dual union all
select 45545,'SHEF' from dual union all
select 45546,'AFCRL' from dual union all
select 45548,'RAE' from dual union all
select 4555,'LMSC' from dual union all
select 45550,'ERNO' from dual union all
select 45550,'SSC' from dual union all
select 45551,'GSFC' from dual union all
select 45554,'UBC' from dual union all
select 45556,'NDRE' from dual union all
select 45557,'NDRE' from dual union all
select 4556,'SUI' from dual union all
select 45564,'AN' from dual union all
select 45565,'JHU' from dual union all
select 4557,'SUI' from dual union all
select 45570,'RAE' from dual union all
select 45572,'UMN' from dual union all
select 45573,'CIT' from dual union all
select 45574,'RAE' from dual union all
select 45575,'UTOR' from dual union all
select 45577,'PRL' from dual union all
select 45578,'PRL' from dual union all
select 45579,'PRL' from dual union all
select 4558,'SUI' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 45581,'PRL' from dual union all
select 45582,'MPIK' from dual union all
select 45583,'MPIK' from dual union all
select 45584,'AFGL' from dual union all
select 45585,'AFGL' from dual union all
select 45586,'AFGL' from dual union all
select 45587,'CLSR' from dual union all
select 45587,'NPL' from dual union all
select 4559,'SUI' from dual union all
select 45592,'LASL' from dual union all
select 45593,'LASL' from dual union all
select 45594,'SSC' from dual union all
select 4560,'SUI' from dual union all
select 45602,'ISAS' from dual union all
select 4561,'SUI' from dual union all
select 4562,'SUI' from dual union all
select 45623,'INTA' from dual union all
select 45624,'INTA' from dual union all
select 45625,'INTA' from dual union all
select 45626,'INTA' from dual union all
select 45627,'INTA' from dual union all
select 45628,'INTA' from dual union all
select 45629,'INTA' from dual union all
select 4563,'SUI' from dual union all
select 45630,'INTA' from dual union all
select 45632,'INTA' from dual union all
select 45633,'INTA' from dual union all
select 45634,'INTA' from dual union all
select 45635,'INTA' from dual union all
select 45636,'INTA' from dual union all
select 45637,'INTA' from dual union all
select 45638,'INTA' from dual union all
select 45639,'INTA' from dual union all
select 4564,'SUI' from dual union all
select 45640,'INTA' from dual union all
select 45641,'INTA' from dual union all
select 45643,'INTA' from dual union all
select 45646,'ISAS' from dual union all
select 45646,'KYO' from dual union all
select 45647,'ISAS' from dual union all
select 45647,'NIPR' from dual union all
select 45648,'ISAS' from dual union all
select 45649,'ISAS' from dual union all
select 45649,'NAG' from dual union all
select 4565,'SUI' from dual union all
select 4566,'LMSC' from dual union all
select 45660,'AN' from dual union all
select 4567,'SUI' from dual union all
select 45675,'UIUC' from dual union all
select 45677,'GSFC' from dual union all
select 45678,'SAO' from dual union all
select 4568,'SUI' from dual union all
select 45680,'NASDA' from dual union all
select 45682,'AFCRL' from dual union all
select 45683,'ISAS' from dual union all
select 45684,'ISAS' from dual union all
select 45685,'NIPR' from dual union all
select 45686,'UMD' from dual union all
select 45687,'FREIB' from dual union all
select 45687,'TUB' from dual union all
select 45688,'FREIB' from dual union all
select 45688,'TUB' from dual union all
select 45689,'ASE' from dual union all
select 4569,'AFCRL' from dual union all
select 45690,'UWI' from dual union all
select 45691,'RSRS' from dual union all
select 45692,'MPE' from dual union all
select 45692,'RSRS' from dual union all
select 45693,'RSRS' from dual union all
select 45693,'UCW' from dual union all
select 45694,'RSRS' from dual union all
select 45695,'RSRS' from dual union all
select 45695,'UCW' from dual union all
select 45696,'NIPR' from dual union all
select 45697,'MPE' from dual union all
select 45697,'RSRS' from dual union all
select 45698,'RSRS' from dual union all
select 45700,'JHU' from dual union all
select 45702,'UKMO' from dual union all
select 45703,'UKMO' from dual union all
select 45704,'NRL' from dual union all
select 45708,'ISAS' from dual union all
select 4571,'AFCRL' from dual union all
select 45710,'NRCC' from dual union all
select 45711,'UAF' from dual union all
select 45713,'CORN' from dual union all
select 45714,'AFCRL' from dual union all
select 45715,'SDCO' from dual union all
select 45716,'NRL' from dual union all
select 45717,'NDRE' from dual union all
select 45718,'MPE' from dual union all
select 45718,'NDRE' from dual union all
select 45719,'SRI' from dual union all
select 4572,'SUI' from dual union all
select 45721,'CORN' from dual union all
select 45722,'LUX' from dual union all
select 45722,'MIT' from dual union all
select 45723,'RICE' from dual union all
select 45724,'UAF' from dual union all
select 45725,'PITT' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 45726,'UMI' from dual union all
select 45727,'GSFC' from dual union all
select 45729,'GSFC' from dual union all
select 4573,'SUI' from dual union all
select 45731,'GSFC' from dual union all
select 4574,'FIAN' from dual union all
select 45747,'GMS' from dual union all
select 4575,'AFCRL' from dual union all
select 45757,'GMS' from dual union all
select 4576,'BIR' from dual union all
select 4576,'ICST' from dual union all
select 4576,'UCL' from dual union all
select 45764,'USAF' from dual union all
select 45766,'USAF' from dual union all
select 45767,'USAF' from dual union all
select 4579,'AFCRL' from dual union all
select 4580,'AFCRL' from dual union all
select 4582,'BRL' from dual union all
select 45827,'WFF' from dual union all
select 45828,'WFF' from dual union all
select 45829,'WFF' from dual union all
select 4583,'NRL' from dual union all
select 4586,'SCEL' from dual union all
select 4586,'UMI' from dual union all
select 45868,'USAF' from dual union all
select 45869,'USAF' from dual union all
select 4587,'SCEL' from dual union all
select 4587,'UMI' from dual union all
select 45870,'USAF' from dual union all
select 45871,'USAF' from dual union all
select 45872,'USAF' from dual union all
select 45873,'USAF' from dual union all
select 45875,'GMS' from dual union all
select 4588,'LMSC' from dual union all
select 45885,'GMS' from dual union all
select 45899,'GMS' from dual union all
select 45912,'GMS' from dual union all
select 45918,'WFF' from dual union all
select 45924,'GMS' from dual union all
select 45936,'GMS' from dual union all
select 45942,'GMS' from dual union all
select 45944,'WFF' from dual union all
select 45946,'WFF' from dual union all
select 45959,'WFF' from dual union all
select 45974,'WFF' from dual union all
select 45984,'WFF' from dual union all
select 45986,'WFF' from dual union all
select 46045,'GSFC' from dual union all
select 46053,'GMS' from dual union all
select 46058,'WFF' from dual union all
select 46062,'WFF' from dual union all
select 46092,'GMS' from dual union all
select 46106,'GMS' from dual union all
select 46124,'GMS' from dual union all
select 46125,'GMS' from dual union all
select 46132,'WFF' from dual union all
select 46134,'WFF' from dual union all
select 46143,'NASDA' from dual union all
select 46144,'GMS' from dual union all
select 46167,'GMS' from dual union all
select 46168,'GMS' from dual union all
select 46187,'GMS' from dual union all
select 46197,'CALG' from dual union all
select 46203,'USAF' from dual union all
select 46204,'USAF' from dual union all
select 46205,'USAF' from dual union all
select 46207,'USAF' from dual union all
select 46208,'USAF' from dual union all
select 46209,'USAF' from dual union all
select 4621,'NRL' from dual union all
select 46210,'USAF' from dual union all
select 46211,'USAF' from dual union all
select 46213,'CALG' from dual union all
select 46279,'WFF' from dual union all
select 46285,'GMS' from dual union all
select 46286,'GMS' from dual union all
select 46292,'GMS' from dual union all
select 46300,'GSFC' from dual union all
select 46301,'GMS' from dual union all
select 46311,'WFF' from dual union all
select 46313,'WFF' from dual union all
select 46320,'GMS' from dual union all
select 46326,'GMS' from dual union all
select 46328,'GSFC' from dual union all
select 46329,'GSFC' from dual union all
select 46330,'PSU' from dual union all
select 46334,'GSFC' from dual union all
select 46335,'GSFC' from dual union all
select 46336,'GSFC' from dual union all
select 46337,'GMS' from dual union all
select 46339,'GMS' from dual union all
select 46345,'PSU' from dual union all
select 46351,'USAF' from dual union all
select 46352,'USAF' from dual union all
select 46353,'USAF' from dual union all
select 46354,'GMS' from dual union all
select 46361,'GMS' from dual union all
select 46362,'GMS' from dual union all
select 46381,'GMS' from dual union all
select 46390,'GMS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 46391,'GMS' from dual union all
select 46396,'WFF' from dual union all
select 46412,'GMS' from dual union all
select 46420,'NRCC' from dual union all
select 46447,'WFF' from dual union all
select 46450,'USAF' from dual union all
select 46451,'USAF' from dual union all
select 46452,'USAF' from dual union all
select 46517,'SDCO' from dual union all
select 46519,'SDCO' from dual union all
select 46523,'WFF' from dual union all
select 46527,'SDCO' from dual union all
select 4653,'NRL' from dual union all
select 46530,'SDCO' from dual union all
select 46532,'SDCO' from dual union all
select 46533,'SDCO' from dual union all
select 46573,'WFF' from dual union all
select 46613,'WFF' from dual union all
select 46635,'WFF' from dual union all
select 46643,'WFF' from dual union all
select 46648,'WFF' from dual union all
select 46672,'GMS' from dual union all
select 46696,'GMS' from dual union all
select 46743,'WFF' from dual union all
select 46746,'WFF' from dual union all
select 46798,'GMS' from dual union all
select 46799,'GMS' from dual union all
select 46806,'GMS' from dual union all
select 46807,'GMS' from dual union all
select 46818,'GMS' from dual union all
select 46857,'UMI' from dual union all
select 46864,'WFF' from dual union all
select 46865,'WFF' from dual union all
select 46867,'WFF' from dual union all
select 46894,'WFF' from dual union all
select 46901,'NASDA' from dual union all
select 46907,'WFF' from dual union all
select 46909,'NASDA' from dual union all
select 46911,'UMI' from dual union all
select 46918,'UMI' from dual union all
select 46930,'WFF' from dual union all
select 46933,'WFF' from dual union all
select 46936,'GMS' from dual union all
select 46949,'GMS' from dual union all
select 46959,'GMS' from dual union all
select 4697,'CNRS' from dual union all
select 4697,'PARIS' from dual union all
select 46975,'GMS' from dual union all
select 46986,'GMS' from dual union all
select 47000,'GMS' from dual union all
select 47001,'WFF' from dual union all
select 47007,'WFF' from dual union all
select 47010,'WFF' from dual union all
select 47013,'WFF' from dual union all
select 47017,'GMS' from dual union all
select 4703,'AFCRL' from dual union all
select 47031,'HOUS' from dual union all
select 47032,'HOUS' from dual union all
select 4704,'CNRS' from dual union all
select 4704,'PARIS' from dual union all
select 4705,'NRL' from dual union all
select 4706,'NRL' from dual union all
select 4707,'NRL' from dual union all
select 4709,'AN' from dual union all
select 4710,'AN' from dual union all
select 4711,'AN' from dual union all
select 47128,'HOUS' from dual union all
select 4715,'AFCRL' from dual union all
select 4715,'UUT' from dual union all
select 4719,'AFCRL' from dual union all
select 4719,'UUT' from dual union all
select 4727,'NRL' from dual union all
select 4728,'NRL' from dual union all
select 47292,'PRL' from dual union all
select 4730,'NRL' from dual union all
select 4733,'AFCRL' from dual union all
select 4735,'AFCRL' from dual union all
select 4737,'LMSC' from dual union all
select 4738,'SCEL' from dual union all
select 4738,'UMI' from dual union all
select 4739,'QUB' from dual union all
select 4740,'NRL' from dual union all
select 4741,'SCEL' from dual union all
select 4741,'UMI' from dual union all
select 47418,'GSFC' from dual union all
select 4743,'NRL' from dual union all
select 47443,'UWO' from dual union all
select 47462,'GMS' from dual union all
select 47465,'GMS' from dual union all
select 4747,'SUI' from dual union all
select 47471,'GMS' from dual union all
select 4748,'SUI' from dual union all
select 4749,'UCO' from dual union all
select 4750,'SUI' from dual union all
select 4751,'SUI' from dual union all
select 4754,'SUI' from dual union all
select 4756,'SUI' from dual union all
select 4757,'SUI' from dual union all
select 4758,'SUI' from dual union all
select 4759,'FIAN' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 47595,'MRN' from dual union all
select 47596,'MRN' from dual union all
select 4760,'SUI' from dual union all
select 47604,'SHEF' from dual union all
select 47609,'MRN' from dual union all
select 4761,'SUI' from dual union all
select 4762,'SUI' from dual union all
select 47624,'AFCRL' from dual union all
select 4763,'SUI' from dual union all
select 47630,'SUSX' from dual union all
select 4764,'SUI' from dual union all
select 47642,'AFCRL' from dual union all
select 47643,'AFCRL' from dual union all
select 47644,'AFCRL' from dual union all
select 47645,'ISRO' from dual union all
select 47646,'ISRO' from dual union all
select 47647,'ISRO' from dual union all
select 47648,'ISRO' from dual union all
select 47649,'ISRO' from dual union all
select 47650,'ISRO' from dual union all
select 47651,'ISRO' from dual union all
select 47652,'ISRO' from dual union all
select 47653,'ISRO' from dual union all
select 47654,'ISRO' from dual union all
select 47655,'ISRO' from dual union all
select 47656,'ISRO' from dual union all
select 47657,'ISRO' from dual union all
select 47658,'ISRO' from dual union all
select 47659,'ISRO' from dual union all
select 4766,'NRL' from dual union all
select 47660,'ISRO' from dual union all
select 47661,'ISRO' from dual union all
select 47662,'ISRO' from dual union all
select 47663,'ISRO' from dual union all
select 47664,'ISRO' from dual union all
select 47665,'ISRO' from dual union all
select 47666,'ISRO' from dual union all
select 47667,'ISRO' from dual union all
select 47668,'ISRO' from dual union all
select 47669,'ISRO' from dual union all
select 47670,'ISRO' from dual union all
select 47671,'ISRO' from dual union all
select 47672,'ISRO' from dual union all
select 47673,'ISRO' from dual union all
select 47674,'ISRO' from dual union all
select 47675,'ISRO' from dual union all
select 47676,'ISRO' from dual union all
select 47677,'ISRO' from dual union all
select 47678,'ISRO' from dual union all
select 47679,'ISRO' from dual union all
select 4768,'BRL' from dual union all
select 47680,'ISRO' from dual union all
select 47681,'ISRO' from dual union all
select 47682,'ISRO' from dual union all
select 47683,'ISRO' from dual union all
select 47684,'ISRO' from dual union all
select 47685,'ISRO' from dual union all
select 47686,'ISRO' from dual union all
select 47687,'ISRO' from dual union all
select 47688,'ISRO' from dual union all
select 47689,'ISRO' from dual union all
select 4769,'IAM' from dual union all
select 47690,'ISRO' from dual union all
select 47691,'ISRO' from dual union all
select 47692,'ISRO' from dual union all
select 47693,'ISRO' from dual union all
select 47694,'ISRO' from dual union all
select 47695,'ISRO' from dual union all
select 47696,'ISRO' from dual union all
select 47697,'ISRO' from dual union all
select 47698,'ISRO' from dual union all
select 47699,'ISRO' from dual union all
select 4770,'AN' from dual union all
select 47700,'ISRO' from dual union all
select 47701,'ISRO' from dual union all
select 47702,'ISRO' from dual union all
select 47703,'NPL' from dual union all
select 47704,'ISRO' from dual union all
select 47705,'ISRO' from dual union all
select 47706,'ISRO' from dual union all
select 47707,'ISRO' from dual union all
select 47708,'ISRO' from dual union all
select 47709,'ISRO' from dual union all
select 4771,'SCEL' from dual union all
select 4771,'UMI' from dual union all
select 47710,'ISRO' from dual union all
select 47711,'ISRO' from dual union all
select 47712,'ISRO' from dual union all
select 47713,'ISRO' from dual union all
select 47714,'ISRO' from dual union all
select 47715,'ISRO' from dual union all
select 47716,'ISRO' from dual union all
select 47717,'ISRO' from dual union all
select 47718,'ISRO' from dual union all
select 47719,'ISRO' from dual union all
select 4772,'SUI' from dual union all
select 47720,'ISRO' from dual union all
select 47721,'ISRO' from dual union all
select 47722,'AFGL' from dual union all
select 47722,'PSRC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 47723,'AFGL' from dual union all
select 47723,'PSRC' from dual union all
select 47724,'ISRO' from dual union all
select 47725,'ISRO' from dual union all
select 47726,'ISRO' from dual union all
select 47727,'ISRO' from dual union all
select 47728,'ISRO' from dual union all
select 47729,'ISRO' from dual union all
select 4773,'NRL' from dual union all
select 47730,'ISRO' from dual union all
select 47731,'ISRO' from dual union all
select 47732,'ISRO' from dual union all
select 47733,'ISRO' from dual union all
select 47734,'ISRO' from dual union all
select 47735,'ISRO' from dual union all
select 47736,'ISRO' from dual union all
select 47737,'ISRO' from dual union all
select 47738,'ISRO' from dual union all
select 47739,'ISRO' from dual union all
select 47740,'ISRO' from dual union all
select 47741,'ISRO' from dual union all
select 47742,'ISRO' from dual union all
select 47743,'ISRO' from dual union all
select 47744,'ISRO' from dual union all
select 47745,'ISRO' from dual union all
select 47746,'ISRO' from dual union all
select 47747,'ISRO' from dual union all
select 47748,'ISRO' from dual union all
select 47749,'ISRO' from dual union all
select 4775,'AFCRL' from dual union all
select 4775,'CIT' from dual union all
select 47750,'ISRO' from dual union all
select 47751,'ISRO' from dual union all
select 47752,'ISRO' from dual union all
select 47753,'ISRO' from dual union all
select 47754,'ISRO' from dual union all
select 47755,'ISRO' from dual union all
select 47756,'ISRO' from dual union all
select 47757,'ISRO' from dual union all
select 47758,'ISRO' from dual union all
select 47759,'ISRO' from dual union all
select 4776,'LRL' from dual union all
select 47760,'ISRO' from dual union all
select 47761,'ISRO' from dual union all
select 47762,'ISRO' from dual union all
select 47763,'ISRO' from dual union all
select 47764,'ISRO' from dual union all
select 47765,'ISRO' from dual union all
select 47766,'ISRO' from dual union all
select 47767,'ISRO' from dual union all
select 47768,'ISRO' from dual union all
select 47769,'ISRO' from dual union all
select 4777,'LRL' from dual union all
select 47770,'ISRO' from dual union all
select 47771,'ISRO' from dual union all
select 47772,'ISRO' from dual union all
select 47773,'ISRO' from dual union all
select 47774,'ISRO' from dual union all
select 47775,'ISRO' from dual union all
select 47776,'ISRO' from dual union all
select 47777,'ISRO' from dual union all
select 47778,'ISRO' from dual union all
select 47779,'ISRO' from dual union all
select 4778,'LRL' from dual union all
select 47780,'ISRO' from dual union all
select 47781,'ISRO' from dual union all
select 47782,'ISRO' from dual union all
select 47783,'ISRO' from dual union all
select 47784,'ISRO' from dual union all
select 47785,'ISRO' from dual union all
select 47786,'ISRO' from dual union all
select 47787,'ISRO' from dual union all
select 47788,'ISRO' from dual union all
select 47789,'ISRO' from dual union all
select 47790,'ISRO' from dual union all
select 47791,'ISRO' from dual union all
select 47792,'ISRO' from dual union all
select 47793,'ISRO' from dual union all
select 47794,'ISRO' from dual union all
select 47795,'ISRO' from dual union all
select 47796,'ISRO' from dual union all
select 47797,'ISRO' from dual union all
select 47798,'ISRO' from dual union all
select 47799,'ISRO' from dual union all
select 47800,'ISRO' from dual union all
select 47801,'ISRO' from dual union all
select 47802,'ISRO' from dual union all
select 47803,'ISRO' from dual union all
select 47804,'ISRO' from dual union all
select 47805,'ISRO' from dual union all
select 47806,'ISRO' from dual union all
select 47817,'UCO' from dual union all
select 47818,'MPE' from dual union all
select 47821,'UAF' from dual union all
select 47824,'SHEF' from dual union all
select 47826,'MPE' from dual union all
select 47828,'UAF' from dual union all
select 47831,'RAE' from dual union all
select 47840,'HAO' from dual union all
select 47840,'HCO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 47842,'GSFC' from dual union all
select 47843,'GSFC' from dual union all
select 47844,'GSFC' from dual union all
select 47845,'GSFC' from dual union all
select 47846,'SAND' from dual union all
select 47850,'UMD' from dual union all
select 47851,'UCW' from dual union all
select 47859,'GSFC' from dual union all
select 47861,'GSFC' from dual union all
select 47864,'NRL' from dual union all
select 47865,'UCO' from dual union all
select 47866,'UMI' from dual union all
select 47868,'CNES' from dual union all
select 47872,'UWI' from dual union all
select 47873,'TIFR' from dual union all
select 47876,'JHU' from dual union all
select 47878,'LPARL' from dual union all
select 47878,'LPSP' from dual union all
select 47888,'NRL' from dual union all
select 47890,'GSFC' from dual union all
select 47891,'RSRS' from dual union all
select 47891,'UCW' from dual union all
select 47892,'RSRS' from dual union all
select 47892,'UCW' from dual union all
select 47894,'RSRS' from dual union all
select 47894,'UCW' from dual union all
select 47896,'NRL' from dual union all
select 47905,'AFCRL' from dual union all
select 47907,'AFCRL' from dual union all
select 47909,'SSC' from dual union all
select 47910,'AFCRL' from dual union all
select 47911,'AFCRL' from dual union all
select 47912,'DOE' from dual union all
select 47913,'ISAS' from dual union all
select 47916,'JHU' from dual union all
select 47916,'MISU' from dual union all
select 47916,'UIO' from dual union all
select 47918,'INPE' from dual union all
select 47919,'NASDA' from dual union all
select 47925,'WFF' from dual union all
select 47927,'ISAS' from dual union all
select 47929,'ISAS' from dual union all
select 47930,'PSU' from dual union all
select 47931,'UMI' from dual union all
select 47932,'USCLA' from dual union all
select 47934,'JHU' from dual union all
select 47935,'CORN' from dual union all
select 47938,'USAF' from dual union all
select 47939,'AN' from dual union all
select 47941,'GSFC' from dual union all
select 47943,'UMI' from dual union all
select 47944,'SUSX' from dual union all
select 47945,'TUBI' from dual union all
select 47946,'LASL' from dual union all
select 47946,'SAND' from dual union all
select 47948,'MSFC' from dual union all
select 47951,'AFCRL' from dual union all
select 47952,'WFF' from dual union all
select 47953,'WFF' from dual union all
select 47954,'WFF' from dual union all
select 47956,'DLR' from dual union all
select 47957,'GSFC' from dual union all
select 47961,'DLR' from dual union all
select 47962,'WFF' from dual union all
select 47963,'WFF' from dual union all
select 47964,'WFF' from dual union all
select 47965,'WFF' from dual union all
select 47966,'ASE' from dual union all
select 47967,'UCO' from dual union all
select 47970,'UMN' from dual union all
select 47972,'DFVLR' from dual union all
select 47972,'VSSC' from dual union all
select 47973,'ASE' from dual union all
select 47976,'NRCC' from dual union all
select 47977,'UMI' from dual union all
select 47981,'UWI' from dual union all
select 47982,'SASK' from dual union all
select 47983,'UMI' from dual union all
select 47990,'CNES' from dual union all
select 47992,'PRL' from dual union all
select 47993,'PRL' from dual union all
select 47995,'ISAS' from dual union all
select 4800,'SUI' from dual union all
select 48025,'UKMO' from dual union all
select 48026,'UKMO' from dual union all
select 48027,'UKMO' from dual union all
select 48028,'UKMO' from dual union all
select 48029,'MPE' from dual union all
select 48031,'IIAE' from dual union all
select 48044,'UCO' from dual union all
select 48045,'ISAS' from dual union all
select 48046,'DOE' from dual union all
select 48048,'ISAS' from dual union all
select 48050,'ISAS' from dual union all
select 48053,'RSRS' from dual union all
select 48054,'RICE' from dual union all
select 48055,'NASDA' from dual union all
select 48056,'KGI' from dual union all
select 48057,'RSRS' from dual union all
select 48058,'KGI' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 48059,'KGI' from dual union all
select 48060,'UMI' from dual union all
select 48061,'AFCRL' from dual union all
select 48063,'SAND' from dual union all
select 48064,'CNES' from dual union all
select 48065,'ISAS' from dual union all
select 48068,'RAE' from dual union all
select 48074,'UMI' from dual union all
select 48076,'MPE' from dual union all
select 48076,'TUBI' from dual union all
select 48081,'BERN' from dual union all
select 48081,'UIUC' from dual union all
select 48082,'UNK' from dual union all
select 48083,'CORN' from dual union all
select 48084,'PITT' from dual union all
select 48085,'AFCRL' from dual union all
select 48086,'AFCRL' from dual union all
select 48087,'AFCRL' from dual union all
select 48088,'PSU' from dual union all
select 48089,'AFCRL' from dual union all
select 48090,'BERN' from dual union all
select 48090,'UIUC' from dual union all
select 48091,'AFCRL' from dual union all
select 48092,'BERN' from dual union all
select 48092,'UIUC' from dual union all
select 48093,'AFCRL' from dual union all
select 48094,'PSU' from dual union all
select 48095,'RAE' from dual union all
select 48096,'NRL' from dual union all
select 4810,'USN' from dual union all
select 48103,'MPE' from dual union all
select 48103,'TUBI' from dual union all
select 48105,'UWI' from dual union all
select 48107,'MPE' from dual union all
select 4811,'USN' from dual union all
select 48110,'USAF' from dual union all
select 48111,'USAF' from dual union all
select 48112,'USAF' from dual union all
select 48113,'USAF' from dual union all
select 48114,'USAF' from dual union all
select 48115,'USAF' from dual union all
select 48116,'USAF' from dual union all
select 48117,'USAF' from dual union all
select 48118,'USAF' from dual union all
select 48119,'USAF' from dual union all
select 4812,'USN' from dual union all
select 4814,'USN' from dual union all
select 48153,'MRN' from dual union all
select 48156,'NASDA' from dual union all
select 48157,'NASDA' from dual union all
select 48160,'WFF' from dual union all
select 4818,'LMSC' from dual union all
select 4819,'SUI' from dual union all
select 4820,'BRL' from dual union all
select 48208,'MRN' from dual union all
select 48215,'JMA' from dual union all
select 48230,'USAF' from dual union all
select 48235,'WFF' from dual union all
select 48250,'WFF' from dual union all
select 48251,'WFF' from dual union all
select 4827,'ISAS' from dual union all
select 48279,'JMA' from dual union all
select 4828,'ISAS' from dual union all
select 48311,'JMA' from dual union all
select 4833,'UMD' from dual union all
select 48338,'WFF' from dual union all
select 4834,'UMD' from dual union all
select 48341,'USAF' from dual union all
select 4835,'UMD' from dual union all
select 48361,'JMA' from dual union all
select 48375,'MRN' from dual union all
select 48382,'GSFC' from dual union all
select 48383,'WFF' from dual union all
select 48384,'WFF' from dual union all
select 48385,'GSFC' from dual union all
select 48387,'WFF' from dual union all
select 4839,'UMD' from dual union all
select 48396,'JMA' from dual union all
select 4840,'UMD' from dual union all
select 4841,'KOBE' from dual union all
select 48434,'WFF' from dual union all
select 48435,'WFF' from dual union all
select 48436,'WFF' from dual union all
select 48438,'WFF' from dual union all
select 48440,'WFF' from dual union all
select 48442,'WFF' from dual union all
select 48443,'GMS' from dual union all
select 48452,'USAF' from dual union all
select 48454,'GSFC' from dual union all
select 48465,'JMA' from dual union all
select 48466,'GMS' from dual union all
select 48467,'GMS' from dual union all
select 48489,'JMA' from dual union all
select 485,'JPL' from dual union all
select 48513,'GMS' from dual union all
select 48540,'GMS' from dual union all
select 48549,'WFF' from dual union all
select 48551,'JMA' from dual union all
select 48563,'USAF' from dual union all
select 48569,'JMA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 48570,'GMS' from dual union all
select 48594,'JMA' from dual union all
select 48598,'GMS' from dual union all
select 486,'JPL' from dual union all
select 48608,'GMS' from dual union all
select 4861,'LARC' from dual union all
select 48610,'WFF' from dual union all
select 48616,'GMS' from dual union all
select 48618,'GMS' from dual union all
select 4862,'SUI' from dual union all
select 48621,'GMS' from dual union all
select 48622,'JMA' from dual union all
select 48623,'GMS' from dual union all
select 4863,'SUI' from dual union all
select 48636,'JMA' from dual union all
select 48637,'JMA' from dual union all
select 48638,'JMA' from dual union all
select 48639,'JMA' from dual union all
select 48640,'JMA' from dual union all
select 48642,'JMA' from dual union all
select 48643,'JMA' from dual union all
select 48644,'JMA' from dual union all
select 48645,'JMA' from dual union all
select 48646,'JMA' from dual union all
select 48647,'JMA' from dual union all
select 48648,'JMA' from dual union all
select 48649,'JMA' from dual union all
select 48650,'JMA' from dual union all
select 48651,'JMA' from dual union all
select 48653,'JMA' from dual union all
select 48654,'JMA' from dual union all
select 48655,'JMA' from dual union all
select 48656,'JMA' from dual union all
select 48657,'JMA' from dual union all
select 48658,'JMA' from dual union all
select 4866,'AFCRL' from dual union all
select 48660,'USAF' from dual union all
select 48663,'MRN' from dual union all
select 48664,'GSFC' from dual union all
select 48665,'GSFC' from dual union all
select 48671,'USAF' from dual union all
select 48672,'MRN' from dual union all
select 48676,'USAF' from dual union all
select 48677,'USAF' from dual union all
select 48678,'USAF' from dual union all
select 48679,'GMS' from dual union all
select 4868,'TOK' from dual union all
select 48682,'USAF' from dual union all
select 48686,'MRN' from dual union all
select 4869,'TAO' from dual union all
select 48693,'USAF' from dual union all
select 48694,'MRN' from dual union all
select 48697,'GMS' from dual union all
select 487,'JPL' from dual union all
select 48702,'MRN' from dual union all
select 48704,'USAF' from dual union all
select 4871,'TOK' from dual union all
select 48715,'USAF' from dual union all
select 48716,'MRN' from dual union all
select 48717,'NASDA' from dual union all
select 48718,'GMS' from dual union all
select 48723,'MRN' from dual union all
select 48726,'USAF' from dual union all
select 48733,'MRN' from dual union all
select 48736,'GMS' from dual union all
select 48737,'USAF' from dual union all
select 48743,'MRN' from dual union all
select 48748,'USAF' from dual union all
select 48752,'MRN' from dual union all
select 48755,'MRN' from dual union all
select 48756,'GMS' from dual union all
select 48759,'USAF' from dual union all
select 48763,'MRN' from dual union all
select 48772,'MRN' from dual union all
select 48781,'GMS' from dual union all
select 48782,'MRN' from dual union all
select 48788,'MRN' from dual union all
select 48791,'MRN' from dual union all
select 488,'JPL' from dual union all
select 48804,'MRN' from dual union all
select 48809,'MRN' from dual union all
select 48817,'MRN' from dual union all
select 48827,'MRN' from dual union all
select 48830,'MRN' from dual union all
select 48835,'MRN' from dual union all
select 48838,'MRN' from dual union all
select 48839,'MRN' from dual union all
select 48846,'GMS' from dual union all
select 48850,'MRN' from dual union all
select 48854,'WFF' from dual union all
select 48859,'MRN' from dual union all
select 48862,'MRN' from dual union all
select 48872,'MRN' from dual union all
select 48883,'MRN' from dual union all
select 48886,'MRN' from dual union all
select 4889,'SMSA' from dual union all
select 48890,'MRN' from dual union all
select 4890,'SMSA' from dual union all
select 48903,'MRN' from dual union all
select 4891,'SMSA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 48912,'MRN' from dual union all
select 48914,'WFF' from dual union all
select 48924,'MRN' from dual union all
select 48925,'MRN' from dual union all
select 48926,'WFF' from dual union all
select 48932,'MRN' from dual union all
select 48937,'MRN' from dual union all
select 48942,'MRN' from dual union all
select 48946,'WFF' from dual union all
select 48948,'MRN' from dual union all
select 48952,'WFF' from dual union all
select 48955,'MRN' from dual union all
select 48957,'WFF' from dual union all
select 48961,'MRN' from dual union all
select 48965,'WFF' from dual union all
select 48966,'SDCO' from dual union all
select 48967,'MRN' from dual union all
select 48975,'MRN' from dual union all
select 48980,'USAF' from dual union all
select 48985,'MRN' from dual union all
select 48992,'MRN' from dual union all
select 49005,'WFF' from dual union all
select 49007,'WFF' from dual union all
select 49008,'WFF' from dual union all
select 49013,'MRN' from dual union all
select 49023,'MRN' from dual union all
select 49029,'MRN' from dual union all
select 49031,'WFF' from dual union all
select 49041,'WFF' from dual union all
select 49075,'WFF' from dual union all
select 49078,'MRN' from dual union all
select 49080,'GMS' from dual union all
select 49085,'GMS' from dual union all
select 49089,'WFF' from dual union all
select 49097,'CALG' from dual union all
select 49098,'GMS' from dual union all
select 49107,'CALG' from dual union all
select 49129,'WFF' from dual union all
select 49130,'WFF' from dual union all
select 49178,'WFF' from dual union all
select 49188,'MRN' from dual union all
select 49198,'GMS' from dual union all
select 49212,'GMS' from dual union all
select 49220,'GMS' from dual union all
select 49236,'GMS' from dual union all
select 49240,'GMS' from dual union all
select 49243,'MRN' from dual union all
select 4925,'LARCN' from dual union all
select 49254,'WFF' from dual union all
select 49256,'GMS' from dual union all
select 49257,'GMS' from dual union all
select 4926,'NMSU' from dual union all
select 49261,'GMS' from dual union all
select 49266,'GMS' from dual union all
select 4927,'NMSU' from dual union all
select 4928,'NMSU' from dual union all
select 49281,'GMS' from dual union all
select 49299,'GMS' from dual union all
select 4930,'NMSU' from dual union all
select 4931,'NMSU' from dual union all
select 49313,'GMS' from dual union all
select 4932,'NMSU' from dual union all
select 49327,'GMS' from dual union all
select 4933,'NOTS' from dual union all
select 4934,'NOTS' from dual union all
select 49340,'GMS' from dual union all
select 49360,'GMS' from dual union all
select 49364,'MRN' from dual union all
select 49369,'GMS' from dual union all
select 49391,'GMS' from dual union all
select 49419,'MRN' from dual union all
select 4942,'RAE' from dual union all
select 4942,'UCL' from dual union all
select 49423,'JMA' from dual union all
select 49425,'GMS' from dual union all
select 49441,'MRN' from dual union all
select 49447,'GMS' from dual union all
select 4945,'ISAS' from dual union all
select 49459,'GMS' from dual union all
select 49460,'GMS' from dual union all
select 49461,'GMS' from dual union all
select 49473,'GSFC' from dual union all
select 49488,'JMA' from dual union all
select 49489,'GMS' from dual union all
select 4949,'ISAS' from dual union all
select 4951,'ISAS' from dual union all
select 4952,'ISAS' from dual union all
select 49543,'JMA' from dual union all
select 49562,'MRN' from dual union all
select 49592,'JMA' from dual union all
select 496,'JPL' from dual union all
select 4961,'ISAS' from dual union all
select 4962,'ISAS' from dual union all
select 49629,'ASL' from dual union all
select 49630,'ASL' from dual union all
select 49631,'ASL' from dual union all
select 49632,'ASL' from dual union all
select 49634,'ASL' from dual union all
select 49635,'ASL' from dual union all
select 4967,'ISAS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 4968,'ISAS' from dual union all
select 4969,'BIR' from dual union all
select 4969,'ICST' from dual union all
select 4969,'UCL' from dual union all
select 497,'JPL' from dual union all
select 4971,'ISAS' from dual union all
select 4972,'KOBE' from dual union all
select 49742,'GSFC' from dual union all
select 4976,'LARC' from dual union all
select 4978,'SAND' from dual union all
select 4979,'SAND' from dual union all
select 498,'JPL' from dual union all
select 4980,'KOBE' from dual union all
select 4982,'NOTS' from dual union all
select 49825,'WFF' from dual union all
select 4985,'AFSWC' from dual union all
select 4988,'NRL' from dual union all
select 4989,'AFSWC' from dual union all
select 4990,'NOTS' from dual union all
select 4992,'SCEL' from dual union all
select 49928,'GSFC' from dual union all
select 49929,'GSFC' from dual union all
select 4993,'NRL' from dual union all
select 4997,'SAND' from dual union all
select 4998,'SAND' from dual union all
select 4999,'SAND' from dual union all
select 5000,'SAND' from dual union all
select 50018,'PSU' from dual union all
select 50019,'PSU' from dual union all
select 5002,'SAND' from dual union all
select 5003,'AFSWP' from dual union all
select 5004,'UMD' from dual union all
select 50046,'GSFC' from dual union all
select 5005,'UMD' from dual union all
select 5006,'UMD' from dual union all
select 50064,'GSFC' from dual union all
select 50065,'GSFC' from dual union all
select 50066,'GSFC' from dual union all
select 5008,'UMD' from dual union all
select 5009,'UMD' from dual union all
select 50095,'GMS' from dual union all
select 5010,'UMD' from dual union all
select 5011,'NOTS' from dual union all
select 5012,'NOTS' from dual union all
select 50121,'GSFC' from dual union all
select 50130,'GMS' from dual union all
select 50161,'USU' from dual union all
select 50162,'NRCC' from dual union all
select 50163,'NRCC' from dual union all
select 50164,'NRCC' from dual union all
select 50166,'ASL' from dual union all
select 50167,'NRCC' from dual union all
select 50168,'ASL' from dual union all
select 50169,'NRCC' from dual union all
select 50170,'MRN' from dual union all
select 50171,'MRN' from dual union all
select 50172,'ASL' from dual union all
select 5035,'LARC' from dual union all
select 50350,'GMS' from dual union all
select 50353,'WFF' from dual union all
select 50377,'PSU' from dual union all
select 50378,'ISRO' from dual union all
select 50379,'PSU' from dual union all
select 50380,'ISRO' from dual union all
select 50381,'UMI' from dual union all
select 50382,'UMI' from dual union all
select 50383,'ISRO' from dual union all
select 50384,'ISRO' from dual union all
select 50385,'ISRO' from dual union all
select 50386,'ISRO' from dual union all
select 50387,'ISRO' from dual union all
select 50388,'ISRO' from dual union all
select 50389,'ISRO' from dual union all
select 50390,'ISRO' from dual union all
select 50391,'ISRO' from dual union all
select 50392,'ISRO' from dual union all
select 50393,'ISRO' from dual union all
select 50394,'ISRO' from dual union all
select 50395,'ISRO' from dual union all
select 50396,'ISRO' from dual union all
select 50397,'ISRO' from dual union all
select 50398,'ISRO' from dual union all
select 50399,'ISRO' from dual union all
select 50400,'ISRO' from dual union all
select 50401,'ISRO' from dual union all
select 50402,'ISRO' from dual union all
select 50403,'ISRO' from dual union all
select 50404,'ISRO' from dual union all
select 50405,'ISRO' from dual union all
select 50406,'ISRO' from dual union all
select 50407,'ISRO' from dual union all
select 50408,'ISRO' from dual union all
select 50409,'ISRO' from dual union all
select 50410,'ISRO' from dual union all
select 50411,'ISRO' from dual union all
select 50412,'ISRO' from dual union all
select 50413,'ISRO' from dual union all
select 50414,'ISRO' from dual union all
select 50415,'ISRO' from dual union all
select 50416,'ISRO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 50417,'ISRO' from dual union all
select 50418,'ISRO' from dual union all
select 50419,'ISRO' from dual union all
select 5042,'AFSWP' from dual union all
select 50420,'ISRO' from dual union all
select 50421,'ISRO' from dual union all
select 50422,'ISRO' from dual union all
select 50423,'ISRO' from dual union all
select 50424,'ISRO' from dual union all
select 50425,'ISRO' from dual union all
select 50426,'ISRO' from dual union all
select 50427,'ISRO' from dual union all
select 50428,'ISRO' from dual union all
select 50429,'ISRO' from dual union all
select 5043,'TAO' from dual union all
select 50430,'ISRO' from dual union all
select 50431,'ISRO' from dual union all
select 50432,'ISRO' from dual union all
select 50433,'ISRO' from dual union all
select 50434,'ISRO' from dual union all
select 50435,'ISRO' from dual union all
select 50436,'ISRO' from dual union all
select 50437,'ISRO' from dual union all
select 50438,'ISRO' from dual union all
select 50439,'ISRO' from dual union all
select 5044,'KOBE' from dual union all
select 50440,'ISRO' from dual union all
select 50441,'ISRO' from dual union all
select 50442,'ISRO' from dual union all
select 50443,'ISRO' from dual union all
select 50444,'ISRO' from dual union all
select 50445,'ISRO' from dual union all
select 50446,'ISRO' from dual union all
select 50447,'ISRO' from dual union all
select 50448,'ISRO' from dual union all
select 50449,'ISRO' from dual union all
select 50450,'ISRO' from dual union all
select 50451,'ISRO' from dual union all
select 50452,'ISRO' from dual union all
select 50453,'ISRO' from dual union all
select 50454,'ISRO' from dual union all
select 50455,'ISRO' from dual union all
select 50456,'ISRO' from dual union all
select 50457,'ISRO' from dual union all
select 50458,'ISRO' from dual union all
select 50459,'AFGL' from dual union all
select 50459,'PSRC' from dual union all
select 50460,'AFGL' from dual union all
select 50460,'PSRC' from dual union all
select 50461,'ISRO' from dual union all
select 50462,'ISRO' from dual union all
select 50463,'ISRO' from dual union all
select 50464,'ISRO' from dual union all
select 50465,'ISRO' from dual union all
select 50466,'ISRO' from dual union all
select 50467,'ISRO' from dual union all
select 50468,'ISRO' from dual union all
select 50469,'ISRO' from dual union all
select 5047,'NRL' from dual union all
select 50470,'ISRO' from dual union all
select 50471,'ISRO' from dual union all
select 50472,'ISRO' from dual union all
select 50473,'ISRO' from dual union all
select 50474,'ISRO' from dual union all
select 50475,'ISRO' from dual union all
select 50476,'ISRO' from dual union all
select 50477,'ISRO' from dual union all
select 50478,'ISRO' from dual union all
select 50479,'ISRO' from dual union all
select 5048,'UMI' from dual union all
select 50480,'ISRO' from dual union all
select 50481,'ISRO' from dual union all
select 50482,'ISRO' from dual union all
select 50483,'ISRO' from dual union all
select 50484,'ISRO' from dual union all
select 50485,'ISRO' from dual union all
select 50486,'ISRO' from dual union all
select 50487,'ISRO' from dual union all
select 50488,'ISRO' from dual union all
select 50489,'ISRO' from dual union all
select 5049,'UMI' from dual union all
select 50490,'ISRO' from dual union all
select 50491,'ISRO' from dual union all
select 50492,'ISRO' from dual union all
select 50493,'ISRO' from dual union all
select 50494,'ISRO' from dual union all
select 50495,'ISRO' from dual union all
select 50496,'ISRO' from dual union all
select 50497,'ISRO' from dual union all
select 50498,'ISRO' from dual union all
select 50499,'ISRO' from dual union all
select 5050,'UMI' from dual union all
select 50500,'ISRO' from dual union all
select 50501,'ISRO' from dual union all
select 50502,'ISRO' from dual union all
select 50503,'ISRO' from dual union all
select 50504,'ISRO' from dual union all
select 50505,'ISRO' from dual union all
select 50506,'ISRO' from dual union all
select 50507,'ISRO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 50508,'ISRO' from dual union all
select 50509,'ISRO' from dual union all
select 5051,'UMI' from dual union all
select 50510,'ISRO' from dual union all
select 50511,'ISRO' from dual union all
select 50512,'ISRO' from dual union all
select 50513,'ISRO' from dual union all
select 50514,'ISRO' from dual union all
select 50515,'ISRO' from dual union all
select 50516,'ISRO' from dual union all
select 50517,'ISRO' from dual union all
select 50518,'ISRO' from dual union all
select 50519,'ISRO' from dual union all
select 5052,'UMI' from dual union all
select 50520,'ISRO' from dual union all
select 50521,'ISRO' from dual union all
select 50522,'ISRO' from dual union all
select 50523,'ISRO' from dual union all
select 50524,'ISRO' from dual union all
select 50525,'ISRO' from dual union all
select 50526,'ISRO' from dual union all
select 50527,'ISRO' from dual union all
select 50528,'ISRO' from dual union all
select 50529,'ISRO' from dual union all
select 5053,'SAND' from dual union all
select 50530,'ISRO' from dual union all
select 50531,'ISRO' from dual union all
select 50532,'ISRO' from dual union all
select 50533,'ISRO' from dual union all
select 50534,'ISRO' from dual union all
select 50535,'ISRO' from dual union all
select 50536,'ISRO' from dual union all
select 50537,'ISRO' from dual union all
select 50538,'ISRO' from dual union all
select 50539,'ISRO' from dual union all
select 5054,'SAND' from dual union all
select 50540,'ISRO' from dual union all
select 50541,'ISRO' from dual union all
select 50542,'ISRO' from dual union all
select 50543,'ISRO' from dual union all
select 5055,'UMI' from dual union all
select 5056,'UMI' from dual union all
select 50561,'MSFC' from dual union all
select 50567,'AFCRL' from dual union all
select 50568,'GSFC' from dual union all
select 5057,'UMI' from dual union all
select 50579,'LASL' from dual union all
select 50580,'NRL' from dual union all
select 50582,'UWI' from dual union all
select 50588,'AERO' from dual union all
select 5059,'WRE' from dual union all
select 5060,'SAND' from dual union all
select 50602,'UCO' from dual union all
select 50604,'SHEF' from dual union all
select 50605,'UMN' from dual union all
select 50606,'GSFC' from dual union all
select 50607,'GSFC' from dual union all
select 50609,'PITT' from dual union all
select 5061,'LMSC' from dual union all
select 50610,'CORN' from dual union all
select 50611,'AFCRL' from dual union all
select 50617,'ISAS' from dual union all
select 50618,'INPE' from dual union all
select 5062,'KOBE' from dual union all
select 50624,'SAND' from dual union all
select 50625,'SAND' from dual union all
select 50626,'ISAS' from dual union all
select 50627,'ISAS' from dual union all
select 50628,'NASDA' from dual union all
select 50629,'GSFC' from dual union all
select 5063,'SCEL' from dual union all
select 50634,'AFCRL' from dual union all
select 50635,'LPARL' from dual union all
select 50636,'LPARL' from dual union all
select 50636,'LPSP' from dual union all
select 50638,'NRL' from dual union all
select 50639,'USCLA' from dual union all
select 5064,'WRE' from dual union all
select 50640,'GSFC' from dual union all
select 50642,'GSFC' from dual union all
select 50642,'NDRE' from dual union all
select 50644,'PITT' from dual union all
select 50645,'GSFC' from dual union all
select 50646,'GSFC' from dual union all
select 5065,'WRE' from dual union all
select 50651,'GSFC' from dual union all
select 50653,'GSFC' from dual union all
select 50654,'AFCRL' from dual union all
select 50656,'UWI' from dual union all
select 50657,'RSRS' from dual union all
select 50657,'UCW' from dual union all
select 50658,'NDRE' from dual union all
select 50659,'MPE' from dual union all
select 50659,'NDRE' from dual union all
select 5066,'WRE' from dual union all
select 50660,'UCL' from dual union all
select 50661,'MPAE' from dual union all
select 50664,'RSRS' from dual union all
select 50664,'UCW' from dual union all
select 50665,'FREIB' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 50665,'WUPP' from dual union all
select 50666,'AFCRL' from dual union all
select 50666,'USU' from dual union all
select 50667,'NDRE' from dual union all
select 50668,'MPE' from dual union all
select 50668,'NDRE' from dual union all
select 50669,'MPAE' from dual union all
select 5067,'WRE' from dual union all
select 50670,'BERN' from dual union all
select 50670,'UIUC' from dual union all
select 50671,'BERN' from dual union all
select 50671,'UIUC' from dual union all
select 50672,'AFGL' from dual union all
select 50672,'UCL' from dual union all
select 50675,'MSFC' from dual union all
select 50676,'GMS' from dual union all
select 50677,'GMS' from dual union all
select 50679,'NDRE' from dual union all
select 5068,'WRE' from dual union all
select 50680,'MPE' from dual union all
select 50680,'NDRE' from dual union all
select 50681,'MPAE' from dual union all
select 50682,'RSRS' from dual union all
select 50682,'UCW' from dual union all
select 50683,'BERN' from dual union all
select 50683,'UIUC' from dual union all
select 50684,'AFGL' from dual union all
select 50684,'UCL' from dual union all
select 50686,'AFGL' from dual union all
select 50686,'UCL' from dual union all
select 50687,'SAND' from dual union all
select 50689,'GMS' from dual union all
select 5069,'SUI' from dual union all
select 50690,'SAND' from dual union all
select 50691,'SAND' from dual union all
select 50692,'SAND' from dual union all
select 50693,'SAND' from dual union all
select 50694,'GSFC' from dual union all
select 50697,'GMS' from dual union all
select 50698,'GMS' from dual union all
select 50699,'GMS' from dual union all
select 50700,'UMD' from dual union all
select 50701,'SAND' from dual union all
select 50702,'SAND' from dual union all
select 50704,'GMS' from dual union all
select 50705,'GMS' from dual union all
select 50706,'UMD' from dual union all
select 50708,'INPE' from dual union all
select 50709,'PRL' from dual union all
select 5071,'AFSWP' from dual union all
select 50710,'NPL' from dual union all
select 50726,'ISAS' from dual union all
select 50726,'USU' from dual union all
select 50727,'DLR' from dual union all
select 50728,'DLR' from dual union all
select 50729,'DLR' from dual union all
select 50737,'ISAS' from dual union all
select 50737,'TSUK' from dual union all
select 5074,'UMI' from dual union all
select 50746,'CNES' from dual union all
select 50746,'CONIE' from dual union all
select 50746,'NASA' from dual union all
select 50747,'CNES' from dual union all
select 50747,'CONIE' from dual union all
select 50747,'NASA' from dual union all
select 50748,'UWI' from dual union all
select 5075,'BRL' from dual union all
select 50759,'BERG' from dual union all
select 50759,'UIO' from dual union all
select 5076,'AFCRL' from dual union all
select 50764,'UMN' from dual union all
select 50765,'NASDA' from dual union all
select 50767,'ISAS' from dual union all
select 50777,'SASK' from dual union all
select 50778,'LASL' from dual union all
select 50779,'LASL' from dual union all
select 5078,'NRL' from dual union all
select 50780,'ISAS' from dual union all
select 50780,'ISRO' from dual union all
select 50781,'NPL' from dual union all
select 50781,'PRL' from dual union all
select 50782,'HAO' from dual union all
select 50782,'HCO' from dual union all
select 50783,'NPL' from dual union all
select 50783,'PRL' from dual union all
select 50784,'ISAS' from dual union all
select 50784,'ISRO' from dual union all
select 50785,'NPL' from dual union all
select 50785,'PRL' from dual union all
select 50786,'ISAS' from dual union all
select 50787,'NPL' from dual union all
select 50787,'PRL' from dual union all
select 50789,'BERG' from dual union all
select 50789,'UIO' from dual union all
select 50791,'SASK' from dual union all
select 50800,'UAF' from dual union all
select 50801,'UAF' from dual union all
select 50803,'SAO' from dual union all
select 50804,'UAF' from dual union all
select 50807,'UMI' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 5081,'SAND' from dual union all
select 50818,'NRCC' from dual union all
select 5082,'NRL' from dual union all
select 5083,'UMI' from dual union all
select 50831,'ERNO' from dual union all
select 50831,'SSC' from dual union all
select 5084,'NRL' from dual union all
select 5085,'SAND' from dual union all
select 50852,'JMA' from dual union all
select 50853,'GMS' from dual union all
select 50883,'JMA' from dual union all
select 5089,'LRL' from dual union all
select 50890,'GMS' from dual union all
select 50891,'GMS' from dual union all
select 50892,'GMS' from dual union all
select 5090,'LRL' from dual union all
select 50901,'GMS' from dual union all
select 50902,'GMS' from dual union all
select 5091,'LRL' from dual union all
select 5092,'LRL' from dual union all
select 50921,'JMA' from dual union all
select 5093,'LRL' from dual union all
select 50939,'JMA' from dual union all
select 5094,'NRL' from dual union all
select 50940,'WFF' from dual union all
select 5095,'LRL' from dual union all
select 5096,'LRL' from dual union all
select 50960,'GMS' from dual union all
select 50961,'JMA' from dual union all
select 50966,'GMS' from dual union all
select 50968,'GMS' from dual union all
select 5097,'LRL' from dual union all
select 5098,'LRL' from dual union all
select 50980,'JMA' from dual union all
select 5099,'LRL' from dual union all
select 50990,'GMS' from dual union all
select 5100,'LRL' from dual union all
select 51008,'JMA' from dual union all
select 5101,'LRL' from dual union all
select 51010,'JMA' from dual union all
select 51011,'JMA' from dual union all
select 51012,'JMA' from dual union all
select 51016,'GMS' from dual union all
select 5102,'LRL' from dual union all
select 51024,'GMS' from dual union all
select 51026,'GMS' from dual union all
select 5103,'AFSWC' from dual union all
select 51036,'JMA' from dual union all
select 5104,'SAND' from dual union all
select 51044,'WFF' from dual union all
select 51049,'GMS' from dual union all
select 5105,'SCEL' from dual union all
select 5105,'UMI' from dual union all
select 51058,'PSU' from dual union all
select 5106,'SAND' from dual union all
select 51062,'WFF' from dual union all
select 51064,'WFF' from dual union all
select 51066,'JMA' from dual union all
select 51067,'GMS' from dual union all
select 5107,'SAND' from dual union all
select 51079,'NASDA' from dual union all
select 5108,'SAND' from dual union all
select 51080,'WFF' from dual union all
select 51084,'GMS' from dual union all
select 51087,'GMS' from dual union all
select 51088,'GMS' from dual union all
select 5109,'FIAN' from dual union all
select 5109,'IAM' from dual union all
select 51090,'GMS' from dual union all
select 51091,'GMS' from dual union all
select 51092,'GMS' from dual union all
select 51093,'GMS' from dual union all
select 51097,'GMS' from dual union all
select 51099,'GMS' from dual union all
select 51100,'GMS' from dual union all
select 51101,'GMS' from dual union all
select 51102,'GMS' from dual union all
select 51103,'GMS' from dual union all
select 51110,'GMS' from dual union all
select 51111,'GMS' from dual union all
select 51112,'GMS' from dual union all
select 51121,'WFF' from dual union all
select 51122,'GMS' from dual union all
select 5113,'SAND' from dual union all
select 51131,'JMA' from dual union all
select 51132,'GMS' from dual union all
select 5114,'SAND' from dual union all
select 51149,'WFF' from dual union all
select 5115,'SAND' from dual union all
select 51157,'GMS' from dual union all
select 51158,'WFF' from dual union all
select 5116,'UMI' from dual union all
select 5117,'SAND' from dual union all
select 51172,'GMS' from dual union all
select 5118,'SAND' from dual union all
select 51189,'WFF' from dual union all
select 5119,'SAND' from dual union all
select 51190,'JMA' from dual union all
select 51192,'GMS' from dual union all
select 51193,'JMA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 5120,'SAND' from dual union all
select 51202,'GMS' from dual union all
select 5121,'LRL' from dual union all
select 5122,'LRL' from dual union all
select 51221,'JMA' from dual union all
select 51222,'GMS' from dual union all
select 5123,'LRL' from dual union all
select 51232,'GMS' from dual union all
select 5124,'LRL' from dual union all
select 5125,'LRL' from dual union all
select 5126,'LRL' from dual union all
select 51266,'JMA' from dual union all
select 5127,'SCEL' from dual union all
select 5127,'UMI' from dual union all
select 51273,'WFF' from dual union all
select 5128,'LRL' from dual union all
select 5129,'LRL' from dual union all
select 51298,'JMA' from dual union all
select 5130,'LRL' from dual union all
select 5131,'LRL' from dual union all
select 51315,'GMS' from dual union all
select 5132,'LRL' from dual union all
select 51328,'GMS' from dual union all
select 5133,'LRL' from dual union all
select 5134,'LRL' from dual union all
select 51340,'GMS' from dual union all
select 5135,'LRL' from dual union all
select 51352,'JMA' from dual union all
select 51358,'GMS' from dual union all
select 5136,'AFCRL' from dual union all
select 51369,'GMS' from dual union all
select 5137,'AN' from dual union all
select 5138,'UMI' from dual union all
select 51380,'GMS' from dual union all
select 51385,'JMA' from dual union all
select 5139,'FIAN' from dual union all
select 51398,'WFF' from dual union all
select 5140,'SCEL' from dual union all
select 51405,'JMA' from dual union all
select 5141,'AFSWC' from dual union all
select 5142,'AFCRL' from dual union all
select 51421,'GMS' from dual union all
select 51432,'JMA' from dual union all
select 5144,'AFSWC' from dual union all
select 5145,'AFSWC' from dual union all
select 51452,'WFF' from dual union all
select 51456,'JMA' from dual union all
select 51457,'JMA' from dual union all
select 51467,'GMS' from dual union all
select 51468,'GMS' from dual union all
select 5147,'AFSWP' from dual union all
select 5147,'TF88' from dual union all
select 51474,'GMS' from dual union all
select 51475,'GMS' from dual union all
select 51477,'GMS' from dual union all
select 51479,'GMS' from dual union all
select 5148,'AFSWC' from dual union all
select 51485,'JMA' from dual union all
select 5150,'FIAN' from dual union all
select 5150,'IAM' from dual union all
select 5151,'AFSWC' from dual union all
select 51510,'WFF' from dual union all
select 51515,'WFF' from dual union all
select 51517,'JMA' from dual union all
select 5152,'AN' from dual union all
select 5153,'AFSWC' from dual union all
select 5154,'AFSWC' from dual union all
select 51552,'GMS' from dual union all
select 5157,'AFSWP' from dual union all
select 5157,'TF88' from dual union all
select 51572,'JMA' from dual union all
select 5158,'AFSWC' from dual union all
select 51581,'GMS' from dual union all
select 51587,'GMS' from dual union all
select 5159,'AFSWC' from dual union all
select 516,'JPL' from dual union all
select 5160,'AFSWC' from dual union all
select 51606,'WFF' from dual union all
select 51607,'WFF' from dual union all
select 5161,'AFSWC' from dual union all
select 51611,'GMS' from dual union all
select 5162,'AFSWC' from dual union all
select 5163,'AFSWC' from dual union all
select 5164,'AFSWC' from dual union all
select 51646,'WFF' from dual union all
select 5165,'AFSWC' from dual union all
select 51657,'JMA' from dual union all
select 51663,'WFF' from dual union all
select 51668,'WFF' from dual union all
select 51670,'WFF' from dual union all
select 51671,'WFF' from dual union all
select 5168,'AFSWC' from dual union all
select 5169,'NRL' from dual union all
select 517,'JPL' from dual union all
select 5170,'AFSWC' from dual union all
select 51704,'JMA' from dual union all
select 5171,'AFSWC' from dual union all
select 51710,'JMA' from dual union all
select 5172,'AFSWC' from dual union all
select 5174,'AFSWP' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 5174,'TF88' from dual union all
select 51750,'JMA' from dual union all
select 51756,'WFF' from dual union all
select 51759,'WFF' from dual union all
select 5176,'LARC' from dual union all
select 5178,'AFCRL' from dual union all
select 51789,'JMA' from dual union all
select 5179,'AFCRL' from dual union all
select 51793,'GMS' from dual union all
select 51797,'GMS' from dual union all
select 518,'JPL' from dual union all
select 5180,'NRL' from dual union all
select 51802,'GMS' from dual union all
select 51816,'GMS' from dual union all
select 51817,'GMS' from dual union all
select 5182,'AFCRL' from dual union all
select 51821,'GMS' from dual union all
select 51829,'GMS' from dual union all
select 51834,'NASDA' from dual union all
select 51836,'NASDA' from dual union all
select 5184,'AN' from dual union all
select 51847,'JMA' from dual union all
select 51856,'GMS' from dual union all
select 5186,'UMI' from dual union all
select 51871,'GMS' from dual union all
select 5189,'NRL' from dual union all
select 51898,'JMA' from dual union all
select 51899,'GMS' from dual union all
select 519,'JPL' from dual union all
select 51900,'GMS' from dual union all
select 51910,'GMS' from dual union all
select 51915,'WFF' from dual union all
select 51917,'WFF' from dual union all
select 51923,'JMA' from dual union all
select 51925,'GMS' from dual union all
select 51927,'WFF' from dual union all
select 51932,'WFF' from dual union all
select 51934,'WFF' from dual union all
select 51935,'WFF' from dual union all
select 51939,'GMS' from dual union all
select 5194,'FIAN' from dual union all
select 51947,'WFF' from dual union all
select 5195,'AN' from dual union all
select 51951,'WFF' from dual union all
select 51953,'WFF' from dual union all
select 51955,'GMS' from dual union all
select 51959,'WFF' from dual union all
select 51961,'WFF' from dual union all
select 51962,'WFF' from dual union all
select 51967,'JMA' from dual union all
select 51968,'GMS' from dual union all
select 5197,'LARC' from dual union all
select 51977,'GMS' from dual union all
select 5199,'AN' from dual union all
select 5201,'FIAN' from dual union all
select 5203,'NRL' from dual union all
select 5204,'NRL' from dual union all
select 5205,'NRL' from dual union all
select 5206,'NRL' from dual union all
select 5207,'NRL' from dual union all
select 5208,'NRL' from dual union all
select 5209,'AFCRL' from dual union all
select 5209,'OKSU' from dual union all
select 5211,'UMI' from dual union all
select 5212,'AFCRL' from dual union all
select 5212,'OKSU' from dual union all
select 5213,'SUI' from dual union all
select 5216,'AFCRL' from dual union all
select 5216,'OKSU' from dual union all
select 5217,'UMI' from dual union all
select 5218,'AFCRL' from dual union all
select 5218,'OKSU' from dual union all
select 5219,'SUI' from dual union all
select 522,'JPL' from dual union all
select 5220,'AFCRL' from dual union all
select 5220,'OKSU' from dual union all
select 5222,'AFCRL' from dual union all
select 5222,'OKSU' from dual union all
select 5223,'AN' from dual union all
select 5224,'SUI' from dual union all
select 5226,'AN' from dual union all
select 52269,'GMS' from dual union all
select 5227,'NRL' from dual union all
select 5228,'BRL' from dual union all
select 52280,'GMS' from dual union all
select 52298,'PSU' from dual union all
select 523,'JPL' from dual union all
select 52308,'GMS' from dual union all
select 5232,'NRL' from dual union all
select 52320,'GMS' from dual union all
select 5233,'BRL' from dual union all
select 5234,'NRL' from dual union all
select 52350,'WFF' from dual union all
select 52351,'WFF' from dual union all
select 52356,'WFF' from dual union all
select 52359,'WFF' from dual union all
select 5236,'AN' from dual union all
select 52360,'WFF' from dual union all
select 52361,'WFF' from dual union all
select 52367,'WFF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 52370,'WFF' from dual union all
select 52375,'AFCRL' from dual union all
select 5238,'CARDE' from dual union all
select 5239,'CARDE' from dual union all
select 524,'JPL' from dual union all
select 5240,'BRL' from dual union all
select 52405,'GMS' from dual union all
select 52406,'GMS' from dual union all
select 52407,'GMS' from dual union all
select 5241,'NRL' from dual union all
select 52413,'GMS' from dual union all
select 52417,'GMS' from dual union all
select 52424,'GMS' from dual union all
select 52425,'HOUS' from dual union all
select 52427,'GMS' from dual union all
select 52428,'GMS' from dual union all
select 52435,'HOUS' from dual union all
select 52436,'GMS' from dual union all
select 52448,'PRL' from dual union all
select 52449,'PRL' from dual union all
select 5245,'BRL' from dual union all
select 52450,'NDRE' from dual union all
select 52451,'NDRE' from dual union all
select 52452,'NDRE' from dual union all
select 52453,'NDRE' from dual union all
select 52454,'NDRE' from dual union all
select 52455,'NDRE' from dual union all
select 52457,'NDRE' from dual union all
select 52458,'NDRE' from dual union all
select 52459,'NDRE' from dual union all
select 5246,'AN' from dual union all
select 52460,'NDRE' from dual union all
select 52461,'NDRE' from dual union all
select 52462,'NDRE' from dual union all
select 52463,'NDRE' from dual union all
select 52464,'NDRE' from dual union all
select 52465,'NDRE' from dual union all
select 52467,'NDRE' from dual union all
select 52468,'NDRE' from dual union all
select 52469,'NDRE' from dual union all
select 5247,'BRL' from dual union all
select 52470,'NDRE' from dual union all
select 52471,'NDRE' from dual union all
select 52472,'NDRE' from dual union all
select 52473,'NDRE' from dual union all
select 52474,'NDRE' from dual union all
select 52475,'NDRE' from dual union all
select 52476,'NDRE' from dual union all
select 52478,'NDRE' from dual union all
select 5248,'SAND' from dual union all
select 5249,'SAND' from dual union all
select 5250,'AN' from dual union all
select 52507,'PSU' from dual union all
select 52509,'PRL' from dual union all
select 5251,'UMI' from dual union all
select 52510,'PSU' from dual union all
select 52511,'PRL' from dual union all
select 5252,'AFCRL' from dual union all
select 5253,'NRL' from dual union all
select 5255,'AN' from dual union all
select 5257,'NRL' from dual union all
select 5260,'BRL' from dual union all
select 5260,'UMI' from dual union all
select 5261,'BRL' from dual union all
select 5262,'NRL' from dual union all
select 5263,'QUB' from dual union all
select 5264,'NRL' from dual union all
select 52661,'GMS' from dual union all
select 52662,'GMS' from dual union all
select 52665,'WFF' from dual union all
select 5267,'NMSU' from dual union all
select 5268,'NMSU' from dual union all
select 52688,'UMI' from dual union all
select 52689,'OTR' from dual union all
select 52690,'OTR' from dual union all
select 52691,'OTR' from dual union all
select 52692,'OTR' from dual union all
select 52693,'OTR' from dual union all
select 52694,'OTR' from dual union all
select 52695,'OTR' from dual union all
select 52696,'OTR' from dual union all
select 52697,'OTR' from dual union all
select 52698,'OTR' from dual union all
select 52699,'OTR' from dual union all
select 52700,'OTR' from dual union all
select 52701,'OTR' from dual union all
select 52702,'PSU' from dual union all
select 52703,'MPIK' from dual union all
select 52704,'NRCC' from dual union all
select 52705,'ISRO' from dual union all
select 52706,'ISRO' from dual union all
select 52707,'ISRO' from dual union all
select 52708,'ISRO' from dual union all
select 52709,'ISRO' from dual union all
select 52710,'ISRO' from dual union all
select 52711,'ISRO' from dual union all
select 52712,'ISRO' from dual union all
select 52713,'ISRO' from dual union all
select 52714,'ISRO' from dual union all
select 52715,'ISRO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 52716,'ISRO' from dual union all
select 52717,'ISRO' from dual union all
select 52718,'ISRO' from dual union all
select 52719,'ISRO' from dual union all
select 52720,'ISRO' from dual union all
select 52721,'ISRO' from dual union all
select 52722,'ISRO' from dual union all
select 52723,'ISRO' from dual union all
select 52724,'ISRO' from dual union all
select 52725,'ISRO' from dual union all
select 52726,'ISRO' from dual union all
select 52727,'ISRO' from dual union all
select 52728,'ISRO' from dual union all
select 52729,'ISRO' from dual union all
select 52730,'ISRO' from dual union all
select 52731,'ISRO' from dual union all
select 52732,'ISRO' from dual union all
select 52733,'ISRO' from dual union all
select 52734,'ISRO' from dual union all
select 52735,'ISRO' from dual union all
select 52736,'ISRO' from dual union all
select 52737,'ISRO' from dual union all
select 52738,'ISRO' from dual union all
select 52739,'ISRO' from dual union all
select 52740,'ISRO' from dual union all
select 52741,'ISRO' from dual union all
select 52742,'ISRO' from dual union all
select 52743,'ISRO' from dual union all
select 52744,'ISRO' from dual union all
select 52745,'ISRO' from dual union all
select 52746,'ISRO' from dual union all
select 52747,'ISRO' from dual union all
select 52748,'ISRO' from dual union all
select 52749,'ISRO' from dual union all
select 5275,'SAND' from dual union all
select 52750,'ISRO' from dual union all
select 52751,'ISRO' from dual union all
select 52752,'ISRO' from dual union all
select 52753,'ISRO' from dual union all
select 52754,'ISRO' from dual union all
select 52755,'ISRO' from dual union all
select 52756,'ISRO' from dual union all
select 52757,'ISRO' from dual union all
select 52758,'ISRO' from dual union all
select 52759,'ISRO' from dual union all
select 5276,'FIAN' from dual union all
select 52760,'ISRO' from dual union all
select 52761,'ISRO' from dual union all
select 52762,'ISRO' from dual union all
select 52763,'ISRO' from dual union all
select 52764,'ISRO' from dual union all
select 52765,'ISRO' from dual union all
select 52766,'ISRO' from dual union all
select 52767,'ISRO' from dual union all
select 52768,'ISRO' from dual union all
select 52769,'ISRO' from dual union all
select 5277,'SAND' from dual union all
select 52770,'ISRO' from dual union all
select 52771,'ISRO' from dual union all
select 52772,'ISRO' from dual union all
select 52773,'ISRO' from dual union all
select 5278,'SAND' from dual union all
select 52785,'JHU' from dual union all
select 52790,'DLR' from dual union all
select 52791,'SAO' from dual union all
select 52792,'UCO' from dual union all
select 52793,'DLR' from dual union all
select 52795,'JHU' from dual union all
select 5280,'AN' from dual union all
select 5281,'FIAN' from dual union all
select 52817,'UMI' from dual union all
select 52818,'UMI' from dual union all
select 52823,'AFCRL' from dual union all
select 52824,'NASDA' from dual union all
select 52825,'CORN' from dual union all
select 52826,'CORN' from dual union all
select 52832,'GSFC' from dual union all
select 52834,'PSU' from dual union all
select 52835,'ISAS' from dual union all
select 52836,'PITT' from dual union all
select 52837,'AN' from dual union all
select 5284,'AN' from dual union all
select 52843,'ISAS' from dual union all
select 52844,'ISAS' from dual union all
select 52844,'TOK' from dual union all
select 52846,'IKI' from dual union all
select 52847,'GSFC' from dual union all
select 52848,'CORN' from dual union all
select 52848,'SSC' from dual union all
select 52851,'UIUC' from dual union all
select 52852,'GSFC' from dual union all
select 52853,'LUX' from dual union all
select 52853,'MIT' from dual union all
select 52855,'BERG' from dual union all
select 52855,'NDRE' from dual union all
select 52856,'AFCRL' from dual union all
select 52857,'AFCRL' from dual union all
select 52858,'UCO' from dual union all
select 52859,'CAL' from dual union all
select 5286,'FIAN' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 52861,'NRL' from dual union all
select 52862,'UCO' from dual union all
select 52865,'UCL' from dual union all
select 52866,'SWRI' from dual union all
select 52867,'NRCC' from dual union all
select 52869,'AERO' from dual union all
select 52870,'UMI' from dual union all
select 52873,'UCL' from dual union all
select 52874,'FREIB' from dual union all
select 52874,'WUPP' from dual union all
select 52875,'INPE' from dual union all
select 52876,'SWRI' from dual union all
select 52877,'UCL' from dual union all
select 52878,'IAA' from dual union all
select 52880,'PRL' from dual union all
select 52881,'PRL' from dual union all
select 52883,'PRL' from dual union all
select 52942,'INTA' from dual union all
select 52943,'INTA' from dual union all
select 52944,'INTA' from dual union all
select 52946,'IIAE' from dual union all
select 5297,'NRL' from dual union all
select 52977,'UMD' from dual union all
select 52978,'NASDA' from dual union all
select 52979,'UTOR' from dual union all
select 52980,'NRL' from dual union all
select 52981,'MSFC' from dual union all
select 52982,'ISAS' from dual union all
select 52983,'NRCC' from dual union all
select 52984,'UMI' from dual union all
select 52985,'ISAS' from dual union all
select 52985,'USU' from dual union all
select 52986,'AFCRL' from dual union all
select 52990,'AFCRL' from dual union all
select 52991,'RSRS' from dual union all
select 52991,'UCW' from dual union all
select 52992,'STO' from dual union all
select 52992,'UIO' from dual union all
select 52995,'ASE' from dual union all
select 52999,'INTA' from dual union all
select 53002,'MITLL' from dual union all
select 53003,'OTR' from dual union all
select 53004,'MITLL' from dual union all
select 53005,'AFCRL' from dual union all
select 53006,'AFCRL' from dual union all
select 53007,'AFCRL' from dual union all
select 53008,'AFCRL' from dual union all
select 53009,'PITT' from dual union all
select 53010,'MITLL' from dual union all
select 53011,'PITT' from dual union all
select 53013,'CIT' from dual union all
select 53014,'UAF' from dual union all
select 53015,'NRCC' from dual union all
select 53016,'NRCC' from dual union all
select 53017,'UAF' from dual union all
select 53019,'UAF' from dual union all
select 53021,'GSFC' from dual union all
select 53022,'GSFC' from dual union all
select 53023,'UAF' from dual union all
select 53031,'JMA' from dual union all
select 53038,'GMS' from dual union all
select 53039,'GMS' from dual union all
select 53043,'GMS' from dual union all
select 53045,'GMS' from dual union all
select 53048,'JMA' from dual union all
select 53052,'GMS' from dual union all
select 53055,'GMS' from dual union all
select 53060,'JMA' from dual union all
select 53062,'WFF' from dual union all
select 53065,'WFF' from dual union all
select 53070,'GMS' from dual union all
select 5308,'SUI' from dual union all
select 53092,'GMS' from dual union all
select 53102,'JMA' from dual union all
select 53103,'GMS' from dual union all
select 53120,'JMA' from dual union all
select 53134,'GMS' from dual union all
select 53136,'GMS' from dual union all
select 53145,'JMA' from dual union all
select 53146,'GMS' from dual union all
select 53149,'GMS' from dual union all
select 53168,'GMS' from dual union all
select 53175,'GMS' from dual union all
select 53180,'GMS' from dual union all
select 53183,'JMA' from dual union all
select 53186,'GMS' from dual union all
select 5319,'NRL' from dual union all
select 53191,'GMS' from dual union all
select 53213,'GMS' from dual union all
select 53232,'JMA' from dual union all
select 53244,'JMA' from dual union all
select 53245,'GMS' from dual union all
select 53247,'GMS' from dual union all
select 53258,'GMS' from dual union all
select 53261,'GMS' from dual union all
select 53263,'GMS' from dual union all
select 53264,'GMS' from dual union all
select 53266,'JMA' from dual union all
select 53268,'JMA' from dual union all
select 53275,'GMS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 53283,'GMS' from dual union all
select 53287,'GMS' from dual union all
select 53290,'WFF' from dual union all
select 53298,'JMA' from dual union all
select 5330,'UMI' from dual union all
select 53308,'GMS' from dual union all
select 53317,'JMA' from dual union all
select 53318,'GMS' from dual union all
select 53326,'GMS' from dual union all
select 53330,'GMS' from dual union all
select 53337,'JMA' from dual union all
select 53338,'GMS' from dual union all
select 53349,'GMS' from dual union all
select 53363,'GMS' from dual union all
select 53367,'JMA' from dual union all
select 53371,'GMS' from dual union all
select 53381,'JMA' from dual union all
select 53382,'GMS' from dual union all
select 53388,'GMS' from dual union all
select 53393,'GMS' from dual union all
select 53399,'GMS' from dual union all
select 53402,'GMS' from dual union all
select 53403,'GMS' from dual union all
select 53404,'GMS' from dual union all
select 53405,'GMS' from dual union all
select 5341,'FIAN' from dual union all
select 53410,'GMS' from dual union all
select 53412,'GMS' from dual union all
select 53414,'JMA' from dual union all
select 53415,'GMS' from dual union all
select 53424,'GMS' from dual union all
select 53428,'GMS' from dual union all
select 53444,'WFF' from dual union all
select 53449,'JMA' from dual union all
select 53477,'JMA' from dual union all
select 53481,'JMA' from dual union all
select 53482,'WFF' from dual union all
select 53488,'GMS' from dual union all
select 53515,'JMA' from dual union all
select 5352,'SUI' from dual union all
select 53522,'PSU' from dual union all
select 53541,'JMA' from dual union all
select 53564,'JMA' from dual union all
select 53570,'GMS' from dual union all
select 53582,'GMS' from dual union all
select 53589,'GMS' from dual union all
select 53596,'WFF' from dual union all
select 53604,'GMS' from dual union all
select 53609,'GMS' from dual union all
select 53620,'GMS' from dual union all
select 53623,'GMS' from dual union all
select 53633,'JMA' from dual union all
select 53634,'GMS' from dual union all
select 53641,'GMS' from dual union all
select 5365,'AN' from dual union all
select 53651,'WFF' from dual union all
select 53657,'JMA' from dual union all
select 53658,'GMS' from dual union all
select 5366,'AN' from dual union all
select 53680,'GMS' from dual union all
select 53693,'JMA' from dual union all
select 53694,'GMS' from dual union all
select 53715,'GMS' from dual union all
select 53719,'GMS' from dual union all
select 53723,'JMA' from dual union all
select 53724,'GMS' from dual union all
select 53727,'WFF' from dual union all
select 5374,'AFCRL' from dual union all
select 53753,'GMS' from dual union all
select 53759,'JMA' from dual union all
select 53770,'JMA' from dual union all
select 53771,'GMS' from dual union all
select 53788,'JMA' from dual union all
select 53796,'NASDA' from dual union all
select 5380,'SCEL' from dual union all
select 53801,'JMA' from dual union all
select 5381,'SAO' from dual union all
select 53816,'JMA' from dual union all
select 5382,'UMI' from dual union all
select 53829,'GMS' from dual union all
select 5383,'UMI' from dual union all
select 53831,'JMA' from dual union all
select 53838,'JMA' from dual union all
select 53839,'JMA' from dual union all
select 5384,'NRL' from dual union all
select 53847,'JMA' from dual union all
select 53848,'GMS' from dual union all
select 53849,'GMS' from dual union all
select 5385,'AFCRL' from dual union all
select 5386,'AFCRL' from dual union all
select 53866,'HOUS' from dual union all
select 5387,'NRL' from dual union all
select 53877,'HOUS' from dual union all
select 5388,'NRL' from dual union all
select 5389,'BRL' from dual union all
select 53897,'GMS' from dual union all
select 5390,'UMI' from dual union all
select 53904,'GMS' from dual union all
select 5395,'LARC' from dual union all
select 5396,'BIR' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 5396,'ICST' from dual union all
select 5396,'UCL' from dual union all
select 5398,'LARC' from dual union all
select 5400,'AFCRL' from dual union all
select 5401,'NRL' from dual union all
select 54027,'GMS' from dual union all
select 5405,'AFCRL' from dual union all
select 5406,'QUB' from dual union all
select 5407,'AFCRL' from dual union all
select 5408,'AFCRL' from dual union all
select 5409,'LARC' from dual union all
select 54096,'PSU' from dual union all
select 5411,'LARC' from dual union all
select 54191,'PRL' from dual union all
select 54192,'PRL' from dual union all
select 5420,'UCO' from dual union all
select 5421,'RAE' from dual union all
select 5421,'WRE' from dual union all
select 54226,'DLR' from dual union all
select 54227,'DLR' from dual union all
select 5425,'BIR' from dual union all
select 5425,'ICST' from dual union all
select 5425,'UCL' from dual union all
select 5426,'BIR' from dual union all
select 5426,'ICST' from dual union all
select 5426,'UCL' from dual union all
select 5429,'UMI' from dual union all
select 5430,'SAND' from dual union all
select 5431,'AN' from dual union all
select 5432,'UUT' from dual union all
select 5433,'UUT' from dual union all
select 5434,'BRL' from dual union all
select 5435,'NRL' from dual union all
select 5436,'NRL' from dual union all
select 5438,'UUT' from dual union all
select 5439,'BRL' from dual union all
select 5441,'LRL' from dual union all
select 5442,'USN' from dual union all
select 5443,'USN' from dual union all
select 54434,'JMA' from dual union all
select 54436,'JMA' from dual union all
select 54437,'NRCC' from dual union all
select 54438,'JMA' from dual union all
select 5444,'USN' from dual union all
select 54442,'JMA' from dual union all
select 5445,'USN' from dual union all
select 5446,'USN' from dual union all
select 54464,'UCO' from dual union all
select 5447,'USN' from dual union all
select 5448,'USN' from dual union all
select 54482,'ISRO' from dual union all
select 54483,'ISRO' from dual union all
select 54484,'ISRO' from dual union all
select 54485,'ISRO' from dual union all
select 54486,'GSFC' from dual union all
select 54487,'GSFC' from dual union all
select 54488,'ISRO' from dual union all
select 54489,'ISRO' from dual union all
select 5449,'USN' from dual union all
select 54490,'ISRO' from dual union all
select 54491,'ISRO' from dual union all
select 54492,'ISRO' from dual union all
select 54493,'ISRO' from dual union all
select 54494,'ISRO' from dual union all
select 54495,'ISRO' from dual union all
select 54496,'ISRO' from dual union all
select 54497,'ISRO' from dual union all
select 54498,'ISRO' from dual union all
select 54499,'ISRO' from dual union all
select 5450,'USN' from dual union all
select 54500,'ISRO' from dual union all
select 54501,'ISRO' from dual union all
select 54502,'ISRO' from dual union all
select 54503,'ISRO' from dual union all
select 54504,'ISRO' from dual union all
select 54505,'ISRO' from dual union all
select 54506,'ISRO' from dual union all
select 54507,'ISRO' from dual union all
select 54508,'ISRO' from dual union all
select 54509,'ISRO' from dual union all
select 5451,'USN' from dual union all
select 54510,'ISRO' from dual union all
select 54511,'ISRO' from dual union all
select 54512,'ISRO' from dual union all
select 54513,'ISRO' from dual union all
select 54514,'ISRO' from dual union all
select 54515,'ISRO' from dual union all
select 54516,'ISRO' from dual union all
select 54517,'ISRO' from dual union all
select 54518,'ISRO' from dual union all
select 54519,'ISRO' from dual union all
select 5452,'USN' from dual union all
select 54520,'ISRO' from dual union all
select 54521,'ISRO' from dual union all
select 54522,'ISRO' from dual union all
select 54523,'ISRO' from dual union all
select 54524,'ISRO' from dual union all
select 54525,'ISRO' from dual union all
select 54526,'ISRO' from dual union all
select 54527,'ISRO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 54528,'ISRO' from dual union all
select 54529,'ISRO' from dual union all
select 5453,'USN' from dual union all
select 54530,'ISRO' from dual union all
select 54531,'AFGL' from dual union all
select 54531,'PSRC' from dual union all
select 5454,'USN' from dual union all
select 54548,'GSFC' from dual union all
select 54549,'GSFC' from dual union all
select 5455,'USN' from dual union all
select 54550,'GSFC' from dual union all
select 54551,'GSFC' from dual union all
select 54552,'GSFC' from dual union all
select 54555,'DLR' from dual union all
select 54559,'DLR' from dual union all
select 5456,'SUI' from dual union all
select 54560,'UCO' from dual union all
select 54567,'AFCRL' from dual union all
select 5457,'SUI' from dual union all
select 54571,'UCB' from dual union all
select 54574,'LASL' from dual union all
select 54578,'AFCRL' from dual union all
select 54579,'INPE' from dual union all
select 5458,'SUI' from dual union all
select 54580,'LPARL' from dual union all
select 54580,'LPSP' from dual union all
select 54581,'UIUC' from dual union all
select 54585,'HAO' from dual union all
select 54585,'HCO' from dual union all
select 54587,'AFGL' from dual union all
select 54587,'UCL' from dual union all
select 54588,'GSFC' from dual union all
select 54589,'MPIK' from dual union all
select 5459,'SUI' from dual union all
select 54590,'BERN' from dual union all
select 54590,'UIO' from dual union all
select 54591,'BERN' from dual union all
select 54591,'UIO' from dual union all
select 54592,'STO' from dual union all
select 54594,'RAL' from dual union all
select 54596,'PITT' from dual union all
select 54597,'USCLA' from dual union all
select 54598,'STO' from dual union all
select 54599,'UCO' from dual union all
select 5460,'SUI' from dual union all
select 54600,'MPIK' from dual union all
select 54600,'RAL' from dual union all
select 54602,'NASDA' from dual union all
select 54603,'ISAS' from dual union all
select 54603,'TAO' from dual union all
select 54604,'AFCRL' from dual union all
select 54606,'AFCRL' from dual union all
select 54606,'INPE' from dual union all
select 54607,'SSI' from dual union all
select 54608,'ISAS' from dual union all
select 54609,'NAG' from dual union all
select 5461,'SUI' from dual union all
select 54610,'AFCRL' from dual union all
select 54611,'AFCRL' from dual union all
select 54611,'INPE' from dual union all
select 54612,'AFCRL' from dual union all
select 54613,'NRCC' from dual union all
select 54614,'INPE' from dual union all
select 54614,'MPE' from dual union all
select 54615,'INPE' from dual union all
select 54615,'MPE' from dual union all
select 54618,'ISAS' from dual union all
select 54619,'MPAE' from dual union all
select 54619,'NDRE' from dual union all
select 5462,'SUI' from dual union all
select 54627,'MPAE' from dual union all
select 54627,'NDRE' from dual union all
select 54628,'UCO' from dual union all
select 5463,'SUI' from dual union all
select 54631,'LUX' from dual union all
select 54631,'MIT' from dual union all
select 54632,'GSFC' from dual union all
select 54633,'UMN' from dual union all
select 54633,'UNH' from dual union all
select 54636,'NRL' from dual union all
select 54639,'SAIC' from dual union all
select 5464,'SUI' from dual union all
select 54641,'MPAE' from dual union all
select 54641,'NDRE' from dual union all
select 54643,'JHU' from dual union all
select 54644,'MPAE' from dual union all
select 54644,'NDRE' from dual union all
select 54647,'PRL' from dual union all
select 54648,'PRL' from dual union all
select 54649,'PRL' from dual union all
select 5465,'SUI' from dual union all
select 54650,'PRL' from dual union all
select 5466,'SUI' from dual union all
select 54723,'INTA' from dual union all
select 54724,'INTA' from dual union all
select 54725,'INTA' from dual union all
select 54726,'INTA' from dual union all
select 54727,'INTA' from dual union all
select 54728,'INTA' from dual union all
select 54729,'INTA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 54739,'ISAS' from dual union all
select 54741,'GSFC' from dual union all
select 54745,'UCB' from dual union all
select 54746,'AFCRL' from dual union all
select 54747,'GSFC' from dual union all
select 54748,'GSFC' from dual union all
select 54749,'GSFC' from dual union all
select 54751,'UMN' from dual union all
select 54759,'ISAS' from dual union all
select 54760,'TOK' from dual union all
select 54761,'ISAS' from dual union all
select 54762,'PITT' from dual union all
select 54766,'PITT' from dual union all
select 54777,'AERO' from dual union all
select 54782,'NRL' from dual union all
select 54783,'IAA' from dual union all
select 54785,'UAF' from dual union all
select 54791,'QUB' from dual union all
select 54792,'QUB' from dual union all
select 54793,'QUB' from dual union all
select 54794,'QUB' from dual union all
select 54795,'QUB' from dual union all
select 54796,'QUB' from dual union all
select 54797,'QUB' from dual union all
select 54798,'UAF' from dual union all
select 54804,'GMS' from dual union all
select 54809,'JMA' from dual union all
select 54810,'GMS' from dual union all
select 54823,'GMS' from dual union all
select 54829,'WFF' from dual union all
select 54830,'WFF' from dual union all
select 54835,'GMS' from dual union all
select 54837,'JMA' from dual union all
select 54838,'GMS' from dual union all
select 54840,'WFF' from dual union all
select 54845,'GMS' from dual union all
select 54851,'GMS' from dual union all
select 54852,'WFF' from dual union all
select 54853,'GMS' from dual union all
select 54858,'WFF' from dual union all
select 54859,'GMS' from dual union all
select 54864,'WFF' from dual union all
select 54867,'GMS' from dual union all
select 54869,'WFF' from dual union all
select 54870,'WFF' from dual union all
select 54871,'GMS' from dual union all
select 54872,'WFF' from dual union all
select 54877,'WFF' from dual union all
select 54882,'WFF' from dual union all
select 54884,'WFF' from dual union all
select 54887,'WFF' from dual union all
select 54891,'WFF' from dual union all
select 54893,'WFF' from dual union all
select 54895,'WFF' from dual union all
select 54896,'WFF' from dual union all
select 54897,'WFF' from dual union all
select 54898,'WFF' from dual union all
select 54902,'WFF' from dual union all
select 54904,'WFF' from dual union all
select 54906,'WFF' from dual union all
select 54907,'WFF' from dual union all
select 54908,'WFF' from dual union all
select 54911,'WFF' from dual union all
select 54915,'WFF' from dual union all
select 54918,'WFF' from dual union all
select 54921,'JMA' from dual union all
select 54922,'JMA' from dual union all
select 54927,'WFF' from dual union all
select 54932,'GMS' from dual union all
select 54941,'WFF' from dual union all
select 54942,'WFF' from dual union all
select 54949,'WFF' from dual union all
select 54961,'JMA' from dual union all
select 54962,'GMS' from dual union all
select 54966,'GMS' from dual union all
select 54969,'JMA' from dual union all
select 54975,'GMS' from dual union all
select 54980,'WFF' from dual union all
select 54981,'WFF' from dual union all
select 54982,'WFF' from dual union all
select 54983,'WFF' from dual union all
select 54985,'JMA' from dual union all
select 54986,'GMS' from dual union all
select 54992,'GMS' from dual union all
select 55000,'GMS' from dual union all
select 55003,'GMS' from dual union all
select 55004,'GMS' from dual union all
select 55006,'GMS' from dual union all
select 55010,'GMS' from dual union all
select 55012,'GMS' from dual union all
select 55015,'JMA' from dual union all
select 55016,'GMS' from dual union all
select 55019,'GMS' from dual union all
select 55020,'GMS' from dual union all
select 55022,'GMS' from dual union all
select 55028,'GMS' from dual union all
select 55030,'GMS' from dual union all
select 55031,'WFF' from dual union all
select 55032,'WFF' from dual union all
select 55034,'JMA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 55035,'GMS' from dual union all
select 55042,'WFF' from dual union all
select 55046,'JMA' from dual union all
select 55053,'WFF' from dual union all
select 55054,'WFF' from dual union all
select 55056,'JMA' from dual union all
select 55066,'JMA' from dual union all
select 55075,'WFF' from dual union all
select 55083,'WFF' from dual union all
select 55092,'JMA' from dual union all
select 55110,'JMA' from dual union all
select 55117,'JMA' from dual union all
select 55124,'JMA' from dual union all
select 55137,'JMA' from dual union all
select 55141,'WFF' from dual union all
select 55142,'WFF' from dual union all
select 55143,'NASDA' from dual union all
select 55145,'JMA' from dual union all
select 55147,'WFF' from dual union all
select 55148,'JMA' from dual union all
select 55150,'WFF' from dual union all
select 55152,'WFF' from dual union all
select 55161,'JMA' from dual union all
select 55162,'WFF' from dual union all
select 55171,'JMA' from dual union all
select 55174,'WFF' from dual union all
select 55180,'JMA' from dual union all
select 55182,'WFF' from dual union all
select 55183,'JMA' from dual union all
select 55188,'GMS' from dual union all
select 55191,'GMS' from dual union all
select 55194,'GMS' from dual union all
select 55195,'JMA' from dual union all
select 55197,'GMS' from dual union all
select 55203,'GMS' from dual union all
select 55208,'GMS' from dual union all
select 55211,'WFF' from dual union all
select 55213,'WFF' from dual union all
select 55218,'JMA' from dual union all
select 55222,'JMA' from dual union all
select 55223,'GMS' from dual union all
select 55225,'GMS' from dual union all
select 55226,'GMS' from dual union all
select 55227,'WFF' from dual union all
select 55231,'GMS' from dual union all
select 55233,'GMS' from dual union all
select 55235,'GMS' from dual union all
select 55238,'JMA' from dual union all
select 55239,'WFF' from dual union all
select 55240,'GMS' from dual union all
select 55241,'GMS' from dual union all
select 55242,'WFF' from dual union all
select 55243,'GMS' from dual union all
select 55245,'JMA' from dual union all
select 55246,'GMS' from dual union all
select 55252,'JMA' from dual union all
select 55257,'GMS' from dual union all
select 55258,'WFF' from dual union all
select 55261,'GMS' from dual union all
select 55263,'JMA' from dual union all
select 55264,'GMS' from dual union all
select 55265,'GMS' from dual union all
select 55268,'JMA' from dual union all
select 55269,'AFGL' from dual union all
select 55270,'AFGL' from dual union all
select 55271,'WFF' from dual union all
select 55272,'WFF' from dual union all
select 55273,'WFF' from dual union all
select 55274,'WFF' from dual union all
select 55275,'WFF' from dual union all
select 55276,'GSFC' from dual union all
select 55277,'WFF' from dual union all
select 55278,'GSFC' from dual union all
select 55283,'JMA' from dual union all
select 55284,'GMS' from dual union all
select 55285,'GMS' from dual union all
select 55292,'GMS' from dual union all
select 55299,'GSFC' from dual union all
select 55300,'GSFC' from dual union all
select 55301,'WFF' from dual union all
select 55305,'NASDA' from dual union all
select 55307,'NASDA' from dual union all
select 55308,'JMA' from dual union all
select 55351,'WFF' from dual union all
select 5539,'LRL' from dual union all
select 55448,'PSU' from dual union all
select 55453,'WFF' from dual union all
select 55456,'WFF' from dual union all
select 55478,'WFF' from dual union all
select 55479,'WFF' from dual union all
select 55492,'PSU' from dual union all
select 5551,'RAE' from dual union all
select 55600,'PSU' from dual union all
select 55631,'WFF' from dual union all
select 55632,'WFF' from dual union all
select 55633,'WFF' from dual union all
select 5564,'RAE' from dual union all
select 5565,'LARC' from dual union all
select 55650,'WFF' from dual union all
select 55651,'WFF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 55652,'WFF' from dual union all
select 5568,'NRL' from dual union all
select 55684,'WFF' from dual union all
select 55685,'WFF' from dual union all
select 55702,'VSSC' from dual union all
select 55717,'WFF' from dual union all
select 55718,'WFF' from dual union all
select 55719,'WFF' from dual union all
select 5572,'BRL' from dual union all
select 55720,'WFF' from dual union all
select 55724,'WFF' from dual union all
select 55728,'WFF' from dual union all
select 5573,'BRL' from dual union all
select 55730,'WFF' from dual union all
select 55741,'HOUS' from dual union all
select 5584,'GSFC' from dual union all
select 5586,'NOTS' from dual union all
select 55910,'GSFC' from dual union all
select 55912,'WFF' from dual union all
select 5592,'GCA' from dual union all
select 55941,'DLR' from dual union all
select 55942,'AFCRL' from dual union all
select 55943,'BONN' from dual union all
select 55943,'NDRE' from dual union all
select 55944,'ISRO' from dual union all
select 55945,'ISRO' from dual union all
select 55946,'ISRO' from dual union all
select 55947,'ISRO' from dual union all
select 55948,'ISRO' from dual union all
select 55949,'ISRO' from dual union all
select 55950,'ISRO' from dual union all
select 55951,'ISRO' from dual union all
select 55952,'ISRO' from dual union all
select 55953,'ISRO' from dual union all
select 55954,'ISRO' from dual union all
select 55955,'ISRO' from dual union all
select 55956,'ISRO' from dual union all
select 55957,'ISRO' from dual union all
select 55958,'ISRO' from dual union all
select 55959,'ISRO' from dual union all
select 55960,'ISRO' from dual union all
select 55961,'ISRO' from dual union all
select 55962,'ISRO' from dual union all
select 55963,'ISRO' from dual union all
select 55964,'ISRO' from dual union all
select 55965,'ISRO' from dual union all
select 55966,'ISRO' from dual union all
select 55967,'ISRO' from dual union all
select 55968,'ISRO' from dual union all
select 55969,'ISRO' from dual union all
select 55970,'ISRO' from dual union all
select 55971,'ISRO' from dual union all
select 55972,'ISRO' from dual union all
select 55973,'ISRO' from dual union all
select 55974,'ISRO' from dual union all
select 55975,'ISRO' from dual union all
select 55976,'ISRO' from dual union all
select 55977,'ISRO' from dual union all
select 55978,'ISRO' from dual union all
select 55979,'ISRO' from dual union all
select 55980,'ISRO' from dual union all
select 55981,'ISRO' from dual union all
select 55982,'ISRO' from dual union all
select 55983,'ISRO' from dual union all
select 55984,'ISRO' from dual union all
select 55985,'ISRO' from dual union all
select 55986,'ISRO' from dual union all
select 55987,'ISRO' from dual union all
select 55988,'ISRO' from dual union all
select 55989,'ISRO' from dual union all
select 55990,'ISRO' from dual union all
select 55991,'ISRO' from dual union all
select 55992,'ISRO' from dual union all
select 55993,'ISRO' from dual union all
select 55994,'ISRO' from dual union all
select 55995,'ISRO' from dual union all
select 55996,'ISRO' from dual union all
select 55997,'ISRO' from dual union all
select 55998,'GSFC' from dual union all
select 55999,'GSFC' from dual union all
select 56000,'GSFC' from dual union all
select 56001,'GSFC' from dual union all
select 56002,'GSFC' from dual union all
select 56003,'GSFC' from dual union all
select 56004,'GSFC' from dual union all
select 56005,'GSFC' from dual union all
select 56006,'GSFC' from dual union all
select 56007,'GSFC' from dual union all
select 56008,'GSFC' from dual union all
select 56009,'GSFC' from dual union all
select 56010,'GSFC' from dual union all
select 56011,'ISRO' from dual union all
select 56012,'ISRO' from dual union all
select 56013,'ISRO' from dual union all
select 56014,'ISRO' from dual union all
select 56015,'ISRO' from dual union all
select 56016,'ISRO' from dual union all
select 56017,'ISRO' from dual union all
select 56018,'ISRO' from dual union all
select 56019,'ISRO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 56020,'ISRO' from dual union all
select 56021,'ISRO' from dual union all
select 56022,'ISRO' from dual union all
select 56023,'ISRO' from dual union all
select 56024,'ISRO' from dual union all
select 56025,'ISRO' from dual union all
select 56026,'ISRO' from dual union all
select 56027,'ISRO' from dual union all
select 56028,'ISRO' from dual union all
select 56029,'ISRO' from dual union all
select 56030,'ISRO' from dual union all
select 56031,'ISRO' from dual union all
select 56032,'ISRO' from dual union all
select 56043,'CAL' from dual union all
select 56051,'INPE' from dual union all
select 56055,'NRCC' from dual union all
select 56058,'AFCRL' from dual union all
select 56060,'MSFC' from dual union all
select 56062,'PITT' from dual union all
select 56068,'STAN' from dual union all
select 56071,'GSFC' from dual union all
select 56073,'GSFC' from dual union all
select 56076,'UCO' from dual union all
select 56092,'USU' from dual union all
select 56093,'ISAS' from dual union all
select 56093,'USU' from dual union all
select 56094,'USCLA' from dual union all
select 56095,'NASDA' from dual union all
select 56098,'ISAS' from dual union all
select 56098,'KOBE' from dual union all
select 56099,'JHU' from dual union all
select 56100,'PITT' from dual union all
select 56109,'PITT' from dual union all
select 56111,'ISAS' from dual union all
select 56112,'ISAS' from dual union all
select 56114,'GSFC' from dual union all
select 56117,'UMI' from dual union all
select 5613,'RCAF' from dual union all
select 56135,'UCO' from dual union all
select 56136,'UCO' from dual union all
select 56146,'AN' from dual union all
select 56147,'AFCRL' from dual union all
select 56148,'AFCRL' from dual union all
select 5615,'LARC' from dual union all
select 56152,'AFCRL' from dual union all
select 56153,'BERG' from dual union all
select 56153,'GSFC' from dual union all
select 56154,'GSFC' from dual union all
select 56155,'GSFC' from dual union all
select 56156,'GSFC' from dual union all
select 56160,'UCB' from dual union all
select 56161,'JPL' from dual union all
select 56163,'USU' from dual union all
select 56163,'WFF' from dual union all
select 56166,'MITLL' from dual union all
select 56170,'WFF' from dual union all
select 56171,'WFF' from dual union all
select 56172,'WFF' from dual union all
select 5618,'LMSC' from dual union all
select 56228,'INTA' from dual union all
select 56229,'INTA' from dual union all
select 56230,'INTA' from dual union all
select 56231,'INTA' from dual union all
select 56232,'INTA' from dual union all
select 56233,'INTA' from dual union all
select 56234,'INTA' from dual union all
select 56235,'INTA' from dual union all
select 56236,'INTA' from dual union all
select 56237,'INTA' from dual union all
select 56238,'INTA' from dual union all
select 56239,'INTA' from dual union all
select 56240,'INTA' from dual union all
select 56241,'INTA' from dual union all
select 56242,'INTA' from dual union all
select 56243,'INTA' from dual union all
select 56245,'INTA' from dual union all
select 5625,'WSMR' from dual union all
select 56265,'ISRO' from dual union all
select 56279,'UCO' from dual union all
select 56280,'UMI' from dual union all
select 56281,'UMI' from dual union all
select 56282,'RIKEN' from dual union all
select 56283,'KGI' from dual union all
select 56283,'UIO' from dual union all
select 56285,'SAIC' from dual union all
select 56286,'NASDA' from dual union all
select 56290,'NRCC' from dual union all
select 56294,'UCO' from dual union all
select 56297,'UIUC' from dual union all
select 56299,'AFCRL' from dual union all
select 56300,'CORN' from dual union all
select 56300,'UIUC' from dual union all
select 56301,'CORN' from dual union all
select 56302,'NRL' from dual union all
select 56303,'GSFC' from dual union all
select 56304,'GSFC' from dual union all
select 56305,'GSFC' from dual union all
select 56306,'GSFC' from dual union all
select 56308,'CORN' from dual union all
select 56309,'UIUC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 56311,'CORN' from dual union all
select 56311,'UIUC' from dual union all
select 56313,'AFCRL' from dual union all
select 56316,'CORN' from dual union all
select 56316,'UAF' from dual union all
select 56317,'CORN' from dual union all
select 56317,'UAF' from dual union all
select 56320,'UMN' from dual union all
select 56322,'UMD' from dual union all
select 56324,'AFCRL' from dual union all
select 56324,'UUT' from dual union all
select 56325,'AFCRL' from dual union all
select 56327,'MPIK' from dual union all
select 5633,'RAE' from dual union all
select 56331,'DLR' from dual union all
select 56333,'NRCC' from dual union all
select 56334,'DLR' from dual union all
select 56335,'WFF' from dual union all
select 56341,'JMA' from dual union all
select 56343,'WFF' from dual union all
select 56344,'WFF' from dual union all
select 56347,'WFF' from dual union all
select 56351,'JMA' from dual union all
select 56352,'WFF' from dual union all
select 56353,'WFF' from dual union all
select 5636,'UUT' from dual union all
select 56367,'JMA' from dual union all
select 56370,'WFF' from dual union all
select 56372,'JMA' from dual union all
select 56379,'JMA' from dual union all
select 56381,'WFF' from dual union all
select 56382,'GMS' from dual union all
select 56384,'GMS' from dual union all
select 56388,'JMA' from dual union all
select 56390,'GMS' from dual union all
select 56394,'WFF' from dual union all
select 56395,'JMA' from dual union all
select 56396,'GMS' from dual union all
select 56400,'GMS' from dual union all
select 56403,'GMS' from dual union all
select 56406,'GMS' from dual union all
select 56408,'WFF' from dual union all
select 56409,'JMA' from dual union all
select 56411,'GMS' from dual union all
select 56413,'GMS' from dual union all
select 56415,'WFF' from dual union all
select 56416,'GMS' from dual union all
select 56418,'GMS' from dual union all
select 56421,'WFF' from dual union all
select 56423,'JMA' from dual union all
select 56425,'GMS' from dual union all
select 56426,'JMA' from dual union all
select 56429,'WFF' from dual union all
select 56431,'GMS' from dual union all
select 56434,'GMS' from dual union all
select 56435,'GMS' from dual union all
select 56437,'WFF' from dual union all
select 56439,'WFF' from dual union all
select 56440,'JMA' from dual union all
select 56446,'WFF' from dual union all
select 56456,'WFF' from dual union all
select 56458,'WFF' from dual union all
select 56459,'JMA' from dual union all
select 56462,'WFF' from dual union all
select 56465,'JMA' from dual union all
select 56466,'WFF' from dual union all
select 56467,'WFF' from dual union all
select 56469,'JMA' from dual union all
select 56471,'WFF' from dual union all
select 56475,'WFF' from dual union all
select 56479,'JMA' from dual union all
select 5648,'BRL' from dual union all
select 5648,'UMI' from dual union all
select 56481,'WFF' from dual union all
select 56482,'WFF' from dual union all
select 56484,'GMS' from dual union all
select 56488,'GMS' from dual union all
select 5649,'UMI' from dual union all
select 56490,'JMA' from dual union all
select 56491,'GMS' from dual union all
select 56492,'WFF' from dual union all
select 56494,'GMS' from dual union all
select 56497,'GMS' from dual union all
select 56500,'GMS' from dual union all
select 56502,'JMA' from dual union all
select 56503,'GMS' from dual union all
select 56504,'GMS' from dual union all
select 56508,'WFF' from dual union all
select 56515,'WFF' from dual union all
select 56520,'JMA' from dual union all
select 56522,'JMA' from dual union all
select 56527,'JMA' from dual union all
select 56531,'WFF' from dual union all
select 56532,'WFF' from dual union all
select 56536,'WFF' from dual union all
select 56537,'WFF' from dual union all
select 56539,'WFF' from dual union all
select 56542,'GMS' from dual union all
select 56544,'GSFC' from dual union all
select 56549,'WFF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 56554,'JMA' from dual union all
select 56556,'WFF' from dual union all
select 56558,'JMA' from dual union all
select 56563,'JMA' from dual union all
select 56567,'WFF' from dual union all
select 56569,'WFF' from dual union all
select 56571,'JMA' from dual union all
select 56581,'WFF' from dual union all
select 56582,'WFF' from dual union all
select 56583,'JMA' from dual union all
select 56586,'GMS' from dual union all
select 56591,'WFF' from dual union all
select 56592,'WFF' from dual union all
select 56597,'GMS' from dual union all
select 56598,'JMA' from dual union all
select 56601,'BONN' from dual union all
select 56604,'BONN' from dual union all
select 56605,'BONN' from dual union all
select 56607,'MPAE' from dual union all
select 56609,'MPAE' from dual union all
select 56612,'JMA' from dual union all
select 56613,'WFF' from dual union all
select 56615,'MPAE' from dual union all
select 56616,'MPAE' from dual union all
select 56617,'BONN' from dual union all
select 56619,'WFF' from dual union all
select 56622,'WFF' from dual union all
select 56630,'JMA' from dual union all
select 56632,'WFF' from dual union all
select 56633,'BONN' from dual union all
select 56635,'JMA' from dual union all
select 56638,'BONN' from dual union all
select 56641,'GMS' from dual union all
select 56644,'JMA' from dual union all
select 56645,'WFF' from dual union all
select 56646,'BONN' from dual union all
select 56647,'BONN' from dual union all
select 56651,'JMA' from dual union all
select 56653,'BONN' from dual union all
select 56654,'BONN' from dual union all
select 56655,'BONN' from dual union all
select 56656,'BONN' from dual union all
select 56657,'JMA' from dual union all
select 56659,'GMS' from dual union all
select 56661,'GMS' from dual union all
select 56664,'JMA' from dual union all
select 56665,'GMS' from dual union all
select 56667,'WFF' from dual union all
select 56671,'GMS' from dual union all
select 56674,'GMS' from dual union all
select 56679,'WFF' from dual union all
select 5668,'SAND' from dual union all
select 56680,'GMS' from dual union all
select 56681,'GMS' from dual union all
select 56683,'WFF' from dual union all
select 56684,'JMA' from dual union all
select 56685,'WFF' from dual union all
select 56686,'GMS' from dual union all
select 56689,'GMS' from dual union all
select 56690,'WFF' from dual union all
select 56694,'GMS' from dual union all
select 56695,'WFF' from dual union all
select 56699,'JMA' from dual union all
select 5670,'PMR' from dual union all
select 56700,'GMS' from dual union all
select 56703,'GMS' from dual union all
select 56708,'WFF' from dual union all
select 56709,'JMA' from dual union all
select 5671,'PMR' from dual union all
select 56710,'GMS' from dual union all
select 56712,'WFF' from dual union all
select 56715,'GMS' from dual union all
select 56716,'WFF' from dual union all
select 5672,'WADD' from dual union all
select 5673,'WADD' from dual union all
select 56738,'PRL' from dual union all
select 5674,'WADD' from dual union all
select 56740,'PRL' from dual union all
select 56741,'PRL' from dual union all
select 56742,'PRL' from dual union all
select 56746,'PRL' from dual union all
select 56747,'PRL' from dual union all
select 56748,'PRL' from dual union all
select 56749,'PRL' from dual union all
select 5675,'WADD' from dual union all
select 56751,'PRL' from dual union all
select 56752,'PRL' from dual union all
select 56753,'PRL' from dual union all
select 56755,'PRL' from dual union all
select 56759,'PRL' from dual union all
select 5676,'WADD' from dual union all
select 56760,'PRL' from dual union all
select 56762,'PRL' from dual union all
select 56771,'WFF' from dual union all
select 56773,'WFF' from dual union all
select 5679,'RCAF' from dual union all
select 56829,'AFCRL' from dual union all
select 56833,'AFCRL' from dual union all
select 56836,'AFCRL' from dual union all
select 56848,'WFF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 56850,'WFF' from dual union all
select 56921,'WFF' from dual union all
select 56937,'HOUS' from dual union all
select 56940,'WFF' from dual union all
select 56941,'WFF' from dual union all
select 56945,'BONN' from dual union all
select 56947,'BONN' from dual union all
select 56948,'HOUS' from dual union all
select 56949,'BONN' from dual union all
select 56950,'BONN' from dual union all
select 56952,'WFF' from dual union all
select 56953,'BONN' from dual union all
select 56956,'BONN' from dual union all
select 56957,'BONN' from dual union all
select 56961,'BONN' from dual union all
select 56962,'BONN' from dual union all
select 56963,'BONN' from dual union all
select 56964,'BONN' from dual union all
select 56965,'BONN' from dual union all
select 56967,'BONN' from dual union all
select 56968,'BONN' from dual union all
select 56969,'BONN' from dual union all
select 56970,'WFF' from dual union all
select 56971,'BONN' from dual union all
select 56972,'BONN' from dual union all
select 56973,'BONN' from dual union all
select 56974,'BONN' from dual union all
select 56994,'WFF' from dual union all
select 5700,'USA' from dual union all
select 57002,'WFF' from dual union all
select 5701,'RCAF' from dual union all
select 5702,'USA' from dual union all
select 57024,'WFF' from dual union all
select 57025,'PSU' from dual union all
select 57028,'PSU' from dual union all
select 57029,'PSU' from dual union all
select 57030,'PSU' from dual union all
select 57032,'WFF' from dual union all
select 57033,'WFF' from dual union all
select 57034,'WFF' from dual union all
select 57035,'WFF' from dual union all
select 57036,'WFF' from dual union all
select 57037,'WFF' from dual union all
select 57038,'WFF' from dual union all
select 57039,'PSU' from dual union all
select 57040,'PSU' from dual union all
select 5709,'NMSU' from dual union all
select 5710,'NMSU' from dual union all
select 5711,'NMSU' from dual union all
select 5712,'RCAF' from dual union all
select 5713,'NMSU' from dual union all
select 5714,'NMSU' from dual union all
select 5717,'WRE' from dual union all
select 5718,'WRE' from dual union all
select 5719,'WRE' from dual union all
select 57199,'JMA' from dual union all
select 5721,'RCAF' from dual union all
select 57211,'STST' from dual union all
select 57223,'SAND' from dual union all
select 57224,'ISRO' from dual union all
select 57225,'ISAS' from dual union all
select 57226,'PITT' from dual union all
select 57227,'MPIK' from dual union all
select 57228,'ISRO' from dual union all
select 57229,'ISRO' from dual union all
select 57230,'ISRO' from dual union all
select 57231,'ISRO' from dual union all
select 57232,'ISRO' from dual union all
select 57233,'ISRO' from dual union all
select 57234,'ISRO' from dual union all
select 57235,'ISRO' from dual union all
select 57236,'ISRO' from dual union all
select 57237,'ISRO' from dual union all
select 57238,'ISRO' from dual union all
select 57239,'ISRO' from dual union all
select 57240,'ISRO' from dual union all
select 57241,'ISRO' from dual union all
select 57242,'ISRO' from dual union all
select 57243,'ISRO' from dual union all
select 57244,'ISRO' from dual union all
select 57245,'ISRO' from dual union all
select 57246,'ISRO' from dual union all
select 57247,'ISRO' from dual union all
select 57248,'ISRO' from dual union all
select 57249,'ISRO' from dual union all
select 5725,'RCAF' from dual union all
select 57250,'ISRO' from dual union all
select 57251,'ISRO' from dual union all
select 57252,'ISRO' from dual union all
select 57253,'ISRO' from dual union all
select 57254,'ISRO' from dual union all
select 57255,'ISRO' from dual union all
select 57256,'ISRO' from dual union all
select 57257,'ISRO' from dual union all
select 57258,'ISRO' from dual union all
select 57259,'ISRO' from dual union all
select 57260,'ISRO' from dual union all
select 57261,'ISRO' from dual union all
select 57262,'ISRO' from dual union all
select 57263,'ISRO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 57264,'ISRO' from dual union all
select 57265,'ISRO' from dual union all
select 57266,'ISRO' from dual union all
select 57267,'ISRO' from dual union all
select 57268,'ISRO' from dual union all
select 57269,'ISRO' from dual union all
select 57270,'ISRO' from dual union all
select 57271,'ISRO' from dual union all
select 57272,'ISRO' from dual union all
select 57273,'ISRO' from dual union all
select 57274,'ISRO' from dual union all
select 57275,'ISRO' from dual union all
select 57276,'ISRO' from dual union all
select 57277,'ISRO' from dual union all
select 57278,'ISRO' from dual union all
select 57279,'ISRO' from dual union all
select 57280,'ISRO' from dual union all
select 57281,'ISRO' from dual union all
select 57282,'ISRO' from dual union all
select 57283,'ISRO' from dual union all
select 57284,'ISRO' from dual union all
select 57285,'ISRO' from dual union all
select 57286,'ISRO' from dual union all
select 57287,'ISRO' from dual union all
select 57288,'ISRO' from dual union all
select 57289,'INPE' from dual union all
select 57289,'WFF' from dual union all
select 57290,'INPE' from dual union all
select 57290,'WFF' from dual union all
select 57291,'INPE' from dual union all
select 57291,'WFF' from dual union all
select 57292,'INPE' from dual union all
select 57292,'WFF' from dual union all
select 57293,'INPE' from dual union all
select 57293,'WFF' from dual union all
select 57294,'INPE' from dual union all
select 57294,'WFF' from dual union all
select 57295,'ISRO' from dual union all
select 57296,'ISRO' from dual union all
select 57297,'ISRO' from dual union all
select 57298,'ISRO' from dual union all
select 57299,'ISRO' from dual union all
select 57300,'ISRO' from dual union all
select 57301,'ISRO' from dual union all
select 57302,'ISRO' from dual union all
select 57303,'ISRO' from dual union all
select 57304,'ISRO' from dual union all
select 57305,'GSFC' from dual union all
select 57306,'GSFC' from dual union all
select 57307,'GSFC' from dual union all
select 57308,'GSFC' from dual union all
select 57309,'GSFC' from dual union all
select 57310,'GSFC' from dual union all
select 57311,'GSFC' from dual union all
select 57312,'GSFC' from dual union all
select 57313,'GSFC' from dual union all
select 57314,'GSFC' from dual union all
select 57315,'INPE' from dual union all
select 57315,'WFF' from dual union all
select 57316,'ISRO' from dual union all
select 57317,'ISRO' from dual union all
select 57318,'ISRO' from dual union all
select 57319,'ISRO' from dual union all
select 57320,'ISRO' from dual union all
select 57321,'ISRO' from dual union all
select 57322,'ISRO' from dual union all
select 57323,'ISRO' from dual union all
select 57324,'ISRO' from dual union all
select 57325,'ISRO' from dual union all
select 57326,'ISRO' from dual union all
select 57327,'ISRO' from dual union all
select 57330,'MISU' from dual union all
select 57330,'RAL' from dual union all
select 57330,'UIO' from dual union all
select 57331,'MISU' from dual union all
select 57331,'RAL' from dual union all
select 57331,'UIO' from dual union all
select 57339,'GSFC' from dual union all
select 57341,'INPE' from dual union all
select 57342,'BU' from dual union all
select 57343,'CORN' from dual union all
select 57345,'UMI' from dual union all
select 57346,'GSFC' from dual union all
select 57347,'PSU' from dual union all
select 5735,'RCAF' from dual union all
select 57354,'GSFC' from dual union all
select 57356,'UCO' from dual union all
select 57362,'PRIN' from dual union all
select 57365,'UMI' from dual union all
select 57366,'UIO' from dual union all
select 57367,'UIUC' from dual union all
select 5737,'ICST' from dual union all
select 5737,'QUB' from dual union all
select 5737,'UCL' from dual union all
select 57372,'INPE' from dual union all
select 57374,'GSFC' from dual union all
select 57378,'INTA' from dual union all
select 57379,'INTA' from dual union all
select 5738,'CNRS' from dual union all
select 57380,'INTA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 57385,'TSAO' from dual union all
select 57386,'TSAO' from dual union all
select 57387,'TSAO' from dual union all
select 57388,'TSAO' from dual union all
select 5742,'LARC' from dual union all
select 57486,'INTA' from dual union all
select 57487,'INTA' from dual union all
select 57488,'INTA' from dual union all
select 57489,'INTA' from dual union all
select 5749,'LARC' from dual union all
select 57490,'INTA' from dual union all
select 57491,'INTA' from dual union all
select 57492,'INTA' from dual union all
select 57493,'INTA' from dual union all
select 57494,'INTA' from dual union all
select 5751,'WSMR' from dual union all
select 57543,'BONN' from dual union all
select 57543,'NDRE' from dual union all
select 57544,'NAG' from dual union all
select 57545,'BONN' from dual union all
select 57545,'NDRE' from dual union all
select 57546,'UCO' from dual union all
select 57550,'BONN' from dual union all
select 57550,'NDRE' from dual union all
select 57551,'MITLL' from dual union all
select 57552,'MPAE' from dual union all
select 57552,'UIO' from dual union all
select 57553,'BONN' from dual union all
select 57553,'NDRE' from dual union all
select 57553,'WUPP' from dual union all
select 57554,'OSLO' from dual union all
select 57554,'UIO' from dual union all
select 57555,'MITLL' from dual union all
select 57556,'UCB' from dual union all
select 57557,'UCB' from dual union all
select 57558,'BONN' from dual union all
select 57558,'NDRE' from dual union all
select 57558,'WUPP' from dual union all
select 57559,'OSLO' from dual union all
select 57559,'UIO' from dual union all
select 57560,'WUPP' from dual union all
select 57561,'BONN' from dual union all
select 57561,'NDRE' from dual union all
select 57562,'BONN' from dual union all
select 57562,'NDRE' from dual union all
select 57563,'PITT' from dual union all
select 57565,'GSFC' from dual union all
select 57566,'UMI' from dual union all
select 57567,'NRCC' from dual union all
select 57568,'UCO' from dual union all
select 57570,'NRCC' from dual union all
select 57571,'JHU' from dual union all
select 57574,'GSFC' from dual union all
select 57577,'GSFC' from dual union all
select 57578,'CORN' from dual union all
select 57578,'UAF' from dual union all
select 57579,'UAF' from dual union all
select 57580,'NIPR' from dual union all
select 57586,'DLR' from dual union all
select 57587,'NIPR' from dual union all
select 57588,'UWI' from dual union all
select 57589,'GSFC' from dual union all
select 57590,'AES' from dual union all
select 57591,'DLR' from dual union all
select 57593,'NIPR' from dual union all
select 57597,'UCO' from dual union all
select 57598,'UCO' from dual union all
select 57603,'BONN' from dual union all
select 57604,'BONN' from dual union all
select 57607,'BONN' from dual union all
select 57608,'BONN' from dual union all
select 57610,'WFF' from dual union all
select 57613,'JMA' from dual union all
select 57617,'JMA' from dual union all
select 57620,'WFF' from dual union all
select 57622,'JMA' from dual union all
select 57634,'JMA' from dual union all
select 57636,'WFF' from dual union all
select 57655,'BONN' from dual union all
select 57658,'JMA' from dual union all
select 57665,'WFF' from dual union all
select 57666,'BONN' from dual union all
select 5767,'NOTS' from dual union all
select 57677,'BONN' from dual union all
select 57680,'JMA' from dual union all
select 57684,'WFF' from dual union all
select 57688,'BONN' from dual union all
select 5769,'GSFC' from dual union all
select 57694,'WFF' from dual union all
select 57696,'JMA' from dual union all
select 57699,'BONN' from dual union all
select 57701,'JMA' from dual union all
select 57706,'JMA' from dual union all
select 57711,'JMA' from dual union all
select 57718,'WFF' from dual union all
select 57722,'JMA' from dual union all
select 57728,'WFF' from dual union all
select 57729,'JMA' from dual union all
select 5773,'LARC' from dual union all
select 57743,'WFF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 57745,'JMA' from dual union all
select 57758,'JMA' from dual union all
select 57768,'JMA' from dual union all
select 57776,'BONN' from dual union all
select 57787,'JMA' from dual union all
select 57790,'GSFC' from dual union all
select 57796,'JMA' from dual union all
select 57809,'BONN' from dual union all
select 57818,'WFF' from dual union all
select 57824,'JMA' from dual union all
select 57828,'WFF' from dual union all
select 57836,'JMA' from dual union all
select 57843,'WFF' from dual union all
select 57863,'JMA' from dual union all
select 57872,'WFF' from dual union all
select 57875,'BONN' from dual union all
select 57883,'JMA' from dual union all
select 57886,'JMA' from dual union all
select 57888,'JMA' from dual union all
select 57892,'WFF' from dual union all
select 57898,'JMA' from dual union all
select 57907,'JMA' from dual union all
select 57908,'BONN' from dual union all
select 57926,'JMA' from dual union all
select 57948,'JMA' from dual union all
select 57963,'BONN' from dual union all
select 57964,'JMA' from dual union all
select 57972,'JMA' from dual union all
select 57974,'BONN' from dual union all
select 57983,'JMA' from dual union all
select 57993,'JMA' from dual union all
select 5800,'FRC' from dual union all
select 58000,'BONN' from dual union all
select 58003,'JMA' from dual union all
select 58005,'BONN' from dual union all
select 5801,'FRC' from dual union all
select 5801,'NOTS' from dual union all
select 58010,'BONN' from dual union all
select 58017,'WFF' from dual union all
select 58020,'BONN' from dual union all
select 58022,'BONN' from dual union all
select 58024,'JMA' from dual union all
select 58026,'BONN' from dual union all
select 58029,'BONN' from dual union all
select 58030,'BONN' from dual union all
select 58031,'BONN' from dual union all
select 58034,'BONN' from dual union all
select 58036,'JMA' from dual union all
select 58048,'WFF' from dual union all
select 58049,'BONN' from dual union all
select 5805,'SAND' from dual union all
select 58051,'BONN' from dual union all
select 58052,'MPAE' from dual union all
select 58053,'BONN' from dual union all
select 58054,'BONN' from dual union all
select 58055,'BONN' from dual union all
select 58057,'BONN' from dual union all
select 58058,'BONN' from dual union all
select 5806,'TAO' from dual union all
select 58068,'WFF' from dual union all
select 58069,'BONN' from dual union all
select 5807,'KOBE' from dual union all
select 58074,'PSU' from dual union all
select 58077,'WFF' from dual union all
select 5808,'TAO' from dual union all
select 58080,'BONN' from dual union all
select 58083,'WFF' from dual union all
select 5809,'KOBE' from dual union all
select 58090,'WFF' from dual union all
select 58091,'BONN' from dual union all
select 58098,'VSSC' from dual union all
select 5810,'SAND' from dual union all
select 58102,'BONN' from dual union all
select 5811,'SAND' from dual union all
select 58113,'BONN' from dual union all
select 5812,'GSFC' from dual union all
select 58124,'BONN' from dual union all
select 5813,'GSFC' from dual union all
select 58135,'BONN' from dual union all
select 5814,'SUI' from dual union all
select 58142,'BONN' from dual union all
select 58143,'BONN' from dual union all
select 58145,'BONN' from dual union all
select 58146,'BONN' from dual union all
select 58147,'BONN' from dual union all
select 58148,'BONN' from dual union all
select 58149,'BONN' from dual union all
select 5815,'SUI' from dual union all
select 58150,'BONN' from dual union all
select 58151,'BONN' from dual union all
select 58152,'BONN' from dual union all
select 58153,'BONN' from dual union all
select 58154,'BONN' from dual union all
select 58155,'BONN' from dual union all
select 58156,'BONN' from dual union all
select 58157,'BONN' from dual union all
select 58158,'BONN' from dual union all
select 58159,'BONN' from dual union all
select 58160,'BONN' from dual union all
select 58162,'BONN' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 58163,'BONN' from dual union all
select 58164,'BONN' from dual union all
select 58165,'BONN' from dual union all
select 58168,'BONN' from dual union all
select 58170,'BONN' from dual union all
select 58172,'BONN' from dual union all
select 58174,'BONN' from dual union all
select 58175,'BONN' from dual union all
select 58177,'BONN' from dual union all
select 58178,'BONN' from dual union all
select 58179,'BONN' from dual union all
select 5818,'SAND' from dual union all
select 58180,'BONN' from dual union all
select 58181,'BONN' from dual union all
select 58182,'WFF' from dual union all
select 58184,'BONN' from dual union all
select 58185,'BONN' from dual union all
select 58186,'BONN' from dual union all
select 58187,'BONN' from dual union all
select 58188,'BONN' from dual union all
select 58189,'BONN' from dual union all
select 5819,'SUI' from dual union all
select 58190,'BONN' from dual union all
select 58191,'BONN' from dual union all
select 58192,'BONN' from dual union all
select 58193,'BONN' from dual union all
select 58194,'BONN' from dual union all
select 58195,'BONN' from dual union all
select 58196,'BONN' from dual union all
select 58197,'BONN' from dual union all
select 58198,'BONN' from dual union all
select 5820,'RAE' from dual union all
select 5820,'WRE' from dual union all
select 58200,'BONN' from dual union all
select 58201,'PSU' from dual union all
select 58202,'BONN' from dual union all
select 5821,'SAND' from dual union all
select 58212,'WFF' from dual union all
select 58217,'PSU' from dual union all
select 58218,'PSU' from dual union all
select 58219,'PSU' from dual union all
select 5822,'SAND' from dual union all
select 58220,'GSFC' from dual union all
select 58221,'PSU' from dual union all
select 58223,'WFF' from dual union all
select 5824,'GSFC' from dual union all
select 5825,'SAND' from dual union all
select 5826,'SAND' from dual union all
select 5827,'SAND' from dual union all
select 5828,'SAND' from dual union all
select 5829,'SAND' from dual union all
select 5830,'SAND' from dual union all
select 58347,'BONN' from dual union all
select 58349,'ASE' from dual union all
select 58354,'JMA' from dual union all
select 58356,'WFF' from dual union all
select 58358,'WFF' from dual union all
select 58359,'INPE' from dual union all
select 5836,'SAND' from dual union all
select 58361,'WFF' from dual union all
select 58362,'WFF' from dual union all
select 58363,'SAND' from dual union all
select 58369,'PITT' from dual union all
select 5837,'ISAS' from dual union all
select 58370,'ISRO' from dual union all
select 58371,'ISRO' from dual union all
select 58372,'ISRO' from dual union all
select 58373,'ISRO' from dual union all
select 58374,'ISRO' from dual union all
select 58375,'ISRO' from dual union all
select 58376,'ISRO' from dual union all
select 58377,'ISRO' from dual union all
select 58378,'ISRO' from dual union all
select 58379,'ISRO' from dual union all
select 5838,'NMSU' from dual union all
select 58380,'ISRO' from dual union all
select 58381,'ISRO' from dual union all
select 58382,'ISRO' from dual union all
select 58383,'ISRO' from dual union all
select 58384,'ISRO' from dual union all
select 58385,'ISRO' from dual union all
select 58386,'ISRO' from dual union all
select 58387,'ISRO' from dual union all
select 58388,'ISRO' from dual union all
select 58389,'ISRO' from dual union all
select 5839,'NMSU' from dual union all
select 58390,'ISRO' from dual union all
select 58391,'ISRO' from dual union all
select 58392,'ISRO' from dual union all
select 58393,'ISRO' from dual union all
select 58394,'ISRO' from dual union all
select 58395,'ISRO' from dual union all
select 58396,'ISRO' from dual union all
select 58397,'ISRO' from dual union all
select 58398,'ISRO' from dual union all
select 58399,'ISRO' from dual union all
select 5840,'SMSA' from dual union all
select 58400,'ISRO' from dual union all
select 58401,'ISRO' from dual union all
select 58402,'ISRO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 58403,'ISRO' from dual union all
select 58404,'ISRO' from dual union all
select 58405,'ISRO' from dual union all
select 58406,'ISRO' from dual union all
select 58407,'ISRO' from dual union all
select 58408,'ISRO' from dual union all
select 58409,'ISRO' from dual union all
select 5841,'FRC' from dual union all
select 58410,'ISRO' from dual union all
select 58411,'ISRO' from dual union all
select 58412,'ISRO' from dual union all
select 58413,'ISRO' from dual union all
select 58414,'ISRO' from dual union all
select 58415,'ISRO' from dual union all
select 58416,'ISRO' from dual union all
select 58417,'ISRO' from dual union all
select 58418,'ISRO' from dual union all
select 58419,'GSFC' from dual union all
select 5842,'WRE' from dual union all
select 58420,'GSFC' from dual union all
select 58421,'GSFC' from dual union all
select 58422,'GSFC' from dual union all
select 58423,'GSFC' from dual union all
select 58424,'GSFC' from dual union all
select 58425,'GSFC' from dual union all
select 58426,'GSFC' from dual union all
select 58427,'GSFC' from dual union all
select 5843,'FIAN' from dual union all
select 58438,'SAND' from dual union all
select 58451,'PITT' from dual union all
select 58452,'PSU' from dual union all
select 58453,'PSU' from dual union all
select 58454,'GSFC' from dual union all
select 58455,'GSFC' from dual union all
select 58456,'PITT' from dual union all
select 58458,'NAG' from dual union all
select 58458,'UCB' from dual union all
select 58460,'UMI' from dual union all
select 58462,'SAND' from dual union all
select 58468,'SAND' from dual union all
select 58469,'PSU' from dual union all
select 5847,'ONERA' from dual union all
select 58471,'LPARL' from dual union all
select 58471,'LPSP' from dual union all
select 58472,'ISRO' from dual union all
select 58477,'NDRE' from dual union all
select 58477,'SWRI' from dual union all
select 58479,'AFCRL' from dual union all
select 5848,'SAND' from dual union all
select 58480,'MSFC' from dual union all
select 58483,'INPE' from dual union all
select 58484,'ISAS' from dual union all
select 58484,'USU' from dual union all
select 58486,'PITT' from dual union all
select 58490,'SAND' from dual union all
select 58491,'INTA' from dual union all
select 58493,'INTA' from dual union all
select 58494,'INTA' from dual union all
select 5850,'LARC' from dual union all
select 58514,'SAND' from dual union all
select 5852,'UMI' from dual union all
select 58523,'JHU' from dual union all
select 5853,'UMI' from dual union all
select 58534,'CORN' from dual union all
select 5854,'FIAN' from dual union all
select 58545,'UCB' from dual union all
select 58547,'INTA' from dual union all
select 58548,'INTA' from dual union all
select 5855,'AFSWC' from dual union all
select 5855,'LMSC' from dual union all
select 58552,'UMI' from dual union all
select 5856,'UCL' from dual union all
select 58563,'MPAE' from dual union all
select 58563,'UIO' from dual union all
select 5857,'PSU' from dual union all
select 58579,'ISAS' from dual union all
select 5858,'FIAN' from dual union all
select 58580,'UMI' from dual union all
select 58581,'UCO' from dual union all
select 58582,'UMN' from dual union all
select 58582,'UNH' from dual union all
select 58583,'UMN' from dual union all
select 58583,'UNH' from dual union all
select 58584,'ISAS' from dual union all
select 58585,'NRCC' from dual union all
select 58586,'NRCC' from dual union all
select 58587,'DLR' from dual union all
select 58589,'DLR' from dual union all
select 5859,'FIAN' from dual union all
select 58595,'CORN' from dual union all
select 5860,'NRL' from dual union all
select 58600,'UAF' from dual union all
select 58601,'AFCRL' from dual union all
select 58603,'LUX' from dual union all
select 58603,'MIT' from dual union all
select 58604,'CORN' from dual union all
select 58605,'CORN' from dual union all
select 58606,'PITT' from dual union all
select 58607,'PITT' from dual union all
select 58608,'GSFC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 5861,'SAND' from dual union all
select 58611,'GSFC' from dual union all
select 58612,'PSU' from dual union all
select 58614,'PRIN' from dual union all
select 58615,'DLR' from dual union all
select 58617,'DLR' from dual union all
select 58618,'AES' from dual union all
select 58619,'MPIK' from dual union all
select 5862,'SAND' from dual union all
select 58623,'NIPR' from dual union all
select 5863,'SAND' from dual union all
select 58630,'AFCRL' from dual union all
select 58631,'JHU' from dual union all
select 58634,'AFCRL' from dual union all
select 58640,'UIUC' from dual union all
select 58641,'UIUC' from dual union all
select 58643,'NIPR' from dual union all
select 58649,'JMA' from dual union all
select 5865,'AN' from dual union all
select 5866,'AFSWC' from dual union all
select 5866,'LMSC' from dual union all
select 58665,'JMA' from dual union all
select 5867,'FIAN' from dual union all
select 5868,'FIAN' from dual union all
select 58681,'JMA' from dual union all
select 58686,'JMA' from dual union all
select 5869,'AFSWC' from dual union all
select 5870,'NRL' from dual union all
select 5871,'FIAN' from dual union all
select 58710,'JMA' from dual union all
select 58712,'JMA' from dual union all
select 58723,'JMA' from dual union all
select 5873,'AN' from dual union all
select 58731,'JMA' from dual union all
select 58739,'JMA' from dual union all
select 5875,'RAE' from dual union all
select 5875,'WRE' from dual union all
select 58753,'JMA' from dual union all
select 58761,'JMA' from dual union all
select 58773,'JMA' from dual union all
select 5878,'NRL' from dual union all
select 58784,'JMA' from dual union all
select 58786,'WFF' from dual union all
select 58787,'GMS' from dual union all
select 58788,'GMS' from dual union all
select 58789,'GMS' from dual union all
select 58790,'GMS' from dual union all
select 58791,'GMS' from dual union all
select 58792,'GMS' from dual union all
select 58793,'WFF' from dual union all
select 58796,'JMA' from dual union all
select 58803,'WFF' from dual union all
select 58804,'WFF' from dual union all
select 58805,'JMA' from dual union all
select 58807,'WFF' from dual union all
select 58813,'GMS' from dual union all
select 58815,'JMA' from dual union all
select 58816,'WFF' from dual union all
select 58818,'WFF' from dual union all
select 58819,'WFF' from dual union all
select 58820,'WFF' from dual union all
select 58825,'JMA' from dual union all
select 5883,'AFSWC' from dual union all
select 58832,'WFF' from dual union all
select 58834,'JMA' from dual union all
select 58837,'WFF' from dual union all
select 58842,'GMS' from dual union all
select 58844,'WFF' from dual union all
select 58845,'GMS' from dual union all
select 58846,'GMS' from dual union all
select 58853,'JMA' from dual union all
select 58855,'GMS' from dual union all
select 58857,'GMS' from dual union all
select 58858,'GMS' from dual union all
select 58866,'WFF' from dual union all
select 58867,'JMA' from dual union all
select 58868,'GMS' from dual union all
select 58869,'WFF' from dual union all
select 58870,'JMA' from dual union all
select 58871,'GMS' from dual union all
select 58873,'WFF' from dual union all
select 58874,'GMS' from dual union all
select 5889,'GSFC' from dual union all
select 58896,'JMA' from dual union all
select 58897,'WFF' from dual union all
select 58899,'WFF' from dual union all
select 58901,'WFF' from dual union all
select 58902,'WFF' from dual union all
select 58903,'WFF' from dual union all
select 58904,'WFF' from dual union all
select 58905,'WFF' from dual union all
select 58906,'JMA' from dual union all
select 58908,'WFF' from dual union all
select 5891,'NRL' from dual union all
select 58912,'WFF' from dual union all
select 58914,'WFF' from dual union all
select 58915,'WFF' from dual union all
select 58918,'JMA' from dual union all
select 58919,'WFF' from dual union all
select 58921,'WFF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 58924,'JMA' from dual union all
select 58925,'WFF' from dual union all
select 58926,'JMA' from dual union all
select 58928,'JMA' from dual union all
select 58929,'JMA' from dual union all
select 58930,'JMA' from dual union all
select 58931,'JMA' from dual union all
select 58932,'NIPR' from dual union all
select 58933,'NIPR' from dual union all
select 58934,'NIPR' from dual union all
select 58935,'NIPR' from dual union all
select 58936,'NIPR' from dual union all
select 58937,'NIPR' from dual union all
select 58939,'NIPR' from dual union all
select 5894,'AFCRL' from dual union all
select 58940,'NIPR' from dual union all
select 58941,'NIPR' from dual union all
select 58942,'NIPR' from dual union all
select 58943,'NIPR' from dual union all
select 58944,'ISAS' from dual union all
select 58951,'JMA' from dual union all
select 58962,'JMA' from dual union all
select 5897,'NRL' from dual union all
select 58974,'JMA' from dual union all
select 58977,'JMA' from dual union all
select 5898,'GCA' from dual union all
select 58988,'JMA' from dual union all
select 5899,'UCL' from dual union all
select 58997,'JMA' from dual union all
select 5900,'AFCRL' from dual union all
select 59000,'WFF' from dual union all
select 59001,'WFF' from dual union all
select 59002,'WFF' from dual union all
select 5901,'NRL' from dual union all
select 59013,'PSU' from dual union all
select 59014,'PSU' from dual union all
select 59028,'WFF' from dual union all
select 5903,'NRL' from dual union all
select 5906,'SAND' from dual union all
select 59075,'WFF' from dual union all
select 59077,'WFF' from dual union all
select 59078,'WFF' from dual union all
select 5909,'PSU' from dual union all
select 5911,'NRL' from dual union all
select 5912,'AFCRL' from dual union all
select 5913,'NRL' from dual union all
select 5914,'NRL' from dual union all
select 5915,'NRL' from dual union all
select 59164,'GMS' from dual union all
select 5917,'NRL' from dual union all
select 5919,'CARDE' from dual union all
select 5920,'CARDE' from dual union all
select 5921,'CARDE' from dual union all
select 5924,'CARDE' from dual union all
select 5925,'GSFC' from dual union all
select 5928,'GSFC' from dual union all
select 5929,'AFCRL' from dual union all
select 59305,'CAL' from dual union all
select 59305,'PSU' from dual union all
select 59309,'SAND' from dual union all
select 59319,'ISRO' from dual union all
select 5932,'BIR' from dual union all
select 5932,'UCL' from dual union all
select 59320,'ISRO' from dual union all
select 59321,'ISRO' from dual union all
select 59322,'ISRO' from dual union all
select 59323,'ISRO' from dual union all
select 59324,'ISRO' from dual union all
select 59325,'ISRO' from dual union all
select 59326,'ISRO' from dual union all
select 59327,'ISRO' from dual union all
select 59328,'ISRO' from dual union all
select 59330,'ISRO' from dual union all
select 59331,'ISRO' from dual union all
select 59332,'ISRO' from dual union all
select 59333,'ISRO' from dual union all
select 59334,'ISRO' from dual union all
select 59335,'ISRO' from dual union all
select 59336,'ISRO' from dual union all
select 59337,'ISRO' from dual union all
select 59338,'ISRO' from dual union all
select 59339,'ISRO' from dual union all
select 5934,'DRTE' from dual union all
select 59340,'ISRO' from dual union all
select 59341,'ISRO' from dual union all
select 59342,'ISRO' from dual union all
select 59343,'ISRO' from dual union all
select 59344,'ISRO' from dual union all
select 59345,'ISRO' from dual union all
select 59346,'ISRO' from dual union all
select 59347,'ISRO' from dual union all
select 59348,'ISRO' from dual union all
select 59349,'ISRO' from dual union all
select 59350,'ISRO' from dual union all
select 59351,'ISRO' from dual union all
select 59352,'ISRO' from dual union all
select 59353,'ISRO' from dual union all
select 59354,'ISRO' from dual union all
select 59355,'ISRO' from dual union all
select 59356,'ISRO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 59357,'ISRO' from dual union all
select 59358,'ISRO' from dual union all
select 59359,'ISRO' from dual union all
select 59360,'ISRO' from dual union all
select 59361,'ISRO' from dual union all
select 59362,'ISRO' from dual union all
select 59363,'ISRO' from dual union all
select 59364,'ISRO' from dual union all
select 59365,'ISRO' from dual union all
select 59366,'ISRO' from dual union all
select 59367,'ISRO' from dual union all
select 59368,'ISRO' from dual union all
select 59369,'ISRO' from dual union all
select 59370,'ISRO' from dual union all
select 59371,'ISRO' from dual union all
select 59372,'ISRO' from dual union all
select 59373,'ISRO' from dual union all
select 59374,'ISRO' from dual union all
select 59375,'ISRO' from dual union all
select 59376,'ISRO' from dual union all
select 59377,'ISRO' from dual union all
select 59378,'ISRO' from dual union all
select 59379,'ISRO' from dual union all
select 5938,'AFCRL' from dual union all
select 59380,'ISRO' from dual union all
select 59381,'ISRO' from dual union all
select 59382,'ISRO' from dual union all
select 59383,'ISRO' from dual union all
select 59384,'ISRO' from dual union all
select 59385,'ISRO' from dual union all
select 59386,'ISRO' from dual union all
select 59387,'ISRO' from dual union all
select 59388,'ISRO' from dual union all
select 59389,'ISRO' from dual union all
select 5939,'AFCRL' from dual union all
select 59390,'ISRO' from dual union all
select 59391,'ISRO' from dual union all
select 59392,'ISRO' from dual union all
select 59393,'ISRO' from dual union all
select 59394,'ISRO' from dual union all
select 59395,'ISRO' from dual union all
select 59396,'ISRO' from dual union all
select 59397,'ISRO' from dual union all
select 59398,'ISRO' from dual union all
select 59399,'ISRO' from dual union all
select 5940,'DRTE' from dual union all
select 59400,'ISRO' from dual union all
select 59401,'ISRO' from dual union all
select 59402,'ISRO' from dual union all
select 59403,'ISRO' from dual union all
select 59404,'ISRO' from dual union all
select 59405,'ISRO' from dual union all
select 59406,'ISRO' from dual union all
select 59407,'ISRO' from dual union all
select 59408,'ISRO' from dual union all
select 59409,'ISRO' from dual union all
select 59410,'ISRO' from dual union all
select 59411,'ISRO' from dual union all
select 59412,'ISRO' from dual union all
select 59413,'ISRO' from dual union all
select 59414,'ISRO' from dual union all
select 59415,'ISRO' from dual union all
select 59416,'ISRO' from dual union all
select 59417,'ISRO' from dual union all
select 59418,'ISRO' from dual union all
select 5942,'AFCRL' from dual union all
select 59431,'GSFC' from dual union all
select 59432,'SAO' from dual union all
select 59433,'INPE' from dual union all
select 5944,'BIR' from dual union all
select 5944,'UCL' from dual union all
select 59442,'IRF' from dual union all
select 59444,'PSU' from dual union all
select 59446,'IRF' from dual union all
select 59447,'GSFC' from dual union all
select 59449,'NPL' from dual union all
select 59449,'PRL' from dual union all
select 59451,'NPL' from dual union all
select 59451,'PRL' from dual union all
select 59452,'NPL' from dual union all
select 59452,'PRL' from dual union all
select 59453,'NPL' from dual union all
select 59453,'PRL' from dual union all
select 59469,'NPL' from dual union all
select 59469,'PRL' from dual union all
select 5947,'AFCRL' from dual union all
select 5948,'CNRS' from dual union all
select 5948,'PARIS' from dual union all
select 5950,'CNRS' from dual union all
select 5950,'PARIS' from dual union all
select 59505,'SWRI' from dual union all
select 5951,'NOTS' from dual union all
select 59516,'ISAS' from dual union all
select 59516,'UNK' from dual union all
select 5952,'AFCRL' from dual union all
select 59524,'UWI' from dual union all
select 59530,'ISAS' from dual union all
select 59530,'RIKK' from dual union all
select 59534,'NRL' from dual union all
select 59535,'JHU' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 59538,'GSFC' from dual union all
select 59539,'UCO' from dual union all
select 5954,'AFCRL' from dual union all
select 59540,'NRL' from dual union all
select 59541,'JHU' from dual union all
select 59545,'GSFC' from dual union all
select 59546,'UAF' from dual union all
select 59547,'AFCRL' from dual union all
select 59548,'GSFC' from dual union all
select 59549,'PITT' from dual union all
select 5955,'AFCRL' from dual union all
select 59550,'UCB' from dual union all
select 59551,'UMD' from dual union all
select 59552,'PITT' from dual union all
select 59553,'BU' from dual union all
select 59554,'DLR' from dual union all
select 59555,'DLR' from dual union all
select 59556,'UAH' from dual union all
select 59557,'UAF' from dual union all
select 5956,'AFCRL' from dual union all
select 5957,'AFCRL' from dual union all
select 59579,'GSFC' from dual union all
select 59580,'PSU' from dual union all
select 59581,'PSU' from dual union all
select 59586,'90SMW' from dual union all
select 59591,'SAC' from dual union all
select 59599,'SDIO' from dual union all
select 59600,'ISAS' from dual union all
select 59600,'TOK' from dual union all
select 59614,'JMA' from dual union all
select 59623,'JMA' from dual union all
select 59626,'JMA' from dual union all
select 59635,'JMA' from dual union all
select 59639,'JMA' from dual union all
select 5964,'AFCRL' from dual union all
select 59646,'JMA' from dual union all
select 59649,'JMA' from dual union all
select 5965,'AFCRL' from dual union all
select 59652,'JMA' from dual union all
select 59659,'JMA' from dual union all
select 59664,'JMA' from dual union all
select 5967,'AFCRL' from dual union all
select 59672,'JMA' from dual union all
select 59678,'GMS' from dual union all
select 59679,'JMA' from dual union all
select 5968,'AFCRL' from dual union all
select 59684,'GMS' from dual union all
select 59688,'GMS' from dual union all
select 59689,'GMS' from dual union all
select 5969,'AFCRL' from dual union all
select 59692,'JMA' from dual union all
select 59693,'GMS' from dual union all
select 59696,'JMA' from dual union all
select 59699,'GMS' from dual union all
select 5971,'LARC' from dual union all
select 59714,'JMA' from dual union all
select 59722,'JMA' from dual union all
select 59724,'JMA' from dual union all
select 5973,'AFCRL' from dual union all
select 59732,'JMA' from dual union all
select 59736,'JMA' from dual union all
select 5974,'AN' from dual union all
select 59744,'JMA' from dual union all
select 59749,'JMA' from dual union all
select 5975,'AN' from dual union all
select 59754,'JMA' from dual union all
select 59757,'JMA' from dual union all
select 5976,'AFCRL' from dual union all
select 59760,'ISAS' from dual union all
select 59761,'ISAS' from dual union all
select 59763,'JMA' from dual union all
select 59764,'JMA' from dual union all
select 59765,'JMA' from dual union all
select 59767,'JMA' from dual union all
select 59768,'GMS' from dual union all
select 59769,'GMS' from dual union all
select 5977,'AFCRL' from dual union all
select 59772,'JMA' from dual union all
select 59773,'GMS' from dual union all
select 59776,'GMS' from dual union all
select 59777,'GMS' from dual union all
select 5978,'AFCRL' from dual union all
select 59780,'GMS' from dual union all
select 59784,'GMS' from dual union all
select 59786,'GMS' from dual union all
select 59788,'GMS' from dual union all
select 59789,'GMS' from dual union all
select 59791,'JMA' from dual union all
select 59792,'GMS' from dual union all
select 59793,'IITM' from dual union all
select 59793,'VSSC' from dual union all
select 59795,'GMS' from dual union all
select 59798,'JMA' from dual union all
select 59800,'GMS' from dual union all
select 59801,'GMS' from dual union all
select 59805,'GMS' from dual union all
select 59807,'GMS' from dual union all
select 59808,'JMA' from dual union all
select 59809,'GMS' from dual union all
select 59813,'GMS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 59815,'GMS' from dual union all
select 59816,'GMS' from dual union all
select 59817,'GMS' from dual union all
select 59818,'JMA' from dual union all
select 59822,'JMA' from dual union all
select 59835,'JMA' from dual union all
select 59839,'JMA' from dual union all
select 59840,'JMA' from dual union all
select 59850,'IITM' from dual union all
select 59850,'VSSC' from dual union all
select 5986,'UUT' from dual union all
select 5988,'UUT' from dual union all
select 5989,'LARC' from dual union all
select 5991,'ICST' from dual union all
select 5991,'RAE' from dual union all
select 59927,'USU' from dual union all
select 5993,'CNRS' from dual union all
select 59930,'USU' from dual union all
select 59931,'USU' from dual union all
select 59935,'USU' from dual union all
select 59936,'USU' from dual union all
select 59972,'IITM' from dual union all
select 59972,'VSSC' from dual union all
select 5998,'SAND' from dual union all
select 59989,'IITM' from dual union all
select 59989,'VSSC' from dual union all
select 5999,'SAND' from dual union all
select 59990,'IITM' from dual union all
select 59990,'VSSC' from dual union all
select 59991,'IITM' from dual union all
select 59991,'VSSC' from dual union all
select 59992,'IITM' from dual union all
select 59992,'VSSC' from dual union all
select 59993,'IITM' from dual union all
select 59993,'VSSC' from dual union all
select 59995,'IITM' from dual union all
select 59995,'VSSC' from dual union all
select 59996,'IITM' from dual union all
select 59996,'VSSC' from dual union all
select 6000,'UUT' from dual union all
select 60009,'GSFC' from dual union all
select 6001,'NRL' from dual union all
select 60010,'GSFC' from dual union all
select 60011,'GSFC' from dual union all
select 60012,'GSFC' from dual union all
select 60013,'GSFC' from dual union all
select 60014,'GSFC' from dual union all
select 60015,'IITM' from dual union all
select 60015,'VSSC' from dual union all
select 60019,'IITM' from dual union all
select 60019,'VSSC' from dual union all
select 6002,'BRL' from dual union all
select 6002,'UMI' from dual union all
select 60023,'IITM' from dual union all
select 60023,'VSSC' from dual union all
select 60030,'IITM' from dual union all
select 60030,'VSSC' from dual union all
select 60035,'IITM' from dual union all
select 60035,'VSSC' from dual union all
select 6004,'CNRS' from dual union all
select 60042,'IITM' from dual union all
select 60042,'VSSC' from dual union all
select 6005,'AFCRL' from dual union all
select 6006,'AFCRL' from dual union all
select 6007,'GCA' from dual union all
select 6010,'AFCRL' from dual union all
select 6011,'AFCRL' from dual union all
select 6012,'GCA' from dual union all
select 6014,'GCA' from dual union all
select 6016,'AFCRL' from dual union all
select 60205,'UCB' from dual union all
select 6022,'NRL' from dual union all
select 60220,'DLR' from dual union all
select 60223,'BONN' from dual union all
select 60225,'DNA' from dual union all
select 60226,'DNA' from dual union all
select 60227,'DNA' from dual union all
select 60228,'DNA' from dual union all
select 60229,'DNA' from dual union all
select 6023,'QUB' from dual union all
select 60234,'ISRO' from dual union all
select 60235,'ISRO' from dual union all
select 60236,'ISRO' from dual union all
select 60237,'ISRO' from dual union all
select 60238,'ISRO' from dual union all
select 60239,'ISRO' from dual union all
select 6024,'ICST' from dual union all
select 6024,'UCL' from dual union all
select 60240,'ISRO' from dual union all
select 60241,'ISRO' from dual union all
select 60242,'ISRO' from dual union all
select 60243,'ISRO' from dual union all
select 60244,'ISRO' from dual union all
select 60245,'ISRO' from dual union all
select 60246,'ISRO' from dual union all
select 60247,'ISRO' from dual union all
select 60248,'ISRO' from dual union all
select 60249,'ISRO' from dual union all
select 6025,'LARC' from dual union all
select 60250,'ISRO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 60251,'ISRO' from dual union all
select 60252,'ISRO' from dual union all
select 60253,'ISRO' from dual union all
select 60254,'ISRO' from dual union all
select 60255,'ISRO' from dual union all
select 60256,'ISRO' from dual union all
select 60257,'ISRO' from dual union all
select 60258,'ISRO' from dual union all
select 60259,'ISRO' from dual union all
select 6026,'AFCRL' from dual union all
select 60260,'ISRO' from dual union all
select 60261,'ISRO' from dual union all
select 60262,'ISRO' from dual union all
select 60263,'ISRO' from dual union all
select 60264,'ISRO' from dual union all
select 60265,'ISRO' from dual union all
select 60266,'ISRO' from dual union all
select 60267,'ISRO' from dual union all
select 60268,'ISRO' from dual union all
select 60269,'ISRO' from dual union all
select 60270,'ISRO' from dual union all
select 60271,'ISRO' from dual union all
select 60272,'ISRO' from dual union all
select 60273,'ISRO' from dual union all
select 60274,'LAPAN' from dual union all
select 60275,'ISRO' from dual union all
select 60276,'ISRO' from dual union all
select 60277,'ISRO' from dual union all
select 60278,'ISRO' from dual union all
select 60279,'ISRO' from dual union all
select 6028,'BIR' from dual union all
select 6028,'ICST' from dual union all
select 6028,'UCL' from dual union all
select 60280,'ISRO' from dual union all
select 60281,'ISRO' from dual union all
select 60282,'ISRO' from dual union all
select 60283,'ISRO' from dual union all
select 60284,'ISRO' from dual union all
select 60285,'ISRO' from dual union all
select 60286,'ISRO' from dual union all
select 6029,'AN' from dual union all
select 6030,'AN' from dual union all
select 60306,'GSFC' from dual union all
select 60309,'PSU' from dual union all
select 60310,'GSFC' from dual union all
select 60315,'UCO' from dual union all
select 60324,'CORN' from dual union all
select 60325,'BU' from dual union all
select 60326,'AFCRL' from dual union all
select 60327,'ASE' from dual union all
select 60330,'MPE' from dual union all
select 60330,'TUBI' from dual union all
select 60348,'UCB' from dual union all
select 60350,'MITLL' from dual union all
select 60353,'BONN' from dual union all
select 60353,'NDRE' from dual union all
select 60354,'PSU' from dual union all
select 60355,'UIUC' from dual union all
select 60360,'NDRE' from dual union all
select 60361,'BONN' from dual union all
select 60361,'NDRE' from dual union all
select 60363,'GSFC' from dual union all
select 60364,'PSU' from dual union all
select 60365,'UIUC' from dual union all
select 60366,'MSFC' from dual union all
select 60366,'STAN' from dual union all
select 60368,'GSFC' from dual union all
select 60369,'90SMW' from dual union all
select 6037,'SAND' from dual union all
select 60370,'INPE' from dual union all
select 60372,'NDRE' from dual union all
select 60374,'BONN' from dual union all
select 60374,'NDRE' from dual union all
select 60375,'PSU' from dual union all
select 60376,'UIUC' from dual union all
select 60377,'PSU' from dual union all
select 60378,'MITLL' from dual union all
select 60379,'UCO' from dual union all
select 60380,'CAL' from dual union all
select 60382,'ASE' from dual union all
select 60384,'NRL' from dual union all
select 60385,'AFCRL' from dual union all
select 60385,'USU' from dual union all
select 60387,'FREIB' from dual union all
select 60387,'PRL' from dual union all
select 60389,'SDIO' from dual union all
select 60409,'SDIO' from dual union all
select 6044,'DRTE' from dual union all
select 6044,'GSFC' from dual union all
select 6044,'NRL' from dual union all
select 60460,'SAMSO' from dual union all
select 60463,'ISAS' from dual union all
select 60466,'91MW' from dual union all
select 60467,'PITT' from dual union all
select 60468,'UMN' from dual union all
select 6047,'SAND' from dual union all
select 60470,'321SMW' from dual union all
select 60470,'90SMW' from dual union all
select 60473,'MPIK' from dual union all
select 60474,'UMI' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 60475,'TOK' from dual union all
select 60476,'NAG' from dual union all
select 60476,'UCB' from dual union all
select 60477,'AFCRL' from dual union all
select 60478,'UAF' from dual union all
select 60479,'UAF' from dual union all
select 60480,'USU' from dual union all
select 60483,'SSC' from dual union all
select 60485,'AFCRL' from dual union all
select 60486,'CLEM' from dual union all
select 60486,'DMI' from dual union all
select 60487,'CLEM' from dual union all
select 60487,'DMI' from dual union all
select 60494,'UIUC' from dual union all
select 60495,'CORN' from dual union all
select 60496,'CORN' from dual union all
select 60497,'CORN' from dual union all
select 60503,'DLR' from dual union all
select 60504,'MPAE' from dual union all
select 60505,'DLR' from dual union all
select 60518,'ISRO' from dual union all
select 60522,'GMS' from dual union all
select 60523,'GMS' from dual union all
select 60525,'GMS' from dual union all
select 60526,'GMS' from dual union all
select 60527,'GMS' from dual union all
select 60528,'GMS' from dual union all
select 60529,'GMS' from dual union all
select 6053,'CNRS' from dual union all
select 6053,'PARIS' from dual union all
select 60530,'GMS' from dual union all
select 60536,'GMS' from dual union all
select 60537,'GMS' from dual union all
select 6054,'CNRS' from dual union all
select 6054,'PARIS' from dual union all
select 60541,'JMA' from dual union all
select 60545,'GMS' from dual union all
select 60548,'GMS' from dual union all
select 6055,'CNRS' from dual union all
select 6055,'PARIS' from dual union all
select 60550,'JMA' from dual union all
select 6056,'CNRS' from dual union all
select 6056,'PARIS' from dual union all
select 60563,'JMA' from dual union all
select 6057,'CNRS' from dual union all
select 6057,'PARIS' from dual union all
select 60570,'JMA' from dual union all
select 60575,'GMS' from dual union all
select 60576,'JMA' from dual union all
select 6058,'SAND' from dual union all
select 60585,'GMS' from dual union all
select 60588,'JMA' from dual union all
select 60589,'GMS' from dual union all
select 60593,'GMS' from dual union all
select 60595,'GMS' from dual union all
select 60599,'JMA' from dual union all
select 60600,'GMS' from dual union all
select 60603,'GMS' from dual union all
select 60607,'JMA' from dual union all
select 60608,'GMS' from dual union all
select 60609,'GMS' from dual union all
select 6061,'WRE' from dual union all
select 60613,'GMS' from dual union all
select 60614,'GMS' from dual union all
select 60616,'BONN' from dual union all
select 60617,'JMA' from dual union all
select 60619,'GMS' from dual union all
select 60620,'GMS' from dual union all
select 60623,'GMS' from dual union all
select 60624,'GMS' from dual union all
select 60625,'WFF' from dual union all
select 60629,'GMS' from dual union all
select 60630,'GMS' from dual union all
select 60632,'GMS' from dual union all
select 60634,'GMS' from dual union all
select 60635,'JMA' from dual union all
select 60636,'GMS' from dual union all
select 60638,'GMS' from dual union all
select 60639,'WFF' from dual union all
select 60640,'GMS' from dual union all
select 60641,'GMS' from dual union all
select 60645,'JMA' from dual union all
select 60655,'JMA' from dual union all
select 60661,'JMA' from dual union all
select 60663,'JMA' from dual union all
select 60664,'GMS' from dual union all
select 60669,'JMA' from dual union all
select 60670,'GMS' from dual union all
select 60673,'GMS' from dual union all
select 60674,'GMS' from dual union all
select 60676,'GMS' from dual union all
select 60683,'GMS' from dual union all
select 60684,'GMS' from dual union all
select 60688,'GMS' from dual union all
select 6069,'NRL' from dual union all
select 60690,'GMS' from dual union all
select 60694,'JMA' from dual union all
select 60700,'BONN' from dual union all
select 60700,'MPAE' from dual union all
select 60703,'JMA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 60711,'JMA' from dual union all
select 60721,'JMA' from dual union all
select 60724,'GMS' from dual union all
select 60727,'JMA' from dual union all
select 60728,'GMS' from dual union all
select 60735,'JMA' from dual union all
select 60736,'GMS' from dual union all
select 60739,'GMS' from dual union all
select 60740,'GMS' from dual union all
select 60741,'GMS' from dual union all
select 60744,'GMS' from dual union all
select 60747,'GMS' from dual union all
select 60749,'GMS' from dual union all
select 60750,'PRL' from dual union all
select 60751,'JMA' from dual union all
select 60753,'GMS' from dual union all
select 60754,'JMA' from dual union all
select 60755,'GMS' from dual union all
select 60758,'GMS' from dual union all
select 60762,'GMS' from dual union all
select 60763,'JMA' from dual union all
select 60765,'ISAS' from dual union all
select 60766,'ISAS' from dual union all
select 60767,'JMA' from dual union all
select 60768,'JMA' from dual union all
select 60769,'JMA' from dual union all
select 60770,'JMA' from dual union all
select 60781,'JMA' from dual union all
select 60786,'JMA' from dual union all
select 60790,'JMA' from dual union all
select 60794,'GMS' from dual union all
select 60796,'GMS' from dual union all
select 6080,'LARC' from dual union all
select 60801,'GMS' from dual union all
select 60804,'JMA' from dual union all
select 60807,'JMA' from dual union all
select 60808,'GMS' from dual union all
select 60811,'GMS' from dual union all
select 60816,'JMA' from dual union all
select 60817,'GMS' from dual union all
select 60820,'GMS' from dual union all
select 60821,'GMS' from dual union all
select 60827,'GMS' from dual union all
select 60829,'JMA' from dual union all
select 60830,'GMS' from dual union all
select 60834,'GMS' from dual union all
select 60836,'GMS' from dual union all
select 60955,'BONN' from dual union all
select 60959,'BONN' from dual union all
select 60960,'BONN' from dual union all
select 60961,'BONN' from dual union all
select 60962,'BONN' from dual union all
select 60963,'BONN' from dual union all
select 60965,'BONN' from dual union all
select 60967,'BONN' from dual union all
select 60970,'BONN' from dual union all
select 60973,'BONN' from dual union all
select 60976,'USU' from dual union all
select 60977,'BONN' from dual union all
select 60978,'BONN' from dual union all
select 60979,'BONN' from dual union all
select 60981,'USU' from dual union all
select 60982,'BONN' from dual union all
select 60983,'BONN' from dual union all
select 60984,'BONN' from dual union all
select 60985,'BONN' from dual union all
select 60986,'BONN' from dual union all
select 60987,'USU' from dual union all
select 60988,'BONN' from dual union all
select 60990,'USU' from dual union all
select 60992,'BONN' from dual union all
select 60993,'BONN' from dual union all
select 60996,'BONN' from dual union all
select 60997,'BONN' from dual union all
select 60998,'BONN' from dual union all
select 60999,'BONN' from dual union all
select 61000,'BONN' from dual union all
select 61001,'BONN' from dual union all
select 61003,'BONN' from dual union all
select 61004,'BONN' from dual union all
select 61006,'BONN' from dual union all
select 61007,'BONN' from dual union all
select 61071,'AFGL' from dual union all
select 61071,'BONN' from dual union all
select 61072,'BONN' from dual union all
select 61072,'MPAE' from dual union all
select 61073,'GSFC' from dual union all
select 61074,'GSFC' from dual union all
select 61075,'BONN' from dual union all
select 61075,'MPAE' from dual union all
select 61081,'AFGL' from dual union all
select 61081,'BONN' from dual union all
select 61082,'BONN' from dual union all
select 61082,'MPAE' from dual union all
select 61083,'GSFC' from dual union all
select 61084,'GSFC' from dual union all
select 61085,'BONN' from dual union all
select 61085,'MPAE' from dual union all
select 61086,'BONN' from dual union all
select 61086,'MPAE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 61095,'GSFC' from dual union all
select 61096,'GSFC' from dual union all
select 61103,'BONN' from dual union all
select 61103,'MPAE' from dual union all
select 61116,'AFGL' from dual union all
select 61116,'BONN' from dual union all
select 61117,'AFGL' from dual union all
select 61117,'BONN' from dual union all
select 61118,'GSFC' from dual union all
select 61119,'GSFC' from dual union all
select 61138,'PRL' from dual union all
select 61139,'PRL' from dual union all
select 61140,'PRL' from dual union all
select 61141,'PRL' from dual union all
select 61142,'PRL' from dual union all
select 61144,'PRL' from dual union all
select 61145,'PRL' from dual union all
select 61146,'PRL' from dual union all
select 61147,'PRL' from dual union all
select 61148,'PRL' from dual union all
select 61149,'PRL' from dual union all
select 61150,'PRL' from dual union all
select 61151,'PRL' from dual union all
select 61152,'PRL' from dual union all
select 61167,'BONN' from dual union all
select 61168,'BONN' from dual union all
select 61169,'BONN' from dual union all
select 61170,'BONN' from dual union all
select 61171,'BONN' from dual union all
select 61172,'BONN' from dual union all
select 61173,'BONN' from dual union all
select 61174,'BONN' from dual union all
select 61175,'BONN' from dual union all
select 61177,'BONN' from dual union all
select 61178,'BONN' from dual union all
select 61179,'BONN' from dual union all
select 61180,'BONN' from dual union all
select 61181,'BONN' from dual union all
select 61182,'BONN' from dual union all
select 61183,'BONN' from dual union all
select 61184,'BONN' from dual union all
select 61185,'BONN' from dual union all
select 61186,'BONN' from dual union all
select 61188,'BONN' from dual union all
select 61189,'BONN' from dual union all
select 61190,'BONN' from dual union all
select 61190,'MPAE' from dual union all
select 61191,'BONN' from dual union all
select 61191,'MPAE' from dual union all
select 61225,'BONN' from dual union all
select 61373,'LINAO' from dual union all
select 61374,'LINAO' from dual union all
select 61375,'SAND' from dual union all
select 61456,'NASDA' from dual union all
select 61457,'CSSAR' from dual union all
select 61458,'CSSAR' from dual union all
select 61459,'CSSAR' from dual union all
select 61460,'CSSAR' from dual union all
select 61461,'ISRO' from dual union all
select 61462,'ISRO' from dual union all
select 61463,'ISRO' from dual union all
select 61464,'ISRO' from dual union all
select 61465,'ISRO' from dual union all
select 61466,'ISRO' from dual union all
select 61467,'ISRO' from dual union all
select 61468,'ISRO' from dual union all
select 61469,'ISRO' from dual union all
select 61470,'ISRO' from dual union all
select 61471,'ISRO' from dual union all
select 61472,'ISRO' from dual union all
select 61473,'ISRO' from dual union all
select 61474,'ISRO' from dual union all
select 61475,'ISRO' from dual union all
select 61476,'ISRO' from dual union all
select 61477,'ISRO' from dual union all
select 61478,'ISRO' from dual union all
select 61479,'ISRO' from dual union all
select 61480,'ISRO' from dual union all
select 61481,'ISRO' from dual union all
select 61482,'ISRO' from dual union all
select 61483,'ISRO' from dual union all
select 61484,'ISRO' from dual union all
select 61485,'ISRO' from dual union all
select 61486,'ISRO' from dual union all
select 61487,'ISRO' from dual union all
select 61488,'ISRO' from dual union all
select 61489,'ISRO' from dual union all
select 61490,'ISRO' from dual union all
select 61491,'ISRO' from dual union all
select 61492,'ISRO' from dual union all
select 61493,'ISRO' from dual union all
select 61494,'ISRO' from dual union all
select 61495,'ISRO' from dual union all
select 61496,'ISRO' from dual union all
select 61497,'ISRO' from dual union all
select 61498,'ISRO' from dual union all
select 61499,'ISRO' from dual union all
select 61500,'ISRO' from dual union all
select 61501,'ISRO' from dual union all
select 61502,'ISRO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 61503,'ISRO' from dual union all
select 61504,'ISRO' from dual union all
select 61505,'ISRO' from dual union all
select 61506,'ISRO' from dual union all
select 61507,'WFF' from dual union all
select 61508,'WFF' from dual union all
select 61509,'WFF' from dual union all
select 61510,'WFF' from dual union all
select 61511,'ISRO' from dual union all
select 61512,'ISRO' from dual union all
select 61513,'ISRO' from dual union all
select 61514,'ISRO' from dual union all
select 61515,'ISRO' from dual union all
select 61516,'ISRO' from dual union all
select 61517,'ISRO' from dual union all
select 61518,'ISRO' from dual union all
select 61519,'ISRO' from dual union all
select 61520,'ISRO' from dual union all
select 61521,'ISRO' from dual union all
select 61522,'ISRO' from dual union all
select 61523,'ISRO' from dual union all
select 61524,'ISRO' from dual union all
select 61525,'ISRO' from dual union all
select 61526,'ISRO' from dual union all
select 61527,'ISRO' from dual union all
select 61528,'ISRO' from dual union all
select 61529,'ISRO' from dual union all
select 61530,'ISRO' from dual union all
select 61531,'ISRO' from dual union all
select 61532,'ISRO' from dual union all
select 61533,'ISRO' from dual union all
select 61534,'ISRO' from dual union all
select 61535,'ISRO' from dual union all
select 61536,'ISRO' from dual union all
select 61537,'ISRO' from dual union all
select 61538,'ISRO' from dual union all
select 61539,'ISRO' from dual union all
select 61540,'ISRO' from dual union all
select 61541,'ISRO' from dual union all
select 61542,'ISRO' from dual union all
select 61543,'ISRO' from dual union all
select 61544,'ISRO' from dual union all
select 61545,'ISRO' from dual union all
select 61546,'ISRO' from dual union all
select 61547,'ISRO' from dual union all
select 61548,'ISRO' from dual union all
select 61549,'ISRO' from dual union all
select 61550,'ISRO' from dual union all
select 61551,'ISRO' from dual union all
select 61552,'ISRO' from dual union all
select 61553,'ISRO' from dual union all
select 61554,'ISRO' from dual union all
select 61555,'ISRO' from dual union all
select 61556,'ISRO' from dual union all
select 61567,'SAO' from dual union all
select 61572,'CORN' from dual union all
select 61573,'USU' from dual union all
select 61574,'SDIO' from dual union all
select 61583,'BONN' from dual union all
select 61584,'BU' from dual union all
select 61585,'PRIN' from dual union all
select 61586,'UCO' from dual union all
select 61594,'WFF' from dual union all
select 61595,'PRL' from dual union all
select 61596,'MITLL' from dual union all
select 61597,'UCO' from dual union all
select 61598,'USCLA' from dual union all
select 61600,'91MW' from dual union all
select 61604,'GSFC' from dual union all
select 61605,'UCO' from dual union all
select 61606,'NRL' from dual union all
select 61608,'MPAE' from dual union all
select 61609,'DLR' from dual union all
select 61613,'DLR' from dual union all
select 61614,'MPAE' from dual union all
select 61615,'MPAE' from dual union all
select 61616,'UWI' from dual union all
select 61618,'UAF' from dual union all
select 6168,'UCO' from dual union all
select 61682,'SAMSO' from dual union all
select 61686,'CORN' from dual union all
select 61687,'GSFC' from dual union all
select 61688,'BOCH' from dual union all
select 61688,'ISAS' from dual union all
select 61689,'ISAS' from dual union all
select 61691,'BU' from dual union all
select 61692,'SDIO' from dual union all
select 61693,'AFCRL' from dual union all
select 61693,'UMN' from dual union all
select 61700,'GSFC' from dual union all
select 61703,'PSU' from dual union all
select 61704,'SSC' from dual union all
select 61705,'UCB' from dual union all
select 61706,'SAMSO' from dual union all
select 61707,'UCO' from dual union all
select 61708,'UCO' from dual union all
select 61720,'GSFC' from dual union all
select 61720,'IRF' from dual union all
select 61731,'ESTEC' from dual union all
select 61731,'MBB' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 61732,'PITT' from dual union all
select 61733,'ESTEC' from dual union all
select 61733,'MBB' from dual union all
select 61734,'DLR' from dual union all
select 61735,'PITT' from dual union all
select 61741,'JMA' from dual union all
select 61749,'JMA' from dual union all
select 61756,'JMA' from dual union all
select 61763,'JMA' from dual union all
select 61764,'GMS' from dual union all
select 61765,'GMS' from dual union all
select 61766,'GMS' from dual union all
select 61767,'JMA' from dual union all
select 61768,'GMS' from dual union all
select 61769,'GMS' from dual union all
select 61774,'JMA' from dual union all
select 61775,'GMS' from dual union all
select 61776,'GMS' from dual union all
select 61779,'GMS' from dual union all
select 61781,'GMS' from dual union all
select 61784,'GMS' from dual union all
select 61786,'JMA' from dual union all
select 6179,'NRL' from dual union all
select 61790,'JMA' from dual union all
select 61792,'JMA' from dual union all
select 61796,'JMA' from dual union all
select 618,'RNII' from dual union all
select 61801,'JMA' from dual union all
select 61807,'GMS' from dual union all
select 61808,'JMA' from dual union all
select 61809,'GMS' from dual union all
select 61811,'GMS' from dual union all
select 61813,'GMS' from dual union all
select 61814,'GMS' from dual union all
select 61815,'GMS' from dual union all
select 61820,'GMS' from dual union all
select 61821,'GMS' from dual union all
select 61825,'GMS' from dual union all
select 61830,'JMA' from dual union all
select 61841,'JMA' from dual union all
select 61843,'JMA' from dual union all
select 61844,'GMS' from dual union all
select 61846,'LINAO' from dual union all
select 61847,'GMS' from dual union all
select 61848,'GMS' from dual union all
select 61850,'LINAO' from dual union all
select 61851,'JMA' from dual union all
select 61852,'GMS' from dual union all
select 61853,'GMS' from dual union all
select 61854,'GMS' from dual union all
select 61855,'GMS' from dual union all
select 61857,'JMA' from dual union all
select 61858,'GMS' from dual union all
select 61859,'GMS' from dual union all
select 61861,'LINAO' from dual union all
select 61862,'GMS' from dual union all
select 61864,'LINAO' from dual union all
select 61865,'GMS' from dual union all
select 61867,'JMA' from dual union all
select 61876,'JMA' from dual union all
select 61878,'JMA' from dual union all
select 61881,'GMS' from dual union all
select 61883,'GMS' from dual union all
select 61884,'GMS' from dual union all
select 61885,'JMA' from dual union all
select 61886,'GMS' from dual union all
select 61887,'GMS' from dual union all
select 61888,'GMS' from dual union all
select 61889,'GMS' from dual union all
select 61890,'GMS' from dual union all
select 61891,'JMA' from dual union all
select 61892,'GMS' from dual union all
select 61895,'GMS' from dual union all
select 61896,'ISAS' from dual union all
select 61896,'TSUK' from dual union all
select 61897,'JMA' from dual union all
select 61898,'JMA' from dual union all
select 61899,'JMA' from dual union all
select 619,'APL' from dual union all
select 61900,'JMA' from dual union all
select 61901,'JMA' from dual union all
select 61902,'JMA' from dual union all
select 61907,'JMA' from dual union all
select 61909,'GMS' from dual union all
select 61911,'JMA' from dual union all
select 61912,'GMS' from dual union all
select 61913,'GMS' from dual union all
select 61914,'GMS' from dual union all
select 61915,'JMA' from dual union all
select 61916,'GMS' from dual union all
select 61917,'GMS' from dual union all
select 61920,'GMS' from dual union all
select 61922,'GMS' from dual union all
select 61923,'GMS' from dual union all
select 61924,'GMS' from dual union all
select 61926,'GMS' from dual union all
select 61927,'GMS' from dual union all
select 61928,'GMS' from dual union all
select 61930,'GMS' from dual union all
select 61932,'GMS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 61935,'GMS' from dual union all
select 61937,'JMA' from dual union all
select 61939,'GMS' from dual union all
select 61943,'GMS' from dual union all
select 61944,'GMS' from dual union all
select 61946,'JMA' from dual union all
select 61947,'GMS' from dual union all
select 61949,'GMS' from dual union all
select 61950,'GMS' from dual union all
select 61951,'GMS' from dual union all
select 61953,'GMS' from dual union all
select 61956,'GMS' from dual union all
select 61957,'GMS' from dual union all
select 61960,'GMS' from dual union all
select 61965,'JMA' from dual union all
select 61969,'GMS' from dual union all
select 61970,'GMS' from dual union all
select 61971,'GMS' from dual union all
select 61972,'GMS' from dual union all
select 61973,'JMA' from dual union all
select 61976,'JMA' from dual union all
select 6201,'LMSC' from dual union all
select 6204,'NMSU' from dual union all
select 6208,'AFCRL' from dual union all
select 6209,'LRL' from dual union all
select 621,'JPL' from dual union all
select 6212,'AFCRL' from dual union all
select 6214,'SAND' from dual union all
select 6215,'SAND' from dual union all
select 6217,'AFCRL' from dual union all
select 6218,'AFCRL' from dual union all
select 62198,'BONN' from dual union all
select 62199,'BONN' from dual union all
select 622,'RNII' from dual union all
select 62200,'BONN' from dual union all
select 62201,'BONN' from dual union all
select 62202,'BONN' from dual union all
select 62203,'BONN' from dual union all
select 62204,'BONN' from dual union all
select 62205,'BONN' from dual union all
select 62206,'BONN' from dual union all
select 62208,'BONN' from dual union all
select 6222,'WRE' from dual union all
select 6225,'AFCRL' from dual union all
select 6226,'SAND' from dual union all
select 6227,'SAND' from dual union all
select 6228,'PSU' from dual union all
select 623,'RNII' from dual union all
select 6233,'AFCRL' from dual union all
select 6235,'PSU' from dual union all
select 62379,'JHU' from dual union all
select 62386,'AMROC' from dual union all
select 62388,'LINAO' from dual union all
select 62389,'UAH' from dual union all
select 6239,'AFCRL' from dual union all
select 62392,'WFF' from dual union all
select 62397,'NASDA' from dual union all
select 62399,'INPE' from dual union all
select 624,'RNII' from dual union all
select 6240,'AFCRL' from dual union all
select 62400,'NASDA' from dual union all
select 62402,'WFF' from dual union all
select 62403,'WFF' from dual union all
select 62404,'WFF' from dual union all
select 62405,'WFF' from dual union all
select 62406,'WFF' from dual union all
select 62407,'WFF' from dual union all
select 62408,'SUPA' from dual union all
select 62409,'ISRO' from dual union all
select 6241,'AFCRL' from dual union all
select 62410,'ISRO' from dual union all
select 62411,'ISRO' from dual union all
select 62412,'ISRO' from dual union all
select 62413,'ISRO' from dual union all
select 62414,'ISRO' from dual union all
select 62415,'ISRO' from dual union all
select 62416,'ISRO' from dual union all
select 62417,'ISRO' from dual union all
select 62418,'ISRO' from dual union all
select 62419,'ISRO' from dual union all
select 6242,'PSU' from dual union all
select 62420,'ISRO' from dual union all
select 62421,'ISRO' from dual union all
select 62422,'ISRO' from dual union all
select 62423,'ISRO' from dual union all
select 62424,'ISRO' from dual union all
select 62425,'ISRO' from dual union all
select 62426,'ISRO' from dual union all
select 62427,'ISRO' from dual union all
select 62428,'ISRO' from dual union all
select 62429,'ISRO' from dual union all
select 62430,'ISRO' from dual union all
select 62431,'ISRO' from dual union all
select 62432,'ISRO' from dual union all
select 62433,'ISRO' from dual union all
select 62434,'ISRO' from dual union all
select 62435,'ISRO' from dual union all
select 62436,'ISRO' from dual union all
select 62437,'ISRO' from dual union all
select 62438,'ISRO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 62439,'ISRO' from dual union all
select 62440,'ISRO' from dual union all
select 62441,'ISRO' from dual union all
select 62442,'ISRO' from dual union all
select 62443,'ISRO' from dual union all
select 62444,'ISRO' from dual union all
select 62445,'ISRO' from dual union all
select 62446,'ISRO' from dual union all
select 62447,'ISRO' from dual union all
select 62448,'ISRO' from dual union all
select 62449,'ISRO' from dual union all
select 62450,'ISRO' from dual union all
select 62451,'ISRO' from dual union all
select 62452,'ISRO' from dual union all
select 62453,'ISRO' from dual union all
select 62454,'ISRO' from dual union all
select 6246,'LARC' from dual union all
select 62467,'LASL' from dual union all
select 62467,'NRL' from dual union all
select 62473,'UCO' from dual union all
select 62474,'MBB' from dual union all
select 62477,'UCB' from dual union all
select 62477,'WFF' from dual union all
select 62478,'MBB' from dual union all
select 6248,'AFCRL' from dual union all
select 62481,'SDIO' from dual union all
select 62482,'SDIO' from dual union all
select 62489,'SUPA' from dual union all
select 62517,'GSFC' from dual union all
select 62521,'MPIK' from dual union all
select 62522,'CORN' from dual union all
select 62522,'CRC' from dual union all
select 62523,'ISAS' from dual union all
select 62523,'TOK' from dual union all
select 62525,'MPAE' from dual union all
select 62526,'MPAE' from dual union all
select 62527,'90SMW' from dual union all
select 62528,'GSFC' from dual union all
select 62528,'IRF' from dual union all
select 62529,'GSFC' from dual union all
select 62529,'IRF' from dual union all
select 6253,'SAND' from dual union all
select 62530,'91MW' from dual union all
select 62531,'UCO' from dual union all
select 62534,'PITT' from dual union all
select 62536,'PITT' from dual union all
select 62537,'INPE' from dual union all
select 62538,'UAH' from dual union all
select 62539,'BU' from dual union all
select 6254,'SAND' from dual union all
select 62540,'GSFC' from dual union all
select 62541,'SSC' from dual union all
select 62542,'GSFC' from dual union all
select 62543,'MBB' from dual union all
select 62544,'MBB' from dual union all
select 62545,'UAH' from dual union all
select 62546,'GSFC' from dual union all
select 62548,'AERO' from dual union all
select 6255,'SAND' from dual union all
select 62550,'UCO' from dual union all
select 62551,'GSFC' from dual union all
select 62558,'90SMW' from dual union all
select 62559,'LANL' from dual union all
select 62560,'UMI' from dual union all
select 62561,'SDIO' from dual union all
select 62562,'SDIO' from dual union all
select 62567,'SDIO' from dual union all
select 62568,'SDIO' from dual union all
select 6257,'AN' from dual union all
select 62570,'SDIO' from dual union all
select 62571,'USCLA' from dual union all
select 62576,'SDIO' from dual union all
select 62577,'NAG' from dual union all
select 62578,'SDIO' from dual union all
select 62579,'SAO' from dual union all
select 6258,'AFCRL' from dual union all
select 62582,'SDIO' from dual union all
select 62588,'GMS' from dual union all
select 62589,'GMS' from dual union all
select 6259,'AFCRL' from dual union all
select 62590,'GMS' from dual union all
select 62591,'GMS' from dual union all
select 62592,'GMS' from dual union all
select 62596,'GMS' from dual union all
select 62597,'GMS' from dual union all
select 626,'NRL' from dual union all
select 62601,'GMS' from dual union all
select 62604,'LINAO' from dual union all
select 62607,'LINAO' from dual union all
select 62608,'GMS' from dual union all
select 6261,'NOTS' from dual union all
select 62611,'GMS' from dual union all
select 62612,'GMS' from dual union all
select 62615,'LINAO' from dual union all
select 62619,'GMS' from dual union all
select 62621,'GMS' from dual union all
select 62630,'GMS' from dual union all
select 62636,'GMS' from dual union all
select 62637,'LINAO' from dual union all
select 62640,'GMS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 62643,'GMS' from dual union all
select 62646,'GMS' from dual union all
select 62649,'GMS' from dual union all
select 6265,'USAF' from dual union all
select 62651,'LINAO' from dual union all
select 62652,'GMS' from dual union all
select 62655,'GMS' from dual union all
select 62658,'GMS' from dual union all
select 62659,'GMS' from dual union all
select 6266,'SMSA' from dual union all
select 62661,'GMS' from dual union all
select 62665,'GMS' from dual union all
select 6267,'WSMR' from dual union all
select 62671,'GMS' from dual union all
select 62676,'GMS' from dual union all
select 62679,'LINAO' from dual union all
select 6268,'BRL' from dual union all
select 62682,'GMS' from dual union all
select 62684,'GMS' from dual union all
select 62685,'GMS' from dual union all
select 6269,'PMR' from dual union all
select 62696,'JMA' from dual union all
select 627,'PRIN' from dual union all
select 6270,'PMR' from dual union all
select 62707,'GMS' from dual union all
select 6271,'PMR' from dual union all
select 6272,'PMR' from dual union all
select 62725,'GMS' from dual union all
select 62727,'ISAS' from dual union all
select 62727,'TSUK' from dual union all
select 62729,'GMS' from dual union all
select 6273,'PMR' from dual union all
select 62730,'LINAO' from dual union all
select 62731,'GMS' from dual union all
select 6274,'PMR' from dual union all
select 62743,'LINAO' from dual union all
select 6275,'PMR' from dual union all
select 62751,'GMS' from dual union all
select 62752,'GMS' from dual union all
select 62754,'LINAO' from dual union all
select 62759,'GMS' from dual union all
select 6276,'PMR' from dual union all
select 62760,'GMS' from dual union all
select 62761,'GMS' from dual union all
select 62762,'GMS' from dual union all
select 62763,'GMS' from dual union all
select 62767,'GMS' from dual union all
select 6277,'PMR' from dual union all
select 62773,'GMS' from dual union all
select 62776,'GMS' from dual union all
select 62778,'GMS' from dual union all
select 6278,'PMR' from dual union all
select 62780,'LINAO' from dual union all
select 62782,'GMS' from dual union all
select 62783,'GMS' from dual union all
select 62784,'GMS' from dual union all
select 62786,'GMS' from dual union all
select 62789,'GMS' from dual union all
select 6279,'PMR' from dual union all
select 62795,'GMS' from dual union all
select 62798,'GMS' from dual union all
select 628,'AFCRL' from dual union all
select 628,'UMI' from dual union all
select 6280,'PMR' from dual union all
select 6281,'SMSA' from dual union all
select 62811,'LINAO' from dual union all
select 6282,'AFCRL' from dual union all
select 62821,'LINAO' from dual union all
select 62825,'JMA' from dual union all
select 62826,'JMA' from dual union all
select 62827,'JMA' from dual union all
select 62828,'GMS' from dual union all
select 62829,'JMA' from dual union all
select 6283,'SMSA' from dual union all
select 62830,'JMA' from dual union all
select 62831,'JMA' from dual union all
select 62832,'JMA' from dual union all
select 62833,'JMA' from dual union all
select 62834,'JMA' from dual union all
select 62835,'JMA' from dual union all
select 62836,'JMA' from dual union all
select 62837,'JMA' from dual union all
select 62838,'JMA' from dual union all
select 62840,'JMA' from dual union all
select 62841,'JMA' from dual union all
select 62842,'JMA' from dual union all
select 62843,'JMA' from dual union all
select 62844,'JMA' from dual union all
select 62845,'JMA' from dual union all
select 62846,'JMA' from dual union all
select 62847,'JMA' from dual union all
select 62848,'JMA' from dual union all
select 62849,'JMA' from dual union all
select 62851,'JMA' from dual union all
select 62852,'JMA' from dual union all
select 62853,'JMA' from dual union all
select 62854,'JMA' from dual union all
select 62855,'JMA' from dual union all
select 62856,'JMA' from dual union all
select 62857,'JMA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 62858,'JMA' from dual union all
select 62859,'JMA' from dual union all
select 62860,'JMA' from dual union all
select 62861,'GMS' from dual union all
select 62862,'GMS' from dual union all
select 62868,'GMS' from dual union all
select 62870,'GMS' from dual union all
select 62871,'GMS' from dual union all
select 62872,'GMS' from dual union all
select 62874,'GMS' from dual union all
select 62876,'GMS' from dual union all
select 62877,'GMS' from dual union all
select 62884,'GMS' from dual union all
select 62886,'GMS' from dual union all
select 62887,'GMS' from dual union all
select 62889,'GMS' from dual union all
select 62893,'GMS' from dual union all
select 62915,'GMS' from dual union all
select 62917,'GMS' from dual union all
select 62925,'GMS' from dual union all
select 63361,'USU' from dual union all
select 63365,'IAP' from dual union all
select 63366,'IAP' from dual union all
select 63367,'SRCI' from dual union all
select 63368,'SRCI' from dual union all
select 63369,'MPAE' from dual union all
select 63370,'WFF' from dual union all
select 63370,'WUPP' from dual union all
select 63371,'WFF' from dual union all
select 63371,'WUPP' from dual union all
select 63372,'WFF' from dual union all
select 63372,'WUPP' from dual union all
select 63379,'PRL' from dual union all
select 63379,'TSAO' from dual union all
select 63380,'PRL' from dual union all
select 63380,'TSAO' from dual union all
select 63381,'PRL' from dual union all
select 63381,'TSAO' from dual union all
select 63382,'PRL' from dual union all
select 63382,'TSAO' from dual union all
select 63383,'PRL' from dual union all
select 63383,'TSAO' from dual union all
select 63384,'PRL' from dual union all
select 63384,'TSAO' from dual union all
select 63385,'PRL' from dual union all
select 63385,'TSAO' from dual union all
select 63386,'PRL' from dual union all
select 63386,'TSAO' from dual union all
select 63387,'PRL' from dual union all
select 63387,'TSAO' from dual union all
select 63388,'PRL' from dual union all
select 63388,'TSAO' from dual union all
select 63389,'PRL' from dual union all
select 63389,'TSAO' from dual union all
select 63390,'PRL' from dual union all
select 63390,'TSAO' from dual union all
select 63391,'PRL' from dual union all
select 63391,'TSAO' from dual union all
select 63392,'PRL' from dual union all
select 63392,'TSAO' from dual union all
select 63393,'PRL' from dual union all
select 63393,'TSAO' from dual union all
select 63394,'PRL' from dual union all
select 63394,'TSAO' from dual union all
select 63395,'SRCI' from dual union all
select 63396,'ISRO' from dual union all
select 63397,'ISRO' from dual union all
select 63398,'ISRO' from dual union all
select 63399,'ISRO' from dual union all
select 63400,'ISRO' from dual union all
select 63401,'ISRO' from dual union all
select 63402,'ISRO' from dual union all
select 63403,'ISRO' from dual union all
select 63404,'ISRO' from dual union all
select 63405,'ISRO' from dual union all
select 63406,'ISRO' from dual union all
select 63407,'ISRO' from dual union all
select 63408,'ISRO' from dual union all
select 63409,'ISRO' from dual union all
select 63410,'ISRO' from dual union all
select 63411,'ISRO' from dual union all
select 63412,'ISRO' from dual union all
select 63413,'ISRO' from dual union all
select 63414,'ISRO' from dual union all
select 63415,'ISRO' from dual union all
select 63416,'ISRO' from dual union all
select 63417,'ISRO' from dual union all
select 63418,'ISRO' from dual union all
select 63419,'ISRO' from dual union all
select 63420,'ISRO' from dual union all
select 63421,'ISRO' from dual union all
select 63432,'UCO' from dual union all
select 63435,'AERO' from dual union all
select 63436,'CAL' from dual union all
select 63437,'USU' from dual union all
select 63437,'WUPP' from dual union all
select 63438,'GSFC' from dual union all
select 63446,'CORN' from dual union all
select 63447,'WUPP' from dual union all
select 63449,'BU' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 63451,'BU' from dual union all
select 63452,'BU' from dual union all
select 63456,'NRL' from dual union all
select 63457,'BU' from dual union all
select 63458,'SDIO' from dual union all
select 6346,'LARC' from dual union all
select 63466,'USCLA' from dual union all
select 63467,'90MW' from dual union all
select 63468,'91MW' from dual union all
select 63474,'GSFC' from dual union all
select 63474,'NDRE' from dual union all
select 63476,'AERO' from dual union all
select 63478,'MITLL' from dual union all
select 6348,'LARC' from dual union all
select 63489,'91MW' from dual union all
select 63490,'MBB' from dual union all
select 63493,'NRL' from dual union all
select 63494,'NRL' from dual union all
select 63495,'MBB' from dual union all
select 63495,'SSC' from dual union all
select 63496,'INPE' from dual union all
select 6350,'GSFC' from dual union all
select 63506,'INPE' from dual union all
select 63509,'NRL' from dual union all
select 63514,'INTA' from dual union all
select 63515,'INTA' from dual union all
select 63516,'INTA' from dual union all
select 63529,'MPAE' from dual union all
select 63533,'INTA' from dual union all
select 63534,'INTA' from dual union all
select 63535,'INTA' from dual union all
select 63545,'UBC' from dual union all
select 63546,'BONN' from dual union all
select 63546,'OSLO' from dual union all
select 63547,'PITT' from dual union all
select 63548,'MPIK' from dual union all
select 63549,'PRL' from dual union all
select 63549,'TSAO' from dual union all
select 63550,'ISAS' from dual union all
select 63550,'TOK' from dual union all
select 63551,'AFCRL' from dual union all
select 63552,'UMN' from dual union all
select 63553,'321SMW' from dual union all
select 63553,'90SMW' from dual union all
select 63557,'BONN' from dual union all
select 63557,'OSLO' from dual union all
select 63558,'STO' from dual union all
select 63559,'INPE' from dual union all
select 63560,'PRL' from dual union all
select 63560,'TSAO' from dual union all
select 63561,'NAG' from dual union all
select 63562,'UNH' from dual union all
select 63563,'BONN' from dual union all
select 63563,'OSLO' from dual union all
select 63564,'STO' from dual union all
select 63565,'ISAS' from dual union all
select 63566,'PRL' from dual union all
select 63566,'TSAO' from dual union all
select 63567,'STO' from dual union all
select 63570,'BONN' from dual union all
select 63570,'OSLO' from dual union all
select 63571,'BONN' from dual union all
select 63571,'OSLO' from dual union all
select 63572,'WUPP' from dual union all
select 63573,'MPAE' from dual union all
select 63574,'90MW' from dual union all
select 63575,'BONN' from dual union all
select 63575,'OSLO' from dual union all
select 63576,'BONN' from dual union all
select 63576,'OSLO' from dual union all
select 63577,'GSFC' from dual union all
select 63578,'BONN' from dual union all
select 63578,'OSLO' from dual union all
select 63580,'BONN' from dual union all
select 63580,'OSLO' from dual union all
select 63581,'PSU' from dual union all
select 63582,'91MW' from dual union all
select 63583,'UCB' from dual union all
select 63584,'HOUS' from dual union all
select 63585,'91MW' from dual union all
select 63590,'MITLL' from dual union all
select 63593,'SSC' from dual union all
select 63594,'NRL' from dual union all
select 63595,'TRW' from dual union all
select 63596,'UCO' from dual union all
select 63597,'UCB' from dual union all
select 63598,'JHU' from dual union all
select 63599,'PITT' from dual union all
select 63599,'SDIO' from dual union all
select 63600,'AFCRL' from dual union all
select 63601,'UCO' from dual union all
select 63604,'MBB' from dual union all
select 63605,'GSFC' from dual union all
select 63606,'PSU' from dual union all
select 63607,'GSFC' from dual union all
select 63608,'MBB' from dual union all
select 63610,'90MW' from dual union all
select 63611,'UAH' from dual union all
select 63615,'SAAB' from dual union all
select 63617,'GMS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 63618,'GMS' from dual union all
select 63620,'GMS' from dual union all
select 63621,'GMS' from dual union all
select 63622,'GMS' from dual union all
select 63623,'GMS' from dual union all
select 63624,'GMS' from dual union all
select 63625,'GMS' from dual union all
select 63630,'GMS' from dual union all
select 63636,'JMA' from dual union all
select 63638,'GMS' from dual union all
select 63643,'GMS' from dual union all
select 63649,'GMS' from dual union all
select 63655,'JMA' from dual union all
select 63656,'GMS' from dual union all
select 63659,'JMA' from dual union all
select 63660,'IAP' from dual union all
select 63663,'GMS' from dual union all
select 63665,'IAP' from dual union all
select 63670,'GMS' from dual union all
select 63671,'IAP' from dual union all
select 63672,'IAP' from dual union all
select 63673,'IAP' from dual union all
select 63674,'IAP' from dual union all
select 63675,'IAP' from dual union all
select 63676,'IAP' from dual union all
select 63681,'LINAO' from dual union all
select 63687,'ISAS' from dual union all
select 63687,'TSUK' from dual union all
select 63689,'IAP' from dual union all
select 63694,'IAP' from dual union all
select 63705,'IAP' from dual union all
select 63706,'IAP' from dual union all
select 63708,'IAP' from dual union all
select 63710,'GMS' from dual union all
select 63711,'IAP' from dual union all
select 63712,'GMS' from dual union all
select 63713,'GMS' from dual union all
select 63716,'IAP' from dual union all
select 63719,'IAP' from dual union all
select 63720,'IAP' from dual union all
select 63721,'IAP' from dual union all
select 63722,'IAP' from dual union all
select 63724,'GMS' from dual union all
select 63726,'ISAS' from dual union all
select 63727,'JMA' from dual union all
select 63728,'JMA' from dual union all
select 63729,'JMA' from dual union all
select 63730,'JMA' from dual union all
select 63731,'JMA' from dual union all
select 63732,'JMA' from dual union all
select 63733,'JMA' from dual union all
select 63734,'JMA' from dual union all
select 63735,'JMA' from dual union all
select 63736,'GMS' from dual union all
select 63737,'JMA' from dual union all
select 63738,'JMA' from dual union all
select 63739,'JMA' from dual union all
select 63740,'JMA' from dual union all
select 63741,'JMA' from dual union all
select 63742,'JMA' from dual union all
select 63743,'JMA' from dual union all
select 63744,'JMA' from dual union all
select 63745,'JMA' from dual union all
select 63746,'JMA' from dual union all
select 63747,'GMS' from dual union all
select 63748,'JMA' from dual union all
select 63749,'JMA' from dual union all
select 6375,'GCA' from dual union all
select 63750,'JMA' from dual union all
select 63751,'JMA' from dual union all
select 63752,'JMA' from dual union all
select 63753,'JMA' from dual union all
select 63754,'JMA' from dual union all
select 63755,'JMA' from dual union all
select 63756,'JMA' from dual union all
select 63757,'WUPP' from dual union all
select 63758,'GMS' from dual union all
select 63759,'GMS' from dual union all
select 63761,'GMS' from dual union all
select 63762,'GMS' from dual union all
select 63769,'GMS' from dual union all
select 63771,'LINAO' from dual union all
select 63774,'GMS' from dual union all
select 63775,'GMS' from dual union all
select 63777,'GMS' from dual union all
select 63779,'JMA' from dual union all
select 63780,'GMS' from dual union all
select 63782,'GMS' from dual union all
select 63786,'GMS' from dual union all
select 63787,'GMS' from dual union all
select 63789,'GMS' from dual union all
select 63790,'GMS' from dual union all
select 63791,'JMA' from dual union all
select 63794,'GMS' from dual union all
select 63795,'GMS' from dual union all
select 638,'JPL' from dual union all
select 63805,'JMA' from dual union all
select 63810,'GMS' from dual union all
select 63811,'GMS' from dual union all
select 63812,'JMA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 63813,'GMS' from dual union all
select 63816,'LINAO' from dual union all
select 63819,'GMS' from dual union all
select 63821,'JMA' from dual union all
select 63822,'ISAS' from dual union all
select 63825,'GMS' from dual union all
select 63832,'GMS' from dual union all
select 63834,'JMA' from dual union all
select 63835,'GMS' from dual union all
select 63846,'MPAE' from dual union all
select 63847,'CEL' from dual union all
select 63847,'WFF' from dual union all
select 63848,'WFF' from dual union all
select 63848,'WUPP' from dual union all
select 63849,'INTA' from dual union all
select 63850,'WFF' from dual union all
select 63851,'WFF' from dual union all
select 63851,'WUPP' from dual union all
select 63853,'ISAS' from dual union all
select 63855,'MPAE' from dual union all
select 63857,'CEL' from dual union all
select 63857,'WFF' from dual union all
select 63858,'WFF' from dual union all
select 63858,'WUPP' from dual union all
select 63859,'INTA' from dual union all
select 63860,'WFF' from dual union all
select 63868,'WFF' from dual union all
select 63868,'WUPP' from dual union all
select 63869,'BONN' from dual union all
select 63869,'MPAE' from dual union all
select 63871,'WFF' from dual union all
select 63871,'WUPP' from dual union all
select 63872,'MPAE' from dual union all
select 63873,'CEL' from dual union all
select 63873,'WFF' from dual union all
select 63874,'INTA' from dual union all
select 63875,'WFF' from dual union all
select 63875,'WUPP' from dual union all
select 63876,'WFF' from dual union all
select 63879,'WFF' from dual union all
select 63879,'WUPP' from dual union all
select 63883,'WFF' from dual union all
select 63883,'WUPP' from dual union all
select 63884,'INTA' from dual union all
select 63885,'WFF' from dual union all
select 6389,'LARC' from dual union all
select 63892,'WFF' from dual union all
select 63892,'WUPP' from dual union all
select 63893,'WFF' from dual union all
select 63893,'WUPP' from dual union all
select 63894,'INTA' from dual union all
select 63895,'WFF' from dual union all
select 63897,'ISAS' from dual union all
select 63898,'WFF' from dual union all
select 63898,'WUPP' from dual union all
select 63899,'BONN' from dual union all
select 63899,'MPAE' from dual union all
select 639,'JPL' from dual union all
select 63901,'WFF' from dual union all
select 63901,'WUPP' from dual union all
select 63905,'WFF' from dual union all
select 63905,'WUPP' from dual union all
select 63906,'INTA' from dual union all
select 63907,'MPAE' from dual union all
select 63908,'CEL' from dual union all
select 63908,'WFF' from dual union all
select 63909,'WFF' from dual union all
select 63910,'MPAE' from dual union all
select 63912,'CEL' from dual union all
select 63912,'WFF' from dual union all
select 63913,'MPAE' from dual union all
select 63914,'MPAE' from dual union all
select 63915,'CEL' from dual union all
select 63915,'WFF' from dual union all
select 63916,'MPAE' from dual union all
select 63921,'WFF' from dual union all
select 63921,'WUPP' from dual union all
select 63923,'MPAE' from dual union all
select 63924,'CEL' from dual union all
select 63924,'WFF' from dual union all
select 63925,'WFF' from dual union all
select 63925,'WUPP' from dual union all
select 63926,'INTA' from dual union all
select 63927,'WFF' from dual union all
select 63929,'ISAS' from dual union all
select 63930,'WFF' from dual union all
select 63930,'WUPP' from dual union all
select 63931,'BONN' from dual union all
select 63931,'MPAE' from dual union all
select 63932,'WFF' from dual union all
select 63932,'WUPP' from dual union all
select 63934,'BONN' from dual union all
select 63934,'MPAE' from dual union all
select 63935,'WFF' from dual union all
select 63935,'WUPP' from dual union all
select 63937,'ISAS' from dual union all
select 63938,'WFF' from dual union all
select 63938,'WUPP' from dual union all
select 63939,'INTA' from dual union all
select 63940,'WFF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 63946,'WFF' from dual union all
select 63946,'WUPP' from dual union all
select 63947,'MPAE' from dual union all
select 63948,'CEL' from dual union all
select 63948,'WFF' from dual union all
select 63949,'WFF' from dual union all
select 63949,'WUPP' from dual union all
select 63950,'INTA' from dual union all
select 63951,'WFF' from dual union all
select 63953,'WFF' from dual union all
select 63953,'WUPP' from dual union all
select 63955,'WFF' from dual union all
select 63955,'WUPP' from dual union all
select 63956,'WFF' from dual union all
select 63956,'WUPP' from dual union all
select 63957,'INTA' from dual union all
select 63958,'BONN' from dual union all
select 63958,'MPAE' from dual union all
select 63959,'WFF' from dual union all
select 63959,'WUPP' from dual union all
select 63960,'BONN' from dual union all
select 63960,'MPAE' from dual union all
select 63961,'WFF' from dual union all
select 63961,'WUPP' from dual union all
select 63962,'WFF' from dual union all
select 63963,'CEL' from dual union all
select 63963,'WFF' from dual union all
select 63964,'MPAE' from dual union all
select 63965,'CEL' from dual union all
select 63965,'WFF' from dual union all
select 63966,'WFF' from dual union all
select 63966,'WUPP' from dual union all
select 63967,'CEL' from dual union all
select 63967,'WFF' from dual union all
select 63968,'MPAE' from dual union all
select 63969,'MPAE' from dual union all
select 63972,'WFF' from dual union all
select 63972,'WUPP' from dual union all
select 63973,'BONN' from dual union all
select 63973,'MPAE' from dual union all
select 63974,'BONN' from dual union all
select 63974,'MPAE' from dual union all
select 63975,'WFF' from dual union all
select 63975,'WUPP' from dual union all
select 63977,'WFF' from dual union all
select 63977,'WUPP' from dual union all
select 63978,'MPAE' from dual union all
select 63979,'WFF' from dual union all
select 63979,'WUPP' from dual union all
select 63980,'MPAE' from dual union all
select 63981,'INTA' from dual union all
select 63982,'BONN' from dual union all
select 63982,'MPAE' from dual union all
select 63983,'BONN' from dual union all
select 63983,'MPAE' from dual union all
select 63984,'WFF' from dual union all
select 63984,'WUPP' from dual union all
select 63985,'BONN' from dual union all
select 63985,'MPAE' from dual union all
select 63986,'BONN' from dual union all
select 63986,'MPAE' from dual union all
select 63987,'WFF' from dual union all
select 63987,'WUPP' from dual union all
select 63988,'WFF' from dual union all
select 63988,'WUPP' from dual union all
select 63989,'BONN' from dual union all
select 63989,'MPAE' from dual union all
select 63990,'WFF' from dual union all
select 63991,'WFF' from dual union all
select 63991,'WUPP' from dual union all
select 63992,'WFF' from dual union all
select 63992,'WUPP' from dual union all
select 63993,'BONN' from dual union all
select 63993,'MPAE' from dual union all
select 63994,'WFF' from dual union all
select 63994,'WUPP' from dual union all
select 63995,'BONN' from dual union all
select 63995,'MPAE' from dual union all
select 63996,'WFF' from dual union all
select 63996,'WUPP' from dual union all
select 63997,'BONN' from dual union all
select 63997,'MPAE' from dual union all
select 63998,'BONN' from dual union all
select 63998,'MPAE' from dual union all
select 63999,'WFF' from dual union all
select 63999,'WUPP' from dual union all
select 640,'JPL' from dual union all
select 64002,'WFF' from dual union all
select 64002,'WUPP' from dual union all
select 64003,'INTA' from dual union all
select 64005,'CEL' from dual union all
select 64005,'WFF' from dual union all
select 64006,'MPAE' from dual union all
select 64007,'CEL' from dual union all
select 64007,'WFF' from dual union all
select 64008,'WFF' from dual union all
select 64009,'MPAE' from dual union all
select 64011,'CEL' from dual union all
select 64011,'WFF' from dual union all
select 64012,'MPAE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 64013,'CEL' from dual union all
select 64013,'WFF' from dual union all
select 64015,'WFF' from dual union all
select 64015,'WUPP' from dual union all
select 64016,'CEL' from dual union all
select 64016,'WFF' from dual union all
select 64017,'INTA' from dual union all
select 64018,'WFF' from dual union all
select 64021,'MPAE' from dual union all
select 6404,'GSFC' from dual union all
select 64054,'CEL' from dual union all
select 64054,'WFF' from dual union all
select 64065,'MPAE' from dual union all
select 64076,'WFF' from dual union all
select 64076,'WUPP' from dual union all
select 64087,'INTA' from dual union all
select 64095,'ISAS' from dual union all
select 64096,'ISAS' from dual union all
select 64097,'WUPP' from dual union all
select 64098,'INTA' from dual union all
select 64099,'WUPP' from dual union all
select 641,'JPL' from dual union all
select 64100,'WUPP' from dual union all
select 64101,'WUPP' from dual union all
select 64102,'WUPP' from dual union all
select 64103,'WUPP' from dual union all
select 64104,'WFF' from dual union all
select 64109,'ISAS' from dual union all
select 64112,'WFF' from dual union all
select 64112,'WUPP' from dual union all
select 64113,'BONN' from dual union all
select 64113,'MPAE' from dual union all
select 64114,'MPAE' from dual union all
select 64115,'CEL' from dual union all
select 64115,'WFF' from dual union all
select 64116,'WFF' from dual union all
select 64116,'WUPP' from dual union all
select 64118,'INTA' from dual union all
select 64119,'INTA' from dual union all
select 64120,'WFF' from dual union all
select 64124,'WFF' from dual union all
select 64124,'WUPP' from dual union all
select 64125,'WFF' from dual union all
select 64125,'WUPP' from dual union all
select 64127,'BONN' from dual union all
select 64127,'MPAE' from dual union all
select 64128,'WFF' from dual union all
select 64128,'WUPP' from dual union all
select 64130,'CEL' from dual union all
select 64130,'WFF' from dual union all
select 64131,'WFF' from dual union all
select 64131,'WUPP' from dual union all
select 64132,'WFF' from dual union all
select 64132,'WUPP' from dual union all
select 64133,'WFF' from dual union all
select 64135,'INTA' from dual union all
select 64144,'WFF' from dual union all
select 64144,'WUPP' from dual union all
select 64146,'WFF' from dual union all
select 64146,'WUPP' from dual union all
select 64148,'INTA' from dual union all
select 64149,'WFF' from dual union all
select 64150,'WFF' from dual union all
select 64150,'WUPP' from dual union all
select 64151,'BONN' from dual union all
select 64151,'MPAE' from dual union all
select 64152,'WFF' from dual union all
select 64152,'WUPP' from dual union all
select 64153,'ISAS' from dual union all
select 64156,'WFF' from dual union all
select 64156,'WUPP' from dual union all
select 64157,'WFF' from dual union all
select 64158,'INTA' from dual union all
select 64162,'ISAS' from dual union all
select 64167,'WFF' from dual union all
select 64167,'WUPP' from dual union all
select 64168,'BONN' from dual union all
select 64168,'MPAE' from dual union all
select 642,'JPL' from dual union all
select 6421,'GSFC' from dual union all
select 6424,'NSA' from dual union all
select 6425,'NOTS' from dual union all
select 64258,'WFF' from dual union all
select 64260,'UAHCM' from dual union all
select 64261,'NAIC' from dual union all
select 64261,'SDIO' from dual union all
select 64264,'IAP' from dual union all
select 64286,'INPE' from dual union all
select 64288,'PITT' from dual union all
select 64289,'UMI' from dual union all
select 64290,'GSFC' from dual union all
select 64291,'ESTEC' from dual union all
select 64291,'SSC' from dual union all
select 64295,'STAN' from dual union all
select 64297,'LPARL' from dual union all
select 64299,'90MW' from dual union all
select 643,'PRIN' from dual union all
select 64302,'321SMW' from dual union all
select 64302,'90SMW' from dual union all
select 64310,'90SMW' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 64311,'SAO' from dual union all
select 64321,'GSFC' from dual union all
select 64322,'PSU' from dual union all
select 64323,'IAP' from dual union all
select 64324,'GSFC' from dual union all
select 64333,'91MW' from dual union all
select 64334,'LANL' from dual union all
select 64335,'UMI' from dual union all
select 64336,'LANL' from dual union all
select 64338,'NASDA' from dual union all
select 64339,'90MW' from dual union all
select 64342,'IAP' from dual union all
select 64343,'IAP' from dual union all
select 64344,'IAP' from dual union all
select 64349,'IAP' from dual union all
select 64352,'IAP' from dual union all
select 64353,'IAP' from dual union all
select 64355,'NAIC' from dual union all
select 64355,'SDIO' from dual union all
select 64358,'90MW' from dual union all
select 64364,'UAH' from dual union all
select 64366,'MBB' from dual union all
select 64371,'BU' from dual union all
select 64372,'BU' from dual union all
select 64374,'INPE' from dual union all
select 64374,'SASK' from dual union all
select 64376,'STO' from dual union all
select 64377,'STO' from dual union all
select 64378,'IAP' from dual union all
select 64379,'PSU' from dual union all
select 64380,'INTA' from dual union all
select 64381,'INTA' from dual union all
select 64382,'INTA' from dual union all
select 64392,'IAP' from dual union all
select 64393,'IAP' from dual union all
select 64394,'IAP' from dual union all
select 64396,'IAP' from dual union all
select 64397,'IAP' from dual union all
select 64398,'IAP' from dual union all
select 64399,'IAP' from dual union all
select 644,'JPL' from dual union all
select 64400,'IAP' from dual union all
select 64401,'IAP' from dual union all
select 64402,'IAP' from dual union all
select 64403,'IAP' from dual union all
select 64404,'IAP' from dual union all
select 64405,'IAP' from dual union all
select 64407,'IAP' from dual union all
select 64408,'IAP' from dual union all
select 64409,'IAP' from dual union all
select 64410,'IAP' from dual union all
select 64411,'IAP' from dual union all
select 64412,'INTA' from dual union all
select 6442,'LARC' from dual union all
select 64423,'CSSAR' from dual union all
select 64440,'JHU' from dual union all
select 64451,'MPIK' from dual union all
select 64452,'NRL' from dual union all
select 64458,'ISAS' from dual union all
select 64458,'TOKAI' from dual union all
select 64459,'CORN' from dual union all
select 64463,'ISAS' from dual union all
select 64463,'OSAKA' from dual union all
select 64464,'SDIO' from dual union all
select 64469,'SAO' from dual union all
select 64478,'90MW' from dual union all
select 64479,'UCO' from dual union all
select 64482,'GSFC' from dual union all
select 64483,'UCO' from dual union all
select 64485,'MPAE' from dual union all
select 64486,'WUPP' from dual union all
select 64487,'MITLL' from dual union all
select 64487,'SDIO' from dual union all
select 64489,'LLNL' from dual union all
select 64489,'SDIO' from dual union all
select 6449,'GSFC' from dual union all
select 64491,'PITT' from dual union all
select 645,'JPL' from dual union all
select 64506,'ISAS' from dual union all
select 64506,'TSUK' from dual union all
select 64510,'ISAS' from dual union all
select 64510,'TSUK' from dual union all
select 64537,'JMA' from dual union all
select 64538,'JMA' from dual union all
select 64539,'JMA' from dual union all
select 64540,'JMA' from dual union all
select 64541,'JMA' from dual union all
select 64542,'JMA' from dual union all
select 64543,'JMA' from dual union all
select 64544,'JMA' from dual union all
select 64546,'JMA' from dual union all
select 64547,'JMA' from dual union all
select 64548,'JMA' from dual union all
select 64549,'JMA' from dual union all
select 64550,'JMA' from dual union all
select 64551,'JMA' from dual union all
select 64552,'JMA' from dual union all
select 64553,'JMA' from dual union all
select 64554,'JMA' from dual union all
select 64555,'JMA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 64557,'JMA' from dual union all
select 64558,'JMA' from dual union all
select 64559,'JMA' from dual union all
select 64560,'JMA' from dual union all
select 64561,'JMA' from dual union all
select 64562,'JMA' from dual union all
select 64563,'JMA' from dual union all
select 64564,'JMA' from dual union all
select 64565,'JMA' from dual union all
select 64566,'JMA' from dual union all
select 64568,'JMA' from dual union all
select 64569,'JMA' from dual union all
select 64570,'JMA' from dual union all
select 64571,'JMA' from dual union all
select 64572,'JMA' from dual union all
select 64573,'JMA' from dual union all
select 64574,'JMA' from dual union all
select 64575,'JMA' from dual union all
select 64576,'JMA' from dual union all
select 64577,'JMA' from dual union all
select 64579,'WUPP' from dual union all
select 64591,'ISAS' from dual union all
select 64591,'TSUK' from dual union all
select 64592,'ISAS' from dual union all
select 64592,'TSUK' from dual union all
select 646,'JPL' from dual union all
select 6462,'GCA' from dual union all
select 64682,'FIT' from dual union all
select 64682,'UNAM' from dual union all
select 64683,'USU' from dual union all
select 64684,'WFF' from dual union all
select 64685,'MPAE' from dual union all
select 64686,'WFF' from dual union all
select 64688,'USU' from dual union all
select 64689,'WFF' from dual union all
select 64690,'USU' from dual union all
select 64691,'WFF' from dual union all
select 64692,'MPAE' from dual union all
select 647,'JPL' from dual union all
select 64705,'WFF' from dual union all
select 64706,'WFF' from dual union all
select 64707,'WFF' from dual union all
select 64708,'WFF' from dual union all
select 64709,'WUPP' from dual union all
select 64710,'WUPP' from dual union all
select 64711,'WUPP' from dual union all
select 64712,'WUPP' from dual union all
select 6474,'LARC' from dual union all
select 6478,'KOBE' from dual union all
select 648,'JPL' from dual union all
select 6480,'AFCRL' from dual union all
select 6481,'GSFC' from dual union all
select 64810,'IAP' from dual union all
select 64811,'ASI' from dual union all
select 64812,'IAP' from dual union all
select 64814,'IAP' from dual union all
select 64821,'USN' from dual union all
select 64822,'SDIO' from dual union all
select 64823,'INPE' from dual union all
select 6483,'BRL' from dual union all
select 64830,'WFF' from dual union all
select 64831,'ISRO' from dual union all
select 64832,'ISRO' from dual union all
select 64844,'DGA' from dual union all
select 64844,'MATRA' from dual union all
select 64845,'PITT' from dual union all
select 64846,'JHU' from dual union all
select 64847,'NRL' from dual union all
select 64850,'INTA' from dual union all
select 64851,'INTA' from dual union all
select 64852,'UCO' from dual union all
select 64854,'PITT' from dual union all
select 64855,'ISAS' from dual union all
select 64856,'UCO' from dual union all
select 64857,'NAG' from dual union all
select 64858,'SDIO' from dual union all
select 6486,'LARC' from dual union all
select 64863,'UCO' from dual union all
select 64864,'CLEM' from dual union all
select 64865,'AERO' from dual union all
select 64866,'CLEM' from dual union all
select 64867,'90MW' from dual union all
select 64868,'CLEM' from dual union all
select 64869,'PITT' from dual union all
select 64872,'PITT' from dual union all
select 64873,'CSA' from dual union all
select 64874,'USU' from dual union all
select 64875,'AFGL' from dual union all
select 64875,'USU' from dual union all
select 64878,'ESTEC' from dual union all
select 64878,'SSC' from dual union all
select 64879,'MITLL' from dual union all
select 64879,'SDIO' from dual union all
select 64882,'LPARL' from dual union all
select 64883,'PITT' from dual union all
select 64884,'SDIO' from dual union all
select 64885,'GSFC' from dual union all
select 64886,'PITT' from dual union all
select 64887,'NRL' from dual union all
select 64888,'INPE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 64889,'341SMW' from dual union all
select 64889,'564SMS' from dual union all
select 64890,'SAIC' from dual union all
select 64891,'CORN' from dual union all
select 64897,'SDIO' from dual union all
select 64898,'GSFC' from dual union all
select 649,'JPL' from dual union all
select 64901,'90MW' from dual union all
select 64902,'AFGL' from dual union all
select 64903,'PRL' from dual union all
select 64904,'AFGL' from dual union all
select 64905,'AERO' from dual union all
select 64906,'UMI' from dual union all
select 64914,'DLR' from dual union all
select 64915,'NASDA' from dual union all
select 64916,'HOUS' from dual union all
select 64917,'NRL' from dual union all
select 64918,'PITT' from dual union all
select 64919,'WFF' from dual union all
select 64920,'PITT' from dual union all
select 64925,'UAH' from dual union all
select 64926,'90MW' from dual union all
select 64927,'321SMW' from dual union all
select 64927,'90SMW' from dual union all
select 64928,'SDIO' from dual union all
select 64931,'UCO' from dual union all
select 64932,'91MW' from dual union all
select 64933,'ESTEC' from dual union all
select 64933,'SSC' from dual union all
select 64938,'ASTD' from dual union all
select 64939,'PITT' from dual union all
select 64942,'JMA' from dual union all
select 64944,'JMA' from dual union all
select 64946,'JMA' from dual union all
select 64953,'JMA' from dual union all
select 64954,'JMA' from dual union all
select 64957,'JMA' from dual union all
select 64959,'JMA' from dual union all
select 6496,'LARC' from dual union all
select 64961,'IAP' from dual union all
select 64962,'JMA' from dual union all
select 64963,'JMA' from dual union all
select 64966,'JMA' from dual union all
select 64967,'JMA' from dual union all
select 64968,'JMA' from dual union all
select 64969,'JMA' from dual union all
select 64970,'JMA' from dual union all
select 64971,'JMA' from dual union all
select 64973,'JMA' from dual union all
select 64974,'JMA' from dual union all
select 64975,'JMA' from dual union all
select 64976,'JMA' from dual union all
select 64978,'JMA' from dual union all
select 64979,'IAP' from dual union all
select 64980,'IAP' from dual union all
select 64983,'JMA' from dual union all
select 64984,'IAP' from dual union all
select 64985,'IAP' from dual union all
select 64986,'IAP' from dual union all
select 64987,'IAP' from dual union all
select 64988,'IAP' from dual union all
select 64989,'IAP' from dual union all
select 64992,'JMA' from dual union all
select 64994,'IAP' from dual union all
select 64996,'JMA' from dual union all
select 64997,'IAP' from dual union all
select 64998,'IAP' from dual union all
select 64999,'IAP' from dual union all
select 650,'JPL' from dual union all
select 6500,'LARC' from dual union all
select 65000,'IAP' from dual union all
select 65003,'IAP' from dual union all
select 65007,'IAP' from dual union all
select 65008,'JMA' from dual union all
select 65009,'JMA' from dual union all
select 65010,'DLR' from dual union all
select 65011,'JMA' from dual union all
select 65016,'JMA' from dual union all
select 65018,'DLR' from dual union all
select 65021,'JMA' from dual union all
select 65022,'JMA' from dual union all
select 65027,'JMA' from dual union all
select 65037,'JMA' from dual union all
select 65040,'JMA' from dual union all
select 65041,'USU' from dual union all
select 65042,'USU' from dual union all
select 65047,'JMA' from dual union all
select 65049,'JMA' from dual union all
select 65054,'JMA' from dual union all
select 65055,'TSUK' from dual union all
select 65058,'JMA' from dual union all
select 6506,'LARC' from dual union all
select 65060,'KSC' from dual union all
select 6509,'NOTS' from dual union all
select 65095,'SPFL' from dual union all
select 651,'JPL' from dual union all
select 65100,'TSAO' from dual union all
select 65101,'TSAO' from dual union all
select 6514,'WSMR' from dual union all
select 65183,'UNH' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 65184,'MITLL' from dual union all
select 65187,'KARI' from dual union all
select 65190,'MDAC' from dual union all
select 65191,'LPARL' from dual union all
select 65192,'KARI' from dual union all
select 65193,'MDAC' from dual union all
select 65194,'USN' from dual union all
select 65195,'MDAC' from dual union all
select 652,'JPL' from dual union all
select 65204,'ISRO' from dual union all
select 65205,'ISRO' from dual union all
select 65206,'ISRO' from dual union all
select 65207,'ISRO' from dual union all
select 65208,'ISRO' from dual union all
select 65209,'ISRO' from dual union all
select 65212,'PRL' from dual union all
select 65213,'PRL' from dual union all
select 65220,'MITLL' from dual union all
select 65223,'MSFC' from dual union all
select 65224,'ISAS' from dual union all
select 65225,'UAH' from dual union all
select 65228,'MPIK' from dual union all
select 65229,'90MW' from dual union all
select 65230,'PITT' from dual union all
select 65231,'321SMW' from dual union all
select 65231,'90SMW' from dual union all
select 65232,'PITT' from dual union all
select 65233,'USU' from dual union all
select 65234,'INPE' from dual union all
select 65235,'UCB' from dual union all
select 65236,'INPE' from dual union all
select 65237,'SAO' from dual union all
select 65238,'UCO' from dual union all
select 65239,'FREIB' from dual union all
select 65239,'PRL' from dual union all
select 65240,'ASTD' from dual union all
select 65241,'UWA' from dual union all
select 65242,'PITT' from dual union all
select 65243,'PITT' from dual union all
select 65244,'BMDO' from dual union all
select 65244,'NAIC' from dual union all
select 65245,'AESPC' from dual union all
select 65246,'BMDO' from dual union all
select 65246,'NAIC' from dual union all
select 65252,'UMI' from dual union all
select 65253,'BMDO' from dual union all
select 65254,'91MW' from dual union all
select 65257,'90MW' from dual union all
select 65258,'CORN' from dual union all
select 65260,'DGA' from dual union all
select 65260,'MATRA' from dual union all
select 65261,'IAP' from dual union all
select 65262,'STO' from dual union all
select 65263,'STO' from dual union all
select 65264,'GSFC' from dual union all
select 65268,'PITT' from dual union all
select 65269,'PITT' from dual union all
select 65270,'90MW' from dual union all
select 65271,'NASDA' from dual union all
select 65272,'HAO' from dual union all
select 65272,'UCO' from dual union all
select 65273,'UCO' from dual union all
select 65275,'IAA' from dual union all
select 65275,'INTA' from dual union all
select 65278,'SSC' from dual union all
select 65279,'INPE' from dual union all
select 65281,'ASTD' from dual union all
select 65282,'DASARI' from dual union all
select 65287,'PRL' from dual union all
select 65288,'PRL' from dual union all
select 65290,'JMA' from dual union all
select 65292,'JMA' from dual union all
select 65293,'JMA' from dual union all
select 65295,'JMA' from dual union all
select 65296,'JMA' from dual union all
select 65297,'JMA' from dual union all
select 65298,'JMA' from dual union all
select 653,'JPL' from dual union all
select 65301,'JMA' from dual union all
select 65306,'JMA' from dual union all
select 65307,'JMA' from dual union all
select 65308,'JMA' from dual union all
select 65310,'JMA' from dual union all
select 65312,'JMA' from dual union all
select 65314,'JMA' from dual union all
select 65317,'JMA' from dual union all
select 65318,'JMA' from dual union all
select 65321,'JMA' from dual union all
select 65324,'JMA' from dual union all
select 65327,'JMA' from dual union all
select 65329,'JMA' from dual union all
select 65331,'JMA' from dual union all
select 65334,'JMA' from dual union all
select 65335,'JMA' from dual union all
select 65336,'TSUK' from dual union all
select 65338,'JMA' from dual union all
select 65340,'JMA' from dual union all
select 65341,'JMA' from dual union all
select 65342,'JMA' from dual union all
select 65343,'JMA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 65346,'JMA' from dual union all
select 65347,'JMA' from dual union all
select 65348,'JMA' from dual union all
select 65349,'JMA' from dual union all
select 65350,'TSUK' from dual union all
select 65351,'JMA' from dual union all
select 65352,'JMA' from dual union all
select 65353,'JMA' from dual union all
select 65391,'WFF' from dual union all
select 65392,'WFF' from dual union all
select 65393,'WFF' from dual union all
select 65394,'WFF' from dual union all
select 65395,'WFF' from dual union all
select 65396,'WFF' from dual union all
select 654,'APL' from dual union all
select 6543,'ISAS' from dual union all
select 6544,'ISAS' from dual union all
select 6547,'LARC' from dual union all
select 65487,'JMA' from dual union all
select 65491,'MDAC' from dual union all
select 65492,'MDAC' from dual union all
select 65497,'STO' from dual union all
select 65497,'TSAO' from dual union all
select 655,'APL' from dual union all
select 65505,'UMI' from dual union all
select 65506,'321SMW' from dual union all
select 65506,'90SMW' from dual union all
select 65507,'BERG' from dual union all
select 65511,'NRCC' from dual union all
select 65512,'CLEM' from dual union all
select 65513,'CLEM' from dual union all
select 65514,'AERO' from dual union all
select 65516,'ISAS' from dual union all
select 65516,'RIKK' from dual union all
select 65516,'TOK' from dual union all
select 65517,'MPAE' from dual union all
select 65519,'DGA' from dual union all
select 65519,'MATRA' from dual union all
select 65520,'UNH' from dual union all
select 65521,'HOUS' from dual union all
select 65522,'90MW' from dual union all
select 65523,'NRL' from dual union all
select 65528,'WFF' from dual union all
select 65529,'IAA' from dual union all
select 65529,'INTA' from dual union all
select 65530,'JHU' from dual union all
select 65531,'LPARL' from dual union all
select 65532,'DASARI' from dual union all
select 65533,'ASTD' from dual union all
select 65534,'90MW' from dual union all
select 65540,'INPE' from dual union all
select 65541,'PSU' from dual union all
select 65543,'UCO' from dual union all
select 65545,'91MW' from dual union all
select 65546,'BMDO' from dual union all
select 65547,'PSU' from dual union all
select 65548,'PSU' from dual union all
select 65549,'SWRIB' from dual union all
select 65551,'BONN' from dual union all
select 65551,'NDRE' from dual union all
select 65552,'BONN' from dual union all
select 65552,'NDRE' from dual union all
select 65553,'USN' from dual union all
select 65554,'BONN' from dual union all
select 65554,'NDRE' from dual union all
select 65555,'SWRIB' from dual union all
select 65556,'BMDO' from dual union all
select 65557,'GSFC' from dual union all
select 65558,'GSFC' from dual union all
select 65559,'COSG' from dual union all
select 65560,'GSFC' from dual union all
select 65561,'GSFC' from dual union all
select 65562,'90MW' from dual union all
select 65563,'GSFC' from dual union all
select 65564,'PITT' from dual union all
select 65566,'PITT' from dual union all
select 65567,'ISAS' from dual union all
select 65568,'GSFC' from dual union all
select 65570,'CLEM' from dual union all
select 65571,'CLEM' from dual union all
select 65572,'CLEM' from dual union all
select 65573,'CLEM' from dual union all
select 65574,'PITT' from dual union all
select 65575,'91MW' from dual union all
select 65576,'GSFC' from dual union all
select 65577,'DART' from dual union all
select 65578,'GSFC' from dual union all
select 6558,'LUX' from dual union all
select 6558,'RAE' from dual union all
select 6558,'UCW' from dual union all
select 65580,'HAO' from dual union all
select 65580,'UCO' from dual union all
select 65581,'STAN' from dual union all
select 65583,'LERC' from dual union all
select 65584,'ISAS' from dual union all
select 65584,'TOK' from dual union all
select 65585,'ASTD' from dual union all
select 65587,'CRLI' from dual union all
select 65587,'ISAS' from dual union all
select 65587,'TOHO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 65588,'UWI' from dual union all
select 65591,'QUK' from dual union all
select 65592,'UCO' from dual union all
select 656,'NRL' from dual union all
select 6560,'GCA' from dual union all
select 65600,'JMA' from dual union all
select 65601,'TSUK' from dual union all
select 65602,'TSUK' from dual union all
select 65603,'TSUK' from dual union all
select 65604,'TSUK' from dual union all
select 65605,'JMA' from dual union all
select 65606,'JMA' from dual union all
select 65607,'JMA' from dual union all
select 65608,'JMA' from dual union all
select 65609,'JMA' from dual union all
select 65610,'JMA' from dual union all
select 65611,'JMA' from dual union all
select 65612,'JMA' from dual union all
select 65613,'JMA' from dual union all
select 65614,'JMA' from dual union all
select 65615,'JMA' from dual union all
select 65616,'JMA' from dual union all
select 65617,'JMA' from dual union all
select 65618,'JMA' from dual union all
select 65619,'JMA' from dual union all
select 65620,'JMA' from dual union all
select 65621,'JMA' from dual union all
select 65622,'JMA' from dual union all
select 65623,'JMA' from dual union all
select 65624,'JMA' from dual union all
select 65625,'JMA' from dual union all
select 65626,'JMA' from dual union all
select 65627,'JMA' from dual union all
select 65628,'JMA' from dual union all
select 65629,'JMA' from dual union all
select 65630,'JMA' from dual union all
select 65631,'JMA' from dual union all
select 65632,'JMA' from dual union all
select 65633,'JMA' from dual union all
select 65634,'JMA' from dual union all
select 65635,'JMA' from dual union all
select 65636,'JMA' from dual union all
select 65637,'JMA' from dual union all
select 65638,'JMA' from dual union all
select 65639,'JMA' from dual union all
select 65640,'WFF' from dual union all
select 65641,'WFF' from dual union all
select 65642,'GSFC' from dual union all
select 65642,'UCO' from dual union all
select 65643,'GSFC' from dual union all
select 65643,'UCO' from dual union all
select 65644,'GSFC' from dual union all
select 65644,'UCO' from dual union all
select 65645,'GSFC' from dual union all
select 65645,'UCO' from dual union all
select 65646,'GSFC' from dual union all
select 65646,'UCO' from dual union all
select 65647,'GSFC' from dual union all
select 65647,'UCO' from dual union all
select 65648,'GSFC' from dual union all
select 65648,'UCO' from dual union all
select 65649,'GSFC' from dual union all
select 65649,'UCO' from dual union all
select 65650,'GSFC' from dual union all
select 65650,'UCO' from dual union all
select 65651,'GSFC' from dual union all
select 65651,'UCO' from dual union all
select 65652,'GSFC' from dual union all
select 65652,'UCO' from dual union all
select 65653,'GSFC' from dual union all
select 65653,'UCO' from dual union all
select 65654,'GSFC' from dual union all
select 65654,'UCO' from dual union all
select 65655,'GSFC' from dual union all
select 65655,'UCO' from dual union all
select 65656,'GSFC' from dual union all
select 65656,'UCO' from dual union all
select 65657,'GSFC' from dual union all
select 65657,'UCO' from dual union all
select 65658,'GSFC' from dual union all
select 65658,'UCO' from dual union all
select 65659,'GSFC' from dual union all
select 65659,'UCO' from dual union all
select 65660,'GSFC' from dual union all
select 65660,'UCO' from dual union all
select 65661,'GSFC' from dual union all
select 65661,'UCO' from dual union all
select 657,'NRL' from dual union all
select 65736,'USN' from dual union all
select 65737,'USN' from dual union all
select 65738,'MDAC' from dual union all
select 65739,'MDAC' from dual union all
select 6574,'ISAS' from dual union all
select 65740,'MDAC' from dual union all
select 65742,'ISAS' from dual union all
select 65748,'LAPAN' from dual union all
select 65749,'WFF' from dual union all
select 65753,'90MW' from dual union all
select 65754,'ISAS' from dual union all
select 65754,'NOBY' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 65755,'CORN' from dual union all
select 65756,'ISAS' from dual union all
select 65756,'NAG' from dual union all
select 65758,'AERO' from dual union all
select 65759,'CLEM' from dual union all
select 6576,'PMR' from dual union all
select 65763,'UNH' from dual union all
select 6577,'PMR' from dual union all
select 65770,'NAXS' from dual union all
select 65771,'91MW' from dual union all
select 65772,'PITT' from dual union all
select 65773,'PITT' from dual union all
select 65774,'UAF' from dual union all
select 65775,'PITT' from dual union all
select 65777,'UMI' from dual union all
select 6578,'PMR' from dual union all
select 65781,'SWRIB' from dual union all
select 65782,'NRL' from dual union all
select 65785,'DASARI' from dual union all
select 65786,'DASARI' from dual union all
select 65787,'GSFC' from dual union all
select 65788,'PSU' from dual union all
select 6579,'PMR' from dual union all
select 65790,'UCO' from dual union all
select 65793,'90MW' from dual union all
select 65794,'SMDC' from dual union all
select 65795,'SMDC' from dual union all
select 65797,'WFF' from dual union all
select 65798,'DLR' from dual union all
select 658,'NRL' from dual union all
select 6580,'PMR' from dual union all
select 65800,'USU' from dual union all
select 65802,'USU' from dual union all
select 65804,'ISRO' from dual union all
select 65806,'NASDA' from dual union all
select 65808,'LERC' from dual union all
select 65809,'90MW' from dual union all
select 6581,'PMR' from dual union all
select 65810,'CORN' from dual union all
select 65812,'USCLA' from dual union all
select 65814,'DLR' from dual union all
select 65818,'PSU' from dual union all
select 65819,'JHU' from dual union all
select 6582,'PMR' from dual union all
select 65822,'UCO' from dual union all
select 65823,'CRC' from dual union all
select 65825,'UCO' from dual union all
select 65826,'UWI' from dual union all
select 65827,'UCO' from dual union all
select 65828,'CLEM' from dual union all
select 65829,'CLEM' from dual union all
select 6583,'PMR' from dual union all
select 65830,'AERO' from dual union all
select 65831,'ESTEC' from dual union all
select 65831,'SSC' from dual union all
select 65832,'DLR' from dual union all
select 65832,'SSC' from dual union all
select 65833,'GSFC' from dual union all
select 65833,'UWI' from dual union all
select 65837,'INPE' from dual union all
select 65843,'JMA' from dual union all
select 65844,'JMA' from dual union all
select 65845,'JMA' from dual union all
select 65846,'DLR' from dual union all
select 65847,'DLR' from dual union all
select 65848,'DLR' from dual union all
select 65849,'DLR' from dual union all
select 6585,'GOYR' from dual union all
select 6585,'WADD' from dual union all
select 65850,'USU' from dual union all
select 65851,'USU' from dual union all
select 65852,'DLR' from dual union all
select 65853,'DLR' from dual union all
select 65854,'JMA' from dual union all
select 65855,'JMA' from dual union all
select 65856,'JMA' from dual union all
select 65857,'JMA' from dual union all
select 65858,'JMA' from dual union all
select 65859,'JMA' from dual union all
select 65860,'JMA' from dual union all
select 65861,'JMA' from dual union all
select 65862,'JMA' from dual union all
select 65863,'JMA' from dual union all
select 65864,'JMA' from dual union all
select 65865,'JMA' from dual union all
select 65866,'JMA' from dual union all
select 65867,'JMA' from dual union all
select 65868,'JMA' from dual union all
select 65869,'JMA' from dual union all
select 6587,'GOYR' from dual union all
select 6587,'WADD' from dual union all
select 65870,'JMA' from dual union all
select 65871,'JMA' from dual union all
select 65872,'JMA' from dual union all
select 65873,'JMA' from dual union all
select 65874,'JMA' from dual union all
select 65875,'JMA' from dual union all
select 65876,'JMA' from dual union all
select 65877,'JMA' from dual union all
select 65878,'JMA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 65879,'JMA' from dual union all
select 6588,'USN' from dual union all
select 65880,'JMA' from dual union all
select 65881,'JMA' from dual union all
select 65882,'JMA' from dual union all
select 65883,'JMA' from dual union all
select 65884,'JMA' from dual union all
select 65885,'JMA' from dual union all
select 65886,'JMA' from dual union all
select 6589,'NASA' from dual union all
select 659,'NRL' from dual union all
select 6590,'USN' from dual union all
select 6591,'WSMR' from dual union all
select 6592,'ATLA' from dual union all
select 6596,'USA' from dual union all
select 65960,'NRO' from dual union all
select 65963,'MDAC' from dual union all
select 65964,'MDAC' from dual union all
select 65965,'MDAC' from dual union all
select 65966,'MDAC' from dual union all
select 65969,'EAC' from dual union all
select 65970,'EAC' from dual union all
select 65971,'TROM' from dual union all
select 65972,'TROM' from dual union all
select 6598,'USA' from dual union all
select 65990,'ISAS' from dual union all
select 65992,'PITT' from dual union all
select 65993,'LERC' from dual union all
select 65994,'ASTD' from dual union all
select 65995,'UCO' from dual union all
select 65996,'90MW' from dual union all
select 65999,'UAH' from dual union all
select 660,'NRL' from dual union all
select 6600,'USA' from dual union all
select 66003,'SSC' from dual union all
select 66004,'90MW' from dual union all
select 66009,'90MW' from dual union all
select 6601,'USA' from dual union all
select 66010,'GSFC' from dual union all
select 66010,'UWI' from dual union all
select 66012,'CIT' from dual union all
select 66013,'LERC' from dual union all
select 66014,'91MW' from dual union all
select 66015,'91MW' from dual union all
select 66016,'USCLA' from dual union all
select 6602,'USA' from dual union all
select 66021,'CAL' from dual union all
select 66026,'MITLL' from dual union all
select 6603,'USA' from dual union all
select 66030,'SWRIB' from dual union all
select 66031,'COSG' from dual union all
select 66032,'ISAS' from dual union all
select 66032,'KYO' from dual union all
select 66033,'MSFC' from dual union all
select 66034,'ISAS' from dual union all
select 66034,'KYO' from dual union all
select 66035,'INPE' from dual union all
select 66036,'BMDO' from dual union all
select 66038,'90MW' from dual union all
select 66039,'NASDA' from dual union all
select 6604,'USA' from dual union all
select 66045,'BMDO' from dual union all
select 66045,'NAIC' from dual union all
select 66046,'LERC' from dual union all
select 66047,'JHU' from dual union all
select 6605,'WRE' from dual union all
select 66050,'UMI' from dual union all
select 66051,'90MW' from dual union all
select 66053,'GSFC' from dual union all
select 66054,'GSFC' from dual union all
select 66055,'ASTD' from dual union all
select 66058,'USN' from dual union all
select 6606,'WRE' from dual union all
select 66064,'JMA' from dual union all
select 66065,'JMA' from dual union all
select 66067,'JMA' from dual union all
select 66069,'JMA' from dual union all
select 6607,'WRE' from dual union all
select 66070,'TSUK' from dual union all
select 66071,'JMA' from dual union all
select 66072,'JMA' from dual union all
select 66073,'JMA' from dual union all
select 66075,'JMA' from dual union all
select 66076,'JMA' from dual union all
select 66077,'WFF' from dual union all
select 66079,'ISAS' from dual union all
select 66080,'TROM' from dual union all
select 661,'APL' from dual union all
select 66144,'NRO' from dual union all
select 66148,'NRO' from dual union all
select 66168,'EAC' from dual union all
select 66195,'EAC' from dual union all
select 66197,'USU' from dual union all
select 662,'AFCRL' from dual union all
select 662,'UMI' from dual union all
select 66201,'JMA' from dual union all
select 6622,'GSFC' from dual union all
select 66231,'NDRE' from dual union all
select 66233,'HAL5' from dual union all
select 66239,'ISAS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 6624,'SIMED' from dual union all
select 66240,'321SMW' from dual union all
select 66240,'90MW' from dual union all
select 66241,'APL' from dual union all
select 66241,'IDG' from dual union all
select 66242,'APL' from dual union all
select 66242,'IDG' from dual union all
select 66244,'UNH' from dual union all
select 66245,'SAND' from dual union all
select 66246,'UNH' from dual union all
select 66247,'BMDO' from dual union all
select 66247,'NAIC' from dual union all
select 6625,'UMI' from dual union all
select 66252,'MITLL' from dual union all
select 66254,'BMDO' from dual union all
select 66254,'NAIC' from dual union all
select 66255,'LERC' from dual union all
select 66256,'MITLL' from dual union all
select 66259,'UWA' from dual union all
select 66263,'UCO' from dual union all
select 66264,'SWRIB' from dual union all
select 66265,'JHU' from dual union all
select 66266,'UWI' from dual union all
select 66268,'DLR' from dual union all
select 66268,'INPE' from dual union all
select 66269,'ISRO' from dual union all
select 66270,'PSU' from dual union all
select 66271,'BU' from dual union all
select 66272,'90MW' from dual union all
select 66273,'UCO' from dual union all
select 66274,'341SMW' from dual union all
select 66275,'SSDC' from dual union all
select 66276,'ISRO' from dual union all
select 66277,'AFGL' from dual union all
select 66277,'CIT' from dual union all
select 66279,'SSC' from dual union all
select 6628,'CNRS' from dual union all
select 6628,'LPA' from dual union all
select 66281,'90MW' from dual union all
select 66285,'KARI' from dual union all
select 66287,'LARC' from dual union all
select 66288,'USCLA' from dual union all
select 66289,'SSDC' from dual union all
select 66290,'SSDC' from dual union all
select 66291,'SSDC' from dual union all
select 66292,'LERC' from dual union all
select 66293,'LERC' from dual union all
select 66294,'SSDC' from dual union all
select 66295,'90MW' from dual union all
select 66296,'PITT' from dual union all
select 66298,'NASDA' from dual union all
select 66299,'NRL' from dual union all
select 663,'NRL' from dual union all
select 66301,'SAND' from dual union all
select 66304,'DLR' from dual union all
select 66304,'INPE' from dual union all
select 66305,'NRL' from dual union all
select 66307,'PSU' from dual union all
select 66308,'90MW' from dual union all
select 66309,'BU' from dual union all
select 66310,'GSFC' from dual union all
select 66311,'MIRC' from dual union all
select 66312,'GSFC' from dual union all
select 66316,'JMA' from dual union all
select 66318,'TSUK' from dual union all
select 66319,'JMA' from dual union all
select 66320,'JMA' from dual union all
select 66321,'JMA' from dual union all
select 66322,'JMA' from dual union all
select 66323,'JMA' from dual union all
select 66325,'JMA' from dual union all
select 66326,'JMA' from dual union all
select 66327,'JMA' from dual union all
select 66330,'JMA' from dual union all
select 66332,'JMA' from dual union all
select 66334,'JMA' from dual union all
select 66336,'JMA' from dual union all
select 66337,'JMA' from dual union all
select 66339,'JMA' from dual union all
select 66340,'JMA' from dual union all
select 66342,'JMA' from dual union all
select 66344,'JMA' from dual union all
select 66346,'BONN' from dual union all
select 66348,'JMA' from dual union all
select 66349,'JMA' from dual union all
select 66351,'WFF' from dual union all
select 66352,'BONN' from dual union all
select 66354,'WFF' from dual union all
select 66355,'WFF' from dual union all
select 66356,'WFF' from dual union all
select 66357,'JMA' from dual union all
select 66361,'BONN' from dual union all
select 66362,'LARC' from dual union all
select 66363,'LARC' from dual union all
select 66364,'WFF' from dual union all
select 66365,'WFF' from dual union all
select 66367,'BONN' from dual union all
select 66368,'BONN' from dual union all
select 66369,'WFF' from dual union all
select 66370,'WFF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 66371,'WFF' from dual union all
select 66372,'JMA' from dual union all
select 66373,'BONN' from dual union all
select 66374,'BONN' from dual union all
select 66375,'WFF' from dual union all
select 66376,'WFF' from dual union all
select 66377,'WFF' from dual union all
select 66378,'WFF' from dual union all
select 66380,'JMA' from dual union all
select 66382,'BONN' from dual union all
select 66383,'TSUK' from dual union all
select 66384,'TSUK' from dual union all
select 66385,'JMA' from dual union all
select 66387,'JMA' from dual union all
select 66388,'JMA' from dual union all
select 66389,'JMA' from dual union all
select 66391,'JMA' from dual union all
select 66392,'DLR' from dual union all
select 66393,'DLR' from dual union all
select 66394,'JMA' from dual union all
select 66395,'JMA' from dual union all
select 66396,'JMA' from dual union all
select 66397,'JMA' from dual union all
select 66398,'JMA' from dual union all
select 66399,'JMA' from dual union all
select 664,'APL' from dual union all
select 664,'USAF' from dual union all
select 66400,'CRLI' from dual union all
select 66400,'ISAS' from dual union all
select 66401,'CRLI' from dual union all
select 66401,'ISAS' from dual union all
select 66407,'WFF' from dual union all
select 66408,'WFF' from dual union all
select 66409,'WFF' from dual union all
select 66411,'WFF' from dual union all
select 66412,'WFF' from dual union all
select 66414,'WFF' from dual union all
select 66415,'WFF' from dual union all
select 66416,'WFF' from dual union all
select 66417,'WFF' from dual union all
select 66418,'WFF' from dual union all
select 66419,'WFF' from dual union all
select 6642,'UMI' from dual union all
select 66420,'WFF' from dual union all
select 66421,'WFF' from dual union all
select 66422,'WFF' from dual union all
select 66423,'WFF' from dual union all
select 66424,'WFF' from dual union all
select 66425,'WFF' from dual union all
select 66426,'WFF' from dual union all
select 66427,'WFF' from dual union all
select 66428,'WFF' from dual union all
select 66429,'WFF' from dual union all
select 66435,'TROM' from dual union all
select 66436,'TROM' from dual union all
select 66441,'NRO' from dual union all
select 6646,'AFCRL' from dual union all
select 66465,'NRO' from dual union all
select 6648,'LARC' from dual union all
select 66491,'NRO' from dual union all
select 66517,'ISAS' from dual union all
select 66518,'ISAS' from dual union all
select 66523,'NRO' from dual union all
select 66536,'ISAS' from dual union all
select 66536,'TSUK' from dual union all
select 66537,'SSC' from dual union all
select 66538,'ISAS' from dual union all
select 66538,'NAOJ' from dual union all
select 66539,'DLR' from dual union all
select 66539,'INPE' from dual union all
select 66540,'ISAS' from dual union all
select 66540,'TOK' from dual union all
select 66541,'SSC' from dual union all
select 66546,'DASARI' from dual union all
select 66549,'CORN' from dual union all
select 66550,'CLEM' from dual union all
select 66551,'91MW' from dual union all
select 66552,'CLEM' from dual union all
select 66553,'CLEM' from dual union all
select 66554,'USU' from dual union all
select 66555,'STO' from dual union all
select 66556,'STO' from dual union all
select 66559,'UTD' from dual union all
select 66562,'CORN' from dual union all
select 66563,'INPE' from dual union all
select 66564,'GSFC' from dual union all
select 66567,'SAND' from dual union all
select 66568,'BU' from dual union all
select 66569,'UCO' from dual union all
select 66570,'KIEL' from dual union all
select 66571,'CALG' from dual union all
select 66572,'WFF' from dual union all
select 66573,'90SW' from dual union all
select 66575,'AFGL' from dual union all
select 66575,'CIT' from dual union all
select 66576,'90MW' from dual union all
select 66576,'91MW' from dual union all
select 66577,'KARI' from dual union all
select 66578,'CIN' from dual union all
select 66579,'91SW' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 66581,'PSU' from dual union all
select 66584,'AFSPC' from dual union all
select 66585,'ARC' from dual union all
select 66586,'KIEL' from dual union all
select 66588,'KIEL' from dual union all
select 66589,'DLR' from dual union all
select 66589,'MBB' from dual union all
select 66591,'SWRIB' from dual union all
select 66593,'LERC' from dual union all
select 66594,'NASDA' from dual union all
select 66596,'ESTEC' from dual union all
select 66596,'SSC' from dual union all
select 66597,'DASARI' from dual union all
select 66600,'NSPO' from dual union all
select 66603,'JMA' from dual union all
select 66604,'JMA' from dual union all
select 66605,'JMA' from dual union all
select 66606,'JMA' from dual union all
select 66608,'JMA' from dual union all
select 66609,'JMA' from dual union all
select 66611,'JMA' from dual union all
select 66612,'JMA' from dual union all
select 66613,'JMA' from dual union all
select 66614,'JMA' from dual union all
select 66616,'JMA' from dual union all
select 66617,'JMA' from dual union all
select 66618,'WSMR' from dual union all
select 66619,'JMA' from dual union all
select 66620,'JMA' from dual union all
select 66622,'JMA' from dual union all
select 66623,'JMA' from dual union all
select 66624,'JMA' from dual union all
select 66625,'JMA' from dual union all
select 66626,'JMA' from dual union all
select 66627,'JMA' from dual union all
select 66628,'JMA' from dual union all
select 66629,'JMA' from dual union all
select 66630,'ISAS' from dual union all
select 66631,'ISAS' from dual union all
select 66632,'JMA' from dual union all
select 66633,'JMA' from dual union all
select 66634,'JMA' from dual union all
select 66635,'JMA' from dual union all
select 66636,'JMA' from dual union all
select 66638,'JMA' from dual union all
select 66639,'JMA' from dual union all
select 66640,'JMA' from dual union all
select 66641,'JMA' from dual union all
select 66642,'JMA' from dual union all
select 66643,'JMA' from dual union all
select 66644,'BAS' from dual union all
select 66644,'BONN' from dual union all
select 66645,'BAS' from dual union all
select 66645,'BONN' from dual union all
select 66646,'BAS' from dual union all
select 66646,'BONN' from dual union all
select 66647,'BAS' from dual union all
select 66647,'BONN' from dual union all
select 66648,'BAS' from dual union all
select 66648,'BONN' from dual union all
select 66649,'BAS' from dual union all
select 66649,'BONN' from dual union all
select 66650,'BAS' from dual union all
select 66650,'BONN' from dual union all
select 66651,'BAS' from dual union all
select 66651,'BONN' from dual union all
select 66652,'BAS' from dual union all
select 66652,'BONN' from dual union all
select 66654,'BAS' from dual union all
select 66654,'BONN' from dual union all
select 66655,'BAS' from dual union all
select 66655,'BONN' from dual union all
select 66656,'BAS' from dual union all
select 66656,'BONN' from dual union all
select 66657,'BAS' from dual union all
select 66657,'BONN' from dual union all
select 66658,'BAS' from dual union all
select 66658,'BONN' from dual union all
select 66659,'BAS' from dual union all
select 66659,'BONN' from dual union all
select 66660,'BAS' from dual union all
select 66660,'BONN' from dual union all
select 66661,'BAS' from dual union all
select 66661,'BONN' from dual union all
select 66662,'BAS' from dual union all
select 66662,'BONN' from dual union all
select 66663,'BAS' from dual union all
select 66663,'BONN' from dual union all
select 66664,'BAS' from dual union all
select 66664,'BONN' from dual union all
select 66665,'BAS' from dual union all
select 66665,'BONN' from dual union all
select 66666,'BAS' from dual union all
select 66666,'BONN' from dual union all
select 66667,'BAS' from dual union all
select 66667,'BONN' from dual union all
select 66668,'BAS' from dual union all
select 66668,'BONN' from dual union all
select 66669,'BAS' from dual union all
select 66669,'BONN' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 6667,'UMI' from dual union all
select 66670,'BAS' from dual union all
select 66670,'BONN' from dual union all
select 66678,'KSC' from dual union all
select 66679,'KSC' from dual union all
select 66680,'SSC' from dual union all
select 66681,'SSC' from dual union all
select 66709,'NRO' from dual union all
select 66757,'WFF' from dual union all
select 66774,'CORN' from dual union all
select 66775,'APL' from dual union all
select 66775,'IDG' from dual union all
select 66776,'AERO' from dual union all
select 66777,'CLEM' from dual union all
select 66778,'ISAS' from dual union all
select 66778,'TOHO' from dual union all
select 66781,'91SW' from dual union all
select 66782,'UNH' from dual union all
select 66783,'BU' from dual union all
select 66784,'90SW' from dual union all
select 66785,'INPE' from dual union all
select 66791,'GSFC' from dual union all
select 66791,'UWI' from dual union all
select 66797,'UWI' from dual union all
select 66804,'NRL' from dual union all
select 66805,'SSC' from dual union all
select 66810,'JHU' from dual union all
select 66811,'GSFC' from dual union all
select 66815,'USNTB' from dual union all
select 66816,'GSFC' from dual union all
select 66817,'CLEM' from dual union all
select 66818,'GSFC' from dual union all
select 66819,'IAP' from dual union all
select 66819,'NDRE' from dual union all
select 66820,'GSFC' from dual union all
select 66821,'CLEM' from dual union all
select 66822,'ERAU' from dual union all
select 66826,'USCLA' from dual union all
select 66827,'341SMW' from dual union all
select 66828,'91SW' from dual union all
select 66829,'UWI' from dual union all
select 66836,'UCO' from dual union all
select 66844,'PRL' from dual union all
select 66850,'PRL' from dual union all
select 66851,'PRL' from dual union all
select 66853,'UMD' from dual union all
select 66863,'IAP' from dual union all
select 66863,'NDRE' from dual union all
select 66864,'IAP' from dual union all
select 66864,'NDRE' from dual union all
select 66865,'IAP' from dual union all
select 66865,'NDRE' from dual union all
select 66866,'IAP' from dual union all
select 66866,'NDRE' from dual union all
select 66867,'IAP' from dual union all
select 66867,'NDRE' from dual union all
select 66868,'IAP' from dual union all
select 66868,'NDRE' from dual union all
select 66869,'IAP' from dual union all
select 66869,'NDRE' from dual union all
select 66870,'IAP' from dual union all
select 66870,'NDRE' from dual union all
select 66871,'IAP' from dual union all
select 66871,'NDRE' from dual union all
select 66872,'IAP' from dual union all
select 66872,'NDRE' from dual union all
select 66873,'IAP' from dual union all
select 66873,'NDRE' from dual union all
select 66874,'IAP' from dual union all
select 66874,'NDRE' from dual union all
select 66875,'IAP' from dual union all
select 66875,'NDRE' from dual union all
select 66876,'IAP' from dual union all
select 66876,'NDRE' from dual union all
select 66877,'IAP' from dual union all
select 66877,'NDRE' from dual union all
select 66878,'IAP' from dual union all
select 66878,'NDRE' from dual union all
select 66879,'IAP' from dual union all
select 66879,'NDRE' from dual union all
select 66880,'BONN' from dual union all
select 66881,'BONN' from dual union all
select 66882,'BONN' from dual union all
select 66883,'BONN' from dual union all
select 66884,'BONN' from dual union all
select 66885,'BONN' from dual union all
select 66886,'BONN' from dual union all
select 66887,'BONN' from dual union all
select 66888,'BONN' from dual union all
select 66889,'BONN' from dual union all
select 66890,'ISAS' from dual union all
select 66891,'BONN' from dual union all
select 66892,'ISAS' from dual union all
select 66893,'BONN' from dual union all
select 66894,'BONN' from dual union all
select 66895,'BONN' from dual union all
select 66896,'BONN' from dual union all
select 66897,'BONN' from dual union all
select 66898,'BONN' from dual union all
select 669,'JPL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 66901,'FLASP' from dual union all
select 66901,'RIU' from dual union all
select 6694,'AEC' from dual union all
select 66949,'NRO' from dual union all
select 6697,'FRC' from dual union all
select 6698,'AEC' from dual union all
select 66982,'NRO' from dual union all
select 66985,'LUX' from dual union all
select 66985,'NRL' from dual union all
select 6699,'FRC' from dual union all
select 66991,'CSXT' from dual union all
select 66997,'UAF' from dual union all
select 66999,'ISRO' from dual union all
select 670,'JPL' from dual union all
select 67000,'ISRO' from dual union all
select 67001,'ISRO' from dual union all
select 67002,'ISRO' from dual union all
select 67003,'ISRO' from dual union all
select 67004,'ISRO' from dual union all
select 67005,'ISRO' from dual union all
select 67006,'ISRO' from dual union all
select 67007,'ISRO' from dual union all
select 67008,'ISRO' from dual union all
select 67009,'ISRO' from dual union all
select 67010,'ISRO' from dual union all
select 67011,'ISRO' from dual union all
select 67012,'ISRO' from dual union all
select 67013,'ISRO' from dual union all
select 67014,'ISRO' from dual union all
select 67015,'ISRO' from dual union all
select 67016,'ISRO' from dual union all
select 67017,'ISRO' from dual union all
select 67018,'ISRO' from dual union all
select 67019,'ISRO' from dual union all
select 67020,'ISRO' from dual union all
select 67021,'ISRO' from dual union all
select 67022,'ISRO' from dual union all
select 67023,'ISRO' from dual union all
select 67024,'ISRO' from dual union all
select 67025,'ISRO' from dual union all
select 67026,'ISRO' from dual union all
select 67027,'ISRO' from dual union all
select 67028,'ISRO' from dual union all
select 67029,'ISRO' from dual union all
select 67030,'ISRO' from dual union all
select 67031,'ISRO' from dual union all
select 67032,'ISRO' from dual union all
select 67033,'ISRO' from dual union all
select 67034,'ISRO' from dual union all
select 67035,'ISRO' from dual union all
select 67036,'ISRO' from dual union all
select 67037,'ISRO' from dual union all
select 67038,'ISRO' from dual union all
select 6705,'NMSU' from dual union all
select 67050,'ISAS' from dual union all
select 67053,'INPE' from dual union all
select 67055,'JHU' from dual union all
select 67056,'UCB' from dual union all
select 67061,'CALG' from dual union all
select 67062,'90SW' from dual union all
select 67065,'ASTD' from dual union all
select 67066,'ASTD' from dual union all
select 6707,'AEC' from dual union all
select 67076,'UCO' from dual union all
select 67077,'IAP' from dual union all
select 67077,'NDRE' from dual union all
select 67078,'IAP' from dual union all
select 67078,'NDRE' from dual union all
select 67079,'PSU' from dual union all
select 67082,'91SW' from dual union all
select 67083,'BU' from dual union all
select 67085,'WFF' from dual union all
select 67086,'UVT' from dual union all
select 67088,'MIT' from dual union all
select 67089,'ATECH' from dual union all
select 6709,'GSFC' from dual union all
select 67093,'UMN' from dual union all
select 67094,'GSFC' from dual union all
select 67095,'BU' from dual union all
select 67096,'INPE' from dual union all
select 67098,'SAO' from dual union all
select 6710,'SAND' from dual union all
select 67104,'90SW' from dual union all
select 67105,'341SW' from dual union all
select 67107,'AFRL' from dual union all
select 67109,'AERO' from dual union all
select 6711,'SAND' from dual union all
select 67112,'ISAS' from dual union all
select 67112,'KYO' from dual union all
select 67113,'INPE' from dual union all
select 67115,'NAWC' from dual union all
select 67118,'JMA' from dual union all
select 67119,'JMA' from dual union all
select 6712,'SAND' from dual union all
select 67120,'FLASP' from dual union all
select 67121,'JMA' from dual union all
select 67122,'JMA' from dual union all
select 67123,'JMA' from dual union all
select 67124,'JMA' from dual union all
select 67125,'JMA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 67126,'JMA' from dual union all
select 67127,'IAP' from dual union all
select 67127,'NDRE' from dual union all
select 67128,'IAP' from dual union all
select 67128,'NDRE' from dual union all
select 67129,'IAP' from dual union all
select 67129,'NDRE' from dual union all
select 67130,'IAP' from dual union all
select 67130,'NDRE' from dual union all
select 67131,'JMA' from dual union all
select 67132,'JMA' from dual union all
select 67133,'JMA' from dual union all
select 67134,'JMA' from dual union all
select 67135,'JMA' from dual union all
select 67136,'JMA' from dual union all
select 67137,'JMA' from dual union all
select 67138,'JMA' from dual union all
select 67139,'JMA' from dual union all
select 6714,'AFCRL' from dual union all
select 67140,'ISAS' from dual union all
select 67141,'ISAS' from dual union all
select 67142,'JMA' from dual union all
select 67143,'JMA' from dual union all
select 67144,'JMA' from dual union all
select 67145,'JMA' from dual union all
select 67146,'JMA' from dual union all
select 67147,'JMA' from dual union all
select 67148,'JMA' from dual union all
select 67149,'FLASP' from dual union all
select 67150,'FLASP' from dual union all
select 67151,'JMA' from dual union all
select 67173,'NRO' from dual union all
select 6718,'GSFC' from dual union all
select 6719,'GSFC' from dual union all
select 67193,'NRO' from dual union all
select 67197,'NRO' from dual union all
select 67199,'NRO' from dual union all
select 6720,'KOBE' from dual union all
select 6721,'UMI' from dual union all
select 67213,'CLEM' from dual union all
select 67213,'NCU' from dual union all
select 67216,'UVA' from dual union all
select 67238,'ISRO' from dual union all
select 67239,'ISRO' from dual union all
select 67240,'ISRO' from dual union all
select 6725,'ATLA' from dual union all
select 6726,'ATLA' from dual union all
select 67260,'ISRO' from dual union all
select 67261,'ISRO' from dual union all
select 67265,'341SW' from dual union all
select 67266,'JHU' from dual union all
select 67267,'LERC' from dual union all
select 6727,'PMR' from dual union all
select 67270,'EADS' from dual union all
select 67270,'SSC' from dual union all
select 67272,'LUX' from dual union all
select 67272,'NRL' from dual union all
select 67277,'ESTEC' from dual union all
select 67277,'SSC' from dual union all
select 67278,'ASTD' from dual union all
select 6728,'PMR' from dual union all
select 67280,'ARR' from dual union all
select 67280,'NDRE' from dual union all
select 67282,'WFF' from dual union all
select 67283,'IAP' from dual union all
select 67283,'NDRE' from dual union all
select 67284,'SAO' from dual union all
select 67285,'IAP' from dual union all
select 67285,'NDRE' from dual union all
select 6729,'NMSU' from dual union all
select 67291,'GSFC' from dual union all
select 67295,'TPS' from dual union all
select 67296,'90SW' from dual union all
select 6730,'NMSU' from dual union all
select 67302,'NAWC' from dual union all
select 67306,'UQU' from dual union all
select 67308,'91SW' from dual union all
select 6731,'NMSU' from dual union all
select 67311,'UWI' from dual union all
select 67317,'STO' from dual union all
select 6732,'AFCRL' from dual union all
select 67328,'ISAS' from dual union all
select 67329,'USU' from dual union all
select 6733,'WRE' from dual union all
select 67330,'IAP' from dual union all
select 67330,'NDRE' from dual union all
select 67331,'IAP' from dual union all
select 67331,'NDRE' from dual union all
select 67332,'IAP' from dual union all
select 67332,'NDRE' from dual union all
select 67333,'IAP' from dual union all
select 67333,'NDRE' from dual union all
select 67334,'IAP' from dual union all
select 67334,'NDRE' from dual union all
select 67335,'IAP' from dual union all
select 67335,'NDRE' from dual union all
select 67336,'IAP' from dual union all
select 67336,'NDRE' from dual union all
select 67337,'IAP' from dual union all
select 67337,'NDRE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 67338,'IAP' from dual union all
select 67338,'NDRE' from dual union all
select 67339,'IAP' from dual union all
select 67339,'NDRE' from dual union all
select 6734,'WRE' from dual union all
select 67340,'IAP' from dual union all
select 67340,'NDRE' from dual union all
select 67341,'IAP' from dual union all
select 67341,'NDRE' from dual union all
select 67342,'STO' from dual union all
select 67343,'STO' from dual union all
select 67344,'STO' from dual union all
select 67345,'STO' from dual union all
select 67346,'IAP' from dual union all
select 67347,'IAP' from dual union all
select 67348,'IAP' from dual union all
select 67349,'IAP' from dual union all
select 6735,'LARC' from dual union all
select 67350,'IAP' from dual union all
select 67351,'IAP' from dual union all
select 67352,'IAP' from dual union all
select 67353,'IAP' from dual union all
select 67354,'IAP' from dual union all
select 67355,'IAP' from dual union all
select 67356,'IAP' from dual union all
select 67357,'IAP' from dual union all
select 67358,'IAP' from dual union all
select 67359,'IAP' from dual union all
select 6736,'WSMR' from dual union all
select 67360,'IAP' from dual union all
select 67361,'IAP' from dual union all
select 67362,'IAP' from dual union all
select 67363,'IAP' from dual union all
select 67364,'IAP' from dual union all
select 67365,'IAP' from dual union all
select 67366,'IAP' from dual union all
select 67367,'IAP' from dual union all
select 67368,'IAP' from dual union all
select 67369,'IAP' from dual union all
select 6737,'AFSWC' from dual union all
select 67370,'IAP' from dual union all
select 67371,'IAP' from dual union all
select 67372,'IAP' from dual union all
select 67373,'IAP' from dual union all
select 67374,'IAP' from dual union all
select 67375,'IAP' from dual union all
select 67376,'STO' from dual union all
select 6738,'NMC' from dual union all
select 6742,'NSA' from dual union all
select 67441,'KARI' from dual union all
select 67442,'CSXT' from dual union all
select 6746,'AFCRL' from dual union all
select 67464,'ISRO' from dual union all
select 67465,'ISRO' from dual union all
select 67466,'ARC' from dual union all
select 67466,'WFF' from dual union all
select 67467,'USN' from dual union all
select 67468,'USN' from dual union all
select 6747,'CARDE' from dual union all
select 6748,'GCA' from dual union all
select 67488,'CORN' from dual union all
select 6749,'GSFC' from dual union all
select 67492,'DART' from dual union all
select 67492,'UIA' from dual union all
select 67493,'UCO' from dual union all
select 67494,'ISAS' from dual union all
select 67495,'AERO' from dual union all
select 67496,'CLEM' from dual union all
select 67497,'CLEM' from dual union all
select 67498,'CLEM' from dual union all
select 67499,'UNH' from dual union all
select 67500,'UNH' from dual union all
select 67501,'UNH' from dual union all
select 67502,'UNH' from dual union all
select 67506,'SSC' from dual union all
select 67507,'UAF' from dual union all
select 67509,'AFSPC' from dual union all
select 6751,'NRL' from dual union all
select 67515,'STAN' from dual union all
select 67517,'PSU' from dual union all
select 67518,'INPE' from dual union all
select 67519,'PSU' from dual union all
select 6752,'GSFC' from dual union all
select 67522,'90SW' from dual union all
select 67523,'ERAU' from dual union all
select 67524,'ERAU' from dual union all
select 67526,'WFF' from dual union all
select 67527,'341SW' from dual union all
select 67528,'CIT' from dual union all
select 67531,'NRL' from dual union all
select 67532,'GSFC' from dual union all
select 67533,'IAP' from dual union all
select 67533,'NDRE' from dual union all
select 67534,'NAWC' from dual union all
select 67535,'NAWC' from dual union all
select 67536,'GSFC' from dual union all
select 67537,'IAP' from dual union all
select 67537,'NDRE' from dual union all
select 67538,'IAP' from dual union all
select 67538,'NDRE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 67539,'ESTEC' from dual union all
select 67539,'NPOL' from dual union all
select 6754,'CARDE' from dual union all
select 67540,'AFSPC' from dual union all
select 67541,'UQU' from dual union all
select 67542,'ISAS' from dual union all
select 67542,'KYO' from dual union all
select 67543,'ISAS' from dual union all
select 67543,'KYO' from dual union all
select 67544,'USCLA' from dual union all
select 67545,'341SW' from dual union all
select 67554,'UCO' from dual union all
select 67555,'AFRL' from dual union all
select 67558,'INPE' from dual union all
select 67560,'INPE' from dual union all
select 67565,'GSFC' from dual union all
select 67572,'PSU' from dual union all
select 67573,'PSU' from dual union all
select 67574,'PSU' from dual union all
select 67575,'UNH' from dual union all
select 67576,'IAP' from dual union all
select 67576,'NDRE' from dual union all
select 67577,'IAP' from dual union all
select 67577,'NDRE' from dual union all
select 67578,'IAP' from dual union all
select 67578,'NDRE' from dual union all
select 67579,'IAP' from dual union all
select 67579,'NDRE' from dual union all
select 67580,'IAP' from dual union all
select 67580,'NDRE' from dual union all
select 67581,'IAP' from dual union all
select 67581,'NDRE' from dual union all
select 67582,'IAP' from dual union all
select 67582,'NDRE' from dual union all
select 67583,'IAP' from dual union all
select 67583,'NDRE' from dual union all
select 67584,'IAP' from dual union all
select 67584,'NDRE' from dual union all
select 67585,'IAP' from dual union all
select 67585,'NDRE' from dual union all
select 67586,'IAP' from dual union all
select 67586,'NDRE' from dual union all
select 67587,'IAP' from dual union all
select 67587,'NDRE' from dual union all
select 67588,'IAP' from dual union all
select 67588,'NDRE' from dual union all
select 67589,'IAP' from dual union all
select 67589,'NDRE' from dual union all
select 6759,'LARC' from dual union all
select 67590,'IAP' from dual union all
select 67590,'NDRE' from dual union all
select 67591,'IAP' from dual union all
select 67591,'NDRE' from dual union all
select 67592,'IAP' from dual union all
select 67592,'NDRE' from dual union all
select 67593,'IAP' from dual union all
select 67593,'NDRE' from dual union all
select 67594,'IAP' from dual union all
select 67594,'NDRE' from dual union all
select 67595,'IAP' from dual union all
select 67595,'NDRE' from dual union all
select 67596,'IAP' from dual union all
select 67596,'NDRE' from dual union all
select 67597,'IAP' from dual union all
select 67597,'NDRE' from dual union all
select 67598,'IAP' from dual union all
select 67598,'NDRE' from dual union all
select 67599,'IAP' from dual union all
select 67599,'NDRE' from dual union all
select 67600,'IAP' from dual union all
select 67600,'NDRE' from dual union all
select 67601,'IAP' from dual union all
select 67601,'NDRE' from dual union all
select 67602,'IAP' from dual union all
select 67603,'IAP' from dual union all
select 67604,'IAP' from dual union all
select 67605,'IAP' from dual union all
select 67606,'IAP' from dual union all
select 67607,'IAP' from dual union all
select 67608,'IAP' from dual union all
select 67609,'IAP' from dual union all
select 67610,'IAP' from dual union all
select 67611,'IAP' from dual union all
select 67612,'IAP' from dual union all
select 67613,'IAP' from dual union all
select 67614,'IAP' from dual union all
select 67615,'IAP' from dual union all
select 67616,'IAP' from dual union all
select 67617,'IAP' from dual union all
select 67618,'IAP' from dual union all
select 67619,'IAP' from dual union all
select 67620,'IAP' from dual union all
select 67621,'IAP' from dual union all
select 67622,'IAP' from dual union all
select 67623,'IAP' from dual union all
select 67624,'IAP' from dual union all
select 6764,'NRL' from dual union all
select 6765,'GSFC' from dual union all
select 67665,'NRO' from dual union all
select 67678,'NRO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 67687,'CLEM' from dual union all
select 67689,'WFF' from dual union all
select 6769,'CNRS' from dual union all
select 67690,'SCAL' from dual union all
select 67691,'SCAL' from dual union all
select 67692,'SCAL' from dual union all
select 67693,'SCAL' from dual union all
select 67694,'SCAL' from dual union all
select 67695,'SCAL' from dual union all
select 67696,'ATKG' from dual union all
select 67697,'SCAL' from dual union all
select 67698,'SCAL' from dual union all
select 67699,'SCAL' from dual union all
select 677,'NRL' from dual union all
select 6770,'AN' from dual union all
select 67701,'SCAL' from dual union all
select 67702,'OSLO' from dual union all
select 6771,'NRL' from dual union all
select 67713,'UVA' from dual union all
select 67714,'ISRO' from dual union all
select 67715,'USN' from dual union all
select 67716,'USN' from dual union all
select 6772,'LARC' from dual union all
select 67729,'GSFC' from dual union all
select 6773,'UMI' from dual union all
select 67730,'DART' from dual union all
select 67731,'GSFC' from dual union all
select 67732,'CIT' from dual union all
select 67734,'90SW' from dual union all
select 67735,'UAF' from dual union all
select 67736,'UAF' from dual union all
select 67739,'CLEM' from dual union all
select 6774,'CNRS' from dual union all
select 67740,'CLEM' from dual union all
select 67741,'CLEM' from dual union all
select 67742,'ESTEC' from dual union all
select 67742,'SSC' from dual union all
select 67743,'ASTD' from dual union all
select 67744,'SWRIB' from dual union all
select 67748,'MITLL' from dual union all
select 67749,'MITLL' from dual union all
select 6775,'QUB' from dual union all
select 67750,'USU' from dual union all
select 67754,'UTD' from dual union all
select 67755,'UTD' from dual union all
select 67756,'UTD' from dual union all
select 67757,'UTD' from dual union all
select 67758,'IAP' from dual union all
select 67758,'NDRE' from dual union all
select 67759,'IAP' from dual union all
select 67759,'NDRE' from dual union all
select 6776,'CNRS' from dual union all
select 67760,'IAP' from dual union all
select 67760,'NDRE' from dual union all
select 67761,'ERAU' from dual union all
select 67762,'ERAU' from dual union all
select 67763,'NSROC' from dual union all
select 67765,'UCO' from dual union all
select 67767,'AFRL' from dual union all
select 67769,'PSU' from dual union all
select 6777,'AFCRL' from dual union all
select 6777,'UMD' from dual union all
select 67770,'NSROC' from dual union all
select 67773,'USCLA' from dual union all
select 67774,'NRL' from dual union all
select 67778,'JHU' from dual union all
select 67779,'CLEM' from dual union all
select 67779,'NCU' from dual union all
select 6778,'ICST' from dual union all
select 6778,'RAE' from dual union all
select 67780,'ISRO' from dual union all
select 67789,'FLASP' from dual union all
select 67790,'WFF' from dual union all
select 67791,'WFF' from dual union all
select 67792,'WFF' from dual union all
select 67793,'GSFC' from dual union all
select 67794,'GSFC' from dual union all
select 67795,'GSFC' from dual union all
select 67796,'GSFC' from dual union all
select 67797,'GSFC' from dual union all
select 67798,'GSFC' from dual union all
select 67799,'GSFC' from dual union all
select 6780,'CNRS' from dual union all
select 67800,'GSFC' from dual union all
select 67801,'GSFC' from dual union all
select 67802,'GSFC' from dual union all
select 67803,'GSFC' from dual union all
select 67804,'GSFC' from dual union all
select 67805,'GSFC' from dual union all
select 67806,'GSFC' from dual union all
select 67807,'GSFC' from dual union all
select 67808,'GSFC' from dual union all
select 67809,'GSFC' from dual union all
select 67810,'GSFC' from dual union all
select 67811,'GSFC' from dual union all
select 67812,'GSFC' from dual union all
select 67813,'GSFC' from dual union all
select 67814,'GSFC' from dual union all
select 67815,'GSFC' from dual union all
select 67816,'WFF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 67817,'WFF' from dual union all
select 67818,'WFF' from dual union all
select 67819,'GSFC' from dual union all
select 67820,'WFF' from dual union all
select 67821,'TROM' from dual union all
select 67822,'TROM' from dual union all
select 67823,'LRRC' from dual union all
select 67824,'LRRC' from dual union all
select 67825,'FLASP' from dual union all
select 67826,'WFF' from dual union all
select 67827,'WFF' from dual union all
select 67828,'WFF' from dual union all
select 67829,'WFF' from dual union all
select 67830,'WFF' from dual union all
select 67831,'WFF' from dual union all
select 67832,'WFF' from dual union all
select 67833,'WFF' from dual union all
select 67834,'WFF' from dual union all
select 67835,'WFF' from dual union all
select 67836,'WFF' from dual union all
select 67837,'WFF' from dual union all
select 67838,'GSFC' from dual union all
select 67839,'GSFC' from dual union all
select 67840,'GSFC' from dual union all
select 67841,'GSFC' from dual union all
select 67842,'GSFC' from dual union all
select 67843,'GSFC' from dual union all
select 67844,'GSFC' from dual union all
select 6785,'AN' from dual union all
select 6786,'FIAN' from dual union all
select 67878,'NRO' from dual union all
select 6789,'GSFC' from dual union all
select 67899,'SCAL' from dual union all
select 6790,'LARC' from dual union all
select 67901,'SCAL' from dual union all
select 67903,'WFF' from dual union all
select 6791,'AFCRL' from dual union all
select 67910,'NSROC' from dual union all
select 67912,'ATKG' from dual union all
select 67915,'SCAL' from dual union all
select 67917,'ISRO' from dual union all
select 67918,'ISRO' from dual union all
select 67919,'LAPAN' from dual union all
select 6792,'AFCRL' from dual union all
select 67920,'LAPAN' from dual union all
select 67921,'ISRO' from dual union all
select 67923,'SDSC' from dual union all
select 67926,'USN' from dual union all
select 67929,'SWRIB' from dual union all
select 6793,'NRL' from dual union all
select 67930,'ISAS' from dual union all
select 67930,'TOK' from dual union all
select 67931,'CORN' from dual union all
select 6794,'AFCRL' from dual union all
select 67941,'NSROC' from dual union all
select 67944,'CSXT' from dual union all
select 67946,'UCO' from dual union all
select 67949,'SCAL' from dual union all
select 6795,'AFCRL' from dual union all
select 67950,'341SW' from dual union all
select 67953,'90SW' from dual union all
select 67955,'CORN' from dual union all
select 67956,'CORN' from dual union all
select 67957,'CORN' from dual union all
select 67958,'ISAS' from dual union all
select 67958,'TOK' from dual union all
select 67961,'CORN' from dual union all
select 67962,'CORN' from dual union all
select 67963,'CORN' from dual union all
select 67964,'CLEM' from dual union all
select 67966,'CORN' from dual union all
select 67967,'GSFC' from dual union all
select 67968,'GSFC' from dual union all
select 67971,'341SW' from dual union all
select 67971,'564SMS' from dual union all
select 67972,'GSFC' from dual union all
select 67973,'GSFC' from dual union all
select 67974,'CORN' from dual union all
select 67975,'CLEM' from dual union all
select 67976,'NSROC' from dual union all
select 67978,'SCAL' from dual union all
select 67979,'LAPAN' from dual union all
select 6798,'AFCRL' from dual union all
select 6798,'UMD' from dual union all
select 67980,'LAPAN' from dual union all
select 67981,'SCAL' from dual union all
select 67984,'UCO' from dual union all
select 67986,'INPE' from dual union all
select 67987,'DLR' from dual union all
select 67987,'SSC' from dual union all
select 67991,'ESTEC' from dual union all
select 67991,'SSC' from dual union all
select 67993,'ASTD' from dual union all
select 67994,'ISAS' from dual union all
select 67995,'NCU' from dual union all
select 67999,'PRL' from dual union all
select 680,'USN' from dual union all
select 68004,'CLEM' from dual union all
select 68005,'CLEM' from dual union all
select 68006,'CLEM' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 68009,'FLASP' from dual union all
select 68009,'LRRC' from dual union all
select 68013,'NRO' from dual union all
select 68025,'NRO' from dual union all
select 68051,'NRO' from dual union all
select 6806,'AFCRL' from dual union all
select 68062,'DART' from dual union all
select 68065,'WFF' from dual union all
select 68068,'ATKG' from dual union all
select 6807,'LARC' from dual union all
select 68074,'LAPAN' from dual union all
select 68075,'NAWC' from dual union all
select 68076,'NAWC' from dual union all
select 68079,'USN' from dual union all
select 68081,'SDSC' from dual union all
select 68082,'ESA' from dual union all
select 68082,'ULM' from dual union all
select 68083,'NRL' from dual union all
select 68083,'STO' from dual union all
select 68084,'NSROC' from dual union all
select 68086,'NAWC' from dual union all
select 68090,'NAWC' from dual union all
select 68093,'NAWC' from dual union all
select 68094,'NAWC' from dual union all
select 68095,'DART' from dual union all
select 68096,'DART' from dual union all
select 68099,'SSC' from dual union all
select 68100,'IBSI' from dual union all
select 68101,'VAP' from dual union all
select 68108,'WFF' from dual union all
select 68109,'OSLO' from dual union all
select 68109,'TROM' from dual union all
select 68110,'NRL' from dual union all
select 68112,'USCLA' from dual union all
select 68117,'91SW' from dual union all
select 68118,'91SW' from dual union all
select 68121,'NAWC' from dual union all
select 68123,'ESTEC' from dual union all
select 68123,'NPOL' from dual union all
select 68127,'DLR' from dual union all
select 68129,'NAWC' from dual union all
select 68130,'NAWC' from dual union all
select 68131,'NAWC' from dual union all
select 68134,'NAWC' from dual union all
select 68138,'WFF' from dual union all
select 6814,'GSFC' from dual union all
select 68140,'SDSC' from dual union all
select 68141,'PRL' from dual union all
select 68146,'STO' from dual union all
select 68147,'STO' from dual union all
select 68148,'STO' from dual union all
select 68149,'IAP' from dual union all
select 6815,'ISAS' from dual union all
select 68150,'IAP' from dual union all
select 68150,'NDRE' from dual union all
select 68151,'IAP' from dual union all
select 68151,'NDRE' from dual union all
select 68152,'IAP' from dual union all
select 68153,'IAP' from dual union all
select 68154,'IAP' from dual union all
select 68155,'IAP' from dual union all
select 68156,'IAP' from dual union all
select 68157,'IAP' from dual union all
select 68158,'IAP' from dual union all
select 68159,'IAP' from dual union all
select 68160,'IAP' from dual union all
select 68161,'IAP' from dual union all
select 68162,'IAP' from dual union all
select 68163,'IAP' from dual union all
select 68164,'IAP' from dual union all
select 68165,'IAP' from dual union all
select 68166,'IAP' from dual union all
select 68167,'IAP' from dual union all
select 68168,'IAP' from dual union all
select 68169,'IAP' from dual union all
select 68170,'IAP' from dual union all
select 68171,'IAP' from dual union all
select 68172,'TROM' from dual union all
select 6819,'AFCRL' from dual union all
select 68199,'NRO' from dual union all
select 682,'APL' from dual union all
select 6820,'BRL' from dual union all
select 6821,'BRL' from dual union all
select 6822,'AFCRL' from dual union all
select 68229,'NRO' from dual union all
select 68236,'BRVCC' from dual union all
select 68238,'STO' from dual union all
select 6824,'AN' from dual union all
select 68241,'UPAE' from dual union all
select 68246,'IRSA' from dual union all
select 6825,'AFCRL' from dual union all
select 68250,'WFF' from dual union all
select 68253,'USN' from dual union all
select 68256,'ISRO' from dual union all
select 68257,'ISRO' from dual union all
select 68258,'ISRO' from dual union all
select 68259,'ISRO' from dual union all
select 6826,'ISAS' from dual union all
select 6826,'RRL' from dual union all
select 68260,'ISRO' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 68261,'ISRO' from dual union all
select 68263,'NCU' from dual union all
select 68264,'ISAS' from dual union all
select 68264,'KOBE' from dual union all
select 68265,'MSU' from dual union all
select 68269,'UQU' from dual union all
select 68270,'NAWC' from dual union all
select 68271,'NAWC' from dual union all
select 68272,'UQU' from dual union all
select 68273,'SSC' from dual union all
select 68274,'JPL' from dual union all
select 68275,'341SW' from dual union all
select 68276,'GSFC' from dual union all
select 6828,'AFCRL' from dual union all
select 68281,'ESTEC' from dual union all
select 68281,'SSC' from dual union all
select 68282,'ESA' from dual union all
select 68285,'NSROC' from dual union all
select 68286,'NAWC' from dual union all
select 68287,'NAWC' from dual union all
select 68288,'NAWC' from dual union all
select 68289,'90SW' from dual union all
select 68290,'TROM' from dual union all
select 68293,'NAWC' from dual union all
select 68295,'PSU' from dual union all
select 6830,'AEC' from dual union all
select 68304,'NAWC' from dual union all
select 68305,'91SW' from dual union all
select 68306,'NAWC' from dual union all
select 68308,'MDA' from dual union all
select 68309,'MDA' from dual union all
select 68314,'IAP' from dual union all
select 68317,'IAP' from dual union all
select 68319,'GSFC' from dual union all
select 68320,'UCO' from dual union all
select 68324,'USCLA' from dual union all
select 68325,'UCO' from dual union all
select 68336,'IRSA' from dual union all
select 6834,'AFCRL' from dual union all
select 68342,'PSU' from dual union all
select 68342,'WFF' from dual union all
select 6835,'AFCRL' from dual union all
select 6836,'GSFC' from dual union all
select 68369,'NRO' from dual union all
select 6837,'AFCRL' from dual union all
select 6839,'GSFC' from dual union all
select 684,'APL' from dual union all
select 68402,'NRO' from dual union all
select 68411,'NAMMO' from dual union all
select 6842,'AFCRL' from dual union all
select 68421,'KYSP' from dual union all
select 68425,'ERAU' from dual union all
select 68427,'TROM' from dual union all
select 68428,'LAPAN' from dual union all
select 6843,'ICST' from dual union all
select 6843,'RAE' from dual union all
select 6843,'UCL' from dual union all
select 68430,'CLEM' from dual union all
select 68431,'CLEM' from dual union all
select 68432,'CLEM' from dual union all
select 68433,'CLEM' from dual union all
select 68437,'91SW' from dual union all
select 68438,'DART' from dual union all
select 68439,'UAF' from dual union all
select 68440,'UAF' from dual union all
select 68441,'UAF' from dual union all
select 68442,'UAF' from dual union all
select 68443,'DART' from dual union all
select 68447,'SSICE' from dual union all
select 6845,'AFCRL' from dual union all
select 68452,'DSTO' from dual union all
select 68454,'NSROC' from dual union all
select 68457,'INPE' from dual union all
select 68458,'UCO' from dual union all
select 68459,'IAP' from dual union all
select 68460,'UCO' from dual union all
select 68461,'JHU' from dual union all
select 68465,'ISAS' from dual union all
select 6847,'AFCRL' from dual union all
select 68473,'LARC' from dual union all
select 68483,'UTD' from dual union all
select 68489,'GSFC' from dual union all
select 68491,'INPE' from dual union all
select 68498,'UIA' from dual union all
select 68499,'UIA' from dual union all
select 68501,'NCU' from dual union all
select 68502,'TOHO' from dual union all
select 68502,'TOYA' from dual union all
select 68503,'TSAO' from dual union all
select 6851,'AFCRL' from dual union all
select 68518,'NRO' from dual union all
select 6852,'WRE' from dual union all
select 6854,'AFCRL' from dual union all
select 6855,'AFCRL' from dual union all
select 68576,'WFF' from dual union all
select 68577,'LAPAN' from dual union all
select 68581,'ARC' from dual union all
select 68581,'LARCN' from dual union all
select 68583,'TSAO' from dual union all
select 68584,'LAPAN' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 68586,'IRSA' from dual union all
select 68587,'TROM' from dual union all
select 68589,'WFF' from dual union all
select 68590,'USN' from dual union all
select 68591,'USN' from dual union all
select 68592,'USN' from dual union all
select 68593,'USN' from dual union all
select 68594,'USN' from dual union all
select 68597,'JHU' from dual union all
select 68599,'CORN' from dual union all
select 6860,'DRTE' from dual union all
select 6860,'GSFC' from dual union all
select 6860,'NRL' from dual union all
select 68601,'LEEDS' from dual union all
select 68602,'ISAS' from dual union all
select 68602,'TOYA' from dual union all
select 68603,'DLR' from dual union all
select 68604,'DLR' from dual union all
select 68608,'341MW' from dual union all
select 68608,'90SMW' from dual union all
select 68608,'91MW' from dual union all
select 68609,'UCO' from dual union all
select 68613,'GSFC' from dual union all
select 68613,'UWI' from dual union all
select 68615,'WFF' from dual union all
select 68617,'ESA' from dual union all
select 68625,'MDA' from dual union all
select 68626,'IAP' from dual union all
select 68628,'IAP' from dual union all
select 68629,'IAP' from dual union all
select 68630,'WFF' from dual union all
select 68632,'ISAS' from dual union all
select 68632,'TEIK' from dual union all
select 68633,'341MW' from dual union all
select 68644,'NRL' from dual union all
select 68646,'AEB' from dual union all
select 68646,'DLR' from dual union all
select 68647,'DLR' from dual union all
select 6865,'AFCRL' from dual union all
select 68651,'90SMW' from dual union all
select 68655,'TSAO' from dual union all
select 68658,'IRSA' from dual union all
select 6866,'AFCRL' from dual union all
select 68660,'OSLO' from dual union all
select 68664,'SPFL' from dual union all
select 6867,'BRL' from dual union all
select 68677,'SSC' from dual union all
select 6868,'NRL' from dual union all
select 68680,'IRSA' from dual union all
select 68681,'PRL' from dual union all
select 68682,'NRO' from dual union all
select 68685,'IRSA' from dual union all
select 6869,'UMI' from dual union all
select 6874,'AFCRL' from dual union all
select 68759,'NMSU' from dual union all
select 6876,'AFCRL' from dual union all
select 68762,'JSC' from dual union all
select 68766,'CITED' from dual union all
select 6877,'AFCRL' from dual union all
select 68777,'LAPAN' from dual union all
select 68782,'UAF' from dual union all
select 68783,'ISAS' from dual union all
select 68784,'UIA' from dual union all
select 68785,'UIA' from dual union all
select 68786,'CLEM' from dual union all
select 68787,'CLEM' from dual union all
select 68788,'CLEM' from dual union all
select 68789,'CLEM' from dual union all
select 6879,'NSA' from dual union all
select 68792,'CIT' from dual union all
select 68794,'BERG' from dual union all
select 68794,'FREIB' from dual union all
select 68795,'IAP' from dual union all
select 68795,'TTY' from dual union all
select 68795,'UPC' from dual union all
select 68797,'DART' from dual union all
select 68800,'USAFA' from dual union all
select 68802,'AFRL' from dual union all
select 68802,'DSTO' from dual union all
select 68803,'DLR' from dual union all
select 68804,'ARC' from dual union all
select 68805,'AEB' from dual union all
select 68805,'DLR' from dual union all
select 68807,'MDA' from dual union all
select 68808,'MDA' from dual union all
select 68809,'WFF' from dual union all
select 68810,'91MW' from dual union all
select 68811,'UCO' from dual union all
select 68819,'MDA' from dual union all
select 68820,'WFF' from dual union all
select 68821,'LARC' from dual union all
select 68822,'341MW' from dual union all
select 68823,'NRL' from dual union all
select 68824,'NRL' from dual union all
select 6884,'AFCRL' from dual union all
select 68840,'UCO' from dual union all
select 68841,'DLR' from dual union all
select 68841,'ESA' from dual union all
select 68841,'JAXA' from dual union all
select 68842,'DLR' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 68842,'ESA' from dual union all
select 68843,'RLABN' from dual union all
select 68848,'WFF' from dual union all
select 68849,'ERAU' from dual union all
select 6885,'ICST' from dual union all
select 6885,'UCL' from dual union all
select 68853,'WFF' from dual union all
select 6887,'AFCRL' from dual union all
select 6887,'UMD' from dual union all
select 68906,'NRO' from dual union all
select 6891,'AFCRL' from dual union all
select 6892,'AFCRL' from dual union all
select 68923,'NRO' from dual union all
select 68931,'VSSC' from dual union all
select 68932,'AFRLRZ' from dual union all
select 68932,'DARPA' from dual union all
select 68936,'SCAL' from dual union all
select 68937,'SCAL' from dual union all
select 68938,'SCAL' from dual union all
select 68939,'SCAL' from dual union all
select 6894,'AFCRL' from dual union all
select 68940,'SCAL' from dual union all
select 68941,'SCAL' from dual union all
select 68942,'SCAL' from dual union all
select 68943,'SCAL' from dual union all
select 68944,'SCAL' from dual union all
select 6895,'AFCRL' from dual union all
select 68957,'VSSC' from dual union all
select 68958,'VSSC' from dual union all
select 68959,'VSSC' from dual union all
select 68960,'VSSC' from dual union all
select 68962,'VSSC' from dual union all
select 68969,'CHFEN' from dual union all
select 6897,'AFCRL' from dual union all
select 68970,'VSSC' from dual union all
select 68971,'VSSC' from dual union all
select 68972,'VSSC' from dual union all
select 68973,'VSSC' from dual union all
select 68974,'VSSC' from dual union all
select 68975,'VSSC' from dual union all
select 68976,'VSSC' from dual union all
select 68977,'MDA' from dual union all
select 6898,'AFCRL' from dual union all
select 68981,'IRSA' from dual union all
select 68983,'UAF' from dual union all
select 68985,'MDA' from dual union all
select 68986,'DART' from dual union all
select 68987,'MDA' from dual union all
select 68989,'AACH' from dual union all
select 68989,'IAP' from dual union all
select 68989,'ROSAP' from dual union all
select 6899,'AFCRL' from dual union all
select 68990,'KTH' from dual union all
select 68990,'TUB' from dual union all
select 68991,'ESTEC' from dual union all
select 68991,'SSC' from dual union all
select 68994,'CALP' from dual union all
select 68994,'UKY' from dual union all
select 68994,'WFF' from dual union all
select 68996,'AFRL' from dual union all
select 68996,'DSTO' from dual union all
select 68998,'UCO' from dual union all
select 68999,'SSICE' from dual union all
select 68999,'UPAE' from dual union all
select 6900,'AFCRL' from dual union all
select 69000,'NCU' from dual union all
select 69002,'MDA' from dual union all
select 69003,'UCO' from dual union all
select 69009,'90SMW' from dual union all
select 6901,'AFCRL' from dual union all
select 69010,'UCO' from dual union all
select 69011,'341MW' from dual union all
select 69012,'CIT' from dual union all
select 69016,'MSFC' from dual union all
select 69017,'WFF' from dual union all
select 69020,'SWRI' from dual union all
select 69022,'KAIT' from dual union all
select 69022,'TMU' from dual union all
select 69023,'91MW' from dual union all
select 69025,'WFF' from dual union all
select 69027,'DLR' from dual union all
select 69028,'MDA' from dual union all
select 6903,'NYU' from dual union all
select 69036,'JPL' from dual union all
select 69038,'AEB' from dual union all
select 69039,'INPE' from dual union all
select 69040,'UNH' from dual union all
select 69041,'IAP' from dual union all
select 69042,'IAP' from dual union all
select 69045,'IAP' from dual union all
select 69049,'USN' from dual union all
select 6905,'AFCRL' from dual union all
select 69054,'NRO' from dual union all
select 69058,'NRO' from dual union all
select 6906,'AFCRL' from dual union all
select 69063,'NRO' from dual union all
select 69066,'NRO' from dual union all
select 6907,'AFCRL' from dual union all
select 69077,'IRSA' from dual union all
select 6908,'AFCRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 6909,'QUB' from dual union all
select 691,'APL' from dual union all
select 6910,'LARC' from dual union all
select 6911,'LARC' from dual union all
select 6912,'AFCRL' from dual union all
select 69138,'SCAL' from dual union all
select 69139,'SCAL' from dual union all
select 6914,'NSA' from dual union all
select 69140,'SCAL' from dual union all
select 69141,'SCAL' from dual union all
select 69142,'SCAL' from dual union all
select 69143,'SCAL' from dual union all
select 69144,'SCAL' from dual union all
select 69145,'SCAL' from dual union all
select 69146,'SCAL' from dual union all
select 69147,'SCAL' from dual union all
select 69148,'SCAL' from dual union all
select 69149,'SCAL' from dual union all
select 6915,'AFCRL' from dual union all
select 69150,'SCAL' from dual union all
select 69151,'SCAL' from dual union all
select 69152,'SCAL' from dual union all
select 69153,'SCAL' from dual union all
select 69154,'AFRLRZ' from dual union all
select 69154,'DARPA' from dual union all
select 69155,'WFF' from dual union all
select 69158,'IRSA' from dual union all
select 69159,'ARMAD' from dual union all
select 69160,'SCAL' from dual union all
select 69161,'SCAL' from dual union all
select 69162,'SCAL' from dual union all
select 69177,'ISRO' from dual union all
select 69184,'UCO' from dual union all
select 69185,'VAP' from dual union all
select 69186,'EUROL' from dual union all
select 69187,'EUROL' from dual union all
select 69190,'IRSA' from dual union all
select 69192,'DLR' from dual union all
select 69193,'UCO' from dual union all
select 69197,'WFF' from dual union all
select 69198,'CSSAR' from dual union all
select 6920,'GSFC' from dual union all
select 69200,'SSICE' from dual union all
select 69200,'UPAE' from dual union all
select 69201,'WFF' from dual union all
select 69204,'WFF' from dual union all
select 69205,'91MW' from dual union all
select 6921,'GSFC' from dual union all
select 69210,'GSFC' from dual union all
select 69211,'GSFC' from dual union all
select 69212,'CITED' from dual union all
select 69214,'WFF' from dual union all
select 69215,'341MW' from dual union all
select 69223,'MISU' from dual union all
select 69231,'UCO' from dual union all
select 69232,'UCO' from dual union all
select 69236,'GSFC' from dual union all
select 69236,'UWI' from dual union all
select 69238,'BU' from dual union all
select 69242,'DLR' from dual union all
select 69243,'AEB' from dual union all
select 69244,'AEB' from dual union all
select 69244,'DLR' from dual union all
select 69245,'DLR' from dual union all
select 69245,'IAE' from dual union all
select 69245,'INPE' from dual union all
select 69246,'OSLO' from dual union all
select 69247,'RGMS' from dual union all
select 69251,'ISAS' from dual union all
select 69252,'UCO' from dual union all
select 6926,'CNRS' from dual union all
select 6926,'PARIS' from dual union all
select 69260,'IRSA' from dual union all
select 69269,'NRO' from dual union all
select 6928,'AFCRL' from dual union all
select 69288,'NRO' from dual union all
select 69289,'NRO' from dual union all
select 6929,'AN' from dual union all
select 69303,'NRO' from dual union all
select 6932,'FSU' from dual union all
select 6932,'GSFC' from dual union all
select 69332,'AFRL' from dual union all
select 69332,'DSTO' from dual union all
select 69332,'NASA' from dual union all
select 69333,'SCAL' from dual union all
select 69334,'SCAL' from dual union all
select 69335,'SCAL' from dual union all
select 69336,'SCAL' from dual union all
select 69337,'AFRLRZ' from dual union all
select 69337,'DARPA' from dual union all
select 69338,'SCAL' from dual union all
select 69339,'SCAL' from dual union all
select 6934,'AFCRL' from dual union all
select 6935,'AN' from dual union all
select 69350,'AFRL' from dual union all
select 69350,'DSTO' from dual union all
select 69356,'SCAL' from dual union all
select 69357,'ARMAD' from dual union all
select 69358,'ARMAD' from dual union all
select 69359,'IRSA' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 69361,'IRSA' from dual union all
select 69362,'IRSA' from dual union all
select 69363,'EUROL' from dual union all
select 69369,'WFF' from dual union all
select 69370,'ISAS' from dual union all
select 69371,'ARMAD' from dual union all
select 69373,'SSC' from dual union all
select 69374,'CORN' from dual union all
select 69375,'90SMW' from dual union all
select 69377,'EUROL' from dual union all
select 69378,'CIT' from dual union all
select 69379,'CLEM' from dual union all
select 6938,'ISAS' from dual union all
select 6938,'RRL' from dual union all
select 69380,'CLEM' from dual union all
select 69381,'CLEM' from dual union all
select 69382,'CLEM' from dual union all
select 69383,'CLEM' from dual union all
select 69384,'AFORS' from dual union all
select 69384,'UPAE' from dual union all
select 6939,'FIAN' from dual union all
select 69397,'WFF' from dual union all
select 69398,'DLR' from dual union all
select 69399,'UCO' from dual union all
select 69402,'MSFC' from dual union all
select 69406,'MSFC' from dual union all
select 69407,'LARCN' from dual union all
select 69408,'USCLA' from dual union all
select 69409,'ISAS' from dual union all
select 6941,'AN' from dual union all
select 69415,'NSWCWS' from dual union all
select 6942,'CARDE' from dual union all
select 69420,'WFF' from dual union all
select 69421,'USAF' from dual union all
select 69421,'WFF' from dual union all
select 69422,'AFRL' from dual union all
select 69422,'DSTO' from dual union all
select 69433,'UCB' from dual union all
select 69434,'341MW' from dual union all
select 69436,'BU' from dual union all
select 69436,'UMAL' from dual union all
select 69437,'UMIAMI' from dual union all
select 69438,'DLR' from dual union all
select 69440,'INPE' from dual union all
select 69441,'HOKK' from dual union all
select 69441,'TOK' from dual union all
select 69441,'TOKAI' from dual union all
select 6945,'ISAS' from dual union all
select 6945,'RRL' from dual union all
select 69457,'RGMS' from dual union all
select 69458,'WFF' from dual union all
select 69459,'WFF' from dual union all
select 6946,'GSFC' from dual union all
select 6947,'AFCRL' from dual union all
select 6949,'AFCRL' from dual union all
select 6949,'ASE' from dual union all
select 69506,'NRO' from dual union all
select 6953,'BRL' from dual union all
select 69535,'NRO' from dual union all
select 69542,'ARMAD' from dual union all
select 69545,'SCAL' from dual union all
select 69546,'SCAL' from dual union all
select 69548,'SCAL' from dual union all
select 69549,'AFRLRZ' from dual union all
select 69549,'DARPA' from dual union all
select 69554,'SCAL' from dual union all
select 69555,'SCAL' from dual union all
select 69556,'SCAL' from dual union all
select 69559,'UQU' from dual union all
select 69567,'SCAL' from dual union all
select 69568,'IIAE' from dual union all
select 69593,'ISRO' from dual union all
select 69597,'CHFEN' from dual union all
select 69598,'IRSA' from dual union all
select 69599,'GSFC' from dual union all
select 69599,'WFF' from dual union all
select 69600,'GSFC' from dual union all
select 69604,'BU' from dual union all
select 69605,'IRSA' from dual union all
select 69607,'341MW' from dual union all
select 69611,'EUROL' from dual union all
select 69612,'EUROL' from dual union all
select 69613,'NCU' from dual union all
select 69627,'DLR' from dual union all
select 69629,'CSSAR' from dual union all
select 69631,'UCO' from dual union all
select 69632,'GSFC' from dual union all
select 69633,'WFF' from dual union all
select 69634,'AFRL' from dual union all
select 69635,'JHU' from dual union all
select 69636,'UIUC' from dual union all
select 69637,'UIUC' from dual union all
select 69638,'AFRL' from dual union all
select 69641,'91MW' from dual union all
select 69642,'CIT' from dual union all
select 69644,'UCO' from dual union all
select 69645,'DFRC' from dual union all
select 69645,'UPAE' from dual union all
select 69649,'GSFC' from dual union all
select 69650,'GSFC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 69651,'MDA' from dual union all
select 69652,'MDA' from dual union all
select 69653,'DLR' from dual union all
select 69654,'ISAS' from dual union all
select 69655,'ISAS' from dual union all
select 69657,'NRL' from dual union all
select 69659,'UCO' from dual union all
select 6966,'CARDE' from dual union all
select 69671,'IAP' from dual union all
select 69671,'IRSS' from dual union all
select 69672,'IAP' from dual union all
select 69672,'IRSS' from dual union all
select 69673,'IAP' from dual union all
select 69673,'IRSS' from dual union all
select 69674,'IAP' from dual union all
select 69674,'IRSS' from dual union all
select 69675,'IAP' from dual union all
select 69675,'IRSS' from dual union all
select 69676,'IAP' from dual union all
select 69676,'IRSS' from dual union all
select 69677,'IAP' from dual union all
select 69677,'IRSS' from dual union all
select 69678,'IAP' from dual union all
select 69678,'IRSS' from dual union all
select 69679,'IAP' from dual union all
select 69679,'IRSS' from dual union all
select 6968,'GSFC' from dual union all
select 69680,'IAP' from dual union all
select 69680,'IRSS' from dual union all
select 69681,'IAP' from dual union all
select 69681,'IRSS' from dual union all
select 69682,'IAP' from dual union all
select 69682,'IRSS' from dual union all
select 69683,'91MW' from dual union all
select 69690,'90SMW' from dual union all
select 69696,'UCO' from dual union all
select 697,'GEMTS' from dual union all
select 69701,'GSFC' from dual union all
select 69701,'UWI' from dual union all
select 69702,'DFRC' from dual union all
select 69702,'UPAE' from dual union all
select 69703,'JHU' from dual union all
select 69724,'NRO' from dual union all
select 6973,'DRTE' from dual union all
select 69731,'NRO' from dual union all
select 69747,'AFSPC' from dual union all
select 6977,'AFCRL' from dual union all
select 6977,'UMD' from dual union all
select 6978,'CARDE' from dual union all
select 69785,'NRO' from dual union all
select 69789,'KHRU' from dual union all
select 6979,'CARDE' from dual union all
select 69795,'SCAL' from dual union all
select 69797,'SCAL' from dual union all
select 69805,'WFF' from dual union all
select 69825,'SCAL' from dual union all
select 69826,'SCAL' from dual union all
select 69828,'SCAL' from dual union all
select 69830,'SCAL' from dual union all
select 69836,'LARCNE' from dual union all
select 69850,'ISRO' from dual union all
select 69851,'ISRO' from dual union all
select 69852,'ISRO' from dual union all
select 69853,'ISRO' from dual union all
select 69855,'MDA' from dual union all
select 69856,'MDA' from dual union all
select 69857,'MDA' from dual union all
select 69859,'SWRI' from dual union all
select 6986,'AFCRL' from dual union all
select 6986,'UMI' from dual union all
select 69871,'RGMS' from dual union all
select 69877,'UAZ' from dual union all
select 69880,'EUROL' from dual union all
select 69881,'EUROL' from dual union all
select 69884,'UCO' from dual union all
select 69885,'MDA' from dual union all
select 69886,'MDA' from dual union all
select 69887,'UCO' from dual union all
select 69890,'KHRU' from dual union all
select 69894,'CHFEN' from dual union all
select 69895,'USCLA' from dual union all
select 69896,'ISAS' from dual union all
select 69897,'ISAS' from dual union all
select 69898,'MDA' from dual union all
select 69898,'NSWCWS' from dual union all
select 6990,'GSFC' from dual union all
select 69901,'GSFC' from dual union all
select 69901,'WFF' from dual union all
select 69902,'IAE' from dual union all
select 69910,'91MW' from dual union all
select 69911,'NRL' from dual union all
select 69913,'MDA' from dual union all
select 69913,'NSWCWS' from dual union all
select 69914,'NCU' from dual union all
select 69915,'NCU' from dual union all
select 69917,'DFRC' from dual union all
select 69917,'UPAE' from dual union all
select 69918,'CSXT' from dual union all
select 69924,'SWRI' from dual union all
select 69930,'UCB' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 69931,'UAF' from dual union all
select 69936,'NSWCPH' from dual union all
select 69944,'IRSA' from dual union all
select 6995,'DRTE' from dual union all
select 69963,'AFSPC' from dual union all
select 6997,'LARC' from dual union all
select 69996,'NRO' from dual union all
select 700,'GEMTS' from dual union all
select 7000,'AFCRL' from dual union all
select 7000,'UMD' from dual union all
select 7002,'DRTE' from dual union all
select 70024,'SPX' from dual union all
select 70026,'UCO' from dual union all
select 70039,'DARE' from dual union all
select 70064,'ADD' from dual union all
select 7007,'UMI' from dual union all
select 7008,'GSFC' from dual union all
select 70083,'ISRO' from dual union all
select 70084,'UAF' from dual union all
select 70085,'CLEM' from dual union all
select 70086,'UAF' from dual union all
select 70087,'CLEM' from dual union all
select 70088,'USU' from dual union all
select 70091,'MDA' from dual union all
select 70092,'MDA' from dual union all
select 70093,'MDA' from dual union all
select 70094,'AFRL' from dual union all
select 70097,'OSLO' from dual union all
select 70098,'CNES' from dual union all
select 70099,'IAP' from dual union all
select 70099,'IRSS' from dual union all
select 701,'JPL' from dual union all
select 70101,'90SMW' from dual union all
select 70103,'EUROL' from dual union all
select 70104,'EUROL' from dual union all
select 70105,'341MW' from dual union all
select 70107,'UCO' from dual union all
select 70109,'DLR' from dual union all
select 70110,'DLR' from dual union all
select 70111,'BLOR' from dual union all
select 70112,'UIA' from dual union all
select 70113,'90SMW' from dual union all
select 70116,'UCO' from dual union all
select 70119,'WFF' from dual union all
select 7012,'AFCRL' from dual union all
select 70120,'AFRL' from dual union all
select 70120,'DSTO' from dual union all
select 70120,'NASA' from dual union all
select 70121,'DLR' from dual union all
select 70122,'UCO' from dual union all
select 70125,'91MW' from dual union all
select 70128,'MSU' from dual union all
select 70129,'UAH' from dual union all
select 70130,'HOKK' from dual union all
select 70130,'TOK' from dual union all
select 70130,'TOKAI' from dual union all
select 70131,'NRL' from dual union all
select 70133,'SSC' from dual union all
select 70134,'WFF' from dual union all
select 70136,'90SMW' from dual union all
select 70137,'SSC' from dual union all
select 7014,'AFCRL' from dual union all
select 70147,'CHFEN' from dual union all
select 70153,'DFRC' from dual union all
select 70153,'UPAE' from dual union all
select 70157,'BLOR' from dual union all
select 70158,'BU' from dual union all
select 70160,'DART' from dual union all
select 70161,'UMIAMI' from dual union all
select 70163,'SSC' from dual union all
select 7017,'AFSWC' from dual union all
select 70173,'UNH' from dual union all
select 70175,'JHU' from dual union all
select 70176,'RGMS' from dual union all
select 70188,'NRO' from dual union all
select 7019,'LPARL' from dual union all
select 702,'JPL' from dual union all
select 7020,'NRL' from dual union all
select 7021,'GSFC' from dual union all
select 70214,'NRO' from dual union all
select 7022,'GSFC' from dual union all
select 70225,'NRO' from dual union all
select 70230,'AFSPC' from dual union all
select 7024,'GSFC' from dual union all
select 7025,'NSA' from dual union all
select 7026,'GSFC' from dual union all
select 7027,'GSFC' from dual union all
select 7029,'GSFC' from dual union all
select 7030,'RAE' from dual union all
select 7030,'WRE' from dual union all
select 70300,'SCAL' from dual union all
select 70304,'SCAL' from dual union all
select 70305,'SCAL' from dual union all
select 70306,'SCAL' from dual union all
select 70307,'SCAL' from dual union all
select 7031,'UMI' from dual union all
select 70310,'SCAL' from dual union all
select 70315,'ISRO' from dual union all
select 70316,'VSSC' from dual union all
select 70320,'ISAS' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 70321,'BLOR' from dual union all
select 70322,'DLR' from dual union all
select 70324,'MDA' from dual union all
select 70325,'SSC' from dual union all
select 70326,'91MW' from dual union all
select 70327,'91MW' from dual union all
select 70328,'UCO' from dual union all
select 70329,'WFF' from dual union all
select 7033,'GSFC' from dual union all
select 70330,'ARC' from dual union all
select 70342,'BLOR' from dual union all
select 70347,'CSSAR' from dual union all
select 70349,'AFRL' from dual union all
select 70349,'DSTO' from dual union all
select 70349,'UQU' from dual union all
select 7035,'GSFC' from dual union all
select 70353,'EUROL' from dual union all
select 70354,'EUROL' from dual union all
select 70355,'UCO' from dual union all
select 70356,'BLOR' from dual union all
select 70359,'UCO' from dual union all
select 7036,'AFCRL' from dual union all
select 70360,'ARRS' from dual union all
select 70360,'TROM' from dual union all
select 70365,'AACHR' from dual union all
select 70365,'DLR' from dual union all
select 70366,'MSFC' from dual union all
select 70368,'UCO' from dual union all
select 70369,'ARRS' from dual union all
select 70369,'TROM' from dual union all
select 7037,'WSMR' from dual union all
select 70374,'341MW' from dual union all
select 70379,'BLOR' from dual union all
select 7038,'ICST' from dual union all
select 7038,'QUB' from dual union all
select 7038,'UCL' from dual union all
select 70384,'SMDC' from dual union all
select 7039,'AFCRL' from dual union all
select 704,'JPL' from dual union all
select 7040,'GSFC' from dual union all
select 70401,'NRO' from dual union all
select 7041,'GSFC' from dual union all
select 7042,'AEC' from dual union all
select 7043,'AEC' from dual union all
select 7044,'GSFC' from dual union all
select 70443,'SCAL' from dual union all
select 70446,'SCAL' from dual union all
select 70447,'SCAL' from dual union all
select 7045,'GSFC' from dual union all
select 7045,'OKSU' from dual union all
select 70450,'SCAL' from dual union all
select 70453,'ISAS' from dual union all
select 7046,'LUX' from dual union all
select 7046,'RAE' from dual union all
select 7046,'UCW' from dual union all
select 70461,'ZARM' from dual union all
select 70463,'VAP' from dual union all
select 70468,'91MW' from dual union all
select 70474,'DART' from dual union all
select 70475,'GSFC' from dual union all
select 70476,'GSFC' from dual union all
select 70477,'DART' from dual union all
select 70482,'EUROL' from dual union all
select 70483,'EUROL' from dual union all
select 70486,'ESTEC' from dual union all
select 70486,'SSC' from dual union all
select 70487,'90SMW' from dual union all
select 70488,'SWRI' from dual union all
select 70489,'DLR' from dual union all
select 70491,'341MW' from dual union all
select 70492,'WFF' from dual union all
select 70496,'MDA' from dual union all
select 705,'APL' from dual union all
select 7050,'GSFC' from dual union all
select 7050,'OKSU' from dual union all
select 70505,'UCO' from dual union all
select 70509,'WFF' from dual union all
select 7051,'NRL' from dual union all
select 70511,'UCO' from dual union all
select 70513,'US8A' from dual union all
select 70517,'AFRL' from dual union all
select 70517,'DSTO' from dual union all
select 70517,'UQU' from dual union all
select 70519,'CHFEN' from dual union all
select 7052,'LARC' from dual union all
select 70522,'US8A' from dual union all
select 70526,'90SMW' from dual union all
select 706,'NRL' from dual union all
select 7060,'AFCRL' from dual union all
select 7063,'AFCRL' from dual union all
select 7063,'UMD' from dual union all
select 7065,'GCA' from dual union all
select 7066,'GCA' from dual union all
select 7068,'GCA' from dual union all
select 7069,'JHU' from dual union all
select 707,'AFCRL' from dual union all
select 707,'UMI' from dual union all
select 7073,'GSFC' from dual union all
select 7075,'AFCRL' from dual union all
select 7075,'UMD' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 7076,'AFCRL' from dual union all
select 7076,'UMD' from dual union all
select 7077,'GSFC' from dual union all
select 708,'NRL' from dual union all
select 7080,'LARC' from dual union all
select 7083,'AFCRL' from dual union all
select 7083,'UMD' from dual union all
select 709,'APL' from dual union all
select 709,'YERK' from dual union all
select 710,'APL' from dual union all
select 7103,'SAND' from dual union all
select 7104,'WRE' from dual union all
select 7105,'WRE' from dual union all
select 7106,'WRE' from dual union all
select 7107,'WRE' from dual union all
select 711,'SCEL' from dual union all
select 7112,'GSFC' from dual union all
select 7113,'CNRS' from dual union all
select 7115,'GSFC' from dual union all
select 7116,'CNRS' from dual union all
select 712,'NRL' from dual union all
select 7120,'NSA' from dual union all
select 7121,'UMI' from dual union all
select 7127,'WRE' from dual union all
select 713,'APL' from dual union all
select 7132,'GSFC' from dual union all
select 7134,'LARC' from dual union all
select 7138,'LARC' from dual union all
select 7139,'AFCRL' from dual union all
select 714,'NRL' from dual union all
select 714,'UMI' from dual union all
select 7141,'BRL' from dual union all
select 7142,'RAE' from dual union all
select 7142,'WRE' from dual union all
select 7144,'NRL' from dual union all
select 7147,'ICST' from dual union all
select 7147,'UCL' from dual union all
select 7148,'AFCRL' from dual union all
select 7150,'GSFC' from dual union all
select 7152,'GSFC' from dual union all
select 7158,'GSFC' from dual union all
select 7167,'CARDE' from dual union all
select 7169,'AFSWC' from dual union all
select 7170,'AFSWC' from dual union all
select 7171,'PMR' from dual union all
select 7172,'PMR' from dual union all
select 7173,'PMR' from dual union all
select 7174,'WSMR' from dual union all
select 7175,'WSMR' from dual union all
select 7176,'WSMR' from dual union all
select 7177,'WSMR' from dual union all
select 7178,'WSMR' from dual union all
select 7179,'WSMR' from dual union all
select 7180,'WSMR' from dual union all
select 7181,'WSMR' from dual union all
select 7182,'WSMR' from dual union all
select 7183,'WSMR' from dual union all
select 7184,'WSMR' from dual union all
select 7185,'WSMR' from dual union all
select 7186,'WSMR' from dual union all
select 7187,'WSMR' from dual union all
select 7188,'WSMR' from dual union all
select 7189,'WSMR' from dual union all
select 7190,'WSMR' from dual union all
select 7191,'WSMR' from dual union all
select 7192,'WSMR' from dual union all
select 7193,'WSMR' from dual union all
select 7194,'SIMED' from dual union all
select 7195,'AFCRL' from dual union all
select 7196,'US' from dual union all
select 7197,'AFCRL' from dual union all
select 7198,'AFCRL' from dual union all
select 7199,'WSMR' from dual union all
select 7200,'WSMR' from dual union all
select 7201,'WSMR' from dual union all
select 7202,'WSMR' from dual union all
select 7203,'WSMR' from dual union all
select 7204,'WSMR' from dual union all
select 7205,'WSMR' from dual union all
select 7206,'WSMR' from dual union all
select 7207,'WSMR' from dual union all
select 7208,'WSMR' from dual union all
select 7209,'AFCRL' from dual union all
select 7211,'AFCRL' from dual union all
select 7212,'AFCRL' from dual union all
select 7216,'SRDL' from dual union all
select 7217,'SRDL' from dual union all
select 7218,'SRDL' from dual union all
select 7219,'SRDL' from dual union all
select 722,'AFCRL' from dual union all
select 722,'UMI' from dual union all
select 7220,'SRDL' from dual union all
select 7221,'SRDL' from dual union all
select 7222,'SRDL' from dual union all
select 7223,'SRDL' from dual union all
select 7224,'SRDL' from dual union all
select 7225,'SRDL' from dual union all
select 7226,'SRDL' from dual union all
select 7227,'SRDL' from dual union all
select 7228,'SRDL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 7229,'SRDL' from dual union all
select 7230,'SRDL' from dual union all
select 7231,'SRDL' from dual union all
select 7232,'SRDL' from dual union all
select 7233,'SRDL' from dual union all
select 7234,'SRDL' from dual union all
select 7235,'SRDL' from dual union all
select 7236,'SRDL' from dual union all
select 7237,'PMR' from dual union all
select 7238,'PMR' from dual union all
select 7239,'PMR' from dual union all
select 724,'SCEL' from dual union all
select 724,'UMI' from dual union all
select 7240,'PMR' from dual union all
select 7241,'PMR' from dual union all
select 7242,'PMR' from dual union all
select 7243,'PMR' from dual union all
select 7244,'HADC' from dual union all
select 7245,'NRL' from dual union all
select 7246,'NRL' from dual union all
select 7247,'AFCRL' from dual union all
select 7248,'AFCRL' from dual union all
select 7284,'WSMR' from dual union all
select 7286,'NMSU' from dual union all
select 7288,'WSMR' from dual union all
select 7289,'WSMR' from dual union all
select 7294,'LARC' from dual union all
select 7297,'LARC' from dual union all
select 7298,'LARC' from dual union all
select 7306,'GSFC' from dual union all
select 7318,'UMI' from dual union all
select 7319,'AFSWC' from dual union all
select 7323,'GSFC' from dual union all
select 7365,'AFCRL' from dual union all
select 7366,'LARC' from dual union all
select 7367,'LARC' from dual union all
select 7368,'LARC' from dual union all
select 7385,'AFCRL' from dual union all
select 7405,'HOG' from dual union all
select 7406,'GCA' from dual union all
select 7419,'NOTS' from dual union all
select 7421,'NOTS' from dual union all
select 7421,'NRL' from dual union all
select 7428,'LARC' from dual union all
select 7436,'USN' from dual union all
select 7441,'AFCRL' from dual union all
select 7445,'LARC' from dual union all
select 7446,'LUX' from dual union all
select 7446,'RAE' from dual union all
select 7446,'UCL' from dual union all
select 745,'ORD' from dual union all
select 7461,'UMI' from dual union all
select 7462,'UMI' from dual union all
select 7463,'ISAS' from dual union all
select 7469,'USAF' from dual union all
select 7476,'AFCRL' from dual union all
select 7483,'ISAS' from dual union all
select 7484,'USAF' from dual union all
select 7485,'PMR' from dual union all
select 7486,'WSMR' from dual union all
select 7487,'WSMR' from dual union all
select 7488,'WSMR' from dual union all
select 7489,'WSMR' from dual union all
select 7490,'WSMR' from dual union all
select 7491,'WSMR' from dual union all
select 7503,'LPA' from dual union all
select 7504,'LARC' from dual union all
select 7506,'LPA' from dual union all
select 7555,'LARC' from dual union all
select 7576,'SAND' from dual union all
select 7578,'GSFC' from dual union all
select 7580,'WRE' from dual union all
select 7585,'GSFC' from dual union all
select 7586,'CNES' from dual union all
select 7587,'CNES' from dual union all
select 7588,'ICST' from dual union all
select 7588,'UCL' from dual union all
select 7589,'GSFC' from dual union all
select 7590,'AFCRL' from dual union all
select 7595,'HOG' from dual union all
select 7596,'GSFC' from dual union all
select 7597,'AFCRL' from dual union all
select 7598,'GSFC' from dual union all
select 7600,'GSFC' from dual union all
select 7601,'GSFC' from dual union all
select 7602,'GSFC' from dual union all
select 7603,'GSFC' from dual union all
select 7604,'NMSU' from dual union all
select 7605,'NMSU' from dual union all
select 7607,'USAF' from dual union all
select 7608,'ATLA' from dual union all
select 7609,'ATLA' from dual union all
select 7610,'WADD' from dual union all
select 7611,'SAND' from dual union all
select 7612,'WRE' from dual union all
select 7613,'WRE' from dual union all
select 7619,'AFSWC' from dual union all
select 7620,'NMC' from dual union all
select 7624,'ICST' from dual union all
select 7624,'UCL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 7625,'GSFC' from dual union all
select 763,'NRL' from dual union all
select 763,'ORD' from dual union all
select 7630,'AFCRL' from dual union all
select 7632,'GSFC' from dual union all
select 7633,'NRL' from dual union all
select 7634,'TAO' from dual union all
select 7634,'TOK' from dual union all
select 7637,'GCA' from dual union all
select 7638,'ROSAP' from dual union all
select 7639,'ROSAP' from dual union all
select 7640,'ROSAP' from dual union all
select 7641,'ROSAP' from dual union all
select 7642,'HCO' from dual union all
select 7643,'GCA' from dual union all
select 7645,'GCA' from dual union all
select 7646,'GCA' from dual union all
select 7647,'LERC' from dual union all
select 7651,'GSFC' from dual union all
select 7654,'AFCRL' from dual union all
select 7654,'UMD' from dual union all
select 7656,'UCL' from dual union all
select 7660,'GSFC' from dual union all
select 7661,'GSFC' from dual union all
select 7663,'GSFC' from dual union all
select 7665,'ICST' from dual union all
select 7665,'UCL' from dual union all
select 7669,'AFCRL' from dual union all
select 7670,'AFCRL' from dual union all
select 7675,'ROSAP' from dual union all
select 7679,'LARC' from dual union all
select 768,'NRL' from dual union all
select 7684,'AFCRL' from dual union all
select 7685,'GSFC' from dual union all
select 7688,'CNET' from dual union all
select 769,'AFCRL' from dual union all
select 7690,'CNET' from dual union all
select 7691,'RAE' from dual union all
select 7691,'WRE' from dual union all
select 7694,'AFCRL' from dual union all
select 7695,'UMI' from dual union all
select 7696,'RAE' from dual union all
select 7696,'WRE' from dual union all
select 7698,'ICST' from dual union all
select 7698,'JBO' from dual union all
select 7698,'UCL' from dual union all
select 7702,'UMI' from dual union all
select 7703,'CNRS' from dual union all
select 7705,'GSFC' from dual union all
select 7707,'CRC' from dual union all
select 7707,'DRTE' from dual union all
select 771,'NRL' from dual union all
select 7710,'NRL' from dual union all
select 7711,'TOK' from dual union all
select 7713,'NRL' from dual union all
select 7714,'AFCRL' from dual union all
select 7716,'CHAIL' from dual union all
select 7716,'NBS' from dual union all
select 7718,'GSFC' from dual union all
select 772,'GEMTS' from dual union all
select 773,'APL' from dual union all
select 7735,'GSFC' from dual union all
select 7736,'UNH' from dual union all
select 7737,'UNH' from dual union all
select 7738,'GSFC' from dual union all
select 774,'NRL' from dual union all
select 774,'SCEL' from dual union all
select 774,'UMI' from dual union all
select 7740,'GSFC' from dual union all
select 7744,'GSFC' from dual union all
select 7745,'UNH' from dual union all
select 7746,'ISAS' from dual union all
select 7746,'KYO' from dual union all
select 775,'APL' from dual union all
select 7751,'LARC' from dual union all
select 7753,'UMI' from dual union all
select 7759,'UCL' from dual union all
select 776,'ORD' from dual union all
select 7762,'GSFC' from dual union all
select 7768,'AFCRL' from dual union all
select 777,'APL' from dual union all
select 7776,'LERC' from dual union all
select 7778,'AFCRL' from dual union all
select 7779,'GSFC' from dual union all
select 778,'APL' from dual union all
select 7782,'GCA' from dual union all
select 7788,'AFCRL' from dual union all
select 779,'APL' from dual union all
select 779,'UMI' from dual union all
select 7792,'AFCRL' from dual union all
select 7793,'NRL' from dual union all
select 7796,'LARC' from dual union all
select 780,'NRL' from dual union all
select 7808,'ROSAP' from dual union all
select 7809,'ROSAP' from dual union all
select 781,'NRL' from dual union all
select 7810,'ROSAP' from dual union all
select 7817,'AFCRL' from dual union all
select 782,'SCEL' from dual union all
select 782,'UMI' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 7820,'GCA' from dual union all
select 7821,'GCA' from dual union all
select 7824,'GCA' from dual union all
select 7825,'GSFC' from dual union all
select 7827,'LARC' from dual union all
select 7829,'GCA' from dual union all
select 783,'NRL' from dual union all
select 783,'ORD' from dual union all
select 7830,'GSFC' from dual union all
select 7830,'UMI' from dual union all
select 7831,'GSFC' from dual union all
select 7833,'AN' from dual union all
select 7838,'BRL' from dual union all
select 7840,'ICST' from dual union all
select 7840,'LUX' from dual union all
select 7840,'UCL' from dual union all
select 7857,'GSFC' from dual union all
select 7862,'GSFC' from dual union all
select 7863,'ONERA' from dual union all
select 7868,'ONERA' from dual union all
select 7871,'GSFC' from dual union all
select 7874,'AFCRL' from dual union all
select 7875,'ASE' from dual union all
select 7876,'GSFC' from dual union all
select 7879,'LERC' from dual union all
select 7882,'AN' from dual union all
select 7883,'AN' from dual union all
select 7884,'AN' from dual union all
select 789,'APL' from dual union all
select 789,'NRL' from dual union all
select 7891,'JBO' from dual union all
select 7891,'LUX' from dual union all
select 7891,'UCL' from dual union all
select 7892,'ISAS' from dual union all
select 7894,'AN' from dual union all
select 7895,'ASE' from dual union all
select 7897,'AN' from dual union all
select 7898,'AN' from dual union all
select 7901,'GCA' from dual union all
select 7905,'LERC' from dual union all
select 7906,'ISAS' from dual union all
select 7909,'NRL' from dual union all
select 7910,'GSFC' from dual union all
select 7915,'JBO' from dual union all
select 7915,'RAE' from dual union all
select 7915,'UCL' from dual union all
select 7917,'JBO' from dual union all
select 7917,'MO' from dual union all
select 7917,'UCL' from dual union all
select 7919,'NMSU' from dual union all
select 7923,'AN' from dual union all
select 7924,'FSU' from dual union all
select 7924,'GSFC' from dual union all
select 7928,'AFSWC' from dual union all
select 7929,'LARC' from dual union all
select 793,'NRL' from dual union all
select 793,'UMI' from dual union all
select 7930,'GSFC' from dual union all
select 7931,'GSFC' from dual union all
select 7934,'AFCRL' from dual union all
select 794,'NRL' from dual union all
select 794,'SCEL' from dual union all
select 794,'UMI' from dual union all
select 7947,'NRL' from dual union all
select 795,'UMI' from dual union all
select 7956,'BIR' from dual union all
select 7956,'LUX' from dual union all
select 7956,'UCL' from dual union all
select 7957,'CNRS' from dual union all
select 7958,'CNRS' from dual union all
select 7963,'AFCRL' from dual union all
select 7965,'AFCRL' from dual union all
select 7966,'CNRS' from dual union all
select 7967,'CNRS' from dual union all
select 7968,'CNRS' from dual union all
select 7969,'GSFC' from dual union all
select 797,'ORD' from dual union all
select 7970,'DFF' from dual union all
select 7970,'LPA' from dual union all
select 7973,'AFCRL' from dual union all
select 7976,'LARC' from dual union all
select 7977,'WRE' from dual union all
select 7979,'AFCRL' from dual union all
select 798,'NRL' from dual union all
select 7982,'AFCRL' from dual union all
select 7983,'AFCRL' from dual union all
select 7984,'UMI' from dual union all
select 7985,'GSFC' from dual union all
select 7986,'UMI' from dual union all
select 7991,'ISAS' from dual union all
select 7992,'ICST' from dual union all
select 7992,'UCL' from dual union all
select 800,'APL' from dual union all
select 8001,'NMC' from dual union all
select 8002,'ATLA' from dual union all
select 8003,'AFCRL' from dual union all
select 8003,'BRL' from dual union all
select 8004,'ATLA' from dual union all
select 8005,'ATLA' from dual union all
select 8006,'WSMR' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 8007,'SAND' from dual union all
select 8008,'NMSU' from dual union all
select 8009,'NMSU' from dual union all
select 801,'APL' from dual union all
select 8012,'WRE' from dual union all
select 8014,'GSFC' from dual union all
select 8025,'FIAN' from dual union all
select 8035,'LPA' from dual union all
select 8037,'GSFC' from dual union all
select 8038,'AFCRL' from dual union all
select 8039,'AFCRL' from dual union all
select 8040,'AFCRL' from dual union all
select 8040,'BRL' from dual union all
select 8045,'CERMA' from dual union all
select 8046,'AFCRL' from dual union all
select 8049,'AFCRL' from dual union all
select 8053,'AFSWC' from dual union all
select 8055,'AFCRL' from dual union all
select 8059,'ICST' from dual union all
select 8059,'QUB' from dual union all
select 8059,'UCL' from dual union all
select 8061,'AFCRL' from dual union all
select 8062,'AFCRL' from dual union all
select 8068,'AFCRL' from dual union all
select 8070,'GSFC' from dual union all
select 8075,'AFCRL' from dual union all
select 8079,'UMI' from dual union all
select 808,'NRL' from dual union all
select 8080,'ISAS' from dual union all
select 8080,'RRL' from dual union all
select 8086,'ISAS' from dual union all
select 8087,'AFSWC' from dual union all
select 8088,'AFCRL' from dual union all
select 8089,'GSFC' from dual union all
select 809,'JPL' from dual union all
select 8090,'AFSWC' from dual union all
select 8091,'WSMR' from dual union all
select 8092,'WSMR' from dual union all
select 8093,'WSMR' from dual union all
select 8094,'WSMR' from dual union all
select 8095,'WSMR' from dual union all
select 8096,'WSMR' from dual union all
select 8097,'WSMR' from dual union all
select 8098,'WSMR' from dual union all
select 8099,'WSMR' from dual union all
select 8100,'WSMR' from dual union all
select 8101,'WSMR' from dual union all
select 8102,'WSMR' from dual union all
select 8103,'USAF' from dual union all
select 8104,'USAF' from dual union all
select 8105,'USAF' from dual union all
select 8106,'USAF' from dual union all
select 8107,'USAF' from dual union all
select 8108,'USAF' from dual union all
select 8109,'USAF' from dual union all
select 8110,'USAF' from dual union all
select 8111,'USAF' from dual union all
select 8112,'USAF' from dual union all
select 8113,'USAF' from dual union all
select 8114,'USAF' from dual union all
select 8115,'USAF' from dual union all
select 8116,'USAF' from dual union all
select 8117,'USAF' from dual union all
select 8118,'USAF' from dual union all
select 8119,'USAF' from dual union all
select 8120,'USAF' from dual union all
select 8121,'USAF' from dual union all
select 8122,'USAF' from dual union all
select 8123,'USAF' from dual union all
select 8124,'USAF' from dual union all
select 8125,'USAF' from dual union all
select 8126,'USAF' from dual union all
select 8127,'USAF' from dual union all
select 8128,'USAF' from dual union all
select 8129,'NOL' from dual union all
select 8130,'USAF' from dual union all
select 8131,'USAF' from dual union all
select 8132,'WFF' from dual union all
select 8133,'WFF' from dual union all
select 8134,'STO' from dual union all
select 8135,'HADC' from dual union all
select 815,'JPL' from dual union all
select 818,'AFCRL' from dual union all
select 820,'US' from dual union all
select 8215,'LARC' from dual union all
select 822,'AFCRL' from dual union all
select 8220,'CARDE' from dual union all
select 8224,'LASL' from dual union all
select 8225,'BRL' from dual union all
select 824,'AFCRL' from dual union all
select 824,'NRL' from dual union all
select 8244,'ISAS' from dual union all
select 825,'APL' from dual union all
select 825,'NRL' from dual union all
select 825,'ORD' from dual union all
select 8253,'LARC' from dual union all
select 826,'UMI' from dual union all
select 8260,'TXWC' from dual union all
select 827,'UMI' from dual union all
select 8279,'LARC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 828,'NRL' from dual union all
select 8280,'LARC' from dual union all
select 8282,'USAF' from dual union all
select 8283,'AFCRL' from dual union all
select 8287,'LARC' from dual union all
select 829,'UMI' from dual union all
select 8290,'LARC' from dual union all
select 8292,'LARC' from dual union all
select 8293,'LARC' from dual union all
select 8297,'TXWC' from dual union all
select 8298,'LARC' from dual union all
select 830,'NRL' from dual union all
select 8304,'GSFC' from dual union all
select 8305,'LARC' from dual union all
select 831,'UMI' from dual union all
select 8311,'LARC' from dual union all
select 8316,'LARC' from dual union all
select 8317,'LARC' from dual union all
select 832,'UMI' from dual union all
select 8320,'LARC' from dual union all
select 8321,'CSIMP' from dual union all
select 8322,'GSFC' from dual union all
select 8323,'LARC' from dual union all
select 8324,'LARC' from dual union all
select 8325,'DRG' from dual union all
select 8327,'DRG' from dual union all
select 8328,'DRG' from dual union all
select 8329,'JPL' from dual union all
select 833,'NRL' from dual union all
select 8336,'CNRS' from dual union all
select 834,'APL' from dual union all
select 834,'NRL' from dual union all
select 8340,'LARC' from dual union all
select 8341,'GSFC' from dual union all
select 8342,'LARC' from dual union all
select 8345,'LARC' from dual union all
select 8349,'LARC' from dual union all
select 835,'ORD' from dual union all
select 8352,'JHU' from dual union all
select 8353,'LARC' from dual union all
select 8355,'LARC' from dual union all
select 8357,'LARC' from dual union all
select 8358,'LARC' from dual union all
select 836,'APL' from dual union all
select 8361,'LARC' from dual union all
select 8363,'LARC' from dual union all
select 8365,'LARC' from dual union all
select 8367,'LARC' from dual union all
select 837,'AFCRL' from dual union all
select 837,'BU' from dual union all
select 838,'APL' from dual union all
select 8380,'NOTS' from dual union all
select 8381,'AERAR' from dual union all
select 8382,'AERAR' from dual union all
select 8383,'AERAR' from dual union all
select 8384,'AERAR' from dual union all
select 8385,'ISAS' from dual union all
select 8386,'ISAS' from dual union all
select 8387,'ISAS' from dual union all
select 8388,'ISAS' from dual union all
select 839,'NRL' from dual union all
select 839,'SCEL' from dual union all
select 839,'UMI' from dual union all
select 8390,'ISAS' from dual union all
select 8394,'GSFC' from dual union all
select 840,'NRL' from dual union all
select 8401,'NOTS' from dual union all
select 8401,'NRL' from dual union all
select 8410,'LARC' from dual union all
select 8412,'ICST' from dual union all
select 8412,'UCL' from dual union all
select 8414,'AFCRL' from dual union all
select 8430,'LARC' from dual union all
select 8431,'LARC' from dual union all
select 8432,'LARC' from dual union all
select 8433,'NOTS' from dual union all
select 8436,'LARC' from dual union all
select 8439,'CNET' from dual union all
select 8442,'CNET' from dual union all
select 8447,'RRL' from dual union all
select 8449,'LARC' from dual union all
select 8456,'CNRS' from dual union all
select 8457,'GCA' from dual union all
select 8459,'US' from dual union all
select 8461,'TXWC' from dual union all
select 847,'AFCRL' from dual union all
select 847,'UMI' from dual union all
select 8474,'UMI' from dual union all
select 8475,'LERC' from dual union all
select 848,'NRL' from dual union all
select 8480,'CNET' from dual union all
select 8481,'CNRS' from dual union all
select 8487,'EGAO' from dual union all
select 8488,'EGAO' from dual union all
select 8489,'AFCRL' from dual union all
select 8489,'STO' from dual union all
select 849,'APL' from dual union all
select 8494,'JPL' from dual union all
select 8498,'AFCRL' from dual union all
select 850,'APL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 850,'NRL' from dual union all
select 8503,'CNRS' from dual union all
select 8504,'CNRS' from dual union all
select 8505,'ISAS' from dual union all
select 8509,'GSFC' from dual union all
select 8511,'CSIMP' from dual union all
select 8514,'NMSU' from dual union all
select 8515,'NMSU' from dual union all
select 8516,'NMSU' from dual union all
select 8517,'NMSU' from dual union all
select 8518,'NMSU' from dual union all
select 8519,'NMSU' from dual union all
select 8520,'NMSU' from dual union all
select 8521,'NMSU' from dual union all
select 8522,'NMSU' from dual union all
select 8523,'AFCRL' from dual union all
select 8524,'AFCRL' from dual union all
select 8535,'GSFC' from dual union all
select 8536,'GCA' from dual union all
select 8537,'NRL' from dual union all
select 8540,'AFCRL' from dual union all
select 8546,'GSFC' from dual union all
select 8546,'RRL' from dual union all
select 8553,'AFCRL' from dual union all
select 8554,'ICST' from dual union all
select 8554,'UCL' from dual union all
select 8555,'AFCRL' from dual union all
select 8557,'AFCRL' from dual union all
select 8560,'GSFC' from dual union all
select 8561,'GSFC' from dual union all
select 8564,'JPL' from dual union all
select 8567,'GSFC' from dual union all
select 8569,'GSFC' from dual union all
select 8569,'RRL' from dual union all
select 8571,'GSFC' from dual union all
select 8571,'RRL' from dual union all
select 8573,'CNET' from dual union all
select 8574,'UMI' from dual union all
select 8578,'GSFC' from dual union all
select 8580,'CNRS' from dual union all
select 8581,'GCA' from dual union all
select 8581,'GSFC' from dual union all
select 8584,'CNRS' from dual union all
select 8585,'CNRS' from dual union all
select 8586,'CNRS' from dual union all
select 8589,'AFCRL' from dual union all
select 8590,'CNRS' from dual union all
select 8591,'WRE' from dual union all
select 8593,'AFCRL' from dual union all
select 8595,'CNRS' from dual union all
select 8601,'CNRS' from dual union all
select 8603,'CNRS' from dual union all
select 8604,'CNRS' from dual union all
select 8605,'CNRS' from dual union all
select 8606,'CNRS' from dual union all
select 8607,'CNRS' from dual union all
select 8608,'AFCRL' from dual union all
select 8610,'CNRS' from dual union all
select 8612,'RAE' from dual union all
select 8612,'WRE' from dual union all
select 8613,'UMI' from dual union all
select 8617,'GSFC' from dual union all
select 8617,'UMI' from dual union all
select 8618,'GCA' from dual union all
select 8619,'SUPA' from dual union all
select 8623,'GSFC' from dual union all
select 8625,'SUPA' from dual union all
select 8626,'AFCRL' from dual union all
select 8628,'CARDE' from dual union all
select 8629,'CARDE' from dual union all
select 8631,'GSFC' from dual union all
select 8632,'CARDE' from dual union all
select 8634,'AFCRL' from dual union all
select 8638,'ASE' from dual union all
select 8639,'LUX' from dual union all
select 8639,'UCL' from dual union all
select 8640,'LERC' from dual union all
select 8645,'WRE' from dual union all
select 8649,'NRL' from dual union all
select 8651,'NRL' from dual union all
select 8656,'JHU' from dual union all
select 8662,'JHU' from dual union all
select 8663,'AFCRL' from dual union all
select 8672,'ICST' from dual union all
select 8672,'UCL' from dual union all
select 8674,'AFCRL' from dual union all
select 8675,'AFCRL' from dual union all
select 8676,'AFCRL' from dual union all
select 8678,'AFCRL' from dual union all
select 8679,'AFCRL' from dual union all
select 8680,'AFCRL' from dual union all
select 8681,'AFCRL' from dual union all
select 8682,'LASL' from dual union all
select 8684,'BRL' from dual union all
select 8685,'JPL' from dual union all
select 8694,'GSFC' from dual union all
select 8695,'AFCRL' from dual union all
select 8704,'AFCRL' from dual union all
select 8706,'AFCRL' from dual union all
select 8707,'USAF' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 8708,'UCO' from dual union all
select 8709,'AFCRL' from dual union all
select 8713,'NOTS' from dual union all
select 8713,'NRL' from dual union all
select 8722,'LARC' from dual union all
select 8729,'AFCRL' from dual union all
select 8731,'AFCRL' from dual union all
select 8731,'UMD' from dual union all
select 8734,'AFCRL' from dual union all
select 8734,'STO' from dual union all
select 8737,'GSFC' from dual union all
select 8747,'AFCRL' from dual union all
select 8747,'STO' from dual union all
select 8748,'UNK' from dual union all
select 8749,'RSRS' from dual union all
select 8749,'SHEF' from dual union all
select 8749,'UCL' from dual union all
select 8751,'ILC' from dual union all
select 8752,'AFCRL' from dual union all
select 8752,'STO' from dual union all
select 8754,'AFCRL' from dual union all
select 876,'SCEL' from dual union all
select 876,'UMI' from dual union all
select 8761,'NRL' from dual union all
select 8763,'ISAS' from dual union all
select 8764,'LPARL' from dual union all
select 8767,'ICST' from dual union all
select 8767,'UCL' from dual union all
select 8769,'WRE' from dual union all
select 877,'AFCRL' from dual union all
select 877,'BU' from dual union all
select 877,'WADC' from dual union all
select 8771,'AFCRL' from dual union all
select 8772,'RAE' from dual union all
select 8772,'WRE' from dual union all
select 8773,'AFCRL' from dual union all
select 8776,'AFCRL' from dual union all
select 8776,'STO' from dual union all
select 878,'AFCRL' from dual union all
select 8786,'RAE' from dual union all
select 8786,'WRE' from dual union all
select 8787,'UCL' from dual union all
select 8788,'AFCRL' from dual union all
select 8796,'UNK' from dual union all
select 8799,'RSRS' from dual union all
select 8799,'SHEF' from dual union all
select 8800,'GSFC' from dual union all
select 8801,'UMI' from dual union all
select 8806,'AFCRL' from dual union all
select 8808,'GSFC' from dual union all
select 8838,'LPARL' from dual union all
select 8844,'AFCRL' from dual union all
select 8845,'UMI' from dual union all
select 8848,'AFCRL' from dual union all
select 885,'UMI' from dual union all
select 8850,'ASE' from dual union all
select 8853,'CERMA' from dual union all
select 8854,'AFCRL' from dual union all
select 8855,'CAM' from dual union all
select 8855,'UCL' from dual union all
select 8857,'AFCRL' from dual union all
select 8858,'GSFC' from dual union all
select 8860,'AFCRL' from dual union all
select 8861,'AFCRL' from dual union all
select 8862,'AFCRL' from dual union all
select 8864,'AN' from dual union all
select 8865,'CERMA' from dual union all
select 8866,'AFCRL' from dual union all
select 8867,'LRBA' from dual union all
select 8868,'AFCRL' from dual union all
select 8870,'AEC' from dual union all
select 8871,'AFCRL' from dual union all
select 8872,'UMI' from dual union all
select 8875,'AN' from dual union all
select 8877,'LRBA' from dual union all
select 8878,'DFF' from dual union all
select 8878,'LPA' from dual union all
select 8879,'AFCRL' from dual union all
select 8880,'AFCRL' from dual union all
select 8881,'AFCRL' from dual union all
select 8882,'WRE' from dual union all
select 8883,'AFCRL' from dual union all
select 8884,'LERC' from dual union all
select 8885,'AFCRL' from dual union all
select 8886,'AFCRL' from dual union all
select 8887,'AFCRL' from dual union all
select 8888,'AFCRL' from dual union all
select 8889,'AFCRL' from dual union all
select 889,'AFCRL' from dual union all
select 889,'RIU' from dual union all
select 8890,'AFCRL' from dual union all
select 8891,'AFCRL' from dual union all
select 8893,'AFCRL' from dual union all
select 8895,'AFCRL' from dual union all
select 8896,'AFCRL' from dual union all
select 8897,'AFCRL' from dual union all
select 8898,'AFCRL' from dual union all
select 8899,'AFCRL' from dual union all
select 8901,'AFCRL' from dual union all
select 8903,'UMI' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 8904,'LPA' from dual union all
select 8905,'UMI' from dual union all
select 8906,'UWI' from dual union all
select 8917,'AFCRL' from dual union all
select 8918,'AFCRL' from dual union all
select 8920,'AFCRL' from dual union all
select 8921,'AFCRL' from dual union all
select 8922,'AFCRL' from dual union all
select 8923,'AFCRL' from dual union all
select 8924,'AFCRL' from dual union all
select 8925,'AFCRL' from dual union all
select 8926,'LARC' from dual union all
select 8927,'AFCRL' from dual union all
select 8928,'AFCRL' from dual union all
select 8929,'AFCRL' from dual union all
select 8931,'AFCRL' from dual union all
select 8932,'AFCRL' from dual union all
select 8933,'AFCRL' from dual union all
select 8934,'AFCRL' from dual union all
select 8935,'AFCRL' from dual union all
select 8936,'GCA' from dual union all
select 8938,'GCA' from dual union all
select 8942,'AFCRL' from dual union all
select 8943,'JBO' from dual union all
select 8943,'LUX' from dual union all
select 8943,'UCL' from dual union all
select 8944,'AFCRL' from dual union all
select 8947,'AFCRL' from dual union all
select 8950,'LERC' from dual union all
select 8953,'GSFC' from dual union all
select 8953,'UMI' from dual union all
select 8954,'USAF' from dual union all
select 8956,'RAE' from dual union all
select 8956,'WRE' from dual union all
select 8960,'CNRS' from dual union all
select 8961,'AFCRL' from dual union all
select 8963,'CNIE' from dual union all
select 8963,'CNRS' from dual union all
select 8964,'WRE' from dual union all
select 8965,'HCO' from dual union all
select 8966,'AFCRL' from dual union all
select 8968,'CNRS' from dual union all
select 8968,'UCL' from dual union all
select 8969,'ICST' from dual union all
select 8969,'UCL' from dual union all
select 8973,'RAE' from dual union all
select 8973,'UCL' from dual union all
select 8981,'LARC' from dual union all
select 8983,'CNRS' from dual union all
select 8983,'UCL' from dual union all
select 8984,'CNRS' from dual union all
select 8985,'CNIE' from dual union all
select 8985,'CNRS' from dual union all
select 8986,'CNRS' from dual union all
select 8986,'MPE' from dual union all
select 8987,'GCA' from dual union all
select 8988,'GCA' from dual union all
select 8989,'GSFC' from dual union all
select 8990,'UMI' from dual union all
select 8991,'GSFC' from dual union all
select 8992,'AFCRL' from dual union all
select 8993,'GSFC' from dual union all
select 8994,'AFCRL' from dual union all
select 8994,'GCA' from dual union all
select 8995,'AFCRL' from dual union all
select 8995,'GCA' from dual union all
select 8996,'CNRS' from dual union all
select 8997,'AFCRL' from dual union all
select 8998,'AFCRL' from dual union all
select 8999,'AFCRL' from dual union all
select 9000,'GSFC' from dual union all
select 9001,'ICST' from dual union all
select 9001,'UCL' from dual union all
select 9002,'WRE' from dual union all
select 9003,'GCA' from dual union all
select 9004,'DMA' from dual union all
select 9006,'AFCRL' from dual union all
select 9008,'GCA' from dual union all
select 9010,'GCA' from dual union all
select 9011,'GSFC' from dual union all
select 9012,'GSFC' from dual union all
select 9013,'AFCRL' from dual union all
select 9014,'GSFC' from dual union all
select 9014,'UMI' from dual union all
select 9020,'CNIE' from dual union all
select 9020,'CNRS' from dual union all
select 9021,'AFCRL' from dual union all
select 9022,'CNIE' from dual union all
select 9022,'CNRS' from dual union all
select 9024,'ROSAP' from dual union all
select 9025,'GSFC' from dual union all
select 9027,'AFCRL' from dual union all
select 9028,'ILC' from dual union all
select 903,'AFCRL' from dual union all
select 9030,'CNRS' from dual union all
select 9031,'CNRS' from dual union all
select 9034,'AFCRL' from dual union all
select 9035,'CARDE' from dual union all
select 9036,'GCA' from dual union all
select 9037,'CARDE' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 9039,'NOTS' from dual union all
select 9042,'ILC' from dual union all
select 9043,'AFCRL' from dual union all
select 9044,'AFCRL' from dual union all
select 9046,'TAO' from dual union all
select 9047,'GSFC' from dual union all
select 9047,'UMI' from dual union all
select 9048,'AFCRL' from dual union all
select 9051,'AFAPL' from dual union all
select 9051,'EOS' from dual union all
select 9055,'AFCRL' from dual union all
select 9061,'AFSWC' from dual union all
select 9062,'AFSWC' from dual union all
select 9063,'AFSWC' from dual union all
select 9064,'AFSWC' from dual union all
select 9065,'AFSWC' from dual union all
select 9066,'AFWL' from dual union all
select 9068,'AFWL' from dual union all
select 9069,'SAND' from dual union all
select 9070,'LRL' from dual union all
select 9071,'LRL' from dual union all
select 9072,'LRL' from dual union all
select 9073,'LASL' from dual union all
select 9074,'LASL' from dual union all
select 9075,'SAND' from dual union all
select 9076,'SAND' from dual union all
select 9077,'LRL' from dual union all
select 9078,'ICST' from dual union all
select 9078,'UCL' from dual union all
select 9079,'LRL' from dual union all
select 908,'LRBA' from dual union all
select 9080,'LASL' from dual union all
select 9081,'LASL' from dual union all
select 9082,'LRL' from dual union all
select 9083,'LASL' from dual union all
select 9084,'SAND' from dual union all
select 9085,'SAND' from dual union all
select 9086,'SAND' from dual union all
select 9087,'SAND' from dual union all
select 9088,'LASL' from dual union all
select 909,'NRL' from dual union all
select 9090,'LASL' from dual union all
select 9091,'LASL' from dual union all
select 9092,'AFCRL' from dual union all
select 9093,'NMSU' from dual union all
select 9094,'NMSU' from dual union all
select 9095,'NMSU' from dual union all
select 9096,'SAND' from dual union all
select 9097,'SAND' from dual union all
select 9098,'LRL' from dual union all
select 9099,'LRL' from dual union all
select 910,'LRBA' from dual union all
select 9100,'ICST' from dual union all
select 9100,'UCL' from dual union all
select 9101,'LRL' from dual union all
select 9102,'SAND' from dual union all
select 9103,'SAND' from dual union all
select 9104,'AFCRL' from dual union all
select 9105,'BRL' from dual union all
select 9106,'BRL' from dual union all
select 9108,'NMSU' from dual union all
select 9109,'NMSU' from dual union all
select 9110,'NMSU' from dual union all
select 9111,'AFCRL' from dual union all
select 9112,'NMSU' from dual union all
select 9113,'NMSU' from dual union all
select 9114,'NMSU' from dual union all
select 9115,'NMSU' from dual union all
select 9116,'NMSU' from dual union all
select 9118,'LRS' from dual union all
select 9120,'NOTS' from dual union all
select 9121,'SAND' from dual union all
select 9123,'SAND' from dual union all
select 9124,'SAND' from dual union all
select 9140,'NOTS' from dual union all
select 9142,'AFCRL' from dual union all
select 9144,'ICST' from dual union all
select 9144,'UCL' from dual union all
select 9148,'GCA' from dual union all
select 9149,'GSFC' from dual union all
select 9149,'UMI' from dual union all
select 9151,'GCA' from dual union all
select 9153,'GSFC' from dual union all
select 9153,'UMI' from dual union all
select 9154,'WRE' from dual union all
select 9155,'WSMR' from dual union all
select 9156,'WSMR' from dual union all
select 9157,'WSMR' from dual union all
select 9159,'LARC' from dual union all
select 9162,'ICST' from dual union all
select 9162,'UCL' from dual union all
select 9173,'GSFC' from dual union all
select 9174,'AFCRL' from dual union all
select 9175,'LARC' from dual union all
select 9176,'UKMO' from dual union all
select 9177,'UNK' from dual union all
select 9178,'UNK' from dual union all
select 9179,'GSFC' from dual union all
select 9180,'GSFC' from dual union all
select 9182,'GSFC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 9183,'GSFC' from dual union all
select 9184,'WSMR' from dual union all
select 9185,'WSMR' from dual union all
select 9186,'WSMR' from dual union all
select 9187,'WSMR' from dual union all
select 9188,'TXWC' from dual union all
select 9189,'WSMR' from dual union all
select 9190,'WSMR' from dual union all
select 9191,'WSMR' from dual union all
select 9192,'WSMR' from dual union all
select 9193,'WSMR' from dual union all
select 9194,'WSMR' from dual union all
select 9195,'WSMR' from dual union all
select 9196,'USAF' from dual union all
select 9197,'USAF' from dual union all
select 9198,'USAF' from dual union all
select 9199,'USAF' from dual union all
select 9200,'USAF' from dual union all
select 9201,'TXWC' from dual union all
select 9202,'TXWC' from dual union all
select 9203,'WSMR' from dual union all
select 9204,'WSMR' from dual union all
select 9205,'TXWC' from dual union all
select 9206,'USAF' from dual union all
select 9207,'USAF' from dual union all
select 9208,'USAF' from dual union all
select 9209,'USAF' from dual union all
select 9210,'USAF' from dual union all
select 9211,'TXWC' from dual union all
select 9212,'USAF' from dual union all
select 9213,'USAF' from dual union all
select 9214,'USAF' from dual union all
select 9215,'USAF' from dual union all
select 9216,'USAF' from dual union all
select 9217,'USAF' from dual union all
select 9218,'USAF' from dual union all
select 9219,'USAF' from dual union all
select 9220,'USAF' from dual union all
select 9221,'USAF' from dual union all
select 9222,'USAF' from dual union all
select 9223,'TXWC' from dual union all
select 9224,'TXWC' from dual union all
select 9225,'TXWC' from dual union all
select 9226,'SUD' from dual union all
select 9227,'SUD' from dual union all
select 9228,'TXWC' from dual union all
select 9229,'TXWC' from dual union all
select 9230,'TXWC' from dual union all
select 9231,'TXWC' from dual union all
select 924,'BRL' from dual union all
select 9290,'PIHM' from dual union all
select 9291,'LARC' from dual union all
select 9292,'LARC' from dual union all
select 9293,'LARC' from dual union all
select 9295,'LARC' from dual union all
select 9296,'LARC' from dual union all
select 9297,'SAND' from dual union all
select 9298,'LARC' from dual union all
select 9299,'SAND' from dual union all
select 9304,'LARC' from dual union all
select 9306,'LARC' from dual union all
select 9307,'LARC' from dual union all
select 9308,'GCA' from dual union all
select 9309,'GCA' from dual union all
select 9311,'RAE' from dual union all
select 9311,'WRE' from dual union all
select 9312,'LARC' from dual union all
select 9313,'LARC' from dual union all
select 9314,'RAE' from dual union all
select 9314,'WRE' from dual union all
select 9316,'LARC' from dual union all
select 9318,'LARC' from dual union all
select 9320,'LARC' from dual union all
select 9321,'MSFC' from dual union all
select 9324,'LARC' from dual union all
select 9325,'LARC' from dual union all
select 9326,'LARC' from dual union all
select 9329,'LARC' from dual union all
select 9334,'LARC' from dual union all
select 9335,'LARC' from dual union all
select 9336,'LARC' from dual union all
select 9337,'SAND' from dual union all
select 9340,'LARC' from dual union all
select 9341,'LARC' from dual union all
select 9344,'LARC' from dual union all
select 9345,'GSFC' from dual union all
select 9346,'LARC' from dual union all
select 9348,'LARC' from dual union all
select 9350,'LARC' from dual union all
select 9351,'LARC' from dual union all
select 9355,'LARC' from dual union all
select 9362,'JHU' from dual union all
select 9364,'LARC' from dual union all
select 9366,'LARC' from dual union all
select 9370,'LARC' from dual union all
select 9371,'LARC' from dual union all
select 9374,'LARC' from dual union all
select 9378,'BIR' from dual union all
select 9378,'LUX' from dual union all
select 9378,'UCW' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 9379,'LARC' from dual union all
select 9384,'LARC' from dual union all
select 9385,'DRG' from dual union all
select 9387,'LARC' from dual union all
select 9392,'LARC' from dual union all
select 9393,'MNF' from dual union all
select 9394,'AERAR' from dual union all
select 9396,'AEC' from dual union all
select 9396,'LRL' from dual union all
select 9397,'LARC' from dual union all
select 9397,'NRL' from dual union all
select 9398,'LARC' from dual union all
select 9398,'NRL' from dual union all
select 9399,'USA' from dual union all
select 940,'UMI' from dual union all
select 9400,'USA' from dual union all
select 9406,'ONR' from dual union all
select 9407,'ONR' from dual union all
select 9408,'ONR' from dual union all
select 9409,'ONR' from dual union all
select 9410,'ONR' from dual union all
select 9411,'ONR' from dual union all
select 9412,'ONR' from dual union all
select 9413,'ONR' from dual union all
select 9414,'LARC' from dual union all
select 9416,'AERAR' from dual union all
select 9417,'AERAR' from dual union all
select 9418,'LARC' from dual union all
select 9422,'GCA' from dual union all
select 9423,'LARC' from dual union all
select 9424,'LARC' from dual union all
select 9426,'MSFC' from dual union all
select 9429,'LARC' from dual union all
select 9432,'LARC' from dual union all
select 9433,'LARC' from dual union all
select 9435,'LARC' from dual union all
select 9436,'LARC' from dual union all
select 9437,'LARC' from dual union all
select 9439,'MSFC' from dual union all
select 944,'APL' from dual union all
select 9442,'LARC' from dual union all
select 9443,'LARC' from dual union all
select 9444,'LARC' from dual union all
select 9445,'LARC' from dual union all
select 9446,'LARC' from dual union all
select 9449,'LARC' from dual union all
select 945,'APL' from dual union all
select 9450,'LARC' from dual union all
select 9452,'LARC' from dual union all
select 9453,'LARC' from dual union all
select 9454,'LARC' from dual union all
select 9456,'LARC' from dual union all
select 9457,'MSFC' from dual union all
select 9458,'LARC' from dual union all
select 9459,'LARC' from dual union all
select 946,'NRL' from dual union all
select 9460,'LARC' from dual union all
select 9464,'LARC' from dual union all
select 9465,'LARC' from dual union all
select 9466,'MSFC' from dual union all
select 947,'SCEL' from dual union all
select 9470,'LARC' from dual union all
select 948,'RIU' from dual union all
select 9482,'LARC' from dual union all
select 9486,'CNRS' from dual union all
select 9487,'LARC' from dual union all
select 950,'SCEL' from dual union all
select 951,'AOGMC' from dual union all
select 9510,'HARP' from dual union all
select 9517,'LERC' from dual union all
select 952,'AOGMC' from dual union all
select 9524,'DRG' from dual union all
select 9525,'DRG' from dual union all
select 9526,'DRG' from dual union all
select 953,'SCEL' from dual union all
select 9533,'GSFC' from dual union all
select 9534,'EGAO' from dual union all
select 9536,'DRG' from dual union all
select 9538,'CNRS' from dual union all
select 954,'UMI' from dual union all
select 9546,'JHU' from dual union all
select 9547,'CNRS' from dual union all
select 9548,'STA' from dual union all
select 9549,'STA' from dual union all
select 9550,'STA' from dual union all
select 9551,'STA' from dual union all
select 9552,'ISAS' from dual union all
select 9558,'DAC' from dual union all
select 956,'NRL' from dual union all
select 9560,'ADEL' from dual union all
select 9561,'DAC' from dual union all
select 9562,'GSFC' from dual union all
select 9562,'LUX' from dual union all
select 9563,'USN' from dual union all
select 9564,'SUD' from dual union all
select 9567,'HARP' from dual union all
select 9568,'HARP' from dual union all
select 957,'SCEL' from dual union all
select 9579,'CITEF' from dual union all
select 958,'NRL' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 9580,'CITEF' from dual union all
select 9588,'LARC' from dual union all
select 959,'NRL' from dual union all
select 9592,'LPARL' from dual union all
select 9597,'AFCRL' from dual union all
select 960,'UMI' from dual union all
select 9600,'CARDE' from dual union all
select 9605,'CARDE' from dual union all
select 9608,'GSFC' from dual union all
select 9609,'UMI' from dual union all
select 961,'NRL' from dual union all
select 9610,'WRE' from dual union all
select 9611,'UMI' from dual union all
select 9613,'GCA' from dual union all
select 9615,'UMI' from dual union all
select 9616,'UMI' from dual union all
select 962,'APL' from dual union all
select 9622,'CNRS' from dual union all
select 9624,'STAN' from dual union all
select 9626,'NRL' from dual union all
select 9628,'SASK' from dual union all
select 9629,'SASK' from dual union all
select 963,'UMI' from dual union all
select 9633,'GSFC' from dual union all
select 964,'APL' from dual union all
select 9640,'CSIMP' from dual union all
select 9641,'KPNO' from dual union all
select 965,'AFCRL' from dual union all
select 965,'BU' from dual union all
select 965,'WAML' from dual union all
select 9652,'QUB' from dual union all
select 9652,'UCL' from dual union all
select 9658,'UMI' from dual union all
select 9660,'CRA' from dual union all
select 9661,'DFF' from dual union all
select 9661,'LPA' from dual union all
select 9664,'LPA' from dual union all
select 9665,'SAND' from dual union all
select 9666,'GSFC' from dual union all
select 9675,'SAND' from dual union all
select 9678,'WFF' from dual union all
select 9679,'WFF' from dual union all
select 968,'BU' from dual union all
select 968,'WADC' from dual union all
select 9681,'NRL' from dual union all
select 9682,'CEA' from dual union all
select 9682,'LPA' from dual union all
select 9683,'DRG' from dual union all
select 9684,'AFCRL' from dual union all
select 9685,'DRTE' from dual union all
select 9687,'JHU' from dual union all
select 9689,'LPARL' from dual union all
select 969,'SCEL' from dual union all
select 9691,'CSIMP' from dual union all
select 9693,'CNRS' from dual union all
select 9694,'NRL' from dual union all
select 9695,'AFCRL' from dual union all
select 9695,'UMD' from dual union all
select 9698,'CNRS' from dual union all
select 9699,'LERC' from dual union all
select 9700,'CNRS' from dual union all
select 9700,'MPE' from dual union all
select 9701,'AFCRL' from dual union all
select 9702,'SUPA' from dual union all
select 9709,'AFCRL' from dual union all
select 9709,'GIT' from dual union all
select 9710,'AFCRL' from dual union all
select 9711,'AFCRL' from dual union all
select 9712,'AFCRL' from dual union all
select 9712,'GIT' from dual union all
select 9713,'ROSAP' from dual union all
select 9715,'TAO' from dual union all
select 9716,'ROSAP' from dual union all
select 9718,'ROSAP' from dual union all
select 9719,'CNRS' from dual union all
select 9720,'CNRS' from dual union all
select 9721,'CNRS' from dual union all
select 9721,'MPE' from dual union all
select 9722,'NRL' from dual union all
select 9723,'AFCRL' from dual union all
select 9723,'GIT' from dual union all
select 9725,'CNRS' from dual union all
select 9725,'GCA' from dual union all
select 9726,'CNRS' from dual union all
select 9726,'GCA' from dual union all
select 9728,'AN' from dual union all
select 9729,'CNRS' from dual union all
select 9729,'GCA' from dual union all
select 9730,'JBO' from dual union all
select 9730,'LUX' from dual union all
select 9730,'UKMO' from dual union all
select 9733,'CNRS' from dual union all
select 9734,'GCA' from dual union all
select 9735,'GCA' from dual union all
select 9737,'CNIE' from dual union all
select 9737,'CNRS' from dual union all
select 9738,'GCA' from dual union all
select 9739,'FIAN' from dual union all
select 9740,'SAND' from dual union all
select 9741,'LARC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 9743,'APGC' from dual union all
select 9744,'LUX' from dual union all
select 9744,'RSRS' from dual union all
select 9744,'UCL' from dual union all
select 9745,'CNIE' from dual union all
select 9745,'CNRS' from dual union all
select 9746,'CNIE' from dual union all
select 9746,'CNRS' from dual union all
select 976,'DENV' from dual union all
select 976,'UCO' from dual union all
select 9765,'AN' from dual union all
select 9766,'NRL' from dual union all
select 9769,'DRTE' from dual union all
select 9770,'AFCRL' from dual union all
select 9770,'ASE' from dual union all
select 9771,'SAND' from dual union all
select 9772,'WRE' from dual union all
select 9773,'CNRS' from dual union all
select 9773,'UCL' from dual union all
select 9777,'CNRS' from dual union all
select 9779,'CNRS' from dual union all
select 978,'NRL' from dual union all
select 9781,'GSFC' from dual union all
select 9782,'LUX' from dual union all
select 9782,'RSRS' from dual union all
select 9782,'UCL' from dual union all
select 9784,'HARP' from dual union all
select 9786,'CNRS' from dual union all
select 9787,'AN' from dual union all
select 9788,'UMI' from dual union all
select 9789,'LERC' from dual union all
select 9791,'UMI' from dual union all
select 9792,'LUX' from dual union all
select 9792,'UCL' from dual union all
select 9793,'NRL' from dual union all
select 9802,'CARDE' from dual union all
select 9803,'CARDE' from dual union all
select 9804,'SAND' from dual union all
select 9806,'KPNO' from dual union all
select 9813,'NRL' from dual union all
select 9817,'AFCRL' from dual union all
select 982,'SCEL' from dual union all
select 9821,'USN' from dual union all
select 9823,'SAND' from dual union all
select 9826,'USAF' from dual union all
select 9829,'GSFC' from dual union all
select 983,'UMI' from dual union all
select 9834,'AFCRL' from dual union all
select 9839,'AN' from dual union all
select 984,'UMI' from dual union all
select 9842,'STAN' from dual union all
select 9843,'AFCRL' from dual union all
select 9846,'GCA' from dual union all
select 9851,'GSFC' from dual union all
select 9853,'GSFC' from dual union all
select 9854,'GCA' from dual union all
select 9855,'AFCRL' from dual union all
select 9855,'UUT' from dual union all
select 9856,'GCA' from dual union all
select 9857,'GCA' from dual union all
select 9859,'UMI' from dual union all
select 986,'NRL' from dual union all
select 9860,'GCA' from dual union all
select 9861,'GSFC' from dual union all
select 9864,'AFCRL' from dual union all
select 9872,'AFCRL' from dual union all
select 9872,'UUT' from dual union all
select 9876,'GSFC' from dual union all
select 9876,'STO' from dual union all
select 9877,'NRL' from dual union all
select 9878,'GSFC' from dual union all
select 9878,'STO' from dual union all
select 9884,'NRL' from dual union all
select 9885,'GSFC' from dual union all
select 9885,'STO' from dual union all
select 9886,'NMC' from dual union all
select 9887,'CRA' from dual union all
select 9888,'UMI' from dual union all
select 9892,'GSFC' from dual union all
select 9892,'STO' from dual union all
select 991,'AFCRL' from dual union all
select 991,'BU' from dual union all
select 9914,'AFCRL' from dual union all
select 9936,'ILC' from dual union all
select 9936,'NDRE' from dual union all
select 9937,'AFCRL' from dual union all
select 994,'LRBA' from dual union all
select 9945,'GSFC' from dual union all
select 9946,'UMN' from dual union all
select 9947,'HARP' from dual union all
select 9948,'RAE' from dual union all
select 9948,'WRE' from dual union all
select 9950,'HCO' from dual union all
select 9953,'UNH' from dual union all
select 9956,'LERC' from dual union all
select 9960,'ILC' from dual union all
select 9960,'NDRE' from dual union all
select 9965,'NRL' from dual union all
select 9972,'AFCRL' from dual union all
select 9974,'LARC' from dual;
insert into LAUNCH_PAYLOAD_ORG
select 9975,'LARC' from dual union all
select 9976,'GSFC' from dual union all
select 9976,'RRL' from dual union all
select 9979,'NRL' from dual union all
select 9982,'AFCRL' from dual union all
select 9983,'AFCRL' from dual union all
select 9984,'AFCRL' from dual union all
select 9987,'GSFC' from dual union all
select 9987,'RRL' from dual union all
select 9988,'GSFC' from dual union all
select 9994,'AFCRL' from dual union all
select 9995,'GSFC' from dual union all
select 9996,'CAM' from dual union all
select 9996,'UCL' from dual;


--------------------------------------------------------------------------------
-- LAUNCH_AGENCY
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing LAUNCH_AGENCY...');

CREATE TABLE "LAUNCH_AGENCY" 
 (	"LAUNCH_ID" NUMBER, 
	"AGENCY_ORG_CODE" VARCHAR2(4000), 
	 CONSTRAINT "LAUNCH_AGENCY_PK" PRIMARY KEY ("LAUNCH_ID", "AGENCY_ORG_CODE")
USING INDEX  ENABLE, 
	 CONSTRAINT "LAUNCH_AGENCY_LAUNCH_FK" FOREIGN KEY ("LAUNCH_ID")
	  REFERENCES "LAUNCH" ("LAUNCH_ID") ENABLE, 
	 CONSTRAINT "LAUNCH_AGENCY_ORG_FK" FOREIGN KEY ("AGENCY_ORG_CODE")
	  REFERENCES "ORGANIZATION" ("ORG_CODE") ENABLE
 ) ;

insert into LAUNCH_AGENCY
select 1,'WEHR' from dual union all
select 10,'WEHR' from dual union all
select 100,'WEHR' from dual union all
select 1000,'USA' from dual union all
select 10000,'USA' from dual union all
select 10001,'AOMC' from dual union all
select 10002,'NASA' from dual union all
select 10003,'NASA' from dual union all
select 10004,'MVS' from dual union all
select 10005,'NASA' from dual union all
select 10006,'MVS' from dual union all
select 10007,'MVS' from dual union all
select 10008,'44FA2' from dual union all
select 10009,'RVSN' from dual union all
select 1001,'ARDC' from dual union all
select 10010,'DRTE' from dual union all
select 10011,'MVS' from dual union all
select 10012,'MVS' from dual union all
select 10013,'RAE' from dual union all
select 10013,'WRE' from dual union all
select 10014,'RAE' from dual union all
select 10014,'WRE' from dual union all
select 10015,'RAE' from dual union all
select 10015,'WRE' from dual union all
select 10016,'NASA' from dual union all
select 10017,'RAE' from dual union all
select 10017,'WRE' from dual union all
select 10018,'44FA2' from dual union all
select 10019,'USN' from dual union all
select 1002,'USAF' from dual union all
select 10020,'1STRAD' from dual union all
select 10021,'ELDO' from dual union all
select 10021,'RAE' from dual union all
select 10022,'NASA' from dual union all
select 10023,'MVS' from dual union all
select 10024,'44FA2' from dual union all
select 10025,'CNES' from dual union all
select 10026,'CARDE' from dual union all
select 10027,'44FA2' from dual union all
select 10028,'RVSN' from dual union all
select 10029,'MVS' from dual union all
select 1003,'3GMS' from dual union all
select 10030,'RVSN' from dual union all
select 10031,'MVS' from dual union all
select 10032,'USN' from dual union all
select 10033,'USN' from dual union all
select 10034,'USAF' from dual union all
select 10035,'USN' from dual union all
select 10036,'USN' from dual union all
select 10037,'USN' from dual union all
select 10038,'USN' from dual union all
select 10039,'CNES' from dual union all
select 1004,'USA' from dual union all
select 10040,'AN' from dual union all
select 10041,'USN' from dual union all
select 10042,'MVS' from dual union all
select 10043,'DRTE' from dual union all
select 10044,'44FA2' from dual union all
select 10045,'USA' from dual union all
select 10046,'MVS' from dual union all
select 10047,'AFSC' from dual union all
select 10048,'MVS' from dual union all
select 10049,'CARDE' from dual union all
select 1005,'USAF' from dual union all
select 10050,'CNES' from dual union all
select 10051,'RAE' from dual union all
select 10051,'WRE' from dual union all
select 10052,'1STRAD' from dual union all
select 10053,'NASA' from dual union all
select 10054,'USAF' from dual union all
select 10055,'NASA' from dual union all
select 10056,'NRL' from dual union all
select 10057,'USA' from dual union all
select 10058,'MVS' from dual union all
select 10059,'MVS' from dual union all
select 1006,'PARD' from dual union all
select 10060,'AFSC' from dual union all
select 10061,'1STRAD' from dual union all
select 10062,'SAC' from dual union all
select 10063,'USAF' from dual union all
select 10064,'USN' from dual union all
select 10065,'USN' from dual union all
select 10066,'KPNO' from dual union all
select 10067,'NASA' from dual union all
select 10068,'USAF' from dual union all
select 10069,'MVS' from dual union all
select 1007,'USAF' from dual union all
select 10070,'RVSN' from dual union all
select 10071,'USAF' from dual union all
select 10072,'USN' from dual union all
select 10073,'AFSC' from dual union all
select 10074,'USN' from dual union all
select 10075,'USN' from dual union all
select 10076,'NASA' from dual union all
select 10077,'AFSC' from dual union all
select 10078,'NRL' from dual union all
select 10079,'SAC' from dual union all
select 1008,'USA' from dual union all
select 10080,'USAF' from dual union all
select 10081,'41FA4' from dual;
insert into LAUNCH_AGENCY
select 10082,'USA' from dual union all
select 10083,'SAC' from dual union all
select 10084,'USN' from dual union all
select 10085,'USN' from dual union all
select 10086,'MVS' from dual union all
select 10087,'NASA' from dual union all
select 10088,'AEC' from dual union all
select 10088,'LRL' from dual union all
select 10089,'MVS' from dual union all
select 1009,'USAF' from dual union all
select 10090,'41FA4' from dual union all
select 10091,'USN' from dual union all
select 10092,'SAND' from dual union all
select 10093,'NASA' from dual union all
select 10094,'RVSN' from dual union all
select 10095,'SAND' from dual union all
select 10096,'MVS' from dual union all
select 10097,'NASA' from dual union all
select 10098,'NASA' from dual union all
select 10099,'USAF' from dual union all
select 101,'WEHR' from dual union all
select 1010,'PARD' from dual union all
select 10100,'1STRAD' from dual union all
select 10101,'MVS' from dual union all
select 10102,'NRL' from dual union all
select 10103,'MVS' from dual union all
select 10104,'MVS' from dual union all
select 10105,'1STRAD' from dual union all
select 10106,'MVS' from dual union all
select 10107,'USA' from dual union all
select 10108,'SIMED' from dual union all
select 10109,'RAE' from dual union all
select 10109,'WRE' from dual union all
select 1011,'USAF' from dual union all
select 10110,'USA' from dual union all
select 10111,'MVS' from dual union all
select 10112,'MVS' from dual union all
select 10113,'RAE' from dual union all
select 10113,'WRE' from dual union all
select 10114,'RVSN' from dual union all
select 10115,'MVS' from dual union all
select 10116,'USAF' from dual union all
select 10117,'41FA4' from dual union all
select 10118,'NASA' from dual union all
select 10119,'NASA' from dual union all
select 1012,'USAF' from dual union all
select 10120,'RVSN' from dual union all
select 10121,'NASA' from dual union all
select 10122,'USAF' from dual union all
select 10123,'USN' from dual union all
select 10124,'MVS' from dual union all
select 10125,'USN' from dual union all
select 10126,'ISAS' from dual union all
select 10127,'RVSN' from dual union all
select 10128,'ISAS' from dual union all
select 10129,'USAF' from dual union all
select 1013,'ARDC' from dual union all
select 10130,'WRE' from dual union all
select 10131,'AFSC' from dual union all
select 10132,'NASA' from dual union all
select 10133,'1STRAD' from dual union all
select 10134,'1STRAD' from dual union all
select 10135,'USA' from dual union all
select 10136,'AFSC' from dual union all
select 10137,'MVS' from dual union all
select 10138,'USAF' from dual union all
select 10139,'AFSC' from dual union all
select 1014,'6555GMS' from dual union all
select 10140,'USN' from dual union all
select 10141,'USN' from dual union all
select 10142,'SAC' from dual union all
select 10143,'NASA' from dual union all
select 10144,'AFSC' from dual union all
select 10145,'1STRAD' from dual union all
select 10146,'RVSN' from dual union all
select 10147,'ONERA' from dual union all
select 10148,'RVSN' from dual union all
select 10149,'AN' from dual union all
select 1015,'6555GMS' from dual union all
select 10150,'RVSN' from dual union all
select 10151,'MVS' from dual union all
select 10152,'RVSN' from dual union all
select 10153,'RVSN' from dual union all
select 10154,'USN' from dual union all
select 10155,'USA' from dual union all
select 10156,'HARP' from dual union all
select 10157,'HARP' from dual union all
select 10158,'HARP' from dual union all
select 10159,'HARP' from dual union all
select 1016,'USA' from dual union all
select 10160,'HARP' from dual union all
select 10161,'HARP' from dual union all
select 10162,'HARP' from dual union all
select 10163,'USN' from dual union all
select 10164,'USN' from dual union all
select 10165,'NASA' from dual union all
select 10166,'USA' from dual union all
select 10167,'USA' from dual union all
select 10168,'41FA4' from dual union all
select 10169,'NOTS' from dual;
insert into LAUNCH_AGENCY
select 1017,'USA' from dual union all
select 10170,'APGC' from dual union all
select 10171,'NMC' from dual union all
select 10172,'NOTS' from dual union all
select 10173,'RVSN' from dual union all
select 10174,'NASA' from dual union all
select 10175,'NASA' from dual union all
select 10176,'RVSN' from dual union all
select 10177,'NASA' from dual union all
select 10178,'RVSN' from dual union all
select 10179,'RVSN' from dual union all
select 1018,'NRL' from dual union all
select 1018,'US' from dual union all
select 10180,'RVSN' from dual union all
select 10181,'RVSN' from dual union all
select 10182,'MVS' from dual union all
select 10183,'RVSN' from dual union all
select 10184,'RVSN' from dual union all
select 10185,'RVSN' from dual union all
select 10186,'MVS' from dual union all
select 10187,'USAF' from dual union all
select 10188,'MVS' from dual union all
select 10189,'MVS' from dual union all
select 1019,'USA' from dual union all
select 10190,'MVS' from dual union all
select 10191,'RVSN' from dual union all
select 10192,'AOMC' from dual union all
select 10193,'NASA' from dual union all
select 10194,'USA' from dual union all
select 10195,'USA' from dual union all
select 10196,'SAC' from dual union all
select 10197,'USA' from dual union all
select 10198,'NASA' from dual union all
select 10199,'NASA' from dual union all
select 102,'WEHR' from dual union all
select 1020,'USAF' from dual union all
select 10200,'MVS' from dual union all
select 10201,'RAE' from dual union all
select 10201,'WRE' from dual union all
select 10202,'AFSC' from dual union all
select 10203,'AOMC' from dual union all
select 10204,'RVSN' from dual union all
select 10205,'USAF' from dual union all
select 10206,'MVS' from dual union all
select 10207,'USN' from dual union all
select 10208,'NASA' from dual union all
select 10209,'NASA' from dual union all
select 1021,'PARD' from dual union all
select 10210,'NASA' from dual union all
select 10211,'RAE' from dual union all
select 10211,'WRE' from dual union all
select 10212,'MVS' from dual union all
select 10213,'SAC' from dual union all
select 10214,'WRE' from dual union all
select 10215,'AOMC' from dual union all
select 10216,'USN' from dual union all
select 10217,'USN' from dual union all
select 10218,'USN' from dual union all
select 10219,'RVSN' from dual union all
select 1022,'USAF' from dual union all
select 10220,'USAF' from dual union all
select 10221,'MVS' from dual union all
select 10222,'SAC' from dual union all
select 10223,'MVS' from dual union all
select 10224,'USAF' from dual union all
select 10225,'LARC' from dual union all
select 10225,'US' from dual union all
select 10226,'LARC' from dual union all
select 10226,'US' from dual union all
select 10227,'LARC' from dual union all
select 10227,'US' from dual union all
select 10228,'TXWC' from dual union all
select 10229,'TXWC' from dual union all
select 1023,'PARD' from dual union all
select 10230,'4WG' from dual union all
select 10231,'GMS' from dual union all
select 10232,'CNES' from dual union all
select 10233,'CNES' from dual union all
select 10234,'CNES' from dual union all
select 10235,'USAF' from dual union all
select 10236,'ONR' from dual union all
select 10237,'ONR' from dual union all
select 10238,'ONR' from dual union all
select 10239,'CNES' from dual union all
select 1024,'USA' from dual union all
select 10240,'CNES' from dual union all
select 10241,'MSFC' from dual union all
select 10242,'TXWC' from dual union all
select 10243,'NASA' from dual union all
select 10243,'UCAN' from dual union all
select 10244,'CNES' from dual union all
select 10245,'CNES' from dual union all
select 10246,'4WG' from dual union all
select 10247,'MSFC' from dual union all
select 10248,'USAF' from dual union all
select 10249,'TXWC' from dual union all
select 1025,'ARDC' from dual union all
select 1025,'US' from dual union all
select 10250,'TXWC' from dual union all
select 10251,'USAF' from dual;
insert into LAUNCH_AGENCY
select 10252,'USAF' from dual union all
select 10253,'USAF' from dual union all
select 10254,'TXWC' from dual union all
select 10255,'TXWC' from dual union all
select 10256,'USAF' from dual union all
select 10257,'LARCN' from dual union all
select 10258,'TXWC' from dual union all
select 10259,'USAF' from dual union all
select 1026,'ORDRSA' from dual union all
select 10260,'USAF' from dual union all
select 10261,'CNES' from dual union all
select 10262,'CNES' from dual union all
select 10263,'USAF' from dual union all
select 10264,'UNK' from dual union all
select 10264,'USAF' from dual union all
select 10265,'TXWC' from dual union all
select 10266,'TXWC' from dual union all
select 10267,'TXWC' from dual union all
select 10268,'TXWC' from dual union all
select 10269,'TXWC' from dual union all
select 1027,'USAF' from dual union all
select 10270,'ONR' from dual union all
select 10271,'ONR' from dual union all
select 10272,'ONR' from dual union all
select 10273,'ONR' from dual union all
select 10274,'WRE' from dual union all
select 10275,'MSFC' from dual union all
select 10276,'MSFC' from dual union all
select 10277,'MSFC' from dual union all
select 10278,'MSFC' from dual union all
select 10279,'NASA' from dual union all
select 10279,'UIO' from dual union all
select 1028,'USAF' from dual union all
select 10280,'NASA' from dual union all
select 10280,'UIO' from dual union all
select 10281,'WRE' from dual union all
select 10282,'WRE' from dual union all
select 10283,'MSFC' from dual union all
select 10284,'MSFC' from dual union all
select 10285,'MSFC' from dual union all
select 10286,'NASA' from dual union all
select 10286,'UIO' from dual union all
select 10287,'USAF' from dual union all
select 10288,'USAF' from dual union all
select 10289,'NASA' from dual union all
select 1029,'PARD' from dual union all
select 10290,'USAF' from dual union all
select 10291,'NASA' from dual union all
select 10292,'RVSN' from dual union all
select 10293,'USAF' from dual union all
select 10294,'USAF' from dual union all
select 10295,'AFSC' from dual union all
select 10296,'RVSN' from dual union all
select 10297,'USAF' from dual union all
select 10298,'AFSC' from dual union all
select 10299,'RVSN' from dual union all
select 103,'WEHR' from dual union all
select 1030,'USA' from dual union all
select 10300,'RVSN' from dual union all
select 10301,'NASA' from dual union all
select 10302,'RVSN' from dual union all
select 10303,'RVSN' from dual union all
select 10304,'NASA' from dual union all
select 10305,'RVSN' from dual union all
select 10306,'AFSC' from dual union all
select 10307,'RVSN' from dual union all
select 10308,'USAF' from dual union all
select 10309,'RVSN' from dual union all
select 1031,'USAF' from dual union all
select 10310,'AFSC' from dual union all
select 10311,'NASA' from dual union all
select 10312,'AFSC' from dual union all
select 10313,'USAF' from dual union all
select 10314,'RVSN' from dual union all
select 10315,'RVSN' from dual union all
select 10316,'USAF' from dual union all
select 10317,'USAF' from dual union all
select 10318,'USAF' from dual union all
select 10319,'RVSN' from dual union all
select 1032,'PARD' from dual union all
select 10320,'RVSN' from dual union all
select 10321,'USAF' from dual union all
select 10322,'AFSC' from dual union all
select 10323,'USAF' from dual union all
select 10324,'RVSN' from dual union all
select 10325,'RVSN' from dual union all
select 10326,'USAF' from dual union all
select 10327,'USAF' from dual union all
select 10328,'RVSN' from dual union all
select 10329,'USAF' from dual union all
select 1033,'ORDRSA' from dual union all
select 10330,'RVSN' from dual union all
select 10331,'AFSC' from dual union all
select 10332,'RVSN' from dual union all
select 10333,'NASA' from dual union all
select 10334,'USAF' from dual union all
select 10335,'RVSN' from dual union all
select 10336,'RVSN' from dual union all
select 10337,'AFSC' from dual union all
select 10338,'USAF' from dual;
insert into LAUNCH_AGENCY
select 10339,'RVSN' from dual union all
select 1034,'USAF' from dual union all
select 10340,'USAF' from dual union all
select 10341,'RVSN' from dual union all
select 10342,'USAF' from dual union all
select 10343,'NASA' from dual union all
select 10344,'AFSC' from dual union all
select 10345,'RVSN' from dual union all
select 10346,'NASA' from dual union all
select 10347,'USAF' from dual union all
select 10348,'RVSN' from dual union all
select 10349,'USAF' from dual union all
select 1035,'USAF' from dual union all
select 10350,'AFSC' from dual union all
select 10351,'RVSN' from dual union all
select 10352,'RVSN' from dual union all
select 10353,'USAF' from dual union all
select 10354,'AFSC' from dual union all
select 10355,'RVSN' from dual union all
select 10356,'RVSN' from dual union all
select 10357,'USAF' from dual union all
select 10358,'USAF' from dual union all
select 10359,'USAF' from dual union all
select 1036,'ORDRSA' from dual union all
select 10360,'NASA' from dual union all
select 10361,'USAF' from dual union all
select 10362,'AFSC' from dual union all
select 10363,'USAF' from dual union all
select 10364,'RVSN' from dual union all
select 10365,'AFSC' from dual union all
select 10366,'RVSN' from dual union all
select 10367,'USAF' from dual union all
select 10368,'NASA' from dual union all
select 10369,'USAF' from dual union all
select 1037,'PARD' from dual union all
select 10370,'NASA' from dual union all
select 10371,'USAF' from dual union all
select 10372,'NASA' from dual union all
select 10373,'USAF' from dual union all
select 10374,'ARPA' from dual union all
select 10375,'NASA' from dual union all
select 10375,'USAF' from dual union all
select 10376,'USN' from dual union all
select 10377,'NASA' from dual union all
select 10378,'NASA' from dual union all
select 10378,'USAF' from dual union all
select 10379,'RVSN' from dual union all
select 1038,'ORDRSA' from dual union all
select 10380,'NASA' from dual union all
select 10381,'NASA' from dual union all
select 10382,'SAND' from dual union all
select 10383,'NASA' from dual union all
select 10384,'NASA' from dual union all
select 10384,'USAF' from dual union all
select 10385,'NASA' from dual union all
select 10386,'1STRAD' from dual union all
select 10387,'SAND' from dual union all
select 10388,'DASA' from dual union all
select 10389,'NASA' from dual union all
select 1039,'PARD' from dual union all
select 10390,'MVS' from dual union all
select 10391,'ISAS' from dual union all
select 10392,'IIAE' from dual union all
select 10393,'NASA' from dual union all
select 10394,'NASA' from dual union all
select 10394,'USAF' from dual union all
select 10395,'ISAS' from dual union all
select 10396,'NASA' from dual union all
select 10397,'NASA' from dual union all
select 10398,'NASA' from dual union all
select 10398,'USAF' from dual union all
select 10399,'NASA' from dual union all
select 104,'WEHR' from dual union all
select 1040,'ORDRSA' from dual union all
select 10400,'DASA' from dual union all
select 10401,'NASA' from dual union all
select 10402,'DASA' from dual union all
select 10403,'RVSN' from dual union all
select 10404,'RVSN' from dual union all
select 10405,'NASA' from dual union all
select 10405,'USAF' from dual union all
select 10406,'NASA' from dual union all
select 10406,'USAF' from dual union all
select 10407,'NASA' from dual union all
select 10408,'NASA' from dual union all
select 10409,'SAND' from dual union all
select 1041,'USAF' from dual union all
select 10410,'NASA' from dual union all
select 10410,'USAF' from dual union all
select 10411,'MVS' from dual union all
select 10412,'NASA' from dual union all
select 10413,'NASA' from dual union all
select 10413,'USAF' from dual union all
select 10414,'NASA' from dual union all
select 10415,'USN' from dual union all
select 10416,'NASA' from dual union all
select 10417,'NRCC' from dual union all
select 10418,'NASA' from dual union all
select 10419,'NASA' from dual union all
select 1042,'USAF' from dual;
insert into LAUNCH_AGENCY
select 10420,'DASA' from dual union all
select 10421,'NASA' from dual union all
select 10421,'USAF' from dual union all
select 10422,'NASA' from dual union all
select 10422,'USAF' from dual union all
select 10423,'NASA' from dual union all
select 10424,'NASA' from dual union all
select 10425,'AFSC' from dual union all
select 10426,'NASA' from dual union all
select 10426,'USAF' from dual union all
select 10427,'NASA' from dual union all
select 10427,'USAF' from dual union all
select 10428,'USA' from dual union all
select 10429,'USA' from dual union all
select 1043,'PARD' from dual union all
select 10430,'USA' from dual union all
select 10431,'USA' from dual union all
select 10432,'ARPA' from dual union all
select 10433,'RAE' from dual union all
select 10434,'NASA' from dual union all
select 10434,'USAF' from dual union all
select 10435,'USAF' from dual union all
select 10436,'NASA' from dual union all
select 10437,'DASA' from dual union all
select 10438,'NASA' from dual union all
select 10439,'NASA' from dual union all
select 10439,'USAF' from dual union all
select 1044,'USAF' from dual union all
select 10440,'MSFC' from dual union all
select 10441,'SAND' from dual union all
select 10442,'CNES' from dual union all
select 10443,'SAND' from dual union all
select 10444,'NASA' from dual union all
select 10445,'NASA' from dual union all
select 10445,'USAF' from dual union all
select 10446,'NASA' from dual union all
select 10446,'USAF' from dual union all
select 10447,'USAF' from dual union all
select 10448,'NASA' from dual union all
select 10449,'USA' from dual union all
select 1045,'PARD' from dual union all
select 10450,'USA' from dual union all
select 10451,'USA' from dual union all
select 10452,'USA' from dual union all
select 10453,'ARPA' from dual union all
select 10454,'ARPA' from dual union all
select 10455,'ARPA' from dual union all
select 10456,'ONERA' from dual union all
select 10457,'DASA' from dual union all
select 10458,'NASA' from dual union all
select 10459,'NASA' from dual union all
select 10459,'USAF' from dual union all
select 1046,'USAF' from dual union all
select 10460,'NASA' from dual union all
select 10461,'NASA' from dual union all
select 10461,'USAF' from dual union all
select 10462,'NASA' from dual union all
select 10463,'APGC' from dual union all
select 10464,'NASA' from dual union all
select 10465,'USAF' from dual union all
select 10466,'NASA' from dual union all
select 10466,'USAF' from dual union all
select 10467,'NASA' from dual union all
select 10468,'NASA' from dual union all
select 10468,'USAF' from dual union all
select 10469,'MVS' from dual union all
select 1047,'USAF' from dual union all
select 10470,'NASA' from dual union all
select 10471,'NASA' from dual union all
select 10472,'NASA' from dual union all
select 10472,'USAF' from dual union all
select 10473,'RAE' from dual union all
select 10474,'NASA' from dual union all
select 10474,'USAF' from dual union all
select 10475,'NASA' from dual union all
select 10476,'NASA' from dual union all
select 10476,'USAF' from dual union all
select 10477,'ONERA' from dual union all
select 10478,'USA' from dual union all
select 10479,'USA' from dual union all
select 1048,'PARD' from dual union all
select 10480,'AEC' from dual union all
select 10480,'LRL' from dual union all
select 10481,'USA' from dual union all
select 10482,'UKMO' from dual union all
select 10483,'ONR' from dual union all
select 10484,'ONR' from dual union all
select 10485,'ONR' from dual union all
select 10486,'ONR' from dual union all
select 10487,'ONR' from dual union all
select 10488,'USA' from dual union all
select 10489,'USA' from dual union all
select 1049,'USAF' from dual union all
select 10490,'WRE' from dual union all
select 10491,'USAF' from dual union all
select 10492,'WRE' from dual union all
select 10493,'LARCN' from dual union all
select 10494,'USN' from dual union all
select 10495,'USN' from dual union all
select 10496,'NOTS' from dual;
insert into LAUNCH_AGENCY
select 10497,'USN' from dual union all
select 10498,'RVSN' from dual union all
select 10499,'RVSN' from dual union all
select 105,'WEHR' from dual union all
select 1050,'PARD' from dual union all
select 10500,'WRE' from dual union all
select 10501,'WRE' from dual union all
select 10502,'MVS' from dual union all
select 10503,'USAF' from dual union all
select 10504,'RAE' from dual union all
select 10505,'SAC' from dual union all
select 10506,'USAF' from dual union all
select 10507,'NASA' from dual union all
select 10507,'USAF' from dual union all
select 10508,'NASA' from dual union all
select 10509,'NASA' from dual union all
select 1051,'USAF' from dual union all
select 10510,'RVSN' from dual union all
select 10511,'NASA' from dual union all
select 10512,'RVSN' from dual union all
select 10513,'ONERA' from dual union all
select 10514,'USAF' from dual union all
select 10515,'RVSN' from dual union all
select 10516,'NASA' from dual union all
select 10516,'USAF' from dual union all
select 10517,'NASA' from dual union all
select 10518,'NASA' from dual union all
select 10519,'NASA' from dual union all
select 10519,'USAF' from dual union all
select 1052,'USAF' from dual union all
select 10520,'NASA' from dual union all
select 10521,'NASA' from dual union all
select 10522,'NASA' from dual union all
select 10523,'1STRAD' from dual union all
select 10524,'SAND' from dual union all
select 10525,'MVS' from dual union all
select 10526,'NASA' from dual union all
select 10526,'USAF' from dual union all
select 10527,'NASA' from dual union all
select 10528,'NASA' from dual union all
select 10528,'USAF' from dual union all
select 10529,'NASA' from dual union all
select 1053,'USAF' from dual union all
select 10530,'NASA' from dual union all
select 10531,'NASA' from dual union all
select 10532,'SAC' from dual union all
select 10533,'NASA' from dual union all
select 10534,'RVSN' from dual union all
select 10535,'NASA' from dual union all
select 10536,'NASA' from dual union all
select 10537,'NASA' from dual union all
select 10538,'ONERA' from dual union all
select 10539,'NASA' from dual union all
select 1054,'USAF' from dual union all
select 10540,'NASA' from dual union all
select 10541,'MVS' from dual union all
select 10542,'RVSN' from dual union all
select 10543,'NASA' from dual union all
select 10543,'USAF' from dual union all
select 10544,'NASA' from dual union all
select 10545,'NASA' from dual union all
select 10546,'NASA' from dual union all
select 10547,'NASA' from dual union all
select 10548,'NASA' from dual union all
select 10548,'USAF' from dual union all
select 10549,'NASA' from dual union all
select 10549,'USAF' from dual union all
select 1055,'LRBA' from dual union all
select 10550,'NASA' from dual union all
select 10551,'USAF' from dual union all
select 10552,'NASA' from dual union all
select 10553,'NASA' from dual union all
select 10554,'NASA' from dual union all
select 10554,'USAF' from dual union all
select 10555,'NASA' from dual union all
select 10556,'NASA' from dual union all
select 10557,'NASA' from dual union all
select 10558,'NASA' from dual union all
select 10559,'NASA' from dual union all
select 1056,'USAF' from dual union all
select 10560,'USAF' from dual union all
select 10561,'ARPA' from dual union all
select 10562,'ARPA' from dual union all
select 10563,'NASA' from dual union all
select 10564,'NASA' from dual union all
select 10565,'NASA' from dual union all
select 10565,'USAF' from dual union all
select 10566,'NASA' from dual union all
select 10567,'NASA' from dual union all
select 10568,'CNES' from dual union all
select 10569,'NASA' from dual union all
select 10569,'USAF' from dual union all
select 1057,'LRBA' from dual union all
select 10570,'NASA' from dual union all
select 10571,'CNES' from dual union all
select 10572,'NASA' from dual union all
select 10573,'NASA' from dual union all
select 10573,'USAF' from dual union all
select 10574,'NASA' from dual union all
select 10574,'USAF' from dual;
insert into LAUNCH_AGENCY
select 10575,'WRE' from dual union all
select 10576,'WRE' from dual union all
select 10577,'ARPA' from dual union all
select 10578,'ARPA' from dual union all
select 10579,'ARPA' from dual union all
select 1058,'LRBA' from dual union all
select 10580,'NASA' from dual union all
select 10580,'USAF' from dual union all
select 10581,'RVSN' from dual union all
select 10582,'NASA' from dual union all
select 10583,'RVSN' from dual union all
select 10584,'USAF' from dual union all
select 10585,'RVSN' from dual union all
select 10586,'RVSN' from dual union all
select 10587,'AFSC' from dual union all
select 10588,'NASA' from dual union all
select 10589,'USAF' from dual union all
select 1059,'ORD' from dual union all
select 10590,'AFSC' from dual union all
select 10591,'RVSN' from dual union all
select 10592,'RVSN' from dual union all
select 10593,'HARP' from dual union all
select 10594,'NRCC' from dual union all
select 10595,'USA' from dual union all
select 10596,'USA' from dual union all
select 10597,'WRE' from dual union all
select 10598,'USA' from dual union all
select 10599,'USAF' from dual union all
select 106,'WEHR' from dual union all
select 1060,'USA' from dual union all
select 10600,'NASA' from dual union all
select 10600,'USAF' from dual union all
select 10601,'NASA' from dual union all
select 10602,'NASA' from dual union all
select 10602,'USAF' from dual union all
select 10603,'DLR' from dual union all
select 10604,'DLR' from dual union all
select 10605,'DLR' from dual union all
select 10606,'DLR' from dual union all
select 10607,'DLR' from dual union all
select 10608,'DLR' from dual union all
select 10609,'DLR' from dual union all
select 1061,'USAF' from dual union all
select 10610,'DLR' from dual union all
select 10611,'DLR' from dual union all
select 10612,'DLR' from dual union all
select 10613,'USA' from dual union all
select 10614,'DLR' from dual union all
select 10615,'DLR' from dual union all
select 10616,'DLR' from dual union all
select 10617,'DLR' from dual union all
select 10618,'DLR' from dual union all
select 10619,'DLR' from dual union all
select 1062,'ORDRSA' from dual union all
select 10620,'DLR' from dual union all
select 10621,'DLR' from dual union all
select 10622,'DLR' from dual union all
select 10623,'DLR' from dual union all
select 10624,'USA' from dual union all
select 10625,'DLR' from dual union all
select 10626,'DLR' from dual union all
select 10627,'DLR' from dual union all
select 10628,'DLR' from dual union all
select 10629,'DLR' from dual union all
select 1063,'ORDRSA' from dual union all
select 10630,'DLR' from dual union all
select 10631,'DLR' from dual union all
select 10632,'DLR' from dual union all
select 10633,'DLR' from dual union all
select 10634,'LARCN' from dual union all
select 10635,'USA' from dual union all
select 10636,'USA' from dual union all
select 10637,'USA' from dual union all
select 10638,'USA' from dual union all
select 10639,'USA' from dual union all
select 1064,'ORDRSA' from dual union all
select 10640,'USA' from dual union all
select 10641,'USA' from dual union all
select 10642,'USA' from dual union all
select 10643,'USA' from dual union all
select 10644,'USA' from dual union all
select 10645,'USA' from dual union all
select 10646,'ONERA' from dual union all
select 10647,'USA' from dual union all
select 10648,'USA' from dual union all
select 10649,'USA' from dual union all
select 1065,'ORDRSA' from dual union all
select 10650,'USA' from dual union all
select 10651,'USA' from dual union all
select 10652,'USA' from dual union all
select 10653,'NOTS' from dual union all
select 10654,'NOTS' from dual union all
select 10655,'NOTS' from dual union all
select 10656,'NASA' from dual union all
select 10656,'USAF' from dual union all
select 10657,'USN' from dual union all
select 10658,'NOTS' from dual union all
select 10659,'USN' from dual union all
select 1066,'USAF' from dual union all
select 10660,'SAND' from dual;
insert into LAUNCH_AGENCY
select 10661,'USA' from dual union all
select 10662,'USA' from dual union all
select 10663,'STA' from dual union all
select 10664,'SIMED' from dual union all
select 10665,'STA' from dual union all
select 10666,'USAF' from dual union all
select 10667,'STA' from dual union all
select 10668,'USA' from dual union all
select 10669,'USA' from dual union all
select 1067,'USAF' from dual union all
select 10670,'USA' from dual union all
select 10671,'ONERA' from dual union all
select 10672,'USAF' from dual union all
select 10673,'CNES' from dual union all
select 10674,'USAF' from dual union all
select 10675,'USA' from dual union all
select 10676,'USAF' from dual union all
select 10677,'ONERA' from dual union all
select 10678,'ONERA' from dual union all
select 10679,'ONERA' from dual union all
select 1068,'ORDRSA' from dual union all
select 10680,'ONERA' from dual union all
select 10681,'DLR' from dual union all
select 10682,'DLR' from dual union all
select 10683,'DLR' from dual union all
select 10684,'DLR' from dual union all
select 10685,'DLR' from dual union all
select 10686,'DLR' from dual union all
select 10687,'DLR' from dual union all
select 10688,'DLR' from dual union all
select 10689,'DLR' from dual union all
select 1069,'ORDRSA' from dual union all
select 10690,'DLR' from dual union all
select 10691,'DLR' from dual union all
select 10692,'DLR' from dual union all
select 10693,'DLR' from dual union all
select 10694,'DLR' from dual union all
select 10695,'DLR' from dual union all
select 10696,'DLR' from dual union all
select 10697,'PRO' from dual union all
select 10698,'PRO' from dual union all
select 10699,'AN' from dual union all
select 107,'WEHR' from dual union all
select 1070,'ORDRSA' from dual union all
select 10700,'AN' from dual union all
select 10701,'AN' from dual union all
select 10702,'AN' from dual union all
select 10703,'USA' from dual union all
select 10704,'USA' from dual union all
select 10705,'USN' from dual union all
select 10706,'AFSC' from dual union all
select 10707,'NRL' from dual union all
select 10708,'MVS' from dual union all
select 10709,'ERDA' from dual union all
select 10709,'USAF' from dual union all
select 1071,'USA' from dual union all
select 10710,'RVSN' from dual union all
select 10711,'ERDA' from dual union all
select 10711,'USAF' from dual union all
select 10712,'MVS' from dual union all
select 10713,'MVS' from dual union all
select 10714,'MVS' from dual union all
select 10715,'MVS' from dual union all
select 10716,'RAE' from dual union all
select 10716,'WRE' from dual union all
select 10717,'NASA' from dual union all
select 10718,'1STRAD' from dual union all
select 10719,'1STRAD' from dual union all
select 1072,'USAF' from dual union all
select 10720,'81FA1' from dual union all
select 10721,'81FA1' from dual union all
select 10722,'AFSC' from dual union all
select 10723,'AFSC' from dual union all
select 10724,'AFSC' from dual union all
select 10725,'RVSN' from dual union all
select 10726,'NASA' from dual union all
select 10727,'AFSC' from dual union all
select 10728,'1STRAD' from dual union all
select 10729,'1STRAD' from dual union all
select 1073,'USA' from dual union all
select 10730,'USA' from dual union all
select 10731,'MVS' from dual union all
select 10732,'NASA' from dual union all
select 10733,'10ADS' from dual union all
select 10734,'USN' from dual union all
select 10735,'RVSN' from dual union all
select 10736,'MVS' from dual union all
select 10737,'RVSN' from dual union all
select 10738,'WRE' from dual union all
select 10739,'1STRAD' from dual union all
select 1074,'USA' from dual union all
select 10740,'NASA' from dual union all
select 10741,'MVS' from dual union all
select 10742,'SAND' from dual union all
select 10743,'RAE' from dual union all
select 10743,'WRE' from dual union all
select 10744,'ONERA' from dual union all
select 10745,'RAE' from dual union all
select 10746,'ONERA' from dual union all
select 10747,'USN' from dual;
insert into LAUNCH_AGENCY
select 10748,'MVS' from dual union all
select 10749,'USAF' from dual union all
select 1075,'ORDRSA' from dual union all
select 10750,'RAE' from dual union all
select 10750,'WRE' from dual union all
select 10751,'NDRE' from dual union all
select 10752,'AFSC' from dual union all
select 10753,'USAF' from dual union all
select 10754,'AFSC' from dual union all
select 10755,'USN' from dual union all
select 10756,'AEC' from dual union all
select 10756,'LRL' from dual union all
select 10757,'ONERA' from dual union all
select 10758,'MVS' from dual union all
select 10759,'NDRE' from dual union all
select 1076,'PARD' from dual union all
select 10760,'MVS' from dual union all
select 10761,'USN' from dual union all
select 10762,'MVS' from dual union all
select 10763,'NOTS' from dual union all
select 10764,'SAND' from dual union all
select 10765,'NASA' from dual union all
select 10766,'NDRE' from dual union all
select 10767,'AFSC' from dual union all
select 10768,'RVSN' from dual union all
select 10769,'NASA' from dual union all
select 1077,'USAF' from dual union all
select 10770,'1STRAD' from dual union all
select 10771,'MVS' from dual union all
select 10772,'NASA' from dual union all
select 10773,'MVS' from dual union all
select 10774,'AFSC' from dual union all
select 10775,'NASA' from dual union all
select 10776,'CRA' from dual union all
select 10777,'NASA' from dual union all
select 10778,'USA' from dual union all
select 10779,'1STRAD' from dual union all
select 1078,'USA' from dual union all
select 10780,'RVSN' from dual union all
select 10781,'NASA' from dual union all
select 10782,'AFSC' from dual union all
select 10783,'APGC' from dual union all
select 10784,'CRA' from dual union all
select 10785,'USAF' from dual union all
select 10786,'1STRAD' from dual union all
select 10787,'DASA' from dual union all
select 10788,'USN' from dual union all
select 10789,'NOTS' from dual union all
select 1079,'USA' from dual union all
select 10790,'ISAS' from dual union all
select 10791,'AFSC' from dual union all
select 10792,'CRA' from dual union all
select 10793,'MVS' from dual union all
select 10794,'MVS' from dual union all
select 10795,'NASA' from dual union all
select 10796,'USN' from dual union all
select 10797,'NASA' from dual union all
select 10798,'AFSC' from dual union all
select 10799,'NRCC' from dual union all
select 108,'WEHR' from dual union all
select 1080,'6555GMS' from dual union all
select 10800,'KPNO' from dual union all
select 10801,'NRCC' from dual union all
select 10802,'MVS' from dual union all
select 10803,'AFSC' from dual union all
select 10804,'NASA' from dual union all
select 10805,'NASA' from dual union all
select 10806,'NASA' from dual union all
select 10807,'CNES' from dual union all
select 10808,'MVS' from dual union all
select 10809,'CNES' from dual union all
select 1081,'USAF' from dual union all
select 10810,'RAE' from dual union all
select 10810,'WRE' from dual union all
select 10811,'1STRAD' from dual union all
select 10812,'CNES' from dual union all
select 10813,'CNES' from dual union all
select 10814,'CNES' from dual union all
select 10815,'NASA' from dual union all
select 10816,'WRE' from dual union all
select 10817,'NASA' from dual union all
select 10818,'NASA' from dual union all
select 10819,'RVSN' from dual union all
select 1082,'USA' from dual union all
select 10820,'NASA' from dual union all
select 10821,'NRCC' from dual union all
select 10822,'NASA' from dual union all
select 10823,'NRCC' from dual union all
select 10824,'NASA' from dual union all
select 10825,'NASA' from dual union all
select 10826,'NASA' from dual union all
select 10827,'USN' from dual union all
select 10828,'USN' from dual union all
select 10829,'FKGR12' from dual union all
select 1083,'USA' from dual union all
select 10830,'NRCC' from dual union all
select 10831,'10ADS' from dual union all
select 10832,'NRCC' from dual union all
select 10833,'NASA' from dual union all
select 10834,'1STRAD' from dual;
insert into LAUNCH_AGENCY
select 10835,'FKGR12' from dual union all
select 10836,'AFSC' from dual union all
select 10837,'1STRAD' from dual union all
select 10838,'MVS' from dual union all
select 10839,'DASA' from dual union all
select 1084,'USAF' from dual union all
select 10840,'1STRAD' from dual union all
select 10841,'DASA' from dual union all
select 10842,'CNES' from dual union all
select 10843,'NRL' from dual union all
select 10844,'1STRAD' from dual union all
select 10845,'USN' from dual union all
select 10846,'CNES' from dual union all
select 10847,'CNES' from dual union all
select 10848,'CNES' from dual union all
select 10849,'1STRAD' from dual union all
select 1085,'USAF' from dual union all
select 10850,'SAND' from dual union all
select 10851,'RVSN' from dual union all
select 10852,'USN' from dual union all
select 10853,'SAND' from dual union all
select 10854,'CNES' from dual union all
select 10855,'RAE' from dual union all
select 10855,'WRE' from dual union all
select 10856,'WRE' from dual union all
select 10857,'SAND' from dual union all
select 10858,'RVSN' from dual union all
select 10859,'RVSN' from dual union all
select 1086,'ORDRSA' from dual union all
select 10860,'1STRAD' from dual union all
select 10861,'NASA' from dual union all
select 10862,'SAND' from dual union all
select 10863,'USAF' from dual union all
select 10864,'USAF' from dual union all
select 10865,'USAF' from dual union all
select 10866,'USAF' from dual union all
select 10867,'USAF' from dual union all
select 10868,'USAF' from dual union all
select 10869,'SAND' from dual union all
select 1087,'ORDRSA' from dual union all
select 10870,'MVS' from dual union all
select 10871,'MVS' from dual union all
select 10872,'USN' from dual union all
select 10873,'SAND' from dual union all
select 10874,'RVSN' from dual union all
select 10875,'1STRAD' from dual union all
select 10876,'SAND' from dual union all
select 10877,'USN' from dual union all
select 10878,'USN' from dual union all
select 10879,'SAND' from dual union all
select 1088,'USA' from dual union all
select 10880,'USAF' from dual union all
select 10881,'NOTS' from dual union all
select 10882,'USAF' from dual union all
select 10883,'USN' from dual union all
select 10884,'RVSN' from dual union all
select 10885,'RVSN' from dual union all
select 10886,'RVSN' from dual union all
select 10887,'USA' from dual union all
select 10888,'USAF' from dual union all
select 10889,'DASA' from dual union all
select 1089,'ORD' from dual union all
select 10890,'RVSN' from dual union all
select 10891,'RVSN' from dual union all
select 10892,'NASA' from dual union all
select 10893,'ELDO' from dual union all
select 10894,'SAND' from dual union all
select 10895,'MVS' from dual union all
select 10896,'USN' from dual union all
select 10897,'USN' from dual union all
select 10898,'CNES' from dual union all
select 10899,'1STRAD' from dual union all
select 109,'WEHR' from dual union all
select 1090,'ORD' from dual union all
select 10900,'ONERA' from dual union all
select 10901,'USAF' from dual union all
select 10902,'NASA' from dual union all
select 10903,'NASA' from dual union all
select 10904,'MVS' from dual union all
select 10905,'1STRAD' from dual union all
select 10906,'MVS' from dual union all
select 10907,'WRE' from dual union all
select 10908,'ONERA' from dual union all
select 10909,'CNES' from dual union all
select 1091,'ARDC' from dual union all
select 10910,'1STRAD' from dual union all
select 10911,'NRL' from dual union all
select 10912,'USN' from dual union all
select 10913,'RVSN' from dual union all
select 10914,'USN' from dual union all
select 10915,'NRCC' from dual union all
select 10916,'SAC' from dual union all
select 10917,'USN' from dual union all
select 10918,'USN' from dual union all
select 10919,'KPNO' from dual union all
select 1092,'ARDC' from dual union all
select 10920,'USN' from dual union all
select 10921,'RVSN' from dual union all
select 10922,'USN' from dual union all
select 10923,'USN' from dual;
insert into LAUNCH_AGENCY
select 10924,'USN' from dual union all
select 10925,'RVSN' from dual union all
select 10926,'AFSC' from dual union all
select 10927,'NASA' from dual union all
select 10928,'NASA' from dual union all
select 10929,'AFSC' from dual union all
select 1093,'USA' from dual union all
select 10930,'1STRAD' from dual union all
select 10931,'SAND' from dual union all
select 10932,'GB5' from dual union all
select 10933,'RVSN' from dual union all
select 10934,'MVS' from dual union all
select 10935,'NASA' from dual union all
select 10936,'SAND' from dual union all
select 10937,'USN' from dual union all
select 10938,'MVS' from dual union all
select 10939,'USAF' from dual union all
select 1094,'ORD' from dual union all
select 10940,'SAND' from dual union all
select 10941,'ESRO' from dual union all
select 10942,'1STRAD' from dual union all
select 10943,'MVS' from dual union all
select 10944,'NRL' from dual union all
select 10945,'NASA' from dual union all
select 10946,'USN' from dual union all
select 10947,'USN' from dual union all
select 10948,'USN' from dual union all
select 10949,'MVS' from dual union all
select 1095,'ARDC' from dual union all
select 10950,'ESRO' from dual union all
select 10951,'USAF' from dual union all
select 10952,'1STRAD' from dual union all
select 10953,'GB5' from dual union all
select 10954,'NASA' from dual union all
select 10955,'WRE' from dual union all
select 10956,'AFSC' from dual union all
select 10957,'MVS' from dual union all
select 10958,'GB5' from dual union all
select 10959,'ISAS' from dual union all
select 1096,'ARDC' from dual union all
select 10960,'1STRAD' from dual union all
select 10961,'WRE' from dual union all
select 10962,'MVS' from dual union all
select 10963,'WRE' from dual union all
select 10964,'RAE' from dual union all
select 10964,'WRE' from dual union all
select 10965,'NASA' from dual union all
select 10966,'WRE' from dual union all
select 10967,'1STRAD' from dual union all
select 10968,'NASA' from dual union all
select 10969,'NASA' from dual union all
select 1097,'ARDC' from dual union all
select 10970,'NASA' from dual union all
select 10971,'NASA' from dual union all
select 10972,'NASA' from dual union all
select 10973,'NASA' from dual union all
select 10974,'SAND' from dual union all
select 10975,'USN' from dual union all
select 10976,'USN' from dual union all
select 10977,'WRE' from dual union all
select 10978,'RVSN' from dual union all
select 10979,'WRE' from dual union all
select 1098,'ARDC' from dual union all
select 10980,'NASA' from dual union all
select 10981,'NOTS' from dual union all
select 10982,'FKGR13' from dual union all
select 10983,'MVS' from dual union all
select 10984,'MVS' from dual union all
select 10985,'RAE' from dual union all
select 10985,'WRE' from dual union all
select 10986,'WRE' from dual union all
select 10987,'USAF' from dual union all
select 10988,'MVS' from dual union all
select 10989,'NASA' from dual union all
select 1099,'USA' from dual union all
select 10990,'USAF' from dual union all
select 10991,'NASA' from dual union all
select 10992,'SAND' from dual union all
select 10993,'SAND' from dual union all
select 10994,'SAND' from dual union all
select 10995,'FKGR13' from dual union all
select 10996,'STA' from dual union all
select 10997,'STA' from dual union all
select 10998,'NASA' from dual union all
select 10999,'NASA' from dual union all
select 11,'WEHR' from dual union all
select 110,'WEHR' from dual union all
select 1100,'USA' from dual union all
select 11000,'NASA' from dual union all
select 11001,'ISAS' from dual union all
select 11002,'ISAS' from dual union all
select 11003,'1STRAD' from dual union all
select 11004,'RVSN' from dual union all
select 11005,'USAF' from dual union all
select 11006,'NASA' from dual union all
select 11007,'SAC' from dual union all
select 11008,'USAF' from dual union all
select 11009,'ISAS' from dual union all
select 1101,'APL' from dual union all
select 11010,'NASA' from dual;
insert into LAUNCH_AGENCY
select 11011,'SAND' from dual union all
select 11012,'USN' from dual union all
select 11013,'USN' from dual union all
select 11014,'SAC' from dual union all
select 11015,'USA' from dual union all
select 11016,'USA' from dual union all
select 11017,'MVS' from dual union all
select 11018,'RVSN' from dual union all
select 11019,'CNES' from dual union all
select 1102,'APL' from dual union all
select 11020,'RVSN' from dual union all
select 11021,'USN' from dual union all
select 11022,'MVS' from dual union all
select 11023,'SAND' from dual union all
select 11024,'RVSN' from dual union all
select 11025,'RAE' from dual union all
select 11026,'MVS' from dual union all
select 11027,'MVS' from dual union all
select 11028,'NASA' from dual union all
select 11029,'1STRAD' from dual union all
select 1103,'GAU4' from dual union all
select 11030,'USN' from dual union all
select 11031,'CNES' from dual union all
select 11032,'CARDE' from dual union all
select 11033,'NASA' from dual union all
select 11034,'NASA' from dual union all
select 11035,'SAC' from dual union all
select 11036,'SAND' from dual union all
select 11037,'NASA' from dual union all
select 11038,'NMC' from dual union all
select 11039,'SAC' from dual union all
select 1104,'GAU4' from dual union all
select 11040,'RAE' from dual union all
select 11040,'WRE' from dual union all
select 11041,'NASA' from dual union all
select 11042,'CARDE' from dual union all
select 11043,'NASA' from dual union all
select 11044,'NASA' from dual union all
select 11045,'NASA' from dual union all
select 11046,'NASA' from dual union all
select 11047,'MVS' from dual union all
select 11048,'SAC' from dual union all
select 11049,'NASA' from dual union all
select 1105,'GAU4' from dual union all
select 11050,'NASA' from dual union all
select 11051,'NASA' from dual union all
select 11052,'NASA' from dual union all
select 11053,'RVSN' from dual union all
select 11054,'1STRAD' from dual union all
select 11055,'NASA' from dual union all
select 11056,'NASA' from dual union all
select 11057,'NASA' from dual union all
select 11058,'NASA' from dual union all
select 11059,'NASA' from dual union all
select 1106,'GAU4' from dual union all
select 11060,'USN' from dual union all
select 11061,'MVS' from dual union all
select 11062,'RAE' from dual union all
select 11062,'WRE' from dual union all
select 11063,'WRE' from dual union all
select 11064,'RVSN' from dual union all
select 11065,'1STRAD' from dual union all
select 11066,'NASA' from dual union all
select 11067,'1STRAD' from dual union all
select 11068,'USAF' from dual union all
select 11069,'MVS' from dual union all
select 1107,'GAU4' from dual union all
select 11070,'NASA' from dual union all
select 11071,'SAC' from dual union all
select 11072,'NASA' from dual union all
select 11073,'USAF' from dual union all
select 11074,'USA' from dual union all
select 11075,'1STRAD' from dual union all
select 11076,'USA' from dual union all
select 11077,'DASA' from dual union all
select 11078,'DASA' from dual union all
select 11079,'DASA' from dual union all
select 1108,'APL' from dual union all
select 11080,'SAC' from dual union all
select 11081,'1STRAD' from dual union all
select 11082,'RAE' from dual union all
select 11082,'WRE' from dual union all
select 11083,'NASA' from dual union all
select 11084,'RAE' from dual union all
select 11084,'WRE' from dual union all
select 11085,'USN' from dual union all
select 11086,'MVS' from dual union all
select 11087,'1STRAD' from dual union all
select 11088,'RVSN' from dual union all
select 11089,'1STRAD' from dual union all
select 1109,'ARDC' from dual union all
select 11090,'RVSN' from dual union all
select 11091,'RVSN' from dual union all
select 11092,'GURVO' from dual union all
select 11093,'1STRAD' from dual union all
select 11094,'NRCC' from dual union all
select 11095,'KPNO' from dual union all
select 11096,'SAC' from dual union all
select 11097,'AFSC' from dual union all
select 11098,'MVS' from dual;
insert into LAUNCH_AGENCY
select 11099,'RAE' from dual union all
select 11099,'WRE' from dual union all
select 111,'WER485' from dual union all
select 1110,'GAU4' from dual union all
select 11100,'WRE' from dual union all
select 11101,'NASA' from dual union all
select 11102,'AFCRL' from dual union all
select 11103,'1STRAD' from dual union all
select 11104,'USN' from dual union all
select 11105,'SAC' from dual union all
select 11106,'1STRAD' from dual union all
select 11107,'AFSC' from dual union all
select 11108,'USN' from dual union all
select 11109,'RVSN' from dual union all
select 1111,'GAU4' from dual union all
select 11110,'RAE' from dual union all
select 11110,'WRE' from dual union all
select 11111,'RVSN' from dual union all
select 11112,'RVSN' from dual union all
select 11113,'SAND' from dual union all
select 11114,'RVSN' from dual union all
select 11115,'USAF' from dual union all
select 11116,'RVSN' from dual union all
select 11117,'NASA' from dual union all
select 11118,'RVSN' from dual union all
select 11119,'NASA' from dual union all
select 1112,'GAU4' from dual union all
select 11120,'USN' from dual union all
select 11121,'1STRAD' from dual union all
select 11122,'RAE' from dual union all
select 11122,'WRE' from dual union all
select 11123,'PRO' from dual union all
select 11124,'USA' from dual union all
select 11125,'MVS' from dual union all
select 11126,'ONERA' from dual union all
select 11127,'ONERA' from dual union all
select 11128,'SAC' from dual union all
select 11129,'RVSN' from dual union all
select 1113,'GAU4' from dual union all
select 11130,'NASA' from dual union all
select 11131,'RVSN' from dual union all
select 11132,'CNES' from dual union all
select 11133,'MVS' from dual union all
select 11134,'CARDE' from dual union all
select 11135,'CNES' from dual union all
select 11136,'CARDE' from dual union all
select 11137,'NASA' from dual union all
select 11138,'SAND' from dual union all
select 11139,'NASA' from dual union all
select 1114,'GAU4' from dual union all
select 11140,'NASA' from dual union all
select 11141,'NASA' from dual union all
select 11142,'NASA' from dual union all
select 11143,'MVS' from dual union all
select 11144,'NASA' from dual union all
select 11145,'RVSN' from dual union all
select 11146,'MVS' from dual union all
select 11147,'USN' from dual union all
select 11148,'USN' from dual union all
select 11149,'NOTS' from dual union all
select 1115,'GAU4' from dual union all
select 11150,'SAND' from dual union all
select 11151,'WRE' from dual union all
select 11152,'MVS' from dual union all
select 11153,'MVS' from dual union all
select 11154,'USAF' from dual union all
select 11155,'APGC' from dual union all
select 11156,'USN' from dual union all
select 11157,'USN' from dual union all
select 11158,'USN' from dual union all
select 11159,'USN' from dual union all
select 1116,'GAU4' from dual union all
select 11160,'NASA' from dual union all
select 11161,'ELDO' from dual union all
select 11162,'RVSN' from dual union all
select 11163,'NASA' from dual union all
select 11164,'WRE' from dual union all
select 11165,'WRE' from dual union all
select 11166,'NASA' from dual union all
select 11167,'SAND' from dual union all
select 11168,'NASA' from dual union all
select 11169,'NASA' from dual union all
select 1117,'GAU4' from dual union all
select 11170,'RAE' from dual union all
select 11170,'WRE' from dual union all
select 11171,'USAF' from dual union all
select 11172,'USAF' from dual union all
select 11173,'SAND' from dual union all
select 11174,'USN' from dual union all
select 11175,'USN' from dual union all
select 11176,'USN' from dual union all
select 11177,'USN' from dual union all
select 11178,'AFSC' from dual union all
select 11179,'RVSN' from dual union all
select 1118,'GAU4' from dual union all
select 11180,'AFSC' from dual union all
select 11181,'MVS' from dual union all
select 11182,'ESRO' from dual union all
select 11183,'NASA' from dual union all
select 11184,'DASA' from dual;
insert into LAUNCH_AGENCY
select 11185,'DASA' from dual union all
select 11186,'DASA' from dual union all
select 11187,'DASA' from dual union all
select 11188,'DASA' from dual union all
select 11189,'DASA' from dual union all
select 1119,'GAU4' from dual union all
select 11190,'USN' from dual union all
select 11191,'SAND' from dual union all
select 11192,'NASA' from dual union all
select 11193,'SAND' from dual union all
select 11194,'SAND' from dual union all
select 11195,'1STRAD' from dual union all
select 11196,'1STRAD' from dual union all
select 11197,'ISAS' from dual union all
select 11198,'NASA' from dual union all
select 11199,'USAF' from dual union all
select 112,'WER836' from dual union all
select 1120,'GAU4' from dual union all
select 11200,'SAND' from dual union all
select 11201,'NASA' from dual union all
select 11202,'NASA' from dual union all
select 11203,'1STRAD' from dual union all
select 11204,'SAC' from dual union all
select 11205,'CNIE' from dual union all
select 11206,'CNES' from dual union all
select 11207,'1STRAD' from dual union all
select 11208,'CNES' from dual union all
select 11209,'CNES' from dual union all
select 1121,'GAU4' from dual union all
select 11210,'CNES' from dual union all
select 11211,'CNES' from dual union all
select 11212,'1STRAD' from dual union all
select 11213,'CNES' from dual union all
select 11214,'ISAS' from dual union all
select 11215,'NASA' from dual union all
select 11216,'NASA' from dual union all
select 11217,'NASA' from dual union all
select 11218,'NASA' from dual union all
select 11219,'CNIE' from dual union all
select 1122,'GAU4' from dual union all
select 11220,'1STRAD' from dual union all
select 11221,'1STRAD' from dual union all
select 11222,'SAND' from dual union all
select 11223,'NASA' from dual union all
select 11224,'CNES' from dual union all
select 11225,'CNES' from dual union all
select 11226,'NASA' from dual union all
select 11227,'NASA' from dual union all
select 11228,'NASA' from dual union all
select 11229,'CNIE' from dual union all
select 1123,'GAU4' from dual union all
select 11230,'NASA' from dual union all
select 11231,'RAE' from dual union all
select 11232,'USAF' from dual union all
select 11233,'NASA' from dual union all
select 11234,'NASA' from dual union all
select 11235,'NASA' from dual union all
select 11236,'NASA' from dual union all
select 11237,'CNES' from dual union all
select 11238,'1STRAD' from dual union all
select 11239,'SAND' from dual union all
select 1124,'GAU4' from dual union all
select 11240,'NASA' from dual union all
select 11241,'USAF' from dual union all
select 11242,'USAF' from dual union all
select 11243,'NASA' from dual union all
select 11244,'ISAS' from dual union all
select 11245,'NASA' from dual union all
select 11246,'NASA' from dual union all
select 11247,'NASA' from dual union all
select 11248,'NASA' from dual union all
select 11249,'USAF' from dual union all
select 1125,'GAU4' from dual union all
select 11250,'WRE' from dual union all
select 11251,'NASA' from dual union all
select 11252,'NASA' from dual union all
select 11253,'ISAS' from dual union all
select 11254,'WRE' from dual union all
select 11255,'NASA' from dual union all
select 11256,'NASA' from dual union all
select 11257,'NASA' from dual union all
select 11258,'NRL' from dual union all
select 11259,'SAND' from dual union all
select 1126,'GAU4' from dual union all
select 11260,'USN' from dual union all
select 11261,'NASA' from dual union all
select 11262,'USAF' from dual union all
select 11263,'NASA' from dual union all
select 11264,'NASA' from dual union all
select 11265,'MVS' from dual union all
select 11266,'NASA' from dual union all
select 11267,'NASA' from dual union all
select 11268,'NASA' from dual union all
select 11269,'SAND' from dual union all
select 1127,'GAU4' from dual union all
select 11270,'84FA3' from dual union all
select 11271,'84FA3' from dual union all
select 11272,'USAF' from dual union all
select 11273,'84FA3' from dual union all
select 11274,'84FA3' from dual;
insert into LAUNCH_AGENCY
select 11275,'MVS' from dual union all
select 11276,'USAF' from dual union all
select 11277,'NRL' from dual union all
select 11278,'81FA1' from dual union all
select 11279,'NASA' from dual union all
select 1128,'GAU4' from dual union all
select 11280,'NASA' from dual union all
select 11281,'NASA' from dual union all
select 11282,'USAF' from dual union all
select 11283,'SAND' from dual union all
select 11284,'USAF' from dual union all
select 11285,'NASA' from dual union all
select 11286,'SAND' from dual union all
select 11287,'NRL' from dual union all
select 11288,'NRCC' from dual union all
select 11289,'81FA1' from dual union all
select 1129,'GAU4' from dual union all
select 11290,'USAF' from dual union all
select 11291,'CNES' from dual union all
select 11292,'MVS' from dual union all
select 11293,'SAND' from dual union all
select 11294,'SAND' from dual union all
select 11295,'USA' from dual union all
select 11296,'DASA' from dual union all
select 11297,'DASA' from dual union all
select 11298,'NASA' from dual union all
select 11299,'MVS' from dual union all
select 113,'WER836' from dual union all
select 1130,'GAU4' from dual union all
select 11300,'CNES' from dual union all
select 11301,'SAND' from dual union all
select 11302,'NASA' from dual union all
select 11303,'NASA' from dual union all
select 11304,'NASA' from dual union all
select 11305,'SAC' from dual union all
select 11306,'USN' from dual union all
select 11307,'CNES' from dual union all
select 11308,'SAC' from dual union all
select 11309,'NASA' from dual union all
select 1131,'GAU4' from dual union all
select 11310,'NASA' from dual union all
select 11311,'USAF' from dual union all
select 11312,'NASA' from dual union all
select 11313,'SAC' from dual union all
select 11314,'WRE' from dual union all
select 11315,'USAF' from dual union all
select 11316,'1STRAD' from dual union all
select 11317,'1STRAD' from dual union all
select 11318,'WRE' from dual union all
select 11319,'NOTS' from dual union all
select 1132,'GAU4' from dual union all
select 11320,'AFSC' from dual union all
select 11321,'MVS' from dual union all
select 11322,'MVS' from dual union all
select 11323,'RVSN' from dual union all
select 11324,'USN' from dual union all
select 11325,'USN' from dual union all
select 11326,'RAE' from dual union all
select 11326,'WRE' from dual union all
select 11327,'NASA' from dual union all
select 11328,'NASA' from dual union all
select 11329,'USN' from dual union all
select 1133,'NII88A' from dual union all
select 11330,'RAE' from dual union all
select 11330,'WRE' from dual union all
select 11331,'NASA' from dual union all
select 11332,'RVSN' from dual union all
select 11333,'USN' from dual union all
select 11334,'AFSC' from dual union all
select 11335,'1STRAD' from dual union all
select 11336,'RVSN' from dual union all
select 11337,'RVSN' from dual union all
select 11338,'USAF' from dual union all
select 11339,'MVS' from dual union all
select 1134,'GAU4' from dual union all
select 11340,'USN' from dual union all
select 11341,'MVS' from dual union all
select 11342,'RVSN' from dual union all
select 11343,'SAC' from dual union all
select 11344,'MVS' from dual union all
select 11345,'MVS' from dual union all
select 11346,'MVS' from dual union all
select 11347,'RVSN' from dual union all
select 11348,'CARDE' from dual union all
select 11349,'CARDE' from dual union all
select 1135,'NII88A' from dual union all
select 11350,'USN' from dual union all
select 11351,'USN' from dual union all
select 11352,'HARP' from dual union all
select 11353,'RVSN' from dual union all
select 11354,'HARP' from dual union all
select 11355,'HARP' from dual union all
select 11356,'HARP' from dual union all
select 11357,'HARP' from dual union all
select 11358,'HARP' from dual union all
select 11359,'HARP' from dual union all
select 1136,'ARDC' from dual union all
select 11360,'HARP' from dual union all
select 11361,'HARP' from dual union all
select 11362,'HARP' from dual;
insert into LAUNCH_AGENCY
select 11363,'HARP' from dual union all
select 11364,'USN' from dual union all
select 11365,'HARP' from dual union all
select 11366,'HARP' from dual union all
select 11367,'HARP' from dual union all
select 11368,'HARP' from dual union all
select 11369,'HARP' from dual union all
select 1137,'USN' from dual union all
select 11370,'HARP' from dual union all
select 11371,'HARP' from dual union all
select 11372,'HARP' from dual union all
select 11373,'HARP' from dual union all
select 11374,'HARP' from dual union all
select 11375,'NASA' from dual union all
select 11376,'HARP' from dual union all
select 11377,'HARP' from dual union all
select 11378,'HARP' from dual union all
select 11379,'HARP' from dual union all
select 1138,'NII88A' from dual union all
select 11380,'HARP' from dual union all
select 11381,'HARP' from dual union all
select 11382,'HARP' from dual union all
select 11383,'HARP' from dual union all
select 11384,'HARP' from dual union all
select 11385,'HARP' from dual union all
select 11386,'MVS' from dual union all
select 11387,'HARP' from dual union all
select 11388,'HARP' from dual union all
select 11389,'HARP' from dual union all
select 1139,'NII88A' from dual union all
select 11390,'HARP' from dual union all
select 11391,'HARP' from dual union all
select 11392,'HARP' from dual union all
select 11393,'HARP' from dual union all
select 11394,'HARP' from dual union all
select 11395,'HARP' from dual union all
select 11396,'HARP' from dual union all
select 11397,'RAE' from dual union all
select 11397,'WRE' from dual union all
select 11398,'HARP' from dual union all
select 11399,'HARP' from dual union all
select 114,'WER444' from dual union all
select 1140,'SCEL' from dual union all
select 1140,'US' from dual union all
select 11400,'HARP' from dual union all
select 11401,'HARP' from dual union all
select 11402,'HARP' from dual union all
select 11403,'HARP' from dual union all
select 11404,'HARP' from dual union all
select 11405,'HARP' from dual union all
select 11406,'HARP' from dual union all
select 11407,'HARP' from dual union all
select 11408,'NASA' from dual union all
select 11409,'HARP' from dual union all
select 1141,'NII88A' from dual union all
select 11410,'HARP' from dual union all
select 11411,'HARP' from dual union all
select 11412,'HARP' from dual union all
select 11413,'HARP' from dual union all
select 11414,'HARP' from dual union all
select 11415,'HARP' from dual union all
select 11416,'HARP' from dual union all
select 11417,'HARP' from dual union all
select 11418,'HARP' from dual union all
select 11419,'NASA' from dual union all
select 1142,'NII88A' from dual union all
select 11420,'HARP' from dual union all
select 11421,'HARP' from dual union all
select 11422,'HARP' from dual union all
select 11423,'HARP' from dual union all
select 11424,'HARP' from dual union all
select 11425,'HARP' from dual union all
select 11426,'NASA' from dual union all
select 11427,'NMC' from dual union all
select 11428,'USN' from dual union all
select 11429,'USN' from dual union all
select 1143,'ARDC' from dual union all
select 11430,'NASA' from dual union all
select 11431,'USN' from dual union all
select 11432,'USN' from dual union all
select 11433,'USN' from dual union all
select 11434,'USN' from dual union all
select 11435,'USN' from dual union all
select 11436,'USN' from dual union all
select 11437,'USN' from dual union all
select 11438,'USA' from dual union all
select 11439,'WRE' from dual union all
select 1144,'SCEL' from dual union all
select 1144,'US' from dual union all
select 11440,'USA' from dual union all
select 11441,'RVSN' from dual union all
select 11442,'LRS' from dual union all
select 11443,'LRS' from dual union all
select 11444,'LRS' from dual union all
select 11445,'USA' from dual union all
select 11446,'USN' from dual union all
select 11447,'USN' from dual union all
select 11448,'USN' from dual union all
select 11449,'USA' from dual union all
select 1145,'USA' from dual;
insert into LAUNCH_AGENCY
select 11450,'USA' from dual union all
select 11451,'USN' from dual union all
select 11452,'MVS' from dual union all
select 11453,'USN' from dual union all
select 11454,'USN' from dual union all
select 11455,'USN' from dual union all
select 11456,'USN' from dual union all
select 11457,'USN' from dual union all
select 11458,'NMC' from dual union all
select 11459,'USN' from dual union all
select 1146,'PARD' from dual union all
select 11460,'USN' from dual union all
select 11461,'USN' from dual union all
select 11462,'RVSN' from dual union all
select 11463,'CNES' from dual union all
select 11464,'RVSN' from dual union all
select 11465,'RVSN' from dual union all
select 11466,'RVSN' from dual union all
select 11467,'RVSN' from dual union all
select 11468,'RVSN' from dual union all
select 11469,'RVSN' from dual union all
select 1147,'ORDRSA' from dual union all
select 11470,'RVSN' from dual union all
select 11471,'RVSN' from dual union all
select 11472,'RVSN' from dual union all
select 11473,'MVS' from dual union all
select 11474,'USAF' from dual union all
select 11475,'USAF' from dual union all
select 11476,'CNES' from dual union all
select 11477,'1STRAD' from dual union all
select 11478,'AFSC' from dual union all
select 11479,'CNES' from dual union all
select 1148,'ORDRSA' from dual union all
select 11480,'CNES' from dual union all
select 11481,'CNES' from dual union all
select 11482,'NASA' from dual union all
select 11483,'MVS' from dual union all
select 11484,'CNES' from dual union all
select 11485,'CNES' from dual union all
select 11486,'NASA' from dual union all
select 11487,'NASA' from dual union all
select 11488,'NASA' from dual union all
select 11489,'MVS' from dual union all
select 1149,'ORDRSA' from dual union all
select 11490,'CNES' from dual union all
select 11491,'USAF' from dual union all
select 11492,'CNES' from dual union all
select 11493,'NASA' from dual union all
select 11494,'NASA' from dual union all
select 11495,'USAF' from dual union all
select 11496,'NASA' from dual union all
select 11497,'NASA' from dual union all
select 11498,'USAF' from dual union all
select 11499,'USAF' from dual union all
select 115,'WER444' from dual union all
select 1150,'ORDRSA' from dual union all
select 11500,'USAF' from dual union all
select 11501,'USAF' from dual union all
select 11502,'USAF' from dual union all
select 11503,'USAF' from dual union all
select 11504,'USAF' from dual union all
select 11505,'USAF' from dual union all
select 11506,'USAF' from dual union all
select 11507,'USAF' from dual union all
select 11508,'USAF' from dual union all
select 11509,'USAF' from dual union all
select 1151,'USAF' from dual union all
select 11510,'USAF' from dual union all
select 11511,'USAF' from dual union all
select 11512,'USAF' from dual union all
select 11513,'USAF' from dual union all
select 11514,'ISAS' from dual union all
select 11515,'ISAS' from dual union all
select 11516,'USAF' from dual union all
select 11517,'USAF' from dual union all
select 11518,'USAF' from dual union all
select 11519,'USAF' from dual union all
select 1152,'USA' from dual union all
select 11520,'USAF' from dual union all
select 11521,'USAF' from dual union all
select 11522,'USAF' from dual union all
select 11523,'USAF' from dual union all
select 11524,'USAF' from dual union all
select 11525,'NASA' from dual union all
select 11525,'UCAN' from dual union all
select 11526,'USAF' from dual union all
select 11527,'DASA' from dual union all
select 11528,'ISRO' from dual union all
select 11529,'USAF' from dual union all
select 1153,'PARD' from dual union all
select 11530,'NASA' from dual union all
select 11531,'NASA' from dual union all
select 11532,'ERDA' from dual union all
select 11533,'ISRO' from dual union all
select 11534,'ISRO' from dual union all
select 11535,'USAF' from dual union all
select 11536,'DASA' from dual union all
select 11537,'DASA' from dual union all
select 11538,'MNF' from dual union all
select 11539,'SUPA' from dual;
insert into LAUNCH_AGENCY
select 1154,'USA' from dual union all
select 11540,'USAF' from dual union all
select 11541,'SUPA' from dual union all
select 11542,'SUPA' from dual union all
select 11543,'SUPA' from dual union all
select 11544,'UKMO' from dual union all
select 11545,'UKMO' from dual union all
select 11546,'UKMO' from dual union all
select 11547,'ONR' from dual union all
select 11548,'ONR' from dual union all
select 11549,'ONR' from dual union all
select 1155,'USAF' from dual union all
select 11550,'ONR' from dual union all
select 11551,'USAF' from dual union all
select 11552,'ONR' from dual union all
select 11553,'ERDA' from dual union all
select 11554,'USAF' from dual union all
select 11555,'ISAS' from dual union all
select 11556,'USAF' from dual union all
select 11557,'USAF' from dual union all
select 11558,'USAF' from dual union all
select 11559,'USAF' from dual union all
select 1156,'USA' from dual union all
select 11560,'USAF' from dual union all
select 11561,'USAF' from dual union all
select 11562,'USAF' from dual union all
select 11563,'USAF' from dual union all
select 11564,'USAF' from dual union all
select 11565,'USAF' from dual union all
select 11566,'USAF' from dual union all
select 11567,'USAF' from dual union all
select 11568,'USAF' from dual union all
select 11569,'USAF' from dual union all
select 1157,'USAF' from dual union all
select 11570,'USAF' from dual union all
select 11571,'USAF' from dual union all
select 11572,'USAF' from dual union all
select 11573,'USAF' from dual union all
select 11574,'NASA' from dual union all
select 11575,'USAF' from dual union all
select 11576,'USAF' from dual union all
select 11577,'USAF' from dual union all
select 11578,'USAF' from dual union all
select 11579,'NASA' from dual union all
select 1158,'USA' from dual union all
select 11580,'ISRO' from dual union all
select 11581,'ISRO' from dual union all
select 11582,'USAF' from dual union all
select 11583,'ISRO' from dual union all
select 11584,'NASA' from dual union all
select 11585,'NASA' from dual union all
select 11586,'USAF' from dual union all
select 11587,'USAF' from dual union all
select 11588,'USAF' from dual union all
select 11589,'USAF' from dual union all
select 1159,'USA' from dual union all
select 11590,'USAF' from dual union all
select 11591,'UMI' from dual union all
select 11592,'USAF' from dual union all
select 11593,'UMI' from dual union all
select 11594,'AWS' from dual union all
select 11595,'AWS' from dual union all
select 11596,'AWS' from dual union all
select 11597,'NASA' from dual union all
select 11598,'NASA' from dual union all
select 11599,'ISRO' from dual union all
select 116,'WER500' from dual union all
select 1160,'ORDRSA' from dual union all
select 11600,'AWS' from dual union all
select 11601,'MSFC' from dual union all
select 11602,'USAF' from dual union all
select 11603,'USAF' from dual union all
select 11604,'MSFC' from dual union all
select 11605,'MSFC' from dual union all
select 11606,'MSFC' from dual union all
select 11607,'USAF' from dual union all
select 11608,'USAF' from dual union all
select 11609,'USAF' from dual union all
select 1161,'ORDRSA' from dual union all
select 11610,'UMI' from dual union all
select 11611,'UMI' from dual union all
select 11612,'USAF' from dual union all
select 11613,'UMI' from dual union all
select 11614,'USAF' from dual union all
select 11615,'DASA' from dual union all
select 11616,'USAF' from dual union all
select 11617,'USAF' from dual union all
select 11618,'USAF' from dual union all
select 11619,'USAF' from dual union all
select 1162,'USA' from dual union all
select 11620,'USAF' from dual union all
select 11621,'USAF' from dual union all
select 11622,'USAF' from dual union all
select 11623,'USAF' from dual union all
select 11624,'MSFC' from dual union all
select 11625,'USAF' from dual union all
select 11626,'USAF' from dual union all
select 11627,'USAF' from dual union all
select 11628,'DASA' from dual union all
select 11629,'USAF' from dual;
insert into LAUNCH_AGENCY
select 1163,'ORDRSA' from dual union all
select 11630,'USAF' from dual union all
select 11631,'USAF' from dual union all
select 11632,'USAF' from dual union all
select 11633,'MSFC' from dual union all
select 11634,'USAF' from dual union all
select 11635,'USAF' from dual union all
select 11636,'USAF' from dual union all
select 11637,'USAF' from dual union all
select 11638,'USAF' from dual union all
select 11639,'ERDL' from dual union all
select 1164,'USAF' from dual union all
select 11640,'ERDL' from dual union all
select 11641,'AN' from dual union all
select 11642,'AN' from dual union all
select 11643,'AN' from dual union all
select 11644,'AN' from dual union all
select 11645,'MSFC' from dual union all
select 11646,'MSFC' from dual union all
select 11647,'MSFC' from dual union all
select 11648,'NASA' from dual union all
select 11648,'UIO' from dual union all
select 11649,'AN' from dual union all
select 1165,'PARD' from dual union all
select 11650,'AN' from dual union all
select 11651,'AN' from dual union all
select 11652,'AN' from dual union all
select 11653,'AN' from dual union all
select 11654,'AN' from dual union all
select 11655,'AN' from dual union all
select 11656,'AN' from dual union all
select 11657,'AN' from dual union all
select 11658,'AN' from dual union all
select 11659,'SUPA' from dual union all
select 1166,'LRBA' from dual union all
select 11660,'MSFC' from dual union all
select 11661,'MSFC' from dual union all
select 11662,'MSFC' from dual union all
select 11663,'AN' from dual union all
select 11664,'AN' from dual union all
select 11665,'AN' from dual union all
select 11666,'AN' from dual union all
select 11667,'AN' from dual union all
select 11668,'AN' from dual union all
select 11669,'AN' from dual union all
select 1167,'USAF' from dual union all
select 11670,'AN' from dual union all
select 11671,'AN' from dual union all
select 11672,'AN' from dual union all
select 11673,'AN' from dual union all
select 11674,'AN' from dual union all
select 11675,'AN' from dual union all
select 11676,'AN' from dual union all
select 11677,'AN' from dual union all
select 11678,'AN' from dual union all
select 11679,'AN' from dual union all
select 1168,'USA' from dual union all
select 11680,'AN' from dual union all
select 11681,'AN' from dual union all
select 11682,'AN' from dual union all
select 11683,'AN' from dual union all
select 11684,'AN' from dual union all
select 11685,'AN' from dual union all
select 11686,'AN' from dual union all
select 11687,'AN' from dual union all
select 11688,'AN' from dual union all
select 11689,'AN' from dual union all
select 1169,'USAF' from dual union all
select 11690,'AN' from dual union all
select 11691,'AN' from dual union all
select 11692,'AN' from dual union all
select 11693,'AN' from dual union all
select 11694,'AN' from dual union all
select 11695,'AN' from dual union all
select 11696,'AN' from dual union all
select 11697,'AN' from dual union all
select 11698,'AN' from dual union all
select 11699,'AN' from dual union all
select 117,'WER500' from dual union all
select 1170,'USA' from dual union all
select 11700,'AN' from dual union all
select 11701,'AN' from dual union all
select 11702,'AN' from dual union all
select 11703,'AN' from dual union all
select 11704,'AN' from dual union all
select 11705,'AN' from dual union all
select 11706,'AN' from dual union all
select 11707,'AN' from dual union all
select 11708,'AN' from dual union all
select 11709,'AN' from dual union all
select 1171,'USA' from dual union all
select 11710,'AN' from dual union all
select 11711,'AN' from dual union all
select 11712,'AN' from dual union all
select 11713,'AN' from dual union all
select 11714,'AN' from dual union all
select 11715,'AN' from dual union all
select 11716,'AN' from dual union all
select 11717,'AN' from dual union all
select 11718,'AN' from dual;
insert into LAUNCH_AGENCY
select 11719,'AN' from dual union all
select 1172,'USAF' from dual union all
select 11720,'AN' from dual union all
select 11721,'AN' from dual union all
select 11722,'AN' from dual union all
select 11723,'AN' from dual union all
select 11724,'AN' from dual union all
select 11725,'AN' from dual union all
select 11726,'AN' from dual union all
select 11727,'AN' from dual union all
select 11728,'AN' from dual union all
select 11729,'AN' from dual union all
select 1173,'USA' from dual union all
select 11730,'AN' from dual union all
select 11731,'AN' from dual union all
select 11732,'AN' from dual union all
select 11733,'AN' from dual union all
select 11734,'AN' from dual union all
select 11735,'AN' from dual union all
select 11736,'AN' from dual union all
select 11737,'AN' from dual union all
select 11738,'AN' from dual union all
select 11739,'AN' from dual union all
select 1174,'ORDRSA' from dual union all
select 11740,'AN' from dual union all
select 11741,'MSFC' from dual union all
select 11742,'MSFC' from dual union all
select 11743,'AN' from dual union all
select 11744,'AN' from dual union all
select 11745,'AN' from dual union all
select 11746,'AN' from dual union all
select 11747,'AN' from dual union all
select 11748,'AN' from dual union all
select 11749,'AN' from dual union all
select 1175,'ORDRSA' from dual union all
select 11750,'AN' from dual union all
select 11751,'RVSN' from dual union all
select 11752,'USAF' from dual union all
select 11753,'S4300' from dual union all
select 11754,'NASA' from dual union all
select 11755,'AFSC' from dual union all
select 11756,'RVSN' from dual union all
select 11757,'NASA' from dual union all
select 11758,'USAF' from dual union all
select 11759,'NASA' from dual union all
select 1176,'USA' from dual union all
select 11760,'USAF' from dual union all
select 11761,'RVSN' from dual union all
select 11762,'RVSN' from dual union all
select 11763,'USAF' from dual union all
select 11764,'RVSN' from dual union all
select 11765,'RVSN' from dual union all
select 11766,'USAF' from dual union all
select 11767,'USAF' from dual union all
select 11768,'RVSN' from dual union all
select 11769,'AFSC' from dual union all
select 1177,'LRBA' from dual union all
select 11770,'RVSN' from dual union all
select 11771,'S4300' from dual union all
select 11772,'RVSN' from dual union all
select 11773,'USAF' from dual union all
select 11774,'NASA' from dual union all
select 11775,'RVSN' from dual union all
select 11776,'USAF' from dual union all
select 11777,'AFSC' from dual union all
select 11778,'NASA' from dual union all
select 11779,'RVSN' from dual union all
select 1178,'ORDRSA' from dual union all
select 11780,'RVSN' from dual union all
select 11781,'AFSC' from dual union all
select 11782,'NASA' from dual union all
select 11783,'USAF' from dual union all
select 11784,'USAF' from dual union all
select 11785,'RVSN' from dual union all
select 11786,'RVSN' from dual union all
select 11787,'USAF' from dual union all
select 11788,'S4300' from dual union all
select 11789,'NASA' from dual union all
select 1179,'ORDRSA' from dual union all
select 11790,'RVSN' from dual union all
select 11791,'AFSC' from dual union all
select 11792,'NASA' from dual union all
select 11793,'NASA' from dual union all
select 11794,'RVSN' from dual union all
select 11795,'USAF' from dual union all
select 11796,'RVSN' from dual union all
select 11797,'USAF' from dual union all
select 11798,'USAF' from dual union all
select 11799,'RVSN' from dual union all
select 118,'WER444' from dual union all
select 1180,'USA' from dual union all
select 11800,'AFSC' from dual union all
select 11801,'NASA' from dual union all
select 11802,'RVSN' from dual union all
select 11803,'RVSN' from dual union all
select 11804,'RVSN' from dual union all
select 11805,'USAF' from dual union all
select 11806,'RVSN' from dual union all
select 11807,'USAF' from dual union all
select 11808,'USAF' from dual;
insert into LAUNCH_AGENCY
select 11809,'RVSN' from dual union all
select 1181,'USA' from dual union all
select 11810,'NASA' from dual union all
select 11811,'RVSN' from dual union all
select 11812,'AFSC' from dual union all
select 11813,'NASA' from dual union all
select 11814,'NASA' from dual union all
select 11815,'USAF' from dual union all
select 11816,'RVSN' from dual union all
select 11817,'USAF' from dual union all
select 11818,'NASA' from dual union all
select 11819,'RVSN' from dual union all
select 1182,'ORDRSA' from dual union all
select 11820,'RVSN' from dual union all
select 11821,'RVSN' from dual union all
select 11822,'S4300' from dual union all
select 11823,'RVSN' from dual union all
select 11824,'USAF' from dual union all
select 11825,'RVSN' from dual union all
select 11826,'AFSC' from dual union all
select 11827,'RVSN' from dual union all
select 11828,'AFSC' from dual union all
select 11829,'USAF' from dual union all
select 1183,'ORDRSA' from dual union all
select 11830,'RVSN' from dual union all
select 11831,'USAF' from dual union all
select 11832,'USAF' from dual union all
select 11833,'RVSN' from dual union all
select 11834,'RVSN' from dual union all
select 11835,'RVSN' from dual union all
select 11836,'USAF' from dual union all
select 11837,'RVSN' from dual union all
select 11838,'RVSN' from dual union all
select 11839,'NASA' from dual union all
select 1184,'PARD' from dual union all
select 11840,'AFSC' from dual union all
select 11841,'RVSN' from dual union all
select 11842,'RVSN' from dual union all
select 11843,'NASA' from dual union all
select 11844,'RVSN' from dual union all
select 11845,'RVSN' from dual union all
select 11846,'CNES' from dual union all
select 11847,'RVSN' from dual union all
select 11848,'USAF' from dual union all
select 11849,'RVSN' from dual union all
select 1185,'PARD' from dual union all
select 11850,'NASA' from dual union all
select 11851,'AFSC' from dual union all
select 11852,'USAF' from dual union all
select 11853,'RVSN' from dual union all
select 11854,'NASA' from dual union all
select 11855,'NASA' from dual union all
select 11856,'RVSN' from dual union all
select 11857,'RVSN' from dual union all
select 11858,'AFSC' from dual union all
select 11859,'AFSC' from dual union all
select 1186,'PARD' from dual union all
select 11860,'USAF' from dual union all
select 11861,'RVSN' from dual union all
select 11862,'RVSN' from dual union all
select 11863,'PIHM' from dual union all
select 11864,'PIHM' from dual union all
select 11865,'PIHM' from dual union all
select 11866,'PIHM' from dual union all
select 11867,'ARPA' from dual union all
select 11868,'NASA' from dual union all
select 11869,'NMC' from dual union all
select 1187,'USA' from dual union all
select 11870,'RVSN' from dual union all
select 11871,'NASA' from dual union all
select 11871,'USAF' from dual union all
select 11872,'NASA' from dual union all
select 11873,'USAF' from dual union all
select 11874,'NASA' from dual union all
select 11875,'RVSN' from dual union all
select 11876,'USAF' from dual union all
select 11877,'USAF' from dual union all
select 11878,'NASA' from dual union all
select 11878,'USAF' from dual union all
select 11879,'ISAS' from dual union all
select 1188,'USAF' from dual union all
select 11880,'ISAS' from dual union all
select 11881,'NASA' from dual union all
select 11882,'RAE' from dual union all
select 11883,'DASA' from dual union all
select 11884,'NASA' from dual union all
select 11884,'USAF' from dual union all
select 11885,'DASA' from dual union all
select 11886,'USAF' from dual union all
select 11887,'NASA' from dual union all
select 11887,'USAF' from dual union all
select 11888,'NASA' from dual union all
select 11888,'USAF' from dual union all
select 11889,'RAE' from dual union all
select 11889,'WRE' from dual union all
select 1189,'USA' from dual union all
select 11890,'RVSN' from dual union all
select 11891,'RVSN' from dual union all
select 11892,'USN' from dual union all
select 11893,'NASA' from dual;
insert into LAUNCH_AGENCY
select 11893,'USAF' from dual union all
select 11894,'NASA' from dual union all
select 11895,'NASA' from dual union all
select 11896,'USAF' from dual union all
select 11897,'NASA' from dual union all
select 11897,'USAF' from dual union all
select 11898,'RVSN' from dual union all
select 11899,'NASA' from dual union all
select 11899,'USAF' from dual union all
select 119,'WER444' from dual union all
select 1190,'USA' from dual union all
select 11900,'RAE' from dual union all
select 11901,'ISAS' from dual union all
select 11902,'ISAS' from dual union all
select 11903,'ARPA' from dual union all
select 11903,'US' from dual union all
select 11904,'RVSN' from dual union all
select 11905,'NRCC' from dual union all
select 11906,'NASA' from dual union all
select 11906,'USAF' from dual union all
select 11907,'NASA' from dual union all
select 11908,'RVSN' from dual union all
select 11909,'NASA' from dual union all
select 1191,'ORDRSA' from dual union all
select 11910,'NASA' from dual union all
select 11910,'USAF' from dual union all
select 11911,'NASA' from dual union all
select 11912,'RVSN' from dual union all
select 11913,'ISAS' from dual union all
select 11914,'USA' from dual union all
select 11915,'NASA' from dual union all
select 11915,'USAF' from dual union all
select 11916,'NASA' from dual union all
select 11916,'USAF' from dual union all
select 11917,'NASA' from dual union all
select 11918,'NASA' from dual union all
select 11919,'NASA' from dual union all
select 11919,'USAF' from dual union all
select 1192,'ORDRSA' from dual union all
select 11920,'NASA' from dual union all
select 11921,'NASA' from dual union all
select 11921,'USAF' from dual union all
select 11922,'44FA2' from dual union all
select 11923,'LARCN' from dual union all
select 11924,'NMC' from dual union all
select 11925,'USA' from dual union all
select 11926,'NASA' from dual union all
select 11927,'USA' from dual union all
select 11928,'RVSN' from dual union all
select 11929,'NASA' from dual union all
select 11929,'USAF' from dual union all
select 1193,'USA' from dual union all
select 11930,'NASA' from dual union all
select 11931,'RVSN' from dual union all
select 11932,'NASA' from dual union all
select 11932,'USAF' from dual union all
select 11933,'ISAS' from dual union all
select 11934,'NASA' from dual union all
select 11934,'USAF' from dual union all
select 11935,'RVSN' from dual union all
select 11936,'NASA' from dual union all
select 11937,'USA' from dual union all
select 11938,'USA' from dual union all
select 11939,'STA' from dual union all
select 1194,'USA' from dual union all
select 11940,'STA' from dual union all
select 11941,'NASA' from dual union all
select 11942,'NASA' from dual union all
select 11943,'USAF' from dual union all
select 11944,'SAND' from dual union all
select 11945,'NASA' from dual union all
select 11946,'ARPA' from dual union all
select 11946,'US' from dual union all
select 11947,'USA' from dual union all
select 11948,'USA' from dual union all
select 11949,'RVSN' from dual union all
select 1195,'USA' from dual union all
select 11950,'NASA' from dual union all
select 11951,'NASA' from dual union all
select 11952,'RVSN' from dual union all
select 11953,'RVSN' from dual union all
select 11954,'USA' from dual union all
select 11955,'USA' from dual union all
select 11956,'SAND' from dual union all
select 11957,'HARP' from dual union all
select 11958,'HARP' from dual union all
select 11959,'ARPA' from dual union all
select 1196,'USA' from dual union all
select 11960,'HARP' from dual union all
select 11961,'HARP' from dual union all
select 11962,'USAF' from dual union all
select 11963,'IIAE' from dual union all
select 11964,'IIAE' from dual union all
select 11965,'IIAE' from dual union all
select 11966,'IIAE' from dual union all
select 11967,'USAF' from dual union all
select 11968,'USAF' from dual union all
select 11969,'USAF' from dual union all
select 1197,'ORDRSA' from dual union all
select 11970,'USN' from dual;
insert into LAUNCH_AGENCY
select 11971,'ARPA' from dual union all
select 11972,'USAF' from dual union all
select 11973,'USA' from dual union all
select 11974,'NASA' from dual union all
select 11975,'USA' from dual union all
select 11976,'USA' from dual union all
select 11977,'USA' from dual union all
select 11978,'USA' from dual union all
select 11979,'USA' from dual union all
select 1198,'ORDRSA' from dual union all
select 11980,'USA' from dual union all
select 11981,'USA' from dual union all
select 11982,'USA' from dual union all
select 11983,'USA' from dual union all
select 11984,'WRE' from dual union all
select 11985,'NASA' from dual union all
select 11985,'USAF' from dual union all
select 11986,'WRE' from dual union all
select 11987,'USA' from dual union all
select 11988,'NASA' from dual union all
select 11989,'41FA4' from dual union all
select 1199,'LRBA' from dual union all
select 11990,'USA' from dual union all
select 11991,'USA' from dual union all
select 11992,'USA' from dual union all
select 11993,'AFCRL' from dual union all
select 11994,'AFCRL' from dual union all
select 11995,'AFCRL' from dual union all
select 11996,'NASA' from dual union all
select 11997,'SAND' from dual union all
select 11998,'SAND' from dual union all
select 11999,'USAF' from dual union all
select 12,'WEHR' from dual union all
select 120,'WER500' from dual union all
select 1200,'USA' from dual union all
select 12000,'NASA' from dual union all
select 12001,'WRE' from dual union all
select 12002,'USA' from dual union all
select 12003,'IIAE' from dual union all
select 12004,'IIAE' from dual union all
select 12005,'USAF' from dual union all
select 12006,'NASA' from dual union all
select 12006,'USAF' from dual union all
select 12007,'NASA' from dual union all
select 12007,'USAF' from dual union all
select 12008,'USAF' from dual union all
select 12009,'NASA' from dual union all
select 12009,'USAF' from dual union all
select 1201,'SUI' from dual union all
select 12010,'NASA' from dual union all
select 12010,'USAF' from dual union all
select 12011,'MVS' from dual union all
select 12012,'ARPA' from dual union all
select 12012,'US' from dual union all
select 12013,'NASA' from dual union all
select 12013,'USAF' from dual union all
select 12014,'NASA' from dual union all
select 12014,'USAF' from dual union all
select 12015,'USA' from dual union all
select 12016,'USAF' from dual union all
select 12017,'NASA' from dual union all
select 12018,'NASA' from dual union all
select 12019,'USAF' from dual union all
select 1202,'SUI' from dual union all
select 12020,'USAF' from dual union all
select 12021,'44SMW' from dual union all
select 12022,'NASA' from dual union all
select 12022,'NTNF' from dual union all
select 12023,'NASA' from dual union all
select 12024,'NASA' from dual union all
select 12024,'NTNF' from dual union all
select 12025,'RAE' from dual union all
select 12025,'WRE' from dual union all
select 12026,'NASA' from dual union all
select 12026,'NTNF' from dual union all
select 12027,'NASA' from dual union all
select 12027,'USAF' from dual union all
select 12028,'USA' from dual union all
select 12029,'PIHM' from dual union all
select 1203,'ARDC' from dual union all
select 12030,'ESRO' from dual union all
select 12031,'NASA' from dual union all
select 12032,'USN' from dual union all
select 12033,'NASA' from dual union all
select 12033,'USAF' from dual union all
select 12034,'RVSN' from dual union all
select 12035,'NASA' from dual union all
select 12035,'USAF' from dual union all
select 12036,'USA' from dual union all
select 12037,'NASA' from dual union all
select 12038,'NASA' from dual union all
select 12039,'ARPA' from dual union all
select 12039,'US' from dual union all
select 1204,'USA' from dual union all
select 12040,'NRL' from dual union all
select 12041,'NASA' from dual union all
select 12041,'USAF' from dual union all
select 12042,'WRE' from dual union all
select 12043,'RAE' from dual union all
select 12044,'NASA' from dual;
insert into LAUNCH_AGENCY
select 12044,'USAF' from dual union all
select 12045,'NASA' from dual union all
select 12046,'NASA' from dual union all
select 12047,'NASA' from dual union all
select 12047,'USAF' from dual union all
select 12048,'NASA' from dual union all
select 12049,'NASA' from dual union all
select 1205,'USAF' from dual union all
select 12050,'RVSN' from dual union all
select 12051,'NRCC' from dual union all
select 12052,'USA' from dual union all
select 12053,'USA' from dual union all
select 12054,'NASA' from dual union all
select 12055,'NASA' from dual union all
select 12055,'USAF' from dual union all
select 12056,'NASA' from dual union all
select 12056,'USAF' from dual union all
select 12057,'NASA' from dual union all
select 12057,'USAF' from dual union all
select 12058,'GMS' from dual union all
select 12059,'PIHM' from dual union all
select 1206,'PARD' from dual union all
select 12060,'NASA' from dual union all
select 12060,'USAF' from dual union all
select 12061,'STA' from dual union all
select 12062,'RVSN' from dual union all
select 12063,'NASA' from dual union all
select 12063,'USAF' from dual union all
select 12064,'RVSN' from dual union all
select 12065,'NASA' from dual union all
select 12066,'ARPA' from dual union all
select 12066,'US' from dual union all
select 12067,'USA' from dual union all
select 12068,'USA' from dual union all
select 12069,'NASA' from dual union all
select 1207,'PARD' from dual union all
select 12070,'NASA' from dual union all
select 12070,'USAF' from dual union all
select 12071,'USN' from dual union all
select 12072,'RVSN' from dual union all
select 12073,'DASA' from dual union all
select 12074,'NASA' from dual union all
select 12074,'USAF' from dual union all
select 12075,'AFSC' from dual union all
select 12076,'RVSN' from dual union all
select 12077,'RVSN' from dual union all
select 12078,'NASA' from dual union all
select 12079,'RVSN' from dual union all
select 1208,'PARD' from dual union all
select 12080,'AFSC' from dual union all
select 12081,'AFSC' from dual union all
select 12082,'RVSN' from dual union all
select 12083,'NASA' from dual union all
select 12084,'USAF' from dual union all
select 12085,'USAF' from dual union all
select 12086,'RVSN' from dual union all
select 12087,'USA' from dual union all
select 12088,'USA' from dual union all
select 12089,'USAF' from dual union all
select 1209,'USAF' from dual union all
select 12090,'USA' from dual union all
select 12091,'USA' from dual union all
select 12092,'USAF' from dual union all
select 12093,'NASA' from dual union all
select 12094,'SAND' from dual union all
select 12095,'USAF' from dual union all
select 12096,'NASA' from dual union all
select 12096,'USAF' from dual union all
select 12097,'CNES' from dual union all
select 12098,'NASA' from dual union all
select 12098,'USAF' from dual union all
select 12099,'USA' from dual union all
select 121,'WEHR' from dual union all
select 1210,'LRBA' from dual union all
select 12100,'USA' from dual union all
select 12101,'USA' from dual union all
select 12102,'SIMED' from dual union all
select 12103,'SIMED' from dual union all
select 12104,'STA' from dual union all
select 12105,'STA' from dual union all
select 12106,'NASA' from dual union all
select 12106,'USAF' from dual union all
select 12107,'STA' from dual union all
select 12108,'NASA' from dual union all
select 12108,'USAF' from dual union all
select 12109,'ONERA' from dual union all
select 1211,'ORDRSA' from dual union all
select 12110,'NASA' from dual union all
select 12110,'USAF' from dual union all
select 12111,'NASA' from dual union all
select 12111,'USAF' from dual union all
select 12112,'NASA' from dual union all
select 12112,'USAF' from dual union all
select 12113,'NASA' from dual union all
select 12113,'USAF' from dual union all
select 12114,'NASA' from dual union all
select 12114,'USAF' from dual union all
select 12115,'HARP' from dual union all
select 12116,'HARP' from dual union all
select 12117,'HARP' from dual;
insert into LAUNCH_AGENCY
select 12118,'DMA' from dual union all
select 12119,'NASA' from dual union all
select 12119,'USAF' from dual union all
select 1212,'ORDRSA' from dual union all
select 12120,'DMA' from dual union all
select 12121,'STA' from dual union all
select 12122,'STA' from dual union all
select 12123,'ONERA' from dual union all
select 12124,'ONERA' from dual union all
select 12125,'ONERA' from dual union all
select 12126,'BRL' from dual union all
select 12127,'BRL' from dual union all
select 12128,'BRL' from dual union all
select 12129,'BRL' from dual union all
select 1213,'USAF' from dual union all
select 12130,'BRL' from dual union all
select 12131,'BRL' from dual union all
select 12132,'BRL' from dual union all
select 12133,'BRL' from dual union all
select 12134,'BRL' from dual union all
select 12135,'BRL' from dual union all
select 12136,'BRL' from dual union all
select 12137,'BRL' from dual union all
select 12138,'BRL' from dual union all
select 12139,'BRL' from dual union all
select 1214,'PARD' from dual union all
select 12140,'BRL' from dual union all
select 12141,'BRL' from dual union all
select 12142,'BRL' from dual union all
select 12143,'BRL' from dual union all
select 12144,'BRL' from dual union all
select 12145,'BRL' from dual union all
select 12146,'BRL' from dual union all
select 12147,'BRL' from dual union all
select 12148,'BRL' from dual union all
select 12149,'BRL' from dual union all
select 1215,'USA' from dual union all
select 12150,'BRL' from dual union all
select 12151,'BRL' from dual union all
select 12152,'BRL' from dual union all
select 12153,'BRL' from dual union all
select 12154,'BRL' from dual union all
select 12155,'BRL' from dual union all
select 12156,'BRL' from dual union all
select 12157,'BRL' from dual union all
select 12158,'BRL' from dual union all
select 12159,'BRL' from dual union all
select 1216,'USA' from dual union all
select 12160,'LRL' from dual union all
select 12160,'USAF' from dual union all
select 12161,'LRL' from dual union all
select 12161,'USAF' from dual union all
select 12162,'BWEHR' from dual union all
select 12163,'SAND' from dual union all
select 12164,'AN' from dual union all
select 12165,'USA' from dual union all
select 12166,'USA' from dual union all
select 12167,'RVSN' from dual union all
select 12168,'RVSN' from dual union all
select 12169,'RVSN' from dual union all
select 1217,'USAF' from dual union all
select 12170,'SAND' from dual union all
select 12171,'NASA' from dual union all
select 12172,'USAF' from dual union all
select 12173,'USAF' from dual union all
select 12174,'RVSN' from dual union all
select 12175,'USAF' from dual union all
select 12176,'CNES' from dual union all
select 12177,'SAC' from dual union all
select 12178,'USAF' from dual union all
select 12179,'NASA' from dual union all
select 1218,'USA' from dual union all
select 12180,'RVSN' from dual union all
select 12181,'USAF' from dual union all
select 12182,'USA' from dual union all
select 12183,'USA' from dual union all
select 12184,'USA' from dual union all
select 12185,'USA' from dual union all
select 12186,'NASA' from dual union all
select 12187,'USAF' from dual union all
select 12188,'USAF' from dual union all
select 12189,'USAF' from dual union all
select 1219,'ORDRSA' from dual union all
select 12190,'USAF' from dual union all
select 12191,'RVSN' from dual union all
select 12192,'1STRAD' from dual union all
select 12193,'USN' from dual union all
select 12194,'NASA' from dual union all
select 12194,'UIO' from dual union all
select 12195,'SAND' from dual union all
select 12196,'SAND' from dual union all
select 12197,'SAC' from dual union all
select 12198,'SAND' from dual union all
select 12199,'USAF' from dual union all
select 122,'WER500' from dual union all
select 1220,'USAF' from dual union all
select 12200,'RAE' from dual union all
select 12200,'WRE' from dual union all
select 12201,'USAF' from dual union all
select 12202,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 12203,'RVSN' from dual union all
select 12204,'CNES' from dual union all
select 12205,'SAND' from dual union all
select 12206,'USN' from dual union all
select 12207,'SAC' from dual union all
select 12208,'USAF' from dual union all
select 12209,'MVS' from dual union all
select 1221,'USA' from dual union all
select 12210,'NASA' from dual union all
select 12210,'UIO' from dual union all
select 12211,'SAND' from dual union all
select 12212,'RVSN' from dual union all
select 12213,'RVSN' from dual union all
select 12214,'NASA' from dual union all
select 12215,'USAF' from dual union all
select 12216,'1STRAD' from dual union all
select 12217,'NASA' from dual union all
select 12218,'NASA' from dual union all
select 12219,'NASA' from dual union all
select 1222,'PARD' from dual union all
select 12220,'NASA' from dual union all
select 12221,'RVSN' from dual union all
select 12222,'NASA' from dual union all
select 12223,'NASA' from dual union all
select 12224,'RVSN' from dual union all
select 12225,'NASA' from dual union all
select 12226,'NASA' from dual union all
select 12227,'SAND' from dual union all
select 12228,'NASA' from dual union all
select 12229,'NASA' from dual union all
select 1223,'SCEL' from dual union all
select 1223,'US' from dual union all
select 12230,'SAC' from dual union all
select 12231,'NASA' from dual union all
select 12232,'USAF' from dual union all
select 12233,'USAF' from dual union all
select 12234,'MVS' from dual union all
select 12235,'RVSN' from dual union all
select 12236,'USN' from dual union all
select 12237,'USN' from dual union all
select 12238,'1STRAD' from dual union all
select 12239,'MVS' from dual union all
select 1224,'PARD' from dual union all
select 12240,'NASA' from dual union all
select 12241,'NASA' from dual union all
select 12242,'NASA' from dual union all
select 12243,'NRCC' from dual union all
select 12244,'NASA' from dual union all
select 12245,'NASA' from dual union all
select 12246,'ONERA' from dual union all
select 12247,'ISAS' from dual union all
select 12248,'NASA' from dual union all
select 12249,'NASA' from dual union all
select 1225,'USA' from dual union all
select 12250,'NASA' from dual union all
select 12251,'NASA' from dual union all
select 12252,'NRCC' from dual union all
select 12253,'NASA' from dual union all
select 12254,'NASA' from dual union all
select 12255,'NASA' from dual union all
select 12256,'USAF' from dual union all
select 12257,'RVSN' from dual union all
select 12258,'USAF' from dual union all
select 12259,'ELDO' from dual union all
select 1226,'USA' from dual union all
select 12260,'RVSN' from dual union all
select 12261,'USAF' from dual union all
select 12262,'USAF' from dual union all
select 12263,'NASA' from dual union all
select 12264,'NRCC' from dual union all
select 12265,'NASA' from dual union all
select 12266,'NASA' from dual union all
select 12267,'1STRAD' from dual union all
select 12268,'RVSN' from dual union all
select 12269,'RAE' from dual union all
select 12269,'WRE' from dual union all
select 1227,'PARD' from dual union all
select 12270,'SAC' from dual union all
select 12271,'NRCC' from dual union all
select 12272,'ISAS' from dual union all
select 12273,'USAF' from dual union all
select 12274,'MVS' from dual union all
select 12275,'AFSC' from dual union all
select 12276,'NASA' from dual union all
select 12277,'NASA' from dual union all
select 12278,'NASA' from dual union all
select 12279,'RVSN' from dual union all
select 1228,'USA' from dual union all
select 12280,'ISAS' from dual union all
select 12281,'NASA' from dual union all
select 12282,'ISAS' from dual union all
select 12283,'USAF' from dual union all
select 12284,'USN' from dual union all
select 12285,'NASA' from dual union all
select 12286,'NASA' from dual union all
select 12287,'RVSN' from dual union all
select 12288,'NRL' from dual union all
select 12289,'WRE' from dual union all
select 1229,'ORDRSA' from dual union all
select 12290,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 12291,'USA' from dual union all
select 12292,'NASA' from dual union all
select 12293,'NASA' from dual union all
select 12294,'ESRO' from dual union all
select 12295,'NASA' from dual union all
select 12296,'NASA' from dual union all
select 12297,'USAF' from dual union all
select 12298,'NASA' from dual union all
select 12299,'NASA' from dual union all
select 123,'WER444' from dual union all
select 1230,'USA' from dual union all
select 12300,'AFSC' from dual union all
select 12301,'RVSN' from dual union all
select 12302,'NASA' from dual union all
select 12303,'RAE' from dual union all
select 12303,'WRE' from dual union all
select 12304,'NASA' from dual union all
select 12305,'NASA' from dual union all
select 12306,'1STRAD' from dual union all
select 12307,'RVSN' from dual union all
select 12308,'NASA' from dual union all
select 12309,'NASA' from dual union all
select 1231,'USAF' from dual union all
select 12310,'USAF' from dual union all
select 12311,'1STRAD' from dual union all
select 12312,'RVSN' from dual union all
select 12313,'1STRAD' from dual union all
select 12314,'NASA' from dual union all
select 12315,'NASA' from dual union all
select 12316,'NASA' from dual union all
select 12317,'NASA' from dual union all
select 12318,'NASA' from dual union all
select 12319,'NASA' from dual union all
select 1232,'USA' from dual union all
select 12320,'NASA' from dual union all
select 12321,'NASA' from dual union all
select 12322,'MVS' from dual union all
select 12323,'RVSN' from dual union all
select 12324,'SAC' from dual union all
select 12325,'RVSN' from dual union all
select 12326,'NRL' from dual union all
select 12327,'RAE' from dual union all
select 12328,'RVSN' from dual union all
select 12329,'NRCC' from dual union all
select 1233,'USA' from dual union all
select 12330,'NASA' from dual union all
select 12331,'NASA' from dual union all
select 12332,'MVS' from dual union all
select 12333,'RVSN' from dual union all
select 12334,'RVSN' from dual union all
select 12335,'NASA' from dual union all
select 12336,'MVS' from dual union all
select 12337,'RVSN' from dual union all
select 12338,'RVSN' from dual union all
select 12339,'RVSN' from dual union all
select 1234,'USA' from dual union all
select 12340,'NRL' from dual union all
select 12341,'NASA' from dual union all
select 12342,'ISRO' from dual union all
select 12343,'RAE' from dual union all
select 12343,'WRE' from dual union all
select 12344,'RVSN' from dual union all
select 12345,'RAE' from dual union all
select 12345,'WRE' from dual union all
select 12346,'RAE' from dual union all
select 12346,'WRE' from dual union all
select 12347,'RAE' from dual union all
select 12347,'WRE' from dual union all
select 12348,'RAE' from dual union all
select 12348,'WRE' from dual union all
select 12349,'NASA' from dual union all
select 1235,'ORDRSA' from dual union all
select 12350,'RAE' from dual union all
select 12350,'WRE' from dual union all
select 12351,'RAE' from dual union all
select 12351,'WRE' from dual union all
select 12352,'1STRAD' from dual union all
select 12353,'DASA' from dual union all
select 12354,'41FA4' from dual union all
select 12355,'RVSN' from dual union all
select 12356,'NASA' from dual union all
select 12357,'SAC' from dual union all
select 12358,'ISRO' from dual union all
select 12359,'NASA' from dual union all
select 1236,'ORDRSA' from dual union all
select 12360,'NASA' from dual union all
select 12361,'RVSN' from dual union all
select 12362,'AFSC' from dual union all
select 12363,'RVSN' from dual union all
select 12364,'1STRAD' from dual union all
select 12365,'WRE' from dual union all
select 12366,'RVSN' from dual union all
select 12367,'WRE' from dual union all
select 12368,'WRE' from dual union all
select 12369,'WRE' from dual union all
select 1237,'PARD' from dual union all
select 12370,'WRE' from dual union all
select 12371,'WRE' from dual union all
select 12372,'WRE' from dual union all
select 12373,'WRE' from dual;
insert into LAUNCH_AGENCY
select 12374,'RAE' from dual union all
select 12374,'WRE' from dual union all
select 12375,'MVS' from dual union all
select 12376,'CNES' from dual union all
select 12377,'NASA' from dual union all
select 12378,'RAE' from dual union all
select 12378,'WRE' from dual union all
select 12379,'RVSN' from dual union all
select 1238,'PARD' from dual union all
select 12380,'RVSN' from dual union all
select 12381,'ONERA' from dual union all
select 12382,'1STRAD' from dual union all
select 12383,'RVSN' from dual union all
select 12384,'RVSN' from dual union all
select 12385,'RAE' from dual union all
select 12385,'WRE' from dual union all
select 12386,'RVSN' from dual union all
select 12387,'RVSN' from dual union all
select 12388,'NASA' from dual union all
select 12389,'ONERA' from dual union all
select 1239,'USA' from dual union all
select 12390,'SAC' from dual union all
select 12391,'NASA' from dual union all
select 12392,'RVSN' from dual union all
select 12393,'NASA' from dual union all
select 12394,'RVSN' from dual union all
select 12395,'WRE' from dual union all
select 12396,'RVSN' from dual union all
select 12397,'RVSN' from dual union all
select 12398,'SAC' from dual union all
select 12399,'USAF' from dual union all
select 124,'WER444' from dual union all
select 1240,'USA' from dual union all
select 12400,'AFSC' from dual union all
select 12401,'44FA2' from dual union all
select 12402,'RVSN' from dual union all
select 12403,'RVSN' from dual union all
select 12404,'MVS' from dual union all
select 12405,'NASA' from dual union all
select 12406,'USAF' from dual union all
select 12407,'RVSN' from dual union all
select 12408,'SAND' from dual union all
select 12409,'USN' from dual union all
select 1241,'USA' from dual union all
select 12410,'SAND' from dual union all
select 12411,'SAND' from dual union all
select 12412,'SAND' from dual union all
select 12413,'SAND' from dual union all
select 12414,'ONERA' from dual union all
select 12415,'DASA' from dual union all
select 12416,'USA' from dual union all
select 12417,'USA' from dual union all
select 12418,'USN' from dual union all
select 12419,'USN' from dual union all
select 1242,'USA' from dual union all
select 12420,'SAC' from dual union all
select 12421,'USA' from dual union all
select 12422,'USA' from dual union all
select 12423,'RVSN' from dual union all
select 12424,'1STRAD' from dual union all
select 12425,'44FA2' from dual union all
select 12426,'NASA' from dual union all
select 12427,'USAF' from dual union all
select 12428,'USAF' from dual union all
select 12429,'NRCC' from dual union all
select 1243,'USA' from dual union all
select 12430,'ONERA' from dual union all
select 12431,'KPNO' from dual union all
select 12432,'AFSC' from dual union all
select 12433,'HARP' from dual union all
select 12434,'CNES' from dual union all
select 12435,'HARP' from dual union all
select 12436,'HARP' from dual union all
select 12437,'DASA' from dual union all
select 12438,'1STRAD' from dual union all
select 12439,'WRE' from dual union all
select 1244,'USAF' from dual union all
select 12440,'AFSC' from dual union all
select 12441,'NASA' from dual union all
select 12442,'WRE' from dual union all
select 12443,'1STRAD' from dual union all
select 12444,'AFSC' from dual union all
select 12445,'SAND' from dual union all
select 12446,'BAL' from dual union all
select 12447,'NASA' from dual union all
select 12448,'SAC' from dual union all
select 12449,'USN' from dual union all
select 1245,'USA' from dual union all
select 12450,'USN' from dual union all
select 12451,'ONERA' from dual union all
select 12452,'NASA' from dual union all
select 12453,'81FA1' from dual union all
select 12454,'USN' from dual union all
select 12455,'NASA' from dual union all
select 12456,'NASA' from dual union all
select 12457,'DASA' from dual union all
select 12458,'BAL' from dual union all
select 12459,'NASA' from dual union all
select 1246,'USA' from dual union all
select 12460,'NASA' from dual;
insert into LAUNCH_AGENCY
select 12461,'CNES' from dual union all
select 12462,'USAF' from dual union all
select 12463,'SAC' from dual union all
select 12464,'DASA' from dual union all
select 12465,'USAF' from dual union all
select 12466,'1STRAD' from dual union all
select 12467,'CNES' from dual union all
select 12468,'CNES' from dual union all
select 12469,'CNES' from dual union all
select 1247,'USA' from dual union all
select 12470,'CNES' from dual union all
select 12471,'NASA' from dual union all
select 12472,'NASA' from dual union all
select 12473,'NASA' from dual union all
select 12474,'USAF' from dual union all
select 12475,'NASA' from dual union all
select 12476,'NASA' from dual union all
select 12477,'KPNO' from dual union all
select 12478,'NASA' from dual union all
select 12479,'CNES' from dual union all
select 1248,'ORDRSA' from dual union all
select 12480,'CNES' from dual union all
select 12481,'CNES' from dual union all
select 12482,'CNES' from dual union all
select 12483,'DASA' from dual union all
select 12484,'1STRAD' from dual union all
select 12485,'MVS' from dual union all
select 12486,'USAF' from dual union all
select 12487,'USAF' from dual union all
select 12488,'NASA' from dual union all
select 12488,'USAF' from dual union all
select 12489,'NASA' from dual union all
select 1249,'ORDRSA' from dual union all
select 12490,'HARP' from dual union all
select 12491,'USA' from dual union all
select 12492,'USA' from dual union all
select 12493,'USA' from dual union all
select 12494,'USA' from dual union all
select 12495,'USA' from dual union all
select 12496,'1STRAD' from dual union all
select 12497,'USA' from dual union all
select 12498,'USA' from dual union all
select 12499,'USA' from dual union all
select 125,'WER500' from dual union all
select 1250,'ORDRSA' from dual union all
select 12500,'USA' from dual union all
select 12501,'USA' from dual union all
select 12502,'USA' from dual union all
select 12503,'USA' from dual union all
select 12504,'USA' from dual union all
select 12505,'USA' from dual union all
select 12506,'USA' from dual union all
select 12507,'SAND' from dual union all
select 12508,'USA' from dual union all
select 12509,'USA' from dual union all
select 1251,'ORDRSA' from dual union all
select 12510,'NASA' from dual union all
select 12511,'SAC' from dual union all
select 12512,'NASA' from dual union all
select 12513,'RAE' from dual union all
select 12513,'WRE' from dual union all
select 12514,'AFSC' from dual union all
select 12515,'1STRAD' from dual union all
select 12516,'ONERA' from dual union all
select 12517,'1STRAD' from dual union all
select 12518,'NRL' from dual union all
select 12519,'USAF' from dual union all
select 1252,'USA' from dual union all
select 12520,'WRE' from dual union all
select 12521,'RVSN' from dual union all
select 12522,'RAE' from dual union all
select 12522,'WRE' from dual union all
select 12523,'NASA' from dual union all
select 12524,'USAF' from dual union all
select 12525,'ISAS' from dual union all
select 12526,'RVSN' from dual union all
select 12527,'USAF' from dual union all
select 12528,'SAC' from dual union all
select 12529,'ISAS' from dual union all
select 1253,'USA' from dual union all
select 12530,'USAF' from dual union all
select 12531,'NASA' from dual union all
select 12532,'USAF' from dual union all
select 12533,'SAND' from dual union all
select 12534,'USAF' from dual union all
select 12535,'ISAS' from dual union all
select 12536,'RAE' from dual union all
select 12537,'ISAS' from dual union all
select 12538,'DASA' from dual union all
select 12539,'USAF' from dual union all
select 1254,'LRBA' from dual union all
select 12540,'CARDE' from dual union all
select 12541,'USAF' from dual union all
select 12542,'WRE' from dual union all
select 12543,'AFSC' from dual union all
select 12544,'AFSC' from dual union all
select 12545,'AFSC' from dual union all
select 12546,'ISAS' from dual union all
select 12547,'USAF' from dual union all
select 12548,'USAF' from dual;
insert into LAUNCH_AGENCY
select 12549,'NASA' from dual union all
select 1255,'USA' from dual union all
select 12550,'NASA' from dual union all
select 12551,'USN' from dual union all
select 12552,'NASA' from dual union all
select 12553,'NASA' from dual union all
select 12554,'NASA' from dual union all
select 12555,'NASA' from dual union all
select 12556,'NASA' from dual union all
select 12557,'NASA' from dual union all
select 12558,'NASA' from dual union all
select 12559,'NASA' from dual union all
select 1256,'USA' from dual union all
select 12560,'NASA' from dual union all
select 12561,'NASA' from dual union all
select 12562,'RVSN' from dual union all
select 12563,'NASA' from dual union all
select 12564,'NASA' from dual union all
select 12564,'USAF' from dual union all
select 12565,'ESRO' from dual union all
select 12566,'USN' from dual union all
select 12567,'ISAS' from dual union all
select 12568,'NASA' from dual union all
select 12569,'USAF' from dual union all
select 1257,'USA' from dual union all
select 12570,'RVSN' from dual union all
select 12571,'SAC' from dual union all
select 12572,'ISAS' from dual union all
select 12573,'CARDE' from dual union all
select 12574,'AFSC' from dual union all
select 12575,'NASA' from dual union all
select 12576,'USAF' from dual union all
select 12577,'AFSC' from dual union all
select 12578,'CNES' from dual union all
select 12579,'1STRAD' from dual union all
select 1258,'USA' from dual union all
select 12580,'NASA' from dual union all
select 12581,'1STRAD' from dual union all
select 12582,'NASA' from dual union all
select 12583,'SAC' from dual union all
select 12584,'NRCC' from dual union all
select 12585,'AFSC' from dual union all
select 12586,'USAF' from dual union all
select 12587,'USAF' from dual union all
select 12588,'ISAS' from dual union all
select 12589,'RVSN' from dual union all
select 1259,'USA' from dual union all
select 12590,'RVSN' from dual union all
select 12591,'NASA' from dual union all
select 12592,'CNES' from dual union all
select 12593,'NASA' from dual union all
select 12594,'AN' from dual union all
select 12595,'NMC' from dual union all
select 12596,'AN' from dual union all
select 12597,'RAE' from dual union all
select 12597,'WRE' from dual union all
select 12598,'NASA' from dual union all
select 12599,'USAF' from dual union all
select 126,'WER500' from dual union all
select 1260,'USN' from dual union all
select 12600,'USAF' from dual union all
select 12601,'NASA' from dual union all
select 12602,'AFCRL' from dual union all
select 12603,'NASA' from dual union all
select 12604,'AN' from dual union all
select 12605,'NASA' from dual union all
select 12606,'SAND' from dual union all
select 12607,'RVSN' from dual union all
select 12608,'NASA' from dual union all
select 12609,'USAF' from dual union all
select 1261,'ORDRSA' from dual union all
select 12610,'AN' from dual union all
select 12611,'NASA' from dual union all
select 12612,'SAC' from dual union all
select 12613,'NASA' from dual union all
select 12614,'NASA' from dual union all
select 12615,'NASA' from dual union all
select 12616,'NASA' from dual union all
select 12617,'AFSC' from dual union all
select 12618,'NASA' from dual union all
select 12619,'RVSN' from dual union all
select 1262,'ORDRSA' from dual union all
select 12620,'USAF' from dual union all
select 12621,'NASA' from dual union all
select 12622,'USAF' from dual union all
select 12623,'NASA' from dual union all
select 12624,'NASA' from dual union all
select 12625,'RAE' from dual union all
select 12626,'NASA' from dual union all
select 12626,'USAF' from dual union all
select 12627,'NASA' from dual union all
select 12628,'NRL' from dual union all
select 12629,'AFSC' from dual union all
select 1263,'PARD' from dual union all
select 12630,'CNIE' from dual union all
select 12631,'USA' from dual union all
select 12632,'FKGR12' from dual union all
select 12633,'RVSN' from dual union all
select 12634,'RVSN' from dual union all
select 12635,'79FA2' from dual;
insert into LAUNCH_AGENCY
select 12636,'ESRO' from dual union all
select 12637,'CNES' from dual union all
select 12638,'AFSC' from dual union all
select 12639,'RVSN' from dual union all
select 1264,'USA' from dual union all
select 12640,'USAF' from dual union all
select 12641,'NASA' from dual union all
select 12642,'AN' from dual union all
select 12643,'ESRO' from dual union all
select 12644,'ISAS' from dual union all
select 12645,'USAF' from dual union all
select 12646,'NASA' from dual union all
select 12647,'NRL' from dual union all
select 12648,'WRE' from dual union all
select 12649,'NASA' from dual union all
select 1265,'ARDC' from dual union all
select 12650,'AFSC' from dual union all
select 12651,'ISAS' from dual union all
select 12652,'ONERA' from dual union all
select 12653,'RVSN' from dual union all
select 12654,'USAF' from dual union all
select 12655,'AN' from dual union all
select 12656,'NASA' from dual union all
select 12657,'NASA' from dual union all
select 12658,'NASA' from dual union all
select 12659,'NASA' from dual union all
select 1266,'USA' from dual union all
select 12660,'NASA' from dual union all
select 12661,'NASA' from dual union all
select 12661,'USAF' from dual union all
select 12662,'RVSN' from dual union all
select 12663,'CNES' from dual union all
select 12664,'USAF' from dual union all
select 12665,'RAE' from dual union all
select 12665,'WRE' from dual union all
select 12666,'NASA' from dual union all
select 12667,'AFSC' from dual union all
select 12668,'NASA' from dual union all
select 12669,'NASA' from dual union all
select 1267,'USAF' from dual union all
select 12670,'NASA' from dual union all
select 12671,'RAE' from dual union all
select 12671,'WRE' from dual union all
select 12672,'SAC' from dual union all
select 12673,'NASA' from dual union all
select 12674,'USA' from dual union all
select 12675,'NASA' from dual union all
select 12676,'USAF' from dual union all
select 12677,'MVS' from dual union all
select 12678,'CNES' from dual union all
select 12679,'NRL' from dual union all
select 1268,'PARD' from dual union all
select 12680,'NASA' from dual union all
select 12681,'NASA' from dual union all
select 12682,'NASA' from dual union all
select 12683,'CNES' from dual union all
select 12684,'AN' from dual union all
select 12685,'NASA' from dual union all
select 12686,'NASA' from dual union all
select 12687,'NASA' from dual union all
select 12688,'NRL' from dual union all
select 12689,'NASA' from dual union all
select 1269,'PARD' from dual union all
select 12690,'CNES' from dual union all
select 12691,'NMC' from dual union all
select 12692,'RVSN' from dual union all
select 12693,'NRL' from dual union all
select 12694,'USA' from dual union all
select 12695,'USA' from dual union all
select 12696,'DASA' from dual union all
select 12697,'DASA' from dual union all
select 12698,'ISAS' from dual union all
select 12699,'RVSN' from dual union all
select 127,'WER500' from dual union all
select 1270,'USAF' from dual union all
select 12700,'RVSN' from dual union all
select 12701,'USAF' from dual union all
select 12702,'USAF' from dual union all
select 12703,'USAF' from dual union all
select 12704,'USAF' from dual union all
select 12705,'RVSN' from dual union all
select 12706,'USN' from dual union all
select 12707,'USN' from dual union all
select 12708,'USN' from dual union all
select 12709,'USN' from dual union all
select 1271,'USA' from dual union all
select 12710,'USA' from dual union all
select 12711,'USAF' from dual union all
select 12712,'ISAS' from dual union all
select 12713,'USAF' from dual union all
select 12714,'AFSC' from dual union all
select 12715,'WRE' from dual union all
select 12716,'NASA' from dual union all
select 12717,'USAF' from dual union all
select 12718,'NASA' from dual union all
select 12719,'USAF' from dual union all
select 1272,'USA' from dual union all
select 12720,'NASA' from dual union all
select 12721,'USA' from dual union all
select 12722,'USAF' from dual;
insert into LAUNCH_AGENCY
select 12723,'AFCRL' from dual union all
select 12724,'USAF' from dual union all
select 12725,'SAND' from dual union all
select 12726,'RVSN' from dual union all
select 12727,'USN' from dual union all
select 12728,'RVSN' from dual union all
select 12729,'NASA' from dual union all
select 1273,'USA' from dual union all
select 12730,'USAF' from dual union all
select 12731,'SAND' from dual union all
select 12732,'DASA' from dual union all
select 12733,'NASA' from dual union all
select 12734,'AFCRL' from dual union all
select 12735,'AFCRL' from dual union all
select 12736,'USAF' from dual union all
select 12737,'AFCRL' from dual union all
select 12738,'AFCRL' from dual union all
select 12739,'USAF' from dual union all
select 1274,'USA' from dual union all
select 12740,'USAF' from dual union all
select 12741,'AFCRL' from dual union all
select 12742,'NASA' from dual union all
select 12743,'79FA2' from dual union all
select 12744,'CNES' from dual union all
select 12745,'RAE' from dual union all
select 12745,'WRE' from dual union all
select 12746,'CNES' from dual union all
select 12747,'NDRE' from dual union all
select 12748,'NASA' from dual union all
select 12749,'KPNO' from dual union all
select 1275,'PARD' from dual union all
select 12750,'CNES' from dual union all
select 12751,'SAND' from dual union all
select 12752,'SAND' from dual union all
select 12753,'STA' from dual union all
select 12754,'79FA2' from dual union all
select 12755,'USAF' from dual union all
select 12756,'USAF' from dual union all
select 12757,'USAF' from dual union all
select 12758,'USAF' from dual union all
select 12759,'USAF' from dual union all
select 1276,'USAF' from dual union all
select 12760,'USAF' from dual union all
select 12761,'NASA' from dual union all
select 12762,'NASA' from dual union all
select 12763,'CNIE' from dual union all
select 12764,'CNIE' from dual union all
select 12765,'ISAS' from dual union all
select 12766,'RAE' from dual union all
select 12767,'CNIE' from dual union all
select 12768,'RVSN' from dual union all
select 12769,'SAC' from dual union all
select 1277,'ORDRSA' from dual union all
select 12770,'AFSC' from dual union all
select 12771,'PRO' from dual union all
select 12772,'USAF' from dual union all
select 12773,'RVSN' from dual union all
select 12774,'RVSN' from dual union all
select 12775,'NASA' from dual union all
select 12776,'DASST' from dual union all
select 12777,'SAC' from dual union all
select 12778,'NASA' from dual union all
select 12779,'USN' from dual union all
select 1278,'ORDRSA' from dual union all
select 12780,'USAF' from dual union all
select 12781,'WRE' from dual union all
select 12782,'RVSN' from dual union all
select 12783,'ESRO' from dual union all
select 12784,'USAF' from dual union all
select 12785,'NRL' from dual union all
select 12786,'WRE' from dual union all
select 12787,'1STRAD' from dual union all
select 12788,'AFCRL' from dual union all
select 12789,'USAF' from dual union all
select 1279,'USAF' from dual union all
select 12790,'AFCRL' from dual union all
select 12791,'USAF' from dual union all
select 12792,'ESRO' from dual union all
select 12793,'AFCRL' from dual union all
select 12794,'AFCRL' from dual union all
select 12795,'ISAS' from dual union all
select 12796,'RAE' from dual union all
select 12796,'WRE' from dual union all
select 12797,'RVSN' from dual union all
select 12798,'1STRAD' from dual union all
select 12799,'ESRO' from dual union all
select 128,'WER500' from dual union all
select 1280,'USA' from dual union all
select 12800,'1STRAD' from dual union all
select 12801,'AFSC' from dual union all
select 12802,'NASA' from dual union all
select 12803,'CNAE' from dual union all
select 12803,'NASA' from dual union all
select 12804,'CNES' from dual union all
select 12805,'CNES' from dual union all
select 12806,'CNES' from dual union all
select 12807,'CNAE' from dual union all
select 12807,'NASA' from dual union all
select 12808,'ISAS' from dual union all
select 12809,'DASA' from dual;
insert into LAUNCH_AGENCY
select 1281,'LRBA' from dual union all
select 12810,'USN' from dual union all
select 12811,'CNES' from dual union all
select 12812,'AFSC' from dual union all
select 12813,'RVSN' from dual union all
select 12814,'SAC' from dual union all
select 12815,'DASST' from dual union all
select 12816,'RVSN' from dual union all
select 12817,'RVSN' from dual union all
select 12818,'RVSN' from dual union all
select 12819,'RVSN' from dual union all
select 1282,'LRBA' from dual union all
select 12820,'SAND' from dual union all
select 12821,'RVSN' from dual union all
select 12822,'RVSN' from dual union all
select 12823,'WRE' from dual union all
select 12824,'MVS' from dual union all
select 12825,'SAND' from dual union all
select 12826,'HARP' from dual union all
select 12827,'HARP' from dual union all
select 12828,'HARP' from dual union all
select 12829,'HARP' from dual union all
select 1283,'USA' from dual union all
select 12830,'HARP' from dual union all
select 12831,'WRE' from dual union all
select 12832,'HARP' from dual union all
select 12833,'HARP' from dual union all
select 12834,'HARP' from dual union all
select 12835,'HARP' from dual union all
select 12836,'HARP' from dual union all
select 12837,'HARP' from dual union all
select 12838,'HARP' from dual union all
select 12839,'HARP' from dual union all
select 1284,'PARD' from dual union all
select 12840,'HARP' from dual union all
select 12841,'HARP' from dual union all
select 12842,'SAND' from dual union all
select 12843,'HARP' from dual union all
select 12844,'HARP' from dual union all
select 12845,'HARP' from dual union all
select 12846,'HARP' from dual union all
select 12847,'HARP' from dual union all
select 12848,'HARP' from dual union all
select 12849,'HARP' from dual union all
select 1285,'USA' from dual union all
select 12850,'HARP' from dual union all
select 12851,'HARP' from dual union all
select 12852,'HARP' from dual union all
select 12853,'NASA' from dual union all
select 12854,'HARP' from dual union all
select 12855,'HARP' from dual union all
select 12856,'HARP' from dual union all
select 12857,'HARP' from dual union all
select 12858,'HARP' from dual union all
select 12859,'HARP' from dual union all
select 1286,'USA' from dual union all
select 12860,'HARP' from dual union all
select 12861,'HARP' from dual union all
select 12862,'HARP' from dual union all
select 12863,'HARP' from dual union all
select 12864,'NASA' from dual union all
select 12865,'HARP' from dual union all
select 12866,'HARP' from dual union all
select 12867,'HARP' from dual union all
select 12868,'HARP' from dual union all
select 12869,'HARP' from dual union all
select 1287,'USA' from dual union all
select 12870,'HARP' from dual union all
select 12871,'HARP' from dual union all
select 12872,'HARP' from dual union all
select 12873,'HARP' from dual union all
select 12874,'NASA' from dual union all
select 12875,'HARP' from dual union all
select 12876,'HARP' from dual union all
select 12877,'HARP' from dual union all
select 12878,'HARP' from dual union all
select 12879,'HARP' from dual union all
select 1288,'USA' from dual union all
select 12880,'HARP' from dual union all
select 12881,'HARP' from dual union all
select 12882,'HARP' from dual union all
select 12883,'HARP' from dual union all
select 12884,'HARP' from dual union all
select 12885,'NASA' from dual union all
select 12886,'HARP' from dual union all
select 12887,'HARP' from dual union all
select 12888,'HARP' from dual union all
select 12889,'HARP' from dual union all
select 1289,'LRBA' from dual union all
select 12890,'HARP' from dual union all
select 12891,'HARP' from dual union all
select 12892,'HARP' from dual union all
select 12893,'HARP' from dual union all
select 12894,'HARP' from dual union all
select 12895,'USN' from dual union all
select 12896,'HARP' from dual union all
select 12897,'USAF' from dual union all
select 12898,'USAF' from dual union all
select 12899,'USA' from dual union all
select 129,'WER485-1' from dual;
insert into LAUNCH_AGENCY
select 1290,'PARD' from dual union all
select 12900,'USA' from dual union all
select 12901,'USA' from dual union all
select 12902,'USA' from dual union all
select 12903,'USA' from dual union all
select 12904,'USA' from dual union all
select 12905,'USA' from dual union all
select 12906,'USN' from dual union all
select 12907,'USA' from dual union all
select 12908,'USA' from dual union all
select 12909,'USA' from dual union all
select 1291,'LRBA' from dual union all
select 12910,'NASA' from dual union all
select 12911,'USA' from dual union all
select 12912,'USA' from dual union all
select 12913,'USA' from dual union all
select 12914,'USA' from dual union all
select 12915,'41FA4' from dual union all
select 12916,'USN' from dual union all
select 12917,'41FA4' from dual union all
select 12918,'USN' from dual union all
select 12919,'USN' from dual union all
select 1292,'USA' from dual union all
select 12920,'SAND' from dual union all
select 12921,'SAND' from dual union all
select 12922,'RVSN' from dual union all
select 12923,'RVSN' from dual union all
select 12924,'RVSN' from dual union all
select 12925,'RVSN' from dual union all
select 12926,'RVSN' from dual union all
select 12927,'USN' from dual union all
select 12928,'RVSN' from dual union all
select 12929,'WRE' from dual union all
select 1293,'USAF' from dual union all
select 12930,'MVS' from dual union all
select 12931,'MVS' from dual union all
select 12932,'RVSN' from dual union all
select 12933,'MVS' from dual union all
select 12934,'MVS' from dual union all
select 12935,'MVS' from dual union all
select 12936,'RVSN' from dual union all
select 12937,'RVSN' from dual union all
select 12938,'ISAS' from dual union all
select 12939,'ISAS' from dual union all
select 1294,'USAF' from dual union all
select 12940,'USAF' from dual union all
select 12941,'1STRAD' from dual union all
select 12942,'USN' from dual union all
select 12943,'NASA' from dual union all
select 12944,'NASA' from dual union all
select 12945,'NASA' from dual union all
select 12946,'MVS' from dual union all
select 12947,'USAF' from dual union all
select 12948,'79FA2' from dual union all
select 12949,'USN' from dual union all
select 1295,'LRBA' from dual union all
select 12950,'USN' from dual union all
select 12951,'CNES' from dual union all
select 12952,'MVS' from dual union all
select 12953,'USN' from dual union all
select 12954,'NASA' from dual union all
select 12955,'USAF' from dual union all
select 12956,'NASA' from dual union all
select 12957,'RVSN' from dual union all
select 12958,'NASA' from dual union all
select 12959,'NASA' from dual union all
select 1296,'USA' from dual union all
select 12960,'NASA' from dual union all
select 12961,'USAF' from dual union all
select 12962,'USAF' from dual union all
select 12963,'USAF' from dual union all
select 12964,'USAF' from dual union all
select 12965,'USAF' from dual union all
select 12966,'USAF' from dual union all
select 12967,'USAF' from dual union all
select 12968,'USAF' from dual union all
select 12969,'USAF' from dual union all
select 1297,'USA' from dual union all
select 12970,'USAF' from dual union all
select 12971,'ISRO' from dual union all
select 12972,'ISRO' from dual union all
select 12973,'DASA' from dual union all
select 12974,'USAF' from dual union all
select 12975,'ISRO' from dual union all
select 12976,'MSFC' from dual union all
select 12977,'MSFC' from dual union all
select 12978,'MSFC' from dual union all
select 12979,'USAF' from dual union all
select 1298,'USA' from dual union all
select 12980,'USAF' from dual union all
select 12981,'USAF' from dual union all
select 12982,'USAF' from dual union all
select 12983,'USAF' from dual union all
select 12984,'USAF' from dual union all
select 12985,'USAF' from dual union all
select 12986,'USAF' from dual union all
select 12987,'USAF' from dual union all
select 12988,'USAF' from dual union all
select 12989,'USAF' from dual union all
select 1299,'LRBA' from dual;
insert into LAUNCH_AGENCY
select 12990,'USAF' from dual union all
select 12991,'USAF' from dual union all
select 12992,'USAF' from dual union all
select 12993,'MSFC' from dual union all
select 12994,'ISRO' from dual union all
select 12995,'MSFC' from dual union all
select 12996,'USAF' from dual union all
select 12997,'MSFC' from dual union all
select 12998,'USAF' from dual union all
select 12999,'USAF' from dual union all
select 13,'WEHR' from dual union all
select 130,'WER500' from dual union all
select 1300,'PARD' from dual union all
select 13000,'USAF' from dual union all
select 13001,'USAF' from dual union all
select 13002,'USAF' from dual union all
select 13003,'USAF' from dual union all
select 13004,'USAF' from dual union all
select 13005,'MSFC' from dual union all
select 13006,'NASA' from dual union all
select 13007,'USAF' from dual union all
select 13008,'NASA' from dual union all
select 13009,'MSFC' from dual union all
select 1301,'ORDRSA' from dual union all
select 13010,'ISRO' from dual union all
select 13011,'MSFC' from dual union all
select 13012,'MSFC' from dual union all
select 13013,'ISAS' from dual union all
select 13014,'MSFC' from dual union all
select 13015,'USAF' from dual union all
select 13016,'USAF' from dual union all
select 13017,'USAF' from dual union all
select 13018,'USAF' from dual union all
select 13019,'USAF' from dual union all
select 1302,'USA' from dual union all
select 13020,'USAF' from dual union all
select 13021,'USAF' from dual union all
select 13022,'USAF' from dual union all
select 13023,'USAF' from dual union all
select 13024,'USAF' from dual union all
select 13025,'MSFC' from dual union all
select 13026,'MSFC' from dual union all
select 13027,'ISRO' from dual union all
select 13028,'USAF' from dual union all
select 13029,'MSFC' from dual union all
select 1303,'ORDRSA' from dual union all
select 13030,'US' from dual union all
select 13031,'MSFC' from dual union all
select 13032,'DASA' from dual union all
select 13033,'USAF' from dual union all
select 13034,'USAF' from dual union all
select 13035,'USAF' from dual union all
select 13036,'USAF' from dual union all
select 13037,'USAF' from dual union all
select 13038,'USAF' from dual union all
select 13039,'USAF' from dual union all
select 1304,'ORDRSA' from dual union all
select 13040,'USAF' from dual union all
select 13041,'MSFC' from dual union all
select 13042,'MSFC' from dual union all
select 13043,'ISRO' from dual union all
select 13044,'MSFC' from dual union all
select 13045,'USAF' from dual union all
select 13046,'USAF' from dual union all
select 13047,'USAF' from dual union all
select 13048,'USAF' from dual union all
select 13049,'USAF' from dual union all
select 1305,'USA' from dual union all
select 13050,'ISRO' from dual union all
select 13051,'USAF' from dual union all
select 13052,'ISRO' from dual union all
select 13053,'USA' from dual union all
select 13054,'MSFC' from dual union all
select 13055,'MSFC' from dual union all
select 13056,'USAF' from dual union all
select 13057,'USAF' from dual union all
select 13058,'USAF' from dual union all
select 13059,'USAF' from dual union all
select 1306,'USA' from dual union all
select 13060,'USAF' from dual union all
select 13061,'MSFC' from dual union all
select 13062,'USAF' from dual union all
select 13063,'USAF' from dual union all
select 13064,'USAF' from dual union all
select 13065,'USAF' from dual union all
select 13066,'USAF' from dual union all
select 13067,'USAF' from dual union all
select 13068,'USAF' from dual union all
select 13069,'USAF' from dual union all
select 1307,'ORDRSA' from dual union all
select 13070,'USAF' from dual union all
select 13071,'USAF' from dual union all
select 13072,'MSFC' from dual union all
select 13073,'USAF' from dual union all
select 13074,'USAF' from dual union all
select 13075,'USAF' from dual union all
select 13076,'USAF' from dual union all
select 13077,'USAF' from dual union all
select 13078,'USAF' from dual union all
select 13079,'USAF' from dual;
insert into LAUNCH_AGENCY
select 1308,'PARD' from dual union all
select 13080,'USAF' from dual union all
select 13081,'MSFC' from dual union all
select 13082,'USAF' from dual union all
select 13083,'MSFC' from dual union all
select 13084,'MRN' from dual union all
select 13085,'ISRO' from dual union all
select 13086,'ISRO' from dual union all
select 13087,'MSFC' from dual union all
select 13088,'ISRO' from dual union all
select 13089,'MSFC' from dual union all
select 1309,'USA' from dual union all
select 13090,'ERDA' from dual union all
select 13091,'SUPA' from dual union all
select 13092,'SUPA' from dual union all
select 13093,'SUPA' from dual union all
select 13094,'USAF' from dual union all
select 13095,'KSC' from dual union all
select 13096,'KSC' from dual union all
select 13097,'KSC' from dual union all
select 13098,'KSC' from dual union all
select 13099,'NASA' from dual union all
select 131,'WER500' from dual union all
select 1310,'USA' from dual union all
select 13100,'NASA' from dual union all
select 13101,'NASA' from dual union all
select 13102,'NASA' from dual union all
select 13103,'NASA' from dual union all
select 13104,'NASA' from dual union all
select 13105,'USAF' from dual union all
select 13106,'NASA' from dual union all
select 13107,'NASA' from dual union all
select 13108,'NASA' from dual union all
select 13109,'NASA' from dual union all
select 1311,'USA' from dual union all
select 13110,'NASA' from dual union all
select 13111,'NASA' from dual union all
select 13112,'NASA' from dual union all
select 13113,'NASA' from dual union all
select 13114,'NASA' from dual union all
select 13115,'NASA' from dual union all
select 13116,'USAF' from dual union all
select 13117,'NASA' from dual union all
select 13118,'NASA' from dual union all
select 13119,'NASA' from dual union all
select 1312,'ORDRSA' from dual union all
select 13120,'NASA' from dual union all
select 13121,'NASA' from dual union all
select 13122,'NASA' from dual union all
select 13123,'NASA' from dual union all
select 13124,'NASA' from dual union all
select 13125,'NASA' from dual union all
select 13126,'USAF' from dual union all
select 13127,'NASA' from dual union all
select 13128,'NASA' from dual union all
select 13129,'NASA' from dual union all
select 1313,'ORDRSA' from dual union all
select 13130,'NASA' from dual union all
select 13131,'NASA' from dual union all
select 13132,'NASA' from dual union all
select 13133,'NASA' from dual union all
select 13134,'NASA' from dual union all
select 13135,'NASA' from dual union all
select 13136,'NASA' from dual union all
select 13137,'USAF' from dual union all
select 13138,'NASA' from dual union all
select 13139,'NASA' from dual union all
select 1314,'ORDRSA' from dual union all
select 13140,'NASA' from dual union all
select 13141,'NASA' from dual union all
select 13142,'NASA' from dual union all
select 13143,'NASA' from dual union all
select 13144,'NASA' from dual union all
select 13145,'NASA' from dual union all
select 13146,'NASA' from dual union all
select 13147,'NASA' from dual union all
select 13148,'USAF' from dual union all
select 13149,'NASA' from dual union all
select 1315,'ORDRSA' from dual union all
select 13150,'NASA' from dual union all
select 13151,'NASA' from dual union all
select 13152,'NASA' from dual union all
select 13153,'NASA' from dual union all
select 13154,'NASA' from dual union all
select 13155,'NASA' from dual union all
select 13156,'NASA' from dual union all
select 13157,'NASA' from dual union all
select 13158,'NASA' from dual union all
select 13159,'USAF' from dual union all
select 1316,'USA' from dual union all
select 13160,'NASA' from dual union all
select 13161,'NASA' from dual union all
select 13162,'NASA' from dual union all
select 13163,'NASA' from dual union all
select 13164,'NASA' from dual union all
select 13165,'NASA' from dual union all
select 13166,'NASA' from dual union all
select 13167,'NASA' from dual union all
select 13168,'NASA' from dual union all
select 13169,'NASA' from dual;
insert into LAUNCH_AGENCY
select 1317,'USA' from dual union all
select 13170,'USAF' from dual union all
select 13171,'SUPA' from dual union all
select 13172,'SUPA' from dual union all
select 13173,'NASA' from dual union all
select 13174,'USAF' from dual union all
select 13175,'USAF' from dual union all
select 13176,'USAF' from dual union all
select 13177,'MSFC' from dual union all
select 13178,'USA' from dual union all
select 13179,'NASA' from dual union all
select 1318,'USAF' from dual union all
select 13180,'ISAS' from dual union all
select 13181,'NASA' from dual union all
select 13182,'MSFC' from dual union all
select 13183,'MSFC' from dual union all
select 13184,'ISAS' from dual union all
select 13185,'ISAS' from dual union all
select 13186,'NASA' from dual union all
select 13187,'NASA' from dual union all
select 13187,'NTNF' from dual union all
select 13188,'ISRO' from dual union all
select 13189,'MSFC' from dual union all
select 1319,'USAF' from dual union all
select 13190,'ERDA' from dual union all
select 13191,'NASA' from dual union all
select 13191,'NTNF' from dual union all
select 13192,'MSFC' from dual union all
select 13193,'USAF' from dual union all
select 13194,'USAF' from dual union all
select 13195,'USAF' from dual union all
select 13196,'USAF' from dual union all
select 13197,'DASA' from dual union all
select 13198,'IAE' from dual union all
select 13199,'MSFC' from dual union all
select 132,'WEHR' from dual union all
select 1320,'ORDRSA' from dual union all
select 13200,'MSFC' from dual union all
select 13201,'MSFC' from dual union all
select 13202,'MSFC' from dual union all
select 13203,'MSFC' from dual union all
select 13204,'MSFC' from dual union all
select 13205,'USAF' from dual union all
select 13206,'USAF' from dual union all
select 13207,'USAF' from dual union all
select 13208,'USAF' from dual union all
select 13209,'USAF' from dual union all
select 1321,'USA' from dual union all
select 13210,'USAF' from dual union all
select 13211,'USAF' from dual union all
select 13212,'AN' from dual union all
select 13213,'MSFC' from dual union all
select 13214,'ISRO' from dual union all
select 13215,'USAF' from dual union all
select 13216,'USAF' from dual union all
select 13217,'MSFC' from dual union all
select 13218,'USAF' from dual union all
select 13219,'USAF' from dual union all
select 1322,'ORDRSA' from dual union all
select 13220,'USAF' from dual union all
select 13221,'USAF' from dual union all
select 13222,'USAF' from dual union all
select 13223,'USAF' from dual union all
select 13224,'MSFC' from dual union all
select 13225,'MSFC' from dual union all
select 13226,'ISRO' from dual union all
select 13227,'MSFC' from dual union all
select 13228,'MSFC' from dual union all
select 13229,'USAF' from dual union all
select 1323,'USA' from dual union all
select 13230,'USAF' from dual union all
select 13231,'USAF' from dual union all
select 13232,'USAF' from dual union all
select 13233,'USAF' from dual union all
select 13234,'USAF' from dual union all
select 13235,'USAF' from dual union all
select 13236,'USAF' from dual union all
select 13237,'MSFC' from dual union all
select 13238,'MSFC' from dual union all
select 13239,'ISAS' from dual union all
select 1324,'USA' from dual union all
select 13240,'ISAS' from dual union all
select 13241,'ISAS' from dual union all
select 13242,'ERDA' from dual union all
select 13243,'MSFC' from dual union all
select 13244,'ERDA' from dual union all
select 13245,'CALT' from dual union all
select 13246,'AN' from dual union all
select 13247,'AN' from dual union all
select 13248,'AN' from dual union all
select 13249,'AN' from dual union all
select 1325,'USAF' from dual union all
select 13250,'AN' from dual union all
select 13251,'AN' from dual union all
select 13252,'AN' from dual union all
select 13253,'AN' from dual union all
select 13254,'AN' from dual union all
select 13255,'AN' from dual union all
select 13256,'AN' from dual union all
select 13257,'AN' from dual;
insert into LAUNCH_AGENCY
select 13258,'AN' from dual union all
select 13259,'AN' from dual union all
select 1326,'USAF' from dual union all
select 13260,'AN' from dual union all
select 13261,'AN' from dual union all
select 13262,'AN' from dual union all
select 13263,'AN' from dual union all
select 13264,'AN' from dual union all
select 13265,'AN' from dual union all
select 13266,'AN' from dual union all
select 13267,'AN' from dual union all
select 13268,'AN' from dual union all
select 13269,'AN' from dual union all
select 1327,'USA' from dual union all
select 13270,'AN' from dual union all
select 13271,'AN' from dual union all
select 13272,'AN' from dual union all
select 13273,'AN' from dual union all
select 13274,'AN' from dual union all
select 13275,'AN' from dual union all
select 13276,'AN' from dual union all
select 13277,'AN' from dual union all
select 13278,'AN' from dual union all
select 13279,'AN' from dual union all
select 1328,'USA' from dual union all
select 13280,'AN' from dual union all
select 13281,'AN' from dual union all
select 13282,'AN' from dual union all
select 13283,'AN' from dual union all
select 13284,'AN' from dual union all
select 13285,'AN' from dual union all
select 13286,'AN' from dual union all
select 13287,'AN' from dual union all
select 13288,'AN' from dual union all
select 13289,'AN' from dual union all
select 1329,'USA' from dual union all
select 13290,'AN' from dual union all
select 13291,'AN' from dual union all
select 13292,'AN' from dual union all
select 13293,'AN' from dual union all
select 13294,'AN' from dual union all
select 13295,'AN' from dual union all
select 13296,'AN' from dual union all
select 13297,'AN' from dual union all
select 13298,'AN' from dual union all
select 13299,'AN' from dual union all
select 133,'WER836-2' from dual union all
select 1330,'USA' from dual union all
select 13300,'AN' from dual union all
select 13301,'AN' from dual union all
select 13302,'AN' from dual union all
select 13303,'AN' from dual union all
select 13304,'AN' from dual union all
select 13305,'AN' from dual union all
select 13306,'AN' from dual union all
select 13307,'AN' from dual union all
select 13308,'AN' from dual union all
select 13309,'NASA' from dual union all
select 13309,'UIO' from dual union all
select 1331,'ORDRSA' from dual union all
select 13310,'AN' from dual union all
select 13311,'AN' from dual union all
select 13312,'AN' from dual union all
select 13313,'AN' from dual union all
select 13314,'AN' from dual union all
select 13315,'AN' from dual union all
select 13316,'AN' from dual union all
select 13317,'AN' from dual union all
select 13318,'AN' from dual union all
select 13319,'NTNF' from dual union all
select 1332,'ORDRSA' from dual union all
select 13320,'NASA' from dual union all
select 13320,'UIO' from dual union all
select 13321,'NASA' from dual union all
select 13321,'NTNF' from dual union all
select 13322,'AN' from dual union all
select 13323,'AN' from dual union all
select 13324,'AN' from dual union all
select 13325,'AN' from dual union all
select 13326,'AN' from dual union all
select 13327,'AN' from dual union all
select 13328,'AN' from dual union all
select 13329,'AN' from dual union all
select 1333,'ORDRSA' from dual union all
select 13330,'AN' from dual union all
select 13331,'AN' from dual union all
select 13332,'AN' from dual union all
select 13333,'AN' from dual union all
select 13334,'AN' from dual union all
select 13335,'AN' from dual union all
select 13336,'AN' from dual union all
select 13337,'AN' from dual union all
select 13338,'AN' from dual union all
select 13339,'AN' from dual union all
select 1334,'USA' from dual union all
select 13340,'AN' from dual union all
select 13341,'AN' from dual union all
select 13342,'AN' from dual union all
select 13343,'AN' from dual union all
select 13344,'AN' from dual;
insert into LAUNCH_AGENCY
select 13345,'AN' from dual union all
select 13346,'AN' from dual union all
select 13347,'AN' from dual union all
select 13348,'AN' from dual union all
select 13349,'AN' from dual union all
select 1335,'USA' from dual union all
select 13350,'AN' from dual union all
select 13351,'AN' from dual union all
select 13352,'AN' from dual union all
select 13353,'AN' from dual union all
select 13354,'AN' from dual union all
select 13355,'AN' from dual union all
select 13356,'AN' from dual union all
select 13357,'AN' from dual union all
select 13358,'AN' from dual union all
select 13359,'AN' from dual union all
select 1336,'USA' from dual union all
select 13360,'AN' from dual union all
select 13361,'AN' from dual union all
select 13362,'AN' from dual union all
select 13363,'AN' from dual union all
select 13364,'AN' from dual union all
select 13365,'AN' from dual union all
select 13366,'AN' from dual union all
select 13367,'AN' from dual union all
select 13368,'AN' from dual union all
select 13369,'AN' from dual union all
select 1337,'USA' from dual union all
select 13370,'AN' from dual union all
select 13371,'AN' from dual union all
select 13372,'AN' from dual union all
select 13373,'AN' from dual union all
select 13374,'AN' from dual union all
select 13375,'AN' from dual union all
select 13376,'AN' from dual union all
select 13377,'AN' from dual union all
select 13378,'AN' from dual union all
select 13379,'AN' from dual union all
select 1338,'PARD' from dual union all
select 13380,'AN' from dual union all
select 13381,'AN' from dual union all
select 13382,'NASA' from dual union all
select 13383,'AN' from dual union all
select 13384,'AN' from dual union all
select 13385,'AN' from dual union all
select 13386,'AN' from dual union all
select 13387,'AN' from dual union all
select 13388,'AN' from dual union all
select 13389,'NASA' from dual union all
select 1339,'ORDRSA' from dual union all
select 13390,'NASA' from dual union all
select 13391,'NASA' from dual union all
select 13392,'NASA' from dual union all
select 13393,'NASA' from dual union all
select 13394,'NASA' from dual union all
select 13395,'AN' from dual union all
select 13396,'AN' from dual union all
select 13397,'AN' from dual union all
select 13398,'AN' from dual union all
select 13399,'AN' from dual union all
select 134,'WER836-2' from dual union all
select 1340,'ORDRSA' from dual union all
select 13400,'AN' from dual union all
select 13401,'AN' from dual union all
select 13402,'AN' from dual union all
select 13403,'AN' from dual union all
select 13404,'AN' from dual union all
select 13405,'AN' from dual union all
select 13406,'AN' from dual union all
select 13407,'AN' from dual union all
select 13408,'RVSN' from dual union all
select 13409,'AFSC' from dual union all
select 1341,'USAF' from dual union all
select 13410,'RVSN' from dual union all
select 13411,'RVSN' from dual union all
select 13412,'AFSC' from dual union all
select 13413,'RVSN' from dual union all
select 13414,'USAF' from dual union all
select 13415,'NASA' from dual union all
select 13416,'USAF' from dual union all
select 13417,'RVSN' from dual union all
select 13418,'RVSN' from dual union all
select 13419,'AFSC' from dual union all
select 1342,'ORDRSA' from dual union all
select 13420,'CNES' from dual union all
select 13421,'RVSN' from dual union all
select 13422,'RVSN' from dual union all
select 13423,'NASA' from dual union all
select 13424,'RVSN' from dual union all
select 13425,'USAF' from dual union all
select 13426,'USAF' from dual union all
select 13427,'NASA' from dual union all
select 13428,'RVSN' from dual union all
select 13429,'AFSC' from dual union all
select 1343,'USA' from dual union all
select 13430,'RVSN' from dual union all
select 13431,'AFSC' from dual union all
select 13432,'AFSC' from dual union all
select 13433,'S4300' from dual union all
select 13434,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 13435,'RVSN' from dual union all
select 13436,'USAF' from dual union all
select 13437,'NASA' from dual union all
select 13438,'USAF' from dual union all
select 13439,'AFSC' from dual union all
select 1344,'USAF' from dual union all
select 13440,'RVSN' from dual union all
select 13441,'AFSC' from dual union all
select 13442,'RVSN' from dual union all
select 13443,'RVSN' from dual union all
select 13444,'RVSN' from dual union all
select 13445,'RVSN' from dual union all
select 13446,'AFSC' from dual union all
select 13447,'USAF' from dual union all
select 13448,'AFSC' from dual union all
select 13449,'USAF' from dual union all
select 1345,'USA' from dual union all
select 13450,'RVSN' from dual union all
select 13451,'NASA' from dual union all
select 13452,'NASA' from dual union all
select 13453,'USAF' from dual union all
select 13454,'NASA' from dual union all
select 13455,'AFSC' from dual union all
select 13456,'USAF' from dual union all
select 13457,'RVSN' from dual union all
select 13458,'AFSC' from dual union all
select 13459,'NASA' from dual union all
select 1346,'USA' from dual union all
select 13460,'USAF' from dual union all
select 13461,'RVSN' from dual union all
select 13462,'USAF' from dual union all
select 13463,'USAF' from dual union all
select 13464,'RVSN' from dual union all
select 13465,'NASA' from dual union all
select 13466,'NASA' from dual union all
select 13467,'RVSN' from dual union all
select 13468,'RVSN' from dual union all
select 13469,'AFSC' from dual union all
select 1347,'USA' from dual union all
select 13470,'AFSC' from dual union all
select 13471,'RVSN' from dual union all
select 13472,'USAF' from dual union all
select 13473,'NASA' from dual union all
select 13474,'RVSN' from dual union all
select 13475,'RVSN' from dual union all
select 13476,'AFSC' from dual union all
select 13477,'AFSC' from dual union all
select 13478,'RVSN' from dual union all
select 13479,'USAF' from dual union all
select 1348,'PARD' from dual union all
select 13480,'AFSC' from dual union all
select 13481,'AFSC' from dual union all
select 13482,'NASA' from dual union all
select 13483,'AFSC' from dual union all
select 13484,'AFSC' from dual union all
select 13485,'RVSN' from dual union all
select 13486,'RVSN' from dual union all
select 13487,'USAF' from dual union all
select 13488,'NASA' from dual union all
select 13489,'S4300' from dual union all
select 1349,'PARD' from dual union all
select 13490,'AFSC' from dual union all
select 13491,'NASA' from dual union all
select 13492,'USAF' from dual union all
select 13493,'USAF' from dual union all
select 13494,'NASA' from dual union all
select 13495,'RVSN' from dual union all
select 13496,'AFSC' from dual union all
select 13497,'AFSC' from dual union all
select 13498,'RVSN' from dual union all
select 13499,'RVSN' from dual union all
select 135,'WER836-2' from dual union all
select 1350,'ORDRSA' from dual union all
select 13500,'RVSN' from dual union all
select 13501,'RVSN' from dual union all
select 13502,'NASA' from dual union all
select 13503,'NASA' from dual union all
select 13504,'AFSC' from dual union all
select 13505,'AFSC' from dual union all
select 13506,'USAF' from dual union all
select 13507,'USAF' from dual union all
select 13508,'RVSN' from dual union all
select 13509,'USAF' from dual union all
select 1351,'ORDRSA' from dual union all
select 13510,'USAF' from dual union all
select 13511,'NASA' from dual union all
select 13512,'RVSN' from dual union all
select 13513,'RVSN' from dual union all
select 13514,'RVSN' from dual union all
select 13515,'RVSN' from dual union all
select 13516,'AFSC' from dual union all
select 13517,'USAF' from dual union all
select 13518,'AFSC' from dual union all
select 13519,'RVSN' from dual union all
select 1352,'USAF' from dual union all
select 13520,'AFSC' from dual union all
select 13521,'NASA' from dual union all
select 13522,'RVSN' from dual union all
select 13523,'RVSN' from dual union all
select 13524,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 13525,'USAF' from dual union all
select 13526,'USA' from dual union all
select 13527,'PIHM' from dual union all
select 13528,'PIHM' from dual union all
select 13529,'PIHM' from dual union all
select 1353,'ORDRSA' from dual union all
select 13530,'PIHM' from dual union all
select 13531,'PIHM' from dual union all
select 13532,'PIHM' from dual union all
select 13533,'PIHM' from dual union all
select 13534,'PIHM' from dual union all
select 13535,'PIHM' from dual union all
select 13536,'USN' from dual union all
select 13537,'NASA' from dual union all
select 13537,'USAF' from dual union all
select 13538,'NASA' from dual union all
select 13538,'USAF' from dual union all
select 13539,'NASA' from dual union all
select 13539,'USAF' from dual union all
select 1354,'USA' from dual union all
select 13540,'GMS' from dual union all
select 13541,'SAND' from dual union all
select 13542,'GMS' from dual union all
select 13543,'NASA' from dual union all
select 13544,'RVSN' from dual union all
select 13545,'NASA' from dual union all
select 13546,'PIHM' from dual union all
select 13547,'RVSN' from dual union all
select 13548,'NASA' from dual union all
select 13548,'USAF' from dual union all
select 13549,'USA' from dual union all
select 1355,'USA' from dual union all
select 13550,'USA' from dual union all
select 13551,'NASA' from dual union all
select 13551,'USAF' from dual union all
select 13552,'NASA' from dual union all
select 13552,'USAF' from dual union all
select 13553,'NASA' from dual union all
select 13553,'USAF' from dual union all
select 13554,'NASA' from dual union all
select 13555,'AFCRL' from dual union all
select 13556,'NASA' from dual union all
select 13557,'PIHM' from dual union all
select 13558,'ISAS' from dual union all
select 13559,'USN' from dual union all
select 1356,'USAF' from dual union all
select 13560,'GMS' from dual union all
select 13561,'NASA' from dual union all
select 13561,'USAF' from dual union all
select 13562,'INTA' from dual union all
select 13563,'NASA' from dual union all
select 13564,'NMC' from dual union all
select 13565,'LARCN' from dual union all
select 13566,'USA' from dual union all
select 13567,'USA' from dual union all
select 13568,'PIHM' from dual union all
select 13569,'USA' from dual union all
select 1357,'SUI' from dual union all
select 13570,'RAE' from dual union all
select 13571,'ISAS' from dual union all
select 13572,'USN' from dual union all
select 13573,'USA' from dual union all
select 13574,'NASA' from dual union all
select 13575,'CNIE' from dual union all
select 13576,'USAF' from dual union all
select 13577,'NASA' from dual union all
select 13577,'USAF' from dual union all
select 13578,'NASA' from dual union all
select 13578,'USAF' from dual union all
select 13579,'NASA' from dual union all
select 1358,'USA' from dual union all
select 13580,'NASA' from dual union all
select 13581,'GMS' from dual union all
select 13582,'NASA' from dual union all
select 13582,'USAF' from dual union all
select 13583,'NASA' from dual union all
select 13584,'ESRO' from dual union all
select 13585,'CNES' from dual union all
select 13586,'NASA' from dual union all
select 13586,'USAF' from dual union all
select 13587,'USA' from dual union all
select 13588,'RAE' from dual union all
select 13588,'WRE' from dual union all
select 13589,'RVSN' from dual union all
select 1359,'USN' from dual union all
select 13590,'GMS' from dual union all
select 13591,'MRN' from dual union all
select 13591,'NASA' from dual union all
select 13592,'NASA' from dual union all
select 13593,'NRCC' from dual union all
select 13594,'KPNO' from dual union all
select 13595,'GMS' from dual union all
select 13596,'GMS' from dual union all
select 13597,'GMS' from dual union all
select 13598,'GMS' from dual union all
select 13599,'NASA' from dual union all
select 13599,'USAF' from dual union all
select 136,'WER836-2' from dual union all
select 1360,'WADD' from dual union all
select 13600,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 13601,'ARPA' from dual union all
select 13601,'US' from dual union all
select 13602,'GMS' from dual union all
select 13603,'SAND' from dual union all
select 13604,'HARP' from dual union all
select 13605,'HARP' from dual union all
select 13606,'HARP' from dual union all
select 13607,'HARP' from dual union all
select 13608,'HARP' from dual union all
select 13609,'HARP' from dual union all
select 1361,'USA' from dual union all
select 13610,'HARP' from dual union all
select 13611,'HARP' from dual union all
select 13612,'ISAS' from dual union all
select 13613,'HARP' from dual union all
select 13614,'HARP' from dual union all
select 13615,'HARP' from dual union all
select 13616,'HARP' from dual union all
select 13617,'HARP' from dual union all
select 13618,'HARP' from dual union all
select 13619,'HARP' from dual union all
select 1362,'USAF' from dual union all
select 13620,'HARP' from dual union all
select 13621,'HARP' from dual union all
select 13622,'HARP' from dual union all
select 13623,'NASA' from dual union all
select 13624,'HARP' from dual union all
select 13625,'HARP' from dual union all
select 13626,'HARP' from dual union all
select 13627,'HARP' from dual union all
select 13628,'GMS' from dual union all
select 13629,'GMS' from dual union all
select 1363,'USAF' from dual union all
select 13630,'GMS' from dual union all
select 13631,'USA' from dual union all
select 13632,'ISAS' from dual union all
select 13633,'USA' from dual union all
select 13634,'MRN' from dual union all
select 13635,'HARP' from dual union all
select 13636,'MRN' from dual union all
select 13637,'MRN' from dual union all
select 13638,'MRN' from dual union all
select 13639,'MRN' from dual union all
select 1364,'USA' from dual union all
select 13640,'GMS' from dual union all
select 13641,'MRN' from dual union all
select 13642,'MRN' from dual union all
select 13643,'MRN' from dual union all
select 13644,'ESRO' from dual union all
select 13645,'HARP' from dual union all
select 13646,'MRN' from dual union all
select 13647,'MRN' from dual union all
select 13648,'AFCRL' from dual union all
select 13649,'NASA' from dual union all
select 1365,'ARDC' from dual union all
select 13650,'MRN' from dual union all
select 13651,'NASA' from dual union all
select 13652,'GMS' from dual union all
select 13653,'NASA' from dual union all
select 13654,'NASA' from dual union all
select 13655,'USAF' from dual union all
select 13656,'USA' from dual union all
select 13657,'CNES' from dual union all
select 13658,'SAND' from dual union all
select 13659,'NASA' from dual union all
select 13659,'USAF' from dual union all
select 1366,'USAF' from dual union all
select 13660,'NASA' from dual union all
select 13660,'USAF' from dual union all
select 13661,'GMS' from dual union all
select 13662,'RVSN' from dual union all
select 13663,'GMS' from dual union all
select 13664,'ORDRSA' from dual union all
select 13665,'ORDRSA' from dual union all
select 13666,'ORDRSA' from dual union all
select 13667,'ISAS' from dual union all
select 13668,'ORDRSA' from dual union all
select 13669,'ORDRSA' from dual union all
select 1367,'USA' from dual union all
select 13670,'ORDRSA' from dual union all
select 13671,'ORDRSA' from dual union all
select 13672,'ORDRSA' from dual union all
select 13673,'ORDRSA' from dual union all
select 13674,'ORDRSA' from dual union all
select 13675,'ORDRSA' from dual union all
select 13676,'ORDRSA' from dual union all
select 13677,'ORDRSA' from dual union all
select 13678,'ISAS' from dual union all
select 13679,'ORDRSA' from dual union all
select 1368,'ORDRSA' from dual union all
select 13680,'ORDRSA' from dual union all
select 13681,'ORDRSA' from dual union all
select 13682,'ORDRSA' from dual union all
select 13683,'WRE' from dual union all
select 13684,'WRE' from dual union all
select 13685,'LRS' from dual union all
select 13686,'SAND' from dual union all
select 13687,'SAMSO' from dual union all
select 13688,'IIAE' from dual union all
select 13689,'USA' from dual;
insert into LAUNCH_AGENCY
select 1369,'ORDRSA' from dual union all
select 13690,'RVSN' from dual union all
select 13691,'USA' from dual union all
select 13692,'USA' from dual union all
select 13693,'USA' from dual union all
select 13694,'USA' from dual union all
select 13695,'USA' from dual union all
select 13696,'USA' from dual union all
select 13697,'MICOM' from dual union all
select 13698,'NASA' from dual union all
select 13698,'USAF' from dual union all
select 13699,'NASA' from dual union all
select 13699,'USAF' from dual union all
select 137,'WER836-2' from dual union all
select 1370,'USAF' from dual union all
select 13700,'NASA' from dual union all
select 13701,'NASA' from dual union all
select 13701,'USAF' from dual union all
select 13702,'ESRO' from dual union all
select 13703,'RVSN' from dual union all
select 13704,'GMS' from dual union all
select 13705,'USN' from dual union all
select 13706,'USA' from dual union all
select 13707,'NASA' from dual union all
select 13707,'USAF' from dual union all
select 13708,'USAF' from dual union all
select 13709,'NASA' from dual union all
select 13709,'USAF' from dual union all
select 1371,'USA' from dual union all
select 13710,'RVSN' from dual union all
select 13711,'GMS' from dual union all
select 13712,'USAF' from dual union all
select 13713,'NASA' from dual union all
select 13713,'USAF' from dual union all
select 13714,'NASA' from dual union all
select 13714,'USAF' from dual union all
select 13715,'USA' from dual union all
select 13716,'USA' from dual union all
select 13717,'NASA' from dual union all
select 13717,'USAF' from dual union all
select 13718,'NASA' from dual union all
select 13718,'USAF' from dual union all
select 13719,'NASA' from dual union all
select 13719,'USAF' from dual union all
select 1372,'USA' from dual union all
select 13720,'ISRO' from dual union all
select 13721,'NASA' from dual union all
select 13721,'USAF' from dual union all
select 13722,'NASA' from dual union all
select 13722,'USAF' from dual union all
select 13723,'NASA' from dual union all
select 13724,'USA' from dual union all
select 13725,'NASA' from dual union all
select 13725,'USAF' from dual union all
select 13726,'ISAS' from dual union all
select 13727,'ISAS' from dual union all
select 13728,'NASA' from dual union all
select 13728,'USAF' from dual union all
select 13729,'SAND' from dual union all
select 1373,'PARD' from dual union all
select 13730,'NASA' from dual union all
select 13730,'USAF' from dual union all
select 13731,'ISAS' from dual union all
select 13732,'RVSN' from dual union all
select 13733,'RVSN' from dual union all
select 13734,'S4300' from dual union all
select 13735,'RVSN' from dual union all
select 13736,'RVSN' from dual union all
select 13737,'RVSN' from dual union all
select 13738,'USAF' from dual union all
select 13739,'RVSN' from dual union all
select 1374,'USA' from dual union all
select 13740,'USAF' from dual union all
select 13741,'USAF' from dual union all
select 13742,'RVSN' from dual union all
select 13743,'ISAS' from dual union all
select 13744,'RVSN' from dual union all
select 13745,'RVSN' from dual union all
select 13746,'ISAS' from dual union all
select 13747,'DMA' from dual union all
select 13748,'LRL' from dual union all
select 13748,'USAF' from dual union all
select 13749,'KPNO' from dual union all
select 1375,'USAF' from dual union all
select 13750,'AFCRL' from dual union all
select 13751,'HARP' from dual union all
select 13752,'HARP' from dual union all
select 13753,'HARP' from dual union all
select 13754,'DMA' from dual union all
select 13755,'HARP' from dual union all
select 13756,'HARP' from dual union all
select 13757,'AFCRL' from dual union all
select 13758,'AFCRL' from dual union all
select 13759,'AFCRL' from dual union all
select 1376,'USA' from dual union all
select 13760,'AFCRL' from dual union all
select 13761,'AFCRL' from dual union all
select 13762,'AFCRL' from dual union all
select 13763,'AFCRL' from dual union all
select 13764,'AFCRL' from dual;
insert into LAUNCH_AGENCY
select 13765,'AFCRL' from dual union all
select 13766,'AFCRL' from dual union all
select 13767,'HARP' from dual union all
select 13768,'AFCRL' from dual union all
select 13769,'AFCRL' from dual union all
select 1377,'PARD' from dual union all
select 13770,'AFCRL' from dual union all
select 13771,'AFCRL' from dual union all
select 13772,'AFCRL' from dual union all
select 13773,'AFCRL' from dual union all
select 13774,'AFCRL' from dual union all
select 13775,'AFCRL' from dual union all
select 13776,'AFCRL' from dual union all
select 13777,'AFCRL' from dual union all
select 13778,'HARP' from dual union all
select 13779,'AFCRL' from dual union all
select 1378,'USA' from dual union all
select 13780,'AFCRL' from dual union all
select 13781,'AFCRL' from dual union all
select 13782,'AFCRL' from dual union all
select 13783,'AFCRL' from dual union all
select 13784,'AFCRL' from dual union all
select 13785,'AFCRL' from dual union all
select 13786,'AFCRL' from dual union all
select 13787,'AFCRL' from dual union all
select 13788,'MRN' from dual union all
select 13789,'HARP' from dual union all
select 1379,'USA' from dual union all
select 13790,'MRN' from dual union all
select 13791,'MRN' from dual union all
select 13792,'INTA' from dual union all
select 13793,'INTA' from dual union all
select 13794,'INTA' from dual union all
select 13795,'INTA' from dual union all
select 13796,'INTA' from dual union all
select 13797,'INTA' from dual union all
select 13798,'INTA' from dual union all
select 13799,'INTA' from dual union all
select 138,'WER836-3' from dual union all
select 1380,'PARD' from dual union all
select 13800,'HARP' from dual union all
select 13801,'INTA' from dual union all
select 13802,'ERDA' from dual union all
select 13803,'ERDA' from dual union all
select 13804,'MSFC' from dual union all
select 13805,'MSFC' from dual union all
select 13806,'BWEHR' from dual union all
select 13807,'HARP' from dual union all
select 13808,'AFCRL' from dual union all
select 13809,'ISAS' from dual union all
select 1381,'PARD' from dual union all
select 13810,'SAND' from dual union all
select 13811,'DMA' from dual union all
select 13812,'DMA' from dual union all
select 13813,'USA' from dual union all
select 13814,'USA' from dual union all
select 13815,'DMA' from dual union all
select 13816,'CNES' from dual union all
select 13817,'DMA' from dual union all
select 13818,'ESRO' from dual union all
select 13819,'NASA' from dual union all
select 13819,'USAF' from dual union all
select 1382,'USAF' from dual union all
select 13820,'ESRO' from dual union all
select 13821,'NASA' from dual union all
select 13821,'USAF' from dual union all
select 13822,'NASA' from dual union all
select 13822,'USAF' from dual union all
select 13823,'NASA' from dual union all
select 13823,'USAF' from dual union all
select 13824,'NASA' from dual union all
select 13824,'USAF' from dual union all
select 13825,'NASA' from dual union all
select 13825,'USAF' from dual union all
select 13826,'NASA' from dual union all
select 13826,'USAF' from dual union all
select 13827,'NASA' from dual union all
select 13827,'USAF' from dual union all
select 13828,'AFCRL' from dual union all
select 13829,'AFSC' from dual union all
select 1383,'USA' from dual union all
select 13830,'AFCRL' from dual union all
select 13831,'BRL' from dual union all
select 13831,'DASA' from dual union all
select 13832,'DMA' from dual union all
select 13833,'BRL' from dual union all
select 13833,'DASA' from dual union all
select 13834,'BRL' from dual union all
select 13834,'DASA' from dual union all
select 13835,'BRL' from dual union all
select 13835,'DASA' from dual union all
select 13836,'BRL' from dual union all
select 13836,'DASA' from dual union all
select 13837,'ONERA' from dual union all
select 13838,'CNES' from dual union all
select 13839,'HARP' from dual union all
select 1384,'USAF' from dual union all
select 13840,'HARP' from dual union all
select 13841,'HARP' from dual union all
select 13842,'HARP' from dual;
insert into LAUNCH_AGENCY
select 13843,'HARP' from dual union all
select 13844,'HARP' from dual union all
select 13845,'HARP' from dual union all
select 13846,'HARP' from dual union all
select 13847,'HARP' from dual union all
select 13848,'HARP' from dual union all
select 13849,'HARP' from dual union all
select 1385,'USA' from dual union all
select 13850,'HARP' from dual union all
select 13851,'HARP' from dual union all
select 13852,'HARP' from dual union all
select 13853,'HARP' from dual union all
select 13854,'HARP' from dual union all
select 13855,'HARP' from dual union all
select 13856,'HARP' from dual union all
select 13857,'HARP' from dual union all
select 13858,'HARP' from dual union all
select 13859,'HARP' from dual union all
select 1386,'USA' from dual union all
select 13860,'HARP' from dual union all
select 13861,'HARP' from dual union all
select 13862,'HARP' from dual union all
select 13863,'HARP' from dual union all
select 13864,'HARP' from dual union all
select 13865,'HARP' from dual union all
select 13866,'HARP' from dual union all
select 13867,'HARP' from dual union all
select 13868,'HARP' from dual union all
select 13869,'HARP' from dual union all
select 1387,'USAF' from dual union all
select 13870,'HARP' from dual union all
select 13871,'HARP' from dual union all
select 13872,'HARP' from dual union all
select 13873,'HARP' from dual union all
select 13874,'HARP' from dual union all
select 13875,'HARP' from dual union all
select 13876,'HARP' from dual union all
select 13877,'HARP' from dual union all
select 13878,'HARP' from dual union all
select 13879,'HARP' from dual union all
select 1388,'PARD' from dual union all
select 13880,'HARP' from dual union all
select 13881,'HARP' from dual union all
select 13882,'HARP' from dual union all
select 13883,'HARP' from dual union all
select 13884,'HARP' from dual union all
select 13885,'HARP' from dual union all
select 13886,'HARP' from dual union all
select 13887,'HARP' from dual union all
select 13888,'HARP' from dual union all
select 13889,'HARP' from dual union all
select 1389,'ORDRSA' from dual union all
select 13890,'HARP' from dual union all
select 13891,'HARP' from dual union all
select 13892,'PRO' from dual union all
select 13893,'PRO' from dual union all
select 13894,'PRO' from dual union all
select 13895,'PRO' from dual union all
select 13896,'PRO' from dual union all
select 13897,'DASST' from dual union all
select 13898,'DASST' from dual union all
select 13899,'DASST' from dual union all
select 139,'WER836-3' from dual union all
select 1390,'ORDRSA' from dual union all
select 13900,'USAF' from dual union all
select 13901,'USAF' from dual union all
select 13902,'NASA' from dual union all
select 13903,'NASA' from dual union all
select 13904,'NASA' from dual union all
select 13905,'NASA' from dual union all
select 13906,'AFSC' from dual union all
select 13907,'NASA' from dual union all
select 13908,'NASA' from dual union all
select 13909,'NASA' from dual union all
select 1391,'ORDRSA' from dual union all
select 13910,'AFSC' from dual union all
select 13911,'1STRAD' from dual union all
select 13912,'RVSN' from dual union all
select 13913,'AFSC' from dual union all
select 13914,'79FA2' from dual union all
select 13915,'AFSC' from dual union all
select 13916,'USN' from dual union all
select 13917,'USAF' from dual union all
select 13918,'SAC' from dual union all
select 13919,'NASA' from dual union all
select 1392,'ORDRSA' from dual union all
select 13920,'NASA' from dual union all
select 13921,'AFCRL' from dual union all
select 13922,'AFSC' from dual union all
select 13923,'RVSN' from dual union all
select 13924,'AFCRL' from dual union all
select 13925,'NASA' from dual union all
select 13926,'AFCRL' from dual union all
select 13927,'NRL' from dual union all
select 13928,'NRL' from dual union all
select 13929,'NRL' from dual union all
select 1393,'USA' from dual union all
select 13930,'ESRO' from dual union all
select 13931,'WRE' from dual union all
select 13932,'ESRO' from dual;
insert into LAUNCH_AGENCY
select 13933,'1STRAD' from dual union all
select 13934,'RVSN' from dual union all
select 13935,'1STRAD' from dual union all
select 13936,'USN' from dual union all
select 13937,'USN' from dual union all
select 13938,'USN' from dual union all
select 13939,'NASA' from dual union all
select 1394,'USA' from dual union all
select 13940,'RVSN' from dual union all
select 13941,'NASA' from dual union all
select 13942,'NASA' from dual union all
select 13943,'MICOM' from dual union all
select 13944,'MICOM' from dual union all
select 13945,'RVSN' from dual union all
select 13946,'SAMSO' from dual union all
select 13947,'USAF' from dual union all
select 13948,'USN' from dual union all
select 13949,'USN' from dual union all
select 1395,'USA' from dual union all
select 13950,'USN' from dual union all
select 13951,'NASA' from dual union all
select 13952,'NASA' from dual union all
select 13953,'USN' from dual union all
select 13954,'NASA' from dual union all
select 13955,'CNES' from dual union all
select 13956,'RVSN' from dual union all
select 13957,'AFSC' from dual union all
select 13958,'ISAS' from dual union all
select 13959,'1STRAD' from dual union all
select 1396,'USAF' from dual union all
select 13960,'NASA' from dual union all
select 13961,'KPNO' from dual union all
select 13962,'ESRO' from dual union all
select 13963,'DASA' from dual union all
select 13964,'USN' from dual union all
select 13965,'AFCRL' from dual union all
select 13966,'ESRO' from dual union all
select 13967,'ONERA' from dual union all
select 13968,'1STRAD' from dual union all
select 13969,'USAF' from dual union all
select 1397,'USAF' from dual union all
select 13970,'ONERA' from dual union all
select 13971,'NASA' from dual union all
select 13972,'USN' from dual union all
select 13973,'USN' from dual union all
select 13974,'USN' from dual union all
select 13975,'USN' from dual union all
select 13976,'DASST' from dual union all
select 13977,'RAE' from dual union all
select 13977,'WRE' from dual union all
select 13978,'RVSN' from dual union all
select 13979,'AFSC' from dual union all
select 1398,'USAF' from dual union all
select 13980,'USN' from dual union all
select 13981,'USN' from dual union all
select 13982,'RVSN' from dual union all
select 13983,'CNES' from dual union all
select 13984,'AFSC' from dual union all
select 13985,'USN' from dual union all
select 13986,'ISAS' from dual union all
select 13987,'1STRAD' from dual union all
select 13988,'NASA' from dual union all
select 13988,'NDRE' from dual union all
select 13989,'RVSN' from dual union all
select 1399,'USAF' from dual union all
select 13990,'WRE' from dual union all
select 13991,'NASA' from dual union all
select 13992,'RVSN' from dual union all
select 13993,'ISRO' from dual union all
select 13994,'NASA' from dual union all
select 13995,'ISRO' from dual union all
select 13996,'CNES' from dual union all
select 13997,'ISRO' from dual union all
select 13998,'1STRAD' from dual union all
select 13999,'ISRO' from dual union all
select 14,'WEHR' from dual union all
select 140,'WER836-3' from dual union all
select 1400,'PARD' from dual union all
select 14000,'RVSN' from dual union all
select 14001,'SAC' from dual union all
select 14002,'USN' from dual union all
select 14003,'NRCC' from dual union all
select 14004,'NASA' from dual union all
select 14005,'ONERA' from dual union all
select 14006,'RVSN' from dual union all
select 14007,'NRCC' from dual union all
select 14008,'AFCRL' from dual union all
select 14009,'MICOM' from dual union all
select 1401,'USA' from dual union all
select 14010,'MICOM' from dual union all
select 14011,'RVSN' from dual union all
select 14012,'USAF' from dual union all
select 14013,'USAF' from dual union all
select 14014,'USN' from dual union all
select 14015,'USAF' from dual union all
select 14016,'USAF' from dual union all
select 14017,'AFSC' from dual union all
select 14018,'RVSN' from dual union all
select 14019,'1STRAD' from dual union all
select 1402,'USA' from dual;
insert into LAUNCH_AGENCY
select 14020,'1STRAD' from dual union all
select 14021,'NASA' from dual union all
select 14022,'RVSN' from dual union all
select 14023,'ONERA' from dual union all
select 14024,'SAC' from dual union all
select 14025,'CNES' from dual union all
select 14026,'USN' from dual union all
select 14027,'USN' from dual union all
select 14028,'NASA' from dual union all
select 14029,'RVSN' from dual union all
select 1403,'USA' from dual union all
select 14030,'USN' from dual union all
select 14031,'NASA' from dual union all
select 14032,'NASA' from dual union all
select 14033,'RVSN' from dual union all
select 14034,'NASA' from dual union all
select 14035,'1STRAD' from dual union all
select 14036,'RVSN' from dual union all
select 14037,'RAE' from dual union all
select 14037,'WRE' from dual union all
select 14038,'SAC' from dual union all
select 14039,'ISAS' from dual union all
select 1404,'USA' from dual union all
select 14040,'USN' from dual union all
select 14041,'USN' from dual union all
select 14042,'USN' from dual union all
select 14043,'RAE' from dual union all
select 14043,'WRE' from dual union all
select 14044,'RVSN' from dual union all
select 14045,'SAND' from dual union all
select 14046,'NASA' from dual union all
select 14047,'1STRAD' from dual union all
select 14048,'CNES' from dual union all
select 14049,'RVSN' from dual union all
select 1405,'USAF' from dual union all
select 14050,'RVSN' from dual union all
select 14051,'NASA' from dual union all
select 14052,'NASA' from dual union all
select 14053,'AFCRL' from dual union all
select 14054,'RVSN' from dual union all
select 14055,'MICOM' from dual union all
select 14056,'USAF' from dual union all
select 14057,'USAF' from dual union all
select 14058,'USAF' from dual union all
select 14059,'AFCRL' from dual union all
select 1406,'USA' from dual union all
select 14060,'NASA' from dual union all
select 14061,'1STRAD' from dual union all
select 14062,'NASA' from dual union all
select 14063,'NASA' from dual union all
select 14064,'NASA' from dual union all
select 14065,'RVSN' from dual union all
select 14066,'AFSC' from dual union all
select 14067,'NASA' from dual union all
select 14068,'NASA' from dual union all
select 14069,'NASA' from dual union all
select 1407,'USA' from dual union all
select 14070,'CNES' from dual union all
select 14071,'NASA' from dual union all
select 14072,'RAE' from dual union all
select 14072,'WRE' from dual union all
select 14073,'USA' from dual union all
select 14074,'NASA' from dual union all
select 14075,'SAND' from dual union all
select 14076,'RVSN' from dual union all
select 14077,'USAF' from dual union all
select 14078,'CNES' from dual union all
select 14079,'AFSC' from dual union all
select 1408,'USAF' from dual union all
select 14080,'ESRO' from dual union all
select 14081,'1STRAD' from dual union all
select 14082,'1STRAD' from dual union all
select 14083,'1STRAD' from dual union all
select 14084,'WRE' from dual union all
select 14085,'NASA' from dual union all
select 14086,'ESRO' from dual union all
select 14087,'RVSN' from dual union all
select 14088,'ESRO' from dual union all
select 14089,'NASA' from dual union all
select 1409,'USA' from dual union all
select 14090,'NASA' from dual union all
select 14091,'NASA' from dual union all
select 14092,'SAND' from dual union all
select 14093,'CNES' from dual union all
select 14094,'ELDO' from dual union all
select 14095,'SAND' from dual union all
select 14096,'SAC' from dual union all
select 14097,'NASA' from dual union all
select 14098,'RVSN' from dual union all
select 14099,'SAND' from dual union all
select 141,'WER836-3' from dual union all
select 1410,'PARD' from dual union all
select 14100,'AFSC' from dual union all
select 14101,'CNES' from dual union all
select 14102,'USAF' from dual union all
select 14103,'USAF' from dual union all
select 14104,'RVSN' from dual union all
select 14105,'1STRAD' from dual union all
select 14106,'1STRAD' from dual union all
select 14107,'MICOM' from dual;
insert into LAUNCH_AGENCY
select 14108,'MICOM' from dual union all
select 14109,'RVSN' from dual union all
select 1411,'PARD' from dual union all
select 14110,'USAF' from dual union all
select 14111,'84FA3' from dual union all
select 14112,'44FA2' from dual union all
select 14113,'RVSN' from dual union all
select 14114,'NMC' from dual union all
select 14115,'RAE' from dual union all
select 14115,'WRE' from dual union all
select 14116,'CNES' from dual union all
select 14117,'1STRAD' from dual union all
select 14118,'RAE' from dual union all
select 14118,'WRE' from dual union all
select 14119,'RVSN' from dual union all
select 1412,'USAF' from dual union all
select 14120,'RVSN' from dual union all
select 14121,'NASA' from dual union all
select 14122,'RVSN' from dual union all
select 14123,'SAND' from dual union all
select 14124,'SAND' from dual union all
select 14125,'RAE' from dual union all
select 14125,'WRE' from dual union all
select 14126,'1STRAD' from dual union all
select 14127,'USN' from dual union all
select 14128,'RVSN' from dual union all
select 14129,'USAF' from dual union all
select 1413,'USA' from dual union all
select 14130,'AFSC' from dual union all
select 14131,'RVSN' from dual union all
select 14132,'SAND' from dual union all
select 14133,'SAND' from dual union all
select 14134,'SAND' from dual union all
select 14135,'SAND' from dual union all
select 14136,'CNES' from dual union all
select 14137,'NASA' from dual union all
select 14138,'CNES' from dual union all
select 14139,'WRE' from dual union all
select 1414,'LRBA' from dual union all
select 14140,'CNES' from dual union all
select 14141,'CNES' from dual union all
select 14142,'RVSN' from dual union all
select 14143,'SAND' from dual union all
select 14144,'SAND' from dual union all
select 14145,'ESRO' from dual union all
select 14146,'NASA' from dual union all
select 14147,'NASA' from dual union all
select 14148,'AFCRL' from dual union all
select 14149,'ESRO' from dual union all
select 1415,'LRBA' from dual union all
select 14150,'ESRO' from dual union all
select 14151,'79FA2' from dual union all
select 14152,'ONERA' from dual union all
select 14153,'RVSN' from dual union all
select 14154,'USN' from dual union all
select 14155,'USAF' from dual union all
select 14156,'RAE' from dual union all
select 14156,'WRE' from dual union all
select 14157,'NASA' from dual union all
select 14158,'NASA' from dual union all
select 14159,'AFCRL' from dual union all
select 1416,'PARD' from dual union all
select 14160,'AFSC' from dual union all
select 14161,'RVSN' from dual union all
select 14162,'AFCRL' from dual union all
select 14163,'AFCRL' from dual union all
select 14164,'RVSN' from dual union all
select 14165,'AFCRL' from dual union all
select 14166,'AFCRL' from dual union all
select 14167,'RVSN' from dual union all
select 14168,'AFSC' from dual union all
select 14169,'NDRE' from dual union all
select 1417,'PARD' from dual union all
select 14170,'CNES' from dual union all
select 14171,'ESRO' from dual union all
select 14172,'USAF' from dual union all
select 14173,'CNIE' from dual union all
select 14174,'USAF' from dual union all
select 14175,'RVSN' from dual union all
select 14176,'USAF' from dual union all
select 14177,'MICOM' from dual union all
select 14178,'MICOM' from dual union all
select 14179,'MICOM' from dual union all
select 1418,'ORDRSA' from dual union all
select 14180,'MICOM' from dual union all
select 14181,'RVSN' from dual union all
select 14182,'RVSN' from dual union all
select 14183,'AFSC' from dual union all
select 14184,'RVSN' from dual union all
select 14185,'USAF' from dual union all
select 14186,'RVSN' from dual union all
select 14187,'NASA' from dual union all
select 14188,'ESRO' from dual union all
select 14189,'AFSC' from dual union all
select 1419,'ORDRSA' from dual union all
select 14190,'1STRAD' from dual union all
select 14191,'USN' from dual union all
select 14192,'NASA' from dual union all
select 14193,'1STRAD' from dual union all
select 14194,'NASA' from dual;
insert into LAUNCH_AGENCY
select 14195,'1STRAD' from dual union all
select 14196,'ESRO' from dual union all
select 14197,'RVSN' from dual union all
select 14198,'USAF' from dual union all
select 14199,'SIMED' from dual union all
select 142,'WER836-1' from dual union all
select 1420,'USAF' from dual union all
select 14200,'NASA' from dual union all
select 14201,'NASA' from dual union all
select 14202,'NASA' from dual union all
select 14203,'ISAS' from dual union all
select 14204,'NASA' from dual union all
select 14205,'NASA' from dual union all
select 14206,'NASA' from dual union all
select 14207,'NASA' from dual union all
select 14208,'RVSN' from dual union all
select 14209,'NASA' from dual union all
select 14209,'NRL' from dual union all
select 1421,'USA' from dual union all
select 14210,'NRCC' from dual union all
select 14211,'WRE' from dual union all
select 14212,'NASA' from dual union all
select 14213,'NRCC' from dual union all
select 14214,'SAC' from dual union all
select 14215,'AFCRL' from dual union all
select 14216,'ISAS' from dual union all
select 14217,'USN' from dual union all
select 14218,'AFSC' from dual union all
select 14219,'RVSN' from dual union all
select 1422,'ORDRSA' from dual union all
select 14220,'1STRAD' from dual union all
select 14221,'FKGR21' from dual union all
select 14222,'FKGR21' from dual union all
select 14223,'US7A' from dual union all
select 14224,'US7A' from dual union all
select 14225,'US7A' from dual union all
select 14226,'US7A' from dual union all
select 14227,'NRL' from dual union all
select 14228,'SIMED' from dual union all
select 14229,'RVSN' from dual union all
select 1423,'USA' from dual union all
select 14230,'RAE' from dual union all
select 14230,'WRE' from dual union all
select 14231,'SAND' from dual union all
select 14232,'SAND' from dual union all
select 14233,'USAF' from dual union all
select 14234,'MICOM' from dual union all
select 14235,'MICOM' from dual union all
select 14236,'MICOM' from dual union all
select 14237,'1STRAD' from dual union all
select 14238,'USAF' from dual union all
select 14239,'USAF' from dual union all
select 1424,'ORDRSA' from dual union all
select 14240,'RVSN' from dual union all
select 14241,'SAND' from dual union all
select 14242,'SAND' from dual union all
select 14243,'NMC' from dual union all
select 14244,'ISAS' from dual union all
select 14245,'SAND' from dual union all
select 14246,'SAND' from dual union all
select 14247,'SAND' from dual union all
select 14248,'1STRAD' from dual union all
select 14249,'USAF' from dual union all
select 1425,'PARD' from dual union all
select 14250,'RVSN' from dual union all
select 14251,'USAF' from dual union all
select 14252,'USAF' from dual union all
select 14253,'NASA' from dual union all
select 14254,'NASA' from dual union all
select 14255,'ISRO' from dual union all
select 14256,'NASA' from dual union all
select 14257,'NASA' from dual union all
select 14258,'NASA' from dual union all
select 14259,'NASA' from dual union all
select 1426,'USN' from dual union all
select 14260,'NASA' from dual union all
select 14261,'RVSN' from dual union all
select 14262,'AFSC' from dual union all
select 14263,'AFCRL' from dual union all
select 14264,'RVSN' from dual union all
select 14265,'ISAS' from dual union all
select 14266,'SAND' from dual union all
select 14267,'ISAS' from dual union all
select 14268,'AFCRL' from dual union all
select 14269,'CNIE' from dual union all
select 1427,'USA' from dual union all
select 14270,'NASA' from dual union all
select 14271,'NASA' from dual union all
select 14272,'RVSN' from dual union all
select 14273,'NASA' from dual union all
select 14274,'NASA' from dual union all
select 14275,'1STRAD' from dual union all
select 14276,'1STRAD' from dual union all
select 14277,'NASA' from dual union all
select 14278,'MPE' from dual union all
select 14279,'NASA' from dual union all
select 1428,'PARD' from dual union all
select 14280,'USAF' from dual union all
select 14281,'USAF' from dual union all
select 14282,'MPE' from dual;
insert into LAUNCH_AGENCY
select 14283,'RVSN' from dual union all
select 14284,'1STRAD' from dual union all
select 14285,'NASA' from dual union all
select 14286,'NASA' from dual union all
select 14287,'RVSN' from dual union all
select 14288,'NASA' from dual union all
select 14289,'1STRAD' from dual union all
select 1429,'ORDRSA' from dual union all
select 14290,'NASA' from dual union all
select 14291,'NASA' from dual union all
select 14292,'NASA' from dual union all
select 14293,'RVSN' from dual union all
select 14294,'NASA' from dual union all
select 14295,'NRCC' from dual union all
select 14296,'NASA' from dual union all
select 14297,'NASA' from dual union all
select 14298,'WRE' from dual union all
select 14299,'1STRAD' from dual union all
select 143,'WEHR' from dual union all
select 1430,'ORDRSA' from dual union all
select 14300,'NASA' from dual union all
select 14301,'WRE' from dual union all
select 14302,'CNIE' from dual union all
select 14303,'MICOM' from dual union all
select 14304,'RVSN' from dual union all
select 14305,'MICOM' from dual union all
select 14306,'USAF' from dual union all
select 14307,'USAF' from dual union all
select 14308,'USAF' from dual union all
select 14309,'SAND' from dual union all
select 1431,'ORDRSA' from dual union all
select 14310,'USN' from dual union all
select 14311,'USAF' from dual union all
select 14312,'NASA' from dual union all
select 14313,'USAF' from dual union all
select 14314,'NASA' from dual union all
select 14315,'RVSN' from dual union all
select 14316,'NASA' from dual union all
select 14317,'NRL' from dual union all
select 14318,'NASA' from dual union all
select 14319,'NASA' from dual union all
select 1432,'USA' from dual union all
select 14320,'NASA' from dual union all
select 14321,'NASA' from dual union all
select 14322,'NASA' from dual union all
select 14323,'1STRAD' from dual union all
select 14324,'SAC' from dual union all
select 14325,'NASA' from dual union all
select 14326,'RVSN' from dual union all
select 14327,'NASA' from dual union all
select 14328,'NASA' from dual union all
select 14329,'NASA' from dual union all
select 1433,'USA' from dual union all
select 14330,'NASA' from dual union all
select 14331,'SAND' from dual union all
select 14332,'1STRAD' from dual union all
select 14333,'USN' from dual union all
select 14334,'USN' from dual union all
select 14335,'NASA' from dual union all
select 14336,'SAND' from dual union all
select 14337,'RVSN' from dual union all
select 14338,'1STRAD' from dual union all
select 14339,'USAF' from dual union all
select 1434,'ARDC' from dual union all
select 14340,'SAND' from dual union all
select 14341,'NASA' from dual union all
select 14342,'AN' from dual union all
select 14343,'NASA' from dual union all
select 14344,'RVSN' from dual union all
select 14345,'WRE' from dual union all
select 14346,'USN' from dual union all
select 14347,'USN' from dual union all
select 14348,'RVSN' from dual union all
select 14349,'AFCRL' from dual union all
select 1435,'LRBA' from dual union all
select 14350,'USA' from dual union all
select 14351,'USA' from dual union all
select 14352,'FKGR22' from dual union all
select 14353,'AN' from dual union all
select 14354,'NASA' from dual union all
select 14355,'NASA' from dual union all
select 14356,'RVSN' from dual union all
select 14357,'NASA' from dual union all
select 14358,'NASA' from dual union all
select 14359,'RVSN' from dual union all
select 1436,'NRL' from dual union all
select 1436,'US' from dual union all
select 14360,'AN' from dual union all
select 14361,'NASA' from dual union all
select 14362,'NASA' from dual union all
select 14363,'NASA' from dual union all
select 14364,'NASA' from dual union all
select 14365,'NASA' from dual union all
select 14366,'1STRAD' from dual union all
select 14367,'CNES' from dual union all
select 14368,'KPNO' from dual union all
select 14368,'NRL' from dual union all
select 14369,'ONERA' from dual union all
select 1437,'SUI' from dual union all
select 14370,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 14371,'RVSN' from dual union all
select 14372,'USAF' from dual union all
select 14373,'NASA' from dual union all
select 14374,'USAF' from dual union all
select 14375,'CNES' from dual union all
select 14376,'USN' from dual union all
select 14377,'AN' from dual union all
select 14378,'AFCRL' from dual union all
select 14379,'KPNO' from dual union all
select 1438,'SUI' from dual union all
select 14380,'NASA' from dual union all
select 14381,'RVSN' from dual union all
select 14382,'1STRAD' from dual union all
select 14383,'AN' from dual union all
select 14384,'USAF' from dual union all
select 14385,'USAF' from dual union all
select 14386,'AFCRL' from dual union all
select 14387,'CNES' from dual union all
select 14388,'ISAS' from dual union all
select 14389,'RVSN' from dual union all
select 1439,'SUI' from dual union all
select 14390,'RVSN' from dual union all
select 14391,'ISAS' from dual union all
select 14392,'RVSN' from dual union all
select 14393,'ISAS' from dual union all
select 14394,'AN' from dual union all
select 14395,'1STRAD' from dual union all
select 14396,'FKGR12' from dual union all
select 14397,'US7A' from dual union all
select 14398,'US7A' from dual union all
select 14399,'RVSN' from dual union all
select 144,'WER836-1' from dual union all
select 1440,'SUI' from dual union all
select 14400,'NASA' from dual union all
select 14401,'RVSN' from dual union all
select 14402,'RAE' from dual union all
select 14402,'WRE' from dual union all
select 14403,'RVSN' from dual union all
select 14404,'RAE' from dual union all
select 14404,'WRE' from dual union all
select 14405,'NASA' from dual union all
select 14406,'NASA' from dual union all
select 14407,'USAF' from dual union all
select 14408,'AN' from dual union all
select 14409,'RVSN' from dual union all
select 1441,'ARDC' from dual union all
select 14410,'PLA2AC' from dual union all
select 14411,'RAE' from dual union all
select 14411,'WRE' from dual union all
select 14412,'RAE' from dual union all
select 14412,'WRE' from dual union all
select 14413,'RAE' from dual union all
select 14413,'WRE' from dual union all
select 14414,'RVSN' from dual union all
select 14415,'ONERA' from dual union all
select 14416,'RVSN' from dual union all
select 14417,'USAF' from dual union all
select 14418,'FKGR22' from dual union all
select 14419,'USN' from dual union all
select 1442,'NRL' from dual union all
select 14420,'USN' from dual union all
select 14421,'AFCRL' from dual union all
select 14422,'RVSN' from dual union all
select 14423,'WRE' from dual union all
select 14424,'NASA' from dual union all
select 14424,'USAF' from dual union all
select 14425,'AFSC' from dual union all
select 14426,'ONERA' from dual union all
select 14427,'1STRAD' from dual union all
select 14428,'CNIE' from dual union all
select 14429,'RVSN' from dual union all
select 1443,'ARDC' from dual union all
select 14430,'ISAS' from dual union all
select 14431,'RVSN' from dual union all
select 14432,'AN' from dual union all
select 14433,'AFCRL' from dual union all
select 14434,'USAF' from dual union all
select 14435,'USAF' from dual union all
select 14436,'ISRO' from dual union all
select 14437,'NASA' from dual union all
select 14438,'CNIE' from dual union all
select 14439,'USAF' from dual union all
select 1444,'ARDC' from dual union all
select 14440,'USAF' from dual union all
select 14441,'WRE' from dual union all
select 14442,'USAF' from dual union all
select 14443,'NASA' from dual union all
select 14444,'CNIE' from dual union all
select 14445,'CNIE' from dual union all
select 14446,'CNIE' from dual union all
select 14447,'NASA' from dual union all
select 14448,'CNES' from dual union all
select 14449,'NASA' from dual union all
select 14449,'NRL' from dual union all
select 1445,'NRL' from dual union all
select 14450,'SAND' from dual union all
select 14451,'CNES' from dual union all
select 14452,'SAND' from dual union all
select 14453,'NASA' from dual union all
select 14454,'NASA' from dual;
insert into LAUNCH_AGENCY
select 14455,'NASA' from dual union all
select 14456,'NASA' from dual union all
select 14457,'SAND' from dual union all
select 14458,'USA' from dual union all
select 14459,'AFCRL' from dual union all
select 1446,'NRL' from dual union all
select 14460,'AFCRL' from dual union all
select 14461,'AFCRL' from dual union all
select 14462,'AFCRL' from dual union all
select 14463,'NASA' from dual union all
select 14464,'NASA' from dual union all
select 14465,'USN' from dual union all
select 14466,'USN' from dual union all
select 14467,'CNES' from dual union all
select 14468,'CNES' from dual union all
select 14469,'USN' from dual union all
select 1447,'ARDC' from dual union all
select 14470,'ELDO' from dual union all
select 14471,'AFCRL' from dual union all
select 14472,'USAF' from dual union all
select 14473,'1STRAD' from dual union all
select 14474,'AFCRL' from dual union all
select 14475,'USN' from dual union all
select 14476,'RVSN' from dual union all
select 14477,'USA' from dual union all
select 14478,'USA' from dual union all
select 14479,'USA' from dual union all
select 1448,'NRL' from dual union all
select 14480,'CNES' from dual union all
select 14481,'USA' from dual union all
select 14482,'USA' from dual union all
select 14483,'RAE' from dual union all
select 14483,'WRE' from dual union all
select 14484,'NASA' from dual union all
select 14485,'NRCC' from dual union all
select 14486,'USA' from dual union all
select 14487,'USA' from dual union all
select 14488,'USA' from dual union all
select 14489,'USA' from dual union all
select 1449,'USA' from dual union all
select 14490,'USA' from dual union all
select 14491,'NASA' from dual union all
select 14492,'USA' from dual union all
select 14493,'USA' from dual union all
select 14494,'ESRO' from dual union all
select 14495,'USA' from dual union all
select 14496,'USA' from dual union all
select 14497,'USA' from dual union all
select 14498,'USA' from dual union all
select 14499,'RVSN' from dual union all
select 145,'WER836-1' from dual union all
select 1450,'SCEL' from dual union all
select 1450,'US' from dual union all
select 14500,'NASA' from dual union all
select 14501,'1STRAD' from dual union all
select 14502,'AFSC' from dual union all
select 14503,'AN' from dual union all
select 14504,'RVSN' from dual union all
select 14505,'SAC' from dual union all
select 14506,'CNES' from dual union all
select 14507,'RVSN' from dual union all
select 14508,'AFCRL' from dual union all
select 14509,'USN' from dual union all
select 1451,'ARDC' from dual union all
select 14510,'USN' from dual union all
select 14511,'USA' from dual union all
select 14512,'ESRO' from dual union all
select 14513,'NASA' from dual union all
select 14514,'NRCC' from dual union all
select 14515,'NRCC' from dual union all
select 14516,'USA' from dual union all
select 14517,'NASA' from dual union all
select 14518,'NASA' from dual union all
select 14519,'ESRO' from dual union all
select 1452,'ARDC' from dual union all
select 14520,'AN' from dual union all
select 14521,'NASA' from dual union all
select 14522,'ISAS' from dual union all
select 14523,'AFCRL' from dual union all
select 14524,'NASA' from dual union all
select 14525,'AN' from dual union all
select 14526,'RVSN' from dual union all
select 14527,'RVSN' from dual union all
select 14528,'CNES' from dual union all
select 14529,'NASA' from dual union all
select 1453,'GAU4' from dual union all
select 14530,'ISAS' from dual union all
select 14531,'NASA' from dual union all
select 14532,'ISAS' from dual union all
select 14533,'AN' from dual union all
select 14534,'NRCC' from dual union all
select 14535,'NASA' from dual union all
select 14536,'ONERA' from dual union all
select 14537,'AFSC' from dual union all
select 14538,'NASA' from dual union all
select 14539,'USA' from dual union all
select 1454,'GAU4' from dual union all
select 14540,'RAE' from dual union all
select 14540,'WRE' from dual union all
select 14541,'AFCRL' from dual;
insert into LAUNCH_AGENCY
select 14542,'USAF' from dual union all
select 14543,'NRL' from dual union all
select 14544,'USAF' from dual union all
select 14545,'NRCC' from dual union all
select 14546,'NASA' from dual union all
select 14547,'AN' from dual union all
select 14548,'NRCC' from dual union all
select 14549,'USAF' from dual union all
select 1455,'GAU4' from dual union all
select 14550,'1STRAD' from dual union all
select 14551,'AN' from dual union all
select 14552,'AN' from dual union all
select 14553,'AN' from dual union all
select 14554,'1STRAD' from dual union all
select 14555,'1STRAD' from dual union all
select 14556,'CALT' from dual union all
select 14557,'NASA' from dual union all
select 14558,'CNIE' from dual union all
select 14559,'USAF' from dual union all
select 1456,'GAU4' from dual union all
select 14560,'USAF' from dual union all
select 14561,'SAND' from dual union all
select 14562,'SAND' from dual union all
select 14563,'SAND' from dual union all
select 14564,'HARP' from dual union all
select 14565,'HARP' from dual union all
select 14566,'HARP' from dual union all
select 14567,'HARP' from dual union all
select 14568,'USAF' from dual union all
select 14569,'HARP' from dual union all
select 1457,'GAU4' from dual union all
select 14570,'HARP' from dual union all
select 14571,'HARP' from dual union all
select 14572,'HARP' from dual union all
select 14573,'HARP' from dual union all
select 14574,'HARP' from dual union all
select 14575,'HARP' from dual union all
select 14576,'HARP' from dual union all
select 14577,'HARP' from dual union all
select 14578,'HARP' from dual union all
select 14579,'USAF' from dual union all
select 1458,'GAU4' from dual union all
select 14580,'HARP' from dual union all
select 14581,'HARP' from dual union all
select 14582,'HARP' from dual union all
select 14583,'HARP' from dual union all
select 14584,'HARP' from dual union all
select 14585,'HARP' from dual union all
select 14586,'HARP' from dual union all
select 14587,'RVSN' from dual union all
select 14588,'RVSN' from dual union all
select 14589,'SAMSO' from dual union all
select 1459,'GAU4' from dual union all
select 14590,'CNES' from dual union all
select 14591,'SAMSO' from dual union all
select 14592,'SAMSO' from dual union all
select 14593,'SAMSO' from dual union all
select 14594,'SAMSO' from dual union all
select 14595,'SAMSO' from dual union all
select 14596,'USA' from dual union all
select 14597,'USA' from dual union all
select 14598,'USA' from dual union all
select 14599,'MICOM' from dual union all
select 146,'WER836-1' from dual union all
select 1460,'GAU4' from dual union all
select 14600,'MICOM' from dual union all
select 14601,'AFSC' from dual union all
select 14602,'MICOM' from dual union all
select 14603,'MICOM' from dual union all
select 14604,'MICOM' from dual union all
select 14605,'MICOM' from dual union all
select 14606,'MICOM' from dual union all
select 14607,'LRS' from dual union all
select 14608,'RAE' from dual union all
select 14609,'RVSN' from dual union all
select 1461,'GAU4' from dual union all
select 14610,'RVSN' from dual union all
select 14611,'79FA2' from dual union all
select 14612,'AFCRL' from dual union all
select 14613,'SAMSO' from dual union all
select 14614,'USAF' from dual union all
select 14615,'USA' from dual union all
select 14616,'MICOM' from dual union all
select 14617,'RVSN' from dual union all
select 14618,'RVSN' from dual union all
select 14619,'RVSN' from dual union all
select 1462,'GAU4' from dual union all
select 14620,'RVSN' from dual union all
select 14621,'RVSN' from dual union all
select 14622,'RVSN' from dual union all
select 14623,'AN' from dual union all
select 14624,'RVSN' from dual union all
select 14625,'RVSN' from dual union all
select 14626,'RVSN' from dual union all
select 14627,'RVSN' from dual union all
select 14628,'RVSN' from dual union all
select 14629,'RVSN' from dual union all
select 1463,'GAU4' from dual union all
select 14630,'RVSN' from dual union all
select 14631,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 14632,'RVSN' from dual union all
select 14633,'RVSN' from dual union all
select 14634,'NASA' from dual union all
select 14635,'RVSN' from dual union all
select 14636,'AFCRL' from dual union all
select 14637,'AFCRL' from dual union all
select 14638,'RVSN' from dual union all
select 14639,'RVSN' from dual union all
select 1464,'SUI' from dual union all
select 14640,'MVS' from dual union all
select 14641,'MVS' from dual union all
select 14642,'MVS' from dual union all
select 14643,'MVS' from dual union all
select 14644,'RVSN' from dual union all
select 14645,'NASA' from dual union all
select 14646,'MVS' from dual union all
select 14647,'MVS' from dual union all
select 14648,'MVS' from dual union all
select 14649,'MVS' from dual union all
select 1465,'GAU4' from dual union all
select 14650,'MVS' from dual union all
select 14651,'MVS' from dual union all
select 14652,'MVS' from dual union all
select 14653,'MVS' from dual union all
select 14654,'MVS' from dual union all
select 14655,'RVSN' from dual union all
select 14656,'WRE' from dual union all
select 14657,'RVSN' from dual union all
select 14658,'RVSN' from dual union all
select 14659,'RVSN' from dual union all
select 1466,'GAU4' from dual union all
select 14660,'RVSN' from dual union all
select 14661,'RVSN' from dual union all
select 14662,'RVSN' from dual union all
select 14663,'SAND' from dual union all
select 14664,'SAND' from dual union all
select 14665,'1STRAD' from dual union all
select 14666,'NRL' from dual union all
select 14667,'FKGR13' from dual union all
select 14668,'79FA2' from dual union all
select 14669,'USN' from dual union all
select 1467,'GAU4' from dual union all
select 14670,'RVSN' from dual union all
select 14671,'NRL' from dual union all
select 14672,'NASA' from dual union all
select 14673,'NASA' from dual union all
select 14674,'NASA' from dual union all
select 14675,'NASA' from dual union all
select 14676,'RAE' from dual union all
select 14676,'WRE' from dual union all
select 14677,'USAF' from dual union all
select 14678,'SAC' from dual union all
select 14679,'NASA' from dual union all
select 1468,'GAU4' from dual union all
select 14680,'NASA' from dual union all
select 14681,'ESRO' from dual union all
select 14682,'SAND' from dual union all
select 14683,'RVSN' from dual union all
select 14684,'RVSN' from dual union all
select 14685,'NASA' from dual union all
select 14686,'NASA' from dual union all
select 14687,'NASA' from dual union all
select 14688,'NASA' from dual union all
select 14689,'NASA' from dual union all
select 1469,'GAU4' from dual union all
select 14690,'AFSC' from dual union all
select 14691,'INPE' from dual union all
select 14692,'USAF' from dual union all
select 14693,'USAF' from dual union all
select 14694,'USAF' from dual union all
select 14695,'USAF' from dual union all
select 14696,'USAF' from dual union all
select 14697,'USAF' from dual union all
select 14698,'GMS' from dual union all
select 14699,'MSFC' from dual union all
select 147,'WER836-1' from dual union all
select 1470,'NRL' from dual union all
select 14700,'ERDA' from dual union all
select 14701,'MRN' from dual union all
select 14702,'MRN' from dual union all
select 14703,'MRN' from dual union all
select 14704,'MRN' from dual union all
select 14705,'MRN' from dual union all
select 14706,'MRN' from dual union all
select 14707,'MRN' from dual union all
select 14708,'MRN' from dual union all
select 14709,'MRN' from dual union all
select 1471,'GAU4' from dual union all
select 14710,'MRN' from dual union all
select 14711,'ERDA' from dual union all
select 14712,'MRN' from dual union all
select 14713,'MRN' from dual union all
select 14714,'MRN' from dual union all
select 14715,'UKMO' from dual union all
select 14716,'MRN' from dual union all
select 14717,'MRN' from dual union all
select 14718,'MRN' from dual union all
select 14719,'MRN' from dual union all
select 1472,'USA' from dual union all
select 14720,'MRN' from dual;
insert into LAUNCH_AGENCY
select 14721,'MRN' from dual union all
select 14722,'GMS' from dual union all
select 14723,'MRN' from dual union all
select 14724,'UKMO' from dual union all
select 14725,'MRN' from dual union all
select 14726,'MRN' from dual union all
select 14727,'MRN' from dual union all
select 14728,'ISRO' from dual union all
select 14729,'SUPA' from dual union all
select 1473,'ARDC' from dual union all
select 14730,'SUPA' from dual union all
select 14731,'MRN' from dual union all
select 14732,'MRN' from dual union all
select 14733,'MSFC' from dual union all
select 14734,'MRN' from dual union all
select 14735,'MRN' from dual union all
select 14736,'MRN' from dual union all
select 14737,'MRN' from dual union all
select 14738,'MRN' from dual union all
select 14739,'MRN' from dual union all
select 1474,'USA' from dual union all
select 14740,'MRN' from dual union all
select 14741,'MRN' from dual union all
select 14742,'MRN' from dual union all
select 14743,'MRN' from dual union all
select 14744,'NASA' from dual union all
select 14745,'MRN' from dual union all
select 14746,'MRN' from dual union all
select 14747,'MRN' from dual union all
select 14748,'MRN' from dual union all
select 14749,'MRN' from dual union all
select 1475,'ARDC' from dual union all
select 14750,'MRN' from dual union all
select 14751,'MRN' from dual union all
select 14752,'MRN' from dual union all
select 14753,'MRN' from dual union all
select 14754,'MRN' from dual union all
select 14755,'SUPA' from dual union all
select 14756,'UKMO' from dual union all
select 14757,'MRN' from dual union all
select 14758,'MSFC' from dual union all
select 14759,'MRN' from dual union all
select 1476,'GAU4' from dual union all
select 14760,'MRN' from dual union all
select 14761,'MRN' from dual union all
select 14762,'MRN' from dual union all
select 14763,'MRN' from dual union all
select 14764,'MRN' from dual union all
select 14765,'MRN' from dual union all
select 14766,'CNAE' from dual union all
select 14767,'MRN' from dual union all
select 14768,'MRN' from dual union all
select 14769,'MRN' from dual union all
select 1477,'GAU4' from dual union all
select 14770,'MRN' from dual union all
select 14771,'UKMO' from dual union all
select 14772,'MSFC' from dual union all
select 14773,'MRN' from dual union all
select 14774,'MRN' from dual union all
select 14775,'MRN' from dual union all
select 14776,'MRN' from dual union all
select 14777,'MRN' from dual union all
select 14778,'MRN' from dual union all
select 14779,'MRN' from dual union all
select 1478,'GAU4' from dual union all
select 14780,'MRN' from dual union all
select 14781,'MRN' from dual union all
select 14782,'MRN' from dual union all
select 14783,'MRN' from dual union all
select 14784,'MRN' from dual union all
select 14785,'MRN' from dual union all
select 14786,'UKMO' from dual union all
select 14787,'MRN' from dual union all
select 14788,'GMS' from dual union all
select 14789,'MRN' from dual union all
select 1479,'GAU4' from dual union all
select 14790,'MRN' from dual union all
select 14791,'MRN' from dual union all
select 14792,'MRN' from dual union all
select 14793,'MRN' from dual union all
select 14794,'MRN' from dual union all
select 14795,'MRN' from dual union all
select 14796,'MRN' from dual union all
select 14797,'MSFC' from dual union all
select 14798,'MRN' from dual union all
select 14799,'ISAS' from dual union all
select 148,'WER836-1' from dual union all
select 1480,'USA' from dual union all
select 14800,'MRN' from dual union all
select 14801,'MRN' from dual union all
select 14802,'MRN' from dual union all
select 14803,'MRN' from dual union all
select 14804,'MRN' from dual union all
select 14805,'MSFC' from dual union all
select 14806,'NASA' from dual union all
select 14807,'MRN' from dual union all
select 14808,'UKMO' from dual union all
select 14809,'MRN' from dual union all
select 1481,'ARDC' from dual union all
select 14810,'MSFC' from dual;
insert into LAUNCH_AGENCY
select 14811,'MRN' from dual union all
select 14812,'USAF' from dual union all
select 14813,'USAF' from dual union all
select 14814,'USAF' from dual union all
select 14815,'USAF' from dual union all
select 14816,'USAF' from dual union all
select 14817,'USAF' from dual union all
select 14818,'USAF' from dual union all
select 14819,'USAF' from dual union all
select 1482,'USN' from dual union all
select 14820,'USAF' from dual union all
select 14821,'ISAS' from dual union all
select 14822,'MRN' from dual union all
select 14823,'MRN' from dual union all
select 14824,'MRN' from dual union all
select 14825,'MRN' from dual union all
select 14826,'MRN' from dual union all
select 14827,'UKMO' from dual union all
select 14828,'MRN' from dual union all
select 14829,'INPE' from dual union all
select 1483,'GAU4' from dual union all
select 14830,'UKMO' from dual union all
select 14831,'UKMO' from dual union all
select 14832,'ISRO' from dual union all
select 14833,'UKMO' from dual union all
select 14834,'GMS' from dual union all
select 14835,'UKMO' from dual union all
select 14836,'GMS' from dual union all
select 14837,'ISRO' from dual union all
select 14838,'INPE' from dual union all
select 14839,'SUPA' from dual union all
select 1484,'GAU4' from dual union all
select 14840,'SUPA' from dual union all
select 14841,'AFCRL' from dual union all
select 14842,'USAF' from dual union all
select 14843,'GMS' from dual union all
select 14844,'USAF' from dual union all
select 14845,'USAF' from dual union all
select 14846,'ISRO' from dual union all
select 14847,'SUPA' from dual union all
select 14848,'SRC' from dual union all
select 14849,'GMS' from dual union all
select 1485,'GAU4' from dual union all
select 14850,'UKMO' from dual union all
select 14851,'GMS' from dual union all
select 14852,'CNIE' from dual union all
select 14853,'USAF' from dual union all
select 14854,'USAF' from dual union all
select 14855,'USAF' from dual union all
select 14856,'USAF' from dual union all
select 14857,'USAF' from dual union all
select 14858,'USAF' from dual union all
select 14859,'USAF' from dual union all
select 1486,'GAU4' from dual union all
select 14860,'USAF' from dual union all
select 14861,'ISAS' from dual union all
select 14862,'ISAS' from dual union all
select 14863,'INPE' from dual union all
select 14864,'ISAS' from dual union all
select 14865,'GMS' from dual union all
select 14866,'USAF' from dual union all
select 14867,'USAF' from dual union all
select 14868,'NASA' from dual union all
select 14869,'NASA' from dual union all
select 1487,'GAU4' from dual union all
select 14870,'NASA' from dual union all
select 14871,'NASA' from dual union all
select 14872,'GMS' from dual union all
select 14873,'SUPA' from dual union all
select 14874,'INPE' from dual union all
select 14875,'CONAE' from dual union all
select 14876,'SUPA' from dual union all
select 14877,'CONAE' from dual union all
select 14878,'US' from dual union all
select 14879,'USAF' from dual union all
select 1488,'PARD' from dual union all
select 14880,'USAF' from dual union all
select 14881,'USAF' from dual union all
select 14882,'USAF' from dual union all
select 14883,'USAF' from dual union all
select 14884,'USAF' from dual union all
select 14885,'USAF' from dual union all
select 14886,'USAF' from dual union all
select 14887,'MRN' from dual union all
select 14888,'USAF' from dual union all
select 14889,'INPE' from dual union all
select 1489,'PARD' from dual union all
select 14890,'UKMO' from dual union all
select 14891,'GMS' from dual union all
select 14892,'GMS' from dual union all
select 14893,'SUPA' from dual union all
select 14894,'CONAE' from dual union all
select 14895,'USAF' from dual union all
select 14896,'USAF' from dual union all
select 14897,'USAF' from dual union all
select 14898,'GMS' from dual union all
select 14899,'USAF' from dual union all
select 149,'WER836-1' from dual union all
select 1490,'PARD' from dual union all
select 14900,'USAF' from dual;
insert into LAUNCH_AGENCY
select 14901,'USAF' from dual union all
select 14902,'USAF' from dual union all
select 14903,'USAF' from dual union all
select 14904,'USAF' from dual union all
select 14905,'USAF' from dual union all
select 14906,'USAF' from dual union all
select 14907,'MSFC' from dual union all
select 14908,'MSFC' from dual union all
select 14909,'MSFC' from dual union all
select 1491,'USA' from dual union all
select 14910,'SUPA' from dual union all
select 14911,'CONAE' from dual union all
select 14912,'CONAE' from dual union all
select 14913,'CONAE' from dual union all
select 14914,'GMS' from dual union all
select 14915,'USAF' from dual union all
select 14916,'USAF' from dual union all
select 14917,'USAF' from dual union all
select 14918,'USAF' from dual union all
select 14919,'USAF' from dual union all
select 1492,'ORDRSA' from dual union all
select 14920,'GMS' from dual union all
select 14921,'USAF' from dual union all
select 14922,'USAF' from dual union all
select 14923,'USAF' from dual union all
select 14924,'USAF' from dual union all
select 14925,'USAF' from dual union all
select 14926,'USAF' from dual union all
select 14927,'SUPA' from dual union all
select 14928,'ISAS' from dual union all
select 14929,'MSFC' from dual union all
select 1493,'USA' from dual union all
select 14930,'MSFC' from dual union all
select 14931,'MRN' from dual union all
select 14932,'GMS' from dual union all
select 14933,'USAF' from dual union all
select 14934,'USAF' from dual union all
select 14935,'USAF' from dual union all
select 14936,'USAF' from dual union all
select 14937,'USAF' from dual union all
select 14938,'USAF' from dual union all
select 14939,'USAF' from dual union all
select 1494,'USA' from dual union all
select 14940,'USAF' from dual union all
select 14941,'USAF' from dual union all
select 14942,'GMS' from dual union all
select 14943,'USAF' from dual union all
select 14944,'USAF' from dual union all
select 14945,'USAF' from dual union all
select 14946,'USAF' from dual union all
select 14947,'USAF' from dual union all
select 14948,'USAF' from dual union all
select 14949,'USAF' from dual union all
select 1495,'USA' from dual union all
select 14950,'USAF' from dual union all
select 14951,'USAF' from dual union all
select 14952,'USAF' from dual union all
select 14953,'MRN' from dual union all
select 14954,'USAF' from dual union all
select 14955,'USAF' from dual union all
select 14956,'USAF' from dual union all
select 14957,'USAF' from dual union all
select 14958,'CONAE' from dual union all
select 14959,'CONAE' from dual union all
select 1496,'ORDRSA' from dual union all
select 14960,'ISAS' from dual union all
select 14961,'CONAE' from dual union all
select 14962,'UKMO' from dual union all
select 14963,'USAF' from dual union all
select 14964,'MRN' from dual union all
select 14965,'AFCRL' from dual union all
select 14966,'USAF' from dual union all
select 14967,'USAF' from dual union all
select 14968,'USAF' from dual union all
select 14969,'USAF' from dual union all
select 1497,'ORDRSA' from dual union all
select 14970,'USAF' from dual union all
select 14971,'USAF' from dual union all
select 14972,'USAF' from dual union all
select 14973,'USAF' from dual union all
select 14974,'USAF' from dual union all
select 14975,'MRN' from dual union all
select 14976,'SUPA' from dual union all
select 14977,'ISAS' from dual union all
select 14978,'CONAE' from dual union all
select 14979,'GMS' from dual union all
select 1498,'USA' from dual union all
select 14980,'USAF' from dual union all
select 14981,'USAF' from dual union all
select 14982,'USAF' from dual union all
select 14983,'USAF' from dual union all
select 14984,'USAF' from dual union all
select 14985,'USAF' from dual union all
select 14986,'MRN' from dual union all
select 14987,'USAF' from dual union all
select 14988,'USAF' from dual union all
select 14989,'GMS' from dual union all
select 1499,'ORDRSA' from dual union all
select 14990,'IAE' from dual union all
select 14991,'IAE' from dual;
insert into LAUNCH_AGENCY
select 14992,'IAE' from dual union all
select 14993,'IAE' from dual union all
select 14994,'IAE' from dual union all
select 14995,'IAE' from dual union all
select 14996,'IAE' from dual union all
select 14997,'GMS' from dual union all
select 14998,'IAE' from dual union all
select 14999,'SUPA' from dual union all
select 15,'WEHR' from dual union all
select 150,'WER836-2' from dual union all
select 1500,'USA' from dual union all
select 15000,'CONAE' from dual union all
select 15001,'CNES' from dual union all
select 15002,'CNES' from dual union all
select 15003,'CNES' from dual union all
select 15004,'USAF' from dual union all
select 15005,'USAF' from dual union all
select 15006,'USAF' from dual union all
select 15007,'USAF' from dual union all
select 15008,'MRN' from dual union all
select 15009,'USAF' from dual union all
select 1501,'USAF' from dual union all
select 15010,'USAF' from dual union all
select 15011,'USAF' from dual union all
select 15012,'USAF' from dual union all
select 15013,'MRN' from dual union all
select 15013,'NASA' from dual union all
select 15014,'MRN' from dual union all
select 15015,'MRN' from dual union all
select 15015,'NASA' from dual union all
select 15016,'MRN' from dual union all
select 15016,'NASA' from dual union all
select 15017,'MRN' from dual union all
select 15018,'MRN' from dual union all
select 15018,'NASA' from dual union all
select 15019,'MRN' from dual union all
select 1502,'ORDRSA' from dual union all
select 15020,'MRN' from dual union all
select 15021,'MRN' from dual union all
select 15021,'NASA' from dual union all
select 15022,'NASA' from dual union all
select 15023,'GMS' from dual union all
select 15024,'CONAE' from dual union all
select 15025,'CONAE' from dual union all
select 15026,'GMS' from dual union all
select 15027,'GMS' from dual union all
select 15028,'GMS' from dual union all
select 15029,'GMS' from dual union all
select 1503,'ORDRSA' from dual union all
select 15030,'MRN' from dual union all
select 15031,'GMS' from dual union all
select 15032,'GMS' from dual union all
select 15033,'GMS' from dual union all
select 15034,'GMS' from dual union all
select 15035,'GMS' from dual union all
select 15036,'GMS' from dual union all
select 15037,'GMS' from dual union all
select 15038,'ERDA' from dual union all
select 15039,'USAF' from dual union all
select 1504,'USA' from dual union all
select 15040,'USAF' from dual union all
select 15041,'MRN' from dual union all
select 15042,'USAF' from dual union all
select 15043,'USAF' from dual union all
select 15044,'USAF' from dual union all
select 15045,'USAF' from dual union all
select 15046,'USAF' from dual union all
select 15047,'USAF' from dual union all
select 15048,'USAF' from dual union all
select 15049,'USAF' from dual union all
select 1505,'ORDRSA' from dual union all
select 15050,'USAF' from dual union all
select 15051,'USAF' from dual union all
select 15052,'MRN' from dual union all
select 15053,'USAF' from dual union all
select 15054,'USAF' from dual union all
select 15055,'USAF' from dual union all
select 15056,'USAF' from dual union all
select 15057,'USAF' from dual union all
select 15058,'USAF' from dual union all
select 15059,'USAF' from dual union all
select 1506,'USA' from dual union all
select 15060,'USAF' from dual union all
select 15061,'USAF' from dual union all
select 15062,'USAF' from dual union all
select 15063,'MRN' from dual union all
select 15064,'USAF' from dual union all
select 15065,'USAF' from dual union all
select 15066,'MRN' from dual union all
select 15067,'MRN' from dual union all
select 15068,'NASA' from dual union all
select 15069,'NASA' from dual union all
select 1507,'USA' from dual union all
select 15070,'NASA' from dual union all
select 15071,'NASA' from dual union all
select 15072,'NASA' from dual union all
select 15073,'NASA' from dual union all
select 15074,'MRN' from dual union all
select 15075,'NASA' from dual union all
select 15076,'NASA' from dual;
insert into LAUNCH_AGENCY
select 15077,'NASA' from dual union all
select 15078,'NASA' from dual union all
select 15079,'NASA' from dual union all
select 1508,'USAF' from dual union all
select 15080,'NASA' from dual union all
select 15081,'NASA' from dual union all
select 15082,'NASA' from dual union all
select 15083,'NASA' from dual union all
select 15084,'NASA' from dual union all
select 15085,'MRN' from dual union all
select 15086,'NASA' from dual union all
select 15087,'NASA' from dual union all
select 15088,'NASA' from dual union all
select 15089,'NASA' from dual union all
select 1509,'USA' from dual union all
select 15090,'NASA' from dual union all
select 15091,'NASA' from dual union all
select 15092,'NASA' from dual union all
select 15093,'NASA' from dual union all
select 15094,'NASA' from dual union all
select 15095,'NASA' from dual union all
select 15096,'MRN' from dual union all
select 15097,'NASA' from dual union all
select 15098,'NASA' from dual union all
select 15099,'NASA' from dual union all
select 151,'WER836-2' from dual union all
select 1510,'USA' from dual union all
select 15100,'INTA' from dual union all
select 15101,'INTA' from dual union all
select 15102,'INTA' from dual union all
select 15103,'ASL' from dual union all
select 15104,'ASL' from dual union all
select 15105,'ASL' from dual union all
select 15106,'GMS' from dual union all
select 15107,'ASL' from dual union all
select 15108,'ASL' from dual union all
select 15109,'ASL' from dual union all
select 1511,'USA' from dual union all
select 15110,'ASL' from dual union all
select 15111,'ASL' from dual union all
select 15112,'ASL' from dual union all
select 15113,'ASL' from dual union all
select 15114,'ASL' from dual union all
select 15115,'ASL' from dual union all
select 15116,'ASL' from dual union all
select 15117,'MRN' from dual union all
select 15118,'ASL' from dual union all
select 15119,'ASL' from dual union all
select 1512,'USA' from dual union all
select 15120,'CONAE' from dual union all
select 15121,'CONAE' from dual union all
select 15122,'CNAE' from dual union all
select 15123,'CNAE' from dual union all
select 15124,'CNAE' from dual union all
select 15125,'INPE' from dual union all
select 15126,'INPE' from dual union all
select 15127,'INPE' from dual union all
select 15128,'MRN' from dual union all
select 15129,'INPE' from dual union all
select 1513,'ORDRSA' from dual union all
select 15130,'INPE' from dual union all
select 15131,'INPE' from dual union all
select 15132,'NASA' from dual union all
select 15133,'NASA' from dual union all
select 15134,'NASA' from dual union all
select 15135,'NASA' from dual union all
select 15136,'NASA' from dual union all
select 15137,'NASA' from dual union all
select 15138,'NASA' from dual union all
select 15139,'USA' from dual union all
select 1514,'ORDRSA' from dual union all
select 15140,'NASA' from dual union all
select 15141,'NASA' from dual union all
select 15142,'NASA' from dual union all
select 15143,'NASA' from dual union all
select 15144,'MRN' from dual union all
select 15145,'MRN' from dual union all
select 15146,'MRN' from dual union all
select 15147,'MRN' from dual union all
select 15148,'MRN' from dual union all
select 15149,'GMS' from dual union all
select 1515,'USA' from dual union all
select 15150,'MRN' from dual union all
select 15151,'MRN' from dual union all
select 15152,'MRN' from dual union all
select 15153,'MRN' from dual union all
select 15154,'MRN' from dual union all
select 15155,'MRN' from dual union all
select 15156,'MRN' from dual union all
select 15157,'MRN' from dual union all
select 15158,'MRN' from dual union all
select 15159,'MRN' from dual union all
select 1516,'ORDRSA' from dual union all
select 15160,'MRN' from dual union all
select 15161,'MRN' from dual union all
select 15162,'MRN' from dual union all
select 15163,'MRN' from dual union all
select 15164,'MRN' from dual union all
select 15165,'MRN' from dual union all
select 15166,'MRN' from dual;
insert into LAUNCH_AGENCY
select 15167,'MRN' from dual union all
select 15168,'MRN' from dual union all
select 15169,'UKMO' from dual union all
select 1517,'GAU4' from dual union all
select 15170,'MRN' from dual union all
select 15171,'MRN' from dual union all
select 15172,'GMS' from dual union all
select 15173,'MRN' from dual union all
select 15174,'MRN' from dual union all
select 15175,'MRN' from dual union all
select 15176,'MRN' from dual union all
select 15177,'MRN' from dual union all
select 15178,'MRN' from dual union all
select 15179,'MRN' from dual union all
select 1518,'USAF' from dual union all
select 15180,'MRN' from dual union all
select 15181,'MRN' from dual union all
select 15182,'MRN' from dual union all
select 15183,'MRN' from dual union all
select 15184,'MRN' from dual union all
select 15185,'MRN' from dual union all
select 15186,'MRN' from dual union all
select 15187,'MRN' from dual union all
select 15188,'MRN' from dual union all
select 15189,'MRN' from dual union all
select 1519,'USA' from dual union all
select 15190,'MRN' from dual union all
select 15191,'MRN' from dual union all
select 15192,'UKMO' from dual union all
select 15193,'GMS' from dual union all
select 15194,'GMS' from dual union all
select 15195,'GMS' from dual union all
select 15196,'GMS' from dual union all
select 15197,'GMS' from dual union all
select 15198,'GMS' from dual union all
select 15199,'GMS' from dual union all
select 152,'WER836-2' from dual union all
select 1520,'USA' from dual union all
select 15200,'NOL' from dual union all
select 15201,'GMS' from dual union all
select 15202,'GMS' from dual union all
select 15203,'NOL' from dual union all
select 15204,'ESRO' from dual union all
select 15205,'GMS' from dual union all
select 15206,'NASA' from dual union all
select 15207,'ESRO' from dual union all
select 15208,'NASA' from dual union all
select 15209,'ESRO' from dual union all
select 1521,'USA' from dual union all
select 15210,'NASA' from dual union all
select 15211,'ESRO' from dual union all
select 15212,'NASA' from dual union all
select 15213,'ESRO' from dual union all
select 15214,'NASA' from dual union all
select 15215,'GMS' from dual union all
select 15216,'NASA' from dual union all
select 15217,'NOL' from dual union all
select 15218,'GMS' from dual union all
select 15219,'NOL' from dual union all
select 1522,'USA' from dual union all
select 15220,'GMS' from dual union all
select 15221,'GMS' from dual union all
select 15222,'GMS' from dual union all
select 15223,'GMS' from dual union all
select 15224,'NASA' from dual union all
select 15225,'GMS' from dual union all
select 15226,'CONAE' from dual union all
select 15227,'GMS' from dual union all
select 15228,'GMS' from dual union all
select 15229,'NASA' from dual union all
select 1523,'USA' from dual union all
select 15230,'NASA' from dual union all
select 15231,'GMS' from dual union all
select 15232,'PSU' from dual union all
select 15233,'GMS' from dual union all
select 15234,'GMS' from dual union all
select 15235,'GMS' from dual union all
select 15236,'INTA' from dual union all
select 15237,'CONAE' from dual union all
select 15237,'NASA' from dual union all
select 15238,'CONAE' from dual union all
select 15238,'NASA' from dual union all
select 15239,'CONAE' from dual union all
select 15239,'NASA' from dual union all
select 1524,'USAF' from dual union all
select 15240,'CONAE' from dual union all
select 15240,'NASA' from dual union all
select 15241,'CONAE' from dual union all
select 15241,'NASA' from dual union all
select 15242,'CONAE' from dual union all
select 15242,'NASA' from dual union all
select 15243,'CONAE' from dual union all
select 15243,'NASA' from dual union all
select 15244,'CONAE' from dual union all
select 15244,'NASA' from dual union all
select 15245,'CONAE' from dual union all
select 15245,'NASA' from dual union all
select 15246,'CONAE' from dual union all
select 15246,'NASA' from dual union all
select 15247,'CONAE' from dual;
insert into LAUNCH_AGENCY
select 15247,'NASA' from dual union all
select 15248,'CONAE' from dual union all
select 15248,'NASA' from dual union all
select 15249,'GMS' from dual union all
select 1525,'GAU4' from dual union all
select 15250,'GMS' from dual union all
select 15251,'AN' from dual union all
select 15252,'GMS' from dual union all
select 15253,'GMS' from dual union all
select 15254,'GMS' from dual union all
select 15255,'GMS' from dual union all
select 15256,'NASA' from dual union all
select 15257,'USAF' from dual union all
select 15258,'USAF' from dual union all
select 15259,'RVSN' from dual union all
select 1526,'USA' from dual union all
select 15260,'RVSN' from dual union all
select 15261,'NASA' from dual union all
select 15262,'AFSC' from dual union all
select 15263,'USAF' from dual union all
select 15264,'RVSN' from dual union all
select 15265,'S4300' from dual union all
select 15266,'CNES' from dual union all
select 15267,'RVSN' from dual union all
select 15268,'RVSN' from dual union all
select 15269,'CNES' from dual union all
select 1527,'USA' from dual union all
select 15270,'USAF' from dual union all
select 15271,'AFSC' from dual union all
select 15272,'RVSN' from dual union all
select 15273,'RVSN' from dual union all
select 15274,'RVSN' from dual union all
select 15275,'NASA' from dual union all
select 15276,'RVSN' from dual union all
select 15277,'RVSN' from dual union all
select 15278,'RVSN' from dual union all
select 15279,'RVSN' from dual union all
select 1528,'USA' from dual union all
select 15280,'RVSN' from dual union all
select 15281,'NASA' from dual union all
select 15282,'RVSN' from dual union all
select 15283,'RVSN' from dual union all
select 15284,'USAF' from dual union all
select 15285,'RVSN' from dual union all
select 15286,'USAF' from dual union all
select 15287,'RVSN' from dual union all
select 15288,'RVSN' from dual union all
select 15289,'AFSC' from dual union all
select 1529,'USA' from dual union all
select 15290,'NASA' from dual union all
select 15291,'NASA' from dual union all
select 15292,'RVSN' from dual union all
select 15293,'CRA' from dual union all
select 15294,'RVSN' from dual union all
select 15295,'USAF' from dual union all
select 15296,'USAF' from dual union all
select 15297,'AFSC' from dual union all
select 15298,'USAF' from dual union all
select 15299,'RVSN' from dual union all
select 153,'WER836-2' from dual union all
select 1530,'ORDRSA' from dual union all
select 15300,'RVSN' from dual union all
select 15301,'RVSN' from dual union all
select 15302,'RVSN' from dual union all
select 15303,'AFSC' from dual union all
select 15304,'RVSN' from dual union all
select 15305,'AFSC' from dual union all
select 15306,'NASA' from dual union all
select 15307,'RVSN' from dual union all
select 15308,'USAF' from dual union all
select 15309,'RVSN' from dual union all
select 1531,'USA' from dual union all
select 15310,'AFSC' from dual union all
select 15311,'RVSN' from dual union all
select 15312,'RVSN' from dual union all
select 15313,'RVSN' from dual union all
select 15314,'RVSN' from dual union all
select 15315,'USAF' from dual union all
select 15316,'RVSN' from dual union all
select 15317,'USAF' from dual union all
select 15318,'RVSN' from dual union all
select 15319,'AFSC' from dual union all
select 1532,'USA' from dual union all
select 15320,'10ADS' from dual union all
select 15321,'USAF' from dual union all
select 15322,'RVSN' from dual union all
select 15323,'NASA' from dual union all
select 15324,'RVSN' from dual union all
select 15325,'NASA' from dual union all
select 15326,'USAF' from dual union all
select 15327,'AFSC' from dual union all
select 15328,'USAF' from dual union all
select 15329,'RVSN' from dual union all
select 1533,'USA' from dual union all
select 15330,'USAF' from dual union all
select 15331,'USAF' from dual union all
select 15332,'RVSN' from dual union all
select 15333,'RVSN' from dual union all
select 15334,'AFSC' from dual union all
select 15335,'10ADS' from dual;
insert into LAUNCH_AGENCY
select 15336,'RVSN' from dual union all
select 15337,'RVSN' from dual union all
select 15338,'NASA' from dual union all
select 15339,'NASA' from dual union all
select 1534,'USA' from dual union all
select 15340,'RVSN' from dual union all
select 15341,'RVSN' from dual union all
select 15342,'USAF' from dual union all
select 15343,'RVSN' from dual union all
select 15344,'RVSN' from dual union all
select 15345,'AFSC' from dual union all
select 15346,'RVSN' from dual union all
select 15347,'AFSC' from dual union all
select 15348,'RVSN' from dual union all
select 15349,'NASA' from dual union all
select 1535,'PARD' from dual union all
select 15350,'RVSN' from dual union all
select 15351,'10ADS' from dual union all
select 15352,'RVSN' from dual union all
select 15353,'RVSN' from dual union all
select 15354,'RVSN' from dual union all
select 15355,'NASA' from dual union all
select 15356,'RVSN' from dual union all
select 15357,'RVSN' from dual union all
select 15358,'AFSC' from dual union all
select 15359,'RVSN' from dual union all
select 1536,'ORDRSA' from dual union all
select 15360,'RVSN' from dual union all
select 15361,'RVSN' from dual union all
select 15362,'RVSN' from dual union all
select 15363,'RVSN' from dual union all
select 15364,'USAF' from dual union all
select 15365,'RVSN' from dual union all
select 15366,'USAF' from dual union all
select 15367,'NASA' from dual union all
select 15368,'NASA' from dual union all
select 15369,'NASA' from dual union all
select 1537,'ORDRSA' from dual union all
select 15370,'RVSN' from dual union all
select 15371,'RVSN' from dual union all
select 15372,'RVSN' from dual union all
select 15373,'USA' from dual union all
select 15374,'RVSN' from dual union all
select 15375,'AFSC' from dual union all
select 15376,'AFSC' from dual union all
select 15377,'USAF' from dual union all
select 15378,'NASA' from dual union all
select 15379,'RVSN' from dual union all
select 1538,'ORDRSA' from dual union all
select 15380,'RVSN' from dual union all
select 15381,'RVSN' from dual union all
select 15382,'RVSN' from dual union all
select 15383,'DASA' from dual union all
select 15384,'PIHM' from dual union all
select 15385,'PIHM' from dual union all
select 15386,'PIHM' from dual union all
select 15387,'PIHM' from dual union all
select 15388,'PIHM' from dual union all
select 15389,'PIHM' from dual union all
select 1539,'USA' from dual union all
select 15390,'PIHM' from dual union all
select 15391,'PIHM' from dual union all
select 15392,'PIHM' from dual union all
select 15393,'USA' from dual union all
select 15394,'RAE' from dual union all
select 15395,'NASA' from dual union all
select 15395,'USAF' from dual union all
select 15396,'USA' from dual union all
select 15397,'NASA' from dual union all
select 15398,'NASA' from dual union all
select 15398,'USAF' from dual union all
select 15399,'NASA' from dual union all
select 15399,'USAF' from dual union all
select 154,'WEHR' from dual union all
select 1540,'USA' from dual union all
select 15400,'NASA' from dual union all
select 15400,'NTNF' from dual union all
select 15401,'NASA' from dual union all
select 15402,'NASA' from dual union all
select 15402,'USAF' from dual union all
select 15403,'PIHM' from dual union all
select 15404,'USAF' from dual union all
select 15405,'NASA' from dual union all
select 15405,'USAF' from dual union all
select 15406,'RVSN' from dual union all
select 15407,'NASA' from dual union all
select 15408,'DASA' from dual union all
select 15409,'RAE' from dual union all
select 1541,'USA' from dual union all
select 15410,'NASA' from dual union all
select 15411,'NASA' from dual union all
select 15412,'NASA' from dual union all
select 15413,'USA' from dual union all
select 15414,'PIHM' from dual union all
select 15415,'NASA' from dual union all
select 15415,'USAF' from dual union all
select 15416,'NASA' from dual union all
select 15417,'RVSN' from dual union all
select 15418,'NASA' from dual union all
select 15419,'NASA' from dual;
insert into LAUNCH_AGENCY
select 15419,'USAF' from dual union all
select 1542,'ORDRSA' from dual union all
select 15420,'NASA' from dual union all
select 15421,'NASA' from dual union all
select 15422,'USA' from dual union all
select 15423,'ESRO' from dual union all
select 15424,'CNES' from dual union all
select 15425,'PIHM' from dual union all
select 15426,'AFCRL' from dual union all
select 15427,'AFSC' from dual union all
select 15428,'DASA' from dual union all
select 15429,'USA' from dual union all
select 1543,'ORDRSA' from dual union all
select 15430,'NASA' from dual union all
select 15430,'USAF' from dual union all
select 15431,'USA' from dual union all
select 15432,'ISRO' from dual union all
select 15433,'ISRO' from dual union all
select 15434,'CNES' from dual union all
select 15435,'ISRO' from dual union all
select 15436,'PIHM' from dual union all
select 15437,'USN' from dual union all
select 15438,'AFCRL' from dual union all
select 15439,'RAE' from dual union all
select 1544,'USA' from dual union all
select 15440,'USA' from dual union all
select 15441,'RVSN' from dual union all
select 15442,'CNIE' from dual union all
select 15443,'RVSN' from dual union all
select 15444,'CNES' from dual union all
select 15445,'ONERA' from dual union all
select 15446,'ONERA' from dual union all
select 15447,'PIHM' from dual union all
select 15448,'ONERA' from dual union all
select 15449,'SAND' from dual union all
select 1545,'USA' from dual union all
select 15450,'SAND' from dual union all
select 15451,'AFCRL' from dual union all
select 15452,'AFCRL' from dual union all
select 15453,'AFCRL' from dual union all
select 15454,'AFCRL' from dual union all
select 15455,'USN' from dual union all
select 15456,'RVSN' from dual union all
select 15457,'CH' from dual union all
select 15458,'PIHM' from dual union all
select 15459,'IIAE' from dual union all
select 1546,'USA' from dual union all
select 15460,'USA' from dual union all
select 15461,'USA' from dual union all
select 15462,'USA' from dual union all
select 15463,'PIHM' from dual union all
select 15464,'PIHM' from dual union all
select 15465,'PIHM' from dual union all
select 15466,'PIHM' from dual union all
select 15467,'PIHM' from dual union all
select 15468,'PIHM' from dual union all
select 15469,'PIHM' from dual union all
select 1547,'USA' from dual union all
select 15470,'PIHM' from dual union all
select 15471,'PIHM' from dual union all
select 15472,'PIHM' from dual union all
select 15473,'PIHM' from dual union all
select 15474,'PIHM' from dual union all
select 15475,'PIHM' from dual union all
select 15476,'PIHM' from dual union all
select 15477,'PIHM' from dual union all
select 15478,'PIHM' from dual union all
select 15479,'PIHM' from dual union all
select 1548,'USA' from dual union all
select 15480,'PIHM' from dual union all
select 15481,'PIHM' from dual union all
select 15482,'PIHM' from dual union all
select 15483,'PIHM' from dual union all
select 15484,'PIHM' from dual union all
select 15485,'PIHM' from dual union all
select 15486,'PIHM' from dual union all
select 15487,'PIHM' from dual union all
select 15488,'DMA' from dual union all
select 15489,'STA' from dual union all
select 1549,'USA' from dual union all
select 15490,'GMS' from dual union all
select 15491,'GMS' from dual union all
select 15492,'GMS' from dual union all
select 15493,'GMS' from dual union all
select 15494,'AFCRL' from dual union all
select 15495,'GMS' from dual union all
select 15496,'AFCRL' from dual union all
select 15497,'GMS' from dual union all
select 15498,'GMS' from dual union all
select 15499,'RVSN' from dual union all
select 155,'WER836-2' from dual union all
select 1550,'USA' from dual union all
select 15500,'NASA' from dual union all
select 15501,'NASA' from dual union all
select 15502,'GMS' from dual union all
select 15503,'USN' from dual union all
select 15504,'USAF' from dual union all
select 15505,'GMS' from dual union all
select 15506,'GMS' from dual union all
select 15507,'ISAS' from dual;
insert into LAUNCH_AGENCY
select 15508,'NASA' from dual union all
select 15508,'USAF' from dual union all
select 15509,'CONAE' from dual union all
select 1551,'ORDRSA' from dual union all
select 15510,'CNES' from dual union all
select 15511,'NASA' from dual union all
select 15511,'USAF' from dual union all
select 15512,'NASA' from dual union all
select 15512,'USAF' from dual union all
select 15513,'PIHM' from dual union all
select 15514,'USA' from dual union all
select 15515,'AFCRL' from dual union all
select 15516,'NASA' from dual union all
select 15516,'USAF' from dual union all
select 15517,'NASA' from dual union all
select 15518,'NASA' from dual union all
select 15518,'USAF' from dual union all
select 15519,'USA' from dual union all
select 1552,'USA' from dual union all
select 15520,'RAE' from dual union all
select 15521,'USAF' from dual union all
select 15522,'NASA' from dual union all
select 15522,'USAF' from dual union all
select 15523,'NASA' from dual union all
select 15523,'USAF' from dual union all
select 15524,'NASA' from dual union all
select 15525,'GMS' from dual union all
select 15526,'NASA' from dual union all
select 15526,'USAF' from dual union all
select 15527,'PIHM' from dual union all
select 15528,'NASA' from dual union all
select 15528,'USAF' from dual union all
select 15529,'SAND' from dual union all
select 1553,'ORDRSA' from dual union all
select 15530,'USA' from dual union all
select 15531,'USN' from dual union all
select 15532,'USN' from dual union all
select 15533,'NASA' from dual union all
select 15533,'USAF' from dual union all
select 15534,'USA' from dual union all
select 15535,'NASA' from dual union all
select 15536,'NASA' from dual union all
select 15536,'USAF' from dual union all
select 15537,'NASA' from dual union all
select 15537,'USAF' from dual union all
select 15538,'NASA' from dual union all
select 15539,'NASA' from dual union all
select 15539,'USAF' from dual union all
select 1554,'1PBS' from dual union all
select 15540,'USN' from dual union all
select 15541,'RAE' from dual union all
select 15542,'RVSN' from dual union all
select 15543,'USA' from dual union all
select 15544,'NASA' from dual union all
select 15544,'USAF' from dual union all
select 15545,'PRO' from dual union all
select 15546,'AFSC' from dual union all
select 15547,'RVSN' from dual union all
select 15548,'ISAS' from dual union all
select 15549,'USAF' from dual union all
select 1555,'USA' from dual union all
select 15550,'AFSC' from dual union all
select 15551,'RVSN' from dual union all
select 15552,'RVSN' from dual union all
select 15553,'RVSN' from dual union all
select 15554,'RVSN' from dual union all
select 15555,'RVSN' from dual union all
select 15556,'RVSN' from dual union all
select 15557,'RVSN' from dual union all
select 15558,'DMA' from dual union all
select 15559,'DMA' from dual union all
select 1556,'USA' from dual union all
select 15560,'DMA' from dual union all
select 15561,'DMA' from dual union all
select 15562,'AFSC' from dual union all
select 15563,'DMA' from dual union all
select 15564,'AFCRL' from dual union all
select 15565,'DMA' from dual union all
select 15566,'NASA' from dual union all
select 15566,'USAF' from dual union all
select 15567,'NASA' from dual union all
select 15568,'USA' from dual union all
select 15569,'USA' from dual union all
select 1557,'USA' from dual union all
select 15570,'NASA' from dual union all
select 15570,'USAF' from dual union all
select 15571,'AFCRL' from dual union all
select 15572,'AFCRL' from dual union all
select 15573,'AFCRL' from dual union all
select 15574,'AFCRL' from dual union all
select 15575,'AFCRL' from dual union all
select 15576,'AFCRL' from dual union all
select 15577,'NASA' from dual union all
select 15577,'USAF' from dual union all
select 15578,'AFCRL' from dual union all
select 15579,'AFCRL' from dual union all
select 1558,'USAF' from dual union all
select 15580,'AFCRL' from dual union all
select 15581,'AFCRL' from dual union all
select 15582,'AFCRL' from dual;
insert into LAUNCH_AGENCY
select 15583,'USA' from dual union all
select 15584,'LRL' from dual union all
select 15584,'USAF' from dual union all
select 15585,'LRL' from dual union all
select 15585,'USAF' from dual union all
select 15586,'LRL' from dual union all
select 15586,'USAF' from dual union all
select 15587,'LRL' from dual union all
select 15587,'USAF' from dual union all
select 15588,'DMA' from dual union all
select 15589,'LRL' from dual union all
select 15589,'USAF' from dual union all
select 1559,'USA' from dual union all
select 15590,'LRL' from dual union all
select 15590,'USAF' from dual union all
select 15591,'LRL' from dual union all
select 15591,'USAF' from dual union all
select 15592,'LRL' from dual union all
select 15592,'USAF' from dual union all
select 15593,'IIAE' from dual union all
select 15594,'IIAE' from dual union all
select 15595,'IIAE' from dual union all
select 15596,'AFCRL' from dual union all
select 15597,'AFCRL' from dual union all
select 15598,'SAND' from dual union all
select 15599,'NASA' from dual union all
select 15599,'USAF' from dual union all
select 156,'WER836-3' from dual union all
select 1560,'USA' from dual union all
select 15600,'DMA' from dual union all
select 15601,'USA' from dual union all
select 15602,'USA' from dual union all
select 15603,'CNIE' from dual union all
select 15604,'AFCRL' from dual union all
select 15605,'AFCRL' from dual union all
select 15606,'AFCRL' from dual union all
select 15607,'AFCRL' from dual union all
select 15608,'AFCRL' from dual union all
select 15609,'AFCRL' from dual union all
select 1561,'USA' from dual union all
select 15610,'AFCRL' from dual union all
select 15611,'AFCRL' from dual union all
select 15612,'AFCRL' from dual union all
select 15613,'AFCRL' from dual union all
select 15614,'AFCRL' from dual union all
select 15615,'AFCRL' from dual union all
select 15616,'DASST' from dual union all
select 15617,'DASST' from dual union all
select 15618,'DASST' from dual union all
select 15619,'DASST' from dual union all
select 1562,'USA' from dual union all
select 15620,'DASST' from dual union all
select 15621,'ONERA' from dual union all
select 15622,'ONERA' from dual union all
select 15623,'ONERA' from dual union all
select 15624,'RVSN' from dual union all
select 15625,'RVSN' from dual union all
select 15626,'NASA' from dual union all
select 15627,'NASA' from dual union all
select 15628,'NASA' from dual union all
select 15629,'NASA' from dual union all
select 1563,'PARD' from dual union all
select 15630,'NASA' from dual union all
select 15631,'NASA' from dual union all
select 15632,'NASA' from dual union all
select 15633,'NASA' from dual union all
select 15634,'NASA' from dual union all
select 15635,'NASA' from dual union all
select 15636,'RVSN' from dual union all
select 15637,'NASA' from dual union all
select 15638,'NASA' from dual union all
select 15639,'AFSC' from dual union all
select 1564,'USA' from dual union all
select 15640,'AN' from dual union all
select 15641,'NASA' from dual union all
select 15642,'NRCC' from dual union all
select 15643,'ESRO' from dual union all
select 15644,'ISAS' from dual union all
select 15645,'1STRAD' from dual union all
select 15646,'NASA' from dual union all
select 15647,'RVSN' from dual union all
select 15648,'AFSC' from dual union all
select 15649,'ESRO' from dual union all
select 1565,'USAF' from dual union all
select 15650,'NASA' from dual union all
select 15651,'1STRAD' from dual union all
select 15652,'1STRAD' from dual union all
select 15653,'ESRO' from dual union all
select 15654,'USAF' from dual union all
select 15655,'CNES' from dual union all
select 15656,'AFSC' from dual union all
select 15657,'NASA' from dual union all
select 15658,'RVSN' from dual union all
select 15659,'USA' from dual union all
select 1566,'UZKA' from dual union all
select 15660,'44FA2' from dual union all
select 15661,'SAND' from dual union all
select 15662,'USN' from dual union all
select 15663,'USAF' from dual union all
select 15664,'NASA' from dual;
insert into LAUNCH_AGENCY
select 15665,'NASA' from dual union all
select 15666,'NDRE' from dual union all
select 15667,'RVSN' from dual union all
select 15668,'RVSN' from dual union all
select 15669,'AFCRL' from dual union all
select 1567,'USA' from dual union all
select 15670,'USN' from dual union all
select 15671,'AFCRL' from dual union all
select 15672,'NASA' from dual union all
select 15673,'1STRAD' from dual union all
select 15674,'CNES' from dual union all
select 15675,'ISAS' from dual union all
select 15676,'NASA' from dual union all
select 15677,'INCO' from dual union all
select 15677,'NASA' from dual union all
select 15678,'INCO' from dual union all
select 15678,'NASA' from dual union all
select 15679,'INCO' from dual union all
select 15679,'NASA' from dual union all
select 1568,'USA' from dual union all
select 15680,'US7A' from dual union all
select 15681,'US7A' from dual union all
select 15682,'CNES' from dual union all
select 15683,'INCO' from dual union all
select 15683,'NASA' from dual union all
select 15684,'RAE' from dual union all
select 15684,'WRE' from dual union all
select 15685,'NASA' from dual union all
select 15686,'NDRE' from dual union all
select 15687,'USAF' from dual union all
select 15688,'AFSC' from dual union all
select 15689,'NASA' from dual union all
select 1569,'PARD' from dual union all
select 15690,'USAF' from dual union all
select 15691,'USAF' from dual union all
select 15692,'SAC' from dual union all
select 15693,'NASA' from dual union all
select 15693,'NRL' from dual union all
select 15694,'NASA' from dual union all
select 15695,'1STRAD' from dual union all
select 15696,'NASA' from dual union all
select 15697,'CNES' from dual union all
select 15698,'RVSN' from dual union all
select 15699,'NASA' from dual union all
select 157,'WER836-3' from dual union all
select 1570,'USA' from dual union all
select 15700,'US7A' from dual union all
select 15701,'US7A' from dual union all
select 15702,'US7A' from dual union all
select 15703,'CNES' from dual union all
select 15704,'CNES' from dual union all
select 15705,'USAF' from dual union all
select 15706,'USAF' from dual union all
select 15707,'NASA' from dual union all
select 15708,'RVSN' from dual union all
select 15709,'SECT' from dual union all
select 1571,'USA' from dual union all
select 15710,'RVSN' from dual union all
select 15711,'RVSN' from dual union all
select 15712,'USAF' from dual union all
select 15713,'NASA' from dual union all
select 15714,'NASA' from dual union all
select 15715,'USAF' from dual union all
select 15716,'NASA' from dual union all
select 15717,'NASA' from dual union all
select 15718,'USN' from dual union all
select 15719,'RVSN' from dual union all
select 1572,'USA' from dual union all
select 15720,'CNES' from dual union all
select 15721,'NASA' from dual union all
select 15722,'RAE' from dual union all
select 15722,'WRE' from dual union all
select 15723,'NASA' from dual union all
select 15724,'USAF' from dual union all
select 15725,'USAF' from dual union all
select 15726,'1STRAD' from dual union all
select 15727,'NASA' from dual union all
select 15728,'AFSC' from dual union all
select 15729,'DLR' from dual union all
select 15729,'NASA' from dual union all
select 1573,'USA' from dual union all
select 15730,'RVSN' from dual union all
select 15731,'DLR' from dual union all
select 15731,'NASA' from dual union all
select 15732,'NASA' from dual union all
select 15733,'RAE' from dual union all
select 15733,'WRE' from dual union all
select 15734,'NASA' from dual union all
select 15735,'NASA' from dual union all
select 15736,'NASA' from dual union all
select 15737,'NASA' from dual union all
select 15738,'USAF' from dual union all
select 15739,'SAC' from dual union all
select 1574,'USAF' from dual union all
select 15740,'NASA' from dual union all
select 15741,'RVSN' from dual union all
select 15742,'RAE' from dual union all
select 15742,'WRE' from dual union all
select 15743,'AFCRL' from dual union all
select 15744,'AFCRL' from dual;
insert into LAUNCH_AGENCY
select 15745,'AFCRL' from dual union all
select 15746,'AFCRL' from dual union all
select 15747,'AFCRL' from dual union all
select 15748,'AFCRL' from dual union all
select 15749,'AFSC' from dual union all
select 15749,'ALCS' from dual union all
select 1575,'USA' from dual union all
select 15750,'AFCRL' from dual union all
select 15751,'AFCRL' from dual union all
select 15752,'RVSN' from dual union all
select 15753,'AFCRL' from dual union all
select 15754,'NASA' from dual union all
select 15755,'ESRO' from dual union all
select 15756,'ISRO' from dual union all
select 15757,'AFCRL' from dual union all
select 15758,'USA' from dual union all
select 15759,'NASA' from dual union all
select 1576,'USA' from dual union all
select 15760,'RAE' from dual union all
select 15760,'WRE' from dual union all
select 15761,'1STRAD' from dual union all
select 15762,'1STRAD' from dual union all
select 15763,'RVSN' from dual union all
select 15764,'NASA' from dual union all
select 15765,'NRCC' from dual union all
select 15766,'1STRAD' from dual union all
select 15767,'USN' from dual union all
select 15768,'NRCC' from dual union all
select 15769,'RAE' from dual union all
select 15769,'WRE' from dual union all
select 1577,'USA' from dual union all
select 15770,'AFSC' from dual union all
select 15770,'ALCS' from dual union all
select 15771,'AFCRL' from dual union all
select 15772,'NASA' from dual union all
select 15773,'PRO' from dual union all
select 15774,'RVSN' from dual union all
select 15775,'USAF' from dual union all
select 15776,'NASA' from dual union all
select 15777,'CNES' from dual union all
select 15777,'SUPA' from dual union all
select 15778,'NASA' from dual union all
select 15779,'USAF' from dual union all
select 1578,'USA' from dual union all
select 15780,'USAF' from dual union all
select 15781,'NASA' from dual union all
select 15782,'NRCC' from dual union all
select 15783,'NASA' from dual union all
select 15784,'CNES' from dual union all
select 15785,'RVSN' from dual union all
select 15786,'NASA' from dual union all
select 15787,'CNES' from dual union all
select 15787,'SUPA' from dual union all
select 15788,'AN' from dual union all
select 15789,'NASA' from dual union all
select 1579,'USA' from dual union all
select 15790,'NASA' from dual union all
select 15790,'NRL' from dual union all
select 15791,'AFSC' from dual union all
select 15791,'ALCS' from dual union all
select 15792,'SAND' from dual union all
select 15793,'SAND' from dual union all
select 15794,'NASA' from dual union all
select 15795,'NASA' from dual union all
select 15796,'RVSN' from dual union all
select 15797,'NRL' from dual union all
select 15798,'1STRAD' from dual union all
select 15799,'NASA' from dual union all
select 158,'WER444' from dual union all
select 1580,'USA' from dual union all
select 15800,'SAND' from dual union all
select 15801,'AFSC' from dual union all
select 15802,'1STRAD' from dual union all
select 15803,'AFSC' from dual union all
select 15804,'SAND' from dual union all
select 15805,'1STRAD' from dual union all
select 15806,'ESRO' from dual union all
select 15807,'RVSN' from dual union all
select 15808,'RVSN' from dual union all
select 15809,'AN' from dual union all
select 1581,'USA' from dual union all
select 15810,'ESRO' from dual union all
select 15811,'1STRAD' from dual union all
select 15812,'USAF' from dual union all
select 15813,'SRC' from dual union all
select 15814,'SRC' from dual union all
select 15815,'RVSN' from dual union all
select 15816,'1STRAD' from dual union all
select 15817,'RVSN' from dual union all
select 15818,'USAF' from dual union all
select 15819,'SAND' from dual union all
select 1582,'LRBA' from dual union all
select 15820,'SAND' from dual union all
select 15821,'SAND' from dual union all
select 15822,'SAND' from dual union all
select 15823,'SAND' from dual union all
select 15824,'SAND' from dual union all
select 15825,'US7A' from dual union all
select 15826,'US7A' from dual union all
select 15827,'NASA' from dual;
insert into LAUNCH_AGENCY
select 15828,'RVSN' from dual union all
select 15829,'SAND' from dual union all
select 1583,'USA' from dual union all
select 15830,'SAND' from dual union all
select 15831,'RVSN' from dual union all
select 15832,'NASA' from dual union all
select 15833,'NASA' from dual union all
select 15834,'AFCRL' from dual union all
select 15835,'USAF' from dual union all
select 15836,'US7A' from dual union all
select 15837,'KPNO' from dual union all
select 15838,'AFSC' from dual union all
select 15839,'RVSN' from dual union all
select 1584,'USA' from dual union all
select 15840,'AN' from dual union all
select 15841,'NASA' from dual union all
select 15842,'RVSN' from dual union all
select 15843,'AN' from dual union all
select 15844,'DFVLR' from dual union all
select 15844,'NASA' from dual union all
select 15845,'DFVLR' from dual union all
select 15845,'NASA' from dual union all
select 15846,'AN' from dual union all
select 15847,'AN' from dual union all
select 15848,'AN' from dual union all
select 15849,'1STRAD' from dual union all
select 1585,'USA' from dual union all
select 15850,'RVSN' from dual union all
select 15851,'NASA' from dual union all
select 15852,'NASA' from dual union all
select 15853,'AN' from dual union all
select 15854,'US7A' from dual union all
select 15855,'US7A' from dual union all
select 15856,'USN' from dual union all
select 15857,'USN' from dual union all
select 15858,'USN' from dual union all
select 15859,'USN' from dual union all
select 1586,'USA' from dual union all
select 15860,'SAC' from dual union all
select 15861,'RVSN' from dual union all
select 15862,'NASA' from dual union all
select 15863,'NASA' from dual union all
select 15864,'NASA' from dual union all
select 15865,'RVSN' from dual union all
select 15866,'CNES' from dual union all
select 15867,'AFCRL' from dual union all
select 15868,'AFSC' from dual union all
select 15869,'MICOM' from dual union all
select 1587,'USA' from dual union all
select 15870,'MICOM' from dual union all
select 15871,'USAF' from dual union all
select 15872,'RVSN' from dual union all
select 15873,'SRC' from dual union all
select 15874,'USA' from dual union all
select 15875,'NRCC' from dual union all
select 15876,'1STRAD' from dual union all
select 15877,'NRCC' from dual union all
select 15878,'CNES' from dual union all
select 15879,'AFCRL' from dual union all
select 1588,'USA' from dual union all
select 15880,'AFSC' from dual union all
select 15881,'USN' from dual union all
select 15882,'RVSN' from dual union all
select 15883,'USN' from dual union all
select 15884,'NASA' from dual union all
select 15885,'RVSN' from dual union all
select 15886,'1STRAD' from dual union all
select 15887,'NRCC' from dual union all
select 15888,'NRCC' from dual union all
select 15889,'1STRAD' from dual union all
select 1589,'LRBA' from dual union all
select 15890,'USAF' from dual union all
select 15891,'AFSC' from dual union all
select 15892,'AFSC' from dual union all
select 15893,'RVSN' from dual union all
select 15894,'1STRAD' from dual union all
select 15895,'RVSN' from dual union all
select 15896,'NASA' from dual union all
select 15897,'NASA' from dual union all
select 15898,'AFSC' from dual union all
select 15899,'NASA' from dual union all
select 159,'WER836-1' from dual union all
select 1590,'USAF' from dual union all
select 15900,'USA' from dual union all
select 15901,'RVSN' from dual union all
select 15902,'SAND' from dual union all
select 15903,'AFSC' from dual union all
select 15904,'RVSN' from dual union all
select 15905,'USAF' from dual union all
select 15906,'SAMSO' from dual union all
select 15907,'USN' from dual union all
select 15908,'ESRO' from dual union all
select 15909,'ESRO' from dual union all
select 1591,'ORDRSA' from dual union all
select 15910,'NRCC' from dual union all
select 15911,'NASA' from dual union all
select 15912,'ELDO' from dual union all
select 15913,'NASA' from dual union all
select 15914,'NASA' from dual union all
select 15915,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 15916,'NASA' from dual union all
select 15917,'NRCC' from dual union all
select 15918,'1STRAD' from dual union all
select 15919,'RVSN' from dual union all
select 1592,'ORDRSA' from dual union all
select 15920,'RAE' from dual union all
select 15920,'WRE' from dual union all
select 15921,'NASA' from dual union all
select 15922,'AFCRL' from dual union all
select 15923,'NASA' from dual union all
select 15924,'NASA' from dual union all
select 15925,'RVSN' from dual union all
select 15926,'NASA' from dual union all
select 15927,'NASA' from dual union all
select 15928,'NASA' from dual union all
select 15929,'NASA' from dual union all
select 1593,'USAF' from dual union all
select 15930,'NASA' from dual union all
select 15931,'AN' from dual union all
select 15932,'NRL' from dual union all
select 15933,'AFCRL' from dual union all
select 15934,'RAE' from dual union all
select 15934,'WRE' from dual union all
select 15935,'AFSC' from dual union all
select 15936,'RVSN' from dual union all
select 15937,'USA' from dual union all
select 15938,'USAF' from dual union all
select 15939,'USAF' from dual union all
select 1594,'USA' from dual union all
select 15940,'RAE' from dual union all
select 15940,'WRE' from dual union all
select 15941,'USAF' from dual union all
select 15942,'NASA' from dual union all
select 15942,'NTNF' from dual union all
select 15943,'NASA' from dual union all
select 15944,'AFCRL' from dual union all
select 15945,'NASA' from dual union all
select 15946,'NASA' from dual union all
select 15947,'RVSN' from dual union all
select 15948,'NASA' from dual union all
select 15949,'US7A' from dual union all
select 1595,'ORDRSA' from dual union all
select 15950,'US7A' from dual union all
select 15951,'US7A' from dual union all
select 15952,'US7A' from dual union all
select 15953,'RAE' from dual union all
select 15953,'WRE' from dual union all
select 15954,'NASA' from dual union all
select 15955,'CONAE' from dual union all
select 15956,'PRO' from dual union all
select 15957,'SAMSO' from dual union all
select 15958,'RVSN' from dual union all
select 15959,'FKGR12' from dual union all
select 1596,'ORDRSA' from dual union all
select 15960,'FKGR13' from dual union all
select 15961,'NASA' from dual union all
select 15962,'NASA' from dual union all
select 15963,'CONAE' from dual union all
select 15964,'NASA' from dual union all
select 15965,'NASA' from dual union all
select 15966,'RVSN' from dual union all
select 15967,'1STRAD' from dual union all
select 15968,'RVSN' from dual union all
select 15969,'CNIE' from dual union all
select 1597,'ORDRSA' from dual union all
select 15970,'AFCRL' from dual union all
select 15971,'CNIE' from dual union all
select 15972,'CNIE' from dual union all
select 15973,'NASA' from dual union all
select 15974,'SAND' from dual union all
select 15975,'USN' from dual union all
select 15976,'RVSN' from dual union all
select 15977,'USAF' from dual union all
select 15978,'NASA' from dual union all
select 15979,'RVSN' from dual union all
select 1598,'USAF' from dual union all
select 15980,'NRL' from dual union all
select 15981,'SAC' from dual union all
select 15982,'USAF' from dual union all
select 15983,'US7A' from dual union all
select 15984,'USN' from dual union all
select 15985,'NASA' from dual union all
select 15986,'NASA' from dual union all
select 15987,'USAF' from dual union all
select 15988,'USA' from dual union all
select 15989,'NASA' from dual union all
select 1599,'USAF' from dual union all
select 15990,'RVSN' from dual union all
select 15991,'USAF' from dual union all
select 15992,'USAF' from dual union all
select 15993,'1STRAD' from dual union all
select 15994,'SAND' from dual union all
select 15995,'SAMSO' from dual union all
select 15996,'RVSN' from dual union all
select 15997,'SAND' from dual union all
select 15998,'NASA' from dual union all
select 15999,'USAF' from dual union all
select 16,'WEHR' from dual union all
select 160,'WER836-1' from dual union all
select 1600,'USA' from dual;
insert into LAUNCH_AGENCY
select 16000,'USAF' from dual union all
select 16001,'RVSN' from dual union all
select 16002,'USAF' from dual union all
select 16003,'1STRAD' from dual union all
select 16004,'USAF' from dual union all
select 16005,'USAF' from dual union all
select 16006,'SAND' from dual union all
select 16007,'ESRO' from dual union all
select 16008,'NRCC' from dual union all
select 16009,'SAND' from dual union all
select 1601,'USA' from dual union all
select 16010,'USAF' from dual union all
select 16011,'USAF' from dual union all
select 16012,'RVSN' from dual union all
select 16013,'USAF' from dual union all
select 16014,'NASA' from dual union all
select 16015,'RVSN' from dual union all
select 16016,'RVSN' from dual union all
select 16017,'SAND' from dual union all
select 16018,'AFCRL' from dual union all
select 16019,'SAND' from dual union all
select 1602,'USA' from dual union all
select 16020,'RVSN' from dual union all
select 16021,'SAND' from dual union all
select 16022,'NASA' from dual union all
select 16023,'RVSN' from dual union all
select 16024,'SAND' from dual union all
select 16025,'AFCRL' from dual union all
select 16026,'ESRO' from dual union all
select 16027,'ESRO' from dual union all
select 16028,'AFCRL' from dual union all
select 16029,'NRCC' from dual union all
select 1603,'USA' from dual union all
select 16030,'NASA' from dual union all
select 16031,'ESRO' from dual union all
select 16032,'ESRO' from dual union all
select 16033,'RVSN' from dual union all
select 16034,'RVSN' from dual union all
select 16035,'NASA' from dual union all
select 16035,'NTNF' from dual union all
select 16036,'AN' from dual union all
select 16036,'CNES' from dual union all
select 16037,'ESRO' from dual union all
select 16038,'AN' from dual union all
select 16038,'CNES' from dual union all
select 16039,'SAMSO' from dual union all
select 1604,'USA' from dual union all
select 16040,'USAF' from dual union all
select 16041,'AFSC' from dual union all
select 16042,'USA' from dual union all
select 16043,'SAMSO' from dual union all
select 16044,'RVSN' from dual union all
select 16045,'RVSN' from dual union all
select 16046,'RVSN' from dual union all
select 16047,'NASA' from dual union all
select 16047,'NTNF' from dual union all
select 16048,'AFSC' from dual union all
select 16049,'NASA' from dual union all
select 1605,'ORDRSA' from dual union all
select 16050,'NASA' from dual union all
select 16051,'NASA' from dual union all
select 16052,'NASA' from dual union all
select 16052,'USAF' from dual union all
select 16053,'NASA' from dual union all
select 16054,'NRCC' from dual union all
select 16055,'1STRAD' from dual union all
select 16056,'RVSN' from dual union all
select 16057,'NRCC' from dual union all
select 16058,'DLR' from dual union all
select 16059,'AFCRL' from dual union all
select 1606,'ORDRSA' from dual union all
select 16060,'AFCRL' from dual union all
select 16061,'RVSN' from dual union all
select 16062,'CH' from dual union all
select 16063,'NASA' from dual union all
select 16064,'AFCRL' from dual union all
select 16065,'AFCRL' from dual union all
select 16066,'RVSN' from dual union all
select 16067,'US7A' from dual union all
select 16068,'US7A' from dual union all
select 16069,'US7A' from dual union all
select 1607,'ORDRSA' from dual union all
select 16070,'USA' from dual union all
select 16071,'USN' from dual union all
select 16072,'USN' from dual union all
select 16073,'USN' from dual union all
select 16074,'USN' from dual union all
select 16075,'RAE' from dual union all
select 16075,'WRE' from dual union all
select 16076,'NASA' from dual union all
select 16077,'USN' from dual union all
select 16078,'AN' from dual union all
select 16079,'AFSC' from dual union all
select 1608,'USAF' from dual union all
select 16080,'USN' from dual union all
select 16081,'USN' from dual union all
select 16082,'USN' from dual union all
select 16083,'AFSC' from dual union all
select 16084,'CNIE' from dual union all
select 16085,'AFSC' from dual;
insert into LAUNCH_AGENCY
select 16086,'AFCRL' from dual union all
select 16087,'USAF' from dual union all
select 16088,'AFSC' from dual union all
select 16089,'AFSC' from dual union all
select 1609,'ORDRSA' from dual union all
select 16090,'1STRAD' from dual union all
select 16091,'1STRAD' from dual union all
select 16092,'RAE' from dual union all
select 16092,'WRE' from dual union all
select 16093,'NASA' from dual union all
select 16093,'NRL' from dual union all
select 16094,'NASA' from dual union all
select 16094,'USAF' from dual union all
select 16095,'AFCRL' from dual union all
select 16096,'USAF' from dual union all
select 16097,'NASA' from dual union all
select 16098,'RAE' from dual union all
select 16098,'WRE' from dual union all
select 16099,'CNES' from dual union all
select 161,'WER836-1' from dual union all
select 1610,'USA' from dual union all
select 16100,'AFSC' from dual union all
select 16101,'USAF' from dual union all
select 16102,'NASA' from dual union all
select 16103,'AFSC' from dual union all
select 16104,'USAF' from dual union all
select 16105,'NASA' from dual union all
select 16106,'ESRO' from dual union all
select 16107,'USN' from dual union all
select 16108,'USN' from dual union all
select 16109,'USN' from dual union all
select 1611,'USA' from dual union all
select 16110,'CNES' from dual union all
select 16111,'USN' from dual union all
select 16112,'ESRO' from dual union all
select 16113,'NASA' from dual union all
select 16114,'AFCRL' from dual union all
select 16115,'AFCRL' from dual union all
select 16116,'USAF' from dual union all
select 16117,'1STRAD' from dual union all
select 16118,'RAE' from dual union all
select 16118,'WRE' from dual union all
select 16119,'AFCRL' from dual union all
select 1612,'ORDRSA' from dual union all
select 16120,'AFCRL' from dual union all
select 16121,'AFCRL' from dual union all
select 16122,'KPNO' from dual union all
select 16123,'ELDO' from dual union all
select 16124,'AFCRL' from dual union all
select 16125,'AFCRL' from dual union all
select 16126,'NASA' from dual union all
select 16127,'NASA' from dual union all
select 16128,'NASA' from dual union all
select 16129,'DLR' from dual union all
select 16129,'NASA' from dual union all
select 1613,'USA' from dual union all
select 16130,'NASA' from dual union all
select 16131,'SAND' from dual union all
select 16132,'AFCRL' from dual union all
select 16133,'AFCRL' from dual union all
select 16134,'AFCRL' from dual union all
select 16135,'ESRO' from dual union all
select 16136,'NASA' from dual union all
select 16137,'AFSC' from dual union all
select 16138,'RVSN' from dual union all
select 16139,'NASA' from dual union all
select 1614,'USA' from dual union all
select 16140,'SAMSO' from dual union all
select 16141,'NASA' from dual union all
select 16142,'NASA' from dual union all
select 16143,'AFCRL' from dual union all
select 16144,'AFCRL' from dual union all
select 16145,'AFCRL' from dual union all
select 16146,'AFCRL' from dual union all
select 16147,'AFCRL' from dual union all
select 16148,'AFCRL' from dual union all
select 16149,'AFCRL' from dual union all
select 1615,'USA' from dual union all
select 16150,'AFCRL' from dual union all
select 16151,'1STRAD' from dual union all
select 16152,'NASA' from dual union all
select 16153,'NASA' from dual union all
select 16154,'SAND' from dual union all
select 16155,'NASA' from dual union all
select 16156,'AFSC' from dual union all
select 16157,'1STRAD' from dual union all
select 16158,'1STRAD' from dual union all
select 16159,'RVSN' from dual union all
select 1616,'USA' from dual union all
select 16160,'1STRAD' from dual union all
select 16161,'AFSC' from dual union all
select 16162,'ONERA' from dual union all
select 16163,'AN' from dual union all
select 16164,'AFCRL' from dual union all
select 16165,'CNES' from dual union all
select 16166,'AN' from dual union all
select 16167,'RVSN' from dual union all
select 16168,'USAF' from dual union all
select 16169,'SAND' from dual union all
select 1617,'USA' from dual;
insert into LAUNCH_AGENCY
select 16170,'SAND' from dual union all
select 16171,'SAND' from dual union all
select 16172,'SAND' from dual union all
select 16173,'USA' from dual union all
select 16174,'BRL' from dual union all
select 16175,'BRL' from dual union all
select 16176,'AFCRL' from dual union all
select 16177,'BRL' from dual union all
select 16178,'USA' from dual union all
select 16179,'USA' from dual union all
select 1618,'USA' from dual union all
select 16180,'USA' from dual union all
select 16181,'USAF' from dual union all
select 16182,'USAF' from dual union all
select 16183,'USAF' from dual union all
select 16184,'USAF' from dual union all
select 16185,'USAF' from dual union all
select 16186,'USAF' from dual union all
select 16187,'AFSC' from dual union all
select 16188,'USAF' from dual union all
select 16189,'USAF' from dual union all
select 1619,'PARD' from dual union all
select 16190,'USAF' from dual union all
select 16191,'USAF' from dual union all
select 16192,'IIAE' from dual union all
select 16193,'SAMSO' from dual union all
select 16194,'SAMSO' from dual union all
select 16195,'SAMSO' from dual union all
select 16196,'SAMSO' from dual union all
select 16197,'CALT' from dual union all
select 16198,'AFSC' from dual union all
select 16199,'CALT' from dual union all
select 162,'WER836-1' from dual union all
select 1620,'USAF' from dual union all
select 16200,'CALT' from dual union all
select 16201,'USA' from dual union all
select 16202,'USA' from dual union all
select 16203,'USA' from dual union all
select 16204,'USA' from dual union all
select 16205,'USA' from dual union all
select 16206,'USA' from dual union all
select 16207,'USA' from dual union all
select 16208,'USA' from dual union all
select 16209,'ONERA' from dual union all
select 1621,'USA' from dual union all
select 16210,'USAF' from dual union all
select 16211,'WRE' from dual union all
select 16212,'44FA2' from dual union all
select 16213,'CNIE' from dual union all
select 16214,'CNIE' from dual union all
select 16215,'USN' from dual union all
select 16216,'USN' from dual union all
select 16217,'USN' from dual union all
select 16218,'USN' from dual union all
select 16219,'USN' from dual union all
select 1622,'USA' from dual union all
select 16220,'AFCRL' from dual union all
select 16221,'USN' from dual union all
select 16222,'USN' from dual union all
select 16223,'US7A' from dual union all
select 16224,'US7A' from dual union all
select 16225,'SAND' from dual union all
select 16226,'SAND' from dual union all
select 16227,'SAND' from dual union all
select 16228,'SAND' from dual union all
select 16229,'SAND' from dual union all
select 1623,'USA' from dual union all
select 16230,'SAND' from dual union all
select 16231,'NRCC' from dual union all
select 16232,'SAND' from dual union all
select 16233,'SAND' from dual union all
select 16234,'SAND' from dual union all
select 16235,'SAND' from dual union all
select 16236,'USA' from dual union all
select 16237,'USA' from dual union all
select 16238,'RVSN' from dual union all
select 16239,'RVSN' from dual union all
select 1624,'USA' from dual union all
select 16240,'RVSN' from dual union all
select 16241,'RVSN' from dual union all
select 16242,'AFSC' from dual union all
select 16243,'RVSN' from dual union all
select 16244,'RVSN' from dual union all
select 16245,'RVSN' from dual union all
select 16246,'RVSN' from dual union all
select 16247,'RVSN' from dual union all
select 16248,'RVSN' from dual union all
select 16249,'RVSN' from dual union all
select 1625,'USA' from dual union all
select 16250,'RVSN' from dual union all
select 16251,'RVSN' from dual union all
select 16252,'RVSN' from dual union all
select 16253,'NRCC' from dual union all
select 16254,'RVSN' from dual union all
select 16255,'RVSN' from dual union all
select 16256,'RVSN' from dual union all
select 16257,'RVSN' from dual union all
select 16258,'RVSN' from dual union all
select 16259,'RVSN' from dual union all
select 1626,'USA' from dual;
insert into LAUNCH_AGENCY
select 16260,'RVSN' from dual union all
select 16261,'RVSN' from dual union all
select 16262,'RVSN' from dual union all
select 16263,'RVSN' from dual union all
select 16264,'AFCRL' from dual union all
select 16265,'AFCRL' from dual union all
select 16266,'AFCRL' from dual union all
select 16267,'AFCRL' from dual union all
select 16268,'1STRAD' from dual union all
select 16269,'RVSN' from dual union all
select 1627,'USAF' from dual union all
select 16270,'RVSN' from dual union all
select 16271,'RVSN' from dual union all
select 16272,'RVSN' from dual union all
select 16273,'RVSN' from dual union all
select 16274,'RVSN' from dual union all
select 16275,'RVSN' from dual union all
select 16276,'RVSN' from dual union all
select 16277,'RVSN' from dual union all
select 16278,'RVSN' from dual union all
select 16279,'ISAS' from dual union all
select 1628,'USA' from dual union all
select 16280,'RVSN' from dual union all
select 16281,'RVSN' from dual union all
select 16282,'CH' from dual union all
select 16283,'USAF' from dual union all
select 16284,'IIAE' from dual union all
select 16285,'MVS' from dual union all
select 16286,'MVS' from dual union all
select 16287,'MVS' from dual union all
select 16288,'MVS' from dual union all
select 16289,'MVS' from dual union all
select 1629,'USA' from dual union all
select 16290,'NASA' from dual union all
select 16291,'MVS' from dual union all
select 16292,'MVS' from dual union all
select 16293,'MVS' from dual union all
select 16294,'MVS' from dual union all
select 16295,'MVS' from dual union all
select 16296,'RVSN' from dual union all
select 16297,'RVSN' from dual union all
select 16298,'RVSN' from dual union all
select 16299,'RVSN' from dual union all
select 163,'WER836-1' from dual union all
select 1630,'USA' from dual union all
select 16300,'RVSN' from dual union all
select 16301,'NASA' from dual union all
select 16302,'RVSN' from dual union all
select 16303,'RVSN' from dual union all
select 16304,'MVS' from dual union all
select 16305,'MVS' from dual union all
select 16306,'NASA' from dual union all
select 16307,'NASA' from dual union all
select 16308,'NASA' from dual union all
select 16309,'NASA' from dual union all
select 1631,'USA' from dual union all
select 16310,'NASA' from dual union all
select 16311,'NASA' from dual union all
select 16312,'CNES' from dual union all
select 16313,'ONERA' from dual union all
select 16314,'ISAS' from dual union all
select 16315,'NASA' from dual union all
select 16316,'CNES' from dual union all
select 16317,'CNES' from dual union all
select 16318,'CNES' from dual union all
select 16319,'1STRAD' from dual union all
select 1632,'USA' from dual union all
select 16320,'ISAS' from dual union all
select 16321,'NASA' from dual union all
select 16322,'NASA' from dual union all
select 16323,'AN' from dual union all
select 16324,'NASA' from dual union all
select 16325,'NASA' from dual union all
select 16326,'NASA' from dual union all
select 16327,'NASA' from dual union all
select 16328,'NASA' from dual union all
select 16329,'NASA' from dual union all
select 1633,'USA' from dual union all
select 16330,'AFSC' from dual union all
select 16331,'AFSC' from dual union all
select 16332,'NASA' from dual union all
select 16333,'USAF' from dual union all
select 16334,'USAF' from dual union all
select 16335,'USAF' from dual union all
select 16336,'USAF' from dual union all
select 16337,'USAF' from dual union all
select 16338,'USAF' from dual union all
select 16339,'USAF' from dual union all
select 1634,'ORDRSA' from dual union all
select 16340,'USAF' from dual union all
select 16341,'USAF' from dual union all
select 16342,'USAF' from dual union all
select 16343,'GMS' from dual union all
select 16344,'GMS' from dual union all
select 16345,'GMS' from dual union all
select 16346,'GMS' from dual union all
select 16347,'GMS' from dual union all
select 16348,'USAF' from dual union all
select 16349,'USAF' from dual union all
select 1635,'ORDRSA' from dual;
insert into LAUNCH_AGENCY
select 16350,'USAF' from dual union all
select 16351,'USAF' from dual union all
select 16352,'USAF' from dual union all
select 16353,'USAF' from dual union all
select 16354,'USAF' from dual union all
select 16355,'USAF' from dual union all
select 16356,'USAF' from dual union all
select 16357,'GMS' from dual union all
select 16358,'GMS' from dual union all
select 16359,'GMS' from dual union all
select 1636,'ORDRSA' from dual union all
select 16360,'GMS' from dual union all
select 16361,'GMS' from dual union all
select 16362,'CONAE' from dual union all
select 16363,'USAF' from dual union all
select 16364,'USAF' from dual union all
select 16365,'USAF' from dual union all
select 16366,'USAF' from dual union all
select 16367,'USAF' from dual union all
select 16368,'USAF' from dual union all
select 16369,'USAF' from dual union all
select 1637,'ORDRSA' from dual union all
select 16370,'USAF' from dual union all
select 16371,'USAF' from dual union all
select 16372,'USAF' from dual union all
select 16373,'USAF' from dual union all
select 16374,'AFCRL' from dual union all
select 16375,'USAF' from dual union all
select 16376,'CONAE' from dual union all
select 16377,'GMS' from dual union all
select 16378,'NASA' from dual union all
select 16379,'GMS' from dual union all
select 1638,'USA' from dual union all
select 16380,'GMS' from dual union all
select 16381,'GMS' from dual union all
select 16382,'GMS' from dual union all
select 16383,'GMS' from dual union all
select 16384,'USAF' from dual union all
select 16385,'USAF' from dual union all
select 16386,'USAF' from dual union all
select 16387,'USAF' from dual union all
select 16388,'USAF' from dual union all
select 16389,'USAF' from dual union all
select 1639,'USA' from dual union all
select 16390,'USAF' from dual union all
select 16391,'USAF' from dual union all
select 16392,'USAF' from dual union all
select 16393,'USAF' from dual union all
select 16394,'SRC' from dual union all
select 16395,'GMS' from dual union all
select 16396,'GMS' from dual union all
select 16397,'USAF' from dual union all
select 16398,'GMS' from dual union all
select 16399,'CONAE' from dual union all
select 164,'WER836-1' from dual union all
select 1640,'USA' from dual union all
select 16400,'GMS' from dual union all
select 16401,'GMS' from dual union all
select 16402,'GMS' from dual union all
select 16403,'GMS' from dual union all
select 16404,'NASA' from dual union all
select 16405,'USAF' from dual union all
select 16406,'USAF' from dual union all
select 16407,'USAF' from dual union all
select 16408,'USAF' from dual union all
select 16409,'USAF' from dual union all
select 1641,'USA' from dual union all
select 16410,'USAF' from dual union all
select 16411,'USAF' from dual union all
select 16412,'USAF' from dual union all
select 16413,'USAF' from dual union all
select 16414,'USAF' from dual union all
select 16415,'NASA' from dual union all
select 16416,'CONAE' from dual union all
select 16417,'GMS' from dual union all
select 16418,'GMS' from dual union all
select 16419,'USAF' from dual union all
select 1642,'USA' from dual union all
select 16420,'NASA' from dual union all
select 16421,'USA' from dual union all
select 16422,'GMS' from dual union all
select 16423,'GMS' from dual union all
select 16424,'USAF' from dual union all
select 16425,'USAF' from dual union all
select 16426,'USAF' from dual union all
select 16427,'GMS' from dual union all
select 16428,'CONAE' from dual union all
select 16429,'NASA' from dual union all
select 1643,'USA' from dual union all
select 16430,'USAF' from dual union all
select 16431,'GMS' from dual union all
select 16432,'NASA' from dual union all
select 16433,'GMS' from dual union all
select 16434,'GMS' from dual union all
select 16435,'USA' from dual union all
select 16436,'USA' from dual union all
select 16437,'NASA' from dual union all
select 16438,'USA' from dual union all
select 16439,'GMS' from dual union all
select 1644,'USA' from dual;
insert into LAUNCH_AGENCY
select 16440,'USAF' from dual union all
select 16441,'GMS' from dual union all
select 16442,'USAF' from dual union all
select 16443,'USAF' from dual union all
select 16444,'USAF' from dual union all
select 16445,'USA' from dual union all
select 16446,'NASA' from dual union all
select 16447,'CONAE' from dual union all
select 16448,'USA' from dual union all
select 16449,'NASA' from dual union all
select 1645,'USA' from dual union all
select 16450,'USA' from dual union all
select 16451,'GMS' from dual union all
select 16452,'GMS' from dual union all
select 16453,'NASA' from dual union all
select 16454,'GMS' from dual union all
select 16455,'NASA' from dual union all
select 16456,'NASA' from dual union all
select 16457,'NASA' from dual union all
select 16458,'GMS' from dual union all
select 16459,'USAF' from dual union all
select 1646,'USA' from dual union all
select 16460,'USAF' from dual union all
select 16461,'USAF' from dual union all
select 16462,'USAF' from dual union all
select 16463,'GMS' from dual union all
select 16464,'USAF' from dual union all
select 16465,'USA' from dual union all
select 16466,'USA' from dual union all
select 16467,'USA' from dual union all
select 16468,'MSFC' from dual union all
select 16469,'MSFC' from dual union all
select 1647,'ORDRSA' from dual union all
select 16470,'GMS' from dual union all
select 16471,'USA' from dual union all
select 16472,'USA' from dual union all
select 16473,'CONAE' from dual union all
select 16474,'GMS' from dual union all
select 16475,'ISRO' from dual union all
select 16476,'GMS' from dual union all
select 16477,'GMS' from dual union all
select 16478,'GMS' from dual union all
select 16479,'GMS' from dual union all
select 1648,'USA' from dual union all
select 16480,'GMS' from dual union all
select 16481,'SUPA' from dual union all
select 16482,'GMS' from dual union all
select 16483,'USAF' from dual union all
select 16484,'USAF' from dual union all
select 16485,'GMS' from dual union all
select 16486,'USAF' from dual union all
select 16487,'USAF' from dual union all
select 16488,'USAF' from dual union all
select 16489,'USAF' from dual union all
select 1649,'USAF' from dual union all
select 16490,'USAF' from dual union all
select 16491,'GMS' from dual union all
select 16492,'SUPA' from dual union all
select 16493,'CONAE' from dual union all
select 16494,'GMS' from dual union all
select 16495,'ISRO' from dual union all
select 16496,'GMS' from dual union all
select 16497,'NASA' from dual union all
select 16498,'NASA' from dual union all
select 16499,'NASA' from dual union all
select 165,'WEHR' from dual union all
select 1650,'PARD' from dual union all
select 16500,'NASA' from dual union all
select 16501,'NASA' from dual union all
select 16502,'NASA' from dual union all
select 16503,'NASA' from dual union all
select 16504,'NASA' from dual union all
select 16505,'CONAE' from dual union all
select 16506,'NASA' from dual union all
select 16507,'NASA' from dual union all
select 16508,'NASA' from dual union all
select 16509,'NASA' from dual union all
select 1651,'PARD' from dual union all
select 16510,'NASA' from dual union all
select 16511,'NASA' from dual union all
select 16512,'NASA' from dual union all
select 16513,'NASA' from dual union all
select 16514,'NASA' from dual union all
select 16515,'NASA' from dual union all
select 16516,'ISAS' from dual union all
select 16517,'NASA' from dual union all
select 16518,'NASA' from dual union all
select 16519,'NASA' from dual union all
select 1652,'USA' from dual union all
select 16520,'NASA' from dual union all
select 16521,'NASA' from dual union all
select 16522,'NASA' from dual union all
select 16523,'NASA' from dual union all
select 16524,'NASA' from dual union all
select 16525,'NASA' from dual union all
select 16526,'NASA' from dual union all
select 16527,'GMS' from dual union all
select 16528,'NASA' from dual union all
select 16529,'NASA' from dual union all
select 1653,'USA' from dual;
insert into LAUNCH_AGENCY
select 16530,'NASA' from dual union all
select 16531,'NASA' from dual union all
select 16532,'NASA' from dual union all
select 16533,'NASA' from dual union all
select 16534,'NASA' from dual union all
select 16535,'NASA' from dual union all
select 16536,'NASA' from dual union all
select 16537,'NASA' from dual union all
select 16538,'GMS' from dual union all
select 16539,'NASA' from dual union all
select 1654,'ORDRSA' from dual union all
select 16540,'NASA' from dual union all
select 16541,'NASA' from dual union all
select 16542,'NASA' from dual union all
select 16543,'NASA' from dual union all
select 16544,'NASA' from dual union all
select 16545,'NASA' from dual union all
select 16546,'NASA' from dual union all
select 16547,'NASA' from dual union all
select 16548,'NASA' from dual union all
select 16549,'ISAS' from dual union all
select 1655,'ORDRSA' from dual union all
select 16550,'NASA' from dual union all
select 16551,'NASA' from dual union all
select 16552,'NASA' from dual union all
select 16553,'NASA' from dual union all
select 16554,'AFCRL' from dual union all
select 16555,'AFCRL' from dual union all
select 16556,'AFCRL' from dual union all
select 16557,'AFCRL' from dual union all
select 16558,'AFCRL' from dual union all
select 16559,'NASA' from dual union all
select 1656,'USAF' from dual union all
select 16560,'ISAS' from dual union all
select 16561,'CNAE' from dual union all
select 16562,'CNAE' from dual union all
select 16563,'CNAE' from dual union all
select 16564,'CNAE' from dual union all
select 16565,'CNAE' from dual union all
select 16566,'CNAE' from dual union all
select 16567,'CNAE' from dual union all
select 16568,'CNAE' from dual union all
select 16569,'CNAE' from dual union all
select 1657,'USA' from dual union all
select 16570,'CNAE' from dual union all
select 16571,'ISAS' from dual union all
select 16572,'CNAE' from dual union all
select 16573,'CNAE' from dual union all
select 16574,'CNAE' from dual union all
select 16575,'CNAE' from dual union all
select 16576,'CNAE' from dual union all
select 16577,'CNAE' from dual union all
select 16578,'NASA' from dual union all
select 16579,'NASA' from dual union all
select 1658,'USA' from dual union all
select 16580,'NASA' from dual union all
select 16581,'NASA' from dual union all
select 16582,'GMS' from dual union all
select 16583,'NASA' from dual union all
select 16584,'NASA' from dual union all
select 16585,'NASA' from dual union all
select 16586,'NASA' from dual union all
select 16587,'NASA' from dual union all
select 16588,'NASA' from dual union all
select 16589,'AFCRL' from dual union all
select 1659,'PARD' from dual union all
select 16590,'AFCRL' from dual union all
select 16591,'AFCRL' from dual union all
select 16592,'AFCRL' from dual union all
select 16593,'ISAS' from dual union all
select 16594,'AFCRL' from dual union all
select 16595,'AFCRL' from dual union all
select 16596,'AFCRL' from dual union all
select 16597,'AFCRL' from dual union all
select 16598,'AFCRL' from dual union all
select 16599,'AFCRL' from dual union all
select 166,'WER836-1' from dual union all
select 1660,'USA' from dual union all
select 16600,'AFCRL' from dual union all
select 16601,'AFCRL' from dual union all
select 16602,'AFCRL' from dual union all
select 16603,'AFCRL' from dual union all
select 16604,'MSFC' from dual union all
select 16605,'AFCRL' from dual union all
select 16606,'AFCRL' from dual union all
select 16607,'GMS' from dual union all
select 16608,'GMS' from dual union all
select 16609,'GMS' from dual union all
select 1661,'USA' from dual union all
select 16610,'USAF' from dual union all
select 16611,'USAF' from dual union all
select 16612,'USAF' from dual union all
select 16613,'USAF' from dual union all
select 16614,'USAF' from dual union all
select 16615,'USAF' from dual union all
select 16616,'USAF' from dual union all
select 16617,'USAF' from dual union all
select 16618,'GMS' from dual union all
select 16619,'GMS' from dual union all
select 1662,'USA' from dual;
insert into LAUNCH_AGENCY
select 16620,'CONAE' from dual union all
select 16621,'USAF' from dual union all
select 16622,'USAF' from dual union all
select 16623,'USAF' from dual union all
select 16624,'USAF' from dual union all
select 16625,'USAF' from dual union all
select 16626,'USAF' from dual union all
select 16627,'USAF' from dual union all
select 16628,'USAF' from dual union all
select 16629,'USAF' from dual union all
select 1663,'ORDRSA' from dual union all
select 16630,'USAF' from dual union all
select 16631,'USN' from dual union all
select 16632,'GMS' from dual union all
select 16633,'GMS' from dual union all
select 16634,'USAF' from dual union all
select 16635,'USAF' from dual union all
select 16636,'USAF' from dual union all
select 16637,'USAF' from dual union all
select 16638,'USAF' from dual union all
select 16639,'USAF' from dual union all
select 1664,'ORDRSA' from dual union all
select 16640,'USAF' from dual union all
select 16641,'USAF' from dual union all
select 16642,'USAF' from dual union all
select 16643,'USAF' from dual union all
select 16644,'USAF' from dual union all
select 16645,'USAF' from dual union all
select 16646,'JMA' from dual union all
select 16646,'NASA' from dual union all
select 16647,'NASA' from dual union all
select 16648,'JMA' from dual union all
select 16648,'NASA' from dual union all
select 16649,'NASA' from dual union all
select 1665,'ORDRSA' from dual union all
select 16650,'JMA' from dual union all
select 16650,'NASA' from dual union all
select 16651,'NASA' from dual union all
select 16652,'JMA' from dual union all
select 16652,'NASA' from dual union all
select 16653,'NASA' from dual union all
select 16654,'JMA' from dual union all
select 16654,'NASA' from dual union all
select 16655,'JMA' from dual union all
select 16655,'NASA' from dual union all
select 16656,'NASA' from dual union all
select 16657,'NASA' from dual union all
select 16658,'JMA' from dual union all
select 16658,'NASA' from dual union all
select 16659,'NASA' from dual union all
select 1666,'ORDRSA' from dual union all
select 16660,'JMA' from dual union all
select 16660,'NASA' from dual union all
select 16661,'NASA' from dual union all
select 16662,'JMA' from dual union all
select 16662,'NASA' from dual union all
select 16663,'NASA' from dual union all
select 16664,'JMA' from dual union all
select 16664,'NASA' from dual union all
select 16665,'NASA' from dual union all
select 16666,'AFCRL' from dual union all
select 16667,'GMS' from dual union all
select 16668,'GMS' from dual union all
select 16669,'CONAE' from dual union all
select 1667,'ORDRSA' from dual union all
select 16670,'GMS' from dual union all
select 16671,'GMS' from dual union all
select 16672,'GMS' from dual union all
select 16673,'GMS' from dual union all
select 16674,'GMS' from dual union all
select 16675,'GMS' from dual union all
select 16676,'GMS' from dual union all
select 16677,'GMS' from dual union all
select 16678,'GMS' from dual union all
select 16679,'GMS' from dual union all
select 1668,'ORDRSA' from dual union all
select 16680,'GMS' from dual union all
select 16681,'GMS' from dual union all
select 16682,'GMS' from dual union all
select 16683,'GMS' from dual union all
select 16684,'GMS' from dual union all
select 16685,'GMS' from dual union all
select 16686,'GMS' from dual union all
select 16687,'GMS' from dual union all
select 16688,'GMS' from dual union all
select 16689,'USA' from dual union all
select 1669,'USA' from dual union all
select 16690,'GMS' from dual union all
select 16691,'GMS' from dual union all
select 16692,'GMS' from dual union all
select 16693,'GMS' from dual union all
select 16694,'GMS' from dual union all
select 16695,'GMS' from dual union all
select 16696,'GMS' from dual union all
select 16697,'GMS' from dual union all
select 16698,'ISAS' from dual union all
select 16698,'NASA' from dual union all
select 16699,'ISAS' from dual union all
select 16699,'NASA' from dual union all
select 167,'WER836-1' from dual;
insert into LAUNCH_AGENCY
select 1670,'USA' from dual union all
select 16700,'ISAS' from dual union all
select 16700,'NASA' from dual union all
select 16701,'ISAS' from dual union all
select 16701,'NASA' from dual union all
select 16702,'ISAS' from dual union all
select 16702,'NASA' from dual union all
select 16703,'ISAS' from dual union all
select 16703,'NASA' from dual union all
select 16704,'ISAS' from dual union all
select 16704,'NASA' from dual union all
select 16705,'ISAS' from dual union all
select 16705,'NASA' from dual union all
select 16706,'ISAS' from dual union all
select 16706,'NASA' from dual union all
select 16707,'ISAS' from dual union all
select 16707,'NASA' from dual union all
select 16708,'GMS' from dual union all
select 16709,'GMS' from dual union all
select 1671,'USA' from dual union all
select 16710,'GMS' from dual union all
select 16711,'GMS' from dual union all
select 16712,'SRC' from dual union all
select 16713,'GMS' from dual union all
select 16714,'GMS' from dual union all
select 16715,'GMS' from dual union all
select 16716,'GMS' from dual union all
select 16717,'GMS' from dual union all
select 16718,'GMS' from dual union all
select 16719,'GMS' from dual union all
select 1672,'PARD' from dual union all
select 16720,'GMS' from dual union all
select 16721,'GMS' from dual union all
select 16722,'GMS' from dual union all
select 16723,'GMS' from dual union all
select 16724,'GMS' from dual union all
select 16725,'GMS' from dual union all
select 16726,'GMS' from dual union all
select 16727,'GMS' from dual union all
select 16728,'NASA' from dual union all
select 16729,'NASA' from dual union all
select 1673,'USA' from dual union all
select 16730,'RVSN' from dual union all
select 16731,'USAF' from dual union all
select 16732,'AFSC' from dual union all
select 16733,'RVSN' from dual union all
select 16734,'NASA' from dual union all
select 16735,'USAF' from dual union all
select 16736,'RVSN' from dual union all
select 16737,'RVSN' from dual union all
select 16738,'RVSN' from dual union all
select 16739,'AFSC' from dual union all
select 1674,'ORDRSA' from dual union all
select 16740,'RVSN' from dual union all
select 16741,'AFSC' from dual union all
select 16742,'RVSN' from dual union all
select 16743,'RVSN' from dual union all
select 16744,'NASA' from dual union all
select 16745,'AFSC' from dual union all
select 16746,'RVSN' from dual union all
select 16747,'USAF' from dual union all
select 16748,'RVSN' from dual union all
select 16749,'RVSN' from dual union all
select 1675,'ORDRSA' from dual union all
select 16750,'RVSN' from dual union all
select 16751,'RVSN' from dual union all
select 16752,'NASA' from dual union all
select 16753,'AFSC' from dual union all
select 16754,'RVSN' from dual union all
select 16755,'RVSN' from dual union all
select 16756,'RVSN' from dual union all
select 16757,'RVSN' from dual union all
select 16758,'AFSC' from dual union all
select 16759,'RVSN' from dual union all
select 1676,'USA' from dual union all
select 16760,'RVSN' from dual union all
select 16761,'RVSN' from dual union all
select 16762,'RVSN' from dual union all
select 16763,'RVSN' from dual union all
select 16764,'RVSN' from dual union all
select 16765,'RVSN' from dual union all
select 16766,'USAF' from dual union all
select 16767,'RVSN' from dual union all
select 16768,'AFSC' from dual union all
select 16769,'10ADS' from dual union all
select 1677,'USA' from dual union all
select 16770,'RVSN' from dual union all
select 16771,'RVSN' from dual union all
select 16772,'RVSN' from dual union all
select 16773,'RVSN' from dual union all
select 16774,'AFSC' from dual union all
select 16775,'RVSN' from dual union all
select 16776,'RVSN' from dual union all
select 16777,'USAF' from dual union all
select 16778,'RVSN' from dual union all
select 16779,'USAF' from dual union all
select 1678,'ORDRSA' from dual union all
select 16780,'RVSN' from dual union all
select 16781,'RVSN' from dual union all
select 16782,'NASA' from dual;
insert into LAUNCH_AGENCY
select 16783,'RVSN' from dual union all
select 16784,'RVSN' from dual union all
select 16785,'RVSN' from dual union all
select 16786,'AFSC' from dual union all
select 16787,'RVSN' from dual union all
select 16788,'RVSN' from dual union all
select 16789,'RVSN' from dual union all
select 1679,'ORDRSA' from dual union all
select 16790,'USAF' from dual union all
select 16791,'AFSC' from dual union all
select 16792,'USAF' from dual union all
select 16793,'AFSC' from dual union all
select 16794,'RVSN' from dual union all
select 16795,'NASA' from dual union all
select 16796,'NASA' from dual union all
select 16797,'RVSN' from dual union all
select 16798,'RVSN' from dual union all
select 16799,'RVSN' from dual union all
select 168,'WER836-1' from dual union all
select 1680,'ORDRSA' from dual union all
select 16800,'RVSN' from dual union all
select 16801,'AFSC' from dual union all
select 16802,'RVSN' from dual union all
select 16803,'RVSN' from dual union all
select 16804,'RVSN' from dual union all
select 16805,'USAF' from dual union all
select 16806,'RVSN' from dual union all
select 16807,'RVSN' from dual union all
select 16808,'USAF' from dual union all
select 16809,'RVSN' from dual union all
select 1681,'ORDRSA' from dual union all
select 16810,'RVSN' from dual union all
select 16811,'AFSC' from dual union all
select 16812,'RVSN' from dual union all
select 16813,'USAF' from dual union all
select 16814,'RVSN' from dual union all
select 16815,'RVSN' from dual union all
select 16816,'NASA' from dual union all
select 16817,'RVSN' from dual union all
select 16818,'RVSN' from dual union all
select 16819,'10ADS' from dual union all
select 1682,'ORDRSA' from dual union all
select 16820,'RVSN' from dual union all
select 16821,'RVSN' from dual union all
select 16822,'RVSN' from dual union all
select 16823,'RVSN' from dual union all
select 16824,'RVSN' from dual union all
select 16825,'USAF' from dual union all
select 16826,'AFSC' from dual union all
select 16827,'NASA' from dual union all
select 16828,'RVSN' from dual union all
select 16829,'RVSN' from dual union all
select 1683,'ORDRSA' from dual union all
select 16830,'RVSN' from dual union all
select 16831,'RVSN' from dual union all
select 16832,'RVSN' from dual union all
select 16833,'RVSN' from dual union all
select 16834,'RVSN' from dual union all
select 16835,'AFSC' from dual union all
select 16836,'NASA' from dual union all
select 16837,'NASA' from dual union all
select 16838,'RVSN' from dual union all
select 16839,'USAF' from dual union all
select 1684,'USA' from dual union all
select 16840,'RVSN' from dual union all
select 16841,'NASA' from dual union all
select 16842,'RVSN' from dual union all
select 16843,'NASA' from dual union all
select 16844,'RVSN' from dual union all
select 16845,'NASA' from dual union all
select 16846,'RVSN' from dual union all
select 16847,'WRE' from dual union all
select 16848,'PIHM' from dual union all
select 16849,'PIHM' from dual union all
select 1685,'ORDRSA' from dual union all
select 16850,'PIHM' from dual union all
select 16851,'PIHM' from dual union all
select 16852,'PIHM' from dual union all
select 16853,'PIHM' from dual union all
select 16854,'PIHM' from dual union all
select 16855,'PIHM' from dual union all
select 16856,'PIHM' from dual union all
select 16857,'NASA' from dual union all
select 16858,'DASA' from dual union all
select 16859,'DASA' from dual union all
select 1686,'ORDRSA' from dual union all
select 16860,'RAE' from dual union all
select 16861,'USN' from dual union all
select 16862,'USA' from dual union all
select 16863,'RVSN' from dual union all
select 16864,'USA' from dual union all
select 16865,'USAF' from dual union all
select 16866,'PIHM' from dual union all
select 16867,'USN' from dual union all
select 16868,'AFSC' from dual union all
select 16869,'USA' from dual union all
select 1687,'ORDRSA' from dual union all
select 16870,'NTNF' from dual union all
select 16871,'USA' from dual union all
select 16872,'NASA' from dual;
insert into LAUNCH_AGENCY
select 16872,'USAF' from dual union all
select 16873,'GMS' from dual union all
select 16874,'CNES' from dual union all
select 16875,'SAND' from dual union all
select 16876,'LARCN' from dual union all
select 16877,'PIHM' from dual union all
select 16878,'USAF' from dual union all
select 16879,'PMR' from dual union all
select 1688,'ORDRSA' from dual union all
select 16880,'PMR' from dual union all
select 16881,'PMR' from dual union all
select 16882,'PMR' from dual union all
select 16883,'PMR' from dual union all
select 16884,'PMR' from dual union all
select 16885,'PMR' from dual union all
select 16886,'PMR' from dual union all
select 16887,'PMR' from dual union all
select 16888,'PIHM' from dual union all
select 16889,'PMR' from dual union all
select 1689,'ORDRSA' from dual union all
select 16890,'PMR' from dual union all
select 16891,'USA' from dual union all
select 16892,'PIHM' from dual union all
select 16893,'PIHM' from dual union all
select 16894,'PIHM' from dual union all
select 16895,'PIHM' from dual union all
select 16896,'PIHM' from dual union all
select 16897,'PIHM' from dual union all
select 16898,'PIHM' from dual union all
select 16899,'PIHM' from dual union all
select 169,'WER836-1' from dual union all
select 1690,'ORDRSA' from dual union all
select 16900,'PIHM' from dual union all
select 16901,'PIHM' from dual union all
select 16902,'PIHM' from dual union all
select 16903,'PIHM' from dual union all
select 16904,'PIHM' from dual union all
select 16905,'PIHM' from dual union all
select 16906,'PIHM' from dual union all
select 16907,'PIHM' from dual union all
select 16908,'PIHM' from dual union all
select 16909,'PIHM' from dual union all
select 1691,'USA' from dual union all
select 16910,'PIHM' from dual union all
select 16911,'PIHM' from dual union all
select 16912,'PIHM' from dual union all
select 16913,'PIHM' from dual union all
select 16914,'PIHM' from dual union all
select 16915,'PIHM' from dual union all
select 16916,'PIHM' from dual union all
select 16917,'PIHM' from dual union all
select 16918,'PIHM' from dual union all
select 16919,'PIHM' from dual union all
select 1692,'USA' from dual union all
select 16920,'PIHM' from dual union all
select 16921,'PIHM' from dual union all
select 16922,'PIHM' from dual union all
select 16923,'PIHM' from dual union all
select 16924,'PIHM' from dual union all
select 16925,'ISRO' from dual union all
select 16926,'ISRO' from dual union all
select 16927,'ISRO' from dual union all
select 16928,'RVSN' from dual union all
select 16929,'RVSN' from dual union all
select 1693,'USA' from dual union all
select 16930,'RVSN' from dual union all
select 16931,'GMS' from dual union all
select 16932,'ISRO' from dual union all
select 16933,'ISRO' from dual union all
select 16934,'NASA' from dual union all
select 16935,'USA' from dual union all
select 16936,'NASA' from dual union all
select 16936,'USAF' from dual union all
select 16937,'NASA' from dual union all
select 16937,'USAF' from dual union all
select 16938,'RVSN' from dual union all
select 16939,'GMS' from dual union all
select 1694,'USA' from dual union all
select 16940,'NASA' from dual union all
select 16941,'USA' from dual union all
select 16942,'NASA' from dual union all
select 16942,'USAF' from dual union all
select 16943,'DASA' from dual union all
select 16944,'NASA' from dual union all
select 16944,'USAF' from dual union all
select 16945,'CNES' from dual union all
select 16946,'USA' from dual union all
select 16947,'NASA' from dual union all
select 16947,'USAF' from dual union all
select 16948,'USN' from dual union all
select 16949,'NASA' from dual union all
select 16949,'USAF' from dual union all
select 1695,'USAF' from dual union all
select 16950,'NASA' from dual union all
select 16951,'USA' from dual union all
select 16952,'NRCC' from dual union all
select 16953,'NASA' from dual union all
select 16954,'USAF' from dual union all
select 16955,'NASA' from dual union all
select 16956,'USAF' from dual;
insert into LAUNCH_AGENCY
select 16957,'USN' from dual union all
select 16958,'NASA' from dual union all
select 16958,'USAF' from dual union all
select 16959,'SRC' from dual union all
select 1696,'USAF' from dual union all
select 16960,'USN' from dual union all
select 16961,'USA' from dual union all
select 16962,'NASA' from dual union all
select 16962,'USAF' from dual union all
select 16963,'RVSN' from dual union all
select 16964,'RAE' from dual union all
select 16965,'USAF' from dual union all
select 16966,'USA' from dual union all
select 16967,'CONAE' from dual union all
select 16967,'NASA' from dual union all
select 16968,'USN' from dual union all
select 16969,'USA' from dual union all
select 1697,'USA' from dual union all
select 16970,'NRCC' from dual union all
select 16971,'USA' from dual union all
select 16972,'NASA' from dual union all
select 16973,'ISRO' from dual union all
select 16974,'USA' from dual union all
select 16975,'USAF' from dual union all
select 16976,'STA' from dual union all
select 16977,'NASA' from dual union all
select 16978,'RVSN' from dual union all
select 16979,'RVSN' from dual union all
select 1698,'USA' from dual union all
select 16980,'RVSN' from dual union all
select 16981,'RVSN' from dual union all
select 16982,'USAF' from dual union all
select 16983,'RVSN' from dual union all
select 16984,'RVSN' from dual union all
select 16985,'AFSC' from dual union all
select 16986,'NASA' from dual union all
select 16987,'ELDO' from dual union all
select 16988,'NASA' from dual union all
select 16989,'DMA' from dual union all
select 1699,'USAF' from dual union all
select 16990,'NASA' from dual union all
select 16991,'NRCC' from dual union all
select 16992,'NASA' from dual union all
select 16992,'USAF' from dual union all
select 16993,'NASA' from dual union all
select 16993,'USAF' from dual union all
select 16994,'USA' from dual union all
select 16995,'CA' from dual union all
select 16996,'CA' from dual union all
select 16997,'ESRO' from dual union all
select 16998,'ESRO' from dual union all
select 16999,'USA' from dual union all
select 17,'WEHR' from dual union all
select 170,'WER444' from dual union all
select 1700,'USAF' from dual union all
select 17000,'USA' from dual union all
select 17001,'NASA' from dual union all
select 17001,'USAF' from dual union all
select 17002,'NASA' from dual union all
select 17002,'USAF' from dual union all
select 17003,'USA' from dual union all
select 17004,'NRL' from dual union all
select 17005,'ISAS' from dual union all
select 17006,'ISAS' from dual union all
select 17007,'ISAS' from dual union all
select 17008,'NASA' from dual union all
select 17008,'USAF' from dual union all
select 17009,'NASA' from dual union all
select 1701,'USA' from dual union all
select 17010,'USA' from dual union all
select 17011,'NASA' from dual union all
select 17012,'NASA' from dual union all
select 17012,'USAF' from dual union all
select 17013,'NASA' from dual union all
select 17013,'NTNF' from dual union all
select 17014,'USA' from dual union all
select 17015,'NASA' from dual union all
select 17016,'WRE' from dual union all
select 17017,'WRE' from dual union all
select 17018,'WRE' from dual union all
select 17019,'WRE' from dual union all
select 1702,'USA' from dual union all
select 17020,'WRE' from dual union all
select 17021,'WRE' from dual union all
select 17022,'WRE' from dual union all
select 17023,'WRE' from dual union all
select 17024,'WRE' from dual union all
select 17025,'WRE' from dual union all
select 17026,'WRE' from dual union all
select 17027,'USAF' from dual union all
select 17028,'WRE' from dual union all
select 17029,'WRE' from dual union all
select 1703,'USA' from dual union all
select 17030,'WRE' from dual union all
select 17031,'AFCRL' from dual union all
select 17032,'INTA' from dual union all
select 17033,'INTA' from dual union all
select 17034,'INTA' from dual union all
select 17035,'INTA' from dual union all
select 17036,'INTA' from dual;
insert into LAUNCH_AGENCY
select 17037,'INTA' from dual union all
select 17038,'INTA' from dual union all
select 17039,'INTA' from dual union all
select 1704,'USA' from dual union all
select 17040,'INTA' from dual union all
select 17041,'INTA' from dual union all
select 17042,'INTA' from dual union all
select 17043,'INTA' from dual union all
select 17044,'INTA' from dual union all
select 17045,'INTA' from dual union all
select 17046,'INTA' from dual union all
select 17047,'IIAE' from dual union all
select 17048,'IIAE' from dual union all
select 17049,'IIAE' from dual union all
select 1705,'USA' from dual union all
select 17050,'BWEHR' from dual union all
select 17051,'BWEHR' from dual union all
select 17052,'BWEHR' from dual union all
select 17053,'30FA2' from dual union all
select 17054,'DASST' from dual union all
select 17055,'DASST' from dual union all
select 17056,'DASST' from dual union all
select 17057,'DASST' from dual union all
select 17058,'SECT' from dual union all
select 17059,'SECT' from dual union all
select 1706,'USA' from dual union all
select 17060,'SECT' from dual union all
select 17061,'DMA' from dual union all
select 17062,'DMA' from dual union all
select 17063,'DMA' from dual union all
select 17064,'1STRAD' from dual union all
select 17064,'ALCS' from dual union all
select 17065,'NASA' from dual union all
select 17066,'NRCC' from dual union all
select 17067,'AFSC' from dual union all
select 17068,'USN' from dual union all
select 17069,'USAF' from dual union all
select 1707,'ORDRSA' from dual union all
select 17070,'ESRO' from dual union all
select 17071,'NASA' from dual union all
select 17072,'NASA' from dual union all
select 17073,'NASA' from dual union all
select 17074,'DMA' from dual union all
select 17075,'NASA' from dual union all
select 17076,'NASA' from dual union all
select 17077,'1STRAD' from dual union all
select 17078,'1STRAD' from dual union all
select 17079,'USA' from dual union all
select 1708,'USA' from dual union all
select 17080,'NASA' from dual union all
select 17081,'NASA' from dual union all
select 17082,'ISRO' from dual union all
select 17083,'ISRO' from dual union all
select 17084,'SRC' from dual union all
select 17085,'DMA' from dual union all
select 17086,'ESRO' from dual union all
select 17087,'NASA' from dual union all
select 17088,'NASA' from dual union all
select 17089,'NASA' from dual union all
select 1709,'USA' from dual union all
select 17090,'SAND' from dual union all
select 17091,'AFSC' from dual union all
select 17092,'RAE' from dual union all
select 17092,'WRE' from dual union all
select 17093,'SAND' from dual union all
select 17094,'NASA' from dual union all
select 17095,'AFSC' from dual union all
select 17096,'RVSN' from dual union all
select 17097,'CNES' from dual union all
select 17098,'RN' from dual union all
select 17099,'DLR' from dual union all
select 17099,'INTA' from dual union all
select 171,'WER444' from dual union all
select 1710,'PARD' from dual union all
select 17100,'DLR' from dual union all
select 17100,'INTA' from dual union all
select 17101,'KPNO' from dual union all
select 17102,'DLR' from dual union all
select 17102,'INTA' from dual union all
select 17103,'USAF' from dual union all
select 17104,'NASA' from dual union all
select 17105,'ISRO' from dual union all
select 17106,'RVSN' from dual union all
select 17107,'NASA' from dual union all
select 17108,'NASA' from dual union all
select 17109,'NASA' from dual union all
select 1711,'USA' from dual union all
select 17110,'NASA' from dual union all
select 17111,'NASA' from dual union all
select 17112,'NASA' from dual union all
select 17113,'NASA' from dual union all
select 17114,'ISRO' from dual union all
select 17115,'NASA' from dual union all
select 17116,'RVSN' from dual union all
select 17117,'NASA' from dual union all
select 17118,'AFSC' from dual union all
select 17119,'USAF' from dual union all
select 1712,'USA' from dual union all
select 17120,'NASA' from dual union all
select 17121,'AN' from dual;
insert into LAUNCH_AGENCY
select 17122,'SAC' from dual union all
select 17123,'NASA' from dual union all
select 17124,'AN' from dual union all
select 17124,'CNES' from dual union all
select 17125,'MICOM' from dual union all
select 17126,'RVSN' from dual union all
select 17127,'RVSN' from dual union all
select 17128,'AN' from dual union all
select 17129,'SRC' from dual union all
select 1713,'USA' from dual union all
select 17130,'NASA' from dual union all
select 17131,'NASA' from dual union all
select 17132,'NASA' from dual union all
select 17133,'RN' from dual union all
select 17134,'MICOM' from dual union all
select 17135,'MICOM' from dual union all
select 17135,'US7A' from dual union all
select 17136,'AFSC' from dual union all
select 17137,'SRC' from dual union all
select 17138,'RVSN' from dual union all
select 17139,'ESRO' from dual union all
select 1714,'SUI' from dual union all
select 17140,'CNES' from dual union all
select 17141,'SAND' from dual union all
select 17142,'SAND' from dual union all
select 17143,'SAND' from dual union all
select 17144,'SECT' from dual union all
select 17145,'CNES' from dual union all
select 17146,'AFCRL' from dual union all
select 17147,'NASA' from dual union all
select 17148,'NASA' from dual union all
select 17149,'RVSN' from dual union all
select 1715,'SUI' from dual union all
select 17150,'NASA' from dual union all
select 17151,'AN' from dual union all
select 17152,'NASA' from dual union all
select 17153,'NASA' from dual union all
select 17154,'NASA' from dual union all
select 17155,'NASA' from dual union all
select 17156,'NASA' from dual union all
select 17157,'NASA' from dual union all
select 17158,'NASA' from dual union all
select 17159,'AN' from dual union all
select 1716,'SUI' from dual union all
select 17160,'RVSN' from dual union all
select 17161,'AN' from dual union all
select 17162,'NASA' from dual union all
select 17163,'AN' from dual union all
select 17164,'AN' from dual union all
select 17165,'RAE' from dual union all
select 17165,'WRE' from dual union all
select 17166,'DLR' from dual union all
select 17167,'NMC' from dual union all
select 17168,'AN' from dual union all
select 17169,'AN' from dual union all
select 1717,'USA' from dual union all
select 17170,'AN' from dual union all
select 17171,'RVSN' from dual union all
select 17172,'USAF' from dual union all
select 17173,'NASA' from dual union all
select 17174,'AN' from dual union all
select 17175,'NASA' from dual union all
select 17176,'DLR' from dual union all
select 17177,'SUPA' from dual union all
select 17178,'AN' from dual union all
select 17179,'NASA' from dual union all
select 1718,'SUI' from dual union all
select 17180,'AN' from dual union all
select 17181,'AN' from dual union all
select 17182,'RVSN' from dual union all
select 17183,'NASA' from dual union all
select 17184,'NASA' from dual union all
select 17185,'AN' from dual union all
select 17186,'SRC' from dual union all
select 17187,'AN' from dual union all
select 17188,'ESRO' from dual union all
select 17189,'CNES' from dual union all
select 1719,'ORDRSA' from dual union all
select 17190,'NRCC' from dual union all
select 17191,'AN' from dual union all
select 17192,'RVSN' from dual union all
select 17193,'RVSN' from dual union all
select 17194,'USAF' from dual union all
select 17195,'USAF' from dual union all
select 17196,'CNES' from dual union all
select 17197,'ISRO' from dual union all
select 17198,'NASA' from dual union all
select 17199,'1STRAD' from dual union all
select 172,'WER836-1' from dual union all
select 1720,'USA' from dual union all
select 17200,'USAF' from dual union all
select 17201,'NRL' from dual union all
select 17202,'AFCRL' from dual union all
select 17203,'CNES' from dual union all
select 17204,'RVSN' from dual union all
select 17205,'USA' from dual union all
select 17206,'AN' from dual union all
select 17207,'ISRO' from dual union all
select 17208,'AN' from dual union all
select 17209,'USAF' from dual;
insert into LAUNCH_AGENCY
select 1721,'USA' from dual union all
select 17210,'USAF' from dual union all
select 17211,'USA' from dual union all
select 17212,'USA' from dual union all
select 17213,'USA' from dual union all
select 17214,'USA' from dual union all
select 17215,'RVSN' from dual union all
select 17216,'USA' from dual union all
select 17217,'USA' from dual union all
select 17218,'USA' from dual union all
select 17219,'USA' from dual union all
select 1722,'USA' from dual union all
select 17220,'FKGR21' from dual union all
select 17221,'FKGR13' from dual union all
select 17222,'ISRO' from dual union all
select 17223,'ISRO' from dual union all
select 17224,'CNES' from dual union all
select 17225,'RVSN' from dual union all
select 17226,'SAC' from dual union all
select 17227,'SUPA' from dual union all
select 17228,'NRCC' from dual union all
select 17229,'NASA' from dual union all
select 1723,'PARD' from dual union all
select 17230,'NASA' from dual union all
select 17231,'DLR' from dual union all
select 17232,'NRCC' from dual union all
select 17233,'NRL' from dual union all
select 17234,'DMA' from dual union all
select 17235,'CNES' from dual union all
select 17236,'RVSN' from dual union all
select 17237,'NASA' from dual union all
select 17238,'1STRAD' from dual union all
select 17239,'USN' from dual union all
select 1724,'SUI' from dual union all
select 17240,'USN' from dual union all
select 17241,'USN' from dual union all
select 17242,'USN' from dual union all
select 17243,'NRCC' from dual union all
select 17244,'AFSC' from dual union all
select 17245,'RVSN' from dual union all
select 17246,'NRCC' from dual union all
select 17247,'RVSN' from dual union all
select 17248,'NRCC' from dual union all
select 17249,'NASA' from dual union all
select 1725,'USAF' from dual union all
select 17250,'ISRO' from dual union all
select 17251,'1STRAD' from dual union all
select 17252,'NRCC' from dual union all
select 17253,'NASA' from dual union all
select 17254,'INTA' from dual union all
select 17255,'NASA' from dual union all
select 17256,'INTA' from dual union all
select 17257,'INTA' from dual union all
select 17258,'RVSN' from dual union all
select 17259,'ISRO' from dual union all
select 1726,'USA' from dual union all
select 17260,'USAF' from dual union all
select 17261,'NASA' from dual union all
select 17262,'NASA' from dual union all
select 17263,'USN' from dual union all
select 17264,'USN' from dual union all
select 17265,'USN' from dual union all
select 17266,'USN' from dual union all
select 17267,'AFSC' from dual union all
select 17268,'NASA' from dual union all
select 17269,'RVSN' from dual union all
select 1727,'SUI' from dual union all
select 17270,'NASA' from dual union all
select 17271,'RVSN' from dual union all
select 17272,'US7A' from dual union all
select 17273,'NASA' from dual union all
select 17274,'CA' from dual union all
select 17275,'USAF' from dual union all
select 17276,'USAF' from dual union all
select 17277,'1STRAD' from dual union all
select 17278,'NASA' from dual union all
select 17279,'AFCRL' from dual union all
select 1728,'USA' from dual union all
select 17280,'RVSN' from dual union all
select 17281,'AFCRL' from dual union all
select 17282,'AFCRL' from dual union all
select 17283,'AFCRL' from dual union all
select 17284,'AFSC' from dual union all
select 17285,'NASA' from dual union all
select 17286,'NASA' from dual union all
select 17287,'DRTE' from dual union all
select 17287,'NASA' from dual union all
select 17288,'AFCRL' from dual union all
select 17289,'USAF' from dual union all
select 1729,'USA' from dual union all
select 17290,'SAND' from dual union all
select 17291,'RVSN' from dual union all
select 17292,'AN' from dual union all
select 17293,'AN' from dual union all
select 17294,'AN' from dual union all
select 17295,'RVSN' from dual union all
select 17296,'AFCRL' from dual union all
select 17297,'AN' from dual union all
select 17298,'AFCRL' from dual union all
select 17299,'CNES' from dual;
insert into LAUNCH_AGENCY
select 17299,'ESRO' from dual union all
select 173,'WER836-1' from dual union all
select 1730,'PARD' from dual union all
select 17300,'USAF' from dual union all
select 17301,'CNES' from dual union all
select 17302,'RVSN' from dual union all
select 17303,'SAND' from dual union all
select 17304,'AN' from dual union all
select 17305,'KPNO' from dual union all
select 17306,'NASA' from dual union all
select 17307,'SAND' from dual union all
select 17308,'SAND' from dual union all
select 17309,'AN' from dual union all
select 1731,'USAF' from dual union all
select 17310,'AFCRL' from dual union all
select 17311,'SAND' from dual union all
select 17312,'SAND' from dual union all
select 17313,'RVSN' from dual union all
select 17314,'AN' from dual union all
select 17315,'SAND' from dual union all
select 17316,'NASA' from dual union all
select 17317,'ESRO' from dual union all
select 17318,'AN' from dual union all
select 17319,'AN' from dual union all
select 1732,'USA' from dual union all
select 17320,'1STRAD' from dual union all
select 17321,'USA' from dual union all
select 17322,'AN' from dual union all
select 17323,'ESRO' from dual union all
select 17324,'RVSN' from dual union all
select 17325,'1STRAD' from dual union all
select 17326,'RVSN' from dual union all
select 17327,'AFCRL' from dual union all
select 17328,'SAND' from dual union all
select 17329,'SAND' from dual union all
select 1733,'USA' from dual union all
select 17330,'SAND' from dual union all
select 17331,'SAND' from dual union all
select 17332,'SAND' from dual union all
select 17333,'SAND' from dual union all
select 17334,'NASA' from dual union all
select 17335,'RVSN' from dual union all
select 17336,'NASA' from dual union all
select 17337,'SRC' from dual union all
select 17338,'AN' from dual union all
select 17339,'AN' from dual union all
select 1734,'USA' from dual union all
select 17340,'RAE' from dual union all
select 17340,'WRE' from dual union all
select 17341,'RAE' from dual union all
select 17341,'WRE' from dual union all
select 17342,'AFSC' from dual union all
select 17343,'SRC' from dual union all
select 17344,'ARPA' from dual union all
select 17345,'NASA' from dual union all
select 17346,'RVSN' from dual union all
select 17347,'RVSN' from dual union all
select 17348,'AN' from dual union all
select 17349,'AN' from dual union all
select 1735,'USAF' from dual union all
select 17350,'ESRO' from dual union all
select 17351,'NASA' from dual union all
select 17352,'NASA' from dual union all
select 17353,'ESRO' from dual union all
select 17354,'NASA' from dual union all
select 17355,'AFCRL' from dual union all
select 17356,'NASA' from dual union all
select 17357,'NASA' from dual union all
select 17358,'USN' from dual union all
select 17359,'NASA' from dual union all
select 1736,'SUI' from dual union all
select 17360,'NASA' from dual union all
select 17361,'SAC' from dual union all
select 17362,'ARPA' from dual union all
select 17363,'RAE' from dual union all
select 17363,'WRE' from dual union all
select 17364,'DLR' from dual union all
select 17364,'NASA' from dual union all
select 17365,'AN' from dual union all
select 17366,'USN' from dual union all
select 17367,'FKGR12' from dual union all
select 17368,'FKGR22' from dual union all
select 17369,'AFSC' from dual union all
select 1737,'NRL' from dual union all
select 17370,'NASA' from dual union all
select 17371,'AN' from dual union all
select 17372,'AN' from dual union all
select 17373,'AN' from dual union all
select 17374,'AFSC' from dual union all
select 17375,'US7A' from dual union all
select 17376,'US7A' from dual union all
select 17377,'AN' from dual union all
select 17378,'DMA' from dual union all
select 17379,'RVSN' from dual union all
select 1738,'USA' from dual union all
select 17380,'1STRAD' from dual union all
select 17381,'RAE' from dual union all
select 17381,'WRE' from dual union all
select 17382,'USN' from dual union all
select 17383,'1STRAD' from dual;
insert into LAUNCH_AGENCY
select 17384,'RVSN' from dual union all
select 17385,'AN' from dual union all
select 17386,'AFCRL' from dual union all
select 17387,'USN' from dual union all
select 17388,'AN' from dual union all
select 17389,'AFCRL' from dual union all
select 1739,'USA' from dual union all
select 17390,'AFCRL' from dual union all
select 17391,'CNES' from dual union all
select 17392,'AFCRL' from dual union all
select 17393,'AFCRL' from dual union all
select 17394,'AFCRL' from dual union all
select 17395,'AFCRL' from dual union all
select 17396,'NASA' from dual union all
select 17397,'NASA' from dual union all
select 17398,'RAE' from dual union all
select 17398,'WRE' from dual union all
select 17399,'NASA' from dual union all
select 174,'WER836-2' from dual union all
select 1740,'USA' from dual union all
select 17400,'NASA' from dual union all
select 17401,'NASA' from dual union all
select 17402,'NASA' from dual union all
select 17403,'NASA' from dual union all
select 17404,'NASA' from dual union all
select 17405,'NASA' from dual union all
select 17406,'NASA' from dual union all
select 17407,'RVSN' from dual union all
select 17408,'NASA' from dual union all
select 17409,'CNES' from dual union all
select 1741,'ORDRSA' from dual union all
select 17410,'AN' from dual union all
select 17411,'AN' from dual union all
select 17412,'DMA' from dual union all
select 17413,'NASA' from dual union all
select 17414,'NASA' from dual union all
select 17415,'AN' from dual union all
select 17416,'USN' from dual union all
select 17417,'ESRO' from dual union all
select 17418,'RVSN' from dual union all
select 17419,'FKGR12' from dual union all
select 1742,'ORDRSA' from dual union all
select 17420,'44FA2' from dual union all
select 17421,'44FA2' from dual union all
select 17422,'44FA2' from dual union all
select 17423,'AN' from dual union all
select 17424,'AN' from dual union all
select 17425,'NASA' from dual union all
select 17426,'AN' from dual union all
select 17427,'AN' from dual union all
select 17428,'NASA' from dual union all
select 17429,'SRC' from dual union all
select 1743,'USA' from dual union all
select 17430,'NRCC' from dual union all
select 17431,'RAE' from dual union all
select 17431,'WRE' from dual union all
select 17432,'USAF' from dual union all
select 17433,'NASA' from dual union all
select 17434,'CALT' from dual union all
select 17435,'USAF' from dual union all
select 17436,'NASA' from dual union all
select 17437,'NASA' from dual union all
select 17438,'NASA' from dual union all
select 17439,'RVSN' from dual union all
select 1744,'USA' from dual union all
select 17440,'NASA' from dual union all
select 17441,'USAF' from dual union all
select 17442,'SAMSO' from dual union all
select 17443,'USAF' from dual union all
select 17444,'NASA' from dual union all
select 17445,'USN' from dual union all
select 17446,'RVSN' from dual union all
select 17447,'USAF' from dual union all
select 17448,'USAF' from dual union all
select 17449,'AFSC' from dual union all
select 1745,'USA' from dual union all
select 17450,'RVSN' from dual union all
select 17451,'USN' from dual union all
select 17452,'USAF' from dual union all
select 17453,'CALT' from dual union all
select 17454,'USAF' from dual union all
select 17455,'USAF' from dual union all
select 17456,'NASA' from dual union all
select 17457,'SAC' from dual union all
select 17458,'NASA' from dual union all
select 17458,'USAF' from dual union all
select 17459,'NASA' from dual union all
select 1746,'USA' from dual union all
select 17460,'NASA' from dual union all
select 17461,'RVSN' from dual union all
select 17462,'NASA' from dual union all
select 17463,'NASA' from dual union all
select 17464,'RVSN' from dual union all
select 17465,'SAND' from dual union all
select 17466,'USAF' from dual union all
select 17467,'USAF' from dual union all
select 17468,'RVSN' from dual union all
select 17469,'RVSN' from dual union all
select 1747,'USA' from dual union all
select 17470,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 17471,'USN' from dual union all
select 17472,'RVSN' from dual union all
select 17473,'USN' from dual union all
select 17474,'NASA' from dual union all
select 17475,'NASA' from dual union all
select 17476,'NASA' from dual union all
select 17477,'RAE' from dual union all
select 17477,'WRE' from dual union all
select 17478,'AFSC' from dual union all
select 17479,'SAMSO' from dual union all
select 1748,'ORDRSA' from dual union all
select 17480,'USAF' from dual union all
select 17481,'USAF' from dual union all
select 17482,'NASA' from dual union all
select 17483,'RVSN' from dual union all
select 17484,'SAMSO' from dual union all
select 17485,'SAMSO' from dual union all
select 17486,'USN' from dual union all
select 17487,'RVSN' from dual union all
select 17488,'AN' from dual union all
select 17489,'ISAS' from dual union all
select 1749,'ORDRSA' from dual union all
select 17490,'ISAS' from dual union all
select 17491,'NASA' from dual union all
select 17492,'NASA' from dual union all
select 17493,'STA' from dual union all
select 17494,'RVSN' from dual union all
select 17495,'STA' from dual union all
select 17496,'AN' from dual union all
select 17497,'NASA' from dual union all
select 17498,'NASA' from dual union all
select 17499,'NASA' from dual union all
select 175,'WER485-2' from dual union all
select 1750,'USA' from dual union all
select 17500,'NASA' from dual union all
select 17501,'NASA' from dual union all
select 17502,'NASA' from dual union all
select 17503,'NASA' from dual union all
select 17504,'AFSC' from dual union all
select 17505,'RVSN' from dual union all
select 17506,'AN' from dual union all
select 17507,'USN' from dual union all
select 17508,'AFSC' from dual union all
select 17509,'NASA' from dual union all
select 1751,'PARD' from dual union all
select 17510,'DASST' from dual union all
select 17511,'NASA' from dual union all
select 17512,'NASA' from dual union all
select 17513,'NASA' from dual union all
select 17514,'AN' from dual union all
select 17515,'RVSN' from dual union all
select 17516,'RVSN' from dual union all
select 17517,'RVSN' from dual union all
select 17518,'ESRO' from dual union all
select 17519,'DMA' from dual union all
select 1752,'PARD' from dual union all
select 17520,'AN' from dual union all
select 17521,'NRCC' from dual union all
select 17522,'ESRO' from dual union all
select 17523,'NRCC' from dual union all
select 17524,'AN' from dual union all
select 17525,'RAE' from dual union all
select 17525,'WRE' from dual union all
select 17526,'RVSN' from dual union all
select 17527,'GMS' from dual union all
select 17528,'NASA' from dual union all
select 17529,'CNES' from dual union all
select 1753,'PARD' from dual union all
select 17530,'NASA' from dual union all
select 17531,'NASA' from dual union all
select 17532,'ESRO' from dual union all
select 17533,'NASA' from dual union all
select 17534,'NASA' from dual union all
select 17535,'NASA' from dual union all
select 17536,'AN' from dual union all
select 17537,'RVSN' from dual union all
select 17538,'NASA' from dual union all
select 17539,'DASA' from dual union all
select 17539,'SAMSO' from dual union all
select 1754,'USA' from dual union all
select 17540,'USN' from dual union all
select 17541,'USN' from dual union all
select 17542,'USN' from dual union all
select 17543,'USN' from dual union all
select 17544,'USN' from dual union all
select 17545,'AN' from dual union all
select 17546,'AN' from dual union all
select 17547,'DASA' from dual union all
select 17547,'SAMSO' from dual union all
select 17548,'RVSN' from dual union all
select 17549,'AFSC' from dual union all
select 1755,'USA' from dual union all
select 17550,'SAMSO' from dual union all
select 17551,'1STRAD' from dual union all
select 17552,'AN' from dual union all
select 17553,'KPNO' from dual union all
select 17553,'NASA' from dual union all
select 17554,'AN' from dual union all
select 17555,'SAND' from dual union all
select 17556,'1STRAD' from dual;
insert into LAUNCH_AGENCY
select 17557,'AN' from dual union all
select 17558,'USAF' from dual union all
select 17559,'RVSN' from dual union all
select 1756,'USAF' from dual union all
select 17560,'USAF' from dual union all
select 17561,'SAND' from dual union all
select 17562,'NRCC' from dual union all
select 17563,'NRCC' from dual union all
select 17564,'ESRO' from dual union all
select 17565,'CNES' from dual union all
select 17566,'ISRO' from dual union all
select 17567,'NMC' from dual union all
select 17568,'81FA1' from dual union all
select 17569,'44FA2' from dual union all
select 1757,'ORDRSA' from dual union all
select 17570,'RVSN' from dual union all
select 17571,'INTA' from dual union all
select 17572,'NASA' from dual union all
select 17573,'SAND' from dual union all
select 17574,'NASA' from dual union all
select 17575,'INTA' from dual union all
select 17576,'DMA' from dual union all
select 17577,'USAF' from dual union all
select 17578,'SRC' from dual union all
select 17579,'INTA' from dual union all
select 1758,'ORDRSA' from dual union all
select 17580,'1STRAD' from dual union all
select 17581,'RVSN' from dual union all
select 17582,'INTA' from dual union all
select 17583,'FKGR12' from dual union all
select 17584,'USAF' from dual union all
select 17585,'USAF' from dual union all
select 17586,'NASA' from dual union all
select 17587,'INTA' from dual union all
select 17588,'DMA' from dual union all
select 17589,'INTA' from dual union all
select 1759,'SUI' from dual union all
select 17590,'SAC' from dual union all
select 17591,'DLR' from dual union all
select 17592,'RVSN' from dual union all
select 17593,'NRCC' from dual union all
select 17594,'AFCRL' from dual union all
select 17595,'NASA' from dual union all
select 17596,'NASA' from dual union all
select 17597,'NASA' from dual union all
select 17598,'USAF' from dual union all
select 17599,'NASA' from dual union all
select 176,'WEHR' from dual union all
select 1760,'SUI' from dual union all
select 17600,'NASA' from dual union all
select 17601,'INTA' from dual union all
select 17602,'RVSN' from dual union all
select 17603,'1STRAD' from dual union all
select 17604,'USAF' from dual union all
select 17605,'NASA' from dual union all
select 17606,'NASA' from dual union all
select 17607,'NASA' from dual union all
select 17608,'NASA' from dual union all
select 17609,'INTA' from dual union all
select 1761,'USA' from dual union all
select 17610,'ESRO' from dual union all
select 17611,'AFSC' from dual union all
select 17612,'ESRO' from dual union all
select 17613,'RVSN' from dual union all
select 17614,'INTA' from dual union all
select 17615,'USN' from dual union all
select 17616,'INTA' from dual union all
select 17617,'WRE' from dual union all
select 17618,'INTA' from dual union all
select 17619,'SRC' from dual union all
select 1762,'ORDRSA' from dual union all
select 17620,'DFVLR' from dual union all
select 17621,'ESRO' from dual union all
select 17622,'DLR' from dual union all
select 17623,'RVSN' from dual union all
select 17624,'RVSN' from dual union all
select 17625,'NTNF' from dual union all
select 17626,'DFVLR' from dual union all
select 17627,'NASA' from dual union all
select 17628,'NRCC' from dual union all
select 17629,'NRCC' from dual union all
select 1763,'ORDRSA' from dual union all
select 17630,'DLR' from dual union all
select 17631,'USAF' from dual union all
select 17632,'USAF' from dual union all
select 17633,'RAE' from dual union all
select 17633,'WRE' from dual union all
select 17634,'RVSN' from dual union all
select 17635,'SRC' from dual union all
select 17636,'USAF' from dual union all
select 17637,'1STRAD' from dual union all
select 17638,'RAE' from dual union all
select 17638,'WRE' from dual union all
select 17639,'NASA' from dual union all
select 1764,'USA' from dual union all
select 17640,'SRC' from dual union all
select 17641,'USAF' from dual union all
select 17642,'USAF' from dual union all
select 17643,'1STRAD' from dual union all
select 17644,'NASA' from dual;
insert into LAUNCH_AGENCY
select 17645,'RVSN' from dual union all
select 17646,'USAF' from dual union all
select 17647,'USAF' from dual union all
select 17648,'WRE' from dual union all
select 17649,'NASA' from dual union all
select 1765,'USA' from dual union all
select 17650,'NASA' from dual union all
select 17651,'USAF' from dual union all
select 17652,'NASA' from dual union all
select 17653,'NASA' from dual union all
select 17654,'NASA' from dual union all
select 17655,'NASA' from dual union all
select 17656,'RVSN' from dual union all
select 17657,'NRCC' from dual union all
select 17658,'NASA' from dual union all
select 17659,'RVSN' from dual union all
select 1766,'USA' from dual union all
select 17660,'CNES' from dual union all
select 17661,'NRCC' from dual union all
select 17662,'AFSC' from dual union all
select 17663,'NASA' from dual union all
select 17664,'DMA' from dual union all
select 17665,'CNES' from dual union all
select 17666,'USAF' from dual union all
select 17667,'RVSN' from dual union all
select 17668,'USAF' from dual union all
select 17669,'FKGR21' from dual union all
select 1767,'USA' from dual union all
select 17670,'USAF' from dual union all
select 17671,'WRE' from dual union all
select 17672,'WRE' from dual union all
select 17673,'WRE' from dual union all
select 17674,'USAF' from dual union all
select 17675,'USAF' from dual union all
select 17676,'USAF' from dual union all
select 17677,'RVSN' from dual union all
select 17678,'USAF' from dual union all
select 17679,'USAF' from dual union all
select 1768,'USAF' from dual union all
select 17680,'USAF' from dual union all
select 17681,'USAF' from dual union all
select 17682,'USAF' from dual union all
select 17683,'USAF' from dual union all
select 17684,'USAF' from dual union all
select 17685,'USAF' from dual union all
select 17686,'USAF' from dual union all
select 17687,'BRL' from dual union all
select 17688,'RVSN' from dual union all
select 17689,'BRL' from dual union all
select 1769,'ORDRSA' from dual union all
select 17690,'BRL' from dual union all
select 17691,'BRL' from dual union all
select 17692,'BRL' from dual union all
select 17693,'CALT' from dual union all
select 17694,'SAMSO' from dual union all
select 17695,'SRC' from dual union all
select 17696,'SAND' from dual union all
select 17697,'AN' from dual union all
select 17698,'USN' from dual union all
select 17699,'RVSN' from dual union all
select 177,'WER485-1' from dual union all
select 1770,'ORDRSA' from dual union all
select 17700,'USN' from dual union all
select 17701,'USN' from dual union all
select 17702,'USN' from dual union all
select 17703,'USN' from dual union all
select 17704,'USN' from dual union all
select 17705,'USN' from dual union all
select 17706,'RVSN' from dual union all
select 17707,'US7A' from dual union all
select 17708,'SAND' from dual union all
select 17709,'SAND' from dual union all
select 1771,'USA' from dual union all
select 17710,'SAND' from dual union all
select 17711,'RVSN' from dual union all
select 17712,'RVSN' from dual union all
select 17713,'RVSN' from dual union all
select 17714,'RVSN' from dual union all
select 17715,'RVSN' from dual union all
select 17716,'RVSN' from dual union all
select 17717,'RVSN' from dual union all
select 17718,'RVSN' from dual union all
select 17719,'RVSN' from dual union all
select 1772,'USA' from dual union all
select 17720,'RVSN' from dual union all
select 17721,'RVSN' from dual union all
select 17722,'RVSN' from dual union all
select 17723,'RVSN' from dual union all
select 17724,'RVSN' from dual union all
select 17725,'RVSN' from dual union all
select 17726,'RVSN' from dual union all
select 17727,'RVSN' from dual union all
select 17728,'RVSN' from dual union all
select 17729,'RVSN' from dual union all
select 1773,'USA' from dual union all
select 17730,'RVSN' from dual union all
select 17731,'RVSN' from dual union all
select 17732,'RVSN' from dual union all
select 17733,'RVSN' from dual union all
select 17734,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 17735,'RVSN' from dual union all
select 17736,'RVSN' from dual union all
select 17737,'RVSN' from dual union all
select 17738,'RVSN' from dual union all
select 17739,'RVSN' from dual union all
select 1774,'ORDRSA' from dual union all
select 17740,'RVSN' from dual union all
select 17741,'RVSN' from dual union all
select 17742,'RVSN' from dual union all
select 17743,'RVSN' from dual union all
select 17744,'RVSN' from dual union all
select 17745,'RVSN' from dual union all
select 17746,'RVSN' from dual union all
select 17747,'RVSN' from dual union all
select 17748,'RVSN' from dual union all
select 17749,'RVSN' from dual union all
select 1775,'ORDRSA' from dual union all
select 17750,'RVSN' from dual union all
select 17751,'RVSN' from dual union all
select 17752,'RVSN' from dual union all
select 17753,'MVS' from dual union all
select 17754,'MVS' from dual union all
select 17755,'MVS' from dual union all
select 17756,'MVS' from dual union all
select 17757,'MVS' from dual union all
select 17758,'RVSN' from dual union all
select 17759,'MVS' from dual union all
select 1776,'USA' from dual union all
select 17760,'MVS' from dual union all
select 17761,'RVSN' from dual union all
select 17762,'RVSN' from dual union all
select 17763,'RVSN' from dual union all
select 17764,'MVS' from dual union all
select 17765,'RVSN' from dual union all
select 17766,'MVS' from dual union all
select 17767,'MVS' from dual union all
select 17768,'MVS' from dual union all
select 17769,'RVSN' from dual union all
select 1777,'USA' from dual union all
select 17770,'MVS' from dual union all
select 17771,'MVS' from dual union all
select 17772,'MVS' from dual union all
select 17773,'RVSN' from dual union all
select 17774,'RVSN' from dual union all
select 17775,'RVSN' from dual union all
select 17776,'RVSN' from dual union all
select 17777,'MVS' from dual union all
select 17778,'USN' from dual union all
select 17779,'1STRAD' from dual union all
select 1778,'USA' from dual union all
select 17780,'USAF' from dual union all
select 17781,'DMA' from dual union all
select 17782,'USN' from dual union all
select 17783,'1STRAD' from dual union all
select 17784,'NASA' from dual union all
select 17785,'NASA' from dual union all
select 17786,'AFCRL' from dual union all
select 17787,'USAF' from dual union all
select 17788,'NRCC' from dual union all
select 17789,'AFCRL' from dual union all
select 1779,'USA' from dual union all
select 17790,'NRCC' from dual union all
select 17791,'NASA' from dual union all
select 17792,'AFCRL' from dual union all
select 17793,'AFCRL' from dual union all
select 17794,'KPNO' from dual union all
select 17794,'NRL' from dual union all
select 17795,'DMA' from dual union all
select 17796,'NASA' from dual union all
select 17797,'NRCC' from dual union all
select 17798,'USN' from dual union all
select 17799,'CNIE' from dual union all
select 178,'WER485-3' from dual union all
select 1780,'ORDRSA' from dual union all
select 17800,'RVSN' from dual union all
select 17801,'RVSN' from dual union all
select 17802,'NASA' from dual union all
select 17803,'USAF' from dual union all
select 17804,'USAF' from dual union all
select 17805,'USAF' from dual union all
select 17806,'USAF' from dual union all
select 17807,'USAF' from dual union all
select 17808,'USAF' from dual union all
select 17809,'USAF' from dual union all
select 1781,'ARDC' from dual union all
select 17810,'USAF' from dual union all
select 17811,'USAF' from dual union all
select 17812,'USAF' from dual union all
select 17813,'MSFC' from dual union all
select 17814,'GMS' from dual union all
select 17815,'GMS' from dual union all
select 17816,'CONAE' from dual union all
select 17816,'NASA' from dual union all
select 17817,'GMS' from dual union all
select 17818,'SAND' from dual union all
select 17819,'SAND' from dual union all
select 1782,'ORDRSA' from dual union all
select 17820,'SAND' from dual union all
select 17821,'MSFC' from dual union all
select 17822,'MSFC' from dual;
insert into LAUNCH_AGENCY
select 17823,'USAF' from dual union all
select 17824,'SAND' from dual union all
select 17825,'CONAE' from dual union all
select 17825,'NASA' from dual union all
select 17826,'USAF' from dual union all
select 17827,'USAF' from dual union all
select 17828,'USAF' from dual union all
select 17829,'USAF' from dual union all
select 1783,'ORDRSA' from dual union all
select 17830,'USAF' from dual union all
select 17831,'USAF' from dual union all
select 17832,'USAF' from dual union all
select 17833,'USAF' from dual union all
select 17834,'USAF' from dual union all
select 17835,'USAF' from dual union all
select 17836,'USAF' from dual union all
select 17837,'USAF' from dual union all
select 17838,'GMS' from dual union all
select 17839,'GMS' from dual union all
select 1784,'ORDRSA' from dual union all
select 17840,'CONAE' from dual union all
select 17840,'NASA' from dual union all
select 17841,'GMS' from dual union all
select 17842,'GMS' from dual union all
select 17843,'CONAE' from dual union all
select 17843,'NASA' from dual union all
select 17844,'USAF' from dual union all
select 17845,'USAF' from dual union all
select 17846,'USAF' from dual union all
select 17847,'USAF' from dual union all
select 17848,'USAF' from dual union all
select 17849,'USAF' from dual union all
select 1785,'USA' from dual union all
select 17850,'USAF' from dual union all
select 17851,'USAF' from dual union all
select 17852,'USAF' from dual union all
select 17853,'USAF' from dual union all
select 17854,'USAF' from dual union all
select 17855,'USAF' from dual union all
select 17856,'USAF' from dual union all
select 17857,'USAF' from dual union all
select 17858,'USAF' from dual union all
select 17859,'USAF' from dual union all
select 1786,'USA' from dual union all
select 17860,'USAF' from dual union all
select 17861,'USAF' from dual union all
select 17862,'USAF' from dual union all
select 17863,'GMS' from dual union all
select 17864,'GMS' from dual union all
select 17865,'GMS' from dual union all
select 17866,'GMS' from dual union all
select 17867,'USAF' from dual union all
select 17868,'GMS' from dual union all
select 17869,'GMS' from dual union all
select 1787,'USA' from dual union all
select 17870,'CONAE' from dual union all
select 17870,'NASA' from dual union all
select 17871,'GMS' from dual union all
select 17872,'NASA' from dual union all
select 17873,'NASA' from dual union all
select 17874,'USAF' from dual union all
select 17875,'USAF' from dual union all
select 17876,'USAF' from dual union all
select 17877,'USAF' from dual union all
select 17878,'USAF' from dual union all
select 17879,'USAF' from dual union all
select 1788,'USA' from dual union all
select 17880,'USAF' from dual union all
select 17881,'GMS' from dual union all
select 17882,'CONAE' from dual union all
select 17882,'NASA' from dual union all
select 17883,'GMS' from dual union all
select 17884,'GMS' from dual union all
select 17885,'GMS' from dual union all
select 17886,'GMS' from dual union all
select 17887,'GMS' from dual union all
select 17888,'GMS' from dual union all
select 17889,'USAF' from dual union all
select 1789,'ORDRSA' from dual union all
select 17890,'CONAE' from dual union all
select 17891,'USAF' from dual union all
select 17892,'USAF' from dual union all
select 17893,'USAF' from dual union all
select 17894,'USAF' from dual union all
select 17895,'USAF' from dual union all
select 17896,'USAF' from dual union all
select 17897,'USAF' from dual union all
select 17898,'USAF' from dual union all
select 17899,'GMS' from dual union all
select 179,'WER836' from dual union all
select 1790,'ORDRSA' from dual union all
select 17900,'USAF' from dual union all
select 17901,'GMS' from dual union all
select 17902,'GMS' from dual union all
select 17903,'GMS' from dual union all
select 17904,'GMS' from dual union all
select 17905,'ISAS' from dual union all
select 17906,'ISAS' from dual union all
select 17907,'GMS' from dual union all
select 17908,'NASA' from dual;
insert into LAUNCH_AGENCY
select 17909,'GMS' from dual union all
select 1791,'USAF' from dual union all
select 17910,'ERDA' from dual union all
select 17911,'USAF' from dual union all
select 17912,'USAF' from dual union all
select 17913,'USAF' from dual union all
select 17914,'USAF' from dual union all
select 17915,'USAF' from dual union all
select 17916,'USAF' from dual union all
select 17917,'USAF' from dual union all
select 17918,'USAF' from dual union all
select 17919,'USAF' from dual union all
select 1792,'USA' from dual union all
select 17920,'USAF' from dual union all
select 17921,'USAF' from dual union all
select 17922,'USAF' from dual union all
select 17923,'USAF' from dual union all
select 17924,'USAF' from dual union all
select 17925,'USAF' from dual union all
select 17926,'USAF' from dual union all
select 17927,'USAF' from dual union all
select 17928,'USAF' from dual union all
select 17929,'USAF' from dual union all
select 1793,'USA' from dual union all
select 17930,'USAF' from dual union all
select 17931,'USAF' from dual union all
select 17932,'USAF' from dual union all
select 17933,'USAF' from dual union all
select 17934,'USAF' from dual union all
select 17935,'USAF' from dual union all
select 17936,'USAF' from dual union all
select 17937,'USAF' from dual union all
select 17938,'USAF' from dual union all
select 17939,'USAF' from dual union all
select 1794,'ORDRSA' from dual union all
select 17940,'USAF' from dual union all
select 17941,'USAF' from dual union all
select 17942,'USAF' from dual union all
select 17943,'USAF' from dual union all
select 17944,'USAF' from dual union all
select 17945,'USAF' from dual union all
select 17946,'USAF' from dual union all
select 17947,'USAF' from dual union all
select 17948,'USAF' from dual union all
select 17949,'USAF' from dual union all
select 1795,'ORDRSA' from dual union all
select 17950,'PIHM' from dual union all
select 17951,'GMS' from dual union all
select 17952,'USA' from dual union all
select 17953,'MSFC' from dual union all
select 17954,'GMS' from dual union all
select 17955,'PIHM' from dual union all
select 17956,'GMS' from dual union all
select 17957,'USAF' from dual union all
select 17958,'USAF' from dual union all
select 17959,'USAF' from dual union all
select 1796,'USA' from dual union all
select 17960,'USAF' from dual union all
select 17961,'USAF' from dual union all
select 17962,'USAF' from dual union all
select 17963,'USAF' from dual union all
select 17964,'USAF' from dual union all
select 17965,'USAF' from dual union all
select 17966,'PIHM' from dual union all
select 17967,'USAF' from dual union all
select 17968,'USAF' from dual union all
select 17969,'USAF' from dual union all
select 1797,'USA' from dual union all
select 17970,'GMS' from dual union all
select 17971,'GMS' from dual union all
select 17972,'MSFC' from dual union all
select 17973,'NASA' from dual union all
select 17974,'GMS' from dual union all
select 17975,'GMS' from dual union all
select 17976,'GMS' from dual union all
select 17977,'GMS' from dual union all
select 17978,'USAF' from dual union all
select 17979,'USAF' from dual union all
select 1798,'USA' from dual union all
select 17980,'USAF' from dual union all
select 17981,'USAF' from dual union all
select 17982,'USAF' from dual union all
select 17983,'USAF' from dual union all
select 17984,'USAF' from dual union all
select 17985,'CONAE' from dual union all
select 17985,'NASA' from dual union all
select 17986,'CNAE' from dual union all
select 17987,'USAF' from dual union all
select 17988,'GMS' from dual union all
select 17989,'USAF' from dual union all
select 1799,'USA' from dual union all
select 17990,'USAF' from dual union all
select 17991,'USAF' from dual union all
select 17992,'USAF' from dual union all
select 17993,'GMS' from dual union all
select 17994,'ERDA' from dual union all
select 17995,'ERDA' from dual union all
select 17996,'USAF' from dual union all
select 17997,'USAF' from dual union all
select 17998,'USAF' from dual;
insert into LAUNCH_AGENCY
select 17999,'GMS' from dual union all
select 18,'WEHR' from dual union all
select 180,'WER836' from dual union all
select 1800,'USA' from dual union all
select 18000,'USAF' from dual union all
select 18001,'USAF' from dual union all
select 18002,'NASA' from dual union all
select 18003,'NASA' from dual union all
select 18004,'GMS' from dual union all
select 18005,'GMS' from dual union all
select 18006,'GMS' from dual union all
select 18007,'GMS' from dual union all
select 18008,'USA' from dual union all
select 18009,'USA' from dual union all
select 1801,'ORDRSA' from dual union all
select 18010,'GMS' from dual union all
select 18011,'USA' from dual union all
select 18012,'USA' from dual union all
select 18013,'USA' from dual union all
select 18014,'USA' from dual union all
select 18015,'USN' from dual union all
select 18016,'USAF' from dual union all
select 18017,'USAF' from dual union all
select 18018,'USAF' from dual union all
select 18019,'ERDA' from dual union all
select 1802,'USA' from dual union all
select 18020,'CONAE' from dual union all
select 18021,'GMS' from dual union all
select 18022,'GMS' from dual union all
select 18023,'NASA' from dual union all
select 18024,'CONAE' from dual union all
select 18024,'NASA' from dual union all
select 18025,'CONAE' from dual union all
select 18026,'CONAE' from dual union all
select 18027,'CONAE' from dual union all
select 18027,'NASA' from dual union all
select 18028,'CONAE' from dual union all
select 18028,'NASA' from dual union all
select 18029,'CONAE' from dual union all
select 18029,'NASA' from dual union all
select 1803,'ORDRSA' from dual union all
select 18030,'CNAE' from dual union all
select 18031,'MSFC' from dual union all
select 18032,'CNAE' from dual union all
select 18033,'CNAE' from dual union all
select 18034,'CNAE' from dual union all
select 18035,'CNAE' from dual union all
select 18036,'CNAE' from dual union all
select 18037,'CNAE' from dual union all
select 18038,'CNAE' from dual union all
select 18039,'CNAE' from dual union all
select 1804,'USA' from dual union all
select 18040,'CNAE' from dual union all
select 18041,'GMS' from dual union all
select 18042,'CNAE' from dual union all
select 18043,'NASA' from dual union all
select 18044,'NASA' from dual union all
select 18045,'NASA' from dual union all
select 18046,'NASA' from dual union all
select 18047,'NASA' from dual union all
select 18048,'NASA' from dual union all
select 18049,'NASA' from dual union all
select 1805,'ORDRSA' from dual union all
select 18050,'NASA' from dual union all
select 18051,'GMS' from dual union all
select 18052,'NASA' from dual union all
select 18053,'NASA' from dual union all
select 18054,'NASA' from dual union all
select 18055,'NASA' from dual union all
select 18056,'NASA' from dual union all
select 18057,'NASA' from dual union all
select 18058,'NASA' from dual union all
select 18059,'NASA' from dual union all
select 1806,'USA' from dual union all
select 18060,'NASA' from dual union all
select 18061,'NASA' from dual union all
select 18062,'USAF' from dual union all
select 18063,'NASA' from dual union all
select 18064,'NASA' from dual union all
select 18065,'NASA' from dual union all
select 18066,'NASA' from dual union all
select 18067,'NASA' from dual union all
select 18068,'NASA' from dual union all
select 18069,'NASA' from dual union all
select 1807,'USA' from dual union all
select 18070,'NASA' from dual union all
select 18071,'NASA' from dual union all
select 18072,'NASA' from dual union all
select 18073,'USAF' from dual union all
select 18074,'NASA' from dual union all
select 18075,'NASA' from dual union all
select 18076,'NASA' from dual union all
select 18077,'NASA' from dual union all
select 18078,'NASA' from dual union all
select 18079,'NASA' from dual union all
select 1808,'ORDRSA' from dual union all
select 18080,'NASA' from dual union all
select 18081,'NASA' from dual union all
select 18082,'NASA' from dual union all
select 18083,'NASA' from dual;
insert into LAUNCH_AGENCY
select 18084,'USAF' from dual union all
select 18085,'NASA' from dual union all
select 18086,'NASA' from dual union all
select 18087,'NASA' from dual union all
select 18088,'NASA' from dual union all
select 18089,'NASA' from dual union all
select 1809,'ORDRSA' from dual union all
select 18090,'NASA' from dual union all
select 18091,'NASA' from dual union all
select 18092,'NASA' from dual union all
select 18093,'NASA' from dual union all
select 18094,'NASA' from dual union all
select 18095,'USAF' from dual union all
select 18096,'USAF' from dual union all
select 18097,'USAF' from dual union all
select 18098,'USAF' from dual union all
select 18099,'USAF' from dual union all
select 181,'WER836-1' from dual union all
select 1810,'ORDRSA' from dual union all
select 18100,'USAF' from dual union all
select 18101,'USAF' from dual union all
select 18102,'USAF' from dual union all
select 18103,'ISRO' from dual union all
select 18104,'GMS' from dual union all
select 18105,'GMS' from dual union all
select 18106,'GMS' from dual union all
select 18107,'CONAE' from dual union all
select 18108,'GMS' from dual union all
select 18109,'GMS' from dual union all
select 1811,'ORDRSA' from dual union all
select 18110,'USAF' from dual union all
select 18111,'USAF' from dual union all
select 18112,'USAF' from dual union all
select 18113,'USAF' from dual union all
select 18114,'USAF' from dual union all
select 18115,'USAF' from dual union all
select 18116,'USAF' from dual union all
select 18117,'USAF' from dual union all
select 18118,'USAF' from dual union all
select 18119,'USAF' from dual union all
select 1812,'USA' from dual union all
select 18120,'USAF' from dual union all
select 18121,'USAF' from dual union all
select 18122,'USAF' from dual union all
select 18123,'USAF' from dual union all
select 18124,'USAF' from dual union all
select 18125,'USAF' from dual union all
select 18126,'GMS' from dual union all
select 18127,'CNES' from dual union all
select 18128,'CONAE' from dual union all
select 18129,'CONAE' from dual union all
select 1813,'USA' from dual union all
select 18130,'CONAE' from dual union all
select 18131,'USAF' from dual union all
select 18132,'USAF' from dual union all
select 18133,'USAF' from dual union all
select 18134,'USAF' from dual union all
select 18135,'USAF' from dual union all
select 18136,'USAF' from dual union all
select 18137,'USAF' from dual union all
select 18138,'USAF' from dual union all
select 18139,'USAF' from dual union all
select 1814,'USA' from dual union all
select 18140,'CONAE' from dual union all
select 18141,'MSFC' from dual union all
select 18142,'GMS' from dual union all
select 18143,'NASA' from dual union all
select 18144,'CONAE' from dual union all
select 18145,'USAF' from dual union all
select 18146,'USAF' from dual union all
select 18147,'CONAE' from dual union all
select 18148,'GMS' from dual union all
select 18149,'USAF' from dual union all
select 1815,'USA' from dual union all
select 18150,'USAF' from dual union all
select 18151,'USAF' from dual union all
select 18152,'USAF' from dual union all
select 18153,'USAF' from dual union all
select 18154,'USAF' from dual union all
select 18155,'USAF' from dual union all
select 18156,'GMS' from dual union all
select 18157,'MSFC' from dual union all
select 18158,'GMS' from dual union all
select 18159,'GMS' from dual union all
select 1816,'USA' from dual union all
select 18160,'GMS' from dual union all
select 18161,'GMS' from dual union all
select 18162,'GMS' from dual union all
select 18163,'GMS' from dual union all
select 18164,'GMS' from dual union all
select 18165,'NASA' from dual union all
select 18165,'PSU' from dual union all
select 18166,'GMS' from dual union all
select 18167,'USAF' from dual union all
select 18168,'SRC' from dual union all
select 18169,'GMS' from dual union all
select 1817,'PARD' from dual union all
select 18170,'GMS' from dual union all
select 18171,'USAF' from dual union all
select 18172,'USAF' from dual;
insert into LAUNCH_AGENCY
select 18173,'GMS' from dual union all
select 18174,'GMS' from dual union all
select 18175,'GMS' from dual union all
select 18176,'GMS' from dual union all
select 18177,'GMS' from dual union all
select 18178,'USAF' from dual union all
select 18179,'GMS' from dual union all
select 1818,'PARD' from dual union all
select 18180,'USAF' from dual union all
select 18181,'GMS' from dual union all
select 18182,'GMS' from dual union all
select 18183,'GMS' from dual union all
select 18184,'NASA' from dual union all
select 18184,'PSU' from dual union all
select 18185,'NASA' from dual union all
select 18186,'NASA' from dual union all
select 18187,'GMS' from dual union all
select 18188,'GMS' from dual union all
select 18189,'SRC' from dual union all
select 1819,'USA' from dual union all
select 18190,'USAF' from dual union all
select 18191,'MSFC' from dual union all
select 18192,'ESSA' from dual union all
select 18192,'NASA' from dual union all
select 18193,'MSFC' from dual union all
select 18194,'GMS' from dual union all
select 18195,'MSFC' from dual union all
select 18196,'MSFC' from dual union all
select 18197,'MSFC' from dual union all
select 18198,'NASA' from dual union all
select 18198,'PSU' from dual union all
select 18199,'GMS' from dual union all
select 182,'WER836-1' from dual union all
select 1820,'USA' from dual union all
select 18200,'GMS' from dual union all
select 18201,'GMS' from dual union all
select 18202,'GMS' from dual union all
select 18203,'ESSA' from dual union all
select 18203,'NASA' from dual union all
select 18204,'GMS' from dual union all
select 18205,'SRC' from dual union all
select 18206,'GMS' from dual union all
select 18207,'GMS' from dual union all
select 18208,'SRC' from dual union all
select 18209,'GMS' from dual union all
select 1821,'ORDRSA' from dual union all
select 18210,'GMS' from dual union all
select 18211,'GMS' from dual union all
select 18212,'GMS' from dual union all
select 18213,'GMS' from dual union all
select 18214,'GMS' from dual union all
select 18215,'SRC' from dual union all
select 18216,'AN' from dual union all
select 18217,'AN' from dual union all
select 18218,'GMS' from dual union all
select 18219,'GMS' from dual union all
select 1822,'ORDRSA' from dual union all
select 18220,'GMS' from dual union all
select 18221,'GMS' from dual union all
select 18222,'NASA' from dual union all
select 18223,'NASA' from dual union all
select 18224,'SRC' from dual union all
select 18225,'GMS' from dual union all
select 18226,'ESRO' from dual union all
select 18227,'GMS' from dual union all
select 18228,'AN' from dual union all
select 18229,'GMS' from dual union all
select 1823,'USA' from dual union all
select 18230,'GMS' from dual union all
select 18231,'GMS' from dual union all
select 18232,'AN' from dual union all
select 18233,'GMS' from dual union all
select 18234,'AN' from dual union all
select 18235,'GMS' from dual union all
select 18236,'GMS' from dual union all
select 18237,'ESRO' from dual union all
select 18238,'GMS' from dual union all
select 18239,'AN' from dual union all
select 1824,'USA' from dual union all
select 18240,'AN' from dual union all
select 18241,'GMS' from dual union all
select 18242,'GMS' from dual union all
select 18243,'GMS' from dual union all
select 18244,'GMS' from dual union all
select 18245,'AN' from dual union all
select 18246,'GMS' from dual union all
select 18247,'GMS' from dual union all
select 18248,'AN' from dual union all
select 18249,'GMS' from dual union all
select 1825,'USA' from dual union all
select 18250,'AN' from dual union all
select 18251,'AN' from dual union all
select 18252,'GMS' from dual union all
select 18253,'GMS' from dual union all
select 18254,'GMS' from dual union all
select 18255,'GMS' from dual union all
select 18256,'GMS' from dual union all
select 18257,'NASA' from dual union all
select 18257,'RTG' from dual union all
select 18258,'GMS' from dual;
insert into LAUNCH_AGENCY
select 18259,'ESSA' from dual union all
select 18259,'NASA' from dual union all
select 1826,'USA' from dual union all
select 18260,'ESSA' from dual union all
select 18260,'NASA' from dual union all
select 18261,'SRC' from dual union all
select 18262,'GMS' from dual union all
select 18263,'SRC' from dual union all
select 18264,'GMS' from dual union all
select 18265,'SRC' from dual union all
select 18266,'GMS' from dual union all
select 18267,'SRC' from dual union all
select 18268,'RVSN' from dual union all
select 18269,'RVSN' from dual union all
select 1827,'USA' from dual union all
select 18270,'RVSN' from dual union all
select 18271,'RVSN' from dual union all
select 18272,'RVSN' from dual union all
select 18273,'NASA' from dual union all
select 18274,'AFSC' from dual union all
select 18275,'RVSN' from dual union all
select 18276,'NASA' from dual union all
select 18277,'USAF' from dual union all
select 18278,'NASA' from dual union all
select 18279,'RVSN' from dual union all
select 1828,'USA' from dual union all
select 18280,'USAF' from dual union all
select 18281,'NASA' from dual union all
select 18282,'RVSN' from dual union all
select 18283,'NASA' from dual union all
select 18284,'RVSN' from dual union all
select 18285,'NASA' from dual union all
select 18286,'AFSC' from dual union all
select 18287,'RVSN' from dual union all
select 18288,'RVSN' from dual union all
select 18289,'RVSN' from dual union all
select 1829,'PARD' from dual union all
select 18290,'RVSN' from dual union all
select 18291,'RVSN' from dual union all
select 18292,'AFSC' from dual union all
select 18293,'USAF' from dual union all
select 18294,'RVSN' from dual union all
select 18295,'RVSN' from dual union all
select 18296,'RVSN' from dual union all
select 18297,'NASA' from dual union all
select 18298,'RVSN' from dual union all
select 18299,'RVSN' from dual union all
select 183,'WER836-1' from dual union all
select 1830,'USA' from dual union all
select 18300,'RVSN' from dual union all
select 18301,'RVSN' from dual union all
select 18302,'RVSN' from dual union all
select 18303,'AFSC' from dual union all
select 18304,'USAF' from dual union all
select 18305,'RVSN' from dual union all
select 18306,'AFSC' from dual union all
select 18307,'RVSN' from dual union all
select 18308,'USAF' from dual union all
select 18309,'RVSN' from dual union all
select 1831,'USA' from dual union all
select 18310,'NASA' from dual union all
select 18311,'RVSN' from dual union all
select 18312,'NASA' from dual union all
select 18313,'USAF' from dual union all
select 18314,'RVSN' from dual union all
select 18315,'RVSN' from dual union all
select 18316,'RVSN' from dual union all
select 18317,'AFSC' from dual union all
select 18318,'USAF' from dual union all
select 18319,'RVSN' from dual union all
select 1832,'USA' from dual union all
select 18320,'NASA' from dual union all
select 18321,'RVSN' from dual union all
select 18322,'RVSN' from dual union all
select 18323,'NASA' from dual union all
select 18324,'RVSN' from dual union all
select 18325,'RVSN' from dual union all
select 18326,'NASA' from dual union all
select 18327,'RVSN' from dual union all
select 18328,'RVSN' from dual union all
select 18329,'10ADS' from dual union all
select 1833,'USA' from dual union all
select 18330,'USAF' from dual union all
select 18331,'NASA' from dual union all
select 18332,'USAF' from dual union all
select 18333,'RVSN' from dual union all
select 18334,'RVSN' from dual union all
select 18335,'NASA' from dual union all
select 18336,'NASA' from dual union all
select 18337,'RVSN' from dual union all
select 18338,'RVSN' from dual union all
select 18339,'RVSN' from dual union all
select 1834,'USAF' from dual union all
select 18340,'RVSN' from dual union all
select 18341,'AFSC' from dual union all
select 18342,'RVSN' from dual union all
select 18343,'RVSN' from dual union all
select 18344,'RVSN' from dual union all
select 18345,'RVSN' from dual union all
select 18346,'USAF' from dual;
insert into LAUNCH_AGENCY
select 18347,'RVSN' from dual union all
select 18348,'RVSN' from dual union all
select 18349,'USAF' from dual union all
select 1835,'USA' from dual union all
select 18350,'AFSC' from dual union all
select 18351,'RVSN' from dual union all
select 18352,'RVSN' from dual union all
select 18353,'RVSN' from dual union all
select 18354,'RVSN' from dual union all
select 18355,'RVSN' from dual union all
select 18356,'RVSN' from dual union all
select 18357,'RVSN' from dual union all
select 18358,'RVSN' from dual union all
select 18359,'RVSN' from dual union all
select 1836,'USA' from dual union all
select 18360,'RVSN' from dual union all
select 18361,'RVSN' from dual union all
select 18362,'AFSC' from dual union all
select 18363,'RVSN' from dual union all
select 18364,'AFSC' from dual union all
select 18365,'RVSN' from dual union all
select 18366,'NASA' from dual union all
select 18367,'RVSN' from dual union all
select 18368,'NASA' from dual union all
select 18369,'RVSN' from dual union all
select 1837,'USA' from dual union all
select 18370,'RVSN' from dual union all
select 18371,'RVSN' from dual union all
select 18372,'USAF' from dual union all
select 18373,'RVSN' from dual union all
select 18374,'RVSN' from dual union all
select 18375,'RVSN' from dual union all
select 18376,'RVSN' from dual union all
select 18377,'RVSN' from dual union all
select 18378,'RVSN' from dual union all
select 18379,'MRN' from dual union all
select 1838,'USAF' from dual union all
select 18380,'MRN' from dual union all
select 18381,'MRN' from dual union all
select 18382,'ISAS' from dual union all
select 18383,'USN' from dual union all
select 18384,'MRN' from dual union all
select 18385,'ISAS' from dual union all
select 18386,'MRN' from dual union all
select 18387,'MRN' from dual union all
select 18388,'MRN' from dual union all
select 18389,'MRN' from dual union all
select 1839,'USAF' from dual union all
select 18390,'USN' from dual union all
select 18391,'MRN' from dual union all
select 18392,'USA' from dual union all
select 18393,'MRN' from dual union all
select 18394,'MRN' from dual union all
select 18395,'MRN' from dual union all
select 18396,'RAE' from dual union all
select 18397,'MRN' from dual union all
select 18398,'DASA' from dual union all
select 18399,'DASA' from dual union all
select 184,'WER836-1' from dual union all
select 1840,'ORDRSA' from dual union all
select 18400,'MRN' from dual union all
select 18401,'MRN' from dual union all
select 18402,'MRN' from dual union all
select 18403,'MRN' from dual union all
select 18404,'MRN' from dual union all
select 18405,'USN' from dual union all
select 18406,'RAE' from dual union all
select 18407,'MRN' from dual union all
select 18408,'MRN' from dual union all
select 18409,'MRN' from dual union all
select 1841,'USAF' from dual union all
select 18410,'MRN' from dual union all
select 18411,'MRN' from dual union all
select 18412,'MRN' from dual union all
select 18413,'USN' from dual union all
select 18414,'RAE' from dual union all
select 18415,'MRN' from dual union all
select 18416,'MRN' from dual union all
select 18417,'MRN' from dual union all
select 18418,'INTA' from dual union all
select 18419,'MRN' from dual union all
select 1842,'USA' from dual union all
select 18420,'USA' from dual union all
select 18421,'ISRO' from dual union all
select 18422,'MRN' from dual union all
select 18423,'MRN' from dual union all
select 18424,'MRN' from dual union all
select 18425,'MRN' from dual union all
select 18426,'MRN' from dual union all
select 18427,'MRN' from dual union all
select 18428,'RVSN' from dual union all
select 18429,'MRN' from dual union all
select 1843,'USA' from dual union all
select 18430,'MRN' from dual union all
select 18431,'USA' from dual union all
select 18432,'MRN' from dual union all
select 18433,'MRN' from dual union all
select 18434,'MRN' from dual union all
select 18435,'MRN' from dual union all
select 18436,'MRN' from dual;
insert into LAUNCH_AGENCY
select 18437,'USN' from dual union all
select 18438,'MRN' from dual union all
select 18439,'USN' from dual union all
select 1844,'USA' from dual union all
select 18440,'NRL' from dual union all
select 18441,'MRN' from dual union all
select 18442,'ISAS' from dual union all
select 18443,'MRN' from dual union all
select 18444,'USA' from dual union all
select 18445,'MRN' from dual union all
select 18446,'MRN' from dual union all
select 18447,'USN' from dual union all
select 18448,'STA' from dual union all
select 18449,'MRN' from dual union all
select 1845,'ORDRSA' from dual union all
select 18450,'MRN' from dual union all
select 18451,'MRN' from dual union all
select 18452,'MRN' from dual union all
select 18453,'MRN' from dual union all
select 18454,'MRN' from dual union all
select 18455,'MRN' from dual union all
select 18456,'UKMO' from dual union all
select 18457,'ISAS' from dual union all
select 18458,'RAE' from dual union all
select 18459,'MRN' from dual union all
select 1846,'PARD' from dual union all
select 18460,'MRN' from dual union all
select 18461,'PIHM' from dual union all
select 18462,'PIHM' from dual union all
select 18463,'MRN' from dual union all
select 18464,'PIHM' from dual union all
select 18465,'USA' from dual union all
select 18466,'MRN' from dual union all
select 18467,'MRN' from dual union all
select 18468,'AFSC' from dual union all
select 18469,'MRN' from dual union all
select 1847,'PARD' from dual union all
select 18470,'MRN' from dual union all
select 18471,'MRN' from dual union all
select 18472,'RAE' from dual union all
select 18473,'MRN' from dual union all
select 18474,'USAF' from dual union all
select 18475,'USA' from dual union all
select 18476,'MRN' from dual union all
select 18477,'MRN' from dual union all
select 18478,'MRN' from dual union all
select 18479,'MRN' from dual union all
select 1848,'USA' from dual union all
select 18480,'AFCRL' from dual union all
select 18481,'AFCRL' from dual union all
select 18482,'USA' from dual union all
select 18483,'MRN' from dual union all
select 18484,'USN' from dual union all
select 18485,'NASA' from dual union all
select 18485,'RTG' from dual union all
select 18486,'USN' from dual union all
select 18487,'USN' from dual union all
select 18488,'MRN' from dual union all
select 18489,'MRN' from dual union all
select 1849,'USA' from dual union all
select 18490,'RAE' from dual union all
select 18491,'MRN' from dual union all
select 18492,'MRN' from dual union all
select 18493,'MRN' from dual union all
select 18494,'USA' from dual union all
select 18495,'MRN' from dual union all
select 18496,'MRN' from dual union all
select 18497,'MRN' from dual union all
select 18498,'MRN' from dual union all
select 18499,'AFSC' from dual union all
select 185,'WER836-1' from dual union all
select 1850,'USA' from dual union all
select 18500,'MRN' from dual union all
select 18501,'INTA' from dual union all
select 18502,'MRN' from dual union all
select 18503,'KPNO' from dual union all
select 18504,'RVSN' from dual union all
select 18505,'UKMO' from dual union all
select 18506,'MRN' from dual union all
select 18507,'MRN' from dual union all
select 18508,'MRN' from dual union all
select 18509,'NASA' from dual union all
select 1851,'USA' from dual union all
select 18510,'LARCN' from dual union all
select 18511,'USAF' from dual union all
select 18512,'USAF' from dual union all
select 18513,'USAF' from dual union all
select 18514,'USAF' from dual union all
select 18515,'USAF' from dual union all
select 18516,'USA' from dual union all
select 18517,'USA' from dual union all
select 18518,'MRN' from dual union all
select 18519,'USA' from dual union all
select 1852,'PARD' from dual union all
select 18520,'USA' from dual union all
select 18521,'MRN' from dual union all
select 18522,'MRN' from dual union all
select 18523,'MRN' from dual union all
select 18524,'RAE' from dual union all
select 18524,'WRE' from dual;
insert into LAUNCH_AGENCY
select 18525,'MRN' from dual union all
select 18526,'MRN' from dual union all
select 18527,'MRN' from dual union all
select 18528,'MRN' from dual union all
select 18529,'MRN' from dual union all
select 1853,'USAF' from dual union all
select 18530,'MRN' from dual union all
select 18531,'MRN' from dual union all
select 18532,'NASA' from dual union all
select 18533,'UKMO' from dual union all
select 18534,'MRN' from dual union all
select 18535,'USN' from dual union all
select 18536,'MRN' from dual union all
select 18537,'MRN' from dual union all
select 18538,'NASA' from dual union all
select 18539,'MRN' from dual union all
select 1854,'ORDRSA' from dual union all
select 18540,'USA' from dual union all
select 18541,'SUPA' from dual union all
select 18542,'MRN' from dual union all
select 18543,'USN' from dual union all
select 18544,'MRN' from dual union all
select 18545,'MRN' from dual union all
select 18546,'MRN' from dual union all
select 18547,'MRN' from dual union all
select 18548,'MRN' from dual union all
select 18549,'USN' from dual union all
select 1855,'ORDRSA' from dual union all
select 18550,'USA' from dual union all
select 18551,'MRN' from dual union all
select 18552,'NASA' from dual union all
select 18553,'NASA' from dual union all
select 18554,'RAE' from dual union all
select 18555,'MRN' from dual union all
select 18556,'NRCC' from dual union all
select 18557,'ESRO' from dual union all
select 18558,'MRN' from dual union all
select 18559,'USN' from dual union all
select 1856,'PARD' from dual union all
select 18560,'MRN' from dual union all
select 18561,'MRN' from dual union all
select 18562,'USA' from dual union all
select 18563,'USA' from dual union all
select 18564,'MRN' from dual union all
select 18565,'USA' from dual union all
select 18566,'USA' from dual union all
select 18567,'MRN' from dual union all
select 18568,'FKGR13' from dual union all
select 18569,'MRN' from dual union all
select 1857,'USA' from dual union all
select 18570,'USN' from dual union all
select 18571,'RAE' from dual union all
select 18572,'MRN' from dual union all
select 18573,'MRN' from dual union all
select 18574,'MRN' from dual union all
select 18575,'MRN' from dual union all
select 18576,'MRN' from dual union all
select 18577,'MRN' from dual union all
select 18578,'MRN' from dual union all
select 18579,'NASA' from dual union all
select 1858,'USA' from dual union all
select 18580,'MRN' from dual union all
select 18581,'MRN' from dual union all
select 18582,'MRN' from dual union all
select 18583,'MRN' from dual union all
select 18584,'MRN' from dual union all
select 18585,'USN' from dual union all
select 18586,'MRN' from dual union all
select 18587,'MRN' from dual union all
select 18588,'USA' from dual union all
select 18589,'MRN' from dual union all
select 1859,'USA' from dual union all
select 18590,'MRN' from dual union all
select 18591,'MRN' from dual union all
select 18592,'NASA' from dual union all
select 18593,'MRN' from dual union all
select 18594,'USA' from dual union all
select 18595,'MRN' from dual union all
select 18596,'MRN' from dual union all
select 18597,'MRN' from dual union all
select 18598,'RVSN' from dual union all
select 18599,'RVSN' from dual union all
select 186,'WER836-1' from dual union all
select 1860,'USA' from dual union all
select 18600,'RVSN' from dual union all
select 18601,'RVSN' from dual union all
select 18602,'RVSN' from dual union all
select 18603,'RVSN' from dual union all
select 18604,'RVSN' from dual union all
select 18605,'ELDO' from dual union all
select 18606,'RVSN' from dual union all
select 18607,'RVSN' from dual union all
select 18608,'NASA' from dual union all
select 18609,'ISAS' from dual union all
select 1861,'USA' from dual union all
select 18610,'RVSN' from dual union all
select 18611,'RVSN' from dual union all
select 18612,'NASA' from dual union all
select 18612,'RTG' from dual union all
select 18613,'STA' from dual;
insert into LAUNCH_AGENCY
select 18614,'STA' from dual union all
select 18615,'USA' from dual union all
select 18616,'SUPA' from dual union all
select 18617,'ESRO' from dual union all
select 18618,'ESRO' from dual union all
select 18619,'ESRO' from dual union all
select 1862,'USA' from dual union all
select 18620,'ESRO' from dual union all
select 18621,'ESRO' from dual union all
select 18622,'UKMO' from dual union all
select 18623,'UKMO' from dual union all
select 18624,'CNES' from dual union all
select 18625,'UKMO' from dual union all
select 18626,'UKMO' from dual union all
select 18627,'UKMO' from dual union all
select 18628,'UKMO' from dual union all
select 18629,'UKMO' from dual union all
select 1863,'USA' from dual union all
select 18630,'UKMO' from dual union all
select 18631,'UKMO' from dual union all
select 18632,'UKMO' from dual union all
select 18633,'USA' from dual union all
select 18634,'USA' from dual union all
select 18635,'USA' from dual union all
select 18636,'CNES' from dual union all
select 18637,'ISAS' from dual union all
select 18638,'STA' from dual union all
select 18639,'UKMO' from dual union all
select 1864,'ORDRSA' from dual union all
select 18640,'ONERA' from dual union all
select 18641,'USA' from dual union all
select 18642,'USA' from dual union all
select 18643,'USA' from dual union all
select 18644,'WRE' from dual union all
select 18645,'INTA' from dual union all
select 18646,'USAF' from dual union all
select 18647,'TSAO' from dual union all
select 18648,'TSAO' from dual union all
select 18649,'INTA' from dual union all
select 1865,'USA' from dual union all
select 18650,'INTA' from dual union all
select 18651,'INTA' from dual union all
select 18652,'USA' from dual union all
select 18653,'IIAE' from dual union all
select 18654,'IIAE' from dual union all
select 18655,'IIAE' from dual union all
select 18656,'BWEHR' from dual union all
select 18657,'SECT' from dual union all
select 18658,'CALT' from dual union all
select 18659,'DMA' from dual union all
select 1866,'ORDRSA' from dual union all
select 18660,'DMA' from dual union all
select 18661,'DMA' from dual union all
select 18662,'RVSN' from dual union all
select 18663,'RVSN' from dual union all
select 18664,'RVSN' from dual union all
select 18665,'RVSN' from dual union all
select 18666,'RVSN' from dual union all
select 18667,'USAF' from dual union all
select 18668,'NASA' from dual union all
select 18669,'NASA' from dual union all
select 1867,'ORDRSA' from dual union all
select 18670,'USAF' from dual union all
select 18671,'1STRAD' from dual union all
select 18672,'SRC' from dual union all
select 18673,'SAMSO' from dual union all
select 18674,'SAMSO' from dual union all
select 18675,'NASA' from dual union all
select 18676,'NASA' from dual union all
select 18677,'RVSN' from dual union all
select 18678,'CNES' from dual union all
select 18679,'NASDA' from dual union all
select 1868,'ORDRSA' from dual union all
select 18680,'NASA' from dual union all
select 18681,'NASA' from dual union all
select 18682,'NASA' from dual union all
select 18683,'NASA' from dual union all
select 18684,'NASA' from dual union all
select 18685,'CNES' from dual union all
select 18686,'CNES' from dual union all
select 18687,'CNES' from dual union all
select 18688,'RVSN' from dual union all
select 18689,'STA' from dual union all
select 1869,'USA' from dual union all
select 18690,'STA' from dual union all
select 18691,'NASA' from dual union all
select 18692,'NASA' from dual union all
select 18693,'STA' from dual union all
select 18694,'SRC' from dual union all
select 18695,'NASA' from dual union all
select 18696,'NASA' from dual union all
select 18697,'NRCC' from dual union all
select 18698,'NASA' from dual union all
select 18699,'RVSN' from dual union all
select 187,'WEHR' from dual union all
select 1870,'USA' from dual union all
select 18700,'ISAS' from dual union all
select 18701,'USAF' from dual union all
select 18702,'ISAS' from dual union all
select 18703,'NRCC' from dual;
insert into LAUNCH_AGENCY
select 18704,'NASA' from dual union all
select 18705,'NASA' from dual union all
select 18706,'NASA' from dual union all
select 18707,'NASA' from dual union all
select 18708,'NASA' from dual union all
select 18709,'NASA' from dual union all
select 1871,'USAF' from dual union all
select 18710,'RVSN' from dual union all
select 18711,'NASA' from dual union all
select 18712,'SRC' from dual union all
select 18713,'SRC' from dual union all
select 18714,'NASA' from dual union all
select 18715,'USN' from dual union all
select 18716,'NRCC' from dual union all
select 18717,'1STRAD' from dual union all
select 18718,'USN' from dual union all
select 18719,'CNES' from dual union all
select 1872,'USA' from dual union all
select 18720,'NASA' from dual union all
select 18721,'RVSN' from dual union all
select 18722,'ESRO' from dual union all
select 18723,'ESRO' from dual union all
select 18724,'ESRO' from dual union all
select 18725,'ISRO' from dual union all
select 18726,'SAMSO' from dual union all
select 18727,'NASA' from dual union all
select 18728,'SECT' from dual union all
select 18729,'USAF' from dual union all
select 1873,'USA' from dual union all
select 18730,'VMF' from dual union all
select 18731,'NASA' from dual union all
select 18732,'RVSN' from dual union all
select 18733,'AN' from dual union all
select 18734,'NASA' from dual union all
select 18735,'ARPA' from dual union all
select 18735,'USAF' from dual union all
select 18736,'ARPA' from dual union all
select 18736,'USAF' from dual union all
select 18737,'AN' from dual union all
select 18738,'CNES' from dual union all
select 18739,'NRCC' from dual union all
select 1874,'ORDRSA' from dual union all
select 18740,'AFSC' from dual union all
select 18741,'NASA' from dual union all
select 18742,'NASA' from dual union all
select 18743,'RVSN' from dual union all
select 18744,'USAF' from dual union all
select 18745,'NASA' from dual union all
select 18746,'AN' from dual union all
select 18747,'AN' from dual union all
select 18748,'NASA' from dual union all
select 18749,'AN' from dual union all
select 1875,'ORDRSA' from dual union all
select 18750,'NRCC' from dual union all
select 18751,'AN' from dual union all
select 18752,'AN' from dual union all
select 18753,'NRCC' from dual union all
select 18754,'RVSN' from dual union all
select 18755,'NASA' from dual union all
select 18756,'ARPA' from dual union all
select 18757,'1STRAD' from dual union all
select 18757,'ALCS' from dual union all
select 18758,'AN' from dual union all
select 18759,'AN' from dual union all
select 1876,'USAF' from dual union all
select 18760,'AN' from dual union all
select 18761,'AN' from dual union all
select 18762,'AN' from dual union all
select 18763,'NASA' from dual union all
select 18764,'NASA' from dual union all
select 18765,'RVSN' from dual union all
select 18766,'AN' from dual union all
select 18767,'AN' from dual union all
select 18768,'ARPA' from dual union all
select 18769,'DFVLR' from dual union all
select 1877,'USAF' from dual union all
select 18770,'ESRO' from dual union all
select 18771,'ESRO' from dual union all
select 18772,'DFVLR' from dual union all
select 18773,'SRC' from dual union all
select 18774,'CNES' from dual union all
select 18775,'NASA' from dual union all
select 18776,'RVSN' from dual union all
select 18777,'DFVLR' from dual union all
select 18778,'SRC' from dual union all
select 18779,'AN' from dual union all
select 1878,'USAF' from dual union all
select 18780,'ARPA' from dual union all
select 18781,'SRC' from dual union all
select 18782,'AN' from dual union all
select 18783,'AN' from dual union all
select 18784,'USN' from dual union all
select 18785,'ARPA' from dual union all
select 18786,'AN' from dual union all
select 18787,'RVSN' from dual union all
select 18788,'AN' from dual union all
select 18789,'AN' from dual union all
select 1879,'USA' from dual union all
select 18790,'USAF' from dual union all
select 18791,'AN' from dual;
insert into LAUNCH_AGENCY
select 18792,'AFSC' from dual union all
select 18793,'AN' from dual union all
select 18794,'1STRAD' from dual union all
select 18795,'AN' from dual union all
select 18796,'1STRAD' from dual union all
select 18797,'AFSC' from dual union all
select 18798,'RVSN' from dual union all
select 18799,'AN' from dual union all
select 188,'WER836-2' from dual union all
select 1880,'USAF' from dual union all
select 18800,'AN' from dual union all
select 18801,'RN' from dual union all
select 18802,'USAF' from dual union all
select 18803,'AN' from dual union all
select 18804,'AN' from dual union all
select 18805,'USN' from dual union all
select 18806,'USN' from dual union all
select 18807,'USN' from dual union all
select 18808,'USN' from dual union all
select 18809,'SAND' from dual union all
select 1881,'USAF' from dual union all
select 18810,'NASA' from dual union all
select 18811,'NRL' from dual union all
select 18812,'NASA' from dual union all
select 18813,'RVSN' from dual union all
select 18814,'NASA' from dual union all
select 18815,'NRCC' from dual union all
select 18816,'NASA' from dual union all
select 18817,'NRCC' from dual union all
select 18818,'AN' from dual union all
select 18819,'SRC' from dual union all
select 1882,'USA' from dual union all
select 18820,'RVSN' from dual union all
select 18821,'USAF' from dual union all
select 18822,'FKGR21' from dual union all
select 18823,'FKGR22' from dual union all
select 18824,'SUPA' from dual union all
select 18825,'NRCC' from dual union all
select 18826,'BAC' from dual union all
select 18827,'AN' from dual union all
select 18828,'RVSN' from dual union all
select 18829,'RVSN' from dual union all
select 1883,'USA' from dual union all
select 18830,'AN' from dual union all
select 18831,'BAC' from dual union all
select 18832,'USN' from dual union all
select 18833,'USAF' from dual union all
select 18834,'USAF' from dual union all
select 18835,'NASA' from dual union all
select 18836,'USN' from dual union all
select 18837,'AN' from dual union all
select 18838,'NTNF' from dual union all
select 18839,'RVSN' from dual union all
select 1884,'USA' from dual union all
select 18840,'AFSC' from dual union all
select 18841,'CNES' from dual union all
select 18842,'NASA' from dual union all
select 18843,'CNES' from dual union all
select 18844,'NASA' from dual union all
select 18845,'RN' from dual union all
select 18846,'NASA' from dual union all
select 18847,'NASA' from dual union all
select 18848,'US7A' from dual union all
select 18849,'US7A' from dual union all
select 1885,'USA' from dual union all
select 18850,'RVSN' from dual union all
select 18851,'ESRO' from dual union all
select 18852,'1STRAD' from dual union all
select 18853,'IIAE' from dual union all
select 18854,'NASA' from dual union all
select 18855,'NASA' from dual union all
select 18856,'BAC' from dual union all
select 18857,'NASA' from dual union all
select 18858,'NASA' from dual union all
select 18859,'CNES' from dual union all
select 1886,'USA' from dual union all
select 18860,'AFSC' from dual union all
select 18860,'ALCS' from dual union all
select 18861,'RVSN' from dual union all
select 18862,'RVSN' from dual union all
select 18863,'AFCRL' from dual union all
select 18864,'AFCRL' from dual union all
select 18865,'AN' from dual union all
select 18866,'AN' from dual union all
select 18867,'AN' from dual union all
select 18868,'AN' from dual union all
select 18869,'AN' from dual union all
select 1887,'PARD' from dual union all
select 18870,'AN' from dual union all
select 18871,'AN' from dual union all
select 18872,'RVSN' from dual union all
select 18873,'AFSC' from dual union all
select 18874,'BAC' from dual union all
select 18875,'1STRAD' from dual union all
select 18876,'DMA' from dual union all
select 18877,'CNES' from dual union all
select 18878,'AN' from dual union all
select 18879,'AN' from dual union all
select 1888,'PARD' from dual union all
select 18880,'1STRAD' from dual;
insert into LAUNCH_AGENCY
select 18881,'RVSN' from dual union all
select 18882,'USA' from dual union all
select 18883,'RVSN' from dual union all
select 18884,'CNES' from dual union all
select 18885,'NASA' from dual union all
select 18886,'NASA' from dual union all
select 18887,'NASA' from dual union all
select 18888,'FKGR12' from dual union all
select 18889,'USAF' from dual union all
select 1889,'ORDRSA' from dual union all
select 18890,'USAF' from dual union all
select 18891,'USAF' from dual union all
select 18892,'USAF' from dual union all
select 18893,'RVSN' from dual union all
select 18894,'US7A' from dual union all
select 18895,'US7A' from dual union all
select 18896,'SAND' from dual union all
select 18897,'NRCC' from dual union all
select 18898,'NASA' from dual union all
select 18899,'AFSC' from dual union all
select 189,'WER836-1' from dual union all
select 1890,'ORDRSA' from dual union all
select 18900,'NRCC' from dual union all
select 18901,'NASA' from dual union all
select 18902,'NASA' from dual union all
select 18903,'NASA' from dual union all
select 18904,'RVSN' from dual union all
select 18905,'NASA' from dual union all
select 18906,'NASA' from dual union all
select 18907,'NASA' from dual union all
select 18908,'AFCRL' from dual union all
select 18909,'AFCRL' from dual union all
select 1891,'USA' from dual union all
select 18910,'BAC' from dual union all
select 18911,'AFCRL' from dual union all
select 18912,'AFCRL' from dual union all
select 18913,'AFCRL' from dual union all
select 18914,'NASA' from dual union all
select 18915,'RVSN' from dual union all
select 18916,'USAF' from dual union all
select 18917,'NASA' from dual union all
select 18918,'SAND' from dual union all
select 18919,'USA' from dual union all
select 1892,'ORDRSA' from dual union all
select 18920,'1STRAD' from dual union all
select 18921,'SAC' from dual union all
select 18922,'RVSN' from dual union all
select 18923,'AFCRL' from dual union all
select 18924,'AFCRL' from dual union all
select 18925,'AFCRL' from dual union all
select 18926,'RVSN' from dual union all
select 18927,'AFCRL' from dual union all
select 18928,'USN' from dual union all
select 18929,'SAND' from dual union all
select 1893,'USA' from dual union all
select 18930,'AFSC' from dual union all
select 18931,'CNES' from dual union all
select 18932,'AFCRL' from dual union all
select 18933,'CNIE' from dual union all
select 18934,'1STRAD' from dual union all
select 18935,'NASA' from dual union all
select 18936,'AFSC' from dual union all
select 18937,'RVSN' from dual union all
select 18938,'SAMSO' from dual union all
select 18939,'DMA' from dual union all
select 1894,'USA' from dual union all
select 18940,'USAF' from dual union all
select 18941,'CALT' from dual union all
select 18942,'SAMSO' from dual union all
select 18943,'USAF' from dual union all
select 18944,'ESRO' from dual union all
select 18945,'USAF' from dual union all
select 18946,'NASA' from dual union all
select 18947,'RVSN' from dual union all
select 18948,'1STRAD' from dual union all
select 18949,'AN' from dual union all
select 1895,'USA' from dual union all
select 18950,'KPNO' from dual union all
select 18951,'NASA' from dual union all
select 18952,'NASA' from dual union all
select 18953,'USN' from dual union all
select 18954,'USAF' from dual union all
select 18955,'1STRAD' from dual union all
select 18955,'ALCS' from dual union all
select 18956,'USAF' from dual union all
select 18957,'SRC' from dual union all
select 18958,'RVSN' from dual union all
select 18959,'SRC' from dual union all
select 1896,'USA' from dual union all
select 18960,'AFSC' from dual union all
select 18961,'USAF' from dual union all
select 18962,'AN' from dual union all
select 18963,'NASA' from dual union all
select 18964,'VMF' from dual union all
select 18965,'NASA' from dual union all
select 18966,'DMA' from dual union all
select 18967,'USAF' from dual union all
select 18968,'AN' from dual union all
select 18969,'RVSN' from dual union all
select 1897,'USA' from dual;
insert into LAUNCH_AGENCY
select 18970,'AFSC' from dual union all
select 18971,'USAF' from dual union all
select 18972,'NASA' from dual union all
select 18973,'RVSN' from dual union all
select 18974,'NASA' from dual union all
select 18975,'1STRAD' from dual union all
select 18976,'CALT' from dual union all
select 18977,'SAMSO' from dual union all
select 18978,'AN' from dual union all
select 18979,'AFCRL' from dual union all
select 1898,'USA' from dual union all
select 18980,'RVSN' from dual union all
select 18981,'1STRAD' from dual union all
select 18982,'AN' from dual union all
select 18983,'ESRO' from dual union all
select 18984,'ESRO' from dual union all
select 18985,'1STRAD' from dual union all
select 18986,'AN' from dual union all
select 18987,'USN' from dual union all
select 18988,'AN' from dual union all
select 18989,'SAMSO' from dual union all
select 1899,'USAF' from dual union all
select 18990,'SAMSO' from dual union all
select 18991,'RVSN' from dual union all
select 18992,'AN' from dual union all
select 18993,'CNES' from dual union all
select 18994,'CNES' from dual union all
select 18995,'RVSN' from dual union all
select 18996,'AN' from dual union all
select 18997,'ESRO' from dual union all
select 18998,'ESRO' from dual union all
select 18999,'BAC' from dual union all
select 19,'WEHR' from dual union all
select 190,'WER836-1' from dual union all
select 1900,'PARD' from dual union all
select 19000,'1STRAD' from dual union all
select 19001,'1STRAD' from dual union all
select 19002,'RVSN' from dual union all
select 19003,'USA' from dual union all
select 19004,'NASA' from dual union all
select 19005,'BAC' from dual union all
select 19006,'1STRAD' from dual union all
select 19006,'ALCS' from dual union all
select 19007,'RVSN' from dual union all
select 19008,'RN' from dual union all
select 19009,'AN' from dual union all
select 1901,'USAF' from dual union all
select 19010,'AFSC' from dual union all
select 19011,'DMA' from dual union all
select 19012,'AN' from dual union all
select 19013,'RVSN' from dual union all
select 19014,'AN' from dual union all
select 19015,'AN' from dual union all
select 19016,'BAC' from dual union all
select 19017,'AN' from dual union all
select 19018,'AN' from dual union all
select 19019,'AN' from dual union all
select 1902,'USA' from dual union all
select 19020,'AN' from dual union all
select 19021,'AN' from dual union all
select 19022,'AN' from dual union all
select 19023,'SRC' from dual union all
select 19024,'RVSN' from dual union all
select 19025,'BAC' from dual union all
select 19026,'SAMSO' from dual union all
select 19027,'SAMSO' from dual union all
select 19028,'AN' from dual union all
select 19029,'AFSC' from dual union all
select 1903,'PARD' from dual union all
select 19030,'SRC' from dual union all
select 19031,'AN' from dual union all
select 19032,'AN' from dual union all
select 19033,'AN' from dual union all
select 19034,'AN' from dual union all
select 19035,'RVSN' from dual union all
select 19036,'AN' from dual union all
select 19037,'RVSN' from dual union all
select 19038,'BRL' from dual union all
select 19039,'ISAS' from dual union all
select 1904,'USAF' from dual union all
select 19040,'AFCRL' from dual union all
select 19041,'AFCRL' from dual union all
select 19042,'AFCRL' from dual union all
select 19043,'ISAS' from dual union all
select 19044,'ISAS' from dual union all
select 19045,'ESRO' from dual union all
select 19046,'RVSN' from dual union all
select 19047,'RN' from dual union all
select 19048,'AFCRL' from dual union all
select 19049,'NASA' from dual union all
select 1905,'USA' from dual union all
select 19050,'ESRO' from dual union all
select 19051,'USAF' from dual union all
select 19052,'ISAS' from dual union all
select 19053,'USN' from dual union all
select 19054,'AFSC' from dual union all
select 19055,'1STRAD' from dual union all
select 19056,'BAC' from dual union all
select 19057,'RVSN' from dual union all
select 19058,'NASA' from dual;
insert into LAUNCH_AGENCY
select 19059,'NASA' from dual union all
select 1906,'USA' from dual union all
select 19060,'CNES' from dual union all
select 19061,'USN' from dual union all
select 19062,'NASA' from dual union all
select 19063,'ISAS' from dual union all
select 19064,'ISAS' from dual union all
select 19065,'1STRAD' from dual union all
select 19065,'ALCS' from dual union all
select 19066,'1STRAD' from dual union all
select 19067,'RVSN' from dual union all
select 19068,'CNIE' from dual union all
select 19069,'CNIE' from dual union all
select 1907,'ORDRSA' from dual union all
select 19070,'USAF' from dual union all
select 19071,'USAF' from dual union all
select 19072,'USAF' from dual union all
select 19073,'44FA2' from dual union all
select 19074,'44FA2' from dual union all
select 19075,'44FA2' from dual union all
select 19076,'44FA2' from dual union all
select 19077,'1STRAD' from dual union all
select 19078,'RVSN' from dual union all
select 19079,'ISAS' from dual union all
select 1908,'USA' from dual union all
select 19080,'ARPA' from dual union all
select 19081,'FKGR12' from dual union all
select 19082,'FKGR13' from dual union all
select 19083,'ARPA' from dual union all
select 19084,'ISAS' from dual union all
select 19085,'USAF' from dual union all
select 19086,'RVSN' from dual union all
select 19087,'1STRAD' from dual union all
select 19088,'USAF' from dual union all
select 19089,'RVSN' from dual union all
select 1909,'USA' from dual union all
select 19090,'NASA' from dual union all
select 19091,'NASDA' from dual union all
select 19092,'NASA' from dual union all
select 19093,'NASA' from dual union all
select 19094,'NRCC' from dual union all
select 19095,'SAND' from dual union all
select 19096,'NRCC' from dual union all
select 19097,'NASA' from dual union all
select 19098,'IIAE' from dual union all
select 19099,'AFSC' from dual union all
select 191,'WER836-1' from dual union all
select 1910,'ORDRSA' from dual union all
select 19100,'RVSN' from dual union all
select 19101,'ARPA' from dual union all
select 19102,'NASDA' from dual union all
select 19103,'SRC' from dual union all
select 19104,'AFSC' from dual union all
select 19105,'NASDA' from dual union all
select 19106,'USN' from dual union all
select 19107,'NASA' from dual union all
select 19108,'USN' from dual union all
select 19109,'NASA' from dual union all
select 1911,'USAF' from dual union all
select 19110,'1STRAD' from dual union all
select 19111,'RVSN' from dual union all
select 19112,'STA' from dual union all
select 19113,'ISRO' from dual union all
select 19114,'NRL' from dual union all
select 19115,'AFSC' from dual union all
select 19116,'NASA' from dual union all
select 19117,'AFSC' from dual union all
select 19118,'1STRAD' from dual union all
select 19119,'SAMSO' from dual union all
select 1912,'USA' from dual union all
select 19120,'NASA' from dual union all
select 19121,'RVSN' from dual union all
select 19122,'USAF' from dual union all
select 19123,'ISAS' from dual union all
select 19124,'USAF' from dual union all
select 19125,'AN' from dual union all
select 19126,'USN' from dual union all
select 19127,'SRC' from dual union all
select 19128,'AFSC' from dual union all
select 19129,'SRC' from dual union all
select 1913,'1PBS' from dual union all
select 19130,'SRC' from dual union all
select 19131,'USAF' from dual union all
select 19132,'1STRAD' from dual union all
select 19133,'NASA' from dual union all
select 19134,'NASA' from dual union all
select 19135,'ESRO' from dual union all
select 19136,'SRC' from dual union all
select 19137,'ESRO' from dual union all
select 19138,'AFCRL' from dual union all
select 19139,'AN' from dual union all
select 1914,'1PBS' from dual union all
select 19140,'SRC' from dual union all
select 19141,'USAF' from dual union all
select 19142,'1STRAD' from dual union all
select 19142,'ALCS' from dual union all
select 19143,'DMA' from dual union all
select 19144,'NRL' from dual union all
select 19145,'AN' from dual union all
select 19146,'SRC' from dual;
insert into LAUNCH_AGENCY
select 19147,'AFSC' from dual union all
select 19148,'USN' from dual union all
select 19149,'BAC' from dual union all
select 1915,'USA' from dual union all
select 19150,'USN' from dual union all
select 19151,'NASA' from dual union all
select 19152,'ISAS' from dual union all
select 19153,'NASA' from dual union all
select 19154,'SRC' from dual union all
select 19155,'BAC' from dual union all
select 19156,'AN' from dual union all
select 19157,'ESRO' from dual union all
select 19158,'AN' from dual union all
select 19159,'AN' from dual union all
select 1916,'1PBS' from dual union all
select 19160,'USN' from dual union all
select 19161,'USN' from dual union all
select 19162,'USN' from dual union all
select 19163,'ISAS' from dual union all
select 19164,'USN' from dual union all
select 19165,'USN' from dual union all
select 19166,'USN' from dual union all
select 19167,'USN' from dual union all
select 19168,'BAC' from dual union all
select 19169,'1STRAD' from dual union all
select 19169,'ALCS' from dual union all
select 1917,'1PBS' from dual union all
select 19170,'AN' from dual union all
select 19171,'SAMSO' from dual union all
select 19172,'BAC' from dual union all
select 19173,'AN' from dual union all
select 19174,'ISAS' from dual union all
select 19175,'RVSN' from dual union all
select 19176,'IIAE' from dual union all
select 19177,'AFCRL' from dual union all
select 19178,'RVSN' from dual union all
select 19179,'AN' from dual union all
select 1918,'USA' from dual union all
select 19180,'ESRO' from dual union all
select 19181,'RVSN' from dual union all
select 19182,'NRCC' from dual union all
select 19183,'NRCC' from dual union all
select 19184,'USN' from dual union all
select 19185,'NASA' from dual union all
select 19186,'USN' from dual union all
select 19187,'USN' from dual union all
select 19188,'USN' from dual union all
select 19189,'USN' from dual union all
select 1919,'PARD' from dual union all
select 19190,'USN' from dual union all
select 19191,'USN' from dual union all
select 19192,'USN' from dual union all
select 19193,'AFSC' from dual union all
select 19194,'RVSN' from dual union all
select 19195,'AFCRL' from dual union all
select 19196,'NASA' from dual union all
select 19197,'AFCRL' from dual union all
select 19198,'AFCRL' from dual union all
select 19199,'AFCRL' from dual union all
select 192,'WER836-1' from dual union all
select 1920,'1PBS' from dual union all
select 19200,'AFCRL' from dual union all
select 19201,'AFCRL' from dual union all
select 19202,'AFCRL' from dual union all
select 19203,'NRCC' from dual union all
select 19204,'AFCRL' from dual union all
select 19205,'BRL' from dual union all
select 19206,'BRL' from dual union all
select 19207,'NASA' from dual union all
select 19208,'AFCRL' from dual union all
select 19209,'USN' from dual union all
select 1921,'1PBS' from dual union all
select 19210,'AFCRL' from dual union all
select 19211,'AFCRL' from dual union all
select 19212,'AFCRL' from dual union all
select 19213,'BRL' from dual union all
select 19214,'AFCRL' from dual union all
select 19215,'AFCRL' from dual union all
select 19216,'RVSN' from dual union all
select 19217,'NASA' from dual union all
select 19218,'USAF' from dual union all
select 19219,'NASA' from dual union all
select 1922,'USA' from dual union all
select 19220,'BRL' from dual union all
select 19221,'AFCRL' from dual union all
select 19222,'BRL' from dual union all
select 19223,'AFCRL' from dual union all
select 19224,'AFCRL' from dual union all
select 19225,'AN' from dual union all
select 19226,'AN' from dual union all
select 19227,'NRCC' from dual union all
select 19228,'AFCRL' from dual union all
select 19229,'ISAS' from dual union all
select 1923,'USA' from dual union all
select 19230,'1STRAD' from dual union all
select 19231,'AFSC' from dual union all
select 19232,'AFCRL' from dual union all
select 19233,'CNES' from dual union all
select 19234,'NRL' from dual union all
select 19235,'AN' from dual;
insert into LAUNCH_AGENCY
select 19236,'ESRO' from dual union all
select 19237,'USN' from dual union all
select 19238,'BAC' from dual union all
select 19239,'CNES' from dual union all
select 1924,'PARD' from dual union all
select 19240,'USAF' from dual union all
select 19241,'SAND' from dual union all
select 19242,'AFSC' from dual union all
select 19243,'BAC' from dual union all
select 19244,'USN' from dual union all
select 19245,'1STRAD' from dual union all
select 19246,'NTNF' from dual union all
select 19247,'USN' from dual union all
select 19248,'BAC' from dual union all
select 19249,'ESRO' from dual union all
select 1925,'USAF' from dual union all
select 19250,'ESRO' from dual union all
select 19251,'SAMSO' from dual union all
select 19252,'NTNF' from dual union all
select 19253,'RVSN' from dual union all
select 19254,'USN' from dual union all
select 19255,'AFSC' from dual union all
select 19256,'NRCC' from dual union all
select 19257,'NRCC' from dual union all
select 19258,'NASA' from dual union all
select 19259,'1STRAD' from dual union all
select 19259,'ALCS' from dual union all
select 1926,'ORDRSA' from dual union all
select 19260,'NASA' from dual union all
select 19261,'AFCRL' from dual union all
select 19262,'NRCC' from dual union all
select 19263,'AFCRL' from dual union all
select 19264,'AFCRL' from dual union all
select 19265,'ISRO' from dual union all
select 19266,'INTA' from dual union all
select 19267,'INTA' from dual union all
select 19268,'DMA' from dual union all
select 19269,'AFCRL' from dual union all
select 1927,'USA' from dual union all
select 19270,'AN' from dual union all
select 19271,'INTA' from dual union all
select 19272,'INTA' from dual union all
select 19273,'ISAS' from dual union all
select 19274,'AN' from dual union all
select 19275,'AN' from dual union all
select 19276,'AFSC' from dual union all
select 19277,'AFCRL' from dual union all
select 19278,'RVSN' from dual union all
select 19279,'AFCRL' from dual union all
select 1928,'ORDRSA' from dual union all
select 19280,'INTA' from dual union all
select 19281,'INTA' from dual union all
select 19282,'NASA' from dual union all
select 19283,'AN' from dual union all
select 19284,'INTA' from dual union all
select 19285,'NASA' from dual union all
select 19286,'NASA' from dual union all
select 19287,'USA' from dual union all
select 19288,'NRCC' from dual union all
select 19289,'INTA' from dual union all
select 1929,'ORDRSA' from dual union all
select 19290,'AFSC' from dual union all
select 19291,'AN' from dual union all
select 19292,'INTA' from dual union all
select 19293,'AFSC' from dual union all
select 19294,'USN' from dual union all
select 19295,'INTA' from dual union all
select 19296,'NASA' from dual union all
select 19297,'AN' from dual union all
select 19298,'AN' from dual union all
select 19299,'INTA' from dual union all
select 193,'WER485-2' from dual union all
select 1930,'ORDRSA' from dual union all
select 19300,'INTA' from dual union all
select 19301,'RVSN' from dual union all
select 19302,'AN' from dual union all
select 19303,'AN' from dual union all
select 19304,'ISAS' from dual union all
select 19305,'IIAE' from dual union all
select 19306,'IIAE' from dual union all
select 19307,'IIAE' from dual union all
select 19308,'AN' from dual union all
select 19309,'RVSN' from dual union all
select 1931,'ORDRSA' from dual union all
select 19310,'BRL' from dual union all
select 19311,'BRL' from dual union all
select 19312,'BRL' from dual union all
select 19313,'BRL' from dual union all
select 19314,'BRL' from dual union all
select 19315,'USAF' from dual union all
select 19316,'BRL' from dual union all
select 19317,'NASA' from dual union all
select 19318,'USAF' from dual union all
select 19319,'USAF' from dual union all
select 1932,'ORDRSA' from dual union all
select 19320,'USAF' from dual union all
select 19321,'USAF' from dual union all
select 19322,'USAF' from dual union all
select 19323,'USAF' from dual union all
select 19324,'NRL' from dual;
insert into LAUNCH_AGENCY
select 19325,'NRL' from dual union all
select 19326,'NASA' from dual union all
select 19326,'RTG' from dual union all
select 19327,'BRL' from dual union all
select 19328,'BRL' from dual union all
select 19329,'BRL' from dual union all
select 1933,'ORDRSA' from dual union all
select 19330,'BRL' from dual union all
select 19331,'CALT' from dual union all
select 19332,'RVSN' from dual union all
select 19333,'RVSN' from dual union all
select 19334,'WRE' from dual union all
select 19335,'WRE' from dual union all
select 19336,'WRE' from dual union all
select 19337,'NASA' from dual union all
select 19338,'WRE' from dual union all
select 19339,'AN' from dual union all
select 1934,'ORDRSA' from dual union all
select 19340,'AN' from dual union all
select 19341,'AFCRL' from dual union all
select 19342,'NRL' from dual union all
select 19343,'SAND' from dual union all
select 19344,'RVSN' from dual union all
select 19345,'RVSN' from dual union all
select 19346,'RVSN' from dual union all
select 19347,'RVSN' from dual union all
select 19348,'NASA' from dual union all
select 19349,'RVSN' from dual union all
select 1935,'USA' from dual union all
select 19350,'RVSN' from dual union all
select 19351,'RVSN' from dual union all
select 19352,'RVSN' from dual union all
select 19353,'RVSN' from dual union all
select 19354,'RVSN' from dual union all
select 19355,'ISAS' from dual union all
select 19356,'RVSN' from dual union all
select 19357,'RVSN' from dual union all
select 19358,'RVSN' from dual union all
select 19359,'RVSN' from dual union all
select 1936,'USA' from dual union all
select 19360,'RVSN' from dual union all
select 19361,'RVSN' from dual union all
select 19362,'RVSN' from dual union all
select 19363,'RVSN' from dual union all
select 19364,'RVSN' from dual union all
select 19365,'RVSN' from dual union all
select 19366,'NASA' from dual union all
select 19367,'RVSN' from dual union all
select 19368,'RVSN' from dual union all
select 19369,'RVSN' from dual union all
select 1937,'ORDRSA' from dual union all
select 19370,'RVSN' from dual union all
select 19371,'USAF' from dual union all
select 19372,'USAF' from dual union all
select 19373,'RVSN' from dual union all
select 19374,'RVSN' from dual union all
select 19375,'RVSN' from dual union all
select 19376,'RVSN' from dual union all
select 19377,'1STRAD' from dual union all
select 19378,'MVS' from dual union all
select 19379,'MVS' from dual union all
select 1938,'USA' from dual union all
select 19380,'MVS' from dual union all
select 19381,'RVSN' from dual union all
select 19382,'RVSN' from dual union all
select 19383,'RVSN' from dual union all
select 19384,'RVSN' from dual union all
select 19385,'MVS' from dual union all
select 19386,'MVS' from dual union all
select 19387,'MVS' from dual union all
select 19388,'AN' from dual union all
select 19389,'MVS' from dual union all
select 1939,'WADD' from dual union all
select 19390,'MVS' from dual union all
select 19391,'MVS' from dual union all
select 19392,'MVS' from dual union all
select 19393,'MVS' from dual union all
select 19394,'MVS' from dual union all
select 19395,'MVS' from dual union all
select 19396,'MVS' from dual union all
select 19397,'RVSN' from dual union all
select 19398,'RVSN' from dual union all
select 19399,'USN' from dual union all
select 194,'WER836-1' from dual union all
select 1940,'WADD' from dual union all
select 19400,'RVSN' from dual union all
select 19401,'RVSN' from dual union all
select 19402,'RVSN' from dual union all
select 19403,'RVSN' from dual union all
select 19404,'RVSN' from dual union all
select 19405,'AFCRL' from dual union all
select 19406,'USAF' from dual union all
select 19407,'NASA' from dual union all
select 19408,'USAF' from dual union all
select 19409,'AN' from dual union all
select 1941,'WADD' from dual union all
select 19410,'RAE' from dual union all
select 19410,'WRE' from dual union all
select 19411,'NASA' from dual union all
select 19411,'RTG' from dual;
insert into LAUNCH_AGENCY
select 19412,'NASA' from dual union all
select 19413,'NASA' from dual union all
select 19413,'RTG' from dual union all
select 19414,'NASA' from dual union all
select 19415,'NASA' from dual union all
select 19416,'NASA' from dual union all
select 19417,'NRCC' from dual union all
select 19418,'AFSC' from dual union all
select 19419,'AFSC' from dual union all
select 1942,'WADD' from dual union all
select 19420,'1STRAD' from dual union all
select 19421,'NASA' from dual union all
select 19422,'NASA' from dual union all
select 19423,'NASA' from dual union all
select 19424,'NASA' from dual union all
select 19425,'NASA' from dual union all
select 19426,'USAF' from dual union all
select 19427,'NASA' from dual union all
select 19428,'NASA' from dual union all
select 19429,'NASA' from dual union all
select 1943,'ORDRSA' from dual union all
select 19430,'RVSN' from dual union all
select 19431,'USAF' from dual union all
select 19432,'USAF' from dual union all
select 19433,'USAF' from dual union all
select 19434,'USAF' from dual union all
select 19435,'PIHM' from dual union all
select 19436,'PIHM' from dual union all
select 19437,'PIHM' from dual union all
select 19438,'MRN' from dual union all
select 19439,'MRN' from dual union all
select 1944,'ORDRSA' from dual union all
select 19440,'MRN' from dual union all
select 19441,'UKMO' from dual union all
select 19442,'MRN' from dual union all
select 19443,'MRN' from dual union all
select 19444,'MRN' from dual union all
select 19445,'MRN' from dual union all
select 19446,'MRN' from dual union all
select 19447,'MRN' from dual union all
select 19448,'MRN' from dual union all
select 19449,'MRN' from dual union all
select 1945,'ORDRSA' from dual union all
select 19450,'MRN' from dual union all
select 19451,'MRN' from dual union all
select 19452,'INTA' from dual union all
select 19453,'MRN' from dual union all
select 19454,'MRN' from dual union all
select 19455,'MRN' from dual union all
select 19456,'MRN' from dual union all
select 19457,'MRN' from dual union all
select 19458,'MRN' from dual union all
select 19459,'MRN' from dual union all
select 1946,'ORDRSA' from dual union all
select 19460,'MRN' from dual union all
select 19461,'MRN' from dual union all
select 19462,'MRN' from dual union all
select 19463,'MRN' from dual union all
select 19464,'MRN' from dual union all
select 19465,'MRN' from dual union all
select 19466,'SUPA' from dual union all
select 19467,'MRN' from dual union all
select 19468,'MRN' from dual union all
select 19469,'MRN' from dual union all
select 1947,'USAF' from dual union all
select 19470,'MRN' from dual union all
select 19471,'MRN' from dual union all
select 19472,'MRN' from dual union all
select 19473,'MRN' from dual union all
select 19474,'MRN' from dual union all
select 19475,'MRN' from dual union all
select 19476,'MRN' from dual union all
select 19477,'MRN' from dual union all
select 19478,'MRN' from dual union all
select 19479,'MRN' from dual union all
select 1948,'USA' from dual union all
select 19480,'MRN' from dual union all
select 19481,'MRN' from dual union all
select 19482,'MRN' from dual union all
select 19483,'MRN' from dual union all
select 19484,'MRN' from dual union all
select 19485,'MRN' from dual union all
select 19486,'MRN' from dual union all
select 19487,'MRN' from dual union all
select 19488,'MRN' from dual union all
select 19489,'MRN' from dual union all
select 1949,'USA' from dual union all
select 19490,'MRN' from dual union all
select 19491,'MRN' from dual union all
select 19492,'AFCRL' from dual union all
select 19493,'ISRO' from dual union all
select 19494,'MRN' from dual union all
select 19495,'MRN' from dual union all
select 19496,'MRN' from dual union all
select 19497,'MRN' from dual union all
select 19498,'MRN' from dual union all
select 19499,'MRN' from dual union all
select 195,'WER836-1' from dual union all
select 1950,'ORDRSA' from dual union all
select 19500,'MRN' from dual;
insert into LAUNCH_AGENCY
select 19501,'MRN' from dual union all
select 19502,'MRN' from dual union all
select 19503,'MRN' from dual union all
select 19504,'MRN' from dual union all
select 19505,'MRN' from dual union all
select 19506,'MRN' from dual union all
select 19507,'MRN' from dual union all
select 19508,'MRN' from dual union all
select 19509,'MRN' from dual union all
select 1951,'ORDRSA' from dual union all
select 19510,'MRN' from dual union all
select 19511,'MRN' from dual union all
select 19512,'MRN' from dual union all
select 19513,'MRN' from dual union all
select 19514,'MRN' from dual union all
select 19515,'MRN' from dual union all
select 19516,'MRN' from dual union all
select 19517,'MRN' from dual union all
select 19518,'MRN' from dual union all
select 19519,'MRN' from dual union all
select 1952,'ORDRSA' from dual union all
select 19520,'MRN' from dual union all
select 19521,'MRN' from dual union all
select 19522,'MRN' from dual union all
select 19523,'MRN' from dual union all
select 19524,'MRN' from dual union all
select 19525,'MRN' from dual union all
select 19526,'MRN' from dual union all
select 19527,'MRN' from dual union all
select 19528,'MRN' from dual union all
select 19529,'MRN' from dual union all
select 1953,'USA' from dual union all
select 19530,'MRN' from dual union all
select 19531,'MRN' from dual union all
select 19532,'MRN' from dual union all
select 19533,'SUPA' from dual union all
select 19534,'MRN' from dual union all
select 19535,'MRN' from dual union all
select 19536,'MRN' from dual union all
select 19537,'MRN' from dual union all
select 19538,'MRN' from dual union all
select 19539,'MRN' from dual union all
select 1954,'USA' from dual union all
select 19540,'MRN' from dual union all
select 19541,'MRN' from dual union all
select 19542,'MRN' from dual union all
select 19543,'MRN' from dual union all
select 19544,'MRN' from dual union all
select 19545,'MRN' from dual union all
select 19546,'MRN' from dual union all
select 19547,'MRN' from dual union all
select 19548,'MRN' from dual union all
select 19549,'MRN' from dual union all
select 1955,'USA' from dual union all
select 19550,'MRN' from dual union all
select 19551,'MRN' from dual union all
select 19552,'MRN' from dual union all
select 19553,'MRN' from dual union all
select 19554,'MRN' from dual union all
select 19555,'MRN' from dual union all
select 19556,'MRN' from dual union all
select 19557,'MRN' from dual union all
select 19558,'ISRO' from dual union all
select 19559,'MRN' from dual union all
select 1956,'USAF' from dual union all
select 19560,'MRN' from dual union all
select 19561,'MRN' from dual union all
select 19562,'MRN' from dual union all
select 19563,'MRN' from dual union all
select 19564,'MRN' from dual union all
select 19565,'MRN' from dual union all
select 19566,'MRN' from dual union all
select 19567,'MRN' from dual union all
select 19568,'MRN' from dual union all
select 19569,'MRN' from dual union all
select 1957,'USA' from dual union all
select 19570,'MRN' from dual union all
select 19571,'MRN' from dual union all
select 19572,'MRN' from dual union all
select 19573,'MRN' from dual union all
select 19574,'USAF' from dual union all
select 19575,'MRN' from dual union all
select 19576,'MRN' from dual union all
select 19577,'MRN' from dual union all
select 19578,'MRN' from dual union all
select 19579,'MRN' from dual union all
select 1958,'USA' from dual union all
select 19580,'MRN' from dual union all
select 19581,'MRN' from dual union all
select 19582,'MRN' from dual union all
select 19583,'MRN' from dual union all
select 19584,'MRN' from dual union all
select 19585,'MRN' from dual union all
select 19586,'MRN' from dual union all
select 19587,'MRN' from dual union all
select 19588,'MRN' from dual union all
select 19589,'MRN' from dual union all
select 1959,'USA' from dual union all
select 19590,'MRN' from dual union all
select 19591,'MRN' from dual;
insert into LAUNCH_AGENCY
select 19592,'MRN' from dual union all
select 19593,'MRN' from dual union all
select 19594,'MRN' from dual union all
select 19595,'MRN' from dual union all
select 19596,'MRN' from dual union all
select 19597,'MRN' from dual union all
select 19598,'MRN' from dual union all
select 19599,'MRN' from dual union all
select 196,'WER836-1' from dual union all
select 1960,'USA' from dual union all
select 19600,'MRN' from dual union all
select 19601,'MRN' from dual union all
select 19602,'MRN' from dual union all
select 19603,'MRN' from dual union all
select 19604,'MRN' from dual union all
select 19605,'MRN' from dual union all
select 19606,'MRN' from dual union all
select 19607,'MRN' from dual union all
select 19608,'MRN' from dual union all
select 19609,'MRN' from dual union all
select 1961,'USA' from dual union all
select 19610,'MRN' from dual union all
select 19611,'MRN' from dual union all
select 19612,'MRN' from dual union all
select 19613,'MRN' from dual union all
select 19614,'MRN' from dual union all
select 19615,'MRN' from dual union all
select 19616,'MRN' from dual union all
select 19617,'MRN' from dual union all
select 19618,'MRN' from dual union all
select 19619,'MRN' from dual union all
select 1962,'USA' from dual union all
select 19620,'MRN' from dual union all
select 19621,'MRN' from dual union all
select 19622,'MRN' from dual union all
select 19623,'MRN' from dual union all
select 19624,'MRN' from dual union all
select 19625,'MRN' from dual union all
select 19626,'MRN' from dual union all
select 19627,'MRN' from dual union all
select 19628,'MRN' from dual union all
select 19629,'INTA' from dual union all
select 1963,'ORDRSA' from dual union all
select 19630,'MRN' from dual union all
select 19631,'MRN' from dual union all
select 19632,'MRN' from dual union all
select 19633,'MRN' from dual union all
select 19634,'MRN' from dual union all
select 19635,'MRN' from dual union all
select 19636,'MRN' from dual union all
select 19637,'MRN' from dual union all
select 19638,'MRN' from dual union all
select 19639,'MRN' from dual union all
select 1964,'USA' from dual union all
select 19640,'USAF' from dual union all
select 19641,'MRN' from dual union all
select 19642,'MRN' from dual union all
select 19643,'MRN' from dual union all
select 19644,'MRN' from dual union all
select 19645,'MRN' from dual union all
select 19646,'MRN' from dual union all
select 19647,'MRN' from dual union all
select 19648,'MRN' from dual union all
select 19649,'MRN' from dual union all
select 1965,'USA' from dual union all
select 19650,'MRN' from dual union all
select 19651,'USAF' from dual union all
select 19652,'MRN' from dual union all
select 19653,'MRN' from dual union all
select 19654,'ISAS' from dual union all
select 19655,'MRN' from dual union all
select 19656,'ISAS' from dual union all
select 19657,'ISAS' from dual union all
select 19658,'MRN' from dual union all
select 19659,'MRN' from dual union all
select 1966,'USAF' from dual union all
select 19660,'MRN' from dual union all
select 19661,'MRN' from dual union all
select 19662,'MRN' from dual union all
select 19663,'MRN' from dual union all
select 19664,'MRN' from dual union all
select 19665,'MRN' from dual union all
select 19666,'MRN' from dual union all
select 19667,'MRN' from dual union all
select 19668,'MRN' from dual union all
select 19669,'MRN' from dual union all
select 1967,'ORDRSA' from dual union all
select 19670,'MRN' from dual union all
select 19671,'MRN' from dual union all
select 19672,'MRN' from dual union all
select 19673,'MRN' from dual union all
select 19674,'MRN' from dual union all
select 19675,'SUPA' from dual union all
select 19676,'MRN' from dual union all
select 19677,'MRN' from dual union all
select 19678,'MRN' from dual union all
select 19679,'MRN' from dual union all
select 1968,'ORDRSA' from dual union all
select 19680,'MRN' from dual union all
select 19681,'MRN' from dual;
insert into LAUNCH_AGENCY
select 19682,'MRN' from dual union all
select 19683,'MRN' from dual union all
select 19684,'MRN' from dual union all
select 19685,'MRN' from dual union all
select 19686,'MRN' from dual union all
select 19687,'MRN' from dual union all
select 19688,'MRN' from dual union all
select 19689,'MRN' from dual union all
select 1969,'USA' from dual union all
select 19690,'MRN' from dual union all
select 19691,'MRN' from dual union all
select 19692,'MRN' from dual union all
select 19693,'MRN' from dual union all
select 19694,'MRN' from dual union all
select 19695,'MRN' from dual union all
select 19696,'MRN' from dual union all
select 19697,'MRN' from dual union all
select 19698,'MRN' from dual union all
select 19699,'MRN' from dual union all
select 197,'WER836-1' from dual union all
select 1970,'USA' from dual union all
select 19700,'MRN' from dual union all
select 19701,'MRN' from dual union all
select 19702,'MRN' from dual union all
select 19703,'MRN' from dual union all
select 19704,'MRN' from dual union all
select 19705,'MRN' from dual union all
select 19706,'MRN' from dual union all
select 19707,'MRN' from dual union all
select 19708,'MRN' from dual union all
select 19709,'MRN' from dual union all
select 1971,'USA' from dual union all
select 19710,'MRN' from dual union all
select 19711,'MRN' from dual union all
select 19712,'MRN' from dual union all
select 19713,'MRN' from dual union all
select 19714,'MRN' from dual union all
select 19715,'MRN' from dual union all
select 19716,'MRN' from dual union all
select 19717,'AFCRL' from dual union all
select 19718,'MRN' from dual union all
select 19719,'MRN' from dual union all
select 1972,'USA' from dual union all
select 19720,'MRN' from dual union all
select 19721,'MRN' from dual union all
select 19722,'MRN' from dual union all
select 19723,'MRN' from dual union all
select 19724,'MRN' from dual union all
select 19725,'MRN' from dual union all
select 19726,'MRN' from dual union all
select 19727,'MRN' from dual union all
select 19728,'MRN' from dual union all
select 19729,'MRN' from dual union all
select 1973,'USA' from dual union all
select 19730,'MRN' from dual union all
select 19731,'MRN' from dual union all
select 19732,'MRN' from dual union all
select 19733,'MRN' from dual union all
select 19734,'MRN' from dual union all
select 19735,'MRN' from dual union all
select 19736,'MRN' from dual union all
select 19737,'MRN' from dual union all
select 19738,'MRN' from dual union all
select 19739,'MRN' from dual union all
select 1974,'USA' from dual union all
select 19740,'MRN' from dual union all
select 19741,'MRN' from dual union all
select 19742,'MRN' from dual union all
select 19743,'MRN' from dual union all
select 19744,'MRN' from dual union all
select 19745,'MRN' from dual union all
select 19746,'MRN' from dual union all
select 19747,'GMS' from dual union all
select 19748,'MRN' from dual union all
select 19749,'AFCRL' from dual union all
select 1975,'USA' from dual union all
select 19750,'MRN' from dual union all
select 19751,'MRN' from dual union all
select 19752,'MRN' from dual union all
select 19753,'MRN' from dual union all
select 19754,'MRN' from dual union all
select 19755,'MRN' from dual union all
select 19756,'MRN' from dual union all
select 19757,'MRN' from dual union all
select 19758,'MRN' from dual union all
select 19759,'MRN' from dual union all
select 1976,'ORDRSA' from dual union all
select 19760,'MRN' from dual union all
select 19761,'MRN' from dual union all
select 19762,'USAF' from dual union all
select 19763,'MRN' from dual union all
select 19764,'MRN' from dual union all
select 19765,'MRN' from dual union all
select 19766,'MRN' from dual union all
select 19767,'MRN' from dual union all
select 19768,'MRN' from dual union all
select 19769,'MRN' from dual union all
select 1977,'USA' from dual union all
select 19770,'MRN' from dual union all
select 19771,'MRN' from dual;
insert into LAUNCH_AGENCY
select 19772,'MRN' from dual union all
select 19773,'MRN' from dual union all
select 19774,'MRN' from dual union all
select 19775,'MRN' from dual union all
select 19776,'MRN' from dual union all
select 19777,'MRN' from dual union all
select 19778,'MRN' from dual union all
select 19779,'MRN' from dual union all
select 1978,'USA' from dual union all
select 19780,'MRN' from dual union all
select 19781,'MRN' from dual union all
select 19782,'MRN' from dual union all
select 19783,'MRN' from dual union all
select 19784,'MRN' from dual union all
select 19785,'MRN' from dual union all
select 19786,'MRN' from dual union all
select 19787,'MRN' from dual union all
select 19788,'MRN' from dual union all
select 19789,'MRN' from dual union all
select 1979,'USA' from dual union all
select 19790,'MRN' from dual union all
select 19791,'ISAS' from dual union all
select 19792,'MRN' from dual union all
select 19793,'MRN' from dual union all
select 19794,'MRN' from dual union all
select 19795,'MRN' from dual union all
select 19796,'MRN' from dual union all
select 19797,'MRN' from dual union all
select 19798,'MRN' from dual union all
select 19799,'MRN' from dual union all
select 198,'WEHR' from dual union all
select 1980,'ORDRSA' from dual union all
select 19800,'MRN' from dual union all
select 19801,'MRN' from dual union all
select 19802,'MRN' from dual union all
select 19803,'MRN' from dual union all
select 19804,'MRN' from dual union all
select 19805,'MRN' from dual union all
select 19806,'MRN' from dual union all
select 19807,'MRN' from dual union all
select 19808,'MRN' from dual union all
select 19809,'MRN' from dual union all
select 1981,'USAF' from dual union all
select 19810,'MRN' from dual union all
select 19811,'MRN' from dual union all
select 19812,'MRN' from dual union all
select 19813,'MRN' from dual union all
select 19814,'MRN' from dual union all
select 19815,'MRN' from dual union all
select 19816,'MRN' from dual union all
select 19817,'MRN' from dual union all
select 19818,'MRN' from dual union all
select 19819,'MRN' from dual union all
select 1982,'USA' from dual union all
select 19820,'MRN' from dual union all
select 19821,'MRN' from dual union all
select 19822,'MRN' from dual union all
select 19823,'MRN' from dual union all
select 19824,'MRN' from dual union all
select 19825,'MRN' from dual union all
select 19826,'MRN' from dual union all
select 19827,'MRN' from dual union all
select 19828,'MRN' from dual union all
select 19829,'INTA' from dual union all
select 1983,'USA' from dual union all
select 19830,'MRN' from dual union all
select 19831,'MRN' from dual union all
select 19832,'MRN' from dual union all
select 19833,'MRN' from dual union all
select 19834,'MRN' from dual union all
select 19835,'MRN' from dual union all
select 19836,'MRN' from dual union all
select 19837,'MRN' from dual union all
select 19838,'MRN' from dual union all
select 19839,'MRN' from dual union all
select 1984,'USA' from dual union all
select 19840,'MRN' from dual union all
select 19841,'MRN' from dual union all
select 19842,'MRN' from dual union all
select 19843,'MRN' from dual union all
select 19844,'MRN' from dual union all
select 19845,'MRN' from dual union all
select 19846,'MRN' from dual union all
select 19847,'MRN' from dual union all
select 19848,'MRN' from dual union all
select 19849,'MRN' from dual union all
select 1985,'USAF' from dual union all
select 19850,'MRN' from dual union all
select 19851,'MRN' from dual union all
select 19852,'MRN' from dual union all
select 19853,'MRN' from dual union all
select 19854,'MRN' from dual union all
select 19855,'MRN' from dual union all
select 19856,'MRN' from dual union all
select 19857,'MRN' from dual union all
select 19858,'MRN' from dual union all
select 19859,'MRN' from dual union all
select 1986,'PARD' from dual union all
select 19860,'MRN' from dual union all
select 19861,'MRN' from dual;
insert into LAUNCH_AGENCY
select 19862,'MRN' from dual union all
select 19863,'MRN' from dual union all
select 19864,'MRN' from dual union all
select 19865,'MRN' from dual union all
select 19866,'MRN' from dual union all
select 19867,'MRN' from dual union all
select 19868,'MRN' from dual union all
select 19869,'MRN' from dual union all
select 1987,'ORDRSA' from dual union all
select 19870,'MRN' from dual union all
select 19871,'MRN' from dual union all
select 19872,'MRN' from dual union all
select 19873,'MRN' from dual union all
select 19874,'INTA' from dual union all
select 19875,'MRN' from dual union all
select 19876,'MRN' from dual union all
select 19877,'MRN' from dual union all
select 19878,'MRN' from dual union all
select 19879,'MRN' from dual union all
select 1988,'USA' from dual union all
select 19880,'MRN' from dual union all
select 19881,'MRN' from dual union all
select 19882,'MRN' from dual union all
select 19883,'MRN' from dual union all
select 19884,'MRN' from dual union all
select 19885,'MRN' from dual union all
select 19886,'MRN' from dual union all
select 19887,'MRN' from dual union all
select 19888,'MRN' from dual union all
select 19889,'MRN' from dual union all
select 1989,'USA' from dual union all
select 19890,'MRN' from dual union all
select 19891,'MRN' from dual union all
select 19892,'MRN' from dual union all
select 19893,'MRN' from dual union all
select 19894,'MRN' from dual union all
select 19895,'MRN' from dual union all
select 19896,'MRN' from dual union all
select 19897,'MRN' from dual union all
select 19898,'MRN' from dual union all
select 19899,'MRN' from dual union all
select 199,'WER836-1' from dual union all
select 1990,'USA' from dual union all
select 19900,'MRN' from dual union all
select 19901,'MRN' from dual union all
select 19902,'MRN' from dual union all
select 19903,'MRN' from dual union all
select 19904,'MRN' from dual union all
select 19905,'MRN' from dual union all
select 19906,'MRN' from dual union all
select 19907,'MRN' from dual union all
select 19908,'MRN' from dual union all
select 19909,'MRN' from dual union all
select 1991,'USA' from dual union all
select 19910,'MRN' from dual union all
select 19911,'MRN' from dual union all
select 19912,'MRN' from dual union all
select 19913,'MRN' from dual union all
select 19914,'MRN' from dual union all
select 19915,'MRN' from dual union all
select 19916,'MRN' from dual union all
select 19917,'SUPA' from dual union all
select 19918,'MRN' from dual union all
select 19919,'MRN' from dual union all
select 1992,'USA' from dual union all
select 19920,'MRN' from dual union all
select 19921,'MRN' from dual union all
select 19922,'MRN' from dual union all
select 19923,'MRN' from dual union all
select 19924,'MRN' from dual union all
select 19925,'MRN' from dual union all
select 19926,'MRN' from dual union all
select 19927,'MRN' from dual union all
select 19928,'MRN' from dual union all
select 19929,'MRN' from dual union all
select 1993,'USA' from dual union all
select 19930,'MRN' from dual union all
select 19931,'MRN' from dual union all
select 19932,'MRN' from dual union all
select 19933,'MRN' from dual union all
select 19934,'MRN' from dual union all
select 19935,'MRN' from dual union all
select 19936,'MRN' from dual union all
select 19937,'MRN' from dual union all
select 19938,'MRN' from dual union all
select 19939,'MRN' from dual union all
select 1994,'USA' from dual union all
select 19940,'MRN' from dual union all
select 19941,'MRN' from dual union all
select 19942,'MRN' from dual union all
select 19943,'MRN' from dual union all
select 19944,'MRN' from dual union all
select 19945,'MRN' from dual union all
select 19946,'MRN' from dual union all
select 19947,'MRN' from dual union all
select 19948,'MRN' from dual union all
select 19949,'MRN' from dual union all
select 1995,'USA' from dual union all
select 19950,'MRN' from dual union all
select 19951,'MRN' from dual;
insert into LAUNCH_AGENCY
select 19952,'MRN' from dual union all
select 19953,'MRN' from dual union all
select 19954,'MRN' from dual union all
select 19955,'MRN' from dual union all
select 19956,'MRN' from dual union all
select 19957,'MRN' from dual union all
select 19958,'MRN' from dual union all
select 19959,'MRN' from dual union all
select 1996,'NRL' from dual union all
select 19960,'MRN' from dual union all
select 19961,'MRN' from dual union all
select 19962,'MRN' from dual union all
select 19963,'MRN' from dual union all
select 19964,'MRN' from dual union all
select 19965,'MRN' from dual union all
select 19966,'MRN' from dual union all
select 19967,'MRN' from dual union all
select 19968,'MRN' from dual union all
select 19969,'MRN' from dual union all
select 1997,'NRL' from dual union all
select 19970,'USAF' from dual union all
select 19971,'MRN' from dual union all
select 19972,'AFCRL' from dual union all
select 19973,'MRN' from dual union all
select 19974,'AFCRL' from dual union all
select 19975,'AFCRL' from dual union all
select 19976,'MRN' from dual union all
select 19977,'MRN' from dual union all
select 19978,'AFCRL' from dual union all
select 19979,'AFCRL' from dual union all
select 1998,'NRL' from dual union all
select 19980,'MRN' from dual union all
select 19981,'MRN' from dual union all
select 19982,'MRN' from dual union all
select 19983,'MRN' from dual union all
select 19984,'MRN' from dual union all
select 19985,'MRN' from dual union all
select 19986,'MRN' from dual union all
select 19987,'MRN' from dual union all
select 19988,'MRN' from dual union all
select 19989,'MRN' from dual union all
select 1999,'NRL' from dual union all
select 19990,'MRN' from dual union all
select 19991,'AFCRL' from dual union all
select 19992,'AFCRL' from dual union all
select 19993,'AFCRL' from dual union all
select 19994,'MRN' from dual union all
select 19995,'MRN' from dual union all
select 19996,'MRN' from dual union all
select 19997,'MRN' from dual union all
select 19998,'MRN' from dual union all
select 19999,'MRN' from dual union all
select 2,'WEHR' from dual union all
select 20,'WEHR' from dual union all
select 200,'WER836-1' from dual union all
select 2000,'NRL' from dual union all
select 20000,'MRN' from dual union all
select 20001,'MRN' from dual union all
select 20002,'MRN' from dual union all
select 20003,'MRN' from dual union all
select 20004,'MRN' from dual union all
select 20005,'MRN' from dual union all
select 20006,'MRN' from dual union all
select 20007,'MRN' from dual union all
select 20008,'MRN' from dual union all
select 20009,'MRN' from dual union all
select 2001,'USA' from dual union all
select 20010,'MRN' from dual union all
select 20011,'MRN' from dual union all
select 20012,'MRN' from dual union all
select 20013,'MRN' from dual union all
select 20014,'MRN' from dual union all
select 20015,'MRN' from dual union all
select 20016,'MRN' from dual union all
select 20017,'MRN' from dual union all
select 20018,'MRN' from dual union all
select 20019,'MRN' from dual union all
select 2002,'NRL' from dual union all
select 20020,'INTA' from dual union all
select 20021,'MRN' from dual union all
select 20022,'MRN' from dual union all
select 20023,'MRN' from dual union all
select 20024,'MRN' from dual union all
select 20025,'MRN' from dual union all
select 20026,'MRN' from dual union all
select 20027,'MRN' from dual union all
select 20028,'MRN' from dual union all
select 20029,'MRN' from dual union all
select 2003,'NRL' from dual union all
select 20030,'MRN' from dual union all
select 20031,'MRN' from dual union all
select 20032,'MRN' from dual union all
select 20033,'MRN' from dual union all
select 20034,'MRN' from dual union all
select 20035,'MRN' from dual union all
select 20036,'MRN' from dual union all
select 20037,'MRN' from dual union all
select 20038,'MRN' from dual union all
select 20039,'MRN' from dual union all
select 2004,'USA' from dual;
insert into LAUNCH_AGENCY
select 20040,'MRN' from dual union all
select 20041,'MRN' from dual union all
select 20042,'MRN' from dual union all
select 20043,'MRN' from dual union all
select 20044,'MRN' from dual union all
select 20045,'MRN' from dual union all
select 20046,'MRN' from dual union all
select 20047,'MRN' from dual union all
select 20048,'MRN' from dual union all
select 20049,'MRN' from dual union all
select 2005,'ARDC' from dual union all
select 20050,'MRN' from dual union all
select 20051,'INTA' from dual union all
select 20052,'MRN' from dual union all
select 20053,'MRN' from dual union all
select 20054,'MRN' from dual union all
select 20055,'SUPA' from dual union all
select 20056,'MRN' from dual union all
select 20057,'MRN' from dual union all
select 20058,'MRN' from dual union all
select 20059,'MRN' from dual union all
select 2006,'GAU4' from dual union all
select 20060,'MRN' from dual union all
select 20061,'MRN' from dual union all
select 20062,'MRN' from dual union all
select 20063,'MRN' from dual union all
select 20064,'MRN' from dual union all
select 20065,'MRN' from dual union all
select 20066,'MRN' from dual union all
select 20067,'SUPA' from dual union all
select 20068,'MRN' from dual union all
select 20069,'MRN' from dual union all
select 2007,'GAU4' from dual union all
select 20070,'MRN' from dual union all
select 20071,'MRN' from dual union all
select 20072,'MRN' from dual union all
select 20073,'MRN' from dual union all
select 20074,'MRN' from dual union all
select 20075,'MRN' from dual union all
select 20076,'MRN' from dual union all
select 20077,'MRN' from dual union all
select 20078,'MRN' from dual union all
select 20079,'MRN' from dual union all
select 2008,'GAU4' from dual union all
select 20080,'MRN' from dual union all
select 20081,'MRN' from dual union all
select 20082,'MRN' from dual union all
select 20083,'MRN' from dual union all
select 20084,'MRN' from dual union all
select 20085,'MRN' from dual union all
select 20086,'MRN' from dual union all
select 20087,'MRN' from dual union all
select 20088,'MRN' from dual union all
select 20089,'MRN' from dual union all
select 2009,'UZKA' from dual union all
select 20090,'MRN' from dual union all
select 20091,'INTA' from dual union all
select 20092,'MRN' from dual union all
select 20093,'MRN' from dual union all
select 20094,'MRN' from dual union all
select 20095,'MRN' from dual union all
select 20096,'MRN' from dual union all
select 20097,'UKMO' from dual union all
select 20098,'MRN' from dual union all
select 20099,'MRN' from dual union all
select 201,'WER836-1' from dual union all
select 2010,'UZKA' from dual union all
select 20100,'MRN' from dual union all
select 20101,'MRN' from dual union all
select 20102,'MRN' from dual union all
select 20103,'MRN' from dual union all
select 20104,'MRN' from dual union all
select 20105,'UKMO' from dual union all
select 20106,'MRN' from dual union all
select 20107,'MRN' from dual union all
select 20108,'USAF' from dual union all
select 20109,'MRN' from dual union all
select 2011,'ARDC' from dual union all
select 20110,'MRN' from dual union all
select 20111,'MRN' from dual union all
select 20112,'MRN' from dual union all
select 20113,'MRN' from dual union all
select 20114,'MRN' from dual union all
select 20115,'MRN' from dual union all
select 20116,'MRN' from dual union all
select 20117,'UKMO' from dual union all
select 20118,'MRN' from dual union all
select 20119,'MRN' from dual union all
select 2012,'UZKA' from dual union all
select 20120,'MRN' from dual union all
select 20121,'MRN' from dual union all
select 20122,'MRN' from dual union all
select 20123,'MRN' from dual union all
select 20124,'MRN' from dual union all
select 20125,'MRN' from dual union all
select 20126,'MRN' from dual union all
select 20127,'MRN' from dual union all
select 20128,'MRN' from dual union all
select 20129,'MRN' from dual union all
select 2013,'USA' from dual;
insert into LAUNCH_AGENCY
select 20130,'MRN' from dual union all
select 20131,'MRN' from dual union all
select 20132,'UKMO' from dual union all
select 20133,'MRN' from dual union all
select 20134,'MRN' from dual union all
select 20135,'MRN' from dual union all
select 20136,'MRN' from dual union all
select 20137,'MRN' from dual union all
select 20138,'MRN' from dual union all
select 20139,'MRN' from dual union all
select 2014,'USA' from dual union all
select 20140,'MRN' from dual union all
select 20141,'MRN' from dual union all
select 20142,'USA' from dual union all
select 20143,'MRN' from dual union all
select 20144,'MRN' from dual union all
select 20145,'MRN' from dual union all
select 20146,'MRN' from dual union all
select 20147,'MRN' from dual union all
select 20148,'UKMO' from dual union all
select 20149,'MRN' from dual union all
select 2015,'UZKA' from dual union all
select 20150,'MRN' from dual union all
select 20151,'MRN' from dual union all
select 20152,'MRN' from dual union all
select 20153,'MRN' from dual union all
select 20154,'MRN' from dual union all
select 20155,'MRN' from dual union all
select 20156,'MRN' from dual union all
select 20157,'MRN' from dual union all
select 20158,'MRN' from dual union all
select 20159,'MRN' from dual union all
select 2016,'UZKA' from dual union all
select 20160,'MRN' from dual union all
select 20161,'MRN' from dual union all
select 20162,'MRN' from dual union all
select 20163,'MRN' from dual union all
select 20164,'MRN' from dual union all
select 20165,'MRN' from dual union all
select 20166,'MRN' from dual union all
select 20167,'UKMO' from dual union all
select 20168,'MRN' from dual union all
select 20169,'USA' from dual union all
select 2017,'UZKA' from dual union all
select 20170,'MRN' from dual union all
select 20171,'SRC' from dual union all
select 20172,'MRN' from dual union all
select 20173,'MRN' from dual union all
select 20174,'MRN' from dual union all
select 20175,'MRN' from dual union all
select 20176,'MRN' from dual union all
select 20177,'MRN' from dual union all
select 20178,'MRN' from dual union all
select 20179,'MRN' from dual union all
select 2018,'UZKA' from dual union all
select 20180,'MRN' from dual union all
select 20181,'MRN' from dual union all
select 20182,'MRN' from dual union all
select 20183,'MRN' from dual union all
select 20184,'MRN' from dual union all
select 20185,'MRN' from dual union all
select 20186,'MRN' from dual union all
select 20187,'MRN' from dual union all
select 20188,'MRN' from dual union all
select 20189,'MRN' from dual union all
select 2019,'ARDC' from dual union all
select 20190,'MRN' from dual union all
select 20191,'MRN' from dual union all
select 20192,'MRN' from dual union all
select 20193,'MRN' from dual union all
select 20194,'MRN' from dual union all
select 20195,'MRN' from dual union all
select 20196,'MRN' from dual union all
select 20197,'MRN' from dual union all
select 20198,'MRN' from dual union all
select 20199,'MRN' from dual union all
select 202,'WER836-3' from dual union all
select 2020,'ARDC' from dual union all
select 20200,'MRN' from dual union all
select 20201,'SUPA' from dual union all
select 20202,'MRN' from dual union all
select 20203,'MRN' from dual union all
select 20204,'MRN' from dual union all
select 20205,'MRN' from dual union all
select 20206,'MRN' from dual union all
select 20207,'MRN' from dual union all
select 20208,'MRN' from dual union all
select 20209,'MRN' from dual union all
select 2021,'UZKA' from dual union all
select 20210,'MRN' from dual union all
select 20211,'MRN' from dual union all
select 20212,'MRN' from dual union all
select 20213,'MRN' from dual union all
select 20214,'MRN' from dual union all
select 20215,'MRN' from dual union all
select 20216,'USA' from dual union all
select 20217,'MRN' from dual union all
select 20218,'MRN' from dual union all
select 20219,'MRN' from dual union all
select 2022,'UZKA' from dual;
insert into LAUNCH_AGENCY
select 20220,'MRN' from dual union all
select 20221,'MRN' from dual union all
select 20222,'MRN' from dual union all
select 20223,'UKMO' from dual union all
select 20224,'MRN' from dual union all
select 20225,'MRN' from dual union all
select 20226,'MRN' from dual union all
select 20227,'MRN' from dual union all
select 20228,'MRN' from dual union all
select 20229,'MRN' from dual union all
select 2023,'ARDC' from dual union all
select 20230,'MRN' from dual union all
select 20231,'MRN' from dual union all
select 20232,'MRN' from dual union all
select 20233,'MRN' from dual union all
select 20234,'MRN' from dual union all
select 20235,'MRN' from dual union all
select 20236,'UKMO' from dual union all
select 20237,'MRN' from dual union all
select 20238,'MRN' from dual union all
select 20239,'MRN' from dual union all
select 2024,'ARDC' from dual union all
select 20240,'MRN' from dual union all
select 20241,'MRN' from dual union all
select 20242,'MRN' from dual union all
select 20243,'MRN' from dual union all
select 20244,'MRN' from dual union all
select 20245,'MRN' from dual union all
select 20246,'MRN' from dual union all
select 20247,'MRN' from dual union all
select 20248,'UKMO' from dual union all
select 20249,'MRN' from dual union all
select 2025,'UZKA' from dual union all
select 20250,'MRN' from dual union all
select 20251,'MRN' from dual union all
select 20252,'UKMO' from dual union all
select 20253,'MRN' from dual union all
select 20254,'MRN' from dual union all
select 20255,'MRN' from dual union all
select 20256,'UKMO' from dual union all
select 20257,'MRN' from dual union all
select 20258,'MRN' from dual union all
select 20259,'MRN' from dual union all
select 2026,'ARDC' from dual union all
select 20260,'SUPA' from dual union all
select 20261,'MRN' from dual union all
select 20262,'MRN' from dual union all
select 20263,'MRN' from dual union all
select 20264,'MRN' from dual union all
select 20265,'MRN' from dual union all
select 20266,'MRN' from dual union all
select 20267,'MRN' from dual union all
select 20268,'MRN' from dual union all
select 20269,'AN' from dual union all
select 2027,'ARDC' from dual union all
select 20270,'NASA' from dual union all
select 20271,'MRN' from dual union all
select 20272,'USAF' from dual union all
select 20273,'USAF' from dual union all
select 20274,'MRN' from dual union all
select 20275,'MRN' from dual union all
select 20276,'ISAS' from dual union all
select 20277,'MRN' from dual union all
select 20278,'MRN' from dual union all
select 20279,'MRN' from dual union all
select 2028,'SUI' from dual union all
select 20280,'NASA' from dual union all
select 20281,'MRN' from dual union all
select 20282,'MRN' from dual union all
select 20283,'MRN' from dual union all
select 20284,'MRN' from dual union all
select 20285,'MRN' from dual union all
select 20286,'MRN' from dual union all
select 20287,'ISAS' from dual union all
select 20288,'MRN' from dual union all
select 20289,'MRN' from dual union all
select 2029,'SUI' from dual union all
select 20290,'MRN' from dual union all
select 20291,'MRN' from dual union all
select 20292,'MRN' from dual union all
select 20293,'MRN' from dual union all
select 20294,'MRN' from dual union all
select 20295,'MRN' from dual union all
select 20296,'MRN' from dual union all
select 20297,'MRN' from dual union all
select 20298,'ISAS' from dual union all
select 20299,'MRN' from dual union all
select 203,'WER836-1' from dual union all
select 2030,'SUI' from dual union all
select 20300,'MRN' from dual union all
select 20301,'MRN' from dual union all
select 20302,'MRN' from dual union all
select 20303,'MRN' from dual union all
select 20304,'MRN' from dual union all
select 20305,'MRN' from dual union all
select 20306,'MRN' from dual union all
select 20307,'MRN' from dual union all
select 20308,'MRN' from dual union all
select 20309,'UKMO' from dual union all
select 2031,'SUI' from dual;
insert into LAUNCH_AGENCY
select 20310,'MRN' from dual union all
select 20311,'MRN' from dual union all
select 20312,'MRN' from dual union all
select 20313,'MRN' from dual union all
select 20314,'MRN' from dual union all
select 20315,'MRN' from dual union all
select 20316,'MRN' from dual union all
select 20317,'NASA' from dual union all
select 20318,'MRN' from dual union all
select 20319,'MRN' from dual union all
select 2032,'USA' from dual union all
select 20320,'MRN' from dual union all
select 20321,'MRN' from dual union all
select 20322,'MRN' from dual union all
select 20323,'MRN' from dual union all
select 20324,'MRN' from dual union all
select 20325,'MRN' from dual union all
select 20326,'MRN' from dual union all
select 20327,'MRN' from dual union all
select 20328,'MRN' from dual union all
select 20329,'MRN' from dual union all
select 2033,'SUI' from dual union all
select 20330,'MRN' from dual union all
select 20331,'MRN' from dual union all
select 20332,'MRN' from dual union all
select 20333,'MRN' from dual union all
select 20334,'MRN' from dual union all
select 20335,'MRN' from dual union all
select 20336,'MRN' from dual union all
select 20337,'MRN' from dual union all
select 20338,'MRN' from dual union all
select 20339,'MRN' from dual union all
select 2034,'SUI' from dual union all
select 20340,'MRN' from dual union all
select 20341,'MRN' from dual union all
select 20342,'MRN' from dual union all
select 20343,'MRN' from dual union all
select 20344,'MRN' from dual union all
select 20345,'MRN' from dual union all
select 20346,'MRN' from dual union all
select 20347,'MRN' from dual union all
select 20348,'MRN' from dual union all
select 20349,'MRN' from dual union all
select 2035,'SUI' from dual union all
select 20350,'MRN' from dual union all
select 20351,'MRN' from dual union all
select 20352,'MRN' from dual union all
select 20353,'MRN' from dual union all
select 20354,'MRN' from dual union all
select 20355,'MRN' from dual union all
select 20356,'MRN' from dual union all
select 20357,'MRN' from dual union all
select 20358,'MRN' from dual union all
select 20359,'MRN' from dual union all
select 2036,'USA' from dual union all
select 20360,'MRN' from dual union all
select 20361,'MRN' from dual union all
select 20362,'MRN' from dual union all
select 20363,'MRN' from dual union all
select 20364,'MRN' from dual union all
select 20365,'MRN' from dual union all
select 20366,'MRN' from dual union all
select 20367,'MRN' from dual union all
select 20368,'MRN' from dual union all
select 20369,'MRN' from dual union all
select 2037,'UZKA' from dual union all
select 20370,'MRN' from dual union all
select 20371,'MRN' from dual union all
select 20372,'MRN' from dual union all
select 20373,'MRN' from dual union all
select 20374,'MRN' from dual union all
select 20375,'MRN' from dual union all
select 20376,'MRN' from dual union all
select 20377,'MRN' from dual union all
select 20378,'MRN' from dual union all
select 20379,'MRN' from dual union all
select 2038,'UZKA' from dual union all
select 20380,'MRN' from dual union all
select 20381,'MRN' from dual union all
select 20382,'MRN' from dual union all
select 20383,'MRN' from dual union all
select 20384,'MRN' from dual union all
select 20385,'MRN' from dual union all
select 20386,'MRN' from dual union all
select 20387,'MRN' from dual union all
select 20388,'MRN' from dual union all
select 20389,'MRN' from dual union all
select 2039,'ARDC' from dual union all
select 20390,'MRN' from dual union all
select 20391,'MRN' from dual union all
select 20392,'MRN' from dual union all
select 20393,'MRN' from dual union all
select 20394,'MRN' from dual union all
select 20395,'MRN' from dual union all
select 20396,'MRN' from dual union all
select 20397,'MRN' from dual union all
select 20398,'MRN' from dual union all
select 20399,'MRN' from dual union all
select 204,'WER836-1' from dual union all
select 2040,'UZKA' from dual;
insert into LAUNCH_AGENCY
select 20400,'MRN' from dual union all
select 20401,'MRN' from dual union all
select 20402,'MRN' from dual union all
select 20403,'MRN' from dual union all
select 20404,'MRN' from dual union all
select 20405,'MRN' from dual union all
select 20406,'MRN' from dual union all
select 20407,'MRN' from dual union all
select 20408,'MRN' from dual union all
select 20409,'MRN' from dual union all
select 2041,'UZKA' from dual union all
select 20410,'MRN' from dual union all
select 20411,'MRN' from dual union all
select 20412,'MRN' from dual union all
select 20413,'MRN' from dual union all
select 20414,'MRN' from dual union all
select 20415,'MRN' from dual union all
select 20416,'MRN' from dual union all
select 20417,'MRN' from dual union all
select 20418,'MRN' from dual union all
select 20419,'UKMO' from dual union all
select 2042,'UZKA' from dual union all
select 20420,'MRN' from dual union all
select 20421,'MRN' from dual union all
select 20422,'MRN' from dual union all
select 20423,'MRN' from dual union all
select 20424,'MRN' from dual union all
select 20425,'MRN' from dual union all
select 20426,'MRN' from dual union all
select 20427,'MRN' from dual union all
select 20428,'MRN' from dual union all
select 20429,'MRN' from dual union all
select 2043,'UZKA' from dual union all
select 20430,'MRN' from dual union all
select 20431,'MRN' from dual union all
select 20432,'MRN' from dual union all
select 20433,'MRN' from dual union all
select 20434,'MRN' from dual union all
select 20435,'MRN' from dual union all
select 20436,'MRN' from dual union all
select 20437,'MRN' from dual union all
select 20438,'MRN' from dual union all
select 20439,'MRN' from dual union all
select 2044,'UZKA' from dual union all
select 20440,'MRN' from dual union all
select 20441,'MRN' from dual union all
select 20442,'MRN' from dual union all
select 20443,'MRN' from dual union all
select 20444,'MRN' from dual union all
select 20445,'MRN' from dual union all
select 20446,'MRN' from dual union all
select 20447,'MRN' from dual union all
select 20448,'MRN' from dual union all
select 20449,'MRN' from dual union all
select 2045,'UZKA' from dual union all
select 20450,'MRN' from dual union all
select 20451,'MRN' from dual union all
select 20452,'MRN' from dual union all
select 20453,'MRN' from dual union all
select 20454,'MRN' from dual union all
select 20455,'MRN' from dual union all
select 20456,'MRN' from dual union all
select 20457,'MRN' from dual union all
select 20458,'MRN' from dual union all
select 20459,'MRN' from dual union all
select 2046,'UZKA' from dual union all
select 20460,'MRN' from dual union all
select 20461,'MRN' from dual union all
select 20462,'MRN' from dual union all
select 20463,'MRN' from dual union all
select 20464,'MRN' from dual union all
select 20465,'MRN' from dual union all
select 20466,'MRN' from dual union all
select 20467,'MRN' from dual union all
select 20468,'MRN' from dual union all
select 20469,'MRN' from dual union all
select 2047,'UZKA' from dual union all
select 20470,'MRN' from dual union all
select 20471,'MRN' from dual union all
select 20472,'MRN' from dual union all
select 20473,'MRN' from dual union all
select 20474,'MRN' from dual union all
select 20475,'MRN' from dual union all
select 20476,'MRN' from dual union all
select 20477,'MRN' from dual union all
select 20478,'MRN' from dual union all
select 20479,'MRN' from dual union all
select 2048,'UZKA' from dual union all
select 20480,'MRN' from dual union all
select 20481,'MRN' from dual union all
select 20482,'MRN' from dual union all
select 20483,'MRN' from dual union all
select 20484,'MRN' from dual union all
select 20485,'MRN' from dual union all
select 20486,'MRN' from dual union all
select 20487,'MRN' from dual union all
select 20488,'MRN' from dual union all
select 20489,'MRN' from dual union all
select 2049,'UZKA' from dual union all
select 20490,'MRN' from dual;
insert into LAUNCH_AGENCY
select 20491,'MRN' from dual union all
select 20492,'MRN' from dual union all
select 20493,'MRN' from dual union all
select 20494,'MRN' from dual union all
select 20495,'MRN' from dual union all
select 20496,'UKMO' from dual union all
select 20497,'MRN' from dual union all
select 20498,'MRN' from dual union all
select 20499,'MRN' from dual union all
select 205,'WER836-1' from dual union all
select 2050,'UZKA' from dual union all
select 20500,'MRN' from dual union all
select 20501,'MRN' from dual union all
select 20502,'MRN' from dual union all
select 20503,'MRN' from dual union all
select 20504,'MRN' from dual union all
select 20505,'MRN' from dual union all
select 20506,'MRN' from dual union all
select 20507,'MRN' from dual union all
select 20508,'MRN' from dual union all
select 20509,'MRN' from dual union all
select 2051,'UZKA' from dual union all
select 20510,'MRN' from dual union all
select 20511,'MRN' from dual union all
select 20512,'MRN' from dual union all
select 20513,'MRN' from dual union all
select 20514,'MRN' from dual union all
select 20515,'MRN' from dual union all
select 20516,'MRN' from dual union all
select 20517,'MRN' from dual union all
select 20518,'MRN' from dual union all
select 20519,'MRN' from dual union all
select 2052,'UZKA' from dual union all
select 20520,'MRN' from dual union all
select 20521,'MRN' from dual union all
select 20522,'MRN' from dual union all
select 20523,'MRN' from dual union all
select 20524,'MRN' from dual union all
select 20525,'MRN' from dual union all
select 20526,'MRN' from dual union all
select 20527,'MRN' from dual union all
select 20528,'MRN' from dual union all
select 20529,'MRN' from dual union all
select 2053,'UZKA' from dual union all
select 20530,'MRN' from dual union all
select 20531,'MRN' from dual union all
select 20532,'MRN' from dual union all
select 20533,'MRN' from dual union all
select 20534,'MRN' from dual union all
select 20535,'MRN' from dual union all
select 20536,'MRN' from dual union all
select 20537,'MRN' from dual union all
select 20538,'MRN' from dual union all
select 20539,'MRN' from dual union all
select 2054,'ARDC' from dual union all
select 20540,'MRN' from dual union all
select 20541,'MRN' from dual union all
select 20542,'USA' from dual union all
select 20543,'MRN' from dual union all
select 20544,'MRN' from dual union all
select 20545,'MRN' from dual union all
select 20546,'MRN' from dual union all
select 20547,'MRN' from dual union all
select 20548,'MRN' from dual union all
select 20549,'MRN' from dual union all
select 2055,'ARDC' from dual union all
select 20550,'MRN' from dual union all
select 20551,'MRN' from dual union all
select 20552,'MRN' from dual union all
select 20553,'MRN' from dual union all
select 20554,'MRN' from dual union all
select 20555,'MRN' from dual union all
select 20556,'AN' from dual union all
select 20557,'MRN' from dual union all
select 20558,'MRN' from dual union all
select 20559,'MRN' from dual union all
select 2056,'UZKA' from dual union all
select 20560,'MRN' from dual union all
select 20561,'MRN' from dual union all
select 20562,'MRN' from dual union all
select 20563,'MRN' from dual union all
select 20564,'MRN' from dual union all
select 20565,'MRN' from dual union all
select 20566,'MRN' from dual union all
select 20567,'MRN' from dual union all
select 20568,'MRN' from dual union all
select 20569,'MRN' from dual union all
select 2057,'UZKA' from dual union all
select 20570,'MRN' from dual union all
select 20571,'MRN' from dual union all
select 20572,'MRN' from dual union all
select 20573,'MRN' from dual union all
select 20574,'MRN' from dual union all
select 20575,'MRN' from dual union all
select 20576,'MRN' from dual union all
select 20577,'MRN' from dual union all
select 20578,'MRN' from dual union all
select 20579,'MRN' from dual union all
select 2058,'NRL' from dual union all
select 20580,'MRN' from dual;
insert into LAUNCH_AGENCY
select 20581,'MRN' from dual union all
select 20582,'MRN' from dual union all
select 20583,'MRN' from dual union all
select 20584,'MRN' from dual union all
select 20585,'MRN' from dual union all
select 20586,'MRN' from dual union all
select 20587,'MRN' from dual union all
select 20588,'MRN' from dual union all
select 20589,'MRN' from dual union all
select 2059,'UZKA' from dual union all
select 20590,'MRN' from dual union all
select 20591,'MRN' from dual union all
select 20592,'MRN' from dual union all
select 20593,'MRN' from dual union all
select 20594,'MRN' from dual union all
select 20595,'UKMO' from dual union all
select 20596,'MRN' from dual union all
select 20597,'MRN' from dual union all
select 20598,'USAF' from dual union all
select 20599,'USAF' from dual union all
select 206,'WER836-1' from dual union all
select 2060,'NRL' from dual union all
select 20600,'MRN' from dual union all
select 20601,'MRN' from dual union all
select 20602,'MRN' from dual union all
select 20603,'MRN' from dual union all
select 20604,'MRN' from dual union all
select 20605,'MRN' from dual union all
select 20606,'MRN' from dual union all
select 20607,'MRN' from dual union all
select 20608,'MRN' from dual union all
select 20609,'MRN' from dual union all
select 2061,'UZKA' from dual union all
select 20610,'MRN' from dual union all
select 20611,'MRN' from dual union all
select 20612,'MRN' from dual union all
select 20613,'MRN' from dual union all
select 20614,'MRN' from dual union all
select 20615,'MRN' from dual union all
select 20616,'MRN' from dual union all
select 20617,'MRN' from dual union all
select 20618,'MRN' from dual union all
select 20619,'MRN' from dual union all
select 2062,'NRL' from dual union all
select 20620,'MRN' from dual union all
select 20621,'MRN' from dual union all
select 20622,'MRN' from dual union all
select 20623,'MRN' from dual union all
select 20624,'MRN' from dual union all
select 20625,'MRN' from dual union all
select 20626,'MRN' from dual union all
select 20627,'MRN' from dual union all
select 20628,'MRN' from dual union all
select 20629,'MRN' from dual union all
select 2063,'UZKA' from dual union all
select 20630,'MRN' from dual union all
select 20631,'MRN' from dual union all
select 20632,'MRN' from dual union all
select 20633,'MRN' from dual union all
select 20634,'MRN' from dual union all
select 20635,'MRN' from dual union all
select 20636,'MRN' from dual union all
select 20637,'MRN' from dual union all
select 20638,'MRN' from dual union all
select 20639,'MRN' from dual union all
select 2064,'UZKA' from dual union all
select 20640,'MRN' from dual union all
select 20641,'MRN' from dual union all
select 20642,'MRN' from dual union all
select 20643,'MRN' from dual union all
select 20644,'MRN' from dual union all
select 20645,'MRN' from dual union all
select 20646,'MRN' from dual union all
select 20647,'MRN' from dual union all
select 20648,'MRN' from dual union all
select 20649,'MRN' from dual union all
select 2065,'UZKA' from dual union all
select 20650,'MRN' from dual union all
select 20651,'MRN' from dual union all
select 20652,'MRN' from dual union all
select 20653,'MRN' from dual union all
select 20654,'MRN' from dual union all
select 20655,'MRN' from dual union all
select 20656,'MRN' from dual union all
select 20657,'MRN' from dual union all
select 20658,'MRN' from dual union all
select 20659,'MRN' from dual union all
select 2066,'UZKA' from dual union all
select 20660,'MRN' from dual union all
select 20661,'MRN' from dual union all
select 20662,'MRN' from dual union all
select 20663,'MRN' from dual union all
select 20664,'MRN' from dual union all
select 20665,'MRN' from dual union all
select 20666,'MRN' from dual union all
select 20667,'MRN' from dual union all
select 20668,'MRN' from dual union all
select 20669,'MRN' from dual union all
select 2067,'UZKA' from dual union all
select 20670,'MRN' from dual;
insert into LAUNCH_AGENCY
select 20671,'MRN' from dual union all
select 20672,'MRN' from dual union all
select 20673,'MRN' from dual union all
select 20674,'MRN' from dual union all
select 20675,'MRN' from dual union all
select 20676,'MRN' from dual union all
select 20677,'MRN' from dual union all
select 20678,'MRN' from dual union all
select 20679,'MRN' from dual union all
select 2068,'UZKA' from dual union all
select 20680,'INTA' from dual union all
select 20681,'MRN' from dual union all
select 20682,'MRN' from dual union all
select 20683,'MRN' from dual union all
select 20684,'MRN' from dual union all
select 20685,'MRN' from dual union all
select 20686,'MRN' from dual union all
select 20687,'MRN' from dual union all
select 20688,'MRN' from dual union all
select 20689,'MRN' from dual union all
select 2069,'UZKA' from dual union all
select 20690,'MRN' from dual union all
select 20691,'MRN' from dual union all
select 20692,'MRN' from dual union all
select 20693,'MRN' from dual union all
select 20694,'MRN' from dual union all
select 20695,'MRN' from dual union all
select 20696,'MRN' from dual union all
select 20697,'MRN' from dual union all
select 20698,'MRN' from dual union all
select 20699,'MRN' from dual union all
select 207,'WER836-1' from dual union all
select 2070,'UZKA' from dual union all
select 20700,'MRN' from dual union all
select 20701,'MRN' from dual union all
select 20702,'MRN' from dual union all
select 20703,'MRN' from dual union all
select 20704,'MRN' from dual union all
select 20705,'MRN' from dual union all
select 20706,'CA' from dual union all
select 20707,'MRN' from dual union all
select 20708,'MRN' from dual union all
select 20709,'MRN' from dual union all
select 2071,'UZKA' from dual union all
select 20710,'MRN' from dual union all
select 20711,'MRN' from dual union all
select 20712,'MRN' from dual union all
select 20713,'MRN' from dual union all
select 20714,'MRN' from dual union all
select 20715,'MRN' from dual union all
select 20716,'MRN' from dual union all
select 20717,'MRN' from dual union all
select 20718,'MRN' from dual union all
select 20719,'MRN' from dual union all
select 2072,'PARD' from dual union all
select 20720,'MRN' from dual union all
select 20721,'MRN' from dual union all
select 20722,'MRN' from dual union all
select 20723,'MRN' from dual union all
select 20724,'MRN' from dual union all
select 20725,'MRN' from dual union all
select 20726,'MRN' from dual union all
select 20727,'MRN' from dual union all
select 20728,'MRN' from dual union all
select 20729,'MRN' from dual union all
select 2073,'USA' from dual union all
select 20730,'MRN' from dual union all
select 20731,'MRN' from dual union all
select 20732,'MRN' from dual union all
select 20733,'MRN' from dual union all
select 20734,'MRN' from dual union all
select 20735,'MRN' from dual union all
select 20736,'MRN' from dual union all
select 20737,'MRN' from dual union all
select 20738,'MRN' from dual union all
select 20739,'MRN' from dual union all
select 2074,'PARD' from dual union all
select 20740,'MRN' from dual union all
select 20741,'MRN' from dual union all
select 20742,'MRN' from dual union all
select 20743,'MRN' from dual union all
select 20744,'MRN' from dual union all
select 20745,'MRN' from dual union all
select 20746,'MRN' from dual union all
select 20747,'MRN' from dual union all
select 20748,'MRN' from dual union all
select 20749,'MRN' from dual union all
select 2075,'USA' from dual union all
select 20750,'MRN' from dual union all
select 20751,'MRN' from dual union all
select 20752,'MRN' from dual union all
select 20753,'MRN' from dual union all
select 20754,'MRN' from dual union all
select 20755,'MRN' from dual union all
select 20756,'MRN' from dual union all
select 20757,'MRN' from dual union all
select 20758,'MRN' from dual union all
select 20759,'MRN' from dual union all
select 2076,'PARD' from dual union all
select 20760,'UKMO' from dual;
insert into LAUNCH_AGENCY
select 20761,'MRN' from dual union all
select 20762,'MRN' from dual union all
select 20763,'INTA' from dual union all
select 20764,'MRN' from dual union all
select 20765,'MRN' from dual union all
select 20766,'MRN' from dual union all
select 20767,'MRN' from dual union all
select 20768,'MRN' from dual union all
select 20769,'MRN' from dual union all
select 2077,'USAF' from dual union all
select 20770,'MRN' from dual union all
select 20771,'MRN' from dual union all
select 20772,'MRN' from dual union all
select 20773,'MRN' from dual union all
select 20774,'MRN' from dual union all
select 20775,'MRN' from dual union all
select 20776,'MRN' from dual union all
select 20777,'MRN' from dual union all
select 20778,'MRN' from dual union all
select 20779,'MRN' from dual union all
select 2078,'USA' from dual union all
select 20780,'MRN' from dual union all
select 20781,'MRN' from dual union all
select 20782,'MRN' from dual union all
select 20783,'MRN' from dual union all
select 20784,'MRN' from dual union all
select 20785,'MRN' from dual union all
select 20786,'MRN' from dual union all
select 20787,'MRN' from dual union all
select 20788,'MRN' from dual union all
select 20789,'MRN' from dual union all
select 2079,'USAF' from dual union all
select 20790,'MRN' from dual union all
select 20791,'MRN' from dual union all
select 20792,'MRN' from dual union all
select 20793,'MRN' from dual union all
select 20794,'MRN' from dual union all
select 20795,'MRN' from dual union all
select 20796,'MRN' from dual union all
select 20797,'MRN' from dual union all
select 20798,'MRN' from dual union all
select 20799,'MRN' from dual union all
select 208,'WER836-1' from dual union all
select 2080,'USAF' from dual union all
select 20800,'MRN' from dual union all
select 20801,'MRN' from dual union all
select 20802,'MRN' from dual union all
select 20803,'MRN' from dual union all
select 20804,'MRN' from dual union all
select 20805,'MRN' from dual union all
select 20806,'MRN' from dual union all
select 20807,'MRN' from dual union all
select 20808,'MRN' from dual union all
select 20809,'MRN' from dual union all
select 2081,'USAF' from dual union all
select 20810,'MRN' from dual union all
select 20811,'MRN' from dual union all
select 20812,'MRN' from dual union all
select 20813,'MRN' from dual union all
select 20814,'INTA' from dual union all
select 20815,'MRN' from dual union all
select 20816,'MRN' from dual union all
select 20817,'MRN' from dual union all
select 20818,'MRN' from dual union all
select 20819,'MRN' from dual union all
select 2082,'USAF' from dual union all
select 20820,'MRN' from dual union all
select 20821,'MRN' from dual union all
select 20822,'MRN' from dual union all
select 20823,'USA' from dual union all
select 20824,'MRN' from dual union all
select 20825,'MRN' from dual union all
select 20826,'MRN' from dual union all
select 20827,'INTA' from dual union all
select 20828,'MRN' from dual union all
select 20829,'MRN' from dual union all
select 2083,'USA' from dual union all
select 20830,'MRN' from dual union all
select 20831,'MRN' from dual union all
select 20832,'MRN' from dual union all
select 20833,'MRN' from dual union all
select 20834,'MRN' from dual union all
select 20835,'MRN' from dual union all
select 20836,'MRN' from dual union all
select 20837,'MRN' from dual union all
select 20838,'MRN' from dual union all
select 20839,'MRN' from dual union all
select 2084,'USA' from dual union all
select 20840,'MRN' from dual union all
select 20841,'MRN' from dual union all
select 20842,'MRN' from dual union all
select 20843,'MRN' from dual union all
select 20844,'MRN' from dual union all
select 20845,'MRN' from dual union all
select 20846,'MRN' from dual union all
select 20847,'MRN' from dual union all
select 20848,'MRN' from dual union all
select 20849,'MRN' from dual union all
select 2085,'USA' from dual union all
select 20850,'MRN' from dual;
insert into LAUNCH_AGENCY
select 20851,'MRN' from dual union all
select 20852,'MRN' from dual union all
select 20853,'INTA' from dual union all
select 20854,'MRN' from dual union all
select 20855,'MRN' from dual union all
select 20856,'MRN' from dual union all
select 20857,'MRN' from dual union all
select 20858,'MRN' from dual union all
select 20859,'USA' from dual union all
select 2086,'USA' from dual union all
select 20860,'MRN' from dual union all
select 20861,'MRN' from dual union all
select 20862,'MRN' from dual union all
select 20863,'MRN' from dual union all
select 20864,'MRN' from dual union all
select 20865,'MRN' from dual union all
select 20866,'MRN' from dual union all
select 20867,'MRN' from dual union all
select 20868,'MRN' from dual union all
select 20869,'MRN' from dual union all
select 2087,'USA' from dual union all
select 20870,'MRN' from dual union all
select 20871,'MRN' from dual union all
select 20872,'MRN' from dual union all
select 20873,'MRN' from dual union all
select 20874,'MRN' from dual union all
select 20875,'MRN' from dual union all
select 20876,'MRN' from dual union all
select 20877,'AN' from dual union all
select 20878,'MRN' from dual union all
select 20879,'MRN' from dual union all
select 2088,'USA' from dual union all
select 20880,'MRN' from dual union all
select 20881,'MRN' from dual union all
select 20882,'MRN' from dual union all
select 20883,'MRN' from dual union all
select 20884,'MRN' from dual union all
select 20885,'MRN' from dual union all
select 20886,'MRN' from dual union all
select 20887,'MRN' from dual union all
select 20888,'MRN' from dual union all
select 20889,'MRN' from dual union all
select 2089,'ORDRSA' from dual union all
select 20890,'MRN' from dual union all
select 20891,'MRN' from dual union all
select 20892,'MRN' from dual union all
select 20893,'MRN' from dual union all
select 20894,'MRN' from dual union all
select 20895,'MRN' from dual union all
select 20896,'MRN' from dual union all
select 20897,'MRN' from dual union all
select 20898,'MRN' from dual union all
select 20899,'MRN' from dual union all
select 209,'WEHR' from dual union all
select 2090,'USA' from dual union all
select 20900,'MRN' from dual union all
select 20901,'MRN' from dual union all
select 20902,'MRN' from dual union all
select 20903,'MRN' from dual union all
select 20904,'MRN' from dual union all
select 20905,'MRN' from dual union all
select 20906,'MRN' from dual union all
select 20907,'MRN' from dual union all
select 20908,'MRN' from dual union all
select 20909,'MRN' from dual union all
select 2091,'USA' from dual union all
select 20910,'MRN' from dual union all
select 20911,'MRN' from dual union all
select 20912,'MRN' from dual union all
select 20913,'MRN' from dual union all
select 20914,'MRN' from dual union all
select 20915,'MRN' from dual union all
select 20916,'MRN' from dual union all
select 20917,'MRN' from dual union all
select 20918,'MRN' from dual union all
select 20919,'MRN' from dual union all
select 2092,'USA' from dual union all
select 20920,'MRN' from dual union all
select 20921,'MRN' from dual union all
select 20922,'MRN' from dual union all
select 20923,'MRN' from dual union all
select 20924,'MRN' from dual union all
select 20925,'MRN' from dual union all
select 20926,'MRN' from dual union all
select 20927,'MRN' from dual union all
select 20928,'MRN' from dual union all
select 20929,'MRN' from dual union all
select 2093,'USA' from dual union all
select 20930,'MRN' from dual union all
select 20931,'MRN' from dual union all
select 20932,'MRN' from dual union all
select 20933,'MRN' from dual union all
select 20934,'MRN' from dual union all
select 20935,'MRN' from dual union all
select 20936,'UKMO' from dual union all
select 20937,'MRN' from dual union all
select 20938,'MRN' from dual union all
select 20939,'MRN' from dual union all
select 2094,'USA' from dual union all
select 20940,'MRN' from dual;
insert into LAUNCH_AGENCY
select 20941,'MRN' from dual union all
select 20942,'MRN' from dual union all
select 20943,'MRN' from dual union all
select 20944,'MRN' from dual union all
select 20945,'MRN' from dual union all
select 20946,'MRN' from dual union all
select 20947,'MRN' from dual union all
select 20948,'MRN' from dual union all
select 20949,'MRN' from dual union all
select 2095,'ORDRSA' from dual union all
select 20950,'MRN' from dual union all
select 20951,'GMS' from dual union all
select 20952,'MRN' from dual union all
select 20953,'MRN' from dual union all
select 20954,'MRN' from dual union all
select 20955,'MRN' from dual union all
select 20956,'MRN' from dual union all
select 20957,'MRN' from dual union all
select 20958,'MRN' from dual union all
select 20959,'MRN' from dual union all
select 2096,'USA' from dual union all
select 20960,'MRN' from dual union all
select 20961,'MRN' from dual union all
select 20962,'MRN' from dual union all
select 20963,'MRN' from dual union all
select 20964,'MRN' from dual union all
select 20965,'MRN' from dual union all
select 20966,'MRN' from dual union all
select 20967,'MRN' from dual union all
select 20968,'MRN' from dual union all
select 20969,'MRN' from dual union all
select 2097,'USA' from dual union all
select 20970,'MRN' from dual union all
select 20971,'MRN' from dual union all
select 20972,'MRN' from dual union all
select 20973,'MRN' from dual union all
select 20974,'MRN' from dual union all
select 20975,'MRN' from dual union all
select 20976,'ISRO' from dual union all
select 20977,'MRN' from dual union all
select 20978,'MRN' from dual union all
select 20979,'MRN' from dual union all
select 2098,'PARD' from dual union all
select 20980,'MRN' from dual union all
select 20981,'MRN' from dual union all
select 20982,'MRN' from dual union all
select 20983,'MRN' from dual union all
select 20984,'MRN' from dual union all
select 20985,'MRN' from dual union all
select 20986,'MRN' from dual union all
select 20987,'NASA' from dual union all
select 20988,'MRN' from dual union all
select 20989,'MRN' from dual union all
select 2099,'ORDRSA' from dual union all
select 20990,'MRN' from dual union all
select 20991,'MRN' from dual union all
select 20992,'MRN' from dual union all
select 20993,'MRN' from dual union all
select 20994,'MRN' from dual union all
select 20995,'MRN' from dual union all
select 20996,'MRN' from dual union all
select 20997,'MRN' from dual union all
select 20998,'MRN' from dual union all
select 20999,'MRN' from dual union all
select 21,'WEHR' from dual union all
select 210,'WER836-1' from dual union all
select 2100,'USA' from dual union all
select 21000,'MRN' from dual union all
select 21001,'MRN' from dual union all
select 21002,'MRN' from dual union all
select 21003,'MRN' from dual union all
select 21004,'MRN' from dual union all
select 21005,'MRN' from dual union all
select 21006,'MRN' from dual union all
select 21007,'MRN' from dual union all
select 21008,'MRN' from dual union all
select 21009,'MRN' from dual union all
select 2101,'USA' from dual union all
select 21010,'MRN' from dual union all
select 21011,'MRN' from dual union all
select 21012,'MRN' from dual union all
select 21013,'MRN' from dual union all
select 21014,'MRN' from dual union all
select 21015,'MRN' from dual union all
select 21016,'MRN' from dual union all
select 21017,'MRN' from dual union all
select 21018,'MRN' from dual union all
select 21019,'SUPA' from dual union all
select 2102,'USA' from dual union all
select 21020,'MRN' from dual union all
select 21021,'MRN' from dual union all
select 21022,'MRN' from dual union all
select 21023,'MRN' from dual union all
select 21024,'MRN' from dual union all
select 21025,'MRN' from dual union all
select 21026,'MRN' from dual union all
select 21027,'MRN' from dual union all
select 21028,'MRN' from dual union all
select 21029,'MRN' from dual union all
select 2103,'USA' from dual;
insert into LAUNCH_AGENCY
select 21030,'MRN' from dual union all
select 21031,'MRN' from dual union all
select 21032,'MRN' from dual union all
select 21033,'MRN' from dual union all
select 21034,'MRN' from dual union all
select 21035,'MRN' from dual union all
select 21036,'MRN' from dual union all
select 21037,'MRN' from dual union all
select 21038,'MRN' from dual union all
select 21039,'MRN' from dual union all
select 2104,'USAF' from dual union all
select 21040,'MRN' from dual union all
select 21041,'MRN' from dual union all
select 21042,'MRN' from dual union all
select 21043,'MRN' from dual union all
select 21044,'MRN' from dual union all
select 21045,'MRN' from dual union all
select 21046,'MRN' from dual union all
select 21047,'MRN' from dual union all
select 21048,'MRN' from dual union all
select 21049,'MRN' from dual union all
select 2105,'USAF' from dual union all
select 21050,'MRN' from dual union all
select 21051,'MRN' from dual union all
select 21052,'ISRO' from dual union all
select 21053,'MRN' from dual union all
select 21054,'MRN' from dual union all
select 21055,'MRN' from dual union all
select 21056,'MRN' from dual union all
select 21057,'MRN' from dual union all
select 21058,'MRN' from dual union all
select 21059,'MRN' from dual union all
select 2106,'USA' from dual union all
select 21060,'MRN' from dual union all
select 21061,'MRN' from dual union all
select 21062,'MRN' from dual union all
select 21063,'MRN' from dual union all
select 21064,'MRN' from dual union all
select 21065,'MRN' from dual union all
select 21066,'MRN' from dual union all
select 21067,'MRN' from dual union all
select 21068,'MRN' from dual union all
select 21069,'MRN' from dual union all
select 2107,'PARD' from dual union all
select 21070,'MRN' from dual union all
select 21071,'MRN' from dual union all
select 21072,'MRN' from dual union all
select 21073,'MRN' from dual union all
select 21074,'MRN' from dual union all
select 21075,'MRN' from dual union all
select 21076,'MRN' from dual union all
select 21077,'MRN' from dual union all
select 21078,'MRN' from dual union all
select 21079,'MRN' from dual union all
select 2108,'USA' from dual union all
select 21080,'MRN' from dual union all
select 21081,'MRN' from dual union all
select 21082,'MRN' from dual union all
select 21083,'MRN' from dual union all
select 21084,'MRN' from dual union all
select 21085,'MRN' from dual union all
select 21086,'MRN' from dual union all
select 21087,'MRN' from dual union all
select 21088,'MRN' from dual union all
select 21089,'MRN' from dual union all
select 2109,'PARD' from dual union all
select 21090,'MRN' from dual union all
select 21091,'MRN' from dual union all
select 21092,'MRN' from dual union all
select 21093,'MRN' from dual union all
select 21094,'MRN' from dual union all
select 21095,'MRN' from dual union all
select 21096,'MRN' from dual union all
select 21097,'MRN' from dual union all
select 21098,'MRN' from dual union all
select 21099,'MRN' from dual union all
select 211,'WER836-1' from dual union all
select 2110,'ORDRSA' from dual union all
select 21100,'MRN' from dual union all
select 21101,'MRN' from dual union all
select 21102,'MRN' from dual union all
select 21103,'MRN' from dual union all
select 21104,'MRN' from dual union all
select 21105,'MRN' from dual union all
select 21106,'MRN' from dual union all
select 21107,'MRN' from dual union all
select 21108,'MRN' from dual union all
select 21109,'MRN' from dual union all
select 2111,'ORDRSA' from dual union all
select 21110,'MRN' from dual union all
select 21111,'MRN' from dual union all
select 21112,'MRN' from dual union all
select 21113,'MRN' from dual union all
select 21114,'MRN' from dual union all
select 21115,'MRN' from dual union all
select 21116,'MRN' from dual union all
select 21117,'MRN' from dual union all
select 21118,'MRN' from dual union all
select 21119,'MRN' from dual union all
select 2112,'USAF' from dual;
insert into LAUNCH_AGENCY
select 21120,'MRN' from dual union all
select 21121,'MRN' from dual union all
select 21122,'MRN' from dual union all
select 21123,'MRN' from dual union all
select 21124,'MRN' from dual union all
select 21125,'MRN' from dual union all
select 21126,'MRN' from dual union all
select 21127,'MRN' from dual union all
select 21128,'MRN' from dual union all
select 21129,'MRN' from dual union all
select 2113,'USA' from dual union all
select 21130,'MRN' from dual union all
select 21131,'MRN' from dual union all
select 21132,'MRN' from dual union all
select 21133,'MRN' from dual union all
select 21134,'MRN' from dual union all
select 21135,'MRN' from dual union all
select 21136,'MRN' from dual union all
select 21137,'MRN' from dual union all
select 21138,'MRN' from dual union all
select 21139,'MRN' from dual union all
select 2114,'USA' from dual union all
select 21140,'ISRO' from dual union all
select 21141,'SUPA' from dual union all
select 21142,'MRN' from dual union all
select 21143,'MRN' from dual union all
select 21144,'MRN' from dual union all
select 21145,'MRN' from dual union all
select 21146,'MRN' from dual union all
select 21147,'MRN' from dual union all
select 21148,'MRN' from dual union all
select 21149,'MRN' from dual union all
select 2115,'USA' from dual union all
select 21150,'MRN' from dual union all
select 21151,'MRN' from dual union all
select 21152,'MRN' from dual union all
select 21153,'MRN' from dual union all
select 21154,'MRN' from dual union all
select 21155,'MRN' from dual union all
select 21156,'MRN' from dual union all
select 21157,'MRN' from dual union all
select 21158,'MRN' from dual union all
select 21159,'MRN' from dual union all
select 2116,'USA' from dual union all
select 21160,'MRN' from dual union all
select 21161,'MRN' from dual union all
select 21162,'MRN' from dual union all
select 21163,'MRN' from dual union all
select 21164,'MRN' from dual union all
select 21165,'MRN' from dual union all
select 21166,'ISRO' from dual union all
select 21167,'MRN' from dual union all
select 21168,'MRN' from dual union all
select 21169,'MRN' from dual union all
select 2117,'ORDRSA' from dual union all
select 21170,'MRN' from dual union all
select 21171,'MRN' from dual union all
select 21172,'MRN' from dual union all
select 21173,'MRN' from dual union all
select 21174,'MRN' from dual union all
select 21175,'MRN' from dual union all
select 21176,'MRN' from dual union all
select 21177,'MRN' from dual union all
select 21178,'CALT' from dual union all
select 21179,'AN' from dual union all
select 2118,'USAF' from dual union all
select 21180,'AN' from dual union all
select 21181,'AN' from dual union all
select 21182,'AN' from dual union all
select 21183,'AN' from dual union all
select 21184,'AN' from dual union all
select 21185,'AN' from dual union all
select 21186,'AN' from dual union all
select 21187,'AN' from dual union all
select 21188,'AN' from dual union all
select 21189,'USA' from dual union all
select 2119,'USAF' from dual union all
select 21190,'AN' from dual union all
select 21191,'MRN' from dual union all
select 21192,'MRN' from dual union all
select 21193,'AN' from dual union all
select 21194,'AN' from dual union all
select 21195,'AN' from dual union all
select 21196,'AN' from dual union all
select 21197,'ESSA' from dual union all
select 21197,'NASA' from dual union all
select 21198,'AN' from dual union all
select 21199,'ESSA' from dual union all
select 21199,'NASA' from dual union all
select 212,'WER836-1' from dual union all
select 2120,'USA' from dual union all
select 21200,'ESSA' from dual union all
select 21200,'NASA' from dual union all
select 21201,'AN' from dual union all
select 21202,'AN' from dual union all
select 21203,'AN' from dual union all
select 21204,'AN' from dual union all
select 21205,'AN' from dual union all
select 21206,'CA' from dual union all
select 21207,'CA' from dual;
insert into LAUNCH_AGENCY
select 21208,'AN' from dual union all
select 21209,'AN' from dual union all
select 2121,'USA' from dual union all
select 21210,'AN' from dual union all
select 21211,'USA' from dual union all
select 21212,'AN' from dual union all
select 21213,'AN' from dual union all
select 21214,'AN' from dual union all
select 21215,'AN' from dual union all
select 21216,'AN' from dual union all
select 21217,'AN' from dual union all
select 21218,'AN' from dual union all
select 21219,'AN' from dual union all
select 2122,'USA' from dual union all
select 21220,'MRN' from dual union all
select 21221,'USA' from dual union all
select 21222,'AN' from dual union all
select 21223,'AN' from dual union all
select 21224,'AN' from dual union all
select 21225,'USA' from dual union all
select 21226,'USA' from dual union all
select 21227,'USA' from dual union all
select 21228,'AN' from dual union all
select 21229,'MRN' from dual union all
select 2123,'ORDRSA' from dual union all
select 21230,'MRN' from dual union all
select 21231,'AN' from dual union all
select 21232,'AN' from dual union all
select 21233,'AN' from dual union all
select 21234,'AN' from dual union all
select 21235,'AN' from dual union all
select 21236,'AN' from dual union all
select 21237,'AN' from dual union all
select 21238,'AN' from dual union all
select 21239,'AN' from dual union all
select 2124,'ORDRSA' from dual union all
select 21240,'AN' from dual union all
select 21241,'AN' from dual union all
select 21242,'AN' from dual union all
select 21243,'AN' from dual union all
select 21244,'AN' from dual union all
select 21245,'AN' from dual union all
select 21246,'AN' from dual union all
select 21247,'AN' from dual union all
select 21248,'AN' from dual union all
select 21249,'AN' from dual union all
select 2125,'ORDRSA' from dual union all
select 21250,'AN' from dual union all
select 21251,'AN' from dual union all
select 21252,'AN' from dual union all
select 21253,'MRN' from dual union all
select 21254,'AN' from dual union all
select 21255,'AN' from dual union all
select 21256,'AN' from dual union all
select 21257,'AN' from dual union all
select 21258,'AN' from dual union all
select 21259,'AN' from dual union all
select 2126,'USA' from dual union all
select 21260,'AN' from dual union all
select 21261,'AN' from dual union all
select 21262,'AN' from dual union all
select 21263,'AN' from dual union all
select 21264,'AN' from dual union all
select 21265,'AN' from dual union all
select 21266,'AN' from dual union all
select 21267,'AN' from dual union all
select 21268,'AN' from dual union all
select 21269,'AN' from dual union all
select 2127,'USA' from dual union all
select 21270,'AN' from dual union all
select 21271,'AN' from dual union all
select 21272,'AN' from dual union all
select 21273,'AN' from dual union all
select 21274,'MRN' from dual union all
select 21275,'AN' from dual union all
select 21276,'AN' from dual union all
select 21277,'AN' from dual union all
select 21278,'AN' from dual union all
select 21279,'AN' from dual union all
select 2128,'USA' from dual union all
select 21280,'AN' from dual union all
select 21281,'AN' from dual union all
select 21282,'NASA' from dual union all
select 21283,'AN' from dual union all
select 21284,'AN' from dual union all
select 21285,'MRN' from dual union all
select 21286,'AN' from dual union all
select 21287,'AN' from dual union all
select 21288,'MRN' from dual union all
select 21289,'AN' from dual union all
select 2129,'ORDRSA' from dual union all
select 21290,'AN' from dual union all
select 21291,'MRN' from dual union all
select 21292,'AN' from dual union all
select 21293,'AN' from dual union all
select 21294,'AN' from dual union all
select 21295,'AN' from dual union all
select 21296,'AN' from dual union all
select 21297,'AN' from dual union all
select 21298,'MRN' from dual;
insert into LAUNCH_AGENCY
select 21299,'AN' from dual union all
select 213,'WER836-1' from dual union all
select 2130,'ORDRSA' from dual union all
select 21300,'AN' from dual union all
select 21301,'AN' from dual union all
select 21302,'AN' from dual union all
select 21303,'AN' from dual union all
select 21304,'MRN' from dual union all
select 21305,'AN' from dual union all
select 21306,'AN' from dual union all
select 21307,'AN' from dual union all
select 21308,'AN' from dual union all
select 21309,'AN' from dual union all
select 2131,'USA' from dual union all
select 21310,'AN' from dual union all
select 21311,'AN' from dual union all
select 21312,'AN' from dual union all
select 21313,'AN' from dual union all
select 21314,'AN' from dual union all
select 21315,'AN' from dual union all
select 21316,'AN' from dual union all
select 21317,'AN' from dual union all
select 21318,'AN' from dual union all
select 21319,'AN' from dual union all
select 2132,'USA' from dual union all
select 21320,'AN' from dual union all
select 21321,'AN' from dual union all
select 21322,'AN' from dual union all
select 21323,'AN' from dual union all
select 21324,'AN' from dual union all
select 21325,'AN' from dual union all
select 21326,'AN' from dual union all
select 21327,'AN' from dual union all
select 21328,'AN' from dual union all
select 21329,'AN' from dual union all
select 2133,'USA' from dual union all
select 21330,'AN' from dual union all
select 21331,'AN' from dual union all
select 21332,'AN' from dual union all
select 21333,'AN' from dual union all
select 21334,'AN' from dual union all
select 21335,'AN' from dual union all
select 21336,'AN' from dual union all
select 21337,'AN' from dual union all
select 21338,'AN' from dual union all
select 21339,'AN' from dual union all
select 2134,'USA' from dual union all
select 21340,'AN' from dual union all
select 21341,'AN' from dual union all
select 21342,'AN' from dual union all
select 21343,'AN' from dual union all
select 21344,'MRN' from dual union all
select 21345,'AN' from dual union all
select 21346,'AN' from dual union all
select 21347,'AN' from dual union all
select 21348,'AN' from dual union all
select 21349,'AN' from dual union all
select 2135,'USA' from dual union all
select 21350,'NASA' from dual union all
select 21351,'AN' from dual union all
select 21352,'USA' from dual union all
select 21353,'AN' from dual union all
select 21354,'AN' from dual union all
select 21355,'AN' from dual union all
select 21356,'AN' from dual union all
select 21357,'AN' from dual union all
select 21358,'AN' from dual union all
select 21359,'AN' from dual union all
select 2136,'USA' from dual union all
select 21360,'AN' from dual union all
select 21361,'AN' from dual union all
select 21362,'AN' from dual union all
select 21363,'AN' from dual union all
select 21364,'AN' from dual union all
select 21365,'AN' from dual union all
select 21366,'MRN' from dual union all
select 21367,'AN' from dual union all
select 21368,'AN' from dual union all
select 21369,'AN' from dual union all
select 2137,'ORDRSA' from dual union all
select 21370,'AN' from dual union all
select 21371,'AN' from dual union all
select 21372,'AN' from dual union all
select 21373,'AN' from dual union all
select 21374,'AN' from dual union all
select 21375,'AN' from dual union all
select 21376,'AN' from dual union all
select 21377,'AN' from dual union all
select 21378,'AN' from dual union all
select 21379,'MRN' from dual union all
select 2138,'ORDRSA' from dual union all
select 21380,'NASA' from dual union all
select 21381,'AN' from dual union all
select 21382,'AN' from dual union all
select 21383,'AN' from dual union all
select 21384,'AN' from dual union all
select 21385,'USA' from dual union all
select 21386,'AN' from dual union all
select 21387,'AN' from dual union all
select 21388,'MRN' from dual;
insert into LAUNCH_AGENCY
select 21389,'MRN' from dual union all
select 2139,'USA' from dual union all
select 21390,'AN' from dual union all
select 21391,'AN' from dual union all
select 21392,'AN' from dual union all
select 21393,'AN' from dual union all
select 21394,'AN' from dual union all
select 21395,'AN' from dual union all
select 21396,'AN' from dual union all
select 21397,'AN' from dual union all
select 21398,'SRC' from dual union all
select 21399,'AN' from dual union all
select 214,'WER836-1' from dual union all
select 2140,'ORDRSA' from dual union all
select 21400,'AN' from dual union all
select 21401,'MRN' from dual union all
select 21402,'AN' from dual union all
select 21403,'AN' from dual union all
select 21404,'AN' from dual union all
select 21405,'AN' from dual union all
select 21406,'AN' from dual union all
select 21407,'AN' from dual union all
select 21408,'AN' from dual union all
select 21409,'AN' from dual union all
select 2141,'ORDRSA' from dual union all
select 21410,'NASA' from dual union all
select 21410,'RTG' from dual union all
select 21411,'AN' from dual union all
select 21412,'MRN' from dual union all
select 21413,'USAF' from dual union all
select 21414,'AN' from dual union all
select 21415,'USAF' from dual union all
select 21416,'AN' from dual union all
select 21417,'AN' from dual union all
select 21418,'USAF' from dual union all
select 21419,'AN' from dual union all
select 2142,'USAF' from dual union all
select 21420,'AN' from dual union all
select 21421,'AN' from dual union all
select 21422,'AN' from dual union all
select 21423,'USAF' from dual union all
select 21424,'AN' from dual union all
select 21425,'AN' from dual union all
select 21426,'AN' from dual union all
select 21427,'USAF' from dual union all
select 21428,'USA' from dual union all
select 21429,'AN' from dual union all
select 2143,'USAF' from dual union all
select 21430,'AN' from dual union all
select 21431,'AN' from dual union all
select 21432,'AN' from dual union all
select 21433,'AN' from dual union all
select 21434,'AN' from dual union all
select 21435,'AN' from dual union all
select 21436,'USAF' from dual union all
select 21437,'AN' from dual union all
select 21438,'NASA' from dual union all
select 21439,'AN' from dual union all
select 2144,'USAF' from dual union all
select 21440,'USA' from dual union all
select 21441,'AN' from dual union all
select 21442,'AN' from dual union all
select 21443,'AN' from dual union all
select 21444,'AN' from dual union all
select 21445,'AN' from dual union all
select 21446,'AN' from dual union all
select 21447,'AN' from dual union all
select 21448,'AN' from dual union all
select 21449,'AN' from dual union all
select 2145,'USAF' from dual union all
select 21450,'AN' from dual union all
select 21451,'AN' from dual union all
select 21452,'AN' from dual union all
select 21453,'AN' from dual union all
select 21454,'AN' from dual union all
select 21455,'AN' from dual union all
select 21456,'AN' from dual union all
select 21457,'AN' from dual union all
select 21458,'AN' from dual union all
select 21459,'AN' from dual union all
select 2146,'USA' from dual union all
select 21460,'AN' from dual union all
select 21461,'AN' from dual union all
select 21462,'AN' from dual union all
select 21463,'AN' from dual union all
select 21464,'AN' from dual union all
select 21465,'AN' from dual union all
select 21466,'AN' from dual union all
select 21467,'AN' from dual union all
select 21468,'AN' from dual union all
select 21469,'AN' from dual union all
select 2147,'USAF' from dual union all
select 21470,'AN' from dual union all
select 21471,'AN' from dual union all
select 21472,'AN' from dual union all
select 21473,'AN' from dual union all
select 21474,'AN' from dual union all
select 21475,'AN' from dual union all
select 21476,'AN' from dual union all
select 21477,'AN' from dual;
insert into LAUNCH_AGENCY
select 21478,'RVSN' from dual union all
select 21479,'AFSC' from dual union all
select 2148,'USA' from dual union all
select 21480,'NASA' from dual union all
select 21481,'RVSN' from dual union all
select 21482,'RVSN' from dual union all
select 21483,'RVSN' from dual union all
select 21484,'RVSN' from dual union all
select 21485,'NASA' from dual union all
select 21486,'USAF' from dual union all
select 21487,'RVSN' from dual union all
select 21488,'ISAS' from dual union all
select 21489,'10ADS' from dual union all
select 2149,'USA' from dual union all
select 21490,'RVSN' from dual union all
select 21491,'RVSN' from dual union all
select 21492,'RVSN' from dual union all
select 21493,'USAF' from dual union all
select 21494,'CNES' from dual union all
select 21495,'RVSN' from dual union all
select 21496,'RVSN' from dual union all
select 21497,'RVSN' from dual union all
select 21498,'NASA' from dual union all
select 21499,'RVSN' from dual union all
select 215,'WER836-1' from dual union all
select 2150,'NRL' from dual union all
select 21500,'RVSN' from dual union all
select 21501,'RVSN' from dual union all
select 21502,'USAF' from dual union all
select 21503,'RVSN' from dual union all
select 21504,'USAF' from dual union all
select 21505,'RVSN' from dual union all
select 21506,'NASA' from dual union all
select 21507,'RVSN' from dual union all
select 21508,'AFSC' from dual union all
select 21509,'NASA' from dual union all
select 2151,'USA' from dual union all
select 21510,'RVSN' from dual union all
select 21511,'CALT' from dual union all
select 21512,'RVSN' from dual union all
select 21513,'RVSN' from dual union all
select 21514,'RVSN' from dual union all
select 21515,'RVSN' from dual union all
select 21516,'RVSN' from dual union all
select 21517,'USAF' from dual union all
select 21518,'RVSN' from dual union all
select 21519,'RVSN' from dual union all
select 2152,'USA' from dual union all
select 21520,'RVSN' from dual union all
select 21521,'RVSN' from dual union all
select 21522,'RVSN' from dual union all
select 21523,'USAF' from dual union all
select 21524,'RVSN' from dual union all
select 21525,'AFSC' from dual union all
select 21526,'RVSN' from dual union all
select 21527,'RVSN' from dual union all
select 21528,'RVSN' from dual union all
select 21529,'RVSN' from dual union all
select 2153,'USAF' from dual union all
select 21530,'RVSN' from dual union all
select 21531,'USAF' from dual union all
select 21532,'NASA' from dual union all
select 21533,'RVSN' from dual union all
select 21534,'RVSN' from dual union all
select 21535,'RVSN' from dual union all
select 21536,'RVSN' from dual union all
select 21537,'RVSN' from dual union all
select 21538,'AFSC' from dual union all
select 21539,'NASA' from dual union all
select 2154,'USAF' from dual union all
select 21540,'RVSN' from dual union all
select 21541,'RVSN' from dual union all
select 21542,'RVSN' from dual union all
select 21543,'USAF' from dual union all
select 21544,'AFSC' from dual union all
select 21545,'RVSN' from dual union all
select 21546,'USAF' from dual union all
select 21547,'10ADS' from dual union all
select 21548,'RVSN' from dual union all
select 21549,'RVSN' from dual union all
select 2155,'USA' from dual union all
select 21550,'RVSN' from dual union all
select 21551,'RVSN' from dual union all
select 21552,'RVSN' from dual union all
select 21553,'RVSN' from dual union all
select 21554,'RVSN' from dual union all
select 21555,'RVSN' from dual union all
select 21556,'RVSN' from dual union all
select 21557,'RVSN' from dual union all
select 21558,'RVSN' from dual union all
select 21559,'RVSN' from dual union all
select 2156,'USA' from dual union all
select 21560,'RVSN' from dual union all
select 21561,'RVSN' from dual union all
select 21562,'RVSN' from dual union all
select 21563,'RVSN' from dual union all
select 21564,'RVSN' from dual union all
select 21565,'RVSN' from dual union all
select 21566,'RVSN' from dual union all
select 21567,'AFSC' from dual;
insert into LAUNCH_AGENCY
select 21568,'RVSN' from dual union all
select 21569,'RVSN' from dual union all
select 2157,'USA' from dual union all
select 21570,'USAF' from dual union all
select 21571,'NASA' from dual union all
select 21572,'RVSN' from dual union all
select 21573,'RVSN' from dual union all
select 21574,'RVSN' from dual union all
select 21575,'USAF' from dual union all
select 21576,'RVSN' from dual union all
select 21577,'RVSN' from dual union all
select 21578,'RVSN' from dual union all
select 21579,'RVSN' from dual union all
select 2158,'USA' from dual union all
select 21580,'RVSN' from dual union all
select 21581,'RVSN' from dual union all
select 21582,'RVSN' from dual union all
select 21583,'NASA' from dual union all
select 21584,'CRA' from dual union all
select 21585,'RVSN' from dual union all
select 21586,'CNES' from dual union all
select 21587,'RVSN' from dual union all
select 21588,'RVSN' from dual union all
select 21589,'RVSN' from dual union all
select 2159,'USA' from dual union all
select 21590,'RVSN' from dual union all
select 21591,'RVSN' from dual union all
select 21592,'WRE' from dual union all
select 21593,'MRN' from dual union all
select 21594,'MRN' from dual union all
select 21595,'MRN' from dual union all
select 21596,'MRN' from dual union all
select 21597,'MRN' from dual union all
select 21598,'NASA' from dual union all
select 21599,'MRN' from dual union all
select 216,'WER836-1' from dual union all
select 2160,'USA' from dual union all
select 21600,'USN' from dual union all
select 21601,'NASA' from dual union all
select 21602,'MRN' from dual union all
select 21603,'MRN' from dual union all
select 21604,'MRN' from dual union all
select 21605,'USN' from dual union all
select 21606,'USN' from dual union all
select 21607,'MRN' from dual union all
select 21608,'MRN' from dual union all
select 21609,'MRN' from dual union all
select 2161,'USA' from dual union all
select 21610,'MRN' from dual union all
select 21611,'MRN' from dual union all
select 21612,'MRN' from dual union all
select 21613,'RVSN' from dual union all
select 21614,'FKGR21' from dual union all
select 21615,'INTA' from dual union all
select 21616,'MRN' from dual union all
select 21617,'USA' from dual union all
select 21618,'MRN' from dual union all
select 21619,'MRN' from dual union all
select 2162,'USAF' from dual union all
select 21620,'US7A' from dual union all
select 21621,'MRN' from dual union all
select 21622,'MRN' from dual union all
select 21623,'MRN' from dual union all
select 21624,'MRN' from dual union all
select 21625,'MRN' from dual union all
select 21626,'MRN' from dual union all
select 21627,'MRN' from dual union all
select 21628,'SRC' from dual union all
select 21629,'USA' from dual union all
select 2163,'USA' from dual union all
select 21630,'MRN' from dual union all
select 21631,'MRN' from dual union all
select 21632,'MRN' from dual union all
select 21633,'MRN' from dual union all
select 21634,'MRN' from dual union all
select 21635,'BAL' from dual union all
select 21636,'MRN' from dual union all
select 21637,'BAL' from dual union all
select 21638,'BAL' from dual union all
select 21639,'MRN' from dual union all
select 2164,'USA' from dual union all
select 21640,'BAL' from dual union all
select 21641,'MRN' from dual union all
select 21642,'MRN' from dual union all
select 21643,'MRN' from dual union all
select 21644,'MRN' from dual union all
select 21645,'MRN' from dual union all
select 21646,'USA' from dual union all
select 21647,'RVSN' from dual union all
select 21648,'JMA' from dual union all
select 21649,'JMA' from dual union all
select 2165,'USA' from dual union all
select 21650,'JMA' from dual union all
select 21651,'JMA' from dual union all
select 21652,'JMA' from dual union all
select 21653,'JMA' from dual union all
select 21654,'JMA' from dual union all
select 21655,'ISRO' from dual union all
select 21656,'MRN' from dual union all
select 21657,'ISRO' from dual;
insert into LAUNCH_AGENCY
select 21658,'IIAE' from dual union all
select 21659,'NASA' from dual union all
select 2166,'USA' from dual union all
select 21660,'NASA' from dual union all
select 21661,'NASA' from dual union all
select 21662,'NASA' from dual union all
select 21663,'NASA' from dual union all
select 21664,'USA' from dual union all
select 21665,'JDA' from dual union all
select 21666,'JDA' from dual union all
select 21667,'MRN' from dual union all
select 21668,'JDA' from dual union all
select 21669,'JDA' from dual union all
select 2167,'USA' from dual union all
select 21670,'JDA' from dual union all
select 21671,'JDA' from dual union all
select 21672,'JDA' from dual union all
select 21673,'JDA' from dual union all
select 21674,'JDA' from dual union all
select 21675,'WRE' from dual union all
select 21676,'WRE' from dual union all
select 21677,'SAND' from dual union all
select 21678,'MRN' from dual union all
select 21679,'SAND' from dual union all
select 2168,'ORDRSA' from dual union all
select 21680,'WSMR' from dual union all
select 21681,'RVSN' from dual union all
select 21682,'SAND' from dual union all
select 21683,'RVSN' from dual union all
select 21684,'RVSN' from dual union all
select 21685,'RVSN' from dual union all
select 21686,'USA' from dual union all
select 21687,'MRN' from dual union all
select 21688,'MRN' from dual union all
select 21689,'RAE' from dual union all
select 2169,'USA' from dual union all
select 21690,'MRN' from dual union all
select 21691,'MRN' from dual union all
select 21692,'MRN' from dual union all
select 21693,'MRN' from dual union all
select 21694,'USN' from dual union all
select 21695,'USN' from dual union all
select 21696,'MRN' from dual union all
select 21697,'MRN' from dual union all
select 21698,'MRN' from dual union all
select 21699,'AFSC' from dual union all
select 217,'WER836-1' from dual union all
select 2170,'USA' from dual union all
select 21700,'AFSC' from dual union all
select 21701,'WRE' from dual union all
select 21702,'MRN' from dual union all
select 21703,'USA' from dual union all
select 21704,'MRN' from dual union all
select 21705,'MRN' from dual union all
select 21706,'DFVLR' from dual union all
select 21707,'MRN' from dual union all
select 21708,'MRN' from dual union all
select 21709,'MRN' from dual union all
select 2171,'USA' from dual union all
select 21710,'MRN' from dual union all
select 21711,'MRN' from dual union all
select 21712,'MRN' from dual union all
select 21713,'MRN' from dual union all
select 21714,'MRN' from dual union all
select 21715,'MRN' from dual union all
select 21716,'RAE' from dual union all
select 21717,'MRN' from dual union all
select 21718,'MRN' from dual union all
select 21719,'MRN' from dual union all
select 2172,'PARD' from dual union all
select 21720,'MRN' from dual union all
select 21721,'MRN' from dual union all
select 21722,'MRN' from dual union all
select 21723,'MRN' from dual union all
select 21724,'USN' from dual union all
select 21725,'MRN' from dual union all
select 21726,'MRN' from dual union all
select 21727,'US7A' from dual union all
select 21728,'INTA' from dual union all
select 21729,'MRN' from dual union all
select 2173,'ORDRSA' from dual union all
select 21730,'MRN' from dual union all
select 21731,'MRN' from dual union all
select 21732,'MRN' from dual union all
select 21733,'MRN' from dual union all
select 21734,'MRN' from dual union all
select 21735,'MRN' from dual union all
select 21736,'USA' from dual union all
select 21737,'MRN' from dual union all
select 21738,'MRN' from dual union all
select 21739,'USA' from dual union all
select 2174,'ORDRSA' from dual union all
select 21740,'MRN' from dual union all
select 21741,'MRN' from dual union all
select 21742,'NASA' from dual union all
select 21743,'ESRO' from dual union all
select 21744,'MRN' from dual union all
select 21745,'MRN' from dual union all
select 21746,'MRN' from dual union all
select 21747,'USA' from dual;
insert into LAUNCH_AGENCY
select 21748,'MRN' from dual union all
select 21749,'MRN' from dual union all
select 2175,'PARD' from dual union all
select 21750,'MRN' from dual union all
select 21751,'MRN' from dual union all
select 21752,'MRN' from dual union all
select 21753,'USN' from dual union all
select 21754,'MRN' from dual union all
select 21755,'MRN' from dual union all
select 21756,'MRN' from dual union all
select 21757,'MRN' from dual union all
select 21758,'MRN' from dual union all
select 21759,'MRN' from dual union all
select 2176,'USA' from dual union all
select 21760,'USN' from dual union all
select 21761,'USA' from dual union all
select 21762,'MRN' from dual union all
select 21763,'MRN' from dual union all
select 21764,'MRN' from dual union all
select 21765,'MRN' from dual union all
select 21766,'MRN' from dual union all
select 21767,'MRN' from dual union all
select 21768,'STA' from dual union all
select 21769,'RVSN' from dual union all
select 2177,'USA' from dual union all
select 21770,'RVSN' from dual union all
select 21771,'RVSN' from dual union all
select 21772,'RVSN' from dual union all
select 21773,'RVSN' from dual union all
select 21774,'RAE' from dual union all
select 21775,'ISAS' from dual union all
select 21776,'RVSN' from dual union all
select 21777,'INTA' from dual union all
select 21778,'ISAS' from dual union all
select 21779,'NASA' from dual union all
select 2178,'USAF' from dual union all
select 21780,'CNES' from dual union all
select 21781,'NASA' from dual union all
select 21782,'AFCRL' from dual union all
select 21783,'AFCRL' from dual union all
select 21784,'NASA' from dual union all
select 21785,'UKMO' from dual union all
select 21786,'WRE' from dual union all
select 21787,'UKMO' from dual union all
select 21788,'UKMO' from dual union all
select 21789,'UKMO' from dual union all
select 2179,'ORDRSA' from dual union all
select 21790,'UKMO' from dual union all
select 21791,'UKMO' from dual union all
select 21792,'UKMO' from dual union all
select 21793,'NASA' from dual union all
select 21794,'UKMO' from dual union all
select 21795,'UKMO' from dual union all
select 21796,'AFCRL' from dual union all
select 21797,'NRCC' from dual union all
select 21798,'USA' from dual union all
select 21799,'USA' from dual union all
select 218,'WER836-1' from dual union all
select 2180,'USAF' from dual union all
select 21800,'AFCRL' from dual union all
select 21801,'PIHM' from dual union all
select 21802,'CNES' from dual union all
select 21803,'NSF' from dual union all
select 21804,'USA' from dual union all
select 21805,'NSF' from dual union all
select 21806,'INTA' from dual union all
select 21807,'MSFC' from dual union all
select 21808,'MSFC' from dual union all
select 21809,'MSFC' from dual union all
select 2181,'USAF' from dual union all
select 21810,'MSFC' from dual union all
select 21811,'MSFC' from dual union all
select 21812,'MSFC' from dual union all
select 21813,'MSFC' from dual union all
select 21814,'BWEHR' from dual union all
select 21815,'BWEHR' from dual union all
select 21816,'DMA' from dual union all
select 21817,'DMA' from dual union all
select 21818,'DMA' from dual union all
select 21819,'DMA' from dual union all
select 2182,'USAF' from dual union all
select 21820,'RVSN' from dual union all
select 21821,'RVSN' from dual union all
select 21822,'RVSN' from dual union all
select 21823,'RVSN' from dual union all
select 21824,'ESRO' from dual union all
select 21825,'NASA' from dual union all
select 21826,'USN' from dual union all
select 21827,'NASA' from dual union all
select 21828,'NASA' from dual union all
select 21829,'USN' from dual union all
select 2183,'USA' from dual union all
select 21830,'ESRO' from dual union all
select 21831,'NASA' from dual union all
select 21832,'USAF' from dual union all
select 21833,'AFSC' from dual union all
select 21834,'RVSN' from dual union all
select 21835,'NASA' from dual union all
select 21836,'NASA' from dual union all
select 21837,'NIPR' from dual;
insert into LAUNCH_AGENCY
select 21838,'BAC' from dual union all
select 21838,'ESRO' from dual union all
select 21839,'AN' from dual union all
select 2184,'USA' from dual union all
select 21840,'AN' from dual union all
select 21841,'DLR' from dual union all
select 21842,'DLR' from dual union all
select 21843,'NASA' from dual union all
select 21844,'RVSN' from dual union all
select 21845,'ESRO' from dual union all
select 21846,'INTA' from dual union all
select 21847,'INTA' from dual union all
select 21848,'NIPR' from dual union all
select 21849,'RVSN' from dual union all
select 2185,'USA' from dual union all
select 21850,'NASA' from dual union all
select 21851,'INTA' from dual union all
select 21852,'RVSN' from dual union all
select 21853,'NASA' from dual union all
select 21854,'INTA' from dual union all
select 21855,'RVSN' from dual union all
select 21856,'RVSN' from dual union all
select 21857,'INTA' from dual union all
select 21858,'INTA' from dual union all
select 21859,'SRC' from dual union all
select 2186,'USAF' from dual union all
select 21860,'INTA' from dual union all
select 21861,'AN' from dual union all
select 21862,'ESRO' from dual union all
select 21863,'INTA' from dual union all
select 21864,'INTA' from dual union all
select 21865,'INTA' from dual union all
select 21866,'NRCC' from dual union all
select 21867,'INTA' from dual union all
select 21868,'INTA' from dual union all
select 21869,'ESRO' from dual union all
select 2187,'USA' from dual union all
select 21870,'INTA' from dual union all
select 21871,'1STRAD' from dual union all
select 21872,'INTA' from dual union all
select 21873,'AN' from dual union all
select 21874,'INTA' from dual union all
select 21875,'INTA' from dual union all
select 21876,'VMF' from dual union all
select 21877,'INTA' from dual union all
select 21878,'INTA' from dual union all
select 21879,'INTA' from dual union all
select 2188,'ORDRSA' from dual union all
select 21880,'ESRO' from dual union all
select 21881,'INTA' from dual union all
select 21882,'INTA' from dual union all
select 21883,'INTA' from dual union all
select 21884,'INTA' from dual union all
select 21885,'ESRO' from dual union all
select 21886,'INTA' from dual union all
select 21887,'RVSN' from dual union all
select 21888,'AN' from dual union all
select 21889,'AN' from dual union all
select 2189,'ORDRSA' from dual union all
select 21890,'INTA' from dual union all
select 21891,'INTA' from dual union all
select 21892,'INTA' from dual union all
select 21893,'INTA' from dual union all
select 21894,'INTA' from dual union all
select 21895,'INTA' from dual union all
select 21896,'INTA' from dual union all
select 21897,'INTA' from dual union all
select 21898,'RVSN' from dual union all
select 21899,'AN' from dual union all
select 219,'WER836-1' from dual union all
select 2190,'USA' from dual union all
select 21900,'AN' from dual union all
select 21901,'RVSN' from dual union all
select 21902,'INTA' from dual union all
select 21903,'NASA' from dual union all
select 21904,'INTA' from dual union all
select 21905,'INTA' from dual union all
select 21906,'INTA' from dual union all
select 21907,'AN' from dual union all
select 21908,'AN' from dual union all
select 21909,'RVSN' from dual union all
select 2191,'USAF' from dual union all
select 21910,'NRL' from dual union all
select 21911,'AN' from dual union all
select 21912,'AN' from dual union all
select 21913,'NASA' from dual union all
select 21914,'NASA' from dual union all
select 21915,'AN' from dual union all
select 21916,'NASA' from dual union all
select 21917,'RAE' from dual union all
select 21918,'1STRAD' from dual union all
select 21919,'NASA' from dual union all
select 2192,'USA' from dual union all
select 21920,'RVSN' from dual union all
select 21921,'NASA' from dual union all
select 21922,'INTA' from dual union all
select 21923,'INTA' from dual union all
select 21924,'NASA' from dual union all
select 21925,'AN' from dual union all
select 21926,'NASA' from dual;
insert into LAUNCH_AGENCY
select 21927,'NASA' from dual union all
select 21928,'DLR' from dual union all
select 21929,'WRE' from dual union all
select 2193,'USA' from dual union all
select 21930,'NASA' from dual union all
select 21931,'RVSN' from dual union all
select 21932,'AN' from dual union all
select 21933,'NASA' from dual union all
select 21934,'AN' from dual union all
select 21935,'NASA' from dual union all
select 21936,'AFCRL' from dual union all
select 21937,'NRCC' from dual union all
select 21938,'NASA' from dual union all
select 21939,'NASA' from dual union all
select 2194,'USA' from dual union all
select 21940,'AFCRL' from dual union all
select 21941,'NASA' from dual union all
select 21942,'RVSN' from dual union all
select 21943,'NASA' from dual union all
select 21944,'NASA' from dual union all
select 21944,'NRL' from dual union all
select 21945,'NASA' from dual union all
select 21946,'NASA' from dual union all
select 21947,'NASA' from dual union all
select 21948,'NASA' from dual union all
select 21949,'NASA' from dual union all
select 2195,'USA' from dual union all
select 21950,'AFCRL' from dual union all
select 21951,'NASA' from dual union all
select 21952,'AFCRL' from dual union all
select 21953,'RVSN' from dual union all
select 21954,'NASA' from dual union all
select 21955,'NASA' from dual union all
select 21956,'NRCC' from dual union all
select 21957,'NRCC' from dual union all
select 21958,'NASA' from dual union all
select 21959,'NRCC' from dual union all
select 2196,'USA' from dual union all
select 21960,'AFCRL' from dual union all
select 21961,'NASA' from dual union all
select 21962,'AFCRL' from dual union all
select 21963,'NASA' from dual union all
select 21964,'RVSN' from dual union all
select 21965,'NASA' from dual union all
select 21966,'NTNF' from dual union all
select 21967,'DLR' from dual union all
select 21968,'RVSN' from dual union all
select 21969,'AFCRL' from dual union all
select 2197,'USA' from dual union all
select 21970,'AFCRL' from dual union all
select 21971,'NASA' from dual union all
select 21972,'NASA' from dual union all
select 21973,'NASA' from dual union all
select 21974,'1STRAD' from dual union all
select 21975,'RVSN' from dual union all
select 21976,'WRE' from dual union all
select 21977,'1STRAD' from dual union all
select 21978,'BAC' from dual union all
select 21979,'SAMSO' from dual union all
select 2198,'USA' from dual union all
select 21980,'AFSC' from dual union all
select 21981,'USAF' from dual union all
select 21982,'NASA' from dual union all
select 21983,'DFVLR' from dual union all
select 21984,'DFVLR' from dual union all
select 21985,'BAC' from dual union all
select 21986,'RVSN' from dual union all
select 21987,'RTG' from dual union all
select 21988,'NASA' from dual union all
select 21989,'RTG' from dual union all
select 2199,'USA' from dual union all
select 21990,'RTG' from dual union all
select 21991,'RTG' from dual union all
select 21992,'NASA' from dual union all
select 21993,'RTG' from dual union all
select 21994,'NASA' from dual union all
select 21995,'BAC' from dual union all
select 21996,'ALCS' from dual union all
select 21996,'SAC' from dual union all
select 21997,'RVSN' from dual union all
select 21998,'NTNF' from dual union all
select 21999,'USN' from dual union all
select 22,'WEHR' from dual union all
select 220,'WEHR' from dual union all
select 2200,'USA' from dual union all
select 22000,'BAC' from dual union all
select 22001,'AFCRL' from dual union all
select 22002,'SAC' from dual union all
select 22003,'AFCRL' from dual union all
select 22004,'NASA' from dual union all
select 22005,'NASA' from dual union all
select 22006,'NASA' from dual union all
select 22007,'SUPA' from dual union all
select 22008,'RVSN' from dual union all
select 22009,'USAF' from dual union all
select 2201,'USA' from dual union all
select 22010,'SUPA' from dual union all
select 22011,'RVSN' from dual union all
select 22012,'RVSN' from dual union all
select 22013,'USAF' from dual;
insert into LAUNCH_AGENCY
select 22014,'SAC' from dual union all
select 22015,'US7A' from dual union all
select 22016,'AFSC' from dual union all
select 22017,'ESRO' from dual union all
select 22018,'ESRO' from dual union all
select 22019,'RVSN' from dual union all
select 2202,'USA' from dual union all
select 22020,'NASA' from dual union all
select 22021,'AFCRL' from dual union all
select 22022,'AFSC' from dual union all
select 22023,'AFCRL' from dual union all
select 22024,'ESRO' from dual union all
select 22025,'BAC' from dual union all
select 22026,'USN' from dual union all
select 22027,'USN' from dual union all
select 22028,'USN' from dual union all
select 22029,'USN' from dual union all
select 2203,'USA' from dual union all
select 22030,'RVSN' from dual union all
select 22031,'USN' from dual union all
select 22032,'USN' from dual union all
select 22033,'USN' from dual union all
select 22034,'USN' from dual union all
select 22035,'RVSN' from dual union all
select 22036,'NASA' from dual union all
select 22037,'RVSN' from dual union all
select 22038,'NASA' from dual union all
select 22039,'USA' from dual union all
select 2204,'USA' from dual union all
select 22040,'NASA' from dual union all
select 22041,'RVSN' from dual union all
select 22042,'RVSN' from dual union all
select 22043,'FKGR12' from dual union all
select 22044,'FKGR12' from dual union all
select 22045,'WRE' from dual union all
select 22046,'NASA' from dual union all
select 22047,'AN' from dual union all
select 22048,'AN' from dual union all
select 22049,'BAC' from dual union all
select 2205,'USA' from dual union all
select 22050,'WRE' from dual union all
select 22051,'AN' from dual union all
select 22052,'RVSN' from dual union all
select 22053,'DMA' from dual union all
select 22054,'RVSN' from dual union all
select 22055,'SAC' from dual union all
select 22056,'USN' from dual union all
select 22057,'USN' from dual union all
select 22058,'USN' from dual union all
select 22059,'USN' from dual union all
select 2206,'PARD' from dual union all
select 22060,'USN' from dual union all
select 22061,'USN' from dual union all
select 22062,'USN' from dual union all
select 22063,'RVSN' from dual union all
select 22064,'AN' from dual union all
select 22065,'SRC' from dual union all
select 22066,'AN' from dual union all
select 22067,'AFSC' from dual union all
select 22068,'USAF' from dual union all
select 22069,'AN' from dual union all
select 2207,'ORDRSA' from dual union all
select 22070,'AN' from dual union all
select 22071,'AN' from dual union all
select 22072,'USAF' from dual union all
select 22073,'NRCC' from dual union all
select 22074,'RVSN' from dual union all
select 22075,'AN' from dual union all
select 22076,'KPNO' from dual union all
select 22077,'SAC' from dual union all
select 22078,'AN' from dual union all
select 22079,'RVSN' from dual union all
select 2208,'ORDRSA' from dual union all
select 22080,'AN' from dual union all
select 22081,'NASA' from dual union all
select 22082,'AFSC' from dual union all
select 22083,'SAMSO' from dual union all
select 22084,'SAMSO' from dual union all
select 22085,'RVSN' from dual union all
select 22086,'NASA' from dual union all
select 22087,'CNIE' from dual union all
select 22088,'SAC' from dual union all
select 22089,'SAND' from dual union all
select 2209,'PARD' from dual union all
select 22090,'CNES' from dual union all
select 22091,'AFCRL' from dual union all
select 22092,'AFCRL' from dual union all
select 22093,'AFSC' from dual union all
select 22094,'NASA' from dual union all
select 22095,'USN' from dual union all
select 22096,'RVSN' from dual union all
select 22097,'USN' from dual union all
select 22098,'USN' from dual union all
select 22099,'USN' from dual union all
select 221,'WER836-1' from dual union all
select 2210,'USA' from dual union all
select 22100,'USN' from dual union all
select 22101,'USN' from dual union all
select 22102,'NASA' from dual union all
select 22103,'SAND' from dual;
insert into LAUNCH_AGENCY
select 22104,'USN' from dual union all
select 22105,'AN' from dual union all
select 22106,'SAND' from dual union all
select 22107,'RVSN' from dual union all
select 22108,'ALCS' from dual union all
select 22108,'SAC' from dual union all
select 22109,'AFCRL' from dual union all
select 2211,'USA' from dual union all
select 22110,'WRE' from dual union all
select 22111,'WRE' from dual union all
select 22112,'AFCRL' from dual union all
select 22113,'AFCRL' from dual union all
select 22114,'USN' from dual union all
select 22115,'AFCRL' from dual union all
select 22116,'NRL' from dual union all
select 22117,'NASA' from dual union all
select 22118,'RVSN' from dual union all
select 22119,'WRE' from dual union all
select 2212,'USA' from dual union all
select 22120,'ARPA' from dual union all
select 22121,'WRE' from dual union all
select 22122,'SAC' from dual union all
select 22123,'AFSC' from dual union all
select 22124,'NASA' from dual union all
select 22125,'AFSC' from dual union all
select 22126,'FKGR13' from dual union all
select 22127,'FKGR13' from dual union all
select 22128,'RVSN' from dual union all
select 22129,'US7A' from dual union all
select 2213,'ORDRSA' from dual union all
select 22130,'US7A' from dual union all
select 22131,'SAND' from dual union all
select 22132,'RN' from dual union all
select 22133,'NASA' from dual union all
select 22134,'NASA' from dual union all
select 22135,'NASA' from dual union all
select 22136,'ARPA' from dual union all
select 22137,'ESRO' from dual union all
select 22138,'USAF' from dual union all
select 22139,'RVSN' from dual union all
select 2214,'ORDRSA' from dual union all
select 22140,'USAF' from dual union all
select 22141,'ALCS' from dual union all
select 22141,'SAC' from dual union all
select 22142,'AFSC' from dual union all
select 22143,'DMA' from dual union all
select 22144,'KPNO' from dual union all
select 22145,'AFSC' from dual union all
select 22146,'USN' from dual union all
select 22147,'RN' from dual union all
select 22148,'SAC' from dual union all
select 22149,'SAC' from dual union all
select 2215,'USAF' from dual union all
select 22150,'RVSN' from dual union all
select 22151,'NASA' from dual union all
select 22152,'NASA' from dual union all
select 22153,'AFSC' from dual union all
select 22154,'USA' from dual union all
select 22155,'SAC' from dual union all
select 22156,'NASA' from dual union all
select 22157,'SAC' from dual union all
select 22158,'AN' from dual union all
select 22159,'NASA' from dual union all
select 2216,'USAF' from dual union all
select 22160,'NASA' from dual union all
select 22161,'RVSN' from dual union all
select 22162,'SRC' from dual union all
select 22163,'USN' from dual union all
select 22164,'RVSN' from dual union all
select 22165,'AN' from dual union all
select 22166,'NASA' from dual union all
select 22167,'AN' from dual union all
select 22168,'ESRO' from dual union all
select 22169,'AN' from dual union all
select 2217,'USAF' from dual union all
select 22170,'AN' from dual union all
select 22171,'AN' from dual union all
select 22172,'RVSN' from dual union all
select 22173,'AN' from dual union all
select 22174,'ESRO' from dual union all
select 22175,'AN' from dual union all
select 22176,'ALCS' from dual union all
select 22176,'SAC' from dual union all
select 22177,'AN' from dual union all
select 22178,'BAC' from dual union all
select 22179,'PIHM' from dual union all
select 2218,'USAF' from dual union all
select 22180,'USAF' from dual union all
select 22181,'SAC' from dual union all
select 22182,'WRE' from dual union all
select 22183,'RVSN' from dual union all
select 22184,'BAC' from dual union all
select 22185,'AN' from dual union all
select 22186,'US7A' from dual union all
select 22187,'US7A' from dual union all
select 22188,'BAC' from dual union all
select 22189,'NASA' from dual union all
select 2219,'USA' from dual union all
select 22190,'SAC' from dual union all
select 22191,'PIHM' from dual;
insert into LAUNCH_AGENCY
select 22192,'AFSC' from dual union all
select 22193,'AN' from dual union all
select 22194,'RVSN' from dual union all
select 22195,'ESRO' from dual union all
select 22196,'ESRO' from dual union all
select 22197,'AN' from dual union all
select 22198,'RVSN' from dual union all
select 22199,'AN' from dual union all
select 222,'WER836-1' from dual union all
select 2220,'USA' from dual union all
select 22200,'AN' from dual union all
select 22201,'AN' from dual union all
select 22202,'AN' from dual union all
select 22203,'RVSN' from dual union all
select 22204,'ESRO' from dual union all
select 22205,'RVSN' from dual union all
select 22206,'NASA' from dual union all
select 22207,'AFCRL' from dual union all
select 22208,'AFSC' from dual union all
select 22209,'RVSN' from dual union all
select 2221,'USA' from dual union all
select 22210,'AN' from dual union all
select 22211,'NASA' from dual union all
select 22212,'AN' from dual union all
select 22213,'SAMSO' from dual union all
select 22214,'RVSN' from dual union all
select 22215,'RVSN' from dual union all
select 22216,'RVSN' from dual union all
select 22217,'ALCS' from dual union all
select 22217,'SAC' from dual union all
select 22218,'USN' from dual union all
select 22219,'NASA' from dual union all
select 2222,'USA' from dual union all
select 22220,'ALCS' from dual union all
select 22220,'SAC' from dual union all
select 22221,'AN' from dual union all
select 22222,'NTNF' from dual union all
select 22223,'AN' from dual union all
select 22224,'US7A' from dual union all
select 22225,'US7A' from dual union all
select 22226,'AN' from dual union all
select 22227,'RVSN' from dual union all
select 22228,'AN' from dual union all
select 22229,'RVSN' from dual union all
select 2223,'USA' from dual union all
select 22230,'RTG' from dual union all
select 22231,'RTG' from dual union all
select 22232,'RTG' from dual union all
select 22233,'RTG' from dual union all
select 22234,'ESRO' from dual union all
select 22235,'ESRO' from dual union all
select 22236,'AN' from dual union all
select 22237,'AN' from dual union all
select 22238,'RVSN' from dual union all
select 22239,'SAC' from dual union all
select 2224,'USA' from dual union all
select 22240,'USAF' from dual union all
select 22241,'AFCRL' from dual union all
select 22242,'NASA' from dual union all
select 22243,'NASA' from dual union all
select 22244,'SAC' from dual union all
select 22245,'NRCC' from dual union all
select 22246,'USN' from dual union all
select 22247,'RVSN' from dual union all
select 22248,'ESRO' from dual union all
select 22249,'NASA' from dual union all
select 2225,'USA' from dual union all
select 22250,'SAC' from dual union all
select 22251,'NASA' from dual union all
select 22252,'ALCS' from dual union all
select 22252,'SAC' from dual union all
select 22253,'SAC' from dual union all
select 22254,'AFSC' from dual union all
select 22255,'SAMSO' from dual union all
select 22256,'USA' from dual union all
select 22257,'NASA' from dual union all
select 22258,'USN' from dual union all
select 22259,'USN' from dual union all
select 2226,'ORDRSA' from dual union all
select 22260,'NASA' from dual union all
select 22261,'USN' from dual union all
select 22262,'NASA' from dual union all
select 22263,'SAC' from dual union all
select 22264,'ISAS' from dual union all
select 22265,'SRC' from dual union all
select 22266,'USN' from dual union all
select 22267,'SRC' from dual union all
select 22268,'USN' from dual union all
select 22269,'USN' from dual union all
select 2227,'ORDRSA' from dual union all
select 22270,'USN' from dual union all
select 22271,'NASA' from dual union all
select 22272,'USN' from dual union all
select 22273,'USN' from dual union all
select 22274,'USN' from dual union all
select 22275,'USN' from dual union all
select 22276,'NASDA' from dual union all
select 22277,'SRC' from dual union all
select 22278,'RVSN' from dual union all
select 22279,'ISAS' from dual;
insert into LAUNCH_AGENCY
select 2228,'USA' from dual union all
select 22280,'SAC' from dual union all
select 22281,'44FA2' from dual union all
select 22282,'NASA' from dual union all
select 22283,'AFSC' from dual union all
select 22284,'ALCS' from dual union all
select 22284,'SAC' from dual union all
select 22285,'NASA' from dual union all
select 22286,'NASA' from dual union all
select 22287,'44FA2' from dual union all
select 22288,'SRC' from dual union all
select 22289,'ISAS' from dual union all
select 2229,'USA' from dual union all
select 22290,'USN' from dual union all
select 22291,'NASA' from dual union all
select 22292,'NASA' from dual union all
select 22293,'NASA' from dual union all
select 22294,'NASA' from dual union all
select 22295,'DMA' from dual union all
select 22296,'USAF' from dual union all
select 22297,'ALCS' from dual union all
select 22297,'SAC' from dual union all
select 22298,'SAC' from dual union all
select 22299,'NASA' from dual union all
select 223,'WER836-1' from dual union all
select 2230,'USA' from dual union all
select 22300,'ISAS' from dual union all
select 22301,'ISAS' from dual union all
select 22302,'NASA' from dual union all
select 22303,'NASA' from dual union all
select 22304,'INTA' from dual union all
select 22305,'US7A' from dual union all
select 22306,'US7A' from dual union all
select 22307,'US7A' from dual union all
select 22308,'US7A' from dual union all
select 22309,'SAND' from dual union all
select 2231,'USA' from dual union all
select 22310,'NASA' from dual union all
select 22311,'DLR' from dual union all
select 22312,'AFSC' from dual union all
select 22313,'ALCS' from dual union all
select 22313,'SAC' from dual union all
select 22314,'USA' from dual union all
select 22315,'SAND' from dual union all
select 22316,'AN' from dual union all
select 22317,'SAC' from dual union all
select 22318,'NASA' from dual union all
select 22319,'CNES' from dual union all
select 2232,'USA' from dual union all
select 22320,'DLR' from dual union all
select 22321,'USN' from dual union all
select 22322,'SAC' from dual union all
select 22323,'NASA' from dual union all
select 22324,'PIHM' from dual union all
select 22325,'RVSN' from dual union all
select 22326,'NASA' from dual union all
select 22327,'BAC' from dual union all
select 22328,'DLR' from dual union all
select 22329,'DLR' from dual union all
select 2233,'USA' from dual union all
select 22330,'NASA' from dual union all
select 22331,'DLR' from dual union all
select 22332,'DFVLR' from dual union all
select 22333,'FKGR21' from dual union all
select 22334,'BAC' from dual union all
select 22335,'NASA' from dual union all
select 22336,'BAC' from dual union all
select 22336,'ESRO' from dual union all
select 22337,'NASA' from dual union all
select 22338,'BAC' from dual union all
select 22338,'ESRO' from dual union all
select 22339,'AN' from dual union all
select 2234,'USA' from dual union all
select 22340,'WRE' from dual union all
select 22341,'AN' from dual union all
select 22342,'AN' from dual union all
select 22343,'AN' from dual union all
select 22344,'SAC' from dual union all
select 22345,'SAC' from dual union all
select 22346,'CNES' from dual union all
select 22347,'DLR' from dual union all
select 22348,'NASA' from dual union all
select 22349,'RVSN' from dual union all
select 2235,'USA' from dual union all
select 22350,'USN' from dual union all
select 22351,'SAC' from dual union all
select 22352,'ESRO' from dual union all
select 22353,'NASA' from dual union all
select 22354,'RVSN' from dual union all
select 22355,'ESRO' from dual union all
select 22356,'RVSN' from dual union all
select 22357,'RVSN' from dual union all
select 22358,'NASA' from dual union all
select 22359,'NASA' from dual union all
select 2236,'USA' from dual union all
select 22360,'SAC' from dual union all
select 22361,'AFSC' from dual union all
select 22362,'ALCS' from dual union all
select 22362,'SAC' from dual union all
select 22363,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 22364,'US7A' from dual union all
select 22365,'US7A' from dual union all
select 22366,'US7A' from dual union all
select 22367,'WRE' from dual union all
select 22368,'SAC' from dual union all
select 22369,'DMA' from dual union all
select 2237,'USA' from dual union all
select 22370,'NASA' from dual union all
select 22371,'NASA' from dual union all
select 22372,'BAC' from dual union all
select 22373,'WRE' from dual union all
select 22374,'SRC' from dual union all
select 22375,'SAC' from dual union all
select 22376,'NRCC' from dual union all
select 22377,'NASA' from dual union all
select 22378,'SRC' from dual union all
select 22379,'NASA' from dual union all
select 2238,'USA' from dual union all
select 22380,'SRC' from dual union all
select 22381,'NASA' from dual union all
select 22382,'BAC' from dual union all
select 22383,'NASA' from dual union all
select 22384,'AFCRL' from dual union all
select 22385,'AFCRL' from dual union all
select 22386,'AFCRL' from dual union all
select 22387,'AFCRL' from dual union all
select 22388,'AFCRL' from dual union all
select 22389,'AFCRL' from dual union all
select 2239,'USA' from dual union all
select 22390,'NASA' from dual union all
select 22391,'USAF' from dual union all
select 22392,'NASA' from dual union all
select 22393,'US7A' from dual union all
select 22394,'US7A' from dual union all
select 22395,'US7A' from dual union all
select 22396,'US7A' from dual union all
select 22397,'NASA' from dual union all
select 22398,'BAC' from dual union all
select 22399,'USN' from dual union all
select 224,'WER836-1' from dual union all
select 2240,'USA' from dual union all
select 22400,'AN' from dual union all
select 22401,'USN' from dual union all
select 22402,'AN' from dual union all
select 22403,'NASA' from dual union all
select 22404,'NASA' from dual union all
select 22405,'AFCRL' from dual union all
select 22406,'AFSC' from dual union all
select 22407,'ESRO' from dual union all
select 22408,'SAC' from dual union all
select 22409,'NRCC' from dual union all
select 2241,'USA' from dual union all
select 22410,'INTA' from dual union all
select 22411,'DLR' from dual union all
select 22412,'RVSN' from dual union all
select 22413,'NASA' from dual union all
select 22414,'INTA' from dual union all
select 22415,'RVSN' from dual union all
select 22416,'INTA' from dual union all
select 22417,'NRCC' from dual union all
select 22418,'AFSC' from dual union all
select 22419,'NASA' from dual union all
select 2242,'USA' from dual union all
select 22420,'NASA' from dual union all
select 22421,'NASA' from dual union all
select 22422,'NASA' from dual union all
select 22423,'NASA' from dual union all
select 22424,'NRCC' from dual union all
select 22425,'AN' from dual union all
select 22426,'DLR' from dual union all
select 22427,'ALCS' from dual union all
select 22427,'SAC' from dual union all
select 22428,'NASA' from dual union all
select 22429,'AN' from dual union all
select 2243,'USA' from dual union all
select 22430,'NASA' from dual union all
select 22431,'AFSC' from dual union all
select 22432,'INTA' from dual union all
select 22433,'AN' from dual union all
select 22434,'AN' from dual union all
select 22435,'NASA' from dual union all
select 22436,'AN' from dual union all
select 22437,'AFSC' from dual union all
select 22438,'AN' from dual union all
select 22439,'AN' from dual union all
select 2244,'USA' from dual union all
select 22440,'NASA' from dual union all
select 22441,'RTG' from dual union all
select 22442,'SAND' from dual union all
select 22443,'CALT' from dual union all
select 22444,'RVSN' from dual union all
select 22445,'AFCRL' from dual union all
select 22446,'NASA' from dual union all
select 22447,'AFCRL' from dual union all
select 22448,'AFCRL' from dual union all
select 22449,'WRE' from dual union all
select 2245,'USA' from dual union all
select 22450,'WRE' from dual union all
select 22451,'WRE' from dual union all
select 22452,'WRE' from dual;
insert into LAUNCH_AGENCY
select 22453,'WRE' from dual union all
select 22454,'AN' from dual union all
select 22455,'WSMR' from dual union all
select 22456,'USAF' from dual union all
select 22457,'NASA' from dual union all
select 22458,'WSMR' from dual union all
select 22459,'WSMR' from dual union all
select 2246,'USA' from dual union all
select 22460,'AFCRL' from dual union all
select 22461,'SAND' from dual union all
select 22462,'SAND' from dual union all
select 22463,'SAND' from dual union all
select 22464,'SAND' from dual union all
select 22465,'SAND' from dual union all
select 22466,'NRL' from dual union all
select 22467,'SAND' from dual union all
select 22468,'NASA' from dual union all
select 22469,'SAND' from dual union all
select 2247,'USA' from dual union all
select 22470,'RVSN' from dual union all
select 22471,'RVSN' from dual union all
select 22472,'RVSN' from dual union all
select 22473,'RVSN' from dual union all
select 22474,'RVSN' from dual union all
select 22475,'RVSN' from dual union all
select 22476,'RVSN' from dual union all
select 22477,'RVSN' from dual union all
select 22478,'RVSN' from dual union all
select 22479,'RVSN' from dual union all
select 2248,'ORDRSA' from dual union all
select 22480,'RVSN' from dual union all
select 22481,'RVSN' from dual union all
select 22482,'RVSN' from dual union all
select 22483,'NASA' from dual union all
select 22484,'RVSN' from dual union all
select 22485,'RVSN' from dual union all
select 22486,'RVSN' from dual union all
select 22487,'RVSN' from dual union all
select 22488,'RVSN' from dual union all
select 22489,'RVSN' from dual union all
select 2249,'ORDRSA' from dual union all
select 22490,'RVSN' from dual union all
select 22491,'RVSN' from dual union all
select 22492,'RVSN' from dual union all
select 22493,'INTA' from dual union all
select 22494,'RVSN' from dual union all
select 22495,'RVSN' from dual union all
select 22496,'MVS' from dual union all
select 22497,'MVS' from dual union all
select 22498,'WRE' from dual union all
select 22499,'WRE' from dual union all
select 225,'SS500' from dual union all
select 2250,'GEMTS' from dual union all
select 22500,'AFCRL' from dual union all
select 22501,'RVSN' from dual union all
select 22502,'RVSN' from dual union all
select 22503,'RVSN' from dual union all
select 22504,'RVSN' from dual union all
select 22505,'RVSN' from dual union all
select 22506,'RVSN' from dual union all
select 22507,'RVSN' from dual union all
select 22508,'RVSN' from dual union all
select 22509,'RVSN' from dual union all
select 2251,'ORDRSA' from dual union all
select 22510,'RVSN' from dual union all
select 22511,'RVSN' from dual union all
select 22512,'MVS' from dual union all
select 22513,'MVS' from dual union all
select 22514,'MVS' from dual union all
select 22515,'ISAS' from dual union all
select 22516,'MVS' from dual union all
select 22517,'MVS' from dual union all
select 22518,'MVS' from dual union all
select 22519,'MVS' from dual union all
select 2252,'ORDRSA' from dual union all
select 22520,'MVS' from dual union all
select 22521,'RVSN' from dual union all
select 22522,'RVSN' from dual union all
select 22523,'RVSN' from dual union all
select 22524,'AFCRL' from dual union all
select 22525,'SAND' from dual union all
select 22526,'NASA' from dual union all
select 22527,'INTA' from dual union all
select 22528,'RVSN' from dual union all
select 22529,'ISAS' from dual union all
select 2253,'USA' from dual union all
select 22530,'INTA' from dual union all
select 22531,'USN' from dual union all
select 22532,'ESRO' from dual union all
select 22533,'NRL' from dual union all
select 22534,'NASA' from dual union all
select 22535,'ISAS' from dual union all
select 22536,'DLR' from dual union all
select 22537,'ISAS' from dual union all
select 22538,'NASA' from dual union all
select 22539,'CALT' from dual union all
select 2254,'USA' from dual union all
select 22540,'RVSN' from dual union all
select 22541,'RVSN' from dual union all
select 22542,'DLR' from dual;
insert into LAUNCH_AGENCY
select 22543,'NASDA' from dual union all
select 22544,'DLR' from dual union all
select 22545,'NASA' from dual union all
select 22546,'NASA' from dual union all
select 22547,'NASDA' from dual union all
select 22548,'NASA' from dual union all
select 22549,'NASDA' from dual union all
select 2255,'USA' from dual union all
select 22550,'NASA' from dual union all
select 22551,'NPOL' from dual union all
select 22552,'ELDO' from dual union all
select 22553,'NASA' from dual union all
select 22554,'AFCRL' from dual union all
select 22555,'AFCRL' from dual union all
select 22556,'AFCRL' from dual union all
select 22557,'AFCRL' from dual union all
select 22558,'AFCRL' from dual union all
select 22559,'MRN' from dual union all
select 2256,'ORDRSA' from dual union all
select 22560,'MRN' from dual union all
select 22561,'MRN' from dual union all
select 22562,'MRN' from dual union all
select 22563,'MRN' from dual union all
select 22564,'MRN' from dual union all
select 22565,'MRN' from dual union all
select 22566,'MRN' from dual union all
select 22567,'MRN' from dual union all
select 22568,'MRN' from dual union all
select 22569,'MRN' from dual union all
select 2257,'USA' from dual union all
select 22570,'MRN' from dual union all
select 22571,'MRN' from dual union all
select 22572,'MRN' from dual union all
select 22573,'MRN' from dual union all
select 22574,'MRN' from dual union all
select 22575,'MRN' from dual union all
select 22576,'MRN' from dual union all
select 22577,'MRN' from dual union all
select 22578,'MRN' from dual union all
select 22579,'MRN' from dual union all
select 2258,'USA' from dual union all
select 22580,'MRN' from dual union all
select 22581,'MRN' from dual union all
select 22582,'MRN' from dual union all
select 22583,'MRN' from dual union all
select 22584,'MRN' from dual union all
select 22585,'MRN' from dual union all
select 22586,'MRN' from dual union all
select 22587,'MRN' from dual union all
select 22588,'MRN' from dual union all
select 22589,'MRN' from dual union all
select 2259,'USA' from dual union all
select 22590,'MRN' from dual union all
select 22591,'MRN' from dual union all
select 22592,'MRN' from dual union all
select 22593,'MRN' from dual union all
select 22594,'MRN' from dual union all
select 22595,'MRN' from dual union all
select 22596,'SUPA' from dual union all
select 22597,'MRN' from dual union all
select 22598,'MRN' from dual union all
select 22599,'MRN' from dual union all
select 226,'WER444' from dual union all
select 2260,'USA' from dual union all
select 22600,'MRN' from dual union all
select 22601,'INTA' from dual union all
select 22602,'MRN' from dual union all
select 22603,'MRN' from dual union all
select 22604,'MRN' from dual union all
select 22605,'MRN' from dual union all
select 22606,'MRN' from dual union all
select 22607,'MRN' from dual union all
select 22608,'MRN' from dual union all
select 22609,'MRN' from dual union all
select 2261,'USA' from dual union all
select 22610,'MRN' from dual union all
select 22611,'MRN' from dual union all
select 22612,'MRN' from dual union all
select 22613,'MRN' from dual union all
select 22614,'MRN' from dual union all
select 22615,'MRN' from dual union all
select 22616,'MRN' from dual union all
select 22617,'MRN' from dual union all
select 22618,'MRN' from dual union all
select 22619,'MRN' from dual union all
select 2262,'USA' from dual union all
select 22620,'MRN' from dual union all
select 22621,'MRN' from dual union all
select 22622,'MRN' from dual union all
select 22623,'MRN' from dual union all
select 22624,'MRN' from dual union all
select 22625,'MRN' from dual union all
select 22626,'MRN' from dual union all
select 22627,'MRN' from dual union all
select 22628,'MRN' from dual union all
select 22629,'MRN' from dual union all
select 2263,'USA' from dual union all
select 22630,'MRN' from dual union all
select 22631,'MRN' from dual union all
select 22632,'MRN' from dual;
insert into LAUNCH_AGENCY
select 22633,'MRN' from dual union all
select 22634,'MRN' from dual union all
select 22635,'MRN' from dual union all
select 22636,'MRN' from dual union all
select 22637,'MRN' from dual union all
select 22638,'MRN' from dual union all
select 22639,'MRN' from dual union all
select 2264,'ORDRSA' from dual union all
select 22640,'MRN' from dual union all
select 22641,'MRN' from dual union all
select 22642,'MRN' from dual union all
select 22643,'MRN' from dual union all
select 22644,'MRN' from dual union all
select 22645,'MRN' from dual union all
select 22646,'MRN' from dual union all
select 22647,'MRN' from dual union all
select 22648,'MRN' from dual union all
select 22649,'MRN' from dual union all
select 2265,'ORDRSA' from dual union all
select 22650,'MRN' from dual union all
select 22651,'MRN' from dual union all
select 22652,'MRN' from dual union all
select 22653,'MRN' from dual union all
select 22654,'MRN' from dual union all
select 22655,'MRN' from dual union all
select 22656,'MRN' from dual union all
select 22657,'MRN' from dual union all
select 22658,'MRN' from dual union all
select 22659,'MRN' from dual union all
select 2266,'USA' from dual union all
select 22660,'MRN' from dual union all
select 22661,'MRN' from dual union all
select 22662,'MRN' from dual union all
select 22663,'MRN' from dual union all
select 22664,'MRN' from dual union all
select 22665,'MRN' from dual union all
select 22666,'MRN' from dual union all
select 22667,'MRN' from dual union all
select 22668,'MRN' from dual union all
select 22669,'MRN' from dual union all
select 2267,'ORDRSA' from dual union all
select 22670,'MRN' from dual union all
select 22671,'MRN' from dual union all
select 22672,'MRN' from dual union all
select 22673,'MRN' from dual union all
select 22674,'MRN' from dual union all
select 22675,'MRN' from dual union all
select 22676,'MRN' from dual union all
select 22677,'MRN' from dual union all
select 22678,'MRN' from dual union all
select 22679,'MRN' from dual union all
select 2268,'ORDRSA' from dual union all
select 22680,'MRN' from dual union all
select 22681,'MRN' from dual union all
select 22682,'INTA' from dual union all
select 22683,'MRN' from dual union all
select 22684,'MRN' from dual union all
select 22685,'MRN' from dual union all
select 22686,'MRN' from dual union all
select 22687,'MRN' from dual union all
select 22688,'MRN' from dual union all
select 22689,'MRN' from dual union all
select 2269,'ORDRSA' from dual union all
select 22690,'MRN' from dual union all
select 22691,'MRN' from dual union all
select 22692,'MRN' from dual union all
select 22693,'MRN' from dual union all
select 22694,'MRN' from dual union all
select 22695,'MRN' from dual union all
select 22696,'MRN' from dual union all
select 22697,'MRN' from dual union all
select 22698,'MRN' from dual union all
select 22699,'MRN' from dual union all
select 227,'WER444' from dual union all
select 2270,'USAF' from dual union all
select 22700,'MRN' from dual union all
select 22701,'MRN' from dual union all
select 22702,'MRN' from dual union all
select 22703,'MRN' from dual union all
select 22704,'MRN' from dual union all
select 22705,'MRN' from dual union all
select 22706,'MRN' from dual union all
select 22707,'MRN' from dual union all
select 22708,'SRC' from dual union all
select 22709,'MRN' from dual union all
select 2271,'USAF' from dual union all
select 22710,'MRN' from dual union all
select 22711,'MRN' from dual union all
select 22712,'MRN' from dual union all
select 22713,'MRN' from dual union all
select 22714,'MRN' from dual union all
select 22715,'MRN' from dual union all
select 22716,'MRN' from dual union all
select 22717,'MRN' from dual union all
select 22718,'MRN' from dual union all
select 22719,'INTA' from dual union all
select 2272,'USAF' from dual union all
select 22720,'MRN' from dual union all
select 22721,'MRN' from dual union all
select 22722,'MRN' from dual;
insert into LAUNCH_AGENCY
select 22723,'MRN' from dual union all
select 22724,'JMA' from dual union all
select 22725,'MRN' from dual union all
select 22726,'MRN' from dual union all
select 22727,'MRN' from dual union all
select 22728,'MRN' from dual union all
select 22729,'MRN' from dual union all
select 2273,'USA' from dual union all
select 22730,'SRC' from dual union all
select 22731,'MRN' from dual union all
select 22732,'MRN' from dual union all
select 22733,'MRN' from dual union all
select 22734,'MRN' from dual union all
select 22735,'MRN' from dual union all
select 22736,'MRN' from dual union all
select 22737,'MRN' from dual union all
select 22738,'MRN' from dual union all
select 22739,'MRN' from dual union all
select 2274,'USA' from dual union all
select 22740,'MRN' from dual union all
select 22741,'MRN' from dual union all
select 22742,'MRN' from dual union all
select 22743,'MRN' from dual union all
select 22744,'MRN' from dual union all
select 22745,'MRN' from dual union all
select 22746,'MRN' from dual union all
select 22747,'MRN' from dual union all
select 22748,'MRN' from dual union all
select 22749,'MRN' from dual union all
select 2275,'USA' from dual union all
select 22750,'MRN' from dual union all
select 22751,'MRN' from dual union all
select 22752,'MRN' from dual union all
select 22753,'MRN' from dual union all
select 22754,'MRN' from dual union all
select 22755,'MRN' from dual union all
select 22756,'MRN' from dual union all
select 22757,'MRN' from dual union all
select 22758,'MRN' from dual union all
select 22759,'MRN' from dual union all
select 2276,'ORDRSA' from dual union all
select 22760,'MRN' from dual union all
select 22761,'MRN' from dual union all
select 22762,'MRN' from dual union all
select 22763,'MRN' from dual union all
select 22764,'MRN' from dual union all
select 22765,'MRN' from dual union all
select 22766,'MRN' from dual union all
select 22767,'MRN' from dual union all
select 22768,'MRN' from dual union all
select 22769,'MRN' from dual union all
select 2277,'ORDRSA' from dual union all
select 22770,'MRN' from dual union all
select 22771,'MRN' from dual union all
select 22772,'MRN' from dual union all
select 22773,'MRN' from dual union all
select 22774,'MRN' from dual union all
select 22775,'MRN' from dual union all
select 22776,'MRN' from dual union all
select 22777,'MRN' from dual union all
select 22778,'MRN' from dual union all
select 22779,'MRN' from dual union all
select 2278,'USA' from dual union all
select 22780,'MRN' from dual union all
select 22781,'MRN' from dual union all
select 22782,'MRN' from dual union all
select 22783,'MRN' from dual union all
select 22784,'MRN' from dual union all
select 22785,'MRN' from dual union all
select 22786,'MRN' from dual union all
select 22787,'MRN' from dual union all
select 22788,'MRN' from dual union all
select 22789,'MRN' from dual union all
select 2279,'USA' from dual union all
select 22790,'MRN' from dual union all
select 22791,'MRN' from dual union all
select 22792,'MRN' from dual union all
select 22793,'MRN' from dual union all
select 22794,'MRN' from dual union all
select 22795,'MRN' from dual union all
select 22796,'MRN' from dual union all
select 22797,'MRN' from dual union all
select 22798,'MRN' from dual union all
select 22799,'MRN' from dual union all
select 228,'WER271-1' from dual union all
select 2280,'USA' from dual union all
select 22800,'MRN' from dual union all
select 22801,'MRN' from dual union all
select 22802,'MRN' from dual union all
select 22803,'MRN' from dual union all
select 22804,'MRN' from dual union all
select 22805,'MRN' from dual union all
select 22806,'MRN' from dual union all
select 22807,'MRN' from dual union all
select 22808,'MRN' from dual union all
select 22809,'MRN' from dual union all
select 2281,'USAF' from dual union all
select 22810,'MRN' from dual union all
select 22811,'MRN' from dual union all
select 22812,'MRN' from dual;
insert into LAUNCH_AGENCY
select 22813,'MRN' from dual union all
select 22814,'MRN' from dual union all
select 22815,'MRN' from dual union all
select 22816,'MRN' from dual union all
select 22817,'MRN' from dual union all
select 22818,'MRN' from dual union all
select 22819,'MRN' from dual union all
select 2282,'USA' from dual union all
select 22820,'MRN' from dual union all
select 22821,'MRN' from dual union all
select 22822,'MRN' from dual union all
select 22823,'MRN' from dual union all
select 22824,'MRN' from dual union all
select 22825,'MRN' from dual union all
select 22826,'MRN' from dual union all
select 22827,'MRN' from dual union all
select 22828,'MRN' from dual union all
select 22829,'MRN' from dual union all
select 2283,'USAF' from dual union all
select 22830,'MRN' from dual union all
select 22831,'MRN' from dual union all
select 22832,'MRN' from dual union all
select 22833,'MRN' from dual union all
select 22834,'MRN' from dual union all
select 22835,'MRN' from dual union all
select 22836,'PIHM' from dual union all
select 22837,'MRN' from dual union all
select 22838,'MRN' from dual union all
select 22839,'MRN' from dual union all
select 2284,'USAF' from dual union all
select 22840,'MRN' from dual union all
select 22841,'MRN' from dual union all
select 22842,'MRN' from dual union all
select 22843,'MRN' from dual union all
select 22844,'MRN' from dual union all
select 22845,'MRN' from dual union all
select 22846,'MRN' from dual union all
select 22847,'MRN' from dual union all
select 22848,'MRN' from dual union all
select 22849,'MRN' from dual union all
select 2285,'USA' from dual union all
select 22850,'MRN' from dual union all
select 22851,'MRN' from dual union all
select 22852,'MRN' from dual union all
select 22853,'MRN' from dual union all
select 22854,'MRN' from dual union all
select 22855,'MRN' from dual union all
select 22856,'MRN' from dual union all
select 22857,'MRN' from dual union all
select 22858,'MRN' from dual union all
select 22859,'MRN' from dual union all
select 2286,'USA' from dual union all
select 22860,'MRN' from dual union all
select 22861,'MRN' from dual union all
select 22862,'MRN' from dual union all
select 22863,'MRN' from dual union all
select 22864,'MRN' from dual union all
select 22865,'MRN' from dual union all
select 22866,'MRN' from dual union all
select 22867,'MRN' from dual union all
select 22868,'MRN' from dual union all
select 22869,'MRN' from dual union all
select 2287,'USA' from dual union all
select 22870,'MRN' from dual union all
select 22871,'MRN' from dual union all
select 22872,'MRN' from dual union all
select 22873,'MRN' from dual union all
select 22874,'MRN' from dual union all
select 22875,'MRN' from dual union all
select 22876,'MRN' from dual union all
select 22877,'MRN' from dual union all
select 22878,'MRN' from dual union all
select 22879,'MRN' from dual union all
select 2288,'USAF' from dual union all
select 22880,'MRN' from dual union all
select 22881,'MRN' from dual union all
select 22882,'MRN' from dual union all
select 22883,'MRN' from dual union all
select 22884,'MRN' from dual union all
select 22885,'INTA' from dual union all
select 22886,'MRN' from dual union all
select 22887,'MRN' from dual union all
select 22888,'PIHM' from dual union all
select 22889,'MRN' from dual union all
select 2289,'PARD' from dual union all
select 22890,'MRN' from dual union all
select 22891,'MRN' from dual union all
select 22892,'MRN' from dual union all
select 22893,'MRN' from dual union all
select 22894,'MRN' from dual union all
select 22895,'MRN' from dual union all
select 22896,'PIHM' from dual union all
select 22897,'MRN' from dual union all
select 22898,'MRN' from dual union all
select 22899,'MRN' from dual union all
select 229,'WER271-1' from dual union all
select 2290,'USA' from dual union all
select 22900,'MRN' from dual union all
select 22901,'MRN' from dual union all
select 22902,'MRN' from dual;
insert into LAUNCH_AGENCY
select 22903,'PIHM' from dual union all
select 22904,'MRN' from dual union all
select 22905,'MRN' from dual union all
select 22906,'MRN' from dual union all
select 22907,'MRN' from dual union all
select 22908,'SRC' from dual union all
select 22909,'MRN' from dual union all
select 2291,'USA' from dual union all
select 22910,'MRN' from dual union all
select 22911,'MRN' from dual union all
select 22912,'INTA' from dual union all
select 22913,'MRN' from dual union all
select 22914,'MRN' from dual union all
select 22915,'MRN' from dual union all
select 22916,'MRN' from dual union all
select 22917,'MRN' from dual union all
select 22918,'MRN' from dual union all
select 22919,'MRN' from dual union all
select 2292,'ORDRSA' from dual union all
select 22920,'MRN' from dual union all
select 22921,'MRN' from dual union all
select 22922,'PIHM' from dual union all
select 22923,'MRN' from dual union all
select 22924,'MRN' from dual union all
select 22925,'MRN' from dual union all
select 22926,'MRN' from dual union all
select 22927,'MRN' from dual union all
select 22928,'MRN' from dual union all
select 22929,'MRN' from dual union all
select 2293,'ORDRSA' from dual union all
select 22930,'MRN' from dual union all
select 22931,'MRN' from dual union all
select 22932,'MRN' from dual union all
select 22933,'MRN' from dual union all
select 22934,'MRN' from dual union all
select 22935,'MRN' from dual union all
select 22936,'MRN' from dual union all
select 22937,'MRN' from dual union all
select 22938,'MRN' from dual union all
select 22939,'MRN' from dual union all
select 2294,'ORDRSA' from dual union all
select 22940,'MRN' from dual union all
select 22941,'MRN' from dual union all
select 22942,'MRN' from dual union all
select 22943,'MRN' from dual union all
select 22944,'MRN' from dual union all
select 22945,'MRN' from dual union all
select 22946,'MRN' from dual union all
select 22947,'MRN' from dual union all
select 22948,'MRN' from dual union all
select 22949,'MRN' from dual union all
select 2295,'USA' from dual union all
select 22950,'MRN' from dual union all
select 22951,'MRN' from dual union all
select 22952,'MRN' from dual union all
select 22953,'MRN' from dual union all
select 22954,'MRN' from dual union all
select 22955,'MRN' from dual union all
select 22956,'MRN' from dual union all
select 22957,'MRN' from dual union all
select 22958,'MRN' from dual union all
select 22959,'MRN' from dual union all
select 2296,'USA' from dual union all
select 22960,'INTA' from dual union all
select 22961,'MRN' from dual union all
select 22962,'MRN' from dual union all
select 22963,'MRN' from dual union all
select 22964,'MRN' from dual union all
select 22965,'MRN' from dual union all
select 22966,'MRN' from dual union all
select 22967,'MRN' from dual union all
select 22968,'MRN' from dual union all
select 22969,'MRN' from dual union all
select 2297,'USA' from dual union all
select 22970,'MRN' from dual union all
select 22971,'MRN' from dual union all
select 22972,'MRN' from dual union all
select 22973,'MRN' from dual union all
select 22974,'MRN' from dual union all
select 22975,'MRN' from dual union all
select 22976,'MRN' from dual union all
select 22977,'MRN' from dual union all
select 22978,'NASA' from dual union all
select 22979,'NASA' from dual union all
select 2298,'USA' from dual union all
select 22980,'MRN' from dual union all
select 22981,'MRN' from dual union all
select 22982,'MRN' from dual union all
select 22983,'MRN' from dual union all
select 22984,'MRN' from dual union all
select 22985,'MRN' from dual union all
select 22986,'MRN' from dual union all
select 22987,'MRN' from dual union all
select 22988,'MRN' from dual union all
select 22989,'MRN' from dual union all
select 2299,'USA' from dual union all
select 22990,'MRN' from dual union all
select 22991,'MRN' from dual union all
select 22992,'MRN' from dual union all
select 22993,'MRN' from dual;
insert into LAUNCH_AGENCY
select 22994,'MRN' from dual union all
select 22995,'MRN' from dual union all
select 22996,'MRN' from dual union all
select 22997,'MRN' from dual union all
select 22998,'MRN' from dual union all
select 22999,'NASA' from dual union all
select 23,'WEHR' from dual union all
select 230,'WER271-1' from dual union all
select 2300,'USAF' from dual union all
select 23000,'MRN' from dual union all
select 23001,'MRN' from dual union all
select 23002,'MRN' from dual union all
select 23003,'MRN' from dual union all
select 23004,'MRN' from dual union all
select 23005,'MRN' from dual union all
select 23006,'MRN' from dual union all
select 23007,'MRN' from dual union all
select 23008,'MRN' from dual union all
select 23009,'MRN' from dual union all
select 2301,'USA' from dual union all
select 23010,'MRN' from dual union all
select 23011,'MRN' from dual union all
select 23012,'MRN' from dual union all
select 23013,'MRN' from dual union all
select 23014,'MRN' from dual union all
select 23015,'MRN' from dual union all
select 23016,'MRN' from dual union all
select 23017,'MRN' from dual union all
select 23018,'MRN' from dual union all
select 23019,'MRN' from dual union all
select 2302,'USA' from dual union all
select 23020,'MRN' from dual union all
select 23021,'MRN' from dual union all
select 23022,'MRN' from dual union all
select 23023,'MRN' from dual union all
select 23024,'MRN' from dual union all
select 23025,'MRN' from dual union all
select 23026,'MRN' from dual union all
select 23027,'MRN' from dual union all
select 23028,'MRN' from dual union all
select 23029,'MRN' from dual union all
select 2303,'USA' from dual union all
select 23030,'ISAS' from dual union all
select 23031,'INTA' from dual union all
select 23032,'MRN' from dual union all
select 23033,'MRN' from dual union all
select 23034,'MRN' from dual union all
select 23035,'MRN' from dual union all
select 23036,'MRN' from dual union all
select 23037,'MRN' from dual union all
select 23038,'MRN' from dual union all
select 23039,'MRN' from dual union all
select 2304,'USA' from dual union all
select 23040,'MRN' from dual union all
select 23041,'MRN' from dual union all
select 23042,'MRN' from dual union all
select 23043,'MRN' from dual union all
select 23044,'MRN' from dual union all
select 23045,'MRN' from dual union all
select 23046,'MRN' from dual union all
select 23047,'MRN' from dual union all
select 23048,'MRN' from dual union all
select 23049,'MRN' from dual union all
select 2305,'USA' from dual union all
select 23050,'INTA' from dual union all
select 23051,'MRN' from dual union all
select 23052,'MRN' from dual union all
select 23053,'MRN' from dual union all
select 23054,'MRN' from dual union all
select 23055,'MRN' from dual union all
select 23056,'MRN' from dual union all
select 23057,'MRN' from dual union all
select 23058,'MRN' from dual union all
select 23059,'MRN' from dual union all
select 2306,'USA' from dual union all
select 23060,'MRN' from dual union all
select 23061,'MRN' from dual union all
select 23062,'MRN' from dual union all
select 23063,'ISAS' from dual union all
select 23064,'MRN' from dual union all
select 23065,'MRN' from dual union all
select 23066,'MRN' from dual union all
select 23067,'MRN' from dual union all
select 23068,'MRN' from dual union all
select 23069,'MRN' from dual union all
select 2307,'ORDRSA' from dual union all
select 23070,'MRN' from dual union all
select 23071,'MRN' from dual union all
select 23072,'MRN' from dual union all
select 23073,'MRN' from dual union all
select 23074,'SRC' from dual union all
select 23075,'MRN' from dual union all
select 23076,'MRN' from dual union all
select 23077,'MRN' from dual union all
select 23078,'MRN' from dual union all
select 23079,'MRN' from dual union all
select 2308,'ORDRSA' from dual union all
select 23080,'USAF' from dual union all
select 23081,'MRN' from dual union all
select 23082,'MRN' from dual;
insert into LAUNCH_AGENCY
select 23083,'INTA' from dual union all
select 23084,'MRN' from dual union all
select 23085,'MRN' from dual union all
select 23086,'MRN' from dual union all
select 23087,'MRN' from dual union all
select 23088,'MRN' from dual union all
select 23089,'MRN' from dual union all
select 2309,'USA' from dual union all
select 23090,'USAF' from dual union all
select 23091,'MRN' from dual union all
select 23092,'MRN' from dual union all
select 23093,'MRN' from dual union all
select 23094,'MRN' from dual union all
select 23095,'USAF' from dual union all
select 23096,'MRN' from dual union all
select 23097,'USAF' from dual union all
select 23098,'USAF' from dual union all
select 23099,'MRN' from dual union all
select 231,'WEHR' from dual union all
select 2310,'USA' from dual union all
select 23100,'MRN' from dual union all
select 23101,'MRN' from dual union all
select 23102,'MRN' from dual union all
select 23103,'MRN' from dual union all
select 23104,'MRN' from dual union all
select 23105,'MRN' from dual union all
select 23106,'MRN' from dual union all
select 23107,'MRN' from dual union all
select 23108,'MRN' from dual union all
select 23109,'MRN' from dual union all
select 2311,'USA' from dual union all
select 23110,'MRN' from dual union all
select 23111,'MRN' from dual union all
select 23112,'MRN' from dual union all
select 23113,'MRN' from dual union all
select 23114,'MRN' from dual union all
select 23115,'MRN' from dual union all
select 23116,'NASA' from dual union all
select 23117,'MRN' from dual union all
select 23118,'MRN' from dual union all
select 23119,'MRN' from dual union all
select 2312,'USA' from dual union all
select 23120,'MRN' from dual union all
select 23121,'MRN' from dual union all
select 23122,'MRN' from dual union all
select 23123,'MRN' from dual union all
select 23124,'MRN' from dual union all
select 23125,'MRN' from dual union all
select 23126,'MRN' from dual union all
select 23127,'MRN' from dual union all
select 23128,'MRN' from dual union all
select 23129,'MRN' from dual union all
select 2313,'USA' from dual union all
select 23130,'MRN' from dual union all
select 23131,'MRN' from dual union all
select 23132,'MRN' from dual union all
select 23133,'MRN' from dual union all
select 23134,'MRN' from dual union all
select 23135,'MRN' from dual union all
select 23136,'MRN' from dual union all
select 23137,'MRN' from dual union all
select 23138,'MRN' from dual union all
select 23139,'MRN' from dual union all
select 2314,'USA' from dual union all
select 23140,'MRN' from dual union all
select 23141,'MRN' from dual union all
select 23142,'MRN' from dual union all
select 23143,'MRN' from dual union all
select 23144,'MRN' from dual union all
select 23145,'MRN' from dual union all
select 23146,'MRN' from dual union all
select 23147,'MRN' from dual union all
select 23148,'MRN' from dual union all
select 23149,'MRN' from dual union all
select 2315,'USA' from dual union all
select 23150,'MRN' from dual union all
select 23151,'MRN' from dual union all
select 23152,'MRN' from dual union all
select 23153,'MRN' from dual union all
select 23154,'MRN' from dual union all
select 23155,'MRN' from dual union all
select 23156,'MRN' from dual union all
select 23157,'MRN' from dual union all
select 23158,'MRN' from dual union all
select 23159,'MRN' from dual union all
select 2316,'PARD' from dual union all
select 23160,'MRN' from dual union all
select 23161,'NASA' from dual union all
select 23162,'MRN' from dual union all
select 23163,'MRN' from dual union all
select 23164,'MRN' from dual union all
select 23165,'MRN' from dual union all
select 23166,'MRN' from dual union all
select 23167,'MRN' from dual union all
select 23168,'MRN' from dual union all
select 23169,'MRN' from dual union all
select 2317,'USAF' from dual union all
select 23170,'MRN' from dual union all
select 23171,'MRN' from dual union all
select 23172,'MRN' from dual;
insert into LAUNCH_AGENCY
select 23173,'MRN' from dual union all
select 23174,'MRN' from dual union all
select 23175,'MRN' from dual union all
select 23176,'MRN' from dual union all
select 23177,'MRN' from dual union all
select 23178,'SUPA' from dual union all
select 23179,'MRN' from dual union all
select 2318,'USAF' from dual union all
select 23180,'MRN' from dual union all
select 23181,'MRN' from dual union all
select 23182,'MRN' from dual union all
select 23183,'MRN' from dual union all
select 23184,'MRN' from dual union all
select 23185,'MRN' from dual union all
select 23186,'MRN' from dual union all
select 23187,'MRN' from dual union all
select 23188,'MRN' from dual union all
select 23189,'MRN' from dual union all
select 2319,'PARD' from dual union all
select 23190,'MRN' from dual union all
select 23191,'MRN' from dual union all
select 23192,'MRN' from dual union all
select 23193,'MRN' from dual union all
select 23194,'MRN' from dual union all
select 23195,'MRN' from dual union all
select 23196,'SRC' from dual union all
select 23197,'MRN' from dual union all
select 23198,'MRN' from dual union all
select 23199,'MRN' from dual union all
select 232,'WER271-1' from dual union all
select 2320,'ORDRSA' from dual union all
select 23200,'MRN' from dual union all
select 23201,'MRN' from dual union all
select 23202,'MRN' from dual union all
select 23203,'MRN' from dual union all
select 23204,'MRN' from dual union all
select 23205,'MRN' from dual union all
select 23206,'MRN' from dual union all
select 23207,'MRN' from dual union all
select 23208,'MRN' from dual union all
select 23209,'MRN' from dual union all
select 2321,'USA' from dual union all
select 23210,'INTA' from dual union all
select 23211,'MRN' from dual union all
select 23212,'MRN' from dual union all
select 23213,'MRN' from dual union all
select 23214,'MRN' from dual union all
select 23215,'MRN' from dual union all
select 23216,'MRN' from dual union all
select 23217,'MRN' from dual union all
select 23218,'MRN' from dual union all
select 23219,'MRN' from dual union all
select 2322,'USAF' from dual union all
select 23220,'MRN' from dual union all
select 23221,'MRN' from dual union all
select 23222,'MRN' from dual union all
select 23223,'MRN' from dual union all
select 23224,'MRN' from dual union all
select 23225,'MRN' from dual union all
select 23226,'MRN' from dual union all
select 23227,'MRN' from dual union all
select 23228,'MRN' from dual union all
select 23229,'MRN' from dual union all
select 2323,'USAF' from dual union all
select 23230,'MRN' from dual union all
select 23231,'MRN' from dual union all
select 23232,'MRN' from dual union all
select 23233,'MRN' from dual union all
select 23234,'MRN' from dual union all
select 23235,'MRN' from dual union all
select 23236,'MRN' from dual union all
select 23237,'MRN' from dual union all
select 23238,'MRN' from dual union all
select 23239,'MRN' from dual union all
select 2324,'USA' from dual union all
select 23240,'MRN' from dual union all
select 23241,'MRN' from dual union all
select 23242,'MRN' from dual union all
select 23243,'MRN' from dual union all
select 23244,'MRN' from dual union all
select 23245,'MRN' from dual union all
select 23246,'MRN' from dual union all
select 23247,'MRN' from dual union all
select 23248,'MRN' from dual union all
select 23249,'MRN' from dual union all
select 2325,'ORDRSA' from dual union all
select 23250,'MRN' from dual union all
select 23251,'MRN' from dual union all
select 23252,'MRN' from dual union all
select 23253,'MRN' from dual union all
select 23254,'MRN' from dual union all
select 23255,'MRN' from dual union all
select 23256,'MRN' from dual union all
select 23257,'MRN' from dual union all
select 23258,'MRN' from dual union all
select 23259,'MRN' from dual union all
select 2326,'ORDRSA' from dual union all
select 23260,'MRN' from dual union all
select 23261,'MRN' from dual union all
select 23262,'MRN' from dual;
insert into LAUNCH_AGENCY
select 23263,'INTA' from dual union all
select 23264,'MRN' from dual union all
select 23265,'MRN' from dual union all
select 23266,'MRN' from dual union all
select 23267,'MRN' from dual union all
select 23268,'MRN' from dual union all
select 23269,'MRN' from dual union all
select 2327,'USA' from dual union all
select 23270,'MRN' from dual union all
select 23271,'MRN' from dual union all
select 23272,'MRN' from dual union all
select 23273,'MRN' from dual union all
select 23274,'MRN' from dual union all
select 23275,'MRN' from dual union all
select 23276,'MRN' from dual union all
select 23277,'MRN' from dual union all
select 23278,'MRN' from dual union all
select 23279,'NASA' from dual union all
select 2328,'USA' from dual union all
select 23280,'MRN' from dual union all
select 23281,'MRN' from dual union all
select 23282,'MRN' from dual union all
select 23283,'MRN' from dual union all
select 23284,'MRN' from dual union all
select 23285,'MRN' from dual union all
select 23286,'MRN' from dual union all
select 23287,'MRN' from dual union all
select 23288,'MRN' from dual union all
select 23289,'MRN' from dual union all
select 2329,'USA' from dual union all
select 23290,'MRN' from dual union all
select 23291,'MRN' from dual union all
select 23292,'MRN' from dual union all
select 23293,'MRN' from dual union all
select 23294,'MRN' from dual union all
select 23295,'MRN' from dual union all
select 23296,'MRN' from dual union all
select 23297,'MRN' from dual union all
select 23298,'MRN' from dual union all
select 23299,'MRN' from dual union all
select 233,'SS500' from dual union all
select 2330,'USA' from dual union all
select 23300,'MRN' from dual union all
select 23301,'MRN' from dual union all
select 23302,'MRN' from dual union all
select 23303,'MRN' from dual union all
select 23304,'MRN' from dual union all
select 23305,'MRN' from dual union all
select 23306,'NASA' from dual union all
select 23307,'SRC' from dual union all
select 23308,'MRN' from dual union all
select 23309,'MRN' from dual union all
select 2331,'USA' from dual union all
select 23310,'MRN' from dual union all
select 23311,'MRN' from dual union all
select 23312,'MRN' from dual union all
select 23313,'MRN' from dual union all
select 23314,'MRN' from dual union all
select 23315,'MRN' from dual union all
select 23316,'MRN' from dual union all
select 23317,'MRN' from dual union all
select 23318,'MRN' from dual union all
select 23319,'MRN' from dual union all
select 2332,'PARD' from dual union all
select 23320,'MRN' from dual union all
select 23321,'MRN' from dual union all
select 23322,'MRN' from dual union all
select 23323,'MRN' from dual union all
select 23324,'MRN' from dual union all
select 23325,'MRN' from dual union all
select 23326,'MRN' from dual union all
select 23327,'MRN' from dual union all
select 23328,'MRN' from dual union all
select 23329,'MRN' from dual union all
select 2333,'USA' from dual union all
select 23330,'MRN' from dual union all
select 23331,'MRN' from dual union all
select 23332,'MRN' from dual union all
select 23333,'MRN' from dual union all
select 23334,'MRN' from dual union all
select 23335,'MRN' from dual union all
select 23336,'MRN' from dual union all
select 23337,'MRN' from dual union all
select 23338,'MRN' from dual union all
select 23339,'MRN' from dual union all
select 2334,'USA' from dual union all
select 23340,'MRN' from dual union all
select 23341,'MRN' from dual union all
select 23342,'MRN' from dual union all
select 23343,'MRN' from dual union all
select 23344,'MRN' from dual union all
select 23345,'MRN' from dual union all
select 23346,'MRN' from dual union all
select 23347,'MRN' from dual union all
select 23348,'MRN' from dual union all
select 23349,'MRN' from dual union all
select 2335,'USAF' from dual union all
select 23350,'MRN' from dual union all
select 23351,'MRN' from dual union all
select 23352,'MRN' from dual;
insert into LAUNCH_AGENCY
select 23353,'MRN' from dual union all
select 23354,'MRN' from dual union all
select 23355,'MRN' from dual union all
select 23356,'MRN' from dual union all
select 23357,'MRN' from dual union all
select 23358,'MRN' from dual union all
select 23359,'MRN' from dual union all
select 2336,'USA' from dual union all
select 23360,'MRN' from dual union all
select 23361,'MRN' from dual union all
select 23362,'MRN' from dual union all
select 23363,'MRN' from dual union all
select 23364,'MRN' from dual union all
select 23365,'MRN' from dual union all
select 23366,'MRN' from dual union all
select 23367,'MRN' from dual union all
select 23368,'MRN' from dual union all
select 23369,'SUPA' from dual union all
select 2337,'USA' from dual union all
select 23370,'MRN' from dual union all
select 23371,'MRN' from dual union all
select 23372,'MRN' from dual union all
select 23373,'MRN' from dual union all
select 23374,'MRN' from dual union all
select 23375,'MRN' from dual union all
select 23376,'MRN' from dual union all
select 23377,'MRN' from dual union all
select 23378,'MRN' from dual union all
select 23379,'MRN' from dual union all
select 2338,'USA' from dual union all
select 23380,'MRN' from dual union all
select 23381,'MRN' from dual union all
select 23382,'MRN' from dual union all
select 23383,'MRN' from dual union all
select 23384,'MRN' from dual union all
select 23385,'MRN' from dual union all
select 23386,'MRN' from dual union all
select 23387,'MRN' from dual union all
select 23388,'MRN' from dual union all
select 23389,'MRN' from dual union all
select 2339,'USA' from dual union all
select 23390,'MRN' from dual union all
select 23391,'MRN' from dual union all
select 23392,'MRN' from dual union all
select 23393,'MRN' from dual union all
select 23394,'MRN' from dual union all
select 23395,'MRN' from dual union all
select 23396,'MRN' from dual union all
select 23397,'MRN' from dual union all
select 23398,'MRN' from dual union all
select 23399,'MRN' from dual union all
select 234,'WER485-1' from dual union all
select 2340,'USA' from dual union all
select 23400,'MRN' from dual union all
select 23401,'MRN' from dual union all
select 23402,'MRN' from dual union all
select 23403,'MRN' from dual union all
select 23404,'MRN' from dual union all
select 23405,'MRN' from dual union all
select 23406,'MRN' from dual union all
select 23407,'MRN' from dual union all
select 23408,'MRN' from dual union all
select 23409,'MRN' from dual union all
select 2341,'ORDRSA' from dual union all
select 23410,'MRN' from dual union all
select 23411,'MRN' from dual union all
select 23412,'MRN' from dual union all
select 23413,'MRN' from dual union all
select 23414,'MRN' from dual union all
select 23415,'MRN' from dual union all
select 23416,'MRN' from dual union all
select 23417,'MRN' from dual union all
select 23418,'MRN' from dual union all
select 23419,'MRN' from dual union all
select 2342,'ORDRSA' from dual union all
select 23420,'MRN' from dual union all
select 23421,'MRN' from dual union all
select 23422,'MRN' from dual union all
select 23423,'MRN' from dual union all
select 23424,'MRN' from dual union all
select 23425,'MRN' from dual union all
select 23426,'MRN' from dual union all
select 23427,'MRN' from dual union all
select 23428,'MRN' from dual union all
select 23429,'SRC' from dual union all
select 2343,'USA' from dual union all
select 23430,'MRN' from dual union all
select 23431,'MRN' from dual union all
select 23432,'MRN' from dual union all
select 23433,'MRN' from dual union all
select 23434,'MRN' from dual union all
select 23435,'MRN' from dual union all
select 23436,'MRN' from dual union all
select 23437,'MRN' from dual union all
select 23438,'MRN' from dual union all
select 23439,'MRN' from dual union all
select 2344,'USAF' from dual union all
select 23440,'MRN' from dual union all
select 23441,'MRN' from dual union all
select 23442,'MRN' from dual;
insert into LAUNCH_AGENCY
select 23443,'MRN' from dual union all
select 23444,'MRN' from dual union all
select 23445,'MRN' from dual union all
select 23446,'MRN' from dual union all
select 23447,'MRN' from dual union all
select 23448,'MRN' from dual union all
select 23449,'MRN' from dual union all
select 2345,'USAF' from dual union all
select 23450,'MRN' from dual union all
select 23451,'MRN' from dual union all
select 23452,'MRN' from dual union all
select 23453,'MRN' from dual union all
select 23454,'MRN' from dual union all
select 23455,'MRN' from dual union all
select 23456,'MRN' from dual union all
select 23457,'MRN' from dual union all
select 23458,'MRN' from dual union all
select 23459,'MRN' from dual union all
select 2346,'USA' from dual union all
select 23460,'MRN' from dual union all
select 23461,'MRN' from dual union all
select 23462,'MRN' from dual union all
select 23463,'MRN' from dual union all
select 23464,'MRN' from dual union all
select 23465,'MRN' from dual union all
select 23466,'MRN' from dual union all
select 23467,'MRN' from dual union all
select 23468,'MRN' from dual union all
select 23469,'MRN' from dual union all
select 2347,'USA' from dual union all
select 23470,'MRN' from dual union all
select 23471,'MRN' from dual union all
select 23472,'MRN' from dual union all
select 23473,'MRN' from dual union all
select 23474,'MRN' from dual union all
select 23475,'MRN' from dual union all
select 23476,'MRN' from dual union all
select 23477,'MRN' from dual union all
select 23478,'MRN' from dual union all
select 23479,'MRN' from dual union all
select 2348,'USA' from dual union all
select 23480,'MRN' from dual union all
select 23481,'INTA' from dual union all
select 23482,'MRN' from dual union all
select 23483,'MRN' from dual union all
select 23484,'MRN' from dual union all
select 23485,'MRN' from dual union all
select 23486,'MRN' from dual union all
select 23487,'MRN' from dual union all
select 23488,'MRN' from dual union all
select 23489,'MRN' from dual union all
select 2349,'ORDRSA' from dual union all
select 23490,'MRN' from dual union all
select 23491,'MRN' from dual union all
select 23492,'MRN' from dual union all
select 23493,'MRN' from dual union all
select 23494,'MRN' from dual union all
select 23495,'MRN' from dual union all
select 23496,'MRN' from dual union all
select 23497,'MRN' from dual union all
select 23498,'MRN' from dual union all
select 23499,'MRN' from dual union all
select 235,'WER485-3' from dual union all
select 2350,'ORDRSA' from dual union all
select 23500,'MRN' from dual union all
select 23501,'MRN' from dual union all
select 23502,'MRN' from dual union all
select 23503,'MRN' from dual union all
select 23504,'MRN' from dual union all
select 23505,'MRN' from dual union all
select 23506,'MRN' from dual union all
select 23507,'MRN' from dual union all
select 23508,'MRN' from dual union all
select 23509,'MRN' from dual union all
select 2351,'USA' from dual union all
select 23510,'MRN' from dual union all
select 23511,'MRN' from dual union all
select 23512,'MRN' from dual union all
select 23513,'MRN' from dual union all
select 23514,'MRN' from dual union all
select 23515,'MRN' from dual union all
select 23516,'MRN' from dual union all
select 23517,'MRN' from dual union all
select 23518,'MRN' from dual union all
select 23519,'MRN' from dual union all
select 2352,'USA' from dual union all
select 23520,'MRN' from dual union all
select 23521,'MRN' from dual union all
select 23522,'MRN' from dual union all
select 23523,'MRN' from dual union all
select 23524,'MRN' from dual union all
select 23525,'MRN' from dual union all
select 23526,'MRN' from dual union all
select 23527,'MRN' from dual union all
select 23528,'MRN' from dual union all
select 23529,'MRN' from dual union all
select 2353,'USA' from dual union all
select 23530,'MRN' from dual union all
select 23531,'MRN' from dual union all
select 23532,'MRN' from dual;
insert into LAUNCH_AGENCY
select 23533,'MRN' from dual union all
select 23534,'MRN' from dual union all
select 23535,'MRN' from dual union all
select 23536,'MRN' from dual union all
select 23537,'MRN' from dual union all
select 23538,'MRN' from dual union all
select 23539,'MRN' from dual union all
select 2354,'ORDRSA' from dual union all
select 23540,'MRN' from dual union all
select 23541,'MRN' from dual union all
select 23542,'MRN' from dual union all
select 23543,'MRN' from dual union all
select 23544,'MRN' from dual union all
select 23545,'MRN' from dual union all
select 23546,'MRN' from dual union all
select 23547,'MRN' from dual union all
select 23548,'MRN' from dual union all
select 23549,'MRN' from dual union all
select 2355,'USA' from dual union all
select 23550,'SUPA' from dual union all
select 23551,'MRN' from dual union all
select 23552,'MRN' from dual union all
select 23553,'MRN' from dual union all
select 23554,'MRN' from dual union all
select 23555,'MRN' from dual union all
select 23556,'MRN' from dual union all
select 23557,'MRN' from dual union all
select 23558,'MRN' from dual union all
select 23559,'MRN' from dual union all
select 2356,'USA' from dual union all
select 23560,'JMA' from dual union all
select 23561,'JMA' from dual union all
select 23562,'MRN' from dual union all
select 23563,'MRN' from dual union all
select 23564,'JMA' from dual union all
select 23565,'UKMO' from dual union all
select 23566,'UKMO' from dual union all
select 23567,'UKMO' from dual union all
select 23568,'UKMO' from dual union all
select 23569,'UKMO' from dual union all
select 2357,'USA' from dual union all
select 23570,'UKMO' from dual union all
select 23571,'UKMO' from dual union all
select 23572,'UKMO' from dual union all
select 23573,'UKMO' from dual union all
select 23574,'MRN' from dual union all
select 23575,'UKMO' from dual union all
select 23576,'UKMO' from dual union all
select 23577,'UKMO' from dual union all
select 23578,'MRN' from dual union all
select 23579,'MRN' from dual union all
select 2358,'USA' from dual union all
select 23580,'MRN' from dual union all
select 23581,'MRN' from dual union all
select 23582,'MRN' from dual union all
select 23583,'MRN' from dual union all
select 23584,'MRN' from dual union all
select 23585,'MRN' from dual union all
select 23586,'MRN' from dual union all
select 23587,'MRN' from dual union all
select 23588,'MRN' from dual union all
select 23589,'MRN' from dual union all
select 2359,'USA' from dual union all
select 23590,'MRN' from dual union all
select 23591,'MRN' from dual union all
select 23592,'MRN' from dual union all
select 23593,'MRN' from dual union all
select 23594,'MRN' from dual union all
select 23595,'MRN' from dual union all
select 23596,'MRN' from dual union all
select 23597,'MRN' from dual union all
select 23598,'MRN' from dual union all
select 23599,'MRN' from dual union all
select 236,'WEHR' from dual union all
select 2360,'USA' from dual union all
select 23600,'MRN' from dual union all
select 23601,'MRN' from dual union all
select 23602,'MRN' from dual union all
select 23603,'MRN' from dual union all
select 23604,'MRN' from dual union all
select 23605,'MRN' from dual union all
select 23606,'MRN' from dual union all
select 23607,'MRN' from dual union all
select 23608,'MRN' from dual union all
select 23609,'MRN' from dual union all
select 2361,'USA' from dual union all
select 23610,'MRN' from dual union all
select 23611,'MRN' from dual union all
select 23612,'MRN' from dual union all
select 23613,'MRN' from dual union all
select 23614,'MRN' from dual union all
select 23615,'MRN' from dual union all
select 23616,'MRN' from dual union all
select 23617,'MRN' from dual union all
select 23618,'MRN' from dual union all
select 23619,'MRN' from dual union all
select 2362,'USA' from dual union all
select 23620,'MRN' from dual union all
select 23621,'MRN' from dual union all
select 23622,'MRN' from dual;
insert into LAUNCH_AGENCY
select 23623,'MRN' from dual union all
select 23624,'MRN' from dual union all
select 23625,'MRN' from dual union all
select 23626,'MRN' from dual union all
select 23627,'MRN' from dual union all
select 23628,'INTA' from dual union all
select 23629,'MRN' from dual union all
select 2363,'ORDRSA' from dual union all
select 23630,'MRN' from dual union all
select 23631,'SUPA' from dual union all
select 23632,'MRN' from dual union all
select 23633,'MRN' from dual union all
select 23634,'MRN' from dual union all
select 23635,'MRN' from dual union all
select 23636,'MRN' from dual union all
select 23637,'MRN' from dual union all
select 23638,'MRN' from dual union all
select 23639,'MRN' from dual union all
select 2364,'ORDRSA' from dual union all
select 23640,'MRN' from dual union all
select 23641,'MRN' from dual union all
select 23642,'MRN' from dual union all
select 23643,'MRN' from dual union all
select 23644,'MRN' from dual union all
select 23645,'MRN' from dual union all
select 23646,'MRN' from dual union all
select 23647,'MRN' from dual union all
select 23648,'MRN' from dual union all
select 23649,'MRN' from dual union all
select 2365,'USAF' from dual union all
select 23650,'MRN' from dual union all
select 23651,'MRN' from dual union all
select 23652,'MRN' from dual union all
select 23653,'MRN' from dual union all
select 23654,'MRN' from dual union all
select 23655,'MRN' from dual union all
select 23656,'MRN' from dual union all
select 23657,'MRN' from dual union all
select 23658,'MRN' from dual union all
select 23659,'MRN' from dual union all
select 2366,'USA' from dual union all
select 23660,'MRN' from dual union all
select 23661,'MRN' from dual union all
select 23662,'MRN' from dual union all
select 23663,'MRN' from dual union all
select 23664,'MRN' from dual union all
select 23665,'MRN' from dual union all
select 23666,'MRN' from dual union all
select 23667,'MRN' from dual union all
select 23668,'MRN' from dual union all
select 23669,'MRN' from dual union all
select 2367,'USAF' from dual union all
select 23670,'MRN' from dual union all
select 23671,'MRN' from dual union all
select 23672,'MRN' from dual union all
select 23673,'MRN' from dual union all
select 23674,'MRN' from dual union all
select 23675,'MRN' from dual union all
select 23676,'MRN' from dual union all
select 23677,'MRN' from dual union all
select 23678,'MRN' from dual union all
select 23679,'MRN' from dual union all
select 2368,'ORDRSA' from dual union all
select 23680,'MRN' from dual union all
select 23681,'MRN' from dual union all
select 23682,'MRN' from dual union all
select 23683,'MRN' from dual union all
select 23684,'NASA' from dual union all
select 23685,'MRN' from dual union all
select 23686,'MRN' from dual union all
select 23687,'MRN' from dual union all
select 23688,'MRN' from dual union all
select 23689,'MRN' from dual union all
select 2369,'USA' from dual union all
select 23690,'MRN' from dual union all
select 23691,'INTA' from dual union all
select 23692,'MRN' from dual union all
select 23693,'MRN' from dual union all
select 23694,'MRN' from dual union all
select 23695,'INTA' from dual union all
select 23696,'MRN' from dual union all
select 23697,'MRN' from dual union all
select 23698,'MRN' from dual union all
select 23699,'MRN' from dual union all
select 237,'WER836' from dual union all
select 2370,'USA' from dual union all
select 23700,'MRN' from dual union all
select 23701,'MRN' from dual union all
select 23702,'MRN' from dual union all
select 23703,'MRN' from dual union all
select 23704,'MRN' from dual union all
select 23705,'MRN' from dual union all
select 23706,'MRN' from dual union all
select 23707,'MRN' from dual union all
select 23708,'MRN' from dual union all
select 23709,'MRN' from dual union all
select 2371,'USA' from dual union all
select 23710,'MRN' from dual union all
select 23711,'MRN' from dual union all
select 23712,'MRN' from dual;
insert into LAUNCH_AGENCY
select 23713,'MRN' from dual union all
select 23714,'MRN' from dual union all
select 23715,'MRN' from dual union all
select 23716,'MRN' from dual union all
select 23717,'MRN' from dual union all
select 23718,'MRN' from dual union all
select 23719,'MRN' from dual union all
select 2372,'ORDRSA' from dual union all
select 23720,'USAF' from dual union all
select 23721,'MRN' from dual union all
select 23722,'MRN' from dual union all
select 23723,'MRN' from dual union all
select 23724,'NASA' from dual union all
select 23725,'MRN' from dual union all
select 23726,'MRN' from dual union all
select 23727,'MRN' from dual union all
select 23728,'MRN' from dual union all
select 23729,'MRN' from dual union all
select 2373,'ORDRSA' from dual union all
select 23730,'MRN' from dual union all
select 23731,'MRN' from dual union all
select 23732,'MRN' from dual union all
select 23733,'MRN' from dual union all
select 23734,'MRN' from dual union all
select 23735,'MRN' from dual union all
select 23736,'INTA' from dual union all
select 23737,'MRN' from dual union all
select 23738,'MRN' from dual union all
select 23739,'MRN' from dual union all
select 2374,'USA' from dual union all
select 23740,'SRC' from dual union all
select 23741,'MRN' from dual union all
select 23742,'MRN' from dual union all
select 23743,'MRN' from dual union all
select 23744,'MRN' from dual union all
select 23745,'MRN' from dual union all
select 23746,'MRN' from dual union all
select 23747,'MRN' from dual union all
select 23748,'MRN' from dual union all
select 23749,'MRN' from dual union all
select 2375,'USA' from dual union all
select 23750,'NASA' from dual union all
select 23751,'MRN' from dual union all
select 23752,'ISRO' from dual union all
select 23753,'MRN' from dual union all
select 23754,'MRN' from dual union all
select 23755,'MRN' from dual union all
select 23756,'MRN' from dual union all
select 23757,'MRN' from dual union all
select 23758,'NASA' from dual union all
select 23759,'NASA' from dual union all
select 2376,'USA' from dual union all
select 23760,'NASA' from dual union all
select 23761,'MRN' from dual union all
select 23762,'NASA' from dual union all
select 23763,'NASA' from dual union all
select 23764,'NASA' from dual union all
select 23765,'NASA' from dual union all
select 23766,'NASA' from dual union all
select 23767,'ISRO' from dual union all
select 23768,'MRN' from dual union all
select 23769,'MRN' from dual union all
select 2377,'USA' from dual union all
select 23770,'MRN' from dual union all
select 23771,'MRN' from dual union all
select 23772,'MRN' from dual union all
select 23773,'MRN' from dual union all
select 23774,'MRN' from dual union all
select 23775,'MRN' from dual union all
select 23776,'MRN' from dual union all
select 23777,'ISRO' from dual union all
select 23778,'MRN' from dual union all
select 23779,'MRN' from dual union all
select 2378,'USA' from dual union all
select 23780,'MRN' from dual union all
select 23781,'MRN' from dual union all
select 23782,'SUPA' from dual union all
select 23783,'MRN' from dual union all
select 23784,'MRN' from dual union all
select 23785,'SUPA' from dual union all
select 23786,'MRN' from dual union all
select 23787,'MRN' from dual union all
select 23788,'MRN' from dual union all
select 23789,'MRN' from dual union all
select 2379,'USA' from dual union all
select 23790,'MRN' from dual union all
select 23791,'MRN' from dual union all
select 23792,'ISRO' from dual union all
select 23793,'MRN' from dual union all
select 23794,'MRN' from dual union all
select 23795,'MRN' from dual union all
select 23796,'MRN' from dual union all
select 23797,'MRN' from dual union all
select 23798,'MRN' from dual union all
select 23799,'MRN' from dual union all
select 238,'WER836' from dual union all
select 2380,'USA' from dual union all
select 23800,'MRN' from dual union all
select 23801,'MRN' from dual union all
select 23802,'MRN' from dual;
insert into LAUNCH_AGENCY
select 23803,'MRN' from dual union all
select 23804,'MRN' from dual union all
select 23805,'MRN' from dual union all
select 23806,'ISRO' from dual union all
select 23807,'ISRO' from dual union all
select 23808,'MRN' from dual union all
select 23809,'MRN' from dual union all
select 2381,'USAF' from dual union all
select 23810,'MRN' from dual union all
select 23811,'MRN' from dual union all
select 23812,'MRN' from dual union all
select 23813,'ISRO' from dual union all
select 23814,'MRN' from dual union all
select 23815,'MRN' from dual union all
select 23816,'MRN' from dual union all
select 23817,'MRN' from dual union all
select 23818,'ISRO' from dual union all
select 23819,'MRN' from dual union all
select 2382,'USA' from dual union all
select 23820,'MRN' from dual union all
select 23821,'MRN' from dual union all
select 23822,'MRN' from dual union all
select 23823,'MRN' from dual union all
select 23824,'MRN' from dual union all
select 23825,'MRN' from dual union all
select 23826,'MRN' from dual union all
select 23827,'MRN' from dual union all
select 23828,'MSFC' from dual union all
select 23829,'MRN' from dual union all
select 2383,'USA' from dual union all
select 23830,'ISRO' from dual union all
select 23831,'MRN' from dual union all
select 23832,'MRN' from dual union all
select 23833,'MSFC' from dual union all
select 23834,'MRN' from dual union all
select 23835,'ISRO' from dual union all
select 23836,'MRN' from dual union all
select 23837,'MRN' from dual union all
select 23838,'MRN' from dual union all
select 23839,'MRN' from dual union all
select 2384,'USA' from dual union all
select 23840,'MSFC' from dual union all
select 23841,'MRN' from dual union all
select 23842,'MRN' from dual union all
select 23843,'MRN' from dual union all
select 23844,'MRN' from dual union all
select 23845,'MRN' from dual union all
select 23846,'MRN' from dual union all
select 23847,'MRN' from dual union all
select 23848,'MRN' from dual union all
select 23849,'MRN' from dual union all
select 2385,'USA' from dual union all
select 23850,'MRN' from dual union all
select 23851,'MRN' from dual union all
select 23852,'MRN' from dual union all
select 23853,'MRN' from dual union all
select 23854,'MSFC' from dual union all
select 23855,'MRN' from dual union all
select 23856,'MRN' from dual union all
select 23857,'MRN' from dual union all
select 23858,'MRN' from dual union all
select 23859,'MRN' from dual union all
select 2386,'PARD' from dual union all
select 23860,'MRN' from dual union all
select 23861,'MRN' from dual union all
select 23862,'MRN' from dual union all
select 23863,'MRN' from dual union all
select 23864,'NASA' from dual union all
select 23865,'MRN' from dual union all
select 23866,'MRN' from dual union all
select 23867,'MRN' from dual union all
select 23868,'MSFC' from dual union all
select 23869,'MRN' from dual union all
select 2387,'PARD' from dual union all
select 23870,'MRN' from dual union all
select 23871,'MRN' from dual union all
select 23872,'SUPA' from dual union all
select 23873,'MRN' from dual union all
select 23874,'MRN' from dual union all
select 23875,'MRN' from dual union all
select 23876,'MRN' from dual union all
select 23877,'MRN' from dual union all
select 23878,'MSFC' from dual union all
select 23879,'MRN' from dual union all
select 2388,'USA' from dual union all
select 23880,'MRN' from dual union all
select 23881,'MRN' from dual union all
select 23882,'MRN' from dual union all
select 23883,'MRN' from dual union all
select 23884,'MRN' from dual union all
select 23885,'MRN' from dual union all
select 23886,'MRN' from dual union all
select 23887,'MRN' from dual union all
select 23888,'MRN' from dual union all
select 23889,'MRN' from dual union all
select 2389,'USA' from dual union all
select 23890,'MRN' from dual union all
select 23891,'MRN' from dual union all
select 23892,'MRN' from dual union all
select 23893,'MRN' from dual;
insert into LAUNCH_AGENCY
select 23894,'MRN' from dual union all
select 23895,'MRN' from dual union all
select 23896,'MRN' from dual union all
select 23897,'MRN' from dual union all
select 23898,'MRN' from dual union all
select 23899,'MRN' from dual union all
select 239,'WER271-1' from dual union all
select 2390,'BRL' from dual union all
select 23900,'MRN' from dual union all
select 23901,'MRN' from dual union all
select 23902,'MRN' from dual union all
select 23903,'MRN' from dual union all
select 23904,'MRN' from dual union all
select 23905,'MRN' from dual union all
select 23906,'MRN' from dual union all
select 23907,'MRN' from dual union all
select 23908,'MSFC' from dual union all
select 23909,'MRN' from dual union all
select 2391,'USAF' from dual union all
select 23910,'MRN' from dual union all
select 23911,'MRN' from dual union all
select 23912,'MRN' from dual union all
select 23913,'MRN' from dual union all
select 23914,'MSFC' from dual union all
select 23915,'SRC' from dual union all
select 23916,'MRN' from dual union all
select 23917,'MRN' from dual union all
select 23918,'MRN' from dual union all
select 23919,'MRN' from dual union all
select 2392,'USA' from dual union all
select 23920,'MSFC' from dual union all
select 23921,'MRN' from dual union all
select 23922,'MRN' from dual union all
select 23923,'MRN' from dual union all
select 23924,'MRN' from dual union all
select 23925,'MRN' from dual union all
select 23926,'MRN' from dual union all
select 23927,'MRN' from dual union all
select 23928,'MRN' from dual union all
select 23929,'MSFC' from dual union all
select 2393,'USA' from dual union all
select 23930,'MRN' from dual union all
select 23931,'MRN' from dual union all
select 23932,'MRN' from dual union all
select 23933,'MRN' from dual union all
select 23934,'MRN' from dual union all
select 23935,'MRN' from dual union all
select 23936,'MRN' from dual union all
select 23937,'MRN' from dual union all
select 23938,'MRN' from dual union all
select 23939,'MRN' from dual union all
select 2394,'ORDRSA' from dual union all
select 23940,'MRN' from dual union all
select 23941,'MRN' from dual union all
select 23942,'MRN' from dual union all
select 23943,'MRN' from dual union all
select 23944,'MRN' from dual union all
select 23945,'MRN' from dual union all
select 23946,'MRN' from dual union all
select 23947,'MRN' from dual union all
select 23948,'MRN' from dual union all
select 23949,'MRN' from dual union all
select 2395,'ORDRSA' from dual union all
select 23950,'MRN' from dual union all
select 23951,'MRN' from dual union all
select 23952,'MRN' from dual union all
select 23953,'MRN' from dual union all
select 23954,'MRN' from dual union all
select 23955,'MRN' from dual union all
select 23956,'MRN' from dual union all
select 23957,'MRN' from dual union all
select 23958,'MRN' from dual union all
select 23959,'MRN' from dual union all
select 2396,'USAF' from dual union all
select 23960,'SUPA' from dual union all
select 23961,'MRN' from dual union all
select 23962,'SUPA' from dual union all
select 23963,'MRN' from dual union all
select 23964,'MSFC' from dual union all
select 23965,'MRN' from dual union all
select 23966,'MRN' from dual union all
select 23967,'MRN' from dual union all
select 23968,'MRN' from dual union all
select 23969,'MRN' from dual union all
select 2397,'USA' from dual union all
select 23970,'MRN' from dual union all
select 23971,'MRN' from dual union all
select 23972,'MRN' from dual union all
select 23973,'NASA' from dual union all
select 23974,'MRN' from dual union all
select 23975,'MSFC' from dual union all
select 23976,'MRN' from dual union all
select 23977,'MRN' from dual union all
select 23978,'MRN' from dual union all
select 23979,'MRN' from dual union all
select 2398,'ORDRSA' from dual union all
select 23980,'MRN' from dual union all
select 23981,'MRN' from dual union all
select 23982,'MSFC' from dual union all
select 23983,'MRN' from dual;
insert into LAUNCH_AGENCY
select 23984,'MRN' from dual union all
select 23985,'MRN' from dual union all
select 23986,'MRN' from dual union all
select 23987,'MRN' from dual union all
select 23988,'MRN' from dual union all
select 23989,'MRN' from dual union all
select 2399,'USA' from dual union all
select 23990,'MRN' from dual union all
select 23991,'MRN' from dual union all
select 23992,'MRN' from dual union all
select 23993,'MSFC' from dual union all
select 23994,'MRN' from dual union all
select 23995,'MRN' from dual union all
select 23996,'MRN' from dual union all
select 23997,'MRN' from dual union all
select 23998,'MRN' from dual union all
select 23999,'MRN' from dual union all
select 24,'WEHR' from dual union all
select 240,'WER271-1' from dual union all
select 2400,'USA' from dual union all
select 24000,'MRN' from dual union all
select 24001,'MRN' from dual union all
select 24002,'MRN' from dual union all
select 24003,'MRN' from dual union all
select 24004,'MRN' from dual union all
select 24005,'MRN' from dual union all
select 24006,'MRN' from dual union all
select 24007,'MRN' from dual union all
select 24008,'MRN' from dual union all
select 24009,'MRN' from dual union all
select 2401,'USA' from dual union all
select 24010,'MRN' from dual union all
select 24011,'MRN' from dual union all
select 24012,'MRN' from dual union all
select 24013,'MRN' from dual union all
select 24014,'MRN' from dual union all
select 24015,'MRN' from dual union all
select 24016,'MRN' from dual union all
select 24017,'MRN' from dual union all
select 24018,'MRN' from dual union all
select 24019,'MRN' from dual union all
select 2402,'USA' from dual union all
select 24020,'MRN' from dual union all
select 24021,'MRN' from dual union all
select 24022,'MRN' from dual union all
select 24023,'MRN' from dual union all
select 24024,'MRN' from dual union all
select 24025,'MRN' from dual union all
select 24026,'MRN' from dual union all
select 24027,'MRN' from dual union all
select 24028,'MRN' from dual union all
select 24029,'MRN' from dual union all
select 2403,'USA' from dual union all
select 24030,'MRN' from dual union all
select 24031,'MRN' from dual union all
select 24032,'MRN' from dual union all
select 24033,'MRN' from dual union all
select 24034,'MRN' from dual union all
select 24035,'MRN' from dual union all
select 24036,'MRN' from dual union all
select 24037,'MRN' from dual union all
select 24038,'MRN' from dual union all
select 24039,'MRN' from dual union all
select 2404,'USAF' from dual union all
select 24040,'MRN' from dual union all
select 24041,'MRN' from dual union all
select 24042,'MRN' from dual union all
select 24043,'MRN' from dual union all
select 24044,'MRN' from dual union all
select 24045,'MRN' from dual union all
select 24046,'MRN' from dual union all
select 24047,'MRN' from dual union all
select 24048,'MRN' from dual union all
select 24049,'MRN' from dual union all
select 2405,'USA' from dual union all
select 24050,'MRN' from dual union all
select 24051,'MRN' from dual union all
select 24052,'MRN' from dual union all
select 24053,'MRN' from dual union all
select 24054,'MRN' from dual union all
select 24055,'MRN' from dual union all
select 24056,'MRN' from dual union all
select 24057,'MRN' from dual union all
select 24058,'SRC' from dual union all
select 24059,'MRN' from dual union all
select 2406,'USA' from dual union all
select 24060,'MRN' from dual union all
select 24061,'MRN' from dual union all
select 24062,'MRN' from dual union all
select 24063,'MRN' from dual union all
select 24064,'MRN' from dual union all
select 24065,'MRN' from dual union all
select 24066,'MRN' from dual union all
select 24067,'MRN' from dual union all
select 24068,'MRN' from dual union all
select 24069,'MRN' from dual union all
select 2407,'USA' from dual union all
select 24070,'MRN' from dual union all
select 24071,'MRN' from dual union all
select 24072,'MRN' from dual;
insert into LAUNCH_AGENCY
select 24073,'MRN' from dual union all
select 24074,'MRN' from dual union all
select 24075,'MRN' from dual union all
select 24076,'MRN' from dual union all
select 24077,'MRN' from dual union all
select 24078,'INTA' from dual union all
select 24079,'MRN' from dual union all
select 2408,'USA' from dual union all
select 24080,'MRN' from dual union all
select 24081,'MRN' from dual union all
select 24082,'MRN' from dual union all
select 24083,'MRN' from dual union all
select 24084,'MRN' from dual union all
select 24085,'MRN' from dual union all
select 24086,'MRN' from dual union all
select 24087,'MRN' from dual union all
select 24088,'MRN' from dual union all
select 24089,'MRN' from dual union all
select 2409,'ORDRSA' from dual union all
select 24090,'MRN' from dual union all
select 24091,'MRN' from dual union all
select 24092,'MRN' from dual union all
select 24093,'MRN' from dual union all
select 24094,'MRN' from dual union all
select 24095,'MRN' from dual union all
select 24096,'MRN' from dual union all
select 24097,'MRN' from dual union all
select 24098,'MRN' from dual union all
select 24099,'MRN' from dual union all
select 241,'WER271-1' from dual union all
select 2410,'USA' from dual union all
select 24100,'MRN' from dual union all
select 24101,'MRN' from dual union all
select 24102,'MRN' from dual union all
select 24103,'MRN' from dual union all
select 24104,'MRN' from dual union all
select 24105,'MRN' from dual union all
select 24106,'MRN' from dual union all
select 24107,'MRN' from dual union all
select 24108,'MRN' from dual union all
select 24109,'MRN' from dual union all
select 2411,'USA' from dual union all
select 24110,'MRN' from dual union all
select 24111,'MRN' from dual union all
select 24112,'MRN' from dual union all
select 24113,'MRN' from dual union all
select 24114,'INTA' from dual union all
select 24115,'MRN' from dual union all
select 24116,'MRN' from dual union all
select 24117,'MRN' from dual union all
select 24118,'MRN' from dual union all
select 24119,'MRN' from dual union all
select 2412,'USA' from dual union all
select 24120,'MRN' from dual union all
select 24121,'MRN' from dual union all
select 24122,'MRN' from dual union all
select 24123,'MRN' from dual union all
select 24124,'MRN' from dual union all
select 24125,'MRN' from dual union all
select 24126,'MRN' from dual union all
select 24127,'MRN' from dual union all
select 24128,'MRN' from dual union all
select 24129,'NASA' from dual union all
select 2413,'USA' from dual union all
select 24130,'MRN' from dual union all
select 24131,'MRN' from dual union all
select 24132,'MRN' from dual union all
select 24133,'MRN' from dual union all
select 24134,'MRN' from dual union all
select 24135,'MRN' from dual union all
select 24136,'MRN' from dual union all
select 24137,'MRN' from dual union all
select 24138,'MRN' from dual union all
select 24139,'MRN' from dual union all
select 2414,'USA' from dual union all
select 24140,'MRN' from dual union all
select 24141,'MRN' from dual union all
select 24142,'MRN' from dual union all
select 24143,'MRN' from dual union all
select 24144,'MRN' from dual union all
select 24145,'MRN' from dual union all
select 24146,'INTA' from dual union all
select 24147,'MRN' from dual union all
select 24148,'MRN' from dual union all
select 24149,'MRN' from dual union all
select 2415,'USA' from dual union all
select 24150,'MRN' from dual union all
select 24151,'MRN' from dual union all
select 24152,'MRN' from dual union all
select 24153,'MRN' from dual union all
select 24154,'MRN' from dual union all
select 24155,'MRN' from dual union all
select 24156,'MRN' from dual union all
select 24157,'MRN' from dual union all
select 24158,'MRN' from dual union all
select 24159,'MRN' from dual union all
select 2416,'USA' from dual union all
select 24160,'MRN' from dual union all
select 24161,'INTA' from dual union all
select 24162,'MRN' from dual;
insert into LAUNCH_AGENCY
select 24163,'MRN' from dual union all
select 24164,'MRN' from dual union all
select 24165,'MRN' from dual union all
select 24166,'MRN' from dual union all
select 24167,'MRN' from dual union all
select 24168,'SRC' from dual union all
select 24169,'MRN' from dual union all
select 2417,'USA' from dual union all
select 24170,'MRN' from dual union all
select 24171,'MRN' from dual union all
select 24172,'MRN' from dual union all
select 24173,'MRN' from dual union all
select 24174,'MRN' from dual union all
select 24175,'MRN' from dual union all
select 24176,'MRN' from dual union all
select 24177,'MRN' from dual union all
select 24178,'MRN' from dual union all
select 24179,'MRN' from dual union all
select 2418,'USA' from dual union all
select 24180,'MRN' from dual union all
select 24181,'MRN' from dual union all
select 24182,'MRN' from dual union all
select 24183,'MRN' from dual union all
select 24184,'MRN' from dual union all
select 24185,'MRN' from dual union all
select 24186,'MRN' from dual union all
select 24187,'MRN' from dual union all
select 24188,'MRN' from dual union all
select 24189,'MRN' from dual union all
select 2419,'ORDRSA' from dual union all
select 24190,'MRN' from dual union all
select 24191,'MRN' from dual union all
select 24192,'MRN' from dual union all
select 24193,'US' from dual union all
select 24194,'MRN' from dual union all
select 24195,'MRN' from dual union all
select 24196,'MRN' from dual union all
select 24197,'MRN' from dual union all
select 24198,'MRN' from dual union all
select 24199,'MRN' from dual union all
select 242,'WEHR' from dual union all
select 2420,'USAF' from dual union all
select 24200,'MRN' from dual union all
select 24201,'MRN' from dual union all
select 24202,'US' from dual union all
select 24203,'MRN' from dual union all
select 24204,'MRN' from dual union all
select 24205,'MRN' from dual union all
select 24206,'US' from dual union all
select 24207,'US' from dual union all
select 24208,'MRN' from dual union all
select 24209,'MRN' from dual union all
select 2421,'USA' from dual union all
select 24210,'US' from dual union all
select 24211,'MRN' from dual union all
select 24212,'MRN' from dual union all
select 24213,'US' from dual union all
select 24214,'MRN' from dual union all
select 24215,'MRN' from dual union all
select 24216,'MRN' from dual union all
select 24217,'INTA' from dual union all
select 24218,'MRN' from dual union all
select 24219,'MRN' from dual union all
select 2422,'USA' from dual union all
select 24220,'MRN' from dual union all
select 24221,'MRN' from dual union all
select 24222,'MRN' from dual union all
select 24223,'MRN' from dual union all
select 24224,'MRN' from dual union all
select 24225,'MRN' from dual union all
select 24226,'MRN' from dual union all
select 24227,'MRN' from dual union all
select 24228,'MRN' from dual union all
select 24229,'MRN' from dual union all
select 2423,'ORDRSA' from dual union all
select 24230,'MRN' from dual union all
select 24231,'MRN' from dual union all
select 24232,'MRN' from dual union all
select 24233,'MRN' from dual union all
select 24234,'MRN' from dual union all
select 24235,'MRN' from dual union all
select 24236,'MRN' from dual union all
select 24237,'MRN' from dual union all
select 24238,'MRN' from dual union all
select 24239,'MRN' from dual union all
select 2424,'USA' from dual union all
select 24240,'MRN' from dual union all
select 24241,'MRN' from dual union all
select 24242,'MRN' from dual union all
select 24243,'MRN' from dual union all
select 24244,'MRN' from dual union all
select 24245,'MRN' from dual union all
select 24246,'MRN' from dual union all
select 24247,'MRN' from dual union all
select 24248,'INTA' from dual union all
select 24249,'MRN' from dual union all
select 2425,'SUI' from dual union all
select 24250,'MRN' from dual union all
select 24251,'MRN' from dual union all
select 24252,'MRN' from dual;
insert into LAUNCH_AGENCY
select 24253,'MRN' from dual union all
select 24254,'MRN' from dual union all
select 24255,'MRN' from dual union all
select 24256,'MRN' from dual union all
select 24257,'MRN' from dual union all
select 24258,'MRN' from dual union all
select 24259,'MRN' from dual union all
select 2426,'USA' from dual union all
select 24260,'MRN' from dual union all
select 24261,'MRN' from dual union all
select 24262,'MRN' from dual union all
select 24263,'MRN' from dual union all
select 24264,'MRN' from dual union all
select 24265,'MRN' from dual union all
select 24266,'MRN' from dual union all
select 24267,'INTA' from dual union all
select 24268,'MRN' from dual union all
select 24269,'MRN' from dual union all
select 2427,'USA' from dual union all
select 24270,'MRN' from dual union all
select 24271,'MRN' from dual union all
select 24272,'MRN' from dual union all
select 24273,'MRN' from dual union all
select 24274,'MRN' from dual union all
select 24275,'MRN' from dual union all
select 24276,'MRN' from dual union all
select 24277,'MRN' from dual union all
select 24278,'SUPA' from dual union all
select 24279,'MRN' from dual union all
select 2428,'NRL' from dual union all
select 24280,'MRN' from dual union all
select 24281,'MRN' from dual union all
select 24282,'MRN' from dual union all
select 24283,'MRN' from dual union all
select 24284,'MRN' from dual union all
select 24285,'MRN' from dual union all
select 24286,'MRN' from dual union all
select 24287,'MRN' from dual union all
select 24288,'MRN' from dual union all
select 24289,'SUPA' from dual union all
select 2429,'SUI' from dual union all
select 24290,'MRN' from dual union all
select 24291,'MRN' from dual union all
select 24292,'INTA' from dual union all
select 24293,'MRN' from dual union all
select 24294,'MRN' from dual union all
select 24295,'MRN' from dual union all
select 24296,'MRN' from dual union all
select 24297,'MRN' from dual union all
select 24298,'MRN' from dual union all
select 24299,'MRN' from dual union all
select 243,'WER271-1' from dual union all
select 2430,'ORDRSA' from dual union all
select 24300,'MRN' from dual union all
select 24301,'MRN' from dual union all
select 24302,'MRN' from dual union all
select 24303,'MRN' from dual union all
select 24304,'MRN' from dual union all
select 24305,'MRN' from dual union all
select 24306,'MRN' from dual union all
select 24307,'MRN' from dual union all
select 24308,'MRN' from dual union all
select 24309,'MRN' from dual union all
select 2431,'PARD' from dual union all
select 24310,'MRN' from dual union all
select 24311,'SUPA' from dual union all
select 24312,'MRN' from dual union all
select 24313,'MRN' from dual union all
select 24314,'MRN' from dual union all
select 24315,'MRN' from dual union all
select 24316,'MRN' from dual union all
select 24317,'MRN' from dual union all
select 24318,'MRN' from dual union all
select 24319,'MRN' from dual union all
select 2432,'ORDRSA' from dual union all
select 24320,'MRN' from dual union all
select 24321,'MRN' from dual union all
select 24322,'MRN' from dual union all
select 24323,'MRN' from dual union all
select 24324,'MRN' from dual union all
select 24325,'MRN' from dual union all
select 24326,'MRN' from dual union all
select 24327,'MRN' from dual union all
select 24328,'MRN' from dual union all
select 24329,'MRN' from dual union all
select 2433,'GEMTS' from dual union all
select 24330,'MRN' from dual union all
select 24331,'MRN' from dual union all
select 24332,'INTA' from dual union all
select 24333,'MRN' from dual union all
select 24334,'MRN' from dual union all
select 24335,'SUPA' from dual union all
select 24336,'MRN' from dual union all
select 24337,'MRN' from dual union all
select 24338,'MRN' from dual union all
select 24339,'MRN' from dual union all
select 2434,'USAF' from dual union all
select 24340,'MRN' from dual union all
select 24341,'MRN' from dual union all
select 24342,'MRN' from dual;
insert into LAUNCH_AGENCY
select 24343,'MRN' from dual union all
select 24344,'MRN' from dual union all
select 24345,'MRN' from dual union all
select 24346,'MRN' from dual union all
select 24347,'MRN' from dual union all
select 24348,'MRN' from dual union all
select 24349,'MRN' from dual union all
select 2435,'USA' from dual union all
select 24350,'MRN' from dual union all
select 24351,'MRN' from dual union all
select 24352,'MRN' from dual union all
select 24353,'MRN' from dual union all
select 24354,'MRN' from dual union all
select 24355,'MRN' from dual union all
select 24356,'MRN' from dual union all
select 24357,'MRN' from dual union all
select 24358,'NASA' from dual union all
select 24359,'MRN' from dual union all
select 2436,'USA' from dual union all
select 24360,'MRN' from dual union all
select 24361,'MRN' from dual union all
select 24362,'MRN' from dual union all
select 24363,'MRN' from dual union all
select 24364,'MRN' from dual union all
select 24365,'MRN' from dual union all
select 24366,'MRN' from dual union all
select 24367,'MRN' from dual union all
select 24368,'MRN' from dual union all
select 24369,'MRN' from dual union all
select 2437,'USA' from dual union all
select 24370,'MRN' from dual union all
select 24371,'MRN' from dual union all
select 24372,'MRN' from dual union all
select 24373,'MRN' from dual union all
select 24374,'MRN' from dual union all
select 24375,'MRN' from dual union all
select 24376,'MRN' from dual union all
select 24377,'NASA' from dual union all
select 24378,'MRN' from dual union all
select 24379,'MRN' from dual union all
select 2438,'USA' from dual union all
select 24380,'MRN' from dual union all
select 24381,'MRN' from dual union all
select 24382,'MRN' from dual union all
select 24383,'MRN' from dual union all
select 24384,'MRN' from dual union all
select 24385,'INTA' from dual union all
select 24386,'MRN' from dual union all
select 24387,'MRN' from dual union all
select 24388,'MRN' from dual union all
select 24389,'MRN' from dual union all
select 2439,'USAF' from dual union all
select 24390,'MRN' from dual union all
select 24391,'MRN' from dual union all
select 24392,'MRN' from dual union all
select 24393,'MRN' from dual union all
select 24394,'MRN' from dual union all
select 24395,'MRN' from dual union all
select 24396,'MRN' from dual union all
select 24397,'MRN' from dual union all
select 24398,'MRN' from dual union all
select 24399,'MRN' from dual union all
select 244,'WER271-1' from dual union all
select 2440,'SUI' from dual union all
select 24400,'MRN' from dual union all
select 24401,'MRN' from dual union all
select 24402,'MRN' from dual union all
select 24403,'MRN' from dual union all
select 24404,'MRN' from dual union all
select 24405,'MRN' from dual union all
select 24406,'MRN' from dual union all
select 24407,'MRN' from dual union all
select 24408,'MRN' from dual union all
select 24409,'MRN' from dual union all
select 2441,'USA' from dual union all
select 24410,'MRN' from dual union all
select 24411,'MRN' from dual union all
select 24412,'MRN' from dual union all
select 24413,'MRN' from dual union all
select 24414,'MRN' from dual union all
select 24415,'MRN' from dual union all
select 24416,'MRN' from dual union all
select 24417,'MRN' from dual union all
select 24418,'MRN' from dual union all
select 24419,'MRN' from dual union all
select 2442,'USAF' from dual union all
select 24420,'MRN' from dual union all
select 24421,'MRN' from dual union all
select 24422,'MRN' from dual union all
select 24423,'INTA' from dual union all
select 24424,'MRN' from dual union all
select 24425,'MRN' from dual union all
select 24426,'MRN' from dual union all
select 24427,'MRN' from dual union all
select 24428,'MRN' from dual union all
select 24429,'MRN' from dual union all
select 2443,'USAF' from dual union all
select 24430,'MRN' from dual union all
select 24431,'MRN' from dual union all
select 24432,'MRN' from dual;
insert into LAUNCH_AGENCY
select 24433,'MRN' from dual union all
select 24434,'MRN' from dual union all
select 24435,'MRN' from dual union all
select 24436,'MRN' from dual union all
select 24437,'MRN' from dual union all
select 24438,'MRN' from dual union all
select 24439,'MRN' from dual union all
select 2444,'USA' from dual union all
select 24440,'MRN' from dual union all
select 24441,'MRN' from dual union all
select 24442,'MRN' from dual union all
select 24443,'MRN' from dual union all
select 24444,'MRN' from dual union all
select 24445,'MRN' from dual union all
select 24446,'MRN' from dual union all
select 24447,'MRN' from dual union all
select 24448,'MRN' from dual union all
select 24449,'MRN' from dual union all
select 2445,'USA' from dual union all
select 24450,'MRN' from dual union all
select 24451,'MRN' from dual union all
select 24452,'MRN' from dual union all
select 24453,'MRN' from dual union all
select 24454,'NASA' from dual union all
select 24455,'MRN' from dual union all
select 24456,'MRN' from dual union all
select 24457,'MRN' from dual union all
select 24458,'MRN' from dual union all
select 24459,'MRN' from dual union all
select 2446,'USA' from dual union all
select 24460,'MRN' from dual union all
select 24461,'MRN' from dual union all
select 24462,'MRN' from dual union all
select 24463,'MRN' from dual union all
select 24464,'INTA' from dual union all
select 24465,'AN' from dual union all
select 24466,'AN' from dual union all
select 24467,'AN' from dual union all
select 24468,'AN' from dual union all
select 24469,'AN' from dual union all
select 2447,'ORDRSA' from dual union all
select 24470,'MRN' from dual union all
select 24471,'AN' from dual union all
select 24472,'AN' from dual union all
select 24473,'SRC' from dual union all
select 24474,'AN' from dual union all
select 24475,'AN' from dual union all
select 24476,'AN' from dual union all
select 24477,'AN' from dual union all
select 24478,'AN' from dual union all
select 24479,'AN' from dual union all
select 2448,'SUI' from dual union all
select 24480,'AN' from dual union all
select 24481,'MRN' from dual union all
select 24482,'AN' from dual union all
select 24483,'AN' from dual union all
select 24484,'AN' from dual union all
select 24485,'AN' from dual union all
select 24486,'AN' from dual union all
select 24487,'NASA' from dual union all
select 24488,'NASA' from dual union all
select 24489,'NASA' from dual union all
select 2449,'USA' from dual union all
select 24490,'AN' from dual union all
select 24491,'AN' from dual union all
select 24492,'AN' from dual union all
select 24493,'AN' from dual union all
select 24494,'AN' from dual union all
select 24495,'AN' from dual union all
select 24496,'AN' from dual union all
select 24497,'AN' from dual union all
select 24498,'AN' from dual union all
select 24499,'AN' from dual union all
select 245,'WER271-1' from dual union all
select 2450,'USA' from dual union all
select 24500,'AN' from dual union all
select 24501,'AN' from dual union all
select 24502,'AN' from dual union all
select 24503,'MRN' from dual union all
select 24504,'AN' from dual union all
select 24505,'AN' from dual union all
select 24506,'MRN' from dual union all
select 24507,'AN' from dual union all
select 24508,'MRN' from dual union all
select 24509,'MRN' from dual union all
select 2451,'USA' from dual union all
select 24510,'AN' from dual union all
select 24511,'AN' from dual union all
select 24512,'AN' from dual union all
select 24513,'AN' from dual union all
select 24514,'SRC' from dual union all
select 24515,'SRC' from dual union all
select 24516,'AN' from dual union all
select 24517,'SRC' from dual union all
select 24518,'SRC' from dual union all
select 24519,'SRC' from dual union all
select 2452,'USA' from dual union all
select 24520,'AN' from dual union all
select 24521,'AN' from dual union all
select 24522,'AN' from dual;
insert into LAUNCH_AGENCY
select 24523,'SRC' from dual union all
select 24524,'AN' from dual union all
select 24525,'MRN' from dual union all
select 24526,'AN' from dual union all
select 24527,'SRC' from dual union all
select 24528,'AN' from dual union all
select 24529,'AN' from dual union all
select 2453,'USA' from dual union all
select 24530,'AN' from dual union all
select 24531,'AN' from dual union all
select 24532,'SRC' from dual union all
select 24533,'AN' from dual union all
select 24534,'AN' from dual union all
select 24535,'AN' from dual union all
select 24536,'AN' from dual union all
select 24537,'AN' from dual union all
select 24538,'AN' from dual union all
select 24539,'AN' from dual union all
select 2454,'USA' from dual union all
select 24540,'SRC' from dual union all
select 24541,'AN' from dual union all
select 24542,'AN' from dual union all
select 24543,'AN' from dual union all
select 24544,'AN' from dual union all
select 24545,'AN' from dual union all
select 24546,'AN' from dual union all
select 24547,'AN' from dual union all
select 24548,'AN' from dual union all
select 24549,'AN' from dual union all
select 2455,'USA' from dual union all
select 24550,'AN' from dual union all
select 24551,'AN' from dual union all
select 24552,'AN' from dual union all
select 24553,'AN' from dual union all
select 24554,'AN' from dual union all
select 24555,'AN' from dual union all
select 24556,'MRN' from dual union all
select 24557,'AN' from dual union all
select 24558,'AN' from dual union all
select 24559,'AN' from dual union all
select 2456,'NRL' from dual union all
select 24560,'AN' from dual union all
select 24561,'AN' from dual union all
select 24562,'AN' from dual union all
select 24563,'AN' from dual union all
select 24564,'AN' from dual union all
select 24565,'AN' from dual union all
select 24566,'AN' from dual union all
select 24567,'AN' from dual union all
select 24568,'SRC' from dual union all
select 24569,'AN' from dual union all
select 2457,'USA' from dual union all
select 24570,'AN' from dual union all
select 24571,'SRC' from dual union all
select 24572,'AN' from dual union all
select 24573,'AN' from dual union all
select 24574,'AN' from dual union all
select 24575,'AN' from dual union all
select 24576,'AN' from dual union all
select 24577,'AN' from dual union all
select 24578,'AN' from dual union all
select 24579,'AN' from dual union all
select 2458,'PARD' from dual union all
select 24580,'AN' from dual union all
select 24581,'AN' from dual union all
select 24582,'AN' from dual union all
select 24583,'AN' from dual union all
select 24584,'MRN' from dual union all
select 24585,'AN' from dual union all
select 24586,'AN' from dual union all
select 24587,'AN' from dual union all
select 24588,'AN' from dual union all
select 24589,'AN' from dual union all
select 2459,'USA' from dual union all
select 24590,'AN' from dual union all
select 24591,'AN' from dual union all
select 24592,'AN' from dual union all
select 24593,'AN' from dual union all
select 24594,'AN' from dual union all
select 24595,'AN' from dual union all
select 24596,'AN' from dual union all
select 24597,'AN' from dual union all
select 24598,'AN' from dual union all
select 24599,'AN' from dual union all
select 246,'WER271-1' from dual union all
select 2460,'USA' from dual union all
select 24600,'AN' from dual union all
select 24601,'AN' from dual union all
select 24602,'AN' from dual union all
select 24603,'AN' from dual union all
select 24604,'AN' from dual union all
select 24605,'AN' from dual union all
select 24606,'AN' from dual union all
select 24607,'AN' from dual union all
select 24608,'AN' from dual union all
select 24609,'AN' from dual union all
select 2461,'USA' from dual union all
select 24610,'AN' from dual union all
select 24611,'AN' from dual union all
select 24612,'AN' from dual;
insert into LAUNCH_AGENCY
select 24613,'AN' from dual union all
select 24614,'AN' from dual union all
select 24615,'AN' from dual union all
select 24616,'AN' from dual union all
select 24617,'AN' from dual union all
select 24618,'AN' from dual union all
select 24619,'AN' from dual union all
select 2462,'ORDRSA' from dual union all
select 24620,'AN' from dual union all
select 24621,'AN' from dual union all
select 24622,'MRN' from dual union all
select 24623,'MRN' from dual union all
select 24624,'AN' from dual union all
select 24625,'AN' from dual union all
select 24626,'AN' from dual union all
select 24627,'MRN' from dual union all
select 24628,'AN' from dual union all
select 24629,'AN' from dual union all
select 2463,'ORDRSA' from dual union all
select 24630,'AN' from dual union all
select 24631,'AN' from dual union all
select 24632,'MRN' from dual union all
select 24633,'AN' from dual union all
select 24634,'MRN' from dual union all
select 24635,'MRN' from dual union all
select 24636,'AN' from dual union all
select 24637,'AN' from dual union all
select 24638,'MRN' from dual union all
select 24639,'AN' from dual union all
select 2464,'USA' from dual union all
select 24640,'AN' from dual union all
select 24641,'AN' from dual union all
select 24642,'AN' from dual union all
select 24643,'MRN' from dual union all
select 24644,'MRN' from dual union all
select 24645,'MRN' from dual union all
select 24646,'MRN' from dual union all
select 24647,'AN' from dual union all
select 24648,'USA' from dual union all
select 24649,'AN' from dual union all
select 2465,'USA' from dual union all
select 24650,'AN' from dual union all
select 24651,'AN' from dual union all
select 24652,'AN' from dual union all
select 24653,'AN' from dual union all
select 24654,'AN' from dual union all
select 24655,'MRN' from dual union all
select 24656,'MRN' from dual union all
select 24657,'AN' from dual union all
select 24658,'AN' from dual union all
select 24659,'MRN' from dual union all
select 2466,'USA' from dual union all
select 24660,'MRN' from dual union all
select 24661,'AN' from dual union all
select 24662,'AN' from dual union all
select 24663,'AN' from dual union all
select 24664,'AN' from dual union all
select 24665,'AN' from dual union all
select 24666,'AN' from dual union all
select 24667,'MRN' from dual union all
select 24668,'MRN' from dual union all
select 24669,'AN' from dual union all
select 2467,'USA' from dual union all
select 24670,'AN' from dual union all
select 24671,'AN' from dual union all
select 24672,'AN' from dual union all
select 24673,'AN' from dual union all
select 24674,'MRN' from dual union all
select 24675,'MRN' from dual union all
select 24676,'AN' from dual union all
select 24677,'AN' from dual union all
select 24678,'AN' from dual union all
select 24679,'AN' from dual union all
select 2468,'USA' from dual union all
select 24680,'AN' from dual union all
select 24681,'MRN' from dual union all
select 24682,'MRN' from dual union all
select 24683,'AN' from dual union all
select 24684,'AN' from dual union all
select 24685,'AN' from dual union all
select 24686,'AN' from dual union all
select 24687,'AN' from dual union all
select 24688,'AN' from dual union all
select 24689,'AN' from dual union all
select 2469,'USA' from dual union all
select 24690,'AN' from dual union all
select 24691,'AN' from dual union all
select 24692,'AN' from dual union all
select 24693,'AN' from dual union all
select 24694,'MRN' from dual union all
select 24695,'MRN' from dual union all
select 24696,'AN' from dual union all
select 24697,'AN' from dual union all
select 24698,'AN' from dual union all
select 24699,'AN' from dual union all
select 247,'WER271-1' from dual union all
select 2470,'USA' from dual union all
select 24700,'AN' from dual union all
select 24701,'AN' from dual union all
select 24702,'MRN' from dual;
insert into LAUNCH_AGENCY
select 24703,'AN' from dual union all
select 24704,'AN' from dual union all
select 24705,'AN' from dual union all
select 24706,'AN' from dual union all
select 24707,'AN' from dual union all
select 24708,'MRN' from dual union all
select 24709,'AN' from dual union all
select 2471,'USA' from dual union all
select 24710,'AN' from dual union all
select 24711,'AN' from dual union all
select 24712,'AN' from dual union all
select 24713,'AN' from dual union all
select 24714,'AN' from dual union all
select 24715,'AN' from dual union all
select 24716,'AN' from dual union all
select 24717,'MRN' from dual union all
select 24718,'AN' from dual union all
select 24719,'AN' from dual union all
select 2472,'USA' from dual union all
select 24720,'AN' from dual union all
select 24721,'AN' from dual union all
select 24722,'AN' from dual union all
select 24723,'AN' from dual union all
select 24724,'AN' from dual union all
select 24725,'AN' from dual union all
select 24726,'AN' from dual union all
select 24727,'AN' from dual union all
select 24728,'MRN' from dual union all
select 24729,'AN' from dual union all
select 2473,'USA' from dual union all
select 24730,'AN' from dual union all
select 24731,'AN' from dual union all
select 24732,'AN' from dual union all
select 24733,'AN' from dual union all
select 24734,'AN' from dual union all
select 24735,'AN' from dual union all
select 24736,'AN' from dual union all
select 24737,'MRN' from dual union all
select 24738,'AN' from dual union all
select 24739,'AN' from dual union all
select 2474,'USA' from dual union all
select 24740,'AN' from dual union all
select 24741,'AN' from dual union all
select 24742,'AN' from dual union all
select 24743,'AN' from dual union all
select 24744,'AN' from dual union all
select 24745,'AN' from dual union all
select 24746,'AN' from dual union all
select 24747,'AN' from dual union all
select 24748,'AN' from dual union all
select 24749,'AN' from dual union all
select 2475,'PARD' from dual union all
select 24750,'AN' from dual union all
select 24751,'AN' from dual union all
select 24752,'AN' from dual union all
select 24753,'AN' from dual union all
select 24754,'MRN' from dual union all
select 24755,'AN' from dual union all
select 24756,'AN' from dual union all
select 24757,'AN' from dual union all
select 24758,'AN' from dual union all
select 24759,'AN' from dual union all
select 2476,'USA' from dual union all
select 24760,'AN' from dual union all
select 24761,'AN' from dual union all
select 24762,'AN' from dual union all
select 24763,'AN' from dual union all
select 24764,'AN' from dual union all
select 24765,'AN' from dual union all
select 24766,'AN' from dual union all
select 24767,'AN' from dual union all
select 24768,'AN' from dual union all
select 24769,'AN' from dual union all
select 2477,'USA' from dual union all
select 24770,'AN' from dual union all
select 24771,'AN' from dual union all
select 24772,'AN' from dual union all
select 24773,'AN' from dual union all
select 24774,'AN' from dual union all
select 24775,'AN' from dual union all
select 24776,'AN' from dual union all
select 24777,'AN' from dual union all
select 24778,'AN' from dual union all
select 24779,'AN' from dual union all
select 2478,'USA' from dual union all
select 24780,'AN' from dual union all
select 24781,'AN' from dual union all
select 24782,'AN' from dual union all
select 24783,'AN' from dual union all
select 24784,'AN' from dual union all
select 24785,'AN' from dual union all
select 24786,'AN' from dual union all
select 24787,'AN' from dual union all
select 24788,'SRC' from dual union all
select 24789,'MRN' from dual union all
select 2479,'USAF' from dual union all
select 24790,'AN' from dual union all
select 24791,'AN' from dual union all
select 24792,'AN' from dual union all
select 24793,'AN' from dual;
insert into LAUNCH_AGENCY
select 24794,'AN' from dual union all
select 24795,'AN' from dual union all
select 24796,'AN' from dual union all
select 24797,'AN' from dual union all
select 24798,'AN' from dual union all
select 24799,'AN' from dual union all
select 248,'WER271-1' from dual union all
select 2480,'USA' from dual union all
select 24800,'AN' from dual union all
select 24801,'AN' from dual union all
select 24802,'AN' from dual union all
select 24803,'AN' from dual union all
select 24804,'AN' from dual union all
select 24805,'AN' from dual union all
select 24806,'AN' from dual union all
select 24807,'AN' from dual union all
select 24808,'AN' from dual union all
select 24809,'AN' from dual union all
select 2481,'USA' from dual union all
select 24810,'AN' from dual union all
select 24811,'AN' from dual union all
select 24812,'AN' from dual union all
select 24813,'AN' from dual union all
select 24814,'AN' from dual union all
select 24815,'AN' from dual union all
select 24816,'AN' from dual union all
select 24817,'AN' from dual union all
select 24818,'USAF' from dual union all
select 24819,'RVSN' from dual union all
select 2482,'ORDRSA' from dual union all
select 24820,'RVSN' from dual union all
select 24821,'RVSN' from dual union all
select 24822,'RVSN' from dual union all
select 24823,'AFSC' from dual union all
select 24824,'NASA' from dual union all
select 24825,'RVSN' from dual union all
select 24826,'NASA' from dual union all
select 24827,'NASA' from dual union all
select 24828,'RVSN' from dual union all
select 24829,'ISAS' from dual union all
select 2483,'ORDRSA' from dual union all
select 24830,'10ADS' from dual union all
select 24831,'RVSN' from dual union all
select 24832,'RVSN' from dual union all
select 24833,'RVSN' from dual union all
select 24834,'RVSN' from dual union all
select 24835,'RVSN' from dual union all
select 24836,'CALT' from dual union all
select 24837,'NASA' from dual union all
select 24838,'RVSN' from dual union all
select 24839,'AFSC' from dual union all
select 2484,'ORDRSA' from dual union all
select 24840,'USAF' from dual union all
select 24841,'RVSN' from dual union all
select 24842,'NASA' from dual union all
select 24843,'RVSN' from dual union all
select 24844,'RVSN' from dual union all
select 24845,'RVSN' from dual union all
select 24846,'RVSN' from dual union all
select 24847,'RVSN' from dual union all
select 24848,'CNES' from dual union all
select 24849,'RVSN' from dual union all
select 2485,'PARD' from dual union all
select 24850,'RVSN' from dual union all
select 24851,'AFSC' from dual union all
select 24852,'RVSN' from dual union all
select 24853,'RVSN' from dual union all
select 24854,'CRA' from dual union all
select 24855,'RVSN' from dual union all
select 24856,'RVSN' from dual union all
select 24857,'USAF' from dual union all
select 24858,'RVSN' from dual union all
select 24859,'RVSN' from dual union all
select 2486,'ORDRSA' from dual union all
select 24860,'RVSN' from dual union all
select 24861,'RVSN' from dual union all
select 24862,'RVSN' from dual union all
select 24863,'RVSN' from dual union all
select 24864,'RVSN' from dual union all
select 24865,'RVSN' from dual union all
select 24866,'RVSN' from dual union all
select 24867,'RVSN' from dual union all
select 24868,'RVSN' from dual union all
select 24869,'NASA' from dual union all
select 2487,'USA' from dual union all
select 24870,'RVSN' from dual union all
select 24871,'RVSN' from dual union all
select 24872,'10ADS' from dual union all
select 24873,'RVSN' from dual union all
select 24874,'AFSC' from dual union all
select 24875,'RVSN' from dual union all
select 24876,'NASA' from dual union all
select 24877,'RVSN' from dual union all
select 24878,'USAF' from dual union all
select 24879,'RVSN' from dual union all
select 2488,'USA' from dual union all
select 24880,'RVSN' from dual union all
select 24881,'NASA' from dual union all
select 24882,'RVSN' from dual union all
select 24883,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 24884,'RVSN' from dual union all
select 24885,'AFSC' from dual union all
select 24886,'RVSN' from dual union all
select 24887,'RVSN' from dual union all
select 24888,'USAF' from dual union all
select 24889,'NASA' from dual union all
select 2489,'USA' from dual union all
select 24890,'RVSN' from dual union all
select 24891,'RVSN' from dual union all
select 24892,'RVSN' from dual union all
select 24893,'RVSN' from dual union all
select 24894,'USAF' from dual union all
select 24895,'RVSN' from dual union all
select 24896,'RVSN' from dual union all
select 24897,'RVSN' from dual union all
select 24898,'ISAS' from dual union all
select 24899,'RVSN' from dual union all
select 249,'WER271-1' from dual union all
select 2490,'USA' from dual union all
select 24900,'RVSN' from dual union all
select 24901,'NASA' from dual union all
select 24902,'RVSN' from dual union all
select 24903,'RVSN' from dual union all
select 24904,'RVSN' from dual union all
select 24905,'10ADS' from dual union all
select 24906,'RVSN' from dual union all
select 24907,'USAF' from dual union all
select 24908,'RVSN' from dual union all
select 24909,'NASA' from dual union all
select 2491,'ORDRSA' from dual union all
select 24910,'USAF' from dual union all
select 24911,'RAE' from dual union all
select 24912,'RVSN' from dual union all
select 24913,'USAF' from dual union all
select 24914,'CRA' from dual union all
select 24915,'RVSN' from dual union all
select 24916,'RVSN' from dual union all
select 24917,'RVSN' from dual union all
select 24918,'RVSN' from dual union all
select 24919,'RVSN' from dual union all
select 2492,'ORDRSA' from dual union all
select 24920,'RVSN' from dual union all
select 24921,'RVSN' from dual union all
select 24922,'RVSN' from dual union all
select 24923,'RVSN' from dual union all
select 24924,'RVSN' from dual union all
select 24925,'RVSN' from dual union all
select 24926,'RVSN' from dual union all
select 24927,'AFSC' from dual union all
select 24928,'USAF' from dual union all
select 24929,'RVSN' from dual union all
select 2493,'USA' from dual union all
select 24930,'RVSN' from dual union all
select 24931,'RVSN' from dual union all
select 24932,'RVSN' from dual union all
select 24933,'RVSN' from dual union all
select 24934,'NASA' from dual union all
select 24935,'RVSN' from dual union all
select 24936,'RVSN' from dual union all
select 24937,'RVSN' from dual union all
select 24938,'RVSN' from dual union all
select 24939,'MRN' from dual union all
select 2494,'USA' from dual union all
select 24940,'ISRO' from dual union all
select 24941,'MRN' from dual union all
select 24942,'MRN' from dual union all
select 24943,'MRN' from dual union all
select 24944,'MRN' from dual union all
select 24945,'MRN' from dual union all
select 24946,'BAC' from dual union all
select 24946,'DLR' from dual union all
select 24947,'MRN' from dual union all
select 24948,'ESRO' from dual union all
select 24949,'MRN' from dual union all
select 2495,'USA' from dual union all
select 24950,'MRN' from dual union all
select 24951,'MRN' from dual union all
select 24952,'MRN' from dual union all
select 24953,'MRN' from dual union all
select 24954,'RAE' from dual union all
select 24955,'RVSN' from dual union all
select 24956,'MRN' from dual union all
select 24957,'RVSN' from dual union all
select 24958,'MRN' from dual union all
select 24959,'MRN' from dual union all
select 2496,'USA' from dual union all
select 24960,'MRN' from dual union all
select 24961,'MRN' from dual union all
select 24962,'RVSN' from dual union all
select 24963,'JMA' from dual union all
select 24964,'JMA' from dual union all
select 24965,'JMA' from dual union all
select 24966,'ESRO' from dual union all
select 24967,'USA' from dual union all
select 24968,'EGAO' from dual union all
select 24969,'ESRO' from dual union all
select 2497,'ORDRSA' from dual union all
select 24970,'UKMO' from dual union all
select 24971,'ESRO' from dual union all
select 24972,'MRN' from dual;
insert into LAUNCH_AGENCY
select 24973,'WRE' from dual union all
select 24974,'MRN' from dual union all
select 24975,'SAND' from dual union all
select 24976,'SAND' from dual union all
select 24977,'RVSN' from dual union all
select 24978,'WRE' from dual union all
select 24979,'WRE' from dual union all
select 2498,'USA' from dual union all
select 24980,'WRE' from dual union all
select 24981,'WRE' from dual union all
select 24982,'WRE' from dual union all
select 24983,'WRE' from dual union all
select 24984,'RVSN' from dual union all
select 24985,'PRO' from dual union all
select 24986,'RVSN' from dual union all
select 24987,'RVSN' from dual union all
select 24988,'RVSN' from dual union all
select 24989,'RVSN' from dual union all
select 2499,'ORDRSA' from dual union all
select 24990,'RVSN' from dual union all
select 24991,'RVSN' from dual union all
select 24992,'RVSN' from dual union all
select 24993,'RVSN' from dual union all
select 24994,'RVSN' from dual union all
select 24995,'RVSN' from dual union all
select 24996,'MRN' from dual union all
select 24997,'RVSN' from dual union all
select 24998,'RVSN' from dual union all
select 24999,'RVSN' from dual union all
select 25,'WEHR' from dual union all
select 250,'WER271-1' from dual union all
select 2500,'USA' from dual union all
select 25000,'RVSN' from dual union all
select 25001,'RVSN' from dual union all
select 25002,'RVSN' from dual union all
select 25003,'RVSN' from dual union all
select 25004,'RVSN' from dual union all
select 25005,'RVSN' from dual union all
select 25006,'RVSN' from dual union all
select 25007,'MRN' from dual union all
select 25008,'RVSN' from dual union all
select 25009,'RVSN' from dual union all
select 2501,'USA' from dual union all
select 25010,'RVSN' from dual union all
select 25011,'RVSN' from dual union all
select 25012,'RVSN' from dual union all
select 25013,'RVSN' from dual union all
select 25014,'RVSN' from dual union all
select 25015,'RVSN' from dual union all
select 25016,'RVSN' from dual union all
select 25017,'RVSN' from dual union all
select 25018,'MRN' from dual union all
select 25019,'RVSN' from dual union all
select 2502,'USA' from dual union all
select 25020,'RVSN' from dual union all
select 25021,'USA' from dual union all
select 25022,'USA' from dual union all
select 25023,'MRN' from dual union all
select 25024,'USA' from dual union all
select 25025,'MRN' from dual union all
select 25026,'USN' from dual union all
select 25027,'USN' from dual union all
select 25028,'MRN' from dual union all
select 25029,'SAND' from dual union all
select 2503,'USA' from dual union all
select 25030,'MRN' from dual union all
select 25031,'MRN' from dual union all
select 25032,'USA' from dual union all
select 25033,'USA' from dual union all
select 25034,'MRN' from dual union all
select 25035,'MRN' from dual union all
select 25036,'USN' from dual union all
select 25037,'USN' from dual union all
select 25038,'BAL' from dual union all
select 25039,'MRN' from dual union all
select 2504,'USA' from dual union all
select 25040,'BAL' from dual union all
select 25041,'BAL' from dual union all
select 25042,'BAL' from dual union all
select 25043,'BAL' from dual union all
select 25044,'MRN' from dual union all
select 25045,'BAL' from dual union all
select 25046,'BAL' from dual union all
select 25047,'BAL' from dual union all
select 25048,'BAL' from dual union all
select 25049,'BAL' from dual union all
select 2505,'USA' from dual union all
select 25050,'BAL' from dual union all
select 25051,'BAL' from dual union all
select 25052,'USA' from dual union all
select 25053,'MRN' from dual union all
select 25054,'MRN' from dual union all
select 25055,'RVSN' from dual union all
select 25056,'MRN' from dual union all
select 25057,'MRN' from dual union all
select 25058,'RAE' from dual union all
select 25059,'USN' from dual union all
select 2506,'USA' from dual union all
select 25060,'USN' from dual union all
select 25061,'RAE' from dual;
insert into LAUNCH_AGENCY
select 25062,'RAE' from dual union all
select 25063,'INTA' from dual union all
select 25064,'MRN' from dual union all
select 25065,'MRN' from dual union all
select 25066,'MRN' from dual union all
select 25067,'MRN' from dual union all
select 25068,'MRN' from dual union all
select 25069,'MRN' from dual union all
select 2507,'USA' from dual union all
select 25070,'MRN' from dual union all
select 25071,'ISAS' from dual union all
select 25072,'MRN' from dual union all
select 25073,'MRN' from dual union all
select 25074,'MRN' from dual union all
select 25075,'MRN' from dual union all
select 25076,'MRN' from dual union all
select 25077,'MRN' from dual union all
select 25078,'SRC' from dual union all
select 25079,'USN' from dual union all
select 2508,'ORDRSA' from dual union all
select 25080,'USN' from dual union all
select 25081,'9FA3' from dual union all
select 25082,'INTA' from dual union all
select 25083,'USA' from dual union all
select 25084,'ISRO' from dual union all
select 25085,'MRN' from dual union all
select 25086,'USA' from dual union all
select 25087,'MRN' from dual union all
select 25088,'MRN' from dual union all
select 25089,'RAE' from dual union all
select 2509,'ORDRSA' from dual union all
select 25090,'RVSN' from dual union all
select 25091,'MRN' from dual union all
select 25092,'ARPA' from dual union all
select 25092,'US' from dual union all
select 25093,'MRN' from dual union all
select 25094,'RVSN' from dual union all
select 25095,'USA' from dual union all
select 25096,'USA' from dual union all
select 25097,'MRN' from dual union all
select 25098,'ESRO' from dual union all
select 25099,'MRN' from dual union all
select 251,'WER271-1' from dual union all
select 2510,'GEMTS' from dual union all
select 25100,'MRN' from dual union all
select 25101,'USAF' from dual union all
select 25102,'RVSN' from dual union all
select 25103,'RVSN' from dual union all
select 25104,'NASA' from dual union all
select 25105,'RVSN' from dual union all
select 25106,'RVSN' from dual union all
select 25107,'RVSN' from dual union all
select 25108,'RVSN' from dual union all
select 25109,'RVSN' from dual union all
select 2511,'USA' from dual union all
select 25110,'ELDO' from dual union all
select 25111,'RVSN' from dual union all
select 25112,'USAF' from dual union all
select 25113,'CNES' from dual union all
select 25114,'STA' from dual union all
select 25115,'STA' from dual union all
select 25116,'NASA' from dual union all
select 25117,'NASA' from dual union all
select 25118,'INTA' from dual union all
select 25119,'INTA' from dual union all
select 2512,'USA' from dual union all
select 25120,'INTA' from dual union all
select 25121,'INTA' from dual union all
select 25122,'INTA' from dual union all
select 25123,'USA' from dual union all
select 25124,'NSF' from dual union all
select 25125,'STA' from dual union all
select 25126,'STA' from dual union all
select 25127,'ISAS' from dual union all
select 25128,'USA' from dual union all
select 25129,'USA' from dual union all
select 2513,'USA' from dual union all
select 25130,'SRC' from dual union all
select 25131,'INTA' from dual union all
select 25132,'INTA' from dual union all
select 25133,'INTA' from dual union all
select 25134,'INTA' from dual union all
select 25135,'INTA' from dual union all
select 25136,'INTA' from dual union all
select 25137,'INTA' from dual union all
select 25138,'INTA' from dual union all
select 25139,'INTA' from dual union all
select 2514,'ORDRSA' from dual union all
select 25140,'INTA' from dual union all
select 25141,'IIAE' from dual union all
select 25142,'WRE' from dual union all
select 25143,'WRE' from dual union all
select 25144,'WRE' from dual union all
select 25145,'WRE' from dual union all
select 25146,'WRE' from dual union all
select 25147,'WRE' from dual union all
select 25148,'WRE' from dual union all
select 25149,'WRE' from dual union all
select 2515,'USA' from dual union all
select 25150,'WRE' from dual;
insert into LAUNCH_AGENCY
select 25151,'WRE' from dual union all
select 25152,'WRE' from dual union all
select 25153,'PRO' from dual union all
select 25154,'PRO' from dual union all
select 25155,'PRO' from dual union all
select 25156,'USN' from dual union all
select 25157,'USN' from dual union all
select 25158,'DMA' from dual union all
select 25159,'DMA' from dual union all
select 2516,'USA' from dual union all
select 25160,'DMA' from dual union all
select 25161,'DMA' from dual union all
select 25162,'DMA' from dual union all
select 25163,'DLR' from dual union all
select 25164,'ESRO' from dual union all
select 25165,'SAND' from dual union all
select 25166,'BAC' from dual union all
select 25167,'NASA' from dual union all
select 25168,'USAF' from dual union all
select 25169,'NASA' from dual union all
select 2517,'USA' from dual union all
select 25170,'NASDA' from dual union all
select 25171,'SAND' from dual union all
select 25172,'RVSN' from dual union all
select 25173,'RVSN' from dual union all
select 25174,'SAND' from dual union all
select 25175,'SAND' from dual union all
select 25176,'SAC' from dual union all
select 25177,'SAC' from dual union all
select 25178,'USN' from dual union all
select 25179,'NRCC' from dual union all
select 2518,'PARD' from dual union all
select 25180,'ESRO' from dual union all
select 25181,'ESRO' from dual union all
select 25182,'ESRO' from dual union all
select 25183,'ALCS' from dual union all
select 25183,'SAC' from dual union all
select 25184,'RVSN' from dual union all
select 25185,'NASA' from dual union all
select 25186,'RVSN' from dual union all
select 25187,'NASA' from dual union all
select 25188,'SAC' from dual union all
select 25189,'RVSN' from dual union all
select 2519,'ORDRSA' from dual union all
select 25190,'NRCC' from dual union all
select 25191,'NASA' from dual union all
select 25192,'BAC' from dual union all
select 25192,'ESRO' from dual union all
select 25193,'AFSC' from dual union all
select 25194,'ONERA' from dual union all
select 25195,'RVSN' from dual union all
select 25196,'ALCS' from dual union all
select 25196,'SAC' from dual union all
select 25197,'NASA' from dual union all
select 25198,'NRL' from dual union all
select 25199,'ESRO' from dual union all
select 252,'WER271-1' from dual union all
select 2520,'USA' from dual union all
select 25200,'SAC' from dual union all
select 25201,'USN' from dual union all
select 25202,'USN' from dual union all
select 25203,'NRCC' from dual union all
select 25204,'NASA' from dual union all
select 25205,'KPNO' from dual union all
select 25206,'RVSN' from dual union all
select 25207,'SAC' from dual union all
select 25208,'WRE' from dual union all
select 25209,'AN' from dual union all
select 2521,'USA' from dual union all
select 25210,'BAC' from dual union all
select 25211,'WRE' from dual union all
select 25212,'WRE' from dual union all
select 25213,'NRCC' from dual union all
select 25214,'NASA' from dual union all
select 25215,'NASA' from dual union all
select 25216,'NASA' from dual union all
select 25217,'RVSN' from dual union all
select 25218,'NASA' from dual union all
select 25219,'NASA' from dual union all
select 2522,'ORDRSA' from dual union all
select 25220,'NASA' from dual union all
select 25221,'USAF' from dual union all
select 25222,'ESRO' from dual union all
select 25223,'SAND' from dual union all
select 25224,'AFCRL' from dual union all
select 25225,'AFCRL' from dual union all
select 25226,'ESRO' from dual union all
select 25227,'SRC' from dual union all
select 25228,'RVSN' from dual union all
select 25229,'NASA' from dual union all
select 2523,'USA' from dual union all
select 25230,'AN' from dual union all
select 25231,'AN' from dual union all
select 25232,'AN' from dual union all
select 25233,'NASA' from dual union all
select 25234,'NASA' from dual union all
select 25235,'RVSN' from dual union all
select 25236,'ESRO' from dual union all
select 25237,'AN' from dual union all
select 25238,'USN' from dual;
insert into LAUNCH_AGENCY
select 25239,'RVSN' from dual union all
select 2524,'USA' from dual union all
select 25240,'NASA' from dual union all
select 25241,'AFCRL' from dual union all
select 25242,'AFSC' from dual union all
select 25243,'NASA' from dual union all
select 25244,'AN' from dual union all
select 25245,'NASA' from dual union all
select 25246,'AN' from dual union all
select 25247,'NASA' from dual union all
select 25248,'NASA' from dual union all
select 25249,'NASA' from dual union all
select 2525,'USA' from dual union all
select 25250,'RVSN' from dual union all
select 25251,'USN' from dual union all
select 25252,'USN' from dual union all
select 25253,'USN' from dual union all
select 25254,'USN' from dual union all
select 25255,'USN' from dual union all
select 25256,'USN' from dual union all
select 25257,'NASA' from dual union all
select 25258,'SAND' from dual union all
select 25259,'NASA' from dual union all
select 2526,'USA' from dual union all
select 25260,'NASA' from dual union all
select 25261,'RVSN' from dual union all
select 25262,'AFCRL' from dual union all
select 25263,'AFCRL' from dual union all
select 25264,'NRCC' from dual union all
select 25265,'NASA' from dual union all
select 25266,'NASA' from dual union all
select 25267,'USAF' from dual union all
select 25268,'NASA' from dual union all
select 25269,'SAC' from dual union all
select 2527,'USA' from dual union all
select 25270,'USN' from dual union all
select 25271,'USN' from dual union all
select 25272,'RVSN' from dual union all
select 25273,'USN' from dual union all
select 25274,'USN' from dual union all
select 25275,'USN' from dual union all
select 25276,'USN' from dual union all
select 25277,'AFCRL' from dual union all
select 25278,'ESRO' from dual union all
select 25279,'ESRO' from dual union all
select 2528,'USA' from dual union all
select 25280,'AN' from dual union all
select 25281,'NRL' from dual union all
select 25282,'SAND' from dual union all
select 25283,'RVSN' from dual union all
select 25284,'AN' from dual union all
select 25285,'AN' from dual union all
select 25286,'AN' from dual union all
select 25287,'USN' from dual union all
select 25288,'USN' from dual union all
select 25289,'USN' from dual union all
select 2529,'USA' from dual union all
select 25290,'USN' from dual union all
select 25291,'USN' from dual union all
select 25292,'USN' from dual union all
select 25293,'USN' from dual union all
select 25294,'RVSN' from dual union all
select 25295,'USN' from dual union all
select 25296,'SAC' from dual union all
select 25297,'NASA' from dual union all
select 25298,'AN' from dual union all
select 25299,'AFCRL' from dual union all
select 253,'WEHR' from dual union all
select 2530,'USA' from dual union all
select 25300,'USAF' from dual union all
select 25301,'AFSC' from dual union all
select 25302,'SRC' from dual union all
select 25303,'USAF' from dual union all
select 25304,'VMF' from dual union all
select 25305,'USAF' from dual union all
select 25306,'AFSC' from dual union all
select 25307,'SRC' from dual union all
select 25308,'USN' from dual union all
select 25309,'SRC' from dual union all
select 2531,'USA' from dual union all
select 25310,'SRC' from dual union all
select 25311,'USAF' from dual union all
select 25312,'NASA' from dual union all
select 25313,'USN' from dual union all
select 25314,'USA' from dual union all
select 25315,'USAF' from dual union all
select 25316,'SRC' from dual union all
select 25317,'SRC' from dual union all
select 25318,'SRC' from dual union all
select 25319,'AN' from dual union all
select 2532,'USA' from dual union all
select 25320,'AN' from dual union all
select 25321,'SAC' from dual union all
select 25322,'NASA' from dual union all
select 25323,'NASA' from dual union all
select 25324,'SRC' from dual union all
select 25325,'NASA' from dual union all
select 25326,'USAF' from dual union all
select 25327,'NASA' from dual union all
select 25328,'SAC' from dual;
insert into LAUNCH_AGENCY
select 25329,'NASA' from dual union all
select 2533,'USAF' from dual union all
select 25330,'NASA' from dual union all
select 25331,'NASA' from dual union all
select 25332,'CNES' from dual union all
select 25332,'INTA' from dual union all
select 25333,'INTA' from dual union all
select 25333,'NASA' from dual union all
select 25334,'US7A' from dual union all
select 25335,'US7A' from dual union all
select 25336,'US7A' from dual union all
select 25337,'USAF' from dual union all
select 25338,'US7A' from dual union all
select 25339,'NASA' from dual union all
select 2534,'USA' from dual union all
select 25340,'NASA' from dual union all
select 25341,'NIPR' from dual union all
select 25342,'USN' from dual union all
select 25343,'NASA' from dual union all
select 25344,'NASA' from dual union all
select 25345,'SRC' from dual union all
select 25346,'AFCRL' from dual union all
select 25347,'USN' from dual union all
select 25348,'RVSN' from dual union all
select 25349,'AFCRL' from dual union all
select 2535,'USA' from dual union all
select 25350,'BAC' from dual union all
select 25351,'USN' from dual union all
select 25352,'USN' from dual union all
select 25353,'US7A' from dual union all
select 25354,'US7A' from dual union all
select 25355,'US7A' from dual union all
select 25356,'US7A' from dual union all
select 25357,'NASA' from dual union all
select 25358,'NASA' from dual union all
select 25359,'RVSN' from dual union all
select 2536,'USA' from dual union all
select 25360,'ESRO' from dual union all
select 25361,'USAF' from dual union all
select 25362,'CNES' from dual union all
select 25363,'USAF' from dual union all
select 25364,'USAF' from dual union all
select 25365,'SAC' from dual union all
select 25366,'AFCRL' from dual union all
select 25367,'AFCRL' from dual union all
select 25368,'AFCRL' from dual union all
select 25369,'AFCRL' from dual union all
select 2537,'USA' from dual union all
select 25370,'RVSN' from dual union all
select 25371,'AFCRL' from dual union all
select 25372,'AFCRL' from dual union all
select 25373,'WRE' from dual union all
select 25374,'AFCRL' from dual union all
select 25375,'AFCRL' from dual union all
select 25376,'AFCRL' from dual union all
select 25377,'NASA' from dual union all
select 25378,'SAC' from dual union all
select 25379,'AFCRL' from dual union all
select 2538,'USA' from dual union all
select 25380,'INTA' from dual union all
select 25381,'RVSN' from dual union all
select 25382,'SAC' from dual union all
select 25383,'SAND' from dual union all
select 25384,'SAC' from dual union all
select 25385,'SAND' from dual union all
select 25386,'SAND' from dual union all
select 25387,'CNES' from dual union all
select 25387,'INTA' from dual union all
select 25388,'INTA' from dual union all
select 25389,'SAC' from dual union all
select 2539,'USA' from dual union all
select 25390,'INTA' from dual union all
select 25391,'CNES' from dual union all
select 25391,'INTA' from dual union all
select 25392,'RVSN' from dual union all
select 25393,'CNES' from dual union all
select 25393,'INTA' from dual union all
select 25394,'INTA' from dual union all
select 25395,'INTA' from dual union all
select 25396,'US7A' from dual union all
select 25397,'INTA' from dual union all
select 25398,'US7A' from dual union all
select 25399,'US7A' from dual union all
select 254,'WER271-1' from dual union all
select 2540,'USA' from dual union all
select 25400,'KPNO' from dual union all
select 25401,'DMA' from dual union all
select 25402,'SAND' from dual union all
select 25403,'RVSN' from dual union all
select 25404,'USAF' from dual union all
select 25405,'SAC' from dual union all
select 25406,'USN' from dual union all
select 25407,'AFCRL' from dual union all
select 25408,'CNES' from dual union all
select 25409,'44FA2' from dual union all
select 2541,'USA' from dual union all
select 25410,'NASA' from dual union all
select 25411,'AFSC' from dual union all
select 25412,'USAF' from dual union all
select 25413,'CNES' from dual;
insert into LAUNCH_AGENCY
select 25414,'RVSN' from dual union all
select 25415,'NASA' from dual union all
select 25416,'RVSN' from dual union all
select 25417,'ALCS' from dual union all
select 25417,'SAC' from dual union all
select 25418,'SAC' from dual union all
select 25419,'NASA' from dual union all
select 2542,'USA' from dual union all
select 25420,'NASA' from dual union all
select 25421,'SAC' from dual union all
select 25422,'NIPR' from dual union all
select 25423,'NASA' from dual union all
select 25424,'NASA' from dual union all
select 25425,'RVSN' from dual union all
select 25426,'NASA' from dual union all
select 25427,'NASA' from dual union all
select 25428,'SRC' from dual union all
select 25429,'AFSC' from dual union all
select 2543,'USA' from dual union all
select 25430,'DMA' from dual union all
select 25431,'RVSN' from dual union all
select 25432,'SRC' from dual union all
select 25433,'NASA' from dual union all
select 25434,'SRC' from dual union all
select 25435,'AFCRL' from dual union all
select 25436,'RVSN' from dual union all
select 25437,'AFSC' from dual union all
select 25438,'AFSC' from dual union all
select 25439,'NASA' from dual union all
select 2544,'ORDRSA' from dual union all
select 25440,'ESRO' from dual union all
select 25441,'NASA' from dual union all
select 25442,'NASA' from dual union all
select 25443,'USN' from dual union all
select 25444,'NASA' from dual union all
select 25445,'NASA' from dual union all
select 25446,'ALCS' from dual union all
select 25446,'SAC' from dual union all
select 25447,'RVSN' from dual union all
select 25448,'NASA' from dual union all
select 25449,'SAC' from dual union all
select 2545,'USA' from dual union all
select 25450,'NASA' from dual union all
select 25451,'NASA' from dual union all
select 25452,'NASA' from dual union all
select 25453,'NASA' from dual union all
select 25454,'NASA' from dual union all
select 25455,'SRC' from dual union all
select 25456,'SRC' from dual union all
select 25457,'KPNO' from dual union all
select 25458,'RVSN' from dual union all
select 25459,'USN' from dual union all
select 2546,'USA' from dual union all
select 25460,'WRE' from dual union all
select 25461,'NASA' from dual union all
select 25462,'NIPR' from dual union all
select 25463,'NASA' from dual union all
select 25464,'USN' from dual union all
select 25465,'ESRO' from dual union all
select 25466,'NASA' from dual union all
select 25467,'NASA' from dual union all
select 25468,'RVSN' from dual union all
select 25469,'ESRO' from dual union all
select 2547,'USA' from dual union all
select 25470,'ALCS' from dual union all
select 25470,'SAC' from dual union all
select 25471,'ALCS' from dual union all
select 25471,'SAC' from dual union all
select 25472,'BAC' from dual union all
select 25473,'SAC' from dual union all
select 25474,'NTNF' from dual union all
select 25475,'NIPR' from dual union all
select 25476,'NASA' from dual union all
select 25477,'NASA' from dual union all
select 25478,'NASA' from dual union all
select 25479,'RVSN' from dual union all
select 2548,'USA' from dual union all
select 25480,'SAC' from dual union all
select 25481,'USAF' from dual union all
select 25482,'USN' from dual union all
select 25483,'ISAS' from dual union all
select 25484,'NASA' from dual union all
select 25485,'ISAS' from dual union all
select 25486,'USAF' from dual union all
select 25487,'AN' from dual union all
select 25488,'NASA' from dual union all
select 25489,'NASA' from dual union all
select 2549,'USA' from dual union all
select 25490,'RVSN' from dual union all
select 25491,'NASA' from dual union all
select 25492,'NASA' from dual union all
select 25493,'NASA' from dual union all
select 25494,'ISAS' from dual union all
select 25495,'NASA' from dual union all
select 25496,'DMI' from dual union all
select 25497,'USN' from dual union all
select 25498,'USN' from dual union all
select 25499,'USN' from dual union all
select 255,'WER271-1' from dual union all
select 2550,'USA' from dual;
insert into LAUNCH_AGENCY
select 25500,'DMI' from dual union all
select 25501,'RVSN' from dual union all
select 25502,'WRE' from dual union all
select 25503,'ISAS' from dual union all
select 25504,'USA' from dual union all
select 25505,'SAC' from dual union all
select 25506,'USN' from dual union all
select 25507,'USN' from dual union all
select 25508,'9FA3' from dual union all
select 25509,'RVSN' from dual union all
select 2551,'USA' from dual union all
select 25510,'AFSC' from dual union all
select 25511,'USN' from dual union all
select 25512,'RVSN' from dual union all
select 25513,'USN' from dual union all
select 25514,'USN' from dual union all
select 25515,'USN' from dual union all
select 25516,'USN' from dual union all
select 25517,'ISAS' from dual union all
select 25518,'AN' from dual union all
select 25519,'AN' from dual union all
select 2552,'ORDRSA' from dual union all
select 25520,'SAC' from dual union all
select 25521,'USN' from dual union all
select 25522,'USN' from dual union all
select 25523,'RVSN' from dual union all
select 25524,'NASA' from dual union all
select 25525,'SAC' from dual union all
select 25526,'ISAS' from dual union all
select 25527,'NASA' from dual union all
select 25528,'NASA' from dual union all
select 25529,'NASA' from dual union all
select 2553,'GEMTS' from dual union all
select 25530,'ALCS' from dual union all
select 25530,'SAC' from dual union all
select 25531,'RVSN' from dual union all
select 25532,'CALT' from dual union all
select 25533,'SAC' from dual union all
select 25534,'RVSN' from dual union all
select 25535,'NASDA' from dual union all
select 25536,'CNES' from dual union all
select 25537,'NIPR' from dual union all
select 25538,'CNES' from dual union all
select 25539,'AN' from dual union all
select 2554,'USAF' from dual union all
select 25540,'AN' from dual union all
select 25541,'AN' from dual union all
select 25542,'AN' from dual union all
select 25543,'USAF' from dual union all
select 25544,'NASA' from dual union all
select 25545,'RVSN' from dual union all
select 25546,'NASDA' from dual union all
select 25547,'NASA' from dual union all
select 25548,'CNES' from dual union all
select 25549,'CNES' from dual union all
select 2555,'ORDRSA' from dual union all
select 25550,'NASA' from dual union all
select 25551,'NASA' from dual union all
select 25552,'WRE' from dual union all
select 25553,'NASA' from dual union all
select 25554,'NASA' from dual union all
select 25555,'NASA' from dual union all
select 25556,'RVSN' from dual union all
select 25557,'NASA' from dual union all
select 25558,'NASA' from dual union all
select 25559,'NASA' from dual union all
select 2556,'USA' from dual union all
select 25560,'AFCRL' from dual union all
select 25561,'NASA' from dual union all
select 25562,'NASA' from dual union all
select 25563,'NASA' from dual union all
select 25564,'NASA' from dual union all
select 25565,'SRC' from dual union all
select 25566,'AN' from dual union all
select 25567,'RVSN' from dual union all
select 25568,'AN' from dual union all
select 25569,'AN' from dual union all
select 2557,'USA' from dual union all
select 25570,'AN' from dual union all
select 25571,'SRC' from dual union all
select 25572,'NIPR' from dual union all
select 25573,'AN' from dual union all
select 25574,'AN' from dual union all
select 25575,'BAC' from dual union all
select 25576,'AFCRL' from dual union all
select 25577,'AN' from dual union all
select 25578,'RVSN' from dual union all
select 25579,'SRC' from dual union all
select 2558,'USA' from dual union all
select 25580,'ESRO' from dual union all
select 25581,'USN' from dual union all
select 25582,'US7A' from dual union all
select 25583,'US7A' from dual union all
select 25584,'AN' from dual union all
select 25585,'AN' from dual union all
select 25586,'AN' from dual union all
select 25587,'AN' from dual union all
select 25588,'RVSN' from dual union all
select 25589,'RVSN' from dual union all
select 2559,'USA' from dual;
insert into LAUNCH_AGENCY
select 25590,'AN' from dual union all
select 25591,'AFCRL' from dual union all
select 25592,'SAC' from dual union all
select 25593,'AFCRL' from dual union all
select 25594,'AFCRL' from dual union all
select 25595,'AFCRL' from dual union all
select 25596,'AFCRL' from dual union all
select 25597,'AFCRL' from dual union all
select 25598,'AN' from dual union all
select 25599,'AFCRL' from dual union all
select 256,'WER271-1' from dual union all
select 2560,'USA' from dual union all
select 25600,'NASA' from dual union all
select 25601,'AFCRL' from dual union all
select 25602,'AN' from dual union all
select 25603,'AFCRL' from dual union all
select 25604,'AFCRL' from dual union all
select 25605,'AN' from dual union all
select 25606,'AFSC' from dual union all
select 25607,'AN' from dual union all
select 25608,'AN' from dual union all
select 25609,'BAC' from dual union all
select 2561,'USA' from dual union all
select 25610,'AN' from dual union all
select 25611,'NASA' from dual union all
select 25612,'AN' from dual union all
select 25613,'AN' from dual union all
select 25614,'NRL' from dual union all
select 25615,'AN' from dual union all
select 25616,'AN' from dual union all
select 25617,'AN' from dual union all
select 25618,'AN' from dual union all
select 25619,'AN' from dual union all
select 2562,'ORDRSA' from dual union all
select 25620,'AN' from dual union all
select 25621,'NASA' from dual union all
select 25622,'BAC' from dual union all
select 25622,'ESRO' from dual union all
select 25623,'NASA' from dual union all
select 25624,'NASA' from dual union all
select 25625,'USN' from dual union all
select 25626,'ALCS' from dual union all
select 25626,'SAC' from dual union all
select 25627,'CNES' from dual union all
select 25628,'AFCRL' from dual union all
select 25629,'SAC' from dual union all
select 2563,'ORDRSA' from dual union all
select 25630,'SAND' from dual union all
select 25631,'WRE' from dual union all
select 25632,'AFCRL' from dual union all
select 25633,'CNES' from dual union all
select 25634,'SAND' from dual union all
select 25635,'NASA' from dual union all
select 25636,'AFSC' from dual union all
select 25637,'AN' from dual union all
select 25638,'ESRO' from dual union all
select 25639,'SAC' from dual union all
select 2564,'ORDRSA' from dual union all
select 25640,'NASA' from dual union all
select 25641,'SAND' from dual union all
select 25642,'BRL' from dual union all
select 25642,'SAND' from dual union all
select 25643,'USAF' from dual union all
select 25644,'BRL' from dual union all
select 25644,'SAND' from dual union all
select 25645,'BAC' from dual union all
select 25646,'NASA' from dual union all
select 25647,'AFCRL' from dual union all
select 25648,'AN' from dual union all
select 25649,'US7A' from dual union all
select 2565,'USA' from dual union all
select 25650,'9FA3' from dual union all
select 25651,'AN' from dual union all
select 25652,'AFCRL' from dual union all
select 25653,'USN' from dual union all
select 25654,'AFCRL' from dual union all
select 25655,'USN' from dual union all
select 25656,'AN' from dual union all
select 25657,'BRL' from dual union all
select 25657,'SAND' from dual union all
select 25658,'BRL' from dual union all
select 25658,'SAND' from dual union all
select 25659,'WRE' from dual union all
select 2566,'PARD' from dual union all
select 25660,'AFCRL' from dual union all
select 25661,'AFCRL' from dual union all
select 25662,'NRL' from dual union all
select 25663,'USN' from dual union all
select 25664,'KPNO' from dual union all
select 25665,'AFSC' from dual union all
select 25666,'AFCRL' from dual union all
select 25667,'SAC' from dual union all
select 25668,'CRA' from dual union all
select 25669,'SRC' from dual union all
select 2567,'PARD' from dual union all
select 25670,'NMC' from dual union all
select 25671,'NASA' from dual union all
select 25672,'NASA' from dual union all
select 25673,'ALCS' from dual union all
select 25673,'SAC' from dual;
insert into LAUNCH_AGENCY
select 25674,'ESRO' from dual union all
select 25675,'SAC' from dual union all
select 25676,'USA' from dual union all
select 25677,'AFSC' from dual union all
select 25678,'AN' from dual union all
select 25679,'USN' from dual union all
select 2568,'USA' from dual union all
select 25680,'SRC' from dual union all
select 25681,'ESRO' from dual union all
select 25682,'BAC' from dual union all
select 25683,'AN' from dual union all
select 25684,'SRC' from dual union all
select 25685,'SAC' from dual union all
select 25686,'RVSN' from dual union all
select 25687,'USA' from dual union all
select 25688,'NIPR' from dual union all
select 25689,'SRC' from dual union all
select 2569,'USA' from dual union all
select 25690,'NASA' from dual union all
select 25691,'NASA' from dual union all
select 25692,'NASA' from dual union all
select 25693,'NASA' from dual union all
select 25694,'NASA' from dual union all
select 25695,'NASA' from dual union all
select 25696,'NASA' from dual union all
select 25697,'NASA' from dual union all
select 25698,'NASA' from dual union all
select 25699,'AN' from dual union all
select 257,'WER271-1' from dual union all
select 2570,'USA' from dual union all
select 25700,'SRC' from dual union all
select 25701,'AN' from dual union all
select 25702,'AN' from dual union all
select 25703,'USN' from dual union all
select 25704,'SRC' from dual union all
select 25705,'USN' from dual union all
select 25706,'USA' from dual union all
select 25707,'SAC' from dual union all
select 25708,'BAC' from dual union all
select 25709,'NASA' from dual union all
select 2571,'USA' from dual union all
select 25710,'SRC' from dual union all
select 25711,'AFCRL' from dual union all
select 25712,'AFCRL' from dual union all
select 25713,'AFCRL' from dual union all
select 25714,'AN' from dual union all
select 25715,'ESRO' from dual union all
select 25716,'AN' from dual union all
select 25717,'AFSC' from dual union all
select 25718,'VMF' from dual union all
select 25719,'WRE' from dual union all
select 2572,'USA' from dual union all
select 25720,'NASA' from dual union all
select 25721,'DMA' from dual union all
select 25722,'AN' from dual union all
select 25723,'CNES' from dual union all
select 25724,'ALCS' from dual union all
select 25724,'SAC' from dual union all
select 25725,'USN' from dual union all
select 25726,'CNIE' from dual union all
select 25727,'AN' from dual union all
select 25728,'AN' from dual union all
select 25729,'AN' from dual union all
select 2573,'ORDRSA' from dual union all
select 25730,'AN' from dual union all
select 25731,'NASA' from dual union all
select 25732,'AN' from dual union all
select 25733,'AN' from dual union all
select 25734,'AN' from dual union all
select 25735,'AN' from dual union all
select 25736,'AN' from dual union all
select 25737,'INTA' from dual union all
select 25738,'AN' from dual union all
select 25739,'USAF' from dual union all
select 2574,'ORDRSA' from dual union all
select 25740,'USAF' from dual union all
select 25741,'USAF' from dual union all
select 25742,'NRCC' from dual union all
select 25743,'USAF' from dual union all
select 25744,'USAF' from dual union all
select 25745,'SAND' from dual union all
select 25746,'SAND' from dual union all
select 25747,'SAND' from dual union all
select 25748,'PLA2AC' from dual union all
select 25749,'RVSN' from dual union all
select 2575,'USAF' from dual union all
select 25750,'WRE' from dual union all
select 25751,'WRE' from dual union all
select 25752,'WRE' from dual union all
select 25753,'USAF' from dual union all
select 25754,'WRE' from dual union all
select 25755,'WRE' from dual union all
select 25756,'WRE' from dual union all
select 25757,'WRE' from dual union all
select 25758,'WRE' from dual union all
select 25759,'WRE' from dual union all
select 2576,'USA' from dual union all
select 25760,'WRE' from dual union all
select 25761,'WRE' from dual union all
select 25762,'AN' from dual;
insert into LAUNCH_AGENCY
select 25763,'CALT' from dual union all
select 25764,'USAF' from dual union all
select 25765,'USAF' from dual union all
select 25766,'USAF' from dual union all
select 25767,'USAF' from dual union all
select 25768,'USAF' from dual union all
select 25769,'WSMR' from dual union all
select 2577,'USA' from dual union all
select 25770,'SAND' from dual union all
select 25771,'SAND' from dual union all
select 25772,'SAND' from dual union all
select 25773,'USAF' from dual union all
select 25774,'USAF' from dual union all
select 25775,'ISAS' from dual union all
select 25776,'USAF' from dual union all
select 25777,'USAF' from dual union all
select 25778,'USAF' from dual union all
select 25779,'RVSN' from dual union all
select 2578,'USA' from dual union all
select 25780,'RVSN' from dual union all
select 25781,'RVSN' from dual union all
select 25782,'RVSN' from dual union all
select 25783,'RVSN' from dual union all
select 25784,'RVSN' from dual union all
select 25785,'ISAS' from dual union all
select 25786,'RVSN' from dual union all
select 25787,'RVSN' from dual union all
select 25788,'RVSN' from dual union all
select 25789,'RVSN' from dual union all
select 2579,'USA' from dual union all
select 25790,'RVSN' from dual union all
select 25791,'RVSN' from dual union all
select 25792,'RVSN' from dual union all
select 25793,'RVSN' from dual union all
select 25794,'USAF' from dual union all
select 25795,'USAF' from dual union all
select 25796,'USAF' from dual union all
select 25797,'ISRO' from dual union all
select 25798,'RVSN' from dual union all
select 25799,'RVSN' from dual union all
select 258,'WER271-1' from dual union all
select 2580,'USA' from dual union all
select 25800,'RVSN' from dual union all
select 25801,'RVSN' from dual union all
select 25802,'MVS' from dual union all
select 25803,'MVS' from dual union all
select 25804,'MVS' from dual union all
select 25805,'MVS' from dual union all
select 25806,'MVS' from dual union all
select 25807,'ESRO' from dual union all
select 25808,'RVSN' from dual union all
select 25809,'RVSN' from dual union all
select 2581,'USA' from dual union all
select 25810,'RVSN' from dual union all
select 25811,'MVS' from dual union all
select 25812,'MVS' from dual union all
select 25813,'MVS' from dual union all
select 25814,'MVS' from dual union all
select 25815,'RVSN' from dual union all
select 25816,'RVSN' from dual union all
select 25817,'RVSN' from dual union all
select 25818,'DLR' from dual union all
select 25819,'RVSN' from dual union all
select 2582,'USA' from dual union all
select 25820,'RVSN' from dual union all
select 25821,'SAND' from dual union all
select 25822,'USAF' from dual union all
select 25823,'USAF' from dual union all
select 25824,'NRCC' from dual union all
select 25825,'USAF' from dual union all
select 25826,'USAF' from dual union all
select 25827,'USAF' from dual union all
select 25828,'USAF' from dual union all
select 25829,'USAF' from dual union all
select 2583,'USA' from dual union all
select 25830,'NASA' from dual union all
select 25831,'NASA' from dual union all
select 25832,'NRCC' from dual union all
select 25833,'ISAS' from dual union all
select 25834,'AN' from dual union all
select 25835,'ISAS' from dual union all
select 25836,'NASA' from dual union all
select 25837,'NASA' from dual union all
select 25838,'ESRO' from dual union all
select 25839,'NASA' from dual union all
select 2584,'ORDRSA' from dual union all
select 25840,'NASA' from dual union all
select 25841,'ESRO' from dual union all
select 25842,'ESRO' from dual union all
select 25843,'NRCC' from dual union all
select 25844,'SAND' from dual union all
select 25845,'USAF' from dual union all
select 25846,'ALCS' from dual union all
select 25846,'SAC' from dual union all
select 25847,'ESRO' from dual union all
select 25848,'SAC' from dual union all
select 25849,'NASA' from dual union all
select 2585,'ORDRSA' from dual union all
select 25850,'NASA' from dual union all
select 25851,'NASA' from dual;
insert into LAUNCH_AGENCY
select 25852,'NASA' from dual union all
select 25853,'NASA' from dual union all
select 25854,'MRN' from dual union all
select 25855,'MRN' from dual union all
select 25856,'MRN' from dual union all
select 25857,'MRN' from dual union all
select 25858,'MRN' from dual union all
select 25859,'MRN' from dual union all
select 2586,'USA' from dual union all
select 25860,'MRN' from dual union all
select 25861,'MRN' from dual union all
select 25862,'MRN' from dual union all
select 25863,'MRN' from dual union all
select 25864,'MRN' from dual union all
select 25865,'MRN' from dual union all
select 25866,'MRN' from dual union all
select 25867,'MRN' from dual union all
select 25868,'MRN' from dual union all
select 25869,'USA' from dual union all
select 2587,'USA' from dual union all
select 25870,'MRN' from dual union all
select 25871,'MRN' from dual union all
select 25872,'MRN' from dual union all
select 25873,'MRN' from dual union all
select 25874,'MRN' from dual union all
select 25875,'MRN' from dual union all
select 25876,'MRN' from dual union all
select 25877,'MRN' from dual union all
select 25878,'MRN' from dual union all
select 25879,'MRN' from dual union all
select 2588,'USA' from dual union all
select 25880,'MRN' from dual union all
select 25881,'MRN' from dual union all
select 25882,'MRN' from dual union all
select 25883,'MRN' from dual union all
select 25884,'MRN' from dual union all
select 25885,'MRN' from dual union all
select 25886,'MRN' from dual union all
select 25887,'MRN' from dual union all
select 25888,'MRN' from dual union all
select 25889,'MRN' from dual union all
select 2589,'USA' from dual union all
select 25890,'MRN' from dual union all
select 25891,'MRN' from dual union all
select 25892,'MRN' from dual union all
select 25893,'MRN' from dual union all
select 25894,'MRN' from dual union all
select 25895,'MRN' from dual union all
select 25896,'MRN' from dual union all
select 25897,'MRN' from dual union all
select 25898,'MRN' from dual union all
select 25899,'MRN' from dual union all
select 259,'WER271-1' from dual union all
select 2590,'USA' from dual union all
select 25900,'MRN' from dual union all
select 25901,'MRN' from dual union all
select 25902,'MRN' from dual union all
select 25903,'MRN' from dual union all
select 25904,'MRN' from dual union all
select 25905,'MRN' from dual union all
select 25906,'MRN' from dual union all
select 25907,'MRN' from dual union all
select 25908,'MRN' from dual union all
select 25909,'MRN' from dual union all
select 2591,'ORDRSA' from dual union all
select 25910,'MRN' from dual union all
select 25911,'NASA' from dual union all
select 25912,'MRN' from dual union all
select 25913,'MRN' from dual union all
select 25914,'MRN' from dual union all
select 25915,'MRN' from dual union all
select 25916,'MRN' from dual union all
select 25917,'MRN' from dual union all
select 25918,'MRN' from dual union all
select 25919,'MRN' from dual union all
select 2592,'ORDRSA' from dual union all
select 25920,'MRN' from dual union all
select 25921,'MRN' from dual union all
select 25922,'MRN' from dual union all
select 25923,'MRN' from dual union all
select 25924,'MRN' from dual union all
select 25925,'MRN' from dual union all
select 25926,'MRN' from dual union all
select 25927,'MRN' from dual union all
select 25928,'MRN' from dual union all
select 25929,'MRN' from dual union all
select 2593,'USAF' from dual union all
select 25930,'MRN' from dual union all
select 25931,'MRN' from dual union all
select 25932,'MRN' from dual union all
select 25933,'MRN' from dual union all
select 25934,'MRN' from dual union all
select 25935,'MRN' from dual union all
select 25936,'MRN' from dual union all
select 25937,'MRN' from dual union all
select 25938,'MRN' from dual union all
select 25939,'MRN' from dual union all
select 2594,'USA' from dual union all
select 25940,'MRN' from dual union all
select 25941,'MRN' from dual;
insert into LAUNCH_AGENCY
select 25942,'NASA' from dual union all
select 25943,'MRN' from dual union all
select 25944,'MRN' from dual union all
select 25945,'MRN' from dual union all
select 25946,'MRN' from dual union all
select 25947,'MRN' from dual union all
select 25948,'NASA' from dual union all
select 25949,'MRN' from dual union all
select 2595,'ORDRSA' from dual union all
select 25950,'MRN' from dual union all
select 25951,'MRN' from dual union all
select 25952,'NASA' from dual union all
select 25953,'MRN' from dual union all
select 25954,'MRN' from dual union all
select 25955,'MRN' from dual union all
select 25956,'MRN' from dual union all
select 25957,'MRN' from dual union all
select 25958,'MRN' from dual union all
select 25959,'MRN' from dual union all
select 2596,'ORDRSA' from dual union all
select 25960,'MRN' from dual union all
select 25961,'MRN' from dual union all
select 25962,'MRN' from dual union all
select 25963,'MRN' from dual union all
select 25964,'MRN' from dual union all
select 25965,'MRN' from dual union all
select 25966,'MRN' from dual union all
select 25967,'MRN' from dual union all
select 25968,'MRN' from dual union all
select 25969,'MRN' from dual union all
select 2597,'USA' from dual union all
select 25970,'MRN' from dual union all
select 25971,'MRN' from dual union all
select 25972,'MRN' from dual union all
select 25973,'MRN' from dual union all
select 25974,'MRN' from dual union all
select 25975,'MRN' from dual union all
select 25976,'MRN' from dual union all
select 25977,'MRN' from dual union all
select 25978,'MRN' from dual union all
select 25979,'MRN' from dual union all
select 2598,'USA' from dual union all
select 25980,'MRN' from dual union all
select 25981,'MRN' from dual union all
select 25982,'NASA' from dual union all
select 25983,'MRN' from dual union all
select 25984,'MRN' from dual union all
select 25985,'MRN' from dual union all
select 25986,'MRN' from dual union all
select 25987,'MRN' from dual union all
select 25988,'MRN' from dual union all
select 25989,'MRN' from dual union all
select 2599,'USA' from dual union all
select 25990,'MRN' from dual union all
select 25991,'MRN' from dual union all
select 25992,'MRN' from dual union all
select 25993,'MRN' from dual union all
select 25994,'MRN' from dual union all
select 25995,'MRN' from dual union all
select 25996,'ESRO' from dual union all
select 25997,'MRN' from dual union all
select 25998,'MRN' from dual union all
select 25999,'MRN' from dual union all
select 26,'WEHR' from dual union all
select 260,'WER271-1' from dual union all
select 2600,'ORDRSA' from dual union all
select 26000,'MRN' from dual union all
select 26001,'MRN' from dual union all
select 26002,'MRN' from dual union all
select 26003,'MRN' from dual union all
select 26004,'MRN' from dual union all
select 26005,'MRN' from dual union all
select 26006,'MRN' from dual union all
select 26007,'MRN' from dual union all
select 26008,'MRN' from dual union all
select 26009,'MRN' from dual union all
select 2601,'ORDRSA' from dual union all
select 26010,'MRN' from dual union all
select 26011,'MRN' from dual union all
select 26012,'MRN' from dual union all
select 26013,'MRN' from dual union all
select 26014,'MRN' from dual union all
select 26015,'MRN' from dual union all
select 26016,'MRN' from dual union all
select 26017,'MRN' from dual union all
select 26018,'MRN' from dual union all
select 26019,'MRN' from dual union all
select 2602,'USA' from dual union all
select 26020,'MRN' from dual union all
select 26021,'MRN' from dual union all
select 26022,'MRN' from dual union all
select 26023,'MRN' from dual union all
select 26024,'MRN' from dual union all
select 26025,'MRN' from dual union all
select 26026,'MRN' from dual union all
select 26027,'MRN' from dual union all
select 26028,'MRN' from dual union all
select 26029,'MRN' from dual union all
select 2603,'USA' from dual union all
select 26030,'MRN' from dual;
insert into LAUNCH_AGENCY
select 26031,'MRN' from dual union all
select 26032,'MRN' from dual union all
select 26033,'MRN' from dual union all
select 26034,'MRN' from dual union all
select 26035,'MRN' from dual union all
select 26036,'MRN' from dual union all
select 26037,'MRN' from dual union all
select 26038,'MRN' from dual union all
select 26039,'MRN' from dual union all
select 2604,'PARD' from dual union all
select 26040,'MRN' from dual union all
select 26041,'MRN' from dual union all
select 26042,'MRN' from dual union all
select 26043,'MRN' from dual union all
select 26044,'MRN' from dual union all
select 26045,'MRN' from dual union all
select 26046,'MRN' from dual union all
select 26047,'MRN' from dual union all
select 26048,'MRN' from dual union all
select 26049,'MRN' from dual union all
select 2605,'LRBA' from dual union all
select 26050,'MRN' from dual union all
select 26051,'MRN' from dual union all
select 26052,'MRN' from dual union all
select 26053,'MRN' from dual union all
select 26054,'MRN' from dual union all
select 26055,'MRN' from dual union all
select 26056,'MRN' from dual union all
select 26057,'MRN' from dual union all
select 26058,'MRN' from dual union all
select 26059,'MRN' from dual union all
select 2606,'USA' from dual union all
select 26060,'MRN' from dual union all
select 26061,'MRN' from dual union all
select 26062,'MRN' from dual union all
select 26063,'MRN' from dual union all
select 26064,'MRN' from dual union all
select 26065,'MRN' from dual union all
select 26066,'MRN' from dual union all
select 26067,'MRN' from dual union all
select 26068,'MRN' from dual union all
select 26069,'MRN' from dual union all
select 2607,'ORDRSA' from dual union all
select 26070,'MRN' from dual union all
select 26071,'MRN' from dual union all
select 26072,'MRN' from dual union all
select 26073,'UKMO' from dual union all
select 26074,'MRN' from dual union all
select 26075,'MRN' from dual union all
select 26076,'MRN' from dual union all
select 26077,'MRN' from dual union all
select 26078,'MRN' from dual union all
select 26079,'MRN' from dual union all
select 2608,'GEMTS' from dual union all
select 26080,'MRN' from dual union all
select 26081,'MRN' from dual union all
select 26082,'MRN' from dual union all
select 26083,'MRN' from dual union all
select 26084,'MRN' from dual union all
select 26085,'UKMO' from dual union all
select 26086,'MRN' from dual union all
select 26087,'MRN' from dual union all
select 26088,'MRN' from dual union all
select 26089,'MRN' from dual union all
select 2609,'ORDRSA' from dual union all
select 26090,'MRN' from dual union all
select 26091,'MRN' from dual union all
select 26092,'MRN' from dual union all
select 26093,'MRN' from dual union all
select 26094,'MRN' from dual union all
select 26095,'MRN' from dual union all
select 26096,'MRN' from dual union all
select 26097,'MRN' from dual union all
select 26098,'MRN' from dual union all
select 26099,'MRN' from dual union all
select 261,'WER271-1' from dual union all
select 2610,'USAF' from dual union all
select 26100,'MRN' from dual union all
select 26101,'MRN' from dual union all
select 26102,'MRN' from dual union all
select 26103,'MRN' from dual union all
select 26104,'MRN' from dual union all
select 26105,'MRN' from dual union all
select 26106,'MRN' from dual union all
select 26107,'MRN' from dual union all
select 26108,'MRN' from dual union all
select 26109,'MRN' from dual union all
select 2611,'USAF' from dual union all
select 26110,'INTA' from dual union all
select 26111,'MRN' from dual union all
select 26112,'MRN' from dual union all
select 26113,'MRN' from dual union all
select 26114,'MRN' from dual union all
select 26115,'MRN' from dual union all
select 26116,'MRN' from dual union all
select 26117,'MRN' from dual union all
select 26118,'MRN' from dual union all
select 26119,'MRN' from dual union all
select 2612,'USAF' from dual union all
select 26120,'MRN' from dual;
insert into LAUNCH_AGENCY
select 26121,'MRN' from dual union all
select 26122,'MRN' from dual union all
select 26123,'MRN' from dual union all
select 26124,'MRN' from dual union all
select 26125,'MRN' from dual union all
select 26126,'MRN' from dual union all
select 26127,'MRN' from dual union all
select 26128,'MRN' from dual union all
select 26129,'USA' from dual union all
select 2613,'USA' from dual union all
select 26130,'MRN' from dual union all
select 26131,'MRN' from dual union all
select 26132,'MRN' from dual union all
select 26133,'MRN' from dual union all
select 26134,'MRN' from dual union all
select 26135,'MRN' from dual union all
select 26136,'MRN' from dual union all
select 26137,'MRN' from dual union all
select 26138,'MRN' from dual union all
select 26139,'MRN' from dual union all
select 2614,'USA' from dual union all
select 26140,'USA' from dual union all
select 26141,'INTA' from dual union all
select 26142,'MRN' from dual union all
select 26143,'MRN' from dual union all
select 26144,'MRN' from dual union all
select 26145,'MRN' from dual union all
select 26146,'MRN' from dual union all
select 26147,'MRN' from dual union all
select 26148,'MRN' from dual union all
select 26149,'MRN' from dual union all
select 2615,'USA' from dual union all
select 26150,'MRN' from dual union all
select 26151,'MRN' from dual union all
select 26152,'MRN' from dual union all
select 26153,'MRN' from dual union all
select 26154,'MRN' from dual union all
select 26155,'MRN' from dual union all
select 26156,'MRN' from dual union all
select 26157,'MRN' from dual union all
select 26158,'MRN' from dual union all
select 26159,'MRN' from dual union all
select 2616,'USA' from dual union all
select 26160,'MRN' from dual union all
select 26161,'MRN' from dual union all
select 26162,'UKMO' from dual union all
select 26163,'MRN' from dual union all
select 26164,'MRN' from dual union all
select 26165,'MRN' from dual union all
select 26166,'MRN' from dual union all
select 26167,'MRN' from dual union all
select 26168,'MRN' from dual union all
select 26169,'MRN' from dual union all
select 2617,'PARD' from dual union all
select 26170,'MRN' from dual union all
select 26171,'MRN' from dual union all
select 26172,'MRN' from dual union all
select 26173,'MRN' from dual union all
select 26174,'MRN' from dual union all
select 26175,'MRN' from dual union all
select 26176,'MRN' from dual union all
select 26177,'MRN' from dual union all
select 26178,'MRN' from dual union all
select 26179,'MRN' from dual union all
select 2618,'ORDRSA' from dual union all
select 26180,'MRN' from dual union all
select 26181,'MRN' from dual union all
select 26182,'MRN' from dual union all
select 26183,'MRN' from dual union all
select 26184,'MRN' from dual union all
select 26185,'MRN' from dual union all
select 26186,'MRN' from dual union all
select 26187,'MRN' from dual union all
select 26188,'MRN' from dual union all
select 26189,'MRN' from dual union all
select 2619,'USAF' from dual union all
select 26190,'MRN' from dual union all
select 26191,'MRN' from dual union all
select 26192,'MRN' from dual union all
select 26193,'INTA' from dual union all
select 26194,'MRN' from dual union all
select 26195,'MRN' from dual union all
select 26196,'ESRO' from dual union all
select 26197,'MRN' from dual union all
select 26198,'MRN' from dual union all
select 26199,'MRN' from dual union all
select 262,'WER271-1' from dual union all
select 2620,'ORDRSA' from dual union all
select 26200,'MRN' from dual union all
select 26201,'MRN' from dual union all
select 26202,'MRN' from dual union all
select 26203,'MRN' from dual union all
select 26204,'INTA' from dual union all
select 26205,'MRN' from dual union all
select 26206,'MRN' from dual union all
select 26207,'UKMO' from dual union all
select 26208,'MRN' from dual union all
select 26209,'MRN' from dual union all
select 2621,'USA' from dual union all
select 26210,'MRN' from dual;
insert into LAUNCH_AGENCY
select 26211,'MRN' from dual union all
select 26212,'MRN' from dual union all
select 26213,'MRN' from dual union all
select 26214,'INTA' from dual union all
select 26215,'MRN' from dual union all
select 26216,'MRN' from dual union all
select 26217,'MRN' from dual union all
select 26218,'MRN' from dual union all
select 26219,'MRN' from dual union all
select 2622,'ORDRSA' from dual union all
select 26220,'MRN' from dual union all
select 26221,'MRN' from dual union all
select 26222,'MRN' from dual union all
select 26223,'MRN' from dual union all
select 26224,'MRN' from dual union all
select 26225,'MRN' from dual union all
select 26226,'MRN' from dual union all
select 26227,'MRN' from dual union all
select 26228,'MRN' from dual union all
select 26229,'ESRO' from dual union all
select 2623,'USA' from dual union all
select 26230,'MRN' from dual union all
select 26231,'MRN' from dual union all
select 26232,'MRN' from dual union all
select 26233,'MRN' from dual union all
select 26234,'MRN' from dual union all
select 26235,'MRN' from dual union all
select 26236,'MRN' from dual union all
select 26237,'MRN' from dual union all
select 26238,'MRN' from dual union all
select 26239,'MRN' from dual union all
select 2624,'USA' from dual union all
select 26240,'MRN' from dual union all
select 26241,'MRN' from dual union all
select 26242,'MRN' from dual union all
select 26243,'MRN' from dual union all
select 26244,'MRN' from dual union all
select 26245,'MRN' from dual union all
select 26246,'MRN' from dual union all
select 26247,'MRN' from dual union all
select 26248,'MRN' from dual union all
select 26249,'MRN' from dual union all
select 2625,'USA' from dual union all
select 26250,'MRN' from dual union all
select 26251,'MRN' from dual union all
select 26252,'MRN' from dual union all
select 26253,'MRN' from dual union all
select 26254,'MRN' from dual union all
select 26255,'MRN' from dual union all
select 26256,'MRN' from dual union all
select 26257,'MRN' from dual union all
select 26258,'MRN' from dual union all
select 26259,'MRN' from dual union all
select 2626,'USA' from dual union all
select 26260,'MRN' from dual union all
select 26261,'MRN' from dual union all
select 26262,'MRN' from dual union all
select 26263,'MRN' from dual union all
select 26264,'MRN' from dual union all
select 26265,'MRN' from dual union all
select 26266,'MRN' from dual union all
select 26267,'MRN' from dual union all
select 26268,'MRN' from dual union all
select 26269,'MRN' from dual union all
select 2627,'ORDRSA' from dual union all
select 26270,'MRN' from dual union all
select 26271,'MRN' from dual union all
select 26272,'MRN' from dual union all
select 26273,'MRN' from dual union all
select 26274,'MRN' from dual union all
select 26275,'MRN' from dual union all
select 26276,'MRN' from dual union all
select 26277,'MRN' from dual union all
select 26278,'US' from dual union all
select 26279,'MRN' from dual union all
select 2628,'USA' from dual union all
select 26280,'MRN' from dual union all
select 26281,'NASA' from dual union all
select 26282,'MRN' from dual union all
select 26283,'MRN' from dual union all
select 26284,'MRN' from dual union all
select 26285,'MRN' from dual union all
select 26286,'MRN' from dual union all
select 26287,'MRN' from dual union all
select 26288,'MRN' from dual union all
select 26289,'MRN' from dual union all
select 2629,'ORDRSA' from dual union all
select 26290,'MRN' from dual union all
select 26291,'MRN' from dual union all
select 26292,'MRN' from dual union all
select 26293,'MRN' from dual union all
select 26294,'MRN' from dual union all
select 26295,'MRN' from dual union all
select 26296,'MRN' from dual union all
select 26297,'MRN' from dual union all
select 26298,'MRN' from dual union all
select 26299,'MRN' from dual union all
select 263,'WER271-1' from dual union all
select 2630,'USA' from dual union all
select 26300,'MRN' from dual;
insert into LAUNCH_AGENCY
select 26301,'MRN' from dual union all
select 26302,'MRN' from dual union all
select 26303,'MRN' from dual union all
select 26304,'MRN' from dual union all
select 26305,'MRN' from dual union all
select 26306,'MRN' from dual union all
select 26307,'MRN' from dual union all
select 26308,'MRN' from dual union all
select 26309,'MRN' from dual union all
select 2631,'USA' from dual union all
select 26310,'MRN' from dual union all
select 26311,'MRN' from dual union all
select 26312,'MRN' from dual union all
select 26313,'MRN' from dual union all
select 26314,'MRN' from dual union all
select 26315,'MRN' from dual union all
select 26316,'MRN' from dual union all
select 26317,'MRN' from dual union all
select 26318,'MRN' from dual union all
select 26319,'MRN' from dual union all
select 2632,'ORDRSA' from dual union all
select 26320,'MRN' from dual union all
select 26321,'MRN' from dual union all
select 26322,'MRN' from dual union all
select 26323,'MRN' from dual union all
select 26324,'MRN' from dual union all
select 26325,'MRN' from dual union all
select 26326,'MRN' from dual union all
select 26327,'MRN' from dual union all
select 26328,'MRN' from dual union all
select 26329,'UKMO' from dual union all
select 2633,'ORDRSA' from dual union all
select 26330,'MRN' from dual union all
select 26331,'MRN' from dual union all
select 26332,'MRN' from dual union all
select 26333,'MRN' from dual union all
select 26334,'MRN' from dual union all
select 26335,'MRN' from dual union all
select 26336,'MRN' from dual union all
select 26337,'MRN' from dual union all
select 26338,'MRN' from dual union all
select 26339,'MRN' from dual union all
select 2634,'ORDRSA' from dual union all
select 26340,'MRN' from dual union all
select 26341,'MRN' from dual union all
select 26342,'MRN' from dual union all
select 26343,'MRN' from dual union all
select 26344,'MRN' from dual union all
select 26345,'MRN' from dual union all
select 26346,'MRN' from dual union all
select 26347,'MRN' from dual union all
select 26348,'MRN' from dual union all
select 26349,'MRN' from dual union all
select 2635,'USA' from dual union all
select 26350,'MRN' from dual union all
select 26351,'MRN' from dual union all
select 26352,'MRN' from dual union all
select 26353,'MRN' from dual union all
select 26354,'MRN' from dual union all
select 26355,'MRN' from dual union all
select 26356,'MRN' from dual union all
select 26357,'MRN' from dual union all
select 26358,'MRN' from dual union all
select 26359,'MRN' from dual union all
select 2636,'ORDRSA' from dual union all
select 26360,'MRN' from dual union all
select 26361,'MRN' from dual union all
select 26362,'MRN' from dual union all
select 26363,'USA' from dual union all
select 26364,'MRN' from dual union all
select 26365,'MRN' from dual union all
select 26366,'AN' from dual union all
select 26367,'MRN' from dual union all
select 26368,'MRN' from dual union all
select 26369,'MRN' from dual union all
select 2637,'ORDRSA' from dual union all
select 26370,'MRN' from dual union all
select 26371,'INTA' from dual union all
select 26372,'MRN' from dual union all
select 26373,'MRN' from dual union all
select 26374,'MRN' from dual union all
select 26375,'MRN' from dual union all
select 26376,'MRN' from dual union all
select 26377,'MRN' from dual union all
select 26378,'MRN' from dual union all
select 26379,'MRN' from dual union all
select 2638,'USA' from dual union all
select 26380,'MRN' from dual union all
select 26381,'MRN' from dual union all
select 26382,'MRN' from dual union all
select 26383,'USA' from dual union all
select 26384,'MRN' from dual union all
select 26385,'MRN' from dual union all
select 26386,'MRN' from dual union all
select 26387,'MRN' from dual union all
select 26388,'MRN' from dual union all
select 26389,'MRN' from dual union all
select 2639,'USA' from dual union all
select 26390,'MRN' from dual union all
select 26391,'MRN' from dual;
insert into LAUNCH_AGENCY
select 26392,'MRN' from dual union all
select 26393,'MRN' from dual union all
select 26394,'MRN' from dual union all
select 26395,'MRN' from dual union all
select 26396,'MRN' from dual union all
select 26397,'MRN' from dual union all
select 26398,'MRN' from dual union all
select 26399,'MRN' from dual union all
select 264,'WEHR' from dual union all
select 2640,'ORDRSA' from dual union all
select 26400,'MRN' from dual union all
select 26401,'MRN' from dual union all
select 26402,'MRN' from dual union all
select 26403,'MRN' from dual union all
select 26404,'MRN' from dual union all
select 26405,'MRN' from dual union all
select 26406,'MRN' from dual union all
select 26407,'MRN' from dual union all
select 26408,'MRN' from dual union all
select 26409,'MRN' from dual union all
select 2641,'ORDRSA' from dual union all
select 26410,'MRN' from dual union all
select 26411,'MRN' from dual union all
select 26412,'MRN' from dual union all
select 26413,'MRN' from dual union all
select 26414,'MRN' from dual union all
select 26415,'MRN' from dual union all
select 26416,'MRN' from dual union all
select 26417,'MRN' from dual union all
select 26418,'MRN' from dual union all
select 26419,'MRN' from dual union all
select 2642,'USA' from dual union all
select 26420,'MRN' from dual union all
select 26421,'MRN' from dual union all
select 26422,'MRN' from dual union all
select 26423,'MRN' from dual union all
select 26424,'MRN' from dual union all
select 26425,'MRN' from dual union all
select 26426,'MRN' from dual union all
select 26427,'MRN' from dual union all
select 26428,'MRN' from dual union all
select 26429,'MRN' from dual union all
select 2643,'USA' from dual union all
select 26430,'MRN' from dual union all
select 26431,'MRN' from dual union all
select 26432,'MRN' from dual union all
select 26433,'MRN' from dual union all
select 26434,'MRN' from dual union all
select 26435,'MRN' from dual union all
select 26436,'MRN' from dual union all
select 26437,'MRN' from dual union all
select 26438,'MRN' from dual union all
select 26439,'MRN' from dual union all
select 2644,'USA' from dual union all
select 26440,'MRN' from dual union all
select 26441,'MRN' from dual union all
select 26442,'MRN' from dual union all
select 26443,'MRN' from dual union all
select 26444,'MRN' from dual union all
select 26445,'MRN' from dual union all
select 26446,'MRN' from dual union all
select 26447,'MRN' from dual union all
select 26448,'MRN' from dual union all
select 26449,'MRN' from dual union all
select 2645,'USA' from dual union all
select 26450,'MRN' from dual union all
select 26451,'MRN' from dual union all
select 26452,'MRN' from dual union all
select 26453,'MRN' from dual union all
select 26454,'MRN' from dual union all
select 26455,'MRN' from dual union all
select 26456,'MRN' from dual union all
select 26457,'MRN' from dual union all
select 26458,'MRN' from dual union all
select 26459,'MRN' from dual union all
select 2646,'USA' from dual union all
select 26460,'MRN' from dual union all
select 26461,'MRN' from dual union all
select 26462,'MRN' from dual union all
select 26463,'MRN' from dual union all
select 26464,'MRN' from dual union all
select 26465,'MRN' from dual union all
select 26466,'MRN' from dual union all
select 26467,'MRN' from dual union all
select 26468,'MRN' from dual union all
select 26469,'MRN' from dual union all
select 2647,'ORDRSA' from dual union all
select 26470,'MRN' from dual union all
select 26471,'MRN' from dual union all
select 26472,'MRN' from dual union all
select 26473,'MRN' from dual union all
select 26474,'MRN' from dual union all
select 26475,'MRN' from dual union all
select 26476,'MRN' from dual union all
select 26477,'MRN' from dual union all
select 26478,'MRN' from dual union all
select 26479,'MRN' from dual union all
select 2648,'ORDRSA' from dual union all
select 26480,'MRN' from dual union all
select 26481,'MRN' from dual;
insert into LAUNCH_AGENCY
select 26482,'MRN' from dual union all
select 26483,'MRN' from dual union all
select 26484,'MRN' from dual union all
select 26485,'MRN' from dual union all
select 26486,'MRN' from dual union all
select 26487,'MRN' from dual union all
select 26488,'MRN' from dual union all
select 26489,'MRN' from dual union all
select 2649,'USA' from dual union all
select 26490,'MRN' from dual union all
select 26491,'MRN' from dual union all
select 26492,'MRN' from dual union all
select 26493,'MRN' from dual union all
select 26494,'MRN' from dual union all
select 26495,'MRN' from dual union all
select 26496,'MRN' from dual union all
select 26497,'MRN' from dual union all
select 26498,'MRN' from dual union all
select 26499,'MRN' from dual union all
select 265,'WER271-1' from dual union all
select 2650,'USA' from dual union all
select 26500,'MRN' from dual union all
select 26501,'MRN' from dual union all
select 26502,'MRN' from dual union all
select 26503,'MRN' from dual union all
select 26504,'MRN' from dual union all
select 26505,'MRN' from dual union all
select 26506,'MRN' from dual union all
select 26507,'MRN' from dual union all
select 26508,'MRN' from dual union all
select 26509,'MRN' from dual union all
select 2651,'USA' from dual union all
select 26510,'MRN' from dual union all
select 26511,'MRN' from dual union all
select 26512,'MRN' from dual union all
select 26513,'MRN' from dual union all
select 26514,'MRN' from dual union all
select 26515,'MRN' from dual union all
select 26516,'UKMO' from dual union all
select 26517,'MRN' from dual union all
select 26518,'MRN' from dual union all
select 26519,'MRN' from dual union all
select 2652,'USA' from dual union all
select 26520,'MRN' from dual union all
select 26521,'MRN' from dual union all
select 26522,'MRN' from dual union all
select 26523,'MRN' from dual union all
select 26524,'MRN' from dual union all
select 26525,'MRN' from dual union all
select 26526,'MRN' from dual union all
select 26527,'MRN' from dual union all
select 26528,'MRN' from dual union all
select 26529,'MRN' from dual union all
select 2653,'ORDRSA' from dual union all
select 26530,'MRN' from dual union all
select 26531,'MRN' from dual union all
select 26532,'MRN' from dual union all
select 26533,'MRN' from dual union all
select 26534,'MRN' from dual union all
select 26535,'MRN' from dual union all
select 26536,'INTA' from dual union all
select 26537,'MRN' from dual union all
select 26538,'MRN' from dual union all
select 26539,'MRN' from dual union all
select 2654,'USA' from dual union all
select 26540,'MRN' from dual union all
select 26541,'MRN' from dual union all
select 26542,'MRN' from dual union all
select 26543,'MRN' from dual union all
select 26544,'MRN' from dual union all
select 26545,'MRN' from dual union all
select 26546,'MRN' from dual union all
select 26547,'MRN' from dual union all
select 26548,'MRN' from dual union all
select 26549,'MRN' from dual union all
select 2655,'USA' from dual union all
select 26550,'MRN' from dual union all
select 26551,'MRN' from dual union all
select 26552,'MRN' from dual union all
select 26553,'MRN' from dual union all
select 26554,'MRN' from dual union all
select 26555,'MRN' from dual union all
select 26556,'MRN' from dual union all
select 26557,'MRN' from dual union all
select 26558,'MRN' from dual union all
select 26559,'MRN' from dual union all
select 2656,'USA' from dual union all
select 26560,'MRN' from dual union all
select 26561,'MRN' from dual union all
select 26562,'MRN' from dual union all
select 26563,'MRN' from dual union all
select 26564,'MRN' from dual union all
select 26565,'MRN' from dual union all
select 26566,'MRN' from dual union all
select 26567,'MRN' from dual union all
select 26568,'MRN' from dual union all
select 26569,'MRN' from dual union all
select 2657,'USA' from dual union all
select 26570,'MRN' from dual union all
select 26571,'MRN' from dual;
insert into LAUNCH_AGENCY
select 26572,'MRN' from dual union all
select 26573,'MRN' from dual union all
select 26574,'MRN' from dual union all
select 26575,'MRN' from dual union all
select 26576,'MRN' from dual union all
select 26577,'MRN' from dual union all
select 26578,'MRN' from dual union all
select 26579,'MRN' from dual union all
select 2658,'USA' from dual union all
select 26580,'MRN' from dual union all
select 26581,'MRN' from dual union all
select 26582,'MRN' from dual union all
select 26583,'MRN' from dual union all
select 26584,'MRN' from dual union all
select 26585,'MRN' from dual union all
select 26586,'MRN' from dual union all
select 26587,'MRN' from dual union all
select 26588,'MRN' from dual union all
select 26589,'MRN' from dual union all
select 2659,'USA' from dual union all
select 26590,'MRN' from dual union all
select 26591,'MRN' from dual union all
select 26592,'MRN' from dual union all
select 26593,'MRN' from dual union all
select 26594,'MRN' from dual union all
select 26595,'MRN' from dual union all
select 26596,'MRN' from dual union all
select 26597,'MRN' from dual union all
select 26598,'INTA' from dual union all
select 26599,'MRN' from dual union all
select 266,'WER271-1' from dual union all
select 2660,'USA' from dual union all
select 26600,'MRN' from dual union all
select 26601,'MRN' from dual union all
select 26602,'MRN' from dual union all
select 26603,'MRN' from dual union all
select 26604,'INTA' from dual union all
select 26605,'MRN' from dual union all
select 26606,'MRN' from dual union all
select 26607,'MRN' from dual union all
select 26608,'MRN' from dual union all
select 26609,'MRN' from dual union all
select 2661,'USA' from dual union all
select 26610,'MRN' from dual union all
select 26611,'MRN' from dual union all
select 26612,'MRN' from dual union all
select 26613,'MRN' from dual union all
select 26614,'MRN' from dual union all
select 26615,'MRN' from dual union all
select 26616,'MRN' from dual union all
select 26617,'MRN' from dual union all
select 26618,'MRN' from dual union all
select 26619,'MRN' from dual union all
select 2662,'USA' from dual union all
select 26620,'MRN' from dual union all
select 26621,'MRN' from dual union all
select 26622,'MRN' from dual union all
select 26623,'MRN' from dual union all
select 26624,'MRN' from dual union all
select 26625,'MRN' from dual union all
select 26626,'MRN' from dual union all
select 26627,'MRN' from dual union all
select 26628,'MRN' from dual union all
select 26629,'MRN' from dual union all
select 2663,'USAF' from dual union all
select 26630,'JMA' from dual union all
select 26631,'JMA' from dual union all
select 26632,'JMA' from dual union all
select 26633,'JMA' from dual union all
select 26634,'JMA' from dual union all
select 26635,'JMA' from dual union all
select 26636,'JMA' from dual union all
select 26637,'JMA' from dual union all
select 26638,'MRN' from dual union all
select 26639,'MRN' from dual union all
select 2664,'ORDRSA' from dual union all
select 26640,'JMA' from dual union all
select 26641,'JMA' from dual union all
select 26642,'JMA' from dual union all
select 26643,'MRN' from dual union all
select 26644,'MRN' from dual union all
select 26645,'ESRO' from dual union all
select 26646,'MRN' from dual union all
select 26647,'MRN' from dual union all
select 26648,'MRN' from dual union all
select 26649,'ESRO' from dual union all
select 2665,'ORDRSA' from dual union all
select 26650,'MRN' from dual union all
select 26651,'UKMO' from dual union all
select 26652,'MRN' from dual union all
select 26653,'MRN' from dual union all
select 26654,'MRN' from dual union all
select 26655,'MRN' from dual union all
select 26656,'MRN' from dual union all
select 26657,'MRN' from dual union all
select 26658,'MRN' from dual union all
select 26659,'MRN' from dual union all
select 2666,'GEMTS' from dual union all
select 26660,'USA' from dual union all
select 26661,'MRN' from dual;
insert into LAUNCH_AGENCY
select 26662,'MRN' from dual union all
select 26663,'MRN' from dual union all
select 26664,'MRN' from dual union all
select 26665,'MRN' from dual union all
select 26666,'MRN' from dual union all
select 26667,'MRN' from dual union all
select 26668,'UKMO' from dual union all
select 26669,'MRN' from dual union all
select 2667,'USA' from dual union all
select 26670,'MRN' from dual union all
select 26671,'MRN' from dual union all
select 26672,'MRN' from dual union all
select 26673,'MRN' from dual union all
select 26674,'MRN' from dual union all
select 26675,'MRN' from dual union all
select 26676,'ESRO' from dual union all
select 26677,'MRN' from dual union all
select 26678,'MRN' from dual union all
select 26679,'MRN' from dual union all
select 2668,'ORDRSA' from dual union all
select 26680,'MRN' from dual union all
select 26681,'UKMO' from dual union all
select 26682,'MRN' from dual union all
select 26683,'MRN' from dual union all
select 26684,'MRN' from dual union all
select 26685,'ESRO' from dual union all
select 26686,'MRN' from dual union all
select 26687,'MRN' from dual union all
select 26688,'MRN' from dual union all
select 26689,'UKMO' from dual union all
select 2669,'USA' from dual union all
select 26690,'MRN' from dual union all
select 26691,'MRN' from dual union all
select 26692,'MRN' from dual union all
select 26693,'MRN' from dual union all
select 26694,'MRN' from dual union all
select 26695,'MRN' from dual union all
select 26696,'MRN' from dual union all
select 26697,'MRN' from dual union all
select 26698,'MRN' from dual union all
select 26699,'UKMO' from dual union all
select 267,'WER271-1' from dual union all
select 2670,'USA' from dual union all
select 26700,'MRN' from dual union all
select 26701,'MRN' from dual union all
select 26702,'MRN' from dual union all
select 26703,'MRN' from dual union all
select 26704,'MRN' from dual union all
select 26705,'MRN' from dual union all
select 26706,'MRN' from dual union all
select 26707,'MRN' from dual union all
select 26708,'MRN' from dual union all
select 26709,'MRN' from dual union all
select 2671,'USA' from dual union all
select 26710,'MRN' from dual union all
select 26711,'ESRO' from dual union all
select 26712,'MRN' from dual union all
select 26713,'MRN' from dual union all
select 26714,'UKMO' from dual union all
select 26715,'UKMO' from dual union all
select 26716,'MRN' from dual union all
select 26717,'MRN' from dual union all
select 26718,'MRN' from dual union all
select 26719,'MRN' from dual union all
select 2672,'USA' from dual union all
select 26720,'MRN' from dual union all
select 26721,'MRN' from dual union all
select 26722,'MRN' from dual union all
select 26723,'MRN' from dual union all
select 26724,'MRN' from dual union all
select 26725,'MRN' from dual union all
select 26726,'MRN' from dual union all
select 26727,'MRN' from dual union all
select 26728,'MRN' from dual union all
select 26729,'MRN' from dual union all
select 2673,'USA' from dual union all
select 26730,'MRN' from dual union all
select 26731,'ESRO' from dual union all
select 26732,'MRN' from dual union all
select 26733,'MRN' from dual union all
select 26734,'MRN' from dual union all
select 26735,'MRN' from dual union all
select 26736,'MRN' from dual union all
select 26737,'MRN' from dual union all
select 26738,'ESRO' from dual union all
select 26739,'MRN' from dual union all
select 2674,'USA' from dual union all
select 26740,'MRN' from dual union all
select 26741,'MRN' from dual union all
select 26742,'MRN' from dual union all
select 26743,'MRN' from dual union all
select 26744,'MRN' from dual union all
select 26745,'MRN' from dual union all
select 26746,'MRN' from dual union all
select 26747,'MRN' from dual union all
select 26748,'MRN' from dual union all
select 26749,'MRN' from dual union all
select 2675,'USA' from dual union all
select 26750,'INTA' from dual union all
select 26751,'MRN' from dual;
insert into LAUNCH_AGENCY
select 26752,'MRN' from dual union all
select 26753,'MRN' from dual union all
select 26754,'MRN' from dual union all
select 26755,'MRN' from dual union all
select 26756,'MRN' from dual union all
select 26757,'MRN' from dual union all
select 26758,'MRN' from dual union all
select 26759,'MRN' from dual union all
select 2676,'USA' from dual union all
select 26760,'MRN' from dual union all
select 26761,'MRN' from dual union all
select 26762,'MRN' from dual union all
select 26763,'MRN' from dual union all
select 26764,'MRN' from dual union all
select 26765,'MRN' from dual union all
select 26766,'MRN' from dual union all
select 26767,'MRN' from dual union all
select 26768,'NASA' from dual union all
select 26769,'MRN' from dual union all
select 2677,'ORDRSA' from dual union all
select 26770,'MRN' from dual union all
select 26771,'MRN' from dual union all
select 26772,'MRN' from dual union all
select 26773,'MRN' from dual union all
select 26774,'MRN' from dual union all
select 26775,'MRN' from dual union all
select 26776,'MRN' from dual union all
select 26777,'MRN' from dual union all
select 26778,'MRN' from dual union all
select 26779,'MRN' from dual union all
select 2678,'ORDRSA' from dual union all
select 26780,'MRN' from dual union all
select 26781,'INTA' from dual union all
select 26782,'MRN' from dual union all
select 26783,'MRN' from dual union all
select 26784,'MRN' from dual union all
select 26785,'MRN' from dual union all
select 26786,'MRN' from dual union all
select 26787,'MRN' from dual union all
select 26788,'MRN' from dual union all
select 26789,'MRN' from dual union all
select 2679,'USA' from dual union all
select 26790,'NASA' from dual union all
select 26791,'MRN' from dual union all
select 26792,'MRN' from dual union all
select 26793,'MRN' from dual union all
select 26794,'MRN' from dual union all
select 26795,'MRN' from dual union all
select 26796,'MRN' from dual union all
select 26797,'MRN' from dual union all
select 26798,'MRN' from dual union all
select 26799,'MRN' from dual union all
select 268,'WER271-1' from dual union all
select 2680,'USA' from dual union all
select 26800,'MRN' from dual union all
select 26801,'MRN' from dual union all
select 26802,'MRN' from dual union all
select 26803,'MRN' from dual union all
select 26804,'MRN' from dual union all
select 26805,'MRN' from dual union all
select 26806,'MRN' from dual union all
select 26807,'MRN' from dual union all
select 26808,'MRN' from dual union all
select 26809,'MRN' from dual union all
select 2681,'USA' from dual union all
select 26810,'MRN' from dual union all
select 26811,'MRN' from dual union all
select 26812,'MRN' from dual union all
select 26813,'NASA' from dual union all
select 26814,'INTA' from dual union all
select 26815,'MRN' from dual union all
select 26816,'MRN' from dual union all
select 26817,'MRN' from dual union all
select 26818,'MRN' from dual union all
select 26819,'MRN' from dual union all
select 2682,'USA' from dual union all
select 26820,'MRN' from dual union all
select 26821,'NASA' from dual union all
select 26822,'MRN' from dual union all
select 26823,'MRN' from dual union all
select 26824,'MRN' from dual union all
select 26825,'MRN' from dual union all
select 26826,'MRN' from dual union all
select 26827,'MRN' from dual union all
select 26828,'MRN' from dual union all
select 26829,'MRN' from dual union all
select 2683,'ORDRSA' from dual union all
select 26830,'MRN' from dual union all
select 26831,'MRN' from dual union all
select 26832,'MRN' from dual union all
select 26833,'MRN' from dual union all
select 26834,'MRN' from dual union all
select 26835,'NASA' from dual union all
select 26836,'MRN' from dual union all
select 26837,'MRN' from dual union all
select 26838,'MRN' from dual union all
select 26839,'MRN' from dual union all
select 2684,'ORDRSA' from dual union all
select 26840,'MRN' from dual union all
select 26841,'MRN' from dual;
insert into LAUNCH_AGENCY
select 26842,'MRN' from dual union all
select 26843,'MRN' from dual union all
select 26844,'MRN' from dual union all
select 26845,'MRN' from dual union all
select 26846,'MRN' from dual union all
select 26847,'MRN' from dual union all
select 26848,'MRN' from dual union all
select 26849,'MRN' from dual union all
select 2685,'ORDRSA' from dual union all
select 26850,'MRN' from dual union all
select 26851,'MRN' from dual union all
select 26852,'MRN' from dual union all
select 26853,'MRN' from dual union all
select 26854,'MRN' from dual union all
select 26855,'MRN' from dual union all
select 26856,'MRN' from dual union all
select 26857,'MRN' from dual union all
select 26858,'MRN' from dual union all
select 26859,'MRN' from dual union all
select 2686,'USA' from dual union all
select 26860,'MRN' from dual union all
select 26861,'MRN' from dual union all
select 26862,'MRN' from dual union all
select 26863,'MRN' from dual union all
select 26864,'MRN' from dual union all
select 26865,'MRN' from dual union all
select 26866,'MRN' from dual union all
select 26867,'MRN' from dual union all
select 26868,'MRN' from dual union all
select 26869,'MRN' from dual union all
select 2687,'ORDRSA' from dual union all
select 26870,'MRN' from dual union all
select 26871,'MRN' from dual union all
select 26872,'MRN' from dual union all
select 26873,'MRN' from dual union all
select 26874,'MRN' from dual union all
select 26875,'INTA' from dual union all
select 26876,'MRN' from dual union all
select 26877,'MRN' from dual union all
select 26878,'MRN' from dual union all
select 26879,'MRN' from dual union all
select 2688,'ORDRSA' from dual union all
select 26880,'MRN' from dual union all
select 26881,'MRN' from dual union all
select 26882,'MRN' from dual union all
select 26883,'MRN' from dual union all
select 26884,'MRN' from dual union all
select 26885,'MRN' from dual union all
select 26886,'MRN' from dual union all
select 26887,'MRN' from dual union all
select 26888,'MRN' from dual union all
select 26889,'MRN' from dual union all
select 2689,'USA' from dual union all
select 26890,'UNK' from dual union all
select 26891,'UNK' from dual union all
select 26892,'MRN' from dual union all
select 26893,'MRN' from dual union all
select 26894,'MRN' from dual union all
select 26895,'MRN' from dual union all
select 26896,'MRN' from dual union all
select 26897,'MRN' from dual union all
select 26898,'MRN' from dual union all
select 26899,'MRN' from dual union all
select 269,'WER271-1' from dual union all
select 2690,'USA' from dual union all
select 26900,'MRN' from dual union all
select 26901,'INTA' from dual union all
select 26902,'MRN' from dual union all
select 26903,'MRN' from dual union all
select 26904,'MRN' from dual union all
select 26905,'MRN' from dual union all
select 26906,'MRN' from dual union all
select 26907,'MRN' from dual union all
select 26908,'MRN' from dual union all
select 26909,'MRN' from dual union all
select 2691,'USA' from dual union all
select 26910,'MRN' from dual union all
select 26911,'MRN' from dual union all
select 26912,'MRN' from dual union all
select 26913,'MRN' from dual union all
select 26914,'USA' from dual union all
select 26915,'MRN' from dual union all
select 26916,'USA' from dual union all
select 26917,'MRN' from dual union all
select 26918,'MRN' from dual union all
select 26919,'MRN' from dual union all
select 2692,'USAF' from dual union all
select 26920,'MRN' from dual union all
select 26921,'MRN' from dual union all
select 26922,'MRN' from dual union all
select 26923,'MRN' from dual union all
select 26924,'MRN' from dual union all
select 26925,'MRN' from dual union all
select 26926,'MRN' from dual union all
select 26927,'MRN' from dual union all
select 26928,'MRN' from dual union all
select 26929,'MRN' from dual union all
select 2693,'ORDRSA' from dual union all
select 26930,'MRN' from dual union all
select 26931,'MRN' from dual;
insert into LAUNCH_AGENCY
select 26932,'MRN' from dual union all
select 26933,'MRN' from dual union all
select 26934,'MRN' from dual union all
select 26935,'MRN' from dual union all
select 26936,'MRN' from dual union all
select 26937,'MRN' from dual union all
select 26938,'MRN' from dual union all
select 26939,'MRN' from dual union all
select 2694,'USA' from dual union all
select 26940,'MRN' from dual union all
select 26941,'MRN' from dual union all
select 26942,'MRN' from dual union all
select 26943,'MRN' from dual union all
select 26944,'MRN' from dual union all
select 26945,'MRN' from dual union all
select 26946,'MRN' from dual union all
select 26947,'MRN' from dual union all
select 26948,'MRN' from dual union all
select 26949,'MRN' from dual union all
select 2695,'USAF' from dual union all
select 26950,'MRN' from dual union all
select 26951,'MRN' from dual union all
select 26952,'MRN' from dual union all
select 26953,'MRN' from dual union all
select 26954,'MRN' from dual union all
select 26955,'MRN' from dual union all
select 26956,'MRN' from dual union all
select 26957,'MRN' from dual union all
select 26958,'MRN' from dual union all
select 26959,'MRN' from dual union all
select 2696,'USA' from dual union all
select 26960,'MRN' from dual union all
select 26961,'MRN' from dual union all
select 26962,'MRN' from dual union all
select 26963,'MRN' from dual union all
select 26964,'MRN' from dual union all
select 26965,'MRN' from dual union all
select 26966,'MRN' from dual union all
select 26967,'MRN' from dual union all
select 26968,'MRN' from dual union all
select 26969,'MRN' from dual union all
select 2697,'USA' from dual union all
select 26970,'MRN' from dual union all
select 26971,'MRN' from dual union all
select 26972,'MRN' from dual union all
select 26973,'INTA' from dual union all
select 26974,'MRN' from dual union all
select 26975,'MRN' from dual union all
select 26976,'MRN' from dual union all
select 26977,'MRN' from dual union all
select 26978,'MRN' from dual union all
select 26979,'MRN' from dual union all
select 2698,'USA' from dual union all
select 26980,'MRN' from dual union all
select 26981,'MRN' from dual union all
select 26982,'MRN' from dual union all
select 26983,'MRN' from dual union all
select 26984,'MRN' from dual union all
select 26985,'MRN' from dual union all
select 26986,'MRN' from dual union all
select 26987,'MRN' from dual union all
select 26988,'MRN' from dual union all
select 26989,'MRN' from dual union all
select 2699,'ORDRSA' from dual union all
select 26990,'MRN' from dual union all
select 26991,'MRN' from dual union all
select 26992,'MRN' from dual union all
select 26993,'INTA' from dual union all
select 26994,'MRN' from dual union all
select 26995,'MRN' from dual union all
select 26996,'MRN' from dual union all
select 26997,'MRN' from dual union all
select 26998,'MRN' from dual union all
select 26999,'MRN' from dual union all
select 27,'WEHR' from dual union all
select 270,'WER271-1' from dual union all
select 2700,'ORDRSA' from dual union all
select 27000,'MRN' from dual union all
select 27001,'MRN' from dual union all
select 27002,'MRN' from dual union all
select 27003,'MRN' from dual union all
select 27004,'MRN' from dual union all
select 27005,'MRN' from dual union all
select 27006,'MRN' from dual union all
select 27007,'MRN' from dual union all
select 27008,'MRN' from dual union all
select 27009,'MRN' from dual union all
select 2701,'ORDRSA' from dual union all
select 27010,'MRN' from dual union all
select 27011,'MRN' from dual union all
select 27012,'MRN' from dual union all
select 27013,'MRN' from dual union all
select 27014,'MRN' from dual union all
select 27015,'MRN' from dual union all
select 27016,'MRN' from dual union all
select 27017,'MRN' from dual union all
select 27018,'MRN' from dual union all
select 27019,'MRN' from dual union all
select 2702,'USA' from dual union all
select 27020,'MRN' from dual;
insert into LAUNCH_AGENCY
select 27021,'MRN' from dual union all
select 27022,'MRN' from dual union all
select 27023,'MRN' from dual union all
select 27024,'MRN' from dual union all
select 27025,'MRN' from dual union all
select 27026,'MRN' from dual union all
select 27027,'MRN' from dual union all
select 27028,'MRN' from dual union all
select 27029,'MRN' from dual union all
select 2703,'USA' from dual union all
select 27030,'MRN' from dual union all
select 27031,'MRN' from dual union all
select 27032,'MRN' from dual union all
select 27033,'MRN' from dual union all
select 27034,'MRN' from dual union all
select 27035,'MRN' from dual union all
select 27036,'MRN' from dual union all
select 27037,'MRN' from dual union all
select 27038,'MRN' from dual union all
select 27039,'MRN' from dual union all
select 2704,'USA' from dual union all
select 27040,'MRN' from dual union all
select 27041,'MRN' from dual union all
select 27042,'MRN' from dual union all
select 27043,'MRN' from dual union all
select 27044,'MRN' from dual union all
select 27045,'MRN' from dual union all
select 27046,'MRN' from dual union all
select 27047,'MRN' from dual union all
select 27048,'MRN' from dual union all
select 27049,'MRN' from dual union all
select 2705,'USA' from dual union all
select 27050,'MRN' from dual union all
select 27051,'MRN' from dual union all
select 27052,'MRN' from dual union all
select 27053,'MRN' from dual union all
select 27054,'MRN' from dual union all
select 27055,'MRN' from dual union all
select 27056,'MRN' from dual union all
select 27057,'MRN' from dual union all
select 27058,'MRN' from dual union all
select 27059,'MRN' from dual union all
select 2706,'ORDRSA' from dual union all
select 27060,'MRN' from dual union all
select 27061,'MRN' from dual union all
select 27062,'MRN' from dual union all
select 27063,'MRN' from dual union all
select 27064,'MRN' from dual union all
select 27065,'MRN' from dual union all
select 27066,'MRN' from dual union all
select 27067,'MRN' from dual union all
select 27068,'MRN' from dual union all
select 27069,'MRN' from dual union all
select 2707,'USA' from dual union all
select 27070,'MRN' from dual union all
select 27071,'MRN' from dual union all
select 27072,'MRN' from dual union all
select 27073,'MRN' from dual union all
select 27074,'MRN' from dual union all
select 27075,'MRN' from dual union all
select 27076,'MRN' from dual union all
select 27077,'MRN' from dual union all
select 27078,'MRN' from dual union all
select 27079,'MRN' from dual union all
select 2708,'USA' from dual union all
select 27080,'MRN' from dual union all
select 27081,'MRN' from dual union all
select 27082,'MRN' from dual union all
select 27083,'MRN' from dual union all
select 27084,'MRN' from dual union all
select 27085,'MRN' from dual union all
select 27086,'MRN' from dual union all
select 27087,'MRN' from dual union all
select 27088,'MRN' from dual union all
select 27089,'MRN' from dual union all
select 2709,'USA' from dual union all
select 27090,'MRN' from dual union all
select 27091,'MRN' from dual union all
select 27092,'MRN' from dual union all
select 27093,'MRN' from dual union all
select 27094,'MRN' from dual union all
select 27095,'MRN' from dual union all
select 27096,'MRN' from dual union all
select 27097,'MRN' from dual union all
select 27098,'MRN' from dual union all
select 27099,'INTA' from dual union all
select 271,'WER271-1' from dual union all
select 2710,'ORDRSA' from dual union all
select 27100,'MRN' from dual union all
select 27101,'MRN' from dual union all
select 27102,'MRN' from dual union all
select 27103,'MRN' from dual union all
select 27104,'MRN' from dual union all
select 27105,'MRN' from dual union all
select 27106,'MRN' from dual union all
select 27107,'MRN' from dual union all
select 27108,'MRN' from dual union all
select 27109,'MRN' from dual union all
select 2711,'ORDRSA' from dual union all
select 27110,'MRN' from dual;
insert into LAUNCH_AGENCY
select 27111,'MRN' from dual union all
select 27112,'MRN' from dual union all
select 27113,'MRN' from dual union all
select 27114,'MRN' from dual union all
select 27115,'MRN' from dual union all
select 27116,'MRN' from dual union all
select 27117,'MRN' from dual union all
select 27118,'MRN' from dual union all
select 27119,'MRN' from dual union all
select 2712,'USA' from dual union all
select 27120,'MRN' from dual union all
select 27121,'MRN' from dual union all
select 27122,'MRN' from dual union all
select 27123,'MRN' from dual union all
select 27124,'MRN' from dual union all
select 27125,'MRN' from dual union all
select 27126,'MRN' from dual union all
select 27127,'MRN' from dual union all
select 27128,'MRN' from dual union all
select 27129,'MRN' from dual union all
select 2713,'USA' from dual union all
select 27130,'MRN' from dual union all
select 27131,'MRN' from dual union all
select 27132,'MRN' from dual union all
select 27133,'MRN' from dual union all
select 27134,'MRN' from dual union all
select 27135,'INTA' from dual union all
select 27136,'MRN' from dual union all
select 27137,'MRN' from dual union all
select 27138,'MRN' from dual union all
select 27139,'MRN' from dual union all
select 2714,'USA' from dual union all
select 27140,'MRN' from dual union all
select 27141,'MRN' from dual union all
select 27142,'MRN' from dual union all
select 27143,'MRN' from dual union all
select 27144,'MRN' from dual union all
select 27145,'MRN' from dual union all
select 27146,'MRN' from dual union all
select 27147,'MRN' from dual union all
select 27148,'MRN' from dual union all
select 27149,'MRN' from dual union all
select 2715,'USA' from dual union all
select 27150,'MRN' from dual union all
select 27151,'MRN' from dual union all
select 27152,'MRN' from dual union all
select 27153,'MRN' from dual union all
select 27154,'MRN' from dual union all
select 27155,'MRN' from dual union all
select 27156,'MRN' from dual union all
select 27157,'MRN' from dual union all
select 27158,'MRN' from dual union all
select 27159,'MRN' from dual union all
select 2716,'USA' from dual union all
select 27160,'MRN' from dual union all
select 27161,'MRN' from dual union all
select 27162,'MRN' from dual union all
select 27163,'MRN' from dual union all
select 27164,'MRN' from dual union all
select 27165,'MRN' from dual union all
select 27166,'MRN' from dual union all
select 27167,'MRN' from dual union all
select 27168,'MRN' from dual union all
select 27169,'MRN' from dual union all
select 2717,'ORDRSA' from dual union all
select 27170,'MRN' from dual union all
select 27171,'MRN' from dual union all
select 27172,'MRN' from dual union all
select 27173,'MRN' from dual union all
select 27174,'MRN' from dual union all
select 27175,'MRN' from dual union all
select 27176,'MRN' from dual union all
select 27177,'MRN' from dual union all
select 27178,'MRN' from dual union all
select 27179,'MRN' from dual union all
select 2718,'ORDRSA' from dual union all
select 27180,'MRN' from dual union all
select 27181,'MRN' from dual union all
select 27182,'MRN' from dual union all
select 27183,'MRN' from dual union all
select 27184,'MRN' from dual union all
select 27185,'MRN' from dual union all
select 27186,'MRN' from dual union all
select 27187,'MRN' from dual union all
select 27188,'MRN' from dual union all
select 27189,'MRN' from dual union all
select 2719,'USA' from dual union all
select 27190,'MRN' from dual union all
select 27191,'MRN' from dual union all
select 27192,'MRN' from dual union all
select 27193,'MRN' from dual union all
select 27194,'MRN' from dual union all
select 27195,'MRN' from dual union all
select 27196,'MRN' from dual union all
select 27197,'MRN' from dual union all
select 27198,'MRN' from dual union all
select 27199,'MRN' from dual union all
select 272,'WER271-1' from dual union all
select 2720,'USA' from dual union all
select 27200,'MRN' from dual;
insert into LAUNCH_AGENCY
select 27201,'MRN' from dual union all
select 27202,'MRN' from dual union all
select 27203,'MRN' from dual union all
select 27204,'MRN' from dual union all
select 27205,'MRN' from dual union all
select 27206,'MRN' from dual union all
select 27207,'MRN' from dual union all
select 27208,'MRN' from dual union all
select 27209,'MRN' from dual union all
select 2721,'USA' from dual union all
select 27210,'MRN' from dual union all
select 27211,'MRN' from dual union all
select 27212,'AN' from dual union all
select 27213,'MRN' from dual union all
select 27214,'MRN' from dual union all
select 27215,'MRN' from dual union all
select 27216,'MRN' from dual union all
select 27217,'MRN' from dual union all
select 27218,'INTA' from dual union all
select 27219,'MRN' from dual union all
select 2722,'USA' from dual union all
select 27220,'MRN' from dual union all
select 27221,'MRN' from dual union all
select 27222,'MRN' from dual union all
select 27223,'MRN' from dual union all
select 27224,'MRN' from dual union all
select 27225,'MRN' from dual union all
select 27226,'MRN' from dual union all
select 27227,'MRN' from dual union all
select 27228,'MRN' from dual union all
select 27229,'MRN' from dual union all
select 2723,'USA' from dual union all
select 27230,'MRN' from dual union all
select 27231,'MRN' from dual union all
select 27232,'MRN' from dual union all
select 27233,'MRN' from dual union all
select 27234,'MRN' from dual union all
select 27235,'ESRO' from dual union all
select 27236,'MRN' from dual union all
select 27237,'MRN' from dual union all
select 27238,'MRN' from dual union all
select 27239,'MRN' from dual union all
select 2724,'USA' from dual union all
select 27240,'MRN' from dual union all
select 27241,'MRN' from dual union all
select 27242,'MRN' from dual union all
select 27243,'MRN' from dual union all
select 27244,'MRN' from dual union all
select 27245,'MRN' from dual union all
select 27246,'MRN' from dual union all
select 27247,'MRN' from dual union all
select 27248,'MRN' from dual union all
select 27249,'MRN' from dual union all
select 2725,'USA' from dual union all
select 27250,'MRN' from dual union all
select 27251,'MRN' from dual union all
select 27252,'MRN' from dual union all
select 27253,'MRN' from dual union all
select 27254,'MRN' from dual union all
select 27255,'MRN' from dual union all
select 27256,'MRN' from dual union all
select 27257,'MRN' from dual union all
select 27258,'MRN' from dual union all
select 27259,'MRN' from dual union all
select 2726,'USA' from dual union all
select 27260,'MRN' from dual union all
select 27261,'MRN' from dual union all
select 27262,'MRN' from dual union all
select 27263,'MRN' from dual union all
select 27264,'MRN' from dual union all
select 27265,'MRN' from dual union all
select 27266,'MRN' from dual union all
select 27267,'MRN' from dual union all
select 27268,'MRN' from dual union all
select 27269,'MRN' from dual union all
select 2727,'USAF' from dual union all
select 27270,'MRN' from dual union all
select 27271,'MRN' from dual union all
select 27272,'MRN' from dual union all
select 27273,'MRN' from dual union all
select 27274,'MRN' from dual union all
select 27275,'MRN' from dual union all
select 27276,'MRN' from dual union all
select 27277,'MRN' from dual union all
select 27278,'MRN' from dual union all
select 27279,'MRN' from dual union all
select 2728,'USAF' from dual union all
select 27280,'MRN' from dual union all
select 27281,'MRN' from dual union all
select 27282,'MRN' from dual union all
select 27283,'MRN' from dual union all
select 27284,'MRN' from dual union all
select 27285,'MRN' from dual union all
select 27286,'MRN' from dual union all
select 27287,'MRN' from dual union all
select 27288,'MRN' from dual union all
select 27289,'MRN' from dual union all
select 2729,'USA' from dual union all
select 27290,'MRN' from dual union all
select 27291,'MRN' from dual;
insert into LAUNCH_AGENCY
select 27292,'MRN' from dual union all
select 27293,'MRN' from dual union all
select 27294,'MRN' from dual union all
select 27295,'MRN' from dual union all
select 27296,'MRN' from dual union all
select 27297,'MRN' from dual union all
select 27298,'MRN' from dual union all
select 27299,'MRN' from dual union all
select 273,'WER271-1' from dual union all
select 2730,'USA' from dual union all
select 27300,'MRN' from dual union all
select 27301,'MRN' from dual union all
select 27302,'MRN' from dual union all
select 27303,'MRN' from dual union all
select 27304,'MRN' from dual union all
select 27305,'MRN' from dual union all
select 27306,'MRN' from dual union all
select 27307,'MRN' from dual union all
select 27308,'MRN' from dual union all
select 27309,'MRN' from dual union all
select 2731,'USA' from dual union all
select 27310,'MRN' from dual union all
select 27311,'MRN' from dual union all
select 27312,'MRN' from dual union all
select 27313,'MRN' from dual union all
select 27314,'MRN' from dual union all
select 27315,'MRN' from dual union all
select 27316,'MRN' from dual union all
select 27317,'MRN' from dual union all
select 27318,'MRN' from dual union all
select 27319,'MRN' from dual union all
select 2732,'USA' from dual union all
select 27320,'MRN' from dual union all
select 27321,'MRN' from dual union all
select 27322,'MRN' from dual union all
select 27323,'MRN' from dual union all
select 27324,'MRN' from dual union all
select 27325,'MRN' from dual union all
select 27326,'MRN' from dual union all
select 27327,'MRN' from dual union all
select 27328,'MRN' from dual union all
select 27329,'MRN' from dual union all
select 2733,'ORDRSA' from dual union all
select 27330,'MRN' from dual union all
select 27331,'MRN' from dual union all
select 27332,'MRN' from dual union all
select 27333,'MRN' from dual union all
select 27334,'MRN' from dual union all
select 27335,'MRN' from dual union all
select 27336,'MRN' from dual union all
select 27337,'USA' from dual union all
select 27338,'MRN' from dual union all
select 27339,'MRN' from dual union all
select 2734,'ORDRSA' from dual union all
select 27340,'MRN' from dual union all
select 27341,'MRN' from dual union all
select 27342,'MRN' from dual union all
select 27343,'MRN' from dual union all
select 27344,'MRN' from dual union all
select 27345,'MRN' from dual union all
select 27346,'MRN' from dual union all
select 27347,'MRN' from dual union all
select 27348,'MRN' from dual union all
select 27349,'MRN' from dual union all
select 2735,'USA' from dual union all
select 27350,'MRN' from dual union all
select 27351,'MRN' from dual union all
select 27352,'MRN' from dual union all
select 27353,'MRN' from dual union all
select 27354,'MRN' from dual union all
select 27355,'MRN' from dual union all
select 27356,'MRN' from dual union all
select 27357,'MRN' from dual union all
select 27358,'MRN' from dual union all
select 27359,'MRN' from dual union all
select 2736,'USA' from dual union all
select 27360,'MRN' from dual union all
select 27361,'MRN' from dual union all
select 27362,'MRN' from dual union all
select 27363,'MRN' from dual union all
select 27364,'MRN' from dual union all
select 27365,'MRN' from dual union all
select 27366,'MRN' from dual union all
select 27367,'MRN' from dual union all
select 27368,'MRN' from dual union all
select 27369,'INTA' from dual union all
select 2737,'ORDRSA' from dual union all
select 27370,'MRN' from dual union all
select 27371,'MRN' from dual union all
select 27372,'MRN' from dual union all
select 27373,'MRN' from dual union all
select 27374,'MRN' from dual union all
select 27375,'MRN' from dual union all
select 27376,'MRN' from dual union all
select 27377,'MRN' from dual union all
select 27378,'MRN' from dual union all
select 27379,'MRN' from dual union all
select 2738,'ORDRSA' from dual union all
select 27380,'MRN' from dual union all
select 27381,'ARC' from dual;
insert into LAUNCH_AGENCY
select 27382,'MRN' from dual union all
select 27383,'ARC' from dual union all
select 27384,'MRN' from dual union all
select 27385,'MRN' from dual union all
select 27386,'MRN' from dual union all
select 27387,'MRN' from dual union all
select 27388,'MRN' from dual union all
select 27389,'MRN' from dual union all
select 2739,'LRBA' from dual union all
select 27390,'MRN' from dual union all
select 27391,'MRN' from dual union all
select 27392,'MRN' from dual union all
select 27393,'MRN' from dual union all
select 27394,'MRN' from dual union all
select 27395,'MRN' from dual union all
select 27396,'MRN' from dual union all
select 27397,'MRN' from dual union all
select 27398,'MRN' from dual union all
select 27399,'MRN' from dual union all
select 274,'WER271-1' from dual union all
select 2740,'USA' from dual union all
select 27400,'MRN' from dual union all
select 27401,'MRN' from dual union all
select 27402,'MRN' from dual union all
select 27403,'MRN' from dual union all
select 27404,'MRN' from dual union all
select 27405,'MRN' from dual union all
select 27406,'MRN' from dual union all
select 27407,'MRN' from dual union all
select 27408,'MRN' from dual union all
select 27409,'MRN' from dual union all
select 2741,'USA' from dual union all
select 27410,'MRN' from dual union all
select 27411,'MRN' from dual union all
select 27412,'MRN' from dual union all
select 27413,'MRN' from dual union all
select 27414,'MRN' from dual union all
select 27415,'MRN' from dual union all
select 27416,'MRN' from dual union all
select 27417,'MRN' from dual union all
select 27418,'MRN' from dual union all
select 27419,'MRN' from dual union all
select 2742,'USA' from dual union all
select 27420,'MRN' from dual union all
select 27421,'MRN' from dual union all
select 27422,'MRN' from dual union all
select 27423,'MRN' from dual union all
select 27424,'MRN' from dual union all
select 27425,'MRN' from dual union all
select 27426,'MRN' from dual union all
select 27427,'MRN' from dual union all
select 27428,'MRN' from dual union all
select 27429,'MRN' from dual union all
select 2743,'USA' from dual union all
select 27430,'MRN' from dual union all
select 27431,'MRN' from dual union all
select 27432,'MRN' from dual union all
select 27433,'MRN' from dual union all
select 27434,'MRN' from dual union all
select 27435,'MRN' from dual union all
select 27436,'MRN' from dual union all
select 27437,'MRN' from dual union all
select 27438,'MRN' from dual union all
select 27439,'MRN' from dual union all
select 2744,'USA' from dual union all
select 27440,'MRN' from dual union all
select 27441,'MRN' from dual union all
select 27442,'MRN' from dual union all
select 27443,'MRN' from dual union all
select 27444,'MRN' from dual union all
select 27445,'MRN' from dual union all
select 27446,'MRN' from dual union all
select 27447,'MRN' from dual union all
select 27448,'MRN' from dual union all
select 27449,'MRN' from dual union all
select 2745,'USA' from dual union all
select 27450,'MRN' from dual union all
select 27451,'MRN' from dual union all
select 27452,'MRN' from dual union all
select 27453,'MRN' from dual union all
select 27454,'MRN' from dual union all
select 27455,'MRN' from dual union all
select 27456,'MRN' from dual union all
select 27457,'MRN' from dual union all
select 27458,'MRN' from dual union all
select 27459,'MRN' from dual union all
select 2746,'USA' from dual union all
select 27460,'MRN' from dual union all
select 27461,'MRN' from dual union all
select 27462,'MRN' from dual union all
select 27463,'MRN' from dual union all
select 27464,'MRN' from dual union all
select 27465,'MRN' from dual union all
select 27466,'MRN' from dual union all
select 27467,'MRN' from dual union all
select 27468,'MRN' from dual union all
select 27469,'MRN' from dual union all
select 2747,'PARD' from dual union all
select 27470,'MRN' from dual union all
select 27471,'MRN' from dual;
insert into LAUNCH_AGENCY
select 27472,'MRN' from dual union all
select 27473,'MRN' from dual union all
select 27474,'MRN' from dual union all
select 27475,'MRN' from dual union all
select 27476,'MRN' from dual union all
select 27477,'ESRO' from dual union all
select 27478,'MRN' from dual union all
select 27479,'MRN' from dual union all
select 2748,'USAF' from dual union all
select 27480,'MRN' from dual union all
select 27481,'MRN' from dual union all
select 27482,'MRN' from dual union all
select 27483,'MRN' from dual union all
select 27484,'MRN' from dual union all
select 27485,'MRN' from dual union all
select 27486,'MRN' from dual union all
select 27487,'MRN' from dual union all
select 27488,'MRN' from dual union all
select 27489,'MRN' from dual union all
select 2749,'USA' from dual union all
select 27490,'MRN' from dual union all
select 27491,'MRN' from dual union all
select 27492,'MRN' from dual union all
select 27493,'MRN' from dual union all
select 27494,'MRN' from dual union all
select 27495,'MRN' from dual union all
select 27496,'MRN' from dual union all
select 27497,'MRN' from dual union all
select 27498,'MRN' from dual union all
select 27499,'MRN' from dual union all
select 275,'WEHR' from dual union all
select 2750,'ORDRSA' from dual union all
select 27500,'MRN' from dual union all
select 27501,'MRN' from dual union all
select 27502,'MRN' from dual union all
select 27503,'MRN' from dual union all
select 27504,'MRN' from dual union all
select 27505,'MRN' from dual union all
select 27506,'MRN' from dual union all
select 27507,'MRN' from dual union all
select 27508,'MRN' from dual union all
select 27509,'MRN' from dual union all
select 2751,'PARD' from dual union all
select 27510,'MRN' from dual union all
select 27511,'MRN' from dual union all
select 27512,'MRN' from dual union all
select 27513,'MRN' from dual union all
select 27514,'MRN' from dual union all
select 27515,'MRN' from dual union all
select 27516,'MRN' from dual union all
select 27517,'MRN' from dual union all
select 27518,'MRN' from dual union all
select 27519,'MRN' from dual union all
select 2752,'USN' from dual union all
select 27520,'MRN' from dual union all
select 27521,'UKMO' from dual union all
select 27522,'MRN' from dual union all
select 27523,'MRN' from dual union all
select 27524,'MRN' from dual union all
select 27525,'MRN' from dual union all
select 27526,'MRN' from dual union all
select 27527,'MRN' from dual union all
select 27528,'MRN' from dual union all
select 27529,'MRN' from dual union all
select 2753,'STA' from dual union all
select 27530,'MRN' from dual union all
select 27531,'MRN' from dual union all
select 27532,'MRN' from dual union all
select 27533,'MRN' from dual union all
select 27534,'MRN' from dual union all
select 27535,'MRN' from dual union all
select 27536,'MRN' from dual union all
select 27537,'MRN' from dual union all
select 27538,'MRN' from dual union all
select 27539,'MRN' from dual union all
select 2754,'ORDRSA' from dual union all
select 27540,'MRN' from dual union all
select 27541,'MRN' from dual union all
select 27542,'MRN' from dual union all
select 27543,'INTA' from dual union all
select 27544,'MRN' from dual union all
select 27545,'MRN' from dual union all
select 27546,'MRN' from dual union all
select 27547,'MRN' from dual union all
select 27548,'MRN' from dual union all
select 27549,'MRN' from dual union all
select 2755,'ORDRSA' from dual union all
select 27550,'MRN' from dual union all
select 27551,'MRN' from dual union all
select 27552,'MRN' from dual union all
select 27553,'MRN' from dual union all
select 27554,'AN' from dual union all
select 27555,'AN' from dual union all
select 27556,'AN' from dual union all
select 27557,'MRN' from dual union all
select 27558,'AN' from dual union all
select 27559,'AN' from dual union all
select 2756,'ORDRSA' from dual union all
select 27560,'AN' from dual union all
select 27561,'MRN' from dual;
insert into LAUNCH_AGENCY
select 27562,'AN' from dual union all
select 27563,'AN' from dual union all
select 27564,'MRN' from dual union all
select 27565,'AN' from dual union all
select 27566,'AN' from dual union all
select 27567,'AN' from dual union all
select 27568,'AN' from dual union all
select 27569,'MRN' from dual union all
select 2757,'ORDRSA' from dual union all
select 27570,'MRN' from dual union all
select 27571,'AN' from dual union all
select 27572,'AN' from dual union all
select 27573,'AN' from dual union all
select 27574,'AN' from dual union all
select 27575,'AN' from dual union all
select 27576,'MRN' from dual union all
select 27577,'MRN' from dual union all
select 27578,'AN' from dual union all
select 27579,'MRN' from dual union all
select 2758,'USA' from dual union all
select 27580,'MRN' from dual union all
select 27581,'AN' from dual union all
select 27582,'AN' from dual union all
select 27583,'AN' from dual union all
select 27584,'AN' from dual union all
select 27585,'AN' from dual union all
select 27586,'AN' from dual union all
select 27587,'MRN' from dual union all
select 27588,'MRN' from dual union all
select 27589,'AN' from dual union all
select 2759,'USA' from dual union all
select 27590,'AN' from dual union all
select 27591,'AN' from dual union all
select 27592,'MRN' from dual union all
select 27593,'AN' from dual union all
select 27594,'AN' from dual union all
select 27595,'AN' from dual union all
select 27596,'AN' from dual union all
select 27597,'MRN' from dual union all
select 27598,'MRN' from dual union all
select 27599,'AN' from dual union all
select 276,'WER271-1' from dual union all
select 2760,'USN' from dual union all
select 27600,'AN' from dual union all
select 27601,'MRN' from dual union all
select 27602,'AN' from dual union all
select 27603,'MRN' from dual union all
select 27604,'AN' from dual union all
select 27605,'AN' from dual union all
select 27606,'AN' from dual union all
select 27607,'AN' from dual union all
select 27608,'MRN' from dual union all
select 27609,'AN' from dual union all
select 2761,'ORDRSA' from dual union all
select 27610,'AN' from dual union all
select 27611,'AN' from dual union all
select 27612,'AN' from dual union all
select 27613,'MRN' from dual union all
select 27614,'MRN' from dual union all
select 27615,'MRN' from dual union all
select 27616,'AN' from dual union all
select 27617,'AN' from dual union all
select 27618,'AN' from dual union all
select 27619,'MRN' from dual union all
select 2762,'USN' from dual union all
select 27620,'AN' from dual union all
select 27621,'AN' from dual union all
select 27622,'MRN' from dual union all
select 27623,'MRN' from dual union all
select 27624,'MRN' from dual union all
select 27625,'AN' from dual union all
select 27626,'AN' from dual union all
select 27627,'MRN' from dual union all
select 27628,'AN' from dual union all
select 27629,'AN' from dual union all
select 2763,'USN' from dual union all
select 27630,'AN' from dual union all
select 27631,'AN' from dual union all
select 27632,'AN' from dual union all
select 27633,'AN' from dual union all
select 27634,'MRN' from dual union all
select 27635,'AN' from dual union all
select 27636,'AN' from dual union all
select 27637,'AN' from dual union all
select 27638,'AN' from dual union all
select 27639,'AN' from dual union all
select 2764,'USN' from dual union all
select 27640,'AN' from dual union all
select 27641,'AN' from dual union all
select 27642,'MRN' from dual union all
select 27643,'MRN' from dual union all
select 27644,'AN' from dual union all
select 27645,'AN' from dual union all
select 27646,'MRN' from dual union all
select 27647,'AN' from dual union all
select 27648,'MRN' from dual union all
select 27649,'AN' from dual union all
select 2765,'USN' from dual union all
select 27650,'AN' from dual union all
select 27651,'MRN' from dual;
insert into LAUNCH_AGENCY
select 27652,'AN' from dual union all
select 27653,'AN' from dual union all
select 27654,'AN' from dual union all
select 27655,'AN' from dual union all
select 27656,'AN' from dual union all
select 27657,'AN' from dual union all
select 27658,'AN' from dual union all
select 27659,'ARC' from dual union all
select 2766,'USN' from dual union all
select 27660,'ARC' from dual union all
select 27661,'AN' from dual union all
select 27662,'AN' from dual union all
select 27663,'MRN' from dual union all
select 27664,'AN' from dual union all
select 27665,'MRN' from dual union all
select 27666,'AN' from dual union all
select 27667,'AN' from dual union all
select 27668,'AN' from dual union all
select 27669,'AN' from dual union all
select 2767,'ARDC' from dual union all
select 27670,'AN' from dual union all
select 27671,'AN' from dual union all
select 27672,'AN' from dual union all
select 27673,'AN' from dual union all
select 27674,'MRN' from dual union all
select 27675,'SRC' from dual union all
select 27676,'AN' from dual union all
select 27677,'MRN' from dual union all
select 27678,'AN' from dual union all
select 27679,'AN' from dual union all
select 2768,'ARDC' from dual union all
select 27680,'AN' from dual union all
select 27681,'MRN' from dual union all
select 27682,'AN' from dual union all
select 27683,'MRN' from dual union all
select 27684,'AN' from dual union all
select 27685,'AN' from dual union all
select 27686,'MRN' from dual union all
select 27687,'AN' from dual union all
select 27688,'AN' from dual union all
select 27689,'AN' from dual union all
select 2769,'ARDC' from dual union all
select 27690,'AN' from dual union all
select 27691,'MRN' from dual union all
select 27692,'SRC' from dual union all
select 27693,'AN' from dual union all
select 27694,'AN' from dual union all
select 27695,'AN' from dual union all
select 27696,'MRN' from dual union all
select 27697,'MRN' from dual union all
select 27698,'AN' from dual union all
select 27699,'MRN' from dual union all
select 277,'WER271-1' from dual union all
select 2770,'ARDC' from dual union all
select 27700,'AN' from dual union all
select 27701,'AN' from dual union all
select 27702,'MRN' from dual union all
select 27703,'AN' from dual union all
select 27704,'MRN' from dual union all
select 27705,'AN' from dual union all
select 27706,'AN' from dual union all
select 27707,'MRN' from dual union all
select 27708,'SRC' from dual union all
select 27709,'AN' from dual union all
select 2771,'USA' from dual union all
select 27710,'AN' from dual union all
select 27711,'MRN' from dual union all
select 27712,'AN' from dual union all
select 27713,'AN' from dual union all
select 27714,'AN' from dual union all
select 27715,'MRN' from dual union all
select 27716,'MRN' from dual union all
select 27717,'AN' from dual union all
select 27718,'MRN' from dual union all
select 27719,'AN' from dual union all
select 2772,'PARD' from dual union all
select 27720,'MRN' from dual union all
select 27721,'AN' from dual union all
select 27722,'AN' from dual union all
select 27723,'AN' from dual union all
select 27724,'MRN' from dual union all
select 27725,'MRN' from dual union all
select 27726,'AN' from dual union all
select 27727,'AN' from dual union all
select 27728,'AN' from dual union all
select 27729,'AN' from dual union all
select 2773,'USA' from dual union all
select 27730,'AN' from dual union all
select 27731,'AN' from dual union all
select 27732,'MRN' from dual union all
select 27733,'WFF' from dual union all
select 27734,'AN' from dual union all
select 27735,'AN' from dual union all
select 27736,'AN' from dual union all
select 27737,'MRN' from dual union all
select 27738,'AN' from dual union all
select 27739,'AN' from dual union all
select 2774,'WADD' from dual union all
select 27740,'MRN' from dual union all
select 27741,'AN' from dual;
insert into LAUNCH_AGENCY
select 27742,'AN' from dual union all
select 27743,'AN' from dual union all
select 27744,'AN' from dual union all
select 27745,'MRN' from dual union all
select 27746,'AN' from dual union all
select 27747,'AN' from dual union all
select 27748,'AN' from dual union all
select 27749,'AN' from dual union all
select 2775,'WADD' from dual union all
select 27750,'AN' from dual union all
select 27751,'MRN' from dual union all
select 27752,'AN' from dual union all
select 27753,'AN' from dual union all
select 27754,'AN' from dual union all
select 27755,'AN' from dual union all
select 27756,'AN' from dual union all
select 27757,'SRC' from dual union all
select 27758,'AN' from dual union all
select 27759,'AN' from dual union all
select 2776,'WADD' from dual union all
select 27760,'AN' from dual union all
select 27761,'UKMO' from dual union all
select 27762,'AN' from dual union all
select 27763,'AN' from dual union all
select 27764,'MRN' from dual union all
select 27765,'AN' from dual union all
select 27766,'MRN' from dual union all
select 27767,'AN' from dual union all
select 27768,'AN' from dual union all
select 27769,'AN' from dual union all
select 2777,'WADD' from dual union all
select 27770,'AN' from dual union all
select 27771,'AN' from dual union all
select 27772,'AN' from dual union all
select 27773,'AN' from dual union all
select 27774,'AN' from dual union all
select 27775,'AN' from dual union all
select 27776,'AN' from dual union all
select 27777,'AN' from dual union all
select 27778,'AN' from dual union all
select 27779,'AN' from dual union all
select 2778,'WADD' from dual union all
select 27780,'AN' from dual union all
select 27781,'AN' from dual union all
select 27782,'AN' from dual union all
select 27783,'AN' from dual union all
select 27784,'MRN' from dual union all
select 27785,'AN' from dual union all
select 27786,'AN' from dual union all
select 27787,'AN' from dual union all
select 27788,'AN' from dual union all
select 27789,'AN' from dual union all
select 2779,'WADD' from dual union all
select 27790,'AN' from dual union all
select 27791,'AN' from dual union all
select 27792,'AN' from dual union all
select 27793,'AN' from dual union all
select 27794,'AN' from dual union all
select 27795,'AN' from dual union all
select 27796,'AN' from dual union all
select 27797,'AN' from dual union all
select 27798,'AN' from dual union all
select 27799,'AN' from dual union all
select 278,'WER271-1' from dual union all
select 2780,'WADD' from dual union all
select 27800,'SRC' from dual union all
select 27801,'AN' from dual union all
select 27802,'AN' from dual union all
select 27803,'MRN' from dual union all
select 27804,'AN' from dual union all
select 27805,'AN' from dual union all
select 27806,'AN' from dual union all
select 27807,'AN' from dual union all
select 27808,'AN' from dual union all
select 27809,'AN' from dual union all
select 2781,'WADD' from dual union all
select 27810,'AN' from dual union all
select 27811,'MRN' from dual union all
select 27812,'AN' from dual union all
select 27813,'AN' from dual union all
select 27814,'MRN' from dual union all
select 27815,'AN' from dual union all
select 27816,'MRN' from dual union all
select 27817,'MRN' from dual union all
select 27818,'MRN' from dual union all
select 27819,'AN' from dual union all
select 2782,'UZKA' from dual union all
select 27820,'AN' from dual union all
select 27821,'AN' from dual union all
select 27822,'MRN' from dual union all
select 27823,'AN' from dual union all
select 27824,'AN' from dual union all
select 27825,'MRN' from dual union all
select 27826,'AN' from dual union all
select 27827,'AN' from dual union all
select 27828,'AN' from dual union all
select 27829,'AN' from dual union all
select 2783,'ORDRSA' from dual union all
select 27830,'MRN' from dual union all
select 27831,'SRC' from dual;
insert into LAUNCH_AGENCY
select 27832,'AN' from dual union all
select 27833,'AN' from dual union all
select 27834,'AN' from dual union all
select 27835,'AN' from dual union all
select 27836,'MRN' from dual union all
select 27837,'AN' from dual union all
select 27838,'UKMO' from dual union all
select 27839,'AN' from dual union all
select 2784,'ORDRSA' from dual union all
select 27840,'MRN' from dual union all
select 27841,'AN' from dual union all
select 27842,'MRN' from dual union all
select 27843,'AN' from dual union all
select 27844,'AN' from dual union all
select 27845,'AN' from dual union all
select 27846,'AN' from dual union all
select 27847,'AN' from dual union all
select 27848,'AN' from dual union all
select 27849,'AN' from dual union all
select 2785,'ORDRSA' from dual union all
select 27850,'AN' from dual union all
select 27851,'AN' from dual union all
select 27852,'MRN' from dual union all
select 27853,'AN' from dual union all
select 27854,'AN' from dual union all
select 27855,'MRN' from dual union all
select 27856,'MRN' from dual union all
select 27857,'MRN' from dual union all
select 27858,'AN' from dual union all
select 27859,'AN' from dual union all
select 2786,'ORDRSA' from dual union all
select 27860,'AN' from dual union all
select 27861,'AN' from dual union all
select 27862,'AN' from dual union all
select 27863,'AN' from dual union all
select 27864,'AN' from dual union all
select 27865,'AN' from dual union all
select 27866,'AN' from dual union all
select 27867,'AN' from dual union all
select 27868,'AN' from dual union all
select 27869,'MRN' from dual union all
select 2787,'ORDRSA' from dual union all
select 27870,'MRN' from dual union all
select 27871,'AN' from dual union all
select 27872,'MRN' from dual union all
select 27873,'AN' from dual union all
select 27874,'AN' from dual union all
select 27875,'AN' from dual union all
select 27876,'AN' from dual union all
select 27877,'MRN' from dual union all
select 27878,'AN' from dual union all
select 27879,'NASA' from dual union all
select 2788,'ORDRSA' from dual union all
select 27880,'AN' from dual union all
select 27881,'AN' from dual union all
select 27882,'AN' from dual union all
select 27883,'AN' from dual union all
select 27884,'AN' from dual union all
select 27885,'MRN' from dual union all
select 27886,'MRN' from dual union all
select 27887,'AN' from dual union all
select 27888,'MRN' from dual union all
select 27889,'AN' from dual union all
select 2789,'USAF' from dual union all
select 27890,'AN' from dual union all
select 27891,'AN' from dual union all
select 27892,'AN' from dual union all
select 27893,'AN' from dual union all
select 27894,'MRN' from dual union all
select 27895,'AN' from dual union all
select 27896,'MRN' from dual union all
select 27897,'AN' from dual union all
select 27898,'AN' from dual union all
select 27899,'AN' from dual union all
select 279,'WER271-1' from dual union all
select 2790,'USA' from dual union all
select 27900,'AN' from dual union all
select 27901,'AN' from dual union all
select 27902,'AN' from dual union all
select 27903,'AN' from dual union all
select 27904,'AN' from dual union all
select 27905,'AN' from dual union all
select 27906,'MRN' from dual union all
select 27907,'AN' from dual union all
select 27908,'MRN' from dual union all
select 27909,'AN' from dual union all
select 2791,'ORDRSA' from dual union all
select 27910,'AN' from dual union all
select 27911,'AN' from dual union all
select 27912,'AN' from dual union all
select 27913,'MRN' from dual union all
select 27914,'MRN' from dual union all
select 27915,'AN' from dual union all
select 27916,'MRN' from dual union all
select 27917,'AN' from dual union all
select 27918,'AN' from dual union all
select 27919,'AN' from dual union all
select 2792,'ORDRSA' from dual union all
select 27920,'AN' from dual union all
select 27921,'MRN' from dual;
insert into LAUNCH_AGENCY
select 27922,'AN' from dual union all
select 27923,'MRN' from dual union all
select 27924,'AN' from dual union all
select 27925,'AN' from dual union all
select 27926,'AN' from dual union all
select 27927,'AN' from dual union all
select 27928,'MRN' from dual union all
select 27929,'AN' from dual union all
select 2793,'USA' from dual union all
select 27930,'AN' from dual union all
select 27931,'AN' from dual union all
select 27932,'AN' from dual union all
select 27933,'RVSN' from dual union all
select 27934,'USAF' from dual union all
select 27935,'NASA' from dual union all
select 27936,'RVSN' from dual union all
select 27937,'NASA' from dual union all
select 27938,'RVSN' from dual union all
select 27939,'RVSN' from dual union all
select 2794,'PARD' from dual union all
select 27940,'RVSN' from dual union all
select 27941,'RVSN' from dual union all
select 27942,'USAF' from dual union all
select 27943,'RVSN' from dual union all
select 27944,'NASA' from dual union all
select 27945,'RVSN' from dual union all
select 27946,'NASA' from dual union all
select 27947,'RVSN' from dual union all
select 27948,'USAF' from dual union all
select 27949,'RVSN' from dual union all
select 2795,'PARD' from dual union all
select 27950,'10ADS' from dual union all
select 27951,'RVSN' from dual union all
select 27952,'RVSN' from dual union all
select 27953,'RVSN' from dual union all
select 27954,'RVSN' from dual union all
select 27955,'RVSN' from dual union all
select 27956,'RVSN' from dual union all
select 27957,'RVSN' from dual union all
select 27958,'RVSN' from dual union all
select 27959,'RVSN' from dual union all
select 2796,'ORDRSA' from dual union all
select 27960,'RVSN' from dual union all
select 27961,'RVSN' from dual union all
select 27962,'RVSN' from dual union all
select 27963,'NASA' from dual union all
select 27964,'USAF' from dual union all
select 27965,'RVSN' from dual union all
select 27966,'RVSN' from dual union all
select 27967,'RVSN' from dual union all
select 27968,'RVSN' from dual union all
select 27969,'RVSN' from dual union all
select 2797,'ORDRSA' from dual union all
select 27970,'RVSN' from dual union all
select 27971,'USAF' from dual union all
select 27972,'RVSN' from dual union all
select 27973,'NASA' from dual union all
select 27974,'RVSN' from dual union all
select 27975,'RVSN' from dual union all
select 27976,'RVSN' from dual union all
select 27977,'RVSN' from dual union all
select 27978,'RVSN' from dual union all
select 27979,'RVSN' from dual union all
select 2798,'PARD' from dual union all
select 27980,'RVSN' from dual union all
select 27981,'RVSN' from dual union all
select 27982,'RVSN' from dual union all
select 27983,'RVSN' from dual union all
select 27984,'USAF' from dual union all
select 27985,'RVSN' from dual union all
select 27986,'RVSN' from dual union all
select 27987,'RVSN' from dual union all
select 27988,'RVSN' from dual union all
select 27989,'RVSN' from dual union all
select 2799,'ORDRSA' from dual union all
select 27990,'NASA' from dual union all
select 27991,'RVSN' from dual union all
select 27992,'RVSN' from dual union all
select 27993,'NASA' from dual union all
select 27994,'RVSN' from dual union all
select 27995,'RVSN' from dual union all
select 27996,'ISAS' from dual union all
select 27997,'NASA' from dual union all
select 27998,'RVSN' from dual union all
select 27999,'RVSN' from dual union all
select 28,'WEHR' from dual union all
select 280,'WER271-1' from dual union all
select 2800,'ORDRSA' from dual union all
select 28000,'USAF' from dual union all
select 28001,'AFSC' from dual union all
select 28002,'RVSN' from dual union all
select 28003,'RVSN' from dual union all
select 28004,'RVSN' from dual union all
select 28005,'NASA' from dual union all
select 28006,'RVSN' from dual union all
select 28007,'RVSN' from dual union all
select 28008,'AFSC' from dual union all
select 28009,'RVSN' from dual union all
select 2801,'USA' from dual union all
select 28010,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 28011,'USAF' from dual union all
select 28012,'RVSN' from dual union all
select 28013,'RVSN' from dual union all
select 28014,'NASA' from dual union all
select 28015,'RVSN' from dual union all
select 28016,'RVSN' from dual union all
select 28017,'RVSN' from dual union all
select 28018,'RVSN' from dual union all
select 28019,'RVSN' from dual union all
select 2802,'ORDRSA' from dual union all
select 28020,'RVSN' from dual union all
select 28021,'10ADS' from dual union all
select 28022,'NASA' from dual union all
select 28023,'CRA' from dual union all
select 28024,'AFSC' from dual union all
select 28025,'RVSN' from dual union all
select 28026,'RVSN' from dual union all
select 28027,'RVSN' from dual union all
select 28028,'NASA' from dual union all
select 28029,'NASA' from dual union all
select 2803,'ORDRSA' from dual union all
select 28030,'RVSN' from dual union all
select 28031,'RVSN' from dual union all
select 28032,'AFSC' from dual union all
select 28033,'USAF' from dual union all
select 28034,'RVSN' from dual union all
select 28035,'USAF' from dual union all
select 28036,'RVSN' from dual union all
select 28037,'RVSN' from dual union all
select 28038,'RVSN' from dual union all
select 28039,'WRE' from dual union all
select 2804,'ORDRSA' from dual union all
select 28040,'RVSN' from dual union all
select 28041,'SRC' from dual union all
select 28042,'USA' from dual union all
select 28043,'MRN' from dual union all
select 28044,'SRC' from dual union all
select 28045,'SRC' from dual union all
select 28046,'USN' from dual union all
select 28047,'MRN' from dual union all
select 28048,'BAL' from dual union all
select 28049,'RVSN' from dual union all
select 2805,'ORDRSA' from dual union all
select 28050,'MRN' from dual union all
select 28051,'MRN' from dual union all
select 28052,'MRN' from dual union all
select 28053,'DLR' from dual union all
select 28054,'USA' from dual union all
select 28055,'SAND' from dual union all
select 28056,'NASA' from dual union all
select 28057,'RVSN' from dual union all
select 28058,'DLR' from dual union all
select 28059,'BAL' from dual union all
select 2806,'ORDRSA' from dual union all
select 28060,'MRN' from dual union all
select 28061,'MRN' from dual union all
select 28062,'USN' from dual union all
select 28063,'MRN' from dual union all
select 28064,'USN' from dual union all
select 28065,'USA' from dual union all
select 28066,'AFCRL' from dual union all
select 28067,'USA' from dual union all
select 28068,'USN' from dual union all
select 28069,'MRN' from dual union all
select 2807,'ORDRSA' from dual union all
select 28070,'BAL' from dual union all
select 28071,'USN' from dual union all
select 28072,'USN' from dual union all
select 28073,'RVSN' from dual union all
select 28074,'SAND' from dual union all
select 28075,'SAND' from dual union all
select 28076,'NASA' from dual union all
select 28077,'SAND' from dual union all
select 28078,'SAND' from dual union all
select 28079,'VMF' from dual union all
select 2808,'USAF' from dual union all
select 28080,'BAL' from dual union all
select 28081,'AFCRL' from dual union all
select 28082,'WRE' from dual union all
select 28083,'SAND' from dual union all
select 28084,'WRE' from dual union all
select 28085,'WRE' from dual union all
select 28086,'JPL' from dual union all
select 28087,'JPL' from dual union all
select 28088,'JPL' from dual union all
select 28089,'JPL' from dual union all
select 2809,'SUI' from dual union all
select 28090,'NASA' from dual union all
select 28091,'MRN' from dual union all
select 28092,'SRC' from dual union all
select 28093,'RVSN' from dual union all
select 28094,'MRN' from dual union all
select 28095,'ARPA' from dual union all
select 28095,'US' from dual union all
select 28096,'NASA' from dual union all
select 28097,'RVSN' from dual union all
select 28098,'MRN' from dual union all
select 28099,'USA' from dual union all
select 281,'WER271-1' from dual union all
select 2810,'PARD' from dual;
insert into LAUNCH_AGENCY
select 28100,'MRN' from dual union all
select 28101,'RVSN' from dual union all
select 28102,'NTNF' from dual union all
select 28103,'NSF' from dual union all
select 28104,'MRN' from dual union all
select 28105,'MRN' from dual union all
select 28106,'MRN' from dual union all
select 28107,'USN' from dual union all
select 28108,'INTA' from dual union all
select 28109,'MRN' from dual union all
select 2811,'USA' from dual union all
select 28110,'MRN' from dual union all
select 28111,'USN' from dual union all
select 28112,'MRN' from dual union all
select 28113,'MRN' from dual union all
select 28114,'MRN' from dual union all
select 28115,'INTA' from dual union all
select 28116,'RVSN' from dual union all
select 28117,'ARPA' from dual union all
select 28117,'US' from dual union all
select 28118,'MRN' from dual union all
select 28119,'MRN' from dual union all
select 2812,'NRL' from dual union all
select 28120,'MRN' from dual union all
select 28121,'USA' from dual union all
select 28122,'MRN' from dual union all
select 28123,'MRN' from dual union all
select 28124,'RVSN' from dual union all
select 28125,'MRN' from dual union all
select 28126,'MRN' from dual union all
select 28127,'MRN' from dual union all
select 28128,'MRN' from dual union all
select 28129,'MRN' from dual union all
select 2813,'LRBA' from dual union all
select 28130,'MRN' from dual union all
select 28131,'MRN' from dual union all
select 28132,'MRN' from dual union all
select 28133,'MRN' from dual union all
select 28134,'RVSN' from dual union all
select 28135,'USA' from dual union all
select 28136,'MRN' from dual union all
select 28137,'MRN' from dual union all
select 28138,'MRN' from dual union all
select 28139,'RVSN' from dual union all
select 2814,'UZKA' from dual union all
select 28140,'USA' from dual union all
select 28141,'RVSN' from dual union all
select 28142,'AFSC' from dual union all
select 28143,'RAE' from dual union all
select 28144,'MRN' from dual union all
select 28145,'MRN' from dual union all
select 28146,'MRN' from dual union all
select 28147,'MRN' from dual union all
select 28148,'BAL' from dual union all
select 28149,'BAL' from dual union all
select 2815,'UZKA' from dual union all
select 28150,'BAL' from dual union all
select 28151,'MRN' from dual union all
select 28152,'USA' from dual union all
select 28153,'AFCRL' from dual union all
select 28154,'MRN' from dual union all
select 28155,'MRN' from dual union all
select 28156,'BAC' from dual union all
select 28157,'USN' from dual union all
select 28158,'USN' from dual union all
select 28159,'ARPA' from dual union all
select 28159,'US' from dual union all
select 2816,'UZKA' from dual union all
select 28160,'ARPA' from dual union all
select 28160,'US' from dual union all
select 28161,'NASA' from dual union all
select 28162,'WFF' from dual union all
select 28163,'MRN' from dual union all
select 28164,'MRN' from dual union all
select 28165,'USA' from dual union all
select 28166,'RVSN' from dual union all
select 28167,'USA' from dual union all
select 28168,'MRN' from dual union all
select 28169,'MRN' from dual union all
select 2817,'UZKA' from dual union all
select 28170,'MRN' from dual union all
select 28171,'MRN' from dual union all
select 28172,'MRN' from dual union all
select 28173,'MRN' from dual union all
select 28174,'MRN' from dual union all
select 28175,'MRN' from dual union all
select 28176,'USAF' from dual union all
select 28177,'RVSN' from dual union all
select 28178,'USAF' from dual union all
select 28179,'RVSN' from dual union all
select 2818,'NRL' from dual union all
select 28180,'RVSN' from dual union all
select 28181,'RVSN' from dual union all
select 28182,'RVSN' from dual union all
select 28183,'SRC' from dual union all
select 28184,'DLR' from dual union all
select 28184,'ISRO' from dual union all
select 28185,'DLR' from dual union all
select 28185,'ISRO' from dual union all
select 28186,'NSF' from dual;
insert into LAUNCH_AGENCY
select 28187,'USA' from dual union all
select 28188,'USA' from dual union all
select 28189,'USA' from dual union all
select 2819,'UZKA' from dual union all
select 28190,'USA' from dual union all
select 28191,'USA' from dual union all
select 28192,'USA' from dual union all
select 28193,'AFCRL' from dual union all
select 28194,'AFCRL' from dual union all
select 28195,'MRN' from dual union all
select 28196,'CNES' from dual union all
select 28197,'MRN' from dual union all
select 28198,'CNES' from dual union all
select 28199,'MRN' from dual union all
select 282,'WER271-1' from dual union all
select 2820,'UZKA' from dual union all
select 28200,'INTA' from dual union all
select 28201,'INTA' from dual union all
select 28202,'INTA' from dual union all
select 28203,'INTA' from dual union all
select 28204,'INTA' from dual union all
select 28205,'INTA' from dual union all
select 28206,'NASA' from dual union all
select 28207,'JMA' from dual union all
select 28208,'JMA' from dual union all
select 28209,'AN' from dual union all
select 2821,'UZKA' from dual union all
select 28210,'USN' from dual union all
select 28211,'USN' from dual union all
select 28212,'DMA' from dual union all
select 28213,'RVSN' from dual union all
select 28214,'RVSN' from dual union all
select 28215,'RVSN' from dual union all
select 28216,'RVSN' from dual union all
select 28217,'RVSN' from dual union all
select 28218,'RVSN' from dual union all
select 28219,'RN' from dual union all
select 2822,'UZKA' from dual union all
select 28220,'AFCRL' from dual union all
select 28221,'NASA' from dual union all
select 28222,'SAND' from dual union all
select 28223,'NASA' from dual union all
select 28224,'DLR' from dual union all
select 28225,'AN' from dual union all
select 28226,'AFCRL' from dual union all
select 28227,'NASA' from dual union all
select 28228,'NASA' from dual union all
select 28229,'RVSN' from dual union all
select 2823,'UZKA' from dual union all
select 28230,'CRA' from dual union all
select 28231,'RVSN' from dual union all
select 28232,'CRA' from dual union all
select 28233,'CRA' from dual union all
select 28234,'AFSC' from dual union all
select 28235,'BAC' from dual union all
select 28236,'SAND' from dual union all
select 28237,'AFCRL' from dual union all
select 28238,'CRA' from dual union all
select 28239,'BAC' from dual union all
select 2824,'UZKA' from dual union all
select 28240,'RVSN' from dual union all
select 28241,'NASA' from dual union all
select 28242,'SAND' from dual union all
select 28243,'USAF' from dual union all
select 28244,'AN' from dual union all
select 28245,'AN' from dual union all
select 28246,'AN' from dual union all
select 28247,'AN' from dual union all
select 28248,'ONERA' from dual union all
select 28249,'NSF' from dual union all
select 2825,'UZKA' from dual union all
select 28250,'AN' from dual union all
select 28251,'RVSN' from dual union all
select 28252,'ALCS' from dual union all
select 28252,'SAC' from dual union all
select 28253,'NRCC' from dual union all
select 28254,'KPNO' from dual union all
select 28255,'CRA' from dual union all
select 28256,'WRE' from dual union all
select 28257,'NRCC' from dual union all
select 28258,'ESRO' from dual union all
select 28259,'WFF' from dual union all
select 2826,'UZKA' from dual union all
select 28260,'BAC' from dual union all
select 28261,'WFF' from dual union all
select 28262,'USAF' from dual union all
select 28263,'WFF' from dual union all
select 28264,'USA' from dual union all
select 28265,'ESRO' from dual union all
select 28266,'RVSN' from dual union all
select 28267,'NASA' from dual union all
select 28268,'NRL' from dual union all
select 28269,'NASA' from dual union all
select 2827,'UZKA' from dual union all
select 28270,'WFF' from dual union all
select 28271,'WFF' from dual union all
select 28272,'AFCRL' from dual union all
select 28273,'USAF' from dual union all
select 28274,'AN' from dual union all
select 28275,'USAF' from dual;
insert into LAUNCH_AGENCY
select 28276,'NASA' from dual union all
select 28277,'SUPA' from dual union all
select 28278,'SUPA' from dual union all
select 28279,'AFCRL' from dual union all
select 2828,'UZKA' from dual union all
select 28280,'RVSN' from dual union all
select 28281,'AFCRL' from dual union all
select 28282,'SAC' from dual union all
select 28283,'AFCRL' from dual union all
select 28284,'CNES' from dual union all
select 28285,'AFCRL' from dual union all
select 28286,'AFCRL' from dual union all
select 28287,'AFCRL' from dual union all
select 28288,'AFCRL' from dual union all
select 28289,'AFCRL' from dual union all
select 2829,'UZKA' from dual union all
select 28290,'AFCRL' from dual union all
select 28291,'AFCRL' from dual union all
select 28292,'DLR' from dual union all
select 28292,'INTA' from dual union all
select 28293,'AFCRL' from dual union all
select 28294,'AFCRL' from dual union all
select 28295,'RVSN' from dual union all
select 28296,'AFCRL' from dual union all
select 28297,'NIPR' from dual union all
select 28298,'9FA3' from dual union all
select 28299,'US7A' from dual union all
select 283,'WER271-1' from dual union all
select 2830,'UZKA' from dual union all
select 28300,'WRE' from dual union all
select 28301,'DLR' from dual union all
select 28301,'INTA' from dual union all
select 28302,'DLR' from dual union all
select 28302,'INTA' from dual union all
select 28303,'NASA' from dual union all
select 28304,'USN' from dual union all
select 28305,'NASA' from dual union all
select 28306,'RVSN' from dual union all
select 28307,'SUPA' from dual union all
select 28308,'DLR' from dual union all
select 28308,'INTA' from dual union all
select 28309,'US7A' from dual union all
select 2831,'UZKA' from dual union all
select 28310,'US7A' from dual union all
select 28311,'US7A' from dual union all
select 28312,'US7A' from dual union all
select 28313,'AFCRL' from dual union all
select 28314,'AFCRL' from dual union all
select 28315,'DLR' from dual union all
select 28315,'INTA' from dual union all
select 28316,'AFSC' from dual union all
select 28317,'RVSN' from dual union all
select 28318,'DLR' from dual union all
select 28318,'INTA' from dual union all
select 28319,'DLR' from dual union all
select 28319,'INTA' from dual union all
select 2832,'UZKA' from dual union all
select 28320,'SAC' from dual union all
select 28321,'NASA' from dual union all
select 28321,'NRL' from dual union all
select 28322,'NIPR' from dual union all
select 28323,'NIPR' from dual union all
select 28324,'SAND' from dual union all
select 28325,'NRCC' from dual union all
select 28326,'NASA' from dual union all
select 28327,'DLR' from dual union all
select 28327,'INTA' from dual union all
select 28328,'RVSN' from dual union all
select 28329,'NASA' from dual union all
select 2833,'UZKA' from dual union all
select 28330,'NASA' from dual union all
select 28331,'NASA' from dual union all
select 28332,'WRE' from dual union all
select 28333,'NASA' from dual union all
select 28334,'NASA' from dual union all
select 28335,'SAND' from dual union all
select 28336,'SAND' from dual union all
select 28337,'SAC' from dual union all
select 28338,'DLR' from dual union all
select 28338,'INTA' from dual union all
select 28339,'RVSN' from dual union all
select 2834,'UZKA' from dual union all
select 28340,'RVSN' from dual union all
select 28341,'WRE' from dual union all
select 28342,'AFSC' from dual union all
select 28343,'USA' from dual union all
select 28344,'US7A' from dual union all
select 28345,'US7A' from dual union all
select 28346,'US7A' from dual union all
select 28347,'US7A' from dual union all
select 28348,'US7A' from dual union all
select 28349,'US7A' from dual union all
select 2835,'UZKA' from dual union all
select 28350,'RVSN' from dual union all
select 28351,'FKG1' from dual union all
select 28352,'FKG1' from dual union all
select 28353,'US7A' from dual union all
select 28354,'US7A' from dual union all
select 28355,'US7A' from dual union all
select 28356,'US7A' from dual;
insert into LAUNCH_AGENCY
select 28357,'RVSN' from dual union all
select 28358,'NASA' from dual union all
select 28358,'NTNF' from dual union all
select 28359,'NASA' from dual union all
select 28359,'NTNF' from dual union all
select 2836,'UZKA' from dual union all
select 28360,'DLR' from dual union all
select 28360,'INTA' from dual union all
select 28361,'RVSN' from dual union all
select 28362,'NASA' from dual union all
select 28363,'SAND' from dual union all
select 28364,'ALCS' from dual union all
select 28364,'SAC' from dual union all
select 28365,'INTA' from dual union all
select 28366,'NASA' from dual union all
select 28367,'NASA' from dual union all
select 28368,'NASA' from dual union all
select 28369,'SAC' from dual union all
select 2837,'UZKA' from dual union all
select 28370,'AFSC' from dual union all
select 28371,'NASA' from dual union all
select 28372,'RVSN' from dual union all
select 28373,'DLR' from dual union all
select 28373,'INTA' from dual union all
select 28374,'WRE' from dual union all
select 28375,'USN' from dual union all
select 28376,'AFSC' from dual union all
select 28377,'ALCS' from dual union all
select 28377,'SAC' from dual union all
select 28378,'DLR' from dual union all
select 28378,'INTA' from dual union all
select 28379,'NASA' from dual union all
select 2838,'UZKA' from dual union all
select 28380,'FKG2' from dual union all
select 28381,'FKG2' from dual union all
select 28382,'ESRO' from dual union all
select 28383,'RVSN' from dual union all
select 28384,'NASA' from dual union all
select 28385,'NASA' from dual union all
select 28386,'BAC' from dual union all
select 28387,'DLR' from dual union all
select 28387,'INTA' from dual union all
select 28388,'USN' from dual union all
select 28389,'NASA' from dual union all
select 2839,'UZKA' from dual union all
select 28390,'NRCC' from dual union all
select 28391,'SAND' from dual union all
select 28392,'NRCC' from dual union all
select 28393,'NRCC' from dual union all
select 28394,'RVSN' from dual union all
select 28395,'NRCC' from dual union all
select 28396,'NRCC' from dual union all
select 28397,'NRCC' from dual union all
select 28398,'NRCC' from dual union all
select 28399,'NRCC' from dual union all
select 284,'WER271-1' from dual union all
select 2840,'UZKA' from dual union all
select 28400,'NASA' from dual union all
select 28401,'NASA' from dual union all
select 28402,'DLR' from dual union all
select 28402,'INTA' from dual union all
select 28403,'RVSN' from dual union all
select 28404,'NASA' from dual union all
select 28405,'RVSN' from dual union all
select 28406,'AFSC' from dual union all
select 28407,'KPNO' from dual union all
select 28408,'AFSC' from dual union all
select 28409,'NASA' from dual union all
select 2841,'UZKA' from dual union all
select 28410,'USAF' from dual union all
select 28411,'VMF' from dual union all
select 28412,'AFSC' from dual union all
select 28413,'USA' from dual union all
select 28414,'NASA' from dual union all
select 28415,'NASA' from dual union all
select 28416,'RVSN' from dual union all
select 28417,'NIPR' from dual union all
select 28418,'SAST' from dual union all
select 28419,'NASA' from dual union all
select 2842,'UZKA' from dual union all
select 28420,'NASA' from dual union all
select 28421,'NIPR' from dual union all
select 28422,'USAF' from dual union all
select 28423,'ISRO' from dual union all
select 28424,'ISRO' from dual union all
select 28425,'USN' from dual union all
select 28426,'USN' from dual union all
select 28427,'RVSN' from dual union all
select 28428,'USN' from dual union all
select 28429,'USN' from dual union all
select 2843,'UZKA' from dual union all
select 28430,'AFCRL' from dual union all
select 28431,'AFCRL' from dual union all
select 28432,'RVSN' from dual union all
select 28433,'SRC' from dual union all
select 28434,'SRC' from dual union all
select 28435,'SRC' from dual union all
select 28436,'VMF' from dual union all
select 28437,'RVSN' from dual union all
select 28438,'AN' from dual;
insert into LAUNCH_AGENCY
select 28439,'SRC' from dual union all
select 2844,'UZKA' from dual union all
select 28440,'SRC' from dual union all
select 28441,'NASA' from dual union all
select 28442,'NASA' from dual union all
select 28443,'ISAS' from dual union all
select 28444,'RVSN' from dual union all
select 28445,'RVSN' from dual union all
select 28446,'NASA' from dual union all
select 28447,'FKG2' from dual union all
select 28448,'FKG2' from dual union all
select 28449,'AN' from dual union all
select 2845,'UZKA' from dual union all
select 28450,'SAC' from dual union all
select 28451,'ISAS' from dual union all
select 28452,'AFCRL' from dual union all
select 28453,'ESRO' from dual union all
select 28454,'SAC' from dual union all
select 28455,'NASA' from dual union all
select 28456,'NASDA' from dual union all
select 28457,'FKG1' from dual union all
select 28458,'FKG1' from dual union all
select 28459,'AFCRL' from dual union all
select 2846,'UZKA' from dual union all
select 28460,'SRC' from dual union all
select 28461,'USN' from dual union all
select 28462,'NASA' from dual union all
select 28463,'USN' from dual union all
select 28464,'USN' from dual union all
select 28465,'USN' from dual union all
select 28466,'USN' from dual union all
select 28467,'9FA3' from dual union all
select 28468,'RVSN' from dual union all
select 28469,'VMF' from dual union all
select 2847,'USN' from dual union all
select 28470,'SAC' from dual union all
select 28471,'AN' from dual union all
select 28472,'USN' from dual union all
select 28473,'SRC' from dual union all
select 28474,'SRC' from dual union all
select 28475,'NASA' from dual union all
select 28476,'SAND' from dual union all
select 28477,'NASA' from dual union all
select 28478,'SRC' from dual union all
select 28479,'US7A' from dual union all
select 2848,'ARDC' from dual union all
select 28480,'US7A' from dual union all
select 28481,'SAC' from dual union all
select 28482,'SRC' from dual union all
select 28483,'NRL' from dual union all
select 28484,'AFCRL' from dual union all
select 28485,'SRC' from dual union all
select 28486,'NASA' from dual union all
select 28487,'ESRO' from dual union all
select 28488,'ISRO' from dual union all
select 28489,'NASA' from dual union all
select 2849,'UZKA' from dual union all
select 28490,'NASA' from dual union all
select 28491,'CNES' from dual union all
select 28491,'ISRO' from dual union all
select 28492,'NASA' from dual union all
select 28493,'SRC' from dual union all
select 28494,'SAC' from dual union all
select 28495,'BAC' from dual union all
select 28496,'DLR' from dual union all
select 28496,'INTA' from dual union all
select 28497,'ALCS' from dual union all
select 28497,'SAC' from dual union all
select 28498,'DLR' from dual union all
select 28498,'INTA' from dual union all
select 28499,'SAND' from dual union all
select 285,'WER271-1' from dual union all
select 2850,'UZKA' from dual union all
select 28500,'SAND' from dual union all
select 28501,'NASA' from dual union all
select 28502,'SRC' from dual union all
select 28503,'BAC' from dual union all
select 28504,'NASA' from dual union all
select 28505,'SRC' from dual union all
select 28506,'SRC' from dual union all
select 28507,'NASA' from dual union all
select 28508,'DLR' from dual union all
select 28508,'INTA' from dual union all
select 28509,'AN' from dual union all
select 2851,'USN' from dual union all
select 28510,'USAF' from dual union all
select 28511,'USA' from dual union all
select 28512,'BAC' from dual union all
select 28513,'AN' from dual union all
select 28514,'AFSC' from dual union all
select 28515,'NASA' from dual union all
select 28516,'AN' from dual union all
select 28517,'SRC' from dual union all
select 28518,'NASA' from dual union all
select 28519,'AN' from dual union all
select 2852,'NII88A' from dual union all
select 28520,'AN' from dual union all
select 28521,'AFCRL' from dual union all
select 28522,'AFCRL' from dual union all
select 28523,'AFCRL' from dual;
insert into LAUNCH_AGENCY
select 28524,'AFCRL' from dual union all
select 28525,'AN' from dual union all
select 28526,'SRC' from dual union all
select 28527,'NASA' from dual union all
select 28528,'NRL' from dual union all
select 28529,'AFCRL' from dual union all
select 2853,'UZKA' from dual union all
select 28530,'USN' from dual union all
select 28531,'USN' from dual union all
select 28532,'USN' from dual union all
select 28533,'USN' from dual union all
select 28534,'NASA' from dual union all
select 28535,'BAC' from dual union all
select 28535,'ESRO' from dual union all
select 28536,'IIAE' from dual union all
select 28537,'NRCC' from dual union all
select 28538,'RVSN' from dual union all
select 28539,'SRC' from dual union all
select 2854,'ARDC' from dual union all
select 28540,'NASA' from dual union all
select 28541,'IIAE' from dual union all
select 28542,'AFCRL' from dual union all
select 28543,'NASA' from dual union all
select 28544,'SRC' from dual union all
select 28545,'IIAE' from dual union all
select 28546,'NASA' from dual union all
select 28547,'NRCC' from dual union all
select 28548,'ESRO' from dual union all
select 28549,'NASA' from dual union all
select 2855,'UZKA' from dual union all
select 28550,'AN' from dual union all
select 28551,'ALCS' from dual union all
select 28551,'SAC' from dual union all
select 28552,'VMF' from dual union all
select 28553,'CRA' from dual union all
select 28554,'VMF' from dual union all
select 28555,'VMF' from dual union all
select 28556,'USA' from dual union all
select 28557,'NRCC' from dual union all
select 28558,'NRCC' from dual union all
select 28559,'AN' from dual union all
select 2856,'UZKA' from dual union all
select 28560,'AN' from dual union all
select 28561,'AN' from dual union all
select 28562,'SAC' from dual union all
select 28563,'AFCRL' from dual union all
select 28564,'AFCRL' from dual union all
select 28565,'NASA' from dual union all
select 28566,'BAC' from dual union all
select 28567,'AFCRL' from dual union all
select 28568,'AFCRL' from dual union all
select 28569,'SRC' from dual union all
select 2857,'UZKA' from dual union all
select 28570,'AFSC' from dual union all
select 28571,'NASA' from dual union all
select 28572,'BAC' from dual union all
select 28573,'USN' from dual union all
select 28574,'USN' from dual union all
select 28575,'USN' from dual union all
select 28576,'USN' from dual union all
select 28577,'AFCRL' from dual union all
select 28578,'AFCRL' from dual union all
select 28579,'AFCRL' from dual union all
select 2858,'NII88A' from dual union all
select 28580,'NASA' from dual union all
select 28581,'ALCS' from dual union all
select 28581,'SAC' from dual union all
select 28582,'SAC' from dual union all
select 28583,'CNES' from dual union all
select 28584,'DFVLR' from dual union all
select 28585,'NASA' from dual union all
select 28586,'NIPR' from dual union all
select 28587,'VMF' from dual union all
select 28588,'VMF' from dual union all
select 28589,'VMF' from dual union all
select 2859,'NII88A' from dual union all
select 28590,'USA' from dual union all
select 28591,'VMF' from dual union all
select 28592,'USN' from dual union all
select 28593,'USN' from dual union all
select 28594,'AFSC' from dual union all
select 28595,'SAC' from dual union all
select 28596,'AN' from dual union all
select 28597,'RVSN' from dual union all
select 28598,'US7A' from dual union all
select 28599,'AFCRL' from dual union all
select 286,'WEHR' from dual union all
select 2860,'NII88A' from dual union all
select 28600,'AN' from dual union all
select 28601,'ISAS' from dual union all
select 28602,'AN' from dual union all
select 28603,'AN' from dual union all
select 28604,'VMF' from dual union all
select 28605,'VMF' from dual union all
select 28606,'VMF' from dual union all
select 28607,'AFCRL' from dual union all
select 28608,'AFCRL' from dual union all
select 28609,'AFCRL' from dual union all
select 2861,'ARDC' from dual union all
select 28610,'AFCRL' from dual;
insert into LAUNCH_AGENCY
select 28611,'AFCRL' from dual union all
select 28612,'ESRO' from dual union all
select 28613,'AFCRL' from dual union all
select 28614,'AFCRL' from dual union all
select 28615,'RVSN' from dual union all
select 28616,'WRE' from dual union all
select 28617,'WRE' from dual union all
select 28618,'WRE' from dual union all
select 28619,'WRE' from dual union all
select 2862,'SUI' from dual union all
select 28620,'WRE' from dual union all
select 28621,'WRE' from dual union all
select 28622,'WRE' from dual union all
select 28623,'USN' from dual union all
select 28624,'WRE' from dual union all
select 28625,'WRE' from dual union all
select 28626,'WRE' from dual union all
select 28627,'WRE' from dual union all
select 28628,'CNES' from dual union all
select 28629,'CNES' from dual union all
select 2863,'NRL' from dual union all
select 28630,'CNES' from dual union all
select 28631,'CNES' from dual union all
select 28632,'CNES' from dual union all
select 28633,'CNES' from dual union all
select 28634,'ESRO' from dual union all
select 28635,'RVSN' from dual union all
select 28636,'RVSN' from dual union all
select 28637,'RVSN' from dual union all
select 28638,'RVSN' from dual union all
select 28639,'RVSN' from dual union all
select 2864,'SUI' from dual union all
select 28640,'RVSN' from dual union all
select 28641,'RVSN' from dual union all
select 28642,'RVSN' from dual union all
select 28643,'RVSN' from dual union all
select 28644,'RVSN' from dual union all
select 28645,'SRC' from dual union all
select 28646,'RVSN' from dual union all
select 28647,'RVSN' from dual union all
select 28648,'RVSN' from dual union all
select 28649,'RVSN' from dual union all
select 2865,'NRL' from dual union all
select 28650,'RVSN' from dual union all
select 28651,'RVSN' from dual union all
select 28652,'ISAS' from dual union all
select 28653,'SAC' from dual union all
select 28654,'NASA' from dual union all
select 28655,'AFCRL' from dual union all
select 28656,'ISAS' from dual union all
select 28657,'AFCRL' from dual union all
select 28658,'USN' from dual union all
select 28659,'NASA' from dual union all
select 2866,'SUI' from dual union all
select 28660,'AFCRL' from dual union all
select 28661,'AFCRL' from dual union all
select 28662,'SRC' from dual union all
select 28663,'NASA' from dual union all
select 28664,'WRE' from dual union all
select 28665,'DLR' from dual union all
select 28666,'SAC' from dual union all
select 28667,'USN' from dual union all
select 28668,'SRC' from dual union all
select 28669,'NASDA' from dual union all
select 2867,'NRL' from dual union all
select 28670,'AFCRL' from dual union all
select 28671,'NASA' from dual union all
select 28671,'RTG' from dual union all
select 28672,'NRCC' from dual union all
select 28673,'ALCS' from dual union all
select 28673,'SAC' from dual union all
select 28674,'NASA' from dual union all
select 28675,'AN' from dual union all
select 28676,'AN' from dual union all
select 28677,'NIPR' from dual union all
select 28678,'RVSN' from dual union all
select 28679,'RVSN' from dual union all
select 2868,'SUI' from dual union all
select 28680,'RVSN' from dual union all
select 28681,'DLR' from dual union all
select 28681,'INTA' from dual union all
select 28682,'DLR' from dual union all
select 28682,'INTA' from dual union all
select 28683,'WRE' from dual union all
select 28684,'SRC' from dual union all
select 28685,'RVSN' from dual union all
select 28686,'RVSN' from dual union all
select 28687,'RVSN' from dual union all
select 28688,'RVSN' from dual union all
select 28689,'RVSN' from dual union all
select 2869,'SUI' from dual union all
select 28690,'RVSN' from dual union all
select 28691,'RVSN' from dual union all
select 28692,'RVSN' from dual union all
select 28693,'NASA' from dual union all
select 28694,'NRCC' from dual union all
select 28695,'NRCC' from dual union all
select 28696,'NASA' from dual union all
select 28697,'ISAS' from dual union all
select 28698,'SRC' from dual;
insert into LAUNCH_AGENCY
select 28699,'AN' from dual union all
select 287,'WER271-1' from dual union all
select 2870,'SUI' from dual union all
select 28700,'ISAS' from dual union all
select 28701,'NASA' from dual union all
select 28702,'ISAS' from dual union all
select 28703,'AN' from dual union all
select 28704,'AN' from dual union all
select 28705,'ISAS' from dual union all
select 28706,'WRE' from dual union all
select 28707,'NASA' from dual union all
select 28708,'ESRO' from dual union all
select 28709,'ESRO' from dual union all
select 2871,'SUI' from dual union all
select 28710,'AN' from dual union all
select 28711,'NRL' from dual union all
select 28712,'ESRO' from dual union all
select 28713,'AN' from dual union all
select 28714,'AN' from dual union all
select 28715,'AN' from dual union all
select 28716,'NPOL' from dual union all
select 28717,'NASA' from dual union all
select 28718,'NASA' from dual union all
select 28719,'AFCRL' from dual union all
select 2872,'SUI' from dual union all
select 28720,'AFCRL' from dual union all
select 28721,'AFCRL' from dual union all
select 28722,'AFCRL' from dual union all
select 28723,'MRN' from dual union all
select 28724,'MRN' from dual union all
select 28725,'MRN' from dual union all
select 28726,'MRN' from dual union all
select 28727,'MRN' from dual union all
select 28728,'MRN' from dual union all
select 28729,'INTA' from dual union all
select 2873,'SUI' from dual union all
select 28730,'MRN' from dual union all
select 28731,'MRN' from dual union all
select 28732,'MRN' from dual union all
select 28733,'MRN' from dual union all
select 28734,'MRN' from dual union all
select 28735,'MRN' from dual union all
select 28736,'MRN' from dual union all
select 28737,'MRN' from dual union all
select 28738,'MRN' from dual union all
select 28739,'MRN' from dual union all
select 2874,'NRL' from dual union all
select 28740,'MRN' from dual union all
select 28741,'MRN' from dual union all
select 28742,'MRN' from dual union all
select 28743,'MRN' from dual union all
select 28744,'MRN' from dual union all
select 28745,'MRN' from dual union all
select 28746,'MRN' from dual union all
select 28747,'MRN' from dual union all
select 28748,'MRN' from dual union all
select 28749,'MRN' from dual union all
select 2875,'NRL' from dual union all
select 28750,'MRN' from dual union all
select 28751,'MRN' from dual union all
select 28752,'MRN' from dual union all
select 28753,'MRN' from dual union all
select 28754,'MRN' from dual union all
select 28755,'MRN' from dual union all
select 28756,'MRN' from dual union all
select 28757,'MRN' from dual union all
select 28758,'MRN' from dual union all
select 28759,'MRN' from dual union all
select 2876,'UZKA' from dual union all
select 28760,'MRN' from dual union all
select 28761,'MRN' from dual union all
select 28762,'MRN' from dual union all
select 28763,'MRN' from dual union all
select 28764,'MRN' from dual union all
select 28765,'MRN' from dual union all
select 28766,'MRN' from dual union all
select 28767,'MRN' from dual union all
select 28768,'MRN' from dual union all
select 28769,'MRN' from dual union all
select 2877,'UZKA' from dual union all
select 28770,'MRN' from dual union all
select 28771,'MRN' from dual union all
select 28772,'MRN' from dual union all
select 28773,'MRN' from dual union all
select 28774,'MRN' from dual union all
select 28775,'MRN' from dual union all
select 28776,'MRN' from dual union all
select 28777,'MRN' from dual union all
select 28778,'MRN' from dual union all
select 28779,'MRN' from dual union all
select 2878,'UZKA' from dual union all
select 28780,'MRN' from dual union all
select 28781,'MRN' from dual union all
select 28782,'MRN' from dual union all
select 28783,'MRN' from dual union all
select 28784,'MRN' from dual union all
select 28785,'MRN' from dual union all
select 28786,'MRN' from dual union all
select 28787,'MRN' from dual union all
select 28788,'MRN' from dual;
insert into LAUNCH_AGENCY
select 28789,'MRN' from dual union all
select 2879,'ARDC' from dual union all
select 28790,'MRN' from dual union all
select 28791,'MRN' from dual union all
select 28792,'MRN' from dual union all
select 28793,'MRN' from dual union all
select 28794,'MRN' from dual union all
select 28795,'MRN' from dual union all
select 28796,'MRN' from dual union all
select 28797,'MRN' from dual union all
select 28798,'MRN' from dual union all
select 28799,'MRN' from dual union all
select 288,'WER271-1' from dual union all
select 2880,'UZKA' from dual union all
select 28800,'MRN' from dual union all
select 28801,'MRN' from dual union all
select 28802,'MRN' from dual union all
select 28803,'MRN' from dual union all
select 28804,'MRN' from dual union all
select 28805,'MRN' from dual union all
select 28806,'MRN' from dual union all
select 28807,'MRN' from dual union all
select 28808,'MRN' from dual union all
select 28809,'MRN' from dual union all
select 2881,'USA' from dual union all
select 28810,'MRN' from dual union all
select 28811,'MRN' from dual union all
select 28812,'MRN' from dual union all
select 28813,'MRN' from dual union all
select 28814,'MRN' from dual union all
select 28815,'MRN' from dual union all
select 28816,'MRN' from dual union all
select 28817,'MRN' from dual union all
select 28818,'MRN' from dual union all
select 28819,'MRN' from dual union all
select 2882,'UZKA' from dual union all
select 28820,'MRN' from dual union all
select 28821,'MRN' from dual union all
select 28822,'MRN' from dual union all
select 28823,'MRN' from dual union all
select 28824,'MRN' from dual union all
select 28825,'MRN' from dual union all
select 28826,'MRN' from dual union all
select 28827,'MRN' from dual union all
select 28828,'MRN' from dual union all
select 28829,'MRN' from dual union all
select 2883,'UZKA' from dual union all
select 28830,'MRN' from dual union all
select 28831,'MRN' from dual union all
select 28832,'MRN' from dual union all
select 28833,'MRN' from dual union all
select 28834,'MRN' from dual union all
select 28835,'MRN' from dual union all
select 28836,'MRN' from dual union all
select 28837,'NASDA' from dual union all
select 28838,'NASDA' from dual union all
select 28839,'MRN' from dual union all
select 2884,'UZKA' from dual union all
select 28840,'MRN' from dual union all
select 28841,'MRN' from dual union all
select 28842,'MRN' from dual union all
select 28843,'MRN' from dual union all
select 28844,'MRN' from dual union all
select 28845,'MRN' from dual union all
select 28846,'MRN' from dual union all
select 28847,'MRN' from dual union all
select 28848,'MRN' from dual union all
select 28849,'MRN' from dual union all
select 2885,'UZKA' from dual union all
select 28850,'MRN' from dual union all
select 28851,'MRN' from dual union all
select 28852,'MRN' from dual union all
select 28853,'MRN' from dual union all
select 28854,'MRN' from dual union all
select 28855,'MRN' from dual union all
select 28856,'MRN' from dual union all
select 28857,'MRN' from dual union all
select 28858,'MRN' from dual union all
select 28859,'MRN' from dual union all
select 2886,'UZKA' from dual union all
select 28860,'MRN' from dual union all
select 28861,'MRN' from dual union all
select 28862,'MRN' from dual union all
select 28863,'MRN' from dual union all
select 28864,'MRN' from dual union all
select 28865,'MRN' from dual union all
select 28866,'MRN' from dual union all
select 28867,'MRN' from dual union all
select 28868,'MRN' from dual union all
select 28869,'MRN' from dual union all
select 2887,'ARDC' from dual union all
select 28870,'MRN' from dual union all
select 28871,'MRN' from dual union all
select 28872,'MRN' from dual union all
select 28873,'MRN' from dual union all
select 28874,'MRN' from dual union all
select 28875,'MRN' from dual union all
select 28876,'MRN' from dual union all
select 28877,'MRN' from dual union all
select 28878,'MRN' from dual;
insert into LAUNCH_AGENCY
select 28879,'MRN' from dual union all
select 2888,'NII88' from dual union all
select 2888,'VMF' from dual union all
select 28880,'MRN' from dual union all
select 28881,'MRN' from dual union all
select 28882,'MRN' from dual union all
select 28883,'MRN' from dual union all
select 28884,'MRN' from dual union all
select 28885,'MRN' from dual union all
select 28886,'MRN' from dual union all
select 28887,'MRN' from dual union all
select 28888,'MRN' from dual union all
select 28889,'MRN' from dual union all
select 2889,'NII88' from dual union all
select 2889,'VMF' from dual union all
select 28890,'MRN' from dual union all
select 28891,'MRN' from dual union all
select 28892,'MRN' from dual union all
select 28893,'MRN' from dual union all
select 28894,'MRN' from dual union all
select 28895,'MRN' from dual union all
select 28896,'MRN' from dual union all
select 28897,'MRN' from dual union all
select 28898,'MRN' from dual union all
select 28899,'MRN' from dual union all
select 289,'WER271-1' from dual union all
select 2890,'UZKA' from dual union all
select 28900,'MRN' from dual union all
select 28901,'MRN' from dual union all
select 28902,'MRN' from dual union all
select 28903,'MRN' from dual union all
select 28904,'MRN' from dual union all
select 28905,'MRN' from dual union all
select 28906,'MRN' from dual union all
select 28907,'MRN' from dual union all
select 28908,'MRN' from dual union all
select 28909,'MRN' from dual union all
select 2891,'UZKA' from dual union all
select 28910,'MRN' from dual union all
select 28911,'MRN' from dual union all
select 28912,'MRN' from dual union all
select 28913,'MRN' from dual union all
select 28914,'MRN' from dual union all
select 28915,'MRN' from dual union all
select 28916,'MRN' from dual union all
select 28917,'MRN' from dual union all
select 28918,'MRN' from dual union all
select 28919,'MRN' from dual union all
select 2892,'UZKA' from dual union all
select 28920,'MRN' from dual union all
select 28921,'MRN' from dual union all
select 28922,'MRN' from dual union all
select 28923,'MRN' from dual union all
select 28924,'MRN' from dual union all
select 28925,'MRN' from dual union all
select 28926,'MRN' from dual union all
select 28927,'MRN' from dual union all
select 28928,'MRN' from dual union all
select 28929,'MRN' from dual union all
select 2893,'NRL' from dual union all
select 28930,'MRN' from dual union all
select 28931,'MRN' from dual union all
select 28932,'MRN' from dual union all
select 28933,'MRN' from dual union all
select 28934,'MRN' from dual union all
select 28935,'MRN' from dual union all
select 28936,'MRN' from dual union all
select 28937,'MRN' from dual union all
select 28938,'MRN' from dual union all
select 28939,'INTA' from dual union all
select 2894,'PARD' from dual union all
select 28940,'MRN' from dual union all
select 28941,'MRN' from dual union all
select 28942,'MRN' from dual union all
select 28943,'MRN' from dual union all
select 28944,'MRN' from dual union all
select 28945,'MRN' from dual union all
select 28946,'MRN' from dual union all
select 28947,'MRN' from dual union all
select 28948,'MRN' from dual union all
select 28949,'MRN' from dual union all
select 2895,'NII88' from dual union all
select 2895,'VMF' from dual union all
select 28950,'MRN' from dual union all
select 28951,'MRN' from dual union all
select 28952,'MRN' from dual union all
select 28953,'MRN' from dual union all
select 28954,'MRN' from dual union all
select 28955,'MRN' from dual union all
select 28956,'MRN' from dual union all
select 28957,'MRN' from dual union all
select 28958,'MRN' from dual union all
select 28959,'MRN' from dual union all
select 2896,'LRBA' from dual union all
select 28960,'MRN' from dual union all
select 28961,'MRN' from dual union all
select 28962,'MRN' from dual union all
select 28963,'MRN' from dual union all
select 28964,'MRN' from dual union all
select 28965,'MRN' from dual;
insert into LAUNCH_AGENCY
select 28966,'MRN' from dual union all
select 28967,'MRN' from dual union all
select 28968,'MRN' from dual union all
select 28969,'MRN' from dual union all
select 2897,'UZKA' from dual union all
select 28970,'MRN' from dual union all
select 28971,'MRN' from dual union all
select 28972,'MRN' from dual union all
select 28973,'MRN' from dual union all
select 28974,'MRN' from dual union all
select 28975,'MRN' from dual union all
select 28976,'MRN' from dual union all
select 28977,'MRN' from dual union all
select 28978,'MRN' from dual union all
select 28979,'MRN' from dual union all
select 2898,'UZKA' from dual union all
select 28980,'MRN' from dual union all
select 28981,'MRN' from dual union all
select 28982,'MRN' from dual union all
select 28983,'MRN' from dual union all
select 28984,'MRN' from dual union all
select 28985,'MRN' from dual union all
select 28986,'MRN' from dual union all
select 28987,'MRN' from dual union all
select 28988,'MRN' from dual union all
select 28989,'MRN' from dual union all
select 2899,'UZKA' from dual union all
select 28990,'MRN' from dual union all
select 28991,'MRN' from dual union all
select 28992,'MRN' from dual union all
select 28993,'MRN' from dual union all
select 28994,'MRN' from dual union all
select 28995,'MRN' from dual union all
select 28996,'MRN' from dual union all
select 28997,'MRN' from dual union all
select 28998,'MRN' from dual union all
select 28999,'MRN' from dual union all
select 29,'WEHR' from dual union all
select 290,'WER271-1' from dual union all
select 2900,'UZKA' from dual union all
select 29000,'MRN' from dual union all
select 29001,'MRN' from dual union all
select 29002,'MRN' from dual union all
select 29003,'MRN' from dual union all
select 29004,'MRN' from dual union all
select 29005,'MRN' from dual union all
select 29006,'MRN' from dual union all
select 29007,'MRN' from dual union all
select 29008,'MRN' from dual union all
select 29009,'MRN' from dual union all
select 2901,'UZKA' from dual union all
select 29010,'MRN' from dual union all
select 29011,'USA' from dual union all
select 29012,'MRN' from dual union all
select 29013,'MRN' from dual union all
select 29014,'MRN' from dual union all
select 29015,'MRN' from dual union all
select 29016,'MRN' from dual union all
select 29017,'MRN' from dual union all
select 29018,'MRN' from dual union all
select 29019,'MRN' from dual union all
select 2902,'UZKA' from dual union all
select 29020,'MRN' from dual union all
select 29021,'MRN' from dual union all
select 29022,'MRN' from dual union all
select 29023,'MRN' from dual union all
select 29024,'MRN' from dual union all
select 29025,'MRN' from dual union all
select 29026,'MRN' from dual union all
select 29027,'MRN' from dual union all
select 29028,'MRN' from dual union all
select 29029,'MRN' from dual union all
select 2903,'UZKA' from dual union all
select 29030,'MRN' from dual union all
select 29031,'MRN' from dual union all
select 29032,'MRN' from dual union all
select 29033,'MRN' from dual union all
select 29034,'MRN' from dual union all
select 29035,'MRN' from dual union all
select 29036,'MRN' from dual union all
select 29037,'MRN' from dual union all
select 29038,'MRN' from dual union all
select 29039,'MRN' from dual union all
select 2904,'UZKA' from dual union all
select 29040,'MRN' from dual union all
select 29041,'MRN' from dual union all
select 29042,'MRN' from dual union all
select 29043,'MRN' from dual union all
select 29044,'MRN' from dual union all
select 29045,'MRN' from dual union all
select 29046,'MRN' from dual union all
select 29047,'MRN' from dual union all
select 29048,'MRN' from dual union all
select 29049,'MRN' from dual union all
select 2905,'UZKA' from dual union all
select 29050,'MRN' from dual union all
select 29051,'MRN' from dual union all
select 29052,'MRN' from dual union all
select 29053,'MRN' from dual union all
select 29054,'MRN' from dual;
insert into LAUNCH_AGENCY
select 29055,'MRN' from dual union all
select 29056,'MRN' from dual union all
select 29057,'MRN' from dual union all
select 29058,'MRN' from dual union all
select 29059,'MRN' from dual union all
select 2906,'UZKA' from dual union all
select 29060,'MRN' from dual union all
select 29061,'MRN' from dual union all
select 29062,'MRN' from dual union all
select 29063,'MRN' from dual union all
select 29064,'MRN' from dual union all
select 29065,'MRN' from dual union all
select 29066,'MRN' from dual union all
select 29067,'MRN' from dual union all
select 29068,'MRN' from dual union all
select 29069,'MRN' from dual union all
select 2907,'UZKA' from dual union all
select 29070,'MRN' from dual union all
select 29071,'MRN' from dual union all
select 29072,'MRN' from dual union all
select 29073,'MRN' from dual union all
select 29074,'MRN' from dual union all
select 29075,'MRN' from dual union all
select 29076,'MRN' from dual union all
select 29077,'MRN' from dual union all
select 29078,'MRN' from dual union all
select 29079,'MRN' from dual union all
select 2908,'UZKA' from dual union all
select 29080,'MRN' from dual union all
select 29081,'MRN' from dual union all
select 29082,'MRN' from dual union all
select 29083,'INTA' from dual union all
select 29084,'MRN' from dual union all
select 29085,'MRN' from dual union all
select 29086,'MRN' from dual union all
select 29087,'MRN' from dual union all
select 29088,'MRN' from dual union all
select 29089,'MRN' from dual union all
select 2909,'UZKA' from dual union all
select 29090,'MRN' from dual union all
select 29091,'MRN' from dual union all
select 29092,'MRN' from dual union all
select 29093,'MRN' from dual union all
select 29094,'MRN' from dual union all
select 29095,'MRN' from dual union all
select 29096,'MRN' from dual union all
select 29097,'MRN' from dual union all
select 29098,'MRN' from dual union all
select 29099,'MRN' from dual union all
select 291,'WER271-1' from dual union all
select 2910,'UZKA' from dual union all
select 29100,'NASA' from dual union all
select 29101,'MRN' from dual union all
select 29102,'MRN' from dual union all
select 29103,'MRN' from dual union all
select 29104,'MRN' from dual union all
select 29105,'MRN' from dual union all
select 29106,'MRN' from dual union all
select 29107,'MRN' from dual union all
select 29108,'MRN' from dual union all
select 29109,'MRN' from dual union all
select 2911,'UZKA' from dual union all
select 29110,'MRN' from dual union all
select 29111,'MRN' from dual union all
select 29112,'NASA' from dual union all
select 29113,'MRN' from dual union all
select 29114,'MRN' from dual union all
select 29115,'MRN' from dual union all
select 29116,'MRN' from dual union all
select 29117,'MRN' from dual union all
select 29118,'MRN' from dual union all
select 29119,'MRN' from dual union all
select 2912,'PARD' from dual union all
select 29120,'MRN' from dual union all
select 29121,'MRN' from dual union all
select 29122,'MRN' from dual union all
select 29123,'MRN' from dual union all
select 29124,'MRN' from dual union all
select 29125,'MRN' from dual union all
select 29126,'MRN' from dual union all
select 29127,'MRN' from dual union all
select 29128,'MRN' from dual union all
select 29129,'MRN' from dual union all
select 2913,'PARD' from dual union all
select 29130,'MRN' from dual union all
select 29131,'MRN' from dual union all
select 29132,'MRN' from dual union all
select 29133,'MRN' from dual union all
select 29134,'MRN' from dual union all
select 29135,'MRN' from dual union all
select 29136,'MRN' from dual union all
select 29137,'MRN' from dual union all
select 29138,'MRN' from dual union all
select 29139,'USA' from dual union all
select 2914,'USA' from dual union all
select 29140,'MRN' from dual union all
select 29141,'MRN' from dual union all
select 29142,'MRN' from dual union all
select 29143,'MRN' from dual union all
select 29144,'MRN' from dual;
insert into LAUNCH_AGENCY
select 29145,'MRN' from dual union all
select 29146,'MRN' from dual union all
select 29147,'MRN' from dual union all
select 29148,'MRN' from dual union all
select 29149,'MRN' from dual union all
select 2915,'PARD' from dual union all
select 29150,'MRN' from dual union all
select 29151,'MRN' from dual union all
select 29152,'MRN' from dual union all
select 29153,'MRN' from dual union all
select 29154,'MRN' from dual union all
select 29155,'MRN' from dual union all
select 29156,'MRN' from dual union all
select 29157,'MRN' from dual union all
select 29158,'MRN' from dual union all
select 29159,'MRN' from dual union all
select 2916,'USA' from dual union all
select 29160,'USA' from dual union all
select 29161,'MRN' from dual union all
select 29162,'MRN' from dual union all
select 29163,'MRN' from dual union all
select 29164,'MRN' from dual union all
select 29165,'MRN' from dual union all
select 29166,'MRN' from dual union all
select 29167,'MRN' from dual union all
select 29168,'MRN' from dual union all
select 29169,'MRN' from dual union all
select 2917,'USA' from dual union all
select 29170,'MRN' from dual union all
select 29171,'MRN' from dual union all
select 29172,'MRN' from dual union all
select 29173,'MRN' from dual union all
select 29174,'MRN' from dual union all
select 29175,'MRN' from dual union all
select 29176,'MRN' from dual union all
select 29177,'MRN' from dual union all
select 29178,'MRN' from dual union all
select 29179,'MRN' from dual union all
select 2918,'USA' from dual union all
select 29180,'MRN' from dual union all
select 29181,'MRN' from dual union all
select 29182,'MRN' from dual union all
select 29183,'MRN' from dual union all
select 29184,'MRN' from dual union all
select 29185,'MRN' from dual union all
select 29186,'MRN' from dual union all
select 29187,'MRN' from dual union all
select 29188,'MRN' from dual union all
select 29189,'MRN' from dual union all
select 2919,'USA' from dual union all
select 29190,'MRN' from dual union all
select 29191,'MRN' from dual union all
select 29192,'MRN' from dual union all
select 29193,'MRN' from dual union all
select 29194,'MRN' from dual union all
select 29195,'MRN' from dual union all
select 29196,'MRN' from dual union all
select 29197,'MRN' from dual union all
select 29198,'MRN' from dual union all
select 29199,'MRN' from dual union all
select 292,'WER271-1' from dual union all
select 2920,'USA' from dual union all
select 29200,'MRN' from dual union all
select 29201,'MRN' from dual union all
select 29202,'MRN' from dual union all
select 29203,'MRN' from dual union all
select 29204,'MRN' from dual union all
select 29205,'MRN' from dual union all
select 29206,'MRN' from dual union all
select 29207,'MRN' from dual union all
select 29208,'MRN' from dual union all
select 29209,'MRN' from dual union all
select 2921,'USA' from dual union all
select 29210,'MRN' from dual union all
select 29211,'MRN' from dual union all
select 29212,'MRN' from dual union all
select 29213,'MRN' from dual union all
select 29214,'MRN' from dual union all
select 29215,'MRN' from dual union all
select 29216,'MRN' from dual union all
select 29217,'MRN' from dual union all
select 29218,'MRN' from dual union all
select 29219,'MRN' from dual union all
select 2922,'USA' from dual union all
select 29220,'MRN' from dual union all
select 29221,'MRN' from dual union all
select 29222,'MRN' from dual union all
select 29223,'MRN' from dual union all
select 29224,'MRN' from dual union all
select 29225,'MRN' from dual union all
select 29226,'MRN' from dual union all
select 29227,'MRN' from dual union all
select 29228,'MRN' from dual union all
select 29229,'MRN' from dual union all
select 2923,'USAF' from dual union all
select 29230,'MRN' from dual union all
select 29231,'MRN' from dual union all
select 29232,'MRN' from dual union all
select 29233,'MRN' from dual union all
select 29234,'INTA' from dual;
insert into LAUNCH_AGENCY
select 29235,'MRN' from dual union all
select 29236,'MRN' from dual union all
select 29237,'MRN' from dual union all
select 29238,'MRN' from dual union all
select 29239,'MRN' from dual union all
select 2924,'USA' from dual union all
select 29240,'MRN' from dual union all
select 29241,'MRN' from dual union all
select 29242,'MRN' from dual union all
select 29243,'MRN' from dual union all
select 29244,'MRN' from dual union all
select 29245,'MRN' from dual union all
select 29246,'MRN' from dual union all
select 29247,'MRN' from dual union all
select 29248,'MRN' from dual union all
select 29249,'MRN' from dual union all
select 2925,'USA' from dual union all
select 29250,'MRN' from dual union all
select 29251,'MRN' from dual union all
select 29252,'MRN' from dual union all
select 29253,'MRN' from dual union all
select 29254,'MRN' from dual union all
select 29255,'MRN' from dual union all
select 29256,'MRN' from dual union all
select 29257,'MRN' from dual union all
select 29258,'MRN' from dual union all
select 29259,'MRN' from dual union all
select 2926,'USA' from dual union all
select 29260,'MRN' from dual union all
select 29261,'MRN' from dual union all
select 29262,'MRN' from dual union all
select 29263,'MRN' from dual union all
select 29264,'MRN' from dual union all
select 29265,'INTA' from dual union all
select 29266,'MRN' from dual union all
select 29267,'MRN' from dual union all
select 29268,'MRN' from dual union all
select 29269,'MRN' from dual union all
select 2927,'USA' from dual union all
select 29270,'MRN' from dual union all
select 29271,'MRN' from dual union all
select 29272,'MRN' from dual union all
select 29273,'MRN' from dual union all
select 29274,'MRN' from dual union all
select 29275,'MRN' from dual union all
select 29276,'MRN' from dual union all
select 29277,'MRN' from dual union all
select 29278,'MRN' from dual union all
select 29279,'MRN' from dual union all
select 2928,'USA' from dual union all
select 29280,'MRN' from dual union all
select 29281,'MRN' from dual union all
select 29282,'MRN' from dual union all
select 29283,'MRN' from dual union all
select 29284,'MRN' from dual union all
select 29285,'MRN' from dual union all
select 29286,'MRN' from dual union all
select 29287,'MRN' from dual union all
select 29288,'MRN' from dual union all
select 29289,'MRN' from dual union all
select 2929,'USA' from dual union all
select 29290,'MRN' from dual union all
select 29291,'MRN' from dual union all
select 29292,'MRN' from dual union all
select 29293,'MRN' from dual union all
select 29294,'MRN' from dual union all
select 29295,'MRN' from dual union all
select 29296,'MRN' from dual union all
select 29297,'MRN' from dual union all
select 29298,'MRN' from dual union all
select 29299,'MRN' from dual union all
select 293,'WER271-1' from dual union all
select 2930,'USA' from dual union all
select 29300,'MRN' from dual union all
select 29301,'MRN' from dual union all
select 29302,'MRN' from dual union all
select 29303,'MRN' from dual union all
select 29304,'MRN' from dual union all
select 29305,'MRN' from dual union all
select 29306,'MRN' from dual union all
select 29307,'MRN' from dual union all
select 29308,'MRN' from dual union all
select 29309,'MRN' from dual union all
select 2931,'USA' from dual union all
select 29310,'MRN' from dual union all
select 29311,'MRN' from dual union all
select 29312,'MRN' from dual union all
select 29313,'MRN' from dual union all
select 29314,'MRN' from dual union all
select 29315,'MRN' from dual union all
select 29316,'MRN' from dual union all
select 29317,'MRN' from dual union all
select 29318,'MRN' from dual union all
select 29319,'MRN' from dual union all
select 2932,'USA' from dual union all
select 29320,'MRN' from dual union all
select 29321,'MRN' from dual union all
select 29322,'MRN' from dual union all
select 29323,'MRN' from dual union all
select 29324,'MRN' from dual;
insert into LAUNCH_AGENCY
select 29325,'MRN' from dual union all
select 29326,'MRN' from dual union all
select 29327,'MRN' from dual union all
select 29328,'MRN' from dual union all
select 29329,'MRN' from dual union all
select 2933,'USA' from dual union all
select 29330,'MRN' from dual union all
select 29331,'MRN' from dual union all
select 29332,'MRN' from dual union all
select 29333,'MRN' from dual union all
select 29334,'MRN' from dual union all
select 29335,'MRN' from dual union all
select 29336,'MRN' from dual union all
select 29337,'MRN' from dual union all
select 29338,'MRN' from dual union all
select 29339,'MRN' from dual union all
select 2934,'USA' from dual union all
select 29340,'MRN' from dual union all
select 29341,'MRN' from dual union all
select 29342,'MRN' from dual union all
select 29343,'MRN' from dual union all
select 29344,'MRN' from dual union all
select 29345,'MRN' from dual union all
select 29346,'MRN' from dual union all
select 29347,'MRN' from dual union all
select 29348,'MRN' from dual union all
select 29349,'MRN' from dual union all
select 2935,'USA' from dual union all
select 29350,'MRN' from dual union all
select 29351,'MRN' from dual union all
select 29352,'MRN' from dual union all
select 29353,'MRN' from dual union all
select 29354,'MRN' from dual union all
select 29355,'MRN' from dual union all
select 29356,'MRN' from dual union all
select 29357,'MRN' from dual union all
select 29358,'MRN' from dual union all
select 29359,'MRN' from dual union all
select 2936,'USA' from dual union all
select 29360,'MRN' from dual union all
select 29361,'MRN' from dual union all
select 29362,'MRN' from dual union all
select 29363,'MRN' from dual union all
select 29364,'MRN' from dual union all
select 29365,'UKMO' from dual union all
select 29366,'MRN' from dual union all
select 29367,'MRN' from dual union all
select 29368,'MRN' from dual union all
select 29369,'MRN' from dual union all
select 2937,'USA' from dual union all
select 29370,'MRN' from dual union all
select 29371,'MRN' from dual union all
select 29372,'MRN' from dual union all
select 29373,'MRN' from dual union all
select 29374,'USA' from dual union all
select 29375,'MRN' from dual union all
select 29376,'MRN' from dual union all
select 29377,'USA' from dual union all
select 29378,'MRN' from dual union all
select 29379,'MRN' from dual union all
select 2938,'11TMS' from dual union all
select 29380,'NASDA' from dual union all
select 29381,'INTA' from dual union all
select 29382,'MRN' from dual union all
select 29383,'MRN' from dual union all
select 29384,'MRN' from dual union all
select 29385,'MRN' from dual union all
select 29386,'MRN' from dual union all
select 29387,'MRN' from dual union all
select 29388,'MRN' from dual union all
select 29389,'MRN' from dual union all
select 2939,'USA' from dual union all
select 29390,'MRN' from dual union all
select 29391,'MRN' from dual union all
select 29392,'MRN' from dual union all
select 29393,'MRN' from dual union all
select 29394,'MRN' from dual union all
select 29395,'MRN' from dual union all
select 29396,'MRN' from dual union all
select 29397,'MRN' from dual union all
select 29398,'MRN' from dual union all
select 29399,'MRN' from dual union all
select 294,'WER836' from dual union all
select 2940,'USA' from dual union all
select 29400,'MRN' from dual union all
select 29401,'MRN' from dual union all
select 29402,'NASDA' from dual union all
select 29403,'MRN' from dual union all
select 29404,'MRN' from dual union all
select 29405,'MRN' from dual union all
select 29406,'MRN' from dual union all
select 29407,'MRN' from dual union all
select 29408,'MRN' from dual union all
select 29409,'MRN' from dual union all
select 2941,'USA' from dual union all
select 29410,'MRN' from dual union all
select 29411,'MRN' from dual union all
select 29412,'MRN' from dual union all
select 29413,'MRN' from dual union all
select 29414,'MRN' from dual;
insert into LAUNCH_AGENCY
select 29415,'MRN' from dual union all
select 29416,'MRN' from dual union all
select 29417,'INTA' from dual union all
select 29418,'MRN' from dual union all
select 29419,'MRN' from dual union all
select 2942,'USA' from dual union all
select 29420,'MRN' from dual union all
select 29421,'MRN' from dual union all
select 29422,'MRN' from dual union all
select 29423,'MRN' from dual union all
select 29424,'MRN' from dual union all
select 29425,'MRN' from dual union all
select 29426,'MRN' from dual union all
select 29427,'MRN' from dual union all
select 29428,'MRN' from dual union all
select 29429,'MRN' from dual union all
select 2943,'PARD' from dual union all
select 29430,'INTA' from dual union all
select 29431,'MRN' from dual union all
select 29432,'MRN' from dual union all
select 29433,'MRN' from dual union all
select 29434,'MRN' from dual union all
select 29435,'MRN' from dual union all
select 29436,'MRN' from dual union all
select 29437,'INTA' from dual union all
select 29438,'MRN' from dual union all
select 29439,'MRN' from dual union all
select 2944,'USA' from dual union all
select 29440,'MRN' from dual union all
select 29441,'MRN' from dual union all
select 29442,'MRN' from dual union all
select 29443,'MRN' from dual union all
select 29444,'USA' from dual union all
select 29445,'MRN' from dual union all
select 29446,'MRN' from dual union all
select 29447,'MRN' from dual union all
select 29448,'MRN' from dual union all
select 29449,'MRN' from dual union all
select 2945,'USA' from dual union all
select 29450,'MRN' from dual union all
select 29451,'MRN' from dual union all
select 29452,'MRN' from dual union all
select 29453,'MRN' from dual union all
select 29454,'MRN' from dual union all
select 29455,'MRN' from dual union all
select 29456,'MRN' from dual union all
select 29457,'MRN' from dual union all
select 29458,'MRN' from dual union all
select 29459,'INTA' from dual union all
select 2946,'USA' from dual union all
select 29460,'MRN' from dual union all
select 29461,'MRN' from dual union all
select 29462,'MRN' from dual union all
select 29463,'MRN' from dual union all
select 29464,'MRN' from dual union all
select 29465,'INTA' from dual union all
select 29466,'MRN' from dual union all
select 29467,'NASA' from dual union all
select 29468,'MRN' from dual union all
select 29469,'MRN' from dual union all
select 2947,'USA' from dual union all
select 29470,'MRN' from dual union all
select 29471,'MRN' from dual union all
select 29472,'MRN' from dual union all
select 29473,'MRN' from dual union all
select 29474,'MRN' from dual union all
select 29475,'MRN' from dual union all
select 29476,'MRN' from dual union all
select 29477,'MRN' from dual union all
select 29478,'MRN' from dual union all
select 29479,'MRN' from dual union all
select 2948,'USA' from dual union all
select 29480,'INTA' from dual union all
select 29481,'MRN' from dual union all
select 29482,'MRN' from dual union all
select 29483,'MRN' from dual union all
select 29484,'MRN' from dual union all
select 29485,'MRN' from dual union all
select 29486,'MRN' from dual union all
select 29487,'MRN' from dual union all
select 29488,'MRN' from dual union all
select 29489,'MRN' from dual union all
select 2949,'USA' from dual union all
select 29490,'MRN' from dual union all
select 29491,'MRN' from dual union all
select 29492,'MRN' from dual union all
select 29493,'MRN' from dual union all
select 29494,'MRN' from dual union all
select 29495,'INTA' from dual union all
select 29496,'MRN' from dual union all
select 29497,'MRN' from dual union all
select 29498,'MRN' from dual union all
select 29499,'MRN' from dual union all
select 295,'WER485-1' from dual union all
select 2950,'USA' from dual union all
select 29500,'MRN' from dual union all
select 29501,'MRN' from dual union all
select 29502,'MRN' from dual union all
select 29503,'MRN' from dual union all
select 29504,'MRN' from dual;
insert into LAUNCH_AGENCY
select 29505,'MRN' from dual union all
select 29506,'MRN' from dual union all
select 29507,'MRN' from dual union all
select 29508,'MRN' from dual union all
select 29509,'MRN' from dual union all
select 2951,'USA' from dual union all
select 29510,'MRN' from dual union all
select 29511,'MRN' from dual union all
select 29512,'MRN' from dual union all
select 29513,'MRN' from dual union all
select 29514,'MRN' from dual union all
select 29515,'MRN' from dual union all
select 29516,'MRN' from dual union all
select 29517,'MRN' from dual union all
select 29518,'MRN' from dual union all
select 29519,'MRN' from dual union all
select 2952,'USAF' from dual union all
select 29520,'MRN' from dual union all
select 29521,'MRN' from dual union all
select 29522,'MRN' from dual union all
select 29523,'MRN' from dual union all
select 29524,'MRN' from dual union all
select 29525,'MRN' from dual union all
select 29526,'MRN' from dual union all
select 29527,'MRN' from dual union all
select 29528,'MRN' from dual union all
select 29529,'MRN' from dual union all
select 2953,'ORDRSA' from dual union all
select 29530,'MRN' from dual union all
select 29531,'MRN' from dual union all
select 29532,'MRN' from dual union all
select 29533,'MRN' from dual union all
select 29534,'MRN' from dual union all
select 29535,'MRN' from dual union all
select 29536,'MRN' from dual union all
select 29537,'MRN' from dual union all
select 29538,'MRN' from dual union all
select 29539,'MRN' from dual union all
select 2954,'PARD' from dual union all
select 29540,'MRN' from dual union all
select 29541,'MRN' from dual union all
select 29542,'MRN' from dual union all
select 29543,'MRN' from dual union all
select 29544,'MRN' from dual union all
select 29545,'MRN' from dual union all
select 29546,'MRN' from dual union all
select 29547,'MRN' from dual union all
select 29548,'MRN' from dual union all
select 29549,'MRN' from dual union all
select 2955,'ORDRSA' from dual union all
select 29550,'MRN' from dual union all
select 29551,'MRN' from dual union all
select 29552,'MRN' from dual union all
select 29553,'MRN' from dual union all
select 29554,'MRN' from dual union all
select 29555,'MRN' from dual union all
select 29556,'MRN' from dual union all
select 29557,'MRN' from dual union all
select 29558,'MRN' from dual union all
select 29559,'MRN' from dual union all
select 2956,'ORDRSA' from dual union all
select 29560,'MRN' from dual union all
select 29561,'MRN' from dual union all
select 29562,'MRN' from dual union all
select 29563,'MRN' from dual union all
select 29564,'MRN' from dual union all
select 29565,'MRN' from dual union all
select 29566,'MRN' from dual union all
select 29567,'MRN' from dual union all
select 29568,'MRN' from dual union all
select 29569,'MRN' from dual union all
select 2957,'ORDRSA' from dual union all
select 29570,'MRN' from dual union all
select 29571,'MRN' from dual union all
select 29572,'MRN' from dual union all
select 29573,'MRN' from dual union all
select 29574,'MRN' from dual union all
select 29575,'MRN' from dual union all
select 29576,'MRN' from dual union all
select 29577,'MRN' from dual union all
select 29578,'MRN' from dual union all
select 29579,'MRN' from dual union all
select 2958,'USAF' from dual union all
select 29580,'MRN' from dual union all
select 29581,'MRN' from dual union all
select 29582,'MRN' from dual union all
select 29583,'MRN' from dual union all
select 29584,'MRN' from dual union all
select 29585,'INTA' from dual union all
select 29586,'MRN' from dual union all
select 29587,'MRN' from dual union all
select 29588,'MRN' from dual union all
select 29589,'MRN' from dual union all
select 2959,'USA' from dual union all
select 29590,'MRN' from dual union all
select 29591,'MRN' from dual union all
select 29592,'MRN' from dual union all
select 29593,'MRN' from dual union all
select 29594,'CNES' from dual union all
select 29595,'MRN' from dual;
insert into LAUNCH_AGENCY
select 29596,'MRN' from dual union all
select 29597,'MRN' from dual union all
select 29598,'MRN' from dual union all
select 29599,'MRN' from dual union all
select 296,'WER485-3' from dual union all
select 2960,'USA' from dual union all
select 29600,'MRN' from dual union all
select 29601,'MRN' from dual union all
select 29602,'MRN' from dual union all
select 29603,'MRN' from dual union all
select 29604,'MRN' from dual union all
select 29605,'MRN' from dual union all
select 29606,'MRN' from dual union all
select 29607,'MRN' from dual union all
select 29608,'MRN' from dual union all
select 29609,'MRN' from dual union all
select 2961,'USA' from dual union all
select 29610,'MRN' from dual union all
select 29611,'MRN' from dual union all
select 29612,'MRN' from dual union all
select 29613,'MRN' from dual union all
select 29614,'MRN' from dual union all
select 29615,'MRN' from dual union all
select 29616,'MRN' from dual union all
select 29617,'MRN' from dual union all
select 29618,'MRN' from dual union all
select 29619,'MRN' from dual union all
select 2962,'USA' from dual union all
select 29620,'MRN' from dual union all
select 29621,'MRN' from dual union all
select 29622,'MRN' from dual union all
select 29623,'MRN' from dual union all
select 29624,'MRN' from dual union all
select 29625,'MRN' from dual union all
select 29626,'MRN' from dual union all
select 29627,'MRN' from dual union all
select 29628,'MRN' from dual union all
select 29629,'MRN' from dual union all
select 2963,'USA' from dual union all
select 29630,'MRN' from dual union all
select 29631,'MRN' from dual union all
select 29632,'MRN' from dual union all
select 29633,'MRN' from dual union all
select 29634,'INTA' from dual union all
select 29635,'MRN' from dual union all
select 29636,'MRN' from dual union all
select 29637,'MRN' from dual union all
select 29638,'MRN' from dual union all
select 29639,'MRN' from dual union all
select 2964,'USA' from dual union all
select 29640,'MRN' from dual union all
select 29641,'MRN' from dual union all
select 29642,'MRN' from dual union all
select 29643,'MRN' from dual union all
select 29644,'MRN' from dual union all
select 29645,'MRN' from dual union all
select 29646,'MRN' from dual union all
select 29647,'MRN' from dual union all
select 29648,'MRN' from dual union all
select 29649,'MRN' from dual union all
select 2965,'USA' from dual union all
select 29650,'MRN' from dual union all
select 29651,'MRN' from dual union all
select 29652,'MRN' from dual union all
select 29653,'MRN' from dual union all
select 29654,'MRN' from dual union all
select 29655,'MRN' from dual union all
select 29656,'MRN' from dual union all
select 29657,'MRN' from dual union all
select 29658,'MRN' from dual union all
select 29659,'MRN' from dual union all
select 2966,'USA' from dual union all
select 29660,'MRN' from dual union all
select 29661,'MRN' from dual union all
select 29662,'MRN' from dual union all
select 29663,'MRN' from dual union all
select 29664,'MRN' from dual union all
select 29665,'MRN' from dual union all
select 29666,'MRN' from dual union all
select 29667,'MRN' from dual union all
select 29668,'MRN' from dual union all
select 29669,'MRN' from dual union all
select 2967,'ORDRSA' from dual union all
select 29670,'MRN' from dual union all
select 29671,'MRN' from dual union all
select 29672,'MRN' from dual union all
select 29673,'MRN' from dual union all
select 29674,'MRN' from dual union all
select 29675,'MRN' from dual union all
select 29676,'MRN' from dual union all
select 29677,'MRN' from dual union all
select 29678,'MRN' from dual union all
select 29679,'MRN' from dual union all
select 2968,'USAF' from dual union all
select 29680,'MRN' from dual union all
select 29681,'MRN' from dual union all
select 29682,'MRN' from dual union all
select 29683,'MRN' from dual union all
select 29684,'MRN' from dual union all
select 29685,'MRN' from dual;
insert into LAUNCH_AGENCY
select 29686,'MRN' from dual union all
select 29687,'MRN' from dual union all
select 29688,'MRN' from dual union all
select 29689,'MRN' from dual union all
select 2969,'USAF' from dual union all
select 29690,'MRN' from dual union all
select 29691,'MRN' from dual union all
select 29692,'MRN' from dual union all
select 29693,'MRN' from dual union all
select 29694,'MRN' from dual union all
select 29695,'MRN' from dual union all
select 29696,'MRN' from dual union all
select 29697,'MRN' from dual union all
select 29698,'MRN' from dual union all
select 29699,'INTA' from dual union all
select 297,'WEHR' from dual union all
select 2970,'USA' from dual union all
select 29700,'MRN' from dual union all
select 29701,'MRN' from dual union all
select 29702,'MRN' from dual union all
select 29703,'MRN' from dual union all
select 29704,'MRN' from dual union all
select 29705,'MRN' from dual union all
select 29706,'MRN' from dual union all
select 29707,'MRN' from dual union all
select 29708,'MRN' from dual union all
select 29709,'MRN' from dual union all
select 2971,'RCAR' from dual union all
select 29710,'MRN' from dual union all
select 29711,'MRN' from dual union all
select 29712,'MRN' from dual union all
select 29713,'MRN' from dual union all
select 29714,'MRN' from dual union all
select 29715,'MRN' from dual union all
select 29716,'MRN' from dual union all
select 29717,'MRN' from dual union all
select 29718,'MRN' from dual union all
select 29719,'MRN' from dual union all
select 2972,'USAF' from dual union all
select 29720,'MRN' from dual union all
select 29721,'MRN' from dual union all
select 29722,'MRN' from dual union all
select 29723,'MRN' from dual union all
select 29724,'MRN' from dual union all
select 29725,'MRN' from dual union all
select 29726,'MRN' from dual union all
select 29727,'MRN' from dual union all
select 29728,'MRN' from dual union all
select 29729,'MRN' from dual union all
select 2973,'USA' from dual union all
select 29730,'MRN' from dual union all
select 29731,'MRN' from dual union all
select 29732,'INTA' from dual union all
select 29733,'MRN' from dual union all
select 29734,'MRN' from dual union all
select 29735,'MRN' from dual union all
select 29736,'MRN' from dual union all
select 29737,'MRN' from dual union all
select 29738,'MRN' from dual union all
select 29739,'MRN' from dual union all
select 2974,'USA' from dual union all
select 29740,'MRN' from dual union all
select 29741,'MRN' from dual union all
select 29742,'MRN' from dual union all
select 29743,'MRN' from dual union all
select 29744,'MRN' from dual union all
select 29745,'MRN' from dual union all
select 29746,'MRN' from dual union all
select 29747,'MRN' from dual union all
select 29748,'MRN' from dual union all
select 29749,'MRN' from dual union all
select 2975,'USA' from dual union all
select 29750,'MRN' from dual union all
select 29751,'MRN' from dual union all
select 29752,'MRN' from dual union all
select 29753,'MRN' from dual union all
select 29754,'MRN' from dual union all
select 29755,'MRN' from dual union all
select 29756,'MRN' from dual union all
select 29757,'MRN' from dual union all
select 29758,'MRN' from dual union all
select 29759,'MRN' from dual union all
select 2976,'USA' from dual union all
select 29760,'MRN' from dual union all
select 29761,'MRN' from dual union all
select 29762,'MRN' from dual union all
select 29763,'MRN' from dual union all
select 29764,'MRN' from dual union all
select 29765,'MRN' from dual union all
select 29766,'MRN' from dual union all
select 29767,'MRN' from dual union all
select 29768,'MRN' from dual union all
select 29769,'MRN' from dual union all
select 2977,'USA' from dual union all
select 29770,'MRN' from dual union all
select 29771,'MRN' from dual union all
select 29772,'MRN' from dual union all
select 29773,'MRN' from dual union all
select 29774,'MRN' from dual union all
select 29775,'MRN' from dual;
insert into LAUNCH_AGENCY
select 29776,'MRN' from dual union all
select 29777,'MRN' from dual union all
select 29778,'MRN' from dual union all
select 29779,'MRN' from dual union all
select 2978,'USA' from dual union all
select 29780,'MRN' from dual union all
select 29781,'MRN' from dual union all
select 29782,'MRN' from dual union all
select 29783,'MRN' from dual union all
select 29784,'MRN' from dual union all
select 29785,'MRN' from dual union all
select 29786,'MRN' from dual union all
select 29787,'MRN' from dual union all
select 29788,'MRN' from dual union all
select 29789,'MRN' from dual union all
select 2979,'USA' from dual union all
select 29790,'MRN' from dual union all
select 29791,'MRN' from dual union all
select 29792,'MRN' from dual union all
select 29793,'MRN' from dual union all
select 29794,'MRN' from dual union all
select 29795,'MRN' from dual union all
select 29796,'MRN' from dual union all
select 29797,'MRN' from dual union all
select 29798,'MRN' from dual union all
select 29799,'MRN' from dual union all
select 298,'WER485-1' from dual union all
select 2980,'USA' from dual union all
select 29800,'MRN' from dual union all
select 29801,'MRN' from dual union all
select 29802,'MRN' from dual union all
select 29803,'MRN' from dual union all
select 29804,'MRN' from dual union all
select 29805,'MRN' from dual union all
select 29806,'MRN' from dual union all
select 29807,'MRN' from dual union all
select 29808,'MRN' from dual union all
select 29809,'MRN' from dual union all
select 2981,'USA' from dual union all
select 29810,'MRN' from dual union all
select 29811,'MRN' from dual union all
select 29812,'MRN' from dual union all
select 29813,'MRN' from dual union all
select 29814,'MRN' from dual union all
select 29815,'MRN' from dual union all
select 29816,'MRN' from dual union all
select 29817,'MRN' from dual union all
select 29818,'MRN' from dual union all
select 29819,'MRN' from dual union all
select 2982,'USA' from dual union all
select 29820,'MRN' from dual union all
select 29821,'MRN' from dual union all
select 29822,'MRN' from dual union all
select 29823,'MRN' from dual union all
select 29824,'MRN' from dual union all
select 29825,'MRN' from dual union all
select 29826,'MRN' from dual union all
select 29827,'MRN' from dual union all
select 29828,'MRN' from dual union all
select 29829,'MRN' from dual union all
select 2983,'USA' from dual union all
select 29830,'MRN' from dual union all
select 29831,'MRN' from dual union all
select 29832,'INTA' from dual union all
select 29833,'MRN' from dual union all
select 29834,'MRN' from dual union all
select 29835,'MRN' from dual union all
select 29836,'MRN' from dual union all
select 29837,'MRN' from dual union all
select 29838,'MRN' from dual union all
select 29839,'MRN' from dual union all
select 2984,'USA' from dual union all
select 29840,'INTA' from dual union all
select 29841,'MRN' from dual union all
select 29842,'MRN' from dual union all
select 29843,'MRN' from dual union all
select 29844,'MRN' from dual union all
select 29845,'MRN' from dual union all
select 29846,'NASA' from dual union all
select 29847,'INTA' from dual union all
select 29848,'MRN' from dual union all
select 29849,'MRN' from dual union all
select 2985,'USA' from dual union all
select 29850,'MRN' from dual union all
select 29851,'MRN' from dual union all
select 29852,'MRN' from dual union all
select 29853,'MRN' from dual union all
select 29854,'MRN' from dual union all
select 29855,'MRN' from dual union all
select 29856,'MRN' from dual union all
select 29857,'MRN' from dual union all
select 29858,'MRN' from dual union all
select 29859,'MRN' from dual union all
select 2986,'USA' from dual union all
select 29860,'MRN' from dual union all
select 29861,'MRN' from dual union all
select 29862,'MRN' from dual union all
select 29863,'MRN' from dual union all
select 29864,'MRN' from dual union all
select 29865,'MRN' from dual;
insert into LAUNCH_AGENCY
select 29866,'MRN' from dual union all
select 29867,'MRN' from dual union all
select 29868,'MRN' from dual union all
select 29869,'MRN' from dual union all
select 2987,'USA' from dual union all
select 29870,'MRN' from dual union all
select 29871,'MRN' from dual union all
select 29872,'MRN' from dual union all
select 29873,'MRN' from dual union all
select 29874,'MRN' from dual union all
select 29875,'MRN' from dual union all
select 29876,'MRN' from dual union all
select 29877,'MRN' from dual union all
select 29878,'MRN' from dual union all
select 29879,'MRN' from dual union all
select 2988,'ORDRSA' from dual union all
select 29880,'MRN' from dual union all
select 29881,'MRN' from dual union all
select 29882,'MRN' from dual union all
select 29883,'MRN' from dual union all
select 29884,'MRN' from dual union all
select 29885,'MRN' from dual union all
select 29886,'MRN' from dual union all
select 29887,'MRN' from dual union all
select 29888,'MRN' from dual union all
select 29889,'MRN' from dual union all
select 2989,'USAF' from dual union all
select 29890,'MRN' from dual union all
select 29891,'MRN' from dual union all
select 29892,'MRN' from dual union all
select 29893,'MRN' from dual union all
select 29894,'MRN' from dual union all
select 29895,'MRN' from dual union all
select 29896,'MRN' from dual union all
select 29897,'MRN' from dual union all
select 29898,'MRN' from dual union all
select 29899,'MRN' from dual union all
select 299,'SS500' from dual union all
select 2990,'USA' from dual union all
select 29900,'MRN' from dual union all
select 29901,'MRN' from dual union all
select 29902,'MRN' from dual union all
select 29903,'MRN' from dual union all
select 29904,'MRN' from dual union all
select 29905,'MRN' from dual union all
select 29906,'MRN' from dual union all
select 29907,'MRN' from dual union all
select 29908,'MRN' from dual union all
select 29909,'MRN' from dual union all
select 2991,'USA' from dual union all
select 29910,'NASA' from dual union all
select 29911,'INTA' from dual union all
select 29912,'MRN' from dual union all
select 29913,'MRN' from dual union all
select 29914,'MRN' from dual union all
select 29915,'MRN' from dual union all
select 29916,'MRN' from dual union all
select 29917,'MRN' from dual union all
select 29918,'MRN' from dual union all
select 29919,'MRN' from dual union all
select 2992,'USA' from dual union all
select 29920,'MRN' from dual union all
select 29921,'MRN' from dual union all
select 29922,'MRN' from dual union all
select 29923,'MRN' from dual union all
select 29924,'MRN' from dual union all
select 29925,'MRN' from dual union all
select 29926,'MRN' from dual union all
select 29927,'MRN' from dual union all
select 29928,'MRN' from dual union all
select 29929,'MRN' from dual union all
select 2993,'USA' from dual union all
select 29930,'MRN' from dual union all
select 29931,'MRN' from dual union all
select 29932,'MRN' from dual union all
select 29933,'MRN' from dual union all
select 29934,'MRN' from dual union all
select 29935,'MRN' from dual union all
select 29936,'MRN' from dual union all
select 29937,'MRN' from dual union all
select 29938,'MRN' from dual union all
select 29939,'MRN' from dual union all
select 2994,'USA' from dual union all
select 29940,'MRN' from dual union all
select 29941,'MRN' from dual union all
select 29942,'MRN' from dual union all
select 29943,'MRN' from dual union all
select 29944,'MRN' from dual union all
select 29945,'NASA' from dual union all
select 29946,'MRN' from dual union all
select 29947,'MRN' from dual union all
select 29948,'MRN' from dual union all
select 29949,'MRN' from dual union all
select 2995,'USA' from dual union all
select 29950,'MRN' from dual union all
select 29951,'MRN' from dual union all
select 29952,'MRN' from dual union all
select 29953,'MRN' from dual union all
select 29954,'MRN' from dual union all
select 29955,'MRN' from dual;
insert into LAUNCH_AGENCY
select 29956,'MRN' from dual union all
select 29957,'MRN' from dual union all
select 29958,'MRN' from dual union all
select 29959,'MRN' from dual union all
select 2996,'USA' from dual union all
select 29960,'MRN' from dual union all
select 29961,'MRN' from dual union all
select 29962,'MRN' from dual union all
select 29963,'MRN' from dual union all
select 29964,'MRN' from dual union all
select 29965,'MRN' from dual union all
select 29966,'MRN' from dual union all
select 29967,'MRN' from dual union all
select 29968,'MRN' from dual union all
select 29969,'MRN' from dual union all
select 2997,'USAF' from dual union all
select 29970,'MRN' from dual union all
select 29971,'MRN' from dual union all
select 29972,'MRN' from dual union all
select 29973,'MRN' from dual union all
select 29974,'MRN' from dual union all
select 29975,'MRN' from dual union all
select 29976,'MRN' from dual union all
select 29977,'MRN' from dual union all
select 29978,'MRN' from dual union all
select 29979,'MRN' from dual union all
select 2998,'USA' from dual union all
select 29980,'MRN' from dual union all
select 29981,'MRN' from dual union all
select 29982,'MRN' from dual union all
select 29983,'NASA' from dual union all
select 29983,'NRCC' from dual union all
select 29984,'MRN' from dual union all
select 29985,'MRN' from dual union all
select 29986,'MRN' from dual union all
select 29987,'MRN' from dual union all
select 29988,'MRN' from dual union all
select 29989,'MRN' from dual union all
select 2999,'USA' from dual union all
select 29990,'MRN' from dual union all
select 29991,'MRN' from dual union all
select 29992,'MRN' from dual union all
select 29993,'MRN' from dual union all
select 29994,'NASA' from dual union all
select 29995,'MRN' from dual union all
select 29996,'MRN' from dual union all
select 29997,'MRN' from dual union all
select 29998,'MRN' from dual union all
select 29999,'MRN' from dual union all
select 3,'WEHR' from dual union all
select 30,'WEHR' from dual union all
select 300,'WER271-1' from dual union all
select 3000,'USA' from dual union all
select 30000,'MRN' from dual union all
select 30001,'MRN' from dual union all
select 30002,'MRN' from dual union all
select 30003,'MRN' from dual union all
select 30004,'MRN' from dual union all
select 30005,'MRN' from dual union all
select 30006,'MRN' from dual union all
select 30007,'MRN' from dual union all
select 30008,'MRN' from dual union all
select 30009,'MRN' from dual union all
select 3001,'PARD' from dual union all
select 30010,'MRN' from dual union all
select 30011,'MRN' from dual union all
select 30012,'MRN' from dual union all
select 30013,'MRN' from dual union all
select 30014,'MRN' from dual union all
select 30015,'MRN' from dual union all
select 30016,'MRN' from dual union all
select 30017,'MRN' from dual union all
select 30018,'MRN' from dual union all
select 30019,'MRN' from dual union all
select 3002,'RCAR' from dual union all
select 30020,'MRN' from dual union all
select 30021,'MRN' from dual union all
select 30022,'MRN' from dual union all
select 30023,'MRN' from dual union all
select 30024,'MRN' from dual union all
select 30025,'MRN' from dual union all
select 30026,'MRN' from dual union all
select 30027,'MRN' from dual union all
select 30028,'MRN' from dual union all
select 30029,'MRN' from dual union all
select 3003,'RCAR' from dual union all
select 30030,'MRN' from dual union all
select 30031,'MRN' from dual union all
select 30032,'MRN' from dual union all
select 30033,'MRN' from dual union all
select 30034,'MRN' from dual union all
select 30035,'MRN' from dual union all
select 30036,'MRN' from dual union all
select 30037,'MRN' from dual union all
select 30038,'MRN' from dual union all
select 30039,'MRN' from dual union all
select 3004,'USA' from dual union all
select 30040,'MRN' from dual union all
select 30041,'MRN' from dual union all
select 30042,'MRN' from dual;
insert into LAUNCH_AGENCY
select 30043,'MRN' from dual union all
select 30044,'MRN' from dual union all
select 30045,'MRN' from dual union all
select 30046,'MRN' from dual union all
select 30047,'MRN' from dual union all
select 30048,'MRN' from dual union all
select 30049,'MRN' from dual union all
select 3005,'USA' from dual union all
select 30050,'MRN' from dual union all
select 30051,'MRN' from dual union all
select 30052,'MRN' from dual union all
select 30053,'MRN' from dual union all
select 30054,'MRN' from dual union all
select 30055,'MRN' from dual union all
select 30056,'MRN' from dual union all
select 30057,'MRN' from dual union all
select 30058,'MRN' from dual union all
select 30059,'MRN' from dual union all
select 3006,'USA' from dual union all
select 30060,'MRN' from dual union all
select 30061,'MRN' from dual union all
select 30062,'MRN' from dual union all
select 30063,'MRN' from dual union all
select 30064,'MRN' from dual union all
select 30065,'MRN' from dual union all
select 30066,'MRN' from dual union all
select 30067,'MRN' from dual union all
select 30068,'MRN' from dual union all
select 30069,'MRN' from dual union all
select 3007,'USA' from dual union all
select 30070,'MRN' from dual union all
select 30071,'MRN' from dual union all
select 30072,'MRN' from dual union all
select 30073,'MRN' from dual union all
select 30074,'MRN' from dual union all
select 30075,'MRN' from dual union all
select 30076,'MRN' from dual union all
select 30077,'MRN' from dual union all
select 30078,'MRN' from dual union all
select 30079,'MRN' from dual union all
select 3008,'USAF' from dual union all
select 30080,'MRN' from dual union all
select 30081,'MRN' from dual union all
select 30082,'MRN' from dual union all
select 30083,'MRN' from dual union all
select 30084,'MRN' from dual union all
select 30085,'MRN' from dual union all
select 30086,'MRN' from dual union all
select 30087,'MRN' from dual union all
select 30088,'MRN' from dual union all
select 30089,'MRN' from dual union all
select 3009,'USA' from dual union all
select 30090,'MRN' from dual union all
select 30091,'MRN' from dual union all
select 30092,'MRN' from dual union all
select 30093,'MRN' from dual union all
select 30094,'MRN' from dual union all
select 30095,'MRN' from dual union all
select 30096,'MRN' from dual union all
select 30097,'MRN' from dual union all
select 30098,'MRN' from dual union all
select 30099,'MRN' from dual union all
select 301,'WER271-1' from dual union all
select 3010,'USA' from dual union all
select 30100,'MRN' from dual union all
select 30101,'MRN' from dual union all
select 30102,'MRN' from dual union all
select 30103,'MRN' from dual union all
select 30104,'MRN' from dual union all
select 30105,'MRN' from dual union all
select 30106,'MRN' from dual union all
select 30107,'MRN' from dual union all
select 30108,'MRN' from dual union all
select 30109,'MRN' from dual union all
select 3011,'ORDRSA' from dual union all
select 30110,'MRN' from dual union all
select 30111,'MRN' from dual union all
select 30112,'MRN' from dual union all
select 30113,'MRN' from dual union all
select 30114,'MRN' from dual union all
select 30115,'MRN' from dual union all
select 30116,'MRN' from dual union all
select 30117,'MRN' from dual union all
select 30118,'LARCN' from dual union all
select 30119,'MRN' from dual union all
select 3012,'ORDRSA' from dual union all
select 30120,'MRN' from dual union all
select 30121,'MRN' from dual union all
select 30122,'MRN' from dual union all
select 30123,'LARCN' from dual union all
select 30124,'LARCN' from dual union all
select 30125,'MRN' from dual union all
select 30126,'MRN' from dual union all
select 30127,'MRN' from dual union all
select 30128,'LARCN' from dual union all
select 30129,'MRN' from dual union all
select 3013,'USA' from dual union all
select 30130,'MRN' from dual union all
select 30131,'MRN' from dual union all
select 30132,'MRN' from dual;
insert into LAUNCH_AGENCY
select 30133,'MRN' from dual union all
select 30134,'MRN' from dual union all
select 30135,'MRN' from dual union all
select 30136,'MRN' from dual union all
select 30137,'MRN' from dual union all
select 30138,'MRN' from dual union all
select 30139,'MRN' from dual union all
select 3014,'USA' from dual union all
select 30140,'MRN' from dual union all
select 30141,'MRN' from dual union all
select 30142,'MRN' from dual union all
select 30143,'MRN' from dual union all
select 30144,'MRN' from dual union all
select 30145,'MRN' from dual union all
select 30146,'MRN' from dual union all
select 30147,'MRN' from dual union all
select 30148,'MRN' from dual union all
select 30149,'MRN' from dual union all
select 3015,'USA' from dual union all
select 30150,'MRN' from dual union all
select 30151,'MRN' from dual union all
select 30152,'MRN' from dual union all
select 30153,'MRN' from dual union all
select 30154,'MRN' from dual union all
select 30155,'MRN' from dual union all
select 30156,'MRN' from dual union all
select 30157,'MRN' from dual union all
select 30158,'MRN' from dual union all
select 30159,'MRN' from dual union all
select 3016,'ORDRSA' from dual union all
select 30160,'MRN' from dual union all
select 30161,'MRN' from dual union all
select 30162,'MRN' from dual union all
select 30163,'MRN' from dual union all
select 30164,'MRN' from dual union all
select 30165,'MRN' from dual union all
select 30166,'MRN' from dual union all
select 30167,'MRN' from dual union all
select 30168,'MRN' from dual union all
select 30169,'MRN' from dual union all
select 3017,'USA' from dual union all
select 30170,'AN' from dual union all
select 30171,'MRN' from dual union all
select 30172,'MRN' from dual union all
select 30173,'MRN' from dual union all
select 30174,'MRN' from dual union all
select 30175,'MRN' from dual union all
select 30176,'MRN' from dual union all
select 30177,'MRN' from dual union all
select 30178,'BAL' from dual union all
select 30179,'BAL' from dual union all
select 3018,'USA' from dual union all
select 30180,'MRN' from dual union all
select 30181,'MRN' from dual union all
select 30182,'BAL' from dual union all
select 30183,'BAL' from dual union all
select 30184,'MRN' from dual union all
select 30185,'BAL' from dual union all
select 30186,'BAL' from dual union all
select 30187,'BAL' from dual union all
select 30188,'MRN' from dual union all
select 30189,'MRN' from dual union all
select 3019,'USA' from dual union all
select 30190,'MRN' from dual union all
select 30191,'BAL' from dual union all
select 30192,'MRN' from dual union all
select 30193,'MRN' from dual union all
select 30194,'MRN' from dual union all
select 30195,'MRN' from dual union all
select 30196,'MRN' from dual union all
select 30197,'MRN' from dual union all
select 30198,'MRN' from dual union all
select 30199,'MRN' from dual union all
select 302,'WER271-1' from dual union all
select 3020,'USA' from dual union all
select 30200,'MRN' from dual union all
select 30201,'MRN' from dual union all
select 30202,'MRN' from dual union all
select 30203,'MRN' from dual union all
select 30204,'MRN' from dual union all
select 30205,'MRN' from dual union all
select 30206,'MRN' from dual union all
select 30207,'MRN' from dual union all
select 30208,'MRN' from dual union all
select 30209,'MRN' from dual union all
select 3021,'USA' from dual union all
select 30210,'MRN' from dual union all
select 30211,'MRN' from dual union all
select 30212,'MRN' from dual union all
select 30213,'MRN' from dual union all
select 30214,'MRN' from dual union all
select 30215,'MRN' from dual union all
select 30216,'MRN' from dual union all
select 30217,'MRN' from dual union all
select 30218,'MRN' from dual union all
select 30219,'MRN' from dual union all
select 3022,'USAF' from dual union all
select 30220,'MRN' from dual union all
select 30221,'MRN' from dual union all
select 30222,'MRN' from dual;
insert into LAUNCH_AGENCY
select 30223,'MRN' from dual union all
select 30224,'MRN' from dual union all
select 30225,'MRN' from dual union all
select 30226,'MRN' from dual union all
select 30227,'MRN' from dual union all
select 30228,'MRN' from dual union all
select 30229,'MRN' from dual union all
select 3023,'USAF' from dual union all
select 30230,'MRN' from dual union all
select 30231,'MRN' from dual union all
select 30232,'MRN' from dual union all
select 30233,'MRN' from dual union all
select 30234,'MRN' from dual union all
select 30235,'MRN' from dual union all
select 30236,'USA' from dual union all
select 30237,'MRN' from dual union all
select 30238,'MRN' from dual union all
select 30239,'MRN' from dual union all
select 3024,'USA' from dual union all
select 30240,'MRN' from dual union all
select 30241,'MRN' from dual union all
select 30242,'MRN' from dual union all
select 30243,'MRN' from dual union all
select 30244,'MRN' from dual union all
select 30245,'MRN' from dual union all
select 30246,'MRN' from dual union all
select 30247,'MRN' from dual union all
select 30248,'MRN' from dual union all
select 30249,'MRN' from dual union all
select 3025,'USA' from dual union all
select 30250,'MRN' from dual union all
select 30251,'MRN' from dual union all
select 30252,'MRN' from dual union all
select 30253,'MRN' from dual union all
select 30254,'MRN' from dual union all
select 30255,'MRN' from dual union all
select 30256,'MRN' from dual union all
select 30257,'MRN' from dual union all
select 30258,'USA' from dual union all
select 30259,'MRN' from dual union all
select 3026,'USA' from dual union all
select 30260,'MRN' from dual union all
select 30261,'MRN' from dual union all
select 30262,'MRN' from dual union all
select 30263,'MRN' from dual union all
select 30264,'MRN' from dual union all
select 30265,'MRN' from dual union all
select 30266,'MRN' from dual union all
select 30267,'MRN' from dual union all
select 30268,'MRN' from dual union all
select 30269,'MRN' from dual union all
select 3027,'USA' from dual union all
select 30270,'MRN' from dual union all
select 30271,'MRN' from dual union all
select 30272,'MRN' from dual union all
select 30273,'MRN' from dual union all
select 30274,'MRN' from dual union all
select 30275,'MRN' from dual union all
select 30276,'MRN' from dual union all
select 30277,'MRN' from dual union all
select 30278,'MRN' from dual union all
select 30279,'MRN' from dual union all
select 3028,'USA' from dual union all
select 30280,'MRN' from dual union all
select 30281,'MRN' from dual union all
select 30282,'MRN' from dual union all
select 30283,'MRN' from dual union all
select 30284,'MRN' from dual union all
select 30285,'MRN' from dual union all
select 30286,'MRN' from dual union all
select 30287,'MRN' from dual union all
select 30288,'MRN' from dual union all
select 30289,'MRN' from dual union all
select 3029,'USA' from dual union all
select 30290,'MRN' from dual union all
select 30291,'AN' from dual union all
select 30292,'AN' from dual union all
select 30293,'AN' from dual union all
select 30294,'AN' from dual union all
select 30295,'AN' from dual union all
select 30296,'AN' from dual union all
select 30297,'AN' from dual union all
select 30298,'AN' from dual union all
select 30299,'AN' from dual union all
select 303,'WER271-1' from dual union all
select 3030,'USA' from dual union all
select 30300,'AN' from dual union all
select 30301,'AN' from dual union all
select 30302,'AN' from dual union all
select 30303,'AN' from dual union all
select 30304,'AN' from dual union all
select 30305,'AN' from dual union all
select 30306,'AN' from dual union all
select 30307,'AN' from dual union all
select 30308,'AN' from dual union all
select 30309,'AN' from dual union all
select 3031,'USA' from dual union all
select 30310,'AN' from dual union all
select 30311,'MRN' from dual union all
select 30312,'AN' from dual;
insert into LAUNCH_AGENCY
select 30313,'AN' from dual union all
select 30314,'AN' from dual union all
select 30315,'AN' from dual union all
select 30316,'AN' from dual union all
select 30317,'MRN' from dual union all
select 30318,'MRN' from dual union all
select 30319,'AN' from dual union all
select 3032,'USA' from dual union all
select 30320,'AN' from dual union all
select 30321,'AN' from dual union all
select 30322,'AN' from dual union all
select 30323,'AN' from dual union all
select 30324,'AN' from dual union all
select 30325,'AN' from dual union all
select 30326,'AN' from dual union all
select 30327,'AN' from dual union all
select 30328,'AN' from dual union all
select 30329,'AN' from dual union all
select 3033,'USA' from dual union all
select 30330,'AN' from dual union all
select 30331,'MRN' from dual union all
select 30332,'SRC' from dual union all
select 30333,'AN' from dual union all
select 30334,'AN' from dual union all
select 30335,'MRN' from dual union all
select 30336,'MRN' from dual union all
select 30337,'AN' from dual union all
select 30338,'MRN' from dual union all
select 30339,'AN' from dual union all
select 3034,'USA' from dual union all
select 30340,'MRN' from dual union all
select 30341,'AN' from dual union all
select 30342,'SRC' from dual union all
select 30343,'AN' from dual union all
select 30344,'AN' from dual union all
select 30345,'MRN' from dual union all
select 30346,'MRN' from dual union all
select 30347,'AN' from dual union all
select 30348,'MRN' from dual union all
select 30349,'AN' from dual union all
select 3035,'ORDRSA' from dual union all
select 30350,'AN' from dual union all
select 30351,'MRN' from dual union all
select 30352,'AN' from dual union all
select 30353,'AN' from dual union all
select 30354,'MRN' from dual union all
select 30355,'AN' from dual union all
select 30356,'AN' from dual union all
select 30357,'AN' from dual union all
select 30358,'AN' from dual union all
select 30359,'AN' from dual union all
select 3036,'ORDRSA' from dual union all
select 30360,'AN' from dual union all
select 30361,'AN' from dual union all
select 30362,'AN' from dual union all
select 30363,'AN' from dual union all
select 30364,'AN' from dual union all
select 30365,'AN' from dual union all
select 30366,'AN' from dual union all
select 30367,'AN' from dual union all
select 30368,'AN' from dual union all
select 30369,'AN' from dual union all
select 3037,'USA' from dual union all
select 30370,'MRN' from dual union all
select 30371,'AN' from dual union all
select 30372,'AN' from dual union all
select 30373,'AN' from dual union all
select 30374,'AN' from dual union all
select 30375,'AN' from dual union all
select 30376,'AN' from dual union all
select 30377,'AN' from dual union all
select 30378,'AN' from dual union all
select 30379,'AN' from dual union all
select 3038,'PARD' from dual union all
select 30380,'AN' from dual union all
select 30381,'AN' from dual union all
select 30382,'MRN' from dual union all
select 30383,'MRN' from dual union all
select 30384,'AN' from dual union all
select 30385,'MRN' from dual union all
select 30386,'AN' from dual union all
select 30387,'AN' from dual union all
select 30388,'AN' from dual union all
select 30389,'AN' from dual union all
select 3039,'ORDRSA' from dual union all
select 30390,'AN' from dual union all
select 30391,'AN' from dual union all
select 30392,'MRN' from dual union all
select 30393,'AN' from dual union all
select 30394,'AN' from dual union all
select 30395,'AN' from dual union all
select 30396,'AN' from dual union all
select 30397,'AN' from dual union all
select 30398,'AN' from dual union all
select 30399,'MRN' from dual union all
select 304,'WER271-1' from dual union all
select 3040,'USA' from dual union all
select 30400,'AN' from dual union all
select 30401,'AN' from dual union all
select 30402,'AN' from dual;
insert into LAUNCH_AGENCY
select 30403,'AN' from dual union all
select 30404,'AN' from dual union all
select 30405,'AN' from dual union all
select 30406,'AN' from dual union all
select 30407,'AN' from dual union all
select 30408,'AN' from dual union all
select 30409,'AN' from dual union all
select 3041,'USA' from dual union all
select 30410,'MRN' from dual union all
select 30411,'AN' from dual union all
select 30412,'AN' from dual union all
select 30413,'MRN' from dual union all
select 30414,'AN' from dual union all
select 30415,'AN' from dual union all
select 30416,'AN' from dual union all
select 30417,'AN' from dual union all
select 30418,'AN' from dual union all
select 30419,'AN' from dual union all
select 3042,'USA' from dual union all
select 30420,'AN' from dual union all
select 30421,'AN' from dual union all
select 30422,'AN' from dual union all
select 30423,'AN' from dual union all
select 30424,'AN' from dual union all
select 30425,'AN' from dual union all
select 30426,'AN' from dual union all
select 30427,'AN' from dual union all
select 30428,'AN' from dual union all
select 30429,'MRN' from dual union all
select 3043,'USA' from dual union all
select 30430,'AN' from dual union all
select 30431,'AN' from dual union all
select 30432,'AN' from dual union all
select 30433,'AN' from dual union all
select 30434,'AN' from dual union all
select 30435,'AN' from dual union all
select 30436,'AN' from dual union all
select 30437,'AN' from dual union all
select 30438,'AN' from dual union all
select 30439,'MRN' from dual union all
select 3044,'USA' from dual union all
select 30440,'MRN' from dual union all
select 30441,'AN' from dual union all
select 30442,'AN' from dual union all
select 30443,'AN' from dual union all
select 30444,'AN' from dual union all
select 30445,'AN' from dual union all
select 30446,'AN' from dual union all
select 30447,'AN' from dual union all
select 30448,'AN' from dual union all
select 30449,'AN' from dual union all
select 3045,'ORDRSA' from dual union all
select 30450,'MRN' from dual union all
select 30451,'AN' from dual union all
select 30452,'AN' from dual union all
select 30453,'AN' from dual union all
select 30454,'AN' from dual union all
select 30455,'AN' from dual union all
select 30456,'MRN' from dual union all
select 30457,'AN' from dual union all
select 30458,'AN' from dual union all
select 30459,'AN' from dual union all
select 3046,'ORDRSA' from dual union all
select 30460,'AN' from dual union all
select 30461,'MRN' from dual union all
select 30462,'MRN' from dual union all
select 30463,'MRN' from dual union all
select 30464,'AN' from dual union all
select 30465,'SRC' from dual union all
select 30466,'AN' from dual union all
select 30467,'AN' from dual union all
select 30468,'AN' from dual union all
select 30469,'AN' from dual union all
select 3047,'USA' from dual union all
select 30470,'AN' from dual union all
select 30471,'AN' from dual union all
select 30472,'MRN' from dual union all
select 30473,'MRN' from dual union all
select 30474,'AN' from dual union all
select 30475,'AN' from dual union all
select 30476,'AN' from dual union all
select 30477,'AN' from dual union all
select 30478,'AN' from dual union all
select 30479,'AN' from dual union all
select 3048,'USA' from dual union all
select 30480,'AN' from dual union all
select 30481,'AN' from dual union all
select 30482,'MRN' from dual union all
select 30483,'MRN' from dual union all
select 30484,'AN' from dual union all
select 30485,'AN' from dual union all
select 30486,'AN' from dual union all
select 30487,'AN' from dual union all
select 30488,'AN' from dual union all
select 30489,'MRN' from dual union all
select 3049,'ORDRSA' from dual union all
select 30490,'AN' from dual union all
select 30491,'AN' from dual union all
select 30492,'AN' from dual union all
select 30493,'AN' from dual;
insert into LAUNCH_AGENCY
select 30494,'AN' from dual union all
select 30495,'AN' from dual union all
select 30496,'AN' from dual union all
select 30497,'AN' from dual union all
select 30498,'AN' from dual union all
select 30499,'MRN' from dual union all
select 305,'WER271-1' from dual union all
select 3050,'ORDRSA' from dual union all
select 30500,'AN' from dual union all
select 30501,'MRN' from dual union all
select 30502,'NASA' from dual union all
select 30503,'AN' from dual union all
select 30504,'AN' from dual union all
select 30505,'AN' from dual union all
select 30506,'AN' from dual union all
select 30507,'AN' from dual union all
select 30508,'AN' from dual union all
select 30509,'AN' from dual union all
select 3051,'ORDRSA' from dual union all
select 30510,'AN' from dual union all
select 30511,'AN' from dual union all
select 30512,'AN' from dual union all
select 30513,'AN' from dual union all
select 30514,'AN' from dual union all
select 30515,'MRN' from dual union all
select 30516,'AN' from dual union all
select 30517,'AN' from dual union all
select 30518,'AN' from dual union all
select 30519,'AN' from dual union all
select 3052,'USAF' from dual union all
select 30520,'AN' from dual union all
select 30521,'AN' from dual union all
select 30522,'AN' from dual union all
select 30523,'MRN' from dual union all
select 30524,'MRN' from dual union all
select 30525,'AN' from dual union all
select 30526,'AN' from dual union all
select 30527,'AN' from dual union all
select 30528,'AN' from dual union all
select 30529,'AN' from dual union all
select 3053,'ORDRSA' from dual union all
select 30530,'AN' from dual union all
select 30531,'AN' from dual union all
select 30532,'AN' from dual union all
select 30533,'AN' from dual union all
select 30534,'AN' from dual union all
select 30535,'AN' from dual union all
select 30536,'AN' from dual union all
select 30537,'AN' from dual union all
select 30538,'AN' from dual union all
select 30539,'MRN' from dual union all
select 3054,'USA' from dual union all
select 30540,'AN' from dual union all
select 30541,'AN' from dual union all
select 30542,'AN' from dual union all
select 30543,'AN' from dual union all
select 30544,'MRN' from dual union all
select 30545,'AN' from dual union all
select 30546,'AN' from dual union all
select 30547,'NASA' from dual union all
select 30548,'AN' from dual union all
select 30549,'AN' from dual union all
select 3055,'USA' from dual union all
select 30550,'AN' from dual union all
select 30551,'AN' from dual union all
select 30552,'AN' from dual union all
select 30553,'AN' from dual union all
select 30554,'AN' from dual union all
select 30555,'AN' from dual union all
select 30556,'AN' from dual union all
select 30557,'AN' from dual union all
select 30558,'AN' from dual union all
select 30559,'AN' from dual union all
select 3056,'USA' from dual union all
select 30560,'MRN' from dual union all
select 30561,'MRN' from dual union all
select 30562,'MRN' from dual union all
select 30563,'AN' from dual union all
select 30564,'AN' from dual union all
select 30565,'MRN' from dual union all
select 30566,'MRN' from dual union all
select 30567,'MRN' from dual union all
select 30568,'AN' from dual union all
select 30569,'AN' from dual union all
select 3057,'USA' from dual union all
select 30570,'AN' from dual union all
select 30571,'AN' from dual union all
select 30572,'MRN' from dual union all
select 30573,'AN' from dual union all
select 30574,'AN' from dual union all
select 30575,'AN' from dual union all
select 30576,'MRN' from dual union all
select 30577,'AN' from dual union all
select 30578,'MRN' from dual union all
select 30579,'MRN' from dual union all
select 3058,'USA' from dual union all
select 30580,'AN' from dual union all
select 30581,'AN' from dual union all
select 30582,'AN' from dual union all
select 30583,'AN' from dual;
insert into LAUNCH_AGENCY
select 30584,'AN' from dual union all
select 30585,'AN' from dual union all
select 30586,'AN' from dual union all
select 30587,'AN' from dual union all
select 30588,'AN' from dual union all
select 30589,'MRN' from dual union all
select 3059,'USA' from dual union all
select 30590,'MRN' from dual union all
select 30591,'AN' from dual union all
select 30592,'AN' from dual union all
select 30593,'AN' from dual union all
select 30594,'AN' from dual union all
select 30595,'AN' from dual union all
select 30596,'AN' from dual union all
select 30597,'AN' from dual union all
select 30598,'AN' from dual union all
select 30599,'AN' from dual union all
select 306,'WER271-1' from dual union all
select 3060,'USA' from dual union all
select 30600,'MRN' from dual union all
select 30601,'MRN' from dual union all
select 30602,'AN' from dual union all
select 30603,'RVSN' from dual union all
select 30604,'RVSN' from dual union all
select 30605,'RVSN' from dual union all
select 30606,'RVSN' from dual union all
select 30607,'RVSN' from dual union all
select 30608,'RVSN' from dual union all
select 30609,'RVSN' from dual union all
select 3061,'PARD' from dual union all
select 30610,'RVSN' from dual union all
select 30611,'RVSN' from dual union all
select 30612,'RVSN' from dual union all
select 30613,'RVSN' from dual union all
select 30614,'RVSN' from dual union all
select 30615,'USAF' from dual union all
select 30616,'USAF' from dual union all
select 30617,'RVSN' from dual union all
select 30618,'RVSN' from dual union all
select 30619,'RVSN' from dual union all
select 3062,'ORDRSA' from dual union all
select 30620,'RVSN' from dual union all
select 30621,'NASA' from dual union all
select 30622,'RVSN' from dual union all
select 30623,'RVSN' from dual union all
select 30624,'RVSN' from dual union all
select 30625,'NASA' from dual union all
select 30626,'RVSN' from dual union all
select 30627,'RVSN' from dual union all
select 30628,'RVSN' from dual union all
select 30629,'NASA' from dual union all
select 3063,'USAF' from dual union all
select 30630,'USAF' from dual union all
select 30631,'RVSN' from dual union all
select 30632,'RVSN' from dual union all
select 30633,'RVSN' from dual union all
select 30634,'NASA' from dual union all
select 30635,'RVSN' from dual union all
select 30636,'RVSN' from dual union all
select 30637,'RVSN' from dual union all
select 30638,'RVSN' from dual union all
select 30639,'RVSN' from dual union all
select 3064,'USA' from dual union all
select 30640,'RVSN' from dual union all
select 30641,'NASA' from dual union all
select 30642,'USAF' from dual union all
select 30643,'RVSN' from dual union all
select 30644,'RVSN' from dual union all
select 30645,'RVSN' from dual union all
select 30646,'RVSN' from dual union all
select 30647,'RVSN' from dual union all
select 30648,'USAF' from dual union all
select 30649,'RVSN' from dual union all
select 3065,'USA' from dual union all
select 30650,'RVSN' from dual union all
select 30651,'RVSN' from dual union all
select 30652,'NASA' from dual union all
select 30653,'RVSN' from dual union all
select 30654,'RVSN' from dual union all
select 30655,'RVSN' from dual union all
select 30656,'10ADS' from dual union all
select 30657,'RVSN' from dual union all
select 30658,'USAF' from dual union all
select 30659,'RVSN' from dual union all
select 3066,'USA' from dual union all
select 30660,'NASA' from dual union all
select 30661,'RVSN' from dual union all
select 30662,'RVSN' from dual union all
select 30663,'RVSN' from dual union all
select 30664,'RVSN' from dual union all
select 30665,'RVSN' from dual union all
select 30666,'RVSN' from dual union all
select 30667,'RVSN' from dual union all
select 30668,'RVSN' from dual union all
select 30669,'RVSN' from dual union all
select 3067,'ORDRSA' from dual union all
select 30670,'USAF' from dual union all
select 30671,'RVSN' from dual union all
select 30672,'RVSN' from dual union all
select 30673,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 30674,'RVSN' from dual union all
select 30675,'RVSN' from dual union all
select 30676,'RVSN' from dual union all
select 30677,'RVSN' from dual union all
select 30678,'RVSN' from dual union all
select 30679,'RVSN' from dual union all
select 3068,'USA' from dual union all
select 30680,'NASA' from dual union all
select 30681,'RVSN' from dual union all
select 30682,'RVSN' from dual union all
select 30683,'AFSC' from dual union all
select 30684,'RVSN' from dual union all
select 30685,'RVSN' from dual union all
select 30686,'RVSN' from dual union all
select 30687,'NASA' from dual union all
select 30688,'NASA' from dual union all
select 30689,'RVSN' from dual union all
select 3069,'USA' from dual union all
select 30690,'USAF' from dual union all
select 30691,'RVSN' from dual union all
select 30692,'NASA' from dual union all
select 30693,'RVSN' from dual union all
select 30694,'RVSN' from dual union all
select 30695,'RVSN' from dual union all
select 30696,'RVSN' from dual union all
select 30697,'RVSN' from dual union all
select 30698,'RVSN' from dual union all
select 30699,'RVSN' from dual union all
select 307,'WEHR' from dual union all
select 3070,'ORDRSA' from dual union all
select 30700,'RVSN' from dual union all
select 30701,'RVSN' from dual union all
select 30702,'USAF' from dual union all
select 30703,'NASA' from dual union all
select 30704,'RVSN' from dual union all
select 30705,'RVSN' from dual union all
select 30706,'RVSN' from dual union all
select 30707,'RVSN' from dual union all
select 30708,'RVSN' from dual union all
select 30709,'RVSN' from dual union all
select 3071,'ORDRSA' from dual union all
select 30710,'RVSN' from dual union all
select 30711,'RVSN' from dual union all
select 30712,'AN' from dual union all
select 30713,'MRN' from dual union all
select 30714,'PRO' from dual union all
select 30715,'MRN' from dual union all
select 30716,'MRN' from dual union all
select 30717,'USA' from dual union all
select 30718,'USA' from dual union all
select 30719,'USA' from dual union all
select 3072,'USAF' from dual union all
select 30720,'AN' from dual union all
select 30721,'MRN' from dual union all
select 30722,'MRN' from dual union all
select 30723,'41FA1' from dual union all
select 30724,'MRN' from dual union all
select 30725,'MRN' from dual union all
select 30726,'MRN' from dual union all
select 30727,'MRN' from dual union all
select 30728,'MRN' from dual union all
select 30729,'RVSN' from dual union all
select 3073,'ORDRSA' from dual union all
select 30730,'MRN' from dual union all
select 30731,'MRN' from dual union all
select 30732,'RVSN' from dual union all
select 30733,'MRN' from dual union all
select 30734,'RVSN' from dual union all
select 30735,'USA' from dual union all
select 30736,'PIHM' from dual union all
select 30737,'PIHM' from dual union all
select 30738,'PIHM' from dual union all
select 30739,'PIHM' from dual union all
select 3074,'USA' from dual union all
select 30740,'PIHM' from dual union all
select 30741,'MRN' from dual union all
select 30742,'MRN' from dual union all
select 30743,'USA' from dual union all
select 30744,'MRN' from dual union all
select 30745,'RVSN' from dual union all
select 30746,'MRN' from dual union all
select 30747,'MRN' from dual union all
select 30748,'NASA' from dual union all
select 30749,'MRN' from dual union all
select 3075,'ORDRSA' from dual union all
select 30750,'SAND' from dual union all
select 30751,'MRN' from dual union all
select 30752,'MRN' from dual union all
select 30753,'MRN' from dual union all
select 30754,'MRN' from dual union all
select 30755,'MRN' from dual union all
select 30756,'USN' from dual union all
select 30757,'RVSN' from dual union all
select 30758,'MRN' from dual union all
select 30759,'PIHM' from dual union all
select 3076,'USA' from dual union all
select 30760,'PIHM' from dual union all
select 30761,'PIHM' from dual union all
select 30762,'MRN' from dual union all
select 30763,'PIHM' from dual;
insert into LAUNCH_AGENCY
select 30764,'NASA' from dual union all
select 30765,'MRN' from dual union all
select 30766,'MRN' from dual union all
select 30767,'MRN' from dual union all
select 30768,'MRN' from dual union all
select 30769,'USA' from dual union all
select 3077,'USA' from dual union all
select 30770,'RVSN' from dual union all
select 30771,'RVSN' from dual union all
select 30772,'NASA' from dual union all
select 30773,'AFCRL' from dual union all
select 30774,'USA' from dual union all
select 30775,'SAND' from dual union all
select 30776,'MRN' from dual union all
select 30777,'MRN' from dual union all
select 30778,'MRN' from dual union all
select 30779,'MRN' from dual union all
select 3078,'USA' from dual union all
select 30780,'MRN' from dual union all
select 30781,'USA' from dual union all
select 30782,'MRN' from dual union all
select 30783,'MRN' from dual union all
select 30784,'MRN' from dual union all
select 30785,'MRN' from dual union all
select 30786,'USN' from dual union all
select 30787,'MRN' from dual union all
select 30788,'AFCRL' from dual union all
select 30789,'AFCRL' from dual union all
select 3079,'USA' from dual union all
select 30790,'PRO' from dual union all
select 30791,'RVSN' from dual union all
select 30792,'EGAO' from dual union all
select 30793,'EGAO' from dual union all
select 30794,'MOD' from dual union all
select 30795,'RVSN' from dual union all
select 30796,'NASA' from dual union all
select 30797,'NASA' from dual union all
select 30798,'NASA' from dual union all
select 30799,'NASA' from dual union all
select 308,'WEHR' from dual union all
select 3080,'USA' from dual union all
select 30800,'NASA' from dual union all
select 30801,'NASA' from dual union all
select 30802,'RVSN' from dual union all
select 30803,'RVSN' from dual union all
select 30804,'USAF' from dual union all
select 30805,'RVSN' from dual union all
select 30806,'NASA' from dual union all
select 30807,'SAST' from dual union all
select 30808,'DLR' from dual union all
select 30808,'INTA' from dual union all
select 30809,'I' from dual union all
select 3081,'ORDRSA' from dual union all
select 30810,'DLR' from dual union all
select 30810,'INTA' from dual union all
select 30811,'SUPA' from dual union all
select 30812,'AFCRL' from dual union all
select 30813,'DLR' from dual union all
select 30813,'INTA' from dual union all
select 30814,'AFCRL' from dual union all
select 30815,'SUPA' from dual union all
select 30816,'USA' from dual union all
select 30817,'USA' from dual union all
select 30818,'NASA' from dual union all
select 30819,'AFCRL' from dual union all
select 3082,'ORDRSA' from dual union all
select 30820,'SRC' from dual union all
select 30821,'USA' from dual union all
select 30822,'CNES' from dual union all
select 30823,'AFCRL' from dual union all
select 30824,'AFCRL' from dual union all
select 30825,'AFCRL' from dual union all
select 30826,'AFCRL' from dual union all
select 30827,'AFCRL' from dual union all
select 30828,'AFCRL' from dual union all
select 30829,'AFCRL' from dual union all
select 3083,'USA' from dual union all
select 30830,'AFCRL' from dual union all
select 30831,'MRN' from dual union all
select 30832,'CNES' from dual union all
select 30833,'CNES' from dual union all
select 30834,'CNES' from dual union all
select 30835,'CNES' from dual union all
select 30836,'CNES' from dual union all
select 30837,'NASA' from dual union all
select 30838,'CNES' from dual union all
select 30839,'CNES' from dual union all
select 3084,'USA' from dual union all
select 30840,'CNES' from dual union all
select 30841,'CNES' from dual union all
select 30842,'CNES' from dual union all
select 30843,'CNES' from dual union all
select 30844,'CNES' from dual union all
select 30845,'INTA' from dual union all
select 30846,'INTA' from dual union all
select 30847,'INTA' from dual union all
select 30848,'INTA' from dual union all
select 30849,'MRN' from dual union all
select 3085,'USA' from dual union all
select 30850,'NASA' from dual;
insert into LAUNCH_AGENCY
select 30851,'NASA' from dual union all
select 30852,'NASA' from dual union all
select 30853,'NASA' from dual union all
select 30854,'NASA' from dual union all
select 30855,'NASA' from dual union all
select 30856,'NASA' from dual union all
select 30857,'NASA' from dual union all
select 30858,'DMA' from dual union all
select 30859,'DMA' from dual union all
select 3086,'USA' from dual union all
select 30860,'RVSN' from dual union all
select 30861,'RVSN' from dual union all
select 30862,'RVSN' from dual union all
select 30863,'RVSN' from dual union all
select 30864,'RVSN' from dual union all
select 30865,'RVSN' from dual union all
select 30866,'USAF' from dual union all
select 30867,'RVSN' from dual union all
select 30868,'AFSC' from dual union all
select 30869,'RVSN' from dual union all
select 3087,'ORDRSA' from dual union all
select 30870,'RVSN' from dual union all
select 30871,'BAC' from dual union all
select 30872,'SUPA' from dual union all
select 30873,'DMA' from dual union all
select 30874,'NASA' from dual union all
select 30875,'84FA3' from dual union all
select 30876,'84FA3' from dual union all
select 30877,'84FA3' from dual union all
select 30878,'DLR' from dual union all
select 30878,'INTA' from dual union all
select 30879,'84FA3' from dual union all
select 3088,'USA' from dual union all
select 30880,'SRC' from dual union all
select 30881,'BAC' from dual union all
select 30882,'NASA' from dual union all
select 30883,'NASA' from dual union all
select 30884,'AFCRL' from dual union all
select 30885,'BAC' from dual union all
select 30886,'DLR' from dual union all
select 30886,'INTA' from dual union all
select 30887,'NASA' from dual union all
select 30888,'AFCRL' from dual union all
select 30889,'USAF' from dual union all
select 3089,'USA' from dual union all
select 30890,'NIPR' from dual union all
select 30891,'AFCRL' from dual union all
select 30892,'AFCRL' from dual union all
select 30893,'NASA' from dual union all
select 30894,'BAC' from dual union all
select 30895,'DLR' from dual union all
select 30895,'INTA' from dual union all
select 30896,'USAF' from dual union all
select 30897,'USAF' from dual union all
select 30898,'NASA' from dual union all
select 30899,'NASA' from dual union all
select 309,'WER271-1' from dual union all
select 3090,'USA' from dual union all
select 30900,'USAF' from dual union all
select 30901,'USN' from dual union all
select 30902,'USN' from dual union all
select 30903,'SRC' from dual union all
select 30904,'NASA' from dual union all
select 30905,'SRC' from dual union all
select 30906,'SRC' from dual union all
select 30907,'SRC' from dual union all
select 30908,'SRC' from dual union all
select 30909,'DLR' from dual union all
select 30909,'INTA' from dual union all
select 3091,'USA' from dual union all
select 30910,'SRC' from dual union all
select 30911,'KPNO' from dual union all
select 30912,'SRC' from dual union all
select 30913,'NASA' from dual union all
select 30914,'SRC' from dual union all
select 30915,'RVSN' from dual union all
select 30916,'RVSN' from dual union all
select 30917,'DLR' from dual union all
select 30917,'INTA' from dual union all
select 30918,'RVSN' from dual union all
select 30919,'NASA' from dual union all
select 3092,'USA' from dual union all
select 30920,'ISRO' from dual union all
select 30921,'AFCRL' from dual union all
select 30922,'USN' from dual union all
select 30923,'AFCRL' from dual union all
select 30924,'ALCS' from dual union all
select 30924,'SAC' from dual union all
select 30925,'CNES' from dual union all
select 30926,'BAC' from dual union all
select 30927,'NIPR' from dual union all
select 30928,'AFSC' from dual union all
select 30929,'USAF' from dual union all
select 3093,'USA' from dual union all
select 30930,'ALCS' from dual union all
select 30930,'SAC' from dual union all
select 30931,'NMC' from dual union all
select 30932,'AFCRL' from dual union all
select 30933,'ISAS' from dual union all
select 30934,'USN' from dual;
insert into LAUNCH_AGENCY
select 30935,'RVSN' from dual union all
select 30936,'AN' from dual union all
select 30937,'RVSN' from dual union all
select 30938,'SAC' from dual union all
select 30939,'AFSC' from dual union all
select 3094,'USA' from dual union all
select 30940,'USN' from dual union all
select 30941,'DFVLR' from dual union all
select 30942,'NRCC' from dual union all
select 30943,'NASA' from dual union all
select 30944,'NASA' from dual union all
select 30945,'SRC' from dual union all
select 30946,'USN' from dual union all
select 30947,'RVSN' from dual union all
select 30948,'NASA' from dual union all
select 30949,'NASA' from dual union all
select 3095,'USAF' from dual union all
select 30950,'AFCRL' from dual union all
select 30951,'AN' from dual union all
select 30952,'AN' from dual union all
select 30953,'AFSC' from dual union all
select 30954,'USA' from dual union all
select 30955,'NASA' from dual union all
select 30956,'USA' from dual union all
select 30957,'USA' from dual union all
select 30958,'USA' from dual union all
select 30959,'USA' from dual union all
select 3096,'USA' from dual union all
select 30960,'USA' from dual union all
select 30961,'USA' from dual union all
select 30962,'USA' from dual union all
select 30963,'USA' from dual union all
select 30964,'9FA3' from dual union all
select 30965,'NASA' from dual union all
select 30966,'US7A' from dual union all
select 30967,'RVSN' from dual union all
select 30968,'RVSN' from dual union all
select 30969,'NASA' from dual union all
select 3097,'USA' from dual union all
select 30970,'NASA' from dual union all
select 30971,'SAC' from dual union all
select 30972,'NIPR' from dual union all
select 30973,'NIPR' from dual union all
select 30974,'NASA' from dual union all
select 30975,'AFCRL' from dual union all
select 30976,'ISAS' from dual union all
select 30977,'AFCRL' from dual union all
select 30978,'NASA' from dual union all
select 30979,'RVSN' from dual union all
select 3098,'USA' from dual union all
select 30980,'BAC' from dual union all
select 30981,'RVSN' from dual union all
select 30982,'NRCC' from dual union all
select 30983,'USA' from dual union all
select 30984,'USA' from dual union all
select 30985,'SAND' from dual union all
select 30986,'US7A' from dual union all
select 30987,'DLR' from dual union all
select 30987,'INTA' from dual union all
select 30988,'US7A' from dual union all
select 30989,'ALCS' from dual union all
select 30989,'SAC' from dual union all
select 3099,'ORDRSA' from dual union all
select 30990,'IZM' from dual union all
select 30991,'SRC' from dual union all
select 30992,'USN' from dual union all
select 30993,'FKG2' from dual union all
select 30994,'FKG2' from dual union all
select 30995,'RVSN' from dual union all
select 30996,'RVSN' from dual union all
select 30997,'AN' from dual union all
select 30998,'ISRO' from dual union all
select 30999,'KPNO' from dual union all
select 31,'WEHR' from dual union all
select 310,'WER271-1' from dual union all
select 3100,'ORDRSA' from dual union all
select 31000,'CRA' from dual union all
select 31001,'NASA' from dual union all
select 31002,'NASA' from dual union all
select 31003,'AN' from dual union all
select 31004,'NASA' from dual union all
select 31005,'SRC' from dual union all
select 31006,'RVSN' from dual union all
select 31007,'ISRO' from dual union all
select 31008,'DFVLR' from dual union all
select 31009,'DLR' from dual union all
select 31009,'INTA' from dual union all
select 3101,'ORDRSA' from dual union all
select 31010,'ALCS' from dual union all
select 31010,'SAC' from dual union all
select 31011,'RVSN' from dual union all
select 31012,'AN' from dual union all
select 31013,'NIPR' from dual union all
select 31014,'BAC' from dual union all
select 31015,'FKG1' from dual union all
select 31016,'FKG1' from dual union all
select 31017,'NASA' from dual union all
select 31018,'NASA' from dual union all
select 31019,'NASA' from dual union all
select 3102,'ORDRSA' from dual;
insert into LAUNCH_AGENCY
select 31020,'NASA' from dual union all
select 31021,'NASA' from dual union all
select 31022,'NASA' from dual union all
select 31023,'AN' from dual union all
select 31024,'AFSC' from dual union all
select 31025,'DMA' from dual union all
select 31026,'AN' from dual union all
select 31027,'SRC' from dual union all
select 31028,'RVSN' from dual union all
select 31029,'NASA' from dual union all
select 3103,'PARD' from dual union all
select 31030,'SAC' from dual union all
select 31031,'USN' from dual union all
select 31032,'NASA' from dual union all
select 31033,'RVSN' from dual union all
select 31034,'AN' from dual union all
select 31035,'AN' from dual union all
select 31036,'AN' from dual union all
select 31037,'AN' from dual union all
select 31038,'AN' from dual union all
select 31039,'AN' from dual union all
select 3104,'USA' from dual union all
select 31040,'AN' from dual union all
select 31041,'AN' from dual union all
select 31042,'USN' from dual union all
select 31043,'RVSN' from dual union all
select 31044,'RVSN' from dual union all
select 31045,'NASA' from dual union all
select 31046,'NASA' from dual union all
select 31047,'ALCS' from dual union all
select 31047,'SAC' from dual union all
select 31048,'NASA' from dual union all
select 31049,'NASA' from dual union all
select 3105,'USA' from dual union all
select 31050,'BAC' from dual union all
select 31051,'NASA' from dual union all
select 31052,'NASA' from dual union all
select 31053,'USN' from dual union all
select 31054,'NASA' from dual union all
select 31055,'AFSC' from dual union all
select 31056,'NASA' from dual union all
select 31057,'AFCRL' from dual union all
select 31058,'AFCRL' from dual union all
select 31059,'NASA' from dual union all
select 3106,'USA' from dual union all
select 31060,'WRE' from dual union all
select 31061,'RVSN' from dual union all
select 31062,'NASA' from dual union all
select 31063,'RVSN' from dual union all
select 31064,'USN' from dual union all
select 31065,'ISAS' from dual union all
select 31066,'ISAS' from dual union all
select 31067,'NIPR' from dual union all
select 31068,'AFSC' from dual union all
select 31069,'NASA' from dual union all
select 3107,'USA' from dual union all
select 31070,'USAF' from dual union all
select 31071,'ISAS' from dual union all
select 31072,'AFSC' from dual union all
select 31073,'NASA' from dual union all
select 31074,'USN' from dual union all
select 31075,'RVSN' from dual union all
select 31076,'RVSN' from dual union all
select 31077,'RVSN' from dual union all
select 31078,'RVSN' from dual union all
select 31079,'SRC' from dual union all
select 3108,'USA' from dual union all
select 31080,'SRC' from dual union all
select 31081,'NASA' from dual union all
select 31082,'SAC' from dual union all
select 31083,'SRC' from dual union all
select 31084,'AFSC' from dual union all
select 31085,'NASDA' from dual union all
select 31086,'DLR' from dual union all
select 31086,'INTA' from dual union all
select 31087,'SRC' from dual union all
select 31088,'SRC' from dual union all
select 31089,'AFCRL' from dual union all
select 3109,'USA' from dual union all
select 31090,'NASA' from dual union all
select 31091,'SAC' from dual union all
select 31092,'SRC' from dual union all
select 31093,'AN' from dual union all
select 31094,'USN' from dual union all
select 31095,'DMA' from dual union all
select 31096,'AFCRL' from dual union all
select 31097,'USA' from dual union all
select 31098,'AFCRL' from dual union all
select 31099,'WRE' from dual union all
select 311,'WER271-1' from dual union all
select 3110,'USA' from dual union all
select 31100,'NASA' from dual union all
select 31101,'ISAS' from dual union all
select 31102,'ALCS' from dual union all
select 31102,'SAC' from dual union all
select 31103,'9FA3' from dual union all
select 31104,'9FA3' from dual union all
select 31105,'AFCRL' from dual union all
select 31106,'AFCRL' from dual union all
select 31107,'AFCRL' from dual;
insert into LAUNCH_AGENCY
select 31108,'WRE' from dual union all
select 31109,'AN' from dual union all
select 3111,'USA' from dual union all
select 31110,'AFSC' from dual union all
select 31111,'WRE' from dual union all
select 31112,'USA' from dual union all
select 31113,'USN' from dual union all
select 31114,'USN' from dual union all
select 31115,'USN' from dual union all
select 31116,'DLR' from dual union all
select 31116,'INTA' from dual union all
select 31117,'FKG2' from dual union all
select 31118,'FKG2' from dual union all
select 31119,'FKG1' from dual union all
select 3112,'USA' from dual union all
select 31120,'FKG1' from dual union all
select 31121,'RVSN' from dual union all
select 31122,'RVSN' from dual union all
select 31123,'SAC' from dual union all
select 31124,'DFVLR' from dual union all
select 31125,'AFCRL' from dual union all
select 31126,'DLR' from dual union all
select 31126,'INTA' from dual union all
select 31127,'AFCRL' from dual union all
select 31128,'NASA' from dual union all
select 31129,'SAC' from dual union all
select 3113,'USA' from dual union all
select 31130,'DFVLR' from dual union all
select 31131,'RVSN' from dual union all
select 31132,'SRC' from dual union all
select 31133,'SRC' from dual union all
select 31134,'BAC' from dual union all
select 31135,'WRE' from dual union all
select 31136,'SRC' from dual union all
select 31137,'NASA' from dual union all
select 31138,'AN' from dual union all
select 31139,'NRL' from dual union all
select 3114,'USA' from dual union all
select 31140,'RVSN' from dual union all
select 31141,'RVSN' from dual union all
select 31142,'AFCRL' from dual union all
select 31143,'AFCRL' from dual union all
select 31144,'AFCRL' from dual union all
select 31145,'AFCRL' from dual union all
select 31146,'AFCRL' from dual union all
select 31147,'DLR' from dual union all
select 31147,'INTA' from dual union all
select 31148,'ISAS' from dual union all
select 31149,'AFCRL' from dual union all
select 3115,'USA' from dual union all
select 31150,'AFCRL' from dual union all
select 31151,'AFCRL' from dual union all
select 31152,'WRE' from dual union all
select 31153,'AN' from dual union all
select 31154,'AFCRL' from dual union all
select 31155,'NASA' from dual union all
select 31156,'AFCRL' from dual union all
select 31157,'AN' from dual union all
select 31158,'BAC' from dual union all
select 31159,'NRCC' from dual union all
select 3116,'USA' from dual union all
select 31160,'SRC' from dual union all
select 31161,'SRC' from dual union all
select 31162,'US7A' from dual union all
select 31163,'US7A' from dual union all
select 31164,'US7A' from dual union all
select 31165,'US7A' from dual union all
select 31166,'RVSN' from dual union all
select 31167,'NASA' from dual union all
select 31168,'NASA' from dual union all
select 31169,'SAC' from dual union all
select 3117,'USA' from dual union all
select 31170,'DLR' from dual union all
select 31170,'INTA' from dual union all
select 31171,'AFCRL' from dual union all
select 31172,'RVSN' from dual union all
select 31173,'RVSN' from dual union all
select 31174,'NASA' from dual union all
select 31175,'NASA' from dual union all
select 31176,'NASA' from dual union all
select 31177,'NASA' from dual union all
select 31178,'NASA' from dual union all
select 31179,'BAC' from dual union all
select 3118,'USAF' from dual union all
select 31180,'SAC' from dual union all
select 31181,'SRC' from dual union all
select 31182,'SRC' from dual union all
select 31183,'SRC' from dual union all
select 31184,'SRC' from dual union all
select 31185,'SRC' from dual union all
select 31186,'NASA' from dual union all
select 31187,'IIAE' from dual union all
select 31188,'IIAE' from dual union all
select 31189,'RVSN' from dual union all
select 3119,'USA' from dual union all
select 31190,'WRE' from dual union all
select 31191,'BAC' from dual union all
select 31192,'ALCS' from dual union all
select 31192,'SAC' from dual union all
select 31193,'IIAE' from dual;
insert into LAUNCH_AGENCY
select 31194,'DFVLR' from dual union all
select 31195,'NASA' from dual union all
select 31196,'NASA' from dual union all
select 31197,'NRCC' from dual union all
select 31198,'BAC' from dual union all
select 31199,'DFVLR' from dual union all
select 312,'WER271-1' from dual union all
select 3120,'USA' from dual union all
select 31200,'NASA' from dual union all
select 31201,'USA' from dual union all
select 31202,'AN' from dual union all
select 31203,'AFSC' from dual union all
select 31204,'RVSN' from dual union all
select 31205,'RVSN' from dual union all
select 31206,'DLR' from dual union all
select 31206,'INTA' from dual union all
select 31207,'SRC' from dual union all
select 31208,'DLR' from dual union all
select 31208,'INTA' from dual union all
select 31209,'NRCC' from dual union all
select 3121,'USA' from dual union all
select 31210,'NASA' from dual union all
select 31211,'NRCC' from dual union all
select 31212,'USA' from dual union all
select 31213,'NRCC' from dual union all
select 31214,'NRCC' from dual union all
select 31215,'DLR' from dual union all
select 31215,'INTA' from dual union all
select 31216,'NRCC' from dual union all
select 31217,'BAC' from dual union all
select 31218,'US7A' from dual union all
select 31219,'US7A' from dual union all
select 3122,'USA' from dual union all
select 31220,'SRC' from dual union all
select 31221,'SRC' from dual union all
select 31222,'AFCRL' from dual union all
select 31223,'USA' from dual union all
select 31224,'AN' from dual union all
select 31225,'DLR' from dual union all
select 31225,'INTA' from dual union all
select 31226,'AN' from dual union all
select 31227,'NASA' from dual union all
select 31228,'AFCRL' from dual union all
select 31229,'AFSC' from dual union all
select 3123,'USA' from dual union all
select 31230,'DLR' from dual union all
select 31230,'INTA' from dual union all
select 31231,'USAF' from dual union all
select 31232,'DLR' from dual union all
select 31232,'INTA' from dual union all
select 31233,'AFSC' from dual union all
select 31234,'RVSN' from dual union all
select 31235,'DLR' from dual union all
select 31235,'INTA' from dual union all
select 31236,'DLR' from dual union all
select 31236,'INTA' from dual union all
select 31237,'RVSN' from dual union all
select 31238,'NASA' from dual union all
select 31239,'RVSN' from dual union all
select 3124,'USA' from dual union all
select 31240,'RVSN' from dual union all
select 31241,'AFSC' from dual union all
select 31242,'AN' from dual union all
select 31243,'RVSN' from dual union all
select 31244,'RVSN' from dual union all
select 31245,'RVSN' from dual union all
select 31246,'RVSN' from dual union all
select 31247,'AFCRL' from dual union all
select 31248,'AFCRL' from dual union all
select 31249,'AFCRL' from dual union all
select 3125,'USA' from dual union all
select 31250,'AN' from dual union all
select 31251,'AN' from dual union all
select 31252,'AN' from dual union all
select 31253,'AN' from dual union all
select 31254,'AN' from dual union all
select 31255,'AN' from dual union all
select 31256,'RVSN' from dual union all
select 31257,'AN' from dual union all
select 31258,'AN' from dual union all
select 31259,'RVSN' from dual union all
select 3126,'USA' from dual union all
select 31260,'RVSN' from dual union all
select 31261,'AFCRL' from dual union all
select 31262,'AFCRL' from dual union all
select 31263,'AFCRL' from dual union all
select 31264,'AFCRL' from dual union all
select 31265,'AFCRL' from dual union all
select 31266,'AFCRL' from dual union all
select 31267,'RVSN' from dual union all
select 31268,'AFCRL' from dual union all
select 31269,'AFCRL' from dual union all
select 3127,'USA' from dual union all
select 31270,'AFCRL' from dual union all
select 31271,'AFCRL' from dual union all
select 31272,'AFCRL' from dual union all
select 31273,'AFCRL' from dual union all
select 31274,'AFCRL' from dual union all
select 31275,'CALT' from dual union all
select 31276,'EGA' from dual;
insert into LAUNCH_AGENCY
select 31277,'EGA' from dual union all
select 31278,'RVSN' from dual union all
select 31279,'EGA' from dual union all
select 3128,'USA' from dual union all
select 31280,'WRE' from dual union all
select 31281,'RAE' from dual union all
select 31282,'RAE' from dual union all
select 31283,'ISAS' from dual union all
select 31284,'AN' from dual union all
select 31285,'RVSN' from dual union all
select 31286,'RVSN' from dual union all
select 31287,'RVSN' from dual union all
select 31288,'RVSN' from dual union all
select 31289,'RVSN' from dual union all
select 3129,'USA' from dual union all
select 31290,'RVSN' from dual union all
select 31291,'RVSN' from dual union all
select 31292,'RVSN' from dual union all
select 31293,'RVSN' from dual union all
select 31294,'RVSN' from dual union all
select 31295,'RVSN' from dual union all
select 31296,'RVSN' from dual union all
select 31297,'RVSN' from dual union all
select 31298,'RVSN' from dual union all
select 31299,'RVSN' from dual union all
select 313,'WEHR' from dual union all
select 3130,'USA' from dual union all
select 31300,'RVSN' from dual union all
select 31301,'RVSN' from dual union all
select 31302,'RVSN' from dual union all
select 31303,'RVSN' from dual union all
select 31304,'RVSN' from dual union all
select 31305,'RVSN' from dual union all
select 31306,'RVSN' from dual union all
select 31307,'RVSN' from dual union all
select 31308,'RVSN' from dual union all
select 31309,'RVSN' from dual union all
select 3131,'USA' from dual union all
select 31310,'RVSN' from dual union all
select 31311,'RVSN' from dual union all
select 31312,'RVSN' from dual union all
select 31313,'RVSN' from dual union all
select 31314,'RVSN' from dual union all
select 31315,'RVSN' from dual union all
select 31316,'RVSN' from dual union all
select 31317,'RVSN' from dual union all
select 31318,'RVSN' from dual union all
select 31319,'RVSN' from dual union all
select 3132,'ORDRSA' from dual union all
select 31320,'RVSN' from dual union all
select 31321,'RVSN' from dual union all
select 31322,'RVSN' from dual union all
select 31323,'RVSN' from dual union all
select 31324,'RVSN' from dual union all
select 31325,'RVSN' from dual union all
select 31326,'RVSN' from dual union all
select 31327,'RVSN' from dual union all
select 31328,'RVSN' from dual union all
select 31329,'RVSN' from dual union all
select 3133,'ORDRSA' from dual union all
select 31330,'RVSN' from dual union all
select 31331,'RVSN' from dual union all
select 31332,'RVSN' from dual union all
select 31333,'DFVLR' from dual union all
select 31334,'RVSN' from dual union all
select 31335,'RVSN' from dual union all
select 31336,'RVSN' from dual union all
select 31337,'RVSN' from dual union all
select 31338,'RVSN' from dual union all
select 31339,'RVSN' from dual union all
select 3134,'USA' from dual union all
select 31340,'RVSN' from dual union all
select 31341,'RVSN' from dual union all
select 31342,'RVSN' from dual union all
select 31343,'RVSN' from dual union all
select 31344,'NRCC' from dual union all
select 31345,'RVSN' from dual union all
select 31346,'RVSN' from dual union all
select 31347,'DLR' from dual union all
select 31347,'INTA' from dual union all
select 31348,'WRE' from dual union all
select 31349,'WRE' from dual union all
select 3135,'USA' from dual union all
select 31350,'WRE' from dual union all
select 31351,'WRE' from dual union all
select 31352,'WRE' from dual union all
select 31353,'WRE' from dual union all
select 31354,'RVSN' from dual union all
select 31355,'NRCC' from dual union all
select 31356,'RVSN' from dual union all
select 31357,'RVSN' from dual union all
select 31358,'RVSN' from dual union all
select 31359,'RVSN' from dual union all
select 3136,'USA' from dual union all
select 31360,'RVSN' from dual union all
select 31361,'RVSN' from dual union all
select 31362,'RVSN' from dual union all
select 31363,'RVSN' from dual union all
select 31364,'RVSN' from dual union all
select 31365,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 31366,'ISRO' from dual union all
select 31367,'RVSN' from dual union all
select 31368,'RVSN' from dual union all
select 31369,'RVSN' from dual union all
select 3137,'USA' from dual union all
select 31370,'RVSN' from dual union all
select 31371,'RVSN' from dual union all
select 31372,'RVSN' from dual union all
select 31373,'RVSN' from dual union all
select 31374,'RVSN' from dual union all
select 31375,'MVS' from dual union all
select 31376,'RVSN' from dual union all
select 31377,'DFVLR' from dual union all
select 31378,'RVSN' from dual union all
select 31379,'RVSN' from dual union all
select 3138,'ORDRSA' from dual union all
select 31380,'RVSN' from dual union all
select 31381,'USA' from dual union all
select 31382,'SAND' from dual union all
select 31383,'SAND' from dual union all
select 31384,'NASA' from dual union all
select 31385,'NASDA' from dual union all
select 31386,'DFVLR' from dual union all
select 31387,'SSC' from dual union all
select 31388,'NRCC' from dual union all
select 31389,'NASA' from dual union all
select 3139,'ORDRSA' from dual union all
select 31390,'RN' from dual union all
select 31391,'DMA' from dual union all
select 31392,'NIPR' from dual union all
select 31393,'DFVLR' from dual union all
select 31394,'CNES' from dual union all
select 31395,'ISAS' from dual union all
select 31396,'SRC' from dual union all
select 31397,'RVSN' from dual union all
select 31398,'84FA3' from dual union all
select 31399,'84FA3' from dual union all
select 314,'WEHR' from dual union all
select 3140,'ORDRSA' from dual union all
select 31400,'84FA3' from dual union all
select 31401,'84FA3' from dual union all
select 31402,'NRCC' from dual union all
select 31403,'SRC' from dual union all
select 31404,'ISAS' from dual union all
select 31405,'SRC' from dual union all
select 31406,'NRCC' from dual union all
select 31407,'NRCC' from dual union all
select 31408,'NASA' from dual union all
select 31409,'BAC' from dual union all
select 3141,'USAF' from dual union all
select 31410,'USN' from dual union all
select 31411,'USN' from dual union all
select 31412,'USN' from dual union all
select 31413,'USN' from dual union all
select 31414,'NASA' from dual union all
select 31415,'CNES' from dual union all
select 31415,'ISRO' from dual union all
select 31416,'ISRO' from dual union all
select 31417,'NASA' from dual union all
select 31418,'NASA' from dual union all
select 31419,'41FA1' from dual union all
select 3142,'USA' from dual union all
select 31420,'41FA1' from dual union all
select 31421,'41FA1' from dual union all
select 31422,'NASA' from dual union all
select 31423,'RVSN' from dual union all
select 31424,'CNES' from dual union all
select 31425,'MRN' from dual union all
select 31426,'MRN' from dual union all
select 31427,'MRN' from dual union all
select 31428,'MRN' from dual union all
select 31429,'MRN' from dual union all
select 3143,'USA' from dual union all
select 31430,'MRN' from dual union all
select 31431,'MRN' from dual union all
select 31432,'MRN' from dual union all
select 31433,'MRN' from dual union all
select 31434,'MRN' from dual union all
select 31435,'MRN' from dual union all
select 31436,'MRN' from dual union all
select 31437,'MRN' from dual union all
select 31438,'MRN' from dual union all
select 31439,'MRN' from dual union all
select 3144,'USA' from dual union all
select 31440,'MRN' from dual union all
select 31441,'MRN' from dual union all
select 31442,'MRN' from dual union all
select 31443,'MRN' from dual union all
select 31444,'MRN' from dual union all
select 31445,'MRN' from dual union all
select 31446,'MRN' from dual union all
select 31447,'MRN' from dual union all
select 31448,'MRN' from dual union all
select 31449,'MRN' from dual union all
select 3145,'PARD' from dual union all
select 31450,'MRN' from dual union all
select 31451,'MRN' from dual union all
select 31452,'MRN' from dual union all
select 31453,'MRN' from dual union all
select 31454,'MRN' from dual;
insert into LAUNCH_AGENCY
select 31455,'MRN' from dual union all
select 31456,'MRN' from dual union all
select 31457,'MRN' from dual union all
select 31458,'MRN' from dual union all
select 31459,'MRN' from dual union all
select 3146,'USAF' from dual union all
select 31460,'MRN' from dual union all
select 31461,'MRN' from dual union all
select 31462,'MRN' from dual union all
select 31463,'MRN' from dual union all
select 31464,'MRN' from dual union all
select 31465,'MRN' from dual union all
select 31466,'MRN' from dual union all
select 31467,'MRN' from dual union all
select 31468,'MRN' from dual union all
select 31469,'MRN' from dual union all
select 3147,'USA' from dual union all
select 31470,'MRN' from dual union all
select 31471,'MRN' from dual union all
select 31472,'MRN' from dual union all
select 31473,'MRN' from dual union all
select 31474,'MRN' from dual union all
select 31475,'NASA' from dual union all
select 31476,'MRN' from dual union all
select 31477,'MRN' from dual union all
select 31478,'NASA' from dual union all
select 31479,'MRN' from dual union all
select 3148,'USA' from dual union all
select 31480,'MRN' from dual union all
select 31481,'MRN' from dual union all
select 31482,'MRN' from dual union all
select 31483,'MRN' from dual union all
select 31484,'MRN' from dual union all
select 31485,'MRN' from dual union all
select 31486,'MRN' from dual union all
select 31487,'MRN' from dual union all
select 31488,'MRN' from dual union all
select 31489,'MRN' from dual union all
select 3149,'USA' from dual union all
select 31490,'MRN' from dual union all
select 31491,'MRN' from dual union all
select 31492,'MRN' from dual union all
select 31493,'MRN' from dual union all
select 31494,'MRN' from dual union all
select 31495,'MRN' from dual union all
select 31496,'MRN' from dual union all
select 31497,'MRN' from dual union all
select 31498,'MRN' from dual union all
select 31499,'MRN' from dual union all
select 315,'WEHR' from dual union all
select 3150,'USA' from dual union all
select 31500,'MRN' from dual union all
select 31501,'MRN' from dual union all
select 31502,'MRN' from dual union all
select 31503,'MRN' from dual union all
select 31504,'MRN' from dual union all
select 31505,'MRN' from dual union all
select 31506,'MRN' from dual union all
select 31507,'MRN' from dual union all
select 31508,'MRN' from dual union all
select 31509,'MRN' from dual union all
select 3151,'USA' from dual union all
select 31510,'MRN' from dual union all
select 31511,'MRN' from dual union all
select 31512,'MRN' from dual union all
select 31513,'MRN' from dual union all
select 31514,'MRN' from dual union all
select 31515,'MRN' from dual union all
select 31516,'MRN' from dual union all
select 31517,'MRN' from dual union all
select 31518,'MRN' from dual union all
select 31519,'MRN' from dual union all
select 3152,'USA' from dual union all
select 31520,'MRN' from dual union all
select 31521,'MRN' from dual union all
select 31522,'MRN' from dual union all
select 31523,'MRN' from dual union all
select 31524,'MRN' from dual union all
select 31525,'MRN' from dual union all
select 31526,'MRN' from dual union all
select 31527,'MRN' from dual union all
select 31528,'MRN' from dual union all
select 31529,'MRN' from dual union all
select 3153,'USA' from dual union all
select 31530,'MRN' from dual union all
select 31531,'MRN' from dual union all
select 31532,'MRN' from dual union all
select 31533,'MRN' from dual union all
select 31534,'MRN' from dual union all
select 31535,'MRN' from dual union all
select 31536,'MRN' from dual union all
select 31537,'MRN' from dual union all
select 31538,'MRN' from dual union all
select 31539,'MRN' from dual union all
select 3154,'USA' from dual union all
select 31540,'MRN' from dual union all
select 31541,'MRN' from dual union all
select 31542,'MRN' from dual union all
select 31543,'MRN' from dual union all
select 31544,'MRN' from dual;
insert into LAUNCH_AGENCY
select 31545,'MRN' from dual union all
select 31546,'MRN' from dual union all
select 31547,'MRN' from dual union all
select 31548,'MRN' from dual union all
select 31549,'MRN' from dual union all
select 3155,'USA' from dual union all
select 31550,'MRN' from dual union all
select 31551,'MRN' from dual union all
select 31552,'MRN' from dual union all
select 31553,'MRN' from dual union all
select 31554,'MRN' from dual union all
select 31555,'MRN' from dual union all
select 31556,'MRN' from dual union all
select 31557,'USA' from dual union all
select 31558,'MRN' from dual union all
select 31559,'MRN' from dual union all
select 3156,'USA' from dual union all
select 31560,'MRN' from dual union all
select 31561,'MRN' from dual union all
select 31562,'MRN' from dual union all
select 31563,'MRN' from dual union all
select 31564,'MRN' from dual union all
select 31565,'MRN' from dual union all
select 31566,'MRN' from dual union all
select 31567,'MRN' from dual union all
select 31568,'MRN' from dual union all
select 31569,'NASA' from dual union all
select 3157,'USAF' from dual union all
select 31570,'MRN' from dual union all
select 31571,'NASDA' from dual union all
select 31572,'NASDA' from dual union all
select 31573,'MRN' from dual union all
select 31574,'MRN' from dual union all
select 31575,'MRN' from dual union all
select 31576,'MRN' from dual union all
select 31577,'MRN' from dual union all
select 31578,'MRN' from dual union all
select 31579,'MRN' from dual union all
select 3158,'USA' from dual union all
select 31580,'MRN' from dual union all
select 31581,'MRN' from dual union all
select 31582,'MRN' from dual union all
select 31583,'MRN' from dual union all
select 31584,'MRN' from dual union all
select 31585,'MRN' from dual union all
select 31586,'MRN' from dual union all
select 31587,'MRN' from dual union all
select 31588,'MRN' from dual union all
select 31589,'MRN' from dual union all
select 3159,'USA' from dual union all
select 31590,'MRN' from dual union all
select 31591,'MRN' from dual union all
select 31592,'MRN' from dual union all
select 31593,'MRN' from dual union all
select 31594,'MRN' from dual union all
select 31595,'MRN' from dual union all
select 31596,'MRN' from dual union all
select 31597,'MRN' from dual union all
select 31598,'MRN' from dual union all
select 31599,'MRN' from dual union all
select 316,'WEHR' from dual union all
select 3160,'USA' from dual union all
select 31600,'MRN' from dual union all
select 31601,'MRN' from dual union all
select 31602,'MRN' from dual union all
select 31603,'MRN' from dual union all
select 31604,'MRN' from dual union all
select 31605,'MRN' from dual union all
select 31606,'MRN' from dual union all
select 31607,'MRN' from dual union all
select 31608,'MRN' from dual union all
select 31609,'MRN' from dual union all
select 3161,'USA' from dual union all
select 31610,'MRN' from dual union all
select 31611,'MRN' from dual union all
select 31612,'MRN' from dual union all
select 31613,'MRN' from dual union all
select 31614,'MRN' from dual union all
select 31615,'MRN' from dual union all
select 31616,'MRN' from dual union all
select 31617,'MRN' from dual union all
select 31618,'MRN' from dual union all
select 31619,'MRN' from dual union all
select 3162,'USAF' from dual union all
select 31620,'MRN' from dual union all
select 31621,'MRN' from dual union all
select 31622,'MRN' from dual union all
select 31623,'MRN' from dual union all
select 31624,'MRN' from dual union all
select 31625,'MRN' from dual union all
select 31626,'MRN' from dual union all
select 31627,'MRN' from dual union all
select 31628,'MRN' from dual union all
select 31629,'MRN' from dual union all
select 3163,'USA' from dual union all
select 31630,'MRN' from dual union all
select 31631,'MRN' from dual union all
select 31632,'MRN' from dual union all
select 31633,'MRN' from dual union all
select 31634,'MRN' from dual;
insert into LAUNCH_AGENCY
select 31635,'MRN' from dual union all
select 31636,'MRN' from dual union all
select 31637,'MRN' from dual union all
select 31638,'MRN' from dual union all
select 31639,'MRN' from dual union all
select 3164,'USA' from dual union all
select 31640,'MRN' from dual union all
select 31641,'MRN' from dual union all
select 31642,'MRN' from dual union all
select 31643,'MRN' from dual union all
select 31644,'MRN' from dual union all
select 31645,'MRN' from dual union all
select 31646,'MRN' from dual union all
select 31647,'MRN' from dual union all
select 31648,'MRN' from dual union all
select 31649,'MRN' from dual union all
select 3165,'USA' from dual union all
select 31650,'MRN' from dual union all
select 31651,'MRN' from dual union all
select 31652,'MRN' from dual union all
select 31653,'MRN' from dual union all
select 31654,'MRN' from dual union all
select 31655,'MRN' from dual union all
select 31656,'MRN' from dual union all
select 31657,'MRN' from dual union all
select 31658,'MRN' from dual union all
select 31659,'MRN' from dual union all
select 3166,'USA' from dual union all
select 31660,'MRN' from dual union all
select 31661,'MRN' from dual union all
select 31662,'MRN' from dual union all
select 31663,'MRN' from dual union all
select 31664,'MRN' from dual union all
select 31665,'MRN' from dual union all
select 31666,'MRN' from dual union all
select 31667,'MRN' from dual union all
select 31668,'MRN' from dual union all
select 31669,'MRN' from dual union all
select 3167,'USA' from dual union all
select 31670,'MRN' from dual union all
select 31671,'MRN' from dual union all
select 31672,'MRN' from dual union all
select 31673,'MRN' from dual union all
select 31674,'MRN' from dual union all
select 31675,'MRN' from dual union all
select 31676,'MRN' from dual union all
select 31677,'MRN' from dual union all
select 31678,'MRN' from dual union all
select 31679,'MRN' from dual union all
select 3168,'USAF' from dual union all
select 31680,'MRN' from dual union all
select 31681,'MRN' from dual union all
select 31682,'MRN' from dual union all
select 31683,'MRN' from dual union all
select 31684,'MRN' from dual union all
select 31685,'MRN' from dual union all
select 31686,'MRN' from dual union all
select 31687,'MRN' from dual union all
select 31688,'MRN' from dual union all
select 31689,'MRN' from dual union all
select 3169,'USA' from dual union all
select 31690,'MRN' from dual union all
select 31691,'MRN' from dual union all
select 31692,'MRN' from dual union all
select 31693,'MRN' from dual union all
select 31694,'MRN' from dual union all
select 31695,'MRN' from dual union all
select 31696,'MRN' from dual union all
select 31697,'MRN' from dual union all
select 31698,'MRN' from dual union all
select 31699,'MRN' from dual union all
select 317,'WEHR' from dual union all
select 3170,'USA' from dual union all
select 31700,'MRN' from dual union all
select 31701,'MRN' from dual union all
select 31702,'MRN' from dual union all
select 31703,'MRN' from dual union all
select 31704,'MRN' from dual union all
select 31705,'MRN' from dual union all
select 31706,'MRN' from dual union all
select 31707,'MRN' from dual union all
select 31708,'MRN' from dual union all
select 31709,'MRN' from dual union all
select 3171,'USA' from dual union all
select 31710,'MRN' from dual union all
select 31711,'MRN' from dual union all
select 31712,'MRN' from dual union all
select 31713,'MRN' from dual union all
select 31714,'MRN' from dual union all
select 31715,'MRN' from dual union all
select 31716,'MRN' from dual union all
select 31717,'MRN' from dual union all
select 31718,'MRN' from dual union all
select 31719,'MRN' from dual union all
select 3172,'USA' from dual union all
select 31720,'MRN' from dual union all
select 31721,'MRN' from dual union all
select 31722,'MRN' from dual union all
select 31723,'MRN' from dual union all
select 31724,'MRN' from dual;
insert into LAUNCH_AGENCY
select 31725,'MRN' from dual union all
select 31726,'MRN' from dual union all
select 31727,'MRN' from dual union all
select 31728,'MRN' from dual union all
select 31729,'MRN' from dual union all
select 3173,'ORDRSA' from dual union all
select 31730,'MRN' from dual union all
select 31731,'MRN' from dual union all
select 31732,'MRN' from dual union all
select 31733,'MRN' from dual union all
select 31734,'MRN' from dual union all
select 31735,'MRN' from dual union all
select 31736,'MRN' from dual union all
select 31737,'MRN' from dual union all
select 31738,'MRN' from dual union all
select 31739,'MRN' from dual union all
select 3174,'ORDRSA' from dual union all
select 31740,'MRN' from dual union all
select 31741,'MRN' from dual union all
select 31742,'MRN' from dual union all
select 31743,'MRN' from dual union all
select 31744,'MRN' from dual union all
select 31745,'MRN' from dual union all
select 31746,'MRN' from dual union all
select 31747,'MRN' from dual union all
select 31748,'MRN' from dual union all
select 31749,'MRN' from dual union all
select 3175,'USA' from dual union all
select 31750,'MRN' from dual union all
select 31751,'MRN' from dual union all
select 31752,'MRN' from dual union all
select 31753,'MRN' from dual union all
select 31754,'MRN' from dual union all
select 31755,'MRN' from dual union all
select 31756,'MRN' from dual union all
select 31757,'MRN' from dual union all
select 31758,'MRN' from dual union all
select 31759,'MRN' from dual union all
select 3176,'USA' from dual union all
select 31760,'MRN' from dual union all
select 31761,'MRN' from dual union all
select 31762,'MRN' from dual union all
select 31763,'MRN' from dual union all
select 31764,'MRN' from dual union all
select 31765,'MRN' from dual union all
select 31766,'MRN' from dual union all
select 31767,'MRN' from dual union all
select 31768,'MRN' from dual union all
select 31769,'MRN' from dual union all
select 3177,'USA' from dual union all
select 31770,'MRN' from dual union all
select 31771,'MRN' from dual union all
select 31772,'MRN' from dual union all
select 31773,'MRN' from dual union all
select 31774,'MRN' from dual union all
select 31775,'MRN' from dual union all
select 31776,'MRN' from dual union all
select 31777,'MRN' from dual union all
select 31778,'MRN' from dual union all
select 31779,'MRN' from dual union all
select 3178,'USA' from dual union all
select 31780,'MRN' from dual union all
select 31781,'MRN' from dual union all
select 31782,'MRN' from dual union all
select 31783,'MRN' from dual union all
select 31784,'MRN' from dual union all
select 31785,'MRN' from dual union all
select 31786,'MRN' from dual union all
select 31787,'MRN' from dual union all
select 31788,'MRN' from dual union all
select 31789,'MRN' from dual union all
select 3179,'USA' from dual union all
select 31790,'MRN' from dual union all
select 31791,'MRN' from dual union all
select 31792,'MRN' from dual union all
select 31793,'MRN' from dual union all
select 31794,'MRN' from dual union all
select 31795,'MRN' from dual union all
select 31796,'MRN' from dual union all
select 31797,'MRN' from dual union all
select 31798,'MRN' from dual union all
select 31799,'MRN' from dual union all
select 318,'WEHR' from dual union all
select 3180,'USAF' from dual union all
select 31800,'MRN' from dual union all
select 31801,'MRN' from dual union all
select 31802,'MRN' from dual union all
select 31803,'MRN' from dual union all
select 31804,'MRN' from dual union all
select 31805,'MRN' from dual union all
select 31806,'MRN' from dual union all
select 31807,'MRN' from dual union all
select 31808,'MRN' from dual union all
select 31809,'MRN' from dual union all
select 3181,'USA' from dual union all
select 31810,'MRN' from dual union all
select 31811,'MRN' from dual union all
select 31812,'MRN' from dual union all
select 31813,'MRN' from dual union all
select 31814,'MRN' from dual;
insert into LAUNCH_AGENCY
select 31815,'MRN' from dual union all
select 31816,'MRN' from dual union all
select 31817,'MRN' from dual union all
select 31818,'MRN' from dual union all
select 31819,'MRN' from dual union all
select 3182,'USA' from dual union all
select 31820,'MRN' from dual union all
select 31821,'MRN' from dual union all
select 31822,'MRN' from dual union all
select 31823,'MRN' from dual union all
select 31824,'MRN' from dual union all
select 31825,'MRN' from dual union all
select 31826,'MRN' from dual union all
select 31827,'MRN' from dual union all
select 31828,'MRN' from dual union all
select 31829,'MRN' from dual union all
select 3183,'USA' from dual union all
select 31830,'MRN' from dual union all
select 31831,'MRN' from dual union all
select 31832,'MRN' from dual union all
select 31833,'MRN' from dual union all
select 31834,'MRN' from dual union all
select 31835,'MRN' from dual union all
select 31836,'MRN' from dual union all
select 31837,'MRN' from dual union all
select 31838,'MRN' from dual union all
select 31839,'MRN' from dual union all
select 3184,'USAF' from dual union all
select 31840,'MRN' from dual union all
select 31841,'MRN' from dual union all
select 31842,'MRN' from dual union all
select 31843,'MRN' from dual union all
select 31844,'MRN' from dual union all
select 31845,'MRN' from dual union all
select 31846,'MRN' from dual union all
select 31847,'MRN' from dual union all
select 31848,'MRN' from dual union all
select 31849,'MRN' from dual union all
select 3185,'USA' from dual union all
select 31850,'MRN' from dual union all
select 31851,'MRN' from dual union all
select 31852,'MRN' from dual union all
select 31853,'MRN' from dual union all
select 31854,'MRN' from dual union all
select 31855,'MRN' from dual union all
select 31856,'MRN' from dual union all
select 31857,'MRN' from dual union all
select 31858,'MRN' from dual union all
select 31859,'MRN' from dual union all
select 3186,'USA' from dual union all
select 31860,'MRN' from dual union all
select 31861,'MRN' from dual union all
select 31862,'MRN' from dual union all
select 31863,'MRN' from dual union all
select 31864,'MRN' from dual union all
select 31865,'MRN' from dual union all
select 31866,'MRN' from dual union all
select 31867,'MRN' from dual union all
select 31868,'MRN' from dual union all
select 31869,'MRN' from dual union all
select 3187,'USA' from dual union all
select 31870,'MRN' from dual union all
select 31871,'MRN' from dual union all
select 31872,'MRN' from dual union all
select 31873,'MRN' from dual union all
select 31874,'MRN' from dual union all
select 31875,'MRN' from dual union all
select 31876,'MRN' from dual union all
select 31877,'MRN' from dual union all
select 31878,'MRN' from dual union all
select 31879,'MRN' from dual union all
select 3188,'USA' from dual union all
select 31880,'MRN' from dual union all
select 31881,'MRN' from dual union all
select 31882,'MRN' from dual union all
select 31883,'MRN' from dual union all
select 31884,'MRN' from dual union all
select 31885,'MRN' from dual union all
select 31886,'MRN' from dual union all
select 31887,'MRN' from dual union all
select 31888,'MRN' from dual union all
select 31889,'MRN' from dual union all
select 3189,'USA' from dual union all
select 31890,'MRN' from dual union all
select 31891,'MRN' from dual union all
select 31892,'MRN' from dual union all
select 31893,'MRN' from dual union all
select 31894,'MRN' from dual union all
select 31895,'MRN' from dual union all
select 31896,'MRN' from dual union all
select 31897,'MRN' from dual union all
select 31898,'MRN' from dual union all
select 31899,'MRN' from dual union all
select 319,'WEHR' from dual union all
select 3190,'USA' from dual union all
select 31900,'MRN' from dual union all
select 31901,'MRN' from dual union all
select 31902,'MRN' from dual union all
select 31903,'MRN' from dual union all
select 31904,'MRN' from dual;
insert into LAUNCH_AGENCY
select 31905,'MRN' from dual union all
select 31906,'MRN' from dual union all
select 31907,'MRN' from dual union all
select 31908,'MRN' from dual union all
select 31909,'MRN' from dual union all
select 3191,'USA' from dual union all
select 31910,'MRN' from dual union all
select 31911,'MRN' from dual union all
select 31912,'MRN' from dual union all
select 31913,'MRN' from dual union all
select 31914,'MRN' from dual union all
select 31915,'MRN' from dual union all
select 31916,'MRN' from dual union all
select 31917,'MRN' from dual union all
select 31918,'MRN' from dual union all
select 31919,'MRN' from dual union all
select 3192,'USA' from dual union all
select 31920,'MRN' from dual union all
select 31921,'MRN' from dual union all
select 31922,'MRN' from dual union all
select 31923,'MRN' from dual union all
select 31924,'MRN' from dual union all
select 31925,'MRN' from dual union all
select 31926,'MRN' from dual union all
select 31927,'MRN' from dual union all
select 31928,'MRN' from dual union all
select 31929,'MRN' from dual union all
select 3193,'ORDRSA' from dual union all
select 31930,'MRN' from dual union all
select 31931,'MRN' from dual union all
select 31932,'MRN' from dual union all
select 31933,'MRN' from dual union all
select 31934,'MRN' from dual union all
select 31935,'MRN' from dual union all
select 31936,'MRN' from dual union all
select 31937,'MRN' from dual union all
select 31938,'MRN' from dual union all
select 31939,'MRN' from dual union all
select 3194,'ORDRSA' from dual union all
select 31940,'MRN' from dual union all
select 31941,'MRN' from dual union all
select 31942,'MRN' from dual union all
select 31943,'MRN' from dual union all
select 31944,'MRN' from dual union all
select 31945,'MRN' from dual union all
select 31946,'NASA' from dual union all
select 31947,'MRN' from dual union all
select 31948,'MRN' from dual union all
select 31949,'MRN' from dual union all
select 3195,'ORDRSA' from dual union all
select 31950,'MRN' from dual union all
select 31951,'MRN' from dual union all
select 31952,'MRN' from dual union all
select 31953,'MRN' from dual union all
select 31954,'MRN' from dual union all
select 31955,'MRN' from dual union all
select 31956,'MRN' from dual union all
select 31957,'MRN' from dual union all
select 31958,'MRN' from dual union all
select 31959,'MRN' from dual union all
select 3196,'USAF' from dual union all
select 31960,'MRN' from dual union all
select 31961,'MRN' from dual union all
select 31962,'MRN' from dual union all
select 31963,'MRN' from dual union all
select 31964,'MRN' from dual union all
select 31965,'MRN' from dual union all
select 31966,'MRN' from dual union all
select 31967,'MRN' from dual union all
select 31968,'MRN' from dual union all
select 31969,'MRN' from dual union all
select 3197,'USA' from dual union all
select 31970,'MRN' from dual union all
select 31971,'MRN' from dual union all
select 31972,'MRN' from dual union all
select 31973,'MRN' from dual union all
select 31974,'MRN' from dual union all
select 31975,'MRN' from dual union all
select 31976,'MRN' from dual union all
select 31977,'MRN' from dual union all
select 31978,'MRN' from dual union all
select 31979,'MRN' from dual union all
select 3198,'USA' from dual union all
select 31980,'MRN' from dual union all
select 31981,'MRN' from dual union all
select 31982,'MRN' from dual union all
select 31983,'MRN' from dual union all
select 31984,'MRN' from dual union all
select 31985,'MRN' from dual union all
select 31986,'MRN' from dual union all
select 31987,'MRN' from dual union all
select 31988,'MRN' from dual union all
select 31989,'MRN' from dual union all
select 3199,'USA' from dual union all
select 31990,'MRN' from dual union all
select 31991,'MRN' from dual union all
select 31992,'MRN' from dual union all
select 31993,'MRN' from dual union all
select 31994,'MRN' from dual union all
select 31995,'MRN' from dual;
insert into LAUNCH_AGENCY
select 31996,'MRN' from dual union all
select 31997,'MRN' from dual union all
select 31998,'MRN' from dual union all
select 31999,'MRN' from dual union all
select 32,'WEHR' from dual union all
select 320,'WEHR' from dual union all
select 3200,'PARD' from dual union all
select 32000,'MRN' from dual union all
select 32001,'MRN' from dual union all
select 32002,'MRN' from dual union all
select 32003,'MRN' from dual union all
select 32004,'MRN' from dual union all
select 32005,'MRN' from dual union all
select 32006,'MRN' from dual union all
select 32007,'MRN' from dual union all
select 32008,'MRN' from dual union all
select 32009,'MRN' from dual union all
select 3201,'USA' from dual union all
select 32010,'MRN' from dual union all
select 32011,'MRN' from dual union all
select 32012,'MRN' from dual union all
select 32013,'MRN' from dual union all
select 32014,'MRN' from dual union all
select 32015,'MRN' from dual union all
select 32016,'MRN' from dual union all
select 32017,'MRN' from dual union all
select 32018,'MRN' from dual union all
select 32019,'MRN' from dual union all
select 3202,'USA' from dual union all
select 32020,'MRN' from dual union all
select 32021,'MRN' from dual union all
select 32022,'MRN' from dual union all
select 32023,'MRN' from dual union all
select 32024,'MRN' from dual union all
select 32025,'MRN' from dual union all
select 32026,'MRN' from dual union all
select 32027,'MRN' from dual union all
select 32028,'MRN' from dual union all
select 32029,'MRN' from dual union all
select 3203,'USA' from dual union all
select 32030,'MRN' from dual union all
select 32031,'MRN' from dual union all
select 32032,'MRN' from dual union all
select 32033,'MRN' from dual union all
select 32034,'MRN' from dual union all
select 32035,'MRN' from dual union all
select 32036,'MRN' from dual union all
select 32037,'MRN' from dual union all
select 32038,'MRN' from dual union all
select 32039,'MRN' from dual union all
select 3204,'USAF' from dual union all
select 32040,'MRN' from dual union all
select 32041,'MRN' from dual union all
select 32042,'MRN' from dual union all
select 32043,'MRN' from dual union all
select 32044,'MRN' from dual union all
select 32045,'MRN' from dual union all
select 32046,'MRN' from dual union all
select 32047,'MRN' from dual union all
select 32048,'MRN' from dual union all
select 32049,'MRN' from dual union all
select 3205,'ORDRSA' from dual union all
select 32050,'MRN' from dual union all
select 32051,'MRN' from dual union all
select 32052,'MRN' from dual union all
select 32053,'MRN' from dual union all
select 32054,'MRN' from dual union all
select 32055,'MRN' from dual union all
select 32056,'MRN' from dual union all
select 32057,'MRN' from dual union all
select 32058,'MRN' from dual union all
select 32059,'MRN' from dual union all
select 3206,'USA' from dual union all
select 32060,'MRN' from dual union all
select 32061,'MRN' from dual union all
select 32062,'MRN' from dual union all
select 32063,'MRN' from dual union all
select 32064,'MRN' from dual union all
select 32065,'MRN' from dual union all
select 32066,'MRN' from dual union all
select 32067,'MRN' from dual union all
select 32068,'MRN' from dual union all
select 32069,'MRN' from dual union all
select 3207,'USA' from dual union all
select 32070,'MRN' from dual union all
select 32071,'MRN' from dual union all
select 32072,'MRN' from dual union all
select 32073,'MRN' from dual union all
select 32074,'MRN' from dual union all
select 32075,'MRN' from dual union all
select 32076,'MRN' from dual union all
select 32077,'MRN' from dual union all
select 32078,'MRN' from dual union all
select 32079,'MRN' from dual union all
select 3208,'USA' from dual union all
select 32080,'MRN' from dual union all
select 32081,'MRN' from dual union all
select 32082,'MRN' from dual union all
select 32083,'MRN' from dual union all
select 32084,'MRN' from dual;
insert into LAUNCH_AGENCY
select 32085,'MRN' from dual union all
select 32086,'MRN' from dual union all
select 32087,'MRN' from dual union all
select 32088,'MRN' from dual union all
select 32089,'MRN' from dual union all
select 3209,'USA' from dual union all
select 32090,'MRN' from dual union all
select 32091,'MRN' from dual union all
select 32092,'MRN' from dual union all
select 32093,'MRN' from dual union all
select 32094,'MRN' from dual union all
select 32095,'MRN' from dual union all
select 32096,'MRN' from dual union all
select 32097,'MRN' from dual union all
select 32098,'MRN' from dual union all
select 32099,'MRN' from dual union all
select 321,'WEHR' from dual union all
select 3210,'USA' from dual union all
select 32100,'MRN' from dual union all
select 32101,'MRN' from dual union all
select 32102,'MRN' from dual union all
select 32103,'CNES' from dual union all
select 32104,'CNES' from dual union all
select 32105,'CNES' from dual union all
select 32106,'CNES' from dual union all
select 32107,'CNES' from dual union all
select 32108,'CNES' from dual union all
select 32109,'CNES' from dual union all
select 3211,'USAF' from dual union all
select 32110,'CNES' from dual union all
select 32111,'CNES' from dual union all
select 32112,'MRN' from dual union all
select 32113,'CNES' from dual union all
select 32114,'CNES' from dual union all
select 32115,'CNES' from dual union all
select 32116,'AN' from dual union all
select 32117,'AN' from dual union all
select 32118,'AN' from dual union all
select 32119,'AN' from dual union all
select 3212,'USA' from dual union all
select 32120,'AN' from dual union all
select 32121,'AN' from dual union all
select 32122,'AN' from dual union all
select 32123,'MRN' from dual union all
select 32124,'AN' from dual union all
select 32125,'UKMO' from dual union all
select 32126,'UKMO' from dual union all
select 32127,'UKMO' from dual union all
select 32128,'UKMO' from dual union all
select 32129,'UKMO' from dual union all
select 3213,'USA' from dual union all
select 32130,'UKMO' from dual union all
select 32131,'UKMO' from dual union all
select 32132,'UKMO' from dual union all
select 32133,'UKMO' from dual union all
select 32134,'MRN' from dual union all
select 32135,'UKMO' from dual union all
select 32136,'UKMO' from dual union all
select 32137,'NOAA' from dual union all
select 32138,'NOAA' from dual union all
select 32139,'NOAA' from dual union all
select 3214,'USA' from dual union all
select 32140,'NOAA' from dual union all
select 32141,'NOAA' from dual union all
select 32142,'NOAA' from dual union all
select 32143,'NOAA' from dual union all
select 32144,'MRN' from dual union all
select 32145,'NOAA' from dual union all
select 32146,'NOAA' from dual union all
select 32147,'NOAA' from dual union all
select 32148,'NOAA' from dual union all
select 32149,'NOAA' from dual union all
select 3215,'ORDRSA' from dual union all
select 32150,'IAE' from dual union all
select 32151,'IAE' from dual union all
select 32152,'IAE' from dual union all
select 32153,'IAE' from dual union all
select 32154,'IAE' from dual union all
select 32155,'MRN' from dual union all
select 32156,'IAE' from dual union all
select 32157,'IAE' from dual union all
select 32158,'IAE' from dual union all
select 32159,'USA' from dual union all
select 3216,'USAF' from dual union all
select 32160,'NOAA' from dual union all
select 32161,'MRN' from dual union all
select 32162,'MRN' from dual union all
select 32163,'MRN' from dual union all
select 32164,'MRN' from dual union all
select 32165,'MRN' from dual union all
select 32166,'MRN' from dual union all
select 32167,'MRN' from dual union all
select 32168,'NASA' from dual union all
select 32169,'MRN' from dual union all
select 3217,'USA' from dual union all
select 32170,'MRN' from dual union all
select 32171,'MRN' from dual union all
select 32172,'MRN' from dual union all
select 32173,'MRN' from dual union all
select 32174,'MRN' from dual;
insert into LAUNCH_AGENCY
select 32175,'MRN' from dual union all
select 32176,'MRN' from dual union all
select 32177,'MRN' from dual union all
select 32178,'NASDA' from dual union all
select 32179,'MRN' from dual union all
select 3218,'USA' from dual union all
select 32180,'MRN' from dual union all
select 32181,'MRN' from dual union all
select 32182,'MRN' from dual union all
select 32183,'MRN' from dual union all
select 32184,'MRN' from dual union all
select 32185,'MRN' from dual union all
select 32186,'MRN' from dual union all
select 32187,'MRN' from dual union all
select 32188,'MRN' from dual union all
select 32189,'MRN' from dual union all
select 3219,'USA' from dual union all
select 32190,'MRN' from dual union all
select 32191,'MRN' from dual union all
select 32192,'INTA' from dual union all
select 32193,'MRN' from dual union all
select 32194,'MRN' from dual union all
select 32195,'MRN' from dual union all
select 32196,'MRN' from dual union all
select 32197,'MRN' from dual union all
select 32198,'MRN' from dual union all
select 32199,'MRN' from dual union all
select 322,'WEHR' from dual union all
select 3220,'6555A' from dual union all
select 32200,'MRN' from dual union all
select 32201,'MRN' from dual union all
select 32202,'NASDA' from dual union all
select 32203,'MRN' from dual union all
select 32204,'MRN' from dual union all
select 32205,'MRN' from dual union all
select 32206,'MRN' from dual union all
select 32207,'MRN' from dual union all
select 32208,'MRN' from dual union all
select 32209,'MRN' from dual union all
select 3221,'6555A' from dual union all
select 32210,'MRN' from dual union all
select 32211,'MRN' from dual union all
select 32212,'MRN' from dual union all
select 32213,'MRN' from dual union all
select 32214,'MRN' from dual union all
select 32215,'MRN' from dual union all
select 32216,'MRN' from dual union all
select 32217,'MRN' from dual union all
select 32218,'MRN' from dual union all
select 32219,'MRN' from dual union all
select 3222,'ORDRSA' from dual union all
select 32220,'MRN' from dual union all
select 32221,'MRN' from dual union all
select 32222,'MRN' from dual union all
select 32223,'MRN' from dual union all
select 32224,'MRN' from dual union all
select 32225,'MRN' from dual union all
select 32226,'MRN' from dual union all
select 32227,'MRN' from dual union all
select 32228,'MRN' from dual union all
select 32229,'MRN' from dual union all
select 3223,'ORDRSA' from dual union all
select 32230,'NASA' from dual union all
select 32231,'MRN' from dual union all
select 32232,'MRN' from dual union all
select 32233,'MRN' from dual union all
select 32234,'MRN' from dual union all
select 32235,'MRN' from dual union all
select 32236,'MRN' from dual union all
select 32237,'MRN' from dual union all
select 32238,'MRN' from dual union all
select 32239,'MRN' from dual union all
select 3224,'USA' from dual union all
select 32240,'MRN' from dual union all
select 32241,'MRN' from dual union all
select 32242,'MRN' from dual union all
select 32243,'MRN' from dual union all
select 32244,'MRN' from dual union all
select 32245,'MRN' from dual union all
select 32246,'MRN' from dual union all
select 32247,'MRN' from dual union all
select 32248,'MRN' from dual union all
select 32249,'MRN' from dual union all
select 3225,'USA' from dual union all
select 32250,'MRN' from dual union all
select 32251,'MRN' from dual union all
select 32252,'MRN' from dual union all
select 32253,'MRN' from dual union all
select 32254,'MRN' from dual union all
select 32255,'MRN' from dual union all
select 32256,'MRN' from dual union all
select 32257,'MRN' from dual union all
select 32258,'MRN' from dual union all
select 32259,'MRN' from dual union all
select 3226,'ORDRSA' from dual union all
select 32260,'MRN' from dual union all
select 32261,'MRN' from dual union all
select 32262,'MRN' from dual union all
select 32263,'MRN' from dual union all
select 32264,'MRN' from dual;
insert into LAUNCH_AGENCY
select 32265,'NASA' from dual union all
select 32266,'MRN' from dual union all
select 32267,'MRN' from dual union all
select 32268,'MRN' from dual union all
select 32269,'MRN' from dual union all
select 3227,'USA' from dual union all
select 32270,'MRN' from dual union all
select 32271,'MRN' from dual union all
select 32272,'MRN' from dual union all
select 32273,'MRN' from dual union all
select 32274,'MRN' from dual union all
select 32275,'MRN' from dual union all
select 32276,'MRN' from dual union all
select 32277,'MRN' from dual union all
select 32278,'MRN' from dual union all
select 32279,'MRN' from dual union all
select 3228,'USA' from dual union all
select 32280,'MRN' from dual union all
select 32281,'MRN' from dual union all
select 32282,'MRN' from dual union all
select 32283,'MRN' from dual union all
select 32284,'MRN' from dual union all
select 32285,'MRN' from dual union all
select 32286,'MRN' from dual union all
select 32287,'MRN' from dual union all
select 32288,'MRN' from dual union all
select 32289,'MRN' from dual union all
select 3229,'USA' from dual union all
select 32290,'MRN' from dual union all
select 32291,'MRN' from dual union all
select 32292,'MRN' from dual union all
select 32293,'MRN' from dual union all
select 32294,'MRN' from dual union all
select 32295,'MRN' from dual union all
select 32296,'MRN' from dual union all
select 32297,'MRN' from dual union all
select 32298,'MRN' from dual union all
select 32299,'MRN' from dual union all
select 323,'WEHR' from dual union all
select 3230,'USA' from dual union all
select 32300,'MRN' from dual union all
select 32301,'MRN' from dual union all
select 32302,'MRN' from dual union all
select 32303,'MRN' from dual union all
select 32304,'MRN' from dual union all
select 32305,'MRN' from dual union all
select 32306,'MRN' from dual union all
select 32307,'MRN' from dual union all
select 32308,'MRN' from dual union all
select 32309,'MRN' from dual union all
select 3231,'USA' from dual union all
select 32310,'MRN' from dual union all
select 32311,'MRN' from dual union all
select 32312,'MRN' from dual union all
select 32313,'MRN' from dual union all
select 32314,'MRN' from dual union all
select 32315,'MRN' from dual union all
select 32316,'MRN' from dual union all
select 32317,'MRN' from dual union all
select 32318,'MRN' from dual union all
select 32319,'MRN' from dual union all
select 3232,'USA' from dual union all
select 32320,'MRN' from dual union all
select 32321,'MRN' from dual union all
select 32322,'MRN' from dual union all
select 32323,'MRN' from dual union all
select 32324,'MRN' from dual union all
select 32325,'MRN' from dual union all
select 32326,'MRN' from dual union all
select 32327,'MRN' from dual union all
select 32328,'MRN' from dual union all
select 32329,'MRN' from dual union all
select 3233,'USA' from dual union all
select 32330,'MRN' from dual union all
select 32331,'MRN' from dual union all
select 32332,'MRN' from dual union all
select 32333,'MRN' from dual union all
select 32334,'MRN' from dual union all
select 32335,'MRN' from dual union all
select 32336,'MRN' from dual union all
select 32337,'MRN' from dual union all
select 32338,'MRN' from dual union all
select 32339,'MRN' from dual union all
select 3234,'USA' from dual union all
select 32340,'MRN' from dual union all
select 32341,'MRN' from dual union all
select 32342,'MRN' from dual union all
select 32343,'INTA' from dual union all
select 32344,'MRN' from dual union all
select 32345,'MRN' from dual union all
select 32346,'MRN' from dual union all
select 32347,'MRN' from dual union all
select 32348,'MRN' from dual union all
select 32349,'MRN' from dual union all
select 3235,'USA' from dual union all
select 32350,'MRN' from dual union all
select 32351,'MRN' from dual union all
select 32352,'MRN' from dual union all
select 32353,'MRN' from dual union all
select 32354,'MRN' from dual;
insert into LAUNCH_AGENCY
select 32355,'MRN' from dual union all
select 32356,'MRN' from dual union all
select 32357,'MRN' from dual union all
select 32358,'MRN' from dual union all
select 32359,'MRN' from dual union all
select 3236,'USA' from dual union all
select 32360,'MRN' from dual union all
select 32361,'MRN' from dual union all
select 32362,'MRN' from dual union all
select 32363,'MRN' from dual union all
select 32364,'MRN' from dual union all
select 32365,'MRN' from dual union all
select 32366,'MRN' from dual union all
select 32367,'MRN' from dual union all
select 32368,'MRN' from dual union all
select 32369,'MRN' from dual union all
select 3237,'USA' from dual union all
select 32370,'MRN' from dual union all
select 32371,'MRN' from dual union all
select 32372,'MRN' from dual union all
select 32373,'MRN' from dual union all
select 32374,'MRN' from dual union all
select 32375,'MRN' from dual union all
select 32376,'MRN' from dual union all
select 32377,'MRN' from dual union all
select 32378,'MRN' from dual union all
select 32379,'MRN' from dual union all
select 3238,'USA' from dual union all
select 32380,'MRN' from dual union all
select 32381,'MRN' from dual union all
select 32382,'MRN' from dual union all
select 32383,'MRN' from dual union all
select 32384,'MRN' from dual union all
select 32385,'MRN' from dual union all
select 32386,'MRN' from dual union all
select 32387,'MRN' from dual union all
select 32388,'MRN' from dual union all
select 32389,'MRN' from dual union all
select 3239,'PARD' from dual union all
select 32390,'MRN' from dual union all
select 32391,'MRN' from dual union all
select 32392,'MRN' from dual union all
select 32393,'MRN' from dual union all
select 32394,'MRN' from dual union all
select 32395,'MRN' from dual union all
select 32396,'MRN' from dual union all
select 32397,'MRN' from dual union all
select 32398,'MRN' from dual union all
select 32399,'MRN' from dual union all
select 324,'WEHR' from dual union all
select 3240,'USA' from dual union all
select 32400,'MRN' from dual union all
select 32401,'MRN' from dual union all
select 32402,'MRN' from dual union all
select 32403,'MRN' from dual union all
select 32404,'MRN' from dual union all
select 32405,'MRN' from dual union all
select 32406,'MRN' from dual union all
select 32407,'MRN' from dual union all
select 32408,'MRN' from dual union all
select 32409,'MRN' from dual union all
select 3241,'USA' from dual union all
select 32410,'MRN' from dual union all
select 32411,'MRN' from dual union all
select 32412,'MRN' from dual union all
select 32413,'MRN' from dual union all
select 32414,'MRN' from dual union all
select 32415,'MRN' from dual union all
select 32416,'MRN' from dual union all
select 32417,'MRN' from dual union all
select 32418,'MRN' from dual union all
select 32419,'MRN' from dual union all
select 3242,'USA' from dual union all
select 32420,'MRN' from dual union all
select 32421,'MRN' from dual union all
select 32422,'MRN' from dual union all
select 32423,'MRN' from dual union all
select 32424,'MRN' from dual union all
select 32425,'MRN' from dual union all
select 32426,'MRN' from dual union all
select 32427,'MRN' from dual union all
select 32428,'MRN' from dual union all
select 32429,'MRN' from dual union all
select 3243,'USA' from dual union all
select 32430,'MRN' from dual union all
select 32431,'MRN' from dual union all
select 32432,'MRN' from dual union all
select 32433,'MRN' from dual union all
select 32434,'MRN' from dual union all
select 32435,'MRN' from dual union all
select 32436,'MRN' from dual union all
select 32437,'MRN' from dual union all
select 32438,'MRN' from dual union all
select 32439,'MRN' from dual union all
select 3244,'USA' from dual union all
select 32440,'MRN' from dual union all
select 32441,'MRN' from dual union all
select 32442,'MRN' from dual union all
select 32443,'MRN' from dual union all
select 32444,'MRN' from dual;
insert into LAUNCH_AGENCY
select 32445,'MRN' from dual union all
select 32446,'MRN' from dual union all
select 32447,'MRN' from dual union all
select 32448,'MRN' from dual union all
select 32449,'MRN' from dual union all
select 3245,'ORDRSA' from dual union all
select 32450,'MRN' from dual union all
select 32451,'MRN' from dual union all
select 32452,'MRN' from dual union all
select 32453,'MRN' from dual union all
select 32454,'MRN' from dual union all
select 32455,'MRN' from dual union all
select 32456,'MRN' from dual union all
select 32457,'MRN' from dual union all
select 32458,'MRN' from dual union all
select 32459,'MRN' from dual union all
select 3246,'ORDRSA' from dual union all
select 32460,'MRN' from dual union all
select 32461,'MRN' from dual union all
select 32462,'MRN' from dual union all
select 32463,'MRN' from dual union all
select 32464,'MRN' from dual union all
select 32465,'MRN' from dual union all
select 32466,'MRN' from dual union all
select 32467,'MRN' from dual union all
select 32468,'MRN' from dual union all
select 32469,'MRN' from dual union all
select 3247,'USA' from dual union all
select 32470,'MRN' from dual union all
select 32471,'MRN' from dual union all
select 32472,'MRN' from dual union all
select 32473,'MRN' from dual union all
select 32474,'MRN' from dual union all
select 32475,'MRN' from dual union all
select 32476,'MRN' from dual union all
select 32477,'MRN' from dual union all
select 32478,'MRN' from dual union all
select 32479,'MRN' from dual union all
select 3248,'USA' from dual union all
select 32480,'MRN' from dual union all
select 32481,'MRN' from dual union all
select 32482,'MRN' from dual union all
select 32483,'MRN' from dual union all
select 32484,'MRN' from dual union all
select 32485,'MRN' from dual union all
select 32486,'MRN' from dual union all
select 32487,'MRN' from dual union all
select 32488,'MRN' from dual union all
select 32489,'MRN' from dual union all
select 3249,'CASDN' from dual union all
select 32490,'MRN' from dual union all
select 32491,'MRN' from dual union all
select 32492,'MRN' from dual union all
select 32493,'MRN' from dual union all
select 32494,'MRN' from dual union all
select 32495,'MRN' from dual union all
select 32496,'MRN' from dual union all
select 32497,'MRN' from dual union all
select 32498,'MRN' from dual union all
select 32499,'MRN' from dual union all
select 325,'WEHR' from dual union all
select 3250,'CASDN' from dual union all
select 32500,'MRN' from dual union all
select 32501,'MRN' from dual union all
select 32502,'MRN' from dual union all
select 32503,'MRN' from dual union all
select 32504,'MRN' from dual union all
select 32505,'MRN' from dual union all
select 32506,'MRN' from dual union all
select 32507,'MRN' from dual union all
select 32508,'MRN' from dual union all
select 32509,'MRN' from dual union all
select 3251,'CASDN' from dual union all
select 32510,'MRN' from dual union all
select 32511,'MRN' from dual union all
select 32512,'MRN' from dual union all
select 32513,'MRN' from dual union all
select 32514,'MRN' from dual union all
select 32515,'MRN' from dual union all
select 32516,'MRN' from dual union all
select 32517,'MRN' from dual union all
select 32518,'MRN' from dual union all
select 32519,'MRN' from dual union all
select 3252,'USA' from dual union all
select 32520,'MRN' from dual union all
select 32521,'MRN' from dual union all
select 32522,'MRN' from dual union all
select 32523,'MRN' from dual union all
select 32524,'MRN' from dual union all
select 32525,'MRN' from dual union all
select 32526,'MRN' from dual union all
select 32527,'MRN' from dual union all
select 32528,'MRN' from dual union all
select 32529,'MRN' from dual union all
select 3253,'USA' from dual union all
select 32530,'MRN' from dual union all
select 32531,'MRN' from dual union all
select 32532,'INTA' from dual union all
select 32533,'MRN' from dual union all
select 32534,'MRN' from dual;
insert into LAUNCH_AGENCY
select 32535,'MRN' from dual union all
select 32536,'MRN' from dual union all
select 32537,'MRN' from dual union all
select 32538,'MRN' from dual union all
select 32539,'MRN' from dual union all
select 3254,'USAF' from dual union all
select 32540,'MRN' from dual union all
select 32541,'MRN' from dual union all
select 32542,'MRN' from dual union all
select 32543,'MRN' from dual union all
select 32544,'MRN' from dual union all
select 32545,'MRN' from dual union all
select 32546,'MRN' from dual union all
select 32547,'MRN' from dual union all
select 32548,'MRN' from dual union all
select 32549,'MRN' from dual union all
select 3255,'USAF' from dual union all
select 32550,'MRN' from dual union all
select 32551,'MRN' from dual union all
select 32552,'MRN' from dual union all
select 32553,'MRN' from dual union all
select 32554,'MRN' from dual union all
select 32555,'MRN' from dual union all
select 32556,'MRN' from dual union all
select 32557,'MRN' from dual union all
select 32558,'MRN' from dual union all
select 32559,'MRN' from dual union all
select 3256,'USA' from dual union all
select 32560,'MRN' from dual union all
select 32561,'MRN' from dual union all
select 32562,'MRN' from dual union all
select 32563,'MRN' from dual union all
select 32564,'MRN' from dual union all
select 32565,'MRN' from dual union all
select 32566,'MRN' from dual union all
select 32567,'MRN' from dual union all
select 32568,'MRN' from dual union all
select 32569,'MRN' from dual union all
select 3257,'USA' from dual union all
select 32570,'MRN' from dual union all
select 32571,'MRN' from dual union all
select 32572,'MRN' from dual union all
select 32573,'MRN' from dual union all
select 32574,'MRN' from dual union all
select 32575,'MRN' from dual union all
select 32576,'MRN' from dual union all
select 32577,'MRN' from dual union all
select 32578,'MRN' from dual union all
select 32579,'MRN' from dual union all
select 3258,'USA' from dual union all
select 32580,'MRN' from dual union all
select 32581,'MRN' from dual union all
select 32582,'MRN' from dual union all
select 32583,'MRN' from dual union all
select 32584,'MRN' from dual union all
select 32585,'MRN' from dual union all
select 32586,'MRN' from dual union all
select 32587,'MRN' from dual union all
select 32588,'MRN' from dual union all
select 32589,'MRN' from dual union all
select 3259,'USA' from dual union all
select 32590,'MRN' from dual union all
select 32591,'MRN' from dual union all
select 32592,'MRN' from dual union all
select 32593,'MRN' from dual union all
select 32594,'MRN' from dual union all
select 32595,'MRN' from dual union all
select 32596,'MRN' from dual union all
select 32597,'MRN' from dual union all
select 32598,'MRN' from dual union all
select 32599,'MRN' from dual union all
select 326,'WEHR' from dual union all
select 3260,'USA' from dual union all
select 32600,'MRN' from dual union all
select 32601,'MRN' from dual union all
select 32602,'MRN' from dual union all
select 32603,'MRN' from dual union all
select 32604,'MRN' from dual union all
select 32605,'MRN' from dual union all
select 32606,'MRN' from dual union all
select 32607,'MRN' from dual union all
select 32608,'MRN' from dual union all
select 32609,'MRN' from dual union all
select 3261,'USA' from dual union all
select 32610,'MRN' from dual union all
select 32611,'MRN' from dual union all
select 32612,'MRN' from dual union all
select 32613,'MRN' from dual union all
select 32614,'MRN' from dual union all
select 32615,'MRN' from dual union all
select 32616,'MRN' from dual union all
select 32617,'MRN' from dual union all
select 32618,'MRN' from dual union all
select 32619,'MRN' from dual union all
select 3262,'USA' from dual union all
select 32620,'MRN' from dual union all
select 32621,'MRN' from dual union all
select 32622,'MRN' from dual union all
select 32623,'MRN' from dual union all
select 32624,'MRN' from dual;
insert into LAUNCH_AGENCY
select 32625,'MRN' from dual union all
select 32626,'MRN' from dual union all
select 32627,'MRN' from dual union all
select 32628,'MRN' from dual union all
select 32629,'MRN' from dual union all
select 3263,'ORDRSA' from dual union all
select 32630,'MRN' from dual union all
select 32631,'MRN' from dual union all
select 32632,'MRN' from dual union all
select 32633,'MRN' from dual union all
select 32634,'MRN' from dual union all
select 32635,'MRN' from dual union all
select 32636,'MRN' from dual union all
select 32637,'MRN' from dual union all
select 32638,'MRN' from dual union all
select 32639,'MRN' from dual union all
select 3264,'USA' from dual union all
select 32640,'MRN' from dual union all
select 32641,'MRN' from dual union all
select 32642,'MRN' from dual union all
select 32643,'MRN' from dual union all
select 32644,'MRN' from dual union all
select 32645,'MRN' from dual union all
select 32646,'MRN' from dual union all
select 32647,'MRN' from dual union all
select 32648,'MRN' from dual union all
select 32649,'MRN' from dual union all
select 3265,'USA' from dual union all
select 32650,'MRN' from dual union all
select 32651,'MRN' from dual union all
select 32652,'MRN' from dual union all
select 32653,'MRN' from dual union all
select 32654,'MRN' from dual union all
select 32655,'MRN' from dual union all
select 32656,'MRN' from dual union all
select 32657,'MRN' from dual union all
select 32658,'MRN' from dual union all
select 32659,'INTA' from dual union all
select 3266,'USA' from dual union all
select 32660,'MRN' from dual union all
select 32661,'MRN' from dual union all
select 32662,'MRN' from dual union all
select 32663,'MRN' from dual union all
select 32664,'MRN' from dual union all
select 32665,'MRN' from dual union all
select 32666,'MRN' from dual union all
select 32667,'MRN' from dual union all
select 32668,'MRN' from dual union all
select 32669,'MRN' from dual union all
select 3267,'USA' from dual union all
select 32670,'MRN' from dual union all
select 32671,'MRN' from dual union all
select 32672,'MRN' from dual union all
select 32673,'MRN' from dual union all
select 32674,'MRN' from dual union all
select 32675,'MRN' from dual union all
select 32676,'MRN' from dual union all
select 32677,'MRN' from dual union all
select 32678,'MRN' from dual union all
select 32679,'MRN' from dual union all
select 3268,'ORDRSA' from dual union all
select 32680,'MRN' from dual union all
select 32681,'MRN' from dual union all
select 32682,'MRN' from dual union all
select 32683,'MRN' from dual union all
select 32684,'MRN' from dual union all
select 32685,'MRN' from dual union all
select 32686,'MRN' from dual union all
select 32687,'MRN' from dual union all
select 32688,'MRN' from dual union all
select 32689,'MRN' from dual union all
select 3269,'ORDRSA' from dual union all
select 32690,'MRN' from dual union all
select 32691,'MRN' from dual union all
select 32692,'MRN' from dual union all
select 32693,'MRN' from dual union all
select 32694,'MRN' from dual union all
select 32695,'NRCC' from dual union all
select 32696,'MRN' from dual union all
select 32697,'MRN' from dual union all
select 32698,'MRN' from dual union all
select 32699,'MRN' from dual union all
select 327,'WEHR' from dual union all
select 3270,'USA' from dual union all
select 32700,'MRN' from dual union all
select 32701,'MRN' from dual union all
select 32702,'MRN' from dual union all
select 32703,'MRN' from dual union all
select 32704,'MRN' from dual union all
select 32705,'MRN' from dual union all
select 32706,'MRN' from dual union all
select 32707,'MRN' from dual union all
select 32708,'MRN' from dual union all
select 32709,'MRN' from dual union all
select 3271,'USA' from dual union all
select 32710,'MRN' from dual union all
select 32711,'MRN' from dual union all
select 32712,'MRN' from dual union all
select 32713,'MRN' from dual union all
select 32714,'MRN' from dual;
insert into LAUNCH_AGENCY
select 32715,'MRN' from dual union all
select 32716,'MRN' from dual union all
select 32717,'MRN' from dual union all
select 32718,'MRN' from dual union all
select 32719,'MRN' from dual union all
select 3272,'USA' from dual union all
select 32720,'MRN' from dual union all
select 32721,'MRN' from dual union all
select 32722,'MRN' from dual union all
select 32723,'MRN' from dual union all
select 32724,'MRN' from dual union all
select 32725,'MRN' from dual union all
select 32726,'MRN' from dual union all
select 32727,'MRN' from dual union all
select 32728,'MRN' from dual union all
select 32729,'MRN' from dual union all
select 3273,'USA' from dual union all
select 32730,'MRN' from dual union all
select 32731,'MRN' from dual union all
select 32732,'MRN' from dual union all
select 32733,'MRN' from dual union all
select 32734,'MRN' from dual union all
select 32735,'MRN' from dual union all
select 32736,'MRN' from dual union all
select 32737,'MRN' from dual union all
select 32738,'MRN' from dual union all
select 32739,'MRN' from dual union all
select 3274,'USA' from dual union all
select 32740,'MRN' from dual union all
select 32741,'MRN' from dual union all
select 32742,'MRN' from dual union all
select 32743,'MRN' from dual union all
select 32744,'MRN' from dual union all
select 32745,'MRN' from dual union all
select 32746,'MRN' from dual union all
select 32747,'MRN' from dual union all
select 32748,'MRN' from dual union all
select 32749,'MRN' from dual union all
select 3275,'USA' from dual union all
select 32750,'MRN' from dual union all
select 32751,'MRN' from dual union all
select 32752,'MRN' from dual union all
select 32753,'MRN' from dual union all
select 32754,'MRN' from dual union all
select 32755,'MRN' from dual union all
select 32756,'MRN' from dual union all
select 32757,'MRN' from dual union all
select 32758,'MRN' from dual union all
select 32759,'MRN' from dual union all
select 3276,'USA' from dual union all
select 32760,'MRN' from dual union all
select 32761,'MRN' from dual union all
select 32762,'MRN' from dual union all
select 32763,'MRN' from dual union all
select 32764,'MRN' from dual union all
select 32765,'MRN' from dual union all
select 32766,'MRN' from dual union all
select 32767,'MRN' from dual union all
select 32768,'MRN' from dual union all
select 32769,'MRN' from dual union all
select 3277,'USA' from dual union all
select 32770,'MRN' from dual union all
select 32771,'MRN' from dual union all
select 32772,'MRN' from dual union all
select 32773,'MRN' from dual union all
select 32774,'MRN' from dual union all
select 32775,'MRN' from dual union all
select 32776,'MRN' from dual union all
select 32777,'MRN' from dual union all
select 32778,'MRN' from dual union all
select 32779,'MRN' from dual union all
select 3278,'USA' from dual union all
select 32780,'MRN' from dual union all
select 32781,'MRN' from dual union all
select 32782,'MRN' from dual union all
select 32783,'MRN' from dual union all
select 32784,'MRN' from dual union all
select 32785,'MRN' from dual union all
select 32786,'MRN' from dual union all
select 32787,'MRN' from dual union all
select 32788,'NASA' from dual union all
select 32789,'MRN' from dual union all
select 3279,'USAF' from dual union all
select 32790,'MRN' from dual union all
select 32791,'MRN' from dual union all
select 32792,'MRN' from dual union all
select 32793,'INTA' from dual union all
select 32794,'MRN' from dual union all
select 32795,'MRN' from dual union all
select 32796,'MRN' from dual union all
select 32797,'MRN' from dual union all
select 32798,'MRN' from dual union all
select 32799,'MRN' from dual union all
select 328,'WEHR' from dual union all
select 3280,'USA' from dual union all
select 32800,'MRN' from dual union all
select 32801,'MRN' from dual union all
select 32802,'MRN' from dual union all
select 32803,'MRN' from dual union all
select 32804,'MRN' from dual;
insert into LAUNCH_AGENCY
select 32805,'MRN' from dual union all
select 32806,'MRN' from dual union all
select 32807,'MRN' from dual union all
select 32808,'MRN' from dual union all
select 32809,'MRN' from dual union all
select 3281,'ORDRSA' from dual union all
select 32810,'MRN' from dual union all
select 32811,'MRN' from dual union all
select 32812,'MRN' from dual union all
select 32813,'MRN' from dual union all
select 32814,'MRN' from dual union all
select 32815,'MRN' from dual union all
select 32816,'MRN' from dual union all
select 32817,'MRN' from dual union all
select 32818,'MRN' from dual union all
select 32819,'MRN' from dual union all
select 3282,'USA' from dual union all
select 32820,'MRN' from dual union all
select 32821,'MRN' from dual union all
select 32822,'MRN' from dual union all
select 32823,'MRN' from dual union all
select 32824,'MRN' from dual union all
select 32825,'MRN' from dual union all
select 32826,'MRN' from dual union all
select 32827,'MRN' from dual union all
select 32828,'MRN' from dual union all
select 32829,'MRN' from dual union all
select 3283,'USA' from dual union all
select 32830,'MRN' from dual union all
select 32831,'AN' from dual union all
select 32832,'MRN' from dual union all
select 32833,'MRN' from dual union all
select 32834,'MRN' from dual union all
select 32835,'MRN' from dual union all
select 32836,'MRN' from dual union all
select 32837,'MRN' from dual union all
select 32838,'MRN' from dual union all
select 32839,'MRN' from dual union all
select 3284,'USA' from dual union all
select 32840,'MRN' from dual union all
select 32841,'MRN' from dual union all
select 32842,'MRN' from dual union all
select 32843,'MRN' from dual union all
select 32844,'MRN' from dual union all
select 32845,'MRN' from dual union all
select 32846,'MRN' from dual union all
select 32847,'MRN' from dual union all
select 32848,'MRN' from dual union all
select 32849,'MRN' from dual union all
select 3285,'USAF' from dual union all
select 32850,'MRN' from dual union all
select 32851,'MRN' from dual union all
select 32852,'MRN' from dual union all
select 32853,'MRN' from dual union all
select 32854,'MRN' from dual union all
select 32855,'MRN' from dual union all
select 32856,'MRN' from dual union all
select 32857,'MRN' from dual union all
select 32858,'MRN' from dual union all
select 32859,'MRN' from dual union all
select 3286,'USAF' from dual union all
select 32860,'MRN' from dual union all
select 32861,'MRN' from dual union all
select 32862,'MRN' from dual union all
select 32863,'MRN' from dual union all
select 32864,'MRN' from dual union all
select 32865,'MRN' from dual union all
select 32866,'MRN' from dual union all
select 32867,'MRN' from dual union all
select 32868,'MRN' from dual union all
select 32869,'MRN' from dual union all
select 3287,'USA' from dual union all
select 32870,'MRN' from dual union all
select 32871,'MRN' from dual union all
select 32872,'MRN' from dual union all
select 32873,'MRN' from dual union all
select 32874,'MRN' from dual union all
select 32875,'MRN' from dual union all
select 32876,'MRN' from dual union all
select 32877,'MRN' from dual union all
select 32878,'MRN' from dual union all
select 32879,'MRN' from dual union all
select 3288,'USA' from dual union all
select 32880,'MRN' from dual union all
select 32881,'MRN' from dual union all
select 32882,'MRN' from dual union all
select 32883,'MRN' from dual union all
select 32884,'MRN' from dual union all
select 32885,'MRN' from dual union all
select 32886,'MRN' from dual union all
select 32887,'MRN' from dual union all
select 32888,'MRN' from dual union all
select 32889,'MRN' from dual union all
select 3289,'USA' from dual union all
select 32890,'MRN' from dual union all
select 32891,'MRN' from dual union all
select 32892,'MRN' from dual union all
select 32893,'MRN' from dual union all
select 32894,'MRN' from dual union all
select 32895,'MRN' from dual;
insert into LAUNCH_AGENCY
select 32896,'MRN' from dual union all
select 32897,'MRN' from dual union all
select 32898,'MRN' from dual union all
select 32899,'MRN' from dual union all
select 329,'WEHR' from dual union all
select 3290,'USA' from dual union all
select 32900,'MRN' from dual union all
select 32901,'MRN' from dual union all
select 32902,'MRN' from dual union all
select 32903,'MRN' from dual union all
select 32904,'MRN' from dual union all
select 32905,'MRN' from dual union all
select 32906,'MRN' from dual union all
select 32907,'MRN' from dual union all
select 32908,'MRN' from dual union all
select 32909,'MRN' from dual union all
select 3291,'USA' from dual union all
select 32910,'MRN' from dual union all
select 32911,'MRN' from dual union all
select 32912,'MRN' from dual union all
select 32913,'MRN' from dual union all
select 32914,'MRN' from dual union all
select 32915,'MRN' from dual union all
select 32916,'MRN' from dual union all
select 32917,'MRN' from dual union all
select 32918,'MRN' from dual union all
select 32919,'MRN' from dual union all
select 3292,'ORDRSA' from dual union all
select 32920,'MRN' from dual union all
select 32921,'MRN' from dual union all
select 32922,'MRN' from dual union all
select 32923,'MRN' from dual union all
select 32924,'MRN' from dual union all
select 32925,'MRN' from dual union all
select 32926,'MRN' from dual union all
select 32927,'MRN' from dual union all
select 32928,'MRN' from dual union all
select 32929,'MRN' from dual union all
select 3293,'USA' from dual union all
select 32930,'MRN' from dual union all
select 32931,'MRN' from dual union all
select 32932,'MRN' from dual union all
select 32933,'MRN' from dual union all
select 32934,'MRN' from dual union all
select 32935,'MRN' from dual union all
select 32936,'MRN' from dual union all
select 32937,'MRN' from dual union all
select 32938,'MRN' from dual union all
select 32939,'MRN' from dual union all
select 3294,'6555A' from dual union all
select 32940,'MRN' from dual union all
select 32941,'MRN' from dual union all
select 32942,'MRN' from dual union all
select 32943,'MRN' from dual union all
select 32944,'MRN' from dual union all
select 32945,'USA' from dual union all
select 32946,'MRN' from dual union all
select 32947,'MRN' from dual union all
select 32948,'MRN' from dual union all
select 32949,'MRN' from dual union all
select 3295,'USA' from dual union all
select 32950,'MRN' from dual union all
select 32951,'MRN' from dual union all
select 32952,'MRN' from dual union all
select 32953,'MRN' from dual union all
select 32954,'MRN' from dual union all
select 32955,'MRN' from dual union all
select 32956,'MRN' from dual union all
select 32957,'MRN' from dual union all
select 32958,'MRN' from dual union all
select 32959,'MRN' from dual union all
select 3296,'USA' from dual union all
select 32960,'MRN' from dual union all
select 32961,'MRN' from dual union all
select 32962,'MRN' from dual union all
select 32963,'USA' from dual union all
select 32964,'NASA' from dual union all
select 32965,'NASA' from dual union all
select 32966,'NASA' from dual union all
select 32967,'NASA' from dual union all
select 32968,'MRN' from dual union all
select 32969,'NASA' from dual union all
select 3297,'USA' from dual union all
select 32970,'MRN' from dual union all
select 32971,'MRN' from dual union all
select 32972,'MRN' from dual union all
select 32973,'NASA' from dual union all
select 32974,'NASA' from dual union all
select 32975,'NASA' from dual union all
select 32976,'NASA' from dual union all
select 32977,'NASA' from dual union all
select 32978,'MRN' from dual union all
select 32979,'MRN' from dual union all
select 3298,'USA' from dual union all
select 32980,'MRN' from dual union all
select 32981,'MRN' from dual union all
select 32982,'MRN' from dual union all
select 32983,'MRN' from dual union all
select 32984,'MRN' from dual union all
select 32985,'MRN' from dual;
insert into LAUNCH_AGENCY
select 32986,'MRN' from dual union all
select 32987,'MRN' from dual union all
select 32988,'MRN' from dual union all
select 32989,'MRN' from dual union all
select 3299,'USA' from dual union all
select 32990,'MRN' from dual union all
select 32991,'MRN' from dual union all
select 32992,'MRN' from dual union all
select 32993,'MRN' from dual union all
select 32994,'MRN' from dual union all
select 32995,'MRN' from dual union all
select 32996,'MRN' from dual union all
select 32997,'MRN' from dual union all
select 32998,'MRN' from dual union all
select 32999,'MRN' from dual union all
select 33,'WEHR' from dual union all
select 330,'WEHR' from dual union all
select 3300,'USA' from dual union all
select 33000,'MRN' from dual union all
select 33001,'MRN' from dual union all
select 33002,'MRN' from dual union all
select 33003,'MRN' from dual union all
select 33004,'MRN' from dual union all
select 33005,'MRN' from dual union all
select 33006,'MRN' from dual union all
select 33007,'MRN' from dual union all
select 33008,'MRN' from dual union all
select 33009,'MRN' from dual union all
select 3301,'USA' from dual union all
select 33010,'MRN' from dual union all
select 33011,'MRN' from dual union all
select 33012,'MRN' from dual union all
select 33013,'MRN' from dual union all
select 33014,'MRN' from dual union all
select 33015,'MRN' from dual union all
select 33016,'MRN' from dual union all
select 33017,'MRN' from dual union all
select 33018,'MRN' from dual union all
select 33019,'MRN' from dual union all
select 3302,'PARD' from dual union all
select 33020,'MRN' from dual union all
select 33021,'MRN' from dual union all
select 33022,'MRN' from dual union all
select 33023,'MRN' from dual union all
select 33024,'MRN' from dual union all
select 33025,'MRN' from dual union all
select 33026,'MRN' from dual union all
select 33027,'MRN' from dual union all
select 33028,'MRN' from dual union all
select 33029,'MRN' from dual union all
select 3303,'ORDRSA' from dual union all
select 33030,'MRN' from dual union all
select 33031,'MRN' from dual union all
select 33032,'MRN' from dual union all
select 33033,'MRN' from dual union all
select 33034,'MRN' from dual union all
select 33035,'MRN' from dual union all
select 33036,'MRN' from dual union all
select 33037,'MRN' from dual union all
select 33038,'MRN' from dual union all
select 33039,'MRN' from dual union all
select 3304,'6555A' from dual union all
select 33040,'MRN' from dual union all
select 33041,'MRN' from dual union all
select 33042,'MRN' from dual union all
select 33043,'MRN' from dual union all
select 33044,'MRN' from dual union all
select 33045,'MRN' from dual union all
select 33046,'MRN' from dual union all
select 33047,'MRN' from dual union all
select 33048,'MRN' from dual union all
select 33049,'MRN' from dual union all
select 3305,'PARD' from dual union all
select 33050,'MRN' from dual union all
select 33051,'MRN' from dual union all
select 33052,'MRN' from dual union all
select 33053,'MRN' from dual union all
select 33054,'MRN' from dual union all
select 33055,'MRN' from dual union all
select 33056,'MRN' from dual union all
select 33057,'MRN' from dual union all
select 33058,'MRN' from dual union all
select 33059,'MRN' from dual union all
select 3306,'PARD' from dual union all
select 33060,'MRN' from dual union all
select 33061,'MRN' from dual union all
select 33062,'MRN' from dual union all
select 33063,'MRN' from dual union all
select 33064,'MRN' from dual union all
select 33065,'MRN' from dual union all
select 33066,'MRN' from dual union all
select 33067,'MRN' from dual union all
select 33068,'MRN' from dual union all
select 33069,'MRN' from dual union all
select 3307,'6555A' from dual union all
select 33070,'MRN' from dual union all
select 33071,'MRN' from dual union all
select 33072,'MRN' from dual union all
select 33073,'MRN' from dual union all
select 33074,'MRN' from dual;
insert into LAUNCH_AGENCY
select 33075,'MRN' from dual union all
select 33076,'MRN' from dual union all
select 33077,'MRN' from dual union all
select 33078,'CNES' from dual union all
select 33079,'CNES' from dual union all
select 3308,'USAF' from dual union all
select 33080,'CNES' from dual union all
select 33081,'CNES' from dual union all
select 33082,'CNES' from dual union all
select 33083,'CNES' from dual union all
select 33084,'CNES' from dual union all
select 33085,'CNES' from dual union all
select 33086,'CNES' from dual union all
select 33087,'CNES' from dual union all
select 33088,'AN' from dual union all
select 33089,'AN' from dual union all
select 3309,'USAF' from dual union all
select 33090,'MRN' from dual union all
select 33091,'AN' from dual union all
select 33092,'AN' from dual union all
select 33093,'AN' from dual union all
select 33094,'AN' from dual union all
select 33095,'MRN' from dual union all
select 33096,'AN' from dual union all
select 33097,'AN' from dual union all
select 33098,'CNES' from dual union all
select 33099,'AN' from dual union all
select 331,'WEHR' from dual union all
select 3310,'USAF' from dual union all
select 33100,'AN' from dual union all
select 33101,'AN' from dual union all
select 33102,'AN' from dual union all
select 33103,'AN' from dual union all
select 33104,'AN' from dual union all
select 33105,'MRN' from dual union all
select 33106,'AN' from dual union all
select 33107,'AN' from dual union all
select 33108,'AN' from dual union all
select 33109,'CNES' from dual union all
select 3311,'6555A' from dual union all
select 33110,'AN' from dual union all
select 33111,'AN' from dual union all
select 33112,'MRN' from dual union all
select 33113,'AN' from dual union all
select 33114,'AN' from dual union all
select 33115,'AN' from dual union all
select 33116,'AN' from dual union all
select 33117,'AN' from dual union all
select 33118,'AN' from dual union all
select 33119,'AN' from dual union all
select 3312,'USAF' from dual union all
select 33120,'AN' from dual union all
select 33121,'AN' from dual union all
select 33122,'AN' from dual union all
select 33123,'AN' from dual union all
select 33124,'MRN' from dual union all
select 33125,'MRN' from dual union all
select 33126,'AN' from dual union all
select 33127,'AN' from dual union all
select 33128,'AN' from dual union all
select 33129,'AN' from dual union all
select 3313,'USAF' from dual union all
select 33130,'AN' from dual union all
select 33131,'AN' from dual union all
select 33132,'AN' from dual union all
select 33133,'AN' from dual union all
select 33134,'AN' from dual union all
select 33135,'AN' from dual union all
select 33136,'AN' from dual union all
select 33137,'AN' from dual union all
select 33138,'AN' from dual union all
select 33139,'MRN' from dual union all
select 3314,'ORDRSA' from dual union all
select 33140,'AN' from dual union all
select 33141,'AN' from dual union all
select 33142,'MRN' from dual union all
select 33143,'MRN' from dual union all
select 33144,'AN' from dual union all
select 33145,'AN' from dual union all
select 33146,'AN' from dual union all
select 33147,'AN' from dual union all
select 33148,'AN' from dual union all
select 33149,'AN' from dual union all
select 3315,'USAF' from dual union all
select 33150,'AN' from dual union all
select 33151,'AN' from dual union all
select 33152,'AN' from dual union all
select 33153,'AN' from dual union all
select 33154,'AN' from dual union all
select 33155,'AN' from dual union all
select 33156,'AN' from dual union all
select 33157,'AN' from dual union all
select 33158,'MRN' from dual union all
select 33159,'AN' from dual union all
select 3316,'USAF' from dual union all
select 33160,'AN' from dual union all
select 33161,'AN' from dual union all
select 33162,'AN' from dual union all
select 33163,'AN' from dual union all
select 33164,'AN' from dual;
insert into LAUNCH_AGENCY
select 33165,'AN' from dual union all
select 33166,'MRN' from dual union all
select 33167,'MRN' from dual union all
select 33168,'MRN' from dual union all
select 33169,'AN' from dual union all
select 3317,'USAF' from dual union all
select 33170,'AN' from dual union all
select 33171,'AN' from dual union all
select 33172,'AN' from dual union all
select 33173,'AN' from dual union all
select 33174,'AN' from dual union all
select 33175,'AN' from dual union all
select 33176,'MRN' from dual union all
select 33177,'MRN' from dual union all
select 33178,'MRN' from dual union all
select 33179,'AN' from dual union all
select 3318,'USAF' from dual union all
select 33180,'AN' from dual union all
select 33181,'AN' from dual union all
select 33182,'AN' from dual union all
select 33183,'AN' from dual union all
select 33184,'AN' from dual union all
select 33185,'AN' from dual union all
select 33186,'AN' from dual union all
select 33187,'MRN' from dual union all
select 33188,'AN' from dual union all
select 33189,'MRN' from dual union all
select 3319,'USAF' from dual union all
select 33190,'AN' from dual union all
select 33191,'AN' from dual union all
select 33192,'AN' from dual union all
select 33193,'AN' from dual union all
select 33194,'AN' from dual union all
select 33195,'AN' from dual union all
select 33196,'AN' from dual union all
select 33197,'AN' from dual union all
select 33198,'AN' from dual union all
select 33199,'AN' from dual union all
select 332,'WEHR' from dual union all
select 3320,'USN' from dual union all
select 33200,'MRN' from dual union all
select 33201,'MRN' from dual union all
select 33202,'AN' from dual union all
select 33203,'AN' from dual union all
select 33204,'AN' from dual union all
select 33205,'MRN' from dual union all
select 33206,'AN' from dual union all
select 33207,'AN' from dual union all
select 33208,'AN' from dual union all
select 33209,'MRN' from dual union all
select 3321,'USAF' from dual union all
select 33210,'AN' from dual union all
select 33211,'AN' from dual union all
select 33212,'AN' from dual union all
select 33213,'AN' from dual union all
select 33214,'AN' from dual union all
select 33215,'MRN' from dual union all
select 33216,'MRN' from dual union all
select 33217,'AN' from dual union all
select 33218,'AN' from dual union all
select 33219,'AN' from dual union all
select 3322,'NRL' from dual union all
select 33220,'AN' from dual union all
select 33221,'AN' from dual union all
select 33222,'AN' from dual union all
select 33223,'AN' from dual union all
select 33224,'AN' from dual union all
select 33225,'AN' from dual union all
select 33226,'MRN' from dual union all
select 33227,'AN' from dual union all
select 33228,'AN' from dual union all
select 33229,'AN' from dual union all
select 3323,'6555A' from dual union all
select 33230,'AN' from dual union all
select 33231,'AN' from dual union all
select 33232,'MRN' from dual union all
select 33233,'AN' from dual union all
select 33234,'AN' from dual union all
select 33235,'AN' from dual union all
select 33236,'MRN' from dual union all
select 33237,'AN' from dual union all
select 33238,'MRN' from dual union all
select 33239,'MRN' from dual union all
select 3324,'PARD' from dual union all
select 33240,'AN' from dual union all
select 33241,'AN' from dual union all
select 33242,'AN' from dual union all
select 33243,'AN' from dual union all
select 33244,'AN' from dual union all
select 33245,'AN' from dual union all
select 33246,'AN' from dual union all
select 33247,'MRN' from dual union all
select 33248,'AN' from dual union all
select 33249,'MRN' from dual union all
select 3325,'USA' from dual union all
select 33250,'AN' from dual union all
select 33251,'AN' from dual union all
select 33252,'AN' from dual union all
select 33253,'MRN' from dual union all
select 33254,'AN' from dual;
insert into LAUNCH_AGENCY
select 33255,'AN' from dual union all
select 33256,'MRN' from dual union all
select 33257,'AN' from dual union all
select 33258,'AN' from dual union all
select 33259,'AN' from dual union all
select 3326,'USAF' from dual union all
select 33260,'AN' from dual union all
select 33261,'MRN' from dual union all
select 33262,'MRN' from dual union all
select 33263,'AN' from dual union all
select 33264,'MRN' from dual union all
select 33265,'MRN' from dual union all
select 33266,'AN' from dual union all
select 33267,'AN' from dual union all
select 33268,'AN' from dual union all
select 33269,'AN' from dual union all
select 3327,'PARD' from dual union all
select 33270,'AN' from dual union all
select 33271,'AN' from dual union all
select 33272,'MRN' from dual union all
select 33273,'AN' from dual union all
select 33274,'AN' from dual union all
select 33275,'AN' from dual union all
select 33276,'AN' from dual union all
select 33277,'MRN' from dual union all
select 33278,'AN' from dual union all
select 33279,'AN' from dual union all
select 3328,'USA' from dual union all
select 33280,'AN' from dual union all
select 33281,'AN' from dual union all
select 33282,'AN' from dual union all
select 33283,'AN' from dual union all
select 33284,'MRN' from dual union all
select 33285,'MRN' from dual union all
select 33286,'AN' from dual union all
select 33287,'AN' from dual union all
select 33288,'MRN' from dual union all
select 33289,'AN' from dual union all
select 3329,'6555A' from dual union all
select 33290,'AN' from dual union all
select 33291,'MRN' from dual union all
select 33292,'MRN' from dual union all
select 33293,'MRN' from dual union all
select 33294,'AN' from dual union all
select 33295,'AN' from dual union all
select 33296,'MRN' from dual union all
select 33297,'MRN' from dual union all
select 33298,'AN' from dual union all
select 33299,'NASA' from dual union all
select 333,'WEHR' from dual union all
select 3330,'SUI' from dual union all
select 33300,'USA' from dual union all
select 33301,'NASA' from dual union all
select 33302,'USA' from dual union all
select 33303,'MRN' from dual union all
select 33304,'MRN' from dual union all
select 33305,'NASA' from dual union all
select 33306,'USA' from dual union all
select 33307,'AN' from dual union all
select 33308,'AN' from dual union all
select 33309,'AN' from dual union all
select 3331,'PARD' from dual union all
select 33310,'AN' from dual union all
select 33311,'AN' from dual union all
select 33312,'AN' from dual union all
select 33313,'MRN' from dual union all
select 33314,'MRN' from dual union all
select 33315,'AN' from dual union all
select 33316,'MRN' from dual union all
select 33317,'AN' from dual union all
select 33318,'NASA' from dual union all
select 33319,'AN' from dual union all
select 3332,'NRL' from dual union all
select 33320,'MRN' from dual union all
select 33321,'MRN' from dual union all
select 33322,'AN' from dual union all
select 33323,'AN' from dual union all
select 33324,'AN' from dual union all
select 33325,'AN' from dual union all
select 33326,'AN' from dual union all
select 33327,'MRN' from dual union all
select 33328,'MRN' from dual union all
select 33329,'AN' from dual union all
select 3333,'NRL' from dual union all
select 33330,'MRN' from dual union all
select 33331,'MRN' from dual union all
select 33332,'MRN' from dual union all
select 33333,'AN' from dual union all
select 33334,'AN' from dual union all
select 33335,'AN' from dual union all
select 33336,'MRN' from dual union all
select 33337,'MRN' from dual union all
select 33338,'MRN' from dual union all
select 33339,'AN' from dual union all
select 3334,'USAF' from dual union all
select 33340,'AN' from dual union all
select 33341,'AN' from dual union all
select 33342,'MRN' from dual union all
select 33343,'AN' from dual union all
select 33344,'AN' from dual;
insert into LAUNCH_AGENCY
select 33345,'MRN' from dual union all
select 33346,'MRN' from dual union all
select 33347,'MRN' from dual union all
select 33348,'AN' from dual union all
select 33349,'AN' from dual union all
select 3335,'PARD' from dual union all
select 33350,'AN' from dual union all
select 33351,'AN' from dual union all
select 33352,'AN' from dual union all
select 33353,'AN' from dual union all
select 33354,'AN' from dual union all
select 33355,'AN' from dual union all
select 33356,'AN' from dual union all
select 33357,'MRN' from dual union all
select 33358,'AN' from dual union all
select 33359,'AN' from dual union all
select 3336,'USA' from dual union all
select 33360,'AN' from dual union all
select 33361,'AN' from dual union all
select 33362,'MRN' from dual union all
select 33363,'AN' from dual union all
select 33364,'AN' from dual union all
select 33365,'MRN' from dual union all
select 33366,'AN' from dual union all
select 33367,'AN' from dual union all
select 33368,'MRN' from dual union all
select 33369,'AN' from dual union all
select 3337,'NRL' from dual union all
select 33370,'AN' from dual union all
select 33371,'AN' from dual union all
select 33372,'AN' from dual union all
select 33373,'MRN' from dual union all
select 33374,'AN' from dual union all
select 33375,'MRN' from dual union all
select 33376,'MRN' from dual union all
select 33377,'MRN' from dual union all
select 33378,'AN' from dual union all
select 33379,'AN' from dual union all
select 3338,'PARD' from dual union all
select 33380,'AN' from dual union all
select 33381,'AN' from dual union all
select 33382,'AN' from dual union all
select 33383,'AN' from dual union all
select 33384,'AN' from dual union all
select 33385,'AN' from dual union all
select 33386,'AN' from dual union all
select 33387,'AN' from dual union all
select 33388,'MRN' from dual union all
select 33389,'MRN' from dual union all
select 3339,'PARD' from dual union all
select 3339,'USN' from dual union all
select 33390,'AN' from dual union all
select 33391,'AN' from dual union all
select 33392,'AN' from dual union all
select 33393,'AN' from dual union all
select 33394,'AN' from dual union all
select 33395,'AN' from dual union all
select 33396,'AN' from dual union all
select 33397,'AN' from dual union all
select 33398,'AN' from dual union all
select 33399,'AN' from dual union all
select 334,'WEHR' from dual union all
select 3340,'PARD' from dual union all
select 3340,'USN' from dual union all
select 33400,'AN' from dual union all
select 33401,'AN' from dual union all
select 33402,'AN' from dual union all
select 33403,'AN' from dual union all
select 33404,'AN' from dual union all
select 33405,'AN' from dual union all
select 33406,'AN' from dual union all
select 33407,'AN' from dual union all
select 33408,'AN' from dual union all
select 33409,'AN' from dual union all
select 3341,'SUI' from dual union all
select 33410,'AN' from dual union all
select 33411,'AN' from dual union all
select 33412,'AN' from dual union all
select 33413,'AN' from dual union all
select 33414,'AN' from dual union all
select 33415,'AN' from dual union all
select 33416,'AN' from dual union all
select 33417,'AN' from dual union all
select 33418,'MRN' from dual union all
select 33419,'AN' from dual union all
select 3342,'SUI' from dual union all
select 33420,'AN' from dual union all
select 33421,'MRN' from dual union all
select 33422,'AN' from dual union all
select 33423,'AN' from dual union all
select 33424,'AN' from dual union all
select 33425,'AN' from dual union all
select 33426,'MRN' from dual union all
select 33427,'MRN' from dual union all
select 33428,'AN' from dual union all
select 33429,'MRN' from dual union all
select 3343,'USAF' from dual union all
select 33430,'MRN' from dual union all
select 33431,'MRN' from dual union all
select 33432,'MRN' from dual;
insert into LAUNCH_AGENCY
select 33433,'MRN' from dual union all
select 33434,'MRN' from dual union all
select 33435,'MRN' from dual union all
select 33436,'MRN' from dual union all
select 33437,'MRN' from dual union all
select 33438,'AN' from dual union all
select 33439,'AN' from dual union all
select 3344,'USAF' from dual union all
select 33440,'AN' from dual union all
select 33441,'AN' from dual union all
select 33442,'AN' from dual union all
select 33443,'AN' from dual union all
select 33444,'AN' from dual union all
select 33445,'MRN' from dual union all
select 33446,'AN' from dual union all
select 33447,'MRN' from dual union all
select 33448,'MRN' from dual union all
select 33449,'AN' from dual union all
select 3345,'USAF' from dual union all
select 33450,'MRN' from dual union all
select 33451,'MRN' from dual union all
select 33452,'AN' from dual union all
select 33453,'AN' from dual union all
select 33454,'AN' from dual union all
select 33455,'AN' from dual union all
select 33456,'MRN' from dual union all
select 33457,'MRN' from dual union all
select 33458,'AN' from dual union all
select 33459,'MRN' from dual union all
select 3346,'PARD' from dual union all
select 33460,'MRN' from dual union all
select 33461,'MRN' from dual union all
select 33462,'AN' from dual union all
select 33463,'AN' from dual union all
select 33464,'MRN' from dual union all
select 33465,'AN' from dual union all
select 33466,'MRN' from dual union all
select 33467,'MRN' from dual union all
select 33468,'MRN' from dual union all
select 33469,'MRN' from dual union all
select 3347,'USA' from dual union all
select 33470,'AN' from dual union all
select 33471,'AN' from dual union all
select 33472,'AN' from dual union all
select 33473,'AN' from dual union all
select 33474,'AN' from dual union all
select 33475,'AN' from dual union all
select 33476,'MRN' from dual union all
select 33477,'AN' from dual union all
select 33478,'MRN' from dual union all
select 33479,'AN' from dual union all
select 3348,'USAF' from dual union all
select 33480,'AN' from dual union all
select 33481,'AN' from dual union all
select 33482,'AN' from dual union all
select 33483,'AN' from dual union all
select 33484,'AN' from dual union all
select 33485,'MRN' from dual union all
select 33486,'MRN' from dual union all
select 33487,'MRN' from dual union all
select 33488,'MRN' from dual union all
select 33489,'AN' from dual union all
select 3349,'USAF' from dual union all
select 33490,'AN' from dual union all
select 33491,'AN' from dual union all
select 33492,'AN' from dual union all
select 33493,'AN' from dual union all
select 33494,'AN' from dual union all
select 33495,'MRN' from dual union all
select 33496,'AN' from dual union all
select 33497,'AN' from dual union all
select 33498,'AN' from dual union all
select 33499,'AN' from dual union all
select 335,'WEHR' from dual union all
select 3350,'USN' from dual union all
select 33500,'AN' from dual union all
select 33501,'AN' from dual union all
select 33502,'MRN' from dual union all
select 33503,'MRN' from dual union all
select 33504,'AN' from dual union all
select 33505,'AN' from dual union all
select 33506,'AN' from dual union all
select 33507,'AN' from dual union all
select 33508,'AN' from dual union all
select 33509,'AN' from dual union all
select 3351,'USAF' from dual union all
select 33510,'AN' from dual union all
select 33511,'AN' from dual union all
select 33512,'AN' from dual union all
select 33513,'MRN' from dual union all
select 33514,'AN' from dual union all
select 33515,'AN' from dual union all
select 33516,'AN' from dual union all
select 33517,'AN' from dual union all
select 33518,'AN' from dual union all
select 33519,'AN' from dual union all
select 3352,'USAF' from dual union all
select 33520,'AN' from dual union all
select 33521,'AN' from dual union all
select 33522,'AN' from dual;
insert into LAUNCH_AGENCY
select 33523,'AN' from dual union all
select 33524,'AN' from dual union all
select 33525,'AN' from dual union all
select 33526,'AN' from dual union all
select 33527,'AN' from dual union all
select 33528,'AN' from dual union all
select 33529,'MRN' from dual union all
select 3353,'NRL' from dual union all
select 33530,'MRN' from dual union all
select 33531,'MRN' from dual union all
select 33532,'AN' from dual union all
select 33533,'AN' from dual union all
select 33534,'AN' from dual union all
select 33535,'AN' from dual union all
select 33536,'AN' from dual union all
select 33537,'AN' from dual union all
select 33538,'AN' from dual union all
select 33539,'AN' from dual union all
select 3354,'PARD' from dual union all
select 33540,'MRN' from dual union all
select 33541,'AN' from dual union all
select 33542,'AN' from dual union all
select 33543,'AN' from dual union all
select 33544,'AN' from dual union all
select 33545,'AN' from dual union all
select 33546,'AN' from dual union all
select 33547,'AN' from dual union all
select 33548,'MRN' from dual union all
select 33549,'AN' from dual union all
select 3355,'PARD' from dual union all
select 33550,'MRN' from dual union all
select 33551,'AN' from dual union all
select 33552,'AN' from dual union all
select 33553,'AN' from dual union all
select 33554,'AN' from dual union all
select 33555,'AN' from dual union all
select 33556,'AN' from dual union all
select 33557,'AN' from dual union all
select 33558,'AN' from dual union all
select 33559,'AN' from dual union all
select 3356,'USAF' from dual union all
select 33560,'MRN' from dual union all
select 33561,'MRN' from dual union all
select 33562,'MRN' from dual union all
select 33563,'AN' from dual union all
select 33564,'NASA' from dual union all
select 33565,'MRN' from dual union all
select 33566,'AN' from dual union all
select 33567,'AN' from dual union all
select 33568,'AN' from dual union all
select 33569,'AN' from dual union all
select 3357,'PARD' from dual union all
select 33570,'AN' from dual union all
select 33571,'AN' from dual union all
select 33572,'AN' from dual union all
select 33573,'MRN' from dual union all
select 33574,'MRN' from dual union all
select 33575,'AN' from dual union all
select 33576,'AN' from dual union all
select 33577,'MRN' from dual union all
select 33578,'AN' from dual union all
select 33579,'AN' from dual union all
select 3358,'USA' from dual union all
select 33580,'AN' from dual union all
select 33581,'AN' from dual union all
select 33582,'AN' from dual union all
select 33583,'MRN' from dual union all
select 33584,'MRN' from dual union all
select 33585,'MRN' from dual union all
select 33586,'AN' from dual union all
select 33587,'AN' from dual union all
select 33588,'AN' from dual union all
select 33589,'AN' from dual union all
select 3359,'USN' from dual union all
select 33590,'AN' from dual union all
select 33591,'AN' from dual union all
select 33592,'AN' from dual union all
select 33593,'MRN' from dual union all
select 33594,'AN' from dual union all
select 33595,'AN' from dual union all
select 33596,'AN' from dual union all
select 33597,'AN' from dual union all
select 33598,'AN' from dual union all
select 33599,'AN' from dual union all
select 336,'WEHR' from dual union all
select 3360,'CASDN' from dual union all
select 33600,'AN' from dual union all
select 33601,'AN' from dual union all
select 33602,'AN' from dual union all
select 33603,'AN' from dual union all
select 33604,'RVSN' from dual union all
select 33605,'NASA' from dual union all
select 33606,'RVSN' from dual union all
select 33607,'RVSN' from dual union all
select 33608,'RVSN' from dual union all
select 33609,'RVSN' from dual union all
select 3361,'STA' from dual union all
select 33610,'AFSC' from dual union all
select 33611,'ISAS' from dual union all
select 33612,'CRA' from dual;
insert into LAUNCH_AGENCY
select 33613,'RVSN' from dual union all
select 33614,'RVSN' from dual union all
select 33615,'RVSN' from dual union all
select 33616,'AFSC' from dual union all
select 33617,'RVSN' from dual union all
select 33618,'10ADS' from dual union all
select 33619,'RVSN' from dual union all
select 3362,'STA' from dual union all
select 33620,'RVSN' from dual union all
select 33621,'RVSN' from dual union all
select 33622,'RVSN' from dual union all
select 33623,'AFSC' from dual union all
select 33624,'RVSN' from dual union all
select 33625,'NASA' from dual union all
select 33626,'RVSN' from dual union all
select 33627,'RVSN' from dual union all
select 33628,'RVSN' from dual union all
select 33629,'RVSN' from dual union all
select 3363,'USA' from dual union all
select 33630,'RVSN' from dual union all
select 33631,'RVSN' from dual union all
select 33632,'RVSN' from dual union all
select 33633,'RVSN' from dual union all
select 33634,'RVSN' from dual union all
select 33635,'RVSN' from dual union all
select 33636,'NASA' from dual union all
select 33637,'RVSN' from dual union all
select 33638,'RVSN' from dual union all
select 33639,'RVSN' from dual union all
select 3364,'USA' from dual union all
select 33640,'RVSN' from dual union all
select 33641,'RVSN' from dual union all
select 33642,'AFSC' from dual union all
select 33643,'AFSC' from dual union all
select 33644,'RVSN' from dual union all
select 33645,'AFSC' from dual union all
select 33646,'RVSN' from dual union all
select 33647,'RVSN' from dual union all
select 33648,'RVSN' from dual union all
select 33649,'RVSN' from dual union all
select 3365,'USA' from dual union all
select 33650,'RVSN' from dual union all
select 33651,'RVSN' from dual union all
select 33652,'RVSN' from dual union all
select 33653,'RVSN' from dual union all
select 33654,'RVSN' from dual union all
select 33655,'RVSN' from dual union all
select 33656,'RVSN' from dual union all
select 33657,'AFSC' from dual union all
select 33658,'AFSC' from dual union all
select 33659,'RVSN' from dual union all
select 3366,'USA' from dual union all
select 33660,'RVSN' from dual union all
select 33661,'RVSN' from dual union all
select 33662,'RVSN' from dual union all
select 33663,'RVSN' from dual union all
select 33664,'RVSN' from dual union all
select 33665,'RVSN' from dual union all
select 33666,'10ADS' from dual union all
select 33667,'RVSN' from dual union all
select 33668,'AFSC' from dual union all
select 33669,'RVSN' from dual union all
select 3367,'USA' from dual union all
select 33670,'RVSN' from dual union all
select 33671,'RVSN' from dual union all
select 33672,'RVSN' from dual union all
select 33673,'AFSC' from dual union all
select 33674,'RVSN' from dual union all
select 33675,'RVSN' from dual union all
select 33676,'RVSN' from dual union all
select 33677,'RVSN' from dual union all
select 33678,'NASA' from dual union all
select 33679,'RVSN' from dual union all
select 3368,'USA' from dual union all
select 33680,'CRA' from dual union all
select 33681,'RVSN' from dual union all
select 33682,'RVSN' from dual union all
select 33683,'RVSN' from dual union all
select 33684,'RVSN' from dual union all
select 33685,'RVSN' from dual union all
select 33686,'RVSN' from dual union all
select 33687,'RVSN' from dual union all
select 33688,'AFSC' from dual union all
select 33689,'RVSN' from dual union all
select 3369,'ORDRSA' from dual union all
select 33690,'RVSN' from dual union all
select 33691,'RVSN' from dual union all
select 33692,'NASA' from dual union all
select 33693,'RVSN' from dual union all
select 33694,'RVSN' from dual union all
select 33695,'RVSN' from dual union all
select 33696,'NASA' from dual union all
select 33697,'NASA' from dual union all
select 33698,'RVSN' from dual union all
select 33699,'RVSN' from dual union all
select 337,'WEHR' from dual union all
select 3370,'USA' from dual union all
select 33700,'USAF' from dual union all
select 33701,'RVSN' from dual union all
select 33702,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 33703,'RVSN' from dual union all
select 33704,'NASA' from dual union all
select 33705,'RVSN' from dual union all
select 33706,'RVSN' from dual union all
select 33707,'RVSN' from dual union all
select 33708,'RVSN' from dual union all
select 33709,'RVSN' from dual union all
select 3371,'USA' from dual union all
select 33710,'SAND' from dual union all
select 33711,'MRN' from dual union all
select 33712,'USA' from dual union all
select 33713,'SAND' from dual union all
select 33714,'MRN' from dual union all
select 33715,'MRN' from dual union all
select 33716,'RVSN' from dual union all
select 33717,'USA' from dual union all
select 33718,'AFCRL' from dual union all
select 33719,'MRN' from dual union all
select 3372,'USA' from dual union all
select 33720,'NASA' from dual union all
select 33721,'USN' from dual union all
select 33722,'MRN' from dual union all
select 33723,'NASA' from dual union all
select 33724,'MRN' from dual union all
select 33725,'USN' from dual union all
select 33726,'USA' from dual union all
select 33727,'RVSN' from dual union all
select 33728,'MRN' from dual union all
select 33729,'MRN' from dual union all
select 3373,'USA' from dual union all
select 33730,'MRN' from dual union all
select 33731,'MRN' from dual union all
select 33732,'NASA' from dual union all
select 33733,'USN' from dual union all
select 33734,'81FA1' from dual union all
select 33735,'MRN' from dual union all
select 33736,'AFCRL' from dual union all
select 33737,'AFCRL' from dual union all
select 33738,'AFCRL' from dual union all
select 33739,'MRN' from dual union all
select 3374,'USA' from dual union all
select 33740,'USA' from dual union all
select 33741,'USAF' from dual union all
select 33742,'MRN' from dual union all
select 33743,'MRN' from dual union all
select 33744,'RVSN' from dual union all
select 33745,'RAE' from dual union all
select 33746,'MRN' from dual union all
select 33747,'SAND' from dual union all
select 33748,'MRN' from dual union all
select 33749,'USN' from dual union all
select 3375,'USA' from dual union all
select 33750,'USA' from dual union all
select 33751,'RVSN' from dual union all
select 33752,'MRN' from dual union all
select 33753,'USN' from dual union all
select 33754,'USA' from dual union all
select 33755,'MRN' from dual union all
select 33756,'AFSC' from dual union all
select 33757,'MRN' from dual union all
select 33758,'MRN' from dual union all
select 33759,'RAE' from dual union all
select 3376,'USA' from dual union all
select 33760,'SAND' from dual union all
select 33761,'MRN' from dual union all
select 33762,'RVSN' from dual union all
select 33763,'RAE' from dual union all
select 33764,'MRN' from dual union all
select 33765,'MRN' from dual union all
select 33766,'DFVLR' from dual union all
select 33767,'NRL' from dual union all
select 33768,'RVSN' from dual union all
select 33769,'SAND' from dual union all
select 3377,'USA' from dual union all
select 33770,'NRL' from dual union all
select 33771,'SAND' from dual union all
select 33772,'SAND' from dual union all
select 33773,'SAND' from dual union all
select 33774,'SAND' from dual union all
select 33775,'SAND' from dual union all
select 33776,'SAND' from dual union all
select 33777,'SAND' from dual union all
select 33778,'SAND' from dual union all
select 33779,'NASA' from dual union all
select 3378,'USA' from dual union all
select 33780,'NASA' from dual union all
select 33781,'NASA' from dual union all
select 33782,'SAND' from dual union all
select 33783,'SAND' from dual union all
select 33784,'SAND' from dual union all
select 33785,'SAND' from dual union all
select 33786,'SAND' from dual union all
select 33787,'USAF' from dual union all
select 33788,'RVSN' from dual union all
select 33789,'RVSN' from dual union all
select 3379,'USA' from dual union all
select 33790,'RVSN' from dual union all
select 33791,'SAST' from dual union all
select 33792,'RVSN' from dual union all
select 33793,'CALT' from dual;
insert into LAUNCH_AGENCY
select 33794,'USA' from dual union all
select 33795,'USA' from dual union all
select 33796,'I' from dual union all
select 33797,'NASA' from dual union all
select 33798,'WFC' from dual union all
select 33799,'CNES' from dual union all
select 338,'WEHR' from dual union all
select 3380,'ORDRSA' from dual union all
select 33800,'CNES' from dual union all
select 33801,'CNES' from dual union all
select 33802,'CNES' from dual union all
select 33803,'CNES' from dual union all
select 33804,'CNES' from dual union all
select 33805,'CNES' from dual union all
select 33806,'CNES' from dual union all
select 33807,'CNES' from dual union all
select 33808,'CNES' from dual union all
select 33809,'CNES' from dual union all
select 3381,'USA' from dual union all
select 33810,'CNES' from dual union all
select 33811,'CNES' from dual union all
select 33812,'CNES' from dual union all
select 33813,'CNES' from dual union all
select 33814,'CNES' from dual union all
select 33815,'CNES' from dual union all
select 33816,'CNES' from dual union all
select 33817,'CNES' from dual union all
select 33818,'CNES' from dual union all
select 33819,'CNES' from dual union all
select 3382,'USA' from dual union all
select 33820,'CNES' from dual union all
select 33821,'CNES' from dual union all
select 33822,'INTA' from dual union all
select 33823,'INTA' from dual union all
select 33824,'INTA' from dual union all
select 33825,'INTA' from dual union all
select 33826,'INTA' from dual union all
select 33827,'INTA' from dual union all
select 33828,'INTA' from dual union all
select 33829,'INTA' from dual union all
select 3383,'USA' from dual union all
select 33830,'INTA' from dual union all
select 33831,'USA' from dual union all
select 33832,'NASA' from dual union all
select 33833,'IIAE' from dual union all
select 33834,'IIAE' from dual union all
select 33835,'BWEHR' from dual union all
select 33836,'WRE' from dual union all
select 33837,'WRE' from dual union all
select 33838,'RVSN' from dual union all
select 33839,'RVSN' from dual union all
select 3384,'USA' from dual union all
select 33840,'RVSN' from dual union all
select 33841,'RVSN' from dual union all
select 33842,'RVSN' from dual union all
select 33843,'RVSN' from dual union all
select 33844,'RVSN' from dual union all
select 33845,'RVSN' from dual union all
select 33846,'NRCC' from dual union all
select 33847,'AFCRL' from dual union all
select 33848,'AN' from dual union all
select 33849,'AFCRL' from dual union all
select 3385,'USA' from dual union all
select 33850,'AFCRL' from dual union all
select 33851,'RVSN' from dual union all
select 33852,'AFCRL' from dual union all
select 33852,'DNA' from dual union all
select 33853,'ESTEC' from dual union all
select 33854,'SSC' from dual union all
select 33855,'ESTEC' from dual union all
select 33856,'RVSN' from dual union all
select 33857,'SSC' from dual union all
select 33858,'RVSN' from dual union all
select 33859,'RVSN' from dual union all
select 3386,'USA' from dual union all
select 33860,'AFCRL' from dual union all
select 33861,'NRCC' from dual union all
select 33862,'NASA' from dual union all
select 33863,'RVSN' from dual union all
select 33864,'AN' from dual union all
select 33865,'AFCRL' from dual union all
select 33866,'RVSN' from dual union all
select 33867,'RVSN' from dual union all
select 33868,'NASA' from dual union all
select 33869,'NRCC' from dual union all
select 3387,'USA' from dual union all
select 33870,'RVSN' from dual union all
select 33871,'SAC' from dual union all
select 33872,'NASA' from dual union all
select 33873,'AN' from dual union all
select 33874,'NASA' from dual union all
select 33875,'AFSC' from dual union all
select 33876,'SAMSO' from dual union all
select 33877,'NASA' from dual union all
select 33878,'RVSN' from dual union all
select 33879,'ALCS' from dual union all
select 33879,'SAC' from dual union all
select 3388,'USA' from dual union all
select 33880,'AN' from dual union all
select 33881,'AN' from dual;
insert into LAUNCH_AGENCY
select 33882,'AN' from dual union all
select 33883,'AN' from dual union all
select 33884,'AN' from dual union all
select 33885,'NASA' from dual union all
select 33886,'SAND' from dual union all
select 33887,'SAND' from dual union all
select 33888,'NRL' from dual union all
select 33889,'RVSN' from dual union all
select 3389,'USA' from dual union all
select 33890,'AN' from dual union all
select 33891,'AN' from dual union all
select 33892,'NASA' from dual union all
select 33893,'NASA' from dual union all
select 33894,'NASA' from dual union all
select 33895,'AN' from dual union all
select 33896,'NASA' from dual union all
select 33897,'NASA' from dual union all
select 33898,'AFSC' from dual union all
select 33899,'SAND' from dual union all
select 339,'WEHR' from dual union all
select 3390,'USA' from dual union all
select 33900,'RVSN' from dual union all
select 33901,'SAMSO' from dual union all
select 33902,'SAC' from dual union all
select 33903,'AFCRL' from dual union all
select 33904,'AERO' from dual union all
select 33905,'AN' from dual union all
select 33906,'RVSN' from dual union all
select 33907,'SRC' from dual union all
select 33908,'SRC' from dual union all
select 33909,'ALCS' from dual union all
select 33909,'SAC' from dual union all
select 3391,'USA' from dual union all
select 33910,'RVSN' from dual union all
select 33911,'RVSN' from dual union all
select 33912,'AFCRL' from dual union all
select 33913,'SRC' from dual union all
select 33914,'SRC' from dual union all
select 33915,'AN' from dual union all
select 33916,'AFCRL' from dual union all
select 33917,'AFSC' from dual union all
select 33918,'AN' from dual union all
select 33919,'NASA' from dual union all
select 3392,'USA' from dual union all
select 33920,'AFCRL' from dual union all
select 33921,'AFCRL' from dual union all
select 33921,'DNA' from dual union all
select 33922,'RVSN' from dual union all
select 33923,'AFCRL' from dual union all
select 33924,'USN' from dual union all
select 33925,'AFCRL' from dual union all
select 33926,'AFSC' from dual union all
select 33927,'RVSN' from dual union all
select 33928,'NASA' from dual union all
select 33929,'AFCRL' from dual union all
select 3393,'USA' from dual union all
select 33930,'NASA' from dual union all
select 33931,'AFGL' from dual union all
select 33932,'BAC' from dual union all
select 33933,'RVSN' from dual union all
select 33934,'ISRO' from dual union all
select 33935,'AFSC' from dual union all
select 33936,'RVSN' from dual union all
select 33937,'RVSN' from dual union all
select 33938,'USA' from dual union all
select 33939,'SRC' from dual union all
select 3394,'USA' from dual union all
select 33940,'AFSC' from dual union all
select 33941,'SAC' from dual union all
select 33942,'DLR' from dual union all
select 33943,'81FA1' from dual union all
select 33944,'RVSN' from dual union all
select 33945,'81FA1' from dual union all
select 33946,'81FA1' from dual union all
select 33947,'81FA1' from dual union all
select 33948,'NASA' from dual union all
select 33949,'NASA' from dual union all
select 3395,'USA' from dual union all
select 33950,'NASA' from dual union all
select 33951,'NASA' from dual union all
select 33952,'USN' from dual union all
select 33953,'USN' from dual union all
select 33954,'SAMSO' from dual union all
select 33955,'RVSN' from dual union all
select 33956,'84FA3' from dual union all
select 33957,'84FA3' from dual union all
select 33958,'84FA3' from dual union all
select 33959,'84FA3' from dual union all
select 3396,'USA' from dual union all
select 33960,'RVSN' from dual union all
select 33961,'SAMSO' from dual union all
select 33962,'RVSN' from dual union all
select 33963,'USA' from dual union all
select 33964,'USN' from dual union all
select 33965,'81FA1' from dual union all
select 33966,'RVSN' from dual union all
select 33967,'81FA1' from dual union all
select 33968,'81FA1' from dual union all
select 33969,'USN' from dual union all
select 3397,'USA' from dual;
insert into LAUNCH_AGENCY
select 33970,'USN' from dual union all
select 33971,'USN' from dual union all
select 33972,'USN' from dual union all
select 33973,'USN' from dual union all
select 33974,'DMA' from dual union all
select 33975,'SAMSO' from dual union all
select 33976,'NASA' from dual union all
select 33977,'RVSN' from dual union all
select 33978,'DASA' from dual union all
select 33979,'DMA' from dual union all
select 3398,'USA' from dual union all
select 33980,'BAC' from dual union all
select 33981,'NRL' from dual union all
select 33982,'NASA' from dual union all
select 33983,'SAND' from dual union all
select 33984,'SAND' from dual union all
select 33985,'AN' from dual union all
select 33986,'AN' from dual union all
select 33987,'RVSN' from dual union all
select 33988,'RVSN' from dual union all
select 33989,'AN' from dual union all
select 3399,'USA' from dual union all
select 33990,'NASA' from dual union all
select 33991,'SAMSO' from dual union all
select 33992,'RVSN' from dual union all
select 33993,'NASA' from dual union all
select 33994,'AFSC' from dual union all
select 33995,'AFCRL' from dual union all
select 33996,'AFCRL' from dual union all
select 33997,'AFCRL' from dual union all
select 33998,'RVSN' from dual union all
select 33999,'NASA' from dual union all
select 34,'WEHR' from dual union all
select 340,'WER444' from dual union all
select 3400,'USA' from dual union all
select 34000,'AFCRL' from dual union all
select 34001,'AFCRL' from dual union all
select 34002,'AFCRL' from dual union all
select 34003,'DNA' from dual union all
select 34004,'RVSN' from dual union all
select 34005,'AFCRL' from dual union all
select 34006,'NASA' from dual union all
select 34007,'AFCRL' from dual union all
select 34008,'NASA' from dual union all
select 34009,'RVSN' from dual union all
select 3401,'USA' from dual union all
select 34010,'NASA' from dual union all
select 34011,'AFCRL' from dual union all
select 34012,'AFCRL' from dual union all
select 34013,'AFCRL' from dual union all
select 34014,'AFCRL' from dual union all
select 34015,'AFCRL' from dual union all
select 34016,'NASA' from dual union all
select 34017,'NASA' from dual union all
select 34018,'AFCRL' from dual union all
select 34019,'SAND' from dual union all
select 3402,'USA' from dual union all
select 34020,'RVSN' from dual union all
select 34021,'AFCRL' from dual union all
select 34022,'AFCRL' from dual union all
select 34023,'NASA' from dual union all
select 34024,'RVSN' from dual union all
select 34025,'NASA' from dual union all
select 34026,'NASA' from dual union all
select 34027,'NASA' from dual union all
select 34028,'SAC' from dual union all
select 34029,'AFSC' from dual union all
select 3403,'USA' from dual union all
select 34030,'RN' from dual union all
select 34031,'USAF' from dual union all
select 34032,'AFSC' from dual union all
select 34033,'USA' from dual union all
select 34034,'SAMSO' from dual union all
select 34035,'NASA' from dual union all
select 34036,'BAC' from dual union all
select 34037,'RVSN' from dual union all
select 34038,'USN' from dual union all
select 34039,'AFSC' from dual union all
select 3404,'USA' from dual union all
select 34040,'USN' from dual union all
select 34041,'NASA' from dual union all
select 34042,'USAF' from dual union all
select 34043,'SAC' from dual union all
select 34044,'AFCRL' from dual union all
select 34045,'NTNF' from dual union all
select 34046,'RVSN' from dual union all
select 34047,'ISAS' from dual union all
select 34048,'USN' from dual union all
select 34049,'RVSN' from dual union all
select 3405,'WADD' from dual union all
select 34050,'RVSN' from dual union all
select 34051,'ISAS' from dual union all
select 34052,'VMF' from dual union all
select 34053,'NASDA' from dual union all
select 34054,'SRC' from dual union all
select 34055,'AFCRL' from dual union all
select 34056,'SRC' from dual union all
select 34057,'NASA' from dual union all
select 34058,'AFSC' from dual union all
select 34059,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 3406,'USA' from dual union all
select 34060,'SRC' from dual union all
select 34061,'AFCRL' from dual union all
select 34062,'CNES' from dual union all
select 34062,'ESTEC' from dual union all
select 34063,'USAF' from dual union all
select 34064,'CNES' from dual union all
select 34064,'ESTEC' from dual union all
select 34065,'CNES' from dual union all
select 34065,'ESTEC' from dual union all
select 34066,'RVSN' from dual union all
select 34067,'ISAS' from dual union all
select 34068,'AFCRL' from dual union all
select 34069,'NASA' from dual union all
select 3407,'USA' from dual union all
select 34070,'ISAS' from dual union all
select 34071,'AN' from dual union all
select 34072,'AN' from dual union all
select 34073,'AN' from dual union all
select 34074,'RVSN' from dual union all
select 34075,'AN' from dual union all
select 34076,'USA' from dual union all
select 34077,'AN' from dual union all
select 34078,'AN' from dual union all
select 34079,'AN' from dual union all
select 3408,'USA' from dual union all
select 34080,'AN' from dual union all
select 34081,'AN' from dual union all
select 34082,'ISAS' from dual union all
select 34083,'AN' from dual union all
select 34084,'AN' from dual union all
select 34085,'RVSN' from dual union all
select 34086,'AN' from dual union all
select 34087,'AN' from dual union all
select 34088,'RVSN' from dual union all
select 34089,'AN' from dual union all
select 3409,'PARD' from dual union all
select 34090,'AN' from dual union all
select 34091,'NASA' from dual union all
select 34092,'SRC' from dual union all
select 34093,'RVSN' from dual union all
select 34094,'SRC' from dual union all
select 34095,'SAC' from dual union all
select 34096,'RVSN' from dual union all
select 34097,'SRC' from dual union all
select 34098,'9FA3' from dual union all
select 34099,'9FA3' from dual union all
select 341,'WEHR' from dual union all
select 3410,'USA' from dual union all
select 34100,'US7A' from dual union all
select 34101,'US7A' from dual union all
select 34102,'NASA' from dual union all
select 34103,'AFSC' from dual union all
select 34104,'BAC' from dual union all
select 34105,'NASA' from dual union all
select 34106,'SAC' from dual union all
select 34107,'RVSN' from dual union all
select 34108,'RVSN' from dual union all
select 34109,'DFVLR' from dual union all
select 3411,'USA' from dual union all
select 34110,'INTA' from dual union all
select 34111,'BAC' from dual union all
select 34112,'NASA' from dual union all
select 34113,'AFSC' from dual union all
select 34114,'AFSC' from dual union all
select 34115,'AFSC' from dual union all
select 34116,'USN' from dual union all
select 34117,'USN' from dual union all
select 34118,'RVSN' from dual union all
select 34119,'USN' from dual union all
select 3412,'USA' from dual union all
select 34120,'USN' from dual union all
select 34121,'USN' from dual union all
select 34122,'AN' from dual union all
select 34123,'AN' from dual union all
select 34124,'AN' from dual union all
select 34125,'RVSN' from dual union all
select 34126,'AFGL' from dual union all
select 34127,'NASA' from dual union all
select 34128,'RVSN' from dual union all
select 34129,'RVSN' from dual union all
select 3413,'USA' from dual union all
select 34130,'ALCS' from dual union all
select 34130,'SAC' from dual union all
select 34131,'USN' from dual union all
select 34132,'USN' from dual union all
select 34133,'USN' from dual union all
select 34134,'USN' from dual union all
select 34135,'RVSN' from dual union all
select 34136,'RVSN' from dual union all
select 34137,'ALCS' from dual union all
select 34137,'SAC' from dual union all
select 34138,'FKG2' from dual union all
select 34139,'FKG2' from dual union all
select 3414,'USA' from dual union all
select 34140,'RVSN' from dual union all
select 34141,'USA' from dual union all
select 34142,'BAC' from dual union all
select 34143,'SAND' from dual union all
select 34144,'NASA' from dual;
insert into LAUNCH_AGENCY
select 34145,'SAND' from dual union all
select 34146,'RVSN' from dual union all
select 34147,'AN' from dual union all
select 34148,'AN' from dual union all
select 34149,'AN' from dual union all
select 3415,'USA' from dual union all
select 34150,'NASA' from dual union all
select 34151,'RVSN' from dual union all
select 34152,'NASA' from dual union all
select 34153,'CNES' from dual union all
select 34154,'AN' from dual union all
select 34155,'USN' from dual union all
select 34156,'USN' from dual union all
select 34157,'USN' from dual union all
select 34158,'USN' from dual union all
select 34159,'NASA' from dual union all
select 3416,'USA' from dual union all
select 34160,'NASA' from dual union all
select 34161,'NASA' from dual union all
select 34162,'RVSN' from dual union all
select 34163,'AFSC' from dual union all
select 34164,'NASA' from dual union all
select 34165,'BAC' from dual union all
select 34166,'SRC' from dual union all
select 34167,'SRC' from dual union all
select 34168,'SRC' from dual union all
select 34169,'FKG2' from dual union all
select 3417,'USA' from dual union all
select 34170,'FKG2' from dual union all
select 34171,'RVSN' from dual union all
select 34172,'SRC' from dual union all
select 34173,'RVSN' from dual union all
select 34174,'ALCS' from dual union all
select 34174,'SAC' from dual union all
select 34175,'NASA' from dual union all
select 34176,'NRCC' from dual union all
select 34177,'USAF' from dual union all
select 34178,'SRC' from dual union all
select 34179,'SRC' from dual union all
select 3418,'USA' from dual union all
select 34180,'SRC' from dual union all
select 34181,'SRC' from dual union all
select 34182,'RVSN' from dual union all
select 34183,'NRCC' from dual union all
select 34184,'RVSN' from dual union all
select 34185,'RVSN' from dual union all
select 34186,'RVSN' from dual union all
select 34187,'NASA' from dual union all
select 34187,'NDRE' from dual union all
select 34188,'AFCRL' from dual union all
select 34189,'USN' from dual union all
select 3419,'USA' from dual union all
select 34190,'RVSN' from dual union all
select 34191,'SAC' from dual union all
select 34192,'RVSN' from dual union all
select 34193,'DFVLR' from dual union all
select 34193,'ESTEC' from dual union all
select 34194,'SRC' from dual union all
select 34195,'RVSN' from dual union all
select 34196,'SRC' from dual union all
select 34197,'SAND' from dual union all
select 34198,'SRC' from dual union all
select 34199,'SRC' from dual union all
select 342,'WEHR' from dual union all
select 3420,'ORDRSA' from dual union all
select 34200,'AN' from dual union all
select 34201,'RVSN' from dual union all
select 34202,'AN' from dual union all
select 34203,'RVSN' from dual union all
select 34204,'NASA' from dual union all
select 34205,'NRL' from dual union all
select 34206,'RVSN' from dual union all
select 34207,'PLA2AC' from dual union all
select 34208,'CALT' from dual union all
select 34209,'CALT' from dual union all
select 3421,'USA' from dual union all
select 34210,'PLA2AC' from dual union all
select 34211,'PLA2AC' from dual union all
select 34212,'RVSN' from dual union all
select 34213,'VMF' from dual union all
select 34214,'VMF' from dual union all
select 34215,'WRE' from dual union all
select 34216,'WRE' from dual union all
select 34217,'RVSN' from dual union all
select 34218,'WRE' from dual union all
select 34219,'USAF' from dual union all
select 3422,'USA' from dual union all
select 34220,'USAF' from dual union all
select 34221,'USAF' from dual union all
select 34222,'RVSN' from dual union all
select 34223,'RVSN' from dual union all
select 34224,'RVSN' from dual union all
select 34225,'RVSN' from dual union all
select 34226,'RVSN' from dual union all
select 34227,'RVSN' from dual union all
select 34228,'RVSN' from dual union all
select 34229,'RVSN' from dual union all
select 3423,'USN' from dual union all
select 34230,'RVSN' from dual union all
select 34231,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 34232,'RVSN' from dual union all
select 34233,'RVSN' from dual union all
select 34234,'RVSN' from dual union all
select 34235,'RVSN' from dual union all
select 34236,'RVSN' from dual union all
select 34237,'RVSN' from dual union all
select 34238,'RVSN' from dual union all
select 34239,'RVSN' from dual union all
select 3424,'ORDRSA' from dual union all
select 34240,'RVSN' from dual union all
select 34241,'RVSN' from dual union all
select 34242,'RVSN' from dual union all
select 34243,'RVSN' from dual union all
select 34244,'RVSN' from dual union all
select 34245,'RVSN' from dual union all
select 34246,'RVSN' from dual union all
select 34247,'RVSN' from dual union all
select 34248,'RVSN' from dual union all
select 34249,'RVSN' from dual union all
select 3425,'ORDRSA' from dual union all
select 34250,'RVSN' from dual union all
select 34251,'RVSN' from dual union all
select 34252,'RVSN' from dual union all
select 34253,'RVSN' from dual union all
select 34254,'RVSN' from dual union all
select 34255,'RVSN' from dual union all
select 34256,'RVSN' from dual union all
select 34257,'RVSN' from dual union all
select 34258,'RVSN' from dual union all
select 34259,'RVSN' from dual union all
select 3426,'USAF' from dual union all
select 34260,'RVSN' from dual union all
select 34261,'RVSN' from dual union all
select 34262,'RVSN' from dual union all
select 34263,'RVSN' from dual union all
select 34264,'RVSN' from dual union all
select 34265,'RVSN' from dual union all
select 34266,'RVSN' from dual union all
select 34267,'RVSN' from dual union all
select 34268,'RVSN' from dual union all
select 34269,'RVSN' from dual union all
select 3427,'ORDRSA' from dual union all
select 34270,'RVSN' from dual union all
select 34271,'RVSN' from dual union all
select 34272,'RVSN' from dual union all
select 34273,'RVSN' from dual union all
select 34274,'RVSN' from dual union all
select 34275,'RVSN' from dual union all
select 34276,'RVSN' from dual union all
select 34277,'RVSN' from dual union all
select 34278,'USAF' from dual union all
select 34279,'USAF' from dual union all
select 3428,'USAF' from dual union all
select 34280,'USAF' from dual union all
select 34281,'MVS' from dual union all
select 34282,'RVSN' from dual union all
select 34283,'MVS' from dual union all
select 34284,'MVS' from dual union all
select 34285,'MVS' from dual union all
select 34286,'MVS' from dual union all
select 34287,'MVS' from dual union all
select 34288,'MVS' from dual union all
select 34289,'MVS' from dual union all
select 3429,'USAF' from dual union all
select 34290,'MVS' from dual union all
select 34291,'WRE' from dual union all
select 34292,'RVSN' from dual union all
select 34293,'NASA' from dual union all
select 34294,'RVSN' from dual union all
select 34295,'RVSN' from dual union all
select 34296,'RVSN' from dual union all
select 34297,'RVSN' from dual union all
select 34298,'RVSN' from dual union all
select 34299,'RVSN' from dual union all
select 343,'WEHR' from dual union all
select 3430,'USAF' from dual union all
select 34300,'RVSN' from dual union all
select 34301,'RVSN' from dual union all
select 34302,'RVSN' from dual union all
select 34303,'RVSN' from dual union all
select 34304,'NASA' from dual union all
select 34305,'RVSN' from dual union all
select 34306,'RVSN' from dual union all
select 34307,'RVSN' from dual union all
select 34308,'RVSN' from dual union all
select 34309,'RVSN' from dual union all
select 3431,'ORDRSA' from dual union all
select 34310,'RVSN' from dual union all
select 34311,'RVSN' from dual union all
select 34312,'RVSN' from dual union all
select 34313,'MVS' from dual union all
select 34314,'MVS' from dual union all
select 34315,'DLR' from dual union all
select 34315,'INTA' from dual union all
select 34316,'MVS' from dual union all
select 34317,'RVSN' from dual union all
select 34318,'RVSN' from dual union all
select 34319,'RVSN' from dual union all
select 3432,'USAF' from dual union all
select 34320,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 34321,'RVSN' from dual union all
select 34322,'RVSN' from dual union all
select 34323,'AN' from dual union all
select 34324,'DLR' from dual union all
select 34324,'INTA' from dual union all
select 34325,'RVSN' from dual union all
select 34326,'DFVLR' from dual union all
select 34327,'NASA' from dual union all
select 34328,'DLR' from dual union all
select 34328,'INTA' from dual union all
select 34329,'NASA' from dual union all
select 3433,'USAF' from dual union all
select 34330,'NASA' from dual union all
select 34331,'ISAS' from dual union all
select 34332,'DLR' from dual union all
select 34332,'INTA' from dual union all
select 34333,'NASA' from dual union all
select 34334,'NASA' from dual union all
select 34335,'CNES' from dual union all
select 34336,'DLR' from dual union all
select 34336,'INTA' from dual union all
select 34337,'RVSN' from dual union all
select 34338,'NASA' from dual union all
select 34339,'DLR' from dual union all
select 34339,'INTA' from dual union all
select 3434,'USAF' from dual union all
select 34340,'AFSC' from dual union all
select 34341,'NASA' from dual union all
select 34342,'ISAS' from dual union all
select 34343,'NASA' from dual union all
select 34344,'NRCC' from dual union all
select 34345,'DLR' from dual union all
select 34345,'INTA' from dual union all
select 34346,'RVSN' from dual union all
select 34347,'DLR' from dual union all
select 34347,'INTA' from dual union all
select 34348,'SAC' from dual union all
select 34349,'RVSN' from dual union all
select 3435,'ARDC' from dual union all
select 34350,'NASA' from dual union all
select 34350,'NDRE' from dual union all
select 34351,'DMA' from dual union all
select 34352,'VMF' from dual union all
select 34353,'VMF' from dual union all
select 34354,'RVSN' from dual union all
select 34355,'NASDA' from dual union all
select 34356,'RVSN' from dual union all
select 34357,'BAC' from dual union all
select 34358,'USN' from dual union all
select 34359,'USN' from dual union all
select 3436,'ARDC' from dual union all
select 34360,'USN' from dual union all
select 34361,'USN' from dual union all
select 34362,'USN' from dual union all
select 34363,'DFVLR' from dual union all
select 34363,'ESTEC' from dual union all
select 34364,'AFCRL' from dual union all
select 34365,'NASA' from dual union all
select 34366,'NASDA' from dual union all
select 34367,'NASA' from dual union all
select 34368,'SAMSO' from dual union all
select 34369,'AN' from dual union all
select 34369,'CNES' from dual union all
select 3437,'ARDC' from dual union all
select 34370,'MRN' from dual union all
select 34371,'MRN' from dual union all
select 34372,'MRN' from dual union all
select 34373,'MRN' from dual union all
select 34374,'MRN' from dual union all
select 34375,'MRN' from dual union all
select 34376,'MRN' from dual union all
select 34377,'MRN' from dual union all
select 34378,'MRN' from dual union all
select 34379,'MRN' from dual union all
select 3438,'ARDC' from dual union all
select 34380,'MRN' from dual union all
select 34381,'MRN' from dual union all
select 34382,'MRN' from dual union all
select 34383,'MRN' from dual union all
select 34384,'MRN' from dual union all
select 34385,'MRN' from dual union all
select 34386,'MRN' from dual union all
select 34387,'MRN' from dual union all
select 34388,'MRN' from dual union all
select 34389,'MRN' from dual union all
select 3439,'ARDC' from dual union all
select 34390,'MRN' from dual union all
select 34391,'MRN' from dual union all
select 34392,'MRN' from dual union all
select 34393,'MRN' from dual union all
select 34394,'MRN' from dual union all
select 34395,'MRN' from dual union all
select 34396,'MRN' from dual union all
select 34397,'MRN' from dual union all
select 34398,'MRN' from dual union all
select 34399,'MRN' from dual union all
select 344,'WEHR' from dual union all
select 3440,'ARDC' from dual union all
select 34400,'MRN' from dual union all
select 34401,'MRN' from dual;
insert into LAUNCH_AGENCY
select 34402,'MRN' from dual union all
select 34403,'MRN' from dual union all
select 34404,'MRN' from dual union all
select 34405,'MRN' from dual union all
select 34406,'MRN' from dual union all
select 34407,'MRN' from dual union all
select 34408,'MRN' from dual union all
select 34409,'MRN' from dual union all
select 3441,'ARDC' from dual union all
select 34410,'MRN' from dual union all
select 34411,'MRN' from dual union all
select 34412,'MRN' from dual union all
select 34413,'MRN' from dual union all
select 34414,'MRN' from dual union all
select 34415,'MRN' from dual union all
select 34416,'MRN' from dual union all
select 34417,'MRN' from dual union all
select 34418,'MRN' from dual union all
select 34419,'MRN' from dual union all
select 3442,'USA' from dual union all
select 34420,'MRN' from dual union all
select 34421,'MRN' from dual union all
select 34422,'MRN' from dual union all
select 34423,'MRN' from dual union all
select 34424,'MRN' from dual union all
select 34425,'MRN' from dual union all
select 34426,'MRN' from dual union all
select 34427,'MRN' from dual union all
select 34428,'MRN' from dual union all
select 34429,'MRN' from dual union all
select 3443,'ARDC' from dual union all
select 34430,'MRN' from dual union all
select 34431,'MRN' from dual union all
select 34432,'MRN' from dual union all
select 34433,'MRN' from dual union all
select 34434,'MRN' from dual union all
select 34435,'MRN' from dual union all
select 34436,'MRN' from dual union all
select 34437,'MRN' from dual union all
select 34438,'MRN' from dual union all
select 34439,'MRN' from dual union all
select 3444,'ARDC' from dual union all
select 34440,'MRN' from dual union all
select 34441,'MRN' from dual union all
select 34442,'MRN' from dual union all
select 34443,'MRN' from dual union all
select 34444,'MRN' from dual union all
select 34445,'MRN' from dual union all
select 34446,'MRN' from dual union all
select 34447,'MRN' from dual union all
select 34448,'MRN' from dual union all
select 34449,'MRN' from dual union all
select 3445,'USA' from dual union all
select 34450,'MRN' from dual union all
select 34451,'MRN' from dual union all
select 34452,'MRN' from dual union all
select 34453,'MRN' from dual union all
select 34454,'MRN' from dual union all
select 34455,'MRN' from dual union all
select 34456,'MRN' from dual union all
select 34457,'MRN' from dual union all
select 34458,'MRN' from dual union all
select 34459,'MRN' from dual union all
select 3446,'USA' from dual union all
select 34460,'MRN' from dual union all
select 34461,'MRN' from dual union all
select 34462,'MRN' from dual union all
select 34463,'MRN' from dual union all
select 34464,'MRN' from dual union all
select 34465,'MRN' from dual union all
select 34466,'MRN' from dual union all
select 34467,'MRN' from dual union all
select 34468,'MRN' from dual union all
select 34469,'MRN' from dual union all
select 3447,'USA' from dual union all
select 34470,'MRN' from dual union all
select 34471,'MRN' from dual union all
select 34472,'MRN' from dual union all
select 34473,'MRN' from dual union all
select 34474,'MRN' from dual union all
select 34475,'MRN' from dual union all
select 34476,'MRN' from dual union all
select 34477,'MRN' from dual union all
select 34478,'MRN' from dual union all
select 34479,'MRN' from dual union all
select 3448,'USA' from dual union all
select 34480,'MRN' from dual union all
select 34481,'MRN' from dual union all
select 34482,'MRN' from dual union all
select 34483,'MRN' from dual union all
select 34484,'MRN' from dual union all
select 34485,'MRN' from dual union all
select 34486,'MRN' from dual union all
select 34487,'MRN' from dual union all
select 34488,'MRN' from dual union all
select 34489,'MRN' from dual union all
select 3449,'USA' from dual union all
select 34490,'MRN' from dual union all
select 34491,'MRN' from dual union all
select 34492,'MRN' from dual;
insert into LAUNCH_AGENCY
select 34493,'MRN' from dual union all
select 34494,'MRN' from dual union all
select 34495,'MRN' from dual union all
select 34496,'MRN' from dual union all
select 34497,'MRN' from dual union all
select 34498,'MRN' from dual union all
select 34499,'MRN' from dual union all
select 345,'WEHR' from dual union all
select 3450,'USA' from dual union all
select 34500,'MRN' from dual union all
select 34501,'MRN' from dual union all
select 34502,'MRN' from dual union all
select 34503,'MRN' from dual union all
select 34504,'MRN' from dual union all
select 34505,'MRN' from dual union all
select 34506,'MRN' from dual union all
select 34507,'MRN' from dual union all
select 34508,'MRN' from dual union all
select 34509,'MRN' from dual union all
select 3451,'USA' from dual union all
select 34510,'MRN' from dual union all
select 34511,'MRN' from dual union all
select 34512,'NASDA' from dual union all
select 34513,'MRN' from dual union all
select 34514,'NASDA' from dual union all
select 34515,'MRN' from dual union all
select 34516,'MRN' from dual union all
select 34517,'MRN' from dual union all
select 34518,'MRN' from dual union all
select 34519,'MRN' from dual union all
select 3452,'USA' from dual union all
select 34520,'MRN' from dual union all
select 34521,'MRN' from dual union all
select 34522,'MRN' from dual union all
select 34523,'MRN' from dual union all
select 34524,'MRN' from dual union all
select 34525,'MRN' from dual union all
select 34526,'MRN' from dual union all
select 34527,'MRN' from dual union all
select 34528,'MRN' from dual union all
select 34529,'MRN' from dual union all
select 3453,'USA' from dual union all
select 34530,'MRN' from dual union all
select 34531,'MRN' from dual union all
select 34532,'MRN' from dual union all
select 34533,'MRN' from dual union all
select 34534,'MRN' from dual union all
select 34535,'MRN' from dual union all
select 34536,'MRN' from dual union all
select 34537,'MRN' from dual union all
select 34538,'MRN' from dual union all
select 34539,'MRN' from dual union all
select 3454,'USA' from dual union all
select 34540,'MRN' from dual union all
select 34541,'MRN' from dual union all
select 34542,'MRN' from dual union all
select 34543,'MRN' from dual union all
select 34544,'MRN' from dual union all
select 34545,'MRN' from dual union all
select 34546,'MRN' from dual union all
select 34547,'MRN' from dual union all
select 34548,'MRN' from dual union all
select 34549,'MRN' from dual union all
select 3455,'USA' from dual union all
select 34550,'MRN' from dual union all
select 34551,'MRN' from dual union all
select 34552,'AN' from dual union all
select 34553,'MRN' from dual union all
select 34554,'MRN' from dual union all
select 34555,'MRN' from dual union all
select 34556,'MRN' from dual union all
select 34557,'MRN' from dual union all
select 34558,'MRN' from dual union all
select 34559,'MRN' from dual union all
select 3456,'USA' from dual union all
select 34560,'MRN' from dual union all
select 34561,'MRN' from dual union all
select 34562,'MRN' from dual union all
select 34563,'MRN' from dual union all
select 34564,'MRN' from dual union all
select 34565,'MRN' from dual union all
select 34566,'MRN' from dual union all
select 34567,'MRN' from dual union all
select 34568,'MRN' from dual union all
select 34569,'MRN' from dual union all
select 3457,'USA' from dual union all
select 34570,'MRN' from dual union all
select 34571,'MRN' from dual union all
select 34572,'MRN' from dual union all
select 34573,'MRN' from dual union all
select 34574,'MRN' from dual union all
select 34575,'MRN' from dual union all
select 34576,'MRN' from dual union all
select 34577,'MRN' from dual union all
select 34578,'MRN' from dual union all
select 34579,'MRN' from dual union all
select 3458,'USA' from dual union all
select 34580,'MRN' from dual union all
select 34581,'MRN' from dual union all
select 34582,'MRN' from dual;
insert into LAUNCH_AGENCY
select 34583,'MRN' from dual union all
select 34584,'MRN' from dual union all
select 34585,'MRN' from dual union all
select 34586,'MRN' from dual union all
select 34587,'MRN' from dual union all
select 34588,'MRN' from dual union all
select 34589,'MRN' from dual union all
select 3459,'USA' from dual union all
select 34590,'MRN' from dual union all
select 34591,'MRN' from dual union all
select 34592,'MRN' from dual union all
select 34593,'MRN' from dual union all
select 34594,'MRN' from dual union all
select 34595,'MRN' from dual union all
select 34596,'MRN' from dual union all
select 34597,'MRN' from dual union all
select 34598,'MRN' from dual union all
select 34599,'MRN' from dual union all
select 346,'WEHR' from dual union all
select 3460,'USA' from dual union all
select 34600,'MRN' from dual union all
select 34601,'MRN' from dual union all
select 34602,'MRN' from dual union all
select 34603,'MRN' from dual union all
select 34604,'MRN' from dual union all
select 34605,'MRN' from dual union all
select 34606,'MRN' from dual union all
select 34607,'MRN' from dual union all
select 34608,'MRN' from dual union all
select 34609,'MRN' from dual union all
select 3461,'USA' from dual union all
select 34610,'MRN' from dual union all
select 34611,'MRN' from dual union all
select 34612,'MRN' from dual union all
select 34613,'MRN' from dual union all
select 34614,'MRN' from dual union all
select 34615,'MRN' from dual union all
select 34616,'MRN' from dual union all
select 34617,'MRN' from dual union all
select 34618,'MRN' from dual union all
select 34619,'MRN' from dual union all
select 3462,'USA' from dual union all
select 34620,'MRN' from dual union all
select 34621,'MRN' from dual union all
select 34622,'MRN' from dual union all
select 34623,'MRN' from dual union all
select 34624,'MRN' from dual union all
select 34625,'MRN' from dual union all
select 34626,'MRN' from dual union all
select 34627,'MRN' from dual union all
select 34628,'MRN' from dual union all
select 34629,'MRN' from dual union all
select 3463,'USA' from dual union all
select 34630,'MRN' from dual union all
select 34631,'MRN' from dual union all
select 34632,'MRN' from dual union all
select 34633,'MRN' from dual union all
select 34634,'MRN' from dual union all
select 34635,'MRN' from dual union all
select 34636,'MRN' from dual union all
select 34637,'MRN' from dual union all
select 34638,'MRN' from dual union all
select 34639,'MRN' from dual union all
select 3464,'USA' from dual union all
select 34640,'MRN' from dual union all
select 34641,'MRN' from dual union all
select 34642,'MRN' from dual union all
select 34643,'MRN' from dual union all
select 34644,'MRN' from dual union all
select 34645,'MRN' from dual union all
select 34646,'MRN' from dual union all
select 34647,'MRN' from dual union all
select 34648,'MRN' from dual union all
select 34649,'MRN' from dual union all
select 3465,'USA' from dual union all
select 34650,'MRN' from dual union all
select 34651,'MRN' from dual union all
select 34652,'MRN' from dual union all
select 34653,'MRN' from dual union all
select 34654,'MRN' from dual union all
select 34655,'MRN' from dual union all
select 34656,'MRN' from dual union all
select 34657,'MRN' from dual union all
select 34658,'MRN' from dual union all
select 34659,'MRN' from dual union all
select 3466,'USA' from dual union all
select 34660,'MRN' from dual union all
select 34661,'MRN' from dual union all
select 34662,'MRN' from dual union all
select 34663,'MRN' from dual union all
select 34664,'MRN' from dual union all
select 34665,'MRN' from dual union all
select 34666,'MRN' from dual union all
select 34667,'MRN' from dual union all
select 34668,'MRN' from dual union all
select 34669,'MRN' from dual union all
select 3467,'USA' from dual union all
select 34670,'MRN' from dual union all
select 34671,'MRN' from dual union all
select 34672,'MRN' from dual;
insert into LAUNCH_AGENCY
select 34673,'MRN' from dual union all
select 34674,'MRN' from dual union all
select 34675,'MRN' from dual union all
select 34676,'MRN' from dual union all
select 34677,'MRN' from dual union all
select 34678,'MRN' from dual union all
select 34679,'MRN' from dual union all
select 3468,'USA' from dual union all
select 34680,'MRN' from dual union all
select 34681,'MRN' from dual union all
select 34682,'MRN' from dual union all
select 34683,'MRN' from dual union all
select 34684,'MRN' from dual union all
select 34685,'MRN' from dual union all
select 34686,'MRN' from dual union all
select 34687,'MRN' from dual union all
select 34688,'MRN' from dual union all
select 34689,'MRN' from dual union all
select 3469,'USA' from dual union all
select 34690,'MRN' from dual union all
select 34691,'MRN' from dual union all
select 34692,'MRN' from dual union all
select 34693,'MRN' from dual union all
select 34694,'MRN' from dual union all
select 34695,'MRN' from dual union all
select 34696,'MRN' from dual union all
select 34697,'MRN' from dual union all
select 34698,'MRN' from dual union all
select 34699,'AN' from dual union all
select 347,'WEHR' from dual union all
select 3470,'USA' from dual union all
select 34700,'MRN' from dual union all
select 34701,'MRN' from dual union all
select 34702,'AN' from dual union all
select 34703,'MRN' from dual union all
select 34704,'MRN' from dual union all
select 34705,'MRN' from dual union all
select 34706,'MRN' from dual union all
select 34707,'MRN' from dual union all
select 34708,'MRN' from dual union all
select 34709,'MRN' from dual union all
select 3471,'USA' from dual union all
select 34710,'MRN' from dual union all
select 34711,'MRN' from dual union all
select 34712,'MRN' from dual union all
select 34713,'MRN' from dual union all
select 34714,'MRN' from dual union all
select 34715,'MRN' from dual union all
select 34716,'MRN' from dual union all
select 34717,'MRN' from dual union all
select 34718,'MRN' from dual union all
select 34719,'MRN' from dual union all
select 3472,'USA' from dual union all
select 34720,'MRN' from dual union all
select 34721,'MRN' from dual union all
select 34722,'MRN' from dual union all
select 34723,'MRN' from dual union all
select 34724,'MRN' from dual union all
select 34725,'MRN' from dual union all
select 34726,'MRN' from dual union all
select 34727,'MRN' from dual union all
select 34728,'MRN' from dual union all
select 34729,'MRN' from dual union all
select 3473,'USA' from dual union all
select 34730,'MRN' from dual union all
select 34731,'MRN' from dual union all
select 34732,'MRN' from dual union all
select 34733,'MRN' from dual union all
select 34734,'MRN' from dual union all
select 34735,'MRN' from dual union all
select 34736,'MRN' from dual union all
select 34737,'MRN' from dual union all
select 34738,'MRN' from dual union all
select 34739,'MRN' from dual union all
select 3474,'USA' from dual union all
select 34740,'MRN' from dual union all
select 34741,'MRN' from dual union all
select 34742,'MRN' from dual union all
select 34743,'MRN' from dual union all
select 34744,'MRN' from dual union all
select 34745,'MRN' from dual union all
select 34746,'MRN' from dual union all
select 34747,'MRN' from dual union all
select 34748,'MRN' from dual union all
select 34749,'MRN' from dual union all
select 3475,'USA' from dual union all
select 34750,'MRN' from dual union all
select 34751,'MRN' from dual union all
select 34752,'MRN' from dual union all
select 34753,'MRN' from dual union all
select 34754,'MRN' from dual union all
select 34755,'MRN' from dual union all
select 34756,'MRN' from dual union all
select 34757,'MRN' from dual union all
select 34758,'MRN' from dual union all
select 34759,'MRN' from dual union all
select 3476,'USA' from dual union all
select 34760,'MRN' from dual union all
select 34761,'MRN' from dual union all
select 34762,'MRN' from dual;
insert into LAUNCH_AGENCY
select 34763,'MRN' from dual union all
select 34764,'MRN' from dual union all
select 34765,'MRN' from dual union all
select 34766,'MRN' from dual union all
select 34767,'MRN' from dual union all
select 34768,'MRN' from dual union all
select 34769,'MRN' from dual union all
select 3477,'USA' from dual union all
select 34770,'MRN' from dual union all
select 34771,'MRN' from dual union all
select 34772,'MRN' from dual union all
select 34773,'MRN' from dual union all
select 34774,'MRN' from dual union all
select 34775,'MRN' from dual union all
select 34776,'MRN' from dual union all
select 34777,'MRN' from dual union all
select 34778,'MRN' from dual union all
select 34779,'MRN' from dual union all
select 3478,'USA' from dual union all
select 34780,'MRN' from dual union all
select 34781,'MRN' from dual union all
select 34782,'MRN' from dual union all
select 34783,'MRN' from dual union all
select 34784,'MRN' from dual union all
select 34785,'MRN' from dual union all
select 34786,'MRN' from dual union all
select 34787,'MRN' from dual union all
select 34788,'MRN' from dual union all
select 34789,'MRN' from dual union all
select 3479,'USA' from dual union all
select 34790,'MRN' from dual union all
select 34791,'MRN' from dual union all
select 34792,'MRN' from dual union all
select 34793,'MRN' from dual union all
select 34794,'MRN' from dual union all
select 34795,'MRN' from dual union all
select 34796,'MRN' from dual union all
select 34797,'MRN' from dual union all
select 34798,'MRN' from dual union all
select 34799,'MRN' from dual union all
select 348,'WEHR' from dual union all
select 3480,'USA' from dual union all
select 34800,'MRN' from dual union all
select 34801,'MRN' from dual union all
select 34802,'MRN' from dual union all
select 34803,'MRN' from dual union all
select 34804,'MRN' from dual union all
select 34805,'MRN' from dual union all
select 34806,'MRN' from dual union all
select 34807,'MRN' from dual union all
select 34808,'MRN' from dual union all
select 34809,'MRN' from dual union all
select 3481,'USA' from dual union all
select 34810,'MRN' from dual union all
select 34811,'MRN' from dual union all
select 34812,'MRN' from dual union all
select 34813,'MRN' from dual union all
select 34814,'MRN' from dual union all
select 34815,'MRN' from dual union all
select 34816,'MRN' from dual union all
select 34817,'MRN' from dual union all
select 34818,'MRN' from dual union all
select 34819,'MRN' from dual union all
select 3482,'USA' from dual union all
select 34820,'MRN' from dual union all
select 34821,'MRN' from dual union all
select 34822,'MRN' from dual union all
select 34823,'MRN' from dual union all
select 34824,'MRN' from dual union all
select 34825,'MRN' from dual union all
select 34826,'MRN' from dual union all
select 34827,'MRN' from dual union all
select 34828,'MRN' from dual union all
select 34829,'MRN' from dual union all
select 3483,'USA' from dual union all
select 34830,'MRN' from dual union all
select 34831,'MRN' from dual union all
select 34832,'MRN' from dual union all
select 34833,'MRN' from dual union all
select 34834,'MRN' from dual union all
select 34835,'MRN' from dual union all
select 34836,'MRN' from dual union all
select 34837,'MRN' from dual union all
select 34838,'MRN' from dual union all
select 34839,'MRN' from dual union all
select 3484,'USA' from dual union all
select 34840,'MRN' from dual union all
select 34841,'MRN' from dual union all
select 34842,'MRN' from dual union all
select 34843,'MRN' from dual union all
select 34844,'MRN' from dual union all
select 34845,'MRN' from dual union all
select 34846,'MRN' from dual union all
select 34847,'MRN' from dual union all
select 34848,'MRN' from dual union all
select 34849,'MRN' from dual union all
select 3485,'USA' from dual union all
select 34850,'MRN' from dual union all
select 34851,'MRN' from dual union all
select 34852,'MRN' from dual;
insert into LAUNCH_AGENCY
select 34853,'MRN' from dual union all
select 34854,'MRN' from dual union all
select 34855,'MRN' from dual union all
select 34856,'MRN' from dual union all
select 34857,'MRN' from dual union all
select 34858,'MRN' from dual union all
select 34859,'MRN' from dual union all
select 3486,'ORDRSA' from dual union all
select 34860,'MRN' from dual union all
select 34861,'MRN' from dual union all
select 34862,'MRN' from dual union all
select 34863,'MRN' from dual union all
select 34864,'MRN' from dual union all
select 34865,'MRN' from dual union all
select 34866,'MRN' from dual union all
select 34867,'MRN' from dual union all
select 34868,'MRN' from dual union all
select 34869,'MRN' from dual union all
select 3487,'USA' from dual union all
select 34870,'MRN' from dual union all
select 34871,'MRN' from dual union all
select 34872,'MRN' from dual union all
select 34873,'MRN' from dual union all
select 34874,'MRN' from dual union all
select 34875,'MRN' from dual union all
select 34876,'MRN' from dual union all
select 34877,'MRN' from dual union all
select 34878,'MRN' from dual union all
select 34879,'MRN' from dual union all
select 3488,'USA' from dual union all
select 34880,'MRN' from dual union all
select 34881,'MRN' from dual union all
select 34882,'MRN' from dual union all
select 34883,'MRN' from dual union all
select 34884,'MRN' from dual union all
select 34885,'MRN' from dual union all
select 34886,'MRN' from dual union all
select 34887,'MRN' from dual union all
select 34888,'MRN' from dual union all
select 34889,'MRN' from dual union all
select 3489,'USA' from dual union all
select 34890,'MRN' from dual union all
select 34891,'MRN' from dual union all
select 34892,'MRN' from dual union all
select 34893,'MRN' from dual union all
select 34894,'MRN' from dual union all
select 34895,'MRN' from dual union all
select 34896,'MRN' from dual union all
select 34897,'MRN' from dual union all
select 34898,'MRN' from dual union all
select 34899,'MRN' from dual union all
select 349,'WEHR' from dual union all
select 3490,'USA' from dual union all
select 34900,'MRN' from dual union all
select 34901,'MRN' from dual union all
select 34902,'MRN' from dual union all
select 34903,'MRN' from dual union all
select 34904,'MRN' from dual union all
select 34905,'MRN' from dual union all
select 34906,'MRN' from dual union all
select 34907,'MRN' from dual union all
select 34908,'MRN' from dual union all
select 34909,'MRN' from dual union all
select 3491,'USA' from dual union all
select 34910,'MRN' from dual union all
select 34911,'MRN' from dual union all
select 34912,'MRN' from dual union all
select 34913,'MRN' from dual union all
select 34914,'MRN' from dual union all
select 34915,'MRN' from dual union all
select 34916,'MRN' from dual union all
select 34917,'MRN' from dual union all
select 34918,'MRN' from dual union all
select 34919,'MRN' from dual union all
select 3492,'USA' from dual union all
select 34920,'MRN' from dual union all
select 34921,'MRN' from dual union all
select 34922,'MRN' from dual union all
select 34923,'NASA' from dual union all
select 34924,'MRN' from dual union all
select 34925,'MRN' from dual union all
select 34926,'MRN' from dual union all
select 34927,'MRN' from dual union all
select 34928,'MRN' from dual union all
select 34929,'MRN' from dual union all
select 3493,'USA' from dual union all
select 34930,'MRN' from dual union all
select 34931,'MRN' from dual union all
select 34932,'MRN' from dual union all
select 34933,'MRN' from dual union all
select 34934,'MRN' from dual union all
select 34935,'MRN' from dual union all
select 34936,'MRN' from dual union all
select 34937,'MRN' from dual union all
select 34938,'MRN' from dual union all
select 34939,'MRN' from dual union all
select 3494,'USA' from dual union all
select 34940,'MRN' from dual union all
select 34941,'MRN' from dual union all
select 34942,'MRN' from dual;
insert into LAUNCH_AGENCY
select 34943,'MRN' from dual union all
select 34944,'MRN' from dual union all
select 34945,'MRN' from dual union all
select 34946,'MRN' from dual union all
select 34947,'MRN' from dual union all
select 34948,'MRN' from dual union all
select 34949,'MRN' from dual union all
select 3495,'USA' from dual union all
select 34950,'MRN' from dual union all
select 34951,'MRN' from dual union all
select 34952,'MRN' from dual union all
select 34953,'MRN' from dual union all
select 34954,'MRN' from dual union all
select 34955,'MRN' from dual union all
select 34956,'MRN' from dual union all
select 34957,'MRN' from dual union all
select 34958,'MRN' from dual union all
select 34959,'MRN' from dual union all
select 3496,'USA' from dual union all
select 34960,'MRN' from dual union all
select 34961,'MRN' from dual union all
select 34962,'MRN' from dual union all
select 34963,'MRN' from dual union all
select 34964,'MRN' from dual union all
select 34965,'MRN' from dual union all
select 34966,'MRN' from dual union all
select 34967,'MRN' from dual union all
select 34968,'MRN' from dual union all
select 34969,'MRN' from dual union all
select 3497,'ORDRSA' from dual union all
select 34970,'MRN' from dual union all
select 34971,'MRN' from dual union all
select 34972,'MRN' from dual union all
select 34973,'MRN' from dual union all
select 34974,'MRN' from dual union all
select 34975,'MRN' from dual union all
select 34976,'MRN' from dual union all
select 34977,'MRN' from dual union all
select 34978,'MRN' from dual union all
select 34979,'MRN' from dual union all
select 3498,'USA' from dual union all
select 34980,'MRN' from dual union all
select 34981,'MRN' from dual union all
select 34982,'MRN' from dual union all
select 34983,'MRN' from dual union all
select 34984,'MRN' from dual union all
select 34985,'MRN' from dual union all
select 34986,'MRN' from dual union all
select 34987,'MRN' from dual union all
select 34988,'MRN' from dual union all
select 34989,'MRN' from dual union all
select 3499,'USA' from dual union all
select 34990,'MRN' from dual union all
select 34991,'MRN' from dual union all
select 34992,'MRN' from dual union all
select 34993,'MRN' from dual union all
select 34994,'MRN' from dual union all
select 34995,'MRN' from dual union all
select 34996,'MRN' from dual union all
select 34997,'MRN' from dual union all
select 34998,'MRN' from dual union all
select 34999,'MRN' from dual union all
select 35,'WEHR' from dual union all
select 350,'WER485' from dual union all
select 3500,'USA' from dual union all
select 35000,'MRN' from dual union all
select 35001,'MRN' from dual union all
select 35002,'MRN' from dual union all
select 35003,'MRN' from dual union all
select 35004,'MRN' from dual union all
select 35005,'MRN' from dual union all
select 35006,'MRN' from dual union all
select 35007,'MRN' from dual union all
select 35008,'MRN' from dual union all
select 35009,'MRN' from dual union all
select 3501,'USA' from dual union all
select 35010,'MRN' from dual union all
select 35011,'MRN' from dual union all
select 35012,'MRN' from dual union all
select 35013,'MRN' from dual union all
select 35014,'MRN' from dual union all
select 35015,'MRN' from dual union all
select 35016,'MRN' from dual union all
select 35017,'MRN' from dual union all
select 35018,'MRN' from dual union all
select 35019,'MRN' from dual union all
select 3502,'USA' from dual union all
select 35020,'MRN' from dual union all
select 35021,'MRN' from dual union all
select 35022,'MRN' from dual union all
select 35023,'MRN' from dual union all
select 35024,'MRN' from dual union all
select 35025,'MRN' from dual union all
select 35026,'MRN' from dual union all
select 35027,'MRN' from dual union all
select 35028,'MRN' from dual union all
select 35029,'MRN' from dual union all
select 3503,'USA' from dual union all
select 35030,'MRN' from dual union all
select 35031,'MRN' from dual;
insert into LAUNCH_AGENCY
select 35032,'MRN' from dual union all
select 35033,'MRN' from dual union all
select 35034,'NASDA' from dual union all
select 35035,'MRN' from dual union all
select 35036,'MRN' from dual union all
select 35037,'MRN' from dual union all
select 35038,'MRN' from dual union all
select 35039,'MRN' from dual union all
select 3504,'USA' from dual union all
select 35040,'MRN' from dual union all
select 35041,'MRN' from dual union all
select 35042,'MRN' from dual union all
select 35043,'MRN' from dual union all
select 35044,'MRN' from dual union all
select 35045,'MRN' from dual union all
select 35046,'MRN' from dual union all
select 35047,'MRN' from dual union all
select 35048,'MRN' from dual union all
select 35049,'MRN' from dual union all
select 3505,'WADD' from dual union all
select 35050,'MRN' from dual union all
select 35051,'MRN' from dual union all
select 35052,'MRN' from dual union all
select 35053,'MRN' from dual union all
select 35054,'MRN' from dual union all
select 35055,'MRN' from dual union all
select 35056,'MRN' from dual union all
select 35057,'MRN' from dual union all
select 35058,'MRN' from dual union all
select 35059,'MRN' from dual union all
select 3506,'WADD' from dual union all
select 35060,'MRN' from dual union all
select 35061,'MRN' from dual union all
select 35062,'MRN' from dual union all
select 35063,'MRN' from dual union all
select 35064,'MRN' from dual union all
select 35065,'MRN' from dual union all
select 35066,'MRN' from dual union all
select 35067,'MRN' from dual union all
select 35068,'MRN' from dual union all
select 35069,'MRN' from dual union all
select 3507,'WADD' from dual union all
select 35070,'MRN' from dual union all
select 35071,'MRN' from dual union all
select 35072,'MRN' from dual union all
select 35073,'MRN' from dual union all
select 35074,'MRN' from dual union all
select 35075,'JMA' from dual union all
select 35076,'JMA' from dual union all
select 35077,'JMA' from dual union all
select 35078,'JMA' from dual union all
select 35079,'MRN' from dual union all
select 3508,'USA' from dual union all
select 35080,'JMA' from dual union all
select 35081,'JMA' from dual union all
select 35082,'JMA' from dual union all
select 35083,'JMA' from dual union all
select 35084,'JMA' from dual union all
select 35085,'JMA' from dual union all
select 35086,'JMA' from dual union all
select 35087,'JMA' from dual union all
select 35088,'JMA' from dual union all
select 35089,'JMA' from dual union all
select 3509,'WADD' from dual union all
select 35090,'MRN' from dual union all
select 35091,'JMA' from dual union all
select 35092,'JMA' from dual union all
select 35093,'JMA' from dual union all
select 35094,'JMA' from dual union all
select 35095,'JMA' from dual union all
select 35096,'JMA' from dual union all
select 35097,'JMA' from dual union all
select 35098,'JMA' from dual union all
select 35099,'JMA' from dual union all
select 351,'WER485-2' from dual union all
select 3510,'WADD' from dual union all
select 35100,'JMA' from dual union all
select 35101,'MRN' from dual union all
select 35102,'JMA' from dual union all
select 35103,'JMA' from dual union all
select 35104,'JMA' from dual union all
select 35105,'JMA' from dual union all
select 35106,'JMA' from dual union all
select 35107,'JMA' from dual union all
select 35108,'JMA' from dual union all
select 35109,'JMA' from dual union all
select 3511,'WADD' from dual union all
select 35110,'JMA' from dual union all
select 35111,'JMA' from dual union all
select 35112,'MRN' from dual union all
select 35113,'JMA' from dual union all
select 35114,'JMA' from dual union all
select 35115,'JMA' from dual union all
select 35116,'JMA' from dual union all
select 35117,'JMA' from dual union all
select 35118,'MRN' from dual union all
select 35119,'USA' from dual union all
select 3512,'WADD' from dual union all
select 35120,'USA' from dual union all
select 35121,'USA' from dual;
insert into LAUNCH_AGENCY
select 35122,'USA' from dual union all
select 35123,'MRN' from dual union all
select 35124,'USA' from dual union all
select 35125,'USA' from dual union all
select 35126,'USA' from dual union all
select 35127,'USA' from dual union all
select 35128,'USA' from dual union all
select 35129,'USA' from dual union all
select 3513,'WADD' from dual union all
select 35130,'USA' from dual union all
select 35131,'USA' from dual union all
select 35132,'USA' from dual union all
select 35133,'USA' from dual union all
select 35134,'MRN' from dual union all
select 35135,'USA' from dual union all
select 35136,'USA' from dual union all
select 35137,'USA' from dual union all
select 35138,'USA' from dual union all
select 35139,'USA' from dual union all
select 3514,'WADD' from dual union all
select 35140,'USA' from dual union all
select 35141,'USA' from dual union all
select 35142,'USA' from dual union all
select 35143,'USA' from dual union all
select 35144,'USA' from dual union all
select 35145,'MRN' from dual union all
select 35146,'USA' from dual union all
select 35147,'USA' from dual union all
select 35148,'MRN' from dual union all
select 35149,'MRN' from dual union all
select 3515,'WADD' from dual union all
select 35150,'MRN' from dual union all
select 35151,'MRN' from dual union all
select 35152,'MRN' from dual union all
select 35153,'MRN' from dual union all
select 35154,'MRN' from dual union all
select 35155,'MRN' from dual union all
select 35156,'MRN' from dual union all
select 35157,'MRN' from dual union all
select 35158,'MRN' from dual union all
select 35159,'MRN' from dual union all
select 3516,'WADD' from dual union all
select 35160,'MRN' from dual union all
select 35161,'MRN' from dual union all
select 35162,'MRN' from dual union all
select 35163,'NASDA' from dual union all
select 35164,'MRN' from dual union all
select 35165,'MRN' from dual union all
select 35166,'MRN' from dual union all
select 35167,'MRN' from dual union all
select 35168,'MRN' from dual union all
select 35169,'MRN' from dual union all
select 3517,'WADD' from dual union all
select 35170,'MRN' from dual union all
select 35171,'MRN' from dual union all
select 35172,'MRN' from dual union all
select 35173,'MRN' from dual union all
select 35174,'MRN' from dual union all
select 35175,'MRN' from dual union all
select 35176,'MRN' from dual union all
select 35177,'MRN' from dual union all
select 35178,'MRN' from dual union all
select 35179,'MRN' from dual union all
select 3518,'WADD' from dual union all
select 35180,'MRN' from dual union all
select 35181,'MRN' from dual union all
select 35182,'MRN' from dual union all
select 35183,'MRN' from dual union all
select 35184,'MRN' from dual union all
select 35185,'MRN' from dual union all
select 35186,'MRN' from dual union all
select 35187,'MRN' from dual union all
select 35188,'MRN' from dual union all
select 35189,'MRN' from dual union all
select 3519,'USA' from dual union all
select 35190,'MRN' from dual union all
select 35191,'MRN' from dual union all
select 35192,'MRN' from dual union all
select 35193,'MRN' from dual union all
select 35194,'MRN' from dual union all
select 35195,'MRN' from dual union all
select 35196,'MRN' from dual union all
select 35197,'MRN' from dual union all
select 35198,'MRN' from dual union all
select 35199,'AN' from dual union all
select 352,'WER485' from dual union all
select 3520,'WADD' from dual union all
select 35200,'MRN' from dual union all
select 35201,'MRN' from dual union all
select 35202,'MRN' from dual union all
select 35203,'MRN' from dual union all
select 35204,'MRN' from dual union all
select 35205,'MRN' from dual union all
select 35206,'MRN' from dual union all
select 35207,'MRN' from dual union all
select 35208,'MRN' from dual union all
select 35209,'MRN' from dual union all
select 3521,'WADD' from dual union all
select 35210,'MRN' from dual union all
select 35211,'MRN' from dual;
insert into LAUNCH_AGENCY
select 35212,'MRN' from dual union all
select 35213,'MRN' from dual union all
select 35214,'MRN' from dual union all
select 35215,'MRN' from dual union all
select 35216,'MRN' from dual union all
select 35217,'MRN' from dual union all
select 35218,'MRN' from dual union all
select 35219,'NASA' from dual union all
select 3522,'WADD' from dual union all
select 35220,'MRN' from dual union all
select 35221,'MRN' from dual union all
select 35222,'MRN' from dual union all
select 35223,'MRN' from dual union all
select 35224,'MRN' from dual union all
select 35225,'MRN' from dual union all
select 35226,'MRN' from dual union all
select 35227,'MRN' from dual union all
select 35228,'MRN' from dual union all
select 35229,'MRN' from dual union all
select 3523,'WADD' from dual union all
select 35230,'MRN' from dual union all
select 35231,'MRN' from dual union all
select 35232,'MRN' from dual union all
select 35233,'MRN' from dual union all
select 35234,'MRN' from dual union all
select 35235,'MRN' from dual union all
select 35236,'MRN' from dual union all
select 35237,'MRN' from dual union all
select 35238,'MRN' from dual union all
select 35239,'MRN' from dual union all
select 3524,'WADD' from dual union all
select 35240,'MRN' from dual union all
select 35241,'MRN' from dual union all
select 35242,'MRN' from dual union all
select 35243,'MRN' from dual union all
select 35244,'MRN' from dual union all
select 35245,'MRN' from dual union all
select 35246,'MRN' from dual union all
select 35247,'MRN' from dual union all
select 35248,'MRN' from dual union all
select 35249,'MRN' from dual union all
select 3525,'WADD' from dual union all
select 35250,'MRN' from dual union all
select 35251,'MRN' from dual union all
select 35252,'MRN' from dual union all
select 35253,'MRN' from dual union all
select 35254,'MRN' from dual union all
select 35255,'MRN' from dual union all
select 35256,'MRN' from dual union all
select 35257,'MRN' from dual union all
select 35258,'MRN' from dual union all
select 35259,'MRN' from dual union all
select 3526,'WADD' from dual union all
select 35260,'MRN' from dual union all
select 35261,'MRN' from dual union all
select 35262,'MRN' from dual union all
select 35263,'MRN' from dual union all
select 35264,'MRN' from dual union all
select 35265,'MRN' from dual union all
select 35266,'MRN' from dual union all
select 35267,'MRN' from dual union all
select 35268,'MRN' from dual union all
select 35269,'MRN' from dual union all
select 3527,'WADD' from dual union all
select 35270,'MRN' from dual union all
select 35271,'MRN' from dual union all
select 35272,'MRN' from dual union all
select 35273,'MRN' from dual union all
select 35274,'MRN' from dual union all
select 35275,'MRN' from dual union all
select 35276,'MRN' from dual union all
select 35277,'MRN' from dual union all
select 35278,'MRN' from dual union all
select 35279,'MRN' from dual union all
select 3528,'WADD' from dual union all
select 35280,'MRN' from dual union all
select 35281,'MRN' from dual union all
select 35282,'MRN' from dual union all
select 35283,'MRN' from dual union all
select 35284,'MRN' from dual union all
select 35285,'MRN' from dual union all
select 35286,'MRN' from dual union all
select 35287,'MRN' from dual union all
select 35288,'MRN' from dual union all
select 35289,'MRN' from dual union all
select 3529,'WADD' from dual union all
select 35290,'MRN' from dual union all
select 35291,'MRN' from dual union all
select 35292,'MRN' from dual union all
select 35293,'MRN' from dual union all
select 35294,'MRN' from dual union all
select 35295,'MRN' from dual union all
select 35296,'MRN' from dual union all
select 35297,'MRN' from dual union all
select 35298,'MRN' from dual union all
select 35299,'MRN' from dual union all
select 353,'WER444' from dual union all
select 3530,'USA' from dual union all
select 35300,'MRN' from dual union all
select 35301,'MRN' from dual;
insert into LAUNCH_AGENCY
select 35302,'MRN' from dual union all
select 35303,'MRN' from dual union all
select 35304,'MRN' from dual union all
select 35305,'MRN' from dual union all
select 35306,'MRN' from dual union all
select 35307,'MRN' from dual union all
select 35308,'MRN' from dual union all
select 35309,'MRN' from dual union all
select 3531,'WADD' from dual union all
select 35310,'MRN' from dual union all
select 35311,'MRN' from dual union all
select 35312,'MRN' from dual union all
select 35313,'MRN' from dual union all
select 35314,'MRN' from dual union all
select 35315,'MRN' from dual union all
select 35316,'MRN' from dual union all
select 35317,'MRN' from dual union all
select 35318,'MRN' from dual union all
select 35319,'MRN' from dual union all
select 3532,'WADD' from dual union all
select 35320,'MRN' from dual union all
select 35321,'MRN' from dual union all
select 35322,'MRN' from dual union all
select 35323,'MRN' from dual union all
select 35324,'MRN' from dual union all
select 35325,'MRN' from dual union all
select 35326,'MRN' from dual union all
select 35327,'MRN' from dual union all
select 35328,'MRN' from dual union all
select 35329,'MRN' from dual union all
select 3533,'WADD' from dual union all
select 35330,'MRN' from dual union all
select 35331,'MRN' from dual union all
select 35332,'MRN' from dual union all
select 35333,'MRN' from dual union all
select 35334,'MRN' from dual union all
select 35335,'MRN' from dual union all
select 35336,'MRN' from dual union all
select 35337,'MRN' from dual union all
select 35338,'MRN' from dual union all
select 35339,'MRN' from dual union all
select 3534,'WADD' from dual union all
select 35340,'MRN' from dual union all
select 35341,'MRN' from dual union all
select 35342,'MRN' from dual union all
select 35343,'MRN' from dual union all
select 35344,'MRN' from dual union all
select 35345,'MRN' from dual union all
select 35346,'MRN' from dual union all
select 35347,'MRN' from dual union all
select 35348,'MRN' from dual union all
select 35349,'MRN' from dual union all
select 3535,'ARDC' from dual union all
select 35350,'MRN' from dual union all
select 35351,'WFC' from dual union all
select 35352,'WFC' from dual union all
select 35353,'WFC' from dual union all
select 35354,'WFC' from dual union all
select 35355,'WFC' from dual union all
select 35356,'WFC' from dual union all
select 35357,'MRN' from dual union all
select 35358,'MRN' from dual union all
select 35359,'WFC' from dual union all
select 3536,'ARDC' from dual union all
select 35360,'WFC' from dual union all
select 35361,'WFC' from dual union all
select 35362,'WFC' from dual union all
select 35363,'WFC' from dual union all
select 35364,'WFC' from dual union all
select 35365,'MRN' from dual union all
select 35366,'WFC' from dual union all
select 35367,'MRN' from dual union all
select 35368,'MRN' from dual union all
select 35369,'WFC' from dual union all
select 3537,'ARDC' from dual union all
select 35370,'WFC' from dual union all
select 35371,'WFC' from dual union all
select 35372,'WFC' from dual union all
select 35373,'WFC' from dual union all
select 35374,'MRN' from dual union all
select 35375,'WFC' from dual union all
select 35376,'WFC' from dual union all
select 35377,'WFC' from dual union all
select 35378,'WFC' from dual union all
select 35379,'MRN' from dual union all
select 3538,'UZKA' from dual union all
select 35380,'WFC' from dual union all
select 35381,'WFC' from dual union all
select 35382,'WFC' from dual union all
select 35383,'WFC' from dual union all
select 35384,'WFC' from dual union all
select 35385,'WFC' from dual union all
select 35386,'MRN' from dual union all
select 35387,'WFC' from dual union all
select 35388,'WFC' from dual union all
select 35389,'WFC' from dual union all
select 3539,'UNRV' from dual union all
select 35390,'MRN' from dual union all
select 35391,'WFC' from dual union all
select 35392,'WFC' from dual;
insert into LAUNCH_AGENCY
select 35393,'WFC' from dual union all
select 35394,'MRN' from dual union all
select 35395,'WFC' from dual union all
select 35396,'WFC' from dual union all
select 35397,'WFC' from dual union all
select 35398,'WFC' from dual union all
select 35399,'WFC' from dual union all
select 354,'WER444' from dual union all
select 3540,'UNRV' from dual union all
select 35400,'WFC' from dual union all
select 35401,'MRN' from dual union all
select 35402,'MRN' from dual union all
select 35403,'WFC' from dual union all
select 35404,'WFC' from dual union all
select 35405,'WFC' from dual union all
select 35406,'MRN' from dual union all
select 35407,'WFC' from dual union all
select 35408,'WFC' from dual union all
select 35409,'WFC' from dual union all
select 3541,'USA' from dual union all
select 35410,'WFC' from dual union all
select 35411,'WFC' from dual union all
select 35412,'MRN' from dual union all
select 35413,'WFC' from dual union all
select 35414,'WFC' from dual union all
select 35415,'MRN' from dual union all
select 35416,'WFC' from dual union all
select 35417,'WFC' from dual union all
select 35418,'WFC' from dual union all
select 35419,'WFC' from dual union all
select 3542,'WADD' from dual union all
select 35420,'WFC' from dual union all
select 35421,'WFC' from dual union all
select 35422,'MRN' from dual union all
select 35423,'MRN' from dual union all
select 35424,'WFC' from dual union all
select 35425,'MRN' from dual union all
select 35426,'WFC' from dual union all
select 35427,'WFC' from dual union all
select 35428,'WFC' from dual union all
select 35429,'WFC' from dual union all
select 3543,'USA' from dual union all
select 35430,'WFC' from dual union all
select 35431,'MRN' from dual union all
select 35432,'MRN' from dual union all
select 35433,'MRN' from dual union all
select 35434,'MRN' from dual union all
select 35435,'MRN' from dual union all
select 35436,'MRN' from dual union all
select 35437,'MRN' from dual union all
select 35438,'MRN' from dual union all
select 35439,'MRN' from dual union all
select 3544,'USA' from dual union all
select 35440,'NRCC' from dual union all
select 35441,'MRN' from dual union all
select 35442,'MRN' from dual union all
select 35443,'MRN' from dual union all
select 35444,'MRN' from dual union all
select 35445,'NASA' from dual union all
select 35446,'MRN' from dual union all
select 35447,'MRN' from dual union all
select 35448,'MRN' from dual union all
select 35449,'MRN' from dual union all
select 3545,'USA' from dual union all
select 35450,'MRN' from dual union all
select 35451,'MRN' from dual union all
select 35452,'MRN' from dual union all
select 35453,'MRN' from dual union all
select 35454,'MRN' from dual union all
select 35455,'MRN' from dual union all
select 35456,'MRN' from dual union all
select 35457,'MRN' from dual union all
select 35458,'MRN' from dual union all
select 35459,'MRN' from dual union all
select 3546,'USA' from dual union all
select 35460,'MRN' from dual union all
select 35461,'MRN' from dual union all
select 35462,'MRN' from dual union all
select 35463,'MRN' from dual union all
select 35464,'MRN' from dual union all
select 35465,'MRN' from dual union all
select 35466,'MRN' from dual union all
select 35467,'MRN' from dual union all
select 35468,'MRN' from dual union all
select 35469,'MRN' from dual union all
select 3547,'USA' from dual union all
select 35470,'MRN' from dual union all
select 35471,'MRN' from dual union all
select 35472,'MRN' from dual union all
select 35473,'MRN' from dual union all
select 35474,'MRN' from dual union all
select 35475,'MRN' from dual union all
select 35476,'MRN' from dual union all
select 35477,'MRN' from dual union all
select 35478,'MRN' from dual union all
select 35479,'MRN' from dual union all
select 3548,'USA' from dual union all
select 35480,'MRN' from dual union all
select 35481,'MRN' from dual union all
select 35482,'MRN' from dual;
insert into LAUNCH_AGENCY
select 35483,'MRN' from dual union all
select 35484,'MRN' from dual union all
select 35485,'MRN' from dual union all
select 35486,'MRN' from dual union all
select 35487,'MRN' from dual union all
select 35488,'MRN' from dual union all
select 35489,'MRN' from dual union all
select 3549,'USA' from dual union all
select 35490,'MRN' from dual union all
select 35491,'MRN' from dual union all
select 35492,'MRN' from dual union all
select 35493,'MRN' from dual union all
select 35494,'MRN' from dual union all
select 35495,'MRN' from dual union all
select 35496,'MRN' from dual union all
select 35497,'MRN' from dual union all
select 35498,'MRN' from dual union all
select 35499,'MRN' from dual union all
select 355,'WEHR' from dual union all
select 3550,'USA' from dual union all
select 35500,'MRN' from dual union all
select 35501,'MRN' from dual union all
select 35502,'MRN' from dual union all
select 35503,'MRN' from dual union all
select 35504,'MRN' from dual union all
select 35505,'MRN' from dual union all
select 35506,'MRN' from dual union all
select 35507,'MRN' from dual union all
select 35508,'MRN' from dual union all
select 35509,'MRN' from dual union all
select 3551,'USA' from dual union all
select 35510,'MRN' from dual union all
select 35511,'MRN' from dual union all
select 35512,'MRN' from dual union all
select 35513,'MRN' from dual union all
select 35514,'MRN' from dual union all
select 35515,'MRN' from dual union all
select 35516,'MRN' from dual union all
select 35517,'MRN' from dual union all
select 35518,'MRN' from dual union all
select 35519,'MRN' from dual union all
select 3552,'USA' from dual union all
select 35520,'MRN' from dual union all
select 35521,'MRN' from dual union all
select 35522,'MRN' from dual union all
select 35523,'MRN' from dual union all
select 35524,'MRN' from dual union all
select 35525,'MRN' from dual union all
select 35526,'MRN' from dual union all
select 35527,'MRN' from dual union all
select 35528,'MRN' from dual union all
select 35529,'MRN' from dual union all
select 3553,'USA' from dual union all
select 35530,'MRN' from dual union all
select 35531,'MRN' from dual union all
select 35532,'MRN' from dual union all
select 35533,'MRN' from dual union all
select 35534,'MRN' from dual union all
select 35535,'MRN' from dual union all
select 35536,'MRN' from dual union all
select 35537,'MRN' from dual union all
select 35538,'MRN' from dual union all
select 35539,'MRN' from dual union all
select 3554,'USA' from dual union all
select 35540,'MRN' from dual union all
select 35541,'MRN' from dual union all
select 35542,'MRN' from dual union all
select 35543,'MRN' from dual union all
select 35544,'MRN' from dual union all
select 35545,'MRN' from dual union all
select 35546,'MRN' from dual union all
select 35547,'MRN' from dual union all
select 35548,'MRN' from dual union all
select 35549,'MRN' from dual union all
select 3555,'ORDRSA' from dual union all
select 35550,'MRN' from dual union all
select 35551,'MRN' from dual union all
select 35552,'MRN' from dual union all
select 35553,'MRN' from dual union all
select 35554,'MRN' from dual union all
select 35555,'MRN' from dual union all
select 35556,'MRN' from dual union all
select 35557,'MRN' from dual union all
select 35558,'MRN' from dual union all
select 35559,'MRN' from dual union all
select 3556,'ORDRSA' from dual union all
select 35560,'MRN' from dual union all
select 35561,'MRN' from dual union all
select 35562,'MRN' from dual union all
select 35563,'MRN' from dual union all
select 35564,'MRN' from dual union all
select 35565,'MRN' from dual union all
select 35566,'MRN' from dual union all
select 35567,'MRN' from dual union all
select 35568,'MRN' from dual union all
select 35569,'MRN' from dual union all
select 3557,'ORDRSA' from dual union all
select 35570,'MRN' from dual union all
select 35571,'MRN' from dual union all
select 35572,'NRCC' from dual;
insert into LAUNCH_AGENCY
select 35573,'MRN' from dual union all
select 35574,'MRN' from dual union all
select 35575,'MRN' from dual union all
select 35576,'MRN' from dual union all
select 35577,'MRN' from dual union all
select 35578,'MRN' from dual union all
select 35579,'MRN' from dual union all
select 3558,'USA' from dual union all
select 35580,'MRN' from dual union all
select 35581,'MRN' from dual union all
select 35582,'MRN' from dual union all
select 35583,'MRN' from dual union all
select 35584,'MRN' from dual union all
select 35585,'MRN' from dual union all
select 35586,'MRN' from dual union all
select 35587,'MRN' from dual union all
select 35588,'MRN' from dual union all
select 35589,'MRN' from dual union all
select 3559,'USA' from dual union all
select 35590,'MRN' from dual union all
select 35591,'MRN' from dual union all
select 35592,'MRN' from dual union all
select 35593,'MRN' from dual union all
select 35594,'MRN' from dual union all
select 35595,'MRN' from dual union all
select 35596,'MRN' from dual union all
select 35597,'MRN' from dual union all
select 35598,'MRN' from dual union all
select 35599,'MRN' from dual union all
select 356,'WEHR' from dual union all
select 3560,'USA' from dual union all
select 35600,'MRN' from dual union all
select 35601,'MRN' from dual union all
select 35602,'MRN' from dual union all
select 35603,'MRN' from dual union all
select 35604,'MRN' from dual union all
select 35605,'MRN' from dual union all
select 35606,'MRN' from dual union all
select 35607,'MRN' from dual union all
select 35608,'MRN' from dual union all
select 35609,'MRN' from dual union all
select 3561,'USA' from dual union all
select 35610,'MRN' from dual union all
select 35611,'MRN' from dual union all
select 35612,'MRN' from dual union all
select 35613,'MRN' from dual union all
select 35614,'MRN' from dual union all
select 35615,'MRN' from dual union all
select 35616,'MRN' from dual union all
select 35617,'MRN' from dual union all
select 35618,'MRN' from dual union all
select 35619,'MRN' from dual union all
select 3562,'USA' from dual union all
select 35620,'MRN' from dual union all
select 35621,'MRN' from dual union all
select 35622,'MRN' from dual union all
select 35623,'MRN' from dual union all
select 35624,'MRN' from dual union all
select 35625,'MRN' from dual union all
select 35626,'MRN' from dual union all
select 35627,'MRN' from dual union all
select 35628,'MRN' from dual union all
select 35629,'MRN' from dual union all
select 3563,'RCAR' from dual union all
select 35630,'MRN' from dual union all
select 35631,'MRN' from dual union all
select 35632,'MRN' from dual union all
select 35633,'MRN' from dual union all
select 35634,'MRN' from dual union all
select 35635,'MRN' from dual union all
select 35636,'MRN' from dual union all
select 35637,'MRN' from dual union all
select 35638,'MRN' from dual union all
select 35639,'MRN' from dual union all
select 3564,'RCAR' from dual union all
select 35640,'MRN' from dual union all
select 35641,'MRN' from dual union all
select 35642,'MRN' from dual union all
select 35643,'MRN' from dual union all
select 35644,'MRN' from dual union all
select 35645,'MRN' from dual union all
select 35646,'MRN' from dual union all
select 35647,'MRN' from dual union all
select 35648,'MRN' from dual union all
select 35649,'MRN' from dual union all
select 3565,'USA' from dual union all
select 35650,'MRN' from dual union all
select 35651,'MRN' from dual union all
select 35652,'MRN' from dual union all
select 35653,'MRN' from dual union all
select 35654,'MRN' from dual union all
select 35655,'MRN' from dual union all
select 35656,'MRN' from dual union all
select 35657,'MRN' from dual union all
select 35658,'MRN' from dual union all
select 35659,'MRN' from dual union all
select 3566,'USAF' from dual union all
select 35660,'MRN' from dual union all
select 35661,'MRN' from dual union all
select 35662,'MRN' from dual;
insert into LAUNCH_AGENCY
select 35663,'MRN' from dual union all
select 35664,'MRN' from dual union all
select 35665,'MRN' from dual union all
select 35666,'MRN' from dual union all
select 35667,'MRN' from dual union all
select 35668,'MRN' from dual union all
select 35669,'MRN' from dual union all
select 3567,'ORDRSA' from dual union all
select 35670,'MRN' from dual union all
select 35671,'MRN' from dual union all
select 35672,'MRN' from dual union all
select 35673,'MRN' from dual union all
select 35674,'MRN' from dual union all
select 35675,'MRN' from dual union all
select 35676,'MRN' from dual union all
select 35677,'MRN' from dual union all
select 35678,'MRN' from dual union all
select 35679,'MRN' from dual union all
select 3568,'ORDRSA' from dual union all
select 35680,'MRN' from dual union all
select 35681,'MRN' from dual union all
select 35682,'MRN' from dual union all
select 35683,'MRN' from dual union all
select 35684,'MRN' from dual union all
select 35685,'MRN' from dual union all
select 35686,'MRN' from dual union all
select 35687,'MRN' from dual union all
select 35688,'MRN' from dual union all
select 35689,'MRN' from dual union all
select 3569,'USA' from dual union all
select 35690,'MRN' from dual union all
select 35691,'MRN' from dual union all
select 35692,'AN' from dual union all
select 35693,'MRN' from dual union all
select 35694,'MRN' from dual union all
select 35695,'MRN' from dual union all
select 35696,'MRN' from dual union all
select 35697,'MRN' from dual union all
select 35698,'MRN' from dual union all
select 35699,'MRN' from dual union all
select 357,'WER485-2' from dual union all
select 3570,'USA' from dual union all
select 35700,'MRN' from dual union all
select 35701,'MRN' from dual union all
select 35702,'MRN' from dual union all
select 35703,'MRN' from dual union all
select 35704,'MRN' from dual union all
select 35705,'MRN' from dual union all
select 35706,'MRN' from dual union all
select 35707,'MRN' from dual union all
select 35708,'MRN' from dual union all
select 35709,'MRN' from dual union all
select 3571,'USA' from dual union all
select 35710,'MRN' from dual union all
select 35711,'MRN' from dual union all
select 35712,'MRN' from dual union all
select 35713,'MRN' from dual union all
select 35714,'MRN' from dual union all
select 35715,'MRN' from dual union all
select 35716,'MRN' from dual union all
select 35717,'MRN' from dual union all
select 35718,'MRN' from dual union all
select 35719,'MRN' from dual union all
select 3572,'USA' from dual union all
select 35720,'MRN' from dual union all
select 35721,'MRN' from dual union all
select 35722,'MRN' from dual union all
select 35723,'MRN' from dual union all
select 35724,'MRN' from dual union all
select 35725,'MRN' from dual union all
select 35726,'MRN' from dual union all
select 35727,'MRN' from dual union all
select 35728,'MRN' from dual union all
select 35729,'MRN' from dual union all
select 3573,'USA' from dual union all
select 35730,'MRN' from dual union all
select 35731,'MRN' from dual union all
select 35732,'MRN' from dual union all
select 35733,'MRN' from dual union all
select 35734,'MRN' from dual union all
select 35735,'MRN' from dual union all
select 35736,'MRN' from dual union all
select 35737,'MRN' from dual union all
select 35738,'MRN' from dual union all
select 35739,'MRN' from dual union all
select 3574,'USA' from dual union all
select 35740,'MRN' from dual union all
select 35741,'MRN' from dual union all
select 35742,'MRN' from dual union all
select 35743,'MRN' from dual union all
select 35744,'MRN' from dual union all
select 35745,'MRN' from dual union all
select 35746,'MRN' from dual union all
select 35747,'MRN' from dual union all
select 35748,'MRN' from dual union all
select 35749,'MRN' from dual union all
select 3575,'USAF' from dual union all
select 35750,'MRN' from dual union all
select 35751,'MRN' from dual union all
select 35752,'MRN' from dual;
insert into LAUNCH_AGENCY
select 35753,'MRN' from dual union all
select 35754,'MRN' from dual union all
select 35755,'MRN' from dual union all
select 35756,'MRN' from dual union all
select 35757,'MRN' from dual union all
select 35758,'MRN' from dual union all
select 35759,'MRN' from dual union all
select 3576,'USAF' from dual union all
select 35760,'MRN' from dual union all
select 35761,'MRN' from dual union all
select 35762,'MRN' from dual union all
select 35763,'MRN' from dual union all
select 35764,'MRN' from dual union all
select 35765,'MRN' from dual union all
select 35766,'MRN' from dual union all
select 35767,'MRN' from dual union all
select 35768,'MRN' from dual union all
select 35769,'MRN' from dual union all
select 3577,'NRL' from dual union all
select 35770,'MRN' from dual union all
select 35771,'MRN' from dual union all
select 35772,'MRN' from dual union all
select 35773,'MRN' from dual union all
select 35774,'MRN' from dual union all
select 35775,'MRN' from dual union all
select 35776,'MRN' from dual union all
select 35777,'MRN' from dual union all
select 35778,'MRN' from dual union all
select 35779,'MRN' from dual union all
select 3578,'USN' from dual union all
select 35780,'MRN' from dual union all
select 35781,'MRN' from dual union all
select 35782,'MRN' from dual union all
select 35783,'MRN' from dual union all
select 35784,'MRN' from dual union all
select 35785,'MRN' from dual union all
select 35786,'MRN' from dual union all
select 35787,'PIHM' from dual union all
select 35788,'MRN' from dual union all
select 35789,'MRN' from dual union all
select 3579,'USN' from dual union all
select 35790,'MRN' from dual union all
select 35791,'MRN' from dual union all
select 35792,'MRN' from dual union all
select 35793,'MRN' from dual union all
select 35794,'MRN' from dual union all
select 35795,'MRN' from dual union all
select 35796,'MRN' from dual union all
select 35797,'MRN' from dual union all
select 35798,'MRN' from dual union all
select 35799,'MRN' from dual union all
select 358,'WER485-2' from dual union all
select 3580,'USAF' from dual union all
select 35800,'MRN' from dual union all
select 35801,'MRN' from dual union all
select 35802,'MRN' from dual union all
select 35803,'MRN' from dual union all
select 35804,'MRN' from dual union all
select 35805,'MRN' from dual union all
select 35806,'MRN' from dual union all
select 35807,'MRN' from dual union all
select 35808,'MRN' from dual union all
select 35809,'MRN' from dual union all
select 3581,'NRL' from dual union all
select 35810,'MRN' from dual union all
select 35811,'MRN' from dual union all
select 35812,'MRN' from dual union all
select 35813,'MRN' from dual union all
select 35814,'MRN' from dual union all
select 35815,'MRN' from dual union all
select 35816,'MRN' from dual union all
select 35817,'MRN' from dual union all
select 35818,'MRN' from dual union all
select 35819,'MRN' from dual union all
select 3582,'NRL' from dual union all
select 35820,'MRN' from dual union all
select 35821,'MRN' from dual union all
select 35822,'MRN' from dual union all
select 35823,'MRN' from dual union all
select 35824,'MRN' from dual union all
select 35825,'MRN' from dual union all
select 35826,'MRN' from dual union all
select 35827,'MRN' from dual union all
select 35828,'MRN' from dual union all
select 35829,'MRN' from dual union all
select 3583,'USN' from dual union all
select 35830,'MRN' from dual union all
select 35831,'MRN' from dual union all
select 35832,'MRN' from dual union all
select 35833,'MRN' from dual union all
select 35834,'MRN' from dual union all
select 35835,'MRN' from dual union all
select 35836,'MRN' from dual union all
select 35837,'MRN' from dual union all
select 35838,'MRN' from dual union all
select 35839,'MRN' from dual union all
select 3584,'USN' from dual union all
select 35840,'MRN' from dual union all
select 35841,'MRN' from dual union all
select 35842,'MRN' from dual;
insert into LAUNCH_AGENCY
select 35843,'MRN' from dual union all
select 35844,'AN' from dual union all
select 35845,'MRN' from dual union all
select 35846,'MRN' from dual union all
select 35847,'MRN' from dual union all
select 35848,'MRN' from dual union all
select 35849,'MRN' from dual union all
select 3585,'BWEHR' from dual union all
select 35850,'MRN' from dual union all
select 35851,'MRN' from dual union all
select 35852,'MRN' from dual union all
select 35853,'MRN' from dual union all
select 35854,'MRN' from dual union all
select 35855,'MRN' from dual union all
select 35856,'MRN' from dual union all
select 35857,'MRN' from dual union all
select 35858,'MRN' from dual union all
select 35859,'MRN' from dual union all
select 3586,'BWEHR' from dual union all
select 35860,'MRN' from dual union all
select 35861,'MRN' from dual union all
select 35862,'MRN' from dual union all
select 35863,'MRN' from dual union all
select 35864,'MRN' from dual union all
select 35865,'MRN' from dual union all
select 35866,'MRN' from dual union all
select 35867,'MRN' from dual union all
select 35868,'MRN' from dual union all
select 35869,'MRN' from dual union all
select 3587,'UZKA' from dual union all
select 35870,'MRN' from dual union all
select 35871,'MRN' from dual union all
select 35872,'MRN' from dual union all
select 35873,'MRN' from dual union all
select 35874,'MRN' from dual union all
select 35875,'MRN' from dual union all
select 35876,'MRN' from dual union all
select 35877,'MRN' from dual union all
select 35878,'MRN' from dual union all
select 35879,'MRN' from dual union all
select 3588,'UZKA' from dual union all
select 35880,'MRN' from dual union all
select 35881,'MRN' from dual union all
select 35882,'MRN' from dual union all
select 35883,'MRN' from dual union all
select 35884,'MRN' from dual union all
select 35885,'MRN' from dual union all
select 35886,'MRN' from dual union all
select 35887,'MRN' from dual union all
select 35888,'NASA' from dual union all
select 35889,'NASA' from dual union all
select 3589,'UZKA' from dual union all
select 35890,'NASA' from dual union all
select 35891,'US' from dual union all
select 35892,'NASA' from dual union all
select 35893,'NASA' from dual union all
select 35894,'NASA' from dual union all
select 35895,'MRN' from dual union all
select 35896,'NASA' from dual union all
select 35897,'MRN' from dual union all
select 35898,'MRN' from dual union all
select 35899,'MRN' from dual union all
select 359,'WER485-2' from dual union all
select 3590,'UZKA' from dual union all
select 35900,'MRN' from dual union all
select 35901,'MRN' from dual union all
select 35902,'MRN' from dual union all
select 35903,'MRN' from dual union all
select 35904,'MRN' from dual union all
select 35905,'MRN' from dual union all
select 35906,'MRN' from dual union all
select 35907,'NASA' from dual union all
select 35908,'NASA' from dual union all
select 35909,'NASA' from dual union all
select 3591,'UZKA' from dual union all
select 35910,'NASA' from dual union all
select 35911,'NASA' from dual union all
select 35912,'NASA' from dual union all
select 35913,'NASA' from dual union all
select 35914,'NASA' from dual union all
select 35915,'NASA' from dual union all
select 35916,'NASA' from dual union all
select 35917,'MRN' from dual union all
select 35918,'NASA' from dual union all
select 35919,'NASA' from dual union all
select 3592,'ARDC' from dual union all
select 35920,'NASA' from dual union all
select 35921,'NASA' from dual union all
select 35922,'MRN' from dual union all
select 35923,'NASA' from dual union all
select 35924,'NASA' from dual union all
select 35925,'MRN' from dual union all
select 35926,'MRN' from dual union all
select 35927,'MRN' from dual union all
select 35928,'MRN' from dual union all
select 35929,'MRN' from dual union all
select 3593,'NII88A' from dual union all
select 35930,'MRN' from dual union all
select 35931,'MRN' from dual union all
select 35932,'MRN' from dual;
insert into LAUNCH_AGENCY
select 35933,'MRN' from dual union all
select 35934,'MRN' from dual union all
select 35935,'MRN' from dual union all
select 35936,'MRN' from dual union all
select 35937,'MRN' from dual union all
select 35938,'MRN' from dual union all
select 35939,'MRN' from dual union all
select 3594,'UZKA' from dual union all
select 35940,'MRN' from dual union all
select 35941,'MRN' from dual union all
select 35942,'MRN' from dual union all
select 35943,'MRN' from dual union all
select 35944,'MRN' from dual union all
select 35945,'MRN' from dual union all
select 35946,'MRN' from dual union all
select 35947,'MRN' from dual union all
select 35948,'MRN' from dual union all
select 35949,'MRN' from dual union all
select 3595,'UZKA' from dual union all
select 35950,'MRN' from dual union all
select 35951,'MRN' from dual union all
select 35952,'MRN' from dual union all
select 35953,'MRN' from dual union all
select 35954,'MRN' from dual union all
select 35955,'MRN' from dual union all
select 35956,'MRN' from dual union all
select 35957,'MRN' from dual union all
select 35958,'MRN' from dual union all
select 35959,'MRN' from dual union all
select 3596,'UZKA' from dual union all
select 35960,'MRN' from dual union all
select 35961,'MRN' from dual union all
select 35962,'MRN' from dual union all
select 35963,'MRN' from dual union all
select 35964,'MRN' from dual union all
select 35965,'MRN' from dual union all
select 35966,'MRN' from dual union all
select 35967,'MRN' from dual union all
select 35968,'MRN' from dual union all
select 35969,'MRN' from dual union all
select 3597,'UNRV' from dual union all
select 35970,'MRN' from dual union all
select 35971,'MRN' from dual union all
select 35972,'MRN' from dual union all
select 35973,'MRN' from dual union all
select 35974,'MRN' from dual union all
select 35975,'MRN' from dual union all
select 35976,'MRN' from dual union all
select 35977,'MRN' from dual union all
select 35978,'MRN' from dual union all
select 35979,'MRN' from dual union all
select 3598,'NRL' from dual union all
select 35980,'MRN' from dual union all
select 35981,'MRN' from dual union all
select 35982,'MRN' from dual union all
select 35983,'MRN' from dual union all
select 35984,'MRN' from dual union all
select 35985,'MRN' from dual union all
select 35986,'MRN' from dual union all
select 35987,'MRN' from dual union all
select 35988,'MRN' from dual union all
select 35989,'MRN' from dual union all
select 3599,'NRL' from dual union all
select 35990,'MRN' from dual union all
select 35991,'MRN' from dual union all
select 35992,'MRN' from dual union all
select 35993,'MRN' from dual union all
select 35994,'MRN' from dual union all
select 35995,'MRN' from dual union all
select 35996,'MRN' from dual union all
select 35997,'MRN' from dual union all
select 35998,'MRN' from dual union all
select 35999,'MRN' from dual union all
select 36,'WEHR' from dual union all
select 360,'WER485-2' from dual union all
select 3600,'UNRV' from dual union all
select 36000,'MRN' from dual union all
select 36001,'MRN' from dual union all
select 36002,'MRN' from dual union all
select 36003,'MRN' from dual union all
select 36004,'MRN' from dual union all
select 36005,'MRN' from dual union all
select 36006,'MRN' from dual union all
select 36007,'MRN' from dual union all
select 36008,'MRN' from dual union all
select 36009,'MRN' from dual union all
select 3601,'NRL' from dual union all
select 36010,'MRN' from dual union all
select 36011,'MRN' from dual union all
select 36012,'MRN' from dual union all
select 36013,'MRN' from dual union all
select 36014,'MRN' from dual union all
select 36015,'MRN' from dual union all
select 36016,'MRN' from dual union all
select 36017,'MRN' from dual union all
select 36018,'NASA' from dual union all
select 36019,'MRN' from dual union all
select 3602,'6555A' from dual union all
select 36020,'MRN' from dual union all
select 36021,'MRN' from dual;
insert into LAUNCH_AGENCY
select 36022,'MRN' from dual union all
select 36023,'MRN' from dual union all
select 36024,'MRN' from dual union all
select 36025,'MRN' from dual union all
select 36026,'MRN' from dual union all
select 36027,'MRN' from dual union all
select 36028,'MRN' from dual union all
select 36029,'MRN' from dual union all
select 3603,'USA' from dual union all
select 36030,'NASA' from dual union all
select 36031,'MRN' from dual union all
select 36032,'NASA' from dual union all
select 36033,'NASA' from dual union all
select 36034,'MRN' from dual union all
select 36035,'MRN' from dual union all
select 36036,'MRN' from dual union all
select 36037,'MRN' from dual union all
select 36038,'MRN' from dual union all
select 36039,'MRN' from dual union all
select 3604,'NRL' from dual union all
select 36040,'MRN' from dual union all
select 36041,'MRN' from dual union all
select 36042,'MRN' from dual union all
select 36043,'MRN' from dual union all
select 36044,'MRN' from dual union all
select 36045,'MRN' from dual union all
select 36046,'MRN' from dual union all
select 36047,'MRN' from dual union all
select 36048,'MRN' from dual union all
select 36049,'MRN' from dual union all
select 3605,'USAF' from dual union all
select 36050,'MRN' from dual union all
select 36051,'MRN' from dual union all
select 36052,'MRN' from dual union all
select 36053,'MRN' from dual union all
select 36054,'MRN' from dual union all
select 36055,'MRN' from dual union all
select 36056,'MRN' from dual union all
select 36057,'MRN' from dual union all
select 36058,'MRN' from dual union all
select 36059,'MRN' from dual union all
select 3606,'UNRV' from dual union all
select 36060,'AN' from dual union all
select 36061,'AN' from dual union all
select 36062,'AN' from dual union all
select 36063,'AN' from dual union all
select 36064,'MRN' from dual union all
select 36065,'AN' from dual union all
select 36066,'AN' from dual union all
select 36067,'AN' from dual union all
select 36068,'AN' from dual union all
select 36069,'AN' from dual union all
select 3607,'UZKA' from dual union all
select 36070,'MRN' from dual union all
select 36071,'AN' from dual union all
select 36072,'AN' from dual union all
select 36073,'AN' from dual union all
select 36074,'AN' from dual union all
select 36075,'AN' from dual union all
select 36076,'AN' from dual union all
select 36077,'MRN' from dual union all
select 36078,'MRN' from dual union all
select 36079,'MRN' from dual union all
select 3608,'UZKA' from dual union all
select 36080,'MRN' from dual union all
select 36081,'AN' from dual union all
select 36082,'AN' from dual union all
select 36083,'AN' from dual union all
select 36084,'AN' from dual union all
select 36085,'AN' from dual union all
select 36086,'AN' from dual union all
select 36087,'MRN' from dual union all
select 36088,'AN' from dual union all
select 36089,'MRN' from dual union all
select 3609,'UZKA' from dual union all
select 36090,'AN' from dual union all
select 36091,'MRN' from dual union all
select 36092,'AN' from dual union all
select 36093,'NASA' from dual union all
select 36094,'AN' from dual union all
select 36095,'AN' from dual union all
select 36096,'AN' from dual union all
select 36097,'AN' from dual union all
select 36098,'AN' from dual union all
select 36099,'AN' from dual union all
select 361,'WER485-2' from dual union all
select 3610,'UZKA' from dual union all
select 36100,'NRCC' from dual union all
select 36101,'AN' from dual union all
select 36102,'AN' from dual union all
select 36103,'AN' from dual union all
select 36104,'AN' from dual union all
select 36105,'AN' from dual union all
select 36106,'AN' from dual union all
select 36107,'AN' from dual union all
select 36108,'AN' from dual union all
select 36109,'MRN' from dual union all
select 3611,'UZKA' from dual union all
select 36110,'AN' from dual union all
select 36111,'AN' from dual;
insert into LAUNCH_AGENCY
select 36112,'AN' from dual union all
select 36113,'AN' from dual union all
select 36114,'AN' from dual union all
select 36115,'AN' from dual union all
select 36116,'AN' from dual union all
select 36117,'AN' from dual union all
select 36118,'AN' from dual union all
select 36119,'MRN' from dual union all
select 3612,'UZKA' from dual union all
select 36120,'AN' from dual union all
select 36121,'AN' from dual union all
select 36122,'AN' from dual union all
select 36123,'AN' from dual union all
select 36124,'AN' from dual union all
select 36125,'AN' from dual union all
select 36126,'AN' from dual union all
select 36127,'AN' from dual union all
select 36128,'AN' from dual union all
select 36129,'AN' from dual union all
select 3613,'UZKA' from dual union all
select 36130,'AN' from dual union all
select 36131,'AN' from dual union all
select 36132,'AN' from dual union all
select 36133,'MRN' from dual union all
select 36134,'AN' from dual union all
select 36135,'DLR' from dual union all
select 36136,'AN' from dual union all
select 36137,'AN' from dual union all
select 36138,'AN' from dual union all
select 36139,'AN' from dual union all
select 3614,'UZKA' from dual union all
select 36140,'AN' from dual union all
select 36141,'MRN' from dual union all
select 36142,'AN' from dual union all
select 36143,'AN' from dual union all
select 36144,'AN' from dual union all
select 36145,'MRN' from dual union all
select 36146,'AN' from dual union all
select 36147,'MRN' from dual union all
select 36148,'MRN' from dual union all
select 36149,'MRN' from dual union all
select 3615,'UZKA' from dual union all
select 36150,'AN' from dual union all
select 36151,'AN' from dual union all
select 36152,'AN' from dual union all
select 36153,'AN' from dual union all
select 36154,'AN' from dual union all
select 36155,'MRN' from dual union all
select 36156,'AN' from dual union all
select 36157,'MRN' from dual union all
select 36158,'AN' from dual union all
select 36159,'AN' from dual union all
select 3616,'UZKA' from dual union all
select 36160,'AN' from dual union all
select 36161,'AN' from dual union all
select 36162,'AN' from dual union all
select 36163,'AN' from dual union all
select 36164,'AN' from dual union all
select 36165,'AN' from dual union all
select 36166,'AN' from dual union all
select 36167,'AN' from dual union all
select 36168,'AN' from dual union all
select 36169,'AN' from dual union all
select 3617,'UZKA' from dual union all
select 36170,'AN' from dual union all
select 36171,'MRN' from dual union all
select 36172,'AN' from dual union all
select 36173,'AN' from dual union all
select 36174,'AN' from dual union all
select 36175,'MRN' from dual union all
select 36176,'AN' from dual union all
select 36177,'AN' from dual union all
select 36178,'AN' from dual union all
select 36179,'NASA' from dual union all
select 3618,'UZKA' from dual union all
select 36180,'AN' from dual union all
select 36181,'AN' from dual union all
select 36182,'AN' from dual union all
select 36183,'AN' from dual union all
select 36184,'MRN' from dual union all
select 36185,'AN' from dual union all
select 36186,'AN' from dual union all
select 36187,'MRN' from dual union all
select 36188,'AN' from dual union all
select 36189,'AN' from dual union all
select 3619,'UZKA' from dual union all
select 36190,'MRN' from dual union all
select 36191,'AN' from dual union all
select 36192,'AN' from dual union all
select 36193,'AN' from dual union all
select 36194,'AN' from dual union all
select 36195,'MRN' from dual union all
select 36196,'AN' from dual union all
select 36197,'AN' from dual union all
select 36198,'AN' from dual union all
select 36199,'MRN' from dual union all
select 362,'WER485-2' from dual union all
select 3620,'UZKA' from dual union all
select 36200,'MRN' from dual union all
select 36201,'AN' from dual;
insert into LAUNCH_AGENCY
select 36202,'MRN' from dual union all
select 36203,'AN' from dual union all
select 36204,'AN' from dual union all
select 36205,'AN' from dual union all
select 36206,'AN' from dual union all
select 36207,'AN' from dual union all
select 36208,'NASA' from dual union all
select 36209,'NASA' from dual union all
select 3621,'UZKA' from dual union all
select 36210,'US' from dual union all
select 36211,'AN' from dual union all
select 36212,'AN' from dual union all
select 36213,'AN' from dual union all
select 36214,'MRN' from dual union all
select 36215,'AN' from dual union all
select 36216,'AN' from dual union all
select 36217,'MRN' from dual union all
select 36218,'MRN' from dual union all
select 36219,'MRN' from dual union all
select 3622,'UZKA' from dual union all
select 36220,'AN' from dual union all
select 36221,'AN' from dual union all
select 36222,'MRN' from dual union all
select 36223,'AN' from dual union all
select 36224,'MRN' from dual union all
select 36225,'AN' from dual union all
select 36226,'AN' from dual union all
select 36227,'MRN' from dual union all
select 36228,'MRN' from dual union all
select 36229,'AN' from dual union all
select 3623,'UZKA' from dual union all
select 36230,'AN' from dual union all
select 36231,'MRN' from dual union all
select 36232,'MRN' from dual union all
select 36233,'MRN' from dual union all
select 36234,'AN' from dual union all
select 36235,'AN' from dual union all
select 36236,'AN' from dual union all
select 36237,'MRN' from dual union all
select 36238,'MRN' from dual union all
select 36239,'AN' from dual union all
select 3624,'UZKA' from dual union all
select 36240,'AN' from dual union all
select 36241,'MRN' from dual union all
select 36242,'AN' from dual union all
select 36243,'AN' from dual union all
select 36244,'AN' from dual union all
select 36245,'AN' from dual union all
select 36246,'AN' from dual union all
select 36247,'MRN' from dual union all
select 36248,'AN' from dual union all
select 36249,'AN' from dual union all
select 3625,'UZKA' from dual union all
select 36250,'MRN' from dual union all
select 36251,'AN' from dual union all
select 36252,'AN' from dual union all
select 36253,'AN' from dual union all
select 36254,'AN' from dual union all
select 36255,'AN' from dual union all
select 36256,'AN' from dual union all
select 36257,'AN' from dual union all
select 36258,'AN' from dual union all
select 36259,'AN' from dual union all
select 3626,'UZKA' from dual union all
select 36260,'AN' from dual union all
select 36261,'MRN' from dual union all
select 36262,'MRN' from dual union all
select 36263,'AN' from dual union all
select 36264,'AN' from dual union all
select 36265,'AN' from dual union all
select 36266,'AN' from dual union all
select 36267,'AN' from dual union all
select 36268,'AN' from dual union all
select 36269,'AN' from dual union all
select 3627,'UZKA' from dual union all
select 36270,'MRN' from dual union all
select 36271,'MRN' from dual union all
select 36272,'AN' from dual union all
select 36273,'MRN' from dual union all
select 36274,'AN' from dual union all
select 36275,'AN' from dual union all
select 36276,'MRN' from dual union all
select 36277,'AN' from dual union all
select 36278,'MRN' from dual union all
select 36279,'MRN' from dual union all
select 3628,'UZKA' from dual union all
select 36280,'AN' from dual union all
select 36281,'AN' from dual union all
select 36282,'AN' from dual union all
select 36283,'AN' from dual union all
select 36284,'AN' from dual union all
select 36285,'MRN' from dual union all
select 36286,'AN' from dual union all
select 36287,'AN' from dual union all
select 36288,'AN' from dual union all
select 36289,'MRN' from dual union all
select 3629,'UZKA' from dual union all
select 36290,'AN' from dual union all
select 36291,'AN' from dual union all
select 36292,'AN' from dual;
insert into LAUNCH_AGENCY
select 36293,'AN' from dual union all
select 36294,'AN' from dual union all
select 36295,'AN' from dual union all
select 36296,'AN' from dual union all
select 36297,'AN' from dual union all
select 36298,'AN' from dual union all
select 36299,'AN' from dual union all
select 363,'WER485-2' from dual union all
select 3630,'UZKA' from dual union all
select 36300,'AN' from dual union all
select 36301,'AN' from dual union all
select 36302,'AN' from dual union all
select 36303,'MRN' from dual union all
select 36304,'AN' from dual union all
select 36305,'AN' from dual union all
select 36306,'AN' from dual union all
select 36307,'AN' from dual union all
select 36308,'AN' from dual union all
select 36309,'MRN' from dual union all
select 3631,'UZKA' from dual union all
select 36310,'MRN' from dual union all
select 36311,'AN' from dual union all
select 36312,'MRN' from dual union all
select 36313,'MRN' from dual union all
select 36314,'MRN' from dual union all
select 36315,'AN' from dual union all
select 36316,'AN' from dual union all
select 36317,'AN' from dual union all
select 36318,'AN' from dual union all
select 36319,'AN' from dual union all
select 3632,'UZKA' from dual union all
select 36320,'MRN' from dual union all
select 36321,'AN' from dual union all
select 36322,'AN' from dual union all
select 36323,'AN' from dual union all
select 36324,'MRN' from dual union all
select 36325,'MRN' from dual union all
select 36326,'MRN' from dual union all
select 36327,'AN' from dual union all
select 36328,'AN' from dual union all
select 36329,'AN' from dual union all
select 3633,'UZKA' from dual union all
select 36330,'AN' from dual union all
select 36331,'MRN' from dual union all
select 36332,'AN' from dual union all
select 36333,'AN' from dual union all
select 36334,'AN' from dual union all
select 36335,'AN' from dual union all
select 36336,'AN' from dual union all
select 36337,'MRN' from dual union all
select 36338,'AN' from dual union all
select 36339,'AN' from dual union all
select 3634,'UNRV' from dual union all
select 36340,'AN' from dual union all
select 36341,'AN' from dual union all
select 36342,'AN' from dual union all
select 36343,'AN' from dual union all
select 36344,'AN' from dual union all
select 36345,'AN' from dual union all
select 36346,'AN' from dual union all
select 36347,'MRN' from dual union all
select 36348,'AN' from dual union all
select 36349,'AN' from dual union all
select 3635,'UNRV' from dual union all
select 36350,'AN' from dual union all
select 36351,'MRN' from dual union all
select 36352,'AN' from dual union all
select 36353,'AN' from dual union all
select 36354,'AN' from dual union all
select 36355,'AN' from dual union all
select 36356,'MRN' from dual union all
select 36357,'AN' from dual union all
select 36358,'MRN' from dual union all
select 36359,'AN' from dual union all
select 3636,'UNRV' from dual union all
select 36360,'AN' from dual union all
select 36361,'AN' from dual union all
select 36362,'AN' from dual union all
select 36363,'AN' from dual union all
select 36364,'AN' from dual union all
select 36365,'MRN' from dual union all
select 36366,'AN' from dual union all
select 36367,'MRN' from dual union all
select 36368,'AN' from dual union all
select 36369,'AN' from dual union all
select 3637,'UNRV' from dual union all
select 36370,'AN' from dual union all
select 36371,'MRN' from dual union all
select 36372,'AN' from dual union all
select 36373,'AN' from dual union all
select 36374,'AN' from dual union all
select 36375,'AN' from dual union all
select 36376,'MRN' from dual union all
select 36377,'AN' from dual union all
select 36378,'AN' from dual union all
select 36379,'AN' from dual union all
select 3638,'USN' from dual union all
select 36380,'AN' from dual union all
select 36381,'AN' from dual union all
select 36382,'AN' from dual;
insert into LAUNCH_AGENCY
select 36383,'MRN' from dual union all
select 36384,'AN' from dual union all
select 36385,'AN' from dual union all
select 36386,'AN' from dual union all
select 36387,'MRN' from dual union all
select 36388,'AN' from dual union all
select 36389,'AN' from dual union all
select 3639,'UNRV' from dual union all
select 36390,'AN' from dual union all
select 36391,'AN' from dual union all
select 36392,'AN' from dual union all
select 36393,'AN' from dual union all
select 36394,'AN' from dual union all
select 36395,'AN' from dual union all
select 36396,'MRN' from dual union all
select 36397,'AN' from dual union all
select 36398,'AN' from dual union all
select 36399,'AN' from dual union all
select 364,'WEHR' from dual union all
select 3640,'UNRV' from dual union all
select 36400,'AN' from dual union all
select 36401,'AN' from dual union all
select 36402,'AN' from dual union all
select 36403,'AN' from dual union all
select 36404,'AN' from dual union all
select 36405,'AN' from dual union all
select 36406,'AN' from dual union all
select 36407,'AN' from dual union all
select 36408,'AN' from dual union all
select 36409,'AN' from dual union all
select 3641,'UNRV' from dual union all
select 36410,'AN' from dual union all
select 36411,'AN' from dual union all
select 36412,'AN' from dual union all
select 36413,'MRN' from dual union all
select 36414,'AN' from dual union all
select 36415,'AN' from dual union all
select 36416,'AN' from dual union all
select 36417,'AN' from dual union all
select 36418,'AN' from dual union all
select 36419,'AN' from dual union all
select 3642,'UNRV' from dual union all
select 36420,'AN' from dual union all
select 36421,'AN' from dual union all
select 36422,'AN' from dual union all
select 36423,'AN' from dual union all
select 36424,'AN' from dual union all
select 36425,'AN' from dual union all
select 36426,'AN' from dual union all
select 36427,'AN' from dual union all
select 36428,'AN' from dual union all
select 36429,'AN' from dual union all
select 3643,'UNRV' from dual union all
select 36430,'AN' from dual union all
select 36431,'MRN' from dual union all
select 36432,'MRN' from dual union all
select 36433,'AN' from dual union all
select 36434,'AN' from dual union all
select 36435,'MRN' from dual union all
select 36436,'AN' from dual union all
select 36437,'AN' from dual union all
select 36438,'AN' from dual union all
select 36439,'AN' from dual union all
select 3644,'UNRV' from dual union all
select 36440,'AN' from dual union all
select 36441,'AN' from dual union all
select 36442,'MRN' from dual union all
select 36443,'AN' from dual union all
select 36444,'AN' from dual union all
select 36445,'AN' from dual union all
select 36446,'AN' from dual union all
select 36447,'AN' from dual union all
select 36448,'RVSN' from dual union all
select 36449,'RVSN' from dual union all
select 3645,'UNRV' from dual union all
select 36450,'RVSN' from dual union all
select 36451,'NASA' from dual union all
select 36452,'RVSN' from dual union all
select 36453,'RVSN' from dual union all
select 36454,'RVSN' from dual union all
select 36455,'RVSN' from dual union all
select 36456,'RVSN' from dual union all
select 36457,'CNES' from dual union all
select 36458,'NASA' from dual union all
select 36459,'RVSN' from dual union all
select 3646,'UNRV' from dual union all
select 36460,'RVSN' from dual union all
select 36461,'ISAS' from dual union all
select 36462,'RVSN' from dual union all
select 36463,'RVSN' from dual union all
select 36464,'USAF' from dual union all
select 36465,'RVSN' from dual union all
select 36466,'RVSN' from dual union all
select 36467,'RVSN' from dual union all
select 36468,'RVSN' from dual union all
select 36469,'RVSN' from dual union all
select 3647,'UNRV' from dual union all
select 36470,'RVSN' from dual union all
select 36471,'RVSN' from dual union all
select 36472,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 36473,'RVSN' from dual union all
select 36474,'NASA' from dual union all
select 36475,'RVSN' from dual union all
select 36476,'RVSN' from dual union all
select 36477,'RVSN' from dual union all
select 36478,'RVSN' from dual union all
select 36479,'USAF' from dual union all
select 3648,'UNRV' from dual union all
select 36480,'RVSN' from dual union all
select 36481,'RVSN' from dual union all
select 36482,'RVSN' from dual union all
select 36483,'RVSN' from dual union all
select 36484,'CRA' from dual union all
select 36485,'NASA' from dual union all
select 36486,'CNES' from dual union all
select 36487,'USAF' from dual union all
select 36488,'RVSN' from dual union all
select 36489,'NASA' from dual union all
select 3649,'NII88A' from dual union all
select 36490,'10ADS' from dual union all
select 36491,'RVSN' from dual union all
select 36492,'RVSN' from dual union all
select 36493,'RVSN' from dual union all
select 36494,'RVSN' from dual union all
select 36495,'RVSN' from dual union all
select 36496,'RVSN' from dual union all
select 36497,'RVSN' from dual union all
select 36498,'USAF' from dual union all
select 36499,'NASA' from dual union all
select 365,'WEHR' from dual union all
select 3650,'UNRV' from dual union all
select 36500,'RVSN' from dual union all
select 36501,'RVSN' from dual union all
select 36502,'AFSC' from dual union all
select 36503,'RVSN' from dual union all
select 36504,'NASA' from dual union all
select 36505,'RVSN' from dual union all
select 36506,'RVSN' from dual union all
select 36507,'RVSN' from dual union all
select 36508,'RVSN' from dual union all
select 36509,'RVSN' from dual union all
select 3651,'UNRV' from dual union all
select 36510,'RVSN' from dual union all
select 36511,'RVSN' from dual union all
select 36512,'RVSN' from dual union all
select 36513,'NASA' from dual union all
select 36514,'RVSN' from dual union all
select 36515,'RVSN' from dual union all
select 36516,'RVSN' from dual union all
select 36517,'SAST' from dual union all
select 36518,'RVSN' from dual union all
select 36519,'NASA' from dual union all
select 3652,'UNRV' from dual union all
select 36520,'RVSN' from dual union all
select 36521,'RVSN' from dual union all
select 36522,'USAF' from dual union all
select 36523,'RVSN' from dual union all
select 36524,'NASA' from dual union all
select 36525,'RVSN' from dual union all
select 36526,'RVSN' from dual union all
select 36527,'RVSN' from dual union all
select 36528,'RVSN' from dual union all
select 36529,'NASDA' from dual union all
select 3653,'UNRV' from dual union all
select 36530,'USAF' from dual union all
select 36531,'RVSN' from dual union all
select 36532,'RVSN' from dual union all
select 36533,'RVSN' from dual union all
select 36534,'RVSN' from dual union all
select 36535,'RVSN' from dual union all
select 36536,'RVSN' from dual union all
select 36537,'RVSN' from dual union all
select 36538,'NASA' from dual union all
select 36539,'CNES' from dual union all
select 3654,'UNRV' from dual union all
select 36540,'RVSN' from dual union all
select 36541,'RVSN' from dual union all
select 36542,'RVSN' from dual union all
select 36543,'NASA' from dual union all
select 36544,'RVSN' from dual union all
select 36545,'USAF' from dual union all
select 36546,'AFSC' from dual union all
select 36547,'NASA' from dual union all
select 36548,'RVSN' from dual union all
select 36549,'RVSN' from dual union all
select 3655,'UNRV' from dual union all
select 36550,'RVSN' from dual union all
select 36551,'RVSN' from dual union all
select 36552,'RVSN' from dual union all
select 36553,'RVSN' from dual union all
select 36554,'NASA' from dual union all
select 36555,'RVSN' from dual union all
select 36556,'RVSN' from dual union all
select 36557,'RVSN' from dual union all
select 36558,'CALT' from dual union all
select 36559,'RVSN' from dual union all
select 3656,'UNRV' from dual union all
select 36560,'RVSN' from dual union all
select 36561,'USAF' from dual union all
select 36562,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 36563,'RVSN' from dual union all
select 36564,'NASA' from dual union all
select 36565,'USAF' from dual union all
select 36566,'SAST' from dual union all
select 36567,'RVSN' from dual union all
select 36568,'RVSN' from dual union all
select 36569,'RVSN' from dual union all
select 3657,'UNRV' from dual union all
select 36570,'RVSN' from dual union all
select 36571,'RVSN' from dual union all
select 36572,'RVSN' from dual union all
select 36573,'SAND' from dual union all
select 36574,'SAND' from dual union all
select 36575,'MRN' from dual union all
select 36576,'RVSN' from dual union all
select 36577,'MRN' from dual union all
select 36578,'USAF' from dual union all
select 36579,'MRN' from dual union all
select 3658,'UNRV' from dual union all
select 36580,'MRN' from dual union all
select 36581,'USAF' from dual union all
select 36582,'MRN' from dual union all
select 36583,'SAND' from dual union all
select 36584,'MRN' from dual union all
select 36585,'USN' from dual union all
select 36586,'MRN' from dual union all
select 36587,'USAF' from dual union all
select 36588,'SAND' from dual union all
select 36589,'USAF' from dual union all
select 3659,'UNRV' from dual union all
select 36590,'NASA' from dual union all
select 36591,'USN' from dual union all
select 36592,'MRN' from dual union all
select 36593,'SAND' from dual union all
select 36594,'SAND' from dual union all
select 36595,'USAF' from dual union all
select 36596,'USAF' from dual union all
select 36597,'USA' from dual union all
select 36598,'MRN' from dual union all
select 36599,'USAF' from dual union all
select 366,'WEHR' from dual union all
select 3660,'UZKA' from dual union all
select 36600,'USAF' from dual union all
select 36601,'USN' from dual union all
select 36602,'USAF' from dual union all
select 36603,'USAF' from dual union all
select 36604,'USAF' from dual union all
select 36605,'SAND' from dual union all
select 36606,'NASA' from dual union all
select 36607,'SAND' from dual union all
select 36608,'USAF' from dual union all
select 36609,'NASA' from dual union all
select 3661,'UNRV' from dual union all
select 36610,'DNA' from dual union all
select 36611,'RVSN' from dual union all
select 36612,'MRN' from dual union all
select 36613,'MRN' from dual union all
select 36614,'RVSN' from dual union all
select 36615,'USN' from dual union all
select 36616,'MRN' from dual union all
select 36617,'81FA1' from dual union all
select 36618,'MRN' from dual union all
select 36619,'41FA1' from dual union all
select 3662,'UNRV' from dual union all
select 36620,'41FA1' from dual union all
select 36621,'SAND' from dual union all
select 36622,'MRN' from dual union all
select 36623,'RVSN' from dual union all
select 36624,'MRN' from dual union all
select 36625,'RVSN' from dual union all
select 36626,'MRN' from dual union all
select 36627,'MRN' from dual union all
select 36628,'MRN' from dual union all
select 36629,'MRN' from dual union all
select 3663,'UNRV' from dual union all
select 36630,'SAND' from dual union all
select 36631,'SAND' from dual union all
select 36632,'MRN' from dual union all
select 36633,'MRN' from dual union all
select 36634,'USAF' from dual union all
select 36635,'USAF' from dual union all
select 36636,'NASA' from dual union all
select 36637,'USAF' from dual union all
select 36638,'MRN' from dual union all
select 36639,'MRN' from dual union all
select 3664,'UNRV' from dual union all
select 36640,'MRN' from dual union all
select 36641,'SAND' from dual union all
select 36642,'MRN' from dual union all
select 36643,'MRN' from dual union all
select 36644,'MRN' from dual union all
select 36645,'MRN' from dual union all
select 36646,'MRN' from dual union all
select 36647,'USN' from dual union all
select 36648,'NASA' from dual union all
select 36649,'MRN' from dual union all
select 3665,'UNRV' from dual union all
select 36650,'MRN' from dual union all
select 36651,'ISRO' from dual union all
select 36652,'USA' from dual;
insert into LAUNCH_AGENCY
select 36653,'RVSN' from dual union all
select 36654,'NASA' from dual union all
select 36655,'RVSN' from dual union all
select 36656,'AFSC' from dual union all
select 36657,'RVSN' from dual union all
select 36658,'RVSN' from dual union all
select 36659,'AFSC' from dual union all
select 3666,'UNRV' from dual union all
select 36660,'RVSN' from dual union all
select 36661,'I' from dual union all
select 36662,'NASA' from dual union all
select 36663,'CNES' from dual union all
select 36664,'CNES' from dual union all
select 36665,'CNES' from dual union all
select 36666,'CNES' from dual union all
select 36667,'CNES' from dual union all
select 36668,'CNES' from dual union all
select 36669,'CNES' from dual union all
select 3667,'UZKA' from dual union all
select 36670,'CNES' from dual union all
select 36671,'CNES' from dual union all
select 36672,'CNES' from dual union all
select 36673,'CNES' from dual union all
select 36674,'CNES' from dual union all
select 36675,'CNES' from dual union all
select 36676,'CNES' from dual union all
select 36677,'CNES' from dual union all
select 36678,'CNES' from dual union all
select 36679,'CNES' from dual union all
select 3668,'ARDC' from dual union all
select 36680,'30FA2' from dual union all
select 36681,'30FA2' from dual union all
select 36682,'RVSN' from dual union all
select 36683,'RVSN' from dual union all
select 36684,'RVSN' from dual union all
select 36685,'RVSN' from dual union all
select 36686,'RVSN' from dual union all
select 36687,'RVSN' from dual union all
select 36688,'RVSN' from dual union all
select 36689,'RVSN' from dual union all
select 3669,'NRL' from dual union all
select 36690,'RVSN' from dual union all
select 36691,'RVSN' from dual union all
select 36692,'NASA' from dual union all
select 36693,'AFCRL' from dual union all
select 36693,'DNA' from dual union all
select 36694,'SSC' from dual union all
select 36695,'DFVLR' from dual union all
select 36696,'NASA' from dual union all
select 36697,'NASA' from dual union all
select 36698,'SRC' from dual union all
select 36699,'SRC' from dual union all
select 367,'WEHR' from dual union all
select 3670,'UNRV' from dual union all
select 36700,'RVSN' from dual union all
select 36701,'NASA' from dual union all
select 36702,'RVSN' from dual union all
select 36703,'SRC' from dual union all
select 36704,'NASA' from dual union all
select 36705,'SRC' from dual union all
select 36706,'RVSN' from dual union all
select 36707,'USA' from dual union all
select 36708,'NASA' from dual union all
select 36709,'NASA' from dual union all
select 3671,'UZKA' from dual union all
select 36710,'AFCRL' from dual union all
select 36711,'AFCRL' from dual union all
select 36712,'AFCRL' from dual union all
select 36713,'RVSN' from dual union all
select 36714,'USN' from dual union all
select 36715,'NASA' from dual union all
select 36716,'USAF' from dual union all
select 36717,'NASA' from dual union all
select 36718,'NASA' from dual union all
select 36719,'USA' from dual union all
select 3672,'NRL' from dual union all
select 36720,'USN' from dual union all
select 36721,'USN' from dual union all
select 36722,'ISRO' from dual union all
select 36723,'AFCRL' from dual union all
select 36724,'RVSN' from dual union all
select 36725,'AFCRL' from dual union all
select 36726,'AFCRL' from dual union all
select 36727,'ISRO' from dual union all
select 36728,'RVSN' from dual union all
select 36729,'USN' from dual union all
select 3673,'USAF' from dual union all
select 36730,'RVSN' from dual union all
select 36731,'AFCRL' from dual union all
select 36732,'AFCRL' from dual union all
select 36733,'AFCRL' from dual union all
select 36734,'USN' from dual union all
select 36735,'RVSN' from dual union all
select 36736,'AN' from dual union all
select 36737,'AN' from dual union all
select 36738,'SRC' from dual union all
select 36739,'DLR' from dual union all
select 36739,'INTA' from dual union all
select 3674,'UNRV' from dual union all
select 36740,'INTA' from dual;
insert into LAUNCH_AGENCY
select 36741,'AFSC' from dual union all
select 36742,'DLR' from dual union all
select 36742,'INTA' from dual union all
select 36743,'INTA' from dual union all
select 36744,'AFSC' from dual union all
select 36745,'RAE' from dual union all
select 36746,'VMF' from dual union all
select 36747,'SAND' from dual union all
select 36748,'SAND' from dual union all
select 36749,'FKG1' from dual union all
select 3675,'UNRV' from dual union all
select 36750,'FKG1' from dual union all
select 36751,'SAND' from dual union all
select 36752,'SAND' from dual union all
select 36753,'NASA' from dual union all
select 36754,'AFSC' from dual union all
select 36755,'AFSC' from dual union all
select 36756,'NASA' from dual union all
select 36757,'RVSN' from dual union all
select 36758,'NASA' from dual union all
select 36759,'NASA' from dual union all
select 3676,'UNRV' from dual union all
select 36760,'NASA' from dual union all
select 36761,'FKG1' from dual union all
select 36762,'FKG1' from dual union all
select 36763,'RVSN' from dual union all
select 36764,'NASA' from dual union all
select 36765,'USN' from dual union all
select 36766,'USN' from dual union all
select 36767,'NASA' from dual union all
select 36768,'ISAS' from dual union all
select 36769,'NASA' from dual union all
select 3677,'USAF' from dual union all
select 36770,'AN' from dual union all
select 36771,'AN' from dual union all
select 36772,'AN' from dual union all
select 36773,'AN' from dual union all
select 36774,'AN' from dual union all
select 36775,'AN' from dual union all
select 36776,'AN' from dual union all
select 36777,'AN' from dual union all
select 36778,'AN' from dual union all
select 36779,'USAF' from dual union all
select 3678,'UNRV' from dual union all
select 36780,'AN' from dual union all
select 36781,'AN' from dual union all
select 36782,'AN' from dual union all
select 36783,'AN' from dual union all
select 36784,'AN' from dual union all
select 36785,'USA' from dual union all
select 36786,'USA' from dual union all
select 36787,'USA' from dual union all
select 36788,'CNES' from dual union all
select 36789,'NASA' from dual union all
select 3679,'UNRV' from dual union all
select 36790,'RVSN' from dual union all
select 36791,'NASA' from dual union all
select 36792,'RVSN' from dual union all
select 36793,'RVSN' from dual union all
select 36794,'NASA' from dual union all
select 36795,'NASA' from dual union all
select 36796,'AFCRL' from dual union all
select 36797,'NTNF' from dual union all
select 36798,'SAC' from dual union all
select 36799,'US7A' from dual union all
select 368,'WEHR' from dual union all
select 3680,'UNRV' from dual union all
select 36800,'US7A' from dual union all
select 36801,'RVSN' from dual union all
select 36802,'RVSN' from dual union all
select 36803,'USN' from dual union all
select 36804,'USN' from dual union all
select 36805,'USN' from dual union all
select 36806,'USN' from dual union all
select 36807,'USN' from dual union all
select 36808,'SAC' from dual union all
select 36809,'USN' from dual union all
select 3681,'USA' from dual union all
select 36810,'USN' from dual union all
select 36811,'DMA' from dual union all
select 36812,'RVSN' from dual union all
select 36813,'USN' from dual union all
select 36814,'USN' from dual union all
select 36815,'BAC' from dual union all
select 36816,'US7A' from dual union all
select 36817,'US7A' from dual union all
select 36818,'RVSN' from dual union all
select 36819,'NASA' from dual union all
select 3682,'USAF' from dual union all
select 36820,'AN' from dual union all
select 36821,'AN' from dual union all
select 36822,'RVSN' from dual union all
select 36823,'RVSN' from dual union all
select 36824,'RVSN' from dual union all
select 36825,'RVSN' from dual union all
select 36826,'AFSC' from dual union all
select 36827,'BAC' from dual union all
select 36828,'NASA' from dual union all
select 36829,'AN' from dual union all
select 3683,'UNRV' from dual;
insert into LAUNCH_AGENCY
select 36830,'NASA' from dual union all
select 36831,'84FA3' from dual union all
select 36832,'84FA3' from dual union all
select 36833,'84FA3' from dual union all
select 36834,'RVSN' from dual union all
select 36835,'NASA' from dual union all
select 36836,'84FA3' from dual union all
select 36837,'AFSC' from dual union all
select 36838,'NASA' from dual union all
select 36839,'81FA1' from dual union all
select 3684,'UNRV' from dual union all
select 36840,'81FA1' from dual union all
select 36841,'81FA1' from dual union all
select 36842,'USAF' from dual union all
select 36843,'SAC' from dual union all
select 36844,'AFCRL' from dual union all
select 36845,'RVSN' from dual union all
select 36846,'NASA' from dual union all
select 36847,'RVSN' from dual union all
select 36848,'RVSN' from dual union all
select 36849,'SAC' from dual union all
select 3685,'UNRV' from dual union all
select 36850,'AFSC' from dual union all
select 36850,'CRLI' from dual union all
select 36851,'RVSN' from dual union all
select 36852,'NASA' from dual union all
select 36853,'NASDA' from dual union all
select 36854,'NASA' from dual union all
select 36855,'AFSC' from dual union all
select 36856,'RVSN' from dual union all
select 36857,'USN' from dual union all
select 36858,'USN' from dual union all
select 36859,'USN' from dual union all
select 3686,'UNRV' from dual union all
select 36860,'41FA1' from dual union all
select 36861,'RVSN' from dual union all
select 36862,'ISAS' from dual union all
select 36863,'41FA1' from dual union all
select 36864,'NASA' from dual union all
select 36865,'RVSN' from dual union all
select 36866,'SAC' from dual union all
select 36867,'RVSN' from dual union all
select 36868,'USN' from dual union all
select 36869,'USN' from dual union all
select 3687,'UNRV' from dual union all
select 36870,'ISAS' from dual union all
select 36871,'AN' from dual union all
select 36872,'ISAS' from dual union all
select 36873,'SAND' from dual union all
select 36874,'AN' from dual union all
select 36875,'ALCS' from dual union all
select 36875,'SAC' from dual union all
select 36876,'SRC' from dual union all
select 36877,'SRC' from dual union all
select 36878,'RVSN' from dual union all
select 36879,'SRC' from dual union all
select 3688,'UNRV' from dual union all
select 36880,'NASA' from dual union all
select 36881,'AFSC' from dual union all
select 36882,'AN' from dual union all
select 36883,'SAC' from dual union all
select 36884,'SAND' from dual union all
select 36885,'AFCRL' from dual union all
select 36886,'USAF' from dual union all
select 36887,'AFCRL' from dual union all
select 36888,'SAC' from dual union all
select 36889,'RVSN' from dual union all
select 3689,'USA' from dual union all
select 36890,'SAND' from dual union all
select 36891,'ISAS' from dual union all
select 36892,'ISAS' from dual union all
select 36893,'NASA' from dual union all
select 36894,'USN' from dual union all
select 36895,'RVSN' from dual union all
select 36896,'AFSC' from dual union all
select 36897,'FKG2' from dual union all
select 36898,'FKG2' from dual union all
select 36899,'FKG1' from dual union all
select 369,'WEHR' from dual union all
select 3690,'UNRV' from dual union all
select 36900,'RVSN' from dual union all
select 36901,'IIAE' from dual union all
select 36902,'RVSN' from dual union all
select 36903,'AN' from dual union all
select 36904,'DFVLR' from dual union all
select 36905,'AN' from dual union all
select 36906,'AN' from dual union all
select 36907,'AN' from dual union all
select 36908,'IIAE' from dual union all
select 36909,'NASA' from dual union all
select 3691,'UNRV' from dual union all
select 36910,'DFVLR' from dual union all
select 36911,'RVSN' from dual union all
select 36912,'NASA' from dual union all
select 36913,'USN' from dual union all
select 36914,'USAF' from dual union all
select 36915,'AN' from dual union all
select 36916,'AFCRL' from dual union all
select 36917,'NASA' from dual union all
select 36918,'INTA' from dual;
insert into LAUNCH_AGENCY
select 36919,'RVSN' from dual union all
select 3692,'UNRV' from dual union all
select 36920,'NASA' from dual union all
select 36921,'AN' from dual union all
select 36922,'SAND' from dual union all
select 36923,'DNA' from dual union all
select 36924,'AN' from dual union all
select 36925,'AN' from dual union all
select 36926,'AN' from dual union all
select 36927,'USAF' from dual union all
select 36928,'USN' from dual union all
select 36929,'USN' from dual union all
select 3693,'UNRV' from dual union all
select 36930,'NASA' from dual union all
select 36931,'AFSC' from dual union all
select 36932,'USN' from dual union all
select 36933,'USAF' from dual union all
select 36934,'ALCS' from dual union all
select 36934,'SAC' from dual union all
select 36935,'NASA' from dual union all
select 36936,'NASA' from dual union all
select 36937,'NASA' from dual union all
select 36938,'SSC' from dual union all
select 36939,'RN' from dual union all
select 3694,'USAF' from dual union all
select 36940,'BAC' from dual union all
select 36941,'NRCC' from dual union all
select 36941,'SAND' from dual union all
select 36942,'SRC' from dual union all
select 36943,'SRC' from dual union all
select 36944,'SAND' from dual union all
select 36945,'NRCC' from dual union all
select 36945,'SAND' from dual union all
select 36946,'AN' from dual union all
select 36947,'AN' from dual union all
select 36948,'AN' from dual union all
select 36949,'AN' from dual union all
select 3695,'UNRV' from dual union all
select 36950,'AFCRL' from dual union all
select 36951,'AFCRL' from dual union all
select 36952,'AFCRL' from dual union all
select 36953,'AFCRL' from dual union all
select 36954,'AFCRL' from dual union all
select 36955,'SAC' from dual union all
select 36956,'AFCRL' from dual union all
select 36957,'NASA' from dual union all
select 36958,'BAL' from dual union all
select 36959,'USAF' from dual union all
select 3696,'UNRV' from dual union all
select 36960,'SAC' from dual union all
select 36961,'AFCRL' from dual union all
select 36962,'NASA' from dual union all
select 36963,'WRE' from dual union all
select 36964,'NASA' from dual union all
select 36965,'NASA' from dual union all
select 36966,'DFVLR' from dual union all
select 36967,'NASA' from dual union all
select 36968,'AFSC' from dual union all
select 36968,'CRLI' from dual union all
select 36969,'NASA' from dual union all
select 3697,'UNRV' from dual union all
select 36970,'SAC' from dual union all
select 36971,'DLR' from dual union all
select 36971,'INTA' from dual union all
select 36972,'I' from dual union all
select 36973,'I' from dual union all
select 36974,'ISRO' from dual union all
select 36975,'FKG1' from dual union all
select 36976,'WRE' from dual union all
select 36977,'AFCRL' from dual union all
select 36978,'ISAS' from dual union all
select 36979,'USAF' from dual union all
select 3698,'UNRV' from dual union all
select 36980,'RVSN' from dual union all
select 36981,'RVSN' from dual union all
select 36982,'WRE' from dual union all
select 36983,'RVSN' from dual union all
select 36984,'RVSN' from dual union all
select 36985,'RVSN' from dual union all
select 36986,'RVSN' from dual union all
select 36987,'RVSN' from dual union all
select 36988,'AFCRL' from dual union all
select 36989,'RVSN' from dual union all
select 3699,'USAF' from dual union all
select 36990,'RVSN' from dual union all
select 36991,'RVSN' from dual union all
select 36992,'RVSN' from dual union all
select 36993,'RVSN' from dual union all
select 36994,'RVSN' from dual union all
select 36995,'RVSN' from dual union all
select 36996,'RVSN' from dual union all
select 36997,'RVSN' from dual union all
select 36998,'RVSN' from dual union all
select 36999,'ISAS' from dual union all
select 37,'WEHR' from dual union all
select 370,'WEHR' from dual union all
select 3700,'UNRV' from dual union all
select 37000,'RVSN' from dual union all
select 37001,'RVSN' from dual union all
select 37002,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 37003,'RVSN' from dual union all
select 37004,'RVSN' from dual union all
select 37005,'RVSN' from dual union all
select 37006,'RVSN' from dual union all
select 37007,'RVSN' from dual union all
select 37008,'RVSN' from dual union all
select 37009,'RVSN' from dual union all
select 3701,'UNRV' from dual union all
select 37010,'AFCRL' from dual union all
select 37011,'RVSN' from dual union all
select 37012,'RVSN' from dual union all
select 37013,'RVSN' from dual union all
select 37014,'RVSN' from dual union all
select 37015,'RVSN' from dual union all
select 37016,'INTA' from dual union all
select 37017,'INTA' from dual union all
select 37018,'INTA' from dual union all
select 37019,'INTA' from dual union all
select 3702,'UNRV' from dual union all
select 37020,'AFCRL' from dual union all
select 37021,'INTA' from dual union all
select 37022,'INTA' from dual union all
select 37023,'WRE' from dual union all
select 37024,'WRE' from dual union all
select 37025,'NASA' from dual union all
select 37026,'RVSN' from dual union all
select 37027,'MVS' from dual union all
select 37028,'MVS' from dual union all
select 37029,'MVS' from dual union all
select 3703,'UNRV' from dual union all
select 37030,'RVSN' from dual union all
select 37031,'AFCRL' from dual union all
select 37032,'RVSN' from dual union all
select 37033,'RVSN' from dual union all
select 37034,'RVSN' from dual union all
select 37035,'AFSC' from dual union all
select 37036,'RVSN' from dual union all
select 37037,'NASA' from dual union all
select 37038,'ISAS' from dual union all
select 37039,'INTA' from dual union all
select 3704,'UNRV' from dual union all
select 37040,'DLR' from dual union all
select 37040,'INTA' from dual union all
select 37041,'CNES' from dual union all
select 37042,'RVSN' from dual union all
select 37043,'SAND' from dual union all
select 37044,'SAC' from dual union all
select 37045,'NRCC' from dual union all
select 37046,'INTA' from dual union all
select 37047,'DLR' from dual union all
select 37047,'INTA' from dual union all
select 37048,'SAND' from dual union all
select 37049,'NASA' from dual union all
select 3705,'NRL' from dual union all
select 37050,'NASA' from dual union all
select 37051,'RVSN' from dual union all
select 37052,'SAC' from dual union all
select 37053,'SAND' from dual union all
select 37054,'NASDA' from dual union all
select 37055,'DLR' from dual union all
select 37055,'INTA' from dual union all
select 37056,'SRC' from dual union all
select 37057,'SRC' from dual union all
select 37058,'SRC' from dual union all
select 37059,'SRC' from dual union all
select 3706,'UNRV' from dual union all
select 37060,'ISRO' from dual union all
select 37061,'SRC' from dual union all
select 37062,'NASA' from dual union all
select 37063,'NASA' from dual union all
select 37064,'NASA' from dual union all
select 37065,'NASA' from dual union all
select 37066,'CNES' from dual union all
select 37067,'ISRO' from dual union all
select 37068,'SRC' from dual union all
select 37069,'SRC' from dual union all
select 3707,'UNRV' from dual union all
select 37070,'ISRO' from dual union all
select 37071,'SRC' from dual union all
select 37072,'BAC' from dual union all
select 37073,'NASA' from dual union all
select 37074,'RVSN' from dual union all
select 37075,'AFCRL' from dual union all
select 37076,'RVSN' from dual union all
select 37077,'BAC' from dual union all
select 37078,'RVSN' from dual union all
select 37079,'AFCRL' from dual union all
select 3708,'UNRV' from dual union all
select 37080,'NASA' from dual union all
select 37081,'DNA' from dual union all
select 37082,'SSC' from dual union all
select 37083,'AFSC' from dual union all
select 37084,'DNA' from dual union all
select 37085,'DNA' from dual union all
select 37086,'SSC' from dual union all
select 37087,'NASA' from dual union all
select 37088,'DNA' from dual union all
select 37089,'NASA' from dual union all
select 3709,'UNRV' from dual union all
select 37090,'SAND' from dual;
insert into LAUNCH_AGENCY
select 37091,'ARPA' from dual union all
select 37091,'DNA' from dual union all
select 37092,'DNA' from dual union all
select 37093,'MRN' from dual union all
select 37094,'MRN' from dual union all
select 37095,'MRN' from dual union all
select 37096,'MRN' from dual union all
select 37097,'MRN' from dual union all
select 37098,'MRN' from dual union all
select 37099,'MRN' from dual union all
select 371,'WEHR' from dual union all
select 3710,'UNRV' from dual union all
select 37100,'MRN' from dual union all
select 37101,'MRN' from dual union all
select 37102,'MRN' from dual union all
select 37103,'MRN' from dual union all
select 37104,'MRN' from dual union all
select 37105,'MRN' from dual union all
select 37106,'MRN' from dual union all
select 37107,'MRN' from dual union all
select 37108,'MRN' from dual union all
select 37109,'MRN' from dual union all
select 3711,'UNRV' from dual union all
select 37110,'MRN' from dual union all
select 37111,'MRN' from dual union all
select 37112,'MRN' from dual union all
select 37113,'NASDA' from dual union all
select 37114,'MRN' from dual union all
select 37115,'MRN' from dual union all
select 37116,'MRN' from dual union all
select 37117,'MRN' from dual union all
select 37118,'MRN' from dual union all
select 37119,'MRN' from dual union all
select 3712,'UNRV' from dual union all
select 37120,'MRN' from dual union all
select 37121,'MRN' from dual union all
select 37122,'MRN' from dual union all
select 37123,'MRN' from dual union all
select 37124,'MRN' from dual union all
select 37125,'MRN' from dual union all
select 37126,'MRN' from dual union all
select 37127,'MRN' from dual union all
select 37128,'MRN' from dual union all
select 37129,'MRN' from dual union all
select 3713,'UNRV' from dual union all
select 37130,'MRN' from dual union all
select 37131,'MRN' from dual union all
select 37132,'MRN' from dual union all
select 37133,'MRN' from dual union all
select 37134,'MRN' from dual union all
select 37135,'MRN' from dual union all
select 37136,'MRN' from dual union all
select 37137,'MRN' from dual union all
select 37138,'MRN' from dual union all
select 37139,'MRN' from dual union all
select 3714,'UNRV' from dual union all
select 37140,'MRN' from dual union all
select 37141,'MRN' from dual union all
select 37142,'MRN' from dual union all
select 37143,'MRN' from dual union all
select 37144,'MRN' from dual union all
select 37145,'MRN' from dual union all
select 37146,'MRN' from dual union all
select 37147,'MRN' from dual union all
select 37148,'MRN' from dual union all
select 37149,'MRN' from dual union all
select 3715,'UNRV' from dual union all
select 37150,'MRN' from dual union all
select 37151,'MRN' from dual union all
select 37152,'MRN' from dual union all
select 37153,'MRN' from dual union all
select 37154,'MRN' from dual union all
select 37155,'MRN' from dual union all
select 37156,'MRN' from dual union all
select 37157,'MRN' from dual union all
select 37158,'MRN' from dual union all
select 37159,'MRN' from dual union all
select 3716,'UNRV' from dual union all
select 37160,'MRN' from dual union all
select 37161,'MRN' from dual union all
select 37162,'MRN' from dual union all
select 37163,'MRN' from dual union all
select 37164,'MRN' from dual union all
select 37165,'MRN' from dual union all
select 37166,'MRN' from dual union all
select 37167,'MRN' from dual union all
select 37168,'MRN' from dual union all
select 37169,'MRN' from dual union all
select 3717,'UNRV' from dual union all
select 37170,'MRN' from dual union all
select 37171,'MRN' from dual union all
select 37172,'MRN' from dual union all
select 37173,'MRN' from dual union all
select 37174,'MRN' from dual union all
select 37175,'MRN' from dual union all
select 37176,'MRN' from dual union all
select 37177,'MRN' from dual union all
select 37178,'MRN' from dual union all
select 37179,'MRN' from dual union all
select 3718,'UNRV' from dual;
insert into LAUNCH_AGENCY
select 37180,'MRN' from dual union all
select 37181,'MRN' from dual union all
select 37182,'MRN' from dual union all
select 37183,'MRN' from dual union all
select 37184,'MRN' from dual union all
select 37185,'MRN' from dual union all
select 37186,'MRN' from dual union all
select 37187,'MRN' from dual union all
select 37188,'MRN' from dual union all
select 37189,'MRN' from dual union all
select 3719,'UNRV' from dual union all
select 37190,'MRN' from dual union all
select 37191,'MRN' from dual union all
select 37192,'MRN' from dual union all
select 37193,'MRN' from dual union all
select 37194,'MRN' from dual union all
select 37195,'MRN' from dual union all
select 37196,'MRN' from dual union all
select 37197,'MRN' from dual union all
select 37198,'MRN' from dual union all
select 37199,'MRN' from dual union all
select 372,'WEHR' from dual union all
select 3720,'USA' from dual union all
select 37200,'MRN' from dual union all
select 37201,'MRN' from dual union all
select 37202,'MRN' from dual union all
select 37203,'MRN' from dual union all
select 37204,'MRN' from dual union all
select 37205,'MRN' from dual union all
select 37206,'MRN' from dual union all
select 37207,'MRN' from dual union all
select 37208,'MRN' from dual union all
select 37209,'MRN' from dual union all
select 3721,'UNRV' from dual union all
select 37210,'MRN' from dual union all
select 37211,'MRN' from dual union all
select 37212,'MRN' from dual union all
select 37213,'MRN' from dual union all
select 37214,'MRN' from dual union all
select 37215,'MRN' from dual union all
select 37216,'MRN' from dual union all
select 37217,'MRN' from dual union all
select 37218,'MRN' from dual union all
select 37219,'MRN' from dual union all
select 3722,'USAF' from dual union all
select 37220,'MRN' from dual union all
select 37221,'MRN' from dual union all
select 37222,'MRN' from dual union all
select 37223,'MRN' from dual union all
select 37224,'MRN' from dual union all
select 37225,'NASDA' from dual union all
select 37226,'MRN' from dual union all
select 37227,'MRN' from dual union all
select 37228,'MRN' from dual union all
select 37229,'MRN' from dual union all
select 3723,'SUI' from dual union all
select 37230,'MRN' from dual union all
select 37231,'MRN' from dual union all
select 37232,'MRN' from dual union all
select 37233,'MRN' from dual union all
select 37234,'MRN' from dual union all
select 37235,'MRN' from dual union all
select 37236,'MRN' from dual union all
select 37237,'MRN' from dual union all
select 37238,'MRN' from dual union all
select 37239,'MRN' from dual union all
select 3724,'SUI' from dual union all
select 37240,'MRN' from dual union all
select 37241,'MRN' from dual union all
select 37242,'MRN' from dual union all
select 37243,'MRN' from dual union all
select 37244,'MRN' from dual union all
select 37245,'MRN' from dual union all
select 37246,'MRN' from dual union all
select 37247,'MRN' from dual union all
select 37248,'MRN' from dual union all
select 37249,'MRN' from dual union all
select 3725,'SUI' from dual union all
select 37250,'MRN' from dual union all
select 37251,'MRN' from dual union all
select 37252,'MRN' from dual union all
select 37253,'MRN' from dual union all
select 37254,'MRN' from dual union all
select 37255,'MRN' from dual union all
select 37256,'MRN' from dual union all
select 37257,'MRN' from dual union all
select 37258,'MRN' from dual union all
select 37259,'MRN' from dual union all
select 3726,'NRL' from dual union all
select 37260,'MRN' from dual union all
select 37261,'MRN' from dual union all
select 37262,'MRN' from dual union all
select 37263,'MRN' from dual union all
select 37264,'MRN' from dual union all
select 37265,'MRN' from dual union all
select 37266,'MRN' from dual union all
select 37267,'MRN' from dual union all
select 37268,'MRN' from dual union all
select 37269,'MRN' from dual union all
select 3727,'SUI' from dual;
insert into LAUNCH_AGENCY
select 37270,'MRN' from dual union all
select 37271,'MRN' from dual union all
select 37272,'MRN' from dual union all
select 37273,'MRN' from dual union all
select 37274,'MRN' from dual union all
select 37275,'MRN' from dual union all
select 37276,'MRN' from dual union all
select 37277,'MRN' from dual union all
select 37278,'MRN' from dual union all
select 37279,'MRN' from dual union all
select 3728,'SUI' from dual union all
select 37280,'MRN' from dual union all
select 37281,'MRN' from dual union all
select 37282,'MRN' from dual union all
select 37283,'MRN' from dual union all
select 37284,'MRN' from dual union all
select 37285,'MRN' from dual union all
select 37286,'MRN' from dual union all
select 37287,'MRN' from dual union all
select 37288,'MRN' from dual union all
select 37289,'MRN' from dual union all
select 3729,'NRL' from dual union all
select 37290,'MRN' from dual union all
select 37291,'MRN' from dual union all
select 37292,'MRN' from dual union all
select 37293,'MRN' from dual union all
select 37294,'MRN' from dual union all
select 37295,'MRN' from dual union all
select 37296,'MRN' from dual union all
select 37297,'MRN' from dual union all
select 37298,'MRN' from dual union all
select 37299,'MRN' from dual union all
select 373,'WER836-3' from dual union all
select 3730,'SUI' from dual union all
select 37300,'MRN' from dual union all
select 37301,'MRN' from dual union all
select 37302,'MRN' from dual union all
select 37303,'MRN' from dual union all
select 37304,'MRN' from dual union all
select 37305,'MRN' from dual union all
select 37306,'MRN' from dual union all
select 37307,'MRN' from dual union all
select 37308,'MRN' from dual union all
select 37309,'MRN' from dual union all
select 3731,'SUI' from dual union all
select 37310,'MRN' from dual union all
select 37311,'MRN' from dual union all
select 37312,'MRN' from dual union all
select 37313,'MRN' from dual union all
select 37314,'MRN' from dual union all
select 37315,'MRN' from dual union all
select 37316,'MRN' from dual union all
select 37317,'MRN' from dual union all
select 37318,'MRN' from dual union all
select 37319,'MRN' from dual union all
select 3732,'SUI' from dual union all
select 37320,'MRN' from dual union all
select 37321,'MRN' from dual union all
select 37322,'MRN' from dual union all
select 37323,'MRN' from dual union all
select 37324,'MRN' from dual union all
select 37325,'MRN' from dual union all
select 37326,'MRN' from dual union all
select 37327,'MRN' from dual union all
select 37328,'MRN' from dual union all
select 37329,'MRN' from dual union all
select 3733,'SUI' from dual union all
select 37330,'MRN' from dual union all
select 37331,'MRN' from dual union all
select 37332,'MRN' from dual union all
select 37333,'MRN' from dual union all
select 37334,'MRN' from dual union all
select 37335,'MRN' from dual union all
select 37336,'MRN' from dual union all
select 37337,'MRN' from dual union all
select 37338,'MRN' from dual union all
select 37339,'MRN' from dual union all
select 3734,'SUI' from dual union all
select 37340,'MRN' from dual union all
select 37341,'MRN' from dual union all
select 37342,'MRN' from dual union all
select 37343,'MRN' from dual union all
select 37344,'MRN' from dual union all
select 37345,'MRN' from dual union all
select 37346,'NRCC' from dual union all
select 37347,'MRN' from dual union all
select 37348,'MRN' from dual union all
select 37349,'MRN' from dual union all
select 3735,'UNRV' from dual union all
select 37350,'MRN' from dual union all
select 37351,'MRN' from dual union all
select 37352,'MRN' from dual union all
select 37353,'MRN' from dual union all
select 37354,'MRN' from dual union all
select 37355,'MRN' from dual union all
select 37356,'MRN' from dual union all
select 37357,'MRN' from dual union all
select 37358,'MRN' from dual union all
select 37359,'MRN' from dual union all
select 3736,'UNRV' from dual;
insert into LAUNCH_AGENCY
select 37360,'MRN' from dual union all
select 37361,'MRN' from dual union all
select 37362,'MRN' from dual union all
select 37363,'MRN' from dual union all
select 37364,'MRN' from dual union all
select 37365,'MRN' from dual union all
select 37366,'MRN' from dual union all
select 37367,'MRN' from dual union all
select 37368,'MRN' from dual union all
select 37369,'MRN' from dual union all
select 3737,'SUI' from dual union all
select 37370,'MRN' from dual union all
select 37371,'MRN' from dual union all
select 37372,'MRN' from dual union all
select 37373,'MRN' from dual union all
select 37374,'MRN' from dual union all
select 37375,'MRN' from dual union all
select 37376,'MRN' from dual union all
select 37377,'MRN' from dual union all
select 37378,'MRN' from dual union all
select 37379,'MRN' from dual union all
select 3738,'SUI' from dual union all
select 37380,'MRN' from dual union all
select 37381,'MRN' from dual union all
select 37382,'MRN' from dual union all
select 37383,'MRN' from dual union all
select 37384,'MRN' from dual union all
select 37385,'MRN' from dual union all
select 37386,'MRN' from dual union all
select 37387,'MRN' from dual union all
select 37388,'MRN' from dual union all
select 37389,'MRN' from dual union all
select 3739,'UNRV' from dual union all
select 37390,'MRN' from dual union all
select 37391,'MRN' from dual union all
select 37392,'MRN' from dual union all
select 37393,'MRN' from dual union all
select 37394,'MRN' from dual union all
select 37395,'MRN' from dual union all
select 37396,'MRN' from dual union all
select 37397,'MRN' from dual union all
select 37398,'MRN' from dual union all
select 37399,'MRN' from dual union all
select 374,'WEHR' from dual union all
select 3740,'SUI' from dual union all
select 37400,'MRN' from dual union all
select 37401,'MRN' from dual union all
select 37402,'MRN' from dual union all
select 37403,'MRN' from dual union all
select 37404,'MRN' from dual union all
select 37405,'MRN' from dual union all
select 37406,'MRN' from dual union all
select 37407,'MRN' from dual union all
select 37408,'MRN' from dual union all
select 37409,'MRN' from dual union all
select 3741,'UNRV' from dual union all
select 37410,'MRN' from dual union all
select 37411,'MRN' from dual union all
select 37412,'MRN' from dual union all
select 37413,'MRN' from dual union all
select 37414,'MRN' from dual union all
select 37415,'MRN' from dual union all
select 37416,'MRN' from dual union all
select 37417,'MRN' from dual union all
select 37418,'MRN' from dual union all
select 37419,'MRN' from dual union all
select 3742,'NRL' from dual union all
select 37420,'MRN' from dual union all
select 37421,'MRN' from dual union all
select 37422,'MRN' from dual union all
select 37423,'MRN' from dual union all
select 37424,'MRN' from dual union all
select 37425,'MRN' from dual union all
select 37426,'MRN' from dual union all
select 37427,'MRN' from dual union all
select 37428,'WFC' from dual union all
select 37429,'MRN' from dual union all
select 3743,'USAF' from dual union all
select 37430,'MRN' from dual union all
select 37431,'MRN' from dual union all
select 37432,'MRN' from dual union all
select 37433,'MRN' from dual union all
select 37434,'MRN' from dual union all
select 37435,'MRN' from dual union all
select 37436,'AN' from dual union all
select 37437,'MRN' from dual union all
select 37438,'MRN' from dual union all
select 37439,'MRN' from dual union all
select 3744,'SUI' from dual union all
select 37440,'MRN' from dual union all
select 37441,'MRN' from dual union all
select 37442,'MRN' from dual union all
select 37443,'MRN' from dual union all
select 37444,'MRN' from dual union all
select 37445,'MRN' from dual union all
select 37446,'MRN' from dual union all
select 37447,'MRN' from dual union all
select 37448,'MRN' from dual union all
select 37449,'MRN' from dual union all
select 3745,'SUI' from dual;
insert into LAUNCH_AGENCY
select 37450,'MRN' from dual union all
select 37451,'MRN' from dual union all
select 37452,'MRN' from dual union all
select 37453,'MRN' from dual union all
select 37454,'MRN' from dual union all
select 37455,'MRN' from dual union all
select 37456,'MRN' from dual union all
select 37457,'MRN' from dual union all
select 37458,'MRN' from dual union all
select 37459,'MRN' from dual union all
select 3746,'NRL' from dual union all
select 37460,'MRN' from dual union all
select 37461,'MRN' from dual union all
select 37462,'MRN' from dual union all
select 37463,'MRN' from dual union all
select 37464,'MRN' from dual union all
select 37465,'MRN' from dual union all
select 37466,'MRN' from dual union all
select 37467,'MRN' from dual union all
select 37468,'MRN' from dual union all
select 37469,'MRN' from dual union all
select 3747,'NRL' from dual union all
select 37470,'MRN' from dual union all
select 37471,'MRN' from dual union all
select 37472,'MRN' from dual union all
select 37473,'MRN' from dual union all
select 37474,'MRN' from dual union all
select 37475,'MRN' from dual union all
select 37476,'MRN' from dual union all
select 37477,'MRN' from dual union all
select 37478,'MRN' from dual union all
select 37479,'MRN' from dual union all
select 3748,'UNRV' from dual union all
select 37480,'AN' from dual union all
select 37481,'MRN' from dual union all
select 37482,'MRN' from dual union all
select 37483,'MRN' from dual union all
select 37484,'MRN' from dual union all
select 37485,'MRN' from dual union all
select 37486,'MRN' from dual union all
select 37487,'MRN' from dual union all
select 37488,'MRN' from dual union all
select 37489,'MRN' from dual union all
select 3749,'UNRV' from dual union all
select 37490,'MRN' from dual union all
select 37491,'MRN' from dual union all
select 37492,'MRN' from dual union all
select 37493,'MRN' from dual union all
select 37494,'MRN' from dual union all
select 37495,'MRN' from dual union all
select 37496,'MRN' from dual union all
select 37497,'MRN' from dual union all
select 37498,'MRN' from dual union all
select 37499,'MRN' from dual union all
select 375,'WEHR' from dual union all
select 3750,'USN' from dual union all
select 37500,'MRN' from dual union all
select 37501,'MRN' from dual union all
select 37502,'MRN' from dual union all
select 37503,'MRN' from dual union all
select 37504,'MRN' from dual union all
select 37505,'MRN' from dual union all
select 37506,'MRN' from dual union all
select 37507,'MRN' from dual union all
select 37508,'MRN' from dual union all
select 37509,'MRN' from dual union all
select 3751,'USN' from dual union all
select 37510,'MRN' from dual union all
select 37511,'MRN' from dual union all
select 37512,'MRN' from dual union all
select 37513,'MRN' from dual union all
select 37514,'MRN' from dual union all
select 37515,'MRN' from dual union all
select 37516,'MRN' from dual union all
select 37517,'MRN' from dual union all
select 37518,'MRN' from dual union all
select 37519,'MRN' from dual union all
select 3752,'CASDN' from dual union all
select 37520,'MRN' from dual union all
select 37521,'MRN' from dual union all
select 37522,'MRN' from dual union all
select 37523,'MRN' from dual union all
select 37524,'MRN' from dual union all
select 37525,'MRN' from dual union all
select 37526,'MRN' from dual union all
select 37527,'MRN' from dual union all
select 37528,'MRN' from dual union all
select 37529,'MRN' from dual union all
select 3753,'USAF' from dual union all
select 37530,'MRN' from dual union all
select 37531,'MRN' from dual union all
select 37532,'MRN' from dual union all
select 37533,'MRN' from dual union all
select 37534,'NASA' from dual union all
select 37535,'MRN' from dual union all
select 37536,'MRN' from dual union all
select 37537,'MRN' from dual union all
select 37538,'MRN' from dual union all
select 37539,'MRN' from dual union all
select 3754,'USAF' from dual;
insert into LAUNCH_AGENCY
select 37540,'MRN' from dual union all
select 37541,'MRN' from dual union all
select 37542,'MRN' from dual union all
select 37543,'MRN' from dual union all
select 37544,'MRN' from dual union all
select 37545,'MRN' from dual union all
select 37546,'MRN' from dual union all
select 37547,'MRN' from dual union all
select 37548,'MRN' from dual union all
select 37549,'MRN' from dual union all
select 3755,'PARD' from dual union all
select 37550,'MRN' from dual union all
select 37551,'MRN' from dual union all
select 37552,'MRN' from dual union all
select 37553,'MRN' from dual union all
select 37554,'MRN' from dual union all
select 37555,'MRN' from dual union all
select 37556,'MRN' from dual union all
select 37557,'MRN' from dual union all
select 37558,'MRN' from dual union all
select 37559,'MRN' from dual union all
select 3756,'USAF' from dual union all
select 37560,'MRN' from dual union all
select 37561,'MRN' from dual union all
select 37562,'MRN' from dual union all
select 37563,'NASA' from dual union all
select 37564,'MRN' from dual union all
select 37565,'MRN' from dual union all
select 37566,'MRN' from dual union all
select 37567,'MRN' from dual union all
select 37568,'MRN' from dual union all
select 37569,'MRN' from dual union all
select 3757,'USAF' from dual union all
select 37570,'MRN' from dual union all
select 37571,'MRN' from dual union all
select 37572,'MRN' from dual union all
select 37573,'MRN' from dual union all
select 37574,'MRN' from dual union all
select 37575,'MRN' from dual union all
select 37576,'MRN' from dual union all
select 37577,'MRN' from dual union all
select 37578,'MRN' from dual union all
select 37579,'MRN' from dual union all
select 3758,'PARD' from dual union all
select 37580,'MRN' from dual union all
select 37581,'MRN' from dual union all
select 37582,'MRN' from dual union all
select 37583,'MRN' from dual union all
select 37584,'MRN' from dual union all
select 37585,'WFC' from dual union all
select 37586,'WFC' from dual union all
select 37587,'WFC' from dual union all
select 37588,'WFC' from dual union all
select 37589,'WFC' from dual union all
select 3759,'PARD' from dual union all
select 37590,'WFC' from dual union all
select 37591,'MRN' from dual union all
select 37592,'WFC' from dual union all
select 37593,'WFC' from dual union all
select 37594,'WFC' from dual union all
select 37595,'WFC' from dual union all
select 37596,'WFC' from dual union all
select 37597,'WFC' from dual union all
select 37598,'WFC' from dual union all
select 37599,'WFC' from dual union all
select 376,'WEHR' from dual union all
select 3760,'PARD' from dual union all
select 37600,'WFC' from dual union all
select 37601,'WFC' from dual union all
select 37602,'MRN' from dual union all
select 37603,'WFC' from dual union all
select 37604,'WFC' from dual union all
select 37605,'WFC' from dual union all
select 37606,'USA' from dual union all
select 37607,'USA' from dual union all
select 37608,'USA' from dual union all
select 37609,'USA' from dual union all
select 3761,'USAF' from dual union all
select 37610,'USA' from dual union all
select 37611,'USA' from dual union all
select 37612,'USA' from dual union all
select 37613,'NASDA' from dual union all
select 37614,'USA' from dual union all
select 37615,'USA' from dual union all
select 37616,'USA' from dual union all
select 37617,'USA' from dual union all
select 37618,'USA' from dual union all
select 37619,'USA' from dual union all
select 3762,'USAF' from dual union all
select 37620,'USA' from dual union all
select 37621,'USA' from dual union all
select 37622,'USA' from dual union all
select 37623,'AN' from dual union all
select 37624,'MRN' from dual union all
select 37625,'MRN' from dual union all
select 37626,'MRN' from dual union all
select 37627,'MRN' from dual union all
select 37628,'MRN' from dual union all
select 37629,'MRN' from dual union all
select 3763,'USA' from dual;
insert into LAUNCH_AGENCY
select 37630,'MRN' from dual union all
select 37631,'MRN' from dual union all
select 37632,'MRN' from dual union all
select 37633,'MRN' from dual union all
select 37634,'MRN' from dual union all
select 37635,'MRN' from dual union all
select 37636,'MRN' from dual union all
select 37637,'MRN' from dual union all
select 37638,'MRN' from dual union all
select 37639,'MRN' from dual union all
select 3764,'USAF' from dual union all
select 37640,'MRN' from dual union all
select 37641,'MRN' from dual union all
select 37642,'MRN' from dual union all
select 37643,'MRN' from dual union all
select 37644,'MRN' from dual union all
select 37645,'MRN' from dual union all
select 37646,'MRN' from dual union all
select 37647,'MRN' from dual union all
select 37648,'MRN' from dual union all
select 37649,'MRN' from dual union all
select 3765,'USA' from dual union all
select 37650,'MRN' from dual union all
select 37651,'MRN' from dual union all
select 37652,'MRN' from dual union all
select 37653,'MRN' from dual union all
select 37654,'MRN' from dual union all
select 37655,'MRN' from dual union all
select 37656,'MRN' from dual union all
select 37657,'MRN' from dual union all
select 37658,'MRN' from dual union all
select 37659,'MRN' from dual union all
select 3766,'NRL' from dual union all
select 37660,'MRN' from dual union all
select 37661,'MRN' from dual union all
select 37662,'MRN' from dual union all
select 37663,'MRN' from dual union all
select 37664,'MRN' from dual union all
select 37665,'MRN' from dual union all
select 37666,'MRN' from dual union all
select 37667,'MRN' from dual union all
select 37668,'MRN' from dual union all
select 37669,'MRN' from dual union all
select 3767,'USAF' from dual union all
select 37670,'MRN' from dual union all
select 37671,'MRN' from dual union all
select 37672,'MRN' from dual union all
select 37673,'MRN' from dual union all
select 37674,'MRN' from dual union all
select 37675,'MRN' from dual union all
select 37676,'MRN' from dual union all
select 37677,'MRN' from dual union all
select 37678,'MRN' from dual union all
select 37679,'MRN' from dual union all
select 3768,'LERC' from dual union all
select 37680,'NASA' from dual union all
select 37681,'MRN' from dual union all
select 37682,'MRN' from dual union all
select 37683,'NASA' from dual union all
select 37684,'MRN' from dual union all
select 37685,'MRN' from dual union all
select 37686,'MRN' from dual union all
select 37687,'MRN' from dual union all
select 37688,'MRN' from dual union all
select 37689,'MRN' from dual union all
select 3769,'USAF' from dual union all
select 37690,'MRN' from dual union all
select 37691,'MRN' from dual union all
select 37692,'MRN' from dual union all
select 37693,'MRN' from dual union all
select 37694,'NASA' from dual union all
select 37695,'MRN' from dual union all
select 37696,'MRN' from dual union all
select 37697,'MRN' from dual union all
select 37698,'MRN' from dual union all
select 37699,'MRN' from dual union all
select 377,'WEHR' from dual union all
select 3770,'USAF' from dual union all
select 37700,'MRN' from dual union all
select 37701,'MRN' from dual union all
select 37702,'MRN' from dual union all
select 37703,'MRN' from dual union all
select 37704,'MRN' from dual union all
select 37705,'MRN' from dual union all
select 37706,'MRN' from dual union all
select 37707,'MRN' from dual union all
select 37708,'MRN' from dual union all
select 37709,'MRN' from dual union all
select 3771,'USAF' from dual union all
select 37710,'MRN' from dual union all
select 37711,'MRN' from dual union all
select 37712,'MRN' from dual union all
select 37713,'MRN' from dual union all
select 37714,'MRN' from dual union all
select 37715,'MRN' from dual union all
select 37716,'MRN' from dual union all
select 37717,'MRN' from dual union all
select 37718,'MRN' from dual union all
select 37719,'MRN' from dual union all
select 3772,'USAF' from dual;
insert into LAUNCH_AGENCY
select 37720,'MRN' from dual union all
select 37721,'MRN' from dual union all
select 37722,'MRN' from dual union all
select 37723,'MRN' from dual union all
select 37724,'MRN' from dual union all
select 37725,'MRN' from dual union all
select 37726,'MRN' from dual union all
select 37727,'MRN' from dual union all
select 37728,'MRN' from dual union all
select 37729,'MRN' from dual union all
select 3773,'PARD' from dual union all
select 37730,'MRN' from dual union all
select 37731,'MRN' from dual union all
select 37732,'MRN' from dual union all
select 37733,'MRN' from dual union all
select 37734,'MRN' from dual union all
select 37735,'MRN' from dual union all
select 37736,'MRN' from dual union all
select 37737,'MRN' from dual union all
select 37738,'MRN' from dual union all
select 37739,'MRN' from dual union all
select 3774,'USAF' from dual union all
select 37740,'MRN' from dual union all
select 37741,'MRN' from dual union all
select 37742,'MRN' from dual union all
select 37743,'MRN' from dual union all
select 37744,'MRN' from dual union all
select 37745,'MRN' from dual union all
select 37746,'MRN' from dual union all
select 37747,'MRN' from dual union all
select 37748,'MRN' from dual union all
select 37749,'MRN' from dual union all
select 3775,'USAF' from dual union all
select 37750,'MRN' from dual union all
select 37751,'MRN' from dual union all
select 37752,'MRN' from dual union all
select 37753,'MRN' from dual union all
select 37754,'MRN' from dual union all
select 37755,'MRN' from dual union all
select 37756,'MRN' from dual union all
select 37757,'MRN' from dual union all
select 37758,'MRN' from dual union all
select 37759,'MRN' from dual union all
select 3776,'ISAS' from dual union all
select 37760,'MRN' from dual union all
select 37761,'MRN' from dual union all
select 37762,'MRN' from dual union all
select 37763,'MRN' from dual union all
select 37764,'MRN' from dual union all
select 37765,'MRN' from dual union all
select 37766,'MRN' from dual union all
select 37767,'MRN' from dual union all
select 37768,'MRN' from dual union all
select 37769,'MRN' from dual union all
select 3777,'USAF' from dual union all
select 37770,'MRN' from dual union all
select 37771,'MRN' from dual union all
select 37772,'MRN' from dual union all
select 37773,'MRN' from dual union all
select 37774,'MRN' from dual union all
select 37775,'MRN' from dual union all
select 37776,'MRN' from dual union all
select 37777,'MRN' from dual union all
select 37778,'MRN' from dual union all
select 37779,'MRN' from dual union all
select 3778,'PARD' from dual union all
select 37780,'MRN' from dual union all
select 37781,'MRN' from dual union all
select 37782,'MRN' from dual union all
select 37783,'MRN' from dual union all
select 37784,'MRN' from dual union all
select 37785,'MRN' from dual union all
select 37786,'MRN' from dual union all
select 37787,'MRN' from dual union all
select 37788,'MRN' from dual union all
select 37789,'MRN' from dual union all
select 3779,'ISAS' from dual union all
select 37790,'MRN' from dual union all
select 37791,'MRN' from dual union all
select 37792,'MRN' from dual union all
select 37793,'MRN' from dual union all
select 37794,'MRN' from dual union all
select 37795,'MRN' from dual union all
select 37796,'MRN' from dual union all
select 37797,'MRN' from dual union all
select 37798,'MRN' from dual union all
select 37799,'MRN' from dual union all
select 378,'WEHR' from dual union all
select 3780,'USN' from dual union all
select 37800,'MRN' from dual union all
select 37801,'MRN' from dual union all
select 37802,'MRN' from dual union all
select 37803,'NASA' from dual union all
select 37804,'MRN' from dual union all
select 37805,'MRN' from dual union all
select 37806,'MRN' from dual union all
select 37807,'MRN' from dual union all
select 37808,'MRN' from dual union all
select 37809,'MRN' from dual union all
select 3781,'ISAS' from dual;
insert into LAUNCH_AGENCY
select 37810,'MRN' from dual union all
select 37811,'MRN' from dual union all
select 37812,'MRN' from dual union all
select 37813,'MRN' from dual union all
select 37814,'MRN' from dual union all
select 37815,'MRN' from dual union all
select 37816,'MRN' from dual union all
select 37817,'MRN' from dual union all
select 37818,'MRN' from dual union all
select 37819,'MRN' from dual union all
select 3782,'USAF' from dual union all
select 37820,'MRN' from dual union all
select 37821,'MRN' from dual union all
select 37822,'MRN' from dual union all
select 37823,'MRN' from dual union all
select 37824,'MRN' from dual union all
select 37825,'MRN' from dual union all
select 37826,'MRN' from dual union all
select 37827,'MRN' from dual union all
select 37828,'MRN' from dual union all
select 37829,'MRN' from dual union all
select 3783,'ISAS' from dual union all
select 37830,'MRN' from dual union all
select 37831,'MRN' from dual union all
select 37832,'MRN' from dual union all
select 37833,'MRN' from dual union all
select 37834,'MRN' from dual union all
select 37835,'MRN' from dual union all
select 37836,'MRN' from dual union all
select 37837,'MRN' from dual union all
select 37838,'MRN' from dual union all
select 37839,'MRN' from dual union all
select 3784,'USAF' from dual union all
select 37840,'MRN' from dual union all
select 37841,'MRN' from dual union all
select 37842,'MRN' from dual union all
select 37843,'MRN' from dual union all
select 37844,'MRN' from dual union all
select 37845,'MRN' from dual union all
select 37846,'MRN' from dual union all
select 37847,'MRN' from dual union all
select 37848,'MRN' from dual union all
select 37849,'MRN' from dual union all
select 3785,'USAF' from dual union all
select 37850,'MRN' from dual union all
select 37851,'MRN' from dual union all
select 37852,'MRN' from dual union all
select 37853,'MRN' from dual union all
select 37854,'MRN' from dual union all
select 37855,'MRN' from dual union all
select 37856,'MRN' from dual union all
select 37857,'MRN' from dual union all
select 37858,'MRN' from dual union all
select 37859,'MRN' from dual union all
select 3786,'USAF' from dual union all
select 37860,'MRN' from dual union all
select 37861,'MRN' from dual union all
select 37862,'MRN' from dual union all
select 37863,'MRN' from dual union all
select 37864,'MRN' from dual union all
select 37865,'MRN' from dual union all
select 37866,'MRN' from dual union all
select 37867,'MRN' from dual union all
select 37868,'MRN' from dual union all
select 37869,'MRN' from dual union all
select 3787,'PARD' from dual union all
select 37870,'MRN' from dual union all
select 37871,'MRN' from dual union all
select 37872,'MRN' from dual union all
select 37873,'MRN' from dual union all
select 37874,'MRN' from dual union all
select 37875,'MRN' from dual union all
select 37876,'MRN' from dual union all
select 37877,'MRN' from dual union all
select 37878,'MRN' from dual union all
select 37879,'MRN' from dual union all
select 3788,'USAF' from dual union all
select 37880,'MRN' from dual union all
select 37881,'MRN' from dual union all
select 37882,'MRN' from dual union all
select 37883,'MRN' from dual union all
select 37884,'MRN' from dual union all
select 37885,'MRN' from dual union all
select 37886,'MRN' from dual union all
select 37887,'MRN' from dual union all
select 37888,'MRN' from dual union all
select 37889,'MRN' from dual union all
select 3789,'ORD' from dual union all
select 37890,'MRN' from dual union all
select 37891,'MRN' from dual union all
select 37892,'MRN' from dual union all
select 37893,'MRN' from dual union all
select 37894,'MRN' from dual union all
select 37895,'MRN' from dual union all
select 37896,'MRN' from dual union all
select 37897,'MRN' from dual union all
select 37898,'MRN' from dual union all
select 37899,'MRN' from dual union all
select 379,'WEHR' from dual union all
select 3790,'USA' from dual;
insert into LAUNCH_AGENCY
select 37900,'MRN' from dual union all
select 37901,'MRN' from dual union all
select 37902,'MRN' from dual union all
select 37903,'MRN' from dual union all
select 37904,'MRN' from dual union all
select 37905,'MRN' from dual union all
select 37906,'MRN' from dual union all
select 37907,'MRN' from dual union all
select 37908,'MRN' from dual union all
select 37909,'MRN' from dual union all
select 3791,'USAF' from dual union all
select 37910,'MRN' from dual union all
select 37911,'MRN' from dual union all
select 37912,'MRN' from dual union all
select 37913,'MRN' from dual union all
select 37914,'MRN' from dual union all
select 37915,'MRN' from dual union all
select 37916,'MRN' from dual union all
select 37917,'MRN' from dual union all
select 37918,'MRN' from dual union all
select 37919,'MRN' from dual union all
select 3792,'USA' from dual union all
select 37920,'MRN' from dual union all
select 37921,'MRN' from dual union all
select 37922,'NASA' from dual union all
select 37923,'MRN' from dual union all
select 37924,'MRN' from dual union all
select 37925,'MRN' from dual union all
select 37926,'MRN' from dual union all
select 37927,'MRN' from dual union all
select 37928,'MRN' from dual union all
select 37929,'MRN' from dual union all
select 3793,'USA' from dual union all
select 37930,'MRN' from dual union all
select 37931,'MRN' from dual union all
select 37932,'MRN' from dual union all
select 37933,'MRN' from dual union all
select 37934,'MRN' from dual union all
select 37935,'MRN' from dual union all
select 37936,'MRN' from dual union all
select 37937,'MRN' from dual union all
select 37938,'MRN' from dual union all
select 37939,'MRN' from dual union all
select 3794,'USA' from dual union all
select 37940,'MRN' from dual union all
select 37941,'MRN' from dual union all
select 37942,'MRN' from dual union all
select 37943,'MRN' from dual union all
select 37944,'MRN' from dual union all
select 37945,'MRN' from dual union all
select 37946,'MRN' from dual union all
select 37947,'MRN' from dual union all
select 37948,'MRN' from dual union all
select 37949,'MRN' from dual union all
select 3795,'USA' from dual union all
select 37950,'MRN' from dual union all
select 37951,'MRN' from dual union all
select 37952,'MRN' from dual union all
select 37953,'MRN' from dual union all
select 37954,'MRN' from dual union all
select 37955,'MRN' from dual union all
select 37956,'MRN' from dual union all
select 37957,'MRN' from dual union all
select 37958,'MRN' from dual union all
select 37959,'MRN' from dual union all
select 3796,'USA' from dual union all
select 37960,'MRN' from dual union all
select 37961,'MRN' from dual union all
select 37962,'MRN' from dual union all
select 37963,'MRN' from dual union all
select 37964,'MRN' from dual union all
select 37965,'MRN' from dual union all
select 37966,'MRN' from dual union all
select 37967,'MRN' from dual union all
select 37968,'MRN' from dual union all
select 37969,'MRN' from dual union all
select 3797,'USA' from dual union all
select 37970,'MRN' from dual union all
select 37971,'MRN' from dual union all
select 37972,'MRN' from dual union all
select 37973,'MRN' from dual union all
select 37974,'MRN' from dual union all
select 37975,'MRN' from dual union all
select 37976,'MRN' from dual union all
select 37977,'MRN' from dual union all
select 37978,'NASA' from dual union all
select 37979,'MRN' from dual union all
select 3798,'USA' from dual union all
select 37980,'MRN' from dual union all
select 37981,'MRN' from dual union all
select 37982,'MRN' from dual union all
select 37983,'MRN' from dual union all
select 37984,'MRN' from dual union all
select 37985,'MRN' from dual union all
select 37986,'MRN' from dual union all
select 37987,'MRN' from dual union all
select 37988,'MRN' from dual union all
select 37989,'NASA' from dual union all
select 3799,'USA' from dual union all
select 37990,'MRN' from dual;
insert into LAUNCH_AGENCY
select 37991,'MRN' from dual union all
select 37992,'MRN' from dual union all
select 37993,'MRN' from dual union all
select 37994,'MRN' from dual union all
select 37995,'MRN' from dual union all
select 37996,'MRN' from dual union all
select 37997,'MRN' from dual union all
select 37998,'MRN' from dual union all
select 37999,'MRN' from dual union all
select 38,'WEHR' from dual union all
select 380,'WEHR' from dual union all
select 3800,'USA' from dual union all
select 38000,'MRN' from dual union all
select 38001,'MRN' from dual union all
select 38002,'MRN' from dual union all
select 38003,'MRN' from dual union all
select 38004,'MRN' from dual union all
select 38005,'MRN' from dual union all
select 38006,'MRN' from dual union all
select 38007,'MRN' from dual union all
select 38008,'MRN' from dual union all
select 38009,'MRN' from dual union all
select 3801,'USA' from dual union all
select 38010,'MRN' from dual union all
select 38011,'MRN' from dual union all
select 38012,'MRN' from dual union all
select 38013,'MRN' from dual union all
select 38014,'MRN' from dual union all
select 38015,'MRN' from dual union all
select 38016,'MRN' from dual union all
select 38017,'MRN' from dual union all
select 38018,'MRN' from dual union all
select 38019,'MRN' from dual union all
select 3802,'USAF' from dual union all
select 38020,'MRN' from dual union all
select 38021,'MRN' from dual union all
select 38022,'MRN' from dual union all
select 38023,'MRN' from dual union all
select 38024,'MRN' from dual union all
select 38025,'MRN' from dual union all
select 38026,'MRN' from dual union all
select 38027,'MRN' from dual union all
select 38028,'MRN' from dual union all
select 38029,'MRN' from dual union all
select 3803,'USA' from dual union all
select 38030,'MRN' from dual union all
select 38031,'MRN' from dual union all
select 38032,'MRN' from dual union all
select 38033,'MRN' from dual union all
select 38034,'MRN' from dual union all
select 38035,'MRN' from dual union all
select 38036,'MRN' from dual union all
select 38037,'MRN' from dual union all
select 38038,'MRN' from dual union all
select 38039,'MRN' from dual union all
select 3804,'USA' from dual union all
select 38040,'MRN' from dual union all
select 38041,'MRN' from dual union all
select 38042,'MRN' from dual union all
select 38043,'MRN' from dual union all
select 38044,'MRN' from dual union all
select 38045,'MRN' from dual union all
select 38046,'MRN' from dual union all
select 38047,'MRN' from dual union all
select 38048,'MRN' from dual union all
select 38049,'MRN' from dual union all
select 3805,'USA' from dual union all
select 38050,'MRN' from dual union all
select 38051,'MRN' from dual union all
select 38052,'MRN' from dual union all
select 38053,'MRN' from dual union all
select 38054,'MRN' from dual union all
select 38055,'MRN' from dual union all
select 38056,'MRN' from dual union all
select 38057,'MRN' from dual union all
select 38058,'MRN' from dual union all
select 38059,'MRN' from dual union all
select 3806,'USA' from dual union all
select 38060,'MRN' from dual union all
select 38061,'MRN' from dual union all
select 38062,'MRN' from dual union all
select 38063,'MRN' from dual union all
select 38064,'MRN' from dual union all
select 38065,'MRN' from dual union all
select 38066,'MRN' from dual union all
select 38067,'MRN' from dual union all
select 38068,'MRN' from dual union all
select 38069,'MRN' from dual union all
select 3807,'USA' from dual union all
select 38070,'MRN' from dual union all
select 38071,'MRN' from dual union all
select 38072,'MRN' from dual union all
select 38073,'MRN' from dual union all
select 38074,'MRN' from dual union all
select 38075,'MRN' from dual union all
select 38076,'MRN' from dual union all
select 38077,'MRN' from dual union all
select 38078,'MRN' from dual union all
select 38079,'MRN' from dual union all
select 3808,'USA' from dual;
insert into LAUNCH_AGENCY
select 38080,'MRN' from dual union all
select 38081,'MRN' from dual union all
select 38082,'MRN' from dual union all
select 38083,'MRN' from dual union all
select 38084,'MRN' from dual union all
select 38085,'MRN' from dual union all
select 38086,'MRN' from dual union all
select 38087,'MRN' from dual union all
select 38088,'MRN' from dual union all
select 38089,'MRN' from dual union all
select 3809,'USA' from dual union all
select 38090,'MRN' from dual union all
select 38091,'MRN' from dual union all
select 38092,'MRN' from dual union all
select 38093,'MRN' from dual union all
select 38094,'MRN' from dual union all
select 38095,'AN' from dual union all
select 38096,'MRN' from dual union all
select 38097,'MRN' from dual union all
select 38098,'MRN' from dual union all
select 38099,'MRN' from dual union all
select 381,'WEHR' from dual union all
select 3810,'USA' from dual union all
select 38100,'MRN' from dual union all
select 38101,'MRN' from dual union all
select 38102,'MRN' from dual union all
select 38103,'MRN' from dual union all
select 38104,'MRN' from dual union all
select 38105,'MRN' from dual union all
select 38106,'MRN' from dual union all
select 38107,'MRN' from dual union all
select 38108,'MRN' from dual union all
select 38109,'MRN' from dual union all
select 3811,'USA' from dual union all
select 38110,'MRN' from dual union all
select 38111,'AN' from dual union all
select 38112,'MRN' from dual union all
select 38113,'MRN' from dual union all
select 38114,'MRN' from dual union all
select 38115,'MRN' from dual union all
select 38116,'MRN' from dual union all
select 38117,'MRN' from dual union all
select 38118,'MRN' from dual union all
select 38119,'MRN' from dual union all
select 3812,'USA' from dual union all
select 38120,'MRN' from dual union all
select 38121,'MRN' from dual union all
select 38122,'MRN' from dual union all
select 38123,'MRN' from dual union all
select 38124,'MRN' from dual union all
select 38125,'MRN' from dual union all
select 38126,'MRN' from dual union all
select 38127,'MRN' from dual union all
select 38128,'NASA' from dual union all
select 38129,'NASA' from dual union all
select 3813,'USAF' from dual union all
select 38130,'NASA' from dual union all
select 38131,'NASA' from dual union all
select 38132,'NASA' from dual union all
select 38133,'NASA' from dual union all
select 38134,'MRN' from dual union all
select 38135,'MRN' from dual union all
select 38136,'MRN' from dual union all
select 38137,'MRN' from dual union all
select 38138,'MRN' from dual union all
select 38139,'MRN' from dual union all
select 3814,'USA' from dual union all
select 38140,'MRN' from dual union all
select 38141,'MRN' from dual union all
select 38142,'MRN' from dual union all
select 38143,'MRN' from dual union all
select 38144,'MRN' from dual union all
select 38145,'MRN' from dual union all
select 38146,'MRN' from dual union all
select 38147,'MRN' from dual union all
select 38148,'MRN' from dual union all
select 38149,'MRN' from dual union all
select 3815,'USA' from dual union all
select 38150,'MRN' from dual union all
select 38151,'MRN' from dual union all
select 38152,'MRN' from dual union all
select 38153,'MRN' from dual union all
select 38154,'MRN' from dual union all
select 38155,'MRN' from dual union all
select 38156,'MRN' from dual union all
select 38157,'MRN' from dual union all
select 38158,'MRN' from dual union all
select 38159,'MRN' from dual union all
select 3816,'USA' from dual union all
select 38160,'MRN' from dual union all
select 38161,'MRN' from dual union all
select 38162,'MRN' from dual union all
select 38163,'MRN' from dual union all
select 38164,'MRN' from dual union all
select 38165,'MRN' from dual union all
select 38166,'MRN' from dual union all
select 38167,'MRN' from dual union all
select 38168,'MRN' from dual union all
select 38169,'MRN' from dual union all
select 3817,'USA' from dual;
insert into LAUNCH_AGENCY
select 38170,'MRN' from dual union all
select 38171,'MRN' from dual union all
select 38172,'MRN' from dual union all
select 38173,'MRN' from dual union all
select 38174,'MRN' from dual union all
select 38175,'MRN' from dual union all
select 38176,'MRN' from dual union all
select 38177,'MRN' from dual union all
select 38178,'MRN' from dual union all
select 38179,'MRN' from dual union all
select 3818,'USA' from dual union all
select 38180,'MRN' from dual union all
select 38181,'MRN' from dual union all
select 38182,'MRN' from dual union all
select 38183,'MRN' from dual union all
select 38184,'MRN' from dual union all
select 38185,'MRN' from dual union all
select 38186,'MRN' from dual union all
select 38187,'MRN' from dual union all
select 38188,'MRN' from dual union all
select 38189,'MRN' from dual union all
select 3819,'USA' from dual union all
select 38190,'MRN' from dual union all
select 38191,'MRN' from dual union all
select 38192,'MRN' from dual union all
select 38193,'MRN' from dual union all
select 38194,'MRN' from dual union all
select 38195,'MRN' from dual union all
select 38196,'MRN' from dual union all
select 38197,'MRN' from dual union all
select 38198,'MRN' from dual union all
select 38199,'MRN' from dual union all
select 382,'WEHR' from dual union all
select 3820,'USA' from dual union all
select 38200,'MRN' from dual union all
select 38201,'MRN' from dual union all
select 38202,'MRN' from dual union all
select 38203,'MRN' from dual union all
select 38204,'MRN' from dual union all
select 38205,'MRN' from dual union all
select 38206,'MRN' from dual union all
select 38207,'MRN' from dual union all
select 38208,'MRN' from dual union all
select 38209,'NASA' from dual union all
select 3821,'USA' from dual union all
select 38210,'MRN' from dual union all
select 38211,'MRN' from dual union all
select 38212,'MRN' from dual union all
select 38213,'MRN' from dual union all
select 38214,'MRN' from dual union all
select 38215,'MRN' from dual union all
select 38216,'MRN' from dual union all
select 38217,'MRN' from dual union all
select 38218,'MRN' from dual union all
select 38219,'MRN' from dual union all
select 3822,'USA' from dual union all
select 38220,'NASA' from dual union all
select 38221,'MRN' from dual union all
select 38222,'MRN' from dual union all
select 38223,'MRN' from dual union all
select 38224,'MRN' from dual union all
select 38225,'MRN' from dual union all
select 38226,'MRN' from dual union all
select 38227,'AN' from dual union all
select 38228,'MRN' from dual union all
select 38229,'MRN' from dual union all
select 3823,'USA' from dual union all
select 38230,'MRN' from dual union all
select 38231,'MRN' from dual union all
select 38232,'MRN' from dual union all
select 38233,'MRN' from dual union all
select 38234,'MRN' from dual union all
select 38235,'MRN' from dual union all
select 38236,'MRN' from dual union all
select 38237,'MRN' from dual union all
select 38238,'MRN' from dual union all
select 38239,'MRN' from dual union all
select 3824,'USN' from dual union all
select 38240,'MRN' from dual union all
select 38241,'MRN' from dual union all
select 38242,'MRN' from dual union all
select 38243,'MRN' from dual union all
select 38244,'MRN' from dual union all
select 38245,'MRN' from dual union all
select 38246,'MRN' from dual union all
select 38247,'MRN' from dual union all
select 38248,'MRN' from dual union all
select 38249,'MRN' from dual union all
select 3825,'USA' from dual union all
select 38250,'MRN' from dual union all
select 38251,'MRN' from dual union all
select 38252,'MRN' from dual union all
select 38253,'MRN' from dual union all
select 38254,'MRN' from dual union all
select 38255,'MRN' from dual union all
select 38256,'MRN' from dual union all
select 38257,'MRN' from dual union all
select 38258,'MRN' from dual union all
select 38259,'MRN' from dual union all
select 3826,'USA' from dual;
insert into LAUNCH_AGENCY
select 38260,'MRN' from dual union all
select 38261,'MRN' from dual union all
select 38262,'MRN' from dual union all
select 38263,'MRN' from dual union all
select 38264,'MRN' from dual union all
select 38265,'MRN' from dual union all
select 38266,'MRN' from dual union all
select 38267,'MRN' from dual union all
select 38268,'MRN' from dual union all
select 38269,'MRN' from dual union all
select 3827,'USA' from dual union all
select 38270,'MRN' from dual union all
select 38271,'MRN' from dual union all
select 38272,'MRN' from dual union all
select 38273,'MRN' from dual union all
select 38274,'MRN' from dual union all
select 38275,'MRN' from dual union all
select 38276,'MRN' from dual union all
select 38277,'MRN' from dual union all
select 38278,'MRN' from dual union all
select 38279,'MRN' from dual union all
select 3828,'USA' from dual union all
select 38280,'MRN' from dual union all
select 38281,'MRN' from dual union all
select 38282,'MRN' from dual union all
select 38283,'MRN' from dual union all
select 38284,'MRN' from dual union all
select 38285,'MRN' from dual union all
select 38286,'MRN' from dual union all
select 38287,'MRN' from dual union all
select 38288,'MRN' from dual union all
select 38289,'MRN' from dual union all
select 3829,'USA' from dual union all
select 38290,'MRN' from dual union all
select 38291,'MRN' from dual union all
select 38292,'MRN' from dual union all
select 38293,'MRN' from dual union all
select 38294,'MRN' from dual union all
select 38295,'MRN' from dual union all
select 38296,'MRN' from dual union all
select 38297,'MRN' from dual union all
select 38298,'MRN' from dual union all
select 38299,'MRN' from dual union all
select 383,'WEHR' from dual union all
select 3830,'USA' from dual union all
select 38300,'MRN' from dual union all
select 38301,'MRN' from dual union all
select 38302,'MRN' from dual union all
select 38303,'MRN' from dual union all
select 38304,'MRN' from dual union all
select 38305,'MRN' from dual union all
select 38306,'MRN' from dual union all
select 38307,'MRN' from dual union all
select 38308,'MRN' from dual union all
select 38309,'MRN' from dual union all
select 3831,'USA' from dual union all
select 38310,'MRN' from dual union all
select 38311,'MRN' from dual union all
select 38312,'MRN' from dual union all
select 38313,'MRN' from dual union all
select 38314,'MRN' from dual union all
select 38315,'MRN' from dual union all
select 38316,'MRN' from dual union all
select 38317,'MRN' from dual union all
select 38318,'MRN' from dual union all
select 38319,'MRN' from dual union all
select 3832,'USA' from dual union all
select 38320,'MRN' from dual union all
select 38321,'MRN' from dual union all
select 38322,'MRN' from dual union all
select 38323,'MRN' from dual union all
select 38324,'MRN' from dual union all
select 38325,'MRN' from dual union all
select 38326,'MRN' from dual union all
select 38327,'MRN' from dual union all
select 38328,'MRN' from dual union all
select 38329,'MRN' from dual union all
select 3833,'USA' from dual union all
select 38330,'MRN' from dual union all
select 38331,'MRN' from dual union all
select 38332,'MRN' from dual union all
select 38333,'MRN' from dual union all
select 38334,'MRN' from dual union all
select 38335,'MRN' from dual union all
select 38336,'MRN' from dual union all
select 38337,'MRN' from dual union all
select 38338,'MRN' from dual union all
select 38339,'MRN' from dual union all
select 3834,'USA' from dual union all
select 38340,'MRN' from dual union all
select 38341,'MRN' from dual union all
select 38342,'MRN' from dual union all
select 38343,'MRN' from dual union all
select 38344,'MRN' from dual union all
select 38345,'MRN' from dual union all
select 38346,'MRN' from dual union all
select 38347,'MRN' from dual union all
select 38348,'MRN' from dual union all
select 38349,'MRN' from dual union all
select 3835,'USN' from dual;
insert into LAUNCH_AGENCY
select 38350,'MRN' from dual union all
select 38351,'MRN' from dual union all
select 38352,'MRN' from dual union all
select 38353,'MRN' from dual union all
select 38354,'MRN' from dual union all
select 38355,'MRN' from dual union all
select 38356,'MRN' from dual union all
select 38357,'MRN' from dual union all
select 38358,'MRN' from dual union all
select 38359,'MRN' from dual union all
select 3836,'USA' from dual union all
select 38360,'MRN' from dual union all
select 38361,'MRN' from dual union all
select 38362,'MRN' from dual union all
select 38363,'MRN' from dual union all
select 38364,'MRN' from dual union all
select 38365,'MRN' from dual union all
select 38366,'MRN' from dual union all
select 38367,'MRN' from dual union all
select 38368,'MRN' from dual union all
select 38369,'MRN' from dual union all
select 3837,'USA' from dual union all
select 38370,'MRN' from dual union all
select 38371,'MRN' from dual union all
select 38372,'MRN' from dual union all
select 38373,'MRN' from dual union all
select 38374,'MRN' from dual union all
select 38375,'MRN' from dual union all
select 38376,'MRN' from dual union all
select 38377,'MRN' from dual union all
select 38378,'MRN' from dual union all
select 38379,'MRN' from dual union all
select 3838,'USA' from dual union all
select 38380,'MRN' from dual union all
select 38381,'MRN' from dual union all
select 38382,'MRN' from dual union all
select 38383,'MRN' from dual union all
select 38384,'MRN' from dual union all
select 38385,'MRN' from dual union all
select 38386,'MRN' from dual union all
select 38387,'MRN' from dual union all
select 38388,'MRN' from dual union all
select 38389,'MRN' from dual union all
select 3839,'USA' from dual union all
select 38390,'MRN' from dual union all
select 38391,'MRN' from dual union all
select 38392,'NASA' from dual union all
select 38393,'MRN' from dual union all
select 38394,'NASA' from dual union all
select 38395,'MRN' from dual union all
select 38396,'MRN' from dual union all
select 38397,'MRN' from dual union all
select 38398,'MRN' from dual union all
select 38399,'MRN' from dual union all
select 384,'WER485-2' from dual union all
select 3840,'USA' from dual union all
select 38400,'MRN' from dual union all
select 38401,'MRN' from dual union all
select 38402,'MRN' from dual union all
select 38403,'MRN' from dual union all
select 38404,'MRN' from dual union all
select 38405,'MRN' from dual union all
select 38406,'MRN' from dual union all
select 38407,'MRN' from dual union all
select 38408,'MRN' from dual union all
select 38409,'MRN' from dual union all
select 3841,'USA' from dual union all
select 38410,'MRN' from dual union all
select 38411,'MRN' from dual union all
select 38412,'MRN' from dual union all
select 38413,'MRN' from dual union all
select 38414,'MRN' from dual union all
select 38415,'MRN' from dual union all
select 38416,'MRN' from dual union all
select 38417,'MRN' from dual union all
select 38418,'MRN' from dual union all
select 38419,'MRN' from dual union all
select 3842,'USA' from dual union all
select 38420,'MRN' from dual union all
select 38421,'MRN' from dual union all
select 38422,'MRN' from dual union all
select 38423,'MRN' from dual union all
select 38424,'MRN' from dual union all
select 38425,'MRN' from dual union all
select 38426,'MRN' from dual union all
select 38427,'MRN' from dual union all
select 38428,'MRN' from dual union all
select 38429,'MRN' from dual union all
select 3843,'USA' from dual union all
select 38430,'MRN' from dual union all
select 38431,'MRN' from dual union all
select 38432,'MRN' from dual union all
select 38433,'MRN' from dual union all
select 38434,'MRN' from dual union all
select 38435,'MRN' from dual union all
select 38436,'MRN' from dual union all
select 38437,'MRN' from dual union all
select 38438,'MRN' from dual union all
select 38439,'MRN' from dual union all
select 3844,'USA' from dual;
insert into LAUNCH_AGENCY
select 38440,'MRN' from dual union all
select 38441,'MRN' from dual union all
select 38442,'MRN' from dual union all
select 38443,'MRN' from dual union all
select 38444,'MRN' from dual union all
select 38445,'MRN' from dual union all
select 38446,'MRN' from dual union all
select 38447,'MRN' from dual union all
select 38448,'MRN' from dual union all
select 38449,'MRN' from dual union all
select 3845,'USA' from dual union all
select 38450,'MRN' from dual union all
select 38451,'MRN' from dual union all
select 38452,'MRN' from dual union all
select 38453,'MRN' from dual union all
select 38454,'MRN' from dual union all
select 38455,'MRN' from dual union all
select 38456,'NASA' from dual union all
select 38457,'MRN' from dual union all
select 38458,'MRN' from dual union all
select 38459,'MRN' from dual union all
select 3846,'USN' from dual union all
select 38460,'MRN' from dual union all
select 38461,'MRN' from dual union all
select 38462,'MRN' from dual union all
select 38463,'MRN' from dual union all
select 38464,'MRN' from dual union all
select 38465,'MRN' from dual union all
select 38466,'MRN' from dual union all
select 38467,'MRN' from dual union all
select 38468,'MRN' from dual union all
select 38469,'MRN' from dual union all
select 3847,'USA' from dual union all
select 38470,'MRN' from dual union all
select 38471,'MRN' from dual union all
select 38472,'MRN' from dual union all
select 38473,'MRN' from dual union all
select 38474,'MRN' from dual union all
select 38475,'MRN' from dual union all
select 38476,'MRN' from dual union all
select 38477,'MRN' from dual union all
select 38478,'MRN' from dual union all
select 38479,'MRN' from dual union all
select 3848,'USA' from dual union all
select 38480,'MRN' from dual union all
select 38481,'MRN' from dual union all
select 38482,'MRN' from dual union all
select 38483,'MRN' from dual union all
select 38484,'MRN' from dual union all
select 38485,'MRN' from dual union all
select 38486,'MRN' from dual union all
select 38487,'MRN' from dual union all
select 38488,'MRN' from dual union all
select 38489,'MRN' from dual union all
select 3849,'USA' from dual union all
select 38490,'MRN' from dual union all
select 38491,'MRN' from dual union all
select 38492,'MRN' from dual union all
select 38493,'MRN' from dual union all
select 38494,'MRN' from dual union all
select 38495,'MRN' from dual union all
select 38496,'MRN' from dual union all
select 38497,'MRN' from dual union all
select 38498,'MRN' from dual union all
select 38499,'MRN' from dual union all
select 385,'WEHR' from dual union all
select 3850,'USA' from dual union all
select 38500,'MRN' from dual union all
select 38501,'MRN' from dual union all
select 38502,'MRN' from dual union all
select 38503,'MRN' from dual union all
select 38504,'MRN' from dual union all
select 38505,'MRN' from dual union all
select 38506,'MRN' from dual union all
select 38507,'MRN' from dual union all
select 38508,'MRN' from dual union all
select 38509,'MRN' from dual union all
select 3851,'USA' from dual union all
select 38510,'MRN' from dual union all
select 38511,'MRN' from dual union all
select 38512,'MRN' from dual union all
select 38513,'MRN' from dual union all
select 38514,'MRN' from dual union all
select 38515,'MRN' from dual union all
select 38516,'MRN' from dual union all
select 38517,'MRN' from dual union all
select 38518,'MRN' from dual union all
select 38519,'MRN' from dual union all
select 3852,'USA' from dual union all
select 38520,'MRN' from dual union all
select 38521,'MRN' from dual union all
select 38522,'MRN' from dual union all
select 38523,'MRN' from dual union all
select 38524,'MRN' from dual union all
select 38525,'MRN' from dual union all
select 38526,'MRN' from dual union all
select 38527,'MRN' from dual union all
select 38528,'MRN' from dual union all
select 38529,'MRN' from dual union all
select 3853,'USA' from dual;
insert into LAUNCH_AGENCY
select 38530,'MRN' from dual union all
select 38531,'MRN' from dual union all
select 38532,'MRN' from dual union all
select 38533,'MRN' from dual union all
select 38534,'MRN' from dual union all
select 38535,'MRN' from dual union all
select 38536,'MRN' from dual union all
select 38537,'MRN' from dual union all
select 38538,'MRN' from dual union all
select 38539,'MRN' from dual union all
select 3854,'USA' from dual union all
select 38540,'MRN' from dual union all
select 38541,'MRN' from dual union all
select 38542,'MRN' from dual union all
select 38543,'MRN' from dual union all
select 38544,'MRN' from dual union all
select 38545,'MRN' from dual union all
select 38546,'MRN' from dual union all
select 38547,'MRN' from dual union all
select 38548,'MRN' from dual union all
select 38549,'MRN' from dual union all
select 3855,'USA' from dual union all
select 38550,'MRN' from dual union all
select 38551,'MRN' from dual union all
select 38552,'MRN' from dual union all
select 38553,'MRN' from dual union all
select 38554,'MRN' from dual union all
select 38555,'MRN' from dual union all
select 38556,'MRN' from dual union all
select 38557,'MRN' from dual union all
select 38558,'MRN' from dual union all
select 38559,'MRN' from dual union all
select 3856,'USA' from dual union all
select 38560,'MRN' from dual union all
select 38561,'AN' from dual union all
select 38562,'AN' from dual union all
select 38563,'MRN' from dual union all
select 38564,'MRN' from dual union all
select 38565,'AN' from dual union all
select 38566,'AN' from dual union all
select 38567,'AN' from dual union all
select 38568,'AN' from dual union all
select 38569,'AN' from dual union all
select 3857,'USN' from dual union all
select 38570,'AN' from dual union all
select 38571,'AN' from dual union all
select 38572,'AN' from dual union all
select 38573,'AN' from dual union all
select 38574,'AN' from dual union all
select 38575,'AN' from dual union all
select 38576,'MRN' from dual union all
select 38577,'AN' from dual union all
select 38578,'AN' from dual union all
select 38579,'AN' from dual union all
select 3858,'USA' from dual union all
select 38580,'AN' from dual union all
select 38581,'AN' from dual union all
select 38582,'AN' from dual union all
select 38583,'AN' from dual union all
select 38584,'MRN' from dual union all
select 38585,'AN' from dual union all
select 38586,'AN' from dual union all
select 38587,'AN' from dual union all
select 38588,'AN' from dual union all
select 38589,'AN' from dual union all
select 3859,'USA' from dual union all
select 38590,'AN' from dual union all
select 38591,'MRN' from dual union all
select 38592,'MRN' from dual union all
select 38593,'AN' from dual union all
select 38594,'AN' from dual union all
select 38595,'AN' from dual union all
select 38596,'AN' from dual union all
select 38597,'AN' from dual union all
select 38598,'AN' from dual union all
select 38599,'AN' from dual union all
select 386,'WEHR' from dual union all
select 3860,'USA' from dual union all
select 38600,'MRN' from dual union all
select 38601,'MRN' from dual union all
select 38602,'AN' from dual union all
select 38603,'AN' from dual union all
select 38604,'AN' from dual union all
select 38605,'AN' from dual union all
select 38606,'AN' from dual union all
select 38607,'AN' from dual union all
select 38608,'AN' from dual union all
select 38609,'AN' from dual union all
select 3861,'USA' from dual union all
select 38610,'AN' from dual union all
select 38611,'MRN' from dual union all
select 38612,'AN' from dual union all
select 38613,'AN' from dual union all
select 38614,'AN' from dual union all
select 38615,'AN' from dual union all
select 38616,'AN' from dual union all
select 38617,'AN' from dual union all
select 38618,'MRN' from dual union all
select 38619,'AN' from dual union all
select 3862,'USA' from dual;
insert into LAUNCH_AGENCY
select 38620,'AN' from dual union all
select 38621,'AN' from dual union all
select 38622,'AN' from dual union all
select 38623,'AN' from dual union all
select 38624,'MRN' from dual union all
select 38625,'MRN' from dual union all
select 38626,'AN' from dual union all
select 38627,'MRN' from dual union all
select 38628,'AN' from dual union all
select 38629,'AN' from dual union all
select 3863,'USA' from dual union all
select 38630,'AN' from dual union all
select 38631,'AN' from dual union all
select 38632,'AN' from dual union all
select 38633,'AN' from dual union all
select 38634,'AN' from dual union all
select 38635,'AN' from dual union all
select 38636,'AN' from dual union all
select 38637,'NASA' from dual union all
select 38638,'NASA' from dual union all
select 38639,'NASA' from dual union all
select 3864,'USA' from dual union all
select 38640,'AN' from dual union all
select 38641,'AN' from dual union all
select 38642,'AN' from dual union all
select 38643,'AN' from dual union all
select 38644,'AN' from dual union all
select 38645,'AN' from dual union all
select 38646,'AN' from dual union all
select 38647,'AN' from dual union all
select 38648,'AN' from dual union all
select 38649,'NASA' from dual union all
select 3865,'USA' from dual union all
select 38650,'NASA' from dual union all
select 38651,'MRN' from dual union all
select 38652,'AN' from dual union all
select 38653,'AN' from dual union all
select 38654,'AN' from dual union all
select 38655,'AN' from dual union all
select 38656,'AN' from dual union all
select 38657,'AN' from dual union all
select 38658,'AN' from dual union all
select 38659,'AN' from dual union all
select 3866,'USA' from dual union all
select 38660,'AN' from dual union all
select 38661,'AN' from dual union all
select 38662,'MRN' from dual union all
select 38663,'AN' from dual union all
select 38664,'AN' from dual union all
select 38665,'AN' from dual union all
select 38666,'AN' from dual union all
select 38667,'AN' from dual union all
select 38668,'AN' from dual union all
select 38669,'AN' from dual union all
select 3867,'USA' from dual union all
select 38670,'AN' from dual union all
select 38671,'AN' from dual union all
select 38672,'AN' from dual union all
select 38673,'AN' from dual union all
select 38674,'AN' from dual union all
select 38675,'AN' from dual union all
select 38676,'AN' from dual union all
select 38677,'AN' from dual union all
select 38678,'AN' from dual union all
select 38679,'AN' from dual union all
select 3868,'RCAR' from dual union all
select 38680,'AN' from dual union all
select 38681,'AN' from dual union all
select 38682,'AN' from dual union all
select 38683,'AN' from dual union all
select 38684,'AN' from dual union all
select 38685,'AN' from dual union all
select 38686,'MRN' from dual union all
select 38687,'MRN' from dual union all
select 38688,'AN' from dual union all
select 38689,'AN' from dual union all
select 3869,'USA' from dual union all
select 38690,'AN' from dual union all
select 38691,'AN' from dual union all
select 38692,'AN' from dual union all
select 38693,'AN' from dual union all
select 38694,'MRN' from dual union all
select 38695,'AN' from dual union all
select 38696,'AN' from dual union all
select 38697,'AN' from dual union all
select 38698,'AN' from dual union all
select 38699,'AN' from dual union all
select 387,'WEHR' from dual union all
select 3870,'USA' from dual union all
select 38700,'MRN' from dual union all
select 38701,'AN' from dual union all
select 38702,'AN' from dual union all
select 38703,'AN' from dual union all
select 38704,'AN' from dual union all
select 38705,'AN' from dual union all
select 38706,'MRN' from dual union all
select 38707,'NASA' from dual union all
select 38708,'MRN' from dual union all
select 38709,'AN' from dual union all
select 3871,'USA' from dual;
insert into LAUNCH_AGENCY
select 38710,'AN' from dual union all
select 38711,'AN' from dual union all
select 38712,'MRN' from dual union all
select 38713,'AN' from dual union all
select 38714,'AN' from dual union all
select 38715,'AN' from dual union all
select 38716,'AN' from dual union all
select 38717,'NASA' from dual union all
select 38718,'NASA' from dual union all
select 38719,'AN' from dual union all
select 3872,'USA' from dual union all
select 38720,'AN' from dual union all
select 38721,'AN' from dual union all
select 38722,'MRN' from dual union all
select 38723,'AN' from dual union all
select 38724,'MRN' from dual union all
select 38725,'AN' from dual union all
select 38726,'AN' from dual union all
select 38727,'MRN' from dual union all
select 38728,'AN' from dual union all
select 38729,'MRN' from dual union all
select 3873,'USA' from dual union all
select 38730,'AN' from dual union all
select 38731,'AN' from dual union all
select 38732,'AN' from dual union all
select 38733,'MRN' from dual union all
select 38734,'AN' from dual union all
select 38735,'AN' from dual union all
select 38736,'MRN' from dual union all
select 38737,'MRN' from dual union all
select 38738,'AN' from dual union all
select 38739,'AN' from dual union all
select 3874,'USA' from dual union all
select 38740,'AN' from dual union all
select 38741,'AN' from dual union all
select 38742,'MRN' from dual union all
select 38743,'MRN' from dual union all
select 38744,'AN' from dual union all
select 38745,'AN' from dual union all
select 38746,'MRN' from dual union all
select 38747,'AN' from dual union all
select 38748,'AN' from dual union all
select 38749,'AN' from dual union all
select 3875,'USA' from dual union all
select 38750,'AN' from dual union all
select 38751,'MRN' from dual union all
select 38752,'AN' from dual union all
select 38753,'AN' from dual union all
select 38754,'AN' from dual union all
select 38755,'AN' from dual union all
select 38756,'AN' from dual union all
select 38757,'AN' from dual union all
select 38758,'MRN' from dual union all
select 38759,'AN' from dual union all
select 3876,'USA' from dual union all
select 38760,'AN' from dual union all
select 38761,'AN' from dual union all
select 38762,'AN' from dual union all
select 38763,'AN' from dual union all
select 38764,'AN' from dual union all
select 38765,'MRN' from dual union all
select 38766,'MRN' from dual union all
select 38767,'AN' from dual union all
select 38768,'AN' from dual union all
select 38769,'AN' from dual union all
select 3877,'USA' from dual union all
select 38770,'AN' from dual union all
select 38771,'AN' from dual union all
select 38772,'MRN' from dual union all
select 38773,'AN' from dual union all
select 38774,'MRN' from dual union all
select 38775,'AN' from dual union all
select 38776,'AN' from dual union all
select 38777,'AN' from dual union all
select 38778,'AN' from dual union all
select 38779,'NASA' from dual union all
select 3878,'USN' from dual union all
select 38780,'AN' from dual union all
select 38781,'AN' from dual union all
select 38782,'AN' from dual union all
select 38783,'MRN' from dual union all
select 38784,'AN' from dual union all
select 38785,'AN' from dual union all
select 38786,'AN' from dual union all
select 38787,'AN' from dual union all
select 38788,'MRN' from dual union all
select 38789,'MRN' from dual union all
select 3879,'USAF' from dual union all
select 38790,'NASA' from dual union all
select 38791,'AN' from dual union all
select 38792,'AN' from dual union all
select 38793,'AN' from dual union all
select 38794,'AN' from dual union all
select 38795,'MRN' from dual union all
select 38796,'AN' from dual union all
select 38797,'AN' from dual union all
select 38798,'AN' from dual union all
select 38799,'AN' from dual union all
select 388,'WEHR' from dual union all
select 3880,'USN' from dual;
insert into LAUNCH_AGENCY
select 38800,'MRN' from dual union all
select 38801,'AN' from dual union all
select 38802,'AN' from dual union all
select 38803,'MRN' from dual union all
select 38804,'AN' from dual union all
select 38805,'AN' from dual union all
select 38806,'AN' from dual union all
select 38807,'AN' from dual union all
select 38808,'AN' from dual union all
select 38809,'AN' from dual union all
select 3881,'USN' from dual union all
select 38810,'AN' from dual union all
select 38811,'MRN' from dual union all
select 38812,'AN' from dual union all
select 38813,'AN' from dual union all
select 38814,'AN' from dual union all
select 38815,'AN' from dual union all
select 38816,'AN' from dual union all
select 38817,'MRN' from dual union all
select 38818,'MRN' from dual union all
select 38819,'MRN' from dual union all
select 3882,'USN' from dual union all
select 38820,'AN' from dual union all
select 38821,'AN' from dual union all
select 38822,'AN' from dual union all
select 38823,'MRN' from dual union all
select 38824,'AN' from dual union all
select 38825,'MRN' from dual union all
select 38826,'MRN' from dual union all
select 38827,'AN' from dual union all
select 38828,'AN' from dual union all
select 38829,'AN' from dual union all
select 3883,'USN' from dual union all
select 38830,'AN' from dual union all
select 38831,'AN' from dual union all
select 38832,'MRN' from dual union all
select 38833,'MRN' from dual union all
select 38834,'MRN' from dual union all
select 38835,'AN' from dual union all
select 38836,'AN' from dual union all
select 38837,'AN' from dual union all
select 38838,'AN' from dual union all
select 38839,'AN' from dual union all
select 3884,'USN' from dual union all
select 38840,'MRN' from dual union all
select 38841,'AN' from dual union all
select 38842,'AN' from dual union all
select 38843,'AN' from dual union all
select 38844,'AN' from dual union all
select 38845,'AN' from dual union all
select 38846,'AN' from dual union all
select 38847,'AN' from dual union all
select 38848,'MRN' from dual union all
select 38849,'AN' from dual union all
select 3885,'USN' from dual union all
select 38850,'AN' from dual union all
select 38851,'AN' from dual union all
select 38852,'AN' from dual union all
select 38853,'AN' from dual union all
select 38854,'MRN' from dual union all
select 38855,'AN' from dual union all
select 38856,'AN' from dual union all
select 38857,'MRN' from dual union all
select 38858,'MRN' from dual union all
select 38859,'MRN' from dual union all
select 3886,'USN' from dual union all
select 38860,'MRN' from dual union all
select 38861,'MRN' from dual union all
select 38862,'AN' from dual union all
select 38863,'MRN' from dual union all
select 38864,'AN' from dual union all
select 38865,'AN' from dual union all
select 38866,'AN' from dual union all
select 38867,'MRN' from dual union all
select 38868,'MRN' from dual union all
select 38869,'AN' from dual union all
select 3887,'USN' from dual union all
select 38870,'NASA' from dual union all
select 38871,'AN' from dual union all
select 38872,'AN' from dual union all
select 38873,'AN' from dual union all
select 38874,'MRN' from dual union all
select 38875,'MRN' from dual union all
select 38876,'MRN' from dual union all
select 38877,'AN' from dual union all
select 38878,'AN' from dual union all
select 38879,'AN' from dual union all
select 3888,'USN' from dual union all
select 38880,'AN' from dual union all
select 38881,'AN' from dual union all
select 38882,'MRN' from dual union all
select 38883,'AN' from dual union all
select 38884,'AN' from dual union all
select 38885,'AN' from dual union all
select 38886,'MRN' from dual union all
select 38887,'AN' from dual union all
select 38888,'AN' from dual union all
select 38889,'AN' from dual union all
select 3889,'USAF' from dual union all
select 38890,'AN' from dual;
insert into LAUNCH_AGENCY
select 38891,'AN' from dual union all
select 38892,'AN' from dual union all
select 38893,'NASA' from dual union all
select 38894,'MRN' from dual union all
select 38895,'AN' from dual union all
select 38896,'MRN' from dual union all
select 38897,'MRN' from dual union all
select 38898,'AN' from dual union all
select 38899,'AN' from dual union all
select 389,'WEHR' from dual union all
select 3890,'PARD' from dual union all
select 38900,'AN' from dual union all
select 38901,'NASA' from dual union all
select 38902,'NASA' from dual union all
select 38903,'AN' from dual union all
select 38904,'AN' from dual union all
select 38905,'AN' from dual union all
select 38906,'AN' from dual union all
select 38907,'AN' from dual union all
select 38908,'AN' from dual union all
select 38909,'MRN' from dual union all
select 3891,'USAF' from dual union all
select 38910,'AN' from dual union all
select 38911,'AN' from dual union all
select 38912,'AN' from dual union all
select 38913,'MRN' from dual union all
select 38914,'AN' from dual union all
select 38915,'MRN' from dual union all
select 38916,'AN' from dual union all
select 38917,'AN' from dual union all
select 38918,'AN' from dual union all
select 38919,'AN' from dual union all
select 3892,'USAF' from dual union all
select 38920,'AN' from dual union all
select 38921,'AN' from dual union all
select 38922,'AN' from dual union all
select 38923,'AN' from dual union all
select 38924,'AN' from dual union all
select 38925,'AN' from dual union all
select 38926,'AN' from dual union all
select 38927,'AN' from dual union all
select 38928,'AN' from dual union all
select 38929,'AN' from dual union all
select 3893,'USAF' from dual union all
select 38930,'AN' from dual union all
select 38931,'AN' from dual union all
select 38932,'AN' from dual union all
select 38933,'AN' from dual union all
select 38934,'AN' from dual union all
select 38935,'AN' from dual union all
select 38936,'AN' from dual union all
select 38937,'AN' from dual union all
select 38938,'AN' from dual union all
select 38939,'AN' from dual union all
select 3894,'USAF' from dual union all
select 38940,'AN' from dual union all
select 38941,'AN' from dual union all
select 38942,'AN' from dual union all
select 38943,'AN' from dual union all
select 38944,'AN' from dual union all
select 38945,'AN' from dual union all
select 38946,'AN' from dual union all
select 38947,'AN' from dual union all
select 38948,'AN' from dual union all
select 38949,'AN' from dual union all
select 3895,'USAF' from dual union all
select 38950,'MRN' from dual union all
select 38951,'MRN' from dual union all
select 38952,'AN' from dual union all
select 38953,'RVSN' from dual union all
select 38954,'RVSN' from dual union all
select 38955,'USAF' from dual union all
select 38956,'NASA' from dual union all
select 38957,'RVSN' from dual union all
select 38958,'RVSN' from dual union all
select 38959,'RVSN' from dual union all
select 3896,'USAF' from dual union all
select 38960,'RVSN' from dual union all
select 38961,'RVSN' from dual union all
select 38962,'NASA' from dual union all
select 38963,'RVSN' from dual union all
select 38964,'RVSN' from dual union all
select 38965,'RVSN' from dual union all
select 38966,'RVSN' from dual union all
select 38967,'RVSN' from dual union all
select 38968,'10ADS' from dual union all
select 38969,'NASA' from dual union all
select 3897,'USAF' from dual union all
select 38970,'RVSN' from dual union all
select 38971,'NASDA' from dual union all
select 38972,'RVSN' from dual union all
select 38973,'RVSN' from dual union all
select 38974,'RVSN' from dual union all
select 38975,'USAF' from dual union all
select 38976,'RVSN' from dual union all
select 38977,'RVSN' from dual union all
select 38978,'RVSN' from dual union all
select 38979,'USAF' from dual union all
select 3898,'USAF' from dual union all
select 38980,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 38981,'NASA' from dual union all
select 38982,'RVSN' from dual union all
select 38983,'RVSN' from dual union all
select 38984,'RVSN' from dual union all
select 38985,'RVSN' from dual union all
select 38986,'RVSN' from dual union all
select 38987,'NASA' from dual union all
select 38988,'RVSN' from dual union all
select 38989,'RVSN' from dual union all
select 3899,'USAF' from dual union all
select 38990,'AFSC' from dual union all
select 38991,'NASA' from dual union all
select 38992,'RVSN' from dual union all
select 38993,'RVSN' from dual union all
select 38994,'NASA' from dual union all
select 38995,'RVSN' from dual union all
select 38996,'RVSN' from dual union all
select 38997,'RVSN' from dual union all
select 38998,'RVSN' from dual union all
select 38999,'AFSC' from dual union all
select 39,'WEHR' from dual union all
select 390,'WEHR' from dual union all
select 3900,'USAF' from dual union all
select 39000,'RVSN' from dual union all
select 39001,'RVSN' from dual union all
select 39002,'USAF' from dual union all
select 39003,'RVSN' from dual union all
select 39004,'RVSN' from dual union all
select 39005,'NASA' from dual union all
select 39006,'RVSN' from dual union all
select 39007,'RVSN' from dual union all
select 39008,'RVSN' from dual union all
select 39009,'RVSN' from dual union all
select 3901,'RCAR' from dual union all
select 39010,'RVSN' from dual union all
select 39011,'USAF' from dual union all
select 39012,'RVSN' from dual union all
select 39013,'RVSN' from dual union all
select 39014,'RVSN' from dual union all
select 39015,'RVSN' from dual union all
select 39016,'RVSN' from dual union all
select 39017,'USAF' from dual union all
select 39018,'NASA' from dual union all
select 39019,'RVSN' from dual union all
select 3902,'USAF' from dual union all
select 39020,'RVSN' from dual union all
select 39021,'RVSN' from dual union all
select 39022,'RVSN' from dual union all
select 39023,'RVSN' from dual union all
select 39024,'RVSN' from dual union all
select 39025,'NASA' from dual union all
select 39026,'RVSN' from dual union all
select 39027,'RVSN' from dual union all
select 39028,'RVSN' from dual union all
select 39029,'NASA' from dual union all
select 3903,'USAF' from dual union all
select 39030,'RVSN' from dual union all
select 39031,'RVSN' from dual union all
select 39032,'USAF' from dual union all
select 39033,'RVSN' from dual union all
select 39034,'RVSN' from dual union all
select 39035,'RVSN' from dual union all
select 39036,'RVSN' from dual union all
select 39037,'RVSN' from dual union all
select 39038,'RVSN' from dual union all
select 39039,'SAST' from dual union all
select 3904,'USAF' from dual union all
select 39040,'RVSN' from dual union all
select 39041,'AFSC' from dual union all
select 39042,'RVSN' from dual union all
select 39043,'10ADS' from dual union all
select 39044,'RVSN' from dual union all
select 39045,'RVSN' from dual union all
select 39046,'USAF' from dual union all
select 39047,'RVSN' from dual union all
select 39048,'RVSN' from dual union all
select 39049,'RVSN' from dual union all
select 3905,'USAF' from dual union all
select 39050,'RVSN' from dual union all
select 39051,'RVSN' from dual union all
select 39052,'RVSN' from dual union all
select 39053,'NASA' from dual union all
select 39054,'RVSN' from dual union all
select 39055,'RVSN' from dual union all
select 39056,'RVSN' from dual union all
select 39057,'RVSN' from dual union all
select 39058,'RVSN' from dual union all
select 39059,'RVSN' from dual union all
select 3906,'USAF' from dual union all
select 39060,'RVSN' from dual union all
select 39061,'RVSN' from dual union all
select 39062,'RVSN' from dual union all
select 39063,'RVSN' from dual union all
select 39064,'RVSN' from dual union all
select 39065,'RVSN' from dual union all
select 39066,'RVSN' from dual union all
select 39067,'RVSN' from dual union all
select 39068,'RVSN' from dual union all
select 39069,'CALT' from dual union all
select 3907,'USAF' from dual;
insert into LAUNCH_AGENCY
select 39070,'RVSN' from dual union all
select 39071,'RVSN' from dual union all
select 39072,'RVSN' from dual union all
select 39073,'RVSN' from dual union all
select 39074,'RVSN' from dual union all
select 39075,'RVSN' from dual union all
select 39076,'RVSN' from dual union all
select 39077,'USAF' from dual union all
select 39078,'RVSN' from dual union all
select 39079,'RVSN' from dual union all
select 3908,'ARDC' from dual union all
select 39080,'RVSN' from dual union all
select 39081,'MRN' from dual union all
select 39082,'MRN' from dual union all
select 39083,'DLR' from dual union all
select 39083,'INTA' from dual union all
select 39084,'USAF' from dual union all
select 39085,'USAF' from dual union all
select 39086,'DFVLR' from dual union all
select 39087,'MRN' from dual union all
select 39088,'MRN' from dual union all
select 39089,'MRN' from dual union all
select 3909,'ARDC' from dual union all
select 39090,'PFRR' from dual union all
select 39091,'IAE' from dual union all
select 39092,'NASA' from dual union all
select 39093,'MRN' from dual union all
select 39094,'MRN' from dual union all
select 39095,'MRN' from dual union all
select 39096,'DLR' from dual union all
select 39097,'MRN' from dual union all
select 39098,'MRN' from dual union all
select 39099,'MRN' from dual union all
select 391,'WEHR' from dual union all
select 3910,'ARDC' from dual union all
select 39100,'MRN' from dual union all
select 39101,'MRN' from dual union all
select 39102,'USAF' from dual union all
select 39103,'USAF' from dual union all
select 39104,'AFGL2' from dual union all
select 39105,'MRN' from dual union all
select 39106,'USN' from dual union all
select 39107,'MRN' from dual union all
select 39108,'MRN' from dual union all
select 39109,'MRN' from dual union all
select 3911,'ARDC' from dual union all
select 39110,'MRN' from dual union all
select 39111,'RVSN' from dual union all
select 39112,'MRN' from dual union all
select 39113,'USN' from dual union all
select 39114,'MRN' from dual union all
select 39115,'MRN' from dual union all
select 39116,'MRN' from dual union all
select 39117,'MRN' from dual union all
select 39118,'MRN' from dual union all
select 39119,'MRN' from dual union all
select 3912,'RCAR' from dual union all
select 39120,'MRN' from dual union all
select 39121,'USN' from dual union all
select 39122,'MRN' from dual union all
select 39123,'MRN' from dual union all
select 39124,'NASA' from dual union all
select 39125,'MRN' from dual union all
select 39126,'MRN' from dual union all
select 39127,'MRN' from dual union all
select 39128,'MRN' from dual union all
select 39129,'MRN' from dual union all
select 3913,'ARDC' from dual union all
select 39130,'MRN' from dual union all
select 39131,'RVSN' from dual union all
select 39132,'AFGL' from dual union all
select 39133,'MRN' from dual union all
select 39134,'MRN' from dual union all
select 39135,'MRN' from dual union all
select 39136,'MRN' from dual union all
select 39137,'MRN' from dual union all
select 39138,'MRN' from dual union all
select 39139,'MRN' from dual union all
select 3914,'ARDC' from dual union all
select 39140,'MRN' from dual union all
select 39141,'USN' from dual union all
select 39142,'MRN' from dual union all
select 39143,'NASA' from dual union all
select 39144,'MRN' from dual union all
select 39145,'MRN' from dual union all
select 39146,'MRN' from dual union all
select 39147,'ISRO' from dual union all
select 39148,'MRN' from dual union all
select 39149,'RVSN' from dual union all
select 3915,'ARDC' from dual union all
select 39150,'MRN' from dual union all
select 39151,'USN' from dual union all
select 39152,'USN' from dual union all
select 39153,'MRN' from dual union all
select 39154,'MRN' from dual union all
select 39155,'MRN' from dual union all
select 39156,'RVSN' from dual union all
select 39157,'SAC' from dual union all
select 39158,'SAC' from dual union all
select 39159,'MRN' from dual;
insert into LAUNCH_AGENCY
select 3916,'ARDC' from dual union all
select 39160,'MRN' from dual union all
select 39161,'RVSN' from dual union all
select 39162,'RVSN' from dual union all
select 39163,'SAND' from dual union all
select 39164,'NASA' from dual union all
select 39165,'SAND' from dual union all
select 39166,'SAND' from dual union all
select 39167,'SAND' from dual union all
select 39168,'SAND' from dual union all
select 39169,'WRE' from dual union all
select 3917,'ARDC' from dual union all
select 39170,'ISAS' from dual union all
select 39171,'RVSN' from dual union all
select 39172,'SAST' from dual union all
select 39173,'NASA' from dual union all
select 39174,'DLR' from dual union all
select 39174,'INTA' from dual union all
select 39175,'DLR' from dual union all
select 39175,'INTA' from dual union all
select 39176,'DLR' from dual union all
select 39176,'INTA' from dual union all
select 39177,'DNA' from dual union all
select 39178,'NASA' from dual union all
select 39179,'NRCC' from dual union all
select 3918,'ARDC' from dual union all
select 39180,'ISRO' from dual union all
select 39181,'NASA' from dual union all
select 39182,'CNES' from dual union all
select 39183,'CNES' from dual union all
select 39184,'NASA' from dual union all
select 39185,'CNES' from dual union all
select 39186,'CNES' from dual union all
select 39187,'CNES' from dual union all
select 39188,'CNES' from dual union all
select 39189,'CNES' from dual union all
select 3919,'ARDC' from dual union all
select 39190,'CNES' from dual union all
select 39191,'CNES' from dual union all
select 39192,'CNES' from dual union all
select 39193,'CNES' from dual union all
select 39194,'CNES' from dual union all
select 39195,'CNES' from dual union all
select 39196,'NASA' from dual union all
select 39197,'CNES' from dual union all
select 39198,'NASA' from dual union all
select 39199,'NASA' from dual union all
select 392,'WEHR' from dual union all
select 3920,'ARDC' from dual union all
select 39200,'PRO' from dual union all
select 39201,'AN' from dual union all
select 39202,'AN' from dual union all
select 39203,'AN' from dual union all
select 39204,'AN' from dual union all
select 39205,'9FA3' from dual union all
select 39206,'9FA3' from dual union all
select 39207,'RVSN' from dual union all
select 39208,'RVSN' from dual union all
select 39209,'RVSN' from dual union all
select 3921,'ARDC' from dual union all
select 39210,'NIPR' from dual union all
select 39211,'USN' from dual union all
select 39212,'RVSN' from dual union all
select 39213,'NASA' from dual union all
select 39214,'NASA' from dual union all
select 39215,'SAC' from dual union all
select 39216,'RAE' from dual union all
select 39217,'SSC' from dual union all
select 39218,'AFGL2' from dual union all
select 39218,'DNA' from dual union all
select 39219,'SAC' from dual union all
select 3922,'ARDC' from dual union all
select 39220,'RVSN' from dual union all
select 39221,'AFGL' from dual union all
select 39222,'WRE' from dual union all
select 39223,'AFSC' from dual union all
select 39224,'AN' from dual union all
select 39225,'AFCRL' from dual union all
select 39225,'DNA' from dual union all
select 39226,'NASA' from dual union all
select 39227,'AN' from dual union all
select 39228,'AN' from dual union all
select 39229,'AN' from dual union all
select 3923,'RCAR' from dual union all
select 39230,'NASA' from dual union all
select 39231,'RVSN' from dual union all
select 39232,'SSC' from dual union all
select 39233,'AN' from dual union all
select 39234,'AFGL2' from dual union all
select 39235,'ALCS' from dual union all
select 39235,'SAC' from dual union all
select 39236,'AN' from dual union all
select 39237,'NASA' from dual union all
select 39238,'NASA' from dual union all
select 39239,'AN' from dual union all
select 3924,'ARDC' from dual union all
select 39240,'SSC' from dual union all
select 39241,'AN' from dual union all
select 39242,'RVSN' from dual union all
select 39243,'AN' from dual;
insert into LAUNCH_AGENCY
select 39244,'AN' from dual union all
select 39245,'SAND' from dual union all
select 39245,'USAF' from dual union all
select 39246,'NASA' from dual union all
select 39247,'RVSN' from dual union all
select 39248,'AFSC' from dual union all
select 39249,'AN' from dual union all
select 3925,'ARDC' from dual union all
select 39250,'AN' from dual union all
select 39251,'AN' from dual union all
select 39252,'NASA' from dual union all
select 39253,'RVSN' from dual union all
select 39254,'NASA' from dual union all
select 39255,'NASA' from dual union all
select 39256,'ISRO' from dual union all
select 39257,'ISRO' from dual union all
select 39258,'NASA' from dual union all
select 39259,'AFGL' from dual union all
select 39259,'DNA' from dual union all
select 3926,'ARDC' from dual union all
select 39260,'NASA' from dual union all
select 39261,'SAND' from dual union all
select 39261,'USAF' from dual union all
select 39262,'NASA' from dual union all
select 39263,'NASA' from dual union all
select 39264,'RVSN' from dual union all
select 39265,'NASA' from dual union all
select 39266,'USN' from dual union all
select 39267,'NASA' from dual union all
select 39268,'NSF' from dual union all
select 39269,'NASA' from dual union all
select 3927,'ARDC' from dual union all
select 39270,'DLR' from dual union all
select 39271,'NASA' from dual union all
select 39272,'AFCRL' from dual union all
select 39272,'DNA' from dual union all
select 39273,'ARPA' from dual union all
select 39274,'USAF' from dual union all
select 39275,'RVSN' from dual union all
select 39276,'DMA' from dual union all
select 39277,'ISRO' from dual union all
select 39278,'USN' from dual union all
select 39279,'USN' from dual union all
select 3928,'ARDC' from dual union all
select 39280,'NASA' from dual union all
select 39281,'USN' from dual union all
select 39282,'USN' from dual union all
select 39283,'SAND' from dual union all
select 39283,'USAF' from dual union all
select 39284,'WRE' from dual union all
select 39285,'USA' from dual union all
select 39286,'RVSN' from dual union all
select 39287,'FKG2' from dual union all
select 39288,'FKG2' from dual union all
select 39289,'FKG' from dual union all
select 3929,'ARDC' from dual union all
select 39290,'FKG' from dual union all
select 39291,'AFGL2' from dual union all
select 39292,'DLR' from dual union all
select 39292,'INTA' from dual union all
select 39293,'SRC' from dual union all
select 39294,'SRC' from dual union all
select 39295,'NASA' from dual union all
select 39296,'NASA' from dual union all
select 39297,'RVSN' from dual union all
select 39298,'BAC' from dual union all
select 39299,'NASA' from dual union all
select 393,'WEHR' from dual union all
select 3930,'ARDC' from dual union all
select 39300,'RVSN' from dual union all
select 39301,'RVSN' from dual union all
select 39302,'NASA' from dual union all
select 39303,'AFGL2' from dual union all
select 39304,'NASA' from dual union all
select 39305,'USN' from dual union all
select 39306,'BAC' from dual union all
select 39307,'SAND' from dual union all
select 39308,'RVSN' from dual union all
select 39309,'US7A' from dual union all
select 3931,'ARDC' from dual union all
select 39310,'US7A' from dual union all
select 39311,'US7A' from dual union all
select 39312,'US7A' from dual union all
select 39313,'DMA' from dual union all
select 39314,'SRC' from dual union all
select 39315,'SRC' from dual union all
select 39316,'SAND' from dual union all
select 39317,'SAC' from dual union all
select 39318,'RVSN' from dual union all
select 39319,'RVSN' from dual union all
select 3932,'USA' from dual union all
select 39320,'BAC' from dual union all
select 39321,'NASA' from dual union all
select 39322,'RVSN' from dual union all
select 39323,'NASA' from dual union all
select 39324,'BAC' from dual union all
select 39325,'SAC' from dual union all
select 39326,'SAC' from dual union all
select 39327,'USN' from dual union all
select 39328,'USN' from dual;
insert into LAUNCH_AGENCY
select 39329,'NIPR' from dual union all
select 3933,'USA' from dual union all
select 39330,'VMF' from dual union all
select 39331,'RVSN' from dual union all
select 39332,'USN' from dual union all
select 39333,'USN' from dual union all
select 39334,'USAF' from dual union all
select 39335,'DLR' from dual union all
select 39336,'SAC' from dual union all
select 39337,'RVSN' from dual union all
select 39338,'RVSN' from dual union all
select 39339,'NASA' from dual union all
select 3934,'RCAR' from dual union all
select 39340,'NASA' from dual union all
select 39341,'RVSN' from dual union all
select 39342,'WRE' from dual union all
select 39343,'SAC' from dual union all
select 39344,'USA' from dual union all
select 39345,'RVSN' from dual union all
select 39346,'SRC' from dual union all
select 39347,'NASA' from dual union all
select 39348,'AFSC' from dual union all
select 39349,'BAC' from dual union all
select 3935,'USA' from dual union all
select 39350,'SAND' from dual union all
select 39351,'RVSN' from dual union all
select 39352,'NIPR' from dual union all
select 39353,'NASA' from dual union all
select 39354,'ISRO' from dual union all
select 39355,'USAF' from dual union all
select 39356,'AN' from dual union all
select 39357,'AN' from dual union all
select 39358,'AN' from dual union all
select 39359,'AN' from dual union all
select 3936,'USA' from dual union all
select 39360,'AN' from dual union all
select 39361,'AN' from dual union all
select 39362,'RVSN' from dual union all
select 39363,'AN' from dual union all
select 39364,'AN' from dual union all
select 39365,'AN' from dual union all
select 39366,'AFGL' from dual union all
select 39367,'NASA' from dual union all
select 39368,'NASA' from dual union all
select 39369,'NASA' from dual union all
select 3937,'USA' from dual union all
select 39370,'RVSN' from dual union all
select 39371,'NASA' from dual union all
select 39372,'NIPR' from dual union all
select 39373,'RVSN' from dual union all
select 39374,'ISAS' from dual union all
select 39375,'AN' from dual union all
select 39376,'AN' from dual union all
select 39377,'AFSC' from dual union all
select 39378,'AN' from dual union all
select 39379,'AN' from dual union all
select 3938,'USA' from dual union all
select 39380,'AN' from dual union all
select 39381,'AN' from dual union all
select 39382,'AN' from dual union all
select 39383,'AN' from dual union all
select 39384,'RVSN' from dual union all
select 39385,'AN' from dual union all
select 39386,'AN' from dual union all
select 39387,'AN' from dual union all
select 39388,'AN' from dual union all
select 39389,'AN' from dual union all
select 3939,'USA' from dual union all
select 39390,'USN' from dual union all
select 39391,'USN' from dual union all
select 39392,'AN' from dual union all
select 39393,'ISAS' from dual union all
select 39394,'NASA' from dual union all
select 39395,'RVSN' from dual union all
select 39396,'ALCS' from dual union all
select 39396,'SAC' from dual union all
select 39397,'NASA' from dual union all
select 39398,'USN' from dual union all
select 39399,'USN' from dual union all
select 394,'WEHR' from dual union all
select 3940,'USA' from dual union all
select 39400,'ISAS' from dual union all
select 39401,'AN' from dual union all
select 39402,'AN' from dual union all
select 39403,'AN' from dual union all
select 39404,'AN' from dual union all
select 39405,'AN' from dual union all
select 39406,'RVSN' from dual union all
select 39407,'AN' from dual union all
select 39408,'AN' from dual union all
select 39409,'AN' from dual union all
select 3941,'USA' from dual union all
select 39410,'AFGL' from dual union all
select 39411,'NIPR' from dual union all
select 39412,'USN' from dual union all
select 39413,'USN' from dual union all
select 39414,'RVSN' from dual union all
select 39415,'USN' from dual union all
select 39416,'USN' from dual union all
select 39417,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 39418,'NIPR' from dual union all
select 39419,'ISAS' from dual union all
select 3942,'USA' from dual union all
select 39420,'NASA' from dual union all
select 39421,'ISAS' from dual union all
select 39422,'USN' from dual union all
select 39423,'IAE' from dual union all
select 39424,'NASA' from dual union all
select 39425,'NASA' from dual union all
select 39426,'NASDA' from dual union all
select 39427,'AN' from dual union all
select 39428,'RVSN' from dual union all
select 39429,'AN' from dual union all
select 3943,'USA' from dual union all
select 39430,'AN' from dual union all
select 39431,'ISRO' from dual union all
select 39432,'ISRO' from dual union all
select 39433,'RVSN' from dual union all
select 39434,'NASA' from dual union all
select 39435,'RVSN' from dual union all
select 39436,'SAC' from dual union all
select 39437,'84FA3' from dual union all
select 39438,'84FA3' from dual union all
select 39439,'RVSN' from dual union all
select 3944,'USA' from dual union all
select 39440,'84FA3' from dual union all
select 39441,'84FA3' from dual union all
select 39442,'ISRO' from dual union all
select 39443,'PRO' from dual union all
select 39444,'NASA' from dual union all
select 39445,'NASA' from dual union all
select 39446,'AN' from dual union all
select 39447,'VMF' from dual union all
select 39448,'BAC' from dual union all
select 39449,'ALCS' from dual union all
select 39449,'SAC' from dual union all
select 3945,'RCAR' from dual union all
select 39450,'RVSN' from dual union all
select 39451,'SAC' from dual union all
select 39452,'NRCC' from dual union all
select 39453,'AFSC' from dual union all
select 39454,'ARPA' from dual union all
select 39455,'DNA' from dual union all
select 39456,'NASA' from dual union all
select 39457,'ISRO' from dual union all
select 39458,'DNA' from dual union all
select 39459,'NASA' from dual union all
select 3946,'USA' from dual union all
select 39460,'USN' from dual union all
select 39461,'RVSN' from dual union all
select 39462,'USN' from dual union all
select 39463,'USN' from dual union all
select 39464,'USN' from dual union all
select 39465,'BAC' from dual union all
select 39466,'SRC' from dual union all
select 39467,'USN' from dual union all
select 39468,'USN' from dual union all
select 39469,'ARPA' from dual union all
select 3947,'USA' from dual union all
select 39470,'NASA' from dual union all
select 39471,'NASA' from dual union all
select 39472,'RVSN' from dual union all
select 39473,'SAC' from dual union all
select 39474,'BAC' from dual union all
select 39475,'DMA' from dual union all
select 39476,'AN' from dual union all
select 39477,'AN' from dual union all
select 39478,'BAC' from dual union all
select 39479,'SRC' from dual union all
select 3948,'USA' from dual union all
select 39480,'RVSN' from dual union all
select 39481,'NASA' from dual union all
select 39482,'USAF' from dual union all
select 39483,'RVSN' from dual union all
select 39484,'RVSN' from dual union all
select 39485,'NRL' from dual union all
select 39486,'RVSN' from dual union all
select 39487,'RVSN' from dual union all
select 39488,'I' from dual union all
select 39489,'US7A' from dual union all
select 3949,'USA' from dual union all
select 39490,'US7A' from dual union all
select 39491,'ISAS' from dual union all
select 39492,'PLA2AC' from dual union all
select 39493,'PLA2AC' from dual union all
select 39494,'RVSN' from dual union all
select 39495,'SAND' from dual union all
select 39496,'WRE' from dual union all
select 39497,'WRE' from dual union all
select 39498,'WRE' from dual union all
select 39499,'WRE' from dual union all
select 395,'WER485-2' from dual union all
select 3950,'USA' from dual union all
select 39500,'DNA' from dual union all
select 39501,'RVSN' from dual union all
select 39502,'WRE' from dual union all
select 39503,'RVSN' from dual union all
select 39504,'RVSN' from dual union all
select 39505,'RVSN' from dual union all
select 39506,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 39507,'RVSN' from dual union all
select 39508,'RVSN' from dual union all
select 39509,'RVSN' from dual union all
select 3951,'USA' from dual union all
select 39510,'RVSN' from dual union all
select 39511,'RVSN' from dual union all
select 39512,'RVSN' from dual union all
select 39513,'RVSN' from dual union all
select 39514,'RVSN' from dual union all
select 39515,'RVSN' from dual union all
select 39516,'RVSN' from dual union all
select 39517,'RVSN' from dual union all
select 39518,'RVSN' from dual union all
select 39519,'RVSN' from dual union all
select 3952,'USA' from dual union all
select 39520,'RVSN' from dual union all
select 39521,'RVSN' from dual union all
select 39522,'RVSN' from dual union all
select 39523,'RVSN' from dual union all
select 39524,'RVSN' from dual union all
select 39525,'RVSN' from dual union all
select 39526,'RVSN' from dual union all
select 39527,'RVSN' from dual union all
select 39528,'RVSN' from dual union all
select 39529,'RVSN' from dual union all
select 3953,'USA' from dual union all
select 39530,'RVSN' from dual union all
select 39531,'DLR' from dual union all
select 39531,'INTA' from dual union all
select 39532,'DLR' from dual union all
select 39532,'INTA' from dual union all
select 39533,'INTA' from dual union all
select 39534,'INTA' from dual union all
select 39535,'INTA' from dual union all
select 39536,'INTA' from dual union all
select 39537,'INTA' from dual union all
select 39538,'RVSN' from dual union all
select 39539,'INTA' from dual union all
select 3954,'USA' from dual union all
select 39540,'INTA' from dual union all
select 39541,'INTA' from dual union all
select 39542,'INTA' from dual union all
select 39543,'INTA' from dual union all
select 39544,'INTA' from dual union all
select 39545,'INTA' from dual union all
select 39546,'INTA' from dual union all
select 39547,'INTA' from dual union all
select 39548,'INTA' from dual union all
select 39549,'RVSN' from dual union all
select 3955,'USA' from dual union all
select 39550,'INTA' from dual union all
select 39551,'INTA' from dual union all
select 39552,'DLR' from dual union all
select 39552,'INTA' from dual union all
select 39553,'DLR' from dual union all
select 39553,'INTA' from dual union all
select 39554,'DLR' from dual union all
select 39554,'INTA' from dual union all
select 39555,'DLR' from dual union all
select 39555,'INTA' from dual union all
select 39556,'WRE' from dual union all
select 39557,'WRE' from dual union all
select 39558,'AN' from dual union all
select 39558,'ISRO' from dual union all
select 39559,'WRE' from dual union all
select 3956,'RCAR' from dual union all
select 39560,'RVSN' from dual union all
select 39561,'WRE' from dual union all
select 39562,'WFC' from dual union all
select 39563,'RVSN' from dual union all
select 39564,'RVSN' from dual union all
select 39565,'RVSN' from dual union all
select 39566,'RVSN' from dual union all
select 39567,'RVSN' from dual union all
select 39568,'RVSN' from dual union all
select 39569,'RVSN' from dual union all
select 3957,'USA' from dual union all
select 39570,'RVSN' from dual union all
select 39571,'RVSN' from dual union all
select 39572,'RVSN' from dual union all
select 39573,'RVSN' from dual union all
select 39574,'RVSN' from dual union all
select 39575,'RVSN' from dual union all
select 39576,'RVSN' from dual union all
select 39577,'RVSN' from dual union all
select 39578,'DLR' from dual union all
select 39578,'INTA' from dual union all
select 39579,'RVSN' from dual union all
select 3958,'USA' from dual union all
select 39580,'AN' from dual union all
select 39581,'AN' from dual union all
select 39582,'AN' from dual union all
select 39583,'AN' from dual union all
select 39584,'DLR' from dual union all
select 39584,'INTA' from dual union all
select 39585,'DLR' from dual union all
select 39586,'DFVLR' from dual union all
select 39587,'DLR' from dual union all
select 39587,'INTA' from dual union all
select 39588,'INTA' from dual;
insert into LAUNCH_AGENCY
select 39589,'DLR' from dual union all
select 39589,'INTA' from dual union all
select 3959,'USA' from dual union all
select 39590,'DLR' from dual union all
select 39590,'INTA' from dual union all
select 39591,'AFSC' from dual union all
select 39592,'RVSN' from dual union all
select 39593,'DLR' from dual union all
select 39593,'INTA' from dual union all
select 39594,'NASA' from dual union all
select 39595,'DLR' from dual union all
select 39595,'INTA' from dual union all
select 39596,'DLR' from dual union all
select 39596,'INTA' from dual union all
select 39597,'NASA' from dual union all
select 39598,'ISAS' from dual union all
select 39599,'NASA' from dual union all
select 396,'WEHR' from dual union all
select 3960,'USA' from dual union all
select 39600,'ISAS' from dual union all
select 39601,'USN' from dual union all
select 39602,'USN' from dual union all
select 39603,'AFGL2' from dual union all
select 39604,'DLR' from dual union all
select 39604,'NTNF' from dual union all
select 39605,'DFVLR' from dual union all
select 39606,'INTA' from dual union all
select 39606,'SRC' from dual union all
select 39607,'INTA' from dual union all
select 39607,'SRC' from dual union all
select 39608,'DLR' from dual union all
select 39608,'INTA' from dual union all
select 39609,'DLR' from dual union all
select 39609,'INTA' from dual union all
select 3961,'USA' from dual union all
select 39610,'SSC' from dual union all
select 39611,'AFSC' from dual union all
select 39612,'DLR' from dual union all
select 39612,'INTA' from dual union all
select 39613,'AFGL2' from dual union all
select 39614,'AFGL' from dual union all
select 39615,'USN' from dual union all
select 39616,'USN' from dual union all
select 39617,'NIPR' from dual union all
select 39618,'DLR' from dual union all
select 39618,'INTA' from dual union all
select 39619,'NASDA' from dual union all
select 3962,'USA' from dual union all
select 39620,'DLR' from dual union all
select 39620,'INTA' from dual union all
select 39621,'SAC' from dual union all
select 39622,'RVSN' from dual union all
select 39623,'BAC' from dual union all
select 39624,'AN' from dual union all
select 39625,'AN' from dual union all
select 39626,'NASA' from dual union all
select 39627,'NASA' from dual union all
select 39627,'NTNF' from dual union all
select 39628,'USN' from dual union all
select 39629,'USN' from dual union all
select 3963,'USA' from dual union all
select 39630,'USN' from dual union all
select 39631,'NRCC' from dual union all
select 39632,'DLR' from dual union all
select 39632,'INTA' from dual union all
select 39633,'DLR' from dual union all
select 39633,'INTA' from dual union all
select 39634,'SAC' from dual union all
select 39635,'DLR' from dual union all
select 39635,'INTA' from dual union all
select 39636,'DLR' from dual union all
select 39636,'INTA' from dual union all
select 39637,'RVSN' from dual union all
select 39638,'NPOL' from dual union all
select 39639,'NASA' from dual union all
select 3964,'USA' from dual union all
select 39640,'MRN' from dual union all
select 39641,'MRN' from dual union all
select 39642,'MRN' from dual union all
select 39643,'AN' from dual union all
select 39644,'MRN' from dual union all
select 39645,'MRN' from dual union all
select 39646,'MRN' from dual union all
select 39647,'MRN' from dual union all
select 39648,'MRN' from dual union all
select 39649,'MRN' from dual union all
select 3965,'USA' from dual union all
select 39650,'MRN' from dual union all
select 39651,'MRN' from dual union all
select 39652,'MRN' from dual union all
select 39653,'MRN' from dual union all
select 39654,'MRN' from dual union all
select 39655,'MRN' from dual union all
select 39656,'MRN' from dual union all
select 39657,'MRN' from dual union all
select 39658,'MRN' from dual union all
select 39659,'MRN' from dual union all
select 3966,'USA' from dual union all
select 39660,'MRN' from dual union all
select 39661,'MRN' from dual;
insert into LAUNCH_AGENCY
select 39662,'MRN' from dual union all
select 39663,'MRN' from dual union all
select 39664,'MRN' from dual union all
select 39665,'MRN' from dual union all
select 39666,'MRN' from dual union all
select 39667,'MRN' from dual union all
select 39668,'MRN' from dual union all
select 39669,'MRN' from dual union all
select 3967,'RCAR' from dual union all
select 39670,'MRN' from dual union all
select 39671,'MRN' from dual union all
select 39672,'MRN' from dual union all
select 39673,'MRN' from dual union all
select 39674,'MRN' from dual union all
select 39675,'MRN' from dual union all
select 39676,'MRN' from dual union all
select 39677,'MRN' from dual union all
select 39678,'MRN' from dual union all
select 39679,'MRN' from dual union all
select 3968,'USA' from dual union all
select 39680,'MRN' from dual union all
select 39681,'MRN' from dual union all
select 39682,'MRN' from dual union all
select 39683,'MRN' from dual union all
select 39684,'AN' from dual union all
select 39685,'MRN' from dual union all
select 39686,'MRN' from dual union all
select 39687,'MRN' from dual union all
select 39688,'MRN' from dual union all
select 39689,'MRN' from dual union all
select 3969,'USA' from dual union all
select 39690,'MRN' from dual union all
select 39691,'MRN' from dual union all
select 39692,'MRN' from dual union all
select 39693,'MRN' from dual union all
select 39694,'MRN' from dual union all
select 39695,'MRN' from dual union all
select 39696,'MRN' from dual union all
select 39697,'MRN' from dual union all
select 39698,'MRN' from dual union all
select 39699,'MRN' from dual union all
select 397,'WEHR' from dual union all
select 3970,'USA' from dual union all
select 39700,'MRN' from dual union all
select 39701,'MRN' from dual union all
select 39702,'MRN' from dual union all
select 39703,'MRN' from dual union all
select 39704,'MRN' from dual union all
select 39705,'MRN' from dual union all
select 39706,'MRN' from dual union all
select 39707,'MRN' from dual union all
select 39708,'MRN' from dual union all
select 39709,'MRN' from dual union all
select 3971,'USA' from dual union all
select 39710,'MRN' from dual union all
select 39711,'MRN' from dual union all
select 39712,'MRN' from dual union all
select 39713,'MRN' from dual union all
select 39714,'MRN' from dual union all
select 39715,'MRN' from dual union all
select 39716,'MRN' from dual union all
select 39717,'MRN' from dual union all
select 39718,'AN' from dual union all
select 39719,'MRN' from dual union all
select 3972,'USA' from dual union all
select 39720,'MRN' from dual union all
select 39721,'MRN' from dual union all
select 39722,'MRN' from dual union all
select 39723,'MRN' from dual union all
select 39724,'MRN' from dual union all
select 39725,'MRN' from dual union all
select 39726,'MRN' from dual union all
select 39727,'MRN' from dual union all
select 39728,'MRN' from dual union all
select 39729,'MRN' from dual union all
select 3973,'USA' from dual union all
select 39730,'MRN' from dual union all
select 39731,'MRN' from dual union all
select 39732,'MRN' from dual union all
select 39733,'MRN' from dual union all
select 39734,'MRN' from dual union all
select 39735,'MRN' from dual union all
select 39736,'MRN' from dual union all
select 39737,'MRN' from dual union all
select 39738,'MRN' from dual union all
select 39739,'MRN' from dual union all
select 3974,'USA' from dual union all
select 39740,'MRN' from dual union all
select 39741,'MRN' from dual union all
select 39742,'MRN' from dual union all
select 39743,'MRN' from dual union all
select 39744,'MRN' from dual union all
select 39745,'MRN' from dual union all
select 39746,'MRN' from dual union all
select 39747,'MRN' from dual union all
select 39748,'MRN' from dual union all
select 39749,'MRN' from dual union all
select 3975,'USA' from dual union all
select 39750,'MRN' from dual union all
select 39751,'MRN' from dual;
insert into LAUNCH_AGENCY
select 39752,'MRN' from dual union all
select 39753,'MRN' from dual union all
select 39754,'MRN' from dual union all
select 39755,'MRN' from dual union all
select 39756,'MRN' from dual union all
select 39757,'MRN' from dual union all
select 39758,'MRN' from dual union all
select 39759,'MRN' from dual union all
select 3976,'USA' from dual union all
select 39760,'MRN' from dual union all
select 39761,'MRN' from dual union all
select 39762,'MRN' from dual union all
select 39763,'MRN' from dual union all
select 39764,'MRN' from dual union all
select 39765,'MRN' from dual union all
select 39766,'MRN' from dual union all
select 39767,'MRN' from dual union all
select 39768,'MRN' from dual union all
select 39769,'MRN' from dual union all
select 3977,'USA' from dual union all
select 39770,'MRN' from dual union all
select 39771,'MRN' from dual union all
select 39772,'MRN' from dual union all
select 39773,'MRN' from dual union all
select 39774,'MRN' from dual union all
select 39775,'MRN' from dual union all
select 39776,'MRN' from dual union all
select 39777,'MRN' from dual union all
select 39778,'MRN' from dual union all
select 39779,'MRN' from dual union all
select 3978,'RCAR' from dual union all
select 39780,'MRN' from dual union all
select 39781,'MRN' from dual union all
select 39782,'GMS' from dual union all
select 39783,'MRN' from dual union all
select 39784,'MRN' from dual union all
select 39785,'MRN' from dual union all
select 39786,'MRN' from dual union all
select 39787,'MRN' from dual union all
select 39788,'MRN' from dual union all
select 39789,'MRN' from dual union all
select 3979,'USA' from dual union all
select 39790,'MRN' from dual union all
select 39791,'MRN' from dual union all
select 39792,'MRN' from dual union all
select 39793,'MRN' from dual union all
select 39794,'MRN' from dual union all
select 39795,'MRN' from dual union all
select 39796,'MRN' from dual union all
select 39797,'MRN' from dual union all
select 39798,'MRN' from dual union all
select 39799,'MRN' from dual union all
select 398,'WEHR' from dual union all
select 3980,'USA' from dual union all
select 39800,'MRN' from dual union all
select 39801,'MRN' from dual union all
select 39802,'MRN' from dual union all
select 39803,'MRN' from dual union all
select 39804,'MRN' from dual union all
select 39805,'MRN' from dual union all
select 39806,'MRN' from dual union all
select 39807,'MRN' from dual union all
select 39808,'MRN' from dual union all
select 39809,'MRN' from dual union all
select 3981,'USA' from dual union all
select 39810,'MRN' from dual union all
select 39811,'MRN' from dual union all
select 39812,'MRN' from dual union all
select 39813,'MRN' from dual union all
select 39814,'MRN' from dual union all
select 39815,'MRN' from dual union all
select 39816,'MRN' from dual union all
select 39817,'MRN' from dual union all
select 39818,'MRN' from dual union all
select 39819,'MRN' from dual union all
select 3982,'USA' from dual union all
select 39820,'MRN' from dual union all
select 39821,'MRN' from dual union all
select 39822,'MRN' from dual union all
select 39823,'MRN' from dual union all
select 39824,'MRN' from dual union all
select 39825,'MRN' from dual union all
select 39826,'MRN' from dual union all
select 39827,'MRN' from dual union all
select 39828,'MRN' from dual union all
select 39829,'AN' from dual union all
select 3983,'USA' from dual union all
select 39830,'MRN' from dual union all
select 39831,'MRN' from dual union all
select 39832,'MRN' from dual union all
select 39833,'MRN' from dual union all
select 39834,'MRN' from dual union all
select 39835,'MRN' from dual union all
select 39836,'MRN' from dual union all
select 39837,'MRN' from dual union all
select 39838,'MRN' from dual union all
select 39839,'MRN' from dual union all
select 3984,'USA' from dual union all
select 39840,'MRN' from dual union all
select 39841,'MRN' from dual;
insert into LAUNCH_AGENCY
select 39842,'MRN' from dual union all
select 39843,'MRN' from dual union all
select 39844,'MRN' from dual union all
select 39845,'MRN' from dual union all
select 39846,'MRN' from dual union all
select 39847,'MRN' from dual union all
select 39848,'MRN' from dual union all
select 39849,'MRN' from dual union all
select 3985,'USA' from dual union all
select 39850,'MRN' from dual union all
select 39851,'MRN' from dual union all
select 39852,'MRN' from dual union all
select 39853,'NASA' from dual union all
select 39854,'MRN' from dual union all
select 39855,'MRN' from dual union all
select 39856,'MRN' from dual union all
select 39857,'MRN' from dual union all
select 39858,'MRN' from dual union all
select 39859,'MRN' from dual union all
select 3986,'USA' from dual union all
select 39860,'MRN' from dual union all
select 39861,'NASA' from dual union all
select 39862,'MRN' from dual union all
select 39863,'MRN' from dual union all
select 39864,'MRN' from dual union all
select 39865,'MRN' from dual union all
select 39866,'MRN' from dual union all
select 39867,'MRN' from dual union all
select 39868,'NASA' from dual union all
select 39869,'MRN' from dual union all
select 3987,'USA' from dual union all
select 39870,'MRN' from dual union all
select 39871,'MRN' from dual union all
select 39872,'MRN' from dual union all
select 39873,'NASA' from dual union all
select 39874,'MRN' from dual union all
select 39875,'MRN' from dual union all
select 39876,'MRN' from dual union all
select 39877,'MRN' from dual union all
select 39878,'MRN' from dual union all
select 39879,'MRN' from dual union all
select 3988,'USA' from dual union all
select 39880,'MRN' from dual union all
select 39881,'GMS' from dual union all
select 39882,'MRN' from dual union all
select 39883,'MRN' from dual union all
select 39884,'MRN' from dual union all
select 39885,'MRN' from dual union all
select 39886,'MRN' from dual union all
select 39887,'MRN' from dual union all
select 39888,'MRN' from dual union all
select 39889,'MRN' from dual union all
select 3989,'RCAR' from dual union all
select 39890,'MRN' from dual union all
select 39891,'MRN' from dual union all
select 39892,'MRN' from dual union all
select 39893,'MRN' from dual union all
select 39894,'MRN' from dual union all
select 39895,'MRN' from dual union all
select 39896,'MRN' from dual union all
select 39897,'MRN' from dual union all
select 39898,'MRN' from dual union all
select 39899,'MRN' from dual union all
select 399,'WEHR' from dual union all
select 3990,'USA' from dual union all
select 39900,'MRN' from dual union all
select 39901,'MRN' from dual union all
select 39902,'MRN' from dual union all
select 39903,'MRN' from dual union all
select 39904,'MRN' from dual union all
select 39905,'MRN' from dual union all
select 39906,'MRN' from dual union all
select 39907,'MRN' from dual union all
select 39908,'MRN' from dual union all
select 39909,'MRN' from dual union all
select 3991,'USA' from dual union all
select 39910,'MRN' from dual union all
select 39911,'MRN' from dual union all
select 39912,'MRN' from dual union all
select 39913,'MRN' from dual union all
select 39914,'MRN' from dual union all
select 39915,'MRN' from dual union all
select 39916,'MRN' from dual union all
select 39917,'MRN' from dual union all
select 39918,'MRN' from dual union all
select 39919,'MRN' from dual union all
select 3992,'USA' from dual union all
select 39920,'MRN' from dual union all
select 39921,'MRN' from dual union all
select 39922,'MRN' from dual union all
select 39923,'MRN' from dual union all
select 39924,'MRN' from dual union all
select 39925,'MRN' from dual union all
select 39926,'MRN' from dual union all
select 39927,'MRN' from dual union all
select 39928,'GMS' from dual union all
select 39929,'MRN' from dual union all
select 3993,'USA' from dual union all
select 39930,'MRN' from dual union all
select 39931,'MRN' from dual;
insert into LAUNCH_AGENCY
select 39932,'MRN' from dual union all
select 39933,'MRN' from dual union all
select 39934,'MRN' from dual union all
select 39935,'MRN' from dual union all
select 39936,'MRN' from dual union all
select 39937,'MRN' from dual union all
select 39938,'MRN' from dual union all
select 39939,'MRN' from dual union all
select 3994,'USA' from dual union all
select 39940,'MRN' from dual union all
select 39941,'MRN' from dual union all
select 39942,'MRN' from dual union all
select 39943,'GMS' from dual union all
select 39944,'MRN' from dual union all
select 39945,'MRN' from dual union all
select 39946,'MRN' from dual union all
select 39947,'MRN' from dual union all
select 39948,'MRN' from dual union all
select 39949,'MRN' from dual union all
select 3995,'USA' from dual union all
select 39950,'MRN' from dual union all
select 39951,'MRN' from dual union all
select 39952,'MRN' from dual union all
select 39953,'MRN' from dual union all
select 39954,'MRN' from dual union all
select 39955,'MRN' from dual union all
select 39956,'MRN' from dual union all
select 39957,'GMS' from dual union all
select 39958,'MRN' from dual union all
select 39959,'NASA' from dual union all
select 3996,'USA' from dual union all
select 39960,'MRN' from dual union all
select 39961,'MRN' from dual union all
select 39962,'MRN' from dual union all
select 39963,'NASA' from dual union all
select 39964,'MRN' from dual union all
select 39965,'MRN' from dual union all
select 39966,'MRN' from dual union all
select 39967,'MRN' from dual union all
select 39968,'MRN' from dual union all
select 39969,'MRN' from dual union all
select 3997,'USA' from dual union all
select 39970,'MRN' from dual union all
select 39971,'MRN' from dual union all
select 39972,'MRN' from dual union all
select 39973,'MRN' from dual union all
select 39974,'MRN' from dual union all
select 39975,'MRN' from dual union all
select 39976,'MRN' from dual union all
select 39977,'MRN' from dual union all
select 39978,'MRN' from dual union all
select 39979,'MRN' from dual union all
select 3998,'USA' from dual union all
select 39980,'GMS' from dual union all
select 39981,'MRN' from dual union all
select 39982,'MRN' from dual union all
select 39983,'MRN' from dual union all
select 39984,'MRN' from dual union all
select 39985,'MRN' from dual union all
select 39986,'MRN' from dual union all
select 39987,'MRN' from dual union all
select 39988,'MRN' from dual union all
select 39989,'MRN' from dual union all
select 3999,'USA' from dual union all
select 39990,'MRN' from dual union all
select 39991,'MRN' from dual union all
select 39992,'MRN' from dual union all
select 39993,'MRN' from dual union all
select 39994,'MRN' from dual union all
select 39995,'MRN' from dual union all
select 39996,'MRN' from dual union all
select 39997,'MRN' from dual union all
select 39998,'MRN' from dual union all
select 39999,'MRN' from dual union all
select 4,'WEHR' from dual union all
select 40,'WEHR' from dual union all
select 400,'WEHR' from dual union all
select 4000,'PARD' from dual union all
select 40000,'MRN' from dual union all
select 40001,'MRN' from dual union all
select 40002,'MRN' from dual union all
select 40003,'MRN' from dual union all
select 40004,'MRN' from dual union all
select 40005,'MRN' from dual union all
select 40006,'MRN' from dual union all
select 40007,'MRN' from dual union all
select 40008,'MRN' from dual union all
select 40009,'MRN' from dual union all
select 4001,'USA' from dual union all
select 40010,'MRN' from dual union all
select 40011,'MRN' from dual union all
select 40012,'MRN' from dual union all
select 40013,'MRN' from dual union all
select 40014,'MRN' from dual union all
select 40015,'MRN' from dual union all
select 40016,'MRN' from dual union all
select 40017,'GMS' from dual union all
select 40018,'MRN' from dual union all
select 40019,'NASA' from dual union all
select 4002,'USA' from dual;
insert into LAUNCH_AGENCY
select 40020,'MRN' from dual union all
select 40021,'MRN' from dual union all
select 40022,'MRN' from dual union all
select 40023,'MRN' from dual union all
select 40024,'MRN' from dual union all
select 40025,'MRN' from dual union all
select 40026,'NASA' from dual union all
select 40027,'MRN' from dual union all
select 40028,'MRN' from dual union all
select 40029,'MRN' from dual union all
select 4003,'USA' from dual union all
select 40030,'MRN' from dual union all
select 40031,'MRN' from dual union all
select 40032,'MRN' from dual union all
select 40033,'MRN' from dual union all
select 40034,'NASDA' from dual union all
select 40035,'MRN' from dual union all
select 40036,'MRN' from dual union all
select 40037,'MRN' from dual union all
select 40038,'MRN' from dual union all
select 40039,'MRN' from dual union all
select 4004,'USA' from dual union all
select 40040,'MRN' from dual union all
select 40041,'MRN' from dual union all
select 40042,'MRN' from dual union all
select 40043,'MRN' from dual union all
select 40044,'MRN' from dual union all
select 40045,'MRN' from dual union all
select 40046,'MRN' from dual union all
select 40047,'MRN' from dual union all
select 40048,'MRN' from dual union all
select 40049,'MRN' from dual union all
select 4005,'USA' from dual union all
select 40050,'MRN' from dual union all
select 40051,'MRN' from dual union all
select 40052,'NASDA' from dual union all
select 40053,'MRN' from dual union all
select 40054,'MRN' from dual union all
select 40055,'MRN' from dual union all
select 40056,'MRN' from dual union all
select 40057,'MRN' from dual union all
select 40058,'MRN' from dual union all
select 40059,'MRN' from dual union all
select 4006,'USA' from dual union all
select 40060,'MRN' from dual union all
select 40061,'MRN' from dual union all
select 40062,'MRN' from dual union all
select 40063,'MRN' from dual union all
select 40064,'MRN' from dual union all
select 40065,'MRN' from dual union all
select 40066,'MRN' from dual union all
select 40067,'MRN' from dual union all
select 40068,'MRN' from dual union all
select 40069,'MRN' from dual union all
select 4007,'USA' from dual union all
select 40070,'NASA' from dual union all
select 40071,'MRN' from dual union all
select 40072,'MRN' from dual union all
select 40073,'MRN' from dual union all
select 40074,'MRN' from dual union all
select 40075,'GMS' from dual union all
select 40076,'MRN' from dual union all
select 40077,'MRN' from dual union all
select 40078,'MRN' from dual union all
select 40079,'MRN' from dual union all
select 4008,'USA' from dual union all
select 40080,'MRN' from dual union all
select 40081,'MRN' from dual union all
select 40082,'MRN' from dual union all
select 40083,'MRN' from dual union all
select 40084,'AN' from dual union all
select 40085,'MRN' from dual union all
select 40086,'MRN' from dual union all
select 40087,'MRN' from dual union all
select 40088,'MRN' from dual union all
select 40089,'MRN' from dual union all
select 4009,'USA' from dual union all
select 40090,'MRN' from dual union all
select 40091,'MRN' from dual union all
select 40092,'MRN' from dual union all
select 40093,'MRN' from dual union all
select 40094,'MRN' from dual union all
select 40095,'MRN' from dual union all
select 40096,'MRN' from dual union all
select 40097,'MRN' from dual union all
select 40098,'MRN' from dual union all
select 40099,'MRN' from dual union all
select 401,'WEHR' from dual union all
select 4010,'USA' from dual union all
select 40100,'MRN' from dual union all
select 40101,'MRN' from dual union all
select 40102,'MRN' from dual union all
select 40103,'MRN' from dual union all
select 40104,'MRN' from dual union all
select 40105,'MRN' from dual union all
select 40106,'MRN' from dual union all
select 40107,'MRN' from dual union all
select 40108,'MRN' from dual union all
select 40109,'NASA' from dual union all
select 4011,'USAF' from dual;
insert into LAUNCH_AGENCY
select 40110,'MRN' from dual union all
select 40111,'MRN' from dual union all
select 40112,'MRN' from dual union all
select 40113,'MRN' from dual union all
select 40114,'MRN' from dual union all
select 40115,'MRN' from dual union all
select 40116,'MRN' from dual union all
select 40117,'MRN' from dual union all
select 40118,'MRN' from dual union all
select 40119,'MRN' from dual union all
select 4012,'USA' from dual union all
select 40120,'MRN' from dual union all
select 40121,'MRN' from dual union all
select 40122,'MRN' from dual union all
select 40123,'GMS' from dual union all
select 40124,'MRN' from dual union all
select 40125,'MRN' from dual union all
select 40126,'MRN' from dual union all
select 40127,'MRN' from dual union all
select 40128,'AN' from dual union all
select 40129,'AN' from dual union all
select 4013,'USA' from dual union all
select 40130,'MRN' from dual union all
select 40131,'MRN' from dual union all
select 40132,'MRN' from dual union all
select 40133,'MRN' from dual union all
select 40134,'MRN' from dual union all
select 40135,'MRN' from dual union all
select 40136,'MRN' from dual union all
select 40137,'MRN' from dual union all
select 40138,'MRN' from dual union all
select 40139,'MRN' from dual union all
select 4014,'USA' from dual union all
select 40140,'MRN' from dual union all
select 40141,'MRN' from dual union all
select 40142,'MRN' from dual union all
select 40143,'MRN' from dual union all
select 40144,'MRN' from dual union all
select 40145,'MRN' from dual union all
select 40146,'MRN' from dual union all
select 40147,'MRN' from dual union all
select 40148,'MRN' from dual union all
select 40149,'MRN' from dual union all
select 4015,'USA' from dual union all
select 40150,'MRN' from dual union all
select 40151,'MRN' from dual union all
select 40152,'MRN' from dual union all
select 40153,'MRN' from dual union all
select 40154,'MRN' from dual union all
select 40155,'MRN' from dual union all
select 40156,'MRN' from dual union all
select 40157,'GMS' from dual union all
select 40158,'MRN' from dual union all
select 40159,'MRN' from dual union all
select 4016,'USA' from dual union all
select 40160,'MRN' from dual union all
select 40161,'MRN' from dual union all
select 40162,'MRN' from dual union all
select 40163,'MRN' from dual union all
select 40164,'MRN' from dual union all
select 40165,'MRN' from dual union all
select 40166,'MRN' from dual union all
select 40167,'MRN' from dual union all
select 40168,'MRN' from dual union all
select 40169,'MRN' from dual union all
select 4017,'USA' from dual union all
select 40170,'MRN' from dual union all
select 40171,'MRN' from dual union all
select 40172,'MRN' from dual union all
select 40173,'MRN' from dual union all
select 40174,'MRN' from dual union all
select 40175,'MRN' from dual union all
select 40176,'MRN' from dual union all
select 40177,'MRN' from dual union all
select 40178,'MRN' from dual union all
select 40179,'MRN' from dual union all
select 4018,'USA' from dual union all
select 40180,'MRN' from dual union all
select 40181,'MRN' from dual union all
select 40182,'MRN' from dual union all
select 40183,'MRN' from dual union all
select 40184,'MRN' from dual union all
select 40185,'MRN' from dual union all
select 40186,'GMS' from dual union all
select 40187,'MRN' from dual union all
select 40188,'MRN' from dual union all
select 40189,'MRN' from dual union all
select 4019,'USA' from dual union all
select 40190,'MRN' from dual union all
select 40191,'MRN' from dual union all
select 40192,'MRN' from dual union all
select 40193,'MRN' from dual union all
select 40194,'GMS' from dual union all
select 40195,'MRN' from dual union all
select 40196,'AN' from dual union all
select 40197,'MRN' from dual union all
select 40198,'MRN' from dual union all
select 40199,'MRN' from dual union all
select 402,'WEHR' from dual union all
select 4020,'USA' from dual;
insert into LAUNCH_AGENCY
select 40200,'MRN' from dual union all
select 40201,'MRN' from dual union all
select 40202,'MRN' from dual union all
select 40203,'MRN' from dual union all
select 40204,'MRN' from dual union all
select 40205,'MRN' from dual union all
select 40206,'MRN' from dual union all
select 40207,'MRN' from dual union all
select 40208,'MRN' from dual union all
select 40209,'MRN' from dual union all
select 4021,'USA' from dual union all
select 40210,'MRN' from dual union all
select 40211,'NASA' from dual union all
select 40212,'MRN' from dual union all
select 40213,'MRN' from dual union all
select 40214,'MRN' from dual union all
select 40215,'MRN' from dual union all
select 40216,'MRN' from dual union all
select 40217,'MRN' from dual union all
select 40218,'MRN' from dual union all
select 40219,'MRN' from dual union all
select 4022,'USAF' from dual union all
select 40220,'MRN' from dual union all
select 40221,'MRN' from dual union all
select 40222,'MRN' from dual union all
select 40223,'AN' from dual union all
select 40224,'MRN' from dual union all
select 40225,'MRN' from dual union all
select 40226,'MRN' from dual union all
select 40227,'MRN' from dual union all
select 40228,'MRN' from dual union all
select 40229,'MRN' from dual union all
select 4023,'USA' from dual union all
select 40230,'MRN' from dual union all
select 40231,'MRN' from dual union all
select 40232,'MRN' from dual union all
select 40233,'MRN' from dual union all
select 40234,'MRN' from dual union all
select 40235,'MRN' from dual union all
select 40236,'MRN' from dual union all
select 40237,'MRN' from dual union all
select 40238,'MRN' from dual union all
select 40239,'MRN' from dual union all
select 4024,'WADD' from dual union all
select 40240,'MRN' from dual union all
select 40241,'MRN' from dual union all
select 40242,'MRN' from dual union all
select 40243,'MRN' from dual union all
select 40244,'MRN' from dual union all
select 40245,'MRN' from dual union all
select 40246,'MRN' from dual union all
select 40247,'AN' from dual union all
select 40248,'MRN' from dual union all
select 40249,'MRN' from dual union all
select 4025,'WADD' from dual union all
select 40250,'MRN' from dual union all
select 40251,'MRN' from dual union all
select 40252,'MRN' from dual union all
select 40253,'MRN' from dual union all
select 40254,'MRN' from dual union all
select 40255,'MRN' from dual union all
select 40256,'MRN' from dual union all
select 40257,'MRN' from dual union all
select 40258,'MRN' from dual union all
select 40259,'MRN' from dual union all
select 4026,'WADD' from dual union all
select 40260,'MRN' from dual union all
select 40261,'MRN' from dual union all
select 40262,'MRN' from dual union all
select 40263,'MRN' from dual union all
select 40264,'MRN' from dual union all
select 40265,'MRN' from dual union all
select 40266,'MRN' from dual union all
select 40267,'MRN' from dual union all
select 40268,'MRN' from dual union all
select 40269,'MRN' from dual union all
select 4027,'WADD' from dual union all
select 40270,'MRN' from dual union all
select 40271,'MRN' from dual union all
select 40272,'MRN' from dual union all
select 40273,'MRN' from dual union all
select 40274,'MRN' from dual union all
select 40275,'MRN' from dual union all
select 40276,'MRN' from dual union all
select 40277,'MRN' from dual union all
select 40278,'MRN' from dual union all
select 40279,'MRN' from dual union all
select 4028,'WADD' from dual union all
select 40280,'MRN' from dual union all
select 40281,'MRN' from dual union all
select 40282,'MRN' from dual union all
select 40283,'MRN' from dual union all
select 40284,'MRN' from dual union all
select 40285,'MRN' from dual union all
select 40286,'MRN' from dual union all
select 40287,'MRN' from dual union all
select 40288,'MRN' from dual union all
select 40289,'MRN' from dual union all
select 4029,'WADD' from dual union all
select 40290,'MRN' from dual;
insert into LAUNCH_AGENCY
select 40291,'MRN' from dual union all
select 40292,'MRN' from dual union all
select 40293,'MRN' from dual union all
select 40294,'MRN' from dual union all
select 40295,'MRN' from dual union all
select 40296,'MRN' from dual union all
select 40297,'MRN' from dual union all
select 40298,'MRN' from dual union all
select 40299,'MRN' from dual union all
select 403,'WEHR' from dual union all
select 4030,'WADD' from dual union all
select 40300,'MRN' from dual union all
select 40301,'MRN' from dual union all
select 40302,'MRN' from dual union all
select 40303,'MRN' from dual union all
select 40304,'MRN' from dual union all
select 40305,'MRN' from dual union all
select 40306,'MRN' from dual union all
select 40307,'MRN' from dual union all
select 40308,'MRN' from dual union all
select 40309,'MRN' from dual union all
select 4031,'WADD' from dual union all
select 40310,'MRN' from dual union all
select 40311,'MRN' from dual union all
select 40312,'MRN' from dual union all
select 40313,'MRN' from dual union all
select 40314,'MRN' from dual union all
select 40315,'MRN' from dual union all
select 40316,'MRN' from dual union all
select 40317,'MRN' from dual union all
select 40318,'MRN' from dual union all
select 40319,'MRN' from dual union all
select 4032,'WADD' from dual union all
select 40320,'MRN' from dual union all
select 40321,'MRN' from dual union all
select 40322,'MRN' from dual union all
select 40323,'MRN' from dual union all
select 40324,'MRN' from dual union all
select 40325,'MRN' from dual union all
select 40326,'MRN' from dual union all
select 40327,'MRN' from dual union all
select 40328,'MRN' from dual union all
select 40329,'NASA' from dual union all
select 4033,'USAF' from dual union all
select 40330,'MRN' from dual union all
select 40331,'MRN' from dual union all
select 40332,'MRN' from dual union all
select 40333,'MRN' from dual union all
select 40334,'MRN' from dual union all
select 40335,'MRN' from dual union all
select 40336,'MRN' from dual union all
select 40337,'MRN' from dual union all
select 40338,'MRN' from dual union all
select 40339,'MRN' from dual union all
select 4034,'WADD' from dual union all
select 40340,'MRN' from dual union all
select 40341,'MRN' from dual union all
select 40342,'MRN' from dual union all
select 40343,'MRN' from dual union all
select 40344,'MRN' from dual union all
select 40345,'NASA' from dual union all
select 40346,'MRN' from dual union all
select 40347,'NASA' from dual union all
select 40348,'MRN' from dual union all
select 40349,'MRN' from dual union all
select 4035,'WADD' from dual union all
select 40350,'MRN' from dual union all
select 40351,'MRN' from dual union all
select 40352,'MRN' from dual union all
select 40353,'MRN' from dual union all
select 40354,'MRN' from dual union all
select 40355,'MRN' from dual union all
select 40356,'MRN' from dual union all
select 40357,'MRN' from dual union all
select 40358,'MRN' from dual union all
select 40359,'MRN' from dual union all
select 4036,'WADD' from dual union all
select 40360,'MRN' from dual union all
select 40361,'MRN' from dual union all
select 40362,'GMS' from dual union all
select 40363,'MRN' from dual union all
select 40364,'MRN' from dual union all
select 40365,'MRN' from dual union all
select 40366,'MRN' from dual union all
select 40367,'MRN' from dual union all
select 40368,'MRN' from dual union all
select 40369,'MRN' from dual union all
select 4037,'WADD' from dual union all
select 40370,'MRN' from dual union all
select 40371,'MRN' from dual union all
select 40372,'MRN' from dual union all
select 40373,'MRN' from dual union all
select 40374,'MRN' from dual union all
select 40375,'MRN' from dual union all
select 40376,'MRN' from dual union all
select 40377,'MRN' from dual union all
select 40378,'NASA' from dual union all
select 40379,'NASA' from dual union all
select 4038,'WADD' from dual union all
select 40380,'MRN' from dual;
insert into LAUNCH_AGENCY
select 40381,'MRN' from dual union all
select 40382,'MRN' from dual union all
select 40383,'MRN' from dual union all
select 40384,'MRN' from dual union all
select 40385,'MRN' from dual union all
select 40386,'MRN' from dual union all
select 40387,'MRN' from dual union all
select 40388,'GMS' from dual union all
select 40389,'MRN' from dual union all
select 4039,'WADD' from dual union all
select 40390,'MRN' from dual union all
select 40391,'MRN' from dual union all
select 40392,'MRN' from dual union all
select 40393,'MRN' from dual union all
select 40394,'MRN' from dual union all
select 40395,'MRN' from dual union all
select 40396,'MRN' from dual union all
select 40397,'MRN' from dual union all
select 40398,'MRN' from dual union all
select 40399,'MRN' from dual union all
select 404,'WEHR' from dual union all
select 4040,'ARDC' from dual union all
select 40400,'MRN' from dual union all
select 40401,'MRN' from dual union all
select 40402,'MRN' from dual union all
select 40403,'MRN' from dual union all
select 40404,'MRN' from dual union all
select 40405,'MRN' from dual union all
select 40406,'MRN' from dual union all
select 40407,'MRN' from dual union all
select 40408,'MRN' from dual union all
select 40409,'MRN' from dual union all
select 4041,'ARDC' from dual union all
select 40410,'MRN' from dual union all
select 40411,'MRN' from dual union all
select 40412,'MRN' from dual union all
select 40413,'MRN' from dual union all
select 40414,'MRN' from dual union all
select 40415,'MRN' from dual union all
select 40416,'MRN' from dual union all
select 40417,'MRN' from dual union all
select 40418,'MRN' from dual union all
select 40419,'MRN' from dual union all
select 4042,'ARDC' from dual union all
select 40420,'MRN' from dual union all
select 40421,'MRN' from dual union all
select 40422,'MRN' from dual union all
select 40423,'MRN' from dual union all
select 40424,'MRN' from dual union all
select 40425,'MRN' from dual union all
select 40426,'MRN' from dual union all
select 40427,'MRN' from dual union all
select 40428,'MRN' from dual union all
select 40429,'MRN' from dual union all
select 4043,'SCEL' from dual union all
select 40430,'AN' from dual union all
select 40431,'MRN' from dual union all
select 40432,'MRN' from dual union all
select 40433,'AN' from dual union all
select 40434,'MRN' from dual union all
select 40435,'MRN' from dual union all
select 40436,'MRN' from dual union all
select 40437,'MRN' from dual union all
select 40438,'MRN' from dual union all
select 40439,'MRN' from dual union all
select 4044,'USAF' from dual union all
select 40440,'MRN' from dual union all
select 40441,'MRN' from dual union all
select 40442,'MRN' from dual union all
select 40443,'MRN' from dual union all
select 40444,'MRN' from dual union all
select 40445,'MRN' from dual union all
select 40446,'MRN' from dual union all
select 40447,'MRN' from dual union all
select 40448,'MRN' from dual union all
select 40449,'MRN' from dual union all
select 4045,'SCEL' from dual union all
select 40450,'MRN' from dual union all
select 40451,'MRN' from dual union all
select 40452,'MRN' from dual union all
select 40453,'MRN' from dual union all
select 40454,'MRN' from dual union all
select 40455,'MRN' from dual union all
select 40456,'MRN' from dual union all
select 40457,'MRN' from dual union all
select 40458,'MRN' from dual union all
select 40459,'MRN' from dual union all
select 4046,'SCEL' from dual union all
select 40460,'MRN' from dual union all
select 40461,'MRN' from dual union all
select 40462,'MRN' from dual union all
select 40463,'MRN' from dual union all
select 40464,'MRN' from dual union all
select 40465,'MRN' from dual union all
select 40466,'AN' from dual union all
select 40467,'MRN' from dual union all
select 40468,'MRN' from dual union all
select 40469,'MRN' from dual union all
select 4047,'USA' from dual union all
select 40470,'AN' from dual;
insert into LAUNCH_AGENCY
select 40471,'NASA' from dual union all
select 40472,'MRN' from dual union all
select 40473,'MRN' from dual union all
select 40474,'MRN' from dual union all
select 40475,'MRN' from dual union all
select 40476,'MRN' from dual union all
select 40477,'MRN' from dual union all
select 40478,'MRN' from dual union all
select 40479,'MRN' from dual union all
select 4048,'UNRV' from dual union all
select 40480,'MRN' from dual union all
select 40481,'MRN' from dual union all
select 40482,'MRN' from dual union all
select 40483,'MRN' from dual union all
select 40484,'NASA' from dual union all
select 40485,'MRN' from dual union all
select 40486,'NASA' from dual union all
select 40487,'MRN' from dual union all
select 40488,'MRN' from dual union all
select 40489,'MRN' from dual union all
select 4049,'UNRV' from dual union all
select 40490,'MRN' from dual union all
select 40491,'MRN' from dual union all
select 40492,'MRN' from dual union all
select 40493,'MRN' from dual union all
select 40494,'MRN' from dual union all
select 40495,'MRN' from dual union all
select 40496,'MRN' from dual union all
select 40497,'MRN' from dual union all
select 40498,'MRN' from dual union all
select 40499,'MRN' from dual union all
select 405,'WEHR' from dual union all
select 4050,'UNRV' from dual union all
select 40500,'MRN' from dual union all
select 40501,'MRN' from dual union all
select 40502,'MRN' from dual union all
select 40503,'MRN' from dual union all
select 40504,'MRN' from dual union all
select 40505,'MRN' from dual union all
select 40506,'MRN' from dual union all
select 40507,'MRN' from dual union all
select 40508,'MRN' from dual union all
select 40509,'MRN' from dual union all
select 4051,'UNRV' from dual union all
select 40510,'MRN' from dual union all
select 40511,'MRN' from dual union all
select 40512,'MRN' from dual union all
select 40513,'MRN' from dual union all
select 40514,'MRN' from dual union all
select 40515,'NASA' from dual union all
select 40516,'MRN' from dual union all
select 40517,'MRN' from dual union all
select 40518,'MRN' from dual union all
select 40519,'MRN' from dual union all
select 4052,'UNRV' from dual union all
select 40520,'MRN' from dual union all
select 40521,'MRN' from dual union all
select 40522,'MRN' from dual union all
select 40523,'MRN' from dual union all
select 40524,'MRN' from dual union all
select 40525,'MRN' from dual union all
select 40526,'MRN' from dual union all
select 40527,'MRN' from dual union all
select 40528,'MRN' from dual union all
select 40529,'MRN' from dual union all
select 4053,'UNRV' from dual union all
select 40530,'MRN' from dual union all
select 40531,'MRN' from dual union all
select 40532,'MRN' from dual union all
select 40533,'MRN' from dual union all
select 40534,'MRN' from dual union all
select 40535,'MRN' from dual union all
select 40536,'AN' from dual union all
select 40537,'MRN' from dual union all
select 40538,'MRN' from dual union all
select 40539,'MRN' from dual union all
select 4054,'UNRV' from dual union all
select 40540,'MRN' from dual union all
select 40541,'MRN' from dual union all
select 40542,'MRN' from dual union all
select 40543,'MRN' from dual union all
select 40544,'MRN' from dual union all
select 40545,'MRN' from dual union all
select 40546,'MRN' from dual union all
select 40547,'MRN' from dual union all
select 40548,'MRN' from dual union all
select 40549,'MRN' from dual union all
select 4055,'USAF' from dual union all
select 40550,'MRN' from dual union all
select 40551,'MRN' from dual union all
select 40552,'MRN' from dual union all
select 40553,'MRN' from dual union all
select 40554,'MRN' from dual union all
select 40555,'MRN' from dual union all
select 40556,'MRN' from dual union all
select 40557,'MRN' from dual union all
select 40558,'MRN' from dual union all
select 40559,'MRN' from dual union all
select 4056,'UNRV' from dual union all
select 40560,'MRN' from dual;
insert into LAUNCH_AGENCY
select 40561,'MRN' from dual union all
select 40562,'MRN' from dual union all
select 40563,'MRN' from dual union all
select 40564,'MRN' from dual union all
select 40565,'MRN' from dual union all
select 40566,'MRN' from dual union all
select 40567,'MRN' from dual union all
select 40568,'MRN' from dual union all
select 40569,'MRN' from dual union all
select 4057,'UNRV' from dual union all
select 40570,'MRN' from dual union all
select 40571,'MRN' from dual union all
select 40572,'MRN' from dual union all
select 40573,'MRN' from dual union all
select 40574,'MRN' from dual union all
select 40575,'MRN' from dual union all
select 40576,'MRN' from dual union all
select 40577,'MRN' from dual union all
select 40578,'MRN' from dual union all
select 40579,'MRN' from dual union all
select 4058,'UNRV' from dual union all
select 40580,'MRN' from dual union all
select 40581,'MRN' from dual union all
select 40582,'MRN' from dual union all
select 40583,'MRN' from dual union all
select 40584,'MRN' from dual union all
select 40585,'MRN' from dual union all
select 40586,'MRN' from dual union all
select 40587,'MRN' from dual union all
select 40588,'MRN' from dual union all
select 40589,'MRN' from dual union all
select 4059,'UNRV' from dual union all
select 40590,'MRN' from dual union all
select 40591,'MRN' from dual union all
select 40592,'MRN' from dual union all
select 40593,'MRN' from dual union all
select 40594,'MRN' from dual union all
select 40595,'MRN' from dual union all
select 40596,'MRN' from dual union all
select 40597,'MRN' from dual union all
select 40598,'GMS' from dual union all
select 40599,'MRN' from dual union all
select 406,'WER444' from dual union all
select 4060,'USAF' from dual union all
select 40600,'MRN' from dual union all
select 40601,'MRN' from dual union all
select 40602,'MRN' from dual union all
select 40603,'MRN' from dual union all
select 40604,'MRN' from dual union all
select 40605,'MRN' from dual union all
select 40606,'MRN' from dual union all
select 40607,'MRN' from dual union all
select 40608,'NASA' from dual union all
select 40609,'MRN' from dual union all
select 4061,'USAF' from dual union all
select 40610,'MRN' from dual union all
select 40611,'MRN' from dual union all
select 40612,'MRN' from dual union all
select 40613,'MRN' from dual union all
select 40614,'MRN' from dual union all
select 40615,'MRN' from dual union all
select 40616,'MRN' from dual union all
select 40617,'MRN' from dual union all
select 40618,'MRN' from dual union all
select 40619,'MRN' from dual union all
select 4062,'USAF' from dual union all
select 40620,'MRN' from dual union all
select 40621,'MRN' from dual union all
select 40622,'MRN' from dual union all
select 40623,'MRN' from dual union all
select 40624,'MRN' from dual union all
select 40625,'MRN' from dual union all
select 40626,'MRN' from dual union all
select 40627,'MRN' from dual union all
select 40628,'AN' from dual union all
select 40629,'MRN' from dual union all
select 4063,'USAF' from dual union all
select 40630,'MRN' from dual union all
select 40631,'MRN' from dual union all
select 40632,'MRN' from dual union all
select 40633,'MRN' from dual union all
select 40634,'MRN' from dual union all
select 40635,'MRN' from dual union all
select 40636,'MRN' from dual union all
select 40637,'MRN' from dual union all
select 40638,'MRN' from dual union all
select 40639,'MRN' from dual union all
select 4064,'USAF' from dual union all
select 40640,'MRN' from dual union all
select 40641,'MRN' from dual union all
select 40642,'MRN' from dual union all
select 40643,'MRN' from dual union all
select 40644,'GMS' from dual union all
select 40645,'GMS' from dual union all
select 40646,'MRN' from dual union all
select 40647,'NASA' from dual union all
select 40648,'MRN' from dual union all
select 40649,'MRN' from dual union all
select 4065,'USAF' from dual union all
select 40650,'NASA' from dual;
insert into LAUNCH_AGENCY
select 40651,'NASA' from dual union all
select 40652,'INTA' from dual union all
select 40653,'NASA' from dual union all
select 40654,'AN' from dual union all
select 40655,'MRN' from dual union all
select 40656,'NASA' from dual union all
select 40657,'MRN' from dual union all
select 40658,'MRN' from dual union all
select 40659,'MRN' from dual union all
select 4066,'USAF' from dual union all
select 40660,'MRN' from dual union all
select 40661,'MRN' from dual union all
select 40662,'MRN' from dual union all
select 40663,'MRN' from dual union all
select 40664,'MRN' from dual union all
select 40665,'MRN' from dual union all
select 40666,'MRN' from dual union all
select 40667,'MRN' from dual union all
select 40668,'MRN' from dual union all
select 40669,'MRN' from dual union all
select 4067,'11TMS' from dual union all
select 40670,'MRN' from dual union all
select 40671,'GMS' from dual union all
select 40672,'MRN' from dual union all
select 40673,'MRN' from dual union all
select 40674,'MRN' from dual union all
select 40675,'MRN' from dual union all
select 40676,'MRN' from dual union all
select 40677,'MRN' from dual union all
select 40678,'MRN' from dual union all
select 40679,'MRN' from dual union all
select 4068,'PARD' from dual union all
select 40680,'MRN' from dual union all
select 40681,'MRN' from dual union all
select 40682,'MRN' from dual union all
select 40683,'MRN' from dual union all
select 40684,'MRN' from dual union all
select 40685,'GMS' from dual union all
select 40686,'MRN' from dual union all
select 40687,'MRN' from dual union all
select 40688,'MRN' from dual union all
select 40689,'MRN' from dual union all
select 4069,'USAF' from dual union all
select 40690,'MRN' from dual union all
select 40691,'MRN' from dual union all
select 40692,'MRN' from dual union all
select 40693,'MRN' from dual union all
select 40694,'MRN' from dual union all
select 40695,'MRN' from dual union all
select 40696,'GMS' from dual union all
select 40697,'MRN' from dual union all
select 40698,'MRN' from dual union all
select 40699,'MRN' from dual union all
select 407,'WEHR' from dual union all
select 4070,'USN' from dual union all
select 40700,'MRN' from dual union all
select 40701,'MRN' from dual union all
select 40702,'MRN' from dual union all
select 40703,'MRN' from dual union all
select 40704,'MRN' from dual union all
select 40705,'MRN' from dual union all
select 40706,'MRN' from dual union all
select 40707,'GMS' from dual union all
select 40708,'MRN' from dual union all
select 40709,'MRN' from dual union all
select 4071,'USN' from dual union all
select 40710,'MRN' from dual union all
select 40711,'GMS' from dual union all
select 40712,'MRN' from dual union all
select 40713,'MRN' from dual union all
select 40714,'MRN' from dual union all
select 40715,'MRN' from dual union all
select 40716,'MRN' from dual union all
select 40717,'MRN' from dual union all
select 40718,'MRN' from dual union all
select 40719,'MRN' from dual union all
select 4072,'NRL' from dual union all
select 40720,'AN' from dual union all
select 40721,'MRN' from dual union all
select 40722,'MRN' from dual union all
select 40723,'MRN' from dual union all
select 40724,'MRN' from dual union all
select 40725,'MRN' from dual union all
select 40726,'MRN' from dual union all
select 40727,'MRN' from dual union all
select 40728,'MRN' from dual union all
select 40729,'MRN' from dual union all
select 4073,'USAF' from dual union all
select 40730,'MRN' from dual union all
select 40731,'MRN' from dual union all
select 40732,'MRN' from dual union all
select 40733,'MRN' from dual union all
select 40734,'MRN' from dual union all
select 40735,'MRN' from dual union all
select 40736,'MRN' from dual union all
select 40737,'MRN' from dual union all
select 40738,'MRN' from dual union all
select 40739,'MRN' from dual union all
select 4074,'PARD' from dual union all
select 40740,'MRN' from dual;
insert into LAUNCH_AGENCY
select 40741,'MRN' from dual union all
select 40742,'MRN' from dual union all
select 40743,'MRN' from dual union all
select 40744,'MRN' from dual union all
select 40745,'MRN' from dual union all
select 40746,'MRN' from dual union all
select 40747,'MRN' from dual union all
select 40748,'MRN' from dual union all
select 40749,'MRN' from dual union all
select 4075,'PARD' from dual union all
select 40750,'MRN' from dual union all
select 40751,'MRN' from dual union all
select 40752,'MRN' from dual union all
select 40753,'MRN' from dual union all
select 40754,'MRN' from dual union all
select 40755,'MRN' from dual union all
select 40756,'MRN' from dual union all
select 40757,'MRN' from dual union all
select 40758,'MRN' from dual union all
select 40759,'MRN' from dual union all
select 4076,'PARD' from dual union all
select 40760,'MRN' from dual union all
select 40761,'MRN' from dual union all
select 40762,'MRN' from dual union all
select 40763,'MRN' from dual union all
select 40764,'MRN' from dual union all
select 40765,'MRN' from dual union all
select 40766,'MRN' from dual union all
select 40767,'MRN' from dual union all
select 40768,'MRN' from dual union all
select 40769,'MRN' from dual union all
select 4077,'6555GMS' from dual union all
select 40770,'MRN' from dual union all
select 40771,'MRN' from dual union all
select 40772,'MRN' from dual union all
select 40773,'MRN' from dual union all
select 40774,'MRN' from dual union all
select 40775,'MRN' from dual union all
select 40776,'GMS' from dual union all
select 40777,'MRN' from dual union all
select 40778,'MRN' from dual union all
select 40779,'MRN' from dual union all
select 4078,'USAF' from dual union all
select 40780,'MRN' from dual union all
select 40781,'MRN' from dual union all
select 40782,'MRN' from dual union all
select 40783,'MRN' from dual union all
select 40784,'MRN' from dual union all
select 40785,'MRN' from dual union all
select 40786,'MRN' from dual union all
select 40787,'MRN' from dual union all
select 40788,'MRN' from dual union all
select 40789,'MRN' from dual union all
select 4079,'USN' from dual union all
select 40790,'MRN' from dual union all
select 40791,'MRN' from dual union all
select 40792,'MRN' from dual union all
select 40793,'MRN' from dual union all
select 40794,'MRN' from dual union all
select 40795,'MRN' from dual union all
select 40796,'MRN' from dual union all
select 40797,'MRN' from dual union all
select 40798,'MRN' from dual union all
select 40799,'MRN' from dual union all
select 408,'WEHR' from dual union all
select 4080,'JTF7' from dual union all
select 40800,'MRN' from dual union all
select 40801,'MRN' from dual union all
select 40802,'MRN' from dual union all
select 40803,'MRN' from dual union all
select 40804,'MRN' from dual union all
select 40805,'MRN' from dual union all
select 40806,'MRN' from dual union all
select 40807,'MRN' from dual union all
select 40808,'MRN' from dual union all
select 40809,'MRN' from dual union all
select 4081,'JTF7' from dual union all
select 40810,'MRN' from dual union all
select 40811,'MRN' from dual union all
select 40812,'MRN' from dual union all
select 40813,'MRN' from dual union all
select 40814,'GMS' from dual union all
select 40815,'MRN' from dual union all
select 40816,'MRN' from dual union all
select 40817,'MRN' from dual union all
select 40818,'MRN' from dual union all
select 40819,'MRN' from dual union all
select 4082,'PARD' from dual union all
select 40820,'MRN' from dual union all
select 40821,'MRN' from dual union all
select 40822,'MRN' from dual union all
select 40823,'MRN' from dual union all
select 40824,'MRN' from dual union all
select 40825,'MRN' from dual union all
select 40826,'MRN' from dual union all
select 40827,'MRN' from dual union all
select 40828,'MRN' from dual union all
select 40829,'MRN' from dual union all
select 4083,'PARD' from dual union all
select 40830,'MRN' from dual;
insert into LAUNCH_AGENCY
select 40831,'MRN' from dual union all
select 40832,'MRN' from dual union all
select 40833,'MRN' from dual union all
select 40834,'MRN' from dual union all
select 40835,'MRN' from dual union all
select 40836,'MRN' from dual union all
select 40837,'GMS' from dual union all
select 40838,'MRN' from dual union all
select 40839,'MRN' from dual union all
select 4084,'USAF' from dual union all
select 40840,'MRN' from dual union all
select 40841,'MRN' from dual union all
select 40842,'MRN' from dual union all
select 40843,'MRN' from dual union all
select 40844,'MRN' from dual union all
select 40845,'MRN' from dual union all
select 40846,'GMS' from dual union all
select 40847,'GMS' from dual union all
select 40848,'MRN' from dual union all
select 40849,'MRN' from dual union all
select 4085,'PARD' from dual union all
select 40850,'MRN' from dual union all
select 40851,'MRN' from dual union all
select 40852,'MRN' from dual union all
select 40853,'MRN' from dual union all
select 40854,'MRN' from dual union all
select 40855,'MRN' from dual union all
select 40856,'MRN' from dual union all
select 40857,'MRN' from dual union all
select 40858,'MRN' from dual union all
select 40859,'MRN' from dual union all
select 4086,'NRL' from dual union all
select 40860,'MRN' from dual union all
select 40861,'MRN' from dual union all
select 40862,'MRN' from dual union all
select 40863,'MRN' from dual union all
select 40864,'MRN' from dual union all
select 40865,'MRN' from dual union all
select 40866,'MRN' from dual union all
select 40867,'MRN' from dual union all
select 40868,'MRN' from dual union all
select 40869,'MRN' from dual union all
select 4087,'PARD' from dual union all
select 40870,'MRN' from dual union all
select 40871,'MRN' from dual union all
select 40872,'MRN' from dual union all
select 40873,'GMS' from dual union all
select 40874,'GMS' from dual union all
select 40875,'MRN' from dual union all
select 40876,'AN' from dual union all
select 40877,'MRN' from dual union all
select 40878,'MRN' from dual union all
select 40879,'MRN' from dual union all
select 4088,'USN' from dual union all
select 40880,'MRN' from dual union all
select 40881,'MRN' from dual union all
select 40882,'MRN' from dual union all
select 40883,'MRN' from dual union all
select 40884,'MRN' from dual union all
select 40885,'MRN' from dual union all
select 40886,'MRN' from dual union all
select 40887,'MRN' from dual union all
select 40888,'GMS' from dual union all
select 40889,'GMS' from dual union all
select 4089,'JTF7' from dual union all
select 40890,'MRN' from dual union all
select 40891,'MRN' from dual union all
select 40892,'MRN' from dual union all
select 40893,'MRN' from dual union all
select 40894,'MRN' from dual union all
select 40895,'MRN' from dual union all
select 40896,'MRN' from dual union all
select 40897,'MRN' from dual union all
select 40898,'MRN' from dual union all
select 40899,'MRN' from dual union all
select 409,'WEHR' from dual union all
select 4090,'JTF7' from dual union all
select 40900,'MRN' from dual union all
select 40901,'MRN' from dual union all
select 40902,'AN' from dual union all
select 40903,'MRN' from dual union all
select 40904,'MRN' from dual union all
select 40905,'MRN' from dual union all
select 40906,'MRN' from dual union all
select 40907,'MRN' from dual union all
select 40908,'NASA' from dual union all
select 40909,'MRN' from dual union all
select 4091,'PARD' from dual union all
select 40910,'MRN' from dual union all
select 40911,'MRN' from dual union all
select 40912,'MRN' from dual union all
select 40913,'MRN' from dual union all
select 40914,'MRN' from dual union all
select 40915,'MRN' from dual union all
select 40916,'MRN' from dual union all
select 40917,'MRN' from dual union all
select 40918,'MRN' from dual union all
select 40919,'MRN' from dual union all
select 4092,'USAF' from dual union all
select 40920,'MRN' from dual;
insert into LAUNCH_AGENCY
select 40921,'MRN' from dual union all
select 40922,'MRN' from dual union all
select 40923,'MRN' from dual union all
select 40924,'MRN' from dual union all
select 40925,'MRN' from dual union all
select 40926,'MRN' from dual union all
select 40927,'MRN' from dual union all
select 40928,'MRN' from dual union all
select 40929,'MRN' from dual union all
select 4093,'USAF' from dual union all
select 40930,'MRN' from dual union all
select 40931,'MRN' from dual union all
select 40932,'MRN' from dual union all
select 40933,'MRN' from dual union all
select 40934,'MRN' from dual union all
select 40935,'MRN' from dual union all
select 40936,'MRN' from dual union all
select 40937,'MRN' from dual union all
select 40938,'MRN' from dual union all
select 40939,'MRN' from dual union all
select 4094,'UNRV' from dual union all
select 40940,'MRN' from dual union all
select 40941,'MRN' from dual union all
select 40942,'MRN' from dual union all
select 40943,'MRN' from dual union all
select 40944,'MRN' from dual union all
select 40945,'MRN' from dual union all
select 40946,'MRN' from dual union all
select 40947,'MRN' from dual union all
select 40948,'MRN' from dual union all
select 40949,'MRN' from dual union all
select 4095,'USN' from dual union all
select 40950,'MRN' from dual union all
select 40951,'MRN' from dual union all
select 40952,'MRN' from dual union all
select 40953,'MRN' from dual union all
select 40954,'MRN' from dual union all
select 40955,'MRN' from dual union all
select 40956,'MRN' from dual union all
select 40957,'MRN' from dual union all
select 40958,'MRN' from dual union all
select 40959,'MRN' from dual union all
select 4096,'NRL' from dual union all
select 40960,'MRN' from dual union all
select 40961,'MRN' from dual union all
select 40962,'MRN' from dual union all
select 40963,'MRN' from dual union all
select 40964,'MRN' from dual union all
select 40965,'MRN' from dual union all
select 40966,'MRN' from dual union all
select 40967,'MRN' from dual union all
select 40968,'MRN' from dual union all
select 40969,'MRN' from dual union all
select 4097,'NRL' from dual union all
select 40970,'MRN' from dual union all
select 40971,'MRN' from dual union all
select 40972,'MRN' from dual union all
select 40973,'MRN' from dual union all
select 40974,'MRN' from dual union all
select 40975,'MRN' from dual union all
select 40976,'MRN' from dual union all
select 40977,'MRN' from dual union all
select 40978,'MRN' from dual union all
select 40979,'MRN' from dual union all
select 4098,'USAF' from dual union all
select 40980,'MRN' from dual union all
select 40981,'MRN' from dual union all
select 40982,'MRN' from dual union all
select 40983,'MRN' from dual union all
select 40984,'MRN' from dual union all
select 40985,'NASA' from dual union all
select 40986,'MRN' from dual union all
select 40987,'MRN' from dual union all
select 40988,'MRN' from dual union all
select 40989,'MRN' from dual union all
select 4099,'6555A' from dual union all
select 40990,'MRN' from dual union all
select 40991,'MRN' from dual union all
select 40992,'MRN' from dual union all
select 40993,'MRN' from dual union all
select 40994,'MRN' from dual union all
select 40995,'MRN' from dual union all
select 40996,'MRN' from dual union all
select 40997,'MRN' from dual union all
select 40998,'MRN' from dual union all
select 40999,'MRN' from dual union all
select 41,'WEHR' from dual union all
select 410,'WEHR' from dual union all
select 4100,'USAF' from dual union all
select 41000,'MRN' from dual union all
select 41001,'MRN' from dual union all
select 41002,'MRN' from dual union all
select 41003,'MRN' from dual union all
select 41004,'MRN' from dual union all
select 41005,'MRN' from dual union all
select 41006,'MRN' from dual union all
select 41007,'MRN' from dual union all
select 41008,'MRN' from dual union all
select 41009,'MRN' from dual union all
select 4101,'6555A' from dual;
insert into LAUNCH_AGENCY
select 41010,'MRN' from dual union all
select 41011,'MRN' from dual union all
select 41012,'MRN' from dual union all
select 41013,'MRN' from dual union all
select 41014,'MRN' from dual union all
select 41015,'MRN' from dual union all
select 41016,'MRN' from dual union all
select 41017,'MRN' from dual union all
select 41018,'MRN' from dual union all
select 41019,'MRN' from dual union all
select 4102,'PARD' from dual union all
select 41020,'MRN' from dual union all
select 41021,'MRN' from dual union all
select 41022,'MRN' from dual union all
select 41023,'MRN' from dual union all
select 41024,'MRN' from dual union all
select 41025,'MRN' from dual union all
select 41026,'MRN' from dual union all
select 41027,'MRN' from dual union all
select 41028,'MRN' from dual union all
select 41029,'MRN' from dual union all
select 4103,'USAF' from dual union all
select 41030,'MRN' from dual union all
select 41031,'MRN' from dual union all
select 41032,'MRN' from dual union all
select 41033,'MRN' from dual union all
select 41034,'MRN' from dual union all
select 41035,'MRN' from dual union all
select 41036,'MRN' from dual union all
select 41037,'MRN' from dual union all
select 41038,'MRN' from dual union all
select 41039,'MRN' from dual union all
select 4104,'USAF' from dual union all
select 41040,'MRN' from dual union all
select 41041,'MRN' from dual union all
select 41042,'MRN' from dual union all
select 41043,'MRN' from dual union all
select 41044,'MRN' from dual union all
select 41045,'MRN' from dual union all
select 41046,'MRN' from dual union all
select 41047,'MRN' from dual union all
select 41048,'MRN' from dual union all
select 41049,'MRN' from dual union all
select 4105,'USAF' from dual union all
select 41050,'MRN' from dual union all
select 41051,'MRN' from dual union all
select 41052,'MRN' from dual union all
select 41053,'AN' from dual union all
select 41054,'MRN' from dual union all
select 41055,'MRN' from dual union all
select 41056,'MRN' from dual union all
select 41057,'MRN' from dual union all
select 41058,'MRN' from dual union all
select 41059,'MRN' from dual union all
select 4106,'PARD' from dual union all
select 41060,'MRN' from dual union all
select 41061,'NASA' from dual union all
select 41062,'MRN' from dual union all
select 41063,'MRN' from dual union all
select 41064,'MRN' from dual union all
select 41065,'MRN' from dual union all
select 41066,'MRN' from dual union all
select 41067,'MRN' from dual union all
select 41068,'MRN' from dual union all
select 41069,'MRN' from dual union all
select 4107,'USAF' from dual union all
select 41070,'MRN' from dual union all
select 41071,'MRN' from dual union all
select 41072,'MRN' from dual union all
select 41073,'MRN' from dual union all
select 41074,'MRN' from dual union all
select 41075,'MRN' from dual union all
select 41076,'MRN' from dual union all
select 41077,'MRN' from dual union all
select 41078,'MRN' from dual union all
select 41079,'MRN' from dual union all
select 4108,'USAF' from dual union all
select 41080,'MRN' from dual union all
select 41081,'MRN' from dual union all
select 41082,'MRN' from dual union all
select 41083,'MRN' from dual union all
select 41084,'MRN' from dual union all
select 41085,'MRN' from dual union all
select 41086,'MRN' from dual union all
select 41087,'MRN' from dual union all
select 41088,'MRN' from dual union all
select 41089,'MRN' from dual union all
select 4109,'USAF' from dual union all
select 41090,'GMS' from dual union all
select 41091,'MRN' from dual union all
select 41092,'MRN' from dual union all
select 41093,'MRN' from dual union all
select 41094,'MRN' from dual union all
select 41095,'MRN' from dual union all
select 41096,'MRN' from dual union all
select 41097,'MRN' from dual union all
select 41098,'MRN' from dual union all
select 41099,'MRN' from dual union all
select 411,'WEHR' from dual union all
select 4110,'USAF' from dual;
insert into LAUNCH_AGENCY
select 41100,'MRN' from dual union all
select 41101,'MRN' from dual union all
select 41102,'MRN' from dual union all
select 41103,'MRN' from dual union all
select 41104,'MRN' from dual union all
select 41105,'MRN' from dual union all
select 41106,'MRN' from dual union all
select 41107,'MRN' from dual union all
select 41108,'MRN' from dual union all
select 41109,'MRN' from dual union all
select 4111,'NSA' from dual union all
select 4111,'PARD' from dual union all
select 41110,'MRN' from dual union all
select 41111,'MRN' from dual union all
select 41112,'MRN' from dual union all
select 41113,'MRN' from dual union all
select 41114,'MRN' from dual union all
select 41115,'MRN' from dual union all
select 41116,'MRN' from dual union all
select 41117,'MRN' from dual union all
select 41118,'MRN' from dual union all
select 41119,'MRN' from dual union all
select 4112,'ISAS' from dual union all
select 41120,'MRN' from dual union all
select 41121,'MRN' from dual union all
select 41122,'MRN' from dual union all
select 41123,'MRN' from dual union all
select 41124,'MRN' from dual union all
select 41125,'MRN' from dual union all
select 41126,'MRN' from dual union all
select 41127,'GMS' from dual union all
select 41128,'MRN' from dual union all
select 41129,'MRN' from dual union all
select 4113,'USAF' from dual union all
select 41130,'MRN' from dual union all
select 41131,'MRN' from dual union all
select 41132,'MRN' from dual union all
select 41133,'MRN' from dual union all
select 41134,'MRN' from dual union all
select 41135,'MRN' from dual union all
select 41136,'MRN' from dual union all
select 41137,'MRN' from dual union all
select 41138,'MRN' from dual union all
select 41139,'MRN' from dual union all
select 4114,'USAF' from dual union all
select 41140,'MRN' from dual union all
select 41141,'MRN' from dual union all
select 41142,'MRN' from dual union all
select 41143,'MRN' from dual union all
select 41144,'MRN' from dual union all
select 41145,'MRN' from dual union all
select 41146,'MRN' from dual union all
select 41147,'MRN' from dual union all
select 41148,'MRN' from dual union all
select 41149,'MRN' from dual union all
select 4115,'USAF' from dual union all
select 41150,'MRN' from dual union all
select 41151,'MRN' from dual union all
select 41152,'MRN' from dual union all
select 41153,'MRN' from dual union all
select 41154,'MRN' from dual union all
select 41155,'MRN' from dual union all
select 41156,'MRN' from dual union all
select 41157,'MRN' from dual union all
select 41158,'MRN' from dual union all
select 41159,'MRN' from dual union all
select 4116,'ISAS' from dual union all
select 41160,'MRN' from dual union all
select 41161,'MRN' from dual union all
select 41162,'MRN' from dual union all
select 41163,'MRN' from dual union all
select 41164,'MRN' from dual union all
select 41165,'MRN' from dual union all
select 41166,'GMS' from dual union all
select 41167,'MRN' from dual union all
select 41168,'MRN' from dual union all
select 41169,'MRN' from dual union all
select 4117,'ISAS' from dual union all
select 41170,'MRN' from dual union all
select 41171,'MRN' from dual union all
select 41172,'MRN' from dual union all
select 41173,'MRN' from dual union all
select 41174,'MRN' from dual union all
select 41175,'MRN' from dual union all
select 41176,'MRN' from dual union all
select 41177,'MRN' from dual union all
select 41178,'MRN' from dual union all
select 41179,'MRN' from dual union all
select 4118,'PARD' from dual union all
select 41180,'MRN' from dual union all
select 41181,'MRN' from dual union all
select 41182,'MRN' from dual union all
select 41183,'MRN' from dual union all
select 41184,'MRN' from dual union all
select 41185,'MRN' from dual union all
select 41186,'MRN' from dual union all
select 41187,'MRN' from dual union all
select 41188,'MRN' from dual union all
select 41189,'MRN' from dual union all
select 4119,'USAF' from dual;
insert into LAUNCH_AGENCY
select 41190,'MRN' from dual union all
select 41191,'MRN' from dual union all
select 41192,'MRN' from dual union all
select 41193,'MRN' from dual union all
select 41194,'MRN' from dual union all
select 41195,'MRN' from dual union all
select 41196,'MRN' from dual union all
select 41197,'MRN' from dual union all
select 41198,'MRN' from dual union all
select 41199,'MRN' from dual union all
select 412,'WEHR' from dual union all
select 4120,'PARD' from dual union all
select 41200,'MRN' from dual union all
select 41201,'MRN' from dual union all
select 41202,'MRN' from dual union all
select 41203,'MRN' from dual union all
select 41204,'MRN' from dual union all
select 41205,'MRN' from dual union all
select 41206,'MRN' from dual union all
select 41207,'MRN' from dual union all
select 41208,'MRN' from dual union all
select 41209,'MRN' from dual union all
select 4121,'USAF' from dual union all
select 41210,'MRN' from dual union all
select 41211,'MRN' from dual union all
select 41212,'MRN' from dual union all
select 41213,'MRN' from dual union all
select 41214,'MRN' from dual union all
select 41215,'MRN' from dual union all
select 41216,'MRN' from dual union all
select 41217,'MRN' from dual union all
select 41218,'MRN' from dual union all
select 41219,'MRN' from dual union all
select 4122,'USA' from dual union all
select 41220,'MRN' from dual union all
select 41221,'MRN' from dual union all
select 41222,'MRN' from dual union all
select 41223,'MRN' from dual union all
select 41224,'MRN' from dual union all
select 41225,'NRCC' from dual union all
select 41226,'MRN' from dual union all
select 41227,'GMS' from dual union all
select 41228,'MRN' from dual union all
select 41229,'MRN' from dual union all
select 4123,'USA' from dual union all
select 41230,'MRN' from dual union all
select 41231,'MRN' from dual union all
select 41232,'MRN' from dual union all
select 41233,'MRN' from dual union all
select 41234,'MRN' from dual union all
select 41235,'MRN' from dual union all
select 41236,'MRN' from dual union all
select 41237,'MRN' from dual union all
select 41238,'MRN' from dual union all
select 41239,'MRN' from dual union all
select 4124,'USA' from dual union all
select 41240,'MRN' from dual union all
select 41241,'MRN' from dual union all
select 41242,'MRN' from dual union all
select 41243,'MRN' from dual union all
select 41244,'MRN' from dual union all
select 41245,'MRN' from dual union all
select 41246,'MRN' from dual union all
select 41247,'MRN' from dual union all
select 41248,'MRN' from dual union all
select 41249,'MRN' from dual union all
select 4125,'NRL' from dual union all
select 41250,'MRN' from dual union all
select 41251,'MRN' from dual union all
select 41252,'MRN' from dual union all
select 41253,'MRN' from dual union all
select 41254,'MRN' from dual union all
select 41255,'MRN' from dual union all
select 41256,'MRN' from dual union all
select 41257,'MRN' from dual union all
select 41258,'MRN' from dual union all
select 41259,'MRN' from dual union all
select 4126,'PARD' from dual union all
select 41260,'MRN' from dual union all
select 41261,'MRN' from dual union all
select 41262,'MRN' from dual union all
select 41263,'MRN' from dual union all
select 41264,'MRN' from dual union all
select 41265,'MRN' from dual union all
select 41266,'AN' from dual union all
select 41267,'MRN' from dual union all
select 41268,'MRN' from dual union all
select 41269,'GMS' from dual union all
select 4127,'ARDC' from dual union all
select 41270,'MRN' from dual union all
select 41271,'AN' from dual union all
select 41272,'MRN' from dual union all
select 41273,'MRN' from dual union all
select 41274,'MRN' from dual union all
select 41275,'MRN' from dual union all
select 41276,'WFC' from dual union all
select 41277,'MRN' from dual union all
select 41278,'MRN' from dual union all
select 41279,'MRN' from dual union all
select 4128,'USA' from dual;
insert into LAUNCH_AGENCY
select 41280,'MRN' from dual union all
select 41281,'MRN' from dual union all
select 41282,'WFC' from dual union all
select 41283,'MRN' from dual union all
select 41284,'MRN' from dual union all
select 41285,'MRN' from dual union all
select 41286,'MRN' from dual union all
select 41287,'WFC' from dual union all
select 41288,'MRN' from dual union all
select 41289,'MRN' from dual union all
select 4129,'USA' from dual union all
select 41290,'MRN' from dual union all
select 41291,'MRN' from dual union all
select 41292,'MRN' from dual union all
select 41293,'MRN' from dual union all
select 41294,'MRN' from dual union all
select 41295,'MRN' from dual union all
select 41296,'MRN' from dual union all
select 41297,'MRN' from dual union all
select 41298,'MRN' from dual union all
select 41299,'MRN' from dual union all
select 413,'WEHR' from dual union all
select 4130,'SCEL' from dual union all
select 41300,'MRN' from dual union all
select 41301,'MRN' from dual union all
select 41302,'MRN' from dual union all
select 41303,'MRN' from dual union all
select 41304,'MRN' from dual union all
select 41305,'MRN' from dual union all
select 41306,'MRN' from dual union all
select 41307,'MRN' from dual union all
select 41308,'GMS' from dual union all
select 41309,'MRN' from dual union all
select 4131,'SCEL' from dual union all
select 41310,'MRN' from dual union all
select 41311,'MRN' from dual union all
select 41312,'MRN' from dual union all
select 41313,'MRN' from dual union all
select 41314,'MRN' from dual union all
select 41315,'MRN' from dual union all
select 41316,'MRN' from dual union all
select 41317,'MRN' from dual union all
select 41318,'MRN' from dual union all
select 41319,'MRN' from dual union all
select 4132,'SCEL' from dual union all
select 41320,'MRN' from dual union all
select 41321,'MRN' from dual union all
select 41322,'MRN' from dual union all
select 41323,'GMS' from dual union all
select 41324,'MRN' from dual union all
select 41325,'MRN' from dual union all
select 41326,'MRN' from dual union all
select 41327,'MRN' from dual union all
select 41328,'MRN' from dual union all
select 41329,'MRN' from dual union all
select 4133,'SCEL' from dual union all
select 41330,'MRN' from dual union all
select 41331,'MRN' from dual union all
select 41332,'MRN' from dual union all
select 41333,'MRN' from dual union all
select 41334,'MRN' from dual union all
select 41335,'MRN' from dual union all
select 41336,'MRN' from dual union all
select 41337,'MRN' from dual union all
select 41338,'MRN' from dual union all
select 41339,'MRN' from dual union all
select 4134,'SCEL' from dual union all
select 41340,'MRN' from dual union all
select 41341,'MRN' from dual union all
select 41342,'MRN' from dual union all
select 41343,'MRN' from dual union all
select 41344,'MRN' from dual union all
select 41345,'MRN' from dual union all
select 41346,'MRN' from dual union all
select 41347,'MRN' from dual union all
select 41348,'MRN' from dual union all
select 41349,'MRN' from dual union all
select 4135,'SCEL' from dual union all
select 41350,'MRN' from dual union all
select 41351,'MRN' from dual union all
select 41352,'MRN' from dual union all
select 41353,'MRN' from dual union all
select 41354,'MRN' from dual union all
select 41355,'MRN' from dual union all
select 41356,'MRN' from dual union all
select 41357,'MRN' from dual union all
select 41358,'MRN' from dual union all
select 41359,'MRN' from dual union all
select 4136,'SCEL' from dual union all
select 41360,'MRN' from dual union all
select 41361,'MRN' from dual union all
select 41362,'MRN' from dual union all
select 41363,'MRN' from dual union all
select 41364,'MRN' from dual union all
select 41365,'MRN' from dual union all
select 41366,'MRN' from dual union all
select 41367,'MRN' from dual union all
select 41368,'MRN' from dual union all
select 41369,'MRN' from dual union all
select 4137,'SCEL' from dual;
insert into LAUNCH_AGENCY
select 41370,'MRN' from dual union all
select 41371,'MRN' from dual union all
select 41372,'MRN' from dual union all
select 41373,'MRN' from dual union all
select 41374,'MRN' from dual union all
select 41375,'MRN' from dual union all
select 41376,'MRN' from dual union all
select 41377,'MRN' from dual union all
select 41378,'MRN' from dual union all
select 41379,'AN' from dual union all
select 4138,'SCEL' from dual union all
select 41380,'GMS' from dual union all
select 41381,'MRN' from dual union all
select 41382,'MRN' from dual union all
select 41383,'MRN' from dual union all
select 41384,'MRN' from dual union all
select 41385,'MRN' from dual union all
select 41386,'MRN' from dual union all
select 41387,'MRN' from dual union all
select 41388,'MRN' from dual union all
select 41389,'MRN' from dual union all
select 4139,'SCEL' from dual union all
select 41390,'MRN' from dual union all
select 41391,'MRN' from dual union all
select 41392,'MRN' from dual union all
select 41393,'MRN' from dual union all
select 41394,'MRN' from dual union all
select 41395,'MRN' from dual union all
select 41396,'MRN' from dual union all
select 41397,'MRN' from dual union all
select 41398,'MRN' from dual union all
select 41399,'MRN' from dual union all
select 414,'WEHR' from dual union all
select 4140,'SCEL' from dual union all
select 41400,'MRN' from dual union all
select 41401,'MRN' from dual union all
select 41402,'MRN' from dual union all
select 41403,'MRN' from dual union all
select 41404,'MRN' from dual union all
select 41405,'GMS' from dual union all
select 41406,'MRN' from dual union all
select 41407,'AN' from dual union all
select 41408,'MRN' from dual union all
select 41409,'AN' from dual union all
select 4141,'SCEL' from dual union all
select 41410,'MRN' from dual union all
select 41411,'MRN' from dual union all
select 41412,'MRN' from dual union all
select 41413,'MRN' from dual union all
select 41414,'MRN' from dual union all
select 41415,'MRN' from dual union all
select 41416,'MRN' from dual union all
select 41417,'MRN' from dual union all
select 41418,'GMS' from dual union all
select 41419,'MRN' from dual union all
select 4142,'SCEL' from dual union all
select 41420,'MRN' from dual union all
select 41421,'MRN' from dual union all
select 41422,'MRN' from dual union all
select 41423,'MRN' from dual union all
select 41424,'MRN' from dual union all
select 41425,'MRN' from dual union all
select 41426,'MRN' from dual union all
select 41427,'MRN' from dual union all
select 41428,'MRN' from dual union all
select 41429,'MRN' from dual union all
select 4143,'SCEL' from dual union all
select 41430,'MRN' from dual union all
select 41431,'MRN' from dual union all
select 41432,'MRN' from dual union all
select 41433,'MRN' from dual union all
select 41434,'MRN' from dual union all
select 41435,'AN' from dual union all
select 41436,'MRN' from dual union all
select 41437,'MRN' from dual union all
select 41438,'MRN' from dual union all
select 41439,'MRN' from dual union all
select 4144,'SCEL' from dual union all
select 41440,'MRN' from dual union all
select 41441,'WFC' from dual union all
select 41442,'MRN' from dual union all
select 41443,'MRN' from dual union all
select 41444,'MRN' from dual union all
select 41445,'MRN' from dual union all
select 41446,'MRN' from dual union all
select 41447,'MRN' from dual union all
select 41448,'MRN' from dual union all
select 41449,'MRN' from dual union all
select 4145,'SCEL' from dual union all
select 41450,'MRN' from dual union all
select 41451,'MRN' from dual union all
select 41452,'MRN' from dual union all
select 41453,'MRN' from dual union all
select 41454,'MRN' from dual union all
select 41455,'MRN' from dual union all
select 41456,'MRN' from dual union all
select 41457,'MRN' from dual union all
select 41458,'MRN' from dual union all
select 41459,'WFC' from dual union all
select 4146,'SCEL' from dual;
insert into LAUNCH_AGENCY
select 41460,'MRN' from dual union all
select 41461,'MRN' from dual union all
select 41462,'MRN' from dual union all
select 41463,'MRN' from dual union all
select 41464,'MRN' from dual union all
select 41465,'NASA' from dual union all
select 41466,'MRN' from dual union all
select 41467,'MRN' from dual union all
select 41468,'MRN' from dual union all
select 41469,'MRN' from dual union all
select 4147,'SCEL' from dual union all
select 41470,'MRN' from dual union all
select 41471,'MRN' from dual union all
select 41472,'MRN' from dual union all
select 41473,'MRN' from dual union all
select 41474,'MRN' from dual union all
select 41475,'MRN' from dual union all
select 41476,'MRN' from dual union all
select 41477,'MRN' from dual union all
select 41478,'MRN' from dual union all
select 41479,'MRN' from dual union all
select 4148,'SCEL' from dual union all
select 41480,'MRN' from dual union all
select 41481,'MRN' from dual union all
select 41482,'MRN' from dual union all
select 41483,'MRN' from dual union all
select 41484,'MRN' from dual union all
select 41485,'MRN' from dual union all
select 41486,'MRN' from dual union all
select 41487,'MRN' from dual union all
select 41488,'MRN' from dual union all
select 41489,'MRN' from dual union all
select 4149,'UNRV' from dual union all
select 41490,'MRN' from dual union all
select 41491,'MRN' from dual union all
select 41492,'MRN' from dual union all
select 41493,'MRN' from dual union all
select 41494,'MRN' from dual union all
select 41495,'MRN' from dual union all
select 41496,'MRN' from dual union all
select 41497,'MRN' from dual union all
select 41498,'MRN' from dual union all
select 41499,'MRN' from dual union all
select 415,'WEHR' from dual union all
select 4150,'UNRV' from dual union all
select 41500,'MRN' from dual union all
select 41501,'MRN' from dual union all
select 41502,'MRN' from dual union all
select 41503,'MRN' from dual union all
select 41504,'MRN' from dual union all
select 41505,'MRN' from dual union all
select 41506,'MRN' from dual union all
select 41507,'MRN' from dual union all
select 41508,'MRN' from dual union all
select 41509,'MRN' from dual union all
select 4151,'UNRV' from dual union all
select 41510,'MRN' from dual union all
select 41511,'MRN' from dual union all
select 41512,'MRN' from dual union all
select 41513,'MRN' from dual union all
select 41514,'MRN' from dual union all
select 41515,'MRN' from dual union all
select 41516,'MRN' from dual union all
select 41517,'MRN' from dual union all
select 41518,'MRN' from dual union all
select 41519,'MRN' from dual union all
select 4152,'UNRV' from dual union all
select 41520,'AN' from dual union all
select 41521,'MRN' from dual union all
select 41522,'WFC' from dual union all
select 41523,'MRN' from dual union all
select 41524,'MRN' from dual union all
select 41525,'MRN' from dual union all
select 41526,'MRN' from dual union all
select 41527,'MRN' from dual union all
select 41528,'MRN' from dual union all
select 41529,'MRN' from dual union all
select 4153,'UNRV' from dual union all
select 41530,'MRN' from dual union all
select 41531,'MRN' from dual union all
select 41532,'MRN' from dual union all
select 41533,'MRN' from dual union all
select 41534,'MRN' from dual union all
select 41535,'MRN' from dual union all
select 41536,'MRN' from dual union all
select 41537,'MRN' from dual union all
select 41538,'MRN' from dual union all
select 41539,'MRN' from dual union all
select 4154,'UNRV' from dual union all
select 41540,'MRN' from dual union all
select 41541,'MRN' from dual union all
select 41542,'MRN' from dual union all
select 41543,'AN' from dual union all
select 41544,'AN' from dual union all
select 41545,'AN' from dual union all
select 41546,'AN' from dual union all
select 41547,'AN' from dual union all
select 41548,'MRN' from dual union all
select 41549,'AN' from dual union all
select 4155,'6555A' from dual;
insert into LAUNCH_AGENCY
select 41550,'AN' from dual union all
select 41551,'AN' from dual union all
select 41552,'NASA' from dual union all
select 41553,'AN' from dual union all
select 41554,'AN' from dual union all
select 41555,'AN' from dual union all
select 41556,'AN' from dual union all
select 41557,'AN' from dual union all
select 41558,'AN' from dual union all
select 41559,'MRN' from dual union all
select 4156,'UNRV' from dual union all
select 41560,'NASA' from dual union all
select 41561,'AN' from dual union all
select 41562,'MRN' from dual union all
select 41563,'AN' from dual union all
select 41564,'AN' from dual union all
select 41565,'AN' from dual union all
select 41566,'AN' from dual union all
select 41567,'AN' from dual union all
select 41568,'AN' from dual union all
select 41569,'AN' from dual union all
select 4157,'MVS' from dual union all
select 41570,'MRN' from dual union all
select 41571,'AN' from dual union all
select 41572,'AN' from dual union all
select 41573,'AN' from dual union all
select 41574,'AN' from dual union all
select 41575,'AN' from dual union all
select 41576,'AN' from dual union all
select 41577,'AN' from dual union all
select 41578,'AN' from dual union all
select 41579,'AN' from dual union all
select 4158,'MVS' from dual union all
select 41580,'AN' from dual union all
select 41581,'AN' from dual union all
select 41582,'MRN' from dual union all
select 41583,'AN' from dual union all
select 41584,'AN' from dual union all
select 41585,'AN' from dual union all
select 41586,'AN' from dual union all
select 41587,'AN' from dual union all
select 41588,'AN' from dual union all
select 41589,'AN' from dual union all
select 4159,'NRL' from dual union all
select 41590,'MRN' from dual union all
select 41591,'AN' from dual union all
select 41592,'MRN' from dual union all
select 41593,'AN' from dual union all
select 41594,'MRN' from dual union all
select 41595,'AN' from dual union all
select 41596,'AN' from dual union all
select 41597,'AN' from dual union all
select 41598,'AN' from dual union all
select 41599,'AN' from dual union all
select 416,'WEHR' from dual union all
select 4160,'6555A' from dual union all
select 41600,'AN' from dual union all
select 41601,'AN' from dual union all
select 41602,'AN' from dual union all
select 41603,'MRN' from dual union all
select 41604,'AN' from dual union all
select 41605,'AN' from dual union all
select 41606,'AN' from dual union all
select 41607,'MRN' from dual union all
select 41608,'AN' from dual union all
select 41609,'AN' from dual union all
select 4161,'NRL' from dual union all
select 41610,'AN' from dual union all
select 41611,'AN' from dual union all
select 41612,'AN' from dual union all
select 41613,'AN' from dual union all
select 41614,'AN' from dual union all
select 41615,'MRN' from dual union all
select 41616,'AN' from dual union all
select 41617,'AN' from dual union all
select 41618,'AN' from dual union all
select 41619,'AN' from dual union all
select 4162,'NRL' from dual union all
select 41620,'AN' from dual union all
select 41621,'AN' from dual union all
select 41622,'AN' from dual union all
select 41623,'AN' from dual union all
select 41624,'AN' from dual union all
select 41625,'AN' from dual union all
select 41626,'AN' from dual union all
select 41627,'AN' from dual union all
select 41628,'AN' from dual union all
select 41629,'AN' from dual union all
select 4163,'6555A' from dual union all
select 41630,'MRN' from dual union all
select 41631,'MRN' from dual union all
select 41632,'AN' from dual union all
select 41633,'AN' from dual union all
select 41634,'MRN' from dual union all
select 41635,'AN' from dual union all
select 41636,'AN' from dual union all
select 41637,'AN' from dual union all
select 41638,'AN' from dual union all
select 41639,'AN' from dual union all
select 4164,'USA' from dual;
insert into LAUNCH_AGENCY
select 41640,'AN' from dual union all
select 41641,'AN' from dual union all
select 41642,'MRN' from dual union all
select 41643,'MRN' from dual union all
select 41644,'AN' from dual union all
select 41645,'MRN' from dual union all
select 41646,'MRN' from dual union all
select 41647,'AN' from dual union all
select 41648,'AN' from dual union all
select 41649,'AN' from dual union all
select 4165,'6555A' from dual union all
select 41650,'AN' from dual union all
select 41651,'MRN' from dual union all
select 41652,'AN' from dual union all
select 41653,'MRN' from dual union all
select 41654,'AN' from dual union all
select 41655,'AN' from dual union all
select 41656,'MRN' from dual union all
select 41657,'AN' from dual union all
select 41658,'AN' from dual union all
select 41659,'AN' from dual union all
select 4166,'USN' from dual union all
select 41660,'AN' from dual union all
select 41661,'MRN' from dual union all
select 41662,'AN' from dual union all
select 41663,'MRN' from dual union all
select 41664,'MRN' from dual union all
select 41665,'AN' from dual union all
select 41666,'AN' from dual union all
select 41667,'AN' from dual union all
select 41668,'AN' from dual union all
select 41669,'AN' from dual union all
select 4167,'6555A' from dual union all
select 41670,'AN' from dual union all
select 41671,'MRN' from dual union all
select 41672,'MRN' from dual union all
select 41673,'AN' from dual union all
select 41674,'AN' from dual union all
select 41675,'AN' from dual union all
select 41676,'AN' from dual union all
select 41677,'AN' from dual union all
select 41678,'MRN' from dual union all
select 41679,'MRN' from dual union all
select 4168,'ARDC' from dual union all
select 41680,'AN' from dual union all
select 41681,'AN' from dual union all
select 41682,'AN' from dual union all
select 41683,'AN' from dual union all
select 41684,'MRN' from dual union all
select 41685,'AN' from dual union all
select 41686,'MRN' from dual union all
select 41687,'AN' from dual union all
select 41688,'AN' from dual union all
select 41689,'AN' from dual union all
select 4169,'UNRV' from dual union all
select 41690,'AN' from dual union all
select 41691,'AN' from dual union all
select 41692,'MRN' from dual union all
select 41693,'MRN' from dual union all
select 41694,'AN' from dual union all
select 41695,'AN' from dual union all
select 41696,'MRN' from dual union all
select 41697,'MRN' from dual union all
select 41698,'AN' from dual union all
select 41699,'AN' from dual union all
select 417,'WER836-3' from dual union all
select 4170,'USA' from dual union all
select 41700,'AN' from dual union all
select 41701,'AN' from dual union all
select 41702,'AN' from dual union all
select 41703,'MRN' from dual union all
select 41704,'MRN' from dual union all
select 41705,'AN' from dual union all
select 41706,'AN' from dual union all
select 41707,'AN' from dual union all
select 41708,'AN' from dual union all
select 41709,'MRN' from dual union all
select 4171,'NII88A' from dual union all
select 41710,'MRN' from dual union all
select 41711,'AN' from dual union all
select 41712,'AN' from dual union all
select 41713,'MRN' from dual union all
select 41714,'MRN' from dual union all
select 41715,'MRN' from dual union all
select 41716,'AN' from dual union all
select 41717,'AN' from dual union all
select 41718,'AN' from dual union all
select 41719,'AN' from dual union all
select 4172,'UNRV' from dual union all
select 41720,'AN' from dual union all
select 41721,'MRN' from dual union all
select 41722,'MRN' from dual union all
select 41723,'AN' from dual union all
select 41724,'AN' from dual union all
select 41725,'AN' from dual union all
select 41726,'AN' from dual union all
select 41727,'AN' from dual union all
select 41728,'MRN' from dual union all
select 41729,'AN' from dual union all
select 4173,'UNRV' from dual;
insert into LAUNCH_AGENCY
select 41730,'AN' from dual union all
select 41731,'AN' from dual union all
select 41732,'MRN' from dual union all
select 41733,'AN' from dual union all
select 41734,'AN' from dual union all
select 41735,'MRN' from dual union all
select 41736,'AN' from dual union all
select 41737,'AN' from dual union all
select 41738,'AN' from dual union all
select 41739,'AN' from dual union all
select 4174,'UNRV' from dual union all
select 41740,'AN' from dual union all
select 41741,'MRN' from dual union all
select 41742,'MRN' from dual union all
select 41743,'MRN' from dual union all
select 41744,'AN' from dual union all
select 41745,'AN' from dual union all
select 41746,'AN' from dual union all
select 41747,'AN' from dual union all
select 41748,'AN' from dual union all
select 41749,'AN' from dual union all
select 4175,'UNRV' from dual union all
select 41750,'AN' from dual union all
select 41751,'AN' from dual union all
select 41752,'MRN' from dual union all
select 41753,'AN' from dual union all
select 41754,'MRN' from dual union all
select 41755,'MRN' from dual union all
select 41756,'AN' from dual union all
select 41757,'MRN' from dual union all
select 41758,'AN' from dual union all
select 41759,'AN' from dual union all
select 4176,'UNRV' from dual union all
select 41760,'AN' from dual union all
select 41761,'MRN' from dual union all
select 41762,'MRN' from dual union all
select 41763,'AN' from dual union all
select 41764,'AN' from dual union all
select 41765,'AN' from dual union all
select 41766,'AN' from dual union all
select 41767,'AN' from dual union all
select 41768,'AN' from dual union all
select 41769,'MRN' from dual union all
select 4177,'UNRV' from dual union all
select 41770,'MRN' from dual union all
select 41771,'AN' from dual union all
select 41772,'AN' from dual union all
select 41773,'AN' from dual union all
select 41774,'AN' from dual union all
select 41775,'MRN' from dual union all
select 41776,'AN' from dual union all
select 41777,'AN' from dual union all
select 41778,'AN' from dual union all
select 41779,'MRN' from dual union all
select 4178,'UNRV' from dual union all
select 41780,'AN' from dual union all
select 41781,'AN' from dual union all
select 41782,'AN' from dual union all
select 41783,'AN' from dual union all
select 41784,'AN' from dual union all
select 41785,'AN' from dual union all
select 41786,'AN' from dual union all
select 41787,'AN' from dual union all
select 41788,'AN' from dual union all
select 41789,'AN' from dual union all
select 4179,'UNRV' from dual union all
select 41790,'AN' from dual union all
select 41791,'AN' from dual union all
select 41792,'MRN' from dual union all
select 41793,'AN' from dual union all
select 41794,'AN' from dual union all
select 41795,'AN' from dual union all
select 41796,'AN' from dual union all
select 41797,'AN' from dual union all
select 41798,'AN' from dual union all
select 41799,'AN' from dual union all
select 418,'WEHR' from dual union all
select 4180,'UNRV' from dual union all
select 41800,'AN' from dual union all
select 41801,'AN' from dual union all
select 41802,'AN' from dual union all
select 41803,'AN' from dual union all
select 41804,'AN' from dual union all
select 41805,'AN' from dual union all
select 41806,'AN' from dual union all
select 41807,'AN' from dual union all
select 41808,'AN' from dual union all
select 41809,'AN' from dual union all
select 4181,'UNRV' from dual union all
select 41810,'MRN' from dual union all
select 41811,'MRN' from dual union all
select 41812,'MRN' from dual union all
select 41813,'MRN' from dual union all
select 41814,'AN' from dual union all
select 41815,'AN' from dual union all
select 41816,'MRN' from dual union all
select 41817,'AN' from dual union all
select 41818,'AN' from dual union all
select 41819,'AN' from dual union all
select 4182,'UNRV' from dual;
insert into LAUNCH_AGENCY
select 41820,'AN' from dual union all
select 41821,'AN' from dual union all
select 41822,'AN' from dual union all
select 41823,'AN' from dual union all
select 41824,'MRN' from dual union all
select 41825,'AN' from dual union all
select 41826,'AN' from dual union all
select 41827,'MRN' from dual union all
select 41828,'AN' from dual union all
select 41829,'AN' from dual union all
select 4183,'UNRV' from dual union all
select 41830,'AN' from dual union all
select 41831,'AN' from dual union all
select 41832,'AN' from dual union all
select 41833,'MRN' from dual union all
select 41834,'AN' from dual union all
select 41835,'AN' from dual union all
select 41836,'AN' from dual union all
select 41837,'AN' from dual union all
select 41838,'AN' from dual union all
select 41839,'AN' from dual union all
select 4184,'UNRV' from dual union all
select 41840,'AN' from dual union all
select 41841,'AN' from dual union all
select 41842,'AN' from dual union all
select 41843,'AN' from dual union all
select 41844,'AN' from dual union all
select 41845,'AN' from dual union all
select 41846,'AN' from dual union all
select 41847,'AN' from dual union all
select 41848,'MRN' from dual union all
select 41849,'AN' from dual union all
select 4185,'UNRV' from dual union all
select 41850,'AN' from dual union all
select 41851,'AN' from dual union all
select 41852,'AN' from dual union all
select 41853,'AN' from dual union all
select 41854,'AN' from dual union all
select 41855,'AN' from dual union all
select 41856,'AN' from dual union all
select 41857,'AN' from dual union all
select 41858,'AN' from dual union all
select 41859,'AN' from dual union all
select 4186,'UNRV' from dual union all
select 41860,'AN' from dual union all
select 41861,'AN' from dual union all
select 41862,'AN' from dual union all
select 41863,'AN' from dual union all
select 41864,'MRN' from dual union all
select 41865,'AN' from dual union all
select 41866,'AN' from dual union all
select 41867,'AN' from dual union all
select 41868,'AN' from dual union all
select 41869,'AN' from dual union all
select 4187,'UNRV' from dual union all
select 41870,'NASA' from dual union all
select 41871,'AN' from dual union all
select 41872,'AN' from dual union all
select 41873,'AN' from dual union all
select 41874,'AN' from dual union all
select 41875,'AN' from dual union all
select 41876,'MRN' from dual union all
select 41877,'AN' from dual union all
select 41878,'AN' from dual union all
select 41879,'AN' from dual union all
select 4188,'UNRV' from dual union all
select 41880,'AN' from dual union all
select 41881,'AN' from dual union all
select 41882,'AN' from dual union all
select 41883,'AN' from dual union all
select 41884,'AN' from dual union all
select 41885,'AN' from dual union all
select 41886,'AN' from dual union all
select 41887,'AN' from dual union all
select 41888,'AN' from dual union all
select 41889,'AN' from dual union all
select 4189,'MVS' from dual union all
select 41890,'MRN' from dual union all
select 41891,'MRN' from dual union all
select 41892,'AN' from dual union all
select 41893,'AN' from dual union all
select 41894,'MRN' from dual union all
select 41895,'AN' from dual union all
select 41896,'MRN' from dual union all
select 41897,'MRN' from dual union all
select 41898,'AN' from dual union all
select 41899,'AN' from dual union all
select 419,'WEHR' from dual union all
select 4190,'UNRV' from dual union all
select 41900,'AN' from dual union all
select 41901,'AN' from dual union all
select 41902,'AN' from dual union all
select 41903,'AN' from dual union all
select 41904,'AN' from dual union all
select 41905,'AN' from dual union all
select 41906,'MRN' from dual union all
select 41907,'MRN' from dual union all
select 41908,'AN' from dual union all
select 41909,'MRN' from dual union all
select 4191,'UNRV' from dual;
insert into LAUNCH_AGENCY
select 41910,'AN' from dual union all
select 41911,'AN' from dual union all
select 41912,'AN' from dual union all
select 41913,'AN' from dual union all
select 41914,'AN' from dual union all
select 41915,'AN' from dual union all
select 41916,'MRN' from dual union all
select 41917,'NASA' from dual union all
select 41918,'AN' from dual union all
select 41919,'AN' from dual union all
select 4192,'UNRV' from dual union all
select 41920,'AN' from dual union all
select 41921,'AN' from dual union all
select 41922,'AN' from dual union all
select 41923,'AN' from dual union all
select 41924,'NASA' from dual union all
select 41925,'AN' from dual union all
select 41926,'AN' from dual union all
select 41927,'AN' from dual union all
select 41928,'AN' from dual union all
select 41929,'MRN' from dual union all
select 4193,'UNRV' from dual union all
select 41930,'AN' from dual union all
select 41931,'AN' from dual union all
select 41932,'AN' from dual union all
select 41933,'AN' from dual union all
select 41934,'AN' from dual union all
select 41935,'AN' from dual union all
select 41936,'AN' from dual union all
select 41937,'MRN' from dual union all
select 41938,'AN' from dual union all
select 41939,'MRN' from dual union all
select 4194,'UNRV' from dual union all
select 41940,'AN' from dual union all
select 41941,'AN' from dual union all
select 41942,'AN' from dual union all
select 41943,'AN' from dual union all
select 41944,'AN' from dual union all
select 41945,'MRN' from dual union all
select 41946,'MRN' from dual union all
select 41947,'AN' from dual union all
select 41948,'AN' from dual union all
select 41949,'AN' from dual union all
select 4195,'UNRV' from dual union all
select 41950,'AN' from dual union all
select 41951,'AN' from dual union all
select 41952,'AN' from dual union all
select 41953,'AN' from dual union all
select 41954,'AN' from dual union all
select 41955,'MRN' from dual union all
select 41956,'MRN' from dual union all
select 41957,'AN' from dual union all
select 41958,'AN' from dual union all
select 41959,'AN' from dual union all
select 4196,'UNRV' from dual union all
select 41960,'AN' from dual union all
select 41961,'AN' from dual union all
select 41962,'AN' from dual union all
select 41963,'AN' from dual union all
select 41964,'AN' from dual union all
select 41965,'MRN' from dual union all
select 41966,'AN' from dual union all
select 41967,'MRN' from dual union all
select 41968,'MRN' from dual union all
select 41969,'AN' from dual union all
select 4197,'UNRV' from dual union all
select 41970,'MRN' from dual union all
select 41971,'AN' from dual union all
select 41972,'AN' from dual union all
select 41973,'AN' from dual union all
select 41974,'AN' from dual union all
select 41975,'AN' from dual union all
select 41976,'AN' from dual union all
select 41977,'MRN' from dual union all
select 41978,'MRN' from dual union all
select 41979,'AN' from dual union all
select 4198,'UNRV' from dual union all
select 41980,'AN' from dual union all
select 41981,'AN' from dual union all
select 41982,'AN' from dual union all
select 41983,'AN' from dual union all
select 41984,'AN' from dual union all
select 41985,'AN' from dual union all
select 41986,'MRN' from dual union all
select 41987,'MRN' from dual union all
select 41988,'MRN' from dual union all
select 41989,'AN' from dual union all
select 4199,'UNRV' from dual union all
select 41990,'AN' from dual union all
select 41991,'AN' from dual union all
select 41992,'AN' from dual union all
select 41993,'AN' from dual union all
select 41994,'AN' from dual union all
select 41995,'MRN' from dual union all
select 41996,'MRN' from dual union all
select 41997,'MRN' from dual union all
select 41998,'AN' from dual union all
select 41999,'AN' from dual union all
select 42,'WEHR' from dual union all
select 420,'WEHR' from dual;
insert into LAUNCH_AGENCY
select 4200,'MVS' from dual union all
select 42000,'AN' from dual union all
select 42001,'AN' from dual union all
select 42002,'MRN' from dual union all
select 42003,'AN' from dual union all
select 42004,'AN' from dual union all
select 42005,'AN' from dual union all
select 42006,'MRN' from dual union all
select 42007,'MRN' from dual union all
select 42008,'MRN' from dual union all
select 42009,'AN' from dual union all
select 4201,'UNRV' from dual union all
select 42010,'AN' from dual union all
select 42011,'AN' from dual union all
select 42012,'AN' from dual union all
select 42013,'AN' from dual union all
select 42014,'AN' from dual union all
select 42015,'MRN' from dual union all
select 42016,'AN' from dual union all
select 42017,'MRN' from dual union all
select 42018,'AN' from dual union all
select 42019,'AN' from dual union all
select 4202,'UNRV' from dual union all
select 42020,'AN' from dual union all
select 42021,'AN' from dual union all
select 42022,'AN' from dual union all
select 42023,'AN' from dual union all
select 42024,'AN' from dual union all
select 42025,'AN' from dual union all
select 42026,'AN' from dual union all
select 42027,'AN' from dual union all
select 42028,'MRN' from dual union all
select 42029,'AN' from dual union all
select 4203,'UNRV' from dual union all
select 42030,'MRN' from dual union all
select 42031,'AN' from dual union all
select 42032,'AN' from dual union all
select 42033,'MRN' from dual union all
select 42034,'MRN' from dual union all
select 42035,'MRN' from dual union all
select 42036,'AN' from dual union all
select 42037,'MRN' from dual union all
select 42038,'AN' from dual union all
select 42039,'AN' from dual union all
select 4204,'UNRV' from dual union all
select 42040,'AN' from dual union all
select 42041,'AN' from dual union all
select 42042,'AN' from dual union all
select 42043,'AN' from dual union all
select 42044,'AN' from dual union all
select 42045,'MRN' from dual union all
select 42046,'AN' from dual union all
select 42047,'AN' from dual union all
select 42048,'MRN' from dual union all
select 42049,'AN' from dual union all
select 4205,'UNRV' from dual union all
select 42050,'MRN' from dual union all
select 42051,'AN' from dual union all
select 42052,'AN' from dual union all
select 42053,'AN' from dual union all
select 42054,'AN' from dual union all
select 42055,'AN' from dual union all
select 42056,'AN' from dual union all
select 42057,'AN' from dual union all
select 42058,'AN' from dual union all
select 42059,'AN' from dual union all
select 4206,'UNRV' from dual union all
select 42060,'AN' from dual union all
select 42061,'AN' from dual union all
select 42062,'AN' from dual union all
select 42063,'AN' from dual union all
select 42064,'MRN' from dual union all
select 42065,'MRN' from dual union all
select 42066,'MRN' from dual union all
select 42067,'AN' from dual union all
select 42068,'AN' from dual union all
select 42069,'AN' from dual union all
select 4207,'UNRV' from dual union all
select 42070,'AN' from dual union all
select 42071,'AN' from dual union all
select 42072,'AN' from dual union all
select 42073,'AN' from dual union all
select 42074,'AN' from dual union all
select 42075,'AN' from dual union all
select 42076,'AN' from dual union all
select 42077,'AN' from dual union all
select 42078,'AN' from dual union all
select 42079,'AN' from dual union all
select 4208,'UNRV' from dual union all
select 42080,'AN' from dual union all
select 42081,'AN' from dual union all
select 42082,'AN' from dual union all
select 42083,'MRN' from dual union all
select 42084,'MRN' from dual union all
select 42085,'AN' from dual union all
select 42086,'AN' from dual union all
select 42087,'AN' from dual union all
select 42088,'AN' from dual union all
select 42089,'AN' from dual union all
select 4209,'UNRV' from dual;
insert into LAUNCH_AGENCY
select 42090,'AN' from dual union all
select 42091,'AN' from dual union all
select 42092,'AN' from dual union all
select 42093,'AN' from dual union all
select 42094,'AN' from dual union all
select 42095,'MRN' from dual union all
select 42096,'AN' from dual union all
select 42097,'AN' from dual union all
select 42098,'AN' from dual union all
select 42099,'MRN' from dual union all
select 421,'WEHR' from dual union all
select 4210,'UNRV' from dual union all
select 42100,'AN' from dual union all
select 42101,'AN' from dual union all
select 42102,'AN' from dual union all
select 42103,'AN' from dual union all
select 42104,'AN' from dual union all
select 42105,'AN' from dual union all
select 42106,'AN' from dual union all
select 42107,'AN' from dual union all
select 42108,'MRN' from dual union all
select 42109,'AN' from dual union all
select 4211,'MVS' from dual union all
select 42110,'MRN' from dual union all
select 42111,'MRN' from dual union all
select 42112,'AN' from dual union all
select 42113,'AN' from dual union all
select 42114,'AN' from dual union all
select 42115,'AN' from dual union all
select 42116,'AN' from dual union all
select 42117,'AN' from dual union all
select 42118,'AN' from dual union all
select 42119,'AN' from dual union all
select 4212,'UNRV' from dual union all
select 42120,'AN' from dual union all
select 42121,'AN' from dual union all
select 42122,'AN' from dual union all
select 42123,'AN' from dual union all
select 42124,'AN' from dual union all
select 42125,'AN' from dual union all
select 42126,'MRN' from dual union all
select 42127,'MRN' from dual union all
select 42128,'AN' from dual union all
select 42129,'RVSN' from dual union all
select 4213,'UNRV' from dual union all
select 42130,'RVSN' from dual union all
select 42131,'RVSN' from dual union all
select 42132,'RVSN' from dual union all
select 42133,'NASA' from dual union all
select 42134,'RVSN' from dual union all
select 42135,'USAF' from dual union all
select 42136,'RVSN' from dual union all
select 42137,'RVSN' from dual union all
select 42138,'RVSN' from dual union all
select 42139,'RVSN' from dual union all
select 4214,'UNRV' from dual union all
select 42140,'ISAS' from dual union all
select 42141,'RVSN' from dual union all
select 42142,'NASDA' from dual union all
select 42143,'RVSN' from dual union all
select 42144,'RVSN' from dual union all
select 42145,'RVSN' from dual union all
select 42146,'NASA' from dual union all
select 42147,'USAF' from dual union all
select 42148,'RVSN' from dual union all
select 42149,'RVSN' from dual union all
select 4215,'UNRV' from dual union all
select 42150,'RVSN' from dual union all
select 42151,'RVSN' from dual union all
select 42152,'RVSN' from dual union all
select 42153,'RVSN' from dual union all
select 42154,'RVSN' from dual union all
select 42155,'RVSN' from dual union all
select 42156,'RVSN' from dual union all
select 42157,'NASA' from dual union all
select 42158,'RVSN' from dual union all
select 42159,'RVSN' from dual union all
select 4216,'UNRV' from dual union all
select 42160,'RVSN' from dual union all
select 42161,'RVSN' from dual union all
select 42162,'USAF' from dual union all
select 42163,'RVSN' from dual union all
select 42164,'RVSN' from dual union all
select 42165,'RVSN' from dual union all
select 42166,'AFSC' from dual union all
select 42167,'RVSN' from dual union all
select 42168,'RVSN' from dual union all
select 42169,'NASA' from dual union all
select 4217,'UNRV' from dual union all
select 42170,'RVSN' from dual union all
select 42171,'RVSN' from dual union all
select 42172,'10ADS' from dual union all
select 42173,'RVSN' from dual union all
select 42174,'RVSN' from dual union all
select 42175,'RVSN' from dual union all
select 42176,'NASA' from dual union all
select 42177,'RVSN' from dual union all
select 42178,'RVSN' from dual union all
select 42179,'RVSN' from dual union all
select 4218,'UNRV' from dual;
insert into LAUNCH_AGENCY
select 42180,'RVSN' from dual union all
select 42181,'AFSC' from dual union all
select 42182,'RVSN' from dual union all
select 42183,'RVSN' from dual union all
select 42184,'USAF' from dual union all
select 42185,'RVSN' from dual union all
select 42186,'RVSN' from dual union all
select 42187,'RVSN' from dual union all
select 42188,'RVSN' from dual union all
select 42189,'RVSN' from dual union all
select 4219,'6555A' from dual union all
select 42190,'RVSN' from dual union all
select 42191,'RVSN' from dual union all
select 42192,'RVSN' from dual union all
select 42193,'NASA' from dual union all
select 42194,'RVSN' from dual union all
select 42195,'RVSN' from dual union all
select 42196,'RVSN' from dual union all
select 42197,'RVSN' from dual union all
select 42198,'RVSN' from dual union all
select 42199,'RVSN' from dual union all
select 422,'WER485-2' from dual union all
select 4220,'USAF' from dual union all
select 42200,'RVSN' from dual union all
select 42201,'RVSN' from dual union all
select 42202,'RVSN' from dual union all
select 42203,'NASA' from dual union all
select 42204,'USAF' from dual union all
select 42205,'RVSN' from dual union all
select 42206,'RVSN' from dual union all
select 42207,'RVSN' from dual union all
select 42208,'NASA' from dual union all
select 42209,'RVSN' from dual union all
select 4221,'USAF' from dual union all
select 42210,'RVSN' from dual union all
select 42211,'RVSN' from dual union all
select 42212,'USAF' from dual union all
select 42213,'RVSN' from dual union all
select 42214,'RVSN' from dual union all
select 42215,'RVSN' from dual union all
select 42216,'RVSN' from dual union all
select 42217,'RVSN' from dual union all
select 42218,'RVSN' from dual union all
select 42219,'RVSN' from dual union all
select 4222,'USA' from dual union all
select 42220,'RVSN' from dual union all
select 42221,'RVSN' from dual union all
select 42222,'USAF' from dual union all
select 42223,'RVSN' from dual union all
select 42224,'RVSN' from dual union all
select 42225,'RVSN' from dual union all
select 42226,'RVSN' from dual union all
select 42227,'RVSN' from dual union all
select 42228,'RVSN' from dual union all
select 42229,'RVSN' from dual union all
select 4223,'MVS' from dual union all
select 42230,'NASA' from dual union all
select 42231,'RVSN' from dual union all
select 42232,'RVSN' from dual union all
select 42233,'RVSN' from dual union all
select 42234,'NASA' from dual union all
select 42235,'RVSN' from dual union all
select 42236,'NASA' from dual union all
select 42237,'RVSN' from dual union all
select 42238,'RVSN' from dual union all
select 42239,'RVSN' from dual union all
select 4224,'UNRV' from dual union all
select 42240,'AFSC' from dual union all
select 42241,'RVSN' from dual union all
select 42242,'AFSC' from dual union all
select 42243,'RVSN' from dual union all
select 42244,'RVSN' from dual union all
select 42245,'RVSN' from dual union all
select 42246,'NASA' from dual union all
select 42247,'RVSN' from dual union all
select 42248,'RVSN' from dual union all
select 42249,'RVSN' from dual union all
select 4225,'USAF' from dual union all
select 42250,'RVSN' from dual union all
select 42251,'RVSN' from dual union all
select 42252,'RVSN' from dual union all
select 42253,'MRN' from dual union all
select 42254,'MRN' from dual union all
select 42255,'MRN' from dual union all
select 42256,'MRN' from dual union all
select 42257,'NASA' from dual union all
select 42258,'MRN' from dual union all
select 42259,'MRN' from dual union all
select 4226,'UNRV' from dual union all
select 42260,'USN' from dual union all
select 42261,'SRC' from dual union all
select 42262,'AN' from dual union all
select 42263,'MRN' from dual union all
select 42264,'MRN' from dual union all
select 42265,'41FA1' from dual union all
select 42266,'41FA1' from dual union all
select 42267,'MRN' from dual union all
select 42268,'MRN' from dual union all
select 42269,'MRN' from dual union all
select 4227,'6555A' from dual;
insert into LAUNCH_AGENCY
select 42270,'USN' from dual union all
select 42271,'SAC' from dual union all
select 42272,'MRN' from dual union all
select 42273,'NASA' from dual union all
select 42274,'MRN' from dual union all
select 42275,'MRN' from dual union all
select 42276,'USN' from dual union all
select 42277,'MRN' from dual union all
select 42278,'MRN' from dual union all
select 42279,'MRN' from dual union all
select 4228,'ATEF' from dual union all
select 42280,'MRN' from dual union all
select 42281,'OTR' from dual union all
select 42282,'NASA' from dual union all
select 42283,'MRN' from dual union all
select 42284,'MRN' from dual union all
select 42285,'MRN' from dual union all
select 42286,'MRN' from dual union all
select 42287,'MRN' from dual union all
select 42288,'USN' from dual union all
select 42289,'USN' from dual union all
select 4229,'ATEF' from dual union all
select 42290,'MRN' from dual union all
select 42291,'MRN' from dual union all
select 42292,'MRN' from dual union all
select 42293,'MRN' from dual union all
select 42294,'MRN' from dual union all
select 42295,'MRN' from dual union all
select 42296,'MRN' from dual union all
select 42297,'MRN' from dual union all
select 42298,'MRN' from dual union all
select 42299,'MRN' from dual union all
select 423,'WER444' from dual union all
select 4230,'USAF' from dual union all
select 42300,'UAF' from dual union all
select 42301,'USN' from dual union all
select 42302,'NRCC' from dual union all
select 42303,'SAC' from dual union all
select 42304,'MRN' from dual union all
select 42305,'ISRO' from dual union all
select 42306,'WRE' from dual union all
select 42307,'WRE' from dual union all
select 42308,'WRE' from dual union all
select 42309,'WRE' from dual union all
select 4231,'NRL' from dual union all
select 42310,'BWEHR' from dual union all
select 42311,'RVSN' from dual union all
select 42312,'RVSN' from dual union all
select 42313,'RVSN' from dual union all
select 42314,'NASA' from dual union all
select 42315,'NASA' from dual union all
select 42316,'RVSN' from dual union all
select 42317,'NASA' from dual union all
select 42318,'MRN' from dual union all
select 42319,'MRN' from dual union all
select 4232,'NII88A' from dual union all
select 42320,'MRN' from dual union all
select 42321,'MRN' from dual union all
select 42322,'MRN' from dual union all
select 42323,'MRN' from dual union all
select 42324,'MRN' from dual union all
select 42325,'MRN' from dual union all
select 42326,'MRN' from dual union all
select 42327,'MRN' from dual union all
select 42328,'MRN' from dual union all
select 42329,'MRN' from dual union all
select 4233,'MVS' from dual union all
select 42330,'MRN' from dual union all
select 42331,'AN' from dual union all
select 42332,'MRN' from dual union all
select 42333,'MRN' from dual union all
select 42334,'MRN' from dual union all
select 42335,'MRN' from dual union all
select 42336,'MRN' from dual union all
select 42337,'MRN' from dual union all
select 42338,'MRN' from dual union all
select 42339,'MRN' from dual union all
select 4234,'USA' from dual union all
select 42340,'MRN' from dual union all
select 42341,'MRN' from dual union all
select 42342,'BWEHR' from dual union all
select 42343,'PRO' from dual union all
select 42344,'RVSN' from dual union all
select 42345,'RVSN' from dual union all
select 42346,'RVSN' from dual union all
select 42347,'RVSN' from dual union all
select 42348,'RVSN' from dual union all
select 42349,'RVSN' from dual union all
select 4235,'USAF' from dual union all
select 42350,'RVSN' from dual union all
select 42351,'RVSN' from dual union all
select 42352,'RVSN' from dual union all
select 42353,'AN' from dual union all
select 42354,'DNA' from dual union all
select 42355,'AN' from dual union all
select 42356,'NRCC' from dual union all
select 42357,'NASA' from dual union all
select 42358,'NASA' from dual union all
select 42359,'NASA' from dual union all
select 4236,'MVS' from dual;
insert into LAUNCH_AGENCY
select 42360,'DNA' from dual union all
select 42361,'AN' from dual union all
select 42362,'DNA' from dual union all
select 42363,'RVSN' from dual union all
select 42364,'DFVLR' from dual union all
select 42365,'DMA' from dual union all
select 42366,'AN' from dual union all
select 42367,'AN' from dual union all
select 42368,'DFVLR' from dual union all
select 42369,'RVSN' from dual union all
select 4237,'NII88A' from dual union all
select 42370,'AN' from dual union all
select 42371,'VMF' from dual union all
select 42372,'AN' from dual union all
select 42373,'AN' from dual union all
select 42374,'RVSN' from dual union all
select 42375,'NASA' from dual union all
select 42376,'VMF' from dual union all
select 42377,'USAF' from dual union all
select 42378,'DLR' from dual union all
select 42379,'NRCC' from dual union all
select 4238,'PARD' from dual union all
select 42380,'NASA' from dual union all
select 42381,'AN' from dual union all
select 42382,'USN' from dual union all
select 42383,'USN' from dual union all
select 42384,'RVSN' from dual union all
select 42385,'RVSN' from dual union all
select 42386,'NIPR' from dual union all
select 42387,'USN' from dual union all
select 42388,'AN' from dual union all
select 42389,'AN' from dual union all
select 4239,'NII88A' from dual union all
select 42390,'AN' from dual union all
select 42391,'AN' from dual union all
select 42392,'AN' from dual union all
select 42393,'USN' from dual union all
select 42394,'USN' from dual union all
select 42395,'USA' from dual union all
select 42396,'RVSN' from dual union all
select 42397,'USA' from dual union all
select 42398,'IAE' from dual union all
select 42399,'AN' from dual union all
select 424,'WER444' from dual union all
select 4240,'USAF' from dual union all
select 42400,'AN' from dual union all
select 42401,'NASA' from dual union all
select 42402,'NIPR' from dual union all
select 42403,'RVSN' from dual union all
select 42404,'DMA' from dual union all
select 42405,'NASA' from dual union all
select 42406,'AFGL' from dual union all
select 42407,'RVSN' from dual union all
select 42408,'BAC' from dual union all
select 42409,'USN' from dual union all
select 4241,'MVS' from dual union all
select 42410,'AN' from dual union all
select 42411,'AN' from dual union all
select 42412,'USA' from dual union all
select 42413,'FKG1' from dual union all
select 42414,'FKG1' from dual union all
select 42415,'FKG' from dual union all
select 42416,'FKG' from dual union all
select 42417,'US7A' from dual union all
select 42418,'RVSN' from dual union all
select 42419,'US7A' from dual union all
select 4242,'USAF' from dual union all
select 42420,'US7A' from dual union all
select 42421,'US7A' from dual union all
select 42422,'RVSN' from dual union all
select 42423,'AFSC' from dual union all
select 42424,'AFGL' from dual union all
select 42425,'RVSN' from dual union all
select 42426,'NASA' from dual union all
select 42427,'RVSN' from dual union all
select 42428,'USN' from dual union all
select 42429,'USN' from dual union all
select 4243,'USAF' from dual union all
select 42430,'USN' from dual union all
select 42431,'SAC' from dual union all
select 42432,'RVSN' from dual union all
select 42433,'AN' from dual union all
select 42434,'AN' from dual union all
select 42435,'AN' from dual union all
select 42436,'AN' from dual union all
select 42437,'AN' from dual union all
select 42438,'RVSN' from dual union all
select 42439,'AN' from dual union all
select 4244,'6555A' from dual union all
select 42440,'AN' from dual union all
select 42441,'AN' from dual union all
select 42442,'RVSN' from dual union all
select 42443,'SAC' from dual union all
select 42444,'USN' from dual union all
select 42445,'USN' from dual union all
select 42446,'DNA' from dual union all
select 42447,'BAC' from dual union all
select 42448,'NASA' from dual union all
select 42449,'DFVLR' from dual union all
select 4245,'USAF' from dual;
insert into LAUNCH_AGENCY
select 42450,'AFSC' from dual union all
select 42451,'USN' from dual union all
select 42452,'VMF' from dual union all
select 42453,'USN' from dual union all
select 42454,'USN' from dual union all
select 42455,'USN' from dual union all
select 42456,'USN' from dual union all
select 42457,'NASA' from dual union all
select 42458,'USN' from dual union all
select 42459,'SRC' from dual union all
select 4246,'NRL' from dual union all
select 42460,'SRC' from dual union all
select 42461,'ALCS' from dual union all
select 42461,'SAC' from dual union all
select 42462,'AN' from dual union all
select 42463,'RVSN' from dual union all
select 42464,'AN' from dual union all
select 42465,'RVSN' from dual union all
select 42466,'DMA' from dual union all
select 42467,'NASA' from dual union all
select 42468,'NIPR' from dual union all
select 42469,'RN' from dual union all
select 4247,'MVS' from dual union all
select 42470,'NASA' from dual union all
select 42471,'NASA' from dual union all
select 42472,'NIPR' from dual union all
select 42473,'NASA' from dual union all
select 42474,'ISRO' from dual union all
select 42475,'NASA' from dual union all
select 42476,'PRO' from dual union all
select 42477,'SAMSO' from dual union all
select 42478,'RVSN' from dual union all
select 42479,'SAC' from dual union all
select 4248,'UNRV' from dual union all
select 42480,'USN' from dual union all
select 42481,'AFGL2' from dual union all
select 42482,'SAC' from dual union all
select 42483,'NASA' from dual union all
select 42484,'NIPR' from dual union all
select 42485,'RVSN' from dual union all
select 42486,'ISAS' from dual union all
select 42487,'NASA' from dual union all
select 42488,'USN' from dual union all
select 42489,'NASA' from dual union all
select 4249,'NRL' from dual union all
select 42490,'SAC' from dual union all
select 42491,'NASDA' from dual union all
select 42492,'AN' from dual union all
select 42493,'USA' from dual union all
select 42494,'LASL' from dual union all
select 42495,'AFSC' from dual union all
select 42496,'RVSN' from dual union all
select 42497,'ISAS' from dual union all
select 42498,'USN' from dual union all
select 42499,'USN' from dual union all
select 425,'WER444' from dual union all
select 4250,'USAF' from dual union all
select 42500,'SAND' from dual union all
select 42501,'LASL' from dual union all
select 42502,'RN' from dual union all
select 42503,'SAST' from dual union all
select 42504,'AN' from dual union all
select 42505,'SAC' from dual union all
select 42506,'ISAS' from dual union all
select 42507,'RVSN' from dual union all
select 42508,'ISAS' from dual union all
select 42509,'ISAS' from dual union all
select 4251,'6555A' from dual union all
select 42510,'AFGL' from dual union all
select 42511,'AFGL2' from dual union all
select 42512,'DNA' from dual union all
select 42513,'NASA' from dual union all
select 42514,'NASA' from dual union all
select 42515,'AN' from dual union all
select 42516,'DFVLR' from dual union all
select 42517,'BAC' from dual union all
select 42518,'RVSN' from dual union all
select 42519,'SRC' from dual union all
select 4252,'NRL' from dual union all
select 42520,'SRC' from dual union all
select 42521,'IAE' from dual union all
select 42522,'USN' from dual union all
select 42523,'SSC' from dual union all
select 42524,'RVSN' from dual union all
select 42525,'RVSN' from dual union all
select 42526,'RVSN' from dual union all
select 42527,'AN' from dual union all
select 42528,'AN' from dual union all
select 42529,'RVSN' from dual union all
select 4253,'NRL' from dual union all
select 42530,'9FA3' from dual union all
select 42531,'9FA3' from dual union all
select 42532,'VMF' from dual union all
select 42533,'VMF' from dual union all
select 42534,'VMF' from dual union all
select 42535,'RVSN' from dual union all
select 42536,'DFVLR' from dual union all
select 42537,'AFSC' from dual union all
select 42538,'RN' from dual union all
select 42539,'AFGL2' from dual;
insert into LAUNCH_AGENCY
select 4254,'USA' from dual union all
select 42540,'RVSN' from dual union all
select 42541,'AFSC' from dual union all
select 42542,'DNA' from dual union all
select 42543,'ARPA' from dual union all
select 42544,'DMA' from dual union all
select 42545,'BAC' from dual union all
select 42546,'SRC' from dual union all
select 42547,'9FA3' from dual union all
select 42548,'RVSN' from dual union all
select 42549,'SAC' from dual union all
select 4255,'NRL' from dual union all
select 42550,'RVSN' from dual union all
select 42551,'VMF' from dual union all
select 42552,'RVSN' from dual union all
select 42553,'NASA' from dual union all
select 42554,'SRC' from dual union all
select 42555,'USN' from dual union all
select 42556,'SAC' from dual union all
select 42557,'NRCC' from dual union all
select 42558,'NASA' from dual union all
select 42559,'NASA' from dual union all
select 4256,'MVS' from dual union all
select 42560,'RVSN' from dual union all
select 42561,'NASA' from dual union all
select 42562,'INTA' from dual union all
select 42563,'AFGL' from dual union all
select 42564,'DFVLR' from dual union all
select 42565,'NASA' from dual union all
select 42566,'INTA' from dual union all
select 42567,'RVSN' from dual union all
select 42568,'RVSN' from dual union all
select 42569,'RVSN' from dual union all
select 4257,'NRL' from dual union all
select 42570,'ISRO' from dual union all
select 42571,'RVSN' from dual union all
select 42572,'AFGL' from dual union all
select 42573,'AFGL' from dual union all
select 42574,'AFGL' from dual union all
select 42575,'AFGL' from dual union all
select 42576,'AFGL' from dual union all
select 42577,'DNA' from dual union all
select 42577,'NRL' from dual union all
select 42578,'PLA2AC' from dual union all
select 42579,'PLA2AC' from dual union all
select 4258,'NRL' from dual union all
select 42580,'IQ224' from dual union all
select 42581,'IQ224' from dual union all
select 42582,'RVSN' from dual union all
select 42583,'IQ224' from dual union all
select 42584,'IQ224' from dual union all
select 42585,'IQ224' from dual union all
select 42586,'IQ224' from dual union all
select 42587,'DNA' from dual union all
select 42588,'DNA' from dual union all
select 42589,'DNA' from dual union all
select 4259,'USAF' from dual union all
select 42590,'DNA' from dual union all
select 42591,'DNA' from dual union all
select 42592,'DNA' from dual union all
select 42593,'RVSN' from dual union all
select 42594,'RVSN' from dual union all
select 42595,'RVSN' from dual union all
select 42596,'RVSN' from dual union all
select 42597,'AN' from dual union all
select 42598,'AN' from dual union all
select 42599,'AN' from dual union all
select 426,'WER485-2' from dual union all
select 4260,'UNRV' from dual union all
select 42600,'AN' from dual union all
select 42601,'AN' from dual union all
select 42602,'RVSN' from dual union all
select 42603,'RVSN' from dual union all
select 42604,'RVSN' from dual union all
select 42605,'RVSN' from dual union all
select 42606,'RVSN' from dual union all
select 42607,'RVSN' from dual union all
select 42608,'RVSN' from dual union all
select 42609,'RVSN' from dual union all
select 4261,'NRL' from dual union all
select 42610,'RVSN' from dual union all
select 42611,'RVSN' from dual union all
select 42612,'RVSN' from dual union all
select 42613,'RVSN' from dual union all
select 42614,'NRCC' from dual union all
select 42615,'RVSN' from dual union all
select 42616,'RVSN' from dual union all
select 42617,'RVSN' from dual union all
select 42618,'RVSN' from dual union all
select 42619,'RVSN' from dual union all
select 4262,'UNRV' from dual union all
select 42620,'RVSN' from dual union all
select 42621,'RVSN' from dual union all
select 42622,'RVSN' from dual union all
select 42623,'RVSN' from dual union all
select 42624,'RVSN' from dual union all
select 42625,'NASA' from dual union all
select 42626,'RVSN' from dual union all
select 42627,'INTA' from dual union all
select 42628,'INTA' from dual;
insert into LAUNCH_AGENCY
select 42629,'INTA' from dual union all
select 4263,'NRL' from dual union all
select 42630,'INTA' from dual union all
select 42631,'INTA' from dual union all
select 42632,'INTA' from dual union all
select 42633,'INTA' from dual union all
select 42634,'INTA' from dual union all
select 42635,'INTA' from dual union all
select 42636,'NASA' from dual union all
select 42637,'INTA' from dual union all
select 42638,'INTA' from dual union all
select 42639,'INTA' from dual union all
select 4264,'NRL' from dual union all
select 42640,'INTA' from dual union all
select 42641,'INTA' from dual union all
select 42642,'INTA' from dual union all
select 42643,'INTA' from dual union all
select 42644,'INTA' from dual union all
select 42645,'INTA' from dual union all
select 42646,'INTA' from dual union all
select 42647,'NRCC' from dual union all
select 42648,'INTA' from dual union all
select 42649,'INTA' from dual union all
select 4265,'UNRV' from dual union all
select 42650,'INTA' from dual union all
select 42651,'INTA' from dual union all
select 42652,'INTA' from dual union all
select 42653,'INTA' from dual union all
select 42654,'INTA' from dual union all
select 42655,'INTA' from dual union all
select 42656,'INTA' from dual union all
select 42657,'INTA' from dual union all
select 42658,'NASA' from dual union all
select 42659,'INTA' from dual union all
select 4266,'MVS' from dual union all
select 42660,'INTA' from dual union all
select 42661,'INTA' from dual union all
select 42662,'INTA' from dual union all
select 42663,'INTA' from dual union all
select 42664,'INTA' from dual union all
select 42665,'INTA' from dual union all
select 42666,'INTA' from dual union all
select 42667,'INTA' from dual union all
select 42668,'INTA' from dual union all
select 42669,'SSC' from dual union all
select 4267,'MVS' from dual union all
select 42670,'INTA' from dual union all
select 42671,'INTA' from dual union all
select 42672,'INTA' from dual union all
select 42673,'INTA' from dual union all
select 42674,'INTA' from dual union all
select 42675,'INTA' from dual union all
select 42676,'INTA' from dual union all
select 42677,'INTA' from dual union all
select 42678,'INTA' from dual union all
select 42679,'INTA' from dual union all
select 4268,'USAF' from dual union all
select 42680,'NASA' from dual union all
select 42681,'INTA' from dual union all
select 42682,'INTA' from dual union all
select 42683,'INTA' from dual union all
select 42684,'INTA' from dual union all
select 42685,'INTA' from dual union all
select 42686,'INTA' from dual union all
select 42687,'INTA' from dual union all
select 42688,'AN' from dual union all
select 42688,'ISRO' from dual union all
select 42689,'AN' from dual union all
select 42689,'ISRO' from dual union all
select 4269,'USA' from dual union all
select 42690,'IIAE' from dual union all
select 42691,'NASA' from dual union all
select 42692,'AN' from dual union all
select 42693,'AN' from dual union all
select 42694,'AN' from dual union all
select 42695,'MRN' from dual union all
select 42696,'AN' from dual union all
select 42697,'AN' from dual union all
select 42698,'AN' from dual union all
select 42699,'AN' from dual union all
select 427,'WER485-2' from dual union all
select 4270,'USAF' from dual union all
select 42700,'AN' from dual union all
select 42701,'AN' from dual union all
select 42702,'ISAS' from dual union all
select 42703,'AN' from dual union all
select 42704,'AN' from dual union all
select 42705,'AN' from dual union all
select 42706,'AN' from dual union all
select 42707,'AN' from dual union all
select 42708,'MRN' from dual union all
select 42709,'MRN' from dual union all
select 4271,'USA' from dual union all
select 42710,'MRN' from dual union all
select 42711,'MRN' from dual union all
select 42712,'MRN' from dual union all
select 42713,'USN' from dual union all
select 42714,'MVS' from dual union all
select 42715,'MVS' from dual union all
select 42716,'MVS' from dual;
insert into LAUNCH_AGENCY
select 42717,'RVSN' from dual union all
select 42718,'RVSN' from dual union all
select 42719,'RVSN' from dual union all
select 4272,'BRL' from dual union all
select 42720,'RVSN' from dual union all
select 42721,'RVSN' from dual union all
select 42722,'RVSN' from dual union all
select 42723,'RVSN' from dual union all
select 42724,'SAC' from dual union all
select 42725,'RVSN' from dual union all
select 42726,'DFVLR' from dual union all
select 42727,'NASA' from dual union all
select 42728,'NASDA' from dual union all
select 42729,'NRL' from dual union all
select 4273,'USA' from dual union all
select 42730,'AN' from dual union all
select 42731,'USN' from dual union all
select 42732,'USN' from dual union all
select 42733,'USN' from dual union all
select 42734,'USN' from dual union all
select 42735,'AFSC' from dual union all
select 42736,'SAC' from dual union all
select 42737,'RVSN' from dual union all
select 42738,'USN' from dual union all
select 42739,'NASA' from dual union all
select 4274,'MVS' from dual union all
select 42740,'NASA' from dual union all
select 42741,'UKMO' from dual union all
select 42742,'SRC' from dual union all
select 42743,'SRC' from dual union all
select 42744,'SSC' from dual union all
select 42745,'DFVLR' from dual union all
select 42746,'AN' from dual union all
select 42747,'NIPR' from dual union all
select 42748,'AN' from dual union all
select 42749,'NASA' from dual union all
select 4275,'6555A' from dual union all
select 42750,'SRC' from dual union all
select 42751,'SRC' from dual union all
select 42752,'SRC' from dual union all
select 42753,'SSC' from dual union all
select 42754,'AN' from dual union all
select 42755,'NASA' from dual union all
select 42756,'NASA' from dual union all
select 42757,'NASA' from dual union all
select 42758,'SSC' from dual union all
select 42759,'USN' from dual union all
select 4276,'6555A' from dual union all
select 42760,'SAC' from dual union all
select 42761,'NASA' from dual union all
select 42762,'NASA' from dual union all
select 42763,'NASA' from dual union all
select 42764,'NASA' from dual union all
select 42765,'84FA3' from dual union all
select 42766,'NASA' from dual union all
select 42767,'DFVLR' from dual union all
select 42768,'AN' from dual union all
select 42769,'NASA' from dual union all
select 4277,'6555A' from dual union all
select 42770,'NASA' from dual union all
select 42771,'NASA' from dual union all
select 42772,'AN' from dual union all
select 42773,'DNA' from dual union all
select 42774,'NASA' from dual union all
select 42775,'DNA' from dual union all
select 42776,'SAC' from dual union all
select 42777,'41FA1' from dual union all
select 42778,'41FA1' from dual union all
select 42779,'NRCC' from dual union all
select 4278,'USA' from dual union all
select 42780,'MRN' from dual union all
select 42781,'MRN' from dual union all
select 42782,'MRN' from dual union all
select 42783,'MRN' from dual union all
select 42784,'MRN' from dual union all
select 42785,'MRN' from dual union all
select 42786,'MRN' from dual union all
select 42787,'MRN' from dual union all
select 42788,'MRN' from dual union all
select 42789,'MRN' from dual union all
select 4279,'NSA' from dual union all
select 4279,'PARD' from dual union all
select 42790,'MRN' from dual union all
select 42791,'GMS' from dual union all
select 42792,'MRN' from dual union all
select 42793,'MRN' from dual union all
select 42794,'MRN' from dual union all
select 42795,'MRN' from dual union all
select 42796,'MRN' from dual union all
select 42797,'MRN' from dual union all
select 42798,'MRN' from dual union all
select 42799,'MRN' from dual union all
select 428,'WER485-2' from dual union all
select 4280,'6555A' from dual union all
select 42800,'MRN' from dual union all
select 42801,'MRN' from dual union all
select 42802,'MRN' from dual union all
select 42803,'NASA' from dual union all
select 42804,'MRN' from dual union all
select 42805,'MRN' from dual;
insert into LAUNCH_AGENCY
select 42806,'MRN' from dual union all
select 42807,'MRN' from dual union all
select 42808,'MRN' from dual union all
select 42809,'MRN' from dual union all
select 4281,'6555A' from dual union all
select 42810,'MRN' from dual union all
select 42811,'MRN' from dual union all
select 42812,'MRN' from dual union all
select 42813,'MRN' from dual union all
select 42814,'MRN' from dual union all
select 42815,'MRN' from dual union all
select 42816,'MRN' from dual union all
select 42817,'GMS' from dual union all
select 42818,'MRN' from dual union all
select 42819,'MRN' from dual union all
select 4282,'6555A' from dual union all
select 42820,'MRN' from dual union all
select 42821,'MRN' from dual union all
select 42822,'MRN' from dual union all
select 42823,'MRN' from dual union all
select 42824,'MRN' from dual union all
select 42825,'MRN' from dual union all
select 42826,'MRN' from dual union all
select 42827,'MRN' from dual union all
select 42828,'MRN' from dual union all
select 42829,'MRN' from dual union all
select 4283,'6555A' from dual union all
select 42830,'MRN' from dual union all
select 42831,'MRN' from dual union all
select 42832,'MRN' from dual union all
select 42833,'MRN' from dual union all
select 42834,'MRN' from dual union all
select 42835,'MRN' from dual union all
select 42836,'MRN' from dual union all
select 42837,'MRN' from dual union all
select 42838,'GMS' from dual union all
select 42839,'MRN' from dual union all
select 4284,'USA' from dual union all
select 42840,'MRN' from dual union all
select 42841,'MRN' from dual union all
select 42842,'MRN' from dual union all
select 42843,'GMS' from dual union all
select 42844,'MRN' from dual union all
select 42845,'MRN' from dual union all
select 42846,'MRN' from dual union all
select 42847,'MRN' from dual union all
select 42848,'MRN' from dual union all
select 42849,'MRN' from dual union all
select 4285,'BRL' from dual union all
select 42850,'GMS' from dual union all
select 42851,'NASA' from dual union all
select 42852,'MRN' from dual union all
select 42853,'NASA' from dual union all
select 42854,'MRN' from dual union all
select 42855,'MRN' from dual union all
select 42856,'MRN' from dual union all
select 42857,'MRN' from dual union all
select 42858,'MRN' from dual union all
select 42859,'MRN' from dual union all
select 4286,'USA' from dual union all
select 42860,'MRN' from dual union all
select 42861,'MRN' from dual union all
select 42862,'MRN' from dual union all
select 42863,'MRN' from dual union all
select 42864,'MRN' from dual union all
select 42865,'MRN' from dual union all
select 42866,'MRN' from dual union all
select 42867,'MRN' from dual union all
select 42868,'MRN' from dual union all
select 42869,'MRN' from dual union all
select 4287,'6555A' from dual union all
select 42870,'MRN' from dual union all
select 42871,'MRN' from dual union all
select 42872,'GMS' from dual union all
select 42873,'MRN' from dual union all
select 42874,'MRN' from dual union all
select 42875,'MRN' from dual union all
select 42876,'MRN' from dual union all
select 42877,'MRN' from dual union all
select 42878,'MRN' from dual union all
select 42879,'MRN' from dual union all
select 4288,'USAF' from dual union all
select 42880,'MRN' from dual union all
select 42881,'MRN' from dual union all
select 42882,'MRN' from dual union all
select 42883,'MRN' from dual union all
select 42884,'MRN' from dual union all
select 42885,'MRN' from dual union all
select 42886,'MRN' from dual union all
select 42887,'MRN' from dual union all
select 42888,'GMS' from dual union all
select 42889,'MRN' from dual union all
select 4289,'UNRV' from dual union all
select 42890,'GMS' from dual union all
select 42891,'MRN' from dual union all
select 42892,'MRN' from dual union all
select 42893,'MRN' from dual union all
select 42894,'MRN' from dual union all
select 42895,'AN' from dual union all
select 42896,'MRN' from dual;
insert into LAUNCH_AGENCY
select 42897,'MRN' from dual union all
select 42898,'MRN' from dual union all
select 42899,'MRN' from dual union all
select 429,'WER444' from dual union all
select 4290,'UNRV' from dual union all
select 42900,'MRN' from dual union all
select 42901,'MRN' from dual union all
select 42902,'MRN' from dual union all
select 42903,'MRN' from dual union all
select 42904,'MRN' from dual union all
select 42905,'MRN' from dual union all
select 42906,'MRN' from dual union all
select 42907,'MRN' from dual union all
select 42908,'MRN' from dual union all
select 42909,'MRN' from dual union all
select 4291,'ARDC' from dual union all
select 42910,'MRN' from dual union all
select 42911,'MRN' from dual union all
select 42912,'MRN' from dual union all
select 42913,'MRN' from dual union all
select 42914,'MRN' from dual union all
select 42915,'MRN' from dual union all
select 42916,'MRN' from dual union all
select 42917,'MRN' from dual union all
select 42918,'MRN' from dual union all
select 42919,'MRN' from dual union all
select 4292,'USAF' from dual union all
select 42920,'MRN' from dual union all
select 42921,'MRN' from dual union all
select 42922,'MRN' from dual union all
select 42923,'MRN' from dual union all
select 42924,'MRN' from dual union all
select 42925,'MRN' from dual union all
select 42926,'MRN' from dual union all
select 42927,'MRN' from dual union all
select 42928,'MRN' from dual union all
select 42929,'MRN' from dual union all
select 4293,'USAF' from dual union all
select 42930,'AN' from dual union all
select 42931,'MRN' from dual union all
select 42932,'MRN' from dual union all
select 42933,'MRN' from dual union all
select 42934,'MRN' from dual union all
select 42935,'MRN' from dual union all
select 42936,'MRN' from dual union all
select 42937,'MRN' from dual union all
select 42938,'MRN' from dual union all
select 42939,'MRN' from dual union all
select 4294,'6555A' from dual union all
select 42940,'MRN' from dual union all
select 42941,'MRN' from dual union all
select 42942,'MRN' from dual union all
select 42943,'MRN' from dual union all
select 42944,'MRN' from dual union all
select 42945,'MRN' from dual union all
select 42946,'MRN' from dual union all
select 42947,'MRN' from dual union all
select 42948,'MRN' from dual union all
select 42949,'MRN' from dual union all
select 4295,'USAF' from dual union all
select 42950,'MRN' from dual union all
select 42951,'MRN' from dual union all
select 42952,'MRN' from dual union all
select 42953,'MRN' from dual union all
select 42954,'MRN' from dual union all
select 42955,'MRN' from dual union all
select 42956,'MRN' from dual union all
select 42957,'MRN' from dual union all
select 42958,'MRN' from dual union all
select 42959,'MRN' from dual union all
select 4296,'USAF' from dual union all
select 42960,'MRN' from dual union all
select 42961,'MRN' from dual union all
select 42962,'GMS' from dual union all
select 42963,'MRN' from dual union all
select 42964,'MRN' from dual union all
select 42965,'MRN' from dual union all
select 42966,'MRN' from dual union all
select 42967,'MRN' from dual union all
select 42968,'MRN' from dual union all
select 42969,'MRN' from dual union all
select 4297,'USA' from dual union all
select 42970,'MRN' from dual union all
select 42971,'MRN' from dual union all
select 42972,'MRN' from dual union all
select 42973,'MRN' from dual union all
select 42974,'MRN' from dual union all
select 42975,'MRN' from dual union all
select 42976,'MRN' from dual union all
select 42977,'MRN' from dual union all
select 42978,'MRN' from dual union all
select 42979,'MRN' from dual union all
select 4298,'ONR' from dual union all
select 42980,'MRN' from dual union all
select 42981,'MRN' from dual union all
select 42982,'MRN' from dual union all
select 42983,'MRN' from dual union all
select 42984,'MRN' from dual union all
select 42985,'MRN' from dual union all
select 42986,'MRN' from dual;
insert into LAUNCH_AGENCY
select 42987,'MRN' from dual union all
select 42988,'MRN' from dual union all
select 42989,'MRN' from dual union all
select 4299,'JTF7' from dual union all
select 42990,'MRN' from dual union all
select 42991,'MRN' from dual union all
select 42992,'MRN' from dual union all
select 42993,'MRN' from dual union all
select 42994,'MRN' from dual union all
select 42995,'MRN' from dual union all
select 42996,'MRN' from dual union all
select 42997,'MRN' from dual union all
select 42998,'MRN' from dual union all
select 42999,'NASA' from dual union all
select 43,'WER836-3' from dual union all
select 430,'WER444' from dual union all
select 4300,'JTF7' from dual union all
select 43000,'MRN' from dual union all
select 43001,'MRN' from dual union all
select 43002,'MRN' from dual union all
select 43003,'MRN' from dual union all
select 43004,'MRN' from dual union all
select 43005,'MRN' from dual union all
select 43006,'MRN' from dual union all
select 43007,'NASA' from dual union all
select 43008,'MRN' from dual union all
select 43009,'GMS' from dual union all
select 4301,'JTF7' from dual union all
select 43010,'MRN' from dual union all
select 43011,'GMS' from dual union all
select 43012,'MRN' from dual union all
select 43013,'MRN' from dual union all
select 43014,'NASA' from dual union all
select 43015,'MRN' from dual union all
select 43016,'MRN' from dual union all
select 43017,'GMS' from dual union all
select 43018,'MRN' from dual union all
select 43019,'NASA' from dual union all
select 4302,'JTF7' from dual union all
select 43020,'MRN' from dual union all
select 43021,'MRN' from dual union all
select 43022,'NASA' from dual union all
select 43023,'MRN' from dual union all
select 43024,'MRN' from dual union all
select 43025,'NASA' from dual union all
select 43026,'NASA' from dual union all
select 43027,'MRN' from dual union all
select 43028,'MRN' from dual union all
select 43029,'MRN' from dual union all
select 4303,'SCEL' from dual union all
select 43030,'NASA' from dual union all
select 43031,'MRN' from dual union all
select 43032,'NASA' from dual union all
select 43033,'MRN' from dual union all
select 43034,'MRN' from dual union all
select 43035,'NASA' from dual union all
select 43036,'MRN' from dual union all
select 43037,'MRN' from dual union all
select 43038,'NASA' from dual union all
select 43039,'MRN' from dual union all
select 4304,'SCEL' from dual union all
select 43040,'MRN' from dual union all
select 43041,'MRN' from dual union all
select 43042,'MRN' from dual union all
select 43043,'MRN' from dual union all
select 43044,'MRN' from dual union all
select 43045,'NASA' from dual union all
select 43046,'MRN' from dual union all
select 43047,'NASA' from dual union all
select 43048,'NASA' from dual union all
select 43049,'MRN' from dual union all
select 4305,'MVS' from dual union all
select 43050,'MRN' from dual union all
select 43051,'MRN' from dual union all
select 43052,'MRN' from dual union all
select 43053,'MRN' from dual union all
select 43054,'MRN' from dual union all
select 43055,'MRN' from dual union all
select 43056,'MRN' from dual union all
select 43057,'MRN' from dual union all
select 43058,'MRN' from dual union all
select 43059,'MRN' from dual union all
select 4306,'MVS' from dual union all
select 43060,'MRN' from dual union all
select 43061,'MRN' from dual union all
select 43062,'MRN' from dual union all
select 43063,'MRN' from dual union all
select 43064,'MRN' from dual union all
select 43065,'MRN' from dual union all
select 43066,'MRN' from dual union all
select 43067,'MRN' from dual union all
select 43068,'MRN' from dual union all
select 43069,'GMS' from dual union all
select 4307,'LERC' from dual union all
select 43070,'MRN' from dual union all
select 43071,'MRN' from dual union all
select 43072,'MRN' from dual union all
select 43073,'MRN' from dual union all
select 43074,'MRN' from dual union all
select 43075,'GMS' from dual;
insert into LAUNCH_AGENCY
select 43076,'MRN' from dual union all
select 43077,'GMS' from dual union all
select 43078,'MRN' from dual union all
select 43079,'MRN' from dual union all
select 4308,'USAF' from dual union all
select 43080,'MRN' from dual union all
select 43081,'MRN' from dual union all
select 43082,'MRN' from dual union all
select 43083,'MRN' from dual union all
select 43084,'MRN' from dual union all
select 43085,'MRN' from dual union all
select 43086,'MRN' from dual union all
select 43087,'MRN' from dual union all
select 43088,'AN' from dual union all
select 43089,'MRN' from dual union all
select 4309,'USAF' from dual union all
select 43090,'MRN' from dual union all
select 43091,'MRN' from dual union all
select 43092,'GMS' from dual union all
select 43093,'MRN' from dual union all
select 43094,'GMS' from dual union all
select 43095,'MRN' from dual union all
select 43096,'MRN' from dual union all
select 43097,'MRN' from dual union all
select 43098,'MRN' from dual union all
select 43099,'MRN' from dual union all
select 431,'WER485-2' from dual union all
select 4310,'USAF' from dual union all
select 43100,'MRN' from dual union all
select 43101,'MRN' from dual union all
select 43102,'MRN' from dual union all
select 43103,'MRN' from dual union all
select 43104,'MRN' from dual union all
select 43105,'MRN' from dual union all
select 43106,'MRN' from dual union all
select 43107,'MRN' from dual union all
select 43108,'MRN' from dual union all
select 43109,'MRN' from dual union all
select 4311,'PARD' from dual union all
select 43110,'MRN' from dual union all
select 43111,'MRN' from dual union all
select 43112,'MRN' from dual union all
select 43113,'MRN' from dual union all
select 43114,'MRN' from dual union all
select 43115,'MRN' from dual union all
select 43116,'MRN' from dual union all
select 43117,'MRN' from dual union all
select 43118,'MRN' from dual union all
select 43119,'MRN' from dual union all
select 4312,'6555A' from dual union all
select 43120,'MRN' from dual union all
select 43121,'MRN' from dual union all
select 43122,'MRN' from dual union all
select 43123,'MRN' from dual union all
select 43124,'MRN' from dual union all
select 43125,'MRN' from dual union all
select 43126,'MRN' from dual union all
select 43127,'MRN' from dual union all
select 43128,'MRN' from dual union all
select 43129,'MRN' from dual union all
select 4313,'6555A' from dual union all
select 43130,'MRN' from dual union all
select 43131,'MRN' from dual union all
select 43132,'MRN' from dual union all
select 43133,'GMS' from dual union all
select 43134,'MRN' from dual union all
select 43135,'MRN' from dual union all
select 43136,'MRN' from dual union all
select 43137,'MRN' from dual union all
select 43138,'MRN' from dual union all
select 43139,'MRN' from dual union all
select 4314,'USAF' from dual union all
select 43140,'MRN' from dual union all
select 43141,'MRN' from dual union all
select 43142,'MRN' from dual union all
select 43143,'MRN' from dual union all
select 43144,'MRN' from dual union all
select 43145,'MRN' from dual union all
select 43146,'MRN' from dual union all
select 43147,'MRN' from dual union all
select 43148,'GMS' from dual union all
select 43149,'MRN' from dual union all
select 4315,'USAF' from dual union all
select 43150,'MRN' from dual union all
select 43151,'MRN' from dual union all
select 43152,'MRN' from dual union all
select 43153,'MRN' from dual union all
select 43154,'MRN' from dual union all
select 43155,'MRN' from dual union all
select 43156,'MRN' from dual union all
select 43157,'MRN' from dual union all
select 43158,'MRN' from dual union all
select 43159,'MRN' from dual union all
select 4316,'PARD' from dual union all
select 43160,'MRN' from dual union all
select 43161,'MRN' from dual union all
select 43162,'MRN' from dual union all
select 43163,'MRN' from dual union all
select 43164,'MRN' from dual union all
select 43165,'MRN' from dual;
insert into LAUNCH_AGENCY
select 43166,'MRN' from dual union all
select 43167,'MRN' from dual union all
select 43168,'MRN' from dual union all
select 43169,'AN' from dual union all
select 4317,'USN' from dual union all
select 43170,'MRN' from dual union all
select 43171,'MRN' from dual union all
select 43172,'MRN' from dual union all
select 43173,'MRN' from dual union all
select 43174,'GMS' from dual union all
select 43175,'MRN' from dual union all
select 43176,'MRN' from dual union all
select 43177,'MRN' from dual union all
select 43178,'MRN' from dual union all
select 43179,'MRN' from dual union all
select 4318,'USAF' from dual union all
select 43180,'MRN' from dual union all
select 43181,'MRN' from dual union all
select 43182,'MRN' from dual union all
select 43183,'GMS' from dual union all
select 43184,'MRN' from dual union all
select 43185,'MRN' from dual union all
select 43186,'MRN' from dual union all
select 43187,'MRN' from dual union all
select 43188,'MRN' from dual union all
select 43189,'MRN' from dual union all
select 4319,'ISAS' from dual union all
select 43190,'MRN' from dual union all
select 43191,'MRN' from dual union all
select 43192,'AN' from dual union all
select 43193,'MRN' from dual union all
select 43194,'MRN' from dual union all
select 43195,'MRN' from dual union all
select 43196,'MRN' from dual union all
select 43197,'NASA' from dual union all
select 43198,'MRN' from dual union all
select 43199,'MRN' from dual union all
select 432,'WER836-3' from dual union all
select 4320,'ISAS' from dual union all
select 43200,'MRN' from dual union all
select 43201,'NASA' from dual union all
select 43202,'MRN' from dual union all
select 43203,'MRN' from dual union all
select 43204,'GMS' from dual union all
select 43205,'MRN' from dual union all
select 43206,'MRN' from dual union all
select 43207,'MRN' from dual union all
select 43208,'MRN' from dual union all
select 43209,'MRN' from dual union all
select 4321,'USAF' from dual union all
select 43210,'MRN' from dual union all
select 43211,'GMS' from dual union all
select 43212,'MRN' from dual union all
select 43213,'MRN' from dual union all
select 43214,'MRN' from dual union all
select 43215,'MRN' from dual union all
select 43216,'MRN' from dual union all
select 43217,'MRN' from dual union all
select 43218,'MRN' from dual union all
select 43219,'MRN' from dual union all
select 4322,'USAF' from dual union all
select 43220,'MRN' from dual union all
select 43221,'AN' from dual union all
select 43222,'NASDA' from dual union all
select 43223,'MRN' from dual union all
select 43224,'MRN' from dual union all
select 43225,'MRN' from dual union all
select 43226,'MRN' from dual union all
select 43227,'MRN' from dual union all
select 43228,'MRN' from dual union all
select 43229,'MRN' from dual union all
select 4323,'USAF' from dual union all
select 43230,'MRN' from dual union all
select 43231,'MRN' from dual union all
select 43232,'MRN' from dual union all
select 43233,'MRN' from dual union all
select 43234,'MRN' from dual union all
select 43235,'MRN' from dual union all
select 43236,'NASDA' from dual union all
select 43237,'MRN' from dual union all
select 43238,'MRN' from dual union all
select 43239,'MRN' from dual union all
select 4324,'USAF' from dual union all
select 43240,'MRN' from dual union all
select 43241,'MRN' from dual union all
select 43242,'GMS' from dual union all
select 43243,'MRN' from dual union all
select 43244,'MRN' from dual union all
select 43245,'MRN' from dual union all
select 43246,'MRN' from dual union all
select 43247,'MRN' from dual union all
select 43248,'MRN' from dual union all
select 43249,'MRN' from dual union all
select 4325,'USAF' from dual union all
select 43250,'MRN' from dual union all
select 43251,'MRN' from dual union all
select 43252,'MRN' from dual union all
select 43253,'MRN' from dual union all
select 43254,'MRN' from dual union all
select 43255,'MRN' from dual;
insert into LAUNCH_AGENCY
select 43256,'MRN' from dual union all
select 43257,'MRN' from dual union all
select 43258,'MRN' from dual union all
select 43259,'MRN' from dual union all
select 4326,'USN' from dual union all
select 43260,'MRN' from dual union all
select 43261,'MRN' from dual union all
select 43262,'MRN' from dual union all
select 43263,'GMS' from dual union all
select 43264,'MRN' from dual union all
select 43265,'MRN' from dual union all
select 43266,'MRN' from dual union all
select 43267,'MRN' from dual union all
select 43268,'MRN' from dual union all
select 43269,'MRN' from dual union all
select 4327,'RAE' from dual union all
select 4327,'WRE' from dual union all
select 43270,'MRN' from dual union all
select 43271,'MRN' from dual union all
select 43272,'MRN' from dual union all
select 43273,'MRN' from dual union all
select 43274,'MRN' from dual union all
select 43275,'MRN' from dual union all
select 43276,'MRN' from dual union all
select 43277,'MRN' from dual union all
select 43278,'MRN' from dual union all
select 43279,'GMS' from dual union all
select 4328,'USAF' from dual union all
select 43280,'MRN' from dual union all
select 43281,'MRN' from dual union all
select 43282,'MRN' from dual union all
select 43283,'MRN' from dual union all
select 43284,'MRN' from dual union all
select 43285,'MRN' from dual union all
select 43286,'MRN' from dual union all
select 43287,'MRN' from dual union all
select 43288,'MRN' from dual union all
select 43289,'MRN' from dual union all
select 4329,'LRL' from dual union all
select 43290,'MRN' from dual union all
select 43291,'MRN' from dual union all
select 43292,'MRN' from dual union all
select 43293,'MRN' from dual union all
select 43294,'MRN' from dual union all
select 43295,'MRN' from dual union all
select 43296,'MRN' from dual union all
select 43297,'MRN' from dual union all
select 43298,'MRN' from dual union all
select 43299,'MRN' from dual union all
select 433,'WER485-2' from dual union all
select 4330,'LRL' from dual union all
select 43300,'MRN' from dual union all
select 43301,'AN' from dual union all
select 43302,'MRN' from dual union all
select 43303,'AN' from dual union all
select 43304,'MRN' from dual union all
select 43305,'MRN' from dual union all
select 43306,'MRN' from dual union all
select 43307,'NASA' from dual union all
select 43308,'MRN' from dual union all
select 43309,'MRN' from dual union all
select 4331,'USAF' from dual union all
select 43310,'MRN' from dual union all
select 43311,'MRN' from dual union all
select 43312,'MRN' from dual union all
select 43313,'MRN' from dual union all
select 43314,'MRN' from dual union all
select 43315,'MRN' from dual union all
select 43316,'MRN' from dual union all
select 43317,'MRN' from dual union all
select 43318,'MRN' from dual union all
select 43319,'MRN' from dual union all
select 4332,'USAF' from dual union all
select 43320,'MRN' from dual union all
select 43321,'MRN' from dual union all
select 43322,'MRN' from dual union all
select 43323,'MRN' from dual union all
select 43324,'MRN' from dual union all
select 43325,'MRN' from dual union all
select 43326,'MRN' from dual union all
select 43327,'MRN' from dual union all
select 43328,'MRN' from dual union all
select 43329,'MRN' from dual union all
select 4333,'USAF' from dual union all
select 43330,'MRN' from dual union all
select 43331,'MRN' from dual union all
select 43332,'MRN' from dual union all
select 43333,'MRN' from dual union all
select 43334,'MRN' from dual union all
select 43335,'AN' from dual union all
select 43336,'MRN' from dual union all
select 43337,'NASA' from dual union all
select 43338,'MRN' from dual union all
select 43339,'MRN' from dual union all
select 4334,'6555A' from dual union all
select 43340,'MRN' from dual union all
select 43341,'MRN' from dual union all
select 43342,'AN' from dual union all
select 43343,'MRN' from dual union all
select 43344,'MRN' from dual;
insert into LAUNCH_AGENCY
select 43345,'MRN' from dual union all
select 43346,'MRN' from dual union all
select 43347,'MRN' from dual union all
select 43348,'MRN' from dual union all
select 43349,'MRN' from dual union all
select 4335,'PARD' from dual union all
select 43350,'MRN' from dual union all
select 43351,'MRN' from dual union all
select 43352,'MRN' from dual union all
select 43353,'MRN' from dual union all
select 43354,'MRN' from dual union all
select 43355,'MRN' from dual union all
select 43356,'MRN' from dual union all
select 43357,'MRN' from dual union all
select 43358,'MRN' from dual union all
select 43359,'MRN' from dual union all
select 4336,'PARD' from dual union all
select 43360,'MRN' from dual union all
select 43361,'MRN' from dual union all
select 43362,'MRN' from dual union all
select 43363,'MRN' from dual union all
select 43364,'MRN' from dual union all
select 43365,'MRN' from dual union all
select 43366,'MRN' from dual union all
select 43367,'MRN' from dual union all
select 43368,'MRN' from dual union all
select 43369,'AN' from dual union all
select 4337,'USAF' from dual union all
select 43370,'MRN' from dual union all
select 43371,'MRN' from dual union all
select 43372,'AN' from dual union all
select 43373,'MRN' from dual union all
select 43374,'MRN' from dual union all
select 43375,'MRN' from dual union all
select 43376,'MRN' from dual union all
select 43377,'MRN' from dual union all
select 43378,'MRN' from dual union all
select 43379,'MRN' from dual union all
select 4338,'PARD' from dual union all
select 43380,'MRN' from dual union all
select 43381,'MRN' from dual union all
select 43382,'MRN' from dual union all
select 43383,'MRN' from dual union all
select 43384,'MRN' from dual union all
select 43385,'MRN' from dual union all
select 43386,'MRN' from dual union all
select 43387,'MRN' from dual union all
select 43388,'MRN' from dual union all
select 43389,'MRN' from dual union all
select 4339,'USAF' from dual union all
select 43390,'MRN' from dual union all
select 43391,'MRN' from dual union all
select 43392,'MRN' from dual union all
select 43393,'MRN' from dual union all
select 43394,'MRN' from dual union all
select 43395,'MRN' from dual union all
select 43396,'MRN' from dual union all
select 43397,'MRN' from dual union all
select 43398,'MRN' from dual union all
select 43399,'MRN' from dual union all
select 434,'WER444' from dual union all
select 4340,'SUI' from dual union all
select 43400,'MRN' from dual union all
select 43401,'MRN' from dual union all
select 43402,'MRN' from dual union all
select 43403,'MRN' from dual union all
select 43404,'MRN' from dual union all
select 43405,'MRN' from dual union all
select 43406,'AN' from dual union all
select 43407,'MRN' from dual union all
select 43408,'MRN' from dual union all
select 43409,'MRN' from dual union all
select 4341,'USAF' from dual union all
select 43410,'MRN' from dual union all
select 43411,'MRN' from dual union all
select 43412,'MRN' from dual union all
select 43413,'MRN' from dual union all
select 43414,'MRN' from dual union all
select 43415,'MRN' from dual union all
select 43416,'MRN' from dual union all
select 43417,'MRN' from dual union all
select 43418,'MRN' from dual union all
select 43419,'MRN' from dual union all
select 4342,'USAF' from dual union all
select 43420,'GMS' from dual union all
select 43421,'AN' from dual union all
select 43422,'GMS' from dual union all
select 43423,'MRN' from dual union all
select 43424,'MRN' from dual union all
select 43425,'MRN' from dual union all
select 43426,'MRN' from dual union all
select 43427,'MRN' from dual union all
select 43428,'MRN' from dual union all
select 43429,'MRN' from dual union all
select 4343,'USAF' from dual union all
select 43430,'MRN' from dual union all
select 43431,'MRN' from dual union all
select 43432,'MRN' from dual union all
select 43433,'NASA' from dual union all
select 43434,'MRN' from dual;
insert into LAUNCH_AGENCY
select 43435,'MRN' from dual union all
select 43436,'MRN' from dual union all
select 43437,'MRN' from dual union all
select 43438,'GMS' from dual union all
select 43439,'MRN' from dual union all
select 4344,'SU' from dual union all
select 43440,'MRN' from dual union all
select 43441,'MRN' from dual union all
select 43442,'MRN' from dual union all
select 43443,'MRN' from dual union all
select 43444,'MRN' from dual union all
select 43445,'MRN' from dual union all
select 43446,'GMS' from dual union all
select 43447,'GMS' from dual union all
select 43448,'MRN' from dual union all
select 43449,'MRN' from dual union all
select 4345,'USAF' from dual union all
select 43450,'MRN' from dual union all
select 43451,'MRN' from dual union all
select 43452,'MRN' from dual union all
select 43453,'MRN' from dual union all
select 43454,'MRN' from dual union all
select 43455,'MRN' from dual union all
select 43456,'MRN' from dual union all
select 43457,'MRN' from dual union all
select 43458,'MRN' from dual union all
select 43459,'AN' from dual union all
select 4346,'USA' from dual union all
select 43460,'AN' from dual union all
select 43461,'MRN' from dual union all
select 43462,'MRN' from dual union all
select 43463,'MRN' from dual union all
select 43464,'MRN' from dual union all
select 43465,'MRN' from dual union all
select 43466,'MRN' from dual union all
select 43467,'MRN' from dual union all
select 43468,'GMS' from dual union all
select 43469,'MRN' from dual union all
select 4347,'USAF' from dual union all
select 43470,'GMS' from dual union all
select 43471,'MRN' from dual union all
select 43472,'MRN' from dual union all
select 43473,'MRN' from dual union all
select 43474,'MRN' from dual union all
select 43475,'MRN' from dual union all
select 43476,'MRN' from dual union all
select 43477,'MRN' from dual union all
select 43478,'MRN' from dual union all
select 43479,'NASA' from dual union all
select 4348,'USAF' from dual union all
select 43480,'MRN' from dual union all
select 43481,'MRN' from dual union all
select 43482,'GMS' from dual union all
select 43483,'MRN' from dual union all
select 43484,'MRN' from dual union all
select 43485,'MRN' from dual union all
select 43486,'MRN' from dual union all
select 43487,'MRN' from dual union all
select 43488,'MRN' from dual union all
select 43489,'MRN' from dual union all
select 4349,'USAF' from dual union all
select 43490,'MRN' from dual union all
select 43491,'MRN' from dual union all
select 43492,'MRN' from dual union all
select 43493,'MRN' from dual union all
select 43494,'AN' from dual union all
select 43495,'MRN' from dual union all
select 43496,'MRN' from dual union all
select 43497,'MRN' from dual union all
select 43498,'MRN' from dual union all
select 43499,'MRN' from dual union all
select 435,'WER836-2' from dual union all
select 4350,'USAF' from dual union all
select 43500,'GMS' from dual union all
select 43501,'MRN' from dual union all
select 43502,'MRN' from dual union all
select 43503,'MRN' from dual union all
select 43504,'MRN' from dual union all
select 43505,'MRN' from dual union all
select 43506,'MRN' from dual union all
select 43507,'MRN' from dual union all
select 43508,'MRN' from dual union all
select 43509,'MRN' from dual union all
select 4351,'PARD' from dual union all
select 43510,'MRN' from dual union all
select 43511,'MRN' from dual union all
select 43512,'MRN' from dual union all
select 43513,'MRN' from dual union all
select 43514,'MRN' from dual union all
select 43515,'MRN' from dual union all
select 43516,'MRN' from dual union all
select 43517,'MRN' from dual union all
select 43518,'MRN' from dual union all
select 43519,'MRN' from dual union all
select 4352,'USAF' from dual union all
select 43520,'MRN' from dual union all
select 43521,'MRN' from dual union all
select 43522,'MRN' from dual union all
select 43523,'MRN' from dual union all
select 43524,'MRN' from dual;
insert into LAUNCH_AGENCY
select 43525,'MRN' from dual union all
select 43526,'MRN' from dual union all
select 43527,'AN' from dual union all
select 43528,'MRN' from dual union all
select 43529,'MRN' from dual union all
select 4353,'USAF' from dual union all
select 43530,'MRN' from dual union all
select 43531,'MRN' from dual union all
select 43532,'MRN' from dual union all
select 43533,'MRN' from dual union all
select 43534,'MRN' from dual union all
select 43535,'MRN' from dual union all
select 43536,'MRN' from dual union all
select 43537,'MRN' from dual union all
select 43538,'MRN' from dual union all
select 43539,'MRN' from dual union all
select 4354,'ABMA' from dual union all
select 4354,'USAF' from dual union all
select 43540,'MRN' from dual union all
select 43541,'MRN' from dual union all
select 43542,'MRN' from dual union all
select 43543,'MRN' from dual union all
select 43544,'MRN' from dual union all
select 43545,'MRN' from dual union all
select 43546,'MRN' from dual union all
select 43547,'MRN' from dual union all
select 43548,'MRN' from dual union all
select 43549,'MRN' from dual union all
select 4355,'USAF' from dual union all
select 43550,'MRN' from dual union all
select 43551,'MRN' from dual union all
select 43552,'MRN' from dual union all
select 43553,'MRN' from dual union all
select 43554,'MRN' from dual union all
select 43555,'MRN' from dual union all
select 43556,'MRN' from dual union all
select 43557,'AN' from dual union all
select 43558,'MRN' from dual union all
select 43559,'MRN' from dual union all
select 4356,'USAF' from dual union all
select 43560,'AN' from dual union all
select 43561,'MRN' from dual union all
select 43562,'MRN' from dual union all
select 43563,'MRN' from dual union all
select 43564,'MRN' from dual union all
select 43565,'MRN' from dual union all
select 43566,'MRN' from dual union all
select 43567,'MRN' from dual union all
select 43568,'MRN' from dual union all
select 43569,'MRN' from dual union all
select 4357,'BRL' from dual union all
select 43570,'MRN' from dual union all
select 43571,'MRN' from dual union all
select 43572,'MRN' from dual union all
select 43573,'MRN' from dual union all
select 43574,'MRN' from dual union all
select 43575,'MRN' from dual union all
select 43576,'MRN' from dual union all
select 43577,'MRN' from dual union all
select 43578,'MRN' from dual union all
select 43579,'MRN' from dual union all
select 4358,'PARD' from dual union all
select 43580,'MRN' from dual union all
select 43581,'MRN' from dual union all
select 43582,'MRN' from dual union all
select 43583,'MRN' from dual union all
select 43584,'MRN' from dual union all
select 43585,'MRN' from dual union all
select 43586,'MRN' from dual union all
select 43587,'MRN' from dual union all
select 43588,'MRN' from dual union all
select 43589,'MRN' from dual union all
select 4359,'STA' from dual union all
select 43590,'MRN' from dual union all
select 43591,'MRN' from dual union all
select 43592,'AN' from dual union all
select 43593,'MRN' from dual union all
select 43594,'MRN' from dual union all
select 43595,'MRN' from dual union all
select 43596,'MRN' from dual union all
select 43597,'AN' from dual union all
select 43598,'MRN' from dual union all
select 43599,'MRN' from dual union all
select 436,'WER485-2' from dual union all
select 4360,'USAF' from dual union all
select 43600,'MRN' from dual union all
select 43601,'MRN' from dual union all
select 43602,'AN' from dual union all
select 43603,'NASA' from dual union all
select 43604,'NASA' from dual union all
select 43605,'MRN' from dual union all
select 43606,'MRN' from dual union all
select 43607,'MRN' from dual union all
select 43608,'MRN' from dual union all
select 43609,'MRN' from dual union all
select 4361,'STA' from dual union all
select 43610,'MRN' from dual union all
select 43611,'MRN' from dual union all
select 43612,'MRN' from dual union all
select 43613,'MRN' from dual;
insert into LAUNCH_AGENCY
select 43614,'MRN' from dual union all
select 43615,'MRN' from dual union all
select 43616,'MRN' from dual union all
select 43617,'MRN' from dual union all
select 43618,'MRN' from dual union all
select 43619,'MRN' from dual union all
select 4362,'BRL' from dual union all
select 43620,'MRN' from dual union all
select 43621,'MRN' from dual union all
select 43622,'MRN' from dual union all
select 43623,'MRN' from dual union all
select 43624,'MRN' from dual union all
select 43625,'MRN' from dual union all
select 43626,'MRN' from dual union all
select 43627,'MRN' from dual union all
select 43628,'MRN' from dual union all
select 43629,'MRN' from dual union all
select 4363,'STA' from dual union all
select 43630,'MRN' from dual union all
select 43631,'MRN' from dual union all
select 43632,'MRN' from dual union all
select 43633,'MRN' from dual union all
select 43634,'MRN' from dual union all
select 43635,'MRN' from dual union all
select 43636,'MRN' from dual union all
select 43637,'MRN' from dual union all
select 43638,'MRN' from dual union all
select 43639,'MRN' from dual union all
select 4364,'STA' from dual union all
select 43640,'MRN' from dual union all
select 43641,'MRN' from dual union all
select 43642,'MRN' from dual union all
select 43643,'MRN' from dual union all
select 43644,'MRN' from dual union all
select 43645,'MRN' from dual union all
select 43646,'AN' from dual union all
select 43647,'AN' from dual union all
select 43648,'MRN' from dual union all
select 43649,'MRN' from dual union all
select 4365,'PARD' from dual union all
select 43650,'MRN' from dual union all
select 43651,'AN' from dual union all
select 43652,'MRN' from dual union all
select 43653,'AN' from dual union all
select 43654,'MRN' from dual union all
select 43655,'MRN' from dual union all
select 43656,'MRN' from dual union all
select 43657,'MRN' from dual union all
select 43658,'MRN' from dual union all
select 43659,'AN' from dual union all
select 4366,'ORD' from dual union all
select 43660,'MRN' from dual union all
select 43661,'MRN' from dual union all
select 43662,'MRN' from dual union all
select 43663,'MRN' from dual union all
select 43664,'MRN' from dual union all
select 43665,'MRN' from dual union all
select 43666,'MRN' from dual union all
select 43667,'MRN' from dual union all
select 43668,'MRN' from dual union all
select 43669,'MRN' from dual union all
select 4367,'USA' from dual union all
select 43670,'AN' from dual union all
select 43671,'MRN' from dual union all
select 43672,'MRN' from dual union all
select 43673,'MRN' from dual union all
select 43674,'MRN' from dual union all
select 43675,'MRN' from dual union all
select 43676,'MRN' from dual union all
select 43677,'MRN' from dual union all
select 43678,'MRN' from dual union all
select 43679,'MRN' from dual union all
select 4368,'USA' from dual union all
select 43680,'MRN' from dual union all
select 43681,'MRN' from dual union all
select 43682,'MRN' from dual union all
select 43683,'MRN' from dual union all
select 43684,'GMS' from dual union all
select 43685,'MRN' from dual union all
select 43686,'MRN' from dual union all
select 43687,'MRN' from dual union all
select 43688,'MRN' from dual union all
select 43689,'MRN' from dual union all
select 4369,'USA' from dual union all
select 43690,'MRN' from dual union all
select 43691,'MRN' from dual union all
select 43692,'MRN' from dual union all
select 43693,'MRN' from dual union all
select 43694,'MRN' from dual union all
select 43695,'MRN' from dual union all
select 43696,'MRN' from dual union all
select 43697,'MRN' from dual union all
select 43698,'MRN' from dual union all
select 43699,'MRN' from dual union all
select 437,'WER444' from dual union all
select 4370,'USA' from dual union all
select 43700,'MRN' from dual union all
select 43701,'MRN' from dual union all
select 43702,'MRN' from dual union all
select 43703,'MRN' from dual;
insert into LAUNCH_AGENCY
select 43704,'MRN' from dual union all
select 43705,'GMS' from dual union all
select 43706,'AN' from dual union all
select 43707,'MRN' from dual union all
select 43708,'MRN' from dual union all
select 43709,'MRN' from dual union all
select 4371,'ABMA' from dual union all
select 4371,'USAF' from dual union all
select 43710,'MRN' from dual union all
select 43711,'MRN' from dual union all
select 43712,'MRN' from dual union all
select 43713,'MRN' from dual union all
select 43714,'MRN' from dual union all
select 43715,'MRN' from dual union all
select 43716,'MRN' from dual union all
select 43717,'MRN' from dual union all
select 43718,'MRN' from dual union all
select 43719,'MRN' from dual union all
select 4372,'USA' from dual union all
select 43720,'MRN' from dual union all
select 43721,'MRN' from dual union all
select 43722,'MRN' from dual union all
select 43723,'MRN' from dual union all
select 43724,'MRN' from dual union all
select 43725,'MRN' from dual union all
select 43726,'MRN' from dual union all
select 43727,'MRN' from dual union all
select 43728,'MRN' from dual union all
select 43729,'MRN' from dual union all
select 4373,'USA' from dual union all
select 43730,'MRN' from dual union all
select 43731,'MRN' from dual union all
select 43732,'MRN' from dual union all
select 43733,'MRN' from dual union all
select 43734,'MRN' from dual union all
select 43735,'MRN' from dual union all
select 43736,'MRN' from dual union all
select 43737,'MRN' from dual union all
select 43738,'GMS' from dual union all
select 43739,'AN' from dual union all
select 4374,'USA' from dual union all
select 43740,'MRN' from dual union all
select 43741,'MRN' from dual union all
select 43742,'MRN' from dual union all
select 43743,'MRN' from dual union all
select 43744,'MRN' from dual union all
select 43745,'MRN' from dual union all
select 43746,'NASA' from dual union all
select 43747,'MRN' from dual union all
select 43748,'MRN' from dual union all
select 43749,'MRN' from dual union all
select 4375,'USA' from dual union all
select 43750,'MRN' from dual union all
select 43751,'MRN' from dual union all
select 43752,'MRN' from dual union all
select 43753,'NASA' from dual union all
select 43754,'MRN' from dual union all
select 43755,'MRN' from dual union all
select 43756,'MRN' from dual union all
select 43757,'MRN' from dual union all
select 43758,'MRN' from dual union all
select 43759,'MRN' from dual union all
select 4376,'USA' from dual union all
select 43760,'MRN' from dual union all
select 43761,'NASA' from dual union all
select 43762,'MRN' from dual union all
select 43763,'MRN' from dual union all
select 43764,'MRN' from dual union all
select 43765,'MRN' from dual union all
select 43766,'MRN' from dual union all
select 43767,'MRN' from dual union all
select 43768,'MRN' from dual union all
select 43769,'MRN' from dual union all
select 4377,'USA' from dual union all
select 43770,'MRN' from dual union all
select 43771,'GMS' from dual union all
select 43772,'AN' from dual union all
select 43773,'MRN' from dual union all
select 43774,'MRN' from dual union all
select 43775,'MRN' from dual union all
select 43776,'MRN' from dual union all
select 43777,'MRN' from dual union all
select 43778,'MRN' from dual union all
select 43779,'MRN' from dual union all
select 4378,'USA' from dual union all
select 43780,'MRN' from dual union all
select 43781,'MRN' from dual union all
select 43782,'MRN' from dual union all
select 43783,'MRN' from dual union all
select 43784,'MRN' from dual union all
select 43785,'MRN' from dual union all
select 43786,'MRN' from dual union all
select 43787,'MRN' from dual union all
select 43788,'MRN' from dual union all
select 43789,'MRN' from dual union all
select 4379,'UNRV' from dual union all
select 43790,'MRN' from dual union all
select 43791,'MRN' from dual union all
select 43792,'MRN' from dual union all
select 43793,'GMS' from dual;
insert into LAUNCH_AGENCY
select 43794,'MRN' from dual union all
select 43795,'MRN' from dual union all
select 43796,'MRN' from dual union all
select 43797,'MRN' from dual union all
select 43798,'MRN' from dual union all
select 43799,'MRN' from dual union all
select 438,'WER836-3' from dual union all
select 4380,'UNRV' from dual union all
select 43800,'NASA' from dual union all
select 43801,'AN' from dual union all
select 43802,'AN' from dual union all
select 43803,'AN' from dual union all
select 43804,'AN' from dual union all
select 43805,'AN' from dual union all
select 43806,'AN' from dual union all
select 43807,'AN' from dual union all
select 43808,'AN' from dual union all
select 43809,'AN' from dual union all
select 4381,'USAF' from dual union all
select 43810,'MRN' from dual union all
select 43811,'AN' from dual union all
select 43812,'AN' from dual union all
select 43813,'AN' from dual union all
select 43814,'AN' from dual union all
select 43815,'MRN' from dual union all
select 43816,'MRN' from dual union all
select 43817,'MRN' from dual union all
select 43818,'MRN' from dual union all
select 43819,'MRN' from dual union all
select 4382,'UNRV' from dual union all
select 43820,'MRN' from dual union all
select 43821,'MRN' from dual union all
select 43822,'MRN' from dual union all
select 43823,'MRN' from dual union all
select 43824,'MRN' from dual union all
select 43825,'AN' from dual union all
select 43826,'MRN' from dual union all
select 43827,'MRN' from dual union all
select 43828,'MRN' from dual union all
select 43829,'MRN' from dual union all
select 4383,'UNRV' from dual union all
select 43830,'MRN' from dual union all
select 43831,'MRN' from dual union all
select 43832,'MRN' from dual union all
select 43833,'MRN' from dual union all
select 43834,'MRN' from dual union all
select 43835,'MRN' from dual union all
select 43836,'MRN' from dual union all
select 43837,'MRN' from dual union all
select 43838,'MRN' from dual union all
select 43839,'MRN' from dual union all
select 4384,'UNRV' from dual union all
select 43840,'MRN' from dual union all
select 43841,'MRN' from dual union all
select 43842,'MRN' from dual union all
select 43843,'MRN' from dual union all
select 43844,'MRN' from dual union all
select 43845,'MRN' from dual union all
select 43846,'MRN' from dual union all
select 43847,'MRN' from dual union all
select 43848,'MRN' from dual union all
select 43849,'MRN' from dual union all
select 4385,'UNRV' from dual union all
select 43850,'MRN' from dual union all
select 43851,'MRN' from dual union all
select 43852,'MRN' from dual union all
select 43853,'MRN' from dual union all
select 43854,'MRN' from dual union all
select 43855,'MRN' from dual union all
select 43856,'MRN' from dual union all
select 43857,'MRN' from dual union all
select 43858,'MRN' from dual union all
select 43859,'MRN' from dual union all
select 4386,'UNRV' from dual union all
select 43860,'MRN' from dual union all
select 43861,'MRN' from dual union all
select 43862,'MRN' from dual union all
select 43863,'MRN' from dual union all
select 43864,'MRN' from dual union all
select 43865,'MRN' from dual union all
select 43866,'MRN' from dual union all
select 43867,'MRN' from dual union all
select 43868,'MRN' from dual union all
select 43869,'MRN' from dual union all
select 4387,'UNRV' from dual union all
select 43870,'MRN' from dual union all
select 43871,'MRN' from dual union all
select 43872,'MRN' from dual union all
select 43873,'MRN' from dual union all
select 43874,'MRN' from dual union all
select 43875,'MRN' from dual union all
select 43876,'MRN' from dual union all
select 43877,'MRN' from dual union all
select 43878,'MRN' from dual union all
select 43879,'AN' from dual union all
select 4388,'USN' from dual union all
select 43880,'MRN' from dual union all
select 43881,'MRN' from dual union all
select 43882,'MRN' from dual union all
select 43883,'MRN' from dual;
insert into LAUNCH_AGENCY
select 43884,'MRN' from dual union all
select 43885,'MRN' from dual union all
select 43886,'MRN' from dual union all
select 43887,'MRN' from dual union all
select 43888,'GMS' from dual union all
select 43889,'MRN' from dual union all
select 4389,'USN' from dual union all
select 43890,'MRN' from dual union all
select 43891,'MRN' from dual union all
select 43892,'MRN' from dual union all
select 43893,'MRN' from dual union all
select 43894,'MRN' from dual union all
select 43895,'MRN' from dual union all
select 43896,'MRN' from dual union all
select 43897,'MRN' from dual union all
select 43898,'MRN' from dual union all
select 43899,'AN' from dual union all
select 439,'WER485-2' from dual union all
select 4390,'USN' from dual union all
select 43900,'MRN' from dual union all
select 43901,'MRN' from dual union all
select 43902,'MRN' from dual union all
select 43903,'MRN' from dual union all
select 43904,'MRN' from dual union all
select 43905,'MRN' from dual union all
select 43906,'MRN' from dual union all
select 43907,'MRN' from dual union all
select 43908,'MRN' from dual union all
select 43909,'MRN' from dual union all
select 4391,'USN' from dual union all
select 43910,'AN' from dual union all
select 43911,'MRN' from dual union all
select 43912,'MRN' from dual union all
select 43913,'MRN' from dual union all
select 43914,'MRN' from dual union all
select 43915,'MRN' from dual union all
select 43916,'MRN' from dual union all
select 43917,'GMS' from dual union all
select 43918,'MRN' from dual union all
select 43919,'MRN' from dual union all
select 4392,'USAF' from dual union all
select 43920,'MRN' from dual union all
select 43921,'MRN' from dual union all
select 43922,'MRN' from dual union all
select 43923,'MRN' from dual union all
select 43924,'MRN' from dual union all
select 43925,'MRN' from dual union all
select 43926,'MRN' from dual union all
select 43927,'MRN' from dual union all
select 43928,'MRN' from dual union all
select 43929,'MRN' from dual union all
select 4393,'USN' from dual union all
select 43930,'MRN' from dual union all
select 43931,'MRN' from dual union all
select 43932,'MRN' from dual union all
select 43933,'MRN' from dual union all
select 43934,'MRN' from dual union all
select 43935,'MRN' from dual union all
select 43936,'MRN' from dual union all
select 43937,'MRN' from dual union all
select 43938,'MRN' from dual union all
select 43939,'MRN' from dual union all
select 4394,'USA' from dual union all
select 43940,'MRN' from dual union all
select 43941,'MRN' from dual union all
select 43942,'MRN' from dual union all
select 43943,'MRN' from dual union all
select 43944,'MRN' from dual union all
select 43945,'MRN' from dual union all
select 43946,'MRN' from dual union all
select 43947,'AN' from dual union all
select 43948,'MRN' from dual union all
select 43949,'MRN' from dual union all
select 4395,'USA' from dual union all
select 43950,'MRN' from dual union all
select 43951,'MRN' from dual union all
select 43952,'MRN' from dual union all
select 43953,'MRN' from dual union all
select 43954,'MRN' from dual union all
select 43955,'MRN' from dual union all
select 43956,'MRN' from dual union all
select 43957,'MRN' from dual union all
select 43958,'MRN' from dual union all
select 43959,'MRN' from dual union all
select 4396,'USAF' from dual union all
select 43960,'MRN' from dual union all
select 43961,'MRN' from dual union all
select 43962,'MRN' from dual union all
select 43963,'MRN' from dual union all
select 43964,'MRN' from dual union all
select 43965,'MRN' from dual union all
select 43966,'MRN' from dual union all
select 43967,'MRN' from dual union all
select 43968,'MRN' from dual union all
select 43969,'MRN' from dual union all
select 4397,'PARD' from dual union all
select 43970,'MRN' from dual union all
select 43971,'MRN' from dual union all
select 43972,'MRN' from dual union all
select 43973,'MRN' from dual;
insert into LAUNCH_AGENCY
select 43974,'MRN' from dual union all
select 43975,'MRN' from dual union all
select 43976,'MRN' from dual union all
select 43977,'MRN' from dual union all
select 43978,'GMS' from dual union all
select 43979,'MRN' from dual union all
select 4398,'USAF' from dual union all
select 43980,'MRN' from dual union all
select 43981,'MRN' from dual union all
select 43982,'MRN' from dual union all
select 43983,'MRN' from dual union all
select 43984,'MRN' from dual union all
select 43985,'MRN' from dual union all
select 43986,'MRN' from dual union all
select 43987,'MRN' from dual union all
select 43988,'MRN' from dual union all
select 43989,'MRN' from dual union all
select 4399,'PARD' from dual union all
select 43990,'MRN' from dual union all
select 43991,'MRN' from dual union all
select 43992,'MRN' from dual union all
select 43993,'MRN' from dual union all
select 43994,'MRN' from dual union all
select 43995,'MRN' from dual union all
select 43996,'MRN' from dual union all
select 43997,'MRN' from dual union all
select 43998,'MRN' from dual union all
select 43999,'MRN' from dual union all
select 44,'WER444' from dual union all
select 440,'WER444' from dual union all
select 4400,'USAF' from dual union all
select 44000,'MRN' from dual union all
select 44001,'NASA' from dual union all
select 44002,'NASA' from dual union all
select 44003,'NASA' from dual union all
select 44004,'MRN' from dual union all
select 44005,'MRN' from dual union all
select 44006,'MRN' from dual union all
select 44007,'MRN' from dual union all
select 44008,'MRN' from dual union all
select 44009,'MRN' from dual union all
select 4401,'USAF' from dual union all
select 44010,'MRN' from dual union all
select 44011,'MRN' from dual union all
select 44012,'MRN' from dual union all
select 44013,'MRN' from dual union all
select 44014,'MRN' from dual union all
select 44015,'MRN' from dual union all
select 44016,'MRN' from dual union all
select 44017,'MRN' from dual union all
select 44018,'MRN' from dual union all
select 44019,'MRN' from dual union all
select 4402,'USAF' from dual union all
select 44020,'MRN' from dual union all
select 44021,'MRN' from dual union all
select 44022,'MRN' from dual union all
select 44023,'MRN' from dual union all
select 44024,'MRN' from dual union all
select 44025,'MRN' from dual union all
select 44026,'MRN' from dual union all
select 44027,'MRN' from dual union all
select 44028,'AN' from dual union all
select 44029,'MRN' from dual union all
select 4403,'USAF' from dual union all
select 44030,'MRN' from dual union all
select 44031,'MRN' from dual union all
select 44032,'MRN' from dual union all
select 44033,'MRN' from dual union all
select 44034,'MRN' from dual union all
select 44035,'NASA' from dual union all
select 44036,'MRN' from dual union all
select 44037,'MRN' from dual union all
select 44038,'MRN' from dual union all
select 44039,'MRN' from dual union all
select 4404,'USAF' from dual union all
select 44040,'MRN' from dual union all
select 44041,'MRN' from dual union all
select 44042,'MRN' from dual union all
select 44043,'NASDA' from dual union all
select 44044,'MRN' from dual union all
select 44045,'MRN' from dual union all
select 44046,'MRN' from dual union all
select 44047,'MRN' from dual union all
select 44048,'MRN' from dual union all
select 44049,'MRN' from dual union all
select 4405,'6555A' from dual union all
select 44050,'MRN' from dual union all
select 44051,'MRN' from dual union all
select 44052,'MRN' from dual union all
select 44053,'MRN' from dual union all
select 44054,'MRN' from dual union all
select 44055,'MRN' from dual union all
select 44056,'MRN' from dual union all
select 44057,'MRN' from dual union all
select 44058,'GMS' from dual union all
select 44059,'MRN' from dual union all
select 4406,'USAF' from dual union all
select 44060,'MRN' from dual union all
select 44061,'MRN' from dual union all
select 44062,'MRN' from dual;
insert into LAUNCH_AGENCY
select 44063,'MRN' from dual union all
select 44064,'MRN' from dual union all
select 44065,'MRN' from dual union all
select 44066,'MRN' from dual union all
select 44067,'MRN' from dual union all
select 44068,'MRN' from dual union all
select 44069,'MRN' from dual union all
select 4407,'ABMA' from dual union all
select 4407,'USAF' from dual union all
select 44070,'MRN' from dual union all
select 44071,'MRN' from dual union all
select 44072,'MRN' from dual union all
select 44073,'MRN' from dual union all
select 44074,'MRN' from dual union all
select 44075,'MRN' from dual union all
select 44076,'MRN' from dual union all
select 44077,'MRN' from dual union all
select 44078,'MRN' from dual union all
select 44079,'MRN' from dual union all
select 4408,'LRL' from dual union all
select 44080,'GMS' from dual union all
select 44081,'MRN' from dual union all
select 44082,'MRN' from dual union all
select 44083,'MRN' from dual union all
select 44084,'MRN' from dual union all
select 44085,'MRN' from dual union all
select 44086,'MRN' from dual union all
select 44087,'MRN' from dual union all
select 44088,'MRN' from dual union all
select 44089,'MRN' from dual union all
select 4409,'LRL' from dual union all
select 44090,'MRN' from dual union all
select 44091,'MRN' from dual union all
select 44092,'MRN' from dual union all
select 44093,'MRN' from dual union all
select 44094,'MRN' from dual union all
select 44095,'MRN' from dual union all
select 44096,'MRN' from dual union all
select 44097,'MRN' from dual union all
select 44098,'MRN' from dual union all
select 44099,'MRN' from dual union all
select 441,'WER485-2' from dual union all
select 4410,'ISAS' from dual union all
select 44100,'MRN' from dual union all
select 44101,'GMS' from dual union all
select 44102,'MRN' from dual union all
select 44103,'MRN' from dual union all
select 44104,'MRN' from dual union all
select 44105,'MRN' from dual union all
select 44106,'MRN' from dual union all
select 44107,'MRN' from dual union all
select 44108,'AN' from dual union all
select 44109,'MRN' from dual union all
select 4411,'ISAS' from dual union all
select 44110,'MRN' from dual union all
select 44111,'NRCC' from dual union all
select 44112,'MRN' from dual union all
select 44113,'MRN' from dual union all
select 44114,'MRN' from dual union all
select 44115,'MRN' from dual union all
select 44116,'MRN' from dual union all
select 44117,'MRN' from dual union all
select 44118,'MRN' from dual union all
select 44119,'MRN' from dual union all
select 4412,'PARD' from dual union all
select 44120,'MRN' from dual union all
select 44121,'MRN' from dual union all
select 44122,'NRCC' from dual union all
select 44123,'MRN' from dual union all
select 44124,'MRN' from dual union all
select 44125,'GMS' from dual union all
select 44126,'AN' from dual union all
select 44127,'MRN' from dual union all
select 44128,'MRN' from dual union all
select 44129,'MRN' from dual union all
select 4413,'USAF' from dual union all
select 44130,'MRN' from dual union all
select 44131,'MRN' from dual union all
select 44132,'MRN' from dual union all
select 44133,'MRN' from dual union all
select 44134,'MRN' from dual union all
select 44135,'MRN' from dual union all
select 44136,'MRN' from dual union all
select 44137,'MRN' from dual union all
select 44138,'MRN' from dual union all
select 44139,'MRN' from dual union all
select 4414,'USAF' from dual union all
select 44140,'GMS' from dual union all
select 44141,'MRN' from dual union all
select 44142,'MRN' from dual union all
select 44143,'MRN' from dual union all
select 44144,'MRN' from dual union all
select 44145,'MRN' from dual union all
select 44146,'MRN' from dual union all
select 44147,'MRN' from dual union all
select 44148,'MRN' from dual union all
select 44149,'MRN' from dual union all
select 4415,'USAF' from dual union all
select 44150,'MRN' from dual union all
select 44151,'MRN' from dual;
insert into LAUNCH_AGENCY
select 44152,'MRN' from dual union all
select 44153,'AN' from dual union all
select 44154,'MRN' from dual union all
select 44155,'MRN' from dual union all
select 44156,'MRN' from dual union all
select 44157,'MRN' from dual union all
select 44158,'MRN' from dual union all
select 44159,'MRN' from dual union all
select 4416,'USAF' from dual union all
select 44160,'MRN' from dual union all
select 44161,'MRN' from dual union all
select 44162,'MRN' from dual union all
select 44163,'MRN' from dual union all
select 44164,'MRN' from dual union all
select 44165,'MRN' from dual union all
select 44166,'MRN' from dual union all
select 44167,'MRN' from dual union all
select 44168,'MRN' from dual union all
select 44169,'AN' from dual union all
select 4417,'6555A' from dual union all
select 44170,'MRN' from dual union all
select 44171,'MRN' from dual union all
select 44172,'MRN' from dual union all
select 44173,'MRN' from dual union all
select 44174,'MRN' from dual union all
select 44175,'NASA' from dual union all
select 44176,'MRN' from dual union all
select 44177,'NASA' from dual union all
select 44178,'MRN' from dual union all
select 44179,'MRN' from dual union all
select 4418,'PARD' from dual union all
select 44180,'GMS' from dual union all
select 44181,'GMS' from dual union all
select 44182,'MRN' from dual union all
select 44183,'MRN' from dual union all
select 44184,'MRN' from dual union all
select 44185,'MRN' from dual union all
select 44186,'MRN' from dual union all
select 44187,'GMS' from dual union all
select 44188,'MRN' from dual union all
select 44189,'MRN' from dual union all
select 4419,'STA' from dual union all
select 44190,'MRN' from dual union all
select 44191,'MRN' from dual union all
select 44192,'MRN' from dual union all
select 44193,'MRN' from dual union all
select 44194,'MRN' from dual union all
select 44195,'MRN' from dual union all
select 44196,'MRN' from dual union all
select 44197,'MRN' from dual union all
select 44198,'MRN' from dual union all
select 44199,'MRN' from dual union all
select 442,'WER444' from dual union all
select 4420,'USAF' from dual union all
select 44200,'MRN' from dual union all
select 44201,'MRN' from dual union all
select 44202,'MRN' from dual union all
select 44203,'MRN' from dual union all
select 44204,'MRN' from dual union all
select 44205,'MRN' from dual union all
select 44206,'MRN' from dual union all
select 44207,'MRN' from dual union all
select 44208,'GMS' from dual union all
select 44209,'GMS' from dual union all
select 4421,'LRL' from dual union all
select 44210,'MRN' from dual union all
select 44211,'AN' from dual union all
select 44212,'MRN' from dual union all
select 44213,'MRN' from dual union all
select 44214,'MRN' from dual union all
select 44215,'MRN' from dual union all
select 44216,'MRN' from dual union all
select 44217,'MRN' from dual union all
select 44218,'MRN' from dual union all
select 44219,'MRN' from dual union all
select 4422,'LRL' from dual union all
select 44220,'MRN' from dual union all
select 44221,'MRN' from dual union all
select 44222,'MRN' from dual union all
select 44223,'MRN' from dual union all
select 44224,'GMS' from dual union all
select 44225,'MRN' from dual union all
select 44226,'MRN' from dual union all
select 44227,'GMS' from dual union all
select 44228,'MRN' from dual union all
select 44229,'MRN' from dual union all
select 4423,'LRL' from dual union all
select 44230,'MRN' from dual union all
select 44231,'MRN' from dual union all
select 44232,'MRN' from dual union all
select 44233,'MRN' from dual union all
select 44234,'MRN' from dual union all
select 44235,'MRN' from dual union all
select 44236,'MRN' from dual union all
select 44237,'MRN' from dual union all
select 44238,'MRN' from dual union all
select 44239,'MRN' from dual union all
select 4424,'LRL' from dual union all
select 44240,'MRN' from dual union all
select 44241,'MRN' from dual;
insert into LAUNCH_AGENCY
select 44242,'MRN' from dual union all
select 44243,'MRN' from dual union all
select 44244,'MRN' from dual union all
select 44245,'MRN' from dual union all
select 44246,'MRN' from dual union all
select 44247,'MRN' from dual union all
select 44248,'MRN' from dual union all
select 44249,'MRN' from dual union all
select 4425,'STA' from dual union all
select 44250,'MRN' from dual union all
select 44251,'MRN' from dual union all
select 44252,'MRN' from dual union all
select 44253,'GMS' from dual union all
select 44254,'MRN' from dual union all
select 44255,'MRN' from dual union all
select 44256,'MRN' from dual union all
select 44257,'MRN' from dual union all
select 44258,'MRN' from dual union all
select 44259,'MRN' from dual union all
select 4426,'USAF' from dual union all
select 44260,'MRN' from dual union all
select 44261,'MRN' from dual union all
select 44262,'MRN' from dual union all
select 44263,'MRN' from dual union all
select 44264,'MRN' from dual union all
select 44265,'GMS' from dual union all
select 44266,'GMS' from dual union all
select 44267,'MRN' from dual union all
select 44268,'MRN' from dual union all
select 44269,'MRN' from dual union all
select 4427,'PARD' from dual union all
select 44270,'NASA' from dual union all
select 44271,'MRN' from dual union all
select 44272,'MRN' from dual union all
select 44273,'MRN' from dual union all
select 44274,'MRN' from dual union all
select 44275,'MRN' from dual union all
select 44276,'MRN' from dual union all
select 44277,'MRN' from dual union all
select 44278,'MRN' from dual union all
select 44279,'MRN' from dual union all
select 4428,'PARD' from dual union all
select 44280,'MRN' from dual union all
select 44281,'MRN' from dual union all
select 44282,'GMS' from dual union all
select 44283,'GMS' from dual union all
select 44284,'MRN' from dual union all
select 44285,'MRN' from dual union all
select 44286,'MRN' from dual union all
select 44287,'MRN' from dual union all
select 44288,'MRN' from dual union all
select 44289,'MRN' from dual union all
select 4429,'USAF' from dual union all
select 44290,'MRN' from dual union all
select 44291,'MRN' from dual union all
select 44292,'MRN' from dual union all
select 44293,'MRN' from dual union all
select 44294,'MRN' from dual union all
select 44295,'MRN' from dual union all
select 44296,'MRN' from dual union all
select 44297,'MRN' from dual union all
select 44298,'MRN' from dual union all
select 44299,'MRN' from dual union all
select 443,'WER485-2' from dual union all
select 4430,'ISAS' from dual union all
select 44300,'MRN' from dual union all
select 44301,'MRN' from dual union all
select 44302,'MRN' from dual union all
select 44303,'MRN' from dual union all
select 44304,'MRN' from dual union all
select 44305,'MRN' from dual union all
select 44306,'MRN' from dual union all
select 44307,'MRN' from dual union all
select 44308,'MRN' from dual union all
select 44309,'MRN' from dual union all
select 4431,'ISAS' from dual union all
select 44310,'MRN' from dual union all
select 44311,'MRN' from dual union all
select 44312,'MRN' from dual union all
select 44313,'MRN' from dual union all
select 44314,'MRN' from dual union all
select 44315,'MRN' from dual union all
select 44316,'MRN' from dual union all
select 44317,'MRN' from dual union all
select 44318,'MRN' from dual union all
select 44319,'MRN' from dual union all
select 4432,'USAF' from dual union all
select 44320,'MRN' from dual union all
select 44321,'MRN' from dual union all
select 44322,'MRN' from dual union all
select 44323,'MRN' from dual union all
select 44324,'MRN' from dual union all
select 44325,'GMS' from dual union all
select 44326,'MRN' from dual union all
select 44327,'MRN' from dual union all
select 44328,'MRN' from dual union all
select 44329,'MRN' from dual union all
select 4433,'USAF' from dual union all
select 44330,'MRN' from dual union all
select 44331,'MRN' from dual;
insert into LAUNCH_AGENCY
select 44332,'MRN' from dual union all
select 44333,'MRN' from dual union all
select 44334,'MRN' from dual union all
select 44335,'MRN' from dual union all
select 44336,'MRN' from dual union all
select 44337,'MRN' from dual union all
select 44338,'MRN' from dual union all
select 44339,'MRN' from dual union all
select 4434,'LRL' from dual union all
select 44340,'MRN' from dual union all
select 44341,'AN' from dual union all
select 44342,'GMS' from dual union all
select 44343,'MRN' from dual union all
select 44344,'MRN' from dual union all
select 44345,'MRN' from dual union all
select 44346,'MRN' from dual union all
select 44347,'MRN' from dual union all
select 44348,'MRN' from dual union all
select 44349,'MRN' from dual union all
select 4435,'LRL' from dual union all
select 44350,'MRN' from dual union all
select 44351,'MRN' from dual union all
select 44352,'MRN' from dual union all
select 44353,'MRN' from dual union all
select 44354,'MRN' from dual union all
select 44355,'MRN' from dual union all
select 44356,'MRN' from dual union all
select 44357,'MRN' from dual union all
select 44358,'MRN' from dual union all
select 44359,'MRN' from dual union all
select 4436,'LRL' from dual union all
select 44360,'MRN' from dual union all
select 44361,'AN' from dual union all
select 44362,'MRN' from dual union all
select 44363,'GMS' from dual union all
select 44364,'MRN' from dual union all
select 44365,'GMS' from dual union all
select 44366,'MRN' from dual union all
select 44367,'MRN' from dual union all
select 44368,'AN' from dual union all
select 44369,'MRN' from dual union all
select 4437,'NRL' from dual union all
select 44370,'MRN' from dual union all
select 44371,'MRN' from dual union all
select 44372,'MRN' from dual union all
select 44373,'AN' from dual union all
select 44374,'MRN' from dual union all
select 44375,'NASA' from dual union all
select 44376,'MRN' from dual union all
select 44377,'MRN' from dual union all
select 44378,'MRN' from dual union all
select 44379,'AN' from dual union all
select 4438,'MVS' from dual union all
select 44380,'MRN' from dual union all
select 44381,'GMS' from dual union all
select 44382,'MRN' from dual union all
select 44383,'AN' from dual union all
select 44384,'MRN' from dual union all
select 44385,'AN' from dual union all
select 44386,'MRN' from dual union all
select 44387,'MRN' from dual union all
select 44388,'MRN' from dual union all
select 44389,'MRN' from dual union all
select 4439,'NRL' from dual union all
select 44390,'MRN' from dual union all
select 44391,'MRN' from dual union all
select 44392,'MRN' from dual union all
select 44393,'MRN' from dual union all
select 44394,'MRN' from dual union all
select 44395,'GMS' from dual union all
select 44396,'MRN' from dual union all
select 44397,'MRN' from dual union all
select 44398,'MRN' from dual union all
select 44399,'MRN' from dual union all
select 444,'WER836-3' from dual union all
select 4440,'USAF' from dual union all
select 44400,'MRN' from dual union all
select 44401,'MRN' from dual union all
select 44402,'MRN' from dual union all
select 44403,'GMS' from dual union all
select 44404,'MRN' from dual union all
select 44405,'MRN' from dual union all
select 44406,'MRN' from dual union all
select 44407,'MRN' from dual union all
select 44408,'MRN' from dual union all
select 44409,'MRN' from dual union all
select 4441,'USAF' from dual union all
select 44410,'MRN' from dual union all
select 44411,'MRN' from dual union all
select 44412,'MRN' from dual union all
select 44413,'MRN' from dual union all
select 44414,'MRN' from dual union all
select 44415,'MRN' from dual union all
select 44416,'MRN' from dual union all
select 44417,'MRN' from dual union all
select 44418,'MRN' from dual union all
select 44419,'MRN' from dual union all
select 4442,'SAND' from dual union all
select 44420,'MRN' from dual union all
select 44421,'MRN' from dual;
insert into LAUNCH_AGENCY
select 44422,'GMS' from dual union all
select 44423,'MRN' from dual union all
select 44424,'MRN' from dual union all
select 44425,'MRN' from dual union all
select 44426,'MRN' from dual union all
select 44427,'MRN' from dual union all
select 44428,'MRN' from dual union all
select 44429,'MRN' from dual union all
select 4443,'BRL' from dual union all
select 44430,'MRN' from dual union all
select 44431,'MRN' from dual union all
select 44432,'MRN' from dual union all
select 44433,'MRN' from dual union all
select 44434,'MRN' from dual union all
select 44435,'GMS' from dual union all
select 44436,'GMS' from dual union all
select 44437,'MRN' from dual union all
select 44438,'MRN' from dual union all
select 44439,'MRN' from dual union all
select 4444,'LRL' from dual union all
select 44440,'MRN' from dual union all
select 44441,'MRN' from dual union all
select 44442,'MRN' from dual union all
select 44443,'MRN' from dual union all
select 44444,'MRN' from dual union all
select 44445,'MRN' from dual union all
select 44446,'MRN' from dual union all
select 44447,'MRN' from dual union all
select 44448,'MRN' from dual union all
select 44449,'MRN' from dual union all
select 4445,'LRL' from dual union all
select 44450,'GMS' from dual union all
select 44451,'MRN' from dual union all
select 44452,'MRN' from dual union all
select 44453,'GMS' from dual union all
select 44454,'MRN' from dual union all
select 44455,'MRN' from dual union all
select 44456,'MRN' from dual union all
select 44457,'MRN' from dual union all
select 44458,'MRN' from dual union all
select 44459,'MRN' from dual union all
select 4446,'SU' from dual union all
select 44460,'MRN' from dual union all
select 44461,'MRN' from dual union all
select 44462,'MRN' from dual union all
select 44463,'MRN' from dual union all
select 44464,'MRN' from dual union all
select 44465,'MRN' from dual union all
select 44466,'MRN' from dual union all
select 44467,'MRN' from dual union all
select 44468,'MRN' from dual union all
select 44469,'GMS' from dual union all
select 4447,'PARD' from dual union all
select 44470,'MRN' from dual union all
select 44471,'MRN' from dual union all
select 44472,'MRN' from dual union all
select 44473,'MRN' from dual union all
select 44474,'MRN' from dual union all
select 44475,'MRN' from dual union all
select 44476,'MRN' from dual union all
select 44477,'MRN' from dual union all
select 44478,'MRN' from dual union all
select 44479,'MRN' from dual union all
select 4448,'NRL' from dual union all
select 44480,'MRN' from dual union all
select 44481,'MRN' from dual union all
select 44482,'MRN' from dual union all
select 44483,'MRN' from dual union all
select 44484,'MRN' from dual union all
select 44485,'MRN' from dual union all
select 44486,'MRN' from dual union all
select 44487,'MRN' from dual union all
select 44488,'MRN' from dual union all
select 44489,'MRN' from dual union all
select 4449,'LERC' from dual union all
select 44490,'MRN' from dual union all
select 44491,'MRN' from dual union all
select 44492,'MRN' from dual union all
select 44493,'MRN' from dual union all
select 44494,'MRN' from dual union all
select 44495,'MRN' from dual union all
select 44496,'GMS' from dual union all
select 44497,'MRN' from dual union all
select 44498,'MRN' from dual union all
select 44499,'MRN' from dual union all
select 445,'WER485-2' from dual union all
select 4450,'SUI' from dual union all
select 44500,'MRN' from dual union all
select 44501,'MRN' from dual union all
select 44502,'MRN' from dual union all
select 44503,'MRN' from dual union all
select 44504,'MRN' from dual union all
select 44505,'MRN' from dual union all
select 44506,'MRN' from dual union all
select 44507,'MRN' from dual union all
select 44508,'MRN' from dual union all
select 44509,'MRN' from dual union all
select 4451,'SUI' from dual union all
select 44510,'MRN' from dual union all
select 44511,'MRN' from dual;
insert into LAUNCH_AGENCY
select 44512,'MRN' from dual union all
select 44513,'MRN' from dual union all
select 44514,'MRN' from dual union all
select 44515,'MRN' from dual union all
select 44516,'MRN' from dual union all
select 44517,'MRN' from dual union all
select 44518,'MRN' from dual union all
select 44519,'MRN' from dual union all
select 4452,'SUI' from dual union all
select 44520,'MRN' from dual union all
select 44521,'MRN' from dual union all
select 44522,'MRN' from dual union all
select 44523,'MRN' from dual union all
select 44524,'MRN' from dual union all
select 44525,'MRN' from dual union all
select 44526,'MRN' from dual union all
select 44527,'MRN' from dual union all
select 44528,'MRN' from dual union all
select 44529,'MRN' from dual union all
select 4453,'USAF' from dual union all
select 44530,'MRN' from dual union all
select 44531,'MRN' from dual union all
select 44532,'MRN' from dual union all
select 44533,'GMS' from dual union all
select 44534,'MRN' from dual union all
select 44535,'MRN' from dual union all
select 44536,'MRN' from dual union all
select 44537,'MRN' from dual union all
select 44538,'MRN' from dual union all
select 44539,'MRN' from dual union all
select 4454,'SUI' from dual union all
select 44540,'MRN' from dual union all
select 44541,'MRN' from dual union all
select 44542,'MRN' from dual union all
select 44543,'MRN' from dual union all
select 44544,'MRN' from dual union all
select 44545,'NASA' from dual union all
select 44546,'MRN' from dual union all
select 44547,'MRN' from dual union all
select 44548,'MRN' from dual union all
select 44549,'NASA' from dual union all
select 4455,'PARD' from dual union all
select 44550,'MRN' from dual union all
select 44551,'MRN' from dual union all
select 44552,'MRN' from dual union all
select 44553,'MRN' from dual union all
select 44554,'MRN' from dual union all
select 44555,'MRN' from dual union all
select 44556,'MRN' from dual union all
select 44557,'MRN' from dual union all
select 44558,'MRN' from dual union all
select 44559,'MRN' from dual union all
select 4456,'USAF' from dual union all
select 44560,'MRN' from dual union all
select 44561,'MRN' from dual union all
select 44562,'MRN' from dual union all
select 44563,'MRN' from dual union all
select 44564,'MRN' from dual union all
select 44565,'MRN' from dual union all
select 44566,'MRN' from dual union all
select 44567,'MRN' from dual union all
select 44568,'MRN' from dual union all
select 44569,'MRN' from dual union all
select 4457,'USAF' from dual union all
select 44570,'MRN' from dual union all
select 44571,'MRN' from dual union all
select 44572,'MRN' from dual union all
select 44573,'MRN' from dual union all
select 44574,'GMS' from dual union all
select 44575,'MRN' from dual union all
select 44576,'MRN' from dual union all
select 44577,'MRN' from dual union all
select 44578,'AN' from dual union all
select 44579,'MRN' from dual union all
select 4458,'USAF' from dual union all
select 44580,'MRN' from dual union all
select 44581,'MRN' from dual union all
select 44582,'MRN' from dual union all
select 44583,'MRN' from dual union all
select 44584,'MRN' from dual union all
select 44585,'MRN' from dual union all
select 44586,'MRN' from dual union all
select 44587,'MRN' from dual union all
select 44588,'MRN' from dual union all
select 44589,'MRN' from dual union all
select 4459,'USA' from dual union all
select 44590,'MRN' from dual union all
select 44591,'MRN' from dual union all
select 44592,'MRN' from dual union all
select 44593,'MRN' from dual union all
select 44594,'AN' from dual union all
select 44595,'MRN' from dual union all
select 44596,'MRN' from dual union all
select 44597,'MRN' from dual union all
select 44598,'MRN' from dual union all
select 44599,'MRN' from dual union all
select 446,'WER500' from dual union all
select 4460,'BRL' from dual union all
select 44600,'MRN' from dual union all
select 44601,'MRN' from dual;
insert into LAUNCH_AGENCY
select 44602,'MRN' from dual union all
select 44603,'MRN' from dual union all
select 44604,'MRN' from dual union all
select 44605,'MRN' from dual union all
select 44606,'MRN' from dual union all
select 44607,'MRN' from dual union all
select 44608,'MRN' from dual union all
select 44609,'MRN' from dual union all
select 4461,'USAF' from dual union all
select 44610,'MRN' from dual union all
select 44611,'MRN' from dual union all
select 44612,'MRN' from dual union all
select 44613,'MRN' from dual union all
select 44614,'MRN' from dual union all
select 44615,'MRN' from dual union all
select 44616,'MRN' from dual union all
select 44617,'MRN' from dual union all
select 44618,'MRN' from dual union all
select 44619,'MRN' from dual union all
select 4462,'USN' from dual union all
select 44620,'MRN' from dual union all
select 44621,'MRN' from dual union all
select 44622,'MRN' from dual union all
select 44623,'GMS' from dual union all
select 44624,'MRN' from dual union all
select 44625,'MRN' from dual union all
select 44626,'MRN' from dual union all
select 44627,'MRN' from dual union all
select 44628,'MRN' from dual union all
select 44629,'MRN' from dual union all
select 4463,'PARD' from dual union all
select 44630,'MRN' from dual union all
select 44631,'MRN' from dual union all
select 44632,'MRN' from dual union all
select 44633,'MRN' from dual union all
select 44634,'MRN' from dual union all
select 44635,'MRN' from dual union all
select 44636,'MRN' from dual union all
select 44637,'MRN' from dual union all
select 44638,'MRN' from dual union all
select 44639,'GMS' from dual union all
select 4464,'USAF' from dual union all
select 44640,'MRN' from dual union all
select 44641,'MRN' from dual union all
select 44642,'GMS' from dual union all
select 44643,'MRN' from dual union all
select 44644,'MRN' from dual union all
select 44645,'MRN' from dual union all
select 44646,'MRN' from dual union all
select 44647,'MRN' from dual union all
select 44648,'MRN' from dual union all
select 44649,'MRN' from dual union all
select 4465,'6555A' from dual union all
select 44650,'MRN' from dual union all
select 44651,'GMS' from dual union all
select 44652,'MRN' from dual union all
select 44653,'MRN' from dual union all
select 44654,'MRN' from dual union all
select 44655,'MRN' from dual union all
select 44656,'MRN' from dual union all
select 44657,'MRN' from dual union all
select 44658,'MRN' from dual union all
select 44659,'MRN' from dual union all
select 4466,'LRL' from dual union all
select 44660,'GMS' from dual union all
select 44661,'GMS' from dual union all
select 44662,'MRN' from dual union all
select 44663,'AN' from dual union all
select 44664,'MRN' from dual union all
select 44665,'MRN' from dual union all
select 44666,'MRN' from dual union all
select 44667,'MRN' from dual union all
select 44668,'MRN' from dual union all
select 44669,'MRN' from dual union all
select 4467,'LRL' from dual union all
select 44670,'MRN' from dual union all
select 44671,'MRN' from dual union all
select 44672,'MRN' from dual union all
select 44673,'MRN' from dual union all
select 44674,'MRN' from dual union all
select 44675,'MRN' from dual union all
select 44676,'MRN' from dual union all
select 44677,'MRN' from dual union all
select 44678,'MRN' from dual union all
select 44679,'MRN' from dual union all
select 4468,'SU' from dual union all
select 44680,'MRN' from dual union all
select 44681,'MRN' from dual union all
select 44682,'MRN' from dual union all
select 44683,'MRN' from dual union all
select 44684,'MRN' from dual union all
select 44685,'MRN' from dual union all
select 44686,'MRN' from dual union all
select 44687,'MRN' from dual union all
select 44688,'MRN' from dual union all
select 44689,'MRN' from dual union all
select 4469,'USAF' from dual union all
select 44690,'MRN' from dual union all
select 44691,'MRN' from dual union all
select 44692,'MRN' from dual;
insert into LAUNCH_AGENCY
select 44693,'MRN' from dual union all
select 44694,'MRN' from dual union all
select 44695,'MRN' from dual union all
select 44696,'MRN' from dual union all
select 44697,'MRN' from dual union all
select 44698,'MRN' from dual union all
select 44699,'MRN' from dual union all
select 447,'WER500' from dual union all
select 4470,'USAF' from dual union all
select 44700,'MRN' from dual union all
select 44701,'GMS' from dual union all
select 44702,'AN' from dual union all
select 44703,'AN' from dual union all
select 44704,'AN' from dual union all
select 44705,'AN' from dual union all
select 44706,'AN' from dual union all
select 44707,'AN' from dual union all
select 44708,'AN' from dual union all
select 44709,'AN' from dual union all
select 4471,'USAF' from dual union all
select 44710,'AN' from dual union all
select 44711,'AN' from dual union all
select 44712,'AN' from dual union all
select 44713,'AN' from dual union all
select 44714,'AN' from dual union all
select 44715,'AN' from dual union all
select 44716,'AN' from dual union all
select 44717,'MRN' from dual union all
select 44718,'MRN' from dual union all
select 44719,'AN' from dual union all
select 4472,'USN' from dual union all
select 44720,'AN' from dual union all
select 44721,'AN' from dual union all
select 44722,'MRN' from dual union all
select 44723,'AN' from dual union all
select 44724,'AN' from dual union all
select 44725,'MRN' from dual union all
select 44726,'AN' from dual union all
select 44727,'AN' from dual union all
select 44728,'AN' from dual union all
select 44729,'AN' from dual union all
select 4473,'USN' from dual union all
select 44730,'AN' from dual union all
select 44731,'AN' from dual union all
select 44732,'AN' from dual union all
select 44733,'NASA' from dual union all
select 44734,'AN' from dual union all
select 44735,'AN' from dual union all
select 44736,'AN' from dual union all
select 44737,'MRN' from dual union all
select 44738,'MRN' from dual union all
select 44739,'MRN' from dual union all
select 4474,'USAF' from dual union all
select 44740,'MRN' from dual union all
select 44741,'AN' from dual union all
select 44742,'AN' from dual union all
select 44743,'AN' from dual union all
select 44744,'AN' from dual union all
select 44745,'AN' from dual union all
select 44746,'AN' from dual union all
select 44747,'AN' from dual union all
select 44748,'AN' from dual union all
select 44749,'AN' from dual union all
select 4475,'USAF' from dual union all
select 44750,'AN' from dual union all
select 44751,'AN' from dual union all
select 44752,'MRN' from dual union all
select 44753,'MRN' from dual union all
select 44754,'AN' from dual union all
select 44755,'AN' from dual union all
select 44756,'MRN' from dual union all
select 44757,'MRN' from dual union all
select 44758,'AN' from dual union all
select 44759,'AN' from dual union all
select 4476,'USN' from dual union all
select 44760,'AN' from dual union all
select 44761,'AN' from dual union all
select 44762,'AN' from dual union all
select 44763,'AN' from dual union all
select 44764,'MRN' from dual union all
select 44765,'MRN' from dual union all
select 44766,'AN' from dual union all
select 44767,'AN' from dual union all
select 44768,'MRN' from dual union all
select 44769,'AN' from dual union all
select 4477,'USA' from dual union all
select 44770,'AN' from dual union all
select 44771,'AN' from dual union all
select 44772,'AN' from dual union all
select 44773,'MRN' from dual union all
select 44774,'MRN' from dual union all
select 44775,'MRN' from dual union all
select 44776,'AN' from dual union all
select 44777,'MRN' from dual union all
select 44778,'AN' from dual union all
select 44779,'AN' from dual union all
select 4478,'SUI' from dual union all
select 44780,'AN' from dual union all
select 44781,'AN' from dual union all
select 44782,'MRN' from dual;
insert into LAUNCH_AGENCY
select 44783,'MRN' from dual union all
select 44784,'AN' from dual union all
select 44785,'MRN' from dual union all
select 44786,'AN' from dual union all
select 44787,'AN' from dual union all
select 44788,'MRN' from dual union all
select 44789,'AN' from dual union all
select 4479,'SUI' from dual union all
select 44790,'AN' from dual union all
select 44791,'MRN' from dual union all
select 44792,'MRN' from dual union all
select 44793,'AN' from dual union all
select 44794,'AN' from dual union all
select 44795,'AN' from dual union all
select 44796,'AN' from dual union all
select 44797,'MRN' from dual union all
select 44798,'AN' from dual union all
select 44799,'MRN' from dual union all
select 448,'WER500' from dual union all
select 4480,'SUI' from dual union all
select 44800,'AN' from dual union all
select 44801,'AN' from dual union all
select 44802,'AN' from dual union all
select 44803,'AN' from dual union all
select 44804,'AN' from dual union all
select 44805,'MRN' from dual union all
select 44806,'MRN' from dual union all
select 44807,'MRN' from dual union all
select 44808,'AN' from dual union all
select 44809,'AN' from dual union all
select 4481,'SAND' from dual union all
select 44810,'AN' from dual union all
select 44811,'AN' from dual union all
select 44812,'AN' from dual union all
select 44813,'AN' from dual union all
select 44814,'AN' from dual union all
select 44815,'AN' from dual union all
select 44816,'MRN' from dual union all
select 44817,'MRN' from dual union all
select 44818,'MRN' from dual union all
select 44819,'AN' from dual union all
select 4482,'USA' from dual union all
select 44820,'AN' from dual union all
select 44821,'MRN' from dual union all
select 44822,'MRN' from dual union all
select 44823,'AN' from dual union all
select 44824,'AN' from dual union all
select 44825,'AN' from dual union all
select 44826,'AN' from dual union all
select 44827,'AN' from dual union all
select 44828,'MRN' from dual union all
select 44829,'AN' from dual union all
select 4483,'ARDC' from dual union all
select 44830,'MRN' from dual union all
select 44831,'MRN' from dual union all
select 44832,'AN' from dual union all
select 44833,'MRN' from dual union all
select 44834,'AN' from dual union all
select 44835,'AN' from dual union all
select 44836,'MRN' from dual union all
select 44837,'AN' from dual union all
select 44838,'AN' from dual union all
select 44839,'AN' from dual union all
select 4484,'RAE' from dual union all
select 4484,'WRE' from dual union all
select 44840,'AN' from dual union all
select 44841,'AN' from dual union all
select 44842,'AN' from dual union all
select 44843,'AN' from dual union all
select 44844,'MRN' from dual union all
select 44845,'MRN' from dual union all
select 44846,'MRN' from dual union all
select 44847,'AN' from dual union all
select 44848,'AN' from dual union all
select 44849,'AN' from dual union all
select 4485,'SUI' from dual union all
select 44850,'AN' from dual union all
select 44851,'AN' from dual union all
select 44852,'AN' from dual union all
select 44853,'MRN' from dual union all
select 44854,'MRN' from dual union all
select 44855,'MRN' from dual union all
select 44856,'AN' from dual union all
select 44857,'AN' from dual union all
select 44858,'AN' from dual union all
select 44859,'AN' from dual union all
select 4486,'USA' from dual union all
select 44860,'MRN' from dual union all
select 44861,'MRN' from dual union all
select 44862,'AN' from dual union all
select 44863,'AN' from dual union all
select 44864,'AN' from dual union all
select 44865,'MRN' from dual union all
select 44866,'AN' from dual union all
select 44867,'MRN' from dual union all
select 44868,'MRN' from dual union all
select 44869,'MRN' from dual union all
select 4487,'USA' from dual union all
select 44870,'AN' from dual union all
select 44871,'MRN' from dual;
insert into LAUNCH_AGENCY
select 44872,'AN' from dual union all
select 44873,'AN' from dual union all
select 44874,'AN' from dual union all
select 44875,'MRN' from dual union all
select 44876,'AN' from dual union all
select 44877,'MRN' from dual union all
select 44878,'MRN' from dual union all
select 44879,'AN' from dual union all
select 4488,'NRL' from dual union all
select 44880,'AN' from dual union all
select 44881,'AN' from dual union all
select 44882,'MRN' from dual union all
select 44883,'MRN' from dual union all
select 44884,'MRN' from dual union all
select 44885,'AN' from dual union all
select 44886,'AN' from dual union all
select 44887,'AN' from dual union all
select 44888,'AN' from dual union all
select 44889,'AN' from dual union all
select 4489,'USN' from dual union all
select 44890,'MRN' from dual union all
select 44891,'AN' from dual union all
select 44892,'AN' from dual union all
select 44893,'MRN' from dual union all
select 44894,'AN' from dual union all
select 44895,'AN' from dual union all
select 44896,'AN' from dual union all
select 44897,'MRN' from dual union all
select 44898,'MRN' from dual union all
select 44899,'AN' from dual union all
select 449,'WER485-2' from dual union all
select 4490,'USA' from dual union all
select 44900,'NASA' from dual union all
select 44901,'MRN' from dual union all
select 44902,'MRN' from dual union all
select 44903,'MRN' from dual union all
select 44904,'AN' from dual union all
select 44905,'AN' from dual union all
select 44906,'AN' from dual union all
select 44907,'MRN' from dual union all
select 44908,'MRN' from dual union all
select 44909,'MRN' from dual union all
select 4491,'USA' from dual union all
select 44910,'MRN' from dual union all
select 44911,'MRN' from dual union all
select 44912,'MRN' from dual union all
select 44913,'AN' from dual union all
select 44914,'AN' from dual union all
select 44915,'AN' from dual union all
select 44916,'AN' from dual union all
select 44917,'MRN' from dual union all
select 44918,'MRN' from dual union all
select 44919,'MRN' from dual union all
select 4492,'NSA' from dual union all
select 44920,'MRN' from dual union all
select 44921,'MRN' from dual union all
select 44922,'AN' from dual union all
select 44923,'AN' from dual union all
select 44924,'AN' from dual union all
select 44925,'AN' from dual union all
select 44926,'MRN' from dual union all
select 44927,'MRN' from dual union all
select 44928,'AN' from dual union all
select 44929,'MRN' from dual union all
select 4493,'NSA' from dual union all
select 44930,'AN' from dual union all
select 44931,'AN' from dual union all
select 44932,'AN' from dual union all
select 44933,'AN' from dual union all
select 44934,'MRN' from dual union all
select 44935,'AN' from dual union all
select 44936,'AN' from dual union all
select 44937,'AN' from dual union all
select 44938,'AN' from dual union all
select 44939,'AN' from dual union all
select 4494,'NSA' from dual union all
select 44940,'MRN' from dual union all
select 44941,'AN' from dual union all
select 44942,'AN' from dual union all
select 44943,'MRN' from dual union all
select 44944,'AN' from dual union all
select 44945,'AN' from dual union all
select 44946,'AN' from dual union all
select 44947,'AN' from dual union all
select 44948,'MRN' from dual union all
select 44949,'MRN' from dual union all
select 4495,'NSA' from dual union all
select 44950,'MRN' from dual union all
select 44951,'MRN' from dual union all
select 44952,'MRN' from dual union all
select 44953,'AN' from dual union all
select 44954,'AN' from dual union all
select 44955,'AN' from dual union all
select 44956,'AN' from dual union all
select 44957,'MRN' from dual union all
select 44958,'AN' from dual union all
select 44959,'AN' from dual union all
select 4496,'SAND' from dual union all
select 44960,'AN' from dual union all
select 44961,'MRN' from dual;
insert into LAUNCH_AGENCY
select 44962,'MRN' from dual union all
select 44963,'MRN' from dual union all
select 44964,'AN' from dual union all
select 44965,'MRN' from dual union all
select 44966,'AN' from dual union all
select 44967,'AN' from dual union all
select 44968,'MRN' from dual union all
select 44969,'MRN' from dual union all
select 4497,'6555A' from dual union all
select 44970,'MRN' from dual union all
select 44971,'MRN' from dual union all
select 44972,'MRN' from dual union all
select 44973,'AN' from dual union all
select 44974,'AN' from dual union all
select 44975,'AN' from dual union all
select 44976,'MRN' from dual union all
select 44977,'MRN' from dual union all
select 44978,'AN' from dual union all
select 44979,'NASA' from dual union all
select 4498,'6555A' from dual union all
select 44980,'MRN' from dual union all
select 44981,'ISRO' from dual union all
select 44982,'AN' from dual union all
select 44983,'AN' from dual union all
select 44984,'AN' from dual union all
select 44985,'AN' from dual union all
select 44986,'AN' from dual union all
select 44987,'MRN' from dual union all
select 44988,'AN' from dual union all
select 44989,'AN' from dual union all
select 4499,'RVSN' from dual union all
select 44990,'AN' from dual union all
select 44991,'AN' from dual union all
select 44992,'AN' from dual union all
select 44993,'AN' from dual union all
select 44994,'AN' from dual union all
select 44995,'MRN' from dual union all
select 44996,'MRN' from dual union all
select 44997,'AN' from dual union all
select 44998,'MRN' from dual union all
select 44999,'AN' from dual union all
select 45,'WER444' from dual union all
select 450,'WER485-2' from dual union all
select 4500,'USA' from dual union all
select 45000,'AN' from dual union all
select 45001,'AN' from dual union all
select 45002,'AN' from dual union all
select 45003,'MRN' from dual union all
select 45004,'MRN' from dual union all
select 45005,'AN' from dual union all
select 45006,'AN' from dual union all
select 45007,'AN' from dual union all
select 45008,'AN' from dual union all
select 45009,'AN' from dual union all
select 4501,'ATEF' from dual union all
select 45010,'AN' from dual union all
select 45011,'AN' from dual union all
select 45012,'AN' from dual union all
select 45013,'MRN' from dual union all
select 45014,'MRN' from dual union all
select 45015,'MRN' from dual union all
select 45016,'MRN' from dual union all
select 45017,'MRN' from dual union all
select 45018,'AN' from dual union all
select 45019,'MRN' from dual union all
select 4502,'USN' from dual union all
select 45020,'AN' from dual union all
select 45021,'AN' from dual union all
select 45022,'AN' from dual union all
select 45023,'AN' from dual union all
select 45024,'MRN' from dual union all
select 45025,'MRN' from dual union all
select 45026,'MRN' from dual union all
select 45027,'MRN' from dual union all
select 45028,'MRN' from dual union all
select 45029,'MRN' from dual union all
select 4503,'MVS' from dual union all
select 45030,'AN' from dual union all
select 45031,'MRN' from dual union all
select 45032,'MRN' from dual union all
select 45033,'MRN' from dual union all
select 45034,'MRN' from dual union all
select 45035,'MRN' from dual union all
select 45036,'MRN' from dual union all
select 45037,'MRN' from dual union all
select 45038,'MRN' from dual union all
select 45039,'MRN' from dual union all
select 4504,'USAF' from dual union all
select 45040,'AN' from dual union all
select 45041,'AN' from dual union all
select 45042,'MRN' from dual union all
select 45043,'MRN' from dual union all
select 45044,'MRN' from dual union all
select 45045,'MRN' from dual union all
select 45046,'MRN' from dual union all
select 45047,'AN' from dual union all
select 45048,'AN' from dual union all
select 45049,'AN' from dual union all
select 4505,'UNRV' from dual union all
select 45050,'AN' from dual;
insert into LAUNCH_AGENCY
select 45051,'AN' from dual union all
select 45052,'AN' from dual union all
select 45053,'MRN' from dual union all
select 45054,'MRN' from dual union all
select 45055,'MRN' from dual union all
select 45056,'AN' from dual union all
select 45057,'MRN' from dual union all
select 45058,'AN' from dual union all
select 45059,'AN' from dual union all
select 4506,'USAF' from dual union all
select 45060,'AN' from dual union all
select 45061,'AN' from dual union all
select 45062,'AN' from dual union all
select 45063,'AN' from dual union all
select 45064,'AN' from dual union all
select 45065,'MRN' from dual union all
select 45066,'MRN' from dual union all
select 45067,'MRN' from dual union all
select 45068,'MRN' from dual union all
select 45069,'AN' from dual union all
select 4507,'UNRV' from dual union all
select 45070,'AN' from dual union all
select 45071,'AN' from dual union all
select 45072,'AN' from dual union all
select 45073,'AN' from dual union all
select 45074,'AN' from dual union all
select 45075,'AN' from dual union all
select 45076,'AN' from dual union all
select 45077,'AN' from dual union all
select 45078,'AN' from dual union all
select 45079,'MRN' from dual union all
select 4508,'USN' from dual union all
select 45080,'MRN' from dual union all
select 45081,'MRN' from dual union all
select 45082,'MRN' from dual union all
select 45083,'AN' from dual union all
select 45084,'AN' from dual union all
select 45085,'MRN' from dual union all
select 45086,'AN' from dual union all
select 45087,'AN' from dual union all
select 45088,'AN' from dual union all
select 45089,'AN' from dual union all
select 4509,'MVS' from dual union all
select 45090,'AN' from dual union all
select 45091,'AN' from dual union all
select 45092,'AN' from dual union all
select 45093,'AN' from dual union all
select 45094,'MRN' from dual union all
select 45095,'MRN' from dual union all
select 45096,'AN' from dual union all
select 45097,'MRN' from dual union all
select 45098,'MRN' from dual union all
select 45099,'MRN' from dual union all
select 451,'WEHR' from dual union all
select 4510,'UNRV' from dual union all
select 45100,'AN' from dual union all
select 45101,'AN' from dual union all
select 45102,'AN' from dual union all
select 45103,'AN' from dual union all
select 45104,'AN' from dual union all
select 45105,'AN' from dual union all
select 45106,'AN' from dual union all
select 45107,'AN' from dual union all
select 45108,'MRN' from dual union all
select 45109,'AN' from dual union all
select 4511,'USA' from dual union all
select 45110,'AN' from dual union all
select 45111,'AN' from dual union all
select 45112,'AN' from dual union all
select 45113,'AN' from dual union all
select 45114,'AN' from dual union all
select 45115,'AN' from dual union all
select 45116,'AN' from dual union all
select 45117,'MRN' from dual union all
select 45118,'AN' from dual union all
select 45119,'AN' from dual union all
select 4512,'ATEF' from dual union all
select 45120,'AN' from dual union all
select 45121,'AN' from dual union all
select 45122,'AN' from dual union all
select 45123,'AN' from dual union all
select 45124,'AN' from dual union all
select 45125,'AN' from dual union all
select 45126,'AN' from dual union all
select 45127,'AN' from dual union all
select 45128,'MRN' from dual union all
select 45129,'AN' from dual union all
select 4513,'USAF' from dual union all
select 45130,'AN' from dual union all
select 45131,'AN' from dual union all
select 45132,'AN' from dual union all
select 45133,'AN' from dual union all
select 45134,'AN' from dual union all
select 45135,'AN' from dual union all
select 45136,'AN' from dual union all
select 45137,'AN' from dual union all
select 45138,'AN' from dual union all
select 45139,'AN' from dual union all
select 4514,'6555A' from dual union all
select 45140,'AN' from dual;
insert into LAUNCH_AGENCY
select 45141,'MRN' from dual union all
select 45142,'AN' from dual union all
select 45143,'AN' from dual union all
select 45144,'AN' from dual union all
select 45145,'AN' from dual union all
select 45146,'AN' from dual union all
select 45147,'AN' from dual union all
select 45148,'AN' from dual union all
select 45149,'AN' from dual union all
select 4515,'USAF' from dual union all
select 45150,'AN' from dual union all
select 45151,'MRN' from dual union all
select 45152,'MRN' from dual union all
select 45153,'MRN' from dual union all
select 45154,'AN' from dual union all
select 45155,'AN' from dual union all
select 45156,'MRN' from dual union all
select 45157,'NASA' from dual union all
select 45158,'MRN' from dual union all
select 45159,'NASA' from dual union all
select 4516,'SUI' from dual union all
select 45160,'AN' from dual union all
select 45161,'AN' from dual union all
select 45162,'AN' from dual union all
select 45163,'AN' from dual union all
select 45164,'AN' from dual union all
select 45165,'AN' from dual union all
select 45166,'AN' from dual union all
select 45167,'MRN' from dual union all
select 45168,'AN' from dual union all
select 45169,'AN' from dual union all
select 4517,'SUI' from dual union all
select 45170,'AN' from dual union all
select 45171,'AN' from dual union all
select 45172,'MRN' from dual union all
select 45173,'AN' from dual union all
select 45174,'MRN' from dual union all
select 45175,'AN' from dual union all
select 45176,'NRCC' from dual union all
select 45177,'AN' from dual union all
select 45178,'AN' from dual union all
select 45179,'AN' from dual union all
select 4518,'WRE' from dual union all
select 45180,'AN' from dual union all
select 45181,'MRN' from dual union all
select 45182,'MRN' from dual union all
select 45183,'AN' from dual union all
select 45184,'AN' from dual union all
select 45185,'MRN' from dual union all
select 45186,'AN' from dual union all
select 45187,'AN' from dual union all
select 45188,'AN' from dual union all
select 45189,'AN' from dual union all
select 4519,'USA' from dual union all
select 45190,'AN' from dual union all
select 45191,'MRN' from dual union all
select 45192,'AN' from dual union all
select 45193,'AN' from dual union all
select 45194,'AN' from dual union all
select 45195,'AN' from dual union all
select 45196,'RVSN' from dual union all
select 45197,'NASA' from dual union all
select 45198,'RVSN' from dual union all
select 45199,'RVSN' from dual union all
select 452,'WER836-1' from dual union all
select 4520,'USAF' from dual union all
select 45200,'RVSN' from dual union all
select 45201,'RVSN' from dual union all
select 45202,'RVSN' from dual union all
select 45203,'RVSN' from dual union all
select 45204,'RVSN' from dual union all
select 45205,'RVSN' from dual union all
select 45206,'CALT' from dual union all
select 45207,'NASA' from dual union all
select 45208,'RVSN' from dual union all
select 45209,'ISAS' from dual union all
select 4521,'SUI' from dual union all
select 45210,'RVSN' from dual union all
select 45211,'NASA' from dual union all
select 45212,'RVSN' from dual union all
select 45213,'NASDA' from dual union all
select 45214,'RVSN' from dual union all
select 45215,'AFSC' from dual union all
select 45216,'USAF' from dual union all
select 45217,'RVSN' from dual union all
select 45218,'RVSN' from dual union all
select 45219,'RVSN' from dual union all
select 4522,'USAF' from dual union all
select 45220,'RVSN' from dual union all
select 45221,'NASA' from dual union all
select 45222,'RVSN' from dual union all
select 45223,'RVSN' from dual union all
select 45224,'AFSC' from dual union all
select 45225,'RVSN' from dual union all
select 45226,'RVSN' from dual union all
select 45227,'RVSN' from dual union all
select 45228,'RVSN' from dual union all
select 45229,'RVSN' from dual union all
select 4523,'ARDC' from dual union all
select 45230,'NASA' from dual;
insert into LAUNCH_AGENCY
select 45231,'RVSN' from dual union all
select 45232,'RVSN' from dual union all
select 45233,'AFSC' from dual union all
select 45234,'NASA' from dual union all
select 45235,'RVSN' from dual union all
select 45236,'NASA' from dual union all
select 45237,'10ADS' from dual union all
select 45238,'RVSN' from dual union all
select 45239,'NASA' from dual union all
select 4524,'6555A' from dual union all
select 45240,'RVSN' from dual union all
select 45241,'RVSN' from dual union all
select 45242,'AFSC' from dual union all
select 45243,'RVSN' from dual union all
select 45244,'RVSN' from dual union all
select 45245,'RVSN' from dual union all
select 45246,'NASA' from dual union all
select 45247,'RVSN' from dual union all
select 45248,'RVSN' from dual union all
select 45249,'RVSN' from dual union all
select 4525,'SUI' from dual union all
select 45250,'RVSN' from dual union all
select 45251,'RVSN' from dual union all
select 45252,'RVSN' from dual union all
select 45253,'AFSC' from dual union all
select 45254,'RVSN' from dual union all
select 45255,'AFSC' from dual union all
select 45256,'RVSN' from dual union all
select 45257,'NASA' from dual union all
select 45258,'RVSN' from dual union all
select 45259,'AFSC' from dual union all
select 4526,'USAF' from dual union all
select 45260,'RVSN' from dual union all
select 45261,'RVSN' from dual union all
select 45262,'RVSN' from dual union all
select 45263,'NASA' from dual union all
select 45264,'RVSN' from dual union all
select 45265,'RVSN' from dual union all
select 45266,'NASA' from dual union all
select 45267,'RVSN' from dual union all
select 45268,'RVSN' from dual union all
select 45269,'RVSN' from dual union all
select 4527,'SUI' from dual union all
select 45270,'AFSC' from dual union all
select 45271,'RVSN' from dual union all
select 45272,'RVSN' from dual union all
select 45273,'NASA' from dual union all
select 45274,'NASA' from dual union all
select 45275,'RVSN' from dual union all
select 45276,'RVSN' from dual union all
select 45277,'RVSN' from dual union all
select 45278,'RVSN' from dual union all
select 45279,'RVSN' from dual union all
select 4528,'SUI' from dual union all
select 45280,'RVSN' from dual union all
select 45281,'RVSN' from dual union all
select 45282,'ISAS' from dual union all
select 45283,'RVSN' from dual union all
select 45284,'RVSN' from dual union all
select 45285,'RVSN' from dual union all
select 45286,'RVSN' from dual union all
select 45287,'RVSN' from dual union all
select 45288,'AFSC' from dual union all
select 45289,'RVSN' from dual union all
select 4529,'SUI' from dual union all
select 45290,'RVSN' from dual union all
select 45291,'AFSC' from dual union all
select 45292,'RVSN' from dual union all
select 45293,'NASA' from dual union all
select 45294,'RVSN' from dual union all
select 45295,'RVSN' from dual union all
select 45296,'RVSN' from dual union all
select 45297,'RVSN' from dual union all
select 45298,'NASA' from dual union all
select 45299,'RVSN' from dual union all
select 453,'WER444' from dual union all
select 4530,'SUI' from dual union all
select 45300,'RVSN' from dual union all
select 45301,'NASA' from dual union all
select 45302,'RVSN' from dual union all
select 45303,'RVSN' from dual union all
select 45304,'RVSN' from dual union all
select 45305,'RVSN' from dual union all
select 45306,'RVSN' from dual union all
select 45307,'AFSC' from dual union all
select 45308,'AFSC' from dual union all
select 45309,'RVSN' from dual union all
select 4531,'SUI' from dual union all
select 45310,'RVSN' from dual union all
select 45311,'NASA' from dual union all
select 45312,'RVSN' from dual union all
select 45313,'RVSN' from dual union all
select 45314,'RVSN' from dual union all
select 45315,'RVSN' from dual union all
select 45316,'RVSN' from dual union all
select 45317,'RVSN' from dual union all
select 45318,'RVSN' from dual union all
select 45319,'RVSN' from dual union all
select 4532,'SUI' from dual union all
select 45320,'NASA' from dual;
insert into LAUNCH_AGENCY
select 45321,'MRN' from dual union all
select 45322,'MRN' from dual union all
select 45323,'MRN' from dual union all
select 45324,'NASA' from dual union all
select 45325,'MRN' from dual union all
select 45326,'MRN' from dual union all
select 45327,'USAF' from dual union all
select 45328,'MRN' from dual union all
select 45329,'MRN' from dual union all
select 4533,'6555A' from dual union all
select 45330,'USN' from dual union all
select 45331,'MRN' from dual union all
select 45332,'MRN' from dual union all
select 45333,'AFGL' from dual union all
select 45334,'MRN' from dual union all
select 45335,'MRN' from dual union all
select 45336,'MRN' from dual union all
select 45337,'MRN' from dual union all
select 45338,'RVSN' from dual union all
select 45339,'MRN' from dual union all
select 4534,'SUI' from dual union all
select 45340,'OTR' from dual union all
select 45341,'RAE' from dual union all
select 45342,'MRN' from dual union all
select 45343,'MRN' from dual union all
select 45344,'MRN' from dual union all
select 45345,'MRN' from dual union all
select 45346,'OTR' from dual union all
select 45347,'MRN' from dual union all
select 45348,'INTA' from dual union all
select 45349,'MRN' from dual union all
select 4535,'SUI' from dual union all
select 45350,'RVSN' from dual union all
select 45351,'MRN' from dual union all
select 45352,'MRN' from dual union all
select 45353,'USAF' from dual union all
select 45354,'MRN' from dual union all
select 45355,'MRN' from dual union all
select 45356,'MRN' from dual union all
select 45357,'MRN' from dual union all
select 45358,'MRN' from dual union all
select 45359,'MRN' from dual union all
select 4536,'SUI' from dual union all
select 45360,'MRN' from dual union all
select 45361,'MRN' from dual union all
select 45362,'MRN' from dual union all
select 45363,'MRN' from dual union all
select 45364,'AFGL' from dual union all
select 45365,'AFGL' from dual union all
select 45366,'MRN' from dual union all
select 45367,'RVSN' from dual union all
select 45368,'MRN' from dual union all
select 45369,'MRN' from dual union all
select 4537,'USAF' from dual union all
select 45370,'MRN' from dual union all
select 45371,'MRN' from dual union all
select 45372,'USAF' from dual union all
select 45373,'USN' from dual union all
select 45374,'MRN' from dual union all
select 45375,'MRN' from dual union all
select 45376,'PRO' from dual union all
select 45377,'ADD' from dual union all
select 45378,'ADD' from dual union all
select 45379,'ADD' from dual union all
select 4538,'SUI' from dual union all
select 45380,'ADD' from dual union all
select 45381,'ADD' from dual union all
select 45382,'ADD' from dual union all
select 45383,'ADD' from dual union all
select 45384,'ADD' from dual union all
select 45385,'ADD' from dual union all
select 45386,'BWEHR' from dual union all
select 45387,'AFSC' from dual union all
select 45388,'RVSN' from dual union all
select 45389,'RVSN' from dual union all
select 4539,'SUI' from dual union all
select 45390,'RVSN' from dual union all
select 45391,'DFVLR' from dual union all
select 45392,'AFGL' from dual union all
select 45393,'AFGL' from dual union all
select 45394,'DNA' from dual union all
select 45395,'RVSN' from dual union all
select 45396,'RVSN' from dual union all
select 45397,'RVSN' from dual union all
select 45398,'RVSN' from dual union all
select 45399,'RVSN' from dual union all
select 454,'WEHR' from dual union all
select 4540,'SUI' from dual union all
select 45400,'RVSN' from dual union all
select 45401,'RVSN' from dual union all
select 45402,'RVSN' from dual union all
select 45403,'RVSN' from dual union all
select 45404,'RVSN' from dual union all
select 45405,'AFSC' from dual union all
select 45406,'DMA' from dual union all
select 45407,'AFGL' from dual union all
select 45408,'AN' from dual union all
select 45409,'NASA' from dual union all
select 4541,'USAF' from dual union all
select 45410,'NASA' from dual;
insert into LAUNCH_AGENCY
select 45411,'USN' from dual union all
select 45412,'USN' from dual union all
select 45413,'USN' from dual union all
select 45414,'USN' from dual union all
select 45415,'RVSN' from dual union all
select 45416,'NRCC' from dual union all
select 45417,'USN' from dual union all
select 45418,'SAST' from dual union all
select 45419,'SSC' from dual union all
select 4542,'SUI' from dual union all
select 45420,'ISRO' from dual union all
select 45421,'AN' from dual union all
select 45422,'AN' from dual union all
select 45423,'AN' from dual union all
select 45424,'AN' from dual union all
select 45425,'AN' from dual union all
select 45426,'RVSN' from dual union all
select 45427,'AN' from dual union all
select 45428,'AN' from dual union all
select 45429,'USN' from dual union all
select 4543,'USAF' from dual union all
select 45430,'USN' from dual union all
select 45431,'USN' from dual union all
select 45432,'USN' from dual union all
select 45433,'IAE' from dual union all
select 45434,'AN' from dual union all
select 45435,'AN' from dual union all
select 45436,'AN' from dual union all
select 45437,'RVSN' from dual union all
select 45438,'AN' from dual union all
select 45439,'NASA' from dual union all
select 4544,'6555A' from dual union all
select 45440,'BAC' from dual union all
select 45441,'USN' from dual union all
select 45442,'USN' from dual union all
select 45443,'USN' from dual union all
select 45444,'USN' from dual union all
select 45445,'NASA' from dual union all
select 45446,'USN' from dual union all
select 45447,'AFGL' from dual union all
select 45448,'RVSN' from dual union all
select 45449,'9FA3' from dual union all
select 4545,'USN' from dual union all
select 45450,'USN' from dual union all
select 45451,'USN' from dual union all
select 45452,'USN' from dual union all
select 45453,'USN' from dual union all
select 45454,'AN' from dual union all
select 45455,'ALCS' from dual union all
select 45455,'SAC' from dual union all
select 45456,'RVSN' from dual union all
select 45457,'RVSN' from dual union all
select 45458,'AN' from dual union all
select 45459,'RVSN' from dual union all
select 4546,'ABMA' from dual union all
select 4546,'USAF' from dual union all
select 45460,'AN' from dual union all
select 45461,'AN' from dual union all
select 45462,'ALCS' from dual union all
select 45462,'SAC' from dual union all
select 45463,'NASA' from dual union all
select 45464,'AN' from dual union all
select 45465,'DLR' from dual union all
select 45466,'RVSN' from dual union all
select 45467,'NASA' from dual union all
select 45468,'AFSC' from dual union all
select 45469,'AN' from dual union all
select 4547,'USN' from dual union all
select 45470,'RVSN' from dual union all
select 45471,'NASA' from dual union all
select 45472,'DMA' from dual union all
select 45473,'AFSC' from dual union all
select 45474,'USN' from dual union all
select 45475,'AN' from dual union all
select 45476,'NASA' from dual union all
select 45477,'AN' from dual union all
select 45478,'SAC' from dual union all
select 45479,'RVSN' from dual union all
select 4548,'USN' from dual union all
select 45480,'USN' from dual union all
select 45481,'RVSN' from dual union all
select 45482,'RVSN' from dual union all
select 45483,'AN' from dual union all
select 45484,'AN' from dual union all
select 45485,'AN' from dual union all
select 45486,'AFSC' from dual union all
select 45487,'USN' from dual union all
select 45488,'USN' from dual union all
select 45489,'USN' from dual union all
select 4549,'6555A' from dual union all
select 45490,'USN' from dual union all
select 45491,'USN' from dual union all
select 45492,'RVSN' from dual union all
select 45493,'81FA1' from dual union all
select 45494,'81FA1' from dual union all
select 45495,'81FA1' from dual union all
select 45496,'NASA' from dual union all
select 45497,'SAC' from dual union all
select 45498,'AFGL2' from dual union all
select 45499,'84FA3' from dual;
insert into LAUNCH_AGENCY
select 455,'WER485' from dual union all
select 4550,'USAF' from dual union all
select 45500,'84FA3' from dual union all
select 45501,'84FA3' from dual union all
select 45502,'RN' from dual union all
select 45503,'RVSN' from dual union all
select 45504,'RVSN' from dual union all
select 45505,'SSC' from dual union all
select 45506,'SRC' from dual union all
select 45507,'SSC' from dual union all
select 45508,'RVSN' from dual union all
select 45509,'USN' from dual union all
select 4551,'SUI' from dual union all
select 45510,'SRC' from dual union all
select 45511,'NASA' from dual union all
select 45512,'ISAS' from dual union all
select 45513,'RVSN' from dual union all
select 45514,'RVSN' from dual union all
select 45515,'NASDA' from dual union all
select 45516,'41FA1' from dual union all
select 45517,'41FA1' from dual union all
select 45518,'41FA1' from dual union all
select 45519,'VMF' from dual union all
select 4552,'SUI' from dual union all
select 45520,'ALCS' from dual union all
select 45520,'SAC' from dual union all
select 45521,'NASA' from dual union all
select 45522,'RVSN' from dual union all
select 45523,'DFVLR' from dual union all
select 45524,'RAE' from dual union all
select 45525,'RVSN' from dual union all
select 45526,'AFGL' from dual union all
select 45527,'AFGL' from dual union all
select 45528,'NASA' from dual union all
select 45529,'USN' from dual union all
select 4553,'SUI' from dual union all
select 45530,'NASA' from dual union all
select 45531,'NASA' from dual union all
select 45532,'IAE' from dual union all
select 45533,'RVSN' from dual union all
select 45534,'AN' from dual union all
select 45535,'DNA' from dual union all
select 45536,'RVSN' from dual union all
select 45537,'DNA' from dual union all
select 45538,'DNA' from dual union all
select 45539,'USN' from dual union all
select 4554,'SUI' from dual union all
select 45540,'AFGL2' from dual union all
select 45541,'NASA' from dual union all
select 45542,'CNES' from dual union all
select 45543,'NASA' from dual union all
select 45544,'RVSN' from dual union all
select 45545,'BAC' from dual union all
select 45546,'DNA' from dual union all
select 45547,'RVSN' from dual union all
select 45548,'BAC' from dual union all
select 45549,'ALCS' from dual union all
select 45549,'SAC' from dual union all
select 4555,'6555A' from dual union all
select 45550,'DFVLR' from dual union all
select 45551,'NASA' from dual union all
select 45552,'USN' from dual union all
select 45553,'USN' from dual union all
select 45554,'NRCC' from dual union all
select 45555,'AFSC' from dual union all
select 45556,'NASA' from dual union all
select 45557,'NASA' from dual union all
select 45558,'RVSN' from dual union all
select 45559,'AN' from dual union all
select 4556,'SUI' from dual union all
select 45560,'AN' from dual union all
select 45561,'AN' from dual union all
select 45562,'RN' from dual union all
select 45563,'AN' from dual union all
select 45564,'AN' from dual union all
select 45565,'NASA' from dual union all
select 45566,'DMA' from dual union all
select 45567,'SAC' from dual union all
select 45568,'USAF' from dual union all
select 45569,'RVSN' from dual union all
select 4557,'SUI' from dual union all
select 45570,'RAE' from dual union all
select 45571,'AFSC' from dual union all
select 45572,'NASA' from dual union all
select 45573,'NASA' from dual union all
select 45574,'BAC' from dual union all
select 45575,'NRCC' from dual union all
select 45576,'RVSN' from dual union all
select 45577,'ISRO' from dual union all
select 45578,'ISRO' from dual union all
select 45579,'ISRO' from dual union all
select 4558,'SUI' from dual union all
select 45580,'RVSN' from dual union all
select 45581,'ISRO' from dual union all
select 45582,'MPIK' from dual union all
select 45583,'MPIK' from dual union all
select 45584,'AFGL' from dual union all
select 45585,'AFGL' from dual union all
select 45586,'AFGL' from dual union all
select 45587,'ISRO' from dual;
insert into LAUNCH_AGENCY
select 45588,'RVSN' from dual union all
select 45589,'RVSN' from dual union all
select 4559,'SUI' from dual union all
select 45590,'RVSN' from dual union all
select 45591,'AN' from dual union all
select 45592,'SAND' from dual union all
select 45593,'SAND' from dual union all
select 45594,'SSC' from dual union all
select 45595,'9FA3' from dual union all
select 45596,'9FA3' from dual union all
select 45597,'9FA3' from dual union all
select 45598,'9FA3' from dual union all
select 45599,'RVSN' from dual union all
select 456,'WER485-1' from dual union all
select 4560,'SUI' from dual union all
select 45600,'RVSN' from dual union all
select 45601,'RVSN' from dual union all
select 45602,'ISAS' from dual union all
select 45603,'RVSN' from dual union all
select 45604,'RVSN' from dual union all
select 45605,'RVSN' from dual union all
select 45606,'RVSN' from dual union all
select 45607,'RVSN' from dual union all
select 45608,'RVSN' from dual union all
select 45609,'RVSN' from dual union all
select 4561,'SUI' from dual union all
select 45610,'RVSN' from dual union all
select 45611,'RVSN' from dual union all
select 45612,'RVSN' from dual union all
select 45613,'RVSN' from dual union all
select 45614,'RVSN' from dual union all
select 45615,'RVSN' from dual union all
select 45616,'RVSN' from dual union all
select 45617,'RVSN' from dual union all
select 45618,'RVSN' from dual union all
select 45619,'RVSN' from dual union all
select 4562,'SUI' from dual union all
select 45620,'RVSN' from dual union all
select 45621,'RVSN' from dual union all
select 45622,'CALT' from dual union all
select 45623,'INTA' from dual union all
select 45624,'INTA' from dual union all
select 45625,'INTA' from dual union all
select 45626,'INTA' from dual union all
select 45627,'INTA' from dual union all
select 45628,'INTA' from dual union all
select 45629,'INTA' from dual union all
select 4563,'SUI' from dual union all
select 45630,'INTA' from dual union all
select 45631,'RVSN' from dual union all
select 45632,'INTA' from dual union all
select 45633,'INTA' from dual union all
select 45634,'INTA' from dual union all
select 45635,'INTA' from dual union all
select 45636,'INTA' from dual union all
select 45637,'INTA' from dual union all
select 45638,'INTA' from dual union all
select 45639,'INTA' from dual union all
select 4564,'SUI' from dual union all
select 45640,'INTA' from dual union all
select 45641,'INTA' from dual union all
select 45642,'RVSN' from dual union all
select 45643,'INTA' from dual union all
select 45644,'MVS' from dual union all
select 45645,'MVS' from dual union all
select 45646,'ISAS' from dual union all
select 45647,'ISAS' from dual union all
select 45648,'ISAS' from dual union all
select 45649,'ISAS' from dual union all
select 4565,'SUI' from dual union all
select 45650,'RVSN' from dual union all
select 45651,'RVSN' from dual union all
select 45652,'RVSN' from dual union all
select 45653,'RVSN' from dual union all
select 45654,'RVSN' from dual union all
select 45655,'MVS' from dual union all
select 45656,'MVS' from dual union all
select 45657,'MVS' from dual union all
select 45658,'MVS' from dual union all
select 45659,'RVSN' from dual union all
select 4566,'6555A' from dual union all
select 45660,'AN' from dual union all
select 45661,'AN' from dual union all
select 45662,'9FA3' from dual union all
select 45663,'9FA3' from dual union all
select 45664,'RVSN' from dual union all
select 45665,'9FA3' from dual union all
select 45666,'9FA3' from dual union all
select 45667,'FKG' from dual union all
select 45668,'FKG' from dual union all
select 45669,'RVSN' from dual union all
select 4567,'SUI' from dual union all
select 45670,'RVSN' from dual union all
select 45671,'RVSN' from dual union all
select 45672,'RVSN' from dual union all
select 45673,'RVSN' from dual union all
select 45674,'RVSN' from dual union all
select 45675,'NASA' from dual union all
select 45676,'AFSC' from dual union all
select 45677,'NASA' from dual;
insert into LAUNCH_AGENCY
select 45678,'NASA' from dual union all
select 45679,'AN' from dual union all
select 4568,'SUI' from dual union all
select 45680,'NASDA' from dual union all
select 45681,'USN' from dual union all
select 45682,'AFGL2' from dual union all
select 45683,'ISAS' from dual union all
select 45684,'ISAS' from dual union all
select 45685,'NIPR' from dual union all
select 45686,'NASA' from dual union all
select 45687,'DFVLR' from dual union all
select 45688,'DFVLR' from dual union all
select 45689,'NASA' from dual union all
select 4569,'USAF' from dual union all
select 45690,'NASA' from dual union all
select 45691,'SRC' from dual union all
select 45692,'SRC' from dual union all
select 45693,'SRC' from dual union all
select 45694,'SRC' from dual union all
select 45695,'SRC' from dual union all
select 45696,'NIPR' from dual union all
select 45697,'SRC' from dual union all
select 45698,'SRC' from dual union all
select 45699,'SAC' from dual union all
select 457,'WER485-1' from dual union all
select 4570,'USN' from dual union all
select 45700,'NASA' from dual union all
select 45701,'RVSN' from dual union all
select 45702,'UKMO' from dual union all
select 45703,'UKMO' from dual union all
select 45704,'NASA' from dual union all
select 45705,'ALCS' from dual union all
select 45705,'SAC' from dual union all
select 45706,'RVSN' from dual union all
select 45707,'USN' from dual union all
select 45708,'ISAS' from dual union all
select 45709,'RVSN' from dual union all
select 4571,'USAF' from dual union all
select 45710,'NRCC' from dual union all
select 45711,'NASA' from dual union all
select 45712,'AN' from dual union all
select 45713,'NASA' from dual union all
select 45714,'DNA' from dual union all
select 45715,'DNA' from dual union all
select 45716,'NASA' from dual union all
select 45717,'NTNF' from dual union all
select 45718,'NTNF' from dual union all
select 45719,'ARPA' from dual union all
select 4572,'SUI' from dual union all
select 45720,'SAC' from dual union all
select 45721,'NASA' from dual union all
select 45722,'NASA' from dual union all
select 45723,'NASA' from dual union all
select 45724,'NASA' from dual union all
select 45725,'NASA' from dual union all
select 45726,'NASA' from dual union all
select 45727,'NASA' from dual union all
select 45728,'RVSN' from dual union all
select 45729,'NASA' from dual union all
select 4573,'SUI' from dual union all
select 45730,'USN' from dual union all
select 45731,'NASA' from dual union all
select 45732,'RVSN' from dual union all
select 45733,'MRN' from dual union all
select 45734,'MRN' from dual union all
select 45735,'MRN' from dual union all
select 45736,'MRN' from dual union all
select 45737,'MRN' from dual union all
select 45738,'AN' from dual union all
select 45739,'MRN' from dual union all
select 4574,'MVS' from dual union all
select 45740,'MRN' from dual union all
select 45741,'MRN' from dual union all
select 45742,'MRN' from dual union all
select 45743,'MRN' from dual union all
select 45744,'MRN' from dual union all
select 45745,'MRN' from dual union all
select 45746,'MRN' from dual union all
select 45747,'GMS' from dual union all
select 45748,'MRN' from dual union all
select 45749,'MRN' from dual union all
select 4575,'USAF' from dual union all
select 45750,'MRN' from dual union all
select 45751,'MRN' from dual union all
select 45752,'MRN' from dual union all
select 45753,'MRN' from dual union all
select 45754,'MRN' from dual union all
select 45755,'MRN' from dual union all
select 45756,'MRN' from dual union all
select 45757,'GMS' from dual union all
select 45758,'MRN' from dual union all
select 45759,'MRN' from dual union all
select 4576,'RAE' from dual union all
select 4576,'WRE' from dual union all
select 45760,'MRN' from dual union all
select 45761,'MRN' from dual union all
select 45762,'MRN' from dual union all
select 45763,'MRN' from dual union all
select 45764,'MRN' from dual union all
select 45765,'MRN' from dual;
insert into LAUNCH_AGENCY
select 45766,'MRN' from dual union all
select 45767,'MRN' from dual union all
select 45768,'MRN' from dual union all
select 45769,'MRN' from dual union all
select 4577,'USA' from dual union all
select 45770,'MRN' from dual union all
select 45771,'MRN' from dual union all
select 45772,'MRN' from dual union all
select 45773,'MRN' from dual union all
select 45774,'MRN' from dual union all
select 45775,'MRN' from dual union all
select 45776,'MRN' from dual union all
select 45777,'MRN' from dual union all
select 45778,'MRN' from dual union all
select 45779,'MRN' from dual union all
select 4578,'USN' from dual union all
select 45780,'MRN' from dual union all
select 45781,'MRN' from dual union all
select 45782,'MRN' from dual union all
select 45783,'MRN' from dual union all
select 45784,'MRN' from dual union all
select 45785,'MRN' from dual union all
select 45786,'MRN' from dual union all
select 45787,'MRN' from dual union all
select 45788,'MRN' from dual union all
select 45789,'MRN' from dual union all
select 4579,'USAF' from dual union all
select 45790,'MRN' from dual union all
select 45791,'MRN' from dual union all
select 45792,'MRN' from dual union all
select 45793,'MRN' from dual union all
select 45794,'MRN' from dual union all
select 45795,'MRN' from dual union all
select 45796,'MRN' from dual union all
select 45797,'MRN' from dual union all
select 45798,'MRN' from dual union all
select 45799,'MRN' from dual union all
select 458,'WER485-2' from dual union all
select 4580,'USAF' from dual union all
select 45800,'MRN' from dual union all
select 45801,'MRN' from dual union all
select 45802,'MRN' from dual union all
select 45803,'MRN' from dual union all
select 45804,'MRN' from dual union all
select 45805,'MRN' from dual union all
select 45806,'MRN' from dual union all
select 45807,'MRN' from dual union all
select 45808,'MRN' from dual union all
select 45809,'MRN' from dual union all
select 4581,'6555A' from dual union all
select 45810,'MRN' from dual union all
select 45811,'MRN' from dual union all
select 45812,'MRN' from dual union all
select 45813,'MRN' from dual union all
select 45814,'MRN' from dual union all
select 45815,'MRN' from dual union all
select 45816,'MRN' from dual union all
select 45817,'MRN' from dual union all
select 45818,'MRN' from dual union all
select 45819,'MRN' from dual union all
select 4582,'USAF' from dual union all
select 45820,'MRN' from dual union all
select 45821,'MRN' from dual union all
select 45822,'MRN' from dual union all
select 45823,'MRN' from dual union all
select 45824,'MRN' from dual union all
select 45825,'MRN' from dual union all
select 45826,'MRN' from dual union all
select 45827,'NASA' from dual union all
select 45828,'NASA' from dual union all
select 45829,'NASA' from dual union all
select 4583,'USN' from dual union all
select 45830,'MRN' from dual union all
select 45831,'MRN' from dual union all
select 45832,'MRN' from dual union all
select 45833,'MRN' from dual union all
select 45834,'MRN' from dual union all
select 45835,'MRN' from dual union all
select 45836,'MRN' from dual union all
select 45837,'MRN' from dual union all
select 45838,'MRN' from dual union all
select 45839,'MRN' from dual union all
select 4584,'USN' from dual union all
select 45840,'MRN' from dual union all
select 45841,'MRN' from dual union all
select 45842,'MRN' from dual union all
select 45843,'MRN' from dual union all
select 45844,'MRN' from dual union all
select 45845,'MRN' from dual union all
select 45846,'MRN' from dual union all
select 45847,'MRN' from dual union all
select 45848,'MRN' from dual union all
select 45849,'MRN' from dual union all
select 4585,'USA' from dual union all
select 45850,'MRN' from dual union all
select 45851,'MRN' from dual union all
select 45852,'MRN' from dual union all
select 45853,'MRN' from dual union all
select 45854,'MRN' from dual union all
select 45855,'MRN' from dual;
insert into LAUNCH_AGENCY
select 45856,'MRN' from dual union all
select 45857,'MRN' from dual union all
select 45858,'MRN' from dual union all
select 45859,'MRN' from dual union all
select 4586,'USA' from dual union all
select 45860,'MRN' from dual union all
select 45861,'MRN' from dual union all
select 45862,'MRN' from dual union all
select 45863,'MRN' from dual union all
select 45864,'MRN' from dual union all
select 45865,'MRN' from dual union all
select 45866,'MRN' from dual union all
select 45867,'MRN' from dual union all
select 45868,'MRN' from dual union all
select 45869,'MRN' from dual union all
select 4587,'USA' from dual union all
select 45870,'MRN' from dual union all
select 45871,'MRN' from dual union all
select 45872,'MRN' from dual union all
select 45873,'MRN' from dual union all
select 45874,'MRN' from dual union all
select 45875,'GMS' from dual union all
select 45876,'MRN' from dual union all
select 45877,'MRN' from dual union all
select 45878,'MRN' from dual union all
select 45879,'MRN' from dual union all
select 4588,'6555A' from dual union all
select 45880,'MRN' from dual union all
select 45881,'MRN' from dual union all
select 45882,'MRN' from dual union all
select 45883,'MRN' from dual union all
select 45884,'MRN' from dual union all
select 45885,'GMS' from dual union all
select 45886,'MRN' from dual union all
select 45887,'MRN' from dual union all
select 45888,'MRN' from dual union all
select 45889,'MRN' from dual union all
select 4589,'USAF' from dual union all
select 45890,'MRN' from dual union all
select 45891,'MRN' from dual union all
select 45892,'MRN' from dual union all
select 45893,'MRN' from dual union all
select 45894,'MRN' from dual union all
select 45895,'MRN' from dual union all
select 45896,'MRN' from dual union all
select 45897,'MRN' from dual union all
select 45898,'MRN' from dual union all
select 45899,'GMS' from dual union all
select 459,'WER485-2' from dual union all
select 4590,'ABMA' from dual union all
select 4590,'USAF' from dual union all
select 45900,'MRN' from dual union all
select 45901,'MRN' from dual union all
select 45902,'MRN' from dual union all
select 45903,'MRN' from dual union all
select 45904,'MRN' from dual union all
select 45905,'MRN' from dual union all
select 45906,'MRN' from dual union all
select 45907,'MRN' from dual union all
select 45908,'MRN' from dual union all
select 45909,'MRN' from dual union all
select 4591,'6555A' from dual union all
select 45910,'MRN' from dual union all
select 45911,'MRN' from dual union all
select 45912,'GMS' from dual union all
select 45913,'MRN' from dual union all
select 45914,'MRN' from dual union all
select 45915,'MRN' from dual union all
select 45916,'MRN' from dual union all
select 45917,'MRN' from dual union all
select 45918,'NASA' from dual union all
select 45919,'MRN' from dual union all
select 4592,'MVS' from dual union all
select 45920,'MRN' from dual union all
select 45921,'MRN' from dual union all
select 45922,'MRN' from dual union all
select 45923,'MRN' from dual union all
select 45924,'GMS' from dual union all
select 45925,'MRN' from dual union all
select 45926,'MRN' from dual union all
select 45927,'MRN' from dual union all
select 45928,'MRN' from dual union all
select 45929,'MRN' from dual union all
select 4593,'UNRV' from dual union all
select 45930,'MRN' from dual union all
select 45931,'MRN' from dual union all
select 45932,'MRN' from dual union all
select 45933,'MRN' from dual union all
select 45934,'MRN' from dual union all
select 45935,'MRN' from dual union all
select 45936,'GMS' from dual union all
select 45937,'MRN' from dual union all
select 45938,'MRN' from dual union all
select 45939,'MRN' from dual union all
select 4594,'UNRV' from dual union all
select 45940,'MRN' from dual union all
select 45941,'MRN' from dual union all
select 45942,'GMS' from dual union all
select 45943,'MRN' from dual union all
select 45944,'NASA' from dual;
insert into LAUNCH_AGENCY
select 45945,'MRN' from dual union all
select 45946,'NASA' from dual union all
select 45947,'MRN' from dual union all
select 45948,'MRN' from dual union all
select 45949,'MRN' from dual union all
select 4595,'UNRV' from dual union all
select 45950,'MRN' from dual union all
select 45951,'MRN' from dual union all
select 45952,'MRN' from dual union all
select 45953,'MRN' from dual union all
select 45954,'MRN' from dual union all
select 45955,'MRN' from dual union all
select 45956,'MRN' from dual union all
select 45957,'AN' from dual union all
select 45958,'MRN' from dual union all
select 45959,'NASA' from dual union all
select 4596,'UNRV' from dual union all
select 45960,'MRN' from dual union all
select 45961,'MRN' from dual union all
select 45962,'MRN' from dual union all
select 45963,'MRN' from dual union all
select 45964,'MRN' from dual union all
select 45965,'MRN' from dual union all
select 45966,'MRN' from dual union all
select 45967,'MRN' from dual union all
select 45968,'MRN' from dual union all
select 45969,'MRN' from dual union all
select 4597,'UNRV' from dual union all
select 45970,'MRN' from dual union all
select 45971,'MRN' from dual union all
select 45972,'MRN' from dual union all
select 45973,'MRN' from dual union all
select 45974,'NASA' from dual union all
select 45975,'MRN' from dual union all
select 45976,'MRN' from dual union all
select 45977,'MRN' from dual union all
select 45978,'MRN' from dual union all
select 45979,'MRN' from dual union all
select 4598,'UNRV' from dual union all
select 45980,'MRN' from dual union all
select 45981,'MRN' from dual union all
select 45982,'MRN' from dual union all
select 45983,'MRN' from dual union all
select 45984,'NASA' from dual union all
select 45985,'MRN' from dual union all
select 45986,'NASA' from dual union all
select 45987,'MRN' from dual union all
select 45988,'MRN' from dual union all
select 45989,'MRN' from dual union all
select 4599,'UNRV' from dual union all
select 45990,'MRN' from dual union all
select 45991,'MRN' from dual union all
select 45992,'MRN' from dual union all
select 45993,'MRN' from dual union all
select 45994,'MRN' from dual union all
select 45995,'MRN' from dual union all
select 45996,'MRN' from dual union all
select 45997,'MRN' from dual union all
select 45998,'MRN' from dual union all
select 45999,'MRN' from dual union all
select 46,'WER485-2' from dual union all
select 460,'WER485-1' from dual union all
select 4600,'UNRV' from dual union all
select 46000,'MRN' from dual union all
select 46001,'MRN' from dual union all
select 46002,'MRN' from dual union all
select 46003,'MRN' from dual union all
select 46004,'MRN' from dual union all
select 46005,'MRN' from dual union all
select 46006,'MRN' from dual union all
select 46007,'MRN' from dual union all
select 46008,'MRN' from dual union all
select 46009,'MRN' from dual union all
select 4601,'UNRV' from dual union all
select 46010,'MRN' from dual union all
select 46011,'MRN' from dual union all
select 46012,'MRN' from dual union all
select 46013,'MRN' from dual union all
select 46014,'MRN' from dual union all
select 46015,'MRN' from dual union all
select 46016,'MRN' from dual union all
select 46017,'MRN' from dual union all
select 46018,'MRN' from dual union all
select 46019,'MRN' from dual union all
select 4602,'UNRV' from dual union all
select 46020,'MRN' from dual union all
select 46021,'MRN' from dual union all
select 46022,'MRN' from dual union all
select 46023,'MRN' from dual union all
select 46024,'MRN' from dual union all
select 46025,'MRN' from dual union all
select 46026,'MRN' from dual union all
select 46027,'MRN' from dual union all
select 46028,'MRN' from dual union all
select 46029,'MRN' from dual union all
select 4603,'UNRV' from dual union all
select 46030,'MRN' from dual union all
select 46031,'MRN' from dual union all
select 46032,'MRN' from dual union all
select 46033,'MRN' from dual;
insert into LAUNCH_AGENCY
select 46034,'MRN' from dual union all
select 46035,'MRN' from dual union all
select 46036,'MRN' from dual union all
select 46037,'MRN' from dual union all
select 46038,'MRN' from dual union all
select 46039,'MRN' from dual union all
select 4604,'UNRV' from dual union all
select 46040,'MRN' from dual union all
select 46041,'MRN' from dual union all
select 46042,'MRN' from dual union all
select 46043,'MRN' from dual union all
select 46044,'MRN' from dual union all
select 46045,'NASA' from dual union all
select 46046,'MRN' from dual union all
select 46047,'MRN' from dual union all
select 46048,'MRN' from dual union all
select 46049,'MRN' from dual union all
select 4605,'UNRV' from dual union all
select 46050,'MRN' from dual union all
select 46051,'MRN' from dual union all
select 46052,'MRN' from dual union all
select 46053,'GMS' from dual union all
select 46054,'MRN' from dual union all
select 46055,'MRN' from dual union all
select 46056,'MRN' from dual union all
select 46057,'MRN' from dual union all
select 46058,'NASA' from dual union all
select 46059,'MRN' from dual union all
select 4606,'UNRV' from dual union all
select 46060,'MRN' from dual union all
select 46061,'MRN' from dual union all
select 46062,'NASA' from dual union all
select 46063,'MRN' from dual union all
select 46064,'MRN' from dual union all
select 46065,'MRN' from dual union all
select 46066,'MRN' from dual union all
select 46067,'MRN' from dual union all
select 46068,'MRN' from dual union all
select 46069,'MRN' from dual union all
select 4607,'UNRV' from dual union all
select 46070,'MRN' from dual union all
select 46071,'MRN' from dual union all
select 46072,'MRN' from dual union all
select 46073,'MRN' from dual union all
select 46074,'MRN' from dual union all
select 46075,'MRN' from dual union all
select 46076,'MRN' from dual union all
select 46077,'MRN' from dual union all
select 46078,'MRN' from dual union all
select 46079,'MRN' from dual union all
select 4608,'UNRV' from dual union all
select 46080,'MRN' from dual union all
select 46081,'MRN' from dual union all
select 46082,'MRN' from dual union all
select 46083,'MRN' from dual union all
select 46084,'MRN' from dual union all
select 46085,'MRN' from dual union all
select 46086,'MRN' from dual union all
select 46087,'MRN' from dual union all
select 46088,'MRN' from dual union all
select 46089,'MRN' from dual union all
select 4609,'UNRV' from dual union all
select 46090,'MRN' from dual union all
select 46091,'MRN' from dual union all
select 46092,'GMS' from dual union all
select 46093,'MRN' from dual union all
select 46094,'MRN' from dual union all
select 46095,'MRN' from dual union all
select 46096,'MRN' from dual union all
select 46097,'MRN' from dual union all
select 46098,'MRN' from dual union all
select 46099,'MRN' from dual union all
select 461,'WER485-2' from dual union all
select 4610,'USA' from dual union all
select 46100,'MRN' from dual union all
select 46101,'MRN' from dual union all
select 46102,'MRN' from dual union all
select 46103,'MRN' from dual union all
select 46104,'MRN' from dual union all
select 46105,'MRN' from dual union all
select 46106,'GMS' from dual union all
select 46107,'MRN' from dual union all
select 46108,'MRN' from dual union all
select 46109,'MRN' from dual union all
select 4611,'UNRV' from dual union all
select 46110,'MRN' from dual union all
select 46111,'MRN' from dual union all
select 46112,'MRN' from dual union all
select 46113,'MRN' from dual union all
select 46114,'MRN' from dual union all
select 46115,'MRN' from dual union all
select 46116,'MRN' from dual union all
select 46117,'MRN' from dual union all
select 46118,'MRN' from dual union all
select 46119,'MRN' from dual union all
select 4612,'UNRV' from dual union all
select 46120,'MRN' from dual union all
select 46121,'MRN' from dual union all
select 46122,'MRN' from dual union all
select 46123,'MRN' from dual;
insert into LAUNCH_AGENCY
select 46124,'GMS' from dual union all
select 46125,'GMS' from dual union all
select 46126,'MRN' from dual union all
select 46127,'MRN' from dual union all
select 46128,'MRN' from dual union all
select 46129,'MRN' from dual union all
select 4613,'UNRV' from dual union all
select 46130,'MRN' from dual union all
select 46131,'MRN' from dual union all
select 46132,'NASA' from dual union all
select 46133,'MRN' from dual union all
select 46134,'NASA' from dual union all
select 46135,'MRN' from dual union all
select 46136,'MRN' from dual union all
select 46137,'MRN' from dual union all
select 46138,'MRN' from dual union all
select 46139,'MRN' from dual union all
select 4614,'UNRV' from dual union all
select 46140,'MRN' from dual union all
select 46141,'MRN' from dual union all
select 46142,'MRN' from dual union all
select 46143,'NASDA' from dual union all
select 46144,'GMS' from dual union all
select 46145,'MRN' from dual union all
select 46146,'MRN' from dual union all
select 46147,'MRN' from dual union all
select 46148,'MRN' from dual union all
select 46149,'MRN' from dual union all
select 4615,'UNRV' from dual union all
select 46150,'MRN' from dual union all
select 46151,'MRN' from dual union all
select 46152,'MRN' from dual union all
select 46153,'MRN' from dual union all
select 46154,'MRN' from dual union all
select 46155,'MRN' from dual union all
select 46156,'MRN' from dual union all
select 46157,'MRN' from dual union all
select 46158,'MRN' from dual union all
select 46159,'MRN' from dual union all
select 4616,'UNRV' from dual union all
select 46160,'MRN' from dual union all
select 46161,'MRN' from dual union all
select 46162,'MRN' from dual union all
select 46163,'MRN' from dual union all
select 46164,'MRN' from dual union all
select 46165,'MRN' from dual union all
select 46166,'MRN' from dual union all
select 46167,'GMS' from dual union all
select 46168,'GMS' from dual union all
select 46169,'MRN' from dual union all
select 4617,'UNRV' from dual union all
select 46170,'MRN' from dual union all
select 46171,'MRN' from dual union all
select 46172,'MRN' from dual union all
select 46173,'MRN' from dual union all
select 46174,'MRN' from dual union all
select 46175,'MRN' from dual union all
select 46176,'MRN' from dual union all
select 46177,'MRN' from dual union all
select 46178,'MRN' from dual union all
select 46179,'MRN' from dual union all
select 4618,'UNRV' from dual union all
select 46180,'MRN' from dual union all
select 46181,'MRN' from dual union all
select 46182,'MRN' from dual union all
select 46183,'MRN' from dual union all
select 46184,'MRN' from dual union all
select 46185,'MRN' from dual union all
select 46186,'MRN' from dual union all
select 46187,'GMS' from dual union all
select 46188,'MRN' from dual union all
select 46189,'MRN' from dual union all
select 4619,'UNRV' from dual union all
select 46190,'MRN' from dual union all
select 46191,'MRN' from dual union all
select 46192,'MRN' from dual union all
select 46193,'MRN' from dual union all
select 46194,'MRN' from dual union all
select 46195,'MRN' from dual union all
select 46196,'MRN' from dual union all
select 46197,'NRCC' from dual union all
select 46198,'MRN' from dual union all
select 46199,'MRN' from dual union all
select 462,'WER485-1' from dual union all
select 4620,'UNRV' from dual union all
select 46200,'MRN' from dual union all
select 46201,'MRN' from dual union all
select 46202,'MRN' from dual union all
select 46203,'MRN' from dual union all
select 46204,'MRN' from dual union all
select 46205,'MRN' from dual union all
select 46206,'MRN' from dual union all
select 46207,'MRN' from dual union all
select 46208,'MRN' from dual union all
select 46209,'MRN' from dual union all
select 4621,'NRL' from dual union all
select 46210,'MRN' from dual union all
select 46211,'MRN' from dual union all
select 46212,'MRN' from dual union all
select 46213,'NRCC' from dual;
insert into LAUNCH_AGENCY
select 46214,'MRN' from dual union all
select 46215,'MRN' from dual union all
select 46216,'MRN' from dual union all
select 46217,'MRN' from dual union all
select 46218,'AN' from dual union all
select 46219,'MRN' from dual union all
select 4622,'UNRV' from dual union all
select 46220,'MRN' from dual union all
select 46221,'MRN' from dual union all
select 46222,'MRN' from dual union all
select 46223,'MRN' from dual union all
select 46224,'MRN' from dual union all
select 46225,'MRN' from dual union all
select 46226,'MRN' from dual union all
select 46227,'MRN' from dual union all
select 46228,'MRN' from dual union all
select 46229,'MRN' from dual union all
select 4623,'UNRV' from dual union all
select 46230,'MRN' from dual union all
select 46231,'MRN' from dual union all
select 46232,'MRN' from dual union all
select 46233,'MRN' from dual union all
select 46234,'MRN' from dual union all
select 46235,'MRN' from dual union all
select 46236,'MRN' from dual union all
select 46237,'MRN' from dual union all
select 46238,'MRN' from dual union all
select 46239,'MRN' from dual union all
select 4624,'UNRV' from dual union all
select 46240,'MRN' from dual union all
select 46241,'MRN' from dual union all
select 46242,'MRN' from dual union all
select 46243,'MRN' from dual union all
select 46244,'MRN' from dual union all
select 46245,'MRN' from dual union all
select 46246,'MRN' from dual union all
select 46247,'MRN' from dual union all
select 46248,'MRN' from dual union all
select 46249,'MRN' from dual union all
select 4625,'UNRV' from dual union all
select 46250,'MRN' from dual union all
select 46251,'MRN' from dual union all
select 46252,'MRN' from dual union all
select 46253,'MRN' from dual union all
select 46254,'MRN' from dual union all
select 46255,'MRN' from dual union all
select 46256,'MRN' from dual union all
select 46257,'MRN' from dual union all
select 46258,'MRN' from dual union all
select 46259,'AN' from dual union all
select 4626,'UNRV' from dual union all
select 46260,'MRN' from dual union all
select 46261,'MRN' from dual union all
select 46262,'MRN' from dual union all
select 46263,'MRN' from dual union all
select 46264,'MRN' from dual union all
select 46265,'MRN' from dual union all
select 46266,'MRN' from dual union all
select 46267,'MRN' from dual union all
select 46268,'MRN' from dual union all
select 46269,'MRN' from dual union all
select 4627,'UNRV' from dual union all
select 46270,'MRN' from dual union all
select 46271,'MRN' from dual union all
select 46272,'MRN' from dual union all
select 46273,'MRN' from dual union all
select 46274,'MRN' from dual union all
select 46275,'MRN' from dual union all
select 46276,'MRN' from dual union all
select 46277,'MRN' from dual union all
select 46278,'MRN' from dual union all
select 46279,'NASA' from dual union all
select 4628,'UNRV' from dual union all
select 46280,'MRN' from dual union all
select 46281,'MRN' from dual union all
select 46282,'MRN' from dual union all
select 46283,'MRN' from dual union all
select 46284,'MRN' from dual union all
select 46285,'GMS' from dual union all
select 46286,'GMS' from dual union all
select 46287,'MRN' from dual union all
select 46288,'MRN' from dual union all
select 46289,'MRN' from dual union all
select 4629,'UNRV' from dual union all
select 46290,'MRN' from dual union all
select 46291,'MRN' from dual union all
select 46292,'GMS' from dual union all
select 46293,'MRN' from dual union all
select 46294,'MRN' from dual union all
select 46295,'MRN' from dual union all
select 46296,'MRN' from dual union all
select 46297,'MRN' from dual union all
select 46298,'MRN' from dual union all
select 46299,'MRN' from dual union all
select 463,'WER485-2' from dual union all
select 4630,'UNRV' from dual union all
select 46300,'NASA' from dual union all
select 46301,'GMS' from dual union all
select 46302,'MRN' from dual union all
select 46303,'MRN' from dual;
insert into LAUNCH_AGENCY
select 46304,'AN' from dual union all
select 46305,'MRN' from dual union all
select 46306,'MRN' from dual union all
select 46307,'MRN' from dual union all
select 46308,'MRN' from dual union all
select 46309,'MRN' from dual union all
select 4631,'UNRV' from dual union all
select 46310,'MRN' from dual union all
select 46311,'NASA' from dual union all
select 46312,'MRN' from dual union all
select 46313,'NASA' from dual union all
select 46314,'MRN' from dual union all
select 46315,'MRN' from dual union all
select 46316,'MRN' from dual union all
select 46317,'MRN' from dual union all
select 46318,'MRN' from dual union all
select 46319,'MRN' from dual union all
select 4632,'UNRV' from dual union all
select 46320,'GMS' from dual union all
select 46321,'MRN' from dual union all
select 46322,'MRN' from dual union all
select 46323,'MRN' from dual union all
select 46324,'MRN' from dual union all
select 46325,'MRN' from dual union all
select 46326,'GMS' from dual union all
select 46327,'MRN' from dual union all
select 46328,'NASA' from dual union all
select 46329,'NASA' from dual union all
select 4633,'UNRV' from dual union all
select 46330,'NASA' from dual union all
select 46331,'MRN' from dual union all
select 46332,'MRN' from dual union all
select 46333,'MRN' from dual union all
select 46334,'NASA' from dual union all
select 46335,'NASA' from dual union all
select 46336,'NASA' from dual union all
select 46337,'GMS' from dual union all
select 46338,'MRN' from dual union all
select 46339,'GMS' from dual union all
select 4634,'UNRV' from dual union all
select 46340,'MRN' from dual union all
select 46341,'MRN' from dual union all
select 46342,'MRN' from dual union all
select 46343,'MRN' from dual union all
select 46344,'MRN' from dual union all
select 46345,'NASA' from dual union all
select 46346,'MRN' from dual union all
select 46347,'MRN' from dual union all
select 46348,'MRN' from dual union all
select 46349,'MRN' from dual union all
select 4635,'UNRV' from dual union all
select 46350,'MRN' from dual union all
select 46351,'MRN' from dual union all
select 46352,'MRN' from dual union all
select 46353,'MRN' from dual union all
select 46354,'GMS' from dual union all
select 46355,'MRN' from dual union all
select 46356,'MRN' from dual union all
select 46357,'MRN' from dual union all
select 46358,'MRN' from dual union all
select 46359,'MRN' from dual union all
select 4636,'UNRV' from dual union all
select 46360,'MRN' from dual union all
select 46361,'GMS' from dual union all
select 46362,'GMS' from dual union all
select 46363,'MRN' from dual union all
select 46364,'MRN' from dual union all
select 46365,'MRN' from dual union all
select 46366,'MRN' from dual union all
select 46367,'MRN' from dual union all
select 46368,'MRN' from dual union all
select 46369,'MRN' from dual union all
select 4637,'UNRV' from dual union all
select 46370,'MRN' from dual union all
select 46371,'MRN' from dual union all
select 46372,'MRN' from dual union all
select 46373,'MRN' from dual union all
select 46374,'MRN' from dual union all
select 46375,'MRN' from dual union all
select 46376,'MRN' from dual union all
select 46377,'MRN' from dual union all
select 46378,'MRN' from dual union all
select 46379,'MRN' from dual union all
select 4638,'UNRV' from dual union all
select 46380,'MRN' from dual union all
select 46381,'GMS' from dual union all
select 46382,'MRN' from dual union all
select 46383,'MRN' from dual union all
select 46384,'MRN' from dual union all
select 46385,'MRN' from dual union all
select 46386,'MRN' from dual union all
select 46387,'MRN' from dual union all
select 46388,'MRN' from dual union all
select 46389,'MRN' from dual union all
select 4639,'UNRV' from dual union all
select 46390,'GMS' from dual union all
select 46391,'GMS' from dual union all
select 46392,'MRN' from dual union all
select 46393,'MRN' from dual union all
select 46394,'MRN' from dual;
insert into LAUNCH_AGENCY
select 46395,'MRN' from dual union all
select 46396,'NASA' from dual union all
select 46397,'MRN' from dual union all
select 46398,'MRN' from dual union all
select 46399,'MRN' from dual union all
select 464,'WER444' from dual union all
select 4640,'UNRV' from dual union all
select 46400,'MRN' from dual union all
select 46401,'MRN' from dual union all
select 46402,'MRN' from dual union all
select 46403,'MRN' from dual union all
select 46404,'MRN' from dual union all
select 46405,'MRN' from dual union all
select 46406,'MRN' from dual union all
select 46407,'MRN' from dual union all
select 46408,'MRN' from dual union all
select 46409,'MRN' from dual union all
select 4641,'UNRV' from dual union all
select 46410,'MRN' from dual union all
select 46411,'MRN' from dual union all
select 46412,'GMS' from dual union all
select 46413,'MRN' from dual union all
select 46414,'MRN' from dual union all
select 46415,'MRN' from dual union all
select 46416,'MRN' from dual union all
select 46417,'MRN' from dual union all
select 46418,'MRN' from dual union all
select 46419,'MRN' from dual union all
select 4642,'UNRV' from dual union all
select 46420,'NRCC' from dual union all
select 46421,'MRN' from dual union all
select 46422,'MRN' from dual union all
select 46423,'MRN' from dual union all
select 46424,'MRN' from dual union all
select 46425,'MRN' from dual union all
select 46426,'MRN' from dual union all
select 46427,'MRN' from dual union all
select 46428,'MRN' from dual union all
select 46429,'MRN' from dual union all
select 4643,'UNRV' from dual union all
select 46430,'MRN' from dual union all
select 46431,'MRN' from dual union all
select 46432,'MRN' from dual union all
select 46433,'MRN' from dual union all
select 46434,'MRN' from dual union all
select 46435,'MRN' from dual union all
select 46436,'MRN' from dual union all
select 46437,'MRN' from dual union all
select 46438,'MRN' from dual union all
select 46439,'MRN' from dual union all
select 4644,'UNRV' from dual union all
select 46440,'MRN' from dual union all
select 46441,'MRN' from dual union all
select 46442,'MRN' from dual union all
select 46443,'MRN' from dual union all
select 46444,'MRN' from dual union all
select 46445,'MRN' from dual union all
select 46446,'MRN' from dual union all
select 46447,'NASA' from dual union all
select 46448,'MRN' from dual union all
select 46449,'MRN' from dual union all
select 4645,'UNRV' from dual union all
select 46450,'MRN' from dual union all
select 46451,'MRN' from dual union all
select 46452,'MRN' from dual union all
select 46453,'MRN' from dual union all
select 46454,'MRN' from dual union all
select 46455,'MRN' from dual union all
select 46456,'MRN' from dual union all
select 46457,'MRN' from dual union all
select 46458,'MRN' from dual union all
select 46459,'MRN' from dual union all
select 4646,'UNRV' from dual union all
select 46460,'MRN' from dual union all
select 46461,'AN' from dual union all
select 46462,'MRN' from dual union all
select 46463,'MRN' from dual union all
select 46464,'MRN' from dual union all
select 46465,'MRN' from dual union all
select 46466,'MRN' from dual union all
select 46467,'MRN' from dual union all
select 46468,'MRN' from dual union all
select 46469,'MRN' from dual union all
select 4647,'UNRV' from dual union all
select 46470,'MRN' from dual union all
select 46471,'MRN' from dual union all
select 46472,'MRN' from dual union all
select 46473,'MRN' from dual union all
select 46474,'MRN' from dual union all
select 46475,'MRN' from dual union all
select 46476,'MRN' from dual union all
select 46477,'MRN' from dual union all
select 46478,'MRN' from dual union all
select 46479,'MRN' from dual union all
select 4648,'UNRV' from dual union all
select 46480,'MRN' from dual union all
select 46481,'MRN' from dual union all
select 46482,'MRN' from dual union all
select 46483,'MRN' from dual union all
select 46484,'MRN' from dual;
insert into LAUNCH_AGENCY
select 46485,'MRN' from dual union all
select 46486,'AN' from dual union all
select 46487,'AN' from dual union all
select 46488,'MRN' from dual union all
select 46489,'MRN' from dual union all
select 4649,'UNRV' from dual union all
select 46490,'MRN' from dual union all
select 46491,'MRN' from dual union all
select 46492,'MRN' from dual union all
select 46493,'MRN' from dual union all
select 46494,'MRN' from dual union all
select 46495,'MRN' from dual union all
select 46496,'MRN' from dual union all
select 46497,'MRN' from dual union all
select 46498,'MRN' from dual union all
select 46499,'MRN' from dual union all
select 465,'WER485' from dual union all
select 4650,'UNRV' from dual union all
select 46500,'MRN' from dual union all
select 46501,'MRN' from dual union all
select 46502,'MRN' from dual union all
select 46503,'MRN' from dual union all
select 46504,'MRN' from dual union all
select 46505,'AN' from dual union all
select 46506,'MRN' from dual union all
select 46507,'MRN' from dual union all
select 46508,'MRN' from dual union all
select 46509,'MRN' from dual union all
select 4651,'UNRV' from dual union all
select 46510,'MRN' from dual union all
select 46511,'MRN' from dual union all
select 46512,'AN' from dual union all
select 46513,'AN' from dual union all
select 46514,'MRN' from dual union all
select 46515,'MRN' from dual union all
select 46516,'MRN' from dual union all
select 46517,'NASA' from dual union all
select 46518,'MRN' from dual union all
select 46519,'NASA' from dual union all
select 4652,'UNRV' from dual union all
select 46520,'MRN' from dual union all
select 46521,'MRN' from dual union all
select 46522,'MRN' from dual union all
select 46523,'NASA' from dual union all
select 46524,'MRN' from dual union all
select 46525,'MRN' from dual union all
select 46526,'MRN' from dual union all
select 46527,'NASA' from dual union all
select 46528,'MRN' from dual union all
select 46529,'MRN' from dual union all
select 4653,'NRL' from dual union all
select 46530,'NASA' from dual union all
select 46531,'MRN' from dual union all
select 46532,'NASA' from dual union all
select 46533,'NASA' from dual union all
select 46534,'MRN' from dual union all
select 46535,'MRN' from dual union all
select 46536,'MRN' from dual union all
select 46537,'MRN' from dual union all
select 46538,'MRN' from dual union all
select 46539,'MRN' from dual union all
select 4654,'UNRV' from dual union all
select 46540,'MRN' from dual union all
select 46541,'MRN' from dual union all
select 46542,'MRN' from dual union all
select 46543,'MRN' from dual union all
select 46544,'MRN' from dual union all
select 46545,'MRN' from dual union all
select 46546,'MRN' from dual union all
select 46547,'MRN' from dual union all
select 46548,'MRN' from dual union all
select 46549,'MRN' from dual union all
select 4655,'UNRV' from dual union all
select 46550,'MRN' from dual union all
select 46551,'MRN' from dual union all
select 46552,'MRN' from dual union all
select 46553,'MRN' from dual union all
select 46554,'MRN' from dual union all
select 46555,'MRN' from dual union all
select 46556,'MRN' from dual union all
select 46557,'MRN' from dual union all
select 46558,'MRN' from dual union all
select 46559,'MRN' from dual union all
select 4656,'UNRV' from dual union all
select 46560,'MRN' from dual union all
select 46561,'MRN' from dual union all
select 46562,'MRN' from dual union all
select 46563,'MRN' from dual union all
select 46564,'AN' from dual union all
select 46565,'AN' from dual union all
select 46566,'AN' from dual union all
select 46567,'MRN' from dual union all
select 46568,'MRN' from dual union all
select 46569,'MRN' from dual union all
select 4657,'UNRV' from dual union all
select 46570,'MRN' from dual union all
select 46571,'MRN' from dual union all
select 46572,'MRN' from dual union all
select 46573,'NASA' from dual union all
select 46574,'MRN' from dual;
insert into LAUNCH_AGENCY
select 46575,'MRN' from dual union all
select 46576,'MRN' from dual union all
select 46577,'MRN' from dual union all
select 46578,'MRN' from dual union all
select 46579,'MRN' from dual union all
select 4658,'UNRV' from dual union all
select 46580,'MRN' from dual union all
select 46581,'MRN' from dual union all
select 46582,'MRN' from dual union all
select 46583,'MRN' from dual union all
select 46584,'MRN' from dual union all
select 46585,'MRN' from dual union all
select 46586,'MRN' from dual union all
select 46587,'MRN' from dual union all
select 46588,'MRN' from dual union all
select 46589,'MRN' from dual union all
select 4659,'UNRV' from dual union all
select 46590,'MRN' from dual union all
select 46591,'MRN' from dual union all
select 46592,'MRN' from dual union all
select 46593,'MRN' from dual union all
select 46594,'AN' from dual union all
select 46595,'MRN' from dual union all
select 46596,'MRN' from dual union all
select 46597,'MRN' from dual union all
select 46598,'MRN' from dual union all
select 46599,'MRN' from dual union all
select 466,'WER485' from dual union all
select 4660,'UNRV' from dual union all
select 46600,'MRN' from dual union all
select 46601,'MRN' from dual union all
select 46602,'MRN' from dual union all
select 46603,'MRN' from dual union all
select 46604,'MRN' from dual union all
select 46605,'MRN' from dual union all
select 46606,'MRN' from dual union all
select 46607,'MRN' from dual union all
select 46608,'MRN' from dual union all
select 46609,'MRN' from dual union all
select 4661,'UNRV' from dual union all
select 46610,'MRN' from dual union all
select 46611,'MRN' from dual union all
select 46612,'MRN' from dual union all
select 46613,'NASA' from dual union all
select 46614,'MRN' from dual union all
select 46615,'MRN' from dual union all
select 46616,'MRN' from dual union all
select 46617,'AN' from dual union all
select 46618,'AN' from dual union all
select 46619,'MRN' from dual union all
select 4662,'UNRV' from dual union all
select 46620,'MRN' from dual union all
select 46621,'MRN' from dual union all
select 46622,'MRN' from dual union all
select 46623,'MRN' from dual union all
select 46624,'MRN' from dual union all
select 46625,'MRN' from dual union all
select 46626,'MRN' from dual union all
select 46627,'MRN' from dual union all
select 46628,'MRN' from dual union all
select 46629,'MRN' from dual union all
select 4663,'UNRV' from dual union all
select 46630,'MRN' from dual union all
select 46631,'MRN' from dual union all
select 46632,'MRN' from dual union all
select 46633,'MRN' from dual union all
select 46634,'MRN' from dual union all
select 46635,'NASA' from dual union all
select 46636,'MRN' from dual union all
select 46637,'MRN' from dual union all
select 46638,'MRN' from dual union all
select 46639,'AN' from dual union all
select 4664,'NII88A' from dual union all
select 46640,'AN' from dual union all
select 46641,'MRN' from dual union all
select 46642,'MRN' from dual union all
select 46643,'NASA' from dual union all
select 46644,'MRN' from dual union all
select 46645,'MRN' from dual union all
select 46646,'MRN' from dual union all
select 46647,'MRN' from dual union all
select 46648,'NASA' from dual union all
select 46649,'MRN' from dual union all
select 4665,'UNRV' from dual union all
select 46650,'MRN' from dual union all
select 46651,'MRN' from dual union all
select 46652,'MRN' from dual union all
select 46653,'MRN' from dual union all
select 46654,'MRN' from dual union all
select 46655,'MRN' from dual union all
select 46656,'MRN' from dual union all
select 46657,'MRN' from dual union all
select 46658,'MRN' from dual union all
select 46659,'MRN' from dual union all
select 4666,'UNRV' from dual union all
select 46660,'MRN' from dual union all
select 46661,'MRN' from dual union all
select 46662,'MRN' from dual union all
select 46663,'MRN' from dual union all
select 46664,'MRN' from dual;
insert into LAUNCH_AGENCY
select 46665,'MRN' from dual union all
select 46666,'MRN' from dual union all
select 46667,'MRN' from dual union all
select 46668,'MRN' from dual union all
select 46669,'MRN' from dual union all
select 4667,'UNRV' from dual union all
select 46670,'MRN' from dual union all
select 46671,'MRN' from dual union all
select 46672,'GMS' from dual union all
select 46673,'MRN' from dual union all
select 46674,'MRN' from dual union all
select 46675,'MRN' from dual union all
select 46676,'MRN' from dual union all
select 46677,'MRN' from dual union all
select 46678,'MRN' from dual union all
select 46679,'MRN' from dual union all
select 4668,'UNRV' from dual union all
select 46680,'MRN' from dual union all
select 46681,'MRN' from dual union all
select 46682,'MRN' from dual union all
select 46683,'MRN' from dual union all
select 46684,'MRN' from dual union all
select 46685,'MRN' from dual union all
select 46686,'MRN' from dual union all
select 46687,'MRN' from dual union all
select 46688,'MRN' from dual union all
select 46689,'MRN' from dual union all
select 4669,'UNRV' from dual union all
select 46690,'MRN' from dual union all
select 46691,'AN' from dual union all
select 46692,'AN' from dual union all
select 46693,'MRN' from dual union all
select 46694,'MRN' from dual union all
select 46695,'MRN' from dual union all
select 46696,'GMS' from dual union all
select 46697,'MRN' from dual union all
select 46698,'MRN' from dual union all
select 46699,'MRN' from dual union all
select 467,'WER485' from dual union all
select 4670,'UNRV' from dual union all
select 46700,'MRN' from dual union all
select 46701,'MRN' from dual union all
select 46702,'MRN' from dual union all
select 46703,'MRN' from dual union all
select 46704,'MRN' from dual union all
select 46705,'MRN' from dual union all
select 46706,'MRN' from dual union all
select 46707,'MRN' from dual union all
select 46708,'MRN' from dual union all
select 46709,'MRN' from dual union all
select 4671,'UNRV' from dual union all
select 46710,'MRN' from dual union all
select 46711,'MRN' from dual union all
select 46712,'MRN' from dual union all
select 46713,'MRN' from dual union all
select 46714,'MRN' from dual union all
select 46715,'AN' from dual union all
select 46716,'MRN' from dual union all
select 46717,'MRN' from dual union all
select 46718,'MRN' from dual union all
select 46719,'MRN' from dual union all
select 4672,'UNRV' from dual union all
select 46720,'MRN' from dual union all
select 46721,'MRN' from dual union all
select 46722,'MRN' from dual union all
select 46723,'MRN' from dual union all
select 46724,'MRN' from dual union all
select 46725,'MRN' from dual union all
select 46726,'MRN' from dual union all
select 46727,'MRN' from dual union all
select 46728,'MRN' from dual union all
select 46729,'MRN' from dual union all
select 4673,'UNRV' from dual union all
select 46730,'MRN' from dual union all
select 46731,'MRN' from dual union all
select 46732,'MRN' from dual union all
select 46733,'MRN' from dual union all
select 46734,'MRN' from dual union all
select 46735,'MRN' from dual union all
select 46736,'MRN' from dual union all
select 46737,'MRN' from dual union all
select 46738,'MRN' from dual union all
select 46739,'MRN' from dual union all
select 4674,'UNRV' from dual union all
select 46740,'MRN' from dual union all
select 46741,'MRN' from dual union all
select 46742,'MRN' from dual union all
select 46743,'NASA' from dual union all
select 46744,'MRN' from dual union all
select 46745,'MRN' from dual union all
select 46746,'NASA' from dual union all
select 46747,'MRN' from dual union all
select 46748,'MRN' from dual union all
select 46749,'MRN' from dual union all
select 4675,'USN' from dual union all
select 46750,'MRN' from dual union all
select 46751,'MRN' from dual union all
select 46752,'MRN' from dual union all
select 46753,'MRN' from dual union all
select 46754,'MRN' from dual;
insert into LAUNCH_AGENCY
select 46755,'MRN' from dual union all
select 46756,'MRN' from dual union all
select 46757,'MRN' from dual union all
select 46758,'MRN' from dual union all
select 46759,'MRN' from dual union all
select 4676,'UNRV' from dual union all
select 46760,'MRN' from dual union all
select 46761,'MRN' from dual union all
select 46762,'MRN' from dual union all
select 46763,'MRN' from dual union all
select 46764,'MRN' from dual union all
select 46765,'MRN' from dual union all
select 46766,'AN' from dual union all
select 46767,'MRN' from dual union all
select 46768,'MRN' from dual union all
select 46769,'MRN' from dual union all
select 4677,'UNRV' from dual union all
select 46770,'MRN' from dual union all
select 46771,'MRN' from dual union all
select 46772,'MRN' from dual union all
select 46773,'MRN' from dual union all
select 46774,'MRN' from dual union all
select 46775,'MRN' from dual union all
select 46776,'MRN' from dual union all
select 46777,'MRN' from dual union all
select 46778,'MRN' from dual union all
select 46779,'MRN' from dual union all
select 4678,'UNRV' from dual union all
select 46780,'MRN' from dual union all
select 46781,'MRN' from dual union all
select 46782,'MRN' from dual union all
select 46783,'MRN' from dual union all
select 46784,'MRN' from dual union all
select 46785,'MRN' from dual union all
select 46786,'MRN' from dual union all
select 46787,'MRN' from dual union all
select 46788,'MRN' from dual union all
select 46789,'MRN' from dual union all
select 4679,'UNRV' from dual union all
select 46790,'MRN' from dual union all
select 46791,'MRN' from dual union all
select 46792,'MRN' from dual union all
select 46793,'MRN' from dual union all
select 46794,'MRN' from dual union all
select 46795,'MRN' from dual union all
select 46796,'MRN' from dual union all
select 46797,'MRN' from dual union all
select 46798,'GMS' from dual union all
select 46799,'GMS' from dual union all
select 468,'WER485' from dual union all
select 4680,'UNRV' from dual union all
select 46800,'MRN' from dual union all
select 46801,'MRN' from dual union all
select 46802,'MRN' from dual union all
select 46803,'MRN' from dual union all
select 46804,'MRN' from dual union all
select 46805,'MRN' from dual union all
select 46806,'GMS' from dual union all
select 46807,'GMS' from dual union all
select 46808,'MRN' from dual union all
select 46809,'MRN' from dual union all
select 4681,'UNRV' from dual union all
select 46810,'MRN' from dual union all
select 46811,'MRN' from dual union all
select 46812,'MRN' from dual union all
select 46813,'MRN' from dual union all
select 46814,'MRN' from dual union all
select 46815,'MRN' from dual union all
select 46816,'MRN' from dual union all
select 46817,'MRN' from dual union all
select 46818,'GMS' from dual union all
select 46819,'MRN' from dual union all
select 4682,'UNRV' from dual union all
select 46820,'MRN' from dual union all
select 46821,'MRN' from dual union all
select 46822,'MRN' from dual union all
select 46823,'MRN' from dual union all
select 46824,'MRN' from dual union all
select 46825,'MRN' from dual union all
select 46826,'MRN' from dual union all
select 46827,'MRN' from dual union all
select 46828,'MRN' from dual union all
select 46829,'MRN' from dual union all
select 4683,'UNRV' from dual union all
select 46830,'MRN' from dual union all
select 46831,'MRN' from dual union all
select 46832,'MRN' from dual union all
select 46833,'MRN' from dual union all
select 46834,'MRN' from dual union all
select 46835,'MRN' from dual union all
select 46836,'MRN' from dual union all
select 46837,'MRN' from dual union all
select 46838,'MRN' from dual union all
select 46839,'MRN' from dual union all
select 4684,'UNRV' from dual union all
select 46840,'MRN' from dual union all
select 46841,'MRN' from dual union all
select 46842,'MRN' from dual union all
select 46843,'MRN' from dual union all
select 46844,'MRN' from dual;
insert into LAUNCH_AGENCY
select 46845,'MRN' from dual union all
select 46846,'MRN' from dual union all
select 46847,'MRN' from dual union all
select 46848,'MRN' from dual union all
select 46849,'MRN' from dual union all
select 4685,'UNRV' from dual union all
select 46850,'MRN' from dual union all
select 46851,'MRN' from dual union all
select 46852,'MRN' from dual union all
select 46853,'MRN' from dual union all
select 46854,'MRN' from dual union all
select 46855,'MRN' from dual union all
select 46856,'MRN' from dual union all
select 46857,'NASA' from dual union all
select 46858,'MRN' from dual union all
select 46859,'MRN' from dual union all
select 4686,'NII88A' from dual union all
select 46860,'MRN' from dual union all
select 46861,'AN' from dual union all
select 46862,'MRN' from dual union all
select 46863,'MRN' from dual union all
select 46864,'NASA' from dual union all
select 46865,'NASA' from dual union all
select 46866,'MRN' from dual union all
select 46867,'NASA' from dual union all
select 46868,'MRN' from dual union all
select 46869,'MRN' from dual union all
select 4687,'UNRV' from dual union all
select 46870,'MRN' from dual union all
select 46871,'MRN' from dual union all
select 46872,'MRN' from dual union all
select 46873,'MRN' from dual union all
select 46874,'MRN' from dual union all
select 46875,'MRN' from dual union all
select 46876,'MRN' from dual union all
select 46877,'MRN' from dual union all
select 46878,'MRN' from dual union all
select 46879,'MRN' from dual union all
select 4688,'UNRV' from dual union all
select 46880,'MRN' from dual union all
select 46881,'MRN' from dual union all
select 46882,'MRN' from dual union all
select 46883,'MRN' from dual union all
select 46884,'MRN' from dual union all
select 46885,'MRN' from dual union all
select 46886,'MRN' from dual union all
select 46887,'MRN' from dual union all
select 46888,'MRN' from dual union all
select 46889,'AN' from dual union all
select 4689,'UNRV' from dual union all
select 46890,'MRN' from dual union all
select 46891,'MRN' from dual union all
select 46892,'MRN' from dual union all
select 46893,'MRN' from dual union all
select 46894,'NASA' from dual union all
select 46895,'MRN' from dual union all
select 46896,'MRN' from dual union all
select 46897,'MRN' from dual union all
select 46898,'MRN' from dual union all
select 46899,'MRN' from dual union all
select 469,'WER485' from dual union all
select 4690,'UNRV' from dual union all
select 46900,'AN' from dual union all
select 46901,'NASDA' from dual union all
select 46902,'MRN' from dual union all
select 46903,'MRN' from dual union all
select 46904,'MRN' from dual union all
select 46905,'MRN' from dual union all
select 46906,'MRN' from dual union all
select 46907,'NASA' from dual union all
select 46908,'MRN' from dual union all
select 46909,'NASDA' from dual union all
select 4691,'UNRV' from dual union all
select 46910,'AN' from dual union all
select 46911,'NASA' from dual union all
select 46912,'MRN' from dual union all
select 46913,'MRN' from dual union all
select 46914,'MRN' from dual union all
select 46915,'MRN' from dual union all
select 46916,'MRN' from dual union all
select 46917,'MRN' from dual union all
select 46918,'NASA' from dual union all
select 46919,'AN' from dual union all
select 4692,'UNRV' from dual union all
select 46920,'MRN' from dual union all
select 46921,'MRN' from dual union all
select 46922,'MRN' from dual union all
select 46923,'MRN' from dual union all
select 46924,'MRN' from dual union all
select 46925,'MRN' from dual union all
select 46926,'MRN' from dual union all
select 46927,'MRN' from dual union all
select 46928,'MRN' from dual union all
select 46929,'MRN' from dual union all
select 4693,'UNRV' from dual union all
select 46930,'NASA' from dual union all
select 46931,'MRN' from dual union all
select 46932,'MRN' from dual union all
select 46933,'NASA' from dual union all
select 46934,'MRN' from dual;
insert into LAUNCH_AGENCY
select 46935,'MRN' from dual union all
select 46936,'GMS' from dual union all
select 46937,'MRN' from dual union all
select 46938,'MRN' from dual union all
select 46939,'MRN' from dual union all
select 4694,'UNRV' from dual union all
select 46940,'MRN' from dual union all
select 46941,'MRN' from dual union all
select 46942,'MRN' from dual union all
select 46943,'MRN' from dual union all
select 46944,'MRN' from dual union all
select 46945,'MRN' from dual union all
select 46946,'MRN' from dual union all
select 46947,'MRN' from dual union all
select 46948,'MRN' from dual union all
select 46949,'GMS' from dual union all
select 4695,'UNRV' from dual union all
select 46950,'MRN' from dual union all
select 46951,'MRN' from dual union all
select 46952,'MRN' from dual union all
select 46953,'MRN' from dual union all
select 46954,'MRN' from dual union all
select 46955,'MRN' from dual union all
select 46956,'MRN' from dual union all
select 46957,'MRN' from dual union all
select 46958,'MRN' from dual union all
select 46959,'GMS' from dual union all
select 4696,'UNRV' from dual union all
select 46960,'MRN' from dual union all
select 46961,'MRN' from dual union all
select 46962,'MRN' from dual union all
select 46963,'MRN' from dual union all
select 46964,'MRN' from dual union all
select 46965,'MRN' from dual union all
select 46966,'MRN' from dual union all
select 46967,'MRN' from dual union all
select 46968,'MRN' from dual union all
select 46969,'MRN' from dual union all
select 4697,'ATEF' from dual union all
select 46970,'MRN' from dual union all
select 46971,'MRN' from dual union all
select 46972,'MRN' from dual union all
select 46973,'MRN' from dual union all
select 46974,'MRN' from dual union all
select 46975,'GMS' from dual union all
select 46976,'MRN' from dual union all
select 46977,'MRN' from dual union all
select 46978,'MRN' from dual union all
select 46979,'MRN' from dual union all
select 4698,'UNRV' from dual union all
select 46980,'MRN' from dual union all
select 46981,'MRN' from dual union all
select 46982,'MRN' from dual union all
select 46983,'MRN' from dual union all
select 46984,'MRN' from dual union all
select 46985,'MRN' from dual union all
select 46986,'GMS' from dual union all
select 46987,'MRN' from dual union all
select 46988,'MRN' from dual union all
select 46989,'MRN' from dual union all
select 4699,'UNRV' from dual union all
select 46990,'MRN' from dual union all
select 46991,'MRN' from dual union all
select 46992,'MRN' from dual union all
select 46993,'MRN' from dual union all
select 46994,'MRN' from dual union all
select 46995,'MRN' from dual union all
select 46996,'MRN' from dual union all
select 46997,'MRN' from dual union all
select 46998,'MRN' from dual union all
select 46999,'MRN' from dual union all
select 47,'WER444' from dual union all
select 470,'WER485' from dual union all
select 4700,'UNRV' from dual union all
select 47000,'GMS' from dual union all
select 47001,'NASA' from dual union all
select 47002,'MRN' from dual union all
select 47003,'MRN' from dual union all
select 47004,'MRN' from dual union all
select 47005,'MRN' from dual union all
select 47006,'MRN' from dual union all
select 47007,'NASA' from dual union all
select 47008,'MRN' from dual union all
select 47009,'MRN' from dual union all
select 4701,'UNRV' from dual union all
select 47010,'NASA' from dual union all
select 47011,'MRN' from dual union all
select 47012,'MRN' from dual union all
select 47013,'NASA' from dual union all
select 47014,'MRN' from dual union all
select 47015,'MRN' from dual union all
select 47016,'MRN' from dual union all
select 47017,'GMS' from dual union all
select 47018,'MRN' from dual union all
select 47019,'MRN' from dual union all
select 4702,'UNRV' from dual union all
select 47020,'MRN' from dual union all
select 47021,'MRN' from dual union all
select 47022,'MRN' from dual union all
select 47023,'MRN' from dual;
insert into LAUNCH_AGENCY
select 47024,'MRN' from dual union all
select 47025,'MRN' from dual union all
select 47026,'MRN' from dual union all
select 47027,'MRN' from dual union all
select 47028,'MRN' from dual union all
select 47029,'MRN' from dual union all
select 4703,'USAF' from dual union all
select 47030,'MRN' from dual union all
select 47031,'NASA' from dual union all
select 47032,'NASA' from dual union all
select 47033,'AN' from dual union all
select 47034,'AN' from dual union all
select 47035,'AN' from dual union all
select 47036,'MRN' from dual union all
select 47037,'MRN' from dual union all
select 47038,'MRN' from dual union all
select 47039,'AN' from dual union all
select 4704,'ATEF' from dual union all
select 47040,'AN' from dual union all
select 47041,'AN' from dual union all
select 47042,'AN' from dual union all
select 47043,'AN' from dual union all
select 47044,'AN' from dual union all
select 47045,'MRN' from dual union all
select 47046,'AN' from dual union all
select 47047,'AN' from dual union all
select 47048,'AN' from dual union all
select 47049,'AN' from dual union all
select 4705,'NRL' from dual union all
select 47050,'AN' from dual union all
select 47051,'AN' from dual union all
select 47052,'AN' from dual union all
select 47053,'MRN' from dual union all
select 47054,'AN' from dual union all
select 47055,'AN' from dual union all
select 47056,'MRN' from dual union all
select 47057,'MRN' from dual union all
select 47058,'AN' from dual union all
select 47059,'MRN' from dual union all
select 4706,'USN' from dual union all
select 47060,'AN' from dual union all
select 47061,'MRN' from dual union all
select 47062,'AN' from dual union all
select 47063,'AN' from dual union all
select 47064,'AN' from dual union all
select 47065,'AN' from dual union all
select 47066,'AN' from dual union all
select 47067,'AN' from dual union all
select 47068,'MRN' from dual union all
select 47069,'MRN' from dual union all
select 4707,'NRL' from dual union all
select 47070,'AN' from dual union all
select 47071,'AN' from dual union all
select 47072,'AN' from dual union all
select 47073,'AN' from dual union all
select 47074,'AN' from dual union all
select 47075,'AN' from dual union all
select 47076,'AN' from dual union all
select 47077,'AN' from dual union all
select 47078,'MRN' from dual union all
select 47079,'AN' from dual union all
select 4708,'USA' from dual union all
select 47080,'AN' from dual union all
select 47081,'AN' from dual union all
select 47082,'AN' from dual union all
select 47083,'AN' from dual union all
select 47084,'AN' from dual union all
select 47085,'AN' from dual union all
select 47086,'AN' from dual union all
select 47087,'AN' from dual union all
select 47088,'MRN' from dual union all
select 47089,'MRN' from dual union all
select 4709,'AN' from dual union all
select 47090,'AN' from dual union all
select 47091,'MRN' from dual union all
select 47092,'AN' from dual union all
select 47093,'AN' from dual union all
select 47094,'AN' from dual union all
select 47095,'AN' from dual union all
select 47096,'AN' from dual union all
select 47097,'AN' from dual union all
select 47098,'AN' from dual union all
select 47099,'AN' from dual union all
select 471,'WER485' from dual union all
select 4710,'AN' from dual union all
select 47100,'AN' from dual union all
select 47101,'MRN' from dual union all
select 47102,'MRN' from dual union all
select 47103,'AN' from dual union all
select 47104,'AN' from dual union all
select 47105,'AN' from dual union all
select 47106,'MRN' from dual union all
select 47107,'MRN' from dual union all
select 47108,'MRN' from dual union all
select 47109,'MRN' from dual union all
select 4711,'AN' from dual union all
select 47110,'AN' from dual union all
select 47111,'AN' from dual union all
select 47112,'AN' from dual union all
select 47113,'AN' from dual;
insert into LAUNCH_AGENCY
select 47114,'AN' from dual union all
select 47115,'AN' from dual union all
select 47116,'AN' from dual union all
select 47117,'MRN' from dual union all
select 47118,'AN' from dual union all
select 47119,'AN' from dual union all
select 4712,'ABMA' from dual union all
select 4712,'USAF' from dual union all
select 47120,'AN' from dual union all
select 47121,'MRN' from dual union all
select 47122,'AN' from dual union all
select 47123,'AN' from dual union all
select 47124,'AN' from dual union all
select 47125,'AN' from dual union all
select 47126,'MRN' from dual union all
select 47127,'AN' from dual union all
select 47128,'NASA' from dual union all
select 47129,'AN' from dual union all
select 4713,'UNRV' from dual union all
select 47130,'AN' from dual union all
select 47131,'MRN' from dual union all
select 47132,'MRN' from dual union all
select 47133,'MRN' from dual union all
select 47134,'MRN' from dual union all
select 47135,'MRN' from dual union all
select 47136,'AN' from dual union all
select 47137,'AN' from dual union all
select 47138,'MRN' from dual union all
select 47139,'AN' from dual union all
select 4714,'UNRV' from dual union all
select 47140,'MRN' from dual union all
select 47141,'AN' from dual union all
select 47142,'AN' from dual union all
select 47143,'AN' from dual union all
select 47144,'AN' from dual union all
select 47145,'AN' from dual union all
select 47146,'AN' from dual union all
select 47147,'MRN' from dual union all
select 47148,'AN' from dual union all
select 47149,'AN' from dual union all
select 4715,'ARDC' from dual union all
select 47150,'AN' from dual union all
select 47151,'AN' from dual union all
select 47152,'AN' from dual union all
select 47153,'AN' from dual union all
select 47154,'AN' from dual union all
select 47155,'AN' from dual union all
select 47156,'MRN' from dual union all
select 47157,'MRN' from dual union all
select 47158,'MRN' from dual union all
select 47159,'AN' from dual union all
select 4716,'MVS' from dual union all
select 47160,'AN' from dual union all
select 47161,'AN' from dual union all
select 47162,'AN' from dual union all
select 47163,'AN' from dual union all
select 47164,'MRN' from dual union all
select 47165,'MRN' from dual union all
select 47166,'AN' from dual union all
select 47167,'AN' from dual union all
select 47168,'AN' from dual union all
select 47169,'AN' from dual union all
select 4717,'UNRV' from dual union all
select 47170,'AN' from dual union all
select 47171,'MRN' from dual union all
select 47172,'AN' from dual union all
select 47173,'AN' from dual union all
select 47174,'AN' from dual union all
select 47175,'AN' from dual union all
select 47176,'AN' from dual union all
select 47177,'AN' from dual union all
select 47178,'MRN' from dual union all
select 47179,'AN' from dual union all
select 4718,'UNRV' from dual union all
select 47180,'MRN' from dual union all
select 47181,'MRN' from dual union all
select 47182,'AN' from dual union all
select 47183,'AN' from dual union all
select 47184,'AN' from dual union all
select 47185,'AN' from dual union all
select 47186,'AN' from dual union all
select 47187,'AN' from dual union all
select 47188,'AN' from dual union all
select 47189,'MRN' from dual union all
select 4719,'ARDC' from dual union all
select 47190,'AN' from dual union all
select 47191,'MRN' from dual union all
select 47192,'MRN' from dual union all
select 47193,'AN' from dual union all
select 47194,'AN' from dual union all
select 47195,'AN' from dual union all
select 47196,'AN' from dual union all
select 47197,'AN' from dual union all
select 47198,'MRN' from dual union all
select 47199,'MRN' from dual union all
select 472,'WER485' from dual union all
select 4720,'USA' from dual union all
select 47200,'MRN' from dual union all
select 47201,'AN' from dual union all
select 47202,'AN' from dual;
insert into LAUNCH_AGENCY
select 47203,'AN' from dual union all
select 47204,'AN' from dual union all
select 47205,'AN' from dual union all
select 47206,'AN' from dual union all
select 47207,'AN' from dual union all
select 47208,'AN' from dual union all
select 47209,'AN' from dual union all
select 4721,'USN' from dual union all
select 47210,'MRN' from dual union all
select 47211,'MRN' from dual union all
select 47212,'MRN' from dual union all
select 47213,'MRN' from dual union all
select 47214,'AN' from dual union all
select 47215,'MRN' from dual union all
select 47216,'AN' from dual union all
select 47217,'AN' from dual union all
select 47218,'AN' from dual union all
select 47219,'AN' from dual union all
select 4722,'UNRV' from dual union all
select 47220,'AN' from dual union all
select 47221,'AN' from dual union all
select 47222,'AN' from dual union all
select 47223,'MRN' from dual union all
select 47224,'MRN' from dual union all
select 47225,'AN' from dual union all
select 47226,'AN' from dual union all
select 47227,'AN' from dual union all
select 47228,'AN' from dual union all
select 47229,'AN' from dual union all
select 4723,'UNRV' from dual union all
select 47230,'MRN' from dual union all
select 47231,'MRN' from dual union all
select 47232,'MRN' from dual union all
select 47233,'AN' from dual union all
select 47234,'AN' from dual union all
select 47235,'AN' from dual union all
select 47236,'AN' from dual union all
select 47237,'MRN' from dual union all
select 47238,'AN' from dual union all
select 47239,'MRN' from dual union all
select 4724,'MVS' from dual union all
select 47240,'MRN' from dual union all
select 47241,'MRN' from dual union all
select 47242,'AN' from dual union all
select 47243,'AN' from dual union all
select 47244,'AN' from dual union all
select 47245,'AN' from dual union all
select 47246,'AN' from dual union all
select 47247,'AN' from dual union all
select 47248,'MRN' from dual union all
select 47249,'AN' from dual union all
select 4725,'MVS' from dual union all
select 47250,'MRN' from dual union all
select 47251,'AN' from dual union all
select 47252,'MRN' from dual union all
select 47253,'AN' from dual union all
select 47254,'AN' from dual union all
select 47255,'AN' from dual union all
select 47256,'AN' from dual union all
select 47257,'AN' from dual union all
select 47258,'AN' from dual union all
select 47259,'AN' from dual union all
select 4726,'MVS' from dual union all
select 47260,'AN' from dual union all
select 47261,'AN' from dual union all
select 47262,'AN' from dual union all
select 47263,'AN' from dual union all
select 47264,'MRN' from dual union all
select 47265,'AN' from dual union all
select 47266,'MRN' from dual union all
select 47267,'MRN' from dual union all
select 47268,'AN' from dual union all
select 47269,'AN' from dual union all
select 4727,'NRL' from dual union all
select 47270,'AN' from dual union all
select 47271,'MRN' from dual union all
select 47272,'AN' from dual union all
select 47273,'AN' from dual union all
select 47274,'AN' from dual union all
select 47275,'AN' from dual union all
select 47276,'AN' from dual union all
select 47277,'MRN' from dual union all
select 47278,'AN' from dual union all
select 47279,'MRN' from dual union all
select 4728,'NRL' from dual union all
select 47280,'AN' from dual union all
select 47281,'AN' from dual union all
select 47282,'MRN' from dual union all
select 47283,'AN' from dual union all
select 47284,'AN' from dual union all
select 47285,'AN' from dual union all
select 47286,'AN' from dual union all
select 47287,'AN' from dual union all
select 47288,'AN' from dual union all
select 47289,'MRN' from dual union all
select 4729,'UNRV' from dual union all
select 47290,'AN' from dual union all
select 47291,'AN' from dual union all
select 47292,'ISRO' from dual union all
select 47293,'MRN' from dual;
insert into LAUNCH_AGENCY
select 47294,'AN' from dual union all
select 47295,'AN' from dual union all
select 47296,'AN' from dual union all
select 47297,'AN' from dual union all
select 47298,'AN' from dual union all
select 47299,'AN' from dual union all
select 473,'WER500' from dual union all
select 4730,'NRL' from dual union all
select 47300,'AN' from dual union all
select 47301,'MRN' from dual union all
select 47302,'MRN' from dual union all
select 47303,'AN' from dual union all
select 47304,'AN' from dual union all
select 47305,'AN' from dual union all
select 47306,'AN' from dual union all
select 47307,'AN' from dual union all
select 47308,'AN' from dual union all
select 47309,'MRN' from dual union all
select 4731,'UNRV' from dual union all
select 47310,'MRN' from dual union all
select 47311,'AN' from dual union all
select 47312,'AN' from dual union all
select 47313,'MRN' from dual union all
select 47314,'AN' from dual union all
select 47315,'AN' from dual union all
select 47316,'AN' from dual union all
select 47317,'MRN' from dual union all
select 47318,'AN' from dual union all
select 47319,'MRN' from dual union all
select 4732,'USA' from dual union all
select 47320,'MRN' from dual union all
select 47321,'AN' from dual union all
select 47322,'MRN' from dual union all
select 47323,'AN' from dual union all
select 47324,'AN' from dual union all
select 47325,'AN' from dual union all
select 47326,'AN' from dual union all
select 47327,'AN' from dual union all
select 47328,'MRN' from dual union all
select 47329,'MRN' from dual union all
select 4733,'USAF' from dual union all
select 47330,'MRN' from dual union all
select 47331,'MRN' from dual union all
select 47332,'AN' from dual union all
select 47333,'AN' from dual union all
select 47334,'AN' from dual union all
select 47335,'MRN' from dual union all
select 47336,'MRN' from dual union all
select 47337,'AN' from dual union all
select 47338,'AN' from dual union all
select 47339,'AN' from dual union all
select 4734,'USN' from dual union all
select 47340,'AN' from dual union all
select 47341,'AN' from dual union all
select 47342,'AN' from dual union all
select 47343,'AN' from dual union all
select 47344,'AN' from dual union all
select 47345,'MRN' from dual union all
select 47346,'MRN' from dual union all
select 47347,'AN' from dual union all
select 47348,'AN' from dual union all
select 47349,'AN' from dual union all
select 4735,'USAF' from dual union all
select 47350,'AN' from dual union all
select 47351,'AN' from dual union all
select 47352,'AN' from dual union all
select 47353,'MRN' from dual union all
select 47354,'MRN' from dual union all
select 47355,'MRN' from dual union all
select 47356,'MRN' from dual union all
select 47357,'MRN' from dual union all
select 47358,'AN' from dual union all
select 47359,'AN' from dual union all
select 4736,'USN' from dual union all
select 47360,'AN' from dual union all
select 47361,'AN' from dual union all
select 47362,'MRN' from dual union all
select 47363,'MRN' from dual union all
select 47364,'MRN' from dual union all
select 47365,'AN' from dual union all
select 47366,'AN' from dual union all
select 47367,'AN' from dual union all
select 47368,'AN' from dual union all
select 47369,'MRN' from dual union all
select 4737,'USAF' from dual union all
select 47370,'MRN' from dual union all
select 47371,'AN' from dual union all
select 47372,'MRN' from dual union all
select 47373,'MRN' from dual union all
select 47374,'AN' from dual union all
select 47375,'MRN' from dual union all
select 47376,'AN' from dual union all
select 47377,'AN' from dual union all
select 47378,'AN' from dual union all
select 47379,'MRN' from dual union all
select 4738,'USA' from dual union all
select 47380,'MRN' from dual union all
select 47381,'MRN' from dual union all
select 47382,'AN' from dual union all
select 47383,'AN' from dual;
insert into LAUNCH_AGENCY
select 47384,'AN' from dual union all
select 47385,'AN' from dual union all
select 47386,'AN' from dual union all
select 47387,'MRN' from dual union all
select 47388,'MRN' from dual union all
select 47389,'MRN' from dual union all
select 4739,'RAE' from dual union all
select 4739,'WRE' from dual union all
select 47390,'MRN' from dual union all
select 47391,'AN' from dual union all
select 47392,'AN' from dual union all
select 47393,'AN' from dual union all
select 47394,'AN' from dual union all
select 47395,'MRN' from dual union all
select 47396,'MRN' from dual union all
select 47397,'AN' from dual union all
select 47398,'MRN' from dual union all
select 47399,'AN' from dual union all
select 474,'WER485' from dual union all
select 4740,'NRL' from dual union all
select 47400,'MRN' from dual union all
select 47401,'AN' from dual union all
select 47402,'AN' from dual union all
select 47403,'AN' from dual union all
select 47404,'MRN' from dual union all
select 47405,'AN' from dual union all
select 47406,'AN' from dual union all
select 47407,'AN' from dual union all
select 47408,'AN' from dual union all
select 47409,'MRN' from dual union all
select 4741,'USA' from dual union all
select 47410,'MRN' from dual union all
select 47411,'MRN' from dual union all
select 47412,'AN' from dual union all
select 47413,'AN' from dual union all
select 47414,'AN' from dual union all
select 47415,'AN' from dual union all
select 47416,'AN' from dual union all
select 47417,'MRN' from dual union all
select 47418,'NASA' from dual union all
select 47419,'AN' from dual union all
select 4742,'USA' from dual union all
select 47420,'AN' from dual union all
select 47421,'AN' from dual union all
select 47422,'AN' from dual union all
select 47423,'MRN' from dual union all
select 47424,'MRN' from dual union all
select 47425,'MRN' from dual union all
select 47426,'MRN' from dual union all
select 47427,'AN' from dual union all
select 47428,'AN' from dual union all
select 47429,'AN' from dual union all
select 4743,'NRL' from dual union all
select 47430,'AN' from dual union all
select 47431,'AN' from dual union all
select 47432,'MRN' from dual union all
select 47433,'AN' from dual union all
select 47434,'AN' from dual union all
select 47435,'AN' from dual union all
select 47436,'AN' from dual union all
select 47437,'AN' from dual union all
select 47438,'MRN' from dual union all
select 47439,'AN' from dual union all
select 4744,'UNRV' from dual union all
select 47440,'AN' from dual union all
select 47441,'MRN' from dual union all
select 47442,'AN' from dual union all
select 47443,'NRCC' from dual union all
select 47444,'AN' from dual union all
select 47445,'AN' from dual union all
select 47446,'AN' from dual union all
select 47447,'AN' from dual union all
select 47448,'MRN' from dual union all
select 47449,'MRN' from dual union all
select 4745,'MVS' from dual union all
select 47450,'AN' from dual union all
select 47451,'AN' from dual union all
select 47452,'AN' from dual union all
select 47453,'AN' from dual union all
select 47454,'AN' from dual union all
select 47455,'AN' from dual union all
select 47456,'MRN' from dual union all
select 47457,'MRN' from dual union all
select 47458,'AN' from dual union all
select 47459,'AN' from dual union all
select 4746,'MVS' from dual union all
select 47460,'AN' from dual union all
select 47461,'MRN' from dual union all
select 47462,'GMS' from dual union all
select 47463,'MRN' from dual union all
select 47464,'AN' from dual union all
select 47465,'GMS' from dual union all
select 47466,'AN' from dual union all
select 47467,'AN' from dual union all
select 47468,'MRN' from dual union all
select 47469,'AN' from dual union all
select 4747,'SUI' from dual union all
select 47470,'AN' from dual union all
select 47471,'GMS' from dual union all
select 47472,'AN' from dual;
insert into LAUNCH_AGENCY
select 47473,'MRN' from dual union all
select 47474,'MRN' from dual union all
select 47475,'AN' from dual union all
select 47476,'MRN' from dual union all
select 47477,'MRN' from dual union all
select 47478,'AN' from dual union all
select 47479,'AN' from dual union all
select 4748,'SUI' from dual union all
select 47480,'AN' from dual union all
select 47481,'AN' from dual union all
select 47482,'AN' from dual union all
select 47483,'MRN' from dual union all
select 47484,'AN' from dual union all
select 47485,'RVSN' from dual union all
select 47486,'RVSN' from dual union all
select 47487,'RVSN' from dual union all
select 47488,'RVSN' from dual union all
select 47489,'RVSN' from dual union all
select 4749,'USAF' from dual union all
select 47490,'RVSN' from dual union all
select 47491,'NASA' from dual union all
select 47492,'RVSN' from dual union all
select 47493,'NASDA' from dual union all
select 47494,'RVSN' from dual union all
select 47495,'RVSN' from dual union all
select 47496,'RVSN' from dual union all
select 47497,'NASA' from dual union all
select 47498,'ISAS' from dual union all
select 47499,'RVSN' from dual union all
select 475,'WER500' from dual union all
select 4750,'SUI' from dual union all
select 47500,'RVSN' from dual union all
select 47501,'AFSC' from dual union all
select 47502,'RVSN' from dual union all
select 47503,'RVSN' from dual union all
select 47504,'RVSN' from dual union all
select 47505,'RVSN' from dual union all
select 47506,'RVSN' from dual union all
select 47507,'RVSN' from dual union all
select 47508,'RVSN' from dual union all
select 47509,'AFSC' from dual union all
select 4751,'SUI' from dual union all
select 47510,'RVSN' from dual union all
select 47511,'RVSN' from dual union all
select 47512,'RVSN' from dual union all
select 47513,'RVSN' from dual union all
select 47514,'RVSN' from dual union all
select 47515,'RVSN' from dual union all
select 47516,'RVSN' from dual union all
select 47517,'RVSN' from dual union all
select 47518,'RVSN' from dual union all
select 47519,'RVSN' from dual union all
select 4752,'UNRV' from dual union all
select 47520,'RVSN' from dual union all
select 47521,'RVSN' from dual union all
select 47522,'NASA' from dual union all
select 47523,'RVSN' from dual union all
select 47524,'RVSN' from dual union all
select 47525,'RVSN' from dual union all
select 47526,'RVSN' from dual union all
select 47527,'RVSN' from dual union all
select 47528,'AFSC' from dual union all
select 47529,'RVSN' from dual union all
select 4753,'USA' from dual union all
select 47530,'RVSN' from dual union all
select 47531,'NASA' from dual union all
select 47532,'RVSN' from dual union all
select 47533,'RVSN' from dual union all
select 47534,'10ADS' from dual union all
select 47535,'RVSN' from dual union all
select 47536,'RVSN' from dual union all
select 47537,'AFSC' from dual union all
select 47538,'RVSN' from dual union all
select 47539,'RVSN' from dual union all
select 4754,'SUI' from dual union all
select 47540,'RVSN' from dual union all
select 47541,'AFSC' from dual union all
select 47542,'RVSN' from dual union all
select 47543,'RVSN' from dual union all
select 47544,'RVSN' from dual union all
select 47545,'RVSN' from dual union all
select 47546,'RVSN' from dual union all
select 47547,'RVSN' from dual union all
select 47548,'RVSN' from dual union all
select 47549,'RVSN' from dual union all
select 4755,'USN' from dual union all
select 47550,'RVSN' from dual union all
select 47551,'RVSN' from dual union all
select 47552,'RVSN' from dual union all
select 47553,'RVSN' from dual union all
select 47554,'RVSN' from dual union all
select 47555,'RVSN' from dual union all
select 47556,'NASA' from dual union all
select 47557,'RVSN' from dual union all
select 47558,'RVSN' from dual union all
select 47559,'RVSN' from dual union all
select 4756,'SUI' from dual union all
select 47560,'RVSN' from dual union all
select 47561,'RVSN' from dual union all
select 47562,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 47563,'RVSN' from dual union all
select 47564,'RVSN' from dual union all
select 47565,'RVSN' from dual union all
select 47566,'NASA' from dual union all
select 47567,'RVSN' from dual union all
select 47568,'RVSN' from dual union all
select 47569,'RVSN' from dual union all
select 4757,'SUI' from dual union all
select 47570,'AFSC' from dual union all
select 47571,'RVSN' from dual union all
select 47572,'RVSN' from dual union all
select 47573,'RVSN' from dual union all
select 47574,'RVSN' from dual union all
select 47575,'RVSN' from dual union all
select 47576,'RVSN' from dual union all
select 47577,'RVSN' from dual union all
select 47578,'NASA' from dual union all
select 47579,'RVSN' from dual union all
select 4758,'SUI' from dual union all
select 47580,'RVSN' from dual union all
select 47581,'RVSN' from dual union all
select 47582,'AFSC' from dual union all
select 47583,'RVSN' from dual union all
select 47584,'RVSN' from dual union all
select 47585,'NASA' from dual union all
select 47586,'RVSN' from dual union all
select 47587,'RVSN' from dual union all
select 47588,'ESA' from dual union all
select 47589,'RVSN' from dual union all
select 4759,'UNRV' from dual union all
select 47590,'RVSN' from dual union all
select 47591,'MRN' from dual union all
select 47592,'USAF' from dual union all
select 47593,'USAF' from dual union all
select 47594,'USAF' from dual union all
select 47595,'MRN' from dual union all
select 47596,'MRN' from dual union all
select 47597,'MRN' from dual union all
select 47598,'USAF' from dual union all
select 47599,'MRN' from dual union all
select 476,'WER485' from dual union all
select 4760,'SUI' from dual union all
select 47600,'MRN' from dual union all
select 47601,'MRN' from dual union all
select 47602,'USN' from dual union all
select 47603,'PRO' from dual union all
select 47604,'SRC' from dual union all
select 47605,'MRN' from dual union all
select 47606,'MRN' from dual union all
select 47607,'MRN' from dual union all
select 47608,'MRN' from dual union all
select 47609,'MRN' from dual union all
select 4761,'SUI' from dual union all
select 47610,'USN' from dual union all
select 47611,'MRN' from dual union all
select 47612,'MRN' from dual union all
select 47613,'MRN' from dual union all
select 47614,'MRN' from dual union all
select 47615,'MRN' from dual union all
select 47616,'MRN' from dual union all
select 47617,'MRN' from dual union all
select 47618,'MRN' from dual union all
select 47619,'MRN' from dual union all
select 4762,'SUI' from dual union all
select 47620,'MRN' from dual union all
select 47621,'MRN' from dual union all
select 47622,'MRN' from dual union all
select 47623,'MRN' from dual union all
select 47624,'AFGL' from dual union all
select 47625,'MRN' from dual union all
select 47626,'MRN' from dual union all
select 47627,'MRN' from dual union all
select 47628,'MRN' from dual union all
select 47629,'MRN' from dual union all
select 4763,'SUI' from dual union all
select 47630,'SRC' from dual union all
select 47631,'MRN' from dual union all
select 47632,'MRN' from dual union all
select 47633,'MRN' from dual union all
select 47634,'USAF' from dual union all
select 47635,'USAF' from dual union all
select 47636,'USAF' from dual union all
select 47637,'MRN' from dual union all
select 47638,'PRO' from dual union all
select 47639,'RVSN' from dual union all
select 4764,'SUI' from dual union all
select 47640,'SAST' from dual union all
select 47641,'ISRO' from dual union all
select 47642,'AFGL' from dual union all
select 47643,'AFGL' from dual union all
select 47644,'AFGL' from dual union all
select 47645,'ISRO' from dual union all
select 47646,'ISRO' from dual union all
select 47647,'ISRO' from dual union all
select 47648,'ISRO' from dual union all
select 47649,'ISRO' from dual union all
select 4765,'USN' from dual union all
select 47650,'ISRO' from dual union all
select 47651,'ISRO' from dual union all
select 47652,'ISRO' from dual;
insert into LAUNCH_AGENCY
select 47653,'ISRO' from dual union all
select 47654,'ISRO' from dual union all
select 47655,'ISRO' from dual union all
select 47656,'ISRO' from dual union all
select 47657,'ISRO' from dual union all
select 47658,'ISRO' from dual union all
select 47659,'ISRO' from dual union all
select 4766,'NRL' from dual union all
select 47660,'ISRO' from dual union all
select 47661,'ISRO' from dual union all
select 47662,'ISRO' from dual union all
select 47663,'ISRO' from dual union all
select 47664,'ISRO' from dual union all
select 47665,'ISRO' from dual union all
select 47666,'ISRO' from dual union all
select 47667,'ISRO' from dual union all
select 47668,'ISRO' from dual union all
select 47669,'ISRO' from dual union all
select 4767,'MVS' from dual union all
select 47670,'ISRO' from dual union all
select 47671,'ISRO' from dual union all
select 47672,'ISRO' from dual union all
select 47673,'ISRO' from dual union all
select 47674,'ISRO' from dual union all
select 47675,'ISRO' from dual union all
select 47676,'ISRO' from dual union all
select 47677,'ISRO' from dual union all
select 47678,'ISRO' from dual union all
select 47679,'ISRO' from dual union all
select 4768,'BRL' from dual union all
select 47680,'ISRO' from dual union all
select 47681,'ISRO' from dual union all
select 47682,'ISRO' from dual union all
select 47683,'ISRO' from dual union all
select 47684,'ISRO' from dual union all
select 47685,'ISRO' from dual union all
select 47686,'ISRO' from dual union all
select 47687,'ISRO' from dual union all
select 47688,'ISRO' from dual union all
select 47689,'ISRO' from dual union all
select 4769,'UNRV' from dual union all
select 47690,'ISRO' from dual union all
select 47691,'ISRO' from dual union all
select 47692,'ISRO' from dual union all
select 47693,'ISRO' from dual union all
select 47694,'ISRO' from dual union all
select 47695,'ISRO' from dual union all
select 47696,'ISRO' from dual union all
select 47697,'ISRO' from dual union all
select 47698,'ISRO' from dual union all
select 47699,'ISRO' from dual union all
select 477,'WER500' from dual union all
select 4770,'AN' from dual union all
select 47700,'ISRO' from dual union all
select 47701,'ISRO' from dual union all
select 47702,'ISRO' from dual union all
select 47703,'ISRO' from dual union all
select 47704,'ISRO' from dual union all
select 47705,'ISRO' from dual union all
select 47706,'ISRO' from dual union all
select 47707,'ISRO' from dual union all
select 47708,'ISRO' from dual union all
select 47709,'ISRO' from dual union all
select 4771,'USA' from dual union all
select 47710,'ISRO' from dual union all
select 47711,'ISRO' from dual union all
select 47712,'ISRO' from dual union all
select 47713,'ISRO' from dual union all
select 47714,'ISRO' from dual union all
select 47715,'ISRO' from dual union all
select 47716,'ISRO' from dual union all
select 47717,'ISRO' from dual union all
select 47718,'ISRO' from dual union all
select 47719,'ISRO' from dual union all
select 4772,'USAF' from dual union all
select 47720,'ISRO' from dual union all
select 47721,'ISRO' from dual union all
select 47722,'AFGL' from dual union all
select 47723,'AFGL' from dual union all
select 47724,'ISRO' from dual union all
select 47725,'ISRO' from dual union all
select 47726,'ISRO' from dual union all
select 47727,'ISRO' from dual union all
select 47728,'ISRO' from dual union all
select 47729,'ISRO' from dual union all
select 4773,'USN' from dual union all
select 47730,'ISRO' from dual union all
select 47731,'ISRO' from dual union all
select 47732,'ISRO' from dual union all
select 47733,'ISRO' from dual union all
select 47734,'ISRO' from dual union all
select 47735,'ISRO' from dual union all
select 47736,'ISRO' from dual union all
select 47737,'ISRO' from dual union all
select 47738,'ISRO' from dual union all
select 47739,'ISRO' from dual union all
select 4774,'ABMA' from dual union all
select 4774,'USAF' from dual union all
select 47740,'ISRO' from dual union all
select 47741,'ISRO' from dual;
insert into LAUNCH_AGENCY
select 47742,'ISRO' from dual union all
select 47743,'ISRO' from dual union all
select 47744,'ISRO' from dual union all
select 47745,'ISRO' from dual union all
select 47746,'ISRO' from dual union all
select 47747,'ISRO' from dual union all
select 47748,'ISRO' from dual union all
select 47749,'ISRO' from dual union all
select 4775,'USAF' from dual union all
select 47750,'ISRO' from dual union all
select 47751,'ISRO' from dual union all
select 47752,'ISRO' from dual union all
select 47753,'ISRO' from dual union all
select 47754,'ISRO' from dual union all
select 47755,'ISRO' from dual union all
select 47756,'ISRO' from dual union all
select 47757,'ISRO' from dual union all
select 47758,'ISRO' from dual union all
select 47759,'ISRO' from dual union all
select 4776,'LRL' from dual union all
select 47760,'ISRO' from dual union all
select 47761,'ISRO' from dual union all
select 47762,'ISRO' from dual union all
select 47763,'ISRO' from dual union all
select 47764,'ISRO' from dual union all
select 47765,'ISRO' from dual union all
select 47766,'ISRO' from dual union all
select 47767,'ISRO' from dual union all
select 47768,'ISRO' from dual union all
select 47769,'ISRO' from dual union all
select 4777,'LRL' from dual union all
select 47770,'ISRO' from dual union all
select 47771,'ISRO' from dual union all
select 47772,'ISRO' from dual union all
select 47773,'ISRO' from dual union all
select 47774,'ISRO' from dual union all
select 47775,'ISRO' from dual union all
select 47776,'ISRO' from dual union all
select 47777,'ISRO' from dual union all
select 47778,'ISRO' from dual union all
select 47779,'ISRO' from dual union all
select 4778,'LRL' from dual union all
select 47780,'ISRO' from dual union all
select 47781,'ISRO' from dual union all
select 47782,'ISRO' from dual union all
select 47783,'ISRO' from dual union all
select 47784,'ISRO' from dual union all
select 47785,'ISRO' from dual union all
select 47786,'ISRO' from dual union all
select 47787,'ISRO' from dual union all
select 47788,'ISRO' from dual union all
select 47789,'ISRO' from dual union all
select 4779,'MVS' from dual union all
select 47790,'ISRO' from dual union all
select 47791,'ISRO' from dual union all
select 47792,'ISRO' from dual union all
select 47793,'ISRO' from dual union all
select 47794,'ISRO' from dual union all
select 47795,'ISRO' from dual union all
select 47796,'ISRO' from dual union all
select 47797,'ISRO' from dual union all
select 47798,'ISRO' from dual union all
select 47799,'ISRO' from dual union all
select 478,'WEHR' from dual union all
select 4780,'MVS' from dual union all
select 47800,'ISRO' from dual union all
select 47801,'ISRO' from dual union all
select 47802,'ISRO' from dual union all
select 47803,'ISRO' from dual union all
select 47804,'ISRO' from dual union all
select 47805,'ISRO' from dual union all
select 47806,'ISRO' from dual union all
select 47807,'AN' from dual union all
select 47808,'RVSN' from dual union all
select 47809,'RVSN' from dual union all
select 4781,'MVS' from dual union all
select 47810,'RVSN' from dual union all
select 47811,'RVSN' from dual union all
select 47812,'RVSN' from dual union all
select 47813,'RVSN' from dual union all
select 47814,'RVSN' from dual union all
select 47815,'RVSN' from dual union all
select 47816,'RVSN' from dual union all
select 47817,'NASA' from dual union all
select 47818,'DFVLR' from dual union all
select 47818,'IIAE' from dual union all
select 47819,'SAC' from dual union all
select 4782,'MVS' from dual union all
select 47820,'USN' from dual union all
select 47821,'NASA' from dual union all
select 47822,'AN' from dual union all
select 47823,'SAC' from dual union all
select 47824,'SRC' from dual union all
select 47825,'AN' from dual union all
select 47826,'DLR' from dual union all
select 47827,'RVSN' from dual union all
select 47828,'NASA' from dual union all
select 47829,'USN' from dual union all
select 4783,'MVS' from dual union all
select 47830,'USN' from dual;
insert into LAUNCH_AGENCY
select 47831,'RAE' from dual union all
select 47832,'RN' from dual union all
select 47833,'AN' from dual union all
select 47834,'AN' from dual union all
select 47835,'USN' from dual union all
select 47836,'USN' from dual union all
select 47837,'USN' from dual union all
select 47838,'RVSN' from dual union all
select 47839,'AN' from dual union all
select 4784,'MVS' from dual union all
select 47840,'NASA' from dual union all
select 47841,'AN' from dual union all
select 47842,'NASA' from dual union all
select 47843,'NASA' from dual union all
select 47844,'NASA' from dual union all
select 47845,'NASA' from dual union all
select 47846,'SAND' from dual union all
select 47847,'AFSC' from dual union all
select 47848,'RVSN' from dual union all
select 47849,'RVSN' from dual union all
select 4785,'MVS' from dual union all
select 47850,'NASA' from dual union all
select 47851,'SRC' from dual union all
select 47852,'US7A' from dual union all
select 47853,'US7A' from dual union all
select 47854,'US7A' from dual union all
select 47855,'US7A' from dual union all
select 47856,'US7A' from dual union all
select 47857,'US7A' from dual union all
select 47858,'RVSN' from dual union all
select 47859,'NASA' from dual union all
select 4786,'AN' from dual union all
select 47860,'RVSN' from dual union all
select 47861,'NASA' from dual union all
select 47862,'USN' from dual union all
select 47863,'RVSN' from dual union all
select 47864,'NASA' from dual union all
select 47865,'NASA' from dual union all
select 47866,'NASA' from dual union all
select 47867,'USN' from dual union all
select 47868,'CNES' from dual union all
select 47869,'AN' from dual union all
select 4787,'MVS' from dual union all
select 47870,'USN' from dual union all
select 47871,'RVSN' from dual union all
select 47872,'NASA' from dual union all
select 47873,'ISRO' from dual union all
select 47874,'RVSN' from dual union all
select 47875,'RVSN' from dual union all
select 47876,'NASA' from dual union all
select 47877,'DMA' from dual union all
select 47878,'NASA' from dual union all
select 47879,'AFSC' from dual union all
select 4788,'AN' from dual union all
select 47880,'RN' from dual union all
select 47881,'SAC' from dual union all
select 47882,'RVSN' from dual union all
select 47883,'SAC' from dual union all
select 47884,'USN' from dual union all
select 47885,'CALT' from dual union all
select 47885,'PLA2AC' from dual union all
select 47886,'ALCS' from dual union all
select 47886,'SAC' from dual union all
select 47887,'DMA' from dual union all
select 47888,'SAND' from dual union all
select 47889,'RVSN' from dual union all
select 4789,'MVS' from dual union all
select 47890,'NASA' from dual union all
select 47891,'SRC' from dual union all
select 47892,'SRC' from dual union all
select 47893,'RVSN' from dual union all
select 47894,'SRC' from dual union all
select 47895,'USN' from dual union all
select 47896,'SAND' from dual union all
select 47897,'SAC' from dual union all
select 47898,'RVSN' from dual union all
select 47899,'USN' from dual union all
select 479,'WER836' from dual union all
select 4790,'AN' from dual union all
select 47900,'USN' from dual union all
select 47901,'RVSN' from dual union all
select 47902,'RVSN' from dual union all
select 47903,'AFSC' from dual union all
select 47904,'RVSN' from dual union all
select 47905,'AFGL2' from dual union all
select 47906,'AN' from dual union all
select 47907,'AFGL2' from dual union all
select 47908,'AN' from dual union all
select 47909,'SSC' from dual union all
select 4791,'MVS' from dual union all
select 47910,'AFGL' from dual union all
select 47911,'AFGL' from dual union all
select 47912,'NASA' from dual union all
select 47913,'ISAS' from dual union all
select 47914,'CALT' from dual union all
select 47914,'PLA2AC' from dual union all
select 47915,'RVSN' from dual union all
select 47916,'SSC' from dual union all
select 47917,'AN' from dual union all
select 47918,'IAE' from dual;
insert into LAUNCH_AGENCY
select 47919,'NASDA' from dual union all
select 4792,'USA' from dual union all
select 47920,'USN' from dual union all
select 47921,'RVSN' from dual union all
select 47922,'AFSC' from dual union all
select 47923,'RN' from dual union all
select 47924,'CALT' from dual union all
select 47924,'PLA2AC' from dual union all
select 47925,'NASA' from dual union all
select 47926,'RVSN' from dual union all
select 47927,'ISAS' from dual union all
select 47928,'RVSN' from dual union all
select 47929,'ISAS' from dual union all
select 4793,'USN' from dual union all
select 47930,'NASA' from dual union all
select 47931,'NASA' from dual union all
select 47932,'NASA' from dual union all
select 47933,'AFSC' from dual union all
select 47934,'NASA' from dual union all
select 47935,'SSC' from dual union all
select 47936,'ALCS' from dual union all
select 47936,'SAC' from dual union all
select 47937,'RVSN' from dual union all
select 47938,'USAF' from dual union all
select 47939,'AN' from dual union all
select 4794,'MVS' from dual union all
select 47940,'USN' from dual union all
select 47941,'NASA' from dual union all
select 47942,'SAC' from dual union all
select 47943,'NASA' from dual union all
select 47944,'SRC' from dual union all
select 47945,'DFVLR' from dual union all
select 47946,'SAND' from dual union all
select 47947,'RVSN' from dual union all
select 47948,'NASA' from dual union all
select 47949,'AN' from dual union all
select 4795,'USA' from dual union all
select 47950,'AN' from dual union all
select 47951,'AFGL2' from dual union all
select 47952,'NASA' from dual union all
select 47953,'NASA' from dual union all
select 47954,'NASA' from dual union all
select 47955,'AN' from dual union all
select 47956,'DLR' from dual union all
select 47957,'NASA' from dual union all
select 47958,'RVSN' from dual union all
select 47959,'USN' from dual union all
select 4796,'USAF' from dual union all
select 47960,'AN' from dual union all
select 47961,'DLR' from dual union all
select 47962,'NASA' from dual union all
select 47963,'NASA' from dual union all
select 47964,'NASA' from dual union all
select 47965,'NASA' from dual union all
select 47966,'NASA' from dual union all
select 47967,'NASA' from dual union all
select 47968,'RN' from dual union all
select 47969,'RVSN' from dual union all
select 4797,'USA' from dual union all
select 47970,'NASA' from dual union all
select 47971,'RVSN' from dual union all
select 47972,'ISRO' from dual union all
select 47973,'NASA' from dual union all
select 47974,'CALT' from dual union all
select 47974,'PLA2AC' from dual union all
select 47975,'PRO' from dual union all
select 47976,'NRCC' from dual union all
select 47977,'NASA' from dual union all
select 47978,'ALCS' from dual union all
select 47978,'SAC' from dual union all
select 47979,'USN' from dual union all
select 4798,'USA' from dual union all
select 47980,'RVSN' from dual union all
select 47981,'NASA' from dual union all
select 47982,'NRCC' from dual union all
select 47983,'NASA' from dual union all
select 47984,'AN' from dual union all
select 47985,'AFSC' from dual union all
select 47986,'AN' from dual union all
select 47987,'RVSN' from dual union all
select 47988,'AN' from dual union all
select 47989,'AN' from dual union all
select 4799,'USAF' from dual union all
select 47990,'CNES' from dual union all
select 47991,'RVSN' from dual union all
select 47992,'ISRO' from dual union all
select 47993,'ISRO' from dual union all
select 47994,'RVSN' from dual union all
select 47995,'ISAS' from dual union all
select 47996,'9FA3' from dual union all
select 47997,'9FA3' from dual union all
select 47998,'RVSN' from dual union all
select 47999,'RVSN' from dual union all
select 48,'WER836-3' from dual union all
select 480,'WER836' from dual union all
select 4800,'USAF' from dual union all
select 48000,'RVSN' from dual union all
select 48001,'RVSN' from dual union all
select 48002,'RVSN' from dual union all
select 48003,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 48004,'RVSN' from dual union all
select 48005,'RVSN' from dual union all
select 48006,'RVSN' from dual union all
select 48007,'RVSN' from dual union all
select 48008,'RVSN' from dual union all
select 48009,'RVSN' from dual union all
select 4801,'USAF' from dual union all
select 48010,'RVSN' from dual union all
select 48011,'RVSN' from dual union all
select 48012,'RVSN' from dual union all
select 48013,'RVSN' from dual union all
select 48014,'RVSN' from dual union all
select 48015,'RVSN' from dual union all
select 48016,'RVSN' from dual union all
select 48017,'RVSN' from dual union all
select 48018,'RVSN' from dual union all
select 48019,'RVSN' from dual union all
select 4802,'USAF' from dual union all
select 48020,'CALT' from dual union all
select 48020,'PLA2AC' from dual union all
select 48021,'CALT' from dual union all
select 48021,'PLA2AC' from dual union all
select 48022,'9FA3' from dual union all
select 48023,'AN' from dual union all
select 48023,'ISRO' from dual union all
select 48024,'AN' from dual union all
select 48024,'ISRO' from dual union all
select 48025,'UKMO' from dual union all
select 48026,'UKMO' from dual union all
select 48027,'UKMO' from dual union all
select 48028,'UKMO' from dual union all
select 48029,'DFVLR' from dual union all
select 48029,'IIAE' from dual union all
select 4803,'PARD' from dual union all
select 48030,'RVSN' from dual union all
select 48031,'IIAE' from dual union all
select 48032,'RVSN' from dual union all
select 48033,'RVSN' from dual union all
select 48034,'MVS' from dual union all
select 48035,'MVS' from dual union all
select 48036,'MVS' from dual union all
select 48037,'MVS' from dual union all
select 48038,'RVSN' from dual union all
select 48039,'RVSN' from dual union all
select 4804,'USAF' from dual union all
select 48040,'RVSN' from dual union all
select 48041,'RVSN' from dual union all
select 48042,'RVSN' from dual union all
select 48043,'CALT' from dual union all
select 48043,'PLA2AC' from dual union all
select 48044,'NASA' from dual union all
select 48045,'ISAS' from dual union all
select 48046,'NASA' from dual union all
select 48047,'AFSC' from dual union all
select 48048,'ISAS' from dual union all
select 48049,'AN' from dual union all
select 4805,'6555A' from dual union all
select 48050,'ISAS' from dual union all
select 48051,'USN' from dual union all
select 48052,'AFSC' from dual union all
select 48053,'SRC' from dual union all
select 48054,'NASA' from dual union all
select 48055,'NASDA' from dual union all
select 48056,'SSC' from dual union all
select 48057,'SRC' from dual union all
select 48058,'SSC' from dual union all
select 48059,'SSC' from dual union all
select 4806,'USAF' from dual union all
select 48060,'NASA' from dual union all
select 48061,'AFGL2' from dual union all
select 48062,'ALCS' from dual union all
select 48062,'SAC' from dual union all
select 48063,'SAND' from dual union all
select 48064,'CNES' from dual union all
select 48065,'ISAS' from dual union all
select 48066,'SAC' from dual union all
select 48067,'AN' from dual union all
select 48068,'RAE' from dual union all
select 48069,'AFSC' from dual union all
select 4807,'LERC' from dual union all
select 48070,'USN' from dual union all
select 48071,'USN' from dual union all
select 48072,'USN' from dual union all
select 48073,'USN' from dual union all
select 48074,'NASA' from dual union all
select 48075,'RN' from dual union all
select 48076,'DFVLR' from dual union all
select 48077,'US7A' from dual union all
select 48078,'US7A' from dual union all
select 48079,'US7A' from dual union all
select 4808,'USAF' from dual union all
select 48080,'AN' from dual union all
select 48081,'NASA' from dual union all
select 48082,'NRCC' from dual union all
select 48083,'NASA' from dual union all
select 48084,'NASA' from dual union all
select 48085,'AFGL' from dual union all
select 48086,'AFGL' from dual union all
select 48087,'AFGL' from dual union all
select 48088,'NASA' from dual;
insert into LAUNCH_AGENCY
select 48089,'AFGL' from dual union all
select 4809,'USAF' from dual union all
select 48090,'NASA' from dual union all
select 48091,'AFGL2' from dual union all
select 48092,'NASA' from dual union all
select 48093,'AFGL2' from dual union all
select 48094,'NASA' from dual union all
select 48095,'BAC' from dual union all
select 48096,'NASA' from dual union all
select 48097,'USN' from dual union all
select 48098,'USN' from dual union all
select 48099,'USN' from dual union all
select 481,'WER500' from dual union all
select 4810,'USN' from dual union all
select 48100,'USN' from dual union all
select 48101,'AN' from dual union all
select 48102,'AN' from dual union all
select 48103,'DFVLR' from dual union all
select 48104,'AN' from dual union all
select 48105,'NASA' from dual union all
select 48106,'AN' from dual union all
select 48107,'DLR' from dual union all
select 48108,'DMA' from dual union all
select 48109,'RVSN' from dual union all
select 4811,'USN' from dual union all
select 48110,'MRN' from dual union all
select 48111,'MRN' from dual union all
select 48112,'MRN' from dual union all
select 48113,'MRN' from dual union all
select 48114,'MRN' from dual union all
select 48115,'MRN' from dual union all
select 48116,'MRN' from dual union all
select 48117,'MRN' from dual union all
select 48118,'MRN' from dual union all
select 48119,'MRN' from dual union all
select 4812,'USN' from dual union all
select 48120,'MRN' from dual union all
select 48121,'MRN' from dual union all
select 48122,'MRN' from dual union all
select 48123,'MRN' from dual union all
select 48124,'MRN' from dual union all
select 48125,'MRN' from dual union all
select 48126,'MRN' from dual union all
select 48127,'MRN' from dual union all
select 48128,'MRN' from dual union all
select 48129,'MRN' from dual union all
select 4813,'USAF' from dual union all
select 48130,'MRN' from dual union all
select 48131,'MRN' from dual union all
select 48132,'MRN' from dual union all
select 48133,'MRN' from dual union all
select 48134,'MRN' from dual union all
select 48135,'MRN' from dual union all
select 48136,'MRN' from dual union all
select 48137,'MRN' from dual union all
select 48138,'MRN' from dual union all
select 48139,'MRN' from dual union all
select 4814,'USN' from dual union all
select 48140,'MRN' from dual union all
select 48141,'MRN' from dual union all
select 48142,'MRN' from dual union all
select 48143,'MRN' from dual union all
select 48144,'MRN' from dual union all
select 48145,'MRN' from dual union all
select 48146,'MRN' from dual union all
select 48147,'MRN' from dual union all
select 48148,'MRN' from dual union all
select 48149,'MRN' from dual union all
select 4815,'USAF' from dual union all
select 48150,'MRN' from dual union all
select 48151,'MRN' from dual union all
select 48152,'MRN' from dual union all
select 48153,'MRN' from dual union all
select 48154,'MRN' from dual union all
select 48155,'MRN' from dual union all
select 48156,'NASDA' from dual union all
select 48157,'NASDA' from dual union all
select 48158,'MRN' from dual union all
select 48159,'MRN' from dual union all
select 4816,'USAF' from dual union all
select 48160,'NASA' from dual union all
select 48161,'MRN' from dual union all
select 48162,'MRN' from dual union all
select 48163,'MRN' from dual union all
select 48164,'MRN' from dual union all
select 48165,'MRN' from dual union all
select 48166,'MRN' from dual union all
select 48167,'MRN' from dual union all
select 48168,'MRN' from dual union all
select 48169,'MRN' from dual union all
select 4817,'USAF' from dual union all
select 48170,'MRN' from dual union all
select 48171,'MRN' from dual union all
select 48172,'MRN' from dual union all
select 48173,'MRN' from dual union all
select 48174,'MRN' from dual union all
select 48175,'MRN' from dual union all
select 48176,'AN' from dual union all
select 48177,'MRN' from dual union all
select 48178,'MRN' from dual;
insert into LAUNCH_AGENCY
select 48179,'MRN' from dual union all
select 4818,'USAF' from dual union all
select 48180,'MRN' from dual union all
select 48181,'MRN' from dual union all
select 48182,'MRN' from dual union all
select 48183,'MRN' from dual union all
select 48184,'MRN' from dual union all
select 48185,'MRN' from dual union all
select 48186,'MRN' from dual union all
select 48187,'MRN' from dual union all
select 48188,'MRN' from dual union all
select 48189,'MRN' from dual union all
select 4819,'SUI' from dual union all
select 48190,'MRN' from dual union all
select 48191,'MRN' from dual union all
select 48192,'MRN' from dual union all
select 48193,'MRN' from dual union all
select 48194,'MRN' from dual union all
select 48195,'MRN' from dual union all
select 48196,'MRN' from dual union all
select 48197,'MRN' from dual union all
select 48198,'MRN' from dual union all
select 48199,'MRN' from dual union all
select 482,'WER500' from dual union all
select 4820,'BRL' from dual union all
select 48200,'MRN' from dual union all
select 48201,'MRN' from dual union all
select 48202,'MRN' from dual union all
select 48203,'MRN' from dual union all
select 48204,'MRN' from dual union all
select 48205,'MRN' from dual union all
select 48206,'MRN' from dual union all
select 48207,'MRN' from dual union all
select 48208,'MRN' from dual union all
select 48209,'MRN' from dual union all
select 4821,'PARD' from dual union all
select 48210,'MRN' from dual union all
select 48211,'MRN' from dual union all
select 48212,'MRN' from dual union all
select 48213,'MRN' from dual union all
select 48214,'MRN' from dual union all
select 48215,'JMA' from dual union all
select 48216,'MRN' from dual union all
select 48217,'MRN' from dual union all
select 48218,'MRN' from dual union all
select 48219,'MRN' from dual union all
select 4822,'PARD' from dual union all
select 48220,'MRN' from dual union all
select 48221,'MRN' from dual union all
select 48222,'MRN' from dual union all
select 48223,'MRN' from dual union all
select 48224,'MRN' from dual union all
select 48225,'MRN' from dual union all
select 48226,'MRN' from dual union all
select 48227,'MRN' from dual union all
select 48228,'MRN' from dual union all
select 48229,'MRN' from dual union all
select 4823,'SU' from dual union all
select 48230,'MRN' from dual union all
select 48231,'MRN' from dual union all
select 48232,'MRN' from dual union all
select 48233,'MRN' from dual union all
select 48234,'MRN' from dual union all
select 48235,'NASA' from dual union all
select 48236,'MRN' from dual union all
select 48237,'MRN' from dual union all
select 48238,'MRN' from dual union all
select 48239,'MRN' from dual union all
select 4824,'USAF' from dual union all
select 48240,'MRN' from dual union all
select 48241,'MRN' from dual union all
select 48242,'MRN' from dual union all
select 48243,'MRN' from dual union all
select 48244,'MRN' from dual union all
select 48245,'MRN' from dual union all
select 48246,'MRN' from dual union all
select 48247,'MRN' from dual union all
select 48248,'MRN' from dual union all
select 48249,'MRN' from dual union all
select 4825,'USAF' from dual union all
select 48250,'NASA' from dual union all
select 48251,'NASA' from dual union all
select 48252,'MRN' from dual union all
select 48253,'MRN' from dual union all
select 48254,'MRN' from dual union all
select 48255,'MRN' from dual union all
select 48256,'MRN' from dual union all
select 48257,'MRN' from dual union all
select 48258,'MRN' from dual union all
select 48259,'MRN' from dual union all
select 4826,'LERC' from dual union all
select 48260,'MRN' from dual union all
select 48261,'MRN' from dual union all
select 48262,'MRN' from dual union all
select 48263,'MRN' from dual union all
select 48264,'MRN' from dual union all
select 48265,'MRN' from dual union all
select 48266,'MRN' from dual union all
select 48267,'MRN' from dual union all
select 48268,'MRN' from dual;
insert into LAUNCH_AGENCY
select 48269,'MRN' from dual union all
select 4827,'ISAS' from dual union all
select 48270,'MRN' from dual union all
select 48271,'MRN' from dual union all
select 48272,'MRN' from dual union all
select 48273,'MRN' from dual union all
select 48274,'MRN' from dual union all
select 48275,'MRN' from dual union all
select 48276,'MRN' from dual union all
select 48277,'MRN' from dual union all
select 48278,'MRN' from dual union all
select 48279,'JMA' from dual union all
select 4828,'ISAS' from dual union all
select 48280,'MRN' from dual union all
select 48281,'MRN' from dual union all
select 48282,'AN' from dual union all
select 48283,'MRN' from dual union all
select 48284,'MRN' from dual union all
select 48285,'MRN' from dual union all
select 48286,'MRN' from dual union all
select 48287,'MRN' from dual union all
select 48288,'MRN' from dual union all
select 48289,'MRN' from dual union all
select 4829,'USAF' from dual union all
select 48290,'MRN' from dual union all
select 48291,'MRN' from dual union all
select 48292,'MRN' from dual union all
select 48293,'MRN' from dual union all
select 48294,'MRN' from dual union all
select 48295,'MRN' from dual union all
select 48296,'MRN' from dual union all
select 48297,'MRN' from dual union all
select 48298,'MRN' from dual union all
select 48299,'MRN' from dual union all
select 483,'WEHR' from dual union all
select 4830,'PARD' from dual union all
select 4830,'USN' from dual union all
select 48300,'MRN' from dual union all
select 48301,'MRN' from dual union all
select 48302,'MRN' from dual union all
select 48303,'MRN' from dual union all
select 48304,'MRN' from dual union all
select 48305,'MRN' from dual union all
select 48306,'MRN' from dual union all
select 48307,'MRN' from dual union all
select 48308,'MRN' from dual union all
select 48309,'MRN' from dual union all
select 4831,'USAF' from dual union all
select 48310,'MRN' from dual union all
select 48311,'JMA' from dual union all
select 48312,'MRN' from dual union all
select 48313,'MRN' from dual union all
select 48314,'MRN' from dual union all
select 48315,'MRN' from dual union all
select 48316,'MRN' from dual union all
select 48317,'MRN' from dual union all
select 48318,'MRN' from dual union all
select 48319,'MRN' from dual union all
select 4832,'USAF' from dual union all
select 48320,'MRN' from dual union all
select 48321,'MRN' from dual union all
select 48322,'MRN' from dual union all
select 48323,'MRN' from dual union all
select 48324,'MRN' from dual union all
select 48325,'MRN' from dual union all
select 48326,'MRN' from dual union all
select 48327,'MRN' from dual union all
select 48328,'MRN' from dual union all
select 48329,'MRN' from dual union all
select 4833,'PARD' from dual union all
select 4833,'UMD' from dual union all
select 48330,'MRN' from dual union all
select 48331,'MRN' from dual union all
select 48332,'MRN' from dual union all
select 48333,'MRN' from dual union all
select 48334,'MRN' from dual union all
select 48335,'MRN' from dual union all
select 48336,'MRN' from dual union all
select 48337,'MRN' from dual union all
select 48338,'NASA' from dual union all
select 48339,'MRN' from dual union all
select 4834,'PARD' from dual union all
select 4834,'UMD' from dual union all
select 48340,'MRN' from dual union all
select 48341,'MRN' from dual union all
select 48342,'MRN' from dual union all
select 48343,'MRN' from dual union all
select 48344,'MRN' from dual union all
select 48345,'MRN' from dual union all
select 48346,'MRN' from dual union all
select 48347,'MRN' from dual union all
select 48348,'MRN' from dual union all
select 48349,'MRN' from dual union all
select 4835,'PARD' from dual union all
select 4835,'UMD' from dual union all
select 48350,'MRN' from dual union all
select 48351,'MRN' from dual union all
select 48352,'MRN' from dual union all
select 48353,'MRN' from dual union all
select 48354,'MRN' from dual;
insert into LAUNCH_AGENCY
select 48355,'MRN' from dual union all
select 48356,'MRN' from dual union all
select 48357,'MRN' from dual union all
select 48358,'MRN' from dual union all
select 48359,'MRN' from dual union all
select 4836,'USAF' from dual union all
select 48360,'MRN' from dual union all
select 48361,'JMA' from dual union all
select 48362,'MRN' from dual union all
select 48363,'MRN' from dual union all
select 48364,'MRN' from dual union all
select 48365,'MRN' from dual union all
select 48366,'MRN' from dual union all
select 48367,'MRN' from dual union all
select 48368,'MRN' from dual union all
select 48369,'MRN' from dual union all
select 4837,'USN' from dual union all
select 48370,'MRN' from dual union all
select 48371,'MRN' from dual union all
select 48372,'MRN' from dual union all
select 48373,'MRN' from dual union all
select 48374,'MRN' from dual union all
select 48375,'MRN' from dual union all
select 48376,'MRN' from dual union all
select 48377,'MRN' from dual union all
select 48378,'MRN' from dual union all
select 48379,'MRN' from dual union all
select 4838,'USAF' from dual union all
select 48380,'MRN' from dual union all
select 48381,'MRN' from dual union all
select 48382,'NASA' from dual union all
select 48383,'NASA' from dual union all
select 48384,'NASA' from dual union all
select 48385,'NASA' from dual union all
select 48386,'MRN' from dual union all
select 48387,'NASA' from dual union all
select 48388,'MRN' from dual union all
select 48389,'MRN' from dual union all
select 4839,'PARD' from dual union all
select 4839,'UMD' from dual union all
select 48390,'MRN' from dual union all
select 48391,'MRN' from dual union all
select 48392,'MRN' from dual union all
select 48393,'MRN' from dual union all
select 48394,'MRN' from dual union all
select 48395,'MRN' from dual union all
select 48396,'JMA' from dual union all
select 48397,'MRN' from dual union all
select 48398,'MRN' from dual union all
select 48399,'MRN' from dual union all
select 484,'WEHR' from dual union all
select 4840,'PARD' from dual union all
select 4840,'UMD' from dual union all
select 48400,'MRN' from dual union all
select 48401,'MRN' from dual union all
select 48402,'MRN' from dual union all
select 48403,'MRN' from dual union all
select 48404,'MRN' from dual union all
select 48405,'MRN' from dual union all
select 48406,'MRN' from dual union all
select 48407,'MRN' from dual union all
select 48408,'MRN' from dual union all
select 48409,'MRN' from dual union all
select 4841,'ISAS' from dual union all
select 48410,'MRN' from dual union all
select 48411,'MRN' from dual union all
select 48412,'MRN' from dual union all
select 48413,'MRN' from dual union all
select 48414,'MRN' from dual union all
select 48415,'MRN' from dual union all
select 48416,'MRN' from dual union all
select 48417,'MRN' from dual union all
select 48418,'MRN' from dual union all
select 48419,'MRN' from dual union all
select 4842,'6555A' from dual union all
select 48420,'MRN' from dual union all
select 48421,'MRN' from dual union all
select 48422,'MRN' from dual union all
select 48423,'MRN' from dual union all
select 48424,'MRN' from dual union all
select 48425,'MRN' from dual union all
select 48426,'MRN' from dual union all
select 48427,'MRN' from dual union all
select 48428,'MRN' from dual union all
select 48429,'MRN' from dual union all
select 4843,'USAF' from dual union all
select 48430,'MRN' from dual union all
select 48431,'MRN' from dual union all
select 48432,'MRN' from dual union all
select 48433,'MRN' from dual union all
select 48434,'NASA' from dual union all
select 48435,'NASA' from dual union all
select 48436,'NASA' from dual union all
select 48437,'MRN' from dual union all
select 48438,'NASA' from dual union all
select 48439,'MRN' from dual union all
select 4844,'MVS' from dual union all
select 48440,'NASA' from dual union all
select 48441,'MRN' from dual union all
select 48442,'NASA' from dual;
insert into LAUNCH_AGENCY
select 48443,'GMS' from dual union all
select 48444,'MRN' from dual union all
select 48445,'MRN' from dual union all
select 48446,'MRN' from dual union all
select 48447,'MRN' from dual union all
select 48448,'MRN' from dual union all
select 48449,'MRN' from dual union all
select 4845,'USN' from dual union all
select 48450,'MRN' from dual union all
select 48451,'MRN' from dual union all
select 48452,'MRN' from dual union all
select 48453,'MRN' from dual union all
select 48454,'NASA' from dual union all
select 48455,'MRN' from dual union all
select 48456,'MRN' from dual union all
select 48457,'MRN' from dual union all
select 48458,'MRN' from dual union all
select 48459,'MRN' from dual union all
select 4846,'ABMA' from dual union all
select 4846,'USAF' from dual union all
select 48460,'MRN' from dual union all
select 48461,'MRN' from dual union all
select 48462,'MRN' from dual union all
select 48463,'MRN' from dual union all
select 48464,'MRN' from dual union all
select 48465,'JMA' from dual union all
select 48466,'GMS' from dual union all
select 48467,'GMS' from dual union all
select 48468,'MRN' from dual union all
select 48469,'MRN' from dual union all
select 4847,'PARD' from dual union all
select 48470,'MRN' from dual union all
select 48471,'MRN' from dual union all
select 48472,'MRN' from dual union all
select 48473,'MRN' from dual union all
select 48474,'MRN' from dual union all
select 48475,'MRN' from dual union all
select 48476,'MRN' from dual union all
select 48477,'MRN' from dual union all
select 48478,'MRN' from dual union all
select 48479,'MRN' from dual union all
select 4848,'USN' from dual union all
select 48480,'MRN' from dual union all
select 48481,'MRN' from dual union all
select 48482,'MRN' from dual union all
select 48483,'MRN' from dual union all
select 48484,'MRN' from dual union all
select 48485,'AN' from dual union all
select 48486,'MRN' from dual union all
select 48487,'MRN' from dual union all
select 48488,'MRN' from dual union all
select 48489,'JMA' from dual union all
select 4849,'PRO' from dual union all
select 48490,'MRN' from dual union all
select 48491,'MRN' from dual union all
select 48492,'AN' from dual union all
select 48493,'MRN' from dual union all
select 48494,'MRN' from dual union all
select 48495,'MRN' from dual union all
select 48496,'MRN' from dual union all
select 48497,'MRN' from dual union all
select 48498,'AN' from dual union all
select 48499,'MRN' from dual union all
select 485,'ORD' from dual union all
select 4850,'USAF' from dual union all
select 48500,'AN' from dual union all
select 48501,'MRN' from dual union all
select 48502,'MRN' from dual union all
select 48503,'MRN' from dual union all
select 48504,'MRN' from dual union all
select 48505,'MRN' from dual union all
select 48506,'MRN' from dual union all
select 48507,'MRN' from dual union all
select 48508,'MRN' from dual union all
select 48509,'MRN' from dual union all
select 4851,'USAF' from dual union all
select 48510,'MRN' from dual union all
select 48511,'MRN' from dual union all
select 48512,'MRN' from dual union all
select 48513,'GMS' from dual union all
select 48514,'MRN' from dual union all
select 48515,'MRN' from dual union all
select 48516,'MRN' from dual union all
select 48517,'MRN' from dual union all
select 48518,'MRN' from dual union all
select 48519,'MRN' from dual union all
select 4852,'USAF' from dual union all
select 48520,'MRN' from dual union all
select 48521,'MRN' from dual union all
select 48522,'MRN' from dual union all
select 48523,'MRN' from dual union all
select 48524,'MRN' from dual union all
select 48525,'MRN' from dual union all
select 48526,'MRN' from dual union all
select 48527,'MRN' from dual union all
select 48528,'MRN' from dual union all
select 48529,'MRN' from dual union all
select 4853,'USAF' from dual union all
select 48530,'MRN' from dual union all
select 48531,'MRN' from dual;
insert into LAUNCH_AGENCY
select 48532,'MRN' from dual union all
select 48533,'MRN' from dual union all
select 48534,'MRN' from dual union all
select 48535,'MRN' from dual union all
select 48536,'MRN' from dual union all
select 48537,'MRN' from dual union all
select 48538,'MRN' from dual union all
select 48539,'MRN' from dual union all
select 4854,'PARD' from dual union all
select 48540,'GMS' from dual union all
select 48541,'MRN' from dual union all
select 48542,'MRN' from dual union all
select 48543,'MRN' from dual union all
select 48544,'MRN' from dual union all
select 48545,'MRN' from dual union all
select 48546,'MRN' from dual union all
select 48547,'MRN' from dual union all
select 48548,'MRN' from dual union all
select 48549,'NASA' from dual union all
select 4855,'USAF' from dual union all
select 48550,'MRN' from dual union all
select 48551,'JMA' from dual union all
select 48552,'MRN' from dual union all
select 48553,'MRN' from dual union all
select 48554,'MRN' from dual union all
select 48555,'MRN' from dual union all
select 48556,'MRN' from dual union all
select 48557,'MRN' from dual union all
select 48558,'MRN' from dual union all
select 48559,'MRN' from dual union all
select 4856,'USAF' from dual union all
select 48560,'MRN' from dual union all
select 48561,'MRN' from dual union all
select 48562,'MRN' from dual union all
select 48563,'MRN' from dual union all
select 48564,'MRN' from dual union all
select 48565,'MRN' from dual union all
select 48566,'MRN' from dual union all
select 48567,'MRN' from dual union all
select 48568,'MRN' from dual union all
select 48569,'JMA' from dual union all
select 4857,'USAF' from dual union all
select 48570,'GMS' from dual union all
select 48571,'MRN' from dual union all
select 48572,'MRN' from dual union all
select 48573,'AN' from dual union all
select 48574,'MRN' from dual union all
select 48575,'MRN' from dual union all
select 48576,'MRN' from dual union all
select 48577,'MRN' from dual union all
select 48578,'MRN' from dual union all
select 48579,'MRN' from dual union all
select 4858,'USAF' from dual union all
select 48580,'MRN' from dual union all
select 48581,'MRN' from dual union all
select 48582,'MRN' from dual union all
select 48583,'MRN' from dual union all
select 48584,'MRN' from dual union all
select 48585,'MRN' from dual union all
select 48586,'MRN' from dual union all
select 48587,'MRN' from dual union all
select 48588,'MRN' from dual union all
select 48589,'MRN' from dual union all
select 4859,'USAF' from dual union all
select 48590,'MRN' from dual union all
select 48591,'MRN' from dual union all
select 48592,'MRN' from dual union all
select 48593,'MRN' from dual union all
select 48594,'JMA' from dual union all
select 48595,'MRN' from dual union all
select 48596,'MRN' from dual union all
select 48597,'MRN' from dual union all
select 48598,'GMS' from dual union all
select 48599,'MRN' from dual union all
select 486,'ORD' from dual union all
select 4860,'PARD' from dual union all
select 48600,'MRN' from dual union all
select 48601,'MRN' from dual union all
select 48602,'MRN' from dual union all
select 48603,'MRN' from dual union all
select 48604,'MRN' from dual union all
select 48605,'MRN' from dual union all
select 48606,'MRN' from dual union all
select 48607,'MRN' from dual union all
select 48608,'GMS' from dual union all
select 48609,'MRN' from dual union all
select 4861,'PARD' from dual union all
select 48610,'NASA' from dual union all
select 48611,'MRN' from dual union all
select 48612,'MRN' from dual union all
select 48613,'MRN' from dual union all
select 48614,'MRN' from dual union all
select 48615,'MRN' from dual union all
select 48616,'GMS' from dual union all
select 48617,'AN' from dual union all
select 48618,'GMS' from dual union all
select 48619,'MRN' from dual union all
select 4862,'SUI' from dual union all
select 48620,'MRN' from dual union all
select 48621,'GMS' from dual;
insert into LAUNCH_AGENCY
select 48622,'JMA' from dual union all
select 48623,'GMS' from dual union all
select 48624,'MRN' from dual union all
select 48625,'MRN' from dual union all
select 48626,'MRN' from dual union all
select 48627,'MRN' from dual union all
select 48628,'MRN' from dual union all
select 48629,'MRN' from dual union all
select 4863,'SUI' from dual union all
select 48630,'MRN' from dual union all
select 48631,'MRN' from dual union all
select 48632,'MRN' from dual union all
select 48633,'MRN' from dual union all
select 48634,'MRN' from dual union all
select 48635,'MRN' from dual union all
select 48636,'JMA' from dual union all
select 48637,'JMA' from dual union all
select 48638,'JMA' from dual union all
select 48639,'JMA' from dual union all
select 4864,'USAF' from dual union all
select 48640,'JMA' from dual union all
select 48641,'MRN' from dual union all
select 48642,'JMA' from dual union all
select 48643,'JMA' from dual union all
select 48644,'JMA' from dual union all
select 48645,'JMA' from dual union all
select 48646,'JMA' from dual union all
select 48647,'JMA' from dual union all
select 48648,'JMA' from dual union all
select 48649,'JMA' from dual union all
select 4865,'USAF' from dual union all
select 48650,'JMA' from dual union all
select 48651,'JMA' from dual union all
select 48652,'MRN' from dual union all
select 48653,'JMA' from dual union all
select 48654,'JMA' from dual union all
select 48655,'JMA' from dual union all
select 48656,'JMA' from dual union all
select 48657,'JMA' from dual union all
select 48658,'JMA' from dual union all
select 48659,'MRN' from dual union all
select 4866,'USAF' from dual union all
select 48660,'MRN' from dual union all
select 48661,'MRN' from dual union all
select 48662,'MRN' from dual union all
select 48663,'MRN' from dual union all
select 48664,'NASA' from dual union all
select 48665,'NASA' from dual union all
select 48666,'MRN' from dual union all
select 48667,'MRN' from dual union all
select 48668,'MRN' from dual union all
select 48669,'MRN' from dual union all
select 4867,'USAF' from dual union all
select 48670,'MRN' from dual union all
select 48671,'MRN' from dual union all
select 48672,'MRN' from dual union all
select 48673,'MRN' from dual union all
select 48674,'MRN' from dual union all
select 48675,'MRN' from dual union all
select 48676,'MRN' from dual union all
select 48677,'MRN' from dual union all
select 48678,'MRN' from dual union all
select 48679,'GMS' from dual union all
select 4868,'ISAS' from dual union all
select 48680,'MRN' from dual union all
select 48681,'MRN' from dual union all
select 48682,'MRN' from dual union all
select 48683,'MRN' from dual union all
select 48684,'MRN' from dual union all
select 48685,'MRN' from dual union all
select 48686,'MRN' from dual union all
select 48687,'MRN' from dual union all
select 48688,'MRN' from dual union all
select 48689,'MRN' from dual union all
select 4869,'ISAS' from dual union all
select 48690,'MRN' from dual union all
select 48691,'MRN' from dual union all
select 48692,'MRN' from dual union all
select 48693,'MRN' from dual union all
select 48694,'MRN' from dual union all
select 48695,'MRN' from dual union all
select 48696,'MRN' from dual union all
select 48697,'GMS' from dual union all
select 48698,'MRN' from dual union all
select 48699,'MRN' from dual union all
select 487,'ORD' from dual union all
select 4870,'USN' from dual union all
select 48700,'MRN' from dual union all
select 48701,'MRN' from dual union all
select 48702,'MRN' from dual union all
select 48703,'MRN' from dual union all
select 48704,'MRN' from dual union all
select 48705,'MRN' from dual union all
select 48706,'MRN' from dual union all
select 48707,'MRN' from dual union all
select 48708,'MRN' from dual union all
select 48709,'MRN' from dual union all
select 4871,'ISAS' from dual union all
select 48710,'MRN' from dual union all
select 48711,'MRN' from dual;
insert into LAUNCH_AGENCY
select 48712,'MRN' from dual union all
select 48713,'MRN' from dual union all
select 48714,'MRN' from dual union all
select 48715,'MRN' from dual union all
select 48716,'MRN' from dual union all
select 48717,'NASDA' from dual union all
select 48718,'GMS' from dual union all
select 48719,'MRN' from dual union all
select 4872,'USAF' from dual union all
select 48720,'MRN' from dual union all
select 48721,'MRN' from dual union all
select 48722,'MRN' from dual union all
select 48723,'MRN' from dual union all
select 48724,'MRN' from dual union all
select 48725,'MRN' from dual union all
select 48726,'MRN' from dual union all
select 48727,'AN' from dual union all
select 48728,'MRN' from dual union all
select 48729,'MRN' from dual union all
select 4873,'USAF' from dual union all
select 48730,'MRN' from dual union all
select 48731,'MRN' from dual union all
select 48732,'MRN' from dual union all
select 48733,'MRN' from dual union all
select 48734,'MRN' from dual union all
select 48735,'MRN' from dual union all
select 48736,'GMS' from dual union all
select 48737,'MRN' from dual union all
select 48738,'MRN' from dual union all
select 48739,'MRN' from dual union all
select 4874,'USAF' from dual union all
select 48740,'MRN' from dual union all
select 48741,'MRN' from dual union all
select 48742,'MRN' from dual union all
select 48743,'MRN' from dual union all
select 48744,'MRN' from dual union all
select 48745,'MRN' from dual union all
select 48746,'MRN' from dual union all
select 48747,'MRN' from dual union all
select 48748,'MRN' from dual union all
select 48749,'MRN' from dual union all
select 4875,'USAF' from dual union all
select 48750,'MRN' from dual union all
select 48751,'MRN' from dual union all
select 48752,'MRN' from dual union all
select 48753,'MRN' from dual union all
select 48754,'MRN' from dual union all
select 48755,'MRN' from dual union all
select 48756,'GMS' from dual union all
select 48757,'MRN' from dual union all
select 48758,'MRN' from dual union all
select 48759,'MRN' from dual union all
select 4876,'USAF' from dual union all
select 48760,'MRN' from dual union all
select 48761,'MRN' from dual union all
select 48762,'MRN' from dual union all
select 48763,'MRN' from dual union all
select 48764,'MRN' from dual union all
select 48765,'MRN' from dual union all
select 48766,'MRN' from dual union all
select 48767,'MRN' from dual union all
select 48768,'MRN' from dual union all
select 48769,'MRN' from dual union all
select 4877,'USAF' from dual union all
select 48770,'MRN' from dual union all
select 48771,'MRN' from dual union all
select 48772,'MRN' from dual union all
select 48773,'MRN' from dual union all
select 48774,'MRN' from dual union all
select 48775,'MRN' from dual union all
select 48776,'MRN' from dual union all
select 48777,'MRN' from dual union all
select 48778,'MRN' from dual union all
select 48779,'MRN' from dual union all
select 4878,'USAF' from dual union all
select 48780,'MRN' from dual union all
select 48781,'GMS' from dual union all
select 48782,'MRN' from dual union all
select 48783,'MRN' from dual union all
select 48784,'MRN' from dual union all
select 48785,'MRN' from dual union all
select 48786,'MRN' from dual union all
select 48787,'MRN' from dual union all
select 48788,'MRN' from dual union all
select 48789,'MRN' from dual union all
select 4879,'USAF' from dual union all
select 48790,'MRN' from dual union all
select 48791,'MRN' from dual union all
select 48792,'MRN' from dual union all
select 48793,'MRN' from dual union all
select 48794,'MRN' from dual union all
select 48795,'MRN' from dual union all
select 48796,'MRN' from dual union all
select 48797,'MRN' from dual union all
select 48798,'MRN' from dual union all
select 48799,'MRN' from dual union all
select 488,'ORD' from dual union all
select 4880,'USAF' from dual union all
select 48800,'MRN' from dual union all
select 48801,'MRN' from dual;
insert into LAUNCH_AGENCY
select 48802,'MRN' from dual union all
select 48803,'MRN' from dual union all
select 48804,'MRN' from dual union all
select 48805,'MRN' from dual union all
select 48806,'MRN' from dual union all
select 48807,'MRN' from dual union all
select 48808,'MRN' from dual union all
select 48809,'MRN' from dual union all
select 4881,'NASA' from dual union all
select 48810,'MRN' from dual union all
select 48811,'MRN' from dual union all
select 48812,'MRN' from dual union all
select 48813,'MRN' from dual union all
select 48814,'MRN' from dual union all
select 48815,'MRN' from dual union all
select 48816,'MRN' from dual union all
select 48817,'MRN' from dual union all
select 48818,'MRN' from dual union all
select 48819,'MRN' from dual union all
select 4882,'PARD' from dual union all
select 48820,'MRN' from dual union all
select 48821,'MRN' from dual union all
select 48822,'MRN' from dual union all
select 48823,'MRN' from dual union all
select 48824,'MRN' from dual union all
select 48825,'MRN' from dual union all
select 48826,'MRN' from dual union all
select 48827,'MRN' from dual union all
select 48828,'MRN' from dual union all
select 48829,'MRN' from dual union all
select 4883,'USAF' from dual union all
select 48830,'MRN' from dual union all
select 48831,'MRN' from dual union all
select 48832,'MRN' from dual union all
select 48833,'MRN' from dual union all
select 48834,'MRN' from dual union all
select 48835,'NRCC' from dual union all
select 48836,'MRN' from dual union all
select 48837,'AN' from dual union all
select 48838,'NRCC' from dual union all
select 48839,'MRN' from dual union all
select 4884,'USAF' from dual union all
select 48840,'MRN' from dual union all
select 48841,'MRN' from dual union all
select 48842,'MRN' from dual union all
select 48843,'AN' from dual union all
select 48844,'MRN' from dual union all
select 48845,'AN' from dual union all
select 48846,'GMS' from dual union all
select 48847,'MRN' from dual union all
select 48848,'MRN' from dual union all
select 48849,'MRN' from dual union all
select 4885,'USAF' from dual union all
select 48850,'MRN' from dual union all
select 48851,'MRN' from dual union all
select 48852,'MRN' from dual union all
select 48853,'MRN' from dual union all
select 48854,'NASA' from dual union all
select 48855,'MRN' from dual union all
select 48856,'MRN' from dual union all
select 48857,'MRN' from dual union all
select 48858,'MRN' from dual union all
select 48859,'MRN' from dual union all
select 4886,'SU' from dual union all
select 48860,'MRN' from dual union all
select 48861,'MRN' from dual union all
select 48862,'MRN' from dual union all
select 48863,'MRN' from dual union all
select 48864,'MRN' from dual union all
select 48865,'MRN' from dual union all
select 48866,'MRN' from dual union all
select 48867,'MRN' from dual union all
select 48868,'MRN' from dual union all
select 48869,'MRN' from dual union all
select 4887,'USN' from dual union all
select 48870,'MRN' from dual union all
select 48871,'MRN' from dual union all
select 48872,'MRN' from dual union all
select 48873,'MRN' from dual union all
select 48874,'MRN' from dual union all
select 48875,'MRN' from dual union all
select 48876,'AN' from dual union all
select 48877,'MRN' from dual union all
select 48878,'MRN' from dual union all
select 48879,'MRN' from dual union all
select 4888,'USAF' from dual union all
select 48880,'MRN' from dual union all
select 48881,'MRN' from dual union all
select 48882,'MRN' from dual union all
select 48883,'MRN' from dual union all
select 48884,'MRN' from dual union all
select 48885,'MRN' from dual union all
select 48886,'MRN' from dual union all
select 48887,'MRN' from dual union all
select 48888,'MRN' from dual union all
select 48889,'MRN' from dual union all
select 4889,'SMSA' from dual union all
select 48890,'MRN' from dual union all
select 48891,'MRN' from dual union all
select 48892,'MRN' from dual;
insert into LAUNCH_AGENCY
select 48893,'MRN' from dual union all
select 48894,'MRN' from dual union all
select 48895,'MRN' from dual union all
select 48896,'MRN' from dual union all
select 48897,'MRN' from dual union all
select 48898,'MRN' from dual union all
select 48899,'MRN' from dual union all
select 489,'AVKO' from dual union all
select 489,'SPOG' from dual union all
select 4890,'SMSA' from dual union all
select 48900,'MRN' from dual union all
select 48901,'MRN' from dual union all
select 48902,'MRN' from dual union all
select 48903,'MRN' from dual union all
select 48904,'AN' from dual union all
select 48905,'MRN' from dual union all
select 48906,'MRN' from dual union all
select 48907,'MRN' from dual union all
select 48908,'MRN' from dual union all
select 48909,'MRN' from dual union all
select 4891,'SMSA' from dual union all
select 48910,'MRN' from dual union all
select 48911,'MRN' from dual union all
select 48912,'MRN' from dual union all
select 48913,'MRN' from dual union all
select 48914,'NASA' from dual union all
select 48915,'MRN' from dual union all
select 48916,'MRN' from dual union all
select 48917,'AN' from dual union all
select 48918,'MRN' from dual union all
select 48919,'MRN' from dual union all
select 4892,'ARGMA' from dual union all
select 48920,'MRN' from dual union all
select 48921,'MRN' from dual union all
select 48922,'MRN' from dual union all
select 48923,'MRN' from dual union all
select 48924,'MRN' from dual union all
select 48925,'MRN' from dual union all
select 48926,'NASA' from dual union all
select 48927,'MRN' from dual union all
select 48928,'MRN' from dual union all
select 48929,'MRN' from dual union all
select 4893,'ARGMA' from dual union all
select 48930,'MRN' from dual union all
select 48931,'MRN' from dual union all
select 48932,'MRN' from dual union all
select 48933,'MRN' from dual union all
select 48934,'MRN' from dual union all
select 48935,'MRN' from dual union all
select 48936,'MRN' from dual union all
select 48937,'MRN' from dual union all
select 48938,'MRN' from dual union all
select 48939,'MRN' from dual union all
select 4894,'ARGMA' from dual union all
select 48940,'MRN' from dual union all
select 48941,'MRN' from dual union all
select 48942,'MRN' from dual union all
select 48943,'MRN' from dual union all
select 48944,'MRN' from dual union all
select 48945,'MRN' from dual union all
select 48946,'NASA' from dual union all
select 48947,'MRN' from dual union all
select 48948,'MRN' from dual union all
select 48949,'MRN' from dual union all
select 4895,'ARGMA' from dual union all
select 48950,'MRN' from dual union all
select 48951,'MRN' from dual union all
select 48952,'NASA' from dual union all
select 48953,'MRN' from dual union all
select 48954,'MRN' from dual union all
select 48955,'MRN' from dual union all
select 48956,'MRN' from dual union all
select 48957,'NASA' from dual union all
select 48958,'MRN' from dual union all
select 48959,'MRN' from dual union all
select 4896,'USAF' from dual union all
select 48960,'MRN' from dual union all
select 48961,'MRN' from dual union all
select 48962,'MRN' from dual union all
select 48963,'MRN' from dual union all
select 48964,'MRN' from dual union all
select 48965,'NASA' from dual union all
select 48966,'NASA' from dual union all
select 48967,'MRN' from dual union all
select 48968,'MRN' from dual union all
select 48969,'MRN' from dual union all
select 4897,'ARGMA' from dual union all
select 48970,'MRN' from dual union all
select 48971,'MRN' from dual union all
select 48972,'MRN' from dual union all
select 48973,'MRN' from dual union all
select 48974,'MRN' from dual union all
select 48975,'MRN' from dual union all
select 48976,'MRN' from dual union all
select 48977,'MRN' from dual union all
select 48978,'MRN' from dual union all
select 48979,'MRN' from dual union all
select 4898,'ARGMA' from dual union all
select 48980,'MRN' from dual union all
select 48981,'MRN' from dual;
insert into LAUNCH_AGENCY
select 48982,'AN' from dual union all
select 48983,'MRN' from dual union all
select 48984,'MRN' from dual union all
select 48985,'MRN' from dual union all
select 48986,'MRN' from dual union all
select 48987,'MRN' from dual union all
select 48988,'MRN' from dual union all
select 48989,'MRN' from dual union all
select 4899,'ARGMA' from dual union all
select 48990,'MRN' from dual union all
select 48991,'MRN' from dual union all
select 48992,'MRN' from dual union all
select 48993,'MRN' from dual union all
select 48994,'MRN' from dual union all
select 48995,'MRN' from dual union all
select 48996,'MRN' from dual union all
select 48997,'MRN' from dual union all
select 48998,'MRN' from dual union all
select 48999,'MRN' from dual union all
select 49,'WER485-2' from dual union all
select 490,'JPL' from dual union all
select 4900,'ARGMA' from dual union all
select 49000,'MRN' from dual union all
select 49001,'MRN' from dual union all
select 49002,'MRN' from dual union all
select 49003,'MRN' from dual union all
select 49004,'MRN' from dual union all
select 49005,'NASA' from dual union all
select 49006,'MRN' from dual union all
select 49007,'NASA' from dual union all
select 49008,'NASA' from dual union all
select 49009,'MRN' from dual union all
select 4901,'ARGMA' from dual union all
select 49010,'MRN' from dual union all
select 49011,'MRN' from dual union all
select 49012,'MRN' from dual union all
select 49013,'MRN' from dual union all
select 49014,'MRN' from dual union all
select 49015,'MRN' from dual union all
select 49016,'MRN' from dual union all
select 49017,'MRN' from dual union all
select 49018,'MRN' from dual union all
select 49019,'MRN' from dual union all
select 4902,'ARGMA' from dual union all
select 49020,'MRN' from dual union all
select 49021,'MRN' from dual union all
select 49022,'MRN' from dual union all
select 49023,'MRN' from dual union all
select 49024,'MRN' from dual union all
select 49025,'MRN' from dual union all
select 49026,'MRN' from dual union all
select 49027,'MRN' from dual union all
select 49028,'MRN' from dual union all
select 49029,'MRN' from dual union all
select 4903,'USA' from dual union all
select 49030,'MRN' from dual union all
select 49031,'NASA' from dual union all
select 49032,'MRN' from dual union all
select 49033,'MRN' from dual union all
select 49034,'MRN' from dual union all
select 49035,'MRN' from dual union all
select 49036,'MRN' from dual union all
select 49037,'MRN' from dual union all
select 49038,'MRN' from dual union all
select 49039,'MRN' from dual union all
select 4904,'USA' from dual union all
select 49040,'MRN' from dual union all
select 49041,'NASA' from dual union all
select 49042,'MRN' from dual union all
select 49043,'MRN' from dual union all
select 49044,'MRN' from dual union all
select 49045,'MRN' from dual union all
select 49046,'MRN' from dual union all
select 49047,'MRN' from dual union all
select 49048,'MRN' from dual union all
select 49049,'MRN' from dual union all
select 4905,'USA' from dual union all
select 49050,'MRN' from dual union all
select 49051,'MRN' from dual union all
select 49052,'MRN' from dual union all
select 49053,'MRN' from dual union all
select 49054,'MRN' from dual union all
select 49055,'MRN' from dual union all
select 49056,'MRN' from dual union all
select 49057,'MRN' from dual union all
select 49058,'MRN' from dual union all
select 49059,'MRN' from dual union all
select 4906,'USA' from dual union all
select 49060,'MRN' from dual union all
select 49061,'MRN' from dual union all
select 49062,'MRN' from dual union all
select 49063,'MRN' from dual union all
select 49064,'MRN' from dual union all
select 49065,'MRN' from dual union all
select 49066,'MRN' from dual union all
select 49067,'MRN' from dual union all
select 49068,'MRN' from dual union all
select 49069,'MRN' from dual union all
select 4907,'USAF' from dual union all
select 49070,'MRN' from dual;
insert into LAUNCH_AGENCY
select 49071,'MRN' from dual union all
select 49072,'MRN' from dual union all
select 49073,'MRN' from dual union all
select 49074,'AN' from dual union all
select 49075,'NASA' from dual union all
select 49076,'AN' from dual union all
select 49077,'MRN' from dual union all
select 49078,'MRN' from dual union all
select 49079,'MRN' from dual union all
select 4908,'USA' from dual union all
select 49080,'GMS' from dual union all
select 49081,'MRN' from dual union all
select 49082,'MRN' from dual union all
select 49083,'MRN' from dual union all
select 49084,'MRN' from dual union all
select 49085,'GMS' from dual union all
select 49086,'MRN' from dual union all
select 49087,'MRN' from dual union all
select 49088,'MRN' from dual union all
select 49089,'NASA' from dual union all
select 4909,'USA' from dual union all
select 49090,'MRN' from dual union all
select 49091,'MRN' from dual union all
select 49092,'MRN' from dual union all
select 49093,'MRN' from dual union all
select 49094,'MRN' from dual union all
select 49095,'MRN' from dual union all
select 49096,'MRN' from dual union all
select 49097,'NRCC' from dual union all
select 49098,'GMS' from dual union all
select 49099,'MRN' from dual union all
select 491,'JPL' from dual union all
select 4910,'USA' from dual union all
select 49100,'MRN' from dual union all
select 49101,'MRN' from dual union all
select 49102,'MRN' from dual union all
select 49103,'MRN' from dual union all
select 49104,'MRN' from dual union all
select 49105,'MRN' from dual union all
select 49106,'MRN' from dual union all
select 49107,'NRCC' from dual union all
select 49108,'MRN' from dual union all
select 49109,'MRN' from dual union all
select 4911,'USA' from dual union all
select 49110,'MRN' from dual union all
select 49111,'MRN' from dual union all
select 49112,'MRN' from dual union all
select 49113,'MRN' from dual union all
select 49114,'MRN' from dual union all
select 49115,'MRN' from dual union all
select 49116,'MRN' from dual union all
select 49117,'MRN' from dual union all
select 49118,'MRN' from dual union all
select 49119,'MRN' from dual union all
select 4912,'USA' from dual union all
select 49120,'MRN' from dual union all
select 49121,'MRN' from dual union all
select 49122,'MRN' from dual union all
select 49123,'MRN' from dual union all
select 49124,'MRN' from dual union all
select 49125,'MRN' from dual union all
select 49126,'MRN' from dual union all
select 49127,'MRN' from dual union all
select 49128,'MRN' from dual union all
select 49129,'NASA' from dual union all
select 4913,'USA' from dual union all
select 49130,'NASA' from dual union all
select 49131,'MRN' from dual union all
select 49132,'MRN' from dual union all
select 49133,'MRN' from dual union all
select 49134,'MRN' from dual union all
select 49135,'MRN' from dual union all
select 49136,'MRN' from dual union all
select 49137,'MRN' from dual union all
select 49138,'MRN' from dual union all
select 49139,'MRN' from dual union all
select 4914,'USA' from dual union all
select 49140,'MRN' from dual union all
select 49141,'MRN' from dual union all
select 49142,'MRN' from dual union all
select 49143,'MRN' from dual union all
select 49144,'MRN' from dual union all
select 49145,'MRN' from dual union all
select 49146,'MRN' from dual union all
select 49147,'MRN' from dual union all
select 49148,'MRN' from dual union all
select 49149,'MRN' from dual union all
select 4915,'USA' from dual union all
select 49150,'MRN' from dual union all
select 49151,'MRN' from dual union all
select 49152,'MRN' from dual union all
select 49153,'MRN' from dual union all
select 49154,'MRN' from dual union all
select 49155,'MRN' from dual union all
select 49156,'MRN' from dual union all
select 49157,'MRN' from dual union all
select 49158,'MRN' from dual union all
select 49159,'MRN' from dual union all
select 4916,'USA' from dual union all
select 49160,'MRN' from dual;
insert into LAUNCH_AGENCY
select 49161,'MRN' from dual union all
select 49162,'MRN' from dual union all
select 49163,'MRN' from dual union all
select 49164,'MRN' from dual union all
select 49165,'MRN' from dual union all
select 49166,'MRN' from dual union all
select 49167,'MRN' from dual union all
select 49168,'MRN' from dual union all
select 49169,'MRN' from dual union all
select 4917,'USA' from dual union all
select 49170,'MRN' from dual union all
select 49171,'MRN' from dual union all
select 49172,'MRN' from dual union all
select 49173,'MRN' from dual union all
select 49174,'MRN' from dual union all
select 49175,'MRN' from dual union all
select 49176,'MRN' from dual union all
select 49177,'MRN' from dual union all
select 49178,'NASA' from dual union all
select 49179,'MRN' from dual union all
select 4918,'USAF' from dual union all
select 49180,'MRN' from dual union all
select 49181,'MRN' from dual union all
select 49182,'MRN' from dual union all
select 49183,'MRN' from dual union all
select 49184,'MRN' from dual union all
select 49185,'MRN' from dual union all
select 49186,'MRN' from dual union all
select 49187,'MRN' from dual union all
select 49188,'MRN' from dual union all
select 49189,'MRN' from dual union all
select 4919,'USA' from dual union all
select 49190,'AN' from dual union all
select 49191,'MRN' from dual union all
select 49192,'MRN' from dual union all
select 49193,'MRN' from dual union all
select 49194,'MRN' from dual union all
select 49195,'MRN' from dual union all
select 49196,'MRN' from dual union all
select 49197,'MRN' from dual union all
select 49198,'GMS' from dual union all
select 49199,'MRN' from dual union all
select 492,'JPL' from dual union all
select 4920,'USA' from dual union all
select 49200,'MRN' from dual union all
select 49201,'MRN' from dual union all
select 49202,'MRN' from dual union all
select 49203,'MRN' from dual union all
select 49204,'MRN' from dual union all
select 49205,'MRN' from dual union all
select 49206,'MRN' from dual union all
select 49207,'MRN' from dual union all
select 49208,'MRN' from dual union all
select 49209,'MRN' from dual union all
select 4921,'USA' from dual union all
select 49210,'MRN' from dual union all
select 49211,'MRN' from dual union all
select 49212,'GMS' from dual union all
select 49213,'MRN' from dual union all
select 49214,'MRN' from dual union all
select 49215,'MRN' from dual union all
select 49216,'MRN' from dual union all
select 49217,'MRN' from dual union all
select 49218,'MRN' from dual union all
select 49219,'MRN' from dual union all
select 4922,'USA' from dual union all
select 49220,'GMS' from dual union all
select 49221,'MRN' from dual union all
select 49222,'MRN' from dual union all
select 49223,'MRN' from dual union all
select 49224,'MRN' from dual union all
select 49225,'AN' from dual union all
select 49226,'MRN' from dual union all
select 49227,'MRN' from dual union all
select 49228,'AN' from dual union all
select 49229,'MRN' from dual union all
select 4923,'USA' from dual union all
select 49230,'MRN' from dual union all
select 49231,'MRN' from dual union all
select 49232,'MRN' from dual union all
select 49233,'MRN' from dual union all
select 49234,'MRN' from dual union all
select 49235,'MRN' from dual union all
select 49236,'GMS' from dual union all
select 49237,'MRN' from dual union all
select 49238,'MRN' from dual union all
select 49239,'MRN' from dual union all
select 4924,'USA' from dual union all
select 49240,'GMS' from dual union all
select 49241,'MRN' from dual union all
select 49242,'MRN' from dual union all
select 49243,'MRN' from dual union all
select 49244,'MRN' from dual union all
select 49245,'MRN' from dual union all
select 49246,'MRN' from dual union all
select 49247,'MRN' from dual union all
select 49248,'MRN' from dual union all
select 49249,'MRN' from dual union all
select 4925,'PARD' from dual union all
select 49250,'MRN' from dual;
insert into LAUNCH_AGENCY
select 49251,'MRN' from dual union all
select 49252,'MRN' from dual union all
select 49253,'MRN' from dual union all
select 49254,'NASA' from dual union all
select 49255,'AN' from dual union all
select 49256,'GMS' from dual union all
select 49257,'GMS' from dual union all
select 49258,'AN' from dual union all
select 49259,'AN' from dual union all
select 4926,'USN' from dual union all
select 49260,'MRN' from dual union all
select 49261,'GMS' from dual union all
select 49262,'MRN' from dual union all
select 49263,'MRN' from dual union all
select 49264,'MRN' from dual union all
select 49265,'MRN' from dual union all
select 49266,'GMS' from dual union all
select 49267,'MRN' from dual union all
select 49268,'MRN' from dual union all
select 49269,'MRN' from dual union all
select 4927,'USN' from dual union all
select 49270,'MRN' from dual union all
select 49271,'MRN' from dual union all
select 49272,'MRN' from dual union all
select 49273,'MRN' from dual union all
select 49274,'MRN' from dual union all
select 49275,'MRN' from dual union all
select 49276,'MRN' from dual union all
select 49277,'MRN' from dual union all
select 49278,'MRN' from dual union all
select 49279,'MRN' from dual union all
select 4928,'BTL' from dual union all
select 49280,'MRN' from dual union all
select 49281,'GMS' from dual union all
select 49282,'MRN' from dual union all
select 49283,'MRN' from dual union all
select 49284,'MRN' from dual union all
select 49285,'MRN' from dual union all
select 49286,'MRN' from dual union all
select 49287,'MRN' from dual union all
select 49288,'MRN' from dual union all
select 49289,'MRN' from dual union all
select 4929,'SU' from dual union all
select 49290,'MRN' from dual union all
select 49291,'MRN' from dual union all
select 49292,'MRN' from dual union all
select 49293,'MRN' from dual union all
select 49294,'MRN' from dual union all
select 49295,'MRN' from dual union all
select 49296,'MRN' from dual union all
select 49297,'MRN' from dual union all
select 49298,'MRN' from dual union all
select 49299,'GMS' from dual union all
select 493,'JPL' from dual union all
select 4930,'USN' from dual union all
select 49300,'MRN' from dual union all
select 49301,'MRN' from dual union all
select 49302,'MRN' from dual union all
select 49303,'MRN' from dual union all
select 49304,'MRN' from dual union all
select 49305,'MRN' from dual union all
select 49306,'MRN' from dual union all
select 49307,'MRN' from dual union all
select 49308,'MRN' from dual union all
select 49309,'MRN' from dual union all
select 4931,'USN' from dual union all
select 49310,'MRN' from dual union all
select 49311,'MRN' from dual union all
select 49312,'MRN' from dual union all
select 49313,'GMS' from dual union all
select 49314,'MRN' from dual union all
select 49315,'MRN' from dual union all
select 49316,'MRN' from dual union all
select 49317,'MRN' from dual union all
select 49318,'AN' from dual union all
select 49319,'MRN' from dual union all
select 4932,'AFMDC' from dual union all
select 49320,'MRN' from dual union all
select 49321,'MRN' from dual union all
select 49322,'MRN' from dual union all
select 49323,'MRN' from dual union all
select 49324,'MRN' from dual union all
select 49325,'MRN' from dual union all
select 49326,'MRN' from dual union all
select 49327,'GMS' from dual union all
select 49328,'MRN' from dual union all
select 49329,'AN' from dual union all
select 4933,'NOTS' from dual union all
select 49330,'MRN' from dual union all
select 49331,'MRN' from dual union all
select 49332,'MRN' from dual union all
select 49333,'MRN' from dual union all
select 49334,'MRN' from dual union all
select 49335,'MRN' from dual union all
select 49336,'MRN' from dual union all
select 49337,'MRN' from dual union all
select 49338,'MRN' from dual union all
select 49339,'MRN' from dual union all
select 4934,'NOTS' from dual union all
select 49340,'GMS' from dual;
insert into LAUNCH_AGENCY
select 49341,'MRN' from dual union all
select 49342,'MRN' from dual union all
select 49343,'MRN' from dual union all
select 49344,'MRN' from dual union all
select 49345,'MRN' from dual union all
select 49346,'MRN' from dual union all
select 49347,'MRN' from dual union all
select 49348,'MRN' from dual union all
select 49349,'MRN' from dual union all
select 4935,'UNRV' from dual union all
select 49350,'MRN' from dual union all
select 49351,'MRN' from dual union all
select 49352,'MRN' from dual union all
select 49353,'MRN' from dual union all
select 49354,'AN' from dual union all
select 49355,'MRN' from dual union all
select 49356,'MRN' from dual union all
select 49357,'MRN' from dual union all
select 49358,'MRN' from dual union all
select 49359,'MRN' from dual union all
select 4936,'UNRV' from dual union all
select 49360,'GMS' from dual union all
select 49361,'MRN' from dual union all
select 49362,'MRN' from dual union all
select 49363,'MRN' from dual union all
select 49364,'MRN' from dual union all
select 49365,'MRN' from dual union all
select 49366,'MRN' from dual union all
select 49367,'MRN' from dual union all
select 49368,'MRN' from dual union all
select 49369,'GMS' from dual union all
select 4937,'PARD' from dual union all
select 49370,'MRN' from dual union all
select 49371,'MRN' from dual union all
select 49372,'MRN' from dual union all
select 49373,'MRN' from dual union all
select 49374,'MRN' from dual union all
select 49375,'MRN' from dual union all
select 49376,'MRN' from dual union all
select 49377,'MRN' from dual union all
select 49378,'MRN' from dual union all
select 49379,'MRN' from dual union all
select 4938,'LERC' from dual union all
select 49380,'MRN' from dual union all
select 49381,'MRN' from dual union all
select 49382,'MRN' from dual union all
select 49383,'MRN' from dual union all
select 49384,'MRN' from dual union all
select 49385,'MRN' from dual union all
select 49386,'MRN' from dual union all
select 49387,'MRN' from dual union all
select 49388,'MRN' from dual union all
select 49389,'MRN' from dual union all
select 4939,'STA' from dual union all
select 49390,'MRN' from dual union all
select 49391,'GMS' from dual union all
select 49392,'MRN' from dual union all
select 49393,'MRN' from dual union all
select 49394,'MRN' from dual union all
select 49395,'MRN' from dual union all
select 49396,'MRN' from dual union all
select 49397,'MRN' from dual union all
select 49398,'MRN' from dual union all
select 49399,'MRN' from dual union all
select 494,'JPL' from dual union all
select 4940,'STA' from dual union all
select 49400,'MRN' from dual union all
select 49401,'MRN' from dual union all
select 49402,'MRN' from dual union all
select 49403,'MRN' from dual union all
select 49404,'MRN' from dual union all
select 49405,'MRN' from dual union all
select 49406,'MRN' from dual union all
select 49407,'MRN' from dual union all
select 49408,'MRN' from dual union all
select 49409,'MRN' from dual union all
select 4941,'STA' from dual union all
select 49410,'MRN' from dual union all
select 49411,'MRN' from dual union all
select 49412,'MRN' from dual union all
select 49413,'MRN' from dual union all
select 49414,'MRN' from dual union all
select 49415,'MRN' from dual union all
select 49416,'MRN' from dual union all
select 49417,'MRN' from dual union all
select 49418,'MRN' from dual union all
select 49419,'MRN' from dual union all
select 4942,'RAE' from dual union all
select 4942,'WRE' from dual union all
select 49420,'MRN' from dual union all
select 49421,'MRN' from dual union all
select 49422,'MRN' from dual union all
select 49423,'JMA' from dual union all
select 49424,'MRN' from dual union all
select 49425,'GMS' from dual union all
select 49426,'MRN' from dual union all
select 49427,'MRN' from dual union all
select 49428,'MRN' from dual union all
select 49429,'MRN' from dual union all
select 4943,'USAF' from dual;
insert into LAUNCH_AGENCY
select 49430,'MRN' from dual union all
select 49431,'MRN' from dual union all
select 49432,'MRN' from dual union all
select 49433,'MRN' from dual union all
select 49434,'MRN' from dual union all
select 49435,'MRN' from dual union all
select 49436,'MRN' from dual union all
select 49437,'MRN' from dual union all
select 49438,'MRN' from dual union all
select 49439,'MRN' from dual union all
select 4944,'USAF' from dual union all
select 49440,'MRN' from dual union all
select 49441,'MRN' from dual union all
select 49442,'MRN' from dual union all
select 49443,'MRN' from dual union all
select 49444,'MRN' from dual union all
select 49445,'MRN' from dual union all
select 49446,'MRN' from dual union all
select 49447,'GMS' from dual union all
select 49448,'MRN' from dual union all
select 49449,'MRN' from dual union all
select 4945,'ISAS' from dual union all
select 49450,'MRN' from dual union all
select 49451,'MRN' from dual union all
select 49452,'MRN' from dual union all
select 49453,'MRN' from dual union all
select 49454,'MRN' from dual union all
select 49455,'MRN' from dual union all
select 49456,'MRN' from dual union all
select 49457,'MRN' from dual union all
select 49458,'MRN' from dual union all
select 49459,'GMS' from dual union all
select 4946,'PARD' from dual union all
select 49460,'GMS' from dual union all
select 49461,'GMS' from dual union all
select 49462,'MRN' from dual union all
select 49463,'MRN' from dual union all
select 49464,'MRN' from dual union all
select 49465,'MRN' from dual union all
select 49466,'MRN' from dual union all
select 49467,'MRN' from dual union all
select 49468,'MRN' from dual union all
select 49469,'MRN' from dual union all
select 4947,'USAF' from dual union all
select 49470,'MRN' from dual union all
select 49471,'MRN' from dual union all
select 49472,'MRN' from dual union all
select 49473,'NASA' from dual union all
select 49474,'MRN' from dual union all
select 49475,'MRN' from dual union all
select 49476,'MRN' from dual union all
select 49477,'MRN' from dual union all
select 49478,'MRN' from dual union all
select 49479,'MRN' from dual union all
select 4948,'6555A' from dual union all
select 49480,'MRN' from dual union all
select 49481,'MRN' from dual union all
select 49482,'MRN' from dual union all
select 49483,'MRN' from dual union all
select 49484,'MRN' from dual union all
select 49485,'MRN' from dual union all
select 49486,'MRN' from dual union all
select 49487,'MRN' from dual union all
select 49488,'JMA' from dual union all
select 49489,'GMS' from dual union all
select 4949,'ISAS' from dual union all
select 49490,'MRN' from dual union all
select 49491,'MRN' from dual union all
select 49492,'MRN' from dual union all
select 49493,'MRN' from dual union all
select 49494,'MRN' from dual union all
select 49495,'MRN' from dual union all
select 49496,'MRN' from dual union all
select 49497,'MRN' from dual union all
select 49498,'MRN' from dual union all
select 49499,'MRN' from dual union all
select 495,'JPL' from dual union all
select 4950,'PARD' from dual union all
select 49500,'MRN' from dual union all
select 49501,'MRN' from dual union all
select 49502,'MRN' from dual union all
select 49503,'MRN' from dual union all
select 49504,'MRN' from dual union all
select 49505,'MRN' from dual union all
select 49506,'MRN' from dual union all
select 49507,'MRN' from dual union all
select 49508,'MRN' from dual union all
select 49509,'MRN' from dual union all
select 4951,'ISAS' from dual union all
select 49510,'MRN' from dual union all
select 49511,'MRN' from dual union all
select 49512,'MRN' from dual union all
select 49513,'MRN' from dual union all
select 49514,'MRN' from dual union all
select 49515,'MRN' from dual union all
select 49516,'MRN' from dual union all
select 49517,'MRN' from dual union all
select 49518,'AN' from dual union all
select 49519,'MRN' from dual union all
select 4952,'ISAS' from dual;
insert into LAUNCH_AGENCY
select 49520,'MRN' from dual union all
select 49521,'MRN' from dual union all
select 49522,'MRN' from dual union all
select 49523,'MRN' from dual union all
select 49524,'MRN' from dual union all
select 49525,'MRN' from dual union all
select 49526,'MRN' from dual union all
select 49527,'MRN' from dual union all
select 49528,'MRN' from dual union all
select 49529,'MRN' from dual union all
select 4953,'SU' from dual union all
select 49530,'MRN' from dual union all
select 49531,'MRN' from dual union all
select 49532,'MRN' from dual union all
select 49533,'MRN' from dual union all
select 49534,'MRN' from dual union all
select 49535,'MRN' from dual union all
select 49536,'MRN' from dual union all
select 49537,'MRN' from dual union all
select 49538,'MRN' from dual union all
select 49539,'MRN' from dual union all
select 4954,'USAF' from dual union all
select 49540,'MRN' from dual union all
select 49541,'MRN' from dual union all
select 49542,'MRN' from dual union all
select 49543,'JMA' from dual union all
select 49544,'MRN' from dual union all
select 49545,'MRN' from dual union all
select 49546,'MRN' from dual union all
select 49547,'MRN' from dual union all
select 49548,'MRN' from dual union all
select 49549,'MRN' from dual union all
select 4955,'USAF' from dual union all
select 49550,'MRN' from dual union all
select 49551,'MRN' from dual union all
select 49552,'MRN' from dual union all
select 49553,'MRN' from dual union all
select 49554,'MRN' from dual union all
select 49555,'MRN' from dual union all
select 49556,'MRN' from dual union all
select 49557,'MRN' from dual union all
select 49558,'MRN' from dual union all
select 49559,'MRN' from dual union all
select 4956,'USAF' from dual union all
select 49560,'MRN' from dual union all
select 49561,'MRN' from dual union all
select 49562,'MRN' from dual union all
select 49563,'MRN' from dual union all
select 49564,'MRN' from dual union all
select 49565,'MRN' from dual union all
select 49566,'MRN' from dual union all
select 49567,'MRN' from dual union all
select 49568,'MRN' from dual union all
select 49569,'MRN' from dual union all
select 4957,'PARD' from dual union all
select 49570,'MRN' from dual union all
select 49571,'MRN' from dual union all
select 49572,'MRN' from dual union all
select 49573,'MRN' from dual union all
select 49574,'MRN' from dual union all
select 49575,'MRN' from dual union all
select 49576,'MRN' from dual union all
select 49577,'MRN' from dual union all
select 49578,'MRN' from dual union all
select 49579,'MRN' from dual union all
select 4958,'USAF' from dual union all
select 49580,'MRN' from dual union all
select 49581,'MRN' from dual union all
select 49582,'MRN' from dual union all
select 49583,'MRN' from dual union all
select 49584,'MRN' from dual union all
select 49585,'MRN' from dual union all
select 49586,'MRN' from dual union all
select 49587,'AN' from dual union all
select 49588,'MRN' from dual union all
select 49589,'MRN' from dual union all
select 4959,'SU' from dual union all
select 49590,'MRN' from dual union all
select 49591,'MRN' from dual union all
select 49592,'JMA' from dual union all
select 49593,'MRN' from dual union all
select 49594,'MRN' from dual union all
select 49595,'AN' from dual union all
select 49596,'AN' from dual union all
select 49597,'AN' from dual union all
select 49598,'MRN' from dual union all
select 49599,'AN' from dual union all
select 496,'ORD' from dual union all
select 4960,'USAF' from dual union all
select 49600,'MRN' from dual union all
select 49601,'AN' from dual union all
select 49602,'MRN' from dual union all
select 49603,'AN' from dual union all
select 49604,'AN' from dual union all
select 49605,'AN' from dual union all
select 49606,'AN' from dual union all
select 49607,'AN' from dual union all
select 49608,'MRN' from dual union all
select 49609,'MRN' from dual union all
select 4961,'ISAS' from dual;
insert into LAUNCH_AGENCY
select 49610,'AN' from dual union all
select 49611,'MRN' from dual union all
select 49612,'AN' from dual union all
select 49613,'AN' from dual union all
select 49614,'AN' from dual union all
select 49615,'AN' from dual union all
select 49616,'AN' from dual union all
select 49617,'AN' from dual union all
select 49618,'AN' from dual union all
select 49619,'AN' from dual union all
select 4962,'ISAS' from dual union all
select 49620,'AN' from dual union all
select 49621,'AN' from dual union all
select 49622,'AN' from dual union all
select 49623,'AN' from dual union all
select 49624,'AN' from dual union all
select 49625,'MRN' from dual union all
select 49626,'AN' from dual union all
select 49627,'AN' from dual union all
select 49628,'AN' from dual union all
select 49629,'NRCC' from dual union all
select 4963,'USAF' from dual union all
select 49630,'NRCC' from dual union all
select 49631,'NRCC' from dual union all
select 49632,'NRCC' from dual union all
select 49633,'AN' from dual union all
select 49634,'NRCC' from dual union all
select 49635,'NRCC' from dual union all
select 49636,'AN' from dual union all
select 49637,'MRN' from dual union all
select 49638,'AN' from dual union all
select 49639,'MRN' from dual union all
select 4964,'PARD' from dual union all
select 49640,'MRN' from dual union all
select 49641,'AN' from dual union all
select 49642,'AN' from dual union all
select 49643,'MRN' from dual union all
select 49644,'AN' from dual union all
select 49645,'MRN' from dual union all
select 49646,'AN' from dual union all
select 49647,'MRN' from dual union all
select 49648,'AN' from dual union all
select 49649,'AN' from dual union all
select 4965,'USAF' from dual union all
select 49650,'AN' from dual union all
select 49651,'AN' from dual union all
select 49652,'AN' from dual union all
select 49653,'MRN' from dual union all
select 49654,'MRN' from dual union all
select 49655,'MRN' from dual union all
select 49656,'AN' from dual union all
select 49657,'MRN' from dual union all
select 49658,'AN' from dual union all
select 49659,'MRN' from dual union all
select 4966,'SU' from dual union all
select 49660,'AN' from dual union all
select 49661,'AN' from dual union all
select 49662,'AN' from dual union all
select 49663,'AN' from dual union all
select 49664,'AN' from dual union all
select 49665,'AN' from dual union all
select 49666,'AN' from dual union all
select 49667,'AN' from dual union all
select 49668,'AN' from dual union all
select 49669,'AN' from dual union all
select 4967,'ISAS' from dual union all
select 49670,'MRN' from dual union all
select 49671,'AN' from dual union all
select 49672,'AN' from dual union all
select 49673,'AN' from dual union all
select 49674,'AN' from dual union all
select 49675,'AN' from dual union all
select 49676,'AN' from dual union all
select 49677,'AN' from dual union all
select 49678,'AN' from dual union all
select 49679,'AN' from dual union all
select 4968,'ISAS' from dual union all
select 49680,'AN' from dual union all
select 49681,'AN' from dual union all
select 49682,'AN' from dual union all
select 49683,'AN' from dual union all
select 49684,'MRN' from dual union all
select 49685,'MRN' from dual union all
select 49686,'MRN' from dual union all
select 49687,'MRN' from dual union all
select 49688,'AN' from dual union all
select 49689,'AN' from dual union all
select 4969,'RAE' from dual union all
select 4969,'WRE' from dual union all
select 49690,'AN' from dual union all
select 49691,'AN' from dual union all
select 49692,'AN' from dual union all
select 49693,'AN' from dual union all
select 49694,'AN' from dual union all
select 49695,'AN' from dual union all
select 49696,'AN' from dual union all
select 49697,'AN' from dual union all
select 49698,'AN' from dual union all
select 49699,'AN' from dual union all
select 497,'ORD' from dual;
insert into LAUNCH_AGENCY
select 4970,'USAF' from dual union all
select 49700,'AN' from dual union all
select 49701,'AN' from dual union all
select 49702,'AN' from dual union all
select 49703,'AN' from dual union all
select 49704,'AN' from dual union all
select 49705,'AN' from dual union all
select 49706,'AN' from dual union all
select 49707,'AN' from dual union all
select 49708,'AN' from dual union all
select 49709,'AN' from dual union all
select 4971,'ISAS' from dual union all
select 49710,'AN' from dual union all
select 49711,'AN' from dual union all
select 49712,'MRN' from dual union all
select 49713,'AN' from dual union all
select 49714,'AN' from dual union all
select 49715,'MRN' from dual union all
select 49716,'AN' from dual union all
select 49717,'AN' from dual union all
select 49718,'AN' from dual union all
select 49719,'AN' from dual union all
select 4972,'ISAS' from dual union all
select 49720,'AN' from dual union all
select 49721,'AN' from dual union all
select 49722,'AN' from dual union all
select 49723,'AN' from dual union all
select 49724,'AN' from dual union all
select 49725,'AN' from dual union all
select 49726,'AN' from dual union all
select 49727,'AN' from dual union all
select 49728,'AN' from dual union all
select 49729,'AN' from dual union all
select 4973,'USAF' from dual union all
select 49730,'AN' from dual union all
select 49731,'AN' from dual union all
select 49732,'AN' from dual union all
select 49733,'AN' from dual union all
select 49734,'MRN' from dual union all
select 49735,'AN' from dual union all
select 49736,'AN' from dual union all
select 49737,'MRN' from dual union all
select 49738,'MRN' from dual union all
select 49739,'AN' from dual union all
select 4974,'PARD' from dual union all
select 49740,'AN' from dual union all
select 49741,'AN' from dual union all
select 49742,'NASA' from dual union all
select 49743,'AN' from dual union all
select 49744,'AN' from dual union all
select 49745,'AN' from dual union all
select 49746,'AN' from dual union all
select 49747,'AN' from dual union all
select 49748,'AN' from dual union all
select 49749,'AN' from dual union all
select 4975,'USAF' from dual union all
select 49750,'AN' from dual union all
select 49751,'AN' from dual union all
select 49752,'AN' from dual union all
select 49753,'AN' from dual union all
select 49754,'AN' from dual union all
select 49755,'AN' from dual union all
select 49755,'ISRO' from dual union all
select 49756,'AN' from dual union all
select 49757,'AN' from dual union all
select 49758,'AN' from dual union all
select 49759,'AN' from dual union all
select 4976,'PARD' from dual union all
select 49760,'AN' from dual union all
select 49761,'AN' from dual union all
select 49762,'AN' from dual union all
select 49763,'AN' from dual union all
select 49764,'MRN' from dual union all
select 49765,'AN' from dual union all
select 49766,'AN' from dual union all
select 49767,'AN' from dual union all
select 49768,'AN' from dual union all
select 49769,'AN' from dual union all
select 4977,'556SMS' from dual union all
select 49770,'AN' from dual union all
select 49771,'AN' from dual union all
select 49772,'MRN' from dual union all
select 49773,'MRN' from dual union all
select 49774,'AN' from dual union all
select 49775,'AN' from dual union all
select 49776,'AN' from dual union all
select 49777,'AN' from dual union all
select 49778,'AN' from dual union all
select 49779,'AN' from dual union all
select 4978,'USN' from dual union all
select 49780,'AN' from dual union all
select 49781,'AN' from dual union all
select 49782,'MRN' from dual union all
select 49783,'AN' from dual union all
select 49784,'MRN' from dual union all
select 49785,'AN' from dual union all
select 49786,'AN' from dual union all
select 49787,'AN' from dual union all
select 49788,'AN' from dual union all
select 49789,'AN' from dual;
insert into LAUNCH_AGENCY
select 4979,'USN' from dual union all
select 49790,'AN' from dual union all
select 49791,'AN' from dual union all
select 49792,'MRN' from dual union all
select 49793,'MRN' from dual union all
select 49794,'AN' from dual union all
select 49795,'MRN' from dual union all
select 49796,'AN' from dual union all
select 49797,'AN' from dual union all
select 49798,'AN' from dual union all
select 49799,'MRN' from dual union all
select 498,'ORD' from dual union all
select 4980,'ISAS' from dual union all
select 49800,'AN' from dual union all
select 49801,'AN' from dual union all
select 49802,'MRN' from dual union all
select 49803,'AN' from dual union all
select 49804,'MRN' from dual union all
select 49805,'MRN' from dual union all
select 49806,'AN' from dual union all
select 49807,'AN' from dual union all
select 49808,'AN' from dual union all
select 49809,'AN' from dual union all
select 4981,'SU' from dual union all
select 49810,'AN' from dual union all
select 49811,'AN' from dual union all
select 49811,'ISRO' from dual union all
select 49812,'AN' from dual union all
select 49813,'MRN' from dual union all
select 49814,'AN' from dual union all
select 49815,'AN' from dual union all
select 49816,'MRN' from dual union all
select 49817,'AN' from dual union all
select 49818,'AN' from dual union all
select 49819,'AN' from dual union all
select 4982,'USN' from dual union all
select 49820,'AN' from dual union all
select 49821,'AN' from dual union all
select 49822,'AN' from dual union all
select 49823,'AN' from dual union all
select 49824,'MRN' from dual union all
select 49825,'AN' from dual union all
select 49825,'NASA' from dual union all
select 49826,'AN' from dual union all
select 49827,'AN' from dual union all
select 49828,'AN' from dual union all
select 49829,'AN' from dual union all
select 4983,'MVS' from dual union all
select 49830,'AN' from dual union all
select 49831,'AN' from dual union all
select 49832,'MRN' from dual union all
select 49833,'MRN' from dual union all
select 49834,'AN' from dual union all
select 49835,'MRN' from dual union all
select 49836,'AN' from dual union all
select 49837,'AN' from dual union all
select 49838,'AN' from dual union all
select 49839,'AN' from dual union all
select 4984,'USAF' from dual union all
select 49840,'AN' from dual union all
select 49841,'AN' from dual union all
select 49842,'AN' from dual union all
select 49843,'AN' from dual union all
select 49844,'MRN' from dual union all
select 49845,'AN' from dual union all
select 49846,'AN' from dual union all
select 49847,'AN' from dual union all
select 49848,'MRN' from dual union all
select 49849,'AN' from dual union all
select 4985,'AFSWC' from dual union all
select 49850,'MRN' from dual union all
select 49851,'AN' from dual union all
select 49852,'AN' from dual union all
select 49853,'MRN' from dual union all
select 49854,'AN' from dual union all
select 49855,'AN' from dual union all
select 49856,'AN' from dual union all
select 49857,'AN' from dual union all
select 49858,'AN' from dual union all
select 49859,'AN' from dual union all
select 4986,'SU' from dual union all
select 49860,'MRN' from dual union all
select 49861,'AN' from dual union all
select 49862,'AN' from dual union all
select 49863,'AN' from dual union all
select 49864,'AN' from dual union all
select 49865,'AN' from dual union all
select 49866,'AN' from dual union all
select 49867,'MRN' from dual union all
select 49868,'AN' from dual union all
select 49869,'AN' from dual union all
select 4987,'LERC' from dual union all
select 49870,'MRN' from dual union all
select 49871,'AN' from dual union all
select 49872,'AN' from dual union all
select 49873,'AN' from dual union all
select 49874,'AN' from dual union all
select 49875,'AN' from dual union all
select 49876,'AN' from dual union all
select 49877,'AN' from dual;
insert into LAUNCH_AGENCY
select 49878,'AN' from dual union all
select 49879,'AN' from dual union all
select 4988,'NRL' from dual union all
select 49880,'MRN' from dual union all
select 49881,'AN' from dual union all
select 49882,'MRN' from dual union all
select 49883,'AN' from dual union all
select 49884,'AN' from dual union all
select 49885,'AN' from dual union all
select 49886,'AN' from dual union all
select 49887,'AN' from dual union all
select 49888,'AN' from dual union all
select 49889,'AN' from dual union all
select 4989,'USAF' from dual union all
select 49890,'AN' from dual union all
select 49891,'AN' from dual union all
select 49892,'AN' from dual union all
select 49893,'MRN' from dual union all
select 49894,'AN' from dual union all
select 49895,'AN' from dual union all
select 49896,'AN' from dual union all
select 49897,'MRN' from dual union all
select 49898,'MRN' from dual union all
select 49899,'MRN' from dual union all
select 499,'JPL' from dual union all
select 4990,'USN' from dual union all
select 49900,'AN' from dual union all
select 49901,'AN' from dual union all
select 49902,'AN' from dual union all
select 49903,'AN' from dual union all
select 49904,'AN' from dual union all
select 49905,'AN' from dual union all
select 49906,'AN' from dual union all
select 49907,'AN' from dual union all
select 49908,'AN' from dual union all
select 49909,'AN' from dual union all
select 4991,'USAF' from dual union all
select 49910,'MRN' from dual union all
select 49911,'MRN' from dual union all
select 49912,'AN' from dual union all
select 49913,'AN' from dual union all
select 49914,'AN' from dual union all
select 49915,'MRN' from dual union all
select 49916,'AN' from dual union all
select 49917,'AN' from dual union all
select 49918,'AN' from dual union all
select 49919,'AN' from dual union all
select 4992,'SRDL' from dual union all
select 49920,'MRN' from dual union all
select 49921,'MRN' from dual union all
select 49922,'AN' from dual union all
select 49923,'MRN' from dual union all
select 49924,'MRN' from dual union all
select 49925,'MRN' from dual union all
select 49926,'AN' from dual union all
select 49927,'MRN' from dual union all
select 49928,'NASA' from dual union all
select 49929,'NASA' from dual union all
select 4993,'NRL' from dual union all
select 49930,'AN' from dual union all
select 49931,'AN' from dual union all
select 49932,'AN' from dual union all
select 49933,'AN' from dual union all
select 49934,'AN' from dual union all
select 49935,'MRN' from dual union all
select 49936,'MRN' from dual union all
select 49937,'AN' from dual union all
select 49938,'AN' from dual union all
select 49939,'AN' from dual union all
select 4994,'USAF' from dual union all
select 49940,'AN' from dual union all
select 49941,'MRN' from dual union all
select 49942,'AN' from dual union all
select 49943,'AN' from dual union all
select 49944,'AN' from dual union all
select 49945,'AN' from dual union all
select 49946,'AN' from dual union all
select 49947,'MRN' from dual union all
select 49948,'MRN' from dual union all
select 49949,'AN' from dual union all
select 4995,'LERC' from dual union all
select 49950,'AN' from dual union all
select 49951,'AN' from dual union all
select 49952,'AN' from dual union all
select 49953,'AN' from dual union all
select 49954,'AN' from dual union all
select 49955,'AN' from dual union all
select 49956,'AN' from dual union all
select 49957,'MRN' from dual union all
select 49958,'MRN' from dual union all
select 49959,'MRN' from dual union all
select 4996,'6555A' from dual union all
select 49960,'AN' from dual union all
select 49961,'MRN' from dual union all
select 49962,'AN' from dual union all
select 49963,'AN' from dual union all
select 49964,'AN' from dual union all
select 49965,'AN' from dual union all
select 49966,'AN' from dual union all
select 49967,'AN' from dual;
insert into LAUNCH_AGENCY
select 49968,'AN' from dual union all
select 49969,'AN' from dual union all
select 4997,'SAND' from dual union all
select 49970,'AN' from dual union all
select 49971,'AN' from dual union all
select 49972,'AN' from dual union all
select 49973,'AN' from dual union all
select 49974,'AN' from dual union all
select 49975,'MRN' from dual union all
select 49976,'MRN' from dual union all
select 49977,'MRN' from dual union all
select 49978,'AN' from dual union all
select 49979,'AN' from dual union all
select 4998,'SAND' from dual union all
select 49980,'AN' from dual union all
select 49981,'AN' from dual union all
select 49982,'AN' from dual union all
select 49983,'AN' from dual union all
select 49984,'AN' from dual union all
select 49985,'AN' from dual union all
select 49986,'AN' from dual union all
select 49987,'AN' from dual union all
select 49988,'AN' from dual union all
select 49989,'AN' from dual union all
select 4999,'SAND' from dual union all
select 49990,'AN' from dual union all
select 49991,'AN' from dual union all
select 49992,'AN' from dual union all
select 49993,'AN' from dual union all
select 49994,'AN' from dual union all
select 49995,'AN' from dual union all
select 49996,'AN' from dual union all
select 49997,'MRN' from dual union all
select 49998,'MRN' from dual union all
select 49999,'AN' from dual union all
select 5,'WEHR' from dual union all
select 50,'WEHR' from dual union all
select 500,'JPL' from dual union all
select 5000,'SAND' from dual union all
select 50000,'AN' from dual union all
select 50001,'MRN' from dual union all
select 50002,'AN' from dual union all
select 50003,'AN' from dual union all
select 50004,'MRN' from dual union all
select 50005,'MRN' from dual union all
select 50006,'AN' from dual union all
select 50007,'AN' from dual union all
select 50008,'MRN' from dual union all
select 50009,'AN' from dual union all
select 5001,'USAF' from dual union all
select 50010,'AN' from dual union all
select 50011,'AN' from dual union all
select 50012,'AN' from dual union all
select 50013,'AN' from dual union all
select 50014,'MRN' from dual union all
select 50015,'MRN' from dual union all
select 50016,'AN' from dual union all
select 50017,'MRN' from dual union all
select 50018,'NASA' from dual union all
select 50019,'NASA' from dual union all
select 5002,'SAND' from dual union all
select 50020,'AN' from dual union all
select 50021,'AN' from dual union all
select 50022,'MRN' from dual union all
select 50023,'AN' from dual union all
select 50024,'AN' from dual union all
select 50025,'MRN' from dual union all
select 50026,'MRN' from dual union all
select 50027,'AN' from dual union all
select 50028,'AN' from dual union all
select 50029,'MRN' from dual union all
select 5003,'USA' from dual union all
select 50030,'MRN' from dual union all
select 50031,'MRN' from dual union all
select 50032,'AN' from dual union all
select 50033,'AN' from dual union all
select 50034,'AN' from dual union all
select 50035,'MRN' from dual union all
select 50036,'AN' from dual union all
select 50037,'AN' from dual union all
select 50038,'AN' from dual union all
select 50039,'MRN' from dual union all
select 5004,'PARD' from dual union all
select 5004,'UMD' from dual union all
select 50040,'AN' from dual union all
select 50041,'AN' from dual union all
select 50042,'AN' from dual union all
select 50043,'AN' from dual union all
select 50044,'MRN' from dual union all
select 50045,'MRN' from dual union all
select 50046,'NASA' from dual union all
select 50047,'MRN' from dual union all
select 50048,'AN' from dual union all
select 50049,'AN' from dual union all
select 5005,'PARD' from dual union all
select 5005,'UMD' from dual union all
select 50050,'AN' from dual union all
select 50051,'MRN' from dual union all
select 50052,'MRN' from dual union all
select 50053,'AN' from dual;
insert into LAUNCH_AGENCY
select 50054,'AN' from dual union all
select 50055,'AN' from dual union all
select 50056,'AN' from dual union all
select 50057,'MRN' from dual union all
select 50058,'MRN' from dual union all
select 50059,'MRN' from dual union all
select 5006,'PARD' from dual union all
select 5006,'UMD' from dual union all
select 50060,'AN' from dual union all
select 50061,'AN' from dual union all
select 50062,'MRN' from dual union all
select 50063,'AN' from dual union all
select 50064,'NASA' from dual union all
select 50065,'NASA' from dual union all
select 50066,'NASA' from dual union all
select 50067,'AN' from dual union all
select 50068,'AN' from dual union all
select 50069,'AN' from dual union all
select 5007,'6555GMS' from dual union all
select 50070,'AN' from dual union all
select 50071,'AN' from dual union all
select 50072,'MRN' from dual union all
select 50073,'AN' from dual union all
select 50074,'MRN' from dual union all
select 50075,'AN' from dual union all
select 50076,'AN' from dual union all
select 50077,'AN' from dual union all
select 50078,'AN' from dual union all
select 50079,'AN' from dual union all
select 5008,'PARD' from dual union all
select 5008,'UMD' from dual union all
select 50080,'AN' from dual union all
select 50081,'AN' from dual union all
select 50082,'AN' from dual union all
select 50083,'AN' from dual union all
select 50084,'AN' from dual union all
select 50085,'AN' from dual union all
select 50086,'AN' from dual union all
select 50087,'AN' from dual union all
select 50088,'AN' from dual union all
select 50089,'AN' from dual union all
select 5009,'PARD' from dual union all
select 5009,'UMD' from dual union all
select 50090,'AN' from dual union all
select 50091,'MRN' from dual union all
select 50092,'AN' from dual union all
select 50093,'AN' from dual union all
select 50094,'AN' from dual union all
select 50095,'GMS' from dual union all
select 50096,'AN' from dual union all
select 50097,'AN' from dual union all
select 50098,'AN' from dual union all
select 50099,'AN' from dual union all
select 501,'JPL' from dual union all
select 5010,'PARD' from dual union all
select 5010,'UMD' from dual union all
select 50100,'MRN' from dual union all
select 50101,'MRN' from dual union all
select 50102,'AN' from dual union all
select 50103,'AN' from dual union all
select 50104,'AN' from dual union all
select 50105,'MRN' from dual union all
select 50106,'AN' from dual union all
select 50107,'AN' from dual union all
select 50108,'AN' from dual union all
select 50109,'MRN' from dual union all
select 5011,'USN' from dual union all
select 50110,'AN' from dual union all
select 50111,'AN' from dual union all
select 50112,'MRN' from dual union all
select 50113,'AN' from dual union all
select 50114,'AN' from dual union all
select 50115,'MRN' from dual union all
select 50116,'AN' from dual union all
select 50117,'AN' from dual union all
select 50118,'AN' from dual union all
select 50119,'MRN' from dual union all
select 5012,'USN' from dual union all
select 50120,'AN' from dual union all
select 50121,'NASA' from dual union all
select 50122,'AN' from dual union all
select 50123,'AN' from dual union all
select 50124,'MRN' from dual union all
select 50125,'MRN' from dual union all
select 50126,'AN' from dual union all
select 50127,'MRN' from dual union all
select 50128,'AN' from dual union all
select 50129,'MRN' from dual union all
select 5013,'USAF' from dual union all
select 50130,'GMS' from dual union all
select 50131,'AN' from dual union all
select 50132,'AN' from dual union all
select 50133,'AN' from dual union all
select 50134,'AN' from dual union all
select 50135,'AN' from dual union all
select 50136,'AN' from dual union all
select 50137,'AN' from dual union all
select 50138,'MRN' from dual union all
select 50139,'AN' from dual union all
select 5014,'USN' from dual;
insert into LAUNCH_AGENCY
select 50140,'AN' from dual union all
select 50141,'MRN' from dual union all
select 50142,'AN' from dual union all
select 50143,'AN' from dual union all
select 50144,'AN' from dual union all
select 50145,'AN' from dual union all
select 50146,'AN' from dual union all
select 50147,'MRN' from dual union all
select 50148,'AN' from dual union all
select 50149,'AN' from dual union all
select 5015,'USA' from dual union all
select 50150,'MRN' from dual union all
select 50151,'MRN' from dual union all
select 50152,'AN' from dual union all
select 50153,'AN' from dual union all
select 50154,'AN' from dual union all
select 50155,'AN' from dual union all
select 50156,'AN' from dual union all
select 50157,'MRN' from dual union all
select 50158,'AN' from dual union all
select 50159,'AN' from dual union all
select 5016,'MVS' from dual union all
select 50160,'AN' from dual union all
select 50161,'NRCC' from dual union all
select 50162,'NRCC' from dual union all
select 50163,'NRCC' from dual union all
select 50164,'NRCC' from dual union all
select 50165,'AN' from dual union all
select 50166,'NRCC' from dual union all
select 50167,'NRCC' from dual union all
select 50168,'NRCC' from dual union all
select 50169,'NRCC' from dual union all
select 5017,'USN' from dual union all
select 50170,'NRCC' from dual union all
select 50171,'NRCC' from dual union all
select 50172,'NRCC' from dual union all
select 50173,'AN' from dual union all
select 50174,'AN' from dual union all
select 50175,'AN' from dual union all
select 50176,'AN' from dual union all
select 50177,'AN' from dual union all
select 50178,'MRN' from dual union all
select 50179,'MRN' from dual union all
select 5018,'USN' from dual union all
select 50180,'MRN' from dual union all
select 50181,'AN' from dual union all
select 50182,'AN' from dual union all
select 50183,'AN' from dual union all
select 50184,'AN' from dual union all
select 50185,'AN' from dual union all
select 50186,'AN' from dual union all
select 50187,'AN' from dual union all
select 50188,'AN' from dual union all
select 50189,'AN' from dual union all
select 5019,'USN' from dual union all
select 50190,'AN' from dual union all
select 50191,'AN' from dual union all
select 50192,'AN' from dual union all
select 50193,'AN' from dual union all
select 50194,'AN' from dual union all
select 50195,'AN' from dual union all
select 50196,'MRN' from dual union all
select 50197,'MRN' from dual union all
select 50198,'AN' from dual union all
select 50199,'AN' from dual union all
select 502,'JPL' from dual union all
select 5020,'USN' from dual union all
select 50200,'AN' from dual union all
select 50201,'AN' from dual union all
select 50202,'AN' from dual union all
select 50203,'AN' from dual union all
select 50204,'AN' from dual union all
select 50205,'AN' from dual union all
select 50206,'AN' from dual union all
select 50207,'AN' from dual union all
select 50208,'AN' from dual union all
select 50209,'AN' from dual union all
select 5021,'USN' from dual union all
select 50210,'AN' from dual union all
select 50211,'RVSN' from dual union all
select 50212,'RVSN' from dual union all
select 50213,'RVSN' from dual union all
select 50214,'NASA' from dual union all
select 50215,'RVSN' from dual union all
select 50216,'RVSN' from dual union all
select 50217,'RVSN' from dual union all
select 50218,'RVSN' from dual union all
select 50219,'RVSN' from dual union all
select 5022,'USAF' from dual union all
select 50220,'AFSC' from dual union all
select 50221,'AFSC' from dual union all
select 50222,'RVSN' from dual union all
select 50223,'RVSN' from dual union all
select 50224,'NASA' from dual union all
select 50225,'ISAS' from dual union all
select 50226,'RVSN' from dual union all
select 50227,'RVSN' from dual union all
select 50228,'NASDA' from dual union all
select 50229,'AFSC' from dual union all
select 5023,'USN' from dual;
insert into LAUNCH_AGENCY
select 50230,'RVSN' from dual union all
select 50231,'RVSN' from dual union all
select 50232,'RVSN' from dual union all
select 50233,'RVSN' from dual union all
select 50234,'RVSN' from dual union all
select 50235,'RVSN' from dual union all
select 50236,'RVSN' from dual union all
select 50237,'RVSN' from dual union all
select 50238,'RVSN' from dual union all
select 50239,'RVSN' from dual union all
select 5024,'USA' from dual union all
select 50240,'RVSN' from dual union all
select 50241,'RVSN' from dual union all
select 50242,'AFSC' from dual union all
select 50243,'RVSN' from dual union all
select 50244,'RVSN' from dual union all
select 50245,'RVSN' from dual union all
select 50246,'RVSN' from dual union all
select 50247,'RVSN' from dual union all
select 50248,'RVSN' from dual union all
select 50249,'RVSN' from dual union all
select 5025,'USN' from dual union all
select 50250,'RVSN' from dual union all
select 50251,'RVSN' from dual union all
select 50252,'RVSN' from dual union all
select 50253,'AFSC' from dual union all
select 50254,'RVSN' from dual union all
select 50255,'RVSN' from dual union all
select 50256,'RVSN' from dual union all
select 50257,'RVSN' from dual union all
select 50258,'RVSN' from dual union all
select 50259,'RVSN' from dual union all
select 5026,'USN' from dual union all
select 50260,'RVSN' from dual union all
select 50261,'RVSN' from dual union all
select 50262,'AFSC' from dual union all
select 50263,'RVSN' from dual union all
select 50264,'RVSN' from dual union all
select 50265,'RVSN' from dual union all
select 50266,'RVSN' from dual union all
select 50267,'RVSN' from dual union all
select 50268,'RVSN' from dual union all
select 50269,'RVSN' from dual union all
select 5027,'USN' from dual union all
select 50270,'RVSN' from dual union all
select 50271,'RVSN' from dual union all
select 50272,'ISRO' from dual union all
select 50273,'RVSN' from dual union all
select 50274,'RVSN' from dual union all
select 50275,'RVSN' from dual union all
select 50276,'RVSN' from dual union all
select 50277,'RVSN' from dual union all
select 50278,'RVSN' from dual union all
select 50279,'RVSN' from dual union all
select 5028,'MVS' from dual union all
select 50280,'RVSN' from dual union all
select 50281,'RVSN' from dual union all
select 50282,'RVSN' from dual union all
select 50283,'RVSN' from dual union all
select 50284,'NASA' from dual union all
select 50285,'RVSN' from dual union all
select 50286,'RVSN' from dual union all
select 50287,'RVSN' from dual union all
select 50288,'RVSN' from dual union all
select 50289,'RVSN' from dual union all
select 5029,'MVS' from dual union all
select 50290,'RVSN' from dual union all
select 50291,'RVSN' from dual union all
select 50292,'RVSN' from dual union all
select 50293,'RVSN' from dual union all
select 50294,'RVSN' from dual union all
select 50295,'RVSN' from dual union all
select 50296,'RVSN' from dual union all
select 50297,'NASA' from dual union all
select 50298,'RVSN' from dual union all
select 50299,'RVSN' from dual union all
select 503,'JPL' from dual union all
select 5030,'USA' from dual union all
select 50300,'RVSN' from dual union all
select 50301,'NASA' from dual union all
select 50302,'RVSN' from dual union all
select 50303,'RVSN' from dual union all
select 50304,'RVSN' from dual union all
select 50305,'RVSN' from dual union all
select 50306,'RVSN' from dual union all
select 50307,'RVSN' from dual union all
select 50308,'NASA' from dual union all
select 50309,'RVSN' from dual union all
select 5031,'USAF' from dual union all
select 50310,'AFSC' from dual union all
select 50311,'RVSN' from dual union all
select 50312,'RVSN' from dual union all
select 50313,'RVSN' from dual union all
select 50314,'RVSN' from dual union all
select 50315,'RVSN' from dual union all
select 50316,'MRN' from dual union all
select 50317,'MRN' from dual union all
select 50318,'MRN' from dual union all
select 50319,'MRN' from dual union all
select 5032,'MVS' from dual;
insert into LAUNCH_AGENCY
select 50320,'MRN' from dual union all
select 50321,'USAF' from dual union all
select 50322,'MRN' from dual union all
select 50323,'MRN' from dual union all
select 50324,'MRN' from dual union all
select 50325,'MRN' from dual union all
select 50326,'MRN' from dual union all
select 50327,'MRN' from dual union all
select 50328,'MRN' from dual union all
select 50329,'MRN' from dual union all
select 5033,'USA' from dual union all
select 50330,'MRN' from dual union all
select 50331,'MRN' from dual union all
select 50332,'MRN' from dual union all
select 50333,'MRN' from dual union all
select 50334,'MRN' from dual union all
select 50335,'MRN' from dual union all
select 50336,'MRN' from dual union all
select 50337,'MRN' from dual union all
select 50338,'MRN' from dual union all
select 50339,'MRN' from dual union all
select 5034,'USA' from dual union all
select 50340,'MRN' from dual union all
select 50341,'MRN' from dual union all
select 50342,'MRN' from dual union all
select 50343,'MRN' from dual union all
select 50344,'MRN' from dual union all
select 50345,'MRN' from dual union all
select 50346,'MRN' from dual union all
select 50347,'USN' from dual union all
select 50348,'MRN' from dual union all
select 50349,'MRN' from dual union all
select 5035,'LARC' from dual union all
select 50350,'GMS' from dual union all
select 50351,'MRN' from dual union all
select 50352,'MRN' from dual union all
select 50353,'WFC' from dual union all
select 50354,'MRN' from dual union all
select 50355,'MRN' from dual union all
select 50356,'MRN' from dual union all
select 50357,'USAF' from dual union all
select 50358,'MRN' from dual union all
select 50359,'MRN' from dual union all
select 5036,'USA' from dual union all
select 50360,'USN' from dual union all
select 50361,'MRN' from dual union all
select 50362,'MRN' from dual union all
select 50363,'MRN' from dual union all
select 50364,'MRN' from dual union all
select 50365,'MRN' from dual union all
select 50366,'MRN' from dual union all
select 50367,'MRN' from dual union all
select 50368,'MRN' from dual union all
select 50369,'USN' from dual union all
select 5037,'USA' from dual union all
select 50370,'MRN' from dual union all
select 50371,'RVSN' from dual union all
select 50372,'RVSN' from dual union all
select 50373,'RVSN' from dual union all
select 50374,'SAC' from dual union all
select 50375,'ESA' from dual union all
select 50376,'S394I' from dual union all
select 50377,'NASA' from dual union all
select 50378,'ISRO' from dual union all
select 50379,'NASA' from dual union all
select 5038,'USA' from dual union all
select 50380,'ISRO' from dual union all
select 50381,'NASA' from dual union all
select 50382,'NASA' from dual union all
select 50383,'ISRO' from dual union all
select 50384,'ISRO' from dual union all
select 50385,'ISRO' from dual union all
select 50386,'ISRO' from dual union all
select 50387,'ISRO' from dual union all
select 50388,'ISRO' from dual union all
select 50389,'ISRO' from dual union all
select 5039,'USA' from dual union all
select 50390,'ISRO' from dual union all
select 50391,'ISRO' from dual union all
select 50392,'ISRO' from dual union all
select 50393,'ISRO' from dual union all
select 50394,'ISRO' from dual union all
select 50395,'ISRO' from dual union all
select 50396,'ISRO' from dual union all
select 50397,'ISRO' from dual union all
select 50398,'ISRO' from dual union all
select 50399,'ISRO' from dual union all
select 504,'JPL' from dual union all
select 5040,'USA' from dual union all
select 50400,'ISRO' from dual union all
select 50401,'ISRO' from dual union all
select 50402,'ISRO' from dual union all
select 50403,'ISRO' from dual union all
select 50404,'ISRO' from dual union all
select 50405,'ISRO' from dual union all
select 50406,'ISRO' from dual union all
select 50407,'ISRO' from dual union all
select 50408,'ISRO' from dual union all
select 50409,'ISRO' from dual union all
select 5041,'USA' from dual;
insert into LAUNCH_AGENCY
select 50410,'ISRO' from dual union all
select 50411,'ISRO' from dual union all
select 50412,'ISRO' from dual union all
select 50413,'ISRO' from dual union all
select 50414,'ISRO' from dual union all
select 50415,'ISRO' from dual union all
select 50416,'ISRO' from dual union all
select 50417,'ISRO' from dual union all
select 50418,'ISRO' from dual union all
select 50419,'ISRO' from dual union all
select 5042,'USA' from dual union all
select 50420,'ISRO' from dual union all
select 50421,'ISRO' from dual union all
select 50422,'ISRO' from dual union all
select 50423,'ISRO' from dual union all
select 50424,'ISRO' from dual union all
select 50425,'ISRO' from dual union all
select 50426,'ISRO' from dual union all
select 50427,'ISRO' from dual union all
select 50428,'ISRO' from dual union all
select 50429,'ISRO' from dual union all
select 5043,'ISAS' from dual union all
select 50430,'ISRO' from dual union all
select 50431,'ISRO' from dual union all
select 50432,'ISRO' from dual union all
select 50433,'ISRO' from dual union all
select 50434,'ISRO' from dual union all
select 50435,'ISRO' from dual union all
select 50436,'ISRO' from dual union all
select 50437,'ISRO' from dual union all
select 50438,'ISRO' from dual union all
select 50439,'ISRO' from dual union all
select 5044,'ISAS' from dual union all
select 50440,'ISRO' from dual union all
select 50441,'ISRO' from dual union all
select 50442,'ISRO' from dual union all
select 50443,'ISRO' from dual union all
select 50444,'ISRO' from dual union all
select 50445,'ISRO' from dual union all
select 50446,'ISRO' from dual union all
select 50447,'ISRO' from dual union all
select 50448,'ISRO' from dual union all
select 50449,'ISRO' from dual union all
select 5045,'USA' from dual union all
select 50450,'ISRO' from dual union all
select 50451,'ISRO' from dual union all
select 50452,'ISRO' from dual union all
select 50453,'ISRO' from dual union all
select 50454,'ISRO' from dual union all
select 50455,'ISRO' from dual union all
select 50456,'ISRO' from dual union all
select 50457,'ISRO' from dual union all
select 50458,'ISRO' from dual union all
select 50459,'AFGL' from dual union all
select 5046,'USA' from dual union all
select 50460,'AFGL' from dual union all
select 50461,'ISRO' from dual union all
select 50462,'ISRO' from dual union all
select 50463,'ISRO' from dual union all
select 50464,'ISRO' from dual union all
select 50465,'ISRO' from dual union all
select 50466,'ISRO' from dual union all
select 50467,'ISRO' from dual union all
select 50468,'ISRO' from dual union all
select 50469,'ISRO' from dual union all
select 5047,'USN' from dual union all
select 50470,'ISRO' from dual union all
select 50471,'ISRO' from dual union all
select 50472,'ISRO' from dual union all
select 50473,'ISRO' from dual union all
select 50474,'ISRO' from dual union all
select 50475,'ISRO' from dual union all
select 50476,'ISRO' from dual union all
select 50477,'ISRO' from dual union all
select 50478,'ISRO' from dual union all
select 50479,'ISRO' from dual union all
select 5048,'USA' from dual union all
select 50480,'ISRO' from dual union all
select 50481,'ISRO' from dual union all
select 50482,'ISRO' from dual union all
select 50483,'ISRO' from dual union all
select 50484,'ISRO' from dual union all
select 50485,'ISRO' from dual union all
select 50486,'ISRO' from dual union all
select 50487,'ISRO' from dual union all
select 50488,'ISRO' from dual union all
select 50489,'ISRO' from dual union all
select 5049,'USA' from dual union all
select 50490,'ISRO' from dual union all
select 50491,'ISRO' from dual union all
select 50492,'ISRO' from dual union all
select 50493,'ISRO' from dual union all
select 50494,'ISRO' from dual union all
select 50495,'ISRO' from dual union all
select 50496,'ISRO' from dual union all
select 50497,'ISRO' from dual union all
select 50498,'ISRO' from dual union all
select 50499,'ISRO' from dual union all
select 505,'JPL' from dual union all
select 5050,'USA' from dual;
insert into LAUNCH_AGENCY
select 50500,'ISRO' from dual union all
select 50501,'ISRO' from dual union all
select 50502,'ISRO' from dual union all
select 50503,'ISRO' from dual union all
select 50504,'ISRO' from dual union all
select 50505,'ISRO' from dual union all
select 50506,'ISRO' from dual union all
select 50507,'ISRO' from dual union all
select 50508,'ISRO' from dual union all
select 50509,'ISRO' from dual union all
select 5051,'USA' from dual union all
select 50510,'ISRO' from dual union all
select 50511,'ISRO' from dual union all
select 50512,'ISRO' from dual union all
select 50513,'ISRO' from dual union all
select 50514,'ISRO' from dual union all
select 50515,'ISRO' from dual union all
select 50516,'ISRO' from dual union all
select 50517,'ISRO' from dual union all
select 50518,'ISRO' from dual union all
select 50519,'ISRO' from dual union all
select 5052,'USA' from dual union all
select 50520,'ISRO' from dual union all
select 50521,'ISRO' from dual union all
select 50522,'ISRO' from dual union all
select 50523,'ISRO' from dual union all
select 50524,'ISRO' from dual union all
select 50525,'ISRO' from dual union all
select 50526,'ISRO' from dual union all
select 50527,'ISRO' from dual union all
select 50528,'ISRO' from dual union all
select 50529,'ISRO' from dual union all
select 5053,'SAND' from dual union all
select 50530,'ISRO' from dual union all
select 50531,'ISRO' from dual union all
select 50532,'ISRO' from dual union all
select 50533,'ISRO' from dual union all
select 50534,'ISRO' from dual union all
select 50535,'ISRO' from dual union all
select 50536,'ISRO' from dual union all
select 50537,'ISRO' from dual union all
select 50538,'ISRO' from dual union all
select 50539,'ISRO' from dual union all
select 5054,'SAND' from dual union all
select 50540,'ISRO' from dual union all
select 50541,'ISRO' from dual union all
select 50542,'ISRO' from dual union all
select 50543,'ISRO' from dual union all
select 50544,'RVSN' from dual union all
select 50545,'DMA' from dual union all
select 50546,'RVSN' from dual union all
select 50547,'RVSN' from dual union all
select 50548,'RVSN' from dual union all
select 50549,'RVSN' from dual union all
select 5055,'USA' from dual union all
select 50550,'RVSN' from dual union all
select 50551,'RVSN' from dual union all
select 50552,'RVSN' from dual union all
select 50553,'RVSN' from dual union all
select 50554,'AN' from dual union all
select 50555,'AN' from dual union all
select 50556,'AN' from dual union all
select 50557,'AN' from dual union all
select 50558,'US7A' from dual union all
select 50559,'US7A' from dual union all
select 5056,'USA' from dual union all
select 50560,'US7A' from dual union all
select 50561,'NASA' from dual union all
select 50562,'US7A' from dual union all
select 50563,'CALT' from dual union all
select 50563,'PLA2AC' from dual union all
select 50564,'RVSN' from dual union all
select 50565,'RN' from dual union all
select 50566,'CALT' from dual union all
select 50566,'PLA2AC' from dual union all
select 50567,'USAF' from dual union all
select 50568,'NASA' from dual union all
select 50569,'USN' from dual union all
select 5057,'USA' from dual union all
select 50570,'USN' from dual union all
select 50571,'USN' from dual union all
select 50572,'USN' from dual union all
select 50573,'AN' from dual union all
select 50574,'AN' from dual union all
select 50575,'RVSN' from dual union all
select 50576,'AN' from dual union all
select 50577,'AN' from dual union all
select 50578,'AN' from dual union all
select 50579,'NASA' from dual union all
select 5058,'USA' from dual union all
select 50580,'NASA' from dual union all
select 50581,'SAC' from dual union all
select 50582,'NASA' from dual union all
select 50583,'USN' from dual union all
select 50584,'USN' from dual union all
select 50585,'USN' from dual union all
select 50586,'RVSN' from dual union all
select 50587,'USN' from dual union all
select 50588,'NASA' from dual union all
select 50589,'AN' from dual;
insert into LAUNCH_AGENCY
select 5059,'WRE' from dual union all
select 50590,'AN' from dual union all
select 50591,'AN' from dual union all
select 50592,'AN' from dual union all
select 50593,'AN' from dual union all
select 50594,'AN' from dual union all
select 50595,'RVSN' from dual union all
select 50596,'USN' from dual union all
select 50597,'VMF' from dual union all
select 50598,'USN' from dual union all
select 50599,'USN' from dual union all
select 506,'JPL' from dual union all
select 5060,'SAND' from dual union all
select 50600,'USN' from dual union all
select 50601,'USN' from dual union all
select 50602,'NASA' from dual union all
select 50603,'USN' from dual union all
select 50604,'SRC' from dual union all
select 50605,'NASA' from dual union all
select 50606,'NASA' from dual union all
select 50607,'NASA' from dual union all
select 50608,'RVSN' from dual union all
select 50609,'NASA' from dual union all
select 5061,'USAF' from dual union all
select 50610,'NASA' from dual union all
select 50611,'AFGL2' from dual union all
select 50612,'SAC' from dual union all
select 50613,'USN' from dual union all
select 50614,'USN' from dual union all
select 50615,'USN' from dual union all
select 50616,'USN' from dual union all
select 50617,'ISAS' from dual union all
select 50618,'IAE' from dual union all
select 50619,'RVSN' from dual union all
select 5062,'ISAS' from dual union all
select 50620,'USN' from dual union all
select 50621,'USN' from dual union all
select 50622,'USN' from dual union all
select 50623,'RVSN' from dual union all
select 50624,'SAND' from dual union all
select 50625,'SAND' from dual union all
select 50626,'ISAS' from dual union all
select 50627,'ISAS' from dual union all
select 50628,'NASDA' from dual union all
select 50629,'NASA' from dual union all
select 5063,'USA' from dual union all
select 50630,'RVSN' from dual union all
select 50631,'AFSC' from dual union all
select 50632,'SAC' from dual union all
select 50633,'USAF' from dual union all
select 50634,'AFGL' from dual union all
select 50635,'NASA' from dual union all
select 50636,'NASA' from dual union all
select 50637,'ALCS' from dual union all
select 50637,'SAC' from dual union all
select 50638,'NASA' from dual union all
select 50639,'NASA' from dual union all
select 5064,'WRE' from dual union all
select 50640,'NASA' from dual union all
select 50641,'RVSN' from dual union all
select 50642,'NASA' from dual union all
select 50643,'AFSC' from dual union all
select 50644,'NASA' from dual union all
select 50645,'NASA' from dual union all
select 50646,'NASA' from dual union all
select 50647,'FKG' from dual union all
select 50648,'FKG' from dual union all
select 50649,'FKG' from dual union all
select 5065,'WRE' from dual union all
select 50650,'FKG2' from dual union all
select 50651,'NASA' from dual union all
select 50652,'RVSN' from dual union all
select 50653,'NASA' from dual union all
select 50654,'AFGL2' from dual union all
select 50655,'USN' from dual union all
select 50656,'NASA' from dual union all
select 50657,'SRC' from dual union all
select 50658,'NTNF' from dual union all
select 50659,'NTNF' from dual union all
select 5066,'WRE' from dual union all
select 50660,'DLR' from dual union all
select 50661,'DFVLR' from dual union all
select 50662,'RN' from dual union all
select 50663,'RVSN' from dual union all
select 50664,'SRC' from dual union all
select 50665,'DFVLR' from dual union all
select 50666,'AFGL' from dual union all
select 50667,'NTNF' from dual union all
select 50668,'DLR' from dual union all
select 50668,'NTNF' from dual union all
select 50669,'DFVLR' from dual union all
select 5067,'WRE' from dual union all
select 50670,'NASA' from dual union all
select 50671,'NASA' from dual union all
select 50672,'DLR' from dual union all
select 50673,'RN' from dual union all
select 50674,'RVSN' from dual union all
select 50675,'NASA' from dual union all
select 50676,'GMS' from dual union all
select 50677,'GMS' from dual;
insert into LAUNCH_AGENCY
select 50678,'DMA' from dual union all
select 50679,'NTNF' from dual union all
select 5068,'WRE' from dual union all
select 50680,'DLR' from dual union all
select 50681,'DFVLR' from dual union all
select 50682,'SRC' from dual union all
select 50683,'NASA' from dual union all
select 50684,'DLR' from dual union all
select 50685,'RVSN' from dual union all
select 50686,'DLR' from dual union all
select 50687,'DNA' from dual union all
select 50687,'SAND' from dual union all
select 50688,'RVSN' from dual union all
select 50689,'GMS' from dual union all
select 5069,'USAF' from dual union all
select 50690,'DNA' from dual union all
select 50690,'SAND' from dual union all
select 50691,'DNA' from dual union all
select 50691,'SAND' from dual union all
select 50692,'DNA' from dual union all
select 50692,'SAND' from dual union all
select 50693,'DNA' from dual union all
select 50693,'SAND' from dual union all
select 50694,'NASA' from dual union all
select 50695,'DMA' from dual union all
select 50696,'RVSN' from dual union all
select 50697,'GMS' from dual union all
select 50698,'GMS' from dual union all
select 50699,'GMS' from dual union all
select 507,'JPL' from dual union all
select 5070,'USAF' from dual union all
select 50700,'NASA' from dual union all
select 50701,'DNA' from dual union all
select 50701,'SAND' from dual union all
select 50702,'DNA' from dual union all
select 50702,'SAND' from dual union all
select 50703,'AFSC' from dual union all
select 50704,'GMS' from dual union all
select 50705,'GMS' from dual union all
select 50706,'NASA' from dual union all
select 50707,'RVSN' from dual union all
select 50708,'IAE' from dual union all
select 50709,'ISRO' from dual union all
select 5071,'USA' from dual union all
select 50710,'ISRO' from dual union all
select 50711,'PLA2AC' from dual union all
select 50712,'PLA2AC' from dual union all
select 50713,'PLA2AC' from dual union all
select 50714,'IQ224' from dual union all
select 50715,'RVSN' from dual union all
select 50716,'US7A' from dual union all
select 50717,'US7A' from dual union all
select 50718,'US7A' from dual union all
select 50719,'US7A' from dual union all
select 5072,'USN' from dual union all
select 50720,'RVSN' from dual union all
select 50721,'RVSN' from dual union all
select 50722,'RVSN' from dual union all
select 50723,'RVSN' from dual union all
select 50724,'RVSN' from dual union all
select 50725,'RVSN' from dual union all
select 50726,'ISAS' from dual union all
select 50727,'DFVLR' from dual union all
select 50728,'DFVLR' from dual union all
select 50729,'DFVLR' from dual union all
select 5073,'6555A' from dual union all
select 50730,'RVSN' from dual union all
select 50731,'RVSN' from dual union all
select 50732,'RVSN' from dual union all
select 50733,'RVSN' from dual union all
select 50734,'RVSN' from dual union all
select 50735,'RVSN' from dual union all
select 50736,'RVSN' from dual union all
select 50737,'ISAS' from dual union all
select 50738,'RVSN' from dual union all
select 50739,'RVSN' from dual union all
select 5074,'USAF' from dual union all
select 50740,'RVSN' from dual union all
select 50741,'RVSN' from dual union all
select 50742,'RVSN' from dual union all
select 50743,'RVSN' from dual union all
select 50744,'RVSN' from dual union all
select 50745,'CALT' from dual union all
select 50745,'PLA2AC' from dual union all
select 50746,'INTA' from dual union all
select 50747,'INTA' from dual union all
select 50748,'NASA' from dual union all
select 50749,'RVSN' from dual union all
select 5075,'USAF' from dual union all
select 50750,'RVSN' from dual union all
select 50751,'RVSN' from dual union all
select 50752,'RVSN' from dual union all
select 50753,'RVSN' from dual union all
select 50754,'RVSN' from dual union all
select 50755,'RVSN' from dual union all
select 50756,'MVS' from dual union all
select 50757,'MVS' from dual union all
select 50758,'MVS' from dual union all
select 50759,'NTNF' from dual union all
select 5076,'USAF' from dual;
insert into LAUNCH_AGENCY
select 50760,'MVS' from dual union all
select 50761,'MVS' from dual union all
select 50762,'RVSN' from dual union all
select 50763,'RVSN' from dual union all
select 50764,'NASA' from dual union all
select 50765,'NASDA' from dual union all
select 50766,'AFSC' from dual union all
select 50767,'ISAS' from dual union all
select 50768,'USN' from dual union all
select 50769,'USN' from dual union all
select 5077,'ABMA' from dual union all
select 5077,'USAF' from dual union all
select 50770,'USN' from dual union all
select 50771,'USN' from dual union all
select 50772,'SAC' from dual union all
select 50773,'USAF' from dual union all
select 50774,'US7A' from dual union all
select 50775,'US7A' from dual union all
select 50776,'USN' from dual union all
select 50777,'NRCC' from dual union all
select 50778,'NASA' from dual union all
select 50779,'NASA' from dual union all
select 5078,'USN' from dual union all
select 50780,'ISRO' from dual union all
select 50781,'ISRO' from dual union all
select 50782,'NASA' from dual union all
select 50783,'ISRO' from dual union all
select 50784,'ISRO' from dual union all
select 50785,'ISRO' from dual union all
select 50786,'ISRO' from dual union all
select 50787,'ISRO' from dual union all
select 50788,'SAC' from dual union all
select 50789,'NTNF' from dual union all
select 5079,'USAF' from dual union all
select 50790,'ALCS' from dual union all
select 50790,'SAC' from dual union all
select 50791,'NRCC' from dual union all
select 50792,'ALCS' from dual union all
select 50792,'SAC' from dual union all
select 50793,'USN' from dual union all
select 50794,'USN' from dual union all
select 50795,'USN' from dual union all
select 50796,'USN' from dual union all
select 50797,'RVSN' from dual union all
select 50798,'RVSN' from dual union all
select 50799,'AFSC' from dual union all
select 508,'JPL' from dual union all
select 5080,'6555A' from dual union all
select 50800,'NASA' from dual union all
select 50801,'NASA' from dual union all
select 50802,'RN' from dual union all
select 50803,'NASA' from dual union all
select 50804,'NASA' from dual union all
select 50805,'USN' from dual union all
select 50806,'AFSC' from dual union all
select 50807,'NASA' from dual union all
select 50808,'AN' from dual union all
select 50809,'AN' from dual union all
select 5081,'USN' from dual union all
select 50810,'AN' from dual union all
select 50811,'AN' from dual union all
select 50812,'AN' from dual union all
select 50813,'AN' from dual union all
select 50814,'USN' from dual union all
select 50815,'USN' from dual union all
select 50816,'USN' from dual union all
select 50817,'USN' from dual union all
select 50818,'NRCC' from dual union all
select 50819,'AFSC' from dual union all
select 5082,'USN' from dual union all
select 50820,'RN' from dual union all
select 50821,'USN' from dual union all
select 50822,'USN' from dual union all
select 50823,'USN' from dual union all
select 50824,'USN' from dual union all
select 50825,'US7A' from dual union all
select 50826,'US7A' from dual union all
select 50827,'US7A' from dual union all
select 50828,'US7A' from dual union all
select 50829,'RVSN' from dual union all
select 5083,'USAF' from dual union all
select 50830,'AN' from dual union all
select 50831,'DFVLR' from dual union all
select 50832,'AN' from dual union all
select 50833,'AN' from dual union all
select 50834,'AFSC' from dual union all
select 50835,'MRN' from dual union all
select 50836,'MRN' from dual union all
select 50837,'MRN' from dual union all
select 50838,'MRN' from dual union all
select 50839,'MRN' from dual union all
select 5084,'USN' from dual union all
select 50840,'MRN' from dual union all
select 50841,'MRN' from dual union all
select 50842,'MRN' from dual union all
select 50843,'MRN' from dual union all
select 50844,'MRN' from dual union all
select 50845,'MRN' from dual union all
select 50846,'MRN' from dual union all
select 50847,'MRN' from dual;
insert into LAUNCH_AGENCY
select 50848,'MRN' from dual union all
select 50849,'MRN' from dual union all
select 5085,'SAND' from dual union all
select 50850,'MRN' from dual union all
select 50851,'MRN' from dual union all
select 50852,'JMA' from dual union all
select 50853,'GMS' from dual union all
select 50854,'MRN' from dual union all
select 50855,'MRN' from dual union all
select 50856,'MRN' from dual union all
select 50857,'MRN' from dual union all
select 50858,'MRN' from dual union all
select 50859,'MRN' from dual union all
select 5086,'MVS' from dual union all
select 50860,'MRN' from dual union all
select 50861,'MRN' from dual union all
select 50862,'MRN' from dual union all
select 50863,'MRN' from dual union all
select 50864,'MRN' from dual union all
select 50865,'MRN' from dual union all
select 50866,'MRN' from dual union all
select 50867,'MRN' from dual union all
select 50868,'MRN' from dual union all
select 50869,'MRN' from dual union all
select 5087,'MVS' from dual union all
select 50870,'MRN' from dual union all
select 50871,'MRN' from dual union all
select 50872,'MRN' from dual union all
select 50873,'MRN' from dual union all
select 50874,'MRN' from dual union all
select 50875,'MRN' from dual union all
select 50876,'MRN' from dual union all
select 50877,'MRN' from dual union all
select 50878,'MRN' from dual union all
select 50879,'MRN' from dual union all
select 5088,'UNRV' from dual union all
select 50880,'MRN' from dual union all
select 50881,'MRN' from dual union all
select 50882,'MRN' from dual union all
select 50883,'JMA' from dual union all
select 50884,'MRN' from dual union all
select 50885,'AN' from dual union all
select 50886,'MRN' from dual union all
select 50887,'MRN' from dual union all
select 50888,'MRN' from dual union all
select 50889,'MRN' from dual union all
select 5089,'LRL' from dual union all
select 50890,'GMS' from dual union all
select 50891,'GMS' from dual union all
select 50892,'GMS' from dual union all
select 50893,'MRN' from dual union all
select 50894,'MRN' from dual union all
select 50895,'MRN' from dual union all
select 50896,'MRN' from dual union all
select 50897,'MRN' from dual union all
select 50898,'MRN' from dual union all
select 50899,'MRN' from dual union all
select 509,'JPL' from dual union all
select 5090,'LRL' from dual union all
select 50900,'MRN' from dual union all
select 50901,'GMS' from dual union all
select 50902,'GMS' from dual union all
select 50903,'MRN' from dual union all
select 50904,'MRN' from dual union all
select 50905,'MRN' from dual union all
select 50906,'MRN' from dual union all
select 50907,'MRN' from dual union all
select 50908,'MRN' from dual union all
select 50909,'MRN' from dual union all
select 5091,'LRL' from dual union all
select 50910,'MRN' from dual union all
select 50911,'MRN' from dual union all
select 50912,'MRN' from dual union all
select 50913,'MRN' from dual union all
select 50914,'MRN' from dual union all
select 50915,'MRN' from dual union all
select 50916,'MRN' from dual union all
select 50917,'MRN' from dual union all
select 50918,'MRN' from dual union all
select 50919,'MRN' from dual union all
select 5092,'LRL' from dual union all
select 50920,'MRN' from dual union all
select 50921,'JMA' from dual union all
select 50922,'MRN' from dual union all
select 50923,'MRN' from dual union all
select 50924,'MRN' from dual union all
select 50925,'MRN' from dual union all
select 50926,'MRN' from dual union all
select 50927,'MRN' from dual union all
select 50928,'MRN' from dual union all
select 50929,'MRN' from dual union all
select 5093,'LRL' from dual union all
select 50930,'MRN' from dual union all
select 50931,'MRN' from dual union all
select 50932,'MRN' from dual union all
select 50933,'MRN' from dual union all
select 50934,'MRN' from dual union all
select 50935,'MRN' from dual union all
select 50936,'MRN' from dual union all
select 50937,'MRN' from dual;
insert into LAUNCH_AGENCY
select 50938,'MRN' from dual union all
select 50939,'JMA' from dual union all
select 5094,'NRL' from dual union all
select 50940,'NASA' from dual union all
select 50941,'MRN' from dual union all
select 50942,'MRN' from dual union all
select 50943,'MRN' from dual union all
select 50944,'MRN' from dual union all
select 50945,'MRN' from dual union all
select 50946,'MRN' from dual union all
select 50947,'MRN' from dual union all
select 50948,'MRN' from dual union all
select 50949,'MRN' from dual union all
select 5095,'LRL' from dual union all
select 50950,'MRN' from dual union all
select 50951,'MRN' from dual union all
select 50952,'MRN' from dual union all
select 50953,'MRN' from dual union all
select 50954,'MRN' from dual union all
select 50955,'MRN' from dual union all
select 50956,'MRN' from dual union all
select 50957,'MRN' from dual union all
select 50958,'MRN' from dual union all
select 50959,'MRN' from dual union all
select 5096,'LRL' from dual union all
select 50960,'GMS' from dual union all
select 50961,'JMA' from dual union all
select 50962,'MRN' from dual union all
select 50963,'MRN' from dual union all
select 50964,'MRN' from dual union all
select 50965,'MRN' from dual union all
select 50966,'GMS' from dual union all
select 50967,'MRN' from dual union all
select 50968,'GMS' from dual union all
select 50969,'MRN' from dual union all
select 5097,'LRL' from dual union all
select 50970,'MRN' from dual union all
select 50971,'MRN' from dual union all
select 50972,'MRN' from dual union all
select 50973,'MRN' from dual union all
select 50974,'MRN' from dual union all
select 50975,'MRN' from dual union all
select 50976,'MRN' from dual union all
select 50977,'MRN' from dual union all
select 50978,'MRN' from dual union all
select 50979,'MRN' from dual union all
select 5098,'LRL' from dual union all
select 50980,'JMA' from dual union all
select 50981,'AN' from dual union all
select 50982,'AN' from dual union all
select 50983,'MRN' from dual union all
select 50984,'MRN' from dual union all
select 50985,'MRN' from dual union all
select 50986,'MRN' from dual union all
select 50987,'MRN' from dual union all
select 50988,'MRN' from dual union all
select 50989,'MRN' from dual union all
select 5099,'LRL' from dual union all
select 50990,'GMS' from dual union all
select 50991,'MRN' from dual union all
select 50992,'MRN' from dual union all
select 50993,'MRN' from dual union all
select 50994,'MRN' from dual union all
select 50995,'MRN' from dual union all
select 50996,'MRN' from dual union all
select 50997,'MRN' from dual union all
select 50998,'MRN' from dual union all
select 50999,'MRN' from dual union all
select 51,'WER444' from dual union all
select 510,'JPL' from dual union all
select 5100,'LRL' from dual union all
select 51000,'MRN' from dual union all
select 51001,'MRN' from dual union all
select 51002,'MRN' from dual union all
select 51003,'MRN' from dual union all
select 51004,'MRN' from dual union all
select 51005,'MRN' from dual union all
select 51006,'MRN' from dual union all
select 51007,'MRN' from dual union all
select 51008,'JMA' from dual union all
select 51009,'MRN' from dual union all
select 5101,'LRL' from dual union all
select 51010,'JMA' from dual union all
select 51011,'JMA' from dual union all
select 51012,'JMA' from dual union all
select 51013,'MRN' from dual union all
select 51014,'MRN' from dual union all
select 51015,'MRN' from dual union all
select 51016,'GMS' from dual union all
select 51017,'MRN' from dual union all
select 51018,'MRN' from dual union all
select 51019,'MRN' from dual union all
select 5102,'LRL' from dual union all
select 51020,'MRN' from dual union all
select 51021,'MRN' from dual union all
select 51022,'MRN' from dual union all
select 51023,'MRN' from dual union all
select 51024,'GMS' from dual union all
select 51025,'MRN' from dual union all
select 51026,'GMS' from dual;
insert into LAUNCH_AGENCY
select 51027,'MRN' from dual union all
select 51028,'MRN' from dual union all
select 51029,'MRN' from dual union all
select 5103,'USAF' from dual union all
select 51030,'MRN' from dual union all
select 51031,'MRN' from dual union all
select 51032,'MRN' from dual union all
select 51033,'MRN' from dual union all
select 51034,'MRN' from dual union all
select 51035,'MRN' from dual union all
select 51036,'JMA' from dual union all
select 51037,'MRN' from dual union all
select 51038,'MRN' from dual union all
select 51039,'MRN' from dual union all
select 5104,'SAND' from dual union all
select 51040,'MRN' from dual union all
select 51041,'MRN' from dual union all
select 51042,'MRN' from dual union all
select 51043,'MRN' from dual union all
select 51044,'NASA' from dual union all
select 51045,'MRN' from dual union all
select 51046,'MRN' from dual union all
select 51047,'MRN' from dual union all
select 51048,'MRN' from dual union all
select 51049,'GMS' from dual union all
select 5105,'USA' from dual union all
select 51050,'MRN' from dual union all
select 51051,'MRN' from dual union all
select 51052,'MRN' from dual union all
select 51053,'MRN' from dual union all
select 51054,'MRN' from dual union all
select 51055,'MRN' from dual union all
select 51056,'MRN' from dual union all
select 51057,'MRN' from dual union all
select 51058,'NASA' from dual union all
select 51059,'MRN' from dual union all
select 5106,'SAND' from dual union all
select 51060,'MRN' from dual union all
select 51061,'AN' from dual union all
select 51062,'NASA' from dual union all
select 51063,'MRN' from dual union all
select 51064,'NASA' from dual union all
select 51065,'MRN' from dual union all
select 51066,'JMA' from dual union all
select 51067,'GMS' from dual union all
select 51068,'MRN' from dual union all
select 51069,'MRN' from dual union all
select 5107,'SAND' from dual union all
select 51070,'MRN' from dual union all
select 51071,'MRN' from dual union all
select 51072,'MRN' from dual union all
select 51073,'MRN' from dual union all
select 51074,'MRN' from dual union all
select 51075,'MRN' from dual union all
select 51076,'MRN' from dual union all
select 51077,'MRN' from dual union all
select 51078,'MRN' from dual union all
select 51079,'NASDA' from dual union all
select 5108,'SAND' from dual union all
select 51080,'NASA' from dual union all
select 51081,'MRN' from dual union all
select 51082,'MRN' from dual union all
select 51083,'MRN' from dual union all
select 51084,'GMS' from dual union all
select 51085,'MRN' from dual union all
select 51086,'MRN' from dual union all
select 51087,'GMS' from dual union all
select 51088,'GMS' from dual union all
select 51089,'MRN' from dual union all
select 5109,'MVS' from dual union all
select 51090,'GMS' from dual union all
select 51091,'GMS' from dual union all
select 51092,'GMS' from dual union all
select 51093,'GMS' from dual union all
select 51094,'MRN' from dual union all
select 51095,'MRN' from dual union all
select 51096,'MRN' from dual union all
select 51097,'GMS' from dual union all
select 51098,'MRN' from dual union all
select 51099,'GMS' from dual union all
select 511,'JPL' from dual union all
select 5110,'USAF' from dual union all
select 51100,'GMS' from dual union all
select 51101,'GMS' from dual union all
select 51102,'GMS' from dual union all
select 51103,'GMS' from dual union all
select 51104,'MRN' from dual union all
select 51105,'MRN' from dual union all
select 51106,'MRN' from dual union all
select 51107,'MRN' from dual union all
select 51108,'MRN' from dual union all
select 51109,'AN' from dual union all
select 5111,'USAF' from dual union all
select 51110,'GMS' from dual union all
select 51111,'GMS' from dual union all
select 51112,'GMS' from dual union all
select 51113,'MRN' from dual union all
select 51114,'MRN' from dual union all
select 51115,'MRN' from dual union all
select 51116,'MRN' from dual;
insert into LAUNCH_AGENCY
select 51117,'MRN' from dual union all
select 51118,'MRN' from dual union all
select 51119,'MRN' from dual union all
select 5112,'UNRV' from dual union all
select 51120,'MRN' from dual union all
select 51121,'NASA' from dual union all
select 51122,'GMS' from dual union all
select 51123,'MRN' from dual union all
select 51124,'MRN' from dual union all
select 51125,'MRN' from dual union all
select 51126,'MRN' from dual union all
select 51127,'MRN' from dual union all
select 51128,'MRN' from dual union all
select 51129,'MRN' from dual union all
select 5113,'SAND' from dual union all
select 51130,'MRN' from dual union all
select 51131,'JMA' from dual union all
select 51132,'GMS' from dual union all
select 51133,'MRN' from dual union all
select 51134,'MRN' from dual union all
select 51135,'MRN' from dual union all
select 51136,'MRN' from dual union all
select 51137,'MRN' from dual union all
select 51138,'MRN' from dual union all
select 51139,'MRN' from dual union all
select 5114,'SAND' from dual union all
select 51140,'MRN' from dual union all
select 51141,'MRN' from dual union all
select 51142,'MRN' from dual union all
select 51143,'MRN' from dual union all
select 51144,'MRN' from dual union all
select 51145,'MRN' from dual union all
select 51146,'MRN' from dual union all
select 51147,'MRN' from dual union all
select 51148,'MRN' from dual union all
select 51149,'NASA' from dual union all
select 5115,'SAND' from dual union all
select 51150,'MRN' from dual union all
select 51151,'MRN' from dual union all
select 51152,'MRN' from dual union all
select 51153,'MRN' from dual union all
select 51154,'MRN' from dual union all
select 51155,'MRN' from dual union all
select 51156,'MRN' from dual union all
select 51157,'GMS' from dual union all
select 51158,'NASA' from dual union all
select 51159,'MRN' from dual union all
select 5116,'USAF' from dual union all
select 51160,'MRN' from dual union all
select 51161,'AN' from dual union all
select 51162,'MRN' from dual union all
select 51163,'MRN' from dual union all
select 51164,'MRN' from dual union all
select 51165,'MRN' from dual union all
select 51166,'MRN' from dual union all
select 51167,'MRN' from dual union all
select 51168,'MRN' from dual union all
select 51169,'MRN' from dual union all
select 5117,'SAND' from dual union all
select 51170,'MRN' from dual union all
select 51171,'MRN' from dual union all
select 51172,'GMS' from dual union all
select 51173,'MRN' from dual union all
select 51174,'MRN' from dual union all
select 51175,'MRN' from dual union all
select 51176,'MRN' from dual union all
select 51177,'MRN' from dual union all
select 51178,'MRN' from dual union all
select 51179,'MRN' from dual union all
select 5118,'SAND' from dual union all
select 51180,'MRN' from dual union all
select 51181,'MRN' from dual union all
select 51182,'MRN' from dual union all
select 51183,'MRN' from dual union all
select 51184,'MRN' from dual union all
select 51185,'MRN' from dual union all
select 51186,'MRN' from dual union all
select 51187,'MRN' from dual union all
select 51188,'MRN' from dual union all
select 51189,'NASA' from dual union all
select 5119,'SAND' from dual union all
select 51190,'JMA' from dual union all
select 51191,'MRN' from dual union all
select 51192,'GMS' from dual union all
select 51193,'JMA' from dual union all
select 51194,'MRN' from dual union all
select 51195,'MRN' from dual union all
select 51196,'MRN' from dual union all
select 51197,'MRN' from dual union all
select 51198,'MRN' from dual union all
select 51199,'MRN' from dual union all
select 512,'JPL' from dual union all
select 5120,'SAND' from dual union all
select 51200,'MRN' from dual union all
select 51201,'MRN' from dual union all
select 51202,'GMS' from dual union all
select 51203,'MRN' from dual union all
select 51204,'MRN' from dual union all
select 51205,'MRN' from dual union all
select 51206,'MRN' from dual;
insert into LAUNCH_AGENCY
select 51207,'MRN' from dual union all
select 51208,'MRN' from dual union all
select 51209,'MRN' from dual union all
select 5121,'LRL' from dual union all
select 51210,'MRN' from dual union all
select 51211,'MRN' from dual union all
select 51212,'MRN' from dual union all
select 51213,'MRN' from dual union all
select 51214,'MRN' from dual union all
select 51215,'MRN' from dual union all
select 51216,'MRN' from dual union all
select 51217,'MRN' from dual union all
select 51218,'MRN' from dual union all
select 51219,'MRN' from dual union all
select 5122,'LRL' from dual union all
select 51220,'MRN' from dual union all
select 51221,'JMA' from dual union all
select 51222,'GMS' from dual union all
select 51223,'MRN' from dual union all
select 51224,'MRN' from dual union all
select 51225,'AN' from dual union all
select 51226,'MRN' from dual union all
select 51227,'MRN' from dual union all
select 51228,'MRN' from dual union all
select 51229,'MRN' from dual union all
select 5123,'LRL' from dual union all
select 51230,'MRN' from dual union all
select 51231,'MRN' from dual union all
select 51232,'GMS' from dual union all
select 51233,'MRN' from dual union all
select 51234,'MRN' from dual union all
select 51235,'MRN' from dual union all
select 51236,'MRN' from dual union all
select 51237,'MRN' from dual union all
select 51238,'MRN' from dual union all
select 51239,'MRN' from dual union all
select 5124,'LRL' from dual union all
select 51240,'MRN' from dual union all
select 51241,'MRN' from dual union all
select 51242,'MRN' from dual union all
select 51243,'MRN' from dual union all
select 51244,'MRN' from dual union all
select 51245,'MRN' from dual union all
select 51246,'MRN' from dual union all
select 51247,'MRN' from dual union all
select 51248,'MRN' from dual union all
select 51249,'MRN' from dual union all
select 5125,'LRL' from dual union all
select 51250,'MRN' from dual union all
select 51251,'MRN' from dual union all
select 51252,'MRN' from dual union all
select 51253,'MRN' from dual union all
select 51254,'MRN' from dual union all
select 51255,'MRN' from dual union all
select 51256,'MRN' from dual union all
select 51257,'MRN' from dual union all
select 51258,'MRN' from dual union all
select 51259,'MRN' from dual union all
select 5126,'LRL' from dual union all
select 51260,'MRN' from dual union all
select 51261,'MRN' from dual union all
select 51262,'MRN' from dual union all
select 51263,'MRN' from dual union all
select 51264,'MRN' from dual union all
select 51265,'MRN' from dual union all
select 51266,'JMA' from dual union all
select 51267,'MRN' from dual union all
select 51268,'MRN' from dual union all
select 51269,'MRN' from dual union all
select 5127,'USA' from dual union all
select 51270,'MRN' from dual union all
select 51271,'MRN' from dual union all
select 51272,'MRN' from dual union all
select 51273,'NASA' from dual union all
select 51274,'MRN' from dual union all
select 51275,'MRN' from dual union all
select 51276,'MRN' from dual union all
select 51277,'MRN' from dual union all
select 51278,'MRN' from dual union all
select 51279,'MRN' from dual union all
select 5128,'LRL' from dual union all
select 51280,'MRN' from dual union all
select 51281,'MRN' from dual union all
select 51282,'MRN' from dual union all
select 51283,'MRN' from dual union all
select 51284,'MRN' from dual union all
select 51285,'MRN' from dual union all
select 51286,'MRN' from dual union all
select 51287,'MRN' from dual union all
select 51288,'MRN' from dual union all
select 51289,'MRN' from dual union all
select 5129,'LRL' from dual union all
select 51290,'MRN' from dual union all
select 51291,'MRN' from dual union all
select 51292,'MRN' from dual union all
select 51293,'MRN' from dual union all
select 51294,'MRN' from dual union all
select 51295,'MRN' from dual union all
select 51296,'MRN' from dual union all
select 51297,'MRN' from dual;
insert into LAUNCH_AGENCY
select 51298,'JMA' from dual union all
select 51299,'MRN' from dual union all
select 513,'JPL' from dual union all
select 5130,'LRL' from dual union all
select 51300,'MRN' from dual union all
select 51301,'MRN' from dual union all
select 51302,'MRN' from dual union all
select 51303,'MRN' from dual union all
select 51304,'MRN' from dual union all
select 51305,'MRN' from dual union all
select 51306,'MRN' from dual union all
select 51307,'MRN' from dual union all
select 51308,'MRN' from dual union all
select 51309,'MRN' from dual union all
select 5131,'LRL' from dual union all
select 51310,'MRN' from dual union all
select 51311,'MRN' from dual union all
select 51312,'MRN' from dual union all
select 51313,'MRN' from dual union all
select 51314,'MRN' from dual union all
select 51315,'GMS' from dual union all
select 51316,'MRN' from dual union all
select 51317,'MRN' from dual union all
select 51318,'MRN' from dual union all
select 51319,'MRN' from dual union all
select 5132,'LRL' from dual union all
select 51320,'MRN' from dual union all
select 51321,'MRN' from dual union all
select 51322,'MRN' from dual union all
select 51323,'MRN' from dual union all
select 51324,'MRN' from dual union all
select 51325,'MRN' from dual union all
select 51326,'MRN' from dual union all
select 51327,'MRN' from dual union all
select 51328,'GMS' from dual union all
select 51329,'MRN' from dual union all
select 5133,'LRL' from dual union all
select 51330,'MRN' from dual union all
select 51331,'MRN' from dual union all
select 51332,'MRN' from dual union all
select 51333,'MRN' from dual union all
select 51334,'MRN' from dual union all
select 51335,'MRN' from dual union all
select 51336,'MRN' from dual union all
select 51337,'MRN' from dual union all
select 51338,'MRN' from dual union all
select 51339,'MRN' from dual union all
select 5134,'LRL' from dual union all
select 51340,'GMS' from dual union all
select 51341,'MRN' from dual union all
select 51342,'MRN' from dual union all
select 51343,'MRN' from dual union all
select 51344,'MRN' from dual union all
select 51345,'MRN' from dual union all
select 51346,'MRN' from dual union all
select 51347,'MRN' from dual union all
select 51348,'MRN' from dual union all
select 51349,'MRN' from dual union all
select 5135,'LRL' from dual union all
select 51350,'MRN' from dual union all
select 51351,'MRN' from dual union all
select 51352,'JMA' from dual union all
select 51353,'MRN' from dual union all
select 51354,'MRN' from dual union all
select 51355,'MRN' from dual union all
select 51356,'MRN' from dual union all
select 51357,'MRN' from dual union all
select 51358,'GMS' from dual union all
select 51359,'MRN' from dual union all
select 5136,'USAF' from dual union all
select 51360,'MRN' from dual union all
select 51361,'MRN' from dual union all
select 51362,'MRN' from dual union all
select 51363,'MRN' from dual union all
select 51364,'MRN' from dual union all
select 51365,'MRN' from dual union all
select 51366,'MRN' from dual union all
select 51367,'MRN' from dual union all
select 51368,'MRN' from dual union all
select 51369,'GMS' from dual union all
select 5137,'AN' from dual union all
select 51370,'MRN' from dual union all
select 51371,'MRN' from dual union all
select 51372,'MRN' from dual union all
select 51373,'MRN' from dual union all
select 51374,'MRN' from dual union all
select 51375,'MRN' from dual union all
select 51376,'MRN' from dual union all
select 51377,'MRN' from dual union all
select 51378,'MRN' from dual union all
select 51379,'MRN' from dual union all
select 5138,'USAF' from dual union all
select 51380,'GMS' from dual union all
select 51381,'MRN' from dual union all
select 51382,'MRN' from dual union all
select 51383,'MRN' from dual union all
select 51384,'MRN' from dual union all
select 51385,'JMA' from dual union all
select 51386,'MRN' from dual union all
select 51387,'MRN' from dual;
insert into LAUNCH_AGENCY
select 51388,'MRN' from dual union all
select 51389,'MRN' from dual union all
select 5139,'MVS' from dual union all
select 51390,'MRN' from dual union all
select 51391,'MRN' from dual union all
select 51392,'MRN' from dual union all
select 51393,'MRN' from dual union all
select 51394,'MRN' from dual union all
select 51395,'MRN' from dual union all
select 51396,'MRN' from dual union all
select 51397,'MRN' from dual union all
select 51398,'NASA' from dual union all
select 51399,'MRN' from dual union all
select 514,'JPL' from dual union all
select 5140,'USAF' from dual union all
select 51400,'MRN' from dual union all
select 51401,'MRN' from dual union all
select 51402,'MRN' from dual union all
select 51403,'MRN' from dual union all
select 51404,'MRN' from dual union all
select 51405,'JMA' from dual union all
select 51406,'MRN' from dual union all
select 51407,'MRN' from dual union all
select 51408,'MRN' from dual union all
select 51409,'MRN' from dual union all
select 5141,'USAF' from dual union all
select 51410,'MRN' from dual union all
select 51411,'MRN' from dual union all
select 51412,'MRN' from dual union all
select 51413,'MRN' from dual union all
select 51414,'MRN' from dual union all
select 51415,'MRN' from dual union all
select 51416,'MRN' from dual union all
select 51417,'MRN' from dual union all
select 51418,'MRN' from dual union all
select 51419,'MRN' from dual union all
select 5142,'USAF' from dual union all
select 51420,'MRN' from dual union all
select 51421,'GMS' from dual union all
select 51422,'MRN' from dual union all
select 51423,'MRN' from dual union all
select 51424,'MRN' from dual union all
select 51425,'MRN' from dual union all
select 51426,'MRN' from dual union all
select 51427,'MRN' from dual union all
select 51428,'MRN' from dual union all
select 51429,'MRN' from dual union all
select 5143,'UNRV' from dual union all
select 51430,'MRN' from dual union all
select 51431,'MRN' from dual union all
select 51432,'JMA' from dual union all
select 51433,'MRN' from dual union all
select 51434,'MRN' from dual union all
select 51435,'MRN' from dual union all
select 51436,'MRN' from dual union all
select 51437,'MRN' from dual union all
select 51438,'MRN' from dual union all
select 51439,'MRN' from dual union all
select 5144,'USAF' from dual union all
select 51440,'MRN' from dual union all
select 51441,'MRN' from dual union all
select 51442,'MRN' from dual union all
select 51443,'MRN' from dual union all
select 51444,'MRN' from dual union all
select 51445,'MRN' from dual union all
select 51446,'MRN' from dual union all
select 51447,'MRN' from dual union all
select 51448,'MRN' from dual union all
select 51449,'AN' from dual union all
select 5145,'USAF' from dual union all
select 51450,'MRN' from dual union all
select 51451,'MRN' from dual union all
select 51452,'NASA' from dual union all
select 51453,'MRN' from dual union all
select 51454,'AN' from dual union all
select 51455,'MRN' from dual union all
select 51456,'JMA' from dual union all
select 51457,'JMA' from dual union all
select 51458,'AN' from dual union all
select 51459,'MRN' from dual union all
select 5146,'UNRV' from dual union all
select 51460,'MRN' from dual union all
select 51461,'MRN' from dual union all
select 51462,'MRN' from dual union all
select 51463,'MRN' from dual union all
select 51464,'MRN' from dual union all
select 51465,'MRN' from dual union all
select 51466,'MRN' from dual union all
select 51467,'GMS' from dual union all
select 51468,'GMS' from dual union all
select 51469,'MRN' from dual union all
select 5147,'USN' from dual union all
select 51470,'MRN' from dual union all
select 51471,'MRN' from dual union all
select 51472,'MRN' from dual union all
select 51473,'MRN' from dual union all
select 51474,'GMS' from dual union all
select 51475,'GMS' from dual union all
select 51476,'MRN' from dual union all
select 51477,'GMS' from dual;
insert into LAUNCH_AGENCY
select 51478,'MRN' from dual union all
select 51479,'GMS' from dual union all
select 5148,'USAF' from dual union all
select 51480,'MRN' from dual union all
select 51481,'MRN' from dual union all
select 51482,'MRN' from dual union all
select 51483,'MRN' from dual union all
select 51484,'MRN' from dual union all
select 51485,'JMA' from dual union all
select 51486,'MRN' from dual union all
select 51487,'MRN' from dual union all
select 51488,'MRN' from dual union all
select 51489,'MRN' from dual union all
select 5149,'MVS' from dual union all
select 51490,'MRN' from dual union all
select 51491,'MRN' from dual union all
select 51492,'MRN' from dual union all
select 51493,'MRN' from dual union all
select 51494,'MRN' from dual union all
select 51495,'MRN' from dual union all
select 51496,'MRN' from dual union all
select 51497,'MRN' from dual union all
select 51498,'MRN' from dual union all
select 51499,'MRN' from dual union all
select 515,'JPL' from dual union all
select 5150,'MVS' from dual union all
select 51500,'MRN' from dual union all
select 51501,'MRN' from dual union all
select 51502,'MRN' from dual union all
select 51503,'MRN' from dual union all
select 51504,'MRN' from dual union all
select 51505,'MRN' from dual union all
select 51506,'MRN' from dual union all
select 51507,'MRN' from dual union all
select 51508,'MRN' from dual union all
select 51509,'MRN' from dual union all
select 5151,'USAF' from dual union all
select 51510,'NASA' from dual union all
select 51511,'MRN' from dual union all
select 51512,'MRN' from dual union all
select 51513,'MRN' from dual union all
select 51514,'MRN' from dual union all
select 51515,'NASA' from dual union all
select 51516,'MRN' from dual union all
select 51517,'JMA' from dual union all
select 51518,'MRN' from dual union all
select 51519,'MRN' from dual union all
select 5152,'AN' from dual union all
select 51520,'MRN' from dual union all
select 51521,'MRN' from dual union all
select 51522,'MRN' from dual union all
select 51523,'AN' from dual union all
select 51524,'MRN' from dual union all
select 51525,'MRN' from dual union all
select 51526,'MRN' from dual union all
select 51527,'MRN' from dual union all
select 51528,'MRN' from dual union all
select 51529,'MRN' from dual union all
select 5153,'USAF' from dual union all
select 51530,'MRN' from dual union all
select 51531,'MRN' from dual union all
select 51532,'MRN' from dual union all
select 51533,'MRN' from dual union all
select 51534,'MRN' from dual union all
select 51535,'MRN' from dual union all
select 51536,'MRN' from dual union all
select 51537,'MRN' from dual union all
select 51538,'MRN' from dual union all
select 51539,'MRN' from dual union all
select 5154,'USAF' from dual union all
select 51540,'MRN' from dual union all
select 51541,'MRN' from dual union all
select 51542,'MRN' from dual union all
select 51543,'MRN' from dual union all
select 51544,'MRN' from dual union all
select 51545,'MRN' from dual union all
select 51546,'MRN' from dual union all
select 51547,'MRN' from dual union all
select 51548,'MRN' from dual union all
select 51549,'MRN' from dual union all
select 5155,'ABMA' from dual union all
select 5155,'USAF' from dual union all
select 51550,'MRN' from dual union all
select 51551,'MRN' from dual union all
select 51552,'GMS' from dual union all
select 51553,'MRN' from dual union all
select 51554,'MRN' from dual union all
select 51555,'MRN' from dual union all
select 51556,'MRN' from dual union all
select 51557,'MRN' from dual union all
select 51558,'MRN' from dual union all
select 51559,'MRN' from dual union all
select 5156,'USAF' from dual union all
select 51560,'MRN' from dual union all
select 51561,'MRN' from dual union all
select 51562,'MRN' from dual union all
select 51563,'MRN' from dual union all
select 51564,'MRN' from dual union all
select 51565,'MRN' from dual union all
select 51566,'MRN' from dual;
insert into LAUNCH_AGENCY
select 51567,'MRN' from dual union all
select 51568,'MRN' from dual union all
select 51569,'MRN' from dual union all
select 5157,'USN' from dual union all
select 51570,'MRN' from dual union all
select 51571,'MRN' from dual union all
select 51572,'JMA' from dual union all
select 51573,'MRN' from dual union all
select 51574,'MRN' from dual union all
select 51575,'MRN' from dual union all
select 51576,'MRN' from dual union all
select 51577,'MRN' from dual union all
select 51578,'MRN' from dual union all
select 51579,'MRN' from dual union all
select 5158,'USAF' from dual union all
select 51580,'MRN' from dual union all
select 51581,'GMS' from dual union all
select 51582,'MRN' from dual union all
select 51583,'MRN' from dual union all
select 51584,'MRN' from dual union all
select 51585,'MRN' from dual union all
select 51586,'MRN' from dual union all
select 51587,'GMS' from dual union all
select 51588,'MRN' from dual union all
select 51589,'MRN' from dual union all
select 5159,'USAF' from dual union all
select 51590,'MRN' from dual union all
select 51591,'MRN' from dual union all
select 51592,'MRN' from dual union all
select 51593,'MRN' from dual union all
select 51594,'MRN' from dual union all
select 51595,'MRN' from dual union all
select 51596,'MRN' from dual union all
select 51597,'MRN' from dual union all
select 51598,'MRN' from dual union all
select 51599,'MRN' from dual union all
select 516,'ORD' from dual union all
select 5160,'USAF' from dual union all
select 51600,'MRN' from dual union all
select 51601,'MRN' from dual union all
select 51602,'MRN' from dual union all
select 51603,'MRN' from dual union all
select 51604,'MRN' from dual union all
select 51605,'MRN' from dual union all
select 51606,'NASA' from dual union all
select 51607,'NASA' from dual union all
select 51608,'MRN' from dual union all
select 51609,'MRN' from dual union all
select 5161,'USAF' from dual union all
select 51610,'MRN' from dual union all
select 51611,'GMS' from dual union all
select 51612,'MRN' from dual union all
select 51613,'MRN' from dual union all
select 51614,'MRN' from dual union all
select 51615,'MRN' from dual union all
select 51616,'MRN' from dual union all
select 51617,'MRN' from dual union all
select 51618,'MRN' from dual union all
select 51619,'MRN' from dual union all
select 5162,'USAF' from dual union all
select 51620,'MRN' from dual union all
select 51621,'MRN' from dual union all
select 51622,'AN' from dual union all
select 51623,'MRN' from dual union all
select 51624,'MRN' from dual union all
select 51625,'MRN' from dual union all
select 51626,'MRN' from dual union all
select 51627,'MRN' from dual union all
select 51628,'MRN' from dual union all
select 51629,'MRN' from dual union all
select 5163,'USAF' from dual union all
select 51630,'MRN' from dual union all
select 51631,'MRN' from dual union all
select 51632,'MRN' from dual union all
select 51633,'MRN' from dual union all
select 51634,'MRN' from dual union all
select 51635,'MRN' from dual union all
select 51636,'MRN' from dual union all
select 51637,'MRN' from dual union all
select 51638,'MRN' from dual union all
select 51639,'AN' from dual union all
select 5164,'USAF' from dual union all
select 51640,'MRN' from dual union all
select 51641,'MRN' from dual union all
select 51642,'MRN' from dual union all
select 51643,'MRN' from dual union all
select 51644,'MRN' from dual union all
select 51645,'MRN' from dual union all
select 51646,'NASA' from dual union all
select 51647,'MRN' from dual union all
select 51648,'MRN' from dual union all
select 51649,'MRN' from dual union all
select 5165,'USAF' from dual union all
select 51650,'MRN' from dual union all
select 51651,'MRN' from dual union all
select 51652,'MRN' from dual union all
select 51653,'MRN' from dual union all
select 51654,'MRN' from dual union all
select 51655,'MRN' from dual union all
select 51656,'MRN' from dual;
insert into LAUNCH_AGENCY
select 51657,'JMA' from dual union all
select 51658,'MRN' from dual union all
select 51659,'MRN' from dual union all
select 5166,'MVS' from dual union all
select 51660,'MRN' from dual union all
select 51661,'MRN' from dual union all
select 51662,'MRN' from dual union all
select 51663,'NASA' from dual union all
select 51664,'MRN' from dual union all
select 51665,'MRN' from dual union all
select 51666,'MRN' from dual union all
select 51667,'MRN' from dual union all
select 51668,'NASA' from dual union all
select 51669,'MRN' from dual union all
select 5167,'MVS' from dual union all
select 51670,'NASA' from dual union all
select 51671,'NASA' from dual union all
select 51672,'MRN' from dual union all
select 51673,'MRN' from dual union all
select 51674,'MRN' from dual union all
select 51675,'MRN' from dual union all
select 51676,'MRN' from dual union all
select 51677,'MRN' from dual union all
select 51678,'MRN' from dual union all
select 51679,'MRN' from dual union all
select 5168,'USAF' from dual union all
select 51680,'MRN' from dual union all
select 51681,'MRN' from dual union all
select 51682,'AN' from dual union all
select 51683,'MRN' from dual union all
select 51684,'MRN' from dual union all
select 51685,'MRN' from dual union all
select 51686,'MRN' from dual union all
select 51687,'MRN' from dual union all
select 51688,'MRN' from dual union all
select 51689,'MRN' from dual union all
select 5169,'NRL' from dual union all
select 51690,'MRN' from dual union all
select 51691,'MRN' from dual union all
select 51692,'MRN' from dual union all
select 51693,'MRN' from dual union all
select 51694,'MRN' from dual union all
select 51695,'MRN' from dual union all
select 51696,'MRN' from dual union all
select 51697,'MRN' from dual union all
select 51698,'MRN' from dual union all
select 51699,'MRN' from dual union all
select 517,'ORD' from dual union all
select 5170,'USAF' from dual union all
select 51700,'MRN' from dual union all
select 51701,'MRN' from dual union all
select 51702,'MRN' from dual union all
select 51703,'AN' from dual union all
select 51704,'JMA' from dual union all
select 51705,'MRN' from dual union all
select 51706,'MRN' from dual union all
select 51707,'MRN' from dual union all
select 51708,'MRN' from dual union all
select 51709,'MRN' from dual union all
select 5171,'USAF' from dual union all
select 51710,'JMA' from dual union all
select 51711,'MRN' from dual union all
select 51712,'MRN' from dual union all
select 51713,'MRN' from dual union all
select 51714,'MRN' from dual union all
select 51715,'MRN' from dual union all
select 51716,'MRN' from dual union all
select 51717,'MRN' from dual union all
select 51718,'MRN' from dual union all
select 51719,'MRN' from dual union all
select 5172,'USAF' from dual union all
select 51720,'MRN' from dual union all
select 51721,'AN' from dual union all
select 51722,'MRN' from dual union all
select 51723,'MRN' from dual union all
select 51724,'MRN' from dual union all
select 51725,'MRN' from dual union all
select 51726,'MRN' from dual union all
select 51727,'MRN' from dual union all
select 51728,'MRN' from dual union all
select 51729,'MRN' from dual union all
select 5173,'UNRV' from dual union all
select 51730,'MRN' from dual union all
select 51731,'MRN' from dual union all
select 51732,'MRN' from dual union all
select 51733,'MRN' from dual union all
select 51734,'MRN' from dual union all
select 51735,'MRN' from dual union all
select 51736,'MRN' from dual union all
select 51737,'MRN' from dual union all
select 51738,'MRN' from dual union all
select 51739,'MRN' from dual union all
select 5174,'USN' from dual union all
select 51740,'MRN' from dual union all
select 51741,'MRN' from dual union all
select 51742,'MRN' from dual union all
select 51743,'MRN' from dual union all
select 51744,'MRN' from dual union all
select 51745,'MRN' from dual union all
select 51746,'MRN' from dual;
insert into LAUNCH_AGENCY
select 51747,'MRN' from dual union all
select 51748,'MRN' from dual union all
select 51749,'MRN' from dual union all
select 5175,'RAE' from dual union all
select 51750,'JMA' from dual union all
select 51751,'MRN' from dual union all
select 51752,'MRN' from dual union all
select 51753,'MRN' from dual union all
select 51754,'MRN' from dual union all
select 51755,'MRN' from dual union all
select 51756,'NASA' from dual union all
select 51757,'MRN' from dual union all
select 51758,'MRN' from dual union all
select 51759,'NASA' from dual union all
select 5176,'LARC' from dual union all
select 51760,'MRN' from dual union all
select 51761,'MRN' from dual union all
select 51762,'MRN' from dual union all
select 51763,'MRN' from dual union all
select 51764,'MRN' from dual union all
select 51765,'MRN' from dual union all
select 51766,'MRN' from dual union all
select 51767,'MRN' from dual union all
select 51768,'MRN' from dual union all
select 51769,'MRN' from dual union all
select 5177,'USAF' from dual union all
select 51770,'MRN' from dual union all
select 51771,'MRN' from dual union all
select 51772,'MRN' from dual union all
select 51773,'MRN' from dual union all
select 51774,'AN' from dual union all
select 51775,'MRN' from dual union all
select 51776,'MRN' from dual union all
select 51777,'MRN' from dual union all
select 51778,'MRN' from dual union all
select 51779,'MRN' from dual union all
select 5178,'USAF' from dual union all
select 51780,'MRN' from dual union all
select 51781,'MRN' from dual union all
select 51782,'MRN' from dual union all
select 51783,'MRN' from dual union all
select 51784,'MRN' from dual union all
select 51785,'MRN' from dual union all
select 51786,'MRN' from dual union all
select 51787,'MRN' from dual union all
select 51788,'MRN' from dual union all
select 51789,'JMA' from dual union all
select 5179,'USAF' from dual union all
select 51790,'MRN' from dual union all
select 51791,'MRN' from dual union all
select 51792,'MRN' from dual union all
select 51793,'GMS' from dual union all
select 51794,'MRN' from dual union all
select 51795,'MRN' from dual union all
select 51796,'MRN' from dual union all
select 51797,'GMS' from dual union all
select 51798,'MRN' from dual union all
select 51799,'MRN' from dual union all
select 518,'ORD' from dual union all
select 5180,'NRL' from dual union all
select 51800,'MRN' from dual union all
select 51801,'MRN' from dual union all
select 51802,'GMS' from dual union all
select 51803,'MRN' from dual union all
select 51804,'MRN' from dual union all
select 51805,'MRN' from dual union all
select 51806,'MRN' from dual union all
select 51807,'MRN' from dual union all
select 51808,'MRN' from dual union all
select 51809,'MRN' from dual union all
select 5181,'USA' from dual union all
select 51810,'MRN' from dual union all
select 51811,'MRN' from dual union all
select 51812,'MRN' from dual union all
select 51813,'MRN' from dual union all
select 51814,'MRN' from dual union all
select 51815,'MRN' from dual union all
select 51816,'GMS' from dual union all
select 51817,'GMS' from dual union all
select 51818,'MRN' from dual union all
select 51819,'MRN' from dual union all
select 5182,'USAF' from dual union all
select 51820,'MRN' from dual union all
select 51821,'GMS' from dual union all
select 51822,'MRN' from dual union all
select 51823,'MRN' from dual union all
select 51824,'MRN' from dual union all
select 51825,'MRN' from dual union all
select 51826,'MRN' from dual union all
select 51827,'MRN' from dual union all
select 51828,'MRN' from dual union all
select 51829,'GMS' from dual union all
select 5183,'USAF' from dual union all
select 51830,'MRN' from dual union all
select 51831,'MRN' from dual union all
select 51832,'MRN' from dual union all
select 51833,'MRN' from dual union all
select 51834,'NASDA' from dual union all
select 51835,'MRN' from dual union all
select 51836,'NASDA' from dual;
insert into LAUNCH_AGENCY
select 51837,'MRN' from dual union all
select 51838,'MRN' from dual union all
select 51839,'MRN' from dual union all
select 5184,'AN' from dual union all
select 51840,'MRN' from dual union all
select 51841,'MRN' from dual union all
select 51842,'MRN' from dual union all
select 51843,'MRN' from dual union all
select 51844,'MRN' from dual union all
select 51845,'MRN' from dual union all
select 51846,'MRN' from dual union all
select 51847,'JMA' from dual union all
select 51848,'MRN' from dual union all
select 51849,'MRN' from dual union all
select 5185,'MVS' from dual union all
select 51850,'MRN' from dual union all
select 51851,'MRN' from dual union all
select 51852,'MRN' from dual union all
select 51853,'MRN' from dual union all
select 51854,'MRN' from dual union all
select 51855,'MRN' from dual union all
select 51856,'GMS' from dual union all
select 51857,'MRN' from dual union all
select 51858,'MRN' from dual union all
select 51859,'MRN' from dual union all
select 5186,'AFCRL' from dual union all
select 5186,'PARD' from dual union all
select 51860,'MRN' from dual union all
select 51861,'MRN' from dual union all
select 51862,'MRN' from dual union all
select 51863,'MRN' from dual union all
select 51864,'MRN' from dual union all
select 51865,'MRN' from dual union all
select 51866,'MRN' from dual union all
select 51867,'MRN' from dual union all
select 51868,'MRN' from dual union all
select 51869,'MRN' from dual union all
select 5187,'USAF' from dual union all
select 51870,'MRN' from dual union all
select 51871,'GMS' from dual union all
select 51872,'MRN' from dual union all
select 51873,'MRN' from dual union all
select 51874,'MRN' from dual union all
select 51875,'MRN' from dual union all
select 51876,'MRN' from dual union all
select 51877,'MRN' from dual union all
select 51878,'MRN' from dual union all
select 51879,'MRN' from dual union all
select 5188,'MVS' from dual union all
select 51880,'MRN' from dual union all
select 51881,'MRN' from dual union all
select 51882,'MRN' from dual union all
select 51883,'MRN' from dual union all
select 51884,'MRN' from dual union all
select 51885,'MRN' from dual union all
select 51886,'MRN' from dual union all
select 51887,'MRN' from dual union all
select 51888,'MRN' from dual union all
select 51889,'MRN' from dual union all
select 5189,'USN' from dual union all
select 51890,'MRN' from dual union all
select 51891,'MRN' from dual union all
select 51892,'MRN' from dual union all
select 51893,'MRN' from dual union all
select 51894,'MRN' from dual union all
select 51895,'MRN' from dual union all
select 51896,'MRN' from dual union all
select 51897,'MRN' from dual union all
select 51898,'JMA' from dual union all
select 51899,'GMS' from dual union all
select 519,'ORD' from dual union all
select 5190,'MVS' from dual union all
select 51900,'GMS' from dual union all
select 51901,'AN' from dual union all
select 51902,'MRN' from dual union all
select 51903,'MRN' from dual union all
select 51904,'MRN' from dual union all
select 51905,'MRN' from dual union all
select 51906,'MRN' from dual union all
select 51907,'MRN' from dual union all
select 51908,'MRN' from dual union all
select 51909,'MRN' from dual union all
select 5191,'USAF' from dual union all
select 51910,'GMS' from dual union all
select 51911,'MRN' from dual union all
select 51912,'MRN' from dual union all
select 51913,'MRN' from dual union all
select 51914,'MRN' from dual union all
select 51915,'WFC' from dual union all
select 51916,'MRN' from dual union all
select 51917,'WFC' from dual union all
select 51918,'MRN' from dual union all
select 51919,'MRN' from dual union all
select 5192,'MVS' from dual union all
select 51920,'MRN' from dual union all
select 51921,'MRN' from dual union all
select 51922,'MRN' from dual union all
select 51923,'JMA' from dual union all
select 51924,'MRN' from dual union all
select 51925,'GMS' from dual;
insert into LAUNCH_AGENCY
select 51926,'MRN' from dual union all
select 51927,'NASA' from dual union all
select 51928,'MRN' from dual union all
select 51929,'MRN' from dual union all
select 5193,'MVS' from dual union all
select 51930,'MRN' from dual union all
select 51931,'MRN' from dual union all
select 51932,'WFC' from dual union all
select 51933,'MRN' from dual union all
select 51934,'WFC' from dual union all
select 51935,'WFC' from dual union all
select 51936,'MRN' from dual union all
select 51937,'MRN' from dual union all
select 51938,'MRN' from dual union all
select 51939,'GMS' from dual union all
select 5194,'MVS' from dual union all
select 51940,'MRN' from dual union all
select 51941,'MRN' from dual union all
select 51942,'MRN' from dual union all
select 51943,'MRN' from dual union all
select 51944,'MRN' from dual union all
select 51945,'MRN' from dual union all
select 51946,'MRN' from dual union all
select 51947,'WFC' from dual union all
select 51948,'MRN' from dual union all
select 51949,'MRN' from dual union all
select 5195,'AN' from dual union all
select 51950,'MRN' from dual union all
select 51951,'NASA' from dual union all
select 51952,'MRN' from dual union all
select 51953,'NASA' from dual union all
select 51954,'MRN' from dual union all
select 51955,'GMS' from dual union all
select 51956,'MRN' from dual union all
select 51957,'MRN' from dual union all
select 51958,'MRN' from dual union all
select 51959,'WFC' from dual union all
select 5196,'MVS' from dual union all
select 51960,'MRN' from dual union all
select 51961,'WFC' from dual union all
select 51962,'WFC' from dual union all
select 51963,'MRN' from dual union all
select 51964,'MRN' from dual union all
select 51965,'MRN' from dual union all
select 51966,'MRN' from dual union all
select 51967,'JMA' from dual union all
select 51968,'GMS' from dual union all
select 51969,'AN' from dual union all
select 5197,'LARC' from dual union all
select 51970,'MRN' from dual union all
select 51971,'AN' from dual union all
select 51972,'MRN' from dual union all
select 51973,'MRN' from dual union all
select 51974,'MRN' from dual union all
select 51975,'MRN' from dual union all
select 51976,'MRN' from dual union all
select 51977,'GMS' from dual union all
select 51978,'MRN' from dual union all
select 51979,'MRN' from dual union all
select 5198,'MVS' from dual union all
select 51980,'MRN' from dual union all
select 51981,'MRN' from dual union all
select 51982,'MRN' from dual union all
select 51983,'MRN' from dual union all
select 51984,'MRN' from dual union all
select 51985,'MRN' from dual union all
select 51986,'AN' from dual union all
select 51987,'AN' from dual union all
select 51988,'AN' from dual union all
select 51989,'AN' from dual union all
select 5199,'AN' from dual union all
select 51990,'MRN' from dual union all
select 51991,'AN' from dual union all
select 51992,'AN' from dual union all
select 51993,'AN' from dual union all
select 51994,'AN' from dual union all
select 51995,'AN' from dual union all
select 51996,'AN' from dual union all
select 51997,'MRN' from dual union all
select 51998,'MRN' from dual union all
select 51999,'AN' from dual union all
select 52,'WER444' from dual union all
select 520,'WEHR' from dual union all
select 5200,'USAF' from dual union all
select 52000,'AN' from dual union all
select 52001,'AN' from dual union all
select 52002,'AN' from dual union all
select 52003,'AN' from dual union all
select 52004,'AN' from dual union all
select 52005,'AN' from dual union all
select 52006,'AN' from dual union all
select 52007,'AN' from dual union all
select 52008,'AN' from dual union all
select 52009,'AN' from dual union all
select 5201,'MVS' from dual union all
select 52010,'AN' from dual union all
select 52011,'AN' from dual union all
select 52012,'AN' from dual union all
select 52013,'AN' from dual union all
select 52014,'AN' from dual;
insert into LAUNCH_AGENCY
select 52015,'AN' from dual union all
select 52016,'AN' from dual union all
select 52017,'AN' from dual union all
select 52018,'AN' from dual union all
select 52019,'AN' from dual union all
select 5202,'USA' from dual union all
select 52020,'AN' from dual union all
select 52021,'AN' from dual union all
select 52022,'AN' from dual union all
select 52023,'AN' from dual union all
select 52024,'AN' from dual union all
select 52025,'MRN' from dual union all
select 52026,'AN' from dual union all
select 52027,'MRN' from dual union all
select 52028,'AN' from dual union all
select 52029,'AN' from dual union all
select 5203,'USN' from dual union all
select 52030,'AN' from dual union all
select 52031,'AN' from dual union all
select 52032,'AN' from dual union all
select 52033,'AN' from dual union all
select 52034,'MRN' from dual union all
select 52035,'AN' from dual union all
select 52036,'AN' from dual union all
select 52037,'AN' from dual union all
select 52038,'AN' from dual union all
select 52039,'AN' from dual union all
select 5204,'USN' from dual union all
select 52040,'AN' from dual union all
select 52041,'MRN' from dual union all
select 52042,'AN' from dual union all
select 52043,'AN' from dual union all
select 52044,'AN' from dual union all
select 52045,'MRN' from dual union all
select 52046,'AN' from dual union all
select 52047,'AN' from dual union all
select 52048,'MRN' from dual union all
select 52049,'AN' from dual union all
select 5205,'USN' from dual union all
select 52050,'MRN' from dual union all
select 52051,'MRN' from dual union all
select 52052,'AN' from dual union all
select 52053,'AN' from dual union all
select 52054,'AN' from dual union all
select 52055,'AN' from dual union all
select 52056,'AN' from dual union all
select 52057,'MRN' from dual union all
select 52058,'AN' from dual union all
select 52059,'MRN' from dual union all
select 5206,'USN' from dual union all
select 52060,'AN' from dual union all
select 52061,'AN' from dual union all
select 52062,'AN' from dual union all
select 52063,'AN' from dual union all
select 52064,'MRN' from dual union all
select 52065,'AN' from dual union all
select 52066,'AN' from dual union all
select 52067,'AN' from dual union all
select 52068,'AN' from dual union all
select 52069,'MRN' from dual union all
select 5207,'USN' from dual union all
select 52070,'AN' from dual union all
select 52071,'AN' from dual union all
select 52072,'AN' from dual union all
select 52073,'MRN' from dual union all
select 52074,'MRN' from dual union all
select 52075,'MRN' from dual union all
select 52076,'AN' from dual union all
select 52077,'AN' from dual union all
select 52078,'MRN' from dual union all
select 52079,'MRN' from dual union all
select 5208,'USN' from dual union all
select 52080,'AN' from dual union all
select 52081,'AN' from dual union all
select 52082,'MRN' from dual union all
select 52083,'AN' from dual union all
select 52084,'AN' from dual union all
select 52085,'MRN' from dual union all
select 52086,'AN' from dual union all
select 52087,'AN' from dual union all
select 52088,'AN' from dual union all
select 52089,'AN' from dual union all
select 5209,'USAF' from dual union all
select 52090,'AN' from dual union all
select 52091,'AN' from dual union all
select 52092,'AN' from dual union all
select 52093,'AN' from dual union all
select 52094,'AN' from dual union all
select 52095,'MRN' from dual union all
select 52096,'AN' from dual union all
select 52097,'AN' from dual union all
select 52098,'AN' from dual union all
select 52099,'AN' from dual union all
select 521,'WEHR' from dual union all
select 5210,'UNRV' from dual union all
select 52100,'AN' from dual union all
select 52101,'AN' from dual union all
select 52102,'AN' from dual union all
select 52103,'AN' from dual union all
select 52104,'AN' from dual;
insert into LAUNCH_AGENCY
select 52105,'AN' from dual union all
select 52106,'AN' from dual union all
select 52107,'MRN' from dual union all
select 52108,'AN' from dual union all
select 52109,'AN' from dual union all
select 5211,'USAF' from dual union all
select 52110,'AN' from dual union all
select 52111,'AN' from dual union all
select 52112,'AN' from dual union all
select 52113,'AN' from dual union all
select 52114,'AN' from dual union all
select 52115,'AN' from dual union all
select 52116,'MRN' from dual union all
select 52117,'AN' from dual union all
select 52118,'AN' from dual union all
select 52119,'AN' from dual union all
select 5212,'USAF' from dual union all
select 52120,'AN' from dual union all
select 52121,'AN' from dual union all
select 52122,'AN' from dual union all
select 52123,'AN' from dual union all
select 52124,'AN' from dual union all
select 52125,'AN' from dual union all
select 52126,'AN' from dual union all
select 52127,'AN' from dual union all
select 52128,'MRN' from dual union all
select 52129,'AN' from dual union all
select 5213,'USAF' from dual union all
select 52130,'AN' from dual union all
select 52131,'AN' from dual union all
select 52132,'MRN' from dual union all
select 52133,'AN' from dual union all
select 52134,'AN' from dual union all
select 52135,'AN' from dual union all
select 52136,'AN' from dual union all
select 52137,'AN' from dual union all
select 52138,'AN' from dual union all
select 52139,'AN' from dual union all
select 5214,'MVS' from dual union all
select 52140,'AN' from dual union all
select 52141,'AN' from dual union all
select 52142,'AN' from dual union all
select 52143,'AN' from dual union all
select 52144,'AN' from dual union all
select 52145,'AN' from dual union all
select 52146,'AN' from dual union all
select 52147,'MRN' from dual union all
select 52148,'AN' from dual union all
select 52149,'MRN' from dual union all
select 5215,'MVS' from dual union all
select 52150,'AN' from dual union all
select 52151,'AN' from dual union all
select 52152,'AN' from dual union all
select 52153,'MRN' from dual union all
select 52154,'AN' from dual union all
select 52155,'AN' from dual union all
select 52156,'AN' from dual union all
select 52157,'AN' from dual union all
select 52158,'AN' from dual union all
select 52159,'AN' from dual union all
select 5216,'USAF' from dual union all
select 52160,'AN' from dual union all
select 52161,'AN' from dual union all
select 52162,'AN' from dual union all
select 52163,'AN' from dual union all
select 52164,'MRN' from dual union all
select 52165,'MRN' from dual union all
select 52166,'MRN' from dual union all
select 52167,'AN' from dual union all
select 52168,'AN' from dual union all
select 52169,'AN' from dual union all
select 5217,'USAF' from dual union all
select 52170,'AN' from dual union all
select 52171,'AN' from dual union all
select 52172,'AN' from dual union all
select 52173,'MRN' from dual union all
select 52174,'AN' from dual union all
select 52175,'AN' from dual union all
select 52176,'AN' from dual union all
select 52177,'AN' from dual union all
select 52178,'AN' from dual union all
select 52179,'AN' from dual union all
select 5218,'USAF' from dual union all
select 52180,'AN' from dual union all
select 52181,'AN' from dual union all
select 52182,'AN' from dual union all
select 52183,'AN' from dual union all
select 52184,'AN' from dual union all
select 52185,'AN' from dual union all
select 52186,'AN' from dual union all
select 52187,'AN' from dual union all
select 52188,'AN' from dual union all
select 52189,'AN' from dual union all
select 5219,'USAF' from dual union all
select 52190,'AN' from dual union all
select 52191,'MRN' from dual union all
select 52192,'AN' from dual union all
select 52193,'MRN' from dual union all
select 52194,'AN' from dual union all
select 52195,'AN' from dual;
insert into LAUNCH_AGENCY
select 52196,'AN' from dual union all
select 52197,'AN' from dual union all
select 52198,'AN' from dual union all
select 52199,'MRN' from dual union all
select 522,'ORD' from dual union all
select 5220,'BRL' from dual union all
select 5220,'USAF' from dual union all
select 52200,'AN' from dual union all
select 52201,'MRN' from dual union all
select 52202,'MRN' from dual union all
select 52203,'AN' from dual union all
select 52204,'AN' from dual union all
select 52205,'AN' from dual union all
select 52206,'MRN' from dual union all
select 52207,'AN' from dual union all
select 52208,'AN' from dual union all
select 52209,'AN' from dual union all
select 5221,'MVS' from dual union all
select 52210,'AN' from dual union all
select 52211,'AN' from dual union all
select 52212,'AN' from dual union all
select 52213,'AN' from dual union all
select 52214,'AN' from dual union all
select 52215,'AN' from dual union all
select 52216,'MRN' from dual union all
select 52217,'AN' from dual union all
select 52218,'AN' from dual union all
select 52219,'AN' from dual union all
select 5222,'BRL' from dual union all
select 5222,'USAF' from dual union all
select 52220,'AN' from dual union all
select 52221,'MRN' from dual union all
select 52222,'MRN' from dual union all
select 52223,'AN' from dual union all
select 52224,'MRN' from dual union all
select 52225,'MRN' from dual union all
select 52226,'AN' from dual union all
select 52227,'AN' from dual union all
select 52228,'AN' from dual union all
select 52229,'AN' from dual union all
select 5223,'AN' from dual union all
select 52230,'MRN' from dual union all
select 52231,'AN' from dual union all
select 52232,'AN' from dual union all
select 52233,'MRN' from dual union all
select 52234,'AN' from dual union all
select 52235,'AN' from dual union all
select 52236,'AN' from dual union all
select 52237,'AN' from dual union all
select 52238,'AN' from dual union all
select 52239,'MRN' from dual union all
select 5224,'USAF' from dual union all
select 52240,'AN' from dual union all
select 52241,'AN' from dual union all
select 52242,'MRN' from dual union all
select 52243,'AN' from dual union all
select 52244,'AN' from dual union all
select 52245,'AN' from dual union all
select 52246,'MRN' from dual union all
select 52247,'MRN' from dual union all
select 52248,'AN' from dual union all
select 52249,'AN' from dual union all
select 5225,'MVS' from dual union all
select 52250,'AN' from dual union all
select 52251,'AN' from dual union all
select 52252,'AN' from dual union all
select 52253,'MRN' from dual union all
select 52254,'AN' from dual union all
select 52255,'AN' from dual union all
select 52256,'AN' from dual union all
select 52257,'AN' from dual union all
select 52258,'AN' from dual union all
select 52259,'AN' from dual union all
select 5226,'AN' from dual union all
select 52260,'AN' from dual union all
select 52261,'AN' from dual union all
select 52262,'AN' from dual union all
select 52263,'MRN' from dual union all
select 52264,'AN' from dual union all
select 52265,'AN' from dual union all
select 52266,'AN' from dual union all
select 52267,'AN' from dual union all
select 52268,'AN' from dual union all
select 52269,'GMS' from dual union all
select 5227,'NRL' from dual union all
select 52270,'AN' from dual union all
select 52271,'MRN' from dual union all
select 52272,'AN' from dual union all
select 52273,'MRN' from dual union all
select 52274,'MRN' from dual union all
select 52275,'AN' from dual union all
select 52276,'AN' from dual union all
select 52277,'MRN' from dual union all
select 52278,'AN' from dual union all
select 52279,'AN' from dual union all
select 5228,'USAF' from dual union all
select 52280,'GMS' from dual union all
select 52281,'AN' from dual union all
select 52282,'MRN' from dual union all
select 52283,'MRN' from dual;
insert into LAUNCH_AGENCY
select 52284,'AN' from dual union all
select 52285,'AN' from dual union all
select 52286,'AN' from dual union all
select 52287,'MRN' from dual union all
select 52288,'AN' from dual union all
select 52289,'MRN' from dual union all
select 5229,'RVSN' from dual union all
select 52290,'AN' from dual union all
select 52291,'AN' from dual union all
select 52292,'AN' from dual union all
select 52293,'AN' from dual union all
select 52294,'AN' from dual union all
select 52295,'AN' from dual union all
select 52296,'MRN' from dual union all
select 52297,'MRN' from dual union all
select 52298,'NASA' from dual union all
select 52299,'AN' from dual union all
select 523,'ORD' from dual union all
select 5230,'RVSN' from dual union all
select 52300,'AN' from dual union all
select 52301,'AN' from dual union all
select 52302,'AN' from dual union all
select 52303,'AN' from dual union all
select 52304,'AN' from dual union all
select 52305,'MRN' from dual union all
select 52306,'MRN' from dual union all
select 52307,'AN' from dual union all
select 52308,'GMS' from dual union all
select 52309,'AN' from dual union all
select 5231,'RVSN' from dual union all
select 52310,'MRN' from dual union all
select 52311,'AN' from dual union all
select 52312,'AN' from dual union all
select 52313,'AN' from dual union all
select 52314,'MRN' from dual union all
select 52315,'AN' from dual union all
select 52316,'AN' from dual union all
select 52317,'MRN' from dual union all
select 52318,'AN' from dual union all
select 52319,'AN' from dual union all
select 5232,'NRL' from dual union all
select 52320,'GMS' from dual union all
select 52321,'MRN' from dual union all
select 52322,'AN' from dual union all
select 52323,'AN' from dual union all
select 52324,'AN' from dual union all
select 52325,'AN' from dual union all
select 52326,'AN' from dual union all
select 52327,'AN' from dual union all
select 52328,'MRN' from dual union all
select 52329,'MRN' from dual union all
select 5233,'USAF' from dual union all
select 52330,'AN' from dual union all
select 52331,'AN' from dual union all
select 52332,'AN' from dual union all
select 52333,'MRN' from dual union all
select 52334,'AN' from dual union all
select 52335,'AN' from dual union all
select 52336,'MRN' from dual union all
select 52337,'AN' from dual union all
select 52338,'MRN' from dual union all
select 52339,'AN' from dual union all
select 5234,'NRL' from dual union all
select 52340,'AN' from dual union all
select 52341,'AN' from dual union all
select 52342,'AN' from dual union all
select 52343,'MRN' from dual union all
select 52344,'AN' from dual union all
select 52345,'MRN' from dual union all
select 52346,'AN' from dual union all
select 52347,'AN' from dual union all
select 52348,'MRN' from dual union all
select 52349,'AN' from dual union all
select 5235,'USAF' from dual union all
select 52350,'WFC' from dual union all
select 52351,'WFC' from dual union all
select 52352,'AN' from dual union all
select 52353,'AN' from dual union all
select 52354,'AN' from dual union all
select 52355,'AN' from dual union all
select 52356,'WFC' from dual union all
select 52357,'MRN' from dual union all
select 52358,'MRN' from dual union all
select 52359,'WFC' from dual union all
select 5236,'AN' from dual union all
select 52360,'WFC' from dual union all
select 52361,'WFC' from dual union all
select 52362,'AN' from dual union all
select 52363,'AN' from dual union all
select 52364,'AN' from dual union all
select 52365,'AN' from dual union all
select 52366,'AN' from dual union all
select 52367,'WFC' from dual union all
select 52368,'MRN' from dual union all
select 52369,'MRN' from dual union all
select 5237,'USA' from dual union all
select 52370,'WFC' from dual union all
select 52371,'AN' from dual union all
select 52372,'MRN' from dual union all
select 52373,'AN' from dual;
insert into LAUNCH_AGENCY
select 52374,'MRN' from dual union all
select 52375,'AFGL2' from dual union all
select 52376,'AN' from dual union all
select 52377,'MRN' from dual union all
select 52378,'AN' from dual union all
select 52379,'MRN' from dual union all
select 5238,'CARDE' from dual union all
select 52380,'AN' from dual union all
select 52381,'AN' from dual union all
select 52382,'AN' from dual union all
select 52383,'AN' from dual union all
select 52384,'MRN' from dual union all
select 52385,'AN' from dual union all
select 52386,'AN' from dual union all
select 52387,'AN' from dual union all
select 52388,'AN' from dual union all
select 52389,'MRN' from dual union all
select 5239,'CARDE' from dual union all
select 52390,'AN' from dual union all
select 52391,'MRN' from dual union all
select 52392,'AN' from dual union all
select 52393,'MRN' from dual union all
select 52394,'AN' from dual union all
select 52395,'AN' from dual union all
select 52396,'MRN' from dual union all
select 52397,'AN' from dual union all
select 52398,'AN' from dual union all
select 52399,'AN' from dual union all
select 524,'ORD' from dual union all
select 5240,'USAF' from dual union all
select 52400,'AN' from dual union all
select 52401,'AN' from dual union all
select 52402,'MRN' from dual union all
select 52403,'MRN' from dual union all
select 52404,'AN' from dual union all
select 52405,'GMS' from dual union all
select 52406,'GMS' from dual union all
select 52407,'GMS' from dual union all
select 52408,'AN' from dual union all
select 52409,'AN' from dual union all
select 5241,'NRL' from dual union all
select 52410,'AN' from dual union all
select 52411,'MRN' from dual union all
select 52412,'AN' from dual union all
select 52413,'GMS' from dual union all
select 52414,'AN' from dual union all
select 52415,'AN' from dual union all
select 52416,'AN' from dual union all
select 52417,'GMS' from dual union all
select 52418,'MRN' from dual union all
select 52419,'AN' from dual union all
select 5242,'USAF' from dual union all
select 52420,'AN' from dual union all
select 52421,'AN' from dual union all
select 52422,'MRN' from dual union all
select 52423,'AN' from dual union all
select 52424,'GMS' from dual union all
select 52425,'NASA' from dual union all
select 52426,'MRN' from dual union all
select 52427,'GMS' from dual union all
select 52428,'GMS' from dual union all
select 52429,'AN' from dual union all
select 5243,'USAF' from dual union all
select 52430,'AN' from dual union all
select 52431,'AN' from dual union all
select 52432,'MRN' from dual union all
select 52433,'MRN' from dual union all
select 52434,'AN' from dual union all
select 52435,'NASA' from dual union all
select 52436,'GMS' from dual union all
select 52437,'AN' from dual union all
select 52438,'AN' from dual union all
select 52439,'AN' from dual union all
select 5244,'USAF' from dual union all
select 52440,'MRN' from dual union all
select 52441,'AN' from dual union all
select 52442,'AN' from dual union all
select 52443,'AN' from dual union all
select 52444,'MRN' from dual union all
select 52445,'AN' from dual union all
select 52446,'AN' from dual union all
select 52447,'AN' from dual union all
select 52448,'ISRO' from dual union all
select 52449,'ISRO' from dual union all
select 5245,'USAF' from dual union all
select 52450,'NTNF' from dual union all
select 52451,'NTNF' from dual union all
select 52452,'NTNF' from dual union all
select 52453,'NTNF' from dual union all
select 52454,'NTNF' from dual union all
select 52455,'NTNF' from dual union all
select 52456,'AN' from dual union all
select 52457,'NTNF' from dual union all
select 52458,'NTNF' from dual union all
select 52459,'NTNF' from dual union all
select 5246,'AN' from dual union all
select 52460,'NTNF' from dual union all
select 52461,'NTNF' from dual union all
select 52462,'NTNF' from dual union all
select 52463,'NTNF' from dual;
insert into LAUNCH_AGENCY
select 52464,'NTNF' from dual union all
select 52465,'NTNF' from dual union all
select 52466,'AN' from dual union all
select 52467,'NTNF' from dual union all
select 52468,'NTNF' from dual union all
select 52469,'NTNF' from dual union all
select 5247,'USAF' from dual union all
select 52470,'NTNF' from dual union all
select 52471,'NTNF' from dual union all
select 52472,'NTNF' from dual union all
select 52473,'NTNF' from dual union all
select 52474,'NTNF' from dual union all
select 52475,'NTNF' from dual union all
select 52476,'NTNF' from dual union all
select 52477,'AN' from dual union all
select 52478,'NTNF' from dual union all
select 52479,'AN' from dual union all
select 5248,'SAND' from dual union all
select 52480,'MRN' from dual union all
select 52481,'AN' from dual union all
select 52482,'AN' from dual union all
select 52483,'AN' from dual union all
select 52484,'MRN' from dual union all
select 52485,'AN' from dual union all
select 52486,'AN' from dual union all
select 52487,'AN' from dual union all
select 52488,'AN' from dual union all
select 52489,'MRN' from dual union all
select 5249,'SAND' from dual union all
select 52490,'MRN' from dual union all
select 52491,'AN' from dual union all
select 52492,'AN' from dual union all
select 52493,'AN' from dual union all
select 52494,'MRN' from dual union all
select 52495,'AN' from dual union all
select 52496,'AN' from dual union all
select 52497,'AN' from dual union all
select 52498,'MRN' from dual union all
select 52499,'AN' from dual union all
select 525,'WER485-1' from dual union all
select 5250,'RVSN' from dual union all
select 52500,'AN' from dual union all
select 52501,'AN' from dual union all
select 52502,'AN' from dual union all
select 52503,'AN' from dual union all
select 52504,'MRN' from dual union all
select 52505,'MRN' from dual union all
select 52506,'AN' from dual union all
select 52507,'NASA' from dual union all
select 52508,'AN' from dual union all
select 52509,'ISRO' from dual union all
select 5251,'USA' from dual union all
select 52510,'NASA' from dual union all
select 52511,'ISRO' from dual union all
select 52512,'AN' from dual union all
select 52513,'AN' from dual union all
select 52514,'MRN' from dual union all
select 52515,'AN' from dual union all
select 52516,'AN' from dual union all
select 52517,'AN' from dual union all
select 52518,'AN' from dual union all
select 52519,'AN' from dual union all
select 5252,'USAF' from dual union all
select 52520,'AN' from dual union all
select 52521,'AN' from dual union all
select 52522,'AN' from dual union all
select 52523,'AN' from dual union all
select 52524,'RVSN' from dual union all
select 52525,'RVSN' from dual union all
select 52526,'RVSN' from dual union all
select 52527,'RVSN' from dual union all
select 52528,'RVSN' from dual union all
select 52529,'RVSN' from dual union all
select 5253,'NRL' from dual union all
select 52530,'RVSN' from dual union all
select 52531,'RVSN' from dual union all
select 52532,'RVSN' from dual union all
select 52533,'RVSN' from dual union all
select 52534,'RVSN' from dual union all
select 52535,'NASDA' from dual union all
select 52536,'RVSN' from dual union all
select 52537,'RVSN' from dual union all
select 52538,'RVSN' from dual union all
select 52539,'RVSN' from dual union all
select 5254,'USAF' from dual union all
select 52540,'ISAS' from dual union all
select 52541,'NASA' from dual union all
select 52542,'AFSC' from dual union all
select 52543,'RVSN' from dual union all
select 52544,'RVSN' from dual union all
select 52545,'RVSN' from dual union all
select 52546,'RVSN' from dual union all
select 52547,'RVSN' from dual union all
select 52548,'AFSC' from dual union all
select 52549,'RVSN' from dual union all
select 5255,'AN' from dual union all
select 52550,'RVSN' from dual union all
select 52551,'RVSN' from dual union all
select 52552,'RVSN' from dual union all
select 52553,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 52554,'RVSN' from dual union all
select 52555,'RVSN' from dual union all
select 52556,'RVSN' from dual union all
select 52557,'NASA' from dual union all
select 52558,'RVSN' from dual union all
select 52559,'RVSN' from dual union all
select 5256,'USAF' from dual union all
select 52560,'RVSN' from dual union all
select 52561,'AFSC' from dual union all
select 52562,'RVSN' from dual union all
select 52563,'RVSN' from dual union all
select 52564,'RVSN' from dual union all
select 52565,'RVSN' from dual union all
select 52566,'RVSN' from dual union all
select 52567,'NASA' from dual union all
select 52568,'RVSN' from dual union all
select 52569,'RVSN' from dual union all
select 5257,'NRL' from dual union all
select 52570,'RVSN' from dual union all
select 52571,'RVSN' from dual union all
select 52572,'NASA' from dual union all
select 52573,'NASA' from dual union all
select 52574,'ISRO' from dual union all
select 52575,'RVSN' from dual union all
select 52576,'RVSN' from dual union all
select 52577,'RVSN' from dual union all
select 52578,'RVSN' from dual union all
select 52579,'RVSN' from dual union all
select 5258,'MVS' from dual union all
select 52580,'ESA' from dual union all
select 52581,'RVSN' from dual union all
select 52582,'AFSC' from dual union all
select 52583,'RVSN' from dual union all
select 52584,'RVSN' from dual union all
select 52585,'RVSN' from dual union all
select 52586,'RVSN' from dual union all
select 52587,'RVSN' from dual union all
select 52588,'RVSN' from dual union all
select 52589,'RVSN' from dual union all
select 5259,'MVS' from dual union all
select 52590,'RVSN' from dual union all
select 52591,'RVSN' from dual union all
select 52592,'RVSN' from dual union all
select 52593,'NASA' from dual union all
select 52594,'RVSN' from dual union all
select 52595,'RVSN' from dual union all
select 52596,'NASA' from dual union all
select 52597,'RVSN' from dual union all
select 52598,'RVSN' from dual union all
select 52599,'NASDA' from dual union all
select 526,'WER485-3' from dual union all
select 5260,'BRL' from dual union all
select 5260,'USAF' from dual union all
select 52600,'RVSN' from dual union all
select 52601,'RVSN' from dual union all
select 52602,'RVSN' from dual union all
select 52603,'RVSN' from dual union all
select 52604,'RVSN' from dual union all
select 52605,'RVSN' from dual union all
select 52606,'RVSN' from dual union all
select 52607,'RVSN' from dual union all
select 52608,'AFSC' from dual union all
select 52609,'RVSN' from dual union all
select 5261,'USAF' from dual union all
select 52610,'RVSN' from dual union all
select 52611,'RVSN' from dual union all
select 52612,'RVSN' from dual union all
select 52613,'RVSN' from dual union all
select 52614,'RVSN' from dual union all
select 52615,'RVSN' from dual union all
select 52616,'SAST' from dual union all
select 52617,'RVSN' from dual union all
select 52618,'RVSN' from dual union all
select 52619,'NASA' from dual union all
select 5262,'BRL' from dual union all
select 5262,'NRL' from dual union all
select 52620,'RVSN' from dual union all
select 52621,'RVSN' from dual union all
select 52622,'RVSN' from dual union all
select 52623,'NASA' from dual union all
select 52624,'RVSN' from dual union all
select 52625,'RVSN' from dual union all
select 52626,'RVSN' from dual union all
select 52627,'RVSN' from dual union all
select 52628,'RVSN' from dual union all
select 52629,'RVSN' from dual union all
select 5263,'RAE' from dual union all
select 5263,'WRE' from dual union all
select 52630,'AFSC' from dual union all
select 52631,'RVSN' from dual union all
select 52632,'RVSN' from dual union all
select 52633,'RVSN' from dual union all
select 52634,'NASA' from dual union all
select 52635,'RVSN' from dual union all
select 52636,'RVSN' from dual union all
select 52637,'NASA' from dual union all
select 52638,'RVSN' from dual union all
select 52639,'RVSN' from dual union all
select 5264,'BRL' from dual union all
select 5264,'NRL' from dual;
insert into LAUNCH_AGENCY
select 52640,'RVSN' from dual union all
select 52641,'RVSN' from dual union all
select 52642,'NASA' from dual union all
select 52643,'RVSN' from dual union all
select 52644,'RVSN' from dual union all
select 52645,'ESA' from dual union all
select 52646,'RVSN' from dual union all
select 52647,'DMA' from dual union all
select 52648,'MRN' from dual union all
select 52649,'MRN' from dual union all
select 5265,'USAF' from dual union all
select 52650,'MRN' from dual union all
select 52651,'MRN' from dual union all
select 52652,'MRN' from dual union all
select 52653,'MRN' from dual union all
select 52654,'MRN' from dual union all
select 52655,'MRN' from dual union all
select 52656,'MRN' from dual union all
select 52657,'MRN' from dual union all
select 52658,'MRN' from dual union all
select 52659,'MRN' from dual union all
select 5266,'USAF' from dual union all
select 52660,'MRN' from dual union all
select 52661,'GMS' from dual union all
select 52662,'GMS' from dual union all
select 52663,'MRN' from dual union all
select 52664,'AN' from dual union all
select 52665,'MRN' from dual union all
select 52666,'MRN' from dual union all
select 52667,'MRN' from dual union all
select 52668,'RVSN' from dual union all
select 52669,'MRN' from dual union all
select 5267,'PARD' from dual union all
select 52670,'MRN' from dual union all
select 52671,'MRN' from dual union all
select 52672,'AN' from dual union all
select 52673,'MRN' from dual union all
select 52674,'MRN' from dual union all
select 52675,'USN' from dual union all
select 52676,'MRN' from dual union all
select 52677,'MRN' from dual union all
select 52678,'MRN' from dual union all
select 52679,'USAF' from dual union all
select 5268,'PARD' from dual union all
select 52680,'MRN' from dual union all
select 52681,'MRN' from dual union all
select 52682,'RVSN' from dual union all
select 52683,'PRO' from dual union all
select 52684,'RVSN' from dual union all
select 52685,'RVSN' from dual union all
select 52686,'RVSN' from dual union all
select 52687,'AFSC' from dual union all
select 52688,'NASA' from dual union all
select 52689,'OTR' from dual union all
select 5269,'RVSN' from dual union all
select 52690,'OTR' from dual union all
select 52691,'OTR' from dual union all
select 52692,'OTR' from dual union all
select 52693,'OTR' from dual union all
select 52694,'OTR' from dual union all
select 52695,'OTR' from dual union all
select 52696,'OTR' from dual union all
select 52697,'OTR' from dual union all
select 52698,'OTR' from dual union all
select 52699,'OTR' from dual union all
select 527,'SS500' from dual union all
select 5270,'MVS' from dual union all
select 52700,'OTR' from dual union all
select 52701,'OTR' from dual union all
select 52702,'NASA' from dual union all
select 52703,'DFVLR' from dual union all
select 52704,'NRCC' from dual union all
select 52705,'ISRO' from dual union all
select 52706,'ISRO' from dual union all
select 52707,'ISRO' from dual union all
select 52708,'ISRO' from dual union all
select 52709,'ISRO' from dual union all
select 5271,'ABMA' from dual union all
select 5271,'USAF' from dual union all
select 52710,'ISRO' from dual union all
select 52711,'ISRO' from dual union all
select 52712,'ISRO' from dual union all
select 52713,'ISRO' from dual union all
select 52714,'ISRO' from dual union all
select 52715,'ISRO' from dual union all
select 52716,'ISRO' from dual union all
select 52717,'ISRO' from dual union all
select 52718,'ISRO' from dual union all
select 52719,'ISRO' from dual union all
select 5272,'RVSN' from dual union all
select 52720,'ISRO' from dual union all
select 52721,'ISRO' from dual union all
select 52722,'ISRO' from dual union all
select 52723,'ISRO' from dual union all
select 52724,'ISRO' from dual union all
select 52725,'ISRO' from dual union all
select 52726,'ISRO' from dual union all
select 52727,'ISRO' from dual union all
select 52728,'ISRO' from dual union all
select 52729,'ISRO' from dual;
insert into LAUNCH_AGENCY
select 5273,'USAF' from dual union all
select 52730,'ISRO' from dual union all
select 52731,'ISRO' from dual union all
select 52732,'ISRO' from dual union all
select 52733,'ISRO' from dual union all
select 52734,'ISRO' from dual union all
select 52735,'ISRO' from dual union all
select 52736,'ISRO' from dual union all
select 52737,'ISRO' from dual union all
select 52738,'ISRO' from dual union all
select 52739,'ISRO' from dual union all
select 5274,'USAF' from dual union all
select 52740,'ISRO' from dual union all
select 52741,'ISRO' from dual union all
select 52742,'ISRO' from dual union all
select 52743,'ISRO' from dual union all
select 52744,'ISRO' from dual union all
select 52745,'ISRO' from dual union all
select 52746,'ISRO' from dual union all
select 52747,'ISRO' from dual union all
select 52748,'ISRO' from dual union all
select 52749,'ISRO' from dual union all
select 5275,'SAND' from dual union all
select 52750,'ISRO' from dual union all
select 52751,'ISRO' from dual union all
select 52752,'ISRO' from dual union all
select 52753,'ISRO' from dual union all
select 52754,'ISRO' from dual union all
select 52755,'ISRO' from dual union all
select 52756,'ISRO' from dual union all
select 52757,'ISRO' from dual union all
select 52758,'ISRO' from dual union all
select 52759,'ISRO' from dual union all
select 5276,'MVS' from dual union all
select 52760,'ISRO' from dual union all
select 52761,'ISRO' from dual union all
select 52762,'ISRO' from dual union all
select 52763,'ISRO' from dual union all
select 52764,'ISRO' from dual union all
select 52765,'ISRO' from dual union all
select 52766,'ISRO' from dual union all
select 52767,'ISRO' from dual union all
select 52768,'ISRO' from dual union all
select 52769,'ISRO' from dual union all
select 5277,'SAND' from dual union all
select 52770,'ISRO' from dual union all
select 52771,'ISRO' from dual union all
select 52772,'ISRO' from dual union all
select 52773,'ISRO' from dual union all
select 52774,'RVSN' from dual union all
select 52775,'DMA' from dual union all
select 52776,'DMA' from dual union all
select 52777,'DMA' from dual union all
select 52778,'RVSN' from dual union all
select 52779,'RVSN' from dual union all
select 5278,'SAND' from dual union all
select 52780,'RVSN' from dual union all
select 52781,'RVSN' from dual union all
select 52782,'RVSN' from dual union all
select 52783,'RVSN' from dual union all
select 52784,'RVSN' from dual union all
select 52785,'NASA' from dual union all
select 52786,'AN' from dual union all
select 52787,'AN' from dual union all
select 52788,'AN' from dual union all
select 52789,'AN' from dual union all
select 5279,'USAF' from dual union all
select 52790,'DFVLR' from dual union all
select 52791,'NASA' from dual union all
select 52792,'NASA' from dual union all
select 52793,'DFVLR' from dual union all
select 52794,'RVSN' from dual union all
select 52795,'NASA' from dual union all
select 52796,'AN' from dual union all
select 52797,'AN' from dual union all
select 52798,'AN' from dual union all
select 52799,'AN' from dual union all
select 528,'WEHR' from dual union all
select 5280,'AN' from dual union all
select 52800,'DMA' from dual union all
select 52801,'SAC' from dual union all
select 52802,'USAF' from dual union all
select 52803,'USN' from dual union all
select 52804,'USN' from dual union all
select 52805,'RVSN' from dual union all
select 52806,'USN' from dual union all
select 52807,'USN' from dual union all
select 52808,'ALCS' from dual union all
select 52808,'SAC' from dual union all
select 52809,'AN' from dual union all
select 5281,'MVS' from dual union all
select 52810,'AN' from dual union all
select 52811,'AN' from dual union all
select 52812,'USN' from dual union all
select 52813,'USN' from dual union all
select 52814,'USN' from dual union all
select 52815,'USN' from dual union all
select 52816,'RVSN' from dual union all
select 52817,'NASA' from dual union all
select 52818,'NASA' from dual;
insert into LAUNCH_AGENCY
select 52819,'AN' from dual union all
select 5282,'MVS' from dual union all
select 52820,'AN' from dual union all
select 52821,'AN' from dual union all
select 52822,'AN' from dual union all
select 52823,'AFGL2' from dual union all
select 52824,'NASDA' from dual union all
select 52825,'NASA' from dual union all
select 52826,'NASA' from dual union all
select 52827,'RVSN' from dual union all
select 52828,'USN' from dual union all
select 52829,'USN' from dual union all
select 5283,'USAF' from dual union all
select 52830,'USN' from dual union all
select 52831,'USN' from dual union all
select 52832,'NASA' from dual union all
select 52833,'USN' from dual union all
select 52834,'NASA' from dual union all
select 52835,'ISAS' from dual union all
select 52836,'NASA' from dual union all
select 52837,'AN' from dual union all
select 52838,'RVSN' from dual union all
select 52839,'USN' from dual union all
select 5284,'AN' from dual union all
select 52840,'USN' from dual union all
select 52841,'USN' from dual union all
select 52842,'USN' from dual union all
select 52843,'ISAS' from dual union all
select 52844,'ISAS' from dual union all
select 52845,'AFSC' from dual union all
select 52846,'RVSN' from dual union all
select 52847,'NASA' from dual union all
select 52848,'SSC' from dual union all
select 52849,'RVSN' from dual union all
select 5285,'MVS' from dual union all
select 52850,'AFSC' from dual union all
select 52851,'NASA' from dual union all
select 52852,'NASA' from dual union all
select 52853,'NASA' from dual union all
select 52854,'USA' from dual union all
select 52855,'NDRE' from dual union all
select 52856,'AFGL2' from dual union all
select 52857,'AFGL2' from dual union all
select 52858,'NASA' from dual union all
select 52859,'NASA' from dual union all
select 5286,'MVS' from dual union all
select 52860,'RVSN' from dual union all
select 52861,'NASA' from dual union all
select 52862,'NASA' from dual union all
select 52863,'SAC' from dual union all
select 52864,'RVSN' from dual union all
select 52865,'SRC' from dual union all
select 52866,'NASA' from dual union all
select 52867,'NRCC' from dual union all
select 52868,'CALT' from dual union all
select 52868,'PLA2AC' from dual union all
select 52869,'NASA' from dual union all
select 5287,'MVS' from dual union all
select 52870,'NASA' from dual union all
select 52871,'RVSN' from dual union all
select 52872,'SAC' from dual union all
select 52873,'SRC' from dual union all
select 52874,'DFVLR' from dual union all
select 52875,'IAE' from dual union all
select 52876,'NASA' from dual union all
select 52877,'SRC' from dual union all
select 52878,'INTA' from dual union all
select 52879,'RVSN' from dual union all
select 5288,'MVS' from dual union all
select 52880,'ISRO' from dual union all
select 52881,'ISRO' from dual union all
select 52882,'RVSN' from dual union all
select 52883,'ISRO' from dual union all
select 52884,'RVSN' from dual union all
select 52885,'RVSN' from dual union all
select 52886,'RVSN' from dual union all
select 52887,'RVSN' from dual union all
select 52888,'RVSN' from dual union all
select 52889,'IQ224' from dual union all
select 5289,'MVS' from dual union all
select 52890,'IQ224' from dual union all
select 52891,'IQ224' from dual union all
select 52892,'RVSN' from dual union all
select 52893,'IQ224' from dual union all
select 52894,'IQ224' from dual union all
select 52895,'IQ224' from dual union all
select 52896,'IQ224' from dual union all
select 52897,'IQ224' from dual union all
select 52898,'IQ224' from dual union all
select 52899,'IQ224' from dual union all
select 529,'WEHR' from dual union all
select 5290,'6555A' from dual union all
select 52900,'IQ224' from dual union all
select 52901,'IQ224' from dual union all
select 52902,'FKG' from dual union all
select 52903,'RVSN' from dual union all
select 52904,'FKG' from dual union all
select 52905,'FKG' from dual union all
select 52906,'FKG' from dual union all
select 52907,'USA' from dual;
insert into LAUNCH_AGENCY
select 52908,'USA' from dual union all
select 52909,'USA' from dual union all
select 5291,'UNRV' from dual union all
select 52910,'RVSN' from dual union all
select 52911,'RVSN' from dual union all
select 52912,'RVSN' from dual union all
select 52913,'RVSN' from dual union all
select 52914,'RVSN' from dual union all
select 52915,'RVSN' from dual union all
select 52916,'RVSN' from dual union all
select 52917,'RVSN' from dual union all
select 52918,'RVSN' from dual union all
select 52919,'RVSN' from dual union all
select 5292,'UNRV' from dual union all
select 52920,'RVSN' from dual union all
select 52921,'RVSN' from dual union all
select 52922,'RVSN' from dual union all
select 52923,'RVSN' from dual union all
select 52924,'RVSN' from dual union all
select 52925,'RVSN' from dual union all
select 52926,'RVSN' from dual union all
select 52927,'RVSN' from dual union all
select 52928,'RVSN' from dual union all
select 52929,'RVSN' from dual union all
select 5293,'UNRV' from dual union all
select 52930,'RVSN' from dual union all
select 52931,'RVSN' from dual union all
select 52932,'RVSN' from dual union all
select 52933,'RVSN' from dual union all
select 52934,'RVSN' from dual union all
select 52935,'RVSN' from dual union all
select 52936,'RVSN' from dual union all
select 52937,'RVSN' from dual union all
select 52938,'RVSN' from dual union all
select 52939,'RVSN' from dual union all
select 5294,'UNRV' from dual union all
select 52940,'RVSN' from dual union all
select 52941,'RVSN' from dual union all
select 52942,'INTA' from dual union all
select 52943,'INTA' from dual union all
select 52944,'INTA' from dual union all
select 52945,'RVSN' from dual union all
select 52946,'IIAE' from dual union all
select 52947,'RVSN' from dual union all
select 52948,'RVSN' from dual union all
select 52949,'RVSN' from dual union all
select 5295,'UNRV' from dual union all
select 52950,'RVSN' from dual union all
select 52951,'MVS' from dual union all
select 52952,'MVS' from dual union all
select 52953,'MVS' from dual union all
select 52954,'VMF' from dual union all
select 52955,'VMF' from dual union all
select 52956,'VMF' from dual union all
select 52957,'VMF' from dual union all
select 52958,'VMF' from dual union all
select 52959,'VMF' from dual union all
select 5296,'UNRV' from dual union all
select 52960,'VMF' from dual union all
select 52961,'RVSN' from dual union all
select 52962,'RVSN' from dual union all
select 52963,'RVSN' from dual union all
select 52964,'RVSN' from dual union all
select 52965,'RVSN' from dual union all
select 52966,'RVSN' from dual union all
select 52967,'RVSN' from dual union all
select 52968,'RVSN' from dual union all
select 52969,'RVSN' from dual union all
select 5297,'NRL' from dual union all
select 52970,'RVSN' from dual union all
select 52971,'RVSN' from dual union all
select 52972,'RVSN' from dual union all
select 52973,'RVSN' from dual union all
select 52974,'RVSN' from dual union all
select 52975,'RVSN' from dual union all
select 52976,'RVSN' from dual union all
select 52977,'NASA' from dual union all
select 52978,'NASDA' from dual union all
select 52979,'NRCC' from dual union all
select 5298,'UNRV' from dual union all
select 52980,'NASA' from dual union all
select 52981,'NASA' from dual union all
select 52982,'ISAS' from dual union all
select 52983,'NRCC' from dual union all
select 52984,'NASA' from dual union all
select 52985,'ISAS' from dual union all
select 52986,'AFGL2' from dual union all
select 52987,'USA' from dual union all
select 52988,'USA' from dual union all
select 52989,'USA' from dual union all
select 5299,'UNRV' from dual union all
select 52990,'AFGL2' from dual union all
select 52991,'SRC' from dual union all
select 52992,'SSC' from dual union all
select 52993,'SAC' from dual union all
select 52994,'ALCS' from dual union all
select 52994,'SAC' from dual union all
select 52995,'NASA' from dual union all
select 52996,'USA' from dual union all
select 52997,'USA' from dual;
insert into LAUNCH_AGENCY
select 52998,'USA' from dual union all
select 52999,'INTA' from dual union all
select 53,'WER500' from dual union all
select 530,'WEHR' from dual union all
select 5300,'UNRV' from dual union all
select 53000,'AFSC' from dual union all
select 53001,'SAC' from dual union all
select 53002,'AFBMO' from dual union all
select 53003,'OTR' from dual union all
select 53004,'AFBMO' from dual union all
select 53005,'AFGL' from dual union all
select 53006,'AFGL' from dual union all
select 53007,'AFGL' from dual union all
select 53008,'AFGL2' from dual union all
select 53009,'NASA' from dual union all
select 5301,'UNRV' from dual union all
select 53010,'AFBMO' from dual union all
select 53011,'NASA' from dual union all
select 53012,'AFSC' from dual union all
select 53013,'NASA' from dual union all
select 53014,'NASA' from dual union all
select 53015,'NRCC' from dual union all
select 53016,'NRCC' from dual union all
select 53017,'DNA' from dual union all
select 53018,'SAC' from dual union all
select 53019,'NASA' from dual union all
select 5302,'UNRV' from dual union all
select 53020,'USN' from dual union all
select 53021,'NASA' from dual union all
select 53022,'NASA' from dual union all
select 53023,'DNA' from dual union all
select 53024,'AFSC' from dual union all
select 53025,'USA' from dual union all
select 53026,'USA' from dual union all
select 53027,'USA' from dual union all
select 53028,'MRN' from dual union all
select 53029,'MRN' from dual union all
select 5303,'UNRV' from dual union all
select 53030,'MRN' from dual union all
select 53031,'JMA' from dual union all
select 53032,'MRN' from dual union all
select 53033,'MRN' from dual union all
select 53034,'MRN' from dual union all
select 53035,'MRN' from dual union all
select 53036,'MRN' from dual union all
select 53037,'MRN' from dual union all
select 53038,'GMS' from dual union all
select 53039,'GMS' from dual union all
select 5304,'UNRV' from dual union all
select 53040,'MRN' from dual union all
select 53041,'MRN' from dual union all
select 53042,'MRN' from dual union all
select 53043,'GMS' from dual union all
select 53044,'MRN' from dual union all
select 53045,'GMS' from dual union all
select 53046,'MRN' from dual union all
select 53047,'MRN' from dual union all
select 53048,'JMA' from dual union all
select 53049,'MRN' from dual union all
select 5305,'UNRV' from dual union all
select 53050,'MRN' from dual union all
select 53051,'MRN' from dual union all
select 53052,'GMS' from dual union all
select 53053,'MRN' from dual union all
select 53054,'MRN' from dual union all
select 53055,'GMS' from dual union all
select 53056,'MRN' from dual union all
select 53057,'MRN' from dual union all
select 53058,'MRN' from dual union all
select 53059,'AN' from dual union all
select 5306,'UNRV' from dual union all
select 53060,'JMA' from dual union all
select 53061,'MRN' from dual union all
select 53062,'NASA' from dual union all
select 53063,'MRN' from dual union all
select 53064,'MRN' from dual union all
select 53065,'NASA' from dual union all
select 53066,'MRN' from dual union all
select 53067,'MRN' from dual union all
select 53068,'MRN' from dual union all
select 53069,'MRN' from dual union all
select 5307,'UNRV' from dual union all
select 53070,'GMS' from dual union all
select 53071,'MRN' from dual union all
select 53072,'MRN' from dual union all
select 53073,'AN' from dual union all
select 53074,'MRN' from dual union all
select 53075,'MRN' from dual union all
select 53076,'MRN' from dual union all
select 53077,'MRN' from dual union all
select 53078,'MRN' from dual union all
select 53079,'MRN' from dual union all
select 5308,'USAF' from dual union all
select 53080,'AN' from dual union all
select 53081,'MRN' from dual union all
select 53082,'MRN' from dual union all
select 53083,'MRN' from dual union all
select 53084,'MRN' from dual union all
select 53085,'MRN' from dual union all
select 53086,'MRN' from dual;
insert into LAUNCH_AGENCY
select 53087,'MRN' from dual union all
select 53088,'MRN' from dual union all
select 53089,'MRN' from dual union all
select 5309,'UNRV' from dual union all
select 53090,'MRN' from dual union all
select 53091,'MRN' from dual union all
select 53092,'GMS' from dual union all
select 53093,'MRN' from dual union all
select 53094,'AN' from dual union all
select 53095,'MRN' from dual union all
select 53096,'MRN' from dual union all
select 53097,'MRN' from dual union all
select 53098,'MRN' from dual union all
select 53099,'MRN' from dual union all
select 531,'WEHR' from dual union all
select 5310,'UNRV' from dual union all
select 53100,'MRN' from dual union all
select 53101,'MRN' from dual union all
select 53102,'JMA' from dual union all
select 53103,'GMS' from dual union all
select 53104,'MRN' from dual union all
select 53105,'MRN' from dual union all
select 53106,'MRN' from dual union all
select 53107,'MRN' from dual union all
select 53108,'MRN' from dual union all
select 53109,'MRN' from dual union all
select 5311,'UNRV' from dual union all
select 53110,'MRN' from dual union all
select 53111,'MRN' from dual union all
select 53112,'MRN' from dual union all
select 53113,'MRN' from dual union all
select 53114,'MRN' from dual union all
select 53115,'MRN' from dual union all
select 53116,'MRN' from dual union all
select 53117,'MRN' from dual union all
select 53118,'MRN' from dual union all
select 53119,'MRN' from dual union all
select 5312,'UNRV' from dual union all
select 53120,'JMA' from dual union all
select 53121,'AN' from dual union all
select 53122,'MRN' from dual union all
select 53123,'MRN' from dual union all
select 53124,'MRN' from dual union all
select 53125,'MRN' from dual union all
select 53126,'MRN' from dual union all
select 53127,'AN' from dual union all
select 53128,'MRN' from dual union all
select 53129,'MRN' from dual union all
select 5313,'UNRV' from dual union all
select 53130,'MRN' from dual union all
select 53131,'MRN' from dual union all
select 53132,'MRN' from dual union all
select 53133,'MRN' from dual union all
select 53134,'GMS' from dual union all
select 53135,'MRN' from dual union all
select 53136,'GMS' from dual union all
select 53137,'MRN' from dual union all
select 53138,'MRN' from dual union all
select 53139,'MRN' from dual union all
select 5314,'UNRV' from dual union all
select 53140,'MRN' from dual union all
select 53141,'MRN' from dual union all
select 53142,'MRN' from dual union all
select 53143,'MRN' from dual union all
select 53144,'MRN' from dual union all
select 53145,'JMA' from dual union all
select 53146,'GMS' from dual union all
select 53147,'MRN' from dual union all
select 53148,'MRN' from dual union all
select 53149,'GMS' from dual union all
select 5315,'UNRV' from dual union all
select 53150,'MRN' from dual union all
select 53151,'MRN' from dual union all
select 53152,'AN' from dual union all
select 53153,'MRN' from dual union all
select 53154,'MRN' from dual union all
select 53155,'MRN' from dual union all
select 53156,'MRN' from dual union all
select 53157,'MRN' from dual union all
select 53158,'AN' from dual union all
select 53159,'MRN' from dual union all
select 5316,'UNRV' from dual union all
select 53160,'MRN' from dual union all
select 53161,'MRN' from dual union all
select 53162,'MRN' from dual union all
select 53163,'MRN' from dual union all
select 53164,'MRN' from dual union all
select 53165,'MRN' from dual union all
select 53166,'MRN' from dual union all
select 53167,'MRN' from dual union all
select 53168,'GMS' from dual union all
select 53169,'MRN' from dual union all
select 5317,'UNRV' from dual union all
select 53170,'MRN' from dual union all
select 53171,'MRN' from dual union all
select 53172,'MRN' from dual union all
select 53173,'MRN' from dual union all
select 53174,'MRN' from dual union all
select 53175,'GMS' from dual union all
select 53176,'MRN' from dual;
insert into LAUNCH_AGENCY
select 53177,'MRN' from dual union all
select 53178,'MRN' from dual union all
select 53179,'MRN' from dual union all
select 5318,'UNRV' from dual union all
select 53180,'GMS' from dual union all
select 53181,'MRN' from dual union all
select 53182,'MRN' from dual union all
select 53183,'JMA' from dual union all
select 53184,'AN' from dual union all
select 53185,'MRN' from dual union all
select 53186,'GMS' from dual union all
select 53187,'MRN' from dual union all
select 53188,'MRN' from dual union all
select 53189,'MRN' from dual union all
select 5319,'NRL' from dual union all
select 53190,'MRN' from dual union all
select 53191,'GMS' from dual union all
select 53192,'MRN' from dual union all
select 53193,'MRN' from dual union all
select 53194,'MRN' from dual union all
select 53195,'MRN' from dual union all
select 53196,'MRN' from dual union all
select 53197,'MRN' from dual union all
select 53198,'MRN' from dual union all
select 53199,'MRN' from dual union all
select 532,'WEHR' from dual union all
select 5320,'UNRV' from dual union all
select 53200,'MRN' from dual union all
select 53201,'MRN' from dual union all
select 53202,'AN' from dual union all
select 53203,'MRN' from dual union all
select 53204,'MRN' from dual union all
select 53205,'MRN' from dual union all
select 53206,'MRN' from dual union all
select 53207,'MRN' from dual union all
select 53208,'MRN' from dual union all
select 53209,'MRN' from dual union all
select 5321,'UNRV' from dual union all
select 53210,'MRN' from dual union all
select 53211,'MRN' from dual union all
select 53212,'MRN' from dual union all
select 53213,'GMS' from dual union all
select 53214,'MRN' from dual union all
select 53215,'MRN' from dual union all
select 53216,'MRN' from dual union all
select 53217,'MRN' from dual union all
select 53218,'MRN' from dual union all
select 53219,'MRN' from dual union all
select 5322,'UNRV' from dual union all
select 53220,'MRN' from dual union all
select 53221,'MRN' from dual union all
select 53222,'MRN' from dual union all
select 53223,'MRN' from dual union all
select 53224,'MRN' from dual union all
select 53225,'MRN' from dual union all
select 53226,'MRN' from dual union all
select 53227,'MRN' from dual union all
select 53228,'MRN' from dual union all
select 53229,'AN' from dual union all
select 5323,'UNRV' from dual union all
select 53230,'MRN' from dual union all
select 53231,'MRN' from dual union all
select 53232,'JMA' from dual union all
select 53233,'MRN' from dual union all
select 53234,'MRN' from dual union all
select 53235,'AN' from dual union all
select 53236,'MRN' from dual union all
select 53237,'MRN' from dual union all
select 53238,'MRN' from dual union all
select 53239,'MRN' from dual union all
select 5324,'UNRV' from dual union all
select 53240,'MRN' from dual union all
select 53241,'MRN' from dual union all
select 53242,'MRN' from dual union all
select 53243,'MRN' from dual union all
select 53244,'JMA' from dual union all
select 53245,'GMS' from dual union all
select 53246,'AN' from dual union all
select 53247,'GMS' from dual union all
select 53248,'MRN' from dual union all
select 53249,'MRN' from dual union all
select 5325,'UNRV' from dual union all
select 53250,'MRN' from dual union all
select 53251,'MRN' from dual union all
select 53252,'MRN' from dual union all
select 53253,'MRN' from dual union all
select 53254,'MRN' from dual union all
select 53255,'MRN' from dual union all
select 53256,'MRN' from dual union all
select 53257,'MRN' from dual union all
select 53258,'GMS' from dual union all
select 53259,'MRN' from dual union all
select 5326,'UNRV' from dual union all
select 53260,'MRN' from dual union all
select 53261,'GMS' from dual union all
select 53262,'MRN' from dual union all
select 53263,'GMS' from dual union all
select 53264,'GMS' from dual union all
select 53265,'MRN' from dual union all
select 53266,'JMA' from dual;
insert into LAUNCH_AGENCY
select 53267,'MRN' from dual union all
select 53268,'JMA' from dual union all
select 53269,'MRN' from dual union all
select 5327,'UNRV' from dual union all
select 53270,'MRN' from dual union all
select 53271,'MRN' from dual union all
select 53272,'MRN' from dual union all
select 53273,'MRN' from dual union all
select 53274,'MRN' from dual union all
select 53275,'GMS' from dual union all
select 53276,'MRN' from dual union all
select 53277,'MRN' from dual union all
select 53278,'MRN' from dual union all
select 53279,'MRN' from dual union all
select 5328,'UNRV' from dual union all
select 53280,'MRN' from dual union all
select 53281,'MRN' from dual union all
select 53282,'MRN' from dual union all
select 53283,'GMS' from dual union all
select 53284,'MRN' from dual union all
select 53285,'MRN' from dual union all
select 53286,'MRN' from dual union all
select 53287,'GMS' from dual union all
select 53288,'MRN' from dual union all
select 53289,'MRN' from dual union all
select 5329,'UNRV' from dual union all
select 53290,'NASA' from dual union all
select 53291,'MRN' from dual union all
select 53292,'MRN' from dual union all
select 53293,'MRN' from dual union all
select 53294,'MRN' from dual union all
select 53295,'MRN' from dual union all
select 53296,'MRN' from dual union all
select 53297,'AN' from dual union all
select 53298,'JMA' from dual union all
select 53299,'MRN' from dual union all
select 533,'WEHR' from dual union all
select 5330,'USAF' from dual union all
select 53300,'MRN' from dual union all
select 53301,'MRN' from dual union all
select 53302,'MRN' from dual union all
select 53303,'MRN' from dual union all
select 53304,'MRN' from dual union all
select 53305,'MRN' from dual union all
select 53306,'MRN' from dual union all
select 53307,'MRN' from dual union all
select 53308,'GMS' from dual union all
select 53309,'MRN' from dual union all
select 5331,'UNRV' from dual union all
select 53310,'MRN' from dual union all
select 53311,'MRN' from dual union all
select 53312,'AN' from dual union all
select 53313,'MRN' from dual union all
select 53314,'AN' from dual union all
select 53315,'AN' from dual union all
select 53316,'MRN' from dual union all
select 53317,'JMA' from dual union all
select 53318,'GMS' from dual union all
select 53319,'MRN' from dual union all
select 5332,'UNRV' from dual union all
select 53320,'AN' from dual union all
select 53321,'MRN' from dual union all
select 53322,'MRN' from dual union all
select 53323,'MRN' from dual union all
select 53324,'MRN' from dual union all
select 53325,'MRN' from dual union all
select 53326,'GMS' from dual union all
select 53327,'MRN' from dual union all
select 53328,'MRN' from dual union all
select 53329,'MRN' from dual union all
select 5333,'UNRV' from dual union all
select 53330,'GMS' from dual union all
select 53331,'MRN' from dual union all
select 53332,'MRN' from dual union all
select 53333,'AN' from dual union all
select 53334,'AN' from dual union all
select 53335,'MRN' from dual union all
select 53336,'MRN' from dual union all
select 53337,'JMA' from dual union all
select 53338,'GMS' from dual union all
select 53339,'MRN' from dual union all
select 5334,'UNRV' from dual union all
select 53340,'AN' from dual union all
select 53341,'MRN' from dual union all
select 53342,'AN' from dual union all
select 53343,'MRN' from dual union all
select 53344,'MRN' from dual union all
select 53345,'MRN' from dual union all
select 53346,'MRN' from dual union all
select 53347,'MRN' from dual union all
select 53348,'MRN' from dual union all
select 53349,'GMS' from dual union all
select 5335,'UNRV' from dual union all
select 53350,'MRN' from dual union all
select 53351,'MRN' from dual union all
select 53352,'MRN' from dual union all
select 53353,'MRN' from dual union all
select 53354,'MRN' from dual union all
select 53355,'MRN' from dual union all
select 53356,'MRN' from dual;
insert into LAUNCH_AGENCY
select 53357,'MRN' from dual union all
select 53358,'MRN' from dual union all
select 53359,'MRN' from dual union all
select 5336,'UNRV' from dual union all
select 53360,'MRN' from dual union all
select 53361,'MRN' from dual union all
select 53362,'MRN' from dual union all
select 53363,'GMS' from dual union all
select 53364,'MRN' from dual union all
select 53365,'MRN' from dual union all
select 53366,'MRN' from dual union all
select 53367,'JMA' from dual union all
select 53368,'MRN' from dual union all
select 53369,'MRN' from dual union all
select 5337,'UNRV' from dual union all
select 53370,'MRN' from dual union all
select 53371,'GMS' from dual union all
select 53372,'MRN' from dual union all
select 53373,'MRN' from dual union all
select 53374,'MRN' from dual union all
select 53375,'MRN' from dual union all
select 53376,'MRN' from dual union all
select 53377,'MRN' from dual union all
select 53378,'MRN' from dual union all
select 53379,'MRN' from dual union all
select 5338,'UNRV' from dual union all
select 53380,'MRN' from dual union all
select 53381,'JMA' from dual union all
select 53382,'GMS' from dual union all
select 53383,'MRN' from dual union all
select 53384,'MRN' from dual union all
select 53385,'MRN' from dual union all
select 53386,'MRN' from dual union all
select 53387,'MRN' from dual union all
select 53388,'GMS' from dual union all
select 53389,'MRN' from dual union all
select 5339,'UNRV' from dual union all
select 53390,'MRN' from dual union all
select 53391,'MRN' from dual union all
select 53392,'MRN' from dual union all
select 53393,'GMS' from dual union all
select 53394,'AN' from dual union all
select 53395,'MRN' from dual union all
select 53396,'MRN' from dual union all
select 53397,'MRN' from dual union all
select 53398,'MRN' from dual union all
select 53399,'GMS' from dual union all
select 534,'WEHR' from dual union all
select 5340,'UNRV' from dual union all
select 53400,'MRN' from dual union all
select 53401,'MRN' from dual union all
select 53402,'GMS' from dual union all
select 53403,'GMS' from dual union all
select 53404,'GMS' from dual union all
select 53405,'GMS' from dual union all
select 53406,'MRN' from dual union all
select 53407,'MRN' from dual union all
select 53408,'MRN' from dual union all
select 53409,'MRN' from dual union all
select 5341,'MVS' from dual union all
select 53410,'GMS' from dual union all
select 53411,'MRN' from dual union all
select 53412,'GMS' from dual union all
select 53413,'AN' from dual union all
select 53414,'JMA' from dual union all
select 53415,'GMS' from dual union all
select 53416,'MRN' from dual union all
select 53417,'MRN' from dual union all
select 53418,'MRN' from dual union all
select 53419,'MRN' from dual union all
select 5342,'UNRV' from dual union all
select 53420,'MRN' from dual union all
select 53421,'MRN' from dual union all
select 53422,'MRN' from dual union all
select 53423,'MRN' from dual union all
select 53424,'GMS' from dual union all
select 53425,'MRN' from dual union all
select 53426,'MRN' from dual union all
select 53427,'MRN' from dual union all
select 53428,'GMS' from dual union all
select 53429,'MRN' from dual union all
select 5343,'UNRV' from dual union all
select 53430,'MRN' from dual union all
select 53431,'MRN' from dual union all
select 53432,'MRN' from dual union all
select 53433,'MRN' from dual union all
select 53434,'MRN' from dual union all
select 53435,'MRN' from dual union all
select 53436,'MRN' from dual union all
select 53437,'MRN' from dual union all
select 53438,'AN' from dual union all
select 53439,'MRN' from dual union all
select 5344,'UNRV' from dual union all
select 53440,'MRN' from dual union all
select 53441,'MRN' from dual union all
select 53442,'MRN' from dual union all
select 53443,'MRN' from dual union all
select 53444,'NASA' from dual union all
select 53445,'MRN' from dual union all
select 53446,'MRN' from dual;
insert into LAUNCH_AGENCY
select 53447,'MRN' from dual union all
select 53448,'AN' from dual union all
select 53449,'JMA' from dual union all
select 5345,'UNRV' from dual union all
select 53450,'MRN' from dual union all
select 53451,'MRN' from dual union all
select 53452,'MRN' from dual union all
select 53453,'MRN' from dual union all
select 53454,'MRN' from dual union all
select 53455,'MRN' from dual union all
select 53456,'MRN' from dual union all
select 53457,'MRN' from dual union all
select 53458,'MRN' from dual union all
select 53459,'MRN' from dual union all
select 5346,'UNRV' from dual union all
select 53460,'MRN' from dual union all
select 53461,'MRN' from dual union all
select 53462,'MRN' from dual union all
select 53463,'MRN' from dual union all
select 53464,'MRN' from dual union all
select 53465,'MRN' from dual union all
select 53466,'MRN' from dual union all
select 53467,'MRN' from dual union all
select 53468,'MRN' from dual union all
select 53469,'AN' from dual union all
select 5347,'UNRV' from dual union all
select 53470,'MRN' from dual union all
select 53471,'MRN' from dual union all
select 53472,'MRN' from dual union all
select 53473,'MRN' from dual union all
select 53474,'MRN' from dual union all
select 53475,'MRN' from dual union all
select 53476,'MRN' from dual union all
select 53477,'JMA' from dual union all
select 53478,'MRN' from dual union all
select 53479,'MRN' from dual union all
select 5348,'UNRV' from dual union all
select 53480,'MRN' from dual union all
select 53481,'JMA' from dual union all
select 53482,'MRN' from dual union all
select 53483,'MRN' from dual union all
select 53484,'MRN' from dual union all
select 53485,'MRN' from dual union all
select 53486,'MRN' from dual union all
select 53487,'AN' from dual union all
select 53488,'GMS' from dual union all
select 53489,'MRN' from dual union all
select 5349,'UNRV' from dual union all
select 53490,'MRN' from dual union all
select 53491,'MRN' from dual union all
select 53492,'MRN' from dual union all
select 53493,'MRN' from dual union all
select 53494,'MRN' from dual union all
select 53495,'MRN' from dual union all
select 53496,'MRN' from dual union all
select 53497,'MRN' from dual union all
select 53498,'MRN' from dual union all
select 53499,'AN' from dual union all
select 535,'WEHR' from dual union all
select 5350,'UNRV' from dual union all
select 53500,'MRN' from dual union all
select 53501,'MRN' from dual union all
select 53502,'MRN' from dual union all
select 53503,'AN' from dual union all
select 53504,'MRN' from dual union all
select 53505,'MRN' from dual union all
select 53506,'MRN' from dual union all
select 53507,'MRN' from dual union all
select 53508,'MRN' from dual union all
select 53509,'MRN' from dual union all
select 5351,'UNRV' from dual union all
select 53510,'MRN' from dual union all
select 53511,'MRN' from dual union all
select 53512,'MRN' from dual union all
select 53513,'MRN' from dual union all
select 53514,'MRN' from dual union all
select 53515,'JMA' from dual union all
select 53516,'MRN' from dual union all
select 53517,'MRN' from dual union all
select 53518,'MRN' from dual union all
select 53519,'MRN' from dual union all
select 5352,'USAF' from dual union all
select 53520,'MRN' from dual union all
select 53521,'MRN' from dual union all
select 53522,'NASA' from dual union all
select 53523,'MRN' from dual union all
select 53524,'MRN' from dual union all
select 53525,'MRN' from dual union all
select 53526,'MRN' from dual union all
select 53527,'MRN' from dual union all
select 53528,'MRN' from dual union all
select 53529,'MRN' from dual union all
select 5353,'UNRV' from dual union all
select 53530,'MRN' from dual union all
select 53531,'MRN' from dual union all
select 53532,'MRN' from dual union all
select 53533,'MRN' from dual union all
select 53534,'MRN' from dual union all
select 53535,'MRN' from dual union all
select 53536,'MRN' from dual;
insert into LAUNCH_AGENCY
select 53537,'MRN' from dual union all
select 53538,'MRN' from dual union all
select 53539,'MRN' from dual union all
select 5354,'UNRV' from dual union all
select 53540,'MRN' from dual union all
select 53541,'JMA' from dual union all
select 53542,'MRN' from dual union all
select 53543,'MRN' from dual union all
select 53544,'MRN' from dual union all
select 53545,'MRN' from dual union all
select 53546,'MRN' from dual union all
select 53547,'MRN' from dual union all
select 53548,'MRN' from dual union all
select 53549,'MRN' from dual union all
select 5355,'UNRV' from dual union all
select 53550,'MRN' from dual union all
select 53551,'MRN' from dual union all
select 53552,'MRN' from dual union all
select 53553,'MRN' from dual union all
select 53554,'MRN' from dual union all
select 53555,'MRN' from dual union all
select 53556,'MRN' from dual union all
select 53557,'MRN' from dual union all
select 53558,'MRN' from dual union all
select 53559,'MRN' from dual union all
select 5356,'UNRV' from dual union all
select 53560,'MRN' from dual union all
select 53561,'MRN' from dual union all
select 53562,'MRN' from dual union all
select 53563,'MRN' from dual union all
select 53564,'JMA' from dual union all
select 53565,'MRN' from dual union all
select 53566,'MRN' from dual union all
select 53567,'AN' from dual union all
select 53568,'AN' from dual union all
select 53569,'MRN' from dual union all
select 5357,'UNRV' from dual union all
select 53570,'GMS' from dual union all
select 53571,'MRN' from dual union all
select 53572,'MRN' from dual union all
select 53573,'MRN' from dual union all
select 53574,'MRN' from dual union all
select 53575,'MRN' from dual union all
select 53576,'MRN' from dual union all
select 53577,'MRN' from dual union all
select 53578,'MRN' from dual union all
select 53579,'MRN' from dual union all
select 5358,'UNRV' from dual union all
select 53580,'MRN' from dual union all
select 53581,'MRN' from dual union all
select 53582,'GMS' from dual union all
select 53583,'MRN' from dual union all
select 53584,'AN' from dual union all
select 53585,'MRN' from dual union all
select 53586,'MRN' from dual union all
select 53587,'MRN' from dual union all
select 53588,'MRN' from dual union all
select 53589,'GMS' from dual union all
select 5359,'UNRV' from dual union all
select 53590,'MRN' from dual union all
select 53591,'MRN' from dual union all
select 53592,'MRN' from dual union all
select 53593,'MRN' from dual union all
select 53594,'MRN' from dual union all
select 53595,'MRN' from dual union all
select 53596,'NASA' from dual union all
select 53597,'MRN' from dual union all
select 53598,'MRN' from dual union all
select 53599,'MRN' from dual union all
select 536,'WEHR' from dual union all
select 5360,'UNRV' from dual union all
select 53600,'MRN' from dual union all
select 53601,'MRN' from dual union all
select 53602,'MRN' from dual union all
select 53603,'MRN' from dual union all
select 53604,'GMS' from dual union all
select 53605,'MRN' from dual union all
select 53606,'MRN' from dual union all
select 53607,'MRN' from dual union all
select 53608,'MRN' from dual union all
select 53609,'GMS' from dual union all
select 5361,'UNRV' from dual union all
select 53610,'MRN' from dual union all
select 53611,'MRN' from dual union all
select 53612,'MRN' from dual union all
select 53613,'MRN' from dual union all
select 53614,'MRN' from dual union all
select 53615,'MRN' from dual union all
select 53616,'AN' from dual union all
select 53617,'MRN' from dual union all
select 53618,'MRN' from dual union all
select 53619,'MRN' from dual union all
select 5362,'UNRV' from dual union all
select 53620,'GMS' from dual union all
select 53621,'MRN' from dual union all
select 53622,'MRN' from dual union all
select 53623,'GMS' from dual union all
select 53624,'MRN' from dual union all
select 53625,'MRN' from dual union all
select 53626,'MRN' from dual;
insert into LAUNCH_AGENCY
select 53627,'MRN' from dual union all
select 53628,'MRN' from dual union all
select 53629,'MRN' from dual union all
select 5363,'USA' from dual union all
select 53630,'MRN' from dual union all
select 53631,'MRN' from dual union all
select 53632,'MRN' from dual union all
select 53633,'JMA' from dual union all
select 53634,'GMS' from dual union all
select 53635,'MRN' from dual union all
select 53636,'MRN' from dual union all
select 53637,'MRN' from dual union all
select 53638,'MRN' from dual union all
select 53639,'MRN' from dual union all
select 5364,'UNRV' from dual union all
select 53640,'MRN' from dual union all
select 53641,'GMS' from dual union all
select 53642,'MRN' from dual union all
select 53643,'MRN' from dual union all
select 53644,'MRN' from dual union all
select 53645,'MRN' from dual union all
select 53646,'MRN' from dual union all
select 53647,'MRN' from dual union all
select 53648,'MRN' from dual union all
select 53649,'MRN' from dual union all
select 5365,'RVSN' from dual union all
select 53650,'MRN' from dual union all
select 53651,'NASA' from dual union all
select 53652,'MRN' from dual union all
select 53653,'MRN' from dual union all
select 53654,'MRN' from dual union all
select 53655,'MRN' from dual union all
select 53656,'MRN' from dual union all
select 53657,'JMA' from dual union all
select 53658,'GMS' from dual union all
select 53659,'MRN' from dual union all
select 5366,'RVSN' from dual union all
select 53660,'MRN' from dual union all
select 53661,'MRN' from dual union all
select 53662,'MRN' from dual union all
select 53663,'MRN' from dual union all
select 53664,'MRN' from dual union all
select 53665,'MRN' from dual union all
select 53666,'MRN' from dual union all
select 53667,'MRN' from dual union all
select 53668,'MRN' from dual union all
select 53669,'MRN' from dual union all
select 5367,'UNRV' from dual union all
select 53670,'MRN' from dual union all
select 53671,'MRN' from dual union all
select 53672,'MRN' from dual union all
select 53673,'MRN' from dual union all
select 53674,'MRN' from dual union all
select 53675,'MRN' from dual union all
select 53676,'MRN' from dual union all
select 53677,'MRN' from dual union all
select 53678,'MRN' from dual union all
select 53679,'MRN' from dual union all
select 5368,'UNRV' from dual union all
select 53680,'GMS' from dual union all
select 53681,'MRN' from dual union all
select 53682,'MRN' from dual union all
select 53683,'MRN' from dual union all
select 53684,'MRN' from dual union all
select 53685,'MRN' from dual union all
select 53686,'MRN' from dual union all
select 53687,'MRN' from dual union all
select 53688,'MRN' from dual union all
select 53689,'MRN' from dual union all
select 5369,'UNRV' from dual union all
select 53690,'MRN' from dual union all
select 53691,'MRN' from dual union all
select 53692,'AN' from dual union all
select 53693,'JMA' from dual union all
select 53694,'GMS' from dual union all
select 53695,'MRN' from dual union all
select 53696,'MRN' from dual union all
select 53697,'MRN' from dual union all
select 53698,'MRN' from dual union all
select 53699,'MRN' from dual union all
select 537,'WEHR' from dual union all
select 5370,'UNRV' from dual union all
select 53700,'MRN' from dual union all
select 53701,'MRN' from dual union all
select 53702,'MRN' from dual union all
select 53703,'MRN' from dual union all
select 53704,'MRN' from dual union all
select 53705,'MRN' from dual union all
select 53706,'MRN' from dual union all
select 53707,'MRN' from dual union all
select 53708,'MRN' from dual union all
select 53709,'MRN' from dual union all
select 5371,'MVS' from dual union all
select 53710,'MRN' from dual union all
select 53711,'MRN' from dual union all
select 53712,'MRN' from dual union all
select 53713,'MRN' from dual union all
select 53714,'MRN' from dual union all
select 53715,'GMS' from dual union all
select 53716,'MRN' from dual;
insert into LAUNCH_AGENCY
select 53717,'MRN' from dual union all
select 53718,'MRN' from dual union all
select 53719,'GMS' from dual union all
select 5372,'UNRV' from dual union all
select 53720,'MRN' from dual union all
select 53721,'MRN' from dual union all
select 53722,'MRN' from dual union all
select 53723,'JMA' from dual union all
select 53724,'GMS' from dual union all
select 53725,'MRN' from dual union all
select 53726,'MRN' from dual union all
select 53727,'NASA' from dual union all
select 53728,'MRN' from dual union all
select 53729,'MRN' from dual union all
select 5373,'UNRV' from dual union all
select 53730,'MRN' from dual union all
select 53731,'MRN' from dual union all
select 53732,'MRN' from dual union all
select 53733,'MRN' from dual union all
select 53734,'MRN' from dual union all
select 53735,'MRN' from dual union all
select 53736,'MRN' from dual union all
select 53737,'MRN' from dual union all
select 53738,'MRN' from dual union all
select 53739,'MRN' from dual union all
select 5374,'USAF' from dual union all
select 53740,'MRN' from dual union all
select 53741,'MRN' from dual union all
select 53742,'MRN' from dual union all
select 53743,'MRN' from dual union all
select 53744,'MRN' from dual union all
select 53745,'MRN' from dual union all
select 53746,'MRN' from dual union all
select 53747,'MRN' from dual union all
select 53748,'MRN' from dual union all
select 53749,'MRN' from dual union all
select 5375,'UNRV' from dual union all
select 53750,'MRN' from dual union all
select 53751,'MRN' from dual union all
select 53752,'MRN' from dual union all
select 53753,'GMS' from dual union all
select 53754,'MRN' from dual union all
select 53755,'MRN' from dual union all
select 53756,'MRN' from dual union all
select 53757,'MRN' from dual union all
select 53758,'MRN' from dual union all
select 53759,'JMA' from dual union all
select 5376,'UNRV' from dual union all
select 53760,'MRN' from dual union all
select 53761,'MRN' from dual union all
select 53762,'MRN' from dual union all
select 53763,'MRN' from dual union all
select 53764,'MRN' from dual union all
select 53765,'MRN' from dual union all
select 53766,'MRN' from dual union all
select 53767,'MRN' from dual union all
select 53768,'MRN' from dual union all
select 53769,'MRN' from dual union all
select 5377,'UNRV' from dual union all
select 53770,'JMA' from dual union all
select 53771,'GMS' from dual union all
select 53772,'MRN' from dual union all
select 53773,'MRN' from dual union all
select 53774,'MRN' from dual union all
select 53775,'MRN' from dual union all
select 53776,'MRN' from dual union all
select 53777,'MRN' from dual union all
select 53778,'MRN' from dual union all
select 53779,'MRN' from dual union all
select 5378,'UNRV' from dual union all
select 53780,'MRN' from dual union all
select 53781,'MRN' from dual union all
select 53782,'MRN' from dual union all
select 53783,'MRN' from dual union all
select 53784,'MRN' from dual union all
select 53785,'MRN' from dual union all
select 53786,'MRN' from dual union all
select 53787,'MRN' from dual union all
select 53788,'JMA' from dual union all
select 53789,'MRN' from dual union all
select 5379,'UNRV' from dual union all
select 53790,'MRN' from dual union all
select 53791,'AN' from dual union all
select 53792,'MRN' from dual union all
select 53793,'MRN' from dual union all
select 53794,'MRN' from dual union all
select 53795,'MRN' from dual union all
select 53796,'NASDA' from dual union all
select 53797,'MRN' from dual union all
select 53798,'MRN' from dual union all
select 53799,'MRN' from dual union all
select 538,'WEHR' from dual union all
select 5380,'USA' from dual union all
select 53800,'MRN' from dual union all
select 53801,'JMA' from dual union all
select 53802,'AN' from dual union all
select 53803,'MRN' from dual union all
select 53804,'MRN' from dual union all
select 53805,'MRN' from dual union all
select 53806,'MRN' from dual;
insert into LAUNCH_AGENCY
select 53807,'MRN' from dual union all
select 53808,'MRN' from dual union all
select 53809,'AN' from dual union all
select 5381,'USAF' from dual union all
select 53810,'MRN' from dual union all
select 53811,'MRN' from dual union all
select 53812,'MRN' from dual union all
select 53813,'MRN' from dual union all
select 53814,'MRN' from dual union all
select 53815,'MRN' from dual union all
select 53816,'JMA' from dual union all
select 53817,'AN' from dual union all
select 53818,'MRN' from dual union all
select 53819,'MRN' from dual union all
select 5382,'USAF' from dual union all
select 53820,'MRN' from dual union all
select 53821,'MRN' from dual union all
select 53822,'MRN' from dual union all
select 53823,'MRN' from dual union all
select 53824,'MRN' from dual union all
select 53825,'MRN' from dual union all
select 53826,'MRN' from dual union all
select 53827,'MRN' from dual union all
select 53828,'MRN' from dual union all
select 53829,'GMS' from dual union all
select 5383,'USAF' from dual union all
select 53830,'MRN' from dual union all
select 53831,'JMA' from dual union all
select 53832,'MRN' from dual union all
select 53833,'MRN' from dual union all
select 53834,'MRN' from dual union all
select 53835,'MRN' from dual union all
select 53836,'MRN' from dual union all
select 53837,'MRN' from dual union all
select 53838,'JMA' from dual union all
select 53839,'JMA' from dual union all
select 5384,'NRL' from dual union all
select 53840,'MRN' from dual union all
select 53841,'MRN' from dual union all
select 53842,'MRN' from dual union all
select 53843,'MRN' from dual union all
select 53844,'MRN' from dual union all
select 53845,'AN' from dual union all
select 53846,'MRN' from dual union all
select 53847,'JMA' from dual union all
select 53848,'GMS' from dual union all
select 53849,'GMS' from dual union all
select 5385,'USAF' from dual union all
select 53850,'MRN' from dual union all
select 53851,'MRN' from dual union all
select 53852,'MRN' from dual union all
select 53853,'MRN' from dual union all
select 53854,'MRN' from dual union all
select 53855,'MRN' from dual union all
select 53856,'MRN' from dual union all
select 53857,'AN' from dual union all
select 53858,'AN' from dual union all
select 53859,'AN' from dual union all
select 5386,'USAF' from dual union all
select 53860,'AN' from dual union all
select 53861,'AN' from dual union all
select 53862,'AN' from dual union all
select 53863,'AN' from dual union all
select 53864,'AN' from dual union all
select 53865,'AN' from dual union all
select 53866,'NASA' from dual union all
select 53867,'AN' from dual union all
select 53868,'AN' from dual union all
select 53869,'AN' from dual union all
select 5387,'NRL' from dual union all
select 53870,'AN' from dual union all
select 53871,'AN' from dual union all
select 53872,'AN' from dual union all
select 53873,'MRN' from dual union all
select 53874,'AN' from dual union all
select 53875,'AN' from dual union all
select 53876,'MRN' from dual union all
select 53877,'NASA' from dual union all
select 53878,'AN' from dual union all
select 53879,'MRN' from dual union all
select 5388,'NRL' from dual union all
select 53880,'AN' from dual union all
select 53881,'AN' from dual union all
select 53882,'AN' from dual union all
select 53883,'AN' from dual union all
select 53884,'AN' from dual union all
select 53885,'AN' from dual union all
select 53886,'AN' from dual union all
select 53887,'MRN' from dual union all
select 53888,'AN' from dual union all
select 53889,'AN' from dual union all
select 5389,'BRL' from dual union all
select 53890,'AN' from dual union all
select 53891,'AN' from dual union all
select 53892,'AN' from dual union all
select 53893,'MRN' from dual union all
select 53894,'AN' from dual union all
select 53895,'AN' from dual union all
select 53896,'AN' from dual union all
select 53897,'GMS' from dual;
insert into LAUNCH_AGENCY
select 53898,'MRN' from dual union all
select 53899,'AN' from dual union all
select 539,'WEHR' from dual union all
select 5390,'USAF' from dual union all
select 53900,'AN' from dual union all
select 53901,'AN' from dual union all
select 53902,'AN' from dual union all
select 53903,'AN' from dual union all
select 53904,'GMS' from dual union all
select 53905,'AN' from dual union all
select 53906,'MRN' from dual union all
select 53907,'MRN' from dual union all
select 53908,'MRN' from dual union all
select 53909,'MRN' from dual union all
select 5391,'MVS' from dual union all
select 53910,'AN' from dual union all
select 53911,'MRN' from dual union all
select 53912,'MRN' from dual union all
select 53913,'MRN' from dual union all
select 53914,'MRN' from dual union all
select 53915,'MRN' from dual union all
select 53916,'MRN' from dual union all
select 53917,'MRN' from dual union all
select 53918,'AN' from dual union all
select 53919,'AN' from dual union all
select 5392,'UNRV' from dual union all
select 53920,'MRN' from dual union all
select 53921,'AN' from dual union all
select 53922,'AN' from dual union all
select 53923,'MRN' from dual union all
select 53924,'AN' from dual union all
select 53925,'AN' from dual union all
select 53926,'AN' from dual union all
select 53927,'AN' from dual union all
select 53928,'MRN' from dual union all
select 53929,'MRN' from dual union all
select 5393,'MVS' from dual union all
select 53930,'AN' from dual union all
select 53931,'AN' from dual union all
select 53932,'AN' from dual union all
select 53933,'AN' from dual union all
select 53934,'AN' from dual union all
select 53935,'MRN' from dual union all
select 53936,'AN' from dual union all
select 53937,'AN' from dual union all
select 53938,'AN' from dual union all
select 53939,'AN' from dual union all
select 5394,'USAF' from dual union all
select 53940,'AN' from dual union all
select 53941,'AN' from dual union all
select 53942,'AN' from dual union all
select 53943,'AN' from dual union all
select 53944,'MRN' from dual union all
select 53945,'AN' from dual union all
select 53946,'AN' from dual union all
select 53947,'AN' from dual union all
select 53948,'AN' from dual union all
select 53949,'AN' from dual union all
select 5395,'LARC' from dual union all
select 53950,'MRN' from dual union all
select 53951,'AN' from dual union all
select 53952,'MRN' from dual union all
select 53953,'AN' from dual union all
select 53954,'AN' from dual union all
select 53955,'AN' from dual union all
select 53956,'AN' from dual union all
select 53957,'AN' from dual union all
select 53958,'AN' from dual union all
select 53959,'AN' from dual union all
select 5396,'RAE' from dual union all
select 5396,'WRE' from dual union all
select 53960,'AN' from dual union all
select 53961,'AN' from dual union all
select 53962,'MRN' from dual union all
select 53963,'AN' from dual union all
select 53964,'AN' from dual union all
select 53965,'MRN' from dual union all
select 53966,'AN' from dual union all
select 53967,'AN' from dual union all
select 53968,'AN' from dual union all
select 53969,'AN' from dual union all
select 5397,'USN' from dual union all
select 53970,'AN' from dual union all
select 53971,'AN' from dual union all
select 53972,'MRN' from dual union all
select 53973,'AN' from dual union all
select 53974,'MRN' from dual union all
select 53975,'AN' from dual union all
select 53976,'AN' from dual union all
select 53977,'MRN' from dual union all
select 53978,'AN' from dual union all
select 53979,'AN' from dual union all
select 5398,'LARC' from dual union all
select 53980,'AN' from dual union all
select 53981,'AN' from dual union all
select 53982,'MRN' from dual union all
select 53983,'AN' from dual union all
select 53984,'AN' from dual union all
select 53985,'MRN' from dual union all
select 53986,'AN' from dual;
insert into LAUNCH_AGENCY
select 53987,'AN' from dual union all
select 53988,'AN' from dual union all
select 53989,'AN' from dual union all
select 5399,'MVS' from dual union all
select 53990,'AN' from dual union all
select 53991,'AN' from dual union all
select 53992,'AN' from dual union all
select 53993,'MRN' from dual union all
select 53994,'AN' from dual union all
select 53995,'AN' from dual union all
select 53996,'AN' from dual union all
select 53997,'AN' from dual union all
select 53998,'AN' from dual union all
select 53999,'AN' from dual union all
select 54,'WER500' from dual union all
select 540,'WEHR' from dual union all
select 5400,'USAF' from dual union all
select 54000,'AN' from dual union all
select 54001,'AN' from dual union all
select 54002,'AN' from dual union all
select 54003,'AN' from dual union all
select 54004,'AN' from dual union all
select 54005,'AN' from dual union all
select 54006,'AN' from dual union all
select 54007,'AN' from dual union all
select 54008,'AN' from dual union all
select 54009,'AN' from dual union all
select 5401,'NRL' from dual union all
select 54010,'AN' from dual union all
select 54011,'AN' from dual union all
select 54012,'AN' from dual union all
select 54013,'AN' from dual union all
select 54014,'AN' from dual union all
select 54015,'MRN' from dual union all
select 54016,'AN' from dual union all
select 54017,'AN' from dual union all
select 54018,'MRN' from dual union all
select 54019,'AN' from dual union all
select 5402,'USN' from dual union all
select 54020,'AN' from dual union all
select 54021,'MRN' from dual union all
select 54022,'AN' from dual union all
select 54023,'AN' from dual union all
select 54024,'AN' from dual union all
select 54025,'AN' from dual union all
select 54026,'AN' from dual union all
select 54027,'GMS' from dual union all
select 54028,'AN' from dual union all
select 54029,'AN' from dual union all
select 5403,'USA' from dual union all
select 54030,'AN' from dual union all
select 54031,'AN' from dual union all
select 54032,'AN' from dual union all
select 54033,'AN' from dual union all
select 54034,'AN' from dual union all
select 54035,'MRN' from dual union all
select 54036,'AN' from dual union all
select 54037,'AN' from dual union all
select 54038,'AN' from dual union all
select 54039,'AN' from dual union all
select 5404,'ABMA' from dual union all
select 5404,'USAF' from dual union all
select 54040,'AN' from dual union all
select 54041,'AN' from dual union all
select 54042,'MRN' from dual union all
select 54043,'AN' from dual union all
select 54044,'AN' from dual union all
select 54045,'AN' from dual union all
select 54046,'MRN' from dual union all
select 54047,'MRN' from dual union all
select 54048,'AN' from dual union all
select 54049,'AN' from dual union all
select 5405,'USAF' from dual union all
select 54050,'AN' from dual union all
select 54051,'AN' from dual union all
select 54052,'AN' from dual union all
select 54053,'AN' from dual union all
select 54054,'AN' from dual union all
select 54055,'MRN' from dual union all
select 54056,'AN' from dual union all
select 54057,'AN' from dual union all
select 54058,'AN' from dual union all
select 54059,'AN' from dual union all
select 5406,'RAE' from dual union all
select 5406,'WRE' from dual union all
select 54060,'AN' from dual union all
select 54061,'AN' from dual union all
select 54062,'MRN' from dual union all
select 54063,'AN' from dual union all
select 54064,'AN' from dual union all
select 54065,'AN' from dual union all
select 54066,'AN' from dual union all
select 54067,'AN' from dual union all
select 54068,'AN' from dual union all
select 54069,'AN' from dual union all
select 5407,'USAF' from dual union all
select 54070,'AN' from dual union all
select 54071,'MRN' from dual union all
select 54072,'AN' from dual union all
select 54073,'AN' from dual;
insert into LAUNCH_AGENCY
select 54074,'MRN' from dual union all
select 54075,'AN' from dual union all
select 54076,'AN' from dual union all
select 54077,'AN' from dual union all
select 54078,'AN' from dual union all
select 54079,'AN' from dual union all
select 5408,'USAF' from dual union all
select 54080,'AN' from dual union all
select 54081,'AN' from dual union all
select 54082,'AN' from dual union all
select 54083,'AN' from dual union all
select 54084,'AN' from dual union all
select 54085,'AN' from dual union all
select 54086,'AN' from dual union all
select 54087,'AN' from dual union all
select 54088,'AN' from dual union all
select 54089,'AN' from dual union all
select 5409,'LARC' from dual union all
select 54090,'AN' from dual union all
select 54091,'AN' from dual union all
select 54092,'AN' from dual union all
select 54093,'AN' from dual union all
select 54094,'MRN' from dual union all
select 54095,'AN' from dual union all
select 54096,'NASA' from dual union all
select 54097,'AN' from dual union all
select 54098,'AN' from dual union all
select 54099,'AN' from dual union all
select 541,'WEHR' from dual union all
select 5410,'MVS' from dual union all
select 54100,'AN' from dual union all
select 54101,'AN' from dual union all
select 54102,'MRN' from dual union all
select 54103,'MRN' from dual union all
select 54104,'MRN' from dual union all
select 54105,'MRN' from dual union all
select 54106,'AN' from dual union all
select 54107,'AN' from dual union all
select 54108,'AN' from dual union all
select 54109,'AN' from dual union all
select 5411,'LARC' from dual union all
select 54110,'AN' from dual union all
select 54111,'MRN' from dual union all
select 54112,'AN' from dual union all
select 54113,'AN' from dual union all
select 54114,'AN' from dual union all
select 54115,'AN' from dual union all
select 54116,'MRN' from dual union all
select 54117,'MRN' from dual union all
select 54118,'MRN' from dual union all
select 54119,'AN' from dual union all
select 5412,'UNRV' from dual union all
select 54120,'MRN' from dual union all
select 54121,'AN' from dual union all
select 54122,'MRN' from dual union all
select 54123,'AN' from dual union all
select 54124,'AN' from dual union all
select 54125,'MRN' from dual union all
select 54126,'AN' from dual union all
select 54127,'AN' from dual union all
select 54128,'AN' from dual union all
select 54129,'AN' from dual union all
select 5413,'UNRV' from dual union all
select 54130,'AN' from dual union all
select 54131,'AN' from dual union all
select 54132,'AN' from dual union all
select 54133,'AN' from dual union all
select 54134,'AN' from dual union all
select 54135,'MRN' from dual union all
select 54136,'AN' from dual union all
select 54137,'MRN' from dual union all
select 54138,'AN' from dual union all
select 54139,'AN' from dual union all
select 5414,'MVS' from dual union all
select 54140,'AN' from dual union all
select 54141,'AN' from dual union all
select 54142,'MRN' from dual union all
select 54143,'AN' from dual union all
select 54144,'AN' from dual union all
select 54145,'AN' from dual union all
select 54146,'MRN' from dual union all
select 54147,'AN' from dual union all
select 54148,'AN' from dual union all
select 54149,'AN' from dual union all
select 5415,'MVS' from dual union all
select 54150,'AN' from dual union all
select 54151,'AN' from dual union all
select 54152,'MRN' from dual union all
select 54153,'AN' from dual union all
select 54154,'AN' from dual union all
select 54155,'AN' from dual union all
select 54156,'MRN' from dual union all
select 54157,'AN' from dual union all
select 54158,'AN' from dual union all
select 54159,'AN' from dual union all
select 5416,'MVS' from dual union all
select 54160,'MRN' from dual union all
select 54161,'AN' from dual union all
select 54162,'AN' from dual union all
select 54163,'AN' from dual;
insert into LAUNCH_AGENCY
select 54164,'AN' from dual union all
select 54165,'AN' from dual union all
select 54166,'AN' from dual union all
select 54167,'AN' from dual union all
select 54168,'MRN' from dual union all
select 54169,'MRN' from dual union all
select 5417,'USA' from dual union all
select 54170,'AN' from dual union all
select 54171,'AN' from dual union all
select 54172,'AN' from dual union all
select 54173,'AN' from dual union all
select 54174,'AN' from dual union all
select 54175,'AN' from dual union all
select 54176,'MRN' from dual union all
select 54177,'AN' from dual union all
select 54178,'AN' from dual union all
select 54179,'AN' from dual union all
select 5418,'USAF' from dual union all
select 54180,'AN' from dual union all
select 54181,'AN' from dual union all
select 54182,'AN' from dual union all
select 54183,'AN' from dual union all
select 54184,'MRN' from dual union all
select 54185,'AN' from dual union all
select 54186,'AN' from dual union all
select 54187,'AN' from dual union all
select 54188,'AN' from dual union all
select 54189,'AN' from dual union all
select 5419,'6555A' from dual union all
select 54190,'AN' from dual union all
select 54191,'ISRO' from dual union all
select 54192,'ISRO' from dual union all
select 54193,'MRN' from dual union all
select 54194,'MRN' from dual union all
select 54195,'MRN' from dual union all
select 54196,'AN' from dual union all
select 54197,'MRN' from dual union all
select 54198,'MRN' from dual union all
select 54199,'MRN' from dual union all
select 542,'WEHR' from dual union all
select 5420,'USAF' from dual union all
select 54200,'MRN' from dual union all
select 54201,'MRN' from dual union all
select 54202,'MRN' from dual union all
select 54203,'MRN' from dual union all
select 54204,'AN' from dual union all
select 54205,'AN' from dual union all
select 54206,'AN' from dual union all
select 54207,'AN' from dual union all
select 54208,'AN' from dual union all
select 54209,'AN' from dual union all
select 5421,'RAE' from dual union all
select 5421,'WRE' from dual union all
select 54210,'AN' from dual union all
select 54211,'AN' from dual union all
select 54212,'MRN' from dual union all
select 54213,'AN' from dual union all
select 54214,'AN' from dual union all
select 54215,'AN' from dual union all
select 54216,'MRN' from dual union all
select 54217,'AN' from dual union all
select 54218,'AN' from dual union all
select 54219,'AN' from dual union all
select 5422,'USN' from dual union all
select 54220,'AN' from dual union all
select 54221,'AN' from dual union all
select 54222,'MRN' from dual union all
select 54223,'AN' from dual union all
select 54224,'MRN' from dual union all
select 54225,'AN' from dual union all
select 54226,'DFVLR' from dual union all
select 54227,'DFVLR' from dual union all
select 54228,'AN' from dual union all
select 54229,'AN' from dual union all
select 5423,'USA' from dual union all
select 54230,'AN' from dual union all
select 54231,'AN' from dual union all
select 54232,'AN' from dual union all
select 54233,'MRN' from dual union all
select 54234,'AN' from dual union all
select 54235,'AN' from dual union all
select 54236,'MRN' from dual union all
select 54237,'MRN' from dual union all
select 54238,'AN' from dual union all
select 54239,'AN' from dual union all
select 5424,'6555A' from dual union all
select 54240,'AN' from dual union all
select 54241,'AN' from dual union all
select 54242,'AN' from dual union all
select 54243,'MRN' from dual union all
select 54244,'AN' from dual union all
select 54245,'AN' from dual union all
select 54246,'AN' from dual union all
select 54247,'AN' from dual union all
select 54248,'AN' from dual union all
select 54249,'AN' from dual union all
select 5425,'RAE' from dual union all
select 5425,'WRE' from dual union all
select 54250,'MRN' from dual union all
select 54251,'MRN' from dual;
insert into LAUNCH_AGENCY
select 54252,'AN' from dual union all
select 54253,'AN' from dual union all
select 54254,'AN' from dual union all
select 54255,'AN' from dual union all
select 54256,'AN' from dual union all
select 54257,'AN' from dual union all
select 54258,'AN' from dual union all
select 54259,'AN' from dual union all
select 5426,'RAE' from dual union all
select 5426,'WRE' from dual union all
select 54260,'AN' from dual union all
select 54261,'AN' from dual union all
select 54262,'AN' from dual union all
select 54263,'MRN' from dual union all
select 54264,'MRN' from dual union all
select 54265,'AN' from dual union all
select 54266,'AN' from dual union all
select 54267,'AN' from dual union all
select 54268,'AN' from dual union all
select 54269,'AN' from dual union all
select 5427,'USN' from dual union all
select 54270,'AN' from dual union all
select 54271,'AN' from dual union all
select 54272,'AN' from dual union all
select 54273,'AN' from dual union all
select 54274,'AN' from dual union all
select 54275,'AN' from dual union all
select 54276,'AN' from dual union all
select 54277,'MRN' from dual union all
select 54278,'MRN' from dual union all
select 54279,'AN' from dual union all
select 5428,'USA' from dual union all
select 54280,'AN' from dual union all
select 54281,'AN' from dual union all
select 54282,'AN' from dual union all
select 54283,'AN' from dual union all
select 54284,'AN' from dual union all
select 54285,'MRN' from dual union all
select 54286,'AN' from dual union all
select 54287,'AN' from dual union all
select 54288,'AN' from dual union all
select 54289,'AN' from dual union all
select 5429,'AFCRL' from dual union all
select 5429,'PARD' from dual union all
select 54290,'MRN' from dual union all
select 54291,'MRN' from dual union all
select 54292,'AN' from dual union all
select 54293,'MRN' from dual union all
select 54294,'AN' from dual union all
select 54295,'AN' from dual union all
select 54296,'AN' from dual union all
select 54297,'AN' from dual union all
select 54298,'AN' from dual union all
select 54299,'AN' from dual union all
select 543,'WEHR' from dual union all
select 5430,'USN' from dual union all
select 54300,'AN' from dual union all
select 54301,'MRN' from dual union all
select 54302,'AN' from dual union all
select 54303,'AN' from dual union all
select 54304,'AN' from dual union all
select 54305,'AN' from dual union all
select 54306,'AN' from dual union all
select 54307,'AN' from dual union all
select 54308,'AN' from dual union all
select 54309,'AN' from dual union all
select 5431,'AN' from dual union all
select 54310,'UNKS' from dual union all
select 54311,'UNKS' from dual union all
select 54312,'UNKS' from dual union all
select 54313,'NASA' from dual union all
select 54314,'UNKS' from dual union all
select 54315,'AFSC' from dual union all
select 54316,'UNKS' from dual union all
select 54317,'UNKS' from dual union all
select 54318,'UNKS' from dual union all
select 54319,'UNKS' from dual union all
select 5432,'USAF' from dual union all
select 54320,'UNKS' from dual union all
select 54321,'UNKS' from dual union all
select 54322,'UNKS' from dual union all
select 54323,'NASA' from dual union all
select 54324,'UNKS' from dual union all
select 54325,'UNKS' from dual union all
select 54326,'NASA' from dual union all
select 54327,'UNKS' from dual union all
select 54328,'AFSC' from dual union all
select 54329,'UNKS' from dual union all
select 5433,'USAF' from dual union all
select 54330,'UNKS' from dual union all
select 54331,'NASA' from dual union all
select 54332,'UNKS' from dual union all
select 54333,'UNKS' from dual union all
select 54334,'UNKS' from dual union all
select 54335,'UNKS' from dual union all
select 54336,'UNKS' from dual union all
select 54337,'UNKS' from dual union all
select 54338,'UNKS' from dual union all
select 54339,'UNKS' from dual union all
select 5434,'USAF' from dual;
insert into LAUNCH_AGENCY
select 54340,'NASA' from dual union all
select 54341,'UNKS' from dual union all
select 54342,'UNKS' from dual union all
select 54343,'UNKS' from dual union all
select 54344,'UNKS' from dual union all
select 54345,'UNKS' from dual union all
select 54346,'UNKS' from dual union all
select 54347,'UNKS' from dual union all
select 54348,'UNKS' from dual union all
select 54349,'UNKS' from dual union all
select 5435,'USN' from dual union all
select 54350,'AFSC' from dual union all
select 54351,'UNKS' from dual union all
select 54352,'UNKS' from dual union all
select 54353,'UNKS' from dual union all
select 54354,'UNKS' from dual union all
select 54355,'UNKS' from dual union all
select 54356,'UNKS' from dual union all
select 54357,'UNKS' from dual union all
select 54358,'UNKS' from dual union all
select 54359,'UNKS' from dual union all
select 5436,'USN' from dual union all
select 54360,'UNKS' from dual union all
select 54361,'UNKS' from dual union all
select 54362,'UNKS' from dual union all
select 54363,'UNKS' from dual union all
select 54364,'UNKS' from dual union all
select 54365,'UNKS' from dual union all
select 54366,'UNKS' from dual union all
select 54367,'NASA' from dual union all
select 54368,'UNKS' from dual union all
select 54369,'UNKS' from dual union all
select 5437,'6555A' from dual union all
select 54370,'UNKS' from dual union all
select 54371,'UNKS' from dual union all
select 54372,'UNKS' from dual union all
select 54373,'UNKS' from dual union all
select 54374,'NASA' from dual union all
select 54375,'UNKS' from dual union all
select 54376,'UNKS' from dual union all
select 54377,'UNKS' from dual union all
select 54378,'UNKS' from dual union all
select 54379,'UNKS' from dual union all
select 5438,'USAF' from dual union all
select 54380,'UNKS' from dual union all
select 54381,'NASA' from dual union all
select 54382,'UNKS' from dual union all
select 54383,'UNKS' from dual union all
select 54384,'UNKS' from dual union all
select 54385,'UNKS' from dual union all
select 54386,'UNKS' from dual union all
select 54387,'UNKS' from dual union all
select 54388,'UNKS' from dual union all
select 54389,'UNKS' from dual union all
select 5439,'USAF' from dual union all
select 54390,'UNKS' from dual union all
select 54391,'NASA' from dual union all
select 54392,'UNKS' from dual union all
select 54393,'UNKS' from dual union all
select 54394,'UNKS' from dual union all
select 54395,'UNKS' from dual union all
select 54396,'NASDA' from dual union all
select 54397,'UNKS' from dual union all
select 54398,'UNKS' from dual union all
select 54399,'CALT' from dual union all
select 544,'WEHR' from dual union all
select 5440,'USN' from dual union all
select 54400,'UNKS' from dual union all
select 54401,'UNKS' from dual union all
select 54402,'UNKS' from dual union all
select 54403,'UNKS' from dual union all
select 54404,'UNKS' from dual union all
select 54405,'UNKS' from dual union all
select 54406,'NASA' from dual union all
select 54407,'UNKS' from dual union all
select 54408,'UNKS' from dual union all
select 54409,'UNKS' from dual union all
select 5441,'LRL' from dual union all
select 54410,'UNKS' from dual union all
select 54411,'UNKS' from dual union all
select 54412,'UNKS' from dual union all
select 54413,'UNKS' from dual union all
select 54414,'NASA' from dual union all
select 54415,'AFSC' from dual union all
select 54416,'UNKS' from dual union all
select 54417,'UNKS' from dual union all
select 54418,'UNKS' from dual union all
select 54419,'NASA' from dual union all
select 5442,'TF88' from dual union all
select 54420,'AFSC' from dual union all
select 54421,'UNKS' from dual union all
select 54422,'UNKS' from dual union all
select 54423,'UNKS' from dual union all
select 54424,'UNKS' from dual union all
select 54425,'UNKS' from dual union all
select 54426,'UNKS' from dual union all
select 54427,'AFSC' from dual union all
select 54428,'UNKS' from dual union all
select 54429,'UNKS' from dual union all
select 5443,'TF88' from dual;
insert into LAUNCH_AGENCY
select 54430,'UNKS' from dual union all
select 54431,'RVSN' from dual union all
select 54432,'MRN' from dual union all
select 54433,'MRN' from dual union all
select 54434,'JMA' from dual union all
select 54435,'MRN' from dual union all
select 54436,'JMA' from dual union all
select 54437,'NASA' from dual union all
select 54438,'JMA' from dual union all
select 54439,'MRN' from dual union all
select 5444,'TF88' from dual union all
select 54440,'MRN' from dual union all
select 54441,'MRN' from dual union all
select 54442,'JMA' from dual union all
select 54443,'MRN' from dual union all
select 54444,'MRN' from dual union all
select 54445,'MRN' from dual union all
select 54446,'USN' from dual union all
select 54447,'USN' from dual union all
select 54448,'MRN' from dual union all
select 54449,'MRN' from dual union all
select 5445,'TF88' from dual union all
select 54450,'MRN' from dual union all
select 54451,'MRN' from dual union all
select 54452,'MRN' from dual union all
select 54453,'MRN' from dual union all
select 54454,'MRN' from dual union all
select 54455,'MRN' from dual union all
select 54456,'MRN' from dual union all
select 54457,'MRN' from dual union all
select 54458,'MRN' from dual union all
select 54459,'MRN' from dual union all
select 5446,'TF88' from dual union all
select 54460,'MRN' from dual union all
select 54461,'MRN' from dual union all
select 54462,'MRN' from dual union all
select 54463,'MRN' from dual union all
select 54464,'NASA' from dual union all
select 54465,'MRN' from dual union all
select 54466,'MRN' from dual union all
select 54467,'MRN' from dual union all
select 54468,'RVSN' from dual union all
select 54469,'MRN' from dual union all
select 5447,'TF88' from dual union all
select 54470,'MRN' from dual union all
select 54471,'PRO' from dual union all
select 54472,'PRO' from dual union all
select 54473,'RVSN' from dual union all
select 54474,'UNKS' from dual union all
select 54475,'UNKS' from dual union all
select 54476,'UNKS' from dual union all
select 54477,'UNKS' from dual union all
select 54478,'UNKS' from dual union all
select 54479,'ESA' from dual union all
select 5448,'TF88' from dual union all
select 54480,'UNKS' from dual union all
select 54481,'UNKS' from dual union all
select 54482,'ISRO' from dual union all
select 54483,'ISRO' from dual union all
select 54484,'ISRO' from dual union all
select 54485,'ISRO' from dual union all
select 54486,'NASA' from dual union all
select 54487,'NASA' from dual union all
select 54488,'ISRO' from dual union all
select 54489,'ISRO' from dual union all
select 5449,'TF88' from dual union all
select 54490,'ISRO' from dual union all
select 54491,'ISRO' from dual union all
select 54492,'ISRO' from dual union all
select 54493,'ISRO' from dual union all
select 54494,'ISRO' from dual union all
select 54495,'ISRO' from dual union all
select 54496,'ISRO' from dual union all
select 54497,'ISRO' from dual union all
select 54498,'ISRO' from dual union all
select 54499,'ISRO' from dual union all
select 545,'WEHR' from dual union all
select 5450,'TF88' from dual union all
select 54500,'ISRO' from dual union all
select 54501,'ISRO' from dual union all
select 54502,'ISRO' from dual union all
select 54503,'ISRO' from dual union all
select 54504,'ISRO' from dual union all
select 54505,'ISRO' from dual union all
select 54506,'ISRO' from dual union all
select 54507,'ISRO' from dual union all
select 54508,'ISRO' from dual union all
select 54509,'ISRO' from dual union all
select 5451,'TF88' from dual union all
select 54510,'ISRO' from dual union all
select 54511,'ISRO' from dual union all
select 54512,'ISRO' from dual union all
select 54513,'ISRO' from dual union all
select 54514,'ISRO' from dual union all
select 54515,'ISRO' from dual union all
select 54516,'ISRO' from dual union all
select 54517,'ISRO' from dual union all
select 54518,'ISRO' from dual union all
select 54519,'ISRO' from dual union all
select 5452,'TF88' from dual;
insert into LAUNCH_AGENCY
select 54520,'ISRO' from dual union all
select 54521,'ISRO' from dual union all
select 54522,'ISRO' from dual union all
select 54523,'ISRO' from dual union all
select 54524,'ISRO' from dual union all
select 54525,'ISRO' from dual union all
select 54526,'ISRO' from dual union all
select 54527,'ISRO' from dual union all
select 54528,'ISRO' from dual union all
select 54529,'ISRO' from dual union all
select 5453,'TF88' from dual union all
select 54530,'ISRO' from dual union all
select 54531,'AFGL' from dual union all
select 54532,'ADD' from dual union all
select 54533,'RVSN' from dual union all
select 54534,'RVSN' from dual union all
select 54535,'RVSN' from dual union all
select 54536,'RVSN' from dual union all
select 54537,'DMA' from dual union all
select 54538,'DMA' from dual union all
select 54539,'DMA' from dual union all
select 5454,'TF88' from dual union all
select 54540,'DMA' from dual union all
select 54541,'RVSN' from dual union all
select 54542,'RVSN' from dual union all
select 54543,'AN' from dual union all
select 54544,'AN' from dual union all
select 54545,'AN' from dual union all
select 54546,'AN' from dual union all
select 54547,'SAC' from dual union all
select 54548,'NASA' from dual union all
select 54549,'NASA' from dual union all
select 5455,'TF88' from dual union all
select 54550,'NASA' from dual union all
select 54551,'NASA' from dual union all
select 54552,'NASA' from dual union all
select 54553,'RVSN' from dual union all
select 54554,'USN' from dual union all
select 54555,'DFVLR' from dual union all
select 54556,'PLA2AC' from dual union all
select 54557,'AN' from dual union all
select 54558,'AN' from dual union all
select 54559,'DFVLR' from dual union all
select 5456,'SUI' from dual union all
select 54560,'NASA' from dual union all
select 54561,'DMA' from dual union all
select 54562,'USN' from dual union all
select 54563,'USN' from dual union all
select 54564,'RVSN' from dual union all
select 54565,'USN' from dual union all
select 54566,'USN' from dual union all
select 54567,'AFGL2' from dual union all
select 54568,'PRO' from dual union all
select 54569,'USN' from dual union all
select 5457,'SUI' from dual union all
select 54570,'RVSN' from dual union all
select 54571,'NASA' from dual union all
select 54572,'SAC' from dual union all
select 54573,'RVSN' from dual union all
select 54574,'NASA' from dual union all
select 54575,'DMA' from dual union all
select 54576,'USN' from dual union all
select 54577,'USN' from dual union all
select 54578,'AFGL' from dual union all
select 54579,'IAE' from dual union all
select 5458,'SUI' from dual union all
select 54580,'NASA' from dual union all
select 54581,'NASA' from dual union all
select 54582,'RVSN' from dual union all
select 54583,'SAC' from dual union all
select 54584,'RVSN' from dual union all
select 54585,'NASA' from dual union all
select 54586,'USA' from dual union all
select 54587,'SSC' from dual union all
select 54588,'NASA' from dual union all
select 54589,'DLR' from dual union all
select 5459,'SUI' from dual union all
select 54590,'SSC' from dual union all
select 54591,'SSC' from dual union all
select 54592,'SSC' from dual union all
select 54593,'ALCS' from dual union all
select 54593,'SAC' from dual union all
select 54594,'SRC' from dual union all
select 54595,'RVSN' from dual union all
select 54596,'NASA' from dual union all
select 54597,'NASA' from dual union all
select 54598,'SSC' from dual union all
select 54599,'NASA' from dual union all
select 546,'WEHR' from dual union all
select 5460,'SUI' from dual union all
select 54600,'SRC' from dual union all
select 54601,'RVSN' from dual union all
select 54602,'NASDA' from dual union all
select 54603,'ISAS' from dual union all
select 54604,'AFGL' from dual union all
select 54605,'RVSN' from dual union all
select 54606,'IAE' from dual union all
select 54607,'DLR' from dual union all
select 54607,'SSI' from dual union all
select 54608,'ISAS' from dual;
insert into LAUNCH_AGENCY
select 54609,'ISAS' from dual union all
select 5461,'SUI' from dual union all
select 54610,'AFGL' from dual union all
select 54611,'IAE' from dual union all
select 54612,'AFGL2' from dual union all
select 54613,'NRCC' from dual union all
select 54614,'IAE' from dual union all
select 54615,'IAE' from dual union all
select 54616,'RVSN' from dual union all
select 54617,'ALCS' from dual union all
select 54617,'SAC' from dual union all
select 54618,'ISAS' from dual union all
select 54619,'NTNF' from dual union all
select 5462,'SUI' from dual union all
select 54620,'AFSC' from dual union all
select 54621,'PLA2AC' from dual union all
select 54622,'SAC' from dual union all
select 54623,'RVSN' from dual union all
select 54624,'RVSN' from dual union all
select 54625,'RVSN' from dual union all
select 54626,'RVSN' from dual union all
select 54627,'NTNF' from dual union all
select 54628,'NASA' from dual union all
select 54629,'RVSN' from dual union all
select 5463,'SUI' from dual union all
select 54630,'RVSN' from dual union all
select 54631,'NASA' from dual union all
select 54632,'NASA' from dual union all
select 54633,'NASA' from dual union all
select 54634,'USN' from dual union all
select 54635,'RVSN' from dual union all
select 54636,'NASA' from dual union all
select 54637,'USA' from dual union all
select 54638,'USN' from dual union all
select 54639,'NASA' from dual union all
select 5464,'SUI' from dual union all
select 54640,'RVSN' from dual union all
select 54641,'NTNF' from dual union all
select 54642,'SAC' from dual union all
select 54643,'NASA' from dual union all
select 54644,'NTNF' from dual union all
select 54645,'RVSN' from dual union all
select 54646,'RVSN' from dual union all
select 54647,'ISRO' from dual union all
select 54648,'ISRO' from dual union all
select 54649,'ISRO' from dual union all
select 5465,'SUI' from dual union all
select 54650,'ISRO' from dual union all
select 54651,'RVSN' from dual union all
select 54652,'RVSN' from dual union all
select 54653,'RVSN' from dual union all
select 54654,'RVSN' from dual union all
select 54655,'RVSN' from dual union all
select 54656,'RVSN' from dual union all
select 54657,'RVSN' from dual union all
select 54658,'RVSN' from dual union all
select 54659,'RVSN' from dual union all
select 5466,'SUI' from dual union all
select 54660,'RVSN' from dual union all
select 54661,'RVSN' from dual union all
select 54662,'RVSN' from dual union all
select 54663,'RVSN' from dual union all
select 54664,'RVSN' from dual union all
select 54665,'RVSN' from dual union all
select 54666,'RVSN' from dual union all
select 54667,'RVSN' from dual union all
select 54668,'RVSN' from dual union all
select 54669,'RVSN' from dual union all
select 5467,'MVS' from dual union all
select 54670,'RVSN' from dual union all
select 54671,'PLA2AC' from dual union all
select 54672,'PLA2AC' from dual union all
select 54673,'IQ224' from dual union all
select 54674,'IQ224' from dual union all
select 54675,'IQ224' from dual union all
select 54676,'IQ224' from dual union all
select 54677,'IQ224' from dual union all
select 54678,'RVSN' from dual union all
select 54679,'IQ224' from dual union all
select 5468,'AN' from dual union all
select 54680,'IQ224' from dual union all
select 54681,'IQ224' from dual union all
select 54682,'IQ224' from dual union all
select 54683,'IQ224' from dual union all
select 54684,'IQ224' from dual union all
select 54685,'IQ224' from dual union all
select 54686,'IQ224' from dual union all
select 54687,'IQ224' from dual union all
select 54688,'IQ224' from dual union all
select 54689,'RVSN' from dual union all
select 5469,'MVS' from dual union all
select 54690,'IQ224' from dual union all
select 54691,'IQ224' from dual union all
select 54692,'IQ224' from dual union all
select 54693,'IQ224' from dual union all
select 54694,'IQ224' from dual union all
select 54695,'IQ224' from dual union all
select 54696,'IQ224' from dual union all
select 54697,'IQ224' from dual union all
select 54698,'IQ224' from dual;
insert into LAUNCH_AGENCY
select 54699,'IQ224' from dual union all
select 547,'WEHR' from dual union all
select 5470,'MVS' from dual union all
select 54700,'RVSN' from dual union all
select 54701,'IQ224' from dual union all
select 54702,'USAF' from dual union all
select 54703,'FKG' from dual union all
select 54704,'FKG' from dual union all
select 54705,'FKG' from dual union all
select 54706,'FKG' from dual union all
select 54707,'RVSN' from dual union all
select 54708,'RVSN' from dual union all
select 54709,'RVSN' from dual union all
select 5471,'AN' from dual union all
select 54710,'RVSN' from dual union all
select 54711,'RVSN' from dual union all
select 54712,'RVSN' from dual union all
select 54713,'RVSN' from dual union all
select 54714,'RVSN' from dual union all
select 54715,'RVSN' from dual union all
select 54716,'RVSN' from dual union all
select 54717,'RVSN' from dual union all
select 54718,'RVSN' from dual union all
select 54719,'RVSN' from dual union all
select 5472,'MVS' from dual union all
select 54720,'RVSN' from dual union all
select 54721,'RVSN' from dual union all
select 54722,'RVSN' from dual union all
select 54723,'INTA' from dual union all
select 54724,'INTA' from dual union all
select 54725,'INTA' from dual union all
select 54726,'INTA' from dual union all
select 54727,'INTA' from dual union all
select 54728,'INTA' from dual union all
select 54729,'INTA' from dual union all
select 5473,'AN' from dual union all
select 54730,'RVSN' from dual union all
select 54731,'RVSN' from dual union all
select 54732,'RVSN' from dual union all
select 54733,'RVSN' from dual union all
select 54734,'MVS' from dual union all
select 54735,'RVSN' from dual union all
select 54736,'RVSN' from dual union all
select 54737,'RVSN' from dual union all
select 54738,'RVSN' from dual union all
select 54739,'ISAS' from dual union all
select 5474,'MVS' from dual union all
select 54740,'USN' from dual union all
select 54741,'NASA' from dual union all
select 54742,'USA' from dual union all
select 54743,'USA' from dual union all
select 54744,'USA' from dual union all
select 54745,'NASA' from dual union all
select 54746,'AFGL2' from dual union all
select 54747,'NASA' from dual union all
select 54748,'NASA' from dual union all
select 54749,'NASA' from dual union all
select 5475,'AN' from dual union all
select 54750,'USN' from dual union all
select 54751,'NASA' from dual union all
select 54752,'SAC' from dual union all
select 54753,'RN' from dual union all
select 54754,'AN' from dual union all
select 54755,'AN' from dual union all
select 54756,'RN' from dual union all
select 54757,'RN' from dual union all
select 54758,'RN' from dual union all
select 54759,'ISAS' from dual union all
select 5476,'MVS' from dual union all
select 54760,'ISAS' from dual union all
select 54761,'ISAS' from dual union all
select 54762,'NASA' from dual union all
select 54763,'USA' from dual union all
select 54764,'USA' from dual union all
select 54765,'USA' from dual union all
select 54766,'NASA' from dual union all
select 54767,'AN' from dual union all
select 54768,'AN' from dual union all
select 54769,'AN' from dual union all
select 5477,'AN' from dual union all
select 54770,'AN' from dual union all
select 54771,'AN' from dual union all
select 54772,'AN' from dual union all
select 54773,'AN' from dual union all
select 54774,'AN' from dual union all
select 54775,'USN' from dual union all
select 54776,'USN' from dual union all
select 54777,'NASA' from dual union all
select 54778,'DMA' from dual union all
select 54779,'USA' from dual union all
select 5478,'MVS' from dual union all
select 54780,'USA' from dual union all
select 54781,'USA' from dual union all
select 54782,'NASA' from dual union all
select 54783,'INTA' from dual union all
select 54784,'USN' from dual union all
select 54785,'NASA' from dual union all
select 54786,'USN' from dual union all
select 54787,'USN' from dual union all
select 54788,'USN' from dual;
insert into LAUNCH_AGENCY
select 54789,'USN' from dual union all
select 5479,'AN' from dual union all
select 54790,'SAC' from dual union all
select 54791,'SRC' from dual union all
select 54792,'SRC' from dual union all
select 54793,'SRC' from dual union all
select 54794,'SRC' from dual union all
select 54795,'SRC' from dual union all
select 54796,'SRC' from dual union all
select 54797,'SRC' from dual union all
select 54798,'NASA' from dual union all
select 54799,'AN' from dual union all
select 548,'WER902' from dual union all
select 5480,'AN' from dual union all
select 54800,'AN' from dual union all
select 54801,'AN' from dual union all
select 54802,'AN' from dual union all
select 54803,'AN' from dual union all
select 54804,'GMS' from dual union all
select 54805,'MRN' from dual union all
select 54806,'MRN' from dual union all
select 54807,'MRN' from dual union all
select 54808,'MRN' from dual union all
select 54809,'JMA' from dual union all
select 5481,'AN' from dual union all
select 54810,'GMS' from dual union all
select 54811,'MRN' from dual union all
select 54812,'MRN' from dual union all
select 54813,'MRN' from dual union all
select 54814,'MRN' from dual union all
select 54815,'MRN' from dual union all
select 54816,'MRN' from dual union all
select 54817,'AN' from dual union all
select 54818,'MRN' from dual union all
select 54819,'MRN' from dual union all
select 5482,'MVS' from dual union all
select 54820,'MRN' from dual union all
select 54821,'MRN' from dual union all
select 54822,'MRN' from dual union all
select 54823,'GMS' from dual union all
select 54824,'AN' from dual union all
select 54825,'MRN' from dual union all
select 54826,'MRN' from dual union all
select 54827,'MRN' from dual union all
select 54828,'MRN' from dual union all
select 54829,'CNIE' from dual union all
select 54829,'NASA' from dual union all
select 5483,'AN' from dual union all
select 54830,'CNIE' from dual union all
select 54830,'NASA' from dual union all
select 54831,'MRN' from dual union all
select 54832,'MRN' from dual union all
select 54833,'MRN' from dual union all
select 54834,'MRN' from dual union all
select 54835,'GMS' from dual union all
select 54836,'MRN' from dual union all
select 54837,'JMA' from dual union all
select 54838,'GMS' from dual union all
select 54839,'MRN' from dual union all
select 5484,'MVS' from dual union all
select 54840,'CNIE' from dual union all
select 54840,'NASA' from dual union all
select 54841,'MRN' from dual union all
select 54842,'MRN' from dual union all
select 54843,'MRN' from dual union all
select 54844,'MRN' from dual union all
select 54845,'GMS' from dual union all
select 54846,'MRN' from dual union all
select 54847,'MRN' from dual union all
select 54848,'MRN' from dual union all
select 54849,'MRN' from dual union all
select 5485,'AN' from dual union all
select 54850,'MRN' from dual union all
select 54851,'GMS' from dual union all
select 54852,'CNIE' from dual union all
select 54852,'NASA' from dual union all
select 54853,'GMS' from dual union all
select 54854,'MRN' from dual union all
select 54855,'MRN' from dual union all
select 54856,'MRN' from dual union all
select 54857,'MRN' from dual union all
select 54858,'NASA' from dual union all
select 54859,'GMS' from dual union all
select 5486,'AN' from dual union all
select 54860,'MRN' from dual union all
select 54861,'MRN' from dual union all
select 54862,'MRN' from dual union all
select 54863,'MRN' from dual union all
select 54864,'NASA' from dual union all
select 54865,'AN' from dual union all
select 54866,'MRN' from dual union all
select 54867,'GMS' from dual union all
select 54868,'MRN' from dual union all
select 54869,'CNIE' from dual union all
select 54869,'NASA' from dual union all
select 5487,'MVS' from dual union all
select 54870,'NASA' from dual union all
select 54871,'GMS' from dual union all
select 54872,'NASA' from dual union all
select 54873,'MRN' from dual;
insert into LAUNCH_AGENCY
select 54874,'MRN' from dual union all
select 54875,'MRN' from dual union all
select 54876,'MRN' from dual union all
select 54877,'NASA' from dual union all
select 54878,'MRN' from dual union all
select 54879,'MRN' from dual union all
select 5488,'AN' from dual union all
select 54880,'MRN' from dual union all
select 54881,'MRN' from dual union all
select 54882,'NASA' from dual union all
select 54883,'AN' from dual union all
select 54884,'NASA' from dual union all
select 54885,'MRN' from dual union all
select 54886,'MRN' from dual union all
select 54887,'NASA' from dual union all
select 54888,'MRN' from dual union all
select 54889,'MRN' from dual union all
select 5489,'AN' from dual union all
select 54890,'MRN' from dual union all
select 54891,'NASA' from dual union all
select 54892,'MRN' from dual union all
select 54893,'NASA' from dual union all
select 54894,'AN' from dual union all
select 54895,'NASA' from dual union all
select 54896,'NASA' from dual union all
select 54897,'NASA' from dual union all
select 54898,'NASA' from dual union all
select 54899,'MRN' from dual union all
select 549,'WER836' from dual union all
select 5490,'MVS' from dual union all
select 54900,'MRN' from dual union all
select 54901,'MRN' from dual union all
select 54902,'NASA' from dual union all
select 54903,'MRN' from dual union all
select 54904,'NASA' from dual union all
select 54905,'MRN' from dual union all
select 54906,'NASA' from dual union all
select 54907,'NASA' from dual union all
select 54908,'NASA' from dual union all
select 54909,'AN' from dual union all
select 5491,'MVS' from dual union all
select 54910,'MRN' from dual union all
select 54911,'NASA' from dual union all
select 54912,'MRN' from dual union all
select 54913,'MRN' from dual union all
select 54914,'MRN' from dual union all
select 54915,'NASA' from dual union all
select 54916,'MRN' from dual union all
select 54917,'MRN' from dual union all
select 54918,'NASA' from dual union all
select 54919,'MRN' from dual union all
select 5492,'AN' from dual union all
select 54920,'MRN' from dual union all
select 54921,'JMA' from dual union all
select 54922,'JMA' from dual union all
select 54923,'AN' from dual union all
select 54924,'MRN' from dual union all
select 54925,'MRN' from dual union all
select 54926,'MRN' from dual union all
select 54927,'NASA' from dual union all
select 54928,'MRN' from dual union all
select 54929,'MRN' from dual union all
select 5493,'MVS' from dual union all
select 54930,'MRN' from dual union all
select 54931,'MRN' from dual union all
select 54932,'GMS' from dual union all
select 54933,'MRN' from dual union all
select 54934,'MRN' from dual union all
select 54935,'MRN' from dual union all
select 54936,'MRN' from dual union all
select 54937,'MRN' from dual union all
select 54938,'MRN' from dual union all
select 54939,'MRN' from dual union all
select 5494,'MVS' from dual union all
select 54940,'MRN' from dual union all
select 54941,'NASA' from dual union all
select 54942,'NASA' from dual union all
select 54943,'MRN' from dual union all
select 54944,'MRN' from dual union all
select 54945,'MRN' from dual union all
select 54946,'MRN' from dual union all
select 54947,'MRN' from dual union all
select 54948,'MRN' from dual union all
select 54949,'NASA' from dual union all
select 5495,'AN' from dual union all
select 54950,'MRN' from dual union all
select 54951,'MRN' from dual union all
select 54952,'MRN' from dual union all
select 54953,'MRN' from dual union all
select 54954,'MRN' from dual union all
select 54955,'AN' from dual union all
select 54956,'MRN' from dual union all
select 54957,'MRN' from dual union all
select 54958,'MRN' from dual union all
select 54959,'MRN' from dual union all
select 5496,'MVS' from dual union all
select 54960,'MRN' from dual union all
select 54961,'JMA' from dual union all
select 54962,'GMS' from dual union all
select 54963,'MRN' from dual;
insert into LAUNCH_AGENCY
select 54964,'MRN' from dual union all
select 54965,'MRN' from dual union all
select 54966,'GMS' from dual union all
select 54967,'MRN' from dual union all
select 54968,'AN' from dual union all
select 54969,'JMA' from dual union all
select 5497,'AN' from dual union all
select 54970,'MRN' from dual union all
select 54971,'MRN' from dual union all
select 54972,'MRN' from dual union all
select 54973,'MRN' from dual union all
select 54974,'MRN' from dual union all
select 54975,'GMS' from dual union all
select 54976,'MRN' from dual union all
select 54977,'MRN' from dual union all
select 54978,'MRN' from dual union all
select 54979,'MRN' from dual union all
select 5498,'MVS' from dual union all
select 54980,'NASA' from dual union all
select 54981,'NASA' from dual union all
select 54982,'NASA' from dual union all
select 54983,'NASA' from dual union all
select 54984,'MRN' from dual union all
select 54985,'JMA' from dual union all
select 54986,'GMS' from dual union all
select 54987,'MRN' from dual union all
select 54988,'MRN' from dual union all
select 54989,'MRN' from dual union all
select 5499,'AN' from dual union all
select 54990,'MRN' from dual union all
select 54991,'MRN' from dual union all
select 54992,'GMS' from dual union all
select 54993,'MRN' from dual union all
select 54994,'MRN' from dual union all
select 54995,'MRN' from dual union all
select 54996,'MRN' from dual union all
select 54997,'MRN' from dual union all
select 54998,'MRN' from dual union all
select 54999,'MRN' from dual union all
select 55,'WER836-1' from dual union all
select 550,'WER485' from dual union all
select 5500,'MVS' from dual union all
select 55000,'GMS' from dual union all
select 55001,'MRN' from dual union all
select 55002,'MRN' from dual union all
select 55003,'GMS' from dual union all
select 55004,'GMS' from dual union all
select 55005,'MRN' from dual union all
select 55006,'GMS' from dual union all
select 55007,'MRN' from dual union all
select 55008,'MRN' from dual union all
select 55009,'MRN' from dual union all
select 5501,'MVS' from dual union all
select 55010,'GMS' from dual union all
select 55011,'MRN' from dual union all
select 55012,'GMS' from dual union all
select 55013,'MRN' from dual union all
select 55014,'MRN' from dual union all
select 55015,'JMA' from dual union all
select 55016,'GMS' from dual union all
select 55017,'MRN' from dual union all
select 55018,'MRN' from dual union all
select 55019,'GMS' from dual union all
select 5502,'AN' from dual union all
select 55020,'GMS' from dual union all
select 55021,'MRN' from dual union all
select 55022,'GMS' from dual union all
select 55023,'MRN' from dual union all
select 55024,'MRN' from dual union all
select 55025,'MRN' from dual union all
select 55026,'MRN' from dual union all
select 55027,'MRN' from dual union all
select 55028,'GMS' from dual union all
select 55029,'MRN' from dual union all
select 5503,'MVS' from dual union all
select 55030,'GMS' from dual union all
select 55031,'NASA' from dual union all
select 55032,'NASA' from dual union all
select 55033,'MRN' from dual union all
select 55034,'JMA' from dual union all
select 55035,'GMS' from dual union all
select 55036,'MRN' from dual union all
select 55037,'MRN' from dual union all
select 55038,'MRN' from dual union all
select 55039,'MRN' from dual union all
select 5504,'MVS' from dual union all
select 55040,'MRN' from dual union all
select 55041,'MRN' from dual union all
select 55042,'NASA' from dual union all
select 55043,'MRN' from dual union all
select 55044,'MRN' from dual union all
select 55045,'AN' from dual union all
select 55046,'JMA' from dual union all
select 55047,'MRN' from dual union all
select 55048,'MRN' from dual union all
select 55049,'MRN' from dual union all
select 5505,'AN' from dual union all
select 55050,'MRN' from dual union all
select 55051,'MRN' from dual union all
select 55052,'MRN' from dual;
insert into LAUNCH_AGENCY
select 55053,'NASA' from dual union all
select 55054,'NASA' from dual union all
select 55055,'MRN' from dual union all
select 55056,'JMA' from dual union all
select 55057,'MRN' from dual union all
select 55058,'MRN' from dual union all
select 55059,'MRN' from dual union all
select 5506,'AN' from dual union all
select 55060,'MRN' from dual union all
select 55061,'MRN' from dual union all
select 55062,'MRN' from dual union all
select 55063,'MRN' from dual union all
select 55064,'MRN' from dual union all
select 55065,'MRN' from dual union all
select 55066,'JMA' from dual union all
select 55067,'MRN' from dual union all
select 55068,'MRN' from dual union all
select 55069,'MRN' from dual union all
select 5507,'MVS' from dual union all
select 55070,'MRN' from dual union all
select 55071,'MRN' from dual union all
select 55072,'MRN' from dual union all
select 55073,'MRN' from dual union all
select 55074,'MRN' from dual union all
select 55075,'NASA' from dual union all
select 55076,'MRN' from dual union all
select 55077,'MRN' from dual union all
select 55078,'MRN' from dual union all
select 55079,'MRN' from dual union all
select 5508,'AN' from dual union all
select 55080,'MRN' from dual union all
select 55081,'MRN' from dual union all
select 55082,'MRN' from dual union all
select 55083,'NASA' from dual union all
select 55084,'MRN' from dual union all
select 55085,'MRN' from dual union all
select 55086,'MRN' from dual union all
select 55087,'MRN' from dual union all
select 55088,'MRN' from dual union all
select 55089,'MRN' from dual union all
select 5509,'MVS' from dual union all
select 55090,'MRN' from dual union all
select 55091,'MRN' from dual union all
select 55092,'JMA' from dual union all
select 55093,'MRN' from dual union all
select 55094,'MRN' from dual union all
select 55095,'MRN' from dual union all
select 55096,'MRN' from dual union all
select 55097,'MRN' from dual union all
select 55098,'MRN' from dual union all
select 55099,'MRN' from dual union all
select 551,'AVKO' from dual union all
select 551,'SPOG' from dual union all
select 5510,'AN' from dual union all
select 55100,'MRN' from dual union all
select 55101,'MRN' from dual union all
select 55102,'MRN' from dual union all
select 55103,'MRN' from dual union all
select 55104,'MRN' from dual union all
select 55105,'MRN' from dual union all
select 55106,'MRN' from dual union all
select 55107,'MRN' from dual union all
select 55108,'MRN' from dual union all
select 55109,'MRN' from dual union all
select 5511,'AN' from dual union all
select 55110,'JMA' from dual union all
select 55111,'MRN' from dual union all
select 55112,'MRN' from dual union all
select 55113,'MRN' from dual union all
select 55114,'MRN' from dual union all
select 55115,'MRN' from dual union all
select 55116,'MRN' from dual union all
select 55117,'JMA' from dual union all
select 55118,'MRN' from dual union all
select 55119,'MRN' from dual union all
select 5512,'AN' from dual union all
select 55120,'MRN' from dual union all
select 55121,'MRN' from dual union all
select 55122,'MRN' from dual union all
select 55123,'MRN' from dual union all
select 55124,'JMA' from dual union all
select 55125,'MRN' from dual union all
select 55126,'MRN' from dual union all
select 55127,'MRN' from dual union all
select 55128,'MRN' from dual union all
select 55129,'MRN' from dual union all
select 5513,'AN' from dual union all
select 55130,'MRN' from dual union all
select 55131,'MRN' from dual union all
select 55132,'MRN' from dual union all
select 55133,'MRN' from dual union all
select 55134,'MRN' from dual union all
select 55135,'MRN' from dual union all
select 55136,'MRN' from dual union all
select 55137,'JMA' from dual union all
select 55138,'MRN' from dual union all
select 55139,'MRN' from dual union all
select 5514,'AN' from dual union all
select 55140,'MRN' from dual union all
select 55141,'NASA' from dual;
insert into LAUNCH_AGENCY
select 55142,'NASA' from dual union all
select 55143,'NASDA' from dual union all
select 55144,'MRN' from dual union all
select 55145,'JMA' from dual union all
select 55146,'MRN' from dual union all
select 55147,'NASA' from dual union all
select 55148,'JMA' from dual union all
select 55149,'MRN' from dual union all
select 5515,'AN' from dual union all
select 55150,'NASA' from dual union all
select 55151,'MRN' from dual union all
select 55152,'NASA' from dual union all
select 55153,'MRN' from dual union all
select 55154,'MRN' from dual union all
select 55155,'MRN' from dual union all
select 55156,'MRN' from dual union all
select 55157,'MRN' from dual union all
select 55158,'MRN' from dual union all
select 55159,'MRN' from dual union all
select 5516,'AN' from dual union all
select 55160,'MRN' from dual union all
select 55161,'JMA' from dual union all
select 55162,'NASA' from dual union all
select 55163,'AN' from dual union all
select 55164,'MRN' from dual union all
select 55165,'MRN' from dual union all
select 55166,'MRN' from dual union all
select 55167,'MRN' from dual union all
select 55168,'MRN' from dual union all
select 55169,'MRN' from dual union all
select 5517,'AN' from dual union all
select 55170,'MRN' from dual union all
select 55171,'JMA' from dual union all
select 55172,'MRN' from dual union all
select 55173,'MRN' from dual union all
select 55174,'NASA' from dual union all
select 55175,'MRN' from dual union all
select 55176,'MRN' from dual union all
select 55177,'MRN' from dual union all
select 55178,'MRN' from dual union all
select 55179,'MRN' from dual union all
select 5518,'AN' from dual union all
select 55180,'JMA' from dual union all
select 55181,'MRN' from dual union all
select 55182,'NASA' from dual union all
select 55183,'JMA' from dual union all
select 55184,'MRN' from dual union all
select 55185,'MRN' from dual union all
select 55186,'MRN' from dual union all
select 55187,'MRN' from dual union all
select 55188,'GMS' from dual union all
select 55189,'MRN' from dual union all
select 5519,'AN' from dual union all
select 55190,'MRN' from dual union all
select 55191,'GMS' from dual union all
select 55192,'AN' from dual union all
select 55193,'MRN' from dual union all
select 55194,'GMS' from dual union all
select 55195,'JMA' from dual union all
select 55196,'MRN' from dual union all
select 55197,'GMS' from dual union all
select 55198,'AN' from dual union all
select 55199,'MRN' from dual union all
select 552,'AVKO' from dual union all
select 552,'SPOG' from dual union all
select 5520,'AN' from dual union all
select 55200,'MRN' from dual union all
select 55201,'MRN' from dual union all
select 55202,'MRN' from dual union all
select 55203,'GMS' from dual union all
select 55204,'MRN' from dual union all
select 55205,'MRN' from dual union all
select 55206,'MRN' from dual union all
select 55207,'MRN' from dual union all
select 55208,'GMS' from dual union all
select 55209,'MRN' from dual union all
select 5521,'AN' from dual union all
select 55210,'MRN' from dual union all
select 55211,'NASA' from dual union all
select 55212,'MRN' from dual union all
select 55213,'NASA' from dual union all
select 55214,'MRN' from dual union all
select 55215,'MRN' from dual union all
select 55216,'MRN' from dual union all
select 55217,'MRN' from dual union all
select 55218,'JMA' from dual union all
select 55219,'MRN' from dual union all
select 5522,'AN' from dual union all
select 55220,'MRN' from dual union all
select 55221,'MRN' from dual union all
select 55222,'JMA' from dual union all
select 55223,'GMS' from dual union all
select 55224,'MRN' from dual union all
select 55225,'GMS' from dual union all
select 55226,'GMS' from dual union all
select 55227,'NASA' from dual union all
select 55228,'MRN' from dual union all
select 55229,'MRN' from dual union all
select 5523,'AN' from dual union all
select 55230,'MRN' from dual;
insert into LAUNCH_AGENCY
select 55231,'GMS' from dual union all
select 55232,'MRN' from dual union all
select 55233,'GMS' from dual union all
select 55234,'MRN' from dual union all
select 55235,'GMS' from dual union all
select 55236,'MRN' from dual union all
select 55237,'MRN' from dual union all
select 55238,'JMA' from dual union all
select 55239,'NASA' from dual union all
select 5524,'AN' from dual union all
select 55240,'GMS' from dual union all
select 55241,'GMS' from dual union all
select 55242,'NASA' from dual union all
select 55243,'GMS' from dual union all
select 55244,'MRN' from dual union all
select 55245,'JMA' from dual union all
select 55246,'GMS' from dual union all
select 55247,'AN' from dual union all
select 55248,'MRN' from dual union all
select 55249,'MRN' from dual union all
select 5525,'USN' from dual union all
select 55250,'MRN' from dual union all
select 55251,'MRN' from dual union all
select 55252,'JMA' from dual union all
select 55253,'AN' from dual union all
select 55254,'MRN' from dual union all
select 55255,'MRN' from dual union all
select 55256,'MRN' from dual union all
select 55257,'GMS' from dual union all
select 55258,'NASA' from dual union all
select 55259,'MRN' from dual union all
select 5526,'USAF' from dual union all
select 55260,'MRN' from dual union all
select 55261,'GMS' from dual union all
select 55262,'MRN' from dual union all
select 55263,'JMA' from dual union all
select 55264,'GMS' from dual union all
select 55265,'GMS' from dual union all
select 55266,'MRN' from dual union all
select 55267,'MRN' from dual union all
select 55268,'JMA' from dual union all
select 55269,'AFGL' from dual union all
select 5527,'USAF' from dual union all
select 55270,'AFGL' from dual union all
select 55271,'NASA' from dual union all
select 55272,'NASA' from dual union all
select 55273,'NASA' from dual union all
select 55274,'NASA' from dual union all
select 55275,'NASA' from dual union all
select 55276,'NASA' from dual union all
select 55277,'NASA' from dual union all
select 55278,'NASA' from dual union all
select 55279,'MRN' from dual union all
select 5528,'USAF' from dual union all
select 55280,'MRN' from dual union all
select 55281,'MRN' from dual union all
select 55282,'MRN' from dual union all
select 55283,'JMA' from dual union all
select 55284,'GMS' from dual union all
select 55285,'GMS' from dual union all
select 55286,'AN' from dual union all
select 55287,'MRN' from dual union all
select 55288,'MRN' from dual union all
select 55289,'MRN' from dual union all
select 5529,'USN' from dual union all
select 55290,'MRN' from dual union all
select 55291,'AN' from dual union all
select 55292,'GMS' from dual union all
select 55293,'MRN' from dual union all
select 55294,'MRN' from dual union all
select 55295,'MRN' from dual union all
select 55296,'MRN' from dual union all
select 55297,'MRN' from dual union all
select 55298,'MRN' from dual union all
select 55299,'NASA' from dual union all
select 553,'WEHR' from dual union all
select 5530,'USAF' from dual union all
select 55300,'NASA' from dual union all
select 55301,'NASA' from dual union all
select 55302,'MRN' from dual union all
select 55303,'MRN' from dual union all
select 55304,'MRN' from dual union all
select 55305,'NASDA' from dual union all
select 55306,'MRN' from dual union all
select 55307,'NASDA' from dual union all
select 55308,'JMA' from dual union all
select 55309,'MRN' from dual union all
select 5531,'NASA' from dual union all
select 55310,'MRN' from dual union all
select 55311,'MRN' from dual union all
select 55312,'MRN' from dual union all
select 55313,'MRN' from dual union all
select 55314,'MRN' from dual union all
select 55315,'MRN' from dual union all
select 55316,'MRN' from dual union all
select 55317,'MRN' from dual union all
select 55318,'MRN' from dual union all
select 55319,'AN' from dual union all
select 5532,'USAF' from dual union all
select 55320,'AN' from dual;
insert into LAUNCH_AGENCY
select 55321,'AN' from dual union all
select 55322,'AN' from dual union all
select 55323,'AN' from dual union all
select 55324,'AN' from dual union all
select 55325,'MRN' from dual union all
select 55326,'MRN' from dual union all
select 55327,'AN' from dual union all
select 55328,'AN' from dual union all
select 55329,'AN' from dual union all
select 5533,'USAF' from dual union all
select 55330,'AN' from dual union all
select 55331,'AN' from dual union all
select 55332,'AN' from dual union all
select 55333,'MRN' from dual union all
select 55334,'MRN' from dual union all
select 55335,'AN' from dual union all
select 55336,'MRN' from dual union all
select 55337,'AN' from dual union all
select 55338,'MRN' from dual union all
select 55339,'MRN' from dual union all
select 5534,'MVS' from dual union all
select 55340,'MRN' from dual union all
select 55341,'AN' from dual union all
select 55342,'AN' from dual union all
select 55343,'MRN' from dual union all
select 55344,'AN' from dual union all
select 55345,'MRN' from dual union all
select 55346,'AN' from dual union all
select 55347,'AN' from dual union all
select 55348,'AN' from dual union all
select 55349,'MRN' from dual union all
select 5535,'USA' from dual union all
select 55350,'MRN' from dual union all
select 55351,'NASA' from dual union all
select 55352,'MRN' from dual union all
select 55353,'MRN' from dual union all
select 55354,'MRN' from dual union all
select 55355,'MRN' from dual union all
select 55356,'MRN' from dual union all
select 55357,'AN' from dual union all
select 55358,'AN' from dual union all
select 55359,'AN' from dual union all
select 5536,'MVS' from dual union all
select 55360,'AN' from dual union all
select 55361,'AN' from dual union all
select 55362,'MRN' from dual union all
select 55363,'AN' from dual union all
select 55364,'AN' from dual union all
select 55365,'AN' from dual union all
select 55366,'AN' from dual union all
select 55367,'AN' from dual union all
select 55368,'AN' from dual union all
select 55369,'AN' from dual union all
select 5537,'MVS' from dual union all
select 55370,'AN' from dual union all
select 55371,'AN' from dual union all
select 55372,'AN' from dual union all
select 55373,'MRN' from dual union all
select 55374,'AN' from dual union all
select 55375,'AN' from dual union all
select 55376,'MRN' from dual union all
select 55377,'AN' from dual union all
select 55378,'AN' from dual union all
select 55379,'AN' from dual union all
select 5538,'USAF' from dual union all
select 55380,'AN' from dual union all
select 55381,'MRN' from dual union all
select 55382,'AN' from dual union all
select 55383,'MRN' from dual union all
select 55384,'AN' from dual union all
select 55385,'AN' from dual union all
select 55386,'AN' from dual union all
select 55387,'AN' from dual union all
select 55388,'MRN' from dual union all
select 55389,'AN' from dual union all
select 5539,'LRL' from dual union all
select 55390,'MRN' from dual union all
select 55391,'AN' from dual union all
select 55392,'AN' from dual union all
select 55393,'MRN' from dual union all
select 55394,'AN' from dual union all
select 55395,'AN' from dual union all
select 55396,'AN' from dual union all
select 55397,'AN' from dual union all
select 55398,'MRN' from dual union all
select 55399,'AN' from dual union all
select 554,'WEHR' from dual union all
select 5540,'USN' from dual union all
select 55400,'AN' from dual union all
select 55401,'AN' from dual union all
select 55402,'AN' from dual union all
select 55403,'AN' from dual union all
select 55404,'MRN' from dual union all
select 55405,'MRN' from dual union all
select 55406,'AN' from dual union all
select 55407,'MRN' from dual union all
select 55408,'AN' from dual union all
select 55409,'AN' from dual union all
select 5541,'NASA' from dual union all
select 55410,'MRN' from dual;
insert into LAUNCH_AGENCY
select 55411,'AN' from dual union all
select 55412,'AN' from dual union all
select 55413,'AN' from dual union all
select 55414,'AN' from dual union all
select 55415,'AN' from dual union all
select 55416,'AN' from dual union all
select 55417,'MRN' from dual union all
select 55418,'AN' from dual union all
select 55419,'AN' from dual union all
select 5542,'NASA' from dual union all
select 55420,'AN' from dual union all
select 55421,'AN' from dual union all
select 55422,'AN' from dual union all
select 55423,'AN' from dual union all
select 55424,'MRN' from dual union all
select 55425,'AN' from dual union all
select 55426,'AN' from dual union all
select 55427,'AN' from dual union all
select 55428,'AN' from dual union all
select 55429,'MRN' from dual union all
select 5543,'NASA' from dual union all
select 55430,'AN' from dual union all
select 55431,'AN' from dual union all
select 55432,'AN' from dual union all
select 55433,'AN' from dual union all
select 55434,'AN' from dual union all
select 55435,'MRN' from dual union all
select 55436,'AN' from dual union all
select 55437,'AN' from dual union all
select 55438,'AN' from dual union all
select 55439,'AN' from dual union all
select 5544,'VMF' from dual union all
select 55440,'AN' from dual union all
select 55441,'AN' from dual union all
select 55442,'AN' from dual union all
select 55443,'AN' from dual union all
select 55444,'AN' from dual union all
select 55445,'AN' from dual union all
select 55446,'AN' from dual union all
select 55447,'AN' from dual union all
select 55448,'NASA' from dual union all
select 55449,'AN' from dual union all
select 5545,'LARCN' from dual union all
select 55450,'AN' from dual union all
select 55451,'MRN' from dual union all
select 55452,'MRN' from dual union all
select 55453,'NASA' from dual union all
select 55454,'MRN' from dual union all
select 55455,'MRN' from dual union all
select 55456,'NASA' from dual union all
select 55457,'MRN' from dual union all
select 55458,'MRN' from dual union all
select 55459,'AN' from dual union all
select 5546,'LARCN' from dual union all
select 55460,'AN' from dual union all
select 55461,'AN' from dual union all
select 55462,'AN' from dual union all
select 55463,'AN' from dual union all
select 55464,'MRN' from dual union all
select 55465,'AN' from dual union all
select 55466,'AN' from dual union all
select 55467,'AN' from dual union all
select 55468,'MRN' from dual union all
select 55469,'AN' from dual union all
select 5547,'USAF' from dual union all
select 55470,'AN' from dual union all
select 55471,'AN' from dual union all
select 55472,'AN' from dual union all
select 55473,'MRN' from dual union all
select 55474,'MRN' from dual union all
select 55475,'MRN' from dual union all
select 55476,'MRN' from dual union all
select 55477,'AN' from dual union all
select 55478,'NASA' from dual union all
select 55479,'NASA' from dual union all
select 5548,'USAF' from dual union all
select 55480,'MRN' from dual union all
select 55481,'AN' from dual union all
select 55482,'MRN' from dual union all
select 55483,'MRN' from dual union all
select 55484,'MRN' from dual union all
select 55485,'MRN' from dual union all
select 55486,'MRN' from dual union all
select 55487,'MRN' from dual union all
select 55488,'AN' from dual union all
select 55489,'AN' from dual union all
select 5549,'RAF' from dual union all
select 55490,'AN' from dual union all
select 55491,'AN' from dual union all
select 55492,'NASA' from dual union all
select 55493,'AN' from dual union all
select 55494,'MRN' from dual union all
select 55495,'AN' from dual union all
select 55496,'AN' from dual union all
select 55497,'AN' from dual union all
select 55498,'MRN' from dual union all
select 55499,'AN' from dual union all
select 555,'WEHR' from dual union all
select 5550,'NASA' from dual union all
select 55500,'AN' from dual;
insert into LAUNCH_AGENCY
select 55501,'AN' from dual union all
select 55502,'AN' from dual union all
select 55503,'AN' from dual union all
select 55504,'MRN' from dual union all
select 55505,'AN' from dual union all
select 55506,'AN' from dual union all
select 55507,'AN' from dual union all
select 55508,'AN' from dual union all
select 55509,'AN' from dual union all
select 5551,'RAE' from dual union all
select 55510,'AN' from dual union all
select 55511,'AN' from dual union all
select 55512,'AN' from dual union all
select 55513,'AN' from dual union all
select 55514,'AN' from dual union all
select 55515,'MRN' from dual union all
select 55516,'MRN' from dual union all
select 55517,'MRN' from dual union all
select 55518,'AN' from dual union all
select 55519,'AN' from dual union all
select 5552,'USA' from dual union all
select 55520,'MRN' from dual union all
select 55521,'AN' from dual union all
select 55522,'MRN' from dual union all
select 55523,'AN' from dual union all
select 55524,'AN' from dual union all
select 55525,'MRN' from dual union all
select 55526,'AN' from dual union all
select 55527,'AN' from dual union all
select 55528,'AN' from dual union all
select 55529,'AN' from dual union all
select 5553,'USA' from dual union all
select 55530,'AN' from dual union all
select 55531,'AN' from dual union all
select 55532,'MRN' from dual union all
select 55533,'MRN' from dual union all
select 55534,'AN' from dual union all
select 55535,'MRN' from dual union all
select 55536,'AN' from dual union all
select 55537,'MRN' from dual union all
select 55538,'AN' from dual union all
select 55539,'AN' from dual union all
select 5554,'USA' from dual union all
select 55540,'AN' from dual union all
select 55541,'AN' from dual union all
select 55542,'MRN' from dual union all
select 55543,'AN' from dual union all
select 55544,'AN' from dual union all
select 55545,'AN' from dual union all
select 55546,'AN' from dual union all
select 55547,'AN' from dual union all
select 55548,'AN' from dual union all
select 55549,'MRN' from dual union all
select 5555,'USA' from dual union all
select 55550,'AN' from dual union all
select 55551,'MRN' from dual union all
select 55552,'AN' from dual union all
select 55553,'AN' from dual union all
select 55554,'AN' from dual union all
select 55555,'AN' from dual union all
select 55556,'MRN' from dual union all
select 55557,'MRN' from dual union all
select 55558,'MRN' from dual union all
select 55559,'MRN' from dual union all
select 5556,'USA' from dual union all
select 55560,'AN' from dual union all
select 55561,'AN' from dual union all
select 55562,'AN' from dual union all
select 55563,'AN' from dual union all
select 55564,'AN' from dual union all
select 55565,'AN' from dual union all
select 55566,'AN' from dual union all
select 55567,'MRN' from dual union all
select 55568,'AN' from dual union all
select 55569,'MRN' from dual union all
select 5557,'USA' from dual union all
select 55570,'AN' from dual union all
select 55571,'AN' from dual union all
select 55572,'AN' from dual union all
select 55573,'MRN' from dual union all
select 55574,'AN' from dual union all
select 55575,'MRN' from dual union all
select 55576,'AN' from dual union all
select 55577,'AN' from dual union all
select 55578,'AN' from dual union all
select 55579,'AN' from dual union all
select 5558,'USA' from dual union all
select 55580,'AN' from dual union all
select 55581,'AN' from dual union all
select 55582,'MRN' from dual union all
select 55583,'AN' from dual union all
select 55584,'AN' from dual union all
select 55585,'AN' from dual union all
select 55586,'MRN' from dual union all
select 55587,'AN' from dual union all
select 55588,'AN' from dual union all
select 55589,'AN' from dual union all
select 5559,'STA' from dual union all
select 55590,'AN' from dual union all
select 55591,'AN' from dual;
insert into LAUNCH_AGENCY
select 55592,'MRN' from dual union all
select 55593,'AN' from dual union all
select 55594,'MRN' from dual union all
select 55595,'AN' from dual union all
select 55596,'AN' from dual union all
select 55597,'AN' from dual union all
select 55598,'AN' from dual union all
select 55599,'AN' from dual union all
select 556,'WEHR' from dual union all
select 5560,'USA' from dual union all
select 55600,'NASA' from dual union all
select 55601,'MRN' from dual union all
select 55602,'AN' from dual union all
select 55603,'AN' from dual union all
select 55604,'AN' from dual union all
select 55605,'AN' from dual union all
select 55606,'MRN' from dual union all
select 55607,'MRN' from dual union all
select 55608,'AN' from dual union all
select 55609,'AN' from dual union all
select 5561,'USA' from dual union all
select 55610,'AN' from dual union all
select 55611,'AN' from dual union all
select 55612,'AN' from dual union all
select 55613,'AN' from dual union all
select 55614,'AN' from dual union all
select 55615,'AN' from dual union all
select 55616,'MRN' from dual union all
select 55617,'AN' from dual union all
select 55618,'AN' from dual union all
select 55619,'AN' from dual union all
select 5562,'USA' from dual union all
select 55620,'AN' from dual union all
select 55621,'AN' from dual union all
select 55622,'AN' from dual union all
select 55623,'MRN' from dual union all
select 55624,'MRN' from dual union all
select 55625,'MRN' from dual union all
select 55626,'MRN' from dual union all
select 55627,'MRN' from dual union all
select 55628,'MRN' from dual union all
select 55629,'MRN' from dual union all
select 5563,'USAF' from dual union all
select 55630,'MRN' from dual union all
select 55631,'NASA' from dual union all
select 55632,'NASA' from dual union all
select 55633,'NASA' from dual union all
select 55634,'AN' from dual union all
select 55635,'AN' from dual union all
select 55636,'MRN' from dual union all
select 55637,'MRN' from dual union all
select 55638,'MRN' from dual union all
select 55639,'MRN' from dual union all
select 5564,'RAE' from dual union all
select 55640,'AN' from dual union all
select 55641,'MRN' from dual union all
select 55642,'MRN' from dual union all
select 55643,'MRN' from dual union all
select 55644,'AN' from dual union all
select 55645,'AN' from dual union all
select 55646,'AN' from dual union all
select 55647,'AN' from dual union all
select 55648,'AN' from dual union all
select 55649,'AN' from dual union all
select 5565,'PARD' from dual union all
select 55650,'NASA' from dual union all
select 55651,'NASA' from dual union all
select 55652,'NASA' from dual union all
select 55653,'AN' from dual union all
select 55654,'AN' from dual union all
select 55655,'AN' from dual union all
select 55656,'AN' from dual union all
select 55657,'AN' from dual union all
select 55658,'AN' from dual union all
select 55659,'AN' from dual union all
select 5566,'USAF' from dual union all
select 55660,'AN' from dual union all
select 55661,'AN' from dual union all
select 55662,'AN' from dual union all
select 55663,'AN' from dual union all
select 55664,'AN' from dual union all
select 55665,'AN' from dual union all
select 55666,'AN' from dual union all
select 55667,'AN' from dual union all
select 55668,'AN' from dual union all
select 55669,'AN' from dual union all
select 5567,'USN' from dual union all
select 55670,'AN' from dual union all
select 55671,'AN' from dual union all
select 55672,'AN' from dual union all
select 55673,'MRN' from dual union all
select 55674,'AN' from dual union all
select 55675,'AN' from dual union all
select 55676,'AN' from dual union all
select 55677,'AN' from dual union all
select 55678,'AN' from dual union all
select 55679,'AN' from dual union all
select 5568,'USN' from dual union all
select 55680,'AN' from dual union all
select 55681,'AN' from dual;
insert into LAUNCH_AGENCY
select 55682,'AN' from dual union all
select 55683,'AN' from dual union all
select 55684,'NASA' from dual union all
select 55685,'NASA' from dual union all
select 55686,'AN' from dual union all
select 55687,'AN' from dual union all
select 55688,'AN' from dual union all
select 55689,'AN' from dual union all
select 5569,'USAF' from dual union all
select 55690,'AN' from dual union all
select 55691,'AN' from dual union all
select 55692,'AN' from dual union all
select 55693,'AN' from dual union all
select 55694,'AN' from dual union all
select 55695,'AN' from dual union all
select 55696,'AN' from dual union all
select 55697,'AN' from dual union all
select 55698,'AN' from dual union all
select 55699,'AN' from dual union all
select 557,'WEHR' from dual union all
select 5570,'STA' from dual union all
select 55700,'AN' from dual union all
select 55701,'AN' from dual union all
select 55702,'ISRO' from dual union all
select 55703,'AN' from dual union all
select 55704,'AN' from dual union all
select 55705,'AN' from dual union all
select 55706,'AN' from dual union all
select 55707,'AN' from dual union all
select 55708,'AN' from dual union all
select 55709,'AN' from dual union all
select 5571,'NASA' from dual union all
select 5571,'USAF' from dual union all
select 55710,'AN' from dual union all
select 55711,'AN' from dual union all
select 55712,'AN' from dual union all
select 55713,'AN' from dual union all
select 55714,'AN' from dual union all
select 55715,'AN' from dual union all
select 55716,'MRN' from dual union all
select 55717,'NASA' from dual union all
select 55718,'NASA' from dual union all
select 55719,'NASA' from dual union all
select 5572,'BRL' from dual union all
select 55720,'NASA' from dual union all
select 55721,'AN' from dual union all
select 55722,'AN' from dual union all
select 55723,'AN' from dual union all
select 55724,'NASA' from dual union all
select 55725,'AN' from dual union all
select 55726,'AN' from dual union all
select 55727,'AN' from dual union all
select 55728,'NASA' from dual union all
select 55729,'AN' from dual union all
select 5573,'BRL' from dual union all
select 55730,'NASA' from dual union all
select 55731,'AN' from dual union all
select 55732,'AN' from dual union all
select 55733,'AN' from dual union all
select 55734,'AN' from dual union all
select 55735,'MRN' from dual union all
select 55736,'AN' from dual union all
select 55737,'AN' from dual union all
select 55738,'AN' from dual union all
select 55739,'AN' from dual union all
select 5574,'USA' from dual union all
select 55740,'AN' from dual union all
select 55741,'NASA' from dual union all
select 55742,'AN' from dual union all
select 55743,'AN' from dual union all
select 55744,'AN' from dual union all
select 55745,'AN' from dual union all
select 55746,'AN' from dual union all
select 55747,'AN' from dual union all
select 55748,'AN' from dual union all
select 55749,'AN' from dual union all
select 5575,'USA' from dual union all
select 55750,'AN' from dual union all
select 55751,'AN' from dual union all
select 55752,'MRN' from dual union all
select 55753,'MRN' from dual union all
select 55754,'AN' from dual union all
select 55755,'MRN' from dual union all
select 55756,'MRN' from dual union all
select 55757,'AN' from dual union all
select 55758,'UNKS' from dual union all
select 55759,'UNKS' from dual union all
select 5576,'USA' from dual union all
select 55760,'UNKS' from dual union all
select 55761,'NASA' from dual union all
select 55762,'UNKS' from dual union all
select 55763,'NASDA' from dual union all
select 55764,'UNKS' from dual union all
select 55765,'AFSC' from dual union all
select 55766,'UNKS' from dual union all
select 55767,'UNKS' from dual union all
select 55768,'ISAS' from dual union all
select 55769,'UNKS' from dual union all
select 5577,'USA' from dual union all
select 55770,'UNKS' from dual;
insert into LAUNCH_AGENCY
select 55771,'UNKS' from dual union all
select 55772,'UNKS' from dual union all
select 55773,'UNKS' from dual union all
select 55774,'UNKS' from dual union all
select 55775,'UNKS' from dual union all
select 55776,'UNKS' from dual union all
select 55777,'UNKS' from dual union all
select 55778,'UNKS' from dual union all
select 55779,'AFSC' from dual union all
select 5578,'USA' from dual union all
select 55780,'UNKS' from dual union all
select 55781,'UNKS' from dual union all
select 55782,'UNKS' from dual union all
select 55783,'NASA' from dual union all
select 55784,'UNKS' from dual union all
select 55785,'UNKS' from dual union all
select 55786,'UNKS' from dual union all
select 55787,'NASA' from dual union all
select 55788,'UNKS' from dual union all
select 55789,'AFSC' from dual union all
select 5579,'USA' from dual union all
select 55790,'ISRO' from dual union all
select 55791,'UNKS' from dual union all
select 55792,'UNKS' from dual union all
select 55793,'UNKS' from dual union all
select 55794,'UNKS' from dual union all
select 55795,'UNKS' from dual union all
select 55796,'UNKS' from dual union all
select 55797,'UNKS' from dual union all
select 55798,'NASA' from dual union all
select 55799,'UNKS' from dual union all
select 558,'WEHR' from dual union all
select 5580,'USA' from dual union all
select 55800,'UNKS' from dual union all
select 55801,'UNKS' from dual union all
select 55802,'UNKS' from dual union all
select 55803,'UNKS' from dual union all
select 55804,'NASA' from dual union all
select 55805,'UNKS' from dual union all
select 55806,'UNKS' from dual union all
select 55807,'UNKS' from dual union all
select 55808,'NASA' from dual union all
select 55809,'UNKS' from dual union all
select 5581,'PARD' from dual union all
select 55810,'UNKS' from dual union all
select 55811,'UNKS' from dual union all
select 55812,'UNKS' from dual union all
select 55813,'AFSC' from dual union all
select 55814,'UNKS' from dual union all
select 55815,'ESA' from dual union all
select 55816,'NASA' from dual union all
select 55817,'AFSC' from dual union all
select 55818,'UNKS' from dual union all
select 55819,'UNKS' from dual union all
select 5582,'NASA' from dual union all
select 55820,'NASA' from dual union all
select 55821,'UNKS' from dual union all
select 55822,'NASA' from dual union all
select 55823,'UNKS' from dual union all
select 55824,'UNKS' from dual union all
select 55825,'UNKS' from dual union all
select 55826,'UNKS' from dual union all
select 55827,'UNKS' from dual union all
select 55828,'UNKS' from dual union all
select 55829,'AFSC' from dual union all
select 5583,'USN' from dual union all
select 55830,'UNKS' from dual union all
select 55831,'UNKS' from dual union all
select 55832,'UNKS' from dual union all
select 55833,'UNKS' from dual union all
select 55834,'NASA' from dual union all
select 55835,'AFSC' from dual union all
select 55836,'UNKS' from dual union all
select 55837,'UNKS' from dual union all
select 55838,'NASDA' from dual union all
select 55839,'UNKS' from dual union all
select 5584,'NASA' from dual union all
select 55840,'UNKS' from dual union all
select 55841,'UNKS' from dual union all
select 55842,'UNKS' from dual union all
select 55843,'CALT' from dual union all
select 55844,'UNKS' from dual union all
select 55845,'UNKS' from dual union all
select 55846,'NASA' from dual union all
select 55847,'UNKS' from dual union all
select 55848,'UNKS' from dual union all
select 55849,'UNKS' from dual union all
select 5585,'USAF' from dual union all
select 55850,'UNKS' from dual union all
select 55851,'NASA' from dual union all
select 55852,'UNKS' from dual union all
select 55853,'UNKS' from dual union all
select 55854,'UNKS' from dual union all
select 55855,'NASA' from dual union all
select 55856,'UNKS' from dual union all
select 55857,'UNKS' from dual union all
select 55858,'UNKS' from dual union all
select 55859,'UNKS' from dual union all
select 5586,'USN' from dual union all
select 55860,'UNKS' from dual;
insert into LAUNCH_AGENCY
select 55861,'UNKS' from dual union all
select 55862,'ESA' from dual union all
select 55863,'UNKS' from dual union all
select 55864,'UNKS' from dual union all
select 55865,'UNKS' from dual union all
select 55866,'UNKS' from dual union all
select 55867,'UNKS' from dual union all
select 55868,'UNKS' from dual union all
select 55869,'UNKS' from dual union all
select 5587,'MVS' from dual union all
select 55870,'AFSC' from dual union all
select 55871,'UNKS' from dual union all
select 55872,'UNKS' from dual union all
select 55873,'NASA' from dual union all
select 55874,'UNKS' from dual union all
select 55875,'UNKS' from dual union all
select 55876,'UNKS' from dual union all
select 55877,'UNKS' from dual union all
select 55878,'UNKS' from dual union all
select 55879,'UNKS' from dual union all
select 5588,'USAF' from dual union all
select 55880,'UNKS' from dual union all
select 55881,'UNKS' from dual union all
select 55882,'UNKS' from dual union all
select 55883,'UNKS' from dual union all
select 55884,'UNKS' from dual union all
select 55885,'MRN' from dual union all
select 55886,'MRN' from dual union all
select 55887,'MRN' from dual union all
select 55888,'MRN' from dual union all
select 55889,'MRN' from dual union all
select 5589,'RAF' from dual union all
select 55890,'MRN' from dual union all
select 55891,'MRN' from dual union all
select 55892,'MRN' from dual union all
select 55893,'MRN' from dual union all
select 55894,'MRN' from dual union all
select 55895,'MRN' from dual union all
select 55896,'MRN' from dual union all
select 55897,'MRN' from dual union all
select 55898,'MRN' from dual union all
select 55899,'MRN' from dual union all
select 559,'WEHR' from dual union all
select 5590,'PARD' from dual union all
select 55900,'MRN' from dual union all
select 55901,'MRN' from dual union all
select 55902,'MRN' from dual union all
select 55903,'USN' from dual union all
select 55904,'MRN' from dual union all
select 55905,'MRN' from dual union all
select 55906,'AN' from dual union all
select 55907,'MRN' from dual union all
select 55908,'RN' from dual union all
select 55909,'USAF' from dual union all
select 5591,'USAF' from dual union all
select 55910,'NASA' from dual union all
select 55911,'USAF' from dual union all
select 55912,'NASA' from dual union all
select 55913,'RVSN' from dual union all
select 55914,'MRN' from dual union all
select 55915,'MRN' from dual union all
select 55916,'MRN' from dual union all
select 55917,'MRN' from dual union all
select 55918,'MRN' from dual union all
select 55919,'MRN' from dual union all
select 5592,'NASA' from dual union all
select 55920,'MRN' from dual union all
select 55921,'MRN' from dual union all
select 55922,'MRN' from dual union all
select 55923,'MRN' from dual union all
select 55924,'MRN' from dual union all
select 55925,'MRN' from dual union all
select 55926,'MRN' from dual union all
select 55927,'MRN' from dual union all
select 55928,'MRN' from dual union all
select 55929,'MRN' from dual union all
select 5593,'NASA' from dual union all
select 55930,'MRN' from dual union all
select 55931,'MRN' from dual union all
select 55932,'MRN' from dual union all
select 55933,'MRN' from dual union all
select 55934,'MRN' from dual union all
select 55935,'MRN' from dual union all
select 55936,'MRN' from dual union all
select 55937,'MRN' from dual union all
select 55938,'RVSN' from dual union all
select 55939,'UNKS' from dual union all
select 5594,'USAF' from dual union all
select 55940,'UNKS' from dual union all
select 55941,'DLR' from dual union all
select 55941,'OTR' from dual union all
select 55942,'IAE' from dual union all
select 55943,'DLR' from dual union all
select 55943,'NTNF' from dual union all
select 55944,'ISRO' from dual union all
select 55945,'ISRO' from dual union all
select 55946,'ISRO' from dual union all
select 55947,'ISRO' from dual union all
select 55948,'ISRO' from dual union all
select 55949,'ISRO' from dual;
insert into LAUNCH_AGENCY
select 5595,'USAF' from dual union all
select 55950,'ISRO' from dual union all
select 55951,'ISRO' from dual union all
select 55952,'ISRO' from dual union all
select 55953,'ISRO' from dual union all
select 55954,'ISRO' from dual union all
select 55955,'ISRO' from dual union all
select 55956,'ISRO' from dual union all
select 55957,'ISRO' from dual union all
select 55958,'ISRO' from dual union all
select 55959,'ISRO' from dual union all
select 5596,'USN' from dual union all
select 55960,'ISRO' from dual union all
select 55961,'ISRO' from dual union all
select 55962,'ISRO' from dual union all
select 55963,'ISRO' from dual union all
select 55964,'ISRO' from dual union all
select 55965,'ISRO' from dual union all
select 55966,'ISRO' from dual union all
select 55967,'ISRO' from dual union all
select 55968,'ISRO' from dual union all
select 55969,'ISRO' from dual union all
select 5597,'USA' from dual union all
select 55970,'ISRO' from dual union all
select 55971,'ISRO' from dual union all
select 55972,'ISRO' from dual union all
select 55973,'ISRO' from dual union all
select 55974,'ISRO' from dual union all
select 55975,'ISRO' from dual union all
select 55976,'ISRO' from dual union all
select 55977,'ISRO' from dual union all
select 55978,'ISRO' from dual union all
select 55979,'ISRO' from dual union all
select 5598,'USA' from dual union all
select 55980,'ISRO' from dual union all
select 55981,'ISRO' from dual union all
select 55982,'ISRO' from dual union all
select 55983,'ISRO' from dual union all
select 55984,'ISRO' from dual union all
select 55985,'ISRO' from dual union all
select 55986,'ISRO' from dual union all
select 55987,'ISRO' from dual union all
select 55988,'ISRO' from dual union all
select 55989,'ISRO' from dual union all
select 5599,'USA' from dual union all
select 55990,'ISRO' from dual union all
select 55991,'ISRO' from dual union all
select 55992,'ISRO' from dual union all
select 55993,'ISRO' from dual union all
select 55994,'ISRO' from dual union all
select 55995,'ISRO' from dual union all
select 55996,'ISRO' from dual union all
select 55997,'ISRO' from dual union all
select 55998,'NASA' from dual union all
select 55999,'NASA' from dual union all
select 56,'WER271-1' from dual union all
select 560,'WEHR' from dual union all
select 5600,'USA' from dual union all
select 56000,'NASA' from dual union all
select 56001,'NASA' from dual union all
select 56002,'NASA' from dual union all
select 56003,'NASA' from dual union all
select 56004,'NASA' from dual union all
select 56005,'NASA' from dual union all
select 56006,'NASA' from dual union all
select 56007,'NASA' from dual union all
select 56008,'NASA' from dual union all
select 56009,'NASA' from dual union all
select 5601,'USA' from dual union all
select 56010,'NASA' from dual union all
select 56011,'ISRO' from dual union all
select 56012,'ISRO' from dual union all
select 56013,'ISRO' from dual union all
select 56014,'ISRO' from dual union all
select 56015,'ISRO' from dual union all
select 56016,'ISRO' from dual union all
select 56017,'ISRO' from dual union all
select 56018,'ISRO' from dual union all
select 56019,'ISRO' from dual union all
select 5602,'USAF' from dual union all
select 56020,'ISRO' from dual union all
select 56021,'ISRO' from dual union all
select 56022,'ISRO' from dual union all
select 56023,'ISRO' from dual union all
select 56024,'ISRO' from dual union all
select 56025,'ISRO' from dual union all
select 56026,'ISRO' from dual union all
select 56027,'ISRO' from dual union all
select 56028,'ISRO' from dual union all
select 56029,'ISRO' from dual union all
select 5603,'USA' from dual union all
select 56030,'ISRO' from dual union all
select 56031,'ISRO' from dual union all
select 56032,'ISRO' from dual union all
select 56033,'RVSN' from dual union all
select 56034,'DMA' from dual union all
select 56035,'RVSN' from dual union all
select 56036,'RVSN' from dual union all
select 56037,'RVSN' from dual union all
select 56038,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 56039,'RVSN' from dual union all
select 5604,'USA' from dual union all
select 56040,'RVSN' from dual union all
select 56041,'RVSN' from dual union all
select 56042,'RVSN' from dual union all
select 56043,'NASA' from dual union all
select 56044,'RN' from dual union all
select 56045,'RN' from dual union all
select 56046,'USA' from dual union all
select 56047,'AFSC' from dual union all
select 56048,'AFSC' from dual union all
select 56049,'RVSN' from dual union all
select 5605,'USA' from dual union all
select 56050,'VMF' from dual union all
select 56051,'IAE' from dual union all
select 56052,'USA' from dual union all
select 56053,'RVSN' from dual union all
select 56054,'RN' from dual union all
select 56055,'NRCC' from dual union all
select 56056,'RN' from dual union all
select 56057,'RN' from dual union all
select 56058,'AFGL' from dual union all
select 56059,'AFSC' from dual union all
select 5606,'USA' from dual union all
select 56060,'NASA' from dual union all
select 56061,'USA' from dual union all
select 56062,'NASA' from dual union all
select 56063,'USA' from dual union all
select 56064,'RVSN' from dual union all
select 56065,'ALCS' from dual union all
select 56065,'SAC' from dual union all
select 56066,'SAC' from dual union all
select 56067,'DMA' from dual union all
select 56068,'NASA' from dual union all
select 56069,'DMA' from dual union all
select 5607,'USAF' from dual union all
select 56070,'RVSN' from dual union all
select 56071,'NASA' from dual union all
select 56072,'USA' from dual union all
select 56073,'NASA' from dual union all
select 56074,'USN' from dual union all
select 56075,'RVSN' from dual union all
select 56076,'NASA' from dual union all
select 56077,'USA' from dual union all
select 56078,'USN' from dual union all
select 56079,'USN' from dual union all
select 5608,'PARD' from dual union all
select 56080,'USN' from dual union all
select 56081,'USN' from dual union all
select 56082,'AN' from dual union all
select 56083,'AN' from dual union all
select 56084,'AN' from dual union all
select 56085,'AN' from dual union all
select 56086,'RVSN' from dual union all
select 56087,'AN' from dual union all
select 56088,'AN' from dual union all
select 56089,'AN' from dual union all
select 5609,'MVS' from dual union all
select 56090,'USN' from dual union all
select 56091,'USN' from dual union all
select 56092,'NASA' from dual union all
select 56093,'ISAS' from dual union all
select 56093,'NASA' from dual union all
select 56094,'NASA' from dual union all
select 56095,'NASDA' from dual union all
select 56096,'USN' from dual union all
select 56097,'RVSN' from dual union all
select 56098,'ISAS' from dual union all
select 56099,'NASA' from dual union all
select 561,'WEHR' from dual union all
select 5610,'NASA' from dual union all
select 5610,'USAF' from dual union all
select 56100,'NASA' from dual union all
select 56101,'AN' from dual union all
select 56102,'AN' from dual union all
select 56103,'AN' from dual union all
select 56104,'AN' from dual union all
select 56105,'AN' from dual union all
select 56106,'AN' from dual union all
select 56107,'USN' from dual union all
select 56108,'RVSN' from dual union all
select 56109,'NASA' from dual union all
select 5611,'NASA' from dual union all
select 56110,'USA' from dual union all
select 56111,'ISAS' from dual union all
select 56112,'ISAS' from dual union all
select 56113,'9FA3' from dual union all
select 56114,'NASA' from dual union all
select 56115,'ALCS' from dual union all
select 56115,'SAC' from dual union all
select 56116,'SAC' from dual union all
select 56117,'NASA' from dual union all
select 56118,'RVSN' from dual union all
select 56119,'USA' from dual union all
select 5612,'USAF' from dual union all
select 56120,'USA' from dual union all
select 56121,'USA' from dual union all
select 56122,'USA' from dual union all
select 56123,'USA' from dual union all
select 56124,'SAC' from dual;
insert into LAUNCH_AGENCY
select 56125,'AN' from dual union all
select 56126,'AN' from dual union all
select 56127,'AN' from dual union all
select 56128,'AN' from dual union all
select 56129,'RVSN' from dual union all
select 5613,'USA' from dual union all
select 56130,'AN' from dual union all
select 56131,'AN' from dual union all
select 56132,'AN' from dual union all
select 56133,'USN' from dual union all
select 56134,'DMA' from dual union all
select 56135,'NASA' from dual union all
select 56136,'NASA' from dual union all
select 56137,'RVSN' from dual union all
select 56138,'USN' from dual union all
select 56139,'USN' from dual union all
select 5614,'NASA' from dual union all
select 5614,'USAF' from dual union all
select 56140,'RVSN' from dual union all
select 56141,'USA' from dual union all
select 56142,'USA' from dual union all
select 56143,'USA' from dual union all
select 56144,'USA' from dual union all
select 56145,'AFSC' from dual union all
select 56146,'AN' from dual union all
select 56147,'AFGL2' from dual union all
select 56148,'IAE' from dual union all
select 56149,'USN' from dual union all
select 5615,'NASA' from dual union all
select 56150,'USN' from dual union all
select 56151,'RVSN' from dual union all
select 56152,'AFGL' from dual union all
select 56153,'NASA' from dual union all
select 56154,'NASA' from dual union all
select 56155,'NASA' from dual union all
select 56156,'NASA' from dual union all
select 56157,'ALCS' from dual union all
select 56157,'SAC' from dual union all
select 56158,'RVSN' from dual union all
select 56159,'DMA' from dual union all
select 5616,'USAF' from dual union all
select 56160,'NASA' from dual union all
select 56161,'NASA' from dual union all
select 56162,'RVSN' from dual union all
select 56163,'NASA' from dual union all
select 56164,'AFSC' from dual union all
select 56165,'AFSC' from dual union all
select 56166,'AFBMO' from dual union all
select 56167,'AFSC' from dual union all
select 56168,'USN' from dual union all
select 56169,'RVSN' from dual union all
select 5617,'USAF' from dual union all
select 56170,'NASA' from dual union all
select 56171,'NASA' from dual union all
select 56172,'NASA' from dual union all
select 56173,'RVSN' from dual union all
select 56174,'IQ224' from dual union all
select 56175,'IQ224' from dual union all
select 56176,'IQ224' from dual union all
select 56177,'IQ224' from dual union all
select 56178,'IQ224' from dual union all
select 56179,'IQ224' from dual union all
select 5618,'USAF' from dual union all
select 56180,'IQ224' from dual union all
select 56181,'IQ224' from dual union all
select 56182,'IQ224' from dual union all
select 56183,'IQ224' from dual union all
select 56184,'RVSN' from dual union all
select 56185,'IQ224' from dual union all
select 56186,'IQ224' from dual union all
select 56187,'IQ224' from dual union all
select 56188,'IQ224' from dual union all
select 56189,'IQ224' from dual union all
select 5619,'USAF' from dual union all
select 56190,'IQ224' from dual union all
select 56191,'IQ224' from dual union all
select 56192,'IQ224' from dual union all
select 56193,'IQ224' from dual union all
select 56194,'IQ224' from dual union all
select 56195,'RVSN' from dual union all
select 56196,'IQ224' from dual union all
select 56197,'IQ224' from dual union all
select 56198,'IQ224' from dual union all
select 56199,'IQ224' from dual union all
select 562,'WEHR' from dual union all
select 5620,'USN' from dual union all
select 56200,'IQ224' from dual union all
select 56201,'IQ224' from dual union all
select 56202,'IQ224' from dual union all
select 56203,'IQ224' from dual union all
select 56204,'IQ224' from dual union all
select 56205,'IQ224' from dual union all
select 56206,'IQ224' from dual union all
select 56207,'IQ224' from dual union all
select 56208,'IQ224' from dual union all
select 56209,'IQ224' from dual union all
select 5621,'SU' from dual union all
select 56210,'IQ224' from dual union all
select 56211,'IQ224' from dual union all
select 56212,'IQ224' from dual;
insert into LAUNCH_AGENCY
select 56213,'FKG' from dual union all
select 56214,'FKG' from dual union all
select 56215,'FKG' from dual union all
select 56216,'FKG' from dual union all
select 56217,'RVSN' from dual union all
select 56218,'RVSN' from dual union all
select 56219,'RVSN' from dual union all
select 5622,'USAF' from dual union all
select 56220,'RVSN' from dual union all
select 56221,'RVSN' from dual union all
select 56222,'RVSN' from dual union all
select 56223,'RVSN' from dual union all
select 56224,'RVSN' from dual union all
select 56225,'RVSN' from dual union all
select 56226,'RVSN' from dual union all
select 56227,'RVSN' from dual union all
select 56228,'INTA' from dual union all
select 56229,'INTA' from dual union all
select 5623,'USN' from dual union all
select 56230,'INTA' from dual union all
select 56231,'INTA' from dual union all
select 56232,'INTA' from dual union all
select 56233,'INTA' from dual union all
select 56234,'INTA' from dual union all
select 56235,'INTA' from dual union all
select 56236,'INTA' from dual union all
select 56237,'INTA' from dual union all
select 56238,'INTA' from dual union all
select 56239,'INTA' from dual union all
select 5624,'USA' from dual union all
select 56240,'INTA' from dual union all
select 56241,'INTA' from dual union all
select 56242,'INTA' from dual union all
select 56243,'INTA' from dual union all
select 56244,'RVSN' from dual union all
select 56245,'INTA' from dual union all
select 56246,'RVSN' from dual union all
select 56247,'RVSN' from dual union all
select 56248,'RVSN' from dual union all
select 56249,'RVSN' from dual union all
select 5625,'US' from dual union all
select 56250,'RVSN' from dual union all
select 56251,'RVSN' from dual union all
select 56252,'RVSN' from dual union all
select 56253,'RVSN' from dual union all
select 56254,'RVSN' from dual union all
select 56255,'RVSN' from dual union all
select 56256,'RVSN' from dual union all
select 56257,'RVSN' from dual union all
select 56258,'RVSN' from dual union all
select 56259,'RVSN' from dual union all
select 5626,'MVS' from dual union all
select 56260,'RVSN' from dual union all
select 56261,'RVSN' from dual union all
select 56262,'RVSN' from dual union all
select 56263,'RVSN' from dual union all
select 56264,'RVSN' from dual union all
select 56265,'ISRO' from dual union all
select 56266,'RVSN' from dual union all
select 56267,'RVSN' from dual union all
select 56268,'RVSN' from dual union all
select 56269,'RVSN' from dual union all
select 5627,'NASA' from dual union all
select 5627,'USAF' from dual union all
select 56270,'RVSN' from dual union all
select 56271,'RVSN' from dual union all
select 56272,'RVSN' from dual union all
select 56273,'RVSN' from dual union all
select 56274,'RVSN' from dual union all
select 56275,'RVSN' from dual union all
select 56276,'AFSC' from dual union all
select 56277,'USN' from dual union all
select 56278,'USN' from dual union all
select 56279,'NASA' from dual union all
select 5628,'USN' from dual union all
select 56280,'NASA' from dual union all
select 56281,'NASA' from dual union all
select 56282,'ISAS' from dual union all
select 56283,'SSC' from dual union all
select 56284,'USA' from dual union all
select 56285,'NASA' from dual union all
select 56286,'NASDA' from dual union all
select 56287,'AFSC' from dual union all
select 56288,'AFSC' from dual union all
select 56289,'USN' from dual union all
select 5629,'NASA' from dual union all
select 5629,'USAF' from dual union all
select 56290,'NRCC' from dual union all
select 56291,'USN' from dual union all
select 56292,'RVSN' from dual union all
select 56293,'USA' from dual union all
select 56294,'NASA' from dual union all
select 56295,'RVSN' from dual union all
select 56296,'SAC' from dual union all
select 56297,'NASA' from dual union all
select 56298,'USA' from dual union all
select 56299,'USAF' from dual union all
select 563,'WEHR' from dual union all
select 5630,'NASA' from dual union all
select 5630,'USAF' from dual;
insert into LAUNCH_AGENCY
select 56300,'NASA' from dual union all
select 56301,'NASA' from dual union all
select 56302,'NASA' from dual union all
select 56303,'NASA' from dual union all
select 56304,'NASA' from dual union all
select 56305,'NASA' from dual union all
select 56306,'NASA' from dual union all
select 56307,'ALCS' from dual union all
select 56307,'SAC' from dual union all
select 56308,'NASA' from dual union all
select 56309,'NASA' from dual union all
select 5631,'PARD' from dual union all
select 56310,'USA' from dual union all
select 56311,'NASA' from dual union all
select 56312,'SAC' from dual union all
select 56313,'AFGL2' from dual union all
select 56314,'USN' from dual union all
select 56315,'USN' from dual union all
select 56316,'NASA' from dual union all
select 56317,'NASA' from dual union all
select 56318,'USA' from dual union all
select 56319,'RVSN' from dual union all
select 5632,'USAF' from dual union all
select 56320,'NASA' from dual union all
select 56321,'USN' from dual union all
select 56322,'NASA' from dual union all
select 56323,'USA' from dual union all
select 56324,'AFGL2' from dual union all
select 56325,'AFGL' from dual union all
select 56326,'USA' from dual union all
select 56327,'DFVLR' from dual union all
select 56328,'RVSN' from dual union all
select 56329,'RVSN' from dual union all
select 5633,'RAE' from dual union all
select 56330,'AFSC' from dual union all
select 56331,'DFVLR' from dual union all
select 56332,'USA' from dual union all
select 56333,'NRCC' from dual union all
select 56334,'DFVLR' from dual union all
select 56335,'NASA' from dual union all
select 56336,'MRN' from dual union all
select 56337,'AN' from dual union all
select 56338,'MRN' from dual union all
select 56339,'MRN' from dual union all
select 5634,'NASA' from dual union all
select 5634,'USAF' from dual union all
select 56340,'MRN' from dual union all
select 56341,'JMA' from dual union all
select 56342,'MRN' from dual union all
select 56343,'MRN' from dual union all
select 56344,'NASA' from dual union all
select 56345,'MRN' from dual union all
select 56346,'MRN' from dual union all
select 56347,'NASA' from dual union all
select 56348,'MRN' from dual union all
select 56349,'MRN' from dual union all
select 5635,'USA' from dual union all
select 56350,'MRN' from dual union all
select 56351,'JMA' from dual union all
select 56352,'NASA' from dual union all
select 56353,'NASA' from dual union all
select 56354,'MRN' from dual union all
select 56355,'MRN' from dual union all
select 56356,'MRN' from dual union all
select 56357,'MRN' from dual union all
select 56358,'MRN' from dual union all
select 56359,'MRN' from dual union all
select 5636,'USAF' from dual union all
select 56360,'MRN' from dual union all
select 56361,'MRN' from dual union all
select 56362,'MRN' from dual union all
select 56363,'MRN' from dual union all
select 56364,'MRN' from dual union all
select 56365,'MRN' from dual union all
select 56366,'MRN' from dual union all
select 56367,'JMA' from dual union all
select 56368,'MRN' from dual union all
select 56369,'MRN' from dual union all
select 5637,'USAF' from dual union all
select 56370,'NASA' from dual union all
select 56371,'MRN' from dual union all
select 56372,'JMA' from dual union all
select 56373,'MRN' from dual union all
select 56374,'MRN' from dual union all
select 56375,'MRN' from dual union all
select 56376,'MRN' from dual union all
select 56377,'MRN' from dual union all
select 56378,'MRN' from dual union all
select 56379,'JMA' from dual union all
select 5638,'USAF' from dual union all
select 56380,'MRN' from dual union all
select 56381,'NASA' from dual union all
select 56382,'GMS' from dual union all
select 56383,'MRN' from dual union all
select 56384,'GMS' from dual union all
select 56385,'MRN' from dual union all
select 56386,'MRN' from dual union all
select 56387,'MRN' from dual union all
select 56388,'JMA' from dual union all
select 56389,'MRN' from dual;
insert into LAUNCH_AGENCY
select 5639,'MVS' from dual union all
select 56390,'GMS' from dual union all
select 56391,'MRN' from dual union all
select 56392,'MRN' from dual union all
select 56393,'MRN' from dual union all
select 56394,'NASA' from dual union all
select 56395,'JMA' from dual union all
select 56396,'GMS' from dual union all
select 56397,'AN' from dual union all
select 56398,'MRN' from dual union all
select 56399,'MRN' from dual union all
select 564,'WEHR' from dual union all
select 5640,'USAF' from dual union all
select 56400,'GMS' from dual union all
select 56401,'MRN' from dual union all
select 56402,'MRN' from dual union all
select 56403,'GMS' from dual union all
select 56404,'MRN' from dual union all
select 56405,'MRN' from dual union all
select 56406,'GMS' from dual union all
select 56407,'MRN' from dual union all
select 56408,'NASA' from dual union all
select 56409,'JMA' from dual union all
select 5641,'USAF' from dual union all
select 56410,'MRN' from dual union all
select 56411,'GMS' from dual union all
select 56412,'MRN' from dual union all
select 56413,'GMS' from dual union all
select 56414,'MRN' from dual union all
select 56415,'NASA' from dual union all
select 56416,'GMS' from dual union all
select 56417,'AN' from dual union all
select 56418,'GMS' from dual union all
select 56419,'MRN' from dual union all
select 5642,'NASA' from dual union all
select 5642,'USAF' from dual union all
select 56420,'MRN' from dual union all
select 56421,'NASA' from dual union all
select 56422,'MRN' from dual union all
select 56423,'JMA' from dual union all
select 56424,'MRN' from dual union all
select 56425,'GMS' from dual union all
select 56426,'JMA' from dual union all
select 56427,'MRN' from dual union all
select 56428,'MRN' from dual union all
select 56429,'NASA' from dual union all
select 5643,'NASA' from dual union all
select 56430,'MRN' from dual union all
select 56431,'GMS' from dual union all
select 56432,'MRN' from dual union all
select 56433,'MRN' from dual union all
select 56434,'GMS' from dual union all
select 56435,'GMS' from dual union all
select 56436,'MRN' from dual union all
select 56437,'NASA' from dual union all
select 56438,'MRN' from dual union all
select 56439,'NASA' from dual union all
select 5644,'NASA' from dual union all
select 5644,'USAF' from dual union all
select 56440,'JMA' from dual union all
select 56441,'MRN' from dual union all
select 56442,'MRN' from dual union all
select 56443,'MRN' from dual union all
select 56444,'AN' from dual union all
select 56445,'MRN' from dual union all
select 56446,'NASA' from dual union all
select 56447,'MRN' from dual union all
select 56448,'MRN' from dual union all
select 56449,'MRN' from dual union all
select 5645,'USAF' from dual union all
select 56450,'MRN' from dual union all
select 56451,'MRN' from dual union all
select 56452,'MRN' from dual union all
select 56453,'MRN' from dual union all
select 56454,'MRN' from dual union all
select 56455,'MRN' from dual union all
select 56456,'NASA' from dual union all
select 56457,'MRN' from dual union all
select 56458,'NASA' from dual union all
select 56459,'JMA' from dual union all
select 5646,'USA' from dual union all
select 56460,'MRN' from dual union all
select 56461,'MRN' from dual union all
select 56462,'NASA' from dual union all
select 56463,'MRN' from dual union all
select 56464,'MRN' from dual union all
select 56465,'JMA' from dual union all
select 56466,'NASA' from dual union all
select 56467,'NASA' from dual union all
select 56468,'MRN' from dual union all
select 56469,'JMA' from dual union all
select 5647,'USAF' from dual union all
select 56470,'MRN' from dual union all
select 56471,'NASA' from dual union all
select 56472,'MRN' from dual union all
select 56473,'MRN' from dual union all
select 56474,'MRN' from dual union all
select 56475,'NASA' from dual union all
select 56476,'AN' from dual union all
select 56477,'MRN' from dual;
insert into LAUNCH_AGENCY
select 56478,'MRN' from dual union all
select 56479,'JMA' from dual union all
select 5648,'BRL' from dual union all
select 5648,'UMI' from dual union all
select 56480,'MRN' from dual union all
select 56481,'NASA' from dual union all
select 56482,'NASA' from dual union all
select 56483,'MRN' from dual union all
select 56484,'GMS' from dual union all
select 56485,'MRN' from dual union all
select 56486,'MRN' from dual union all
select 56487,'MRN' from dual union all
select 56488,'GMS' from dual union all
select 56489,'AN' from dual union all
select 5649,'USAF' from dual union all
select 56490,'JMA' from dual union all
select 56491,'GMS' from dual union all
select 56492,'NASA' from dual union all
select 56493,'MRN' from dual union all
select 56494,'GMS' from dual union all
select 56495,'MRN' from dual union all
select 56496,'MRN' from dual union all
select 56497,'GMS' from dual union all
select 56498,'MRN' from dual union all
select 56499,'MRN' from dual union all
select 565,'WEHR' from dual union all
select 5650,'USAF' from dual union all
select 56500,'GMS' from dual union all
select 56501,'MRN' from dual union all
select 56502,'JMA' from dual union all
select 56503,'GMS' from dual union all
select 56504,'GMS' from dual union all
select 56505,'MRN' from dual union all
select 56506,'MRN' from dual union all
select 56507,'MRN' from dual union all
select 56508,'NASA' from dual union all
select 56509,'MRN' from dual union all
select 5651,'USAF' from dual union all
select 56510,'MRN' from dual union all
select 56511,'MRN' from dual union all
select 56512,'MRN' from dual union all
select 56513,'AN' from dual union all
select 56514,'MRN' from dual union all
select 56515,'NASA' from dual union all
select 56516,'MRN' from dual union all
select 56517,'MRN' from dual union all
select 56518,'MRN' from dual union all
select 56519,'MRN' from dual union all
select 5652,'USAF' from dual union all
select 56520,'JMA' from dual union all
select 56521,'MRN' from dual union all
select 56522,'JMA' from dual union all
select 56523,'MRN' from dual union all
select 56524,'MRN' from dual union all
select 56525,'MRN' from dual union all
select 56526,'MRN' from dual union all
select 56527,'JMA' from dual union all
select 56528,'MRN' from dual union all
select 56529,'MRN' from dual union all
select 5653,'NASA' from dual union all
select 56530,'MRN' from dual union all
select 56531,'NASA' from dual union all
select 56532,'NASA' from dual union all
select 56533,'MRN' from dual union all
select 56534,'MRN' from dual union all
select 56535,'MRN' from dual union all
select 56536,'NASA' from dual union all
select 56537,'NASA' from dual union all
select 56538,'MRN' from dual union all
select 56539,'NASA' from dual union all
select 5654,'NASA' from dual union all
select 56540,'MRN' from dual union all
select 56541,'MRN' from dual union all
select 56542,'GMS' from dual union all
select 56543,'MRN' from dual union all
select 56544,'MRN' from dual union all
select 56545,'AN' from dual union all
select 56546,'AN' from dual union all
select 56547,'MRN' from dual union all
select 56548,'MRN' from dual union all
select 56549,'NASA' from dual union all
select 5655,'USA' from dual union all
select 56550,'MRN' from dual union all
select 56551,'MRN' from dual union all
select 56552,'MRN' from dual union all
select 56553,'MRN' from dual union all
select 56554,'JMA' from dual union all
select 56555,'MRN' from dual union all
select 56556,'NASA' from dual union all
select 56557,'MRN' from dual union all
select 56558,'JMA' from dual union all
select 56559,'MRN' from dual union all
select 5656,'USA' from dual union all
select 56560,'MRN' from dual union all
select 56561,'MRN' from dual union all
select 56562,'MRN' from dual union all
select 56563,'JMA' from dual union all
select 56564,'MRN' from dual union all
select 56565,'MRN' from dual union all
select 56566,'MRN' from dual;
insert into LAUNCH_AGENCY
select 56567,'NASA' from dual union all
select 56568,'MRN' from dual union all
select 56569,'NASA' from dual union all
select 5657,'USA' from dual union all
select 56570,'MRN' from dual union all
select 56571,'JMA' from dual union all
select 56572,'MRN' from dual union all
select 56573,'MRN' from dual union all
select 56574,'MRN' from dual union all
select 56575,'MRN' from dual union all
select 56576,'MRN' from dual union all
select 56577,'MRN' from dual union all
select 56578,'MRN' from dual union all
select 56579,'MRN' from dual union all
select 5658,'RAF' from dual union all
select 56580,'MRN' from dual union all
select 56581,'NASA' from dual union all
select 56582,'NASA' from dual union all
select 56583,'JMA' from dual union all
select 56584,'MRN' from dual union all
select 56585,'MRN' from dual union all
select 56586,'GMS' from dual union all
select 56587,'MRN' from dual union all
select 56588,'MRN' from dual union all
select 56589,'MRN' from dual union all
select 5659,'USAF' from dual union all
select 56590,'MRN' from dual union all
select 56591,'NASA' from dual union all
select 56592,'NASA' from dual union all
select 56593,'MRN' from dual union all
select 56594,'MRN' from dual union all
select 56595,'MRN' from dual union all
select 56596,'MRN' from dual union all
select 56597,'GMS' from dual union all
select 56598,'JMA' from dual union all
select 56599,'MRN' from dual union all
select 566,'WEHR' from dual union all
select 5660,'USAF' from dual union all
select 56600,'MRN' from dual union all
select 56601,'DFVLR' from dual union all
select 56602,'MRN' from dual union all
select 56603,'MRN' from dual union all
select 56604,'DFVLR' from dual union all
select 56605,'DFVLR' from dual union all
select 56606,'MRN' from dual union all
select 56607,'DFVLR' from dual union all
select 56608,'MRN' from dual union all
select 56609,'DFVLR' from dual union all
select 5661,'USAF' from dual union all
select 56610,'MRN' from dual union all
select 56611,'MRN' from dual union all
select 56612,'JMA' from dual union all
select 56613,'WFC' from dual union all
select 56614,'MRN' from dual union all
select 56615,'DFVLR' from dual union all
select 56616,'DFVLR' from dual union all
select 56617,'DFVLR' from dual union all
select 56618,'MRN' from dual union all
select 56619,'NASA' from dual union all
select 5662,'USAF' from dual union all
select 56620,'MRN' from dual union all
select 56621,'MRN' from dual union all
select 56622,'WFC' from dual union all
select 56623,'MRN' from dual union all
select 56624,'MRN' from dual union all
select 56625,'MRN' from dual union all
select 56626,'MRN' from dual union all
select 56627,'MRN' from dual union all
select 56628,'MRN' from dual union all
select 56629,'MRN' from dual union all
select 5663,'RAF' from dual union all
select 56630,'JMA' from dual union all
select 56631,'MRN' from dual union all
select 56632,'WFC' from dual union all
select 56633,'DFVLR' from dual union all
select 56634,'MRN' from dual union all
select 56635,'JMA' from dual union all
select 56636,'MRN' from dual union all
select 56637,'MRN' from dual union all
select 56638,'DFVLR' from dual union all
select 56639,'MRN' from dual union all
select 5664,'USAF' from dual union all
select 56640,'MRN' from dual union all
select 56641,'GMS' from dual union all
select 56642,'MRN' from dual union all
select 56643,'MRN' from dual union all
select 56644,'JMA' from dual union all
select 56645,'WFC' from dual union all
select 56646,'DFVLR' from dual union all
select 56647,'DFVLR' from dual union all
select 56648,'MRN' from dual union all
select 56649,'MRN' from dual union all
select 5665,'USA' from dual union all
select 56650,'MRN' from dual union all
select 56651,'JMA' from dual union all
select 56652,'MRN' from dual union all
select 56653,'DFVLR' from dual union all
select 56654,'DFVLR' from dual union all
select 56655,'DFVLR' from dual union all
select 56656,'DFVLR' from dual;
insert into LAUNCH_AGENCY
select 56657,'JMA' from dual union all
select 56658,'MRN' from dual union all
select 56659,'GMS' from dual union all
select 5666,'NASA' from dual union all
select 5666,'USAF' from dual union all
select 56660,'MRN' from dual union all
select 56661,'GMS' from dual union all
select 56662,'AN' from dual union all
select 56663,'MRN' from dual union all
select 56664,'JMA' from dual union all
select 56665,'GMS' from dual union all
select 56666,'MRN' from dual union all
select 56667,'NASA' from dual union all
select 56668,'MRN' from dual union all
select 56669,'MRN' from dual union all
select 5667,'USAF' from dual union all
select 56670,'MRN' from dual union all
select 56671,'GMS' from dual union all
select 56672,'MRN' from dual union all
select 56673,'MRN' from dual union all
select 56674,'GMS' from dual union all
select 56675,'AN' from dual union all
select 56676,'MRN' from dual union all
select 56677,'MRN' from dual union all
select 56678,'MRN' from dual union all
select 56679,'NASA' from dual union all
select 5668,'SAND' from dual union all
select 56680,'GMS' from dual union all
select 56681,'GMS' from dual union all
select 56682,'MRN' from dual union all
select 56683,'NASA' from dual union all
select 56684,'JMA' from dual union all
select 56685,'NASA' from dual union all
select 56686,'GMS' from dual union all
select 56687,'MRN' from dual union all
select 56688,'MRN' from dual union all
select 56689,'GMS' from dual union all
select 5669,'USN' from dual union all
select 56690,'NASA' from dual union all
select 56691,'MRN' from dual union all
select 56692,'AN' from dual union all
select 56693,'MRN' from dual union all
select 56694,'GMS' from dual union all
select 56695,'NASA' from dual union all
select 56696,'MRN' from dual union all
select 56697,'MRN' from dual union all
select 56698,'MRN' from dual union all
select 56699,'JMA' from dual union all
select 567,'WEHR' from dual union all
select 5670,'USN' from dual union all
select 56700,'GMS' from dual union all
select 56701,'AN' from dual union all
select 56702,'MRN' from dual union all
select 56703,'GMS' from dual union all
select 56704,'MRN' from dual union all
select 56705,'MRN' from dual union all
select 56706,'MRN' from dual union all
select 56707,'MRN' from dual union all
select 56708,'NASA' from dual union all
select 56709,'JMA' from dual union all
select 5671,'USN' from dual union all
select 56710,'GMS' from dual union all
select 56711,'MRN' from dual union all
select 56712,'NASA' from dual union all
select 56713,'MRN' from dual union all
select 56714,'MRN' from dual union all
select 56715,'GMS' from dual union all
select 56716,'NASA' from dual union all
select 56717,'MRN' from dual union all
select 56718,'MRN' from dual union all
select 56719,'AN' from dual union all
select 5672,'WADD' from dual union all
select 56720,'AN' from dual union all
select 56721,'AN' from dual union all
select 56722,'AN' from dual union all
select 56723,'AN' from dual union all
select 56724,'MRN' from dual union all
select 56725,'AN' from dual union all
select 56726,'AN' from dual union all
select 56727,'AN' from dual union all
select 56728,'AN' from dual union all
select 56729,'MRN' from dual union all
select 5673,'WADD' from dual union all
select 56730,'AN' from dual union all
select 56731,'AN' from dual union all
select 56732,'AN' from dual union all
select 56733,'AN' from dual union all
select 56734,'AN' from dual union all
select 56735,'AN' from dual union all
select 56736,'AN' from dual union all
select 56737,'AN' from dual union all
select 56738,'AN' from dual union all
select 56739,'AN' from dual union all
select 5674,'WADD' from dual union all
select 56740,'AN' from dual union all
select 56741,'AN' from dual union all
select 56742,'AN' from dual union all
select 56743,'AN' from dual union all
select 56744,'AN' from dual union all
select 56745,'AN' from dual;
insert into LAUNCH_AGENCY
select 56746,'AN' from dual union all
select 56747,'AN' from dual union all
select 56748,'AN' from dual union all
select 56749,'AN' from dual union all
select 5675,'WADD' from dual union all
select 56750,'AN' from dual union all
select 56751,'AN' from dual union all
select 56752,'AN' from dual union all
select 56753,'AN' from dual union all
select 56754,'MRN' from dual union all
select 56755,'AN' from dual union all
select 56756,'AN' from dual union all
select 56757,'AN' from dual union all
select 56758,'AN' from dual union all
select 56759,'AN' from dual union all
select 5676,'WADD' from dual union all
select 56760,'AN' from dual union all
select 56761,'AN' from dual union all
select 56762,'AN' from dual union all
select 56763,'MRN' from dual union all
select 56764,'MRN' from dual union all
select 56765,'MRN' from dual union all
select 56766,'AN' from dual union all
select 56767,'AN' from dual union all
select 56768,'AN' from dual union all
select 56769,'AN' from dual union all
select 5677,'ARGMA' from dual union all
select 56770,'MRN' from dual union all
select 56771,'NASA' from dual union all
select 56772,'AN' from dual union all
select 56773,'NASA' from dual union all
select 56774,'MRN' from dual union all
select 56775,'MRN' from dual union all
select 56776,'MRN' from dual union all
select 56777,'MRN' from dual union all
select 56778,'MRN' from dual union all
select 56779,'MRN' from dual union all
select 5678,'ARGMA' from dual union all
select 56780,'AN' from dual union all
select 56781,'AN' from dual union all
select 56782,'MRN' from dual union all
select 56783,'AN' from dual union all
select 56784,'AN' from dual union all
select 56785,'AN' from dual union all
select 56786,'AN' from dual union all
select 56787,'MRN' from dual union all
select 56788,'AN' from dual union all
select 56789,'AN' from dual union all
select 5679,'USA' from dual union all
select 56790,'MRN' from dual union all
select 56791,'AN' from dual union all
select 56792,'MRN' from dual union all
select 56793,'MRN' from dual union all
select 56794,'AN' from dual union all
select 56795,'AN' from dual union all
select 56796,'AN' from dual union all
select 56797,'AN' from dual union all
select 56798,'AN' from dual union all
select 56799,'MRN' from dual union all
select 568,'WEHR' from dual union all
select 5680,'ARGMA' from dual union all
select 56800,'AN' from dual union all
select 56801,'AN' from dual union all
select 56802,'AN' from dual union all
select 56803,'AN' from dual union all
select 56804,'MRN' from dual union all
select 56805,'AN' from dual union all
select 56806,'MRN' from dual union all
select 56807,'AN' from dual union all
select 56808,'AN' from dual union all
select 56809,'AN' from dual union all
select 5681,'ARGMA' from dual union all
select 56810,'AN' from dual union all
select 56811,'AN' from dual union all
select 56812,'MRN' from dual union all
select 56813,'AN' from dual union all
select 56814,'AN' from dual union all
select 56815,'AN' from dual union all
select 56816,'AN' from dual union all
select 56817,'AN' from dual union all
select 56818,'MRN' from dual union all
select 56819,'AN' from dual union all
select 5682,'ARGMA' from dual union all
select 56820,'AN' from dual union all
select 56821,'AN' from dual union all
select 56822,'AN' from dual union all
select 56823,'AN' from dual union all
select 56824,'AN' from dual union all
select 56825,'AN' from dual union all
select 56826,'MRN' from dual union all
select 56827,'AN' from dual union all
select 56828,'AN' from dual union all
select 56829,'AFGL2' from dual union all
select 5683,'ARGMA' from dual union all
select 56830,'AN' from dual union all
select 56831,'AN' from dual union all
select 56832,'AN' from dual union all
select 56833,'AFGL2' from dual union all
select 56834,'MRN' from dual union all
select 56835,'MRN' from dual;
insert into LAUNCH_AGENCY
select 56836,'AFGL2' from dual union all
select 56837,'AN' from dual union all
select 56838,'AN' from dual union all
select 56839,'AN' from dual union all
select 5684,'ARGMA' from dual union all
select 56840,'MRN' from dual union all
select 56841,'AN' from dual union all
select 56842,'AN' from dual union all
select 56843,'AN' from dual union all
select 56844,'AN' from dual union all
select 56845,'AN' from dual union all
select 56846,'AN' from dual union all
select 56847,'MRN' from dual union all
select 56848,'NASA' from dual union all
select 56849,'AN' from dual union all
select 5685,'ARGMA' from dual union all
select 56850,'NASA' from dual union all
select 56851,'MRN' from dual union all
select 56852,'MRN' from dual union all
select 56853,'MRN' from dual union all
select 56854,'MRN' from dual union all
select 56855,'AN' from dual union all
select 56856,'MRN' from dual union all
select 56857,'MRN' from dual union all
select 56858,'AN' from dual union all
select 56859,'AN' from dual union all
select 5686,'ARGMA' from dual union all
select 56860,'AN' from dual union all
select 56861,'AN' from dual union all
select 56862,'AN' from dual union all
select 56863,'AN' from dual union all
select 56864,'MRN' from dual union all
select 56865,'AN' from dual union all
select 56866,'AN' from dual union all
select 56867,'AN' from dual union all
select 56868,'MRN' from dual union all
select 56869,'AN' from dual union all
select 5687,'ARGMA' from dual union all
select 56870,'MRN' from dual union all
select 56871,'AN' from dual union all
select 56872,'AN' from dual union all
select 56873,'AN' from dual union all
select 56874,'AN' from dual union all
select 56875,'AN' from dual union all
select 56876,'AN' from dual union all
select 56877,'AN' from dual union all
select 56878,'AN' from dual union all
select 56879,'MRN' from dual union all
select 5688,'ARGMA' from dual union all
select 56880,'AN' from dual union all
select 56881,'AN' from dual union all
select 56882,'AN' from dual union all
select 56883,'AN' from dual union all
select 56884,'AN' from dual union all
select 56885,'AN' from dual union all
select 56886,'AN' from dual union all
select 56887,'MRN' from dual union all
select 56888,'AN' from dual union all
select 56889,'MRN' from dual union all
select 5689,'ARGMA' from dual union all
select 56890,'AN' from dual union all
select 56891,'AN' from dual union all
select 56892,'AN' from dual union all
select 56893,'AN' from dual union all
select 56894,'AN' from dual union all
select 56895,'AN' from dual union all
select 56896,'AN' from dual union all
select 56897,'AN' from dual union all
select 56898,'AN' from dual union all
select 56899,'AN' from dual union all
select 569,'WEHR' from dual union all
select 5690,'PARD' from dual union all
select 56900,'AN' from dual union all
select 56901,'MRN' from dual union all
select 56902,'AN' from dual union all
select 56903,'AN' from dual union all
select 56904,'AN' from dual union all
select 56905,'MRN' from dual union all
select 56906,'AN' from dual union all
select 56907,'AN' from dual union all
select 56908,'AN' from dual union all
select 56909,'AN' from dual union all
select 5691,'ARGMA' from dual union all
select 56910,'MRN' from dual union all
select 56911,'AN' from dual union all
select 56912,'AN' from dual union all
select 56913,'AN' from dual union all
select 56914,'MRN' from dual union all
select 56915,'AN' from dual union all
select 56916,'MRN' from dual union all
select 56917,'MRN' from dual union all
select 56918,'MRN' from dual union all
select 56919,'AN' from dual union all
select 5692,'ARGMA' from dual union all
select 56920,'MRN' from dual union all
select 56921,'NASA' from dual union all
select 56922,'MRN' from dual union all
select 56923,'MRN' from dual union all
select 56924,'AN' from dual union all
select 56925,'AN' from dual;
insert into LAUNCH_AGENCY
select 56926,'AN' from dual union all
select 56927,'MRN' from dual union all
select 56928,'MRN' from dual union all
select 56929,'AN' from dual union all
select 5693,'ARGMA' from dual union all
select 56930,'AN' from dual union all
select 56931,'AN' from dual union all
select 56932,'AN' from dual union all
select 56933,'MRN' from dual union all
select 56934,'AN' from dual union all
select 56935,'AN' from dual union all
select 56936,'MRN' from dual union all
select 56937,'NASA' from dual union all
select 56938,'MRN' from dual union all
select 56939,'MRN' from dual union all
select 5694,'ARGMA' from dual union all
select 56940,'NASA' from dual union all
select 56941,'NASA' from dual union all
select 56942,'MRN' from dual union all
select 56943,'MRN' from dual union all
select 56944,'MRN' from dual union all
select 56945,'DFVLR' from dual union all
select 56946,'MRN' from dual union all
select 56947,'DFVLR' from dual union all
select 56948,'NASA' from dual union all
select 56949,'DFVLR' from dual union all
select 5695,'ARGMA' from dual union all
select 56950,'DFVLR' from dual union all
select 56951,'MRN' from dual union all
select 56952,'WFC' from dual union all
select 56953,'DFVLR' from dual union all
select 56954,'MRN' from dual union all
select 56955,'MRN' from dual union all
select 56956,'DFVLR' from dual union all
select 56957,'DFVLR' from dual union all
select 56958,'MRN' from dual union all
select 56959,'MRN' from dual union all
select 5696,'ARGMA' from dual union all
select 56960,'MRN' from dual union all
select 56961,'DFVLR' from dual union all
select 56962,'DFVLR' from dual union all
select 56963,'DFVLR' from dual union all
select 56964,'DFVLR' from dual union all
select 56965,'DFVLR' from dual union all
select 56966,'MRN' from dual union all
select 56967,'DFVLR' from dual union all
select 56968,'DFVLR' from dual union all
select 56969,'DFVLR' from dual union all
select 5697,'ARGMA' from dual union all
select 56970,'NASA' from dual union all
select 56971,'DFVLR' from dual union all
select 56972,'DFVLR' from dual union all
select 56973,'DFVLR' from dual union all
select 56974,'DFVLR' from dual union all
select 56975,'AN' from dual union all
select 56976,'AN' from dual union all
select 56977,'AN' from dual union all
select 56978,'AN' from dual union all
select 56979,'AN' from dual union all
select 5698,'ARGMA' from dual union all
select 56980,'AN' from dual union all
select 56981,'AN' from dual union all
select 56982,'AN' from dual union all
select 56983,'AN' from dual union all
select 56984,'AN' from dual union all
select 56985,'AN' from dual union all
select 56986,'AN' from dual union all
select 56987,'AN' from dual union all
select 56988,'AN' from dual union all
select 56989,'AN' from dual union all
select 5699,'ARGMA' from dual union all
select 56990,'AN' from dual union all
select 56991,'AN' from dual union all
select 56992,'AN' from dual union all
select 56993,'MRN' from dual union all
select 56994,'NASA' from dual union all
select 56995,'AN' from dual union all
select 56996,'AN' from dual union all
select 56997,'AN' from dual union all
select 56998,'MRN' from dual union all
select 56999,'AN' from dual union all
select 57,'WER271-1' from dual union all
select 570,'WEHR' from dual union all
select 5700,'USA' from dual union all
select 57000,'AN' from dual union all
select 57001,'AN' from dual union all
select 57002,'NASA' from dual union all
select 57003,'AN' from dual union all
select 57004,'AN' from dual union all
select 57005,'AN' from dual union all
select 57006,'AN' from dual union all
select 57007,'AN' from dual union all
select 57008,'AN' from dual union all
select 57009,'AN' from dual union all
select 5701,'USA' from dual union all
select 57010,'AN' from dual union all
select 57011,'AN' from dual union all
select 57012,'AN' from dual union all
select 57013,'AN' from dual union all
select 57014,'AN' from dual;
insert into LAUNCH_AGENCY
select 57015,'AN' from dual union all
select 57016,'AN' from dual union all
select 57017,'AN' from dual union all
select 57018,'AN' from dual union all
select 57019,'AN' from dual union all
select 5702,'USA' from dual union all
select 57020,'AN' from dual union all
select 57021,'AN' from dual union all
select 57022,'AN' from dual union all
select 57023,'AN' from dual union all
select 57024,'NASA' from dual union all
select 57025,'NASA' from dual union all
select 57026,'AN' from dual union all
select 57027,'AN' from dual union all
select 57028,'NASA' from dual union all
select 57029,'NASA' from dual union all
select 5703,'USA' from dual union all
select 57030,'NASA' from dual union all
select 57031,'AN' from dual union all
select 57032,'NASA' from dual union all
select 57033,'NASA' from dual union all
select 57034,'NASA' from dual union all
select 57035,'NASA' from dual union all
select 57036,'NASA' from dual union all
select 57037,'NASA' from dual union all
select 57038,'NASA' from dual union all
select 57039,'NASA' from dual union all
select 5704,'USA' from dual union all
select 57040,'NASA' from dual union all
select 57041,'AN' from dual union all
select 57042,'AN' from dual union all
select 57043,'AN' from dual union all
select 57044,'MRN' from dual union all
select 57045,'AN' from dual union all
select 57046,'AN' from dual union all
select 57047,'MRN' from dual union all
select 57048,'AN' from dual union all
select 57049,'AN' from dual union all
select 5705,'USA' from dual union all
select 57050,'AN' from dual union all
select 57051,'AN' from dual union all
select 57052,'AN' from dual union all
select 57053,'AN' from dual union all
select 57054,'AN' from dual union all
select 57055,'UNKS' from dual union all
select 57056,'UNKS' from dual union all
select 57057,'UNKS' from dual union all
select 57058,'UNKS' from dual union all
select 57059,'NASDA' from dual union all
select 5706,'USA' from dual union all
select 57060,'UNKS' from dual union all
select 57061,'UNKS' from dual union all
select 57062,'CALT' from dual union all
select 57063,'AFSC' from dual union all
select 57064,'UNKS' from dual union all
select 57065,'NASA' from dual union all
select 57066,'AFSC' from dual union all
select 57067,'UNKS' from dual union all
select 57068,'UNKS' from dual union all
select 57069,'ISAS' from dual union all
select 5707,'USA' from dual union all
select 57070,'UNKS' from dual union all
select 57071,'UNKS' from dual union all
select 57072,'UNKS' from dual union all
select 57073,'UNKS' from dual union all
select 57074,'UNKS' from dual union all
select 57075,'NASA' from dual union all
select 57076,'UNKS' from dual union all
select 57077,'ESA' from dual union all
select 57078,'UNKS' from dual union all
select 57079,'UNKS' from dual union all
select 5708,'USA' from dual union all
select 57080,'UNKS' from dual union all
select 57081,'UNKS' from dual union all
select 57082,'UNKS' from dual union all
select 57083,'UNKS' from dual union all
select 57084,'UNKS' from dual union all
select 57085,'UNKS' from dual union all
select 57086,'UNKS' from dual union all
select 57087,'UNKS' from dual union all
select 57088,'NASA' from dual union all
select 57089,'CALT' from dual union all
select 5709,'BTL' from dual union all
select 57090,'UNKS' from dual union all
select 57091,'AFSC' from dual union all
select 57092,'UNKS' from dual union all
select 57093,'AFSC' from dual union all
select 57094,'UNKS' from dual union all
select 57095,'UNKS' from dual union all
select 57096,'UNKS' from dual union all
select 57097,'UNKS' from dual union all
select 57098,'UNKS' from dual union all
select 57099,'UNKS' from dual union all
select 571,'WEHR' from dual union all
select 5710,'HAC' from dual union all
select 57100,'UNKS' from dual union all
select 57101,'UNKS' from dual union all
select 57102,'UNKS' from dual union all
select 57103,'AE' from dual union all
select 57104,'UNKS' from dual;
insert into LAUNCH_AGENCY
select 57105,'UNKS' from dual union all
select 57106,'UNKS' from dual union all
select 57107,'UNKS' from dual union all
select 57108,'UNKS' from dual union all
select 57109,'UNKS' from dual union all
select 5711,'HAC' from dual union all
select 57110,'UNKS' from dual union all
select 57111,'NASA' from dual union all
select 57112,'UNKS' from dual union all
select 57113,'AFSC' from dual union all
select 57114,'UNKS' from dual union all
select 57115,'UNKS' from dual union all
select 57116,'UNKS' from dual union all
select 57117,'UNKS' from dual union all
select 57118,'UNKS' from dual union all
select 57119,'AFSC' from dual union all
select 5712,'USA' from dual union all
select 57120,'UNKS' from dual union all
select 57121,'UNKS' from dual union all
select 57122,'UNKS' from dual union all
select 57123,'UNKS' from dual union all
select 57124,'UNKS' from dual union all
select 57125,'UNKS' from dual union all
select 57126,'UNKS' from dual union all
select 57127,'UNKS' from dual union all
select 57128,'UNKS' from dual union all
select 57129,'UNKS' from dual union all
select 5713,'HAC' from dual union all
select 57130,'UNKS' from dual union all
select 57131,'UNKS' from dual union all
select 57132,'UNKS' from dual union all
select 57133,'UNKS' from dual union all
select 57134,'NASDA' from dual union all
select 57135,'AE' from dual union all
select 57136,'UNKS' from dual union all
select 57137,'UNKS' from dual union all
select 57138,'UNKS' from dual union all
select 57139,'UNKS' from dual union all
select 5714,'HAC' from dual union all
select 57140,'UNKS' from dual union all
select 57141,'UNKS' from dual union all
select 57142,'NASA' from dual union all
select 57143,'UNKS' from dual union all
select 57144,'UNKS' from dual union all
select 57145,'AFSC' from dual union all
select 57146,'UNKS' from dual union all
select 57147,'NASA' from dual union all
select 57148,'UNKS' from dual union all
select 57149,'UNKS' from dual union all
select 5715,'RVSN' from dual union all
select 57150,'UNKS' from dual union all
select 57151,'AFSC' from dual union all
select 57152,'CALT' from dual union all
select 57153,'UNKS' from dual union all
select 57154,'UNKS' from dual union all
select 57155,'NASA' from dual union all
select 57156,'UNKS' from dual union all
select 57157,'UNKS' from dual union all
select 57158,'UNKS' from dual union all
select 57159,'UNKS' from dual union all
select 5716,'RVSN' from dual union all
select 57160,'UNKS' from dual union all
select 57161,'UNKS' from dual union all
select 57162,'NASA' from dual union all
select 57163,'UNKS' from dual union all
select 57164,'NASA' from dual union all
select 57165,'UNKS' from dual union all
select 57166,'UNKS' from dual union all
select 57167,'NASA' from dual union all
select 57168,'AE' from dual union all
select 57169,'NASA' from dual union all
select 5717,'WRE' from dual union all
select 57170,'UNKS' from dual union all
select 57171,'UNKS' from dual union all
select 57172,'UNKS' from dual union all
select 57173,'UNKS' from dual union all
select 57174,'UNKS' from dual union all
select 57175,'UNKS' from dual union all
select 57176,'AFSC' from dual union all
select 57177,'AFSC' from dual union all
select 57178,'UNKS' from dual union all
select 57179,'UNKS' from dual union all
select 5718,'WRE' from dual union all
select 57180,'UNKS' from dual union all
select 57181,'UNKS' from dual union all
select 57182,'UNKS' from dual union all
select 57183,'AFSC' from dual union all
select 57184,'MRN' from dual union all
select 57185,'MRN' from dual union all
select 57186,'MRN' from dual union all
select 57187,'MRN' from dual union all
select 57188,'MRN' from dual union all
select 57189,'MRN' from dual union all
select 5719,'WRE' from dual union all
select 57190,'MRN' from dual union all
select 57191,'MRN' from dual union all
select 57192,'MRN' from dual union all
select 57193,'MRN' from dual union all
select 57194,'MRN' from dual union all
select 57195,'MRN' from dual;
insert into LAUNCH_AGENCY
select 57196,'MRN' from dual union all
select 57197,'MRN' from dual union all
select 57198,'MRN' from dual union all
select 57199,'JMA' from dual union all
select 572,'WEHR' from dual union all
select 5720,'USAF' from dual union all
select 57200,'MRN' from dual union all
select 57201,'MRN' from dual union all
select 57202,'MRN' from dual union all
select 57203,'MRN' from dual union all
select 57204,'MRN' from dual union all
select 57205,'MRN' from dual union all
select 57206,'MRN' from dual union all
select 57207,'MRN' from dual union all
select 57208,'MRN' from dual union all
select 57209,'MRN' from dual union all
select 5721,'USA' from dual union all
select 57210,'MRN' from dual union all
select 57211,'STST' from dual union all
select 57212,'MRN' from dual union all
select 57213,'MRN' from dual union all
select 57214,'MRN' from dual union all
select 57215,'MRN' from dual union all
select 57216,'MRN' from dual union all
select 57217,'MRN' from dual union all
select 57218,'MRN' from dual union all
select 57219,'MRN' from dual union all
select 5722,'USAF' from dual union all
select 57220,'RVSN' from dual union all
select 57221,'MRN' from dual union all
select 57222,'MRN' from dual union all
select 57223,'SAND' from dual union all
select 57224,'ISRO' from dual union all
select 57225,'ISAS' from dual union all
select 57226,'NASA' from dual union all
select 57227,'DFVLR' from dual union all
select 57228,'ISRO' from dual union all
select 57229,'ISRO' from dual union all
select 5723,'USA' from dual union all
select 57230,'ISRO' from dual union all
select 57231,'ISRO' from dual union all
select 57232,'ISRO' from dual union all
select 57233,'ISRO' from dual union all
select 57234,'ISRO' from dual union all
select 57235,'ISRO' from dual union all
select 57236,'ISRO' from dual union all
select 57237,'ISRO' from dual union all
select 57238,'ISRO' from dual union all
select 57239,'ISRO' from dual union all
select 5724,'USAF' from dual union all
select 57240,'ISRO' from dual union all
select 57241,'ISRO' from dual union all
select 57242,'ISRO' from dual union all
select 57243,'ISRO' from dual union all
select 57244,'ISRO' from dual union all
select 57245,'ISRO' from dual union all
select 57246,'ISRO' from dual union all
select 57247,'ISRO' from dual union all
select 57248,'ISRO' from dual union all
select 57249,'ISRO' from dual union all
select 5725,'USA' from dual union all
select 57250,'ISRO' from dual union all
select 57251,'ISRO' from dual union all
select 57252,'ISRO' from dual union all
select 57253,'ISRO' from dual union all
select 57254,'ISRO' from dual union all
select 57255,'ISRO' from dual union all
select 57256,'ISRO' from dual union all
select 57257,'ISRO' from dual union all
select 57258,'ISRO' from dual union all
select 57259,'ISRO' from dual union all
select 5726,'USAF' from dual union all
select 57260,'ISRO' from dual union all
select 57261,'ISRO' from dual union all
select 57262,'ISRO' from dual union all
select 57263,'ISRO' from dual union all
select 57264,'ISRO' from dual union all
select 57265,'ISRO' from dual union all
select 57266,'ISRO' from dual union all
select 57267,'ISRO' from dual union all
select 57268,'ISRO' from dual union all
select 57269,'ISRO' from dual union all
select 5727,'USAF' from dual union all
select 57270,'ISRO' from dual union all
select 57271,'ISRO' from dual union all
select 57272,'ISRO' from dual union all
select 57273,'ISRO' from dual union all
select 57274,'ISRO' from dual union all
select 57275,'ISRO' from dual union all
select 57276,'ISRO' from dual union all
select 57277,'ISRO' from dual union all
select 57278,'ISRO' from dual union all
select 57279,'ISRO' from dual union all
select 5728,'USAF' from dual union all
select 57280,'ISRO' from dual union all
select 57281,'ISRO' from dual union all
select 57282,'ISRO' from dual union all
select 57283,'ISRO' from dual union all
select 57284,'ISRO' from dual union all
select 57285,'ISRO' from dual;
insert into LAUNCH_AGENCY
select 57286,'ISRO' from dual union all
select 57287,'ISRO' from dual union all
select 57288,'ISRO' from dual union all
select 57289,'NASA' from dual union all
select 5729,'USAF' from dual union all
select 57290,'NASA' from dual union all
select 57291,'NASA' from dual union all
select 57292,'NASA' from dual union all
select 57293,'NASA' from dual union all
select 57294,'NASA' from dual union all
select 57295,'ISRO' from dual union all
select 57296,'ISRO' from dual union all
select 57297,'ISRO' from dual union all
select 57298,'ISRO' from dual union all
select 57299,'ISRO' from dual union all
select 573,'WEHR' from dual union all
select 5730,'USN' from dual union all
select 57300,'ISRO' from dual union all
select 57301,'ISRO' from dual union all
select 57302,'ISRO' from dual union all
select 57303,'ISRO' from dual union all
select 57304,'ISRO' from dual union all
select 57305,'WFC' from dual union all
select 57306,'NASA' from dual union all
select 57307,'NASA' from dual union all
select 57308,'NASA' from dual union all
select 57309,'NASA' from dual union all
select 5731,'USA' from dual union all
select 57310,'NASA' from dual union all
select 57311,'NASA' from dual union all
select 57312,'NASA' from dual union all
select 57313,'NASA' from dual union all
select 57314,'NASA' from dual union all
select 57315,'NASA' from dual union all
select 57316,'ISRO' from dual union all
select 57317,'ISRO' from dual union all
select 57318,'ISRO' from dual union all
select 57319,'ISRO' from dual union all
select 5732,'USA' from dual union all
select 57320,'ISRO' from dual union all
select 57321,'ISRO' from dual union all
select 57322,'ISRO' from dual union all
select 57323,'ISRO' from dual union all
select 57324,'ISRO' from dual union all
select 57325,'ISRO' from dual union all
select 57326,'ISRO' from dual union all
select 57327,'ISRO' from dual union all
select 57328,'RVSN' from dual union all
select 57329,'RVSN' from dual union all
select 5733,'USA' from dual union all
select 57330,'SSC' from dual union all
select 57331,'SSC' from dual union all
select 57332,'RVSN' from dual union all
select 57333,'RVSN' from dual union all
select 57334,'RVSN' from dual union all
select 57335,'RVSN' from dual union all
select 57336,'RVSN' from dual union all
select 57337,'RVSN' from dual union all
select 57338,'DMA' from dual union all
select 57339,'NASA' from dual union all
select 5734,'USA' from dual union all
select 57340,'USN' from dual union all
select 57341,'IAE' from dual union all
select 57342,'NASA' from dual union all
select 57343,'NASA' from dual union all
select 57344,'USA' from dual union all
select 57345,'NASA' from dual union all
select 57346,'NASA' from dual union all
select 57347,'NASA' from dual union all
select 57348,'RVSN' from dual union all
select 57349,'USN' from dual union all
select 5735,'USA' from dual union all
select 57350,'USN' from dual union all
select 57351,'KPA' from dual union all
select 57352,'KPA' from dual union all
select 57353,'SAC' from dual union all
select 57354,'NASA' from dual union all
select 57355,'ALCS' from dual union all
select 57355,'SAC' from dual union all
select 57356,'NASA' from dual union all
select 57357,'9FA3' from dual union all
select 57358,'AFSC' from dual union all
select 57359,'RVSN' from dual union all
select 5736,'USAF' from dual union all
select 57360,'9FA3' from dual union all
select 57361,'SAC' from dual union all
select 57362,'NASA' from dual union all
select 57363,'USN' from dual union all
select 57364,'AFSC' from dual union all
select 57365,'NASA' from dual union all
select 57366,'SSC' from dual union all
select 57367,'NASA' from dual union all
select 57368,'RVSN' from dual union all
select 57369,'RVSN' from dual union all
select 5737,'RAE' from dual union all
select 5737,'WRE' from dual union all
select 57370,'RVSN' from dual union all
select 57371,'USAF' from dual union all
select 57372,'IAE' from dual union all
select 57373,'USN' from dual;
insert into LAUNCH_AGENCY
select 57374,'NASA' from dual union all
select 57375,'USN' from dual union all
select 57376,'USN' from dual union all
select 57377,'USN' from dual union all
select 57378,'INTA' from dual union all
select 57379,'INTA' from dual union all
select 5738,'CASDN' from dual union all
select 57380,'INTA' from dual union all
select 57381,'RVSN' from dual union all
select 57382,'RVSN' from dual union all
select 57383,'RVSN' from dual union all
select 57384,'VMF' from dual union all
select 57385,'TSAO' from dual union all
select 57386,'TSAO' from dual union all
select 57387,'TSAO' from dual union all
select 57388,'TSAO' from dual union all
select 57389,'IQ224' from dual union all
select 5739,'USAF' from dual union all
select 57390,'IQ224' from dual union all
select 57391,'IQ224' from dual union all
select 57392,'RVSN' from dual union all
select 57393,'IQ224' from dual union all
select 57394,'IQ224' from dual union all
select 57395,'IQ224' from dual union all
select 57396,'IQ224' from dual union all
select 57397,'IQ224' from dual union all
select 57398,'IQ224' from dual union all
select 57399,'IQ224' from dual union all
select 574,'WEHR' from dual union all
select 5740,'NASA' from dual union all
select 5740,'USAF' from dual union all
select 57400,'IQ224' from dual union all
select 57401,'IQ224' from dual union all
select 57402,'IQ224' from dual union all
select 57403,'RVSN' from dual union all
select 57404,'IQ224' from dual union all
select 57405,'IQ224' from dual union all
select 57406,'IQ224' from dual union all
select 57407,'IQ224' from dual union all
select 57408,'IQ224' from dual union all
select 57409,'IQ224' from dual union all
select 5741,'USAF' from dual union all
select 57410,'IQ224' from dual union all
select 57411,'IQ224' from dual union all
select 57412,'IQ224' from dual union all
select 57413,'IQ224' from dual union all
select 57414,'RVSN' from dual union all
select 57415,'IQ224' from dual union all
select 57416,'IQ224' from dual union all
select 57417,'IQ224' from dual union all
select 57418,'IQ224' from dual union all
select 57419,'IQ224' from dual union all
select 5742,'PARD' from dual union all
select 57420,'IQ224' from dual union all
select 57421,'IQ224' from dual union all
select 57422,'IQ224' from dual union all
select 57423,'IQ224' from dual union all
select 57424,'IQ224' from dual union all
select 57425,'RVSN' from dual union all
select 57426,'IQ224' from dual union all
select 57427,'IQ224' from dual union all
select 57428,'IQ224' from dual union all
select 57429,'IQ224' from dual union all
select 5743,'SU' from dual union all
select 57430,'IQ224' from dual union all
select 57431,'IQ224' from dual union all
select 57432,'IQ224' from dual union all
select 57433,'IQ224' from dual union all
select 57434,'IQ224' from dual union all
select 57435,'RVSN' from dual union all
select 57436,'IQ224' from dual union all
select 57437,'IQ224' from dual union all
select 57438,'IQ224' from dual union all
select 57439,'IQ224' from dual union all
select 5744,'USAF' from dual union all
select 57440,'IQ224' from dual union all
select 57441,'IQ224' from dual union all
select 57442,'IQ224' from dual union all
select 57443,'IQ224' from dual union all
select 57444,'IQ224' from dual union all
select 57445,'IQ224' from dual union all
select 57446,'RVSN' from dual union all
select 57447,'IQ224' from dual union all
select 57448,'FKG' from dual union all
select 57449,'FKG' from dual union all
select 5745,'USA' from dual union all
select 57450,'FKG' from dual union all
select 57451,'FKG' from dual union all
select 57452,'RVSN' from dual union all
select 57453,'RVSN' from dual union all
select 57454,'RVSN' from dual union all
select 57455,'RVSN' from dual union all
select 57456,'RVSN' from dual union all
select 57457,'RVSN' from dual union all
select 57458,'RVSN' from dual union all
select 57459,'RVSN' from dual union all
select 5746,'USA' from dual union all
select 57460,'RVSN' from dual union all
select 57461,'RVSN' from dual union all
select 57462,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 57463,'RVSN' from dual union all
select 57464,'RVSN' from dual union all
select 57465,'RVSN' from dual union all
select 57466,'RVSN' from dual union all
select 57467,'RVSN' from dual union all
select 57468,'RVSN' from dual union all
select 57469,'RVSN' from dual union all
select 5747,'USA' from dual union all
select 57470,'RVSN' from dual union all
select 57471,'RVSN' from dual union all
select 57472,'RVSN' from dual union all
select 57473,'RVSN' from dual union all
select 57474,'RVSN' from dual union all
select 57475,'RVSN' from dual union all
select 57476,'RVSN' from dual union all
select 57477,'RVSN' from dual union all
select 57478,'RVSN' from dual union all
select 57479,'RVSN' from dual union all
select 5748,'USA' from dual union all
select 57480,'RVSN' from dual union all
select 57481,'RVSN' from dual union all
select 57482,'RVSN' from dual union all
select 57483,'RVSN' from dual union all
select 57484,'RVSN' from dual union all
select 57485,'RVSN' from dual union all
select 57486,'INTA' from dual union all
select 57487,'INTA' from dual union all
select 57488,'INTA' from dual union all
select 57489,'INTA' from dual union all
select 5749,'LARCN' from dual union all
select 57490,'INTA' from dual union all
select 57491,'INTA' from dual union all
select 57492,'INTA' from dual union all
select 57493,'INTA' from dual union all
select 57494,'INTA' from dual union all
select 57495,'RVSN' from dual union all
select 57496,'RVSN' from dual union all
select 57497,'RVSN' from dual union all
select 57498,'RVSN' from dual union all
select 57499,'RVSN' from dual union all
select 575,'WEHR' from dual union all
select 5750,'NASA' from dual union all
select 5750,'USAF' from dual union all
select 57500,'RVSN' from dual union all
select 57501,'RVSN' from dual union all
select 57502,'RVSN' from dual union all
select 57503,'RVSN' from dual union all
select 57504,'RVSN' from dual union all
select 57505,'RVSN' from dual union all
select 57506,'RVSN' from dual union all
select 57507,'RVSN' from dual union all
select 57508,'RVSN' from dual union all
select 57509,'RVSN' from dual union all
select 5751,'US' from dual union all
select 57510,'RVSN' from dual union all
select 57511,'RVSN' from dual union all
select 57512,'RVSN' from dual union all
select 57513,'RVSN' from dual union all
select 57514,'RVSN' from dual union all
select 57515,'RVSN' from dual union all
select 57516,'RVSN' from dual union all
select 57517,'RVSN' from dual union all
select 57518,'RVSN' from dual union all
select 57519,'RVSN' from dual union all
select 5752,'USAF' from dual union all
select 57520,'RVSN' from dual union all
select 57521,'RVSN' from dual union all
select 57522,'RVSN' from dual union all
select 57523,'RVSN' from dual union all
select 57524,'RVSN' from dual union all
select 57525,'MVS' from dual union all
select 57526,'MVS' from dual union all
select 57527,'RVSN' from dual union all
select 57528,'KPA' from dual union all
select 57529,'RVSN' from dual union all
select 5753,'NASA' from dual union all
select 5753,'USAF' from dual union all
select 57530,'KPA' from dual union all
select 57531,'KPA' from dual union all
select 57532,'RVSN' from dual union all
select 57533,'RVSN' from dual union all
select 57534,'RVSN' from dual union all
select 57535,'RVSN' from dual union all
select 57536,'RVSN' from dual union all
select 57537,'RVSN' from dual union all
select 57538,'RVSN' from dual union all
select 57539,'RVSN' from dual union all
select 5754,'USAF' from dual union all
select 57540,'RVSN' from dual union all
select 57541,'RVSN' from dual union all
select 57542,'RVSN' from dual union all
select 57543,'DLR' from dual union all
select 57543,'NTNF' from dual union all
select 57544,'ISAS' from dual union all
select 57545,'DLR' from dual union all
select 57545,'NTNF' from dual union all
select 57546,'NASA' from dual union all
select 57547,'USAF' from dual union all
select 57548,'SAC' from dual union all
select 57549,'SAC' from dual;
insert into LAUNCH_AGENCY
select 5755,'SU' from dual union all
select 57550,'DLR' from dual union all
select 57550,'NTNF' from dual union all
select 57551,'AFBMO' from dual union all
select 57552,'DFVLR' from dual union all
select 57553,'DLR' from dual union all
select 57553,'NTNF' from dual union all
select 57554,'NTNF' from dual union all
select 57555,'AFBMO' from dual union all
select 57556,'NASA' from dual union all
select 57557,'NASA' from dual union all
select 57558,'DLR' from dual union all
select 57558,'NTNF' from dual union all
select 57559,'NTNF' from dual union all
select 5756,'USAF' from dual union all
select 57560,'DFVLR' from dual union all
select 57561,'DLR' from dual union all
select 57561,'NTNF' from dual union all
select 57562,'DLR' from dual union all
select 57562,'NTNF' from dual union all
select 57563,'NASA' from dual union all
select 57564,'DMA' from dual union all
select 57565,'NASA' from dual union all
select 57566,'NASA' from dual union all
select 57567,'NRCC' from dual union all
select 57568,'NASA' from dual union all
select 57569,'ALCS' from dual union all
select 57569,'SAC' from dual union all
select 5757,'USAF' from dual union all
select 57570,'NRCC' from dual union all
select 57571,'NASA' from dual union all
select 57572,'USN' from dual union all
select 57573,'AFSC' from dual union all
select 57574,'NASA' from dual union all
select 57575,'KPA' from dual union all
select 57576,'RVSN' from dual union all
select 57577,'NASA' from dual union all
select 57578,'NASA' from dual union all
select 57579,'NASA' from dual union all
select 5758,'USAF' from dual union all
select 57580,'NIPR' from dual union all
select 57581,'USN' from dual union all
select 57582,'SAC' from dual union all
select 57583,'ALCS' from dual union all
select 57583,'SAC' from dual union all
select 57584,'USN' from dual union all
select 57585,'USN' from dual union all
select 57586,'DFVLR' from dual union all
select 57587,'NIPR' from dual union all
select 57588,'NASA' from dual union all
select 57589,'NASA' from dual union all
select 5759,'USAF' from dual union all
select 57590,'NRCC' from dual union all
select 57591,'DFVLR' from dual union all
select 57592,'USA' from dual union all
select 57593,'NIPR' from dual union all
select 57594,'AFSC' from dual union all
select 57595,'AFSC' from dual union all
select 57596,'AFSC' from dual union all
select 57597,'NASA' from dual union all
select 57598,'NASA' from dual union all
select 57599,'USN' from dual union all
select 576,'WEHR' from dual union all
select 5760,'USAF' from dual union all
select 57600,'USN' from dual union all
select 57601,'UNKS' from dual union all
select 57602,'MRN' from dual union all
select 57603,'DFVLR' from dual union all
select 57604,'DFVLR' from dual union all
select 57605,'MRN' from dual union all
select 57606,'MRN' from dual union all
select 57607,'DFVLR' from dual union all
select 57608,'DFVLR' from dual union all
select 57609,'MRN' from dual union all
select 5761,'USAF' from dual union all
select 57610,'WFC' from dual union all
select 57611,'MRN' from dual union all
select 57612,'MRN' from dual union all
select 57613,'JMA' from dual union all
select 57614,'MRN' from dual union all
select 57615,'MRN' from dual union all
select 57616,'MRN' from dual union all
select 57617,'JMA' from dual union all
select 57618,'MRN' from dual union all
select 57619,'MRN' from dual union all
select 5762,'USAF' from dual union all
select 57620,'WFC' from dual union all
select 57621,'MRN' from dual union all
select 57622,'JMA' from dual union all
select 57623,'MRN' from dual union all
select 57624,'MRN' from dual union all
select 57625,'MRN' from dual union all
select 57626,'MRN' from dual union all
select 57627,'MRN' from dual union all
select 57628,'MRN' from dual union all
select 57629,'MRN' from dual union all
select 5763,'USA' from dual union all
select 57630,'MRN' from dual union all
select 57631,'MRN' from dual union all
select 57632,'MRN' from dual;
insert into LAUNCH_AGENCY
select 57633,'MRN' from dual union all
select 57634,'JMA' from dual union all
select 57635,'MRN' from dual union all
select 57636,'WFC' from dual union all
select 57637,'MRN' from dual union all
select 57638,'MRN' from dual union all
select 57639,'MRN' from dual union all
select 5764,'USA' from dual union all
select 57640,'MRN' from dual union all
select 57641,'MRN' from dual union all
select 57642,'MRN' from dual union all
select 57643,'MRN' from dual union all
select 57644,'MRN' from dual union all
select 57645,'MRN' from dual union all
select 57646,'MRN' from dual union all
select 57647,'MRN' from dual union all
select 57648,'MRN' from dual union all
select 57649,'MRN' from dual union all
select 5765,'USA' from dual union all
select 57650,'MRN' from dual union all
select 57651,'MRN' from dual union all
select 57652,'MRN' from dual union all
select 57653,'MRN' from dual union all
select 57654,'MRN' from dual union all
select 57655,'DFVLR' from dual union all
select 57656,'MRN' from dual union all
select 57657,'MRN' from dual union all
select 57658,'JMA' from dual union all
select 57659,'MRN' from dual union all
select 5766,'USA' from dual union all
select 57660,'MRN' from dual union all
select 57661,'MRN' from dual union all
select 57662,'MRN' from dual union all
select 57663,'MRN' from dual union all
select 57664,'MRN' from dual union all
select 57665,'WFC' from dual union all
select 57666,'DFVLR' from dual union all
select 57667,'MRN' from dual union all
select 57668,'MRN' from dual union all
select 57669,'MRN' from dual union all
select 5767,'USN' from dual union all
select 57670,'MRN' from dual union all
select 57671,'MRN' from dual union all
select 57672,'MRN' from dual union all
select 57673,'MRN' from dual union all
select 57674,'MRN' from dual union all
select 57675,'MRN' from dual union all
select 57676,'MRN' from dual union all
select 57677,'DFVLR' from dual union all
select 57678,'MRN' from dual union all
select 57679,'MRN' from dual union all
select 5768,'USAF' from dual union all
select 57680,'JMA' from dual union all
select 57681,'MRN' from dual union all
select 57682,'MRN' from dual union all
select 57683,'MRN' from dual union all
select 57684,'WFC' from dual union all
select 57685,'MRN' from dual union all
select 57686,'MRN' from dual union all
select 57687,'MRN' from dual union all
select 57688,'DFVLR' from dual union all
select 57689,'MRN' from dual union all
select 5769,'NASA' from dual union all
select 57690,'MRN' from dual union all
select 57691,'MRN' from dual union all
select 57692,'MRN' from dual union all
select 57693,'MRN' from dual union all
select 57694,'WFC' from dual union all
select 57695,'MRN' from dual union all
select 57696,'JMA' from dual union all
select 57697,'MRN' from dual union all
select 57698,'MRN' from dual union all
select 57699,'DFVLR' from dual union all
select 577,'WEHR' from dual union all
select 5770,'USN' from dual union all
select 57700,'MRN' from dual union all
select 57701,'JMA' from dual union all
select 57702,'MRN' from dual union all
select 57703,'MRN' from dual union all
select 57704,'MRN' from dual union all
select 57705,'MRN' from dual union all
select 57706,'JMA' from dual union all
select 57707,'MRN' from dual union all
select 57708,'MRN' from dual union all
select 57709,'MRN' from dual union all
select 5771,'USAF' from dual union all
select 57710,'MRN' from dual union all
select 57711,'JMA' from dual union all
select 57712,'MRN' from dual union all
select 57713,'MRN' from dual union all
select 57714,'MRN' from dual union all
select 57715,'MRN' from dual union all
select 57716,'MRN' from dual union all
select 57717,'MRN' from dual union all
select 57718,'NASA' from dual union all
select 57719,'MRN' from dual union all
select 5772,'NASA' from dual union all
select 5772,'USAF' from dual union all
select 57720,'MRN' from dual union all
select 57721,'MRN' from dual;
insert into LAUNCH_AGENCY
select 57722,'JMA' from dual union all
select 57723,'MRN' from dual union all
select 57724,'MRN' from dual union all
select 57725,'MRN' from dual union all
select 57726,'MRN' from dual union all
select 57727,'MRN' from dual union all
select 57728,'NASA' from dual union all
select 57729,'JMA' from dual union all
select 5773,'NASA' from dual union all
select 57730,'MRN' from dual union all
select 57731,'MRN' from dual union all
select 57732,'MRN' from dual union all
select 57733,'MRN' from dual union all
select 57734,'MRN' from dual union all
select 57735,'MRN' from dual union all
select 57736,'MRN' from dual union all
select 57737,'MRN' from dual union all
select 57738,'MRN' from dual union all
select 57739,'MRN' from dual union all
select 5774,'USAF' from dual union all
select 57740,'MRN' from dual union all
select 57741,'MRN' from dual union all
select 57742,'MRN' from dual union all
select 57743,'WFC' from dual union all
select 57744,'MRN' from dual union all
select 57745,'JMA' from dual union all
select 57746,'MRN' from dual union all
select 57747,'MRN' from dual union all
select 57748,'MRN' from dual union all
select 57749,'MRN' from dual union all
select 5775,'USAF' from dual union all
select 57750,'MRN' from dual union all
select 57751,'MRN' from dual union all
select 57752,'MRN' from dual union all
select 57753,'MRN' from dual union all
select 57754,'MRN' from dual union all
select 57755,'MRN' from dual union all
select 57756,'MRN' from dual union all
select 57757,'MRN' from dual union all
select 57758,'JMA' from dual union all
select 57759,'MRN' from dual union all
select 5776,'PARD' from dual union all
select 57760,'MRN' from dual union all
select 57761,'MRN' from dual union all
select 57762,'MRN' from dual union all
select 57763,'MRN' from dual union all
select 57764,'MRN' from dual union all
select 57765,'MRN' from dual union all
select 57766,'MRN' from dual union all
select 57767,'MRN' from dual union all
select 57768,'JMA' from dual union all
select 57769,'MRN' from dual union all
select 5777,'USA' from dual union all
select 57770,'MRN' from dual union all
select 57771,'MRN' from dual union all
select 57772,'MRN' from dual union all
select 57773,'MRN' from dual union all
select 57774,'MRN' from dual union all
select 57775,'MRN' from dual union all
select 57776,'DFVLR' from dual union all
select 57777,'MRN' from dual union all
select 57778,'MRN' from dual union all
select 57779,'MRN' from dual union all
select 5778,'USA' from dual union all
select 57780,'MRN' from dual union all
select 57781,'MRN' from dual union all
select 57782,'MRN' from dual union all
select 57783,'MRN' from dual union all
select 57784,'MRN' from dual union all
select 57785,'MRN' from dual union all
select 57786,'MRN' from dual union all
select 57787,'JMA' from dual union all
select 57788,'MRN' from dual union all
select 57789,'MRN' from dual union all
select 5779,'USA' from dual union all
select 57790,'NASA' from dual union all
select 57791,'MRN' from dual union all
select 57792,'MRN' from dual union all
select 57793,'MRN' from dual union all
select 57794,'MRN' from dual union all
select 57795,'MRN' from dual union all
select 57796,'JMA' from dual union all
select 57797,'MRN' from dual union all
select 57798,'MRN' from dual union all
select 57799,'MRN' from dual union all
select 578,'WEHR' from dual union all
select 5780,'USA' from dual union all
select 57800,'MRN' from dual union all
select 57801,'MRN' from dual union all
select 57802,'MRN' from dual union all
select 57803,'MRN' from dual union all
select 57804,'MRN' from dual union all
select 57805,'MRN' from dual union all
select 57806,'MRN' from dual union all
select 57807,'MRN' from dual union all
select 57808,'MRN' from dual union all
select 57809,'DFVLR' from dual union all
select 5781,'USA' from dual union all
select 57810,'MRN' from dual union all
select 57811,'MRN' from dual;
insert into LAUNCH_AGENCY
select 57812,'MRN' from dual union all
select 57813,'MRN' from dual union all
select 57814,'MRN' from dual union all
select 57815,'MRN' from dual union all
select 57816,'MRN' from dual union all
select 57817,'MRN' from dual union all
select 57818,'NASA' from dual union all
select 57819,'MRN' from dual union all
select 5782,'STA' from dual union all
select 57820,'MRN' from dual union all
select 57821,'MRN' from dual union all
select 57822,'MRN' from dual union all
select 57823,'MRN' from dual union all
select 57824,'JMA' from dual union all
select 57825,'MRN' from dual union all
select 57826,'MRN' from dual union all
select 57827,'MRN' from dual union all
select 57828,'NASA' from dual union all
select 57829,'MRN' from dual union all
select 5783,'STA' from dual union all
select 57830,'MRN' from dual union all
select 57831,'MRN' from dual union all
select 57832,'MRN' from dual union all
select 57833,'MRN' from dual union all
select 57834,'MRN' from dual union all
select 57835,'MRN' from dual union all
select 57836,'JMA' from dual union all
select 57837,'MRN' from dual union all
select 57838,'MRN' from dual union all
select 57839,'MRN' from dual union all
select 5784,'NASA' from dual union all
select 5784,'USAF' from dual union all
select 57840,'MRN' from dual union all
select 57841,'MRN' from dual union all
select 57842,'MRN' from dual union all
select 57843,'NASA' from dual union all
select 57844,'MRN' from dual union all
select 57845,'MRN' from dual union all
select 57846,'MRN' from dual union all
select 57847,'MRN' from dual union all
select 57848,'MRN' from dual union all
select 57849,'MRN' from dual union all
select 5785,'USAF' from dual union all
select 57850,'MRN' from dual union all
select 57851,'MRN' from dual union all
select 57852,'MRN' from dual union all
select 57853,'MRN' from dual union all
select 57854,'MRN' from dual union all
select 57855,'MRN' from dual union all
select 57856,'MRN' from dual union all
select 57857,'MRN' from dual union all
select 57858,'MRN' from dual union all
select 57859,'MRN' from dual union all
select 5786,'USAF' from dual union all
select 57860,'MRN' from dual union all
select 57861,'MRN' from dual union all
select 57862,'MRN' from dual union all
select 57863,'JMA' from dual union all
select 57864,'MRN' from dual union all
select 57865,'MRN' from dual union all
select 57866,'MRN' from dual union all
select 57867,'MRN' from dual union all
select 57868,'MRN' from dual union all
select 57869,'MRN' from dual union all
select 5787,'USAF' from dual union all
select 57870,'MRN' from dual union all
select 57871,'MRN' from dual union all
select 57872,'NASA' from dual union all
select 57873,'MRN' from dual union all
select 57874,'MRN' from dual union all
select 57875,'DFVLR' from dual union all
select 57876,'MRN' from dual union all
select 57877,'MRN' from dual union all
select 57878,'MRN' from dual union all
select 57879,'MRN' from dual union all
select 5788,'USAF' from dual union all
select 57880,'MRN' from dual union all
select 57881,'MRN' from dual union all
select 57882,'MRN' from dual union all
select 57883,'JMA' from dual union all
select 57884,'MRN' from dual union all
select 57885,'MRN' from dual union all
select 57886,'JMA' from dual union all
select 57887,'MRN' from dual union all
select 57888,'JMA' from dual union all
select 57889,'MRN' from dual union all
select 5789,'USN' from dual union all
select 57890,'MRN' from dual union all
select 57891,'MRN' from dual union all
select 57892,'NASA' from dual union all
select 57893,'MRN' from dual union all
select 57894,'MRN' from dual union all
select 57895,'MRN' from dual union all
select 57896,'MRN' from dual union all
select 57897,'MRN' from dual union all
select 57898,'JMA' from dual union all
select 57899,'MRN' from dual union all
select 579,'WEHR' from dual union all
select 5790,'USAF' from dual union all
select 57900,'MRN' from dual;
insert into LAUNCH_AGENCY
select 57901,'MRN' from dual union all
select 57902,'MRN' from dual union all
select 57903,'MRN' from dual union all
select 57904,'MRN' from dual union all
select 57905,'MRN' from dual union all
select 57906,'MRN' from dual union all
select 57907,'JMA' from dual union all
select 57908,'DFVLR' from dual union all
select 57909,'MRN' from dual union all
select 5791,'USAF' from dual union all
select 57910,'MRN' from dual union all
select 57911,'MRN' from dual union all
select 57912,'MRN' from dual union all
select 57913,'MRN' from dual union all
select 57914,'MRN' from dual union all
select 57915,'MRN' from dual union all
select 57916,'MRN' from dual union all
select 57917,'MRN' from dual union all
select 57918,'MRN' from dual union all
select 57919,'MRN' from dual union all
select 5792,'USN' from dual union all
select 57920,'MRN' from dual union all
select 57921,'MRN' from dual union all
select 57922,'MRN' from dual union all
select 57923,'MRN' from dual union all
select 57924,'MRN' from dual union all
select 57925,'MRN' from dual union all
select 57926,'JMA' from dual union all
select 57927,'MRN' from dual union all
select 57928,'MRN' from dual union all
select 57929,'MRN' from dual union all
select 5793,'USAF' from dual union all
select 57930,'MRN' from dual union all
select 57931,'MRN' from dual union all
select 57932,'MRN' from dual union all
select 57933,'MRN' from dual union all
select 57934,'MRN' from dual union all
select 57935,'MRN' from dual union all
select 57936,'MRN' from dual union all
select 57937,'MRN' from dual union all
select 57938,'MRN' from dual union all
select 57939,'MRN' from dual union all
select 5794,'USN' from dual union all
select 57940,'MRN' from dual union all
select 57941,'MRN' from dual union all
select 57942,'MRN' from dual union all
select 57943,'MRN' from dual union all
select 57944,'MRN' from dual union all
select 57945,'MRN' from dual union all
select 57946,'MRN' from dual union all
select 57947,'MRN' from dual union all
select 57948,'JMA' from dual union all
select 57949,'MRN' from dual union all
select 5795,'NASA' from dual union all
select 57950,'MRN' from dual union all
select 57951,'MRN' from dual union all
select 57952,'MRN' from dual union all
select 57953,'MRN' from dual union all
select 57954,'MRN' from dual union all
select 57955,'MRN' from dual union all
select 57956,'MRN' from dual union all
select 57957,'MRN' from dual union all
select 57958,'MRN' from dual union all
select 57959,'MRN' from dual union all
select 5796,'MVS' from dual union all
select 57960,'MRN' from dual union all
select 57961,'MRN' from dual union all
select 57962,'MRN' from dual union all
select 57963,'DFVLR' from dual union all
select 57964,'JMA' from dual union all
select 57965,'MRN' from dual union all
select 57966,'MRN' from dual union all
select 57967,'MRN' from dual union all
select 57968,'MRN' from dual union all
select 57969,'MRN' from dual union all
select 5797,'USA' from dual union all
select 57970,'MRN' from dual union all
select 57971,'MRN' from dual union all
select 57972,'JMA' from dual union all
select 57973,'MRN' from dual union all
select 57974,'DFVLR' from dual union all
select 57975,'MRN' from dual union all
select 57976,'MRN' from dual union all
select 57977,'MRN' from dual union all
select 57978,'MRN' from dual union all
select 57979,'MRN' from dual union all
select 5798,'USAF' from dual union all
select 57980,'MRN' from dual union all
select 57981,'MRN' from dual union all
select 57982,'MRN' from dual union all
select 57983,'JMA' from dual union all
select 57984,'MRN' from dual union all
select 57985,'MRN' from dual union all
select 57986,'MRN' from dual union all
select 57987,'MRN' from dual union all
select 57988,'MRN' from dual union all
select 57989,'MRN' from dual union all
select 5799,'USAF' from dual union all
select 57990,'MRN' from dual union all
select 57991,'MRN' from dual;
insert into LAUNCH_AGENCY
select 57992,'MRN' from dual union all
select 57993,'JMA' from dual union all
select 57994,'MRN' from dual union all
select 57995,'MRN' from dual union all
select 57996,'MRN' from dual union all
select 57997,'MRN' from dual union all
select 57998,'MRN' from dual union all
select 57999,'MRN' from dual union all
select 58,'WER271-1' from dual union all
select 580,'WEHR' from dual union all
select 5800,'NASA' from dual union all
select 58000,'DFVLR' from dual union all
select 58001,'MRN' from dual union all
select 58002,'MRN' from dual union all
select 58003,'JMA' from dual union all
select 58004,'MRN' from dual union all
select 58005,'DFVLR' from dual union all
select 58006,'MRN' from dual union all
select 58007,'MRN' from dual union all
select 58008,'MRN' from dual union all
select 58009,'MRN' from dual union all
select 5801,'NASA' from dual union all
select 58010,'DFVLR' from dual union all
select 58011,'MRN' from dual union all
select 58012,'MRN' from dual union all
select 58013,'MRN' from dual union all
select 58014,'MRN' from dual union all
select 58015,'MRN' from dual union all
select 58016,'MRN' from dual union all
select 58017,'WFC' from dual union all
select 58018,'MRN' from dual union all
select 58019,'MRN' from dual union all
select 5802,'USN' from dual union all
select 58020,'DFVLR' from dual union all
select 58021,'MRN' from dual union all
select 58022,'DFVLR' from dual union all
select 58023,'MRN' from dual union all
select 58024,'JMA' from dual union all
select 58025,'MRN' from dual union all
select 58026,'DFVLR' from dual union all
select 58027,'MRN' from dual union all
select 58028,'MRN' from dual union all
select 58029,'DFVLR' from dual union all
select 5803,'USA' from dual union all
select 58030,'DFVLR' from dual union all
select 58031,'DFVLR' from dual union all
select 58032,'MRN' from dual union all
select 58033,'MRN' from dual union all
select 58034,'DFVLR' from dual union all
select 58035,'MRN' from dual union all
select 58036,'JMA' from dual union all
select 58037,'MRN' from dual union all
select 58038,'MRN' from dual union all
select 58039,'MRN' from dual union all
select 5804,'USA' from dual union all
select 58040,'MRN' from dual union all
select 58041,'MRN' from dual union all
select 58042,'MRN' from dual union all
select 58043,'MRN' from dual union all
select 58044,'MRN' from dual union all
select 58045,'MRN' from dual union all
select 58046,'MRN' from dual union all
select 58047,'MRN' from dual union all
select 58048,'WFC' from dual union all
select 58049,'DFVLR' from dual union all
select 5805,'SAND' from dual union all
select 58050,'MRN' from dual union all
select 58051,'DFVLR' from dual union all
select 58052,'DFVLR' from dual union all
select 58053,'DFVLR' from dual union all
select 58054,'DFVLR' from dual union all
select 58055,'DFVLR' from dual union all
select 58056,'MRN' from dual union all
select 58057,'DFVLR' from dual union all
select 58058,'DFVLR' from dual union all
select 58059,'MRN' from dual union all
select 5806,'ISAS' from dual union all
select 58060,'MRN' from dual union all
select 58061,'MRN' from dual union all
select 58062,'MRN' from dual union all
select 58063,'MRN' from dual union all
select 58064,'MRN' from dual union all
select 58065,'MRN' from dual union all
select 58066,'MRN' from dual union all
select 58067,'MRN' from dual union all
select 58068,'NASA' from dual union all
select 58069,'DFVLR' from dual union all
select 5807,'ISAS' from dual union all
select 58070,'MRN' from dual union all
select 58071,'MRN' from dual union all
select 58072,'MRN' from dual union all
select 58073,'MRN' from dual union all
select 58074,'NASA' from dual union all
select 58075,'MRN' from dual union all
select 58076,'MRN' from dual union all
select 58077,'NASA' from dual union all
select 58078,'MRN' from dual union all
select 58079,'MRN' from dual union all
select 5808,'ISAS' from dual union all
select 58080,'DFVLR' from dual;
insert into LAUNCH_AGENCY
select 58081,'MRN' from dual union all
select 58082,'MRN' from dual union all
select 58083,'NASA' from dual union all
select 58084,'MRN' from dual union all
select 58085,'MRN' from dual union all
select 58086,'MRN' from dual union all
select 58087,'MRN' from dual union all
select 58088,'MRN' from dual union all
select 58089,'MRN' from dual union all
select 5809,'ISAS' from dual union all
select 58090,'NASA' from dual union all
select 58091,'DFVLR' from dual union all
select 58092,'MRN' from dual union all
select 58093,'MRN' from dual union all
select 58094,'MRN' from dual union all
select 58095,'MRN' from dual union all
select 58096,'MRN' from dual union all
select 58097,'MRN' from dual union all
select 58098,'ISRO' from dual union all
select 58099,'AN' from dual union all
select 581,'WEHR' from dual union all
select 5810,'SAND' from dual union all
select 58100,'AN' from dual union all
select 58101,'AN' from dual union all
select 58102,'DFVLR' from dual union all
select 58103,'AN' from dual union all
select 58104,'AN' from dual union all
select 58105,'AN' from dual union all
select 58106,'AN' from dual union all
select 58107,'AN' from dual union all
select 58108,'AN' from dual union all
select 58109,'AN' from dual union all
select 5811,'SAND' from dual union all
select 58110,'AN' from dual union all
select 58111,'AN' from dual union all
select 58112,'AN' from dual union all
select 58113,'DFVLR' from dual union all
select 58114,'AN' from dual union all
select 58115,'AN' from dual union all
select 58116,'AN' from dual union all
select 58117,'AN' from dual union all
select 58118,'AN' from dual union all
select 58119,'AN' from dual union all
select 5812,'NASA' from dual union all
select 58120,'AN' from dual union all
select 58121,'AN' from dual union all
select 58122,'AN' from dual union all
select 58123,'AN' from dual union all
select 58124,'DFVLR' from dual union all
select 58125,'AN' from dual union all
select 58126,'AN' from dual union all
select 58127,'AN' from dual union all
select 58128,'AN' from dual union all
select 58129,'AN' from dual union all
select 5813,'NASA' from dual union all
select 58130,'AN' from dual union all
select 58131,'AN' from dual union all
select 58132,'AN' from dual union all
select 58133,'AN' from dual union all
select 58134,'AN' from dual union all
select 58135,'DFVLR' from dual union all
select 58136,'AN' from dual union all
select 58137,'AN' from dual union all
select 58138,'AN' from dual union all
select 58139,'AN' from dual union all
select 5814,'USAF' from dual union all
select 58140,'AN' from dual union all
select 58141,'AN' from dual union all
select 58142,'DFVLR' from dual union all
select 58143,'DFVLR' from dual union all
select 58144,'MRN' from dual union all
select 58145,'DFVLR' from dual union all
select 58146,'DFVLR' from dual union all
select 58147,'DFVLR' from dual union all
select 58148,'DFVLR' from dual union all
select 58149,'DFVLR' from dual union all
select 5815,'USAF' from dual union all
select 58150,'DFVLR' from dual union all
select 58151,'DFVLR' from dual union all
select 58152,'DFVLR' from dual union all
select 58153,'DFVLR' from dual union all
select 58154,'DFVLR' from dual union all
select 58155,'DFVLR' from dual union all
select 58156,'DFVLR' from dual union all
select 58157,'DFVLR' from dual union all
select 58158,'DFVLR' from dual union all
select 58159,'DFVLR' from dual union all
select 5816,'USAF' from dual union all
select 58160,'DFVLR' from dual union all
select 58161,'MRN' from dual union all
select 58162,'DFVLR' from dual union all
select 58163,'DFVLR' from dual union all
select 58164,'DFVLR' from dual union all
select 58165,'DFVLR' from dual union all
select 58166,'MRN' from dual union all
select 58167,'MRN' from dual union all
select 58168,'DFVLR' from dual union all
select 58169,'MRN' from dual union all
select 5817,'USAF' from dual union all
select 58170,'DFVLR' from dual;
insert into LAUNCH_AGENCY
select 58171,'MRN' from dual union all
select 58172,'DFVLR' from dual union all
select 58173,'MRN' from dual union all
select 58174,'DFVLR' from dual union all
select 58175,'DFVLR' from dual union all
select 58176,'MRN' from dual union all
select 58177,'DFVLR' from dual union all
select 58178,'DFVLR' from dual union all
select 58179,'DFVLR' from dual union all
select 5818,'SAND' from dual union all
select 58180,'DFVLR' from dual union all
select 58181,'DFVLR' from dual union all
select 58182,'WFC' from dual union all
select 58183,'MRN' from dual union all
select 58184,'DFVLR' from dual union all
select 58185,'DFVLR' from dual union all
select 58186,'DFVLR' from dual union all
select 58187,'DFVLR' from dual union all
select 58188,'DFVLR' from dual union all
select 58189,'DFVLR' from dual union all
select 5819,'USAF' from dual union all
select 58190,'DFVLR' from dual union all
select 58191,'DFVLR' from dual union all
select 58192,'DFVLR' from dual union all
select 58193,'DFVLR' from dual union all
select 58194,'DFVLR' from dual union all
select 58195,'DFVLR' from dual union all
select 58196,'DFVLR' from dual union all
select 58197,'DFVLR' from dual union all
select 58198,'DFVLR' from dual union all
select 58199,'MRN' from dual union all
select 582,'WEHR' from dual union all
select 5820,'RAE' from dual union all
select 5820,'WRE' from dual union all
select 58200,'DFVLR' from dual union all
select 58201,'NASA' from dual union all
select 58202,'DFVLR' from dual union all
select 58203,'MRN' from dual union all
select 58204,'MRN' from dual union all
select 58205,'MRN' from dual union all
select 58206,'MRN' from dual union all
select 58207,'MRN' from dual union all
select 58208,'MRN' from dual union all
select 58209,'MRN' from dual union all
select 5821,'SAND' from dual union all
select 58210,'MRN' from dual union all
select 58211,'MRN' from dual union all
select 58212,'WFC' from dual union all
select 58213,'MRN' from dual union all
select 58214,'MRN' from dual union all
select 58215,'MRN' from dual union all
select 58216,'MRN' from dual union all
select 58217,'NASA' from dual union all
select 58218,'NASA' from dual union all
select 58219,'NASA' from dual union all
select 5822,'SAND' from dual union all
select 58220,'WFC' from dual union all
select 58221,'NASA' from dual union all
select 58222,'MRN' from dual union all
select 58223,'NASA' from dual union all
select 58224,'ISAS' from dual union all
select 58225,'UNKS' from dual union all
select 58226,'UNKS' from dual union all
select 58227,'UNKS' from dual union all
select 58228,'UNKS' from dual union all
select 58229,'UNKS' from dual union all
select 5823,'USA' from dual union all
select 58230,'UNKS' from dual union all
select 58231,'UNKS' from dual union all
select 58232,'UNKS' from dual union all
select 58233,'NASA' from dual union all
select 58234,'UNKS' from dual union all
select 58235,'UNKS' from dual union all
select 58236,'UNKS' from dual union all
select 58237,'AFSC' from dual union all
select 58238,'AE' from dual union all
select 58239,'UNKS' from dual union all
select 5824,'NASA' from dual union all
select 58240,'UNKS' from dual union all
select 58241,'UNKS' from dual union all
select 58242,'UNKS' from dual union all
select 58243,'UNKS' from dual union all
select 58244,'AFSC' from dual union all
select 58245,'UNKS' from dual union all
select 58246,'UNKS' from dual union all
select 58247,'UNKS' from dual union all
select 58248,'NASA' from dual union all
select 58249,'UNKS' from dual union all
select 5825,'SAND' from dual union all
select 58250,'UNKS' from dual union all
select 58251,'NASA' from dual union all
select 58252,'UNKS' from dual union all
select 58253,'UNKS' from dual union all
select 58254,'UNKS' from dual union all
select 58255,'UNKS' from dual union all
select 58256,'UNKS' from dual union all
select 58257,'NASA' from dual union all
select 58258,'AE' from dual union all
select 58259,'UNKS' from dual union all
select 5826,'SAND' from dual;
insert into LAUNCH_AGENCY
select 58260,'UNKS' from dual union all
select 58261,'UNKS' from dual union all
select 58262,'UNKS' from dual union all
select 58263,'UNKS' from dual union all
select 58264,'UNKS' from dual union all
select 58265,'UNKS' from dual union all
select 58266,'UNKS' from dual union all
select 58267,'UNKS' from dual union all
select 58268,'UNKS' from dual union all
select 58269,'UNKS' from dual union all
select 5827,'SAND' from dual union all
select 58270,'UNKS' from dual union all
select 58271,'NASA' from dual union all
select 58272,'UNKS' from dual union all
select 58273,'UNKS' from dual union all
select 58274,'UNKS' from dual union all
select 58275,'UNKS' from dual union all
select 58276,'UNKS' from dual union all
select 58277,'UNKS' from dual union all
select 58278,'NASA' from dual union all
select 58279,'AE' from dual union all
select 5828,'SAND' from dual union all
select 58280,'UNKS' from dual union all
select 58281,'UNKS' from dual union all
select 58282,'UNKS' from dual union all
select 58283,'UNKS' from dual union all
select 58284,'UNKS' from dual union all
select 58285,'UNKS' from dual union all
select 58286,'NASA' from dual union all
select 58287,'UNKS' from dual union all
select 58288,'UNKS' from dual union all
select 58289,'NASA' from dual union all
select 5829,'SAND' from dual union all
select 58290,'UNKS' from dual union all
select 58291,'UNKS' from dual union all
select 58292,'UNKS' from dual union all
select 58293,'UNKS' from dual union all
select 58294,'UNKS' from dual union all
select 58295,'UNKS' from dual union all
select 58296,'ISAS' from dual union all
select 58297,'UNKS' from dual union all
select 58298,'UNKS' from dual union all
select 58299,'NASA' from dual union all
select 583,'WEHR' from dual union all
select 5830,'SAND' from dual union all
select 58300,'UNKS' from dual union all
select 58301,'UNKS' from dual union all
select 58302,'UNKS' from dual union all
select 58303,'UNKS' from dual union all
select 58304,'UNKS' from dual union all
select 58305,'UNKS' from dual union all
select 58306,'UNKS' from dual union all
select 58307,'UNKS' from dual union all
select 58308,'UNKS' from dual union all
select 58309,'UNKS' from dual union all
select 5831,'USA' from dual union all
select 58310,'NASA' from dual union all
select 58311,'UNKS' from dual union all
select 58312,'UNKS' from dual union all
select 58313,'UNKS' from dual union all
select 58314,'UNKS' from dual union all
select 58315,'NASA' from dual union all
select 58316,'AFSC' from dual union all
select 58317,'UNKS' from dual union all
select 58318,'UNKS' from dual union all
select 58319,'CALT' from dual union all
select 5832,'USA' from dual union all
select 58320,'UNKS' from dual union all
select 58321,'UNKS' from dual union all
select 58322,'UNKS' from dual union all
select 58323,'UNKS' from dual union all
select 58324,'UNKS' from dual union all
select 58325,'UNKS' from dual union all
select 58326,'UNKS' from dual union all
select 58327,'NASA' from dual union all
select 58328,'UNKS' from dual union all
select 58329,'UNKS' from dual union all
select 5833,'USA' from dual union all
select 58330,'UNKS' from dual union all
select 58331,'UNKS' from dual union all
select 58332,'NASA' from dual union all
select 58333,'UNKS' from dual union all
select 58334,'UNKS' from dual union all
select 58335,'UNKS' from dual union all
select 58336,'UNKS' from dual union all
select 58337,'NASA' from dual union all
select 58338,'UNKS' from dual union all
select 58339,'UNKS' from dual union all
select 5834,'NASA' from dual union all
select 58340,'UNKS' from dual union all
select 58341,'UNKS' from dual union all
select 58342,'UNKS' from dual union all
select 58343,'UNKS' from dual union all
select 58344,'UNKS' from dual union all
select 58345,'MRN' from dual union all
select 58346,'MRN' from dual union all
select 58347,'DFVLR' from dual union all
select 58348,'RN' from dual union all
select 58349,'NASA' from dual union all
select 5835,'USA' from dual;
insert into LAUNCH_AGENCY
select 58350,'MRN' from dual union all
select 58351,'PLA2AC' from dual union all
select 58352,'MRN' from dual union all
select 58353,'AFTAC' from dual union all
select 58354,'JMA' from dual union all
select 58355,'MRN' from dual union all
select 58356,'NASA' from dual union all
select 58357,'MRN' from dual union all
select 58358,'NASA' from dual union all
select 58359,'IAE' from dual union all
select 5836,'SAND' from dual union all
select 58360,'MRN' from dual union all
select 58361,'NASA' from dual union all
select 58362,'NASA' from dual union all
select 58363,'SAND' from dual union all
select 58364,'RVSN' from dual union all
select 58365,'UNKS' from dual union all
select 58366,'AFSC' from dual union all
select 58367,'AE' from dual union all
select 58368,'UNKS' from dual union all
select 58369,'NASA' from dual union all
select 5837,'ISAS' from dual union all
select 58370,'ISRO' from dual union all
select 58371,'ISRO' from dual union all
select 58372,'ISRO' from dual union all
select 58373,'ISRO' from dual union all
select 58374,'ISRO' from dual union all
select 58375,'ISRO' from dual union all
select 58376,'ISRO' from dual union all
select 58377,'ISRO' from dual union all
select 58378,'ISRO' from dual union all
select 58379,'ISRO' from dual union all
select 5838,'BTL' from dual union all
select 58380,'ISRO' from dual union all
select 58381,'ISRO' from dual union all
select 58382,'ISRO' from dual union all
select 58383,'ISRO' from dual union all
select 58384,'ISRO' from dual union all
select 58385,'ISRO' from dual union all
select 58386,'ISRO' from dual union all
select 58387,'ISRO' from dual union all
select 58388,'ISRO' from dual union all
select 58389,'ISRO' from dual union all
select 5839,'BTL' from dual union all
select 58390,'ISRO' from dual union all
select 58391,'ISRO' from dual union all
select 58392,'ISRO' from dual union all
select 58393,'ISRO' from dual union all
select 58394,'ISRO' from dual union all
select 58395,'ISRO' from dual union all
select 58396,'ISRO' from dual union all
select 58397,'ISRO' from dual union all
select 58398,'ISRO' from dual union all
select 58399,'ISRO' from dual union all
select 584,'WEHR' from dual union all
select 5840,'SMSA' from dual union all
select 58400,'ISRO' from dual union all
select 58401,'ISRO' from dual union all
select 58402,'ISRO' from dual union all
select 58403,'ISRO' from dual union all
select 58404,'ISRO' from dual union all
select 58405,'ISRO' from dual union all
select 58406,'ISRO' from dual union all
select 58407,'ISRO' from dual union all
select 58408,'ISRO' from dual union all
select 58409,'ISRO' from dual union all
select 5841,'NASA' from dual union all
select 58410,'ISRO' from dual union all
select 58411,'ISRO' from dual union all
select 58412,'ISRO' from dual union all
select 58413,'ISRO' from dual union all
select 58414,'ISRO' from dual union all
select 58415,'ISRO' from dual union all
select 58416,'ISRO' from dual union all
select 58417,'ISRO' from dual union all
select 58418,'ISRO' from dual union all
select 58419,'NASA' from dual union all
select 5842,'WRE' from dual union all
select 58420,'NASA' from dual union all
select 58421,'NASA' from dual union all
select 58422,'NASA' from dual union all
select 58423,'NASA' from dual union all
select 58424,'NASA' from dual union all
select 58425,'NASA' from dual union all
select 58426,'NASA' from dual union all
select 58427,'NASA' from dual union all
select 58428,'RVSN' from dual union all
select 58429,'RVSN' from dual union all
select 5843,'MVS' from dual union all
select 58430,'RVSN' from dual union all
select 58431,'RVSN' from dual union all
select 58432,'RVSN' from dual union all
select 58433,'RVSN' from dual union all
select 58434,'RVSN' from dual union all
select 58435,'RVSN' from dual union all
select 58436,'RVSN' from dual union all
select 58437,'RVSN' from dual union all
select 58438,'MITLL' from dual union all
select 58438,'SDIO' from dual union all
select 58439,'SAC' from dual;
insert into LAUNCH_AGENCY
select 5844,'USAF' from dual union all
select 58440,'AN' from dual union all
select 58441,'AN' from dual union all
select 58442,'USN' from dual union all
select 58443,'AFSC' from dual union all
select 58444,'AN' from dual union all
select 58445,'AN' from dual union all
select 58446,'AN' from dual union all
select 58447,'AN' from dual union all
select 58448,'RVSN' from dual union all
select 58449,'USN' from dual union all
select 5845,'USA' from dual union all
select 58450,'USN' from dual union all
select 58451,'NASA' from dual union all
select 58452,'NASA' from dual union all
select 58453,'NASA' from dual union all
select 58454,'NASA' from dual union all
select 58455,'NASA' from dual union all
select 58456,'NASA' from dual union all
select 58457,'USAF' from dual union all
select 58458,'ISAS' from dual union all
select 58459,'RVSN' from dual union all
select 5846,'ABMA' from dual union all
select 5846,'USAF' from dual union all
select 58460,'NASA' from dual union all
select 58461,'SAC' from dual union all
select 58462,'MITLL' from dual union all
select 58462,'SDIO' from dual union all
select 58463,'AN' from dual union all
select 58464,'AN' from dual union all
select 58465,'AN' from dual union all
select 58466,'AN' from dual union all
select 58467,'AN' from dual union all
select 58468,'MITLL' from dual union all
select 58468,'SDIO' from dual union all
select 58469,'NASA' from dual union all
select 5847,'ONERA' from dual union all
select 58470,'RVSN' from dual union all
select 58471,'NASA' from dual union all
select 58472,'ISRO' from dual union all
select 58473,'AN' from dual union all
select 58474,'AN' from dual union all
select 58475,'AN' from dual union all
select 58476,'AN' from dual union all
select 58477,'NASA' from dual union all
select 58477,'NDRE' from dual union all
select 58478,'AFSC' from dual union all
select 58479,'IAE' from dual union all
select 5848,'SAND' from dual union all
select 58480,'NASA' from dual union all
select 58481,'RVSN' from dual union all
select 58482,'USN' from dual union all
select 58483,'IAE' from dual union all
select 58484,'NASA' from dual union all
select 58485,'US7A' from dual union all
select 58486,'NASA' from dual union all
select 58487,'US7A' from dual union all
select 58488,'US7A' from dual union all
select 58489,'RVSN' from dual union all
select 5849,'USAF' from dual union all
select 58490,'MITLL' from dual union all
select 58490,'SDIO' from dual union all
select 58491,'INTA' from dual union all
select 58492,'RVSN' from dual union all
select 58493,'INTA' from dual union all
select 58494,'INTA' from dual union all
select 58495,'RVSN' from dual union all
select 58496,'IR' from dual union all
select 58497,'IR' from dual union all
select 58498,'IR' from dual union all
select 58499,'IR' from dual union all
select 585,'WEHR' from dual union all
select 5850,'LARCN' from dual union all
select 58500,'IR' from dual union all
select 58501,'IR' from dual union all
select 58502,'IR' from dual union all
select 58503,'RVSN' from dual union all
select 58504,'IR' from dual union all
select 58505,'IQ223' from dual union all
select 58506,'PLA2AC' from dual union all
select 58507,'IQ224' from dual union all
select 58508,'IQ224' from dual union all
select 58509,'IQ224' from dual union all
select 5851,'RAE' from dual union all
select 58510,'IQ224' from dual union all
select 58511,'IQ224' from dual union all
select 58512,'RVSN' from dual union all
select 58513,'RVSN' from dual union all
select 58514,'SAND' from dual union all
select 58515,'RVSN' from dual union all
select 58516,'RVSN' from dual union all
select 58517,'RVSN' from dual union all
select 58518,'RVSN' from dual union all
select 58519,'RVSN' from dual union all
select 5852,'USAF' from dual union all
select 58520,'RVSN' from dual union all
select 58521,'RVSN' from dual union all
select 58522,'RVSN' from dual union all
select 58523,'NASA' from dual union all
select 58524,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 58525,'RVSN' from dual union all
select 58526,'RVSN' from dual union all
select 58527,'RVSN' from dual union all
select 58528,'RVSN' from dual union all
select 58529,'RVSN' from dual union all
select 5853,'USAF' from dual union all
select 58530,'RVSN' from dual union all
select 58531,'RVSN' from dual union all
select 58532,'RVSN' from dual union all
select 58533,'RVSN' from dual union all
select 58534,'NASA' from dual union all
select 58535,'RVSN' from dual union all
select 58536,'RVSN' from dual union all
select 58537,'RVSN' from dual union all
select 58538,'RVSN' from dual union all
select 58539,'RVSN' from dual union all
select 5854,'MVS' from dual union all
select 58540,'RVSN' from dual union all
select 58541,'RVSN' from dual union all
select 58542,'RVSN' from dual union all
select 58543,'RVSN' from dual union all
select 58544,'RVSN' from dual union all
select 58545,'NASA' from dual union all
select 58546,'RVSN' from dual union all
select 58547,'INTA' from dual union all
select 58548,'INTA' from dual union all
select 58549,'PLA2AC' from dual union all
select 5855,'AFSWC' from dual union all
select 58550,'RVSN' from dual union all
select 58551,'RVSN' from dual union all
select 58552,'NASA' from dual union all
select 58553,'RVSN' from dual union all
select 58554,'RVSN' from dual union all
select 58555,'RVSN' from dual union all
select 58556,'RVSN' from dual union all
select 58557,'RVSN' from dual union all
select 58558,'RVSN' from dual union all
select 58559,'RVSN' from dual union all
select 5856,'RAE' from dual union all
select 5856,'WRE' from dual union all
select 58560,'RVSN' from dual union all
select 58561,'RVSN' from dual union all
select 58562,'RVSN' from dual union all
select 58563,'DFVLR' from dual union all
select 58564,'RVSN' from dual union all
select 58565,'RVSN' from dual union all
select 58566,'RVSN' from dual union all
select 58567,'RVSN' from dual union all
select 58568,'RVSN' from dual union all
select 58569,'RVSN' from dual union all
select 5857,'ABMA' from dual union all
select 5857,'USAF' from dual union all
select 58570,'AN' from dual union all
select 58571,'AN' from dual union all
select 58572,'AN' from dual union all
select 58573,'AN' from dual union all
select 58574,'AN' from dual union all
select 58575,'AN' from dual union all
select 58576,'AN' from dual union all
select 58577,'AFSC' from dual union all
select 58578,'SAC' from dual union all
select 58579,'ISAS' from dual union all
select 5858,'MVS' from dual union all
select 58580,'NASA' from dual union all
select 58581,'NASA' from dual union all
select 58582,'NASA' from dual union all
select 58583,'NASA' from dual union all
select 58584,'ISAS' from dual union all
select 58585,'NRCC' from dual union all
select 58586,'NRCC' from dual union all
select 58587,'DLR' from dual union all
select 58588,'SAC' from dual union all
select 58589,'DLR' from dual union all
select 5859,'MVS' from dual union all
select 58590,'RVSN' from dual union all
select 58591,'AN' from dual union all
select 58592,'AN' from dual union all
select 58593,'AN' from dual union all
select 58594,'AN' from dual union all
select 58595,'NASA' from dual union all
select 58596,'USN' from dual union all
select 58597,'USN' from dual union all
select 58598,'USN' from dual union all
select 58599,'USN' from dual union all
select 586,'WEHR' from dual union all
select 5860,'USN' from dual union all
select 58600,'NASA' from dual union all
select 58601,'AFGL2' from dual union all
select 58602,'AN' from dual union all
select 58603,'NASA' from dual union all
select 58604,'NASA' from dual union all
select 58605,'NASA' from dual union all
select 58606,'NASA' from dual union all
select 58607,'NASA' from dual union all
select 58608,'NASA' from dual union all
select 58609,'AN' from dual union all
select 5861,'SAND' from dual union all
select 58610,'AN' from dual union all
select 58611,'NASA' from dual union all
select 58612,'NASA' from dual;
insert into LAUNCH_AGENCY
select 58613,'UNKS' from dual union all
select 58614,'NASA' from dual union all
select 58615,'DFVLR' from dual union all
select 58616,'PLA2AC' from dual union all
select 58617,'DFVLR' from dual union all
select 58618,'NRCC' from dual union all
select 58619,'DFVLR' from dual union all
select 5862,'SAND' from dual union all
select 58620,'RN' from dual union all
select 58621,'AFSC' from dual union all
select 58622,'RVSN' from dual union all
select 58623,'NIPR' from dual union all
select 58624,'AFSC' from dual union all
select 58625,'RN' from dual union all
select 58626,'USN' from dual union all
select 58627,'USN' from dual union all
select 58628,'USN' from dual union all
select 58629,'USN' from dual union all
select 5863,'SAND' from dual union all
select 58630,'AFGL' from dual union all
select 58631,'NASA' from dual union all
select 58632,'ALCS' from dual union all
select 58632,'SAC' from dual union all
select 58633,'SAC' from dual union all
select 58634,'AFGL2' from dual union all
select 58635,'USN' from dual union all
select 58636,'USN' from dual union all
select 58637,'USN' from dual union all
select 58638,'USN' from dual union all
select 58639,'RVSN' from dual union all
select 5864,'MVS' from dual union all
select 58640,'NASA' from dual union all
select 58641,'NASA' from dual union all
select 58642,'USN' from dual union all
select 58643,'NIPR' from dual union all
select 58644,'MRN' from dual union all
select 58645,'MRN' from dual union all
select 58646,'MRN' from dual union all
select 58647,'MRN' from dual union all
select 58648,'MRN' from dual union all
select 58649,'JMA' from dual union all
select 5865,'AN' from dual union all
select 58650,'MRN' from dual union all
select 58651,'MRN' from dual union all
select 58652,'MRN' from dual union all
select 58653,'MRN' from dual union all
select 58654,'MRN' from dual union all
select 58655,'MRN' from dual union all
select 58656,'MRN' from dual union all
select 58657,'MRN' from dual union all
select 58658,'MRN' from dual union all
select 58659,'MRN' from dual union all
select 5866,'AFSWC' from dual union all
select 58660,'MRN' from dual union all
select 58661,'MRN' from dual union all
select 58662,'MRN' from dual union all
select 58663,'MRN' from dual union all
select 58664,'MRN' from dual union all
select 58665,'JMA' from dual union all
select 58666,'MRN' from dual union all
select 58667,'MRN' from dual union all
select 58668,'MRN' from dual union all
select 58669,'MRN' from dual union all
select 5867,'MVS' from dual union all
select 58670,'MRN' from dual union all
select 58671,'MRN' from dual union all
select 58672,'MRN' from dual union all
select 58673,'MRN' from dual union all
select 58674,'MRN' from dual union all
select 58675,'MRN' from dual union all
select 58676,'MRN' from dual union all
select 58677,'MRN' from dual union all
select 58678,'MRN' from dual union all
select 58679,'MRN' from dual union all
select 5868,'MVS' from dual union all
select 58680,'MRN' from dual union all
select 58681,'JMA' from dual union all
select 58682,'MRN' from dual union all
select 58683,'MRN' from dual union all
select 58684,'MRN' from dual union all
select 58685,'MRN' from dual union all
select 58686,'JMA' from dual union all
select 58687,'MRN' from dual union all
select 58688,'MRN' from dual union all
select 58689,'MRN' from dual union all
select 5869,'USAF' from dual union all
select 58690,'MRN' from dual union all
select 58691,'MRN' from dual union all
select 58692,'MRN' from dual union all
select 58693,'MRN' from dual union all
select 58694,'MRN' from dual union all
select 58695,'MRN' from dual union all
select 58696,'MRN' from dual union all
select 58697,'MRN' from dual union all
select 58698,'MRN' from dual union all
select 58699,'MRN' from dual union all
select 587,'WEHR' from dual union all
select 5870,'NRL' from dual union all
select 58700,'MRN' from dual union all
select 58701,'MRN' from dual;
insert into LAUNCH_AGENCY
select 58702,'MRN' from dual union all
select 58703,'MRN' from dual union all
select 58704,'MRN' from dual union all
select 58705,'MRN' from dual union all
select 58706,'MRN' from dual union all
select 58707,'MRN' from dual union all
select 58708,'MRN' from dual union all
select 58709,'MRN' from dual union all
select 5871,'MVS' from dual union all
select 58710,'JMA' from dual union all
select 58711,'MRN' from dual union all
select 58712,'JMA' from dual union all
select 58713,'MRN' from dual union all
select 58714,'MRN' from dual union all
select 58715,'MRN' from dual union all
select 58716,'MRN' from dual union all
select 58717,'MRN' from dual union all
select 58718,'MRN' from dual union all
select 58719,'MRN' from dual union all
select 5872,'USAF' from dual union all
select 58720,'MRN' from dual union all
select 58721,'MRN' from dual union all
select 58722,'MRN' from dual union all
select 58723,'JMA' from dual union all
select 58724,'MRN' from dual union all
select 58725,'MRN' from dual union all
select 58726,'MRN' from dual union all
select 58727,'MRN' from dual union all
select 58728,'MRN' from dual union all
select 58729,'MRN' from dual union all
select 5873,'MVS' from dual union all
select 58730,'MRN' from dual union all
select 58731,'JMA' from dual union all
select 58732,'MRN' from dual union all
select 58733,'MRN' from dual union all
select 58734,'MRN' from dual union all
select 58735,'MRN' from dual union all
select 58736,'MRN' from dual union all
select 58737,'MRN' from dual union all
select 58738,'MRN' from dual union all
select 58739,'JMA' from dual union all
select 5874,'USA' from dual union all
select 58740,'MRN' from dual union all
select 58741,'MRN' from dual union all
select 58742,'MRN' from dual union all
select 58743,'MRN' from dual union all
select 58744,'MRN' from dual union all
select 58745,'MRN' from dual union all
select 58746,'MRN' from dual union all
select 58747,'MRN' from dual union all
select 58748,'MRN' from dual union all
select 58749,'MRN' from dual union all
select 5875,'RAE' from dual union all
select 5875,'WRE' from dual union all
select 58750,'MRN' from dual union all
select 58751,'MRN' from dual union all
select 58752,'MRN' from dual union all
select 58753,'JMA' from dual union all
select 58754,'MRN' from dual union all
select 58755,'MRN' from dual union all
select 58756,'MRN' from dual union all
select 58757,'MRN' from dual union all
select 58758,'MRN' from dual union all
select 58759,'MRN' from dual union all
select 5876,'MVS' from dual union all
select 58760,'MRN' from dual union all
select 58761,'JMA' from dual union all
select 58762,'MRN' from dual union all
select 58763,'MRN' from dual union all
select 58764,'AN' from dual union all
select 58765,'AN' from dual union all
select 58766,'MRN' from dual union all
select 58767,'MRN' from dual union all
select 58768,'AN' from dual union all
select 58769,'MRN' from dual union all
select 5877,'USAF' from dual union all
select 58770,'AN' from dual union all
select 58771,'MRN' from dual union all
select 58772,'MRN' from dual union all
select 58773,'JMA' from dual union all
select 58774,'MRN' from dual union all
select 58775,'MRN' from dual union all
select 58776,'MRN' from dual union all
select 58777,'MRN' from dual union all
select 58778,'MRN' from dual union all
select 58779,'MRN' from dual union all
select 5878,'USN' from dual union all
select 58780,'MRN' from dual union all
select 58781,'MRN' from dual union all
select 58782,'MRN' from dual union all
select 58783,'AN' from dual union all
select 58784,'JMA' from dual union all
select 58785,'MRN' from dual union all
select 58786,'NASA' from dual union all
select 58787,'GMS' from dual union all
select 58788,'GMS' from dual union all
select 58789,'GMS' from dual union all
select 5879,'MVS' from dual union all
select 58790,'GMS' from dual union all
select 58791,'GMS' from dual;
insert into LAUNCH_AGENCY
select 58792,'GMS' from dual union all
select 58793,'NASA' from dual union all
select 58794,'MRN' from dual union all
select 58795,'MRN' from dual union all
select 58796,'JMA' from dual union all
select 58797,'MRN' from dual union all
select 58798,'MRN' from dual union all
select 58799,'AN' from dual union all
select 588,'WEHR' from dual union all
select 5880,'MVS' from dual union all
select 58800,'MRN' from dual union all
select 58801,'AN' from dual union all
select 58802,'MRN' from dual union all
select 58803,'NASA' from dual union all
select 58804,'NASA' from dual union all
select 58805,'JMA' from dual union all
select 58806,'MRN' from dual union all
select 58807,'NASA' from dual union all
select 58808,'MRN' from dual union all
select 58809,'MRN' from dual union all
select 5881,'MVS' from dual union all
select 58810,'MRN' from dual union all
select 58811,'MRN' from dual union all
select 58812,'MRN' from dual union all
select 58813,'GMS' from dual union all
select 58814,'MRN' from dual union all
select 58815,'JMA' from dual union all
select 58816,'NASA' from dual union all
select 58817,'MRN' from dual union all
select 58818,'NASA' from dual union all
select 58819,'NASA' from dual union all
select 5882,'USAF' from dual union all
select 58820,'NASA' from dual union all
select 58821,'MRN' from dual union all
select 58822,'MRN' from dual union all
select 58823,'MRN' from dual union all
select 58824,'MRN' from dual union all
select 58825,'JMA' from dual union all
select 58826,'AN' from dual union all
select 58827,'AN' from dual union all
select 58828,'MRN' from dual union all
select 58829,'MRN' from dual union all
select 5883,'USAF' from dual union all
select 58830,'MRN' from dual union all
select 58831,'MRN' from dual union all
select 58832,'NASA' from dual union all
select 58833,'MRN' from dual union all
select 58834,'JMA' from dual union all
select 58835,'AN' from dual union all
select 58836,'MRN' from dual union all
select 58837,'NASA' from dual union all
select 58838,'AN' from dual union all
select 58839,'MRN' from dual union all
select 5884,'MVS' from dual union all
select 58840,'AN' from dual union all
select 58841,'MRN' from dual union all
select 58842,'GMS' from dual union all
select 58843,'MRN' from dual union all
select 58844,'NASA' from dual union all
select 58845,'GMS' from dual union all
select 58846,'GMS' from dual union all
select 58847,'AN' from dual union all
select 58848,'MRN' from dual union all
select 58849,'MRN' from dual union all
select 5885,'MVS' from dual union all
select 58850,'MRN' from dual union all
select 58851,'MRN' from dual union all
select 58852,'MRN' from dual union all
select 58853,'JMA' from dual union all
select 58854,'MRN' from dual union all
select 58855,'GMS' from dual union all
select 58856,'AN' from dual union all
select 58857,'GMS' from dual union all
select 58858,'GMS' from dual union all
select 58859,'MRN' from dual union all
select 5886,'RAF' from dual union all
select 58860,'MRN' from dual union all
select 58861,'MRN' from dual union all
select 58862,'MRN' from dual union all
select 58863,'MRN' from dual union all
select 58864,'MRN' from dual union all
select 58865,'MRN' from dual union all
select 58866,'NASA' from dual union all
select 58867,'JMA' from dual union all
select 58868,'GMS' from dual union all
select 58869,'NASA' from dual union all
select 5887,'USAF' from dual union all
select 58870,'JMA' from dual union all
select 58871,'GMS' from dual union all
select 58872,'MRN' from dual union all
select 58873,'NASA' from dual union all
select 58874,'GMS' from dual union all
select 58875,'MRN' from dual union all
select 58876,'MRN' from dual union all
select 58877,'MRN' from dual union all
select 58878,'MRN' from dual union all
select 58879,'AN' from dual union all
select 5888,'USA' from dual union all
select 58880,'MRN' from dual union all
select 58881,'AN' from dual;
insert into LAUNCH_AGENCY
select 58882,'AN' from dual union all
select 58883,'MRN' from dual union all
select 58884,'AN' from dual union all
select 58885,'AN' from dual union all
select 58886,'MRN' from dual union all
select 58887,'MRN' from dual union all
select 58888,'MRN' from dual union all
select 58889,'MRN' from dual union all
select 5889,'NASA' from dual union all
select 58890,'MRN' from dual union all
select 58891,'MRN' from dual union all
select 58892,'MRN' from dual union all
select 58893,'MRN' from dual union all
select 58894,'MRN' from dual union all
select 58895,'MRN' from dual union all
select 58896,'JMA' from dual union all
select 58897,'NASA' from dual union all
select 58898,'MRN' from dual union all
select 58899,'NASA' from dual union all
select 589,'WEHR' from dual union all
select 5890,'MVS' from dual union all
select 58900,'MRN' from dual union all
select 58901,'NASA' from dual union all
select 58902,'NASA' from dual union all
select 58903,'NASA' from dual union all
select 58904,'NASA' from dual union all
select 58905,'NASA' from dual union all
select 58906,'JMA' from dual union all
select 58907,'MRN' from dual union all
select 58908,'NASA' from dual union all
select 58909,'MRN' from dual union all
select 5891,'USN' from dual union all
select 58910,'MRN' from dual union all
select 58911,'MRN' from dual union all
select 58912,'NASA' from dual union all
select 58913,'MRN' from dual union all
select 58914,'NASA' from dual union all
select 58915,'NASA' from dual union all
select 58916,'MRN' from dual union all
select 58917,'MRN' from dual union all
select 58918,'JMA' from dual union all
select 58919,'NASA' from dual union all
select 5892,'USAF' from dual union all
select 58920,'MRN' from dual union all
select 58921,'NASA' from dual union all
select 58922,'MRN' from dual union all
select 58923,'AN' from dual union all
select 58924,'JMA' from dual union all
select 58925,'NASA' from dual union all
select 58926,'JMA' from dual union all
select 58927,'MRN' from dual union all
select 58928,'JMA' from dual union all
select 58929,'JMA' from dual union all
select 5893,'MVS' from dual union all
select 58930,'JMA' from dual union all
select 58931,'JMA' from dual union all
select 58932,'NIPR' from dual union all
select 58933,'NIPR' from dual union all
select 58934,'NIPR' from dual union all
select 58935,'NIPR' from dual union all
select 58936,'NIPR' from dual union all
select 58937,'NIPR' from dual union all
select 58938,'MRN' from dual union all
select 58939,'NIPR' from dual union all
select 5894,'USAF' from dual union all
select 58940,'NIPR' from dual union all
select 58941,'NIPR' from dual union all
select 58942,'NIPR' from dual union all
select 58943,'NIPR' from dual union all
select 58944,'ISAS' from dual union all
select 58945,'MRN' from dual union all
select 58946,'MRN' from dual union all
select 58947,'MRN' from dual union all
select 58948,'MRN' from dual union all
select 58949,'MRN' from dual union all
select 5895,'USN' from dual union all
select 58950,'MRN' from dual union all
select 58951,'JMA' from dual union all
select 58952,'MRN' from dual union all
select 58953,'MRN' from dual union all
select 58954,'MRN' from dual union all
select 58955,'MRN' from dual union all
select 58956,'MRN' from dual union all
select 58957,'MRN' from dual union all
select 58958,'MRN' from dual union all
select 58959,'MRN' from dual union all
select 5896,'USAF' from dual union all
select 58960,'MRN' from dual union all
select 58961,'MRN' from dual union all
select 58962,'JMA' from dual union all
select 58963,'MRN' from dual union all
select 58964,'MRN' from dual union all
select 58965,'MRN' from dual union all
select 58966,'MRN' from dual union all
select 58967,'MRN' from dual union all
select 58968,'MRN' from dual union all
select 58969,'MRN' from dual union all
select 5897,'USN' from dual union all
select 58970,'MRN' from dual union all
select 58971,'MRN' from dual;
insert into LAUNCH_AGENCY
select 58972,'MRN' from dual union all
select 58973,'MRN' from dual union all
select 58974,'JMA' from dual union all
select 58975,'MRN' from dual union all
select 58976,'MRN' from dual union all
select 58977,'JMA' from dual union all
select 58978,'MRN' from dual union all
select 58979,'MRN' from dual union all
select 5898,'NASA' from dual union all
select 58980,'MRN' from dual union all
select 58981,'MRN' from dual union all
select 58982,'MRN' from dual union all
select 58983,'MRN' from dual union all
select 58984,'MRN' from dual union all
select 58985,'MRN' from dual union all
select 58986,'MRN' from dual union all
select 58987,'MRN' from dual union all
select 58988,'JMA' from dual union all
select 58989,'MRN' from dual union all
select 5899,'RAE' from dual union all
select 5899,'WRE' from dual union all
select 58990,'MRN' from dual union all
select 58991,'MRN' from dual union all
select 58992,'MRN' from dual union all
select 58993,'MRN' from dual union all
select 58994,'MRN' from dual union all
select 58995,'MRN' from dual union all
select 58996,'MRN' from dual union all
select 58997,'JMA' from dual union all
select 58998,'MRN' from dual union all
select 58999,'MRN' from dual union all
select 59,'WER271-1' from dual union all
select 590,'WEHR' from dual union all
select 5900,'USAF' from dual union all
select 59000,'NASA' from dual union all
select 59001,'NASA' from dual union all
select 59002,'NASA' from dual union all
select 59003,'MRN' from dual union all
select 59004,'MRN' from dual union all
select 59005,'MRN' from dual union all
select 59006,'MRN' from dual union all
select 59007,'MRN' from dual union all
select 59008,'MRN' from dual union all
select 59009,'MRN' from dual union all
select 5901,'USN' from dual union all
select 59010,'MRN' from dual union all
select 59011,'MRN' from dual union all
select 59012,'MRN' from dual union all
select 59013,'NASA' from dual union all
select 59014,'NASA' from dual union all
select 59015,'MRN' from dual union all
select 59016,'MRN' from dual union all
select 59017,'MRN' from dual union all
select 59018,'AN' from dual union all
select 59019,'AN' from dual union all
select 5902,'USAF' from dual union all
select 59020,'AN' from dual union all
select 59021,'AN' from dual union all
select 59022,'AN' from dual union all
select 59023,'AN' from dual union all
select 59024,'MRN' from dual union all
select 59025,'AN' from dual union all
select 59026,'AN' from dual union all
select 59027,'AN' from dual union all
select 59028,'NASA' from dual union all
select 59029,'AN' from dual union all
select 5903,'USN' from dual union all
select 59030,'MRN' from dual union all
select 59031,'AN' from dual union all
select 59032,'AN' from dual union all
select 59033,'AN' from dual union all
select 59034,'AN' from dual union all
select 59035,'AN' from dual union all
select 59036,'AN' from dual union all
select 59037,'AN' from dual union all
select 59038,'AN' from dual union all
select 59039,'MRN' from dual union all
select 5904,'MVS' from dual union all
select 59040,'AN' from dual union all
select 59041,'AN' from dual union all
select 59042,'AN' from dual union all
select 59043,'AN' from dual union all
select 59044,'AN' from dual union all
select 59045,'AN' from dual union all
select 59046,'AN' from dual union all
select 59047,'AN' from dual union all
select 59048,'AN' from dual union all
select 59049,'AN' from dual union all
select 5905,'USAF' from dual union all
select 59050,'MRN' from dual union all
select 59051,'AN' from dual union all
select 59052,'AN' from dual union all
select 59053,'AN' from dual union all
select 59054,'AN' from dual union all
select 59055,'AN' from dual union all
select 59056,'AN' from dual union all
select 59057,'AN' from dual union all
select 59058,'AN' from dual union all
select 59059,'AN' from dual union all
select 5906,'SAND' from dual;
insert into LAUNCH_AGENCY
select 59060,'AN' from dual union all
select 59061,'MRN' from dual union all
select 59062,'AN' from dual union all
select 59063,'AN' from dual union all
select 59064,'AN' from dual union all
select 59065,'AN' from dual union all
select 59066,'AN' from dual union all
select 59067,'AN' from dual union all
select 59068,'AN' from dual union all
select 59069,'AN' from dual union all
select 5907,'USN' from dual union all
select 59070,'AN' from dual union all
select 59071,'AN' from dual union all
select 59072,'MRN' from dual union all
select 59073,'AN' from dual union all
select 59074,'AN' from dual union all
select 59075,'NASA' from dual union all
select 59076,'AN' from dual union all
select 59077,'NASA' from dual union all
select 59078,'NASA' from dual union all
select 59079,'AN' from dual union all
select 5908,'USAF' from dual union all
select 59080,'AN' from dual union all
select 59081,'AN' from dual union all
select 59082,'AN' from dual union all
select 59083,'MRN' from dual union all
select 59084,'AN' from dual union all
select 59085,'AN' from dual union all
select 59086,'AN' from dual union all
select 59087,'AN' from dual union all
select 59088,'AN' from dual union all
select 59089,'AN' from dual union all
select 5909,'ABMA' from dual union all
select 5909,'USAF' from dual union all
select 59090,'AN' from dual union all
select 59091,'AN' from dual union all
select 59092,'AN' from dual union all
select 59093,'AN' from dual union all
select 59094,'MRN' from dual union all
select 59095,'AN' from dual union all
select 59096,'AN' from dual union all
select 59097,'AN' from dual union all
select 59098,'AN' from dual union all
select 59099,'AN' from dual union all
select 591,'WEHR' from dual union all
select 5910,'MVS' from dual union all
select 59100,'AN' from dual union all
select 59101,'AN' from dual union all
select 59102,'AN' from dual union all
select 59103,'AN' from dual union all
select 59104,'AN' from dual union all
select 59105,'MRN' from dual union all
select 59106,'AN' from dual union all
select 59107,'AN' from dual union all
select 59108,'AN' from dual union all
select 59109,'AN' from dual union all
select 5911,'USN' from dual union all
select 59110,'AN' from dual union all
select 59111,'AN' from dual union all
select 59112,'AN' from dual union all
select 59113,'MRN' from dual union all
select 59114,'MRN' from dual union all
select 59115,'MRN' from dual union all
select 59116,'MRN' from dual union all
select 59117,'AN' from dual union all
select 59118,'AN' from dual union all
select 59119,'AN' from dual union all
select 5912,'USAF' from dual union all
select 59120,'AN' from dual union all
select 59121,'AN' from dual union all
select 59122,'AN' from dual union all
select 59123,'AN' from dual union all
select 59124,'AN' from dual union all
select 59125,'AN' from dual union all
select 59126,'MRN' from dual union all
select 59127,'AN' from dual union all
select 59128,'AN' from dual union all
select 59129,'AN' from dual union all
select 5913,'USN' from dual union all
select 59130,'AN' from dual union all
select 59131,'AN' from dual union all
select 59132,'AN' from dual union all
select 59133,'AN' from dual union all
select 59134,'AN' from dual union all
select 59135,'AN' from dual union all
select 59136,'AN' from dual union all
select 59137,'AN' from dual union all
select 59138,'AN' from dual union all
select 59139,'MRN' from dual union all
select 5914,'USN' from dual union all
select 59140,'AN' from dual union all
select 59141,'AN' from dual union all
select 59142,'AN' from dual union all
select 59143,'AN' from dual union all
select 59144,'AN' from dual union all
select 59145,'AN' from dual union all
select 59146,'AN' from dual union all
select 59147,'AN' from dual union all
select 59148,'AN' from dual union all
select 59149,'AN' from dual;
insert into LAUNCH_AGENCY
select 5915,'USN' from dual union all
select 59150,'AN' from dual union all
select 59151,'AN' from dual union all
select 59152,'AN' from dual union all
select 59153,'AN' from dual union all
select 59154,'AN' from dual union all
select 59155,'AN' from dual union all
select 59156,'AN' from dual union all
select 59157,'AN' from dual union all
select 59158,'AN' from dual union all
select 59159,'AN' from dual union all
select 5916,'USAF' from dual union all
select 59160,'AN' from dual union all
select 59161,'AN' from dual union all
select 59162,'MRN' from dual union all
select 59163,'MRN' from dual union all
select 59164,'GMS' from dual union all
select 59165,'AN' from dual union all
select 59166,'AN' from dual union all
select 59167,'AN' from dual union all
select 59168,'AN' from dual union all
select 59169,'AN' from dual union all
select 5917,'USN' from dual union all
select 59170,'AN' from dual union all
select 59171,'AN' from dual union all
select 59172,'AN' from dual union all
select 59173,'AN' from dual union all
select 59174,'AN' from dual union all
select 59175,'AN' from dual union all
select 59176,'AN' from dual union all
select 59177,'AN' from dual union all
select 59178,'AN' from dual union all
select 59179,'AN' from dual union all
select 5918,'MVS' from dual union all
select 59180,'AN' from dual union all
select 59181,'AN' from dual union all
select 59182,'AN' from dual union all
select 59183,'MRN' from dual union all
select 59184,'AN' from dual union all
select 59185,'MRN' from dual union all
select 59186,'MRN' from dual union all
select 59187,'MRN' from dual union all
select 59188,'AN' from dual union all
select 59189,'AN' from dual union all
select 5919,'CARDE' from dual union all
select 59190,'AN' from dual union all
select 59191,'AN' from dual union all
select 59192,'AN' from dual union all
select 59193,'AN' from dual union all
select 59194,'UNKS' from dual union all
select 59195,'UNKS' from dual union all
select 59196,'NASA' from dual union all
select 59197,'UNKS' from dual union all
select 59198,'UNKS' from dual union all
select 59199,'UNKS' from dual union all
select 592,'WEHR' from dual union all
select 5920,'CARDE' from dual union all
select 59200,'UNKS' from dual union all
select 59201,'UNKS' from dual union all
select 59202,'UNKS' from dual union all
select 59203,'CALT' from dual union all
select 59204,'UNKS' from dual union all
select 59205,'UNKS' from dual union all
select 59206,'UNKS' from dual union all
select 59207,'AFSC' from dual union all
select 59208,'UNKS' from dual union all
select 59209,'NASDA' from dual union all
select 5921,'CARDE' from dual union all
select 59210,'UNKS' from dual union all
select 59211,'UNKS' from dual union all
select 59212,'AE' from dual union all
select 59213,'UNKS' from dual union all
select 59214,'UNKS' from dual union all
select 59215,'UNKS' from dual union all
select 59216,'UNKS' from dual union all
select 59217,'UNKS' from dual union all
select 59218,'UNKS' from dual union all
select 59219,'AE' from dual union all
select 5922,'USAF' from dual union all
select 59220,'UNKS' from dual union all
select 59221,'UNKS' from dual union all
select 59222,'UNKS' from dual union all
select 59223,'UNKS' from dual union all
select 59224,'UNKS' from dual union all
select 59225,'UNKS' from dual union all
select 59226,'UNKS' from dual union all
select 59227,'UNKS' from dual union all
select 59228,'UNKS' from dual union all
select 59229,'UNKS' from dual union all
select 5923,'SAC' from dual union all
select 59230,'UNKS' from dual union all
select 59231,'UNKS' from dual union all
select 59232,'UNKS' from dual union all
select 59233,'UNKS' from dual union all
select 59234,'UNKS' from dual union all
select 59235,'UNKS' from dual union all
select 59236,'UNKS' from dual union all
select 59237,'UNKS' from dual union all
select 59238,'UNKS' from dual union all
select 59239,'UNKS' from dual;
insert into LAUNCH_AGENCY
select 5924,'CARDE' from dual union all
select 59240,'UNKS' from dual union all
select 59241,'UNKS' from dual union all
select 59242,'UNKS' from dual union all
select 59243,'UNKS' from dual union all
select 59244,'UNKS' from dual union all
select 59245,'UNKS' from dual union all
select 59246,'UNKS' from dual union all
select 59247,'UNKS' from dual union all
select 59248,'UNKS' from dual union all
select 59249,'UNKS' from dual union all
select 5925,'NASA' from dual union all
select 59250,'UNKS' from dual union all
select 59251,'UNKS' from dual union all
select 59252,'UNKS' from dual union all
select 59253,'UNKS' from dual union all
select 59254,'NASDA' from dual union all
select 59255,'UNKS' from dual union all
select 59256,'UNKS' from dual union all
select 59257,'UNKS' from dual union all
select 59258,'UNKS' from dual union all
select 59259,'UNKS' from dual union all
select 5926,'USAF' from dual union all
select 59260,'UNKS' from dual union all
select 59261,'UNKS' from dual union all
select 59262,'NASA' from dual union all
select 59263,'UNKS' from dual union all
select 59264,'UNKS' from dual union all
select 59265,'UNKS' from dual union all
select 59266,'AFSC' from dual union all
select 59267,'UNKS' from dual union all
select 59268,'UNKS' from dual union all
select 59269,'CALT' from dual union all
select 5927,'USAF' from dual union all
select 59270,'UNKS' from dual union all
select 59271,'UNKS' from dual union all
select 59272,'UNKS' from dual union all
select 59273,'UNKS' from dual union all
select 59274,'UNKS' from dual union all
select 59275,'UNKS' from dual union all
select 59276,'UNKS' from dual union all
select 59277,'UNKS' from dual union all
select 59278,'UNKS' from dual union all
select 59279,'UNKS' from dual union all
select 5928,'NASA' from dual union all
select 59280,'UNKS' from dual union all
select 59281,'NASA' from dual union all
select 59282,'UNKS' from dual union all
select 59283,'UNKS' from dual union all
select 59284,'UNKS' from dual union all
select 59285,'UNKS' from dual union all
select 59286,'UNKS' from dual union all
select 59287,'UNKS' from dual union all
select 59288,'UNKS' from dual union all
select 59289,'NASA' from dual union all
select 5929,'USAF' from dual union all
select 59290,'UNKS' from dual union all
select 59291,'UNKS' from dual union all
select 59292,'UNKS' from dual union all
select 59293,'UNKS' from dual union all
select 59294,'UNKS' from dual union all
select 59295,'UNKS' from dual union all
select 59296,'UNKS' from dual union all
select 59297,'MRN' from dual union all
select 59298,'MRN' from dual union all
select 59299,'MRN' from dual union all
select 593,'WEHR' from dual union all
select 5930,'MVS' from dual union all
select 59300,'MRN' from dual union all
select 59301,'MRN' from dual union all
select 59302,'RVSN' from dual union all
select 59303,'MRN' from dual union all
select 59304,'RVSN' from dual union all
select 59305,'NASA' from dual union all
select 59306,'MRN' from dual union all
select 59307,'MRN' from dual union all
select 59308,'RVSN' from dual union all
select 59309,'SAND' from dual union all
select 5931,'ABMA' from dual union all
select 5931,'USAF' from dual union all
select 59310,'RVSN' from dual union all
select 59311,'USA' from dual union all
select 59312,'NASA' from dual union all
select 59313,'UNKS' from dual union all
select 59314,'AFSC' from dual union all
select 59315,'NASA' from dual union all
select 59316,'AE' from dual union all
select 59317,'UNKS' from dual union all
select 59318,'UNKS' from dual union all
select 59319,'ISRO' from dual union all
select 5932,'RAE' from dual union all
select 5932,'WRE' from dual union all
select 59320,'ISRO' from dual union all
select 59321,'ISRO' from dual union all
select 59322,'ISRO' from dual union all
select 59323,'ISRO' from dual union all
select 59324,'ISRO' from dual union all
select 59325,'ISRO' from dual union all
select 59326,'ISRO' from dual union all
select 59327,'ISRO' from dual;
insert into LAUNCH_AGENCY
select 59328,'ISRO' from dual union all
select 59329,'ADD' from dual union all
select 5933,'USAF' from dual union all
select 59330,'ISRO' from dual union all
select 59331,'ISRO' from dual union all
select 59332,'ISRO' from dual union all
select 59333,'ISRO' from dual union all
select 59334,'ISRO' from dual union all
select 59335,'ISRO' from dual union all
select 59336,'ISRO' from dual union all
select 59337,'ISRO' from dual union all
select 59338,'ISRO' from dual union all
select 59339,'ISRO' from dual union all
select 5934,'NASA' from dual union all
select 59340,'ISRO' from dual union all
select 59341,'ISRO' from dual union all
select 59342,'ISRO' from dual union all
select 59343,'ISRO' from dual union all
select 59344,'ISRO' from dual union all
select 59345,'ISRO' from dual union all
select 59346,'ISRO' from dual union all
select 59347,'ISRO' from dual union all
select 59348,'ISRO' from dual union all
select 59349,'ISRO' from dual union all
select 5935,'RAF' from dual union all
select 59350,'ISRO' from dual union all
select 59351,'ISRO' from dual union all
select 59352,'ISRO' from dual union all
select 59353,'ISRO' from dual union all
select 59354,'ISRO' from dual union all
select 59355,'ISRO' from dual union all
select 59356,'ISRO' from dual union all
select 59357,'ISRO' from dual union all
select 59358,'ISRO' from dual union all
select 59359,'ISRO' from dual union all
select 5936,'USA' from dual union all
select 59360,'ISRO' from dual union all
select 59361,'ISRO' from dual union all
select 59362,'ISRO' from dual union all
select 59363,'ISRO' from dual union all
select 59364,'ISRO' from dual union all
select 59365,'ISRO' from dual union all
select 59366,'ISRO' from dual union all
select 59367,'ISRO' from dual union all
select 59368,'ISRO' from dual union all
select 59369,'ISRO' from dual union all
select 5937,'MVS' from dual union all
select 59370,'ISRO' from dual union all
select 59371,'ISRO' from dual union all
select 59372,'ISRO' from dual union all
select 59373,'ISRO' from dual union all
select 59374,'ISRO' from dual union all
select 59375,'ISRO' from dual union all
select 59376,'ISRO' from dual union all
select 59377,'ISRO' from dual union all
select 59378,'ISRO' from dual union all
select 59379,'ISRO' from dual union all
select 5938,'USAF' from dual union all
select 59380,'ISRO' from dual union all
select 59381,'ISRO' from dual union all
select 59382,'ISRO' from dual union all
select 59383,'ISRO' from dual union all
select 59384,'ISRO' from dual union all
select 59385,'ISRO' from dual union all
select 59386,'ISRO' from dual union all
select 59387,'ISRO' from dual union all
select 59388,'ISRO' from dual union all
select 59389,'ISRO' from dual union all
select 5939,'USAF' from dual union all
select 59390,'ISRO' from dual union all
select 59391,'ISRO' from dual union all
select 59392,'ISRO' from dual union all
select 59393,'ISRO' from dual union all
select 59394,'ISRO' from dual union all
select 59395,'ISRO' from dual union all
select 59396,'ISRO' from dual union all
select 59397,'ISRO' from dual union all
select 59398,'ISRO' from dual union all
select 59399,'ISRO' from dual union all
select 594,'WEHR' from dual union all
select 5940,'NASA' from dual union all
select 59400,'ISRO' from dual union all
select 59401,'ISRO' from dual union all
select 59402,'ISRO' from dual union all
select 59403,'ISRO' from dual union all
select 59404,'ISRO' from dual union all
select 59405,'ISRO' from dual union all
select 59406,'ISRO' from dual union all
select 59407,'ISRO' from dual union all
select 59408,'ISRO' from dual union all
select 59409,'ISRO' from dual union all
select 5941,'USN' from dual union all
select 59410,'ISRO' from dual union all
select 59411,'ISRO' from dual union all
select 59412,'ISRO' from dual union all
select 59413,'ISRO' from dual union all
select 59414,'ISRO' from dual union all
select 59415,'ISRO' from dual union all
select 59416,'ISRO' from dual union all
select 59417,'ISRO' from dual;
insert into LAUNCH_AGENCY
select 59418,'ISRO' from dual union all
select 59419,'RVSN' from dual union all
select 5942,'USAF' from dual union all
select 59420,'RVSN' from dual union all
select 59421,'RVSN' from dual union all
select 59422,'RVSN' from dual union all
select 59423,'RVSN' from dual union all
select 59424,'RVSN' from dual union all
select 59425,'RVSN' from dual union all
select 59426,'RVSN' from dual union all
select 59427,'RVSN' from dual union all
select 59428,'RVSN' from dual union all
select 59429,'USAF' from dual union all
select 5943,'USAF' from dual union all
select 59430,'USN' from dual union all
select 59431,'NASA' from dual union all
select 59432,'NASA' from dual union all
select 59433,'IAE' from dual union all
select 59434,'AN' from dual union all
select 59435,'AN' from dual union all
select 59436,'AN' from dual union all
select 59437,'AN' from dual union all
select 59438,'USN' from dual union all
select 59439,'RVSN' from dual union all
select 5944,'RAE' from dual union all
select 5944,'WRE' from dual union all
select 59440,'USN' from dual union all
select 59441,'USN' from dual union all
select 59442,'SSC' from dual union all
select 59443,'USN' from dual union all
select 59444,'NASA' from dual union all
select 59445,'AFSC' from dual union all
select 59446,'SSC' from dual union all
select 59447,'NASA' from dual union all
select 59448,'UNKS' from dual union all
select 59449,'ISRO' from dual union all
select 5945,'MVS' from dual union all
select 59450,'RVSN' from dual union all
select 59451,'ISRO' from dual union all
select 59452,'ISRO' from dual union all
select 59453,'ISRO' from dual union all
select 59454,'LY' from dual union all
select 59455,'LY' from dual union all
select 59456,'PLA2AC' from dual union all
select 59457,'PLA2AC' from dual union all
select 59458,'FKG' from dual union all
select 59459,'FKG' from dual union all
select 5946,'MVS' from dual union all
select 59460,'FKG' from dual union all
select 59461,'RVSN' from dual union all
select 59462,'FKG' from dual union all
select 59463,'US7A' from dual union all
select 59464,'US7A' from dual union all
select 59465,'US7A' from dual union all
select 59466,'RVSN' from dual union all
select 59467,'RVSN' from dual union all
select 59468,'RVSN' from dual union all
select 59469,'ISRO' from dual union all
select 5947,'USAF' from dual union all
select 59470,'AN' from dual union all
select 59471,'AN' from dual union all
select 59472,'RVSN' from dual union all
select 59473,'AN' from dual union all
select 59474,'AN' from dual union all
select 59475,'AN' from dual union all
select 59476,'AN' from dual union all
select 59477,'AN' from dual union all
select 59478,'AN' from dual union all
select 59479,'AN' from dual union all
select 5948,'ATEF' from dual union all
select 59480,'AN' from dual union all
select 59481,'AN' from dual union all
select 59482,'AN' from dual union all
select 59483,'RVSN' from dual union all
select 59484,'AN' from dual union all
select 59485,'AN' from dual union all
select 59486,'AN' from dual union all
select 59487,'AN' from dual union all
select 59488,'AN' from dual union all
select 59489,'AN' from dual union all
select 5949,'ABMA' from dual union all
select 5949,'USAF' from dual union all
select 59490,'AN' from dual union all
select 59491,'AN' from dual union all
select 59492,'AN' from dual union all
select 59493,'AN' from dual union all
select 59494,'RVSN' from dual union all
select 59495,'AN' from dual union all
select 59496,'AN' from dual union all
select 59497,'AN' from dual union all
select 59498,'AN' from dual union all
select 59499,'AN' from dual union all
select 595,'WEHR' from dual union all
select 5950,'ATEF' from dual union all
select 59500,'RVSN' from dual union all
select 59501,'RVSN' from dual union all
select 59502,'RVSN' from dual union all
select 59503,'RVSN' from dual union all
select 59504,'RVSN' from dual union all
select 59505,'NASA' from dual;
insert into LAUNCH_AGENCY
select 59506,'RVSN' from dual union all
select 59507,'RVSN' from dual union all
select 59508,'RVSN' from dual union all
select 59509,'RVSN' from dual union all
select 5951,'USN' from dual union all
select 59510,'RVSN' from dual union all
select 59511,'RVSN' from dual union all
select 59512,'RVSN' from dual union all
select 59513,'RVSN' from dual union all
select 59514,'RVSN' from dual union all
select 59515,'RVSN' from dual union all
select 59516,'ISAS' from dual union all
select 59517,'RVSN' from dual union all
select 59518,'RVSN' from dual union all
select 59519,'RVSN' from dual union all
select 5952,'USAF' from dual union all
select 59520,'RVSN' from dual union all
select 59521,'RVSN' from dual union all
select 59522,'RVSN' from dual union all
select 59523,'RVSN' from dual union all
select 59524,'NASA' from dual union all
select 59525,'RVSN' from dual union all
select 59526,'RVSN' from dual union all
select 59527,'RVSN' from dual union all
select 59528,'RVSN' from dual union all
select 59529,'KPA' from dual union all
select 5953,'ABMA' from dual union all
select 5953,'USAF' from dual union all
select 59530,'ISAS' from dual union all
select 59531,'DMA' from dual union all
select 59532,'SAC' from dual union all
select 59533,'USN' from dual union all
select 59534,'NASA' from dual union all
select 59535,'NASA' from dual union all
select 59536,'DMA' from dual union all
select 59537,'AFSC' from dual union all
select 59538,'NASA' from dual union all
select 59539,'NASA' from dual union all
select 5954,'USAF' from dual union all
select 59540,'NASA' from dual union all
select 59541,'NASA' from dual union all
select 59542,'SAC' from dual union all
select 59543,'SAC' from dual union all
select 59544,'AFSC' from dual union all
select 59545,'NASA' from dual union all
select 59546,'NASA' from dual union all
select 59547,'AFGL2' from dual union all
select 59548,'NASA' from dual union all
select 59549,'NASA' from dual union all
select 5955,'USAF' from dual union all
select 59550,'NASA' from dual union all
select 59551,'NASA' from dual union all
select 59552,'NASA' from dual union all
select 59553,'NASA' from dual union all
select 59554,'DFVLR' from dual union all
select 59555,'DFVLR' from dual union all
select 59556,'NASA' from dual union all
select 59557,'NASA' from dual union all
select 59558,'USN' from dual union all
select 59559,'AFSC' from dual union all
select 5956,'USAF' from dual union all
select 59560,'AN' from dual union all
select 59561,'AN' from dual union all
select 59562,'USN' from dual union all
select 59563,'USN' from dual union all
select 59564,'USN' from dual union all
select 59565,'USN' from dual union all
select 59566,'SAC' from dual union all
select 59567,'US7A' from dual union all
select 59568,'US7A' from dual union all
select 59569,'US7A' from dual union all
select 5957,'USAF' from dual union all
select 59570,'US7A' from dual union all
select 59571,'US7A' from dual union all
select 59572,'AN' from dual union all
select 59573,'AN' from dual union all
select 59574,'AN' from dual union all
select 59575,'AN' from dual union all
select 59576,'AN' from dual union all
select 59577,'RN' from dual union all
select 59578,'RN' from dual union all
select 59579,'NASA' from dual union all
select 59579,'SSC' from dual union all
select 5958,'USAF' from dual union all
select 59580,'NASA' from dual union all
select 59580,'SSC' from dual union all
select 59581,'NASA' from dual union all
select 59581,'SSC' from dual union all
select 59582,'USN' from dual union all
select 59583,'USN' from dual union all
select 59584,'AN' from dual union all
select 59585,'AN' from dual union all
select 59586,'SAC' from dual union all
select 59587,'RVSN' from dual union all
select 59588,'RN' from dual union all
select 59589,'RN' from dual union all
select 5959,'USAF' from dual union all
select 59590,'USAF' from dual union all
select 59591,'AFSC' from dual union all
select 59592,'SAC' from dual;
insert into LAUNCH_AGENCY
select 59593,'AN' from dual union all
select 59594,'AN' from dual union all
select 59595,'AN' from dual union all
select 59596,'AN' from dual union all
select 59597,'VMF' from dual union all
select 59598,'USN' from dual union all
select 59599,'SDIO' from dual union all
select 596,'WEHR' from dual union all
select 5960,'USAF' from dual union all
select 59600,'ISAS' from dual union all
select 59601,'USN' from dual union all
select 59602,'USN' from dual union all
select 59603,'AFSC' from dual union all
select 59604,'SAC' from dual union all
select 59605,'AN' from dual union all
select 59606,'AN' from dual union all
select 59607,'AN' from dual union all
select 59608,'AN' from dual union all
select 59609,'AN' from dual union all
select 5961,'RAF' from dual union all
select 59610,'MRN' from dual union all
select 59611,'AN' from dual union all
select 59612,'AN' from dual union all
select 59613,'MRN' from dual union all
select 59614,'JMA' from dual union all
select 59615,'MRN' from dual union all
select 59616,'MRN' from dual union all
select 59617,'MRN' from dual union all
select 59618,'MRN' from dual union all
select 59619,'MRN' from dual union all
select 5962,'MVS' from dual union all
select 59620,'AN' from dual union all
select 59621,'MRN' from dual union all
select 59622,'MRN' from dual union all
select 59623,'JMA' from dual union all
select 59624,'AN' from dual union all
select 59625,'MRN' from dual union all
select 59626,'JMA' from dual union all
select 59627,'MRN' from dual union all
select 59628,'MRN' from dual union all
select 59629,'MRN' from dual union all
select 5963,'MVS' from dual union all
select 59630,'MRN' from dual union all
select 59631,'MRN' from dual union all
select 59632,'MRN' from dual union all
select 59633,'MRN' from dual union all
select 59634,'MRN' from dual union all
select 59635,'JMA' from dual union all
select 59636,'MRN' from dual union all
select 59637,'MRN' from dual union all
select 59638,'AN' from dual union all
select 59639,'JMA' from dual union all
select 5964,'USAF' from dual union all
select 59640,'AN' from dual union all
select 59641,'AN' from dual union all
select 59642,'AN' from dual union all
select 59643,'MRN' from dual union all
select 59644,'MRN' from dual union all
select 59645,'MRN' from dual union all
select 59646,'JMA' from dual union all
select 59647,'MRN' from dual union all
select 59648,'MRN' from dual union all
select 59649,'JMA' from dual union all
select 5965,'USAF' from dual union all
select 59650,'MRN' from dual union all
select 59651,'MRN' from dual union all
select 59652,'JMA' from dual union all
select 59653,'MRN' from dual union all
select 59654,'MRN' from dual union all
select 59655,'MRN' from dual union all
select 59656,'MRN' from dual union all
select 59657,'MRN' from dual union all
select 59658,'MRN' from dual union all
select 59659,'JMA' from dual union all
select 5966,'USAF' from dual union all
select 59660,'AN' from dual union all
select 59661,'MRN' from dual union all
select 59662,'MRN' from dual union all
select 59663,'MRN' from dual union all
select 59664,'JMA' from dual union all
select 59665,'MRN' from dual union all
select 59666,'MRN' from dual union all
select 59667,'MRN' from dual union all
select 59668,'MRN' from dual union all
select 59669,'MRN' from dual union all
select 5967,'USAF' from dual union all
select 59670,'MRN' from dual union all
select 59671,'MRN' from dual union all
select 59672,'JMA' from dual union all
select 59673,'MRN' from dual union all
select 59674,'MRN' from dual union all
select 59675,'MRN' from dual union all
select 59676,'MRN' from dual union all
select 59677,'MRN' from dual union all
select 59678,'GMS' from dual union all
select 59679,'JMA' from dual union all
select 5968,'USAF' from dual union all
select 59680,'MRN' from dual union all
select 59681,'MRN' from dual union all
select 59682,'AN' from dual;
insert into LAUNCH_AGENCY
select 59683,'MRN' from dual union all
select 59684,'GMS' from dual union all
select 59685,'MRN' from dual union all
select 59686,'MRN' from dual union all
select 59687,'MRN' from dual union all
select 59688,'GMS' from dual union all
select 59689,'GMS' from dual union all
select 5969,'USAF' from dual union all
select 59690,'MRN' from dual union all
select 59691,'MRN' from dual union all
select 59692,'JMA' from dual union all
select 59693,'GMS' from dual union all
select 59694,'MRN' from dual union all
select 59695,'MRN' from dual union all
select 59696,'JMA' from dual union all
select 59697,'AN' from dual union all
select 59698,'MRN' from dual union all
select 59699,'GMS' from dual union all
select 597,'WER902' from dual union all
select 5970,'USAF' from dual union all
select 59700,'MRN' from dual union all
select 59701,'MRN' from dual union all
select 59702,'MRN' from dual union all
select 59703,'MRN' from dual union all
select 59704,'MRN' from dual union all
select 59705,'MRN' from dual union all
select 59706,'MRN' from dual union all
select 59707,'MRN' from dual union all
select 59708,'MRN' from dual union all
select 59709,'MRN' from dual union all
select 5971,'NASA' from dual union all
select 59710,'MRN' from dual union all
select 59711,'MRN' from dual union all
select 59712,'MRN' from dual union all
select 59713,'MRN' from dual union all
select 59714,'JMA' from dual union all
select 59715,'MRN' from dual union all
select 59716,'MRN' from dual union all
select 59717,'MRN' from dual union all
select 59718,'MRN' from dual union all
select 59719,'MRN' from dual union all
select 5972,'USAF' from dual union all
select 59720,'MRN' from dual union all
select 59721,'MRN' from dual union all
select 59722,'JMA' from dual union all
select 59723,'MRN' from dual union all
select 59724,'JMA' from dual union all
select 59725,'MRN' from dual union all
select 59726,'MRN' from dual union all
select 59727,'MRN' from dual union all
select 59728,'MRN' from dual union all
select 59729,'MRN' from dual union all
select 5973,'USAF' from dual union all
select 59730,'MRN' from dual union all
select 59731,'MRN' from dual union all
select 59732,'JMA' from dual union all
select 59733,'MRN' from dual union all
select 59734,'MRN' from dual union all
select 59735,'MRN' from dual union all
select 59736,'JMA' from dual union all
select 59737,'MRN' from dual union all
select 59738,'MRN' from dual union all
select 59739,'MRN' from dual union all
select 5974,'AN' from dual union all
select 59740,'MRN' from dual union all
select 59741,'MRN' from dual union all
select 59742,'MRN' from dual union all
select 59743,'MRN' from dual union all
select 59744,'JMA' from dual union all
select 59745,'MRN' from dual union all
select 59746,'MRN' from dual union all
select 59747,'MRN' from dual union all
select 59748,'MRN' from dual union all
select 59749,'JMA' from dual union all
select 5975,'AN' from dual union all
select 59750,'MRN' from dual union all
select 59751,'MRN' from dual union all
select 59752,'MRN' from dual union all
select 59753,'MRN' from dual union all
select 59754,'JMA' from dual union all
select 59755,'MRN' from dual union all
select 59756,'MRN' from dual union all
select 59757,'JMA' from dual union all
select 59758,'MRN' from dual union all
select 59759,'MRN' from dual union all
select 5976,'USAF' from dual union all
select 59760,'ISAS' from dual union all
select 59761,'ISAS' from dual union all
select 59762,'AN' from dual union all
select 59763,'JMA' from dual union all
select 59764,'JMA' from dual union all
select 59765,'JMA' from dual union all
select 59766,'MRN' from dual union all
select 59767,'JMA' from dual union all
select 59768,'GMS' from dual union all
select 59769,'GMS' from dual union all
select 5977,'USAF' from dual union all
select 59770,'MRN' from dual union all
select 59771,'MRN' from dual union all
select 59772,'JMA' from dual;
insert into LAUNCH_AGENCY
select 59773,'GMS' from dual union all
select 59774,'MRN' from dual union all
select 59775,'MRN' from dual union all
select 59776,'GMS' from dual union all
select 59777,'GMS' from dual union all
select 59778,'AN' from dual union all
select 59779,'AN' from dual union all
select 5978,'USAF' from dual union all
select 59780,'GMS' from dual union all
select 59781,'MRN' from dual union all
select 59782,'MRN' from dual union all
select 59783,'MRN' from dual union all
select 59784,'GMS' from dual union all
select 59785,'MRN' from dual union all
select 59786,'GMS' from dual union all
select 59787,'MRN' from dual union all
select 59788,'GMS' from dual union all
select 59789,'GMS' from dual union all
select 5979,'RAF' from dual union all
select 59790,'AN' from dual union all
select 59791,'JMA' from dual union all
select 59792,'GMS' from dual union all
select 59793,'AN' from dual union all
select 59794,'MRN' from dual union all
select 59795,'GMS' from dual union all
select 59796,'MRN' from dual union all
select 59797,'MRN' from dual union all
select 59798,'JMA' from dual union all
select 59799,'MRN' from dual union all
select 598,'WER902' from dual union all
select 5980,'ONERA' from dual union all
select 59800,'GMS' from dual union all
select 59801,'GMS' from dual union all
select 59802,'MRN' from dual union all
select 59803,'MRN' from dual union all
select 59804,'MRN' from dual union all
select 59805,'GMS' from dual union all
select 59806,'AN' from dual union all
select 59807,'GMS' from dual union all
select 59808,'JMA' from dual union all
select 59809,'GMS' from dual union all
select 5981,'ABMA' from dual union all
select 5981,'USAF' from dual union all
select 59810,'MRN' from dual union all
select 59811,'MRN' from dual union all
select 59812,'MRN' from dual union all
select 59813,'GMS' from dual union all
select 59814,'AN' from dual union all
select 59815,'GMS' from dual union all
select 59816,'GMS' from dual union all
select 59817,'GMS' from dual union all
select 59818,'JMA' from dual union all
select 59819,'AN' from dual union all
select 5982,'USA' from dual union all
select 59820,'MRN' from dual union all
select 59821,'MRN' from dual union all
select 59822,'JMA' from dual union all
select 59823,'MRN' from dual union all
select 59824,'MRN' from dual union all
select 59825,'AN' from dual union all
select 59826,'MRN' from dual union all
select 59827,'MRN' from dual union all
select 59828,'MRN' from dual union all
select 59829,'AN' from dual union all
select 5983,'MVS' from dual union all
select 59830,'MRN' from dual union all
select 59831,'MRN' from dual union all
select 59832,'AN' from dual union all
select 59833,'AN' from dual union all
select 59834,'AN' from dual union all
select 59835,'JMA' from dual union all
select 59836,'MRN' from dual union all
select 59837,'MRN' from dual union all
select 59838,'MRN' from dual union all
select 59839,'JMA' from dual union all
select 5984,'MVS' from dual union all
select 59840,'JMA' from dual union all
select 59841,'AN' from dual union all
select 59842,'AN' from dual union all
select 59843,'AN' from dual union all
select 59844,'AN' from dual union all
select 59845,'AN' from dual union all
select 59846,'AN' from dual union all
select 59847,'AN' from dual union all
select 59848,'AN' from dual union all
select 59849,'AN' from dual union all
select 5985,'MVS' from dual union all
select 59850,'AN' from dual union all
select 59851,'AN' from dual union all
select 59852,'AN' from dual union all
select 59853,'AN' from dual union all
select 59854,'AN' from dual union all
select 59855,'AN' from dual union all
select 59856,'AN' from dual union all
select 59857,'AN' from dual union all
select 59858,'AN' from dual union all
select 59859,'AN' from dual union all
select 5986,'USAF' from dual union all
select 59860,'AN' from dual union all
select 59861,'AN' from dual;
insert into LAUNCH_AGENCY
select 59862,'AN' from dual union all
select 59863,'AN' from dual union all
select 59864,'AN' from dual union all
select 59865,'AN' from dual union all
select 59866,'AN' from dual union all
select 59867,'AN' from dual union all
select 59868,'AN' from dual union all
select 59869,'AN' from dual union all
select 5987,'USAF' from dual union all
select 59870,'AN' from dual union all
select 59871,'AN' from dual union all
select 59872,'AN' from dual union all
select 59873,'AN' from dual union all
select 59874,'AN' from dual union all
select 59875,'AN' from dual union all
select 59876,'AN' from dual union all
select 59877,'AN' from dual union all
select 59878,'AN' from dual union all
select 59879,'AN' from dual union all
select 5988,'USAF' from dual union all
select 59880,'AN' from dual union all
select 59881,'AN' from dual union all
select 59882,'AN' from dual union all
select 59883,'AN' from dual union all
select 59884,'AN' from dual union all
select 59885,'AN' from dual union all
select 59886,'AN' from dual union all
select 59887,'AN' from dual union all
select 59888,'AN' from dual union all
select 59889,'AN' from dual union all
select 5989,'LARCN' from dual union all
select 59890,'AN' from dual union all
select 59891,'AN' from dual union all
select 59892,'AN' from dual union all
select 59893,'AN' from dual union all
select 59894,'AN' from dual union all
select 59895,'AN' from dual union all
select 59896,'AN' from dual union all
select 59897,'AN' from dual union all
select 59898,'AN' from dual union all
select 59899,'AN' from dual union all
select 599,'WER902' from dual union all
select 5990,'USAF' from dual union all
select 59900,'AN' from dual union all
select 59901,'AN' from dual union all
select 59902,'AN' from dual union all
select 59903,'AN' from dual union all
select 59904,'AN' from dual union all
select 59905,'AN' from dual union all
select 59906,'AN' from dual union all
select 59907,'AN' from dual union all
select 59908,'AN' from dual union all
select 59909,'AN' from dual union all
select 5991,'RAE' from dual union all
select 59910,'AN' from dual union all
select 59911,'AN' from dual union all
select 59912,'AN' from dual union all
select 59913,'AN' from dual union all
select 59914,'AN' from dual union all
select 59915,'AN' from dual union all
select 59916,'AN' from dual union all
select 59917,'AN' from dual union all
select 59918,'AN' from dual union all
select 59919,'AN' from dual union all
select 5992,'RVSN' from dual union all
select 59920,'AN' from dual union all
select 59921,'AN' from dual union all
select 59922,'AN' from dual union all
select 59923,'AN' from dual union all
select 59924,'AN' from dual union all
select 59925,'AN' from dual union all
select 59926,'AN' from dual union all
select 59927,'NASA' from dual union all
select 59928,'AN' from dual union all
select 59929,'AN' from dual union all
select 5993,'CASDN' from dual union all
select 59930,'NASA' from dual union all
select 59931,'NASA' from dual union all
select 59932,'AN' from dual union all
select 59933,'AN' from dual union all
select 59934,'AN' from dual union all
select 59935,'NASA' from dual union all
select 59936,'NASA' from dual union all
select 59937,'AN' from dual union all
select 59938,'AN' from dual union all
select 59939,'AN' from dual union all
select 5994,'MVS' from dual union all
select 59940,'AN' from dual union all
select 59941,'AN' from dual union all
select 59942,'AN' from dual union all
select 59943,'AN' from dual union all
select 59944,'AN' from dual union all
select 59945,'AN' from dual union all
select 59946,'AN' from dual union all
select 59947,'AN' from dual union all
select 59948,'AN' from dual union all
select 59949,'AN' from dual union all
select 5995,'USAF' from dual union all
select 59950,'AN' from dual union all
select 59951,'AN' from dual;
insert into LAUNCH_AGENCY
select 59952,'AN' from dual union all
select 59953,'AN' from dual union all
select 59954,'AN' from dual union all
select 59955,'AN' from dual union all
select 59956,'AN' from dual union all
select 59957,'AN' from dual union all
select 59958,'AN' from dual union all
select 59959,'AN' from dual union all
select 5996,'USAF' from dual union all
select 59960,'AN' from dual union all
select 59961,'AN' from dual union all
select 59962,'AN' from dual union all
select 59963,'AN' from dual union all
select 59964,'AN' from dual union all
select 59965,'AN' from dual union all
select 59966,'AN' from dual union all
select 59967,'AN' from dual union all
select 59968,'AN' from dual union all
select 59969,'AN' from dual union all
select 5997,'ABMA' from dual union all
select 5997,'USAF' from dual union all
select 59970,'AN' from dual union all
select 59971,'AN' from dual union all
select 59972,'AN' from dual union all
select 59973,'AN' from dual union all
select 59974,'AN' from dual union all
select 59975,'AN' from dual union all
select 59976,'AN' from dual union all
select 59977,'AN' from dual union all
select 59978,'AN' from dual union all
select 59979,'AN' from dual union all
select 5998,'SAND' from dual union all
select 59980,'AN' from dual union all
select 59981,'AN' from dual union all
select 59982,'AN' from dual union all
select 59983,'AN' from dual union all
select 59984,'AN' from dual union all
select 59985,'MRN' from dual union all
select 59986,'AN' from dual union all
select 59987,'MRN' from dual union all
select 59988,'MRN' from dual union all
select 59989,'AN' from dual union all
select 5999,'SAND' from dual union all
select 59990,'AN' from dual union all
select 59991,'AN' from dual union all
select 59992,'AN' from dual union all
select 59993,'AN' from dual union all
select 59994,'AN' from dual union all
select 59995,'AN' from dual union all
select 59996,'AN' from dual union all
select 59997,'AN' from dual union all
select 59998,'AN' from dual union all
select 59999,'AN' from dual union all
select 6,'WEHR' from dual union all
select 60,'WER271-1' from dual union all
select 600,'WER902' from dual union all
select 6000,'USAF' from dual union all
select 60000,'AN' from dual union all
select 60001,'AN' from dual union all
select 60002,'AN' from dual union all
select 60003,'AN' from dual union all
select 60004,'AN' from dual union all
select 60005,'AN' from dual union all
select 60006,'AN' from dual union all
select 60007,'AN' from dual union all
select 60008,'AN' from dual union all
select 60009,'GSFC' from dual union all
select 6001,'NRL' from dual union all
select 60010,'GSFC' from dual union all
select 60011,'GSFC' from dual union all
select 60012,'GSFC' from dual union all
select 60013,'GSFC' from dual union all
select 60014,'GSFC' from dual union all
select 60015,'AN' from dual union all
select 60016,'AN' from dual union all
select 60017,'AN' from dual union all
select 60018,'AN' from dual union all
select 60019,'AN' from dual union all
select 6002,'BRL' from dual union all
select 6002,'UMI' from dual union all
select 60020,'AN' from dual union all
select 60021,'AN' from dual union all
select 60022,'AN' from dual union all
select 60023,'AN' from dual union all
select 60024,'AN' from dual union all
select 60025,'AN' from dual union all
select 60026,'AN' from dual union all
select 60027,'AN' from dual union all
select 60028,'AN' from dual union all
select 60029,'AN' from dual union all
select 6003,'RAF' from dual union all
select 60030,'AN' from dual union all
select 60031,'AN' from dual union all
select 60032,'AN' from dual union all
select 60033,'AN' from dual union all
select 60034,'AN' from dual union all
select 60035,'AN' from dual union all
select 60036,'AN' from dual union all
select 60037,'AN' from dual union all
select 60038,'AN' from dual;
insert into LAUNCH_AGENCY
select 60039,'AN' from dual union all
select 6004,'CASDN' from dual union all
select 60040,'AN' from dual union all
select 60041,'AN' from dual union all
select 60042,'AN' from dual union all
select 60043,'AN' from dual union all
select 60044,'AN' from dual union all
select 60045,'AN' from dual union all
select 60046,'AN' from dual union all
select 60047,'AN' from dual union all
select 60048,'AN' from dual union all
select 60049,'AN' from dual union all
select 6005,'USAF' from dual union all
select 60050,'AN' from dual union all
select 60051,'AN' from dual union all
select 60052,'AN' from dual union all
select 60053,'AN' from dual union all
select 60054,'AN' from dual union all
select 60055,'AN' from dual union all
select 60056,'AN' from dual union all
select 60057,'AN' from dual union all
select 60058,'AN' from dual union all
select 60059,'AN' from dual union all
select 6006,'USAF' from dual union all
select 60060,'AN' from dual union all
select 60061,'AN' from dual union all
select 60062,'AN' from dual union all
select 60063,'AN' from dual union all
select 60064,'AN' from dual union all
select 60065,'AN' from dual union all
select 60066,'AN' from dual union all
select 60067,'AN' from dual union all
select 60068,'AN' from dual union all
select 60069,'AN' from dual union all
select 6007,'NASA' from dual union all
select 60070,'AN' from dual union all
select 60071,'AN' from dual union all
select 60072,'AN' from dual union all
select 60073,'AN' from dual union all
select 60074,'AN' from dual union all
select 60075,'AN' from dual union all
select 60076,'AN' from dual union all
select 60077,'AN' from dual union all
select 60078,'AN' from dual union all
select 60079,'AN' from dual union all
select 6008,'USAF' from dual union all
select 60080,'AN' from dual union all
select 60081,'AN' from dual union all
select 60082,'AN' from dual union all
select 60083,'AN' from dual union all
select 60084,'AN' from dual union all
select 60085,'AN' from dual union all
select 60086,'AN' from dual union all
select 60087,'AN' from dual union all
select 60088,'AN' from dual union all
select 60089,'AN' from dual union all
select 6009,'ABMA' from dual union all
select 6009,'USAF' from dual union all
select 60090,'AN' from dual union all
select 60091,'UNKS' from dual union all
select 60092,'UNKS' from dual union all
select 60093,'UNKS' from dual union all
select 60094,'UNKS' from dual union all
select 60095,'UNKS' from dual union all
select 60096,'UNKS' from dual union all
select 60097,'UNKS' from dual union all
select 60098,'UNKS' from dual union all
select 60099,'UNKS' from dual union all
select 601,'WER902' from dual union all
select 6010,'USAF' from dual union all
select 60100,'UNKS' from dual union all
select 60101,'UNKS' from dual union all
select 60102,'ISAS' from dual union all
select 60103,'UNKS' from dual union all
select 60104,'UNKS' from dual union all
select 60105,'AFSC' from dual union all
select 60106,'UNKS' from dual union all
select 60107,'UNKS' from dual union all
select 60108,'NASDA' from dual union all
select 60109,'UNKS' from dual union all
select 6011,'USAF' from dual union all
select 60110,'UNKS' from dual union all
select 60111,'UNKS' from dual union all
select 60112,'NASA' from dual union all
select 60113,'UNKS' from dual union all
select 60114,'UNKS' from dual union all
select 60115,'UNKS' from dual union all
select 60116,'UNKS' from dual union all
select 60117,'UNKS' from dual union all
select 60118,'UNKS' from dual union all
select 60119,'NASA' from dual union all
select 6012,'NASA' from dual union all
select 60120,'UNKS' from dual union all
select 60121,'UNKS' from dual union all
select 60122,'UNKS' from dual union all
select 60123,'UNKS' from dual union all
select 60124,'UNKS' from dual union all
select 60125,'UNKS' from dual union all
select 60126,'UNKS' from dual union all
select 60127,'UNKS' from dual;
insert into LAUNCH_AGENCY
select 60128,'UNKS' from dual union all
select 60129,'UNKS' from dual union all
select 6013,'USN' from dual union all
select 60130,'UNKS' from dual union all
select 60131,'UNKS' from dual union all
select 60132,'UNKS' from dual union all
select 60133,'AFSC' from dual union all
select 60134,'UNKS' from dual union all
select 60135,'UNKS' from dual union all
select 60136,'UNKS' from dual union all
select 60137,'UNKS' from dual union all
select 60138,'UNKS' from dual union all
select 60139,'UNKS' from dual union all
select 6014,'NASA' from dual union all
select 60140,'UNKS' from dual union all
select 60141,'UNKS' from dual union all
select 60142,'UNKS' from dual union all
select 60143,'AFSC' from dual union all
select 60144,'UNKS' from dual union all
select 60145,'UNKS' from dual union all
select 60146,'UNKS' from dual union all
select 60147,'UNKS' from dual union all
select 60148,'UNKS' from dual union all
select 60149,'UNKS' from dual union all
select 6015,'RAE' from dual union all
select 60150,'UNKS' from dual union all
select 60151,'UNKS' from dual union all
select 60152,'UNKS' from dual union all
select 60153,'UNKS' from dual union all
select 60154,'UNKS' from dual union all
select 60155,'UNKS' from dual union all
select 60156,'UNKS' from dual union all
select 60157,'CALT' from dual union all
select 60158,'UNKS' from dual union all
select 60159,'UNKS' from dual union all
select 6016,'USAF' from dual union all
select 60160,'NASDA' from dual union all
select 60161,'UNKS' from dual union all
select 60162,'UNKS' from dual union all
select 60163,'UNKS' from dual union all
select 60164,'UNKS' from dual union all
select 60165,'CALT' from dual union all
select 60166,'UNKS' from dual union all
select 60167,'UNKS' from dual union all
select 60168,'AE' from dual union all
select 60169,'UNKS' from dual union all
select 6017,'MVS' from dual union all
select 60170,'NASA' from dual union all
select 60171,'UNKS' from dual union all
select 60172,'UNKS' from dual union all
select 60173,'UNKS' from dual union all
select 60174,'UNKS' from dual union all
select 60175,'UNKS' from dual union all
select 60176,'UNKS' from dual union all
select 60177,'UNKS' from dual union all
select 60178,'UNKS' from dual union all
select 60179,'UNKS' from dual union all
select 6018,'USAF' from dual union all
select 60180,'AFSC' from dual union all
select 60181,'UNKS' from dual union all
select 60182,'UNKS' from dual union all
select 60183,'UNKS' from dual union all
select 60184,'UNKS' from dual union all
select 60185,'AE' from dual union all
select 60186,'UNKS' from dual union all
select 60187,'AFSC' from dual union all
select 60188,'UNKS' from dual union all
select 60189,'UNKS' from dual union all
select 6019,'MVS' from dual union all
select 60190,'UNKS' from dual union all
select 60191,'UNKS' from dual union all
select 60192,'UNKS' from dual union all
select 60193,'UNKS' from dual union all
select 60194,'UNKS' from dual union all
select 60195,'UNKS' from dual union all
select 60196,'UNKS' from dual union all
select 60197,'UNKS' from dual union all
select 60198,'UNKS' from dual union all
select 60199,'UNKS' from dual union all
select 602,'WER902' from dual union all
select 6020,'MVS' from dual union all
select 60200,'UNKS' from dual union all
select 60201,'RVSN' from dual union all
select 60202,'MRN' from dual union all
select 60203,'MRN' from dual union all
select 60204,'MRN' from dual union all
select 60205,'NASA' from dual union all
select 60206,'MRN' from dual union all
select 60207,'MRN' from dual union all
select 60208,'RN' from dual union all
select 60209,'RVSN' from dual union all
select 6021,'RVSN' from dual union all
select 60210,'RVSN' from dual union all
select 60211,'MRN' from dual union all
select 60212,'MRN' from dual union all
select 60213,'MRN' from dual union all
select 60214,'MRN' from dual union all
select 60215,'RVSN' from dual union all
select 60216,'AN' from dual union all
select 60217,'AN' from dual;
insert into LAUNCH_AGENCY
select 60218,'MRN' from dual union all
select 60219,'MRN' from dual union all
select 6022,'NRL' from dual union all
select 60220,'DFVLR' from dual union all
select 60221,'MRN' from dual union all
select 60222,'MRN' from dual union all
select 60223,'DLR' from dual union all
select 60224,'IQ' from dual union all
select 60225,'DNA' from dual union all
select 60226,'DNA' from dual union all
select 60227,'DNA' from dual union all
select 60228,'DNA' from dual union all
select 60229,'DNA' from dual union all
select 6023,'RAE' from dual union all
select 6023,'WRE' from dual union all
select 60230,'USA' from dual union all
select 60231,'ISRO' from dual union all
select 60232,'NASA' from dual union all
select 60233,'UNKS' from dual union all
select 60234,'ISRO' from dual union all
select 60235,'ISRO' from dual union all
select 60236,'ISRO' from dual union all
select 60237,'ISRO' from dual union all
select 60238,'ISRO' from dual union all
select 60239,'ISRO' from dual union all
select 6024,'RAE' from dual union all
select 6024,'WRE' from dual union all
select 60240,'ISRO' from dual union all
select 60241,'ISRO' from dual union all
select 60242,'ISRO' from dual union all
select 60243,'ISRO' from dual union all
select 60244,'ISRO' from dual union all
select 60245,'ISRO' from dual union all
select 60246,'ISRO' from dual union all
select 60247,'ISRO' from dual union all
select 60248,'ISRO' from dual union all
select 60249,'ISRO' from dual union all
select 6025,'NASA' from dual union all
select 60250,'ISRO' from dual union all
select 60251,'ISRO' from dual union all
select 60252,'ISRO' from dual union all
select 60253,'ISRO' from dual union all
select 60254,'ISRO' from dual union all
select 60255,'ISRO' from dual union all
select 60256,'ISRO' from dual union all
select 60257,'ISRO' from dual union all
select 60258,'ISRO' from dual union all
select 60259,'ISRO' from dual union all
select 6026,'USAF' from dual union all
select 60260,'ISRO' from dual union all
select 60261,'ISRO' from dual union all
select 60262,'ISRO' from dual union all
select 60263,'ISRO' from dual union all
select 60264,'ISRO' from dual union all
select 60265,'ISRO' from dual union all
select 60266,'ISRO' from dual union all
select 60267,'ISRO' from dual union all
select 60268,'ISRO' from dual union all
select 60269,'ISRO' from dual union all
select 6027,'USAF' from dual union all
select 60270,'ISRO' from dual union all
select 60271,'ISRO' from dual union all
select 60272,'ISRO' from dual union all
select 60273,'ISRO' from dual union all
select 60274,'LAPAN' from dual union all
select 60275,'ISRO' from dual union all
select 60276,'ISRO' from dual union all
select 60277,'ISRO' from dual union all
select 60278,'ISRO' from dual union all
select 60279,'ISRO' from dual union all
select 6028,'RAE' from dual union all
select 6028,'WRE' from dual union all
select 60280,'ISRO' from dual union all
select 60281,'ISRO' from dual union all
select 60282,'ISRO' from dual union all
select 60283,'ISRO' from dual union all
select 60284,'ISRO' from dual union all
select 60285,'ISRO' from dual union all
select 60286,'ISRO' from dual union all
select 60287,'RVSN' from dual union all
select 60288,'RVSN' from dual union all
select 60289,'RVSN' from dual union all
select 6029,'AN' from dual union all
select 60290,'RVSN' from dual union all
select 60291,'RVSN' from dual union all
select 60292,'RVSN' from dual union all
select 60293,'RVSN' from dual union all
select 60294,'RVSN' from dual union all
select 60295,'RVSN' from dual union all
select 60296,'RVSN' from dual union all
select 60297,'SAC' from dual union all
select 60298,'AN' from dual union all
select 60299,'AN' from dual union all
select 603,'WER902' from dual union all
select 6030,'AN' from dual union all
select 60300,'AN' from dual union all
select 60301,'AN' from dual union all
select 60302,'AN' from dual union all
select 60303,'AN' from dual union all
select 60304,'ALCS' from dual;
insert into LAUNCH_AGENCY
select 60304,'SAC' from dual union all
select 60305,'ALCS' from dual union all
select 60305,'SAC' from dual union all
select 60306,'NASA' from dual union all
select 60307,'RVSN' from dual union all
select 60308,'USN' from dual union all
select 60309,'NASA' from dual union all
select 6031,'NASA' from dual union all
select 60310,'NASA' from dual union all
select 60311,'US7A' from dual union all
select 60312,'US7A' from dual union all
select 60313,'US7A' from dual union all
select 60314,'US7A' from dual union all
select 60315,'NASA' from dual union all
select 60316,'AN' from dual union all
select 60317,'AN' from dual union all
select 60318,'RVSN' from dual union all
select 60319,'AN' from dual union all
select 6032,'USN' from dual union all
select 60320,'AN' from dual union all
select 60321,'AN' from dual union all
select 60322,'AN' from dual union all
select 60323,'USN' from dual union all
select 60324,'NASA' from dual union all
select 60325,'NASA' from dual union all
select 60326,'AFGL' from dual union all
select 60327,'NASA' from dual union all
select 60328,'SDIO' from dual union all
select 60329,'RVSN' from dual union all
select 6033,'USAF' from dual union all
select 60330,'DFVLR' from dual union all
select 60331,'UNKS' from dual union all
select 60332,'USN' from dual union all
select 60333,'USN' from dual union all
select 60334,'USN' from dual union all
select 60335,'USN' from dual union all
select 60336,'AN' from dual union all
select 60337,'AN' from dual union all
select 60338,'AN' from dual union all
select 60339,'AN' from dual union all
select 6034,'ABMA' from dual union all
select 6034,'USAF' from dual union all
select 60340,'RVSN' from dual union all
select 60341,'AFSC' from dual union all
select 60342,'USN' from dual union all
select 60343,'USN' from dual union all
select 60344,'USN' from dual union all
select 60345,'USN' from dual union all
select 60346,'USN' from dual union all
select 60347,'AFSC' from dual union all
select 60348,'NASA' from dual union all
select 60349,'RVSN' from dual union all
select 6035,'USN' from dual union all
select 60350,'AFBMD' from dual union all
select 60351,'RVSN' from dual union all
select 60352,'USN' from dual union all
select 60353,'DLR' from dual union all
select 60353,'NDRE' from dual union all
select 60354,'NASA' from dual union all
select 60355,'NASA' from dual union all
select 60356,'USN' from dual union all
select 60357,'USN' from dual union all
select 60358,'USN' from dual union all
select 60359,'USN' from dual union all
select 6036,'ONERA' from dual union all
select 60360,'DLR' from dual union all
select 60361,'DLR' from dual union all
select 60362,'RVSN' from dual union all
select 60363,'NASA' from dual union all
select 60364,'NASA' from dual union all
select 60365,'NASA' from dual union all
select 60366,'NASA' from dual union all
select 60367,'ALCS' from dual union all
select 60367,'SAC' from dual union all
select 60368,'NASA' from dual union all
select 60369,'SAC' from dual union all
select 6037,'SAND' from dual union all
select 60370,'IAE' from dual union all
select 60371,'SAC' from dual union all
select 60372,'DLR' from dual union all
select 60373,'RVSN' from dual union all
select 60374,'DLR' from dual union all
select 60375,'NASA' from dual union all
select 60376,'NASA' from dual union all
select 60377,'NASA' from dual union all
select 60378,'AFBMD' from dual union all
select 60379,'NASA' from dual union all
select 6038,'MVS' from dual union all
select 60380,'NASA' from dual union all
select 60381,'USN' from dual union all
select 60382,'NASA' from dual union all
select 60383,'RVSN' from dual union all
select 60384,'NASA' from dual union all
select 60385,'SDIO' from dual union all
select 60386,'RVSN' from dual union all
select 60387,'ISRO' from dual union all
select 60388,'PLA2AC' from dual union all
select 60389,'SDIO' from dual union all
select 6039,'USAF' from dual union all
select 60390,'IQ' from dual;
insert into LAUNCH_AGENCY
select 60391,'IQ' from dual union all
select 60392,'IQ' from dual union all
select 60393,'IQ' from dual union all
select 60394,'RVSN' from dual union all
select 60395,'IQ224' from dual union all
select 60396,'IQ224' from dual union all
select 60397,'FKG' from dual union all
select 60398,'FKG' from dual union all
select 60399,'FKG' from dual union all
select 604,'WER902' from dual union all
select 6040,'ABMA' from dual union all
select 6040,'USAF' from dual union all
select 60400,'FKG' from dual union all
select 60401,'US7A' from dual union all
select 60402,'US7A' from dual union all
select 60403,'US7A' from dual union all
select 60404,'US7A' from dual union all
select 60405,'RVSN' from dual union all
select 60406,'US7A' from dual union all
select 60407,'RVSN' from dual union all
select 60408,'RVSN' from dual union all
select 60409,'SDIO' from dual union all
select 6041,'MVS' from dual union all
select 60410,'RVSN' from dual union all
select 60411,'RVSN' from dual union all
select 60412,'RVSN' from dual union all
select 60413,'RVSN' from dual union all
select 60414,'RVSN' from dual union all
select 60415,'RVSN' from dual union all
select 60416,'RVSN' from dual union all
select 60417,'RVSN' from dual union all
select 60418,'RVSN' from dual union all
select 60419,'RVSN' from dual union all
select 6042,'USAF' from dual union all
select 60420,'RVSN' from dual union all
select 60421,'RVSN' from dual union all
select 60422,'RVSN' from dual union all
select 60423,'RVSN' from dual union all
select 60424,'RVSN' from dual union all
select 60425,'RVSN' from dual union all
select 60426,'RVSN' from dual union all
select 60427,'RVSN' from dual union all
select 60428,'RVSN' from dual union all
select 60429,'RVSN' from dual union all
select 6043,'MVS' from dual union all
select 60430,'RVSN' from dual union all
select 60431,'RVSN' from dual union all
select 60432,'RVSN' from dual union all
select 60433,'RVSN' from dual union all
select 60434,'RVSN' from dual union all
select 60435,'RVSN' from dual union all
select 60436,'RVSN' from dual union all
select 60437,'RVSN' from dual union all
select 60438,'RVSN' from dual union all
select 60439,'RVSN' from dual union all
select 6044,'NASA' from dual union all
select 60440,'MVS' from dual union all
select 60441,'MVS' from dual union all
select 60442,'RVSN' from dual union all
select 60443,'RVSN' from dual union all
select 60444,'RVSN' from dual union all
select 60445,'RVSN' from dual union all
select 60446,'RVSN' from dual union all
select 60447,'RVSN' from dual union all
select 60448,'RVSN' from dual union all
select 60449,'RVSN' from dual union all
select 6045,'RVSN' from dual union all
select 60450,'RVSN' from dual union all
select 60451,'RVSN' from dual union all
select 60452,'RVSN' from dual union all
select 60453,'RVSN' from dual union all
select 60454,'RVSN' from dual union all
select 60455,'RVSN' from dual union all
select 60456,'RVSN' from dual union all
select 60457,'RVSN' from dual union all
select 60458,'RVSN' from dual union all
select 60459,'RVSN' from dual union all
select 6046,'MVS' from dual union all
select 60460,'AFBMD' from dual union all
select 60461,'RVSN' from dual union all
select 60462,'RVSN' from dual union all
select 60463,'ISAS' from dual union all
select 60464,'USN' from dual union all
select 60465,'AFSC' from dual union all
select 60466,'SAC' from dual union all
select 60467,'NASA' from dual union all
select 60468,'NASA' from dual union all
select 60469,'USN' from dual union all
select 6047,'SAND' from dual union all
select 60470,'ALCS' from dual union all
select 60470,'SAC' from dual union all
select 60471,'RVSN' from dual union all
select 60472,'AFSC' from dual union all
select 60473,'DLR' from dual union all
select 60474,'NASA' from dual union all
select 60475,'ISAS' from dual union all
select 60476,'ISAS' from dual union all
select 60477,'USAF' from dual union all
select 60478,'NASA' from dual union all
select 60479,'NASA' from dual;
insert into LAUNCH_AGENCY
select 6048,'MVS' from dual union all
select 60480,'NASA' from dual union all
select 60481,'ALCS' from dual union all
select 60481,'SAC' from dual union all
select 60482,'USN' from dual union all
select 60483,'SSC' from dual union all
select 60484,'AFSC' from dual union all
select 60485,'AFGL' from dual union all
select 60486,'NASA' from dual union all
select 60487,'NASA' from dual union all
select 60488,'US7A' from dual union all
select 60489,'US7A' from dual union all
select 6049,'MVS' from dual union all
select 60490,'US7A' from dual union all
select 60491,'US7A' from dual union all
select 60492,'US7A' from dual union all
select 60493,'US7A' from dual union all
select 60494,'NASA' from dual union all
select 60495,'NASA' from dual union all
select 60496,'NASA' from dual union all
select 60497,'NASA' from dual union all
select 60498,'AFSC' from dual union all
select 60499,'RN' from dual union all
select 605,'WER902' from dual union all
select 6050,'MVS' from dual union all
select 60500,'RN' from dual union all
select 60501,'IDF' from dual union all
select 60502,'USN' from dual union all
select 60503,'DFVLR' from dual union all
select 60504,'DLR' from dual union all
select 60505,'DFVLR' from dual union all
select 60506,'RN' from dual union all
select 60507,'US7A' from dual union all
select 60508,'US7A' from dual union all
select 60509,'US7A' from dual union all
select 6051,'MVS' from dual union all
select 60510,'US7A' from dual union all
select 60511,'US7A' from dual union all
select 60512,'US7A' from dual union all
select 60513,'AN' from dual union all
select 60514,'AN' from dual union all
select 60515,'AN' from dual union all
select 60516,'AN' from dual union all
select 60517,'AN' from dual union all
select 60518,'ISRO' from dual union all
select 60519,'DMA' from dual union all
select 6052,'MVS' from dual union all
select 60520,'USN' from dual union all
select 60521,'UNKS' from dual union all
select 60522,'GMS' from dual union all
select 60523,'GMS' from dual union all
select 60524,'AN' from dual union all
select 60525,'GMS' from dual union all
select 60526,'GMS' from dual union all
select 60527,'GMS' from dual union all
select 60528,'GMS' from dual union all
select 60529,'GMS' from dual union all
select 6053,'ATEF' from dual union all
select 60530,'GMS' from dual union all
select 60531,'MRN' from dual union all
select 60532,'MRN' from dual union all
select 60533,'MRN' from dual union all
select 60534,'MRN' from dual union all
select 60535,'MRN' from dual union all
select 60536,'GMS' from dual union all
select 60537,'GMS' from dual union all
select 60538,'AN' from dual union all
select 60539,'MRN' from dual union all
select 6054,'ATEF' from dual union all
select 60540,'MRN' from dual union all
select 60541,'JMA' from dual union all
select 60542,'AN' from dual union all
select 60543,'MRN' from dual union all
select 60544,'MRN' from dual union all
select 60545,'GMS' from dual union all
select 60546,'MRN' from dual union all
select 60547,'MRN' from dual union all
select 60548,'GMS' from dual union all
select 60549,'MRN' from dual union all
select 6055,'ATEF' from dual union all
select 60550,'JMA' from dual union all
select 60551,'MRN' from dual union all
select 60552,'MRN' from dual union all
select 60553,'MRN' from dual union all
select 60554,'MRN' from dual union all
select 60555,'MRN' from dual union all
select 60556,'AN' from dual union all
select 60557,'MRN' from dual union all
select 60558,'MRN' from dual union all
select 60559,'MRN' from dual union all
select 6056,'ATEF' from dual union all
select 60560,'MRN' from dual union all
select 60561,'MRN' from dual union all
select 60562,'MRN' from dual union all
select 60563,'JMA' from dual union all
select 60564,'MRN' from dual union all
select 60565,'AN' from dual union all
select 60566,'MRN' from dual union all
select 60567,'MRN' from dual union all
select 60568,'MRN' from dual;
insert into LAUNCH_AGENCY
select 60569,'MRN' from dual union all
select 6057,'ATEF' from dual union all
select 60570,'JMA' from dual union all
select 60571,'AN' from dual union all
select 60572,'MRN' from dual union all
select 60573,'MRN' from dual union all
select 60574,'MRN' from dual union all
select 60575,'GMS' from dual union all
select 60576,'JMA' from dual union all
select 60577,'AN' from dual union all
select 60578,'MRN' from dual union all
select 60579,'AN' from dual union all
select 6058,'SAND' from dual union all
select 60580,'MRN' from dual union all
select 60581,'MRN' from dual union all
select 60582,'AN' from dual union all
select 60583,'MRN' from dual union all
select 60584,'MRN' from dual union all
select 60585,'GMS' from dual union all
select 60586,'MRN' from dual union all
select 60587,'MRN' from dual union all
select 60588,'JMA' from dual union all
select 60589,'GMS' from dual union all
select 6059,'MVS' from dual union all
select 60590,'AN' from dual union all
select 60591,'MRN' from dual union all
select 60592,'MRN' from dual union all
select 60593,'GMS' from dual union all
select 60594,'MRN' from dual union all
select 60595,'GMS' from dual union all
select 60596,'MRN' from dual union all
select 60597,'MRN' from dual union all
select 60598,'MRN' from dual union all
select 60599,'JMA' from dual union all
select 606,'WER902' from dual union all
select 6060,'USAF' from dual union all
select 60600,'GMS' from dual union all
select 60601,'MRN' from dual union all
select 60602,'AN' from dual union all
select 60603,'GMS' from dual union all
select 60604,'MRN' from dual union all
select 60605,'MRN' from dual union all
select 60606,'MRN' from dual union all
select 60607,'JMA' from dual union all
select 60608,'GMS' from dual union all
select 60609,'GMS' from dual union all
select 6061,'WRE' from dual union all
select 60610,'AN' from dual union all
select 60611,'MRN' from dual union all
select 60612,'MRN' from dual union all
select 60613,'GMS' from dual union all
select 60614,'GMS' from dual union all
select 60615,'AN' from dual union all
select 60616,'DLR' from dual union all
select 60617,'JMA' from dual union all
select 60618,'MRN' from dual union all
select 60619,'GMS' from dual union all
select 6062,'UNRV' from dual union all
select 60620,'GMS' from dual union all
select 60621,'AN' from dual union all
select 60622,'MRN' from dual union all
select 60623,'GMS' from dual union all
select 60624,'GMS' from dual union all
select 60625,'WFF' from dual union all
select 60626,'MRN' from dual union all
select 60627,'AN' from dual union all
select 60628,'MRN' from dual union all
select 60629,'GMS' from dual union all
select 6063,'UNRV' from dual union all
select 60630,'GMS' from dual union all
select 60631,'MRN' from dual union all
select 60632,'GMS' from dual union all
select 60633,'MRN' from dual union all
select 60634,'GMS' from dual union all
select 60635,'JMA' from dual union all
select 60636,'GMS' from dual union all
select 60637,'MRN' from dual union all
select 60638,'GMS' from dual union all
select 60639,'MRN' from dual union all
select 6064,'UNRV' from dual union all
select 60640,'GMS' from dual union all
select 60641,'GMS' from dual union all
select 60642,'AN' from dual union all
select 60643,'AN' from dual union all
select 60644,'MRN' from dual union all
select 60645,'JMA' from dual union all
select 60646,'MRN' from dual union all
select 60647,'MRN' from dual union all
select 60648,'MRN' from dual union all
select 60649,'AN' from dual union all
select 6065,'RVSN' from dual union all
select 60650,'MRN' from dual union all
select 60651,'MRN' from dual union all
select 60652,'MRN' from dual union all
select 60653,'MRN' from dual union all
select 60654,'MRN' from dual union all
select 60655,'JMA' from dual union all
select 60656,'AN' from dual union all
select 60657,'MRN' from dual union all
select 60658,'MRN' from dual;
insert into LAUNCH_AGENCY
select 60659,'MRN' from dual union all
select 6066,'RVSN' from dual union all
select 60660,'MRN' from dual union all
select 60661,'JMA' from dual union all
select 60662,'MRN' from dual union all
select 60663,'JMA' from dual union all
select 60664,'GMS' from dual union all
select 60665,'AN' from dual union all
select 60666,'MRN' from dual union all
select 60667,'MRN' from dual union all
select 60668,'MRN' from dual union all
select 60669,'JMA' from dual union all
select 6067,'UNRV' from dual union all
select 60670,'GMS' from dual union all
select 60671,'MRN' from dual union all
select 60672,'MRN' from dual union all
select 60673,'GMS' from dual union all
select 60674,'GMS' from dual union all
select 60675,'MRN' from dual union all
select 60676,'GMS' from dual union all
select 60677,'MRN' from dual union all
select 60678,'MRN' from dual union all
select 60679,'MRN' from dual union all
select 6068,'UNRV' from dual union all
select 60680,'MRN' from dual union all
select 60681,'MRN' from dual union all
select 60682,'AN' from dual union all
select 60683,'GMS' from dual union all
select 60684,'GMS' from dual union all
select 60685,'AN' from dual union all
select 60686,'MRN' from dual union all
select 60687,'MRN' from dual union all
select 60688,'GMS' from dual union all
select 60689,'MRN' from dual union all
select 6069,'NRL' from dual union all
select 60690,'GMS' from dual union all
select 60691,'MRN' from dual union all
select 60692,'MRN' from dual union all
select 60693,'MRN' from dual union all
select 60694,'JMA' from dual union all
select 60695,'MRN' from dual union all
select 60696,'MRN' from dual union all
select 60697,'MRN' from dual union all
select 60698,'MRN' from dual union all
select 60699,'AN' from dual union all
select 607,'WEHR' from dual union all
select 6070,'UNRV' from dual union all
select 60700,'DLR' from dual union all
select 60701,'MRN' from dual union all
select 60702,'AN' from dual union all
select 60703,'JMA' from dual union all
select 60704,'AN' from dual union all
select 60705,'AN' from dual union all
select 60706,'MRN' from dual union all
select 60707,'MRN' from dual union all
select 60708,'MRN' from dual union all
select 60709,'AN' from dual union all
select 6071,'RVSN' from dual union all
select 60710,'MRN' from dual union all
select 60711,'JMA' from dual union all
select 60712,'AN' from dual union all
select 60713,'AN' from dual union all
select 60714,'MRN' from dual union all
select 60715,'MRN' from dual union all
select 60716,'MRN' from dual union all
select 60717,'MRN' from dual union all
select 60718,'MRN' from dual union all
select 60719,'MRN' from dual union all
select 6072,'RVSN' from dual union all
select 60720,'AN' from dual union all
select 60721,'JMA' from dual union all
select 60722,'AN' from dual union all
select 60723,'AN' from dual union all
select 60724,'GMS' from dual union all
select 60725,'AN' from dual union all
select 60726,'AN' from dual union all
select 60727,'JMA' from dual union all
select 60728,'GMS' from dual union all
select 60729,'MRN' from dual union all
select 6073,'UNRV' from dual union all
select 60730,'AN' from dual union all
select 60731,'MRN' from dual union all
select 60732,'MRN' from dual union all
select 60733,'AN' from dual union all
select 60734,'AN' from dual union all
select 60735,'JMA' from dual union all
select 60736,'GMS' from dual union all
select 60737,'MRN' from dual union all
select 60738,'MRN' from dual union all
select 60739,'GMS' from dual union all
select 6074,'UNRV' from dual union all
select 60740,'GMS' from dual union all
select 60741,'GMS' from dual union all
select 60742,'MRN' from dual union all
select 60743,'MRN' from dual union all
select 60744,'GMS' from dual union all
select 60745,'AN' from dual union all
select 60746,'AN' from dual union all
select 60747,'GMS' from dual union all
select 60748,'MRN' from dual;
insert into LAUNCH_AGENCY
select 60749,'GMS' from dual union all
select 6075,'UNRV' from dual union all
select 60750,'AN' from dual union all
select 60751,'JMA' from dual union all
select 60752,'AN' from dual union all
select 60753,'GMS' from dual union all
select 60754,'JMA' from dual union all
select 60755,'GMS' from dual union all
select 60756,'AN' from dual union all
select 60757,'MRN' from dual union all
select 60758,'GMS' from dual union all
select 60759,'MRN' from dual union all
select 6076,'RVSN' from dual union all
select 60760,'MRN' from dual union all
select 60761,'MRN' from dual union all
select 60762,'GMS' from dual union all
select 60763,'JMA' from dual union all
select 60764,'AN' from dual union all
select 60765,'ISAS' from dual union all
select 60766,'ISAS' from dual union all
select 60767,'JMA' from dual union all
select 60768,'JMA' from dual union all
select 60769,'JMA' from dual union all
select 6077,'RVSN' from dual union all
select 60770,'JMA' from dual union all
select 60771,'MRN' from dual union all
select 60772,'AN' from dual union all
select 60773,'MRN' from dual union all
select 60774,'MRN' from dual union all
select 60775,'MRN' from dual union all
select 60776,'MRN' from dual union all
select 60777,'MRN' from dual union all
select 60778,'AN' from dual union all
select 60779,'MRN' from dual union all
select 6078,'RVSN' from dual union all
select 60780,'AN' from dual union all
select 60781,'JMA' from dual union all
select 60782,'MRN' from dual union all
select 60783,'MRN' from dual union all
select 60784,'MRN' from dual union all
select 60785,'MRN' from dual union all
select 60786,'JMA' from dual union all
select 60787,'MRN' from dual union all
select 60788,'MRN' from dual union all
select 60789,'MRN' from dual union all
select 6079,'UNRV' from dual union all
select 60790,'JMA' from dual union all
select 60791,'AN' from dual union all
select 60792,'MRN' from dual union all
select 60793,'MRN' from dual union all
select 60794,'GMS' from dual union all
select 60795,'MRN' from dual union all
select 60796,'GMS' from dual union all
select 60797,'AN' from dual union all
select 60798,'AN' from dual union all
select 60799,'MRN' from dual union all
select 608,'WEHR' from dual union all
select 6080,'NASA' from dual union all
select 60800,'MRN' from dual union all
select 60801,'GMS' from dual union all
select 60802,'AN' from dual union all
select 60803,'AN' from dual union all
select 60804,'JMA' from dual union all
select 60805,'MRN' from dual union all
select 60806,'MRN' from dual union all
select 60807,'JMA' from dual union all
select 60808,'GMS' from dual union all
select 60809,'MRN' from dual union all
select 6081,'UNRV' from dual union all
select 60810,'MRN' from dual union all
select 60811,'GMS' from dual union all
select 60812,'MRN' from dual union all
select 60813,'MRN' from dual union all
select 60814,'MRN' from dual union all
select 60815,'MRN' from dual union all
select 60816,'JMA' from dual union all
select 60817,'GMS' from dual union all
select 60818,'AN' from dual union all
select 60819,'MRN' from dual union all
select 6082,'UNRV' from dual union all
select 60820,'GMS' from dual union all
select 60821,'GMS' from dual union all
select 60822,'AN' from dual union all
select 60823,'MRN' from dual union all
select 60824,'MRN' from dual union all
select 60825,'MRN' from dual union all
select 60826,'MRN' from dual union all
select 60827,'GMS' from dual union all
select 60828,'MRN' from dual union all
select 60829,'JMA' from dual union all
select 6083,'UNRV' from dual union all
select 60830,'GMS' from dual union all
select 60831,'MRN' from dual union all
select 60832,'MRN' from dual union all
select 60833,'MRN' from dual union all
select 60834,'GMS' from dual union all
select 60835,'MRN' from dual union all
select 60836,'GMS' from dual union all
select 60837,'MRN' from dual union all
select 60838,'MRN' from dual;
insert into LAUNCH_AGENCY
select 60839,'AN' from dual union all
select 6084,'RVSN' from dual union all
select 60840,'AN' from dual union all
select 60841,'AN' from dual union all
select 60842,'AN' from dual union all
select 60843,'AN' from dual union all
select 60844,'AN' from dual union all
select 60845,'AN' from dual union all
select 60846,'AN' from dual union all
select 60847,'AN' from dual union all
select 60848,'AN' from dual union all
select 60849,'AN' from dual union all
select 6085,'RVSN' from dual union all
select 60850,'AN' from dual union all
select 60851,'AN' from dual union all
select 60852,'MRN' from dual union all
select 60853,'AN' from dual union all
select 60854,'AN' from dual union all
select 60855,'AN' from dual union all
select 60856,'AN' from dual union all
select 60857,'AN' from dual union all
select 60858,'AN' from dual union all
select 60859,'AN' from dual union all
select 6086,'RVSN' from dual union all
select 60860,'AN' from dual union all
select 60861,'AN' from dual union all
select 60862,'AN' from dual union all
select 60863,'AN' from dual union all
select 60864,'AN' from dual union all
select 60865,'AN' from dual union all
select 60866,'AN' from dual union all
select 60867,'AN' from dual union all
select 60868,'AN' from dual union all
select 60869,'AN' from dual union all
select 6087,'RVSN' from dual union all
select 60870,'AN' from dual union all
select 60871,'AN' from dual union all
select 60872,'AN' from dual union all
select 60873,'MRN' from dual union all
select 60874,'AN' from dual union all
select 60875,'AN' from dual union all
select 60876,'AN' from dual union all
select 60877,'AN' from dual union all
select 60878,'AN' from dual union all
select 60879,'AN' from dual union all
select 6088,'RVSN' from dual union all
select 60880,'AN' from dual union all
select 60881,'AN' from dual union all
select 60882,'AN' from dual union all
select 60883,'AN' from dual union all
select 60884,'AN' from dual union all
select 60885,'AN' from dual union all
select 60886,'AN' from dual union all
select 60887,'AN' from dual union all
select 60888,'AN' from dual union all
select 60889,'AN' from dual union all
select 6089,'RVSN' from dual union all
select 60890,'AN' from dual union all
select 60891,'AN' from dual union all
select 60892,'AN' from dual union all
select 60893,'AN' from dual union all
select 60894,'MRN' from dual union all
select 60895,'AN' from dual union all
select 60896,'MRN' from dual union all
select 60897,'AN' from dual union all
select 60898,'AN' from dual union all
select 60899,'AN' from dual union all
select 609,'WEHR' from dual union all
select 6090,'UNRV' from dual union all
select 60900,'AN' from dual union all
select 60901,'AN' from dual union all
select 60902,'AN' from dual union all
select 60903,'AN' from dual union all
select 60904,'AN' from dual union all
select 60905,'MRN' from dual union all
select 60906,'AN' from dual union all
select 60907,'AN' from dual union all
select 60908,'AN' from dual union all
select 60909,'AN' from dual union all
select 6091,'MVS' from dual union all
select 60910,'AN' from dual union all
select 60911,'MRN' from dual union all
select 60912,'AN' from dual union all
select 60913,'AN' from dual union all
select 60914,'AN' from dual union all
select 60915,'AN' from dual union all
select 60916,'AN' from dual union all
select 60917,'AN' from dual union all
select 60918,'AN' from dual union all
select 60919,'AN' from dual union all
select 6092,'UNRV' from dual union all
select 60920,'AN' from dual union all
select 60921,'AN' from dual union all
select 60922,'AN' from dual union all
select 60923,'AN' from dual union all
select 60924,'AN' from dual union all
select 60925,'AN' from dual union all
select 60926,'AN' from dual union all
select 60927,'AN' from dual union all
select 60928,'AN' from dual;
insert into LAUNCH_AGENCY
select 60929,'AN' from dual union all
select 6093,'UNRV' from dual union all
select 60930,'AN' from dual union all
select 60931,'AN' from dual union all
select 60932,'AN' from dual union all
select 60933,'AN' from dual union all
select 60934,'AN' from dual union all
select 60935,'AN' from dual union all
select 60936,'MRN' from dual union all
select 60937,'AN' from dual union all
select 60938,'MRN' from dual union all
select 60939,'AN' from dual union all
select 6094,'UNRV' from dual union all
select 60940,'AN' from dual union all
select 60941,'AN' from dual union all
select 60942,'AN' from dual union all
select 60943,'AN' from dual union all
select 60944,'AN' from dual union all
select 60945,'AN' from dual union all
select 60946,'AN' from dual union all
select 60947,'AN' from dual union all
select 60948,'AN' from dual union all
select 60949,'AN' from dual union all
select 6095,'UNRV' from dual union all
select 60950,'AN' from dual union all
select 60951,'MRN' from dual union all
select 60952,'AN' from dual union all
select 60953,'AN' from dual union all
select 60954,'AN' from dual union all
select 60955,'DLR' from dual union all
select 60956,'AN' from dual union all
select 60957,'AN' from dual union all
select 60958,'MRN' from dual union all
select 60959,'DLR' from dual union all
select 6096,'RVSN' from dual union all
select 60960,'DLR' from dual union all
select 60961,'DLR' from dual union all
select 60962,'DLR' from dual union all
select 60963,'DLR' from dual union all
select 60964,'AN' from dual union all
select 60965,'DLR' from dual union all
select 60966,'AN' from dual union all
select 60967,'DLR' from dual union all
select 60968,'AN' from dual union all
select 60969,'AN' from dual union all
select 6097,'RVSN' from dual union all
select 60970,'DLR' from dual union all
select 60971,'AN' from dual union all
select 60972,'AN' from dual union all
select 60973,'DLR' from dual union all
select 60974,'AN' from dual union all
select 60975,'AN' from dual union all
select 60976,'NASA' from dual union all
select 60977,'DLR' from dual union all
select 60978,'DLR' from dual union all
select 60979,'DLR' from dual union all
select 6098,'RVSN' from dual union all
select 60980,'AN' from dual union all
select 60981,'NASA' from dual union all
select 60982,'DLR' from dual union all
select 60983,'DLR' from dual union all
select 60984,'DLR' from dual union all
select 60985,'DLR' from dual union all
select 60986,'DLR' from dual union all
select 60987,'NASA' from dual union all
select 60988,'DLR' from dual union all
select 60989,'AN' from dual union all
select 6099,'RVSN' from dual union all
select 60990,'NASA' from dual union all
select 60991,'AN' from dual union all
select 60992,'DLR' from dual union all
select 60993,'DLR' from dual union all
select 60994,'AN' from dual union all
select 60995,'AN' from dual union all
select 60996,'DLR' from dual union all
select 60997,'DLR' from dual union all
select 60998,'DLR' from dual union all
select 60999,'DLR' from dual union all
select 61,'WER271-1' from dual union all
select 610,'WEHR' from dual union all
select 6100,'RVSN' from dual union all
select 61000,'DLR' from dual union all
select 61001,'DLR' from dual union all
select 61002,'MRN' from dual union all
select 61003,'DLR' from dual union all
select 61004,'DLR' from dual union all
select 61005,'AN' from dual union all
select 61006,'DLR' from dual union all
select 61007,'DLR' from dual union all
select 61008,'AN' from dual union all
select 61009,'AN' from dual union all
select 6101,'UNRV' from dual union all
select 61010,'AN' from dual union all
select 61011,'AN' from dual union all
select 61012,'AN' from dual union all
select 61013,'MRN' from dual union all
select 61014,'AN' from dual union all
select 61015,'AN' from dual union all
select 61016,'AN' from dual union all
select 61017,'AN' from dual;
insert into LAUNCH_AGENCY
select 61018,'AN' from dual union all
select 61019,'AN' from dual union all
select 6102,'USAF' from dual union all
select 61020,'AN' from dual union all
select 61021,'AN' from dual union all
select 61022,'AN' from dual union all
select 61023,'AN' from dual union all
select 61024,'AN' from dual union all
select 61025,'AN' from dual union all
select 61026,'AN' from dual union all
select 61027,'AN' from dual union all
select 61028,'AN' from dual union all
select 61029,'AN' from dual union all
select 6103,'UNRV' from dual union all
select 61030,'AN' from dual union all
select 61031,'AN' from dual union all
select 61032,'AN' from dual union all
select 61033,'AN' from dual union all
select 61034,'AN' from dual union all
select 61035,'AN' from dual union all
select 61036,'AN' from dual union all
select 61037,'AN' from dual union all
select 61038,'AN' from dual union all
select 61039,'AN' from dual union all
select 6104,'UNRV' from dual union all
select 61040,'AN' from dual union all
select 61041,'AN' from dual union all
select 61042,'AN' from dual union all
select 61043,'AN' from dual union all
select 61044,'AN' from dual union all
select 61045,'AN' from dual union all
select 61046,'AN' from dual union all
select 61047,'AN' from dual union all
select 61048,'AN' from dual union all
select 61049,'MRN' from dual union all
select 6105,'UNRV' from dual union all
select 61050,'AN' from dual union all
select 61051,'AN' from dual union all
select 61052,'AN' from dual union all
select 61053,'AN' from dual union all
select 61054,'AN' from dual union all
select 61055,'AN' from dual union all
select 61056,'AN' from dual union all
select 61057,'AN' from dual union all
select 61058,'AN' from dual union all
select 61059,'AN' from dual union all
select 6106,'RVSN' from dual union all
select 61060,'AN' from dual union all
select 61061,'AN' from dual union all
select 61062,'AN' from dual union all
select 61063,'AN' from dual union all
select 61064,'AN' from dual union all
select 61065,'AN' from dual union all
select 61066,'AN' from dual union all
select 61067,'AN' from dual union all
select 61068,'AN' from dual union all
select 61069,'AN' from dual union all
select 6107,'RVSN' from dual union all
select 61070,'AN' from dual union all
select 61071,'DLR' from dual union all
select 61072,'DLR' from dual union all
select 61073,'NASA' from dual union all
select 61074,'NASA' from dual union all
select 61075,'DLR' from dual union all
select 61076,'AN' from dual union all
select 61077,'AN' from dual union all
select 61078,'AN' from dual union all
select 61079,'AN' from dual union all
select 6108,'RVSN' from dual union all
select 61080,'AN' from dual union all
select 61081,'DLR' from dual union all
select 61082,'DLR' from dual union all
select 61083,'NASA' from dual union all
select 61084,'NASA' from dual union all
select 61085,'DLR' from dual union all
select 61086,'DLR' from dual union all
select 61087,'AN' from dual union all
select 61088,'AN' from dual union all
select 61089,'AN' from dual union all
select 6109,'RVSN' from dual union all
select 61090,'AN' from dual union all
select 61091,'AN' from dual union all
select 61092,'AN' from dual union all
select 61093,'AN' from dual union all
select 61094,'AN' from dual union all
select 61095,'NASA' from dual union all
select 61096,'NASA' from dual union all
select 61097,'AN' from dual union all
select 61098,'AN' from dual union all
select 61099,'AN' from dual union all
select 611,'WEHR' from dual union all
select 6110,'RVSN' from dual union all
select 61100,'AN' from dual union all
select 61101,'AN' from dual union all
select 61102,'AN' from dual union all
select 61103,'DLR' from dual union all
select 61104,'AN' from dual union all
select 61105,'AN' from dual union all
select 61106,'AN' from dual union all
select 61107,'AN' from dual;
insert into LAUNCH_AGENCY
select 61108,'AN' from dual union all
select 61109,'AN' from dual union all
select 6111,'RVSN' from dual union all
select 61110,'AN' from dual union all
select 61111,'AN' from dual union all
select 61112,'AN' from dual union all
select 61113,'AN' from dual union all
select 61114,'AN' from dual union all
select 61115,'AN' from dual union all
select 61116,'DLR' from dual union all
select 61117,'DLR' from dual union all
select 61118,'NASA' from dual union all
select 61119,'NASA' from dual union all
select 6112,'RVSN' from dual union all
select 61120,'AN' from dual union all
select 61121,'AN' from dual union all
select 61122,'AN' from dual union all
select 61123,'AN' from dual union all
select 61124,'AN' from dual union all
select 61125,'AN' from dual union all
select 61126,'AN' from dual union all
select 61127,'AN' from dual union all
select 61128,'AN' from dual union all
select 61129,'AN' from dual union all
select 6113,'USAF' from dual union all
select 61130,'AN' from dual union all
select 61131,'AN' from dual union all
select 61132,'AN' from dual union all
select 61133,'AN' from dual union all
select 61134,'AN' from dual union all
select 61135,'AN' from dual union all
select 61136,'AN' from dual union all
select 61137,'AN' from dual union all
select 61138,'AN' from dual union all
select 61139,'AN' from dual union all
select 6114,'RVSN' from dual union all
select 61140,'AN' from dual union all
select 61141,'AN' from dual union all
select 61142,'AN' from dual union all
select 61143,'AN' from dual union all
select 61144,'AN' from dual union all
select 61145,'AN' from dual union all
select 61146,'AN' from dual union all
select 61147,'AN' from dual union all
select 61148,'AN' from dual union all
select 61149,'AN' from dual union all
select 6115,'UNRV' from dual union all
select 61150,'AN' from dual union all
select 61151,'AN' from dual union all
select 61152,'AN' from dual union all
select 61153,'AN' from dual union all
select 61154,'AN' from dual union all
select 61155,'AN' from dual union all
select 61156,'AN' from dual union all
select 61157,'AN' from dual union all
select 61158,'AN' from dual union all
select 61159,'AN' from dual union all
select 6116,'UNRV' from dual union all
select 61160,'AN' from dual union all
select 61161,'AN' from dual union all
select 61162,'AN' from dual union all
select 61163,'AN' from dual union all
select 61164,'AN' from dual union all
select 61165,'AN' from dual union all
select 61166,'AN' from dual union all
select 61167,'DLR' from dual union all
select 61168,'DLR' from dual union all
select 61169,'DLR' from dual union all
select 6117,'UNRV' from dual union all
select 61170,'DLR' from dual union all
select 61171,'DLR' from dual union all
select 61172,'DLR' from dual union all
select 61173,'DLR' from dual union all
select 61174,'DLR' from dual union all
select 61175,'DLR' from dual union all
select 61176,'AN' from dual union all
select 61177,'DLR' from dual union all
select 61178,'DLR' from dual union all
select 61179,'DLR' from dual union all
select 6118,'UNRV' from dual union all
select 61180,'DLR' from dual union all
select 61181,'DLR' from dual union all
select 61182,'DLR' from dual union all
select 61183,'DLR' from dual union all
select 61184,'DLR' from dual union all
select 61185,'DLR' from dual union all
select 61186,'DLR' from dual union all
select 61187,'AN' from dual union all
select 61188,'DLR' from dual union all
select 61189,'DLR' from dual union all
select 6119,'RVSN' from dual union all
select 61190,'DLR' from dual union all
select 61191,'DLR' from dual union all
select 61192,'AN' from dual union all
select 61193,'AN' from dual union all
select 61194,'AN' from dual union all
select 61195,'AN' from dual union all
select 61196,'AN' from dual union all
select 61197,'AN' from dual union all
select 61198,'AN' from dual;
insert into LAUNCH_AGENCY
select 61199,'AN' from dual union all
select 612,'WEHR' from dual union all
select 6120,'RVSN' from dual union all
select 61200,'AN' from dual union all
select 61201,'AN' from dual union all
select 61202,'AN' from dual union all
select 61203,'AN' from dual union all
select 61204,'AN' from dual union all
select 61205,'AN' from dual union all
select 61206,'AN' from dual union all
select 61207,'AN' from dual union all
select 61208,'AN' from dual union all
select 61209,'AN' from dual union all
select 6121,'RVSN' from dual union all
select 61210,'AN' from dual union all
select 61211,'AN' from dual union all
select 61212,'AN' from dual union all
select 61213,'AN' from dual union all
select 61214,'AN' from dual union all
select 61215,'AN' from dual union all
select 61216,'AN' from dual union all
select 61217,'AN' from dual union all
select 61218,'AN' from dual union all
select 61219,'AN' from dual union all
select 6122,'RVSN' from dual union all
select 61220,'AN' from dual union all
select 61221,'AN' from dual union all
select 61222,'AN' from dual union all
select 61223,'AN' from dual union all
select 61224,'AN' from dual union all
select 61225,'DLR' from dual union all
select 61226,'AN' from dual union all
select 61227,'AN' from dual union all
select 61228,'AN' from dual union all
select 61229,'AN' from dual union all
select 6123,'RVSN' from dual union all
select 61230,'AN' from dual union all
select 61231,'AN' from dual union all
select 61232,'AN' from dual union all
select 61233,'AN' from dual union all
select 61234,'AN' from dual union all
select 61235,'AN' from dual union all
select 61236,'AN' from dual union all
select 61237,'AN' from dual union all
select 61238,'AN' from dual union all
select 61239,'AN' from dual union all
select 6124,'USAF' from dual union all
select 61240,'AN' from dual union all
select 61241,'AN' from dual union all
select 61242,'AN' from dual union all
select 61243,'AN' from dual union all
select 61244,'AN' from dual union all
select 61245,'AN' from dual union all
select 61246,'AN' from dual union all
select 61247,'AN' from dual union all
select 61248,'AN' from dual union all
select 61249,'AN' from dual union all
select 6125,'RVSN' from dual union all
select 61250,'UNKS' from dual union all
select 61251,'UNKS' from dual union all
select 61252,'UNKS' from dual union all
select 61253,'UNKS' from dual union all
select 61254,'UNKS' from dual union all
select 61255,'AFSC' from dual union all
select 61256,'UNKS' from dual union all
select 61257,'NASA' from dual union all
select 61258,'UNKS' from dual union all
select 61259,'UNKS' from dual union all
select 6126,'RVSN' from dual union all
select 61260,'UNKS' from dual union all
select 61261,'NASDA' from dual union all
select 61262,'UNKS' from dual union all
select 61263,'CALT' from dual union all
select 61264,'UNKS' from dual union all
select 61265,'UNKS' from dual union all
select 61266,'UNKS' from dual union all
select 61267,'AE' from dual union all
select 61268,'UNKS' from dual union all
select 61269,'UNKS' from dual union all
select 6127,'RVSN' from dual union all
select 61270,'UNKS' from dual union all
select 61271,'UNKS' from dual union all
select 61272,'UNKS' from dual union all
select 61273,'UNKS' from dual union all
select 61274,'UNKS' from dual union all
select 61275,'CRA' from dual union all
select 61276,'UNKS' from dual union all
select 61277,'UNKS' from dual union all
select 61278,'UNKS' from dual union all
select 61279,'UNKS' from dual union all
select 6128,'RVSN' from dual union all
select 61280,'UNKS' from dual union all
select 61281,'UNKS' from dual union all
select 61282,'NASA' from dual union all
select 61283,'UNKS' from dual union all
select 61284,'UNKS' from dual union all
select 61285,'UNKS' from dual union all
select 61286,'UNKS' from dual union all
select 61287,'UNKS' from dual union all
select 61288,'UNKS' from dual;
insert into LAUNCH_AGENCY
select 61289,'AE' from dual union all
select 6129,'UNRV' from dual union all
select 61290,'UNKS' from dual union all
select 61291,'UNKS' from dual union all
select 61292,'UNKS' from dual union all
select 61293,'UNKS' from dual union all
select 61294,'UNKS' from dual union all
select 61295,'UNKS' from dual union all
select 61296,'UNKS' from dual union all
select 61297,'UNKS' from dual union all
select 61298,'UNKS' from dual union all
select 61299,'UNKS' from dual union all
select 613,'WEHR' from dual union all
select 6130,'UNRV' from dual union all
select 61300,'AE' from dual union all
select 61301,'NASA' from dual union all
select 61302,'UNKS' from dual union all
select 61303,'UNKS' from dual union all
select 61304,'UNKS' from dual union all
select 61305,'UNKS' from dual union all
select 61306,'UNKS' from dual union all
select 61307,'UNKS' from dual union all
select 61308,'UNKS' from dual union all
select 61309,'UNKS' from dual union all
select 6131,'UNRV' from dual union all
select 61310,'UNKS' from dual union all
select 61311,'UNKS' from dual union all
select 61312,'AE' from dual union all
select 61313,'UNKS' from dual union all
select 61314,'UNKS' from dual union all
select 61315,'UNKS' from dual union all
select 61316,'CALT' from dual union all
select 61317,'UNKS' from dual union all
select 61318,'UNKS' from dual union all
select 61319,'UNKS' from dual union all
select 6132,'UNRV' from dual union all
select 61320,'UNKS' from dual union all
select 61321,'UNKS' from dual union all
select 61322,'UNKS' from dual union all
select 61323,'NASA' from dual union all
select 61324,'UNKS' from dual union all
select 61325,'UNKS' from dual union all
select 61326,'AFSC' from dual union all
select 61327,'AFSC' from dual union all
select 61328,'UNKS' from dual union all
select 61329,'SAST' from dual union all
select 6133,'UNRV' from dual union all
select 61330,'AE' from dual union all
select 61331,'UNKS' from dual union all
select 61332,'UNKS' from dual union all
select 61333,'UNKS' from dual union all
select 61334,'UNKS' from dual union all
select 61335,'NASDA' from dual union all
select 61336,'ISA' from dual union all
select 61337,'UNKS' from dual union all
select 61338,'AFSC' from dual union all
select 61339,'UNKS' from dual union all
select 6134,'UNRV' from dual union all
select 61340,'NASA' from dual union all
select 61341,'UNKS' from dual union all
select 61342,'UNKS' from dual union all
select 61343,'UNKS' from dual union all
select 61344,'UNKS' from dual union all
select 61345,'UNKS' from dual union all
select 61346,'UNKS' from dual union all
select 61347,'AE' from dual union all
select 61348,'AFSC' from dual union all
select 61349,'UNKS' from dual union all
select 6135,'MVS' from dual union all
select 61350,'UNKS' from dual union all
select 61351,'UNKS' from dual union all
select 61352,'UNKS' from dual union all
select 61353,'UNKS' from dual union all
select 61354,'UNKS' from dual union all
select 61355,'NASA' from dual union all
select 61356,'UNKS' from dual union all
select 61357,'UNKS' from dual union all
select 61358,'AE' from dual union all
select 61359,'UNKS' from dual union all
select 6136,'UNRV' from dual union all
select 61360,'CALT' from dual union all
select 61361,'UNKS' from dual union all
select 61362,'UNKS' from dual union all
select 61363,'UNKS' from dual union all
select 61364,'UNKS' from dual union all
select 61365,'UNKS' from dual union all
select 61366,'MRN' from dual union all
select 61367,'MRN' from dual union all
select 61368,'USN' from dual union all
select 61369,'RVSN' from dual union all
select 6137,'UNRV' from dual union all
select 61370,'MRN' from dual union all
select 61371,'USN' from dual union all
select 61372,'USN' from dual union all
select 61373,'GMS' from dual union all
select 61374,'GMS' from dual union all
select 61375,'SAND' from dual union all
select 61376,'RVSN' from dual union all
select 61377,'RVSN' from dual union all
select 61378,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 61379,'RVSN' from dual union all
select 6138,'UNRV' from dual union all
select 61380,'RVSN' from dual union all
select 61381,'ISAS' from dual union all
select 61382,'RVSN' from dual union all
select 61383,'RVSN' from dual union all
select 61384,'RVSN' from dual union all
select 61385,'RVSN' from dual union all
select 61386,'RVSN' from dual union all
select 61387,'RVSN' from dual union all
select 61388,'RVSN' from dual union all
select 61389,'RVSN' from dual union all
select 6139,'UNRV' from dual union all
select 61390,'RVSN' from dual union all
select 61391,'RVSN' from dual union all
select 61392,'RVSN' from dual union all
select 61393,'RVSN' from dual union all
select 61394,'RVSN' from dual union all
select 61395,'RVSN' from dual union all
select 61396,'RVSN' from dual union all
select 61397,'RVSN' from dual union all
select 61398,'RVSN' from dual union all
select 61399,'RVSN' from dual union all
select 614,'WEHR' from dual union all
select 6140,'RVSN' from dual union all
select 61400,'RVSN' from dual union all
select 61401,'RVSN' from dual union all
select 61402,'RVSN' from dual union all
select 61403,'RVSN' from dual union all
select 61404,'RVSN' from dual union all
select 61405,'RVSN' from dual union all
select 61406,'RVSN' from dual union all
select 61407,'RVSN' from dual union all
select 61408,'RVSN' from dual union all
select 61409,'RVSN' from dual union all
select 6141,'UNRV' from dual union all
select 61410,'RVSN' from dual union all
select 61411,'RVSN' from dual union all
select 61412,'RVSN' from dual union all
select 61413,'RVSN' from dual union all
select 61414,'RVSN' from dual union all
select 61415,'RVSN' from dual union all
select 61416,'RVSN' from dual union all
select 61417,'RVSN' from dual union all
select 61418,'RVSN' from dual union all
select 61419,'RVSN' from dual union all
select 6142,'UNRV' from dual union all
select 61420,'RVSN' from dual union all
select 61421,'RVSN' from dual union all
select 61422,'RVSN' from dual union all
select 61423,'RVSN' from dual union all
select 61424,'RVSN' from dual union all
select 61425,'RVSN' from dual union all
select 61426,'RVSN' from dual union all
select 61427,'RVSN' from dual union all
select 61428,'RVSN' from dual union all
select 61429,'RVSN' from dual union all
select 6143,'UNRV' from dual union all
select 61430,'RVSN' from dual union all
select 61431,'RVSN' from dual union all
select 61432,'RVSN' from dual union all
select 61433,'RVSN' from dual union all
select 61434,'RVSN' from dual union all
select 61435,'RVSN' from dual union all
select 61436,'RVSN' from dual union all
select 61437,'RVSN' from dual union all
select 61438,'RVSN' from dual union all
select 61439,'RVSN' from dual union all
select 6144,'UNRV' from dual union all
select 61440,'RVSN' from dual union all
select 61441,'RVSN' from dual union all
select 61442,'RVSN' from dual union all
select 61443,'RVSN' from dual union all
select 61444,'RVSN' from dual union all
select 61445,'RVSN' from dual union all
select 61446,'RVSN' from dual union all
select 61447,'RVSN' from dual union all
select 61448,'RVSN' from dual union all
select 61449,'RVSN' from dual union all
select 6145,'UNRV' from dual union all
select 61450,'UNKS' from dual union all
select 61451,'UNKS' from dual union all
select 61452,'ISRO' from dual union all
select 61453,'UNKS' from dual union all
select 61454,'UNKS' from dual union all
select 61455,'USN' from dual union all
select 61456,'NASDA' from dual union all
select 61457,'CSSAR' from dual union all
select 61458,'CSSAR' from dual union all
select 61459,'CSSAR' from dual union all
select 6146,'USAF' from dual union all
select 61460,'CSSAR' from dual union all
select 61461,'ISRO' from dual union all
select 61462,'ISRO' from dual union all
select 61463,'ISRO' from dual union all
select 61464,'ISRO' from dual union all
select 61465,'ISRO' from dual union all
select 61466,'ISRO' from dual union all
select 61467,'ISRO' from dual union all
select 61468,'ISRO' from dual;
insert into LAUNCH_AGENCY
select 61469,'ISRO' from dual union all
select 6147,'UNRV' from dual union all
select 61470,'ISRO' from dual union all
select 61471,'ISRO' from dual union all
select 61472,'ISRO' from dual union all
select 61473,'ISRO' from dual union all
select 61474,'ISRO' from dual union all
select 61475,'ISRO' from dual union all
select 61476,'ISRO' from dual union all
select 61477,'ISRO' from dual union all
select 61478,'ISRO' from dual union all
select 61479,'ISRO' from dual union all
select 6148,'UNRV' from dual union all
select 61480,'ISRO' from dual union all
select 61481,'ISRO' from dual union all
select 61482,'ISRO' from dual union all
select 61483,'ISRO' from dual union all
select 61484,'ISRO' from dual union all
select 61485,'ISRO' from dual union all
select 61486,'ISRO' from dual union all
select 61487,'ISRO' from dual union all
select 61488,'ISRO' from dual union all
select 61489,'ISRO' from dual union all
select 6149,'UNRV' from dual union all
select 61490,'ISRO' from dual union all
select 61491,'ISRO' from dual union all
select 61492,'ISRO' from dual union all
select 61493,'ISRO' from dual union all
select 61494,'ISRO' from dual union all
select 61495,'ISRO' from dual union all
select 61496,'ISRO' from dual union all
select 61497,'ISRO' from dual union all
select 61498,'ISRO' from dual union all
select 61499,'ISRO' from dual union all
select 615,'WEHR' from dual union all
select 6150,'UNRV' from dual union all
select 61500,'ISRO' from dual union all
select 61501,'ISRO' from dual union all
select 61502,'ISRO' from dual union all
select 61503,'ISRO' from dual union all
select 61504,'ISRO' from dual union all
select 61505,'ISRO' from dual union all
select 61506,'ISRO' from dual union all
select 61507,'WFF' from dual union all
select 61508,'WFF' from dual union all
select 61509,'WFF' from dual union all
select 6151,'UNRV' from dual union all
select 61510,'WFF' from dual union all
select 61511,'ISRO' from dual union all
select 61512,'ISRO' from dual union all
select 61513,'ISRO' from dual union all
select 61514,'ISRO' from dual union all
select 61515,'ISRO' from dual union all
select 61516,'ISRO' from dual union all
select 61517,'ISRO' from dual union all
select 61518,'ISRO' from dual union all
select 61519,'ISRO' from dual union all
select 6152,'UNRV' from dual union all
select 61520,'ISRO' from dual union all
select 61521,'ISRO' from dual union all
select 61522,'ISRO' from dual union all
select 61523,'ISRO' from dual union all
select 61524,'ISRO' from dual union all
select 61525,'ISRO' from dual union all
select 61526,'ISRO' from dual union all
select 61527,'ISRO' from dual union all
select 61528,'ISRO' from dual union all
select 61529,'ISRO' from dual union all
select 6153,'UNRV' from dual union all
select 61530,'ISRO' from dual union all
select 61531,'ISRO' from dual union all
select 61532,'ISRO' from dual union all
select 61533,'ISRO' from dual union all
select 61534,'ISRO' from dual union all
select 61535,'ISRO' from dual union all
select 61536,'ISRO' from dual union all
select 61537,'ISRO' from dual union all
select 61538,'ISRO' from dual union all
select 61539,'ISRO' from dual union all
select 6154,'UNRV' from dual union all
select 61540,'ISRO' from dual union all
select 61541,'ISRO' from dual union all
select 61542,'ISRO' from dual union all
select 61543,'ISRO' from dual union all
select 61544,'ISRO' from dual union all
select 61545,'ISRO' from dual union all
select 61546,'ISRO' from dual union all
select 61547,'ISRO' from dual union all
select 61548,'ISRO' from dual union all
select 61549,'ISRO' from dual union all
select 6155,'UNRV' from dual union all
select 61550,'ISRO' from dual union all
select 61551,'ISRO' from dual union all
select 61552,'ISRO' from dual union all
select 61553,'ISRO' from dual union all
select 61554,'ISRO' from dual union all
select 61555,'ISRO' from dual union all
select 61556,'ISRO' from dual union all
select 61557,'PLA2AC' from dual union all
select 61558,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 61559,'RVSN' from dual union all
select 6156,'UNRV' from dual union all
select 61560,'RVSN' from dual union all
select 61561,'RVSN' from dual union all
select 61562,'RVSN' from dual union all
select 61563,'RVSN' from dual union all
select 61564,'RVSN' from dual union all
select 61565,'RVSN' from dual union all
select 61566,'RVSN' from dual union all
select 61567,'NASA' from dual union all
select 61568,'AN' from dual union all
select 61569,'AN' from dual union all
select 6157,'MVS' from dual union all
select 61570,'ALCS' from dual union all
select 61570,'SAC' from dual union all
select 61571,'SAC' from dual union all
select 61572,'NASA' from dual union all
select 61573,'NASA' from dual union all
select 61574,'SDIO' from dual union all
select 61575,'AN' from dual union all
select 61576,'AN' from dual union all
select 61577,'RVSN' from dual union all
select 61578,'AN' from dual union all
select 61579,'USN' from dual union all
select 6158,'UNRV' from dual union all
select 61580,'AN' from dual union all
select 61581,'AN' from dual union all
select 61582,'AN' from dual union all
select 61583,'DFVLR' from dual union all
select 61584,'NASA' from dual union all
select 61585,'NASA' from dual union all
select 61586,'NASA' from dual union all
select 61587,'RVSN' from dual union all
select 61588,'RVSN' from dual union all
select 61589,'PLA2AC' from dual union all
select 6159,'UNRV' from dual union all
select 61590,'AN' from dual union all
select 61591,'AN' from dual union all
select 61592,'AN' from dual union all
select 61593,'AN' from dual union all
select 61594,'NASA' from dual union all
select 61595,'ISRO' from dual union all
select 61596,'AFBMD' from dual union all
select 61597,'NASA' from dual union all
select 61598,'NASA' from dual union all
select 61599,'RVSN' from dual union all
select 616,'WEHR' from dual union all
select 6160,'UNRV' from dual union all
select 61600,'ALCS' from dual union all
select 61600,'SAC' from dual union all
select 61601,'AN' from dual union all
select 61602,'AN' from dual union all
select 61603,'USN' from dual union all
select 61604,'NASA' from dual union all
select 61605,'NASA' from dual union all
select 61606,'NASA' from dual union all
select 61607,'DGA' from dual union all
select 61608,'DFVLR' from dual union all
select 61609,'DFVLR' from dual union all
select 6161,'UNRV' from dual union all
select 61610,'RVSN' from dual union all
select 61611,'AN' from dual union all
select 61612,'AN' from dual union all
select 61613,'DFVLR' from dual union all
select 61614,'DFVLR' from dual union all
select 61615,'DLR' from dual union all
select 61616,'NASA' from dual union all
select 61617,'USN' from dual union all
select 61618,'NASA' from dual union all
select 61619,'IDF' from dual union all
select 6162,'UNRV' from dual union all
select 61620,'IQ' from dual union all
select 61621,'RVSN' from dual union all
select 61622,'IQ' from dual union all
select 61623,'IQ' from dual union all
select 61624,'IQ' from dual union all
select 61625,'IQ' from dual union all
select 61626,'IQ' from dual union all
select 61627,'IQ' from dual union all
select 61628,'IQ' from dual union all
select 61629,'IQ' from dual union all
select 6163,'UNRV' from dual union all
select 61630,'IQ' from dual union all
select 61631,'IQ' from dual union all
select 61632,'RVSN' from dual union all
select 61633,'US7A' from dual union all
select 61634,'US7A' from dual union all
select 61635,'US7A' from dual union all
select 61636,'US7A' from dual union all
select 61637,'US7A' from dual union all
select 61638,'FKG2' from dual union all
select 61639,'FKG2' from dual union all
select 6164,'UNRV' from dual union all
select 61640,'FKG1' from dual union all
select 61641,'FKG1' from dual union all
select 61642,'RVSN' from dual union all
select 61643,'9FA3' from dual union all
select 61644,'RVSN' from dual union all
select 61645,'RVSN' from dual union all
select 61646,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 61647,'RVSN' from dual union all
select 61648,'RVSN' from dual union all
select 61649,'RVSN' from dual union all
select 6165,'RVSN' from dual union all
select 61650,'RVSN' from dual union all
select 61651,'RVSN' from dual union all
select 61652,'RVSN' from dual union all
select 61653,'RVSN' from dual union all
select 61654,'RVSN' from dual union all
select 61655,'RVSN' from dual union all
select 61656,'RVSN' from dual union all
select 61657,'9FA3' from dual union all
select 61658,'9FA3' from dual union all
select 61659,'US7A' from dual union all
select 6166,'RVSN' from dual union all
select 61660,'RVSN' from dual union all
select 61661,'RVSN' from dual union all
select 61662,'RVSN' from dual union all
select 61663,'RVSN' from dual union all
select 61664,'RVSN' from dual union all
select 61665,'RVSN' from dual union all
select 61666,'RVSN' from dual union all
select 61667,'RVSN' from dual union all
select 61668,'RVSN' from dual union all
select 61669,'RVSN' from dual union all
select 6167,'RVSN' from dual union all
select 61670,'RVSN' from dual union all
select 61671,'RVSN' from dual union all
select 61672,'RVSN' from dual union all
select 61673,'RVSN' from dual union all
select 61674,'RVSN' from dual union all
select 61675,'RVSN' from dual union all
select 61676,'RVSN' from dual union all
select 61677,'RVSN' from dual union all
select 61678,'RVSN' from dual union all
select 61679,'RVSN' from dual union all
select 6168,'USAF' from dual union all
select 61680,'RVSN' from dual union all
select 61681,'RVSN' from dual union all
select 61682,'AFBMD' from dual union all
select 61683,'RVSN' from dual union all
select 61684,'9FA3' from dual union all
select 61685,'AFSC' from dual union all
select 61686,'NASA' from dual union all
select 61687,'NASA' from dual union all
select 61688,'ISAS' from dual union all
select 61689,'ISAS' from dual union all
select 6169,'RVSN' from dual union all
select 61690,'ALCS' from dual union all
select 61690,'SAC' from dual union all
select 61691,'NASA' from dual union all
select 61692,'SDIO' from dual union all
select 61693,'NASA' from dual union all
select 61694,'US7A' from dual union all
select 61695,'US7A' from dual union all
select 61696,'US7A' from dual union all
select 61697,'US7A' from dual union all
select 61698,'US7A' from dual union all
select 61699,'US7A' from dual union all
select 617,'WEHR' from dual union all
select 6170,'RVSN' from dual union all
select 61700,'NASA' from dual union all
select 61701,'IDRDL' from dual union all
select 61702,'USN' from dual union all
select 61703,'NASA' from dual union all
select 61704,'SSC' from dual union all
select 61705,'NASA' from dual union all
select 61706,'AFBMD' from dual union all
select 61707,'NASA' from dual union all
select 61708,'NASA' from dual union all
select 61709,'US7A' from dual union all
select 6171,'RVSN' from dual union all
select 61710,'US7A' from dual union all
select 61711,'US7A' from dual union all
select 61712,'US7A' from dual union all
select 61713,'US7A' from dual union all
select 61714,'AN' from dual union all
select 61715,'USN' from dual union all
select 61716,'USN' from dual union all
select 61717,'USN' from dual union all
select 61718,'USN' from dual union all
select 61719,'USN' from dual union all
select 6172,'RVSN' from dual union all
select 61720,'NASA' from dual union all
select 61720,'SSC' from dual union all
select 61721,'USN' from dual union all
select 61722,'USN' from dual union all
select 61723,'USN' from dual union all
select 61724,'USN' from dual union all
select 61725,'USN' from dual union all
select 61726,'AN' from dual union all
select 61727,'AN' from dual union all
select 61728,'AN' from dual union all
select 61729,'AN' from dual union all
select 6173,'UNRV' from dual union all
select 61730,'AN' from dual union all
select 61731,'MBB' from dual union all
select 61732,'NASA' from dual union all
select 61733,'MBB' from dual union all
select 61734,'DFVLR' from dual;
insert into LAUNCH_AGENCY
select 61735,'NASA' from dual union all
select 61736,'USN' from dual union all
select 61737,'USN' from dual union all
select 61738,'AN' from dual union all
select 61739,'SAC' from dual union all
select 6174,'UNRV' from dual union all
select 61740,'UNKS' from dual union all
select 61741,'JMA' from dual union all
select 61742,'MRN' from dual union all
select 61743,'MRN' from dual union all
select 61744,'AN' from dual union all
select 61745,'AN' from dual union all
select 61746,'AN' from dual union all
select 61747,'AN' from dual union all
select 61748,'AN' from dual union all
select 61749,'JMA' from dual union all
select 6175,'UNRV' from dual union all
select 61750,'MRN' from dual union all
select 61751,'AN' from dual union all
select 61752,'AN' from dual union all
select 61753,'AN' from dual union all
select 61754,'AN' from dual union all
select 61755,'MRN' from dual union all
select 61756,'JMA' from dual union all
select 61757,'MRN' from dual union all
select 61758,'AN' from dual union all
select 61759,'AN' from dual union all
select 6176,'UNRV' from dual union all
select 61760,'MRN' from dual union all
select 61761,'AN' from dual union all
select 61762,'MRN' from dual union all
select 61763,'JMA' from dual union all
select 61764,'GMS' from dual union all
select 61765,'GMS' from dual union all
select 61766,'GMS' from dual union all
select 61767,'JMA' from dual union all
select 61768,'GMS' from dual union all
select 61769,'GMS' from dual union all
select 6177,'UNRV' from dual union all
select 61770,'MRN' from dual union all
select 61771,'MRN' from dual union all
select 61772,'AN' from dual union all
select 61773,'MRN' from dual union all
select 61774,'JMA' from dual union all
select 61775,'GMS' from dual union all
select 61776,'GMS' from dual union all
select 61777,'MRN' from dual union all
select 61778,'MRN' from dual union all
select 61779,'GMS' from dual union all
select 6178,'UNRV' from dual union all
select 61780,'MRN' from dual union all
select 61781,'GMS' from dual union all
select 61782,'MRN' from dual union all
select 61783,'MRN' from dual union all
select 61784,'GMS' from dual union all
select 61785,'AN' from dual union all
select 61786,'JMA' from dual union all
select 61787,'AN' from dual union all
select 61788,'MRN' from dual union all
select 61789,'AN' from dual union all
select 6179,'NRL' from dual union all
select 61790,'JMA' from dual union all
select 61791,'MRN' from dual union all
select 61792,'JMA' from dual union all
select 61793,'AN' from dual union all
select 61794,'MRN' from dual union all
select 61795,'AN' from dual union all
select 61796,'JMA' from dual union all
select 61797,'MRN' from dual union all
select 61798,'AN' from dual union all
select 61799,'MRN' from dual union all
select 618,'FIAN' from dual union all
select 6180,'UNRV' from dual union all
select 61800,'AN' from dual union all
select 61801,'JMA' from dual union all
select 61802,'MRN' from dual union all
select 61803,'AN' from dual union all
select 61804,'MRN' from dual union all
select 61805,'AN' from dual union all
select 61806,'MRN' from dual union all
select 61807,'GMS' from dual union all
select 61808,'JMA' from dual union all
select 61809,'GMS' from dual union all
select 6181,'UNRV' from dual union all
select 61810,'MRN' from dual union all
select 61811,'GMS' from dual union all
select 61812,'MRN' from dual union all
select 61813,'GMS' from dual union all
select 61814,'GMS' from dual union all
select 61815,'GMS' from dual union all
select 61816,'AN' from dual union all
select 61817,'AN' from dual union all
select 61818,'MRN' from dual union all
select 61819,'MRN' from dual union all
select 6182,'UNRV' from dual union all
select 61820,'GMS' from dual union all
select 61821,'GMS' from dual union all
select 61822,'MRN' from dual union all
select 61823,'AN' from dual union all
select 61824,'MRN' from dual;
insert into LAUNCH_AGENCY
select 61825,'GMS' from dual union all
select 61826,'MRN' from dual union all
select 61827,'MRN' from dual union all
select 61828,'AN' from dual union all
select 61829,'MRN' from dual union all
select 6183,'UNRV' from dual union all
select 61830,'JMA' from dual union all
select 61831,'MRN' from dual union all
select 61832,'MRN' from dual union all
select 61833,'MRN' from dual union all
select 61834,'MRN' from dual union all
select 61835,'MRN' from dual union all
select 61836,'MRN' from dual union all
select 61837,'MRN' from dual union all
select 61838,'AN' from dual union all
select 61839,'MRN' from dual union all
select 6184,'UNRV' from dual union all
select 61840,'MRN' from dual union all
select 61841,'JMA' from dual union all
select 61842,'MRN' from dual union all
select 61843,'JMA' from dual union all
select 61844,'GMS' from dual union all
select 61845,'MRN' from dual union all
select 61846,'GMS' from dual union all
select 61847,'GMS' from dual union all
select 61848,'GMS' from dual union all
select 61849,'AN' from dual union all
select 6185,'UNRV' from dual union all
select 61850,'GMS' from dual union all
select 61851,'JMA' from dual union all
select 61852,'GMS' from dual union all
select 61853,'GMS' from dual union all
select 61854,'GMS' from dual union all
select 61855,'GMS' from dual union all
select 61856,'AN' from dual union all
select 61857,'JMA' from dual union all
select 61858,'GMS' from dual union all
select 61859,'GMS' from dual union all
select 6186,'UNRV' from dual union all
select 61860,'AN' from dual union all
select 61861,'GMS' from dual union all
select 61862,'GMS' from dual union all
select 61863,'AN' from dual union all
select 61864,'GMS' from dual union all
select 61865,'GMS' from dual union all
select 61866,'MRN' from dual union all
select 61867,'JMA' from dual union all
select 61868,'MRN' from dual union all
select 61869,'MRN' from dual union all
select 6187,'UNRV' from dual union all
select 61870,'MRN' from dual union all
select 61871,'MRN' from dual union all
select 61872,'MRN' from dual union all
select 61873,'MRN' from dual union all
select 61874,'MRN' from dual union all
select 61875,'MRN' from dual union all
select 61876,'JMA' from dual union all
select 61877,'AN' from dual union all
select 61878,'JMA' from dual union all
select 61879,'AN' from dual union all
select 6188,'UNRV' from dual union all
select 61880,'MRN' from dual union all
select 61881,'GMS' from dual union all
select 61882,'AN' from dual union all
select 61883,'GMS' from dual union all
select 61884,'GMS' from dual union all
select 61885,'JMA' from dual union all
select 61886,'GMS' from dual union all
select 61887,'GMS' from dual union all
select 61888,'GMS' from dual union all
select 61889,'GMS' from dual union all
select 6189,'UNRV' from dual union all
select 61890,'GMS' from dual union all
select 61891,'JMA' from dual union all
select 61892,'GMS' from dual union all
select 61893,'MRN' from dual union all
select 61894,'AN' from dual union all
select 61895,'GMS' from dual union all
select 61896,'ISAS' from dual union all
select 61897,'JMA' from dual union all
select 61898,'JMA' from dual union all
select 61899,'JMA' from dual union all
select 619,'US' from dual union all
select 619,'WSPG' from dual union all
select 6190,'USAF' from dual union all
select 61900,'JMA' from dual union all
select 61901,'JMA' from dual union all
select 61902,'JMA' from dual union all
select 61903,'AN' from dual union all
select 61904,'MRN' from dual union all
select 61905,'MRN' from dual union all
select 61906,'MRN' from dual union all
select 61907,'JMA' from dual union all
select 61908,'MRN' from dual union all
select 61909,'GMS' from dual union all
select 6191,'MVS' from dual union all
select 61910,'MRN' from dual union all
select 61911,'JMA' from dual union all
select 61912,'GMS' from dual union all
select 61913,'GMS' from dual;
insert into LAUNCH_AGENCY
select 61914,'GMS' from dual union all
select 61915,'JMA' from dual union all
select 61916,'GMS' from dual union all
select 61917,'GMS' from dual union all
select 61918,'MRN' from dual union all
select 61919,'AN' from dual union all
select 6192,'RVSN' from dual union all
select 61920,'GMS' from dual union all
select 61921,'MRN' from dual union all
select 61922,'GMS' from dual union all
select 61923,'GMS' from dual union all
select 61924,'GMS' from dual union all
select 61925,'MRN' from dual union all
select 61926,'GMS' from dual union all
select 61927,'GMS' from dual union all
select 61928,'GMS' from dual union all
select 61929,'AN' from dual union all
select 6193,'RVSN' from dual union all
select 61930,'GMS' from dual union all
select 61931,'MRN' from dual union all
select 61932,'GMS' from dual union all
select 61933,'AN' from dual union all
select 61934,'MRN' from dual union all
select 61935,'GMS' from dual union all
select 61936,'MRN' from dual union all
select 61937,'JMA' from dual union all
select 61938,'AN' from dual union all
select 61939,'GMS' from dual union all
select 6194,'RVSN' from dual union all
select 61940,'MRN' from dual union all
select 61941,'AN' from dual union all
select 61942,'MRN' from dual union all
select 61943,'GMS' from dual union all
select 61944,'GMS' from dual union all
select 61945,'MRN' from dual union all
select 61946,'JMA' from dual union all
select 61947,'GMS' from dual union all
select 61948,'MRN' from dual union all
select 61949,'GMS' from dual union all
select 6195,'RVSN' from dual union all
select 61950,'GMS' from dual union all
select 61951,'GMS' from dual union all
select 61952,'AN' from dual union all
select 61953,'GMS' from dual union all
select 61954,'AN' from dual union all
select 61955,'MRN' from dual union all
select 61956,'GMS' from dual union all
select 61957,'GMS' from dual union all
select 61958,'AN' from dual union all
select 61959,'MRN' from dual union all
select 6196,'UNRV' from dual union all
select 61960,'GMS' from dual union all
select 61961,'AN' from dual union all
select 61962,'MRN' from dual union all
select 61963,'AN' from dual union all
select 61964,'MRN' from dual union all
select 61965,'JMA' from dual union all
select 61966,'MRN' from dual union all
select 61967,'MRN' from dual union all
select 61968,'MRN' from dual union all
select 61969,'GMS' from dual union all
select 6197,'UNRV' from dual union all
select 61970,'GMS' from dual union all
select 61971,'GMS' from dual union all
select 61972,'GMS' from dual union all
select 61973,'JMA' from dual union all
select 61974,'AN' from dual union all
select 61975,'MRN' from dual union all
select 61976,'JMA' from dual union all
select 61977,'MRN' from dual union all
select 61978,'AN' from dual union all
select 61979,'AN' from dual union all
select 6198,'UNRV' from dual union all
select 61980,'AN' from dual union all
select 61981,'AN' from dual union all
select 61982,'AN' from dual union all
select 61983,'AN' from dual union all
select 61984,'AN' from dual union all
select 61985,'AN' from dual union all
select 61986,'AN' from dual union all
select 61987,'AN' from dual union all
select 61988,'AN' from dual union all
select 61989,'AN' from dual union all
select 6199,'UNRV' from dual union all
select 61990,'AN' from dual union all
select 61991,'AN' from dual union all
select 61992,'AN' from dual union all
select 61993,'AN' from dual union all
select 61994,'AN' from dual union all
select 61995,'AN' from dual union all
select 61996,'AN' from dual union all
select 61997,'AN' from dual union all
select 61998,'AN' from dual union all
select 61999,'AN' from dual union all
select 62,'WEHR' from dual union all
select 620,'PARD' from dual union all
select 6200,'UNRV' from dual union all
select 62000,'AN' from dual union all
select 62001,'AN' from dual union all
select 62002,'AN' from dual;
insert into LAUNCH_AGENCY
select 62003,'AN' from dual union all
select 62004,'AN' from dual union all
select 62005,'AN' from dual union all
select 62006,'AN' from dual union all
select 62007,'AN' from dual union all
select 62008,'AN' from dual union all
select 62009,'AN' from dual union all
select 6201,'USAF' from dual union all
select 62010,'AN' from dual union all
select 62011,'AN' from dual union all
select 62012,'AN' from dual union all
select 62013,'AN' from dual union all
select 62014,'AN' from dual union all
select 62015,'AN' from dual union all
select 62016,'AN' from dual union all
select 62017,'AN' from dual union all
select 62018,'AN' from dual union all
select 62019,'AN' from dual union all
select 6202,'RVSN' from dual union all
select 62020,'AN' from dual union all
select 62021,'AN' from dual union all
select 62022,'AN' from dual union all
select 62023,'AN' from dual union all
select 62024,'AN' from dual union all
select 62025,'AN' from dual union all
select 62026,'AN' from dual union all
select 62027,'AN' from dual union all
select 62028,'AN' from dual union all
select 62029,'AN' from dual union all
select 6203,'RVSN' from dual union all
select 62030,'AN' from dual union all
select 62031,'AN' from dual union all
select 62032,'AN' from dual union all
select 62033,'AN' from dual union all
select 62034,'AN' from dual union all
select 62035,'AN' from dual union all
select 62036,'AN' from dual union all
select 62037,'AN' from dual union all
select 62038,'AN' from dual union all
select 62039,'AN' from dual union all
select 6204,'PARD' from dual union all
select 62040,'AN' from dual union all
select 62041,'AN' from dual union all
select 62042,'AN' from dual union all
select 62043,'AN' from dual union all
select 62044,'AN' from dual union all
select 62045,'AN' from dual union all
select 62046,'AN' from dual union all
select 62047,'AN' from dual union all
select 62048,'AN' from dual union all
select 62049,'AN' from dual union all
select 6205,'RVSN' from dual union all
select 62050,'AN' from dual union all
select 62051,'AN' from dual union all
select 62052,'AN' from dual union all
select 62053,'AN' from dual union all
select 62054,'AN' from dual union all
select 62055,'AN' from dual union all
select 62056,'AN' from dual union all
select 62057,'AN' from dual union all
select 62058,'AN' from dual union all
select 62059,'AN' from dual union all
select 6206,'UNRV' from dual union all
select 62060,'AN' from dual union all
select 62061,'AN' from dual union all
select 62062,'AN' from dual union all
select 62063,'AN' from dual union all
select 62064,'AN' from dual union all
select 62065,'AN' from dual union all
select 62066,'AN' from dual union all
select 62067,'AN' from dual union all
select 62068,'AN' from dual union all
select 62069,'AN' from dual union all
select 6207,'ABMA' from dual union all
select 6207,'USAF' from dual union all
select 62070,'AN' from dual union all
select 62071,'AN' from dual union all
select 62072,'AN' from dual union all
select 62073,'AN' from dual union all
select 62074,'AN' from dual union all
select 62075,'AN' from dual union all
select 62076,'AN' from dual union all
select 62077,'AN' from dual union all
select 62078,'AN' from dual union all
select 62079,'AN' from dual union all
select 6208,'USAF' from dual union all
select 62080,'AN' from dual union all
select 62081,'AN' from dual union all
select 62082,'AN' from dual union all
select 62083,'AN' from dual union all
select 62084,'AN' from dual union all
select 62085,'AN' from dual union all
select 62086,'AN' from dual union all
select 62087,'AN' from dual union all
select 62088,'AN' from dual union all
select 62089,'AN' from dual union all
select 6209,'USAF' from dual union all
select 62090,'AN' from dual union all
select 62091,'AN' from dual union all
select 62092,'AN' from dual;
insert into LAUNCH_AGENCY
select 62093,'AN' from dual union all
select 62094,'AN' from dual union all
select 62095,'AN' from dual union all
select 62096,'AN' from dual union all
select 62097,'AN' from dual union all
select 62098,'AN' from dual union all
select 62099,'AN' from dual union all
select 621,'ORD' from dual union all
select 6210,'MVS' from dual union all
select 62100,'AN' from dual union all
select 62101,'AN' from dual union all
select 62102,'AN' from dual union all
select 62103,'AN' from dual union all
select 62104,'AN' from dual union all
select 62105,'AN' from dual union all
select 62106,'AN' from dual union all
select 62107,'AN' from dual union all
select 62108,'AN' from dual union all
select 62109,'AN' from dual union all
select 6211,'MVS' from dual union all
select 62110,'AN' from dual union all
select 62111,'AN' from dual union all
select 62112,'AN' from dual union all
select 62113,'AN' from dual union all
select 62114,'AN' from dual union all
select 62115,'AN' from dual union all
select 62116,'AN' from dual union all
select 62117,'AN' from dual union all
select 62118,'AN' from dual union all
select 62119,'AN' from dual union all
select 6212,'USAF' from dual union all
select 62120,'AN' from dual union all
select 62121,'AN' from dual union all
select 62122,'AN' from dual union all
select 62123,'AN' from dual union all
select 62124,'MRN' from dual union all
select 62125,'AN' from dual union all
select 62126,'AN' from dual union all
select 62127,'AN' from dual union all
select 62128,'AN' from dual union all
select 62129,'AN' from dual union all
select 6213,'RAF' from dual union all
select 62130,'AN' from dual union all
select 62131,'AN' from dual union all
select 62132,'MRN' from dual union all
select 62133,'MRN' from dual union all
select 62134,'MRN' from dual union all
select 62135,'MRN' from dual union all
select 62136,'AN' from dual union all
select 62137,'AN' from dual union all
select 62138,'AN' from dual union all
select 62139,'AN' from dual union all
select 6214,'SAND' from dual union all
select 62140,'AN' from dual union all
select 62141,'AN' from dual union all
select 62142,'AN' from dual union all
select 62143,'AN' from dual union all
select 62144,'AN' from dual union all
select 62145,'AN' from dual union all
select 62146,'AN' from dual union all
select 62147,'AN' from dual union all
select 62148,'AN' from dual union all
select 62149,'AN' from dual union all
select 6215,'SAND' from dual union all
select 62150,'AN' from dual union all
select 62151,'AN' from dual union all
select 62152,'AN' from dual union all
select 62153,'AN' from dual union all
select 62154,'MRN' from dual union all
select 62155,'AN' from dual union all
select 62156,'AN' from dual union all
select 62157,'AN' from dual union all
select 62158,'AN' from dual union all
select 62159,'AN' from dual union all
select 6216,'USN' from dual union all
select 62160,'AN' from dual union all
select 62161,'AN' from dual union all
select 62162,'AN' from dual union all
select 62163,'AN' from dual union all
select 62164,'AN' from dual union all
select 62165,'AN' from dual union all
select 62166,'AN' from dual union all
select 62167,'AN' from dual union all
select 62168,'AN' from dual union all
select 62169,'AN' from dual union all
select 6217,'USAF' from dual union all
select 62170,'AN' from dual union all
select 62171,'AN' from dual union all
select 62172,'AN' from dual union all
select 62173,'AN' from dual union all
select 62174,'AN' from dual union all
select 62175,'AN' from dual union all
select 62176,'AN' from dual union all
select 62177,'AN' from dual union all
select 62178,'AN' from dual union all
select 62179,'MRN' from dual union all
select 6218,'USAF' from dual union all
select 62180,'MRN' from dual union all
select 62181,'MRN' from dual union all
select 62182,'MRN' from dual;
insert into LAUNCH_AGENCY
select 62183,'MRN' from dual union all
select 62184,'MRN' from dual union all
select 62185,'AN' from dual union all
select 62186,'MRN' from dual union all
select 62187,'AN' from dual union all
select 62188,'AN' from dual union all
select 62189,'AN' from dual union all
select 6219,'USAF' from dual union all
select 62190,'AN' from dual union all
select 62191,'AN' from dual union all
select 62192,'AN' from dual union all
select 62193,'AN' from dual union all
select 62194,'AN' from dual union all
select 62195,'AN' from dual union all
select 62196,'AN' from dual union all
select 62197,'AN' from dual union all
select 62198,'DLR' from dual union all
select 62199,'DLR' from dual union all
select 622,'FIAN' from dual union all
select 6220,'USAF' from dual union all
select 62200,'DLR' from dual union all
select 62201,'DLR' from dual union all
select 62202,'DLR' from dual union all
select 62203,'DLR' from dual union all
select 62204,'DLR' from dual union all
select 62205,'DLR' from dual union all
select 62206,'DLR' from dual union all
select 62207,'AN' from dual union all
select 62208,'DLR' from dual union all
select 62209,'AN' from dual union all
select 6221,'MVS' from dual union all
select 62210,'AN' from dual union all
select 62211,'AN' from dual union all
select 62212,'AN' from dual union all
select 62213,'AN' from dual union all
select 62214,'AN' from dual union all
select 62215,'AN' from dual union all
select 62216,'AN' from dual union all
select 62217,'AN' from dual union all
select 62218,'AN' from dual union all
select 62219,'MRN' from dual union all
select 6222,'WRE' from dual union all
select 62220,'AN' from dual union all
select 62221,'AN' from dual union all
select 62222,'AN' from dual union all
select 62223,'MRN' from dual union all
select 62224,'AN' from dual union all
select 62225,'AN' from dual union all
select 62226,'AN' from dual union all
select 62227,'AN' from dual union all
select 62228,'AN' from dual union all
select 62229,'AN' from dual union all
select 6223,'ONERA' from dual union all
select 62230,'AN' from dual union all
select 62231,'AN' from dual union all
select 62232,'AN' from dual union all
select 62233,'AN' from dual union all
select 62234,'AN' from dual union all
select 62235,'AN' from dual union all
select 62236,'AN' from dual union all
select 62237,'AN' from dual union all
select 62238,'MRN' from dual union all
select 62239,'MRN' from dual union all
select 6224,'USAF' from dual union all
select 62240,'AN' from dual union all
select 62241,'AN' from dual union all
select 62242,'AN' from dual union all
select 62243,'AN' from dual union all
select 62244,'AN' from dual union all
select 62245,'AN' from dual union all
select 62246,'AN' from dual union all
select 62247,'AN' from dual union all
select 62248,'AN' from dual union all
select 62249,'AN' from dual union all
select 6225,'USAF' from dual union all
select 62250,'AN' from dual union all
select 62251,'AN' from dual union all
select 62252,'AN' from dual union all
select 62253,'AN' from dual union all
select 62254,'AN' from dual union all
select 62255,'AN' from dual union all
select 62256,'AN' from dual union all
select 62257,'AN' from dual union all
select 62258,'AN' from dual union all
select 62259,'AN' from dual union all
select 6226,'SAND' from dual union all
select 62260,'AN' from dual union all
select 62261,'AN' from dual union all
select 62262,'AN' from dual union all
select 62263,'AN' from dual union all
select 62264,'AN' from dual union all
select 62265,'AN' from dual union all
select 62266,'AN' from dual union all
select 62267,'AN' from dual union all
select 62268,'AN' from dual union all
select 62269,'AN' from dual union all
select 6227,'SAND' from dual union all
select 62270,'AN' from dual union all
select 62271,'AN' from dual union all
select 62272,'AN' from dual;
insert into LAUNCH_AGENCY
select 62273,'AN' from dual union all
select 62274,'AN' from dual union all
select 62275,'AN' from dual union all
select 62276,'AN' from dual union all
select 62277,'AN' from dual union all
select 62278,'UNKS' from dual union all
select 62279,'UNKS' from dual union all
select 6228,'ABMA' from dual union all
select 6228,'USAF' from dual union all
select 62280,'UNKS' from dual union all
select 62281,'UNKS' from dual union all
select 62282,'UNKS' from dual union all
select 62283,'AE' from dual union all
select 62284,'UNKS' from dual union all
select 62285,'UNKS' from dual union all
select 62286,'UNKS' from dual union all
select 62287,'UNKS' from dual union all
select 62288,'UNKS' from dual union all
select 62289,'UNKS' from dual union all
select 6229,'USN' from dual union all
select 62290,'MDSSC' from dual union all
select 62291,'UNKS' from dual union all
select 62292,'UNKS' from dual union all
select 62293,'ISAS' from dual union all
select 62294,'UNKS' from dual union all
select 62295,'UNKS' from dual union all
select 62296,'UNKS' from dual union all
select 62297,'AE' from dual union all
select 62298,'NASA' from dual union all
select 62299,'UNKS' from dual union all
select 623,'FIAN' from dual union all
select 6230,'MVS' from dual union all
select 62300,'UNKS' from dual union all
select 62301,'UNKS' from dual union all
select 62302,'UNKS' from dual union all
select 62303,'MDSSC' from dual union all
select 62304,'AE' from dual union all
select 62305,'UNKS' from dual union all
select 62306,'UNKS' from dual union all
select 62307,'UNKS' from dual union all
select 62308,'UNKS' from dual union all
select 62309,'UNKS' from dual union all
select 6231,'USA' from dual union all
select 62310,'NASA' from dual union all
select 62311,'UNKS' from dual union all
select 62312,'AFSC' from dual union all
select 62313,'UNKS' from dual union all
select 62314,'UNKS' from dual union all
select 62315,'UNKS' from dual union all
select 62316,'UNKS' from dual union all
select 62317,'UNKS' from dual union all
select 62318,'AE' from dual union all
select 62319,'UNKS' from dual union all
select 6232,'USAF' from dual union all
select 62320,'UNKS' from dual union all
select 62321,'MDSSC' from dual union all
select 62322,'UNKS' from dual union all
select 62323,'AFSC' from dual union all
select 62324,'UNKS' from dual union all
select 62325,'UNKS' from dual union all
select 62326,'UNKS' from dual union all
select 62327,'UNKS' from dual union all
select 62328,'UNKS' from dual union all
select 62329,'UNKS' from dual union all
select 6233,'USAF' from dual union all
select 62330,'AE' from dual union all
select 62331,'UNKS' from dual union all
select 62332,'UNKS' from dual union all
select 62333,'UNKS' from dual union all
select 62334,'UNKS' from dual union all
select 62335,'UNKS' from dual union all
select 62336,'UNKS' from dual union all
select 62337,'UNKS' from dual union all
select 62338,'NASA' from dual union all
select 62339,'AE' from dual union all
select 6234,'MVS' from dual union all
select 62340,'UNKS' from dual union all
select 62341,'MDSSC' from dual union all
select 62342,'UNKS' from dual union all
select 62343,'UNKS' from dual union all
select 62344,'MDSSC' from dual union all
select 62345,'UNKS' from dual union all
select 62346,'AFSC' from dual union all
select 62347,'NASDA' from dual union all
select 62348,'UNKS' from dual union all
select 62349,'AFSC' from dual union all
select 6235,'ABMA' from dual union all
select 6235,'USAF' from dual union all
select 62350,'UNKS' from dual union all
select 62351,'UNKS' from dual union all
select 62352,'UNKS' from dual union all
select 62353,'UNKS' from dual union all
select 62354,'NASA' from dual union all
select 62355,'UNKS' from dual union all
select 62356,'UNKS' from dual union all
select 62357,'UNKS' from dual union all
select 62358,'UNKS' from dual union all
select 62359,'UNKS' from dual union all
select 6236,'MVS' from dual union all
select 62360,'UNKS' from dual;
insert into LAUNCH_AGENCY
select 62361,'NASA' from dual union all
select 62362,'MDSSC' from dual union all
select 62363,'UNKS' from dual union all
select 62364,'AE' from dual union all
select 62365,'UNKS' from dual union all
select 62366,'NASA' from dual union all
select 62367,'NASA' from dual union all
select 62368,'UNKS' from dual union all
select 62369,'UNKS' from dual union all
select 6237,'USAF' from dual union all
select 62370,'UNKS' from dual union all
select 62371,'UNKS' from dual union all
select 62372,'UNKS' from dual union all
select 62373,'UNKS' from dual union all
select 62374,'MDSSC' from dual union all
select 62375,'UNKS' from dual union all
select 62376,'UNKS' from dual union all
select 62377,'UNKS' from dual union all
select 62378,'UNKS' from dual union all
select 62379,'NASA' from dual union all
select 6238,'USAF' from dual union all
select 62380,'USN' from dual union all
select 62381,'MRN' from dual union all
select 62382,'MRN' from dual union all
select 62383,'MRN' from dual union all
select 62384,'USN' from dual union all
select 62385,'MRN' from dual union all
select 62386,'AMROC' from dual union all
select 62387,'MRN' from dual union all
select 62388,'MDD' from dual union all
select 62389,'SSI' from dual union all
select 6239,'USAF' from dual union all
select 62390,'MRN' from dual union all
select 62391,'MRN' from dual union all
select 62392,'NASA' from dual union all
select 62393,'RVSN' from dual union all
select 62394,'RVSN' from dual union all
select 62395,'BWEHR' from dual union all
select 62396,'UNKS' from dual union all
select 62397,'NASDA' from dual union all
select 62398,'AFSC' from dual union all
select 62399,'IAE' from dual union all
select 624,'FIAN' from dual union all
select 6240,'USAF' from dual union all
select 62400,'NASDA' from dual union all
select 62401,'IQ' from dual union all
select 62402,'WFF' from dual union all
select 62403,'WFF' from dual union all
select 62404,'WFF' from dual union all
select 62405,'WFF' from dual union all
select 62406,'WFF' from dual union all
select 62407,'WFF' from dual union all
select 62408,'SUPA' from dual union all
select 62409,'ISRO' from dual union all
select 6241,'USAF' from dual union all
select 62410,'ISRO' from dual union all
select 62411,'ISRO' from dual union all
select 62412,'ISRO' from dual union all
select 62413,'ISRO' from dual union all
select 62414,'ISRO' from dual union all
select 62415,'ISRO' from dual union all
select 62416,'ISRO' from dual union all
select 62417,'ISRO' from dual union all
select 62418,'ISRO' from dual union all
select 62419,'ISRO' from dual union all
select 6242,'ABMA' from dual union all
select 6242,'USAF' from dual union all
select 62420,'ISRO' from dual union all
select 62421,'ISRO' from dual union all
select 62422,'ISRO' from dual union all
select 62423,'ISRO' from dual union all
select 62424,'ISRO' from dual union all
select 62425,'ISRO' from dual union all
select 62426,'ISRO' from dual union all
select 62427,'ISRO' from dual union all
select 62428,'ISRO' from dual union all
select 62429,'ISRO' from dual union all
select 6243,'MVS' from dual union all
select 62430,'ISRO' from dual union all
select 62431,'ISRO' from dual union all
select 62432,'ISRO' from dual union all
select 62433,'ISRO' from dual union all
select 62434,'ISRO' from dual union all
select 62435,'ISRO' from dual union all
select 62436,'ISRO' from dual union all
select 62437,'ISRO' from dual union all
select 62438,'ISRO' from dual union all
select 62439,'ISRO' from dual union all
select 6244,'RVSN' from dual union all
select 62440,'ISRO' from dual union all
select 62441,'ISRO' from dual union all
select 62442,'ISRO' from dual union all
select 62443,'ISRO' from dual union all
select 62444,'ISRO' from dual union all
select 62445,'ISRO' from dual union all
select 62446,'ISRO' from dual union all
select 62447,'ISRO' from dual union all
select 62448,'ISRO' from dual union all
select 62449,'ISRO' from dual union all
select 6245,'MVS' from dual;
insert into LAUNCH_AGENCY
select 62450,'ISRO' from dual union all
select 62451,'ISRO' from dual union all
select 62452,'ISRO' from dual union all
select 62453,'ISRO' from dual union all
select 62454,'ISRO' from dual union all
select 62455,'RVSN' from dual union all
select 62456,'RVSN' from dual union all
select 62457,'RVSN' from dual union all
select 62458,'RVSN' from dual union all
select 62459,'RVSN' from dual union all
select 6246,'NASA' from dual union all
select 62460,'RVSN' from dual union all
select 62461,'RVSN' from dual union all
select 62462,'RVSN' from dual union all
select 62463,'RVSN' from dual union all
select 62464,'RVSN' from dual union all
select 62465,'AN' from dual union all
select 62466,'AN' from dual union all
select 62467,'NASA' from dual union all
select 62468,'AN' from dual union all
select 62469,'AN' from dual union all
select 6247,'USAF' from dual union all
select 62470,'AN' from dual union all
select 62471,'AN' from dual union all
select 62472,'ALCS' from dual union all
select 62472,'SAC' from dual union all
select 62473,'NASA' from dual union all
select 62474,'MBB' from dual union all
select 62475,'RVSN' from dual union all
select 62476,'USN' from dual union all
select 62477,'NASA' from dual union all
select 62478,'MBB' from dual union all
select 62479,'USN' from dual union all
select 6248,'USAF' from dual union all
select 62480,'USN' from dual union all
select 62481,'SDIO' from dual union all
select 62482,'SDIO' from dual union all
select 62483,'DMA' from dual union all
select 62484,'IQ' from dual union all
select 62485,'IQ' from dual union all
select 62486,'RVSN' from dual union all
select 62487,'IQ' from dual union all
select 62488,'IQ' from dual union all
select 62489,'SUPA' from dual union all
select 6249,'USAF' from dual union all
select 62490,'RVSN' from dual union all
select 62491,'RVSN' from dual union all
select 62492,'RVSN' from dual union all
select 62493,'RVSN' from dual union all
select 62494,'RVSN' from dual union all
select 62495,'RVSN' from dual union all
select 62496,'RVSN' from dual union all
select 62497,'RVSN' from dual union all
select 62498,'RVSN' from dual union all
select 62499,'RVSN' from dual union all
select 625,'PARD' from dual union all
select 6250,'RAE' from dual union all
select 62500,'PLA2AC' from dual union all
select 62501,'RVSN' from dual union all
select 62502,'AN' from dual union all
select 62503,'RVSN' from dual union all
select 62504,'RVSN' from dual union all
select 62505,'RVSN' from dual union all
select 62506,'RVSN' from dual union all
select 62507,'RVSN' from dual union all
select 62508,'RVSN' from dual union all
select 62509,'RVSN' from dual union all
select 6251,'MVS' from dual union all
select 62510,'RVSN' from dual union all
select 62511,'RVSN' from dual union all
select 62512,'RVSN' from dual union all
select 62513,'RVSN' from dual union all
select 62514,'RVSN' from dual union all
select 62515,'RVSN' from dual union all
select 62516,'USN' from dual union all
select 62517,'NASA' from dual union all
select 62518,'USN' from dual union all
select 62519,'SAC' from dual union all
select 6252,'MVS' from dual union all
select 62520,'USN' from dual union all
select 62521,'DLR' from dual union all
select 62522,'CSA' from dual union all
select 62523,'ISAS' from dual union all
select 62524,'SDIO' from dual union all
select 62525,'DFVLR' from dual union all
select 62526,'DFVLR' from dual union all
select 62527,'SAC' from dual union all
select 62528,'NASA' from dual union all
select 62528,'SSC' from dual union all
select 62529,'NASA' from dual union all
select 62529,'SSC' from dual union all
select 6253,'SAND' from dual union all
select 62530,'SAC' from dual union all
select 62531,'NASA' from dual union all
select 62532,'DGA' from dual union all
select 62533,'USN' from dual union all
select 62534,'NASA' from dual union all
select 62535,'AFSC' from dual union all
select 62536,'NASA' from dual union all
select 62537,'IAE' from dual;
insert into LAUNCH_AGENCY
select 62538,'SSI' from dual union all
select 62539,'NASA' from dual union all
select 6254,'SAND' from dual union all
select 62540,'NASA' from dual union all
select 62541,'SSC' from dual union all
select 62542,'NASA' from dual union all
select 62543,'MBB' from dual union all
select 62544,'MBB' from dual union all
select 62545,'NASA' from dual union all
select 62546,'NASA' from dual union all
select 62547,'IDRDL' from dual union all
select 62548,'AERO' from dual union all
select 62549,'DMA' from dual union all
select 6255,'SAND' from dual union all
select 62550,'NASA' from dual union all
select 62551,'NASA' from dual union all
select 62552,'AN' from dual union all
select 62553,'AN' from dual union all
select 62554,'AN' from dual union all
select 62555,'AN' from dual union all
select 62556,'HOUW' from dual union all
select 62556,'SAAF' from dual union all
select 62557,'SAC' from dual union all
select 62558,'ALCS' from dual union all
select 62558,'SAC' from dual union all
select 62559,'SDIO' from dual union all
select 6256,'USAF' from dual union all
select 62560,'NASA' from dual union all
select 62561,'SDIO' from dual union all
select 62562,'SDIO' from dual union all
select 62563,'AN' from dual union all
select 62564,'AN' from dual union all
select 62565,'AN' from dual union all
select 62566,'AN' from dual union all
select 62567,'SDIO' from dual union all
select 62568,'SDIO' from dual union all
select 62569,'USN' from dual union all
select 6257,'AN' from dual union all
select 62570,'SDIO' from dual union all
select 62571,'NASA' from dual union all
select 62572,'AN' from dual union all
select 62573,'AN' from dual union all
select 62574,'AN' from dual union all
select 62575,'AN' from dual union all
select 62576,'SDIO' from dual union all
select 62577,'ISAS' from dual union all
select 62578,'SDIO' from dual union all
select 62579,'NASA' from dual union all
select 6258,'USAF' from dual union all
select 62580,'IDF' from dual union all
select 62581,'SAC' from dual union all
select 62582,'SDIO' from dual union all
select 62583,'SAC' from dual union all
select 62584,'IDRDL' from dual union all
select 62585,'USN' from dual union all
select 62586,'AN' from dual union all
select 62587,'AN' from dual union all
select 62588,'GMS' from dual union all
select 62589,'GMS' from dual union all
select 6259,'USAF' from dual union all
select 62590,'GMS' from dual union all
select 62591,'GMS' from dual union all
select 62592,'GMS' from dual union all
select 62593,'MRN' from dual union all
select 62594,'AN' from dual union all
select 62595,'MRN' from dual union all
select 62596,'GMS' from dual union all
select 62597,'GMS' from dual union all
select 62598,'MRN' from dual union all
select 62599,'MRN' from dual union all
select 626,'GEMTS' from dual union all
select 626,'US' from dual union all
select 6260,'USAF' from dual union all
select 62600,'MRN' from dual union all
select 62601,'GMS' from dual union all
select 62602,'MRN' from dual union all
select 62603,'AN' from dual union all
select 62604,'MDD' from dual union all
select 62605,'MRN' from dual union all
select 62606,'MRN' from dual union all
select 62607,'MDD' from dual union all
select 62608,'GMS' from dual union all
select 62609,'AN' from dual union all
select 6261,'USN' from dual union all
select 62610,'MRN' from dual union all
select 62611,'GMS' from dual union all
select 62612,'GMS' from dual union all
select 62613,'MRN' from dual union all
select 62614,'MRN' from dual union all
select 62615,'MDD' from dual union all
select 62616,'MRN' from dual union all
select 62617,'MRN' from dual union all
select 62618,'MRN' from dual union all
select 62619,'GMS' from dual union all
select 6262,'USN' from dual union all
select 62620,'MRN' from dual union all
select 62621,'GMS' from dual union all
select 62622,'MRN' from dual union all
select 62623,'MRN' from dual union all
select 62624,'AN' from dual;
insert into LAUNCH_AGENCY
select 62625,'MRN' from dual union all
select 62626,'MRN' from dual union all
select 62627,'MRN' from dual union all
select 62628,'MRN' from dual union all
select 62629,'MRN' from dual union all
select 6263,'USAF' from dual union all
select 62630,'GMS' from dual union all
select 62631,'MRN' from dual union all
select 62632,'MRN' from dual union all
select 62633,'MRN' from dual union all
select 62634,'MRN' from dual union all
select 62635,'MRN' from dual union all
select 62636,'GMS' from dual union all
select 62637,'MDD' from dual union all
select 62638,'AN' from dual union all
select 62639,'MRN' from dual union all
select 6264,'USAF' from dual union all
select 62640,'GMS' from dual union all
select 62641,'MRN' from dual union all
select 62642,'MRN' from dual union all
select 62643,'GMS' from dual union all
select 62644,'MRN' from dual union all
select 62645,'MRN' from dual union all
select 62646,'GMS' from dual union all
select 62647,'MRN' from dual union all
select 62648,'MRN' from dual union all
select 62649,'GMS' from dual union all
select 6265,'AFETR' from dual union all
select 62650,'AN' from dual union all
select 62651,'MDD' from dual union all
select 62652,'GMS' from dual union all
select 62653,'MRN' from dual union all
select 62654,'AN' from dual union all
select 62655,'GMS' from dual union all
select 62656,'AN' from dual union all
select 62657,'AN' from dual union all
select 62658,'GMS' from dual union all
select 62659,'GMS' from dual union all
select 6266,'SMSA' from dual union all
select 62660,'MRN' from dual union all
select 62661,'GMS' from dual union all
select 62662,'AN' from dual union all
select 62663,'MRN' from dual union all
select 62664,'MRN' from dual union all
select 62665,'GMS' from dual union all
select 62666,'MRN' from dual union all
select 62667,'AN' from dual union all
select 62668,'MRN' from dual union all
select 62669,'AN' from dual union all
select 6267,'US' from dual union all
select 62670,'AN' from dual union all
select 62671,'GMS' from dual union all
select 62672,'MRN' from dual union all
select 62673,'MRN' from dual union all
select 62674,'MRN' from dual union all
select 62675,'MRN' from dual union all
select 62676,'GMS' from dual union all
select 62677,'AN' from dual union all
select 62678,'AN' from dual union all
select 62679,'MDD' from dual union all
select 6268,'BRL' from dual union all
select 62680,'MRN' from dual union all
select 62681,'MRN' from dual union all
select 62682,'GMS' from dual union all
select 62683,'MRN' from dual union all
select 62684,'GMS' from dual union all
select 62685,'GMS' from dual union all
select 62686,'AN' from dual union all
select 62687,'MRN' from dual union all
select 62688,'AN' from dual union all
select 62689,'AN' from dual union all
select 6269,'USN' from dual union all
select 62690,'AN' from dual union all
select 62691,'AN' from dual union all
select 62692,'AN' from dual union all
select 62693,'MRN' from dual union all
select 62694,'AN' from dual union all
select 62695,'MRN' from dual union all
select 62696,'JMA' from dual union all
select 62697,'MRN' from dual union all
select 62698,'MRN' from dual union all
select 62699,'MRN' from dual union all
select 627,'PRIN' from dual union all
select 627,'US' from dual union all
select 6270,'USN' from dual union all
select 62700,'AN' from dual union all
select 62701,'MRN' from dual union all
select 62702,'MRN' from dual union all
select 62703,'MRN' from dual union all
select 62704,'MRN' from dual union all
select 62705,'MRN' from dual union all
select 62706,'MRN' from dual union all
select 62707,'GMS' from dual union all
select 62708,'MRN' from dual union all
select 62709,'MRN' from dual union all
select 6271,'USN' from dual union all
select 62710,'MRN' from dual union all
select 62711,'MRN' from dual union all
select 62712,'MRN' from dual union all
select 62713,'MRN' from dual;
insert into LAUNCH_AGENCY
select 62714,'AN' from dual union all
select 62715,'AN' from dual union all
select 62716,'MRN' from dual union all
select 62717,'MRN' from dual union all
select 62718,'AN' from dual union all
select 62719,'AN' from dual union all
select 6272,'USN' from dual union all
select 62720,'AN' from dual union all
select 62721,'AN' from dual union all
select 62722,'MRN' from dual union all
select 62723,'AN' from dual union all
select 62724,'AN' from dual union all
select 62725,'GMS' from dual union all
select 62726,'AN' from dual union all
select 62727,'ISAS' from dual union all
select 62728,'AN' from dual union all
select 62729,'GMS' from dual union all
select 6273,'USN' from dual union all
select 62730,'MDD' from dual union all
select 62731,'GMS' from dual union all
select 62732,'AN' from dual union all
select 62733,'AN' from dual union all
select 62734,'MRN' from dual union all
select 62735,'MRN' from dual union all
select 62736,'MRN' from dual union all
select 62737,'MRN' from dual union all
select 62738,'AN' from dual union all
select 62739,'MRN' from dual union all
select 6274,'USN' from dual union all
select 62740,'MRN' from dual union all
select 62741,'AN' from dual union all
select 62742,'AN' from dual union all
select 62743,'MDD' from dual union all
select 62744,'AN' from dual union all
select 62745,'AN' from dual union all
select 62746,'MRN' from dual union all
select 62747,'AN' from dual union all
select 62748,'AN' from dual union all
select 62749,'MRN' from dual union all
select 6275,'USN' from dual union all
select 62750,'MRN' from dual union all
select 62751,'GMS' from dual union all
select 62752,'GMS' from dual union all
select 62753,'AN' from dual union all
select 62754,'MDD' from dual union all
select 62755,'MRN' from dual union all
select 62756,'MRN' from dual union all
select 62757,'MRN' from dual union all
select 62758,'MRN' from dual union all
select 62759,'GMS' from dual union all
select 6276,'USN' from dual union all
select 62760,'GMS' from dual union all
select 62761,'GMS' from dual union all
select 62762,'GMS' from dual union all
select 62763,'GMS' from dual union all
select 62764,'MRN' from dual union all
select 62765,'MRN' from dual union all
select 62766,'MRN' from dual union all
select 62767,'GMS' from dual union all
select 62768,'MRN' from dual union all
select 62769,'MRN' from dual union all
select 6277,'USN' from dual union all
select 62770,'AN' from dual union all
select 62771,'MRN' from dual union all
select 62772,'MRN' from dual union all
select 62773,'GMS' from dual union all
select 62774,'MRN' from dual union all
select 62775,'MRN' from dual union all
select 62776,'GMS' from dual union all
select 62777,'MRN' from dual union all
select 62778,'GMS' from dual union all
select 62779,'AN' from dual union all
select 6278,'USN' from dual union all
select 62780,'MDD' from dual union all
select 62781,'MRN' from dual union all
select 62782,'GMS' from dual union all
select 62783,'GMS' from dual union all
select 62784,'GMS' from dual union all
select 62785,'MRN' from dual union all
select 62786,'GMS' from dual union all
select 62787,'MRN' from dual union all
select 62788,'MRN' from dual union all
select 62789,'GMS' from dual union all
select 6279,'USN' from dual union all
select 62790,'AN' from dual union all
select 62791,'MRN' from dual union all
select 62792,'MRN' from dual union all
select 62793,'MRN' from dual union all
select 62794,'MRN' from dual union all
select 62795,'GMS' from dual union all
select 62796,'MRN' from dual union all
select 62797,'MRN' from dual union all
select 62798,'GMS' from dual union all
select 62799,'MRN' from dual union all
select 628,'ARDC' from dual union all
select 628,'US' from dual union all
select 6280,'USN' from dual union all
select 62800,'MRN' from dual union all
select 62801,'MRN' from dual union all
select 62802,'MRN' from dual;
insert into LAUNCH_AGENCY
select 62803,'MRN' from dual union all
select 62804,'AN' from dual union all
select 62805,'MRN' from dual union all
select 62806,'MRN' from dual union all
select 62807,'AN' from dual union all
select 62808,'MRN' from dual union all
select 62809,'MRN' from dual union all
select 6281,'SMSA' from dual union all
select 62810,'MRN' from dual union all
select 62811,'MDD' from dual union all
select 62812,'MRN' from dual union all
select 62813,'MRN' from dual union all
select 62814,'MRN' from dual union all
select 62815,'MRN' from dual union all
select 62816,'MRN' from dual union all
select 62817,'MRN' from dual union all
select 62818,'MRN' from dual union all
select 62819,'MRN' from dual union all
select 6282,'AFCRL' from dual union all
select 62820,'MRN' from dual union all
select 62821,'MDD' from dual union all
select 62822,'AN' from dual union all
select 62823,'MRN' from dual union all
select 62824,'MRN' from dual union all
select 62825,'JMA' from dual union all
select 62826,'JMA' from dual union all
select 62827,'JMA' from dual union all
select 62828,'GMS' from dual union all
select 62829,'JMA' from dual union all
select 6283,'SMSA' from dual union all
select 62830,'JMA' from dual union all
select 62831,'JMA' from dual union all
select 62832,'JMA' from dual union all
select 62833,'JMA' from dual union all
select 62834,'JMA' from dual union all
select 62835,'JMA' from dual union all
select 62836,'JMA' from dual union all
select 62837,'JMA' from dual union all
select 62838,'JMA' from dual union all
select 62839,'MRN' from dual union all
select 6284,'AN' from dual union all
select 62840,'JMA' from dual union all
select 62841,'JMA' from dual union all
select 62842,'JMA' from dual union all
select 62843,'JMA' from dual union all
select 62844,'JMA' from dual union all
select 62845,'JMA' from dual union all
select 62846,'JMA' from dual union all
select 62847,'JMA' from dual union all
select 62848,'JMA' from dual union all
select 62849,'JMA' from dual union all
select 6285,'AN' from dual union all
select 62850,'AN' from dual union all
select 62851,'JMA' from dual union all
select 62852,'JMA' from dual union all
select 62853,'JMA' from dual union all
select 62854,'JMA' from dual union all
select 62855,'JMA' from dual union all
select 62856,'JMA' from dual union all
select 62857,'JMA' from dual union all
select 62858,'JMA' from dual union all
select 62859,'JMA' from dual union all
select 6286,'AN' from dual union all
select 62860,'JMA' from dual union all
select 62861,'GMS' from dual union all
select 62862,'GMS' from dual union all
select 62863,'MRN' from dual union all
select 62864,'MRN' from dual union all
select 62865,'AN' from dual union all
select 62866,'MRN' from dual union all
select 62867,'AN' from dual union all
select 62868,'GMS' from dual union all
select 62869,'MRN' from dual union all
select 6287,'AN' from dual union all
select 62870,'GMS' from dual union all
select 62871,'GMS' from dual union all
select 62872,'GMS' from dual union all
select 62873,'MRN' from dual union all
select 62874,'GMS' from dual union all
select 62875,'AN' from dual union all
select 62876,'GMS' from dual union all
select 62877,'GMS' from dual union all
select 62878,'AN' from dual union all
select 62879,'AN' from dual union all
select 6288,'AN' from dual union all
select 62880,'MRN' from dual union all
select 62881,'AN' from dual union all
select 62882,'MRN' from dual union all
select 62883,'MRN' from dual union all
select 62884,'GMS' from dual union all
select 62885,'MRN' from dual union all
select 62886,'GMS' from dual union all
select 62887,'GMS' from dual union all
select 62888,'MRN' from dual union all
select 62889,'GMS' from dual union all
select 6289,'AN' from dual union all
select 62890,'MRN' from dual union all
select 62891,'AN' from dual union all
select 62892,'MRN' from dual union all
select 62893,'GMS' from dual;
insert into LAUNCH_AGENCY
select 62894,'AN' from dual union all
select 62895,'MRN' from dual union all
select 62896,'MRN' from dual union all
select 62897,'MRN' from dual union all
select 62898,'MRN' from dual union all
select 62899,'AN' from dual union all
select 629,'USA' from dual union all
select 6290,'AN' from dual union all
select 62900,'MRN' from dual union all
select 62901,'MRN' from dual union all
select 62902,'AN' from dual union all
select 62903,'MRN' from dual union all
select 62904,'MRN' from dual union all
select 62905,'MRN' from dual union all
select 62906,'AN' from dual union all
select 62907,'MRN' from dual union all
select 62908,'MRN' from dual union all
select 62909,'MRN' from dual union all
select 6291,'AN' from dual union all
select 62910,'MRN' from dual union all
select 62911,'MRN' from dual union all
select 62912,'AN' from dual union all
select 62913,'MRN' from dual union all
select 62914,'AN' from dual union all
select 62915,'GMS' from dual union all
select 62916,'MRN' from dual union all
select 62917,'GMS' from dual union all
select 62918,'MRN' from dual union all
select 62919,'MRN' from dual union all
select 6292,'AN' from dual union all
select 62920,'MRN' from dual union all
select 62921,'MRN' from dual union all
select 62922,'MRN' from dual union all
select 62923,'MRN' from dual union all
select 62924,'MRN' from dual union all
select 62925,'GMS' from dual union all
select 62926,'MRN' from dual union all
select 62927,'AN' from dual union all
select 62928,'AN' from dual union all
select 62929,'AN' from dual union all
select 6293,'AN' from dual union all
select 62930,'AN' from dual union all
select 62931,'AN' from dual union all
select 62932,'AN' from dual union all
select 62933,'AN' from dual union all
select 62934,'AN' from dual union all
select 62935,'AN' from dual union all
select 62936,'AN' from dual union all
select 62937,'AN' from dual union all
select 62938,'MRN' from dual union all
select 62939,'MRN' from dual union all
select 6294,'AN' from dual union all
select 62940,'AN' from dual union all
select 62941,'AN' from dual union all
select 62942,'AN' from dual union all
select 62943,'AN' from dual union all
select 62944,'MRN' from dual union all
select 62945,'MRN' from dual union all
select 62946,'MRN' from dual union all
select 62947,'AN' from dual union all
select 62948,'MRN' from dual union all
select 62949,'MRN' from dual union all
select 6295,'AN' from dual union all
select 62950,'MRN' from dual union all
select 62951,'MRN' from dual union all
select 62952,'MRN' from dual union all
select 62953,'MRN' from dual union all
select 62954,'AN' from dual union all
select 62955,'AN' from dual union all
select 62956,'AN' from dual union all
select 62957,'AN' from dual union all
select 62958,'AN' from dual union all
select 62959,'AN' from dual union all
select 6296,'AN' from dual union all
select 62960,'AN' from dual union all
select 62961,'AN' from dual union all
select 62962,'AN' from dual union all
select 62963,'AN' from dual union all
select 62964,'AN' from dual union all
select 62965,'AN' from dual union all
select 62966,'AN' from dual union all
select 62967,'AN' from dual union all
select 62968,'AN' from dual union all
select 62969,'AN' from dual union all
select 6297,'AN' from dual union all
select 62970,'AN' from dual union all
select 62971,'AN' from dual union all
select 62972,'AN' from dual union all
select 62973,'AN' from dual union all
select 62974,'AN' from dual union all
select 62975,'AN' from dual union all
select 62976,'AN' from dual union all
select 62977,'AN' from dual union all
select 62978,'AN' from dual union all
select 62979,'AN' from dual union all
select 6298,'AN' from dual union all
select 62980,'AN' from dual union all
select 62981,'AN' from dual union all
select 62982,'AN' from dual union all
select 62983,'AN' from dual;
insert into LAUNCH_AGENCY
select 62984,'AN' from dual union all
select 62985,'AN' from dual union all
select 62986,'AN' from dual union all
select 62987,'AN' from dual union all
select 62988,'AN' from dual union all
select 62989,'AN' from dual union all
select 6299,'AN' from dual union all
select 62990,'MRN' from dual union all
select 62991,'AN' from dual union all
select 62992,'AN' from dual union all
select 62993,'AN' from dual union all
select 62994,'AN' from dual union all
select 62995,'AN' from dual union all
select 62996,'MRN' from dual union all
select 62997,'AN' from dual union all
select 62998,'AN' from dual union all
select 62999,'AN' from dual union all
select 63,'WEHR' from dual union all
select 630,'USA' from dual union all
select 6300,'AN' from dual union all
select 63000,'AN' from dual union all
select 63001,'AN' from dual union all
select 63002,'AN' from dual union all
select 63003,'AN' from dual union all
select 63004,'AN' from dual union all
select 63005,'AN' from dual union all
select 63006,'AN' from dual union all
select 63007,'AN' from dual union all
select 63008,'AN' from dual union all
select 63009,'MRN' from dual union all
select 6301,'AN' from dual union all
select 63010,'MRN' from dual union all
select 63011,'AN' from dual union all
select 63012,'AN' from dual union all
select 63013,'AN' from dual union all
select 63014,'AN' from dual union all
select 63015,'AN' from dual union all
select 63016,'MRN' from dual union all
select 63017,'MRN' from dual union all
select 63018,'AN' from dual union all
select 63019,'AN' from dual union all
select 6302,'AN' from dual union all
select 63020,'AN' from dual union all
select 63021,'AN' from dual union all
select 63022,'AN' from dual union all
select 63023,'AN' from dual union all
select 63024,'AN' from dual union all
select 63025,'AN' from dual union all
select 63026,'AN' from dual union all
select 63027,'AN' from dual union all
select 63028,'AN' from dual union all
select 63029,'AN' from dual union all
select 6303,'AN' from dual union all
select 63030,'AN' from dual union all
select 63031,'AN' from dual union all
select 63032,'AN' from dual union all
select 63033,'AN' from dual union all
select 63034,'AN' from dual union all
select 63035,'AN' from dual union all
select 63036,'AN' from dual union all
select 63037,'AN' from dual union all
select 63038,'AN' from dual union all
select 63039,'AN' from dual union all
select 6304,'AN' from dual union all
select 63040,'AN' from dual union all
select 63041,'AN' from dual union all
select 63042,'AN' from dual union all
select 63043,'AN' from dual union all
select 63044,'AN' from dual union all
select 63045,'AN' from dual union all
select 63046,'AN' from dual union all
select 63047,'AN' from dual union all
select 63048,'AN' from dual union all
select 63049,'AN' from dual union all
select 6305,'AN' from dual union all
select 63050,'AN' from dual union all
select 63051,'AN' from dual union all
select 63052,'AN' from dual union all
select 63053,'AN' from dual union all
select 63054,'AN' from dual union all
select 63055,'AN' from dual union all
select 63056,'AN' from dual union all
select 63057,'AN' from dual union all
select 63058,'AN' from dual union all
select 63059,'AN' from dual union all
select 6306,'AN' from dual union all
select 63060,'AN' from dual union all
select 63061,'AN' from dual union all
select 63062,'AN' from dual union all
select 63063,'AN' from dual union all
select 63064,'AN' from dual union all
select 63065,'AN' from dual union all
select 63066,'AN' from dual union all
select 63067,'AN' from dual union all
select 63068,'AN' from dual union all
select 63069,'AN' from dual union all
select 6307,'AN' from dual union all
select 63070,'AN' from dual union all
select 63071,'AN' from dual union all
select 63072,'AN' from dual;
insert into LAUNCH_AGENCY
select 63073,'AN' from dual union all
select 63074,'AN' from dual union all
select 63075,'AN' from dual union all
select 63076,'AN' from dual union all
select 63077,'AN' from dual union all
select 63078,'AN' from dual union all
select 63079,'AN' from dual union all
select 6308,'AN' from dual union all
select 63080,'AN' from dual union all
select 63081,'AN' from dual union all
select 63082,'AN' from dual union all
select 63083,'AN' from dual union all
select 63084,'AN' from dual union all
select 63085,'AN' from dual union all
select 63086,'AN' from dual union all
select 63087,'AN' from dual union all
select 63088,'AN' from dual union all
select 63089,'AN' from dual union all
select 6309,'AN' from dual union all
select 63090,'MRN' from dual union all
select 63091,'AN' from dual union all
select 63092,'AN' from dual union all
select 63093,'AN' from dual union all
select 63094,'AN' from dual union all
select 63095,'AN' from dual union all
select 63096,'AN' from dual union all
select 63097,'AN' from dual union all
select 63098,'AN' from dual union all
select 63099,'AN' from dual union all
select 631,'USA' from dual union all
select 6310,'AN' from dual union all
select 63100,'AN' from dual union all
select 63101,'AN' from dual union all
select 63102,'AN' from dual union all
select 63103,'MRN' from dual union all
select 63104,'MRN' from dual union all
select 63105,'AN' from dual union all
select 63106,'AN' from dual union all
select 63107,'AN' from dual union all
select 63108,'AN' from dual union all
select 63109,'AN' from dual union all
select 6311,'AN' from dual union all
select 63110,'AN' from dual union all
select 63111,'AN' from dual union all
select 63112,'AN' from dual union all
select 63113,'AN' from dual union all
select 63114,'AN' from dual union all
select 63115,'AN' from dual union all
select 63116,'AN' from dual union all
select 63117,'AN' from dual union all
select 63118,'AN' from dual union all
select 63119,'AN' from dual union all
select 6312,'AN' from dual union all
select 63120,'AN' from dual union all
select 63121,'AN' from dual union all
select 63122,'AN' from dual union all
select 63123,'AN' from dual union all
select 63124,'AN' from dual union all
select 63125,'AN' from dual union all
select 63126,'MRN' from dual union all
select 63127,'AN' from dual union all
select 63128,'AN' from dual union all
select 63129,'AN' from dual union all
select 6313,'AN' from dual union all
select 63130,'AN' from dual union all
select 63131,'AN' from dual union all
select 63132,'AN' from dual union all
select 63133,'AN' from dual union all
select 63134,'AN' from dual union all
select 63135,'AN' from dual union all
select 63136,'AN' from dual union all
select 63137,'AN' from dual union all
select 63138,'AN' from dual union all
select 63139,'AN' from dual union all
select 6314,'AN' from dual union all
select 63140,'AN' from dual union all
select 63141,'AN' from dual union all
select 63142,'AN' from dual union all
select 63143,'AN' from dual union all
select 63144,'MRN' from dual union all
select 63145,'AN' from dual union all
select 63146,'AN' from dual union all
select 63147,'MRN' from dual union all
select 63148,'MRN' from dual union all
select 63149,'AN' from dual union all
select 6315,'AN' from dual union all
select 63150,'MRN' from dual union all
select 63151,'MRN' from dual union all
select 63152,'MRN' from dual union all
select 63153,'AN' from dual union all
select 63154,'AN' from dual union all
select 63155,'MRN' from dual union all
select 63156,'AN' from dual union all
select 63157,'AN' from dual union all
select 63158,'AN' from dual union all
select 63159,'AN' from dual union all
select 6316,'AN' from dual union all
select 63160,'AN' from dual union all
select 63161,'AN' from dual union all
select 63162,'AN' from dual;
insert into LAUNCH_AGENCY
select 63163,'MRN' from dual union all
select 63164,'AN' from dual union all
select 63165,'MRN' from dual union all
select 63166,'MRN' from dual union all
select 63167,'AN' from dual union all
select 63168,'AN' from dual union all
select 63169,'AN' from dual union all
select 6317,'USAF' from dual union all
select 63170,'AN' from dual union all
select 63171,'AN' from dual union all
select 63172,'AN' from dual union all
select 63173,'AN' from dual union all
select 63174,'AN' from dual union all
select 63175,'AN' from dual union all
select 63176,'AN' from dual union all
select 63177,'AN' from dual union all
select 63178,'AN' from dual union all
select 63179,'AN' from dual union all
select 6318,'USAF' from dual union all
select 63180,'AN' from dual union all
select 63181,'AN' from dual union all
select 63182,'AN' from dual union all
select 63183,'AN' from dual union all
select 63184,'AN' from dual union all
select 63185,'AN' from dual union all
select 63186,'AN' from dual union all
select 63187,'AN' from dual union all
select 63188,'AN' from dual union all
select 63189,'AN' from dual union all
select 6319,'USAF' from dual union all
select 63190,'AN' from dual union all
select 63191,'AN' from dual union all
select 63192,'AN' from dual union all
select 63193,'AN' from dual union all
select 63194,'AN' from dual union all
select 63195,'AN' from dual union all
select 63196,'AN' from dual union all
select 63197,'AN' from dual union all
select 63198,'AN' from dual union all
select 63199,'AN' from dual union all
select 632,'USA' from dual union all
select 6320,'RVSN' from dual union all
select 63200,'AN' from dual union all
select 63201,'AN' from dual union all
select 63202,'AN' from dual union all
select 63203,'AN' from dual union all
select 63204,'AN' from dual union all
select 63205,'AN' from dual union all
select 63206,'AN' from dual union all
select 63207,'AN' from dual union all
select 63208,'AN' from dual union all
select 63209,'AN' from dual union all
select 6321,'USAF' from dual union all
select 63210,'AN' from dual union all
select 63211,'AN' from dual union all
select 63212,'AN' from dual union all
select 63213,'AN' from dual union all
select 63214,'AN' from dual union all
select 63215,'AN' from dual union all
select 63216,'AN' from dual union all
select 63217,'AN' from dual union all
select 63218,'MRN' from dual union all
select 63219,'MRN' from dual union all
select 6322,'USAF' from dual union all
select 63220,'AN' from dual union all
select 63221,'AN' from dual union all
select 63222,'MRN' from dual union all
select 63223,'AN' from dual union all
select 63224,'MRN' from dual union all
select 63225,'MRN' from dual union all
select 63226,'MRN' from dual union all
select 63227,'AN' from dual union all
select 63228,'AN' from dual union all
select 63229,'AN' from dual union all
select 6323,'NASA' from dual union all
select 63230,'AN' from dual union all
select 63231,'AN' from dual union all
select 63232,'AN' from dual union all
select 63233,'AN' from dual union all
select 63234,'AN' from dual union all
select 63235,'AN' from dual union all
select 63236,'AN' from dual union all
select 63237,'AN' from dual union all
select 63238,'AN' from dual union all
select 63239,'AN' from dual union all
select 6324,'USAF' from dual union all
select 63240,'AN' from dual union all
select 63241,'AN' from dual union all
select 63242,'AN' from dual union all
select 63243,'MMA' from dual union all
select 63244,'NASA' from dual union all
select 63245,'UNKS' from dual union all
select 63246,'UNKS' from dual union all
select 63247,'AE' from dual union all
select 63248,'UNKS' from dual union all
select 63249,'ISAS' from dual union all
select 6325,'RVSN' from dual union all
select 63250,'MDSSC' from dual union all
select 63251,'UNKS' from dual union all
select 63252,'UNKS' from dual;
insert into LAUNCH_AGENCY
select 63253,'CALT' from dual union all
select 63254,'UNKS' from dual union all
select 63255,'NASDA' from dual union all
select 63256,'UNKS' from dual union all
select 63257,'MDSSC' from dual union all
select 63258,'UNKS' from dual union all
select 63259,'UNKS' from dual union all
select 6326,'USAF' from dual union all
select 63260,'UNKS' from dual union all
select 63261,'NASA' from dual union all
select 63262,'UNKS' from dual union all
select 63263,'MMA' from dual union all
select 63264,'UNKS' from dual union all
select 63265,'UNKS' from dual union all
select 63266,'UNKS' from dual union all
select 63267,'MDSSC' from dual union all
select 63268,'UNKS' from dual union all
select 63269,'ISA' from dual union all
select 6327,'USAF' from dual union all
select 63270,'OSC' from dual union all
select 63271,'UNKS' from dual union all
select 63272,'CALT' from dual union all
select 63273,'AFSC' from dual union all
select 63274,'UNKS' from dual union all
select 63275,'UNKS' from dual union all
select 63276,'MDSSC' from dual union all
select 63277,'UNKS' from dual union all
select 63278,'UNKS' from dual union all
select 63279,'NASA' from dual union all
select 6328,'USAF' from dual union all
select 63280,'UNKS' from dual union all
select 63281,'UNKS' from dual union all
select 63282,'UNKS' from dual union all
select 63283,'UNKS' from dual union all
select 63284,'UNKS' from dual union all
select 63285,'NASA' from dual union all
select 63286,'UNKS' from dual union all
select 63287,'UNKS' from dual union all
select 63288,'UNKS' from dual union all
select 63289,'UNKS' from dual union all
select 6329,'NASA' from dual union all
select 63290,'UNKS' from dual union all
select 63291,'MDSSC' from dual union all
select 63292,'AFSC' from dual union all
select 63293,'MDSSC' from dual union all
select 63294,'UNKS' from dual union all
select 63295,'UNKS' from dual union all
select 63296,'UNKS' from dual union all
select 63297,'UNKS' from dual union all
select 63298,'MMA' from dual union all
select 63299,'UNKS' from dual union all
select 633,'USA' from dual union all
select 6330,'RVSN' from dual union all
select 63300,'UNKS' from dual union all
select 63301,'CALT' from dual union all
select 63302,'UNKS' from dual union all
select 63303,'UNKS' from dual union all
select 63304,'UNKS' from dual union all
select 63305,'AE' from dual union all
select 63306,'UNKS' from dual union all
select 63307,'GDCLS' from dual union all
select 63308,'UNKS' from dual union all
select 63309,'UNKS' from dual union all
select 6331,'USAF' from dual union all
select 63310,'MDSSC' from dual union all
select 63311,'UNKS' from dual union all
select 63312,'UNKS' from dual union all
select 63313,'UNKS' from dual union all
select 63314,'UNKS' from dual union all
select 63315,'UNKS' from dual union all
select 63316,'MDSSC' from dual union all
select 63317,'UNKS' from dual union all
select 63318,'UNKS' from dual union all
select 63319,'NASDA' from dual union all
select 6332,'USAF' from dual union all
select 63320,'UNKS' from dual union all
select 63321,'AE' from dual union all
select 63322,'UNKS' from dual union all
select 63323,'SAST' from dual union all
select 63324,'UNKS' from dual union all
select 63325,'UNKS' from dual union all
select 63326,'UNKS' from dual union all
select 63327,'UNKS' from dual union all
select 63328,'UNKS' from dual union all
select 63329,'UNKS' from dual union all
select 6333,'USAF' from dual union all
select 63330,'MDSSC' from dual union all
select 63331,'CALT' from dual union all
select 63332,'NASA' from dual union all
select 63333,'AE' from dual union all
select 63334,'UNKS' from dual union all
select 63335,'MDSSC' from dual union all
select 63336,'UNKS' from dual union all
select 63337,'AFSC' from dual union all
select 63338,'UNKS' from dual union all
select 63339,'NASA' from dual union all
select 6334,'NASA' from dual union all
select 63340,'UNKS' from dual union all
select 63341,'UNKS' from dual union all
select 63342,'AE' from dual;
insert into LAUNCH_AGENCY
select 63343,'UNKS' from dual union all
select 63344,'UNKS' from dual union all
select 63345,'MDSSC' from dual union all
select 63346,'UNKS' from dual union all
select 63347,'AFSPC' from dual union all
select 63348,'NASA' from dual union all
select 63349,'UNKS' from dual union all
select 6335,'USAF' from dual union all
select 63350,'UNKS' from dual union all
select 63351,'UNKS' from dual union all
select 63352,'UNKS' from dual union all
select 63353,'UNKS' from dual union all
select 63354,'UNKS' from dual union all
select 63355,'UNKS' from dual union all
select 63356,'UNKS' from dual union all
select 63357,'UNKS' from dual union all
select 63358,'UNKS' from dual union all
select 63359,'SDIO' from dual union all
select 6336,'RAE' from dual union all
select 63360,'AN' from dual union all
select 63361,'SDIO' from dual union all
select 63362,'IAI' from dual union all
select 63363,'MRN' from dual union all
select 63364,'MRN' from dual union all
select 63365,'GMS' from dual union all
select 63366,'GMS' from dual union all
select 63367,'IQMIC' from dual union all
select 63368,'IQMIC' from dual union all
select 63369,'DLR' from dual union all
select 6337,'PRO' from dual union all
select 63370,'NASA' from dual union all
select 63371,'DLR' from dual union all
select 63372,'NASA' from dual union all
select 63373,'RVSN' from dual union all
select 63374,'AE' from dual union all
select 63375,'UNKS' from dual union all
select 63376,'UNKS' from dual union all
select 63377,'UNKS' from dual union all
select 63378,'UNKS' from dual union all
select 63379,'ISRO' from dual union all
select 6338,'PRO' from dual union all
select 63380,'ISRO' from dual union all
select 63381,'ISRO' from dual union all
select 63382,'ISRO' from dual union all
select 63383,'ISRO' from dual union all
select 63384,'ISRO' from dual union all
select 63385,'ISRO' from dual union all
select 63386,'ISRO' from dual union all
select 63387,'ISRO' from dual union all
select 63388,'ISRO' from dual union all
select 63389,'ISRO' from dual union all
select 6339,'PRO' from dual union all
select 63390,'ISRO' from dual union all
select 63391,'ISRO' from dual union all
select 63392,'ISRO' from dual union all
select 63393,'ISRO' from dual union all
select 63394,'ISRO' from dual union all
select 63395,'IQMIC' from dual union all
select 63396,'ISRO' from dual union all
select 63397,'ISRO' from dual union all
select 63398,'ISRO' from dual union all
select 63399,'ISRO' from dual union all
select 634,'USA' from dual union all
select 6340,'PRO' from dual union all
select 63400,'ISRO' from dual union all
select 63401,'ISRO' from dual union all
select 63402,'ISRO' from dual union all
select 63403,'ISRO' from dual union all
select 63404,'ISRO' from dual union all
select 63405,'ISRO' from dual union all
select 63406,'ISRO' from dual union all
select 63407,'ISRO' from dual union all
select 63408,'ISRO' from dual union all
select 63409,'ISRO' from dual union all
select 6341,'PRO' from dual union all
select 63410,'ISRO' from dual union all
select 63411,'ISRO' from dual union all
select 63412,'ISRO' from dual union all
select 63413,'ISRO' from dual union all
select 63414,'ISRO' from dual union all
select 63415,'ISRO' from dual union all
select 63416,'ISRO' from dual union all
select 63417,'ISRO' from dual union all
select 63418,'ISRO' from dual union all
select 63419,'ISRO' from dual union all
select 6342,'PRO' from dual union all
select 63420,'ISRO' from dual union all
select 63421,'ISRO' from dual union all
select 63422,'AN' from dual union all
select 63423,'AN' from dual union all
select 63424,'AN' from dual union all
select 63425,'AN' from dual union all
select 63426,'AN' from dual union all
select 63427,'RVSN' from dual union all
select 63428,'RVSN' from dual union all
select 63429,'RVSN' from dual union all
select 6343,'PRO' from dual union all
select 63430,'RVSN' from dual union all
select 63431,'RVSN' from dual union all
select 63432,'NASA' from dual;
insert into LAUNCH_AGENCY
select 63433,'RVSN' from dual union all
select 63434,'SAC' from dual union all
select 63435,'AERO' from dual union all
select 63436,'NASA' from dual union all
select 63437,'DLR' from dual union all
select 63438,'NASA' from dual union all
select 63439,'AN' from dual union all
select 6344,'PRO' from dual union all
select 63440,'AN' from dual union all
select 63441,'AN' from dual union all
select 63442,'RVSN' from dual union all
select 63443,'AN' from dual union all
select 63444,'AN' from dual union all
select 63445,'RVSN' from dual union all
select 63446,'NASA' from dual union all
select 63447,'DLR' from dual union all
select 63448,'RVSN' from dual union all
select 63449,'NASA' from dual union all
select 6345,'PRO' from dual union all
select 63450,'IQ' from dual union all
select 63451,'NASA' from dual union all
select 63452,'NASA' from dual union all
select 63453,'RVSN' from dual union all
select 63454,'RVSN' from dual union all
select 63455,'RVSN' from dual union all
select 63456,'NASA' from dual union all
select 63457,'NASA' from dual union all
select 63458,'SDIO' from dual union all
select 63459,'RVSN' from dual union all
select 6346,'NASA' from dual union all
select 63460,'AN' from dual union all
select 63461,'AN' from dual union all
select 63462,'AN' from dual union all
select 63463,'AN' from dual union all
select 63464,'RVSN' from dual union all
select 63465,'AN' from dual union all
select 63466,'NASA' from dual union all
select 63467,'SAC' from dual union all
select 63468,'SAC' from dual union all
select 63469,'USN' from dual union all
select 6347,'NASA' from dual union all
select 6347,'USAF' from dual union all
select 63470,'AN' from dual union all
select 63471,'AN' from dual union all
select 63472,'AN' from dual union all
select 63473,'AN' from dual union all
select 63474,'NASA' from dual union all
select 63475,'RVSN' from dual union all
select 63476,'AERO' from dual union all
select 63477,'RVSN' from dual union all
select 63478,'SDIO' from dual union all
select 63479,'AN' from dual union all
select 6348,'NASA' from dual union all
select 63480,'AN' from dual union all
select 63481,'AN' from dual union all
select 63482,'AN' from dual union all
select 63483,'RVSN' from dual union all
select 63484,'USN' from dual union all
select 63485,'USN' from dual union all
select 63486,'RVSN' from dual union all
select 63487,'USN' from dual union all
select 63488,'USN' from dual union all
select 63489,'SAC' from dual union all
select 6349,'USAF' from dual union all
select 63490,'MBB' from dual union all
select 63491,'HOUW' from dual union all
select 63491,'SAAF' from dual union all
select 63492,'VKS' from dual union all
select 63493,'NASA' from dual union all
select 63494,'NASA' from dual union all
select 63495,'DLR' from dual union all
select 63495,'SSC' from dual union all
select 63496,'IAE' from dual union all
select 63496,'INPE' from dual union all
select 63497,'RVSN' from dual union all
select 63498,'RVSN' from dual union all
select 63499,'USN' from dual union all
select 635,'USA' from dual union all
select 6350,'NASA' from dual union all
select 63500,'AN' from dual union all
select 63501,'AN' from dual union all
select 63502,'AN' from dual union all
select 63503,'IQ' from dual union all
select 63504,'IQ' from dual union all
select 63505,'IQ' from dual union all
select 63506,'IAE' from dual union all
select 63507,'RVSN' from dual union all
select 63508,'RVSN' from dual union all
select 63509,'OSC' from dual union all
select 6351,'RAE' from dual union all
select 63510,'IDF' from dual union all
select 63511,'RVSN' from dual union all
select 63512,'RVSN' from dual union all
select 63513,'RVSN' from dual union all
select 63514,'INTA' from dual union all
select 63515,'INTA' from dual union all
select 63516,'INTA' from dual union all
select 63517,'IQ' from dual union all
select 63518,'IQ' from dual union all
select 63519,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 6352,'RAE' from dual union all
select 63520,'IQ' from dual union all
select 63521,'IQ' from dual union all
select 63522,'IQ' from dual union all
select 63523,'IQ' from dual union all
select 63524,'IQ' from dual union all
select 63525,'IQ' from dual union all
select 63526,'IQ' from dual union all
select 63527,'IQ' from dual union all
select 63528,'PLA' from dual union all
select 63529,'DLR' from dual union all
select 6353,'USA' from dual union all
select 63530,'RVSN' from dual union all
select 63531,'IQ' from dual union all
select 63532,'RVSN' from dual union all
select 63533,'INTA' from dual union all
select 63534,'INTA' from dual union all
select 63535,'INTA' from dual union all
select 63536,'RVSN' from dual union all
select 63537,'RVSN' from dual union all
select 63538,'RVSN' from dual union all
select 63539,'RVSN' from dual union all
select 6354,'USA' from dual union all
select 63540,'RVSN' from dual union all
select 63541,'RVSN' from dual union all
select 63542,'RVSN' from dual union all
select 63543,'USN' from dual union all
select 63544,'USN' from dual union all
select 63545,'CSA' from dual union all
select 63546,'NDRE' from dual union all
select 63547,'NASA' from dual union all
select 63548,'DLR' from dual union all
select 63549,'ISRO' from dual union all
select 6355,'USA' from dual union all
select 63550,'ISAS' from dual union all
select 63551,'OSC' from dual union all
select 63552,'NASA' from dual union all
select 63553,'SAC' from dual union all
select 63554,'USN' from dual union all
select 63555,'USN' from dual union all
select 63556,'AFSC' from dual union all
select 63557,'DLR' from dual union all
select 63558,'SSC' from dual union all
select 63559,'IAE' from dual union all
select 63559,'INPE' from dual union all
select 6356,'PRO' from dual union all
select 63560,'ISRO' from dual union all
select 63561,'ISAS' from dual union all
select 63562,'NASA' from dual union all
select 63563,'NDRE' from dual union all
select 63564,'SSC' from dual union all
select 63565,'ISAS' from dual union all
select 63566,'ISRO' from dual union all
select 63567,'SSC' from dual union all
select 63568,'RVSN' from dual union all
select 63569,'RVSN' from dual union all
select 6357,'USA' from dual union all
select 63570,'NDRE' from dual union all
select 63571,'DLR' from dual union all
select 63572,'DLR' from dual union all
select 63573,'DLR' from dual union all
select 63574,'SAC' from dual union all
select 63575,'NDRE' from dual union all
select 63576,'NDRE' from dual union all
select 63577,'NASA' from dual union all
select 63578,'NDRE' from dual union all
select 63579,'USN' from dual union all
select 6358,'RVSN' from dual union all
select 63580,'DLR' from dual union all
select 63581,'NASA' from dual union all
select 63582,'ALCS' from dual union all
select 63582,'SAC' from dual union all
select 63583,'NASA' from dual union all
select 63584,'NASA' from dual union all
select 63585,'ALCS' from dual union all
select 63585,'SAC' from dual union all
select 63586,'USN' from dual union all
select 63587,'USN' from dual union all
select 63588,'USN' from dual union all
select 63589,'USN' from dual union all
select 6359,'STA' from dual union all
select 63590,'SDIO' from dual union all
select 63591,'RVSN' from dual union all
select 63592,'RVSN' from dual union all
select 63593,'SSC' from dual union all
select 63594,'NASA' from dual union all
select 63595,'NASA' from dual union all
select 63596,'NASA' from dual union all
select 63597,'NASA' from dual union all
select 63598,'NASA' from dual union all
select 63599,'SDIO' from dual union all
select 636,'USA' from dual union all
select 6360,'RAE' from dual union all
select 63600,'SDIO' from dual union all
select 63601,'NASA' from dual union all
select 63602,'KPA' from dual union all
select 63603,'USN' from dual union all
select 63604,'MBB' from dual union all
select 63605,'NASA' from dual union all
select 63606,'NASA' from dual;
insert into LAUNCH_AGENCY
select 63607,'NASA' from dual union all
select 63608,'MBB' from dual union all
select 63609,'RVSN' from dual union all
select 6361,'STA' from dual union all
select 63610,'SAC' from dual union all
select 63611,'SSI' from dual union all
select 63612,'RVSN' from dual union all
select 63613,'RVSN' from dual union all
select 63614,'RVSN' from dual union all
select 63615,'SSC' from dual union all
select 63616,'KPA' from dual union all
select 63617,'GMS' from dual union all
select 63618,'GMS' from dual union all
select 63619,'AN' from dual union all
select 6362,'NASA' from dual union all
select 6362,'USAF' from dual union all
select 63620,'GMS' from dual union all
select 63621,'GMS' from dual union all
select 63622,'GMS' from dual union all
select 63623,'GMS' from dual union all
select 63624,'GMS' from dual union all
select 63625,'GMS' from dual union all
select 63626,'AN' from dual union all
select 63627,'AN' from dual union all
select 63628,'AN' from dual union all
select 63629,'AN' from dual union all
select 6363,'SAC' from dual union all
select 63630,'GMS' from dual union all
select 63631,'MRN' from dual union all
select 63632,'MRN' from dual union all
select 63633,'AN' from dual union all
select 63634,'AN' from dual union all
select 63635,'AN' from dual union all
select 63636,'JMA' from dual union all
select 63637,'AN' from dual union all
select 63638,'GMS' from dual union all
select 63639,'AN' from dual union all
select 6364,'NASA' from dual union all
select 6364,'USAF' from dual union all
select 63640,'AN' from dual union all
select 63641,'AN' from dual union all
select 63642,'MRN' from dual union all
select 63643,'GMS' from dual union all
select 63644,'AN' from dual union all
select 63645,'AN' from dual union all
select 63646,'AN' from dual union all
select 63647,'AN' from dual union all
select 63648,'AN' from dual union all
select 63649,'GMS' from dual union all
select 6365,'USAF' from dual union all
select 63650,'AN' from dual union all
select 63651,'AN' from dual union all
select 63652,'AN' from dual union all
select 63653,'AN' from dual union all
select 63654,'AN' from dual union all
select 63655,'JMA' from dual union all
select 63656,'GMS' from dual union all
select 63657,'AN' from dual union all
select 63658,'AN' from dual union all
select 63659,'JMA' from dual union all
select 6366,'USAF' from dual union all
select 63660,'GMS' from dual union all
select 63661,'AN' from dual union all
select 63662,'AN' from dual union all
select 63663,'GMS' from dual union all
select 63664,'AN' from dual union all
select 63665,'GMS' from dual union all
select 63666,'MRN' from dual union all
select 63667,'MRN' from dual union all
select 63668,'MRN' from dual union all
select 63669,'MRN' from dual union all
select 6367,'RAE' from dual union all
select 63670,'GMS' from dual union all
select 63671,'GMS' from dual union all
select 63672,'GMS' from dual union all
select 63673,'GMS' from dual union all
select 63674,'GMS' from dual union all
select 63675,'GMS' from dual union all
select 63676,'GMS' from dual union all
select 63677,'MRN' from dual union all
select 63678,'MRN' from dual union all
select 63679,'MRN' from dual union all
select 6368,'USAF' from dual union all
select 63680,'MRN' from dual union all
select 63681,'MDD' from dual union all
select 63682,'MRN' from dual union all
select 63683,'MRN' from dual union all
select 63684,'MRN' from dual union all
select 63685,'MRN' from dual union all
select 63686,'MRN' from dual union all
select 63687,'ISAS' from dual union all
select 63688,'MRN' from dual union all
select 63689,'GMS' from dual union all
select 6369,'PRO' from dual union all
select 63690,'MRN' from dual union all
select 63691,'MRN' from dual union all
select 63692,'AN' from dual union all
select 63693,'MRN' from dual union all
select 63694,'GMS' from dual union all
select 63695,'MRN' from dual;
insert into LAUNCH_AGENCY
select 63696,'MRN' from dual union all
select 63697,'MRN' from dual union all
select 63698,'MRN' from dual union all
select 63699,'MRN' from dual union all
select 637,'USA' from dual union all
select 6370,'NASA' from dual union all
select 6370,'USAF' from dual union all
select 63700,'MRN' from dual union all
select 63701,'MRN' from dual union all
select 63702,'MRN' from dual union all
select 63703,'AN' from dual union all
select 63704,'MRN' from dual union all
select 63705,'GMS' from dual union all
select 63706,'GMS' from dual union all
select 63707,'MRN' from dual union all
select 63708,'GMS' from dual union all
select 63709,'MRN' from dual union all
select 6371,'USAF' from dual union all
select 63710,'GMS' from dual union all
select 63711,'GMS' from dual union all
select 63712,'GMS' from dual union all
select 63713,'GMS' from dual union all
select 63714,'AN' from dual union all
select 63715,'AN' from dual union all
select 63716,'GMS' from dual union all
select 63717,'AN' from dual union all
select 63718,'AN' from dual union all
select 63719,'GMS' from dual union all
select 6372,'NASA' from dual union all
select 6372,'USAF' from dual union all
select 63720,'GMS' from dual union all
select 63721,'GMS' from dual union all
select 63722,'GMS' from dual union all
select 63723,'AN' from dual union all
select 63724,'GMS' from dual union all
select 63725,'AN' from dual union all
select 63726,'ISAS' from dual union all
select 63727,'JMA' from dual union all
select 63728,'JMA' from dual union all
select 63729,'JMA' from dual union all
select 6373,'USAF' from dual union all
select 63730,'JMA' from dual union all
select 63731,'JMA' from dual union all
select 63732,'JMA' from dual union all
select 63733,'JMA' from dual union all
select 63734,'JMA' from dual union all
select 63735,'JMA' from dual union all
select 63736,'GMS' from dual union all
select 63737,'JMA' from dual union all
select 63738,'JMA' from dual union all
select 63739,'JMA' from dual union all
select 6374,'USAF' from dual union all
select 63740,'JMA' from dual union all
select 63741,'JMA' from dual union all
select 63742,'JMA' from dual union all
select 63743,'JMA' from dual union all
select 63744,'JMA' from dual union all
select 63745,'JMA' from dual union all
select 63746,'JMA' from dual union all
select 63747,'GMS' from dual union all
select 63748,'JMA' from dual union all
select 63749,'JMA' from dual union all
select 6375,'NASA' from dual union all
select 63750,'JMA' from dual union all
select 63751,'JMA' from dual union all
select 63752,'JMA' from dual union all
select 63753,'JMA' from dual union all
select 63754,'JMA' from dual union all
select 63755,'JMA' from dual union all
select 63756,'JMA' from dual union all
select 63757,'DLR' from dual union all
select 63758,'GMS' from dual union all
select 63759,'GMS' from dual union all
select 6376,'NASA' from dual union all
select 6376,'USAF' from dual union all
select 63760,'AN' from dual union all
select 63761,'GMS' from dual union all
select 63762,'GMS' from dual union all
select 63763,'AN' from dual union all
select 63764,'AN' from dual union all
select 63765,'AN' from dual union all
select 63766,'AN' from dual union all
select 63767,'MRN' from dual union all
select 63768,'MRN' from dual union all
select 63769,'GMS' from dual union all
select 6377,'RAE' from dual union all
select 63770,'MRN' from dual union all
select 63771,'MDD' from dual union all
select 63772,'MRN' from dual union all
select 63773,'MRN' from dual union all
select 63774,'GMS' from dual union all
select 63775,'GMS' from dual union all
select 63776,'AN' from dual union all
select 63777,'GMS' from dual union all
select 63778,'AN' from dual union all
select 63779,'JMA' from dual union all
select 6378,'RAE' from dual union all
select 63780,'GMS' from dual union all
select 63781,'AN' from dual union all
select 63782,'GMS' from dual;
insert into LAUNCH_AGENCY
select 63783,'AN' from dual union all
select 63784,'AN' from dual union all
select 63785,'MRN' from dual union all
select 63786,'GMS' from dual union all
select 63787,'GMS' from dual union all
select 63788,'AN' from dual union all
select 63789,'GMS' from dual union all
select 6379,'NASA' from dual union all
select 6379,'USAF' from dual union all
select 63790,'GMS' from dual union all
select 63791,'JMA' from dual union all
select 63792,'AN' from dual union all
select 63793,'AN' from dual union all
select 63794,'GMS' from dual union all
select 63795,'GMS' from dual union all
select 63796,'AN' from dual union all
select 63797,'AN' from dual union all
select 63798,'AN' from dual union all
select 63799,'MRN' from dual union all
select 638,'ORD' from dual union all
select 6380,'USA' from dual union all
select 63800,'AN' from dual union all
select 63801,'AN' from dual union all
select 63802,'AN' from dual union all
select 63803,'AN' from dual union all
select 63804,'AN' from dual union all
select 63805,'JMA' from dual union all
select 63806,'AN' from dual union all
select 63807,'AN' from dual union all
select 63808,'AN' from dual union all
select 63809,'MRN' from dual union all
select 6381,'USA' from dual union all
select 63810,'GMS' from dual union all
select 63811,'GMS' from dual union all
select 63812,'JMA' from dual union all
select 63813,'GMS' from dual union all
select 63814,'AN' from dual union all
select 63815,'MRN' from dual union all
select 63816,'MDD' from dual union all
select 63817,'AN' from dual union all
select 63818,'MRN' from dual union all
select 63819,'GMS' from dual union all
select 6382,'USA' from dual union all
select 63820,'AN' from dual union all
select 63821,'JMA' from dual union all
select 63822,'ISAS' from dual union all
select 63823,'AN' from dual union all
select 63824,'AN' from dual union all
select 63825,'GMS' from dual union all
select 63826,'AN' from dual union all
select 63827,'AN' from dual union all
select 63828,'AN' from dual union all
select 63829,'AN' from dual union all
select 6383,'USA' from dual union all
select 63830,'AN' from dual union all
select 63831,'AN' from dual union all
select 63832,'GMS' from dual union all
select 63833,'AN' from dual union all
select 63834,'JMA' from dual union all
select 63835,'GMS' from dual union all
select 63836,'AN' from dual union all
select 63837,'AN' from dual union all
select 63838,'AN' from dual union all
select 63839,'AN' from dual union all
select 6384,'USA' from dual union all
select 63840,'AN' from dual union all
select 63841,'AN' from dual union all
select 63842,'AN' from dual union all
select 63843,'AN' from dual union all
select 63844,'AN' from dual union all
select 63845,'AN' from dual union all
select 63846,'DLR' from dual union all
select 63847,'DGA' from dual union all
select 63848,'DLR' from dual union all
select 63849,'INTA' from dual union all
select 6385,'USA' from dual union all
select 63850,'AETE' from dual union all
select 63850,'NASA' from dual union all
select 63851,'DLR' from dual union all
select 63852,'AN' from dual union all
select 63853,'ISAS' from dual union all
select 63854,'AN' from dual union all
select 63855,'DLR' from dual union all
select 63856,'AN' from dual union all
select 63857,'DGA' from dual union all
select 63858,'DLR' from dual union all
select 63859,'INTA' from dual union all
select 6386,'USA' from dual union all
select 63860,'AETE' from dual union all
select 63860,'NASA' from dual union all
select 63861,'AN' from dual union all
select 63862,'AN' from dual union all
select 63863,'AN' from dual union all
select 63864,'AN' from dual union all
select 63865,'AN' from dual union all
select 63866,'AN' from dual union all
select 63867,'AN' from dual union all
select 63868,'NASA' from dual union all
select 63869,'DLR' from dual union all
select 6387,'USA' from dual;
insert into LAUNCH_AGENCY
select 63870,'AN' from dual union all
select 63871,'NASA' from dual union all
select 63872,'DLR' from dual union all
select 63873,'DGA' from dual union all
select 63874,'INTA' from dual union all
select 63875,'DLR' from dual union all
select 63876,'AETE' from dual union all
select 63876,'NASA' from dual union all
select 63877,'AN' from dual union all
select 63878,'AN' from dual union all
select 63879,'NASA' from dual union all
select 6388,'USA' from dual union all
select 63880,'AN' from dual union all
select 63881,'AN' from dual union all
select 63882,'AN' from dual union all
select 63883,'DLR' from dual union all
select 63884,'INTA' from dual union all
select 63885,'AETE' from dual union all
select 63885,'NASA' from dual union all
select 63886,'AN' from dual union all
select 63887,'AN' from dual union all
select 63888,'AN' from dual union all
select 63889,'AN' from dual union all
select 6389,'NASA' from dual union all
select 63890,'AN' from dual union all
select 63891,'AN' from dual union all
select 63892,'NASA' from dual union all
select 63893,'DLR' from dual union all
select 63894,'INTA' from dual union all
select 63895,'AETE' from dual union all
select 63895,'NASA' from dual union all
select 63896,'AN' from dual union all
select 63897,'ISAS' from dual union all
select 63898,'NASA' from dual union all
select 63899,'DLR' from dual union all
select 639,'ORD' from dual union all
select 6390,'USAF' from dual union all
select 63900,'AN' from dual union all
select 63901,'NASA' from dual union all
select 63902,'AN' from dual union all
select 63903,'AN' from dual union all
select 63904,'AN' from dual union all
select 63905,'DLR' from dual union all
select 63906,'INTA' from dual union all
select 63907,'DLR' from dual union all
select 63908,'DGA' from dual union all
select 63909,'AETE' from dual union all
select 63909,'NASA' from dual union all
select 6391,'NASA' from dual union all
select 6391,'USAF' from dual union all
select 63910,'DLR' from dual union all
select 63911,'AN' from dual union all
select 63912,'DGA' from dual union all
select 63913,'DLR' from dual union all
select 63914,'DLR' from dual union all
select 63915,'DGA' from dual union all
select 63916,'DLR' from dual union all
select 63917,'AN' from dual union all
select 63918,'AN' from dual union all
select 63919,'AN' from dual union all
select 6392,'USAF' from dual union all
select 63920,'AN' from dual union all
select 63921,'NASA' from dual union all
select 63922,'AN' from dual union all
select 63923,'DLR' from dual union all
select 63924,'DGA' from dual union all
select 63925,'DLR' from dual union all
select 63926,'INTA' from dual union all
select 63927,'AETE' from dual union all
select 63927,'NASA' from dual union all
select 63928,'AN' from dual union all
select 63929,'ISAS' from dual union all
select 6393,'USAF' from dual union all
select 63930,'NASA' from dual union all
select 63931,'DLR' from dual union all
select 63932,'NASA' from dual union all
select 63933,'AN' from dual union all
select 63934,'DLR' from dual union all
select 63935,'NASA' from dual union all
select 63936,'AN' from dual union all
select 63937,'ISAS' from dual union all
select 63938,'DLR' from dual union all
select 63939,'INTA' from dual union all
select 6394,'USAF' from dual union all
select 63940,'AETE' from dual union all
select 63940,'NASA' from dual union all
select 63941,'AN' from dual union all
select 63942,'AN' from dual union all
select 63943,'AN' from dual union all
select 63944,'AN' from dual union all
select 63945,'AN' from dual union all
select 63946,'NASA' from dual union all
select 63947,'DLR' from dual union all
select 63948,'DGA' from dual union all
select 63949,'DLR' from dual union all
select 6395,'NASA' from dual union all
select 6395,'USAF' from dual union all
select 63950,'INTA' from dual union all
select 63951,'AETE' from dual union all
select 63951,'NASA' from dual;
insert into LAUNCH_AGENCY
select 63952,'AN' from dual union all
select 63953,'NASA' from dual union all
select 63954,'MRN' from dual union all
select 63955,'DLR' from dual union all
select 63956,'DLR' from dual union all
select 63957,'INTA' from dual union all
select 63958,'DLR' from dual union all
select 63959,'NASA' from dual union all
select 6396,'USAF' from dual union all
select 63960,'DLR' from dual union all
select 63961,'NASA' from dual union all
select 63962,'AETE' from dual union all
select 63962,'NASA' from dual union all
select 63963,'DGA' from dual union all
select 63964,'DLR' from dual union all
select 63965,'DGA' from dual union all
select 63966,'DLR' from dual union all
select 63967,'DGA' from dual union all
select 63968,'DLR' from dual union all
select 63969,'DLR' from dual union all
select 6397,'USAF' from dual union all
select 63970,'AN' from dual union all
select 63971,'AN' from dual union all
select 63972,'NASA' from dual union all
select 63973,'DLR' from dual union all
select 63974,'DLR' from dual union all
select 63975,'NASA' from dual union all
select 63976,'MRN' from dual union all
select 63977,'NASA' from dual union all
select 63978,'DLR' from dual union all
select 63979,'DLR' from dual union all
select 6398,'USAF' from dual union all
select 63980,'DLR' from dual union all
select 63981,'INTA' from dual union all
select 63982,'DLR' from dual union all
select 63983,'DLR' from dual union all
select 63984,'NASA' from dual union all
select 63985,'DLR' from dual union all
select 63986,'DLR' from dual union all
select 63987,'NASA' from dual union all
select 63988,'NASA' from dual union all
select 63989,'DLR' from dual union all
select 6399,'USAF' from dual union all
select 63990,'AETE' from dual union all
select 63990,'NASA' from dual union all
select 63991,'NASA' from dual union all
select 63992,'NASA' from dual union all
select 63993,'DLR' from dual union all
select 63994,'NASA' from dual union all
select 63995,'DLR' from dual union all
select 63996,'NASA' from dual union all
select 63997,'DLR' from dual union all
select 63998,'DLR' from dual union all
select 63999,'NASA' from dual union all
select 64,'WEHR' from dual union all
select 640,'ORD' from dual union all
select 6400,'USAF' from dual union all
select 64000,'AN' from dual union all
select 64001,'AN' from dual union all
select 64002,'DLR' from dual union all
select 64003,'INTA' from dual union all
select 64004,'AN' from dual union all
select 64005,'DGA' from dual union all
select 64006,'DLR' from dual union all
select 64007,'DGA' from dual union all
select 64008,'AETE' from dual union all
select 64008,'NASA' from dual union all
select 64009,'DLR' from dual union all
select 6401,'USAF' from dual union all
select 64010,'AN' from dual union all
select 64011,'DGA' from dual union all
select 64012,'DLR' from dual union all
select 64013,'DGA' from dual union all
select 64014,'AN' from dual union all
select 64015,'DLR' from dual union all
select 64016,'DGA' from dual union all
select 64017,'INTA' from dual union all
select 64018,'AETE' from dual union all
select 64018,'NASA' from dual union all
select 64019,'AN' from dual union all
select 6402,'USAF' from dual union all
select 64020,'AN' from dual union all
select 64021,'DLR' from dual union all
select 64022,'AN' from dual union all
select 64023,'AN' from dual union all
select 64024,'MRN' from dual union all
select 64025,'AN' from dual union all
select 64026,'AN' from dual union all
select 64027,'AN' from dual union all
select 64028,'MRN' from dual union all
select 64029,'MRN' from dual union all
select 6403,'USAF' from dual union all
select 64030,'MRN' from dual union all
select 64031,'MRN' from dual union all
select 64032,'AN' from dual union all
select 64033,'MRN' from dual union all
select 64034,'AN' from dual union all
select 64035,'MRN' from dual union all
select 64036,'MRN' from dual union all
select 64037,'MRN' from dual;
insert into LAUNCH_AGENCY
select 64038,'MRN' from dual union all
select 64039,'MRN' from dual union all
select 6404,'NASA' from dual union all
select 64040,'MRN' from dual union all
select 64041,'MRN' from dual union all
select 64042,'MRN' from dual union all
select 64043,'AN' from dual union all
select 64044,'AN' from dual union all
select 64045,'AN' from dual union all
select 64046,'AN' from dual union all
select 64047,'AN' from dual union all
select 64048,'AN' from dual union all
select 64049,'AN' from dual union all
select 6405,'USAF' from dual union all
select 64050,'AN' from dual union all
select 64051,'AN' from dual union all
select 64052,'AN' from dual union all
select 64053,'AN' from dual union all
select 64054,'DGA' from dual union all
select 64055,'AN' from dual union all
select 64056,'AN' from dual union all
select 64057,'AN' from dual union all
select 64058,'AN' from dual union all
select 64059,'AN' from dual union all
select 6406,'USAF' from dual union all
select 64060,'AN' from dual union all
select 64061,'AN' from dual union all
select 64062,'AN' from dual union all
select 64063,'AN' from dual union all
select 64064,'AN' from dual union all
select 64065,'DLR' from dual union all
select 64066,'AN' from dual union all
select 64067,'AN' from dual union all
select 64068,'AN' from dual union all
select 64069,'AN' from dual union all
select 6407,'USAF' from dual union all
select 64070,'AN' from dual union all
select 64071,'AN' from dual union all
select 64072,'AN' from dual union all
select 64073,'AN' from dual union all
select 64074,'AN' from dual union all
select 64075,'AN' from dual union all
select 64076,'NASA' from dual union all
select 64077,'AN' from dual union all
select 64078,'AN' from dual union all
select 64079,'AN' from dual union all
select 6408,'USA' from dual union all
select 64080,'AN' from dual union all
select 64081,'AN' from dual union all
select 64082,'AN' from dual union all
select 64083,'AN' from dual union all
select 64084,'AN' from dual union all
select 64085,'AN' from dual union all
select 64086,'AN' from dual union all
select 64087,'INTA' from dual union all
select 64088,'AN' from dual union all
select 64089,'AN' from dual union all
select 6409,'USA' from dual union all
select 64090,'AN' from dual union all
select 64091,'AN' from dual union all
select 64092,'AN' from dual union all
select 64093,'AN' from dual union all
select 64094,'AN' from dual union all
select 64095,'ISAS' from dual union all
select 64096,'ISAS' from dual union all
select 64097,'DLR' from dual union all
select 64098,'INTA' from dual union all
select 64099,'DLR' from dual union all
select 641,'ORD' from dual union all
select 6410,'USN' from dual union all
select 64100,'DLR' from dual union all
select 64101,'DLR' from dual union all
select 64102,'DLR' from dual union all
select 64103,'DLR' from dual union all
select 64104,'AETE' from dual union all
select 64104,'NASA' from dual union all
select 64105,'AN' from dual union all
select 64106,'AN' from dual union all
select 64107,'AN' from dual union all
select 64108,'AN' from dual union all
select 64109,'ISAS' from dual union all
select 6411,'USAF' from dual union all
select 64110,'AN' from dual union all
select 64111,'AN' from dual union all
select 64112,'NASA' from dual union all
select 64113,'DLR' from dual union all
select 64114,'DLR' from dual union all
select 64115,'DGA' from dual union all
select 64116,'DLR' from dual union all
select 64117,'AN' from dual union all
select 64118,'INTA' from dual union all
select 64119,'INTA' from dual union all
select 6412,'USAF' from dual union all
select 64120,'AETE' from dual union all
select 64120,'NASA' from dual union all
select 64121,'MRN' from dual union all
select 64122,'MRN' from dual union all
select 64123,'AN' from dual union all
select 64124,'NASA' from dual union all
select 64125,'NASA' from dual;
insert into LAUNCH_AGENCY
select 64126,'AN' from dual union all
select 64127,'DLR' from dual union all
select 64128,'NASA' from dual union all
select 64129,'AN' from dual union all
select 6413,'USN' from dual union all
select 64130,'DGA' from dual union all
select 64131,'DLR' from dual union all
select 64132,'DLR' from dual union all
select 64133,'AETE' from dual union all
select 64133,'NASA' from dual union all
select 64134,'AN' from dual union all
select 64135,'INTA' from dual union all
select 64136,'AN' from dual union all
select 64137,'AN' from dual union all
select 64138,'AN' from dual union all
select 64139,'AN' from dual union all
select 6414,'USN' from dual union all
select 64140,'AN' from dual union all
select 64141,'AN' from dual union all
select 64142,'AN' from dual union all
select 64143,'AN' from dual union all
select 64144,'NASA' from dual union all
select 64145,'MRN' from dual union all
select 64146,'DLR' from dual union all
select 64147,'AN' from dual union all
select 64148,'INTA' from dual union all
select 64149,'AETE' from dual union all
select 64149,'NASA' from dual union all
select 6415,'702SMW' from dual union all
select 64150,'NASA' from dual union all
select 64151,'DLR' from dual union all
select 64152,'DLR' from dual union all
select 64153,'ISAS' from dual union all
select 64154,'AN' from dual union all
select 64155,'AN' from dual union all
select 64156,'DLR' from dual union all
select 64157,'AETE' from dual union all
select 64157,'NASA' from dual union all
select 64158,'INTA' from dual union all
select 64159,'AN' from dual union all
select 6416,'USAF' from dual union all
select 64160,'AN' from dual union all
select 64161,'AN' from dual union all
select 64162,'ISAS' from dual union all
select 64163,'AN' from dual union all
select 64164,'AN' from dual union all
select 64165,'AN' from dual union all
select 64166,'AN' from dual union all
select 64167,'NASA' from dual union all
select 64168,'DLR' from dual union all
select 64169,'AN' from dual union all
select 6417,'RAE' from dual union all
select 64170,'MDSSC' from dual union all
select 64171,'UNKS' from dual union all
select 64172,'AE' from dual union all
select 64173,'UNKS' from dual union all
select 64174,'UNKS' from dual union all
select 64175,'UNKS' from dual union all
select 64176,'UNKS' from dual union all
select 64177,'UNKS' from dual union all
select 64178,'UNKS' from dual union all
select 64179,'UNKS' from dual union all
select 6418,'USAF' from dual union all
select 64180,'UNKS' from dual union all
select 64181,'UNKS' from dual union all
select 64182,'UNKS' from dual union all
select 64183,'UNKS' from dual union all
select 64184,'AE' from dual union all
select 64185,'UNKS' from dual union all
select 64186,'AFSC' from dual union all
select 64187,'MDSSC' from dual union all
select 64188,'UNKS' from dual union all
select 64189,'UNKS' from dual union all
select 6419,'USAF' from dual union all
select 64190,'UNKS' from dual union all
select 64191,'UNKS' from dual union all
select 64192,'UNKS' from dual union all
select 64193,'UNKS' from dual union all
select 64194,'UNKS' from dual union all
select 64195,'AE' from dual union all
select 64196,'NASA' from dual union all
select 64197,'MDSSC' from dual union all
select 64198,'UNKS' from dual union all
select 64199,'UNKS' from dual union all
select 642,'ORD' from dual union all
select 6420,'RAE' from dual union all
select 64200,'NASA' from dual union all
select 64201,'AFSPC' from dual union all
select 64202,'UNKS' from dual union all
select 64203,'UNKS' from dual union all
select 64204,'UNKS' from dual union all
select 64205,'UNKS' from dual union all
select 64206,'MDSSC' from dual union all
select 64207,'UNKS' from dual union all
select 64208,'UNKS' from dual union all
select 64209,'NASA' from dual union all
select 6421,'NASA' from dual union all
select 64210,'UNKS' from dual union all
select 64211,'UNKS' from dual union all
select 64212,'UNKS' from dual;
insert into LAUNCH_AGENCY
select 64213,'UNKS' from dual union all
select 64214,'NASA' from dual union all
select 64215,'UNKS' from dual union all
select 64216,'MDSSC' from dual union all
select 64217,'UNKS' from dual union all
select 64218,'UNKS' from dual union all
select 64219,'AE' from dual union all
select 6422,'NASA' from dual union all
select 64220,'OSC' from dual union all
select 64221,'UNKS' from dual union all
select 64222,'UNKS' from dual union all
select 64223,'NASA' from dual union all
select 64224,'AE' from dual union all
select 64225,'UNKS' from dual union all
select 64226,'UNKS' from dual union all
select 64227,'UNKS' from dual union all
select 64228,'UNKS' from dual union all
select 64229,'NASDA' from dual union all
select 6423,'SAC' from dual union all
select 64230,'UNKS' from dual union all
select 64231,'ISAS' from dual union all
select 64232,'NASA' from dual union all
select 64233,'UNKS' from dual union all
select 64234,'UNKS' from dual union all
select 64235,'UNKS' from dual union all
select 64236,'AE' from dual union all
select 64237,'UNKS' from dual union all
select 64238,'UNKS' from dual union all
select 64239,'UNKS' from dual union all
select 6424,'USAF' from dual union all
select 64240,'UNKS' from dual union all
select 64241,'UNKS' from dual union all
select 64242,'UNKS' from dual union all
select 64243,'UNKS' from dual union all
select 64244,'AE' from dual union all
select 64245,'AFSC' from dual union all
select 64246,'UNKS' from dual union all
select 64247,'UNKS' from dual union all
select 64248,'UNKS' from dual union all
select 64249,'NASA' from dual union all
select 6425,'NOTS' from dual union all
select 64250,'UNKS' from dual union all
select 64251,'AFSPC' from dual union all
select 64252,'GDCLS' from dual union all
select 64253,'AE' from dual union all
select 64254,'UNKS' from dual union all
select 64255,'UNKS' from dual union all
select 64256,'UNKS' from dual union all
select 64257,'CALT' from dual union all
select 64258,'NASA' from dual union all
select 64259,'IAI' from dual union all
select 6426,'USAF' from dual union all
select 64260,'OSC' from dual union all
select 64261,'BMDO' from dual union all
select 64262,'SDIO' from dual union all
select 64263,'IAI' from dual union all
select 64264,'DLR' from dual union all
select 64265,'RVSN' from dual union all
select 64266,'GDCLS' from dual union all
select 64267,'UNKS' from dual union all
select 64268,'UNKS' from dual union all
select 64269,'IDRDL' from dual union all
select 6427,'USA' from dual union all
select 64270,'IDRDL' from dual union all
select 64271,'RVSN' from dual union all
select 64272,'RVSN' from dual union all
select 64273,'RVSN' from dual union all
select 64274,'RVSN' from dual union all
select 64275,'RVSN' from dual union all
select 64276,'AN' from dual union all
select 64277,'AN' from dual union all
select 64278,'RVSN' from dual union all
select 64279,'RVSN' from dual union all
select 6428,'USA' from dual union all
select 64280,'RVSN' from dual union all
select 64281,'RVSN' from dual union all
select 64282,'RVSN' from dual union all
select 64283,'RVSN' from dual union all
select 64284,'RVSN' from dual union all
select 64285,'RVSN' from dual union all
select 64286,'IAE' from dual union all
select 64287,'IR' from dual union all
select 64288,'NASA' from dual union all
select 64289,'NASA' from dual union all
select 6429,'USA' from dual union all
select 64290,'NASA' from dual union all
select 64291,'SSC' from dual union all
select 64292,'USN' from dual union all
select 64293,'USN' from dual union all
select 64294,'AFSC' from dual union all
select 64295,'NASA' from dual union all
select 64296,'RVSN' from dual union all
select 64297,'NASA' from dual union all
select 64298,'RVSN' from dual union all
select 64299,'SAC' from dual union all
select 643,'PRIN' from dual union all
select 643,'US' from dual union all
select 6430,'USA' from dual union all
select 64300,'AFSC' from dual union all
select 64301,'DMA' from dual;
insert into LAUNCH_AGENCY
select 64302,'SAC' from dual union all
select 64303,'RVSN' from dual union all
select 64304,'AN' from dual union all
select 64305,'AN' from dual union all
select 64306,'AN' from dual union all
select 64307,'RVSN' from dual union all
select 64308,'AN' from dual union all
select 64309,'KPA' from dual union all
select 6431,'USA' from dual union all
select 64310,'SAC' from dual union all
select 64311,'NASA' from dual union all
select 64312,'USN' from dual union all
select 64313,'USN' from dual union all
select 64314,'USN' from dual union all
select 64315,'USN' from dual union all
select 64316,'AN' from dual union all
select 64317,'AN' from dual union all
select 64318,'RVSN' from dual union all
select 64319,'AN' from dual union all
select 6432,'USA' from dual union all
select 64320,'AN' from dual union all
select 64321,'NASA' from dual union all
select 64322,'NASA' from dual union all
select 64323,'DLR' from dual union all
select 64324,'NASA' from dual union all
select 64325,'RVSN' from dual union all
select 64326,'RVSN' from dual union all
select 64327,'USN' from dual union all
select 64328,'AN' from dual union all
select 64329,'RVSN' from dual union all
select 6433,'NASA' from dual union all
select 6433,'USAF' from dual union all
select 64330,'AN' from dual union all
select 64331,'AN' from dual union all
select 64332,'AN' from dual union all
select 64333,'SAC' from dual union all
select 64334,'SAND' from dual union all
select 64335,'NASA' from dual union all
select 64336,'SAND' from dual union all
select 64337,'RVSN' from dual union all
select 64338,'NASDA' from dual union all
select 64339,'SAC' from dual union all
select 6434,'USA' from dual union all
select 64340,'RVSN' from dual union all
select 64341,'RVSN' from dual union all
select 64342,'DLR' from dual union all
select 64343,'DLR' from dual union all
select 64344,'DLR' from dual union all
select 64345,'AN' from dual union all
select 64346,'AN' from dual union all
select 64347,'AN' from dual union all
select 64348,'AN' from dual union all
select 64349,'DLR' from dual union all
select 6435,'USA' from dual union all
select 64350,'RVSN' from dual union all
select 64351,'RVSN' from dual union all
select 64352,'DLR' from dual union all
select 64353,'DLR' from dual union all
select 64354,'RVSN' from dual union all
select 64355,'BMDO' from dual union all
select 64356,'IDF' from dual union all
select 64357,'USN' from dual union all
select 64358,'ALCS' from dual union all
select 64358,'SAC' from dual union all
select 64359,'USN' from dual union all
select 6436,'USA' from dual union all
select 64360,'USN' from dual union all
select 64361,'USN' from dual union all
select 64362,'IQ' from dual union all
select 64363,'USN' from dual union all
select 64364,'SSI' from dual union all
select 64365,'RVSN' from dual union all
select 64366,'MBB' from dual union all
select 64367,'RVSN' from dual union all
select 64368,'RVSN' from dual union all
select 64369,'MAK' from dual union all
select 6437,'USN' from dual union all
select 64370,'VMF' from dual union all
select 64371,'NASA' from dual union all
select 64372,'NASA' from dual union all
select 64373,'IQ' from dual union all
select 64374,'IAE' from dual union all
select 64374,'INPE' from dual union all
select 64375,'VKS' from dual union all
select 64376,'SSC' from dual union all
select 64377,'SSC' from dual union all
select 64378,'DLR' from dual union all
select 64379,'NASA' from dual union all
select 6438,'GMS' from dual union all
select 64380,'INTA' from dual union all
select 64381,'INTA' from dual union all
select 64382,'INTA' from dual union all
select 64383,'USN' from dual union all
select 64384,'IQ' from dual union all
select 64385,'USN' from dual union all
select 64386,'USN' from dual union all
select 64387,'IQ' from dual union all
select 64388,'IQ' from dual union all
select 64389,'IQ' from dual union all
select 6439,'USN' from dual;
insert into LAUNCH_AGENCY
select 64390,'IQ' from dual union all
select 64391,'IQ' from dual union all
select 64392,'DLR' from dual union all
select 64393,'DLR' from dual union all
select 64394,'DLR' from dual union all
select 64395,'IQ' from dual union all
select 64396,'DLR' from dual union all
select 64397,'DLR' from dual union all
select 64398,'DLR' from dual union all
select 64399,'DLR' from dual union all
select 644,'ORD' from dual union all
select 6440,'MVS' from dual union all
select 64400,'DLR' from dual union all
select 64401,'DLR' from dual union all
select 64402,'DLR' from dual union all
select 64403,'DLR' from dual union all
select 64404,'DLR' from dual union all
select 64405,'DLR' from dual union all
select 64406,'IQ' from dual union all
select 64407,'DLR' from dual union all
select 64408,'DLR' from dual union all
select 64409,'DLR' from dual union all
select 6441,'GMS' from dual union all
select 64410,'DLR' from dual union all
select 64411,'DLR' from dual union all
select 64412,'INTA' from dual union all
select 64413,'PKO' from dual union all
select 64414,'IQ' from dual union all
select 64415,'IQ' from dual union all
select 64416,'IQ' from dual union all
select 64417,'IQ' from dual union all
select 64418,'IQ' from dual union all
select 64419,'IQ' from dual union all
select 6442,'LARCN' from dual union all
select 64420,'IQ' from dual union all
select 64421,'IQ' from dual union all
select 64422,'IQ' from dual union all
select 64423,'CSSAR' from dual union all
select 64424,'IQ' from dual union all
select 64425,'IQ' from dual union all
select 64426,'IQ' from dual union all
select 64427,'IQ' from dual union all
select 64428,'IQ' from dual union all
select 64429,'IQ' from dual union all
select 6443,'GMS' from dual union all
select 64430,'IQ' from dual union all
select 64431,'IQ' from dual union all
select 64432,'IQ' from dual union all
select 64433,'IQ' from dual union all
select 64434,'IQ' from dual union all
select 64435,'IQ' from dual union all
select 64436,'IQ' from dual union all
select 64437,'IQ' from dual union all
select 64438,'IQ' from dual union all
select 64439,'IQ' from dual union all
select 6444,'GMS' from dual union all
select 64440,'NASA' from dual union all
select 64441,'IQ' from dual union all
select 64442,'IQ' from dual union all
select 64443,'AFSC' from dual union all
select 64444,'USA' from dual union all
select 64445,'IQ' from dual union all
select 64446,'IQ' from dual union all
select 64447,'IQ' from dual union all
select 64448,'IQ' from dual union all
select 64449,'IQ' from dual union all
select 6445,'NASA' from dual union all
select 6445,'USAF' from dual union all
select 64450,'IQ' from dual union all
select 64451,'DLR' from dual union all
select 64452,'OSC' from dual union all
select 64453,'AN' from dual union all
select 64454,'RVSN' from dual union all
select 64455,'IQ' from dual union all
select 64456,'IDRDL' from dual union all
select 64457,'IQ' from dual union all
select 64458,'ISAS' from dual union all
select 64459,'NASA' from dual union all
select 6446,'GMS' from dual union all
select 64460,'IQ' from dual union all
select 64461,'IQ' from dual union all
select 64462,'IQ' from dual union all
select 64463,'ISAS' from dual union all
select 64464,'SDIO' from dual union all
select 64465,'IQ' from dual union all
select 64466,'IQ' from dual union all
select 64467,'IQ' from dual union all
select 64468,'IQ' from dual union all
select 64469,'NASA' from dual union all
select 6447,'GMS' from dual union all
select 64470,'IQ' from dual union all
select 64471,'IQ' from dual union all
select 64472,'IQ' from dual union all
select 64473,'IQ' from dual union all
select 64474,'IQ' from dual union all
select 64475,'IQ' from dual union all
select 64476,'IQ' from dual union all
select 64477,'IQ' from dual union all
select 64478,'SAC' from dual union all
select 64479,'NASA' from dual;
insert into LAUNCH_AGENCY
select 6448,'GMS' from dual union all
select 64480,'USN' from dual union all
select 64481,'USN' from dual union all
select 64482,'NASA' from dual union all
select 64483,'NASA' from dual union all
select 64484,'RVSN' from dual union all
select 64485,'DLR' from dual union all
select 64486,'DLR' from dual union all
select 64487,'OSC' from dual union all
select 64488,'USN' from dual union all
select 64489,'SDIO' from dual union all
select 6449,'NASA' from dual union all
select 64490,'AFSC' from dual union all
select 64491,'NASA' from dual union all
select 64492,'AN' from dual union all
select 64493,'AN' from dual union all
select 64494,'AN' from dual union all
select 64495,'AN' from dual union all
select 64496,'AN' from dual union all
select 64497,'AN' from dual union all
select 64498,'AN' from dual union all
select 64499,'AN' from dual union all
select 645,'ORD' from dual union all
select 6450,'USAF' from dual union all
select 64500,'AN' from dual union all
select 64501,'AN' from dual union all
select 64502,'MRN' from dual union all
select 64503,'MRN' from dual union all
select 64504,'MRN' from dual union all
select 64505,'MRN' from dual union all
select 64506,'ISAS' from dual union all
select 64507,'MRN' from dual union all
select 64508,'MRN' from dual union all
select 64509,'MRN' from dual union all
select 6451,'USA' from dual union all
select 64510,'ISAS' from dual union all
select 64511,'MRN' from dual union all
select 64512,'AN' from dual union all
select 64513,'MRN' from dual union all
select 64514,'MRN' from dual union all
select 64515,'MRN' from dual union all
select 64516,'MRN' from dual union all
select 64517,'MRN' from dual union all
select 64518,'MRN' from dual union all
select 64519,'MRN' from dual union all
select 6452,'NASA' from dual union all
select 6452,'USAF' from dual union all
select 64520,'MRN' from dual union all
select 64521,'MRN' from dual union all
select 64522,'MRN' from dual union all
select 64523,'AN' from dual union all
select 64524,'MRN' from dual union all
select 64525,'MRN' from dual union all
select 64526,'MRN' from dual union all
select 64527,'MRN' from dual union all
select 64528,'MRN' from dual union all
select 64529,'MRN' from dual union all
select 6453,'GMS' from dual union all
select 64530,'MRN' from dual union all
select 64531,'MRN' from dual union all
select 64532,'MRN' from dual union all
select 64533,'MRN' from dual union all
select 64534,'AN' from dual union all
select 64535,'MRN' from dual union all
select 64536,'MRN' from dual union all
select 64537,'JMA' from dual union all
select 64538,'JMA' from dual union all
select 64539,'JMA' from dual union all
select 6454,'NASA' from dual union all
select 6454,'USAF' from dual union all
select 64540,'JMA' from dual union all
select 64541,'JMA' from dual union all
select 64542,'JMA' from dual union all
select 64543,'JMA' from dual union all
select 64544,'JMA' from dual union all
select 64545,'AN' from dual union all
select 64546,'JMA' from dual union all
select 64547,'JMA' from dual union all
select 64548,'JMA' from dual union all
select 64549,'JMA' from dual union all
select 6455,'RVSN' from dual union all
select 64550,'JMA' from dual union all
select 64551,'JMA' from dual union all
select 64552,'JMA' from dual union all
select 64553,'JMA' from dual union all
select 64554,'JMA' from dual union all
select 64555,'JMA' from dual union all
select 64556,'AN' from dual union all
select 64557,'JMA' from dual union all
select 64558,'JMA' from dual union all
select 64559,'JMA' from dual union all
select 6456,'GMS' from dual union all
select 64560,'JMA' from dual union all
select 64561,'JMA' from dual union all
select 64562,'JMA' from dual union all
select 64563,'JMA' from dual union all
select 64564,'JMA' from dual union all
select 64565,'JMA' from dual union all
select 64566,'JMA' from dual union all
select 64567,'AN' from dual;
insert into LAUNCH_AGENCY
select 64568,'JMA' from dual union all
select 64569,'JMA' from dual union all
select 6457,'GMS' from dual union all
select 64570,'JMA' from dual union all
select 64571,'JMA' from dual union all
select 64572,'JMA' from dual union all
select 64573,'JMA' from dual union all
select 64574,'JMA' from dual union all
select 64575,'JMA' from dual union all
select 64576,'JMA' from dual union all
select 64577,'JMA' from dual union all
select 64578,'AN' from dual union all
select 64579,'DLR' from dual union all
select 6458,'GMS' from dual union all
select 64580,'AN' from dual union all
select 64581,'AN' from dual union all
select 64582,'AN' from dual union all
select 64583,'AN' from dual union all
select 64584,'MRN' from dual union all
select 64585,'MRN' from dual union all
select 64586,'MRN' from dual union all
select 64587,'MRN' from dual union all
select 64588,'MRN' from dual union all
select 64589,'MRN' from dual union all
select 6459,'RAE' from dual union all
select 64590,'MRN' from dual union all
select 64591,'ISAS' from dual union all
select 64592,'ISAS' from dual union all
select 64593,'MRN' from dual union all
select 64594,'MRN' from dual union all
select 64595,'MRN' from dual union all
select 64596,'MRN' from dual union all
select 64597,'MRN' from dual union all
select 64598,'MRN' from dual union all
select 64599,'MRN' from dual union all
select 646,'ORD' from dual union all
select 6460,'NASA' from dual union all
select 6460,'USAF' from dual union all
select 64600,'MRN' from dual union all
select 64601,'MRN' from dual union all
select 64602,'MRN' from dual union all
select 64603,'MRN' from dual union all
select 64604,'MRN' from dual union all
select 64605,'MRN' from dual union all
select 64606,'MRN' from dual union all
select 64607,'MRN' from dual union all
select 64608,'MRN' from dual union all
select 64609,'MRN' from dual union all
select 6461,'NASA' from dual union all
select 6461,'USAF' from dual union all
select 64610,'MRN' from dual union all
select 64611,'MRN' from dual union all
select 64612,'MRN' from dual union all
select 64613,'MRN' from dual union all
select 64614,'MRN' from dual union all
select 64615,'MRN' from dual union all
select 64616,'MRN' from dual union all
select 64617,'MRN' from dual union all
select 64618,'MRN' from dual union all
select 64619,'MRN' from dual union all
select 6462,'NASA' from dual union all
select 64620,'MRN' from dual union all
select 64621,'MRN' from dual union all
select 64622,'MRN' from dual union all
select 64623,'MRN' from dual union all
select 64624,'MRN' from dual union all
select 64625,'MRN' from dual union all
select 64626,'MRN' from dual union all
select 64627,'MRN' from dual union all
select 64628,'MRN' from dual union all
select 64629,'MRN' from dual union all
select 6463,'USAF' from dual union all
select 64630,'MRN' from dual union all
select 64631,'MRN' from dual union all
select 64632,'MRN' from dual union all
select 64633,'MRN' from dual union all
select 64634,'MRN' from dual union all
select 64635,'MRN' from dual union all
select 64636,'MRN' from dual union all
select 64637,'MRN' from dual union all
select 64638,'MRN' from dual union all
select 64639,'MRN' from dual union all
select 6464,'USAF' from dual union all
select 64640,'MRN' from dual union all
select 64641,'MRN' from dual union all
select 64642,'MRN' from dual union all
select 64643,'MRN' from dual union all
select 64644,'MRN' from dual union all
select 64645,'MRN' from dual union all
select 64646,'MRN' from dual union all
select 64647,'MRN' from dual union all
select 64648,'MRN' from dual union all
select 64649,'MRN' from dual union all
select 6465,'USN' from dual union all
select 64650,'MRN' from dual union all
select 64651,'MRN' from dual union all
select 64652,'MRN' from dual union all
select 64653,'MRN' from dual union all
select 64654,'MRN' from dual union all
select 64655,'MRN' from dual;
insert into LAUNCH_AGENCY
select 64656,'MRN' from dual union all
select 64657,'MRN' from dual union all
select 64658,'MRN' from dual union all
select 64659,'MRN' from dual union all
select 6466,'USAF' from dual union all
select 64660,'MRN' from dual union all
select 64661,'MRN' from dual union all
select 64662,'MRN' from dual union all
select 64663,'MRN' from dual union all
select 64664,'MRN' from dual union all
select 64665,'MRN' from dual union all
select 64666,'MRN' from dual union all
select 64667,'MRN' from dual union all
select 64668,'MRN' from dual union all
select 64669,'MRN' from dual union all
select 6467,'SAC' from dual union all
select 64670,'MRN' from dual union all
select 64671,'MRN' from dual union all
select 64672,'MRN' from dual union all
select 64673,'MRN' from dual union all
select 64674,'MRN' from dual union all
select 64675,'MRN' from dual union all
select 64676,'MRN' from dual union all
select 64677,'MRN' from dual union all
select 64678,'MRN' from dual union all
select 64679,'MRN' from dual union all
select 6468,'USA' from dual union all
select 64680,'MRN' from dual union all
select 64681,'MRN' from dual union all
select 64682,'SPFL' from dual union all
select 64683,'NASA' from dual union all
select 64684,'NASA' from dual union all
select 64685,'DLR' from dual union all
select 64686,'NASA' from dual union all
select 64687,'MRN' from dual union all
select 64688,'NASA' from dual union all
select 64689,'NASA' from dual union all
select 6469,'USA' from dual union all
select 64690,'NASA' from dual union all
select 64691,'NASA' from dual union all
select 64692,'DLR' from dual union all
select 64693,'MRN' from dual union all
select 64694,'MRN' from dual union all
select 64695,'MRN' from dual union all
select 64696,'MRN' from dual union all
select 64697,'MRN' from dual union all
select 64698,'MRN' from dual union all
select 64699,'MRN' from dual union all
select 647,'ORD' from dual union all
select 6470,'USA' from dual union all
select 64700,'MRN' from dual union all
select 64701,'MRN' from dual union all
select 64702,'MRN' from dual union all
select 64703,'MRN' from dual union all
select 64704,'MRN' from dual union all
select 64705,'NASA' from dual union all
select 64706,'NASA' from dual union all
select 64707,'NASA' from dual union all
select 64708,'NASA' from dual union all
select 64709,'DLR' from dual union all
select 6471,'USA' from dual union all
select 64710,'DLR' from dual union all
select 64711,'DLR' from dual union all
select 64712,'DLR' from dual union all
select 64713,'UNKSR' from dual union all
select 64714,'NASA' from dual union all
select 64715,'UNKSR' from dual union all
select 64716,'UNKSR' from dual union all
select 64717,'UNKSR' from dual union all
select 64718,'GDCLS' from dual union all
select 64719,'NASDA' from dual union all
select 6472,'NASA' from dual union all
select 6472,'USAF' from dual union all
select 64720,'UNKSR' from dual union all
select 64721,'MDSSC' from dual union all
select 64722,'AE' from dual union all
select 64723,'UNKSR' from dual union all
select 64724,'UNKSR' from dual union all
select 64725,'GDCLS' from dual union all
select 64726,'UNKSR' from dual union all
select 64727,'NASA' from dual union all
select 64728,'UNKSR' from dual union all
select 64729,'UNKSR' from dual union all
select 6473,'USAF' from dual union all
select 64730,'UNKSR' from dual union all
select 64731,'MDSSC' from dual union all
select 64732,'UNKSR' from dual union all
select 64733,'AE' from dual union all
select 64734,'UNKSR' from dual union all
select 64735,'AFSC' from dual union all
select 64736,'UNKSR' from dual union all
select 64737,'UNKSR' from dual union all
select 64738,'NASA' from dual union all
select 64739,'MDSSC' from dual union all
select 6474,'NASA' from dual union all
select 64740,'ISRO' from dual union all
select 64741,'UNKSR' from dual union all
select 64742,'UNKSR' from dual union all
select 64743,'MDSSC' from dual union all
select 64744,'GDCLS' from dual;
insert into LAUNCH_AGENCY
select 64745,'UNKSR' from dual union all
select 64746,'NASA' from dual union all
select 64747,'UNKSR' from dual union all
select 64748,'UNKSR' from dual union all
select 64749,'GDCLS' from dual union all
select 6475,'NASA' from dual union all
select 6475,'USAF' from dual union all
select 64750,'NASA' from dual union all
select 64751,'MDSSC' from dual union all
select 64752,'UNKSR' from dual union all
select 64753,'AE' from dual union all
select 64754,'UNKSR' from dual union all
select 64755,'UNKSR' from dual union all
select 64756,'MDSSC' from dual union all
select 64757,'UNKSR' from dual union all
select 64758,'UNKSR' from dual union all
select 64759,'UNKSR' from dual union all
select 6476,'USN' from dual union all
select 64760,'UNKSR' from dual union all
select 64761,'NASA' from dual union all
select 64762,'UNKSR' from dual union all
select 64763,'SAST' from dual union all
select 64764,'AE' from dual union all
select 64765,'VKSR' from dual union all
select 64766,'CALT' from dual union all
select 64767,'VKSR' from dual union all
select 64768,'VKSR' from dual union all
select 64769,'MDSSC' from dual union all
select 6477,'USAF' from dual union all
select 64770,'MDSSC' from dual union all
select 64771,'VKSR' from dual union all
select 64772,'AE' from dual union all
select 64773,'NASA' from dual union all
select 64774,'VKSR' from dual union all
select 64775,'MMA' from dual union all
select 64776,'CALT' from dual union all
select 64777,'VKSR' from dual union all
select 64778,'MDSSC' from dual union all
select 64779,'VKSR' from dual union all
select 6478,'ISAS' from dual union all
select 64780,'VKSR' from dual union all
select 64781,'VKSR' from dual union all
select 64782,'NASA' from dual union all
select 64783,'VKSR' from dual union all
select 64784,'AE' from dual union all
select 64785,'VKSR' from dual union all
select 64786,'VKSR' from dual union all
select 64787,'VKSR' from dual union all
select 64788,'VKSR' from dual union all
select 64789,'VKSR' from dual union all
select 6479,'NASA' from dual union all
select 6479,'USAF' from dual union all
select 64790,'NASA' from dual union all
select 64791,'MDSSC' from dual union all
select 64792,'VKSR' from dual union all
select 64793,'VKSR' from dual union all
select 64794,'VKSR' from dual union all
select 64795,'AFMC' from dual union all
select 64796,'AE' from dual union all
select 64797,'VKSR' from dual union all
select 64798,'NASA' from dual union all
select 64799,'VKSR' from dual union all
select 648,'ORD' from dual union all
select 6480,'USAF' from dual union all
select 64800,'VKSR' from dual union all
select 64801,'MDSSC' from dual union all
select 64802,'CALT' from dual union all
select 64803,'VKSR' from dual union all
select 64804,'VKSR' from dual union all
select 64805,'VKSR' from dual union all
select 64806,'VKSR' from dual union all
select 64807,'VKSR' from dual union all
select 64808,'IDRDL' from dual union all
select 64809,'MRN' from dual union all
select 6481,'NASA' from dual union all
select 64810,'GMS' from dual union all
select 64811,'ASI' from dual union all
select 64812,'GMS' from dual union all
select 64813,'MRN' from dual union all
select 64814,'GMS' from dual union all
select 64815,'PLA2AC' from dual union all
select 64816,'PLA2AC' from dual union all
select 64817,'IDRDL' from dual union all
select 64818,'KPA' from dual union all
select 64819,'SDIO' from dual union all
select 6482,'USAF' from dual union all
select 64820,'IAI' from dual union all
select 64821,'USN' from dual union all
select 64822,'SDIO' from dual union all
select 64823,'IAE' from dual union all
select 64823,'INPE' from dual union all
select 64824,'USN' from dual union all
select 64825,'RVSNR' from dual union all
select 64826,'MRN' from dual union all
select 64827,'UNKSR' from dual union all
select 64828,'GDCLS' from dual union all
select 64829,'ISAS' from dual union all
select 6483,'USAF' from dual union all
select 64830,'NASA' from dual union all
select 64831,'ISRO' from dual;
insert into LAUNCH_AGENCY
select 64832,'ISRO' from dual union all
select 64833,'RVSN' from dual union all
select 64834,'DGA' from dual union all
select 64835,'AN' from dual union all
select 64836,'RVSNR' from dual union all
select 64837,'RVSNR' from dual union all
select 64838,'RVSNR' from dual union all
select 64839,'RVSNR' from dual union all
select 6484,'USN' from dual union all
select 64840,'RVSNR' from dual union all
select 64841,'RVSNR' from dual union all
select 64842,'RVSNR' from dual union all
select 64843,'VMFR' from dual union all
select 64844,'MATRA' from dual union all
select 64845,'NASA' from dual union all
select 64846,'NASA' from dual union all
select 64847,'NASA' from dual union all
select 64848,'PLA2AC' from dual union all
select 64849,'IDRDL' from dual union all
select 6485,'USN' from dual union all
select 64850,'INTA' from dual union all
select 64851,'INTA' from dual union all
select 64852,'NASA' from dual union all
select 64853,'USA' from dual union all
select 64854,'NASA' from dual union all
select 64855,'ISAS' from dual union all
select 64856,'NASA' from dual union all
select 64857,'ISAS' from dual union all
select 64858,'SDIO' from dual union all
select 64859,'USN' from dual union all
select 6486,'NASA' from dual union all
select 64860,'USN' from dual union all
select 64861,'USN' from dual union all
select 64862,'USN' from dual union all
select 64863,'NASA' from dual union all
select 64864,'NASA' from dual union all
select 64865,'NASA' from dual union all
select 64866,'NASA' from dual union all
select 64867,'SAC' from dual union all
select 64868,'NASA' from dual union all
select 64869,'NASA' from dual union all
select 6487,'NASA' from dual union all
select 6487,'USAF' from dual union all
select 64870,'AFSC' from dual union all
select 64871,'USA' from dual union all
select 64872,'NASA' from dual union all
select 64873,'CSA' from dual union all
select 64874,'NASA' from dual union all
select 64875,'OSC' from dual union all
select 64875,'SDIO' from dual union all
select 64876,'USN' from dual union all
select 64877,'USN' from dual union all
select 64878,'SSC' from dual union all
select 64879,'OSC' from dual union all
select 6488,'GMS' from dual union all
select 64880,'ALCS' from dual union all
select 64880,'SAC' from dual union all
select 64881,'IDRDL' from dual union all
select 64882,'NASA' from dual union all
select 64883,'NASA' from dual union all
select 64884,'SDIO' from dual union all
select 64885,'NASA' from dual union all
select 64886,'NASA' from dual union all
select 64887,'NASA' from dual union all
select 64888,'IAE' from dual union all
select 64889,'ACC' from dual union all
select 6489,'SAC' from dual union all
select 64890,'NASA' from dual union all
select 64891,'NASA' from dual union all
select 64892,'USA' from dual union all
select 64893,'USN' from dual union all
select 64894,'USN' from dual union all
select 64895,'USN' from dual union all
select 64896,'USN' from dual union all
select 64897,'SDIO' from dual union all
select 64898,'NASA' from dual union all
select 64899,'SYMC' from dual union all
select 649,'ORD' from dual union all
select 6490,'USAF' from dual union all
select 64900,'SYMC' from dual union all
select 64901,'ACC' from dual union all
select 64902,'NASA' from dual union all
select 64903,'ISRO' from dual union all
select 64904,'NASA' from dual union all
select 64905,'NASA' from dual union all
select 64906,'NASA' from dual union all
select 64907,'RVSNR' from dual union all
select 64908,'USN' from dual union all
select 64909,'USN' from dual union all
select 6491,'USAF' from dual union all
select 64910,'USN' from dual union all
select 64911,'USN' from dual union all
select 64912,'USN' from dual union all
select 64913,'IDRDL' from dual union all
select 64914,'DLR' from dual union all
select 64915,'NASDA' from dual union all
select 64916,'NASA' from dual union all
select 64917,'NASA' from dual union all
select 64918,'NASA' from dual union all
select 64919,'NASA' from dual;
insert into LAUNCH_AGENCY
select 6492,'USA' from dual union all
select 64920,'NASA' from dual union all
select 64921,'USN' from dual union all
select 64922,'USN' from dual union all
select 64923,'USN' from dual union all
select 64924,'USN' from dual union all
select 64925,'SSI' from dual union all
select 64926,'ACC' from dual union all
select 64927,'ACC' from dual union all
select 64928,'OSC' from dual union all
select 64929,'USN' from dual union all
select 6493,'USA' from dual union all
select 64930,'AFMC' from dual union all
select 64931,'NASA' from dual union all
select 64932,'ACC' from dual union all
select 64933,'DLR' from dual union all
select 64933,'SSC' from dual union all
select 64934,'USN' from dual union all
select 64935,'USN' from dual union all
select 64936,'USN' from dual union all
select 64937,'USN' from dual union all
select 64938,'ASTD' from dual union all
select 64939,'NASA' from dual union all
select 6494,'USA' from dual union all
select 64940,'RVSN' from dual union all
select 64941,'MRN' from dual union all
select 64942,'JMA' from dual union all
select 64943,'MRN' from dual union all
select 64944,'JMA' from dual union all
select 64945,'MRN' from dual union all
select 64946,'JMA' from dual union all
select 64947,'MRN' from dual union all
select 64948,'MRN' from dual union all
select 64949,'MRN' from dual union all
select 6495,'USA' from dual union all
select 64950,'MRN' from dual union all
select 64951,'MRN' from dual union all
select 64952,'MRN' from dual union all
select 64953,'JMA' from dual union all
select 64954,'JMA' from dual union all
select 64955,'MRN' from dual union all
select 64956,'MRN' from dual union all
select 64957,'JMA' from dual union all
select 64958,'MRN' from dual union all
select 64959,'JMA' from dual union all
select 6496,'NASA' from dual union all
select 64960,'MRN' from dual union all
select 64961,'GMS' from dual union all
select 64962,'JMA' from dual union all
select 64963,'JMA' from dual union all
select 64964,'MRN' from dual union all
select 64965,'MRN' from dual union all
select 64966,'JMA' from dual union all
select 64967,'JMA' from dual union all
select 64968,'JMA' from dual union all
select 64969,'JMA' from dual union all
select 6497,'MVS' from dual union all
select 64970,'JMA' from dual union all
select 64971,'JMA' from dual union all
select 64972,'MRN' from dual union all
select 64973,'JMA' from dual union all
select 64974,'JMA' from dual union all
select 64975,'JMA' from dual union all
select 64976,'JMA' from dual union all
select 64977,'MRN' from dual union all
select 64978,'JMA' from dual union all
select 64979,'GMS' from dual union all
select 6498,'USAF' from dual union all
select 64980,'GMS' from dual union all
select 64981,'MRN' from dual union all
select 64982,'MRN' from dual union all
select 64983,'JMA' from dual union all
select 64984,'GMS' from dual union all
select 64985,'GMS' from dual union all
select 64986,'GMS' from dual union all
select 64987,'GMS' from dual union all
select 64988,'GMS' from dual union all
select 64989,'GMS' from dual union all
select 6499,'USA' from dual union all
select 64990,'MRN' from dual union all
select 64991,'MRN' from dual union all
select 64992,'JMA' from dual union all
select 64993,'MRN' from dual union all
select 64994,'GMS' from dual union all
select 64995,'MRN' from dual union all
select 64996,'JMA' from dual union all
select 64997,'GMS' from dual union all
select 64998,'GMS' from dual union all
select 64999,'GMS' from dual union all
select 65,'WEHR' from dual union all
select 650,'ORD' from dual union all
select 6500,'NASA' from dual union all
select 65000,'GMS' from dual union all
select 65001,'MRN' from dual union all
select 65002,'MRN' from dual union all
select 65003,'GMS' from dual union all
select 65004,'MRN' from dual union all
select 65005,'MRN' from dual union all
select 65006,'MRN' from dual union all
select 65007,'GMS' from dual;
insert into LAUNCH_AGENCY
select 65008,'JMA' from dual union all
select 65009,'JMA' from dual union all
select 6501,'NASA' from dual union all
select 6501,'USAF' from dual union all
select 65010,'DLR' from dual union all
select 65011,'JMA' from dual union all
select 65012,'MRN' from dual union all
select 65013,'MRN' from dual union all
select 65014,'MRN' from dual union all
select 65015,'MRN' from dual union all
select 65016,'JMA' from dual union all
select 65017,'MRN' from dual union all
select 65018,'DLR' from dual union all
select 65019,'MRN' from dual union all
select 6502,'USAF' from dual union all
select 65020,'MRN' from dual union all
select 65021,'JMA' from dual union all
select 65022,'JMA' from dual union all
select 65023,'MRN' from dual union all
select 65024,'MRN' from dual union all
select 65025,'MRN' from dual union all
select 65026,'MRN' from dual union all
select 65027,'JMA' from dual union all
select 65028,'MRN' from dual union all
select 65029,'MRN' from dual union all
select 6503,'NASA' from dual union all
select 6503,'USAF' from dual union all
select 65030,'MRN' from dual union all
select 65031,'MRN' from dual union all
select 65032,'MRN' from dual union all
select 65033,'MRN' from dual union all
select 65034,'MRN' from dual union all
select 65035,'MRN' from dual union all
select 65036,'MRN' from dual union all
select 65037,'JMA' from dual union all
select 65038,'MRN' from dual union all
select 65039,'MRN' from dual union all
select 6504,'SAC' from dual union all
select 65040,'JMA' from dual union all
select 65041,'NASA' from dual union all
select 65042,'NASA' from dual union all
select 65043,'MRN' from dual union all
select 65044,'MRN' from dual union all
select 65045,'MRN' from dual union all
select 65046,'MRN' from dual union all
select 65047,'JMA' from dual union all
select 65048,'MRN' from dual union all
select 65049,'JMA' from dual union all
select 6505,'SAC' from dual union all
select 65050,'MRN' from dual union all
select 65051,'MRN' from dual union all
select 65052,'MRN' from dual union all
select 65053,'MRN' from dual union all
select 65054,'JMA' from dual union all
select 65055,'ISAS' from dual union all
select 65056,'MRN' from dual union all
select 65057,'MRN' from dual union all
select 65058,'JMA' from dual union all
select 65059,'MRN' from dual union all
select 6506,'NASA' from dual union all
select 65060,'NASA' from dual union all
select 65061,'MRN' from dual union all
select 65062,'MRN' from dual union all
select 65063,'MRN' from dual union all
select 65064,'MRN' from dual union all
select 65065,'MRN' from dual union all
select 65066,'MRN' from dual union all
select 65067,'MRN' from dual union all
select 65068,'MRN' from dual union all
select 65069,'MRN' from dual union all
select 6507,'NASA' from dual union all
select 6507,'USAF' from dual union all
select 65070,'MRN' from dual union all
select 65071,'MRN' from dual union all
select 65072,'MRN' from dual union all
select 65073,'MRN' from dual union all
select 65074,'MRN' from dual union all
select 65075,'MRN' from dual union all
select 65076,'MRN' from dual union all
select 65077,'MRN' from dual union all
select 65078,'MRN' from dual union all
select 65079,'MRN' from dual union all
select 6508,'USAF' from dual union all
select 65080,'MRN' from dual union all
select 65081,'MRN' from dual union all
select 65082,'MRN' from dual union all
select 65083,'MRN' from dual union all
select 65084,'MRN' from dual union all
select 65085,'MRN' from dual union all
select 65086,'MRN' from dual union all
select 65087,'MRN' from dual union all
select 65088,'MRN' from dual union all
select 65089,'MRN' from dual union all
select 6509,'NOTS' from dual union all
select 65090,'MRN' from dual union all
select 65091,'MRN' from dual union all
select 65092,'MRN' from dual union all
select 65093,'MRN' from dual union all
select 65094,'MRN' from dual union all
select 65095,'SPFL' from dual;
insert into LAUNCH_AGENCY
select 65096,'MRN' from dual union all
select 65097,'MRN' from dual union all
select 65098,'MRN' from dual union all
select 65099,'MRN' from dual union all
select 651,'ORD' from dual union all
select 6510,'USA' from dual union all
select 65100,'TSAO' from dual union all
select 65101,'TSAO' from dual union all
select 65102,'MRN' from dual union all
select 65103,'VKSR' from dual union all
select 65104,'VKSR' from dual union all
select 65105,'NASA' from dual union all
select 65106,'VKSR' from dual union all
select 65107,'VKSR' from dual union all
select 65108,'VKSR' from dual union all
select 65109,'MDSSC' from dual union all
select 6511,'USAF' from dual union all
select 65110,'VKSR' from dual union all
select 65111,'OSC' from dual union all
select 65112,'VKSR' from dual union all
select 65113,'ISAS' from dual union all
select 65114,'VKSR' from dual union all
select 65115,'VKSR' from dual union all
select 65116,'RVSNR' from dual union all
select 65117,'GDCLS' from dual union all
select 65118,'VKSR' from dual union all
select 65119,'MDSSC' from dual union all
select 6512,'USAF' from dual union all
select 65120,'VKSR' from dual union all
select 65121,'VKSR' from dual union all
select 65122,'VKSR' from dual union all
select 65123,'VKSR' from dual union all
select 65124,'VKSR' from dual union all
select 65125,'NASA' from dual union all
select 65126,'VKSR' from dual union all
select 65127,'VKSR' from dual union all
select 65128,'OSC' from dual union all
select 65129,'NASA' from dual union all
select 6513,'NASA' from dual union all
select 6513,'USAF' from dual union all
select 65130,'VKSR' from dual union all
select 65131,'VKSR' from dual union all
select 65132,'VKSR' from dual union all
select 65133,'AE' from dual union all
select 65134,'MDSSC' from dual union all
select 65135,'VKSR' from dual union all
select 65136,'VKSR' from dual union all
select 65137,'VKSR' from dual union all
select 65138,'VKSR' from dual union all
select 65139,'NASA' from dual union all
select 6514,'SMSA' from dual union all
select 6514,'WSMR' from dual union all
select 65140,'VKSR' from dual union all
select 65141,'AE' from dual union all
select 65142,'VKSR' from dual union all
select 65143,'NASA' from dual union all
select 65144,'MDSSC' from dual union all
select 65145,'VKSR' from dual union all
select 65146,'VKSR' from dual union all
select 65147,'VKSR' from dual union all
select 65148,'GDCLS' from dual union all
select 65149,'VKSR' from dual union all
select 6515,'USAF' from dual union all
select 65150,'AE' from dual union all
select 65151,'VKSR' from dual union all
select 65152,'AFSPC' from dual union all
select 65153,'VKSR' from dual union all
select 65154,'VKSR' from dual union all
select 65155,'VKSR' from dual union all
select 65156,'MDSSC' from dual union all
select 65157,'VKSR' from dual union all
select 65158,'GDCLS' from dual union all
select 65159,'VKSR' from dual union all
select 6516,'NASA' from dual union all
select 6516,'USAF' from dual union all
select 65160,'NASA' from dual union all
select 65161,'VKSR' from dual union all
select 65162,'VKSR' from dual union all
select 65163,'AE' from dual union all
select 65164,'VKSR' from dual union all
select 65165,'CALT' from dual union all
select 65166,'VKSR' from dual union all
select 65167,'NASA' from dual union all
select 65168,'AE' from dual union all
select 65169,'VKSR' from dual union all
select 6517,'NASA' from dual union all
select 6517,'USAF' from dual union all
select 65170,'MDSSC' from dual union all
select 65171,'VKSR' from dual union all
select 65172,'VKSR' from dual union all
select 65173,'VKSR' from dual union all
select 65174,'VKSR' from dual union all
select 65175,'AE' from dual union all
select 65176,'GDCLS' from dual union all
select 65177,'NASA' from dual union all
select 65178,'MDSSC' from dual union all
select 65179,'GDCLS' from dual union all
select 6518,'NASA' from dual union all
select 65180,'AE' from dual union all
select 65181,'VKSR' from dual;
insert into LAUNCH_AGENCY
select 65182,'PLA2AC' from dual union all
select 65183,'NASA' from dual union all
select 65184,'OSC' from dual union all
select 65185,'IAI' from dual union all
select 65186,'IAI' from dual union all
select 65187,'KARI' from dual union all
select 65188,'AFMC' from dual union all
select 65189,'IAI' from dual union all
select 6519,'USAF' from dual union all
select 65190,'SDIO' from dual union all
select 65191,'NASA' from dual union all
select 65192,'KARI' from dual union all
select 65193,'SDIO' from dual union all
select 65194,'USN' from dual union all
select 65195,'SDIO' from dual union all
select 65196,'IAI' from dual union all
select 65197,'IAI' from dual union all
select 65198,'PRO' from dual union all
select 65199,'USA' from dual union all
select 652,'ORD' from dual union all
select 6520,'USA' from dual union all
select 65200,'VKSR' from dual union all
select 65201,'AFMC' from dual union all
select 65202,'ISRO' from dual union all
select 65203,'AFMC' from dual union all
select 65204,'ISRO' from dual union all
select 65205,'ISRO' from dual union all
select 65206,'ISRO' from dual union all
select 65207,'ISRO' from dual union all
select 65208,'ISRO' from dual union all
select 65209,'ISRO' from dual union all
select 6521,'GMS' from dual union all
select 65210,'IDRDL' from dual union all
select 65211,'PRO' from dual union all
select 65212,'ISRO' from dual union all
select 65213,'ISRO' from dual union all
select 65214,'RVSNR' from dual union all
select 65215,'RVSNR' from dual union all
select 65216,'RVSNR' from dual union all
select 65217,'RVSNR' from dual union all
select 65218,'RVSNR' from dual union all
select 65219,'RVSNR' from dual union all
select 6522,'SAC' from dual union all
select 65220,'OSC' from dual union all
select 65221,'USA' from dual union all
select 65222,'IDRDL' from dual union all
select 65223,'NASA' from dual union all
select 65224,'ISAS' from dual union all
select 65225,'SSI' from dual union all
select 65226,'RVSNR' from dual union all
select 65227,'SDC' from dual union all
select 65228,'DLR' from dual union all
select 65229,'ACC' from dual union all
select 6523,'USAF' from dual union all
select 65230,'NASA' from dual union all
select 65231,'ACC' from dual union all
select 65232,'NASA' from dual union all
select 65233,'SDIO' from dual union all
select 65234,'IAE' from dual union all
select 65234,'INPE' from dual union all
select 65235,'NASA' from dual union all
select 65236,'INPE' from dual union all
select 65237,'NASA' from dual union all
select 65238,'NASA' from dual union all
select 65239,'ISRO' from dual union all
select 6524,'GMS' from dual union all
select 65240,'ASTD' from dual union all
select 65241,'NASA' from dual union all
select 65242,'NASA' from dual union all
select 65243,'NASA' from dual union all
select 65244,'BMDO' from dual union all
select 65245,'DLR' from dual union all
select 65246,'BMDO' from dual union all
select 65247,'KPA' from dual union all
select 65248,'KPA' from dual union all
select 65249,'KPA' from dual union all
select 6525,'NASA' from dual union all
select 6525,'USAF' from dual union all
select 65250,'KPA' from dual union all
select 65251,'IDRDL' from dual union all
select 65252,'NASA' from dual union all
select 65253,'BMDO' from dual union all
select 65254,'AFSPC' from dual union all
select 65254,'ALCS' from dual union all
select 65255,'USN' from dual union all
select 65256,'USN' from dual union all
select 65257,'AFSPC' from dual union all
select 65258,'NASA' from dual union all
select 65259,'RVSNR' from dual union all
select 6526,'USN' from dual union all
select 65260,'MATRA' from dual union all
select 65261,'NDRE' from dual union all
select 65262,'DLR' from dual union all
select 65262,'SSC' from dual union all
select 65263,'DLR' from dual union all
select 65263,'SSC' from dual union all
select 65264,'NASA' from dual union all
select 65265,'USN' from dual union all
select 65266,'SDC' from dual union all
select 65267,'AFSPC' from dual;
insert into LAUNCH_AGENCY
select 65268,'NASA' from dual union all
select 65269,'NASA' from dual union all
select 6527,'RAE' from dual union all
select 65270,'AFSPC' from dual union all
select 65271,'NASDA' from dual union all
select 65272,'NASA' from dual union all
select 65273,'NASA' from dual union all
select 65274,'USA' from dual union all
select 65275,'INTA' from dual union all
select 65276,'USA' from dual union all
select 65277,'DMA' from dual union all
select 65278,'SSC' from dual union all
select 65279,'IAE' from dual union all
select 6528,'GMS' from dual union all
select 65280,'USN' from dual union all
select 65281,'ASTD' from dual union all
select 65282,'DLR' from dual union all
select 65283,'RVSN' from dual union all
select 65284,'VMFR' from dual union all
select 65285,'USA' from dual union all
select 65286,'USA' from dual union all
select 65287,'ISRO' from dual union all
select 65288,'ISRO' from dual union all
select 65289,'PLA2AC' from dual union all
select 6529,'NASA' from dual union all
select 6529,'USAF' from dual union all
select 65290,'JMA' from dual union all
select 65291,'MRN' from dual union all
select 65292,'JMA' from dual union all
select 65293,'JMA' from dual union all
select 65294,'MRN' from dual union all
select 65295,'JMA' from dual union all
select 65296,'JMA' from dual union all
select 65297,'JMA' from dual union all
select 65298,'JMA' from dual union all
select 65299,'MRN' from dual union all
select 653,'ORD' from dual union all
select 6530,'GMS' from dual union all
select 65300,'MRN' from dual union all
select 65301,'JMA' from dual union all
select 65302,'MRN' from dual union all
select 65303,'MRN' from dual union all
select 65304,'MRN' from dual union all
select 65305,'MRN' from dual union all
select 65306,'JMA' from dual union all
select 65307,'JMA' from dual union all
select 65308,'JMA' from dual union all
select 65309,'MRN' from dual union all
select 6531,'USA' from dual union all
select 65310,'JMA' from dual union all
select 65311,'MRN' from dual union all
select 65312,'JMA' from dual union all
select 65313,'MRN' from dual union all
select 65314,'JMA' from dual union all
select 65315,'MRN' from dual union all
select 65316,'MRN' from dual union all
select 65317,'JMA' from dual union all
select 65318,'JMA' from dual union all
select 65319,'MRN' from dual union all
select 6532,'GMS' from dual union all
select 65320,'MRN' from dual union all
select 65321,'JMA' from dual union all
select 65322,'MRN' from dual union all
select 65323,'MRN' from dual union all
select 65324,'JMA' from dual union all
select 65325,'MRN' from dual union all
select 65326,'MRN' from dual union all
select 65327,'JMA' from dual union all
select 65328,'MRN' from dual union all
select 65329,'JMA' from dual union all
select 6533,'NASA' from dual union all
select 65330,'MRN' from dual union all
select 65331,'JMA' from dual union all
select 65332,'MRN' from dual union all
select 65333,'MRN' from dual union all
select 65334,'JMA' from dual union all
select 65335,'JMA' from dual union all
select 65336,'ISAS' from dual union all
select 65337,'MRN' from dual union all
select 65338,'JMA' from dual union all
select 65339,'MRN' from dual union all
select 6534,'NASA' from dual union all
select 6534,'USAF' from dual union all
select 65340,'JMA' from dual union all
select 65341,'JMA' from dual union all
select 65342,'JMA' from dual union all
select 65343,'JMA' from dual union all
select 65344,'MRN' from dual union all
select 65345,'MRN' from dual union all
select 65346,'JMA' from dual union all
select 65347,'JMA' from dual union all
select 65348,'JMA' from dual union all
select 65349,'JMA' from dual union all
select 6535,'PRO' from dual union all
select 65350,'ISAS' from dual union all
select 65351,'JMA' from dual union all
select 65352,'JMA' from dual union all
select 65353,'JMA' from dual union all
select 65354,'MRN' from dual union all
select 65355,'MRN' from dual;
insert into LAUNCH_AGENCY
select 65356,'MRN' from dual union all
select 65357,'MRN' from dual union all
select 65358,'MRN' from dual union all
select 65359,'MRN' from dual union all
select 6536,'GMS' from dual union all
select 65360,'MRN' from dual union all
select 65361,'MRN' from dual union all
select 65362,'MRN' from dual union all
select 65363,'MRN' from dual union all
select 65364,'MRN' from dual union all
select 65365,'MRN' from dual union all
select 65366,'MRN' from dual union all
select 65367,'MRN' from dual union all
select 65368,'MRN' from dual union all
select 65369,'MRN' from dual union all
select 6537,'GMS' from dual union all
select 65370,'MRN' from dual union all
select 65371,'MRN' from dual union all
select 65372,'MRN' from dual union all
select 65373,'MRN' from dual union all
select 65374,'MRN' from dual union all
select 65375,'MRN' from dual union all
select 65376,'AN' from dual union all
select 65377,'MRN' from dual union all
select 65378,'MRN' from dual union all
select 65379,'MRN' from dual union all
select 6538,'USA' from dual union all
select 65380,'MRN' from dual union all
select 65381,'MRN' from dual union all
select 65382,'MRN' from dual union all
select 65383,'MRN' from dual union all
select 65384,'MRN' from dual union all
select 65385,'AN' from dual union all
select 65386,'AN' from dual union all
select 65387,'AN' from dual union all
select 65388,'AN' from dual union all
select 65389,'AN' from dual union all
select 6539,'USA' from dual union all
select 65390,'AN' from dual union all
select 65391,'NASA' from dual union all
select 65392,'NASA' from dual union all
select 65393,'NASA' from dual union all
select 65394,'NASA' from dual union all
select 65395,'NASA' from dual union all
select 65396,'NASA' from dual union all
select 65397,'VKSR' from dual union all
select 65398,'VKSR' from dual union all
select 65399,'VKSR' from dual union all
select 654,'US' from dual union all
select 654,'WSPG' from dual union all
select 6540,'USA' from dual union all
select 65400,'AFMC' from dual union all
select 65401,'VKSR' from dual union all
select 65402,'NASA' from dual union all
select 65403,'NASDA' from dual union all
select 65404,'VKSR' from dual union all
select 65405,'AFSC' from dual union all
select 65406,'CALT' from dual union all
select 65407,'VKSR' from dual union all
select 65408,'VKSR' from dual union all
select 65409,'MDSSC' from dual union all
select 6541,'USA' from dual union all
select 65410,'VKSR' from dual union all
select 65411,'NASA' from dual union all
select 65412,'MDSSC' from dual union all
select 65413,'OSC' from dual union all
select 65414,'VKSR' from dual union all
select 65415,'VKSR' from dual union all
select 65416,'NASA' from dual union all
select 65417,'VKSR' from dual union all
select 65418,'GDCLS' from dual union all
select 65419,'VKSR' from dual union all
select 6542,'GMS' from dual union all
select 65420,'VKSR' from dual union all
select 65421,'VKSR' from dual union all
select 65422,'USAF' from dual union all
select 65423,'ISRO' from dual union all
select 65424,'NASA' from dual union all
select 65425,'OSC' from dual union all
select 65426,'VKSR' from dual union all
select 65427,'VKSR' from dual union all
select 65428,'VKSR' from dual union all
select 65429,'VKSR' from dual union all
select 6543,'ISAS' from dual union all
select 65430,'AE' from dual union all
select 65431,'MMCLS' from dual union all
select 65432,'VKSR' from dual union all
select 65433,'SAST' from dual union all
select 65434,'VKSR' from dual union all
select 65435,'NASA' from dual union all
select 65436,'AE' from dual union all
select 65437,'VKSR' from dual union all
select 65438,'VKSR' from dual union all
select 65439,'CALT' from dual union all
select 6544,'ISAS' from dual union all
select 65440,'VKSR' from dual union all
select 65441,'VKSR' from dual union all
select 65442,'OSC' from dual union all
select 65443,'MMCLS' from dual union all
select 65444,'VKSR' from dual;
insert into LAUNCH_AGENCY
select 65445,'AE' from dual union all
select 65446,'VKSR' from dual union all
select 65447,'VKSR' from dual union all
select 65448,'VKSR' from dual union all
select 65449,'VKSR' from dual union all
select 6545,'GMS' from dual union all
select 65450,'USAF' from dual union all
select 65451,'CALT' from dual union all
select 65452,'NASDA' from dual union all
select 65453,'AFSPC' from dual union all
select 65454,'AE' from dual union all
select 65455,'NASA' from dual union all
select 65456,'VKSR' from dual union all
select 65457,'VKSR' from dual union all
select 65458,'NASA' from dual union all
select 65459,'VKSR' from dual union all
select 6546,'NASA' from dual union all
select 6546,'USAF' from dual union all
select 65460,'MMCLS' from dual union all
select 65461,'AE' from dual union all
select 65462,'VKSR' from dual union all
select 65463,'VKSR' from dual union all
select 65464,'ISRO' from dual union all
select 65465,'VKSR' from dual union all
select 65466,'AE' from dual union all
select 65467,'MDSSC' from dual union all
select 65468,'VKSR' from dual union all
select 65469,'NASA' from dual union all
select 6547,'NASA' from dual union all
select 65470,'VKSR' from dual union all
select 65471,'VKSR' from dual union all
select 65472,'VKSR' from dual union all
select 65473,'VKSR' from dual union all
select 65474,'VKSR' from dual union all
select 65475,'MMCLS' from dual union all
select 65476,'CALT' from dual union all
select 65477,'VKSR' from dual union all
select 65478,'VKSR' from dual union all
select 65479,'VKSR' from dual union all
select 6548,'GMS' from dual union all
select 65480,'USAF' from dual union all
select 65481,'VKSR' from dual union all
select 65482,'VKSR' from dual union all
select 65483,'VKSR' from dual union all
select 65484,'VKSR' from dual union all
select 65485,'MMCLS' from dual union all
select 65486,'USN' from dual union all
select 65487,'JMA' from dual union all
select 65488,'LLL' from dual union all
select 65489,'IAI' from dual union all
select 6549,'GMS' from dual union all
select 65490,'IAI' from dual union all
select 65491,'SDIO' from dual union all
select 65492,'SDIO' from dual union all
select 65493,'AE' from dual union all
select 65494,'VKSR' from dual union all
select 65495,'OSC' from dual union all
select 65496,'AE' from dual union all
select 65497,'SSC' from dual union all
select 65498,'RVSNR' from dual union all
select 65499,'RVSNR' from dual union all
select 655,'GEMTS' from dual union all
select 655,'US' from dual union all
select 6550,'USAF' from dual union all
select 65500,'RVSNR' from dual union all
select 65501,'USN' from dual union all
select 65502,'USN' from dual union all
select 65503,'USN' from dual union all
select 65504,'USN' from dual union all
select 65505,'NASA' from dual union all
select 65506,'AFSPC' from dual union all
select 65506,'ALCS' from dual union all
select 65507,'NASA' from dual union all
select 65507,'NTNF' from dual union all
select 65508,'IR' from dual union all
select 65509,'IR' from dual union all
select 6551,'GMS' from dual union all
select 65510,'IR' from dual union all
select 65511,'CSA' from dual union all
select 65511,'NASA' from dual union all
select 65512,'NASA' from dual union all
select 65513,'NASA' from dual union all
select 65514,'NASA' from dual union all
select 65515,'USA' from dual union all
select 65516,'ISAS' from dual union all
select 65517,'DLR' from dual union all
select 65518,'IDRDL' from dual union all
select 65519,'MATRA' from dual union all
select 6552,'SAC' from dual union all
select 65520,'NASA' from dual union all
select 65521,'NASA' from dual union all
select 65522,'AFSPC' from dual union all
select 65523,'NASA' from dual union all
select 65524,'USN' from dual union all
select 65525,'USN' from dual union all
select 65526,'USN' from dual union all
select 65527,'USN' from dual union all
select 65528,'NASA' from dual union all
select 65529,'INTA' from dual union all
select 6553,'GMS' from dual;
insert into LAUNCH_AGENCY
select 65530,'NASA' from dual union all
select 65531,'NASA' from dual union all
select 65532,'DLR' from dual union all
select 65533,'ASTD' from dual union all
select 65534,'AFSPC' from dual union all
select 65535,'RN' from dual union all
select 65536,'IDRDL' from dual union all
select 65537,'IDRDL' from dual union all
select 65538,'AFSPC' from dual union all
select 65539,'RN' from dual union all
select 6554,'GMS' from dual union all
select 65540,'IAE' from dual union all
select 65540,'INPE' from dual union all
select 65541,'NASA' from dual union all
select 65542,'RVSNR' from dual union all
select 65543,'NASA' from dual union all
select 65544,'SYMC' from dual union all
select 65545,'AFSPC' from dual union all
select 65546,'BMDO' from dual union all
select 65547,'NASA' from dual union all
select 65548,'NASA' from dual union all
select 65549,'NASA' from dual union all
select 6555,'NASA' from dual union all
select 6555,'USAF' from dual union all
select 65550,'SDC' from dual union all
select 65551,'DLR' from dual union all
select 65552,'DLR' from dual union all
select 65553,'USN' from dual union all
select 65554,'DLR' from dual union all
select 65555,'NASA' from dual union all
select 65556,'DOD' from dual union all
select 65557,'GSFC' from dual union all
select 65558,'GSFC' from dual union all
select 65559,'GSFC' from dual union all
select 6556,'USAF' from dual union all
select 65560,'GSFC' from dual union all
select 65561,'GSFC' from dual union all
select 65562,'AFSPC' from dual union all
select 65563,'NASA' from dual union all
select 65564,'GSFC' from dual union all
select 65565,'IDF' from dual union all
select 65566,'GSFC' from dual union all
select 65567,'ISAS' from dual union all
select 65568,'NASA' from dual union all
select 65569,'RVSNR' from dual union all
select 6557,'USAF' from dual union all
select 65570,'GSFC' from dual union all
select 65571,'GSFC' from dual union all
select 65572,'GSFC' from dual union all
select 65573,'GSFC' from dual union all
select 65574,'NASA' from dual union all
select 65575,'AFSPC' from dual union all
select 65576,'NASA' from dual union all
select 65577,'NASA' from dual union all
select 65578,'NASA' from dual union all
select 65579,'USN' from dual union all
select 6558,'RAE' from dual union all
select 6558,'WRE' from dual union all
select 65580,'NASA' from dual union all
select 65581,'NASA' from dual union all
select 65582,'RVSNR' from dual union all
select 65583,'NASA' from dual union all
select 65584,'ISAS' from dual union all
select 65585,'ASTD' from dual union all
select 65586,'USN' from dual union all
select 65587,'ISAS' from dual union all
select 65588,'NASA' from dual union all
select 65589,'USN' from dual union all
select 6559,'NASA' from dual union all
select 6559,'USAF' from dual union all
select 65590,'USN' from dual union all
select 65591,'CSA' from dual union all
select 65592,'NASA' from dual union all
select 65593,'RVSNR' from dual union all
select 65594,'YSP' from dual union all
select 65595,'YSP' from dual union all
select 65596,'YSP' from dual union all
select 65597,'USN' from dual union all
select 65598,'USN' from dual union all
select 65599,'BMDO' from dual union all
select 656,'GEMTS' from dual union all
select 656,'US' from dual union all
select 6560,'NASA' from dual union all
select 65600,'JMA' from dual union all
select 65601,'ISAS' from dual union all
select 65602,'ISAS' from dual union all
select 65603,'ISAS' from dual union all
select 65604,'ISAS' from dual union all
select 65605,'JMA' from dual union all
select 65606,'JMA' from dual union all
select 65607,'JMA' from dual union all
select 65608,'JMA' from dual union all
select 65609,'JMA' from dual union all
select 6561,'PRO' from dual union all
select 65610,'JMA' from dual union all
select 65611,'JMA' from dual union all
select 65612,'JMA' from dual union all
select 65613,'JMA' from dual union all
select 65614,'JMA' from dual union all
select 65615,'JMA' from dual;
insert into LAUNCH_AGENCY
select 65616,'JMA' from dual union all
select 65617,'JMA' from dual union all
select 65618,'JMA' from dual union all
select 65619,'JMA' from dual union all
select 6562,'GMS' from dual union all
select 65620,'JMA' from dual union all
select 65621,'JMA' from dual union all
select 65622,'JMA' from dual union all
select 65623,'JMA' from dual union all
select 65624,'JMA' from dual union all
select 65625,'JMA' from dual union all
select 65626,'JMA' from dual union all
select 65627,'JMA' from dual union all
select 65628,'JMA' from dual union all
select 65629,'JMA' from dual union all
select 6563,'GMS' from dual union all
select 65630,'JMA' from dual union all
select 65631,'JMA' from dual union all
select 65632,'JMA' from dual union all
select 65633,'JMA' from dual union all
select 65634,'JMA' from dual union all
select 65635,'JMA' from dual union all
select 65636,'JMA' from dual union all
select 65637,'JMA' from dual union all
select 65638,'JMA' from dual union all
select 65639,'JMA' from dual union all
select 6564,'GMS' from dual union all
select 65640,'NASA' from dual union all
select 65641,'NASA' from dual union all
select 65642,'NASA' from dual union all
select 65643,'NASA' from dual union all
select 65644,'NASA' from dual union all
select 65645,'NASA' from dual union all
select 65646,'NASA' from dual union all
select 65647,'NASA' from dual union all
select 65648,'NASA' from dual union all
select 65649,'NASA' from dual union all
select 6565,'RAE' from dual union all
select 65650,'NASA' from dual union all
select 65651,'NASA' from dual union all
select 65652,'NASA' from dual union all
select 65653,'NASA' from dual union all
select 65654,'NASA' from dual union all
select 65655,'NASA' from dual union all
select 65656,'NASA' from dual union all
select 65657,'NASA' from dual union all
select 65658,'NASA' from dual union all
select 65659,'NASA' from dual union all
select 6566,'NASA' from dual union all
select 6566,'USAF' from dual union all
select 65660,'NASA' from dual union all
select 65661,'NASA' from dual union all
select 65662,'MMCLS' from dual union all
select 65663,'VKSR' from dual union all
select 65664,'MMCLS' from dual union all
select 65665,'NASA' from dual union all
select 65666,'VKSR' from dual union all
select 65667,'VKSR' from dual union all
select 65668,'NASA' from dual union all
select 65669,'VKSR' from dual union all
select 6567,'SU' from dual union all
select 65670,'VKSR' from dual union all
select 65671,'VKSR' from dual union all
select 65672,'NASDA' from dual union all
select 65673,'VKSR' from dual union all
select 65674,'LMA' from dual union all
select 65675,'VKSR' from dual union all
select 65676,'AFSPC' from dual union all
select 65677,'AE' from dual union all
select 65678,'OSC' from dual union all
select 65679,'ISA' from dual union all
select 6568,'USAF' from dual union all
select 65680,'LMA' from dual union all
select 65681,'VKSR' from dual union all
select 65682,'AE' from dual union all
select 65683,'USAF' from dual union all
select 65684,'AE' from dual union all
select 65685,'VKSR' from dual union all
select 65686,'LMA' from dual union all
select 65687,'VKSR' from dual union all
select 65688,'LMA' from dual union all
select 65689,'VKSR' from dual union all
select 6569,'ONERA' from dual union all
select 65690,'AE' from dual union all
select 65691,'NASA' from dual union all
select 65692,'VKSR' from dual union all
select 65693,'VKSR' from dual union all
select 65694,'AE' from dual union all
select 65695,'USAF' from dual union all
select 65696,'NASA' from dual union all
select 65697,'VKSR' from dual union all
select 65698,'VKSR' from dual union all
select 65699,'LMA' from dual union all
select 657,'NRL' from dual union all
select 657,'US' from dual union all
select 6570,'ONERA' from dual union all
select 65700,'VKSR' from dual union all
select 65701,'AE' from dual union all
select 65702,'MDSSC' from dual union all
select 65703,'VKSR' from dual;
insert into LAUNCH_AGENCY
select 65704,'LMA' from dual union all
select 65705,'AE' from dual union all
select 65706,'VKSR' from dual union all
select 65707,'VKSR' from dual union all
select 65708,'VKSR' from dual union all
select 65709,'NASA' from dual union all
select 6571,'USAF' from dual union all
select 65710,'AE' from dual union all
select 65711,'VKSR' from dual union all
select 65712,'VKSR' from dual union all
select 65713,'VKSR' from dual union all
select 65714,'VKSR' from dual union all
select 65715,'VKSR' from dual union all
select 65716,'AE' from dual union all
select 65717,'NASA' from dual union all
select 65718,'LMA' from dual union all
select 65719,'VKSR' from dual union all
select 6572,'PRO' from dual union all
select 65720,'MDSSC' from dual union all
select 65721,'USAF' from dual union all
select 65722,'NASA' from dual union all
select 65723,'AE' from dual union all
select 65724,'VKSR' from dual union all
select 65725,'CALT' from dual union all
select 65726,'LMA' from dual union all
select 65727,'AFSPC' from dual union all
select 65728,'AE' from dual union all
select 65729,'VKSR' from dual union all
select 6573,'PRO' from dual union all
select 65730,'LMA' from dual union all
select 65731,'VKSR' from dual union all
select 65732,'VKSR' from dual union all
select 65733,'VKSR' from dual union all
select 65734,'CALT' from dual union all
select 65735,'MDSSC' from dual union all
select 65736,'USN' from dual union all
select 65737,'USN' from dual union all
select 65738,'SDIO' from dual union all
select 65739,'SDIO' from dual union all
select 6574,'ISAS' from dual union all
select 65740,'SDIO' from dual union all
select 65741,'IAI' from dual union all
select 65742,'ISAS' from dual union all
select 65743,'CALT' from dual union all
select 65744,'RVSNR' from dual union all
select 65745,'OSC' from dual union all
select 65746,'LMSC' from dual union all
select 65747,'EER' from dual union all
select 65748,'LAPAN' from dual union all
select 65749,'NASA' from dual union all
select 6575,'NASA' from dual union all
select 6575,'USAF' from dual union all
select 65750,'PLA2AC' from dual union all
select 65751,'USN' from dual union all
select 65752,'USN' from dual union all
select 65753,'AFSPC' from dual union all
select 65754,'ISAS' from dual union all
select 65755,'NASA' from dual union all
select 65756,'ISAS' from dual union all
select 65757,'AFSPC' from dual union all
select 65758,'NASA' from dual union all
select 65759,'NASA' from dual union all
select 6576,'USN' from dual union all
select 65760,'USA' from dual union all
select 65761,'BMDO' from dual union all
select 65762,'DMA' from dual union all
select 65763,'NASA' from dual union all
select 65764,'BMDO' from dual union all
select 65765,'USN' from dual union all
select 65766,'USN' from dual union all
select 65767,'USN' from dual union all
select 65768,'USN' from dual union all
select 65769,'USN' from dual union all
select 6577,'USN' from dual union all
select 65770,'GSFC' from dual union all
select 65771,'AFSPC' from dual union all
select 65772,'GSFC' from dual union all
select 65773,'NASA' from dual union all
select 65774,'GSFC' from dual union all
select 65775,'GSFC' from dual union all
select 65776,'BMDO' from dual union all
select 65777,'NASA' from dual union all
select 65778,'USN' from dual union all
select 65779,'USN' from dual union all
select 6578,'USN' from dual union all
select 65780,'RVSNR' from dual union all
select 65781,'NASA' from dual union all
select 65782,'NASA' from dual union all
select 65783,'USA' from dual union all
select 65784,'COLE' from dual union all
select 65784,'USA' from dual union all
select 65785,'DLR' from dual union all
select 65786,'DLR' from dual union all
select 65787,'NASA' from dual union all
select 65788,'NASA' from dual union all
select 65789,'CALT' from dual union all
select 6579,'USN' from dual union all
select 65790,'NASA' from dual union all
select 65791,'VMFR' from dual union all
select 65792,'RVSNR' from dual;
insert into LAUNCH_AGENCY
select 65793,'AFSPC' from dual union all
select 65794,'SMDC' from dual union all
select 65795,'SMDC' from dual union all
select 65796,'PLA2AC' from dual union all
select 65797,'NASA' from dual union all
select 65798,'DLR' from dual union all
select 65799,'RN' from dual union all
select 658,'GEMTS' from dual union all
select 658,'US' from dual union all
select 6580,'USN' from dual union all
select 65800,'GSFC' from dual union all
select 65801,'USA' from dual union all
select 65802,'GSFC' from dual union all
select 65803,'SMDC' from dual union all
select 65804,'ISRO' from dual union all
select 65805,'RN' from dual union all
select 65806,'NASDA' from dual union all
select 65807,'VMF' from dual union all
select 65808,'GSFC' from dual union all
select 65809,'AFSPC' from dual union all
select 6581,'USN' from dual union all
select 65810,'GSFC' from dual union all
select 65811,'RVSNR' from dual union all
select 65812,'NASA' from dual union all
select 65813,'COLE' from dual union all
select 65813,'USA' from dual union all
select 65814,'DLR' from dual union all
select 65815,'RVSNR' from dual union all
select 65816,'SMDC' from dual union all
select 65817,'USA' from dual union all
select 65818,'GSFC' from dual union all
select 65819,'GSFC' from dual union all
select 6582,'USN' from dual union all
select 65820,'USN' from dual union all
select 65821,'RVSNR' from dual union all
select 65822,'GSFC' from dual union all
select 65823,'CSA' from dual union all
select 65823,'NASA' from dual union all
select 65824,'PLA2AC' from dual union all
select 65825,'GSFC' from dual union all
select 65826,'GSFC' from dual union all
select 65827,'GSFC' from dual union all
select 65828,'GSFC' from dual union all
select 65829,'GSFC' from dual union all
select 6583,'USN' from dual union all
select 65830,'GSFC' from dual union all
select 65831,'DLR' from dual union all
select 65831,'SSC' from dual union all
select 65832,'SSC' from dual union all
select 65833,'GSFC' from dual union all
select 65834,'USN' from dual union all
select 65835,'SMDC' from dual union all
select 65836,'USA' from dual union all
select 65837,'IAE' from dual union all
select 65838,'USN' from dual union all
select 65839,'RVSNR' from dual union all
select 6584,'PRO' from dual union all
select 65840,'PLA2AC' from dual union all
select 65841,'PLA2AC' from dual union all
select 65842,'ISAS' from dual union all
select 65843,'JMA' from dual union all
select 65844,'JMA' from dual union all
select 65845,'JMA' from dual union all
select 65846,'DLR' from dual union all
select 65847,'DLR' from dual union all
select 65848,'DLR' from dual union all
select 65849,'DLR' from dual union all
select 6585,'USAF' from dual union all
select 65850,'NASA' from dual union all
select 65851,'NASA' from dual union all
select 65852,'DLR' from dual union all
select 65853,'DLR' from dual union all
select 65854,'JMA' from dual union all
select 65855,'JMA' from dual union all
select 65856,'JMA' from dual union all
select 65857,'JMA' from dual union all
select 65858,'JMA' from dual union all
select 65859,'JMA' from dual union all
select 6586,'GMS' from dual union all
select 65860,'JMA' from dual union all
select 65861,'JMA' from dual union all
select 65862,'JMA' from dual union all
select 65863,'JMA' from dual union all
select 65864,'JMA' from dual union all
select 65865,'JMA' from dual union all
select 65866,'JMA' from dual union all
select 65867,'JMA' from dual union all
select 65868,'JMA' from dual union all
select 65869,'JMA' from dual union all
select 6587,'USAF' from dual union all
select 65870,'JMA' from dual union all
select 65871,'JMA' from dual union all
select 65872,'JMA' from dual union all
select 65873,'JMA' from dual union all
select 65874,'JMA' from dual union all
select 65875,'JMA' from dual union all
select 65876,'JMA' from dual union all
select 65877,'JMA' from dual union all
select 65878,'JMA' from dual union all
select 65879,'JMA' from dual;
insert into LAUNCH_AGENCY
select 6588,'USN' from dual union all
select 65880,'JMA' from dual union all
select 65881,'JMA' from dual union all
select 65882,'JMA' from dual union all
select 65883,'JMA' from dual union all
select 65884,'JMA' from dual union all
select 65885,'JMA' from dual union all
select 65886,'JMA' from dual union all
select 65887,'MRN' from dual union all
select 65888,'MRN' from dual union all
select 65889,'NASA' from dual union all
select 6589,'NASA' from dual union all
select 65890,'AE' from dual union all
select 65891,'MDSSC' from dual union all
select 65892,'VKSR' from dual union all
select 65893,'VKSR' from dual union all
select 65894,'LMA' from dual union all
select 65895,'AE' from dual union all
select 65896,'MDSSC' from dual union all
select 65897,'VKSR' from dual union all
select 65898,'VKSR' from dual union all
select 65899,'VKSR' from dual union all
select 659,'APL' from dual union all
select 659,'US' from dual union all
select 6590,'USN' from dual union all
select 65900,'NASA' from dual union all
select 65901,'MDSSC' from dual union all
select 65902,'OSC' from dual union all
select 65903,'AE' from dual union all
select 65904,'VKSR' from dual union all
select 65905,'ISRO' from dual union all
select 65906,'NASA' from dual union all
select 65907,'MDSSC' from dual union all
select 65908,'LMA' from dual union all
select 65909,'ILSK' from dual union all
select 6591,'WSMR' from dual union all
select 65910,'AE' from dual union all
select 65911,'VKSR' from dual union all
select 65912,'MDSSC' from dual union all
select 65913,'VKSR' from dual union all
select 65914,'USAF' from dual union all
select 65915,'LMA' from dual union all
select 65916,'VKSR' from dual union all
select 65917,'AFSPC' from dual union all
select 65918,'AE' from dual union all
select 65919,'OSC' from dual union all
select 6592,'USN' from dual union all
select 65920,'NASA' from dual union all
select 65921,'MDSSC' from dual union all
select 65922,'VKSR' from dual union all
select 65923,'AE' from dual union all
select 65924,'NASA' from dual union all
select 65925,'OSC' from dual union all
select 65926,'USAF' from dual union all
select 65927,'CALT' from dual union all
select 65928,'AE' from dual union all
select 65929,'MDSSC' from dual union all
select 6593,'RAE' from dual union all
select 65930,'LMA' from dual union all
select 65931,'VKSR' from dual union all
select 65932,'AE' from dual union all
select 65933,'VKSR' from dual union all
select 65934,'NASDA' from dual union all
select 65935,'VKSR' from dual union all
select 65936,'CALT' from dual union all
select 65937,'OSC' from dual union all
select 65938,'VKSR' from dual union all
select 65939,'VKSR' from dual union all
select 6594,'RAE' from dual union all
select 65940,'VKSR' from dual union all
select 65941,'VKSR' from dual union all
select 65942,'LMA' from dual union all
select 65943,'AE' from dual union all
select 65944,'MDSSC' from dual union all
select 65945,'NASA' from dual union all
select 65946,'VKSR' from dual union all
select 65947,'SAST' from dual union all
select 65948,'VKSR' from dual union all
select 65949,'OSC' from dual union all
select 6595,'WADD' from dual union all
select 65950,'MDSSC' from dual union all
select 65951,'AE' from dual union all
select 65952,'VKSR' from dual union all
select 65953,'NASA' from dual union all
select 65954,'VKSR' from dual union all
select 65955,'LMA' from dual union all
select 65956,'MDSSC' from dual union all
select 65957,'VKSR' from dual union all
select 65958,'LMA' from dual union all
select 65959,'VKSR' from dual union all
select 6596,'USA' from dual union all
select 65960,'AFSPC' from dual union all
select 65961,'VKSR' from dual union all
select 65962,'RVSN' from dual union all
select 65962,'VMF' from dual union all
select 65963,'MSFC' from dual union all
select 65964,'MSFC' from dual union all
select 65965,'MSFC' from dual union all
select 65966,'MSFC' from dual union all
select 65967,'USN' from dual;
insert into LAUNCH_AGENCY
select 65968,'VMF' from dual union all
select 65969,'MSFC' from dual union all
select 6597,'USAF' from dual union all
select 65970,'MSFC' from dual union all
select 65971,'ARR' from dual union all
select 65972,'ARR' from dual union all
select 65973,'BMDO' from dual union all
select 65974,'DMA' from dual union all
select 65975,'BMDO' from dual union all
select 65976,'BMDO' from dual union all
select 65977,'CALT' from dual union all
select 65978,'VKSR' from dual union all
select 65979,'ESA' from dual union all
select 6598,'USA' from dual union all
select 65980,'VKSR' from dual union all
select 65981,'IAI' from dual union all
select 65982,'IAI' from dual union all
select 65983,'IAI' from dual union all
select 65984,'RVSN' from dual union all
select 65985,'IDRDL' from dual union all
select 65986,'RVSNR' from dual union all
select 65987,'PLA2AC' from dual union all
select 65988,'IDRDL' from dual union all
select 65989,'COLE' from dual union all
select 65989,'USA' from dual union all
select 6599,'USA' from dual union all
select 65990,'ISAS' from dual union all
select 65991,'NASDA' from dual union all
select 65992,'GSFC' from dual union all
select 65993,'GSFC' from dual union all
select 65994,'ASTD' from dual union all
select 65995,'GSFC' from dual union all
select 65996,'AFSPC' from dual union all
select 65997,'COLE' from dual union all
select 65997,'USA' from dual union all
select 65998,'USA' from dual union all
select 65999,'SSI' from dual union all
select 66,'WEHR' from dual union all
select 660,'NRL' from dual union all
select 660,'US' from dual union all
select 6600,'USA' from dual union all
select 66000,'RVSNR' from dual union all
select 66001,'USN' from dual union all
select 66002,'USN' from dual union all
select 66003,'SSC' from dual union all
select 66004,'AFSPC' from dual union all
select 66005,'USN' from dual union all
select 66006,'USN' from dual union all
select 66007,'USN' from dual union all
select 66008,'USN' from dual union all
select 66009,'AFSPC' from dual union all
select 6601,'USA' from dual union all
select 66010,'GSFC' from dual union all
select 66011,'RVSNR' from dual union all
select 66012,'GSFC' from dual union all
select 66013,'GSFC' from dual union all
select 66014,'AFSPC' from dual union all
select 66014,'ALCS' from dual union all
select 66015,'AFSPC' from dual union all
select 66016,'GSFC' from dual union all
select 66017,'VMFR' from dual union all
select 66018,'VMFR' from dual union all
select 66019,'VMFR' from dual union all
select 6602,'USA' from dual union all
select 66020,'AFSMC' from dual union all
select 66021,'NASA' from dual union all
select 66022,'VMFR' from dual union all
select 66023,'VMFR' from dual union all
select 66024,'COLE' from dual union all
select 66024,'USA' from dual union all
select 66025,'USA' from dual union all
select 66026,'OSC' from dual union all
select 66027,'USN' from dual union all
select 66028,'USN' from dual union all
select 66029,'RVSNR' from dual union all
select 6603,'USA' from dual union all
select 66030,'NASA' from dual union all
select 66031,'GSFC' from dual union all
select 66032,'ISAS' from dual union all
select 66033,'NASA' from dual union all
select 66034,'ISAS' from dual union all
select 66035,'IAE' from dual union all
select 66035,'INPE' from dual union all
select 66036,'BMDO' from dual union all
select 66037,'SDC' from dual union all
select 66038,'AFSPC' from dual union all
select 66039,'NASDA' from dual union all
select 6604,'USA' from dual union all
select 66040,'AFMC' from dual union all
select 66041,'USN' from dual union all
select 66042,'VMFR' from dual union all
select 66043,'RVSNR' from dual union all
select 66044,'COLE' from dual union all
select 66044,'USA' from dual union all
select 66045,'BMDO' from dual union all
select 66046,'GSFC' from dual union all
select 66047,'GSFC' from dual union all
select 66048,'USN' from dual union all
select 66049,'USN' from dual union all
select 6605,'WRE' from dual;
insert into LAUNCH_AGENCY
select 66050,'GSFC' from dual union all
select 66051,'AFSPC' from dual union all
select 66052,'RVSNR' from dual union all
select 66053,'GSFC' from dual union all
select 66054,'GSFC' from dual union all
select 66055,'ASTD' from dual union all
select 66056,'RVSN' from dual union all
select 66057,'PLA2AC' from dual union all
select 66058,'USN' from dual union all
select 66059,'PLA2AC' from dual union all
select 6606,'WRE' from dual union all
select 66060,'PLA2AC' from dual union all
select 66061,'PLA2AC' from dual union all
select 66062,'PLA2AC' from dual union all
select 66063,'MRN' from dual union all
select 66064,'JMA' from dual union all
select 66065,'JMA' from dual union all
select 66066,'MRN' from dual union all
select 66067,'JMA' from dual union all
select 66068,'MRN' from dual union all
select 66069,'JMA' from dual union all
select 6607,'WRE' from dual union all
select 66070,'ISAS' from dual union all
select 66071,'JMA' from dual union all
select 66072,'JMA' from dual union all
select 66073,'JMA' from dual union all
select 66074,'MRN' from dual union all
select 66075,'JMA' from dual union all
select 66076,'JMA' from dual union all
select 66077,'NASA' from dual union all
select 66078,'MRN' from dual union all
select 66079,'ISAS' from dual union all
select 6608,'GMS' from dual union all
select 66080,'ARR' from dual union all
select 66081,'NASA' from dual union all
select 66082,'AE' from dual union all
select 66083,'VKSR' from dual union all
select 66084,'NASA' from dual union all
select 66085,'ISAS' from dual union all
select 66086,'VKSR' from dual union all
select 66087,'LMA' from dual union all
select 66088,'USAF' from dual union all
select 66089,'AE' from dual union all
select 6609,'GMS' from dual union all
select 66090,'VKSR' from dual union all
select 66091,'LMA' from dual union all
select 66092,'AFSPC' from dual union all
select 66093,'NASA' from dual union all
select 66094,'VKSR' from dual union all
select 66095,'VKSR' from dual union all
select 66096,'AE' from dual union all
select 66097,'VKSR' from dual union all
select 66098,'OSC' from dual union all
select 66099,'LMA' from dual union all
select 661,'APL' from dual union all
select 661,'US' from dual union all
select 6610,'GMS' from dual union all
select 66100,'MDSSC' from dual union all
select 66101,'CALT' from dual union all
select 66102,'VKSR' from dual union all
select 66103,'NASA' from dual union all
select 66104,'VKSR' from dual union all
select 66105,'MDSSC' from dual union all
select 66106,'ILSK' from dual union all
select 66107,'AE' from dual union all
select 66108,'VKSR' from dual union all
select 66109,'CALT' from dual union all
select 6611,'GMS' from dual union all
select 66110,'VKSR' from dual union all
select 66111,'AE' from dual union all
select 66112,'NASA' from dual union all
select 66113,'VKSR' from dual union all
select 66114,'MDSSC' from dual union all
select 66115,'MDSSC' from dual union all
select 66116,'LMA' from dual union all
select 66117,'OSC' from dual union all
select 66118,'VKSR' from dual union all
select 66119,'NASA' from dual union all
select 6612,'GMS' from dual union all
select 66120,'AE' from dual union all
select 66121,'VKSR' from dual union all
select 66122,'CALT' from dual union all
select 66123,'BLS' from dual union all
select 66124,'LMA' from dual union all
select 66125,'BLS' from dual union all
select 66126,'ILSK' from dual union all
select 66127,'OSC' from dual union all
select 66128,'CALT' from dual union all
select 66129,'AE' from dual union all
select 6613,'USAF' from dual union all
select 66130,'LMA' from dual union all
select 66131,'VKSR' from dual union all
select 66132,'VKSR' from dual union all
select 66133,'AE' from dual union all
select 66134,'VKSR' from dual union all
select 66135,'NASA' from dual union all
select 66136,'BLS' from dual union all
select 66137,'ISRO' from dual union all
select 66138,'VKSR' from dual union all
select 66139,'LMA' from dual;
insert into LAUNCH_AGENCY
select 6614,'GMS' from dual union all
select 66140,'VKSR' from dual union all
select 66141,'NASA' from dual union all
select 66142,'CALT' from dual union all
select 66143,'OSC' from dual union all
select 66144,'AFSPC' from dual union all
select 66145,'LMA' from dual union all
select 66146,'ESA' from dual union all
select 66147,'BLS' from dual union all
select 66148,'USAF' from dual union all
select 66149,'BLS' from dual union all
select 6615,'USAF' from dual union all
select 66150,'RVSNR' from dual union all
select 66151,'AE' from dual union all
select 66152,'VKSR' from dual union all
select 66153,'NASA' from dual union all
select 66154,'NASDA' from dual union all
select 66155,'AE' from dual union all
select 66156,'ILSK' from dual union all
select 66157,'CALT' from dual union all
select 66158,'LMA' from dual union all
select 66159,'RVSNR' from dual union all
select 6616,'GMS' from dual union all
select 66160,'RVSNR' from dual union all
select 66161,'RVSNR' from dual union all
select 66162,'BLS' from dual union all
select 66163,'AE' from dual union all
select 66164,'OSC' from dual union all
select 66165,'RVSNR' from dual union all
select 66166,'ILSK' from dual union all
select 66167,'RVSN' from dual union all
select 66167,'VMF' from dual union all
select 66168,'MSFC' from dual union all
select 66169,'USN' from dual union all
select 6617,'NASA' from dual union all
select 6617,'USAF' from dual union all
select 66170,'OSCC' from dual union all
select 66170,'SSDC' from dual union all
select 66171,'NAIC' from dual union all
select 66171,'SVC' from dual union all
select 66172,'MRN' from dual union all
select 66173,'VMF' from dual union all
select 66174,'VMF' from dual union all
select 66175,'VMF' from dual union all
select 66176,'VMF' from dual union all
select 66177,'VMF' from dual union all
select 66178,'VMF' from dual union all
select 66179,'VMF' from dual union all
select 6618,'USAF' from dual union all
select 66180,'VMF' from dual union all
select 66181,'VMF' from dual union all
select 66182,'VMF' from dual union all
select 66183,'VMF' from dual union all
select 66184,'VMF' from dual union all
select 66185,'VMF' from dual union all
select 66186,'VMF' from dual union all
select 66187,'VMF' from dual union all
select 66188,'VMF' from dual union all
select 66189,'VMF' from dual union all
select 6619,'GMS' from dual union all
select 66190,'VMF' from dual union all
select 66191,'VMF' from dual union all
select 66192,'VMF' from dual union all
select 66193,'VMF' from dual union all
select 66194,'MRN' from dual union all
select 66195,'MSFC' from dual union all
select 66196,'MRN' from dual union all
select 66197,'GSFC' from dual union all
select 66198,'MRN' from dual union all
select 66199,'USN' from dual union all
select 662,'ARDC' from dual union all
select 662,'US' from dual union all
select 6620,'MVS' from dual union all
select 66200,'RVSN' from dual union all
select 66200,'VMF' from dual union all
select 66201,'JMA' from dual union all
select 66202,'VMF' from dual union all
select 66203,'VMF' from dual union all
select 66204,'VMF' from dual union all
select 66205,'VMF' from dual union all
select 66206,'VMF' from dual union all
select 66207,'VMF' from dual union all
select 66208,'VMF' from dual union all
select 66209,'VMF' from dual union all
select 6621,'GMS' from dual union all
select 66210,'VMF' from dual union all
select 66211,'VMF' from dual union all
select 66212,'VMF' from dual union all
select 66213,'VMF' from dual union all
select 66214,'VMF' from dual union all
select 66215,'VMF' from dual union all
select 66216,'VMF' from dual union all
select 66217,'VMF' from dual union all
select 66218,'VMF' from dual union all
select 66219,'VMF' from dual union all
select 6622,'NASA' from dual union all
select 66220,'VMF' from dual union all
select 66221,'VMF' from dual union all
select 66222,'MRN' from dual union all
select 66223,'BMDO' from dual;
insert into LAUNCH_AGENCY
select 66224,'MDSSC' from dual union all
select 66225,'VKSR' from dual union all
select 66226,'AEB' from dual union all
select 66227,'IAI' from dual union all
select 66228,'IAI' from dual union all
select 66229,'IAI' from dual union all
select 6623,'NASA' from dual union all
select 6623,'USAF' from dual union all
select 66230,'COLE' from dual union all
select 66230,'USA' from dual union all
select 66231,'NDRE' from dual union all
select 66232,'IQMIC' from dual union all
select 66233,'HAL5' from dual union all
select 66234,'RVSN' from dual union all
select 66235,'RVSNR' from dual union all
select 66236,'RVSNR' from dual union all
select 66237,'VMFR' from dual union all
select 66238,'AFMC' from dual union all
select 66239,'ISAS' from dual union all
select 6624,'SIMED' from dual union all
select 66240,'AFSPC' from dual union all
select 66241,'APL' from dual union all
select 66241,'IDG' from dual union all
select 66242,'APL' from dual union all
select 66242,'IDG' from dual union all
select 66243,'USA' from dual union all
select 66244,'NASA' from dual union all
select 66245,'SAND' from dual union all
select 66246,'NASA' from dual union all
select 66247,'OSC' from dual union all
select 66248,'USN' from dual union all
select 66249,'USN' from dual union all
select 6625,'USAF' from dual union all
select 66250,'USN' from dual union all
select 66251,'USN' from dual union all
select 66252,'OSC' from dual union all
select 66253,'IDRDL' from dual union all
select 66254,'OSC' from dual union all
select 66255,'GSFC' from dual union all
select 66256,'OSC' from dual union all
select 66257,'COLE' from dual union all
select 66257,'USA' from dual union all
select 66258,'USA' from dual union all
select 66259,'GSFC' from dual union all
select 6626,'SU' from dual union all
select 66260,'USN' from dual union all
select 66261,'USN' from dual union all
select 66262,'USA' from dual union all
select 66263,'NASA' from dual union all
select 66264,'NASA' from dual union all
select 66265,'NASA' from dual union all
select 66266,'NASA' from dual union all
select 66267,'RVSNR' from dual union all
select 66268,'DLR' from dual union all
select 66268,'INPE' from dual union all
select 66269,'ISRO' from dual union all
select 6627,'RAE' from dual union all
select 66270,'GSFC' from dual union all
select 66271,'GSFC' from dual union all
select 66272,'AFSPC' from dual union all
select 66273,'GSFC' from dual union all
select 66274,'AFSPC' from dual union all
select 66274,'ALCS' from dual union all
select 66275,'NASA' from dual union all
select 66275,'SMDC' from dual union all
select 66276,'ISRO' from dual union all
select 66277,'GSFC' from dual union all
select 66278,'USN' from dual union all
select 66279,'SSC' from dual union all
select 6628,'CASDN' from dual union all
select 66280,'RVSNR' from dual union all
select 66281,'AFSPC' from dual union all
select 66282,'AFMC' from dual union all
select 66283,'SSDC' from dual union all
select 66284,'RVSNR' from dual union all
select 66285,'KARI' from dual union all
select 66286,'VMFR' from dual union all
select 66287,'NASA' from dual union all
select 66288,'GSFC' from dual union all
select 66289,'GSFC' from dual union all
select 6629,'USN' from dual union all
select 66290,'GSFC' from dual union all
select 66291,'GSFC' from dual union all
select 66292,'GSFC' from dual union all
select 66293,'GSFC' from dual union all
select 66294,'GSFC' from dual union all
select 66295,'90SW' from dual union all
select 66296,'GSFC' from dual union all
select 66297,'OSCC' from dual union all
select 66297,'SSDC' from dual union all
select 66298,'NASDA' from dual union all
select 66299,'GSFC' from dual union all
select 663,'NRL' from dual union all
select 663,'US' from dual union all
select 6630,'USA' from dual union all
select 66300,'RVSNR' from dual union all
select 66301,'SAND' from dual union all
select 66302,'RN' from dual union all
select 66303,'RN' from dual union all
select 66304,'DLR' from dual;
insert into LAUNCH_AGENCY
select 66304,'INPE' from dual union all
select 66305,'GSFC' from dual union all
select 66306,'CALT' from dual union all
select 66307,'GSFC' from dual union all
select 66308,'90SW' from dual union all
select 66309,'GSFC' from dual union all
select 6631,'USA' from dual union all
select 66310,'GSFC' from dual union all
select 66311,'GSFC' from dual union all
select 66312,'GSFC' from dual union all
select 66313,'USN' from dual union all
select 66314,'PLA2AC' from dual union all
select 66315,'VMFR' from dual union all
select 66316,'JMA' from dual union all
select 66317,'MRN' from dual union all
select 66318,'ISAS' from dual union all
select 66319,'JMA' from dual union all
select 6632,'USA' from dual union all
select 66320,'JMA' from dual union all
select 66321,'JMA' from dual union all
select 66322,'JMA' from dual union all
select 66323,'JMA' from dual union all
select 66324,'MRN' from dual union all
select 66325,'JMA' from dual union all
select 66326,'JMA' from dual union all
select 66327,'JMA' from dual union all
select 66328,'MRN' from dual union all
select 66329,'MRN' from dual union all
select 6633,'USA' from dual union all
select 66330,'JMA' from dual union all
select 66331,'MRN' from dual union all
select 66332,'JMA' from dual union all
select 66333,'MRN' from dual union all
select 66334,'JMA' from dual union all
select 66335,'MRN' from dual union all
select 66336,'JMA' from dual union all
select 66337,'JMA' from dual union all
select 66338,'MRN' from dual union all
select 66339,'JMA' from dual union all
select 6634,'USAF' from dual union all
select 66340,'JMA' from dual union all
select 66341,'MRN' from dual union all
select 66342,'JMA' from dual union all
select 66343,'MRN' from dual union all
select 66344,'JMA' from dual union all
select 66345,'MRN' from dual union all
select 66346,'DLR' from dual union all
select 66347,'MRN' from dual union all
select 66348,'JMA' from dual union all
select 66349,'JMA' from dual union all
select 6635,'USAF' from dual union all
select 66350,'MRN' from dual union all
select 66351,'NASA' from dual union all
select 66352,'DLR' from dual union all
select 66353,'MRN' from dual union all
select 66354,'NASA' from dual union all
select 66355,'NASA' from dual union all
select 66356,'NASA' from dual union all
select 66357,'JMA' from dual union all
select 66358,'MRN' from dual union all
select 66359,'MRN' from dual union all
select 6636,'SU' from dual union all
select 66360,'MRN' from dual union all
select 66361,'DLR' from dual union all
select 66362,'NASA' from dual union all
select 66363,'NASA' from dual union all
select 66364,'NASA' from dual union all
select 66365,'NASA' from dual union all
select 66366,'MRN' from dual union all
select 66367,'DLR' from dual union all
select 66368,'DLR' from dual union all
select 66369,'NASA' from dual union all
select 6637,'USAF' from dual union all
select 66370,'NASA' from dual union all
select 66371,'NASA' from dual union all
select 66372,'JMA' from dual union all
select 66373,'DLR' from dual union all
select 66374,'DLR' from dual union all
select 66375,'NASA' from dual union all
select 66376,'NASA' from dual union all
select 66377,'NASA' from dual union all
select 66378,'NASA' from dual union all
select 66379,'MRN' from dual union all
select 6638,'USAF' from dual union all
select 66380,'JMA' from dual union all
select 66381,'MRN' from dual union all
select 66382,'DLR' from dual union all
select 66383,'ISAS' from dual union all
select 66384,'ISAS' from dual union all
select 66385,'JMA' from dual union all
select 66386,'MRN' from dual union all
select 66387,'JMA' from dual union all
select 66388,'JMA' from dual union all
select 66389,'JMA' from dual union all
select 6639,'USAF' from dual union all
select 66390,'MRN' from dual union all
select 66391,'JMA' from dual union all
select 66392,'DLR' from dual union all
select 66393,'DLR' from dual union all
select 66394,'JMA' from dual;
insert into LAUNCH_AGENCY
select 66395,'JMA' from dual union all
select 66396,'JMA' from dual union all
select 66397,'JMA' from dual union all
select 66398,'JMA' from dual union all
select 66399,'JMA' from dual union all
select 664,'APL' from dual union all
select 664,'US' from dual union all
select 6640,'GMS' from dual union all
select 66400,'ISAS' from dual union all
select 66401,'ISAS' from dual union all
select 66402,'MRN' from dual union all
select 66403,'MRN' from dual union all
select 66404,'MRN' from dual union all
select 66405,'MRN' from dual union all
select 66406,'MRN' from dual union all
select 66407,'NASA' from dual union all
select 66408,'NASA' from dual union all
select 66409,'NASA' from dual union all
select 6641,'GMS' from dual union all
select 66410,'MRN' from dual union all
select 66411,'NASA' from dual union all
select 66412,'NASA' from dual union all
select 66413,'MRN' from dual union all
select 66414,'NASA' from dual union all
select 66415,'NASA' from dual union all
select 66416,'NASA' from dual union all
select 66417,'NASA' from dual union all
select 66418,'NASA' from dual union all
select 66419,'NASA' from dual union all
select 6642,'NASA' from dual union all
select 66420,'NASA' from dual union all
select 66421,'NASA' from dual union all
select 66422,'NASA' from dual union all
select 66423,'NASA' from dual union all
select 66424,'NASA' from dual union all
select 66425,'NASA' from dual union all
select 66426,'NASA' from dual union all
select 66427,'NASA' from dual union all
select 66428,'NASA' from dual union all
select 66429,'NASA' from dual union all
select 6643,'NASA' from dual union all
select 6643,'USAF' from dual union all
select 66430,'MRN' from dual union all
select 66431,'MRN' from dual union all
select 66432,'MRN' from dual union all
select 66433,'MRN' from dual union all
select 66434,'MRN' from dual union all
select 66435,'NDRE' from dual union all
select 66436,'ARR' from dual union all
select 66437,'LMA' from dual union all
select 66438,'BLS' from dual union all
select 66439,'NASA' from dual union all
select 6644,'NASA' from dual union all
select 6644,'USAF' from dual union all
select 66440,'RVSNR' from dual union all
select 66441,'LMA' from dual union all
select 66442,'AE' from dual union all
select 66443,'OSC' from dual union all
select 66444,'BLS' from dual union all
select 66445,'RVSNR' from dual union all
select 66446,'BLS' from dual union all
select 66447,'NASDA' from dual union all
select 66448,'OSC' from dual union all
select 66449,'AE' from dual union all
select 6645,'GMS' from dual union all
select 66450,'LMA' from dual union all
select 66451,'RVSNR' from dual union all
select 66452,'LMA' from dual union all
select 66453,'AE' from dual union all
select 66454,'CALT' from dual union all
select 66455,'BLS' from dual union all
select 66456,'OSC' from dual union all
select 66457,'RVSNR' from dual union all
select 66458,'NASA' from dual union all
select 66459,'BLS' from dual union all
select 6646,'AFCRL' from dual union all
select 66460,'AE' from dual union all
select 66461,'RVSNR' from dual union all
select 66462,'CALT' from dual union all
select 66463,'RVSNR' from dual union all
select 66464,'ILSK' from dual union all
select 66465,'USAF' from dual union all
select 66466,'4SLS' from dual union all
select 66467,'RVSNR' from dual union all
select 66468,'2SLS' from dual union all
select 66469,'CALT' from dual union all
select 6647,'SU' from dual union all
select 66470,'NASA' from dual union all
select 66471,'2SLS' from dual union all
select 66472,'RVSNR' from dual union all
select 66473,'LMA' from dual union all
select 66474,'RVSNR' from dual union all
select 66475,'RVSNR' from dual union all
select 66476,'RVSNR' from dual union all
select 66477,'ISAS' from dual union all
select 66478,'VMFR' from dual union all
select 66479,'RVSNR' from dual union all
select 6648,'NASA' from dual union all
select 66480,'CALT' from dual union all
select 66481,'RVSNR' from dual;
insert into LAUNCH_AGENCY
select 66482,'OSC' from dual union all
select 66483,'RVSNR' from dual union all
select 66484,'CALT' from dual union all
select 66485,'AE' from dual union all
select 66486,'ILSK' from dual union all
select 66487,'2SLS' from dual union all
select 66488,'AE' from dual union all
select 66489,'OSC' from dual union all
select 6649,'GMS' from dual union all
select 66490,'RVSNR' from dual union all
select 66491,'OSC' from dual union all
select 66492,'AE' from dual union all
select 66493,'LMA' from dual union all
select 66494,'LMA' from dual union all
select 66495,'AE' from dual union all
select 66496,'OSC' from dual union all
select 66497,'2SLS' from dual union all
select 66498,'RVSNR' from dual union all
select 66499,'AE' from dual union all
select 665,'USA' from dual union all
select 6650,'NASA' from dual union all
select 6650,'USAF' from dual union all
select 66500,'NASA' from dual union all
select 66501,'ILSK' from dual union all
select 66502,'2SLS' from dual union all
select 66503,'RVSNR' from dual union all
select 66504,'2SLS' from dual union all
select 66505,'NASA' from dual union all
select 66506,'AE' from dual union all
select 66507,'OSC' from dual union all
select 66508,'RVSNR' from dual union all
select 66509,'2SLS' from dual union all
select 6651,'GMS' from dual union all
select 66510,'CALT' from dual union all
select 66511,'AE' from dual union all
select 66512,'RVSNR' from dual union all
select 66513,'RVSNR' from dual union all
select 66514,'USA' from dual union all
select 66515,'IR' from dual union all
select 66516,'RVSNR' from dual union all
select 66517,'ISAS' from dual union all
select 66518,'ISAS' from dual union all
select 66519,'IQMIC' from dual union all
select 6652,'NASA' from dual union all
select 6652,'USAF' from dual union all
select 66520,'IQMIC' from dual union all
select 66521,'SSC' from dual union all
select 66522,'ISA' from dual union all
select 66523,'4SLS' from dual union all
select 66524,'2SLS' from dual union all
select 66525,'KCST' from dual union all
select 66526,'RVSNR' from dual union all
select 66527,'COLE' from dual union all
select 66527,'USA' from dual union all
select 66528,'IAI' from dual union all
select 66529,'COLE' from dual union all
select 66529,'USA' from dual union all
select 6653,'GMS' from dual union all
select 66530,'IQMIC' from dual union all
select 66531,'IQMIC' from dual union all
select 66532,'IQMIC' from dual union all
select 66533,'RVSNR' from dual union all
select 66534,'AFMC' from dual union all
select 66535,'SSDC' from dual union all
select 66536,'ISAS' from dual union all
select 66537,'DLR' from dual union all
select 66537,'SSC' from dual union all
select 66538,'ISAS' from dual union all
select 66539,'DLR' from dual union all
select 66539,'INPE' from dual union all
select 6654,'USA' from dual union all
select 66540,'ISAS' from dual union all
select 66541,'SSC' from dual union all
select 66542,'USN' from dual union all
select 66543,'USN' from dual union all
select 66544,'USN' from dual union all
select 66545,'USN' from dual union all
select 66546,'DLR' from dual union all
select 66547,'VMFR' from dual union all
select 66548,'VMFR' from dual union all
select 66549,'NASA' from dual union all
select 6655,'USA' from dual union all
select 66550,'NASA' from dual union all
select 66551,'AFSPC' from dual union all
select 66552,'NASA' from dual union all
select 66553,'NASA' from dual union all
select 66554,'GSFC' from dual union all
select 66555,'DLR' from dual union all
select 66555,'SSC' from dual union all
select 66556,'DLR' from dual union all
select 66556,'SSC' from dual union all
select 66557,'USN' from dual union all
select 66558,'USN' from dual union all
select 66559,'NASA' from dual union all
select 6656,'USA' from dual union all
select 66560,'USN' from dual union all
select 66561,'USN' from dual union all
select 66562,'GSFC' from dual union all
select 66563,'INPE' from dual union all
select 66564,'NASA' from dual;
insert into LAUNCH_AGENCY
select 66565,'KRLP' from dual union all
select 66566,'RVSNR' from dual union all
select 66567,'BMDO' from dual union all
select 66567,'SAND' from dual union all
select 66568,'GSFC' from dual union all
select 66569,'GSFC' from dual union all
select 6657,'USA' from dual union all
select 66570,'DLR' from dual union all
select 66570,'ISRO' from dual union all
select 66571,'CSA' from dual union all
select 66572,'NASA' from dual union all
select 66573,'90SW' from dual union all
select 66574,'COLE' from dual union all
select 66574,'USA' from dual union all
select 66575,'GSFC' from dual union all
select 66576,'AFSPC' from dual union all
select 66577,'KARI' from dual union all
select 66578,'GSFC' from dual union all
select 66579,'AFSPC' from dual union all
select 6658,'USA' from dual union all
select 66580,'AFSPC' from dual union all
select 66581,'GSFC' from dual union all
select 66582,'VMFR' from dual union all
select 66583,'RVSNR' from dual union all
select 66584,'AFSPC' from dual union all
select 66585,'ARC' from dual union all
select 66586,'DLR' from dual union all
select 66586,'ISRO' from dual union all
select 66587,'OSCC' from dual union all
select 66587,'USA' from dual union all
select 66588,'DLR' from dual union all
select 66588,'ISRO' from dual union all
select 66589,'DLR' from dual union all
select 6659,'USA' from dual union all
select 66590,'RVSNR' from dual union all
select 66591,'GSFC' from dual union all
select 66592,'OSCC' from dual union all
select 66592,'USAF' from dual union all
select 66593,'GSFC' from dual union all
select 66594,'NASDA' from dual union all
select 66595,'SMDC' from dual union all
select 66596,'ESA' from dual union all
select 66596,'SSC' from dual union all
select 66597,'DLR' from dual union all
select 66597,'SSC' from dual union all
select 66598,'RVSNR' from dual union all
select 66599,'RVSN' from dual union all
select 666,'USA' from dual union all
select 6660,'USA' from dual union all
select 66600,'NSPO' from dual union all
select 66601,'USN' from dual union all
select 66602,'USN' from dual union all
select 66603,'JMA' from dual union all
select 66604,'JMA' from dual union all
select 66605,'JMA' from dual union all
select 66606,'JMA' from dual union all
select 66607,'MRN' from dual union all
select 66608,'JMA' from dual union all
select 66609,'JMA' from dual union all
select 6661,'USA' from dual union all
select 66610,'MRN' from dual union all
select 66611,'JMA' from dual union all
select 66612,'JMA' from dual union all
select 66613,'JMA' from dual union all
select 66614,'JMA' from dual union all
select 66615,'MRN' from dual union all
select 66616,'JMA' from dual union all
select 66617,'JMA' from dual union all
select 66618,'USA' from dual union all
select 66619,'JMA' from dual union all
select 6662,'USAF' from dual union all
select 66620,'JMA' from dual union all
select 66621,'MRN' from dual union all
select 66622,'JMA' from dual union all
select 66623,'JMA' from dual union all
select 66624,'JMA' from dual union all
select 66625,'JMA' from dual union all
select 66626,'JMA' from dual union all
select 66627,'JMA' from dual union all
select 66628,'JMA' from dual union all
select 66629,'JMA' from dual union all
select 6663,'GMS' from dual union all
select 66630,'ISAS' from dual union all
select 66631,'ISAS' from dual union all
select 66632,'JMA' from dual union all
select 66633,'JMA' from dual union all
select 66634,'JMA' from dual union all
select 66635,'JMA' from dual union all
select 66636,'JMA' from dual union all
select 66637,'MRN' from dual union all
select 66638,'JMA' from dual union all
select 66639,'JMA' from dual union all
select 6664,'NASA' from dual union all
select 6664,'USAF' from dual union all
select 66640,'JMA' from dual union all
select 66641,'JMA' from dual union all
select 66642,'JMA' from dual union all
select 66643,'JMA' from dual union all
select 66644,'DLR' from dual union all
select 66645,'DLR' from dual;
insert into LAUNCH_AGENCY
select 66646,'DLR' from dual union all
select 66647,'DLR' from dual union all
select 66648,'DLR' from dual union all
select 66649,'DLR' from dual union all
select 6665,'USAF' from dual union all
select 66650,'DLR' from dual union all
select 66651,'DLR' from dual union all
select 66652,'DLR' from dual union all
select 66653,'MRN' from dual union all
select 66654,'DLR' from dual union all
select 66655,'DLR' from dual union all
select 66656,'DLR' from dual union all
select 66657,'DLR' from dual union all
select 66658,'DLR' from dual union all
select 66659,'DLR' from dual union all
select 6666,'USAF' from dual union all
select 66660,'DLR' from dual union all
select 66661,'DLR' from dual union all
select 66662,'DLR' from dual union all
select 66663,'DLR' from dual union all
select 66664,'DLR' from dual union all
select 66665,'DLR' from dual union all
select 66666,'DLR' from dual union all
select 66667,'DLR' from dual union all
select 66668,'DLR' from dual union all
select 66669,'DLR' from dual union all
select 6667,'RRF' from dual union all
select 66670,'DLR' from dual union all
select 66671,'MRN' from dual union all
select 66672,'MRN' from dual union all
select 66673,'MRN' from dual union all
select 66674,'MRN' from dual union all
select 66675,'MRN' from dual union all
select 66676,'MRN' from dual union all
select 66677,'MRN' from dual union all
select 66678,'KSC' from dual union all
select 66679,'KSC' from dual union all
select 6668,'USAF' from dual union all
select 66680,'SSC' from dual union all
select 66681,'SSC' from dual union all
select 66682,'2SLS' from dual union all
select 66683,'LMA' from dual union all
select 66684,'2SLS' from dual union all
select 66685,'STSM' from dual union all
select 66686,'ILSK' from dual union all
select 66687,'LMA' from dual union all
select 66688,'RVSNR' from dual union all
select 66689,'2SLS' from dual union all
select 6669,'USAF' from dual union all
select 66690,'AE' from dual union all
select 66691,'RVSNR' from dual union all
select 66692,'OSC' from dual union all
select 66693,'STSM' from dual union all
select 66694,'ILSK' from dual union all
select 66695,'SEALC' from dual union all
select 66696,'RVSNR' from dual union all
select 66697,'AE' from dual union all
select 66698,'3SLS' from dual union all
select 66699,'LMA' from dual union all
select 667,'USA' from dual union all
select 6670,'USAF' from dual union all
select 66700,'STSM' from dual union all
select 66701,'2SLS' from dual union all
select 66702,'KTRAS' from dual union all
select 66702,'RVSNR' from dual union all
select 66703,'RVSNR' from dual union all
select 66704,'3SLS' from dual union all
select 66705,'2SLS' from dual union all
select 66706,'SAST' from dual union all
select 66707,'OSC' from dual union all
select 66708,'ILSK' from dual union all
select 66709,'USAF' from dual union all
select 6671,'NASA' from dual union all
select 6671,'USAF' from dual union all
select 66710,'ISRO' from dual union all
select 66711,'NASA' from dual union all
select 66712,'2SLS' from dual union all
select 66713,'CALT' from dual union all
select 66714,'ILSK' from dual union all
select 66715,'USAF' from dual union all
select 66716,'2SLS' from dual union all
select 66717,'RVSNR' from dual union all
select 66718,'2SLS' from dual union all
select 66719,'RVSNR' from dual union all
select 6672,'USN' from dual union all
select 66720,'RVSNR' from dual union all
select 66721,'NASA' from dual union all
select 66722,'2SLS' from dual union all
select 66723,'AE' from dual union all
select 66724,'2SLS' from dual union all
select 66725,'RVSNR' from dual union all
select 66726,'RVSNR' from dual union all
select 66727,'AE' from dual union all
select 66728,'KHRU' from dual union all
select 66729,'RVSNR' from dual union all
select 6673,'USAF' from dual union all
select 66730,'STSM' from dual union all
select 66731,'LMA' from dual union all
select 66732,'LMA' from dual union all
select 66733,'AE' from dual;
insert into LAUNCH_AGENCY
select 66734,'ILSK' from dual union all
select 66735,'RVSNR' from dual union all
select 66736,'2SLS' from dual union all
select 66737,'SEALC' from dual union all
select 66738,'SAST' from dual union all
select 66739,'STSM' from dual union all
select 6674,'RVSN' from dual union all
select 66740,'AE' from dual union all
select 66741,'AE' from dual union all
select 66742,'CALT' from dual union all
select 66743,'STSM' from dual union all
select 66744,'LMA' from dual union all
select 66745,'AE' from dual union all
select 66746,'OSC' from dual union all
select 66747,'AE' from dual union all
select 66748,'USAF' from dual union all
select 66749,'LMA' from dual union all
select 6675,'USAF' from dual union all
select 66750,'NASA' from dual union all
select 66751,'OSC' from dual union all
select 66752,'AE' from dual union all
select 66753,'RVSNR' from dual union all
select 66754,'RVSNR' from dual union all
select 66755,'PK' from dual union all
select 66756,'USN' from dual union all
select 66757,'NASA' from dual union all
select 66758,'VMFR' from dual union all
select 66759,'PRO' from dual union all
select 6676,'USAF' from dual union all
select 66760,'PVO' from dual union all
select 66761,'LMA' from dual union all
select 66762,'RVSNR' from dual union all
select 66763,'KHRU' from dual union all
select 66764,'NASDA' from dual union all
select 66765,'AEB' from dual union all
select 66766,'USN' from dual union all
select 66767,'IAI' from dual union all
select 66768,'IAI' from dual union all
select 66769,'IQMIC' from dual union all
select 6677,'USAF' from dual union all
select 66770,'IQMIC' from dual union all
select 66771,'IQMIC' from dual union all
select 66772,'IQMIC' from dual union all
select 66773,'IQMIC' from dual union all
select 66774,'NASA' from dual union all
select 66775,'NASA' from dual union all
select 66776,'NASA' from dual union all
select 66777,'NASA' from dual union all
select 66778,'ISAS' from dual union all
select 66779,'USN' from dual union all
select 6678,'NASA' from dual union all
select 66780,'USN' from dual union all
select 66781,'AFSPC' from dual union all
select 66782,'GSFC' from dual union all
select 66783,'GSFC' from dual union all
select 66784,'90SW' from dual union all
select 66785,'AEB' from dual union all
select 66786,'COLE' from dual union all
select 66786,'USA' from dual union all
select 66787,'USN' from dual union all
select 66788,'USN' from dual union all
select 66789,'USN' from dual union all
select 6679,'RVSN' from dual union all
select 66790,'USN' from dual union all
select 66791,'GSFC' from dual union all
select 66792,'COLE' from dual union all
select 66792,'USA' from dual union all
select 66793,'USA' from dual union all
select 66794,'COLE' from dual union all
select 66794,'USAF' from dual union all
select 66795,'VMFR' from dual union all
select 66796,'IDRDL' from dual union all
select 66797,'GSFC' from dual union all
select 66798,'KRLP' from dual union all
select 66799,'PK' from dual union all
select 668,'PARD' from dual union all
select 6680,'NASA' from dual union all
select 66800,'USN' from dual union all
select 66801,'USN' from dual union all
select 66802,'DMA' from dual union all
select 66803,'DMA' from dual union all
select 66804,'GSFC' from dual union all
select 66805,'SSC' from dual union all
select 66806,'COLE' from dual union all
select 66806,'USA' from dual union all
select 66807,'RVSNR' from dual union all
select 66808,'COLE' from dual union all
select 66808,'USA' from dual union all
select 66809,'USA' from dual union all
select 6681,'USAF' from dual union all
select 66810,'GSFC' from dual union all
select 66811,'GSFC' from dual union all
select 66812,'USA' from dual union all
select 66813,'USA' from dual union all
select 66814,'USA' from dual union all
select 66815,'USN' from dual union all
select 66816,'NASA' from dual union all
select 66817,'NASA' from dual union all
select 66818,'NASA' from dual union all
select 66819,'NDRE' from dual;
insert into LAUNCH_AGENCY
select 6682,'RVSN' from dual union all
select 66820,'NASA' from dual union all
select 66821,'NASA' from dual union all
select 66822,'NASA' from dual union all
select 66823,'PLA2AC' from dual union all
select 66824,'COLE' from dual union all
select 66824,'USA' from dual union all
select 66825,'USA' from dual union all
select 66826,'GSFC' from dual union all
select 66827,'AFSPC' from dual union all
select 66828,'AFSPC' from dual union all
select 66829,'GSFC' from dual union all
select 6683,'USAF' from dual union all
select 66830,'RVSNR' from dual union all
select 66831,'RVSNR' from dual union all
select 66832,'OSCC' from dual union all
select 66833,'AFSMC' from dual union all
select 66834,'COLE' from dual union all
select 66834,'SMDC' from dual union all
select 66835,'USN' from dual union all
select 66836,'GSFC' from dual union all
select 66837,'VMFR' from dual union all
select 66838,'RVSNR' from dual union all
select 66839,'VMFR' from dual union all
select 6684,'USAF' from dual union all
select 66840,'AFMC' from dual union all
select 66841,'SSDC' from dual union all
select 66842,'RVSNR' from dual union all
select 66843,'PRO' from dual union all
select 66844,'ISRO' from dual union all
select 66845,'USN' from dual union all
select 66846,'USN' from dual union all
select 66847,'AFSPC' from dual union all
select 66848,'RVSN' from dual union all
select 66849,'RVSN' from dual union all
select 6685,'RVSN' from dual union all
select 66850,'ISRO' from dual union all
select 66851,'ISRO' from dual union all
select 66852,'RVSNR' from dual union all
select 66853,'GSFC' from dual union all
select 66854,'IR' from dual union all
select 66855,'IR' from dual union all
select 66856,'IR' from dual union all
select 66857,'IR' from dual union all
select 66858,'IR' from dual union all
select 66859,'IR' from dual union all
select 6686,'USAF' from dual union all
select 66860,'MRN' from dual union all
select 66861,'MRN' from dual union all
select 66862,'MRN' from dual union all
select 66863,'NDRE' from dual union all
select 66864,'NDRE' from dual union all
select 66865,'NDRE' from dual union all
select 66866,'NDRE' from dual union all
select 66867,'NDRE' from dual union all
select 66868,'NDRE' from dual union all
select 66869,'NDRE' from dual union all
select 6687,'RVSN' from dual union all
select 66870,'NDRE' from dual union all
select 66871,'NDRE' from dual union all
select 66872,'NDRE' from dual union all
select 66873,'NDRE' from dual union all
select 66874,'NDRE' from dual union all
select 66875,'NDRE' from dual union all
select 66876,'NDRE' from dual union all
select 66877,'NDRE' from dual union all
select 66878,'NDRE' from dual union all
select 66879,'NDRE' from dual union all
select 6688,'USAF' from dual union all
select 66880,'DLR' from dual union all
select 66881,'DLR' from dual union all
select 66882,'DLR' from dual union all
select 66883,'DLR' from dual union all
select 66884,'DLR' from dual union all
select 66885,'DLR' from dual union all
select 66886,'DLR' from dual union all
select 66887,'DLR' from dual union all
select 66888,'DLR' from dual union all
select 66889,'DLR' from dual union all
select 6689,'USAF' from dual union all
select 66890,'ISAS' from dual union all
select 66891,'DLR' from dual union all
select 66892,'ISAS' from dual union all
select 66893,'DLR' from dual union all
select 66894,'DLR' from dual union all
select 66895,'DLR' from dual union all
select 66896,'DLR' from dual union all
select 66897,'DLR' from dual union all
select 66898,'DLR' from dual union all
select 66899,'MRN' from dual union all
select 669,'ORD' from dual union all
select 6690,'NASA' from dual union all
select 66900,'MRN' from dual union all
select 66901,'SPFL' from dual union all
select 66902,'MRN' from dual union all
select 66903,'LMA' from dual union all
select 66904,'AE' from dual union all
select 66905,'CALT' from dual union all
select 66906,'AFMC' from dual union all
select 66907,'RAKA' from dual;
insert into LAUNCH_AGENCY
select 66908,'RVSNR' from dual union all
select 66909,'LMA' from dual union all
select 6691,'USAF' from dual union all
select 66910,'2SLS' from dual union all
select 66911,'STSM' from dual union all
select 66912,'NASA' from dual union all
select 66913,'ILSK' from dual union all
select 66914,'AE' from dual union all
select 66915,'KHRU' from dual union all
select 66916,'OSC' from dual union all
select 66917,'STSM' from dual union all
select 66918,'AE' from dual union all
select 66919,'BLS' from dual union all
select 6692,'RVSN' from dual union all
select 66920,'RAKA' from dual union all
select 66921,'KHRU' from dual union all
select 66922,'AE' from dual union all
select 66923,'RAKA' from dual union all
select 66924,'ILSL' from dual union all
select 66925,'RVSNR' from dual union all
select 66926,'USAF' from dual union all
select 66927,'BLS' from dual union all
select 66928,'RVSNR' from dual union all
select 66929,'NASA' from dual union all
select 6693,'USA' from dual union all
select 66930,'ILSL' from dual union all
select 66931,'KHRU' from dual union all
select 66932,'OSC' from dual union all
select 66933,'KHRU' from dual union all
select 66934,'CALT' from dual union all
select 66935,'RVSNR' from dual union all
select 66936,'ILSL' from dual union all
select 66937,'ILSK' from dual union all
select 66938,'KHRU' from dual union all
select 66939,'KHRU' from dual union all
select 6694,'AEC' from dual union all
select 66940,'ILSL' from dual union all
select 66941,'RVSNR' from dual union all
select 66942,'BLS' from dual union all
select 66943,'STSM' from dual union all
select 66944,'AFMC' from dual union all
select 66945,'SEALC' from dual union all
select 66946,'RAKA' from dual union all
select 66947,'STSM' from dual union all
select 66948,'AE' from dual union all
select 66949,'USAF' from dual union all
select 6695,'USA' from dual union all
select 66950,'BLS' from dual union all
select 66951,'KHRU' from dual union all
select 66952,'SAST' from dual union all
select 66953,'ILSK' from dual union all
select 66954,'AE' from dual union all
select 66955,'NASA' from dual union all
select 66956,'AE' from dual union all
select 66957,'USAF' from dual union all
select 66958,'RVSNR' from dual union all
select 66959,'KTRAS' from dual union all
select 66959,'RVSNR' from dual union all
select 6696,'USN' from dual union all
select 66960,'RVSNR' from dual union all
select 66961,'ILSK' from dual union all
select 66962,'AE' from dual union all
select 66963,'OSC' from dual union all
select 66964,'NASA' from dual union all
select 66965,'KHRU' from dual union all
select 66966,'RAKA' from dual union all
select 66967,'ILSL' from dual union all
select 66968,'SEALP' from dual union all
select 66969,'ILSK' from dual union all
select 6697,'NASA' from dual union all
select 66970,'AE' from dual union all
select 66971,'CALT' from dual union all
select 66972,'RAKA' from dual union all
select 66973,'BLS' from dual union all
select 66974,'AE' from dual union all
select 66975,'RAKA' from dual union all
select 66976,'PU' from dual union all
select 66977,'BLS' from dual union all
select 66978,'AE' from dual union all
select 66979,'ILSK' from dual union all
select 6698,'AEC' from dual union all
select 66980,'NASA' from dual union all
select 66981,'PU' from dual union all
select 66982,'ILSL' from dual union all
select 66983,'AE' from dual union all
select 66984,'CALT' from dual union all
select 66985,'GSFC' from dual union all
select 66986,'IDRDL' from dual union all
select 66987,'USN' from dual union all
select 66988,'USN' from dual union all
select 66989,'USN' from dual union all
select 6699,'NASA' from dual union all
select 66990,'IDF' from dual union all
select 66991,'CSXT' from dual union all
select 66992,'IQMIC' from dual union all
select 66993,'IQMIC' from dual union all
select 66994,'IR' from dual union all
select 66995,'SEALC' from dual union all
select 66996,'RVSNR' from dual union all
select 66997,'NASA' from dual;
insert into LAUNCH_AGENCY
select 66998,'COLE' from dual union all
select 66998,'SSDC' from dual union all
select 66999,'ISRO' from dual union all
select 67,'WEHR' from dual union all
select 670,'ORD' from dual union all
select 6700,'USA' from dual union all
select 67000,'ISRO' from dual union all
select 67001,'ISRO' from dual union all
select 67002,'ISRO' from dual union all
select 67003,'ISRO' from dual union all
select 67004,'ISRO' from dual union all
select 67005,'ISRO' from dual union all
select 67006,'ISRO' from dual union all
select 67007,'ISRO' from dual union all
select 67008,'ISRO' from dual union all
select 67009,'ISRO' from dual union all
select 6701,'USA' from dual union all
select 67010,'ISRO' from dual union all
select 67011,'ISRO' from dual union all
select 67012,'ISRO' from dual union all
select 67013,'ISRO' from dual union all
select 67014,'ISRO' from dual union all
select 67015,'ISRO' from dual union all
select 67016,'ISRO' from dual union all
select 67017,'ISRO' from dual union all
select 67018,'ISRO' from dual union all
select 67019,'ISRO' from dual union all
select 6702,'USA' from dual union all
select 67020,'ISRO' from dual union all
select 67021,'ISRO' from dual union all
select 67022,'ISRO' from dual union all
select 67023,'ISRO' from dual union all
select 67024,'ISRO' from dual union all
select 67025,'ISRO' from dual union all
select 67026,'ISRO' from dual union all
select 67027,'ISRO' from dual union all
select 67028,'ISRO' from dual union all
select 67029,'ISRO' from dual union all
select 6703,'USA' from dual union all
select 67030,'ISRO' from dual union all
select 67031,'ISRO' from dual union all
select 67032,'ISRO' from dual union all
select 67033,'ISRO' from dual union all
select 67034,'ISRO' from dual union all
select 67035,'ISRO' from dual union all
select 67036,'ISRO' from dual union all
select 67037,'ISRO' from dual union all
select 67038,'ISRO' from dual union all
select 67039,'IAI' from dual union all
select 6704,'USA' from dual union all
select 67040,'IQMIC' from dual union all
select 67041,'IQMIC' from dual union all
select 67042,'IQMIC' from dual union all
select 67043,'IQMIC' from dual union all
select 67044,'IQMIC' from dual union all
select 67045,'IQMIC' from dual union all
select 67046,'IQMIC' from dual union all
select 67047,'IQMIC' from dual union all
select 67048,'IQMIC' from dual union all
select 67049,'PK' from dual union all
select 6705,'NMSU' from dual union all
select 67050,'ISAS' from dual union all
select 67051,'AFMC' from dual union all
select 67052,'SSDC' from dual union all
select 67053,'AEB' from dual union all
select 67054,'RVSNR' from dual union all
select 67055,'GSFC' from dual union all
select 67056,'NASA' from dual union all
select 67057,'USN' from dual union all
select 67058,'USN' from dual union all
select 67059,'USN' from dual union all
select 6706,'USN' from dual union all
select 67060,'USN' from dual union all
select 67061,'CSA' from dual union all
select 67062,'576FTS' from dual union all
select 67063,'VMFR' from dual union all
select 67064,'VMFR' from dual union all
select 67065,'SSC' from dual union all
select 67066,'SSC' from dual union all
select 67067,'IDF' from dual union all
select 67068,'VMFR' from dual union all
select 67069,'USN' from dual union all
select 6707,'AEC' from dual union all
select 67070,'USN' from dual union all
select 67071,'USA' from dual union all
select 67072,'USA' from dual union all
select 67073,'USA' from dual union all
select 67074,'USA' from dual union all
select 67075,'USA' from dual union all
select 67076,'GSFC' from dual union all
select 67077,'NDRE' from dual union all
select 67078,'NDRE' from dual union all
select 67079,'GSFC' from dual union all
select 6708,'USA' from dual union all
select 67080,'576FTS' from dual union all
select 67081,'AFMC' from dual union all
select 67082,'AFSPC' from dual union all
select 67083,'GSFC' from dual union all
select 67084,'IDRDL' from dual union all
select 67085,'NASA' from dual;
insert into LAUNCH_AGENCY
select 67086,'GSFC' from dual union all
select 67087,'USN' from dual union all
select 67088,'GSFC' from dual union all
select 67089,'GSFC' from dual union all
select 6709,'NASA' from dual union all
select 67090,'AFMC' from dual union all
select 67091,'SSDC' from dual union all
select 67092,'IR' from dual union all
select 67093,'GSFC' from dual union all
select 67094,'GSFC' from dual union all
select 67095,'GSFC' from dual union all
select 67096,'AEB' from dual union all
select 67096,'DLR' from dual union all
select 67097,'SYMC' from dual union all
select 67098,'GSFC' from dual union all
select 67099,'VMFR' from dual union all
select 671,'PARD' from dual union all
select 6710,'AEC' from dual union all
select 67100,'RN' from dual union all
select 67101,'SYMC' from dual union all
select 67102,'RVSNR' from dual union all
select 67103,'RVSNR' from dual union all
select 67104,'AFSPC' from dual union all
select 67105,'AFSPC' from dual union all
select 67106,'RVSNR' from dual union all
select 67107,'GSFC' from dual union all
select 67108,'OSCC' from dual union all
select 67108,'SSDC' from dual union all
select 67109,'NASA' from dual union all
select 6711,'AEC' from dual union all
select 67110,'RVSNR' from dual union all
select 67111,'PLA2AC' from dual union all
select 67112,'ISAS' from dual union all
select 67113,'AEB' from dual union all
select 67114,'PLA2AC' from dual union all
select 67115,'NASA' from dual union all
select 67115,'SMDC' from dual union all
select 67116,'VMFR' from dual union all
select 67117,'ISAS' from dual union all
select 67118,'JMA' from dual union all
select 67119,'JMA' from dual union all
select 6712,'AEC' from dual union all
select 67120,'SPFL' from dual union all
select 67121,'JMA' from dual union all
select 67122,'JMA' from dual union all
select 67123,'JMA' from dual union all
select 67124,'JMA' from dual union all
select 67125,'JMA' from dual union all
select 67126,'JMA' from dual union all
select 67127,'NDRE' from dual union all
select 67128,'NDRE' from dual union all
select 67129,'NDRE' from dual union all
select 6713,'USA' from dual union all
select 67130,'NDRE' from dual union all
select 67131,'JMA' from dual union all
select 67132,'JMA' from dual union all
select 67133,'JMA' from dual union all
select 67134,'JMA' from dual union all
select 67135,'JMA' from dual union all
select 67136,'JMA' from dual union all
select 67137,'JMA' from dual union all
select 67138,'JMA' from dual union all
select 67139,'JMA' from dual union all
select 6714,'USAF' from dual union all
select 67140,'ISAS' from dual union all
select 67141,'ISAS' from dual union all
select 67142,'JMA' from dual union all
select 67143,'JMA' from dual union all
select 67144,'JMA' from dual union all
select 67145,'JMA' from dual union all
select 67146,'JMA' from dual union all
select 67147,'JMA' from dual union all
select 67148,'JMA' from dual union all
select 67149,'SPFL' from dual union all
select 6715,'USA' from dual union all
select 67150,'SPFL' from dual union all
select 67151,'JMA' from dual union all
select 67152,'MRN' from dual union all
select 67153,'MRN' from dual union all
select 67154,'CALT' from dual union all
select 67155,'AE' from dual union all
select 67156,'RAKA' from dual union all
select 67157,'BLS' from dual union all
select 67158,'AE' from dual union all
select 67159,'NASA' from dual union all
select 6716,'USA' from dual union all
select 67160,'PU' from dual union all
select 67161,'RAKA' from dual union all
select 67162,'USAF' from dual union all
select 67163,'NASA' from dual union all
select 67164,'AE' from dual union all
select 67165,'SEALP' from dual union all
select 67166,'BLS' from dual union all
select 67167,'KHRU' from dual union all
select 67168,'ISRO' from dual union all
select 67169,'NASA' from dual union all
select 6717,'USA' from dual union all
select 67170,'RAKA' from dual union all
select 67171,'SEALP' from dual union all
select 67172,'ILSK' from dual;
insert into LAUNCH_AGENCY
select 67173,'BLS' from dual union all
select 67174,'RAKA' from dual union all
select 67175,'KVR' from dual union all
select 67176,'KVR' from dual union all
select 67177,'AE' from dual union all
select 67178,'ILSK' from dual union all
select 67179,'ILSL' from dual union all
select 6718,'NASA' from dual union all
select 67180,'BLS' from dual union all
select 67181,'NASA' from dual union all
select 67182,'AE' from dual union all
select 67183,'KVR' from dual union all
select 67184,'ILSL' from dual union all
select 67185,'KVR' from dual union all
select 67186,'USAF' from dual union all
select 67187,'BLS' from dual union all
select 67188,'NASA' from dual union all
select 67189,'RAKA' from dual union all
select 6719,'NASA' from dual union all
select 67190,'KHRU' from dual union all
select 67191,'NASDA' from dual union all
select 67192,'AE' from dual union all
select 67193,'ILSL' from dual union all
select 67194,'RAKA' from dual union all
select 67195,'AE' from dual union all
select 67196,'LMA' from dual union all
select 67197,'USAF' from dual union all
select 67198,'KHRU' from dual union all
select 67199,'ILSL' from dual union all
select 672,'PARD' from dual union all
select 6720,'ISAS' from dual union all
select 67200,'BLS' from dual union all
select 67201,'RAKA' from dual union all
select 67202,'ISRO' from dual union all
select 67203,'KVR' from dual union all
select 67204,'RAKA' from dual union all
select 67205,'AE' from dual union all
select 67206,'KHRU' from dual union all
select 67207,'NASA' from dual union all
select 67208,'BLS' from dual union all
select 67209,'KVR' from dual union all
select 6721,'AEC' from dual union all
select 67210,'KVR' from dual union all
select 67211,'KVR' from dual union all
select 67212,'OSC' from dual union all
select 67213,'NSPO' from dual union all
select 67214,'SMDC' from dual union all
select 67215,'AFSPC' from dual union all
select 67216,'NASA' from dual union all
select 67217,'IDF' from dual union all
select 67218,'IQMIC' from dual union all
select 67219,'IQMIC' from dual union all
select 6722,'USA' from dual union all
select 67220,'IQMIC' from dual union all
select 67221,'IQMIC' from dual union all
select 67222,'IQMIC' from dual union all
select 67223,'IQMIC' from dual union all
select 67224,'IQMIC' from dual union all
select 67225,'IQMIC' from dual union all
select 67226,'IQMIC' from dual union all
select 67227,'IQMIC' from dual union all
select 67228,'IQMIC' from dual union all
select 67229,'IQMIC' from dual union all
select 6723,'USA' from dual union all
select 67230,'IQMIC' from dual union all
select 67231,'IQMIC' from dual union all
select 67232,'IQMIC' from dual union all
select 67233,'IQMIC' from dual union all
select 67234,'IQMIC' from dual union all
select 67235,'IQMIC' from dual union all
select 67236,'IQMIC' from dual union all
select 67237,'IQMIC' from dual union all
select 67238,'ISRO' from dual union all
select 67239,'ISRO' from dual union all
select 6724,'USA' from dual union all
select 67240,'ISRO' from dual union all
select 67241,'IDRDL' from dual union all
select 67242,'IQMIC' from dual union all
select 67243,'IQMIC' from dual union all
select 67244,'IQMIC' from dual union all
select 67245,'IQMIC' from dual union all
select 67246,'IQMIC' from dual union all
select 67247,'IQMIC' from dual union all
select 67248,'IQMIC' from dual union all
select 67249,'IQMIC' from dual union all
select 6725,'USN' from dual union all
select 67250,'IQMIC' from dual union all
select 67251,'IQMIC' from dual union all
select 67252,'IQMIC' from dual union all
select 67253,'IQMIC' from dual union all
select 67254,'IQMIC' from dual union all
select 67255,'IQMIC' from dual union all
select 67256,'IQMIC' from dual union all
select 67257,'IQMIC' from dual union all
select 67258,'IQMIC' from dual union all
select 67259,'IR' from dual union all
select 6726,'USN' from dual union all
select 67260,'ISRO' from dual union all
select 67261,'ISRO' from dual union all
select 67262,'IDRDL' from dual;
insert into LAUNCH_AGENCY
select 67263,'USN' from dual union all
select 67264,'USN' from dual union all
select 67265,'AFSPC' from dual union all
select 67266,'NASA' from dual union all
select 67267,'NASA' from dual union all
select 67268,'VMFR' from dual union all
select 67269,'RVSNR' from dual union all
select 6727,'USN' from dual union all
select 67270,'DLR' from dual union all
select 67271,'SSDC' from dual union all
select 67272,'NASA' from dual union all
select 67273,'AFSMC' from dual union all
select 67273,'USN' from dual union all
select 67274,'COLE' from dual union all
select 67274,'SSDC' from dual union all
select 67275,'IDRDL' from dual union all
select 67276,'DMA' from dual union all
select 67277,'ESA' from dual union all
select 67277,'SSC' from dual union all
select 67278,'SSC' from dual union all
select 67279,'USN' from dual union all
select 6728,'USN' from dual union all
select 67280,'NDRE' from dual union all
select 67281,'VMFR' from dual union all
select 67282,'NASA' from dual union all
select 67283,'NDRE' from dual union all
select 67284,'GSFC' from dual union all
select 67285,'NDRE' from dual union all
select 67286,'USN' from dual union all
select 67287,'USN' from dual union all
select 67288,'USN' from dual union all
select 67289,'IDF' from dual union all
select 6729,'NMSU' from dual union all
select 67290,'RVSNR' from dual union all
select 67291,'NASA' from dual union all
select 67292,'COLE' from dual union all
select 67292,'SSDC' from dual union all
select 67293,'AFMC' from dual union all
select 67294,'SSDC' from dual union all
select 67295,'VMFR' from dual union all
select 67296,'AFSPC' from dual union all
select 67297,'IAI' from dual union all
select 67298,'AFMC' from dual union all
select 67299,'VMFR' from dual union all
select 673,'PARD' from dual union all
select 6730,'NMSU' from dual union all
select 67300,'VMFR' from dual union all
select 67301,'VMFR' from dual union all
select 67302,'NASA' from dual union all
select 67302,'SMDC' from dual union all
select 67303,'RVSNR' from dual union all
select 67304,'VMFR' from dual union all
select 67305,'RVSNR' from dual union all
select 67306,'UQU' from dual union all
select 67307,'RVSNR' from dual union all
select 67308,'AFSPC' from dual union all
select 67309,'AFSPC' from dual union all
select 6731,'NMSU' from dual union all
select 67310,'SSDC' from dual union all
select 67311,'GSFC' from dual union all
select 67312,'USN' from dual union all
select 67313,'USN' from dual union all
select 67314,'USN' from dual union all
select 67315,'USN' from dual union all
select 67316,'IDRDL' from dual union all
select 67317,'SSC' from dual union all
select 67318,'USN' from dual union all
select 67319,'USN' from dual union all
select 6732,'AFCRL' from dual union all
select 67320,'USN' from dual union all
select 67321,'USN' from dual union all
select 67322,'SYMC' from dual union all
select 67323,'PLA2AC' from dual union all
select 67324,'IR' from dual union all
select 67325,'IR' from dual union all
select 67326,'IR' from dual union all
select 67327,'OSC' from dual union all
select 67328,'ISAS' from dual union all
select 67329,'NASA' from dual union all
select 6733,'WRE' from dual union all
select 67330,'NDRE' from dual union all
select 67331,'NDRE' from dual union all
select 67332,'NDRE' from dual union all
select 67333,'NDRE' from dual union all
select 67334,'NDRE' from dual union all
select 67335,'NDRE' from dual union all
select 67336,'NDRE' from dual union all
select 67337,'NDRE' from dual union all
select 67338,'NDRE' from dual union all
select 67339,'NDRE' from dual union all
select 6734,'WRE' from dual union all
select 67340,'NDRE' from dual union all
select 67341,'NDRE' from dual union all
select 67342,'SSC' from dual union all
select 67343,'SSC' from dual union all
select 67344,'SSC' from dual union all
select 67345,'SSC' from dual union all
select 67346,'DLR' from dual union all
select 67347,'DLR' from dual union all
select 67348,'DLR' from dual;
insert into LAUNCH_AGENCY
select 67349,'DLR' from dual union all
select 6735,'LARCN' from dual union all
select 67350,'DLR' from dual union all
select 67351,'DLR' from dual union all
select 67352,'DLR' from dual union all
select 67353,'DLR' from dual union all
select 67354,'DLR' from dual union all
select 67355,'DLR' from dual union all
select 67356,'DLR' from dual union all
select 67357,'DLR' from dual union all
select 67358,'DLR' from dual union all
select 67359,'DLR' from dual union all
select 6736,'USA' from dual union all
select 67360,'DLR' from dual union all
select 67361,'DLR' from dual union all
select 67362,'DLR' from dual union all
select 67363,'DLR' from dual union all
select 67364,'DLR' from dual union all
select 67365,'DLR' from dual union all
select 67366,'DLR' from dual union all
select 67367,'DLR' from dual union all
select 67368,'DLR' from dual union all
select 67369,'DLR' from dual union all
select 6737,'AFSWC' from dual union all
select 67370,'DLR' from dual union all
select 67371,'DLR' from dual union all
select 67372,'DLR' from dual union all
select 67373,'DLR' from dual union all
select 67374,'DLR' from dual union all
select 67375,'DLR' from dual union all
select 67376,'SSC' from dual union all
select 67377,'USAF' from dual union all
select 67378,'AE' from dual union all
select 67379,'NASDA' from dual union all
select 6738,'NMC' from dual union all
select 67380,'OSC' from dual union all
select 67381,'BLS' from dual union all
select 67382,'ILSL' from dual union all
select 67383,'AE' from dual union all
select 67384,'KVR' from dual union all
select 67385,'AE' from dual union all
select 67386,'NASA' from dual union all
select 67387,'ILSL' from dual union all
select 67388,'KVR' from dual union all
select 67389,'RAKA' from dual union all
select 6739,'USN' from dual union all
select 67390,'CALT' from dual union all
select 67391,'AE' from dual union all
select 67392,'ILSK' from dual union all
select 67393,'KVR' from dual union all
select 67394,'NASA' from dual union all
select 67395,'AE' from dual union all
select 67396,'RAKA' from dual union all
select 67397,'AE' from dual union all
select 67398,'BLS' from dual union all
select 67399,'ILSK' from dual union all
select 674,'PARD' from dual union all
select 6740,'USAF' from dual union all
select 67400,'SAST' from dual union all
select 67401,'ISA' from dual union all
select 67402,'KVR' from dual union all
select 67403,'AE' from dual union all
select 67404,'NASA' from dual union all
select 67405,'KHRU' from dual union all
select 67406,'SEALP' from dual union all
select 67407,'KVR' from dual union all
select 67408,'USAF' from dual union all
select 67409,'RAKA' from dual union all
select 6741,'USN' from dual union all
select 67410,'BLS' from dual union all
select 67411,'AE' from dual union all
select 67412,'KVR' from dual union all
select 67413,'KHRU' from dual union all
select 67414,'ILSL' from dual union all
select 67415,'ILSK' from dual union all
select 67416,'AE' from dual union all
select 67417,'AE' from dual union all
select 67418,'NASDA' from dual union all
select 67419,'ISRO' from dual union all
select 6742,'USAF' from dual union all
select 67420,'ILSL' from dual union all
select 67421,'RAKA' from dual union all
select 67422,'KVR' from dual union all
select 67423,'NASA' from dual union all
select 67424,'KHRU' from dual union all
select 67425,'SAST' from dual union all
select 67426,'RAKA' from dual union all
select 67427,'BLS' from dual union all
select 67428,'NASA' from dual union all
select 67429,'ILSK' from dual union all
select 6743,'USAF' from dual union all
select 67430,'KVR' from dual union all
select 67431,'ILSL' from dual union all
select 67432,'NASDA' from dual union all
select 67433,'AE' from dual union all
select 67434,'KTRAS' from dual union all
select 67434,'RVSNR' from dual union all
select 67435,'KVR' from dual union all
select 67436,'KHRU' from dual union all
select 67437,'CALT' from dual;
insert into LAUNCH_AGENCY
select 67438,'ILSK' from dual union all
select 67439,'NAL' from dual union all
select 6744,'USN' from dual union all
select 67440,'MDA' from dual union all
select 67441,'KARI' from dual union all
select 67442,'CSXT' from dual union all
select 67443,'IQMIC' from dual union all
select 67444,'IQMIC' from dual union all
select 67445,'IQMIC' from dual union all
select 67446,'IQMIC' from dual union all
select 67447,'IQMIC' from dual union all
select 67448,'IQMIC' from dual union all
select 67449,'IQMIC' from dual union all
select 6745,'RAE' from dual union all
select 67450,'IQMIC' from dual union all
select 67451,'IQMIC' from dual union all
select 67452,'IQMIC' from dual union all
select 67453,'IQMIC' from dual union all
select 67454,'IQMIC' from dual union all
select 67455,'IQMIC' from dual union all
select 67456,'IQMIC' from dual union all
select 67457,'IQMIC' from dual union all
select 67458,'IQMIC' from dual union all
select 67459,'IQMIC' from dual union all
select 6746,'USAF' from dual union all
select 67460,'USN' from dual union all
select 67461,'PLA2AC' from dual union all
select 67462,'RAKA' from dual union all
select 67463,'AE' from dual union all
select 67464,'ISRO' from dual union all
select 67465,'ISRO' from dual union all
select 67466,'ARC' from dual union all
select 67467,'USN' from dual union all
select 67468,'USN' from dual union all
select 67469,'IQMIC' from dual union all
select 6747,'CARDE' from dual union all
select 67470,'IQMIC' from dual union all
select 67471,'IQMIC' from dual union all
select 67472,'IQMIC' from dual union all
select 67473,'IQMIC' from dual union all
select 67474,'IQMIC' from dual union all
select 67475,'IQMIC' from dual union all
select 67476,'IQMIC' from dual union all
select 67477,'IQMIC' from dual union all
select 67478,'IQMIC' from dual union all
select 67479,'IQMIC' from dual union all
select 6748,'NASA' from dual union all
select 67480,'IQMIC' from dual union all
select 67481,'IQMIC' from dual union all
select 67482,'IQMIC' from dual union all
select 67483,'IQMIC' from dual union all
select 67484,'NESCOMP' from dual union all
select 67485,'PK' from dual union all
select 67486,'IR' from dual union all
select 67487,'PLA2AC' from dual union all
select 67488,'NASA' from dual union all
select 67489,'IDRDL' from dual union all
select 6749,'NASA' from dual union all
select 67490,'USN' from dual union all
select 67491,'USN' from dual union all
select 67492,'NASA' from dual union all
select 67493,'GSFC' from dual union all
select 67494,'ISAS' from dual union all
select 67495,'NASA' from dual union all
select 67496,'NASA' from dual union all
select 67497,'NASA' from dual union all
select 67498,'NASA' from dual union all
select 67499,'GSFC' from dual union all
select 675,'PARD' from dual union all
select 6750,'USAF' from dual union all
select 67500,'GSFC' from dual union all
select 67501,'GSFC' from dual union all
select 67502,'GSFC' from dual union all
select 67503,'USN' from dual union all
select 67504,'AFSPC' from dual union all
select 67505,'SSDC' from dual union all
select 67506,'SSC' from dual union all
select 67507,'GSFC' from dual union all
select 67508,'COLE' from dual union all
select 67508,'SSDC' from dual union all
select 67509,'AFSPC' from dual union all
select 67509,'ALCS' from dual union all
select 6751,'NRL' from dual union all
select 67510,'AFSMC' from dual union all
select 67510,'USN' from dual union all
select 67511,'USN' from dual union all
select 67512,'USN' from dual union all
select 67513,'USN' from dual union all
select 67514,'OSCC' from dual union all
select 67514,'SSDC' from dual union all
select 67515,'GSFC' from dual union all
select 67516,'IR' from dual union all
select 67517,'GSFC' from dual union all
select 67518,'AEB' from dual union all
select 67519,'GSFC' from dual union all
select 6752,'NASA' from dual union all
select 67520,'KRLP' from dual union all
select 67521,'SSDC' from dual union all
select 67522,'USAF' from dual union all
select 67523,'GSFC' from dual;
insert into LAUNCH_AGENCY
select 67524,'GSFC' from dual union all
select 67525,'RVSNR' from dual union all
select 67526,'GSFC' from dual union all
select 67527,'AFSPC' from dual union all
select 67528,'GSFC' from dual union all
select 67529,'USN' from dual union all
select 6753,'USAF' from dual union all
select 67530,'USN' from dual union all
select 67531,'GSFC' from dual union all
select 67532,'GSFC' from dual union all
select 67533,'DLR' from dual union all
select 67534,'GSFC' from dual union all
select 67535,'GSFC' from dual union all
select 67536,'GSFC' from dual union all
select 67537,'DLR' from dual union all
select 67538,'DLR' from dual union all
select 67539,'VMFR' from dual union all
select 6754,'CARDE' from dual union all
select 67540,'AFSPC' from dual union all
select 67541,'UQU' from dual union all
select 67542,'ISAS' from dual union all
select 67543,'ISAS' from dual union all
select 67544,'GSFC' from dual union all
select 67545,'AFSPC' from dual union all
select 67546,'PRO' from dual union all
select 67547,'NESCOMN' from dual union all
select 67548,'VMFR' from dual union all
select 67549,'RVSNR' from dual union all
select 6755,'MVS' from dual union all
select 67550,'VMFR' from dual union all
select 67551,'AFSPC' from dual union all
select 67552,'SSDC' from dual union all
select 67553,'USN' from dual union all
select 67554,'GSFC' from dual union all
select 67555,'GSFC' from dual union all
select 67556,'USN' from dual union all
select 67557,'USN' from dual union all
select 67558,'AEB' from dual union all
select 67559,'MDA' from dual union all
select 6756,'MVS' from dual union all
select 67560,'AEB' from dual union all
select 67561,'RVSNR' from dual union all
select 67562,'USN' from dual union all
select 67563,'AFSPC' from dual union all
select 67564,'SSDC' from dual union all
select 67565,'GSFC' from dual union all
select 67566,'PLA2AC' from dual union all
select 67567,'PLA2AC' from dual union all
select 67568,'PLA2AC' from dual union all
select 67569,'IR' from dual union all
select 6757,'MVS' from dual union all
select 67570,'PLA2AC' from dual union all
select 67571,'RVSNR' from dual union all
select 67572,'NASA' from dual union all
select 67573,'NASA' from dual union all
select 67574,'NASA' from dual union all
select 67575,'NASA' from dual union all
select 67576,'DLR' from dual union all
select 67577,'DLR' from dual union all
select 67578,'DLR' from dual union all
select 67579,'DLR' from dual union all
select 6758,'RVSN' from dual union all
select 67580,'DLR' from dual union all
select 67581,'DLR' from dual union all
select 67582,'DLR' from dual union all
select 67583,'DLR' from dual union all
select 67584,'DLR' from dual union all
select 67585,'DLR' from dual union all
select 67586,'DLR' from dual union all
select 67587,'DLR' from dual union all
select 67588,'DLR' from dual union all
select 67589,'DLR' from dual union all
select 6759,'LARCN' from dual union all
select 67590,'DLR' from dual union all
select 67591,'DLR' from dual union all
select 67592,'DLR' from dual union all
select 67593,'DLR' from dual union all
select 67594,'DLR' from dual union all
select 67595,'DLR' from dual union all
select 67596,'DLR' from dual union all
select 67597,'DLR' from dual union all
select 67598,'DLR' from dual union all
select 67599,'DLR' from dual union all
select 676,'ORD' from dual union all
select 6760,'USA' from dual union all
select 67600,'DLR' from dual union all
select 67601,'DLR' from dual union all
select 67602,'DLR' from dual union all
select 67603,'DLR' from dual union all
select 67604,'DLR' from dual union all
select 67605,'DLR' from dual union all
select 67606,'DLR' from dual union all
select 67607,'DLR' from dual union all
select 67608,'DLR' from dual union all
select 67609,'DLR' from dual union all
select 6761,'USA' from dual union all
select 67610,'DLR' from dual union all
select 67611,'DLR' from dual union all
select 67612,'DLR' from dual union all
select 67613,'DLR' from dual;
insert into LAUNCH_AGENCY
select 67614,'DLR' from dual union all
select 67615,'DLR' from dual union all
select 67616,'DLR' from dual union all
select 67617,'DLR' from dual union all
select 67618,'DLR' from dual union all
select 67619,'DLR' from dual union all
select 6762,'RVSN' from dual union all
select 67620,'DLR' from dual union all
select 67621,'DLR' from dual union all
select 67622,'DLR' from dual union all
select 67623,'DLR' from dual union all
select 67624,'DLR' from dual union all
select 67625,'USAF' from dual union all
select 67626,'BLS' from dual union all
select 67627,'NASA' from dual union all
select 67628,'OSC' from dual union all
select 67629,'BLS' from dual union all
select 6763,'RVSN' from dual union all
select 67630,'RAKA' from dual union all
select 67631,'AE' from dual union all
select 67632,'BLS' from dual union all
select 67633,'NASDA' from dual union all
select 67634,'BLS' from dual union all
select 67635,'KVR' from dual union all
select 67636,'USAF' from dual union all
select 67637,'AE' from dual union all
select 67638,'ILSL' from dual union all
select 67639,'KHRU' from dual union all
select 6764,'AFSWC' from dual union all
select 67640,'RAKA' from dual union all
select 67641,'OSC' from dual union all
select 67642,'ISRO' from dual union all
select 67643,'ISAS' from dual union all
select 67644,'ILSL' from dual union all
select 67645,'CALT' from dual union all
select 67646,'STSM' from dual union all
select 67647,'KVR' from dual union all
select 67648,'ILSK' from dual union all
select 67649,'RAKA' from dual union all
select 6765,'NASA' from dual union all
select 67650,'SEALP' from dual union all
select 67651,'BLS' from dual union all
select 67652,'AE' from dual union all
select 67653,'KVR' from dual union all
select 67654,'OSC' from dual union all
select 67655,'KVR' from dual union all
select 67656,'BLS' from dual union all
select 67657,'ILSL' from dual union all
select 67658,'SEALP' from dual union all
select 67659,'KVR' from dual union all
select 6766,'ABMA' from dual union all
select 67660,'OSC' from dual union all
select 67661,'KVR' from dual union all
select 67662,'BLS' from dual union all
select 67663,'RAKA' from dual union all
select 67664,'BLS' from dual union all
select 67665,'USAF' from dual union all
select 67666,'KVR' from dual union all
select 67667,'AE' from dual union all
select 67668,'SEALP' from dual union all
select 67669,'CALT' from dual union all
select 6767,'RVSN' from dual union all
select 67670,'ISRO' from dual union all
select 67671,'RAKA' from dual union all
select 67672,'USAF' from dual union all
select 67673,'SAST' from dual union all
select 67674,'KVR' from dual union all
select 67675,'SAST' from dual union all
select 67676,'CALT' from dual union all
select 67677,'KHRU' from dual union all
select 67678,'ILSL' from dual union all
select 67679,'RVSNR' from dual union all
select 6768,'USAF' from dual union all
select 67680,'KHRU' from dual union all
select 67681,'ILSL' from dual union all
select 67682,'BLS' from dual union all
select 67683,'STSM' from dual union all
select 67684,'KHRU' from dual union all
select 67685,'CALT' from dual union all
select 67686,'IAI' from dual union all
select 67687,'GSFC' from dual union all
select 67688,'SCAL' from dual union all
select 67689,'GSFC' from dual union all
select 6769,'CASDN' from dual union all
select 67690,'SCAL' from dual union all
select 67691,'SCAL' from dual union all
select 67692,'SCAL' from dual union all
select 67693,'SCAL' from dual union all
select 67694,'SCAL' from dual union all
select 67695,'SCAL' from dual union all
select 67696,'DARPA' from dual union all
select 67697,'SCAL' from dual union all
select 67698,'SCAL' from dual union all
select 67699,'SCAL' from dual union all
select 677,'NRL' from dual union all
select 677,'US' from dual union all
select 6770,'AN' from dual union all
select 67700,'IAI' from dual union all
select 67701,'SCAL' from dual union all
select 67702,'NDRE' from dual;
insert into LAUNCH_AGENCY
select 67703,'IDF' from dual union all
select 67704,'IQA' from dual union all
select 67705,'IQA' from dual union all
select 67706,'IQA' from dual union all
select 67707,'IQA' from dual union all
select 67708,'IQA' from dual union all
select 67709,'MDA' from dual union all
select 6771,'NRL' from dual union all
select 67710,'MDA' from dual union all
select 67711,'AEB' from dual union all
select 67712,'NASDA' from dual union all
select 67713,'GSFC' from dual union all
select 67714,'ISRO' from dual union all
select 67715,'USN' from dual union all
select 67716,'USN' from dual union all
select 67717,'IQA' from dual union all
select 67718,'IQA' from dual union all
select 67719,'IQA' from dual union all
select 6772,'LARCN' from dual union all
select 67720,'IQA' from dual union all
select 67721,'IQA' from dual union all
select 67722,'IQA' from dual union all
select 67723,'IQA' from dual union all
select 67724,'IQA' from dual union all
select 67725,'NESCOMP' from dual union all
select 67726,'IQA' from dual union all
select 67727,'PK' from dual union all
select 67728,'IDRDL' from dual union all
select 67729,'GSFC' from dual union all
select 6773,'NASA' from dual union all
select 67730,'GSFC' from dual union all
select 67731,'GSFC' from dual union all
select 67732,'GSFC' from dual union all
select 67733,'OSC' from dual union all
select 67734,'USAF' from dual union all
select 67735,'GSFC' from dual union all
select 67736,'GSFC' from dual union all
select 67737,'IDRDL' from dual union all
select 67738,'RVSNR' from dual union all
select 67739,'GSFC' from dual union all
select 6774,'CASDN' from dual union all
select 67740,'NASA' from dual union all
select 67741,'GSFC' from dual union all
select 67742,'ESA' from dual union all
select 67742,'SSC' from dual union all
select 67743,'SSC' from dual union all
select 67744,'GSFC' from dual union all
select 67745,'USN' from dual union all
select 67746,'USN' from dual union all
select 67747,'IDRDL' from dual union all
select 67748,'MITLL' from dual union all
select 67748,'NASA' from dual union all
select 67749,'MITLL' from dual union all
select 67749,'NASA' from dual union all
select 6775,'RAE' from dual union all
select 6775,'WRE' from dual union all
select 67750,'GSFC' from dual union all
select 67751,'AFSPC' from dual union all
select 67751,'ALCS' from dual union all
select 67752,'USN' from dual union all
select 67753,'USN' from dual union all
select 67754,'GSFC' from dual union all
select 67755,'GSFC' from dual union all
select 67756,'GSFC' from dual union all
select 67757,'GSFC' from dual union all
select 67758,'DLR' from dual union all
select 67759,'DLR' from dual union all
select 6776,'CASDN' from dual union all
select 67760,'DLR' from dual union all
select 67761,'GSFC' from dual union all
select 67762,'GSFC' from dual union all
select 67763,'GSFC' from dual union all
select 67764,'AFSPC' from dual union all
select 67765,'GSFC' from dual union all
select 67766,'OSC' from dual union all
select 67767,'GSFC' from dual union all
select 67768,'AFSPC' from dual union all
select 67769,'GSFC' from dual union all
select 6777,'AFCRL' from dual union all
select 67770,'GSFC' from dual union all
select 67771,'USN' from dual union all
select 67772,'USN' from dual union all
select 67773,'GSFC' from dual union all
select 67774,'GSFC' from dual union all
select 67775,'RVSN' from dual union all
select 67776,'USN' from dual union all
select 67777,'USN' from dual union all
select 67778,'GSFC' from dual union all
select 67779,'NSPO' from dual union all
select 6778,'RAE' from dual union all
select 67780,'ISRO' from dual union all
select 67781,'VMFR' from dual union all
select 67782,'IR' from dual union all
select 67783,'NESCOMN' from dual union all
select 67784,'VMFR' from dual union all
select 67785,'NESCOMN' from dual union all
select 67786,'MDA' from dual union all
select 67787,'RVSNR' from dual union all
select 67788,'PLA2AC' from dual union all
select 67789,'FLASP' from dual;
insert into LAUNCH_AGENCY
select 6779,'USN' from dual union all
select 67790,'NASA' from dual union all
select 67791,'NASA' from dual union all
select 67792,'NASA' from dual union all
select 67793,'NASA' from dual union all
select 67794,'NASA' from dual union all
select 67795,'NASA' from dual union all
select 67796,'NASA' from dual union all
select 67797,'NASA' from dual union all
select 67798,'NASA' from dual union all
select 67799,'NASA' from dual union all
select 678,'ORD' from dual union all
select 6780,'CASDN' from dual union all
select 67800,'NASA' from dual union all
select 67801,'NASA' from dual union all
select 67802,'NASA' from dual union all
select 67803,'NASA' from dual union all
select 67804,'NASA' from dual union all
select 67805,'NASA' from dual union all
select 67806,'NASA' from dual union all
select 67807,'NASA' from dual union all
select 67808,'NASA' from dual union all
select 67809,'NASA' from dual union all
select 6781,'USAF' from dual union all
select 67810,'NASA' from dual union all
select 67811,'NASA' from dual union all
select 67812,'NASA' from dual union all
select 67813,'NASA' from dual union all
select 67814,'NASA' from dual union all
select 67815,'NASA' from dual union all
select 67816,'NASA' from dual union all
select 67817,'NASA' from dual union all
select 67818,'NASA' from dual union all
select 67819,'NASA' from dual union all
select 6782,'RAF' from dual union all
select 67820,'NASA' from dual union all
select 67821,'DLR' from dual union all
select 67822,'DLR' from dual union all
select 67823,'FLASP' from dual union all
select 67824,'FLASP' from dual union all
select 67825,'FLASP' from dual union all
select 67826,'NASA' from dual union all
select 67827,'NASA' from dual union all
select 67828,'NASA' from dual union all
select 67829,'NASA' from dual union all
select 6783,'MVS' from dual union all
select 67830,'NASA' from dual union all
select 67831,'NASA' from dual union all
select 67832,'NASA' from dual union all
select 67833,'NASA' from dual union all
select 67834,'NASA' from dual union all
select 67835,'NASA' from dual union all
select 67836,'NASA' from dual union all
select 67837,'NASA' from dual union all
select 67838,'NASA' from dual union all
select 67839,'NASA' from dual union all
select 6784,'USN' from dual union all
select 67840,'NASA' from dual union all
select 67841,'NASA' from dual union all
select 67842,'NASA' from dual union all
select 67843,'NASA' from dual union all
select 67844,'NASA' from dual union all
select 67845,'SEALP' from dual union all
select 67846,'RAKA' from dual union all
select 67847,'ILSL' from dual union all
select 67848,'USAF' from dual union all
select 67849,'KVR' from dual union all
select 6785,'AN' from dual union all
select 67850,'AE' from dual union all
select 67851,'ILSL' from dual union all
select 67852,'ILSK' from dual union all
select 67853,'BLS' from dual union all
select 67854,'KHRU' from dual union all
select 67855,'ILSL' from dual union all
select 67856,'CALT' from dual union all
select 67857,'FKA' from dual union all
select 67858,'BLS' from dual union all
select 67859,'KHRU' from dual union all
select 6786,'GURVO' from dual union all
select 67860,'SEALP' from dual union all
select 67861,'ILSL' from dual union all
select 67862,'OSC' from dual union all
select 67863,'FKA' from dual union all
select 67864,'KVR' from dual union all
select 67865,'KVR' from dual union all
select 67866,'ILSK' from dual union all
select 67867,'BLS' from dual union all
select 67868,'SEALP' from dual union all
select 67869,'KTRAS' from dual union all
select 67869,'RVSNR' from dual union all
select 6787,'MVS' from dual union all
select 67870,'BLS' from dual union all
select 67871,'AE' from dual union all
select 67872,'KVR' from dual union all
select 67873,'CALT' from dual union all
select 67874,'BLS' from dual union all
select 67875,'ILSK' from dual union all
select 67876,'FKA' from dual union all
select 67877,'CALT' from dual union all
select 67878,'ILSL' from dual;
insert into LAUNCH_AGENCY
select 67879,'SAST' from dual union all
select 6788,'USAF' from dual union all
select 67880,'ISRO' from dual union all
select 67881,'KVR' from dual union all
select 67882,'KVR' from dual union all
select 67883,'SAST' from dual union all
select 67884,'FKA' from dual union all
select 67885,'ILSK' from dual union all
select 67886,'CALT' from dual union all
select 67887,'ILSK' from dual union all
select 67888,'SAST' from dual union all
select 67889,'BLS' from dual union all
select 6789,'NASA' from dual union all
select 67890,'CALT' from dual union all
select 67891,'BLS' from dual union all
select 67892,'ILSL' from dual union all
select 67893,'AE' from dual union all
select 67894,'BLS' from dual union all
select 67895,'FKA' from dual union all
select 67896,'KVR' from dual union all
select 67897,'KHRU' from dual union all
select 67898,'VMFR' from dual union all
select 67899,'SCAL' from dual union all
select 679,'PARD' from dual union all
select 6790,'NASA' from dual union all
select 67900,'OSC' from dual union all
select 67901,'SCAL' from dual union all
select 67902,'USN' from dual union all
select 67903,'GSFC' from dual union all
select 67904,'PLA2AC' from dual union all
select 67905,'MDA' from dual union all
select 67906,'MDA' from dual union all
select 67907,'USN' from dual union all
select 67908,'VMF' from dual union all
select 67909,'OSC' from dual union all
select 6791,'USAF' from dual union all
select 67910,'GSFC' from dual union all
select 67911,'USN' from dual union all
select 67912,'DARPA' from dual union all
select 67913,'PVO' from dual union all
select 67914,'ISA' from dual union all
select 67915,'SCAL' from dual union all
select 67916,'MDA' from dual union all
select 67917,'ISRO' from dual union all
select 67918,'ISRO' from dual union all
select 67919,'LAPAN' from dual union all
select 6792,'USAF' from dual union all
select 67920,'LAPAN' from dual union all
select 67921,'ISRO' from dual union all
select 67922,'IDRDL' from dual union all
select 67923,'ISRO' from dual union all
select 67924,'IDRDL' from dual union all
select 67925,'IDRDL' from dual union all
select 67926,'USN' from dual union all
select 67927,'NESCOMP' from dual union all
select 67928,'AFSPC' from dual union all
select 67929,'GSFC' from dual union all
select 6793,'NRL' from dual union all
select 67930,'ISAS' from dual union all
select 67931,'GSFC' from dual union all
select 67932,'IDRDL' from dual union all
select 67933,'OSC' from dual union all
select 67934,'RVSNR' from dual union all
select 67935,'RVSNR' from dual union all
select 67936,'USN' from dual union all
select 67937,'USN' from dual union all
select 67938,'NESCOMN' from dual union all
select 67939,'VMFR' from dual union all
select 6794,'USAF' from dual union all
select 67940,'VMFR' from dual union all
select 67941,'GSFC' from dual union all
select 67942,'RVSNR' from dual union all
select 67943,'COLE' from dual union all
select 67943,'USA' from dual union all
select 67944,'CSXT' from dual union all
select 67945,'KRLP' from dual union all
select 67946,'GSFC' from dual union all
select 67947,'DMA' from dual union all
select 67948,'KRLP' from dual union all
select 67949,'SCAL' from dual union all
select 6795,'USAF' from dual union all
select 67950,'AFSPC' from dual union all
select 67951,'VMFR' from dual union all
select 67952,'IDRDL' from dual union all
select 67953,'USAF' from dual union all
select 67954,'AFSPC' from dual union all
select 67954,'ALCS' from dual union all
select 67955,'GSFC' from dual union all
select 67956,'GSFC' from dual union all
select 67957,'GSFC' from dual union all
select 67958,'ISAS' from dual union all
select 67959,'IR' from dual union all
select 6796,'USAF' from dual union all
select 67960,'RVSNR' from dual union all
select 67961,'GSFC' from dual union all
select 67962,'GSFC' from dual union all
select 67963,'GSFC' from dual union all
select 67964,'GSFC' from dual union all
select 67965,'IDRDL' from dual union all
select 67966,'GSFC' from dual;
insert into LAUNCH_AGENCY
select 67967,'NASA' from dual union all
select 67968,'GSFC' from dual union all
select 67969,'VMFR' from dual union all
select 6797,'MVS' from dual union all
select 67970,'VMFR' from dual union all
select 67971,'AFSPC' from dual union all
select 67972,'NASA' from dual union all
select 67973,'GSFC' from dual union all
select 67974,'GSFC' from dual union all
select 67975,'GSFC' from dual union all
select 67976,'GSFC' from dual union all
select 67977,'MDA' from dual union all
select 67977,'SMDC' from dual union all
select 67978,'SCAL' from dual union all
select 67979,'LAPAN' from dual union all
select 6798,'AFCRL' from dual union all
select 67980,'LAPAN' from dual union all
select 67981,'SCAL' from dual union all
select 67982,'MDA' from dual union all
select 67982,'SMDC' from dual union all
select 67983,'KRLP' from dual union all
select 67984,'GSFC' from dual union all
select 67985,'IR' from dual union all
select 67986,'AEB' from dual union all
select 67987,'EUROL' from dual union all
select 67988,'VMFR' from dual union all
select 67989,'RVSNR' from dual union all
select 6799,'MVS' from dual union all
select 67990,'MDA' from dual union all
select 67990,'OSCC' from dual union all
select 67991,'ESA' from dual union all
select 67991,'SSC' from dual union all
select 67992,'PRO' from dual union all
select 67993,'SSC' from dual union all
select 67994,'JAXA' from dual union all
select 67995,'NSPO' from dual union all
select 67996,'SMDC' from dual union all
select 67997,'RVSNR' from dual union all
select 67998,'RVSNR' from dual union all
select 67999,'ISRO' from dual union all
select 68,'WEHR' from dual union all
select 680,'USN' from dual union all
select 6800,'MVS' from dual union all
select 68000,'PLA2AC' from dual union all
select 68001,'USN' from dual union all
select 68002,'USN' from dual union all
select 68003,'USN' from dual union all
select 68004,'NASA' from dual union all
select 68005,'NASA' from dual union all
select 68006,'NASA' from dual union all
select 68007,'NESCOMN' from dual union all
select 68008,'KVR' from dual union all
select 68009,'FLASP' from dual union all
select 6801,'MVS' from dual union all
select 68010,'BLS' from dual union all
select 68011,'KVR' from dual union all
select 68012,'ILSK' from dual union all
select 68013,'ILSL' from dual union all
select 68014,'ESA' from dual union all
select 68015,'RSC' from dual union all
select 68016,'FKA' from dual union all
select 68017,'SEALP' from dual union all
select 68018,'ILSL' from dual union all
select 68019,'KHRU' from dual union all
select 6802,'MVS' from dual union all
select 68020,'USAF' from dual union all
select 68021,'CALT' from dual union all
select 68022,'FKA' from dual union all
select 68023,'OSC' from dual union all
select 68024,'SEALP' from dual union all
select 68025,'USAF' from dual union all
select 68026,'ISRO' from dual union all
select 68027,'BLS' from dual union all
select 68028,'ILSK' from dual union all
select 68029,'FKA' from dual union all
select 6803,'RVSN' from dual union all
select 68030,'FKA' from dual union all
select 68031,'SEALP' from dual union all
select 68032,'KHRU' from dual union all
select 68033,'SAST' from dual union all
select 68034,'ISAS' from dual union all
select 68035,'NASA' from dual union all
select 68036,'CALT' from dual union all
select 68037,'AE' from dual union all
select 68038,'ILSL' from dual union all
select 68039,'STSM' from dual union all
select 6804,'ABMA' from dual union all
select 68040,'RVSNR' from dual union all
select 68041,'KVR' from dual union all
select 68042,'SAST' from dual union all
select 68043,'KVR' from dual union all
select 68044,'FKA' from dual union all
select 68045,'KHRU' from dual union all
select 68046,'USAF' from dual union all
select 68047,'BLS' from dual union all
select 68048,'FKA' from dual union all
select 68049,'CALT' from dual union all
select 6805,'USN' from dual union all
select 68050,'AE' from dual union all
select 68051,'USAF' from dual;
insert into LAUNCH_AGENCY
select 68052,'KVR' from dual union all
select 68053,'SEALP' from dual union all
select 68054,'STSM' from dual union all
select 68055,'ESA' from dual union all
select 68056,'FKA' from dual union all
select 68057,'KVR' from dual union all
select 68058,'AE' from dual union all
select 68059,'KHRU' from dual union all
select 6806,'USAF' from dual union all
select 68060,'STSM' from dual union all
select 68061,'ILSK' from dual union all
select 68062,'NASA' from dual union all
select 68063,'USN' from dual union all
select 68064,'USN' from dual union all
select 68065,'GSFC' from dual union all
select 68066,'NAL' from dual union all
select 68067,'IAI' from dual union all
select 68068,'DARPA' from dual union all
select 68068,'NASA' from dual union all
select 68069,'IDF' from dual union all
select 6807,'NASA' from dual union all
select 68070,'KVR' from dual union all
select 68071,'VMFR' from dual union all
select 68072,'KVR' from dual union all
select 68073,'PLA2AC' from dual union all
select 68074,'LAPAN' from dual union all
select 68075,'GSFC' from dual union all
select 68076,'GSFC' from dual union all
select 68077,'IDRDL' from dual union all
select 68078,'IDRDL' from dual union all
select 68079,'USN' from dual union all
select 6808,'USAF' from dual union all
select 68080,'PK' from dual union all
select 68081,'ISRO' from dual union all
select 68082,'DLR' from dual union all
select 68082,'SSC' from dual union all
select 68083,'SSC' from dual union all
select 68084,'GSFC' from dual union all
select 68085,'DMA' from dual union all
select 68086,'NASA' from dual union all
select 68086,'NAWC' from dual union all
select 68087,'SMDC' from dual union all
select 68088,'USN' from dual union all
select 68089,'USN' from dual union all
select 6809,'GURVO' from dual union all
select 68090,'NASA' from dual union all
select 68090,'NAWC' from dual union all
select 68091,'USN' from dual union all
select 68092,'USN' from dual union all
select 68093,'NASA' from dual union all
select 68093,'NAWC' from dual union all
select 68094,'NASA' from dual union all
select 68094,'NAWC' from dual union all
select 68095,'GSFC' from dual union all
select 68096,'GSFC' from dual union all
select 68097,'NESCOMN' from dual union all
select 68098,'OSC' from dual union all
select 68099,'SSC' from dual union all
select 681,'USA' from dual union all
select 6810,'RVSN' from dual union all
select 68100,'GSFC' from dual union all
select 68101,'GSFC' from dual union all
select 68102,'SYMC' from dual union all
select 68103,'SYMC' from dual union all
select 68104,'SYMC' from dual union all
select 68105,'PLA2AC' from dual union all
select 68106,'USA' from dual union all
select 68107,'USA' from dual union all
select 68108,'GSFC' from dual union all
select 68109,'NDRE' from dual union all
select 6811,'USA' from dual union all
select 68110,'GSFC' from dual union all
select 68111,'AFSPC' from dual union all
select 68112,'GSFC' from dual union all
select 68113,'USA' from dual union all
select 68114,'VMFR' from dual union all
select 68115,'USA' from dual union all
select 68116,'AFSPC' from dual union all
select 68117,'AFSPC' from dual union all
select 68118,'AFSPC' from dual union all
select 68118,'ALCS' from dual union all
select 68119,'COLE' from dual union all
select 68119,'MDA' from dual union all
select 6812,'GURVO' from dual union all
select 68120,'VMF' from dual union all
select 68121,'GSFC' from dual union all
select 68122,'VMFR' from dual union all
select 68123,'VMFR' from dual union all
select 68124,'VMFR' from dual union all
select 68125,'RN' from dual union all
select 68126,'RVSNR' from dual union all
select 68127,'DLR' from dual union all
select 68128,'RVSNR' from dual union all
select 68129,'NASA' from dual union all
select 68129,'NAWC' from dual union all
select 6813,'RVSN' from dual union all
select 68130,'GSFC' from dual union all
select 68131,'NASA' from dual union all
select 68131,'NAWC' from dual union all
select 68132,'USN' from dual;
insert into LAUNCH_AGENCY
select 68133,'USN' from dual union all
select 68134,'NASA' from dual union all
select 68134,'NAWC' from dual union all
select 68135,'LM' from dual union all
select 68136,'RVSNR' from dual union all
select 68137,'MDA' from dual union all
select 68138,'GSFC' from dual union all
select 68139,'VMF' from dual union all
select 6814,'NASA' from dual union all
select 68140,'ISRO' from dual union all
select 68141,'ISRO' from dual union all
select 68142,'USN' from dual union all
select 68143,'USN' from dual union all
select 68144,'PLA2AC' from dual union all
select 68145,'RVSNR' from dual union all
select 68146,'SSC' from dual union all
select 68147,'SSC' from dual union all
select 68148,'SSC' from dual union all
select 68149,'DLR' from dual union all
select 6815,'ISAS' from dual union all
select 68150,'DLR' from dual union all
select 68151,'DLR' from dual union all
select 68152,'DLR' from dual union all
select 68153,'DLR' from dual union all
select 68154,'DLR' from dual union all
select 68155,'DLR' from dual union all
select 68156,'DLR' from dual union all
select 68157,'DLR' from dual union all
select 68158,'DLR' from dual union all
select 68159,'DLR' from dual union all
select 6816,'RVSN' from dual union all
select 68160,'DLR' from dual union all
select 68161,'DLR' from dual union all
select 68162,'DLR' from dual union all
select 68163,'DLR' from dual union all
select 68164,'DLR' from dual union all
select 68165,'DLR' from dual union all
select 68166,'DLR' from dual union all
select 68167,'DLR' from dual union all
select 68168,'DLR' from dual union all
select 68169,'DLR' from dual union all
select 6817,'MVS' from dual union all
select 68170,'DLR' from dual union all
select 68171,'DLR' from dual union all
select 68172,'DLR' from dual union all
select 68173,'ILSL' from dual union all
select 68174,'RSC' from dual union all
select 68175,'SEALP' from dual union all
select 68176,'RSC' from dual union all
select 68177,'ISAS' from dual union all
select 68178,'ILSK' from dual union all
select 68179,'AE' from dual union all
select 6818,'MVS' from dual union all
select 68180,'OSC' from dual union all
select 68181,'FKA' from dual union all
select 68182,'SEALP' from dual union all
select 68183,'USAF' from dual union all
select 68184,'ILSL' from dual union all
select 68185,'FKA' from dual union all
select 68186,'PU' from dual union all
select 68187,'SAST' from dual union all
select 68188,'BLS' from dual union all
select 68189,'KVR' from dual union all
select 6819,'USAF' from dual union all
select 68190,'BLS' from dual union all
select 68191,'VMFR' from dual union all
select 68192,'AE' from dual union all
select 68193,'FKA' from dual union all
select 68194,'KHRU' from dual union all
select 68195,'SEALP' from dual union all
select 68196,'BLS' from dual union all
select 68197,'FKA' from dual union all
select 68198,'KVR' from dual union all
select 68199,'BLS' from dual union all
select 682,'APL' from dual union all
select 6820,'BRL' from dual union all
select 6820,'UMI' from dual union all
select 68200,'NASA' from dual union all
select 68201,'RVSNR' from dual union all
select 68202,'KVR' from dual union all
select 68203,'KVR' from dual union all
select 68204,'ILSK' from dual union all
select 68205,'AE' from dual union all
select 68206,'SEALP' from dual union all
select 68207,'CALT' from dual union all
select 68208,'NASA' from dual union all
select 68209,'MHI' from dual union all
select 6821,'BRL' from dual union all
select 6821,'UMI' from dual union all
select 68210,'CALT' from dual union all
select 68211,'KVR' from dual union all
select 68212,'FKA' from dual union all
select 68213,'ISAS' from dual union all
select 68214,'BLS' from dual union all
select 68215,'AE' from dual union all
select 68216,'FKA' from dual union all
select 68217,'FKA' from dual union all
select 68218,'SAST' from dual union all
select 68219,'BLS' from dual union all
select 6822,'USAF' from dual;
insert into LAUNCH_AGENCY
select 68220,'CALT' from dual union all
select 68221,'SEALP' from dual union all
select 68222,'BLS' from dual union all
select 68223,'ILSK' from dual union all
select 68224,'BLS' from dual union all
select 68225,'CALT' from dual union all
select 68226,'AE' from dual union all
select 68227,'NASA' from dual union all
select 68228,'ILSK' from dual union all
select 68229,'ULAB' from dual union all
select 6823,'USN' from dual union all
select 68230,'USAF' from dual union all
select 68231,'MHI' from dual union all
select 68232,'KVR' from dual union all
select 68233,'KVR' from dual union all
select 68234,'KHRU' from dual union all
select 68235,'STSM' from dual union all
select 68236,'SPFL' from dual union all
select 68237,'USN' from dual union all
select 68238,'ARR' from dual union all
select 68239,'KCST' from dual union all
select 6824,'AN' from dual union all
select 68240,'VMF' from dual union all
select 68241,'UPAE' from dual union all
select 68242,'VMF' from dual union all
select 68243,'PRO' from dual union all
select 68244,'USN' from dual union all
select 68245,'USN' from dual union all
select 68246,'IRSA' from dual union all
select 68247,'SPX' from dual union all
select 68248,'ISRO' from dual union all
select 68249,'KTRAS' from dual union all
select 68249,'RVSNR' from dual union all
select 6825,'USAF' from dual union all
select 68250,'GSFC' from dual union all
select 68251,'LM' from dual union all
select 68251,'US' from dual union all
select 68252,'ISFC' from dual union all
select 68253,'USN' from dual union all
select 68254,'NESCOMP' from dual union all
select 68255,'NESCOMP' from dual union all
select 68256,'ISRO' from dual union all
select 68257,'ISRO' from dual union all
select 68258,'ISRO' from dual union all
select 68259,'ISRO' from dual union all
select 6826,'ISAS' from dual union all
select 68260,'ISRO' from dual union all
select 68261,'ISRO' from dual union all
select 68262,'IR' from dual union all
select 68263,'NSPO' from dual union all
select 68264,'JAXA' from dual union all
select 68265,'GSFC' from dual union all
select 68266,'AFSPC' from dual union all
select 68267,'SMDC' from dual union all
select 68268,'USN' from dual union all
select 68269,'UQU' from dual union all
select 6827,'RAF' from dual union all
select 68270,'GSFC' from dual union all
select 68271,'GSFC' from dual union all
select 68272,'UQU' from dual union all
select 68273,'EUROL' from dual union all
select 68274,'GSFC' from dual union all
select 68275,'AFSPC' from dual union all
select 68276,'GSFC' from dual union all
select 68277,'MDA' from dual union all
select 68277,'SMDC' from dual union all
select 68278,'RVSNR' from dual union all
select 68279,'MDA' from dual union all
select 68279,'SMDC' from dual union all
select 6828,'USAF' from dual union all
select 68280,'NESCOMN' from dual union all
select 68281,'ESA' from dual union all
select 68281,'SSC' from dual union all
select 68282,'DLR' from dual union all
select 68282,'SSC' from dual union all
select 68283,'LM' from dual union all
select 68283,'US' from dual union all
select 68284,'IR' from dual union all
select 68285,'GSFC' from dual union all
select 68286,'NASA' from dual union all
select 68286,'NAWC' from dual union all
select 68287,'NASA' from dual union all
select 68287,'NAWC' from dual union all
select 68288,'NASA' from dual union all
select 68288,'NAWC' from dual union all
select 68289,'AFSPC' from dual union all
select 6829,'USN' from dual union all
select 68290,'ARR' from dual union all
select 68291,'USN' from dual union all
select 68292,'USN' from dual union all
select 68293,'NASA' from dual union all
select 68293,'NAWC' from dual union all
select 68294,'VMFR' from dual union all
select 68295,'GSFC' from dual union all
select 68296,'KPAMJ' from dual union all
select 68297,'KPAMJ' from dual union all
select 68298,'KPAMJ' from dual union all
select 68299,'KPAMJ' from dual union all
select 683,'USA' from dual union all
select 6830,'AEC' from dual;
insert into LAUNCH_AGENCY
select 68300,'KPAMJ' from dual union all
select 68301,'KPAMJ' from dual union all
select 68302,'IDRDL' from dual union all
select 68303,'COLE' from dual union all
select 68303,'MDA' from dual union all
select 68304,'GSFC' from dual union all
select 68305,'AFSPC' from dual union all
select 68305,'ALCS' from dual union all
select 68306,'GSFC' from dual union all
select 68307,'RVSNR' from dual union all
select 68308,'GSFC' from dual union all
select 68309,'GSFC' from dual union all
select 6831,'RVSN' from dual union all
select 68310,'SMDC' from dual union all
select 68311,'MDA' from dual union all
select 68312,'PLA2AC' from dual union all
select 68313,'VMFR' from dual union all
select 68314,'ARR' from dual union all
select 68314,'DLR' from dual union all
select 68315,'VMFR' from dual union all
select 68316,'COLE' from dual union all
select 68316,'MDA' from dual union all
select 68317,'ARR' from dual union all
select 68317,'DLR' from dual union all
select 68318,'VMF' from dual union all
select 68319,'NASA' from dual union all
select 6832,'USN' from dual union all
select 68320,'GSFC' from dual union all
select 68321,'IR' from dual union all
select 68322,'DMA' from dual union all
select 68323,'RVSNR' from dual union all
select 68324,'GSFC' from dual union all
select 68325,'GSFC' from dual union all
select 68326,'KRLP' from dual union all
select 68327,'ISFC' from dual union all
select 68328,'IDRDL' from dual union all
select 68329,'IDRDL' from dual union all
select 6833,'USN' from dual union all
select 68330,'NESCOMN' from dual union all
select 68331,'USN' from dual union all
select 68332,'USN' from dual union all
select 68333,'USN' from dual union all
select 68334,'USN' from dual union all
select 68335,'USN' from dual union all
select 68336,'IRSA' from dual union all
select 68337,'PLA2AC' from dual union all
select 68338,'USN' from dual union all
select 68339,'RVSNR' from dual union all
select 6834,'USAF' from dual union all
select 68340,'IR' from dual union all
select 68341,'ADD' from dual union all
select 68342,'NASA' from dual union all
select 68343,'ISRO' from dual union all
select 68344,'FKA' from dual union all
select 68345,'CALT' from dual union all
select 68346,'ULAB' from dual union all
select 68347,'MHI' from dual union all
select 68348,'ULAL' from dual union all
select 68349,'AE' from dual union all
select 6835,'USAF' from dual union all
select 68350,'FKA' from dual union all
select 68351,'ILSK' from dual union all
select 68352,'CALT' from dual union all
select 68353,'CALT' from dual union all
select 68354,'KTRAS' from dual union all
select 68354,'RVSNR' from dual union all
select 68355,'ISRO' from dual union all
select 68356,'USAF' from dual union all
select 68357,'OSC' from dual union all
select 68358,'AE' from dual union all
select 68359,'FKA' from dual union all
select 6836,'NASA' from dual union all
select 68360,'CALT' from dual union all
select 68361,'SAST' from dual union all
select 68362,'STSM' from dual union all
select 68363,'CALT' from dual union all
select 68364,'KVR' from dual union all
select 68365,'ULAB' from dual union all
select 68366,'NASA' from dual union all
select 68367,'ISA' from dual union all
select 68368,'KTRAS' from dual union all
select 68368,'RVSNR' from dual union all
select 68369,'ULAL' from dual union all
select 6837,'USAF' from dual union all
select 68370,'KTRAS' from dual union all
select 68370,'RVSNR' from dual union all
select 68371,'KVR' from dual union all
select 68372,'KVR' from dual union all
select 68373,'CALT' from dual union all
select 68374,'ILSK' from dual union all
select 68375,'FKA' from dual union all
select 68376,'ULAB' from dual union all
select 68377,'NASA' from dual union all
select 68378,'AE' from dual union all
select 68379,'ISRO' from dual union all
select 6838,'USA' from dual union all
select 68380,'KVR' from dual union all
select 68381,'MHI' from dual union all
select 68382,'FKA' from dual union all
select 68383,'ULAB' from dual;
insert into LAUNCH_AGENCY
select 68384,'SAST' from dual union all
select 68385,'ULAB' from dual union all
select 68386,'AE' from dual union all
select 68387,'FKA' from dual union all
select 68388,'ULAL' from dual union all
select 68389,'ULAB' from dual union all
select 6839,'NASA' from dual union all
select 68390,'STSM' from dual union all
select 68391,'KVR' from dual union all
select 68392,'NASA' from dual union all
select 68393,'CALT' from dual union all
select 68394,'KHRU' from dual union all
select 68395,'KVR' from dual union all
select 68396,'ULAB' from dual union all
select 68397,'SAST' from dual union all
select 68398,'AE' from dual union all
select 68399,'ILSK' from dual union all
select 684,'APL' from dual union all
select 6840,'GURVO' from dual union all
select 68400,'KHRU' from dual union all
select 68401,'ULAB' from dual union all
select 68402,'ULAL' from dual union all
select 68403,'STSM' from dual union all
select 68404,'ULAB' from dual union all
select 68405,'AE' from dual union all
select 68406,'FKA' from dual union all
select 68407,'KHRU' from dual union all
select 68408,'IAI' from dual union all
select 68409,'IAI' from dual union all
select 6841,'MVS' from dual union all
select 68410,'SMDC' from dual union all
select 68411,'ARR' from dual union all
select 68411,'NAMMO' from dual union all
select 68412,'USA' from dual union all
select 68413,'PRO' from dual union all
select 68414,'PRO' from dual union all
select 68415,'MDA' from dual union all
select 68416,'VMF' from dual union all
select 68417,'IDF' from dual union all
select 68418,'USN' from dual union all
select 68419,'IDRDL' from dual union all
select 6842,'USAF' from dual union all
select 68420,'IDRDL' from dual union all
select 68421,'KYSP' from dual union all
select 68422,'SEALP' from dual union all
select 68423,'SPX' from dual union all
select 68424,'ILSK' from dual union all
select 68425,'ERAU' from dual union all
select 68426,'USA' from dual union all
select 68427,'ARR' from dual union all
select 68428,'LAPAN' from dual union all
select 68429,'PLA2AC' from dual union all
select 6843,'RAE' from dual union all
select 68430,'NASA' from dual union all
select 68431,'GSFC' from dual union all
select 68432,'GSFC' from dual union all
select 68433,'GSFC' from dual union all
select 68434,'USA' from dual union all
select 68435,'USA' from dual union all
select 68436,'SYMC' from dual union all
select 68437,'AFSPC' from dual union all
select 68438,'NASA' from dual union all
select 68439,'GSFC' from dual union all
select 6844,'MVS' from dual union all
select 68440,'GSFC' from dual union all
select 68441,'GSFC' from dual union all
select 68442,'GSFC' from dual union all
select 68443,'NASA' from dual union all
select 68444,'MDA' from dual union all
select 68444,'OSCC' from dual union all
select 68445,'AFSPC' from dual union all
select 68446,'USA' from dual union all
select 68447,'UPAE' from dual union all
select 68448,'USN' from dual union all
select 68449,'USN' from dual union all
select 6845,'USAF' from dual union all
select 68450,'IDRDL' from dual union all
select 68451,'RVSNR' from dual union all
select 68452,'DSTO' from dual union all
select 68453,'DMA' from dual union all
select 68454,'GSFC' from dual union all
select 68455,'USN' from dual union all
select 68456,'USN' from dual union all
select 68457,'AEB' from dual union all
select 68458,'GSFC' from dual union all
select 68459,'ARR' from dual union all
select 68459,'DLR' from dual union all
select 6846,'ABMA' from dual union all
select 68460,'GSFC' from dual union all
select 68461,'GSFC' from dual union all
select 68462,'USAF' from dual union all
select 68463,'SAND' from dual union all
select 68463,'USN' from dual union all
select 68464,'SAND' from dual union all
select 68464,'USN' from dual union all
select 68465,'JAXA' from dual union all
select 68466,'USN' from dual union all
select 68467,'USN' from dual union all
select 68468,'VMFR' from dual union all
select 68469,'VMF' from dual;
insert into LAUNCH_AGENCY
select 6847,'USAF' from dual union all
select 68470,'SMDC' from dual union all
select 68471,'MDA' from dual union all
select 68472,'RVSNR' from dual union all
select 68473,'GSFC' from dual union all
select 68474,'NESCOMN' from dual union all
select 68475,'PK' from dual union all
select 68476,'IDRDL' from dual union all
select 68477,'ISFC' from dual union all
select 68478,'ISFC' from dual union all
select 68479,'IDRDL' from dual union all
select 6848,'MVS' from dual union all
select 68480,'USA' from dual union all
select 68481,'USA' from dual union all
select 68482,'RVSNR' from dual union all
select 68483,'GSFC' from dual union all
select 68484,'NAMFI' from dual union all
select 68485,'USN' from dual union all
select 68486,'USN' from dual union all
select 68487,'USN' from dual union all
select 68488,'USN' from dual union all
select 68489,'GSFC' from dual union all
select 6849,'ABMA' from dual union all
select 6849,'USAF' from dual union all
select 68490,'USN' from dual union all
select 68491,'AEB' from dual union all
select 68492,'VMFR' from dual union all
select 68493,'VMFR' from dual union all
select 68494,'RVSNR' from dual union all
select 68495,'RVSNR' from dual union all
select 68496,'USN' from dual union all
select 68497,'JMSDF' from dual union all
select 68498,'NASA' from dual union all
select 68499,'NASA' from dual union all
select 685,'USA' from dual union all
select 6850,'USAF' from dual union all
select 68500,'USN' from dual union all
select 68501,'NSPO' from dual union all
select 68502,'JAXA' from dual union all
select 68503,'TSAO' from dual union all
select 68504,'IR' from dual union all
select 68505,'IDRDL' from dual union all
select 68506,'USN' from dual union all
select 68507,'USN' from dual union all
select 68508,'USN' from dual union all
select 68509,'SEALP' from dual union all
select 6851,'USAF' from dual union all
select 68510,'ISRO' from dual union all
select 68511,'KHRU' from dual union all
select 68512,'FKA' from dual union all
select 68513,'NASA' from dual union all
select 68514,'ILSK' from dual union all
select 68515,'MHI' from dual union all
select 68516,'AE' from dual union all
select 68517,'NASA' from dual union all
select 68518,'ULAL' from dual union all
select 68519,'ILSK' from dual union all
select 6852,'WRE' from dual union all
select 68520,'ULAB' from dual union all
select 68521,'SEALP' from dual union all
select 68522,'KVR' from dual union all
select 68523,'FKA' from dual union all
select 68524,'ULAL' from dual union all
select 68525,'OSC' from dual union all
select 68526,'AE' from dual union all
select 68527,'CALT' from dual union all
select 68528,'STSM' from dual union all
select 68529,'ISRO' from dual union all
select 6853,'MVS' from dual union all
select 68530,'SIS' from dual union all
select 68531,'FKA' from dual union all
select 68532,'SEALP' from dual union all
select 68533,'KVR' from dual union all
select 68534,'SAST' from dual union all
select 68535,'NASA' from dual union all
select 68536,'CALT' from dual union all
select 68537,'ULAB' from dual union all
select 68538,'AE' from dual union all
select 68539,'KVR' from dual union all
select 6854,'USAF' from dual union all
select 68540,'ULAB' from dual union all
select 68541,'KHRU' from dual union all
select 68542,'AE' from dual union all
select 68543,'SEALP' from dual union all
select 68544,'KVR' from dual union all
select 68545,'KVR' from dual union all
select 68546,'AE' from dual union all
select 68547,'ILSK' from dual union all
select 68548,'KTRAS' from dual union all
select 68548,'RVSNR' from dual union all
select 68549,'CALT' from dual union all
select 6855,'USAF' from dual union all
select 68550,'ULAB' from dual union all
select 68551,'FKA' from dual union all
select 68552,'ILSK' from dual union all
select 68553,'SEALP' from dual union all
select 68554,'KHRU' from dual union all
select 68555,'CALT' from dual union all
select 68556,'SPX' from dual union all
select 68557,'KTRAS' from dual;
insert into LAUNCH_AGENCY
select 68557,'RVSNR' from dual union all
select 68558,'FKA' from dual union all
select 68559,'OSC' from dual union all
select 6856,'USN' from dual union all
select 68560,'ISRO' from dual union all
select 68561,'SAST' from dual union all
select 68562,'ULAB' from dual union all
select 68563,'CALT' from dual union all
select 68564,'SAST' from dual union all
select 68565,'ILSK' from dual union all
select 68566,'KVR' from dual union all
select 68567,'NASA' from dual union all
select 68568,'FKA' from dual union all
select 68569,'SAST' from dual union all
select 6857,'MVS' from dual union all
select 68570,'KVR' from dual union all
select 68571,'ILSK' from dual union all
select 68572,'SAST' from dual union all
select 68573,'AE' from dual union all
select 68574,'CALT' from dual union all
select 68575,'KHRU' from dual union all
select 68576,'WFF' from dual union all
select 68577,'LAPAN' from dual union all
select 68578,'ISFC' from dual union all
select 68579,'USN' from dual union all
select 6858,'MVS' from dual union all
select 68580,'USN' from dual union all
select 68581,'NASA' from dual union all
select 68582,'USA' from dual union all
select 68583,'TSAO' from dual union all
select 68584,'LAPAN' from dual union all
select 68585,'SPX' from dual union all
select 68586,'IRSA' from dual union all
select 68587,'ARR' from dual union all
select 68588,'USA' from dual union all
select 68589,'GSFC' from dual union all
select 6859,'RVSN' from dual union all
select 68590,'USN' from dual union all
select 68591,'USN' from dual union all
select 68592,'USN' from dual union all
select 68593,'USN' from dual union all
select 68594,'USN' from dual union all
select 68595,'PKAFSC' from dual union all
select 68596,'RVSN' from dual union all
select 68597,'GSFC' from dual union all
select 68598,'IDF' from dual union all
select 68599,'NASA' from dual union all
select 686,'USA' from dual union all
select 6860,'NASA' from dual union all
select 68600,'PKAFSC' from dual union all
select 68601,'ARR' from dual union all
select 68602,'JAXA' from dual union all
select 68603,'DLR' from dual union all
select 68603,'SSC' from dual union all
select 68604,'DLR' from dual union all
select 68604,'SSC' from dual union all
select 68605,'ABMDA' from dual union all
select 68605,'USN' from dual union all
select 68606,'DRDO' from dual union all
select 68607,'ISFC' from dual union all
select 68608,'USAF' from dual union all
select 68609,'GSFC' from dual union all
select 6861,'RVSN' from dual union all
select 68610,'IDF' from dual union all
select 68611,'NESCOMN' from dual union all
select 68612,'PKAFSC' from dual union all
select 68613,'GSFC' from dual union all
select 68614,'IDRDL' from dual union all
select 68615,'WFF' from dual union all
select 68616,'USN' from dual union all
select 68617,'SSC' from dual union all
select 68618,'USN' from dual union all
select 68619,'USAF' from dual union all
select 6862,'RVSN' from dual union all
select 68620,'PLA2AC' from dual union all
select 68621,'USN' from dual union all
select 68622,'USN' from dual union all
select 68623,'USN' from dual union all
select 68624,'MDA' from dual union all
select 68625,'NASA' from dual union all
select 68626,'ARR' from dual union all
select 68626,'DLR' from dual union all
select 68627,'IR' from dual union all
select 68628,'ARR' from dual union all
select 68628,'DLR' from dual union all
select 68629,'ARR' from dual union all
select 68629,'DLR' from dual union all
select 6863,'RVSN' from dual union all
select 68630,'GSFC' from dual union all
select 68631,'VMFR' from dual union all
select 68632,'JAXA' from dual union all
select 68633,'USAF' from dual union all
select 68634,'USN' from dual union all
select 68635,'USN' from dual union all
select 68636,'RVSNR' from dual union all
select 68637,'VMF' from dual union all
select 68638,'USA' from dual union all
select 68639,'USAF' from dual union all
select 6864,'GURVO' from dual union all
select 68640,'VMFR' from dual;
insert into LAUNCH_AGENCY
select 68641,'RVSNR' from dual union all
select 68642,'VMFR' from dual union all
select 68643,'VMFR' from dual union all
select 68644,'GSFC' from dual union all
select 68645,'RVSNR' from dual union all
select 68646,'AEB' from dual union all
select 68646,'DLR' from dual union all
select 68647,'EUROL' from dual union all
select 68648,'USA' from dual union all
select 68649,'USA' from dual union all
select 6865,'USAF' from dual union all
select 68650,'USA' from dual union all
select 68651,'ALCS' from dual union all
select 68651,'USAF' from dual union all
select 68652,'DRDO' from dual union all
select 68653,'IR' from dual union all
select 68654,'DMA' from dual union all
select 68655,'TSAO' from dual union all
select 68656,'USA' from dual union all
select 68657,'JMSDF' from dual union all
select 68658,'IRSA' from dual union all
select 68659,'RVSNR' from dual union all
select 6866,'USAF' from dual union all
select 68660,'ARR' from dual union all
select 68660,'FORSK' from dual union all
select 68661,'SMDC' from dual union all
select 68662,'MDA' from dual union all
select 68663,'VMF' from dual union all
select 68664,'SPFL' from dual union all
select 68665,'VMF' from dual union all
select 68666,'USN' from dual union all
select 68667,'NAMFI' from dual union all
select 68668,'NAMFI' from dual union all
select 68669,'NAMFI' from dual union all
select 6867,'BRL' from dual union all
select 6867,'UMI' from dual union all
select 68670,'NAMFI' from dual union all
select 68671,'NAMFI' from dual union all
select 68672,'NAMFI' from dual union all
select 68673,'NAMFI' from dual union all
select 68674,'NAMFI' from dual union all
select 68675,'NAMFI' from dual union all
select 68676,'NAMFI' from dual union all
select 68677,'SSC' from dual union all
select 68678,'PKAFSC' from dual union all
select 68679,'SMDC' from dual union all
select 6868,'NRL' from dual union all
select 68680,'IRSA' from dual union all
select 68681,'ISRO' from dual union all
select 68682,'ULAB' from dual union all
select 68683,'MHI' from dual union all
select 68684,'KVR' from dual union all
select 68685,'IRSA' from dual union all
select 68686,'ULAB' from dual union all
select 68687,'FKA' from dual union all
select 68688,'KHRU' from dual union all
select 68689,'AE' from dual union all
select 6869,'NASA' from dual union all
select 68690,'SIS' from dual union all
select 68691,'KHRU' from dual union all
select 68692,'ULAB' from dual union all
select 68693,'NASA' from dual union all
select 68694,'KVR' from dual union all
select 68695,'ULAB' from dual union all
select 68696,'FKA' from dual union all
select 68697,'ILSK' from dual union all
select 68698,'ULAL' from dual union all
select 68699,'CALT' from dual union all
select 687,'USA' from dual union all
select 6870,'USN' from dual union all
select 68700,'ISRO' from dual union all
select 68701,'SEALP' from dual union all
select 68702,'CALT' from dual union all
select 68703,'KVR' from dual union all
select 68704,'ULAB' from dual union all
select 68705,'FKA' from dual union all
select 68706,'NASA' from dual union all
select 68707,'AE' from dual union all
select 68708,'ILSK' from dual union all
select 68709,'USAF' from dual union all
select 6871,'SAC' from dual union all
select 68710,'KVR' from dual union all
select 68711,'FKA' from dual union all
select 68712,'ULAL' from dual union all
select 68713,'SIS' from dual union all
select 68714,'ULAB' from dual union all
select 68715,'ILSK' from dual union all
select 68716,'AE' from dual union all
select 68717,'KVR' from dual union all
select 68718,'SPX' from dual union all
select 68719,'NASA' from dual union all
select 6872,'MVS' from dual union all
select 68720,'KVR' from dual union all
select 68721,'FKA' from dual union all
select 68722,'KTRAS' from dual union all
select 68722,'RVSNR' from dual union all
select 68723,'ILSK' from dual union all
select 68724,'ULAB' from dual union all
select 68725,'AE' from dual union all
select 68726,'NASA' from dual;
insert into LAUNCH_AGENCY
select 68727,'CALT' from dual union all
select 68728,'ULAL' from dual union all
select 68729,'JAXA' from dual union all
select 6873,'MVS' from dual union all
select 68730,'FKA' from dual union all
select 68731,'ILSK' from dual union all
select 68732,'ISRO' from dual union all
select 68733,'ULAB' from dual union all
select 68734,'FKA' from dual union all
select 68735,'AE' from dual union all
select 68736,'ULAB' from dual union all
select 68737,'FKA' from dual union all
select 68738,'ULAL' from dual union all
select 68739,'AE' from dual union all
select 6874,'USAF' from dual union all
select 68740,'KVR' from dual union all
select 68741,'FKA' from dual union all
select 68742,'CALT' from dual union all
select 68743,'NASA' from dual union all
select 68744,'KVR' from dual union all
select 68745,'ULAL' from dual union all
select 68746,'ILSK' from dual union all
select 68747,'MHI' from dual union all
select 68748,'SIS' from dual union all
select 68749,'ULAB' from dual union all
select 6875,'MVS' from dual union all
select 68750,'SAST' from dual union all
select 68751,'KHRU' from dual union all
select 68752,'ULAB' from dual union all
select 68753,'SAST' from dual union all
select 68754,'AE' from dual union all
select 68755,'FKA' from dual union all
select 68756,'ILSK' from dual union all
select 68757,'IAI' from dual union all
select 68758,'USN' from dual union all
select 68759,'UPAE' from dual union all
select 6876,'USAF' from dual union all
select 68760,'USA' from dual union all
select 68761,'VMF' from dual union all
select 68762,'NASA' from dual union all
select 68763,'PRO' from dual union all
select 68764,'ISFC' from dual union all
select 68765,'MDA' from dual union all
select 68766,'CITED' from dual union all
select 68767,'AEB' from dual union all
select 68768,'AEB' from dual union all
select 68769,'AEB' from dual union all
select 6877,'USAF' from dual union all
select 68770,'AEB' from dual union all
select 68771,'OSC' from dual union all
select 68772,'KCST' from dual union all
select 68773,'KARI' from dual union all
select 68774,'IDRDL' from dual union all
select 68775,'USN' from dual union all
select 68776,'ISFC' from dual union all
select 68777,'LAPAN' from dual union all
select 68778,'USA' from dual union all
select 68779,'ISFC' from dual union all
select 6878,'USAF' from dual union all
select 68780,'ISFC' from dual union all
select 68781,'IDRDL' from dual union all
select 68782,'GSFC' from dual union all
select 68783,'JAXA' from dual union all
select 68784,'GSFC' from dual union all
select 68785,'GSFC' from dual union all
select 68786,'GSFC' from dual union all
select 68787,'GSFC' from dual union all
select 68788,'GSFC' from dual union all
select 68789,'GSFC' from dual union all
select 6879,'USAF' from dual union all
select 68790,'USN' from dual union all
select 68791,'USN' from dual union all
select 68792,'GSFC' from dual union all
select 68793,'IDRDL' from dual union all
select 68794,'EUROL' from dual union all
select 68795,'EUROL' from dual union all
select 68796,'USA' from dual union all
select 68797,'NASA' from dual union all
select 68798,'COLE' from dual union all
select 68798,'MDA' from dual union all
select 68799,'RVSNR' from dual union all
select 688,'GAU4' from dual union all
select 6880,'MVS' from dual union all
select 68800,'USAFA' from dual union all
select 68801,'IR' from dual union all
select 68802,'DSTO' from dual union all
select 68803,'DLR' from dual union all
select 68803,'SSC' from dual union all
select 68804,'GSFC' from dual union all
select 68805,'AEB' from dual union all
select 68805,'DLR' from dual union all
select 68806,'ISFC' from dual union all
select 68807,'MDA' from dual union all
select 68808,'MDA' from dual union all
select 68809,'GSFC' from dual union all
select 6881,'USA' from dual union all
select 68810,'ALCS' from dual union all
select 68810,'USAF' from dual union all
select 68811,'GSFC' from dual union all
select 68812,'VMFR' from dual;
insert into LAUNCH_AGENCY
select 68813,'VMFR' from dual union all
select 68814,'COLE' from dual union all
select 68814,'MDA' from dual union all
select 68815,'USN' from dual union all
select 68816,'USA' from dual union all
select 68817,'USA' from dual union all
select 68818,'USA' from dual union all
select 68819,'MDA' from dual union all
select 6882,'USN' from dual union all
select 68820,'WFF' from dual union all
select 68821,'GSFC' from dual union all
select 68822,'USAF' from dual union all
select 68823,'GSFC' from dual union all
select 68824,'NASA' from dual union all
select 68825,'IR' from dual union all
select 68826,'IR' from dual union all
select 68827,'IR' from dual union all
select 68828,'IR' from dual union all
select 68829,'VMFR' from dual union all
select 6883,'USAF' from dual union all
select 68830,'VMFR' from dual union all
select 68831,'USN' from dual union all
select 68832,'USN' from dual union all
select 68833,'VMFR' from dual union all
select 68834,'MDA' from dual union all
select 68835,'USA' from dual union all
select 68836,'JMSDF' from dual union all
select 68837,'USA' from dual union all
select 68838,'USA' from dual union all
select 68839,'USA' from dual union all
select 6884,'USAF' from dual union all
select 68840,'GSFC' from dual union all
select 68841,'DLR' from dual union all
select 68841,'SSC' from dual union all
select 68842,'DLR' from dual union all
select 68842,'SSC' from dual union all
select 68843,'RLABN' from dual union all
select 68844,'VMF' from dual union all
select 68845,'RVSNR' from dual union all
select 68846,'MDA' from dual union all
select 68847,'IR' from dual union all
select 68848,'WFF' from dual union all
select 68849,'GSFC' from dual union all
select 6885,'RAE' from dual union all
select 6885,'WRE' from dual union all
select 68850,'RVSNR' from dual union all
select 68851,'USA' from dual union all
select 68852,'USA' from dual union all
select 68853,'WFF' from dual union all
select 68854,'KPAMJ' from dual union all
select 68855,'KPAMJ' from dual union all
select 68856,'KPAMJ' from dual union all
select 68857,'KPAMJ' from dual union all
select 68858,'KPAMJ' from dual union all
select 68859,'KPAMJ' from dual union all
select 6886,'MVS' from dual union all
select 68860,'KPAMJ' from dual union all
select 68861,'CALT' from dual union all
select 68862,'KHRU' from dual union all
select 68863,'FKA' from dual union all
select 68864,'NASA' from dual union all
select 68865,'ULAL' from dual union all
select 68866,'ILSK' from dual union all
select 68867,'KHRU' from dual union all
select 68868,'ULAB' from dual union all
select 68869,'SAST' from dual union all
select 6887,'AFCRL' from dual union all
select 68870,'ILSK' from dual union all
select 68871,'FKA' from dual union all
select 68872,'NASA' from dual union all
select 68873,'KTRAS' from dual union all
select 68873,'RVSNR' from dual union all
select 68874,'KVR' from dual union all
select 68875,'ULAL' from dual union all
select 68876,'ILSK' from dual union all
select 68877,'KVR' from dual union all
select 68878,'FKA' from dual union all
select 68879,'NASA' from dual union all
select 6888,'USN' from dual union all
select 68880,'MHI' from dual union all
select 68881,'AE' from dual union all
select 68882,'ULAB' from dual union all
select 68883,'KVR' from dual union all
select 68884,'CALT' from dual union all
select 68885,'ILSK' from dual union all
select 68886,'SPX' from dual union all
select 68887,'SAST' from dual union all
select 68888,'KTRAS' from dual union all
select 68888,'RVSNR' from dual union all
select 68889,'FKA' from dual union all
select 6889,'MVS' from dual union all
select 68890,'KTRAS' from dual union all
select 68890,'RVSNR' from dual union all
select 68891,'ISA' from dual union all
select 68892,'AE' from dual union all
select 68893,'FKA' from dual union all
select 68894,'ILSK' from dual union all
select 68895,'ISRO' from dual union all
select 68896,'CALT' from dual union all
select 68897,'AE' from dual;
insert into LAUNCH_AGENCY
select 68898,'SAST' from dual union all
select 68899,'ULAL' from dual union all
select 689,'USA' from dual union all
select 6890,'USAF' from dual union all
select 68900,'SAST' from dual union all
select 68901,'KHRU' from dual union all
select 68902,'CALT' from dual union all
select 68903,'KVR' from dual union all
select 68904,'FKA' from dual union all
select 68905,'MHI' from dual union all
select 68906,'ULAL' from dual union all
select 68907,'SAST' from dual union all
select 68908,'USAF' from dual union all
select 68909,'KVR' from dual union all
select 6891,'USAF' from dual union all
select 68910,'CALT' from dual union all
select 68911,'SAST' from dual union all
select 68912,'FKA' from dual union all
select 68913,'ILSK' from dual union all
select 68914,'FKA' from dual union all
select 68915,'FKA' from dual union all
select 68916,'AE' from dual union all
select 68917,'CALT' from dual union all
select 68918,'KVR' from dual union all
select 68919,'SAST' from dual union all
select 6892,'USAF' from dual union all
select 68920,'ULAB' from dual union all
select 68921,'ILSK' from dual union all
select 68922,'USAF' from dual union all
select 68923,'ULAB' from dual union all
select 68924,'CALT' from dual union all
select 68925,'AE' from dual union all
select 68926,'SPX' from dual union all
select 68927,'FKA' from dual union all
select 68928,'CALT' from dual union all
select 68929,'ILSK' from dual union all
select 6893,'USAF' from dual union all
select 68930,'AE' from dual union all
select 68931,'ISRO' from dual union all
select 68932,'AFRLRZ' from dual union all
select 68933,'USA' from dual union all
select 68934,'IDRDL' from dual union all
select 68935,'IDRDL' from dual union all
select 68936,'SCAL' from dual union all
select 68937,'SCAL' from dual union all
select 68938,'SCAL' from dual union all
select 68939,'SCAL' from dual union all
select 6894,'USAF' from dual union all
select 68940,'SCAL' from dual union all
select 68941,'SCAL' from dual union all
select 68942,'SCAL' from dual union all
select 68943,'SCAL' from dual union all
select 68944,'SCAL' from dual union all
select 68945,'AEB' from dual union all
select 68946,'AEB' from dual union all
select 68947,'AEB' from dual union all
select 68948,'AEB' from dual union all
select 68949,'AEB' from dual union all
select 6895,'USAF' from dual union all
select 68950,'AEB' from dual union all
select 68951,'AEB' from dual union all
select 68952,'NASA' from dual union all
select 68953,'ISRO' from dual union all
select 68954,'KARI' from dual union all
select 68955,'KHRU' from dual union all
select 68956,'ISRO' from dual union all
select 68957,'ISRO' from dual union all
select 68958,'ISRO' from dual union all
select 68959,'ISRO' from dual union all
select 6896,'USN' from dual union all
select 68960,'ISRO' from dual union all
select 68961,'IDRDL' from dual union all
select 68962,'ISRO' from dual union all
select 68963,'ISFC' from dual union all
select 68964,'ISFC' from dual union all
select 68965,'PKAFSC' from dual union all
select 68966,'PKAFSC' from dual union all
select 68967,'IR' from dual union all
select 68968,'PLA2AC' from dual union all
select 68969,'CHFEN' from dual union all
select 68969,'PLA2AC' from dual union all
select 6897,'USAF' from dual union all
select 68970,'ISRO' from dual union all
select 68971,'ISRO' from dual union all
select 68972,'ISRO' from dual union all
select 68973,'ISRO' from dual union all
select 68974,'ISRO' from dual union all
select 68975,'ISRO' from dual union all
select 68976,'ISRO' from dual union all
select 68977,'MDA' from dual union all
select 68978,'DMA' from dual union all
select 68979,'MDA' from dual union all
select 6898,'USAF' from dual union all
select 68980,'MDA' from dual union all
select 68981,'IRSA' from dual union all
select 68982,'IDRDL' from dual union all
select 68983,'GSFC' from dual union all
select 68984,'MDA' from dual union all
select 68985,'MDA' from dual union all
select 68986,'NASA' from dual;
insert into LAUNCH_AGENCY
select 68987,'MDA' from dual union all
select 68988,'VMFR' from dual union all
select 68989,'EUROL' from dual union all
select 6899,'USAF' from dual union all
select 68990,'EUROL' from dual union all
select 68991,'ESA' from dual union all
select 68991,'SSC' from dual union all
select 68992,'IDRDL' from dual union all
select 68993,'ISFC' from dual union all
select 68994,'GSFC' from dual union all
select 68995,'ISFC' from dual union all
select 68996,'DSTO' from dual union all
select 68997,'USAF' from dual union all
select 68998,'GSFC' from dual union all
select 68999,'UPAE' from dual union all
select 69,'WEHR' from dual union all
select 690,'USA' from dual union all
select 6900,'USAF' from dual union all
select 69000,'NSPO' from dual union all
select 69001,'ISFC' from dual union all
select 69002,'MDA' from dual union all
select 69003,'GSFC' from dual union all
select 69004,'MDA' from dual union all
select 69005,'USN' from dual union all
select 69006,'USN' from dual union all
select 69007,'USN' from dual union all
select 69008,'USN' from dual union all
select 69009,'USAF' from dual union all
select 6901,'USAF' from dual union all
select 69010,'GSFC' from dual union all
select 69011,'USAF' from dual union all
select 69012,'GSFC' from dual union all
select 69013,'MDA' from dual union all
select 69014,'DMA' from dual union all
select 69015,'IDRDL' from dual union all
select 69016,'GSFC' from dual union all
select 69017,'GSFC' from dual union all
select 69018,'VMFR' from dual union all
select 69019,'VMFR' from dual union all
select 6902,'MVS' from dual union all
select 69020,'GSFC' from dual union all
select 69021,'IR' from dual union all
select 69022,'JAXA' from dual union all
select 69023,'USAF' from dual union all
select 69024,'MDA' from dual union all
select 69025,'GSFC' from dual union all
select 69026,'VMF' from dual union all
select 69027,'DLR' from dual union all
select 69027,'SSC' from dual union all
select 69028,'MDA' from dual union all
select 69029,'RVSNR' from dual union all
select 6903,'NASA' from dual union all
select 69030,'VMFR' from dual union all
select 69031,'VMFR' from dual union all
select 69032,'VMF' from dual union all
select 69033,'USA' from dual union all
select 69034,'JMSDF' from dual union all
select 69035,'RVSNR' from dual union all
select 69036,'GSFC' from dual union all
select 69037,'ISFC' from dual union all
select 69038,'AEB' from dual union all
select 69039,'AEB' from dual union all
select 6904,'MVS' from dual union all
select 69040,'NASA' from dual union all
select 69041,'ARR' from dual union all
select 69041,'DLR' from dual union all
select 69042,'ARR' from dual union all
select 69042,'DLR' from dual union all
select 69043,'MDA' from dual union all
select 69044,'MDA' from dual union all
select 69045,'ARR' from dual union all
select 69045,'DLR' from dual union all
select 69046,'PKAFSC' from dual union all
select 69047,'MDA' from dual union all
select 69048,'IR' from dual union all
select 69049,'USN' from dual union all
select 6905,'USAF' from dual union all
select 69050,'USN' from dual union all
select 69051,'USN' from dual union all
select 69052,'ISFC' from dual union all
select 69053,'KVR' from dual union all
select 69054,'ULAB' from dual union all
select 69055,'JAXA' from dual union all
select 69056,'FKA' from dual union all
select 69057,'KVR' from dual union all
select 69058,'USAF' from dual union all
select 69059,'AE' from dual union all
select 6906,'USAF' from dual union all
select 69060,'NASA' from dual union all
select 69061,'KVR' from dual union all
select 69062,'ULAL' from dual union all
select 69063,'ULAB' from dual union all
select 69064,'FKA' from dual union all
select 69065,'CALT' from dual union all
select 69066,'ULAL' from dual union all
select 69067,'ISRO' from dual union all
select 69068,'AE' from dual union all
select 69069,'FKA' from dual union all
select 6907,'USAF' from dual union all
select 69070,'KVR' from dual;
insert into LAUNCH_AGENCY
select 69071,'ULAL' from dual union all
select 69072,'NASA' from dual union all
select 69073,'ILSK' from dual union all
select 69074,'AE' from dual union all
select 69075,'FKA' from dual union all
select 69076,'ULAB' from dual union all
select 69077,'IRSA' from dual union all
select 69078,'CALT' from dual union all
select 69079,'FKA' from dual union all
select 6908,'USAF' from dual union all
select 69080,'KVR' from dual union all
select 69081,'USAF' from dual union all
select 69082,'CALT' from dual union all
select 69083,'NASA' from dual union all
select 69084,'CALT' from dual union all
select 69085,'FKA' from dual union all
select 69086,'ISRO' from dual union all
select 69087,'ILSK' from dual union all
select 69088,'ULAB' from dual union all
select 69089,'KVR' from dual union all
select 6909,'RAE' from dual union all
select 6909,'WRE' from dual union all
select 69090,'CALT' from dual union all
select 69091,'CALT' from dual union all
select 69092,'ULAL' from dual union all
select 69093,'AE' from dual union all
select 69094,'CALT' from dual union all
select 69095,'SAST' from dual union all
select 69096,'KTRAS' from dual union all
select 69096,'RVSNR' from dual union all
select 69097,'KHRU' from dual union all
select 69098,'ULAB' from dual union all
select 69099,'CALT' from dual union all
select 691,'APL' from dual union all
select 6910,'NASA' from dual union all
select 69100,'KHRU' from dual union all
select 69101,'AE' from dual union all
select 69102,'MHI' from dual union all
select 69103,'ELUS' from dual union all
select 69104,'USAF' from dual union all
select 69105,'CALT' from dual union all
select 69106,'ILSK' from dual union all
select 69107,'KVR' from dual union all
select 69108,'ELUS' from dual union all
select 69108,'SIS' from dual union all
select 69109,'CALT' from dual union all
select 6911,'NASA' from dual union all
select 69110,'ISRO' from dual union all
select 69111,'ILSK' from dual union all
select 69112,'AE' from dual union all
select 69113,'ULAB' from dual union all
select 69114,'FKA' from dual union all
select 69115,'CALT' from dual union all
select 69116,'KHRU' from dual union all
select 69117,'RKA' from dual union all
select 69118,'SAST' from dual union all
select 69119,'FKA' from dual union all
select 6912,'USAF' from dual union all
select 69120,'SAST' from dual union all
select 69121,'ILSK' from dual union all
select 69122,'ULAL' from dual union all
select 69123,'VVKO' from dual union all
select 69124,'CALT' from dual union all
select 69125,'CALT' from dual union all
select 69126,'ILSK' from dual union all
select 69127,'MHI' from dual union all
select 69128,'AE' from dual union all
select 69129,'CALT' from dual union all
select 6913,'USAF' from dual union all
select 69130,'FKA' from dual union all
select 69131,'SAST' from dual union all
select 69132,'FKA' from dual union all
select 69133,'IAI' from dual union all
select 69134,'IAI' from dual union all
select 69135,'IDRDL' from dual union all
select 69136,'IDRDL' from dual union all
select 69137,'PRO' from dual union all
select 69138,'SCAL' from dual union all
select 69139,'SCAL' from dual union all
select 6914,'USAF' from dual union all
select 69140,'SCAL' from dual union all
select 69141,'SCAL' from dual union all
select 69142,'SCAL' from dual union all
select 69143,'SCAL' from dual union all
select 69144,'SCAL' from dual union all
select 69145,'SCAL' from dual union all
select 69146,'SCAL' from dual union all
select 69147,'SCAL' from dual union all
select 69148,'SCAL' from dual union all
select 69149,'SCAL' from dual union all
select 6915,'USAF' from dual union all
select 69150,'SCAL' from dual union all
select 69151,'SCAL' from dual union all
select 69152,'SCAL' from dual union all
select 69153,'SCAL' from dual union all
select 69154,'AFRLRZ' from dual union all
select 69155,'GSFC' from dual union all
select 69156,'DRDO' from dual union all
select 69157,'RVSNR' from dual union all
select 69158,'IRSA' from dual;
insert into LAUNCH_AGENCY
select 69159,'ARMAD' from dual union all
select 6916,'RVSN' from dual union all
select 69160,'SCAL' from dual union all
select 69161,'SCAL' from dual union all
select 69162,'SCAL' from dual union all
select 69163,'AEB' from dual union all
select 69164,'AEB' from dual union all
select 69165,'AEB' from dual union all
select 69166,'AEB' from dual union all
select 69167,'AEB' from dual union all
select 69168,'IDF' from dual union all
select 69169,'DGA' from dual union all
select 69169,'IDF' from dual union all
select 6917,'RVSN' from dual union all
select 69170,'USN' from dual union all
select 69171,'OSC' from dual union all
select 69172,'CALT' from dual union all
select 69173,'FKA' from dual union all
select 69174,'VVKO' from dual union all
select 69175,'IDRDL' from dual union all
select 69176,'ISFC' from dual union all
select 69177,'ISRO' from dual union all
select 69178,'IDRDL' from dual union all
select 69179,'IDRDL' from dual union all
select 6918,'GB5' from dual union all
select 69180,'AEB' from dual union all
select 69181,'AEB' from dual union all
select 69182,'AEB' from dual union all
select 69183,'USN' from dual union all
select 69184,'GSFC' from dual union all
select 69185,'GSFC' from dual union all
select 69186,'EUROL' from dual union all
select 69187,'EUROL' from dual union all
select 69188,'MDA' from dual union all
select 69189,'USN' from dual union all
select 6919,'USN' from dual union all
select 69190,'IRSA' from dual union all
select 69191,'MDA' from dual union all
select 69192,'DLR' from dual union all
select 69192,'SSC' from dual union all
select 69193,'GSFC' from dual union all
select 69194,'MDA' from dual union all
select 69195,'USN' from dual union all
select 69196,'VMFR' from dual union all
select 69197,'GSFC' from dual union all
select 69198,'CSSAR' from dual union all
select 69199,'VMFR' from dual union all
select 692,'GAU4' from dual union all
select 6920,'NASA' from dual union all
select 69200,'UPAE' from dual union all
select 69201,'GSFC' from dual union all
select 69202,'IR' from dual union all
select 69203,'IR' from dual union all
select 69204,'GSFC' from dual union all
select 69205,'USAF' from dual union all
select 69206,'IR' from dual union all
select 69207,'IR' from dual union all
select 69208,'IR' from dual union all
select 69209,'VMF' from dual union all
select 6921,'NASA' from dual union all
select 69210,'NASA' from dual union all
select 69211,'GSFC' from dual union all
select 69212,'CITED' from dual union all
select 69213,'COLE' from dual union all
select 69213,'MDA' from dual union all
select 69214,'GSFC' from dual union all
select 69215,'USAF' from dual union all
select 69216,'VMFR' from dual union all
select 69217,'MDA' from dual union all
select 69218,'USAF' from dual union all
select 69219,'VMF' from dual union all
select 6922,'MVS' from dual union all
select 69220,'MDA' from dual union all
select 69221,'USN' from dual union all
select 69222,'RVSNR' from dual union all
select 69223,'SSC' from dual union all
select 69224,'ISFC' from dual union all
select 69225,'VMFR' from dual union all
select 69226,'ISFC' from dual union all
select 69227,'COLE' from dual union all
select 69227,'MDA' from dual union all
select 69228,'MDA' from dual union all
select 69229,'MDA' from dual union all
select 6923,'VMF' from dual union all
select 69230,'MDA' from dual union all
select 69231,'GSFC' from dual union all
select 69232,'GSFC' from dual union all
select 69233,'IDF' from dual union all
select 69234,'VMF' from dual union all
select 69235,'RVSNR' from dual union all
select 69236,'GSFC' from dual union all
select 69237,'IDRDL' from dual union all
select 69238,'GSFC' from dual union all
select 69239,'SMDC' from dual union all
select 6924,'SAC' from dual union all
select 69240,'IR' from dual union all
select 69241,'IR' from dual union all
select 69242,'DLR' from dual union all
select 69242,'SSC' from dual union all
select 69243,'AEB' from dual;
insert into LAUNCH_AGENCY
select 69244,'AEB' from dual union all
select 69245,'AEB' from dual union all
select 69246,'ARR' from dual union all
select 69246,'FORSK' from dual union all
select 69247,'RGMS' from dual union all
select 69248,'VMF' from dual union all
select 69249,'VMF' from dual union all
select 6925,'USN' from dual union all
select 69250,'RVSNR' from dual union all
select 69251,'JAXA' from dual union all
select 69252,'GSFC' from dual union all
select 69253,'USN' from dual union all
select 69254,'ISFC' from dual union all
select 69255,'USN' from dual union all
select 69256,'SAST' from dual union all
select 69257,'CALT' from dual union all
select 69258,'ULAB' from dual union all
select 69259,'FKA' from dual union all
select 6926,'ATEF' from dual union all
select 69260,'IRSA' from dual union all
select 69261,'ESA' from dual union all
select 69262,'ILSK' from dual union all
select 69263,'CALT' from dual union all
select 69264,'ULAL' from dual union all
select 69265,'AE' from dual union all
select 69266,'ILSK' from dual union all
select 69267,'KHRU' from dual union all
select 69268,'CALT' from dual union all
select 69269,'ULAB' from dual union all
select 6927,'MVS' from dual union all
select 69270,'FKA' from dual union all
select 69271,'ILSK' from dual union all
select 69272,'ISRO' from dual union all
select 69273,'CALT' from dual union all
select 69274,'ULAL' from dual union all
select 69275,'SAST' from dual union all
select 69276,'SAST' from dual union all
select 69277,'FKA' from dual union all
select 69278,'AE' from dual union all
select 69279,'VVKO' from dual union all
select 6928,'USAF' from dual union all
select 69280,'MHI' from dual union all
select 69281,'ILSK' from dual union all
select 69282,'SPX' from dual union all
select 69283,'CALT' from dual union all
select 69284,'SAST' from dual union all
select 69285,'ELUS' from dual union all
select 69286,'OSC' from dual union all
select 69287,'CALT' from dual union all
select 69288,'ULAL' from dual union all
select 69289,'ULAB' from dual union all
select 6929,'AN' from dual union all
select 69290,'AE' from dual union all
select 69291,'ILSK' from dual union all
select 69292,'FKA' from dual union all
select 69293,'JAXA' from dual union all
select 69294,'FKA' from dual union all
select 69295,'CALT' from dual union all
select 69296,'KVR' from dual union all
select 69297,'FKA' from dual union all
select 69298,'AE' from dual union all
select 69299,'KHRU' from dual union all
select 693,'GAU4' from dual union all
select 6930,'MVS' from dual union all
select 69300,'ELUS' from dual union all
select 69301,'ULAL' from dual union all
select 69302,'ISRO' from dual union all
select 69303,'ULAL' from dual union all
select 69304,'FKA' from dual union all
select 69305,'CALT' from dual union all
select 69306,'AE' from dual union all
select 69307,'SAST' from dual union all
select 69308,'ULAB' from dual union all
select 69309,'SPX' from dual union all
select 6931,'USAF' from dual union all
select 69310,'AE' from dual union all
select 69311,'CALT' from dual union all
select 69312,'ILSK' from dual union all
select 69313,'FKA' from dual union all
select 69314,'CALT' from dual union all
select 69315,'FKA' from dual union all
select 69316,'KHRU' from dual union all
select 69317,'AE' from dual union all
select 69318,'VVKO' from dual union all
select 69319,'CALT' from dual union all
select 6932,'NASA' from dual union all
select 69320,'ILSK' from dual union all
select 69321,'SAST' from dual union all
select 69322,'CALT' from dual union all
select 69323,'AE' from dual union all
select 69324,'ELUS' from dual union all
select 69325,'ILSK' from dual union all
select 69326,'ULAL' from dual union all
select 69327,'KCST' from dual union all
select 69328,'SAST' from dual union all
select 69329,'FKA' from dual union all
select 6933,'USAF' from dual union all
select 69330,'AE' from dual union all
select 69331,'IDRDL' from dual union all
select 69332,'AFRL' from dual;
insert into LAUNCH_AGENCY
select 69333,'SCAL' from dual union all
select 69334,'SCAL' from dual union all
select 69335,'SCAL' from dual union all
select 69336,'SCAL' from dual union all
select 69337,'AFRLRZ' from dual union all
select 69338,'SCAL' from dual union all
select 69339,'SCAL' from dual union all
select 6934,'USAF' from dual union all
select 69340,'SPX' from dual union all
select 69341,'AEB' from dual union all
select 69342,'AEB' from dual union all
select 69343,'AEB' from dual union all
select 69344,'AEB' from dual union all
select 69345,'AEB' from dual union all
select 69346,'AEB' from dual union all
select 69347,'AEB' from dual union all
select 69348,'AEB' from dual union all
select 69349,'PRO' from dual union all
select 6935,'AN' from dual union all
select 69350,'DLR' from dual union all
select 69351,'AEB' from dual union all
select 69352,'SPX' from dual union all
select 69353,'AEB' from dual union all
select 69354,'IDRDL' from dual union all
select 69355,'SPX' from dual union all
select 69356,'SCAL' from dual union all
select 69357,'ARMAD' from dual union all
select 69358,'ARMAD' from dual union all
select 69359,'IRSA' from dual union all
select 6936,'MVS' from dual union all
select 69360,'KCST' from dual union all
select 69361,'IRSA' from dual union all
select 69362,'IRSA' from dual union all
select 69363,'EUROL' from dual union all
select 69364,'IDRDL' from dual union all
select 69365,'AEB' from dual union all
select 69366,'IR' from dual union all
select 69367,'SYMC' from dual union all
select 69368,'AEB' from dual union all
select 69369,'GSFC' from dual union all
select 6937,'RVSN' from dual union all
select 69370,'JAXA' from dual union all
select 69371,'ARMAD' from dual union all
select 69372,'IDRDL' from dual union all
select 69373,'SSC' from dual union all
select 69374,'GSFC' from dual union all
select 69375,'USAF' from dual union all
select 69376,'USN' from dual union all
select 69377,'EUROL' from dual union all
select 69378,'GSFC' from dual union all
select 69379,'GSFC' from dual union all
select 6938,'ISAS' from dual union all
select 69380,'GSFC' from dual union all
select 69381,'GSFC' from dual union all
select 69382,'GSFC' from dual union all
select 69383,'GSFC' from dual union all
select 69384,'UPAE' from dual union all
select 69385,'USA' from dual union all
select 69386,'IDRDL' from dual union all
select 69387,'NESCOMN' from dual union all
select 69388,'USA' from dual union all
select 69389,'USN' from dual union all
select 6939,'RVSN' from dual union all
select 69390,'RVSNR' from dual union all
select 69391,'USN' from dual union all
select 69392,'USN' from dual union all
select 69393,'USN' from dual union all
select 69394,'USN' from dual union all
select 69395,'USN' from dual union all
select 69396,'RVSNR' from dual union all
select 69397,'GSFC' from dual union all
select 69398,'DLR' from dual union all
select 69399,'GSFC' from dual union all
select 694,'ORD' from dual union all
select 6940,'USN' from dual union all
select 69400,'USA' from dual union all
select 69401,'USN' from dual union all
select 69402,'GSFC' from dual union all
select 69403,'IR' from dual union all
select 69404,'IR' from dual union all
select 69405,'IR' from dual union all
select 69406,'GSFC' from dual union all
select 69407,'NASA' from dual union all
select 69408,'GSFC' from dual union all
select 69409,'JAXA' from dual union all
select 6941,'AN' from dual union all
select 69410,'ISFC' from dual union all
select 69411,'ISFC' from dual union all
select 69412,'PLA2AC' from dual union all
select 69413,'IDF' from dual union all
select 69414,'MDA' from dual union all
select 69415,'MDA' from dual union all
select 69416,'IDRDL' from dual union all
select 69417,'PLA2AC' from dual union all
select 69418,'PLA2AC' from dual union all
select 69419,'PLA2AC' from dual union all
select 6942,'CARDE' from dual union all
select 69420,'GSFC' from dual union all
select 69421,'GSFC' from dual union all
select 69422,'DLR' from dual;
insert into LAUNCH_AGENCY
select 69423,'ISFC' from dual union all
select 69424,'ISFC' from dual union all
select 69425,'ISFC' from dual union all
select 69426,'VMFR' from dual union all
select 69427,'RVSNR' from dual union all
select 69428,'COLE' from dual union all
select 69428,'MDA' from dual union all
select 69429,'MDA' from dual union all
select 6943,'MVS' from dual union all
select 69430,'MDA' from dual union all
select 69431,'MDA' from dual union all
select 69432,'RVSNR' from dual union all
select 69433,'GSFC' from dual union all
select 69434,'USAF' from dual union all
select 69435,'MDA' from dual union all
select 69436,'GSFC' from dual union all
select 69437,'GSFC' from dual union all
select 69438,'DLR' from dual union all
select 69438,'SSC' from dual union all
select 69439,'PKAFSC' from dual union all
select 6944,'ONERA' from dual union all
select 69440,'AEB' from dual union all
select 69441,'JAXA' from dual union all
select 69442,'ISFC' from dual union all
select 69443,'USN' from dual union all
select 69444,'ISFC' from dual union all
select 69445,'SYMC' from dual union all
select 69446,'SYMC' from dual union all
select 69447,'SYMC' from dual union all
select 69448,'SYMC' from dual union all
select 69449,'SYMC' from dual union all
select 6945,'ISAS' from dual union all
select 69450,'SYMC' from dual union all
select 69451,'SYMC' from dual union all
select 69452,'SYMC' from dual union all
select 69453,'SYMC' from dual union all
select 69454,'SYMC' from dual union all
select 69455,'PLA2AC' from dual union all
select 69456,'PLA2AC' from dual union all
select 69457,'RGMS' from dual union all
select 69458,'WFF' from dual union all
select 69459,'WFF' from dual union all
select 6946,'NASA' from dual union all
select 69460,'SMDC' from dual union all
select 69461,'SYMC' from dual union all
select 69462,'SYMC' from dual union all
select 69463,'ADD' from dual union all
select 69464,'KVR' from dual union all
select 69465,'MHI' from dual union all
select 69466,'KARI' from dual union all
select 69467,'ULAL' from dual union all
select 69468,'FKA' from dual union all
select 69469,'AE' from dual union all
select 6947,'USAF' from dual union all
select 69470,'FKA' from dual union all
select 69471,'ULAL' from dual union all
select 69472,'ISRO' from dual union all
select 69473,'SPX' from dual union all
select 69474,'ULAL' from dual union all
select 69475,'ILSK' from dual union all
select 69476,'FKA' from dual union all
select 69477,'ILSK' from dual union all
select 69478,'FKA' from dual union all
select 69479,'OSC' from dual union all
select 6948,'USAF' from dual union all
select 69480,'FKA' from dual union all
select 69481,'SAST' from dual union all
select 69482,'VVKO' from dual union all
select 69483,'CALT' from dual union all
select 69484,'AE' from dual union all
select 69485,'ILSK' from dual union all
select 69486,'ULAL' from dual union all
select 69487,'ULAB' from dual union all
select 69488,'FKA' from dual union all
select 69489,'ILSK' from dual union all
select 6949,'USAF' from dual union all
select 69490,'AE' from dual union all
select 69491,'VVKO' from dual union all
select 69492,'CALT' from dual union all
select 69493,'FKA' from dual union all
select 69494,'AE' from dual union all
select 69495,'RVSNR' from dual union all
select 69496,'OSC' from dual union all
select 69497,'ISRO' from dual union all
select 69498,'CALT' from dual union all
select 69499,'ULAL' from dual union all
select 695,'GAU4' from dual union all
select 6950,'MVS' from dual union all
select 69500,'SAST' from dual union all
select 69501,'AE' from dual union all
select 69502,'FKA' from dual union all
select 69503,'MHI' from dual union all
select 69504,'ULAB' from dual union all
select 69505,'KTRAS' from dual union all
select 69505,'RVSNR' from dual union all
select 69506,'ULAB' from dual union all
select 69507,'AE' from dual union all
select 69508,'ELUS' from dual union all
select 69508,'SIS' from dual union all
select 69509,'SAST' from dual;
insert into LAUNCH_AGENCY
select 6951,'USAF' from dual union all
select 69510,'OSC' from dual union all
select 69511,'KVR' from dual union all
select 69512,'JAXA' from dual union all
select 69513,'ULAL' from dual union all
select 69514,'OSC' from dual union all
select 69515,'SAST' from dual union all
select 69516,'PLA2AC' from dual union all
select 69517,'FKA' from dual union all
select 69518,'SPX' from dual union all
select 69519,'ILSK' from dual union all
select 6952,'ABMA' from dual union all
select 69520,'SAST' from dual union all
select 69521,'ILSK' from dual union all
select 69522,'CALT' from dual union all
select 69523,'ISRO' from dual union all
select 69524,'FKA' from dual union all
select 69525,'KHRU' from dual union all
select 69526,'ULAL' from dual union all
select 69527,'USAF' from dual union all
select 69528,'SAST' from dual union all
select 69529,'KTRAS' from dual union all
select 69529,'RVSNR' from dual union all
select 6953,'USAF' from dual union all
select 69530,'KVR' from dual union all
select 69531,'SAST' from dual union all
select 69532,'FKA' from dual union all
select 69533,'CALT' from dual union all
select 69534,'SPX' from dual union all
select 69535,'ULAL' from dual union all
select 69536,'ILSK' from dual union all
select 69537,'AE' from dual union all
select 69538,'CALT' from dual union all
select 69539,'KVR' from dual union all
select 6954,'MVS' from dual union all
select 69540,'KHRU' from dual union all
select 69541,'VVKO' from dual union all
select 69542,'ARMAD' from dual union all
select 69543,'SPX' from dual union all
select 69544,'AEB' from dual union all
select 69545,'SCAL' from dual union all
select 69546,'SCAL' from dual union all
select 69547,'SPX' from dual union all
select 69548,'SCAL' from dual union all
select 69549,'AFRLRZ' from dual union all
select 6955,'USAF' from dual union all
select 69550,'DMA' from dual union all
select 69551,'AEB' from dual union all
select 69552,'SPX' from dual union all
select 69553,'SPX' from dual union all
select 69554,'SCAL' from dual union all
select 69555,'SCAL' from dual union all
select 69556,'SCAL' from dual union all
select 69557,'MDA' from dual union all
select 69558,'AEB' from dual union all
select 69559,'DLR' from dual union all
select 6956,'SAC' from dual union all
select 69560,'SPX' from dual union all
select 69561,'PRO' from dual union all
select 69562,'RVSN' from dual union all
select 69563,'RVSN' from dual union all
select 69564,'RVSN' from dual union all
select 69565,'PVO' from dual union all
select 69566,'PVO' from dual union all
select 69567,'SCAL' from dual union all
select 69568,'IIAE' from dual union all
select 69569,'JRG' from dual union all
select 6957,'MVS' from dual union all
select 69570,'JRG' from dual union all
select 69571,'JRG' from dual union all
select 69572,'JRG' from dual union all
select 69573,'JRG' from dual union all
select 69574,'SMDC' from dual union all
select 69575,'SMDC' from dual union all
select 69576,'SMDC' from dual union all
select 69577,'SMDC' from dual union all
select 69578,'SMDC' from dual union all
select 69579,'SMDC' from dual union all
select 6958,'MVS' from dual union all
select 69580,'SMDC' from dual union all
select 69581,'SMDC' from dual union all
select 69582,'SMDC' from dual union all
select 69583,'SMDC' from dual union all
select 69584,'ELUS' from dual union all
select 69585,'KHRU' from dual union all
select 69586,'SAST' from dual union all
select 69587,'SYMC' from dual union all
select 69588,'SYMC' from dual union all
select 69589,'SYMC' from dual union all
select 6959,'USN' from dual union all
select 69590,'AEB' from dual union all
select 69591,'AEB' from dual union all
select 69592,'RVSN' from dual union all
select 69593,'ISRO' from dual union all
select 69594,'MDA' from dual union all
select 69595,'DRDO' from dual union all
select 69596,'PLA2AC' from dual union all
select 69597,'CHFEN' from dual union all
select 69597,'PLA2AC' from dual union all
select 69598,'IRSA' from dual;
insert into LAUNCH_AGENCY
select 69599,'GSFC' from dual union all
select 696,'PARD' from dual union all
select 6960,'MVS' from dual union all
select 69600,'GSFC' from dual union all
select 69601,'USA' from dual union all
select 69602,'USN' from dual union all
select 69603,'SYMC' from dual union all
select 69604,'GSFC' from dual union all
select 69605,'IRSA' from dual union all
select 69606,'RVSNR' from dual union all
select 69607,'USAF' from dual union all
select 69608,'RVSNR' from dual union all
select 69609,'RVSNR' from dual union all
select 6961,'MVS' from dual union all
select 69610,'ISFC' from dual union all
select 69611,'EUROL' from dual union all
select 69612,'EUROL' from dual union all
select 69613,'NSPO' from dual union all
select 69614,'SYMC' from dual union all
select 69615,'PLA2AC' from dual union all
select 69616,'RVSNR' from dual union all
select 69617,'SYMC' from dual union all
select 69618,'SYMC' from dual union all
select 69619,'SYMC' from dual union all
select 6962,'RVSN' from dual union all
select 69620,'SYMC' from dual union all
select 69621,'SYMC' from dual union all
select 69622,'SYMC' from dual union all
select 69623,'SYMC' from dual union all
select 69624,'SYMC' from dual union all
select 69625,'IDF' from dual union all
select 69626,'MDA' from dual union all
select 69627,'DLR' from dual union all
select 69627,'SSC' from dual union all
select 69628,'NESCOMN' from dual union all
select 69629,'CSSAR' from dual union all
select 6963,'RVSN' from dual union all
select 69630,'ISFC' from dual union all
select 69631,'GSFC' from dual union all
select 69632,'GSFC' from dual union all
select 69633,'GSFC' from dual union all
select 69634,'GSFC' from dual union all
select 69635,'GSFC' from dual union all
select 69636,'GSFC' from dual union all
select 69637,'GSFC' from dual union all
select 69638,'GSFC' from dual union all
select 69639,'USA' from dual union all
select 6964,'USA' from dual union all
select 69640,'USN' from dual union all
select 69641,'USAF' from dual union all
select 69642,'GSFC' from dual union all
select 69643,'RVSNR' from dual union all
select 69644,'GSFC' from dual union all
select 69645,'UPAE' from dual union all
select 69646,'SYMC' from dual union all
select 69647,'SYMC' from dual union all
select 69648,'SYMC' from dual union all
select 69649,'GSFC' from dual union all
select 6965,'ONERA' from dual union all
select 69650,'GSFC' from dual union all
select 69651,'MDA' from dual union all
select 69652,'MDA' from dual union all
select 69653,'DLR' from dual union all
select 69654,'JAXA' from dual union all
select 69655,'JAXA' from dual union all
select 69656,'IDF' from dual union all
select 69657,'GSFC' from dual union all
select 69658,'ISFC' from dual union all
select 69659,'GSFC' from dual union all
select 6966,'CARDE' from dual union all
select 69660,'MDA' from dual union all
select 69661,'IDF' from dual union all
select 69662,'MDA' from dual union all
select 69663,'USN' from dual union all
select 69664,'MDA' from dual union all
select 69665,'MDA' from dual union all
select 69666,'VMF' from dual union all
select 69667,'IDRDL' from dual union all
select 69668,'USA' from dual union all
select 69669,'USN' from dual union all
select 6967,'USN' from dual union all
select 69670,'USN' from dual union all
select 69671,'DLR' from dual union all
select 69672,'DLR' from dual union all
select 69673,'DLR' from dual union all
select 69674,'DLR' from dual union all
select 69675,'DLR' from dual union all
select 69676,'DLR' from dual union all
select 69677,'DLR' from dual union all
select 69678,'DLR' from dual union all
select 69679,'DLR' from dual union all
select 6968,'NASA' from dual union all
select 69680,'DLR' from dual union all
select 69681,'DLR' from dual union all
select 69682,'DLR' from dual union all
select 69683,'USAF' from dual union all
select 69684,'USN' from dual union all
select 69685,'USN' from dual union all
select 69686,'USN' from dual union all
select 69687,'USN' from dual;
insert into LAUNCH_AGENCY
select 69688,'USN' from dual union all
select 69689,'PLA2AC' from dual union all
select 6969,'USA' from dual union all
select 69690,'USAF' from dual union all
select 69691,'ISFC' from dual union all
select 69692,'RVSNR' from dual union all
select 69693,'USA' from dual union all
select 69694,'USN' from dual union all
select 69695,'ISFC' from dual union all
select 69696,'GSFC' from dual union all
select 69697,'RVSNR' from dual union all
select 69698,'RVSNR' from dual union all
select 69699,'VMFR' from dual union all
select 697,'GEMTS' from dual union all
select 6970,'ABMA' from dual union all
select 6970,'USAF' from dual union all
select 69700,'VMFR' from dual union all
select 69701,'GSFC' from dual union all
select 69702,'UPAE' from dual union all
select 69703,'GSFC' from dual union all
select 69704,'PLA2AC' from dual union all
select 69705,'ISRO' from dual union all
select 69706,'SPX' from dual union all
select 69707,'OSC' from dual union all
select 69708,'ULAL' from dual union all
select 69709,'FKA' from dual union all
select 6971,'MVS' from dual union all
select 69710,'AE' from dual union all
select 69711,'ILSK' from dual union all
select 69712,'ULAB' from dual union all
select 69713,'MHI' from dual union all
select 69714,'KHRU' from dual union all
select 69715,'AE' from dual union all
select 69716,'VVKO' from dual union all
select 69717,'FKA' from dual union all
select 69718,'CALT' from dual union all
select 69719,'ULAL' from dual union all
select 6972,'USAF' from dual union all
select 69720,'AE' from dual union all
select 69721,'ISRO' from dual union all
select 69722,'FKA' from dual union all
select 69723,'ISA' from dual union all
select 69724,'ULAL' from dual union all
select 69725,'FKA' from dual union all
select 69726,'SPX' from dual union all
select 69727,'KHRU' from dual union all
select 69728,'AE' from dual union all
select 69729,'VVKO' from dual union all
select 6973,'CARDE' from dual union all
select 69730,'ULAB' from dual union all
select 69731,'ULAL' from dual union all
select 69732,'KVR' from dual union all
select 69733,'MHI' from dual union all
select 69734,'ELUS' from dual union all
select 69735,'FKA' from dual union all
select 69736,'VVKO' from dual union all
select 69737,'KTRAS' from dual union all
select 69737,'RVSNR' from dual union all
select 69738,'ISRO' from dual union all
select 69739,'ULAB' from dual union all
select 6974,'USN' from dual union all
select 69740,'KVR' from dual union all
select 69741,'FKA' from dual union all
select 69742,'AE' from dual union all
select 69743,'OSC' from dual union all
select 69744,'SPX' from dual union all
select 69745,'FKA' from dual union all
select 69746,'FKA' from dual union all
select 69747,'ULAB' from dual union all
select 69748,'AE' from dual union all
select 69749,'ULAL' from dual union all
select 6975,'USAF' from dual union all
select 69750,'SPX' from dual union all
select 69751,'SAST' from dual union all
select 69752,'ULAL' from dual union all
select 69753,'SAST' from dual union all
select 69754,'AE' from dual union all
select 69755,'SAST' from dual union all
select 69756,'SPX' from dual union all
select 69757,'SAST' from dual union all
select 69758,'AE' from dual union all
select 69759,'ULAL' from dual union all
select 6976,'RAF' from dual union all
select 69760,'SPX' from dual union all
select 69761,'FKA' from dual union all
select 69762,'KHRU' from dual union all
select 69763,'CALT' from dual union all
select 69764,'MHI' from dual union all
select 69765,'ISRO' from dual union all
select 69766,'AE' from dual union all
select 69767,'SAST' from dual union all
select 69768,'KHRU' from dual union all
select 69769,'CALT' from dual union all
select 6977,'AFCRL' from dual union all
select 69770,'CALT' from dual union all
select 69771,'FKA' from dual union all
select 69772,'ULAL' from dual union all
select 69773,'VVKO' from dual union all
select 69774,'KTRAS' from dual union all
select 69774,'RVSNR' from dual;
insert into LAUNCH_AGENCY
select 69775,'CALT' from dual union all
select 69776,'SAST' from dual union all
select 69777,'PLA2AC' from dual union all
select 69778,'FKA' from dual union all
select 69779,'VVKO' from dual union all
select 6978,'CARDE' from dual union all
select 69780,'MHI' from dual union all
select 69781,'ULAB' from dual union all
select 69782,'AE' from dual union all
select 69783,'SAST' from dual union all
select 69784,'SAST' from dual union all
select 69785,'ULAL' from dual union all
select 69786,'ILSK' from dual union all
select 69787,'AE' from dual union all
select 69788,'RVSNR' from dual union all
select 69789,'KHRU' from dual union all
select 6979,'CARDE' from dual union all
select 69790,'VVKO' from dual union all
select 69791,'FKA' from dual union all
select 69792,'SAST' from dual union all
select 69793,'ILSK' from dual union all
select 69794,'CALT' from dual union all
select 69795,'SCAL' from dual union all
select 69796,'ISFC' from dual union all
select 69797,'SCAL' from dual union all
select 69798,'PLA2AC' from dual union all
select 69799,'AEB' from dual union all
select 698,'USA' from dual union all
select 6980,'USAF' from dual union all
select 69800,'SPX' from dual union all
select 69801,'SPX' from dual union all
select 69802,'PRO' from dual union all
select 69803,'SPX' from dual union all
select 69804,'JPL' from dual union all
select 69805,'GSFC' from dual union all
select 69806,'PLA2AC' from dual union all
select 69807,'SPX' from dual union all
select 69808,'SMDC' from dual union all
select 69809,'AEB' from dual union all
select 6981,'USAF' from dual union all
select 69810,'JRG' from dual union all
select 69811,'JRG' from dual union all
select 69812,'JRG' from dual union all
select 69813,'JRG' from dual union all
select 69814,'JRG' from dual union all
select 69815,'JRG' from dual union all
select 69816,'JRG' from dual union all
select 69817,'JRG' from dual union all
select 69818,'JRG' from dual union all
select 69819,'JRG' from dual union all
select 6982,'USN' from dual union all
select 69820,'JRG' from dual union all
select 69821,'JRG' from dual union all
select 69822,'JRG' from dual union all
select 69823,'JRG' from dual union all
select 69824,'AEB' from dual union all
select 69825,'SCAL' from dual union all
select 69826,'SCAL' from dual union all
select 69827,'IAI' from dual union all
select 69828,'SCAL' from dual union all
select 69829,'AEB' from dual union all
select 6983,'USN' from dual union all
select 69830,'SCAL' from dual union all
select 69831,'USN' from dual union all
select 69832,'USN' from dual union all
select 69833,'ISFC' from dual union all
select 69834,'PLA2AC' from dual union all
select 69835,'AEB' from dual union all
select 69836,'GSFC' from dual union all
select 69837,'RVSNR' from dual union all
select 69838,'USN' from dual union all
select 69839,'USN' from dual union all
select 6984,'USN' from dual union all
select 69840,'USN' from dual union all
select 69841,'USN' from dual union all
select 69842,'USN' from dual union all
select 69843,'PRO' from dual union all
select 69844,'KHRU' from dual union all
select 69845,'OSC' from dual union all
select 69846,'AEB' from dual union all
select 69847,'PKAFSC' from dual union all
select 69848,'PKAFSC' from dual union all
select 69849,'ISFC' from dual union all
select 6985,'MVS' from dual union all
select 69850,'ISRO' from dual union all
select 69851,'ISRO' from dual union all
select 69852,'ISRO' from dual union all
select 69853,'ISRO' from dual union all
select 69854,'IDF' from dual union all
select 69855,'MDA' from dual union all
select 69856,'MDA' from dual union all
select 69857,'MDA' from dual union all
select 69858,'IDRDL' from dual union all
select 69859,'GSFC' from dual union all
select 6986,'USAF' from dual union all
select 69860,'RVSNR' from dual union all
select 69861,'JRG' from dual union all
select 69862,'JRG' from dual union all
select 69863,'JRG' from dual union all
select 69864,'JRG' from dual;
insert into LAUNCH_AGENCY
select 69865,'JRG' from dual union all
select 69866,'JRG' from dual union all
select 69867,'JRG' from dual union all
select 69868,'JRG' from dual union all
select 69869,'ADD' from dual union all
select 6987,'USN' from dual union all
select 69870,'RVSNR' from dual union all
select 69871,'RGMS' from dual union all
select 69872,'IDRDL' from dual union all
select 69873,'IDRDL' from dual union all
select 69874,'RVSNR' from dual union all
select 69875,'VMFR' from dual union all
select 69876,'VMFR' from dual union all
select 69877,'GSFC' from dual union all
select 69878,'RVSNR' from dual union all
select 69879,'USN' from dual union all
select 6988,'MVS' from dual union all
select 69880,'EUROL' from dual union all
select 69881,'EUROL' from dual union all
select 69882,'USN' from dual union all
select 69883,'USN' from dual union all
select 69884,'GSFC' from dual union all
select 69885,'MDA' from dual union all
select 69886,'MDA' from dual union all
select 69887,'GSFC' from dual union all
select 69888,'JRG' from dual union all
select 69889,'JRG' from dual union all
select 6989,'MVS' from dual union all
select 69890,'KHRU' from dual union all
select 69891,'JRG' from dual union all
select 69892,'JRG' from dual union all
select 69893,'PLA2AC' from dual union all
select 69894,'CHFEN' from dual union all
select 69894,'PLA2AC' from dual union all
select 69895,'GSFC' from dual union all
select 69896,'JAXA' from dual union all
select 69897,'JAXA' from dual union all
select 69898,'GSFC' from dual union all
select 69899,'JRG' from dual union all
select 699,'USA' from dual union all
select 6990,'NASA' from dual union all
select 69900,'JRG' from dual union all
select 69901,'GSFC' from dual union all
select 69902,'AEB' from dual union all
select 69903,'VMF' from dual union all
select 69904,'ISFC' from dual union all
select 69905,'ISFC' from dual union all
select 69906,'USN' from dual union all
select 69907,'USN' from dual union all
select 69908,'USN' from dual union all
select 69909,'IDF' from dual union all
select 6991,'MVS' from dual union all
select 69910,'USAF' from dual union all
select 69911,'GSFC' from dual union all
select 69912,'PLA2AC' from dual union all
select 69913,'GSFC' from dual union all
select 69914,'NSPO' from dual union all
select 69915,'NSPO' from dual union all
select 69916,'USA' from dual union all
select 69917,'UPAE' from dual union all
select 69918,'CSXT' from dual union all
select 69919,'VMF' from dual union all
select 6992,'USN' from dual union all
select 69920,'RVSNR' from dual union all
select 69921,'VMFR' from dual union all
select 69922,'USA' from dual union all
select 69923,'USN' from dual union all
select 69924,'GSFC' from dual union all
select 69925,'PKAFSC' from dual union all
select 69926,'NESCOMN' from dual union all
select 69927,'VMF' from dual union all
select 69928,'ISFC' from dual union all
select 69929,'ISFC' from dual union all
select 6993,'MVS' from dual union all
select 69930,'GSFC' from dual union all
select 69931,'GSFC' from dual union all
select 69932,'ISRO' from dual union all
select 69933,'IDF' from dual union all
select 69934,'PLA2AC' from dual union all
select 69935,'RVSNR' from dual union all
select 69936,'GSFC' from dual union all
select 69937,'JRG' from dual union all
select 69938,'KRAT' from dual union all
select 69938,'MDA' from dual union all
select 69939,'SPX' from dual union all
select 6994,'ABMA' from dual union all
select 69940,'ULAL' from dual union all
select 69941,'ULAB' from dual union all
select 69942,'MHI' from dual union all
select 69943,'ILSK' from dual union all
select 69944,'IRSA' from dual union all
select 69945,'SPX' from dual union all
select 69946,'FKA' from dual union all
select 69947,'VVKO' from dual union all
select 69948,'SPX' from dual union all
select 69949,'ULAL' from dual union all
select 6995,'DRTE' from dual union all
select 69950,'KHRU' from dual union all
select 69951,'ULAB' from dual union all
select 69952,'KTRAS' from dual;
insert into LAUNCH_AGENCY
select 69952,'RVSNR' from dual union all
select 69953,'MHI' from dual union all
select 69954,'FKA' from dual union all
select 69955,'AE' from dual union all
select 69956,'ISRO' from dual union all
select 69957,'CALT' from dual union all
select 69958,'KVR' from dual union all
select 69959,'SPX' from dual union all
select 6996,'MVS' from dual union all
select 69960,'AE' from dual union all
select 69961,'SPX' from dual union all
select 69962,'FKA' from dual union all
select 69963,'ULAL' from dual union all
select 69964,'AE' from dual union all
select 69965,'VVKO' from dual union all
select 69966,'AE' from dual union all
select 69967,'VVKO' from dual union all
select 69968,'SAST' from dual union all
select 69969,'FKA' from dual union all
select 6997,'NASA' from dual union all
select 69970,'ISRO' from dual union all
select 69971,'ULAL' from dual union all
select 69972,'AE' from dual union all
select 69973,'FKA' from dual union all
select 69974,'ULAB' from dual union all
select 69975,'CALT' from dual union all
select 69976,'MHI' from dual union all
select 69977,'AE' from dual union all
select 69978,'SAST' from dual union all
select 69979,'ISRO' from dual union all
select 6998,'USAF' from dual union all
select 69980,'ILSK' from dual union all
select 69981,'FKA' from dual union all
select 69982,'ULAL' from dual union all
select 69983,'AE' from dual union all
select 69984,'CALT' from dual union all
select 69985,'SAST' from dual union all
select 69986,'KHRU' from dual union all
select 69987,'CALT' from dual union all
select 69988,'KVR' from dual union all
select 69989,'CASC' from dual union all
select 6999,'USAF' from dual union all
select 69990,'ISRO' from dual union all
select 69991,'CALT' from dual union all
select 69992,'AE' from dual union all
select 69993,'FKA' from dual union all
select 69994,'ULAL' from dual union all
select 69995,'SAST' from dual union all
select 69996,'ULAL' from dual union all
select 69997,'CALT' from dual union all
select 69998,'ILSK' from dual union all
select 69999,'SAST' from dual union all
select 7,'WEHR' from dual union all
select 70,'JPL' from dual union all
select 700,'GEMTS' from dual union all
select 700,'US' from dual union all
select 7000,'AFCRL' from dual union all
select 70000,'ULAL' from dual union all
select 70001,'CALT' from dual union all
select 70002,'SAST' from dual union all
select 70003,'AE' from dual union all
select 70004,'VVKO' from dual union all
select 70005,'CALT' from dual union all
select 70006,'MHI' from dual union all
select 70007,'SAST' from dual union all
select 70008,'AE' from dual union all
select 70009,'VVKO' from dual union all
select 7001,'MVS' from dual union all
select 70010,'ULAL' from dual union all
select 70011,'CALT' from dual union all
select 70012,'KVR' from dual union all
select 70013,'KHRU' from dual union all
select 70014,'FKA' from dual union all
select 70015,'ISRO' from dual union all
select 70016,'SAST' from dual union all
select 70017,'AE' from dual union all
select 70018,'FKA' from dual union all
select 70019,'SPX' from dual union all
select 7002,'DRTE' from dual union all
select 70020,'KHRU' from dual union all
select 70021,'CALT' from dual union all
select 70022,'ISFC' from dual union all
select 70023,'RVSNR' from dual union all
select 70024,'SPX' from dual union all
select 70025,'JRG' from dual union all
select 70026,'GSFC' from dual union all
select 70027,'JPL' from dual union all
select 70028,'PRO' from dual union all
select 70029,'PLA2AC' from dual union all
select 7003,'USAF' from dual union all
select 70030,'MDA' from dual union all
select 70031,'USN' from dual union all
select 70032,'USN' from dual union all
select 70033,'USN' from dual union all
select 70034,'USN' from dual union all
select 70035,'PLA2AC' from dual union all
select 70036,'AEB' from dual union all
select 70037,'AEB' from dual union all
select 70038,'JRG' from dual union all
select 70039,'DARE' from dual;
insert into LAUNCH_AGENCY
select 7004,'RVSN' from dual union all
select 70040,'USN' from dual union all
select 70041,'USN' from dual union all
select 70042,'PLA2AC' from dual union all
select 70043,'ANSAL' from dual union all
select 70044,'ANSAL' from dual union all
select 70045,'ANSAL' from dual union all
select 70046,'ANSAL' from dual union all
select 70047,'ANSAL' from dual union all
select 70048,'ANSAL' from dual union all
select 70049,'ANSAL' from dual union all
select 7005,'RVSN' from dual union all
select 70050,'ANSAL' from dual union all
select 70051,'ANSAL' from dual union all
select 70052,'ANSAL' from dual union all
select 70053,'ANSAL' from dual union all
select 70054,'ANSAL' from dual union all
select 70055,'ANSAL' from dual union all
select 70056,'ANSAL' from dual union all
select 70057,'ANSAL' from dual union all
select 70058,'ANSAL' from dual union all
select 70059,'PRO' from dual union all
select 7006,'MVS' from dual union all
select 70060,'MDA' from dual union all
select 70061,'JRG' from dual union all
select 70062,'JRG' from dual union all
select 70063,'JRG' from dual union all
select 70064,'ADD' from dual union all
select 70065,'JRG' from dual union all
select 70066,'JRG' from dual union all
select 70067,'JRG' from dual union all
select 70068,'JRG' from dual union all
select 70069,'JRG' from dual union all
select 7007,'NASA' from dual union all
select 70070,'JRG' from dual union all
select 70071,'JRG' from dual union all
select 70072,'JRG' from dual union all
select 70073,'JRG' from dual union all
select 70074,'JRG' from dual union all
select 70075,'SMDC' from dual union all
select 70076,'AEB' from dual union all
select 70077,'ILSK' from dual union all
select 70078,'SPX' from dual union all
select 70079,'USAF' from dual union all
select 7008,'NASA' from dual union all
select 70080,'JRG' from dual union all
select 70081,'JRG' from dual union all
select 70082,'ISFC' from dual union all
select 70083,'ISRO' from dual union all
select 70084,'GSFC' from dual union all
select 70085,'GSFC' from dual union all
select 70086,'GSFC' from dual union all
select 70087,'GSFC' from dual union all
select 70088,'GSFC' from dual union all
select 70089,'IDRDL' from dual union all
select 7009,'MVS' from dual union all
select 70090,'PLA2AC' from dual union all
select 70091,'GSFC' from dual union all
select 70092,'GSFC' from dual union all
select 70093,'GSFC' from dual union all
select 70094,'GSFC' from dual union all
select 70095,'USN' from dual union all
select 70096,'USN' from dual union all
select 70097,'ARR' from dual union all
select 70098,'SSC' from dual union all
select 70099,'DLR' from dual union all
select 701,'ORD' from dual union all
select 7010,'MVS' from dual union all
select 70100,'PKAFSC' from dual union all
select 70101,'USAF' from dual union all
select 70102,'RVSNR' from dual union all
select 70103,'EUROL' from dual union all
select 70104,'EUROL' from dual union all
select 70105,'USAF' from dual union all
select 70106,'PKAFSC' from dual union all
select 70107,'GSFC' from dual union all
select 70108,'ISFC' from dual union all
select 70109,'DLR' from dual union all
select 70109,'SSC' from dual union all
select 7011,'MVS' from dual union all
select 70110,'DLR' from dual union all
select 70110,'SSC' from dual union all
select 70111,'BLOR' from dual union all
select 70112,'GSFC' from dual union all
select 70113,'USAF' from dual union all
select 70114,'ANSAL' from dual union all
select 70115,'JMSDF' from dual union all
select 70115,'MDA' from dual union all
select 70116,'GSFC' from dual union all
select 70117,'ANSAL' from dual union all
select 70118,'ADD' from dual union all
select 70119,'GSFC' from dual union all
select 7012,'USAF' from dual union all
select 70120,'DLR' from dual union all
select 70121,'DLR' from dual union all
select 70122,'GSFC' from dual union all
select 70123,'MDA' from dual union all
select 70124,'MDA' from dual union all
select 70125,'USAF' from dual union all
select 70126,'RVSNR' from dual;
insert into LAUNCH_AGENCY
select 70127,'ANSAL' from dual union all
select 70128,'GSFC' from dual union all
select 70129,'GSFC' from dual union all
select 7013,'SKB30' from dual union all
select 70130,'JAXA' from dual union all
select 70131,'GSFC' from dual union all
select 70132,'DMA' from dual union all
select 70133,'SSC' from dual union all
select 70134,'GSFC' from dual union all
select 70135,'IR' from dual union all
select 70136,'USAF' from dual union all
select 70137,'SSC' from dual union all
select 70138,'RVSNR' from dual union all
select 70139,'VMFR' from dual union all
select 7014,'USAF' from dual union all
select 70140,'VMFR' from dual union all
select 70141,'RVSNR' from dual union all
select 70142,'MDA' from dual union all
select 70143,'MDA' from dual union all
select 70144,'MDA' from dual union all
select 70145,'MDA' from dual union all
select 70146,'PLA2AC' from dual union all
select 70147,'CHFEN' from dual union all
select 70147,'PLA2AC' from dual union all
select 70148,'USN' from dual union all
select 70149,'USN' from dual union all
select 7015,'GB5' from dual union all
select 70150,'USN' from dual union all
select 70151,'USN' from dual union all
select 70152,'ISFC' from dual union all
select 70153,'UPAE' from dual union all
select 70154,'VMF' from dual union all
select 70155,'VMF' from dual union all
select 70156,'RVSNR' from dual union all
select 70157,'BLOR' from dual union all
select 70158,'GSFC' from dual union all
select 70159,'PRO' from dual union all
select 7016,'USN' from dual union all
select 70160,'GSFC' from dual union all
select 70161,'GSFC' from dual union all
select 70162,'IR' from dual union all
select 70163,'SSC' from dual union all
select 70164,'JMSDF' from dual union all
select 70164,'MDA' from dual union all
select 70165,'IDF' from dual union all
select 70166,'IDF' from dual union all
select 70167,'MDA' from dual union all
select 70168,'USN' from dual union all
select 70169,'MDA' from dual union all
select 7017,'USAF' from dual union all
select 70170,'VMFR' from dual union all
select 70171,'PKAFSC' from dual union all
select 70172,'NESCOMN' from dual union all
select 70173,'GSFC' from dual union all
select 70174,'RVSNR' from dual union all
select 70175,'GSFC' from dual union all
select 70176,'RGMS' from dual union all
select 70177,'PLA2AC' from dual union all
select 70178,'AE' from dual union all
select 70179,'CALT' from dual union all
select 7018,'USN' from dual union all
select 70180,'SPX' from dual union all
select 70181,'ISRO' from dual union all
select 70182,'AE' from dual union all
select 70183,'ILSK' from dual union all
select 70184,'CALT' from dual union all
select 70185,'ULAL' from dual union all
select 70186,'VVKO' from dual union all
select 70187,'NADA' from dual union all
select 70188,'ULAB' from dual union all
select 70189,'KVR' from dual union all
select 7019,'NASA' from dual union all
select 70190,'MHI' from dual union all
select 70191,'SPX' from dual union all
select 70192,'AE' from dual union all
select 70193,'ISRO' from dual union all
select 70194,'FKA' from dual union all
select 70195,'KHRU' from dual union all
select 70196,'FKA' from dual union all
select 70197,'ULAL' from dual union all
select 70198,'VVKO' from dual union all
select 70199,'CALT' from dual union all
select 702,'ORD' from dual union all
select 7020,'NRL' from dual union all
select 70200,'FKA' from dual union all
select 70201,'SAST' from dual union all
select 70202,'SPX' from dual union all
select 70203,'AE' from dual union all
select 70204,'FKA' from dual union all
select 70205,'ISRO' from dual union all
select 70206,'SPX' from dual union all
select 70207,'SAST' from dual union all
select 70208,'AE' from dual union all
select 70209,'SPX' from dual union all
select 7021,'NASA' from dual union all
select 70210,'VVKO' from dual union all
select 70211,'SAST' from dual union all
select 70212,'KVR' from dual union all
select 70213,'ILSK' from dual union all
select 70214,'ULAB' from dual;
insert into LAUNCH_AGENCY
select 70215,'CALT' from dual union all
select 70216,'SPX' from dual union all
select 70217,'AE' from dual union all
select 70218,'ISRO' from dual union all
select 70219,'ULAL' from dual union all
select 7022,'NASA' from dual union all
select 70220,'CALT' from dual union all
select 70221,'SAST' from dual union all
select 70222,'FKA' from dual union all
select 70223,'FKA' from dual union all
select 70224,'SPX' from dual union all
select 70225,'ULAL' from dual union all
select 70226,'CALT' from dual union all
select 70227,'SAST' from dual union all
select 70228,'SPX' from dual union all
select 70229,'SAST' from dual union all
select 7023,'MVS' from dual union all
select 70230,'ULAB' from dual union all
select 70231,'AE' from dual union all
select 70232,'ISRO' from dual union all
select 70233,'ULAL' from dual union all
select 70234,'ISA' from dual union all
select 70235,'CALT' from dual union all
select 70236,'AE' from dual union all
select 70237,'ISRO' from dual union all
select 70238,'AE' from dual union all
select 70239,'CALT' from dual union all
select 7024,'NASA' from dual union all
select 70240,'OSC' from dual union all
select 70241,'FKA' from dual union all
select 70242,'MHI' from dual union all
select 70243,'CALT' from dual union all
select 70244,'CASC' from dual union all
select 70245,'ULAL' from dual union all
select 70246,'SAST' from dual union all
select 70247,'AE' from dual union all
select 70248,'FKA' from dual union all
select 70249,'ULAL' from dual union all
select 7025,'USAF' from dual union all
select 70250,'CALT' from dual union all
select 70251,'AE' from dual union all
select 70252,'ISRO' from dual union all
select 70253,'ULAB' from dual union all
select 70254,'MHI' from dual union all
select 70255,'CALT' from dual union all
select 70256,'OSC' from dual union all
select 70257,'ULAL' from dual union all
select 70258,'JAXA' from dual union all
select 70259,'SAST' from dual union all
select 7026,'NASA' from dual union all
select 70260,'AE' from dual union all
select 70261,'SAST' from dual union all
select 70262,'USN' from dual union all
select 70263,'USN' from dual union all
select 70264,'USN' from dual union all
select 70265,'USN' from dual union all
select 70266,'USN' from dual union all
select 70267,'ANSAL' from dual union all
select 70268,'ANSAL' from dual union all
select 70269,'ANSAL' from dual union all
select 7027,'NASA' from dual union all
select 70270,'ANSAL' from dual union all
select 70271,'ANSAL' from dual union all
select 70272,'ANSAL' from dual union all
select 70273,'ANSAL' from dual union all
select 70274,'ANSAL' from dual union all
select 70275,'ANSAL' from dual union all
select 70276,'ANSAL' from dual union all
select 70277,'ANSAL' from dual union all
select 70278,'ANSAL' from dual union all
select 70279,'ANSAL' from dual union all
select 7028,'USAF' from dual union all
select 70280,'ANSAL' from dual union all
select 70281,'ANSAL' from dual union all
select 70282,'ANSAL' from dual union all
select 70283,'ANSAL' from dual union all
select 70284,'ANSAL' from dual union all
select 70285,'ANSAL' from dual union all
select 70286,'JRG' from dual union all
select 70287,'JRG' from dual union all
select 70288,'JRG' from dual union all
select 70289,'JRG' from dual union all
select 7029,'NASA' from dual union all
select 70290,'JRG' from dual union all
select 70291,'ANSAL' from dual union all
select 70292,'ANSAL' from dual union all
select 70293,'ANSAL' from dual union all
select 70294,'ANSAL' from dual union all
select 70295,'JRG' from dual union all
select 70296,'ANSAL' from dual union all
select 70297,'JRG' from dual union all
select 70298,'PRO' from dual union all
select 70299,'JRG' from dual union all
select 703,'USA' from dual union all
select 7030,'RAE' from dual union all
select 7030,'WRE' from dual union all
select 70300,'SCAL' from dual union all
select 70301,'VMF' from dual union all
select 70302,'JRG' from dual union all
select 70303,'JRG' from dual;
insert into LAUNCH_AGENCY
select 70304,'SCAL' from dual union all
select 70305,'SCAL' from dual union all
select 70306,'SCAL' from dual union all
select 70307,'SCAL' from dual union all
select 70308,'USN' from dual union all
select 70309,'USN' from dual union all
select 7031,'NASA' from dual union all
select 70310,'SCAL' from dual union all
select 70311,'USN' from dual union all
select 70312,'SPX' from dual union all
select 70313,'SAST' from dual union all
select 70314,'FKA' from dual union all
select 70315,'ISRO' from dual union all
select 70316,'ISRO' from dual union all
select 70317,'RVSN' from dual union all
select 70318,'ISFC' from dual union all
select 70319,'ISFC' from dual union all
select 7032,'ABMA' from dual union all
select 70320,'JAXA' from dual union all
select 70321,'BLOR' from dual union all
select 70322,'DLR' from dual union all
select 70322,'SSC' from dual union all
select 70323,'MDA' from dual union all
select 70324,'MDA' from dual union all
select 70325,'SSC' from dual union all
select 70326,'USAF' from dual union all
select 70327,'USAF' from dual union all
select 70328,'GSFC' from dual union all
select 70329,'GSFC' from dual union all
select 7033,'NASA' from dual union all
select 70330,'GSFC' from dual union all
select 70331,'IR' from dual union all
select 70332,'IR' from dual union all
select 70333,'IR' from dual union all
select 70334,'IR' from dual union all
select 70335,'IR' from dual union all
select 70336,'JRG' from dual union all
select 70337,'JRG' from dual union all
select 70338,'JRG' from dual union all
select 70339,'USN' from dual union all
select 7034,'RVSN' from dual union all
select 70340,'USN' from dual union all
select 70341,'USN' from dual union all
select 70342,'BLOR' from dual union all
select 70343,'RVSNR' from dual union all
select 70344,'IRSA' from dual union all
select 70345,'PLA2AC' from dual union all
select 70346,'DRDO' from dual union all
select 70347,'CSSAR' from dual union all
select 70348,'MDA' from dual union all
select 70349,'DLR' from dual union all
select 70349,'DSTO' from dual union all
select 7035,'NASA' from dual union all
select 70350,'PRO' from dual union all
select 70351,'USN' from dual union all
select 70352,'USN' from dual union all
select 70353,'EUROL' from dual union all
select 70354,'EUROL' from dual union all
select 70355,'GSFC' from dual union all
select 70356,'BLOR' from dual union all
select 70357,'JRG' from dual union all
select 70358,'JRG' from dual union all
select 70359,'GSFC' from dual union all
select 7036,'USAF' from dual union all
select 70360,'ARRS' from dual union all
select 70360,'DLR' from dual union all
select 70361,'DMA' from dual union all
select 70362,'JRG' from dual union all
select 70363,'JRG' from dual union all
select 70364,'JRG' from dual union all
select 70365,'DLR' from dual union all
select 70366,'GSFC' from dual union all
select 70367,'JRG' from dual union all
select 70368,'GSFC' from dual union all
select 70369,'ARRS' from dual union all
select 70369,'DLR' from dual union all
select 7037,'US' from dual union all
select 70370,'JRG' from dual union all
select 70371,'JRG' from dual union all
select 70372,'JRG' from dual union all
select 70373,'JRG' from dual union all
select 70374,'USAF' from dual union all
select 70375,'RVSNR' from dual union all
select 70376,'RVSNR' from dual union all
select 70377,'USN' from dual union all
select 70378,'VMF' from dual union all
select 70379,'BLOR' from dual union all
select 7038,'RAE' from dual union all
select 7038,'WRE' from dual union all
select 70380,'VMFR' from dual union all
select 70381,'VMFR' from dual union all
select 70382,'RVSNR' from dual union all
select 70383,'RVSNR' from dual union all
select 70384,'SMDC' from dual union all
select 70385,'MDA' from dual union all
select 70386,'PRO' from dual union all
select 70387,'IDRDL' from dual union all
select 70388,'IR' from dual union all
select 70389,'PLA2AC' from dual union all
select 7039,'USAF' from dual;
insert into LAUNCH_AGENCY
select 70390,'PLA2AC' from dual union all
select 70391,'CALT' from dual union all
select 70392,'EXPACE' from dual union all
select 70393,'SPX' from dual union all
select 70394,'ULAL' from dual union all
select 70395,'MHI' from dual union all
select 70396,'AE' from dual union all
select 70397,'AE' from dual union all
select 70398,'ISRO' from dual union all
select 70399,'SPX' from dual union all
select 704,'ORD' from dual union all
select 7040,'NASA' from dual union all
select 70400,'FKA' from dual union all
select 70401,'ULAL' from dual union all
select 70402,'CASIC4A' from dual union all
select 70403,'AE' from dual union all
select 70404,'SPX' from dual union all
select 70405,'MHI' from dual union all
select 70406,'ULAB' from dual union all
select 70407,'SPX' from dual union all
select 70408,'CALT' from dual union all
select 70409,'ULAL' from dual union all
select 7041,'NASA' from dual union all
select 70410,'FKA' from dual union all
select 70411,'CALT' from dual union all
select 70412,'SPX' from dual union all
select 70413,'AE' from dual union all
select 70414,'ISRO' from dual union all
select 70415,'SPX' from dual union all
select 70416,'AE' from dual union all
select 70417,'VVKO' from dual union all
select 70418,'MHI' from dual union all
select 70419,'AE' from dual union all
select 7042,'AEC' from dual union all
select 70420,'SPX' from dual union all
select 70421,'ISRO' from dual union all
select 70422,'ILSK' from dual union all
select 70423,'FKA' from dual union all
select 70424,'SAST' from dual union all
select 70425,'CALT' from dual union all
select 70426,'ISRO' from dual union all
select 70427,'VVKO' from dual union all
select 70428,'SPX' from dual union all
select 70429,'SPX' from dual union all
select 7043,'AEC' from dual union all
select 70430,'AE' from dual union all
select 70431,'SPX' from dual union all
select 70432,'FKA' from dual union all
select 70433,'FKA' from dual union all
select 70434,'AE' from dual union all
select 70435,'SPX' from dual union all
select 70436,'KHRU' from dual union all
select 70437,'ULAL' from dual union all
select 70438,'MHI' from dual union all
select 70439,'SPX' from dual union all
select 7044,'NASA' from dual union all
select 70440,'OATK' from dual union all
select 70441,'ISRO' from dual union all
select 70442,'IDF' from dual union all
select 70443,'SCAL' from dual union all
select 70444,'JRG' from dual union all
select 70445,'JRG' from dual union all
select 70446,'SCAL' from dual union all
select 70447,'SCAL' from dual union all
select 70448,'PRO' from dual union all
select 70449,'IST' from dual union all
select 7045,'NASA' from dual union all
select 70450,'SCAL' from dual union all
select 70451,'JRG' from dual union all
select 70452,'USN' from dual union all
select 70453,'JAXA' from dual union all
select 70454,'RLABU' from dual union all
select 70455,'CALT' from dual union all
select 70456,'JRG' from dual union all
select 70457,'ISFC' from dual union all
select 70458,'RVSN' from dual union all
select 70459,'RVSN' from dual union all
select 7046,'RAE' from dual union all
select 7046,'WRE' from dual union all
select 70460,'RVSNR' from dual union all
select 70461,'DLR' from dual union all
select 70462,'PKAFSC' from dual union all
select 70463,'GSFC' from dual union all
select 70464,'PLA2AC' from dual union all
select 70465,'IR' from dual union all
select 70466,'KRAT' from dual union all
select 70466,'MDA' from dual union all
select 70467,'MDA' from dual union all
select 70468,'USAF' from dual union all
select 70469,'JRG' from dual union all
select 7047,'USAF' from dual union all
select 70470,'USN' from dual union all
select 70471,'USN' from dual union all
select 70472,'USN' from dual union all
select 70473,'USN' from dual union all
select 70474,'GSFC' from dual union all
select 70475,'GSFC' from dual union all
select 70476,'GSFC' from dual union all
select 70477,'GSFC' from dual union all
select 70478,'JRG' from dual;
insert into LAUNCH_AGENCY
select 70479,'JRG' from dual union all
select 7048,'RVSN' from dual union all
select 70480,'JRG' from dual union all
select 70481,'JRG' from dual union all
select 70482,'EUROL' from dual union all
select 70483,'EUROL' from dual union all
select 70484,'JRG' from dual union all
select 70485,'ADD' from dual union all
select 70486,'ESA' from dual union all
select 70486,'SSC' from dual union all
select 70487,'USAF' from dual union all
select 70488,'GSFC' from dual union all
select 70489,'DLR' from dual union all
select 7049,'ONERA' from dual union all
select 70490,'JRG' from dual union all
select 70491,'USAF' from dual union all
select 70492,'GSFC' from dual union all
select 70493,'JRG' from dual union all
select 70494,'JRG' from dual union all
select 70495,'MDA' from dual union all
select 70496,'MDA' from dual union all
select 70497,'ISFC' from dual union all
select 70498,'IR' from dual union all
select 70499,'IR' from dual union all
select 705,'APL' from dual union all
select 7050,'NASA' from dual union all
select 70500,'IR' from dual union all
select 70501,'IR' from dual union all
select 70502,'IR' from dual union all
select 70503,'IR' from dual union all
select 70504,'KRAT' from dual union all
select 70504,'MDA' from dual union all
select 70505,'GSFC' from dual union all
select 70506,'ADD' from dual union all
select 70507,'MDA' from dual union all
select 70508,'VMF' from dual union all
select 70509,'GSFC' from dual union all
select 7051,'NRL' from dual union all
select 70510,'KRAT' from dual union all
select 70510,'MDA' from dual union all
select 70511,'GSFC' from dual union all
select 70512,'JRG' from dual union all
select 70513,'US8A' from dual union all
select 70514,'ADD' from dual union all
select 70515,'MDA' from dual union all
select 70516,'MDA' from dual union all
select 70517,'DLR' from dual union all
select 70517,'DSTO' from dual union all
select 70518,'PLA2AC' from dual union all
select 70519,'CHFEN' from dual union all
select 70519,'PLA2AC' from dual union all
select 7052,'LARCN' from dual union all
select 70520,'IRSA' from dual union all
select 70521,'JRG' from dual union all
select 70522,'US8A' from dual union all
select 70523,'ADD' from dual union all
select 70524,'MDA' from dual union all
select 70525,'MDA' from dual union all
select 70526,'USAF' from dual union all
select 70527,'PLA2AC' from dual union all
select 70528,'JRG' from dual union all
select 70529,'JRG' from dual union all
select 7053,'MVS' from dual union all
select 70530,'ADD' from dual union all
select 70531,'ADD' from dual union all
select 70532,'ADD' from dual union all
select 70533,'JRG' from dual union all
select 70534,'MDA' from dual union all
select 70535,'ADD' from dual union all
select 7054,'USA' from dual union all
select 7055,'ONERA' from dual union all
select 7056,'GB5' from dual union all
select 7057,'GB5' from dual union all
select 7058,'ABMA' from dual union all
select 7059,'USAF' from dual union all
select 706,'NRL' from dual union all
select 706,'US' from dual union all
select 7060,'USAF' from dual union all
select 7061,'USA' from dual union all
select 7062,'USN' from dual union all
select 7063,'AFCRL' from dual union all
select 7064,'SKB30' from dual union all
select 7065,'NASA' from dual union all
select 7066,'NASA' from dual union all
select 7067,'SKB30' from dual union all
select 7068,'NASA' from dual union all
select 7069,'USAF' from dual union all
select 707,'ARDC' from dual union all
select 707,'US' from dual union all
select 7070,'MVS' from dual union all
select 7071,'MVS' from dual union all
select 7072,'ABMA' from dual union all
select 7073,'NASA' from dual union all
select 7074,'RAF' from dual union all
select 7075,'AFCRL' from dual union all
select 7076,'AFCRL' from dual union all
select 7077,'NASA' from dual union all
select 7078,'MVS' from dual union all
select 7079,'MVS' from dual union all
select 708,'NRL' from dual;
insert into LAUNCH_AGENCY
select 708,'US' from dual union all
select 7080,'LARCN' from dual union all
select 7081,'MVS' from dual union all
select 7082,'MVS' from dual union all
select 7083,'AFCRL' from dual union all
select 7084,'USAF' from dual union all
select 7085,'SKB30' from dual union all
select 7086,'NASA' from dual union all
select 7087,'MVS' from dual union all
select 7088,'USN' from dual union all
select 7089,'MVS' from dual union all
select 709,'APL' from dual union all
select 709,'US' from dual union all
select 7090,'MVS' from dual union all
select 7091,'RVSN' from dual union all
select 7092,'SKB30' from dual union all
select 7093,'MVS' from dual union all
select 7094,'SKB30' from dual union all
select 7095,'MVS' from dual union all
select 7096,'RVSN' from dual union all
select 7097,'SKB30' from dual union all
select 7098,'MVS' from dual union all
select 7099,'MVS' from dual union all
select 71,'JPL' from dual union all
select 710,'APL' from dual union all
select 710,'US' from dual union all
select 7100,'RVSN' from dual union all
select 7101,'SKB30' from dual union all
select 7102,'USAF' from dual union all
select 7103,'SAND' from dual union all
select 7104,'WRE' from dual union all
select 7105,'WRE' from dual union all
select 7106,'WRE' from dual union all
select 7107,'WRE' from dual union all
select 7108,'MVS' from dual union all
select 7109,'UNRV' from dual union all
select 711,'GEMTS' from dual union all
select 711,'US' from dual union all
select 7110,'UNRV' from dual union all
select 7111,'UNRV' from dual union all
select 7112,'NASA' from dual union all
select 7113,'CASDN' from dual union all
select 7114,'RAF' from dual union all
select 7115,'NASA' from dual union all
select 7116,'CASDN' from dual union all
select 7117,'USAF' from dual union all
select 7118,'USN' from dual union all
select 7119,'USA' from dual union all
select 712,'NRL' from dual union all
select 712,'US' from dual union all
select 7120,'USAF' from dual union all
select 7121,'NASA' from dual union all
select 7122,'RVSN' from dual union all
select 7123,'USN' from dual union all
select 7124,'MVS' from dual union all
select 7125,'USAF' from dual union all
select 7126,'USA' from dual union all
select 7127,'WRE' from dual union all
select 7128,'MVS' from dual union all
select 7129,'RVSN' from dual union all
select 713,'APL' from dual union all
select 713,'US' from dual union all
select 7130,'USN' from dual union all
select 7131,'MVS' from dual union all
select 7132,'NASA' from dual union all
select 7133,'MVS' from dual union all
select 7134,'NASA' from dual union all
select 7135,'USN' from dual union all
select 7136,'MVS' from dual union all
select 7137,'MVS' from dual union all
select 7138,'LARCN' from dual union all
select 7139,'USAF' from dual union all
select 714,'GEMTS' from dual union all
select 714,'US' from dual union all
select 7140,'USAF' from dual union all
select 7141,'USAF' from dual union all
select 7142,'RAE' from dual union all
select 7142,'WRE' from dual union all
select 7143,'USA' from dual union all
select 7144,'NRL' from dual union all
select 7145,'ABMA' from dual union all
select 7146,'USAF' from dual union all
select 7147,'RAE' from dual union all
select 7147,'WRE' from dual union all
select 7148,'USAF' from dual union all
select 7149,'SAC' from dual union all
select 715,'GAU4' from dual union all
select 7150,'NASA' from dual union all
select 7151,'USN' from dual union all
select 7152,'NASA' from dual union all
select 7153,'USA' from dual union all
select 7154,'USAF' from dual union all
select 7155,'USN' from dual union all
select 7156,'USN' from dual union all
select 7157,'MVS' from dual union all
select 7158,'NASA' from dual union all
select 7159,'MVS' from dual union all
select 716,'GAU4' from dual union all
select 7160,'ONERA' from dual union all
select 7161,'MVS' from dual;
insert into LAUNCH_AGENCY
select 7162,'ONERA' from dual union all
select 7163,'MVS' from dual union all
select 7164,'ABMA' from dual union all
select 7165,'USAF' from dual union all
select 7166,'USN' from dual union all
select 7167,'CARDE' from dual union all
select 7168,'RVSN' from dual union all
select 7169,'USAF' from dual union all
select 717,'GAU4' from dual union all
select 7170,'NASA' from dual union all
select 7171,'USN' from dual union all
select 7172,'USN' from dual union all
select 7173,'USN' from dual union all
select 7174,'SMSA' from dual union all
select 7174,'WSMR' from dual union all
select 7175,'SMSA' from dual union all
select 7175,'WSMR' from dual union all
select 7176,'SMSA' from dual union all
select 7176,'WSMR' from dual union all
select 7177,'SMSA' from dual union all
select 7177,'WSMR' from dual union all
select 7178,'SMSA' from dual union all
select 7178,'WSMR' from dual union all
select 7179,'SMSA' from dual union all
select 7179,'WSMR' from dual union all
select 718,'GAU4' from dual union all
select 7180,'SMSA' from dual union all
select 7180,'WSMR' from dual union all
select 7181,'SMSA' from dual union all
select 7181,'WSMR' from dual union all
select 7182,'SMSA' from dual union all
select 7182,'WSMR' from dual union all
select 7183,'SMSA' from dual union all
select 7183,'WSMR' from dual union all
select 7184,'SMSA' from dual union all
select 7184,'WSMR' from dual union all
select 7185,'SMSA' from dual union all
select 7185,'WSMR' from dual union all
select 7186,'SMSA' from dual union all
select 7186,'WSMR' from dual union all
select 7187,'SMSA' from dual union all
select 7187,'WSMR' from dual union all
select 7188,'SMSA' from dual union all
select 7188,'WSMR' from dual union all
select 7189,'SMSA' from dual union all
select 7189,'WSMR' from dual union all
select 719,'GAU4' from dual union all
select 7190,'SMSA' from dual union all
select 7190,'WSMR' from dual union all
select 7191,'SMSA' from dual union all
select 7191,'WSMR' from dual union all
select 7192,'SMSA' from dual union all
select 7192,'WSMR' from dual union all
select 7193,'SMSA' from dual union all
select 7193,'WSMR' from dual union all
select 7194,'SIMED' from dual union all
select 7195,'USAF' from dual union all
select 7196,'US' from dual union all
select 7197,'USAF' from dual union all
select 7198,'USAF' from dual union all
select 7199,'SMSA' from dual union all
select 7199,'WSMR' from dual union all
select 72,'JPL' from dual union all
select 720,'GAU4' from dual union all
select 7200,'SMSA' from dual union all
select 7200,'WSMR' from dual union all
select 7201,'SMSA' from dual union all
select 7201,'WSMR' from dual union all
select 7202,'SMSA' from dual union all
select 7202,'WSMR' from dual union all
select 7203,'SMSA' from dual union all
select 7203,'WSMR' from dual union all
select 7204,'SMSA' from dual union all
select 7204,'WSMR' from dual union all
select 7205,'SMSA' from dual union all
select 7205,'WSMR' from dual union all
select 7206,'SMSA' from dual union all
select 7206,'WSMR' from dual union all
select 7207,'SMSA' from dual union all
select 7207,'WSMR' from dual union all
select 7208,'SMSA' from dual union all
select 7208,'WSMR' from dual union all
select 7209,'USAF' from dual union all
select 721,'GAU4' from dual union all
select 7210,'GMS' from dual union all
select 7211,'USAF' from dual union all
select 7212,'USAF' from dual union all
select 7213,'GMS' from dual union all
select 7214,'GMS' from dual union all
select 7215,'GMS' from dual union all
select 7216,'SRDL' from dual union all
select 7217,'SRDL' from dual union all
select 7218,'SRDL' from dual union all
select 7219,'SRDL' from dual union all
select 722,'AFAMC' from dual union all
select 722,'US' from dual union all
select 7220,'SRDL' from dual union all
select 7221,'SRDL' from dual union all
select 7222,'SRDL' from dual union all
select 7223,'SRDL' from dual;
insert into LAUNCH_AGENCY
select 7224,'SRDL' from dual union all
select 7225,'SRDL' from dual union all
select 7226,'SRDL' from dual union all
select 7227,'SRDL' from dual union all
select 7228,'SRDL' from dual union all
select 7229,'SRDL' from dual union all
select 723,'PARD' from dual union all
select 7230,'SRDL' from dual union all
select 7231,'SRDL' from dual union all
select 7232,'SRDL' from dual union all
select 7233,'SRDL' from dual union all
select 7234,'SRDL' from dual union all
select 7235,'SRDL' from dual union all
select 7236,'SRDL' from dual union all
select 7237,'USN' from dual union all
select 7238,'USN' from dual union all
select 7239,'USN' from dual union all
select 724,'GEMTS' from dual union all
select 724,'US' from dual union all
select 7240,'USN' from dual union all
select 7241,'USN' from dual union all
select 7242,'USN' from dual union all
select 7243,'USN' from dual union all
select 7244,'HADC' from dual union all
select 7245,'USN' from dual union all
select 7246,'USN' from dual union all
select 7247,'USAF' from dual union all
select 7248,'USAF' from dual union all
select 7249,'USAF' from dual union all
select 725,'USAF' from dual union all
select 7250,'USAF' from dual union all
select 7251,'USAF' from dual union all
select 7252,'USAF' from dual union all
select 7253,'USAF' from dual union all
select 7254,'USAF' from dual union all
select 7255,'USAF' from dual union all
select 7256,'USAF' from dual union all
select 7257,'USAF' from dual union all
select 7258,'USAF' from dual union all
select 7259,'USAF' from dual union all
select 726,'PARD' from dual union all
select 7260,'USAF' from dual union all
select 7261,'RVSN' from dual union all
select 7262,'NASA' from dual union all
select 7263,'NASA' from dual union all
select 7264,'USAF' from dual union all
select 7265,'USAF' from dual union all
select 7266,'RVSN' from dual union all
select 7267,'RVSN' from dual union all
select 7268,'NASA' from dual union all
select 7269,'USAF' from dual union all
select 727,'USA' from dual union all
select 7270,'RVSN' from dual union all
select 7271,'NASA' from dual union all
select 7272,'USAF' from dual union all
select 7273,'USAF' from dual union all
select 7274,'USAF' from dual union all
select 7275,'USAF' from dual union all
select 7276,'USAF' from dual union all
select 7277,'NASA' from dual union all
select 7278,'USAF' from dual union all
select 7279,'RVSN' from dual union all
select 728,'USA' from dual union all
select 7280,'RVSN' from dual union all
select 7281,'NASA' from dual union all
select 7282,'USAF' from dual union all
select 7283,'USAF' from dual union all
select 7284,'SMSA' from dual union all
select 7284,'WSMR' from dual union all
select 7285,'USA' from dual union all
select 7286,'NMC' from dual union all
select 7287,'USA' from dual union all
select 7288,'SMSA' from dual union all
select 7288,'WSMR' from dual union all
select 7289,'SMSA' from dual union all
select 7289,'WSMR' from dual union all
select 729,'USA' from dual union all
select 7290,'RVSN' from dual union all
select 7291,'ONERA' from dual union all
select 7292,'PRO' from dual union all
select 7293,'PRO' from dual union all
select 7294,'PARD' from dual union all
select 7295,'USAF' from dual union all
select 7296,'RVSN' from dual union all
select 7297,'NASA' from dual union all
select 7298,'NASA' from dual union all
select 7299,'RVSN' from dual union all
select 73,'JPL' from dual union all
select 730,'USA' from dual union all
select 7300,'RVSN' from dual union all
select 7301,'RAE' from dual union all
select 7302,'NASA' from dual union all
select 7302,'USAF' from dual union all
select 7303,'RVSN' from dual union all
select 7304,'PRO' from dual union all
select 7305,'RAE' from dual union all
select 7306,'NASA' from dual union all
select 7307,'USAF' from dual union all
select 7308,'NASA' from dual union all
select 7309,'USA' from dual;
insert into LAUNCH_AGENCY
select 731,'USAF' from dual union all
select 7310,'USA' from dual union all
select 7311,'USA' from dual union all
select 7312,'USA' from dual union all
select 7313,'USA' from dual union all
select 7314,'USA' from dual union all
select 7315,'PRO' from dual union all
select 7316,'USA' from dual union all
select 7317,'USA' from dual union all
select 7318,'NASA' from dual union all
select 7319,'USAF' from dual union all
select 732,'USA' from dual union all
select 7320,'USAF' from dual union all
select 7321,'USAF' from dual union all
select 7322,'NASA' from dual union all
select 7322,'USAF' from dual union all
select 7323,'NASA' from dual union all
select 7324,'RVSN' from dual union all
select 7325,'USN' from dual union all
select 7326,'PRO' from dual union all
select 7327,'NASA' from dual union all
select 7327,'USAF' from dual union all
select 7328,'RVSN' from dual union all
select 7329,'USA' from dual union all
select 733,'USA' from dual union all
select 7330,'USA' from dual union all
select 7331,'USA' from dual union all
select 7332,'USA' from dual union all
select 7333,'USA' from dual union all
select 7334,'USA' from dual union all
select 7335,'USA' from dual union all
select 7336,'USA' from dual union all
select 7337,'PRO' from dual union all
select 7338,'USA' from dual union all
select 7339,'USAF' from dual union all
select 734,'PARD' from dual union all
select 7340,'USAF' from dual union all
select 7341,'USAF' from dual union all
select 7342,'4504MTW' from dual union all
select 7343,'USAF' from dual union all
select 7344,'RVSN' from dual union all
select 7345,'ONERA' from dual union all
select 7346,'RVSN' from dual union all
select 7347,'ONERA' from dual union all
select 7348,'PRO' from dual union all
select 7349,'USAF' from dual union all
select 735,'USA' from dual union all
select 7350,'PRO' from dual union all
select 7351,'USAF' from dual union all
select 7352,'RVSN' from dual union all
select 7353,'NASA' from dual union all
select 7353,'USAF' from dual union all
select 7354,'USAF' from dual union all
select 7355,'USAF' from dual union all
select 7356,'USAF' from dual union all
select 7357,'NASA' from dual union all
select 7357,'USAF' from dual union all
select 7358,'USA' from dual union all
select 7359,'PRO' from dual union all
select 736,'USA' from dual union all
select 7360,'USA' from dual union all
select 7361,'USA' from dual union all
select 7362,'USA' from dual union all
select 7363,'USA' from dual union all
select 7364,'USA' from dual union all
select 7365,'USAF' from dual union all
select 7366,'NASA' from dual union all
select 7367,'NASA' from dual union all
select 7368,'NASA' from dual union all
select 7369,'USAF' from dual union all
select 737,'USAF' from dual union all
select 7370,'ABMA' from dual union all
select 7371,'RAE' from dual union all
select 7372,'RVSN' from dual union all
select 7373,'RAE' from dual union all
select 7374,'USA' from dual union all
select 7375,'USA' from dual union all
select 7376,'USA' from dual union all
select 7377,'USA' from dual union all
select 7378,'USA' from dual union all
select 7379,'USA' from dual union all
select 738,'USA' from dual union all
select 7380,'USA' from dual union all
select 7381,'USN' from dual union all
select 7382,'USA' from dual union all
select 7383,'USA' from dual union all
select 7384,'USA' from dual union all
select 7385,'AFCRL' from dual union all
select 7386,'RVSN' from dual union all
select 7387,'USAF' from dual union all
select 7388,'ABMA' from dual union all
select 7389,'NASA' from dual union all
select 7389,'USAF' from dual union all
select 739,'USA' from dual union all
select 7390,'RVSN' from dual union all
select 7391,'USN' from dual union all
select 7392,'NASA' from dual union all
select 7392,'USAF' from dual union all
select 7393,'RAE' from dual union all
select 7394,'RAE' from dual;
insert into LAUNCH_AGENCY
select 7395,'USAF' from dual union all
select 7396,'USA' from dual union all
select 7397,'USA' from dual union all
select 7398,'USA' from dual union all
select 7399,'USAF' from dual union all
select 74,'JPL' from dual union all
select 740,'USA' from dual union all
select 7400,'USA' from dual union all
select 7401,'RVSN' from dual union all
select 7402,'NASA' from dual union all
select 7402,'USAF' from dual union all
select 7403,'PRO' from dual union all
select 7404,'USA' from dual union all
select 7405,'DRG' from dual union all
select 7406,'NASA' from dual union all
select 7407,'RVSN' from dual union all
select 7408,'USAF' from dual union all
select 7409,'RVSN' from dual union all
select 741,'USA' from dual union all
select 7410,'NASA' from dual union all
select 7410,'USAF' from dual union all
select 7411,'NASA' from dual union all
select 7411,'USAF' from dual union all
select 7412,'USA' from dual union all
select 7413,'USA' from dual union all
select 7414,'USN' from dual union all
select 7415,'USA' from dual union all
select 7416,'USA' from dual union all
select 7417,'USA' from dual union all
select 7418,'RVSN' from dual union all
select 7419,'NOTS' from dual union all
select 742,'USA' from dual union all
select 7420,'NASA' from dual union all
select 7420,'USAF' from dual union all
select 7421,'NOTS' from dual union all
select 7422,'USA' from dual union all
select 7423,'RVSN' from dual union all
select 7424,'RVSN' from dual union all
select 7425,'PRO' from dual union all
select 7426,'NASA' from dual union all
select 7426,'USAF' from dual union all
select 7427,'RVSN' from dual union all
select 7428,'NASA' from dual union all
select 7429,'NMC' from dual union all
select 743,'PARD' from dual union all
select 7430,'RVSN' from dual union all
select 7431,'PRO' from dual union all
select 7432,'PRO' from dual union all
select 7433,'NASA' from dual union all
select 7433,'USAF' from dual union all
select 7434,'USA' from dual union all
select 7435,'USA' from dual union all
select 7436,'USN' from dual union all
select 7437,'RAE' from dual union all
select 7438,'NASA' from dual union all
select 7438,'USAF' from dual union all
select 7439,'NASA' from dual union all
select 7439,'USAF' from dual union all
select 744,'USA' from dual union all
select 7440,'NASA' from dual union all
select 7440,'USAF' from dual union all
select 7441,'USAF' from dual union all
select 7442,'ONERA' from dual union all
select 7443,'USAF' from dual union all
select 7444,'ONERA' from dual union all
select 7445,'NASA' from dual union all
select 7446,'RAE' from dual union all
select 7446,'WRE' from dual union all
select 7447,'USAF' from dual union all
select 7448,'USA' from dual union all
select 7449,'USA' from dual union all
select 745,'USA' from dual union all
select 7450,'USA' from dual union all
select 7451,'USA' from dual union all
select 7452,'USA' from dual union all
select 7453,'RVSN' from dual union all
select 7454,'USAF' from dual union all
select 7455,'RVSN' from dual union all
select 7456,'USA' from dual union all
select 7457,'NASA' from dual union all
select 7457,'USAF' from dual union all
select 7458,'USAF' from dual union all
select 7459,'NASA' from dual union all
select 7459,'USAF' from dual union all
select 746,'USA' from dual union all
select 7460,'RVSN' from dual union all
select 7461,'UMI' from dual union all
select 7462,'UMI' from dual union all
select 7463,'ISAS' from dual union all
select 7464,'USA' from dual union all
select 7465,'USA' from dual union all
select 7466,'USA' from dual union all
select 7467,'USA' from dual union all
select 7468,'USA' from dual union all
select 7469,'USAF' from dual union all
select 747,'PARD' from dual union all
select 7470,'USA' from dual union all
select 7471,'USA' from dual union all
select 7472,'USA' from dual union all
select 7473,'USA' from dual;
insert into LAUNCH_AGENCY
select 7474,'USA' from dual union all
select 7475,'USA' from dual union all
select 7476,'AFCRL' from dual union all
select 7477,'NMC' from dual union all
select 7478,'RVSN' from dual union all
select 7479,'RVSN' from dual union all
select 748,'USA' from dual union all
select 7480,'USAF' from dual union all
select 7481,'RVSN' from dual union all
select 7482,'RVSN' from dual union all
select 7483,'ISAS' from dual union all
select 7484,'USAF' from dual union all
select 7485,'USN' from dual union all
select 7486,'SMSA' from dual union all
select 7486,'WSMR' from dual union all
select 7487,'SMSA' from dual union all
select 7487,'WSMR' from dual union all
select 7488,'SMSA' from dual union all
select 7488,'WSMR' from dual union all
select 7489,'SMSA' from dual union all
select 7489,'WSMR' from dual union all
select 749,'USA' from dual union all
select 7490,'SMSA' from dual union all
select 7490,'WSMR' from dual union all
select 7491,'SMSA' from dual union all
select 7491,'WSMR' from dual union all
select 7492,'USA' from dual union all
select 7493,'USA' from dual union all
select 7494,'USA' from dual union all
select 7495,'USA' from dual union all
select 7496,'USA' from dual union all
select 7497,'USA' from dual union all
select 7498,'NASA' from dual union all
select 7498,'USAF' from dual union all
select 7499,'RVSN' from dual union all
select 75,'JPL' from dual union all
select 750,'USA' from dual union all
select 7500,'USN' from dual union all
select 7501,'NASA' from dual union all
select 7501,'USAF' from dual union all
select 7502,'USAF' from dual union all
select 7503,'CASDN' from dual union all
select 7504,'NASA' from dual union all
select 7505,'PRO' from dual union all
select 7506,'CASDN' from dual union all
select 7507,'NASA' from dual union all
select 7507,'USAF' from dual union all
select 7508,'PRO' from dual union all
select 7509,'NASA' from dual union all
select 7509,'USAF' from dual union all
select 751,'USA' from dual union all
select 7510,'USA' from dual union all
select 7511,'USA' from dual union all
select 7512,'USA' from dual union all
select 7513,'USA' from dual union all
select 7514,'USA' from dual union all
select 7515,'USA' from dual union all
select 7516,'USA' from dual union all
select 7517,'USA' from dual union all
select 7518,'USA' from dual union all
select 7519,'USA' from dual union all
select 752,'GAU4' from dual union all
select 7520,'USA' from dual union all
select 7521,'USA' from dual union all
select 7522,'USA' from dual union all
select 7523,'USA' from dual union all
select 7524,'MVS' from dual union all
select 7525,'PRO' from dual union all
select 7526,'USAF' from dual union all
select 7527,'PRO' from dual union all
select 7528,'USAF' from dual union all
select 7529,'NASA' from dual union all
select 7529,'USAF' from dual union all
select 753,'USA' from dual union all
select 7530,'USAF' from dual union all
select 7531,'USAF' from dual union all
select 7532,'ONERA' from dual union all
select 7533,'NASA' from dual union all
select 7534,'ONERA' from dual union all
select 7535,'NASA' from dual union all
select 7535,'USAF' from dual union all
select 7536,'USN' from dual union all
select 7537,'USN' from dual union all
select 7538,'PRO' from dual union all
select 7539,'USAF' from dual union all
select 754,'PARD' from dual union all
select 7540,'USA' from dual union all
select 7541,'USA' from dual union all
select 7542,'USA' from dual union all
select 7543,'USA' from dual union all
select 7544,'USA' from dual union all
select 7545,'USA' from dual union all
select 7546,'USA' from dual union all
select 7547,'USA' from dual union all
select 7548,'USA' from dual union all
select 7549,'USA' from dual union all
select 755,'USAF' from dual union all
select 7550,'USA' from dual union all
select 7551,'USA' from dual union all
select 7552,'USA' from dual;
insert into LAUNCH_AGENCY
select 7553,'USA' from dual union all
select 7554,'USAF' from dual union all
select 7555,'NASA' from dual union all
select 7556,'NASA' from dual union all
select 7557,'USAF' from dual union all
select 7558,'USAF' from dual union all
select 7559,'NASA' from dual union all
select 756,'USA' from dual union all
select 7560,'USAF' from dual union all
select 7561,'NASA' from dual union all
select 7562,'USAF' from dual union all
select 7563,'USAF' from dual union all
select 7564,'USAF' from dual union all
select 7565,'USAF' from dual union all
select 7566,'RVSN' from dual union all
select 7567,'USAF' from dual union all
select 7568,'USAF' from dual union all
select 7569,'RVSN' from dual union all
select 757,'PARD' from dual union all
select 7570,'RVSN' from dual union all
select 7571,'USA' from dual union all
select 7572,'USA' from dual union all
select 7573,'USA' from dual union all
select 7574,'STA' from dual union all
select 7575,'STA' from dual union all
select 7576,'AEC' from dual union all
select 7577,'NASA' from dual union all
select 7577,'USAF' from dual union all
select 7578,'NASA' from dual union all
select 7579,'USN' from dual union all
select 758,'USA' from dual union all
select 7580,'WRE' from dual union all
select 7581,'USA' from dual union all
select 7582,'USA' from dual union all
select 7583,'USA' from dual union all
select 7584,'USA' from dual union all
select 7585,'NASA' from dual union all
select 7586,'CNET' from dual union all
select 7587,'CNET' from dual union all
select 7588,'RAE' from dual union all
select 7588,'WRE' from dual union all
select 7589,'NASA' from dual union all
select 759,'USA' from dual union all
select 7590,'USAF' from dual union all
select 7591,'USA' from dual union all
select 7592,'USA' from dual union all
select 7593,'USA' from dual union all
select 7594,'USA' from dual union all
select 7595,'DRG' from dual union all
select 7596,'NASA' from dual union all
select 7597,'USAF' from dual union all
select 7598,'NASA' from dual union all
select 7599,'NASA' from dual union all
select 7599,'USAF' from dual union all
select 76,'JPL' from dual union all
select 760,'USAF' from dual union all
select 7600,'NASA' from dual union all
select 7601,'NASA' from dual union all
select 7602,'NASA' from dual union all
select 7603,'NASA' from dual union all
select 7604,'NMSU' from dual union all
select 7605,'NMSU' from dual union all
select 7606,'RVSN' from dual union all
select 7607,'USAF' from dual union all
select 7608,'USN' from dual union all
select 7609,'USN' from dual union all
select 761,'USA' from dual union all
select 7610,'WADD' from dual union all
select 7611,'SAND' from dual union all
select 7612,'WRE' from dual union all
select 7613,'WRE' from dual union all
select 7614,'RVSN' from dual union all
select 7615,'RVSN' from dual union all
select 7616,'RVSN' from dual union all
select 7617,'RVSN' from dual union all
select 7618,'RVSN' from dual union all
select 7619,'AFSWC' from dual union all
select 762,'USAF' from dual union all
select 7620,'NMC' from dual union all
select 7621,'RVSN' from dual union all
select 7622,'MVS' from dual union all
select 7623,'MVS' from dual union all
select 7624,'RAE' from dual union all
select 7624,'WRE' from dual union all
select 7625,'NASA' from dual union all
select 7626,'USN' from dual union all
select 7627,'MVS' from dual union all
select 7628,'RVSN' from dual union all
select 7629,'MVS' from dual union all
select 763,'GEMTS' from dual union all
select 763,'USA' from dual union all
select 7630,'USAF' from dual union all
select 7631,'USA' from dual union all
select 7632,'NASA' from dual union all
select 7633,'NRL' from dual union all
select 7634,'ISAS' from dual union all
select 7635,'USN' from dual union all
select 7636,'MVS' from dual union all
select 7637,'NASA' from dual union all
select 7638,'CRA' from dual;
insert into LAUNCH_AGENCY
select 7639,'CRA' from dual union all
select 764,'USA' from dual union all
select 7640,'CRA' from dual union all
select 7641,'CRA' from dual union all
select 7642,'USAF' from dual union all
select 7643,'NASA' from dual union all
select 7644,'RVSN' from dual union all
select 7645,'NASA' from dual union all
select 7646,'NASA' from dual union all
select 7647,'NASA' from dual union all
select 7648,'ABMA' from dual union all
select 7649,'I' from dual union all
select 765,'USA' from dual union all
select 7650,'MVS' from dual union all
select 7651,'NASA' from dual union all
select 7652,'MVS' from dual union all
select 7653,'USN' from dual union all
select 7654,'AFCRL' from dual union all
select 7655,'USN' from dual union all
select 7656,'RAE' from dual union all
select 7656,'WRE' from dual union all
select 7657,'USN' from dual union all
select 7658,'USAF' from dual union all
select 7659,'NASA' from dual union all
select 766,'USAF' from dual union all
select 7660,'NASA' from dual union all
select 7661,'NASA' from dual union all
select 7662,'USN' from dual union all
select 7663,'NASA' from dual union all
select 7664,'MVS' from dual union all
select 7665,'RAE' from dual union all
select 7666,'MVS' from dual union all
select 7667,'ONERA' from dual union all
select 7668,'USAF' from dual union all
select 7669,'USAF' from dual union all
select 767,'PARD' from dual union all
select 7670,'USAF' from dual union all
select 7671,'RVSN' from dual union all
select 7672,'USN' from dual union all
select 7673,'USN' from dual union all
select 7674,'USN' from dual union all
select 7675,'CRA' from dual union all
select 7676,'USN' from dual union all
select 7677,'USN' from dual union all
select 7678,'USA' from dual union all
select 7679,'NASA' from dual union all
select 768,'NRL' from dual union all
select 768,'US' from dual union all
select 7680,'USAF' from dual union all
select 7681,'ABMA' from dual union all
select 7682,'USAF' from dual union all
select 7683,'USAF' from dual union all
select 7684,'USAF' from dual union all
select 7685,'NASA' from dual union all
select 7686,'MVS' from dual union all
select 7687,'USAF' from dual union all
select 7688,'CNES' from dual union all
select 7689,'MVS' from dual union all
select 769,'ARDC' from dual union all
select 769,'USA' from dual union all
select 7690,'CNES' from dual union all
select 7691,'RAE' from dual union all
select 7691,'WRE' from dual union all
select 7692,'RVSN' from dual union all
select 7693,'RVSN' from dual union all
select 7694,'USAF' from dual union all
select 7695,'NASA' from dual union all
select 7696,'RAE' from dual union all
select 7696,'WRE' from dual union all
select 7697,'SKB30' from dual union all
select 7698,'RAE' from dual union all
select 7699,'MVS' from dual union all
select 77,'JPL' from dual union all
select 770,'USAF' from dual union all
select 7700,'SKB30' from dual union all
select 7701,'ABMA' from dual union all
select 7702,'NASA' from dual union all
select 7703,'CRSF' from dual union all
select 7704,'USN' from dual union all
select 7705,'NASA' from dual union all
select 7706,'MVS' from dual union all
select 7707,'NASA' from dual union all
select 7708,'MVS' from dual union all
select 7709,'RVSN' from dual union all
select 771,'NRL' from dual union all
select 771,'US' from dual union all
select 7710,'NRL' from dual union all
select 7711,'ISAS' from dual union all
select 7712,'RAF' from dual union all
select 7713,'NRL' from dual union all
select 7714,'USAF' from dual union all
select 7715,'SKB30' from dual union all
select 7716,'NASA' from dual union all
select 7717,'USN' from dual union all
select 7718,'NASA' from dual union all
select 7719,'SKB30' from dual union all
select 772,'GEMTS' from dual union all
select 7720,'USA' from dual union all
select 7721,'MVS' from dual union all
select 7722,'SKB30' from dual;
insert into LAUNCH_AGENCY
select 7723,'USA' from dual union all
select 7724,'USA' from dual union all
select 7725,'USA' from dual union all
select 7726,'USA' from dual union all
select 7727,'USA' from dual union all
select 7728,'ABMA' from dual union all
select 7729,'RVSN' from dual union all
select 773,'APL' from dual union all
select 7730,'RVSN' from dual union all
select 7731,'RVSN' from dual union all
select 7732,'USA' from dual union all
select 7733,'USAF' from dual union all
select 7734,'USN' from dual union all
select 7735,'NASA' from dual union all
select 7736,'NASA' from dual union all
select 7737,'NASA' from dual union all
select 7738,'NASA' from dual union all
select 7739,'RVSN' from dual union all
select 774,'UMI' from dual union all
select 774,'US' from dual union all
select 7740,'NASA' from dual union all
select 7741,'RVSN' from dual union all
select 7742,'USAF' from dual union all
select 7743,'ABMA' from dual union all
select 7744,'NASA' from dual union all
select 7745,'NASA' from dual union all
select 7746,'ISAS' from dual union all
select 7747,'NASA' from dual union all
select 7748,'MVS' from dual union all
select 7749,'MVS' from dual union all
select 775,'APL' from dual union all
select 7750,'RVSN' from dual union all
select 7751,'NASA' from dual union all
select 7752,'USAF' from dual union all
select 7753,'NASA' from dual union all
select 7754,'USAF' from dual union all
select 7755,'MVS' from dual union all
select 7756,'RVSN' from dual union all
select 7757,'USA' from dual union all
select 7758,'USAF' from dual union all
select 7759,'RAE' from dual union all
select 7759,'WRE' from dual union all
select 776,'USA' from dual union all
select 7760,'USN' from dual union all
select 7761,'USAF' from dual union all
select 7762,'NASA' from dual union all
select 7763,'I' from dual union all
select 7764,'RVSN' from dual union all
select 7765,'USAF' from dual union all
select 7766,'UNRV' from dual union all
select 7767,'UNRV' from dual union all
select 7768,'USAF' from dual union all
select 7769,'USN' from dual union all
select 777,'APL' from dual union all
select 777,'USA' from dual union all
select 7770,'USN' from dual union all
select 7771,'USN' from dual union all
select 7772,'USN' from dual union all
select 7773,'USA' from dual union all
select 7774,'USN' from dual union all
select 7775,'USN' from dual union all
select 7776,'NASA' from dual union all
select 7777,'MVS' from dual union all
select 7778,'USAF' from dual union all
select 7779,'NASA' from dual union all
select 778,'APL' from dual union all
select 7780,'UNRV' from dual union all
select 7781,'RVSN' from dual union all
select 7782,'NASA' from dual union all
select 7783,'ABMA' from dual union all
select 7784,'USAF' from dual union all
select 7785,'RVSN' from dual union all
select 7786,'RVSN' from dual union all
select 7787,'USAF' from dual union all
select 7788,'USAF' from dual union all
select 7789,'RVSN' from dual union all
select 779,'APL' from dual union all
select 779,'USA' from dual union all
select 7790,'RVSN' from dual union all
select 7791,'RVSN' from dual union all
select 7792,'USAF' from dual union all
select 7793,'NRL' from dual union all
select 7794,'MVS' from dual union all
select 7795,'USAF' from dual union all
select 7796,'NASA' from dual union all
select 7797,'MVS' from dual union all
select 7798,'MVS' from dual union all
select 7799,'RVSN' from dual union all
select 78,'JPL' from dual union all
select 780,'NRL' from dual union all
select 780,'USA' from dual union all
select 7800,'RVSN' from dual union all
select 7801,'USA' from dual union all
select 7802,'RVSN' from dual union all
select 7803,'RVSN' from dual union all
select 7804,'USAF' from dual union all
select 7805,'RAF' from dual union all
select 7806,'ABMA' from dual union all
select 7807,'USAF' from dual union all
select 7808,'CRA' from dual;
insert into LAUNCH_AGENCY
select 7809,'CRA' from dual union all
select 781,'NRL' from dual union all
select 7810,'CRA' from dual union all
select 7811,'USAF' from dual union all
select 7812,'RVSN' from dual union all
select 7813,'RVSN' from dual union all
select 7814,'MVS' from dual union all
select 7815,'RVSN' from dual union all
select 7816,'ABMA' from dual union all
select 7817,'USAF' from dual union all
select 7818,'MVS' from dual union all
select 7819,'RVSN' from dual union all
select 782,'UMI' from dual union all
select 782,'US' from dual union all
select 7820,'NASA' from dual union all
select 7821,'NASA' from dual union all
select 7822,'RVSN' from dual union all
select 7823,'RVSN' from dual union all
select 7824,'NASA' from dual union all
select 7825,'NASA' from dual union all
select 7826,'RVSN' from dual union all
select 7827,'NASA' from dual union all
select 7828,'RVSN' from dual union all
select 7829,'NASA' from dual union all
select 783,'USA' from dual union all
select 7830,'NASA' from dual union all
select 7831,'NASA' from dual union all
select 7832,'RVSN' from dual union all
select 7833,'AN' from dual union all
select 7834,'USAF' from dual union all
select 7835,'ABMA' from dual union all
select 7836,'MVS' from dual union all
select 7837,'RVSN' from dual union all
select 7838,'USAF' from dual union all
select 7839,'RVSN' from dual union all
select 784,'GAU4' from dual union all
select 7840,'RAE' from dual union all
select 7840,'WRE' from dual union all
select 7841,'USAF' from dual union all
select 7842,'USN' from dual union all
select 7843,'MVS' from dual union all
select 7844,'MVS' from dual union all
select 7845,'MVS' from dual union all
select 7846,'MVS' from dual union all
select 7847,'MVS' from dual union all
select 7848,'MVS' from dual union all
select 7849,'MVS' from dual union all
select 785,'GAU4' from dual union all
select 7850,'NASA' from dual union all
select 7851,'MVS' from dual union all
select 7852,'MVS' from dual union all
select 7853,'MVS' from dual union all
select 7854,'MVS' from dual union all
select 7855,'MVS' from dual union all
select 7856,'USA' from dual union all
select 7857,'NASA' from dual union all
select 7858,'USAF' from dual union all
select 7859,'USN' from dual union all
select 786,'GAU4' from dual union all
select 7860,'RVSN' from dual union all
select 7861,'MVS' from dual union all
select 7862,'NASA' from dual union all
select 7863,'ONERA' from dual union all
select 7864,'USAF' from dual union all
select 7865,'RVSN' from dual union all
select 7866,'RVSN' from dual union all
select 7867,'USAF' from dual union all
select 7868,'ONERA' from dual union all
select 7869,'RVSN' from dual union all
select 787,'GAU4' from dual union all
select 7870,'ABMA' from dual union all
select 7871,'NASA' from dual union all
select 7872,'USAF' from dual union all
select 7873,'RVSN' from dual union all
select 7874,'USAF' from dual union all
select 7875,'USAF' from dual union all
select 7876,'NASA' from dual union all
select 7877,'RVSN' from dual union all
select 7878,'USN' from dual union all
select 7879,'NASA' from dual union all
select 788,'GAU4' from dual union all
select 7880,'RVSN' from dual union all
select 7881,'RVSN' from dual union all
select 7882,'AN' from dual union all
select 7883,'AN' from dual union all
select 7884,'AN' from dual union all
select 7885,'RVSN' from dual union all
select 7886,'RVSN' from dual union all
select 7887,'USN' from dual union all
select 7888,'RVSN' from dual union all
select 7889,'RVSN' from dual union all
select 789,'APL' from dual union all
select 7890,'USAF' from dual union all
select 7891,'RAE' from dual union all
select 7891,'WRE' from dual union all
select 7892,'ISAS' from dual union all
select 7893,'RVSN' from dual union all
select 7894,'AN' from dual union all
select 7895,'USAF' from dual union all
select 7896,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 7897,'AN' from dual union all
select 7898,'AN' from dual union all
select 7899,'RVSN' from dual union all
select 79,'JPL' from dual union all
select 790,'GAU4' from dual union all
select 7900,'RVSN' from dual union all
select 7901,'NASA' from dual union all
select 7902,'NASA' from dual union all
select 7903,'RVSN' from dual union all
select 7904,'RVSN' from dual union all
select 7905,'NASA' from dual union all
select 7906,'ISAS' from dual union all
select 7907,'MVS' from dual union all
select 7908,'RVSN' from dual union all
select 7909,'NRL' from dual union all
select 791,'GAU4' from dual union all
select 7910,'NASA' from dual union all
select 7911,'ABMA' from dual union all
select 7912,'USN' from dual union all
select 7913,'USN' from dual union all
select 7914,'USN' from dual union all
select 7915,'RAE' from dual union all
select 7916,'USN' from dual union all
select 7917,'RAE' from dual union all
select 7917,'WRE' from dual union all
select 7918,'USN' from dual union all
select 7919,'US' from dual union all
select 792,'GAU4' from dual union all
select 7920,'ABMA' from dual union all
select 7921,'USA' from dual union all
select 7922,'USA' from dual union all
select 7923,'AN' from dual union all
select 7924,'NASA' from dual union all
select 7925,'USAF' from dual union all
select 7926,'RVSN' from dual union all
select 7927,'USA' from dual union all
select 7928,'USAF' from dual union all
select 7929,'NASA' from dual union all
select 793,'GEMTS' from dual union all
select 793,'US' from dual union all
select 7930,'NASA' from dual union all
select 7931,'NASA' from dual union all
select 7932,'USA' from dual union all
select 7933,'USAF' from dual union all
select 7934,'USAF' from dual union all
select 7935,'MVS' from dual union all
select 7936,'RVSN' from dual union all
select 7937,'MVS' from dual union all
select 7938,'RVSN' from dual union all
select 7939,'RVSN' from dual union all
select 794,'SCEL' from dual union all
select 794,'US' from dual union all
select 7940,'RVSN' from dual union all
select 7941,'USAF' from dual union all
select 7942,'SAC' from dual union all
select 7943,'MVS' from dual union all
select 7944,'MVS' from dual union all
select 7945,'USA' from dual union all
select 7946,'USA' from dual union all
select 7947,'NRL' from dual union all
select 7948,'MVS' from dual union all
select 7949,'ABMA' from dual union all
select 795,'USA' from dual union all
select 7950,'USN' from dual union all
select 7951,'RVSN' from dual union all
select 7952,'USAF' from dual union all
select 7953,'MVS' from dual union all
select 7954,'RVSN' from dual union all
select 7955,'USN' from dual union all
select 7956,'RAE' from dual union all
select 7956,'WRE' from dual union all
select 7957,'CNES' from dual union all
select 7958,'CNES' from dual union all
select 7959,'USAF' from dual union all
select 796,'PARD' from dual union all
select 7960,'RAF' from dual union all
select 7961,'I' from dual union all
select 7962,'SAC' from dual union all
select 7963,'USAF' from dual union all
select 7964,'RVSN' from dual union all
select 7965,'AFCRL' from dual union all
select 7966,'CNES' from dual union all
select 7967,'CNES' from dual union all
select 7968,'CNES' from dual union all
select 7969,'NASA' from dual union all
select 797,'USA' from dual union all
select 7970,'CASDN' from dual union all
select 7971,'USAF' from dual union all
select 7972,'USAF' from dual union all
select 7973,'USAF' from dual union all
select 7974,'USA' from dual union all
select 7975,'USA' from dual union all
select 7976,'NASA' from dual union all
select 7977,'WRE' from dual union all
select 7978,'USAF' from dual union all
select 7979,'USAF' from dual union all
select 798,'NRL' from dual union all
select 7980,'USAF' from dual union all
select 7981,'RVSN' from dual union all
select 7982,'USAF' from dual;
insert into LAUNCH_AGENCY
select 7983,'USAF' from dual union all
select 7984,'NASA' from dual union all
select 7985,'NASA' from dual union all
select 7986,'NASA' from dual union all
select 7987,'MVS' from dual union all
select 7988,'RVSN' from dual union all
select 7989,'RVSN' from dual union all
select 799,'PARD' from dual union all
select 7990,'RVSN' from dual union all
select 7991,'ISAS' from dual union all
select 7992,'RAE' from dual union all
select 7992,'WRE' from dual union all
select 7993,'RVSN' from dual union all
select 7994,'ABMA' from dual union all
select 7995,'MVS' from dual union all
select 7996,'RVSN' from dual union all
select 7997,'RVSN' from dual union all
select 7998,'MVS' from dual union all
select 7999,'RVSN' from dual union all
select 8,'WEHR' from dual union all
select 80,'JPL' from dual union all
select 800,'APL' from dual union all
select 8000,'MVS' from dual union all
select 8001,'NMC' from dual union all
select 8002,'USN' from dual union all
select 8003,'BRL' from dual union all
select 8004,'USN' from dual union all
select 8005,'USN' from dual union all
select 8006,'USA' from dual union all
select 8007,'SAND' from dual union all
select 8008,'USA' from dual union all
select 8009,'USN' from dual union all
select 801,'APL' from dual union all
select 8010,'MVS' from dual union all
select 8011,'RVSN' from dual union all
select 8012,'WRE' from dual union all
select 8013,'GURVO' from dual union all
select 8014,'NASA' from dual union all
select 8015,'RVSN' from dual union all
select 8016,'PRO' from dual union all
select 8017,'PRO' from dual union all
select 8018,'RVSN' from dual union all
select 8019,'RVSN' from dual union all
select 802,'PARD' from dual union all
select 8020,'RVSN' from dual union all
select 8021,'MVS' from dual union all
select 8022,'SKB30' from dual union all
select 8023,'SKB30' from dual union all
select 8024,'RVSN' from dual union all
select 8025,'AN' from dual union all
select 8026,'RVSN' from dual union all
select 8027,'RVSN' from dual union all
select 8028,'MVS' from dual union all
select 8029,'MVS' from dual union all
select 803,'PARD' from dual union all
select 8030,'MVS' from dual union all
select 8031,'MVS' from dual union all
select 8032,'MVS' from dual union all
select 8033,'RVSN' from dual union all
select 8034,'MVS' from dual union all
select 8035,'CASDN' from dual union all
select 8036,'ABMA' from dual union all
select 8037,'NASA' from dual union all
select 8038,'USAF' from dual union all
select 8039,'USAF' from dual union all
select 804,'PARD' from dual union all
select 8040,'BRL' from dual union all
select 8041,'SKB30' from dual union all
select 8042,'USAF' from dual union all
select 8043,'USAF' from dual union all
select 8044,'SKB30' from dual union all
select 8045,'CASDN' from dual union all
select 8046,'USAF' from dual union all
select 8047,'USAF' from dual union all
select 8048,'RVSN' from dual union all
select 8049,'USAF' from dual union all
select 805,'PARD' from dual union all
select 8050,'USN' from dual union all
select 8051,'SKB30' from dual union all
select 8052,'ABMA' from dual union all
select 8053,'USAF' from dual union all
select 8054,'ONERA' from dual union all
select 8055,'USAF' from dual union all
select 8056,'RVSN' from dual union all
select 8057,'SKB30' from dual union all
select 8058,'ONERA' from dual union all
select 8059,'RAE' from dual union all
select 8059,'WRE' from dual union all
select 806,'PARD' from dual union all
select 8060,'ONERA' from dual union all
select 8061,'USAF' from dual union all
select 8062,'AFCRL' from dual union all
select 8063,'USN' from dual union all
select 8064,'USA' from dual union all
select 8065,'MVS' from dual union all
select 8066,'MVS' from dual union all
select 8067,'USAF' from dual union all
select 8068,'USAF' from dual union all
select 8069,'USN' from dual union all
select 807,'PARD' from dual;
insert into LAUNCH_AGENCY
select 8070,'NASA' from dual union all
select 8071,'ABMA' from dual union all
select 8072,'MVS' from dual union all
select 8073,'USN' from dual union all
select 8074,'NASA' from dual union all
select 8075,'USAF' from dual union all
select 8076,'MVS' from dual union all
select 8077,'USAF' from dual union all
select 8078,'SKB30' from dual union all
select 8079,'NASA' from dual union all
select 808,'GEMTS' from dual union all
select 808,'USA' from dual union all
select 8080,'ISAS' from dual union all
select 8081,'MVS' from dual union all
select 8082,'MVS' from dual union all
select 8083,'USAF' from dual union all
select 8084,'RAF' from dual union all
select 8085,'USA' from dual union all
select 8086,'ISAS' from dual union all
select 8087,'USAF' from dual union all
select 8088,'NASA' from dual union all
select 8089,'NASA' from dual union all
select 809,'ORD' from dual union all
select 8090,'USAF' from dual union all
select 8091,'SMSA' from dual union all
select 8091,'WSMR' from dual union all
select 8092,'SMSA' from dual union all
select 8092,'WSMR' from dual union all
select 8093,'US' from dual union all
select 8094,'SMSA' from dual union all
select 8094,'WSMR' from dual union all
select 8095,'US' from dual union all
select 8096,'US' from dual union all
select 8097,'US' from dual union all
select 8098,'US' from dual union all
select 8099,'US' from dual union all
select 81,'JPL' from dual union all
select 810,'PARD' from dual union all
select 8100,'US' from dual union all
select 8101,'US' from dual union all
select 8102,'US' from dual union all
select 8103,'USAF' from dual union all
select 8104,'USAF' from dual union all
select 8105,'USAF' from dual union all
select 8106,'USAF' from dual union all
select 8107,'USAF' from dual union all
select 8108,'USAF' from dual union all
select 8109,'USAF' from dual union all
select 811,'USA' from dual union all
select 8110,'USAF' from dual union all
select 8111,'USAF' from dual union all
select 8112,'USAF' from dual union all
select 8113,'USAF' from dual union all
select 8114,'USAF' from dual union all
select 8115,'USAF' from dual union all
select 8116,'USAF' from dual union all
select 8117,'USAF' from dual union all
select 8118,'USAF' from dual union all
select 8119,'USAF' from dual union all
select 812,'USA' from dual union all
select 8120,'USAF' from dual union all
select 8121,'USAF' from dual union all
select 8122,'USAF' from dual union all
select 8123,'USAF' from dual union all
select 8124,'USAF' from dual union all
select 8125,'USAF' from dual union all
select 8126,'USAF' from dual union all
select 8127,'USAF' from dual union all
select 8128,'WXB' from dual union all
select 8129,'USN' from dual union all
select 813,'USA' from dual union all
select 8130,'USAF' from dual union all
select 8131,'USAF' from dual union all
select 8132,'NASA' from dual union all
select 8133,'NASA' from dual union all
select 8134,'RTG' from dual union all
select 8135,'HADC' from dual union all
select 8136,'NASA' from dual union all
select 8137,'USAF' from dual union all
select 8138,'USAF' from dual union all
select 8139,'RVSN' from dual union all
select 814,'PARD' from dual union all
select 8140,'NASA' from dual union all
select 8141,'USAF' from dual union all
select 8142,'USAF' from dual union all
select 8143,'RVSN' from dual union all
select 8144,'USAF' from dual union all
select 8145,'USAF' from dual union all
select 8146,'RVSN' from dual union all
select 8147,'RVSN' from dual union all
select 8148,'RVSN' from dual union all
select 8149,'AFSC' from dual union all
select 815,'ORD' from dual union all
select 8150,'RVSN' from dual union all
select 8151,'RVSN' from dual union all
select 8152,'NASA' from dual union all
select 8153,'USAF' from dual union all
select 8154,'USAF' from dual union all
select 8155,'RVSN' from dual union all
select 8156,'USAF' from dual;
insert into LAUNCH_AGENCY
select 8157,'USAF' from dual union all
select 8158,'RVSN' from dual union all
select 8159,'USAF' from dual union all
select 816,'ORD' from dual union all
select 8160,'USAF' from dual union all
select 8161,'NASA' from dual union all
select 8162,'USAF' from dual union all
select 8163,'NASA' from dual union all
select 8164,'USAF' from dual union all
select 8165,'USAF' from dual union all
select 8166,'USAF' from dual union all
select 8167,'NASA' from dual union all
select 8168,'RVSN' from dual union all
select 8169,'USAF' from dual union all
select 817,'PARD' from dual union all
select 8170,'NASA' from dual union all
select 8171,'USAF' from dual union all
select 8172,'RVSN' from dual union all
select 8173,'RVSN' from dual union all
select 8174,'USAF' from dual union all
select 8175,'USAF' from dual union all
select 8176,'AFSC' from dual union all
select 8177,'AFSC' from dual union all
select 8178,'USAF' from dual union all
select 8179,'USAF' from dual union all
select 818,'ARDC' from dual union all
select 818,'US' from dual union all
select 8180,'USAF' from dual union all
select 8181,'RVSN' from dual union all
select 8182,'NASA' from dual union all
select 8183,'RVSN' from dual union all
select 8184,'RVSN' from dual union all
select 8185,'NASA' from dual union all
select 8186,'USAF' from dual union all
select 8187,'USAF' from dual union all
select 8188,'USAF' from dual union all
select 8189,'USAF' from dual union all
select 819,'PARD' from dual union all
select 8190,'USAF' from dual union all
select 8191,'RVSN' from dual union all
select 8192,'USAF' from dual union all
select 8193,'USAF' from dual union all
select 8194,'USAF' from dual union all
select 8195,'RVSN' from dual union all
select 8196,'USAF' from dual union all
select 8197,'NASA' from dual union all
select 8198,'USAF' from dual union all
select 8199,'USAF' from dual union all
select 82,'JPL' from dual union all
select 820,'USA' from dual union all
select 8200,'USAF' from dual union all
select 8201,'USAF' from dual union all
select 8202,'USAF' from dual union all
select 8203,'USAF' from dual union all
select 8204,'RVSN' from dual union all
select 8205,'RVSN' from dual union all
select 8206,'RVSN' from dual union all
select 8207,'NASA' from dual union all
select 8208,'PRO' from dual union all
select 8209,'PRO' from dual union all
select 821,'PARD' from dual union all
select 8210,'PRO' from dual union all
select 8211,'USA' from dual union all
select 8212,'NASA' from dual union all
select 8212,'USAF' from dual union all
select 8213,'USAF' from dual union all
select 8214,'USAF' from dual union all
select 8215,'NASA' from dual union all
select 8216,'NASA' from dual union all
select 8216,'USAF' from dual union all
select 8217,'RAE' from dual union all
select 8218,'USA' from dual union all
select 8219,'NASA' from dual union all
select 8219,'USAF' from dual union all
select 822,'ARDC' from dual union all
select 8220,'CARDE' from dual union all
select 8221,'USAF' from dual union all
select 8222,'NASA' from dual union all
select 8222,'USAF' from dual union all
select 8223,'USA' from dual union all
select 8224,'SAND' from dual union all
select 8225,'USAF' from dual union all
select 8226,'NASA' from dual union all
select 8226,'USAF' from dual union all
select 8227,'NASA' from dual union all
select 8227,'USAF' from dual union all
select 8228,'USN' from dual union all
select 8229,'USAF' from dual union all
select 823,'PARD' from dual union all
select 8230,'RVSN' from dual union all
select 8231,'NASA' from dual union all
select 8231,'USAF' from dual union all
select 8232,'NASA' from dual union all
select 8232,'USAF' from dual union all
select 8233,'NASA' from dual union all
select 8233,'USAF' from dual union all
select 8234,'USAF' from dual union all
select 8235,'NASA' from dual union all
select 8235,'USAF' from dual union all
select 8236,'MVS' from dual;
insert into LAUNCH_AGENCY
select 8237,'RVSN' from dual union all
select 8238,'USA' from dual union all
select 8239,'USAF' from dual union all
select 824,'NRL' from dual union all
select 824,'USA' from dual union all
select 8240,'USA' from dual union all
select 8241,'USA' from dual union all
select 8242,'USA' from dual union all
select 8243,'USAF' from dual union all
select 8244,'ISAS' from dual union all
select 8245,'RAE' from dual union all
select 8246,'NASA' from dual union all
select 8246,'USAF' from dual union all
select 8247,'GMS' from dual union all
select 8248,'NASA' from dual union all
select 8248,'USAF' from dual union all
select 8249,'GMS' from dual union all
select 825,'GEMTS' from dual union all
select 825,'USA' from dual union all
select 8250,'USAF' from dual union all
select 8251,'GMS' from dual union all
select 8252,'NASA' from dual union all
select 8252,'USAF' from dual union all
select 8253,'NASA' from dual union all
select 8254,'USAF' from dual union all
select 8255,'GMS' from dual union all
select 8256,'NASA' from dual union all
select 8256,'USAF' from dual union all
select 8257,'USAF' from dual union all
select 8258,'GMS' from dual union all
select 8259,'GMS' from dual union all
select 826,'USA' from dual union all
select 8260,'TXWC' from dual union all
select 8261,'USA' from dual union all
select 8262,'USN' from dual union all
select 8263,'GMS' from dual union all
select 8264,'NASA' from dual union all
select 8264,'USAF' from dual union all
select 8265,'GMS' from dual union all
select 8266,'GMS' from dual union all
select 8267,'NASA' from dual union all
select 8267,'USAF' from dual union all
select 8268,'USA' from dual union all
select 8269,'USA' from dual union all
select 827,'USA' from dual union all
select 8270,'USA' from dual union all
select 8271,'USA' from dual union all
select 8272,'USAF' from dual union all
select 8273,'USAF' from dual union all
select 8274,'USA' from dual union all
select 8275,'USA' from dual union all
select 8276,'RAE' from dual union all
select 8277,'USAF' from dual union all
select 8278,'MVS' from dual union all
select 8279,'NASA' from dual union all
select 828,'USN' from dual union all
select 8280,'NASA' from dual union all
select 8281,'NASA' from dual union all
select 8281,'USAF' from dual union all
select 8282,'USAF' from dual union all
select 8283,'AFCRL' from dual union all
select 8284,'NASA' from dual union all
select 8284,'USAF' from dual union all
select 8285,'USA' from dual union all
select 8286,'USA' from dual union all
select 8287,'NASA' from dual union all
select 8288,'NASA' from dual union all
select 8288,'USAF' from dual union all
select 8289,'USN' from dual union all
select 829,'USA' from dual union all
select 8290,'NASA' from dual union all
select 8291,'NASA' from dual union all
select 8291,'USAF' from dual union all
select 8292,'NASA' from dual union all
select 8293,'NASA' from dual union all
select 8294,'USN' from dual union all
select 8295,'USAF' from dual union all
select 8296,'USAF' from dual union all
select 8297,'TXWC' from dual union all
select 8298,'NASA' from dual union all
select 8299,'RVSN' from dual union all
select 83,'JPL' from dual union all
select 830,'NRL' from dual union all
select 830,'US' from dual union all
select 8300,'PRO' from dual union all
select 8301,'NASA' from dual union all
select 8301,'USAF' from dual union all
select 8302,'RVSN' from dual union all
select 8303,'PRO' from dual union all
select 8304,'NASA' from dual union all
select 8305,'NASA' from dual union all
select 8306,'USA' from dual union all
select 8307,'USA' from dual union all
select 8308,'USA' from dual union all
select 8309,'USAF' from dual union all
select 831,'USA' from dual union all
select 8310,'PRO' from dual union all
select 8311,'NASA' from dual union all
select 8312,'USAF' from dual union all
select 8313,'USAF' from dual;
insert into LAUNCH_AGENCY
select 8314,'USN' from dual union all
select 8315,'USAF' from dual union all
select 8316,'NASA' from dual union all
select 8317,'NASA' from dual union all
select 8318,'USAF' from dual union all
select 8319,'NASA' from dual union all
select 8319,'USAF' from dual union all
select 832,'USA' from dual union all
select 8320,'NASA' from dual union all
select 8321,'NASA' from dual union all
select 8322,'NASA' from dual union all
select 8323,'NASA' from dual union all
select 8324,'NASA' from dual union all
select 8325,'DRG' from dual union all
select 8326,'USA' from dual union all
select 8327,'DRG' from dual union all
select 8328,'DRG' from dual union all
select 8329,'NASA' from dual union all
select 833,'NRL' from dual union all
select 8330,'RVSN' from dual union all
select 8331,'USAF' from dual union all
select 8332,'GMS' from dual union all
select 8333,'USA' from dual union all
select 8334,'RAE' from dual union all
select 8335,'ARPA' from dual union all
select 8336,'CNES' from dual union all
select 8337,'USAF' from dual union all
select 8338,'GMS' from dual union all
select 8339,'USAF' from dual union all
select 834,'APL' from dual union all
select 834,'USA' from dual union all
select 8340,'NASA' from dual union all
select 8341,'GSFC' from dual union all
select 8341,'NASA' from dual union all
select 8342,'NASA' from dual union all
select 8343,'GMS' from dual union all
select 8344,'USAF' from dual union all
select 8345,'NASA' from dual union all
select 8346,'GMS' from dual union all
select 8347,'USAF' from dual union all
select 8348,'USAF' from dual union all
select 8349,'NASA' from dual union all
select 835,'GEMTS' from dual union all
select 835,'USA' from dual union all
select 8350,'GMS' from dual union all
select 8351,'NASA' from dual union all
select 8351,'USAF' from dual union all
select 8352,'NASA' from dual union all
select 8353,'NASA' from dual union all
select 8354,'NASA' from dual union all
select 8354,'USAF' from dual union all
select 8355,'NASA' from dual union all
select 8356,'GMS' from dual union all
select 8357,'NASA' from dual union all
select 8358,'NASA' from dual union all
select 8359,'GMS' from dual union all
select 836,'APL' from dual union all
select 8360,'AFSC' from dual union all
select 8361,'NASA' from dual union all
select 8362,'NASA' from dual union all
select 8362,'USAF' from dual union all
select 8363,'NASA' from dual union all
select 8364,'GMS' from dual union all
select 8365,'NASA' from dual union all
select 8366,'GMS' from dual union all
select 8367,'NASA' from dual union all
select 8368,'RVSN' from dual union all
select 8369,'USA' from dual union all
select 837,'ARDC' from dual union all
select 837,'US' from dual union all
select 8370,'USA' from dual union all
select 8371,'USA' from dual union all
select 8372,'RAE' from dual union all
select 8373,'USA' from dual union all
select 8374,'USA' from dual union all
select 8375,'USA' from dual union all
select 8376,'USA' from dual union all
select 8377,'USA' from dual union all
select 8378,'USA' from dual union all
select 8379,'USA' from dual union all
select 838,'APL' from dual union all
select 8380,'NOTS' from dual union all
select 8381,'IIAE' from dual union all
select 8382,'IIAE' from dual union all
select 8383,'IIAE' from dual union all
select 8384,'IIAE' from dual union all
select 8385,'ISAS' from dual union all
select 8386,'ISAS' from dual union all
select 8387,'ISAS' from dual union all
select 8388,'ISAS' from dual union all
select 8389,'USA' from dual union all
select 839,'UMI' from dual union all
select 839,'US' from dual union all
select 8390,'ISAS' from dual union all
select 8391,'USA' from dual union all
select 8392,'USA' from dual union all
select 8393,'USAF' from dual union all
select 8394,'NASA' from dual union all
select 8395,'USAF' from dual union all
select 8396,'USA' from dual;
insert into LAUNCH_AGENCY
select 8397,'ONERA' from dual union all
select 8398,'RVSN' from dual union all
select 8399,'GB5' from dual union all
select 84,'JPL' from dual union all
select 840,'USN' from dual union all
select 8400,'ONERA' from dual union all
select 8401,'NOTS' from dual union all
select 8402,'USA' from dual union all
select 8403,'NASA' from dual union all
select 8403,'USAF' from dual union all
select 8404,'USN' from dual union all
select 8405,'NASA' from dual union all
select 8405,'USAF' from dual union all
select 8406,'PRO' from dual union all
select 8407,'USA' from dual union all
select 8408,'NASA' from dual union all
select 8408,'USAF' from dual union all
select 8409,'RAE' from dual union all
select 841,'NII88A' from dual union all
select 8410,'NASA' from dual union all
select 8411,'USAF' from dual union all
select 8412,'RAE' from dual union all
select 8412,'WRE' from dual union all
select 8413,'RVSN' from dual union all
select 8414,'USAF' from dual union all
select 8415,'GMS' from dual union all
select 8416,'GMS' from dual union all
select 8417,'NASA' from dual union all
select 8417,'USAF' from dual union all
select 8418,'USAF' from dual union all
select 8419,'USA' from dual union all
select 842,'NII88A' from dual union all
select 8420,'NASA' from dual union all
select 8420,'USAF' from dual union all
select 8421,'RVSN' from dual union all
select 8422,'USAF' from dual union all
select 8423,'AOMC' from dual union all
select 8424,'NASA' from dual union all
select 8424,'USAF' from dual union all
select 8425,'NASA' from dual union all
select 8425,'USAF' from dual union all
select 8426,'NASA' from dual union all
select 8426,'USAF' from dual union all
select 8427,'USA' from dual union all
select 8428,'USA' from dual union all
select 8429,'NASA' from dual union all
select 8429,'USAF' from dual union all
select 843,'NII88A' from dual union all
select 8430,'NASA' from dual union all
select 8431,'NASA' from dual union all
select 8432,'NASA' from dual union all
select 8433,'NOTS' from dual union all
select 8434,'NASA' from dual union all
select 8434,'USAF' from dual union all
select 8435,'NASA' from dual union all
select 8435,'USAF' from dual union all
select 8436,'NASA' from dual union all
select 8437,'NASA' from dual union all
select 8437,'USAF' from dual union all
select 8438,'NASA' from dual union all
select 8439,'CNES' from dual union all
select 844,'NII88A' from dual union all
select 8440,'AOMC' from dual union all
select 8441,'USN' from dual union all
select 8442,'CNES' from dual union all
select 8443,'GMS' from dual union all
select 8444,'NASA' from dual union all
select 8444,'USAF' from dual union all
select 8445,'GMS' from dual union all
select 8446,'GMS' from dual union all
select 8447,'ISAS' from dual union all
select 8448,'NASA' from dual union all
select 8448,'USAF' from dual union all
select 8449,'NASA' from dual union all
select 845,'NII88A' from dual union all
select 8450,'GMS' from dual union all
select 8451,'NASA' from dual union all
select 8451,'USAF' from dual union all
select 8452,'USA' from dual union all
select 8453,'USA' from dual union all
select 8454,'NASA' from dual union all
select 8454,'USAF' from dual union all
select 8455,'USN' from dual union all
select 8456,'CNES' from dual union all
select 8457,'NASA' from dual union all
select 8458,'NASA' from dual union all
select 8458,'USAF' from dual union all
select 8459,'USA' from dual union all
select 846,'NII88A' from dual union all
select 8460,'AOMC' from dual union all
select 8461,'TXWC' from dual union all
select 8462,'USAF' from dual union all
select 8463,'RAE' from dual union all
select 8464,'USAF' from dual union all
select 8465,'USAF' from dual union all
select 8466,'AFSC' from dual union all
select 8467,'USAF' from dual union all
select 8468,'AFSC' from dual union all
select 8469,'RVSN' from dual union all
select 847,'ARDC' from dual;
insert into LAUNCH_AGENCY
select 847,'US' from dual union all
select 8470,'USAF' from dual union all
select 8471,'RVSN' from dual union all
select 8472,'AFSC' from dual union all
select 8473,'AFSC' from dual union all
select 8474,'NASA' from dual union all
select 8475,'NASA' from dual union all
select 8476,'NASA' from dual union all
select 8476,'USAF' from dual union all
select 8477,'NASA' from dual union all
select 8477,'USAF' from dual union all
select 8478,'NASA' from dual union all
select 8478,'USAF' from dual union all
select 8479,'USA' from dual union all
select 848,'NRL' from dual union all
select 8480,'CNES' from dual union all
select 8481,'CNES' from dual union all
select 8482,'NASA' from dual union all
select 8482,'USAF' from dual union all
select 8483,'NASA' from dual union all
select 8483,'USAF' from dual union all
select 8484,'USA' from dual union all
select 8485,'USA' from dual union all
select 8486,'USA' from dual union all
select 8487,'EGAO' from dual union all
select 8488,'EGAO' from dual union all
select 8489,'AFCRL' from dual union all
select 8489,'RTG' from dual union all
select 849,'APL' from dual union all
select 8490,'NASA' from dual union all
select 8490,'USAF' from dual union all
select 8491,'USA' from dual union all
select 8492,'USA' from dual union all
select 8493,'USA' from dual union all
select 8494,'NASA' from dual union all
select 8495,'USAF' from dual union all
select 8496,'USA' from dual union all
select 8497,'USA' from dual union all
select 8498,'USAF' from dual union all
select 8499,'USA' from dual union all
select 85,'JPL' from dual union all
select 850,'APL' from dual union all
select 8500,'USA' from dual union all
select 8501,'USA' from dual union all
select 8502,'USN' from dual union all
select 8503,'CNES' from dual union all
select 8504,'CNES' from dual union all
select 8505,'ISAS' from dual union all
select 8506,'USA' from dual union all
select 8507,'USA' from dual union all
select 8508,'USA' from dual union all
select 8509,'NASA' from dual union all
select 851,'GAU4' from dual union all
select 8510,'USN' from dual union all
select 8511,'NASA' from dual union all
select 8512,'ONERA' from dual union all
select 8513,'USAF' from dual union all
select 8514,'USA' from dual union all
select 8515,'USA' from dual union all
select 8516,'USA' from dual union all
select 8517,'USA' from dual union all
select 8518,'USA' from dual union all
select 8519,'USA' from dual union all
select 852,'GAU4' from dual union all
select 8520,'USA' from dual union all
select 8521,'USA' from dual union all
select 8522,'USA' from dual union all
select 8523,'AFCRL' from dual union all
select 8524,'AFCRL' from dual union all
select 8525,'AN' from dual union all
select 8526,'AN' from dual union all
select 8527,'AN' from dual union all
select 8528,'AN' from dual union all
select 8529,'AN' from dual union all
select 853,'GAU4' from dual union all
select 8530,'USA' from dual union all
select 8531,'USA' from dual union all
select 8532,'RVSN' from dual union all
select 8533,'USAF' from dual union all
select 8534,'RVSN' from dual union all
select 8535,'NASA' from dual union all
select 8536,'NASA' from dual union all
select 8537,'NRL' from dual union all
select 8538,'RVSN' from dual union all
select 8539,'TUAF' from dual union all
select 854,'GAU4' from dual union all
select 8540,'USAF' from dual union all
select 8541,'RVSN' from dual union all
select 8542,'USN' from dual union all
select 8543,'NASA' from dual union all
select 8544,'MVS' from dual union all
select 8545,'RVSN' from dual union all
select 8546,'NASA' from dual union all
select 8546,'RRL' from dual union all
select 8547,'AOMC' from dual union all
select 8548,'USA' from dual union all
select 8549,'SAC' from dual union all
select 855,'GAU4' from dual union all
select 8550,'RVSN' from dual union all
select 8551,'AN' from dual;
insert into LAUNCH_AGENCY
select 8552,'USA' from dual union all
select 8553,'USAF' from dual union all
select 8554,'RAE' from dual union all
select 8555,'USAF' from dual union all
select 8556,'RVSN' from dual union all
select 8557,'USAF' from dual union all
select 8558,'USAF' from dual union all
select 8559,'USN' from dual union all
select 856,'GAU4' from dual union all
select 8560,'NASA' from dual union all
select 8561,'NASA' from dual union all
select 8562,'AFSC' from dual union all
select 8563,'USN' from dual union all
select 8564,'NASA' from dual union all
select 8565,'USAF' from dual union all
select 8566,'USN' from dual union all
select 8567,'NASA' from dual union all
select 8568,'SAC' from dual union all
select 8569,'NASA' from dual union all
select 8569,'RRL' from dual union all
select 857,'GAU4' from dual union all
select 8570,'MVS' from dual union all
select 8571,'NASA' from dual union all
select 8571,'RRL' from dual union all
select 8572,'USAF' from dual union all
select 8573,'CNES' from dual union all
select 8574,'NASA' from dual union all
select 8575,'GURVO' from dual union all
select 8576,'MVS' from dual union all
select 8577,'USN' from dual union all
select 8578,'USAF' from dual union all
select 8579,'USN' from dual union all
select 858,'GAU4' from dual union all
select 8580,'CNES' from dual union all
select 8581,'NASA' from dual union all
select 8582,'USA' from dual union all
select 8583,'MVS' from dual union all
select 8584,'CNES' from dual union all
select 8585,'CNES' from dual union all
select 8586,'CNES' from dual union all
select 8587,'MVS' from dual union all
select 8588,'USA' from dual union all
select 8589,'USAF' from dual union all
select 859,'GAU4' from dual union all
select 8590,'CNES' from dual union all
select 8591,'WRE' from dual union all
select 8592,'USAF' from dual union all
select 8593,'AFCRL' from dual union all
select 8594,'RVSN' from dual union all
select 8595,'CNES' from dual union all
select 8596,'USN' from dual union all
select 8597,'USN' from dual union all
select 8598,'AOMC' from dual union all
select 8599,'RVSN' from dual union all
select 86,'JPL' from dual union all
select 860,'GAU4' from dual union all
select 8600,'AOMC' from dual union all
select 8601,'CNES' from dual union all
select 8602,'USAF' from dual union all
select 8603,'CNES' from dual union all
select 8604,'CNES' from dual union all
select 8605,'CNES' from dual union all
select 8606,'CNES' from dual union all
select 8607,'CNES' from dual union all
select 8608,'USAF' from dual union all
select 8609,'RVSN' from dual union all
select 861,'GAU4' from dual union all
select 8610,'CNES' from dual union all
select 8611,'RVSN' from dual union all
select 8612,'RAE' from dual union all
select 8612,'WRE' from dual union all
select 8613,'NASA' from dual union all
select 8614,'MVS' from dual union all
select 8615,'RVSN' from dual union all
select 8616,'AFSC' from dual union all
select 8617,'NASA' from dual union all
select 8618,'NASA' from dual union all
select 8619,'SUPA' from dual union all
select 862,'GAU4' from dual union all
select 8620,'USAF' from dual union all
select 8621,'RVSN' from dual union all
select 8622,'RVSN' from dual union all
select 8623,'NASA' from dual union all
select 8624,'RVSN' from dual union all
select 8625,'SUPA' from dual union all
select 8626,'USAF' from dual union all
select 8627,'RVSN' from dual union all
select 8628,'CARDE' from dual union all
select 8629,'CARDE' from dual union all
select 863,'GAU4' from dual union all
select 8630,'MVS' from dual union all
select 8631,'NASA' from dual union all
select 8632,'CARDE' from dual union all
select 8633,'SAC' from dual union all
select 8634,'AFCRL' from dual union all
select 8635,'MVS' from dual union all
select 8636,'RVSN' from dual union all
select 8637,'RAF' from dual union all
select 8638,'USAF' from dual union all
select 8639,'RAE' from dual;
insert into LAUNCH_AGENCY
select 8639,'WRE' from dual union all
select 864,'GAU4' from dual union all
select 8640,'NASA' from dual union all
select 8641,'RVSN' from dual union all
select 8642,'MVS' from dual union all
select 8643,'MVS' from dual union all
select 8644,'SAC' from dual union all
select 8645,'WRE' from dual union all
select 8646,'SAC' from dual union all
select 8647,'MVS' from dual union all
select 8648,'ONERA' from dual union all
select 8649,'USN' from dual union all
select 865,'GAU4' from dual union all
select 8650,'MVS' from dual union all
select 8651,'USN' from dual union all
select 8652,'USAF' from dual union all
select 8653,'USN' from dual union all
select 8654,'MVS' from dual union all
select 8655,'RVSN' from dual union all
select 8656,'NASA' from dual union all
select 8657,'RVSN' from dual union all
select 8658,'RVSN' from dual union all
select 8659,'RVSN' from dual union all
select 866,'GAU4' from dual union all
select 8660,'USA' from dual union all
select 8661,'ONERA' from dual union all
select 8662,'NASA' from dual union all
select 8663,'USAF' from dual union all
select 8664,'RVSN' from dual union all
select 8665,'MVS' from dual union all
select 8666,'SAC' from dual union all
select 8667,'MVS' from dual union all
select 8668,'MVS' from dual union all
select 8669,'MVS' from dual union all
select 867,'GAU4' from dual union all
select 8670,'MVS' from dual union all
select 8671,'MVS' from dual union all
select 8672,'RAE' from dual union all
select 8672,'WRE' from dual union all
select 8673,'ONERA' from dual union all
select 8674,'USAF' from dual union all
select 8675,'USAF' from dual union all
select 8676,'USAF' from dual union all
select 8677,'AOMC' from dual union all
select 8678,'USAF' from dual union all
select 8679,'USAF' from dual union all
select 868,'GAU4' from dual union all
select 8680,'USAF' from dual union all
select 8681,'USAF' from dual union all
select 8682,'SAND' from dual union all
select 8683,'USAF' from dual union all
select 8684,'USAF' from dual union all
select 8685,'NASA' from dual union all
select 8686,'AFSC' from dual union all
select 8687,'USN' from dual union all
select 8688,'USAF' from dual union all
select 8689,'USN' from dual union all
select 869,'GAU4' from dual union all
select 8690,'SAC' from dual union all
select 8691,'USAF' from dual union all
select 8692,'USA' from dual union all
select 8693,'NASA' from dual union all
select 8693,'USAF' from dual union all
select 8694,'USAF' from dual union all
select 8695,'USAF' from dual union all
select 8696,'SAC' from dual union all
select 8697,'USA' from dual union all
select 8698,'USN' from dual union all
select 8699,'RVSN' from dual union all
select 87,'JPL' from dual union all
select 870,'GAU4' from dual union all
select 8700,'USN' from dual union all
select 8701,'USN' from dual union all
select 8702,'RVSN' from dual union all
select 8703,'RVSN' from dual union all
select 8704,'USAF' from dual union all
select 8705,'RVSN' from dual union all
select 8706,'AFCRL' from dual union all
select 8707,'USAF' from dual union all
select 8708,'NASA' from dual union all
select 8709,'USAF' from dual union all
select 871,'GAU4' from dual union all
select 8710,'USA' from dual union all
select 8711,'MVS' from dual union all
select 8712,'MVS' from dual union all
select 8713,'NOTS' from dual union all
select 8714,'AFSC' from dual union all
select 8715,'USN' from dual union all
select 8716,'USN' from dual union all
select 8717,'USN' from dual union all
select 8718,'USN' from dual union all
select 8719,'USN' from dual union all
select 872,'GAU4' from dual union all
select 8720,'MVS' from dual union all
select 8721,'MVS' from dual union all
select 8722,'NASA' from dual union all
select 8723,'MVS' from dual union all
select 8724,'RVSN' from dual union all
select 8725,'USA' from dual union all
select 8726,'USA' from dual;
insert into LAUNCH_AGENCY
select 8727,'I' from dual union all
select 8728,'USAF' from dual union all
select 8729,'USAF' from dual union all
select 873,'GAU4' from dual union all
select 8730,'RVSN' from dual union all
select 8731,'AFCRL' from dual union all
select 8732,'RVSN' from dual union all
select 8733,'USN' from dual union all
select 8734,'AFCRL' from dual union all
select 8734,'RTG' from dual union all
select 8735,'MVS' from dual union all
select 8736,'RVSN' from dual union all
select 8737,'NASA' from dual union all
select 8738,'SAC' from dual union all
select 8739,'SAC' from dual union all
select 874,'GAU4' from dual union all
select 8740,'USN' from dual union all
select 8741,'MVS' from dual union all
select 8742,'MVS' from dual union all
select 8743,'USAF' from dual union all
select 8744,'MVS' from dual union all
select 8745,'MVS' from dual union all
select 8746,'MVS' from dual union all
select 8747,'AFCRL' from dual union all
select 8747,'RTG' from dual union all
select 8748,'USAF' from dual union all
select 8749,'RAE' from dual union all
select 8749,'WRE' from dual union all
select 875,'GAU4' from dual union all
select 8750,'MVS' from dual union all
select 8751,'NTNF' from dual union all
select 8752,'AFCRL' from dual union all
select 8752,'RTG' from dual union all
select 8753,'AOMC' from dual union all
select 8754,'USAF' from dual union all
select 8755,'MVS' from dual union all
select 8756,'MVS' from dual union all
select 8757,'MVS' from dual union all
select 8758,'MVS' from dual union all
select 8759,'MVS' from dual union all
select 876,'UMI' from dual union all
select 876,'US' from dual union all
select 8760,'MVS' from dual union all
select 8761,'NRL' from dual union all
select 8762,'MVS' from dual union all
select 8763,'ISAS' from dual union all
select 8764,'NASA' from dual union all
select 8765,'MVS' from dual union all
select 8766,'USA' from dual union all
select 8767,'RAE' from dual union all
select 8768,'USN' from dual union all
select 8769,'WRE' from dual union all
select 877,'ARDC' from dual union all
select 8770,'USA' from dual union all
select 8771,'USAF' from dual union all
select 8772,'RAE' from dual union all
select 8772,'WRE' from dual union all
select 8773,'USAF' from dual union all
select 8774,'RVSN' from dual union all
select 8775,'USA' from dual union all
select 8776,'AFCRL' from dual union all
select 8776,'RTG' from dual union all
select 8777,'MVS' from dual union all
select 8778,'MVS' from dual union all
select 8779,'MVS' from dual union all
select 878,'ARDC' from dual union all
select 878,'US' from dual union all
select 8780,'MVS' from dual union all
select 8781,'MVS' from dual union all
select 8782,'MVS' from dual union all
select 8783,'USN' from dual union all
select 8784,'MVS' from dual union all
select 8785,'MVS' from dual union all
select 8786,'RAE' from dual union all
select 8786,'WRE' from dual union all
select 8787,'RAE' from dual union all
select 8787,'WRE' from dual union all
select 8788,'USAF' from dual union all
select 8789,'AFSC' from dual union all
select 879,'PARD' from dual union all
select 8790,'USN' from dual union all
select 8791,'USN' from dual union all
select 8792,'MVS' from dual union all
select 8793,'USAF' from dual union all
select 8794,'USAF' from dual union all
select 8795,'MVS' from dual union all
select 8796,'USAF' from dual union all
select 8797,'USA' from dual union all
select 8798,'MVS' from dual union all
select 8799,'RAE' from dual union all
select 8799,'WRE' from dual union all
select 88,'JPL' from dual union all
select 880,'PARD' from dual union all
select 8800,'NASA' from dual union all
select 8801,'NASA' from dual union all
select 8802,'AOMC' from dual union all
select 8803,'RVSN' from dual union all
select 8804,'USA' from dual union all
select 8805,'MVS' from dual union all
select 8806,'USAF' from dual;
insert into LAUNCH_AGENCY
select 8807,'MVS' from dual union all
select 8808,'NASA' from dual union all
select 8809,'AN' from dual union all
select 881,'USA' from dual union all
select 8810,'AN' from dual union all
select 8811,'AN' from dual union all
select 8812,'AN' from dual union all
select 8813,'AN' from dual union all
select 8814,'AN' from dual union all
select 8815,'AN' from dual union all
select 8816,'AN' from dual union all
select 8817,'USN' from dual union all
select 8818,'USN' from dual union all
select 8819,'RVSN' from dual union all
select 882,'USA' from dual union all
select 8820,'USN' from dual union all
select 8821,'USN' from dual union all
select 8822,'MVS' from dual union all
select 8823,'MVS' from dual union all
select 8824,'MVS' from dual union all
select 8825,'MVS' from dual union all
select 8826,'MVS' from dual union all
select 8827,'MVS' from dual union all
select 8828,'MVS' from dual union all
select 8829,'MVS' from dual union all
select 883,'USA' from dual union all
select 8830,'USAF' from dual union all
select 8831,'MVS' from dual union all
select 8832,'MVS' from dual union all
select 8833,'MVS' from dual union all
select 8834,'RVSN' from dual union all
select 8835,'RVSN' from dual union all
select 8836,'RVSN' from dual union all
select 8837,'RVSN' from dual union all
select 8838,'NASA' from dual union all
select 8839,'SAC' from dual union all
select 884,'USA' from dual union all
select 8840,'RVSN' from dual union all
select 8841,'USAF' from dual union all
select 8842,'RVSN' from dual union all
select 8843,'AFSC' from dual union all
select 8844,'USAF' from dual union all
select 8845,'USAF' from dual union all
select 8846,'RVSN' from dual union all
select 8847,'RVSN' from dual union all
select 8848,'USAF' from dual union all
select 8849,'AFSC' from dual union all
select 885,'USA' from dual union all
select 8850,'USAF' from dual union all
select 8851,'USN' from dual union all
select 8852,'SAC' from dual union all
select 8853,'CNES' from dual union all
select 8854,'USAF' from dual union all
select 8855,'RAE' from dual union all
select 8855,'WRE' from dual union all
select 8856,'RVSN' from dual union all
select 8857,'USAF' from dual union all
select 8858,'NASA' from dual union all
select 8859,'RVSN' from dual union all
select 886,'USA' from dual union all
select 8860,'USAF' from dual union all
select 8861,'USAF' from dual union all
select 8862,'USAF' from dual union all
select 8863,'MVS' from dual union all
select 8864,'AN' from dual union all
select 8865,'CNES' from dual union all
select 8866,'USAF' from dual union all
select 8867,'CNES' from dual union all
select 8868,'USAF' from dual union all
select 8869,'AFSC' from dual union all
select 887,'USA' from dual union all
select 8870,'AEC' from dual union all
select 8871,'USAF' from dual union all
select 8872,'USAF' from dual union all
select 8873,'AOMC' from dual union all
select 8874,'MVS' from dual union all
select 8875,'AN' from dual union all
select 8876,'RVSN' from dual union all
select 8877,'CNES' from dual union all
select 8878,'CNES' from dual union all
select 8879,'USAF' from dual union all
select 888,'USA' from dual union all
select 8880,'USAF' from dual union all
select 8881,'USAF' from dual union all
select 8882,'WRE' from dual union all
select 8883,'USAF' from dual union all
select 8884,'NASA' from dual union all
select 8885,'USAF' from dual union all
select 8886,'USAF' from dual union all
select 8887,'USAF' from dual union all
select 8888,'USAF' from dual union all
select 8889,'USAF' from dual union all
select 889,'ARDC' from dual union all
select 8890,'USAF' from dual union all
select 8891,'USAF' from dual union all
select 8892,'USAF' from dual union all
select 8893,'USAF' from dual union all
select 8894,'AOMC' from dual union all
select 8895,'USAF' from dual union all
select 8896,'USAF' from dual;
insert into LAUNCH_AGENCY
select 8897,'USAF' from dual union all
select 8898,'USAF' from dual union all
select 8899,'AFSC' from dual union all
select 89,'JPL' from dual union all
select 890,'USA' from dual union all
select 8900,'AFSC' from dual union all
select 8901,'USAF' from dual union all
select 8902,'RVSN' from dual union all
select 8903,'USAF' from dual union all
select 8904,'CNES' from dual union all
select 8905,'NASA' from dual union all
select 8906,'NASA' from dual union all
select 8907,'MVS' from dual union all
select 8908,'MVS' from dual union all
select 8909,'MVS' from dual union all
select 891,'USA' from dual union all
select 8910,'RVSN' from dual union all
select 8911,'RVSN' from dual union all
select 8912,'RVSN' from dual union all
select 8913,'RVSN' from dual union all
select 8914,'RVSN' from dual union all
select 8915,'MVS' from dual union all
select 8916,'ONERA' from dual union all
select 8917,'USAF' from dual union all
select 8918,'USAF' from dual union all
select 8919,'USAF' from dual union all
select 892,'PARD' from dual union all
select 8920,'USAF' from dual union all
select 8921,'USAF' from dual union all
select 8922,'USAF' from dual union all
select 8923,'USAF' from dual union all
select 8924,'USAF' from dual union all
select 8925,'USAF' from dual union all
select 8926,'NASA' from dual union all
select 8927,'USAF' from dual union all
select 8928,'USAF' from dual union all
select 8929,'USAF' from dual union all
select 893,'USA' from dual union all
select 8930,'RVSN' from dual union all
select 8931,'USAF' from dual union all
select 8932,'USAF' from dual union all
select 8933,'USAF' from dual union all
select 8934,'USAF' from dual union all
select 8935,'USAF' from dual union all
select 8936,'NASA' from dual union all
select 8937,'SAC' from dual union all
select 8938,'NASA' from dual union all
select 8939,'AFSC' from dual union all
select 894,'USA' from dual union all
select 8940,'USA' from dual union all
select 8941,'USA' from dual union all
select 8942,'USAF' from dual union all
select 8943,'RAE' from dual union all
select 8943,'WRE' from dual union all
select 8944,'USAF' from dual union all
select 8945,'AFSC' from dual union all
select 8946,'AOMC' from dual union all
select 8947,'USAF' from dual union all
select 8948,'RVSN' from dual union all
select 8949,'NASA' from dual union all
select 895,'USA' from dual union all
select 8950,'NASA' from dual union all
select 8951,'USAF' from dual union all
select 8952,'MVS' from dual union all
select 8953,'NASA' from dual union all
select 8954,'USAF' from dual union all
select 8955,'RVSN' from dual union all
select 8956,'RAE' from dual union all
select 8956,'WRE' from dual union all
select 8957,'MVS' from dual union all
select 8958,'RVSN' from dual union all
select 8959,'USN' from dual union all
select 896,'USA' from dual union all
select 8960,'CNES' from dual union all
select 8961,'USAF' from dual union all
select 8962,'AOMC' from dual union all
select 8963,'CNES' from dual union all
select 8964,'WRE' from dual union all
select 8965,'NASA' from dual union all
select 8966,'USAF' from dual union all
select 8967,'USA' from dual union all
select 8968,'CNES' from dual union all
select 8969,'RAE' from dual union all
select 8969,'WRE' from dual union all
select 897,'USA' from dual union all
select 8970,'USAF' from dual union all
select 8971,'MVS' from dual union all
select 8972,'USA' from dual union all
select 8973,'RAE' from dual union all
select 8974,'USA' from dual union all
select 8975,'USA' from dual union all
select 8976,'USA' from dual union all
select 8977,'RVSN' from dual union all
select 8978,'USA' from dual union all
select 8979,'MVS' from dual union all
select 898,'USA' from dual union all
select 8980,'MVS' from dual union all
select 8981,'NASA' from dual union all
select 8982,'MVS' from dual union all
select 8983,'CNES' from dual;
insert into LAUNCH_AGENCY
select 8984,'CNES' from dual union all
select 8985,'CNES' from dual union all
select 8986,'CNES' from dual union all
select 8987,'NASA' from dual union all
select 8988,'NASA' from dual union all
select 8989,'NASA' from dual union all
select 899,'PARD' from dual union all
select 8990,'NASA' from dual union all
select 8991,'NASA' from dual union all
select 8992,'USAF' from dual union all
select 8993,'NASA' from dual union all
select 8994,'NASA' from dual union all
select 8995,'AFCRL' from dual union all
select 8995,'NASA' from dual union all
select 8996,'CNES' from dual union all
select 8997,'USAF' from dual union all
select 8998,'USAF' from dual union all
select 8999,'USAF' from dual union all
select 9,'WEHR' from dual union all
select 90,'JPL' from dual union all
select 900,'PARD' from dual union all
select 9000,'NASA' from dual union all
select 9001,'RAE' from dual union all
select 9001,'WRE' from dual union all
select 9002,'WRE' from dual union all
select 9003,'NASA' from dual union all
select 9004,'CNES' from dual union all
select 9005,'AFSC' from dual union all
select 9006,'USAF' from dual union all
select 9007,'AFSC' from dual union all
select 9008,'NASA' from dual union all
select 9009,'MVS' from dual union all
select 901,'USA' from dual union all
select 9010,'NASA' from dual union all
select 9011,'NASA' from dual union all
select 9012,'NASA' from dual union all
select 9013,'USAF' from dual union all
select 9014,'NASA' from dual union all
select 9015,'AFSC' from dual union all
select 9016,'USAF' from dual union all
select 9017,'MVS' from dual union all
select 9018,'MVS' from dual union all
select 9019,'MVS' from dual union all
select 902,'PARD' from dual union all
select 9020,'CNES' from dual union all
select 9021,'USAF' from dual union all
select 9022,'CNES' from dual union all
select 9023,'AOMC' from dual union all
select 9024,'CRA' from dual union all
select 9025,'NASA' from dual union all
select 9026,'MVS' from dual union all
select 9027,'USAF' from dual union all
select 9028,'NTNF' from dual union all
select 9029,'RVSN' from dual union all
select 903,'ARDC' from dual union all
select 9030,'CNES' from dual union all
select 9031,'CNES' from dual union all
select 9032,'AFSC' from dual union all
select 9033,'USA' from dual union all
select 9034,'USAF' from dual union all
select 9035,'CARDE' from dual union all
select 9036,'NASA' from dual union all
select 9037,'CARDE' from dual union all
select 9038,'RVSN' from dual union all
select 9039,'USN' from dual union all
select 904,'PARD' from dual union all
select 9040,'USAF' from dual union all
select 9041,'MVS' from dual union all
select 9042,'NTNF' from dual union all
select 9043,'USAF' from dual union all
select 9044,'USAF' from dual union all
select 9045,'RVSN' from dual union all
select 9046,'ISAS' from dual union all
select 9047,'NASA' from dual union all
select 9048,'USAF' from dual union all
select 9049,'USN' from dual union all
select 905,'PARD' from dual union all
select 9050,'MVS' from dual union all
select 9051,'USAF' from dual union all
select 9052,'USA' from dual union all
select 9053,'AFSC' from dual union all
select 9054,'MVS' from dual union all
select 9055,'SAC' from dual union all
select 9056,'USA' from dual union all
select 9057,'MVS' from dual union all
select 9058,'RVSN' from dual union all
select 9059,'RVSN' from dual union all
select 906,'PARD' from dual union all
select 9060,'USAF' from dual union all
select 9061,'DASA' from dual union all
select 9062,'DASA' from dual union all
select 9063,'DASA' from dual union all
select 9064,'DASA' from dual union all
select 9065,'DASA' from dual union all
select 9066,'DASA' from dual union all
select 9067,'USA' from dual union all
select 9068,'DASA' from dual union all
select 9069,'SAND' from dual union all
select 907,'PARD' from dual union all
select 9070,'SAND' from dual;
insert into LAUNCH_AGENCY
select 9071,'SAND' from dual union all
select 9072,'SAND' from dual union all
select 9073,'SAND' from dual union all
select 9074,'SAND' from dual union all
select 9075,'SAND' from dual union all
select 9076,'SAND' from dual union all
select 9077,'SAND' from dual union all
select 9078,'RAE' from dual union all
select 9078,'WRE' from dual union all
select 9079,'SAND' from dual union all
select 908,'LRBA' from dual union all
select 9080,'SAND' from dual union all
select 9081,'SAND' from dual union all
select 9082,'SAND' from dual union all
select 9083,'SAND' from dual union all
select 9084,'SAND' from dual union all
select 9085,'SAND' from dual union all
select 9086,'SAND' from dual union all
select 9087,'SAND' from dual union all
select 9088,'SAND' from dual union all
select 9089,'MVS' from dual union all
select 909,'NRL' from dual union all
select 9090,'SAND' from dual union all
select 9091,'SAND' from dual union all
select 9092,'USAF' from dual union all
select 9093,'SAND' from dual union all
select 9094,'SAND' from dual union all
select 9095,'SAND' from dual union all
select 9096,'SAND' from dual union all
select 9097,'SAND' from dual union all
select 9098,'SAND' from dual union all
select 9099,'SAND' from dual union all
select 91,'JPL' from dual union all
select 910,'LRBA' from dual union all
select 9100,'RAE' from dual union all
select 9100,'WRE' from dual union all
select 9101,'SAND' from dual union all
select 9102,'SAND' from dual union all
select 9103,'SAND' from dual union all
select 9104,'USAF' from dual union all
select 9105,'USAF' from dual union all
select 9106,'USAF' from dual union all
select 9107,'USAF' from dual union all
select 9108,'USA' from dual union all
select 9109,'USA' from dual union all
select 911,'PARD' from dual union all
select 9110,'USA' from dual union all
select 9111,'USAF' from dual union all
select 9112,'USA' from dual union all
select 9113,'USA' from dual union all
select 9114,'USA' from dual union all
select 9115,'USA' from dual union all
select 9116,'USA' from dual union all
select 9117,'RVSN' from dual union all
select 9118,'LRS' from dual union all
select 9119,'USA' from dual union all
select 912,'PARD' from dual union all
select 9120,'NOTS' from dual union all
select 9121,'SAND' from dual union all
select 9122,'USAF' from dual union all
select 9123,'SAND' from dual union all
select 9124,'SAND' from dual union all
select 9125,'RVSN' from dual union all
select 9126,'RVSN' from dual union all
select 9127,'RVSN' from dual union all
select 9128,'RVSN' from dual union all
select 9129,'PRO' from dual union all
select 913,'PARD' from dual union all
select 9130,'PRO' from dual union all
select 9131,'MVS' from dual union all
select 9132,'MVS' from dual union all
select 9133,'USA' from dual union all
select 9134,'MVS' from dual union all
select 9135,'MVS' from dual union all
select 9136,'RVSN' from dual union all
select 9137,'RVSN' from dual union all
select 9138,'PRO' from dual union all
select 9139,'USA' from dual union all
select 914,'PARD' from dual union all
select 9140,'USN' from dual union all
select 9141,'AOMC' from dual union all
select 9142,'USAF' from dual union all
select 9143,'USAF' from dual union all
select 9144,'RAE' from dual union all
select 9144,'WRE' from dual union all
select 9145,'RAF' from dual union all
select 9146,'RVSN' from dual union all
select 9147,'USAF' from dual union all
select 9148,'NASA' from dual union all
select 9149,'NASA' from dual union all
select 915,'USA' from dual union all
select 9150,'SAC' from dual union all
select 9151,'NASA' from dual union all
select 9152,'RVSN' from dual union all
select 9153,'NASA' from dual union all
select 9154,'WRE' from dual union all
select 9155,'USA' from dual union all
select 9156,'USA' from dual union all
select 9157,'USA' from dual union all
select 9158,'MVS' from dual;
insert into LAUNCH_AGENCY
select 9159,'NASA' from dual union all
select 916,'PARD' from dual union all
select 9160,'AOMC' from dual union all
select 9161,'USA' from dual union all
select 9162,'RAE' from dual union all
select 9162,'WRE' from dual union all
select 9163,'MVS' from dual union all
select 9164,'RVSN' from dual union all
select 9165,'MVS' from dual union all
select 9166,'RVSN' from dual union all
select 9167,'RVSN' from dual union all
select 9168,'USA' from dual union all
select 9169,'SAC' from dual union all
select 917,'PARD' from dual union all
select 9170,'MVS' from dual union all
select 9171,'MVS' from dual union all
select 9172,'MVS' from dual union all
select 9173,'NASA' from dual union all
select 9174,'USAF' from dual union all
select 9175,'NASA' from dual union all
select 9176,'UKMO' from dual union all
select 9177,'NASA' from dual union all
select 9178,'NASA' from dual union all
select 9179,'NASA' from dual union all
select 918,'PARD' from dual union all
select 9180,'NASA' from dual union all
select 9181,'GMS' from dual union all
select 9182,'NASA' from dual union all
select 9183,'NASA' from dual union all
select 9184,'US' from dual union all
select 9185,'US' from dual union all
select 9186,'US' from dual union all
select 9187,'US' from dual union all
select 9188,'TXWC' from dual union all
select 9189,'US' from dual union all
select 919,'PARD' from dual union all
select 9190,'US' from dual union all
select 9191,'US' from dual union all
select 9192,'US' from dual union all
select 9193,'US' from dual union all
select 9194,'US' from dual union all
select 9195,'US' from dual union all
select 9196,'USAF' from dual union all
select 9197,'USAF' from dual union all
select 9198,'USAF' from dual union all
select 9199,'USAF' from dual union all
select 92,'JPL' from dual union all
select 920,'USA' from dual union all
select 9200,'USAF' from dual union all
select 9201,'TXWC' from dual union all
select 9202,'TXWC' from dual union all
select 9203,'US' from dual union all
select 9204,'US' from dual union all
select 9205,'TXWC' from dual union all
select 9206,'USAF' from dual union all
select 9207,'USAF' from dual union all
select 9208,'USAF' from dual union all
select 9209,'USAF' from dual union all
select 921,'USA' from dual union all
select 9210,'USAF' from dual union all
select 9211,'TXWC' from dual union all
select 9212,'USAF' from dual union all
select 9213,'USAF' from dual union all
select 9214,'USAF' from dual union all
select 9215,'USAF' from dual union all
select 9216,'USAF' from dual union all
select 9217,'USAF' from dual union all
select 9218,'USAF' from dual union all
select 9219,'USAF' from dual union all
select 922,'3GMS' from dual union all
select 9220,'USAF' from dual union all
select 9221,'USAF' from dual union all
select 9222,'USAF' from dual union all
select 9223,'TXWC' from dual union all
select 9224,'TXWC' from dual union all
select 9225,'TXWC' from dual union all
select 9226,'CNES' from dual union all
select 9227,'CNES' from dual union all
select 9228,'TXWC' from dual union all
select 9229,'TXWC' from dual union all
select 923,'3GMS' from dual union all
select 9230,'TXWC' from dual union all
select 9231,'TXWC' from dual union all
select 9232,'RVSN' from dual union all
select 9233,'USAF' from dual union all
select 9234,'USAF' from dual union all
select 9235,'NASA' from dual union all
select 9236,'AFSC' from dual union all
select 9237,'RVSN' from dual union all
select 9238,'USAF' from dual union all
select 9239,'RVSN' from dual union all
select 924,'BRL' from dual union all
select 924,'USA' from dual union all
select 9240,'NASA' from dual union all
select 9241,'RVSN' from dual union all
select 9242,'RVSN' from dual union all
select 9243,'RVSN' from dual union all
select 9244,'NASA' from dual union all
select 9245,'AFSC' from dual union all
select 9246,'USAF' from dual;
insert into LAUNCH_AGENCY
select 9247,'USAF' from dual union all
select 9248,'RVSN' from dual union all
select 9249,'RVSN' from dual union all
select 925,'USA' from dual union all
select 9250,'USAF' from dual union all
select 9251,'RVSN' from dual union all
select 9252,'USAF' from dual union all
select 9253,'AFSC' from dual union all
select 9254,'RVSN' from dual union all
select 9255,'NASA' from dual union all
select 9256,'USAF' from dual union all
select 9257,'NASA' from dual union all
select 9258,'USAF' from dual union all
select 9259,'AFSC' from dual union all
select 926,'PARD' from dual union all
select 9260,'USAF' from dual union all
select 9261,'AFSC' from dual union all
select 9262,'NASA' from dual union all
select 9263,'USAF' from dual union all
select 9264,'RVSN' from dual union all
select 9265,'USAF' from dual union all
select 9266,'USAF' from dual union all
select 9267,'AFSC' from dual union all
select 9268,'USAF' from dual union all
select 9269,'USAF' from dual union all
select 927,'USA' from dual union all
select 9270,'USAF' from dual union all
select 9271,'RVSN' from dual union all
select 9272,'AFSC' from dual union all
select 9273,'USAF' from dual union all
select 9274,'RVSN' from dual union all
select 9275,'RVSN' from dual union all
select 9276,'RVSN' from dual union all
select 9277,'NASA' from dual union all
select 9278,'NASA' from dual union all
select 9279,'USAF' from dual union all
select 928,'USA' from dual union all
select 9280,'USAF' from dual union all
select 9281,'RVSN' from dual union all
select 9282,'AFSC' from dual union all
select 9283,'RVSN' from dual union all
select 9284,'AFSC' from dual union all
select 9285,'NASA' from dual union all
select 9286,'USAF' from dual union all
select 9287,'PRO' from dual union all
select 9288,'NASA' from dual union all
select 9289,'USA' from dual union all
select 929,'PARD' from dual union all
select 9290,'PIHM' from dual union all
select 9291,'NASA' from dual union all
select 9292,'NASA' from dual union all
select 9293,'NASA' from dual union all
select 9294,'ARPA' from dual union all
select 9295,'NASA' from dual union all
select 9296,'NASA' from dual union all
select 9297,'SAND' from dual union all
select 9298,'NASA' from dual union all
select 9299,'SAND' from dual union all
select 93,'JPL' from dual union all
select 930,'PARD' from dual union all
select 9300,'NASA' from dual union all
select 9300,'USAF' from dual union all
select 9301,'AFSC' from dual union all
select 9302,'4505M' from dual union all
select 9303,'NASA' from dual union all
select 9303,'USAF' from dual union all
select 9304,'NASA' from dual union all
select 9305,'NASA' from dual union all
select 9305,'USAF' from dual union all
select 9306,'NASA' from dual union all
select 9307,'NASA' from dual union all
select 9308,'NASA' from dual union all
select 9309,'NASA' from dual union all
select 931,'PARD' from dual union all
select 9310,'RVSN' from dual union all
select 9311,'RAE' from dual union all
select 9311,'WRE' from dual union all
select 9312,'NASA' from dual union all
select 9313,'NASA' from dual union all
select 9314,'RAE' from dual union all
select 9314,'WRE' from dual union all
select 9315,'NASA' from dual union all
select 9315,'USAF' from dual union all
select 9316,'NASA' from dual union all
select 9317,'NASA' from dual union all
select 9317,'USAF' from dual union all
select 9318,'NASA' from dual union all
select 9319,'NASA' from dual union all
select 9319,'USAF' from dual union all
select 932,'USA' from dual union all
select 9320,'NASA' from dual union all
select 9321,'MSFC' from dual union all
select 9322,'NASA' from dual union all
select 9323,'USAF' from dual union all
select 9324,'NASA' from dual union all
select 9325,'NASA' from dual union all
select 9326,'NASA' from dual union all
select 9327,'RVSN' from dual union all
select 9328,'USA' from dual union all
select 9329,'NASA' from dual;
insert into LAUNCH_AGENCY
select 933,'USA' from dual union all
select 9330,'USA' from dual union all
select 9331,'USA' from dual union all
select 9332,'USA' from dual union all
select 9333,'ARPA' from dual union all
select 9334,'NASA' from dual union all
select 9335,'NASA' from dual union all
select 9336,'NASA' from dual union all
select 9337,'SAND' from dual union all
select 9338,'NASA' from dual union all
select 9338,'USAF' from dual union all
select 9339,'AFSC' from dual union all
select 934,'3GMS' from dual union all
select 9340,'NASA' from dual union all
select 9341,'NASA' from dual union all
select 9342,'NASA' from dual union all
select 9342,'USAF' from dual union all
select 9343,'SAC' from dual union all
select 9344,'NASA' from dual union all
select 9345,'NASA' from dual union all
select 9346,'NASA' from dual union all
select 9347,'NASA' from dual union all
select 9347,'USAF' from dual union all
select 9348,'NASA' from dual union all
select 9349,'RVSN' from dual union all
select 935,'PARD' from dual union all
select 9350,'NASA' from dual union all
select 9351,'NASA' from dual union all
select 9352,'44FA2' from dual union all
select 9353,'NASA' from dual union all
select 9353,'USAF' from dual union all
select 9354,'NASA' from dual union all
select 9354,'USAF' from dual union all
select 9355,'NASA' from dual union all
select 9356,'USA' from dual union all
select 9357,'USA' from dual union all
select 9358,'USA' from dual union all
select 9359,'USA' from dual union all
select 936,'USA' from dual union all
select 9360,'ARPA' from dual union all
select 9361,'ARPA' from dual union all
select 9362,'NASA' from dual union all
select 9363,'ARPA' from dual union all
select 9364,'NASA' from dual union all
select 9365,'RVSN' from dual union all
select 9366,'NASA' from dual union all
select 9367,'AFSC' from dual union all
select 9368,'NASA' from dual union all
select 9369,'NASA' from dual union all
select 9369,'USAF' from dual union all
select 937,'PARD' from dual union all
select 9370,'NASA' from dual union all
select 9371,'NASA' from dual union all
select 9372,'NASA' from dual union all
select 9372,'USAF' from dual union all
select 9373,'USA' from dual union all
select 9374,'NASA' from dual union all
select 9375,'ONERA' from dual union all
select 9376,'RVSN' from dual union all
select 9377,'NASA' from dual union all
select 9377,'USAF' from dual union all
select 9378,'RAE' from dual union all
select 9378,'WRE' from dual union all
select 9379,'NASA' from dual union all
select 938,'USA' from dual union all
select 9380,'NASA' from dual union all
select 9380,'USAF' from dual union all
select 9381,'ONERA' from dual union all
select 9382,'NASA' from dual union all
select 9382,'USAF' from dual union all
select 9383,'RVSN' from dual union all
select 9384,'NASA' from dual union all
select 9385,'DRG' from dual union all
select 9386,'NASA' from dual union all
select 9386,'USAF' from dual union all
select 9387,'NASA' from dual union all
select 9388,'RVSN' from dual union all
select 9389,'MVS' from dual union all
select 939,'USA' from dual union all
select 9390,'RVSN' from dual union all
select 9391,'NASA' from dual union all
select 9391,'USAF' from dual union all
select 9392,'NASA' from dual union all
select 9393,'CNES' from dual union all
select 9394,'IIAE' from dual union all
select 9395,'GMS' from dual union all
select 9396,'USN' from dual union all
select 9397,'NASA' from dual union all
select 9398,'NASA' from dual union all
select 9399,'USA' from dual union all
select 94,'WEHR' from dual union all
select 940,'USA' from dual union all
select 9400,'USA' from dual union all
select 9401,'USA' from dual union all
select 9402,'USA' from dual union all
select 9403,'GMS' from dual union all
select 9404,'USA' from dual union all
select 9405,'USA' from dual union all
select 9406,'ONR' from dual union all
select 9407,'ONR' from dual;
insert into LAUNCH_AGENCY
select 9408,'ONR' from dual union all
select 9409,'ONR' from dual union all
select 941,'PARD' from dual union all
select 9410,'ONR' from dual union all
select 9411,'ONR' from dual union all
select 9412,'ONR' from dual union all
select 9413,'ONR' from dual union all
select 9414,'NASA' from dual union all
select 9415,'RVSN' from dual union all
select 9416,'IIAE' from dual union all
select 9417,'IIAE' from dual union all
select 9418,'NASA' from dual union all
select 9419,'RVSN' from dual union all
select 942,'USAF' from dual union all
select 9420,'AFSC' from dual union all
select 9421,'USN' from dual union all
select 9422,'NASA' from dual union all
select 9423,'NASA' from dual union all
select 9424,'NASA' from dual union all
select 9425,'USA' from dual union all
select 9426,'MSFC' from dual union all
select 9427,'ONERA' from dual union all
select 9428,'USN' from dual union all
select 9429,'NASA' from dual union all
select 943,'USA' from dual union all
select 9430,'SAC' from dual union all
select 9431,'RVSN' from dual union all
select 9432,'NASA' from dual union all
select 9433,'NASA' from dual union all
select 9434,'GMS' from dual union all
select 9435,'NASA' from dual union all
select 9436,'NASA' from dual union all
select 9437,'NASA' from dual union all
select 9438,'SAC' from dual union all
select 9439,'MSFC' from dual union all
select 944,'APL' from dual union all
select 9440,'RAE' from dual union all
select 9441,'RVSN' from dual union all
select 9442,'NASA' from dual union all
select 9443,'NASA' from dual union all
select 9444,'NASA' from dual union all
select 9445,'NASA' from dual union all
select 9446,'NASA' from dual union all
select 9447,'USN' from dual union all
select 9448,'NASA' from dual union all
select 9448,'USAF' from dual union all
select 9449,'NASA' from dual union all
select 945,'APL' from dual union all
select 9450,'NASA' from dual union all
select 9451,'GMS' from dual union all
select 9452,'NASA' from dual union all
select 9453,'NASA' from dual union all
select 9454,'NASA' from dual union all
select 9455,'NASA' from dual union all
select 9455,'USAF' from dual union all
select 9456,'NASA' from dual union all
select 9457,'MSFC' from dual union all
select 9458,'NASA' from dual union all
select 9459,'NASA' from dual union all
select 946,'USN' from dual union all
select 9460,'NASA' from dual union all
select 9461,'NASA' from dual union all
select 9461,'USAF' from dual union all
select 9462,'PRO' from dual union all
select 9463,'AFSC' from dual union all
select 9464,'NASA' from dual union all
select 9465,'NASA' from dual union all
select 9466,'MSFC' from dual union all
select 9467,'USA' from dual union all
select 9468,'NASA' from dual union all
select 9468,'USAF' from dual union all
select 9469,'NASA' from dual union all
select 9469,'USAF' from dual union all
select 947,'USA' from dual union all
select 9470,'NASA' from dual union all
select 9471,'NASA' from dual union all
select 9471,'USAF' from dual union all
select 9472,'ONERA' from dual union all
select 9473,'AFSC' from dual union all
select 9474,'RVSN' from dual union all
select 9475,'ONERA' from dual union all
select 9476,'AFSC' from dual union all
select 9477,'NASA' from dual union all
select 9477,'USAF' from dual union all
select 9478,'ARPA' from dual union all
select 9479,'ARPA' from dual union all
select 948,'ARDC' from dual union all
select 9480,'USAF' from dual union all
select 9481,'USAF' from dual union all
select 9482,'NASA' from dual union all
select 9483,'USAF' from dual union all
select 9484,'USN' from dual union all
select 9485,'RVSN' from dual union all
select 9486,'CNES' from dual union all
select 9487,'NASA' from dual union all
select 9488,'RVSN' from dual union all
select 9489,'NASA' from dual union all
select 9489,'USAF' from dual union all
select 949,'ORD' from dual union all
select 9490,'USAF' from dual;
insert into LAUNCH_AGENCY
select 9491,'ARPA' from dual union all
select 9492,'ARPA' from dual union all
select 9493,'ARPA' from dual union all
select 9494,'RVSN' from dual union all
select 9495,'RVSN' from dual union all
select 9496,'RVSN' from dual union all
select 9497,'USAF' from dual union all
select 9498,'USAF' from dual union all
select 9499,'AFSC' from dual union all
select 95,'WER444' from dual union all
select 950,'USA' from dual union all
select 9500,'RVSN' from dual union all
select 9501,'AFSC' from dual union all
select 9502,'USAF' from dual union all
select 9503,'RVSN' from dual union all
select 9504,'AFSC' from dual union all
select 9505,'RVSN' from dual union all
select 9506,'AFSC' from dual union all
select 9507,'RVSN' from dual union all
select 9508,'USAF' from dual union all
select 9509,'RVSN' from dual union all
select 951,'1GMR' from dual union all
select 951,'GEMTS' from dual union all
select 9510,'HARP' from dual union all
select 9511,'USA' from dual union all
select 9512,'USA' from dual union all
select 9513,'USA' from dual union all
select 9514,'USA' from dual union all
select 9515,'USA' from dual union all
select 9516,'USA' from dual union all
select 9517,'NASA' from dual union all
select 9518,'USA' from dual union all
select 9519,'USA' from dual union all
select 952,'1GMR' from dual union all
select 952,'GEMTS' from dual union all
select 9520,'USA' from dual union all
select 9521,'USA' from dual union all
select 9522,'BWEHR' from dual union all
select 9523,'BWEHR' from dual union all
select 9524,'DRG' from dual union all
select 9525,'DRG' from dual union all
select 9526,'DRG' from dual union all
select 9527,'USA' from dual union all
select 9528,'USA' from dual union all
select 9529,'USA' from dual union all
select 953,'USA' from dual union all
select 9530,'USA' from dual union all
select 9531,'ONERA' from dual union all
select 9532,'ONERA' from dual union all
select 9533,'GSFC' from dual union all
select 9534,'EGAO' from dual union all
select 9535,'USA' from dual union all
select 9536,'DRG' from dual union all
select 9537,'NASA' from dual union all
select 9537,'USAF' from dual union all
select 9538,'CNES' from dual union all
select 9539,'USA' from dual union all
select 954,'USA' from dual union all
select 9540,'BWEHR' from dual union all
select 9541,'BWEHR' from dual union all
select 9542,'NASA' from dual union all
select 9542,'USAF' from dual union all
select 9543,'ONERA' from dual union all
select 9544,'ONERA' from dual union all
select 9545,'NASA' from dual union all
select 9545,'USAF' from dual union all
select 9546,'NASA' from dual union all
select 9547,'CNES' from dual union all
select 9548,'STA' from dual union all
select 9549,'STA' from dual union all
select 955,'ORD' from dual union all
select 9550,'STA' from dual union all
select 9551,'STA' from dual union all
select 9552,'ISAS' from dual union all
select 9553,'USAF' from dual union all
select 9554,'USA' from dual union all
select 9555,'USA' from dual union all
select 9556,'USA' from dual union all
select 9557,'ONERA' from dual union all
select 9558,'USN' from dual union all
select 9559,'USA' from dual union all
select 956,'USN' from dual union all
select 9560,'WRE' from dual union all
select 9561,'USN' from dual union all
select 9562,'NASA' from dual union all
select 9563,'USN' from dual union all
select 9564,'CNES' from dual union all
select 9565,'USA' from dual union all
select 9566,'USA' from dual union all
select 9567,'HARP' from dual union all
select 9568,'HARP' from dual union all
select 9569,'USA' from dual union all
select 957,'USA' from dual union all
select 9570,'USA' from dual union all
select 9571,'USA' from dual union all
select 9572,'USA' from dual union all
select 9573,'USA' from dual union all
select 9574,'USA' from dual union all
select 9575,'USA' from dual union all
select 9576,'USA' from dual;
insert into LAUNCH_AGENCY
select 9577,'USA' from dual union all
select 9578,'USA' from dual union all
select 9579,'CITEF' from dual union all
select 958,'NRL' from dual union all
select 9580,'CITEF' from dual union all
select 9581,'PRO' from dual union all
select 9582,'USA' from dual union all
select 9583,'USA' from dual union all
select 9584,'RVSN' from dual union all
select 9585,'RVSN' from dual union all
select 9586,'RVSN' from dual union all
select 9587,'RVSN' from dual union all
select 9588,'NASA' from dual union all
select 9589,'USAF' from dual union all
select 959,'NRL' from dual union all
select 959,'US' from dual union all
select 9590,'AFSC' from dual union all
select 9591,'SAC' from dual union all
select 9592,'NASA' from dual union all
select 9593,'SAC' from dual union all
select 9594,'USAF' from dual union all
select 9595,'USN' from dual union all
select 9596,'MVS' from dual union all
select 9597,'USAF' from dual union all
select 9598,'AOMC' from dual union all
select 9599,'AFSC' from dual union all
select 96,'WER485' from dual union all
select 960,'USA' from dual union all
select 9600,'CARDE' from dual union all
select 9601,'MVS' from dual union all
select 9602,'RVSN' from dual union all
select 9603,'AFSC' from dual union all
select 9604,'MVS' from dual union all
select 9605,'CARDE' from dual union all
select 9606,'USAF' from dual union all
select 9607,'MVS' from dual union all
select 9608,'NASA' from dual union all
select 9609,'NASA' from dual union all
select 961,'USN' from dual union all
select 9610,'WRE' from dual union all
select 9611,'NASA' from dual union all
select 9612,'MVS' from dual union all
select 9613,'NASA' from dual union all
select 9614,'NASA' from dual union all
select 9615,'USN' from dual union all
select 9616,'USN' from dual union all
select 9617,'RVSN' from dual union all
select 9618,'USA' from dual union all
select 9619,'SAC' from dual union all
select 962,'APL' from dual union all
select 9620,'RVSN' from dual union all
select 9621,'AOMC' from dual union all
select 9622,'CNES' from dual union all
select 9623,'AOMC' from dual union all
select 9624,'NASA' from dual union all
select 9625,'AFSC' from dual union all
select 9626,'NRL' from dual union all
select 9627,'AOMC' from dual union all
select 9628,'NRCC' from dual union all
select 9629,'NRCC' from dual union all
select 963,'ARDC' from dual union all
select 9630,'USN' from dual union all
select 9631,'MVS' from dual union all
select 9632,'NASA' from dual union all
select 9632,'USAF' from dual union all
select 9633,'NASA' from dual union all
select 9634,'USAF' from dual union all
select 9635,'MVS' from dual union all
select 9636,'AFSC' from dual union all
select 9637,'RVSN' from dual union all
select 9638,'AFSC' from dual union all
select 9639,'44FA2' from dual union all
select 964,'APL' from dual union all
select 9640,'NASA' from dual union all
select 9641,'KPNO' from dual union all
select 9642,'USA' from dual union all
select 9643,'I' from dual union all
select 9644,'SAC' from dual union all
select 9645,'44FA2' from dual union all
select 9646,'USN' from dual union all
select 9647,'USN' from dual union all
select 9648,'USN' from dual union all
select 9649,'MVS' from dual union all
select 965,'ARDC' from dual union all
select 965,'USA' from dual union all
select 9650,'MVS' from dual union all
select 9651,'MVS' from dual union all
select 9652,'RAE' from dual union all
select 9652,'WRE' from dual union all
select 9653,'USN' from dual union all
select 9654,'MVS' from dual union all
select 9655,'USN' from dual union all
select 9656,'USN' from dual union all
select 9657,'USA' from dual union all
select 9658,'NASA' from dual union all
select 9659,'AFSC' from dual union all
select 966,'GAU4' from dual union all
select 9660,'NASA' from dual union all
select 9661,'CNES' from dual union all
select 9662,'RVSN' from dual;
insert into LAUNCH_AGENCY
select 9663,'AFSC' from dual union all
select 9664,'CNES' from dual union all
select 9665,'SAND' from dual union all
select 9666,'NASA' from dual union all
select 9667,'44FA2' from dual union all
select 9668,'USA' from dual union all
select 9669,'MVS' from dual union all
select 967,'GAU4' from dual union all
select 9670,'AFSC' from dual union all
select 9671,'MVS' from dual union all
select 9672,'USN' from dual union all
select 9673,'RVSN' from dual union all
select 9674,'AFSC' from dual union all
select 9675,'SAND' from dual union all
select 9676,'AFSC' from dual union all
select 9677,'MVS' from dual union all
select 9678,'NASA' from dual union all
select 9679,'NASA' from dual union all
select 968,'ARDC' from dual union all
select 9680,'AFSC' from dual union all
select 9681,'NRL' from dual union all
select 9682,'CNES' from dual union all
select 9683,'DRG' from dual union all
select 9684,'USAF' from dual union all
select 9685,'NRCC' from dual union all
select 9686,'USAF' from dual union all
select 9687,'NASA' from dual union all
select 9688,'AFSC' from dual union all
select 9689,'NASA' from dual union all
select 969,'USA' from dual union all
select 9690,'AFSC' from dual union all
select 9691,'NASA' from dual union all
select 9692,'USN' from dual union all
select 9693,'CNES' from dual union all
select 9694,'NRL' from dual union all
select 9695,'AFCRL' from dual union all
select 9696,'AFSC' from dual union all
select 9697,'MVS' from dual union all
select 9698,'CNES' from dual union all
select 9699,'NASA' from dual union all
select 97,'WEHR' from dual union all
select 970,'GAU4' from dual union all
select 9700,'CNES' from dual union all
select 9701,'USAF' from dual union all
select 9702,'NASA' from dual union all
select 9703,'USN' from dual union all
select 9704,'USAF' from dual union all
select 9705,'RVSN' from dual union all
select 9706,'MVS' from dual union all
select 9707,'RVSN' from dual union all
select 9708,'RVSN' from dual union all
select 9709,'NASA' from dual union all
select 971,'GAU4' from dual union all
select 9710,'USAF' from dual union all
select 9711,'USAF' from dual union all
select 9712,'NASA' from dual union all
select 9713,'NASA' from dual union all
select 9714,'RVSN' from dual union all
select 9715,'ISAS' from dual union all
select 9716,'NASA' from dual union all
select 9717,'SAC' from dual union all
select 9718,'NASA' from dual union all
select 9719,'CNES' from dual union all
select 972,'GAU4' from dual union all
select 9720,'CNES' from dual union all
select 9721,'CNES' from dual union all
select 9722,'NRL' from dual union all
select 9723,'NASA' from dual union all
select 9724,'NASA' from dual union all
select 9725,'NASA' from dual union all
select 9726,'NASA' from dual union all
select 9727,'USA' from dual union all
select 9728,'AN' from dual union all
select 9729,'NASA' from dual union all
select 973,'GAU4' from dual union all
select 9730,'RAE' from dual union all
select 9730,'WRE' from dual union all
select 9731,'AFSC' from dual union all
select 9732,'AFSC' from dual union all
select 9733,'CNES' from dual union all
select 9734,'NASA' from dual union all
select 9735,'NASA' from dual union all
select 9736,'RVSN' from dual union all
select 9737,'CNES' from dual union all
select 9738,'NASA' from dual union all
select 9739,'GURVO' from dual union all
select 974,'GAU4' from dual union all
select 9740,'USAF' from dual union all
select 9741,'NASA' from dual union all
select 9742,'AFSC' from dual union all
select 9743,'APGC' from dual union all
select 9744,'RAE' from dual union all
select 9744,'WRE' from dual union all
select 9745,'CNES' from dual union all
select 9746,'CNES' from dual union all
select 9747,'USA' from dual union all
select 9748,'USA' from dual union all
select 9749,'USA' from dual union all
select 975,'GAU4' from dual union all
select 9750,'AFSC' from dual;
insert into LAUNCH_AGENCY
select 9751,'USA' from dual union all
select 9752,'RVSN' from dual union all
select 9753,'USN' from dual union all
select 9754,'USN' from dual union all
select 9755,'USN' from dual union all
select 9756,'USN' from dual union all
select 9757,'USN' from dual union all
select 9758,'USN' from dual union all
select 9759,'AFSC' from dual union all
select 976,'ARDC' from dual union all
select 9760,'USA' from dual union all
select 9761,'AOMC' from dual union all
select 9762,'AFSC' from dual union all
select 9763,'USN' from dual union all
select 9764,'MVS' from dual union all
select 9765,'AN' from dual union all
select 9766,'NRL' from dual union all
select 9767,'MVS' from dual union all
select 9768,'MVS' from dual union all
select 9769,'NRCC' from dual union all
select 977,'USA' from dual union all
select 9770,'USAF' from dual union all
select 9771,'SAND' from dual union all
select 9772,'WRE' from dual union all
select 9773,'CNES' from dual union all
select 9774,'SAC' from dual union all
select 9775,'USA' from dual union all
select 9776,'USN' from dual union all
select 9777,'CNES' from dual union all
select 9778,'MVS' from dual union all
select 9779,'CNES' from dual union all
select 978,'USN' from dual union all
select 9780,'USN' from dual union all
select 9781,'NASA' from dual union all
select 9782,'RAE' from dual union all
select 9782,'WRE' from dual union all
select 9783,'USA' from dual union all
select 9784,'HARP' from dual union all
select 9785,'1STRAD' from dual union all
select 9786,'CNES' from dual union all
select 9787,'AN' from dual union all
select 9788,'USN' from dual union all
select 9789,'NASA' from dual union all
select 979,'GAU4' from dual union all
select 9790,'AFSC' from dual union all
select 9791,'USN' from dual union all
select 9792,'RAE' from dual union all
select 9792,'WRE' from dual union all
select 9793,'NASA' from dual union all
select 9794,'USA' from dual union all
select 9795,'USN' from dual union all
select 9796,'USN' from dual union all
select 9797,'USN' from dual union all
select 9798,'USN' from dual union all
select 9799,'USN' from dual union all
select 98,'WEHR' from dual union all
select 980,'GAU4' from dual union all
select 9800,'USN' from dual union all
select 9801,'USN' from dual union all
select 9802,'CARDE' from dual union all
select 9803,'CARDE' from dual union all
select 9804,'SAND' from dual union all
select 9805,'SAC' from dual union all
select 9806,'KPNO' from dual union all
select 9807,'MVS' from dual union all
select 9808,'AFSC' from dual union all
select 9809,'ONERA' from dual union all
select 981,'GAU4' from dual union all
select 9810,'NASA' from dual union all
select 9810,'USAF' from dual union all
select 9811,'1STRAD' from dual union all
select 9812,'RVSN' from dual union all
select 9813,'NASA' from dual union all
select 9814,'RVSN' from dual union all
select 9815,'MVS' from dual union all
select 9816,'RVSN' from dual union all
select 9817,'USAF' from dual union all
select 9818,'USA' from dual union all
select 9819,'USA' from dual union all
select 982,'USA' from dual union all
select 9820,'RVSN' from dual union all
select 9821,'USN' from dual union all
select 9822,'AFSC' from dual union all
select 9823,'SAND' from dual union all
select 9824,'MVS' from dual union all
select 9825,'USA' from dual union all
select 9826,'USAF' from dual union all
select 9827,'USN' from dual union all
select 9828,'USN' from dual union all
select 9829,'NASA' from dual union all
select 983,'ARDC' from dual union all
select 9830,'USN' from dual union all
select 9831,'USA' from dual union all
select 9832,'USA' from dual union all
select 9833,'ONERA' from dual union all
select 9834,'USAF' from dual union all
select 9835,'SAC' from dual union all
select 9836,'AFSC' from dual union all
select 9837,'USN' from dual union all
select 9838,'MVS' from dual;
insert into LAUNCH_AGENCY
select 9839,'AN' from dual union all
select 984,'USA' from dual union all
select 9840,'RVSN' from dual union all
select 9841,'RVSN' from dual union all
select 9842,'NASA' from dual union all
select 9843,'USAF' from dual union all
select 9844,'1STRAD' from dual union all
select 9845,'MVS' from dual union all
select 9846,'NASA' from dual union all
select 9847,'USN' from dual union all
select 9848,'SAC' from dual union all
select 9849,'USN' from dual union all
select 985,'GAU4' from dual union all
select 9850,'MVS' from dual union all
select 9851,'NASA' from dual union all
select 9852,'NASA' from dual union all
select 9852,'USAF' from dual union all
select 9853,'NASA' from dual union all
select 9854,'NASA' from dual union all
select 9855,'USAF' from dual union all
select 9856,'NASA' from dual union all
select 9857,'NASA' from dual union all
select 9858,'USN' from dual union all
select 9859,'NASA' from dual union all
select 986,'NRL' from dual union all
select 986,'USA' from dual union all
select 9860,'NASA' from dual union all
select 9861,'NASA' from dual union all
select 9862,'AFSC' from dual union all
select 9863,'RVSN' from dual union all
select 9864,'USAF' from dual union all
select 9865,'USN' from dual union all
select 9866,'MVS' from dual union all
select 9867,'MVS' from dual union all
select 9868,'MVS' from dual union all
select 9869,'USN' from dual union all
select 987,'USA' from dual union all
select 9870,'USA' from dual union all
select 9871,'SAC' from dual union all
select 9872,'USAF' from dual union all
select 9873,'1STRAD' from dual union all
select 9874,'MVS' from dual union all
select 9875,'RVSN' from dual union all
select 9876,'RTG' from dual union all
select 9877,'NRL' from dual union all
select 9878,'RTG' from dual union all
select 9879,'SAC' from dual union all
select 988,'PARD' from dual union all
select 9880,'MVS' from dual union all
select 9881,'USA' from dual union all
select 9882,'USA' from dual union all
select 9883,'SAC' from dual union all
select 9884,'NRL' from dual union all
select 9885,'RTG' from dual union all
select 9886,'NMC' from dual union all
select 9887,'NASA' from dual union all
select 9888,'NASA' from dual union all
select 9889,'AFSC' from dual union all
select 989,'PARD' from dual union all
select 9890,'USA' from dual union all
select 9891,'AFSC' from dual union all
select 9892,'RTG' from dual union all
select 9893,'1STRAD' from dual union all
select 9894,'USA' from dual union all
select 9895,'USN' from dual union all
select 9896,'USN' from dual union all
select 9897,'MVS' from dual union all
select 9898,'USN' from dual union all
select 9899,'RVSN' from dual union all
select 99,'WEHR' from dual union all
select 990,'USAF' from dual union all
select 9900,'USA' from dual union all
select 9901,'SAC' from dual union all
select 9902,'USN' from dual union all
select 9903,'USN' from dual union all
select 9904,'USN' from dual union all
select 9905,'USN' from dual union all
select 9906,'USN' from dual union all
select 9907,'USN' from dual union all
select 9908,'USN' from dual union all
select 9909,'USN' from dual union all
select 991,'ARDC' from dual union all
select 991,'US' from dual union all
select 9910,'USA' from dual union all
select 9911,'44FA2' from dual union all
select 9912,'RVSN' from dual union all
select 9913,'MVS' from dual union all
select 9914,'AFSC' from dual union all
select 9915,'NASA' from dual union all
select 9915,'USAF' from dual union all
select 9916,'USN' from dual union all
select 9917,'USN' from dual union all
select 9918,'MVS' from dual union all
select 9919,'SAC' from dual union all
select 992,'USA' from dual union all
select 9920,'USA' from dual union all
select 9921,'AFSC' from dual union all
select 9922,'44FA2' from dual union all
select 9923,'SAC' from dual union all
select 9924,'USN' from dual;
insert into LAUNCH_AGENCY
select 9925,'AFSC' from dual union all
select 9926,'USN' from dual union all
select 9927,'MVS' from dual union all
select 9928,'RVSN' from dual union all
select 9929,'SAC' from dual union all
select 993,'PARD' from dual union all
select 9930,'RVSN' from dual union all
select 9931,'PRO' from dual union all
select 9932,'USA' from dual union all
select 9933,'USA' from dual union all
select 9934,'MVS' from dual union all
select 9935,'RVSN' from dual union all
select 9936,'NDRE' from dual union all
select 9937,'USAF' from dual union all
select 9938,'USN' from dual union all
select 9939,'USN' from dual union all
select 994,'LRBA' from dual union all
select 9940,'USN' from dual union all
select 9941,'USN' from dual union all
select 9942,'USN' from dual union all
select 9943,'USN' from dual union all
select 9944,'RVSN' from dual union all
select 9945,'NASA' from dual union all
select 9946,'NASA' from dual union all
select 9947,'HARP' from dual union all
select 9948,'RAE' from dual union all
select 9948,'WRE' from dual union all
select 9949,'MVS' from dual union all
select 995,'3GMS' from dual union all
select 9950,'NASA' from dual union all
select 9951,'SAC' from dual union all
select 9952,'MVS' from dual union all
select 9953,'NASA' from dual union all
select 9954,'RVSN' from dual union all
select 9955,'USA' from dual union all
select 9956,'NASA' from dual union all
select 9957,'MVS' from dual union all
select 9958,'SAC' from dual union all
select 9959,'RVSN' from dual union all
select 996,'PARD' from dual union all
select 9960,'NDRE' from dual union all
select 9961,'RVSN' from dual union all
select 9962,'SAC' from dual union all
select 9963,'MVS' from dual union all
select 9964,'MVS' from dual union all
select 9965,'NRL' from dual union all
select 9966,'MVS' from dual union all
select 9967,'USA' from dual union all
select 9968,'RVSN' from dual union all
select 9969,'AFSC' from dual union all
select 997,'PARD' from dual union all
select 9970,'44FA2' from dual union all
select 9971,'44FA2' from dual union all
select 9972,'USAF' from dual union all
select 9973,'44FA2' from dual union all
select 9974,'NASA' from dual union all
select 9975,'NASA' from dual union all
select 9976,'NASA' from dual union all
select 9977,'SAC' from dual union all
select 9978,'AFSC' from dual union all
select 9979,'SAC' from dual union all
select 998,'WADD' from dual union all
select 9980,'USN' from dual union all
select 9981,'RVSN' from dual union all
select 9982,'USAF' from dual union all
select 9983,'USAF' from dual union all
select 9984,'USAF' from dual union all
select 9985,'44FA2' from dual union all
select 9986,'44FA2' from dual union all
select 9987,'NASA' from dual union all
select 9988,'NASA' from dual union all
select 9989,'USA' from dual union all
select 999,'WADD' from dual union all
select 9990,'USA' from dual union all
select 9991,'USN' from dual union all
select 9992,'MVS' from dual union all
select 9993,'RVSN' from dual union all
select 9994,'USAF' from dual union all
select 9995,'NASA' from dual union all
select 9996,'RAE' from dual union all
select 9996,'WRE' from dual union all
select 9997,'AFSC' from dual union all
select 9998,'USN' from dual union all
select 9999,'MVS' from dual;


--------------------------------------------------------------------------------
-- SATELLITE
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing SATELLITE...');

CREATE TABLE "SATELLITE" 
 (	"SATELLITE_ID" NUMBER, 
	"NORAD_ID" VARCHAR2(28), 
	"COSPAR" VARCHAR2(15), 
	"OFFICIAL_NAME" VARCHAR2(41), 
	"SECONDARY_NAME" VARCHAR2(25), 
	"LAUNCH_ID" NUMBER, 
	"CURRENT_STATUS" VARCHAR2(19), 
	"STATUS_DATE" DATE, 
	"ORBIT_EPOCH_DATE" DATE, 
	"ORBIT_PERIOD" NUMBER, 
	"PERIGEE" NUMBER, 
	"APOGEE" NUMBER, 
	"INCLINATION" NUMBER, 
	"ORBIT_CLASS" VARCHAR2(35), 
	 CONSTRAINT "SATELLITE_PK" PRIMARY KEY ("SATELLITE_ID")
USING INDEX  ENABLE, 
	 CONSTRAINT "SATELLITE_FK" FOREIGN KEY ("LAUNCH_ID")
	  REFERENCES "LAUNCH" ("LAUNCH_ID") ENABLE
 ) ;


CREATE INDEX "SATELLITE_IDX1" ON "SATELLITE" ("LAUNCH_ID") 
;
insert into SATELLITE
select 1,'000001','57 Alpha 1','8K71A M1-10','8K71A M1-10 (M1-1PS)',4305,'Reentered','1957-12-01 00:00:00','1957-10-04 00:00:00',96.18,214,938,65.1,'Intermediate' from dual union all
select 10,'000010','58 Zeta','SCORE','SCORE',4799,'Reentered Attached','1959-01-21 00:00:00','1958-12-30 00:00:00',98.2,159,1187,32.29,'Intermediate' from dual union all
select 100,'000100','61 Lambda 1','Discoverer 23','ARGON 2',7269,'Reentered Attached','1962-04-16 00:00:00','1961-05-10 00:00:00',93.77,294,624,82.31,'Intermediate' from dual union all
select 1000,'001000','1965-008B','Transtage 3','Transtage 3',11758,'In Earth orbit','','1965-03-29 00:00:00',145.6,2783,2797,32.13,'Medium' from dual union all
select 10000,'010000','1977-034A','DSCS II F-7','DSCS II C-7',42162,'In Earth orbit','','1977-06-12 00:00:00',1436.06,35779,35793,2.38,'Stationary' from dual union all
select 10001,'010001','1977-034B','DSCS II F-8','DSCS II C-8',42162,'In Earth orbit','','1977-08-09 00:00:00',1436.09,35779,35794,2.25,'Stationary' from dual union all
select 10002,'010002','1977-034C','Transtage 32','Transtage 32',42162,'In Earth orbit','','1993-06-01 00:00:00',1506.85,35881,38439,12.18,'Very High' from dual union all
select 10003,'010003','1977-034D','Titan 23C-14 Stage 2','LR91AJ11 23C-14',42162,'Reentered','1977-05-14 00:00:00','1977-05-13 00:00:00',88.63,152,262,28.56,'Intermediate' from dual union all
select 10004,'010004','1977-033E','Blenda','Sunshade',42161,'Reentered','1977-05-17 00:00:00','1977-05-16 00:00:00',88.55,158,248,62.81,'Intermediate' from dual union all
select 10005,'010005','1977-033F','KDU','KDU',42161,'Reentered','1977-05-20 00:00:00','1977-05-16 00:00:00',89.28,163,315,62.8,'Intermediate' from dual union all
select 10006,'010006','1977-033G','KDU part','KDU part',42161,'Reentered','1977-05-24 00:00:00','1977-05-16 00:00:00',88.79,160,270,62.78,'Intermediate' from dual union all
select 10007,'010007','1977-035A','Kosmos-908','Zenit-4MK',42163,'Landed','1977-05-31 00:00:00','1977-05-19 00:00:00',89.09,174,285,51.79,'Intermediate' from dual union all
select 10008,'010008','1977-035B','11S510','11S510',42163,'Reentered','1977-05-21 00:00:00','1977-05-20 00:00:00',88.2,158,213,51.79,'Intermediate' from dual union all
select 10009,'010009','1977-035C','Sensor cover','Zenit-4MK sensor cover',42163,'Reentered','1977-05-22 00:00:00','1977-05-17 00:00:00',88.73,164,260,51.8,'Intermediate' from dual union all
select 1001,'001001','1965-008A','Transtage Payload Truss','Payload Truss',11758,'In Earth orbit','','1965-03-21 00:00:00',145.34,2761,2797,32.14,'Medium' from dual union all
select 10010,'010010','1977-036A','Kosmos-909','Lira',42164,'In Earth orbit','','1977-06-18 00:00:00',117.04,990,2106,65.87,'Intermediate' from dual union all
select 10011,'010011','1977-036B','S3M','S3M',42164,'In Earth orbit','','1993-08-08 00:00:00',116.87,988,2093,65.86,'Intermediate' from dual union all
select 10012,'010012','1977-019B','Fairing','GAMBIT SRV-2 Fairing',42147,'Reentered','1977-05-21 00:00:00','1977-05-21 00:00:00',88.5,115,286,96.42,'Sun-Synch' from dual union all
select 10013,'010013','1977-036C','deb Kosmos-909','DS-P1-M part [RCS 0.55]',42164,'In Earth orbit','','1993-08-07 00:00:00',117.02,991,2103,65.86,'Intermediate' from dual union all
select 10014,'010014','1977-037A','Kosmos-910','IS',42165,'Deorbited','1977-05-23 00:00:00','1977-05-23 00:00:00',107.69,465,1775,65.9,'Intermediate' from dual union all
select 10015,'010015','1977-037B','11S692','11S692',42165,'Reentered','1977-05-27 00:00:00','1977-05-25 00:00:00',89.24,139,334,65.1,'Intermediate' from dual union all
select 10016,'010016','1977-038A','OPS 9751','CANYON 7',42166,'In Earth orbit','','1977-05-31 00:00:00',1421.47,31000,40000,9,'Near-synch' from dual union all
select 10017,'010017','1977-038B','Agena D 2807','Agena D',42166,'In Earth orbit','1990-01-01 00:00:00','1977-05-31 00:00:00',714.64,200,40000,27,'GEO Transfer' from dual union all
select 10018,'010018','1977-037C','deb Kosmos-910','',42165,'Reentered','1977-05-24 00:00:00','1977-05-24 00:00:00',88.08,132,227,65.27,'Intermediate' from dual union all
select 10019,'010019','1977-039A','Kosmos-911','Geoid',42167,'In Earth orbit','','1992-05-24 00:00:00',104.74,964,1001,82.95,'Intermediate' from dual union all
select 1002,'001002','1965-008C','LES 1','LES 1',11758,'In Earth orbit','','1965-03-29 00:00:00',145.69,2782,2805,32.14,'Medium' from dual union all
select 10020,'010020','1977-039B','S3M','S3M',42167,'In Earth orbit','','1991-03-25 00:00:00',104.58,952,998,82.95,'Intermediate' from dual union all
select 10021,'010021','1977-040A','Kosmos-912','Fram',42168,'Landed','1977-06-08 00:00:00','1977-05-28 00:00:00',88.91,214,228,81.36,'Intermediate' from dual union all
select 10022,'010022','1977-040B','11S510','11S510',42168,'Reentered','1977-05-31 00:00:00','1977-05-28 00:00:00',88.61,197,215,81.37,'Intermediate' from dual union all
select 10023,'010023','1977-030D','deb Agregatniy otsek','deb  AO-PO',42158,'Reentered','1977-05-26 00:00:00','1977-05-26 00:00:00',89.11,166,294,67.12,'Intermediate' from dual union all
select 10024,'010024','1977-041A','INTELSAT IVA F4','INTELSAT IVA F4',42169,'In Earth orbit','','1977-06-28 00:00:00',1436.11,35779,35795,.05,'Stationary' from dual union all
select 10025,'010025','1977-041B','Centaur AC-39','Centaur D-1AR AC-39',42169,'In Earth orbit','','1977-07-03 00:00:00',649.74,606,36337,21.89,'GEO Transfer' from dual union all
select 10026,'010026','1977-040C','Sensor cover','Fram sensor cover',42168,'Reentered','1977-05-27 00:00:00','1977-05-26 00:00:00',88.86,212,224,81.34,'Intermediate' from dual union all
select 10027,'010027','1977-035D','KDU','KDU',42163,'Reentered','1977-06-03 00:00:00','1977-05-30 00:00:00',88.6,167,243,51.8,'Intermediate' from dual union all
select 10028,'010028','1977-042A','Kosmos-913','Romb 3',42170,'Reentered','1979-12-29 00:00:00','1977-06-30 00:00:00',94.5,470,519,74.04,'Intermediate' from dual union all
select 10029,'010029','1977-042B','S3M','S3M',42170,'Reentered','1979-11-02 00:00:00','1977-06-29 00:00:00',94.34,460,514,74.04,'Intermediate' from dual union all
select 1003,'001003','1964-070R','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',88.36,163,224,51.4,'Intermediate' from dual union all
select 10030,'010030','1977-043A','Kosmos-914','Zenit-2M',42171,'Landed','1977-06-13 00:00:00','1977-06-06 00:00:00',89.52,201,300,65,'Intermediate' from dual union all
select 10031,'010031','1977-043B','11S510','11S510',42171,'Reentered','1977-06-11 00:00:00','1977-06-02 00:00:00',89.37,203,283,65.02,'Intermediate' from dual union all
select 10032,'010032','1977-043C','Sensor cover','Zenit-2M sensor cover',42171,'Reentered','1977-06-16 00:00:00','1977-06-07 00:00:00',88.87,188,250,65.03,'Intermediate' from dual union all
select 10033,'010033','1977-044A','DMSP F-2','DMSP Block 5D S-2',42172,'In Earth orbit','','1991-09-14 00:00:00',101.11,790,833,99.03,'Sun-Synch' from dual union all
select 10034,'010034','1977-044B','DMSP F-2 solar array band','',42172,'Reentered','1984-01-01 00:00:00','1977-06-30 00:00:00',101.4,790,860,99.1,'Sun-Synch' from dual union all
select 10035,'010035','1977-040D','Blenda','Sunshade',42168,'Reentered','1977-06-12 00:00:00','1977-06-07 00:00:00',88.77,208,220,81.36,'Intermediate' from dual union all
select 10036,'010036','1977-040E','KDU','KDU',42168,'Reentered','1977-06-18 00:00:00','1977-06-08 00:00:00',88.71,207,215,81.35,'Intermediate' from dual union all
select 10037,'010037','1977-044C','DMSP F-2 solar array band','',42172,'Reentered','1984-01-01 00:00:00','1977-06-30 00:00:00',101.4,790,860,99.1,'Sun-Synch' from dual union all
select 10038,'010038','1977-045A','Kosmos-915','Zenit-4MK',42173,'Landed','1977-06-21 00:00:00','1977-06-10 00:00:00',89.29,176,303,62.81,'Intermediate' from dual union all
select 10039,'010039','1977-045B','11S510','11S510',42173,'Reentered','1977-06-11 00:00:00','1977-06-10 00:00:00',88.61,160,252,62.79,'Intermediate' from dual union all
select 1004,'001004','1964-070S','deb Kosmos-50','',10356,'Reentered','1964-11-14 00:00:00','1964-11-05 00:00:00',89.94,195,348,51.1,'Intermediate' from dual union all
select 10040,'010040','1977-046A','Kosmos-916','Orion',42174,'Landed','1977-06-21 00:00:00','1977-06-12 00:00:00',89.93,245,297,62.8,'Intermediate' from dual union all
select 10041,'010041','1977-046B','11S510','11S510',42174,'Reentered','1977-07-01 00:00:00','1977-06-17 00:00:00',89.58,232,275,62.79,'Intermediate' from dual union all
select 10042,'010042','1977-046C','Sensor cover','Zenit-4MT sensor cover',42174,'Reentered','1977-06-29 00:00:00','1977-06-17 00:00:00',90.13,249,313,62.81,'Intermediate' from dual union all
select 10043,'010043','1977-046D','Sensor cover','Zenit-4MT sensor cover',42174,'Reentered','1977-06-18 00:00:00','1977-06-12 00:00:00',89.94,249,294,62.77,'Intermediate' from dual union all
select 10044,'010044','1977-046E','Sensor cover','Zenit-4MT sensor cover',42174,'Reentered','1977-06-25 00:00:00','1977-06-10 00:00:00',89.72,240,282,62.76,'Intermediate' from dual union all
select 10045,'010045','1977-043D','Nauka part','',42171,'Reentered','1977-06-18 00:00:00','1977-06-12 00:00:00',89.28,196,282,64.98,'Intermediate' from dual union all
select 10046,'010046','1977-043E','Kapsula Kosmosa-914','Nauka 27KS',42171,'Reentered','1977-07-02 00:00:00','1977-06-13 00:00:00',89.41,200,291,65.01,'Intermediate' from dual union all
select 10047,'010047','1976-063C','deb Kosmos-838','',39015,'Reentered','1977-09-13 00:00:00','1977-07-01 00:00:00',94.04,396,549,64.96,'Intermediate' from dual union all
select 10048,'010048','1976-063D','deb Kosmos-838','',39015,'Reentered','1977-08-14 00:00:00','1977-07-28 00:00:00',93.3,380,493,64.92,'Intermediate' from dual union all
select 10049,'010049','1976-063E','deb Kosmos-838','',39015,'Reentered','1978-04-10 00:00:00','1978-03-26 00:00:00',91.71,327,389,64.58,'Intermediate' from dual union all
select 1005,'001005','1964-070T','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.28,166,213,51.2,'Intermediate' from dual union all
select 10050,'010050','1976-063F','deb Kosmos-838','',39015,'Reentered','1977-07-23 00:00:00','1977-07-15 00:00:00',92.6,359,444,65.1,'Intermediate' from dual union all
select 10051,'010051','1976-063G','deb Kosmos-838','',39015,'Reentered','1977-11-01 00:00:00','1977-10-23 00:00:00',91.53,350,350,65.03,'Intermediate' from dual union all
select 10052,'010052','1976-063H','deb Kosmos-838','',39015,'Reentered','1978-03-25 00:00:00','1978-02-22 00:00:00',92.6,384,420,64.56,'Intermediate' from dual union all
select 10053,'010053','1976-063J','deb Kosmos-838','',39015,'Reentered','1977-11-15 00:00:00','1977-10-26 00:00:00',91.53,344,356,64.95,'Intermediate' from dual union all
select 10054,'010054','1976-063K','deb Kosmos-838','',39015,'Reentered','1978-08-08 00:00:00','1978-08-03 00:00:00',88.99,212,238,65,'Intermediate' from dual union all
select 10055,'010055','1976-063L','deb Kosmos-838','',39015,'Reentered','1977-12-02 00:00:00','1977-11-11 00:00:00',91.69,351,363,64.99,'Intermediate' from dual union all
select 10056,'010056','1976-063M','deb Kosmos-838','',39015,'Reentered','1977-11-03 00:00:00','1977-10-26 00:00:00',91.42,343,346,64.69,'Intermediate' from dual union all
select 10057,'010057','1976-063N','deb Kosmos-838','',39015,'Reentered','1978-04-15 00:00:00','1978-04-14 00:00:00',88.87,213,225,65.03,'Intermediate' from dual union all
select 10058,'010058','1976-063P','deb Kosmos-838','',39015,'Reentered','1977-09-20 00:00:00','1977-09-08 00:00:00',90.97,315,330,65.03,'Intermediate' from dual union all
select 10059,'010059','1977-047A','Kosmos-917','Oko',42175,'In Earth orbit','','1977-07-25 00:00:00',717.6,621,39725,62.9,'Molniya' from dual union all
select 1006,'001006','1964-070U','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',89.66,191,325,51.2,'Intermediate' from dual union all
select 10060,'010060','1977-047B','11S510','11S510',42175,'Reentered','1977-08-17 00:00:00','1977-07-16 00:00:00',91.36,210,473,62.78,'Intermediate' from dual union all
select 10061,'010061','1977-048A','GOES 2','GOES B',42176,'In Earth orbit','','1977-07-17 00:00:00',1436.16,35258,36318,.82,'Inclined GEO' from dual union all
select 10062,'010062','1977-048B','Delta 131','Delta SSPU DSV-3P-4',42176,'In Earth orbit','','1991-03-12 00:00:00',108.33,571,1728,28.41,'Intermediate' from dual union all
select 10063,'010063','1977-048C','Delta 131 clamp band','',42176,'Reentered','1977-07-12 00:00:00','1977-06-29 00:00:00',91.94,179,561,28.41,'Intermediate' from dual union all
select 10064,'010064','1977-049A','Signe 3','Signe 3',42177,'Reentered','1979-06-20 00:00:00','1977-07-18 00:00:00',94.36,458,518,50.68,'Intermediate' from dual union all
select 10065,'010065','1977-050A','Kosmos-918','IS',42178,'Deorbited','1977-06-17 00:00:00','1977-06-17 00:00:00',105.15,243,1760,65.9,'Intermediate' from dual union all
select 10066,'010066','1977-047C','BOZ','BOZ',42175,'Reentered','1977-07-14 00:00:00','1977-06-23 00:00:00',91.72,181,537,62.86,'Intermediate' from dual union all
select 10067,'010067','1977-048D','Delta 131 clamp band','',42176,'Reentered','1977-06-29 00:00:00','1977-06-18 00:00:00',93.19,180,682,28.4,'Intermediate' from dual union all
select 10068,'010068','1977-050B','11S692','11S692',42178,'Reentered','1977-06-18 00:00:00','1977-06-17 00:00:00',87.67,122,197,65.07,'Intermediate' from dual union all
select 10069,'010069','1977-049B','S3M','S3M',42177,'Reentered','1980-02-05 00:00:00','1977-07-22 00:00:00',94.28,451,517,50.67,'Intermediate' from dual union all
select 1007,'001007','1964-070V','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.74,160,265,51.4,'Intermediate' from dual union all
select 10070,'010070','1977-051A','Kosmos-919','DS-P1-I No. 19',42179,'Reentered','1978-08-28 00:00:00','1977-07-21 00:00:00',95.36,266,806,71.02,'Intermediate' from dual union all
select 10071,'010071','1977-051B','S1M','S1M',42179,'Reentered','1978-04-21 00:00:00','1977-07-18 00:00:00',95.22,266,793,71.03,'Intermediate' from dual union all
select 10072,'010072','1977-051C','deb Kosmos-919','',42179,'Reentered','1977-07-15 00:00:00','1977-06-25 00:00:00',94.52,273,718,71.02,'Intermediate' from dual union all
select 10073,'010073','1977-045C','KDU','KDU',42173,'Reentered','1977-06-25 00:00:00','1977-06-20 00:00:00',89.13,171,292,62.78,'Intermediate' from dual union all
select 10074,'010074','1977-045D','Blenda','Sunshade',42173,'Reentered','1977-06-23 00:00:00','1977-06-20 00:00:00',89.26,172,304,62.79,'Intermediate' from dual union all
select 10075,'010075','1977-048E','deb Delta 131','',42176,'Reentered','1977-07-11 00:00:00','1977-06-29 00:00:00',91.83,180,549,28.4,'Intermediate' from dual union all
select 10076,'010076','1976-063Q','deb Kosmos-838','',39015,'Reentered','1978-02-13 00:00:00','1978-01-04 00:00:00',91.64,345,365,64.85,'Intermediate' from dual union all
select 10077,'010077','1976-063R','deb Kosmos-838','',39015,'Reentered','1978-03-12 00:00:00','1978-01-02 00:00:00',92.04,356,393,64.71,'Intermediate' from dual union all
select 10078,'010078','1976-063S','deb Kosmos-838','',39015,'Reentered','1977-11-16 00:00:00','1977-11-05 00:00:00',90.84,316,316,64.67,'Intermediate' from dual union all
select 10079,'010079','1976-063T','deb Kosmos-838','',39015,'Reentered','1977-10-03 00:00:00','1977-09-20 00:00:00',91.66,345,367,64.69,'Intermediate' from dual union all
select 1008,'001008','1964-070W','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',89.55,192,313,51.2,'Intermediate' from dual union all
select 10080,'010080','1976-063U','deb Kosmos-838','',39015,'Reentered','1978-09-19 00:00:00','1978-09-12 00:00:00',89.32,234,247,64.97,'Intermediate' from dual union all
select 10081,'010081','1976-063V','deb Kosmos-838','',39015,'Reentered','1978-04-18 00:00:00','1977-06-20 00:00:00',93.5,417,475,64.98,'Intermediate' from dual union all
select 10082,'010082','1976-063W','deb Kosmos-838','',39015,'Reentered','1977-09-23 00:00:00','1977-09-12 00:00:00',92.11,328,428,64.49,'Intermediate' from dual union all
select 10083,'010083','1976-063X','deb Kosmos-838','',39015,'Reentered','1977-07-27 00:00:00','1977-06-24 00:00:00',93.72,405,508,64.56,'Intermediate' from dual union all
select 10084,'010084','1977-045E','PTDU cover','KDU part',42173,'Reentered','1977-06-23 00:00:00','1977-06-21 00:00:00',88.94,167,278,62.81,'Intermediate' from dual union all
select 10085,'010085','1977-044D','DMSP F-2 OLS Cover','',42172,'Reentered','1995-01-01 00:00:00','1977-06-30 00:00:00',101.4,790,860,99.1,'Sun-Synch' from dual union all
select 10086,'010086','1977-052A','Kosmos-920','Zenit-4MK',42180,'Landed','1977-07-05 00:00:00','1977-06-24 00:00:00',89.37,168,319,64.99,'Intermediate' from dual union all
select 10087,'010087','1977-052B','11S510','11S510',42180,'Reentered','1977-06-27 00:00:00','1977-06-24 00:00:00',89.06,163,293,65.01,'Intermediate' from dual;
insert into SATELLITE
select 10088,'010088','1977-046F','Blenda','KBE part',42174,'Reentered','1977-06-27 00:00:00','1977-06-22 00:00:00',89.76,244,281,62.8,'Intermediate' from dual union all
select 10089,'010089','1977-047D','Blok-L','Blok-L',42175,'In Earth orbit','','1977-07-31 00:00:00',722.39,630,39952,62.88,'Molniya' from dual union all
select 1009,'001009','1964-070X','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',89.47,190,307,51.4,'Intermediate' from dual union all
select 10090,'010090','1977-046G','KB electropitaniya','KBE',42174,'Reentered','1977-07-01 00:00:00','1977-06-22 00:00:00',89.82,247,285,62.81,'Intermediate' from dual union all
select 10091,'010091','1977-053A','NTS 2','NTS 2',42181,'In Earth orbit','','1993-04-25 00:00:00',718.08,20074,20296,63.92,'Medium' from dual union all
select 10092,'010092','1977-054A','Molniya-1','Molniya-1 11F658 No. 45',42182,'Reentered','1993-05-20 00:00:00','1977-08-02 00:00:00',717.78,458,39897,63.02,'Molniya' from dual union all
select 10093,'010093','1977-054B','11S510','11S510',42182,'Reentered','1977-08-24 00:00:00','1977-07-27 00:00:00',90.68,222,393,62.94,'Intermediate' from dual union all
select 10094,'010094','1977-054C','BOZ','BOZ',42182,'Reentered','1977-08-04 00:00:00','1977-07-06 00:00:00',90.96,208,436,62.95,'Intermediate' from dual union all
select 10095,'010095','1977-055A','Kosmos-921','EPN No. 03.0380',42183,'In Earth orbit','','1990-06-28 00:00:00',97.35,598,666,75.84,'Intermediate' from dual union all
select 10096,'010096','1977-055B','S5M','S5M',42183,'In Earth orbit','','1987-10-10 00:00:00',97.54,603,679,75.84,'Intermediate' from dual union all
select 10097,'010097','1976-063Y','deb Kosmos-838','',39015,'Reentered','1978-03-03 00:00:00','1977-08-11 00:00:00',92.94,402,435,65.05,'Intermediate' from dual union all
select 10098,'010098','1976-063Z','deb Kosmos-838','',39015,'Reentered','1977-07-23 00:00:00','1977-07-02 00:00:00',92.61,385,420,65.06,'Intermediate' from dual union all
select 10099,'010099','1976-063AA','deb Kosmos-838','',39015,'Reentered','1977-09-05 00:00:00','1977-08-24 00:00:00',91.96,361,381,65.04,'Intermediate' from dual union all
select 101,'000101','60 Beta 3','Tiros 1 despin weight','Despin weight',6318,'Reentered','1983-04-17 00:00:00','1961-03-27 00:00:00',97.92,602,717,48.46,'Intermediate' from dual union all
select 1010,'001010','1964-070Y','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.69,195,225,51.3,'Intermediate' from dual union all
select 10100,'010100','1976-063AB','deb Kosmos-838','',39015,'Reentered','1977-10-29 00:00:00','1977-10-11 00:00:00',91.25,329,343,65.05,'Intermediate' from dual union all
select 10101,'010101','1976-063AC','deb Kosmos-838','',39015,'Reentered','1977-09-20 00:00:00','1977-07-05 00:00:00',90.5,281,318,64.97,'Intermediate' from dual union all
select 10102,'010102','1976-063AD','deb Kosmos-838','',39015,'Reentered','1977-10-25 00:00:00','1977-10-18 00:00:00',90.57,297,308,64.98,'Intermediate' from dual union all
select 10103,'010103','1976-063AE','deb Kosmos-838','',39015,'Reentered','1978-07-03 00:00:00','1978-05-19 00:00:00',91.42,338,350,65.04,'Intermediate' from dual union all
select 10104,'010104','1976-063AF','deb Kosmos-838','',39015,'Reentered','1978-06-19 00:00:00','1978-05-30 00:00:00',90.36,280,305,65.05,'Intermediate' from dual union all
select 10105,'010105','1976-063AG','deb Kosmos-838','',39015,'Reentered','1977-08-17 00:00:00','1977-07-26 00:00:00',92.49,389,405,65.22,'Intermediate' from dual union all
select 10106,'010106','1976-063AH','deb Kosmos-838','',39015,'Reentered','1978-04-18 00:00:00','1978-01-08 00:00:00',92.67,390,421,65.06,'Intermediate' from dual union all
select 10107,'010107','1976-063AJ','deb Kosmos-838','',39015,'Reentered','1977-09-03 00:00:00','1977-08-11 00:00:00',92.03,368,381,65.22,'Intermediate' from dual union all
select 10108,'010108','1976-063AK','deb Kosmos-838','',39015,'Reentered','1977-08-27 00:00:00','1977-08-19 00:00:00',90.94,311,330,65.03,'Intermediate' from dual union all
select 10109,'010109','1976-063AL','deb Kosmos-838','',39015,'Reentered','1977-09-22 00:00:00','1977-09-18 00:00:00',89.27,226,251,64.97,'Intermediate' from dual union all
select 1011,'001011','1964-070Z','deb Kosmos-50','',10356,'Reentered','1964-11-09 00:00:00','1964-11-05 00:00:00',88.95,188,258,51.3,'Intermediate' from dual union all
select 10110,'010110','1976-063AM','deb Kosmos-838','',39015,'Reentered','1978-09-03 00:00:00','1978-06-17 00:00:00',91.66,348,363,65.16,'Intermediate' from dual union all
select 10111,'010111','1977-056A','OPS 4800','HEXAGON SV-13',42184,'Deorbited','1977-12-23 00:00:00','1977-07-27 00:00:00',88.48,157,242,97,'Sun-Synch' from dual union all
select 10112,'010112','1977-056B','Titan 23D-17 Stage 2','LR91AJ11 23D-17',42184,'Reentered','1977-06-29 00:00:00','1977-06-28 00:00:00',88.19,151,219,97.02,'Sun-Synch' from dual union all
select 10113,'010113','1977-057A','Meteor-Priroda','Meteor-Priroda No. 2-2',42185,'Reentered','1993-08-28 00:00:00','1990-02-26 00:00:00',95.24,508,553,97.36,'Sun-Synch' from dual union all
select 10114,'010114','1977-057B','8A92ME','8A92ME',42185,'In Earth orbit','','1991-03-26 00:00:00',96.59,584,607,97.81,'Sun-Synch' from dual union all
select 10115,'010115','1977-058A','Kosmos-922','Zenit-2M',42186,'Landed','1977-07-13 00:00:00','1977-07-02 00:00:00',89.51,204,297,62.79,'Intermediate' from dual union all
select 10116,'010116','1977-057C','deb Meteor','',42185,'Reentered','1977-08-02 00:00:00','1977-06-30 00:00:00',97.56,633,651,97.9,'Sun-Synch' from dual union all
select 10117,'010117','1977-058B','11S510','11S510',42186,'Reentered','1977-07-08 00:00:00','1977-07-02 00:00:00',89.27,196,280,62.8,'Intermediate' from dual union all
select 10118,'010118','1977-058C','Sensor cover','Zenit-2M sensor cover',42186,'Reentered','1977-07-02 00:00:00','1977-07-01 00:00:00',89.45,205,290,62.8,'Intermediate' from dual union all
select 10119,'010119','1977-058D','Sensor cover','Zenit-2M sensor cover',42186,'Reentered','1977-07-02 00:00:00','1977-07-01 00:00:00',88.99,196,254,62.58,'Intermediate' from dual union all
select 1012,'001012','1964-070AA','deb Kosmos-50','',10356,'Reentered','1964-11-10 00:00:00','1964-11-05 00:00:00',88.29,176,204,51.2,'Intermediate' from dual union all
select 10120,'010120','1977-059A','Kosmos-923','Strela-2M',42187,'In Earth orbit','','1992-06-01 00:00:00',100.72,784,802,74.05,'Intermediate' from dual union all
select 10121,'010121','1977-059B','S3M','S3M',42187,'In Earth orbit','','1991-03-27 00:00:00',100.58,771,801,74.03,'Intermediate' from dual union all
select 10122,'010122','1976-063AN','deb Kosmos-838','',39015,'Reentered','1977-11-09 00:00:00','1977-10-22 00:00:00',91.17,324,339,65.02,'Intermediate' from dual union all
select 10123,'010123','1976-063AP','deb Kosmos-838','',39015,'Reentered','1977-08-18 00:00:00','1977-07-09 00:00:00',93,401,442,65.02,'Intermediate' from dual union all
select 10124,'010124','1976-063AQ','deb Kosmos-838','',39015,'Reentered','1977-08-17 00:00:00','1977-07-29 00:00:00',91.82,360,368,64.95,'Intermediate' from dual union all
select 10125,'010125','1976-063AR','deb Kosmos-838','',39015,'Reentered','1977-07-28 00:00:00','1977-07-20 00:00:00',92.42,360,427,64.89,'Intermediate' from dual union all
select 10126,'010126','1976-063AS','deb Kosmos-838','',39015,'Reentered','1978-01-12 00:00:00','1977-07-01 00:00:00',94.3,430,540,64.62,'Intermediate' from dual union all
select 10127,'010127','1977-052C','KDU','KDU',42180,'Reentered','1977-07-10 00:00:00','1977-07-04 00:00:00',89.06,167,288,65,'Intermediate' from dual union all
select 10128,'010128','1977-052D','Blenda','Sunshade',42180,'Reentered','1977-07-08 00:00:00','1977-07-05 00:00:00',88.5,178,223,64.99,'Intermediate' from dual union all
select 10129,'010129','1977-060A','Kosmos-924','Tselina-OK',42188,'Reentered','1981-02-10 00:00:00','1977-08-04 00:00:00',95.22,512,547,74.03,'Intermediate' from dual union all
select 1013,'001013','1964-070AB','deb Kosmos-50','',10356,'Reentered','1964-11-09 00:00:00','1964-11-05 00:00:00',89.1,186,274,51.3,'Intermediate' from dual union all
select 10130,'010130','1977-060B','S3M','S3M',42188,'Reentered','1981-01-20 00:00:00','1977-08-04 00:00:00',95.1,500,547,74.03,'Intermediate' from dual union all
select 10131,'010131','1977-060C','deb Kosmos-924','Tselina-O part',42188,'Reentered','1978-01-12 00:00:00','1977-11-16 00:00:00',94.59,472,526,74.04,'Intermediate' from dual union all
select 10132,'010132','1977-060D','deb Kosmos-924','Tselina-O part',42188,'Reentered','1977-12-03 00:00:00','1977-08-08 00:00:00',94.02,436,507,74,'Intermediate' from dual union all
select 10133,'010133','1977-052E','PTDU cover','KDU part',42180,'Reentered','1977-07-10 00:00:00','1977-07-05 00:00:00',88.86,159,278,65.03,'Intermediate' from dual union all
select 10134,'010134','1977-061A','Kosmos-925','Tselina-D',42189,'Reentered','1993-04-29 00:00:00','1991-03-28 00:00:00',94.42,483,498,81.21,'Intermediate' from dual union all
select 10135,'010135','1977-061B','8A92ME','8A92ME',42189,'Reentered','2000-05-17 00:00:00','1991-03-28 00:00:00',95.17,502,552,81.24,'Intermediate' from dual union all
select 10136,'010136','1970-025NQ','deb Agena D','',21502,'In Earth orbit','','1993-08-05 00:00:00',106.6,1068,1071,100,'Sun-Synch' from dual union all
select 10137,'010137','1977-062A','Kosmos-926','Tsikada',42190,'In Earth orbit','','1977-08-07 00:00:00',105.05,974,1020,82.93,'Intermediate' from dual union all
select 10138,'010138','1977-062B','S3M','S3M',42190,'In Earth orbit','','1990-03-28 00:00:00',104.89,974,1005,82.94,'Intermediate' from dual union all
select 10139,'010139','1977-063A','Kosmos-927','Zenit-4MKM',42191,'Landed','1977-07-25 00:00:00','1977-07-14 00:00:00',89.57,148,359,72.88,'Intermediate' from dual union all
select 1014,'001014','1964-070AC','deb Kosmos-50','',10356,'Reentered','1964-11-17 00:00:00','1964-11-05 00:00:00',89.65,192,323,51.3,'Intermediate' from dual union all
select 10140,'010140','1977-063B','11S510','11S510',42191,'Reentered','1977-07-18 00:00:00','1977-07-14 00:00:00',89.47,165,332,72.88,'Intermediate' from dual union all
select 10141,'010141','1977-064A','Kosmos-928','Parus',42192,'In Earth orbit','','1992-05-24 00:00:00',104.64,951,1004,82.97,'Intermediate' from dual union all
select 10142,'010142','1977-064B','S3M','S3M',42192,'In Earth orbit','','1991-03-24 00:00:00',104.53,941,1004,82.97,'Intermediate' from dual union all
select 10143,'010143','1977-065A','Himawari','GMS',42193,'In Earth orbit','','1977-08-28 00:00:00',1436.09,35770,35803,1.11,'Stationary' from dual union all
select 10144,'010144','1977-065B','Delta 132','Delta SSPU DSV-3P-4',42193,'Exploded','','1993-08-05 00:00:00',108.62,516,1810,29.09,'Intermediate' from dual union all
select 10145,'010145','1977-065C','Delta 132 clamp band','',42193,'Reentered','1978-03-31 00:00:00','1978-02-22 00:00:00',93.2,263,600,28.74,'Intermediate' from dual union all
select 10146,'010146','1977-066A','Kosmos-929','TKS No. 16101',42194,'Deorbited','1978-02-02 00:00:00','1977-08-16 00:00:00',88.7,194,227,51.58,'Intermediate' from dual union all
select 10147,'010147','1977-066B','8S812','8S812',42194,'Reentered','1977-07-29 00:00:00','1977-07-20 00:00:00',89.02,209,244,51.59,'Intermediate' from dual union all
select 10148,'010148','1977-066C','TKS Adapter','Adapter',42194,'Reentered','1977-07-24 00:00:00','1977-07-18 00:00:00',89.41,213,278,51.58,'Intermediate' from dual union all
select 10149,'010149','1977-067A','Kosmos-930','Vektor',42195,'Reentered Attached','1980-05-12 00:00:00','1977-08-19 00:00:00',94.53,479,512,74.02,'Intermediate' from dual union all
select 1015,'001015','1964-070AD','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.84,195,240,51.2,'Intermediate' from dual union all
select 10150,'010150','1977-068A','Kosmos-931','Oko',42196,'In Earth orbit','','1977-08-19 00:00:00',717.8,637,39719,62.8,'Molniya' from dual union all
select 10151,'010151','1977-068B','11S510','11S510',42196,'Reentered','1977-09-22 00:00:00','1977-08-19 00:00:00',91.52,207,492,62.8,'Intermediate' from dual union all
select 10152,'010152','1977-068C','BOZ','BOZ',42196,'Reentered','1977-08-17 00:00:00','1977-07-28 00:00:00',91.76,180,541,62.86,'Intermediate' from dual union all
select 10153,'010153','1977-069A','Kosmos-932','Zenit-4MKM',42197,'Landed','1977-08-02 00:00:00','1977-07-22 00:00:00',89.07,147,310,65.02,'Intermediate' from dual union all
select 10154,'010154','1977-069B','11S510','11S510',42197,'Reentered','1977-07-24 00:00:00','1977-07-22 00:00:00',88.91,169,272,65.02,'Intermediate' from dual union all
select 10155,'010155','1977-054D','Blok-L','Blok-L',42182,'In Earth orbit','','1977-08-19 00:00:00',695.48,455,38794,63,'Molniya' from dual union all
select 10156,'010156','1977-065D','Star 37E S/N 40049','Star 37E',42193,'Reentered','1981-11-06 00:00:00','1981-11-05 00:00:00',90.27,128,448,26.77,'Intermediate' from dual union all
select 10157,'010157','1977-070A','Kosmos-933','Vektor',42198,'Reentered','1978-11-01 00:00:00','1977-08-21 00:00:00',92.42,380,406,65.84,'Intermediate' from dual union all
select 10158,'010158','1977-070B','S3M','S3M',42198,'Reentered','1978-06-07 00:00:00','1977-08-22 00:00:00',92.31,369,407,65.85,'Intermediate' from dual union all
select 10159,'010159','1977-071A','Raduga','Gran'' No. 13L',42199,'In Earth orbit','','1977-09-03 00:00:00',1436.04,35770,35801,.35,'Stationary' from dual union all
select 1016,'001016','1964-070AE','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',89.31,190,291,51.2,'Intermediate' from dual union all
select 10160,'010160','1977-071B','8S812','8S812',42199,'Reentered','1977-07-26 00:00:00','1977-07-24 00:00:00',88.19,179,191,51.47,'Intermediate' from dual union all
select 10161,'010161','1977-071C','Perekhodnik','Sredniy perekhodnik',42199,'Reentered','1977-07-26 00:00:00','1977-07-25 00:00:00',88.21,186,186,51.47,'Intermediate' from dual union all
select 10162,'010162','1977-063C','KDU','KDU',42191,'Reentered','1977-07-27 00:00:00','1977-07-24 00:00:00',88.77,114,314,72.89,'Intermediate' from dual union all
select 10163,'010163','1977-063D','Blenda','Sunshade',42191,'Reentered','1977-07-26 00:00:00','1977-07-26 00:00:00',88.05,138,218,72.85,'Intermediate' from dual union all
select 10164,'010164','1977-072A','Kosmos-934','Zenit-6',42200,'Landed','1977-08-09 00:00:00','1977-07-29 00:00:00',89.57,166,340,62.82,'Intermediate' from dual union all
select 10165,'010165','1977-072B','11S510','11S510',42200,'Reentered','1977-08-05 00:00:00','1977-07-30 00:00:00',89.08,219,239,62.8,'Intermediate' from dual union all
select 10166,'010166','1977-072C','Sensor cover','Zenit-6 sensor cover',42200,'Reentered','1977-08-02 00:00:00','1977-07-28 00:00:00',89.21,233,237,62.84,'Intermediate' from dual union all
select 10167,'010167','1977-068D','Blok-L','Blok-L',42196,'In Earth orbit','','1977-08-20 00:00:00',720.99,636,39877,62.88,'Molniya' from dual union all
select 10168,'010168','1977-073A','Kosmos-935','Zenit-2M',42201,'Landed','1977-08-11 00:00:00','1977-07-31 00:00:00',89.11,214,247,81.32,'Intermediate' from dual union all
select 10169,'010169','1977-073B','11S510','11S510',42201,'Reentered','1977-08-03 00:00:00','1977-07-31 00:00:00',88.71,204,218,81.33,'Intermediate' from dual union all
select 1017,'001017','1964-070AF','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',88.15,161,205,51.1,'Intermediate' from dual union all
select 10170,'010170','1977-069C','Blenda','Sunshade',42197,'Reentered','1977-08-03 00:00:00','1977-08-02 00:00:00',87.9,130,212,65.05,'Intermediate' from dual union all
select 10171,'010171','1977-069D','KDU','KDU',42197,'Reentered','1977-08-03 00:00:00','1977-08-01 00:00:00',88.59,150,260,65.05,'Intermediate' from dual union all
select 10172,'010172','1977-074A','Kosmos-936','Bion No. 4',42202,'Landed','1977-08-22 00:00:00','1977-08-10 00:00:00',90.59,215,391,62.81,'Intermediate' from dual union all
select 10173,'010173','1977-074B','11S510','11S510',42202,'Reentered','1977-08-28 00:00:00','1977-08-10 00:00:00',90.22,212,358,62.79,'Intermediate' from dual union all
select 10174,'010174','1977-072D','KDU','KDU',42200,'Reentered','1977-08-17 00:00:00','1977-08-09 00:00:00',89.36,167,319,62.82,'Intermediate' from dual union all
select 10175,'010175','1977-072E','Blenda','Sunshade',42200,'Reentered','1977-08-10 00:00:00','1977-08-09 00:00:00',88.84,155,280,62.81,'Intermediate' from dual union all
select 10176,'010176','1977-065E','deb Delta 132','',42193,'Reentered','1979-05-22 00:00:00','1977-08-14 00:00:00',105.49,334,1701,28.24,'Intermediate' from dual union all
select 10177,'010177','1977-065F','deb Delta 132','',42193,'In Earth orbit','','1993-08-06 00:00:00',106.7,513,1635,28.23,'Intermediate' from dual;
insert into SATELLITE
select 10178,'010178','1977-065G','deb Delta 132','',42193,'In Earth orbit','','1993-08-05 00:00:00',120.52,925,2484,28.73,'Medium' from dual union all
select 10179,'010179','1977-065H','deb Delta 132','',42193,'Reentered','1991-02-25 00:00:00','1991-02-15 00:00:00',94.71,474,536,27.86,'Intermediate' from dual union all
select 1018,'001018','1964-070AG','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.78,186,243,51.3,'Intermediate' from dual union all
select 10180,'010180','1977-065J','deb Delta 132','',42193,'Reentered','1990-10-23 00:00:00','1990-10-22 00:00:00',91.9,332,403,27.77,'Intermediate' from dual union all
select 10181,'010181','1977-065K','deb Delta 132','',42193,'Reentered','2001-07-19 00:00:00','1993-08-03 00:00:00',101.41,488,1163,29.93,'Intermediate' from dual union all
select 10182,'010182','1977-065L','deb Delta 132','',42193,'Reentered','1981-01-01 00:00:00','1977-08-15 00:00:00',112.08,447,2198,29.68,'Intermediate' from dual union all
select 10183,'010183','1977-065M','deb Delta 132','',42193,'In Earth orbit','','1993-08-06 00:00:00',107.73,572,1672,29.21,'Intermediate' from dual union all
select 10184,'010184','1977-065N','deb Delta 132','',42193,'In Earth orbit','','1993-08-08 00:00:00',109.79,531,1903,29.06,'Intermediate' from dual union all
select 10185,'010185','1977-065P','deb Delta 132','',42193,'Reentered','1985-06-23 00:00:00','1977-08-14 00:00:00',110.96,505,2037,29.62,'Intermediate' from dual union all
select 10186,'010186','1977-065Q','deb Delta 132','',42193,'Reentered','1992-01-25 00:00:00','1992-01-23 00:00:00',90.47,286,310,29.11,'Intermediate' from dual union all
select 10187,'010187','1977-065R','deb Delta 132','',42193,'Reentered','1978-10-27 00:00:00','1978-09-30 00:00:00',101.82,525,1164,29.01,'Intermediate' from dual union all
select 10188,'010188','1977-065S','deb Delta 132','',42193,'In Earth orbit','','1993-08-07 00:00:00',108.28,534,1761,29.07,'Intermediate' from dual union all
select 10189,'010189','1977-065T','deb Delta 132','',42193,'Reentered','1979-02-16 00:00:00','1977-08-17 00:00:00',107.66,406,1831,28.99,'Intermediate' from dual union all
select 1019,'001019','1964-070AH','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',89.13,191,272,51.3,'Intermediate' from dual union all
select 10190,'010190','1977-065U','deb Delta 132','',42193,'Reentered','1989-04-24 00:00:00','1989-04-21 00:00:00',91.02,291,358,28.94,'Intermediate' from dual union all
select 10191,'010191','1977-065V','deb Delta 132','',42193,'Reentered','1978-12-04 00:00:00','1977-08-13 00:00:00',107.31,520,1685,29.57,'Intermediate' from dual union all
select 10192,'010192','1977-065W','deb Delta 132','',42193,'Reentered','1983-08-17 00:00:00','1977-08-22 00:00:00',109.05,462,1904,29.1,'Intermediate' from dual union all
select 10193,'010193','1977-065X','deb Delta 132','',42193,'Reentered','1979-04-17 00:00:00','1977-08-14 00:00:00',108.58,439,1884,29.11,'Intermediate' from dual union all
select 10194,'010194','1977-065Y','deb Delta 132','',42193,'Reentered','1985-09-22 00:00:00','1977-09-01 00:00:00',118.99,646,2626,30.9,'Intermediate' from dual union all
select 10195,'010195','1977-072F','KDU part','KDU part',42200,'Reentered','1977-08-12 00:00:00','1977-08-09 00:00:00',89.27,165,312,62.81,'Intermediate' from dual union all
select 10196,'010196','1977-065Z','deb Delta 132','',42193,'Reentered','1982-11-10 00:00:00','1977-09-01 00:00:00',113.74,535,2262,29.8,'Intermediate' from dual union all
select 10197,'010197','1977-065AA','deb Delta 132','',42193,'Reentered','1981-07-18 00:00:00','1981-07-03 00:00:00',98.74,478,919,29.6,'Intermediate' from dual union all
select 10198,'010198','1977-065AB','deb Delta 132','',42193,'In Earth orbit','','1993-08-08 00:00:00',111.1,712,1843,29.56,'Intermediate' from dual union all
select 10199,'010199','1977-065AC','deb Delta 132','',42193,'Reentered','1980-11-30 00:00:00','1980-11-17 00:00:00',93.55,329,568,29.55,'Intermediate' from dual union all
select 102,'000102','61 Lambda 2','SRV 521','SRV',7269,'Reentered','1962-05-23 00:00:00','1961-05-11 00:00:00',101.06,202,1416,81.94,'Intermediate' from dual union all
select 1020,'001020','1964-070AJ','deb Kosmos-50','',10356,'Reentered','1964-11-14 00:00:00','1964-11-05 00:00:00',88.03,145,209,51.1,'Intermediate' from dual union all
select 10200,'010200','1977-065AD','deb Delta 132','',42193,'Reentered','1981-05-15 00:00:00','1981-04-19 00:00:00',100.45,544,1016,29.25,'Intermediate' from dual union all
select 10201,'010201','1977-065AE','deb Delta 132','',42193,'In Earth orbit','','1993-08-06 00:00:00',107.45,501,1717,29.11,'Intermediate' from dual union all
select 10202,'010202','1977-065AF','deb Delta 132','',42193,'Reentered','1998-09-20 00:00:00','1993-08-24 00:00:00',97.02,459,773,29.17,'Intermediate' from dual union all
select 10203,'010203','1977-065AG','deb Delta 132','',42193,'Reentered','1986-01-01 00:00:00','1977-08-16 00:00:00',109.97,495,1956,29.01,'Intermediate' from dual union all
select 10204,'010204','1977-065AH','deb Delta 132','',42193,'In Earth orbit','','1993-08-08 00:00:00',105.92,570,1505,28.86,'Intermediate' from dual union all
select 10205,'010205','1977-065AJ','deb Delta 132','',42193,'Reentered','1981-08-09 00:00:00','1981-07-16 00:00:00',103.28,577,1251,29.38,'Intermediate' from dual union all
select 10206,'010206','1977-065AK','deb Delta 132','',42193,'In Earth orbit','','1993-08-07 00:00:00',111.18,687,1875,29.24,'Intermediate' from dual union all
select 10207,'010207','1977-065AL','deb Delta 132','',42193,'In Earth orbit','','1993-07-29 00:00:00',114.57,868,2003,29.1,'Intermediate' from dual union all
select 10208,'010208','1977-065AM','deb Delta 132','',42193,'In Earth orbit','','1993-08-03 00:00:00',106.72,534,1616,28.93,'Intermediate' from dual union all
select 10209,'010209','1977-065AN','deb Delta 132','',42193,'Reentered','1982-09-26 00:00:00','1977-08-19 00:00:00',111.37,569,2011,29.35,'Intermediate' from dual union all
select 1021,'001021','1964-070AK','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',89.1,185,275,51.1,'Intermediate' from dual union all
select 10210,'010210','1977-065AP','deb Delta 132','',42193,'Reentered','1981-11-17 00:00:00','1981-10-11 00:00:00',98.1,532,804,28.56,'Intermediate' from dual union all
select 10211,'010211','1977-065AQ','deb Delta 132','',42193,'In Earth orbit','','1993-08-07 00:00:00',114.88,865,2035,29.42,'Intermediate' from dual union all
select 10212,'010212','1977-065AR','deb Delta 132','',42193,'Reentered','2000-12-06 00:00:00','1993-08-21 00:00:00',102.56,541,1218,28.94,'Intermediate' from dual union all
select 10213,'010213','1977-065AS','deb Delta 132','',42193,'Reentered','1981-04-13 00:00:00','1977-08-19 00:00:00',110.66,584,1931,29.09,'Intermediate' from dual union all
select 10214,'010214','1977-065AT','deb Delta 132','',42193,'In Earth orbit','','1993-08-08 00:00:00',109.49,606,1801,28.91,'Intermediate' from dual union all
select 10215,'010215','1977-065AU','deb Delta 132','',42193,'Reentered','1981-04-13 00:00:00','1977-08-19 00:00:00',111.22,539,2027,29.09,'Intermediate' from dual union all
select 10216,'010216','1977-065AV','deb Delta 132','',42193,'Reentered','1982-06-05 00:00:00','1977-08-23 00:00:00',110.08,500,1961,29.11,'Intermediate' from dual union all
select 10217,'010217','1977-075A','HEAO 1','HEAO A',42203,'Reentered','1979-03-15 00:00:00','1977-09-15 00:00:00',93.35,428,449,22.76,'Equatorial' from dual union all
select 10218,'010218','1977-075B','Centaur AC-45','Centaur D-1AR AC-45',42203,'Reentered','1977-11-25 00:00:00','1977-09-11 00:00:00',91.48,325,370,22.81,'Equatorial' from dual union all
select 10219,'010219','1977-065AW','deb Delta 132','',42193,'Reentered','1987-02-01 00:00:00','1987-02-01 00:00:00',89.58,252,255,29.01,'Intermediate' from dual union all
select 1022,'001022','1964-070AL','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.86,164,273,51.2,'Intermediate' from dual union all
select 10220,'010220','1977-065AX','deb Delta 132','',42193,'In Earth orbit','','1993-08-08 00:00:00',109.01,673,1689,29.12,'Intermediate' from dual union all
select 10221,'010221','1977-065AY','deb Delta 132','',42193,'In Earth orbit','','1993-08-06 00:00:00',105.55,505,1536,29.08,'Intermediate' from dual union all
select 10222,'010222','1977-065AZ','deb Delta 132','',42193,'Reentered','1978-12-24 00:00:00','1978-10-10 00:00:00',111.07,542,2010,29.06,'Intermediate' from dual union all
select 10223,'010223','1977-065BA','deb Delta 132','',42193,'Reentered','1999-09-11 00:00:00','1993-08-08 00:00:00',97.82,430,879,29.08,'Intermediate' from dual union all
select 10224,'010224','1977-065BB','deb Delta 132','',42193,'Reentered','1981-06-20 00:00:00','1981-07-24 00:00:00',101.67,544,1131,28.75,'Intermediate' from dual union all
select 10225,'010225','1977-065BC','deb Delta 132','',42193,'Reentered','1980-10-09 00:00:00','1980-08-27 00:00:00',96.99,449,781,29.51,'Intermediate' from dual union all
select 10226,'010226','1977-065BD','deb Delta 132','',42193,'Reentered','2002-04-30 00:00:00','1993-08-08 00:00:00',101.12,493,1131,29.14,'Intermediate' from dual union all
select 10227,'010227','1977-065BE','deb Delta 132','',42193,'In Earth orbit','','1993-08-03 00:00:00',108.93,567,1788,28.97,'Intermediate' from dual union all
select 10228,'010228','1977-065BF','deb Delta 132','',42193,'Reentered','1981-03-11 00:00:00','1977-08-14 00:00:00',108.88,458,1892,28.54,'Intermediate' from dual union all
select 10229,'010229','1977-065BG','deb Delta 132','',42193,'In Earth orbit','','1993-08-05 00:00:00',107.25,681,1518,28.81,'Intermediate' from dual union all
select 1023,'001023','1964-070AM','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.49,192,208,51.4,'Intermediate' from dual union all
select 10230,'010230','1977-065BH','deb Delta 132','',42193,'In Earth orbit','','1993-08-09 00:00:00',104.63,552,1403,29.03,'Intermediate' from dual union all
select 10231,'010231','1977-065BJ','deb Delta 132','',42193,'Reentered','1982-07-20 00:00:00','1977-08-14 00:00:00',111.67,591,2016,28.62,'Intermediate' from dual union all
select 10232,'010232','1977-065BK','deb Delta 132','',42193,'Reentered','1991-01-21 00:00:00','1977-08-14 00:00:00',109.69,475,1950,29.07,'Intermediate' from dual union all
select 10233,'010233','1977-065BL','deb Delta 132','',42193,'Reentered','1990-08-28 00:00:00','1990-08-26 00:00:00',89.46,233,263,28.97,'Intermediate' from dual union all
select 10234,'010234','1977-065BM','deb Delta 132','',42193,'In Earth orbit','','1993-08-07 00:00:00',105.29,486,1530,28.95,'Intermediate' from dual union all
select 10235,'010235','1977-065BN','deb Delta 132','',42193,'In Earth orbit','','1993-08-04 00:00:00',103.13,481,1332,28.85,'Intermediate' from dual union all
select 10236,'010236','1977-065BP','deb Delta 132','',42193,'Reentered','1980-08-27 00:00:00','1980-08-23 00:00:00',90.01,258,292,28.8,'Intermediate' from dual union all
select 10237,'010237','1977-042C','Kosmos-913 SS 1','ESO',42170,'Reentered','1978-04-18 00:00:00','1977-08-18 00:00:00',94.42,468,513,74.03,'Intermediate' from dual union all
select 10238,'010238','1977-042D','Kosmos-913 SS 2','ESO',42170,'Reentered','1978-02-18 00:00:00','1977-08-20 00:00:00',94.53,473,519,74.05,'Intermediate' from dual union all
select 10239,'010239','1977-065BQ','deb Delta 132','',42193,'Reentered','1984-04-27 00:00:00','1977-08-19 00:00:00',110.91,646,1892,28.62,'Intermediate' from dual union all
select 1024,'001024','1964-070AN','deb Kosmos-50','',10356,'Reentered','1964-11-14 00:00:00','1964-11-05 00:00:00',88.81,169,263,50.9,'Intermediate' from dual union all
select 10240,'010240','1977-065BR','deb Delta 132','',42193,'In Earth orbit','','1993-08-07 00:00:00',111.64,681,1924,29.41,'Intermediate' from dual union all
select 10241,'010241','1977-065BS','deb Delta 132','',42193,'Reentered','2003-03-20 00:00:00','1993-08-07 00:00:00',102.21,486,1241,29.01,'Intermediate' from dual union all
select 10242,'010242','1977-065BT','deb Delta 132','',42193,'Reentered','1981-04-29 00:00:00','1981-04-22 00:00:00',93.17,366,494,29.09,'Intermediate' from dual union all
select 10243,'010243','1977-065BU','deb Delta 132','',42193,'Reentered','1994-09-12 00:00:00','1993-09-15 00:00:00',96.68,493,707,28.18,'Intermediate' from dual union all
select 10244,'010244','1977-065BV','deb Delta 132','',42193,'Reentered','1991-09-22 00:00:00','1991-09-17 00:00:00',91.5,321,376,29.49,'Intermediate' from dual union all
select 10245,'010245','1977-065BW','deb Delta 132','',42193,'Reentered','1977-11-04 00:00:00','1977-09-23 00:00:00',110.72,569,1951,29,'Intermediate' from dual union all
select 10246,'010246','1977-065BX','deb Delta 132','',42193,'Reentered','2015-12-27 00:00:00','1993-08-03 00:00:00',104.1,512,1393,29.04,'Intermediate' from dual union all
select 10247,'010247','1977-065BY','deb Delta 132','',42193,'Reentered','1979-08-09 00:00:00','1977-08-18 00:00:00',109.12,372,2001,29.24,'Intermediate' from dual union all
select 10248,'010248','1977-065BZ','deb Delta 132','',42193,'Reentered','1979-02-14 00:00:00','1977-08-18 00:00:00',108.5,436,1879,28.94,'Intermediate' from dual union all
select 10249,'010249','1977-065CA','deb Delta 132','',42193,'Reentered','1989-10-24 00:00:00','1989-10-23 00:00:00',89.73,244,279,28.98,'Intermediate' from dual union all
select 1025,'001025','1964-070AP','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.14,169,196,51.2,'Intermediate' from dual union all
select 10250,'010250','1977-065CB','deb Delta 132','',42193,'In Earth orbit','','1993-08-02 00:00:00',106.52,529,1602,29.08,'Intermediate' from dual union all
select 10251,'010251','1977-065CC','deb Delta 132','',42193,'In Earth orbit','','1993-08-03 00:00:00',107.22,466,1730,28.98,'Intermediate' from dual union all
select 10252,'010252','1977-065CD','deb Delta 132','',42193,'In Earth orbit','','1993-08-01 00:00:00',110.39,593,1897,28.85,'Intermediate' from dual union all
select 10253,'010253','1977-065CE','deb Delta 132','',42193,'Reentered','1978-02-20 00:00:00','1977-08-19 00:00:00',107.68,255,1984,29.62,'Intermediate' from dual union all
select 10254,'010254','1977-065CF','deb Delta 132','',42193,'Reentered','1981-06-04 00:00:00','1981-05-26 00:00:00',96.11,437,708,29.28,'Intermediate' from dual union all
select 10255,'010255','1977-065CG','deb Delta 132','',42193,'Reentered','1990-01-04 00:00:00','1990-01-03 00:00:00',89.16,218,248,29.06,'Intermediate' from dual union all
select 10256,'010256','1977-065CH','deb Delta 132','',42193,'Reentered','1982-02-05 00:00:00','1977-08-18 00:00:00',110.03,466,1991,29.18,'Intermediate' from dual union all
select 10257,'010257','1977-065CJ','deb Delta 132','',42193,'Reentered','2014-03-01 00:00:00','1993-08-05 00:00:00',104.58,529,1421,29.06,'Intermediate' from dual union all
select 10258,'010258','1977-065CK','deb Delta 132','',42193,'Reentered','1992-09-04 00:00:00','1992-09-02 00:00:00',89.89,264,274,28.89,'Intermediate' from dual union all
select 10259,'010259','1977-065CL','deb Delta 132','',42193,'Reentered','1982-12-07 00:00:00','1982-05-07 00:00:00',99.59,458,1020,28.97,'Intermediate' from dual union all
select 1026,'001026','1964-070AQ','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',89.07,190,267,51.4,'Intermediate' from dual union all
select 10260,'010260','1977-065CM','deb Delta 132','',42193,'Reentered','1983-04-11 00:00:00','1977-08-18 00:00:00',111.64,639,1966,28.66,'Intermediate' from dual union all
select 10261,'010261','1977-065CN','deb Delta 132','',42193,'In Earth orbit','','1993-08-04 00:00:00',105.24,512,1500,29.05,'Intermediate' from dual union all
select 10262,'010262','1977-065CP','deb Delta 132','',42193,'In Earth orbit','','1993-08-02 00:00:00',107.59,521,1710,29.08,'Intermediate' from dual union all
select 10263,'010263','1977-065CQ','deb Delta 132','',42193,'Reentered','1981-05-01 00:00:00','1977-08-18 00:00:00',109.5,408,2000,29.56,'Intermediate' from dual union all
select 10264,'010264','1977-065CR','deb Delta 132','',42193,'Reentered','1989-03-14 00:00:00','1989-03-16 00:00:00',94.11,410,541,28.9,'Intermediate' from dual union all
select 10265,'010265','1977-065CS','deb Delta 132','',42193,'Reentered','1980-06-22 00:00:00','1977-09-01 00:00:00',116.62,855,2203,29.44,'Intermediate' from dual union all
select 10266,'010266','1977-065CT','deb Delta 132','',42193,'Reentered','1985-08-06 00:00:00','1977-09-01 00:00:00',114.48,723,2141,29.4,'Intermediate' from dual union all
select 10267,'010267','1977-065CU','deb Delta 132','',42193,'Reentered','1982-01-21 00:00:00','1982-01-20 00:00:00',88.99,208,242,29.28,'Intermediate' from dual;
insert into SATELLITE
select 10268,'010268','1977-065CV','deb Delta 132','',42193,'Reentered','1979-08-13 00:00:00','1979-02-25 00:00:00',104.72,527,1436,29.21,'Intermediate' from dual union all
select 10269,'010269','1977-065CW','deb Delta 132','',42193,'Reentered','2002-10-19 00:00:00','1993-08-09 00:00:00',100.16,383,1149,29.09,'Intermediate' from dual union all
select 1027,'001027','1964-070AR','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',89.13,183,280,51.4,'Intermediate' from dual union all
select 10270,'010270','1977-065CX','deb Delta 132','',42193,'Reentered','1989-09-11 00:00:00','1989-09-11 00:00:00',90.58,300,306,29,'Intermediate' from dual union all
select 10271,'010271','1977-076A','Voyager 2','Voyager 2',42204,'Deep Space','1977-08-20 00:00:00','1977-08-20 00:00:00',40.03,325,20834,41.86,'Earth Escape' from dual union all
select 10272,'010272','1977-076B','Centaur TC-7','Centaur D-1T TC-7',42204,'Deep Space','1977-08-20 00:00:00','1977-08-20 00:00:00',125.23,195,29535,41.87,'Earth Escape' from dual union all
select 10273,'010273','1977-076C','Star 37E S/N 40065','Star 37E',42204,'Deep Space','1977-08-20 00:00:00','1977-08-20 00:00:00',40.03,325,20834,41.86,'Earth Escape' from dual union all
select 10274,'010274','1977-074C','KBE part','KBE part',42202,'Reentered','1977-09-05 00:00:00','1977-08-22 00:00:00',90.26,215,359,62.81,'Intermediate' from dual union all
select 10275,'010275','1977-074D','KB electropitaniya','KBE',42202,'Reentered','1977-10-08 00:00:00','1977-09-02 00:00:00',90.14,212,351,62.8,'Intermediate' from dual union all
select 10276,'010276','1977-071D','SOZ','SOZ',42199,'Reentered','1978-06-05 00:00:00','1977-09-01 00:00:00',647.15,339,36472,47.09,'GEO Transfer' from dual union all
select 10277,'010277','1977-071E','SOZ','SOZ',42199,'Reentered','1978-03-25 00:00:00','1977-08-22 00:00:00',645.9,273,36474,47.06,'GEO Transfer' from dual union all
select 10278,'010278','1977-077A','Kosmos-937','US-P',42205,'Reentered','1978-10-19 00:00:00','1977-09-24 00:00:00',93.28,427,443,65.05,'Intermediate' from dual union all
select 10279,'010279','1977-077B','11S692','11S692',42205,'Reentered','1977-08-25 00:00:00','1977-08-24 00:00:00',89.26,99,376,65.03,'Intermediate' from dual union all
select 1028,'001028','1964-070AS','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.03,142,212,51.1,'Intermediate' from dual union all
select 10280,'010280','1977-077C','deb Kosmos-937','',42205,'Reentered','1977-08-25 00:00:00','1977-08-24 00:00:00',93.27,424,446,65.05,'Intermediate' from dual union all
select 10281,'010281','1977-078A','Kosmos-938','Zenit-4MKM',42206,'Landed','1977-09-06 00:00:00','1977-08-26 00:00:00',89.33,156,327,62.82,'Intermediate' from dual union all
select 10282,'010282','1977-079A','Kosmos-939','Strela-1M',42207,'In Earth orbit','','1992-05-05 00:00:00',114.8,1431,1462,74.02,'Intermediate' from dual union all
select 10283,'010283','1977-078B','11S510','11S510',42206,'Reentered','1977-08-30 00:00:00','1977-08-26 00:00:00',89.25,172,303,62.8,'Intermediate' from dual union all
select 10284,'010284','1977-078C','Sensor cover','Zenit-4MK sensor cover',42206,'Reentered','1977-09-11 00:00:00','1977-08-24 00:00:00',89.53,199,303,62.8,'Intermediate' from dual union all
select 10285,'010285','1977-078D','Sensor cover','Zenit-4MK sensor cover',42206,'Reentered','1977-09-21 00:00:00','1977-08-25 00:00:00',90.05,178,376,62.83,'Intermediate' from dual union all
select 10286,'010286','1977-079B','Kosmos-940','Strela-1M',42207,'In Earth orbit','','1992-07-19 00:00:00',114.39,1394,1461,74.02,'Intermediate' from dual union all
select 10287,'010287','1977-079C','Kosmos-941','Strela-1M',42207,'In Earth orbit','','1992-07-18 00:00:00',114.6,1413,1461,74.02,'Intermediate' from dual union all
select 10288,'010288','1977-079D','Kosmos-942','Strela-1M',42207,'In Earth orbit','','1992-07-17 00:00:00',115.92,1462,1533,74.02,'Intermediate' from dual union all
select 10289,'010289','1977-079E','Kosmos-943','Strela-1M',42207,'In Earth orbit','','1992-07-17 00:00:00',115.01,1450,1462,74.02,'Intermediate' from dual union all
select 1029,'001029','1964-070AT','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.3,181,200,51.3,'Intermediate' from dual union all
select 10290,'010290','1977-079F','Kosmos-944','Strela-1M',42207,'In Earth orbit','','1992-05-12 00:00:00',115.23,1461,1471,74.02,'Intermediate' from dual union all
select 10291,'010291','1977-079G','Kosmos-945','Strela-1M',42207,'In Earth orbit','','1992-07-17 00:00:00',115.45,1462,1490,74.02,'Intermediate' from dual union all
select 10292,'010292','1977-079H','Kosmos-946','Strela-1M',42207,'In Earth orbit','','1992-05-06 00:00:00',115.68,1462,1511,74.03,'Intermediate' from dual union all
select 10293,'010293','1977-079J','S3M','S3M',42207,'In Earth orbit','','1991-02-20 00:00:00',117.53,1463,1677,74.03,'Intermediate' from dual union all
select 10294,'010294','1977-080A','Sirio 1','Sirio 1',42208,'In Earth orbit','','1979-10-23 00:00:00',1437.67,35612,36023,1.33,'Drift GEO' from dual union all
select 10295,'010295','1977-080B','Delta 133','Delta SSPU DSV-3P-4',42208,'In Earth orbit','','1990-02-16 00:00:00',115.42,871,2078,27.09,'Intermediate' from dual union all
select 10296,'010296','1977-080C','Star 37D S/N 029','Star 37D',42208,'Reentered','1982-11-10 00:00:00','1977-11-01 00:00:00',659.37,258,37175,22.92,'GEO Transfer' from dual union all
select 10297,'010297','1977-080D','Delta 133 clamp band','',42208,'Reentered','1978-01-18 00:00:00','1977-09-01 00:00:00',98.52,201,1175,28.06,'Intermediate' from dual union all
select 10298,'010298','1977-080E','Delta 133 clamp band','',42208,'Reentered','1977-12-07 00:00:00','1977-10-05 00:00:00',96.81,209,1003,28.03,'Intermediate' from dual union all
select 10299,'010299','1977-081A','Kosmos-947','Zenit-2M',42209,'Landed','1977-09-09 00:00:00','1977-08-29 00:00:00',89.7,201,318,72.85,'Intermediate' from dual union all
select 103,'000103','61 Mu 1','Vostok','Vostok-3A No. 3',7270,'Landed','1961-04-12 00:00:00','1961-04-12 00:00:00',89.32,168,314,64.95,'Intermediate' from dual union all
select 1030,'001030','1964-070AU','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.59,191,219,51.4,'Intermediate' from dual union all
select 10300,'010300','1977-081B','11S510','11S510',42209,'Reentered','1977-09-07 00:00:00','1977-08-29 00:00:00',89.47,200,297,72.84,'Intermediate' from dual union all
select 10301,'010301','1977-081C','Sensor cover','Zenit-2M sensor cover',42209,'Reentered','1977-09-07 00:00:00','1977-08-29 00:00:00',89.61,199,312,72.82,'Intermediate' from dual union all
select 10302,'010302','1977-065CY','deb Delta 132','',42193,'Reentered','1989-04-16 00:00:00','1989-04-11 00:00:00',91.12,309,350,29.02,'Intermediate' from dual union all
select 10303,'010303','1977-065CZ','deb Delta 132','',42193,'Reentered','1990-06-15 00:00:00','1990-06-12 00:00:00',89.82,240,292,29.03,'Intermediate' from dual union all
select 10304,'010304','1977-065DA','deb Delta 132','',42193,'Reentered','1982-11-08 00:00:00','1977-08-29 00:00:00',119.22,769,2523,28.68,'Intermediate' from dual union all
select 10305,'010305','1977-065DB','deb Delta 132','',42193,'In Earth orbit','','1993-08-06 00:00:00',107.88,680,1577,29.1,'Intermediate' from dual union all
select 10306,'010306','1977-065DC','deb Delta 132','',42193,'In Earth orbit','','1993-08-07 00:00:00',108.41,536,1770,29.11,'Intermediate' from dual union all
select 10307,'010307','1977-065DD','deb Delta 132','',42193,'Reentered','1981-06-06 00:00:00','1977-08-29 00:00:00',111.08,573,1980,29.41,'Intermediate' from dual union all
select 10308,'010308','1977-065DE','deb Delta 132','',42193,'Reentered','1979-06-16 00:00:00','1979-03-16 00:00:00',101.75,453,1230,28.88,'Intermediate' from dual union all
select 10309,'010309','1977-065DF','deb Delta 132','',42193,'Reentered','2014-03-30 00:00:00','1993-08-09 00:00:00',103.56,515,1339,29.06,'Intermediate' from dual union all
select 1031,'001031','1964-070AV','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',89.13,194,269,51.2,'Intermediate' from dual union all
select 10310,'010310','1977-065DG','deb Delta 132','',42193,'Reentered','1980-07-11 00:00:00','1980-07-08 00:00:00',91.76,339,383,29.02,'Intermediate' from dual union all
select 10311,'010311','1977-065DH','deb Delta 132','',42193,'Reentered','1989-03-26 00:00:00','1989-03-12 00:00:00',93.05,370,478,29.13,'Intermediate' from dual union all
select 10312,'010312','1977-065DJ','deb Delta 132','',42193,'Reentered','2013-07-28 00:00:00','1993-08-07 00:00:00',103.61,497,1362,29.02,'Intermediate' from dual union all
select 10313,'010313','1977-065DK','deb Delta 132','',42193,'Reentered','1993-05-31 00:00:00','1993-05-31 00:00:00',89.09,224,234,28.88,'Intermediate' from dual union all
select 10314,'010314','1977-065DL','deb Delta 132','',42193,'In Earth orbit','','1993-08-07 00:00:00',109.88,587,1856,28.78,'Intermediate' from dual union all
select 10315,'010315','1977-082A','Molniya-1','Molniya-1',42210,'Reentered','1993-09-28 00:00:00','1977-10-07 00:00:00',717.72,498,39854,62.87,'Molniya' from dual union all
select 10316,'010316','1977-082B','11S510','11S510',42210,'Reentered','1977-09-26 00:00:00','1977-09-06 00:00:00',90.67,205,410,62.85,'Intermediate' from dual union all
select 10317,'010317','1977-082C','BOZ','BOZ',42210,'Reentered','1977-09-23 00:00:00','1977-09-07 00:00:00',90.71,194,425,62.84,'Intermediate' from dual union all
select 10318,'010318','1977-082D','deb Blok-I','',42210,'Reentered','1977-09-23 00:00:00','1977-09-07 00:00:00',90.71,195,424,62.84,'Intermediate' from dual union all
select 10319,'010319','1977-083A','Kosmos-948','Fram',42211,'Landed','1977-09-15 00:00:00','1977-09-05 00:00:00',88.86,211,226,81.36,'Intermediate' from dual union all
select 1032,'001032','1964-070AW','deb Kosmos-50','',10356,'Reentered','1964-11-16 00:00:00','1964-11-05 00:00:00',88.72,187,236,51.1,'Intermediate' from dual union all
select 10320,'010320','1977-083B','11S510','11S510',42211,'Reentered','1977-09-07 00:00:00','1977-09-05 00:00:00',88.48,189,210,81.35,'Intermediate' from dual union all
select 10321,'010321','1977-084A','Voyager 1','Voyager 1',42212,'Deep Space','1977-09-05 00:00:00','1977-09-05 00:00:00',38.66,327,20658,28.51,'Earth Escape' from dual union all
select 10322,'010322','1977-084B','Centaur TC-6','Centaur D-1T TC-6',42212,'Deep Space','1977-09-05 00:00:00','1977-09-05 00:00:00',117.41,198,28840,28.51,'Earth Escape' from dual union all
select 10323,'010323','1977-084C','Star 37E S/N 40066','Star 37E',42212,'Deep Space','1977-09-05 00:00:00','1977-09-05 00:00:00',38.66,327,20658,28.51,'Earth Escape' from dual union all
select 10324,'010324','1977-078E','KDU','KDU',42206,'Reentered','1977-09-07 00:00:00','1977-09-05 00:00:00',88.67,152,266,62.85,'Intermediate' from dual union all
select 10325,'010325','1977-078F','Blenda','Sunshade',42206,'Reentered','1977-09-06 00:00:00','1977-09-05 00:00:00',88.29,141,238,62.67,'Intermediate' from dual union all
select 10326,'010326','1977-085A','Kosmos-949','Yantar''-2K No. 7',42213,'Landed','1977-10-06 00:00:00','1977-09-13 00:00:00',89.07,169,287,62.8,'Intermediate' from dual union all
select 10327,'010327','1977-085B','11S510','11S510',42213,'Reentered','1977-09-11 00:00:00','1977-09-08 00:00:00',89.22,172,300,62.79,'Intermediate' from dual union all
select 10328,'010328','1977-085C','Kosmos-949 part','Feniks part',42213,'Reentered','1977-09-10 00:00:00','1977-09-09 00:00:00',89.14,174,290,62.8,'Intermediate' from dual union all
select 10329,'010329','1977-042E','Kosmos-913 SS 3','ESO',42170,'Reentered','1978-02-26 00:00:00','1977-09-09 00:00:00',94.59,471,527,74.05,'Intermediate' from dual union all
select 1033,'001033','1964-070AX','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',89.18,183,284,51.2,'Intermediate' from dual union all
select 10330,'010330','1977-042F','Kosmos-913 SS 4','ESO',42170,'Reentered','1978-02-23 00:00:00','1977-09-09 00:00:00',94.53,471,521,74.04,'Intermediate' from dual union all
select 10331,'010331','1977-042G','Kosmos-913 SS 5','ESO',42170,'Reentered','1978-02-23 00:00:00','1977-09-08 00:00:00',94.54,472,521,74.05,'Intermediate' from dual union all
select 10332,'010332','1977-042H','Kosmos-913 SS 6','ESO',42170,'Reentered','1978-04-18 00:00:00','1977-09-08 00:00:00',94.5,471,518,74.07,'Intermediate' from dual union all
select 10333,'010333','1977-042J','Kosmos-913 SS 7','ESO',42170,'Reentered','1978-03-11 00:00:00','1977-09-09 00:00:00',94.48,471,515,74.03,'Intermediate' from dual union all
select 10334,'010334','1977-042K','Kosmos-913 SS 8','ESO',42170,'Reentered','1978-02-15 00:00:00','1977-09-08 00:00:00',94.47,471,515,74.04,'Intermediate' from dual union all
select 10335,'010335','1977-042L','Kosmos-913 SS 9','ESO',42170,'Reentered','1978-02-15 00:00:00','1977-09-08 00:00:00',94.42,471,510,74.04,'Intermediate' from dual union all
select 10336,'010336','1977-042M','Kosmos-913 SS 10','ESO',42170,'Reentered','1978-02-04 00:00:00','1977-09-09 00:00:00',94.34,470,503,74.03,'Intermediate' from dual union all
select 10337,'010337','1977-042N','Kosmos-913 SS 11','ESO',42170,'Reentered','1978-02-23 00:00:00','1977-09-07 00:00:00',94.45,470,514,74.02,'Intermediate' from dual union all
select 10338,'010338','1977-065DM','deb Delta 132','',42193,'In Earth orbit','','1993-08-07 00:00:00',107.34,510,1698,29.07,'Intermediate' from dual union all
select 10339,'010339','1977-065DN','deb Delta 132','',42193,'Reentered','1995-04-20 00:00:00','1993-08-24 00:00:00',98.45,469,901,29.06,'Intermediate' from dual union all
select 1034,'001034','1964-070AY','deb Kosmos-50','',10356,'Reentered','1964-11-09 00:00:00','1964-11-05 00:00:00',89.68,192,326,51.2,'Intermediate' from dual union all
select 10340,'010340','1977-065DP','deb Delta 132','',42193,'Reentered','1983-11-12 00:00:00','1983-08-21 00:00:00',100.93,491,1115,29.13,'Intermediate' from dual union all
select 10341,'010341','1977-065DQ','deb Delta 132','',42193,'Reentered','1994-04-05 00:00:00','1992-10-01 00:00:00',98.29,506,848,29.14,'Intermediate' from dual union all
select 10342,'010342','1977-065DR','deb Delta 132','',42193,'Reentered','1983-01-26 00:00:00','1977-09-09 00:00:00',109.64,474,1947,29.01,'Intermediate' from dual union all
select 10343,'010343','1977-065DS','deb Delta 132','',42193,'Reentered','1978-10-14 00:00:00','1978-02-09 00:00:00',110.64,558,1955,29.09,'Intermediate' from dual union all
select 10344,'010344','1977-065DT','deb Delta 132','',42193,'Reentered','1979-07-02 00:00:00','1979-05-22 00:00:00',94.95,347,686,28.96,'Intermediate' from dual union all
select 10345,'010345','1977-065DU','deb Delta 132','',42193,'Reentered','2010-12-26 00:00:00','1993-08-08 00:00:00',104.33,518,1409,29.86,'Intermediate' from dual union all
select 10346,'010346','1977-065DV','deb Delta 132','',42193,'In Earth orbit','','1993-08-01 00:00:00',109.72,608,1820,29.23,'Intermediate' from dual union all
select 10347,'010347','1977-065DW','deb Delta 132','',42193,'Reentered','1981-07-06 00:00:00','1985-06-20 00:00:00',89.61,246,265,29.36,'Intermediate' from dual union all
select 10348,'010348','1977-065DX','deb Delta 132','',42193,'Reentered','1992-08-07 00:00:00','1992-08-06 00:00:00',89.45,237,258,29.11,'Intermediate' from dual union all
select 10349,'010349','1977-042P','Kosmos-913 SS 12','ESO',42170,'Reentered','1978-04-18 00:00:00','1977-09-19 00:00:00',94.3,468,502,74.04,'Intermediate' from dual union all
select 1035,'001035','1964-070AZ','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',89.81,181,349,51.3,'Intermediate' from dual union all
select 10350,'010350','1977-042Q','Kosmos-913 SS 13','ESO',42170,'Reentered','1978-03-15 00:00:00','1977-09-28 00:00:00',94.44,467,516,74.03,'Intermediate' from dual union all
select 10351,'010351','1977-086A','Kosmos-950','Zenit-2M',42214,'Landed','1977-09-27 00:00:00','1977-09-15 00:00:00',89.34,204,279,62.81,'Intermediate' from dual union all
select 10352,'010352','1977-087A','Kosmos-951','Parus',42215,'In Earth orbit','','1992-05-27 00:00:00',104.83,960,1013,82.97,'Intermediate' from dual union all
select 10353,'010353','1977-086B','11S510','11S510',42214,'Reentered','1977-09-18 00:00:00','1977-09-15 00:00:00',88.84,190,245,62.8,'Intermediate' from dual union all
select 10354,'010354','1977-086C','Sensor cover','Zenit-2M sensor cover',42214,'Reentered','1977-09-20 00:00:00','1977-09-15 00:00:00',88.84,177,258,62.74,'Intermediate' from dual union all
select 10355,'010355','1977-087B','S3M','S3M',42215,'In Earth orbit','','1990-04-01 00:00:00',104.74,956,1008,82.97,'Intermediate' from dual union all
select 10356,'010356','1977-083C','KDU','KDU',42211,'Reentered','1977-09-20 00:00:00','1977-09-14 00:00:00',88.66,203,214,81.35,'Intermediate' from dual union all
select 10357,'010357','1977-083D','Blenda','Sunshade',42211,'Reentered','1977-09-15 00:00:00','1977-09-14 00:00:00',88.43,193,201,81.34,'Intermediate' from dual;
insert into SATELLITE
select 10358,'010358','1977-088A','Kosmos-952','US-A reactor',42216,'In Earth orbit','','1977-10-18 00:00:00',104.11,904,1002,64.94,'Intermediate' from dual union all
select 10359,'010359','1977-089A','Kosmos-953','Zenit-4MKM',42217,'Landed','1977-09-29 00:00:00','1977-09-18 00:00:00',89.35,173,312,62.8,'Intermediate' from dual union all
select 1036,'001036','1964-070BA','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.78,189,240,51.1,'Intermediate' from dual union all
select 10360,'010360','1977-089B','11S510','11S510',42217,'Reentered','1977-09-21 00:00:00','1977-09-18 00:00:00',89.1,168,292,62.78,'Intermediate' from dual union all
select 10361,'010361','1977-090A','Kosmos-954','US-A',42218,'Reentered Attached','1978-01-24 00:00:00','1977-10-18 00:00:00',89.63,250,262,64.98,'Intermediate' from dual union all
select 10362,'010362','1977-091A','Kosmos-955','Tselina-D',42219,'Reentered','2000-09-07 00:00:00','1990-03-26 00:00:00',95.93,559,568,81.22,'Intermediate' from dual union all
select 10363,'010363','1977-091B','8A92ME','8A92ME',42219,'Reentered','2001-11-14 00:00:00','1990-03-25 00:00:00',96,535,598,81.24,'Intermediate' from dual union all
select 10364,'010364','1977-091C','deb Kosmos-955','',42219,'Reentered','1982-06-03 00:00:00','1977-10-01 00:00:00',97.42,629,642,81.2,'Intermediate' from dual union all
select 10365,'010365','1977-092A','Ekran','Ekran No. 12L',42220,'In Earth orbit','','1977-11-01 00:00:00',1436.09,35770,35803,.33,'Stationary' from dual union all
select 10366,'010366','1977-092B','8S812','8S812',42220,'Reentered','1977-09-21 00:00:00','1977-09-20 00:00:00',88.19,170,200,51.46,'Intermediate' from dual union all
select 10367,'010367','1977-092C','Perekhodnik','Sredniy perekhodnik',42220,'Reentered','1977-09-21 00:00:00','1977-09-20 00:00:00',88.28,186,193,51.44,'Intermediate' from dual union all
select 10368,'010368','1977-092D','deb Blok Proton','',42220,'Reentered','1977-09-25 00:00:00','1977-09-20 00:00:00',88.64,140,275,51.44,'Intermediate' from dual union all
select 10369,'010369','1977-082E','Blok-L','Blok-L',42210,'Reentered','1988-07-04 00:00:00','1977-10-03 00:00:00',732.12,497,40563,62.84,'Molniya' from dual union all
select 1037,'001037','1964-070BB','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',89.48,192,306,51.1,'Intermediate' from dual union all
select 10370,'010370','1977-093A','Prognoz-6','SO-M No. 506',42221,'Deep Space','1977-09-22 00:00:00','1978-04-04 00:00:00',5683.19,1850,196378,74.36,'Deep Space' from dual union all
select 10371,'010371','1977-093B','11S510','11S510',42221,'Reentered','1977-10-29 00:00:00','1977-09-29 00:00:00',91.14,221,440,65.04,'Intermediate' from dual union all
select 10372,'010372','1977-093C','BOZ','BOZ',42221,'Reentered','1977-10-23 00:00:00','1977-09-29 00:00:00',91.08,204,452,65.06,'Intermediate' from dual union all
select 10373,'010373','1977-093D','deb Blok-I','',42221,'Reentered','1977-09-28 00:00:00','1977-09-26 00:00:00',89.02,175,278,65.01,'Intermediate' from dual union all
select 10374,'010374','1977-094A','OPS 7471','GAMBIT-3 Mission 4349',42222,'Reentered Attached','1977-12-08 00:00:00','1977-10-23 00:00:00',89.17,122,345,96.48,'Sun-Synch' from dual union all
select 10375,'010375','1977-095A','Kosmos-956','EPN No. 03.0380',42223,'Reentered','1982-06-27 00:00:00','1977-10-25 00:00:00',96.82,352,861,75.83,'Intermediate' from dual union all
select 10376,'010376','1977-096A','Interkosmos-17','AUOS-Z-R-IK',42224,'Reentered','1979-11-08 00:00:00','1977-10-24 00:00:00',94.34,465,509,82.97,'Intermediate' from dual union all
select 10377,'010377','1977-096B','S3M','S3M',42224,'Reentered','1979-09-11 00:00:00','1977-10-28 00:00:00',94.26,454,512,82.97,'Intermediate' from dual union all
select 10378,'010378','1977-095B','S5M','S5M',42223,'Reentered','1982-11-01 00:00:00','1977-10-25 00:00:00',96.8,351,860,75.84,'Intermediate' from dual union all
select 10379,'010379','1977-096C','IK-17 instrument cover','',42224,'Reentered','1978-04-08 00:00:00','1977-10-24 00:00:00',93.79,428,492,82.91,'Intermediate' from dual union all
select 1038,'001038','1964-070BC','deb Kosmos-50','',10356,'Reentered','1964-11-15 00:00:00','1964-11-05 00:00:00',88.57,190,218,51.1,'Intermediate' from dual union all
select 10380,'010380','1977-089C','KDU','KDU',42217,'Reentered','1977-09-30 00:00:00','1977-09-28 00:00:00',89.07,148,309,62.8,'Intermediate' from dual union all
select 10381,'010381','1977-089D','Blenda','Sunshade',42217,'Reentered','1977-09-29 00:00:00','1977-09-28 00:00:00',88.89,149,291,62.82,'Intermediate' from dual union all
select 10382,'010382','1977-097A','Salyut-6','DOS 5',42225,'Deorbited','1982-07-29 00:00:00','1977-10-29 00:00:00',91.34,336,345,51.59,'Intermediate' from dual union all
select 10383,'010383','1977-097B','8S812','8S812',42225,'Reentered','1977-10-05 00:00:00','1977-10-03 00:00:00',88.37,194,194,51.61,'Intermediate' from dual union all
select 10384,'010384','1977-097C','Salyut-6 instrument cover','DOS 5 instrument cover',42225,'Reentered','1977-10-03 00:00:00','1977-09-29 00:00:00',89.01,200,252,51.57,'Intermediate' from dual union all
select 10385,'010385','1977-098A','Kosmos-957','Zenit-4MKM',42226,'Landed','1977-10-13 00:00:00','1977-10-02 00:00:00',89.77,169,358,64.98,'Intermediate' from dual union all
select 10386,'010386','1977-098B','11S510','11S510',42226,'Reentered','1977-10-05 00:00:00','1977-10-02 00:00:00',89.29,171,308,64.98,'Intermediate' from dual union all
select 10387,'010387','1977-098C','Sensor cover','Zenit-4MK sensor cover',42226,'Reentered','1977-10-13 00:00:00','1977-09-30 00:00:00',89.39,161,328,65.04,'Intermediate' from dual union all
select 10388,'010388','1977-098D','Sensor cover','Zenit-4MK sensor cover',42226,'Reentered','1977-10-13 00:00:00','1977-09-30 00:00:00',89.38,166,322,64.98,'Intermediate' from dual union all
select 10389,'010389','1977-065DY','deb Delta 132','',42193,'Reentered','2004-04-26 00:00:00','1993-08-08 00:00:00',103.29,517,1312,28.98,'Intermediate' from dual union all
select 1039,'001039','1964-070BD','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',89.06,194,262,51.3,'Intermediate' from dual union all
select 10390,'010390','1977-065DZ','deb Delta 132','',42193,'Reentered','1981-12-06 00:00:00','1981-05-24 00:00:00',106.72,556,1594,28.85,'Intermediate' from dual union all
select 10391,'010391','1976-124U','Kosmos-885 SS 17','ESO',39076,'Reentered','1978-01-11 00:00:00','1977-10-03 00:00:00',93.94,450,485,65.87,'Intermediate' from dual union all
select 10392,'010392','1974-089EH','deb Delta 104','',33692,'In Earth orbit','','1993-07-29 00:00:00',114.28,1383,1463,101.2,'Sun-Synch' from dual union all
select 10393,'010393','1974-089EJ','deb Delta 104','',33692,'In Earth orbit','','1993-07-29 00:00:00',117.69,1451,1703,101.52,'Sun-Synch' from dual union all
select 10394,'010394','1968-097DX','deb Kosmos-252','',16824,'Reentered','1991-04-09 00:00:00','1991-04-09 00:00:00',88.03,108,246,62.21,'Intermediate' from dual union all
select 10395,'010395','1973-086GK','Unknown deb                       ','',30688,'In Earth orbit','','1993-08-21 00:00:00',116.35,1320,1714,102.26,'Sun-Synch' from dual union all
select 10396,'010396','1976-126BJ','deb Kosmos-886','',39078,'Reentered','1984-01-08 00:00:00','1977-10-01 00:00:00',106.81,887,1270,65.8,'Intermediate' from dual union all
select 10397,'010397','1975-004HF','deb Delta 107','',36451,'Reentered','1981-01-06 00:00:00','1977-10-02 00:00:00',101.3,747,893,98.84,'Sun-Synch' from dual union all
select 10398,'010398','1971-015CM','deb Kosmos-397','',24833,'Reentered','1991-02-28 00:00:00','1991-02-28 00:00:00',89.81,243,288,66.06,'Intermediate' from dual union all
select 10399,'010399','1977-088B','Kosmos-952 DU','US-A DU',42216,'Reentered','1977-11-07 00:00:00','1977-10-08 00:00:00',89.5,238,261,64.97,'Intermediate' from dual union all
select 104,'000104','61 Mu 2','8K72KE','8K72KE',7270,'Reentered','1961-04-16 00:00:00','1961-04-14 00:00:00',88.7,154,267,65.07,'Intermediate' from dual union all
select 1040,'001040','1964-070BE','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',89.09,191,268,51.2,'Intermediate' from dual union all
select 10400,'010400','1977-088C','Kosmos-952 RLS','US-A RLS',42216,'Reentered','1977-10-11 00:00:00','1977-10-08 00:00:00',89.33,233,250,64.96,'Intermediate' from dual union all
select 10401,'010401','1977-099A','Soyuz-25','Soyuz 7K-T No. 42',42227,'Landed','1977-10-11 00:00:00','1977-10-10 00:00:00',91.34,329,352,51.62,'Intermediate' from dual union all
select 10402,'010402','1977-099B','11S510','11S510',42227,'Reentered','1977-10-11 00:00:00','1977-10-09 00:00:00',88.6,191,220,51.64,'Intermediate' from dual union all
select 10403,'010403','1977-100A','Kosmos-958','Zenit-6',42228,'Landed','1977-10-24 00:00:00','1977-10-13 00:00:00',91.93,317,421,62.81,'Intermediate' from dual union all
select 10404,'010404','1977-100B','11S510','11S510',42228,'Reentered','1977-11-09 00:00:00','1977-10-18 00:00:00',90.24,247,326,62.79,'Intermediate' from dual union all
select 10405,'010405','1977-100C','Sensor cover','Zenit-6 sensor cover',42228,'Reentered','1977-10-31 00:00:00','1977-10-12 00:00:00',90.71,253,365,63,'Intermediate' from dual union all
select 10406,'010406','1977-098E','KDU','KDU',42226,'Reentered','1977-10-15 00:00:00','1977-10-13 00:00:00',88.73,157,266,64.97,'Intermediate' from dual union all
select 10407,'010407','1977-098F','Blenda','Sunshade',42226,'Reentered','1977-10-13 00:00:00','1977-10-12 00:00:00',88.8,158,273,64.96,'Intermediate' from dual union all
select 10408,'010408','1977-092E','SOZ','SOZ',42220,'Reentered','1978-04-30 00:00:00','1977-10-26 00:00:00',626.4,240,35506,47.18,'GEO Transfer' from dual union all
select 10409,'010409','1977-048F','Star 37E S/N 40047','Star 37E',42176,'Reentered','1983-03-26 00:00:00','1977-10-08 00:00:00',660.06,262,37206,22.94,'GEO Transfer' from dual union all
select 1041,'001041','1964-070BF','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',87.84,118,218,51.2,'Intermediate' from dual union all
select 10410,'010410','1976-067D','deb Kosmos-839','',39019,'In Earth orbit','','1993-07-31 00:00:00',116.8,999,2075,66.08,'Intermediate' from dual union all
select 10411,'010411','1976-067E','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-02 00:00:00',117.18,1032,2077,66.02,'Intermediate' from dual union all
select 10412,'010412','1976-067F','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-03 00:00:00',116.71,995,2071,65.67,'Intermediate' from dual union all
select 10413,'010413','1976-067G','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-05 00:00:00',117.32,1009,2112,65.88,'Intermediate' from dual union all
select 10414,'010414','1976-067H','deb Kosmos-839','',39019,'In Earth orbit','','1993-07-31 00:00:00',118.18,1093,2106,65.87,'Intermediate' from dual union all
select 10415,'010415','1976-067J','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-05 00:00:00',118.15,1051,2145,65.83,'Intermediate' from dual union all
select 10416,'010416','1976-067K','deb Kosmos-839','',39019,'In Earth orbit','','1993-07-31 00:00:00',117.99,1078,2104,65.71,'Intermediate' from dual union all
select 10417,'010417','1976-067L','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-08 00:00:00',115.6,909,2057,65.58,'Intermediate' from dual union all
select 10418,'010418','1977-042R','Kosmos-913 SS 14','ESO',42170,'Reentered','1978-06-18 00:00:00','1978-01-26 00:00:00',94.47,476,510,74.01,'Intermediate' from dual union all
select 10419,'010419','1977-101A','Kosmos-959','Lira',42229,'Reentered','1977-11-30 00:00:00','1977-10-28 00:00:00',94.12,141,811,65.83,'Intermediate' from dual union all
select 1042,'001042','1964-070BG','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.63,182,232,51.2,'Intermediate' from dual union all
select 10420,'010420','1977-101B','S3M','S3M',42229,'Reentered','1977-11-08 00:00:00','1977-10-30 00:00:00',92.59,140,662,65.83,'Intermediate' from dual union all
select 10421,'010421','1977-101C','deb Kosmos-959','DS-P1-M part',42229,'Reentered','1977-11-05 00:00:00','1977-10-21 00:00:00',94.87,161,864,65.83,'Intermediate' from dual union all
select 10422,'010422','1977-102A','ISEE 1','ISEE A',42230,'Reentered','1987-09-26 00:00:00','1977-11-29 00:00:00',3442.71,763,137531,30.38,'Very High' from dual union all
select 10423,'010423','1977-102B','ISEE 2','ISEE B',42230,'Reentered','1987-09-26 00:00:00','1977-11-29 00:00:00',3442.57,779,137511,30.41,'Very High' from dual union all
select 10424,'010424','1977-102C','Delta 135','Delta SSPU DSV-3P-4',42230,'Reentered','1978-10-23 00:00:00','1977-11-21 00:00:00',95.67,275,827,28.75,'Intermediate' from dual union all
select 10425,'010425','1977-102D','Star 37E S/N 40051','Star 37E/3731A Adaptor',42230,'Reentered','1987-10-31 00:00:00','1977-11-29 00:00:00',3442.71,763,137531,30.38,'Very High' from dual union all
select 10426,'010426','1977-100D','Blenda','Sunshade',42228,'Reentered','1978-03-05 00:00:00','1977-11-10 00:00:00',91.93,333,406,62.83,'Intermediate' from dual union all
select 10427,'010427','1977-100E','KDU','KDU',42228,'Reentered','1978-09-15 00:00:00','1977-11-10 00:00:00',92.21,351,415,62.81,'Intermediate' from dual union all
select 10428,'010428','1977-100F','KDU part','KDU part',42228,'Reentered','1978-06-18 00:00:00','1978-05-01 00:00:00',91.32,232,447,62.81,'Intermediate' from dual union all
select 10429,'010429','1977-100G','KDU part','KDU part',42228,'Reentered','1978-01-16 00:00:00','1977-11-10 00:00:00',92.03,348,401,62.82,'Intermediate' from dual union all
select 1043,'001043','1964-070BH','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.72,188,234,51.1,'Intermediate' from dual union all
select 10430,'010430','1977-103A','Kosmos-960','Tselina-OM',42231,'Reentered','1980-10-22 00:00:00','1977-11-24 00:00:00',95.07,499,545,74.05,'Intermediate' from dual union all
select 10431,'010431','1977-103B','S3M','S3M',42231,'Reentered','1980-11-30 00:00:00','1977-11-26 00:00:00',94.95,493,540,74.05,'Intermediate' from dual union all
select 10432,'010432','1977-103C','deb Kosmos-960','',42231,'Reentered','1978-06-24 00:00:00','1978-01-15 00:00:00',94.57,470,526,74.09,'Intermediate' from dual union all
select 10433,'010433','1975-109L','deb Kosmos-781','Tselina-O part',36556,'Reentered','1978-05-19 00:00:00','1977-10-28 00:00:00',94.77,485,530,74.07,'Intermediate' from dual union all
select 10434,'010434','1977-104A','Kosmos-961','IS',42232,'Deorbited','1977-10-26 00:00:00','1977-10-26 00:00:00',88.76,125,302,66,'Intermediate' from dual union all
select 10435,'010435','1977-104B','11S692','11S692',42232,'Reentered','1977-10-27 00:00:00','1977-10-26 00:00:00',87.89,121,220,65.08,'Intermediate' from dual union all
select 10436,'010436','1976-067M','deb Kosmos-839','',39019,'In Earth orbit','','1993-07-28 00:00:00',116.48,980,2065,66.02,'Intermediate' from dual union all
select 10437,'010437','1976-067N','deb Kosmos-839','',39019,'In Earth orbit','','1993-07-30 00:00:00',117.17,1010,2098,65.81,'Intermediate' from dual union all
select 10438,'010438','1976-067P','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-04 00:00:00',119.25,1114,2181,65.99,'Intermediate' from dual union all
select 10439,'010439','1976-067Q','deb Kosmos-839','',39019,'In Earth orbit','','1993-07-29 00:00:00',117.62,1044,2104,65.77,'Intermediate' from dual union all
select 1044,'001044','1964-070BJ','deb Kosmos-50','',10356,'Reentered','1964-11-17 00:00:00','1964-11-05 00:00:00',87.99,146,204,51.3,'Intermediate' from dual union all
select 10440,'010440','1976-067R','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-06 00:00:00',116.31,947,2083,65.64,'Intermediate' from dual union all
select 10441,'010441','1976-067S','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-07 00:00:00',118.86,1143,2117,65.77,'Intermediate' from dual union all
select 10442,'010442','1976-067T','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-08 00:00:00',118.36,1081,2134,66,'Intermediate' from dual union all
select 10443,'010443','1976-067U','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-07 00:00:00',115.68,903,2070,65.81,'Intermediate' from dual union all
select 10444,'010444','1976-067V','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-02 00:00:00',115.2,871,2058,65.79,'Intermediate' from dual union all
select 10445,'010445','1976-067W','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-05 00:00:00',119.2,1110,2180,65.96,'Intermediate' from dual union all
select 10446,'010446','1976-067X','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-02 00:00:00',118.34,1093,2120,65.87,'Intermediate' from dual union all
select 10447,'010447','1976-067Y','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-03 00:00:00',116.49,955,2091,65.86,'Intermediate' from dual;
insert into SATELLITE
select 10448,'010448','1976-067Z','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-04 00:00:00',116.46,925,2119,66.17,'Intermediate' from dual union all
select 10449,'010449','1976-067AA','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-07 00:00:00',115.63,894,2074,66.02,'Intermediate' from dual union all
select 1045,'001045','1964-070BK','deb Kosmos-50','',10356,'Reentered','1964-11-17 00:00:00','1964-11-05 00:00:00',89.47,195,302,51.2,'Intermediate' from dual union all
select 10450,'010450','1976-067AB','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-08 00:00:00',115.5,882,2075,65.74,'Intermediate' from dual union all
select 10451,'010451','1976-067AC','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-01 00:00:00',115.59,871,2094,65.85,'Intermediate' from dual union all
select 10452,'010452','1976-067AD','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-06 00:00:00',119.45,1179,2134,65.84,'Intermediate' from dual union all
select 10453,'010453','1976-067AE','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-04 00:00:00',116,924,2078,65.8,'Intermediate' from dual union all
select 10454,'010454','1976-067AF','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-08 00:00:00',115.07,876,2042,65.68,'Intermediate' from dual union all
select 10455,'010455','1977-105A','Molniya-3','Molniya-3 No. 18',42233,'In Earth orbit','','1977-11-28 00:00:00',718.06,379,39990,62.81,'Molniya' from dual union all
select 10456,'010456','1977-105B','11S510','11S510',42233,'Reentered','1977-11-18 00:00:00','1977-11-04 00:00:00',90.55,200,403,62.81,'Intermediate' from dual union all
select 10457,'010457','1977-106A','Transat O-11','NNS O-11',42234,'In Earth orbit','','1992-06-10 00:00:00',106.85,1062,1099,89.7,'Polar' from dual union all
select 10458,'010458','1977-105C','BOZ','BOZ',42233,'Reentered','1977-11-26 00:00:00','1977-11-04 00:00:00',90.86,206,428,62.8,'Intermediate' from dual union all
select 10459,'010459','1977-107A','Kosmos-962','Zaliv',42235,'In Earth orbit','','1992-05-26 00:00:00',104.79,965,1005,82.95,'Intermediate' from dual union all
select 1046,'001046','1964-070BL','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.82,193,240,51.3,'Intermediate' from dual union all
select 10460,'010460','1977-105D','deb Blok-I','',42233,'Reentered','1977-11-20 00:00:00','1977-10-30 00:00:00',91.11,209,448,62.8,'Intermediate' from dual union all
select 10461,'010461','1977-107B','S3M','S3M',42235,'In Earth orbit','','1991-03-25 00:00:00',104.65,955,1002,82.94,'Intermediate' from dual union all
select 10462,'010462','1977-106B','Star 20 E-20','Star 20 E-20',42234,'In Earth orbit','','1993-08-13 00:00:00',106.87,1062,1102,89.7,'Polar' from dual union all
select 10463,'010463','1976-067AG','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-02 00:00:00',116.91,985,2099,65.82,'Intermediate' from dual union all
select 10464,'010464','1976-067AH','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-03 00:00:00',115.84,914,2073,65.82,'Intermediate' from dual union all
select 10465,'010465','1976-067AJ','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-02 00:00:00',114.83,864,2032,65.57,'Intermediate' from dual union all
select 10466,'010466','1976-067AK','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-08 00:00:00',115.63,906,2062,65.84,'Intermediate' from dual union all
select 10467,'010467','1976-067AL','deb Kosmos-839','',39019,'Reentered','1982-06-03 00:00:00','1977-11-01 00:00:00',115.74,911,2067,65.8,'Intermediate' from dual union all
select 10468,'010468','1977-065EA','deb Delta 132','',42193,'Reentered','1980-09-15 00:00:00','1980-08-25 00:00:00',95.77,457,655,28.69,'Intermediate' from dual union all
select 10469,'010469','1977-065EB','deb Delta 132','',42193,'In Earth orbit','','1993-08-06 00:00:00',113.57,700,2081,29.11,'Intermediate' from dual union all
select 1047,'001047','1964-070BM','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',88.14,168,197,51.2,'Intermediate' from dual union all
select 10470,'010470','1977-065EC','deb Delta 132','',42193,'In Earth orbit','','1993-08-07 00:00:00',107.16,545,1646,29.07,'Intermediate' from dual union all
select 10471,'010471','1977-065ED','deb Delta 132','',42193,'In Earth orbit','','1993-08-06 00:00:00',109.7,530,1896,29.06,'Intermediate' from dual union all
select 10472,'010472','1977-065EE','deb Delta 132','',42193,'Reentered','1981-03-16 00:00:00','1977-11-14 00:00:00',111.6,578,2023,29.06,'Intermediate' from dual union all
select 10473,'010473','1977-065EF','deb Delta 132','',42193,'Reentered','1982-01-07 00:00:00','1982-01-07 00:00:00',90.2,260,309,28.84,'Intermediate' from dual union all
select 10474,'010474','1977-065EG','deb Delta 132','',42193,'Reentered','1978-06-12 00:00:00','1978-02-24 00:00:00',106.8,495,1661,29.63,'Intermediate' from dual union all
select 10475,'010475','1977-065EH','deb Delta 132','',42193,'Reentered','1981-11-10 00:00:00','1981-09-28 00:00:00',97.34,446,816,29.12,'Intermediate' from dual union all
select 10476,'010476','1977-065EJ','deb Delta 132','',42193,'Reentered','2014-04-16 00:00:00','1993-08-07 00:00:00',103.01,492,1310,28.96,'Intermediate' from dual union all
select 10477,'010477','1977-065EK','deb Delta 132','',42193,'Reentered','1981-12-10 00:00:00','1981-07-25 00:00:00',98.82,497,908,29.15,'Intermediate' from dual union all
select 10478,'010478','1977-065EL','deb Delta 132','',42193,'In Earth orbit','','1993-08-05 00:00:00',106.96,502,1670,29.29,'Intermediate' from dual union all
select 10479,'010479','1977-065EM','deb Delta 132','',42193,'Reentered','2011-11-22 00:00:00','1993-08-06 00:00:00',104.2,525,1389,29.08,'Intermediate' from dual union all
select 1048,'001048','1964-070BN','deb Kosmos-50','',10356,'Reentered','1964-11-14 00:00:00','1964-11-05 00:00:00',88.14,167,198,51.3,'Intermediate' from dual union all
select 10480,'010480','1977-065EN','deb Delta 132','',42193,'Reentered','1992-06-18 00:00:00','1992-02-12 00:00:00',97.95,522,800,29.12,'Intermediate' from dual union all
select 10481,'010481','1977-065EP','deb Delta 132','',42193,'Reentered','1987-02-03 00:00:00','1977-11-24 00:00:00',108.55,468,1852,28.64,'Intermediate' from dual union all
select 10482,'010482','1977-065EQ','deb Delta 132','',42193,'Reentered','1982-11-10 00:00:00','1980-05-25 00:00:00',116.5,259,2788,30.32,'Intermediate' from dual union all
select 10483,'010483','1977-065ER','deb Delta 132','',42193,'Reentered','1981-08-20 00:00:00','1981-04-10 00:00:00',103.66,559,1304,29.4,'Intermediate' from dual union all
select 10484,'010484','1977-092F','SOZ','SOZ',42220,'Reentered','1978-06-18 00:00:00','1977-11-17 00:00:00',627.47,269,35532,47.14,'GEO Transfer' from dual union all
select 10485,'010485','1977-105E','Blok-L','Blok-L',42233,'In Earth orbit','','1977-12-01 00:00:00',731.69,433,40606,62.86,'Molniya' from dual union all
select 10486,'010486','1977-011C','deb Kosmos-893','',42139,'Reentered','1978-03-18 00:00:00','1977-11-12 00:00:00',99.13,317,1117,73.99,'Intermediate' from dual union all
select 10487,'010487','1977-042S','Kosmos-913 SS 15','ESO',42170,'Reentered','1978-03-22 00:00:00','1977-11-20 00:00:00',94.44,469,514,73.98,'Intermediate' from dual union all
select 10488,'010488','1977-042T','Kosmos-913 SS 16','ESO',42170,'Reentered','1978-04-18 00:00:00','1977-11-20 00:00:00',94.44,468,515,74.11,'Intermediate' from dual union all
select 10489,'010489','1977-108A','Meteosat 1','Meteosat 1',42236,'In Earth orbit','','1978-02-20 00:00:00',1436.24,35776,35803,.22,'Stationary' from dual union all
select 1049,'001049','1964-070BP','deb Kosmos-50','',10356,'Reentered','1964-11-15 00:00:00','1964-11-05 00:00:00',88.26,181,195,51.3,'Intermediate' from dual union all
select 10490,'010490','1977-108B','Delta 136','Delta SSPU DSV-3P-4',42236,'In Earth orbit','','1991-03-12 00:00:00',115.34,487,2455,28.3,'Intermediate' from dual union all
select 10491,'010491','1977-109A','Kosmos-963','Geoid',42237,'In Earth orbit','','1992-06-10 00:00:00',109.25,1178,1207,82.93,'Intermediate' from dual union all
select 10492,'010492','1977-109B','S3M','S3M',42237,'In Earth orbit','','1993-08-08 00:00:00',109.14,1171,1202,82.92,'Intermediate' from dual union all
select 10493,'010493','1977-065ES','deb Delta 132','',42193,'Reentered','2001-11-01 00:00:00','1993-08-22 00:00:00',104.83,591,1382,29.73,'Intermediate' from dual union all
select 10494,'010494','1977-042U','Kosmos-913 SS 17','ESO',42170,'Reentered','1978-03-18 00:00:00','1977-11-30 00:00:00',94.43,468,514,74.05,'Intermediate' from dual union all
select 10495,'010495','1977-042V','Kosmos-913 SS 18','ESO',42170,'Reentered','1978-03-18 00:00:00','1977-11-30 00:00:00',94.43,468,514,73.99,'Intermediate' from dual union all
select 10496,'010496','1977-042W','Kosmos-913 SS 19','ESO',42170,'Reentered','1978-04-18 00:00:00','1977-11-30 00:00:00',94.44,468,515,74.08,'Intermediate' from dual union all
select 10497,'010497','1977-042X','Kosmos-913 SS 20','ESO',42170,'Reentered','1978-03-27 00:00:00','1977-11-30 00:00:00',94.44,469,514,74.05,'Intermediate' from dual union all
select 10498,'010498','1977-110A','Kosmos-964','Zenit-4MKM',42238,'Landed','1977-12-17 00:00:00','1977-12-06 00:00:00',89.76,169,357,72.88,'Intermediate' from dual union all
select 10499,'010499','1977-110B','11S510','11S510',42238,'Reentered','1977-12-09 00:00:00','1977-12-07 00:00:00',89.16,162,304,72.88,'Intermediate' from dual union all
select 105,'000105','60 Xi 4','Explorer VIII despin weight','',6334,'Reentered','1972-03-03 00:00:00','1961-04-20 00:00:00',112.33,422,2246,50.52,'Intermediate' from dual union all
select 1050,'001050','1964-070BQ','deb Kosmos-50','',10356,'Reentered','1964-11-14 00:00:00','1964-11-05 00:00:00',89,179,271,51.3,'Intermediate' from dual union all
select 10500,'010500','1977-110C','Sensor cover','Zenit-4MK sensor cover',42238,'Reentered','1978-01-06 00:00:00','1977-12-04 00:00:00',89.4,167,323,72.91,'Intermediate' from dual union all
select 10501,'010501','1977-111A','Kosmos-965','Romb 4',42239,'Reentered','1979-12-16 00:00:00','1978-01-07 00:00:00',94.36,463,513,74.03,'Intermediate' from dual union all
select 10502,'010502','1977-112A','OPS 8781','PARCAE 2',42240,'In Earth orbit','','1990-05-14 00:00:00',107.42,912,1302,63.41,'Intermediate' from dual union all
select 10503,'010503','1977-111B','S3M','S3M',42239,'Reentered','1979-11-14 00:00:00','1978-01-09 00:00:00',94.28,456,512,74.03,'Intermediate' from dual union all
select 10504,'010504','1977-112B','MSD plume shield','',42240,'In Earth orbit','','1997-11-22 00:00:00',105.8,616,1448,63.36,'Intermediate' from dual union all
select 10505,'010505','1977-111C','Kosmos-965 SS 1','ESO',42239,'Reentered','1978-04-15 00:00:00','1978-01-16 00:00:00',93.59,413,487,74.06,'Intermediate' from dual union all
select 10506,'010506','1977-113A','Soyuz-26','Soyuz 7K-T No. 43',42241,'Landed','1978-01-16 00:00:00','1977-12-27 00:00:00',91.38,333,352,51.59,'Intermediate' from dual union all
select 10507,'010507','1977-113B','11S510','11S510',42241,'Reentered','1977-12-12 00:00:00','1977-12-11 00:00:00',88.37,180,208,51.65,'Intermediate' from dual union all
select 10508,'010508','1977-114A','OPS 4258','AQUACADE 3',42242,'In Earth orbit','','1977-12-12 00:00:00',1435.5,35750,35800,0,'Stationary' from dual union all
select 10509,'010509','1977-114B','Agena D 1553','Agena D',42242,'In Earth orbit','1990-01-01 00:00:00','1977-12-12 00:00:00',591.76,182,33759,27.8,'Highly Ellip' from dual union all
select 1051,'001051','1964-070BR','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.53,170,234,51.1,'Intermediate' from dual union all
select 10510,'010510','1977-115A','Kosmos-966','Zenit-2M',42243,'Landed','1977-12-24 00:00:00','1977-12-18 00:00:00',89.4,201,288,65.03,'Intermediate' from dual union all
select 10511,'010511','1977-115B','11S510','11S510',42243,'Reentered','1977-12-18 00:00:00','1977-12-14 00:00:00',89.14,194,270,65.05,'Intermediate' from dual union all
select 10512,'010512','1977-116A','Kosmos-967','Lira',42244,'In Earth orbit','','1978-01-13 00:00:00',104.75,962,1004,65.83,'Intermediate' from dual union all
select 10513,'010513','1977-116B','S3M','S3M',42244,'In Earth orbit','','1991-03-28 00:00:00',104.53,960,985,65.83,'Intermediate' from dual union all
select 10514,'010514','1977-117A','Meteor-2','Meteor-2 No. 3',42245,'In Earth orbit','','1992-06-05 00:00:00',102.21,849,878,81.21,'Intermediate' from dual union all
select 10515,'010515','1977-117B','8A92ME','8A92ME',42245,'In Earth orbit','','1993-08-08 00:00:00',102.31,836,900,81.23,'Intermediate' from dual union all
select 10516,'010516','1977-118A','Sakura','CS',42246,'In Earth orbit','','1978-02-02 00:00:00',1436.04,35782,35789,.08,'Stationary' from dual union all
select 10517,'010517','1977-118B','Delta 137','Delta SSPU DSV-3P-4',42246,'In Earth orbit','','1990-02-19 00:00:00',109.73,482,1947,28.66,'Intermediate' from dual union all
select 10518,'010518','1977-116C','deb Kosmos-967','DS-P1-M part [RCS 0.16]',42244,'In Earth orbit','','1993-08-05 00:00:00',104.63,966,989,65.83,'Intermediate' from dual union all
select 10519,'010519','1977-118C','deb Delta 137','[UNID079] RCS 0.04',42246,'In Earth orbit','','1993-08-15 00:00:00',109.56,538,1875,29.06,'Intermediate' from dual union all
select 1052,'001052','1964-070BS','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.66,188,229,51.3,'Intermediate' from dual union all
select 10520,'010520','1977-119A','Kosmos-968','Strela-2M',42247,'In Earth orbit','','1992-06-17 00:00:00',100.46,767,794,74.03,'Intermediate' from dual union all
select 10521,'010521','1977-119B','S3M','S3M',42247,'In Earth orbit','','1991-03-27 00:00:00',100.28,756,788,74.02,'Intermediate' from dual union all
select 10522,'010522','1977-110D','KDU','KDU',42238,'Reentered','1977-12-20 00:00:00','1977-12-16 00:00:00',89.55,163,342,72.87,'Intermediate' from dual union all
select 10523,'010523','1977-110E','Blenda','Sunshade',42238,'Reentered','1977-12-20 00:00:00','1977-12-16 00:00:00',89.24,163,311,72.88,'Intermediate' from dual union all
select 10524,'010524','1977-119C','deb Kosmos-968','Strela-2M debris',42247,'In Earth orbit','','1993-08-11 00:00:00',99.86,739,764,74.03,'Intermediate' from dual union all
select 10525,'010525','1977-119D','deb Kosmos-968','Strela-2M debris',42247,'In Earth orbit','','1993-08-10 00:00:00',99.9,740,768,74.03,'Intermediate' from dual union all
select 10526,'010526','1977-116D','deb Kosmos-967','DS-P1-M part [RCS 0.26]',42244,'In Earth orbit','','1993-08-11 00:00:00',104.74,966,999,65.83,'Intermediate' from dual union all
select 10527,'010527','1977-120A','Kosmos-969','Zenit-4MKM',42248,'Landed','1978-01-03 00:00:00','1977-12-22 00:00:00',89.45,179,316,62.82,'Intermediate' from dual union all
select 10528,'010528','1977-112C','SS 1 cover','SSU cover',42240,'In Earth orbit','','1997-11-22 00:00:00',106.87,660,1504,63.41,'Intermediate' from dual union all
select 10529,'010529','1977-112D','SS 1','SSU',42240,'In Earth orbit','','1997-11-22 00:00:00',107.4,668,1545,63.41,'Intermediate' from dual union all
select 1053,'001053','1964-070BT','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',89.82,194,338,51.5,'Intermediate' from dual union all
select 10530,'010530','1977-120B','11S510','11S510',42248,'Reentered','1977-12-26 00:00:00','1977-12-22 00:00:00',89.19,175,294,62.81,'Intermediate' from dual union all
select 10531,'010531','1977-121A','Kosmos-970','IS',42249,'In Earth orbit','','1978-01-06 00:00:00',105.93,937,1139,65.86,'Intermediate' from dual union all
select 10532,'010532','1977-121B','11S692','11S692',42249,'Reentered','1977-12-25 00:00:00','1977-12-24 00:00:00',89.67,128,389,65.14,'Intermediate' from dual union all
select 10533,'010533','1977-097D','Garbage bag','',42225,'Reentered','1978-02-22 00:00:00','1977-12-25 00:00:00',91.27,326,348,51.59,'Intermediate' from dual union all
select 10534,'010534','1977-115C','Nauka part','',42243,'Reentered','1977-12-29 00:00:00','1977-12-22 00:00:00',89.15,198,266,65.03,'Intermediate' from dual union all
select 10535,'010535','1977-121C','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-05 00:00:00',105.31,907,1111,65.85,'Intermediate' from dual union all
select 10536,'010536','1977-122A','Kosmos-971','Parus',42250,'In Earth orbit','','1992-06-07 00:00:00',104.9,974,1006,82.93,'Intermediate' from dual union all
select 10537,'010537','1977-122B','S3M','S3M',42250,'In Earth orbit','','1991-03-25 00:00:00',104.75,967,999,82.93,'Intermediate' from dual;
insert into SATELLITE
select 10538,'010538','1977-115D','Kapsula Kosmosa-966','Nauka 26KS',42243,'Reentered','1978-01-07 00:00:00','1977-12-24 00:00:00',89.24,198,275,65.03,'Intermediate' from dual union all
select 10539,'010539','1977-123A','Kosmos-972','EPN No. 03.0380',42251,'In Earth orbit','','1993-04-24 00:00:00',103.74,712,1159,75.83,'Intermediate' from dual union all
select 1054,'001054','1964-070BU','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',87.81,137,196,51.5,'Intermediate' from dual union all
select 10540,'010540','1977-124A','Kosmos-973','Zenit-2M',42252,'Landed','1978-01-09 00:00:00','1977-12-31 00:00:00',89.73,201,322,71.43,'Intermediate' from dual union all
select 10541,'010541','1977-123B','S5M','S5M',42251,'In Earth orbit','','1991-03-28 00:00:00',103.74,714,1157,75.84,'Intermediate' from dual union all
select 10542,'010542','1977-124B','11S510','11S510',42252,'Reentered','1978-01-05 00:00:00','1977-12-29 00:00:00',89.45,210,285,71.43,'Intermediate' from dual union all
select 10543,'010543','1977-124C','Sensor cover','Zenit-2M sensor cover',42252,'Reentered','1977-12-31 00:00:00','1977-12-27 00:00:00',89.14,193,271,71.46,'Intermediate' from dual union all
select 10544,'010544','1977-112E','SS 2','SSU',42240,'In Earth orbit','','1997-11-22 00:00:00',107.4,668,1545,63.41,'Intermediate' from dual union all
select 10545,'010545','1977-097E','Garbage bag','',42225,'Reentered','1978-04-24 00:00:00','1977-12-31 00:00:00',91.33,329,351,51.6,'Intermediate' from dual union all
select 10546,'010546','1977-123C','deb Kosmos-972','',42251,'Reentered','1977-12-31 00:00:00','1977-12-30 00:00:00',103.82,715,1164,75.83,'Intermediate' from dual union all
select 10547,'010547','1977-097F','Garbage bag','',42225,'Reentered','1978-05-12 00:00:00','1977-12-30 00:00:00',91.33,329,351,51.57,'Intermediate' from dual union all
select 10548,'010548','1977-097G','Garbage bag','',42225,'Reentered','1978-03-25 00:00:00','1977-12-31 00:00:00',91.41,336,351,51.6,'Intermediate' from dual union all
select 10549,'010549','1977-120C','KDU','KDU',42248,'Reentered','1978-01-05 00:00:00','1978-01-03 00:00:00',89.37,159,327,62.82,'Intermediate' from dual union all
select 1055,'001055','1964-070BV','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',88.93,195,249,50.9,'Intermediate' from dual union all
select 10550,'010550','1977-120D','Blenda','Sunshade',42248,'Reentered','1978-01-04 00:00:00','1978-01-02 00:00:00',89.26,159,317,62.81,'Intermediate' from dual union all
select 10551,'010551','1977-120E','KDU part','KDU part',42248,'Reentered','1978-03-05 00:00:00','1978-01-03 00:00:00',89.18,163,305,62.8,'Intermediate' from dual union all
select 10552,'010552','1977-120F','KDU part','KDU part',42248,'Reentered','1978-03-05 00:00:00','1978-01-03 00:00:00',89.4,180,310,62.8,'Intermediate' from dual union all
select 10553,'010553','1977-097H','Garbage bag','',42225,'Reentered','1978-05-27 00:00:00','1978-01-04 00:00:00',91.23,325,344,51.58,'Intermediate' from dual union all
select 10554,'010554','1978-001A','Kosmos-974','Zenit-4MKM',45196,'Landed','1978-01-19 00:00:00','1978-01-08 00:00:00',89.58,178,330,62.83,'Intermediate' from dual union all
select 10555,'010555','1978-001B','11S510','11S510',45196,'Reentered','1978-01-11 00:00:00','1978-01-08 00:00:00',89.28,174,304,62.85,'Intermediate' from dual union all
select 10556,'010556','1978-001C','Sensor cover','Zenit-4MK sensor cover',45196,'Reentered','1978-01-25 00:00:00','1978-01-06 00:00:00',89.42,183,309,62.82,'Intermediate' from dual union all
select 10557,'010557','1978-002A','INTELSAT IVA F3','INTELSAT IVA F3',45197,'In Earth orbit','','1978-03-15 00:00:00',1436.09,35783,35790,.37,'Stationary' from dual union all
select 10558,'010558','1977-124D','Kapsula Kosmosa-973','Nauka 27KS',42252,'Reentered','1978-01-25 00:00:00','1978-01-08 00:00:00',89.6,198,312,71.43,'Intermediate' from dual union all
select 10559,'010559','1977-124E','Nauka part','',42252,'Reentered','1978-01-11 00:00:00','1978-01-08 00:00:00',89.38,198,290,71.43,'Intermediate' from dual union all
select 1056,'001056','1964-070BW','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.9,192,249,51.1,'Intermediate' from dual union all
select 10560,'010560','1978-003A','Soyuz-27','Soyuz 7K-T No. 44',45198,'Landed','1978-03-16 00:00:00','1978-02-20 00:00:00',91.18,326,339,51.63,'Intermediate' from dual union all
select 10561,'010561','1978-004A','Kosmos-975','Tselina-D',45199,'Reentered','2001-09-19 00:00:00','1991-03-27 00:00:00',95.79,550,564,81.2,'Intermediate' from dual union all
select 10562,'010562','1977-121D','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-01 00:00:00',108.02,1056,1215,65.87,'Intermediate' from dual union all
select 10563,'010563','1977-121E','deb Kosmos-970','',42249,'In Earth orbit','','1993-07-31 00:00:00',104.88,897,1080,65.85,'Intermediate' from dual union all
select 10564,'010564','1977-121F','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-03 00:00:00',105.36,894,1129,65.86,'Intermediate' from dual union all
select 10565,'010565','1977-121G','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-05 00:00:00',105.91,938,1135,65.83,'Intermediate' from dual union all
select 10566,'010566','1977-121H','Unknown debris                    ','',42249,'In Earth orbit','','1993-08-07 00:00:00',109.68,877,1547,65.83,'Intermediate' from dual union all
select 10567,'010567','1977-121J','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-07 00:00:00',106.33,977,1136,65.79,'Intermediate' from dual union all
select 10568,'010568','1977-121K','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-03 00:00:00',106.02,883,1202,65.77,'Intermediate' from dual union all
select 10569,'010569','1977-121L','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-05 00:00:00',105.03,891,1101,65.4,'Intermediate' from dual union all
select 1057,'001057','1964-070BX','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.5,170,231,51.4,'Intermediate' from dual union all
select 10570,'010570','1977-121M','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-02 00:00:00',106.38,1056,1062,65.38,'Intermediate' from dual union all
select 10571,'010571','1977-121N','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-02 00:00:00',104.67,869,1089,65.94,'Intermediate' from dual union all
select 10572,'010572','1977-121P','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-07 00:00:00',104.74,850,1115,65.03,'Intermediate' from dual union all
select 10573,'010573','1977-121Q','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-06 00:00:00',106.7,1010,1138,66.36,'Intermediate' from dual union all
select 10574,'010574','1977-121R','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-02 00:00:00',106.27,953,1155,65.54,'Intermediate' from dual union all
select 10575,'010575','1977-121S','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-08 00:00:00',106.03,948,1138,65.88,'Intermediate' from dual union all
select 10576,'010576','1977-121T','deb Kosmos-970','',42249,'In Earth orbit','','1993-07-30 00:00:00',105.62,911,1136,65.84,'Intermediate' from dual union all
select 10577,'010577','1977-121U','deb Kosmos-970','',42249,'In Earth orbit','','1991-10-08 00:00:00',105.53,909,1130,65.79,'Intermediate' from dual union all
select 10578,'010578','1977-121V','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-08 00:00:00',105.51,910,1126,65.77,'Intermediate' from dual union all
select 10579,'010579','1977-121W','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-04 00:00:00',108.24,1104,1186,65.65,'Intermediate' from dual union all
select 1058,'001058','1964-070BY','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.25,165,211,51.3,'Intermediate' from dual union all
select 10580,'010580','1977-121X','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-06 00:00:00',107.45,1057,1161,65.68,'Intermediate' from dual union all
select 10581,'010581','1978-005A','Kosmos-976','Strela-1M',45200,'In Earth orbit','','1992-05-06 00:00:00',115.07,1455,1462,74.03,'Intermediate' from dual union all
select 10582,'010582','1978-004B','8A92ME','8A92ME',45199,'Reentered','2011-10-28 00:00:00','1991-03-28 00:00:00',96.19,545,608,81.18,'Intermediate' from dual union all
select 10583,'010583','1978-003B','11S510','11S510',45198,'Reentered','1978-01-13 00:00:00','1978-01-12 00:00:00',88,157,194,51.64,'Intermediate' from dual union all
select 10584,'010584','1978-005B','Kosmos-977','Strela-1M',45200,'In Earth orbit','','1992-05-06 00:00:00',114.47,1400,1462,74.03,'Intermediate' from dual union all
select 10585,'010585','1978-005C','Kosmos-978','Strela-1M',45200,'In Earth orbit','','1992-05-06 00:00:00',114.68,1419,1463,74.03,'Intermediate' from dual union all
select 10586,'010586','1978-005D','Kosmos-979','Strela-1M',45200,'In Earth orbit','','1992-05-06 00:00:00',114.88,1437,1463,74.03,'Intermediate' from dual union all
select 10587,'010587','1978-005E','Kosmos-980','Strela-1M',45200,'In Earth orbit','','1992-05-06 00:00:00',115.29,1463,1475,74.03,'Intermediate' from dual union all
select 10588,'010588','1978-005F','Kosmos-981','Strela-1M',45200,'In Earth orbit','','1993-04-16 00:00:00',115.53,1463,1495,74.03,'Intermediate' from dual union all
select 10589,'010589','1978-005G','Kosmos-982','Strela-1M',45200,'In Earth orbit','','1992-05-06 00:00:00',115.75,1464,1515,74.03,'Intermediate' from dual union all
select 1059,'001059','1964-070BZ','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',88.48,174,225,51.2,'Intermediate' from dual union all
select 10590,'010590','1978-005H','Kosmos-983','Strela-1M',45200,'In Earth orbit','','1992-05-06 00:00:00',115.99,1464,1536,74.03,'Intermediate' from dual union all
select 10591,'010591','1978-005J','S3M','S3M',45200,'In Earth orbit','','1992-05-10 00:00:00',117.71,1462,1694,74.04,'Intermediate' from dual union all
select 10592,'010592','1978-006A','Kosmos-984','Zenit-2M',45201,'Landed','1978-01-26 00:00:00','1978-01-15 00:00:00',89.44,205,289,62.81,'Intermediate' from dual union all
select 10593,'010593','1978-006B','11S510','11S510',45201,'Reentered','1978-01-22 00:00:00','1978-01-17 00:00:00',89.06,204,251,62.79,'Intermediate' from dual union all
select 10594,'010594','1977-112F','SS 3','SSU',42240,'In Earth orbit','','1990-05-14 00:00:00',107.43,870,1346,63.44,'Intermediate' from dual union all
select 10595,'010595','1977-112G','SS 3 cover','SSU cover',42240,'In Earth orbit','','1997-11-21 00:00:00',106.95,672,1499,63.4,'Intermediate' from dual union all
select 10596,'010596','1977-097J','Garbage bag','',42225,'Reentered','1978-05-14 00:00:00','1978-01-14 00:00:00',91.25,323,349,51.58,'Intermediate' from dual union all
select 10597,'010597','1977-097K','Garbage bag','',42225,'Reentered','1978-03-21 00:00:00','1978-01-14 00:00:00',91.34,330,351,51.59,'Intermediate' from dual union all
select 10598,'010598','1977-097L','Garbage bag','',42225,'Reentered','1978-02-08 00:00:00','1978-01-16 00:00:00',91.3,328,349,51.58,'Intermediate' from dual union all
select 10599,'010599','1978-007A','Kosmos-985','Parus',45202,'In Earth orbit','','1992-06-06 00:00:00',104.64,938,1018,82.94,'Intermediate' from dual union all
select 106,'000106','61 Lambda 3','Star 12','Star 12',7269,'Reentered','1961-09-10 00:00:00','1961-05-09 00:00:00',100.37,197,1356,81.94,'Intermediate' from dual union all
select 1060,'001060','1964-070CA','deb Kosmos-50','',10356,'Reentered','1964-11-10 00:00:00','1964-11-05 00:00:00',89.44,195,299,51.2,'Intermediate' from dual union all
select 10600,'010600','1978-007B','S3M','S3M',45202,'In Earth orbit','','1991-03-28 00:00:00',104.55,938,1009,82.93,'Intermediate' from dual union all
select 10601,'010601','1978-001D','KDU','KDU',45196,'Reentered','1978-01-26 00:00:00','1978-01-18 00:00:00',89.3,172,307,62.81,'Intermediate' from dual union all
select 10602,'010602','1978-001E','Blenda','Sunshade',45196,'Reentered','1978-01-21 00:00:00','1978-01-18 00:00:00',89.11,172,289,62.81,'Intermediate' from dual union all
select 10603,'010603','1978-008A','Progress-1','Progress 7K-TG No. 102',45203,'Deorbited','1978-02-08 00:00:00','1978-02-06 00:00:00',91.21,324,344,51.63,'Intermediate' from dual union all
select 10604,'010604','1978-008B','11S510','11S510',45203,'Reentered','1978-01-23 00:00:00','1978-01-22 00:00:00',88.29,174,206,51.67,'Intermediate' from dual union all
select 10605,'010605','1978-009A','Molniya-3','Molniya-3 No. 20',45204,'Reentered','1990-04-24 00:00:00','1978-02-23 00:00:00',718,675,39691,62.83,'Molniya' from dual union all
select 10606,'010606','1978-009B','11S510','11S510',45204,'Reentered','1978-03-12 00:00:00','1978-02-01 00:00:00',92.57,213,588,62.77,'Intermediate' from dual union all
select 10607,'010607','1978-010A','Kosmos-986','Zenit-4MKM',45205,'Landed','1978-02-07 00:00:00','1978-01-26 00:00:00',89.61,170,341,65.02,'Intermediate' from dual union all
select 10608,'010608','1978-009C','BOZ','BOZ',45204,'Reentered','1978-03-03 00:00:00','1978-02-12 00:00:00',91.64,192,517,62.82,'Intermediate' from dual union all
select 10609,'010609','1978-010B','11S510','11S510',45205,'Reentered','1978-01-28 00:00:00','1978-01-26 00:00:00',88.77,169,259,65.01,'Intermediate' from dual union all
select 1061,'001061','1964-070CB','deb Kosmos-50','',10356,'Reentered','1964-11-09 00:00:00','1964-11-05 00:00:00',89.04,183,272,51.2,'Intermediate' from dual union all
select 10610,'010610','1978-009D','deb Blok-I','',45204,'Reentered','1978-03-08 00:00:00','1978-02-01 00:00:00',92.47,196,596,62.84,'Intermediate' from dual union all
select 10611,'010611','1978-011A','Fanhui Shi Weixing (03)','FSW No. 4',45206,'Reentered','1978-02-07 00:00:00','1978-01-28 00:00:00',90.82,161,469,57.03,'Intermediate' from dual union all
select 10612,'010612','1978-011B','CZ-2C Stage 2','CZ-2C Stage 2',45206,'Reentered','1978-02-06 00:00:00','1978-01-28 00:00:00',90.49,159,439,57.02,'Intermediate' from dual union all
select 10613,'010613','1976-067AM','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-05 00:00:00',114.76,865,2024,65.79,'Intermediate' from dual union all
select 10614,'010614','1976-067AN','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-09 00:00:00',116.21,923,2098,65.73,'Intermediate' from dual union all
select 10615,'010615','1976-067AP','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-04 00:00:00',115.38,932,2014,66.01,'Intermediate' from dual union all
select 10616,'010616','1976-067AQ','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-08 00:00:00',115.68,915,2058,65.6,'Intermediate' from dual union all
select 10617,'010617','1976-067AR','deb Kosmos-839','RCS 0.39',39019,'In Earth orbit','','1993-08-01 00:00:00',116.49,945,2101,65.63,'Intermediate' from dual union all
select 10618,'010618','1976-067AS','deb Kosmos-839','',39019,'In Earth orbit','','1993-07-31 00:00:00',116.31,934,2096,65.84,'Intermediate' from dual union all
select 10619,'010619','1976-067AT','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-01 00:00:00',116.77,1003,2069,65.82,'Intermediate' from dual union all
select 1062,'001062','1964-070CC','deb Kosmos-50','',10356,'Reentered','1964-11-16 00:00:00','1964-11-05 00:00:00',88.29,146,234,51.2,'Intermediate' from dual union all
select 10620,'010620','1976-067AU','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-08 00:00:00',116.52,992,2057,65.96,'Intermediate' from dual union all
select 10621,'010621','1976-067AV','deb Kosmos-839','RCS 0.33',39019,'In Earth orbit','','1993-08-04 00:00:00',117.02,1021,2073,66.03,'Intermediate' from dual union all
select 10622,'010622','1976-067AW','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-05 00:00:00',116.73,941,2127,65.91,'Intermediate' from dual union all
select 10623,'010623','1976-067AX','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-08 00:00:00',116.14,941,2074,65.78,'Intermediate' from dual union all
select 10624,'010624','1977-065ET','deb Delta 132','',42193,'Reentered','1980-06-27 00:00:00','1980-06-22 00:00:00',91.75,306,415,29.2,'Intermediate' from dual union all
select 10625,'010625','1977-065EU','deb Delta 132','',42193,'Reentered','1982-07-30 00:00:00','1978-01-26 00:00:00',110.66,475,2040,29.14,'Intermediate' from dual union all
select 10626,'010626','1977-065EV','deb Delta 132','',42193,'In Earth orbit','','1993-08-05 00:00:00',108.74,535,1801,29.18,'Intermediate' from dual union all
select 10627,'010627','1977-065EW','deb Delta 132','',42193,'In Earth orbit','','1993-08-09 00:00:00',111.33,759,1817,28.95,'Intermediate' from dual;
insert into SATELLITE
select 10628,'010628','1977-065EX','deb Delta 132','',42193,'Reentered','1990-03-21 00:00:00','1990-03-21 00:00:00',89.04,211,244,28.94,'Intermediate' from dual union all
select 10629,'010629','1977-065EY','deb Delta 132','',42193,'In Earth orbit','','1993-08-06 00:00:00',113.26,718,2035,29.83,'Intermediate' from dual union all
select 1063,'001063','1964-070CD','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',88.33,180,204,51.1,'Intermediate' from dual union all
select 10630,'010630','1977-065EZ','deb Delta 132','',42193,'In Earth orbit','','1993-08-06 00:00:00',114.85,766,2132,29.25,'Intermediate' from dual union all
select 10631,'010631','1977-065FA','deb Delta 132','',42193,'Reentered','1980-10-08 00:00:00','1980-10-04 00:00:00',92.75,354,464,29.01,'Intermediate' from dual union all
select 10632,'010632','1977-065FB','deb Delta 132','',42193,'Reentered','1989-02-03 00:00:00','1989-02-01 00:00:00',93.83,395,529,31.33,'Intermediate' from dual union all
select 10633,'010633','1977-065FC','deb Delta 132','',42193,'Reentered','1990-01-03 00:00:00','1991-01-21 00:00:00',89.58,245,263,28.84,'Intermediate' from dual union all
select 10634,'010634','1977-065FD','deb Delta 132','',42193,'Reentered','2013-08-10 00:00:00','1993-08-07 00:00:00',102.51,440,1315,28.9,'Intermediate' from dual union all
select 10635,'010635','1977-065FE','deb Delta 132','',42193,'Reentered','1980-09-30 00:00:00','1980-09-19 00:00:00',92.04,298,452,30.06,'Intermediate' from dual union all
select 10636,'010636','1977-065FF','deb Delta 132','',42193,'Reentered','1983-05-20 00:00:00','1978-01-27 00:00:00',110.25,498,1979,28.93,'Intermediate' from dual union all
select 10637,'010637','1978-012A','IUE','IUE',45207,'In Earth orbit','','1978-03-03 00:00:00',1436.24,25687,45892,28.59,'Synchronous' from dual union all
select 10638,'010638','1978-012B','Delta 138','Delta SSPU DSV-3P-4',45207,'Reentered','1978-02-21 00:00:00','1978-02-03 00:00:00',95.11,161,887,28.73,'Intermediate' from dual union all
select 10639,'010639','1978-013A','Kosmos-987','Zenit-4MKM',45208,'Landed','1978-02-14 00:00:00','1978-02-03 00:00:00',89.35,172,313,62.8,'Intermediate' from dual union all
select 1064,'001064','1964-070CE','deb Kosmos-50','',10356,'Reentered','1964-11-15 00:00:00','1964-11-05 00:00:00',88.11,159,203,51.1,'Intermediate' from dual union all
select 10640,'010640','1978-013B','11S510','11S510',45208,'Reentered','1978-02-04 00:00:00','1978-02-02 00:00:00',89.17,167,300,62.8,'Intermediate' from dual union all
select 10641,'010641','1978-013C','Sensor cover','Zenit-4MK sensor cover',45208,'Reentered','1978-02-05 00:00:00','1978-01-31 00:00:00',88.97,183,265,62.8,'Intermediate' from dual union all
select 10642,'010642','1976-077C','deb Delta 126','',39029,'In Earth orbit','','1993-07-08 00:00:00',117.87,1503,1668,102.54,'Sun-Synch' from dual union all
select 10643,'010643','1976-077D','deb Delta 126','',39029,'In Earth orbit','','1993-07-26 00:00:00',119.03,1501,1774,102.14,'Sun-Synch' from dual union all
select 10644,'010644','1976-077E','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',114.23,1329,1511,101.34,'Sun-Synch' from dual union all
select 10645,'010645','1976-077F','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',120.12,1505,1868,102.42,'Medium' from dual union all
select 10646,'010646','1976-077G','deb Delta 126','',39029,'In Earth orbit','','1993-08-02 00:00:00',118.61,1504,1732,102.78,'Sun-Synch' from dual union all
select 10647,'010647','1976-077H','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',118.95,1502,1766,102.66,'Sun-Synch' from dual union all
select 10648,'010648','1976-077J','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',118.19,1500,1700,101.98,'Sun-Synch' from dual union all
select 10649,'010649','1976-077K','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',122.07,1465,2082,102.3,'Medium' from dual union all
select 1065,'001065','1964-070CF','deb Kosmos-50','',10356,'Reentered','1964-11-17 00:00:00','1964-11-05 00:00:00',88.37,183,205,51.3,'Intermediate' from dual union all
select 10650,'010650','1976-077L','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',124.49,1520,2242,102.07,'Medium' from dual union all
select 10651,'010651','1976-077M','deb Delta 126','',39029,'Reentered','1989-11-15 00:00:00','1989-11-14 00:00:00',93.87,412,516,102.12,'Retrograde' from dual union all
select 10652,'010652','1976-077N','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',120.55,1491,1920,102.99,'Medium' from dual union all
select 10653,'010653','1976-077P','deb Delta 126','',39029,'In Earth orbit','','1993-07-29 00:00:00',117.67,1474,1678,102.86,'Sun-Synch' from dual union all
select 10654,'010654','1976-077Q','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',111.92,1145,1484,102.52,'Sun-Synch' from dual union all
select 10655,'010655','1976-077R','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',118.93,1412,1854,102.64,'Sun-Synch' from dual union all
select 10656,'010656','1976-077S','deb Delta 126','',39029,'In Earth orbit','','1993-07-31 00:00:00',116.46,1460,1583,101.85,'Sun-Synch' from dual union all
select 10657,'010657','1976-077T','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',117.49,1502,1635,102.49,'Sun-Synch' from dual union all
select 10658,'010658','1976-077U','deb Delta 126','',39029,'In Earth orbit','','1993-07-29 00:00:00',117.08,1505,1595,102.3,'Sun-Synch' from dual union all
select 10659,'010659','1976-077V','deb Delta 126','',39029,'In Earth orbit','','1993-08-03 00:00:00',119.14,1500,1785,102.16,'Sun-Synch' from dual union all
select 1066,'001066','1964-070CG','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',88.49,188,211,51.4,'Intermediate' from dual union all
select 10660,'010660','1976-077W','deb Delta 126','',39029,'In Earth orbit','','1993-08-05 00:00:00',123.38,1504,2159,102.25,'Medium' from dual union all
select 10661,'010661','1976-077X','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',115.16,1415,1511,101.86,'Sun-Synch' from dual union all
select 10662,'010662','1976-077Y','deb Delta 126','',39029,'In Earth orbit','','1993-07-20 00:00:00',112.22,1181,1477,100.84,'Sun-Synch' from dual union all
select 10663,'010663','1976-077Z','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',108.98,1009,1351,100.68,'Sun-Synch' from dual union all
select 10664,'010664','1978-014A','Kyokko','EXOS-A',45209,'In Earth orbit','','1992-05-23 00:00:00',134.03,642,3953,65.36,'Medium' from dual union all
select 10665,'010665','1978-014B','M-3A','M-3A',45209,'Reentered','1979-02-12 00:00:00','1978-03-11 00:00:00',94.06,330,616,65.09,'Intermediate' from dual union all
select 10666,'010666','1978-015A','Kosmos-988','Orion',45210,'Landed','1978-02-20 00:00:00','1978-02-10 00:00:00',89.83,199,334,72.84,'Intermediate' from dual union all
select 10667,'010667','1978-015B','11S510','11S510',45210,'Reentered','1978-02-17 00:00:00','1978-02-11 00:00:00',89.44,196,298,72.84,'Intermediate' from dual union all
select 10668,'010668','1978-015C','Sensor cover','Zenit-4MT sensor cover',45210,'Reentered','1978-03-05 00:00:00','1978-02-09 00:00:00',89.11,124,336,72.86,'Intermediate' from dual union all
select 10669,'010669','1978-016A','FLTSATCOM F1','FLTSATCOM F1',45211,'In Earth orbit','','1978-03-12 00:00:00',1426.11,35488,35694,2.74,'Drift GEO' from dual union all
select 1067,'001067','1964-070CH','deb Kosmos-50','',10356,'Reentered','1964-11-17 00:00:00','1964-11-05 00:00:00',88.52,191,212,51.3,'Intermediate' from dual union all
select 10670,'010670','1978-013D','Blenda','Sunshade',45208,'Reentered','1978-02-17 00:00:00','1978-02-13 00:00:00',88.64,151,264,62.8,'Intermediate' from dual union all
select 10671,'010671','1978-013E','KDU','KDU',45208,'Reentered','1978-02-20 00:00:00','1978-02-13 00:00:00',88.4,150,241,62.8,'Intermediate' from dual union all
select 10672,'010672','1978-017A','Kosmos-989','Zenit-4MKM',45212,'Landed','1978-02-28 00:00:00','1978-02-16 00:00:00',89.46,169,327,65.05,'Intermediate' from dual union all
select 10673,'010673','1978-017B','11S510','11S510',45212,'Reentered','1978-02-17 00:00:00','1978-02-17 00:00:00',88.34,149,236,65.06,'Intermediate' from dual union all
select 10674,'010674','1978-018A','Ume-2','ISS-b',45213,'In Earth orbit','','1990-06-27 00:00:00',107.19,974,1220,69.36,'Intermediate' from dual union all
select 10675,'010675','1978-018B','Star 37N S/N 14005','Star 37N',45213,'In Earth orbit','','1990-06-19 00:00:00',107.14,975,1214,69.36,'Intermediate' from dual union all
select 10676,'010676','1978-019A','Kosmos-990','Strela-2M',45214,'In Earth orbit','','1992-06-17 00:00:00',100.45,768,791,74.04,'Intermediate' from dual union all
select 10677,'010677','1978-019B','S3M','S3M',45214,'In Earth orbit','','1991-03-27 00:00:00',100.31,761,785,74.03,'Intermediate' from dual union all
select 10678,'010678','1978-015D','KBE part','KBE part',45210,'Reentered','1978-02-22 00:00:00','1978-02-21 00:00:00',88.79,176,254,72.83,'Intermediate' from dual union all
select 10679,'010679','1978-015E','KBE part','KBE part',45210,'Reentered','1978-03-02 00:00:00','1978-02-20 00:00:00',89.24,203,271,72.8,'Intermediate' from dual union all
select 1068,'001068','1964-070CJ','deb Kosmos-50','',10356,'Reentered','1964-11-14 00:00:00','1964-11-05 00:00:00',88.44,177,218,51.1,'Intermediate' from dual union all
select 10680,'010680','1978-015F','KB electropitaniya','KBE',45210,'Reentered','1978-03-11 00:00:00','1978-02-20 00:00:00',90.53,198,402,72.85,'Intermediate' from dual union all
select 10681,'010681','1978-015G','KBE part','KBE part',45210,'Reentered','1978-02-24 00:00:00','1978-02-21 00:00:00',91.1,213,444,72.84,'Intermediate' from dual union all
select 10682,'010682','1977-111D','Kosmos-965 SS 2','ESO',42239,'Reentered','1978-04-17 00:00:00','1978-02-22 00:00:00',94.28,461,506,74.08,'Intermediate' from dual union all
select 10683,'010683','1977-111E','Kosmos-965 SS 3','ESO',42239,'Reentered','1978-05-01 00:00:00','1978-02-22 00:00:00',94.34,463,510,73.97,'Intermediate' from dual union all
select 10684,'010684','1978-020A','Navstar 1','Navstar GPS SVN 1',45215,'In Earth orbit','','1978-04-11 00:00:00',718,20075,20291,63.31,'Medium' from dual union all
select 10685,'010685','1977-097M','Garbage bag','',42225,'Reentered','1978-03-06 00:00:00','1978-02-24 00:00:00',90.97,310,334,51.63,'Intermediate' from dual union all
select 10686,'010686','1977-097N','Garbage bag','',42225,'Reentered','1978-03-06 00:00:00','1978-02-24 00:00:00',91.13,318,342,51.63,'Intermediate' from dual union all
select 10687,'010687','1977-097P','Garbage bag','',42225,'Reentered','1978-03-06 00:00:00','1978-02-24 00:00:00',91.16,319,344,51.62,'Intermediate' from dual union all
select 10688,'010688','1978-021A','OPS 6031','JUMPSEAT 5 (or QUASAR 3)',45216,'In Earth orbit','','1978-03-01 00:00:00',613.49,805,34272,63.5,'GEO Transfer' from dual union all
select 10689,'010689','1978-021B','Agena D','Agena D',45216,'In Earth orbit','','1978-03-01 00:00:00',613.49,805,34272,63.5,'GEO Transfer' from dual union all
select 1069,'001069','1964-070CK','deb Kosmos-50','',10356,'Reentered','1964-11-16 00:00:00','1964-11-05 00:00:00',87.91,142,200,51.1,'Intermediate' from dual union all
select 10690,'010690','1978-017C','KDU','KDU',45212,'Reentered','1978-03-01 00:00:00','1978-02-27 00:00:00',88.72,160,263,65.03,'Intermediate' from dual union all
select 10691,'010691','1978-017D','Blenda','Sunshade',45212,'Reentered','1978-02-28 00:00:00','1978-02-28 00:00:00',88.48,169,230,65,'Intermediate' from dual union all
select 10692,'010692','1978-022A','Kosmos-991','Parus',45217,'In Earth orbit','','1992-06-16 00:00:00',104.67,951,1007,82.99,'Intermediate' from dual union all
select 10693,'010693','1978-022B','S3M','S3M',45217,'In Earth orbit','','1991-03-25 00:00:00',104.6,961,991,82.99,'Intermediate' from dual union all
select 10694,'010694','1978-023A','Soyuz-28','Soyuz 7K-T No. 45',45218,'Landed','1978-03-10 00:00:00','1978-03-03 00:00:00',91.39,334,352,51.62,'Intermediate' from dual union all
select 10695,'010695','1978-023B','11S510','11S510',45218,'Reentered','1978-03-05 00:00:00','1978-03-04 00:00:00',88.37,176,212,51.62,'Intermediate' from dual union all
select 10696,'010696','1978-024A','Molniya-1','Molniya-1',45219,'Reentered','1992-03-09 00:00:00','1978-04-06 00:00:00',717.78,608,39747,62.91,'Molniya' from dual union all
select 10697,'010697','1978-024B','11S510','11S510',45219,'Reentered','1978-04-10 00:00:00','1978-03-12 00:00:00',92.22,207,560,62.83,'Intermediate' from dual union all
select 10698,'010698','1978-024C','BOZ','BOZ',45219,'Reentered','1978-04-09 00:00:00','1978-03-09 00:00:00',92.19,216,548,62.76,'Intermediate' from dual union all
select 10699,'010699','1978-025A','Kosmos-992','Zenit-2M',45220,'Landed','1978-03-17 00:00:00','1978-03-06 00:00:00',89.7,201,318,71.33,'Intermediate' from dual union all
select 107,'000107','61 Nu 1','Explorer 11','NASA S-15',7271,'In Earth orbit','','1961-05-29 00:00:00',108.06,497,1777,28.76,'Intermediate' from dual union all
select 1070,'001070','1964-070CL','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',87.62,115,199,51.1,'Intermediate' from dual union all
select 10700,'010700','1978-025B','11S510','11S510',45220,'Reentered','1978-03-10 00:00:00','1978-03-07 00:00:00',89.17,188,279,71.33,'Intermediate' from dual union all
select 10701,'010701','1978-025C','Sensor cover','Zenit-2M sensor cover',45220,'Reentered','1978-03-05 00:00:00','1978-03-04 00:00:00',89.6,199,311,71.35,'Intermediate' from dual union all
select 10702,'010702','1978-026A','Landsat 3','Landsat C',45221,'In Earth orbit','','1990-05-15 00:00:00',103.16,898,919,98.76,'Sun-Synch' from dual union all
select 10703,'010703','1978-026B','AMSAT-OSCAR-8','OSCAR 8',45221,'In Earth orbit','','1991-05-12 00:00:00',103.03,898,906,99.08,'Sun-Synch' from dual union all
select 10704,'010704','1978-026C','Delta 139','Delta SSPU DSV-3P-4',45221,'In Earth orbit','','1991-03-11 00:00:00',100.27,668,875,98.63,'Sun-Synch' from dual union all
select 10705,'010705','1976-077AA','deb Delta 126','',39029,'Reentered','2010-03-02 00:00:00','1978-03-08 00:00:00',115.17,1416,1511,101.69,'Sun-Synch' from dual union all
select 10706,'010706','1976-077AB','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',113.08,978,1757,101.54,'Sun-Synch' from dual union all
select 10707,'010707','1976-077AC','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',120.65,1485,1935,102.21,'Medium' from dual union all
select 10708,'010708','1976-077AD','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',114.68,1312,1569,101.38,'Sun-Synch' from dual union all
select 10709,'010709','1976-077AE','deb Delta 126','',39029,'In Earth orbit','','1993-08-02 00:00:00',115.76,1475,1505,101.92,'Sun-Synch' from dual union all
select 1071,'001071','1964-070CM','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.62,171,242,51.3,'Intermediate' from dual union all
select 10710,'010710','1976-077AF','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',120.66,1493,1928,101.76,'Medium' from dual union all
select 10711,'010711','1976-077AG','deb Delta 126','',39029,'In Earth orbit','','1978-03-07 00:00:00',117.16,1503,1604,102.26,'Sun-Synch' from dual union all
select 10712,'010712','1976-077AH','deb Delta 126','',39029,'In Earth orbit','','1993-08-01 00:00:00',116.69,1467,1596,102.22,'Sun-Synch' from dual union all
select 10713,'010713','1976-077AJ','deb Delta 126','',39029,'In Earth orbit','','1978-03-08 00:00:00',114.9,1399,1503,101.52,'Sun-Synch' from dual union all
select 10714,'010714','1976-077AK','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',125.86,1438,2445,103.24,'Medium' from dual union all
select 10715,'010715','1976-077AL','deb Delta 126','',39029,'In Earth orbit','','1993-08-03 00:00:00',123.8,1486,2215,102.49,'Medium' from dual union all
select 10716,'010716','1976-077AM','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',116.13,1295,1719,102.82,'Sun-Synch' from dual union all
select 10717,'010717','1976-077AN','deb Delta 126','',39029,'In Earth orbit','','1993-07-29 00:00:00',117.75,1500,1660,102.53,'Sun-Synch' from dual;
insert into SATELLITE
select 10718,'010718','1976-077AP','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',116,1459,1543,101.62,'Sun-Synch' from dual union all
select 10719,'010719','1976-077AQ','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',120.41,1489,1910,101.93,'Medium' from dual union all
select 1072,'001072','1964-070CN','deb Kosmos-50','',10356,'Reentered','1964-11-10 00:00:00','1964-11-05 00:00:00',88.09,159,201,51.1,'Intermediate' from dual union all
select 10720,'010720','1976-077AR','deb Delta 126','',39029,'In Earth orbit','','1993-07-30 00:00:00',119.51,1370,1948,102.11,'Sun-Synch' from dual union all
select 10721,'010721','1976-077AS','deb Delta 126','',39029,'In Earth orbit','','1993-08-03 00:00:00',114.84,1413,1484,101.8,'Sun-Synch' from dual union all
select 10722,'010722','1978-002B','Centaur AC-46','Centaur D-1AR AC-46',45197,'In Earth orbit','','1978-03-16 00:00:00',650.97,624,36382,21.29,'GEO Transfer' from dual union all
select 10723,'010723','1978-012C','Star 37E S/N 40062','Star 37E',45207,'In Earth orbit','','1978-10-21 00:00:00',821.49,417,44938,29.38,'Highly Ellip' from dual union all
select 10724,'010724','1978-016B','deb Centaur AC-44','',45211,'Reentered','1982-08-31 00:00:00','1978-03-15 00:00:00',494.38,160,28506,26.86,'Highly Ellip' from dual union all
select 10725,'010725','1978-027A','Kosmos-993','Zenit-4MKM',45222,'Landed','1978-03-23 00:00:00','1978-03-14 00:00:00',89.48,168,330,72.85,'Intermediate' from dual union all
select 10726,'010726','1978-027B','11S510','11S510',45222,'Reentered','1978-03-14 00:00:00','1978-03-12 00:00:00',88.67,158,260,72.85,'Intermediate' from dual union all
select 10727,'010727','1978-027C','Sensor cover','Zenit-4MK sensor cover',45222,'Reentered','1978-03-12 00:00:00','1978-03-11 00:00:00',89.26,167,308,72.85,'Intermediate' from dual union all
select 10728,'010728','1975-027C','deb Delta 109','',36474,'In Earth orbit','','1993-08-17 00:00:00',101.42,777,875,115.21,'Retrograde' from dual union all
select 10729,'010729','1975-027D','deb Delta 109','',36474,'Reentered','1983-12-07 00:00:00','1978-07-27 00:00:00',101.55,806,859,114.97,'Retrograde' from dual union all
select 1073,'001073','1964-070CP','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.86,190,247,51.1,'Intermediate' from dual union all
select 10730,'010730','1975-027E','deb Delta 109','',36474,'In Earth orbit','','1993-08-16 00:00:00',103.53,857,993,114.95,'Retrograde' from dual union all
select 10731,'010731','1978-028A','Kosmos-994','Zaliv',45223,'In Earth orbit','','1992-05-27 00:00:00',104.9,974,1006,82.93,'Intermediate' from dual union all
select 10732,'010732','1978-028B','S3M','S3M',45223,'In Earth orbit','','1991-03-25 00:00:00',104.76,969,998,82.93,'Intermediate' from dual union all
select 10733,'010733','1978-029A','OPS 0460','HEXAGON SV-14',45224,'Deorbited','1978-09-11 00:00:00','1978-04-15 00:00:00',88.57,160,247,96.4,'Sun-Synch' from dual union all
select 10734,'010734','1978-029B','RAQUEL IA','P-11 No. 4432',45224,'Reentered','1992-01-01 00:00:00','1979-11-02 00:00:00',97.25,620,635,95.83,'Sun-Synch' from dual union all
select 10735,'010735','1978-030A','Kosmos-995','Zenit-2M',45225,'Landed','1978-03-30 00:00:00','1978-03-19 00:00:00',88.94,214,231,81.34,'Intermediate' from dual union all
select 10736,'010736','1978-030B','11S510','11S510',45225,'Reentered','1978-03-20 00:00:00','1978-03-19 00:00:00',87.95,173,173,81.34,'Intermediate' from dual union all
select 10737,'010737','1978-029C','Titan 23D-20 Stage 2','LR91AJ11 23D-20',45224,'Reentered','1978-03-17 00:00:00','1978-03-16 00:00:00',88.46,166,231,96.42,'Sun-Synch' from dual union all
select 10738,'010738','1977-097Q','Garbage bag','',42225,'Reentered','1978-04-26 00:00:00','1978-03-20 00:00:00',91.24,328,343,51.62,'Intermediate' from dual union all
select 10739,'010739','1978-027D','KDU','KDU',45222,'Reentered','1978-03-24 00:00:00','1978-03-22 00:00:00',88.82,163,270,72.87,'Intermediate' from dual union all
select 1074,'001074','1964-070CQ','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.76,191,236,51.1,'Intermediate' from dual union all
select 10740,'010740','1977-097R','Garbage bag','',42225,'Reentered','1978-06-15 00:00:00','1978-03-22 00:00:00',91.26,329,344,51.63,'Intermediate' from dual union all
select 10741,'010741','1977-111F','Kosmos-965 SS 4','ESO',42239,'Reentered','1978-05-15 00:00:00','1978-03-23 00:00:00',94.2,461,498,74.02,'Intermediate' from dual union all
select 10742,'010742','1977-111G','Kosmos-965 SS 5','ESO',42239,'Reentered','1978-07-18 00:00:00','1978-03-23 00:00:00',94.35,463,512,74.03,'Intermediate' from dual union all
select 10743,'010743','1977-097S','Garbage bag','',42225,'Reentered','1978-06-17 00:00:00','1978-03-25 00:00:00',91.26,328,345,51.62,'Intermediate' from dual union all
select 10744,'010744','1978-031A','Kosmos-996','Parus',45226,'In Earth orbit','','1992-06-06 00:00:00',104.65,952,1004,82.93,'Intermediate' from dual union all
select 10745,'010745','1978-031B','S3M','S3M',45226,'In Earth orbit','','1991-03-23 00:00:00',104.54,947,999,82.93,'Intermediate' from dual union all
select 10746,'010746','1976-077AT','deb Delta 126','',39029,'In Earth orbit','','1977-12-31 00:00:00',113.89,1121,1689,101.71,'Sun-Synch' from dual union all
select 10747,'010747','1976-077AU','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',109.38,1039,1357,101.05,'Sun-Synch' from dual union all
select 10748,'010748','1976-077AV','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',117.16,1495,1612,102.36,'Sun-Synch' from dual union all
select 10749,'010749','1976-077AW','deb Delta 126','',39029,'In Earth orbit','','1993-08-03 00:00:00',117.69,1505,1649,102.37,'Sun-Synch' from dual union all
select 1075,'001075','1964-070CR','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.95,179,267,51.4,'Intermediate' from dual union all
select 10750,'010750','1976-077AX','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',112.35,1017,1653,101.73,'Sun-Synch' from dual union all
select 10751,'010751','1976-077AY','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',125.53,1399,2455,102.15,'Medium' from dual union all
select 10752,'010752','1976-077AZ','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',114.4,1356,1500,101.8,'Sun-Synch' from dual union all
select 10753,'010753','1976-077BA','deb Delta 126','',39029,'Reentered','1982-01-02 00:00:00','1982-01-02 00:00:00',92.92,385,451,101.85,'Retrograde' from dual union all
select 10754,'010754','1976-077BB','deb Delta 126','',39029,'In Earth orbit','','1993-07-30 00:00:00',117.87,1487,1683,102.58,'Sun-Synch' from dual union all
select 10755,'010755','1976-077BC','deb Delta 126','',39029,'In Earth orbit','','1993-08-01 00:00:00',117.47,1503,1632,102.51,'Sun-Synch' from dual union all
select 10756,'010756','1976-077BD','deb Delta 126','',39029,'In Earth orbit','','1977-12-31 00:00:00',118.36,1320,1895,101.06,'Sun-Synch' from dual union all
select 10757,'010757','1976-077BE','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',120.28,1494,1893,101.94,'Medium' from dual union all
select 10758,'010758','1976-077BF','deb Delta 126','',39029,'In Earth orbit','','1993-07-26 00:00:00',118.81,1506,1749,102.77,'Sun-Synch' from dual union all
select 10759,'010759','1976-077BG','deb Delta 126','',39029,'In Earth orbit','','1978-03-28 00:00:00',117.16,1491,1616,102.48,'Sun-Synch' from dual union all
select 1076,'001076','1964-070CS','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.63,192,222,51.2,'Intermediate' from dual union all
select 10760,'010760','1976-077BH','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',118.14,1321,1874,101.49,'Sun-Synch' from dual union all
select 10761,'010761','1976-077BJ','deb Delta 126','',39029,'In Earth orbit','','1978-03-28 00:00:00',120.32,1501,1890,100.78,'Medium' from dual union all
select 10762,'010762','1976-077BK','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',126.41,1288,2643,102.95,'Medium' from dual union all
select 10763,'010763','1976-077BL','deb Delta 126','',39029,'In Earth orbit','','1993-08-04 00:00:00',116.74,1497,1572,102.09,'Sun-Synch' from dual union all
select 10764,'010764','1976-077BM','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',119.6,1480,1846,102.19,'Sun-Synch' from dual union all
select 10765,'010765','1976-077BN','deb Delta 126','',39029,'In Earth orbit','','1993-08-04 00:00:00',118.36,1503,1711,102.08,'Sun-Synch' from dual union all
select 10766,'010766','1976-077BP','deb Delta 126','',39029,'In Earth orbit','','1993-08-02 00:00:00',116.96,1502,1587,102.22,'Sun-Synch' from dual union all
select 10767,'010767','1976-077BQ','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',116.95,1492,1596,102.29,'Sun-Synch' from dual union all
select 10768,'010768','1976-077BR','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',117.33,1503,1619,102.01,'Sun-Synch' from dual union all
select 10769,'010769','1976-077BS','deb Delta 126','',39029,'In Earth orbit','','1993-08-04 00:00:00',115.01,1381,1530,101.67,'Sun-Synch' from dual union all
select 1077,'001077','1964-070CT','deb Kosmos-50','',10356,'Reentered','1964-11-14 00:00:00','1964-11-05 00:00:00',89.62,192,320,51.2,'Intermediate' from dual union all
select 10770,'010770','1978-032A','Kosmos-997','TKS VA No. 009A/P2',45227,'Landed','1978-03-30 00:00:00','1978-03-30 00:00:00',88.5,188,213,51.6,'Intermediate' from dual union all
select 10771,'010771','1978-032B','Kosmos-998','TKS VA No. 009P/2',45227,'Landed','1978-03-30 00:00:00','1978-03-30 00:00:00',88.5,188,213,51.6,'Intermediate' from dual union all
select 10772,'010772','1978-032C','8S812','8S812',45227,'Reentered','1978-04-02 00:00:00','1978-04-02 00:00:00',87.3,141,141,51.6,'Intermediate' from dual union all
select 10773,'010773','1978-033A','Kosmos-999','Zenit-4MKM',45228,'Landed','1978-04-12 00:00:00','1978-04-02 00:00:00',89.69,172,347,71.39,'Intermediate' from dual union all
select 10774,'010774','1978-033B','11S510','11S510',45228,'Reentered','1978-04-03 00:00:00','1978-04-02 00:00:00',88.56,160,247,71.39,'Intermediate' from dual union all
select 10775,'010775','1978-032D','82LB72 LVI-3','TKS GVM',45227,'Reentered','1978-04-03 00:00:00','1978-04-01 00:00:00',88.3,187,194,51.59,'Intermediate' from dual union all
select 10776,'010776','1978-034A','Kosmos-1000','Tsikada',45229,'In Earth orbit','','1978-05-08 00:00:00',104.83,962,1011,82.95,'Intermediate' from dual union all
select 10777,'010777','1978-034B','S3M','S3M',45229,'In Earth orbit','','1991-03-28 00:00:00',104.61,956,997,82.94,'Intermediate' from dual union all
select 10778,'010778','1978-035A','INTELSAT IVA F6','INTELSAT IVA F6',45230,'In Earth orbit','','1978-05-25 00:00:00',1446.85,35980,36014,.27,'Drift GEO' from dual union all
select 10779,'010779','1978-035B','Centaur AC-48','Centaur D-1AR AC-48',45230,'In Earth orbit','','1978-05-07 00:00:00',649.15,614,36299,21.91,'GEO Transfer' from dual union all
select 1078,'001078','1964-070CU','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',87.9,144,198,51.4,'Intermediate' from dual union all
select 10780,'010780','1977-111H','Kosmos-965 SS 6','ESO',42239,'Reentered','1978-05-24 00:00:00','1978-04-01 00:00:00',94.22,459,502,74.03,'Intermediate' from dual union all
select 10781,'010781','1977-097T','Garbage bag','',42225,'Reentered','1978-05-29 00:00:00','1978-03-26 00:00:00',91.13,327,333,51.63,'Intermediate' from dual union all
select 10782,'010782','1977-111J','Kosmos-965 SS 7','ESO',42239,'Reentered','1978-07-18 00:00:00','1978-04-01 00:00:00',94.24,459,504,74.04,'Intermediate' from dual union all
select 10783,'010783','1978-036A','Kosmos-1001','Soyuz 7K-ST No. 4L',45231,'Landed','1978-04-15 00:00:00','1978-04-06 00:00:00',89.33,196,287,51.63,'Intermediate' from dual union all
select 10784,'010784','1978-036B','11S510','11S510',45231,'Reentered','1978-04-06 00:00:00','1978-04-05 00:00:00',88.03,167,187,51.63,'Intermediate' from dual union all
select 10785,'010785','1978-037A','Kosmos-1002','Zenit-2M',45232,'Landed','1978-04-19 00:00:00','1978-04-08 00:00:00',89.35,204,281,65.05,'Intermediate' from dual union all
select 10786,'010786','1978-037B','11S510','11S510',45232,'Reentered','1978-04-10 00:00:00','1978-04-09 00:00:00',88.58,175,234,65.05,'Intermediate' from dual union all
select 10787,'010787','1978-038A','OPS 8790','AQUACADE 4',45233,'In Earth orbit','','1978-04-08 00:00:00',1436.01,35780,35790,0,'Stationary' from dual union all
select 10788,'010788','1978-038B','Agena D 1554','Agena D',45233,'Reentered','1980-01-01 00:00:00','1978-04-08 00:00:00',615.53,150,35033,28.4,'GEO Transfer' from dual union all
select 10789,'010789','1977-111K','Kosmos-965 SS 8','ESO',42239,'Reentered','1978-06-06 00:00:00','1978-04-07 00:00:00',94.21,460,500,73.95,'Intermediate' from dual union all
select 1079,'001079','1964-070CV','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.56,194,213,51.3,'Intermediate' from dual union all
select 10790,'010790','1978-037C','Sensor cover','Zenit-2M sensor cover',45232,'Reentered','1978-04-24 00:00:00','1978-04-07 00:00:00',88.87,192,246,65.04,'Intermediate' from dual union all
select 10791,'010791','1978-037D','Sensor cover','Zenit-2M sensor cover',45232,'Reentered','1978-04-24 00:00:00','1978-04-07 00:00:00',88.97,185,263,65.07,'Intermediate' from dual union all
select 10792,'010792','1978-039A','Yuri','BSE',45234,'In Earth orbit','','1978-05-07 00:00:00',1436.09,35766,35807,.04,'Stationary' from dual union all
select 10793,'010793','1978-039B','Delta 140','Delta SSPU DSV-3P-4',45234,'In Earth orbit','','1990-02-19 00:00:00',110.98,569,1975,28.25,'Intermediate' from dual union all
select 10794,'010794','1978-039C','Star 37E S/N 40057','Star 37E',45234,'Reentered','1998-09-04 00:00:00','1992-09-14 00:00:00',173.83,219,7659,27.02,'Medium' from dual union all
select 10795,'010795','1977-111L','Kosmos-965 SS 9','ESO',42239,'Reentered','1978-06-01 00:00:00','1978-04-09 00:00:00',94.21,458,502,74.11,'Intermediate' from dual union all
select 10796,'010796','61 A Eta 4','deb Transit IVB','',7253,'In Earth orbit','','1993-08-01 00:00:00',105.69,952,1102,32.44,'Intermediate' from dual union all
select 10797,'010797','1977-111M','Kosmos-965 SS 10','ESO',42239,'Reentered','1978-07-18 00:00:00','1978-04-09 00:00:00',94.24,456,508,74.11,'Intermediate' from dual union all
select 10798,'010798','1978-033C','KDU part','KDU part',45228,'Reentered','1978-04-13 00:00:00','1978-04-12 00:00:00',88.91,174,267,71.39,'Intermediate' from dual union all
select 10799,'010799','1978-033D','Blenda','Sunshade',45228,'Reentered','1978-04-16 00:00:00','1978-04-11 00:00:00',89.42,177,314,71.4,'Intermediate' from dual union all
select 108,'000108','61 Xi 1','Discoverer 25','CORONA C''-5',7282,'Reentered Attached','1961-07-12 00:00:00','1961-06-24 00:00:00',90.36,223,361,82.11,'Intermediate' from dual union all
select 1080,'001080','1964-070CW','deb Kosmos-50','',10356,'Reentered','1964-11-17 00:00:00','1964-11-05 00:00:00',90.09,188,369,51.2,'Intermediate' from dual union all
select 10800,'010800','1978-033E','KDU','KDU',45228,'Reentered','1978-04-24 00:00:00','1978-04-12 00:00:00',89.36,189,297,71.3,'Intermediate' from dual union all
select 10801,'010801','1978-020B','SVS Stage 2','Star 37E S/N 40056',45215,'Reentered','2012-08-11 00:00:00','1978-04-06 00:00:00',330.28,166,18732,63.14,'Highly Ellip' from dual union all
select 10802,'010802','1978-009E','Blok-L','Blok-L',45204,'Reentered','1990-08-09 00:00:00','1978-04-18 00:00:00',732.67,700,40387,62.87,'Molniya' from dual union all
select 10803,'010803','1978-024D','Blok-L','Blok-L',45219,'Reentered','1992-12-11 00:00:00','1992-10-05 00:00:00',713.61,127,40022,62.01,'Molniya' from dual union all
select 10804,'010804','1978-036C','Kosmos-1001 BO','Bitovoy otsek',45231,'Reentered','1978-06-01 00:00:00','1978-04-14 00:00:00',90.85,312,321,51.61,'Intermediate' from dual union all
select 10805,'010805','1978-036D','Kosmos-1001 part','Soyuz part',45231,'Reentered','1978-05-02 00:00:00','1978-04-15 00:00:00',90.76,310,314,51.6,'Intermediate' from dual union all
select 10806,'010806','1978-036E','Kosmos-1001 part','Soyuz part',45231,'Reentered','1978-04-25 00:00:00','1978-04-15 00:00:00',90.57,297,308,51.62,'Intermediate' from dual union all
select 10807,'010807','1977-111N','Kosmos-965 SS 11','ESO',42239,'Reentered','1978-05-23 00:00:00','1978-04-16 00:00:00',93.53,432,462,74.03,'Intermediate' from dual;
insert into SATELLITE
select 10808,'010808','1977-051D','deb S1M','',42179,'Reentered','1978-07-18 00:00:00','1978-04-18 00:00:00',88.78,203,226,71,'Intermediate' from dual union all
select 10809,'010809','1977-111P','Kosmos-965 SS 12','ESO',42239,'Reentered','1978-06-20 00:00:00','1978-04-16 00:00:00',93.51,431,462,74.03,'Intermediate' from dual union all
select 1081,'001081','1964-070CX','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',88.88,193,246,51.3,'Intermediate' from dual union all
select 10810,'010810','1978-036F','Kosmos-1001 part','Soyuz part',45231,'Reentered','1978-05-23 00:00:00','1978-04-15 00:00:00',90.64,297,315,51.62,'Intermediate' from dual union all
select 10811,'010811','1978-040A','Kosmos-1003','Zenit-4MKM',45235,'Landed','1978-05-04 00:00:00','1978-04-22 00:00:00',88.97,163,285,62.81,'Intermediate' from dual union all
select 10812,'010812','1978-040B','11S510','11S510',45235,'Reentered','1978-04-23 00:00:00','1978-04-22 00:00:00',88.72,157,266,62.79,'Intermediate' from dual union all
select 10813,'010813','1978-040C','Sensor cover','Zenit-4MK sensor cover',45235,'Reentered','1978-04-22 00:00:00','1978-04-20 00:00:00',89.03,171,283,62.78,'Intermediate' from dual union all
select 10814,'010814','1977-111Q','Kosmos-965 SS 13','ESO',42239,'Reentered','1978-06-18 00:00:00','1978-04-22 00:00:00',94.07,459,487,74.02,'Intermediate' from dual union all
select 10815,'010815','1977-111R','Kosmos-965 SS 14','ESO',42239,'Reentered','1978-07-09 00:00:00','1978-04-23 00:00:00',94.37,459,518,74.03,'Intermediate' from dual union all
select 10816,'010816','1977-111S','Kosmos-965 SS 15','ESO',42239,'Reentered','1978-07-06 00:00:00','1978-04-23 00:00:00',94.38,458,520,74.06,'Intermediate' from dual union all
select 10817,'010817','1977-111T','Kosmos-965 SS 16','ESO',42239,'Reentered','1978-06-20 00:00:00','1978-04-23 00:00:00',94.04,456,489,74.01,'Intermediate' from dual union all
select 10818,'010818','1978-041A','HCMM','AEM-A',45236,'Reentered','1981-12-22 00:00:00','1978-06-10 00:00:00',97.13,610,632,97.6,'Sun-Synch' from dual union all
select 10819,'010819','1978-041B','Star 20 E-23','Star 20 E-23',45236,'Reentered','1980-11-24 00:00:00','1978-06-24 00:00:00',96.77,558,650,97.59,'Sun-Synch' from dual union all
select 1082,'001082','1964-070CY','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',88.84,178,257,51.1,'Intermediate' from dual union all
select 10820,'010820','1978-042A','DMSP F-3','DMSP Block 5D S-3',45237,'In Earth orbit','','1990-02-11 00:00:00',100.95,797,811,98.64,'Sun-Synch' from dual union all
select 10821,'010821','1978-040D','KDU','KDU',45235,'Reentered','1978-05-04 00:00:00','1978-05-03 00:00:00',88.81,159,273,62.78,'Intermediate' from dual union all
select 10822,'010822','1978-040E','Blenda','Sunshade',45235,'Reentered','1978-05-04 00:00:00','1978-05-03 00:00:00',88.85,162,273,62.86,'Intermediate' from dual union all
select 10823,'010823','1976-077BT','deb Delta 126','',39029,'In Earth orbit','','1977-12-31 00:00:00',115.92,1462,1533,101.71,'Sun-Synch' from dual union all
select 10824,'010824','1976-077BU','deb Delta 126','',39029,'In Earth orbit','','1993-08-01 00:00:00',111.59,1263,1337,102.01,'Sun-Synch' from dual union all
select 10825,'010825','1976-077BV','deb Delta 126','',39029,'In Earth orbit','','1993-08-01 00:00:00',116.65,1496,1565,102.31,'Sun-Synch' from dual union all
select 10826,'010826','1976-077BW','deb Delta 126','',39029,'In Earth orbit','','1978-05-04 00:00:00',119.5,1431,1886,102.25,'Sun-Synch' from dual union all
select 10827,'010827','1976-077BX','deb Delta 126','',39029,'Reentered','1988-09-11 00:00:00','1988-09-08 00:00:00',95.41,513,564,101.04,'Retrograde' from dual union all
select 10828,'010828','1976-077BY','deb Delta 126','',39029,'In Earth orbit','','1993-07-29 00:00:00',117.81,1503,1661,102.6,'Sun-Synch' from dual union all
select 10829,'010829','1976-077BZ','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',116.58,1497,1556,101.87,'Sun-Synch' from dual union all
select 1083,'001083','1964-070CZ','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',89.54,195,309,51.4,'Intermediate' from dual union all
select 10830,'010830','1976-077CA','deb Delta 126','',39029,'In Earth orbit','','1993-07-29 00:00:00',117.25,1487,1628,101.69,'Sun-Synch' from dual union all
select 10831,'010831','1976-077CB','deb Delta 126','',39029,'In Earth orbit','','1993-08-12 00:00:00',116.94,1448,1639,101.86,'Sun-Synch' from dual union all
select 10832,'010832','1976-077CC','deb Delta 126','',39029,'In Earth orbit','','1993-08-03 00:00:00',118.11,1499,1693,102.19,'Sun-Synch' from dual union all
select 10833,'010833','1976-077CD','deb Delta 126','',39029,'In Earth orbit','','1993-08-04 00:00:00',117.04,1452,1644,102.25,'Sun-Synch' from dual union all
select 10834,'010834','1976-077CE','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',119.08,1361,1919,100.6,'Sun-Synch' from dual union all
select 10835,'010835','1976-077CF','deb Delta 126','',39029,'In Earth orbit','','1993-08-03 00:00:00',112.31,1319,1347,102.06,'Sun-Synch' from dual union all
select 10836,'010836','1976-077CG','deb Delta 126','',39029,'In Earth orbit','','1993-08-03 00:00:00',119.33,1505,1797,102.1,'Sun-Synch' from dual union all
select 10837,'010837','1976-077CH','deb Delta 126','',39029,'In Earth orbit','','1993-07-31 00:00:00',119.69,1505,1829,102.3,'Sun-Synch' from dual union all
select 10838,'010838','1976-077CJ','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',116.06,1473,1534,101.62,'Sun-Synch' from dual union all
select 10839,'010839','1976-077CK','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',110.78,1226,1299,102.04,'Sun-Synch' from dual union all
select 1084,'001084','1964-070DA','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',88.07,152,206,51.3,'Intermediate' from dual union all
select 10840,'010840','1976-077CL','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',118.46,1500,1724,102.55,'Sun-Synch' from dual union all
select 10841,'010841','1976-077CM','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',114.96,1399,1509,101.78,'Sun-Synch' from dual union all
select 10842,'010842','1976-077CN','deb Delta 126','',39029,'In Earth orbit','','1993-08-05 00:00:00',114.5,1198,1668,102.08,'Sun-Synch' from dual union all
select 10843,'010843','1976-077CP','deb Delta 126','',39029,'In Earth orbit','','1993-08-01 00:00:00',114.48,1340,1523,101.54,'Sun-Synch' from dual union all
select 10844,'010844','1976-077CQ','deb Delta 126','',39029,'In Earth orbit','','1978-05-04 00:00:00',111.99,1128,1509,101.35,'Sun-Synch' from dual union all
select 10845,'010845','1976-077CR','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',119.11,1500,1782,102.15,'Sun-Synch' from dual union all
select 10846,'010846','1978-043A','Kosmos-1004','Zenit-2M',45238,'Landed','1978-05-18 00:00:00','1978-05-07 00:00:00',89.41,204,287,62.81,'Intermediate' from dual union all
select 10847,'010847','1978-043B','11S510','11S510',45238,'Reentered','1978-05-09 00:00:00','1978-05-08 00:00:00',88.74,189,236,62.79,'Intermediate' from dual union all
select 10848,'010848','1978-043C','Sensor cover','Zenit-2M sensor cover',45238,'Reentered','1978-05-09 00:00:00','1978-05-05 00:00:00',89.93,219,322,62.8,'Intermediate' from dual union all
select 10849,'010849','1978-043D','Sensor cover','Zenit-2M sensor cover',45238,'Reentered','1978-05-09 00:00:00','1978-05-05 00:00:00',89.25,219,256,62.8,'Intermediate' from dual union all
select 1085,'001085','1965-009A','Pegasus 1','Pegasus 1',11759,'Reentered Attached','1978-09-17 00:00:00','1965-03-18 00:00:00',97,500,731,31.75,'Intermediate' from dual union all
select 10850,'010850','1978-043E','Sensor cover','Zenit-2M sensor cover',45238,'Reentered','1978-05-14 00:00:00','1978-05-08 00:00:00',89.27,202,275,62.8,'Intermediate' from dual union all
select 10851,'010851','1978-043F','Kapsula Kosmosa-1004','Nauka 23KS',45238,'Reentered','1978-06-16 00:00:00','1978-05-12 00:00:00',90.68,202,413,62.8,'Intermediate' from dual union all
select 10852,'010852','1978-043G','Nauka part','',45238,'Reentered','1978-06-08 00:00:00','1978-05-12 00:00:00',90.69,202,415,62.8,'Intermediate' from dual union all
select 10853,'010853','1978-042B','DMSP F-3 solar array band','',45237,'Reentered','1991-10-21 00:00:00','1991-10-17 00:00:00',91.49,337,359,98.24,'Sun-Synch' from dual union all
select 10854,'010854','1978-042C','DMSP F-3 solar array band','',45237,'Reentered','1992-03-06 00:00:00','1992-03-06 00:00:00',88,172,179,98.17,'Sun-Synch' from dual union all
select 10855,'010855','1978-044A','OTS 2','OTS 2',45239,'In Earth orbit','','1978-07-19 00:00:00',1436.04,35777,35794,.06,'Stationary' from dual union all
select 10856,'010856','1978-044B','Delta 141','Delta SSPU DSV-3P-4',45239,'In Earth orbit','','1990-02-18 00:00:00',139.83,1568,3524,27.96,'Medium' from dual union all
select 10857,'010857','1978-044C','Star 37E S/N 40058','Star 37E',45239,'Reentered','1991-10-23 00:00:00','1978-08-10 00:00:00',633.12,18046,18046,27.3,'Medium' from dual union all
select 10858,'010858','1977-111U','Kosmos-965 SS 17','ESO',42239,'Reentered','1978-07-30 00:00:00','1978-05-13 00:00:00',94.28,457,511,74.01,'Intermediate' from dual union all
select 10859,'010859','1977-111V','Kosmos-965 SS 18','ESO',42239,'Reentered','1978-07-26 00:00:00','1978-05-13 00:00:00',94.06,452,494,74.06,'Intermediate' from dual union all
select 1086,'001086','1965-010A','Ranger 8','RA-8',11760,'Deep Space','1965-02-20 00:00:00','1965-02-17 00:00:00',25701.21,199,564023,28.8,'Cislunar' from dual union all
select 10860,'010860','1978-045A','Kosmos-1005','Tselina-D',45240,'Reentered','2000-06-15 00:00:00','1991-03-27 00:00:00',95.48,536,548,81.22,'Intermediate' from dual union all
select 10861,'010861','1978-045B','8A92ME','8A92ME',45240,'Reentered','2013-11-15 00:00:00','1991-03-28 00:00:00',96.33,553,613,81.21,'Intermediate' from dual union all
select 10862,'010862','1978-046A','Kosmos-1006','Vektor',45241,'Reentered','1979-03-14 00:00:00','1978-06-12 00:00:00',92.35,376,403,65.84,'Intermediate' from dual union all
select 10863,'010863','1978-046B','S3M','S3M',45241,'Reentered','1978-12-06 00:00:00','1978-07-22 00:00:00',91.96,344,397,65.85,'Intermediate' from dual union all
select 10864,'010864','1976-077CS','deb Delta 126','',39029,'In Earth orbit','','1993-08-05 00:00:00',122.72,1504,2101,101.13,'Medium' from dual union all
select 10865,'010865','1976-077CT','deb Delta 126','',39029,'In Earth orbit','','1993-07-29 00:00:00',118.01,1502,1681,102.6,'Sun-Synch' from dual union all
select 10866,'010866','1976-077CU','deb Delta 126','',39029,'In Earth orbit','','1993-08-05 00:00:00',117.08,1430,1670,102.65,'Sun-Synch' from dual union all
select 10867,'010867','1976-077CV','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',113.9,1223,1588,101.88,'Sun-Synch' from dual union all
select 10868,'010868','1976-077CW','deb Delta 126','',39029,'In Earth orbit','','1993-08-03 00:00:00',117.11,1484,1618,102.26,'Sun-Synch' from dual union all
select 10869,'010869','1976-077CX','deb Delta 126','',39029,'In Earth orbit','','1993-08-03 00:00:00',115.96,1478,1520,102.39,'Sun-Synch' from dual union all
select 1087,'001087','1965-010B','Agena B 6006','Agena B 6006',11760,'Deep Space','1965-02-20 00:00:00','1965-02-17 00:00:00',25701.21,199,564023,28.8,'Cislunar' from dual union all
select 10870,'010870','1976-077CY','deb Delta 126','',39029,'In Earth orbit','','1993-08-04 00:00:00',115.9,1483,1510,101.87,'Sun-Synch' from dual union all
select 10871,'010871','1976-077CZ','deb Delta 126','',39029,'In Earth orbit','','1978-05-11 00:00:00',117.12,1338,1765,102.04,'Sun-Synch' from dual union all
select 10872,'010872','1976-077DA','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',117.54,1495,1646,101.5,'Sun-Synch' from dual union all
select 10873,'010873','1976-077DB','deb Delta 126','',39029,'In Earth orbit','','1993-07-26 00:00:00',118.64,1507,1732,102.04,'Sun-Synch' from dual union all
select 10874,'010874','1976-077DC','deb Delta 126','',39029,'In Earth orbit','','1993-08-08 00:00:00',112.86,1232,1484,102.03,'Sun-Synch' from dual union all
select 10875,'010875','1976-077DD','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',115.53,1452,1507,102.14,'Sun-Synch' from dual union all
select 10876,'010876','1976-077DE','deb Delta 126','',39029,'In Earth orbit','','1993-08-02 00:00:00',115.43,1394,1556,102.63,'Sun-Synch' from dual union all
select 10877,'010877','1977-121Y','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-07 00:00:00',105.02,879,1112,65.76,'Intermediate' from dual union all
select 10878,'010878','1977-121Z','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-02 00:00:00',105.9,941,1132,65.84,'Intermediate' from dual union all
select 10879,'010879','1977-121AA','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-09 00:00:00',105.25,903,1110,65.83,'Intermediate' from dual union all
select 1088,'001088','1965-009B','Apollo BP-16','Apollo BP-16 CM',11759,'Reentered','1985-07-10 00:00:00','1965-03-22 00:00:00',97.06,500,736,31.75,'Intermediate' from dual union all
select 10880,'010880','1977-121AB','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-05 00:00:00',105.88,941,1131,65.83,'Intermediate' from dual union all
select 10881,'010881','1977-121AC','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-03 00:00:00',105.51,909,1128,65.84,'Intermediate' from dual union all
select 10882,'010882','1977-121AD','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-07 00:00:00',105.18,890,1116,65.83,'Intermediate' from dual union all
select 10883,'010883','1977-121AE','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-03 00:00:00',105.36,908,1115,65.84,'Intermediate' from dual union all
select 10884,'010884','1977-121AF','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-06 00:00:00',104.86,831,1145,66.39,'Intermediate' from dual union all
select 10885,'010885','1977-121AG','deb Kosmos-970','',42249,'In Earth orbit','','1993-07-28 00:00:00',106.08,990,1100,65.79,'Intermediate' from dual union all
select 10886,'010886','1977-121AH','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-06 00:00:00',105.46,921,1111,65.85,'Intermediate' from dual union all
select 10887,'010887','1977-121AJ','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-05 00:00:00',104.64,850,1106,65.75,'Intermediate' from dual union all
select 10888,'010888','1977-121AK','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-06 00:00:00',106.03,943,1142,65.51,'Intermediate' from dual union all
select 10889,'010889','1977-121AL','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-05 00:00:00',105.23,896,1114,65.83,'Intermediate' from dual union all
select 1089,'001089','1965-011A','Kosmos-54','Strela-1',11761,'Reentered','1968-09-15 00:00:00','1965-03-23 00:00:00',104.8,260,1711,56.05,'Intermediate' from dual union all
select 10890,'010890','1977-121AM','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-04 00:00:00',105.43,905,1125,65.78,'Intermediate' from dual union all
select 10891,'010891','1977-111W','Kosmos-965 SS 19','ESO',42239,'Reentered','1978-07-19 00:00:00','1978-05-13 00:00:00',94.14,455,499,74.04,'Intermediate' from dual union all
select 10892,'010892','1977-111X','Kosmos-965 SS 20','ESO',42239,'Reentered','1978-07-25 00:00:00','1978-05-13 00:00:00',94.24,452,512,74.06,'Intermediate' from dual union all
select 10893,'010893','1978-047A','Navstar 2','Navstar GPS SVN 2',45242,'In Earth orbit','','1978-06-20 00:00:00',715.67,20056,20195,63.18,'Medium' from dual union all
select 10894,'010894','1978-047B','SVS Stage 2','Star 37E S/N 40064',45242,'In Earth orbit','','1978-06-28 00:00:00',337.34,172,19177,63.11,'Highly Ellip' from dual union all
select 10895,'010895','1978-048A','Kosmos-1007','Zenit-4MKM',45243,'Landed','1978-05-29 00:00:00','1978-05-18 00:00:00',89.71,168,353,72.85,'Intermediate' from dual union all
select 10896,'010896','1978-048B','11S510','11S510',45243,'Reentered','1978-05-20 00:00:00','1978-05-20 00:00:00',88.34,151,234,72.84,'Intermediate' from dual union all
select 10897,'010897','1978-048C','Sensor cover','Zenit-4MK sensor cover',45243,'Reentered','1978-05-18 00:00:00','1978-05-16 00:00:00',88.92,152,290,72.84,'Intermediate' from dual union all
select 10898,'010898','1978-049A','Kosmos-1008','Tselina-OK',45244,'Reentered','1981-01-08 00:00:00','1978-06-17 00:00:00',95.05,495,547,74.05,'Intermediate' from dual;
insert into SATELLITE
select 10899,'010899','1978-049B','S3M','S3M',45244,'Reentered','1981-02-06 00:00:00','1978-07-21 00:00:00',94.89,479,547,74.05,'Intermediate' from dual union all
select 109,'000109','61 Xi 2','deb Discoverer XXV','',7282,'Reentered','1961-06-19 00:00:00','1961-06-18 00:00:00',89.35,223,262,82.11,'Intermediate' from dual union all
select 1090,'001090','1965-011B','Kosmos-55','Strela-1',11761,'Reentered','1968-02-02 00:00:00','1965-03-24 00:00:00',105.07,260,1736,56.03,'Intermediate' from dual union all
select 10900,'010900','1978-049C','deb Kosmos-1008','Tselina-O part',45244,'Reentered','1978-12-17 00:00:00','1978-07-17 00:00:00',94.21,447,514,74.06,'Intermediate' from dual union all
select 10901,'010901','1978-049D','deb Kosmos-1008','Tselina-O part',45244,'Reentered','1978-12-30 00:00:00','1978-07-17 00:00:00',94.27,453,514,73.98,'Intermediate' from dual union all
select 10902,'010902','1978-049E','deb Kosmos-1008','Tselina-O part',45244,'Reentered','1978-07-17 00:00:00','1978-07-17 00:00:00',94.34,458,516,74.02,'Intermediate' from dual union all
select 10903,'010903','1978-049F','deb Kosmos-1008','Tselina-O part',45244,'Reentered','1978-09-12 00:00:00','1978-07-17 00:00:00',94.91,479,550,73.9,'Intermediate' from dual union all
select 10904,'010904','1978-050A','Kosmos-1009','IS',45245,'Deorbited','1978-05-19 00:00:00','1978-05-19 00:00:00',108.84,964,1383,65.84,'Intermediate' from dual union all
select 10905,'010905','1978-050B','11S692','11S692',45245,'Reentered','1978-06-05 00:00:00','1978-05-19 00:00:00',97.43,147,1125,65.14,'Intermediate' from dual union all
select 10906,'010906','1978-050C','deb Kosmos-1009','',45245,'Reentered','1978-07-18 00:00:00','1978-05-20 00:00:00',93.55,140,757,65.15,'Intermediate' from dual union all
select 10907,'010907','1978-050D','deb Kosmos-1009','',45245,'Reentered','1978-05-23 00:00:00','1978-05-21 00:00:00',91.75,147,574,65.12,'Intermediate' from dual union all
select 10908,'010908','1977-111Y','Kosmos-965 SS 21','ESO',42239,'Reentered','1978-08-04 00:00:00','1978-05-20 00:00:00',94.21,453,508,74.05,'Intermediate' from dual union all
select 10909,'010909','1977-111Z','Radar error','',42239,'Spurious','1977-12-08 00:00:00','1978-05-19 00:00:00',94.2,455,504,74.03,'Intermediate' from dual union all
select 1091,'001091','1965-011C','Kosmos-56','Strela-1',11761,'Reentered','1967-11-02 00:00:00','1965-03-24 00:00:00',104.32,258,1668,56.05,'Intermediate' from dual union all
select 10910,'010910','1977-111AA','Radar error','',42239,'Spurious','1977-12-08 00:00:00','1978-05-22 00:00:00',94.2,455,504,74.03,'Intermediate' from dual union all
select 10911,'010911','1978-051A','Pioneer Venus Orbiter','Pioneer Venus Orbiter',45246,'Deep Space','1978-05-20 00:00:00','1978-05-20 00:00:00',456.42,170,52200,28.5,'Earth Escape' from dual union all
select 10912,'010912','1978-051B','Centaur AC-50','Centaur D-1AR AC-50',45246,'Deep Space','1978-05-20 00:00:00','1978-05-20 00:00:00',456.42,170,52200,28.5,'Earth Escape' from dual union all
select 10913,'010913','1978-042D','DMSP F-3 OLS Cover','',45237,'Reentered','1986-05-01 00:00:00','1978-06-02 00:00:00',101.39,819,830,98.71,'Sun-Synch' from dual union all
select 10914,'010914','1978-042E','DMSP F-3 OLS Cooler Cover','',45237,'Reentered','1989-06-16 00:00:00','1978-07-23 00:00:00',101.38,818,830,98.71,'Sun-Synch' from dual union all
select 10915,'010915','1978-052A','Kosmos-1010','Fram',45247,'Landed','1978-06-05 00:00:00','1978-05-27 00:00:00',88.84,212,223,81.37,'Intermediate' from dual union all
select 10916,'010916','1978-052B','11S510','11S510',45247,'Reentered','1978-05-26 00:00:00','1978-05-25 00:00:00',88.31,183,199,81.35,'Intermediate' from dual union all
select 10917,'010917','1978-053A','Kosmos-1011','Parus',45248,'In Earth orbit','','1992-10-09 00:00:00',104.75,955,1011,82.92,'Intermediate' from dual union all
select 10918,'010918','1978-053B','S3M','S3M',45248,'In Earth orbit','','1990-03-28 00:00:00',104.63,954,1000,82.92,'Intermediate' from dual union all
select 10919,'010919','1978-054A','Kosmos-1012','Zenit-2M',45249,'Landed','1978-06-07 00:00:00','1978-05-27 00:00:00',89.13,201,262,62.81,'Intermediate' from dual union all
select 1092,'001092','1965-011D','S3','S3',11761,'Reentered','1969-12-23 00:00:00','1965-03-28 00:00:00',106.12,275,1819,56.05,'Intermediate' from dual union all
select 10920,'010920','1978-054B','11S510','11S510',45249,'Reentered','1978-05-28 00:00:00','1978-05-27 00:00:00',88.57,175,233,62.8,'Intermediate' from dual union all
select 10921,'010921','1978-048D','KDU','KDU',45243,'Reentered','1978-05-31 00:00:00','1978-05-28 00:00:00',89.25,159,315,72.8,'Intermediate' from dual union all
select 10922,'010922','1978-048E','Blenda','Sunshade',45243,'Reentered','1978-07-03 00:00:00','1978-05-28 00:00:00',88.96,142,305,72.83,'Intermediate' from dual union all
select 10923,'010923','1977-111AB','Kosmos-965 SS 22','ESO',42239,'Reentered','1978-08-24 00:00:00','1978-06-01 00:00:00',94.17,454,503,74.03,'Intermediate' from dual union all
select 10924,'010924','1977-111AC','Kosmos-965 SS 23','ESO',42239,'Reentered','1978-08-17 00:00:00','1978-06-03 00:00:00',94.11,452,499,74.04,'Intermediate' from dual union all
select 10925,'010925','1978-055A','Molniya-1','Molniya-1',45250,'In Earth orbit','','1978-07-03 00:00:00',717.72,434,39918,62.88,'Molniya' from dual union all
select 10926,'010926','1978-055B','11S510','11S510',45250,'Reentered','1978-06-21 00:00:00','1978-06-09 00:00:00',90.25,204,370,62.86,'Intermediate' from dual union all
select 10927,'010927','1978-055C','BOZ','BOZ',45250,'Reentered','1978-06-17 00:00:00','1978-06-10 00:00:00',89.91,187,353,62.83,'Intermediate' from dual union all
select 10928,'010928','1978-052C','KDU','KDU',45247,'Reentered','1978-06-07 00:00:00','1978-06-05 00:00:00',88.38,191,198,81.3,'Intermediate' from dual union all
select 10929,'010929','1978-055D','deb Blok-I','',45250,'Reentered','1978-06-25 00:00:00','1978-06-16 00:00:00',89.41,192,299,62.84,'Intermediate' from dual union all
select 1093,'001093','1965-012A','Kosmos-57','Voskhod 3KD No. 1',11762,'Exploded','1965-02-22 00:00:00','1965-02-22 00:00:00',90.43,165,427,64.7,'Intermediate' from dual union all
select 10930,'010930','1978-056A','Kosmos-1013','Strela-1M',45251,'In Earth orbit','','1992-09-27 00:00:00',116.33,1478,1553,74.02,'Intermediate' from dual union all
select 10931,'010931','1978-056B','Kosmos-1014','Strela-1M',45251,'In Earth orbit','','1992-09-26 00:00:00',116.08,1479,1530,74.02,'Intermediate' from dual union all
select 10932,'010932','1978-056C','Kosmos-1015','Strela-1M',45251,'In Earth orbit','','1992-09-28 00:00:00',115.87,1474,1516,74.02,'Intermediate' from dual union all
select 10933,'010933','1978-056D','Kosmos-1016','Strela-1M',45251,'In Earth orbit','','1992-09-28 00:00:00',115.64,1471,1498,74.02,'Intermediate' from dual union all
select 10934,'010934','1978-056E','Kosmos-1017','Strela-1M',45251,'In Earth orbit','','1992-09-27 00:00:00',115.42,1457,1492,74.03,'Intermediate' from dual union all
select 10935,'010935','1978-056F','Kosmos-1018','Strela-1M',45251,'In Earth orbit','','1992-09-23 00:00:00',115.21,1442,1488,74.03,'Intermediate' from dual union all
select 10936,'010936','1978-056G','Kosmos-1019','Strela-1M',45251,'In Earth orbit','','1992-09-27 00:00:00',115.01,1424,1488,74.02,'Intermediate' from dual union all
select 10937,'010937','1978-056H','Kosmos-1020','Strela-1M',45251,'In Earth orbit','','1992-09-26 00:00:00',114.78,1407,1484,74.02,'Intermediate' from dual union all
select 10938,'010938','1978-056J','S3M','S3M',45251,'In Earth orbit','','1992-05-11 00:00:00',117.88,1480,1692,74.03,'Intermediate' from dual union all
select 10939,'010939','1978-057A','Kosmos-1021','Zenit-4MKM',45252,'Landed','1978-06-23 00:00:00','1978-06-13 00:00:00',89.29,171,308,65.03,'Intermediate' from dual union all
select 1094,'001094','1965-011E','deb Kosmos-54','',11761,'Reentered','1966-04-08 00:00:00','1965-03-25 00:00:00',103.49,259,1588,56.04,'Intermediate' from dual union all
select 10940,'010940','1978-057B','11S510','11S510',45252,'Reentered','1978-06-13 00:00:00','1978-06-12 00:00:00',88.76,165,261,65.04,'Intermediate' from dual union all
select 10941,'010941','1978-058A','OPS 9454','CHALET 8',45253,'In Earth orbit','','1978-07-01 00:00:00',1446.19,29929,42039,12,'Near-synch' from dual union all
select 10942,'010942','1978-058B','Transtage 33','Transtage 33',45253,'In Earth orbit','','1978-07-01 00:00:00',1446.19,29929,42039,12,'Near-synch' from dual union all
select 10943,'010943','1978-058C','Titan 23C-15 Stage 2','LR91AJ11 23C-15',45253,'Reentered','1978-06-14 00:00:00','1978-06-10 00:00:00',91.32,153,526,29.98,'Intermediate' from dual union all
select 10944,'010944','1978-059A','Kosmos-1022','Zenit-4MKM',45254,'Landed','1978-06-25 00:00:00','1978-06-14 00:00:00',89.63,170,343,72.85,'Intermediate' from dual union all
select 10945,'010945','1978-059B','11S510','11S510',45254,'Reentered','1978-06-16 00:00:00','1978-06-15 00:00:00',88.81,162,270,72.85,'Intermediate' from dual union all
select 10946,'010946','1977-027E','deb Kosmos-903','',42155,'Reentered','2005-06-24 00:00:00','1979-05-07 00:00:00',713.06,1739,38383,63.82,'Molniya' from dual union all
select 10947,'010947','1978-060A','OPS 4515','KENNEN 2',45255,'Deorbited','1981-08-23 00:00:00','1978-07-15 00:00:00',92.07,270,483,96.82,'Sun-Synch' from dual union all
select 10948,'010948','1978-060B','Titan 23D-18 Stage 2','LR91AJ11 23D-18',45255,'Reentered','1978-07-24 00:00:00','1978-06-25 00:00:00',91.17,217,447,96.96,'Sun-Synch' from dual union all
select 10949,'010949','1978-055E','Blok-L','Blok-L',45250,'In Earth orbit','','1978-07-15 00:00:00',732.69,425,40662,62.86,'Molniya' from dual union all
select 1095,'001095','1965-012B','11A57I','11A57I',11762,'Reentered','1965-03-05 00:00:00','1965-02-25 00:00:00',90.41,173,417,64.74,'Intermediate' from dual union all
select 10950,'010950','1977-108C','Star 37E S/N 40050','Star 37E',42236,'Reentered','1990-08-12 00:00:00','1990-08-12 00:00:00',88.35,119,267,27.02,'Intermediate' from dual union all
select 10951,'010951','1978-060C','deb KH-11 F-2','',45255,'Reentered','1978-06-18 00:00:00','1978-06-15 00:00:00',91.11,211,447,96.94,'Sun-Synch' from dual union all
select 10952,'010952','1978-061A','Soyuz-29','Soyuz 7K-T No. 46',45256,'Landed','1978-09-03 00:00:00','1978-07-29 00:00:00',91.1,327,329,51.63,'Intermediate' from dual union all
select 10953,'010953','1978-062A','GOES 3','GOES C',45257,'In Earth orbit','','1978-07-30 00:00:00',1436.11,35767,35807,.96,'Stationary' from dual union all
select 10954,'010954','1978-062B','Delta 142','Delta SSPU DSV-3P-4',45257,'In Earth orbit','','1990-01-30 00:00:00',107.51,551,1672,28.45,'Intermediate' from dual union all
select 10955,'010955','1978-062C','Star 37E S/N 40079','Star 37E',45257,'Reentered','1983-08-20 00:00:00','1978-07-16 00:00:00',637.28,168,36138,23.63,'GEO Transfer' from dual union all
select 10956,'010956','1978-061B','11S510','11S510',45256,'Reentered','1978-06-18 00:00:00','1978-06-17 00:00:00',88.34,174,210,51.63,'Intermediate' from dual union all
select 10957,'010957','1978-060D','deb KH-11 F-2','',45255,'Reentered','1978-06-18 00:00:00','1978-06-16 00:00:00',91.32,242,436,96.97,'Sun-Synch' from dual union all
select 10958,'010958','1978-060E','deb KH-11 F-2','',45255,'Reentered','1978-06-18 00:00:00','1978-06-15 00:00:00',91.8,222,504,96.98,'Sun-Synch' from dual union all
select 10959,'010959','1978-060F','deb KH-11 F-2','',45255,'Reentered','1978-06-18 00:00:00','1978-06-15 00:00:00',91.82,222,506,96.98,'Sun-Synch' from dual union all
select 1096,'001096','1965-013A','OPS 4782','CORONA J-14',11763,'Reentered Attached','1965-03-18 00:00:00','1965-03-05 00:00:00',89.76,176,350,75.08,'Intermediate' from dual union all
select 10960,'010960','1977-053B','SVS Stage 2','Star 37E S/N 40055',42181,'Reentered','2001-04-28 00:00:00','1977-07-31 00:00:00',343.6,178,19567,63.21,'Highly Ellip' from dual union all
select 10961,'010961','1978-063A','Kosmos-1023','Strela-2M',45258,'In Earth orbit','','1992-09-30 00:00:00',100.42,768,788,74.07,'Intermediate' from dual union all
select 10962,'010962','1978-063B','S3M','S3M',45258,'In Earth orbit','','1991-03-28 00:00:00',100.27,753,789,74.07,'Intermediate' from dual union all
select 10963,'010963','1978-057C','Blenda','Sunshade',45252,'Reentered','1978-06-25 00:00:00','1978-06-22 00:00:00',88.94,163,282,65.03,'Intermediate' from dual union all
select 10964,'010964','1978-057D','KDU','KDU',45252,'Reentered','1978-06-26 00:00:00','1978-06-22 00:00:00',89.19,162,307,65.02,'Intermediate' from dual union all
select 10965,'010965','1978-059C','KDU','KDU',45254,'Reentered','1978-06-28 00:00:00','1978-06-24 00:00:00',89.41,163,328,72.86,'Intermediate' from dual union all
select 10966,'010966','1978-059D','Blenda','Sunshade',45254,'Reentered','1978-06-28 00:00:00','1978-06-25 00:00:00',89.33,154,329,72.85,'Intermediate' from dual union all
select 10967,'010967','1978-064A','Seasat 1','Seasat A',45259,'In Earth orbit','','1978-08-04 00:00:00',100.57,775,797,108.02,'Retrograde' from dual union all
select 10968,'010968','1978-065A','Soyuz-30','Soyuz 7K-T(A9) No. 67',45260,'Landed','1978-07-05 00:00:00','1978-06-28 00:00:00',90.11,255,305,51.63,'Intermediate' from dual union all
select 10969,'010969','1978-065B','11S510','11S510',45260,'Reentered','1978-06-30 00:00:00','1978-06-28 00:00:00',88.57,183,225,51.64,'Intermediate' from dual union all
select 1097,'001097','1965-014A','Kosmos-58','Meteor 11F614 No. 2',11764,'Reentered','1990-02-25 00:00:00','1965-03-31 00:00:00',96.78,561,648,65.04,'Intermediate' from dual union all
select 10970,'010970','1978-066A','Kosmos-1024','Oko',45261,'In Earth orbit','','1978-07-28 00:00:00',717.44,628,39710,62.75,'Molniya' from dual union all
select 10971,'010971','1978-066B','11S510','11S510',45261,'Reentered','1978-07-31 00:00:00','1978-07-05 00:00:00',91.56,209,494,62.81,'Intermediate' from dual union all
select 10972,'010972','1978-066C','BOZ','BOZ',45261,'Reentered','1978-07-12 00:00:00','1978-07-01 00:00:00',91.68,170,544,62.86,'Intermediate' from dual union all
select 10973,'010973','1978-067A','Kosmos-1025','EPN No. 03.0380',45262,'Reentered','2007-03-10 00:00:00','1987-12-07 00:00:00',97.04,604,630,82.49,'Intermediate' from dual union all
select 10974,'010974','1978-067B','S5M','S5M',45262,'In Earth orbit','','1987-10-12 00:00:00',97.47,621,655,82.48,'Intermediate' from dual union all
select 10975,'010975','1978-068A','Comstar D-3','Comstar D-3',45263,'In Earth orbit','','1978-07-29 00:00:00',1427.84,35467,35783,.06,'Drift GEO' from dual union all
select 10976,'010976','1978-068B','Centaur AC-41','Centaur D-1AR AC-41',45263,'In Earth orbit','','1978-08-02 00:00:00',649.68,619,36321,21.91,'GEO Transfer' from dual union all
select 10977,'010977','1978-069A','Kosmos-1026','Energiya No. 2',45264,'Landed','1978-07-06 00:00:00','1978-07-04 00:00:00',88.99,206,244,51.78,'Intermediate' from dual union all
select 10978,'010978','1978-069B','11S510','11S510',45264,'Reentered','1978-07-07 00:00:00','1978-07-04 00:00:00',88.8,204,226,51.78,'Intermediate' from dual union all
select 10979,'010979','1978-070A','Progress-2','Progress 7K-TG No. 101',45265,'Deorbited','1978-08-04 00:00:00','1978-07-29 00:00:00',91.1,327,329,51.63,'Intermediate' from dual union all
select 1098,'001098','1965-014B','8A92E','8A92E',11764,'Reentered','1988-07-25 00:00:00','1965-04-07 00:00:00',96.86,511,706,65.04,'Intermediate' from dual union all
select 10980,'010980','1978-070B','11S510','11S510',45265,'Reentered','1978-07-10 00:00:00','1978-07-09 00:00:00',88.06,171,185,51.61,'Intermediate' from dual union all
select 10981,'010981','1978-071A','Geos 2','Geos 2',45266,'In Earth orbit','','1978-09-03 00:00:00',1435.98,35768,35800,.69,'Stationary' from dual union all
select 10982,'010982','1978-071B','Delta 143','Delta SSPU DSV-3P-4',45266,'Reentered','1978-12-11 00:00:00','1978-12-03 00:00:00',91.62,166,542,28.23,'Intermediate' from dual union all
select 10983,'010983','1978-071C','Star 37E S/N 40061','Star 37E',45266,'Reentered','2006-10-19 00:00:00','1978-08-21 00:00:00',627.12,232,35551,25.59,'GEO Transfer' from dual union all
select 10984,'010984','1978-072A','Molniya-1','Molniya-1',45267,'Reentered','1992-02-08 00:00:00','1978-08-21 00:00:00',717.76,608,39746,62.89,'Molniya' from dual union all
select 10985,'010985','1978-072B','11S510','11S510',45267,'Reentered','1978-08-30 00:00:00','1978-07-21 00:00:00',92.34,211,567,62.84,'Intermediate' from dual union all
select 10986,'010986','1978-072C','BOZ','BOZ',45267,'Reentered','1978-08-23 00:00:00','1978-07-21 00:00:00',92.4,201,584,62.84,'Intermediate' from dual union all
select 10987,'010987','1978-073A','Raduga','Gran'' No. 14L',45268,'In Earth orbit','','1978-09-10 00:00:00',1435.91,35764,35802,.33,'Stationary' from dual union all
select 10988,'010988','1978-073B','8S812','8S812',45268,'Reentered','1978-07-20 00:00:00','1978-07-19 00:00:00',88.11,170,192,51.63,'Intermediate' from dual;
insert into SATELLITE
select 10989,'010989','1978-073C','Perekhodnik','Sredniy perekhodnik',45268,'Reentered','1978-07-19 00:00:00','1978-07-19 00:00:00',88.05,161,194,51.62,'Intermediate' from dual union all
select 1099,'001099','1964-083G','deb NNS O-2','',10369,'In Earth orbit','','1965-03-19 00:00:00',106.29,1023,1086,90,'Polar' from dual union all
select 10990,'010990','1974-089EK','deb Delta 104','',33692,'In Earth orbit','','1993-08-02 00:00:00',113.85,1346,1461,101.18,'Sun-Synch' from dual union all
select 10991,'010991','1978-074A','Kosmos-1027','Zaliv',45269,'In Earth orbit','','1992-10-11 00:00:00',104.67,960,997,82.94,'Intermediate' from dual union all
select 10992,'010992','1978-074B','S3M','S3M',45269,'In Earth orbit','','1991-03-28 00:00:00',104.58,961,989,82.92,'Intermediate' from dual union all
select 10993,'010993','1978-075A','OPS 7310','QUASAR 3 (or JUMPSEAT 5)',45270,'Reentered','2001-02-08 00:00:00','1978-08-05 00:00:00',704.3,311,39377,63.2,'Molniya' from dual union all
select 10994,'010994','1978-075B','Agena D','Agena D',45270,'Reentered','2002-06-25 00:00:00','1978-08-05 00:00:00',704.3,311,39377,63.2,'Molniya' from dual union all
select 10995,'010995','1978-076A','Kosmos-1028','Feniks No. 14',45271,'Landed','1978-09-04 00:00:00','1978-08-12 00:00:00',89.15,162,303,67.14,'Intermediate' from dual union all
select 10996,'010996','1978-076B','11S510','11S510',45271,'Reentered','1978-08-07 00:00:00','1978-08-06 00:00:00',88.49,168,232,67.13,'Intermediate' from dual union all
select 10997,'010997','1978-076C','Kosmos-1028 part','Feniks part',45271,'Reentered','1978-08-08 00:00:00','1978-08-06 00:00:00',88.59,153,257,67.13,'Intermediate' from dual union all
select 10998,'010998','1978-066D','Blok-L','Blok-L No. 71642627',45261,'In Earth orbit','','1978-08-08 00:00:00',720.46,631,39856,62.76,'Molniya' from dual union all
select 10999,'010999','1978-077A','Progress-3','Progress 7K-TG No. 103',45272,'Deorbited','1978-08-24 00:00:00','1978-08-21 00:00:00',91.4,335,352,51.63,'Intermediate' from dual union all
select 11,'000011','59 Alpha 1','Vanguard II','Cloud cover satellite',5525,'In Earth orbit','','1959-05-15 00:00:00',125.69,564,3304,32.88,'Medium' from dual union all
select 110,'000110','58 Eta','Pioneer 1','Able 2',4796,'Reentered','1958-10-13 00:00:00','1958-10-11 00:00:00',2614.91,869,113860,35.89,'Suborbital' from dual union all
select 1100,'001100','1965-012C','deb Kosmos-57','',11762,'Reentered','1965-03-09 00:00:00','1965-03-02 00:00:00',89.79,145,383,64.85,'Intermediate' from dual union all
select 11000,'011000','1978-077B','11S510','11S510',45272,'Reentered','1978-08-09 00:00:00','1978-08-08 00:00:00',88.59,184,226,51.66,'Intermediate' from dual union all
select 11001,'011001','1978-078A','Pioneer Venus Multiprobe','Pioneer Venus Multiprobe',45273,'Deep Space','1978-08-08 00:00:00','1978-08-08 00:00:00',1683.97,170,106699,28.6,'Earth Escape' from dual union all
select 11002,'011002','1978-078B','Centaur AC-51 debris','[duplicate for AC-51]',45273,'Reentered','1978-08-08 00:00:00','1978-08-08 00:00:00',87.63,150,165,28.6,'Intermediate' from dual union all
select 11003,'011003','1978-078C','Centaur AC-51','Centaur D-1AR AC-51',45273,'Deep Space','1978-08-08 00:00:00','1978-08-08 00:00:00',1683.97,170,106699,28.6,'Earth Escape' from dual union all
select 11004,'011004','1978-079A','ISEE 3','ISEE C',45274,'Deep Space','1978-08-13 00:00:00','1978-08-12 00:00:00',73654.64,180,1151164,28.9,'Cislunar' from dual union all
select 11005,'011005','1978-079B','Delta 144 part','debris',45274,'Reentered','1978-10-23 00:00:00','1978-09-11 00:00:00',98.12,174,1163,28.73,'Intermediate' from dual union all
select 11006,'011006','1978-079C','Delta 144','Delta SSPU DSV-3P-4',45274,'In Earth orbit','','1979-12-31 00:00:00',118.92,585,2680,28.22,'Intermediate' from dual union all
select 11007,'011007','1978-080A','Molniya-1','Molniya-1',45275,'Reentered','2015-08-11 00:00:00','1978-09-23 00:00:00',717.7,488,39863,62.86,'Molniya' from dual union all
select 11008,'011008','1978-080B','11S510','11S510',45275,'Reentered','1978-09-15 00:00:00','1978-08-30 00:00:00',90.64,208,404,62.83,'Intermediate' from dual union all
select 11009,'011009','1978-080C','BOZ','BOZ',45275,'Reentered','1978-09-06 00:00:00','1978-08-31 00:00:00',90.02,181,370,62.8,'Intermediate' from dual union all
select 1101,'001101','1965-012D','deb Kosmos-57','',11762,'Reentered','1965-02-25 00:00:00','1965-02-23 00:00:00',90.12,156,404,64.98,'Intermediate' from dual union all
select 11010,'011010','1978-081A','Soyuz-31','Soyuz 7K-T No. 47',45276,'Landed','1978-11-02 00:00:00','1978-10-21 00:00:00',91.73,358,361,51.64,'Intermediate' from dual union all
select 11011,'011011','1978-081B','11S510','11S510',45276,'Reentered','1978-08-28 00:00:00','1978-08-27 00:00:00',88.7,192,229,51.63,'Intermediate' from dual union all
select 11012,'011012','1978-082A','Kosmos-1029','Zenit-4MKM',45277,'Landed','1978-09-08 00:00:00','1978-08-31 00:00:00',89.31,169,312,62.81,'Intermediate' from dual union all
select 11013,'011013','1978-082B','11S510','11S510',45277,'Reentered','1978-09-03 00:00:00','1978-08-31 00:00:00',89.22,172,300,62.8,'Intermediate' from dual union all
select 11014,'011014','1978-076D','deb Agregatniy otsek','AO-PO',45271,'Reentered','1978-09-04 00:00:00','1978-09-04 00:00:00',88.84,147,288,67.13,'Intermediate' from dual union all
select 11015,'011015','1978-083A','Kosmos-1030','Oko',45278,'Reentered','2004-08-17 00:00:00','1978-10-12 00:00:00',718.77,667,39737,62.85,'Molniya' from dual union all
select 11016,'011016','1978-083B','11S510','11S510',45278,'Reentered','1978-10-08 00:00:00','1978-09-13 00:00:00',91.98,207,537,62.79,'Intermediate' from dual union all
select 11017,'011017','1978-083C','BOZ','BOZ',45278,'Reentered','1978-09-21 00:00:00','1978-09-13 00:00:00',91.07,167,487,62.87,'Intermediate' from dual union all
select 11018,'011018','1978-082C','KDU','KDU',45277,'Reentered','1978-09-11 00:00:00','1978-09-07 00:00:00',89.58,168,340,62.82,'Intermediate' from dual union all
select 11019,'011019','1978-082D','Blenda','Sunshade',45277,'Reentered','1978-09-10 00:00:00','1978-09-08 00:00:00',88.98,164,284,62.81,'Intermediate' from dual union all
select 1102,'001102','1965-012E','deb Kosmos-57','',11762,'Reentered','1965-02-26 00:00:00','1965-02-25 00:00:00',87.65,138,178,64.75,'Intermediate' from dual union all
select 11020,'011020','1978-084A','Venera-11','4V-1 No. 360',45279,'Deep Space','1978-09-09 00:00:00','',null,null,null,null,'' from dual union all
select 11021,'011021','1978-084B','SOZ','SOZ',45279,'Reentered','1978-09-10 00:00:00','1978-09-09 00:00:00',87.81,152,181,51.55,'Intermediate' from dual union all
select 11022,'011022','1978-085A','Kosmos-1031','Zenit-4MKM',45280,'Landed','1978-09-22 00:00:00','1978-09-11 00:00:00',89.33,170,313,62.82,'Intermediate' from dual union all
select 11023,'011023','1978-085B','11S510','11S510',45280,'Reentered','1978-09-14 00:00:00','1978-09-11 00:00:00',89.15,172,293,62.81,'Intermediate' from dual union all
select 11024,'011024','1978-085C','Sensor cover','Zenit-4MK sensor cover',45280,'Reentered','1979-04-17 00:00:00','1978-09-09 00:00:00',88.94,168,276,62.79,'Intermediate' from dual union all
select 11025,'011025','1978-086A','Venera-12','4V-1 No. 361',45281,'Deep Space','1978-09-14 00:00:00','',null,null,null,null,'' from dual union all
select 11026,'011026','1978-086B','SOZ','SOZ',45281,'Reentered','1978-09-15 00:00:00','1978-09-14 00:00:00',87.98,165,184,51.51,'Intermediate' from dual union all
select 11027,'011027','1978-087A','Jikiken','EXOS-B',45282,'Reentered','2018-04-22 00:00:00','1979-02-09 00:00:00',502.62,249,28876,31.43,'Highly Ellip' from dual union all
select 11028,'011028','1978-087B','KM-B','KM-B',45282,'In Earth orbit','','1979-02-09 00:00:00',511.23,282,29320,31.47,'Highly Ellip' from dual union all
select 11029,'011029','1978-088A','Kosmos-1032','Zenit-2M',45283,'Landed','1978-10-02 00:00:00','1978-09-23 00:00:00',88.77,212,216,81.34,'Intermediate' from dual union all
select 1103,'001103','1965-012F','deb Kosmos-57','',11762,'Reentered','1965-03-02 00:00:00','1965-02-25 00:00:00',88.33,167,216,64.6,'Intermediate' from dual union all
select 11030,'011030','1978-088B','11S510','11S510',45283,'Reentered','1978-09-22 00:00:00','1978-09-21 00:00:00',88.26,186,191,81.34,'Intermediate' from dual union all
select 11031,'011031','1978-085D','KDU','KDU',45280,'Reentered','1978-09-24 00:00:00','1978-09-23 00:00:00',88.74,155,270,62.82,'Intermediate' from dual union all
select 11032,'011032','1977-097U','Garbage bag','',42225,'Reentered','1978-10-21 00:00:00','1978-09-26 00:00:00',91.28,331,344,51.63,'Intermediate' from dual union all
select 11033,'011033','1977-097V','Garbage bag','',42225,'Reentered','1978-10-14 00:00:00','1978-09-26 00:00:00',91.08,318,338,51.63,'Intermediate' from dual union all
select 11034,'011034','1977-097W','Garbage bag','',42225,'Reentered','1978-12-03 00:00:00','1978-09-26 00:00:00',91.2,322,345,51.63,'Intermediate' from dual union all
select 11035,'011035','1977-097X','Garbage bag','',42225,'Reentered','1978-11-26 00:00:00','1978-09-26 00:00:00',91.23,326,343,51.64,'Intermediate' from dual union all
select 11036,'011036','1977-097Y','Garbage bag','',42225,'Reentered','1979-05-06 00:00:00','1978-09-28 00:00:00',91.16,320,342,51.63,'Intermediate' from dual union all
select 11037,'011037','1978-088C','Nauka part','',45283,'Reentered','1978-10-02 00:00:00','1978-10-01 00:00:00',88.21,178,194,81.34,'Intermediate' from dual union all
select 11038,'011038','1978-088D','Kapsula Kosmosa-1032','Nauka 25KS',45283,'Reentered','1978-10-03 00:00:00','1978-10-02 00:00:00',88.07,171,186,81.36,'Intermediate' from dual union all
select 11039,'011039','1978-089A','Kosmos-1033','Fram',45284,'Landed','1978-10-16 00:00:00','1978-10-05 00:00:00',88.93,211,233,81.36,'Intermediate' from dual union all
select 1104,'001104','1965-012G','deb Kosmos-57','',11762,'Reentered','1965-03-10 00:00:00','1965-03-03 00:00:00',90.67,155,460,64.81,'Intermediate' from dual union all
select 11040,'011040','1978-090A','Progress-4','Progress 7K-TG No. 105',45285,'Deorbited','1978-10-26 00:00:00','1978-10-11 00:00:00',91.21,324,344,51.63,'Intermediate' from dual union all
select 11041,'011041','1978-089B','11S510','11S510',45284,'Reentered','1978-10-06 00:00:00','1978-10-05 00:00:00',88.5,190,211,81.36,'Intermediate' from dual union all
select 11042,'011042','1978-091A','Kosmos-1034','Strela-1M',45286,'In Earth orbit','','1992-05-02 00:00:00',114.91,1422,1481,74.03,'Intermediate' from dual union all
select 11043,'011043','1978-090B','11S510','11S510',45285,'Reentered','1978-10-05 00:00:00','1978-10-04 00:00:00',88.71,183,239,51.65,'Intermediate' from dual union all
select 11044,'011044','1978-091B','Kosmos-1035','Strela-1M',45286,'In Earth orbit','','1992-05-07 00:00:00',114.68,1402,1480,74.04,'Intermediate' from dual union all
select 11045,'011045','1978-091C','Kosmos-1036','Strela-1M',45286,'In Earth orbit','','1992-05-06 00:00:00',115.13,1442,1481,74.03,'Intermediate' from dual union all
select 11046,'011046','1978-091D','Kosmos-1037','Strela-1M',45286,'In Earth orbit','','1992-05-03 00:00:00',115.35,1461,1482,74.03,'Intermediate' from dual union all
select 11047,'011047','1978-091E','Kosmos-1038','Strela-1M',45286,'In Earth orbit','','1992-05-06 00:00:00',115.57,1477,1486,74.03,'Intermediate' from dual union all
select 11048,'011048','1978-091F','Kosmos-1039','Strela-1M',45286,'In Earth orbit','','1992-05-05 00:00:00',116.31,1478,1552,74.03,'Intermediate' from dual union all
select 11049,'011049','1978-091G','Kosmos-1040','Strela-1M',45286,'In Earth orbit','','1992-05-06 00:00:00',116.04,1479,1527,74.03,'Intermediate' from dual union all
select 1105,'001105','1965-012H','deb Kosmos-57','',11762,'Reentered','1965-02-25 00:00:00','1965-02-25 00:00:00',87.62,124,189,64.89,'Intermediate' from dual union all
select 11050,'011050','1978-091H','Kosmos-1041','Strela-1M',45286,'In Earth orbit','','1992-05-02 00:00:00',115.82,1478,1507,74.03,'Intermediate' from dual union all
select 11051,'011051','1978-091J','S3M','S3M',45286,'In Earth orbit','','1992-05-08 00:00:00',117.98,1482,1699,74.04,'Intermediate' from dual union all
select 11052,'011052','1978-092A','Kosmos-1042','Zenit-4MKM',45287,'Landed','1978-10-19 00:00:00','1978-10-08 00:00:00',89.26,178,298,62.8,'Intermediate' from dual union all
select 11053,'011053','1978-092B','11S510','11S510',45287,'Reentered','1978-10-10 00:00:00','1978-10-08 00:00:00',88.76,169,258,62.79,'Intermediate' from dual union all
select 11054,'011054','1978-093A','Navstar 3','Navstar GPS SVN 3',45288,'In Earth orbit','','1978-11-08 00:00:00',718.02,20181,20186,62.81,'Medium' from dual union all
select 11055,'011055','1978-094A','Kosmos-1043','Tselina-D',45289,'Reentered','1998-02-27 00:00:00','1990-07-05 00:00:00',95.39,530,544,81.19,'Intermediate' from dual union all
select 11056,'011056','1978-094B','8A92ME','8A92ME',45289,'Reentered','2002-01-19 00:00:00','1990-07-05 00:00:00',95.86,525,596,81.19,'Intermediate' from dual union all
select 11057,'011057','1978-095A','Molniya-3','Molniya-3 No. 22',45290,'In Earth orbit','','1978-11-14 00:00:00',717.66,443,39906,62.9,'Molniya' from dual union all
select 11058,'011058','1978-095B','11S510','11S510',45290,'Reentered','1978-10-28 00:00:00','1978-10-21 00:00:00',90.15,202,362,62.82,'Intermediate' from dual union all
select 11059,'011059','1978-095C','BOZ','BOZ',45290,'Reentered','1978-10-20 00:00:00','1978-10-17 00:00:00',89.71,169,352,62.78,'Intermediate' from dual union all
select 1106,'001106','1965-012J','deb Kosmos-57','',11762,'Reentered','1965-03-02 00:00:00','1965-02-25 00:00:00',88.44,169,226,64.74,'Intermediate' from dual union all
select 11060,'011060','1978-096A','Tiros-N','Tiros-N',45291,'In Earth orbit','','1978-11-12 00:00:00',102.05,847,865,98.92,'Sun-Synch' from dual union all
select 11061,'011061','1978-096B','Tiros-N despin weight','',45291,'In Earth orbit','','1993-08-16 00:00:00',100.07,760,764,98.89,'Sun-Synch' from dual union all
select 11062,'011062','1978-096C','Tiros-N despin weight','',45291,'In Earth orbit','','1993-08-16 00:00:00',100.04,758,763,98.89,'Sun-Synch' from dual union all
select 11063,'011063','1978-089C','KDU','KDU',45284,'Reentered','1978-10-17 00:00:00','1978-10-14 00:00:00',88.56,196,211,81.35,'Intermediate' from dual union all
select 11064,'011064','1978-095D','deb Blok-I','',45290,'Reentered','1978-10-31 00:00:00','1978-10-22 00:00:00',89.08,152,305,63.07,'Intermediate' from dual union all
select 11065,'011065','1978-097A','Kosmos-1044','Zenit-2M',45292,'Landed','1978-10-30 00:00:00','1978-10-19 00:00:00',89.41,202,289,62.83,'Intermediate' from dual union all
select 11066,'011066','1978-097B','11S510','11S510',45292,'Reentered','1978-10-22 00:00:00','1978-10-19 00:00:00',89.04,202,253,62.82,'Intermediate' from dual union all
select 11067,'011067','1978-097C','Sensor cover','Zenit-2M sensor cover',45292,'Reentered','1978-10-21 00:00:00','1978-10-18 00:00:00',89.42,189,303,62.79,'Intermediate' from dual union all
select 11068,'011068','1978-097D','Sensor cover','Zenit-2M sensor cover',45292,'Reentered','1978-10-21 00:00:00','1978-10-18 00:00:00',89.36,193,293,62.81,'Intermediate' from dual union all
select 11069,'011069','1978-097E','Sensor cover','Zenit-2M sensor cover',45292,'Reentered','1978-10-21 00:00:00','1978-10-17 00:00:00',88.86,185,252,62.8,'Intermediate' from dual union all
select 1107,'001107','1965-012K','deb Kosmos-57','',11762,'Reentered','1965-03-25 00:00:00','1965-02-25 00:00:00',87.85,118,219,64.75,'Intermediate' from dual union all
select 11070,'011070','1978-092C','KDU','KDU',45287,'Reentered','1978-10-21 00:00:00','1978-10-19 00:00:00',89.25,171,303,62.8,'Intermediate' from dual union all
select 11071,'011071','1978-092D','Blenda','Sunshade',45287,'Reentered','1978-10-21 00:00:00','1978-10-19 00:00:00',88.94,169,275,62.8,'Intermediate' from dual union all
select 11072,'011072','1977-118D','Star 37E S/N 40052','Star 37E',42246,'Reentered','1981-10-09 00:00:00','1978-10-11 00:00:00',464.66,201,26788,28.44,'Highly Ellip' from dual union all
select 11073,'011073','1978-072D','Blok-L','Blok-ML No. 76712447',45267,'Reentered','1992-03-17 00:00:00','1992-03-14 00:00:00',535.96,78,30879,61.81,'ERROR - Unexpected value "TA"' from dual union all
select 11074,'011074','1978-073D','SOZ','SOZ',45268,'Reentered','1979-02-21 00:00:00','1983-05-24 00:00:00',567.15,346,32291,46.41,'Highly Ellip' from dual union all
select 11075,'011075','1978-080D','Blok-L','Blok-L',45275,'Reentered','1997-07-03 00:00:00','1993-08-18 00:00:00',728.84,252,40647,62.7,'Molniya' from dual union all
select 11076,'011076','1978-083D','Blok-L','Blok-L',45278,'In Earth orbit','','1978-11-01 00:00:00',723.18,655,39966,62.88,'Molniya' from dual union all
select 11077,'011077','1978-073E','SOZ','SOZ',45268,'Reentered','1979-07-13 00:00:00','1983-05-24 00:00:00',567.21,350,32290,46.4,'Highly Ellip' from dual;
insert into SATELLITE
select 11078,'011078','1978-093B','SVS Stage 2','Star 37E S/N 40067',45288,'Reentered','1991-12-25 00:00:00','1991-10-18 00:00:00',112.08,149,2496,62.85,'Intermediate' from dual union all
select 11079,'011079','1978-095E','Blok-L','Blok-L',45290,'In Earth orbit','','1978-11-20 00:00:00',735.1,405,40801,62.91,'Molniya' from dual union all
select 1108,'001108','1965-012L','deb Kosmos-57','',11762,'Reentered','1965-03-01 00:00:00','1965-02-25 00:00:00',88.93,177,267,64.7,'Intermediate' from dual union all
select 11080,'011080','1978-098A','Nimbus 7','Nimbus G',45293,'In Earth orbit','','1980-01-02 00:00:00',104.14,948,961,99.29,'Sun-Synch' from dual union all
select 11081,'011081','1978-098B','CAMEO','CAMEO',45293,'In Earth orbit','','1993-09-28 00:00:00',103.99,926,968,99.65,'Sun-Synch' from dual union all
select 11082,'011082','1978-099A','Interkosmos-18','AUOS-Z Mag-IK',45294,'Reentered','1981-03-17 00:00:00','1978-11-23 00:00:00',96.27,402,758,82.95,'Intermediate' from dual union all
select 11083,'011083','1978-099B','S3M','S3M',45294,'Reentered','1981-05-13 00:00:00','1978-11-23 00:00:00',96.16,399,751,82.96,'Intermediate' from dual union all
select 11084,'011084','1978-100A','Kosmos-1045','Meteor-2 GVM',45295,'In Earth orbit','','1992-05-07 00:00:00',120.32,1685,1706,82.54,'Medium' from dual union all
select 11085,'011085','1978-100B','Radio RS-1','RS-1',45295,'In Earth orbit','','1992-05-07 00:00:00',120.33,1685,1707,82.54,'Medium' from dual union all
select 11086,'011086','1978-100C','Radio RS-2','RS-2',45295,'In Earth orbit','','1992-05-07 00:00:00',120.32,1685,1706,82.55,'Medium' from dual union all
select 11087,'011087','1978-100D','S5M','S5M',45295,'In Earth orbit','','1993-07-27 00:00:00',120.29,1687,1701,82.54,'Medium' from dual union all
select 11088,'011088','1978-101A','Prognoz-7','SO-M No. 507',45296,'Deep Space','1978-10-31 00:00:00','1979-01-19 00:00:00',6001.08,1126,204897,65.68,'Deep Space' from dual union all
select 11089,'011089','1978-101C','BOZ','BOZ',45296,'Reentered','1978-11-17 00:00:00','1978-11-07 00:00:00',90.52,202,398,65.02,'Intermediate' from dual union all
select 1109,'001109','1965-012M','deb Kosmos-57','',11762,'Reentered','1965-02-26 00:00:00','1965-02-25 00:00:00',87.48,142,158,64.75,'Intermediate' from dual union all
select 11090,'011090','1978-101B','11S510','11S510',45296,'Reentered','1978-11-25 00:00:00','1978-11-07 00:00:00',90.96,227,417,65,'Intermediate' from dual union all
select 11091,'011091','1977-097Z','Garbage bag','',42225,'Reentered','1979-04-14 00:00:00','1978-10-30 00:00:00',91.75,355,366,51.64,'Intermediate' from dual union all
select 11092,'011092','1977-097AA','Garbage bag','',42225,'Reentered','1978-12-12 00:00:00','1978-10-30 00:00:00',91.74,356,363,51.64,'Intermediate' from dual union all
select 11093,'011093','1977-097AB','Garbage bag','',42225,'Reentered','1978-11-26 00:00:00','1978-10-30 00:00:00',91.74,357,362,51.65,'Intermediate' from dual union all
select 11094,'011094','1977-097AC','Garbage bag','',42225,'Reentered','1979-01-28 00:00:00','1978-10-30 00:00:00',91.62,352,355,51.65,'Intermediate' from dual union all
select 11095,'011095','1977-097AD','Garbage bag','',42225,'Reentered','1978-12-15 00:00:00','1978-10-30 00:00:00',91.57,348,356,51.65,'Intermediate' from dual union all
select 11096,'011096','1977-097AE','Garbage bag','',42225,'Reentered','1979-07-30 00:00:00','1978-10-30 00:00:00',91.56,348,355,51.63,'Intermediate' from dual union all
select 11097,'011097','1977-097AF','Garbage bag','',42225,'Reentered','1978-12-19 00:00:00','1978-10-30 00:00:00',91.65,355,356,51.62,'Intermediate' from dual union all
select 11098,'011098','1978-102A','Kosmos-1046','Orion',45297,'Landed','1978-11-13 00:00:00','1978-11-04 00:00:00',89.73,201,322,72.86,'Intermediate' from dual union all
select 11099,'011099','1978-102B','11S510','11S510',45297,'Reentered','1978-11-07 00:00:00','1978-11-04 00:00:00',89.09,184,275,72.86,'Intermediate' from dual union all
select 111,'000111','58 Theta','Pioneer 3','Pioneer 3',4798,'Reentered','1958-12-07 00:00:00','1958-12-06 00:00:00',2283.58,69,102200,28.5,'Suborbital' from dual union all
select 1110,'001110','1965-012N','deb Kosmos-57','',11762,'Reentered','1965-02-27 00:00:00','1965-02-26 00:00:00',88.01,141,211,64.7,'Intermediate' from dual union all
select 11100,'011100','1977-097AG','Garbage bag','',42225,'Reentered','1979-02-16 00:00:00','1978-11-07 00:00:00',91.62,353,355,51.63,'Intermediate' from dual union all
select 11101,'011101','1978-103A','Einstein Observatory','HEAO B',45298,'Reentered','1982-03-25 00:00:00','1978-12-14 00:00:00',95.24,520,541,23.5,'Equatorial' from dual union all
select 11102,'011102','1978-103B','Centaur AC-52','Centaur D-1AR AC-52',45298,'Reentered','1979-03-13 00:00:00','1978-12-14 00:00:00',92.8,373,451,23.5,'Equatorial' from dual union all
select 11103,'011103','1978-102C','KBE part','KBE part',45297,'Reentered','1978-11-14 00:00:00','1978-11-13 00:00:00',89.22,193,279,72.86,'Intermediate' from dual union all
select 11104,'011104','1978-102D','KBE part','KBE part',45297,'Reentered','1978-11-15 00:00:00','1978-11-13 00:00:00',91.41,207,480,72.9,'Intermediate' from dual union all
select 11105,'011105','1978-102E','KB electropitaniya','KBE',45297,'Reentered','1978-11-14 00:00:00','1978-11-13 00:00:00',89.49,189,310,73.07,'Intermediate' from dual union all
select 11106,'011106','1978-102F','KBE part','KBE part',45297,'Reentered','1978-11-14 00:00:00','1978-11-13 00:00:00',88.81,179,253,72.86,'Intermediate' from dual union all
select 11107,'011107','1978-102G','KBE part','KBE part',45297,'Reentered','1978-11-14 00:00:00','1978-11-13 00:00:00',88.8,179,252,72.89,'Intermediate' from dual union all
select 11108,'011108','1978-104A','Kosmos-1047','Zenit-4MKM',45299,'Landed','1978-11-28 00:00:00','1978-11-18 00:00:00',89.63,169,344,72.86,'Intermediate' from dual union all
select 11109,'011109','1978-104B','11S510','11S510',45299,'Reentered','1978-11-19 00:00:00','1978-11-17 00:00:00',89.1,164,296,72.85,'Intermediate' from dual union all
select 1111,'001111','1965-012P','deb Kosmos-57','',11762,'Reentered','1965-03-02 00:00:00','1965-02-25 00:00:00',89.23,158,315,64.95,'Intermediate' from dual union all
select 11110,'011110','1978-099C','Magion','S2 Magion',45294,'Reentered','1981-09-11 00:00:00','1978-11-24 00:00:00',96.28,402,758,82.95,'Intermediate' from dual union all
select 11111,'011111','1978-105A','Kosmos-1048','Strela-2M',45300,'In Earth orbit','','1991-05-08 00:00:00',100.6,773,801,74.03,'Intermediate' from dual union all
select 11112,'011112','1978-105B','S3M','S3M',45300,'In Earth orbit','','1990-03-25 00:00:00',100.53,757,811,74.03,'Intermediate' from dual union all
select 11113,'011113','1978-105C','deb Kosmos-1048','Strela-2M debris',45300,'In Earth orbit','','1993-08-16 00:00:00',99.82,741,758,74.04,'Intermediate' from dual union all
select 11114,'011114','1978-105D','deb Kosmos-1048','Strela-2M debris',45300,'In Earth orbit','','1993-08-17 00:00:00',99.59,730,748,74.03,'Intermediate' from dual union all
select 11115,'011115','1978-106A','NATO 3C','NATO 3C',45301,'In Earth orbit','','1978-12-19 00:00:00',1428.4,35459,35813,4.36,'Inclined Drift' from dual union all
select 11116,'011116','1978-106B','Delta 146','Delta SSPU DSV-3P-4',45301,'Reentered','1978-12-12 00:00:00','1978-11-26 00:00:00',92.89,176,657,28.24,'Intermediate' from dual union all
select 11117,'011117','1978-106C','Star 37E S/N 40078','Star 37E',45301,'Reentered','1982-11-28 00:00:00','1982-11-28 00:00:00',87.8,124,208,26.85,'Intermediate' from dual union all
select 11118,'011118','1978-107A','Kosmos-1049','Zenit-4MKM',45302,'Landed','1978-12-04 00:00:00','1978-11-24 00:00:00',89.66,168,348,72.87,'Intermediate' from dual union all
select 11119,'011119','1978-107B','11S510','11S510',45302,'Reentered','1978-11-26 00:00:00','1978-11-23 00:00:00',89.12,165,297,72.86,'Intermediate' from dual union all
select 1112,'001112','1965-012Q','deb Kosmos-57','',11762,'Reentered','1965-03-01 00:00:00','1965-02-26 00:00:00',88.06,110,246,64.6,'Intermediate' from dual union all
select 11120,'011120','1978-104C','KDU','KDU',45299,'Reentered','1978-12-02 00:00:00','1978-11-27 00:00:00',89.48,162,336,72.85,'Intermediate' from dual union all
select 11121,'011121','1978-108A','Kosmos-1050','Zenit-6',45303,'Landed','1978-12-12 00:00:00','1978-11-30 00:00:00',89.79,253,276,62.81,'Intermediate' from dual union all
select 11122,'011122','1978-108B','11S510','11S510',45303,'Reentered','1978-12-08 00:00:00','1978-12-01 00:00:00',89.58,244,264,62.8,'Intermediate' from dual union all
select 11123,'011123','1978-108C','Sensor cover','Zenit-6 sensor cover',45303,'Reentered','1979-01-02 00:00:00','1978-11-29 00:00:00',89.81,245,285,62.84,'Intermediate' from dual union all
select 11124,'011124','1978-108D','Sensor cover','Zenit-6 sensor cover',45303,'Reentered','1979-01-02 00:00:00','1978-11-29 00:00:00',89.87,247,289,62.79,'Intermediate' from dual union all
select 11125,'011125','1978-107C','KDU','KDU',45302,'Reentered','1978-12-10 00:00:00','1978-12-03 00:00:00',89.04,167,288,72.86,'Intermediate' from dual union all
select 11126,'011126','1978-107D','Blenda','Sunshade',45302,'Reentered','1978-12-08 00:00:00','1978-12-03 00:00:00',89.4,167,323,72.86,'Intermediate' from dual union all
select 11127,'011127','1978-107E','Garbage bag','Spurious ID',45302,'Reentered','1978-12-11 00:00:00','1978-12-03 00:00:00',91.75,357,363,51.64,'Intermediate' from dual union all
select 11128,'011128','1978-109A','Kosmos-1051','Strela-1M',45304,'In Earth orbit','','1992-05-07 00:00:00',114.66,1394,1486,74.01,'Intermediate' from dual union all
select 11129,'011129','1978-109B','Kosmos-1052','Strela-1M',45304,'In Earth orbit','','1992-05-02 00:00:00',114.85,1410,1488,74.01,'Intermediate' from dual union all
select 1113,'001113','1965-012R','deb Kosmos-57','',11762,'Reentered','1965-02-25 00:00:00','1965-02-24 00:00:00',88.35,160,225,64.64,'Intermediate' from dual union all
select 11130,'011130','1978-109C','Kosmos-1053','Strela-1M',45304,'In Earth orbit','','1992-07-18 00:00:00',115.06,1429,1488,74.01,'Intermediate' from dual union all
select 11131,'011131','1978-109D','Kosmos-1054','Strela-1M',45304,'In Earth orbit','','1992-05-06 00:00:00',115.26,1446,1489,74.02,'Intermediate' from dual union all
select 11132,'011132','1978-109E','Kosmos-1055','Strela-1M',45304,'In Earth orbit','','1992-05-07 00:00:00',115.47,1464,1490,74.02,'Intermediate' from dual union all
select 11133,'011133','1978-109F','Kosmos-1056','Strela-1M',45304,'In Earth orbit','','1992-05-07 00:00:00',115.7,1472,1502,74.01,'Intermediate' from dual union all
select 11134,'011134','1978-109G','Kosmos-1057','Strela-1M',45304,'In Earth orbit','','1992-05-02 00:00:00',115.93,1480,1516,74.01,'Intermediate' from dual union all
select 11135,'011135','1978-109H','Kosmos-1058','Strela-1M',45304,'In Earth orbit','','1988-03-01 00:00:00',116.18,1480,1538,74.02,'Intermediate' from dual union all
select 11136,'011136','1978-109J','S3M','S3M',45304,'In Earth orbit','','1991-02-16 00:00:00',118.11,1492,1699,74.01,'Intermediate' from dual union all
select 11137,'011137','1978-110A','Kosmos-1059','Zenit-4MKM',45305,'Landed','1978-12-20 00:00:00','1978-12-09 00:00:00',89.34,171,313,62.81,'Intermediate' from dual union all
select 11138,'011138','1978-110B','11S510','11S510',45305,'Reentered','1978-12-11 00:00:00','1978-12-10 00:00:00',88.74,158,267,62.8,'Intermediate' from dual union all
select 11139,'011139','1978-111A','Kosmos-1060','Zenit-2M',45306,'Landed','1978-12-21 00:00:00','1978-12-12 00:00:00',89.36,201,285,65.03,'Intermediate' from dual union all
select 1114,'001114','1965-012S','deb Kosmos-57','',11762,'Reentered','1965-03-03 00:00:00','1965-02-26 00:00:00',88.2,106,265,64.67,'Intermediate' from dual union all
select 11140,'011140','1978-111B','11S510','11S510',45306,'Reentered','1978-12-13 00:00:00','1978-12-10 00:00:00',88.97,200,248,65.04,'Intermediate' from dual union all
select 11141,'011141','1978-112A','Navstar 4','Navstar GPS SVN 4',45307,'In Earth orbit','','1979-02-01 00:00:00',717.96,20163,20201,63.28,'Medium' from dual union all
select 11142,'011142','1978-112B','SVS Stage 2','Star 37E S/N 40071',45307,'In Earth orbit','','1979-02-01 00:00:00',328.93,167,18646,63.06,'Highly Ellip' from dual union all
select 11143,'011143','1978-108E','KDU','KDU',45303,'Reentered','1978-12-15 00:00:00','1978-12-11 00:00:00',89.23,160,313,62.81,'Intermediate' from dual union all
select 11144,'011144','1978-113A','DSCS II F-11','DSCS II C-11',45308,'In Earth orbit','','1979-02-13 00:00:00',1435.96,35776,35792,2.31,'Stationary' from dual union all
select 11145,'011145','1978-113B','DSCS II F-12','DSCS II C-12',45308,'In Earth orbit','','1979-02-13 00:00:00',1436.09,35783,35790,2.32,'Stationary' from dual union all
select 11146,'011146','1978-113C','Titan 23C-18 Stage 2','LR91AJ11 23C-18',45308,'Reentered','1978-12-14 00:00:00','1978-12-14 00:00:00',88.73,148,276,28.59,'Intermediate' from dual union all
select 11147,'011147','1978-113D','Transtage 36','Transtage 36',45308,'In Earth orbit','','1983-12-14 00:00:00',1533.33,36445,38893,2.18,'Very High' from dual union all
select 11148,'011148','1978-114A','Kosmos-1061','Zenit-2M',45309,'Landed','1978-12-27 00:00:00','1978-12-18 00:00:00',89.56,200,306,62.81,'Intermediate' from dual union all
select 11149,'011149','1978-114B','11S510','11S510',45309,'Reentered','1978-12-20 00:00:00','1978-12-17 00:00:00',88.99,188,262,62.8,'Intermediate' from dual union all
select 1115,'001115','1965-012T','deb Kosmos-57','',11762,'Reentered','1965-02-27 00:00:00','1965-02-24 00:00:00',89.19,130,338,64.83,'Intermediate' from dual union all
select 11150,'011150','1978-115A','Kosmos-1062','Tselina-OM',45310,'Reentered','1981-04-20 00:00:00','1979-01-15 00:00:00',95.09,501,544,74.05,'Intermediate' from dual union all
select 11151,'011151','1978-115B','S3M','S3M',45310,'Reentered','1981-06-20 00:00:00','1979-01-14 00:00:00',95,492,546,74.05,'Intermediate' from dual union all
select 11152,'011152','1978-115C','deb Kosmos-1062','Tselina-O part',45310,'Reentered','1979-03-07 00:00:00','1979-01-15 00:00:00',94.45,468,515,74.02,'Intermediate' from dual union all
select 11153,'011153','1978-116A','Anik B','Anik B',45311,'In Earth orbit','','1979-01-31 00:00:00',1436.04,35781,35790,.03,'Stationary' from dual union all
select 11154,'011154','1978-116B','Delta 147','Delta SSPU DSV-3P-4/DRIM',45311,'Reentered','1979-04-23 00:00:00','1979-01-17 00:00:00',105.25,181,1832,28.37,'Intermediate' from dual union all
select 11155,'011155','1978-117A','Kosmos-1063','Tselina-D',45312,'Reentered','2001-11-25 00:00:00','1991-03-28 00:00:00',95.76,555,556,81.2,'Intermediate' from dual union all
select 11156,'011156','1978-117B','8A92ME','8A92ME',45312,'Reentered','2002-11-20 00:00:00','1991-03-27 00:00:00',95.82,524,592,81.23,'Intermediate' from dual union all
select 11157,'011157','1978-110C','KDU','KDU',45305,'Reentered','1978-12-23 00:00:00','1978-12-19 00:00:00',89.7,166,354,62.82,'Intermediate' from dual union all
select 11158,'011158','1978-118A','Gorizont','Gorizont No. 11L',45313,'In Earth orbit','','1979-02-02 00:00:00',1436.01,22616,48954,11.26,'Synchronous' from dual union all
select 11159,'011159','1978-118B','8S812','8S812',45313,'Reentered','1978-12-21 00:00:00','1978-12-20 00:00:00',88.12,173,189,51.63,'Intermediate' from dual union all
select 1116,'001116','1965-012U','deb Kosmos-57','',11762,'Reentered','1965-03-04 00:00:00','1965-02-25 00:00:00',89.3,142,338,64.73,'Intermediate' from dual union all
select 11160,'011160','1978-110D','Blenda','Sunshade',45305,'Reentered','1978-12-23 00:00:00','1978-12-20 00:00:00',89.03,156,297,62.83,'Intermediate' from dual union all
select 11161,'011161','1978-119A','Kosmos-1064','Parus',45314,'Reentered','1989-11-12 00:00:00','1979-01-19 00:00:00',98.58,422,959,82.95,'Intermediate' from dual union all
select 11162,'011162','1978-119B','S3M','S3M',45314,'Reentered','1988-11-15 00:00:00','1987-10-11 00:00:00',93.23,352,514,82.9,'Intermediate' from dual union all
select 11163,'011163','1978-120A','Kosmos-1065','Romb 5',45315,'Reentered','1979-08-01 00:00:00','1979-01-21 00:00:00',93.34,341,535,50.68,'Intermediate' from dual union all
select 11164,'011164','1978-120B','S3M','S3M',45315,'Reentered','1979-08-18 00:00:00','1979-01-22 00:00:00',93.27,347,523,50.69,'Intermediate' from dual union all
select 11165,'011165','1978-121A','Kosmos-1066','Astrofizika 11F653 No. 1',45316,'In Earth orbit','','1992-06-13 00:00:00',102.09,821,895,81.24,'Intermediate' from dual union all
select 11166,'011166','1978-121B','8A92ME','8A92ME',45316,'In Earth orbit','','1993-08-17 00:00:00',101.95,803,899,81.24,'Intermediate' from dual union all
select 11167,'011167','1978-114C','Kapsula Kosmosa-1061','Nauka 32KS',45309,'Reentered','1979-01-04 00:00:00','1978-12-25 00:00:00',89.43,197,296,62.81,'Intermediate' from dual;
insert into SATELLITE
select 11168,'011168','1978-122A','Kosmos-1067','Geoid',45317,'In Earth orbit','','1992-06-07 00:00:00',109.08,1156,1212,82.97,'Intermediate' from dual union all
select 11169,'011169','1978-123A','Kosmos-1068','Zenit-4MKM',45318,'Landed','1979-01-08 00:00:00','1978-12-28 00:00:00',89.36,172,313,62.8,'Intermediate' from dual union all
select 1117,'001117','1965-012V','deb Kosmos-57','',11762,'Reentered','1965-03-02 00:00:00','1965-02-25 00:00:00',89.04,159,296,64.78,'Intermediate' from dual union all
select 11170,'011170','1978-122B','S3M','S3M',45317,'In Earth orbit','','1993-08-17 00:00:00',108.94,1159,1197,82.97,'Intermediate' from dual union all
select 11171,'011171','1978-123B','11S510','11S510',45318,'Reentered','1978-12-31 00:00:00','1978-12-28 00:00:00',89.67,168,349,62.78,'Intermediate' from dual union all
select 11172,'011172','1978-123C','Sensor cover','Zenit-4MK sensor cover',45318,'Reentered','1979-01-01 00:00:00','1978-12-27 00:00:00',90.13,176,386,62.8,'Intermediate' from dual union all
select 11173,'011173','1978-124A','Kosmos-1069','Orion',45319,'Landed','1979-01-10 00:00:00','1978-12-31 00:00:00',89.72,240,282,62.82,'Intermediate' from dual union all
select 11174,'011174','1978-124B','11S510','11S510',45319,'Reentered','1979-01-05 00:00:00','1978-12-30 00:00:00',89.57,230,277,62.81,'Intermediate' from dual union all
select 11175,'011175','1978-124C','Sensor cover','Zenit-4MT sensor cover',45319,'Reentered','1978-12-31 00:00:00','1978-12-29 00:00:00',89.99,250,297,62.78,'Intermediate' from dual union all
select 11176,'011176','1978-124D','Sensor cover','Zenit-4MT sensor cover',45319,'Reentered','1979-01-16 00:00:00','1978-12-29 00:00:00',89.78,238,290,62.82,'Intermediate' from dual union all
select 11177,'011177','1978-100E','deb S5M','',45295,'Reentered','1989-02-02 00:00:00','1984-11-19 00:00:00',114.82,1422,1473,74.02,'Intermediate' from dual union all
select 11178,'011178','1976-120D','deb Kosmos-880','',39072,'Reentered','1981-06-12 00:00:00','1981-06-10 00:00:00',88.56,196,211,65.81,'Intermediate' from dual union all
select 11179,'011179','1976-120E','deb Kosmos-880','',39072,'Reentered','1980-03-06 00:00:00','1980-02-27 00:00:00',90.96,313,331,65.81,'Intermediate' from dual union all
select 1118,'001118','1965-012W','deb Kosmos-57','',11762,'Reentered','1965-03-01 00:00:00','1965-02-27 00:00:00',88.53,147,257,64.95,'Intermediate' from dual union all
select 11180,'011180','1976-120F','deb Kosmos-880','',39072,'Reentered','1979-02-25 00:00:00','1979-01-22 00:00:00',96.04,529,609,65.84,'Intermediate' from dual union all
select 11181,'011181','1976-120G','deb Kosmos-880','',39072,'Reentered','1979-07-20 00:00:00','1979-06-27 00:00:00',91.83,356,373,65.8,'Intermediate' from dual union all
select 11182,'011182','1976-120H','deb Kosmos-880','',39072,'Reentered','1979-10-27 00:00:00','1979-08-16 00:00:00',93.89,455,475,65.79,'Intermediate' from dual union all
select 11183,'011183','1976-120J','deb Kosmos-880','',39072,'Reentered','1979-04-14 00:00:00','1979-03-10 00:00:00',92.7,402,412,65.82,'Intermediate' from dual union all
select 11184,'011184','1976-120K','deb Kosmos-880','',39072,'Reentered','1979-07-06 00:00:00','1979-06-27 00:00:00',91.68,336,378,65.81,'Intermediate' from dual union all
select 11185,'011185','1976-120L','deb Kosmos-880','',39072,'Reentered','1979-04-05 00:00:00','1979-03-26 00:00:00',91.06,315,339,65.83,'Intermediate' from dual union all
select 11186,'011186','1976-120M','deb Kosmos-880','',39072,'Reentered','1980-11-13 00:00:00','1980-11-02 00:00:00',89.04,220,235,65.79,'Intermediate' from dual union all
select 11187,'011187','1976-120N','deb Kosmos-880','',39072,'Reentered','1979-04-02 00:00:00','1979-03-20 00:00:00',92.48,383,410,65.84,'Intermediate' from dual union all
select 11188,'011188','1976-120P','deb Kosmos-880','',39072,'Reentered','1979-03-05 00:00:00','1979-02-25 00:00:00',92.4,373,411,65.84,'Intermediate' from dual union all
select 11189,'011189','1976-120Q','deb Kosmos-880','',39072,'Reentered','1979-02-23 00:00:00','1979-02-06 00:00:00',94.15,469,485,65.85,'Intermediate' from dual union all
select 1119,'001119','1965-012X','deb Kosmos-57','',11762,'Reentered','1965-03-01 00:00:00','1965-02-26 00:00:00',88.69,132,288,64.73,'Intermediate' from dual union all
select 11190,'011190','1976-120R','deb Kosmos-880','',39072,'Reentered','1979-03-26 00:00:00','1979-03-20 00:00:00',93.11,409,444,65.85,'Intermediate' from dual union all
select 11191,'011191','1976-120S','deb Kosmos-880','',39072,'Reentered','1979-06-16 00:00:00','1979-02-21 00:00:00',94.89,501,526,65.81,'Intermediate' from dual union all
select 11192,'011192','1976-120T','deb Kosmos-880','',39072,'Reentered','1979-07-05 00:00:00','1979-07-02 00:00:00',91.44,332,359,65.83,'Intermediate' from dual union all
select 11193,'011193','1976-120U','deb Kosmos-880','RCS 0.81',39072,'Reentered','1991-09-03 00:00:00','1991-09-01 00:00:00',87.62,150,164,65.79,'Intermediate' from dual union all
select 11194,'011194','1976-120V','deb Kosmos-880','',39072,'Reentered','1979-07-12 00:00:00','1979-06-09 00:00:00',92.89,403,430,65.78,'Intermediate' from dual union all
select 11195,'011195','1976-120W','deb Kosmos-880','',39072,'Reentered','1979-02-23 00:00:00','1979-02-14 00:00:00',93.23,430,436,65.85,'Intermediate' from dual union all
select 11196,'011196','1976-120X','deb Kosmos-880','',39072,'Reentered','1979-04-07 00:00:00','1979-03-09 00:00:00',93.93,448,485,65.84,'Intermediate' from dual union all
select 11197,'011197','1976-120Y','deb Kosmos-880','',39072,'Reentered','1979-03-31 00:00:00','1979-03-08 00:00:00',93.57,434,465,65.85,'Intermediate' from dual union all
select 11198,'011198','1976-120Z','deb Kosmos-880','',39072,'Reentered','1979-03-15 00:00:00','1979-03-05 00:00:00',93.13,414,442,65.83,'Intermediate' from dual union all
select 11199,'011199','1976-120AA','deb Kosmos-880','',39072,'Reentered','1979-02-18 00:00:00','1979-01-21 00:00:00',93.95,453,483,65.85,'Intermediate' from dual union all
select 112,'000112','59 Mu 1','AMS Luna-1','E-1 No. 4',5534,'Deep Space','1959-01-04 00:00:00','',null,null,null,null,'' from dual union all
select 1120,'001120','1965-012Y','deb Kosmos-57','',11762,'Reentered','1965-03-02 00:00:00','1965-02-27 00:00:00',89.65,198,317,64.74,'Intermediate' from dual union all
select 11200,'011200','1976-120AB','deb Kosmos-880','',39072,'Reentered','1979-07-10 00:00:00','1979-06-06 00:00:00',93.33,421,454,65.85,'Intermediate' from dual union all
select 11201,'011201','1976-120AC','deb Kosmos-880','',39072,'Reentered','1979-12-09 00:00:00','1979-12-07 00:00:00',90.17,276,290,65.82,'Intermediate' from dual union all
select 11202,'011202','1976-120AD','deb Kosmos-880','',39072,'Reentered','1979-07-13 00:00:00','1979-06-19 00:00:00',92.81,404,421,65.76,'Intermediate' from dual union all
select 11203,'011203','1976-120AE','deb Kosmos-880','',39072,'Reentered','1979-09-10 00:00:00','1979-08-15 00:00:00',91.72,352,366,65.84,'Intermediate' from dual union all
select 11204,'011204','1976-120AF','deb Kosmos-880','',39072,'Reentered','1991-08-01 00:00:00','1991-07-31 00:00:00',87.84,157,179,65.83,'Intermediate' from dual union all
select 11205,'011205','1976-120AG','deb Kosmos-880','',39072,'Reentered','1990-07-11 00:00:00','1990-07-11 00:00:00',87.48,141,158,65.8,'Intermediate' from dual union all
select 11206,'011206','1976-120AH','deb Kosmos-880','',39072,'Reentered','1979-02-26 00:00:00','1979-01-22 00:00:00',95.98,532,600,65.84,'Intermediate' from dual union all
select 11207,'011207','1976-120AJ','deb Kosmos-880','',39072,'Reentered','1985-11-20 00:00:00','1985-11-18 00:00:00',88.21,179,193,65.81,'Intermediate' from dual union all
select 11208,'011208','1976-120AK','deb Kosmos-880','',39072,'Reentered','1979-11-11 00:00:00','1979-10-27 00:00:00',92.68,398,413,65.8,'Intermediate' from dual union all
select 11209,'011209','1976-120AL','deb Kosmos-880','',39072,'Reentered','1982-12-01 00:00:00','1982-11-29 00:00:00',88.14,178,187,65.82,'Intermediate' from dual union all
select 1121,'001121','1965-012Z','deb Kosmos-57','',11762,'Reentered','1965-03-01 00:00:00','1965-02-27 00:00:00',88.51,186,216,65.2,'Intermediate' from dual union all
select 11210,'011210','1976-120AM','deb Kosmos-880','',39072,'Reentered','1983-07-30 00:00:00','1983-07-30 00:00:00',87.54,143,163,65.81,'Intermediate' from dual union all
select 11211,'011211','1976-120AN','deb Kosmos-880','',39072,'Reentered','1979-03-29 00:00:00','1979-03-26 00:00:00',91.33,328,352,65.83,'Intermediate' from dual union all
select 11212,'011212','1976-120AP','deb Kosmos-880','',39072,'Reentered','1984-07-15 00:00:00','1984-07-14 00:00:00',87.18,126,144,65.81,'Intermediate' from dual union all
select 11213,'011213','1976-120AQ','deb Kosmos-880','',39072,'Reentered','1990-11-23 00:00:00','1990-11-22 00:00:00',87.41,142,151,65.82,'Intermediate' from dual union all
select 11214,'011214','1976-120AR','deb Kosmos-880','',39072,'Reentered','1991-09-15 00:00:00','1991-09-15 00:00:00',87.6,149,163,65.85,'Intermediate' from dual union all
select 11215,'011215','1976-120AS','deb Kosmos-880','',39072,'Reentered','1979-05-24 00:00:00','1979-05-07 00:00:00',92.91,411,424,65.84,'Intermediate' from dual union all
select 11216,'011216','1976-120AT','deb Kosmos-880','',39072,'Reentered','2001-12-09 00:00:00','1993-08-05 00:00:00',94.88,468,558,65.85,'Intermediate' from dual union all
select 11217,'011217','1976-120AU','deb Kosmos-880','',39072,'Reentered','1989-12-24 00:00:00','1989-12-24 00:00:00',87.62,154,160,65.84,'Intermediate' from dual union all
select 11218,'011218','1976-120AV','deb Kosmos-880','',39072,'Reentered','1980-02-05 00:00:00','1980-01-30 00:00:00',89.36,235,251,65.82,'Intermediate' from dual union all
select 11219,'011219','1976-120AW','deb Kosmos-880','',39072,'Reentered','1982-11-19 00:00:00','1982-11-18 00:00:00',88.56,197,209,65.83,'Intermediate' from dual union all
select 1122,'001122','1965-012AA','deb Kosmos-57','',11762,'Reentered','1965-03-01 00:00:00','1965-02-26 00:00:00',89.01,117,335,64.68,'Intermediate' from dual union all
select 11220,'011220','1976-120AX','deb Kosmos-880','',39072,'Reentered','1983-05-19 00:00:00','1983-05-19 00:00:00',87.39,137,154,65.82,'Intermediate' from dual union all
select 11221,'011221','1976-120AY','deb Kosmos-880','',39072,'Reentered','2000-09-26 00:00:00','1993-08-04 00:00:00',94.4,451,527,65.84,'Intermediate' from dual union all
select 11222,'011222','1976-120AZ','deb Kosmos-880','',39072,'Reentered','1979-06-16 00:00:00','1979-03-16 00:00:00',95.57,512,581,65.81,'Intermediate' from dual union all
select 11223,'011223','1976-120BA','deb Kosmos-880','',39072,'Reentered','1991-08-14 00:00:00','1991-08-14 00:00:00',87.62,151,163,65.79,'Intermediate' from dual union all
select 11224,'011224','1976-120BB','deb Kosmos-880','',39072,'Reentered','1979-06-03 00:00:00','1979-05-19 00:00:00',92.26,382,388,65.88,'Intermediate' from dual union all
select 11225,'011225','1976-120BC','deb Kosmos-880','',39072,'Reentered','1979-08-19 00:00:00','1979-08-10 00:00:00',90.54,286,316,65.82,'Intermediate' from dual union all
select 11226,'011226','1978-123D','KDU','KDU',45318,'Reentered','1979-01-09 00:00:00','1979-01-06 00:00:00',89.59,164,344,62.8,'Intermediate' from dual union all
select 11227,'011227','1978-123E','Blenda','Sunshade',45318,'Reentered','1979-01-09 00:00:00','1979-01-06 00:00:00',89.37,165,322,62.8,'Intermediate' from dual union all
select 11228,'011228','1978-124E','KB electropitaniya','KBE',45319,'Reentered','1979-01-11 00:00:00','1979-01-10 00:00:00',89.12,219,243,62.82,'Intermediate' from dual union all
select 11229,'011229','1979-001A','Kosmos-1070','Zenit-2M',47485,'Landed','1979-01-20 00:00:00','1979-01-14 00:00:00',89.43,203,290,62.8,'Intermediate' from dual union all
select 1123,'001123','1965-012AB','deb Kosmos-57','',11762,'Reentered','1965-02-27 00:00:00','1965-02-26 00:00:00',87.48,86,214,64.84,'Intermediate' from dual union all
select 11230,'011230','1979-001B','11S510','11S510',47485,'Reentered','1979-01-16 00:00:00','1979-01-13 00:00:00',88.96,203,244,62.8,'Intermediate' from dual union all
select 11231,'011231','1979-001C','Sensor cover','Zenit-2M sensor cover',47485,'Reentered','1979-01-18 00:00:00','1979-01-12 00:00:00',88.82,192,240,62.81,'Intermediate' from dual union all
select 11232,'011232','1979-001D','Sensor cover','Zenit-2M sensor cover',47485,'Reentered','1979-01-19 00:00:00','1979-01-12 00:00:00',89.35,196,289,62.81,'Intermediate' from dual union all
select 11233,'011233','1979-002A','Kosmos-1071','Zenit-4MKM',47486,'Landed','1979-01-26 00:00:00','1979-01-15 00:00:00',89.65,179,336,62.81,'Intermediate' from dual union all
select 11234,'011234','1979-001E','Sensor cover','Zenit-2M sensor cover',47485,'Reentered','1979-01-22 00:00:00','1979-01-13 00:00:00',89.38,202,286,62.81,'Intermediate' from dual union all
select 11235,'011235','1979-002B','11S510','11S510',47486,'Reentered','1979-01-17 00:00:00','1979-01-15 00:00:00',89.13,171,292,62.79,'Intermediate' from dual union all
select 11236,'011236','1979-002C','Sensor cover','Zenit-4MK sensor cover',47486,'Reentered','1979-01-18 00:00:00','1979-01-15 00:00:00',89.06,165,291,62.81,'Intermediate' from dual union all
select 11237,'011237','1978-124F','KBE part','KBE part',45319,'Reentered','1979-01-22 00:00:00','1979-01-14 00:00:00',90.42,239,351,62.81,'Intermediate' from dual union all
select 11238,'011238','1979-003A','Kosmos-1072','Parus',47487,'In Earth orbit','','1992-06-18 00:00:00',104.84,959,1014,82.94,'Intermediate' from dual union all
select 11239,'011239','1979-003B','S3M','S3M',47487,'In Earth orbit','','1991-03-23 00:00:00',104.73,951,1013,82.94,'Intermediate' from dual union all
select 1124,'001124','1965-012AC','deb Kosmos-57','',11762,'Reentered','1965-03-02 00:00:00','1965-02-25 00:00:00',89.35,157,328,65,'Intermediate' from dual union all
select 11240,'011240','1979-004A','Molniya-3','Molniya-3 No. 23',47488,'Reentered','1998-12-02 00:00:00','1979-02-19 00:00:00',717.66,468,39881,62.84,'Molniya' from dual union all
select 11241,'011241','1979-004B','11S510','11S510',47488,'Reentered','1979-01-31 00:00:00','1979-01-31 00:00:00',88.3,165,215,62.76,'Intermediate' from dual union all
select 11242,'011242','1979-004C','BOZ','BOZ',47488,'Reentered','1979-01-29 00:00:00','1979-01-21 00:00:00',90.65,184,429,62.82,'Intermediate' from dual union all
select 11243,'011243','1978-120C','Kosmos-1065 SS 1','ESO',45315,'Reentered','1979-03-01 00:00:00','1979-01-21 00:00:00',93.2,337,526,50.67,'Intermediate' from dual union all
select 11244,'011244','1978-120D','Kosmos-1065 SS 2','ESO',45315,'Reentered','1979-03-02 00:00:00','1979-01-21 00:00:00',93.39,343,537,50.68,'Intermediate' from dual union all
select 11245,'011245','1979-001F','Kapsula Kosmosa-1070','Nauka 30KS',47485,'Reentered','1979-01-27 00:00:00','1979-01-19 00:00:00',89.29,199,280,62.81,'Intermediate' from dual union all
select 11246,'011246','1979-001G','Nauka part','',47485,'Reentered','1979-01-21 00:00:00','1979-01-19 00:00:00',89.17,185,281,62.81,'Intermediate' from dual union all
select 11247,'011247','1979-001H','Nauka part','',47485,'Reentered','1979-01-20 00:00:00','1979-01-19 00:00:00',89.07,192,264,62.8,'Intermediate' from dual union all
select 11248,'011248','1978-120E','Kosmos-1065 SS 3','ESO',45315,'Reentered','1979-03-03 00:00:00','1979-01-24 00:00:00',93.35,342,535,50.66,'Intermediate' from dual union all
select 11249,'011249','1978-120F','Kosmos-1065 SS 4','ESO',45315,'Reentered','1979-03-01 00:00:00','1979-01-23 00:00:00',93.25,337,531,50.69,'Intermediate' from dual union all
select 1125,'001125','1965-012AD','deb Kosmos-57','',11762,'Reentered','1965-03-02 00:00:00','1965-02-28 00:00:00',88.14,147,218,65.14,'Intermediate' from dual union all
select 11250,'011250','1979-002D','Blenda','Sunshade',47486,'Reentered','1979-01-27 00:00:00','1979-01-25 00:00:00',89.83,155,378,62.81,'Intermediate' from dual union all
select 11251,'011251','1979-005A','Meteor-Priroda','Meteor-Priroda No. 2-3',47489,'In Earth orbit','','1991-03-28 00:00:00',96.52,563,620,97.64,'Sun-Synch' from dual union all
select 11252,'011252','1979-005B','8A92ME','8A92ME',47489,'Reentered','2001-05-24 00:00:00','1990-02-13 00:00:00',95.86,551,570,97.47,'Sun-Synch' from dual union all
select 11253,'011253','1979-002E','KDU part','KDU part',47486,'Reentered','1979-01-27 00:00:00','1979-01-27 00:00:00',88.78,118,311,62.81,'Intermediate' from dual union all
select 11254,'011254','1979-002F','KDU','KDU',47486,'Reentered','1979-01-29 00:00:00','1979-01-26 00:00:00',91.45,267,425,62.88,'Intermediate' from dual union all
select 11255,'011255','1979-006A','Kosmos-1073','Zenit-4MKM',47490,'Landed','1979-02-12 00:00:00','1979-02-01 00:00:00',89.51,178,322,62.8,'Intermediate' from dual union all
select 11256,'011256','1979-007A','SCATHA','P78-2',47491,'In Earth orbit','','1979-03-01 00:00:00',1416.18,27580,43211,7.85,'Near-synch' from dual union all
select 11257,'011257','1979-006B','11S510','11S510',47490,'Reentered','1979-02-03 00:00:00','1979-02-01 00:00:00',89.02,168,284,62.79,'Intermediate' from dual;
insert into SATELLITE
select 11258,'011258','1979-007B','Delta 148','Delta SSPU DSV-3P-4',47491,'Reentered','1979-02-25 00:00:00','1979-02-06 00:00:00',94.04,176,769,28.63,'Intermediate' from dual union all
select 11259,'011259','1979-008A','Kosmos-1074','Soyuz 7K-ST No. 5L',47492,'Landed','1979-04-01 00:00:00','1979-03-02 00:00:00',91.86,358,373,51.65,'Intermediate' from dual union all
select 1126,'001126','1965-012AE','deb Kosmos-57','',11762,'Reentered','1965-03-01 00:00:00','1965-02-22 00:00:00',88.99,150,299,65,'Intermediate' from dual union all
select 11260,'011260','1979-008B','11S510','11S510',47492,'Reentered','1979-02-02 00:00:00','1979-02-01 00:00:00',88.52,186,217,51.61,'Intermediate' from dual union all
select 11261,'011261','1979-009A','Ayame','ECS',47493,'In Earth orbit','','1979-07-02 00:00:00',1351.27,31411,36808,.53,'Medium' from dual union all
select 11262,'011262','1979-010A','Kosmos-1075','Vektor',47494,'Reentered','1981-10-19 00:00:00','1979-03-10 00:00:00',94.43,468,514,65.83,'Intermediate' from dual union all
select 11263,'011263','1979-010B','S3M','S3M',47494,'Reentered','1980-08-29 00:00:00','1979-03-10 00:00:00',94.3,457,513,65.82,'Intermediate' from dual union all
select 11264,'011264','1979-006C','KDU','KDU',47490,'Reentered','1979-02-14 00:00:00','1979-02-11 00:00:00',89.75,153,372,62.81,'Intermediate' from dual union all
select 11265,'011265','1979-006D','Blenda','Sunshade',47490,'Reentered','1979-02-13 00:00:00','1979-02-11 00:00:00',89.32,152,330,62.8,'Intermediate' from dual union all
select 11266,'011266','1979-011A','Kosmos-1076','Okean-E NKh No. 1',47495,'Reentered','2001-08-10 00:00:00','1990-07-05 00:00:00',96.19,567,586,82.52,'Intermediate' from dual union all
select 11267,'011267','1979-011B','S5M','S5M',47495,'In Earth orbit','','1987-10-13 00:00:00',97.42,620,651,82.53,'Intermediate' from dual union all
select 11268,'011268','1979-012A','Kosmos-1077','Tselina-D',47496,'Reentered','2000-06-26 00:00:00','1991-03-27 00:00:00',95.41,535,541,81.22,'Intermediate' from dual union all
select 11269,'011269','1979-012B','8A92ME','8A92ME',47496,'Reentered','2002-02-14 00:00:00','1991-03-28 00:00:00',95.67,517,585,81.22,'Intermediate' from dual union all
select 1127,'001127','1965-012AF','deb Kosmos-57','',11762,'Reentered','1965-02-28 00:00:00','1965-02-22 00:00:00',88.99,150,299,65,'Intermediate' from dual union all
select 11270,'011270','1979-013A','SAGE','AEM-B',47497,'Reentered','1989-04-11 00:00:00','1979-03-20 00:00:00',96.74,547,659,54.93,'Intermediate' from dual union all
select 11271,'011271','1979-013B','Star 20 E-27','Star 20 E-27',47497,'Reentered','1981-09-19 00:00:00','1979-03-20 00:00:00',96.7,546,656,54.93,'Intermediate' from dual union all
select 11272,'011272','1979-014A','Hakucho','CORSA-b',47498,'Reentered','1985-04-15 00:00:00','1979-03-23 00:00:00',95.74,540,568,29.89,'Intermediate' from dual union all
select 11273,'011273','1979-015A','Ekran','Ekran No. 16L',47499,'In Earth orbit','','1979-04-20 00:00:00',1436.16,35633,35943,.28,'Stationary' from dual union all
select 11274,'011274','1979-015B','8S812','8S812',47499,'Reentered','1979-02-22 00:00:00','1979-02-21 00:00:00',88.2,180,190,51.6,'Intermediate' from dual union all
select 11275,'011275','1979-015C','Perekhodnik','Sredniy perekhodnik',47499,'Reentered','1979-02-22 00:00:00','1979-02-21 00:00:00',88.11,174,188,51.62,'Intermediate' from dual union all
select 11276,'011276','1979-016A','Kosmos-1078','Zenit-4MKM',47500,'Landed','1979-03-02 00:00:00','1979-02-24 00:00:00',88.73,164,260,72.86,'Intermediate' from dual union all
select 11277,'011277','1979-016B','11S510','11S510',47500,'Reentered','1979-02-24 00:00:00','1979-02-23 00:00:00',88.63,163,251,72.88,'Intermediate' from dual union all
select 11278,'011278','1979-017A','P78-1','P78-1',47501,'Reentered','1992-07-20 00:00:00','1979-04-14 00:00:00',96.26,558,600,97.66,'Sun-Synch' from dual union all
select 11279,'011279','1979-017B','OIS','Star 27 TE-M-616-11 S/N',47501,'Reentered','1983-10-11 00:00:00','1979-04-14 00:00:00',96.19,555,598,97.66,'Sun-Synch' from dual union all
select 1128,'001128','1965-012AG','deb Kosmos-57','',11762,'Reentered','1965-03-05 00:00:00','1965-02-22 00:00:00',91.72,354,363,65,'Intermediate' from dual union all
select 11280,'011280','1979-014B','M-3A','M-3A',47498,'Reentered','1982-02-08 00:00:00','1979-03-25 00:00:00',95.74,539,570,29.9,'Intermediate' from dual union all
select 11281,'011281','1979-018A','Soyuz-32','Soyuz 7K-T No. 48',47502,'Landed','1979-06-13 00:00:00','1979-03-28 00:00:00',90.37,279,307,51.63,'Intermediate' from dual union all
select 11282,'011282','1979-018B','11S510','11S510',47502,'Reentered','1979-02-27 00:00:00','1979-02-26 00:00:00',88.65,183,233,51.6,'Intermediate' from dual union all
select 11283,'011283','1979-019A','Kosmos-1079','Feniks No. 15',47503,'Landed','1979-03-11 00:00:00','1979-03-01 00:00:00',89.43,170,322,67.14,'Intermediate' from dual union all
select 11284,'011284','1979-019B','11S510','11S510',47503,'Reentered','1979-03-02 00:00:00','1979-03-01 00:00:00',88.63,159,255,67.12,'Intermediate' from dual union all
select 11285,'011285','1979-020A','Interkosmos-19','AUOS-Z Ionozond-IK',47504,'Reentered','2002-09-23 00:00:00','1990-07-05 00:00:00',97.55,474,809,73.97,'Intermediate' from dual union all
select 11286,'011286','1979-020B','S3M','S3M',47504,'Reentered','2003-08-06 00:00:00','1988-06-12 00:00:00',98.43,481,886,73.99,'Intermediate' from dual union all
select 11287,'011287','1979-016C','KDU','KDU',47500,'Reentered','1979-03-02 00:00:00','1979-03-01 00:00:00',87.99,147,203,72.86,'Intermediate' from dual union all
select 11288,'011288','1979-021A','Meteor-2','Meteor-2 No. 4',47505,'In Earth orbit','','1992-06-19 00:00:00',102.07,841,873,81.22,'Intermediate' from dual union all
select 11289,'011289','1979-021B','8A92ME','8A92ME',47505,'In Earth orbit','','1993-08-16 00:00:00',102.09,802,914,81.24,'Intermediate' from dual union all
select 1129,'001129','1965-012AH','deb Kosmos-57','',11762,'Reentered','1965-03-05 00:00:00','1965-02-22 00:00:00',91.6,349,358,65,'Intermediate' from dual union all
select 11290,'011290','1979-021C','deb Meteor-2','',47505,'In Earth orbit','','1993-08-16 00:00:00',102.14,836,884,81.2,'Intermediate' from dual union all
select 11291,'011291','1979-017C','Despin weight','',47501,'Reentered','1979-07-29 00:00:00','1979-03-26 00:00:00',95.07,460,584,97.23,'Sun-Synch' from dual union all
select 11292,'011292','1979-022A','Progress-5','Progress 7K-TG No. 104',47506,'Deorbited','1979-04-05 00:00:00','1979-03-31 00:00:00',90.68,278,338,51.62,'Intermediate' from dual union all
select 11293,'011293','1979-022B','11S510','11S510',47506,'Reentered','1979-03-14 00:00:00','1979-03-13 00:00:00',88.17,165,202,51.62,'Intermediate' from dual union all
select 11294,'011294','1979-023A','Kosmos-1080','Zenit-4MKM',47507,'Landed','1979-03-28 00:00:00','1979-03-16 00:00:00',88.53,166,238,72.85,'Intermediate' from dual union all
select 11295,'011295','1979-023B','11S510','11S510',47507,'Reentered','1979-03-16 00:00:00','1979-03-15 00:00:00',88.74,164,261,72.85,'Intermediate' from dual union all
select 11296,'011296','1979-024A','Kosmos-1081','Strela-1M',47508,'In Earth orbit','','1992-05-07 00:00:00',114.54,1404,1464,74.03,'Intermediate' from dual union all
select 11297,'011297','1979-024B','Kosmos-1082','Strela-1M',47508,'In Earth orbit','','1992-05-02 00:00:00',114.74,1423,1465,74.03,'Intermediate' from dual union all
select 11298,'011298','1979-024C','Kosmos-1083','Strela-1M',47508,'In Earth orbit','','1992-05-27 00:00:00',114.95,1442,1464,74.03,'Intermediate' from dual union all
select 11299,'011299','1979-024D','Kosmos-1084','Strela-1M',47508,'In Earth orbit','','1992-07-24 00:00:00',115.15,1461,1463,74.03,'Intermediate' from dual union all
select 113,'000113','59 Nu 1','Pioneer 4','Pioneer 4',5535,'Deep Space','1959-03-04 00:00:00','',null,null,null,null,'' from dual union all
select 1130,'001130','1965-012AJ','deb Kosmos-57','',11762,'Reentered','1965-03-11 00:00:00','1965-03-04 00:00:00',90.12,159,402,64.83,'Intermediate' from dual union all
select 11300,'011300','1979-024E','Kosmos-1085','Strela-1M',47508,'In Earth orbit','','1992-05-07 00:00:00',115.64,1465,1504,74.03,'Intermediate' from dual union all
select 11301,'011301','1979-024F','Kosmos-1086','Strela-1M',47508,'In Earth orbit','','1992-05-03 00:00:00',115.39,1465,1482,74.03,'Intermediate' from dual union all
select 11302,'011302','1979-024G','Kosmos-1087','Strela-1M',47508,'In Earth orbit','','1992-05-02 00:00:00',115.87,1466,1524,74.03,'Intermediate' from dual union all
select 11303,'011303','1979-024H','Kosmos-1088','Strela-1M',47508,'In Earth orbit','','1992-05-06 00:00:00',116.12,1466,1547,74.03,'Intermediate' from dual union all
select 11304,'011304','1979-024J','S3M','S3M',47508,'In Earth orbit','','1991-05-24 00:00:00',117.64,1460,1689,74.04,'Intermediate' from dual union all
select 11305,'011305','1979-025A','OPS 3854','HEXAGON SV-15',47509,'Deorbited','1979-09-22 00:00:00','1979-04-15 00:00:00',88.71,167,255,96.37,'Sun-Synch' from dual union all
select 11306,'011306','1979-025B','URSALA IV','P-11 No. 4431',47509,'Reentered','1992-01-01 00:00:00','1979-03-16 00:00:00',97.19,621,628,95.8,'Sun-Synch' from dual union all
select 11307,'011307','1979-025C','Titan 23D-21 Stage 2','LR91AJ11 23D-21',47509,'Reentered','1979-03-18 00:00:00','1979-03-17 00:00:00',88.63,161,252,96.4,'Sun-Synch' from dual union all
select 11308,'011308','1979-026A','Kosmos-1089','Parus',47510,'In Earth orbit','','1992-06-20 00:00:00',104.76,966,1001,82.97,'Intermediate' from dual union all
select 11309,'011309','1979-026B','S3M','S3M',47510,'In Earth orbit','','1991-03-25 00:00:00',104.64,965,991,82.97,'Intermediate' from dual union all
select 1131,'001131','1965-012AK','deb Kosmos-57','',11762,'Reentered','1965-03-07 00:00:00','1965-03-03 00:00:00',90.56,154,450,64.73,'Intermediate' from dual union all
select 11310,'011310','1979-023C','Blenda','Sunshade',47507,'Reentered','1979-03-27 00:00:00','1979-03-27 00:00:00',88.02,136,217,72.84,'Intermediate' from dual union all
select 11311,'011311','1979-023D','KDU part','KDU part',47507,'Reentered','1979-03-27 00:00:00','1979-03-27 00:00:00',87.62,108,206,72.82,'Intermediate' from dual union all
select 11312,'011312','1979-023E','KDU','KDU',47507,'Reentered','1979-03-28 00:00:00','1979-03-28 00:00:00',88.58,150,259,72.88,'Intermediate' from dual union all
select 11313,'011313','1979-027A','Kosmos-1090','Zenit-2M',47511,'Landed','1979-04-13 00:00:00','1979-04-02 00:00:00',89.74,200,323,72.85,'Intermediate' from dual union all
select 11314,'011314','1979-027B','11S510','11S510',47511,'Reentered','1979-04-05 00:00:00','1979-04-02 00:00:00',89.3,190,289,72.85,'Intermediate' from dual union all
select 11315,'011315','1979-008C','Kosmos-1074 BO','Bitovoy otsek',47492,'Reentered','1979-08-10 00:00:00','1979-03-31 00:00:00',91.37,325,359,51.65,'Intermediate' from dual union all
select 11316,'011316','1979-008D','Kosmos-1074 BO part','Soyuz BO part',47492,'Reentered','1979-06-06 00:00:00','1979-03-31 00:00:00',91.54,345,356,51.67,'Intermediate' from dual union all
select 11317,'011317','1979-008E','Kosmos-1074 BO part','Soyuz BO part',47492,'Reentered','1979-08-06 00:00:00','1979-03-31 00:00:00',90.84,303,329,51.66,'Intermediate' from dual union all
select 11318,'011318','1979-008F','Kosmos-1074 BO part','Soyuz BO part',47492,'Reentered','1979-05-30 00:00:00','1979-03-31 00:00:00',91.54,343,358,51.65,'Intermediate' from dual union all
select 11319,'011319','1979-008G','Kosmos-1074 BO part','Soyuz BO part',47492,'Reentered','1979-06-22 00:00:00','1979-04-02 00:00:00',91.57,345,359,51.66,'Intermediate' from dual union all
select 1132,'001132','1965-012AL','deb Kosmos-57','',11762,'Reentered','1965-03-07 00:00:00','1965-03-04 00:00:00',90.62,51,558,64.67,'ERROR - Unexpected value "TA"' from dual union all
select 11320,'011320','1979-028A','Kosmos-1091','Parus',47512,'In Earth orbit','','1992-06-20 00:00:00',104.79,963,1006,82.92,'Intermediate' from dual union all
select 11321,'011321','1979-028B','S3M','S3M',47512,'In Earth orbit','','1991-03-25 00:00:00',104.69,966,993,82.92,'Intermediate' from dual union all
select 11322,'011322','1979-017D','Despin weight','',47501,'Reentered','1982-02-27 00:00:00','1979-04-07 00:00:00',96.17,554,596,97.66,'Sun-Synch' from dual union all
select 11323,'011323','1979-020C','IK-19 instrument cover','',47504,'Reentered','1980-09-02 00:00:00','1979-04-09 00:00:00',99.02,469,955,74.11,'Intermediate' from dual union all
select 11324,'011324','1979-029A','Soyuz-33','Soyuz 7K-T No. 49',47513,'Landed','1979-04-12 00:00:00','1979-04-11 00:00:00',90.07,244,312,51.64,'Intermediate' from dual union all
select 11325,'011325','1979-029B','11S510','11S510',47513,'Reentered','1979-04-13 00:00:00','1979-04-12 00:00:00',87.64,156,160,51.61,'Intermediate' from dual union all
select 11326,'011326','1979-030A','Kosmos-1092','Tsikada',47514,'In Earth orbit','','1979-05-11 00:00:00',104.83,966,1006,82.95,'Intermediate' from dual union all
select 11327,'011327','1979-030B','S3M','S3M',47514,'In Earth orbit','','1991-03-28 00:00:00',104.67,958,1000,82.95,'Intermediate' from dual union all
select 11328,'011328','1979-031A','Molniya-1','Molniya-1',47515,'Reentered','1989-12-09 00:00:00','1979-05-27 00:00:00',717.7,645,39706,62.93,'Molniya' from dual union all
select 11329,'011329','1979-031B','11S510','11S510',47515,'Reentered','1979-05-08 00:00:00','1979-04-20 00:00:00',91.85,206,524,62.85,'Intermediate' from dual union all
select 1133,'001133','1965-012AM','deb Kosmos-57','',11762,'Reentered','1965-03-21 00:00:00','1965-03-02 00:00:00',91.04,179,473,64.55,'Intermediate' from dual union all
select 11330,'011330','1979-031C','BOZ','BOZ',47515,'Reentered','1979-05-05 00:00:00','1979-04-19 00:00:00',92.13,194,564,62.87,'Intermediate' from dual union all
select 11331,'011331','1979-032A','Kosmos-1093','Tselina-D',47516,'Reentered','2000-03-23 00:00:00','1990-03-25 00:00:00',95.71,550,556,81.24,'Intermediate' from dual union all
select 11332,'011332','1979-032B','8A92ME','8A92ME',47516,'Reentered','2009-05-09 00:00:00','1991-03-28 00:00:00',96.14,537,611,81.22,'Intermediate' from dual union all
select 11333,'011333','1979-033A','Kosmos-1094','US-P',47517,'Reentered','1979-11-07 00:00:00','1979-05-18 00:00:00',93.28,427,443,65.04,'Intermediate' from dual union all
select 11334,'011334','1979-033B','11S692','11S692',47517,'Reentered','1979-04-18 00:00:00','1979-04-18 00:00:00',89.34,103,381,65.04,'Intermediate' from dual union all
select 11335,'011335','1979-034A','Kosmos-1095','Zenit-6',47518,'Landed','1979-05-04 00:00:00','1979-04-22 00:00:00',92.26,352,419,72.85,'Intermediate' from dual union all
select 11336,'011336','1979-034B','11S510','11S510',47518,'Reentered','1979-04-27 00:00:00','1979-04-22 00:00:00',89.91,193,347,72.85,'Intermediate' from dual union all
select 11337,'011337','1979-034C','Sensor cover','Zenit-6 sensor cover',47518,'Reentered','1979-05-01 00:00:00','1979-04-22 00:00:00',90.09,216,342,72.9,'Intermediate' from dual union all
select 11338,'011338','1977-097AH','Garbage bag','',42225,'Reentered','1979-06-22 00:00:00','1979-04-22 00:00:00',91.43,334,355,51.63,'Intermediate' from dual union all
select 11339,'011339','1977-097AJ','Garbage bag','',42225,'Reentered','1979-05-25 00:00:00','1979-04-22 00:00:00',91.4,336,351,51.62,'Intermediate' from dual union all
select 1134,'001134','1965-012AN','deb Kosmos-57','',11762,'Reentered','1965-03-19 00:00:00','1965-03-03 00:00:00',91.38,154,531,64.77,'Intermediate' from dual union all
select 11340,'011340','1977-097AK','Garbage bag','',42225,'Reentered','1979-05-03 00:00:00','1979-04-22 00:00:00',91.26,327,345,51.63,'Intermediate' from dual union all
select 11341,'011341','1977-097AL','Garbage bag','',42225,'Reentered','1979-07-09 00:00:00','1979-04-22 00:00:00',91.29,333,343,51.64,'Intermediate' from dual union all
select 11342,'011342','1977-097AM','Garbage bag','',42225,'Reentered','1979-05-25 00:00:00','1979-04-22 00:00:00',91.31,326,352,51.62,'Intermediate' from dual union all
select 11343,'011343','1979-035A','Raduga','Gran'' No. 15L',47519,'In Earth orbit','','1979-06-04 00:00:00',1436.04,35750,35821,.34,'Stationary' from dual union all
select 11344,'011344','1979-035B','8S812','8S812',47519,'Reentered','1979-04-26 00:00:00','1979-04-25 00:00:00',88.03,173,181,51.62,'Intermediate' from dual union all
select 11345,'011345','1979-035C','Perekhodnik','Sredniy perekhodnik',47519,'Reentered','1979-04-25 00:00:00','1979-04-25 00:00:00',87.61,148,165,51.61,'Intermediate' from dual union all
select 11346,'011346','1979-036A','Kosmos-1096','US-P',47520,'Reentered','1979-11-24 00:00:00','1979-05-25 00:00:00',93.28,428,442,65.05,'Intermediate' from dual union all
select 11347,'011347','1979-036B','11S692','11S692',47520,'Reentered','1979-04-25 00:00:00','1979-04-25 00:00:00',89.31,112,369,65.02,'Intermediate' from dual;
insert into SATELLITE
select 11348,'011348','1979-037A','Kosmos-1097','Yantar''-4K1 No. 1',47521,'Landed','1979-05-27 00:00:00','1979-05-04 00:00:00',88.87,162,276,62.78,'Intermediate' from dual union all
select 11349,'011349','1979-037B','11S510','11S510',47521,'Reentered','1979-04-30 00:00:00','1979-04-29 00:00:00',88.82,164,268,62.77,'Intermediate' from dual union all
select 1135,'001135','1965-012AP','deb Kosmos-57','',11762,'Reentered','1965-03-13 00:00:00','1965-03-01 00:00:00',90.39,173,414,64.72,'Intermediate' from dual union all
select 11350,'011350','1977-097AN','Garbage bag','',42225,'Reentered','1979-07-17 00:00:00','1979-04-30 00:00:00',91.24,328,343,51.62,'Intermediate' from dual union all
select 11351,'011351','1979-034D','KDU','KDU',47518,'Reentered','1979-11-18 00:00:00','1979-05-21 00:00:00',92.11,349,407,72.85,'Intermediate' from dual union all
select 11352,'011352','1979-034E','Blenda','Sunshade',47518,'Reentered','1979-06-21 00:00:00','1979-05-27 00:00:00',91.24,319,351,72.85,'Intermediate' from dual union all
select 11353,'011353','1979-038A','FLTSATCOM F2','FLTSATCOM F2',47522,'In Earth orbit','','1979-08-29 00:00:00',1436.27,35665,35915,2.43,'Stationary' from dual union all
select 11354,'011354','1979-034F','KDU part','KDU part',47518,'Reentered','1979-07-10 00:00:00','1979-05-09 00:00:00',92.06,344,407,72.85,'Intermediate' from dual union all
select 11355,'011355','1979-034G','KDU part','KDU part',47518,'Reentered','1979-06-01 00:00:00','1979-05-08 00:00:00',92.09,345,409,72.84,'Intermediate' from dual union all
select 11356,'011356','1979-039A','Progress-6','Progress 7K-TG No. 106',47523,'Deorbited','1979-06-09 00:00:00','1979-05-21 00:00:00',91.08,319,337,51.63,'Intermediate' from dual union all
select 11357,'011357','1979-039B','11S510','11S510',47523,'Reentered','1979-05-15 00:00:00','1979-05-14 00:00:00',88.41,174,217,51.62,'Intermediate' from dual union all
select 11358,'011358','1979-040A','Kosmos-1098','Zenit-4MKM',47524,'Landed','1979-05-28 00:00:00','1979-05-20 00:00:00',89.57,167,340,72.86,'Intermediate' from dual union all
select 11359,'011359','1979-040B','11S510','11S510',47524,'Reentered','1979-05-19 00:00:00','1979-05-17 00:00:00',89,163,288,72.87,'Intermediate' from dual union all
select 1136,'001136','1965-012AQ','deb Kosmos-57','',11762,'Reentered','1965-03-17 00:00:00','1965-03-04 00:00:00',90.63,166,445,64.83,'Intermediate' from dual union all
select 11360,'011360','1979-041A','Kosmos-1099','Fram',47525,'Landed','1979-05-30 00:00:00','1979-05-19 00:00:00',88.9,208,233,81.35,'Intermediate' from dual union all
select 11361,'011361','1979-041B','11S510','11S510',47525,'Reentered','1979-05-21 00:00:00','1979-05-19 00:00:00',88.32,187,196,81.36,'Intermediate' from dual union all
select 11362,'011362','1979-042A','Kosmos-1100','TKS VA No. 0102A',47526,'Deorbited','1979-05-23 00:00:00','1979-05-23 00:00:00',88.64,193,222,51.58,'Intermediate' from dual union all
select 11363,'011363','1979-042B','Kosmos-1101','TKS VA No. 0102',47526,'Landed','1979-05-23 00:00:00','1979-05-22 00:00:00',88.59,190,220,51.6,'Intermediate' from dual union all
select 11364,'011364','1979-042C','8S812','8S812',47526,'Reentered','1979-05-26 00:00:00','1979-05-24 00:00:00',88.35,186,200,51.59,'Intermediate' from dual union all
select 11365,'011365','1979-042D','82LB72 LVI-4','TKS GVM',47526,'Reentered','1979-06-16 00:00:00','1979-05-23 00:00:00',88.59,190,220,51.59,'Intermediate' from dual union all
select 11366,'011366','1979-042E','TKS VA cover','TKS GVM cover',47526,'Reentered','1979-05-25 00:00:00','1979-05-24 00:00:00',87.93,172,172,51.58,'Intermediate' from dual union all
select 11367,'011367','1979-033C','deb Kosmos-1094','',47517,'Reentered','1979-06-17 00:00:00','1979-05-27 00:00:00',93.16,417,442,65.39,'Intermediate' from dual union all
select 11368,'011368','1979-043A','Kosmos-1102','Gektor-Priroda',47527,'Landed','1979-06-07 00:00:00','1979-05-29 00:00:00',89.12,208,254,81.34,'Intermediate' from dual union all
select 11369,'011369','1979-043B','11S510','11S510',47527,'Reentered','1979-05-29 00:00:00','1979-05-27 00:00:00',88.69,197,223,81.33,'Intermediate' from dual union all
select 1137,'001137','1965-012AR','deb Kosmos-57','',11762,'Reentered','1965-03-09 00:00:00','1965-02-28 00:00:00',91.07,181,473,64.89,'Intermediate' from dual union all
select 11370,'011370','1979-040C','KDU','KDU',47524,'Reentered','1979-05-30 00:00:00','1979-05-27 00:00:00',89.14,161,303,72.86,'Intermediate' from dual union all
select 11371,'011371','1979-041C','KDU','KDU',47525,'Reentered','1979-06-22 00:00:00','1979-05-28 00:00:00',88.52,195,208,81.34,'Intermediate' from dual union all
select 11372,'011372','1979-044A','OPS 7164','GAMBIT-3 Mission 4350',47528,'Reentered Attached','1979-08-26 00:00:00','1979-06-27 00:00:00',88.71,119,303,96.4,'Sun-Synch' from dual union all
select 11373,'011373','1979-041D','Blenda','Sunshade',47525,'Reentered','1979-06-21 00:00:00','1979-05-30 00:00:00',88.12,177,186,83.3,'Intermediate' from dual union all
select 11374,'011374','1979-041E','KDU part','KDU part',47525,'Reentered','1979-06-02 00:00:00','1979-05-29 00:00:00',88.4,187,204,81.33,'Intermediate' from dual union all
select 11375,'011375','1979-043C','Sensor cover','Zenit-2M sensor cover',47527,'Reentered','1979-06-06 00:00:00','1979-05-29 00:00:00',88.94,205,240,81.34,'Intermediate' from dual union all
select 11376,'011376','1979-045A','Kosmos-1103','Zenit-6',47529,'Landed','1979-06-14 00:00:00','1979-06-04 00:00:00',91.94,324,416,62.82,'Intermediate' from dual union all
select 11377,'011377','1979-045B','11S510','11S510',47529,'Reentered','1979-06-24 00:00:00','1979-06-07 00:00:00',90.39,245,342,62.8,'Intermediate' from dual union all
select 11378,'011378','1979-046A','Kosmos-1104','Parus',47530,'In Earth orbit','','1992-06-17 00:00:00',104.72,957,1006,82.95,'Intermediate' from dual union all
select 11379,'011379','1979-046B','S3M','S3M',47530,'In Earth orbit','','1991-03-27 00:00:00',104.61,960,993,82.94,'Intermediate' from dual union all
select 1138,'001138','1965-012AS','deb Kosmos-57','',11762,'Reentered','1965-03-08 00:00:00','1965-02-22 00:00:00',92.46,166,625,64.8,'Intermediate' from dual union all
select 11380,'011380','1979-045C','Sensor cover','Zenit-6 sensor cover',47529,'Reentered','1979-06-05 00:00:00','1979-06-01 00:00:00',90.39,252,335,62.79,'Intermediate' from dual union all
select 11381,'011381','1979-045D','Sensor cover','Zenit-6 sensor cover',47529,'Reentered','1979-06-08 00:00:00','1979-06-01 00:00:00',90.94,254,388,62.79,'Intermediate' from dual union all
select 11382,'011382','1979-047A','Ariel 6','Ariel 6',47531,'Reentered','1990-09-23 00:00:00','1979-07-15 00:00:00',97.22,599,653,55.03,'Intermediate' from dual union all
select 11383,'011383','1979-047B','Star 20 E-29','Star 20 E-29',47531,'Reentered','1988-04-10 00:00:00','1979-08-13 00:00:00',97.21,597,654,55.03,'Intermediate' from dual union all
select 11384,'011384','1979-048A','Molniya-3','Molniya-3 No. 21',47532,'Reentered','1992-09-26 00:00:00','1979-07-10 00:00:00',717.92,465,39896,62.85,'Molniya' from dual union all
select 11385,'011385','1979-048B','11S510','11S510',47532,'Reentered','1979-06-24 00:00:00','1979-06-13 00:00:00',90.52,201,399,62.85,'Intermediate' from dual union all
select 11386,'011386','1979-048C','BOZ','BOZ',47532,'Reentered','1979-06-21 00:00:00','1979-06-13 00:00:00',90.17,191,375,62.82,'Intermediate' from dual union all
select 11387,'011387','1979-049A','Soyuz-34','Soyuz 7K-T No. 50',47533,'Landed','1979-08-19 00:00:00','1979-07-06 00:00:00',91.57,349,355,51.62,'Intermediate' from dual union all
select 11388,'011388','1979-049B','11S510','11S510',47533,'Reentered','1979-06-09 00:00:00','1979-06-08 00:00:00',88.36,177,210,51.62,'Intermediate' from dual union all
select 11389,'011389','1979-050A','DMSP F-4','DMSP Block 5D S-5',47534,'In Earth orbit','','1992-10-31 00:00:00',101.05,803,814,98.41,'Sun-Synch' from dual union all
select 1139,'001139','1965-012AT','deb Kosmos-57','',11762,'Reentered','1965-03-10 00:00:00','1965-02-22 00:00:00',90.71,163,455,64.9,'Intermediate' from dual union all
select 11390,'011390','1979-043D','Kapsula Kosmosa-1102','Nauka 32KS',47527,'Reentered','1979-07-13 00:00:00','1979-06-09 00:00:00',87.9,162,180,81.34,'Intermediate' from dual union all
select 11391,'011391','1979-043E','Nauka part','',47527,'Reentered','1979-06-03 00:00:00','1979-06-02 00:00:00',88.75,195,231,81.34,'Intermediate' from dual union all
select 11392,'011392','1979-051A','Bhaskara','SEO',47535,'Reentered','1989-02-17 00:00:00','1979-07-12 00:00:00',95.21,518,540,50.66,'Intermediate' from dual union all
select 11393,'011393','1979-051B','S3M','S3M',47535,'Reentered','1982-02-17 00:00:00','1979-07-12 00:00:00',95.12,499,550,50.67,'Intermediate' from dual union all
select 11394,'011394','1979-052A','Kosmos-1105','Fram',47536,'Landed','1979-06-21 00:00:00','1979-06-10 00:00:00',88.88,208,231,81.35,'Intermediate' from dual union all
select 11395,'011395','1979-052B','11S510','11S510',47536,'Reentered','1979-06-12 00:00:00','1979-06-10 00:00:00',88.65,199,217,81.35,'Intermediate' from dual union all
select 11396,'011396','1979-052C','Sensor cover','Fram sensor cover',47536,'Reentered','1979-06-08 00:00:00','1979-06-08 00:00:00',88.98,193,256,81.33,'Intermediate' from dual union all
select 11397,'011397','1979-053A','DSP F8','DSP 11',47537,'In Earth orbit','','1979-06-11 00:00:00',1433.1,35671,35785,3,'Inclined Drift' from dual union all
select 11398,'011398','1979-053B','Titan 23C-13 Stage 2','LR91AJ11 23C-13',47537,'Reentered','1979-06-11 00:00:00','1979-06-10 00:00:00',88.72,152,271,28.62,'Intermediate' from dual union all
select 11399,'011399','1979-054A','Kosmos-1106','Gektor-Priroda',47538,'Landed','1979-06-25 00:00:00','1979-06-14 00:00:00',88.94,212,233,81.36,'Intermediate' from dual union all
select 114,'000114','59 Xi 1','AMS Luna-2','E-1 No. 7',5537,'Deep Space','1959-09-13 00:00:00','1959-09-12 00:00:00',15550.88,0,400000,65,'Cislunar' from dual union all
select 1140,'001140','1965-012AU','deb Kosmos-57','',11762,'Reentered','1965-03-25 00:00:00','1965-03-03 00:00:00',90.23,164,408,64.67,'Intermediate' from dual union all
select 11400,'011400','1979-054B','11S510','11S510',47538,'Reentered','1979-06-15 00:00:00','1979-06-14 00:00:00',88.48,189,210,81.36,'Intermediate' from dual union all
select 11401,'011401','1979-054C','Sensor cover','Zenit-2M sensor cover',47538,'Reentered','1979-08-07 00:00:00','1979-06-13 00:00:00',88.97,213,235,81.35,'Intermediate' from dual union all
select 11402,'011402','1979-045E','Blenda','Sunshade',47529,'Reentered','1979-07-27 00:00:00','1979-06-13 00:00:00',91.77,309,414,62.79,'Intermediate' from dual union all
select 11403,'011403','1979-050B','DMSP F-4 OLS Cooler Cover','',47534,'Reentered','1989-01-01 00:00:00','1979-07-15 00:00:00',101.39,802,847,98.78,'Sun-Synch' from dual union all
select 11404,'011404','1979-055A','Kosmos-1107','Zenit-6',47539,'Landed','1979-06-29 00:00:00','1979-06-18 00:00:00',89.62,225,287,72.86,'Intermediate' from dual union all
select 11405,'011405','1979-055B','11S510','11S510',47539,'Reentered','1979-06-20 00:00:00','1979-06-17 00:00:00',89.07,191,266,72.87,'Intermediate' from dual union all
select 11406,'011406','1979-045F','KDU part','KDU part',47529,'Reentered','1979-07-09 00:00:00','1979-06-17 00:00:00',91.76,315,406,62.81,'Intermediate' from dual union all
select 11407,'011407','1979-045G','KDU','KDU',47529,'Reentered','1979-11-15 00:00:00','1979-07-03 00:00:00',91.79,321,404,62.82,'Intermediate' from dual union all
select 11408,'011408','1979-050C','DMSP F-4 solar array band','',47534,'Reentered','1984-01-01 00:00:00','1979-07-01 00:00:00',101.5,810,850,98.78,'Sun-Synch' from dual union all
select 11409,'011409','1979-052D','KDU','KDU',47536,'Reentered','1979-07-28 00:00:00','1979-06-21 00:00:00',88.64,205,210,81.36,'Intermediate' from dual union all
select 1141,'001141','1965-012AV','deb Kosmos-57','',11762,'Reentered','1965-03-10 00:00:00','1965-03-01 00:00:00',91.56,173,529,64.81,'Intermediate' from dual union all
select 11410,'011410','1979-050D','DMSP F-4 OLS Cover','',47534,'Reentered','1989-01-01 00:00:00','1979-07-01 00:00:00',101.5,810,850,98.78,'Sun-Synch' from dual union all
select 11411,'011411','1979-050E','Radar error','[Dup of 11534]',47534,'Spurious','1982-06-08 00:00:00','1979-07-15 00:00:00',101.39,802,847,98.78,'Sun-Synch' from dual union all
select 11412,'011412','1979-050F','Radar error','[Dup of 11403 and 11411',47534,'Spurious','1982-02-23 00:00:00','1979-07-07 00:00:00',101.43,816,837,98.77,'Sun-Synch' from dual union all
select 11413,'011413','1979-056A','Kosmos-1108','Fram',47540,'Landed','1979-07-05 00:00:00','1979-06-26 00:00:00',88.85,209,227,81.33,'Intermediate' from dual union all
select 11414,'011414','1979-056B','11S510','11S510',47540,'Reentered','1979-06-25 00:00:00','1979-06-23 00:00:00',88.66,201,216,81.33,'Intermediate' from dual union all
select 11415,'011415','1979-054D','Kapsula Kosmosa-1106','Nauka 33KS',47538,'Reentered','1979-06-30 00:00:00','1979-06-25 00:00:00',88.5,197,204,81.36,'Intermediate' from dual union all
select 11416,'011416','1979-057A','NOAA 6','NOAA A',47541,'In Earth orbit','','1979-07-27 00:00:00',101.24,809,826,98.75,'Sun-Synch' from dual union all
select 11417,'011417','1979-058A','Kosmos-1109','Oko',47542,'In Earth orbit','','1979-07-27 00:00:00',717.48,665,39675,62.92,'Molniya' from dual union all
select 11418,'011418','1979-058B','11S510','11S510',47542,'Reentered','1979-07-26 00:00:00','1979-07-12 00:00:00',91.06,201,453,62.83,'Intermediate' from dual union all
select 11419,'011419','1979-057B','NOAA 6 solar array band','',47541,'Reentered','2003-03-16 00:00:00','1993-08-18 00:00:00',98.85,701,707,98.32,'Sun-Synch' from dual union all
select 1142,'001142','1965-012AW','deb Kosmos-57','',11762,'Reentered','1965-03-14 00:00:00','1965-03-05 00:00:00',90.25,148,426,64.81,'Intermediate' from dual union all
select 11420,'011420','1979-055C','Blenda','Sunshade',47539,'Reentered','1979-07-02 00:00:00','1979-06-28 00:00:00',89.51,160,341,72.86,'Intermediate' from dual union all
select 11421,'011421','1979-059A','Progress-7','Progress 7K-TG No. 107',47543,'Deorbited','1979-07-20 00:00:00','1979-07-06 00:00:00',91.21,323,345,51.62,'Intermediate' from dual union all
select 11422,'011422','1979-059B','11S510','11S510',47543,'Reentered','1979-06-30 00:00:00','1979-06-29 00:00:00',88.53,181,223,51.66,'Intermediate' from dual union all
select 11423,'011423','1979-058C','BOZ','BOZ',47542,'Reentered','1979-07-13 00:00:00','1979-07-08 00:00:00',90.32,160,421,62.85,'Intermediate' from dual union all
select 11424,'011424','1978-120G','Kosmos-1065 SS 5','ESO',45315,'Reentered','1979-07-16 00:00:00','1979-06-28 00:00:00',90.59,265,342,50.67,'Intermediate' from dual union all
select 11425,'011425','1979-060A','Kosmos-1110','Strela-2M',47544,'In Earth orbit','','1991-11-10 00:00:00',100.66,780,799,74.01,'Intermediate' from dual union all
select 11426,'011426','1978-120H','Kosmos-1065 SS 6','ESO',45315,'Reentered','1979-08-05 00:00:00','1979-06-28 00:00:00',90.76,276,348,50.69,'Intermediate' from dual union all
select 11427,'011427','1979-060B','S3M','S3M',47544,'In Earth orbit','','1991-03-27 00:00:00',100.49,766,798,74.02,'Intermediate' from dual union all
select 11428,'011428','1979-055D','KDU','KDU',47539,'Reentered','1979-07-03 00:00:00','1979-07-02 00:00:00',88.06,142,214,72.86,'Intermediate' from dual union all
select 11429,'011429','1979-061A','Kosmos-1111','Zenit-6',47545,'Landed','1979-07-14 00:00:00','1979-07-11 00:00:00',91.94,328,412,62.81,'Intermediate' from dual union all
select 1143,'001143','1965-012AX','deb Kosmos-57','',11762,'Reentered','1965-03-20 00:00:00','1965-03-04 00:00:00',91.74,167,553,64.64,'Intermediate' from dual union all
select 11430,'011430','1979-061B','11S510','11S510',47545,'Reentered','1979-07-20 00:00:00','1979-07-12 00:00:00',89.55,224,281,62.77,'Intermediate' from dual union all
select 11431,'011431','1977-097AP','Garbage bag','',42225,'Reentered','1979-09-04 00:00:00','1979-07-08 00:00:00',91.51,339,359,51.67,'Intermediate' from dual union all
select 11432,'011432','1977-097AQ','Garbage bag','',42225,'Reentered','1979-10-05 00:00:00','1979-07-02 00:00:00',91.68,353,360,51.63,'Intermediate' from dual union all
select 11433,'011433','1977-097AR','Garbage bag','',42225,'Reentered','1979-07-16 00:00:00','1979-07-01 00:00:00',91.72,355,362,51.62,'Intermediate' from dual union all
select 11434,'011434','1977-097AS','Garbage bag','',42225,'Reentered','1979-11-27 00:00:00','1979-07-02 00:00:00',91.73,356,362,51.67,'Intermediate' from dual union all
select 11435,'011435','1977-097AT','Garbage bag','',42225,'Reentered','1979-08-25 00:00:00','1979-07-01 00:00:00',91.68,355,359,51.62,'Intermediate' from dual union all
select 11436,'011436','1979-053C','Transtage 31','Transtage 31',47537,'In Earth orbit','','1979-08-01 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 11437,'011437','1979-056C','Blenda','Sunshade',47540,'Reentered','1979-07-08 00:00:00','1979-07-04 00:00:00',88.5,193,208,81.33,'Intermediate' from dual;
insert into SATELLITE
select 11438,'011438','1979-056D','KDU part','KDU part',47540,'Reentered','1979-07-11 00:00:00','1979-07-04 00:00:00',88.02,155,198,81.38,'Intermediate' from dual union all
select 11439,'011439','1979-056E','KDU','KDU',47540,'Reentered','1979-07-21 00:00:00','1979-07-05 00:00:00',88.39,172,218,81.28,'Intermediate' from dual union all
select 1144,'001144','1965-012AY','deb Kosmos-57','',11762,'Reentered','1965-03-21 00:00:00','1965-03-06 00:00:00',90.84,161,471,64.74,'Intermediate' from dual union all
select 11440,'011440','1979-062A','Gorizont','Gorizont No. 12L',47546,'In Earth orbit','','1979-08-05 00:00:00',1436.14,35763,35812,.78,'Stationary' from dual union all
select 11441,'011441','1979-062B','8S812','8S812',47546,'Reentered','1979-07-07 00:00:00','1979-07-07 00:00:00',87.39,141,150,51.62,'Intermediate' from dual union all
select 11442,'011442','1979-062C','Perekhodnik','Sredniy perekhodnik',47546,'Reentered','1979-07-07 00:00:00','1979-07-05 00:00:00',88.39,188,202,51.6,'Intermediate' from dual union all
select 11443,'011443','1979-063A','Kosmos-1112','Romb',47547,'Reentered','1980-01-21 00:00:00','1979-08-09 00:00:00',93.27,342,527,50.68,'Intermediate' from dual union all
select 11444,'011444','1979-063B','S3M','S3M',47547,'Reentered','1980-01-08 00:00:00','1979-08-08 00:00:00',93.13,333,523,50.69,'Intermediate' from dual union all
select 11445,'011445','1979-063C','Kosmos-1112 SS 1','ESO',47547,'Reentered','1979-07-08 00:00:00','1979-07-07 00:00:00',93.31,338,536,50.69,'Intermediate' from dual union all
select 11446,'011446','1979-063D','Kosmos-1112 SS 2','ESO',47547,'Reentered','1979-08-01 00:00:00','1979-07-14 00:00:00',92.64,307,501,50.67,'Intermediate' from dual union all
select 11447,'011447','1979-064A','Kosmos-1113','Zenit-4MKM',47548,'Landed','1979-07-23 00:00:00','1979-07-14 00:00:00',89.38,170,318,65,'Intermediate' from dual union all
select 11448,'011448','1979-064B','11S510','11S510',47548,'Reentered','1979-07-15 00:00:00','1979-07-13 00:00:00',88.41,157,234,65,'Intermediate' from dual union all
select 11449,'011449','1979-065A','Kosmos-1114','Tselina-OM',47549,'Reentered','1981-12-26 00:00:00','1979-08-12 00:00:00',95.15,503,548,74.04,'Intermediate' from dual union all
select 1145,'001145','1965-012AZ','deb Kosmos-57','',11762,'Reentered','1965-03-21 00:00:00','1965-03-03 00:00:00',90.59,169,438,64.7,'Intermediate' from dual union all
select 11450,'011450','1979-065B','S3M','S3M',47549,'Reentered','1982-02-05 00:00:00','1979-08-10 00:00:00',95,489,549,74.06,'Intermediate' from dual union all
select 11451,'011451','1979-066A','Kosmos-1115','Fram',47550,'Landed','1979-07-26 00:00:00','1979-07-15 00:00:00',88.89,213,227,81.36,'Intermediate' from dual union all
select 11452,'011452','1979-066B','11S510','11S510',47550,'Reentered','1979-07-17 00:00:00','1979-07-15 00:00:00',88.53,195,208,81.36,'Intermediate' from dual union all
select 11453,'011453','1979-061C','KDU','KDU',47545,'Reentered','1979-12-05 00:00:00','1979-07-29 00:00:00',91.84,327,403,62.81,'Intermediate' from dual union all
select 11454,'011454','1979-061D','Blenda','Sunshade',47545,'Reentered','1979-08-25 00:00:00','1979-07-13 00:00:00',92.4,310,475,62.72,'Intermediate' from dual union all
select 11455,'011455','1979-061E','KDU part','KDU part',47545,'Reentered','1979-07-30 00:00:00','1979-07-14 00:00:00',92.32,351,426,62.11,'Intermediate' from dual union all
select 11456,'011456','1979-061F','KDU part','KDU part',47545,'Reentered','1979-08-25 00:00:00','1979-07-14 00:00:00',92.04,359,391,62.66,'Intermediate' from dual union all
select 11457,'011457','1979-067A','Kosmos-1116','Tselina-D',47551,'Reentered','1993-03-11 00:00:00','1991-03-27 00:00:00',94.4,475,504,81.19,'Intermediate' from dual union all
select 11458,'011458','1979-067B','8A92ME','8A92ME',47551,'Reentered','2001-05-10 00:00:00','1990-03-14 00:00:00',95.81,526,589,81.2,'Intermediate' from dual union all
select 11459,'011459','1979-064C','Blenda','Sunshade',47548,'Reentered','1979-07-26 00:00:00','1979-07-21 00:00:00',89.51,165,336,64.99,'Intermediate' from dual union all
select 1146,'001146','1965-012BA','deb Kosmos-57','',11762,'Reentered','1965-03-17 00:00:00','1965-03-04 00:00:00',90.36,157,428,64.91,'Intermediate' from dual union all
select 11460,'011460','1979-064D','KDU','KDU',47548,'Reentered','1979-07-27 00:00:00','1979-07-22 00:00:00',89.45,174,321,65,'Intermediate' from dual union all
select 11461,'011461','1979-064E','KDU part','KDU part',47548,'Reentered','1979-07-25 00:00:00','1979-07-22 00:00:00',89.11,172,289,65,'Intermediate' from dual union all
select 11462,'011462','1979-066C','KDU','KDU',47550,'Reentered','1979-08-20 00:00:00','1979-07-24 00:00:00',88.78,214,215,81.36,'Intermediate' from dual union all
select 11463,'011463','1979-068A','Kosmos-1117','Zenit-4MKM',47552,'Landed','1979-08-07 00:00:00','1979-07-29 00:00:00',89.44,177,316,62.8,'Intermediate' from dual union all
select 11464,'011464','1979-068B','11S510','11S510',47552,'Reentered','1979-07-29 00:00:00','1979-07-27 00:00:00',89.08,166,291,62.75,'Intermediate' from dual union all
select 11465,'011465','1979-069A','Kosmos-1118','Gektor-Priroda',47553,'Landed','1979-08-09 00:00:00','1979-07-31 00:00:00',88.96,211,236,81.35,'Intermediate' from dual union all
select 11466,'011466','1979-069B','11S510','11S510',47553,'Reentered','1979-07-31 00:00:00','1979-07-29 00:00:00',88.61,195,217,81.35,'Intermediate' from dual union all
select 11467,'011467','1977-097AU','Garbage bag','',42225,'Reentered','1979-10-23 00:00:00','1979-08-01 00:00:00',92.53,391,405,51.62,'Intermediate' from dual union all
select 11468,'011468','1977-097AV','Garbage bag','',42225,'Reentered','1979-10-17 00:00:00','1979-08-01 00:00:00',92.51,390,405,51.63,'Intermediate' from dual union all
select 11469,'011469','1977-097AW','Garbage bag','',42225,'Reentered','1979-10-17 00:00:00','1979-07-30 00:00:00',92.4,384,401,51.64,'Intermediate' from dual union all
select 1147,'001147','1965-012BB','deb Kosmos-57','',11762,'Reentered','1965-03-17 00:00:00','1965-03-08 00:00:00',90.99,169,478,64.67,'Intermediate' from dual union all
select 11470,'011470','1977-097AX','Garbage bag','',42225,'Reentered','1979-10-23 00:00:00','1979-07-30 00:00:00',92.42,383,404,51.62,'Intermediate' from dual union all
select 11471,'011471','1977-097AY','Garbage bag','',42225,'Reentered','1979-10-15 00:00:00','1979-08-05 00:00:00',92.41,384,402,51.64,'Intermediate' from dual union all
select 11472,'011472','1977-097AZ','Garbage bag','',42225,'Reentered','1980-03-16 00:00:00','1979-08-02 00:00:00',92.55,393,406,51.62,'Intermediate' from dual union all
select 11473,'011473','1977-097BA','Garbage bag','',42225,'Reentered','1979-10-26 00:00:00','1979-08-11 00:00:00',92.51,360,436,51.64,'Intermediate' from dual union all
select 11474,'011474','1979-070A','Molniya-1','Molniya-1',47554,'Reentered','2017-10-23 00:00:00','1979-08-31 00:00:00',717.7,457,39894,62.86,'Molniya' from dual union all
select 11475,'011475','1979-070B','11S510','11S510',47554,'Reentered','1979-08-20 00:00:00','1979-08-08 00:00:00',90.24,208,365,62.84,'Intermediate' from dual union all
select 11476,'011476','1979-070C','BOZ','BOZ',47554,'Reentered','1979-08-14 00:00:00','1979-08-04 00:00:00',90.43,190,401,62.82,'Intermediate' from dual union all
select 11477,'011477','1979-069C','Sensor cover','Zenit-2M sensor cover',47553,'Reentered','1979-08-07 00:00:00','1979-07-31 00:00:00',88.9,209,232,81.35,'Intermediate' from dual union all
select 11478,'011478','1979-071A','Kosmos-1119','Orion',47555,'Landed','1979-08-15 00:00:00','1979-08-07 00:00:00',88.94,211,234,81.34,'Intermediate' from dual union all
select 11479,'011479','1979-071B','11S510','11S510',47555,'Reentered','1979-08-07 00:00:00','1979-08-06 00:00:00',87.81,152,181,81.33,'Intermediate' from dual union all
select 1148,'001148','1965-012BC','deb Kosmos-57','',11762,'Reentered','1965-03-11 00:00:00','1965-02-22 00:00:00',92.98,159,682,64.7,'Intermediate' from dual union all
select 11480,'011480','1977-097BB','Garbage bag','',42225,'Reentered','1979-10-15 00:00:00','1979-08-01 00:00:00',92.39,382,402,51.62,'Intermediate' from dual union all
select 11481,'011481','1979-068C','KDU','KDU',47552,'Reentered','1979-08-11 00:00:00','1979-08-06 00:00:00',89.12,169,293,62.8,'Intermediate' from dual union all
select 11482,'011482','1979-068D','Blenda','Sunshade',47552,'Reentered','1979-08-10 00:00:00','1979-08-07 00:00:00',88.38,153,236,62.79,'Intermediate' from dual union all
select 11483,'011483','1979-068E','KDU part','KDU part',47552,'Reentered','1979-08-11 00:00:00','1979-08-07 00:00:00',88.96,167,280,62.81,'Intermediate' from dual union all
select 11484,'011484','1979-072A','Westar 3','Westar 3',47556,'In Earth orbit','','1979-09-11 00:00:00',1435.93,35781,35786,.23,'Stationary' from dual union all
select 11485,'011485','1979-073A','Kosmos-1120','Zenit-4MKM',47557,'Landed','1979-08-24 00:00:00','1979-08-14 00:00:00',89.55,173,332,70.42,'Intermediate' from dual union all
select 11486,'011486','1979-073B','11S510','11S510',47557,'Reentered','1979-08-15 00:00:00','1979-08-13 00:00:00',89.13,165,298,70.42,'Intermediate' from dual union all
select 11487,'011487','1979-074A','Kosmos-1121','Feniks No. 929',47558,'Landed','1979-09-13 00:00:00','1979-08-21 00:00:00',89.14,161,303,67.15,'Intermediate' from dual union all
select 11488,'011488','1979-074B','11S510','11S510',47558,'Reentered','1979-08-18 00:00:00','1979-08-16 00:00:00',88.98,161,287,67.14,'Intermediate' from dual union all
select 11489,'011489','1977-097BC','D-5-3 EVA-1 debris','',42225,'Reentered','1979-10-15 00:00:00','1979-08-16 00:00:00',92.43,385,403,51.62,'Intermediate' from dual union all
select 1149,'001149','1965-012BD','deb Kosmos-57','',11762,'Reentered','1965-03-19 00:00:00','1965-02-22 00:00:00',90.96,169,475,64.8,'Intermediate' from dual union all
select 11490,'011490','1979-072B','Delta 149','Delta SSPU DSV-3P-4',47556,'Reentered','1979-08-10 00:00:00','1979-08-10 00:00:00',88.18,130,239,31.47,'Intermediate' from dual union all
select 11491,'011491','1979-075A','Kosmos-1122','Gektor-Priroda',47559,'Landed','1979-08-30 00:00:00','1979-08-19 00:00:00',88.94,210,235,81.36,'Intermediate' from dual union all
select 11492,'011492','1979-075B','11S510','11S510',47559,'Reentered','1979-08-20 00:00:00','1979-08-20 00:00:00',88.02,166,186,81.34,'Intermediate' from dual union all
select 11493,'011493','1977-097BD','KRT-10','KRT-10',42225,'Reentered','1979-08-26 00:00:00','1979-08-20 00:00:00',91.98,359,385,51.64,'Intermediate' from dual union all
select 11494,'011494','1977-097BE','D-5-3 EVA-1 debris','',42225,'Reentered','1979-12-31 00:00:00','1979-08-19 00:00:00',92.29,369,405,51.6,'Intermediate' from dual union all
select 11495,'011495','1977-097BF','D-5-3 EVA-1 debris','',42225,'Reentered','1979-10-11 00:00:00','1979-08-20 00:00:00',92.28,384,389,51.62,'Intermediate' from dual union all
select 11496,'011496','1979-076A','Kosmos-1123','Fram',47560,'Landed','1979-09-03 00:00:00','1979-08-23 00:00:00',88.99,214,236,81.37,'Intermediate' from dual union all
select 11497,'011497','1979-076B','11S510','11S510',47560,'Reentered','1979-08-24 00:00:00','1979-08-23 00:00:00',88.56,192,215,81.35,'Intermediate' from dual union all
select 11498,'011498','1979-075C','Sensor cover','Zenit-2M sensor cover',47559,'Reentered','1979-08-27 00:00:00','1979-08-22 00:00:00',88.81,204,228,81.36,'Intermediate' from dual union all
select 11499,'011499','1979-075D','Sensor cover','Zenit-2M sensor cover',47559,'Reentered','1979-08-25 00:00:00','1979-08-22 00:00:00',88.73,203,221,81.4,'Intermediate' from dual union all
select 115,'000115','60 Beta 4','Tiros 1 despin weight','Despin weight',6318,'In Earth orbit','','1961-06-15 00:00:00',99.9,699,809,48.15,'Intermediate' from dual union all
select 1150,'001150','1965-012BE','deb Kosmos-57','',11762,'Reentered','1965-03-17 00:00:00','1965-03-01 00:00:00',90.27,177,399,64.69,'Intermediate' from dual union all
select 11500,'011500','1977-097BG','Garbage bag','',42225,'Reentered','1979-12-21 00:00:00','1979-08-22 00:00:00',92.21,374,392,51.6,'Intermediate' from dual union all
select 11501,'011501','1977-097BH','Garbage bag','',42225,'Reentered','1979-12-28 00:00:00','1979-08-26 00:00:00',92.23,377,391,51.63,'Intermediate' from dual union all
select 11502,'011502','1977-097BJ','Garbage bag','',42225,'Reentered','1979-09-11 00:00:00','1979-08-22 00:00:00',92.31,383,393,51.63,'Intermediate' from dual union all
select 11503,'011503','1977-097BK','Garbage bag','',42225,'Reentered','1979-12-21 00:00:00','1979-08-23 00:00:00',92.26,372,399,51.62,'Intermediate' from dual union all
select 11504,'011504','1977-097BL','Garbage bag','',42225,'Reentered','1980-03-22 00:00:00','1979-08-25 00:00:00',92.28,383,390,51.62,'Intermediate' from dual union all
select 11505,'011505','1979-073C','Blenda','Sunshade',47557,'Reentered','1979-08-28 00:00:00','1979-08-20 00:00:00',89.31,169,312,70.41,'Intermediate' from dual union all
select 11506,'011506','1979-073D','KDU','KDU',47557,'Reentered','1979-08-31 00:00:00','1979-08-24 00:00:00',89.13,165,298,70.41,'Intermediate' from dual union all
select 11507,'011507','1979-075E','Sensor cover','Zenit-2M sensor cover',47559,'Reentered','1979-08-27 00:00:00','1979-08-25 00:00:00',88.33,179,204,81.34,'Intermediate' from dual union all
select 11508,'011508','1977-097BM','Garbage bag','',42225,'Reentered','1979-10-20 00:00:00','1979-08-26 00:00:00',92.25,378,392,51.62,'Intermediate' from dual union all
select 11509,'011509','1979-077A','Kosmos-1124','Oko',47561,'In Earth orbit','','1979-09-30 00:00:00',716.65,599,39700,63.02,'Molniya' from dual union all
select 1151,'001151','1965-012BF','deb Kosmos-57','',11762,'Reentered','1965-03-10 00:00:00','1965-02-22 00:00:00',92.21,381,385,64.8,'Intermediate' from dual union all
select 11510,'011510','1979-078A','Kosmos-1125','Strela-2M',47562,'In Earth orbit','','1980-01-10 00:00:00',100.88,791,810,74.03,'Intermediate' from dual union all
select 11511,'011511','1979-078B','S3M','S3M',47562,'In Earth orbit','','1980-01-04 00:00:00',100.77,783,808,74.04,'Intermediate' from dual union all
select 11512,'011512','1979-077B','11S510','11S510',47561,'Reentered','1979-09-28 00:00:00','1979-09-04 00:00:00',92.14,212,547,62.8,'Intermediate' from dual union all
select 11513,'011513','1979-077C','BOZ','BOZ',47561,'Reentered','1979-09-11 00:00:00','1979-08-30 00:00:00',92,180,566,62.89,'Intermediate' from dual union all
select 11514,'011514','1979-063E','Kosmos-1112 SS 3','ESO',47547,'Reentered','1979-09-23 00:00:00','1979-08-29 00:00:00',92.5,316,479,50.69,'Intermediate' from dual union all
select 11515,'011515','1979-079A','Kosmos-1126','Zenit-6',47563,'Landed','1979-09-14 00:00:00','1979-09-03 00:00:00',92.25,377,393,72.86,'Intermediate' from dual union all
select 11516,'011516','1979-079B','11S510','11S510',47563,'Reentered','1979-09-09 00:00:00','1979-09-02 00:00:00',90.16,192,373,72.86,'Intermediate' from dual union all
select 11517,'011517','1979-076C','Blenda','Sunshade',47560,'Reentered','1979-09-06 00:00:00','1979-09-02 00:00:00',88.68,202,217,81.37,'Intermediate' from dual union all
select 11518,'011518','1979-076D','KDU part','KDU part',47560,'Reentered','1979-09-06 00:00:00','1979-09-03 00:00:00',87.94,166,179,81.35,'Intermediate' from dual union all
select 11519,'011519','1979-076E','KDU','KDU',47560,'Reentered','1979-09-07 00:00:00','1979-09-03 00:00:00',88.69,202,217,81.36,'Intermediate' from dual union all
select 1152,'001152','1965-012BG','deb Kosmos-57','',11762,'Reentered','1965-03-16 00:00:00','1965-03-03 00:00:00',90.92,213,427,64.75,'Intermediate' from dual union all
select 11520,'011520','1979-080A','Kosmos-1127','Resurs-F1 17F41 No. 11',47564,'Landed','1979-09-18 00:00:00','1979-09-07 00:00:00',89.76,257,269,81.36,'Intermediate' from dual union all
select 11521,'011521','1979-080B','11S510','11S510',47564,'Reentered','1979-09-10 00:00:00','1979-09-08 00:00:00',88.71,191,231,81.35,'Intermediate' from dual union all
select 11522,'011522','1979-080C','Sensor cover','Sensor cover',47564,'Reentered','1979-09-06 00:00:00','1979-09-05 00:00:00',89.24,211,263,81.35,'Intermediate' from dual union all
select 11523,'011523','1977-097BN','Garbage bag','',42225,'Reentered','1980-01-19 00:00:00','1979-09-09 00:00:00',92.28,378,395,51.63,'Intermediate' from dual union all
select 11524,'011524','1979-079C','Blenda','Sunshade',47563,'Reentered','1979-10-26 00:00:00','1979-09-13 00:00:00',92.1,376,378,72.87,'Intermediate' from dual union all
select 11525,'011525','1979-079D','KDU','KDU',47563,'Reentered','1980-02-16 00:00:00','1979-09-30 00:00:00',92.12,371,386,72.86,'Intermediate' from dual union all
select 11526,'011526','1979-079E','KDU part','KDU part',47563,'Reentered','1980-02-14 00:00:00','1979-09-30 00:00:00',92.12,370,386,72.87,'Intermediate' from dual union all
select 11527,'011527','1979-079F','KDU part','KDU part',47563,'Reentered','1979-09-16 00:00:00','1979-09-14 00:00:00',92.19,376,388,73.1,'Intermediate' from dual;
insert into SATELLITE
select 11528,'011528','1979-079G','KDU part','KDU part',47563,'Reentered','1979-10-17 00:00:00','1979-09-14 00:00:00',92.1,374,381,72.84,'Intermediate' from dual union all
select 11529,'011529','1979-081A','Kosmos-1128','Zenit-4MKM',47565,'Landed','1979-09-27 00:00:00','1979-09-16 00:00:00',89.66,176,340,62.8,'Intermediate' from dual union all
select 1153,'001153','1965-012BH','deb Kosmos-57','',11762,'Reentered','1965-03-07 00:00:00','1965-02-28 00:00:00',90.31,165,415,64.89,'Intermediate' from dual union all
select 11530,'011530','1979-081B','11S510','11S510',47565,'Reentered','1979-09-17 00:00:00','1979-09-16 00:00:00',88.9,157,284,62.8,'Intermediate' from dual union all
select 11531,'011531','1979-080D','KDU','KDU',47564,'Reentered','1979-09-30 00:00:00','1979-09-17 00:00:00',89.73,254,269,81.37,'Intermediate' from dual union all
select 11532,'011532','1979-082A','HEAO 3','HEAO C',47566,'Reentered','1981-12-07 00:00:00','1979-10-28 00:00:00',94.38,481,497,43.6,'Intermediate' from dual union all
select 11533,'011533','1979-082B','Centaur AC-53','Centaur D-1AR AC-53',47566,'Reentered','1979-11-13 00:00:00','1979-09-27 00:00:00',92.21,355,410,43.66,'Intermediate' from dual union all
select 11534,'011534','1979-050G','DMSP F-4 solar array band','',47534,'Reentered','1986-01-01 00:00:00','1979-09-25 00:00:00',101.61,820,850,98.8,'Sun-Synch' from dual union all
select 11535,'011535','1979-063F','Kosmos-1112 SS 4','ESO',47547,'Reentered','1979-10-20 00:00:00','1979-09-25 00:00:00',92.85,330,499,50.72,'Intermediate' from dual union all
select 11536,'011536','1979-083A','Kosmos-1129','Bion No. 5',47567,'Landed','1979-10-14 00:00:00','1979-10-02 00:00:00',90.33,215,367,62.82,'Intermediate' from dual union all
select 11537,'011537','1979-083B','11S510','11S510',47567,'Reentered','1979-10-06 00:00:00','1979-09-27 00:00:00',90.26,214,360,62.81,'Intermediate' from dual union all
select 11538,'011538','1979-084A','Kosmos-1130','Strela-1M',47568,'In Earth orbit','','1980-02-12 00:00:00',114.63,1397,1481,74.02,'Intermediate' from dual union all
select 11539,'011539','1979-084B','Kosmos-1131','Strela-1M',47568,'In Earth orbit','','1980-01-01 00:00:00',114.81,1411,1483,74.02,'Intermediate' from dual union all
select 1154,'001154','1965-012BJ','deb Kosmos-57','',11762,'Reentered','1965-03-07 00:00:00','1965-02-28 00:00:00',90.74,153,469,64.81,'Intermediate' from dual union all
select 11540,'011540','1979-084C','Kosmos-1132','Strela-1M',47568,'In Earth orbit','','1980-01-23 00:00:00',114.98,1426,1483,74.02,'Intermediate' from dual union all
select 11541,'011541','1979-084D','Kosmos-1133','Strela-1M',47568,'In Earth orbit','','1980-01-01 00:00:00',115.13,1439,1484,74.02,'Intermediate' from dual union all
select 11542,'011542','1979-084E','Kosmos-1134','Strela-1M',47568,'In Earth orbit','','1980-01-01 00:00:00',115.29,1454,1484,74.02,'Intermediate' from dual union all
select 11543,'011543','1979-084F','Kosmos-1135','Strela-1M',47568,'In Earth orbit','','1980-01-02 00:00:00',115.47,1462,1492,74.02,'Intermediate' from dual union all
select 11544,'011544','1979-084G','Kosmos-1136','Strela-1M',47568,'In Earth orbit','','1980-01-09 00:00:00',115.65,1473,1497,74.02,'Intermediate' from dual union all
select 11545,'011545','1979-084H','Kosmos-1137','Strela-1M',47568,'In Earth orbit','','1980-01-13 00:00:00',115.84,1473,1514,74.02,'Intermediate' from dual union all
select 11546,'011546','1979-084J','S3M','S3M',47568,'In Earth orbit','','1980-01-03 00:00:00',117.83,1480,1687,74.02,'Intermediate' from dual union all
select 11547,'011547','1979-081C','KDU','KDU',47565,'Reentered','1979-09-27 00:00:00','1979-09-27 00:00:00',87.78,125,205,62.71,'Intermediate' from dual union all
select 11548,'011548','1979-085A','Kosmos-1138','Zenit-6',47569,'Landed','1979-10-12 00:00:00','1979-09-30 00:00:00',92.25,362,408,72.87,'Intermediate' from dual union all
select 11549,'011549','1979-085B','11S510','11S510',47569,'Reentered','1979-10-05 00:00:00','1979-09-30 00:00:00',89.74,190,334,72.85,'Intermediate' from dual union all
select 1155,'001155','1965-012BK','deb Kosmos-57','',11762,'Reentered','1965-03-07 00:00:00','1965-03-03 00:00:00',89.84,153,381,64.84,'Intermediate' from dual union all
select 11550,'011550','1979-077D','Blok-L','Blok-L',47561,'In Earth orbit','','1979-09-29 00:00:00',723.89,588,40068,63,'Molniya' from dual union all
select 11551,'011551','1979-031D','Blok-L','Blok-L',47515,'Reentered','1990-06-30 00:00:00','1980-01-05 00:00:00',732.34,677,40394,63.17,'Molniya' from dual union all
select 11552,'011552','1979-035D','SOZ','SOZ',47519,'Reentered','1980-01-01 00:00:00','1979-09-27 00:00:00',627.14,170,35614,46.9,'GEO Transfer' from dual union all
select 11553,'011553','1979-004D','Blok-L','Blok-L',47488,'Reentered','1999-05-08 00:00:00','1979-07-25 00:00:00',732.96,667,40434,62.88,'Molniya' from dual union all
select 11554,'011554','1979-048D','Blok-L','Blok-L',47532,'Reentered','1990-08-05 00:00:00','1980-01-10 00:00:00',730.98,559,40445,62.93,'Molniya' from dual union all
select 11555,'011555','1979-058D','Blok-L','Blok-2BL No. 77338631',47542,'Reentered','2012-01-04 00:00:00','1979-07-27 00:00:00',721.8,664,39889,62.9,'Molniya' from dual union all
select 11556,'011556','1979-070D','Blok-L','Blok-ML  No. 76738451',47554,'Reentered','2002-02-02 00:00:00','1979-09-06 00:00:00',733.22,460,40654,62.82,'Molniya' from dual union all
select 11557,'011557','1979-063G','Kosmos-1112 SS 5','ESO',47547,'Reentered','1979-10-21 00:00:00','1979-09-27 00:00:00',92.8,329,495,50.65,'Intermediate' from dual union all
select 11558,'011558','1979-086A','OPS 1948','VORTEX 9',47570,'In Earth orbit','','1979-11-01 00:00:00',1445.47,30443,41497,7.5,'Near-synch' from dual union all
select 11559,'011559','1979-086B','Titan 23C-16 Stage 2','LR91AJ11 23C-16',47570,'Reentered','1979-10-07 00:00:00','1979-10-03 00:00:00',90.04,145,408,29.04,'Intermediate' from dual union all
select 1156,'001156','1965-012BL','deb Kosmos-57','',11762,'Reentered','1965-03-18 00:00:00','1965-03-01 00:00:00',91.85,180,550,64.78,'Intermediate' from dual union all
select 11560,'011560','1979-086C','Transtage 34','Transtage 34',47570,'In Earth orbit','','1979-11-01 00:00:00',1444.45,30400,41500,7.5,'Near-synch' from dual union all
select 11561,'011561','1979-087A','Ekran','Ekran No. 17L',47571,'In Earth orbit','','1979-11-10 00:00:00',1436.09,35762,35811,.37,'Stationary' from dual union all
select 11562,'011562','1979-087B','8S812','8S812',47571,'Reentered','1979-10-05 00:00:00','1979-10-04 00:00:00',88.16,170,197,51.59,'Intermediate' from dual union all
select 11563,'011563','1977-097BP','Garbage bag','',42225,'Reentered','1979-11-09 00:00:00','1979-10-14 00:00:00',91.47,345,349,51.62,'Intermediate' from dual union all
select 11564,'011564','1979-088A','Kosmos-1139','Orion',47572,'Landed','1979-10-18 00:00:00','1979-10-07 00:00:00',89.79,200,329,72.86,'Intermediate' from dual union all
select 11565,'011565','1979-088B','11S510','11S510',47572,'Reentered','1979-10-11 00:00:00','1979-10-07 00:00:00',89.39,202,287,72.85,'Intermediate' from dual union all
select 11566,'011566','1979-088C','Sensor cover','Zenit-4MT sensor cover',47572,'Reentered','1979-10-07 00:00:00','1979-10-06 00:00:00',89.79,194,335,72.86,'Intermediate' from dual union all
select 11567,'011567','1974-017F','Blok DM  No. 1L','11S86 No. 1L',33620,'In Earth orbit','','1974-05-11 00:00:00',1425.68,35410,35755,.29,'Drift GEO' from dual union all
select 11568,'011568','1975-123F','Blok DM  No. 3L','11S86 No. 3L',36570,'In Earth orbit','','1977-04-10 00:00:00',1432.87,35639,35808,.75,'Drift GEO' from dual union all
select 11569,'011569','1976-107F','Blok DM  No. 6L','11S86 No. 6L',39059,'In Earth orbit','','1993-06-01 00:00:00',1419.26,35423,35490,12.51,'Inclined Drift' from dual union all
select 1157,'001157','1965-012BM','deb Kosmos-57','',11762,'Reentered','1965-03-13 00:00:00','1965-03-03 00:00:00',90.18,188,379,64.68,'Intermediate' from dual union all
select 11570,'011570','1977-071F','Blok DM  No. 7L','11S86 No. 7L',42199,'In Earth orbit','','1993-05-30 00:00:00',1473.28,36471,36552,12.77,'Inclined Drift' from dual union all
select 11571,'011571','1977-092G','Blok DM  No. 9L','11S86 No. 9L',42220,'In Earth orbit','','1993-06-07 00:00:00',1421.75,35474,35537,12.12,'Inclined Drift' from dual union all
select 11572,'011572','1977-097BQ','Garbage bag','',42225,'Reentered','1979-10-27 00:00:00','1979-10-09 00:00:00',90.87,307,327,51.62,'Intermediate' from dual union all
select 11573,'011573','1979-089A','Kosmos-1140','Strela-2M',47573,'In Earth orbit','','1992-06-27 00:00:00',100.42,767,790,74.06,'Intermediate' from dual union all
select 11574,'011574','1979-089B','S3M','S3M',47573,'In Earth orbit','','1991-03-28 00:00:00',100.29,761,783,74.05,'Intermediate' from dual union all
select 11575,'011575','1979-085C','Blenda','Sunshade',47569,'Reentered','1979-11-02 00:00:00','1979-10-11 00:00:00',92.01,351,396,72.86,'Intermediate' from dual union all
select 11576,'011576','1979-085D','KDU part','KDU part',47569,'Reentered','1979-10-13 00:00:00','1979-10-11 00:00:00',92.3,382,393,73.38,'Intermediate' from dual union all
select 11577,'011577','1979-085E','KDU part','KDU part',47569,'Reentered','1979-11-12 00:00:00','1979-10-12 00:00:00',92.04,350,400,72.87,'Intermediate' from dual union all
select 11578,'011578','1979-085F','KDU','KDU',47569,'Reentered','1980-02-26 00:00:00','1979-10-28 00:00:00',92.05,355,396,72.85,'Intermediate' from dual union all
select 11579,'011579','1979-083C','KBE part','KBE part',47567,'Reentered','1979-10-31 00:00:00','1979-10-14 00:00:00',90.02,211,340,62.81,'Intermediate' from dual union all
select 1158,'001158','1965-012BN','deb Kosmos-57','',11762,'Reentered','1965-03-04 00:00:00','1965-02-26 00:00:00',89.17,153,314,64.85,'Intermediate' from dual union all
select 11580,'011580','1979-083D','KBE part','KBE part',47567,'Reentered','1979-10-18 00:00:00','1979-10-14 00:00:00',89.9,208,331,62.81,'Intermediate' from dual union all
select 11581,'011581','1977-092H','deb Ekran battery part','',42220,'In Earth orbit','','1997-12-19 00:00:00',1435.42,35733,35814,14.47,'Inclined Drift' from dual union all
select 11582,'011582','1979-083E','KB electropitaniya','KBE',47567,'Reentered','1980-03-16 00:00:00','1979-10-22 00:00:00',90.31,198,382,62.76,'Intermediate' from dual union all
select 11583,'011583','1979-063H','Kosmos-1112 SS 6','ESO',47547,'Reentered','1979-11-05 00:00:00','1979-10-16 00:00:00',92.68,320,492,50.68,'Intermediate' from dual union all
select 11584,'011584','1979-063J','Kosmos-1112 SS 7','ESO',47547,'Reentered','1979-11-05 00:00:00','1979-10-15 00:00:00',92.59,323,480,50.68,'Intermediate' from dual union all
select 11585,'011585','1979-090A','Kosmos-1141','Parus',47574,'In Earth orbit','','1992-06-18 00:00:00',104.62,955,999,82.95,'Intermediate' from dual union all
select 11586,'011586','1979-090B','S3M','S3M',47574,'In Earth orbit','','1991-03-25 00:00:00',104.49,950,992,82.94,'Intermediate' from dual union all
select 11587,'011587','1978-007C','deb Kosmos-985','',45202,'In Earth orbit','','1979-11-16 00:00:00',104.56,953,995,82.96,'Intermediate' from dual union all
select 11588,'011588','1979-090C','deb Kosmos-1141   [*]','',47574,'Reentered','1982-09-07 00:00:00','1979-11-17 00:00:00',104.96,977,1009,82.94,'Intermediate' from dual union all
select 11589,'011589','1979-091A','Molniya-1','Molniya-1',47575,'Reentered','1994-02-18 00:00:00','1979-11-19 00:00:00',717.72,608,39744,62.91,'Molniya' from dual union all
select 1159,'001159','1965-012BP','deb Kosmos-57','',11762,'Reentered','1965-03-18 00:00:00','1965-03-03 00:00:00',90.56,159,444,64.71,'Intermediate' from dual union all
select 11590,'011590','1979-091B','11S510','11S510',47575,'Reentered','1979-11-11 00:00:00','1979-10-28 00:00:00',91.6,208,499,62.85,'Intermediate' from dual union all
select 11591,'011591','1979-091C','BOZ','BOZ',47575,'Reentered','1979-11-10 00:00:00','1979-10-28 00:00:00',91.69,192,523,62.85,'Intermediate' from dual union all
select 11592,'011592','1979-092A','Kosmos-1142','Zenit-6',47576,'Landed','1979-11-04 00:00:00','1979-10-28 00:00:00',92.22,351,416,72.86,'Intermediate' from dual union all
select 11593,'011593','1979-092B','11S510','11S510',47576,'Reentered','1979-10-28 00:00:00','1979-10-24 00:00:00',89.91,189,350,72.85,'Intermediate' from dual union all
select 11594,'011594','1979-063K','Kosmos-1112 SS 8','ESO',47547,'Reentered','1979-11-07 00:00:00','1979-10-23 00:00:00',92.48,315,478,50.68,'Intermediate' from dual union all
select 11595,'011595','1979-063L','Kosmos-1112 SS 9','ESO',47547,'Reentered','1979-11-11 00:00:00','1979-10-23 00:00:00',92.6,327,477,50.68,'Intermediate' from dual union all
select 11596,'011596','1979-063M','Kosmos-1112 SS 10','ESO',47547,'Reentered','1980-01-05 00:00:00','1979-10-24 00:00:00',92.45,316,474,50.69,'Intermediate' from dual union all
select 11597,'011597','1979-063N','Kosmos-1112 SS 11','ESO',47547,'Reentered','1980-02-20 00:00:00','1979-10-24 00:00:00',92.67,329,482,50.7,'Intermediate' from dual union all
select 11598,'011598','1979-063P','Kosmos-1112 SS 12','ESO',47547,'Reentered','1979-11-01 00:00:00','1979-10-24 00:00:00',92.45,314,476,50.68,'Intermediate' from dual union all
select 11599,'011599','1979-063Q','Kosmos-1112 SS 13','ESO',47547,'Reentered','1979-12-07 00:00:00','1979-10-24 00:00:00',92.9,355,479,50.68,'Intermediate' from dual union all
select 116,'000116','61 Omicron 1','Transit 4A','Transit 4A',7273,'In Earth orbit','','1961-08-02 00:00:00',103.8,880,997,66.81,'Intermediate' from dual union all
select 1160,'001160','1965-012BQ','deb Kosmos-57','',11762,'Reentered','1965-03-04 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 11600,'011600','1979-093A','Kosmos-1143','Tselina-D',47577,'Reentered','2002-02-17 00:00:00','1979-11-25 00:00:00',97.35,623,641,81.24,'Intermediate' from dual union all
select 11601,'011601','1979-093B','8A92ME','8A92ME',47577,'Reentered','2010-04-30 00:00:00','1979-11-25 00:00:00',97.41,590,680,81.25,'Intermediate' from dual union all
select 11602,'011602','1979-091D','Blok-L','Blok-ML No. 150',47575,'Reentered','1994-10-14 00:00:00','1979-11-20 00:00:00',732.55,614,40467,62.98,'Molniya' from dual union all
select 11603,'011603','1979-063R','Kosmos-1112 SS 14','ESO',47547,'Reentered','1979-11-12 00:00:00','1979-10-28 00:00:00',92.32,317,460,50.67,'Intermediate' from dual union all
select 11604,'011604','1979-094A','Magsat','AEM-C',47578,'Reentered','1980-06-11 00:00:00','1979-11-29 00:00:00',93.46,347,541,96.79,'Sun-Synch' from dual union all
select 11605,'011605','1979-095A','Meteor-2','Meteor-2 No. 7',47579,'In Earth orbit','','1979-12-05 00:00:00',102.54,870,888,81.21,'Intermediate' from dual union all
select 11606,'011606','1979-094B','Star 20 E-26','Star 20 E-26',47578,'Reentered','1980-01-27 00:00:00','1979-11-29 00:00:00',92.96,335,503,96.79,'Sun-Synch' from dual union all
select 11607,'011607','1979-009B','N-I Stage 2 S/N N-5(F)','LE-3',47493,'Reentered','1979-02-10 00:00:00','1979-02-10 00:00:00',88.12,171,191,30.4,'Intermediate' from dual union all
select 11608,'011608','1979-095B','8A92ME','8A92ME',47579,'In Earth orbit','','1979-12-02 00:00:00',102.59,840,922,81.21,'Intermediate' from dual union all
select 11609,'011609','1979-096A','Interkosmos-20','AUOS-Z ',47580,'Reentered','1981-03-03 00:00:00','1979-12-02 00:00:00',94.24,454,510,74.04,'Intermediate' from dual union all
select 1161,'001161','1965-012BR','deb Kosmos-57','',11762,'Reentered','1965-03-05 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 11610,'011610','1979-096B','S3M','S3M',47580,'Reentered','1981-03-26 00:00:00','1979-12-01 00:00:00',94.17,448,508,74.04,'Intermediate' from dual union all
select 11611,'011611','1979-097A','Kosmos-1144','Feniks No. 939',47581,'Landed','1979-12-04 00:00:00','1979-11-09 00:00:00',89.63,156,357,67.16,'Intermediate' from dual union all
select 11612,'011612','1979-097B','11S510','11S510',47581,'Reentered','1979-11-05 00:00:00','1979-11-04 00:00:00',88.7,151,270,67.16,'Intermediate' from dual union all
select 11613,'011613','1979-092C','KDU','KDU',47576,'Reentered','1980-03-25 00:00:00','1979-11-25 00:00:00',91.9,340,396,72.86,'Intermediate' from dual union all
select 11614,'011614','1979-092D','Blenda','Sunshade',47576,'Reentered','1979-12-01 00:00:00','1979-11-04 00:00:00',92.03,337,412,72.86,'Intermediate' from dual union all
select 11615,'011615','1979-092E','KDU part','KDU part',47576,'Reentered','1979-11-21 00:00:00','1979-11-05 00:00:00',92.18,348,415,72.86,'Intermediate' from dual union all
select 11616,'011616','1979-092F','KDU part','KDU part',47576,'Reentered','1980-03-09 00:00:00','1979-11-25 00:00:00',91.9,340,396,72.85,'Intermediate' from dual union all
select 11617,'011617','1979-063S','Kosmos-1112 SS 15','ESO',47547,'Reentered','1979-12-05 00:00:00','1979-11-20 00:00:00',92.05,318,433,50.66,'Intermediate' from dual;
insert into SATELLITE
select 11618,'011618','1979-063T','Kosmos-1112 SS 16','ESO',47547,'Reentered','1979-11-22 00:00:00','1979-11-20 00:00:00',92.24,329,440,50.69,'Intermediate' from dual union all
select 11619,'011619','1979-063U','Kosmos-1112 SS 17','ESO',47547,'Reentered','1979-12-05 00:00:00','1979-11-21 00:00:00',91.88,301,433,50.67,'Intermediate' from dual union all
select 1162,'001162','1965-012BS','deb Kosmos-57','',11762,'Reentered','1965-03-25 00:00:00','1965-03-03 00:00:00',92.64,168,640,64.67,'Intermediate' from dual union all
select 11620,'011620','1979-063V','Kosmos-1112 SS 18','ESO',47547,'Reentered','1979-11-30 00:00:00','1979-11-21 00:00:00',91.81,298,429,50.71,'Intermediate' from dual union all
select 11621,'011621','1979-098A','DSCS II F-13','DSCS II D-13',47582,'In Earth orbit','','1979-12-21 00:00:00',1429.71,35536,35787,2.37,'Drift GEO' from dual union all
select 11622,'011622','1979-098B','DSCS II F-14','DSCS II D-14',47582,'In Earth orbit','','1980-01-02 00:00:00',1440.94,35793,35970,2.28,'Drift GEO' from dual union all
select 11623,'011623','1979-098C','Transtage 37','Transtage 37',47582,'In Earth orbit','','1979-12-22 00:00:00',1510.95,35803,38676,2.33,'Very High' from dual union all
select 11624,'011624','1979-098D','Titan 23C-19 Stage 2','LR91AJ11 23C-19',47582,'Reentered','1979-11-21 00:00:00','1979-11-21 00:00:00',87.86,152,186,28.63,'Intermediate' from dual union all
select 11625,'011625','1979-063W','Kosmos-1112 SS 19','ESO',47547,'Reentered','1979-12-04 00:00:00','1979-11-23 00:00:00',91.58,284,421,50.69,'Intermediate' from dual union all
select 11626,'011626','1979-063X','Kosmos-1112 SS 20','ESO',47547,'Reentered','1979-12-04 00:00:00','1979-11-26 00:00:00',91.36,289,394,50.68,'Intermediate' from dual union all
select 11627,'011627','1979-063Y','Kosmos-1112 SS 21','ESO',47547,'Reentered','1979-12-05 00:00:00','1979-11-25 00:00:00',91.59,296,410,50.69,'Intermediate' from dual union all
select 11628,'011628','1979-063Z','Kosmos-1112 SS 22','ESO',47547,'Reentered','1979-12-07 00:00:00','1979-11-25 00:00:00',91.76,309,412,50.67,'Intermediate' from dual union all
select 11629,'011629','1979-099A','Kosmos-1145','Tselina-D',47583,'Reentered','2000-06-16 00:00:00','1979-12-27 00:00:00',97.23,622,631,81.22,'Intermediate' from dual union all
select 1163,'001163','1965-012BT','deb Kosmos-57','',11762,'Reentered','1965-03-05 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 11630,'011630','1979-099B','8A92ME','8A92ME',47583,'Reentered','2004-05-07 00:00:00','1990-04-03 00:00:00',96.27,541,619,81.19,'Intermediate' from dual union all
select 11631,'011631','1965-027F','deb SNAPSHOT','',11777,'In Earth orbit','','1993-08-05 00:00:00',111.32,1265,1310,90.28,'Polar' from dual union all
select 11632,'011632','1979-100A','Kosmos-1146','Taifun-1B',47584,'Reentered','1981-11-25 00:00:00','1980-01-04 00:00:00',93.9,438,492,65.86,'Intermediate' from dual union all
select 11633,'011633','1979-100B','S3M','S3M',47584,'Reentered','1981-01-31 00:00:00','1980-01-04 00:00:00',93.78,431,488,65.86,'Intermediate' from dual union all
select 11634,'011634','1979-057C','NOAA 6 solar array band','',47541,'Reentered','2004-07-13 00:00:00','1993-08-17 00:00:00',98.91,704,710,98.28,'Sun-Synch' from dual union all
select 11635,'011635','1979-101A','Satcom 3','RCA-C',47585,'In Earth orbit','','1982-11-18 00:00:00',788.82,8356,35448,8.2,'Medium' from dual union all
select 11636,'011636','1979-101B','Delta 150','Delta SSPU DSV-3P-4',47585,'Reentered','1980-04-09 00:00:00','2000-11-22 00:00:00',118.28,581,2627,28.22,'Intermediate' from dual union all
select 11637,'011637','1979-101C','deb Delta 150','',47585,'Reentered','1980-03-27 00:00:00','1979-12-14 00:00:00',109.32,161,2230,27.36,'Intermediate' from dual union all
select 11638,'011638','1979-102A','Kosmos-1147','Zenit-6',47586,'Landed','1979-12-26 00:00:00','1979-12-14 00:00:00',92.24,351,418,72.87,'Intermediate' from dual union all
select 11639,'011639','1979-102B','11S510','11S510',47586,'Reentered','1979-12-19 00:00:00','1979-12-14 00:00:00',89.88,187,350,72.87,'Intermediate' from dual union all
select 1164,'001164','1965-012BU','deb Kosmos-57','',11762,'Reentered','1965-03-13 00:00:00','1965-03-04 00:00:00',89.41,191,300,64.85,'Intermediate' from dual union all
select 11640,'011640','1979-103A','Soyuz T','Soyuz 7K-ST No. 6L',47587,'Landed','1980-03-25 00:00:00','1980-01-18 00:00:00',91.81,357,370,51.61,'Intermediate' from dual union all
select 11641,'011641','1979-103B','11S510','11S510',47587,'Reentered','1979-12-18 00:00:00','1979-12-17 00:00:00',88.31,179,202,51.59,'Intermediate' from dual union all
select 11642,'011642','1979-103C','Soyuz T part','Soyuz part',47587,'Reentered','1979-12-18 00:00:00','1979-12-23 00:00:00',91.38,339,345,51.62,'Intermediate' from dual union all
select 11643,'011643','1979-063AA','Kosmos-1112 SS 23','ESO',47547,'Reentered','1979-12-21 00:00:00','1979-12-20 00:00:00',91.15,281,381,50.68,'Intermediate' from dual union all
select 11644,'011644','1979-063AB','Kosmos-1112 SS 24','ESO',47547,'Reentered','1979-12-21 00:00:00','1979-12-20 00:00:00',90.86,276,358,50.66,'Intermediate' from dual union all
select 11645,'011645','1979-104A','CAT 01','CAT',47588,'Reentered','1989-11-27 00:00:00','1980-01-23 00:00:00',631.45,182,35824,17.63,'GEO Transfer' from dual union all
select 11646,'011646','1979-102C','Blenda','Sunshade',47586,'Reentered','1980-01-15 00:00:00','1979-12-25 00:00:00',91.47,281,413,72.86,'Intermediate' from dual union all
select 11647,'011647','1979-102D','KDU','KDU',47586,'Reentered','1980-03-09 00:00:00','1980-01-12 00:00:00',91.27,287,387,72.86,'Intermediate' from dual union all
select 11648,'011648','1979-105A','Gorizont','Gorizont No. 13L',47589,'In Earth orbit','','1980-02-16 00:00:00',1436.01,35729,35841,.65,'Stationary' from dual union all
select 11649,'011649','1979-106A','Kosmos-1148','Zenit-4MKM',47590,'Landed','1980-01-10 00:00:00','1979-12-30 00:00:00',89.59,170,339,67.14,'Intermediate' from dual union all
select 1165,'001165','1965-012BV','deb Kosmos-57','',11762,'Reentered','1965-03-09 00:00:00','1965-03-07 00:00:00',89.34,183,301,64.94,'Intermediate' from dual union all
select 11650,'011650','1979-106B','11S510','11S510',47590,'Reentered','1979-12-31 00:00:00','1979-12-30 00:00:00',88.85,155,281,67.13,'Intermediate' from dual union all
select 11651,'011651','1979-105B','8S812','8S812',47589,'Reentered','1979-12-30 00:00:00','1979-12-29 00:00:00',87.58,151,158,51.64,'Intermediate' from dual union all
select 11652,'011652','1980-001A','Kosmos-1149','Zenit-6',50211,'Landed','1980-01-23 00:00:00','1980-01-11 00:00:00',92.25,352,418,72.87,'Intermediate' from dual union all
select 11653,'011653','1980-001B','11S510','11S510',50211,'Reentered','1980-01-17 00:00:00','1980-01-11 00:00:00',90.03,190,362,72.87,'Intermediate' from dual union all
select 11654,'011654','1979-106C','Blenda','Sunshade',47590,'Reentered','1980-01-10 00:00:00','1980-01-10 00:00:00',88.17,142,226,67.12,'Intermediate' from dual union all
select 11655,'011655','1979-106D','KDU','KDU',47590,'Reentered','1980-01-15 00:00:00','1980-01-10 00:00:00',89.25,161,313,67.14,'Intermediate' from dual union all
select 11656,'011656','1980-001C','Sensor cover','Zenit-6 sensor cover',50211,'Reentered','1980-01-19 00:00:00','1980-01-10 00:00:00',92.2,333,432,72.87,'Intermediate' from dual union all
select 11657,'011657','1979-106E','KDU part','KDU part',47590,'Reentered','1980-01-15 00:00:00','1980-01-10 00:00:00',89.27,173,303,67.13,'Intermediate' from dual union all
select 11658,'011658','1979-106F','KDU part','KDU part',47590,'Reentered','1980-01-11 00:00:00','1980-01-10 00:00:00',88.14,134,231,67.13,'Intermediate' from dual union all
select 11659,'011659','1979-104B','Ariane H8 LO1','Ariane H8 S/N LO1',47588,'Reentered','1982-11-14 00:00:00','1980-01-24 00:00:00',614.1,159,34950,17.76,'GEO Transfer' from dual union all
select 1166,'001166','1965-012BW','deb Kosmos-57','',11762,'Reentered','1965-03-07 00:00:00','1965-02-27 00:00:00',89.5,190,310,64.69,'Intermediate' from dual union all
select 11660,'011660','1979-105C','SOZ','SOZ',47589,'Reentered','1982-04-15 00:00:00','1980-01-28 00:00:00',641.24,268,36241,47.16,'GEO Transfer' from dual union all
select 11661,'011661','1979-105D','SOZ','SOZ',47589,'Reentered','1984-12-03 00:00:00','1980-01-27 00:00:00',643.06,324,36278,47.21,'GEO Transfer' from dual union all
select 11662,'011662','1980-002A','Molniya-1','Molniya-1',50212,'Reentered','1992-10-22 00:00:00','1980-02-10 00:00:00',717.7,484,39867,62.86,'Molniya' from dual union all
select 11663,'011663','1980-002B','11S510','11S510',50212,'Reentered','1980-01-28 00:00:00','1980-01-18 00:00:00',89.96,179,366,62.84,'Intermediate' from dual union all
select 11664,'011664','1980-002C','BOZ','BOZ',50212,'Reentered','1980-01-24 00:00:00','1980-01-24 00:00:00',87.66,131,187,62.83,'Intermediate' from dual union all
select 11665,'011665','1980-002D','deb Blok-I','',50212,'Reentered','1980-01-11 00:00:00','1980-01-11 00:00:00',91.53,138,562,62.91,'Intermediate' from dual union all
select 11666,'011666','1980-002E','deb Blok-I','',50212,'Reentered','1980-01-21 00:00:00','1980-01-14 00:00:00',90.61,198,411,62.83,'Intermediate' from dual union all
select 11667,'011667','1980-003A','Kosmos-1150','Parus',50213,'In Earth orbit','','1980-02-18 00:00:00',104.93,968,1015,82.95,'Intermediate' from dual union all
select 11668,'011668','1980-003B','S3M','S3M',50213,'In Earth orbit','','1991-03-25 00:00:00',104.74,965,1000,82.95,'Intermediate' from dual union all
select 11669,'011669','1980-004A','FLTSATCOM F3','FLTSATCOM F3',50214,'In Earth orbit','','1980-02-22 00:00:00',1436.04,35744,35827,2.63,'Stationary' from dual union all
select 1167,'001167','1965-012BX','deb Kosmos-57','',11762,'Reentered','1965-03-20 00:00:00','1965-03-04 00:00:00',90.43,168,423,64.78,'Intermediate' from dual union all
select 11670,'011670','1980-002F','Blok-L','Blok-ML No. 77352-446',50212,'Reentered','1992-12-04 00:00:00','1992-10-01 00:00:00',695.32,111,39130,63.43,'Molniya' from dual union all
select 11671,'011671','1980-005A','Kosmos-1151','Okean-E NKh No. 2',50215,'Reentered','2014-08-04 00:00:00','1980-02-25 00:00:00',97.69,633,664,82.52,'Intermediate' from dual union all
select 11672,'011672','1980-005B','S5M','S5M',50215,'In Earth orbit','','1987-10-07 00:00:00',97.48,623,654,82.52,'Intermediate' from dual union all
select 11673,'011673','1980-001D','KDU','KDU',50211,'Reentered','1980-06-11 00:00:00','1980-02-08 00:00:00',92.09,347,407,72.87,'Intermediate' from dual union all
select 11674,'011674','1980-001E','Blenda','Sunshade',50211,'Reentered','1980-02-18 00:00:00','1980-01-23 00:00:00',92.24,349,419,72.86,'Intermediate' from dual union all
select 11675,'011675','1980-001F','KDU part','KDU part',50211,'Reentered','1980-06-11 00:00:00','1980-01-23 00:00:00',91.85,339,391,72.87,'Intermediate' from dual union all
select 11676,'011676','1975-097F','Blok DM','11S86 No. 4L',36544,'In Earth orbit','','1977-01-27 00:00:00',1438,35747,35900,1.06,'Drift GEO' from dual union all
select 11677,'011677','1980-001G','KDU part','KDU part',50211,'Reentered','1980-01-27 00:00:00','1980-01-24 00:00:00',91.96,330,412,72.9,'Intermediate' from dual union all
select 11678,'011678','1980-006A','Kosmos-1152','Feniks No. 928',50216,'Reentered','1980-02-06 00:00:00','1980-01-26 00:00:00',89.96,171,374,67.14,'Intermediate' from dual union all
select 11679,'011679','1980-006B','11S510','11S510',50216,'Reentered','1980-01-28 00:00:00','1980-01-26 00:00:00',89.25,166,309,67.14,'Intermediate' from dual union all
select 1168,'001168','1965-012BY','deb Kosmos-57','',11762,'Reentered','1965-03-06 00:00:00','1965-03-01 00:00:00',89.32,113,368,64.61,'Intermediate' from dual union all
select 11680,'011680','1980-007A','Kosmos-1153','Parus',50217,'In Earth orbit','','1980-02-24 00:00:00',104.92,964,1018,82.93,'Intermediate' from dual union all
select 11681,'011681','1980-007B','S3M','S3M',50217,'In Earth orbit','','1980-07-02 00:00:00',104.83,958,1015,82.93,'Intermediate' from dual union all
select 11682,'011682','1980-008A','Kosmos-1154','Tselina-D',50218,'Reentered','2002-11-05 00:00:00','1980-02-29 00:00:00',97.41,629,641,81.22,'Intermediate' from dual union all
select 11683,'011683','1980-008B','8A92ME','8A92ME',50218,'Reentered','2014-10-12 00:00:00','1980-03-01 00:00:00',97.46,586,689,81.23,'Intermediate' from dual union all
select 11684,'011684','1979-105E','Blok DM No. 24L','11S86 No. 24L',47589,'In Earth orbit','','1980-01-28 00:00:00',1459.19,36154,36321,.76,'Drift GEO' from dual union all
select 11685,'011685','1980-009A','Kosmos-1155','Zenit-6',50219,'Landed','1980-02-21 00:00:00','1980-02-09 00:00:00',92.23,354,414,72.87,'Intermediate' from dual union all
select 11686,'011686','1980-009B','11S510','11S510',50219,'Reentered','1980-02-16 00:00:00','1980-02-09 00:00:00',90.12,189,372,72.87,'Intermediate' from dual union all
select 11687,'011687','1980-010A','OPS 2581','KENNEN 3',50220,'Deorbited','1982-10-30 00:00:00','1980-03-09 00:00:00',92.44,304,485,97.05,'Sun-Synch' from dual union all
select 11688,'011688','1980-010B','Titan 23D-19 Stage 2','LR91AJ11 23D-19',50220,'Reentered','1980-03-13 00:00:00','1980-02-14 00:00:00',91.33,229,451,96.97,'Sun-Synch' from dual union all
select 11689,'011689','1980-010C','deb KH-11 F-3','(spurious)',50220,'Reentered','1980-02-08 00:00:00','1980-02-07 00:00:00',91.82,228,500,96.98,'Sun-Synch' from dual union all
select 1169,'001169','1965-012BZ','deb Kosmos-57','',11762,'Reentered','1965-03-05 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 11690,'011690','1980-011A','Navstar 5','Navstar GPS SVN 5',50221,'In Earth orbit','','1980-03-12 00:00:00',717.94,20144,20219,63.22,'Medium' from dual union all
select 11691,'011691','1980-012A','Kosmos-1156','Strela-1M',50222,'In Earth orbit','','1980-08-28 00:00:00',114.58,1398,1475,74.02,'Intermediate' from dual union all
select 11692,'011692','1980-012B','Kosmos-1157','Strela-1M',50222,'In Earth orbit','','1980-06-30 00:00:00',114.79,1415,1476,74.02,'Intermediate' from dual union all
select 11693,'011693','1980-012C','Kosmos-1158','Strela-1M',50222,'In Earth orbit','','1980-09-17 00:00:00',115,1434,1477,74.02,'Intermediate' from dual union all
select 11694,'011694','1980-012D','Kosmos-1159','Strela-1M',50222,'In Earth orbit','','1980-09-07 00:00:00',115.2,1450,1479,74.02,'Intermediate' from dual union all
select 11695,'011695','1980-012E','Kosmos-1160','Strela-1M',50222,'In Earth orbit','','1980-06-29 00:00:00',115.42,1466,1483,74.02,'Intermediate' from dual union all
select 11696,'011696','1980-012F','Kosmos-1161','Strela-1M',50222,'In Earth orbit','','1980-09-01 00:00:00',115.66,1468,1503,74.02,'Intermediate' from dual union all
select 11697,'011697','1980-012G','Kosmos-1162','Strela-1M',50222,'In Earth orbit','','1980-10-13 00:00:00',115.88,1472,1518,74.02,'Intermediate' from dual union all
select 11698,'011698','1980-012H','Kosmos-1163','Strela-1M',50222,'In Earth orbit','','1980-11-02 00:00:00',116.14,1471,1543,74.02,'Intermediate' from dual union all
select 11699,'011699','1980-012J','S3M','S3M',50222,'In Earth orbit','','1980-09-05 00:00:00',117.79,1469,1695,74.02,'Intermediate' from dual union all
select 117,'000117','61 Omicron 2','Injun','Injun',7273,'In Earth orbit','','1961-07-31 00:00:00',103.81,882,996,66.82,'Intermediate' from dual union all
select 1170,'001170','1965-012CA','deb Kosmos-57','',11762,'Reentered','1965-03-04 00:00:00','1965-03-01 00:00:00',88.63,135,279,65.15,'Intermediate' from dual union all
select 11700,'011700','1980-013A','Kosmos-1164','Oko',50223,'Reentered Attached','1980-02-12 00:00:00','1980-02-12 00:00:00',92.45,212,578,62.81,'Intermediate' from dual union all
select 11701,'011701','1980-013B','11S510','11S510',50223,'Reentered','1980-03-10 00:00:00','1980-02-19 00:00:00',91.84,207,523,62.81,'Intermediate' from dual union all
select 11702,'011702','1980-013C','BOZ','BOZ',50223,'Reentered','1980-04-03 00:00:00','1980-02-19 00:00:00',92.74,232,586,62.86,'Intermediate' from dual union all
select 11703,'011703','1980-014A','Solar Maximum Mission','SMM',50224,'Reentered','1989-12-02 00:00:00','1980-03-15 00:00:00',96.01,566,569,28.51,'Intermediate' from dual union all
select 11704,'011704','1980-014B','Delta 151','Delta SSPU DSV-3P-4',50224,'Reentered','1989-05-16 00:00:00','1988-06-14 00:00:00',93.37,436,443,28.5,'Intermediate' from dual union all
select 11705,'011705','1980-011B','SVS Stage 2','Star 37E S/N 40073',50221,'In Earth orbit','','1980-03-11 00:00:00',339.74,170,19331,63.09,'Highly Ellip' from dual union all
select 11706,'011706','1980-015A','Tansei-4','MS-T4',50225,'Reentered','1983-05-12 00:00:00','1980-03-19 00:00:00',95.82,517,600,38.69,'Intermediate' from dual union all
select 11707,'011707','1980-015B','M-3A','M-3A',50225,'Reentered','1984-01-20 00:00:00','1980-03-18 00:00:00',95.83,517,601,38.68,'Intermediate' from dual;
insert into SATELLITE
select 11708,'011708','1980-016A','Raduga','Gran'' No. 16L',50226,'In Earth orbit','','1980-03-31 00:00:00',1436.09,35712,35861,.33,'Stationary' from dual union all
select 11709,'011709','1980-016B','8S812','8S812',50226,'Reentered','1980-02-22 00:00:00','1980-02-21 00:00:00',87.9,163,179,51.62,'Intermediate' from dual union all
select 1171,'001171','1965-012CB','deb Kosmos-57','',11762,'Reentered','1965-03-05 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 11710,'011710','1980-016C','Perekhodnik','Sredniy perekhodnik',50226,'Reentered','1980-02-22 00:00:00','1980-02-21 00:00:00',88.01,159,193,51.65,'Intermediate' from dual union all
select 11711,'011711','1980-009C','Blenda','Sunshade',50219,'Reentered','1980-03-13 00:00:00','1980-02-21 00:00:00',91.14,292,369,72.87,'Intermediate' from dual union all
select 11712,'011712','1980-009D','KDU part','KDU part',50219,'Reentered','1980-03-02 00:00:00','1980-02-21 00:00:00',91.27,293,381,72.86,'Intermediate' from dual union all
select 11713,'011713','1980-017A','Kosmos-1165','Zenit-4MKM',50227,'Landed','1980-03-05 00:00:00','1980-02-23 00:00:00',89.66,168,348,72.88,'Intermediate' from dual union all
select 11714,'011714','1980-017B','11S510','11S510',50227,'Reentered','1980-02-25 00:00:00','1980-02-24 00:00:00',88.6,151,260,72.87,'Intermediate' from dual union all
select 11715,'011715','1980-018A','Ayame 2','ECS-b',50228,'In Earth orbit','','1980-02-23 00:00:00',807.2,9185,35494,8.5,'Medium' from dual union all
select 11716,'011716','1980-018B','N-I Stage 2 S/N N-6(F)','LE-3',50228,'Reentered','1980-02-28 00:00:00','1980-02-24 00:00:00',89.21,199,272,30.44,'Intermediate' from dual union all
select 11717,'011717','1980-009E','KDU','KDU',50219,'Reentered','1980-04-10 00:00:00','1980-03-09 00:00:00',90.83,279,352,72.86,'Intermediate' from dual union all
select 11718,'011718','1980-018C','Star 37N S/N 14007','Star 37N',50228,'Reentered','2009-04-06 00:00:00','1980-03-26 00:00:00',623.96,224,35396,24.48,'GEO Transfer' from dual union all
select 11719,'011719','1980-004B','Centaur AC-49','Centaur D-1AR AC-49',50214,'Reentered','1990-01-05 00:00:00','1980-02-20 00:00:00',595.47,186,33950,26.08,'Highly Ellip' from dual union all
select 1172,'001172','1965-012CC','deb Kosmos-57','',11762,'Reentered','1965-03-05 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 11720,'011720','1980-019A','OPS 7245','PARCAE 3',50229,'In Earth orbit','','1980-06-29 00:00:00',107.46,1056,1163,63.48,'Intermediate' from dual union all
select 11721,'011721','1980-019B','MSD plume shield','',50229,'In Earth orbit','','1997-11-22 00:00:00',106.12,663,1431,63.46,'Intermediate' from dual union all
select 11722,'011722','1980-020A','Kosmos-1166','Zenit-6',50230,'Landed','1980-03-18 00:00:00','1980-03-06 00:00:00',92.24,354,414,72.85,'Intermediate' from dual union all
select 11723,'011723','1980-020B','11S510','11S510',50230,'Reentered','1980-03-13 00:00:00','1980-03-06 00:00:00',89.82,188,344,72.84,'Intermediate' from dual union all
select 11724,'011724','1980-020C','Sensor cover','Zenit-6 sensor cover',50230,'Reentered','1980-03-10 00:00:00','1980-03-09 00:00:00',89.19,178,291,72.84,'Intermediate' from dual union all
select 11725,'011725','1980-012K','Radar error','',50222,'Spurious','1980-03-05 00:00:00','1980-03-04 00:00:00',115.6,1466,1500,74,'Intermediate' from dual union all
select 11726,'011726','1980-017C','Blenda','Sunshade',50227,'Reentered','1980-03-07 00:00:00','1980-03-05 00:00:00',89.21,153,318,72.86,'Intermediate' from dual union all
select 11727,'011727','1980-017D','KDU','KDU',50227,'Reentered','1980-03-06 00:00:00','1980-03-06 00:00:00',88.99,161,289,72.78,'Intermediate' from dual union all
select 11728,'011728','1980-016D','Blok DM','11S86 No. 16L',50226,'In Earth orbit','','1993-06-05 00:00:00',1475.04,36449,36642,11.29,'Inclined Drift' from dual union all
select 11729,'011729','1980-021A','Kosmos-1167','US-P',50231,'Reentered','1981-10-01 00:00:00','1980-04-15 00:00:00',93.28,429,441,65.02,'Intermediate' from dual union all
select 1173,'001173','1965-012CD','deb Kosmos-57','',11762,'Reentered','1965-03-04 00:00:00','1965-03-02 00:00:00',88.36,152,235,64.93,'Intermediate' from dual union all
select 11730,'011730','1980-021B','11S692','11S692',50231,'Reentered','1980-03-14 00:00:00','1980-03-14 00:00:00',87.91,92,251,65,'Intermediate' from dual union all
select 11731,'011731','1980-019C','EP 1','SSU',50229,'In Earth orbit','','1997-11-22 00:00:00',107.42,730,1485,63.46,'Intermediate' from dual union all
select 11732,'011732','1980-019D','EP 2','SSU',50229,'In Earth orbit','','1997-11-22 00:00:00',107.41,730,1484,63.46,'Intermediate' from dual union all
select 11733,'011733','1980-019E','EP 1 cover','SSU cover',50229,'In Earth orbit','','1995-01-03 00:00:00',107.41,905,1308,63.44,'Intermediate' from dual union all
select 11734,'011734','1980-019F','EP 2 cover','SSU cover',50229,'In Earth orbit','','1997-11-21 00:00:00',106.89,718,1448,63.47,'Intermediate' from dual union all
select 11735,'011735','1980-022A','Kosmos-1168','Tsikada',50232,'In Earth orbit','','1980-04-16 00:00:00',104.84,961,1012,82.95,'Intermediate' from dual union all
select 11736,'011736','1980-022B','S3M','S3M',50232,'In Earth orbit','','1991-03-28 00:00:00',104.68,956,1003,82.94,'Intermediate' from dual union all
select 11737,'011737','1980-020D','KDU','KDU',50230,'Reentered','1980-09-04 00:00:00','1980-04-04 00:00:00',92.04,348,402,72.84,'Intermediate' from dual union all
select 11738,'011738','1980-020E','Blenda','Sunshade',50230,'Reentered','1980-04-28 00:00:00','1980-03-18 00:00:00',92.06,344,407,72.85,'Intermediate' from dual union all
select 11739,'011739','1980-020F','KDU part','KDU part',50230,'Reentered','1980-04-09 00:00:00','1980-03-18 00:00:00',92.23,353,415,72.85,'Intermediate' from dual union all
select 1174,'001174','1965-012CE','deb Kosmos-57','',11762,'Reentered','1965-03-22 00:00:00','1965-03-04 00:00:00',91.03,165,486,64.77,'Intermediate' from dual union all
select 11740,'011740','1977-097BR','Soyuz T BO','7K-ST No. 6L BO',42225,'Reentered','1980-05-28 00:00:00','1980-03-27 00:00:00',91.35,340,341,51.62,'Intermediate' from dual union all
select 11741,'011741','1980-023A','Kosmos-1169','Vektor',50233,'Reentered','1983-03-03 00:00:00','1980-04-26 00:00:00',94.45,471,513,65.84,'Intermediate' from dual union all
select 11742,'011742','1980-023B','S3M','S3M',50233,'Reentered','1981-11-04 00:00:00','1980-04-26 00:00:00',94.35,465,509,65.85,'Intermediate' from dual union all
select 11743,'011743','1980-024A','Progress-8','Progress 7K-TG No. 108',50234,'Deorbited','1980-04-26 00:00:00','1980-04-03 00:00:00',91.38,340,345,51.62,'Intermediate' from dual union all
select 11744,'011744','1980-024B','11S510','11S510',50234,'Reentered','1980-03-29 00:00:00','1980-03-29 00:00:00',89.16,226,240,51.59,'Intermediate' from dual union all
select 11745,'011745','1980-019G','EP 3','SSU',50229,'In Earth orbit','','1997-11-22 00:00:00',107.42,765,1450,63.47,'Intermediate' from dual union all
select 11746,'011746','1980-019H','EP 3 cover','SSU cover',50229,'In Earth orbit','','1997-11-21 00:00:00',106.93,728,1441,63.46,'Intermediate' from dual union all
select 11747,'011747','1980-025A','Kosmos-1170','Zenit-4MKM',50235,'Landed','1980-04-12 00:00:00','1980-04-03 00:00:00',89.37,157,329,70.38,'Intermediate' from dual union all
select 11748,'011748','1980-025B','11S510','11S510',50235,'Reentered','1980-04-05 00:00:00','1980-04-03 00:00:00',88.94,156,289,70.39,'Intermediate' from dual union all
select 11749,'011749','1980-025C','Sensor cover','Zenit-4MK sensor cover',50235,'Reentered','1980-04-03 00:00:00','1980-04-01 00:00:00',88.94,164,281,70.39,'Intermediate' from dual union all
select 1175,'001175','1965-012CF','deb Kosmos-57','',11762,'Reentered','1965-03-05 00:00:00','1996-09-12 00:00:00',101.6,819,849,114.98,'Retrograde' from dual union all
select 11750,'011750','1980-026A','Kosmos-1171','Lira',50236,'In Earth orbit','','1980-05-03 00:00:00',104.85,965,1010,65.84,'Intermediate' from dual union all
select 11751,'011751','1980-026B','S3M','S3M',50236,'In Earth orbit','','1991-02-27 00:00:00',104.64,963,993,65.84,'Intermediate' from dual union all
select 11752,'011752','1980-026C','deb Kosmos-1171','DS-P1-M part [RCS 0.74]',50236,'In Earth orbit','','1980-06-30 00:00:00',104.85,964,1011,65.84,'Intermediate' from dual union all
select 11753,'011753','1980-027A','Soyuz-35','Soyuz 7K-T No. 51',50237,'Landed','1980-06-03 00:00:00','1980-04-18 00:00:00',91.29,332,344,51.62,'Intermediate' from dual union all
select 11754,'011754','1980-027B','11S510','11S510',50237,'Reentered','1980-04-11 00:00:00','1980-04-10 00:00:00',88.67,189,228,51.65,'Intermediate' from dual union all
select 11755,'011755','1980-025D','KDU','KDU',50235,'Reentered','1980-04-14 00:00:00','1980-04-12 00:00:00',89.16,154,312,70.37,'Intermediate' from dual union all
select 11756,'011756','1980-025E','Blenda','Sunshade',50235,'Reentered','1980-04-13 00:00:00','1980-04-12 00:00:00',89.29,157,322,70.36,'Intermediate' from dual union all
select 11757,'011757','1980-025F','KDU part','KDU part',50235,'Reentered','1980-04-13 00:00:00','1980-04-12 00:00:00',89.17,168,298,70.4,'Intermediate' from dual union all
select 11758,'011758','1980-028A','Kosmos-1172','Oko',50238,'Reentered','1997-12-26 00:00:00','1980-05-13 00:00:00',717.56,646,39698,62.85,'Molniya' from dual union all
select 11759,'011759','1980-028B','11S510','11S510',50238,'Reentered','1980-04-26 00:00:00','1980-04-15 00:00:00',92.07,176,577,62.86,'Intermediate' from dual union all
select 1176,'001176','1965-012CG','deb Kosmos-57','',11762,'Reentered','1965-03-05 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 11760,'011760','1980-028C','BOZ','BOZ',50238,'Reentered','1980-05-08 00:00:00','1980-04-19 00:00:00',91.99,208,537,62.79,'Intermediate' from dual union all
select 11761,'011761','1980-028D','deb Blok-I','',50238,'Reentered','1980-04-20 00:00:00','1980-04-15 00:00:00',92.59,217,586,62.81,'Intermediate' from dual union all
select 11762,'011762','1980-028E','Blok-L','Blok-2BL No. 71648-633',50238,'Reentered','1996-07-13 00:00:00','1980-06-30 00:00:00',722.12,700,39869,63.09,'Molniya' from dual union all
select 11763,'011763','1980-029A','Kosmos-1173','Zenit-4MKM',50239,'Landed','1980-04-28 00:00:00','1980-04-19 00:00:00',89.49,153,346,70.3,'Intermediate' from dual union all
select 11764,'011764','1980-029B','11S510','11S510',50239,'Reentered','1980-04-21 00:00:00','1980-04-19 00:00:00',89.18,161,307,70.31,'Intermediate' from dual union all
select 11765,'011765','1980-030A','Kosmos-1174','IS',50240,'In Earth orbit','','1980-04-19 00:00:00',105.51,379,1658,66.13,'Intermediate' from dual union all
select 11766,'011766','1980-030B','11S692','11S692',50240,'Reentered','1980-04-20 00:00:00','1980-04-19 00:00:00',89.66,132,384,65.12,'Intermediate' from dual union all
select 11767,'011767','1980-031A','BOZ','BOZ',50241,'Reentered','1980-05-28 00:00:00','1980-04-26 00:00:00',91.26,246,427,62.82,'Intermediate' from dual union all
select 11768,'011768','1980-031B','11S510','11S510',50241,'Reentered','1980-05-02 00:00:00','1980-04-21 00:00:00',90.74,206,416,62.84,'Intermediate' from dual union all
select 11769,'011769','1980-031C','Kosmos-1175','Molniya-3 No. 26',50241,'Reentered','1980-09-29 00:00:00','1980-05-20 00:00:00',91.95,306,435,62.8,'Intermediate' from dual union all
select 1177,'001177','1965-012CH','deb Kosmos-57','',11762,'Reentered','1965-03-27 00:00:00','1965-03-04 00:00:00',91.03,182,469,64.81,'Intermediate' from dual union all
select 11770,'011770','1980-031D','Blok-L','Blok-ML No. 71628-508',50241,'Reentered','1980-05-30 00:00:00','1980-04-26 00:00:00',91.5,260,437,62.81,'Intermediate' from dual union all
select 11771,'011771','1980-030C','deb Kosmos-1174','',50240,'Reentered','1980-12-30 00:00:00','1980-12-29 00:00:00',88.61,170,242,66.06,'Intermediate' from dual union all
select 11772,'011772','1980-030D','deb Kosmos-1174','',50240,'Reentered','1981-04-19 00:00:00','1981-04-18 00:00:00',90.4,238,351,66.04,'Intermediate' from dual union all
select 11773,'011773','1980-030E','deb Kosmos-1174','',50240,'Reentered','1981-01-17 00:00:00','1980-05-23 00:00:00',103.66,323,1541,66.06,'Intermediate' from dual union all
select 11774,'011774','1980-030F','deb Kosmos-1174','',50240,'Reentered','1981-04-17 00:00:00','1981-04-14 00:00:00',95,338,699,66.06,'Intermediate' from dual union all
select 11775,'011775','1980-030G','deb Kosmos-1174','',50240,'Reentered','1989-08-10 00:00:00','1989-08-09 00:00:00',87.43,135,160,66.04,'Intermediate' from dual union all
select 11776,'011776','1980-030H','deb Kosmos-1174','',50240,'Reentered','1982-02-17 00:00:00','1982-02-15 00:00:00',90.09,239,319,66.09,'Intermediate' from dual union all
select 11777,'011777','1980-030J','deb Kosmos-1174','',50240,'Reentered','2001-09-06 00:00:00','1980-06-29 00:00:00',106.6,484,1655,66.05,'Intermediate' from dual union all
select 11778,'011778','1980-030K','deb Kosmos-1174','',50240,'In Earth orbit','','1980-06-30 00:00:00',107.9,634,1626,66.52,'Intermediate' from dual union all
select 11779,'011779','1980-030L','deb Kosmos-1174','',50240,'Reentered','1983-03-10 00:00:00','1983-03-10 00:00:00',88.27,172,206,66,'Intermediate' from dual union all
select 1178,'001178','1965-012CJ','deb Kosmos-57','',11762,'Reentered','1965-03-05 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 11780,'011780','1980-030M','deb Kosmos-1174','',50240,'Reentered','1981-02-17 00:00:00','1980-06-04 00:00:00',106.18,482,1618,66.16,'Intermediate' from dual union all
select 11781,'011781','1980-030N','deb Kosmos-1174','',50240,'Reentered','2005-06-12 00:00:00','1993-08-04 00:00:00',100.2,392,1143,66.22,'Intermediate' from dual union all
select 11782,'011782','1980-030P','deb Kosmos-1174','',50240,'Reentered','1984-06-24 00:00:00','1984-06-24 00:00:00',87.69,155,166,65.95,'Intermediate' from dual union all
select 11783,'011783','1980-032A','Navstar 6','Navstar GPS SVN 6',50242,'In Earth orbit','','1980-05-27 00:00:00',717.94,20006,20357,62.88,'Medium' from dual union all
select 11784,'011784','1980-033A','Progress-9','Progress 7K-TG No. 109',50243,'Deorbited','1980-05-22 00:00:00','1980-05-13 00:00:00',91.34,326,355,51.63,'Intermediate' from dual union all
select 11785,'011785','1980-033B','11S510','11S510',50243,'Reentered','1980-04-29 00:00:00','1980-04-27 00:00:00',88.59,178,232,51.64,'Intermediate' from dual union all
select 11786,'011786','1980-029C','KDU','KDU',50239,'Reentered','1980-04-30 00:00:00','1980-04-28 00:00:00',89.29,152,327,70.29,'Intermediate' from dual union all
select 11787,'011787','1980-029D','Blenda','Sunshade',50239,'Reentered','1980-04-30 00:00:00','1980-04-28 00:00:00',90.28,166,411,70.32,'Intermediate' from dual union all
select 11788,'011788','1980-034A','Kosmos-1176','US-A reactor',50244,'In Earth orbit','','1980-05-29 00:00:00',89.62,249,263,65.02,'Intermediate' from dual union all
select 11789,'011789','1980-035A','Kosmos-1177','Yantar''-4K1 No. 2',50245,'Landed','1980-06-12 00:00:00','1980-05-09 00:00:00',89.53,164,339,67.14,'Intermediate' from dual union all
select 1179,'001179','1965-012CK','deb Kosmos-57','',11762,'Reentered','1965-03-16 00:00:00','1965-03-05 00:00:00',91.31,177,501,64.77,'Intermediate' from dual union all
select 11790,'011790','1980-035B','11S510','11S510',50245,'Reentered','1980-05-02 00:00:00','1980-05-01 00:00:00',88.97,162,286,67.14,'Intermediate' from dual union all
select 11791,'011791','1980-032B','SVS Stage 2','Star 37E S/N 40075',50242,'Reentered','1996-05-22 00:00:00','1992-10-01 00:00:00',208.06,201,10304,63.13,'Medium' from dual union all
select 11792,'011792','1966-096C','FW4D S/N 00003','[UNID080] RCS 0.7',13503,'In Earth orbit','','1980-07-30 00:00:00',600.74,381,34031,17.99,'GEO Transfer' from dual union all
select 11793,'011793','1980-036A','Kosmos-1178','Zenit-6',50246,'Landed','1980-05-22 00:00:00','1980-05-14 00:00:00',92.24,356,413,72.83,'Intermediate' from dual union all
select 11794,'011794','1980-036B','11S510','11S510',50246,'Reentered','1980-05-14 00:00:00','1980-05-10 00:00:00',89.77,185,342,72.82,'Intermediate' from dual union all
select 11795,'011795','1980-036C','Sensor cover','Zenit-4MK sensor cover',50246,'Reentered','1980-05-12 00:00:00','1980-05-07 00:00:00',89.66,187,329,72.82,'Intermediate' from dual union all
select 11796,'011796','1980-037A','Kosmos-1179','Taifun-1B',50247,'Reentered','1989-07-18 00:00:00','1980-06-13 00:00:00',103.4,300,1539,82.97,'Intermediate' from dual union all
select 11797,'011797','1980-037B','S3M','S3M',50247,'Reentered','1983-01-11 00:00:00','1980-06-13 00:00:00',103.18,297,1520,82.97,'Intermediate' from dual union all
select 11798,'011798','1980-038A','Kosmos-1180','Orion',50248,'Landed','1980-05-26 00:00:00','1980-05-19 00:00:00',89.73,237,286,62.81,'Intermediate' from dual;
insert into SATELLITE
select 11799,'011799','1980-038B','11S510','11S510',50248,'Reentered','1980-05-23 00:00:00','1980-05-18 00:00:00',89.3,234,246,62.79,'Intermediate' from dual union all
select 118,'000118','61 Omicron 3','Ablestar 008','AJ10-104S S/N 008',7273,'In Earth orbit','','1961-09-11 00:00:00',103.36,851,983,66.82,'Intermediate' from dual union all
select 1180,'001180','1965-012CL','deb Kosmos-57','',11762,'Reentered','1965-03-04 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 11800,'011800','1980-038C','Sensor cover','Zenit-4MT sensor cover',50248,'Reentered','1980-05-17 00:00:00','1980-05-15 00:00:00',89.42,228,263,62.8,'Intermediate' from dual union all
select 11801,'011801','1980-016E','SOZ','SOZ',50226,'Reentered','1981-01-12 00:00:00','1980-04-03 00:00:00',646.93,239,36561,47.26,'GEO Transfer' from dual union all
select 11802,'011802','1980-016F','SOZ','SOZ',50226,'Reentered','1982-04-15 00:00:00','1980-03-26 00:00:00',649.01,298,36608,47.32,'GEO Transfer' from dual union all
select 11803,'011803','1980-039A','Kosmos-1181','Parus',50249,'In Earth orbit','','1980-09-03 00:00:00',104.9,973,1007,82.95,'Intermediate' from dual union all
select 11804,'011804','1980-039B','S3M','S3M',50249,'In Earth orbit','','1991-03-28 00:00:00',104.72,969,994,82.98,'Intermediate' from dual union all
select 11805,'011805','1980-036D','Blenda','Sunshade',50246,'Reentered','1980-07-07 00:00:00','1980-06-07 00:00:00',91.48,320,375,72.82,'Intermediate' from dual union all
select 11806,'011806','1980-036E','KDU','KDU',50246,'Reentered','1980-11-16 00:00:00','1980-06-11 00:00:00',92.05,350,401,72.82,'Intermediate' from dual union all
select 11807,'011807','1980-036F','KDU part','KDU part',50246,'Reentered','1980-06-22 00:00:00','1980-05-22 00:00:00',92.24,357,412,72.82,'Intermediate' from dual union all
select 11808,'011808','1980-040A','Kosmos-1182','Fram',50250,'Landed','1980-06-05 00:00:00','1980-05-25 00:00:00',88.92,208,234,82.34,'Intermediate' from dual union all
select 11809,'011809','1980-040B','11S510','11S510',50250,'Reentered','1980-05-26 00:00:00','1980-05-25 00:00:00',88.43,186,208,82.33,'Intermediate' from dual union all
select 1181,'001181','1965-012CM','deb Kosmos-57','',11762,'Reentered','1965-03-06 00:00:00','1965-03-04 00:00:00',89.73,157,366,64.39,'Intermediate' from dual union all
select 11810,'011810','1980-015C','Tansei-4 yo-yo mass','',50225,'Reentered','1980-08-27 00:00:00','1980-04-16 00:00:00',95.02,489,551,38.46,'Intermediate' from dual union all
select 11811,'011811','1980-041A','Soyuz-36','Soyuz 7K-T No. 52',50251,'Landed','1980-07-31 00:00:00','1980-06-25 00:00:00',91.21,327,340,51.63,'Intermediate' from dual union all
select 11812,'011812','1980-041B','11S510','11S510',50251,'Reentered','1980-05-29 00:00:00','1980-05-28 00:00:00',88.42,172,220,51.6,'Intermediate' from dual union all
select 11813,'011813','1980-038D','KBE part','KBE part',50248,'Reentered','1980-05-30 00:00:00','1980-05-27 00:00:00',88.91,205,237,62.82,'Intermediate' from dual union all
select 11814,'011814','1980-038E','KB electropitaniya','KBE',50248,'Reentered','1980-05-31 00:00:00','1980-05-28 00:00:00',91.13,263,397,62.8,'Intermediate' from dual union all
select 11815,'011815','1980-038F','KBE part','KBE part',50248,'Reentered','1980-05-31 00:00:00','1980-05-27 00:00:00',88.98,197,252,62.79,'Intermediate' from dual union all
select 11816,'011816','1980-042A','Kosmos-1183','Zenit-6',50252,'Landed','1980-06-11 00:00:00','1980-05-30 00:00:00',92.25,356,413,72.89,'Intermediate' from dual union all
select 11817,'011817','1980-042B','11S510','11S510',50252,'Reentered','1980-06-05 00:00:00','1980-05-30 00:00:00',89.89,189,349,72.89,'Intermediate' from dual union all
select 11818,'011818','1980-042C','Sensor cover','Zenit-6 sensor cover',50252,'Reentered','1980-06-09 00:00:00','1980-05-30 00:00:00',89.68,205,313,72.94,'Intermediate' from dual union all
select 11819,'011819','1980-043A','NOAA B','NOAA B',50253,'Reentered','1981-05-03 00:00:00','1980-06-28 00:00:00',101.34,267,1378,92.23,'Polar' from dual union all
select 1182,'001182','1965-012CN','deb Kosmos-57','',11762,'Reentered','1965-03-07 00:00:00','1965-02-28 00:00:00',91.66,199,513,64.57,'Intermediate' from dual union all
select 11820,'011820','1980-043B','NOAA-B solar array band','',50253,'Reentered','1980-09-11 00:00:00','1980-06-28 00:00:00',98.87,265,1145,92.27,'Polar' from dual union all
select 11821,'011821','1980-044A','Kosmos-1184','Tselina-D',50254,'Reentered','2002-04-29 00:00:00','1980-08-28 00:00:00',97.35,619,645,81.25,'Intermediate' from dual union all
select 11822,'011822','1980-044B','8A92ME','8A92ME',50254,'In Earth orbit','','1991-03-28 00:00:00',96.59,564,627,81.23,'Intermediate' from dual union all
select 11823,'011823','1980-040C','KDU','KDU',50250,'Reentered','1980-06-10 00:00:00','1980-06-05 00:00:00',88.77,209,219,82.33,'Intermediate' from dual union all
select 11824,'011824','1980-040D','Blenda','Sunshade',50250,'Reentered','1980-06-09 00:00:00','1980-06-05 00:00:00',88.4,190,201,82.42,'Intermediate' from dual union all
select 11825,'011825','1980-045A','Soyuz T-2','Soyuz 7K-ST No. 7L',50255,'Landed','1980-06-09 00:00:00','1980-06-06 00:00:00',90.3,262,317,51.62,'Intermediate' from dual union all
select 11826,'011826','1980-045B','11S510','11S510',50255,'Reentered','1980-06-07 00:00:00','1980-06-06 00:00:00',88.44,187,208,51.63,'Intermediate' from dual union all
select 11827,'011827','1980-046A','Kosmos-1185','Resurs-F1 17F41 No. 12',50256,'Landed','1980-06-20 00:00:00','1980-06-08 00:00:00',89.81,258,273,82.34,'Intermediate' from dual union all
select 11828,'011828','1980-046B','11S510','11S510',50256,'Reentered','1980-06-11 00:00:00','1980-06-08 00:00:00',88.82,194,239,82.33,'Intermediate' from dual union all
select 11829,'011829','1980-047A','Kosmos-1186','Romb',50257,'Reentered','1982-01-01 00:00:00','1980-07-06 00:00:00',94.44,468,515,74.02,'Intermediate' from dual union all
select 1183,'001183','1965-012CP','deb Kosmos-57','',11762,'Reentered','1965-03-08 00:00:00','1965-03-03 00:00:00',88.65,138,277,64.68,'Intermediate' from dual union all
select 11830,'011830','1980-047B','S3M','S3M',50257,'Reentered','1981-12-16 00:00:00','1980-07-06 00:00:00',94.35,460,515,74.02,'Intermediate' from dual union all
select 11831,'011831','1980-047C','deb Kosmos-1186','',50257,'Reentered','1980-06-17 00:00:00','1980-06-14 00:00:00',94.12,441,510,74.02,'Intermediate' from dual union all
select 11832,'011832','1977-097BS','Soyuz T-2 BO','7K-ST No. 7L BO',42225,'Reentered','1980-09-05 00:00:00','1980-07-08 00:00:00',90.86,311,323,51.63,'Intermediate' from dual union all
select 11833,'011833','1980-042D','Blenda','Sunshade',50252,'Reentered','1980-07-13 00:00:00','1980-06-11 00:00:00',92.24,356,413,72.9,'Intermediate' from dual union all
select 11834,'011834','1980-042E','KDU part','KDU part',50252,'Reentered','1980-08-03 00:00:00','1980-06-29 00:00:00',91.46,320,372,72.89,'Intermediate' from dual union all
select 11835,'011835','1980-042F','KDU part','KDU part',50252,'Reentered','1980-06-14 00:00:00','1980-06-11 00:00:00',92.09,341,413,72.82,'Intermediate' from dual union all
select 11836,'011836','1980-042G','KDU','KDU',50252,'Reentered','1980-12-04 00:00:00','1980-07-08 00:00:00',92.04,349,401,72.89,'Intermediate' from dual union all
select 11837,'011837','1980-048A','Kosmos-1187','Zenit-6',50258,'Landed','1980-06-26 00:00:00','1980-06-14 00:00:00',89.63,226,287,72.85,'Intermediate' from dual union all
select 11838,'011838','1980-048B','11S510','11S510',50258,'Reentered','1980-06-17 00:00:00','1980-06-14 00:00:00',89.17,186,281,72.85,'Intermediate' from dual union all
select 11839,'011839','1980-048C','Sensor cover','Zenit-6 sensor cover',50258,'Reentered','1980-06-17 00:00:00','1980-06-12 00:00:00',88.73,180,244,72.87,'Intermediate' from dual union all
select 1184,'001184','1965-012CQ','deb Kosmos-57','',11762,'Reentered','1965-03-07 00:00:00','1965-03-05 00:00:00',89,156,295,64.87,'Intermediate' from dual union all
select 11840,'011840','1980-035C','Kosmos-1177 AO part','Yantar''-4K1 AO part',50245,'Reentered','1980-06-16 00:00:00','1980-06-13 00:00:00',88.1,147,214,67.16,'Intermediate' from dual union all
select 11841,'011841','1980-049A','Gorizont','Gorizont No. 15L',50259,'In Earth orbit','','1980-07-22 00:00:00',1436.01,35742,35828,.75,'Stationary' from dual union all
select 11842,'011842','1980-049B','8S812','8S812',50259,'Reentered','1980-06-15 00:00:00','1980-06-14 00:00:00',88.02,168,185,51.64,'Intermediate' from dual union all
select 11843,'011843','1980-049C','Perekhodnik','Sredniy perekhodnik',50259,'Reentered','1980-06-14 00:00:00','1980-06-14 00:00:00',88.15,177,189,51.62,'Intermediate' from dual union all
select 11844,'011844','1980-050A','Kosmos-1188','Oko',50260,'Reentered','2013-05-24 00:00:00','1980-07-14 00:00:00',717.56,665,39679,62.89,'Molniya' from dual union all
select 11845,'011845','1980-050C','11S510','11S510',50260,'Reentered','1980-07-17 00:00:00','1980-06-21 00:00:00',92.04,210,540,62.8,'Intermediate' from dual union all
select 11846,'011846','1980-050D','BOZ','BOZ',50260,'Reentered','1980-07-06 00:00:00','1980-06-21 00:00:00',91.92,185,553,62.88,'Intermediate' from dual union all
select 11847,'011847','1980-050B','Blok-L','Blok-2BL No. 34',50260,'In Earth orbit','','1980-07-15 00:00:00',722.98,659,39952,62.95,'Molniya' from dual union all
select 11848,'011848','1980-051A','Meteor-Priroda','Meteor-Priroda No. 3-1',50261,'Reentered','1992-03-01 00:00:00','1980-07-18 00:00:00',97.15,583,661,97.94,'Sun-Synch' from dual union all
select 11849,'011849','1980-051B','8A92ME','8A92ME',50261,'Reentered','2016-05-13 00:00:00','1991-03-28 00:00:00',96.46,567,612,97.55,'Sun-Synch' from dual union all
select 1185,'001185','1965-012CR','deb Kosmos-57','',11762,'Reentered','1965-03-11 00:00:00','1965-03-06 00:00:00',89.54,160,344,64.79,'Intermediate' from dual union all
select 11850,'011850','1980-052A','OPS 3123','HEXAGON SV-16',50262,'Deorbited','1981-03-06 00:00:00','1980-07-18 00:00:00',88.58,164,245,96.44,'Sun-Synch' from dual union all
select 11851,'011851','1980-052B','Titan 23D-16 Stage 2','LR91AJ11 23D-16',50262,'Reentered','1980-06-20 00:00:00','1980-06-19 00:00:00',88.52,162,241,96.46,'Sun-Synch' from dual union all
select 11852,'011852','1980-052C','OPS 1292','P801-6',50262,'In Earth orbit','','1989-08-27 00:00:00',112.22,1329,1329,96.6,'Sun-Synch' from dual union all
select 11853,'011853','1980-046C','KDU','KDU',50256,'Reentered','1980-07-04 00:00:00','1980-06-20 00:00:00',89.66,249,267,82.33,'Intermediate' from dual union all
select 11854,'011854','1980-046D','Blenda','Sunshade',50256,'Reentered','1980-06-25 00:00:00','1980-06-20 00:00:00',89.51,236,264,82.34,'Intermediate' from dual union all
select 11855,'011855','1965-065T','deb NNS O-5','',11815,'Reentered','1990-06-15 00:00:00','1980-06-20 00:00:00',108.02,1082,1188,90,'Polar' from dual union all
select 11856,'011856','1980-053A','Molniya-1','Molniya-1',50263,'Reentered','1991-04-01 00:00:00','1980-07-21 00:00:00',717.82,632,39725,62.9,'Molniya' from dual union all
select 11857,'011857','1980-053B','11S510','11S510',50263,'Reentered','1980-07-28 00:00:00','1980-07-07 00:00:00',91.74,205,515,62.84,'Intermediate' from dual union all
select 11858,'011858','1980-053C','BOZ','BOZ',50263,'Reentered','1980-07-25 00:00:00','1980-06-30 00:00:00',92.27,198,574,62.84,'Intermediate' from dual union all
select 11859,'011859','1980-049D','SOZ','SOZ',50259,'Reentered','1982-04-20 00:00:00','1980-07-18 00:00:00',645.82,266,36477,47.19,'GEO Transfer' from dual union all
select 1186,'001186','1965-012CS','deb Kosmos-57','',11762,'Reentered','1965-03-01 00:00:00','1965-02-22 00:00:00',91.97,365,378,64.7,'Intermediate' from dual union all
select 11860,'011860','1980-049E','SOZ','SOZ',50259,'Reentered','1982-03-22 00:00:00','1980-07-15 00:00:00',647.46,320,36507,47.24,'GEO Transfer' from dual union all
select 11861,'011861','1980-053D','Blok-L','Blok-ML No. 71645-452',50263,'Reentered','1991-03-13 00:00:00','1980-07-21 00:00:00',733.45,632,40493,62.87,'Molniya' from dual union all
select 11862,'011862','1980-049F','Blok DM No. 27L','11S86 No. 27L',50259,'In Earth orbit','','1993-06-04 00:00:00',1470.3,36320,36587,10.74,'Inclined Drift' from dual union all
select 11863,'011863','1980-054A','Kosmos-1189','Zenit-6',50264,'Landed','1980-07-10 00:00:00','1980-06-28 00:00:00',89.63,225,287,72.86,'Intermediate' from dual union all
select 11864,'011864','1980-054B','11S510','11S510',50264,'Reentered','1980-07-01 00:00:00','1980-06-28 00:00:00',88.83,180,254,72.85,'Intermediate' from dual union all
select 11865,'011865','1980-048D','Blenda','Sunshade',50258,'Reentered','1980-06-29 00:00:00','1980-06-26 00:00:00',89.41,218,272,72.84,'Intermediate' from dual union all
select 11866,'011866','1980-048E','KDU','KDU',50258,'Reentered','1980-07-18 00:00:00','1980-06-26 00:00:00',89.5,214,286,72.82,'Intermediate' from dual union all
select 11867,'011867','1980-055A','Progress-10','Progress 7K-TG No. 110',50265,'Deorbited','1980-07-19 00:00:00','1980-07-08 00:00:00',91.11,321,337,51.62,'Intermediate' from dual union all
select 11868,'011868','1980-055B','11S510','11S510',50265,'Reentered','1980-07-02 00:00:00','1980-07-01 00:00:00',88.17,164,204,51.61,'Intermediate' from dual union all
select 11869,'011869','1980-056A','Kosmos-1190','Strela-2M',50266,'In Earth orbit','','1980-09-09 00:00:00',100.81,789,805,74.04,'Intermediate' from dual union all
select 1187,'001187','1965-012CT','deb Kosmos-57','',11762,'Reentered','1965-03-01 00:00:00','1965-02-25 00:00:00',89.19,137,332,64.6,'Intermediate' from dual union all
select 11870,'011870','1980-056B','S3M','S3M',50266,'In Earth orbit','','1990-03-26 00:00:00',100.55,773,797,74.05,'Intermediate' from dual union all
select 11871,'011871','1980-057A','Kosmos-1191','Oko',50267,'In Earth orbit','','1980-08-02 00:00:00',717.6,641,39705,62.66,'Molniya' from dual union all
select 11872,'011872','1980-057B','11S510','11S510',50267,'Reentered','1980-08-02 00:00:00','1980-07-09 00:00:00',92.15,207,553,62.71,'Intermediate' from dual union all
select 11873,'011873','1980-057C','BOZ','BOZ',50267,'Reentered','1980-07-31 00:00:00','1980-07-10 00:00:00',92.17,194,568,62.8,'Intermediate' from dual union all
select 11874,'011874','1980-043C','NOAA-B solar array band','',50253,'Reentered','1980-08-27 00:00:00','1980-07-04 00:00:00',97.95,255,1067,92.23,'Polar' from dual union all
select 11875,'011875','1980-058A','Kosmos-1192','Strela-1M',50268,'In Earth orbit','','1980-09-06 00:00:00',114.56,1397,1473,74.01,'Intermediate' from dual union all
select 11876,'011876','1980-058B','Kosmos-1193','Strela-1M',50268,'In Earth orbit','','1992-07-30 00:00:00',114.74,1413,1474,74.01,'Intermediate' from dual union all
select 11877,'011877','1980-058C','Kosmos-1194','Strela-1M',50268,'In Earth orbit','','1993-04-25 00:00:00',114.94,1432,1474,74.01,'Intermediate' from dual union all
select 11878,'011878','1980-058D','Kosmos-1195','Strela-1M',50268,'In Earth orbit','','1992-07-30 00:00:00',115.14,1449,1475,74.01,'Intermediate' from dual union all
select 11879,'011879','1980-058E','Kosmos-1196','Strela-1M',50268,'In Earth orbit','','1992-05-06 00:00:00',115.34,1467,1475,74.02,'Intermediate' from dual union all
select 1188,'001188','1965-012CU','deb Kosmos-57','',11762,'Reentered','1965-03-01 00:00:00','1965-02-23 00:00:00',89.54,137,367,64.81,'Intermediate' from dual union all
select 11880,'011880','1980-058F','Kosmos-1197','Strela-1M',50268,'In Earth orbit','','1992-05-07 00:00:00',115.57,1471,1492,74.02,'Intermediate' from dual union all
select 11881,'011881','1980-058G','Kosmos-1198','Strela-1M',50268,'In Earth orbit','','1992-05-07 00:00:00',115.77,1473,1507,74.02,'Intermediate' from dual union all
select 11882,'011882','1980-058H','Kosmos-1199','Strela-1M',50268,'In Earth orbit','','1992-05-06 00:00:00',116.01,1473,1529,74.02,'Intermediate' from dual union all
select 11883,'011883','1980-058J','S3M','S3M',50268,'In Earth orbit','','1992-04-30 00:00:00',117.65,1469,1681,74.02,'Intermediate' from dual union all
select 11884,'011884','1980-059A','Kosmos-1200','Zenit-6',50269,'Landed','1980-07-23 00:00:00','1980-07-11 00:00:00',89.63,225,287,72.86,'Intermediate' from dual union all
select 11885,'011885','1980-059B','11S510','11S510',50269,'Reentered','1980-07-15 00:00:00','1980-07-11 00:00:00',89.26,191,285,72.85,'Intermediate' from dual union all
select 11886,'011886','1980-054C','Blenda','Sunshade',50264,'Reentered','1980-07-13 00:00:00','1980-07-10 00:00:00',89.79,218,311,72.86,'Intermediate' from dual union all
select 11887,'011887','1980-054D','KDU','KDU',50264,'Reentered','1980-08-04 00:00:00','1980-07-11 00:00:00',89.97,224,322,72.86,'Intermediate' from dual union all
select 11888,'011888','1980-057D','Blok-L','Blok-L',50267,'In Earth orbit','','1980-08-01 00:00:00',722.1,638,39930,62.7,'Molniya' from dual;
insert into SATELLITE
select 11889,'011889','1980-054E','KDU part','KDU part',50264,'Reentered','1980-07-18 00:00:00','1980-07-13 00:00:00',89.21,205,265,72.87,'Intermediate' from dual union all
select 1189,'001189','1965-012CV','deb Kosmos-57','',11762,'Reentered','1965-03-01 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 11890,'011890','1980-060A','Ekran','Ekran No. 19L',50270,'In Earth orbit','','1980-12-31 00:00:00',1435.93,35736,35831,.06,'Stationary' from dual union all
select 11891,'011891','1980-060B','8S812','8S812',50270,'Reentered','1980-07-16 00:00:00','1980-07-15 00:00:00',88.15,177,189,51.64,'Intermediate' from dual union all
select 11892,'011892','1980-060C','Perekhodnik','Sredniy perekhodnik',50270,'Reentered','1980-07-15 00:00:00','1980-07-15 00:00:00',88.14,166,199,51.61,'Intermediate' from dual union all
select 11893,'011893','1977-097BT','Garbage bag','',42225,'Reentered','1980-09-04 00:00:00','1980-07-15 00:00:00',90.75,305,318,51.62,'Intermediate' from dual union all
select 11894,'011894','1980-061A','Kosmos-1201','Fram',50271,'Landed','1980-07-28 00:00:00','1980-07-17 00:00:00',88.91,209,232,82.34,'Intermediate' from dual union all
select 11895,'011895','1980-061B','11S510','11S510',50271,'Reentered','1980-07-18 00:00:00','1980-07-17 00:00:00',88.59,196,214,82.34,'Intermediate' from dual union all
select 11896,'011896','1980-063A','Molniya-3','Molniya-3 No. 27',50273,'In Earth orbit','','1980-08-20 00:00:00',717.8,446,39910,62.85,'Molniya' from dual union all
select 11897,'011897','1980-063B','11S510','11S510',50273,'Reentered','1980-08-02 00:00:00','1980-07-25 00:00:00',90.15,199,365,62.79,'Intermediate' from dual union all
select 11898,'011898','1980-063C','BOZ','BOZ',50273,'Reentered','1980-08-04 00:00:00','1980-07-25 00:00:00',90.37,198,388,62.79,'Intermediate' from dual union all
select 11899,'011899','1980-062A','Rohini RS-1','Rohini RS-1',50272,'Reentered','1981-05-20 00:00:00','1980-08-17 00:00:00',96.7,304,898,44.76,'Intermediate' from dual union all
select 119,'000119','61 Omicron 4','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-11 00:00:00',102.91,785,1007,66.82,'Intermediate' from dual union all
select 1190,'001190','1965-012CW','deb Kosmos-57','',11762,'Reentered','1965-03-01 00:00:00','1965-02-25 00:00:00',89.94,120,423,63.86,'Intermediate' from dual union all
select 11900,'011900','1980-062B','SLV-3-4 E2','SLV-3-4 E2',50272,'Reentered','1981-07-23 00:00:00','1980-08-17 00:00:00',96.73,305,900,44.75,'Intermediate' from dual union all
select 11901,'011901','1980-059C','Blenda','Sunshade',50269,'Reentered','1980-07-30 00:00:00','1980-07-23 00:00:00',89.82,209,323,72.86,'Intermediate' from dual union all
select 11902,'011902','1980-059D','KDU','KDU',50269,'Reentered','1980-08-17 00:00:00','1980-07-24 00:00:00',90.07,222,334,72.87,'Intermediate' from dual union all
select 11903,'011903','1980-059E','KDU part','KDU part',50269,'Reentered','1980-07-27 00:00:00','1980-07-24 00:00:00',89.79,211,318,72.86,'Intermediate' from dual union all
select 11904,'011904','1980-059F','KDU part','KDU part',50269,'Reentered','1980-07-28 00:00:00','1980-07-23 00:00:00',90.05,221,333,72.87,'Intermediate' from dual union all
select 11905,'011905','1980-064A','Soyuz-37','Soyuz 7K-T No. 53',50274,'Landed','1980-10-11 00:00:00','1980-08-30 00:00:00',91.16,324,339,51.62,'Intermediate' from dual union all
select 11906,'011906','1980-064B','11S510','11S510',50274,'Reentered','1980-07-27 00:00:00','1980-07-25 00:00:00',88.79,180,249,51.58,'Intermediate' from dual union all
select 11907,'011907','1980-065A','Kosmos-1202','Zenit-6',50275,'Landed','1980-08-07 00:00:00','1980-07-28 00:00:00',89.61,224,286,72.85,'Intermediate' from dual union all
select 11908,'011908','1980-065B','11S510','11S510',50275,'Reentered','1980-07-29 00:00:00','1980-07-26 00:00:00',88.84,181,254,72.84,'Intermediate' from dual union all
select 11909,'011909','1980-063D','Blok-L','Blok-ML No. 71627-514',50273,'In Earth orbit','','1980-08-18 00:00:00',733.08,446,40661,62.88,'Molniya' from dual union all
select 1191,'001191','1965-015A','Kosmos-59','Zenit-4 No. 11',11765,'Landed','1965-03-15 00:00:00','1965-03-08 00:00:00',89.77,205,321,64.98,'Intermediate' from dual union all
select 11910,'011910','1980-061C','Blenda','Sunshade',50271,'Reentered','1980-07-31 00:00:00','1980-07-29 00:00:00',87.9,163,179,82.31,'Intermediate' from dual union all
select 11911,'011911','1980-061D','KDU','KDU',50271,'Reentered','1980-08-02 00:00:00','1980-07-28 00:00:00',88.8,195,236,82.32,'Intermediate' from dual union all
select 11912,'011912','1980-047D','Kosmos-1186 SS 1','ESO',50257,'Reentered','1980-09-24 00:00:00','1980-07-29 00:00:00',94.54,480,513,74.02,'Intermediate' from dual union all
select 11913,'011913','1980-047E','Kosmos-1186 SS 2','ESO',50257,'Reentered','1980-09-14 00:00:00','1980-07-29 00:00:00',94.22,451,511,74.03,'Intermediate' from dual union all
select 11914,'011914','1980-060D','SOZ','SOZ',50270,'Reentered','1982-04-20 00:00:00','1980-08-13 00:00:00',628.07,333,35499,47.26,'GEO Transfer' from dual union all
select 11915,'011915','1980-066A','Kosmos-1203','Resurs-F1 17F41 No. 13',50276,'Landed','1980-08-14 00:00:00','1980-08-02 00:00:00',89.8,258,272,82.32,'Intermediate' from dual union all
select 11916,'011916','1980-066B','11S510','11S510',50276,'Reentered','1980-08-06 00:00:00','1980-08-02 00:00:00',89.03,204,249,82.32,'Intermediate' from dual union all
select 11917,'011917','1980-067A','Kosmos-1204','Romb',50277,'Reentered','1981-02-23 00:00:00','1980-08-30 00:00:00',93.21,342,521,50.66,'Intermediate' from dual union all
select 11918,'011918','1980-067B','S3M','S3M',50277,'Reentered','1981-02-13 00:00:00','1980-08-30 00:00:00',93.09,337,515,50.67,'Intermediate' from dual union all
select 11919,'011919','1980-066C','Sensor cover','Sensor cover',50276,'Reentered','1980-08-02 00:00:00','1980-08-01 00:00:00',88.53,135,269,82.11,'Intermediate' from dual union all
select 1192,'001192','1965-015B','11A57I','11A57I',11765,'Reentered','1965-03-19 00:00:00','1965-03-09 00:00:00',89.53,189,313,64.97,'Intermediate' from dual union all
select 11920,'011920','1980-065C','KDU','KDU',50275,'Reentered','1980-08-23 00:00:00','1980-08-07 00:00:00',89.32,218,264,72.85,'Intermediate' from dual union all
select 11921,'011921','1980-065D','Blenda','Sunshade',50275,'Reentered','1980-08-10 00:00:00','1980-08-07 00:00:00',89.47,216,281,72.85,'Intermediate' from dual union all
select 11922,'011922','1980-065E','KDU part','KDU part',50275,'Reentered','1980-08-11 00:00:00','1980-08-07 00:00:00',89.4,218,272,72.86,'Intermediate' from dual union all
select 11923,'011923','1980-060E','SOZ','SOZ',50270,'Reentered','1981-04-02 00:00:00','1980-08-13 00:00:00',626.83,283,35485,47.2,'GEO Transfer' from dual union all
select 11924,'011924','1980-068A','Kosmos-1205','Zenit-6',50278,'Landed','1980-08-26 00:00:00','1980-08-14 00:00:00',89.63,227,286,72.81,'Intermediate' from dual union all
select 11925,'011925','1980-068B','11S510','11S510',50278,'Reentered','1980-08-17 00:00:00','1980-08-14 00:00:00',89.12,187,275,72.81,'Intermediate' from dual union all
select 11926,'011926','1978-118C','Blok DM No. 17L','11S86 No. 17L',45313,'In Earth orbit','','1980-08-22 00:00:00',1417.35,22089,48749,10.7,'Near-synch' from dual union all
select 11927,'011927','1980-066D','KDU','KDU',50276,'Reentered','1980-08-30 00:00:00','1980-08-14 00:00:00',89.72,254,268,82.32,'Intermediate' from dual union all
select 11928,'011928','1980-066E','Blenda','Sunshade',50276,'Reentered','1980-08-21 00:00:00','1980-08-14 00:00:00',89.59,242,267,82.32,'Intermediate' from dual union all
select 11929,'011929','1980-066F','KDU part','KDU part',50276,'Reentered','1980-08-19 00:00:00','1980-08-16 00:00:00',91.13,248,412,82.32,'Intermediate' from dual union all
select 1193,'001193','1965-013B','Fairing','Fairing',11763,'Reentered','1965-03-11 00:00:00','1965-03-07 00:00:00',88.9,167,273,75.06,'Intermediate' from dual union all
select 11930,'011930','1977-097BU','Garbage bag','',42225,'Reentered','1980-10-15 00:00:00','1980-08-15 00:00:00',91.25,329,343,51.62,'Intermediate' from dual union all
select 11931,'011931','1977-097BV','Garbage bag','',42225,'Reentered','1980-10-21 00:00:00','1980-08-14 00:00:00',91.34,326,355,51.62,'Intermediate' from dual union all
select 11932,'011932','1980-069A','Kosmos-1206','Tselina-D',50279,'Reentered','2002-01-13 00:00:00','1980-09-15 00:00:00',97.3,628,630,81.21,'Intermediate' from dual union all
select 11933,'011933','1980-069B','8A92ME','8A92ME',50279,'Reentered','2014-05-22 00:00:00','1991-03-28 00:00:00',96.45,551,627,81.22,'Intermediate' from dual union all
select 11934,'011934','1977-097BW','Garbage bag','',42225,'Reentered','1980-09-08 00:00:00','1980-08-15 00:00:00',91.18,328,337,51.64,'Intermediate' from dual union all
select 11935,'011935','1977-097BX','Garbage bag','',42225,'Reentered','1980-10-15 00:00:00','1980-08-15 00:00:00',91.28,328,347,51.63,'Intermediate' from dual union all
select 11936,'011936','1977-097BY','Garbage bag','',42225,'Reentered','1980-09-14 00:00:00','1980-08-15 00:00:00',91.23,327,342,51.64,'Intermediate' from dual union all
select 11937,'011937','1977-097BZ','Garbage bag','',42225,'Reentered','1980-10-16 00:00:00','1980-08-21 00:00:00',91.2,325,342,51.61,'Intermediate' from dual union all
select 11938,'011938','1980-070A','Kosmos-1207','Fram',50280,'Landed','1980-09-04 00:00:00','1980-08-24 00:00:00',88.92,206,237,82.32,'Intermediate' from dual union all
select 11939,'011939','1980-070B','11S510','11S510',50280,'Reentered','1980-08-26 00:00:00','1980-08-24 00:00:00',88.6,193,217,82.32,'Intermediate' from dual union all
select 1194,'001194','1965-012CX','deb Kosmos-57','',11762,'Reentered','1965-03-16 00:00:00','1965-03-06 00:00:00',90.41,186,404,64.7,'Intermediate' from dual union all
select 11940,'011940','1973-040B','Transtage 24','Transtage 24',30642,'In Earth orbit','','1973-07-01 00:00:00',1431.67,35600,35800,2,'Drift GEO' from dual union all
select 11941,'011941','1978-073F','Blok DM','11S86 No. 10L',45268,'In Earth orbit','','1993-06-05 00:00:00',1475.84,36506,36616,12.27,'Inclined Drift' from dual union all
select 11942,'011942','1977-097CA','Garbage bag','',42225,'Reentered','1980-09-14 00:00:00','1980-08-23 00:00:00',91.12,295,364,51.64,'Intermediate' from dual union all
select 11943,'011943','1980-068C','KDU','KDU',50278,'Reentered','1980-09-10 00:00:00','1980-08-26 00:00:00',89.45,221,274,72.81,'Intermediate' from dual union all
select 11944,'011944','1980-068D','Blenda','Sunshade',50278,'Reentered','1980-08-29 00:00:00','1980-08-26 00:00:00',89.3,219,261,72.83,'Intermediate' from dual union all
select 11945,'011945','1980-071A','Kosmos-1208','Feniks No. 927',50281,'Landed','1980-09-24 00:00:00','1980-09-02 00:00:00',89.61,163,348,67.14,'Intermediate' from dual union all
select 11946,'011946','1980-071B','11S510','11S510',50281,'Reentered','1980-08-30 00:00:00','1980-08-28 00:00:00',88.89,156,284,67.14,'Intermediate' from dual union all
select 11947,'011947','1977-097CB','Garbage bag','',42225,'Reentered','1980-10-16 00:00:00','1980-08-31 00:00:00',91.02,316,333,51.63,'Intermediate' from dual union all
select 11948,'011948','1980-015D','Tansei-4 yo-yo mass','',50225,'Reentered','1980-10-08 00:00:00','1980-05-23 00:00:00',95.05,458,583,39.07,'Intermediate' from dual union all
select 11949,'011949','1980-013D','Blok-L part','',50223,'Reentered','1980-11-08 00:00:00','1980-03-13 00:00:00',94.84,414,608,62.77,'Intermediate' from dual union all
select 1195,'001195','1965-012CY','deb Kosmos-57','',11762,'Reentered','1965-03-09 00:00:00','1965-02-27 00:00:00',90.24,173,399,64.77,'Intermediate' from dual union all
select 11950,'011950','1980-072A','Kosmos-1209','Resurs-F1 17F41 No. 14',50282,'Landed','1980-09-17 00:00:00','1980-09-06 00:00:00',89.79,258,271,82.34,'Intermediate' from dual union all
select 11951,'011951','1980-072B','11S510','11S510',50282,'Reentered','1980-09-08 00:00:00','1980-09-06 00:00:00',88.66,185,232,82.35,'Intermediate' from dual union all
select 11952,'011952','1965-048G','deb NNS O-4','',11798,'Reentered','1989-02-24 00:00:00','1980-08-24 00:00:00',106.8,1020,1137,89.88,'Polar' from dual union all
select 11953,'011953','1965-048H','deb NNS O-4','',11798,'Reentered','1989-03-07 00:00:00','1980-08-28 00:00:00',106.8,1021,1135,89.88,'Polar' from dual union all
select 11954,'011954','1980-070C','KDU','KDU',50280,'Reentered','1980-09-08 00:00:00','1980-09-04 00:00:00',88.72,207,216,82.32,'Intermediate' from dual union all
select 11955,'011955','1977-097CC','Garbage bag','',42225,'Reentered','1980-10-01 00:00:00','1980-09-04 00:00:00',90.99,313,333,51.63,'Intermediate' from dual union all
select 11956,'011956','1977-097CD','Garbage bag','',42225,'Reentered','1980-10-27 00:00:00','1980-09-05 00:00:00',91.1,321,335,51.61,'Intermediate' from dual union all
select 11957,'011957','1977-097CE','Garbage bag','',42225,'Reentered','1980-10-21 00:00:00','1980-09-05 00:00:00',91.12,323,336,51.64,'Intermediate' from dual union all
select 11958,'011958','1977-097CF','Garbage bag','',42225,'Reentered','1980-09-24 00:00:00','1980-09-04 00:00:00',91.02,315,335,51.63,'Intermediate' from dual union all
select 11959,'011959','1980-070D','Blenda','Sunshade',50280,'Reentered','1980-09-08 00:00:00','1980-09-04 00:00:00',88.36,190,197,82.31,'Intermediate' from dual union all
select 1196,'001196','1965-012CZ','deb Kosmos-57','',11762,'Reentered','1965-03-09 00:00:00','1965-02-28 00:00:00',91.18,175,489,64.77,'Intermediate' from dual union all
select 11960,'011960','1980-067C','Kosmos-1204 SS 1','ESO',50277,'Reentered','1980-10-11 00:00:00','1980-09-05 00:00:00',93.14,338,519,50.66,'Intermediate' from dual union all
select 11961,'011961','1980-067D','Kosmos-1204 SS 2','ESO',50277,'Reentered','1980-10-14 00:00:00','1980-09-04 00:00:00',93.2,341,521,50.66,'Intermediate' from dual union all
select 11962,'011962','1980-073A','Meteor-2','Meteor-2 No. 5',50283,'In Earth orbit','','1980-10-15 00:00:00',102.33,845,893,81.24,'Intermediate' from dual union all
select 11963,'011963','1980-073B','8A92ME','8A92ME',50283,'In Earth orbit','','1980-10-09 00:00:00',102.35,832,908,81.25,'Intermediate' from dual union all
select 11964,'011964','1980-074A','GOES 4','GOES D',50284,'In Earth orbit','','1980-10-09 00:00:00',1436.11,35774,35800,.19,'Stationary' from dual union all
select 11965,'011965','1980-074B','Delta 152','Delta SSPU DSV-3P-4',50284,'Reentered','1980-12-01 00:00:00','1980-10-09 00:00:00',108.29,164,2132,28.11,'Intermediate' from dual union all
select 11966,'011966','1977-097CG','Garbage bag','',42225,'Reentered','1980-11-23 00:00:00','1980-09-10 00:00:00',91.29,333,343,51.61,'Intermediate' from dual union all
select 11967,'011967','1977-097CH','Garbage bag','',42225,'Reentered','1980-10-06 00:00:00','1980-09-10 00:00:00',91.27,328,346,51.64,'Intermediate' from dual union all
select 11968,'011968','1980-034B','Kosmos-1176 DU','US-A DU',50244,'Reentered','1980-10-04 00:00:00','1980-09-10 00:00:00',89.55,243,262,65.01,'Intermediate' from dual union all
select 11969,'011969','1980-034C','Kosmos-1176 RLS','US-A RLS',50244,'Reentered','1980-09-12 00:00:00','1980-09-10 00:00:00',89.38,233,255,65.01,'Intermediate' from dual union all
select 1197,'001197','1965-012DA','deb Kosmos-57','',11762,'Reentered','1965-03-21 00:00:00','1965-03-07 00:00:00',91.37,136,548,64.81,'Intermediate' from dual union all
select 11970,'011970','1980-074C','Star 27 S/N 0020','Star 27 TE-M-616-5',50284,'In Earth orbit','','1990-10-08 00:00:00',1767.25,34340,49744,.08,'Very High' from dual union all
select 11971,'011971','1980-034D','Kosmos-1176 reactor core','US-A reactor core',50244,'In Earth orbit','','1991-11-12 00:00:00',103.09,894,915,64.83,'Intermediate' from dual union all
select 11972,'011972','1977-097CJ','Garbage bag','',42225,'Reentered','1980-11-03 00:00:00','1980-09-14 00:00:00',91.22,331,338,51.62,'Intermediate' from dual union all
select 11973,'011973','1977-097CK','Garbage bag','',42225,'Reentered','1980-10-15 00:00:00','1980-09-17 00:00:00',91.2,324,343,51.62,'Intermediate' from dual union all
select 11974,'011974','1977-097CL','Garbage bag','',42225,'Reentered','1980-11-13 00:00:00','1980-09-17 00:00:00',91.23,331,339,51.62,'Intermediate' from dual union all
select 11975,'011975','1977-097CM','Garbage bag','',42225,'Reentered','1980-10-13 00:00:00','1980-09-17 00:00:00',91.26,329,344,51.62,'Intermediate' from dual union all
select 11976,'011976','1980-072C','KDU','KDU',50282,'Reentered','1980-09-30 00:00:00','1980-09-17 00:00:00',89.64,245,269,82.34,'Intermediate' from dual union all
select 11977,'011977','1980-075A','Soyuz-38','Soyuz 7K-T No. 54',50285,'Landed','1980-09-26 00:00:00','1980-09-19 00:00:00',90.26,262,313,51.63,'Intermediate' from dual union all
select 11978,'011978','1980-075B','11S510','11S510',50285,'Reentered','1980-09-21 00:00:00','1980-09-20 00:00:00',88.4,176,215,51.62,'Intermediate' from dual;
insert into SATELLITE
select 11979,'011979','1980-075C','Soyuz-38 part','Soyuz part',50285,'Reentered','1980-09-21 00:00:00','1980-09-19 00:00:00',88.55,174,231,51.59,'Intermediate' from dual union all
select 1198,'001198','1965-012DB','deb Kosmos-57','',11762,'Reentered','1965-03-04 00:00:00','1965-02-25 00:00:00',89.73,137,386,64.64,'Intermediate' from dual union all
select 11980,'011980','1980-076A','Kosmos-1210','Zenit-6',50286,'Landed','1980-10-03 00:00:00','1980-09-21 00:00:00',89.98,227,319,82.33,'Intermediate' from dual union all
select 11981,'011981','1980-076B','11S510','11S510',50286,'Reentered','1980-09-21 00:00:00','1980-09-20 00:00:00',88.29,170,210,82.32,'Intermediate' from dual union all
select 11982,'011982','1980-077A','Kosmos-1211','Orion',50287,'Landed','1980-10-04 00:00:00','1980-10-03 00:00:00',88.68,200,219,82.35,'Intermediate' from dual union all
select 11983,'011983','1980-077B','11S510','11S510',50287,'Reentered','1980-09-26 00:00:00','1980-09-23 00:00:00',88.86,210,227,82.3,'Intermediate' from dual union all
select 11984,'011984','1967-026C','FW4D','FW4D S/N 00005',15281,'Reentered','1981-06-20 00:00:00','1999-08-07 00:00:00',344.86,275,19548,26.12,'Highly Ellip' from dual union all
select 11985,'011985','1980-078A','Kosmos-1212','Fram',50288,'Landed','1980-10-09 00:00:00','1980-10-03 00:00:00',88.77,197,231,82.34,'Intermediate' from dual union all
select 11986,'011986','1980-078B','11S510','11S510',50288,'Reentered','1980-09-29 00:00:00','1980-09-26 00:00:00',88.91,204,237,82.4,'Intermediate' from dual union all
select 11987,'011987','1980-067E','Kosmos-1204 SS 3','ESO',50277,'Reentered','1980-10-31 00:00:00','1980-10-02 00:00:00',92.89,340,493,50.65,'Intermediate' from dual union all
select 11988,'011988','1980-067F','Kosmos-1204 SS 4','ESO',50277,'Reentered','1980-10-24 00:00:00','1980-10-02 00:00:00',92.67,331,480,50.66,'Intermediate' from dual union all
select 11989,'011989','1980-067G','Kosmos-1204 SS 5','ESO',50277,'Reentered','1980-10-24 00:00:00','1980-10-02 00:00:00',92.58,323,478,50.67,'Intermediate' from dual union all
select 1199,'001199','1965-012DC','deb Kosmos-57','',11762,'Reentered','1965-03-17 00:00:00','1965-03-07 00:00:00',91.34,173,508,64.7,'Intermediate' from dual union all
select 11990,'011990','1966-005G','NNS O-7 insulation','',13412,'Reentered','1983-01-11 00:00:00','1980-10-01 00:00:00',105.64,859,1190,89.79,'Polar' from dual union all
select 11991,'011991','1966-005H','NNS O-7 insulation','RCS 0.25',13412,'In Earth orbit','','1991-05-07 00:00:00',105.19,848,1158,89.84,'Polar' from dual union all
select 11992,'011992','1980-067H','Kosmos-1204 SS 6','ESO',50277,'Reentered','1980-10-26 00:00:00','1980-10-01 00:00:00',92.84,336,492,50.67,'Intermediate' from dual union all
select 11993,'011993','1980-079A','Progress-11','Progress 7K-TG No. 111',50289,'Deorbited','1980-12-11 00:00:00','1980-11-01 00:00:00',90.73,303,318,51.62,'Intermediate' from dual union all
select 11994,'011994','1980-079B','11S510','11S510',50289,'Reentered','1980-09-30 00:00:00','1983-10-26 00:00:00',95.98,559,572,65,'Intermediate' from dual union all
select 11995,'011995','1980-076C','Blenda','Sunshade',50286,'Reentered','1980-10-11 00:00:00','1980-10-03 00:00:00',90.35,232,352,82.33,'Intermediate' from dual union all
select 11996,'011996','1980-076D','KDU part','KDU part',50286,'Reentered','1980-10-06 00:00:00','1980-10-03 00:00:00',90.19,229,339,82.35,'Intermediate' from dual union all
select 11997,'011997','1980-080A','Kosmos-1213','Zenit-6',50290,'Landed','1980-10-17 00:00:00','1980-10-05 00:00:00',89.64,225,289,72.87,'Intermediate' from dual union all
select 11998,'011998','1980-080B','11S510','11S510',50290,'Reentered','1980-10-07 00:00:00','1980-10-06 00:00:00',88.36,163,224,72.85,'Intermediate' from dual union all
select 11999,'011999','1980-076E','KDU','KDU',50286,'Reentered','1980-10-29 00:00:00','1980-10-03 00:00:00',90.16,230,334,82.35,'Intermediate' from dual union all
select 12,'000012','59 Alpha 2','GRC 33-KS-2800','GRC 33-KS-2800 175-15-21',5525,'In Earth orbit','','1959-04-28 00:00:00',129.97,564,3679,32.88,'Medium' from dual union all
select 120,'000120','61 Omicron 5','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-11 00:00:00',103.21,796,1025,66.82,'Intermediate' from dual union all
select 1200,'001200','1965-012DD','deb Kosmos-57','',11762,'Reentered','1965-03-11 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12000,'012000','1973-086GL','deb Delta 98','',30688,'In Earth orbit','','1980-10-04 00:00:00',116.56,1509,1544,101.87,'Sun-Synch' from dual union all
select 12001,'012001','1980-076F','KDU part','KDU part',50286,'Reentered','1980-10-10 00:00:00','1980-10-04 00:00:00',91.94,311,429,82.34,'Intermediate' from dual union all
select 12002,'012002','1980-076G','KDU part','KDU part',50286,'Reentered','1980-10-05 00:00:00','1980-10-04 00:00:00',91.38,209,475,82.33,'Intermediate' from dual union all
select 12003,'012003','1980-081A','Raduga','Gran'' No. 17L',50291,'In Earth orbit','','1980-11-05 00:00:00',1447.37,35972,36042,.32,'Drift GEO' from dual union all
select 12004,'012004','1980-081B','8S812','8S812',50291,'Reentered','1980-10-07 00:00:00','1980-10-06 00:00:00',88.09,173,186,51.68,'Intermediate' from dual union all
select 12005,'012005','1980-081C','Perekhodnik','Sredniy perekhodnik',50291,'Reentered','1980-10-06 00:00:00','1980-10-05 00:00:00',116.11,176,2836,51.39,'Intermediate' from dual union all
select 12006,'012006','1980-078C','Blenda','Sunshade',50288,'Reentered','1980-10-10 00:00:00','1980-10-08 00:00:00',88.09,172,187,82.34,'Intermediate' from dual union all
select 12007,'012007','1980-078D','KDU','KDU',50288,'Reentered','1980-10-11 00:00:00','1980-10-09 00:00:00',88.4,184,207,82.34,'Intermediate' from dual union all
select 12008,'012008','1980-082A','Kosmos-1214','Zenit-4MKM',50292,'Landed','1980-10-23 00:00:00','1980-10-12 00:00:00',89.58,170,338,67.16,'Intermediate' from dual union all
select 12009,'012009','1980-082B','11S510','11S510',50292,'Reentered','1980-10-13 00:00:00','1980-10-12 00:00:00',88.93,160,284,67.15,'Intermediate' from dual union all
select 1201,'001201','1965-012DE','deb Kosmos-57','',11762,'Reentered','1965-03-04 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12010,'012010','1980-067J','Kosmos-1204 SS 7','ESO',50277,'Reentered','1980-11-03 00:00:00','1980-10-10 00:00:00',92.88,333,499,50.68,'Intermediate' from dual union all
select 12011,'012011','1980-067K','Kosmos-1204 SS 8','ESO',50277,'Reentered','1980-11-03 00:00:00','1980-10-10 00:00:00',92.79,333,490,50.67,'Intermediate' from dual union all
select 12012,'012012','1980-081D','SOZ','SOZ',50291,'Reentered','1982-04-15 00:00:00','1980-11-06 00:00:00',633.22,232,35865,47.29,'GEO Transfer' from dual union all
select 12013,'012013','1980-081E','SOZ','SOZ',50291,'Reentered','1982-06-14 00:00:00','1980-11-05 00:00:00',634.93,284,35901,47.34,'GEO Transfer' from dual union all
select 12014,'012014','1980-067L','Kosmos-1204 SS 9','ESO',50277,'Reentered','1980-11-04 00:00:00','1980-10-11 00:00:00',92.81,331,494,50.68,'Intermediate' from dual union all
select 12015,'012015','1980-067M','Kosmos-1204 SS 10','ESO',50277,'Reentered','1980-11-10 00:00:00','1980-10-11 00:00:00',92.77,333,488,50.65,'Intermediate' from dual union all
select 12016,'012016','1980-083A','Kosmos-1215','Tselina-OM',50293,'Reentered','1983-05-12 00:00:00','1980-11-13 00:00:00',94.99,492,544,74.05,'Intermediate' from dual union all
select 12017,'012017','1980-083B','S3M','S3M',50293,'Reentered','1982-08-27 00:00:00','1980-11-13 00:00:00',94.93,478,553,74.06,'Intermediate' from dual union all
select 12018,'012018','1980-083C','deb Kosmos-1215','Tselina-O part',50293,'Reentered','1981-01-10 00:00:00','1980-11-02 00:00:00',94.46,465,520,74.06,'Intermediate' from dual union all
select 12019,'012019','1980-084A','Kosmos-1216','Zenit-6',50294,'Landed','1980-10-30 00:00:00','1980-10-18 00:00:00',92.23,354,414,72.87,'Intermediate' from dual union all
select 1202,'001202','1965-012DF','deb Kosmos-57','',11762,'Reentered','1965-03-09 00:00:00','1965-03-03 00:00:00',90.23,161,411,64.77,'Intermediate' from dual union all
select 12020,'012020','1980-084B','11S510','11S510',50294,'Reentered','1980-10-23 00:00:00','1980-10-18 00:00:00',89.87,189,347,72.88,'Intermediate' from dual union all
select 12021,'012021','1980-084C','Sensor cover','Sensor cover',50294,'Reentered','1980-10-22 00:00:00','1980-10-16 00:00:00',89.51,189,312,72.86,'Intermediate' from dual union all
select 12022,'012022','1980-080C','KDU','KDU',50290,'Reentered','1980-11-06 00:00:00','1980-10-17 00:00:00',90.13,231,331,72.87,'Intermediate' from dual union all
select 12023,'012023','1980-080D','Blenda','Sunshade',50290,'Reentered','1980-10-24 00:00:00','1980-10-17 00:00:00',92.17,257,505,72.86,'Intermediate' from dual union all
select 12024,'012024','1980-080E','KDU part','KDU part',50290,'Reentered','1980-10-24 00:00:00','1980-10-17 00:00:00',89.9,225,314,72.87,'Intermediate' from dual union all
select 12025,'012025','1980-080F','KDU part','KDU part',50290,'Reentered','1980-10-22 00:00:00','1980-10-17 00:00:00',89.86,229,305,72.88,'Intermediate' from dual union all
select 12026,'012026','1980-083D','deb Kosmos-1215','Tselina-O part',50293,'Reentered','1980-11-18 00:00:00','1980-10-24 00:00:00',94.73,472,539,73.67,'Intermediate' from dual union all
select 12027,'012027','1978-084C','SA Veneri-11','SA',45279,'Deep Space Attached','1978-09-09 00:00:00','1978-12-23 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 12028,'012028','1978-086C','SA Veneri-12','SA',45281,'Deep Space Attached','1978-09-14 00:00:00','1978-12-19 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 12029,'012029','1980-082C','KDU','KDU',50292,'Reentered','1980-10-25 00:00:00','1980-10-23 00:00:00',90.57,178,427,67.17,'Intermediate' from dual union all
select 1203,'001203','1965-012DG','deb Kosmos-57','',11762,'Reentered','1965-03-14 00:00:00','1965-03-06 00:00:00',89.7,179,340,64.72,'Intermediate' from dual union all
select 12030,'012030','1980-082D','Blenda','Sunshade',50292,'Reentered','1980-10-25 00:00:00','1980-10-23 00:00:00',88.08,139,220,67.14,'Intermediate' from dual union all
select 12031,'012031','1980-082E','KDU part','KDU part',50292,'Reentered','1980-10-25 00:00:00','1980-10-23 00:00:00',88.79,155,274,67.04,'Intermediate' from dual union all
select 12032,'012032','1980-085A','Kosmos-1217','Oko',50295,'In Earth orbit','','1981-02-11 00:00:00',717.72,770,39582,63.05,'Molniya' from dual union all
select 12033,'012033','1980-085B','11S510','11S510',50295,'Reentered','1980-11-17 00:00:00','1980-10-31 00:00:00',92.03,206,543,62.8,'Intermediate' from dual union all
select 12034,'012034','1980-085C','BOZ','BOZ',50295,'Reentered','1980-11-07 00:00:00','1980-10-26 00:00:00',92.34,177,601,62.93,'Intermediate' from dual union all
select 12035,'012035','1980-085D','Blok-L','Blok-2BL No. 76755-638',50295,'Reentered','2011-11-26 00:00:00','1980-11-23 00:00:00',722.02,647,39917,62.98,'Molniya' from dual union all
select 12036,'012036','1980-067N','Kosmos-1204 SS 11','ESO',50277,'Reentered','1980-11-27 00:00:00','1980-10-29 00:00:00',92.66,327,483,50.64,'Intermediate' from dual union all
select 12037,'012037','1980-067P','Kosmos-1204 SS 12','ESO',50277,'Reentered','1980-11-18 00:00:00','1980-10-28 00:00:00',92.63,329,478,50.69,'Intermediate' from dual union all
select 12038,'012038','1980-067Q','Kosmos-1204 SS 13','ESO',50277,'Reentered','1980-11-18 00:00:00','1980-10-30 00:00:00',92.54,324,474,50.69,'Intermediate' from dual union all
select 12039,'012039','1980-086A','Kosmos-1218','Yantar''-4K1 No. 3',50296,'Landed','1980-12-12 00:00:00','1980-11-06 00:00:00',89.62,169,343,64.89,'Intermediate' from dual union all
select 1204,'001204','1965-012DH','deb Kosmos-57','',11762,'Reentered','1965-03-15 00:00:00','1965-03-06 00:00:00',89.99,155,393,64.68,'Intermediate' from dual union all
select 12040,'012040','1980-084D','Blenda','Sunshade',50294,'Reentered','1980-12-05 00:00:00','1980-10-30 00:00:00',92.05,342,409,72.88,'Intermediate' from dual union all
select 12041,'012041','1980-084E','KDU part','KDU part',50294,'Reentered','1981-03-26 00:00:00','1997-12-07 00:00:00',1435.35,35749,35795,14.43,'Inclined Drift' from dual union all
select 12042,'012042','1980-086B','11S510','11S510',50296,'Reentered','1980-11-02 00:00:00','1980-11-01 00:00:00',88.92,158,285,64.89,'Intermediate' from dual union all
select 12043,'012043','1980-084F','KDU','KDU',50294,'Reentered','1981-04-13 00:00:00','1980-10-30 00:00:00',92.47,359,433,72.47,'Intermediate' from dual union all
select 12044,'012044','1980-084G','KDU part','KDU part',50294,'Reentered','1981-03-02 00:00:00','1980-10-30 00:00:00',92.43,338,450,72.95,'Intermediate' from dual union all
select 12045,'012045','1980-084H','KDU part','KDU part',50294,'Reentered','1980-11-21 00:00:00','1980-10-30 00:00:00',92.21,352,414,72.88,'Intermediate' from dual union all
select 12046,'012046','1980-087A','FLTSATCOM F4','FLTSATCOM F4',50297,'In Earth orbit','','1980-12-03 00:00:00',1436.01,35534,36036,2.39,'Stationary' from dual union all
select 12047,'012047','1980-067R','Kosmos-1204 SS 14','ESO',50277,'Reentered','1980-11-23 00:00:00','1980-10-30 00:00:00',92.67,326,485,50.66,'Intermediate' from dual union all
select 12048,'012048','1980-067S','Kosmos-1204 SS 15','ESO',50277,'Reentered','1980-12-03 00:00:00','1980-10-30 00:00:00',92.65,328,480,50.66,'Intermediate' from dual union all
select 12049,'012049','1980-067T','Kosmos-1204 SS 16','ESO',50277,'Reentered','1980-11-19 00:00:00','1980-10-31 00:00:00',92.55,326,473,50.62,'Intermediate' from dual union all
select 1205,'001205','1965-012DJ','deb Kosmos-57','',11762,'Reentered','1965-03-16 00:00:00','1965-03-06 00:00:00',90.47,118,478,64.64,'Intermediate' from dual union all
select 12050,'012050','1980-088A','Kosmos-1219','Zenit-6',50298,'Landed','1980-11-13 00:00:00','1980-11-02 00:00:00',89.63,226,287,72.85,'Intermediate' from dual union all
select 12051,'012051','1980-088B','11S510','11S510',50298,'Reentered','1980-11-04 00:00:00','1980-11-02 00:00:00',89.03,174,280,72.85,'Intermediate' from dual union all
select 12052,'012052','1980-067U','Kosmos-1204 SS 17','ESO',50277,'Reentered','1980-11-17 00:00:00','1980-11-01 00:00:00',92.3,312,463,50.66,'Intermediate' from dual union all
select 12053,'012053','1980-067V','Kosmos-1204 SS 18','ESO',50277,'Reentered','1980-11-17 00:00:00','1980-11-01 00:00:00',92.18,306,457,50.65,'Intermediate' from dual union all
select 12054,'012054','1980-089A','Kosmos-1220','US-P',50299,'Reentered','2014-02-16 00:00:00','1989-07-24 00:00:00',98.81,572,832,65,'Intermediate' from dual union all
select 12055,'012055','1980-089B','11S692','11S692',50299,'Reentered','1980-11-05 00:00:00','1980-11-04 00:00:00',88.51,147,255,65.25,'Intermediate' from dual union all
select 12056,'012056','1980-067W','Kosmos-1204 SS 19','ESO',50277,'Reentered','1980-11-16 00:00:00','1980-11-08 00:00:00',91.58,291,414,50.66,'Intermediate' from dual union all
select 12057,'012057','1980-067X','Kosmos-1204 SS 20','ESO',50277,'Reentered','1980-11-23 00:00:00','1980-11-08 00:00:00',92.06,315,437,50.65,'Intermediate' from dual union all
select 12058,'012058','1980-090A','Kosmos-1221','Zenit-6',50300,'Landed','1980-11-26 00:00:00','1980-11-15 00:00:00',92.23,354,414,72.89,'Intermediate' from dual union all
select 12059,'012059','1980-090B','11S510','11S510',50300,'Reentered','1980-11-20 00:00:00','1980-11-14 00:00:00',90.05,189,365,72.89,'Intermediate' from dual union all
select 1206,'001206','1965-012DK','deb Kosmos-57','',11762,'Reentered','1965-03-09 00:00:00','1965-03-02 00:00:00',90.02,158,393,64.73,'Intermediate' from dual union all
select 12060,'012060','1980-088C','Blenda','Sunshade',50298,'Reentered','1980-11-18 00:00:00','1980-11-13 00:00:00',89.56,221,285,72.84,'Intermediate' from dual union all
select 12061,'012061','1980-088D','KDU part','KDU part',50298,'Reentered','1980-11-24 00:00:00','1980-11-13 00:00:00',89.35,222,263,72.84,'Intermediate' from dual union all
select 12062,'012062','1980-088E','KDU','KDU',50298,'Reentered','1980-11-26 00:00:00','1980-11-13 00:00:00',89.63,214,298,72.84,'Intermediate' from dual union all
select 12063,'012063','1980-088F','KDU part','KDU part',50298,'Reentered','1980-11-23 00:00:00','1980-11-13 00:00:00',90.1,180,379,72.97,'Intermediate' from dual union all
select 12064,'012064','1980-088G','KDU part','KDU part',50298,'Reentered','1980-11-15 00:00:00','1980-11-14 00:00:00',90.95,237,406,72.86,'Intermediate' from dual union all
select 12065,'012065','1980-091A','SBS 1','SBS 1',50301,'In Earth orbit','','1980-12-15 00:00:00',1436.01,35778,35792,.16,'Stationary' from dual union all
select 12066,'012066','1980-092A','Molniya-1','Molniya-1',50302,'Reentered','1995-11-18 00:00:00','1981-09-15 00:00:00',717.76,617,39737,63.4,'Molniya' from dual union all
select 12067,'012067','1980-092B','BOZ','BOZ',50302,'Reentered','1980-12-13 00:00:00','1980-11-23 00:00:00',92.1,207,547,62.8,'Intermediate' from dual;
insert into SATELLITE
select 12068,'012068','1980-092C','11S510','11S510',50302,'Reentered','1980-12-12 00:00:00','1980-11-23 00:00:00',91.97,189,554,62.84,'Intermediate' from dual union all
select 12069,'012069','1980-087B','Centaur AC-57','Centaur D-1AR AC-57',50297,'Reentered','2011-10-28 00:00:00','1981-01-30 00:00:00',570.94,165,32674,26.24,'Highly Ellip' from dual union all
select 1207,'001207','1965-015C','Volga airlock','',11765,'Reentered','1965-03-11 00:00:00','1965-03-08 00:00:00',90.56,218,386,65,'Intermediate' from dual union all
select 12070,'012070','1980-092D','Blok-L','Blok-ML No. 71628-449',50302,'Reentered','1996-08-07 00:00:00','1980-12-16 00:00:00',733.73,583,40556,62.91,'Molniya' from dual union all
select 12071,'012071','1980-093A','Kosmos-1222','Tselina-D',50303,'Reentered','2011-04-27 00:00:00','1980-12-21 00:00:00',97.29,628,630,81.23,'Intermediate' from dual union all
select 12072,'012072','1980-093B','8A92ME','8A92ME',50303,'Reentered','2015-04-18 00:00:00','1980-12-21 00:00:00',97.37,580,686,81.24,'Intermediate' from dual union all
select 12073,'012073','1980-090C','KDU','KDU',50300,'Reentered','1981-04-14 00:00:00','1981-01-20 00:00:00',91.64,331,379,72.88,'Intermediate' from dual union all
select 12074,'012074','1980-067Y','Kosmos-1204 SS 21','ESO',50277,'Reentered','1980-12-14 00:00:00','1980-11-27 00:00:00',92.26,318,453,50.66,'Intermediate' from dual union all
select 12075,'012075','1980-067Z','Kosmos-1204 SS 22','ESO',50277,'Reentered','1980-12-13 00:00:00','1980-11-26 00:00:00',92.16,312,449,50.66,'Intermediate' from dual union all
select 12076,'012076','1980-090D','Blenda','Sunshade',50300,'Reentered','1981-01-03 00:00:00','1980-11-27 00:00:00',92.03,341,407,72.89,'Intermediate' from dual union all
select 12077,'012077','1980-094A','Soyuz T-3','Soyuz 7K-ST No. 8L',50304,'Landed','1980-12-10 00:00:00','1980-11-28 00:00:00',89.63,254,259,51.62,'Intermediate' from dual union all
select 12078,'012078','1980-095A','Kosmos-1223','Oko',50305,'In Earth orbit','','1981-02-11 00:00:00',717.9,664,39697,62.96,'Molniya' from dual union all
select 12079,'012079','1980-094B','11S510','11S510',50304,'Reentered','1980-11-29 00:00:00','1980-11-28 00:00:00',88.28,179,200,51.62,'Intermediate' from dual union all
select 1208,'001208','1965-016E','EGRS III','Secor Type II S/N 1',11766,'In Earth orbit','','1965-04-19 00:00:00',103.47,907,938,70.08,'Intermediate' from dual union all
select 12080,'012080','1980-095B','11S510','11S510',50305,'Reentered','1980-12-22 00:00:00','1980-12-06 00:00:00',91.73,196,523,62.81,'Intermediate' from dual union all
select 12081,'012081','1980-095C','BOZ','BOZ',50305,'Reentered','1980-12-21 00:00:00','1980-12-04 00:00:00',91.94,207,533,62.78,'Intermediate' from dual union all
select 12082,'012082','1980-095D','deb Blok-I','',50305,'Reentered','1980-12-08 00:00:00','1980-11-29 00:00:00',91.94,159,581,62.89,'Intermediate' from dual union all
select 12083,'012083','1980-090E','KDU part','KDU part',50300,'Reentered','1980-12-22 00:00:00','1980-11-27 00:00:00',92.18,350,413,72.89,'Intermediate' from dual union all
select 12084,'012084','1980-096A','Kosmos-1224','Zenit-6',50306,'Landed','1980-12-15 00:00:00','1980-12-06 00:00:00',92.24,355,414,72.86,'Intermediate' from dual union all
select 12085,'012085','1980-096B','11S510','11S510',50306,'Reentered','1980-12-09 00:00:00','1980-12-03 00:00:00',89.92,192,349,72.86,'Intermediate' from dual union all
select 12086,'012086','1980-095E','Blok-L','Blok-2BL No. 32',50305,'In Earth orbit','','1980-12-27 00:00:00',723.4,624,40008,62.88,'Molniya' from dual union all
select 12087,'012087','1980-097A','Kosmos-1225','Parus',50307,'In Earth orbit','','1981-01-31 00:00:00',104.86,946,1030,82.95,'Intermediate' from dual union all
select 12088,'012088','1980-097B','S3M','S3M',50307,'In Earth orbit','','1981-01-21 00:00:00',104.71,942,1020,82.94,'Intermediate' from dual union all
select 12089,'012089','1980-098A','INTELSAT 502','INTELSAT V F2',50308,'In Earth orbit','','1981-03-17 00:00:00',1436.09,35764,35809,.69,'Stationary' from dual union all
select 1209,'001209','1965-012DL','deb Kosmos-57','',11762,'Reentered','1965-03-11 00:00:00','1965-02-28 00:00:00',91.38,237,448,65.28,'Intermediate' from dual union all
select 12090,'012090','1977-097CN','Soyuz T-3 BO','7K-ST No. 8L BO',42225,'Reentered','1981-01-26 00:00:00','1980-12-27 00:00:00',90.38,267,320,51.62,'Intermediate' from dual union all
select 12091,'012091','1980-099A','Kosmos-1226','Tsikada',50309,'In Earth orbit','','1981-02-22 00:00:00',104.85,961,1013,82.94,'Intermediate' from dual union all
select 12092,'012092','1980-099B','S3M','S3M',50309,'In Earth orbit','','1981-01-24 00:00:00',104.72,964,999,82.94,'Intermediate' from dual union all
select 12093,'012093','1980-100A','OPS 5805','QUASAR 4 (or JUMPSEAT 6)',50310,'In Earth orbit','','1980-12-13 00:00:00',698.11,250,39130,63.8,'Molniya' from dual union all
select 12094,'012094','1980-100B','Agena D','Agena D',50310,'In Earth orbit','','1980-12-13 00:00:00',698.11,250,39130,63.8,'Molniya' from dual union all
select 12095,'012095','1980-096C','KDU','KDU',50306,'Reentered','1981-05-04 00:00:00','1981-01-20 00:00:00',91.89,343,392,72.87,'Intermediate' from dual union all
select 12096,'012096','1980-096D','Blenda','Sunshade',50306,'Reentered','1981-03-15 00:00:00','1980-12-15 00:00:00',91.44,249,442,72.89,'Intermediate' from dual union all
select 12097,'012097','1980-096E','KDU part','KDU part',50306,'Reentered','1981-01-26 00:00:00','1980-12-21 00:00:00',91.84,332,398,72.86,'Intermediate' from dual union all
select 12098,'012098','1980-030Q','deb Kosmos-1174','',50240,'Reentered','1982-12-01 00:00:00','1982-11-28 00:00:00',88.98,202,247,66.04,'Intermediate' from dual union all
select 12099,'012099','1965-016K','deb GREB 6','',11766,'In Earth orbit','','1980-12-23 00:00:00',103.37,901,935,70.08,'Intermediate' from dual union all
select 121,'000121','61 Omicron 6','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-08 00:00:00',103.21,839,982,66.74,'Intermediate' from dual union all
select 1210,'001210','1965-012DM','deb Kosmos-57','',11762,'Reentered','1965-03-04 00:00:00','1965-02-25 00:00:00',90.93,168,473,64.75,'Intermediate' from dual union all
select 12100,'012100','1980-101A','Kosmos-1227','Zenit-6',50311,'Landed','1980-12-28 00:00:00','1980-12-18 00:00:00',89.61,225,286,72.84,'Intermediate' from dual union all
select 12101,'012101','1980-101B','11S510','11S510',50311,'Reentered','1980-12-20 00:00:00','1980-12-18 00:00:00',88.97,184,264,72.84,'Intermediate' from dual union all
select 12102,'012102','1965-027G','deb SNAPSHOT','',11777,'In Earth orbit','','1980-12-18 00:00:00',111.48,1271,1318,90.25,'Polar' from dual union all
select 12103,'012103','1978-078D','Sounder Probe','Large Probe',45273,'Deep Space Attached','1978-08-08 00:00:00','1978-11-16 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 12104,'012104','1978-078E','North Probe','Small Probe 1',45273,'Deep Space Attached','1978-08-08 00:00:00','1978-11-20 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 12105,'012105','1978-078F','Day Probe','Small Probe 2',45273,'Deep Space Attached','1978-08-08 00:00:00','1978-11-20 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 12106,'012106','1978-078G','Night Probe','Small Probe 3',45273,'Deep Space Attached','1978-08-08 00:00:00','1978-11-20 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 12107,'012107','1980-102A','Kosmos-1228','Strela-1M',50312,'In Earth orbit','','1981-02-07 00:00:00',114.41,1394,1464,73.99,'Intermediate' from dual union all
select 12108,'012108','1980-102B','Kosmos-1229','Strela-1M',50312,'In Earth orbit','','1981-02-09 00:00:00',114.66,1416,1464,73.99,'Intermediate' from dual union all
select 12109,'012109','1980-102C','Kosmos-1230','Strela-1M',50312,'In Earth orbit','','1981-02-21 00:00:00',114.49,1401,1463,73.99,'Intermediate' from dual union all
select 1211,'001211','1965-012DN','deb Kosmos-57','',11762,'Reentered','1965-03-16 00:00:00','1965-03-06 00:00:00',91,162,486,64.88,'Intermediate' from dual union all
select 12110,'012110','1980-102D','Kosmos-1231','Strela-1M',50312,'In Earth orbit','','1981-01-28 00:00:00',114.56,1407,1463,73.99,'Intermediate' from dual union all
select 12111,'012111','1980-102E','Kosmos-1232','Strela-1M',50312,'In Earth orbit','','1981-01-29 00:00:00',114.63,1414,1464,73.99,'Intermediate' from dual union all
select 12112,'012112','1980-102F','Kosmos-1233','Strela-1M',50312,'In Earth orbit','','1981-02-05 00:00:00',114.7,1420,1464,73.99,'Intermediate' from dual union all
select 12113,'012113','1980-102G','Kosmos-1234','Strela-1M',50312,'In Earth orbit','','1981-01-25 00:00:00',114.59,1410,1463,73.99,'Intermediate' from dual union all
select 12114,'012114','1980-102H','Kosmos-1235','Strela-1M',50312,'In Earth orbit','','1981-02-05 00:00:00',114.65,1415,1464,73.99,'Intermediate' from dual union all
select 12115,'012115','1980-102J','S3M','S3M',50312,'In Earth orbit','','1981-01-22 00:00:00',114.94,1441,1464,73.99,'Intermediate' from dual union all
select 12116,'012116','1980-103A','Prognoz-8','SO-M No. 508',50313,'Deep Space','1980-12-25 00:00:00','1980-12-25 00:00:00',5687.96,977,197368,65.83,'Deep Space' from dual union all
select 12117,'012117','1980-103D','deb Blok-I','',50313,'Reentered','1981-01-12 00:00:00','1980-12-30 00:00:00',91.41,227,461,65.06,'Intermediate' from dual union all
select 12118,'012118','1980-103B','11S510','11S510',50313,'Reentered','1981-01-20 00:00:00','1980-12-27 00:00:00',91.38,203,482,65.1,'Intermediate' from dual union all
select 12119,'012119','1980-103C','BOZ','BOZ',50313,'Reentered','1981-01-14 00:00:00','1980-12-31 00:00:00',91.36,218,465,65.05,'Intermediate' from dual union all
select 1212,'001212','1965-012DP','deb Kosmos-57','',11762,'Reentered','1965-03-12 00:00:00','1965-03-12 00:00:00',87.54,153,153,64.9,'Intermediate' from dual union all
select 12120,'012120','1980-104A','Ekran','Ekran No. 20L',50314,'In Earth orbit','','1981-04-07 00:00:00',1436.09,35773,35799,.19,'Stationary' from dual union all
select 12121,'012121','1980-105A','Kosmos-1236','Feniks No. 941',50315,'Landed','1981-01-21 00:00:00','1980-12-31 00:00:00',88.9,165,276,67.13,'Intermediate' from dual union all
select 12122,'012122','1980-104B','8S812','8S812',50314,'Reentered','1980-12-28 00:00:00','1980-12-27 00:00:00',88.19,181,189,51.6,'Intermediate' from dual union all
select 12123,'012123','1980-104C','Perekhodnik','Sredniy perekhodnik',50314,'Reentered','1980-12-27 00:00:00','1980-12-26 00:00:00',88.33,159,225,51.6,'Intermediate' from dual union all
select 12124,'012124','1980-105B','11S510','11S510',50315,'Reentered','1980-12-30 00:00:00','1980-12-28 00:00:00',89.29,161,318,67.14,'Intermediate' from dual union all
select 12125,'012125','1980-074D','Star 37E S/N 40084','Star 37E',50284,'Reentered','1981-04-23 00:00:00','1980-09-09 00:00:00',920.52,166,49770,26.5,'Highly Ellip' from dual union all
select 12126,'012126','1980-105D','Kosmos-1236 part','Feniks part',50315,'Reentered','1981-01-13 00:00:00','1980-12-26 00:00:00',90.12,168,393,67.1,'Intermediate' from dual union all
select 12127,'012127','1980-101C','KDU','KDU',50311,'Reentered','1981-01-15 00:00:00','1980-12-28 00:00:00',89.7,223,297,72.84,'Intermediate' from dual union all
select 12128,'012128','1980-101D','Blenda','Sunshade',50311,'Reentered','1981-01-03 00:00:00','1980-12-28 00:00:00',89.36,209,277,72.9,'Intermediate' from dual union all
select 12129,'012129','1980-101E','KDU part','KDU part',50311,'Reentered','1980-12-30 00:00:00','1980-12-28 00:00:00',89.5,215,285,72.84,'Intermediate' from dual union all
select 1213,'001213','1965-012DQ','deb Kosmos-57','',11762,'Reentered','1965-03-14 00:00:00','1965-03-07 00:00:00',90.08,146,411,65.04,'Intermediate' from dual union all
select 12130,'012130','1981-001A','Kosmos-1237','Zenit-6',52524,'Landed','1981-01-20 00:00:00','1981-01-15 00:00:00',92.24,355,414,72.86,'Intermediate' from dual union all
select 12131,'012131','1981-001B','11S510','11S510',52524,'Reentered','1981-01-15 00:00:00','1981-01-06 00:00:00',90.15,193,371,72.8,'Intermediate' from dual union all
select 12132,'012132','1981-001C','Sensor cover','Sensor cover',52524,'Reentered','1981-01-12 00:00:00','1981-01-06 00:00:00',89.77,187,340,72.9,'Intermediate' from dual union all
select 12133,'012133','1981-002A','Molniya-3','Molniya-3 No. 25',52525,'Reentered','1999-07-03 00:00:00','1981-02-13 00:00:00',717.74,489,39863,62.83,'Molniya' from dual union all
select 12134,'012134','1981-002B','Blok-L','Blok-ML No. 315',52525,'Reentered','1999-10-23 00:00:00','1981-02-14 00:00:00',732.24,484,40582,62.86,'Molniya' from dual union all
select 12135,'012135','1981-002C','11S510','11S510',52525,'Reentered','1981-01-27 00:00:00','1981-01-09 00:00:00',91.17,209,455,62.8,'Intermediate' from dual union all
select 12136,'012136','1981-002D','BOZ','BOZ',52525,'Reentered','1981-01-22 00:00:00','1981-01-09 00:00:00',91.07,187,468,62.8,'Intermediate' from dual union all
select 12137,'012137','1980-104D','SOZ','SOZ',50314,'Reentered','1982-04-20 00:00:00','1981-01-25 00:00:00',625.59,273,35431,47.2,'GEO Transfer' from dual union all
select 12138,'012138','1981-003A','Kosmos-1238','Vektor',52526,'In Earth orbit','','1981-02-20 00:00:00',108.95,404,1953,82.98,'Intermediate' from dual union all
select 12139,'012139','1981-003B','S3M','S3M',52526,'In Earth orbit','','1981-02-19 00:00:00',108.85,402,1946,82.98,'Intermediate' from dual union all
select 1214,'001214','1965-012DR','deb Kosmos-57','',11762,'Reentered','1965-03-20 00:00:00','1965-03-09 00:00:00',90.08,164,393,64.83,'Intermediate' from dual union all
select 12140,'012140','1981-004A','Kosmos-1239','Orion',52527,'Landed','1981-01-28 00:00:00','1981-01-28 00:00:00',88.53,196,208,82.33,'Intermediate' from dual union all
select 12141,'012141','1981-004B','11S510','11S510',52527,'Reentered','1981-01-20 00:00:00','1981-01-16 00:00:00',88.83,210,223,82.4,'Intermediate' from dual union all
select 12142,'012142','1964-026F','deb Transit VC','',10312,'Reentered','1981-10-25 00:00:00','1981-01-20 00:00:00',102.5,834,920,90.47,'Polar' from dual union all
select 12143,'012143','1981-005A','Kosmos-1240','Feniks No. 975',52528,'Landed','1981-02-17 00:00:00','1981-01-27 00:00:00',89.58,163,345,64.88,'Intermediate' from dual union all
select 12144,'012144','1981-005B','11S510','11S510',52528,'Reentered','1981-01-24 00:00:00','1981-01-20 00:00:00',89.76,171,354,64.89,'Intermediate' from dual union all
select 12145,'012145','1981-001D','KDU','KDU',52524,'Reentered','1981-06-05 00:00:00','1981-02-05 00:00:00',92.11,350,406,72.85,'Intermediate' from dual union all
select 12146,'012146','1981-001E','Blenda','Sunshade',52524,'Reentered','1981-01-25 00:00:00','1981-01-20 00:00:00',92.24,354,414,72.87,'Intermediate' from dual union all
select 12147,'012147','1981-001F','KDU part','KDU part',52524,'Reentered','1981-03-03 00:00:00','1981-01-20 00:00:00',92.06,342,410,72.86,'Intermediate' from dual union all
select 12148,'012148','1981-001G','KDU part','KDU part',52524,'Reentered','1981-02-10 00:00:00','1981-02-01 00:00:00',89.44,246,248,72.84,'Intermediate' from dual union all
select 12149,'012149','1981-006A','Kosmos-1241','Lira',52529,'In Earth orbit','','1981-02-20 00:00:00',104.95,975,1010,65.83,'Intermediate' from dual union all
select 1215,'001215','1965-012DS','deb Kosmos-57','',11762,'Reentered','1965-03-16 00:00:00','1965-03-08 00:00:00',90.23,164,408,64.79,'Intermediate' from dual union all
select 12150,'012150','1981-006B','S3M','S3M',52529,'In Earth orbit','','1981-02-22 00:00:00',104.82,972,1000,65.83,'Intermediate' from dual union all
select 12151,'012151','1981-006C','deb Kosmos-1241','DS-P1-M part [RCS 0.50]',52529,'In Earth orbit','','1981-03-01 00:00:00',104.93,974,1009,65.83,'Intermediate' from dual union all
select 12152,'012152','1981-007A','Progress-12','Progress 7K-TG No. 113',52530,'Deorbited','1981-03-20 00:00:00','1981-01-31 00:00:00',91.59,347,359,51.62,'Intermediate' from dual union all
select 12153,'012153','1981-007B','11S510','11S510',52530,'Reentered','1981-01-27 00:00:00','1981-01-25 00:00:00',88.23,157,217,51.7,'Intermediate' from dual union all
select 12154,'012154','1981-008A','Kosmos-1242','Tselina-D',52531,'Reentered','2014-05-08 00:00:00','1981-02-26 00:00:00',97.5,625,654,81.18,'Intermediate' from dual union all
select 12155,'012155','1981-008B','8A92ME','8A92ME',52531,'In Earth orbit','','1981-03-08 00:00:00',97.54,583,698,81.18,'Intermediate' from dual union all
select 12156,'012156','1981-009A','Molniya-1','Molniya-1',52532,'In Earth orbit','','1981-03-01 00:00:00',739.77,458,40976,62.86,'Molniya' from dual union all
select 12157,'012157','1981-009B','11S510','11S510',52532,'Reentered','1981-02-14 00:00:00','1981-02-06 00:00:00',90.37,198,388,62.83,'Intermediate' from dual;
insert into SATELLITE
select 12158,'012158','1981-009C','BOZ','BOZ',52532,'Reentered','1981-02-09 00:00:00','1981-02-01 00:00:00',90.71,188,431,62.83,'Intermediate' from dual union all
select 12159,'012159','1981-009D','Blok-L','Blok-ML No. 148',52532,'In Earth orbit','','1981-03-06 00:00:00',731.75,458,40584,62.86,'Molniya' from dual union all
select 1216,'001216','1965-012DT','deb Kosmos-57','',11762,'Reentered','1965-03-17 00:00:00','1965-03-08 00:00:00',89.94,153,390,64.75,'Intermediate' from dual union all
select 12160,'012160','1981-010A','Kosmos-1243','IS',52533,'Deorbited','1981-02-02 00:00:00','1981-02-02 00:00:00',97.82,295,1013,65.82,'Intermediate' from dual union all
select 12161,'012161','1981-010B','11S692','11S692',52533,'Reentered','1981-02-04 00:00:00','1981-02-03 00:00:00',88.04,115,239,65.08,'Intermediate' from dual union all
select 12162,'012162','1981-011A','Interkosmos-21','AUOS-Z R-P-IK',52534,'Reentered','1982-07-07 00:00:00','1981-03-14 00:00:00',94.35,467,508,74.03,'Intermediate' from dual union all
select 12163,'012163','1981-011B','S3M','S3M',52534,'Reentered','1982-06-22 00:00:00','1981-03-14 00:00:00',94.27,461,506,74.04,'Intermediate' from dual union all
select 12164,'012164','1978-026D','deb Delta 139','',45221,'In Earth orbit','','1981-02-06 00:00:00',102.4,798,947,98.85,'Sun-Synch' from dual union all
select 12165,'012165','1978-026E','deb Delta 139','',45221,'In Earth orbit','','1981-02-05 00:00:00',102.39,838,906,98.84,'Sun-Synch' from dual union all
select 12166,'012166','1978-026F','deb Delta 139','',45221,'In Earth orbit','','1993-07-30 00:00:00',102.06,827,885,98.98,'Sun-Synch' from dual union all
select 12167,'012167','1978-026G','deb Delta 139','',45221,'In Earth orbit','','1993-08-02 00:00:00',102.44,745,1004,99.1,'Sun-Synch' from dual union all
select 12168,'012168','1978-026H','deb Delta 139','',45221,'In Earth orbit','','1981-02-06 00:00:00',104.19,813,1100,98.81,'Sun-Synch' from dual union all
select 12169,'012169','1978-026J','deb Delta 139','',45221,'In Earth orbit','','1993-07-31 00:00:00',101.25,762,874,98.67,'Sun-Synch' from dual union all
select 1217,'001217','1965-012DU','deb Kosmos-57','',11762,'Reentered','1965-03-28 00:00:00','1965-03-09 00:00:00',91.68,165,548,64.72,'Intermediate' from dual union all
select 12170,'012170','1978-026K','deb Delta 139','',45221,'In Earth orbit','','1993-08-01 00:00:00',101.95,816,886,98.96,'Sun-Synch' from dual union all
select 12171,'012171','1978-026L','deb Delta 139','',45221,'In Earth orbit','','1981-02-07 00:00:00',102.91,881,912,98.83,'Sun-Synch' from dual union all
select 12172,'012172','1978-026M','deb Delta 139','',45221,'In Earth orbit','','1981-02-05 00:00:00',101.99,801,905,98.82,'Sun-Synch' from dual union all
select 12173,'012173','1978-026N','deb Delta 139','',45221,'Reentered','2014-07-07 00:00:00','1993-08-02 00:00:00',100.52,738,828,98.78,'Sun-Synch' from dual union all
select 12174,'012174','1978-026P','deb Delta 139','',45221,'In Earth orbit','','1993-08-03 00:00:00',102.35,839,900,99.04,'Sun-Synch' from dual union all
select 12175,'012175','1978-026Q','deb Delta 139','',45221,'In Earth orbit','','1993-08-04 00:00:00',103.49,874,974,99.01,'Sun-Synch' from dual union all
select 12176,'012176','1978-026R','deb Delta 139','',45221,'In Earth orbit','','1993-07-31 00:00:00',103.83,891,989,98.94,'Sun-Synch' from dual union all
select 12177,'012177','1978-026S','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',103.87,820,1062,98.74,'Sun-Synch' from dual union all
select 12178,'012178','1978-026T','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',103.56,879,975,99.02,'Sun-Synch' from dual union all
select 12179,'012179','1978-026U','deb Delta 139','',45221,'In Earth orbit','','1981-02-05 00:00:00',104.08,792,1111,98.8,'Sun-Synch' from dual union all
select 1218,'001218','1965-012DV','deb Kosmos-57','',11762,'Reentered','1965-03-20 00:00:00','1965-02-22 00:00:00',91.38,158,527,64.9,'Intermediate' from dual union all
select 12180,'012180','1978-026V','deb Delta 139','',45221,'Reentered','2000-05-18 00:00:00','1993-08-05 00:00:00',100.12,724,805,98.81,'Sun-Synch' from dual union all
select 12181,'012181','1978-026W','deb Delta 139','',45221,'In Earth orbit','','1993-08-02 00:00:00',101.3,745,895,98.62,'Sun-Synch' from dual union all
select 12182,'012182','1978-026X','deb Delta 139','',45221,'Reentered','1989-10-28 00:00:00','1989-10-28 00:00:00',89.26,216,259,98.7,'Sun-Synch' from dual union all
select 12183,'012183','1978-026Y','deb Delta 139','',45221,'In Earth orbit','','1993-08-01 00:00:00',102.21,836,890,99.01,'Sun-Synch' from dual union all
select 12184,'012184','1978-026Z','deb Delta 139','',45221,'Reentered','2015-06-14 00:00:00','1993-08-01 00:00:00',100.96,792,817,98.94,'Sun-Synch' from dual union all
select 12185,'012185','1978-026AA','deb Delta 139','',45221,'In Earth orbit','','1993-07-31 00:00:00',102.61,805,959,99.1,'Sun-Synch' from dual union all
select 12186,'012186','1978-026AB','deb Delta 139','',45221,'Reentered','1989-12-11 00:00:00','1989-12-11 00:00:00',89.91,259,281,98.67,'Sun-Synch' from dual union all
select 12187,'012187','1978-026AC','deb Delta 139','',45221,'Reentered','1988-02-12 00:00:00','1988-02-11 00:00:00',90.72,298,322,98.83,'Sun-Synch' from dual union all
select 12188,'012188','1978-026AD','deb Delta 139','',45221,'Reentered','1993-03-13 00:00:00','1992-10-01 00:00:00',96.12,567,579,98.78,'Sun-Synch' from dual union all
select 12189,'012189','1978-026AE','deb Delta 139','',45221,'In Earth orbit','','1981-02-06 00:00:00',103.75,883,988,98.88,'Sun-Synch' from dual union all
select 1219,'001219','1965-012DW','deb Kosmos-57','',11762,'Reentered','1965-03-14 00:00:00','1965-03-09 00:00:00',89.82,154,378,64.85,'Intermediate' from dual union all
select 12190,'012190','1978-026AF','deb Delta 139','',45221,'In Earth orbit','','1981-02-06 00:00:00',103.3,902,928,98.84,'Sun-Synch' from dual union all
select 12191,'012191','1978-026AG','deb Delta 139','',45221,'Reentered','1989-02-28 00:00:00','1989-02-27 00:00:00',90.89,301,336,98.75,'Sun-Synch' from dual union all
select 12192,'012192','1978-026AH','deb Delta 139','',45221,'Reentered','1989-03-22 00:00:00','1989-03-21 00:00:00',91.37,331,353,98.93,'Sun-Synch' from dual union all
select 12193,'012193','1978-026AJ','deb Delta 139','',45221,'In Earth orbit','','1993-08-04 00:00:00',106.7,829,1319,98.89,'Sun-Synch' from dual union all
select 12194,'012194','1978-026AK','deb Delta 139','',45221,'In Earth orbit','','1993-07-31 00:00:00',101.39,760,889,99.06,'Sun-Synch' from dual union all
select 12195,'012195','1978-026AL','deb Delta 139','',45221,'In Earth orbit','','1993-07-31 00:00:00',103.65,884,978,98.96,'Sun-Synch' from dual union all
select 12196,'012196','1978-026AM','deb Delta 139','',45221,'Reentered','2001-01-11 00:00:00','1993-08-05 00:00:00',101.01,764,849,98.82,'Sun-Synch' from dual union all
select 12197,'012197','1978-026AN','deb Delta 139','',45221,'Reentered','1982-02-05 00:00:00','1982-02-05 00:00:00',91.3,314,362,98.8,'Sun-Synch' from dual union all
select 12198,'012198','1978-026AP','deb Delta 139','',45221,'In Earth orbit','','1993-08-05 00:00:00',103.4,829,1009,98.87,'Sun-Synch' from dual union all
select 12199,'012199','1978-026AQ','deb Delta 139','',45221,'Reentered','1990-02-20 00:00:00','1980-10-23 00:00:00',717.72,1698,38654,64.87,'GEO Transfer' from dual union all
select 122,'000122','61 Omicron 7','deb r/b Ablestar','',7273,'In Earth orbit','','1961-08-10 00:00:00',102.16,732,990,66.77,'Intermediate' from dual union all
select 1220,'001220','1965-012DX','deb Kosmos-57','',11762,'Reentered','1965-03-12 00:00:00','1965-03-05 00:00:00',89.77,154,373,64.82,'Intermediate' from dual union all
select 12200,'012200','1978-026AR','deb Delta 139','',45221,'In Earth orbit','','1993-05-13 00:00:00',100.98,720,890,98.45,'Sun-Synch' from dual union all
select 12201,'012201','1978-026AS','deb Delta 139','',45221,'Reentered','2001-10-15 00:00:00','1993-07-31 00:00:00',103.85,886,994,98.91,'Sun-Synch' from dual union all
select 12202,'012202','1978-026AT','deb Delta 139','',45221,'Reentered','1983-11-12 00:00:00','1983-11-10 00:00:00',91.14,317,344,98.66,'Sun-Synch' from dual union all
select 12203,'012203','1978-026AU','deb Delta 139','',45221,'In Earth orbit','','1981-02-07 00:00:00',101.83,789,902,98.86,'Sun-Synch' from dual union all
select 12204,'012204','1978-026AV','deb Delta 139','',45221,'In Earth orbit','','1993-08-01 00:00:00',101.33,758,885,98.69,'Sun-Synch' from dual union all
select 12205,'012205','1978-026AW','deb Delta 139','',45221,'Reentered','1989-07-21 00:00:00','1989-07-19 00:00:00',92.06,345,407,98.86,'Sun-Synch' from dual union all
select 12206,'012206','1978-026AX','deb Delta 139','',45221,'In Earth orbit','','1981-02-07 00:00:00',104.91,830,1151,98.77,'Sun-Synch' from dual union all
select 12207,'012207','1978-026AY','deb Delta 139','',45221,'Reentered','1984-07-16 00:00:00','1984-07-14 00:00:00',90.43,289,303,98.84,'Sun-Synch' from dual union all
select 12208,'012208','1978-026AZ','deb Delta 139','',45221,'Reentered','2001-10-22 00:00:00','1993-08-07 00:00:00',103.78,787,1088,98.63,'Sun-Synch' from dual union all
select 12209,'012209','1978-026BA','deb Delta 139','',45221,'In Earth orbit','','1993-08-03 00:00:00',101.32,771,871,98.77,'Sun-Synch' from dual union all
select 1221,'001221','1965-012DY','deb Kosmos-57','',11762,'Reentered','1965-03-18 00:00:00','1965-02-22 00:00:00',92.48,143,650,65.1,'Intermediate' from dual union all
select 12210,'012210','1978-026BB','deb Delta 139','',45221,'Reentered','1990-02-11 00:00:00','1981-02-07 00:00:00',105.25,849,1164,98.92,'Sun-Synch' from dual union all
select 12211,'012211','1978-026BC','deb Delta 139','',45221,'In Earth orbit','','1993-08-05 00:00:00',103.05,807,999,99.05,'Sun-Synch' from dual union all
select 12212,'012212','1978-026BD','deb Delta 139','',45221,'In Earth orbit','','1981-02-07 00:00:00',103.46,902,942,98.83,'Sun-Synch' from dual union all
select 12213,'012213','1978-026BE','deb Delta 139','',45221,'Reentered','2004-08-14 00:00:00','1981-02-07 00:00:00',109.08,900,1468,98.78,'Sun-Synch' from dual union all
select 12214,'012214','1978-026BF','deb Delta 139','',45221,'Reentered','2003-10-28 00:00:00','1993-08-06 00:00:00',101.22,708,925,99.01,'Sun-Synch' from dual union all
select 12215,'012215','1978-026BG','deb Delta 139','',45221,'Reentered','2006-10-01 00:00:00','1993-08-07 00:00:00',100.97,754,856,98.96,'Sun-Synch' from dual union all
select 12216,'012216','1978-026BH','deb Delta 139','',45221,'In Earth orbit','','1993-08-04 00:00:00',104.4,792,1140,98.91,'Sun-Synch' from dual union all
select 12217,'012217','1978-026BJ','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',102.09,841,875,99.05,'Sun-Synch' from dual union all
select 12218,'012218','1978-026BK','deb Delta 139','',45221,'In Earth orbit','','1993-08-05 00:00:00',103.37,877,959,99.13,'Sun-Synch' from dual union all
select 12219,'012219','1978-026BL','deb Delta 139','',45221,'Reentered','1989-08-10 00:00:00','1989-08-10 00:00:00',88.78,209,220,98.86,'Sun-Synch' from dual union all
select 1222,'001222','1965-012DZ','deb Kosmos-57','',11762,'Reentered','1965-03-12 00:00:00','1965-03-01 00:00:00',90.67,220,395,64.38,'Intermediate' from dual union all
select 12220,'012220','1978-026BM','deb Delta 139','',45221,'Reentered','1991-03-13 00:00:00','1981-02-09 00:00:00',102.93,869,926,98.83,'Sun-Synch' from dual union all
select 12221,'012221','1978-026BN','deb Delta 139','',45221,'Reentered','1994-08-01 00:00:00','1981-02-07 00:00:00',103.88,903,981,98.85,'Sun-Synch' from dual union all
select 12222,'012222','1978-026BP','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',103.73,880,990,98.92,'Sun-Synch' from dual union all
select 12223,'012223','1978-026BQ','deb Delta 139','',45221,'In Earth orbit','','1993-08-04 00:00:00',102.7,856,917,99.16,'Sun-Synch' from dual union all
select 12224,'012224','1978-026BR','deb Delta 139','',45221,'Reentered','1992-12-31 00:00:00','1992-10-01 00:00:00',95.43,537,542,98.81,'Sun-Synch' from dual union all
select 12225,'012225','1978-026BS','deb Delta 139','',45221,'In Earth orbit','','1981-02-07 00:00:00',103.56,893,961,98.82,'Sun-Synch' from dual union all
select 12226,'012226','1978-026BT','deb Delta 139','',45221,'In Earth orbit','','1981-02-06 00:00:00',108.3,904,1393,98.88,'Sun-Synch' from dual union all
select 12227,'012227','1978-026BU','deb Delta 139','',45221,'In Earth orbit','','1993-08-03 00:00:00',99.13,616,818,98.82,'Sun-Synch' from dual union all
select 12228,'012228','1978-026BV','deb Delta 139','',45221,'Reentered','1990-10-05 00:00:00','1981-02-06 00:00:00',102.56,856,903,98.83,'Sun-Synch' from dual union all
select 12229,'012229','1978-026BW','deb Delta 139','',45221,'Reentered','1981-05-05 00:00:00','1981-02-05 00:00:00',96.85,326,890,98.91,'Sun-Synch' from dual union all
select 1223,'001223','1965-012EA','deb Kosmos-57','',11762,'Reentered','1965-03-31 00:00:00','1965-03-09 00:00:00',92.71,137,678,64.63,'Intermediate' from dual union all
select 12230,'012230','1978-026BX','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',100.17,688,846,98.35,'Sun-Synch' from dual union all
select 12231,'012231','1978-026BY','deb Delta 139','',45221,'Reentered','1993-03-19 00:00:00','1992-10-01 00:00:00',94.54,471,522,98.73,'Sun-Synch' from dual union all
select 12232,'012232','1978-026BZ','deb Delta 139','',45221,'Reentered','2014-04-07 00:00:00','1993-08-06 00:00:00',101.87,741,954,98.92,'Sun-Synch' from dual union all
select 12233,'012233','1978-026CA','deb Delta 139','',45221,'Reentered','1988-02-12 00:00:00','1988-02-10 00:00:00',91.06,315,339,98.87,'Sun-Synch' from dual union all
select 12234,'012234','1978-026CB','deb Delta 139','',45221,'In Earth orbit','','1981-02-05 00:00:00',102.91,876,916,98.84,'Sun-Synch' from dual union all
select 12235,'012235','1978-026CC','deb Delta 139','',45221,'Reentered','1989-02-08 00:00:00','1989-02-07 00:00:00',91.08,300,355,98.34,'Sun-Synch' from dual union all
select 12236,'012236','1978-026CD','deb Delta 139','',45221,'In Earth orbit','','1981-02-05 00:00:00',105.75,828,1231,98.83,'Sun-Synch' from dual union all
select 12237,'012237','1978-026CE','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',107.18,809,1384,98.89,'Sun-Synch' from dual union all
select 12238,'012238','1978-026CF','deb Delta 139','',45221,'Reentered','1982-02-15 00:00:00','1982-02-11 00:00:00',92.38,371,412,98.76,'Sun-Synch' from dual union all
select 12239,'012239','1978-026CG','deb Delta 139','',45221,'Reentered','1982-02-05 00:00:00','1982-02-05 00:00:00',91.37,330,353,98.78,'Sun-Synch' from dual union all
select 1224,'001224','1965-012EB','deb Kosmos-57','',11762,'Reentered','1965-03-16 00:00:00','1965-02-22 00:00:00',89.57,170,337,64.7,'Intermediate' from dual union all
select 12240,'012240','1978-026CH','deb Delta 139','',45221,'Reentered','1984-04-16 00:00:00','1984-04-16 00:00:00',91.11,307,351,98.98,'Sun-Synch' from dual union all
select 12241,'012241','1978-026CJ','deb Delta 139','',45221,'Reentered','1982-04-11 00:00:00','1982-04-10 00:00:00',89.68,251,266,98.66,'Sun-Synch' from dual union all
select 12242,'012242','1978-026CK','deb Delta 139','',45221,'In Earth orbit','','1993-08-01 00:00:00',103.62,876,984,98.85,'Sun-Synch' from dual union all
select 12243,'012243','1978-026CL','deb Delta 139','',45221,'Reentered','1995-11-24 00:00:00','1992-09-30 00:00:00',100.07,681,843,98.61,'Sun-Synch' from dual union all
select 12244,'012244','1978-026CM','deb Delta 139','',45221,'Reentered','1989-06-18 00:00:00','1989-06-18 00:00:00',91.57,332,372,98.6,'Sun-Synch' from dual union all
select 12245,'012245','1978-026CN','deb Delta 139','',45221,'Reentered','1989-07-15 00:00:00','1989-07-15 00:00:00',87.51,142,161,98.77,'Retrograde' from dual union all
select 12246,'012246','1978-026CP','deb Delta 139','',45221,'In Earth orbit','','1993-07-31 00:00:00',103.23,861,962,99.02,'Sun-Synch' from dual union all
select 12247,'012247','1978-026CQ','deb Delta 139','',45221,'Reentered','1988-06-02 00:00:00','1988-06-01 00:00:00',91.27,327,347,98.9,'Sun-Synch' from dual;
insert into SATELLITE
select 12248,'012248','1978-026CR','deb Delta 139','',45221,'Reentered','1981-08-17 00:00:00','1981-02-05 00:00:00',97.52,377,902,98.89,'Sun-Synch' from dual union all
select 12249,'012249','1978-026CS','deb Delta 139','',45221,'Reentered','1989-04-16 00:00:00','1989-04-15 00:00:00',90.94,302,340,98.87,'Sun-Synch' from dual union all
select 1225,'001225','1965-012EC','deb Kosmos-57','',11762,'Reentered','1965-03-14 00:00:00','1965-03-09 00:00:00',89.73,162,361,64.7,'Intermediate' from dual union all
select 12250,'012250','1978-026CT','deb Delta 139','',45221,'In Earth orbit','','1993-08-07 00:00:00',101.81,824,865,98.99,'Sun-Synch' from dual union all
select 12251,'012251','1978-026CU','deb Delta 139','',45221,'Reentered','1981-02-18 00:00:00','1981-02-11 00:00:00',93.21,363,501,98.92,'Sun-Synch' from dual union all
select 12252,'012252','1978-026CV','deb Delta 139','',45221,'Reentered','1982-01-02 00:00:00','1981-01-30 00:00:00',101.73,565,1116,98.9,'Sun-Synch' from dual union all
select 12253,'012253','1978-026CW','deb Delta 139','',45221,'Reentered','1981-12-13 00:00:00','1981-12-12 00:00:00',88.43,185,209,98.77,'Sun-Synch' from dual union all
select 12254,'012254','1978-026CX','deb Delta 139','',45221,'Reentered','1989-03-20 00:00:00','1989-03-15 00:00:00',93.33,432,443,98.81,'Sun-Synch' from dual union all
select 12255,'012255','1978-026CY','deb Delta 139','',45221,'Reentered','1981-07-01 00:00:00','1981-03-01 00:00:00',102.88,876,914,98.91,'Sun-Synch' from dual union all
select 12256,'012256','1978-026CZ','deb Delta 139','',45221,'In Earth orbit','','1993-07-30 00:00:00',103.69,844,1022,98.81,'Sun-Synch' from dual union all
select 12257,'012257','1978-026DA','deb Delta 139','',45221,'In Earth orbit','','1993-08-08 00:00:00',103.09,823,987,99.11,'Sun-Synch' from dual union all
select 12258,'012258','1978-026DB','deb Delta 139','',45221,'Reentered','1981-07-01 00:00:00','1981-02-10 00:00:00',100.49,577,987,98.89,'Sun-Synch' from dual union all
select 12259,'012259','1978-026DC','deb Delta 139','',45221,'Reentered','1989-03-15 00:00:00','1989-12-14 00:00:00',89.27,228,249,98.86,'Sun-Synch' from dual union all
select 1226,'001226','1965-012ED','deb Kosmos-57','',11762,'Reentered','1965-03-18 00:00:00','1965-03-09 00:00:00',89.69,189,330,64.77,'Intermediate' from dual union all
select 12260,'012260','1978-026DD','deb Delta 139','',45221,'Reentered','1991-05-29 00:00:00','1981-02-09 00:00:00',103.87,889,994,98.83,'Sun-Synch' from dual union all
select 12261,'012261','1978-026DE','deb Delta 139','',45221,'Reentered','1991-09-27 00:00:00','1991-09-22 00:00:00',93.34,417,458,98.73,'Sun-Synch' from dual union all
select 12262,'012262','1978-026DF','deb Delta 139','',45221,'In Earth orbit','','1981-02-10 00:00:00',105.46,890,1142,98.77,'Sun-Synch' from dual union all
select 12263,'012263','1978-026DG','deb Delta 139','',45221,'In Earth orbit','','1993-08-05 00:00:00',110.58,871,1636,98.92,'Sun-Synch' from dual union all
select 12264,'012264','1978-026DH','deb Delta 139','',45221,'Reentered','1981-12-10 00:00:00','1981-12-09 00:00:00',90.53,295,306,98.73,'Sun-Synch' from dual union all
select 12265,'012265','1978-026DJ','deb Delta 139','',45221,'In Earth orbit','','1993-08-05 00:00:00',103.98,794,1099,98.8,'Sun-Synch' from dual union all
select 12266,'012266','1978-026DK','deb Delta 139','',45221,'In Earth orbit','','1993-08-01 00:00:00',104.87,856,1121,98.77,'Sun-Synch' from dual union all
select 12267,'012267','1978-026DL','deb Delta 139','',45221,'Reentered','2002-03-07 00:00:00','1993-08-04 00:00:00',97.58,560,726,98.76,'Sun-Synch' from dual union all
select 12268,'012268','1978-026DM','deb Delta 139','',45221,'Reentered','1993-07-18 00:00:00','1992-11-06 00:00:00',96.49,588,593,98.86,'Sun-Synch' from dual union all
select 12269,'012269','1978-026DN','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',100.4,704,851,98.4,'Sun-Synch' from dual union all
select 1227,'001227','1965-012EE','deb Kosmos-57','',11762,'Reentered','1965-03-11 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12270,'012270','1978-026DP','deb Delta 139','',45221,'In Earth orbit','','1981-02-07 00:00:00',103.81,884,994,98.82,'Sun-Synch' from dual union all
select 12271,'012271','1978-026DQ','deb Delta 139','',45221,'Reentered','1981-11-18 00:00:00','1981-11-04 00:00:00',97.1,611,629,98.88,'Sun-Synch' from dual union all
select 12272,'012272','1978-026DR','deb Delta 139','',45221,'Reentered','1989-08-07 00:00:00','1989-08-07 00:00:00',89.64,240,274,98.7,'Sun-Synch' from dual union all
select 12273,'012273','1978-026DS','deb Delta 139','',45221,'Reentered','2010-10-13 00:00:00','1993-08-04 00:00:00',102.06,805,908,98.86,'Sun-Synch' from dual union all
select 12274,'012274','1976-077DF','deb Delta 126','[ex UNID 081, now ID]',39029,'In Earth orbit','','1981-02-10 00:00:00',117.87,1501,1670,102.4,'Sun-Synch' from dual union all
select 12275,'012275','1978-026DT','deb Delta 139','',45221,'Reentered','1981-12-27 00:00:00','1981-12-26 00:00:00',89.25,222,253,98.74,'Sun-Synch' from dual union all
select 12276,'012276','1978-026DU','deb Delta 139','',45221,'Reentered','1987-04-20 00:00:00','1987-04-18 00:00:00',89.5,236,264,98.73,'Sun-Synch' from dual union all
select 12277,'012277','1978-026DV','deb Delta 139','',45221,'Reentered','1989-08-17 00:00:00','1989-08-16 00:00:00',92.59,380,423,99.1,'Sun-Synch' from dual union all
select 12278,'012278','1978-026DW','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',102.37,822,920,99.16,'Sun-Synch' from dual union all
select 12279,'012279','1978-026DX','deb Delta 139','',45221,'In Earth orbit','','1993-07-31 00:00:00',102.81,827,956,98.78,'Sun-Synch' from dual union all
select 1228,'001228','1965-017C','Ablestar 018','AJ10-104D',11767,'Reentered','1967-09-04 00:00:00','1965-04-15 00:00:00',97.74,284,1018,89.98,'Polar' from dual union all
select 12280,'012280','1978-026DY','deb Delta 139','',45221,'Reentered','1999-06-22 00:00:00','1993-08-07 00:00:00',98.1,640,696,98.85,'Sun-Synch' from dual union all
select 12281,'012281','1978-026DZ','deb Delta 139','',45221,'Reentered','1982-11-07 00:00:00','1982-11-04 00:00:00',91.5,327,369,98.78,'Sun-Synch' from dual union all
select 12282,'012282','1978-026EA','deb Delta 139','',45221,'Reentered','1982-04-29 00:00:00','1982-04-29 00:00:00',88.63,197,217,98.81,'Sun-Synch' from dual union all
select 12283,'012283','1978-026EB','deb Delta 139','',45221,'In Earth orbit','','1993-08-04 00:00:00',101.26,749,888,98.63,'Sun-Synch' from dual union all
select 12284,'012284','1978-026EC','deb Delta 139','',45221,'In Earth orbit','','1993-08-04 00:00:00',103.85,855,1026,98.83,'Sun-Synch' from dual union all
select 12285,'012285','1978-026ED','deb Delta 139','',45221,'In Earth orbit','','1981-02-09 00:00:00',104.27,911,1010,98.83,'Sun-Synch' from dual union all
select 12286,'012286','1978-026EE','deb Delta 139','',45221,'Reentered','1981-10-14 00:00:00','1981-07-12 00:00:00',101.11,690,933,98.92,'Sun-Synch' from dual union all
select 12287,'012287','1978-026EF','deb Delta 139','',45221,'Reentered','1994-06-07 00:00:00','1981-02-10 00:00:00',103.81,900,978,98.81,'Sun-Synch' from dual union all
select 12288,'012288','1978-026EG','deb Delta 139','',45221,'In Earth orbit','','1981-02-07 00:00:00',99.69,523,965,98.72,'Sun-Synch' from dual union all
select 12289,'012289','1978-026EH','deb Delta 139','',45221,'Reentered','1991-11-10 00:00:00','1991-11-10 00:00:00',89.95,265,279,98.68,'Sun-Synch' from dual union all
select 1229,'001229','1965-012EF','deb Kosmos-57','',11762,'Reentered','1965-03-29 00:00:00','1965-03-11 00:00:00',91.5,167,530,64.83,'Intermediate' from dual union all
select 12290,'012290','1978-026EJ','deb Delta 139','',45221,'In Earth orbit','','1993-07-30 00:00:00',102.44,811,938,99.11,'Sun-Synch' from dual union all
select 12291,'012291','1978-026EK','deb Delta 139','',45221,'In Earth orbit','','1981-02-10 00:00:00',103.94,895,994,98.82,'Sun-Synch' from dual union all
select 12292,'012292','1978-026EL','deb Delta 139','',45221,'In Earth orbit','','1993-08-02 00:00:00',103.49,876,972,98.7,'Sun-Synch' from dual union all
select 12293,'012293','1978-026EM','deb Delta 139','',45221,'Reentered','2014-12-01 00:00:00','1993-08-01 00:00:00',100.22,736,801,98.57,'Sun-Synch' from dual union all
select 12294,'012294','1978-026EN','deb Delta 139','',45221,'In Earth orbit','','1993-08-08 00:00:00',100.71,711,874,98.41,'Sun-Synch' from dual union all
select 12295,'012295','1981-012A','Kiku-3','ETS-4',52535,'In Earth orbit','','1981-03-13 00:00:00',633.29,236,35865,28.48,'GEO Transfer' from dual union all
select 12296,'012296','1981-012B','N-II Stage 2 S/N N-7(F)','AJ10-118FJ',52535,'Reentered','1981-02-28 00:00:00','1981-02-19 00:00:00',90.01,211,339,30.35,'Intermediate' from dual union all
select 12297,'012297','1981-013A','Kosmos-1244','Parus',52536,'In Earth orbit','','1981-03-16 00:00:00',104.83,964,1008,82.95,'Intermediate' from dual union all
select 12298,'012298','1981-013B','S3M','S3M',52536,'In Earth orbit','','1991-03-28 00:00:00',104.67,956,1001,82.94,'Intermediate' from dual union all
select 12299,'012299','1981-014A','Kosmos-1245','Zenit-6',52537,'Landed','1981-02-27 00:00:00','1981-02-15 00:00:00',92.22,353,414,72.84,'Intermediate' from dual union all
select 123,'000123','61 Omicron 8','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-13 00:00:00',102.23,739,990,66.82,'Intermediate' from dual union all
select 1230,'001230','1965-012EG','deb Kosmos-57','',11762,'Reentered','1965-03-13 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12300,'012300','1981-014B','11S510','11S510',52537,'Reentered','1981-02-21 00:00:00','1981-02-15 00:00:00',89.94,193,349,72.84,'Intermediate' from dual union all
select 12301,'012301','1981-015A','Kosmos-1246','Siluet No. 1',52538,'Landed','1981-03-13 00:00:00','1981-02-26 00:00:00',89.1,204,256,64.91,'Intermediate' from dual union all
select 12302,'012302','1981-015B','11S510','11S510',52538,'Reentered','1981-02-21 00:00:00','1981-02-20 00:00:00',88.5,178,223,64.89,'Intermediate' from dual union all
select 12303,'012303','1981-016A','Kosmos-1247','Oko',52539,'In Earth orbit','','1981-03-22 00:00:00',717.82,649,39708,62.96,'Molniya' from dual union all
select 12304,'012304','1981-016B','11S510','11S510',52539,'Reentered','1981-03-13 00:00:00','1981-02-26 00:00:00',91.9,202,534,62.84,'Intermediate' from dual union all
select 12305,'012305','1981-016C','BOZ','BOZ',52539,'Reentered','1981-02-25 00:00:00','1981-02-21 00:00:00',91.35,145,536,62.86,'Intermediate' from dual union all
select 12306,'012306','1981-016D','deb Blok-I','',52539,'Reentered','1981-04-13 00:00:00','1981-02-26 00:00:00',91.84,197,533,62.85,'Intermediate' from dual union all
select 12307,'012307','1981-017A','Hinotori','ASTRO-A',52540,'Reentered','1991-07-11 00:00:00','1981-03-28 00:00:00',96.76,571,635,31.34,'Intermediate' from dual union all
select 12308,'012308','1981-017B','M-3A','M-3A',52540,'Reentered','1991-05-26 00:00:00','1981-03-26 00:00:00',96.76,572,635,31.34,'Intermediate' from dual union all
select 12309,'012309','1981-018A','Comstar D-4','Comstar D-4',52541,'In Earth orbit','','1981-03-23 00:00:00',1436.14,35772,35803,.33,'Stationary' from dual union all
select 1231,'001231','1965-012EH','deb Kosmos-57','',11762,'Reentered','1965-03-17 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12310,'012310','1981-017C','Hinotori despin weight','',52540,'Reentered','1981-11-25 00:00:00','1981-03-31 00:00:00',96.39,499,673,31.36,'Intermediate' from dual union all
select 12311,'012311','1981-016E','Blok-L','Blok-2BL No. 71655-639',52539,'In Earth orbit','','1981-04-06 00:00:00',703.54,664,38986,62.95,'Molniya' from dual union all
select 12312,'012312','1981-014C','Blenda','Sunshade',52537,'Reentered','1981-04-01 00:00:00','1981-02-27 00:00:00',92.05,342,409,72.82,'Intermediate' from dual union all
select 12313,'012313','1981-014D','KDU part','KDU part',52537,'Reentered','1981-03-05 00:00:00','1981-02-27 00:00:00',92.19,354,409,72.81,'Intermediate' from dual union all
select 12314,'012314','1981-014E','KDU part','KDU part',52537,'Reentered','1981-03-15 00:00:00','1981-02-27 00:00:00',92.2,353,412,72.84,'Intermediate' from dual union all
select 12315,'012315','1981-019A','OPS 1166','GAMBIT-3 Mission 4351',52542,'Reentered Attached','1981-06-20 00:00:00','1981-04-02 00:00:00',88.38,126,263,96.32,'Sun-Synch' from dual union all
select 12316,'012316','1981-014F','KDU','KDU',52537,'Reentered','1981-07-22 00:00:00','1981-03-26 00:00:00',91.89,344,391,72.84,'Intermediate' from dual union all
select 12317,'012317','1981-020A','Kosmos-1248','Feniks No. 940',52543,'Landed','1981-04-04 00:00:00','1981-03-12 00:00:00',88.98,158,291,67.14,'Intermediate' from dual union all
select 12318,'012318','1981-020B','11S510','11S510',52543,'Reentered','1981-03-08 00:00:00','1981-03-07 00:00:00',88.81,153,278,67.11,'Intermediate' from dual union all
select 12319,'012319','1981-021A','Kosmos-1249','US-A reactor',52544,'In Earth orbit','','1981-04-04 00:00:00',89.63,250,262,64.99,'Intermediate' from dual union all
select 1232,'001232','1965-012EJ','deb Kosmos-57','',11762,'Reentered','1965-03-24 00:00:00','1965-03-03 00:00:00',91.39,167,519,64.79,'Intermediate' from dual union all
select 12320,'012320','1981-022A','Kosmos-1250','Strela-1M',52545,'In Earth orbit','','1992-05-07 00:00:00',114.45,1390,1470,74.03,'Intermediate' from dual union all
select 12321,'012321','1981-022B','Kosmos-1251','Strela-1M',52545,'In Earth orbit','','1992-05-16 00:00:00',114.6,1403,1472,74.03,'Intermediate' from dual union all
select 12322,'012322','1981-022C','Kosmos-1252','Strela-1M',52545,'In Earth orbit','','1992-05-06 00:00:00',114.77,1418,1472,74.02,'Intermediate' from dual union all
select 12323,'012323','1981-022D','Kosmos-1253','Strela-1M',52545,'In Earth orbit','','1992-05-06 00:00:00',115.58,1468,1496,74.03,'Intermediate' from dual union all
select 12324,'012324','1981-022E','Kosmos-1254','Strela-1M',52545,'In Earth orbit','','1992-05-07 00:00:00',114.91,1431,1472,74.03,'Intermediate' from dual union all
select 12325,'012325','1981-022F','Kosmos-1255','Strela-1M',52545,'In Earth orbit','','1992-05-06 00:00:00',115.06,1445,1471,74.03,'Intermediate' from dual union all
select 12326,'012326','1981-022G','Kosmos-1256','Strela-1M',52545,'In Earth orbit','','1992-08-12 00:00:00',115.23,1456,1476,74.02,'Intermediate' from dual union all
select 12327,'012327','1981-022H','Kosmos-1257','Strela-1M',52545,'In Earth orbit','','1992-05-18 00:00:00',115.39,1468,1479,74.03,'Intermediate' from dual union all
select 12328,'012328','1981-022J','S3M','S3M',52545,'In Earth orbit','','1992-05-07 00:00:00',117.66,1456,1696,74.02,'Intermediate' from dual union all
select 12329,'012329','1978-014C','KM-A','KM-A',45209,'In Earth orbit','','1981-03-10 00:00:00',134.14,651,3954,65.27,'Medium' from dual union all
select 1233,'001233','1965-012EK','deb Kosmos-57','',11762,'Reentered','1965-03-20 00:00:00','1965-02-22 00:00:00',94.8,354,664,64.9,'Intermediate' from dual union all
select 12330,'012330','1978-014D','deb Kyokko','',45209,'In Earth orbit','','1981-03-10 00:00:00',134.33,644,3977,65.37,'Medium' from dual union all
select 12331,'012331','1978-014E','Kyokko despin weight','',45209,'In Earth orbit','','1981-03-09 00:00:00',133.61,644,3915,64.83,'Medium' from dual union all
select 12332,'012332','1976-067AY','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-03 00:00:00',116.85,973,2106,66.06,'Intermediate' from dual union all
select 12333,'012333','1976-067AZ','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-05 00:00:00',116.8,974,2100,65.85,'Intermediate' from dual union all
select 12334,'012334','1981-023A','Soyuz T-4','Soyuz 7K-ST No. 10L',52546,'Landed','1981-05-26 00:00:00','1981-04-13 00:00:00',91.48,343,352,51.63,'Intermediate' from dual union all
select 12335,'012335','1981-023B','11S510','11S510',52546,'Reentered','1981-03-14 00:00:00','1981-03-13 00:00:00',88.54,189,215,51.62,'Intermediate' from dual union all
select 12336,'012336','1980-091B','PAM-D','Star 48  S/N 10005-8',50301,'Reentered','1981-05-03 00:00:00','1981-02-04 00:00:00',571.47,130,32737,27.35,'Highly Ellip' from dual union all
select 12337,'012337','1981-024A','Kosmos-1258','IS',52547,'Deorbited','1981-03-14 00:00:00','1981-03-14 00:00:00',97.85,289,1023,65.82,'Intermediate' from dual;
insert into SATELLITE
select 12338,'012338','1981-024B','11S692','11S692',52547,'Reentered','1981-03-15 00:00:00','1981-03-15 00:00:00',89.11,148,313,65.09,'Intermediate' from dual union all
select 12339,'012339','1981-025A','DSP F9','DSP 10',52548,'In Earth orbit','','1981-05-01 00:00:00',1435.5,35750,35800,0,'Stationary' from dual union all
select 1234,'001234','1965-012EL','deb Kosmos-57','',11762,'Reentered','1965-03-20 00:00:00','1965-03-03 00:00:00',92.02,180,568,64.71,'Intermediate' from dual union all
select 12340,'012340','1981-025B','Titan 23C-22 Stage 2','LR91AJ11 23C-22',52548,'Reentered','1981-03-18 00:00:00','1981-03-17 00:00:00',88.87,145,292,28.6,'Intermediate' from dual union all
select 12341,'012341','1981-026A','Kosmos-1259','Zenit-6',52549,'Landed','1981-03-31 00:00:00','1981-03-19 00:00:00',90.34,205,377,70.35,'Intermediate' from dual union all
select 12342,'012342','1981-026B','11S510','11S510',52549,'Reentered','1981-03-25 00:00:00','1981-03-19 00:00:00',89.91,197,342,70.36,'Intermediate' from dual union all
select 12343,'012343','1980-030R','deb Kosmos-1174','',50240,'In Earth orbit','','1993-08-06 00:00:00',104.75,624,1342,66.44,'Intermediate' from dual union all
select 12344,'012344','1980-030S','deb Kosmos-1174','',50240,'Reentered','1982-09-08 00:00:00','1982-09-05 00:00:00',90.3,257,322,65.98,'Intermediate' from dual union all
select 12345,'012345','1980-030T','deb Kosmos-1174','',50240,'Reentered','1992-01-09 00:00:00','1991-05-26 00:00:00',95.79,384,729,66.03,'Intermediate' from dual union all
select 12346,'012346','1980-030U','deb Kosmos-1174','',50240,'Reentered','1982-04-24 00:00:00','1982-04-25 00:00:00',89.1,204,256,66.02,'Intermediate' from dual union all
select 12347,'012347','1980-030V','deb Kosmos-1174','',50240,'Reentered','1996-10-21 00:00:00','1992-09-21 00:00:00',96.49,401,780,66.03,'Intermediate' from dual union all
select 12348,'012348','1980-030W','deb Kosmos','[UNID082] RCS 0.1',50240,'Reentered','1991-02-26 00:00:00','1973-01-02 00:00:00',114.34,564,2287,65.52,'Intermediate' from dual union all
select 12349,'012349','1980-030X','deb Kosmos-1174','',50240,'Reentered','1982-12-15 00:00:00','1982-12-14 00:00:00',89.29,208,271,65.94,'Intermediate' from dual union all
select 1235,'001235','1965-012EM','deb Kosmos-57','',11762,'Reentered','1965-03-17 00:00:00','1965-03-03 00:00:00',90.99,143,504,64.71,'Intermediate' from dual union all
select 12350,'012350','1981-026C','Sensor cover','Sensor cover',52549,'Reentered','1981-04-11 00:00:00','1981-03-17 00:00:00',90.29,199,379,70.3,'Intermediate' from dual union all
select 12351,'012351','1981-027A','Raduga','Gran'' No. 18L',52550,'In Earth orbit','','1981-04-21 00:00:00',1434.86,35398,36127,.31,'Drift GEO' from dual union all
select 12352,'012352','1981-027B','Perekhodnik','Sredniy perekhodnik',52550,'Reentered','1981-03-18 00:00:00','1981-03-18 00:00:00',88.41,188,204,51.65,'Intermediate' from dual union all
select 12353,'012353','1981-027C','8S812','8S812',52550,'Reentered','1981-03-20 00:00:00','1981-03-19 00:00:00',87.89,165,176,51.63,'Intermediate' from dual union all
select 12354,'012354','1980-030Y','deb Kosmos-1174','',50240,'In Earth orbit','','1981-03-18 00:00:00',105.55,424,1617,66.13,'Intermediate' from dual union all
select 12355,'012355','1980-030Z','deb Kosmos-1174','',50240,'Reentered','1981-12-03 00:00:00','1981-12-02 00:00:00',89.67,207,310,66.06,'Intermediate' from dual union all
select 12356,'012356','1980-030AA','deb Kosmos-1174','',50240,'Reentered','1982-03-21 00:00:00','1982-03-21 00:00:00',87.2,122,150,66.38,'Intermediate' from dual union all
select 12357,'012357','1980-030AB','deb Kosmos-1174','',50240,'Reentered','1989-11-09 00:00:00','1989-11-07 00:00:00',92.3,373,402,66.13,'Intermediate' from dual union all
select 12358,'012358','1980-030AC','deb Kosmos-1174','',50240,'Reentered','1982-10-25 00:00:00','1982-10-14 00:00:00',91.33,276,404,66.02,'Intermediate' from dual union all
select 12359,'012359','1980-030AD','deb Kosmos-1174','',50240,'Reentered','1981-08-17 00:00:00','1981-03-18 00:00:00',99.29,363,1086,66.05,'Intermediate' from dual union all
select 1236,'001236','1965-012EN','deb Kosmos-57','',11762,'Reentered','1965-03-17 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12360,'012360','1980-030AE','deb Kosmos-1174','',50240,'Reentered','1994-02-16 00:00:00','1992-11-05 00:00:00',95.02,378,661,66.02,'Intermediate' from dual union all
select 12361,'012361','1980-030AF','deb Kosmos-1174','',50240,'Reentered','1981-07-27 00:00:00','1981-03-19 00:00:00',99.29,366,1084,66.11,'Intermediate' from dual union all
select 12362,'012362','1980-030AG','deb Kosmos-1174','',50240,'Reentered','1982-04-18 00:00:00','1982-04-17 00:00:00',89.34,212,272,66.04,'Intermediate' from dual union all
select 12363,'012363','1981-018B','Centaur AC-42','Centaur D-1AR AC-42',52541,'In Earth orbit','','1981-03-26 00:00:00',650.34,625,36349,20.55,'GEO Transfer' from dual union all
select 12364,'012364','1981-028A','Kosmos-1260','US-P',52551,'Reentered','1990-03-17 00:00:00','1981-04-19 00:00:00',93.29,429,443,65.03,'Intermediate' from dual union all
select 12365,'012365','1981-028B','11S692','11S692',52551,'Reentered','1981-03-21 00:00:00','1981-03-21 00:00:00',89.51,110,390,65,'Intermediate' from dual union all
select 12366,'012366','1981-029A','Soyuz-39','Soyuz 7K-T No. 55',52552,'Landed','1981-03-30 00:00:00','1981-03-24 00:00:00',91.37,335,349,51.63,'Intermediate' from dual union all
select 12367,'012367','1981-029B','11S510','11S510',52552,'Reentered','1981-03-25 00:00:00','1981-03-24 00:00:00',88.42,171,222,51.63,'Intermediate' from dual union all
select 12368,'012368','1981-030A','Molniya-3','Molniya-3 No. 24',52553,'Reentered','1992-10-19 00:00:00','1981-04-23 00:00:00',717.76,638,39716,62.75,'Molniya' from dual union all
select 12369,'012369','1981-030B','11S510','11S510',52553,'Reentered','1981-04-13 00:00:00','1981-03-31 00:00:00',91.55,207,495,62.8,'Intermediate' from dual union all
select 1237,'001237','1965-012EP','deb Kosmos-57','',11762,'Reentered','1965-03-20 00:00:00','1965-02-22 00:00:00',93.62,183,721,65.3,'Intermediate' from dual union all
select 12370,'012370','1981-030C','BOZ','BOZ',52553,'Reentered','1981-04-12 00:00:00','1981-04-01 00:00:00',91.71,188,528,62.82,'Intermediate' from dual union all
select 12371,'012371','1981-025C','Transtage 40','Transtage 40',52548,'In Earth orbit','','1981-03-16 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 12372,'012372','1978-026EP','deb Delta 139','[UNID083] RCS 0.02',45221,'In Earth orbit','','1993-07-31 00:00:00',104.02,908,989,82.99,'Intermediate' from dual union all
select 12373,'012373','1978-026EQ','deb Delta 139','',45221,'In Earth orbit','','1993-08-05 00:00:00',102.89,879,912,99.19,'Sun-Synch' from dual union all
select 12374,'012374','1978-026ER','deb Delta 139','',45221,'Reentered','1989-12-27 00:00:00','1989-12-27 00:00:00',90.12,266,294,98.65,'Sun-Synch' from dual union all
select 12375,'012375','1978-026ES','deb Delta 139','',45221,'Reentered','2014-06-13 00:00:00','1993-08-07 00:00:00',101.52,735,927,98.93,'Sun-Synch' from dual union all
select 12376,'012376','1981-031A','Kosmos-1261','Oko',52554,'In Earth orbit','','1981-04-30 00:00:00',718.39,638,39747,63.05,'Molniya' from dual union all
select 12377,'012377','1981-031B','11S510','11S510',52554,'Reentered','1981-04-17 00:00:00','1981-04-07 00:00:00',91.5,183,514,62.91,'Intermediate' from dual union all
select 12378,'012378','1981-031C','BOZ','BOZ',52554,'Reentered','1981-04-15 00:00:00','1981-04-07 00:00:00',91.33,205,475,62.81,'Intermediate' from dual union all
select 12379,'012379','1981-026D','Blenda','Sunshade',52549,'Reentered','1981-04-29 00:00:00','1981-03-31 00:00:00',92.16,348,413,70.37,'Intermediate' from dual union all
select 1238,'001238','1965-012EQ','deb Kosmos-57','',11762,'Reentered','1965-03-17 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12380,'012380','1981-026E','KDU part','KDU part',52549,'Reentered','1981-07-08 00:00:00','1981-04-16 00:00:00',91.89,339,396,70.36,'Intermediate' from dual union all
select 12381,'012381','1981-026F','KDU','KDU',52549,'Reentered','1981-08-30 00:00:00','1981-04-16 00:00:00',91.9,340,396,70.36,'Intermediate' from dual union all
select 12382,'012382','1981-026G','KDU part','KDU part',52549,'Reentered','1981-04-19 00:00:00','1981-03-31 00:00:00',94.1,379,570,70.36,'Intermediate' from dual union all
select 12383,'012383','1981-030D','Blok-L','Blok-ML No. 76738-510',52553,'Reentered','1993-06-29 00:00:00','1993-01-11 00:00:00',621.66,110,35390,63.92,'GEO Transfer' from dual union all
select 12384,'012384','1981-031D','Blok-L','Blok-2BL No. 41',52554,'In Earth orbit','','1981-05-03 00:00:00',707.51,619,39228,62.92,'Molniya' from dual union all
select 12385,'012385','1981-032A','Kosmos-1262','Zenit-6',52555,'Landed','1981-04-21 00:00:00','1981-04-09 00:00:00',90.35,195,389,72.87,'Intermediate' from dual union all
select 12386,'012386','1981-032B','11S510','11S510',52555,'Reentered','1981-04-13 00:00:00','1981-04-09 00:00:00',89.78,185,343,72.87,'Intermediate' from dual union all
select 12387,'012387','1981-032C','Sensor cover','Sensor cover',52555,'Reentered','1981-04-20 00:00:00','1981-04-07 00:00:00',90.14,192,370,72.9,'Intermediate' from dual union all
select 12388,'012388','1981-033A','Kosmos-1263','Vektor',52556,'In Earth orbit','','1981-05-10 00:00:00',108.95,394,1963,82.98,'Intermediate' from dual union all
select 12389,'012389','1981-033B','S3M','S3M',52556,'Reentered','2015-07-11 00:00:00','1990-07-06 00:00:00',105.54,379,1661,82.96,'Intermediate' from dual union all
select 1239,'001239','1965-012ER','deb Kosmos-57','',11762,'Reentered','1965-03-29 00:00:00','1965-03-10 00:00:00',92.44,176,613,64.68,'Intermediate' from dual union all
select 12390,'012390','1977-097CP','Garbage bag','',42225,'Reentered','1981-05-08 00:00:00','1981-04-10 00:00:00',91.54,329,372,51.64,'Intermediate' from dual union all
select 12391,'012391','1977-097CQ','Garbage bag','',42225,'Reentered','1981-06-22 00:00:00','1981-04-10 00:00:00',91.51,344,354,51.63,'Intermediate' from dual union all
select 12392,'012392','1977-097CR','Garbage bag','',42225,'Reentered','1981-04-29 00:00:00','1981-04-10 00:00:00',91.51,344,354,51.63,'Intermediate' from dual union all
select 12393,'012393','1977-097CS','Garbage bag','',42225,'Reentered','1981-04-25 00:00:00','1981-04-10 00:00:00',91.58,349,356,51.63,'Intermediate' from dual union all
select 12394,'012394','1977-097CT','Garbage bag','',42225,'Reentered','1981-05-05 00:00:00','1981-04-10 00:00:00',91.5,345,351,51.63,'Intermediate' from dual union all
select 12395,'012395','1977-097CU','Garbage bag','',42225,'Reentered','1981-05-08 00:00:00','1981-04-11 00:00:00',91.48,340,355,51.63,'Intermediate' from dual union all
select 12396,'012396','1981-027D','SOZ','SOZ',52550,'Reentered','1982-04-15 00:00:00','1981-04-17 00:00:00',647.97,253,36600,47.31,'GEO Transfer' from dual union all
select 12397,'012397','1981-027E','SOZ','SOZ',52550,'Reentered','1982-04-15 00:00:00','1981-04-18 00:00:00',649.54,306,36627,47.36,'GEO Transfer' from dual union all
select 12398,'012398','1977-097CV','Garbage bag','',42225,'Reentered','1981-06-27 00:00:00','1981-04-14 00:00:00',91.31,333,345,51.63,'Intermediate' from dual union all
select 12399,'012399','1981-034A','Columbia (STS-1)','OV-102',52557,'Landed','1981-04-14 00:00:00','1981-04-13 00:00:00',89.88,266,271,40.34,'Intermediate' from dual union all
select 124,'000124','61 Omicron 9','deb r/b Ablestar','',7273,'In Earth orbit','','1961-10-28 00:00:00',102.95,818,979,66.65,'Intermediate' from dual union all
select 1240,'001240','1965-012ES','deb Kosmos-57','',11762,'Reentered','1965-03-17 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12400,'012400','1981-035A','Kosmos-1264','Zenit-6',52558,'Landed','1981-04-29 00:00:00','1981-04-17 00:00:00',92.31,360,416,70.38,'Intermediate' from dual union all
select 12401,'012401','1981-035B','11S510','11S510',52558,'Reentered','1981-04-23 00:00:00','1981-04-17 00:00:00',90.21,202,368,70.39,'Intermediate' from dual union all
select 12402,'012402','1981-036A','Kosmos-1265','Zenit-6',52559,'Landed','1981-04-28 00:00:00','1981-04-18 00:00:00',89.62,225,287,72.85,'Intermediate' from dual union all
select 12403,'012403','1981-036B','11S510','11S510',52559,'Reentered','1981-04-19 00:00:00','1981-04-18 00:00:00',88.81,183,249,72.85,'Intermediate' from dual union all
select 12404,'012404','1980-022C','deb Kosmos-1168','',50232,'In Earth orbit','','1981-04-16 00:00:00',104.53,950,995,82.94,'Intermediate' from dual union all
select 12405,'012405','1979-013C','SAGE radiometer cover','',47497,'Reentered','1982-08-14 00:00:00','1979-03-31 00:00:00',97.5,519,760,54.92,'Intermediate' from dual union all
select 12406,'012406','1978-014F','Kyokko despin weight','',45209,'In Earth orbit','','1981-04-14 00:00:00',134.01,642,3950,65.88,'Medium' from dual union all
select 12407,'012407','1980-047F','Kosmos-1186 SS 3','ESO',50257,'Reentered','1981-05-03 00:00:00','1981-04-20 00:00:00',93.27,411,459,74.05,'Intermediate' from dual union all
select 12408,'012408','1980-047G','Kosmos-1186 SS 4','ESO',50257,'Reentered','1981-05-03 00:00:00','1981-04-20 00:00:00',93.44,431,455,74.06,'Intermediate' from dual union all
select 12409,'012409','1981-037A','Kosmos-1266','US-A reactor',52560,'In Earth orbit','','1981-05-21 00:00:00',103.61,892,967,64.76,'Intermediate' from dual union all
select 1241,'001241','1965-012ET','deb Kosmos-57','',11762,'Reentered','1965-03-17 00:00:00','1965-03-04 00:00:00',91.76,178,543,64.81,'Intermediate' from dual union all
select 12410,'012410','1981-032D','Blenda','Sunshade',52555,'Reentered','1981-04-23 00:00:00','1981-04-21 00:00:00',89.32,208,274,72.83,'Intermediate' from dual union all
select 12411,'012411','1981-032E','KDU','KDU',52555,'Reentered','1981-04-28 00:00:00','1981-04-21 00:00:00',89.52,204,298,72.87,'Intermediate' from dual union all
select 12412,'012412','1980-047H','Kosmos-1186 SS 5','ESO',50257,'Reentered','1981-05-01 00:00:00','1981-04-21 00:00:00',93.24,414,453,74.02,'Intermediate' from dual union all
select 12413,'012413','1980-047J','Kosmos-1186 SS 6','ESO',50257,'Reentered','1981-05-03 00:00:00','1981-04-21 00:00:00',93.27,416,454,74.01,'Intermediate' from dual union all
select 12414,'012414','1980-047K','Kosmos-1186 SS 7','ESO',50257,'Reentered','1981-05-03 00:00:00','1981-04-21 00:00:00',93.28,421,449,73.99,'Intermediate' from dual union all
select 12415,'012415','1980-047L','Kosmos-1186 SS 8','ESO',50257,'Reentered','1981-05-03 00:00:00','1981-04-21 00:00:00',93.43,425,459,74.01,'Intermediate' from dual union all
select 12416,'012416','1980-047M','Kosmos-1186 SS 9','ESO',50257,'Reentered','1981-05-03 00:00:00','1981-04-21 00:00:00',93.42,426,458,74.04,'Intermediate' from dual union all
select 12417,'012417','1981-032F','KDU part','KDU part',52555,'Reentered','1981-04-24 00:00:00','1981-04-21 00:00:00',89.31,199,282,72.87,'Intermediate' from dual union all
select 12418,'012418','1981-038A','OPS 7225','JUMPSEAT 6  (or QUASAR 4',52561,'In Earth orbit','','1981-04-24 00:00:00',706.56,500,39300,63,'Molniya' from dual union all
select 12419,'012419','1981-039A','Kosmos-1267','TKS No. 16301',52562,'Reentered Attached','1982-07-29 00:00:00','1981-05-25 00:00:00',89.84,257,277,51.58,'Intermediate' from dual union all
select 1242,'001242','1965-012EU','deb Kosmos-57','',11762,'Reentered','1965-03-24 00:00:00','1965-03-10 00:00:00',91.69,152,562,64.77,'Intermediate' from dual union all
select 12420,'012420','1981-039B','8S812','8S812',52562,'Reentered','1981-04-28 00:00:00','1981-04-27 00:00:00',88.14,169,196,51.6,'Intermediate' from dual union all
select 12421,'012421','1981-039C','TKS adapter','TKS adapter',52562,'Reentered','1981-04-28 00:00:00','1981-04-26 00:00:00',88.37,177,211,51.6,'Intermediate' from dual union all
select 12422,'012422','1980-047N','Kosmos-1186 SS 10','ESO',50257,'Reentered','1981-05-03 00:00:00','1981-04-25 00:00:00',92.84,399,429,73.99,'Intermediate' from dual union all
select 12423,'012423','1981-040A','Kosmos-1268','Zenit-6',52563,'Landed','1981-05-12 00:00:00','1981-04-30 00:00:00',90.58,240,366,70.38,'Intermediate' from dual union all
select 12424,'012424','1981-040B','11S510','11S510',52563,'Reentered','1981-05-06 00:00:00','1981-04-30 00:00:00',89.95,202,342,70.39,'Intermediate' from dual union all
select 12425,'012425','1981-036C','Blenda','Sunshade',52559,'Reentered','1981-05-15 00:00:00','1981-04-29 00:00:00',90.23,220,352,72.85,'Intermediate' from dual union all
select 12426,'012426','1981-036D','KDU part','KDU part',52559,'Reentered','1981-05-01 00:00:00','1981-04-28 00:00:00',89.95,215,329,72.85,'Intermediate' from dual union all
select 12427,'012427','1978-026ET','deb Delta 139','',45221,'In Earth orbit','','1981-10-24 00:00:00',103.58,789,1066,98.78,'Sun-Synch' from dual;
insert into SATELLITE
select 12428,'012428','1981-036E','KDU','KDU',52559,'Reentered','1982-03-19 00:00:00','1981-05-19 00:00:00',91.91,267,469,72.92,'Intermediate' from dual union all
select 12429,'012429','1981-037B','Kosmos-1266 DU','US-A DU',52560,'Reentered','1981-05-20 00:00:00','1981-04-28 00:00:00',89.54,246,258,64.97,'Intermediate' from dual union all
select 1243,'001243','1965-012EV','deb Kosmos-57','',11762,'Reentered','1965-03-17 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12430,'012430','1981-035C','KDU','KDU',52558,'Reentered','1981-10-24 00:00:00','1981-05-15 00:00:00',92.12,353,404,70.37,'Intermediate' from dual union all
select 12431,'012431','1981-035D','Blenda','Sunshade',52558,'Reentered','1981-06-08 00:00:00','1981-04-30 00:00:00',92.26,356,415,70.37,'Intermediate' from dual union all
select 12432,'012432','1981-035E','KDU part','KDU part',52558,'Reentered','1981-05-04 00:00:00','1981-04-29 00:00:00',92.13,355,403,70.39,'Intermediate' from dual union all
select 12433,'012433','1981-035F','KDU part','KDU part',52558,'Reentered','1981-04-30 00:00:00','1981-04-30 00:00:00',91.88,275,459,70.33,'Intermediate' from dual union all
select 12434,'012434','1981-037C','Kosmos-1266 RLS','US-A RLS',52560,'Reentered','1981-05-01 00:00:00','1981-05-01 00:00:00',87.52,144,160,64.95,'Intermediate' from dual union all
select 12435,'012435','1981-037D','Kosmos-1266 reactor core','US-A reactor core',52560,'In Earth orbit','','1991-11-01 00:00:00',103.37,892,944,64.77,'Intermediate' from dual union all
select 12436,'012436','1977-097CW','Garbage bag','',42225,'Reentered','1981-07-27 00:00:00','1981-05-06 00:00:00',91.17,329,335,51.62,'Intermediate' from dual union all
select 12437,'012437','1977-097CX','Garbage bag','',42225,'Reentered','1981-07-06 00:00:00','1981-05-06 00:00:00',91.23,331,339,51.62,'Intermediate' from dual union all
select 12438,'012438','1977-097CY','Garbage bag','',42225,'Reentered','1981-06-12 00:00:00','1981-05-06 00:00:00',91.2,331,336,51.65,'Intermediate' from dual union all
select 12439,'012439','1977-097CZ','Garbage bag','',42225,'Reentered','1981-07-06 00:00:00','1981-05-06 00:00:00',91.18,330,335,51.64,'Intermediate' from dual union all
select 1244,'001244','1965-016B','GGSE 2','POPPY 4 SV3',11766,'In Earth orbit','','1965-04-08 00:00:00',103.47,907,938,70.06,'Intermediate' from dual union all
select 12440,'012440','1977-097DA','Garbage bag','',42225,'Reentered','1981-06-11 00:00:00','1981-05-06 00:00:00',91.18,331,334,51.62,'Intermediate' from dual union all
select 12441,'012441','1977-097DB','Garbage bag','',42225,'Reentered','1981-05-24 00:00:00','1981-05-06 00:00:00',91.24,329,342,51.64,'Intermediate' from dual union all
select 12442,'012442','1981-041A','Kosmos-1269','Strela-2M',52564,'In Earth orbit','','1992-06-27 00:00:00',100.71,785,800,74.05,'Intermediate' from dual union all
select 12443,'012443','1981-041B','S3M','S3M',52564,'In Earth orbit','','1991-03-27 00:00:00',100.63,784,793,74.04,'Intermediate' from dual union all
select 12444,'012444','1977-097DC','Garbage bag','',42225,'Reentered','1981-07-10 00:00:00','1981-05-08 00:00:00',91.13,328,332,51.63,'Intermediate' from dual union all
select 12445,'012445','1980-098B','Centaur AC-54','Centaur D-1AR AC-54',50308,'In Earth orbit','','1981-02-20 00:00:00',541.39,151,31101,23.5,'Highly Ellip' from dual union all
select 12446,'012446','1981-038B','Agena D','Agena D',52561,'In Earth orbit','','1981-05-01 00:00:00',690.49,1000,38000,63.3,'Molniya' from dual union all
select 12447,'012447','1980-081F','Blok DM','11S86 No. 19L',50291,'In Earth orbit','','1993-06-07 00:00:00',1440.51,35851,35895,10.57,'Inclined Drift' from dual union all
select 12448,'012448','1981-040C','KDU','KDU',52563,'Reentered','1981-05-20 00:00:00','1981-05-12 00:00:00',89.37,218,269,70.38,'Intermediate' from dual union all
select 12449,'012449','1981-040D','Blenda','Sunshade',52563,'Reentered','1981-05-14 00:00:00','1981-05-12 00:00:00',91.42,238,451,70.39,'Intermediate' from dual union all
select 1245,'001245','1965-016J','Agena D 2701','Agena D',11766,'In Earth orbit','','1965-04-15 00:00:00',103.44,906,936,70.09,'Intermediate' from dual union all
select 12450,'012450','1981-040E','KDU part','KDU part',52563,'Reentered','1981-05-15 00:00:00','1981-05-12 00:00:00',91.42,238,451,70.39,'Intermediate' from dual union all
select 12451,'012451','1981-040F','KDU part','KDU part',52563,'Reentered','1981-05-15 00:00:00','1981-05-12 00:00:00',91.42,241,447,70.39,'Intermediate' from dual union all
select 12452,'012452','1981-040G','KDU part','KDU part',52563,'Reentered','1981-05-14 00:00:00','1981-05-12 00:00:00',89.14,196,268,70.33,'Intermediate' from dual union all
select 12453,'012453','1980-047P','Kosmos-1186 SS 11','ESO',50257,'Reentered','1981-05-28 00:00:00','1981-05-14 00:00:00',93.17,410,450,74.02,'Intermediate' from dual union all
select 12454,'012454','1981-042A','Soyuz-40','Soyuz 7K-T No. 56',52565,'Landed','1981-05-22 00:00:00','1981-05-15 00:00:00',90.63,268,343,51.63,'Intermediate' from dual union all
select 12455,'012455','1981-042B','11S510','11S510',52565,'Reentered','1981-05-16 00:00:00','1981-05-15 00:00:00',88.8,185,246,51.63,'Intermediate' from dual union all
select 12456,'012456','1981-043A','Meteor-2','Meteor-2 No. 8',52566,'In Earth orbit','','1981-06-14 00:00:00',102.39,852,892,81.27,'Intermediate' from dual union all
select 12457,'012457','1981-043B','8A92ME','8A92ME',52566,'In Earth orbit','','1981-06-16 00:00:00',102.49,829,924,81.28,'Intermediate' from dual union all
select 12458,'012458','1981-044A','Nova 1','Nova 1',52567,'In Earth orbit','','1991-03-24 00:00:00',108.92,1165,1189,90.07,'Polar' from dual union all
select 12459,'012459','1980-047Q','Kosmos-1186 SS 12','ESO',50257,'Reentered','1981-05-20 00:00:00','1981-05-15 00:00:00',93.09,406,446,74.03,'Intermediate' from dual union all
select 1246,'001246','1965-018A','Kosmos-60','E-6 No. 9',11768,'Reentered Attached','1965-03-17 00:00:00','1965-03-13 00:00:00',88.9,194,247,64.72,'Intermediate' from dual union all
select 12460,'012460','1981-044B','Star 20 E-25','Star 20 E-25',52567,'Reentered','1982-04-17 00:00:00','1981-06-16 00:00:00',97.32,348,913,90.15,'Polar' from dual union all
select 12461,'012461','1981-045A','Kosmos-1270','Feniks No. 980',52568,'Landed','1981-06-17 00:00:00','1981-05-25 00:00:00',89.22,163,309,64.86,'Intermediate' from dual union all
select 12462,'012462','1981-045B','11S510','11S510',52568,'Reentered','1981-05-21 00:00:00','1981-05-20 00:00:00',88.95,162,283,64.87,'Intermediate' from dual union all
select 12463,'012463','1981-045C','Kosmos-1270 part','Feniks part',52568,'Reentered','1981-05-19 00:00:00','1981-05-18 00:00:00',89.01,167,285,64.85,'Intermediate' from dual union all
select 12464,'012464','1981-046A','Kosmos-1271','Tselina-D',52569,'Reentered','2013-05-03 00:00:00','1991-03-28 00:00:00',96.66,590,607,81.19,'Intermediate' from dual union all
select 12465,'012465','1981-046B','8A92ME','8A92ME',52569,'In Earth orbit','','1991-03-28 00:00:00',96.94,576,649,81.21,'Intermediate' from dual union all
select 12466,'012466','1981-047A','Kosmos-1272','Zenit-6',52570,'Landed','1981-06-04 00:00:00','1981-05-23 00:00:00',92.3,360,415,70.39,'Intermediate' from dual union all
select 12467,'012467','1981-047B','11S510','11S510',52570,'Reentered','1981-05-31 00:00:00','1981-05-23 00:00:00',89.99,201,347,70.38,'Intermediate' from dual union all
select 12468,'012468','1981-047C','Sensor cover','Sensor cover',52570,'Reentered','1981-05-23 00:00:00','1981-05-23 00:00:00',89.32,184,298,70.35,'Intermediate' from dual union all
select 12469,'012469','1981-048A','Kosmos-1273','Fram',52571,'Landed','1981-06-04 00:00:00','1981-05-24 00:00:00',88.94,209,236,82.32,'Intermediate' from dual union all
select 1247,'001247','1965-019A','OPS 4920','GAMBIT SV 966',11769,'Deorbited','1965-03-17 00:00:00','1965-03-13 00:00:00',88.81,147,285,107.63,'Retrograde' from dual union all
select 12470,'012470','1981-048B','11S510','11S510',52571,'Reentered','1981-05-25 00:00:00','1981-05-24 00:00:00',88.52,191,212,82.32,'Intermediate' from dual union all
select 12471,'012471','1980-104E','Blok DM','11S86 No. 32L',50314,'In Earth orbit','','1993-05-29 00:00:00',1420.89,35359,35618,10.27,'Inclined Drift' from dual union all
select 12472,'012472','1981-049A','GOES 5','GOES E',52572,'In Earth orbit','','1981-06-23 00:00:00',1436.21,35715,35863,.41,'Stationary' from dual union all
select 12473,'012473','1981-049B','Delta 154','Delta SSPU DSV-3P-4',52572,'Reentered','1981-09-08 00:00:00','1981-06-21 00:00:00',108.97,159,2200,28.1,'Intermediate' from dual union all
select 12474,'012474','1981-050A','INTELSAT 501','INTELSAT V F1',52573,'In Earth orbit','','1981-06-23 00:00:00',1436.11,35778,35796,.5,'Stationary' from dual union all
select 12475,'012475','1981-039D','VA adapter','VA adapter',52562,'Reentered','1981-05-26 00:00:00','1981-05-24 00:00:00',89.65,248,267,51.58,'Intermediate' from dual union all
select 12476,'012476','1977-097DD','Garbage bag','',42225,'Reentered','1981-07-22 00:00:00','1981-05-25 00:00:00',91.69,340,375,51.63,'Intermediate' from dual union all
select 12477,'012477','1977-097DE','Garbage bag','',42225,'Reentered','1981-09-14 00:00:00','1981-05-25 00:00:00',91.66,343,369,51.64,'Intermediate' from dual union all
select 12478,'012478','1977-097DF','Garbage bag','',42225,'Reentered','1981-07-21 00:00:00','1981-05-25 00:00:00',91.56,333,370,51.62,'Intermediate' from dual union all
select 12479,'012479','1977-097DG','Garbage bag','',42225,'Reentered','1981-07-19 00:00:00','1981-05-25 00:00:00',91.52,338,361,51.63,'Intermediate' from dual union all
select 1248,'001248','1965-017D','USN satellite','',11767,'Reentered','1967-07-29 00:00:00','1965-04-17 00:00:00',97.78,289,1016,90,'Polar' from dual union all
select 12480,'012480','1980-047R','Kosmos-1186 SS 13','ESO',50257,'Reentered','1981-06-22 00:00:00','1981-05-25 00:00:00',93.41,429,453,73.99,'Intermediate' from dual union all
select 12481,'012481','1980-047S','Kosmos-1186 SS 14','ESO',50257,'Reentered','1981-06-18 00:00:00','1981-05-25 00:00:00',93.29,424,448,74.04,'Intermediate' from dual union all
select 12482,'012482','1980-047T','Kosmos-1186 SS 15 LRCS','ESO Large RCS',50257,'Reentered','1981-09-27 00:00:00','1981-05-25 00:00:00',93.2,414,449,74.02,'Intermediate' from dual union all
select 12483,'012483','1980-047U','Kosmos-1186 SS 16','ESO',50257,'Reentered','1981-06-22 00:00:00','1981-05-25 00:00:00',93.04,399,448,74.01,'Intermediate' from dual union all
select 12484,'012484','1980-047V','Kosmos-1186 SS 17','ESO',50257,'Reentered','1981-06-18 00:00:00','1981-05-25 00:00:00',92.98,395,445,74,'Intermediate' from dual union all
select 12485,'012485','1980-047W','Kosmos-1186 SS 18','ESO',50257,'Reentered','1981-06-17 00:00:00','1981-05-25 00:00:00',92.86,384,446,74.05,'Intermediate' from dual union all
select 12486,'012486','1980-047X','Kosmos-1186 SS 19','ESO',50257,'Reentered','1981-09-01 00:00:00','1981-05-25 00:00:00',93.06,404,445,74.05,'Intermediate' from dual union all
select 12487,'012487','1980-047Y','Kosmos-1186 SS 20','ESO',50257,'Reentered','1981-06-14 00:00:00','1981-05-25 00:00:00',93.08,406,445,74,'Intermediate' from dual union all
select 12488,'012488','1977-097DH','Soyuz T part','',42225,'Reentered','1981-09-09 00:00:00','1981-05-25 00:00:00',91.6,337,370,51.64,'Intermediate' from dual union all
select 12489,'012489','1977-097DJ','Soyuz T-4 BO','7K-ST No. 10L BO',42225,'Reentered','1981-10-07 00:00:00','1981-05-25 00:00:00',91.65,339,372,51.63,'Intermediate' from dual union all
select 1249,'001249','1965-017F','deb Oscar 03','',11767,'Reentered','1965-09-10 00:00:00','1965-04-17 00:00:00',97.61,270,1018,90,'Polar' from dual union all
select 12490,'012490','1980-047Z','Kosmos-1186 SS 21','ESO',50257,'Reentered','1981-06-20 00:00:00','1981-05-29 00:00:00',92.78,387,435,74.03,'Intermediate' from dual union all
select 12491,'012491','1981-051A','Rohini RS-D-1','Rohini RS-D-1',52574,'Reentered','1981-06-08 00:00:00','1981-06-02 00:00:00',90.06,181,374,46.25,'Intermediate' from dual union all
select 12492,'012492','1981-051B','SLV-3-4 D3','SLV-3-4 D3',52574,'Reentered','1981-06-06 00:00:00','1981-06-02 00:00:00',89.84,178,356,46.26,'Intermediate' from dual union all
select 12493,'012493','1977-097DK','Soyuz T part','',42225,'Reentered','1981-09-09 00:00:00','1981-05-31 00:00:00',91.56,335,368,51.64,'Intermediate' from dual union all
select 12494,'012494','1981-049C','Star 37E S/N 40091','Star 37E',52572,'Reentered','1982-04-15 00:00:00','1981-06-21 00:00:00',906.83,197,49116,26.54,'Highly Ellip' from dual union all
select 12495,'012495','1981-052A','Kosmos-1274','Feniks No. 942',52575,'Landed','1981-07-03 00:00:00','1981-06-11 00:00:00',89.19,162,307,67.15,'Intermediate' from dual union all
select 12496,'012496','1981-052B','11S510','11S510',52575,'Reentered','1981-06-07 00:00:00','1981-06-05 00:00:00',88.96,160,287,67.14,'Intermediate' from dual union all
select 12497,'012497','1981-050B','Centaur AC-56','Centaur D-1AR AC-56',52573,'In Earth orbit','','1981-06-23 00:00:00',586.37,174,33483,24.41,'Highly Ellip' from dual union all
select 12498,'012498','1981-047D','KDU','KDU',52570,'Reentered','1981-11-24 00:00:00','1981-06-20 00:00:00',92.22,358,409,70.39,'Intermediate' from dual union all
select 12499,'012499','1981-047E','Blenda','Sunshade',52570,'Reentered','1981-06-09 00:00:00','1981-06-04 00:00:00',91.9,312,424,70.39,'Intermediate' from dual union all
select 125,'000125','61 Omicron 10','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-14 00:00:00',103.62,865,995,66.93,'Intermediate' from dual union all
select 1250,'001250','1965-017B','EGRS II','Secor Type II S/N 3',11767,'Reentered','1968-02-26 00:00:00','1966-02-01 00:00:00',97.19,282,967,89.98,'Polar' from dual union all
select 12500,'012500','1981-048C','Blenda','Sunshade',52571,'Reentered','1981-06-05 00:00:00','1981-06-04 00:00:00',88.31,185,197,82.31,'Intermediate' from dual union all
select 12501,'012501','1981-048D','KDU','KDU',52571,'Reentered','1981-06-11 00:00:00','1981-06-04 00:00:00',88.74,212,212,82.3,'Intermediate' from dual union all
select 12502,'012502','1981-048E','KDU part','KDU part',52571,'Reentered','1981-06-06 00:00:00','1981-06-04 00:00:00',90.02,208,342,82.31,'Intermediate' from dual union all
select 12503,'012503','1981-048F','KDU part','KDU part',52571,'Reentered','1981-06-06 00:00:00','1981-06-04 00:00:00',90.68,214,402,82.3,'Intermediate' from dual union all
select 12504,'012504','1981-053A','Kosmos-1275','Parus',52576,'In Earth orbit','','1992-06-04 00:00:00',104.72,958,1005,82.96,'Intermediate' from dual union all
select 12505,'012505','1981-047F','KDU part','KDU part',52570,'Reentered','1981-07-08 00:00:00','1981-06-04 00:00:00',92.28,360,413,70.39,'Intermediate' from dual union all
select 12506,'012506','1981-047G','KDU part','KDU part',52570,'Reentered','1981-07-06 00:00:00','1981-06-04 00:00:00',92.14,348,411,70.39,'Intermediate' from dual union all
select 12507,'012507','1981-047H','KDU part','KDU part',52570,'Reentered','1981-08-05 00:00:00','1981-06-20 00:00:00',91.82,337,390,70.39,'Intermediate' from dual union all
select 12508,'012508','1981-053B','S3M','S3M',52576,'In Earth orbit','','1981-07-22 00:00:00',104.74,958,1007,82.96,'Intermediate' from dual union all
select 12509,'012509','1980-047AA','Kosmos-1186 SS 22','ESO',50257,'Reentered','1981-06-14 00:00:00','1981-06-04 00:00:00',91.75,337,384,74,'Intermediate' from dual union all
select 1251,'001251','1965-017E','deb Oscar 03','',11767,'Reentered','1965-08-01 00:00:00','1965-04-13 00:00:00',96.97,217,1011,90,'Polar' from dual union all
select 12510,'012510','1980-047AB','Kosmos-1186 SS 23','ESO',50257,'Reentered','1981-06-20 00:00:00','1981-06-07 00:00:00',91.42,322,367,74.02,'Intermediate' from dual union all
select 12511,'012511','1980-047AC','Kosmos-1186 SS 24','ESO',50257,'Reentered','1981-07-13 00:00:00','1981-06-07 00:00:00',90.92,294,345,74,'Intermediate' from dual union all
select 12512,'012512','1981-054A','Molniya-3','Molniya-3 No. 30',52577,'Reentered','1998-02-10 00:00:00','1981-07-09 00:00:00',717.78,477,39878,62.83,'Molniya' from dual union all
select 12513,'012513','1981-054B','11S510','11S510',52577,'Reentered','1981-06-27 00:00:00','1981-06-16 00:00:00',90.23,203,369,62.84,'Intermediate' from dual union all
select 12514,'012514','1981-054C','BOZ','BOZ',52577,'Reentered','1981-06-23 00:00:00','1981-06-12 00:00:00',90.68,192,423,62.84,'Intermediate' from dual union all
select 12515,'012515','1981-054D','deb Blok-I','',52577,'Reentered','1981-06-22 00:00:00','1981-06-11 00:00:00',90.92,206,434,62.81,'Intermediate' from dual union all
select 12516,'012516','1965-048J','deb NNS O-4','',11798,'Reentered','1981-12-04 00:00:00','1976-01-31 00:00:00',106.84,1020,1141,89.87,'Polar' from dual union all
select 12517,'012517','1981-055A','Kosmos-1276','Fram',52578,'Landed','1981-06-29 00:00:00','1981-06-18 00:00:00',88.96,212,234,82.37,'Intermediate' from dual;
insert into SATELLITE
select 12518,'012518','1981-055B','11S510','11S510',52578,'Reentered','1981-06-19 00:00:00','1981-06-18 00:00:00',88.44,188,207,82.36,'Intermediate' from dual union all
select 12519,'012519','1981-054E','Blok-L','Blok-L',52577,'Reentered','1999-04-30 00:00:00','1981-07-10 00:00:00',733.69,480,40657,62.83,'Molniya' from dual union all
select 1252,'001252','1965-018B','8K78I','8K78I',11768,'Reentered','1965-03-16 00:00:00','1965-03-14 00:00:00',88.45,189,207,64.74,'Intermediate' from dual union all
select 12520,'012520','1981-056A','Kosmos-1277','Zenit-6',52579,'Landed','1981-07-01 00:00:00','1981-06-19 00:00:00',92.3,359,416,70.4,'Intermediate' from dual union all
select 12521,'012521','1981-056B','11S510','11S510',52579,'Reentered','1981-06-28 00:00:00','1981-06-19 00:00:00',90.14,204,359,70.38,'Intermediate' from dual union all
select 12522,'012522','1981-056C','Kosmos-1277 part','Zenit-6 deb',52579,'Reentered','1981-06-22 00:00:00','1981-06-20 00:00:00',89.94,126,417,70.37,'Intermediate' from dual union all
select 12523,'012523','1981-056D','Kosmos-1277 part','Zenit-6 deb',52579,'Reentered','1981-06-28 00:00:00','1981-06-20 00:00:00',89.55,192,313,70.36,'Intermediate' from dual union all
select 12524,'012524','1981-056E','Kosmos-1277 part','Zenit-6 deb',52579,'Reentered','1981-06-29 00:00:00','1981-06-20 00:00:00',89.92,200,341,70.36,'Intermediate' from dual union all
select 12525,'012525','1981-056F','Kosmos-1277 part','Zenit-6 deb',52579,'Reentered','1981-06-22 00:00:00','1981-06-21 00:00:00',89.34,154,330,70.33,'Intermediate' from dual union all
select 12526,'012526','1981-056G','Kosmos-1277 part','Zenit-6 deb',52579,'Reentered','1981-06-23 00:00:00','1981-06-22 00:00:00',89.1,166,294,70.42,'Intermediate' from dual union all
select 12527,'012527','1976-077DG','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',116.39,1488,1549,102.01,'Sun-Synch' from dual union all
select 12528,'012528','1976-077DH','deb Delta 126','',39029,'In Earth orbit','','1993-07-31 00:00:00',117.37,1501,1625,102.45,'Sun-Synch' from dual union all
select 12529,'012529','1976-077DJ','deb Delta 126','',39029,'In Earth orbit','','1981-06-18 00:00:00',130.83,1454,2864,103.61,'Medium' from dual union all
select 1253,'001253','1965-018C','BOZ','BOZ',11768,'Reentered','1965-03-15 00:00:00','1965-03-14 00:00:00',88.56,178,229,64.75,'Intermediate' from dual union all
select 12530,'012530','1976-077DK','deb Delta 126','',39029,'In Earth orbit','','1981-06-18 00:00:00',117.38,1462,1665,102.08,'Sun-Synch' from dual union all
select 12531,'012531','1976-077DL','deb Delta 126','',39029,'In Earth orbit','','1993-07-30 00:00:00',118.23,1490,1712,101.88,'Sun-Synch' from dual union all
select 12532,'012532','1976-077DM','Delta 126 large debris','[UNID084] RCS 6.6',39029,'In Earth orbit','','1993-07-30 00:00:00',113.85,1346,1461,101.19,'Sun-Synch' from dual union all
select 12533,'012533','1976-077DN','deb Delta 126','',39029,'In Earth orbit','','1981-06-18 00:00:00',116.24,1425,1599,101.79,'Sun-Synch' from dual union all
select 12534,'012534','1976-077DP','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',116.19,1466,1553,102.07,'Sun-Synch' from dual union all
select 12535,'012535','1976-077DQ','deb Delta 126','',39029,'In Earth orbit','','1993-07-29 00:00:00',123.49,1507,2166,102.29,'Medium' from dual union all
select 12536,'012536','1976-077DR','deb Delta 126','',39029,'In Earth orbit','','1981-06-21 00:00:00',125.76,1451,2423,102.01,'Medium' from dual union all
select 12537,'012537','1976-077DS','deb Delta 126','',39029,'In Earth orbit','','1993-07-14 00:00:00',118.91,1502,1762,102.49,'Sun-Synch' from dual union all
select 12538,'012538','1976-077DT','deb Delta 126','',39029,'In Earth orbit','','1993-07-26 00:00:00',119.51,1498,1820,102.75,'Sun-Synch' from dual union all
select 12539,'012539','1976-077DU','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',114.83,1427,1468,101.81,'Sun-Synch' from dual union all
select 1254,'001254','1965-012EW','deb Kosmos-57','',11762,'Reentered','1965-03-12 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12540,'012540','1976-077DV','deb Delta 126','',39029,'In Earth orbit','','1981-06-18 00:00:00',116.2,1472,1548,102.41,'Sun-Synch' from dual union all
select 12541,'012541','1976-077DW','deb Delta 126','',39029,'Reentered','1992-05-10 00:00:00','1992-02-17 00:00:00',100.48,729,833,102.21,'Retrograde' from dual union all
select 12542,'012542','1976-077DX','deb Delta 126','',39029,'In Earth orbit','','1993-08-05 00:00:00',115.1,1412,1508,101.83,'Sun-Synch' from dual union all
select 12543,'012543','1976-067BA','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-07 00:00:00',116.43,1006,2035,66.02,'Intermediate' from dual union all
select 12544,'012544','1981-057A','Meteosat 2','Meteosat F2',52580,'In Earth orbit','','1981-07-27 00:00:00',1442.07,35838,35969,1.01,'Drift GEO' from dual union all
select 12545,'012545','1981-057B','Apple','Apple',52580,'In Earth orbit','','1983-01-27 00:00:00',1439.38,35746,35956,.28,'Drift GEO' from dual union all
select 12546,'012546','1981-057C','CAT 03','CAT 03',52580,'In Earth orbit','','1981-08-30 00:00:00',629.8,221,35700,10.23,'GEO Transfer' from dual union all
select 12547,'012547','1981-058A','Kosmos-1278','Oko',52581,'Reentered','2000-09-02 00:00:00','1981-07-19 00:00:00',718.49,665,39725,62.87,'Molniya' from dual union all
select 12548,'012548','1981-058B','11S510','11S510',52581,'Reentered','1981-07-28 00:00:00','1981-06-26 00:00:00',92.3,212,563,62.84,'Intermediate' from dual union all
select 12549,'012549','1981-058C','BOZ','BOZ',52581,'Reentered','1981-07-29 00:00:00','1981-06-26 00:00:00',92.45,210,580,62.81,'Intermediate' from dual union all
select 1255,'001255','1965-012EX','deb Kosmos-57','',11762,'Reentered','1965-03-20 00:00:00','1965-03-06 00:00:00',91.96,199,543,64.68,'Intermediate' from dual union all
select 12550,'012550','1981-021B','Kosmos-1249 RLS','US-A RLS',52544,'Reentered','1981-06-22 00:00:00','1981-06-20 00:00:00',89.4,235,255,64.99,'Intermediate' from dual union all
select 12551,'012551','1981-021C','Kosmos-1249 reactor core','US-A reactor core',52544,'In Earth orbit','','1991-11-17 00:00:00',103.56,898,955,64.96,'Intermediate' from dual union all
select 12552,'012552','1981-021D','Kosmos-1249 DU','US-A DU',52544,'Reentered','1981-07-19 00:00:00','1981-06-21 00:00:00',89.52,242,260,64.99,'Intermediate' from dual union all
select 12553,'012553','1981-059A','NOAA 7','NOAA C',52582,'In Earth orbit','','1981-07-24 00:00:00',101.97,843,860,98.9,'Sun-Synch' from dual union all
select 12554,'012554','1967-092E','NNS O-14 insulation','',15347,'Reentered','1989-03-20 00:00:00','1981-06-06 00:00:00',106.6,1024,1114,89.27,'Polar' from dual union all
select 12555,'012555','1967-092F','NNS O-14 insulation','',15347,'Reentered','1989-02-02 00:00:00','1981-06-05 00:00:00',106.62,1032,1109,89.26,'Polar' from dual union all
select 12556,'012556','1981-060A','Molniya-1','Molniya-1',52583,'Reentered','1991-12-14 00:00:00','1981-07-24 00:00:00',717.8,615,39741,62.84,'Molniya' from dual union all
select 12557,'012557','1981-060B','11S510','11S510',52583,'Reentered','1981-08-05 00:00:00','1981-07-01 00:00:00',92.51,213,583,62.81,'Intermediate' from dual union all
select 12558,'012558','1981-060C','BOZ','BOZ',52583,'Reentered','1981-07-22 00:00:00','1981-07-01 00:00:00',92.12,189,568,62.84,'Intermediate' from dual union all
select 12559,'012559','1981-059B','NOAA 7 solar array band','',52582,'In Earth orbit','','1990-05-16 00:00:00',101.32,817,825,99,'Sun-Synch' from dual union all
select 1256,'001256','1965-012EY','deb Kosmos-57','',11762,'Reentered','1965-03-21 00:00:00','1965-03-11 00:00:00',91.41,162,526,64.87,'Intermediate' from dual union all
select 12560,'012560','1981-059C','NOAA 7 solar array band','',52582,'In Earth orbit','','1990-05-17 00:00:00',101.31,817,824,99,'Sun-Synch' from dual union all
select 12561,'012561','1981-058D','Blok-L','Blok-2BL No. 71655642',52581,'Reentered','2013-06-10 00:00:00','1981-07-20 00:00:00',724.01,665,39997,62.87,'Molniya' from dual union all
select 12562,'012562','1981-057D','Ariane H8 L03','Ariane H8 S/N L04a',52580,'Reentered','1992-06-23 00:00:00','1991-01-16 00:00:00',93.53,126,769,10.27,'Equatorial' from dual union all
select 12563,'012563','1981-060D','Blok-L','Blok-L',52583,'Reentered','1992-03-27 00:00:00','1992-03-21 00:00:00',645.35,97,36622,63.78,'GEO Transfer' from dual union all
select 12564,'012564','1981-061A','Ekran','Ekran No. 21L',52584,'In Earth orbit','','1981-09-07 00:00:00',1436.04,35778,35793,.07,'Stationary' from dual union all
select 12565,'012565','1981-061B','Perekhodnik','Sredniy perekhodnik',52584,'Reentered','1981-06-26 00:00:00','1981-06-26 00:00:00',88.33,186,198,51.68,'Intermediate' from dual union all
select 12566,'012566','1981-061C','8S812','8S812',52584,'Reentered','1981-06-28 00:00:00','1981-06-27 00:00:00',88.06,174,183,51.62,'Intermediate' from dual union all
select 12567,'012567','1981-055C','KDU','KDU',52578,'Reentered','1981-07-06 00:00:00','1981-06-29 00:00:00',88.83,206,228,82.37,'Intermediate' from dual union all
select 12568,'012568','1981-055D','Blenda','Sunshade',52578,'Reentered','1981-06-30 00:00:00','1981-06-29 00:00:00',88.65,199,217,82.37,'Intermediate' from dual union all
select 12569,'012569','1981-055E','KDU part','KDU part',52578,'Reentered','1981-07-01 00:00:00','1981-06-29 00:00:00',90.61,220,389,82.36,'Intermediate' from dual union all
select 1257,'001257','1965-012EZ','deb Kosmos-57','',11762,'Reentered','1965-03-21 00:00:00','1965-03-12 00:00:00',90.63,161,450,64.68,'Intermediate' from dual union all
select 12570,'012570','1981-055F','KDU part','KDU part',52578,'Reentered','1981-07-01 00:00:00','1981-06-29 00:00:00',91.16,225,437,82.35,'Intermediate' from dual union all
select 12571,'012571','1981-062A','Kosmos-1279','Zenit-6',52585,'Landed','1981-07-15 00:00:00','1981-07-03 00:00:00',92.31,358,418,70.4,'Intermediate' from dual union all
select 12572,'012572','1981-062B','11S510','11S510',52585,'Reentered','1981-07-14 00:00:00','1981-07-03 00:00:00',90,206,342,70.39,'Intermediate' from dual union all
select 12573,'012573','1981-056H','Blenda','Sunshade',52579,'Reentered','1981-07-31 00:00:00','1981-07-01 00:00:00',92.3,359,416,70.4,'Intermediate' from dual union all
select 12574,'012574','1981-056J','KDU','KDU',52579,'Reentered','1981-12-12 00:00:00','1981-07-18 00:00:00',92.21,357,409,70.4,'Intermediate' from dual union all
select 12575,'012575','1981-056K','KDU part','KDU part',52579,'Reentered','1981-08-28 00:00:00','1981-07-21 00:00:00',91.7,332,384,70.39,'Intermediate' from dual union all
select 12576,'012576','1981-062C','Sensor cover','Sensor cover',52585,'Reentered','1981-07-02 00:00:00','1981-07-01 00:00:00',89.76,208,318,70.4,'Intermediate' from dual union all
select 12577,'012577','1981-063A','Kosmos-1280','Resurs-F1 17F41 No. 15',52586,'Landed','1981-07-16 00:00:00','1981-07-04 00:00:00',89.8,258,272,82.31,'Intermediate' from dual union all
select 12578,'012578','1981-063B','11S510','11S510',52586,'Reentered','1981-07-08 00:00:00','1981-07-04 00:00:00',89.09,202,257,82.3,'Intermediate' from dual union all
select 12579,'012579','1981-052C','Kosmos-1274 AO part','Feniks AO part',52575,'Reentered','1981-07-08 00:00:00','1981-07-03 00:00:00',89.16,152,314,67.14,'Intermediate' from dual union all
select 1258,'001258','1965-012FA','deb Kosmos-57','',11762,'Reentered','1965-03-17 00:00:00','1965-03-10 00:00:00',89.6,160,350,64.86,'Intermediate' from dual union all
select 12580,'012580','1976-077DY','deb Delta 126','',39029,'In Earth orbit','','1993-08-03 00:00:00',127.73,1505,2542,102.41,'Medium' from dual union all
select 12581,'012581','1976-077DZ','deb Delta 126','',39029,'In Earth orbit','','1993-07-31 00:00:00',123.08,1506,2131,102.47,'Medium' from dual union all
select 12582,'012582','1965-048K','deb NNS O-4','',11798,'Reentered','1982-03-09 00:00:00','1978-09-29 00:00:00',106.8,1023,1134,89.86,'Polar' from dual union all
select 12583,'012583','1981-064A','Kosmos-1281','Zenit-6',52587,'Landed','1981-07-21 00:00:00','1981-07-09 00:00:00',92.25,356,413,72.84,'Intermediate' from dual union all
select 12584,'012584','1981-064B','11S510','11S510',52587,'Reentered','1981-07-17 00:00:00','1981-07-09 00:00:00',90.14,194,369,72.84,'Intermediate' from dual union all
select 12585,'012585','1981-065A','Meteor-Priroda','Meteor-Priroda No. 2-4',52588,'In Earth orbit','','1990-08-20 00:00:00',97.06,595,641,97.77,'Sun-Synch' from dual union all
select 12586,'012586','1981-065B','8A92ME','8A92ME',52588,'In Earth orbit','','1991-03-27 00:00:00',97.07,605,632,97.81,'Sun-Synch' from dual union all
select 12587,'012587','1981-065C','Cataloging error','',52588,'Spurious','1981-07-10 00:00:00','1981-07-10 00:00:00',97.66,626,667,97.95,'Sun-Synch' from dual union all
select 12588,'012588','1981-066A','Kosmos-1282','Feniks No. 951',52589,'Landed','1981-08-14 00:00:00','1981-07-23 00:00:00',89.28,163,315,64.9,'Intermediate' from dual union all
select 12589,'012589','1981-066B','11S510','11S510',52589,'Reentered','1981-07-19 00:00:00','1981-07-17 00:00:00',89.16,166,300,64.92,'Intermediate' from dual union all
select 1259,'001259','1965-019B','Agena D 4816','Agena D',11769,'Reentered','1965-03-16 00:00:00','1965-03-13 00:00:00',88.95,146,300,107.65,'Retrograde' from dual union all
select 12590,'012590','1981-062D','KDU','KDU',52585,'Reentered','1981-12-20 00:00:00','1981-08-01 00:00:00',92.21,355,410,70.4,'Intermediate' from dual union all
select 12591,'012591','1981-062E','Blenda','Sunshade',52585,'Reentered','1981-09-07 00:00:00','1981-07-31 00:00:00',91.92,341,397,70.4,'Intermediate' from dual union all
select 12592,'012592','1981-062F','KDU part','KDU part',52585,'Reentered','1981-07-16 00:00:00','1981-07-16 00:00:00',89.62,204,308,70.45,'Intermediate' from dual union all
select 12593,'012593','1981-062G','KDU part','KDU part',52585,'Reentered','1981-08-22 00:00:00','1981-07-16 00:00:00',92.31,357,419,70.4,'Intermediate' from dual union all
select 12594,'012594','1981-063C','Blenda','Sunshade',52586,'Reentered','1981-08-02 00:00:00','1981-07-16 00:00:00',89.74,252,272,82.31,'Intermediate' from dual union all
select 12595,'012595','1981-063D','KDU','KDU',52586,'Reentered','1981-08-10 00:00:00','1981-07-16 00:00:00',89.59,238,271,82.3,'Intermediate' from dual union all
select 12596,'012596','1981-063E','KDU part','KDU part',52586,'Reentered','1981-07-22 00:00:00','1981-07-17 00:00:00',91.56,260,442,82.32,'Intermediate' from dual union all
select 12597,'012597','1981-063F','KDU part','KDU part',52586,'Reentered','1981-07-22 00:00:00','1981-07-16 00:00:00',89.37,197,289,82.37,'Intermediate' from dual union all
select 12598,'012598','1981-067A','Kosmos-1283','Zenit-6',52590,'Landed','1981-07-31 00:00:00','1981-07-19 00:00:00',91.48,324,371,82.34,'Intermediate' from dual union all
select 12599,'012599','1981-067B','11S510','11S510',52590,'Reentered','1981-07-19 00:00:00','1981-07-18 00:00:00',88.41,172,220,82.34,'Intermediate' from dual union all
select 126,'000126','61 Omicron 11','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-12 00:00:00',102.98,820,979,66.82,'Intermediate' from dual union all
select 1260,'001260','1965-019C','deb KH-7','',11769,'Reentered','1965-03-21 00:00:00','1965-03-13 00:00:00',88.73,145,278,107.53,'Retrograde' from dual union all
select 12600,'012600','1981-064C','Blenda','Sunshade',52587,'Reentered','1981-08-20 00:00:00','1981-07-21 00:00:00',92.24,356,413,72.84,'Intermediate' from dual union all
select 12601,'012601','1981-064D','KDU part','KDU part',52587,'Reentered','1981-09-06 00:00:00','1981-08-06 00:00:00',91.73,330,389,72.84,'Intermediate' from dual union all
select 12602,'012602','1981-064E','KDU','KDU',52587,'Reentered','1981-12-09 00:00:00','1981-10-13 00:00:00',91.31,322,356,72.83,'Intermediate' from dual union all
select 12603,'012603','1981-061D','SOZ','SOZ',52584,'Reentered','1982-04-15 00:00:00','1981-09-14 00:00:00',626.4,223,35523,47.08,'GEO Transfer' from dual union all
select 12604,'012604','1981-061E','SOZ','SOZ',52584,'Reentered','1982-04-20 00:00:00','1981-09-21 00:00:00',628.63,269,35592,47.05,'GEO Transfer' from dual union all
select 12605,'012605','1980-021C','deb Kosmos-1167','',50231,'Reentered','1981-08-06 00:00:00','1981-08-04 00:00:00',89.7,244,276,64.97,'Intermediate' from dual union all
select 12606,'012606','1980-021D','deb Kosmos-1167','',50231,'Reentered','1982-12-01 00:00:00','1981-07-26 00:00:00',92.2,344,421,65,'Intermediate' from dual union all
select 12607,'012607','1980-021E','deb Kosmos-1167','',50231,'Reentered','1981-08-22 00:00:00','1981-08-18 00:00:00',90.14,256,307,64.96,'Intermediate' from dual;
insert into SATELLITE
select 12608,'012608','1980-021F','deb Kosmos-1167','',50231,'Reentered','1981-08-12 00:00:00','1981-07-26 00:00:00',92.24,347,422,65.01,'Intermediate' from dual union all
select 12609,'012609','1980-021G','deb Kosmos-1167','',50231,'Reentered','1981-10-15 00:00:00','1981-08-22 00:00:00',92.13,339,419,65.01,'Intermediate' from dual union all
select 1261,'001261','1965-012FB','deb Kosmos-57','',11762,'Reentered','1965-04-06 00:00:00','1965-03-12 00:00:00',93.16,182,677,64.71,'Intermediate' from dual union all
select 12610,'012610','1980-021H','deb Kosmos-1167','',50231,'Reentered','1981-12-14 00:00:00','1981-07-26 00:00:00',92.17,371,390,65.01,'Intermediate' from dual union all
select 12611,'012611','1980-021J','deb Kosmos-1167','',50231,'Reentered','1981-08-22 00:00:00','1981-08-21 00:00:00',89.23,210,263,64.95,'Intermediate' from dual union all
select 12612,'012612','1980-021K','deb Kosmos-1167','',50231,'Reentered','1981-08-16 00:00:00','1981-08-14 00:00:00',88.27,172,206,64.97,'Intermediate' from dual union all
select 12613,'012613','1980-021L','deb Kosmos-1167','',50231,'Reentered','1981-08-02 00:00:00','1981-07-29 00:00:00',89.44,228,266,64.99,'Intermediate' from dual union all
select 12614,'012614','1981-068A','Kosmos-1284','Zenit-6',52591,'Landed','1981-08-12 00:00:00','1981-08-03 00:00:00',91.47,324,369,82.33,'Intermediate' from dual union all
select 12615,'012615','1980-021M','deb Kosmos-1167','',50231,'Reentered','1981-08-27 00:00:00','1981-07-30 00:00:00',92.01,350,397,65.03,'Intermediate' from dual union all
select 12616,'012616','1980-021N','deb Kosmos-1167','',50231,'Reentered','1981-08-27 00:00:00','1981-07-28 00:00:00',91.84,320,410,64.98,'Intermediate' from dual union all
select 12617,'012617','1981-068B','11S510','11S510',52591,'Reentered','1981-07-31 00:00:00','1981-07-30 00:00:00',88.26,169,208,82.33,'Intermediate' from dual union all
select 12618,'012618','1981-069A','Raduga','Gran'' No. 19L',52592,'In Earth orbit','','1981-08-29 00:00:00',1436.29,35730,35851,.38,'Stationary' from dual union all
select 12619,'012619','1981-069B','8S812','8S812',52592,'Reentered','1981-08-01 00:00:00','1981-07-31 00:00:00',88.25,183,193,51.62,'Intermediate' from dual union all
select 1262,'001262','1965-012FC','deb Kosmos-57','',11762,'Reentered','1965-03-15 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12620,'012620','1981-069C','Perekhodnik','Sredniy perekhodnik',52592,'Reentered','1981-07-31 00:00:00','1981-07-31 00:00:00',88.32,182,201,51.61,'Intermediate' from dual union all
select 12621,'012621','1981-067C','Blenda','Sunshade',52590,'Reentered','1981-08-05 00:00:00','1981-07-30 00:00:00',91.22,320,349,82.34,'Intermediate' from dual union all
select 12622,'012622','1981-067D','KDU','KDU',52590,'Reentered','1981-10-08 00:00:00','1981-08-16 00:00:00',91.28,314,361,82.34,'Intermediate' from dual union all
select 12623,'012623','1981-067E','KDU part','KDU part',52590,'Reentered','1981-08-21 00:00:00','1981-08-02 00:00:00',91.38,313,371,82.33,'Intermediate' from dual union all
select 12624,'012624','1981-070A','Dynamics Explorer 1','DE A',52593,'In Earth orbit','','1981-09-07 00:00:00',410.85,558,23300,89.92,'Highly Ellip' from dual union all
select 12625,'012625','1981-070B','Dynamics Explorer 2','DE B',52593,'Reentered','1983-02-19 00:00:00','1981-09-03 00:00:00',97.61,299,989,89.99,'Polar' from dual union all
select 12626,'012626','1981-070C','Delta 155','Delta SSPU DSV-3P-4',52593,'Reentered','1988-11-23 00:00:00','1981-09-03 00:00:00',98.08,377,957,89.99,'Polar' from dual union all
select 12627,'012627','1981-071A','Kosmos-1285','Oko',52594,'In Earth orbit','','1981-09-03 00:00:00',727.37,623,40204,63.01,'Molniya' from dual union all
select 12628,'012628','1981-070D','PAF','Payload Attach Fitting',52593,'Reentered','1982-03-17 00:00:00','1981-09-06 00:00:00',97.3,298,961,89.98,'Polar' from dual union all
select 12629,'012629','1981-071B','BOZ','BOZ',52594,'Reentered','1981-08-26 00:00:00','1981-08-11 00:00:00',91.84,186,544,62.92,'Intermediate' from dual union all
select 1263,'001263','1965-012FD','deb Kosmos-57','',11762,'Reentered','1965-03-21 00:00:00','1965-03-12 00:00:00',90.28,148,429,64.81,'Intermediate' from dual union all
select 12630,'012630','1981-071C','11S510','11S510',52594,'Reentered','1981-09-01 00:00:00','1981-08-11 00:00:00',91.88,212,522,62.83,'Intermediate' from dual union all
select 12631,'012631','1981-072A','Kosmos-1286','US-PM1',52595,'Reentered','1982-10-16 00:00:00','1981-09-03 00:00:00',93.28,428,442,65.04,'Intermediate' from dual union all
select 12632,'012632','1981-072B','11S692','11S692',52595,'Reentered','1981-08-04 00:00:00','1981-08-04 00:00:00',88.23,91,283,65,'Intermediate' from dual union all
select 12633,'012633','1981-069D','SOZ','SOZ',52592,'Reentered','1982-04-20 00:00:00','1981-09-01 00:00:00',648.68,334,36555,47.27,'GEO Transfer' from dual union all
select 12634,'012634','1981-069E','SOZ','SOZ',52592,'Reentered','1982-04-15 00:00:00','1981-09-03 00:00:00',647.34,280,36541,47.2,'GEO Transfer' from dual union all
select 12635,'012635','1981-073A','FLTSATCOM F5','FLTSATCOM F5',52596,'In Earth orbit','','1981-09-06 00:00:00',1470.76,34863,38062,6.37,'Inclined Drift' from dual union all
select 12636,'012636','1981-074A','Kosmos-1287','Strela-1M',52597,'In Earth orbit','','1992-05-07 00:00:00',115.72,1464,1513,74.04,'Intermediate' from dual union all
select 12637,'012637','1981-074B','Kosmos-1288','Strela-1M',52597,'In Earth orbit','','1992-05-07 00:00:00',115.5,1464,1492,74.03,'Intermediate' from dual union all
select 12638,'012638','1981-074C','Kosmos-1289','Strela-1M',52597,'In Earth orbit','','1992-05-25 00:00:00',114.78,1427,1463,74.03,'Intermediate' from dual union all
select 12639,'012639','1981-074D','Kosmos-1290','Strela-1M',52597,'In Earth orbit','','1992-05-06 00:00:00',114.94,1441,1464,74.03,'Intermediate' from dual union all
select 1264,'001264','1965-012FE','deb Kosmos-57','',11762,'Reentered','1965-03-26 00:00:00','1965-03-13 00:00:00',90.34,171,412,64.73,'Intermediate' from dual union all
select 12640,'012640','1981-074E','Kosmos-1291','Strela-1M',52597,'In Earth orbit','','1992-05-16 00:00:00',115.12,1457,1464,74.03,'Intermediate' from dual union all
select 12641,'012641','1981-074F','Kosmos-1292','Strela-1M',52597,'In Earth orbit','','1992-05-20 00:00:00',115.32,1464,1476,74.03,'Intermediate' from dual union all
select 12642,'012642','1981-074G','Kosmos-1293','Strela-1M',52597,'In Earth orbit','','1992-05-17 00:00:00',114.59,1409,1465,74.03,'Intermediate' from dual union all
select 12643,'012643','1981-074H','Kosmos-1294','Strela-1M',52597,'In Earth orbit','','1992-05-07 00:00:00',114.4,1392,1465,74.03,'Intermediate' from dual union all
select 12644,'012644','1981-074J','S3M','S3M',52597,'In Earth orbit','','1991-03-23 00:00:00',117.46,1463,1671,74.05,'Intermediate' from dual union all
select 12645,'012645','1981-075A','IK Bolgaria-1300','IK Bulgaria-1300',52598,'In Earth orbit','','1991-04-28 00:00:00',101.69,793,884,81.22,'Intermediate' from dual union all
select 12646,'012646','1981-075B','8A92ME','8A92ME',52598,'In Earth orbit','','1981-09-07 00:00:00',101.9,798,900,81.23,'Intermediate' from dual union all
select 12647,'012647','1976-077EA','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',122.22,1524,2036,100.66,'Medium' from dual union all
select 12648,'012648','1981-053C','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',104.62,964,990,82.97,'Intermediate' from dual union all
select 12649,'012649','1981-053D','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-08 00:00:00',104.99,960,1027,82.92,'Intermediate' from dual union all
select 1265,'001265','1965-012FF','deb Kosmos-57','',11762,'Reentered','1965-03-15 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12650,'012650','1981-053E','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-03 00:00:00',105.02,950,1040,82.94,'Intermediate' from dual union all
select 12651,'012651','1981-053F','deb Kosmos-1275','',52576,'Reentered','2014-03-01 00:00:00','1993-08-06 00:00:00',103.59,832,1025,82.91,'Intermediate' from dual union all
select 12652,'012652','1981-053G','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',104.36,924,1005,82.95,'Intermediate' from dual union all
select 12653,'012653','1981-053H','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',104.52,947,997,82.94,'Intermediate' from dual union all
select 12654,'012654','1981-053J','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',103.45,893,951,83,'Intermediate' from dual union all
select 12655,'012655','1981-053K','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',104.8,956,1014,82.96,'Intermediate' from dual union all
select 12656,'012656','1981-053L','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-29 00:00:00',104.21,936,979,82.96,'Intermediate' from dual union all
select 12657,'012657','1981-053M','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-29 00:00:00',103.27,840,987,82.76,'Intermediate' from dual union all
select 12658,'012658','1981-053N','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-29 00:00:00',105.08,956,1041,82.97,'Intermediate' from dual union all
select 12659,'012659','1981-053P','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',105.01,939,1051,82.76,'Intermediate' from dual union all
select 1266,'001266','1965-012FG','deb Kosmos-57','',11762,'Reentered','1965-03-06 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12660,'012660','1981-053Q','deb Kosmos-1275','',52576,'In Earth orbit','','1981-09-01 00:00:00',107.04,978,1202,82.86,'Intermediate' from dual union all
select 12661,'012661','1981-053R','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',106.12,952,1141,82.8,'Intermediate' from dual union all
select 12662,'012662','1981-053S','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-29 00:00:00',104.88,964,1014,82.79,'Intermediate' from dual union all
select 12663,'012663','1981-053T','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-29 00:00:00',105.22,959,1051,82.96,'Intermediate' from dual union all
select 12664,'012664','1981-053U','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',105.39,951,1075,82.9,'Intermediate' from dual union all
select 12665,'012665','1981-053V','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',104.55,946,1001,82.97,'Intermediate' from dual union all
select 12666,'012666','1981-053W','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',105.32,953,1066,82.88,'Intermediate' from dual union all
select 12667,'012667','1981-053X','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-03 00:00:00',104.85,958,1017,82.84,'Intermediate' from dual union all
select 12668,'012668','1981-053Y','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',103.4,876,963,83.11,'Intermediate' from dual union all
select 12669,'012669','1981-053Z','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',104.75,919,1046,83,'Intermediate' from dual union all
select 1267,'001267','1965-020A','Kosmos-61','Strela-1',11770,'Reentered','1968-01-15 00:00:00','1965-04-14 00:00:00',104.92,262,1719,56.01,'Intermediate' from dual union all
select 12670,'012670','1981-053AA','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-31 00:00:00',105.24,959,1053,82.93,'Intermediate' from dual union all
select 12671,'012671','1981-053AB','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',103.99,930,965,82.92,'Intermediate' from dual union all
select 12672,'012672','1981-053AC','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',104.73,961,1003,82.95,'Intermediate' from dual union all
select 12673,'012673','1981-053AD','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-30 00:00:00',105.38,963,1062,82.86,'Intermediate' from dual union all
select 12674,'012674','1981-053AE','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',105.54,936,1104,82.94,'Intermediate' from dual union all
select 12675,'012675','1981-053AF','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',105.16,949,1055,82.94,'Intermediate' from dual union all
select 12676,'012676','1981-053AG','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',104.55,939,1008,82.95,'Intermediate' from dual union all
select 12677,'012677','1981-076A','Himawari 2','GMS 2',52599,'In Earth orbit','','1981-10-23 00:00:00',1436.04,35779,35792,1.1,'Stationary' from dual union all
select 12678,'012678','1981-076B','N-II Stage 2 S/N N-8(F)','AJ10-118FJ',52599,'Reentered','1981-08-24 00:00:00','1981-08-12 00:00:00',91.38,171,514,30.33,'Intermediate' from dual union all
select 12679,'012679','1981-070E','Star 37N S/N 14009','Star 37N',52593,'In Earth orbit','','1981-09-02 00:00:00',412.91,560,23420,89.9,'Highly Ellip' from dual union all
select 1268,'001268','1965-020B','Kosmos-62','Strela-1',11770,'Reentered','1968-09-24 00:00:00','1965-04-14 00:00:00',104.65,261,1696,56.04,'Intermediate' from dual union all
select 12680,'012680','1981-071D','Blok-L','Blok-L',52594,'In Earth orbit','','1981-09-08 00:00:00',722.79,621,39981,62.98,'Molniya' from dual union all
select 12681,'012681','1981-077A','Kosmos-1295','Parus',52600,'In Earth orbit','','1992-07-02 00:00:00',104.65,946,1011,82.92,'Intermediate' from dual union all
select 12682,'012682','1981-077B','S3M','S3M',52600,'In Earth orbit','','1991-03-25 00:00:00',104.53,947,998,82.92,'Intermediate' from dual union all
select 12683,'012683','1981-068C','KDU','KDU',52591,'Reentered','1981-10-16 00:00:00','1981-08-28 00:00:00',91.19,309,357,82.33,'Intermediate' from dual union all
select 12684,'012684','1981-068D','Blenda','Sunshade',52591,'Reentered','1981-08-29 00:00:00','1981-08-12 00:00:00',91.46,323,370,82.31,'Intermediate' from dual union all
select 12685,'012685','1981-068E','KDU part','KDU part',52591,'Reentered','1981-08-16 00:00:00','1981-08-12 00:00:00',91.48,338,357,82.27,'Intermediate' from dual union all
select 12686,'012686','1981-068F','KDU part','KDU part',52591,'Reentered','1981-09-06 00:00:00','1981-08-12 00:00:00',91.3,310,367,82.34,'Intermediate' from dual union all
select 12687,'012687','1981-078A','Kosmos-1296','Feniks No. 943',52601,'Landed','1981-09-13 00:00:00','1981-08-20 00:00:00',89.83,173,359,67.14,'Intermediate' from dual union all
select 12688,'012688','1981-078B','11S510','11S510',52601,'Reentered','1981-08-17 00:00:00','1981-08-15 00:00:00',89.02,160,293,67.14,'Intermediate' from dual union all
select 12689,'012689','1981-053AH','deb Kosmos-1275','',52576,'Reentered','1991-03-04 00:00:00','1981-08-29 00:00:00',101.19,653,976,82.89,'Intermediate' from dual union all
select 1269,'001269','1965-020C','Kosmos-63','Strela-1',11770,'Reentered','1967-11-04 00:00:00','1965-04-14 00:00:00',104.2,259,1655,56.03,'Intermediate' from dual union all
select 12690,'012690','1981-053AJ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-30 00:00:00',105.62,947,1099,82.95,'Intermediate' from dual union all
select 12691,'012691','1981-053AK','deb Kosmos-1275','',52576,'Reentered','2008-10-23 00:00:00','1981-08-30 00:00:00',101.99,724,982,82.89,'Intermediate' from dual union all
select 12692,'012692','1981-053AL','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',102.27,802,931,82.9,'Intermediate' from dual union all
select 12693,'012693','1981-053AM','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',102.77,845,934,83.15,'Intermediate' from dual union all
select 12694,'012694','1981-053AN','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-27 00:00:00',106.3,963,1148,83.08,'Intermediate' from dual union all
select 12695,'012695','1981-053AP','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',106.26,958,1149,83.01,'Intermediate' from dual union all
select 12696,'012696','1981-053AQ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',107.03,965,1214,83.08,'Intermediate' from dual union all
select 12697,'012697','1981-053AR','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-29 00:00:00',105.3,941,1076,82.79,'Intermediate' from dual union all
select 12698,'012698','1981-053AS','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',103.77,915,959,82.94,'Intermediate' from dual;
insert into SATELLITE
select 12699,'012699','1981-053AT','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',105.48,941,1093,82.95,'Intermediate' from dual union all
select 127,'000127','61 Omicron 12','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-11 00:00:00',103.09,810,1000,66.82,'Intermediate' from dual union all
select 1270,'001270','1965-020D','S3','S3',11770,'Reentered','1966-08-05 00:00:00','1965-04-14 00:00:00',105.31,247,1771,56.07,'Intermediate' from dual union all
select 12700,'012700','1981-053AU','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-29 00:00:00',104.33,928,999,82.79,'Intermediate' from dual union all
select 12701,'012701','1981-053AV','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',104.47,938,1002,82.85,'Intermediate' from dual union all
select 12702,'012702','1981-053AW','deb Kosmos-1275','',52576,'Reentered','2002-01-01 00:00:00','1981-08-30 00:00:00',101.53,687,976,82.92,'Intermediate' from dual union all
select 12703,'012703','1981-053AX','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',103.55,913,939,82.98,'Intermediate' from dual union all
select 12704,'012704','1981-053AY','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',105.62,957,1090,82.99,'Intermediate' from dual union all
select 12705,'012705','1981-053AZ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',104.24,930,988,82.9,'Intermediate' from dual union all
select 12706,'012706','1981-053BA','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-03 00:00:00',105.78,942,1119,82.9,'Intermediate' from dual union all
select 12707,'012707','1981-053BB','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',106.48,963,1165,83.01,'Intermediate' from dual union all
select 12708,'012708','1981-053BC','deb Kosmos-1275','',52576,'In Earth orbit','','1981-09-07 00:00:00',104.43,849,1087,82.87,'Intermediate' from dual union all
select 12709,'012709','1981-053BD','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',106.35,960,1155,82.98,'Intermediate' from dual union all
select 1271,'001271','1965-016A','GREB 6','POPPY 4 SV2',11766,'In Earth orbit','','1965-04-08 00:00:00',103.48,907,940,70.08,'Intermediate' from dual union all
select 12710,'012710','1981-053BE','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',106.2,951,1150,83.02,'Intermediate' from dual union all
select 12711,'012711','1981-053BF','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-03 00:00:00',105.37,963,1060,82.97,'Intermediate' from dual union all
select 12712,'012712','1981-053BG','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-29 00:00:00',106.2,949,1152,83.01,'Intermediate' from dual union all
select 12713,'012713','1981-053BH','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',103.48,909,938,82.95,'Intermediate' from dual union all
select 12714,'012714','1981-053BJ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',106.53,963,1169,82.98,'Intermediate' from dual union all
select 12715,'012715','1981-053BK','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',102.37,811,930,83.12,'Intermediate' from dual union all
select 12716,'012716','1981-079A','Kosmos-1297','Zenit-6',52602,'Landed','1981-08-30 00:00:00','1981-08-20 00:00:00',89.77,225,302,72.86,'Intermediate' from dual union all
select 12717,'012717','1981-079B','11S510','11S510',52602,'Reentered','1981-08-25 00:00:00','1981-08-20 00:00:00',89.65,189,326,72.86,'Intermediate' from dual union all
select 12718,'012718','1981-079C','Sensor cover','Sensor cover',52602,'Reentered','1981-08-19 00:00:00','1981-08-18 00:00:00',89.45,188,307,72.88,'Intermediate' from dual union all
select 12719,'012719','1981-053BL','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',104.21,940,974,82.96,'Intermediate' from dual union all
select 1272,'001272','1965-016H','Dodecapole I','Dodecapole I',11766,'In Earth orbit','','1965-04-08 00:00:00',103.47,906,940,70.07,'Intermediate' from dual union all
select 12720,'012720','1981-053BM','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-29 00:00:00',106.1,933,1159,83.26,'Intermediate' from dual union all
select 12721,'012721','1981-053BN','deb Kosmos-1275','',52576,'Reentered','1989-11-28 00:00:00','1989-11-28 00:00:00',90.71,283,335,82.91,'Intermediate' from dual union all
select 12722,'012722','1981-053BP','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',104.75,930,1036,82.91,'Intermediate' from dual union all
select 12723,'012723','1981-053BQ','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-29 00:00:00',104.7,959,1002,82.97,'Intermediate' from dual union all
select 12724,'012724','1981-053BR','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',104.04,935,964,82.95,'Intermediate' from dual union all
select 12725,'012725','1981-053BS','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',105.71,938,1117,83.15,'Intermediate' from dual union all
select 12726,'012726','1981-053BT','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',104.13,914,994,82.96,'Intermediate' from dual union all
select 12727,'012727','1981-053BU','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',104.9,911,1069,83.29,'Intermediate' from dual union all
select 12728,'012728','1981-053BV','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',105.27,954,1061,82.81,'Intermediate' from dual union all
select 12729,'012729','1981-053BW','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',106.55,964,1170,83.05,'Intermediate' from dual union all
select 1273,'001273','1965-021A','OPS 7353','DAPP 11',11771,'Reentered','1989-12-31 00:00:00','1965-04-18 00:00:00',97.53,526,755,99.03,'Sun-Synch' from dual union all
select 12730,'012730','1981-053BX','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',103.46,906,939,82.95,'Intermediate' from dual union all
select 12731,'012731','1981-053BY','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',104.95,929,1056,82.94,'Intermediate' from dual union all
select 12732,'012732','1981-053BZ','deb Kosmos-1275','',52576,'Reentered','1982-08-03 00:00:00','1981-09-01 00:00:00',103.73,897,972,82.98,'Intermediate' from dual union all
select 12733,'012733','1981-053CA','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',103.57,899,956,82.9,'Intermediate' from dual union all
select 12734,'012734','1981-053CB','deb Kosmos-1275','',52576,'Reentered','1991-11-04 00:00:00','1991-11-04 00:00:00',89.48,237,261,82.62,'Intermediate' from dual union all
select 12735,'012735','1981-053CC','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',105.82,955,1110,83.15,'Intermediate' from dual union all
select 12736,'012736','1981-053CD','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',106.65,969,1173,83.03,'Intermediate' from dual union all
select 12737,'012737','1981-053CE','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-30 00:00:00',107.32,971,1234,83.02,'Intermediate' from dual union all
select 12738,'012738','1981-053CF','deb Kosmos-1275','',52576,'Reentered','1989-09-12 00:00:00','1989-09-11 00:00:00',90.73,301,320,82.66,'Intermediate' from dual union all
select 12739,'012739','1981-053CG','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-31 00:00:00',104.42,958,977,82.95,'Intermediate' from dual union all
select 1274,'001274','1965-022A','Voskhod-2','Voskhod 3KD No. 4',11772,'Landed','1965-03-19 00:00:00','1965-03-18 00:00:00',90.91,167,472,64.79,'Intermediate' from dual union all
select 12740,'012740','1981-053CH','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',103.57,907,948,82.86,'Intermediate' from dual union all
select 12741,'012741','1981-053CJ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',104.64,931,1025,82.96,'Intermediate' from dual union all
select 12742,'012742','1981-053CK','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',104.94,957,1026,82.95,'Intermediate' from dual union all
select 12743,'012743','1981-053CL','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',104.96,947,1039,82.93,'Intermediate' from dual union all
select 12744,'012744','1981-053CM','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',103.86,906,976,82.95,'Intermediate' from dual union all
select 12745,'012745','1981-053CN','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',104.8,945,1026,83.15,'Intermediate' from dual union all
select 12746,'012746','1981-053CP','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-31 00:00:00',105.27,959,1055,82.96,'Intermediate' from dual union all
select 12747,'012747','1981-053CQ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',106.56,962,1173,83.07,'Intermediate' from dual union all
select 12748,'012748','1981-053CR','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-31 00:00:00',105.39,977,1049,82.99,'Intermediate' from dual union all
select 12749,'012749','1981-053CS','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',105.19,955,1052,82.81,'Intermediate' from dual union all
select 1275,'001275','1965-012FH','deb Kosmos-57','',11762,'Reentered','1965-03-21 00:00:00','1965-03-17 00:00:00',89.22,215,257,64.69,'Intermediate' from dual union all
select 12750,'012750','1981-053CT','deb Kosmos-1275','',52576,'Reentered','2001-07-19 00:00:00','1993-07-31 00:00:00',100.18,732,803,82.91,'Intermediate' from dual union all
select 12751,'012751','1981-053CU','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',104.7,958,1003,82.95,'Intermediate' from dual union all
select 12752,'012752','1981-053CV','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',106.59,963,1175,83.03,'Intermediate' from dual union all
select 12753,'012753','1981-053CW','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',105.52,939,1099,82.81,'Intermediate' from dual union all
select 12754,'012754','1981-053CX','deb Kosmos-1275','',52576,'Reentered','1983-08-03 00:00:00','1983-08-02 00:00:00',92.25,368,402,82.88,'Intermediate' from dual union all
select 12755,'012755','1981-053CY','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-30 00:00:00',102.7,876,896,82.96,'Intermediate' from dual union all
select 12756,'012756','1981-053CZ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',102.21,837,890,82.99,'Intermediate' from dual union all
select 12757,'012757','1981-053DA','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-30 00:00:00',104.88,945,1033,82.95,'Intermediate' from dual union all
select 12758,'012758','1981-053DB','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',104.95,947,1038,82.87,'Intermediate' from dual union all
select 12759,'012759','1981-053DC','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',103.64,889,973,82.86,'Intermediate' from dual union all
select 1276,'001276','1965-012FJ','deb Kosmos-57','',11762,'Reentered','1965-03-20 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12760,'012760','1981-053DD','deb Kosmos-1275','',52576,'Reentered','2002-11-21 00:00:00','1981-08-31 00:00:00',102.18,747,977,83.02,'Intermediate' from dual union all
select 12761,'012761','1981-053DE','deb Kosmos-1275','',52576,'Reentered','1990-03-16 00:00:00','1981-08-29 00:00:00',101.96,661,1042,82.9,'Intermediate' from dual union all
select 12762,'012762','1981-053DF','deb Kosmos-1275','',52576,'Reentered','1981-11-04 00:00:00','1981-11-03 00:00:00',90.76,271,353,82.83,'Intermediate' from dual union all
select 12763,'012763','1981-053DG','deb Kosmos-1275','',52576,'Reentered','2002-08-20 00:00:00','1981-08-29 00:00:00',104.8,953,1018,82.96,'Intermediate' from dual union all
select 12764,'012764','1981-053DH','deb Kosmos-1275','',52576,'Reentered','2015-02-13 00:00:00','1993-08-04 00:00:00',102.69,865,907,82.94,'Intermediate' from dual union all
select 12765,'012765','1981-053DJ','deb Kosmos-1275','',52576,'Reentered','1988-04-12 00:00:00','1988-04-11 00:00:00',92.53,364,433,83.2,'Intermediate' from dual union all
select 12766,'012766','1981-053DK','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',105.9,955,1118,82.95,'Intermediate' from dual union all
select 12767,'012767','1981-053DL','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',103.26,883,943,82.97,'Intermediate' from dual union all
select 12768,'012768','1981-053DM','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',104.57,936,1012,82.94,'Intermediate' from dual union all
select 12769,'012769','1981-053DN','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-29 00:00:00',105.76,965,1095,82.82,'Intermediate' from dual union all
select 1277,'001277','1965-012FK','deb Kosmos-57','',11762,'Reentered','1965-03-20 00:00:00','1965-03-16 00:00:00',90.43,154,438,64.96,'Intermediate' from dual union all
select 12770,'012770','1981-053DP','deb Kosmos-1275','',52576,'Reentered','2011-11-03 00:00:00','1981-08-29 00:00:00',102.34,759,980,82.87,'Intermediate' from dual union all
select 12771,'012771','1981-053DQ','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-30 00:00:00',107.37,972,1237,82.88,'Intermediate' from dual union all
select 12772,'012772','1981-053DR','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',104.15,907,1003,82.93,'Intermediate' from dual union all
select 12773,'012773','1981-053DS','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',104.89,819,1160,83,'Intermediate' from dual union all
select 12774,'012774','1981-053DT','deb Kosmos-1275','',52576,'Reentered','1991-10-07 00:00:00','1991-10-06 00:00:00',91,309,339,82.62,'Intermediate' from dual union all
select 12775,'012775','1981-053DU','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',106.89,958,1208,83.34,'Intermediate' from dual union all
select 12776,'012776','1981-080A','Kosmos-1298','Yantar''-4K2 No. 1',52603,'Landed','1981-10-02 00:00:00','1981-08-28 00:00:00',88.92,162,281,64.9,'Intermediate' from dual union all
select 12777,'012777','1981-080B','11S510','11S510',52603,'Reentered','1981-08-24 00:00:00','1981-08-23 00:00:00',88.35,155,231,64.9,'Intermediate' from dual union all
select 12778,'012778','1981-073B','Centaur AC-59','Centaur D-1AR AC-59',52596,'Reentered','1983-04-16 00:00:00','1981-09-05 00:00:00',598.73,166,34141,26.61,'Highly Ellip' from dual union all
select 12779,'012779','1981-053DV','deb Kosmos-1275','',52576,'In Earth orbit','','1981-08-30 00:00:00',104.58,957,993,82.6,'Intermediate' from dual union all
select 1278,'001278','1965-012FL','deb Kosmos-57','',11762,'Reentered','1965-03-22 00:00:00','1965-03-16 00:00:00',89.91,151,389,65.03,'Intermediate' from dual union all
select 12780,'012780','1981-053DW','deb Kosmos-1275','',52576,'In Earth orbit','','1981-09-01 00:00:00',104.95,973,1012,82.95,'Intermediate' from dual union all
select 12781,'012781','1981-053DX','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',105.22,949,1061,83.18,'Intermediate' from dual union all
select 12782,'012782','1981-053DY','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-03 00:00:00',105.49,956,1079,83,'Intermediate' from dual union all
select 12783,'012783','1981-081A','Kosmos-1299','US-A reactor',52604,'In Earth orbit','','1991-04-08 00:00:00',103.95,927,964,65.11,'Intermediate' from dual union all
select 12784,'012784','1978-118D','SOZ','SOZ',45313,'Reentered','1979-08-01 00:00:00','1979-08-01 00:00:00',97.61,131,1158,46.7,'Intermediate' from dual union all
select 12785,'012785','1981-082A','Kosmos-1300','Tselina-D',52605,'In Earth orbit','','1991-03-28 00:00:00',97.07,606,630,82.48,'Intermediate' from dual union all
select 12786,'012786','1981-082B','S5M','S5M',52605,'In Earth orbit','','1991-03-27 00:00:00',97.42,620,651,82.49,'Intermediate' from dual union all
select 12787,'012787','1981-012C','Star 37E S/N 40089','Star 37E',52535,'In Earth orbit','','1981-04-12 00:00:00',629.37,251,35648,28.28,'GEO Transfer' from dual union all
select 12788,'012788','1981-083A','Kosmos-1301','Resurs-F1 17F41 No. 16',52606,'Landed','1981-09-10 00:00:00','1981-08-29 00:00:00',89.8,258,272,82.31,'Intermediate' from dual;
insert into SATELLITE
select 12789,'012789','1981-083B','11S510','11S510',52606,'Reentered','1981-08-31 00:00:00','1981-08-29 00:00:00',88.9,200,240,82.32,'Intermediate' from dual union all
select 1279,'001279','1965-012FM','deb Kosmos-57','',11762,'Reentered','1965-03-31 00:00:00','1965-03-15 00:00:00',91.95,177,563,64.53,'Intermediate' from dual union all
select 12790,'012790','1981-083C','Sensor cover','Sensor cover',52606,'Reentered','1981-09-02 00:00:00','1981-08-28 00:00:00',89.11,198,263,82.33,'Intermediate' from dual union all
select 12791,'012791','1981-084A','Kosmos-1302','Strela-2M',52607,'In Earth orbit','','1992-06-26 00:00:00',100.58,773,799,74.02,'Intermediate' from dual union all
select 12792,'012792','1981-084B','S3M','S3M',52607,'In Earth orbit','','1991-03-28 00:00:00',100.48,772,791,74.02,'Intermediate' from dual union all
select 12793,'012793','1981-084C','deb Kosmos-1302','Strela-2M debris',52607,'In Earth orbit','','1981-09-29 00:00:00',101,802,809,74.05,'Intermediate' from dual union all
select 12794,'012794','1981-079D','KDU','KDU',52602,'Reentered','1981-09-05 00:00:00','1981-08-31 00:00:00',88.97,215,233,72.86,'Intermediate' from dual union all
select 12795,'012795','1981-079E','Blenda','Sunshade',52602,'Reentered','1981-08-31 00:00:00','1981-08-30 00:00:00',88.4,179,212,72.89,'Intermediate' from dual union all
select 12796,'012796','1981-079F','KDU part','KDU part',52602,'Reentered','1981-09-02 00:00:00','1981-08-30 00:00:00',88.47,196,201,72.85,'Intermediate' from dual union all
select 12797,'012797','1981-079G','KDU part','KDU part',52602,'Reentered','1981-08-31 00:00:00','1981-08-31 00:00:00',89.98,201,346,72.85,'Intermediate' from dual union all
select 12798,'012798','1981-070F','deb Delta','',52593,'Reentered','1981-08-31 00:00:00','1981-08-31 00:00:00',91.02,290,360,90.1,'Polar' from dual union all
select 12799,'012799','1981-085A','OPS 3984','KENNEN 4',52608,'Deorbited','1984-11-23 00:00:00','1981-11-05 00:00:00',92.42,276,511,97,'Sun-Synch' from dual union all
select 128,'000128','61 Omicron 13','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-11 00:00:00',103.26,840,986,66.82,'Intermediate' from dual union all
select 1280,'001280','1965-012FN','deb Kosmos-57','',11762,'Reentered','1965-03-24 00:00:00','1965-03-08 00:00:00',92.4,168,617,64.67,'Intermediate' from dual union all
select 12800,'012800','1981-085B','Titan 23D-22 Stage 2','LR91AJ11 23D-22',52608,'Reentered','1981-10-12 00:00:00','1981-09-14 00:00:00',91.53,234,466,96.97,'Sun-Synch' from dual union all
select 12801,'012801','1981-086A','Kosmos-1303','Zenit-6',52609,'Landed','1981-09-18 00:00:00','1981-09-06 00:00:00',92.29,359,414,70.41,'Intermediate' from dual union all
select 12802,'012802','1981-086B','11S510','11S510',52609,'Reentered','1981-09-12 00:00:00','1981-09-06 00:00:00',89.9,199,340,70.4,'Intermediate' from dual union all
select 12803,'012803','1981-087A','Kosmos-1304','Tsikada',52610,'In Earth orbit','','1981-10-12 00:00:00',103.91,910,977,82.94,'Intermediate' from dual union all
select 12804,'012804','1981-087B','S3M','S3M',52610,'In Earth orbit','','1991-03-28 00:00:00',103.75,904,967,82.93,'Intermediate' from dual union all
select 12805,'012805','1981-086C','Sensor cover','Sensor cover',52609,'Reentered','1981-09-05 00:00:00','1981-09-04 00:00:00',88.99,182,268,70.3,'Intermediate' from dual union all
select 12806,'012806','1981-086D','Sensor cover','Sensor cover',52609,'Reentered','1981-09-05 00:00:00','1981-09-05 00:00:00',88.93,182,262,70.4,'Intermediate' from dual union all
select 12807,'012807','1981-081B','Kosmos-1299 RLS','US-A DU',52604,'Reentered','1981-09-07 00:00:00','1981-09-06 00:00:00',89.36,233,252,65.01,'Intermediate' from dual union all
select 12808,'012808','1981-081C','Kosmos-1299 reactor core','US-A reactor core',52604,'In Earth orbit','','1995-05-02 00:00:00',103.71,908,960,65.11,'Intermediate' from dual union all
select 12809,'012809','1981-081D','Kosmos-1299 DU','US-A',52604,'Reentered','1981-09-27 00:00:00','1981-09-06 00:00:00',89.56,245,261,65,'Intermediate' from dual union all
select 1281,'001281','1965-012FP','deb Kosmos-57','',11762,'Reentered','1965-03-19 00:00:00','1965-03-04 00:00:00',92.66,175,634,64.79,'Intermediate' from dual union all
select 12810,'012810','1981-076C','Star 37E S/N 40090','Star 37E',52599,'Reentered','1990-11-23 00:00:00','1981-09-13 00:00:00',648.76,185,36708,28.74,'GEO Transfer' from dual union all
select 12811,'012811','1981-083D','KDU','KDU',52606,'Reentered','1981-09-24 00:00:00','1981-09-10 00:00:00',89.83,262,271,82.32,'Intermediate' from dual union all
select 12812,'012812','1981-083E','Blenda','Sunshade',52606,'Reentered','1981-09-17 00:00:00','1981-09-10 00:00:00',89.79,260,269,82.32,'Intermediate' from dual union all
select 12813,'012813','1981-083F','KDU part','KDU part',52606,'Reentered','1981-11-09 00:00:00','1981-09-10 00:00:00',89.94,221,322,82.12,'Intermediate' from dual union all
select 12814,'012814','1979-077E','deb Kosmos-1124','',47561,'Reentered','2015-06-14 00:00:00','1992-05-20 00:00:00',720.21,2001,38474,68.33,'GEO Transfer' from dual union all
select 12815,'012815','1977-068E','deb Kosmos-931','',42196,'Reentered','1989-01-04 00:00:00','2004-07-18 00:00:00',540.71,1023,30192,26.72,'Highly Ellip' from dual union all
select 12816,'012816','1979-077F','deb Kosmos-1124','',47561,'In Earth orbit','','1982-08-15 00:00:00',686.51,1904,36897,63.62,'GEO Transfer' from dual union all
select 12817,'012817','1980-057E','deb Kosmos-1191','',50267,'In Earth orbit','','1992-05-06 00:00:00',720.7,2498,38001,68.42,'GEO Transfer' from dual union all
select 12818,'012818','1981-088A','Kosmos-1305','Molniya-3 No. 28',52611,'In Earth orbit','','1981-10-12 00:00:00',263.69,628,13859,62.87,'Medium' from dual union all
select 12819,'012819','1981-088B','11S510','11S510',52611,'Reentered','1981-10-05 00:00:00','1981-09-19 00:00:00',91.94,207,533,62.78,'Intermediate' from dual union all
select 1282,'001282','1965-012FQ','deb Kosmos-57','',11762,'Reentered','1965-03-21 00:00:00','1965-03-10 00:00:00',91.24,180,491,64.87,'Intermediate' from dual union all
select 12820,'012820','1981-088C','BOZ','BOZ',52611,'Reentered','1981-10-03 00:00:00','1981-09-18 00:00:00',92.05,194,557,62.81,'Intermediate' from dual union all
select 12821,'012821','1981-088D','deb Blok-I','',52611,'Reentered','1982-01-13 00:00:00','1981-10-11 00:00:00',92.87,254,577,62.83,'Intermediate' from dual union all
select 12822,'012822','1981-088E','deb Blok-I','',52611,'Reentered','1981-12-04 00:00:00','1981-09-12 00:00:00',93.16,230,628,62.83,'Intermediate' from dual union all
select 12823,'012823','1978-026EU','deb Delta 139','',45221,'Reentered','1990-12-14 00:00:00','1990-12-14 00:00:00',89.46,240,255,98.58,'Sun-Synch' from dual union all
select 12824,'012824','1978-026EV','deb Delta 139','',45221,'Reentered','2003-02-02 00:00:00','1993-08-07 00:00:00',100.93,737,868,99.02,'Sun-Synch' from dual union all
select 12825,'012825','1978-026EW','deb Delta 139','',45221,'In Earth orbit','','1993-08-01 00:00:00',103.54,879,972,99.03,'Sun-Synch' from dual union all
select 12826,'012826','1978-026EX','deb Delta 139','',45221,'Reentered','2011-12-30 00:00:00','1993-08-06 00:00:00',101.25,729,907,99,'Sun-Synch' from dual union all
select 12827,'012827','1981-088F','Blok-L','Blok-ML No. 76745-512',52611,'In Earth orbit','','1981-10-11 00:00:00',262.43,609,13791,62.83,'Medium' from dual union all
select 12828,'012828','1981-089A','Kosmos-1306','US-PM',52612,'Exploded','1982-07-12 00:00:00','1981-10-15 00:00:00',93.27,428,442,64.96,'Intermediate' from dual union all
select 12829,'012829','1981-089B','11S692','11S692',52612,'Reentered','1981-09-15 00:00:00','1981-09-14 00:00:00',89.86,111,423,65.04,'Intermediate' from dual union all
select 1283,'001283','1965-012FR','deb Kosmos-57','',11762,'Reentered','1965-03-23 00:00:00','1965-03-14 00:00:00',90.82,176,454,64.73,'Intermediate' from dual union all
select 12830,'012830','1981-090A','Kosmos-1307','Zenit-6',52613,'Landed','1981-09-29 00:00:00','1981-09-17 00:00:00',92.25,354,416,72.87,'Intermediate' from dual union all
select 12831,'012831','1981-090B','11S510','11S510',52613,'Reentered','1981-09-23 00:00:00','1981-09-17 00:00:00',90.07,191,365,72.87,'Intermediate' from dual union all
select 12832,'012832','1981-090C','Sensor cover','Sensor cover',52613,'Reentered','1981-09-16 00:00:00','1981-09-17 00:00:00',87.97,86,261,72.87,'Intermediate' from dual union all
select 12833,'012833','1979-058E','deb Kosmos-1109','',47542,'In Earth orbit','','1981-09-11 00:00:00',715.09,1496,38726,65.33,'GEO Transfer' from dual union all
select 12834,'012834','1980-057F','deb Kosmos-1191','',50267,'In Earth orbit','','1981-05-24 00:00:00',719.06,1180,39238,62.54,'Molniya' from dual union all
select 12835,'012835','1981-091A','Kosmos-1308','Parus',52614,'In Earth orbit','','1992-06-22 00:00:00',104.74,962,1003,82.92,'Intermediate' from dual union all
select 12836,'012836','1981-091B','S3M','S3M',52614,'In Earth orbit','','1991-03-25 00:00:00',104.67,962,996,82.92,'Intermediate' from dual union all
select 12837,'012837','1981-092A','Kosmos-1309','Orion',52615,'Landed','1981-10-01 00:00:00','1981-09-20 00:00:00',89.1,210,250,82.31,'Intermediate' from dual union all
select 12838,'012838','1981-092B','11S510','11S510',52615,'Reentered','1981-09-21 00:00:00','1981-09-20 00:00:00',88.58,190,219,82.31,'Intermediate' from dual union all
select 12839,'012839','1981-086E','Blenda','Sunshade',52609,'Reentered','1981-10-09 00:00:00','1981-09-18 00:00:00',92.25,358,412,70.4,'Intermediate' from dual union all
select 1284,'001284','1965-012FS','deb Kosmos-57','',11762,'Reentered','1965-03-19 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12840,'012840','1981-086F','KDU part','KDU part',52609,'Reentered','1981-10-25 00:00:00','1981-09-18 00:00:00',92.12,344,413,70.38,'Intermediate' from dual union all
select 12841,'012841','1981-086G','KDU','KDU',52609,'Reentered','1982-02-06 00:00:00','1981-10-04 00:00:00',92.09,351,403,70.4,'Intermediate' from dual union all
select 12842,'012842','1981-093A','Shi Jian 2B','shijian erhao B xing',52616,'Reentered','1981-09-26 00:00:00','1981-09-21 00:00:00',101.16,240,1388,59.47,'Intermediate' from dual union all
select 12843,'012843','1981-093B','Shi Jian 2A','shijian erhao A xing',52616,'Reentered','1982-10-06 00:00:00','1981-10-22 00:00:00',102.56,233,1527,59.47,'Intermediate' from dual union all
select 12844,'012844','1981-093C','SJ-2A adapter','',52616,'Reentered','1981-11-22 00:00:00','1981-10-19 00:00:00',99.19,229,1211,59.5,'Intermediate' from dual union all
select 12845,'012845','1981-093D','Shi Jian 2','shijian erhao',52616,'Reentered','1982-08-17 00:00:00','1981-10-23 00:00:00',102.28,231,1503,59.46,'Intermediate' from dual union all
select 12846,'012846','1981-093E','FB-1 Stage 2','FB-1 Stage 2',52616,'Reentered','1983-04-10 00:00:00','1981-10-22 00:00:00',102.7,234,1539,59.47,'Intermediate' from dual union all
select 12847,'012847','1981-093F','deb SJ-2','',52616,'Reentered','1981-11-25 00:00:00','1981-10-19 00:00:00',99.38,230,1228,59.4,'Intermediate' from dual union all
select 12848,'012848','1981-094A','Aureole 3','AUOS-Z 401 M-A-IK',52617,'In Earth orbit','','1991-03-28 00:00:00',106.83,398,1762,82.5,'Intermediate' from dual union all
select 12849,'012849','1981-094B','S5M','S5M',52617,'In Earth orbit','','1990-02-19 00:00:00',108.64,403,1925,82.5,'Intermediate' from dual union all
select 1285,'001285','1965-022B','11A57I','11A57I',11772,'Reentered','1965-03-27 00:00:00','1965-03-20 00:00:00',90.56,163,441,64.77,'Intermediate' from dual union all
select 12850,'012850','1981-069F','Blok DM','11S86 No. 22L',52592,'In Earth orbit','','1993-05-30 00:00:00',1473.88,36433,36613,10.26,'Inclined Drift' from dual union all
select 12851,'012851','1981-061F','Blok DM','11S86 No. 20L',52584,'In Earth orbit','','1993-06-01 00:00:00',1425.63,35565,35598,9.98,'Inclined Drift' from dual union all
select 12852,'012852','1981-095A','Kosmos-1310','Vektor',52618,'Reentered','1989-04-03 00:00:00','1988-06-11 00:00:00',92.49,380,414,65.82,'Intermediate' from dual union all
select 12853,'012853','1981-095B','S3M','S3M',52618,'Reentered','1983-09-17 00:00:00','1981-10-23 00:00:00',94.27,459,507,65.82,'Intermediate' from dual union all
select 12854,'012854','1970-103G','deb SOZ','Kosmos-382 part',21580,'In Earth orbit','','1971-02-27 00:00:00',160.36,2913,3886,50.74,'Medium' from dual union all
select 12855,'012855','1981-096A','SBS 2','SBS 2',52619,'In Earth orbit','','1981-10-24 00:00:00',1424.35,35492,35621,.24,'Drift GEO' from dual union all
select 12856,'012856','1966-024F','NNS O-8 insulation','',13431,'Reentered','1988-07-20 00:00:00','1981-07-09 00:00:00',105.12,887,1114,89.89,'Polar' from dual union all
select 12857,'012857','1977-012H','deb Tansei 3','',42140,'In Earth orbit','','1981-09-28 00:00:00',134.31,816,3803,66.3,'Medium' from dual union all
select 12858,'012858','1977-106C','NNS O-11 despin weight','',42234,'In Earth orbit','','1981-09-28 00:00:00',107.02,1076,1101,89.47,'Polar' from dual union all
select 12859,'012859','1977-112H','SS 2 cover','SSU cover',42240,'In Earth orbit','','1997-11-22 00:00:00',106.93,653,1516,63.41,'Intermediate' from dual union all
select 1286,'001286','1965-022C','Volga airlock','',11772,'Reentered','1965-03-25 00:00:00','1965-03-20 00:00:00',90.53,168,433,64.72,'Intermediate' from dual union all
select 12860,'012860','1977-121AN','deb Kosmos-970','',42249,'In Earth orbit','','1993-07-30 00:00:00',105.86,931,1139,65.81,'Intermediate' from dual union all
select 12861,'012861','1977-121AP','deb Kosmos-970','',42249,'In Earth orbit','','1981-09-28 00:00:00',107.99,1060,1207,65.78,'Intermediate' from dual union all
select 12862,'012862','1977-121AQ','deb Kosmos-970','',42249,'In Earth orbit','','1981-09-28 00:00:00',104.63,832,1122,66.23,'Intermediate' from dual union all
select 12863,'012863','1977-121AR','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-04 00:00:00',106.13,995,1100,65.78,'Intermediate' from dual union all
select 12864,'012864','1977-121AS','deb Kosmos-970','',42249,'In Earth orbit','','1981-09-28 00:00:00',105.62,927,1120,65.85,'Intermediate' from dual union all
select 12865,'012865','1977-121AT','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-05 00:00:00',105.88,978,1093,65.77,'Intermediate' from dual union all
select 12866,'012866','1977-121AU','deb Kosmos-970','',42249,'In Earth orbit','','1981-09-29 00:00:00',106.31,971,1141,65.82,'Intermediate' from dual union all
select 12867,'012867','1977-121AV','deb Kosmos-970','',42249,'In Earth orbit','','1981-09-28 00:00:00',108.2,1085,1202,65.96,'Intermediate' from dual union all
select 12868,'012868','1977-121AW','deb Kosmos-970','',42249,'In Earth orbit','','1993-07-30 00:00:00',105.94,989,1088,65.77,'Intermediate' from dual union all
select 12869,'012869','1977-121AX','deb Kosmos-970','',42249,'In Earth orbit','','1993-07-31 00:00:00',105.57,945,1098,65.79,'Intermediate' from dual union all
select 1287,'001287','1965-022D','Volga part','Volga part',11772,'Reentered','1965-03-19 00:00:00','1965-03-18 00:00:00',90.28,156,420,64.72,'Intermediate' from dual union all
select 12870,'012870','1977-121AY','deb Kosmos-970','',42249,'In Earth orbit','','1993-07-31 00:00:00',104.83,890,1083,65.73,'Intermediate' from dual union all
select 12871,'012871','1981-097A','Kosmos-1311','Romb',52620,'Reentered','1983-08-28 00:00:00','1981-10-28 00:00:00',94.29,461,508,82.95,'Intermediate' from dual union all
select 12872,'012872','1981-097B','S3M','S3M',52620,'Reentered','1983-02-16 00:00:00','1981-10-28 00:00:00',94.12,450,502,82.95,'Intermediate' from dual union all
select 12873,'012873','1981-090D','Blenda','Sunshade',52613,'Reentered','1982-01-04 00:00:00','1981-10-14 00:00:00',89.11,216,245,72.87,'Intermediate' from dual union all
select 12874,'012874','1981-096B','PAM-D','Star 48  S/N 10007-8',52619,'Reentered','1982-04-15 00:00:00','1981-11-26 00:00:00',606.46,202,34509,27.55,'GEO Transfer' from dual union all
select 12875,'012875','1981-090E','KDU part','KDU part',52613,'Reentered','1981-09-30 00:00:00','1981-09-29 00:00:00',91.74,314,406,72.87,'Intermediate' from dual union all
select 12876,'012876','1981-090F','KDU part','KDU part',52613,'Reentered','1981-11-04 00:00:00','1981-09-29 00:00:00',92.03,331,418,72.86,'Intermediate' from dual union all
select 12877,'012877','1981-090G','KDU','KDU',52613,'Reentered','1982-02-04 00:00:00','1981-10-22 00:00:00',91.91,343,394,72.86,'Intermediate' from dual union all
select 12878,'012878','1981-090H','KDU part','',52613,'Reentered','1981-10-23 00:00:00','1981-09-30 00:00:00',94.11,387,563,72.88,'Intermediate' from dual;
insert into SATELLITE
select 12879,'012879','1981-098A','Kosmos-1312','Musson No. 12',52621,'In Earth orbit','','1992-05-20 00:00:00',115.91,1492,1501,82.6,'Intermediate' from dual union all
select 1288,'001288','1965-021B','P35 despin weight','',11771,'Reentered','1965-10-05 00:00:00','1965-03-18 00:00:00',99.49,733,735,99.1,'Sun-Synch' from dual union all
select 12880,'012880','1981-098B','S5M','S5M',52621,'In Earth orbit','','1982-01-04 00:00:00',115.86,1491,1497,82.59,'Intermediate' from dual union all
select 12881,'012881','1981-099A','Kosmos-1313','Zenit-6',52622,'Landed','1981-10-15 00:00:00','1981-10-03 00:00:00',89.58,230,278,70.34,'Intermediate' from dual union all
select 12882,'012882','1981-099B','11S510','11S510',52622,'Reentered','1981-10-05 00:00:00','1981-10-03 00:00:00',88.94,191,254,70.36,'Intermediate' from dual union all
select 12883,'012883','1981-099C','Sensor cover','Sensor cover',52622,'Reentered','1981-10-02 00:00:00','1981-10-01 00:00:00',88.46,172,224,70.37,'Intermediate' from dual union all
select 12884,'012884','1981-093G','deb FB-1','',52616,'Reentered','1982-05-22 00:00:00','1981-10-25 00:00:00',103.35,325,1508,59.55,'Intermediate' from dual union all
select 12885,'012885','1981-093H','deb FB-1','',52616,'Reentered','1982-05-20 00:00:00','1981-10-19 00:00:00',103.56,319,1535,59.48,'Intermediate' from dual union all
select 12886,'012886','1981-053DZ','deb Kosmos-1275','',52576,'Reentered','2001-10-13 00:00:00','1993-08-06 00:00:00',99.84,698,804,82.9,'Intermediate' from dual union all
select 12887,'012887','1981-100A','Solar Mesosphere Explorer','Solar Mesosphere Explore',52623,'Reentered','1991-03-05 00:00:00','1989-04-25 00:00:00',94.27,481,486,97.69,'Sun-Synch' from dual union all
select 12888,'012888','1981-100B','UoSAT-OSCAR-9','UoSAT 1',52623,'Reentered','1989-10-13 00:00:00','1984-10-25 00:00:00',94.31,483,487,97.61,'Sun-Synch' from dual union all
select 12889,'012889','1981-100C','Delta 157','Delta SSPU DSV-3P-4',52623,'In Earth orbit','','1990-02-19 00:00:00',118.98,551,2720,99.91,'Sun-Synch' from dual union all
select 1289,'001289','1965-021C','MG-18','MG-18',11771,'Reentered','1979-09-06 00:00:00','1965-04-24 00:00:00',97.55,526,757,99.02,'Sun-Synch' from dual union all
select 12890,'012890','1981-093J','deb FB-1','',52616,'Reentered','1981-10-20 00:00:00','1981-10-07 00:00:00',100.26,191,1350,59.34,'Intermediate' from dual union all
select 12891,'012891','1981-100D','SME IR radiator cover','',52623,'Reentered','1981-11-14 00:00:00','1981-10-13 00:00:00',94.98,513,523,97.47,'Sun-Synch' from dual union all
select 12892,'012892','1981-031E','deb Kosmos-1261','',52554,'In Earth orbit','','1981-05-01 00:00:00',719.36,680,39753,62.77,'Molniya' from dual union all
select 12893,'012893','1981-031F','deb Kosmos-1261','',52554,'In Earth orbit','','1984-04-03 00:00:00',715.41,2164,38074,63.25,'Molniya' from dual union all
select 12894,'012894','1981-031G','deb Kosmos-1261','',52554,'In Earth orbit','','1986-06-09 00:00:00',718.45,3094,37294,65.2,'GEO Transfer' from dual union all
select 12895,'012895','1981-101A','Kosmos-1314','Fram',52624,'Landed','1981-10-22 00:00:00','1981-10-11 00:00:00',88.9,210,231,82.34,'Intermediate' from dual union all
select 12896,'012896','1981-101B','11S510','11S510',52624,'Reentered','1981-10-11 00:00:00','1981-10-10 00:00:00',88.68,204,215,82.35,'Intermediate' from dual union all
select 12897,'012897','1981-102A','Raduga','Gran'' No. 20L',52625,'In Earth orbit','','1981-11-08 00:00:00',1435.96,35772,35796,.37,'Stationary' from dual union all
select 12898,'012898','1981-102B','8S812','8S812',52625,'Reentered','1981-10-11 00:00:00','1981-10-10 00:00:00',88.24,184,191,51.62,'Intermediate' from dual union all
select 12899,'012899','1981-102C','Perekhodnik','Sredniy perekhodnik',52625,'Reentered','1981-10-10 00:00:00','1981-10-09 00:00:00',88.1,175,186,51.61,'Intermediate' from dual union all
select 129,'000129','61 Omicron 14','deb r/b Ablestar','',7273,'Reentered','1981-08-31 00:00:00','1961-09-04 00:00:00',100.87,620,979,66.75,'Intermediate' from dual union all
select 1290,'001290','1965-021D','P35 despin weight','',11771,'Reentered','1965-10-01 00:00:00','1965-03-18 00:00:00',99.81,724,775,99,'Sun-Synch' from dual union all
select 12900,'012900','1981-101C','Sensor cover','Sensor cover',52624,'Reentered','1981-11-09 00:00:00','1981-10-13 00:00:00',88.95,215,231,82.32,'Intermediate' from dual union all
select 12901,'012901','1981-102D','SOZ','SOZ',52625,'Reentered','1982-07-01 00:00:00','1981-11-09 00:00:00',635.24,301,35900,47.33,'GEO Transfer' from dual union all
select 12902,'012902','1981-102E','SOZ','SOZ',52625,'Reentered','1982-05-13 00:00:00','1981-11-10 00:00:00',633.57,246,35868,47.27,'GEO Transfer' from dual union all
select 12903,'012903','1981-103A','Kosmos-1315','Tselina-D No. 426',52626,'Reentered','2015-08-31 00:00:00','1991-03-27 00:00:00',96.83,591,623,81.18,'Intermediate' from dual union all
select 12904,'012904','1981-103B','8A92ME','8A92ME',52626,'In Earth orbit','','1991-03-28 00:00:00',97.12,589,653,81.19,'Intermediate' from dual union all
select 12905,'012905','1981-104A','Kosmos-1316','Zenit-6',52627,'Landed','1981-10-29 00:00:00','1981-10-17 00:00:00',89.57,231,275,70.33,'Intermediate' from dual union all
select 12906,'012906','1977-068F','deb Kosmos-931','',42196,'In Earth orbit','','1990-11-21 00:00:00',717.62,5766,34581,67.72,'GEO Transfer' from dual union all
select 12907,'012907','1978-083E','deb Kosmos-1030','',45278,'In Earth orbit','','1981-10-15 00:00:00',719.4,1816,38619,63.75,'Molniya' from dual union all
select 12908,'012908','1978-016C','Centaur AC-44','Centaur D-1AR AC-44',45211,'Reentered','2005-12-05 00:00:00','1978-02-26 00:00:00',602.4,193,34306,26.45,'GEO Transfer' from dual union all
select 12909,'012909','1979-058F','deb Kosmos-1109','',47542,'Reentered','2015-10-03 00:00:00','1980-03-01 00:00:00',719.55,1019,39423,63.44,'Molniya' from dual union all
select 1291,'001291','1965-016D','SR 7B','POPPY 4 SV1',11766,'In Earth orbit','','1965-04-22 00:00:00',103.48,909,938,70.09,'Intermediate' from dual union all
select 12910,'012910','1981-099D','Blenda','Sunshade',52622,'Reentered','1981-10-16 00:00:00','1981-10-15 00:00:00',89.15,212,253,70.35,'Intermediate' from dual union all
select 12911,'012911','1981-099E','KDU part','KDU part',52622,'Reentered','1981-10-16 00:00:00','1981-10-15 00:00:00',89.19,212,257,70.34,'Intermediate' from dual union all
select 12912,'012912','1981-099F','KDU','KDU',52622,'Reentered','1981-10-26 00:00:00','1981-10-15 00:00:00',89.38,223,265,70.35,'Intermediate' from dual union all
select 12913,'012913','1981-104B','11S510','11S510',52627,'Reentered','1981-10-23 00:00:00','1981-10-17 00:00:00',90.1,203,356,70.34,'Intermediate' from dual union all
select 12914,'012914','1981-053EA','deb Kosmos-1275','',52576,'Reentered','1981-10-29 00:00:00','1981-10-21 00:00:00',100.86,753,846,83.09,'Intermediate' from dual union all
select 12915,'012915','1981-105A','Molniya-3','Molniya-3 No. 31',52628,'Reentered','1997-01-09 00:00:00','1981-11-16 00:00:00',717.78,610,39745,62.9,'Molniya' from dual union all
select 12916,'012916','1981-105B','BOZ','BOZ',52628,'Reentered','1981-11-04 00:00:00','1981-10-24 00:00:00',91.78,186,538,62.86,'Intermediate' from dual union all
select 12917,'012917','1981-105C','11S510','11S510',52628,'Reentered','1981-11-06 00:00:00','1981-10-24 00:00:00',91.59,206,500,62.81,'Intermediate' from dual union all
select 12918,'012918','1981-105D','deb Blok-I','',52628,'Reentered','1981-11-08 00:00:00','1981-10-21 00:00:00',93.13,228,628,62.8,'Intermediate' from dual union all
select 12919,'012919','1978-083F','deb Kosmos-1030','',45278,'In Earth orbit','','1983-07-26 00:00:00',719.55,3020,37422,63.96,'Molniya' from dual union all
select 1292,'001292','1965-016C','GGSE 3','POPPY 4 SV4',11766,'In Earth orbit','','1965-04-25 00:00:00',103.48,906,941,70.09,'Intermediate' from dual union all
select 12920,'012920','1981-105E','Blok-L','Blok-L',52628,'Reentered','1997-12-15 00:00:00','1981-12-27 00:00:00',733.28,633,40483,63.01,'Molniya' from dual union all
select 12921,'012921','1981-101D','KDU','KDU',52624,'Reentered','1981-10-25 00:00:00','1981-10-22 00:00:00',88.71,203,219,82.33,'Intermediate' from dual union all
select 12922,'012922','1981-101E','Blenda','Sunshade',52624,'Reentered','1981-10-25 00:00:00','1981-10-23 00:00:00',87.86,163,175,82.34,'Intermediate' from dual union all
select 12923,'012923','1981-101F','KDU part','KDU part',52624,'Reentered','1981-10-25 00:00:00','1981-10-23 00:00:00',89.1,189,270,82.34,'Intermediate' from dual union all
select 12924,'012924','1981-104C','KDU','KDU',52627,'Reentered','1981-11-08 00:00:00','1981-10-29 00:00:00',89.45,225,270,70.33,'Intermediate' from dual union all
select 12925,'012925','1981-104D','Blenda','Sunshade',52627,'Reentered','1981-10-31 00:00:00','1981-10-29 00:00:00',89.35,220,265,70.34,'Intermediate' from dual union all
select 12926,'012926','1981-104E','KDU part','KDU part',52627,'Reentered','1981-10-31 00:00:00','1981-10-29 00:00:00',89.24,221,253,70.33,'Intermediate' from dual union all
select 12927,'012927','1981-106A','Venera-13','4V-1M No. 760',52629,'Deep Space','1981-10-30 00:00:00','',null,null,null,null,'' from dual union all
select 12928,'012928','1981-106B','SOZ','SOZ',52629,'Reentered','1981-10-31 00:00:00','1981-10-30 00:00:00',87.9,139,203,51.56,'Intermediate' from dual union all
select 12929,'012929','1981-106C','SOZ','SOZ',52629,'Reentered','1981-10-31 00:00:00','1981-10-30 00:00:00',87.9,154,188,51.52,'Intermediate' from dual union all
select 1293,'001293','1965-016F','OSCAR 3','OSCAR 3',11766,'In Earth orbit','','1965-04-09 00:00:00',103.46,904,941,70.08,'Intermediate' from dual union all
select 12930,'012930','1981-107A','OPS 4029','VORTEX 10',52630,'In Earth orbit','','1981-12-01 00:00:00',1447.01,30500,41500,10,'Near-synch' from dual union all
select 12931,'012931','1981-107B','Titan 23C-21 Stage 2','LR91AJ11 23C-21',52630,'Reentered','1981-11-02 00:00:00','1981-10-31 00:00:00',89.66,134,382,29.3,'Intermediate' from dual union all
select 12932,'012932','1981-107C','Transtage 39','Transtage 39',52630,'In Earth orbit','','1981-12-01 00:00:00',1447.01,30500,41500,10,'Near-synch' from dual union all
select 12933,'012933','1981-108A','Kosmos-1317','Oko',52631,'In Earth orbit','','1981-12-02 00:00:00',717.29,608,39723,62.91,'Molniya' from dual union all
select 12934,'012934','1981-108B','11S510','11S510',52631,'Reentered','1981-11-23 00:00:00','1981-11-07 00:00:00',91.76,207,515,62.84,'Intermediate' from dual union all
select 12935,'012935','1981-108C','BOZ','BOZ',52631,'Reentered','1981-11-14 00:00:00','1981-11-02 00:00:00',92.34,181,598,62.91,'Intermediate' from dual union all
select 12936,'012936','1981-109A','Kosmos-1318','Feniks No. 944',52632,'Landed','1981-12-04 00:00:00','1981-11-10 00:00:00',89.56,162,344,67.14,'Intermediate' from dual union all
select 12937,'012937','1981-109B','11S510','11S510',52632,'Reentered','1981-11-06 00:00:00','1981-11-05 00:00:00',88.95,159,286,67.14,'Intermediate' from dual union all
select 12938,'012938','1981-110A','Venera-14','4V-1M No. 761',52633,'Deep Space','1981-11-04 00:00:00','',null,null,null,null,'' from dual union all
select 12939,'012939','1981-110B','SOZ','SOZ',52633,'Reentered','1981-11-05 00:00:00','1981-11-04 00:00:00',87.83,153,182,51.56,'Intermediate' from dual union all
select 1294,'001294','1965-023A','Ranger 9','RA-9',11773,'Deep Space','1965-03-24 00:00:00','1965-03-21 00:00:00',40808.27,188,772332,28.5,'Cislunar' from dual union all
select 12940,'012940','1981-108D','Blok-L','Blok-L',52631,'In Earth orbit','','1981-11-30 00:00:00',723.32,605,40023,62.93,'Molniya' from dual union all
select 12941,'012941','1981-110C','SOZ','SOZ',52633,'Reentered','1981-11-05 00:00:00','1981-11-04 00:00:00',87.71,150,172,51.57,'Intermediate' from dual union all
select 12942,'012942','1977-121AZ','deb Kosmos-970','',42249,'In Earth orbit','','1993-07-31 00:00:00',107.98,1046,1221,65.83,'Intermediate' from dual union all
select 12943,'012943','1963-038L','deb Transit VBN-1','[UNID085] RCS 0.04',9269,'Lost','','1981-12-27 00:00:00',104.91,889,1092,89.95,'Polar' from dual union all
select 12944,'012944','1981-044C','Nova 1 despin weight','',52567,'Reentered','1981-12-08 00:00:00','1981-10-27 00:00:00',94.1,315,635,89.9,'Polar' from dual union all
select 12945,'012945','1981-070G','deb Delta','',52593,'Reentered','1982-01-25 00:00:00','1981-10-27 00:00:00',95.33,285,785,89.96,'Polar' from dual union all
select 12946,'012946','1981-070H','deb Delta','',52593,'Reentered','1982-02-16 00:00:00','1981-10-27 00:00:00',95.71,291,815,89.97,'Polar' from dual union all
select 12947,'012947','1981-053EB','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',107.82,961,1291,83.09,'Intermediate' from dual union all
select 12948,'012948','1981-053EC','deb Kosmos-1275','',52576,'Reentered','2001-03-08 00:00:00','1993-08-06 00:00:00',100.32,744,803,82.87,'Intermediate' from dual union all
select 12949,'012949','1981-053ED','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',104.12,928,979,82.85,'Intermediate' from dual union all
select 1295,'001295','1965-012FT','deb Kosmos-57','',11762,'Reentered','1965-03-20 00:00:00','1965-03-13 00:00:00',89.86,200,335,65.33,'Intermediate' from dual union all
select 12950,'012950','1981-053EE','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',102.77,837,943,83.02,'Intermediate' from dual union all
select 12951,'012951','1981-053EF','deb Kosmos-1275','',52576,'Reentered','1992-06-19 00:00:00','1992-06-19 00:00:00',89.31,228,253,83.08,'Intermediate' from dual union all
select 12952,'012952','1981-053EG','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-03 00:00:00',103.99,938,957,82.96,'Intermediate' from dual union all
select 12953,'012953','1981-111A','Columbia (STS-2)','OV-102',52634,'Landed','1981-11-14 00:00:00','1981-11-13 00:00:00',89.68,254,264,38.03,'Intermediate' from dual union all
select 12954,'012954','1981-112A','Kosmos-1319','Zenit-6',52635,'Landed','1981-11-27 00:00:00','1981-11-15 00:00:00',92.17,347,415,70.36,'Intermediate' from dual union all
select 12955,'012955','1981-112B','11S510','11S510',52635,'Reentered','1981-11-22 00:00:00','1981-11-15 00:00:00',89.95,200,344,70.36,'Intermediate' from dual union all
select 12956,'012956','1981-112C','Sensor cover','Sensor cover',52635,'Reentered','1981-11-16 00:00:00','1981-11-13 00:00:00',89.8,204,326,70.56,'Intermediate' from dual union all
select 12957,'012957','1981-112D','Sensor cover','Sensor cover',52635,'Reentered','1981-11-13 00:00:00','1981-11-13 00:00:00',90.25,203,371,70.34,'Intermediate' from dual union all
select 12958,'012958','1981-093K','deb FB-1','',52616,'Reentered','1981-11-16 00:00:00','1981-11-14 00:00:00',92.1,148,607,59.63,'Intermediate' from dual union all
select 12959,'012959','1981-113A','Molniya-1','Molniya-1',52636,'Reentered','1993-11-02 00:00:00','1981-12-17 00:00:00',717.82,487,39870,62.78,'Molniya' from dual union all
select 1296,'001296','1965-012FU','deb Kosmos-57','',11762,'Reentered','1965-03-15 00:00:00','1965-03-10 00:00:00',89.43,191,302,65.01,'Intermediate' from dual union all
select 12960,'012960','1981-113B','11S510','11S510',52636,'Reentered','1981-11-27 00:00:00','1981-11-19 00:00:00',90.44,204,389,62.79,'Intermediate' from dual union all
select 12961,'012961','1981-113C','BOZ','BOZ',52636,'Reentered','1981-11-29 00:00:00','1981-11-19 00:00:00',90.84,193,439,62.83,'Intermediate' from dual union all
select 12962,'012962','1981-053EH','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',104.23,932,984,82.97,'Intermediate' from dual union all
select 12963,'012963','1981-053EJ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',104.69,961,999,82.95,'Intermediate' from dual union all
select 12964,'012964','1981-053EK','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',104.35,934,994,82.94,'Intermediate' from dual union all
select 12965,'012965','1981-053EL','deb Kosmos-1275','',52576,'Reentered','1991-02-01 00:00:00','1991-02-01 00:00:00',89.6,249,260,83.14,'Intermediate' from dual union all
select 12966,'012966','1981-053EM','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',104.07,922,980,82.95,'Intermediate' from dual union all
select 12967,'012967','1981-114A','Satcom 3R','RCA-D',52637,'In Earth orbit','','1981-12-24 00:00:00',1436.04,35778,35793,.14,'Stationary' from dual union all
select 12968,'012968','1981-115A','Bhaskara 2','SEO B',52638,'Reentered','1991-11-30 00:00:00','1990-06-28 00:00:00',93.44,438,447,50.67,'Intermediate' from dual;
insert into SATELLITE
select 12969,'012969','1981-115B','S3M','S3M',52638,'Reentered','1989-10-08 00:00:00','1981-12-20 00:00:00',95.07,493,551,50.64,'Intermediate' from dual union all
select 1297,'001297','1965-012FV','deb Kosmos-57','',11762,'Reentered','1965-03-22 00:00:00','1965-03-15 00:00:00',89.71,162,359,65.06,'Intermediate' from dual union all
select 12970,'012970','1981-112E','Blenda','Sunshade',52635,'Reentered','1981-12-13 00:00:00','1981-11-27 00:00:00',92.16,347,414,70.37,'Intermediate' from dual union all
select 12971,'012971','1981-112F','KDU','KDU',52635,'Reentered','1982-04-15 00:00:00','1981-12-15 00:00:00',91.94,340,399,70.36,'Intermediate' from dual union all
select 12972,'012972','1981-112G','KDU part','KDU part',52635,'Reentered','1982-01-02 00:00:00','1981-11-27 00:00:00',91.99,333,411,70.36,'Intermediate' from dual union all
select 12973,'012973','1981-112H','KDU part','KDU part',52635,'Reentered','1981-12-27 00:00:00','1981-11-28 00:00:00',94.19,379,579,70.37,'Intermediate' from dual union all
select 12974,'012974','1981-112J','KDU part','KDU part',52635,'Reentered','1981-12-08 00:00:00','1981-11-28 00:00:00',94.86,376,648,70.33,'Intermediate' from dual union all
select 12975,'012975','1981-116A','Kosmos-1320','Strela-1M',52639,'In Earth orbit','','1992-05-06 00:00:00',117.26,1481,1635,73.98,'Intermediate' from dual union all
select 12976,'012976','1981-116B','Kosmos-1321','Strela-1M',52639,'In Earth orbit','','1992-05-20 00:00:00',117.22,1481,1631,73.98,'Intermediate' from dual union all
select 12977,'012977','1981-116C','Kosmos-1322','Strela-1M',52639,'In Earth orbit','','1992-05-05 00:00:00',117.2,1481,1629,73.98,'Intermediate' from dual union all
select 12978,'012978','1981-116D','Kosmos-1323','Strela-1M',52639,'In Earth orbit','','1992-05-06 00:00:00',117.14,1481,1623,73.98,'Intermediate' from dual union all
select 12979,'012979','1981-116E','Kosmos-1324','Strela-1M',52639,'In Earth orbit','','1992-05-06 00:00:00',117.1,1481,1620,73.98,'Intermediate' from dual union all
select 1298,'001298','1965-023B','Agena B 6007','Agena B 6007',11773,'Deep Space','1965-03-24 00:00:00','1965-03-21 00:00:00',40808.27,188,772332,28.5,'Cislunar' from dual union all
select 12980,'012980','1981-116F','Kosmos-1325','Strela-1M',52639,'In Earth orbit','','1992-05-18 00:00:00',117.05,1481,1615,73.98,'Intermediate' from dual union all
select 12981,'012981','1981-116G','Kosmos-1326','Strela-1M',52639,'In Earth orbit','','1992-05-20 00:00:00',116.98,1480,1611,73.98,'Intermediate' from dual union all
select 12982,'012982','1981-116H','Kosmos-1327','Strela-1M',52639,'In Earth orbit','','1992-05-06 00:00:00',116.91,1481,1602,73.98,'Intermediate' from dual union all
select 12983,'012983','1981-116J','S3M','S3M',52639,'In Earth orbit','','1991-02-24 00:00:00',117.56,1480,1663,73.98,'Intermediate' from dual union all
select 12984,'012984','1981-016F','deb Kosmos-1247','',52539,'In Earth orbit','','1981-11-21 00:00:00',710.45,1007,38986,63.08,'Molniya' from dual union all
select 12985,'012985','1981-016G','deb Kosmos-1247','',52539,'Reentered','2009-07-07 00:00:00','1981-12-31 00:00:00',709.4,998,38943,62.75,'Molniya' from dual union all
select 12986,'012986','1981-113D','Blok-L','Blok-ML No. 77346-343',52636,'Reentered','1997-05-21 00:00:00','1981-12-28 00:00:00',699.03,495,38931,62.8,'Molniya' from dual union all
select 12987,'012987','1981-117A','Kosmos-1328','Tselina-D',52640,'In Earth orbit','','1991-03-28 00:00:00',97.16,609,637,82.5,'Intermediate' from dual union all
select 12988,'012988','1981-117B','S5M','S5M',52640,'In Earth orbit','','1991-03-28 00:00:00',97.44,621,652,82.52,'Intermediate' from dual union all
select 12989,'012989','1981-118A','Kosmos-1329','Zenit-6',52641,'Landed','1981-12-18 00:00:00','1981-12-06 00:00:00',89.4,232,258,65.02,'Intermediate' from dual union all
select 1299,'001299','1965-012FW','deb Kosmos-57','',11762,'Reentered','1965-03-20 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 12990,'012990','1981-118B','11S510','11S510',52641,'Reentered','1981-12-08 00:00:00','1981-12-06 00:00:00',89,213,238,65.03,'Intermediate' from dual union all
select 12991,'012991','1981-118C','Sensor cover','Sensor cover',52641,'Reentered','1981-12-10 00:00:00','1981-12-05 00:00:00',88.78,205,224,64.98,'Intermediate' from dual union all
select 12992,'012992','1981-016H','deb Kosmos-1247','',52539,'In Earth orbit','','1981-12-05 00:00:00',708.46,1141,38753,62.25,'Molniya' from dual union all
select 12993,'012993','1981-071E','deb Kosmos-1285','',52594,'In Earth orbit','','1981-11-27 00:00:00',727.78,731,40116,63.03,'Molniya' from dual union all
select 12994,'012994','1981-119A','INTELSAT 503','INTELSAT V F3',52642,'In Earth orbit','','1982-01-16 00:00:00',1436.06,35775,35797,.09,'Stationary' from dual union all
select 12995,'012995','1979-058G','deb Kosmos-1109','',47542,'Reentered','2012-05-20 00:00:00','1980-09-13 00:00:00',726.35,1267,39510,64.05,'GEO Transfer' from dual union all
select 12996,'012996','1977-092J','Ekran 2 debris','',42220,'In Earth orbit','','2003-10-01 00:00:00',1439.2,35770,35925,14.88,'Inclined Drift' from dual union all
select 12997,'012997','1981-120A','Radio-3','RS-3',52643,'In Earth orbit','','1984-08-12 00:00:00',118.44,1565,1656,82.96,'Intermediate' from dual union all
select 12998,'012998','1981-120B','Radio-4','RS-4',52643,'In Earth orbit','','1984-11-19 00:00:00',119.7,1652,1683,82.96,'Intermediate' from dual union all
select 12999,'012999','1981-120C','Radio-5','RS-5',52643,'In Earth orbit','','1984-11-05 00:00:00',119.47,1649,1665,82.95,'Intermediate' from dual union all
select 13,'000013','59 Beta','Discoverer 1','CORONA Test Vehicle 2',5526,'Reentered Attached','1959-03-05 00:00:00','1959-02-28 00:00:00',95.97,163,967,89.7,'Polar' from dual union all
select 130,'000130','61 Omicron 15','deb r/b Ablestar','',7273,'In Earth orbit','','1961-12-19 00:00:00',103.39,850,988,66.82,'Intermediate' from dual union all
select 1300,'001300','1965-012FX','deb Kosmos-57','',11762,'Reentered','1965-03-23 00:00:00','1965-02-22 00:00:00',89.6,160,350,64.9,'Intermediate' from dual union all
select 13000,'013000','1981-120D','Radio-6','RS-6',52643,'In Earth orbit','','1991-05-08 00:00:00',119.32,1637,1664,82.97,'Intermediate' from dual union all
select 13001,'013001','1981-120E','Radio-7','RS-7',52643,'In Earth orbit','','1985-04-17 00:00:00',119.12,1623,1660,82.96,'Intermediate' from dual union all
select 13002,'013002','1981-120F','Radio-8','RS-8',52643,'In Earth orbit','','1991-03-19 00:00:00',118.63,1580,1659,82.96,'Intermediate' from dual union all
select 13003,'013003','1981-120G','S3M','S3M',52643,'In Earth orbit','','1990-06-21 00:00:00',120.84,1654,1782,82.95,'Medium' from dual union all
select 13004,'013004','1981-118D','KDU','KDU',52641,'Reentered','1982-01-05 00:00:00','1981-12-18 00:00:00',89.64,229,285,65.02,'Intermediate' from dual union all
select 13005,'013005','1981-118E','Blenda','Sunshade',52641,'Reentered','1981-12-20 00:00:00','1981-12-18 00:00:00',89.55,226,279,65.03,'Intermediate' from dual union all
select 13006,'013006','1981-118F','KDU part','KDU part',52641,'Reentered','1981-12-22 00:00:00','1981-12-19 00:00:00',89.36,221,265,65.02,'Intermediate' from dual union all
select 13007,'013007','1981-119B','Centaur AC-55','Centaur D-1AR AC-55',52642,'In Earth orbit','','1982-11-08 00:00:00',506.53,203,29139,23.55,'Highly Ellip' from dual union all
select 13008,'013008','1981-121A','Kosmos-1330','Feniks No. 952',52644,'Landed','1982-01-19 00:00:00','1981-12-26 00:00:00',89.58,166,341,70.36,'Intermediate' from dual union all
select 13009,'013009','1981-121B','11S510','11S510',52644,'Reentered','1981-12-23 00:00:00','1981-12-21 00:00:00',89.33,160,323,70.37,'Intermediate' from dual union all
select 1301,'001301','1965-024A','Gemini III','Gemini SC3',11774,'Landed','1965-03-23 00:00:00','1965-03-23 00:00:00',88.35,161,225,32.62,'Intermediate' from dual union all
select 13010,'013010','1981-122A','Marecs 1','Marecs A',52645,'In Earth orbit','','1982-02-10 00:00:00',1436.01,35774,35796,2.17,'Stationary' from dual union all
select 13011,'013011','1981-122B','CAT/Thesee','CAT 04/VID',52645,'In Earth orbit','','1982-01-21 00:00:00',630.01,216,35716,10.38,'GEO Transfer' from dual union all
select 13012,'013012','1981-123A','Molniya-1','Molniya-1 11F658 No. 55',52646,'In Earth orbit','','1982-01-22 00:00:00',717.72,530,39822,62.85,'Molniya' from dual union all
select 13013,'013013','1981-123B','11S510','11S510',52646,'Reentered','1982-01-16 00:00:00','1981-12-30 00:00:00',90.97,225,420,62.98,'Intermediate' from dual union all
select 13014,'013014','1981-123C','BOZ','BOZ',52646,'Reentered','1982-01-10 00:00:00','1981-12-30 00:00:00',90.8,197,431,62.99,'Intermediate' from dual union all
select 13015,'013015','1981-053EN','deb Kosmos-1275','',52576,'Reentered','1991-03-24 00:00:00','1991-03-25 00:00:00',89.11,218,242,82.81,'Intermediate' from dual union all
select 13016,'013016','1981-123D','Blok-L','Blok-ML No. 77333-458',52646,'In Earth orbit','','1982-01-27 00:00:00',695.26,510,38728,62.96,'Molniya' from dual union all
select 13017,'013017','1981-053EP','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',104.09,922,982,82.94,'Intermediate' from dual union all
select 13018,'013018','1981-053EQ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',104.7,936,1025,82.68,'Intermediate' from dual union all
select 13019,'013019','1981-053ER','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',104.79,927,1043,82.95,'Intermediate' from dual union all
select 1302,'001302','1965-024B','Titan GT-3 Stage 2','LR91AJ5 GT-3 (65-12558)',11774,'Reentered','1965-03-24 00:00:00','1965-03-23 00:00:00',88.05,164,192,32.62,'Intermediate' from dual union all
select 13020,'013020','1981-053ES','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-03 00:00:00',105.22,914,1096,82.69,'Intermediate' from dual union all
select 13021,'013021','1981-053ET','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-03 00:00:00',104.07,929,973,83.16,'Intermediate' from dual union all
select 13022,'013022','1981-053EU','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',102.86,850,937,82.92,'Intermediate' from dual union all
select 13023,'013023','1981-053EV','deb Kosmos-1275','',52576,'Reentered','2002-01-09 00:00:00','1993-08-07 00:00:00',100.65,750,829,82.77,'Intermediate' from dual union all
select 13024,'013024','1981-053EW','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',104.1,910,994,82.94,'Intermediate' from dual union all
select 13025,'013025','1981-122C','Ariane H8 L04','Ariane H8 S/N L03a',52645,'Reentered','1988-11-21 00:00:00','1982-01-26 00:00:00',610.49,180,34741,10.31,'GEO Transfer' from dual union all
select 13026,'013026','1981-053EX','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',103.2,884,936,82.96,'Intermediate' from dual union all
select 13027,'013027','1982-001A','Kosmos-1331','Strela-2M',54310,'In Earth orbit','','1982-02-12 00:00:00',100.68,772,810,74.05,'Intermediate' from dual union all
select 13028,'013028','1982-001B','S3M','S3M',54310,'In Earth orbit','','1991-03-27 00:00:00',100.43,763,794,74.04,'Intermediate' from dual union all
select 13029,'013029','1982-001C','deb Kosmos-1331 [UNID]','Strela-2M debris 2',54310,'In Earth orbit','','1982-02-07 00:00:00',100.91,789,815,74.08,'Intermediate' from dual union all
select 1303,'001303','1965-017A','NNS O-3','NNS O-3',11767,'Reentered','1965-06-14 00:00:00','1965-04-10 00:00:00',95.24,207,854,89.97,'Polar' from dual union all
select 13030,'013030','1982-001D','deb Kosmos-1331','Strela-2M debris 1',54310,'In Earth orbit','','1982-02-06 00:00:00',100.6,766,807,74.05,'Intermediate' from dual union all
select 13031,'013031','1982-002A','Kosmos-1332','Orion',54311,'Landed','1982-01-25 00:00:00','1982-01-14 00:00:00',89.03,208,246,82.31,'Intermediate' from dual union all
select 13032,'013032','1982-002B','11S510','11S510',54311,'Reentered','1982-01-16 00:00:00','1982-01-14 00:00:00',88.49,189,211,82.31,'Intermediate' from dual union all
select 13033,'013033','1982-003A','Kosmos-1333','Parus',54312,'In Earth orbit','','1982-02-19 00:00:00',104.95,969,1015,82.94,'Intermediate' from dual union all
select 13034,'013034','1982-003B','S3M','S3M',54312,'In Earth orbit','','1991-03-25 00:00:00',104.77,961,1006,82.94,'Intermediate' from dual union all
select 13035,'013035','1982-004A','Satcom 4','RCA-C''',54313,'In Earth orbit','','1982-02-15 00:00:00',1436.09,35776,35797,.16,'Stationary' from dual union all
select 13036,'013036','1982-005A','Kosmos-1334','Zenit-6',54314,'Landed','1982-02-03 00:00:00','1982-01-22 00:00:00',89.64,225,289,72.86,'Intermediate' from dual union all
select 13037,'013037','1982-005B','11S510','11S510',54314,'Reentered','1982-01-24 00:00:00','1982-01-22 00:00:00',88.78,183,246,72.86,'Intermediate' from dual union all
select 13038,'013038','1982-005C','Sensor cover','Sensor cover',54314,'Reentered','1982-01-22 00:00:00','1982-01-21 00:00:00',88.13,142,222,72.91,'Intermediate' from dual union all
select 13039,'013039','1982-005D','Sensor cover','Sensor cover',54314,'Reentered','1982-01-20 00:00:00','1982-01-20 00:00:00',89.08,110,348,72.94,'Intermediate' from dual union all
select 1304,'001304','1965-012FY','deb Kosmos-57','',11762,'Reentered','1965-03-28 00:00:00','1965-03-20 00:00:00',89.95,165,379,64.79,'Intermediate' from dual union all
select 13040,'013040','1982-006A','OPS 2849','GAMBIT-3 Mission 4352',54315,'Reentered Attached','1982-05-23 00:00:00','1982-02-20 00:00:00',97.21,606,645,97.25,'Sun-Synch' from dual union all
select 13041,'013041','1982-006B','OPS 2849 viewport hatch','KH-8 part',54315,'Reentered','1982-01-22 00:00:00','1982-01-21 00:00:00',91.2,157,510,97.51,'Sun-Synch' from dual union all
select 13042,'013042','1982-007A','Kosmos-1335','Romb',54316,'Reentered','1987-04-05 00:00:00','1982-02-28 00:00:00',94.46,474,511,74.05,'Intermediate' from dual union all
select 13043,'013043','1982-007B','S3M','S3M',54316,'Reentered','1985-11-24 00:00:00','1982-02-28 00:00:00',94.41,471,509,74.05,'Intermediate' from dual union all
select 13044,'013044','1982-007C','deb Kosmos-1335','',54316,'Reentered','1982-01-30 00:00:00','1982-01-29 00:00:00',94.32,458,513,74.07,'Intermediate' from dual union all
select 13045,'013045','1982-008A','Kosmos-1336','Feniks No. 953',54317,'Landed','1982-02-26 00:00:00','1982-02-06 00:00:00',89.83,169,363,70.35,'Intermediate' from dual union all
select 13046,'013046','1982-008B','11S510','11S510',54317,'Reentered','1982-02-02 00:00:00','1982-02-01 00:00:00',89.15,159,306,70.35,'Intermediate' from dual union all
select 13047,'013047','1982-008C','Kosmos-1336 part','Feniks part',54317,'Reentered','1982-01-31 00:00:00','1982-01-30 00:00:00',88.52,145,257,70.34,'Intermediate' from dual union all
select 13048,'013048','1982-005E','Blenda','Sunshade',54314,'Reentered','1982-02-04 00:00:00','1982-02-03 00:00:00',89.47,221,276,72.86,'Intermediate' from dual union all
select 13049,'013049','1982-005F','KDU part','KDU part',54314,'Reentered','1982-02-06 00:00:00','1982-02-03 00:00:00',89.57,225,282,72.85,'Intermediate' from dual union all
select 1305,'001305','1965-025A','Kosmos-64','Zenit-2 No. 17',11775,'Landed','1965-04-02 00:00:00','1965-03-28 00:00:00',89.09,203,256,65,'Intermediate' from dual union all
select 13050,'013050','1982-005G','KDU','KDU',54314,'Reentered','1982-02-17 00:00:00','1982-02-03 00:00:00',89.58,219,289,72.84,'Intermediate' from dual union all
select 13051,'013051','1982-005H','KDU part','KDU part',54314,'Reentered','1982-02-06 00:00:00','1982-02-03 00:00:00',89.43,222,271,72.86,'Intermediate' from dual union all
select 13052,'013052','1982-005J','KDU part','KDU part',54314,'Reentered','1982-02-05 00:00:00','1982-02-03 00:00:00',91.2,234,433,72.87,'Intermediate' from dual union all
select 13053,'013053','1981-053EY','deb Kosmos-1275','',52576,'In Earth orbit','','1982-02-03 00:00:00',107.15,977,1213,83.09,'Intermediate' from dual union all
select 13054,'013054','1981-097C','Kosmos-1311 SS 1','ESO',52620,'Reentered','1982-02-27 00:00:00','1982-02-04 00:00:00',93.88,445,484,82.95,'Intermediate' from dual union all
select 13055,'013055','1981-097D','Kosmos-1311 SS 2','ESO',52620,'Reentered','1982-02-26 00:00:00','1982-02-05 00:00:00',93.84,442,483,82.95,'Intermediate' from dual union all
select 13056,'013056','1982-009A','Ekran','Ekran No. 22L',54318,'In Earth orbit','','1982-11-05 00:00:00',1436.06,35769,35802,.19,'Stationary' from dual union all
select 13057,'013057','1982-009B','8S812','8S812',54318,'Reentered','1982-02-07 00:00:00','1982-02-06 00:00:00',87.89,168,173,51.62,'Intermediate' from dual;
insert into SATELLITE
select 13058,'013058','1982-009C','Perekhodnik','Sredniy perekhodnik',54318,'Reentered','1982-02-06 00:00:00','1982-02-05 00:00:00',88.42,186,207,51.66,'Intermediate' from dual union all
select 13059,'013059','1982-009D','SOZ','SOZ',54318,'Reentered','1991-09-07 00:00:00','1982-06-08 00:00:00',620.99,164,35302,46.85,'GEO Transfer' from dual union all
select 1306,'001306','1965-025B','8A92E','8A92E',11775,'Reentered','1965-04-04 00:00:00','1965-03-28 00:00:00',89.14,210,253,65.03,'Intermediate' from dual union all
select 13060,'013060','1982-009E','SOZ','SOZ',54318,'Reentered','1984-11-26 00:00:00','1982-03-08 00:00:00',629.74,330,35588,47.26,'GEO Transfer' from dual union all
select 13061,'013061','1982-010A','Kosmos-1337','US-PM1',54319,'Reentered','1982-07-25 00:00:00','1982-03-14 00:00:00',92.78,383,439,65,'Intermediate' from dual union all
select 13062,'013062','1982-010B','11S692','11S692',54319,'Reentered','1982-02-11 00:00:00','1982-02-11 00:00:00',89.8,113,417,65.02,'Intermediate' from dual union all
select 13063,'013063','1982-011A','Kosmos-1338','Zenit-6',54320,'Landed','1982-03-02 00:00:00','1982-02-18 00:00:00',92.25,356,413,72.85,'Intermediate' from dual union all
select 13064,'013064','1982-011B','11S510','11S510',54320,'Reentered','1982-02-24 00:00:00','1982-02-18 00:00:00',89.78,193,335,72.84,'Intermediate' from dual union all
select 13065,'013065','1982-012A','Kosmos-1339','Tsikada',54321,'In Earth orbit','','1982-03-25 00:00:00',104.77,952,1016,82.9,'Intermediate' from dual union all
select 13066,'013066','1982-012B','S3M','S3M',54321,'In Earth orbit','','1991-03-25 00:00:00',104.61,943,1010,82.9,'Intermediate' from dual union all
select 13067,'013067','1982-013A','Kosmos-1340','Tselina-D',54322,'In Earth orbit','','1982-03-21 00:00:00',97.48,629,648,81.21,'Intermediate' from dual union all
select 13068,'013068','1982-013B','8A92ME','8A92ME',54322,'In Earth orbit','','1991-03-28 00:00:00',96.99,590,640,81.2,'Intermediate' from dual union all
select 13069,'013069','1982-014A','Westar 4','Westar 4',54323,'In Earth orbit','','1982-04-09 00:00:00',1436.11,35783,35790,.05,'Stationary' from dual union all
select 1307,'001307','1965-026A','OPS 4803','CORONA J-19',11776,'Reentered Attached','1965-04-05 00:00:00','1965-03-28 00:00:00',88.87,179,259,96.02,'Sun-Synch' from dual union all
select 13070,'013070','1982-015A','Molniya-1','Molniya-1',54324,'In Earth orbit','','1982-03-28 00:00:00',717.68,467,39883,62.91,'Molniya' from dual union all
select 13071,'013071','1982-015B','11S510','11S510',54324,'Reentered','1982-03-08 00:00:00','1982-02-28 00:00:00',90.86,182,452,62.85,'Intermediate' from dual union all
select 13072,'013072','1982-015C','BOZ','BOZ',54324,'Reentered','1982-03-13 00:00:00','1982-03-05 00:00:00',90.41,196,394,62.84,'Intermediate' from dual union all
select 13073,'013073','1981-053EZ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',105.12,936,1065,82.94,'Intermediate' from dual union all
select 13074,'013074','1981-053FA','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',104.74,959,1006,82.95,'Intermediate' from dual union all
select 13075,'013075','1982-015D','Blok-L','Blok-ML No. 77337-455',54324,'In Earth orbit','','1982-03-28 00:00:00',731.1,463,40547,62.9,'Molniya' from dual union all
select 13076,'013076','1982-011C','Blenda','Sunshade',54320,'Reentered','1982-03-21 00:00:00','1982-03-02 00:00:00',92.23,355,413,72.84,'Intermediate' from dual union all
select 13077,'013077','1982-011D','KDU','KDU',54320,'Reentered','1982-09-27 00:00:00','1982-03-18 00:00:00',92.06,350,402,72.84,'Intermediate' from dual union all
select 13078,'013078','1982-011E','KDU part','KDU part',54320,'Reentered','1982-04-06 00:00:00','1982-03-04 00:00:00',91.96,338,404,72.85,'Intermediate' from dual union all
select 13079,'013079','1982-011F','KDU part','KDU part',54320,'Reentered','1982-03-19 00:00:00','1982-03-02 00:00:00',92.19,350,414,72.85,'Intermediate' from dual union all
select 1308,'001308','1965-025C','deb Kosmos-64','',11775,'Reentered','1965-03-26 00:00:00','1965-03-28 00:00:00',89.14,210,253,65.03,'Intermediate' from dual union all
select 13080,'013080','1982-016A','Kosmos-1341','Oko',54325,'In Earth orbit','','1982-04-03 00:00:00',717.9,673,39688,62.9,'Molniya' from dual union all
select 13081,'013081','1982-016B','11S510','11S510',54325,'Reentered','1982-03-27 00:00:00','1982-03-10 00:00:00',91.95,205,536,62.82,'Intermediate' from dual union all
select 13082,'013082','1982-016C','BOZ','BOZ',54325,'Reentered','1982-03-16 00:00:00','1982-03-05 00:00:00',92.21,175,591,62.9,'Intermediate' from dual union all
select 13083,'013083','1982-017A','INTELSAT 504','INTELSAT V F4',54326,'In Earth orbit','','1982-04-06 00:00:00',1430.42,35602,35748,.32,'Drift GEO' from dual union all
select 13084,'013084','1982-018A','Kosmos-1342','Zenit-6',54327,'Landed','1982-03-19 00:00:00','1982-03-07 00:00:00',89.63,226,287,72.86,'Intermediate' from dual union all
select 13085,'013085','1982-018B','11S510','11S510',54327,'Reentered','1982-03-09 00:00:00','1982-03-07 00:00:00',88.95,184,262,72.86,'Intermediate' from dual union all
select 13086,'013086','1982-019A','DSP F10','DSP 13',54328,'In Earth orbit','','1982-04-05 00:00:00',1435.17,35733,35803,1.93,'Drift GEO' from dual union all
select 13087,'013087','1982-017B','Centaur AC-58','Centaur D-1AR AC-58',54326,'Reentered','1983-05-06 00:00:00','1982-05-04 00:00:00',530.58,167,30497,23.99,'Highly Ellip' from dual union all
select 13088,'013088','1982-004B','PAM-D','Star 48  S/N 10009-8',54313,'Reentered','1988-11-02 00:00:00','1982-02-23 00:00:00',609.18,177,34676,27.32,'GEO Transfer' from dual union all
select 13089,'013089','1982-019B','Transtage 38','Transtage 38',54328,'In Earth orbit','','1982-04-01 00:00:00',1431.67,35600,35800,2,'Drift GEO' from dual union all
select 1309,'001309','1965-012FZ','deb Kosmos-57','',11762,'Reentered','1965-03-28 00:00:00','1965-03-19 00:00:00',90.38,140,447,64.82,'Intermediate' from dual union all
select 13090,'013090','1982-016D','Blok-L','Blok-2BL No. 44',54325,'In Earth orbit','','1982-04-02 00:00:00',708.88,730,39185,62.91,'Molniya' from dual union all
select 13091,'013091','1964-049F','deb Kosmos-41','',10335,'Reentered','2002-12-31 00:00:00','1975-06-27 00:00:00',716.02,1635,38633,69.54,'GEO Transfer' from dual union all
select 13092,'013092','1982-020A','Gorizont','Gorizont No. 14L',54329,'In Earth orbit','','1982-04-21 00:00:00',1436.11,35242,36332,.72,'Inclined GEO' from dual union all
select 13093,'013093','1982-020B','8S812','8S812',54329,'Reentered','1982-03-16 00:00:00','1982-03-15 00:00:00',87.69,144,177,51.63,'Intermediate' from dual union all
select 13094,'013094','1982-020C','Perekhodnik','Sredniy perekhodnik',54329,'Reentered','1982-03-16 00:00:00','1982-03-15 00:00:00',88.02,171,182,51.63,'Intermediate' from dual union all
select 13095,'013095','1982-020D','deb Blok Proton','',54329,'Reentered','1982-03-16 00:00:00','1982-03-16 00:00:00',87.84,165,171,51.64,'Intermediate' from dual union all
select 13096,'013096','1982-021A','Kosmos-1343','Zenit-6',54330,'Landed','1982-03-31 00:00:00','1982-03-19 00:00:00',89.63,227,286,72.84,'Intermediate' from dual union all
select 13097,'013097','1982-021B','11S510','11S510',54330,'Reentered','1982-03-21 00:00:00','1982-03-19 00:00:00',88.65,178,238,72.84,'Intermediate' from dual union all
select 13098,'013098','1981-114B','PAM-D','Star 48  S/N 10008-8',52637,'Reentered','1989-02-26 00:00:00','1982-07-16 00:00:00',439.81,183,25376,27.33,'Highly Ellip' from dual union all
select 13099,'013099','1982-020E','SOZ','SOZ',54329,'Reentered','1983-09-14 00:00:00','1982-04-20 00:00:00',642.06,238,36313,47.3,'GEO Transfer' from dual union all
select 131,'000131','61 Omicron 16','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-11 00:00:00',103.46,856,989,66.82,'Intermediate' from dual union all
select 1310,'001310','1965-016G','SURCAL 4','SURCAL',11766,'Reentered','1981-03-27 00:00:00','1965-04-24 00:00:00',103.47,909,937,70.09,'Intermediate' from dual union all
select 13100,'013100','1982-018C','Blenda','Sunshade',54327,'Reentered','1982-03-21 00:00:00','1982-03-19 00:00:00',89.54,227,277,72.85,'Intermediate' from dual union all
select 13101,'013101','1982-018D','KDU part','KDU part',54327,'Reentered','1982-03-22 00:00:00','1982-03-19 00:00:00',89.21,218,253,72.85,'Intermediate' from dual union all
select 13102,'013102','1982-018E','KDU','KDU',54327,'Reentered','1982-04-01 00:00:00','1982-03-19 00:00:00',89.53,222,281,72.86,'Intermediate' from dual union all
select 13103,'013103','1982-006C','SRV-1 heat shield','',54315,'Reentered','1999-02-18 00:00:00','1982-05-16 00:00:00',97.34,609,654,97.24,'Sun-Synch' from dual union all
select 13104,'013104','1982-006D','SRV-1 thermal cover','',54315,'Reentered','1998-09-01 00:00:00','1982-05-16 00:00:00',97.33,617,645,97.24,'Sun-Synch' from dual union all
select 13105,'013105','1982-006E','SRV Mission 4352-1','SRV-1',54315,'Reentered','2002-09-28 00:00:00','1982-05-16 00:00:00',97.37,619,647,97.24,'Sun-Synch' from dual union all
select 13106,'013106','1982-022A','Columbia (STS-3)','OV-102',54331,'Landed','1982-03-30 00:00:00','1982-03-24 00:00:00',89.32,237,245,38.03,'Intermediate' from dual union all
select 13107,'013107','1982-023A','Molniya-3','Molniya-3 No. 29',54332,'Reentered','1992-06-23 00:00:00','1982-04-23 00:00:00',717.74,629,39724,62.86,'Molniya' from dual union all
select 13108,'013108','1982-023B','BOZ','BOZ',54332,'Reentered','1982-04-12 00:00:00','1982-03-31 00:00:00',91.85,186,545,62.85,'Intermediate' from dual union all
select 13109,'013109','1982-023C','11S510','11S510',54332,'Reentered','1982-04-19 00:00:00','1982-03-31 00:00:00',92.13,209,548,62.83,'Intermediate' from dual union all
select 1311,'001311','1965-026B','CORONA Fairing','Fairing',11776,'Reentered','1965-04-01 00:00:00','1965-03-31 00:00:00',91.14,280,380,96.02,'Sun-Synch' from dual union all
select 13110,'013110','1982-024A','Kosmos-1344','Parus',54333,'In Earth orbit','','1982-04-28 00:00:00',104.88,968,1010,82.92,'Intermediate' from dual union all
select 13111,'013111','1982-024B','S3M','S3M',54333,'In Earth orbit','','1991-03-28 00:00:00',104.74,953,1012,82.92,'Intermediate' from dual union all
select 13112,'013112','1982-023D','Blok-L','Blok-ML No. 77323-517',54332,'Reentered','1992-09-17 00:00:00','1992-06-09 00:00:00',719.36,183,40250,63.55,'Molniya' from dual union all
select 13113,'013113','1982-025A','Meteor-2','Meteor-2 No. 12',54334,'In Earth orbit','','1982-04-25 00:00:00',104.06,939,962,82.54,'Intermediate' from dual union all
select 13114,'013114','1982-025B','S5M','S5M',54334,'In Earth orbit','','1991-03-25 00:00:00',104.02,937,960,82.53,'Intermediate' from dual union all
select 13115,'013115','1982-021C','KDU','KDU',54330,'Reentered','1982-04-05 00:00:00','1982-03-31 00:00:00',88.97,214,234,72.84,'Intermediate' from dual union all
select 13116,'013116','1982-021D','Blenda','Sunshade',54330,'Reentered','1982-04-01 00:00:00','1982-03-31 00:00:00',88.86,207,230,72.97,'Intermediate' from dual union all
select 13117,'013117','1982-021E','KDU part','KDU part',54330,'Reentered','1982-04-01 00:00:00','1982-03-31 00:00:00',88.94,211,234,72.86,'Intermediate' from dual union all
select 13118,'013118','1982-026A','Kosmos-1345','Tselina-OM',54335,'Reentered','1989-09-27 00:00:00','1982-04-30 00:00:00',95.07,501,542,74.04,'Intermediate' from dual union all
select 13119,'013119','1982-026B','S3M','S3M',54335,'Reentered','1989-08-10 00:00:00','1988-06-13 00:00:00',93.89,450,480,74.04,'Intermediate' from dual union all
select 1312,'001312','1965-004C','Tiros IX despin weight','',11754,'In Earth orbit','','1965-03-28 00:00:00',118.02,668,2516,96.36,'Retrograde' from dual union all
select 13120,'013120','1982-027A','Kosmos-1346','Tselina-D',54336,'Reentered','2017-12-17 00:00:00','1982-04-30 00:00:00',97.52,622,658,81.17,'Intermediate' from dual union all
select 13121,'013121','1982-027B','8A92ME','8A92ME',54336,'In Earth orbit','','1991-03-28 00:00:00',97.08,589,649,81.16,'Intermediate' from dual union all
select 13122,'013122','1982-028A','Kosmos-1347','Yantar''-4K2 No. 2',54337,'Landed','1982-05-21 00:00:00','1982-04-09 00:00:00',89.63,162,351,70.33,'Intermediate' from dual union all
select 13123,'013123','1982-028B','11S510','11S510',54337,'Reentered','1982-04-05 00:00:00','1982-04-04 00:00:00',88.73,158,266,70.35,'Intermediate' from dual union all
select 13124,'013124','1982-029A','Kosmos-1348','Oko',54338,'In Earth orbit','','1982-05-07 00:00:00',717.74,612,39741,62.91,'Molniya' from dual union all
select 13125,'013125','1982-029B','11S510','11S510',54338,'Reentered','1982-05-05 00:00:00','1982-04-14 00:00:00',91.91,208,529,62.85,'Intermediate' from dual union all
select 13126,'013126','1982-029C','BOZ','BOZ',54338,'Reentered','1982-04-27 00:00:00','1982-04-14 00:00:00',91.85,186,545,62.91,'Intermediate' from dual union all
select 13127,'013127','1982-030A','Kosmos-1349','Parus',54339,'In Earth orbit','','1982-05-24 00:00:00',104.9,967,1013,82.93,'Intermediate' from dual union all
select 13128,'013128','1982-030B','S3M','S3M',54339,'In Earth orbit','','1991-03-25 00:00:00',104.73,961,1003,82.93,'Intermediate' from dual union all
select 13129,'013129','1982-031A','Insat 1A','Insat 1A',54340,'In Earth orbit','','1982-06-25 00:00:00',1435.98,35393,36176,.37,'Stationary' from dual union all
select 1313,'001313','1965-004D','Tiros IX despin weight','',11754,'In Earth orbit','','1965-04-01 00:00:00',120.37,730,2665,96.45,'Medium' from dual union all
select 13130,'013130','1974-089EL','deb Delta 104','',33692,'Reentered','1990-09-14 00:00:00','1990-09-14 00:00:00',92.18,362,401,101.37,'Retrograde' from dual union all
select 13131,'013131','1965-027H','deb SNAPSHOT','',11777,'In Earth orbit','','1993-07-29 00:00:00',111.31,1271,1302,90.27,'Polar' from dual union all
select 13132,'013132','1978-018C','deb Ume-2','',45213,'In Earth orbit','','1993-08-07 00:00:00',107.9,973,1287,69.2,'Intermediate' from dual union all
select 13133,'013133','1977-012J','deb Tansei 3','',42140,'In Earth orbit','','1982-04-12 00:00:00',133.52,800,3751,65.84,'Medium' from dual union all
select 13134,'013134','1982-032A','Kosmos-1350','Feniks No. 978',54341,'Landed','1982-05-16 00:00:00','1982-04-23 00:00:00',89.5,171,329,67.13,'Intermediate' from dual union all
select 13135,'013135','1982-032B','11S510','11S510',54341,'Reentered','1982-04-18 00:00:00','1982-04-17 00:00:00',88.94,159,286,67.13,'Intermediate' from dual union all
select 13136,'013136','1982-014B','PAM-D','Star 48  S/N 10010-8',54323,'Reentered','1989-08-31 00:00:00','1982-04-16 00:00:00',611.47,187,34785,27.26,'GEO Transfer' from dual union all
select 13137,'013137','1982-031B','PAM-D','Star 48  S/N 10011-8',54340,'Reentered','1984-04-18 00:00:00','1982-05-16 00:00:00',587.66,148,33577,28.52,'Highly Ellip' from dual union all
select 13138,'013138','1982-033A','Salyut-7','DOS 6 (17K No. 125-02)',54342,'Reentered','1991-02-07 00:00:00','1982-05-19 00:00:00',91.38,341,344,51.59,'Intermediate' from dual union all
select 13139,'013139','1982-033B','8S812','8S812',54342,'Reentered','1982-04-24 00:00:00','1982-04-21 00:00:00',88.71,201,220,51.6,'Intermediate' from dual union all
select 1314,'001314','1965-027A','SNAPSHOT-1','SNAP-10A FS-4',11777,'In Earth orbit','','1965-05-08 00:00:00',111.49,1273,1318,90.21,'Polar' from dual union all
select 13140,'013140','1982-007D','Kosmos-1335 SS 1','ESO',54316,'Reentered','1982-06-15 00:00:00','1982-04-20 00:00:00',94.37,473,503,74.05,'Intermediate' from dual union all
select 13141,'013141','1982-007E','Kosmos-1335 SS 2','ESO',54316,'Reentered','1982-06-11 00:00:00','1982-04-20 00:00:00',94.23,464,498,74.05,'Intermediate' from dual union all
select 13142,'013142','1982-034A','Kosmos-1351','Romb',54343,'Reentered','1983-03-14 00:00:00','1982-05-21 00:00:00',93.4,345,536,50.69,'Intermediate' from dual union all
select 13143,'013143','1982-034B','S3M','S3M',54343,'Reentered','1983-02-22 00:00:00','1982-05-21 00:00:00',93.29,340,532,50.71,'Intermediate' from dual union all
select 13144,'013144','1982-035A','Kosmos-1352','Zenit-6',54344,'Landed','1982-05-05 00:00:00','1982-04-23 00:00:00',92.18,348,415,70.38,'Intermediate' from dual union all
select 13145,'013145','1982-035B','11S510','11S510',54344,'Reentered','1982-04-29 00:00:00','1982-04-23 00:00:00',89.77,200,327,70.37,'Intermediate' from dual union all
select 13146,'013146','1982-036A','Kosmos-1353','Fram',54345,'Landed','1982-05-06 00:00:00','1982-04-25 00:00:00',88.96,209,238,82.35,'Intermediate' from dual union all
select 13147,'013147','1982-036B','11S510','11S510',54345,'Reentered','1982-04-26 00:00:00','1982-04-25 00:00:00',88.4,190,201,82.35,'Intermediate' from dual;
insert into SATELLITE
select 13148,'013148','1982-037A','Kosmos-1354','Strela-2M',54346,'In Earth orbit','','1982-05-30 00:00:00',100.86,792,807,74.04,'Intermediate' from dual union all
select 13149,'013149','1982-037B','S3M','S3M',54346,'In Earth orbit','','1991-03-28 00:00:00',100.61,778,797,74.04,'Intermediate' from dual union all
select 1315,'001315','1965-027B','EGRS IV','Secor Type II S/N 5',11777,'In Earth orbit','','1965-05-20 00:00:00',111.43,1265,1320,90.22,'Polar' from dual union all
select 13150,'013150','1982-038A','Kosmos-1355','US-PM1',54347,'Reentered','1984-03-07 00:00:00','1982-05-29 00:00:00',93.29,429,443,65.07,'Intermediate' from dual union all
select 13151,'013151','1982-038B','11S692','11S692',54347,'Reentered','1982-04-29 00:00:00','1982-04-29 00:00:00',89.8,127,403,65.06,'Intermediate' from dual union all
select 13152,'013152','1982-006F','Fairing','GAMBIT SRV-2 Fairing',54315,'Reentered','1995-04-01 00:00:00','1982-05-10 00:00:00',96.81,601,611,97.23,'Sun-Synch' from dual union all
select 13153,'013153','1982-039A','Kosmos-1356','Tselina-D',54348,'In Earth orbit','','1982-06-06 00:00:00',97.7,631,667,81.18,'Intermediate' from dual union all
select 13154,'013154','1982-039B','8A92ME','8A92ME',54348,'In Earth orbit','','1991-03-26 00:00:00',97.41,593,677,81.19,'Intermediate' from dual union all
select 13155,'013155','1982-035C','Blenda','Sunshade',54344,'Reentered','1982-06-14 00:00:00','1982-05-07 00:00:00',92.15,353,407,70.39,'Intermediate' from dual union all
select 13156,'013156','1982-035D','KDU','KDU',54344,'Reentered','1982-11-30 00:00:00','1982-05-21 00:00:00',92.11,353,403,70.38,'Intermediate' from dual union all
select 13157,'013157','1982-035E','KDU part','KDU part',54344,'Reentered','1982-05-30 00:00:00','1982-05-06 00:00:00',92,348,397,70.41,'Intermediate' from dual union all
select 13158,'013158','1982-035F','KDU part','KDU part',54344,'Reentered','1982-07-02 00:00:00','1982-05-21 00:00:00',91.67,328,385,70.37,'Intermediate' from dual union all
select 13159,'013159','1982-036C','KDU','KDU',54345,'Reentered','1982-05-10 00:00:00','1982-05-06 00:00:00',88.69,200,220,82.35,'Intermediate' from dual union all
select 1316,'001316','1965-027C','SNAPSHOT Heat Shield','',11777,'In Earth orbit','','1965-05-08 00:00:00',111.49,1261,1330,90.22,'Polar' from dual union all
select 13160,'013160','1982-040A','Kosmos-1357','Strela-1M',54349,'In Earth orbit','','1982-06-09 00:00:00',114.66,1401,1478,74.01,'Intermediate' from dual union all
select 13161,'013161','1982-040B','Kosmos-1358','Strela-1M',54349,'In Earth orbit','','1992-09-28 00:00:00',114.83,1415,1481,74.01,'Intermediate' from dual union all
select 13162,'013162','1982-040C','Kosmos-1359','Strela-1M',54349,'In Earth orbit','','1992-09-28 00:00:00',115.01,1432,1480,74.01,'Intermediate' from dual union all
select 13163,'013163','1982-040D','Kosmos-1360','Strela-1M',54349,'In Earth orbit','','1992-09-27 00:00:00',115.18,1446,1481,74.01,'Intermediate' from dual union all
select 13164,'013164','1982-040E','Kosmos-1361','Strela-1M',54349,'In Earth orbit','','1992-09-24 00:00:00',115.37,1461,1484,74.01,'Intermediate' from dual union all
select 13165,'013165','1982-040F','Kosmos-1362','Strela-1M',54349,'In Earth orbit','','1992-09-28 00:00:00',115.55,1466,1495,74.01,'Intermediate' from dual union all
select 13166,'013166','1982-040G','Kosmos-1363','Strela-1M',54349,'In Earth orbit','','1992-09-25 00:00:00',115.74,1474,1503,74.01,'Intermediate' from dual union all
select 13167,'013167','1982-040H','Kosmos-1364','Strela-1M',54349,'In Earth orbit','','1992-09-23 00:00:00',115.96,1474,1524,74.01,'Intermediate' from dual union all
select 13168,'013168','1982-040J','S3M','S3M',54349,'In Earth orbit','','1991-03-21 00:00:00',117.75,1472,1687,74.05,'Intermediate' from dual union all
select 13169,'013169','1982-029D','Blok-L','Blok-2BL No. 71637-645',54338,'In Earth orbit','','1982-05-07 00:00:00',705.45,611,39134,62.88,'Molniya' from dual union all
select 1317,'001317','1965-028A','Early Bird','INTELSAT I',11778,'In Earth orbit','','1966-09-07 00:00:00',1436.6,35785,35808,1.39,'Drift GEO' from dual union all
select 13170,'013170','1982-041A','OPS 5642','HEXAGON SV-17',54350,'Deorbited','1982-12-05 00:00:00','1982-06-11 00:00:00',88.68,167,252,96.39,'Sun-Synch' from dual union all
select 13171,'013171','1982-041B','Titan 23D-24 Stage 2','LR91AJ11 23D-24',54350,'Reentered','1982-05-13 00:00:00','1982-05-12 00:00:00',88.74,168,257,96.41,'Sun-Synch' from dual union all
select 13172,'013172','1982-041C','FARRAH I','P-11 No. 4433',54350,'In Earth orbit','','1982-06-14 00:00:00',98.79,697,705,95.99,'Sun-Synch' from dual union all
select 13173,'013173','1982-042A','Soyuz T-5','Soyuz 7K-ST No. 11L',54351,'Landed','1982-08-27 00:00:00','1982-06-14 00:00:00',90.5,285,313,51.63,'Intermediate' from dual union all
select 13174,'013174','1982-042B','11S510','11S510',54351,'Reentered','1982-05-15 00:00:00','1982-05-14 00:00:00',88.41,182,210,51.61,'Intermediate' from dual union all
select 13175,'013175','1982-043A','Kosmos-1365','US-A reactor',54352,'In Earth orbit','','1982-06-13 00:00:00',89.63,250,262,65,'Intermediate' from dual union all
select 13176,'013176','1982-033C','Iskra-2','RK-02',54342,'Reentered','1982-07-09 00:00:00','1982-05-19 00:00:00',91.29,334,342,51.59,'Intermediate' from dual union all
select 13177,'013177','1982-044A','Kosmos-1366','Geizer No. 11L',54353,'In Earth orbit','','1982-06-20 00:00:00',1437.59,35738,35894,1.43,'Drift GEO' from dual union all
select 13178,'013178','1982-044B','Perekhodnik','Sredniy perekhodnik',54353,'Reentered','1982-05-18 00:00:00','1982-05-18 00:00:00',88.26,182,195,51.63,'Intermediate' from dual union all
select 13179,'013179','1982-044C','8S812','8S812',54353,'Reentered','1982-05-19 00:00:00','1982-05-18 00:00:00',88.26,182,195,51.63,'Intermediate' from dual union all
select 1318,'001318','1965-028B','Altair 2 X-258C4 RH-83','X-258C4 RH-83',11778,'In Earth orbit','','1965-06-05 00:00:00',679.87,1409,37059,18.14,'GEO Transfer' from dual union all
select 13180,'013180','1981-028C','deb Kosmos-1260','',52551,'Reentered','1982-09-05 00:00:00','1982-09-03 00:00:00',90.31,258,322,65.11,'Intermediate' from dual union all
select 13181,'013181','1981-028D','deb Kosmos-1260','',52551,'Reentered','1982-09-10 00:00:00','1982-05-18 00:00:00',94.85,373,650,65.05,'Intermediate' from dual union all
select 13182,'013182','1981-028E','deb Kosmos-1260','',52551,'Reentered','1989-12-07 00:00:00','1989-12-21 00:00:00',92.21,379,386,65.83,'Intermediate' from dual union all
select 13183,'013183','1981-028F','deb Kosmos-1260','',52551,'Reentered','1991-08-06 00:00:00','1991-08-05 00:00:00',89.4,236,254,64.98,'Intermediate' from dual union all
select 13184,'013184','1981-028G','deb Kosmos-1260','',52551,'Reentered','1983-08-09 00:00:00','1982-05-18 00:00:00',96.23,441,715,65.03,'Intermediate' from dual union all
select 13185,'013185','1981-028H','deb Kosmos-1260','',52551,'Reentered','1982-11-04 00:00:00','1982-05-18 00:00:00',96.42,486,689,64.89,'Intermediate' from dual union all
select 13186,'013186','1981-028J','deb Kosmos-1260','',52551,'Reentered','1983-01-11 00:00:00','1982-05-09 00:00:00',96.45,454,724,64.91,'Intermediate' from dual union all
select 13187,'013187','1981-028K','deb Kosmos-1260','',52551,'Reentered','1983-10-10 00:00:00','1983-10-10 00:00:00',90.27,279,296,64.98,'Intermediate' from dual union all
select 13188,'013188','1981-028L','deb Kosmos-1260','',52551,'Reentered','1982-06-15 00:00:00','1982-05-31 00:00:00',96.4,387,786,65.04,'Intermediate' from dual union all
select 13189,'013189','1981-028M','deb Kosmos-1260','',52551,'Reentered','1988-10-22 00:00:00','1982-03-22 00:00:00',89.37,240,246,38.02,'Intermediate' from dual union all
select 1319,'001319','1965-017G','deb Ablestar','',11767,'Reentered','1965-06-28 00:00:00','1965-04-10 00:00:00',96.32,261,903,90,'Polar' from dual union all
select 13190,'013190','1981-028N','deb Kosmos-1260','',52551,'Reentered','1982-11-02 00:00:00','1982-10-09 00:00:00',93.83,382,541,65.04,'Intermediate' from dual union all
select 13191,'013191','1981-028P','deb Kosmos-1260','',52551,'Reentered','1983-09-19 00:00:00','1983-08-08 00:00:00',91.73,342,377,64.99,'Intermediate' from dual union all
select 13192,'013192','1981-028Q','deb Kosmos-1260','',52551,'Reentered','1985-06-15 00:00:00','1982-05-18 00:00:00',96.32,442,723,64.06,'Intermediate' from dual union all
select 13193,'013193','1981-028R','deb Kosmos-1260','',52551,'Reentered','1982-06-13 00:00:00','1982-06-13 00:00:00',92.33,331,447,64.61,'Intermediate' from dual union all
select 13194,'013194','1981-028S','deb Kosmos-1260','',52551,'Reentered','1982-09-01 00:00:00','1982-09-01 00:00:00',89.07,207,250,64.88,'Intermediate' from dual union all
select 13195,'013195','1981-028T','deb Kosmos-1260','',52551,'Reentered','1982-09-10 00:00:00','1982-05-29 00:00:00',95.98,433,699,64.87,'Intermediate' from dual union all
select 13196,'013196','1981-028U','deb Kosmos-1260','',52551,'Reentered','1982-05-31 00:00:00','1982-05-30 00:00:00',90.37,267,319,64.85,'Intermediate' from dual union all
select 13197,'013197','1981-028V','deb Kosmos-1260','',52551,'Reentered','1984-12-11 00:00:00','1982-05-18 00:00:00',98.04,482,848,64.97,'Intermediate' from dual union all
select 13198,'013198','1981-028W','deb Kosmos-1260','',52551,'Reentered','1982-10-03 00:00:00','1982-09-27 00:00:00',90.57,282,323,64.87,'Intermediate' from dual union all
select 13199,'013199','1981-028X','deb Kosmos-1260','',52551,'Reentered','1982-08-31 00:00:00','1982-08-29 00:00:00',91.25,298,373,64.92,'Intermediate' from dual union all
select 132,'000132','61 Omicron 17','Transit 4A despin weight','',7273,'In Earth orbit','','1961-08-11 00:00:00',103.55,870,983,66.66,'Intermediate' from dual union all
select 1320,'001320','1965-029A','Kosmos-65','Zenit-4 No. 6',11779,'Landed','1965-04-25 00:00:00','1965-04-20 00:00:00',89.72,205,317,65,'Intermediate' from dual union all
select 13200,'013200','1981-028Y','deb Kosmos-1260','',52551,'Reentered','1982-06-16 00:00:00','1982-06-17 00:00:00',92.5,345,450,64.93,'Intermediate' from dual union all
select 13201,'013201','1981-028Z','deb Kosmos-1260','',52551,'Reentered','1982-06-15 00:00:00','1982-06-30 00:00:00',95.33,419,651,64.95,'Intermediate' from dual union all
select 13202,'013202','1981-028AA','deb Kosmos-1260','',52551,'Reentered','1982-08-14 00:00:00','1982-08-14 00:00:00',89.07,203,253,64.97,'Intermediate' from dual union all
select 13203,'013203','1981-028AB','deb Kosmos-1260','',52551,'Reentered','1982-06-21 00:00:00','1982-06-21 00:00:00',90.64,286,326,64.85,'Intermediate' from dual union all
select 13204,'013204','1982-033D','Iskra-2 cover','RK-02 cover',54342,'Reentered','1982-06-20 00:00:00','1982-05-19 00:00:00',91.31,333,345,51.58,'Intermediate' from dual union all
select 13205,'013205','1982-045A','Kosmos-1367','Oko',54354,'In Earth orbit','','1982-06-20 00:00:00',717.96,604,39760,62.98,'Molniya' from dual union all
select 13206,'013206','1982-045B','11S510','11S510',54354,'Reentered','1982-06-17 00:00:00','1982-05-27 00:00:00',91.73,205,514,62.81,'Intermediate' from dual union all
select 13207,'013207','1982-045C','BOZ','BOZ',54354,'Reentered','1982-06-04 00:00:00','1982-05-27 00:00:00',91.27,169,505,62.86,'Intermediate' from dual union all
select 13208,'013208','1982-046A','Kosmos-1368','Zenit-6',54355,'Landed','1982-06-03 00:00:00','1982-05-23 00:00:00',89.88,235,302,70.37,'Intermediate' from dual union all
select 13209,'013209','1982-046B','11S510','11S510',54355,'Reentered','1982-05-29 00:00:00','1982-05-23 00:00:00',89.66,203,313,70.38,'Intermediate' from dual union all
select 1321,'001321','1965-029B','11A57I','11A57I',11779,'Reentered','1965-05-06 00:00:00','1965-04-17 00:00:00',89.86,211,324,65,'Intermediate' from dual union all
select 13210,'013210','1982-047A','Progress-13','Progress 7K-TG No. 114',54356,'Deorbited','1982-06-06 00:00:00','1982-05-25 00:00:00',90.88,290,346,51.62,'Intermediate' from dual union all
select 13211,'013211','1982-047B','11S510','11S510',54356,'Reentered','1982-05-25 00:00:00','1982-05-24 00:00:00',88.44,171,224,51.62,'Intermediate' from dual union all
select 13212,'013212','1982-047C','Blok-I part','11S510 part',54356,'Reentered','1982-05-25 00:00:00','1982-05-23 00:00:00',87.85,156,181,51.62,'Intermediate' from dual union all
select 13213,'013213','1982-048A','Kosmos-1369','Resurs-F1 17F41 No. 17',54357,'Landed','1982-06-08 00:00:00','1982-05-27 00:00:00',89.81,258,273,82.31,'Intermediate' from dual union all
select 13214,'013214','1982-048B','11S510','11S510',54357,'Reentered','1982-05-30 00:00:00','1982-05-27 00:00:00',88.95,205,240,82.32,'Intermediate' from dual union all
select 13215,'013215','1982-045D','Blok-L','Blok-2BL No. 71623-646',54354,'Reentered','2009-01-18 00:00:00','1982-06-21 00:00:00',704.14,606,39074,62.97,'Molniya' from dual union all
select 13216,'013216','1982-048C','Sensor cover','Sensor cover',54357,'Reentered','1982-05-26 00:00:00','1982-05-25 00:00:00',89.15,202,263,82.3,'Intermediate' from dual union all
select 13217,'013217','1982-044D','SOZ','SOZ',54353,'Reentered','1983-09-14 00:00:00','1982-06-17 00:00:00',634.52,333,35831,47.3,'GEO Transfer' from dual union all
select 13218,'013218','1982-044E','SOZ','SOZ',54353,'Reentered','1983-09-14 00:00:00','1982-06-21 00:00:00',633.18,276,35819,47.26,'GEO Transfer' from dual union all
select 13219,'013219','1982-049A','Kosmos-1370','Siluet No. 2',54358,'Landed','1982-07-11 00:00:00','1982-06-04 00:00:00',89.16,205,261,64.86,'Intermediate' from dual union all
select 1322,'001322','1965-029C','deb Kosmos-65','',11779,'Reentered','1965-04-20 00:00:00','1965-04-20 00:00:00',87.52,152,152,65.1,'Intermediate' from dual union all
select 13220,'013220','1982-049B','11S510','11S510',54358,'Reentered','1982-05-31 00:00:00','1982-05-30 00:00:00',88.63,183,231,64.87,'Intermediate' from dual union all
select 13221,'013221','1981-028AC','deb Kosmos-1260','',52551,'Reentered','1982-06-15 00:00:00','1982-05-31 00:00:00',95.35,377,694,64.99,'Intermediate' from dual union all
select 13222,'013222','1981-028AD','deb Kosmos-1260','',52551,'Reentered','1982-08-14 00:00:00','1982-06-17 00:00:00',95.73,423,685,64.94,'Intermediate' from dual union all
select 13223,'013223','1981-028AE','deb Kosmos-1260','',52551,'Reentered','1982-09-09 00:00:00','1982-09-04 00:00:00',92.2,333,432,65.01,'Intermediate' from dual union all
select 13224,'013224','1981-028AF','deb Kosmos-1260','',52551,'Reentered','1982-12-31 00:00:00','1982-12-21 00:00:00',91.6,322,385,64.99,'Intermediate' from dual union all
select 13225,'013225','1981-028AG','deb Kosmos-1260','',52551,'Reentered','1982-08-09 00:00:00','1982-08-02 00:00:00',96.24,450,706,65.03,'Intermediate' from dual union all
select 13226,'013226','1981-028AH','deb Kosmos-1260','',52551,'Reentered','1982-10-05 00:00:00','1982-08-01 00:00:00',96.73,488,717,64.99,'Intermediate' from dual union all
select 13227,'013227','1981-028AJ','deb Kosmos-1260','',52551,'Reentered','1982-06-02 00:00:00','1982-06-01 00:00:00',92.84,300,528,64.89,'Intermediate' from dual union all
select 13228,'013228','1981-028AK','deb Kosmos-1260','',52551,'Reentered','1982-11-04 00:00:00','1982-11-03 00:00:00',91.31,301,377,64.99,'Intermediate' from dual union all
select 13229,'013229','1981-028AL','deb Kosmos-1260','',52551,'Reentered','1989-02-16 00:00:00','1982-05-28 00:00:00',96.89,527,692,65.05,'Intermediate' from dual union all
select 1323,'001323','1965-017H','deb Ablestar','',11767,'Reentered','1965-12-23 00:00:00','1965-04-12 00:00:00',97.18,282,966,90,'Polar' from dual union all
select 13230,'013230','1981-028AM','deb Kosmos-1260','',52551,'Reentered','1982-06-17 00:00:00','1982-05-28 00:00:00',95.95,392,737,64.99,'Intermediate' from dual union all
select 13231,'013231','1981-028AN','deb Kosmos-1260','',52551,'Reentered','1982-08-19 00:00:00','1982-06-04 00:00:00',95.87,427,695,64.94,'Intermediate' from dual union all
select 13232,'013232','1981-028AP','deb Kosmos-1260','',52551,'Reentered','1982-10-15 00:00:00','1982-06-22 00:00:00',96.43,444,732,65.02,'Intermediate' from dual union all
select 13233,'013233','1981-028AQ','deb Kosmos-1260','',52551,'Reentered','1982-12-08 00:00:00','1982-12-08 00:00:00',90.12,265,295,64.99,'Intermediate' from dual union all
select 13234,'013234','1981-028AR','deb Kosmos-1260','',52551,'Reentered','1982-08-25 00:00:00','1982-08-08 00:00:00',95.61,424,673,65.04,'Intermediate' from dual union all
select 13235,'013235','1981-028AS','deb Kosmos-1260','',52551,'Reentered','1982-10-17 00:00:00','1982-10-13 00:00:00',93.15,367,491,65.02,'Intermediate' from dual union all
select 13236,'013236','1981-028AT','deb Kosmos-1260','',52551,'Reentered','1982-09-28 00:00:00','1982-06-24 00:00:00',96.29,446,716,65.06,'Intermediate' from dual union all
select 13237,'013237','1982-050A','Molniya-1','Molniya-1',54359,'Reentered','1992-11-19 00:00:00','1982-06-27 00:00:00',717.76,642,39712,62.84,'Molniya' from dual;
insert into SATELLITE
select 13238,'013238','1982-050B','11S510','11S510',54359,'Reentered','1982-07-12 00:00:00','1982-06-04 00:00:00',92.7,211,603,62.84,'Intermediate' from dual union all
select 13239,'013239','1982-050C','BOZ','BOZ',54359,'Reentered','1982-06-28 00:00:00','1982-06-04 00:00:00',92.38,194,589,62.81,'Intermediate' from dual union all
select 1324,'001324','1965-030A','Molniya-1','Molniya-1 11F67 No. 3',11780,'Reentered','1979-08-16 00:00:00','1965-05-26 00:00:00',720.5,533,39956,65.25,'GEO Transfer' from dual union all
select 13240,'013240','1982-050D','deb Blok-I','',54359,'Reentered','1982-06-01 00:00:00','1982-06-01 00:00:00',92.87,212,619,62.84,'Intermediate' from dual union all
select 13241,'013241','1982-051A','Kosmos-1371','Strela-2M',54360,'In Earth orbit','','1992-09-26 00:00:00',100.74,784,803,74.04,'Intermediate' from dual union all
select 13242,'013242','1982-051B','S3M','S3M',54360,'In Earth orbit','','1991-03-28 00:00:00',100.57,766,806,74.04,'Intermediate' from dual union all
select 13243,'013243','1982-052A','Kosmos-1372','US-A reactor',54361,'In Earth orbit','','1982-07-01 00:00:00',89.63,250,262,64.99,'Intermediate' from dual union all
select 13244,'013244','1982-053A','Kosmos-1373','Zenit-6',54362,'Landed','1982-06-16 00:00:00','1982-06-04 00:00:00',92.67,364,447,70.38,'Intermediate' from dual union all
select 13245,'013245','1982-053B','11S510','11S510',54362,'Reentered','1982-06-11 00:00:00','1982-06-04 00:00:00',89.78,204,324,70.39,'Intermediate' from dual union all
select 13246,'013246','1982-007F','Kosmos-1335 SS 3','ESO',54316,'Reentered','1982-08-17 00:00:00','1982-06-03 00:00:00',94.48,469,517,74.07,'Intermediate' from dual union all
select 13247,'013247','1982-007G','Kosmos-1335 SS 4','ESO',54316,'Reentered','1982-08-14 00:00:00','1982-06-03 00:00:00',94.4,468,511,74.04,'Intermediate' from dual union all
select 13248,'013248','1982-007H','Kosmos-1335 SS 5','ESO',54316,'Reentered','1982-08-11 00:00:00','1982-06-03 00:00:00',94.33,468,505,74.05,'Intermediate' from dual union all
select 13249,'013249','1982-007J','Kosmos-1335 SS 6','ESO',54316,'Reentered','1982-08-01 00:00:00','1982-06-03 00:00:00',94.2,464,496,74.06,'Intermediate' from dual union all
select 1325,'001325','1965-030B','BOZ','BOZ',11780,'Reentered','1965-07-20 00:00:00','1965-05-24 00:00:00',93.46,187,701,64.83,'Intermediate' from dual union all
select 13250,'013250','1982-007K','Kosmos-1335 SS 7','ESO',54316,'Reentered','1982-07-29 00:00:00','1982-06-03 00:00:00',94.13,465,488,74.04,'Intermediate' from dual union all
select 13251,'013251','1982-007L','Kosmos-1335 SS 8','ESO',54316,'Reentered','1982-07-26 00:00:00','1982-06-02 00:00:00',94.07,462,485,74.06,'Intermediate' from dual union all
select 13252,'013252','1982-007M','Kosmos-1335 SS 9','ESO',54316,'Reentered','1982-07-24 00:00:00','1982-06-03 00:00:00',93.96,457,480,74.03,'Intermediate' from dual union all
select 13253,'013253','1982-050E','Blok-L','Blok-ML No. 76727-453',54359,'Reentered','1993-04-02 00:00:00','1993-01-07 00:00:00',649.76,112,36832,64.19,'GEO Transfer' from dual union all
select 13254,'013254','1982-046C','Blenda','Sunshade',54355,'Reentered','1982-06-06 00:00:00','1982-06-03 00:00:00',88.89,217,223,70.39,'Intermediate' from dual union all
select 13255,'013255','1982-046D','KDU part','',54355,'Reentered','1982-06-06 00:00:00','1982-06-03 00:00:00',88.96,216,231,70.39,'Intermediate' from dual union all
select 13256,'013256','1982-046E','KDU','',54355,'Reentered','1982-06-09 00:00:00','1982-06-03 00:00:00',89.12,225,237,70.38,'Intermediate' from dual union all
select 13257,'013257','1982-054A','Kosmos-1374','BOR-4 No. 404',54363,'Landed','1982-06-03 00:00:00','1982-06-04 00:00:00',88.68,190,229,49.62,'Intermediate' from dual union all
select 13258,'013258','1982-054B','S3M','S3M',54363,'Reentered','1982-06-05 00:00:00','1982-06-04 00:00:00',88.38,167,221,50.67,'Intermediate' from dual union all
select 13259,'013259','1982-055A','Kosmos-1375','Lira',54364,'In Earth orbit','','1982-07-06 00:00:00',105.02,979,1012,65.84,'Intermediate' from dual union all
select 1326,'001326','1965-030C','8K78I','8K78I',11780,'Reentered','1965-07-02 00:00:00','1965-05-24 00:00:00',92.92,193,643,64.83,'Intermediate' from dual union all
select 13260,'013260','1982-055B','S3M','S3M',54364,'In Earth orbit','','1991-03-27 00:00:00',104.82,967,1005,65.83,'Intermediate' from dual union all
select 13261,'013261','1982-055C','deb Kosmos-1375','DS-P1-M part [RCS 0.36]',54364,'In Earth orbit','','1993-08-06 00:00:00',104.95,988,997,65.83,'Intermediate' from dual union all
select 13262,'013262','1982-020G','SOZ','SOZ',54329,'Reentered','1982-06-08 00:00:00','1982-06-07 00:00:00',125.01,102,3705,46.93,'Medium' from dual union all
select 13263,'013263','1982-056A','Kosmos-1376','Resurs-F1 17F41 No. 18',54365,'Landed','1982-06-22 00:00:00','1982-06-10 00:00:00',89.8,259,270,82.34,'Intermediate' from dual union all
select 13264,'013264','1982-056B','11S510','11S510',54365,'Reentered','1982-06-12 00:00:00','1982-06-10 00:00:00',88.68,199,220,82.35,'Intermediate' from dual union all
select 13265,'013265','1982-057A','Kosmos-1377','Oktan No. 215',54366,'Landed','1982-07-22 00:00:00','1982-06-15 00:00:00',88.72,163,260,64.9,'Intermediate' from dual union all
select 13266,'013266','1982-057B','11S510','11S510',54366,'Reentered','1982-06-12 00:00:00','1982-06-10 00:00:00',89.04,163,292,64.91,'Intermediate' from dual union all
select 13267,'013267','1982-048D','KDU','KDU',54357,'Reentered','1982-06-27 00:00:00','1982-06-08 00:00:00',89.84,252,281,82.32,'Intermediate' from dual union all
select 13268,'013268','1982-048E','Blenda','Sunshade',54357,'Reentered','1982-06-15 00:00:00','1982-06-09 00:00:00',89.64,239,275,82.31,'Intermediate' from dual union all
select 13269,'013269','1982-058A','Westar 5','Westar 5',54367,'In Earth orbit','','1982-07-14 00:00:00',1428.76,35566,35720,.05,'Drift GEO' from dual union all
select 1327,'001327','1965-031A','OPS 4983','GAMBIT SV 967',11781,'Reentered','1965-05-03 00:00:00','1965-05-01 00:00:00',88.88,180,259,95.6,'Sun-Synch' from dual union all
select 13270,'013270','1982-033E','Garbage bag','',54342,'Reentered','1982-06-24 00:00:00','1982-06-10 00:00:00',90.47,284,312,51.64,'Intermediate' from dual union all
select 13271,'013271','1982-059A','Kosmos-1378','Tselina-D',54368,'In Earth orbit','','1982-07-10 00:00:00',97.68,633,663,82.51,'Intermediate' from dual union all
select 13272,'013272','1982-059B','S5M','S5M',54368,'In Earth orbit','','1991-03-28 00:00:00',97.44,623,649,82.51,'Intermediate' from dual union all
select 13273,'013273','1978-118E','SOZ','SOZ',45313,'Reentered','1979-12-04 00:00:00','1979-12-03 00:00:00',152.3,108,6029,46.54,'Medium' from dual union all
select 13274,'013274','1982-033F','Garbage bag','',54342,'Reentered','1982-06-27 00:00:00','1982-06-14 00:00:00',90.5,284,315,51.64,'Intermediate' from dual union all
select 13275,'013275','1982-033G','Garbage bag','',54342,'Reentered','1982-06-27 00:00:00','1982-06-14 00:00:00',90.43,284,308,51.62,'Intermediate' from dual union all
select 13276,'013276','1982-053C','Blenda','Sunshade',54362,'Reentered','1982-07-18 00:00:00','1982-06-16 00:00:00',91.97,357,386,70.38,'Intermediate' from dual union all
select 13277,'013277','1982-053D','KDU part','KDU part',54362,'Reentered','1982-08-10 00:00:00','1982-07-02 00:00:00',91.49,326,370,70.38,'Intermediate' from dual union all
select 13278,'013278','1982-053E','KDU','KDU',54362,'Reentered','1982-11-04 00:00:00','1982-07-02 00:00:00',91.85,352,378,70.4,'Intermediate' from dual union all
select 13279,'013279','1982-053F','KDU part','KDU part',54362,'Reentered','1982-08-05 00:00:00','1982-07-02 00:00:00',93.49,347,544,70.38,'Intermediate' from dual union all
select 1328,'001328','1965-032A','Explorer 27','Beacon Explorer C',11782,'In Earth orbit','','1965-05-30 00:00:00',107.82,936,1316,41.18,'Intermediate' from dual union all
select 13280,'013280','1982-054C','deb BOR-4','',54363,'Reentered','1982-06-03 00:00:00','1982-06-03 00:00:00',88.68,190,229,49.62,'Intermediate' from dual union all
select 13281,'013281','1982-060A','Kosmos-1379','IS-P Uran',54369,'Deorbited','1982-06-18 00:00:00','1982-06-18 00:00:00',91.35,140,542,65.09,'Intermediate' from dual union all
select 13282,'013282','1982-061A','Kosmos-1380','Parus',54370,'Reentered','1982-06-27 00:00:00','1982-06-20 00:00:00',92.64,143,665,82.9,'Intermediate' from dual union all
select 13283,'013283','1982-062A','Kosmos-1381','Zenit-6',54371,'Landed','1982-07-01 00:00:00','1982-06-20 00:00:00',92.67,360,451,70.37,'Intermediate' from dual union all
select 13284,'013284','1982-060B','11S692','11S692',54369,'Reentered','1982-06-18 00:00:00','1982-06-18 00:00:00',90.04,143,410,65.12,'Intermediate' from dual union all
select 13285,'013285','1982-062B','11S510','11S510',54371,'Reentered','1982-06-28 00:00:00','1982-06-20 00:00:00',90,202,347,70.38,'Intermediate' from dual union all
select 13286,'013286','1982-061B','S3M','S3M',54370,'Reentered','1982-06-25 00:00:00','1982-06-21 00:00:00',91.88,138,596,82.92,'Intermediate' from dual union all
select 13287,'013287','1982-033H','Garbage bag','',54342,'Reentered','1982-07-02 00:00:00','1982-06-21 00:00:00',90.47,283,313,51.64,'Intermediate' from dual union all
select 13288,'013288','1982-056C','KDU','KDU',54365,'Reentered','1982-07-14 00:00:00','1982-06-22 00:00:00',89.91,258,281,82.35,'Intermediate' from dual union all
select 13289,'013289','1982-056D','Blenda','Sunshade',54365,'Reentered','1982-06-29 00:00:00','1982-06-22 00:00:00',89.77,249,278,82.36,'Intermediate' from dual union all
select 1329,'001329','1965-031B','PUNDIT IV','P-11 No. 4401',11781,'Reentered','1969-10-31 00:00:00','1965-05-29 00:00:00',95.11,500,547,95.21,'Sun-Synch' from dual union all
select 13290,'013290','1982-056E','KDU part','KDU part',54365,'Reentered','1982-06-22 00:00:00','1982-06-22 00:00:00',89.39,216,273,82.45,'Intermediate' from dual union all
select 13291,'013291','1982-033J','Garbage bag','',54342,'Reentered','1982-07-09 00:00:00','1982-06-23 00:00:00',90.36,281,304,51.62,'Intermediate' from dual union all
select 13292,'013292','1982-063A','Soyuz T-6','Soyuz 7K-ST No. 9L',54372,'Landed','1982-07-02 00:00:00','1982-06-26 00:00:00',90.38,282,305,51.63,'Intermediate' from dual union all
select 13293,'013293','1982-063B','11S510','11S510',54372,'Reentered','1982-06-26 00:00:00','1982-06-25 00:00:00',88.29,181,199,51.61,'Intermediate' from dual union all
select 13294,'013294','1982-058B','PAM-D','Star 48  S/N 10012-8',54367,'Reentered','1990-03-22 00:00:00','1982-07-14 00:00:00',619.15,193,35178,27.49,'GEO Transfer' from dual union all
select 13295,'013295','1982-064A','Kosmos-1382','Oko',54373,'In Earth orbit','','1982-07-25 00:00:00',717.82,619,39738,62.84,'Molniya' from dual union all
select 13296,'013296','1982-064B','11S510','11S510',54373,'Reentered','1982-08-07 00:00:00','1982-07-02 00:00:00',92.33,214,564,62.8,'Intermediate' from dual union all
select 13297,'013297','1982-064C','BOZ','BOZ',54373,'Reentered','1982-07-15 00:00:00','1982-07-02 00:00:00',91.64,180,530,62.83,'Intermediate' from dual union all
select 13298,'013298','1982-064D','Blok-L','Blok-2BL No. 77323-647',54373,'In Earth orbit','','1982-07-25 00:00:00',708.37,621,39269,62.84,'Molniya' from dual union all
select 13299,'013299','1982-033K','Garbage bag','',54342,'Reentered','1982-07-12 00:00:00','1982-07-01 00:00:00',90.08,261,296,51.61,'Intermediate' from dual union all
select 133,'000133','61 Omicron 18','Transit 4A despin weight','',7273,'In Earth orbit','','1961-08-27 00:00:00',103.61,876,983,67.06,'Intermediate' from dual union all
select 1330,'001330','1965-033A','OPS 5023','CORONA J-4',11783,'Reentered','1965-05-26 00:00:00','1965-04-29 00:00:00',91.01,182,467,85.03,'Polar' from dual union all
select 13300,'013300','1982-065A','Columbia (STS-4)','OV-102',54374,'Landed','1982-07-04 00:00:00','1982-06-29 00:00:00',90.5,295,304,28.52,'Intermediate' from dual union all
select 13301,'013301','1982-066A','Kosmos-1383','Nadezhda 11F643N No. 514',54375,'In Earth orbit','','1982-08-10 00:00:00',105.29,987,1029,82.93,'Intermediate' from dual union all
select 13302,'013302','1982-066B','S3M','S3M',54375,'In Earth orbit','','1991-03-25 00:00:00',105.14,971,1031,82.93,'Intermediate' from dual union all
select 13303,'013303','1982-067A','Kosmos-1384','Feniks No. 954',54376,'Landed','1982-07-30 00:00:00','1982-07-07 00:00:00',89.63,169,344,67.14,'Intermediate' from dual union all
select 13304,'013304','1982-067B','11S510','11S510',54376,'Reentered','1982-07-05 00:00:00','1982-07-02 00:00:00',89.2,164,305,67.14,'Intermediate' from dual union all
select 13305,'013305','1982-062C','Blenda','Sunshade',54371,'Reentered','1982-08-11 00:00:00','1982-07-02 00:00:00',92.49,356,438,70.38,'Intermediate' from dual union all
select 13306,'013306','1982-062D','KDU part','KDU part',54371,'Reentered','1982-09-10 00:00:00','1982-07-19 00:00:00',92.06,334,417,70.37,'Intermediate' from dual union all
select 13307,'013307','1982-062E','KDU part','KDU part',54371,'Reentered','1982-07-13 00:00:00','1982-07-02 00:00:00',92.7,355,459,70.38,'Intermediate' from dual union all
select 13308,'013308','1980-089C','deb Kosmos-1220','',50299,'Reentered','1987-10-19 00:00:00','1982-07-01 00:00:00',99.81,620,879,65.09,'Intermediate' from dual union all
select 13309,'013309','1980-089D','deb Kosmos-1220','',50299,'Reentered','1984-08-25 00:00:00','1984-08-22 00:00:00',90.89,307,329,65.13,'Intermediate' from dual union all
select 1331,'001331','1965-031C','deb Agena D','',11781,'Reentered','1965-04-30 00:00:00','1965-04-29 00:00:00',88.59,160,250,95.6,'Sun-Synch' from dual union all
select 13310,'013310','1980-089E','deb Kosmos-1220','',50299,'Reentered','1987-11-12 00:00:00','1987-11-12 00:00:00',89.07,226,230,64.84,'Intermediate' from dual union all
select 13311,'013311','1980-089F','deb Kosmos-1220','',50299,'Reentered','1982-11-21 00:00:00','1982-11-15 00:00:00',94.9,489,538,65.34,'Intermediate' from dual union all
select 13312,'013312','1980-089G','deb Kosmos-1220','',50299,'Reentered','1984-03-05 00:00:00','1984-03-01 00:00:00',91.62,350,358,64.96,'Intermediate' from dual union all
select 13313,'013313','1980-089H','deb Kosmos-1220','',50299,'Reentered','1989-01-07 00:00:00','1982-07-01 00:00:00',99.11,557,876,65.16,'Intermediate' from dual union all
select 13314,'013314','1980-089J','deb Kosmos-1220','',50299,'Reentered','1982-08-12 00:00:00','1982-07-05 00:00:00',97.53,430,851,64.18,'Intermediate' from dual union all
select 13315,'013315','1980-089K','deb Kosmos-1220','RCS 0.8m2',50299,'Reentered','1983-11-04 00:00:00','1983-11-01 00:00:00',90.85,296,337,64.74,'Intermediate' from dual union all
select 13316,'013316','1980-089L','deb Kosmos-1220','',50299,'Reentered','1988-04-05 00:00:00','1988-04-04 00:00:00',90.08,259,297,64.94,'Intermediate' from dual union all
select 13317,'013317','1980-089M','deb Kosmos-1220','',50299,'Reentered','1983-03-28 00:00:00','1982-06-21 00:00:00',99.05,545,882,65,'Intermediate' from dual union all
select 13318,'013318','1980-089N','deb Kosmos-1220','',50299,'Reentered','1989-09-05 00:00:00','1982-07-01 00:00:00',99.24,569,876,64.97,'Intermediate' from dual union all
select 13319,'013319','1980-089P','deb Kosmos-1220','',50299,'Reentered','1990-02-13 00:00:00','1990-02-13 00:00:00',89.08,226,232,64.96,'Intermediate' from dual union all
select 1332,'001332','1965-031D','deb Agena D','',11781,'Reentered','1965-04-30 00:00:00','1965-04-29 00:00:00',88.59,160,250,95.6,'Sun-Synch' from dual union all
select 13320,'013320','1980-089Q','deb Kosmos-1220','',50299,'Reentered','1985-09-03 00:00:00','1985-08-31 00:00:00',89.56,242,264,64.9,'Intermediate' from dual union all
select 13321,'013321','1980-089R','deb Kosmos-1220','',50299,'Reentered','1982-10-28 00:00:00','1983-06-07 00:00:00',97.91,534,784,64.99,'Intermediate' from dual union all
select 13322,'013322','1980-089S','deb Kosmos-1220','',50299,'Reentered','1988-12-19 00:00:00','1988-12-19 00:00:00',88.78,204,224,64.89,'Intermediate' from dual union all
select 13323,'013323','1980-089T','deb Kosmos-1220','',50299,'Reentered','1984-02-17 00:00:00','1982-07-02 00:00:00',99.28,567,882,64.99,'Intermediate' from dual union all
select 13324,'013324','1980-089U','deb Kosmos-1220','',50299,'Reentered','1989-03-28 00:00:00','1989-07-06 00:00:00',88.33,189,195,51.61,'Intermediate' from dual union all
select 13325,'013325','1980-089V','deb Kosmos-1220','',50299,'Reentered','1984-04-25 00:00:00','1984-04-22 00:00:00',90.5,289,310,64.95,'Intermediate' from dual union all
select 13326,'013326','1980-089W','deb Kosmos-1220','',50299,'Reentered','1988-01-08 00:00:00','1988-11-25 00:00:00',90.58,236,370,62.79,'Intermediate' from dual union all
select 13327,'013327','1980-089X','deb Kosmos-1220','[No TLE]',50299,'Reentered','1982-07-23 00:00:00','1982-06-30 00:00:00',98.9,553,860,65,'Intermediate' from dual;
insert into SATELLITE
select 13328,'013328','1980-089Y','deb Kosmos-1220','',50299,'Reentered','1989-01-11 00:00:00','1989-08-08 00:00:00',89.47,230,267,82.54,'Intermediate' from dual union all
select 13329,'013329','1980-089Z','deb Kosmos-1220','',50299,'Reentered','1982-07-13 00:00:00','1982-07-09 00:00:00',95.45,424,657,64.32,'Intermediate' from dual union all
select 1333,'001333','1965-031E','deb Agena D','',11781,'Reentered','1965-05-04 00:00:00','1965-04-29 00:00:00',88.8,167,264,95.55,'Sun-Synch' from dual union all
select 13330,'013330','1980-089AA','deb Kosmos-1220','',50299,'Reentered','1983-08-12 00:00:00','1983-08-05 00:00:00',92.73,366,450,64.97,'Intermediate' from dual union all
select 13331,'013331','1980-089AB','deb Kosmos-1220','',50299,'Reentered','1990-04-28 00:00:00','1990-04-27 00:00:00',89.76,254,272,64.94,'Intermediate' from dual union all
select 13332,'013332','1982-033L','Soyuz T-6 BO','7K-ST No. 9L BO',54342,'Reentered','1982-08-11 00:00:00','1982-07-02 00:00:00',90.47,282,313,51.63,'Intermediate' from dual union all
select 13333,'013333','1980-089AC','deb Kosmos-1220','',50299,'Reentered','1989-01-16 00:00:00','1989-10-08 00:00:00',100.82,740,855,99.43,'Sun-Synch' from dual union all
select 13334,'013334','1980-089AD','deb Kosmos-1220','',50299,'Reentered','1983-04-10 00:00:00','1983-04-05 00:00:00',91.81,331,396,65.05,'Intermediate' from dual union all
select 13335,'013335','1980-089AE','deb Kosmos-1220','',50299,'Reentered','1983-12-01 00:00:00','1983-11-29 00:00:00',90.59,299,308,64.93,'Intermediate' from dual union all
select 13336,'013336','1980-089AF','deb Kosmos-1220','',50299,'Reentered','1986-03-15 00:00:00','1982-07-02 00:00:00',99.27,567,881,65,'Intermediate' from dual union all
select 13337,'013337','1980-089AG','deb Kosmos-1220','',50299,'Reentered','1982-10-30 00:00:00','1982-08-17 00:00:00',98.63,574,813,65.42,'Intermediate' from dual union all
select 13338,'013338','1980-089AH','deb Kosmos-1220','',50299,'Reentered','1983-09-10 00:00:00','1983-09-10 00:00:00',88.72,198,224,64.86,'Intermediate' from dual union all
select 13339,'013339','1980-089AJ','deb Kosmos-1220','',50299,'Reentered','1985-08-17 00:00:00','1985-08-16 00:00:00',89.5,233,267,64.96,'Intermediate' from dual union all
select 1334,'001334','1965-031F','deb Agena D','',11781,'Reentered','1965-05-08 00:00:00','1965-05-01 00:00:00',88.81,153,278,95.59,'Sun-Synch' from dual union all
select 13340,'013340','1980-089AK','deb Kosmos-1220','',50299,'Reentered','1991-02-06 00:00:00','1991-02-06 00:00:00',87.95,166,180,64.95,'Intermediate' from dual union all
select 13341,'013341','1980-089AL','deb Kosmos-1220','',50299,'Reentered','1988-12-07 00:00:00','1988-12-06 00:00:00',90.35,287,296,64.94,'Intermediate' from dual union all
select 13342,'013342','1980-089AM','deb Kosmos-1220','',50299,'Reentered','1983-04-27 00:00:00','1983-04-17 00:00:00',92.76,377,443,64.98,'Intermediate' from dual union all
select 13343,'013343','1980-089AN','deb Kosmos-1220','',50299,'Reentered','1989-02-05 00:00:00','1989-10-08 00:00:00',775.68,279,42895,18.12,'Highly Ellip' from dual union all
select 13344,'013344','1980-089AP','deb Kosmos-1220','',50299,'Reentered','1988-08-29 00:00:00','1988-08-29 00:00:00',89.59,244,264,64.95,'Intermediate' from dual union all
select 13345,'013345','1982-068A','Kosmos-1385','Zenit-6',54377,'Landed','1982-07-20 00:00:00','1982-07-08 00:00:00',91.47,305,389,82.33,'Intermediate' from dual union all
select 13346,'013346','1982-068B','11S510','11S510',54377,'Reentered','1982-07-08 00:00:00','1982-07-07 00:00:00',88.14,167,198,82.32,'Intermediate' from dual union all
select 13347,'013347','1980-089AQ','deb Kosmos-1220','',50299,'Reentered','1982-12-22 00:00:00','1982-10-14 00:00:00',97.3,492,767,64.97,'Intermediate' from dual union all
select 13348,'013348','1980-089AR','deb Kosmos-1220','',50299,'Reentered','1982-10-06 00:00:00','1982-08-21 00:00:00',98.96,543,875,64.99,'Intermediate' from dual union all
select 13349,'013349','1980-089AS','deb Kosmos-1220 solar panel','RCS 16.89m2',50299,'Reentered','1982-12-15 00:00:00','1982-10-14 00:00:00',97.85,555,757,65.09,'Intermediate' from dual union all
select 1335,'001335','1965-020E','S3 large debris','RCS 0.39',11770,'In Earth orbit','','1965-04-26 00:00:00',112.17,593,2060,56.11,'Intermediate' from dual union all
select 13350,'013350','1980-089AT','deb Kosmos-1220','',50299,'Reentered','1983-07-26 00:00:00','1983-07-24 00:00:00',90.81,292,337,65.02,'Intermediate' from dual union all
select 13351,'013351','1980-089AU','deb Kosmos-1220','',50299,'Reentered','1984-05-09 00:00:00','1984-04-30 00:00:00',91.42,337,351,64.93,'Intermediate' from dual union all
select 13352,'013352','1980-089AV','deb Kosmos-1220','',50299,'Reentered','1987-10-10 00:00:00','1987-10-10 00:00:00',88.61,201,211,64.95,'Intermediate' from dual union all
select 13353,'013353','1982-069A','Kosmos-1386','Parus',54378,'In Earth orbit','','1982-08-09 00:00:00',104.71,953,1009,82.96,'Intermediate' from dual union all
select 13354,'013354','1982-069B','S3M','S3M',54378,'In Earth orbit','','1991-03-25 00:00:00',104.52,933,1011,82.95,'Intermediate' from dual union all
select 13355,'013355','1980-089AW','deb Kosmos-1220','',50299,'Reentered','1983-05-07 00:00:00','1983-05-06 00:00:00',90.16,261,303,64.98,'Intermediate' from dual union all
select 13356,'013356','1980-089AX','deb Kosmos-1220','',50299,'Reentered','1988-07-16 00:00:00','1988-07-16 00:00:00',89.06,213,243,64.93,'Intermediate' from dual union all
select 13357,'013357','1980-089AY','deb Kosmos-1220','',50299,'Reentered','1983-01-23 00:00:00','1982-07-09 00:00:00',99.26,564,883,64.99,'Intermediate' from dual union all
select 13358,'013358','1980-089AZ','deb Kosmos-1220','',50299,'Reentered','1989-07-06 00:00:00','1989-07-06 00:00:00',88.6,195,215,64.93,'Intermediate' from dual union all
select 13359,'013359','1980-089BA','deb Kosmos-1220','',50299,'Reentered','1990-02-02 00:00:00','1990-02-02 00:00:00',87.89,164,177,64.92,'Intermediate' from dual union all
select 1336,'001336','1965-020F','deb r/b S3','',11770,'Reentered','1968-07-24 00:00:00','1965-04-26 00:00:00',106.65,335,1808,56.01,'Intermediate' from dual union all
select 13360,'013360','1964-026G','deb Transit VC','',10312,'Reentered','1988-01-11 00:00:00','1982-07-15 00:00:00',102.56,837,923,90.49,'Polar' from dual union all
select 13361,'013361','1982-070A','Progress-14','Progress 7K-TG No. 117',54379,'Deorbited','1982-08-13 00:00:00','1982-07-17 00:00:00',90.75,303,320,51.63,'Intermediate' from dual union all
select 13362,'013362','1982-070B','11S510','11S510',54379,'Reentered','1982-07-12 00:00:00','1982-07-11 00:00:00',88.47,179,219,51.62,'Intermediate' from dual union all
select 13363,'013363','1982-062F','KDU','KDU',54371,'Reentered','1983-02-24 00:00:00','1982-07-19 00:00:00',92.43,358,430,70.37,'Intermediate' from dual union all
select 13364,'013364','61 Omicron 257','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-02 00:00:00',105.53,933,1106,66.27,'Intermediate' from dual union all
select 13365,'013365','1982-071A','Kosmos-1387','Fram',54380,'Landed','1982-07-26 00:00:00','1982-07-15 00:00:00',88.96,208,239,82.34,'Intermediate' from dual union all
select 13366,'013366','1982-071B','11S510','11S510',54380,'Reentered','1982-07-17 00:00:00','1982-07-15 00:00:00',88.28,185,194,82.34,'Intermediate' from dual union all
select 13367,'013367','1982-072A','Landsat 4','Landsat D',54381,'In Earth orbit','','1982-08-17 00:00:00',98.81,700,704,98.25,'Sun-Synch' from dual union all
select 13368,'013368','1982-072B','Delta 163','Delta SSPS AJ10-118K',54381,'Reentered','1984-07-12 00:00:00','1982-08-16 00:00:00',95.06,356,687,101.77,'Retrograde' from dual union all
select 13369,'013369','1981-089C','deb Kosmos-1306','',52612,'Reentered','1982-12-12 00:00:00','1981-10-15 00:00:00',93.27,428,442,64.96,'Intermediate' from dual union all
select 1337,'001337','1965-020G','deb r/b S3','',11770,'Reentered','1966-09-29 00:00:00','1965-04-26 00:00:00',105.9,316,1757,56.07,'Intermediate' from dual union all
select 13370,'013370','1982-068C','Blenda','Sunshade',54377,'Reentered','1982-07-20 00:00:00','1982-07-20 00:00:00',90.79,285,342,82.33,'Intermediate' from dual union all
select 13371,'013371','1982-068D','KDU part','KDU part',54377,'Reentered','1982-08-17 00:00:00','1982-07-21 00:00:00',91.25,288,384,82.33,'Intermediate' from dual union all
select 13372,'013372','1982-068E','KDU','KDU',54377,'Reentered','1982-10-18 00:00:00','1982-08-06 00:00:00',91.26,297,376,82.32,'Intermediate' from dual union all
select 13373,'013373','1982-068F','KDU part','KDU part',54377,'Reentered','1982-08-09 00:00:00','1982-07-21 00:00:00',91.41,301,387,82.33,'Intermediate' from dual union all
select 13374,'013374','1982-068G','KDU part','KDU part',54377,'Reentered','1982-08-19 00:00:00','1982-07-20 00:00:00',93.54,345,550,82.33,'Intermediate' from dual union all
select 13375,'013375','1982-073A','Kosmos-1388','Strela-1M',54382,'In Earth orbit','','1982-08-30 00:00:00',114.52,1393,1474,74.02,'Intermediate' from dual union all
select 13376,'013376','1982-073B','Kosmos-1389','Strela-1M',54382,'In Earth orbit','','1992-09-27 00:00:00',114.7,1409,1474,74.01,'Intermediate' from dual union all
select 13377,'013377','1982-073C','Kosmos-1390','Strela-1M',54382,'In Earth orbit','','1992-08-27 00:00:00',114.89,1426,1475,74.02,'Intermediate' from dual union all
select 13378,'013378','1982-073D','Kosmos-1391','Strela-1M',54382,'In Earth orbit','','1992-09-27 00:00:00',115.06,1442,1475,74.01,'Intermediate' from dual union all
select 13379,'013379','1982-073E','Kosmos-1392','Strela-1M',54382,'In Earth orbit','','1992-09-26 00:00:00',115.25,1459,1475,74.02,'Intermediate' from dual union all
select 1338,'001338','1965-020H','deb r/b S3','',11770,'Reentered','1967-04-30 00:00:00','1965-04-26 00:00:00',106.13,329,1766,56.21,'Intermediate' from dual union all
select 13380,'013380','1982-073F','Kosmos-1393','Strela-1M',54382,'In Earth orbit','','1992-09-26 00:00:00',115.45,1470,1482,74.02,'Intermediate' from dual union all
select 13381,'013381','1982-073G','Kosmos-1394','Strela-1M',54382,'In Earth orbit','','1992-09-25 00:00:00',115.64,1474,1495,74.02,'Intermediate' from dual union all
select 13382,'013382','1982-073H','Kosmos-1395','Strela-1M',54382,'In Earth orbit','','1992-09-26 00:00:00',115.87,1474,1516,74.01,'Intermediate' from dual union all
select 13383,'013383','1982-074A','Molniya-1','Molniya-1 11F658 No. 62',54383,'Reentered','1992-10-10 00:00:00','1982-08-21 00:00:00',717.68,606,39744,62.99,'Molniya' from dual union all
select 13384,'013384','1982-074B','11S510','11S510',54383,'Reentered','1982-09-20 00:00:00','1982-08-20 00:00:00',91.17,189,475,62.96,'Intermediate' from dual union all
select 13385,'013385','1982-074C','BOZ','BOZ',54383,'Reentered','1982-09-03 00:00:00','1982-07-28 00:00:00',93.16,232,627,62.96,'Intermediate' from dual union all
select 13386,'013386','1982-073J','S3M','S3M',54382,'In Earth orbit','','1992-05-07 00:00:00',117.94,1465,1712,74,'Intermediate' from dual union all
select 13387,'013387','1981-089D','Kosmos-1306 main debris','US-PM1',52612,'Reentered','1982-10-11 00:00:00','1982-10-11 00:00:00',88,167,184,64.92,'Intermediate' from dual union all
select 13388,'013388','1982-071C','KDU','KDU',54380,'Reentered','1982-07-31 00:00:00','1982-07-26 00:00:00',88.61,198,214,82.33,'Intermediate' from dual union all
select 13389,'013389','1982-071D','Blenda','Sunshade',54380,'Reentered','1982-07-27 00:00:00','1982-07-26 00:00:00',88.44,193,201,82.33,'Intermediate' from dual union all
select 1339,'001339','1965-020J','deb r/b S3','',11770,'Reentered','1969-10-14 00:00:00','1965-05-04 00:00:00',107.54,464,1761,55.97,'Intermediate' from dual union all
select 13390,'013390','1982-074D','Blok-L','Blok-ML No. 160',54383,'Reentered','1995-03-14 00:00:00','1982-08-23 00:00:00',698.79,592,38822,63.03,'Molniya' from dual union all
select 13391,'013391','1982-075A','Kosmos-1396','Zenit-6',54384,'Landed','1982-08-10 00:00:00','1982-07-29 00:00:00',89.65,228,287,72.86,'Intermediate' from dual union all
select 13392,'013392','1982-075B','11S510','11S510',54384,'Reentered','1982-08-01 00:00:00','1982-07-29 00:00:00',89.04,190,265,72.85,'Intermediate' from dual union all
select 13393,'013393','1981-089E','deb Kosmos-1306','',52612,'Reentered','1983-05-01 00:00:00','1983-05-01 00:00:00',89.61,237,274,64.93,'Intermediate' from dual union all
select 13394,'013394','1982-076A','Kosmos-1397','Romb',54385,'Reentered','1983-05-18 00:00:00','1982-08-28 00:00:00',93.31,342,532,50.69,'Intermediate' from dual union all
select 13395,'013395','1982-076B','S3M','S3M',54385,'Reentered','1983-04-21 00:00:00','1982-08-28 00:00:00',93.21,341,523,50.69,'Intermediate' from dual union all
select 13396,'013396','1982-077A','Kosmos-1398','Orion',54386,'Landed','1982-08-13 00:00:00','1982-08-05 00:00:00',88.94,213,232,82.35,'Intermediate' from dual union all
select 13397,'013397','1982-077B','11S510','11S510',54386,'Reentered','1982-08-07 00:00:00','1982-08-06 00:00:00',88.02,166,186,82.35,'Intermediate' from dual union all
select 13398,'013398','1982-033M','EVA debris','',54342,'Reentered','1982-08-21 00:00:00','1982-08-03 00:00:00',90.41,288,302,51.64,'Intermediate' from dual union all
select 13399,'013399','1982-078A','Kosmos-1399','Oktan No. 216',54387,'Landed','1982-09-16 00:00:00','1982-08-11 00:00:00',89.01,162,290,64.89,'Intermediate' from dual union all
select 134,'000134','61 Omicron 19','deb r/b Ablestar','',7273,'In Earth orbit','','1964-04-14 00:00:00',103.64,861,1000,66.9,'Intermediate' from dual union all
select 1340,'001340','1965-020K','deb r/b S3','',11770,'Reentered','1968-05-30 00:00:00','1965-04-26 00:00:00',105.6,247,1797,56.07,'Intermediate' from dual union all
select 13400,'013400','1982-078B','11S510','11S510',54387,'Reentered','1982-08-08 00:00:00','1982-08-06 00:00:00',89.15,165,299,64.9,'Intermediate' from dual union all
select 13401,'013401','1982-078C','Kosmos-1399 part','Oktan part',54387,'Reentered','1982-08-08 00:00:00','1982-08-04 00:00:00',89.7,170,350,64.9,'Intermediate' from dual union all
select 13402,'013402','1982-079A','Kosmos-1400','Tselina-D',54388,'Reentered','2014-09-13 00:00:00','1982-09-04 00:00:00',97.49,629,649,81.16,'Intermediate' from dual union all
select 13403,'013403','1982-079B','8A92ME','8A92ME',54388,'In Earth orbit','','1991-03-28 00:00:00',97.15,586,659,81.16,'Intermediate' from dual union all
select 13404,'013404','1981-089F','deb Kosmos-1306','',52612,'Reentered','1989-03-07 00:00:00','1989-03-06 00:00:00',88.95,217,229,64.93,'Intermediate' from dual union all
select 13405,'013405','1981-100E','SME duplicate (mistag)','[Radar error]',52623,'Spurious','1982-10-19 00:00:00','1982-08-24 00:00:00',95.11,522,526,97.49,'Sun-Synch' from dual union all
select 13406,'013406','1982-075C','KDU','KDU',54384,'Reentered','1982-08-19 00:00:00','1982-08-10 00:00:00',89.12,220,241,72.85,'Intermediate' from dual union all
select 13407,'013407','1982-075D','Blenda','Sunshade',54384,'Reentered','1982-08-12 00:00:00','1982-08-10 00:00:00',88.95,213,233,72.85,'Intermediate' from dual union all
select 13408,'013408','1981-089G','deb Kosmos-1306','',52612,'Reentered','1982-08-19 00:00:00','1982-08-19 00:00:00',88.65,199,217,64.9,'Intermediate' from dual union all
select 13409,'013409','1982-033N','EVA debris','',54342,'Reentered','1982-08-25 00:00:00','1982-08-10 00:00:00',90.47,291,305,51.64,'Intermediate' from dual union all
select 1341,'001341','1965-020L','deb r/b S3','',11770,'Reentered','1969-02-13 00:00:00','1965-05-03 00:00:00',106.73,342,1808,56.04,'Intermediate' from dual union all
select 13410,'013410','1982-033P','EVA debris','',54342,'Reentered','1982-08-13 00:00:00','1982-08-11 00:00:00',90.2,271,297,51.6,'Intermediate' from dual union all
select 13411,'013411','1982-052B','Kosmos-1372 DU','US-A DU',54361,'Reentered','1982-09-09 00:00:00','1982-08-11 00:00:00',89.54,242,261,64.99,'Intermediate' from dual union all
select 13412,'013412','1982-052C','Kosmos-1372 RLS','US-A RLS',54361,'Reentered','1982-08-13 00:00:00','1982-08-11 00:00:00',89.27,225,252,64.99,'Intermediate' from dual union all
select 13413,'013413','1978-079D','Star 37E S/N 40069','Star 37E',45274,'Deep Space','1978-08-13 00:00:00','1978-08-12 00:00:00',73654.64,180,1151164,28.9,'Cislunar' from dual union all
select 13414,'013414','1982-077C','KB electropitaniya','KBE',54386,'Reentered','1982-08-13 00:00:00','1982-08-13 00:00:00',88.39,192,198,82.35,'Intermediate' from dual union all
select 13415,'013415','1982-077D','KBE part','KBE part',54386,'Reentered','1982-08-13 00:00:00','1982-08-13 00:00:00',88.21,181,191,82.35,'Intermediate' from dual union all
select 13416,'013416','1982-052D','Kosmos-1372 reactor core','US-A reactor core',54361,'In Earth orbit','','1991-11-01 00:00:00',103.6,903,954,64.9,'Intermediate' from dual union all
select 13417,'013417','1982-033Q','EVA debris','',54342,'Reentered','1982-09-01 00:00:00','1982-08-15 00:00:00',90.42,291,299,51.64,'Intermediate' from dual;
insert into SATELLITE
select 13418,'013418','1982-033R','EVA debris','',54342,'Reentered','1982-08-24 00:00:00','1982-08-15 00:00:00',90.29,281,297,51.61,'Intermediate' from dual union all
select 13419,'013419','1982-007P','Kosmos-1335 SS 11','ESO',54316,'Reentered','1982-10-05 00:00:00','1982-08-19 00:00:00',94.25,473,492,74.05,'Intermediate' from dual union all
select 1342,'001342','1965-020M','deb r/b S3','',11770,'Reentered','1969-09-09 00:00:00','1965-04-26 00:00:00',107.75,372,1874,56.08,'Intermediate' from dual union all
select 13420,'013420','1982-007Q','Kosmos-1335 SS 12','ESO',54316,'Reentered','1982-10-05 00:00:00','1982-08-17 00:00:00',94.23,470,492,74.07,'Intermediate' from dual union all
select 13421,'013421','1982-007R','Kosmos-1335 SS 13','ESO',54316,'Reentered','1982-09-29 00:00:00','1982-08-18 00:00:00',94,449,492,74.04,'Intermediate' from dual union all
select 13422,'013422','1982-007S','Kosmos-1335 SS 14','ESO',54316,'Reentered','1982-09-28 00:00:00','1982-08-17 00:00:00',93.98,447,492,74.05,'Intermediate' from dual union all
select 13423,'013423','1982-033S','EVA debris','',54342,'Reentered','1982-08-31 00:00:00','1982-08-18 00:00:00',90.26,284,291,51.63,'Intermediate' from dual union all
select 13424,'013424','1982-033T','EVA debris','',54342,'Reentered','1982-09-02 00:00:00','1982-08-20 00:00:00',90.32,285,295,51.63,'Intermediate' from dual union all
select 13425,'013425','1982-080A','Soyuz T-7','Soyuz 7K-ST No. 12L',54389,'Landed','1982-12-10 00:00:00','1982-09-20 00:00:00',90.8,302,326,51.62,'Intermediate' from dual union all
select 13426,'013426','1982-080B','11S510','11S510',54389,'Reentered','1982-08-21 00:00:00','1982-08-20 00:00:00',88.47,193,205,51.62,'Intermediate' from dual union all
select 13427,'013427','1982-081A','Kosmos-1401','Resurs-F1 17F41 No. 19',54390,'Landed','1982-09-03 00:00:00','1982-08-22 00:00:00',89.81,260,271,82.34,'Intermediate' from dual union all
select 13428,'013428','1982-081B','11S510','11S510',54390,'Reentered','1982-08-25 00:00:00','1982-08-22 00:00:00',88.82,200,233,82.34,'Intermediate' from dual union all
select 13429,'013429','1982-007T','Kosmos-1335 SS 15','ESO',54316,'Reentered','1982-10-04 00:00:00','1982-08-24 00:00:00',94.15,462,493,74.05,'Intermediate' from dual union all
select 1343,'001343','1965-020N','deb r/b S3','',11770,'Reentered','1966-12-20 00:00:00','1965-04-28 00:00:00',105.5,297,1739,56.03,'Intermediate' from dual union all
select 13430,'013430','1982-007U','Kosmos-1335 SS 16','ESO',54316,'Reentered','1982-10-04 00:00:00','1982-08-24 00:00:00',94.07,454,493,74.04,'Intermediate' from dual union all
select 13431,'013431','1982-082A','Anik D1','Anik D-1',54391,'In Earth orbit','','1982-09-27 00:00:00',1436.06,35781,35791,.03,'Stationary' from dual union all
select 13432,'013432','1982-083A','Molniya-3','Molniya-3 No. 33',54392,'Reentered','2002-01-13 00:00:00','1982-09-28 00:00:00',717.74,498,39854,62.89,'Molniya' from dual union all
select 13433,'013433','1982-083B','11S510','11S510',54392,'Reentered','1982-09-12 00:00:00','1982-09-03 00:00:00',90.3,199,380,62.86,'Intermediate' from dual union all
select 13434,'013434','1982-083C','BOZ','BOZ',54392,'Reentered','1982-09-10 00:00:00','1982-08-29 00:00:00',91.05,192,461,62.85,'Intermediate' from dual union all
select 13435,'013435','1982-033U','Soyuz T part','Soyuz T part',54342,'Reentered','1982-08-29 00:00:00','1982-08-27 00:00:00',90.52,286,314,51.63,'Intermediate' from dual union all
select 13436,'013436','1982-033V','Soyuz T-5 BO','7K-ST No. 11L BO',54342,'Reentered','1982-10-07 00:00:00','1982-08-28 00:00:00',90.53,290,311,51.62,'Intermediate' from dual union all
select 13437,'013437','1982-083D','deb Blok-I','',54392,'Reentered','1982-09-02 00:00:00','1982-08-29 00:00:00',91.18,205,460,62.84,'Intermediate' from dual union all
select 13438,'013438','1982-082B','Delta 164','Delta SSPS AJ10-118K',54391,'Reentered','1982-09-18 00:00:00','1982-09-02 00:00:00',91.07,215,439,25.96,'Intermediate' from dual union all
select 13439,'013439','1982-033W','Soyuz T part','Soyuz T part',54342,'Reentered','1982-09-06 00:00:00','1982-08-29 00:00:00',90.52,290,310,51.62,'Intermediate' from dual union all
select 1344,'001344','1965-020P','deb r/b S3','',11770,'Reentered','1969-11-14 00:00:00','1965-04-26 00:00:00',106.07,278,1811,56.06,'Intermediate' from dual union all
select 13440,'013440','1982-033X','Soyuz T part','Soyuz T part',54342,'Reentered','1982-09-27 00:00:00','1982-09-04 00:00:00',90.38,284,303,51.62,'Intermediate' from dual union all
select 13441,'013441','1982-084A','Kosmos-1402','US-A',54393,'Reentered','1983-01-23 00:00:00','1982-09-29 00:00:00',89.64,251,263,65.01,'Intermediate' from dual union all
select 13442,'013442','1981-028AU','deb Kosmos-1260','',52551,'Reentered','1990-10-20 00:00:00','1990-10-19 00:00:00',88.82,209,224,64.99,'Intermediate' from dual union all
select 13443,'013443','1981-028AV','deb Kosmos-1260','',52551,'Reentered','1990-03-29 00:00:00','1990-03-28 00:00:00',90.88,305,330,65.1,'Intermediate' from dual union all
select 13444,'013444','1981-028AW','deb Kosmos-1260','',52551,'Reentered','1983-01-02 00:00:00','1982-08-31 00:00:00',96.83,469,745,65.03,'Intermediate' from dual union all
select 13445,'013445','1981-028AX','deb Kosmos-1260','',52551,'Reentered','1982-11-11 00:00:00','1982-09-28 00:00:00',96.71,496,707,65,'Intermediate' from dual union all
select 13446,'013446','1982-083E','Blok-L','Blok-ML No. 76727-513',54392,'Reentered','2001-12-21 00:00:00','1982-10-01 00:00:00',733.16,499,40612,62.88,'Molniya' from dual union all
select 13447,'013447','1982-082C','PAM-D','Star 48  S/N 10016-8',54391,'Reentered','1985-08-23 00:00:00','1982-09-30 00:00:00',634.09,218,35924,24.43,'GEO Transfer' from dual union all
select 13448,'013448','1982-085A','Kosmos-1403','Zenit-6',54394,'Landed','1982-09-15 00:00:00','1982-09-03 00:00:00',92.21,352,414,70.38,'Intermediate' from dual union all
select 13449,'013449','1982-086A','Kosmos-1404','Zenit-6',54395,'Landed','1982-09-15 00:00:00','1982-09-04 00:00:00',92.26,356,415,72.85,'Intermediate' from dual union all
select 1345,'001345','1965-020Q','S3 large debris','RCS 0.95',11770,'Reentered','1988-11-26 00:00:00','1965-04-26 00:00:00',108.78,446,1895,56.01,'Intermediate' from dual union all
select 13450,'013450','1982-085B','11S510','11S510',54394,'Reentered','1982-09-09 00:00:00','1982-09-03 00:00:00',89.84,204,330,70.38,'Intermediate' from dual union all
select 13451,'013451','1982-086B','11S510','11S510',54395,'Reentered','1982-09-10 00:00:00','1982-09-04 00:00:00',89.58,189,319,72.85,'Intermediate' from dual union all
select 13452,'013452','1982-085C','Sensor cover','Sensor cover',54394,'Reentered','1982-09-02 00:00:00','1982-09-01 00:00:00',89.68,207,311,70.37,'Intermediate' from dual union all
select 13453,'013453','1982-086C','Sensor cover','Sensor cover',54395,'Reentered','1982-09-02 00:00:00','1982-09-01 00:00:00',89.5,192,308,72.82,'Intermediate' from dual union all
select 13454,'013454','1982-086D','Sensor cover','Sensor cover',54395,'Reentered','1982-09-03 00:00:00','1982-09-02 00:00:00',89.5,184,316,72.84,'Intermediate' from dual union all
select 13455,'013455','1981-053FB','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-03 00:00:00',103.11,883,929,82.89,'Intermediate' from dual union all
select 13456,'013456','1981-053FC','deb Kosmos-1275','',52576,'In Earth orbit','','1983-02-09 00:00:00',107.53,962,1262,82.91,'Intermediate' from dual union all
select 13457,'013457','1981-053FD','deb Kosmos-1275','',52576,'Reentered','2012-03-22 00:00:00','1993-07-29 00:00:00',101.82,823,867,82.99,'Intermediate' from dual union all
select 13458,'013458','1981-053FE','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',103.83,919,961,83.1,'Intermediate' from dual union all
select 13459,'013459','1981-053FF','deb Kosmos-1275','',52576,'Reentered','2001-10-15 00:00:00','1993-08-04 00:00:00',101.9,803,895,83.14,'Intermediate' from dual union all
select 1346,'001346','1965-020R','deb r/b S3','',11770,'Reentered','1975-05-18 00:00:00','1965-04-26 00:00:00',108.79,409,1933,56.07,'Intermediate' from dual union all
select 13460,'013460','1981-053FG','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',105.49,949,1086,83.14,'Intermediate' from dual union all
select 13461,'013461','1981-053FH','deb Kosmos-1275','',52576,'Reentered','1990-03-21 00:00:00','1990-03-20 00:00:00',90.72,291,329,82.62,'Intermediate' from dual union all
select 13462,'013462','1981-053FJ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',106.68,930,1216,82.84,'Intermediate' from dual union all
select 13463,'013463','1981-053FK','deb Kosmos-1275','',52576,'Reentered','1990-03-16 00:00:00','1990-03-14 00:00:00',90.88,294,342,82.77,'Intermediate' from dual union all
select 13464,'013464','1981-053FL','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',103.89,906,978,82.94,'Intermediate' from dual union all
select 13465,'013465','1981-053FM','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',101.57,790,876,83.02,'Intermediate' from dual union all
select 13466,'013466','1981-053FN','deb Kosmos-1275','',52576,'In Earth orbit','','1983-02-10 00:00:00',107.2,976,1218,83.08,'Intermediate' from dual union all
select 13467,'013467','1981-053FP','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',103.07,879,929,82.96,'Intermediate' from dual union all
select 13468,'013468','1981-053FQ','deb Kosmos-1275','',52576,'In Earth orbit','','1983-02-09 00:00:00',105.76,964,1096,82.72,'Intermediate' from dual union all
select 13469,'013469','1981-053FR','deb Kosmos-1275','',52576,'Reentered','1999-05-30 00:00:00','1993-08-08 00:00:00',97.82,596,713,82.76,'Intermediate' from dual union all
select 1347,'001347','1965-020S','deb r/b S3','',11770,'Reentered','2001-06-26 00:00:00','1965-04-26 00:00:00',109.54,593,1818,56.05,'Intermediate' from dual union all
select 13470,'013470','1981-053FS','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',108.35,964,1337,82.83,'Intermediate' from dual union all
select 13471,'013471','1981-053FT','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',104.64,929,1027,82.82,'Intermediate' from dual union all
select 13472,'013472','1981-053FU','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',103.99,925,970,82.85,'Intermediate' from dual union all
select 13473,'013473','1981-053FV','deb Kosmos-1275','',52576,'In Earth orbit','','1981-09-24 00:00:00',99.41,641,820,31.55,'Intermediate' from dual union all
select 13474,'013474','1978-034C','deb Kosmos-1000','',45229,'In Earth orbit','','1993-08-07 00:00:00',102.88,881,909,82.94,'Intermediate' from dual union all
select 13475,'013475','1977-062C','Unidentified Parus deb','[UNID087] RCS 0.0',42190,'In Earth orbit','','1992-07-24 00:00:00',104.21,915,1000,83.12,'Intermediate' from dual union all
select 13476,'013476','1981-053FW','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',104.54,928,1018,82.82,'Intermediate' from dual union all
select 13477,'013477','1981-053FX','deb Kosmos-1275','',52576,'In Earth orbit','','1991-05-20 00:00:00',104.63,901,1054,83.02,'Intermediate' from dual union all
select 13478,'013478','1981-053FY','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',105.27,938,1076,82.88,'Intermediate' from dual union all
select 13479,'013479','1981-053FZ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',101.67,787,889,82.92,'Intermediate' from dual union all
select 1348,'001348','1965-020T','S3 large debris','RCS 0.20',11770,'Reentered','1981-11-15 00:00:00','1965-04-26 00:00:00',109.48,448,1958,56.1,'Intermediate' from dual union all
select 13480,'013480','1981-053GA','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',104.87,931,1046,82.98,'Intermediate' from dual union all
select 13481,'013481','1981-053GB','deb Kosmos-1275','',52576,'In Earth orbit','','1983-02-07 00:00:00',104.99,936,1052,83.01,'Intermediate' from dual union all
select 13482,'013482','1981-053GC','deb Kosmos-1275','',52576,'In Earth orbit','','1983-02-06 00:00:00',105.43,933,1097,82.98,'Intermediate' from dual union all
select 13483,'013483','1981-053GD','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',103.63,916,945,82.94,'Intermediate' from dual union all
select 13484,'013484','1981-053GE','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',104.52,934,1010,82.93,'Intermediate' from dual union all
select 13485,'013485','1981-053GF','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',104.6,926,1026,82.94,'Intermediate' from dual union all
select 13486,'013486','1981-053GG','deb Kosmos-1275','',52576,'In Earth orbit','','1983-02-10 00:00:00',104.65,949,1008,82.91,'Intermediate' from dual union all
select 13487,'013487','1981-053GH','deb Kosmos-1275','',52576,'In Earth orbit','','1983-02-06 00:00:00',105.88,950,1122,83.01,'Intermediate' from dual union all
select 13488,'013488','1981-053GJ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',107.84,955,1298,83.08,'Intermediate' from dual union all
select 13489,'013489','1981-053GK','deb Kosmos-1275','',52576,'In Earth orbit','','1983-02-07 00:00:00',108.9,971,1381,83.15,'Intermediate' from dual union all
select 1349,'001349','1965-020U','deb r/b S3','',11770,'Reentered','1979-04-09 00:00:00','1965-04-25 00:00:00',108.47,405,1907,56.09,'Intermediate' from dual union all
select 13490,'013490','1981-053GL','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',105.17,949,1056,82.84,'Intermediate' from dual union all
select 13491,'013491','1977-062D','deb Kosmos-926','[UNID088] RCS 0.0',42190,'In Earth orbit','','1993-08-06 00:00:00',105.04,983,1010,82.93,'Intermediate' from dual union all
select 13492,'013492','1982-087A','Kiku-4','ETS 3',54396,'In Earth orbit','','1982-10-03 00:00:00',107.08,966,1217,44.61,'Intermediate' from dual union all
select 13493,'013493','1982-087B','Star 37N S/N 14008','Star 37N',54396,'In Earth orbit','','1992-10-11 00:00:00',105.07,990,1006,44.63,'Intermediate' from dual union all
select 13494,'013494','1982-081C','Blenda','Sunshade',54390,'Reentered','1982-09-07 00:00:00','1982-09-05 00:00:00',90.94,243,399,82.36,'Intermediate' from dual union all
select 13495,'013495','1982-081D','KDU part','KDU part',54390,'Reentered','1982-09-09 00:00:00','1982-09-03 00:00:00',89.75,247,278,82.32,'Intermediate' from dual union all
select 13496,'013496','1982-081E','KDU','KDU',54390,'Reentered','1982-09-22 00:00:00','1982-09-03 00:00:00',89.89,257,281,82.34,'Intermediate' from dual union all
select 13497,'013497','1978-063C','deb Kosmos-407 [UNID]','Strela-2M debris',45258,'In Earth orbit','','1993-08-07 00:00:00',100.23,757,782,74.06,'Intermediate' from dual union all
select 13498,'013498','1981-041C','deb Kosmos-1269','Strela-2M deb1 ex [UNID0',52564,'In Earth orbit','','1983-02-08 00:00:00',100.93,793,813,74.04,'Intermediate' from dual union all
select 13499,'013499','1976-069D','deb Kosmos-841 [UNID]','Strela-2M debris',39021,'In Earth orbit','','1983-02-06 00:00:00',101.09,791,830,74.06,'Intermediate' from dual union all
select 135,'000135','61 Omicron 20','deb r/b Ablestar','',7273,'Reentered','2002-04-17 00:00:00','1961-09-16 00:00:00',103.62,868,992,66.77,'Intermediate' from dual union all
select 1350,'001350','1965-020V','deb r/b S3','',11770,'Reentered','1965-12-19 00:00:00','1965-04-20 00:00:00',102.97,271,1528,55.95,'Intermediate' from dual union all
select 13500,'013500','1978-019C','deb Kosmos-990','[UNID090] Strela-2M debr',45214,'In Earth orbit','','1983-02-08 00:00:00',100.78,784,808,74.08,'Intermediate' from dual union all
select 13501,'013501','1976-067BB','deb Kosmos-839','',39019,'In Earth orbit','','1983-02-08 00:00:00',117.28,1029,2089,66.05,'Intermediate' from dual union all
select 13502,'013502','61 Omicron 258','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-06 00:00:00',109.09,907,1462,66.5,'Intermediate' from dual union all
select 13503,'013503','1977-121BA','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-06 00:00:00',106.83,1016,1143,66.47,'Intermediate' from dual union all
select 13504,'013504','1977-121BB','deb Kosmos-970','',42249,'In Earth orbit','','1983-02-18 00:00:00',104.73,830,1134,66.11,'Intermediate' from dual union all
select 13505,'013505','1977-121BC','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-01 00:00:00',107.41,1086,1128,65.36,'Intermediate' from dual union all
select 13506,'013506','1980-089BB','deb Kosmos-1220','[UNID091] RCS 0.01',50299,'Reentered','1989-10-25 00:00:00','1989-10-25 00:00:00',88.29,182,198,64.89,'Intermediate' from dual union all
select 13507,'013507','1976-126BK','deb Kosmos-886','',39078,'In Earth orbit','','1993-07-29 00:00:00',120.9,1012,2431,65.75,'Medium' from dual;
insert into SATELLITE
select 13508,'013508','1982-088A','Kosmos-1405','US-P',54397,'Reentered','1984-02-05 00:00:00','1982-10-04 00:00:00',93.27,428,442,65.02,'Intermediate' from dual union all
select 13509,'013509','1982-088B','11S692','11S692',54397,'Reentered','1982-09-05 00:00:00','1982-09-04 00:00:00',89.89,195,343,65.67,'Intermediate' from dual union all
select 1351,'001351','1965-020W','deb r/b S3','',11770,'Reentered','1972-08-06 00:00:00','1965-04-26 00:00:00',109.19,446,1933,56.08,'Intermediate' from dual union all
select 13510,'013510','1982-087C','Kiku-4 despin weight','',54396,'In Earth orbit','','1983-02-06 00:00:00',107.34,957,1250,44.61,'Intermediate' from dual union all
select 13511,'013511','61 Omicron 259','deb r/b Ablestar','',7273,'In Earth orbit','','1993-07-29 00:00:00',105.67,926,1126,65.85,'Intermediate' from dual union all
select 13512,'013512','61 Omicron 260','deb r/b Ablestar','',7273,'In Earth orbit','','1980-01-17 00:00:00',100.17,732,801,89.84,'Polar' from dual union all
select 13513,'013513','61 Omicron 261','deb r/b Ablestar','',7273,'In Earth orbit','','1993-07-24 00:00:00',104.99,922,1066,65.69,'Intermediate' from dual union all
select 13514,'013514','61 Omicron 262','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-03 00:00:00',107.11,907,1278,66.71,'Intermediate' from dual union all
select 13515,'013515','1976-067BC','deb Kosmos-839','',39019,'In Earth orbit','','1981-04-28 00:00:00',90.1,199,360,70.38,'Intermediate' from dual union all
select 13516,'013516','1976-067BD','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-01 00:00:00',116.86,967,2113,65.66,'Intermediate' from dual union all
select 13517,'013517','1965-020FF','deb Blok Kosmos','',11770,'In Earth orbit','','1983-02-07 00:00:00',110.33,729,1755,55.58,'Intermediate' from dual union all
select 13518,'013518','1970-025NR','deb Agena D','',21502,'In Earth orbit','','1993-07-29 00:00:00',105.65,1024,1026,100.01,'Sun-Synch' from dual union all
select 13519,'013519','1982-089A','Kosmos-1406','Fram',54398,'Landed','1982-09-21 00:00:00','1982-09-10 00:00:00',88.88,210,229,82.31,'Intermediate' from dual union all
select 1352,'001352','1965-020X','deb r/b S3','',11770,'Reentered','1967-07-03 00:00:00','1965-04-26 00:00:00',105.7,276,1778,56.09,'Intermediate' from dual union all
select 13520,'013520','1982-089B','11S510','11S510',54398,'Reentered','1982-09-11 00:00:00','1982-09-10 00:00:00',88.2,182,189,82.31,'Intermediate' from dual union all
select 13521,'013521','1982-090A','Fanhui Shi Weixing (04)','Jianbing-1 No. 5',54399,'Reentered','1982-09-21 00:00:00','1982-09-11 00:00:00',90.07,173,383,62.98,'Intermediate' from dual union all
select 13522,'013522','1982-033Y','Garbage bag','',54342,'Reentered','1982-10-03 00:00:00','1982-09-09 00:00:00',90.92,306,334,51.62,'Intermediate' from dual union all
select 13523,'013523','1982-033Z','Garbage bag','',54342,'Reentered','1982-10-02 00:00:00','1982-09-09 00:00:00',90.96,308,335,51.61,'Intermediate' from dual union all
select 13524,'013524','1973-086GM','deb Delta 98','',30688,'In Earth orbit','','1993-08-06 00:00:00',115.59,1436,1528,102.8,'Sun-Synch' from dual union all
select 13525,'013525','1973-086GN','deb Delta 98','',30688,'In Earth orbit','','1993-08-07 00:00:00',115.18,1404,1524,101.84,'Sun-Synch' from dual union all
select 13526,'013526','1974-089EM','deb Delta 104','',33692,'In Earth orbit','','1993-08-06 00:00:00',122.07,1412,2135,100.95,'Medium' from dual union all
select 13527,'013527','1974-089EN','deb Delta 104','',33692,'In Earth orbit','','1980-07-02 00:00:00',92.77,197,623,62.79,'Intermediate' from dual union all
select 13528,'013528','1974-089EP','deb Delta 104','',33692,'In Earth orbit','','1983-02-06 00:00:00',113.92,1332,1481,103.13,'Sun-Synch' from dual union all
select 13529,'013529','1976-077EB','deb Delta 126','',39029,'In Earth orbit','','1983-02-13 00:00:00',123.66,1525,2163,101.86,'Medium' from dual union all
select 1353,'001353','1965-020Y','deb r/b S3','',11770,'Reentered','1966-01-30 00:00:00','1965-04-26 00:00:00',103.74,293,1578,55.88,'Intermediate' from dual union all
select 13530,'013530','1976-077EC','deb Delta 126','',39029,'In Earth orbit','','1983-02-06 00:00:00',117.81,1481,1683,102.05,'Sun-Synch' from dual union all
select 13531,'013531','1976-077ED','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',118.36,1493,1721,102.66,'Sun-Synch' from dual union all
select 13532,'013532','1976-077EE','deb Delta 126','',39029,'In Earth orbit','','1980-02-11 00:00:00',117.58,1453,1692,74.04,'Intermediate' from dual union all
select 13533,'013533','1976-077EF','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',122.73,1507,2099,101.85,'Medium' from dual union all
select 13534,'013534','1976-077EG','deb Delta 126','',39029,'In Earth orbit','','1993-08-04 00:00:00',123.33,1499,2160,101.8,'Medium' from dual union all
select 13535,'013535','1976-077EH','deb Delta 126','',39029,'In Earth orbit','','1983-02-17 00:00:00',116.24,1494,1530,101.98,'Sun-Synch' from dual union all
select 13536,'013536','1976-077EJ','deb Delta 126','',39029,'In Earth orbit','','1993-07-25 00:00:00',122.52,1452,2135,101.74,'Medium' from dual union all
select 13537,'013537','1976-077EK','deb Delta 126','',39029,'In Earth orbit','','1993-08-02 00:00:00',117.38,1504,1623,102.42,'Sun-Synch' from dual union all
select 13538,'013538','1976-077EL','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',117.94,1501,1676,102.61,'Sun-Synch' from dual union all
select 13539,'013539','1976-077EM','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',123.68,1503,2187,101.87,'Medium' from dual union all
select 1354,'001354','1965-020Z','deb r/b S3','',11770,'Reentered','1968-05-15 00:00:00','1965-06-02 00:00:00',107.58,395,1835,56.02,'Intermediate' from dual union all
select 13540,'013540','1982-090B','CZ-2C Stage 2','CZ-2C Stage 2',54399,'Reentered','1982-09-09 00:00:00','1982-09-09 00:00:00',89.64,168,345,62.93,'Intermediate' from dual union all
select 13541,'013541','1982-033AA','Garbage bag','',54342,'Reentered','1982-10-01 00:00:00','1982-09-10 00:00:00',90.84,304,328,51.61,'Intermediate' from dual union all
select 13542,'013542','1982-085D','KDU','KDU',54394,'Reentered','1983-04-05 00:00:00','1982-10-01 00:00:00',92.1,348,407,70.37,'Intermediate' from dual union all
select 13543,'013543','1982-085E','Blenda','Sunshade',54394,'Reentered','1982-10-30 00:00:00','1982-09-22 00:00:00',91.89,333,402,70.37,'Intermediate' from dual union all
select 13544,'013544','1982-086E','Blenda','Sunshade',54395,'Reentered','1982-10-29 00:00:00','1982-09-15 00:00:00',94.41,388,592,72.85,'Intermediate' from dual union all
select 13545,'013545','1982-085F','KDU part','KDU part',54394,'Reentered','1982-10-14 00:00:00','1982-09-15 00:00:00',92.27,353,419,70.37,'Intermediate' from dual union all
select 13546,'013546','1982-091A','Kosmos-1407','Feniks No. 955',54400,'Landed','1982-10-16 00:00:00','1982-09-22 00:00:00',89,165,285,67.15,'Intermediate' from dual union all
select 13547,'013547','1982-091B','11S510','11S510',54400,'Reentered','1982-09-19 00:00:00','1982-09-17 00:00:00',89.12,163,299,67.15,'Intermediate' from dual union all
select 13548,'013548','1982-086F','KDU part','KDU part',54395,'Reentered','1982-10-17 00:00:00','1982-09-16 00:00:00',92.25,354,416,72.85,'Intermediate' from dual union all
select 13549,'013549','1982-086G','KDU','KDU',54395,'Reentered','1983-04-12 00:00:00','1982-10-01 00:00:00',92.14,350,409,72.85,'Intermediate' from dual union all
select 1355,'001355','1965-020AA','deb r/b S3','',11770,'Reentered','1966-10-09 00:00:00','1965-04-28 00:00:00',105.26,268,1746,56.08,'Intermediate' from dual union all
select 13550,'013550','1982-086H','KDU part','KDU part',54395,'Reentered','1982-10-30 00:00:00','1982-09-16 00:00:00',92.07,341,412,72.85,'Intermediate' from dual union all
select 13551,'013551','1982-085G','KDU part','KDU part',54394,'Reentered','1982-10-26 00:00:00','1982-09-16 00:00:00',94.35,383,592,70.38,'Intermediate' from dual union all
select 13552,'013552','1982-092A','Kosmos-1408','Tselina-D',54401,'In Earth orbit','','1982-10-31 00:00:00',97.69,631,666,82.57,'Intermediate' from dual union all
select 13553,'013553','1982-092B','S5M','S5M',54401,'In Earth orbit','','1987-10-29 00:00:00',97.63,628,663,82.58,'Intermediate' from dual union all
select 13554,'013554','1982-093A','Ekran','Ekran No. 24L',54402,'In Earth orbit','','1982-10-31 00:00:00',1436.11,35773,35801,.28,'Stationary' from dual union all
select 13555,'013555','1982-093B','8S812','8S812',54402,'Reentered','1982-09-18 00:00:00','1982-09-17 00:00:00',88.21,183,189,51.63,'Intermediate' from dual union all
select 13556,'013556','1982-093C','Perekhodnik','Sredniy perekhodnik',54402,'Reentered','1982-09-17 00:00:00','1982-09-16 00:00:00',88.37,188,200,51.62,'Intermediate' from dual union all
select 13557,'013557','1982-090C','CZ-2C sep motor cover','',54399,'Reentered','1982-09-11 00:00:00','1982-09-10 00:00:00',94.44,123,860,62.2,'Intermediate' from dual union all
select 13558,'013558','1982-094A','Progress-15','Progress 7K-TG No. 112',54403,'Deorbited','1982-10-16 00:00:00','1982-09-25 00:00:00',90.74,299,322,51.62,'Intermediate' from dual union all
select 13559,'013559','1982-094B','11S510','11S510',54403,'Reentered','1982-09-20 00:00:00','1982-09-19 00:00:00',88.43,179,215,51.65,'Intermediate' from dual union all
select 1356,'001356','1965-020AB','deb r/b S3','',11770,'Reentered','1965-08-14 00:00:00','1965-04-28 00:00:00',102.52,227,1529,56.18,'Intermediate' from dual union all
select 13560,'013560','1982-033AB','Garbage bag','',54342,'Reentered','1982-10-04 00:00:00','1982-09-19 00:00:00',90.71,296,323,51.61,'Intermediate' from dual union all
select 13561,'013561','1978-026EY','deb Delta 139','',45221,'In Earth orbit','','1981-04-20 00:00:00',93.27,411,459,74.05,'Intermediate' from dual union all
select 13562,'013562','1978-026EZ','deb Delta 139','',45221,'In Earth orbit','','1993-08-04 00:00:00',101.64,805,868,98.92,'Sun-Synch' from dual union all
select 13563,'013563','1978-026FA','deb Delta 139','',45221,'In Earth orbit','','1993-08-05 00:00:00',101.79,809,877,98.79,'Sun-Synch' from dual union all
select 13564,'013564','1978-026FB','deb Delta 139','',45221,'In Earth orbit','','1993-08-04 00:00:00',104.43,806,1130,98.87,'Sun-Synch' from dual union all
select 13565,'013565','1978-026FC','deb Delta 139','',45221,'In Earth orbit','','1993-08-02 00:00:00',103.61,876,983,98.92,'Sun-Synch' from dual union all
select 13566,'013566','1978-026FD','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',101.59,803,865,98.91,'Sun-Synch' from dual union all
select 13567,'013567','1978-026FE','deb Delta 139','',45221,'In Earth orbit','','1993-08-05 00:00:00',105.5,816,1220,98.82,'Sun-Synch' from dual union all
select 13568,'013568','1978-026FF','deb Delta 139','',45221,'Reentered','1990-11-17 00:00:00','1990-11-17 00:00:00',90.14,275,288,98.59,'Sun-Synch' from dual union all
select 13569,'013569','1978-026FG','deb Delta 139','',45221,'In Earth orbit','','1993-08-07 00:00:00',103.93,838,1051,98.85,'Sun-Synch' from dual union all
select 1357,'001357','1965-031G','OPS 6717 sensor cover','',11781,'Reentered','1966-03-18 00:00:00','1965-05-29 00:00:00',95.09,500,545,95.21,'Sun-Synch' from dual union all
select 13570,'013570','1978-026FH','deb Delta 139','',45221,'In Earth orbit','','1993-08-01 00:00:00',101.22,774,858,98.77,'Sun-Synch' from dual union all
select 13571,'013571','1978-026FJ','deb Delta 139','',45221,'In Earth orbit','','1993-08-03 00:00:00',101.31,799,843,98.92,'Sun-Synch' from dual union all
select 13572,'013572','1978-026FK','deb Delta 139','',45221,'In Earth orbit','','1993-07-27 00:00:00',103.5,776,1073,98.8,'Sun-Synch' from dual union all
select 13573,'013573','1978-026FL','deb Delta 139','',45221,'In Earth orbit','','1993-08-02 00:00:00',103.85,886,994,98.9,'Sun-Synch' from dual union all
select 13574,'013574','1978-026FM','deb Delta 139','',45221,'In Earth orbit','','1993-08-01 00:00:00',102.39,841,902,99.1,'Sun-Synch' from dual union all
select 13575,'013575','1978-026FN','deb Delta 139','',45221,'In Earth orbit','','1993-08-02 00:00:00',102.71,873,901,99.18,'Sun-Synch' from dual union all
select 13576,'013576','1978-026FP','deb Delta 139','',45221,'Reentered','2000-06-06 00:00:00','1993-08-07 00:00:00',98.78,659,741,98.82,'Sun-Synch' from dual union all
select 13577,'013577','1978-026FQ','deb Delta 139','',45221,'Reentered','2000-04-19 00:00:00','1983-02-06 00:00:00',101.52,792,870,98.77,'Sun-Synch' from dual union all
select 13578,'013578','1978-026FR','deb Delta 139','',45221,'Reentered','2005-11-11 00:00:00','1993-08-05 00:00:00',102.87,722,1067,98.87,'Sun-Synch' from dual union all
select 13579,'013579','1978-026FS','deb Delta 139','',45221,'In Earth orbit','','1983-02-07 00:00:00',102.86,879,908,98.77,'Sun-Synch' from dual union all
select 1358,'001358','1965-032B','Altair 2 X-258C3 RH-81','X-258C3',11782,'In Earth orbit','','1965-06-11 00:00:00',107.81,935,1316,41.18,'Intermediate' from dual union all
select 13580,'013580','1978-026FT','deb Delta 139','',45221,'Reentered','1983-03-06 00:00:00','1983-03-01 00:00:00',102,806,901,98.76,'Sun-Synch' from dual union all
select 13581,'013581','1982-089C','KDU','KDU',54398,'Reentered','1982-09-25 00:00:00','1982-09-21 00:00:00',88.73,208,216,82.31,'Intermediate' from dual union all
select 13582,'013582','1982-089D','Blenda','Sunshade',54398,'Reentered','1982-09-23 00:00:00','1982-09-21 00:00:00',88.27,182,196,82.31,'Intermediate' from dual union all
select 13583,'013583','1982-093D','SOZ','SOZ',54402,'Reentered','1983-10-12 00:00:00','1982-10-30 00:00:00',629.04,283,35599,47.31,'GEO Transfer' from dual union all
select 13584,'013584','1982-093E','SOZ','SOZ',54402,'Reentered','1983-09-14 00:00:00','1982-09-27 00:00:00',629.49,312,35593,47.46,'GEO Transfer' from dual union all
select 13585,'013585','1982-095A','Kosmos-1409','Oko',54404,'Reentered','2009-06-08 00:00:00','1982-11-01 00:00:00',717.76,643,39711,63.2,'Molniya' from dual union all
select 13586,'013586','1982-095B','11S510','11S510',54404,'Reentered','1982-10-17 00:00:00','1982-09-29 00:00:00',91.56,211,492,62.85,'Intermediate' from dual union all
select 13587,'013587','1982-095C','BOZ','BOZ',54404,'Reentered','1982-10-13 00:00:00','1982-09-29 00:00:00',91.77,192,531,62.85,'Intermediate' from dual union all
select 13588,'013588','1981-089H','deb Kosmos-1306','',52612,'Reentered','1982-12-06 00:00:00','1982-12-04 00:00:00',87.23,133,142,64.95,'Intermediate' from dual union all
select 13589,'013589','1982-096A','Kosmos-1410','Musson No. 13',54405,'In Earth orbit','','1982-10-31 00:00:00',115.92,1493,1502,82.61,'Intermediate' from dual union all
select 1359,'001359','1965-034A','Titan 3A Ballast','',11784,'In Earth orbit','','1965-05-06 00:00:00',157,2780,3745,32.09,'Medium' from dual union all
select 13590,'013590','1982-096B','S5M','S5M',54405,'In Earth orbit','','1993-08-05 00:00:00',115.87,1491,1499,82.61,'Intermediate' from dual union all
select 13591,'013591','1982-095D','Blok-L','Blok-2BL No. 76723-648',54404,'Reentered','2005-07-02 00:00:00','1982-10-29 00:00:00',707.29,634,39202,63.17,'Molniya' from dual union all
select 13592,'013592','1982-043B','Kosmos-1365 RLS','US-A RLS',54352,'Reentered','1982-09-29 00:00:00','1982-09-27 00:00:00',89.4,231,259,65,'Intermediate' from dual union all
select 13593,'013593','1982-043C','Kosmos-1365 DU','US-A DU',54352,'Reentered','1982-10-19 00:00:00','1982-09-27 00:00:00',89.54,241,263,65,'Intermediate' from dual union all
select 13594,'013594','1982-043D','Kosmos-1365 reactor core','US-A reactor core',54352,'In Earth orbit','','1991-11-17 00:00:00',103.37,891,945,65.06,'Intermediate' from dual union all
select 13595,'013595','1982-097A','INTELSAT 505','INTELSAT V F5',54406,'In Earth orbit','','1982-11-17 00:00:00',1428.81,35627,35661,1.52,'Drift GEO' from dual union all
select 13596,'013596','1982-033AC','Astrozond','Astrozond',54342,'Reentered','1982-10-28 00:00:00','1982-09-29 00:00:00',91.24,307,364,51.62,'Intermediate' from dual union all
select 13597,'013597','1982-098A','Kosmos-1411','Zenit-6',54407,'Landed','1982-10-14 00:00:00','1982-10-02 00:00:00',89.64,228,286,72.84,'Intermediate' from dual union all
select 13598,'013598','1982-098B','11S510','11S510',54407,'Reentered','1982-10-06 00:00:00','1982-10-02 00:00:00',89.57,188,319,72.84,'Intermediate' from dual;
insert into SATELLITE
select 13599,'013599','1982-097B','Centaur AC-60','Centaur D-1AR AC-60',54406,'Reentered','1992-02-16 00:00:00','1982-11-01 00:00:00',608.32,223,34584,24.42,'GEO Transfer' from dual union all
select 136,'000136','61 Omicron 21','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-06 00:00:00',103.31,858,973,67.56,'Intermediate' from dual union all
select 1360,'001360','1965-034B','LES 2','LES 2',11784,'In Earth orbit','','1966-03-17 00:00:00',309.8,2783,14794,32.21,'Medium' from dual union all
select 13600,'013600','1982-099A','Kosmos-1412','US-A reactor',54408,'In Earth orbit','','1982-11-01 00:00:00',89.62,249,263,65,'Intermediate' from dual union all
select 13601,'013601','1981-072C','deb Kosmos-1286','',52595,'Reentered','1982-10-22 00:00:00','1982-10-14 00:00:00',89.69,248,270,65,'Intermediate' from dual union all
select 13602,'013602','1982-043E','deb Kosmos-1365 ROCh','',54352,'Reentered','1982-10-16 00:00:00','1982-10-08 00:00:00',98.56,669,711,65.08,'Intermediate' from dual union all
select 13603,'013603','1982-100A','Kosmos-1413','Uragan No. 11L',54409,'In Earth orbit','','1982-11-17 00:00:00',673.31,19068,19070,64.82,'Medium' from dual union all
select 13604,'013604','1982-100B','8S812','8S812',54409,'Reentered','1982-10-14 00:00:00','1982-10-13 00:00:00',88.22,179,194,51.65,'Intermediate' from dual union all
select 13605,'013605','1982-100C','Perekhodnik','Sredniy perekhodnik',54409,'Reentered','1982-10-13 00:00:00','1982-10-12 00:00:00',88.29,183,197,51.64,'Intermediate' from dual union all
select 13606,'013606','1982-100D','Kosmos-1414','Uragan GVM',54409,'In Earth orbit','','1982-11-12 00:00:00',673.53,19071,19078,64.81,'Medium' from dual union all
select 13607,'013607','1982-100E','Kosmos-1415','Uragan GVM',54409,'In Earth orbit','','1982-11-12 00:00:00',673.47,19070,19076,64.8,'Medium' from dual union all
select 13608,'013608','1982-100F','SOZ','SOZ',54409,'In Earth orbit','','1982-11-12 00:00:00',339.32,396,19077,51.95,'Highly Ellip' from dual union all
select 13609,'013609','1982-100G','SOZ','SOZ',54409,'In Earth orbit','','1982-11-11 00:00:00',338.97,378,19074,52.1,'Highly Ellip' from dual union all
select 1361,'001361','1965-034C','LCS 1','LCS 1',11784,'In Earth orbit','','1965-07-12 00:00:00',145.55,2776,2800,32.13,'Medium' from dual union all
select 13610,'013610','1982-100H','Blok DM2','11S861 No. 1L',54409,'In Earth orbit','','1983-02-11 00:00:00',672.85,19046,19069,64.79,'Medium' from dual union all
select 13611,'013611','1982-101A','Kosmos-1416','Zenit-6',54410,'Landed','1982-10-28 00:00:00','1982-10-14 00:00:00',90.13,207,355,70.35,'Intermediate' from dual union all
select 13612,'013612','1982-101B','11S510','11S510',54410,'Reentered','1982-10-23 00:00:00','1982-10-14 00:00:00',89.75,221,303,70.3,'Intermediate' from dual union all
select 13613,'013613','1982-098C','KDU','KDU',54407,'Reentered','1982-10-23 00:00:00','1982-10-14 00:00:00',89.18,219,249,72.83,'Intermediate' from dual union all
select 13614,'013614','1982-098D','Blenda','Sunshade',54407,'Reentered','1982-10-15 00:00:00','1982-10-14 00:00:00',89,217,234,72.84,'Intermediate' from dual union all
select 13615,'013615','1982-043F','deb Kosmos-1365 ROCh','',54352,'Reentered','1982-10-08 00:00:00','1982-10-06 00:00:00',97.17,587,659,64.8,'Intermediate' from dual union all
select 13616,'013616','1982-098E','KDU part','KDU part',54407,'Reentered','1982-10-16 00:00:00','1982-10-14 00:00:00',88.73,205,219,72.8,'Intermediate' from dual union all
select 13617,'013617','1982-102A','Kosmos-1417','Parus',54411,'In Earth orbit','','1983-02-07 00:00:00',104.78,959,1009,82.97,'Intermediate' from dual union all
select 13618,'013618','1982-102B','S3M','S3M',54411,'In Earth orbit','','1991-03-25 00:00:00',104.64,956,1000,82.97,'Intermediate' from dual union all
select 13619,'013619','1981-053GM','deb Kosmos-1275','',52576,'In Earth orbit','','1983-02-08 00:00:00',105.1,964,1035,83.06,'Intermediate' from dual union all
select 1362,'001362','1965-035A','Kosmos-66','Zenit-2 No. 27',11785,'Landed','1965-05-15 00:00:00','1965-05-09 00:00:00',89.28,200,278,65.01,'Intermediate' from dual union all
select 13620,'013620','1981-028AY','deb Kosmos-1260','',52551,'Reentered','1989-01-30 00:00:00','1989-01-30 00:00:00',89.63,250,262,64.99,'Intermediate' from dual union all
select 13621,'013621','1981-028AZ','deb Kosmos-1260','',52551,'Reentered','1989-03-01 00:00:00','1989-02-28 00:00:00',89.36,237,249,65,'Intermediate' from dual union all
select 13622,'013622','1981-028BA','deb Kosmos-1260','',52551,'Reentered','1988-09-23 00:00:00','1988-09-22 00:00:00',88.91,212,229,64.98,'Intermediate' from dual union all
select 13623,'013623','1981-028BB','deb Kosmos-1260','',52551,'Reentered','1990-02-12 00:00:00','1990-02-11 00:00:00',87.94,163,182,64.94,'Intermediate' from dual union all
select 13624,'013624','1982-103A','Gorizont','Gorizont No. 16L',54412,'In Earth orbit','','1982-11-19 00:00:00',1436.04,35746,35825,.74,'Stationary' from dual union all
select 13625,'013625','1982-103B','8S812','8S812',54412,'Reentered','1982-10-22 00:00:00','1982-10-20 00:00:00',88.09,177,183,51.7,'Intermediate' from dual union all
select 13626,'013626','1982-103C','Perekhodnik','Sredniy perekhodnik',54412,'Reentered','1982-10-21 00:00:00','1982-10-20 00:00:00',88.4,183,207,51.6,'Intermediate' from dual union all
select 13627,'013627','1982-104A','Kosmos-1418','Taifun-1B',54413,'Reentered','1983-09-30 00:00:00','1982-11-20 00:00:00',92.22,363,403,50.68,'Intermediate' from dual union all
select 13628,'013628','1982-104B','S3M','S3M',54413,'Reentered','1983-03-30 00:00:00','1982-11-20 00:00:00',91.99,351,393,50.68,'Intermediate' from dual union all
select 13629,'013629','1982-103D','SOZ','SOZ',54412,'Reentered','1983-07-05 00:00:00','1982-11-24 00:00:00',633.35,295,35809,47.33,'GEO Transfer' from dual union all
select 1363,'001363','1965-035B','8A92E','8A92E',11785,'Reentered','1965-05-23 00:00:00','1965-05-14 00:00:00',89.14,216,248,65.01,'Intermediate' from dual union all
select 13630,'013630','1982-103E','Blok DM No. 37L','11S86 No. 37L',54412,'In Earth orbit','','1993-06-09 00:00:00',1434.2,35711,35788,8.66,'Inclined Drift' from dual union all
select 13631,'013631','1982-105A','Satcom 5','RCA-E',54414,'In Earth orbit','','1982-11-27 00:00:00',1436.06,35763,35809,.12,'Stationary' from dual union all
select 13632,'013632','1982-105B','Delta 165','Delta SSPS AJ10-118K',54414,'Reentered','1988-12-16 00:00:00','1982-11-27 00:00:00',131.63,237,4150,26.64,'Medium' from dual union all
select 13633,'013633','1982-101C','Blenda','Sunshade',54410,'Reentered','1982-10-30 00:00:00','1982-10-28 00:00:00',89.17,215,252,70.4,'Intermediate' from dual union all
select 13634,'013634','1982-101D','KDU','KDU',54410,'Reentered','1982-11-08 00:00:00','1982-10-30 00:00:00',89.18,216,252,70.36,'Intermediate' from dual union all
select 13635,'013635','1981-028BC','deb Kosmos-1260','',52551,'Reentered','1983-02-13 00:00:00','1983-02-12 00:00:00',90.02,259,292,64.99,'Intermediate' from dual union all
select 13636,'013636','1982-106A','DSCS II F-16','DSCS II F-16',54415,'In Earth orbit','','1982-11-30 00:00:00',1432.13,35643,35775,2.39,'Drift GEO' from dual union all
select 13637,'013637','1982-106B','DSCS III A-1','DSCS III A-1',54415,'In Earth orbit','','1982-11-29 00:00:00',1436.32,35726,35856,.45,'Stationary' from dual union all
select 13638,'013638','1982-107A','Progress-16','Progress 7K-TG No. 115',54416,'Deorbited','1982-12-15 00:00:00','1982-11-11 00:00:00',91.64,352,358,51.62,'Intermediate' from dual union all
select 13639,'013639','1982-106C','IUS-2 SRM-1','Orbus 21',54415,'Reentered','1983-09-14 00:00:00','1982-11-27 00:00:00',560.38,130,32145,27.99,'Highly Ellip' from dual union all
select 1364,'001364','1965-035C','deb Kosmos-66','',11785,'Reentered','1965-05-09 00:00:00','1965-05-07 00:00:00',88.75,189,236,64.97,'Intermediate' from dual union all
select 13640,'013640','1982-107B','11S510','11S510',54416,'Reentered','1982-11-02 00:00:00','1982-11-01 00:00:00',88.43,176,218,51.62,'Intermediate' from dual union all
select 13641,'013641','1982-108A','Kosmos-1419','Zenit-6',54417,'Landed','1982-11-16 00:00:00','1982-11-04 00:00:00',89.59,229,280,70.34,'Intermediate' from dual union all
select 13642,'013642','1982-108B','11S510','11S510',54417,'Reentered','1982-11-06 00:00:00','1982-11-04 00:00:00',88.68,190,229,70.34,'Intermediate' from dual union all
select 13643,'013643','1982-106D','IUS-2 SRM-2','Orbus 6E',54415,'In Earth orbit','','1993-05-30 00:00:00',1448.88,35871,36202,7.35,'Inclined Drift' from dual union all
select 13644,'013644','1982-103F','SOZ','SOZ',54412,'Reentered','1983-05-21 00:00:00','1982-11-27 00:00:00',631.49,238,35770,47.28,'GEO Transfer' from dual union all
select 13645,'013645','1982-099B','Kosmos-1412 DU','US-A DU',54408,'Reentered','1982-12-04 00:00:00','1982-11-11 00:00:00',89.51,245,256,64.99,'Intermediate' from dual union all
select 13646,'013646','1982-099C','Kosmos-1412 RLS','US-A RLS',54408,'Reentered','1982-11-12 00:00:00','1982-11-10 00:00:00',89.34,236,248,64.99,'Intermediate' from dual union all
select 13647,'013647','1982-099D','deb Kosmos-1412','',54408,'Reentered','1982-11-14 00:00:00','1982-11-10 00:00:00',89.23,226,247,64.97,'Intermediate' from dual union all
select 13648,'013648','1982-109A','Kosmos-1420','Strela-2M',54418,'In Earth orbit','','1983-02-09 00:00:00',100.73,778,809,74,'Intermediate' from dual union all
select 13649,'013649','1982-109B','S3M','S3M',54418,'In Earth orbit','','1991-03-27 00:00:00',100.52,768,799,73.99,'Intermediate' from dual union all
select 1365,'001365','1965-035D','deb Kosmos-66','',11785,'Reentered','1965-05-09 00:00:00','1965-05-07 00:00:00',89.16,194,272,64.97,'Intermediate' from dual union all
select 13650,'013650','1982-110A','Columbia (STS-5)','OV-102',54419,'Landed','1982-11-16 00:00:00','1982-11-13 00:00:00',90.66,296,318,28.47,'Intermediate' from dual union all
select 13651,'013651','1982-110B','SBS 3','SBS 3',54419,'In Earth orbit','','1982-12-11 00:00:00',1436.14,35782,35793,.26,'Stationary' from dual union all
select 13652,'013652','1982-110C','Anik C3','Anik C3',54419,'In Earth orbit','','1982-12-11 00:00:00',1435.98,35718,35851,.05,'Stationary' from dual union all
select 13653,'013653','1982-099E','Kosmos-1412 reactor core','US-A reactor core',54408,'In Earth orbit','','1982-11-15 00:00:00',103.6,904,953,64.83,'Intermediate' from dual union all
select 13654,'013654','1982-108C','KDU','KDU',54417,'Reentered','1982-11-21 00:00:00','1982-11-16 00:00:00',88.97,213,235,70.34,'Intermediate' from dual union all
select 13655,'013655','1982-108D','Blenda','Sunshade',54417,'Reentered','1982-11-17 00:00:00','1982-11-16 00:00:00',88.92,209,234,70.35,'Intermediate' from dual union all
select 13656,'013656','1982-108E','KDU part','KDU part',54417,'Reentered','1982-11-17 00:00:00','1982-11-16 00:00:00',88.94,210,235,70.34,'Intermediate' from dual union all
select 13657,'013657','1982-108F','KDU part','KDU part',54417,'Reentered','1982-11-16 00:00:00','1982-11-16 00:00:00',88.45,191,205,70.33,'Intermediate' from dual union all
select 13658,'013658','1982-110D','PAM-D','Star 48  S/N 10017-12',54419,'In Earth orbit','','1982-12-19 00:00:00',662.86,329,37281,23.75,'GEO Transfer' from dual union all
select 13659,'013659','1982-111A','OPS 9627','KENNEN 5',54420,'Deorbited','1985-08-13 00:00:00','1983-02-07 00:00:00',91.79,271,453,96.97,'Sun-Synch' from dual union all
select 1366,'001366','1965-036A','Luna-5','E-6 No. 10',11786,'Deep Space','1965-05-12 00:00:00','1965-05-09 00:00:00',21544.43,200,500000,65,'Cislunar' from dual union all
select 13660,'013660','1982-111B','Titan 23D-23 Stage 2','LR91AJ11 23D-23',54420,'Reentered','1982-12-27 00:00:00','1982-11-24 00:00:00',91.71,233,483,96.97,'Sun-Synch' from dual union all
select 13661,'013661','1982-112A','Kosmos-1421','Zenit-6',54421,'Landed','1982-12-02 00:00:00','1982-11-20 00:00:00',89.59,229,280,70.34,'Intermediate' from dual union all
select 13662,'013662','1982-112B','11S510','11S510',54421,'Reentered','1982-11-22 00:00:00','1982-11-20 00:00:00',88.61,187,225,70.35,'Intermediate' from dual union all
select 13663,'013663','1982-033AD','Iskra-3','RK-03',54342,'Reentered','1982-12-16 00:00:00','1982-11-18 00:00:00',91.56,348,355,51.64,'Intermediate' from dual union all
select 13664,'013664','1982-033AE','Iskra-3 cover','RK-03 cover',54342,'Reentered','1982-12-28 00:00:00','1982-11-18 00:00:00',91.52,344,355,51.62,'Intermediate' from dual union all
select 13665,'013665','1982-112C','Sensor cover','Sensor cover',54421,'Reentered','1982-11-20 00:00:00','1982-11-22 00:00:00',88.01,161,190,70.36,'Intermediate' from dual union all
select 13666,'013666','1982-110E','PAM-D','Star 48  S/N 10018-3',54419,'In Earth orbit','','1982-12-12 00:00:00',662.13,331,37242,23.54,'GEO Transfer' from dual union all
select 13667,'013667','1981-097E','Kosmos-1311 SS 3','ESO',52620,'Reentered','1982-12-05 00:00:00','1982-11-26 00:00:00',92.56,392,408,82.83,'Intermediate' from dual union all
select 13668,'013668','1981-097F','Kosmos-1311 SS 4','ESO',52620,'Reentered','1982-12-05 00:00:00','1982-11-25 00:00:00',92.6,391,413,82.87,'Intermediate' from dual union all
select 13669,'013669','1982-113A','Raduga','Gran'' No. 21L',54422,'In Earth orbit','','1982-12-31 00:00:00',1436.14,35763,35812,1.22,'Stationary' from dual union all
select 1367,'001367','1965-033B','SRV 627','SRV 1019-2',11783,'Reentered','1965-06-08 00:00:00','1965-05-09 00:00:00',98.09,193,1142,84.85,'Intermediate' from dual union all
select 13670,'013670','1982-113B','8S812','8S812',54422,'Reentered','1982-11-27 00:00:00','1982-11-26 00:00:00',88.22,182,191,51.6,'Intermediate' from dual union all
select 13671,'013671','1982-113C','Perekhodnik','Sredniy perekhodnik',54422,'Reentered','1982-11-28 00:00:00','1982-11-27 00:00:00',88.16,181,186,51.61,'Intermediate' from dual union all
select 13672,'013672','1982-112D','KDU part','KDU part',54421,'Reentered','1982-12-05 00:00:00','1982-12-04 00:00:00',87.51,151,152,70.33,'Intermediate' from dual union all
select 13673,'013673','1982-112E','Blenda','Sunshade',54421,'Reentered','1982-12-06 00:00:00','1983-10-01 00:00:00',87.73,160,165,51.6,'Intermediate' from dual union all
select 13674,'013674','1982-112F','KDU','KDU',54421,'Reentered','1982-12-14 00:00:00','1982-12-03 00:00:00',89.31,217,264,70.34,'Intermediate' from dual union all
select 13675,'013675','1982-113D','SOZ','SOZ',54422,'Reentered','1984-06-15 00:00:00','1982-12-27 00:00:00',648.83,255,36642,47.21,'GEO Transfer' from dual union all
select 13676,'013676','1982-113E','SOZ','SOZ',54422,'Reentered','1983-10-12 00:00:00','1982-12-27 00:00:00',650.52,308,36675,47.25,'GEO Transfer' from dual union all
select 13677,'013677','1982-114A','Kosmos-1422','Zenit-6',54423,'Landed','1982-12-17 00:00:00','1982-12-05 00:00:00',89.63,226,287,72.85,'Intermediate' from dual union all
select 13678,'013678','1982-114B','11S510','11S510',54423,'Reentered','1982-12-07 00:00:00','1982-12-05 00:00:00',88.79,183,246,72.85,'Intermediate' from dual union all
select 13679,'013679','1982-114C','Sensor cover','Sensor cover',54423,'Reentered','1982-12-04 00:00:00','1982-12-04 00:00:00',88.25,157,218,72.85,'Intermediate' from dual union all
select 1368,'001368','1965-036B','BOZ','BOZ',11786,'Reentered','1965-05-10 00:00:00','1965-05-09 00:00:00',88.27,159,219,64.83,'Intermediate' from dual union all
select 13680,'013680','1980-089BC','deb Kosmos-1220','',50299,'Reentered','1988-07-05 00:00:00','1988-07-04 00:00:00',88.19,163,207,64.93,'Intermediate' from dual union all
select 13681,'013681','1981-028BD','deb Kosmos-1260','',52551,'Reentered','1992-01-21 00:00:00','1991-10-07 00:00:00',96.66,561,637,65.08,'Intermediate' from dual union all
select 13682,'013682','1981-028BE','deb Kosmos-1260','',52551,'Reentered','1996-10-11 00:00:00','1983-02-06 00:00:00',99.38,707,750,65.07,'Intermediate' from dual union all
select 13683,'013683','1976-126BL','deb Kosmos-886','',39078,'Reentered','2002-06-17 00:00:00','1993-08-08 00:00:00',103.05,533,1273,65.66,'Intermediate' from dual union all
select 13684,'013684','1982-105C','Star 37E S/N 40093','Star 37E',54414,'Reentered','1984-02-09 00:00:00','1982-12-09 00:00:00',571.5,129,32740,24.8,'Highly Ellip' from dual union all
select 13685,'013685','1982-115A','Kosmos-1423','Molniya-1 11F658 No. 60',54424,'Reentered','1986-01-18 00:00:00','1983-01-08 00:00:00',93.68,422,488,62.83,'Intermediate' from dual union all
select 13686,'013686','1982-115B','11S510','11S510',54424,'Reentered','1982-12-27 00:00:00','1982-12-15 00:00:00',90.47,221,375,62.94,'Intermediate' from dual union all
select 13687,'013687','1980-089BD','deb Kosmos-1220','',50299,'Reentered','1983-09-08 00:00:00','1983-09-06 00:00:00',90.64,292,320,64.92,'Intermediate' from dual union all
select 13688,'013688','1980-089BE','deb Kosmos-1220','',50299,'Reentered','1983-03-03 00:00:00','1983-03-01 00:00:00',90.04,260,293,64.99,'Intermediate' from dual;
insert into SATELLITE
select 13689,'013689','1981-028BF','deb Kosmos-1260','',52551,'Reentered','1991-02-04 00:00:00','1991-02-03 00:00:00',87.93,168,176,64.99,'Intermediate' from dual union all
select 1369,'001369','1965-036C','11S510','11S510',11786,'Reentered','1965-05-10 00:00:00','1965-05-09 00:00:00',88.27,159,219,64.83,'Intermediate' from dual union all
select 13690,'013690','1981-028BG','deb Kosmos-1260 solar array','RCS 40m2',52551,'Reentered','1990-03-18 00:00:00','1990-03-01 00:00:00',89.12,214,248,64.97,'Intermediate' from dual union all
select 13691,'013691','1981-028BH','deb Kosmos-1260','',52551,'Reentered','1984-10-04 00:00:00','1984-09-23 00:00:00',91.54,333,368,64.95,'Intermediate' from dual union all
select 13692,'013692','1982-115C','deb Blok-I','',54424,'Reentered','1982-12-09 00:00:00','1982-12-08 00:00:00',91.07,218,437,62.97,'Intermediate' from dual union all
select 13693,'013693','1982-115D','BOZ','BOZ',54424,'Reentered','1982-12-22 00:00:00','1982-12-22 00:00:00',90.03,258,294,62.92,'Intermediate' from dual union all
select 13694,'013694','1981-097G','Kosmos-1311 SS 5','ESO',52620,'Reentered','1982-12-17 00:00:00','1982-12-09 00:00:00',92.65,395,414,82.91,'Intermediate' from dual union all
select 13695,'013695','1981-097H','Kosmos-1311 SS 6','ESO',52620,'Reentered','1982-12-16 00:00:00','1982-12-10 00:00:00',92.38,378,405,82.95,'Intermediate' from dual union all
select 13696,'013696','1982-115E','Blok-L','Blok-ML No. 76714-461',54424,'Reentered','1988-05-16 00:00:00','1983-01-07 00:00:00',94.37,392,584,62.83,'Intermediate' from dual union all
select 13697,'013697','1982-115F','deb Blok-L','',54424,'Reentered','1982-12-19 00:00:00','1982-12-17 00:00:00',90.18,241,326,62.95,'Intermediate' from dual union all
select 13698,'013698','1982-115G','deb Blok-L','',54424,'Reentered','1982-12-21 00:00:00','1982-12-17 00:00:00',91.47,303,391,62.92,'Intermediate' from dual union all
select 13699,'013699','1982-115H','deb Blok-L','',54424,'Reentered','1982-12-23 00:00:00','1982-12-22 00:00:00',89.75,248,276,62.93,'Intermediate' from dual union all
select 137,'000137','61 Omicron 22','deb r/b Ablestar','',7273,'Reentered','1980-12-15 00:00:00','1961-09-06 00:00:00',103.47,858,988,66.57,'Intermediate' from dual union all
select 1370,'001370','1965-020AC','deb r/b S3','',11770,'Reentered','2004-01-17 00:00:00','1992-01-30 00:00:00',102.63,527,1240,56.06,'Intermediate' from dual union all
select 13700,'013700','1982-033AF','Soyuz T-7 BO','7K-ST No. 12L BO',54342,'Reentered','1983-03-26 00:00:00','1982-12-11 00:00:00',91.5,345,351,51.62,'Intermediate' from dual union all
select 13701,'013701','1982-115J','deb Blok-L','',54424,'Reentered','1982-12-12 00:00:00','1982-12-12 00:00:00',88.32,147,236,62.9,'Intermediate' from dual union all
select 13702,'013702','1982-115K','deb Blok-L','',54424,'Reentered','1982-12-26 00:00:00','1982-12-25 00:00:00',88.41,156,236,62.92,'Intermediate' from dual union all
select 13703,'013703','1982-115L','deb Blok-L','',54424,'Reentered','1982-12-15 00:00:00','1982-12-11 00:00:00',91.36,256,427,62.89,'Intermediate' from dual union all
select 13704,'013704','1982-115M','deb Blok-L','',54424,'Reentered','1984-04-11 00:00:00','1984-04-12 00:00:00',88.2,181,190,62.76,'Intermediate' from dual union all
select 13705,'013705','1982-115N','deb Blok-L','',54424,'Reentered','1985-05-26 00:00:00','1985-05-25 00:00:00',88.29,187,193,62.81,'Intermediate' from dual union all
select 13706,'013706','1982-115P','deb Blok-L','',54424,'Reentered','1983-04-08 00:00:00','1982-12-14 00:00:00',93.75,386,530,62.82,'Intermediate' from dual union all
select 13707,'013707','1982-115Q','deb Blok-L','',54424,'Reentered','1984-06-07 00:00:00','1984-06-06 00:00:00',87.86,165,173,62.86,'Intermediate' from dual union all
select 13708,'013708','1982-115R','deb Blok-L','',54424,'Reentered','1989-03-23 00:00:00','1989-03-23 00:00:00',87.42,139,155,62.82,'Intermediate' from dual union all
select 13709,'013709','1982-115S','deb Blok-L','',54424,'Reentered','1983-11-01 00:00:00','1983-11-01 00:00:00',88.21,180,192,62.92,'Intermediate' from dual union all
select 1371,'001371','1965-020AD','S3 large debris','RCS 0.28',11770,'Reentered','1980-09-26 00:00:00','1965-05-10 00:00:00',109.36,551,1844,55.84,'Intermediate' from dual union all
select 13710,'013710','1982-115T','deb Blok-L','',54424,'Reentered','1984-02-29 00:00:00','1984-02-29 00:00:00',88.83,201,233,62.94,'Intermediate' from dual union all
select 13711,'013711','1982-115U','deb Blok-L','',54424,'Reentered','1983-09-02 00:00:00','1983-09-03 00:00:00',86.96,117,131,62.66,'Intermediate' from dual union all
select 13712,'013712','1982-115V','deb Blok-L','',54424,'Reentered','1984-12-06 00:00:00','1984-12-06 00:00:00',87.74,157,169,62.65,'Intermediate' from dual union all
select 13713,'013713','1982-115W','deb Blok-L','',54424,'Reentered','1985-12-11 00:00:00','1985-12-10 00:00:00',88.21,177,194,62.78,'Intermediate' from dual union all
select 13714,'013714','1982-115X','deb Blok-L','',54424,'Reentered','1986-02-17 00:00:00','1986-02-16 00:00:00',87.82,159,175,62.91,'Intermediate' from dual union all
select 13715,'013715','1982-115Y','deb Blok-L','',54424,'Reentered','1984-07-16 00:00:00','1984-07-14 00:00:00',88.49,196,203,62.68,'Intermediate' from dual union all
select 13716,'013716','1982-115Z','deb Blok-L','',54424,'Reentered','1984-05-26 00:00:00','1984-05-24 00:00:00',87.19,132,139,62.75,'Intermediate' from dual union all
select 13717,'013717','1982-115AA','deb Blok-L','',54424,'Reentered','1984-03-08 00:00:00','1984-03-07 00:00:00',88.27,178,200,62.92,'Intermediate' from dual union all
select 13718,'013718','1982-116A','Meteor-2','Meteor-2 No. 9',54425,'In Earth orbit','','1983-02-07 00:00:00',101.91,808,891,81.25,'Intermediate' from dual union all
select 13719,'013719','1982-116B','8A92ME','8A92ME',54425,'In Earth orbit','','1983-02-09 00:00:00',101.97,799,905,81.25,'Intermediate' from dual union all
select 1372,'001372','1965-020AE','deb r/b S3','',11770,'Reentered','1989-06-24 00:00:00','1965-05-14 00:00:00',110.8,585,1942,56.02,'Intermediate' from dual union all
select 13720,'013720','1982-116C','deb Meteor-2','',54425,'In Earth orbit','','1983-02-09 00:00:00',101.9,807,890,81.24,'Intermediate' from dual union all
select 13721,'013721','1982-115AB','deb Blok-L','',54424,'Reentered','1983-09-15 00:00:00','1983-09-14 00:00:00',88.51,190,211,62.64,'Intermediate' from dual union all
select 13722,'013722','1982-115AC','deb Blok-L','',54424,'Reentered','1984-01-26 00:00:00','1984-01-24 00:00:00',88.44,192,203,62.69,'Intermediate' from dual union all
select 13723,'013723','1981-097J','Kosmos-1311 SS 7','ESO',52620,'Reentered','1982-12-22 00:00:00','1982-12-16 00:00:00',92.61,385,420,82.93,'Intermediate' from dual union all
select 13724,'013724','1981-097K','Kosmos-1311 SS 8','ESO',52620,'Reentered','1982-12-23 00:00:00','1982-12-16 00:00:00',92.49,388,406,82.94,'Intermediate' from dual union all
select 13725,'013725','1982-117A','Kosmos-1424','Oktan No. 217',54426,'Landed','1983-01-28 00:00:00','1982-12-23 00:00:00',89.18,170,298,64.89,'Intermediate' from dual union all
select 13726,'013726','1982-117B','11S510','11S510',54426,'Reentered','1982-12-19 00:00:00','1982-12-18 00:00:00',89.04,161,294,64.91,'Intermediate' from dual union all
select 13727,'013727','1982-114D','KDU','KDU',54423,'Reentered','1982-12-29 00:00:00','1982-12-17 00:00:00',89.35,214,271,72.85,'Intermediate' from dual union all
select 13728,'013728','1982-114E','Blenda','Sunshade',54423,'Reentered','1982-12-20 00:00:00','1982-12-17 00:00:00',89.16,220,246,72.86,'Intermediate' from dual union all
select 13729,'013729','1981-097L','Kosmos-1311 SS 9','ESO',52620,'Reentered','1982-12-24 00:00:00','1982-12-17 00:00:00',92.3,376,399,82.95,'Intermediate' from dual union all
select 1373,'001373','1965-020AF','S3 large debris','RCS 0.39',11770,'Reentered','1979-02-04 00:00:00','1965-05-10 00:00:00',109.85,466,1974,56.08,'Intermediate' from dual union all
select 13730,'013730','1982-114F','KDU part','KDU part',54423,'Reentered','1982-12-18 00:00:00','1982-12-17 00:00:00',88.98,204,245,72.85,'Intermediate' from dual union all
select 13731,'013731','1981-097M','Kosmos-1311 SS 10','ESO',52620,'Reentered','1982-12-18 00:00:00','1982-12-16 00:00:00',90.95,308,334,82.9,'Intermediate' from dual union all
select 13732,'013732','1981-097N','Kosmos-1311 SS 11','ESO',52620,'Reentered','1982-12-17 00:00:00','1982-12-17 00:00:00',90.08,264,292,82.9,'Intermediate' from dual union all
select 13733,'013733','1981-097P','Kosmos-1311 SS 12','ESO',52620,'Reentered','1982-12-17 00:00:00','1982-12-16 00:00:00',89.06,213,243,82.94,'Intermediate' from dual union all
select 13734,'013734','1981-097Q','Kosmos-1311 SS 13','ESO',52620,'Reentered','1982-12-16 00:00:00','1982-12-16 00:00:00',88.62,198,215,82.94,'Intermediate' from dual union all
select 13735,'013735','1981-097R','Kosmos-1311 SS 14','ESO',52620,'Reentered','1982-12-31 00:00:00','1982-12-17 00:00:00',92.42,376,411,82.94,'Intermediate' from dual union all
select 13736,'013736','1982-118A','DMSP F-6','DMSP Block 5D-2 S-6',54427,'In Earth orbit','','1983-02-12 00:00:00',101.29,813,827,98.73,'Sun-Synch' from dual union all
select 13737,'013737','1982-118B','DMSP F-6 solar array band','',54427,'Reentered','1991-10-01 00:00:00','1991-09-22 00:00:00',93.16,422,437,98.52,'Sun-Synch' from dual union all
select 13738,'013738','1982-118C','DMSP F-6 solar array band','',54427,'Reentered','1999-10-26 00:00:00','1983-02-07 00:00:00',101.26,815,822,98.71,'Sun-Synch' from dual union all
select 13739,'013739','1982-119A','Kosmos-1425','Zenit-6',54428,'Landed','1983-01-06 00:00:00','1982-12-25 00:00:00',92.16,346,415,69.97,'Intermediate' from dual union all
select 1374,'001374','1965-037A','OPS 8431','CORONA J-21',11787,'Reentered Attached','1965-06-15 00:00:00','1965-05-26 00:00:00',89.67,192,325,75.02,'Intermediate' from dual union all
select 13740,'013740','1982-119B','11S510','11S510',54428,'Reentered','1983-01-06 00:00:00','1982-12-25 00:00:00',90.05,222,332,69.97,'Intermediate' from dual union all
select 13741,'013741','1982-119C','Sensor cover','Sensor cover',54428,'Reentered','1982-12-28 00:00:00','1982-12-23 00:00:00',90.57,163,442,69.9,'Intermediate' from dual union all
select 13742,'013742','1982-119D','Sensor cover','Sensor cover',54428,'Reentered','1982-12-28 00:00:00','1982-12-23 00:00:00',89.84,226,308,70.04,'Intermediate' from dual union all
select 13743,'013743','1981-097S','Kosmos-1311 SS 15','ESO',52620,'Reentered','1982-12-31 00:00:00','1982-12-24 00:00:00',90.17,270,296,82.92,'Intermediate' from dual union all
select 13744,'013744','1981-097T','Kosmos-1311 SS 16','ESO',52620,'Reentered','1982-12-31 00:00:00','1982-12-24 00:00:00',90.36,248,337,82.98,'Intermediate' from dual union all
select 13745,'013745','1982-120A','Kosmos-1426','Yantar''-4KS1',54429,'Deorbited','1983-03-05 00:00:00','1983-01-27 00:00:00',89.66,206,309,50.53,'Intermediate' from dual union all
select 13746,'013746','1982-120B','11S510','11S510',54429,'Reentered','1983-01-07 00:00:00','1982-12-30 00:00:00',89.81,199,332,50.53,'Intermediate' from dual union all
select 13747,'013747','1982-084B','Kosmos-1402 RLS','US-A RLS',54393,'Reentered','1982-12-30 00:00:00','1982-12-28 00:00:00',89.49,243,255,65,'Intermediate' from dual union all
select 13748,'013748','1982-084C','Kosmos-1402 reactor core','US-A reactor core',54393,'Reentered','1983-02-07 00:00:00','1982-12-29 00:00:00',89.38,239,249,65,'Intermediate' from dual union all
select 13749,'013749','1982-120C','radar error','',54429,'Spurious','1983-03-23 00:00:00','1983-02-02 00:00:00',89.38,199,289,50.53,'Intermediate' from dual union all
select 1375,'001375','1965-020AG','deb r/b S3','',11770,'Reentered','1965-10-14 00:00:00','1965-03-15 00:00:00',102.57,399,1362,55.9,'Intermediate' from dual union all
select 13750,'013750','1982-121A','Kosmos-1427','Taifun-1B',54430,'Reentered','1989-10-05 00:00:00','1983-01-28 00:00:00',93.99,443,497,65.84,'Intermediate' from dual union all
select 13751,'013751','1982-121B','S3M','S3M',54430,'Reentered','1988-03-26 00:00:00','1983-01-28 00:00:00',93.88,438,491,65.84,'Intermediate' from dual union all
select 13752,'013752','1982-120D','radar error','Dup of 13746',54429,'Spurious','1983-01-07 00:00:00','1983-01-01 00:00:00',89.47,193,304,50.52,'Intermediate' from dual union all
select 13753,'013753','1976-023K','P74-1 truss part','[UNID092] RCS 2.5',38975,'In Earth orbit','','1980-11-09 00:00:00',1420.89,35479,35498,.36,'Drift GEO' from dual union all
select 13754,'013754','1982-119E','KDU','KDU',54428,'Reentered','1983-10-27 00:00:00','1983-01-29 00:00:00',92.03,343,406,69.97,'Intermediate' from dual union all
select 13755,'013755','1982-119F','Blenda','Sunshade',54428,'Reentered','1983-03-03 00:00:00','1983-01-24 00:00:00',91.52,319,380,69.96,'Intermediate' from dual union all
select 13756,'013756','1982-119G','KDU part','KDU part',54428,'Reentered','1983-02-11 00:00:00','1983-01-07 00:00:00',92.15,346,414,69.97,'Intermediate' from dual union all
select 13757,'013757','1983-001A','Kosmos-1428','Parus',55758,'In Earth orbit','','1983-02-15 00:00:00',104.67,954,1004,82.92,'Intermediate' from dual union all
select 13758,'013758','1983-001B','S3M','S3M',55758,'In Earth orbit','','1990-03-21 00:00:00',104.55,953,994,82.91,'Intermediate' from dual union all
select 13759,'013759','1981-053GN','deb Kosmos-1275','',52576,'Reentered','2000-10-27 00:00:00','1993-08-06 00:00:00',99.84,714,788,82.88,'Intermediate' from dual union all
select 1376,'001376','1965-021E','deb DAPP','',11771,'Reentered','1970-01-01 00:00:00','1965-05-18 00:00:00',96.44,520,657,98.99,'Sun-Synch' from dual union all
select 13760,'013760','1981-053GP','deb Kosmos-1275','',52576,'Reentered','1991-11-09 00:00:00','1991-11-07 00:00:00',91.15,316,346,83,'Intermediate' from dual union all
select 13761,'013761','1983-002A','Kosmos-1429','Strela-1M',55759,'In Earth orbit','','1983-04-08 00:00:00',115.81,1466,1519,74.02,'Intermediate' from dual union all
select 13762,'013762','1983-002B','Kosmos-1430','Strela-1M',55759,'In Earth orbit','','1992-09-28 00:00:00',115.59,1467,1498,74.03,'Intermediate' from dual union all
select 13763,'013763','1983-002C','Kosmos-1431','Strela-1M',55759,'In Earth orbit','','1992-09-22 00:00:00',115.4,1465,1482,74.03,'Intermediate' from dual union all
select 13764,'013764','1983-002D','Kosmos-1432','Strela-1M',55759,'In Earth orbit','','1992-09-26 00:00:00',115.22,1463,1467,74.03,'Intermediate' from dual union all
select 13765,'013765','1983-002E','Kosmos-1433','Strela-1M',55759,'In Earth orbit','','1992-09-25 00:00:00',115.02,1446,1467,74.02,'Intermediate' from dual union all
select 13766,'013766','1983-002F','Kosmos-1434','Strela-1M',55759,'In Earth orbit','','1992-09-27 00:00:00',114.85,1431,1467,74.02,'Intermediate' from dual union all
select 13767,'013767','1983-002G','Kosmos-1435','Strela-1M',55759,'In Earth orbit','','1992-09-27 00:00:00',114.68,1414,1467,74.02,'Intermediate' from dual union all
select 13768,'013768','1983-002H','Kosmos-1436','Strela-1M',55759,'In Earth orbit','','1992-09-25 00:00:00',114.5,1399,1466,74.02,'Intermediate' from dual union all
select 13769,'013769','1983-002J','S3M','S3M',55759,'In Earth orbit','','1992-05-04 00:00:00',117.91,1479,1695,74,'Intermediate' from dual union all
select 1377,'001377','1965-038A','OPS 8386','DAPP 12 (Block 3)',11788,'Reentered','2012-03-09 00:00:00','1965-06-19 00:00:00',99.98,554,961,98.64,'Sun-Synch' from dual union all
select 13770,'013770','1983-003A','Kosmos-1437','Tselina-D',55760,'In Earth orbit','','1983-02-19 00:00:00',97.54,627,655,81.17,'Intermediate' from dual union all
select 13771,'013771','1983-003B','8A92ME','8A92ME',55760,'In Earth orbit','','1988-06-10 00:00:00',97.49,599,679,81.17,'Intermediate' from dual union all
select 13772,'013772','1968-097DY','deb Kosmos-252','',16824,'Reentered','2002-02-12 00:00:00','1993-08-08 00:00:00',102.22,536,1192,62.3,'Intermediate' from dual union all
select 13773,'013773','1982-118D','DMSP F-6 OLS Cover','',54427,'Reentered','1991-06-06 00:00:00','1983-01-21 00:00:00',101.27,818,820,98.72,'Sun-Synch' from dual union all
select 13774,'013774','1982-118E','DMSP F-6 OLS Cooler Cover','',54427,'Reentered','1991-10-29 00:00:00','1991-10-14 00:00:00',93.97,462,476,98.46,'Sun-Synch' from dual union all
select 13775,'013775','1970-091AR','deb Kosmos-375','',21568,'Reentered','2000-09-11 00:00:00','1980-10-24 00:00:00',100.93,787,819,74.08,'Intermediate' from dual union all
select 13776,'013776','1971-015CN','Unknown deb                       ','',24833,'In Earth orbit','','1993-08-07 00:00:00',107.04,670,1510,65.87,'Intermediate' from dual union all
select 13777,'013777','1983-004A','IRAS','IRAS',55761,'In Earth orbit','','1983-03-04 00:00:00',103.03,893,911,99.11,'Sun-Synch' from dual union all
select 13778,'013778','1983-004B','Delta 166','Delta SSPU DSV-3P-4B',55761,'In Earth orbit','','1990-02-18 00:00:00',102.37,855,887,100.06,'Sun-Synch' from dual;
insert into SATELLITE
select 13779,'013779','1983-005A','Kosmos-1438','Zenit-6',55762,'Landed','1983-02-07 00:00:00','1983-01-29 00:00:00',89.57,165,342,70.4,'Intermediate' from dual union all
select 1378,'001378','1965-038B','FW4S S/N 20021','FW4S',11788,'Reentered','1995-01-01 00:00:00','1965-08-29 00:00:00',100,553,964,98.62,'Sun-Synch' from dual union all
select 13780,'013780','1983-005B','11S510','11S510',55762,'Reentered','1983-01-30 00:00:00','1983-01-29 00:00:00',88.09,176,184,70.39,'Intermediate' from dual union all
select 13781,'013781','1981-028BJ','deb Kosmos-1260','',52551,'Reentered','1989-08-04 00:00:00','1989-08-03 00:00:00',88.92,199,244,65.02,'Intermediate' from dual union all
select 13782,'013782','1983-006A','Sakura 2A','CS 2A',55763,'In Earth orbit','','1983-03-19 00:00:00',1435.98,35783,35786,.05,'Stationary' from dual union all
select 13783,'013783','1983-004C','IRAS dewar cover','Infrared Cover (IRC)',55761,'In Earth orbit','','1990-05-16 00:00:00',102.92,888,906,99.04,'Sun-Synch' from dual union all
select 13784,'013784','1983-007A','Kosmos-1439','Feniks No. 956',55764,'Landed','1983-02-22 00:00:00','1983-02-13 00:00:00',89.02,165,288,70.35,'Intermediate' from dual union all
select 13785,'013785','1983-007B','11S510','11S510',55764,'Reentered','1983-02-10 00:00:00','1983-02-08 00:00:00',88.92,160,283,70.38,'Intermediate' from dual union all
select 13786,'013786','1983-006B','Star 37E S/N 40092','Star 37E',55763,'Reentered','1996-05-27 00:00:00','1992-10-02 00:00:00',145.84,223,5377,28.49,'Medium' from dual union all
select 13787,'013787','1983-005C','Blenda','Sunshade',55762,'Reentered','1983-02-12 00:00:00','1983-02-07 00:00:00',88.98,195,254,70.38,'Intermediate' from dual union all
select 13788,'013788','1983-005D','KDU part','KDU part',55762,'Reentered','1983-02-08 00:00:00','1983-02-07 00:00:00',88.77,187,241,70.38,'Intermediate' from dual union all
select 13789,'013789','1983-005E','KDU part','KDU part',55762,'Reentered','1983-02-11 00:00:00','1983-02-08 00:00:00',90.64,243,369,70.39,'Intermediate' from dual union all
select 1379,'001379','1965-038C','DAPP despin weight','',11788,'Reentered','1983-04-07 00:00:00','1965-08-23 00:00:00',99.93,558,953,98.65,'Sun-Synch' from dual union all
select 13790,'013790','1983-005F','KDU','KDU',55762,'Reentered','1983-02-13 00:00:00','1983-02-09 00:00:00',90.56,243,360,70.39,'Intermediate' from dual union all
select 13791,'013791','1983-008A','OPS 0252','PARCAE 5',55765,'In Earth orbit','','1997-11-21 00:00:00',107.41,802,1412,63.43,'Intermediate' from dual union all
select 13792,'013792','1983-008B','LIPS 2','SS D',55765,'In Earth orbit','','1997-11-22 00:00:00',107.48,822,1399,63.39,'Intermediate' from dual union all
select 13793,'013793','1983-009A','Kosmos-1440','Resurs-F1 17F41 No. 20',55766,'Landed','1983-02-24 00:00:00','1983-02-12 00:00:00',89.81,258,273,82.34,'Intermediate' from dual union all
select 13794,'013794','1983-009B','11S510','11S510',55766,'Reentered','1983-02-15 00:00:00','1983-02-12 00:00:00',88.83,201,233,82.34,'Intermediate' from dual union all
select 13795,'013795','1966-056CG','deb Pageos','RCS 60.9',13463,'Reentered','1987-04-25 00:00:00','1979-12-12 00:00:00',180.02,3019,5347,88.17,'Medium' from dual union all
select 13796,'013796','1981-053GQ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',105.75,935,1123,83.03,'Intermediate' from dual union all
select 13797,'013797','1981-053GR','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-29 00:00:00',104.59,935,1015,82.84,'Intermediate' from dual union all
select 13798,'013798','1981-053GS','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',105.58,951,1093,83.01,'Intermediate' from dual union all
select 13799,'013799','1981-053GT','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',105.7,948,1106,83.14,'Intermediate' from dual union all
select 138,'000138','61 Omicron 23','deb r/b Ablestar','',7273,'In Earth orbit','','1961-10-01 00:00:00',104.28,902,1020,66.82,'Intermediate' from dual union all
select 1380,'001380','1965-038D','DAPP despin weight','',11788,'Reentered','1966-05-02 00:00:00','1965-06-19 00:00:00',99.67,546,940,98.73,'Sun-Synch' from dual union all
select 13800,'013800','1981-053GU','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',107.45,950,1267,83.25,'Intermediate' from dual union all
select 13801,'013801','1981-053GV','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',104.1,916,989,82.82,'Intermediate' from dual union all
select 13802,'013802','1981-053GW','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',105.37,959,1065,82.81,'Intermediate' from dual union all
select 13803,'013803','1981-053GX','deb Kosmos-1275','',52576,'In Earth orbit','','1983-02-15 00:00:00',105.23,954,1057,82.85,'Intermediate' from dual union all
select 13804,'013804','1981-053GY','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',104.32,914,1012,82.97,'Intermediate' from dual union all
select 13805,'013805','1981-053GZ','deb Kosmos-1275','',52576,'Reentered','2002-12-30 00:00:00','1993-08-05 00:00:00',102.46,844,907,82.97,'Intermediate' from dual union all
select 13806,'013806','1981-053HA','deb Kosmos-1275','',52576,'Reentered','2017-09-12 00:00:00','1993-08-05 00:00:00',101.87,812,883,82.94,'Intermediate' from dual union all
select 13807,'013807','1981-053HB','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',104.69,951,1008,82.96,'Intermediate' from dual union all
select 13808,'013808','1981-053HC','deb Kosmos-1275','',52576,'In Earth orbit','','1983-02-14 00:00:00',105.75,971,1088,83.04,'Intermediate' from dual union all
select 13809,'013809','1981-053HD','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',102.07,801,913,82.85,'Intermediate' from dual union all
select 1381,'001381','1965-039A','Pegasus 2','Pegasus 2',11789,'Reentered Attached','1979-11-03 00:00:00','1965-07-03 00:00:00',97.15,508,737,31.76,'Intermediate' from dual union all
select 13810,'013810','1981-053HE','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',106.58,961,1176,83.02,'Intermediate' from dual union all
select 13811,'013811','1981-053HF','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-30 00:00:00',104.05,926,974,82.95,'Intermediate' from dual union all
select 13812,'013812','1981-053HG','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',104.14,901,1008,83.27,'Intermediate' from dual union all
select 13813,'013813','1981-053HH','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-03 00:00:00',104.07,904,998,82.95,'Intermediate' from dual union all
select 13814,'013814','1981-053HJ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',103.49,906,942,82.95,'Intermediate' from dual union all
select 13815,'013815','1981-053HK','deb Kosmos-1275','',52576,'In Earth orbit','','1983-02-14 00:00:00',106.88,972,1193,83.03,'Intermediate' from dual union all
select 13816,'013816','1981-053HL','deb Kosmos-1275','',52576,'In Earth orbit','','1983-02-14 00:00:00',106.68,909,1236,82.99,'Intermediate' from dual union all
select 13817,'013817','1981-053HM','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',104.06,926,975,82.95,'Intermediate' from dual union all
select 13818,'013818','1983-010A','Kosmos-1441','Tselina-D',55767,'Reentered','2014-11-08 00:00:00','1983-03-18 00:00:00',97.39,629,639,81.11,'Intermediate' from dual union all
select 13819,'013819','1983-010B','8A92ME','8A92ME',55767,'In Earth orbit','','1991-03-28 00:00:00',96.95,573,653,81.1,'Intermediate' from dual union all
select 1382,'001382','1965-040A','Kosmos-67','Zenit-4 No. 10',11790,'Landed','1965-06-02 00:00:00','1965-05-28 00:00:00',89.91,203,337,51.83,'Intermediate' from dual union all
select 13820,'013820','1981-097U','Kosmos-1311 SS 17','ESO',52620,'Reentered','1983-02-28 00:00:00','1983-02-16 00:00:00',92.16,367,394,82.94,'Intermediate' from dual union all
select 13821,'013821','1981-097V','Kosmos-1311 SS 18','ESO',52620,'Reentered','1983-02-28 00:00:00','1983-02-20 00:00:00',91.77,349,374,82.99,'Intermediate' from dual union all
select 13822,'013822','1981-097W','Kosmos-1311 SS 19','ESO',52620,'Reentered','1983-02-28 00:00:00','1983-02-19 00:00:00',91.93,357,382,82.99,'Intermediate' from dual union all
select 13823,'013823','1981-097X','Kosmos-1311 SS 20','ESO',52620,'Reentered','1983-02-28 00:00:00','1983-02-16 00:00:00',92.15,369,391,82.95,'Intermediate' from dual union all
select 13824,'013824','1981-097Y','Kosmos-1311 SS 21','ESO',52620,'Reentered','1983-02-28 00:00:00','1983-02-16 00:00:00',92.13,367,391,82.94,'Intermediate' from dual union all
select 13825,'013825','1978-026FU','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',103.62,788,1071,98.8,'Sun-Synch' from dual union all
select 13826,'013826','1978-026FV','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',100.51,736,830,98.5,'Sun-Synch' from dual union all
select 13827,'013827','1978-026FW','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',100.58,716,857,98.5,'Sun-Synch' from dual union all
select 13828,'013828','1981-097Z','Kosmos-1311 SS 22','ESO',52620,'Reentered','1983-02-28 00:00:00','1983-02-21 00:00:00',91.7,345,370,82.93,'Intermediate' from dual union all
select 13829,'013829','1983-011A','Tenma','ASTRO-B',55768,'Reentered','1988-12-17 00:00:00','1983-03-23 00:00:00',94.5,488,501,31.49,'Intermediate' from dual union all
select 1383,'001383','1965-040B','11A57I','11A57I',11790,'Reentered','1965-06-04 00:00:00','1965-05-28 00:00:00',89.35,187,298,51.82,'Intermediate' from dual union all
select 13830,'013830','1983-011B','M-3A','M-3A',55768,'Reentered','1989-01-19 00:00:00','1983-03-22 00:00:00',94.36,470,506,31.7,'Intermediate' from dual union all
select 13831,'013831','1981-097AA','Kosmos-1311 SS 23','ESO',52620,'Reentered','1983-02-28 00:00:00','1983-02-21 00:00:00',91.62,341,366,82.88,'Intermediate' from dual union all
select 13832,'013832','1981-097AB','Kosmos-1311 SS 24','ESO',52620,'Reentered','1983-02-28 00:00:00','1983-02-21 00:00:00',91.67,342,371,82.89,'Intermediate' from dual union all
select 13833,'013833','1982-034C','Kosmos-1351 SS 1','ESO',54343,'Reentered','1983-02-27 00:00:00','1983-02-22 00:00:00',90.21,255,315,50.71,'Intermediate' from dual union all
select 13834,'013834','1983-008C','SSB cover','SSU cover',55765,'In Earth orbit','','1997-11-22 00:00:00',107.42,796,1419,63.43,'Intermediate' from dual union all
select 13835,'013835','1983-008D','SSA cover','SSU cover',55765,'In Earth orbit','','1997-11-22 00:00:00',107.42,796,1419,63.43,'Intermediate' from dual union all
select 13836,'013836','1982-034D','Kosmos-1351 SS 2','ESO',54343,'Reentered','1983-02-25 00:00:00','1983-02-22 00:00:00',90.13,254,308,50.69,'Intermediate' from dual union all
select 13837,'013837','1982-034E','Kosmos-1351 SS 3','ESO',54343,'Reentered','1983-02-26 00:00:00','1983-02-22 00:00:00',90.04,254,299,50.63,'Intermediate' from dual union all
select 13838,'013838','1982-034F','Kosmos-1351 SS 4','ESO',54343,'Reentered','1983-02-25 00:00:00','1983-02-22 00:00:00',90.03,253,298,50.69,'Intermediate' from dual union all
select 13839,'013839','1982-034G','Kosmos-1351 SS 5','ESO',54343,'Reentered','1983-02-25 00:00:00','1983-02-21 00:00:00',90.07,258,298,50.64,'Intermediate' from dual union all
select 1384,'001384','1965-040C','deb Kosmos-67','',11790,'Reentered','1965-05-31 00:00:00','1965-05-25 00:00:00',90.14,197,366,51.86,'Intermediate' from dual union all
select 13840,'013840','1982-034H','Kosmos-1351 SS 6','ESO',54343,'Reentered','1983-03-03 00:00:00','1983-02-21 00:00:00',90.13,249,313,50.64,'Intermediate' from dual union all
select 13841,'013841','1982-034J','Kosmos-1351 SS 7','ESO',54343,'Reentered','1983-02-27 00:00:00','1983-02-22 00:00:00',90.38,265,322,50.67,'Intermediate' from dual union all
select 13842,'013842','1982-034K','Kosmos-1351 SS 8','ESO',54343,'Reentered','1983-03-02 00:00:00','1983-02-22 00:00:00',90.17,259,307,50.66,'Intermediate' from dual union all
select 13843,'013843','1983-007C','deb Kosmos-1439','deb Feniks AO',55764,'Reentered','1983-02-22 00:00:00','1983-02-22 00:00:00',89.1,162,298,70.33,'Intermediate' from dual union all
select 13844,'013844','1983-008E','SSA','SSU',55765,'In Earth orbit','','1992-11-29 00:00:00',107.43,946,1269,63.4,'Intermediate' from dual union all
select 13845,'013845','1983-008F','SSB','SSU',55765,'In Earth orbit','','1990-05-17 00:00:00',107.44,980,1237,63.43,'Intermediate' from dual union all
select 13846,'013846','1983-011C','ASTRO-B despin weight','',55768,'Reentered','1983-03-03 00:00:00','1983-02-24 00:00:00',94.46,466,519,31.66,'Intermediate' from dual union all
select 13847,'013847','1983-009C','KDU','KDU',55766,'Reentered','1983-03-15 00:00:00','1983-02-24 00:00:00',89.74,254,269,82.34,'Intermediate' from dual union all
select 13848,'013848','1983-009D','Blenda','Sunshade',55766,'Reentered','1983-03-02 00:00:00','1983-02-24 00:00:00',89.58,239,269,82.35,'Intermediate' from dual union all
select 13849,'013849','1983-008G','SSC cover','SSU cover',55765,'In Earth orbit','','1997-11-22 00:00:00',107.43,803,1413,63.43,'Intermediate' from dual union all
select 1385,'001385','1965-039B','Apollo BP-26','Apollo BP-26 CM',11789,'Reentered','1989-07-08 00:00:00','1965-07-04 00:00:00',97.2,511,739,31.76,'Intermediate' from dual union all
select 13850,'013850','1983-012A','Kosmos-1442','Oktan No. 248',55769,'Landed','1983-04-11 00:00:00','1983-03-04 00:00:00',89.47,162,335,67.14,'Intermediate' from dual union all
select 13851,'013851','1983-012B','11S510','11S510',55769,'Reentered','1983-03-01 00:00:00','1983-02-27 00:00:00',88.95,161,284,67.14,'Intermediate' from dual union all
select 13852,'013852','1982-034L','Kosmos-1351 SS 9','ESO',54343,'Reentered','1983-02-26 00:00:00','1983-02-26 00:00:00',88.68,204,215,50.66,'Intermediate' from dual union all
select 13853,'013853','1982-034M','Kosmos-1351 SS 10','ESO',54343,'Reentered','1983-02-26 00:00:00','1983-02-26 00:00:00',88.26,178,199,50.66,'Intermediate' from dual union all
select 13854,'013854','1982-034N','Kosmos-1351 SS 11','ESO',54343,'Reentered','1983-02-26 00:00:00','1983-02-25 00:00:00',88.93,215,229,50.7,'Intermediate' from dual union all
select 13855,'013855','1982-034P','Kosmos-1351 SS 12','ESO',54343,'Reentered','1983-02-26 00:00:00','1983-02-25 00:00:00',88.71,205,216,50.68,'Intermediate' from dual union all
select 13856,'013856','1982-034Q','Kosmos-1351 SS 13','ESO',54343,'Reentered','1983-02-26 00:00:00','1983-02-25 00:00:00',88.81,208,224,50.6,'Intermediate' from dual union all
select 13857,'013857','1982-034R','Kosmos-1351 SS 14','ESO',54343,'Reentered','1983-03-05 00:00:00','1983-02-28 00:00:00',88.56,196,211,50.64,'Intermediate' from dual union all
select 13858,'013858','1982-034S','Kosmos-1351 SS 15','ESO',54343,'Reentered','1983-02-27 00:00:00','1983-02-25 00:00:00',88.71,202,220,50.7,'Intermediate' from dual union all
select 13859,'013859','1982-034T','Kosmos-1351 SS 16','ESO',54343,'Reentered','1983-02-26 00:00:00','1983-02-25 00:00:00',88.81,207,224,50.7,'Intermediate' from dual union all
select 1386,'001386','1965-041A','OPS 5236','GAMBIT SV 968',11791,'Reentered','1965-06-01 00:00:00','1965-05-29 00:00:00',88.65,149,266,95.78,'Sun-Synch' from dual union all
select 13860,'013860','1982-034U','Kosmos-1351 SS 17','ESO',54343,'Reentered','1983-02-26 00:00:00','1983-02-25 00:00:00',88.73,203,221,50.7,'Intermediate' from dual union all
select 13861,'013861','1982-034V','Kosmos-1351 SS 18','ESO',54343,'Reentered','1983-02-26 00:00:00','1983-02-25 00:00:00',88.7,206,215,50.7,'Intermediate' from dual union all
select 13862,'013862','1982-034W','Kosmos-1351 SS 19','ESO',54343,'Reentered','1983-02-26 00:00:00','1983-02-25 00:00:00',88.63,199,215,50.7,'Intermediate' from dual union all
select 13863,'013863','1982-034X','Kosmos-1351 SS 20','ESO',54343,'Reentered','1983-02-26 00:00:00','1983-02-25 00:00:00',88.36,178,209,50.6,'Intermediate' from dual union all
select 13864,'013864','1982-034Y','Kosmos-1351 SS 21','ESO',54343,'Reentered','1983-02-26 00:00:00','1983-02-25 00:00:00',88.38,186,202,50.6,'Intermediate' from dual union all
select 13865,'013865','1982-034Z','Kosmos-1351 SS 22','ESO',54343,'Reentered','1983-03-03 00:00:00','1983-02-28 00:00:00',88.93,214,230,50.69,'Intermediate' from dual union all
select 13866,'013866','1982-034AA','Kosmos-1351 SS 23','ESO',54343,'Reentered','1983-02-26 00:00:00','1983-02-25 00:00:00',89.15,221,244,50.7,'Intermediate' from dual union all
select 13867,'013867','1982-034AB','Kosmos-1351 SS 24','ESO',54343,'Reentered','1983-02-25 00:00:00','1983-02-25 00:00:00',88.28,181,198,50.6,'Intermediate' from dual union all
select 13868,'013868','1983-013A','Kosmos-1443','TKS-M No. 16401',55770,'Deorbited','1983-09-19 00:00:00','1983-04-01 00:00:00',90.91,319,320,51.61,'Intermediate' from dual;
insert into SATELLITE
select 13869,'013869','1983-013B','8S812','8S812',55770,'Reentered','1983-03-05 00:00:00','1983-03-04 00:00:00',88.15,171,195,51.65,'Intermediate' from dual union all
select 1387,'001387','1965-040D','deb Kosmos-67','',11790,'Reentered','1965-05-31 00:00:00','1965-05-25 00:00:00',89.7,190,329,51.86,'Intermediate' from dual union all
select 13870,'013870','1983-014A','Kosmos-1444','Zenit-6',55771,'Landed','1983-03-16 00:00:00','1983-03-04 00:00:00',92.25,355,415,72.85,'Intermediate' from dual union all
select 13871,'013871','1983-014B','11S510','11S510',55771,'Reentered','1983-03-11 00:00:00','1983-03-04 00:00:00',89.9,186,352,72.84,'Intermediate' from dual union all
select 13872,'013872','1983-013C','TKS adapter','TKS adapter',55770,'Reentered','1983-03-04 00:00:00','1983-03-02 00:00:00',88.69,184,236,51.6,'Intermediate' from dual union all
select 13873,'013873','1982-120E','Kosmos-1426 part','Terilen AO part',54429,'Reentered','1983-03-08 00:00:00','1983-03-05 00:00:00',89.31,198,283,50.53,'Intermediate' from dual union all
select 13874,'013874','1983-008H','SSC','SSU',55765,'In Earth orbit','','1990-05-17 00:00:00',107.44,957,1259,63.47,'Intermediate' from dual union all
select 13875,'013875','1983-015A','Molniya-3','Molniya-3 No. 34',55772,'In Earth orbit','','1983-04-10 00:00:00',717.8,439,39917,62.85,'Molniya' from dual union all
select 13876,'013876','1983-015B','11S510','11S510',55772,'Reentered','1983-03-27 00:00:00','1983-03-18 00:00:00',90.38,190,397,62.83,'Intermediate' from dual union all
select 13877,'013877','1983-015C','BOZ','BOZ',55772,'Reentered','1983-04-01 00:00:00','1983-03-18 00:00:00',90.61,204,404,62.85,'Intermediate' from dual union all
select 13878,'013878','1983-016A','Ekran','Ekran No. 18L',55773,'In Earth orbit','','1983-04-15 00:00:00',1435.96,35759,35809,.09,'Stationary' from dual union all
select 13879,'013879','1983-016B','8S812','8S812',55773,'Reentered','1983-03-14 00:00:00','1983-03-13 00:00:00',88.18,179,190,51.61,'Intermediate' from dual union all
select 1388,'001388','1965-042A','IMP 3','IMP C',11792,'Deep Space','1965-05-30 00:00:00','1965-07-29 00:00:00',8558.78,195,264252,33.86,'Deep Space' from dual union all
select 13880,'013880','1983-016C','Perekhodnik','Sredniy perekhodnik',55773,'Reentered','1983-03-13 00:00:00','1983-03-12 00:00:00',88.32,185,198,51.6,'Intermediate' from dual union all
select 13881,'013881','1983-015D','deb Blok-I','',55772,'Reentered','1983-03-26 00:00:00','1983-03-19 00:00:00',90.11,196,364,62.82,'Intermediate' from dual union all
select 13882,'013882','1983-015E','Blok-ML','Blok-ML No. 318',55772,'In Earth orbit','','1983-04-17 00:00:00',732,443,40611,62.85,'Molniya' from dual union all
select 13883,'013883','1983-017A','Kosmos-1445','BOR-4 No. 403',55774,'Landed','1983-03-16 00:00:00','1983-03-16 00:00:00',88.5,177,224,50.69,'Intermediate' from dual union all
select 13884,'013884','1983-016D','SOZ','SOZ',55773,'Reentered','1986-10-27 00:00:00','1983-04-11 00:00:00',629.78,340,35580,47.42,'GEO Transfer' from dual union all
select 13885,'013885','1983-016E','SOZ','SOZ',55773,'Reentered','1986-04-02 00:00:00','1983-04-12 00:00:00',628.3,285,35559,47.39,'GEO Transfer' from dual union all
select 13886,'013886','1983-018A','Kosmos-1446','Zenit-6',55775,'Landed','1983-03-30 00:00:00','1983-03-18 00:00:00',89.6,226,284,69.94,'Intermediate' from dual union all
select 13887,'013887','1983-018B','11S510','11S510',55775,'Reentered','1983-03-29 00:00:00','1983-03-18 00:00:00',90.01,223,327,69.96,'Intermediate' from dual union all
select 13888,'013888','1983-017B','S3M','S3M',55774,'Reentered','1983-03-17 00:00:00','1983-03-16 00:00:00',88.46,185,212,50.69,'Intermediate' from dual union all
select 13889,'013889','1983-017C','deb BOR-4','',55774,'Reentered','1983-03-18 00:00:00','1983-03-17 00:00:00',87.63,151,163,50.68,'Intermediate' from dual union all
select 1389,'001389','1965-027D','SNAPSHOT Heat Shield','Heat shield',11777,'In Earth orbit','','1965-05-25 00:00:00',111.48,1267,1323,90.19,'Polar' from dual union all
select 13890,'013890','1983-019A','Molniya-1','Molniya-1',55776,'In Earth orbit','','1983-04-16 00:00:00',717.68,446,39904,62.84,'Molniya' from dual union all
select 13891,'013891','1983-014C','Blenda','Sunshade',55771,'Reentered','1983-03-16 00:00:00','1983-03-16 00:00:00',92.3,357,418,72.93,'Intermediate' from dual union all
select 13892,'013892','1983-014D','KDU part','KDU part',55771,'Reentered','1983-04-20 00:00:00','1983-03-16 00:00:00',92.25,355,415,72.85,'Intermediate' from dual union all
select 13893,'013893','1983-014E','KDU part','KDU part',55771,'Reentered','1983-05-26 00:00:00','1983-04-01 00:00:00',91.81,334,393,72.84,'Intermediate' from dual union all
select 13894,'013894','1983-014F','KDU','KDU',55771,'Reentered','1984-02-04 00:00:00','1983-04-01 00:00:00',92.17,353,409,72.85,'Intermediate' from dual union all
select 13895,'013895','1983-019B','11S510','11S510',55776,'Reentered','1983-04-04 00:00:00','1983-03-23 00:00:00',90.65,201,411,62.82,'Intermediate' from dual union all
select 13896,'013896','1983-019C','BOZ','BOZ',55776,'Reentered','1983-03-31 00:00:00','1983-03-23 00:00:00',90.41,185,405,62.82,'Intermediate' from dual union all
select 13897,'013897','1983-019D','Blok-ML','Blok-ML',55776,'In Earth orbit','','1983-04-16 00:00:00',732.79,436,40657,62.85,'Molniya' from dual union all
select 13898,'013898','1983-019E','deb BOZ','',55776,'Reentered','1983-03-31 00:00:00','1983-03-24 00:00:00',90.54,202,399,62.81,'Intermediate' from dual union all
select 13899,'013899','1982-020F','Blok DM No. 35L','11S86 No. 35L',54329,'In Earth orbit','','1993-06-07 00:00:00',1459.88,36138,36364,9.64,'Inclined Drift' from dual union all
select 139,'000139','61 Omicron 24','deb r/b Ablestar','',7273,'Reentered','1978-07-08 00:00:00','1961-09-06 00:00:00',100.17,548,986,67.14,'Intermediate' from dual union all
select 1390,'001390','1965-043A','Gemini IV','Gemini SC4',11793,'Landed','1965-06-07 00:00:00','1965-06-03 00:00:00',89.03,165,289,32.5,'Intermediate' from dual union all
select 13900,'013900','1979-015D','Blok DM','11S86 No. 13L',47499,'In Earth orbit','','1979-10-31 00:00:00',1420.89,35431,35546,.12,'Drift GEO' from dual union all
select 13901,'013901','1983-020A','Astron','1A No. 602',55777,'Deep Space','1983-03-24 00:00:00','1985-03-20 00:00:00',5915.79,25128,178817,79.76,'Deep Space' from dual union all
select 13902,'013902','1983-020B','SOZ','SOZ',55777,'Reentered','1985-06-18 00:00:00','1984-09-03 00:00:00',99.27,219,1229,51.53,'Intermediate' from dual union all
select 13903,'013903','1983-020C','SOZ','SOZ',55777,'Reentered','1985-08-05 00:00:00','1984-09-03 00:00:00',99.27,219,1229,51.53,'Intermediate' from dual union all
select 13904,'013904','1981-088G','deb Kosmos-1305','',52611,'In Earth orbit','','1999-12-08 00:00:00',253.57,1601,12184,64.04,'Medium' from dual union all
select 13905,'013905','1967-001Z','deb unknown                       ','[UNID093] RCS 1.6',15256,'Reentered','1992-09-20 00:00:00','1992-09-19 00:00:00',89.6,126,384,23.42,'Equatorial' from dual union all
select 13906,'013906','1965-108U','deb Transtage 8','',11858,'In Earth orbit','','1983-01-31 00:00:00',528.89,574,29998,26.56,'Highly Ellip' from dual union all
select 13907,'013907','1977-108D','SVM-6A','SVM-6A',42236,'In Earth orbit','','1998-10-10 00:00:00',1472.25,36099,36884,15.53,'Inclined Drift' from dual union all
select 13908,'013908','1981-088H','deb Kosmos-1305','',52611,'In Earth orbit','','1999-07-31 00:00:00',264.91,1757,12813,64,'Medium' from dual union all
select 13909,'013909','1981-088J','deb Kosmos-1305','',52611,'In Earth orbit','','1999-07-31 00:00:00',255.59,972,12954,63.6,'Medium' from dual union all
select 1391,'001391','1965-043B','Titan GT-4 Stage 2','LR91AJ5 GT-4 (65-599)',11793,'Reentered','1965-06-05 00:00:00','1965-06-03 00:00:00',88.92,161,282,32.5,'Intermediate' from dual union all
select 13910,'013910','1981-088K','deb Kosmos-1305','',52611,'In Earth orbit','','1999-07-31 00:00:00',265.19,1676,12914,64,'Medium' from dual union all
select 13911,'013911','1981-088L','deb Kosmos-1305','',52611,'In Earth orbit','','1999-07-31 00:00:00',243.57,1185,11897,63.5,'Medium' from dual union all
select 13912,'013912','1965-108V','OV2-3','',11858,'In Earth orbit','','1966-04-29 00:00:00',610.7,283,34649,26.76,'GEO Transfer' from dual union all
select 13913,'013913','1979-015E','SOZ','SOZ',47499,'Reentered','1980-02-04 00:00:00','1979-02-28 00:00:00',627.45,200,35600,47,'GEO Transfer' from dual union all
select 13914,'013914','1979-015F','SOZ','SOZ',47499,'Reentered','1979-11-12 00:00:00','1979-02-28 00:00:00',627.45,200,35600,47,'GEO Transfer' from dual union all
select 13915,'013915','1978-116C','Star 37E S/N 40070','Star 37E',45311,'Reentered','1987-12-23 00:00:00','1978-12-19 00:00:00',631.04,185,35800,26.9,'GEO Transfer' from dual union all
select 13916,'013916','1983-021A','Kosmos-1447','Nadezhda 11F643N No. 530',55778,'In Earth orbit','','1983-05-01 00:00:00',104.82,958,1014,82.94,'Intermediate' from dual union all
select 13917,'013917','1983-021B','S3M','S3M',55778,'In Earth orbit','','1990-06-19 00:00:00',104.68,958,1001,82.94,'Intermediate' from dual union all
select 13918,'013918','1981-028BK','deb Kosmos-1260','',52551,'Reentered','1989-06-04 00:00:00','1989-06-02 00:00:00',90.46,287,308,64.92,'Intermediate' from dual union all
select 13919,'013919','1981-028BL','deb Kosmos-1260','',52551,'Reentered','1989-11-13 00:00:00','1989-11-12 00:00:00',88.21,176,196,64.89,'Intermediate' from dual union all
select 1392,'001392','1965-020AH','S3 large debris','RCS 0.35',11770,'In Earth orbit','','1965-06-04 00:00:00',110.24,552,1924,55.95,'Intermediate' from dual union all
select 13920,'013920','1981-028BM','deb Kosmos-1260','',52551,'Reentered','1989-10-11 00:00:00','1989-10-10 00:00:00',91.12,321,338,64.82,'Intermediate' from dual union all
select 13921,'013921','1981-028BN','deb Kosmos-1260','',52551,'Reentered','1990-01-06 00:00:00','1990-01-05 00:00:00',87.32,138,146,64.95,'Intermediate' from dual union all
select 13922,'013922','1981-028BP','deb Kosmos-1260','',52551,'Reentered','1990-12-31 00:00:00','1990-12-29 00:00:00',90.44,286,307,64.91,'Intermediate' from dual union all
select 13923,'013923','1983-022A','NOAA 8','NOAA E',55779,'In Earth orbit','','1983-05-04 00:00:00',101.23,805,829,98.75,'Sun-Synch' from dual union all
select 13924,'013924','1983-022B','NOAA 8 solar array band','',55779,'Reentered','1991-10-12 00:00:00','1991-10-12 00:00:00',90.48,283,314,98.48,'Sun-Synch' from dual union all
select 13925,'013925','1980-030AH','deb Kosmos-1174','',50240,'Reentered','1991-03-10 00:00:00','1991-03-10 00:00:00',89.5,230,269,66.09,'Intermediate' from dual union all
select 13926,'013926','1980-030AJ','deb Kosmos-1174','',50240,'Reentered','1988-08-31 00:00:00','1988-08-31 00:00:00',89.73,254,269,66.03,'Intermediate' from dual union all
select 13927,'013927','1980-030AK','deb Kosmos-1174','',50240,'Reentered','1989-03-15 00:00:00','1989-03-12 00:00:00',91.35,297,385,65.74,'Intermediate' from dual union all
select 13928,'013928','1980-030AL','deb Kosmos-1174','',50240,'Reentered','1988-06-21 00:00:00','1988-06-20 00:00:00',90.12,258,303,65.62,'Intermediate' from dual union all
select 13929,'013929','1980-030AM','deb Kosmos-1174','',50240,'Reentered','1983-10-29 00:00:00','1983-10-26 00:00:00',89.82,243,289,65.7,'Intermediate' from dual union all
select 1393,'001393','1965-044A','Luna-6','E-6 No. 7',11794,'Deep Space','1965-06-11 00:00:00','1965-06-08 00:00:00',88.61,164,248,64.76,'Intermediate' from dual union all
select 13930,'013930','1980-030AN','deb Kosmos-1174','',50240,'Reentered','1991-05-02 00:00:00','1991-09-15 00:00:00',171.62,981,6722,85.06,'Medium' from dual union all
select 13931,'013931','1980-030AP','deb Kosmos-1174','',50240,'Reentered','1991-02-18 00:00:00','1991-02-18 00:00:00',89.39,233,255,66.14,'Intermediate' from dual union all
select 13932,'013932','1980-030AQ','deb Kosmos-1174','',50240,'Reentered','1993-09-09 00:00:00','1992-10-10 00:00:00',96.69,460,740,65.64,'Intermediate' from dual union all
select 13933,'013933','1972-058JA','deb Delta 89','',27990,'Reentered','1989-09-24 00:00:00','1989-09-23 00:00:00',90.13,273,288,99.94,'Retrograde' from dual union all
select 13934,'013934','1972-058JB','deb Delta 89','',27990,'Reentered','1986-08-05 00:00:00','1986-06-19 00:00:00',98.65,384,1005,102.35,'Retrograde' from dual union all
select 13935,'013935','1972-058JC','deb Delta 89','',27990,'Reentered','1985-05-27 00:00:00','1985-05-26 00:00:00',89.79,262,267,102.31,'Retrograde' from dual union all
select 13936,'013936','1970-089CV','deb Kosmos-374 [*}','',21566,'Reentered','1983-10-20 00:00:00','1983-10-19 00:00:00',89.77,254,273,62.76,'Intermediate' from dual union all
select 13937,'013937','1982-115AD','deb Blok-L','',54424,'Reentered','1983-04-21 00:00:00','1983-04-17 00:00:00',89.12,215,247,62.83,'Intermediate' from dual union all
select 13938,'013938','1982-115AE','deb Blok-L','',54424,'Reentered','1984-05-05 00:00:00','1984-05-04 00:00:00',87.94,163,182,62.85,'Intermediate' from dual union all
select 13939,'013939','1973-023C','Star 37C S/N 40021','Star 37C S/N 40021',30625,'Reentered','1990-05-28 00:00:00','1973-04-20 00:00:00',644.82,212,36480,26.7,'GEO Transfer' from dual union all
select 1394,'001394','1965-044B','BOZ','BOZ',11794,'Reentered','1965-06-12 00:00:00','1965-06-08 00:00:00',88.61,164,248,64.76,'Intermediate' from dual union all
select 13940,'013940','1979-072C','Star 37E S/N 40083','Star 37E',47556,'Reentered','1994-06-13 00:00:00','1992-10-01 00:00:00',224.53,185,11532,24.36,'Medium' from dual union all
select 13941,'013941','61 Omicron 263','deb r/b Ablestar','',7273,'Reentered','1988-09-21 00:00:00','1988-09-20 00:00:00',90.23,269,303,66.65,'Intermediate' from dual union all
select 13942,'013942','61 Omicron 264','deb r/b Ablestar','',7273,'In Earth orbit','','1993-07-28 00:00:00',110.97,967,1576,65.86,'Intermediate' from dual union all
select 13943,'013943','1976-126BM','deb Kosmos-397','[UNID094] RCS 0.04',39078,'In Earth orbit','','1993-08-03 00:00:00',111.54,559,2036,65.73,'Intermediate' from dual union all
select 13944,'013944','1971-015CP','deb Kosmos-397','',24833,'Reentered','1989-03-24 00:00:00','1989-03-22 00:00:00',90.98,313,333,65.94,'Intermediate' from dual union all
select 13945,'013945','1980-030AR','deb Kosmos-1174','',50240,'Reentered','1983-09-30 00:00:00','1983-09-26 00:00:00',90.14,251,312,65.99,'Intermediate' from dual union all
select 13946,'013946','1976-067BE','deb Kosmos-839','',39019,'In Earth orbit','','1981-05-25 00:00:00',91.52,338,361,51.63,'Intermediate' from dual union all
select 13947,'013947','1976-067BF','deb Kosmos-839','',39019,'In Earth orbit','','1983-04-24 00:00:00',117.07,969,2130,65.81,'Intermediate' from dual union all
select 13948,'013948','1977-121BD','deb Kosmos-970','',42249,'In Earth orbit','','1982-03-18 00:00:00',93.59,394,507,72.8,'Intermediate' from dual union all
select 13949,'013949','1983-023A','Kosmos-1448','Parus',55780,'In Earth orbit','','1983-05-12 00:00:00',104.74,960,1005,82.96,'Intermediate' from dual union all
select 1395,'001395','1965-044C','11S510','11S510',11794,'Reentered','1965-06-10 00:00:00','1965-06-08 00:00:00',88.61,164,248,64.76,'Intermediate' from dual union all
select 13950,'013950','1983-023B','S3M','S3M',55780,'In Earth orbit','','1991-03-28 00:00:00',104.65,950,1006,82.95,'Intermediate' from dual union all
select 13951,'013951','1983-018C','KDU','KDU',55775,'Reentered','1983-04-04 00:00:00','1983-03-30 00:00:00',88.82,198,234,69.97,'Intermediate' from dual union all
select 13952,'013952','1983-018D','Blenda','Sunshade',55775,'Reentered','1983-03-30 00:00:00','1983-03-30 00:00:00',88.5,183,217,69.93,'Intermediate' from dual union all
select 13953,'013953','1983-018E','KDU part','KDU part',55775,'Reentered','1983-03-31 00:00:00','1983-03-30 00:00:00',88.64,189,226,69.95,'Intermediate' from dual union all
select 13954,'013954','1982-113F','Blok DM','11S86 No. 29L',54422,'In Earth orbit','','1993-06-07 00:00:00',1475.76,36455,36664,8.38,'Inclined Drift' from dual union all
select 13955,'013955','1983-024A','Kosmos-1449','Zenit-6',55781,'Landed','1983-04-15 00:00:00','1983-04-07 00:00:00',92.25,356,413,72.87,'Intermediate' from dual union all
select 13956,'013956','1983-024B','11S510','11S510',55781,'Reentered','1983-04-09 00:00:00','1983-04-02 00:00:00',89.87,191,345,72.87,'Intermediate' from dual union all
select 13957,'013957','1965-108W','deb Transtage 8','',11858,'In Earth orbit','','1983-01-31 00:00:00',527.39,576,29914,26.72,'Highly Ellip' from dual union all
select 13958,'013958','1965-108X','deb Transtage 8','',11858,'In Earth orbit','','2004-08-19 00:00:00',534.27,538,30327,26.99,'Highly Ellip' from dual;
insert into SATELLITE
select 13959,'013959','1978-083G','deb Kosmos-1030','',45278,'In Earth orbit','','1983-07-12 00:00:00',721.76,2948,37603,63.72,'Molniya' from dual union all
select 1396,'001396','1965-045A','OPS 8425','CORONA J-20',11795,'Reentered Attached','1965-06-22 00:00:00','1965-06-11 00:00:00',89.77,174,353,75.07,'Intermediate' from dual union all
select 13960,'013960','1979-058H','deb Kosmos-1109','',47542,'Reentered','2012-05-15 00:00:00','1980-02-25 00:00:00',720.4,976,39508,63.31,'Molniya' from dual union all
select 13961,'013961','1981-071F','deb Kosmos-1285','',52594,'In Earth orbit','','1981-12-14 00:00:00',726.84,743,40058,63.25,'Molniya' from dual union all
select 13962,'013962','1983-024C','Sensor cover','Sensor cover',55781,'Reentered','1983-04-01 00:00:00','1983-03-31 00:00:00',89.63,188,325,72.87,'Intermediate' from dual union all
select 13963,'013963','1983-024D','Sensor cover','Sensor cover',55781,'Reentered','1983-04-10 00:00:00','1983-04-03 00:00:00',89.78,188,339,72.87,'Intermediate' from dual union all
select 13964,'013964','1983-025A','Molniya-1','Molniya-1 11F658T No. 68',55782,'Reentered','2000-01-26 00:00:00','1983-05-07 00:00:00',717.72,522,39830,62.97,'Molniya' from dual union all
select 13965,'013965','1983-025B','11S510','11S510',55782,'Reentered','1983-05-01 00:00:00','1983-04-09 00:00:00',90.95,230,412,62.95,'Intermediate' from dual union all
select 13966,'013966','1983-025C','BOZ','BOZ',55782,'Reentered','1983-04-16 00:00:00','1983-04-08 00:00:00',90.53,183,418,62.95,'Intermediate' from dual union all
select 13967,'013967','1983-025D','Blok-ML','Blok-ML No. 76714-462',55782,'Reentered','2000-08-16 00:00:00','1983-05-03 00:00:00',699.33,515,38926,62.85,'Molniya' from dual union all
select 13968,'013968','1983-026A','Challenger (STS-6)','OV-099',55783,'Landed','1983-04-09 00:00:00','1983-04-06 00:00:00',90.19,279,289,28.47,'Intermediate' from dual union all
select 13969,'013969','1983-026B','TDRS 1','TDRS A',55783,'In Earth orbit','','1984-04-01 00:00:00',1436.27,35779,35801,1.37,'Stationary' from dual union all
select 1397,'001397','1965-020AJ','deb Blok Kosmos','',11770,'Reentered','1968-04-06 00:00:00','1965-06-02 00:00:00',107.41,389,1825,56.11,'Intermediate' from dual union all
select 13970,'013970','1983-026C','IUS-1 SRM-2','Orbus 6E',55783,'In Earth orbit','','1983-05-07 00:00:00',1089.6,22032,35363,2.28,'Medium' from dual union all
select 13971,'013971','1983-026D','IUS-1 SRM-1','Orbus 21',55783,'In Earth orbit','','1983-05-04 00:00:00',624.02,251,35372,25.91,'GEO Transfer' from dual union all
select 13972,'013972','1983-027A','Kosmos-1450','Vektor',55784,'Reentered','1990-05-30 00:00:00','1983-05-07 00:00:00',94.42,468,513,65.85,'Intermediate' from dual union all
select 13973,'013973','1983-027B','S3M','S3M',55784,'Reentered','1989-03-23 00:00:00','1983-05-06 00:00:00',94.26,457,509,65.85,'Intermediate' from dual union all
select 13974,'013974','1983-028A','Raduga','Gran'' No. 23L',55785,'In Earth orbit','','1983-05-10 00:00:00',1435.98,35777,35792,1.19,'Stationary' from dual union all
select 13975,'013975','1983-029A','Kosmos-1451','Zenit-6',55786,'Landed','1983-04-22 00:00:00','1983-04-10 00:00:00',89.99,226,322,82.35,'Intermediate' from dual union all
select 13976,'013976','1983-028B','Perekhodnik','Sredniy perekhodnik',55785,'Reentered','1983-04-08 00:00:00','1983-04-08 00:00:00',87.77,159,170,51.61,'Intermediate' from dual union all
select 13977,'013977','1983-028C','8S812','8S812',55785,'Reentered','1983-04-10 00:00:00','1983-04-09 00:00:00',88.13,179,185,51.63,'Intermediate' from dual union all
select 13978,'013978','1983-029B','11S510','11S510',55786,'Reentered','1983-04-09 00:00:00','1983-04-08 00:00:00',88.43,173,221,82.34,'Intermediate' from dual union all
select 13979,'013979','1983-028D','SOZ','SOZ',55785,'Reentered','1984-05-15 00:00:00','1983-05-14 00:00:00',635.1,247,35947,47.29,'GEO Transfer' from dual union all
select 1398,'001398','1965-020AK','deb Blok Kosmos','',11770,'Reentered','1970-06-25 00:00:00','1965-06-02 00:00:00',108.96,381,1977,56.39,'Intermediate' from dual union all
select 13980,'013980','1983-028E','SOZ','SOZ',55785,'Reentered','1984-06-15 00:00:00','1983-05-10 00:00:00',636.64,303,35970,47.39,'GEO Transfer' from dual union all
select 13981,'013981','1982-076C','Kosmos-1397 SS 1','ESO',54385,'Reentered','1983-04-22 00:00:00','1983-04-11 00:00:00',90.75,274,349,50.67,'Intermediate' from dual union all
select 13982,'013982','1982-076D','Kosmos-1397 SS 2','ESO',54385,'Reentered','1983-04-21 00:00:00','1983-04-11 00:00:00',90.67,277,337,50.69,'Intermediate' from dual union all
select 13983,'013983','1983-028F','Blok DM','11S86 No. 38L',55785,'In Earth orbit','','1991-05-13 00:00:00',1439.46,35736,35969,6.2,'Inclined Drift' from dual union all
select 13984,'013984','1983-030A','Satcom 1R','RCA-F',55787,'In Earth orbit','','1983-05-13 00:00:00',1436.06,35776,35796,.09,'Stationary' from dual union all
select 13985,'013985','1983-030B','Delta 167','Delta SSPS AJ10-118K',55787,'Reentered','2001-10-18 00:00:00','1990-02-19 00:00:00',121.24,297,3176,25.35,'Medium' from dual union all
select 13986,'013986','1982-076E','Kosmos-1397 SS 3','ESO',54385,'Reentered','1983-04-13 00:00:00','1983-04-11 00:00:00',90.78,278,348,50.67,'Intermediate' from dual union all
select 13987,'013987','1982-076F','Kosmos-1397 SS 4','ESO',54385,'Reentered','1983-04-25 00:00:00','1983-04-12 00:00:00',90.65,275,337,50.66,'Intermediate' from dual union all
select 13988,'013988','1982-076G','Kosmos-1397 SS 5','ESO',54385,'Reentered','1983-04-18 00:00:00','1983-04-12 00:00:00',90.63,276,335,50.69,'Intermediate' from dual union all
select 13989,'013989','1982-076H','Kosmos-1397 SS 6','ESO',54385,'Reentered','1983-04-13 00:00:00','1983-04-11 00:00:00',90.61,277,332,50.76,'Intermediate' from dual union all
select 1399,'001399','1965-027E','TRIAD debris                     ','[UNID010] RCS 0.0',11777,'In Earth orbit','','1972-11-01 00:00:00',101.06,749,869,90.5,'Polar' from dual union all
select 13990,'013990','1983-030C','Star 37E S/N 40095','Star 37E',55787,'Reentered','1986-10-27 00:00:00','1983-05-12 00:00:00',629.27,199,35695,24.06,'GEO Transfer' from dual union all
select 13991,'013991','1983-031A','Kosmos-1452','Strela-2M',55788,'In Earth orbit','','1983-05-22 00:00:00',100.78,783,808,74.06,'Intermediate' from dual union all
select 13992,'013992','1983-031B','S3M','S3M',55788,'In Earth orbit','','1990-03-26 00:00:00',100.6,783,791,74.05,'Intermediate' from dual union all
select 13993,'013993','1983-024E','Blenda','Sunshade',55781,'Reentered','1983-04-23 00:00:00','1983-04-14 00:00:00',92.42,356,431,73.53,'Intermediate' from dual union all
select 13994,'013994','1983-024F','KDU part','KDU part',55781,'Reentered','1983-05-18 00:00:00','1983-04-14 00:00:00',92.24,354,414,72.87,'Intermediate' from dual union all
select 13995,'013995','1983-024G','KDU','KDU',55781,'Reentered','1984-03-25 00:00:00','1983-05-01 00:00:00',92.16,353,408,72.87,'Intermediate' from dual union all
select 13996,'013996','1983-024H','KDU part','KDU part',55781,'Reentered','1983-06-24 00:00:00','1983-05-02 00:00:00',91.72,330,387,72.87,'Intermediate' from dual union all
select 13997,'013997','1983-024J','KDU part','KDU part',55781,'Reentered','1983-04-22 00:00:00','1983-04-17 00:00:00',91.69,332,383,72.89,'Intermediate' from dual union all
select 13998,'013998','1978-016D','deb Centaur AC-44','',45211,'Reentered','1989-06-21 00:00:00','1982-09-02 00:00:00',263.98,309,14196,26.44,'Highly Ellip' from dual union all
select 13999,'013999','1980-057G','deb Kosmos-1191','',50267,'In Earth orbit','','1981-09-13 00:00:00',715.19,1196,39031,62.79,'Molniya' from dual union all
select 14,'000014','59 Gamma','Discoverer 2','CORONA BIO 1',5530,'Reentered Attached','1959-04-26 00:00:00','1959-04-13 00:00:00',90.36,239,346,89.9,'Polar' from dual union all
select 140,'000140','61 Omicron 25','deb r/b Ablestar','',7273,'Reentered','1962-09-30 00:00:00','1961-09-18 00:00:00',97.33,315,947,66.6,'Intermediate' from dual union all
select 1400,'001400','1965-020AL','deb Blok Kosmos','',11770,'Reentered','1965-10-05 00:00:00','1965-06-08 00:00:00',102.03,273,1437,56.12,'Intermediate' from dual union all
select 14000,'014000','1977-068G','deb Kosmos-931','',42196,'In Earth orbit','','1983-07-16 00:00:00',718.25,3500,36878,65.07,'GEO Transfer' from dual union all
select 14001,'014001','1983-032A','OPS 2925','GAMBIT-3 Mission 4353',55789,'Reentered Attached','1983-08-21 00:00:00','1983-05-15 00:00:00',88.4,127,264,96.51,'Sun-Synch' from dual union all
select 14002,'014002','1983-033A','Rohini RS-D-2','Rohini RS-D-2',55790,'Reentered','1990-04-19 00:00:00','1983-05-17 00:00:00',97.06,387,849,46.61,'Intermediate' from dual union all
select 14003,'014003','1983-033B','SLV-3-D4 AS-4','AS-4',55790,'Reentered','1989-03-18 00:00:00','1983-05-18 00:00:00',97.09,389,850,46.63,'Intermediate' from dual union all
select 14004,'014004','1983-033C','deb Rohini RS-D-2','',55790,'Reentered','1987-05-11 00:00:00','1983-05-17 00:00:00',96.84,384,831,46.6,'Intermediate' from dual union all
select 14005,'014005','1979-062D','Blok DM No. 21L','11S86 No. 21L',47546,'In Earth orbit','','1993-05-29 00:00:00',1474.32,36511,36552,11.39,'Inclined Drift' from dual union all
select 14006,'014006','1983-034A','Kosmos-1453','Romb',55791,'Reentered','1989-05-08 00:00:00','1983-05-19 00:00:00',94.43,468,514,74.03,'Intermediate' from dual union all
select 14007,'014007','1983-034B','S3M','S3M',55791,'Reentered','1989-03-25 00:00:00','1983-05-20 00:00:00',94.34,460,514,74.04,'Intermediate' from dual union all
select 14008,'014008','1982-076J','Kosmos-1397 SS 7','ESO',54385,'Reentered','1983-04-24 00:00:00','1983-04-19 00:00:00',90.64,279,333,50.69,'Intermediate' from dual union all
select 14009,'014009','1982-076K','Kosmos-1397 SS 8','ESO',54385,'Reentered','1983-04-23 00:00:00','1983-04-19 00:00:00',90.53,272,329,50.68,'Intermediate' from dual union all
select 1401,'001401','1965-020AM','deb Blok Kosmos','',11770,'Reentered','1966-03-20 00:00:00','1965-06-09 00:00:00',103.85,238,1643,56.24,'Intermediate' from dual union all
select 14010,'014010','1982-076L','Kosmos-1397 SS 9','ESO',54385,'Reentered','1983-04-24 00:00:00','1983-04-19 00:00:00',90.43,271,321,50.68,'Intermediate' from dual union all
select 14011,'014011','1982-076M','Kosmos-1397 SS 10','ESO',54385,'Reentered','1983-04-25 00:00:00','1983-04-19 00:00:00',90.41,265,325,50.67,'Intermediate' from dual union all
select 14012,'014012','1982-076N','Kosmos-1397 SS 11','ESO',54385,'Reentered','1983-04-24 00:00:00','1983-04-19 00:00:00',90.35,262,322,50.66,'Intermediate' from dual union all
select 14013,'014013','1982-076P','Kosmos-1397 SS 12','ESO',54385,'Reentered','1983-04-24 00:00:00','1983-04-19 00:00:00',90.54,274,328,50.68,'Intermediate' from dual union all
select 14014,'014014','1983-035A','Soyuz T-8','Soyuz 7K-ST No. 13L',55792,'Landed','1983-04-22 00:00:00','1983-04-21 00:00:00',89.52,226,276,51.61,'Intermediate' from dual union all
select 14015,'014015','1983-035B','11S510','11S510',55792,'Reentered','1983-04-22 00:00:00','1983-04-21 00:00:00',88.21,184,188,51.63,'Intermediate' from dual union all
select 14016,'014016','1983-029C','KDU','KDU',55786,'Reentered','1983-05-20 00:00:00','1983-04-22 00:00:00',89.97,234,311,82.35,'Intermediate' from dual union all
select 14017,'014017','1983-036A','Kosmos-1454','Feniks No. 957',55793,'Landed','1983-05-22 00:00:00','1983-04-29 00:00:00',88.93,162,282,67.14,'Intermediate' from dual union all
select 14018,'014018','1982-076Q','Kosmos-1397 SS 13','ESO',54385,'Reentered','1983-04-24 00:00:00','1983-04-22 00:00:00',89.86,253,282,50.66,'Intermediate' from dual union all
select 14019,'014019','1982-076R','Kosmos-1397 SS 14','ESO',54385,'Reentered','1983-04-24 00:00:00','1983-04-22 00:00:00',89.88,257,280,50.66,'Intermediate' from dual union all
select 1402,'001402','1965-020AN','deb Blok Kosmos','',11770,'Reentered','1967-05-05 00:00:00','1965-06-04 00:00:00',105.18,291,1715,56.09,'Intermediate' from dual union all
select 14020,'014020','1982-076S','Kosmos-1397 SS 15','ESO',54385,'Reentered','1983-04-26 00:00:00','1983-04-23 00:00:00',89.16,228,238,50.65,'Intermediate' from dual union all
select 14021,'014021','1982-076T','Kosmos-1397 SS 16','ESO',54385,'Reentered','1983-04-24 00:00:00','1983-04-22 00:00:00',89.72,244,277,50.65,'Intermediate' from dual union all
select 14022,'014022','1982-076U','Kosmos-1397 SS 17','ESO',54385,'Reentered','1983-04-25 00:00:00','1983-04-23 00:00:00',89.53,246,257,50.68,'Intermediate' from dual union all
select 14023,'014023','1982-076V','Kosmos-1397 SS 18','ESO',54385,'Reentered','1983-04-27 00:00:00','1983-04-22 00:00:00',89.64,230,284,50.67,'Intermediate' from dual union all
select 14024,'014024','1982-076W','Kosmos-1397 SS 19','ESO',54385,'Reentered','1983-04-27 00:00:00','1983-04-22 00:00:00',89.98,237,310,50.68,'Intermediate' from dual union all
select 14025,'014025','1982-076X','Kosmos-1397 SS 20','ESO',54385,'Reentered','1983-04-24 00:00:00','1983-04-22 00:00:00',89.28,224,254,50.66,'Intermediate' from dual union all
select 14026,'014026','1983-029D','Blenda','Sunshade',55786,'Reentered','1983-04-25 00:00:00','1983-04-22 00:00:00',89.77,228,299,82.35,'Intermediate' from dual union all
select 14027,'014027','1983-029E','KDU part','KDU part',55786,'Reentered','1983-04-27 00:00:00','1983-04-22 00:00:00',89.73,231,291,82.35,'Intermediate' from dual union all
select 14028,'014028','1983-036B','11S510','11S510',55793,'Reentered','1983-04-26 00:00:00','1983-04-24 00:00:00',89.06,163,293,67.14,'Intermediate' from dual union all
select 14029,'014029','1983-035C','Soyuz T-8 BO','Bitovoy otsek',55792,'Reentered','1983-05-27 00:00:00','1983-04-27 00:00:00',90.25,282,292,51.62,'Intermediate' from dual union all
select 1403,'001403','1965-020AP','deb Blok Kosmos','',11770,'Reentered','1965-12-03 00:00:00','1965-06-05 00:00:00',102.37,270,1472,56.05,'Intermediate' from dual union all
select 14030,'014030','1983-036C','Kosmos-1454 part','Feniks part',55793,'Reentered','1983-04-23 00:00:00','1983-04-22 00:00:00',89.01,161,291,67.11,'Intermediate' from dual union all
select 14031,'014031','1982-076Y','Kosmos-1397 SS 21','ESO',54385,'Reentered','1983-04-24 00:00:00','1983-04-23 00:00:00',88.45,184,212,50.62,'Intermediate' from dual union all
select 14032,'014032','1983-037A','Kosmos-1455','Tselina-D',55794,'In Earth orbit','','1987-10-29 00:00:00',97.65,632,661,82.51,'Intermediate' from dual union all
select 14033,'014033','1983-037B','S5M','S5M',55794,'In Earth orbit','','1987-10-26 00:00:00',97.65,632,661,82.51,'Intermediate' from dual union all
select 14034,'014034','1983-038A','Kosmos-1456','Oko',55795,'Reentered','1998-05-11 00:00:00','1983-05-27 00:00:00',717.92,652,39710,63.03,'Molniya' from dual union all
select 14035,'014035','1982-076Z','Kosmos-1397 SS 22','ESO',54385,'Reentered','1983-04-25 00:00:00','1983-04-22 00:00:00',88.8,211,220,50.8,'Intermediate' from dual union all
select 14036,'014036','1983-038B','11S510','11S510',55795,'Reentered','1983-05-31 00:00:00','1983-05-02 00:00:00',92.09,212,542,62.87,'Intermediate' from dual union all
select 14037,'014037','1983-038C','BOZ','BOZ',55795,'Reentered','1983-05-07 00:00:00','1983-04-27 00:00:00',92.18,168,595,62.97,'Intermediate' from dual union all
select 14038,'014038','1983-038D','deb Blok-I','',55795,'Reentered','1983-05-17 00:00:00','1983-05-02 00:00:00',92.12,206,551,62.88,'Intermediate' from dual union all
select 14039,'014039','1983-039A','Kosmos-1457','Oktan No. 214',55796,'Landed','1983-06-08 00:00:00','1983-05-03 00:00:00',89.74,169,355,70.39,'Intermediate' from dual union all
select 1404,'001404','1965-046A','Kosmos-68','Zenit-2 No. 29',11796,'Landed','1965-06-23 00:00:00','1965-06-17 00:00:00',89.69,200,318,65.02,'Intermediate' from dual union all
select 14040,'014040','1983-039B','11S510','11S510',55796,'Reentered','1983-04-30 00:00:00','1983-04-28 00:00:00',88.91,160,282,70.4,'Intermediate' from dual union all
select 14041,'014041','1983-038E','Blok-2BL','Blok-2BL No. 71629-652',55795,'Reentered','1998-09-08 00:00:00','1983-05-28 00:00:00',707.29,650,39186,63.06,'Molniya' from dual union all
select 14042,'014042','1979-072D','Star 37E yo weight','',47556,'Reentered','1997-07-26 00:00:00','1979-08-10 00:00:00',649.13,231,36681,24,'GEO Transfer' from dual union all
select 14043,'014043','1983-038F','deb Blok-I','',55795,'Reentered','1983-04-29 00:00:00','1983-04-25 00:00:00',92.74,212,606,62.8,'Intermediate' from dual union all
select 14044,'014044','1983-040A','Kosmos-1458','Fram',55797,'Landed','1983-05-11 00:00:00','1983-04-30 00:00:00',89.02,209,244,82.32,'Intermediate' from dual union all
select 14045,'014045','1983-040B','11S510','11S510',55797,'Reentered','1983-05-02 00:00:00','1983-04-30 00:00:00',88.34,186,199,82.32,'Intermediate' from dual union all
select 14046,'014046','1983-040C','Sensor cover','Sensor cover',55797,'Reentered','1983-05-19 00:00:00','1983-05-08 00:00:00',88.31,170,212,82.35,'Intermediate' from dual union all
select 14047,'014047','1965-027J','deb SNAPSHOT','',11777,'In Earth orbit','','1983-04-28 00:00:00',111.43,1270,1315,90.26,'Polar' from dual;
insert into SATELLITE
select 14048,'014048','1965-027K','deb SNAPSHOT','',11777,'In Earth orbit','','1983-05-02 00:00:00',111.39,1268,1314,90.25,'Polar' from dual union all
select 14049,'014049','1965-027L','deb SNAPSHOT','',11777,'In Earth orbit','','1993-08-07 00:00:00',111.31,1266,1308,90.27,'Polar' from dual union all
select 1405,'001405','1965-046B','8A92E','8A92E',11796,'Reentered','1965-07-07 00:00:00','1965-06-22 00:00:00',89.56,213,293,65.02,'Intermediate' from dual union all
select 14050,'014050','1983-041A','GOES 6','GOES F',55798,'In Earth orbit','','1983-05-30 00:00:00',1435.63,35724,35831,.28,'Stationary' from dual union all
select 14051,'014051','1983-041B','Delta 168','Delta SSPU DSV-3P-4B',55798,'In Earth orbit','','1990-02-19 00:00:00',116.97,403,2686,25.34,'Intermediate' from dual union all
select 14052,'014052','1982-007V','Kosmos-1335 SS 17','ESO',54316,'Reentered','1983-06-11 00:00:00','1983-04-30 00:00:00',93.38,426,454,74.05,'Intermediate' from dual union all
select 14053,'014053','1982-007W','Kosmos-1335 SS 18','ESO',54316,'Reentered','1983-06-10 00:00:00','1983-04-29 00:00:00',93.54,434,461,74.05,'Intermediate' from dual union all
select 14054,'014054','1982-007X','Kosmos-1335 SS 19','ESO',54316,'Reentered','1983-06-11 00:00:00','1983-04-29 00:00:00',93.53,435,460,74.05,'Intermediate' from dual union all
select 14055,'014055','1982-007Y','Kosmos-1335 SS 20','ESO',54316,'Reentered','1983-06-09 00:00:00','1983-05-01 00:00:00',93.41,430,453,74.1,'Intermediate' from dual union all
select 14056,'014056','1982-007Z','Kosmos-1335 SS 21','ESO',54316,'Reentered','1983-06-09 00:00:00','1983-05-06 00:00:00',93.24,420,447,74.04,'Intermediate' from dual union all
select 14057,'014057','1983-042A','Kosmos-1459','Parus',55799,'In Earth orbit','','1983-06-08 00:00:00',104.7,945,1016,82.97,'Intermediate' from dual union all
select 14058,'014058','1983-043A','Kosmos-1460','Zenit-6',55800,'Landed','1983-05-20 00:00:00','1983-05-08 00:00:00',92.19,349,415,70.34,'Intermediate' from dual union all
select 14059,'014059','1983-042B','S3M','S3M',55799,'In Earth orbit','','1991-03-26 00:00:00',104.55,941,1006,82.97,'Intermediate' from dual union all
select 1406,'001406','1965-046C','deb Kosmos-68','',11796,'Reentered','1965-06-20 00:00:00','1965-06-16 00:00:00',89.08,206,252,65,'Intermediate' from dual union all
select 14060,'014060','1983-043B','11S510','11S510',55800,'Reentered','1983-05-15 00:00:00','1983-05-08 00:00:00',89.71,201,320,70.33,'Intermediate' from dual union all
select 14061,'014061','1983-043C','Sensor cover','Sensor cover',55800,'Reentered','1983-05-21 00:00:00','1983-05-07 00:00:00',90.06,205,350,70.37,'Intermediate' from dual union all
select 14062,'014062','1983-043D','Sensor cover','Sensor cover',55800,'Reentered','1983-05-06 00:00:00','1983-05-06 00:00:00',89.46,202,294,70.35,'Intermediate' from dual union all
select 14063,'014063','1983-043E','Sensor cover','Sensor cover',55800,'Reentered','1983-05-08 00:00:00','1983-05-07 00:00:00',89.76,203,322,70.3,'Intermediate' from dual union all
select 14064,'014064','1983-044A','Kosmos-1461','US-P',55801,'In Earth orbit','','1983-06-06 00:00:00',93.28,428,442,65.03,'Intermediate' from dual union all
select 14065,'014065','1983-044B','11S692','11S692',55801,'Reentered','1983-05-07 00:00:00','1983-05-07 00:00:00',89.16,110,355,64.99,'Intermediate' from dual union all
select 14066,'014066','1983-040D','Blenda','Sunshade',55797,'Reentered','1983-05-12 00:00:00','1983-05-11 00:00:00',88.9,204,237,82.32,'Intermediate' from dual union all
select 14067,'014067','1983-040E','KDU','KDU',55797,'Reentered','1983-05-17 00:00:00','1983-05-11 00:00:00',88.87,204,234,82.28,'Intermediate' from dual union all
select 14068,'014068','1982-007AA','Kosmos-1335 SS 22','ESO',54316,'Reentered','1983-06-09 00:00:00','1983-05-13 00:00:00',92.98,410,431,74.11,'Intermediate' from dual union all
select 14069,'014069','1983-041C','Star 27 S/N 0035','Star 27 TE-M-616-5',55798,'In Earth orbit','','1993-06-27 00:00:00',1707.31,32719,49162,11.49,'Very High' from dual union all
select 1407,'001407','1965-046D','deb Kosmos-68','',11796,'Reentered','1965-06-19 00:00:00','1965-06-16 00:00:00',89.08,181,276,65.01,'Intermediate' from dual union all
select 14070,'014070','1979-062E','SOZ','SOZ',47546,'Reentered','1984-04-28 00:00:00','1982-11-09 00:00:00',285.65,168,15811,46.62,'Highly Ellip' from dual union all
select 14071,'014071','1983-045A','Kosmos-1462','Resurs-F1 17F41 No. 21',55802,'Landed','1983-05-31 00:00:00','1983-05-19 00:00:00',89.8,257,273,82.32,'Intermediate' from dual union all
select 14072,'014072','1983-045B','11S510','11S510',55802,'Reentered','1983-05-23 00:00:00','1983-05-19 00:00:00',89.06,198,257,82.31,'Intermediate' from dual union all
select 14073,'014073','1983-045C','Sensor cover','Sensor cover',55802,'Reentered','1983-05-21 00:00:00','1983-05-17 00:00:00',89.35,207,278,82.32,'Intermediate' from dual union all
select 14074,'014074','1983-045D','Sensor cover','Sensor cover',55802,'Reentered','1983-06-09 00:00:00','1983-05-31 00:00:00',91.52,256,443,82.33,'Intermediate' from dual union all
select 14075,'014075','1983-046A','Kosmos-1463','Taifun-1B',55803,'Reentered','1993-01-24 00:00:00','1983-06-19 00:00:00',103.42,300,1541,82.94,'Intermediate' from dual union all
select 14076,'014076','1983-046B','S3M','S3M',55803,'Reentered','1989-11-07 00:00:00','1988-06-13 00:00:00',98.52,290,1085,82.93,'Intermediate' from dual union all
select 14077,'014077','1983-047A','INTELSAT 506','INTELSAT V F6',55804,'In Earth orbit','','1983-06-29 00:00:00',1442.12,35859,35950,.16,'Drift GEO' from dual union all
select 14078,'014078','1983-043F','Blenda','Sunshade',55800,'Reentered','1983-07-11 00:00:00','1983-06-05 00:00:00',91.8,335,391,70.33,'Intermediate' from dual union all
select 14079,'014079','1983-043G','KDU','KDU',55800,'Reentered','1984-04-09 00:00:00','1983-06-05 00:00:00',92.11,346,410,70.33,'Intermediate' from dual union all
select 1408,'001408','1965-046E','deb Kosmos-68','',11796,'Reentered','1965-06-19 00:00:00','1965-06-16 00:00:00',90.92,242,398,65,'Intermediate' from dual union all
select 14080,'014080','1983-043H','KDU part','KDU part',55800,'Reentered','1983-08-08 00:00:00','1983-06-05 00:00:00',91.77,329,394,70.33,'Intermediate' from dual union all
select 14081,'014081','1983-047B','Centaur AC-61','Centaur D-1AR AC-61',55804,'Reentered','1988-02-13 00:00:00','1983-06-20 00:00:00',607.52,201,34565,23.53,'GEO Transfer' from dual union all
select 14082,'014082','1983-043J','KDU part','KDU part',55800,'Reentered','1983-05-26 00:00:00','1983-05-21 00:00:00',91.72,317,401,70.74,'Intermediate' from dual union all
select 14083,'014083','1983-043K','KDU part','KDU part',55800,'Reentered','1983-05-27 00:00:00','1983-05-22 00:00:00',91.87,350,382,70.34,'Intermediate' from dual union all
select 14084,'014084','1983-048A','Kosmos-1464','Parus',55805,'In Earth orbit','','1983-07-06 00:00:00',104.85,965,1010,82.94,'Intermediate' from dual union all
select 14085,'014085','1983-048B','S3M','S3M',55805,'In Earth orbit','','1991-03-25 00:00:00',104.71,961,1000,82.95,'Intermediate' from dual union all
select 14086,'014086','1983-016F','Blok DM','11S86 No. 34L',55773,'In Earth orbit','','1993-05-31 00:00:00',1424.3,35440,35671,9.08,'Inclined Drift' from dual union all
select 14087,'014087','1983-049A','Kosmos-1465','Romb',55806,'Reentered','1985-01-23 00:00:00','1983-06-26 00:00:00',93.4,347,534,50.66,'Intermediate' from dual union all
select 14088,'014088','1983-049B','S3M','S3M',55806,'Reentered','1985-01-11 00:00:00','1983-06-26 00:00:00',93.33,346,528,50.67,'Intermediate' from dual union all
select 14089,'014089','1983-050A','Kosmos-1466','Oktan No. 250',55807,'Landed','1983-07-06 00:00:00','1983-06-02 00:00:00',89.02,166,286,64.88,'Intermediate' from dual union all
select 1409,'001409','1965-020AQ','deb Blok Kosmos','',11770,'Reentered','1966-05-19 00:00:00','1966-01-31 00:00:00',99.15,305,1131,56.04,'Intermediate' from dual union all
select 14090,'014090','1983-050B','11S510','11S510',55807,'Reentered','1983-05-30 00:00:00','1983-05-28 00:00:00',89.15,166,299,64.89,'Intermediate' from dual union all
select 14091,'014091','1983-049C','Dispenser cover','Romb dispenser cover',55806,'Reentered','1983-07-15 00:00:00','1983-06-02 00:00:00',93,345,498,50.65,'Intermediate' from dual union all
select 14092,'014092','1983-049D','Dispenser cover','Romb dispenser cover',55806,'Reentered','1983-07-02 00:00:00','1983-06-02 00:00:00',92.94,341,495,50.69,'Intermediate' from dual union all
select 14093,'014093','1983-049E','Dispenser cover','Romb dispenser cover',55806,'Reentered','1983-07-05 00:00:00','1983-06-02 00:00:00',92.8,333,491,50.68,'Intermediate' from dual union all
select 14094,'014094','1983-049F','Dispenser cover','Romb dispenser cover',55806,'Reentered','1983-12-09 00:00:00','1983-06-26 00:00:00',91.36,294,389,50.62,'Intermediate' from dual union all
select 14095,'014095','1983-051A','Exosat','Exosat',55808,'Deep Space','1983-05-27 00:00:00','1983-07-02 00:00:00',5437.12,583,191510,72,'Deep Space' from dual union all
select 14096,'014096','1983-051B','Delta 169','Delta SSPU DSV-3P-4B',55808,'In Earth orbit','','1990-01-30 00:00:00',119.12,756,2527,72.31,'Intermediate' from dual union all
select 14097,'014097','1983-050C','Kosmos-1466 part','Oktan part',55807,'Reentered','1983-05-27 00:00:00','1983-05-26 00:00:00',88.81,156,276,64.87,'Intermediate' from dual union all
select 14098,'014098','1983-045E','Blenda','Sunshade',55802,'Reentered','1983-06-07 00:00:00','1983-05-31 00:00:00',89.64,245,269,82.31,'Intermediate' from dual union all
select 14099,'014099','1983-045F','KDU','KDU',55802,'Reentered','1983-06-20 00:00:00','1983-05-31 00:00:00',89.77,256,271,82.32,'Intermediate' from dual union all
select 141,'000141','61 Omicron 26','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-06 00:00:00',105.18,950,1056,66.59,'Intermediate' from dual union all
select 1410,'001410','1965-020AR','deb Blok Kosmos','',11770,'Reentered','1967-02-17 00:00:00','1965-06-16 00:00:00',105.14,288,1713,56.12,'Intermediate' from dual union all
select 14100,'014100','1983-052A','Kosmos-1467','Zenit-6',55809,'Landed','1983-06-12 00:00:00','1983-06-02 00:00:00',92.26,356,415,72.86,'Intermediate' from dual union all
select 14101,'014101','1983-052B','11S510','11S510',55809,'Reentered','1983-06-10 00:00:00','1983-06-02 00:00:00',89.86,195,340,72.85,'Intermediate' from dual union all
select 14102,'014102','1983-052C','Sensor cover','Sensor cover',55809,'Reentered','1983-06-02 00:00:00','1983-06-01 00:00:00',89.98,196,350,72.86,'Intermediate' from dual union all
select 14103,'014103','1983-052D','Sensor cover','Sensor cover',55809,'Reentered','1983-06-02 00:00:00','1983-06-01 00:00:00',89.3,186,294,72.85,'Intermediate' from dual union all
select 14104,'014104','1983-053A','Venera-15','4V-2 No. 860',55810,'Deep Space','1983-06-02 00:00:00','',null,null,null,null,'' from dual union all
select 14105,'014105','1983-053B','SOZ','SOZ',55810,'Reentered','1983-06-03 00:00:00','1983-06-02 00:00:00',88.03,156,198,51.55,'Intermediate' from dual union all
select 14106,'014106','1983-053C','SOZ','SOZ',55810,'Reentered','1983-06-03 00:00:00','1983-06-02 00:00:00',88.02,158,195,51.52,'Intermediate' from dual union all
select 14107,'014107','1983-054A','Venera-16','4V-2 No. 861',55811,'Deep Space','1983-06-07 00:00:00','',null,null,null,null,'' from dual union all
select 14108,'014108','1983-054B','SOZ','SOZ',55811,'Reentered','1983-06-08 00:00:00','1983-06-07 00:00:00',88.06,158,199,51.51,'Intermediate' from dual union all
select 14109,'014109','1983-054C','SOZ','SOZ',55811,'Reentered','1983-06-08 00:00:00','1983-06-07 00:00:00',88.01,156,196,51.55,'Intermediate' from dual union all
select 1411,'001411','1964-076L','deb Explorer XXIV','',10362,'Reentered','1968-03-05 00:00:00','1965-06-10 00:00:00',116.35,541,2493,81.36,'Intermediate' from dual union all
select 14110,'014110','1983-055A','Kosmos-1468','Resurs-F1 17F41 No. 22',55812,'Landed','1983-06-21 00:00:00','1983-06-09 00:00:00',89.8,253,277,82.34,'Intermediate' from dual union all
select 14111,'014111','1983-055B','11S510','11S510',55812,'Reentered','1983-06-12 00:00:00','1983-06-09 00:00:00',88.82,207,226,82.35,'Intermediate' from dual union all
select 14112,'014112','1983-056A','OPS 6432','PARCAE 6',55813,'In Earth orbit','','1996-10-17 00:00:00',107.41,851,1363,63.43,'Intermediate' from dual union all
select 14113,'014113','1983-056B','MSD plume shield','',55813,'In Earth orbit','','1997-11-21 00:00:00',106.86,821,1341,63.34,'Intermediate' from dual union all
select 14114,'014114','1982-044F','Blok DM','11S86 No. 28L',54353,'In Earth orbit','','1993-06-07 00:00:00',1437.21,35740,35877,9.17,'Inclined Drift' from dual union all
select 14115,'014115','1982-093F','Blok DM','11S86 No. 31L',54402,'In Earth orbit','','1993-06-08 00:00:00',1422.09,35472,35552,9.05,'Inclined Drift' from dual union all
select 14116,'014116','1983-052E','KDU','KDU',55809,'Reentered','1984-06-06 00:00:00','1983-06-30 00:00:00',92.18,354,409,72.85,'Intermediate' from dual union all
select 14117,'014117','1982-009F','Blok DM No. 26L','11S86 No. 26L',54318,'In Earth orbit','','1993-05-28 00:00:00',1425.91,35447,35727,9.51,'Inclined Drift' from dual union all
select 14118,'014118','1983-052F','Blenda','Sunshade',55809,'Reentered','1983-09-13 00:00:00','1983-07-01 00:00:00',91.85,336,394,72.85,'Intermediate' from dual union all
select 14119,'014119','1983-052G','KDU part','KDU part',55809,'Reentered','1983-06-19 00:00:00','1983-06-12 00:00:00',91.99,335,410,73.09,'Intermediate' from dual union all
select 1412,'001412','1965-047A','Titan 3A Fairing','Lead Ballast',11797,'Reentered','1965-06-29 00:00:00','1965-06-20 00:00:00',88.06,168,188,32.14,'Intermediate' from dual union all
select 14120,'014120','1983-052H','KDU part','KDU part',55809,'Reentered','1983-06-21 00:00:00','1983-06-12 00:00:00',92.46,376,415,72.86,'Intermediate' from dual union all
select 14121,'014121','1983-052J','KDU part','KDU part',55809,'Reentered','1983-08-07 00:00:00','1983-06-30 00:00:00',91.74,334,386,72.85,'Intermediate' from dual union all
select 14122,'014122','1970-089CW','deb Kosmos-374','',21566,'In Earth orbit','','1993-08-07 00:00:00',108.43,512,1797,62.91,'Intermediate' from dual union all
select 14123,'014123','1983-057A','Kosmos-1469','Zenit-6',55814,'Landed','1983-06-24 00:00:00','1983-06-16 00:00:00',90.21,232,338,72.85,'Intermediate' from dual union all
select 14124,'014124','1983-057B','11S510','11S510',55814,'Reentered','1983-06-23 00:00:00','1983-06-16 00:00:00',89.87,193,343,72.84,'Intermediate' from dual union all
select 14125,'014125','1981-057E','Meteosat adapter','MTO',52580,'Reentered','1991-01-17 00:00:00','1981-09-18 00:00:00',598.56,193,34105,10.16,'Highly Ellip' from dual union all
select 14126,'014126','1983-057C','Sensor cover','Sensor cover',55814,'Reentered','1983-10-04 00:00:00','1983-07-23 00:00:00',89.66,186,330,72.84,'Intermediate' from dual union all
select 14127,'014127','1983-057D','Sensor cover','Sensor cover',55814,'Reentered','1983-06-15 00:00:00','1983-06-15 00:00:00',89.25,182,293,72.78,'Intermediate' from dual union all
select 14128,'014128','1983-058A','ECS 1','ECS 1',55815,'In Earth orbit','','1983-07-27 00:00:00',1436.19,35780,35797,.01,'Stationary' from dual union all
select 14129,'014129','1983-058B','AMSAT-OSCAR-10','AMSAT Phase III-B',55815,'In Earth orbit','','1983-07-16 00:00:00',699.59,3956,35498,26.16,'GEO Transfer' from dual union all
select 1413,'001413','1965-047B','Transtage 7','Transtage 7',11797,'Reentered','1965-06-21 00:00:00','1965-06-20 00:00:00',88.06,168,188,32.14,'Intermediate' from dual union all
select 14130,'014130','1983-058C','Ariane H8 L6','Ariane H8 S/N T6',55815,'Reentered','1995-08-19 00:00:00','1983-07-21 00:00:00',617.38,209,35070,8.45,'GEO Transfer' from dual union all
select 14131,'014131','1981-088M','deb Kosmos-1305','',52611,'In Earth orbit','','1983-05-10 00:00:00',252.12,585,13098,62.87,'Medium' from dual union all
select 14132,'014132','1983-059A','Challenger (STS-7)','OV-099',55816,'Landed','1983-06-24 00:00:00','1983-06-20 00:00:00',90.69,296,321,28.46,'Intermediate' from dual union all
select 14133,'014133','1983-059B','Anik C2','Anik C2',55816,'In Earth orbit','','1983-07-19 00:00:00',1436.09,35786,35787,.13,'Stationary' from dual union all
select 14134,'014134','1983-059C','Palapa B1','Palapa B1',55816,'In Earth orbit','','1983-07-19 00:00:00',1436.04,35784,35786,.09,'Stationary' from dual union all
select 14135,'014135','1983-059D','PAM-D','Star 48  S/N 10026-3',55816,'In Earth orbit','','1983-07-24 00:00:00',639.88,295,36144,22.97,'GEO Transfer' from dual union all
select 14136,'014136','1983-059E','PAM-D','Star 48  S/N 10024-3',55816,'In Earth orbit','','1983-07-28 00:00:00',661.54,298,37244,25.4,'GEO Transfer' from dual union all
select 14137,'014137','1983-060A','OPS 0721','HEXAGON SV-18',55817,'Deorbited','1984-03-21 00:00:00','1983-06-20 00:00:00',88.47,169,229,96.5,'Sun-Synch' from dual;
insert into SATELLITE
select 14138,'014138','1983-060B','Titan 34D-5 Stage 2','LR91AJ11 34D-5',55817,'Reentered','1983-06-22 00:00:00','1983-06-21 00:00:00',88.47,157,240,96.45,'Sun-Synch' from dual union all
select 14139,'014139','1983-060C','OPS 3899','P801-7',55817,'In Earth orbit','','1997-11-21 00:00:00',111.31,1278,1295,96.66,'Sun-Synch' from dual union all
select 1414,'001414','1965-045B','Fairing','Fairing',11795,'Reentered','1965-06-20 00:00:00','1965-06-17 00:00:00',88.17,108,260,75.13,'Intermediate' from dual union all
select 14140,'014140','1983-055C','KDU','KDU',55812,'Reentered','1983-07-15 00:00:00','1983-06-21 00:00:00',89.77,253,274,82.33,'Intermediate' from dual union all
select 14141,'014141','1983-055D','Blenda','Sunshade',55812,'Reentered','1983-06-29 00:00:00','1983-06-21 00:00:00',89.63,240,273,82.3,'Intermediate' from dual union all
select 14142,'014142','1983-059F','SPAS 1','SPAS 1',55816,'Landed Attached','1983-06-24 00:00:00','1983-06-22 00:00:00',90.38,291,296,28.46,'Intermediate' from dual union all
select 14143,'014143','1983-056C','GB1','SSU',55813,'In Earth orbit','','1989-10-16 00:00:00',107.45,1069,1149,63.45,'Intermediate' from dual union all
select 14144,'014144','1983-056D','GB2','SSU',55813,'In Earth orbit','','1990-05-06 00:00:00',107.44,1007,1210,63.44,'Intermediate' from dual union all
select 14145,'014145','1983-056E','GB1 cover','SSU cover',55813,'In Earth orbit','','1997-11-22 00:00:00',107.02,757,1421,63.42,'Intermediate' from dual union all
select 14146,'014146','1983-056F','GB2 cover','SSU cover',55813,'In Earth orbit','','1997-11-22 00:00:00',107.02,757,1421,63.42,'Intermediate' from dual union all
select 14147,'014147','1983-061A','Kosmos-1470','Tselina-D',55818,'In Earth orbit','','1983-07-22 00:00:00',97.72,632,668,82.52,'Intermediate' from dual union all
select 14148,'014148','1983-061B','S5M','S5M',55818,'In Earth orbit','','1990-03-29 00:00:00',97.57,626,659,82.52,'Intermediate' from dual union all
select 14149,'014149','1983-057E','Blenda','Sunshade',55814,'Reentered','1983-06-27 00:00:00','1983-06-24 00:00:00',89.22,224,247,72.84,'Intermediate' from dual union all
select 1415,'001415','1965-037B','Fairing','Fairing',11787,'Reentered','1965-06-02 00:00:00','1965-05-28 00:00:00',89.67,192,325,75.02,'Intermediate' from dual union all
select 14150,'014150','1983-057F','KDU','KDU',55814,'Reentered','1983-07-04 00:00:00','1983-06-24 00:00:00',89.37,226,261,72.85,'Intermediate' from dual union all
select 14151,'014151','1983-058D','Sylda 3900 No. 2','Sylda 3900',55815,'Reentered','1986-07-03 00:00:00','1983-08-26 00:00:00',560.3,212,32059,8.51,'Highly Ellip' from dual union all
select 14152,'014152','1983-062A','Soyuz T-9','Soyuz 7K-ST No. 14L',55819,'Landed','1983-11-23 00:00:00','1983-07-27 00:00:00',91.03,318,333,51.61,'Intermediate' from dual union all
select 14153,'014153','1983-062B','11S510','11S510',55819,'Reentered','1983-06-29 00:00:00','1983-06-28 00:00:00',88.51,190,211,51.6,'Intermediate' from dual union all
select 14154,'014154','1983-063A','P83-1 Hilat','NNS O-16',55820,'In Earth orbit','','1983-07-27 00:00:00',100.94,770,837,82.03,'Intermediate' from dual union all
select 14155,'014155','1983-063B','Star 20 E-31','Star 20 E-31',55820,'In Earth orbit','','1983-07-28 00:00:00',100.93,769,837,82.03,'Intermediate' from dual union all
select 14156,'014156','1983-064A','Kosmos-1471','Feniks No. 958',55821,'Landed','1983-07-28 00:00:00','1983-07-05 00:00:00',88.95,173,273,67.14,'Intermediate' from dual union all
select 14157,'014157','1983-064B','11S510','11S510',55821,'Reentered','1983-07-03 00:00:00','1983-07-01 00:00:00',88.97,162,286,67.15,'Intermediate' from dual union all
select 14158,'014158','1983-065A','Galaxy 1','Galaxy I',55822,'In Earth orbit','','1983-08-02 00:00:00',1436.01,35784,35785,.14,'Stationary' from dual union all
select 14159,'014159','1983-065B','Delta 170','Delta SSPS AJ10-118K',55822,'Reentered','1983-08-11 00:00:00','1983-07-06 00:00:00',90.82,274,356,26.51,'Intermediate' from dual union all
select 1416,'001416','1965-020AS','deb Blok Kosmos','',11770,'Reentered','1967-05-05 00:00:00','1966-01-30 00:00:00',104.12,292,1615,56.21,'Intermediate' from dual union all
select 14160,'014160','1983-066A','Gorizont','Gorizont No. 17L',55823,'In Earth orbit','','1983-08-08 00:00:00',1436.01,35742,35828,1.31,'Stationary' from dual union all
select 14161,'014161','1983-066B','8S812','8S812',55823,'Reentered','1983-07-03 00:00:00','1983-07-02 00:00:00',87.93,168,176,51.62,'Intermediate' from dual union all
select 14162,'014162','1983-066C','Perekhodnik','Sredniy perekhodnik',55823,'Reentered','1983-07-01 00:00:00','1983-07-01 00:00:00',88.48,194,205,51.6,'Intermediate' from dual union all
select 14163,'014163','1983-067A','Prognoz-9','SO-M No. 509',55824,'Deep Space','1983-07-02 00:00:00','1983-07-01 00:00:00',36812.18,380,719999,65.5,'Cislunar' from dual union all
select 14164,'014164','1983-067B','11S510','11S510',55824,'Reentered','1983-08-05 00:00:00','1983-07-08 00:00:00',91.19,225,441,64.93,'Intermediate' from dual union all
select 14165,'014165','1983-067C','BOZ','BOZ',55824,'Reentered','1983-07-25 00:00:00','1983-07-08 00:00:00',90.91,200,439,65.01,'Intermediate' from dual union all
select 14166,'014166','1983-066D','SOZ','SOZ',55823,'Reentered','1984-02-22 00:00:00','1983-08-09 00:00:00',648.13,276,36584,47.13,'GEO Transfer' from dual union all
select 14167,'014167','1983-066E','SOZ','SOZ',55823,'Reentered','1995-11-25 00:00:00','1992-09-14 00:00:00',158.73,207,6459,46.52,'Medium' from dual union all
select 14168,'014168','1983-065C','PAM-D','Star 48  S/N 10019-8',55822,'Reentered','1999-07-17 00:00:00','1983-08-05 00:00:00',636.78,204,36076,23.45,'GEO Transfer' from dual union all
select 14169,'014169','1983-068A','Kosmos-1472','Zenit-6',55825,'Landed','1983-07-19 00:00:00','1983-07-07 00:00:00',91.49,336,360,82.35,'Intermediate' from dual union all
select 1417,'001417','1965-020AT','deb Blok Kosmos','',11770,'Reentered','1965-09-09 00:00:00','1965-03-15 00:00:00',100.08,277,1248,56.1,'Intermediate' from dual union all
select 14170,'014170','1983-068B','11S510','11S510',55825,'Reentered','1983-07-07 00:00:00','1983-07-06 00:00:00',88.39,175,214,82.35,'Intermediate' from dual union all
select 14171,'014171','1983-069A','Kosmos-1473','Strela-1M',55826,'In Earth orbit','','1983-08-13 00:00:00',114.41,1395,1463,74.02,'Intermediate' from dual union all
select 14172,'014172','1983-069B','Kosmos-1474','Strela-1M',55826,'In Earth orbit','','1992-05-21 00:00:00',114.59,1411,1463,74.02,'Intermediate' from dual union all
select 14173,'014173','1983-069C','Kosmos-1475','Strela-1M',55826,'In Earth orbit','','1992-05-20 00:00:00',114.78,1428,1463,74.02,'Intermediate' from dual union all
select 14174,'014174','1983-069D','Kosmos-1476','Strela-1M',55826,'In Earth orbit','','1992-05-20 00:00:00',114.96,1445,1463,74.02,'Intermediate' from dual union all
select 14175,'014175','1983-069E','Kosmos-1477','Strela-1M',55826,'In Earth orbit','','1992-05-21 00:00:00',115.16,1461,1465,74.02,'Intermediate' from dual union all
select 14176,'014176','1983-069F','Kosmos-1478','Strela-1M',55826,'In Earth orbit','','1992-05-20 00:00:00',115.36,1463,1481,74.02,'Intermediate' from dual union all
select 14177,'014177','1983-069G','Kosmos-1479','Strela-1M',55826,'In Earth orbit','','1992-05-11 00:00:00',115.56,1462,1500,74.03,'Intermediate' from dual union all
select 14178,'014178','1983-069H','Kosmos-1480','Strela-1M',55826,'In Earth orbit','','1992-05-21 00:00:00',115.79,1463,1520,74.03,'Intermediate' from dual union all
select 14179,'014179','1983-069J','S3M','S3M',55826,'In Earth orbit','','1991-02-28 00:00:00',117.47,1461,1674,74.02,'Intermediate' from dual union all
select 1418,'001418','1965-020AU','deb Blok Kosmos','',11770,'Reentered','1965-08-10 00:00:00','1965-06-16 00:00:00',97.49,177,1101,56.02,'Intermediate' from dual union all
select 14180,'014180','1983-056G','GB3','SSU',55813,'In Earth orbit','','1990-05-07 00:00:00',107.44,1085,1132,63.44,'Intermediate' from dual union all
select 14181,'014181','1983-056H','GB3 cover','SSU cover',55813,'In Earth orbit','','1997-11-22 00:00:00',107.04,792,1388,63.42,'Intermediate' from dual union all
select 14182,'014182','1983-070A','Kosmos-1481','Oko',55827,'In Earth orbit','','1983-08-09 00:00:00',707.31,690,39147,62.98,'Molniya' from dual union all
select 14183,'014183','1983-070B','11S510','11S510',55827,'Reentered','1983-08-21 00:00:00','1983-07-15 00:00:00',92.2,210,555,62.83,'Intermediate' from dual union all
select 14184,'014184','1983-070C','BOZ','BOZ',55827,'Reentered','1983-07-24 00:00:00','1983-07-15 00:00:00',91.4,166,521,62.93,'Intermediate' from dual union all
select 14185,'014185','1983-071A','Kosmos-1482','Zenit-6',55828,'Landed','1983-07-27 00:00:00','1983-07-15 00:00:00',92.17,351,411,69.98,'Intermediate' from dual union all
select 14186,'014186','1983-071B','11S510','11S510',55828,'Reentered','1983-07-25 00:00:00','1983-07-15 00:00:00',89.93,206,336,69.97,'Intermediate' from dual union all
select 14187,'014187','1983-071C','Sensor cover','Sensor cover',55828,'Reentered','1983-07-15 00:00:00','1983-07-14 00:00:00',89.39,200,289,69.99,'Intermediate' from dual union all
select 14188,'014188','1983-071D','Sensor cover','Sensor cover',55828,'Reentered','1983-12-09 00:00:00','1983-09-06 00:00:00',89.87,206,329,70,'Intermediate' from dual union all
select 14189,'014189','1983-072A','GPS 8','Navstar GPS SVN 8',55829,'In Earth orbit','','1983-08-14 00:00:00',717.96,19924,20440,62.85,'Medium' from dual union all
select 1419,'001419','1965-020AV','deb Blok Kosmos','',11770,'Reentered','1966-09-01 00:00:00','1965-03-15 00:00:00',98.99,220,1201,56,'Intermediate' from dual union all
select 14190,'014190','1983-072B','SGS-II','Star 48  S/N 10004-9',55829,'In Earth orbit','','1983-08-14 00:00:00',371.82,561,20940,62.98,'Highly Ellip' from dual union all
select 14191,'014191','1983-070D','Blok-2BL','Blok-2BL No. 71629-653',55827,'In Earth orbit','','1983-08-08 00:00:00',707.95,802,39067,63.01,'Molniya' from dual union all
select 14192,'014192','1983-070E','deb Kosmos-1481','',55827,'In Earth orbit','','1983-08-08 00:00:00',708.82,689,39223,63.01,'Molniya' from dual union all
select 14193,'014193','1980-060F','Blok DM','11S86 No. 25L',50270,'In Earth orbit','','1993-05-30 00:00:00',1417.23,35330,35503,10.56,'Inclined Drift' from dual union all
select 14194,'014194','1981-027F','Blok DM','11S86 No. 18L',52550,'In Earth orbit','','1993-06-09 00:00:00',1474.42,36459,36608,10.72,'Inclined Drift' from dual union all
select 14195,'014195','1981-102F','Blok DM','11S86 No. 36L',52625,'In Earth orbit','','1993-05-30 00:00:00',1437.9,35807,35837,9.83,'Inclined Drift' from dual union all
select 14196,'014196','1983-041D','Star 37E S/N 40096','Star 37E',55798,'Reentered','1990-06-16 00:00:00','1990-06-15 00:00:00',89.94,106,437,23.69,'Equatorial' from dual union all
select 14197,'014197','1983-068C','KDU','KDU',55825,'Reentered','1983-12-11 00:00:00','1983-08-05 00:00:00',91.36,332,351,82.35,'Intermediate' from dual union all
select 14198,'014198','1983-068D','Blenda','Sunshade',55825,'Reentered','1983-09-03 00:00:00','1983-08-05 00:00:00',90.93,317,323,82.35,'Intermediate' from dual union all
select 14199,'014199','1983-073A','Molniya-1','Molniya-1 11F658 No. 66',55830,'Reentered','1995-02-23 00:00:00','1983-08-18 00:00:00',717.76,478,39876,62.93,'Molniya' from dual union all
select 142,'000142','61 Omicron 27','deb r/b Ablestar','',7273,'Reentered','1967-12-25 00:00:00','1961-09-06 00:00:00',99.13,456,979,67.16,'Intermediate' from dual union all
select 1420,'001420','1965-048A','NNS O-4','NNS O-4',11798,'In Earth orbit','','1965-08-01 00:00:00',106.9,1027,1140,90,'Polar' from dual union all
select 14200,'014200','1983-073B','11S510','11S510',55830,'Reentered','1983-08-20 00:00:00','1983-07-26 00:00:00',90.8,230,398,62.93,'Intermediate' from dual union all
select 14201,'014201','1983-073C','BOZ','BOZ',55830,'Reentered','1983-08-11 00:00:00','1983-07-26 00:00:00',90.9,197,441,62.93,'Intermediate' from dual union all
select 14202,'014202','1983-068E','KDU part','KDU part',55825,'Reentered','1983-08-16 00:00:00','1983-07-19 00:00:00',91.49,334,362,82.35,'Intermediate' from dual union all
select 14203,'014203','1983-068F','KDU part','KDU part',55825,'Reentered','1983-08-28 00:00:00','1983-07-20 00:00:00',93.42,353,530,82.36,'Intermediate' from dual union all
select 14204,'014204','1983-074A','Kosmos-1483','Resurs-F1 17F41 No. 23',55831,'Landed','1983-08-03 00:00:00','1983-07-22 00:00:00',89.8,258,272,82.32,'Intermediate' from dual union all
select 14205,'014205','1983-074B','11S510','11S510',55831,'Reentered','1983-07-26 00:00:00','1983-07-22 00:00:00',89.07,205,252,82.32,'Intermediate' from dual union all
select 14206,'014206','1983-073D','Blok-ML','Blok-ML No. 77334-463',55830,'Reentered','1990-02-03 00:00:00','1990-02-01 00:00:00',91.63,107,602,63.74,'Intermediate' from dual union all
select 14207,'014207','1983-075A','Kosmos-1484','Resurs-OE No. 3-2',55832,'Reentered','2013-01-28 00:00:00','1983-08-25 00:00:00',97.22,593,659,98.01,'Sun-Synch' from dual union all
select 14208,'014208','1983-075B','8A92ME','8A92ME',55832,'In Earth orbit','','1988-06-14 00:00:00',97.41,606,664,97.78,'Sun-Synch' from dual union all
select 14209,'014209','1983-075C','Kosmos-1484 part','',55832,'In Earth orbit','','1983-08-25 00:00:00',97.2,589,660,98.02,'Sun-Synch' from dual union all
select 1421,'001421','1965-049A','Kosmos-69','Zenit-4 No. 4',11799,'Landed','1965-07-03 00:00:00','1965-06-27 00:00:00',89.64,205,309,64.89,'Intermediate' from dual union all
select 14210,'014210','1983-076A','Kosmos-1485','Zenit-6',55833,'Landed','1983-08-09 00:00:00','1983-07-28 00:00:00',92.25,356,413,72.85,'Intermediate' from dual union all
select 14211,'014211','1983-076B','11S510','11S510',55833,'Reentered','1983-08-05 00:00:00','1983-07-28 00:00:00',89.83,192,341,72.85,'Intermediate' from dual union all
select 14212,'014212','1983-049G','Kosmos-1465 SS 1','ESO',55806,'Reentered','1983-11-02 00:00:00','1983-07-26 00:00:00',93.37,347,532,50.67,'Intermediate' from dual union all
select 14213,'014213','1983-049H','Kosmos-1465 SS 2','ESO',55806,'Reentered','1983-10-31 00:00:00','1983-07-26 00:00:00',93.35,347,530,50.66,'Intermediate' from dual union all
select 14214,'014214','1983-049J','Kosmos-1465 SS 3','ESO',55806,'Reentered','1983-10-29 00:00:00','1983-07-26 00:00:00',93.33,344,531,50.68,'Intermediate' from dual union all
select 14215,'014215','1983-049K','Kosmos-1465 SS 4','ESO',55806,'Reentered','1983-10-25 00:00:00','1983-07-26 00:00:00',93.29,342,530,50.66,'Intermediate' from dual union all
select 14216,'014216','1983-076C','Sensor cover','Sensor cover',55833,'Reentered','1983-09-14 00:00:00','1983-08-07 00:00:00',89.62,176,336,72.85,'Intermediate' from dual union all
select 14217,'014217','1983-071E','Blenda','Sunshade',55828,'Reentered','1983-09-20 00:00:00','1983-08-12 00:00:00',91.89,340,395,69.97,'Intermediate' from dual union all
select 14218,'014218','1983-071F','KDU part','KDU part',55828,'Reentered','1983-10-14 00:00:00','1983-08-13 00:00:00',91.79,331,394,69.97,'Intermediate' from dual union all
select 14219,'014219','1983-071G','KDU part','KDU part',55828,'Reentered','1983-10-20 00:00:00','1983-08-13 00:00:00',93.92,373,559,69.97,'Intermediate' from dual union all
select 1422,'001422','1965-050A','FANION I','P-11 No. 4402',11800,'Reentered','1968-08-22 00:00:00','1965-08-05 00:00:00',94.63,495,506,107.64,'Retrograde' from dual union all
select 14220,'014220','1981-028BQ','deb Kosmos-1260','',52551,'Reentered','1989-10-16 00:00:00','1989-10-14 00:00:00',90.25,270,304,64.98,'Intermediate' from dual union all
select 14221,'014221','1981-028BR','deb Kosmos-1260','',52551,'Reentered','1991-02-04 00:00:00','1991-02-04 00:00:00',87.58,150,160,64.96,'Intermediate' from dual union all
select 14222,'014222','1983-063C','P83-1 despin weight','',55820,'In Earth orbit','','1983-07-27 00:00:00',100.41,757,799,82.38,'Intermediate' from dual union all
select 14223,'014223','1983-063D','P83-1 despin weight','',55820,'In Earth orbit','','1983-08-01 00:00:00',101.39,771,878,81.66,'Intermediate' from dual union all
select 14224,'014224','1983-033D','deb Rohini RS-D-2','',55790,'Reentered','1988-06-06 00:00:00','1983-07-27 00:00:00',96.73,378,827,46.77,'Intermediate' from dual union all
select 14225,'014225','1983-033E','deb Rohini RS-D-2','',55790,'Reentered','1989-01-04 00:00:00','1983-07-27 00:00:00',97.03,394,839,46.44,'Intermediate' from dual union all
select 14226,'014226','1966-005J','NNS O-7 insulation','',13412,'Reentered','1989-07-15 00:00:00','1982-12-14 00:00:00',105.5,846,1190,89.74,'Polar' from dual union all
select 14227,'014227','1982-109C','deb Kosmos-1420','Strela-2M debris 1',54418,'Reentered','1990-09-19 00:00:00','1983-07-27 00:00:00',100.41,761,795,73.99,'Intermediate' from dual;
insert into SATELLITE
select 14228,'014228','1981-053HN','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',105.92,951,1124,82.97,'Intermediate' from dual union all
select 14229,'014229','1983-075D','Kosmos-1484 part','Kosmos-1484 high debris',55832,'In Earth orbit','','1983-08-25 00:00:00',97.73,619,682,97.96,'Sun-Synch' from dual union all
select 1423,'001423','1965-049B','11A57I','11A57I',11799,'Reentered','1965-07-06 00:00:00','1965-07-02 00:00:00',88.86,180,257,64.9,'Intermediate' from dual union all
select 14230,'014230','1983-071H','KDU','KDU',55828,'Reentered','1984-06-25 00:00:00','1983-08-12 00:00:00',92.11,349,407,69.97,'Intermediate' from dual union all
select 14231,'014231','1983-071J','KDU part','KDU part',55828,'Reentered','1983-07-31 00:00:00','1983-07-28 00:00:00',91.45,303,389,69.96,'Intermediate' from dual union all
select 14232,'014232','1983-064C','Kosmos-1471 AO part','Feniks AO part',55821,'Reentered','1983-07-28 00:00:00','1983-07-28 00:00:00',89.5,161,339,67.14,'Intermediate' from dual union all
select 14233,'014233','1983-064D','Kosmos-1471 AO part','Feniks AO part',55821,'Reentered','1983-07-28 00:00:00','1983-07-28 00:00:00',89.61,154,357,67.13,'Intermediate' from dual union all
select 14234,'014234','1983-077A','Telstar 301','Telstar 301',55834,'In Earth orbit','','1983-08-28 00:00:00',1436.11,35442,36132,.15,'Stationary' from dual union all
select 14235,'014235','1983-077B','Delta 171','Delta SSPS AJ10-118K',55834,'Reentered','1983-09-28 00:00:00','1983-08-27 00:00:00',90.94,263,379,25.99,'Intermediate' from dual union all
select 14236,'014236','1983-077C','PAM-D','Star 48  S/N 10027-8',55834,'Reentered','2001-09-14 00:00:00','1983-08-27 00:00:00',655.35,218,37011,22.95,'GEO Transfer' from dual union all
select 14237,'014237','1983-078A','OPS 7304','JUMPSEAT 7 (or QUASAR 5)',55835,'In Earth orbit','','1983-07-31 00:00:00',684.27,342,38347,63.6,'GEO Transfer' from dual union all
select 14238,'014238','1983-078B','Agena D','Agena D',55835,'In Earth orbit','','1983-07-31 00:00:00',684.27,342,38347,63.6,'GEO Transfer' from dual union all
select 14239,'014239','1983-074C','Blenda','Sunshade',55831,'Reentered','1983-08-11 00:00:00','1983-08-03 00:00:00',89.66,243,273,82.33,'Intermediate' from dual union all
select 1424,'001424','1965-050B','OPS 5501','GAMBIT SV 969',11800,'Reentered','1965-06-30 00:00:00','1965-06-26 00:00:00',88.79,148,282,107.64,'Retrograde' from dual union all
select 14240,'014240','1983-079A','Kosmos-1486','Strela-2M',55836,'In Earth orbit','','1992-06-29 00:00:00',100.6,777,797,74.05,'Intermediate' from dual union all
select 14241,'014241','1983-079B','S3M','S3M',55836,'In Earth orbit','','1990-03-25 00:00:00',100.6,776,798,74.06,'Intermediate' from dual union all
select 14242,'014242','1983-074D','KDU part','KDU part',55831,'Reentered','1983-08-11 00:00:00','1983-08-03 00:00:00',91.62,259,449,82.33,'Intermediate' from dual union all
select 14243,'014243','1983-074E','KDU','KDU',55831,'Reentered','1983-08-26 00:00:00','1983-08-03 00:00:00',89.77,254,273,82.32,'Intermediate' from dual union all
select 14244,'014244','1978-064B','deb Seasat 1','Insulation',45259,'Reentered','1986-10-30 00:00:00','1983-08-03 00:00:00',100.4,773,781,107.99,'Retrograde' from dual union all
select 14245,'014245','1983-080A','Kosmos-1487','Resurs-F1 17F41 No. 25',55837,'Landed','1983-08-19 00:00:00','1983-08-10 00:00:00',89.78,257,270,82.31,'Intermediate' from dual union all
select 14246,'014246','1983-080B','11S510','11S510',55837,'Reentered','1983-08-11 00:00:00','1983-08-09 00:00:00',88.54,184,221,82.31,'Intermediate' from dual union all
select 14247,'014247','1983-043L','KDU part','KDU part',55800,'Reentered','1983-08-17 00:00:00','1983-08-05 00:00:00',91.83,317,412,70.32,'Intermediate' from dual union all
select 14248,'014248','1983-081A','Sakura 2B','CS-2B',55838,'In Earth orbit','','1983-09-26 00:00:00',1436.14,35785,35790,.18,'Stationary' from dual union all
select 14249,'014249','1983-080C','Sensor cover','Sensor cover',55837,'Reentered','1983-08-14 00:00:00','1983-08-06 00:00:00',89.29,203,276,82.32,'Intermediate' from dual union all
select 1425,'001425','1965-048C','NNS adapter','NNS adapter',11798,'In Earth orbit','','1965-08-02 00:00:00',106.87,1024,1140,90,'Polar' from dual union all
select 14250,'014250','1983-081B','N-II Stage 2 S/N N-11(F)','AJ10-118FJ',55838,'Reentered','1983-09-27 00:00:00','1983-08-15 00:00:00',99.38,167,1291,30.13,'Intermediate' from dual union all
select 14251,'014251','1983-082A','Kosmos-1488','Zenit-6',55839,'Landed','1983-08-23 00:00:00','1983-08-11 00:00:00',92.25,355,415,72.86,'Intermediate' from dual union all
select 14252,'014252','1983-082B','11S510','11S510',55839,'Reentered','1983-08-18 00:00:00','1983-08-11 00:00:00',89.83,192,341,72.85,'Intermediate' from dual union all
select 14253,'014253','1983-076D','Blenda','Sunshade',55833,'Reentered','1983-09-29 00:00:00','1983-08-25 00:00:00',91.81,340,386,72.85,'Intermediate' from dual union all
select 14254,'014254','1983-076E','KDU part','KDU part',55833,'Reentered','1983-10-30 00:00:00','1983-08-25 00:00:00',91.87,335,398,72.85,'Intermediate' from dual union all
select 14255,'014255','1983-076F','KDU','KDU',55833,'Reentered','1984-08-14 00:00:00','1983-08-25 00:00:00',92.19,351,413,72.86,'Intermediate' from dual union all
select 14256,'014256','1983-083A','Kosmos-1489','Oktan No. 252',55840,'Landed','1983-09-23 00:00:00','1983-08-17 00:00:00',89.63,167,346,64.72,'Intermediate' from dual union all
select 14257,'014257','1983-083B','11S510','11S510',55840,'Reentered','1983-08-14 00:00:00','1983-08-12 00:00:00',88.76,167,260,64.74,'Intermediate' from dual union all
select 14258,'014258','1983-084A','Kosmos-1490','Uragan No. 12L',55841,'In Earth orbit','','1983-09-09 00:00:00',675.71,19087,19172,64.84,'Medium' from dual union all
select 14259,'014259','1983-084B','Kosmos-1491','Uragan No. 13L',55841,'In Earth orbit','','1983-09-09 00:00:00',675.71,19104,19155,64.91,'Medium' from dual union all
select 1426,'001426','1965-050C','Agena D 4817','Agena D',11800,'Reentered','1965-06-27 00:00:00','1965-06-26 00:00:00',88.52,150,252,107.63,'Retrograde' from dual union all
select 14260,'014260','1983-084C','Kosmos-1492','Uragan GVM',55841,'In Earth orbit','','1983-09-13 00:00:00',676.81,19151,19163,64.82,'Medium' from dual union all
select 14261,'014261','1983-084D','Perekhodnik','Sredniy perekhodnik',55841,'Reentered','1983-08-11 00:00:00','1983-08-10 00:00:00',88.41,188,204,51.57,'Intermediate' from dual union all
select 14262,'014262','1983-084E','8S812','8S812',55841,'Reentered','1983-08-12 00:00:00','1983-08-11 00:00:00',88.12,170,193,51.63,'Intermediate' from dual union all
select 14263,'014263','1983-083C','Kosmos-1489 part','Oktan part',55840,'Reentered','1983-08-11 00:00:00','1983-08-11 00:00:00',88.4,158,233,64.72,'Intermediate' from dual union all
select 14264,'014264','1983-084F','Blok DM2','11S861 No. 2L',55841,'In Earth orbit','','1983-09-17 00:00:00',676.27,19137,19150,64.83,'Medium' from dual union all
select 14265,'014265','1982-038C','deb Kosmos-1355','',54347,'Reentered','1983-09-13 00:00:00','1983-09-12 00:00:00',92.07,283,470,64.54,'Intermediate' from dual union all
select 14266,'014266','1982-038D','deb Kosmos-1355','',54347,'Reentered','1983-08-20 00:00:00','1983-08-16 00:00:00',91.39,202,484,64.89,'Intermediate' from dual union all
select 14267,'014267','1982-038E','deb Kosmos-1355','',54347,'Reentered','1983-09-20 00:00:00','1983-09-17 00:00:00',91.02,299,351,65.08,'Intermediate' from dual union all
select 14268,'014268','1982-038F','deb Kosmos-1355','',54347,'Reentered','1983-08-31 00:00:00','1983-08-29 00:00:00',90.11,272,288,65.02,'Intermediate' from dual union all
select 14269,'014269','1982-038G','deb Kosmos-1355','',54347,'Reentered','1983-08-25 00:00:00','1983-08-25 00:00:00',91.35,313,368,64.76,'Intermediate' from dual union all
select 1427,'001427','1965-050D','OPS 6749 sensor cover','',11800,'Reentered','1965-11-26 00:00:00','1965-07-27 00:00:00',94.51,489,501,107.64,'Retrograde' from dual union all
select 14270,'014270','1982-038H','deb Kosmos-1355','',54347,'Reentered','1983-09-12 00:00:00','1983-09-12 00:00:00',89.51,239,262,65.06,'Intermediate' from dual union all
select 14271,'014271','1982-038J','deb Kosmos-1355','',54347,'Reentered','1982-08-14 00:00:00','1983-08-13 00:00:00',90.83,269,362,65.24,'Intermediate' from dual union all
select 14272,'014272','1982-038K','deb Kosmos-1355','',54347,'Reentered','1983-09-12 00:00:00','1983-09-11 00:00:00',89.99,240,307,65.39,'Intermediate' from dual union all
select 14273,'014273','1982-038L','deb Kosmos-1355','',54347,'Reentered','1983-08-20 00:00:00','1983-08-20 00:00:00',88.13,170,194,65.04,'Intermediate' from dual union all
select 14274,'014274','1982-038M','deb Kosmos-1355','',54347,'Reentered','1983-09-26 00:00:00','1983-09-26 00:00:00',87.9,159,183,65.07,'Intermediate' from dual union all
select 14275,'014275','1982-038N','deb Kosmos-1355','',54347,'Reentered','1983-08-27 00:00:00','1984-02-16 00:00:00',90.27,279,296,65.04,'Intermediate' from dual union all
select 14276,'014276','1982-038P','deb Kosmos-1355','',54347,'Reentered','1983-08-22 00:00:00','1983-08-16 00:00:00',91.5,333,364,65.06,'Intermediate' from dual union all
select 14277,'014277','1983-084G','SOZ','SOZ',55841,'In Earth orbit','','1983-09-11 00:00:00',340.69,396,19165,51.97,'Highly Ellip' from dual union all
select 14278,'014278','1983-084H','SOZ','SOZ',55841,'In Earth orbit','','1983-09-10 00:00:00',340.36,375,19165,52.09,'Highly Ellip' from dual union all
select 14279,'014279','1982-038Q','deb Kosmos-1355','',54347,'Reentered','1983-08-29 00:00:00','1983-08-27 00:00:00',90.25,239,335,65.03,'Intermediate' from dual union all
select 1428,'001428','1965-048B','Ablestar 019','AJ10-104D',11798,'In Earth orbit','','1965-08-13 00:00:00',106.57,1024,1111,90,'Polar' from dual union all
select 14280,'014280','1982-038R','deb Kosmos-1355','',54347,'Reentered','1983-11-26 00:00:00','1983-11-23 00:00:00',90,262,286,65.05,'Intermediate' from dual union all
select 14281,'014281','1982-038S','deb Kosmos-1355','',54347,'Reentered','1983-08-24 00:00:00','1983-08-21 00:00:00',90.58,289,317,64.93,'Intermediate' from dual union all
select 14282,'014282','1982-038T','deb Kosmos-1355','',54347,'Reentered','1983-08-20 00:00:00','1983-08-16 00:00:00',91,302,346,65.16,'Intermediate' from dual union all
select 14283,'014283','1983-085A','Progress-17','Progress 7K-TG No. 119',55842,'Deorbited','1983-09-19 00:00:00','1983-08-24 00:00:00',91.1,319,338,51.61,'Intermediate' from dual union all
select 14284,'014284','1983-085B','11S510','11S510',55842,'Reentered','1983-08-20 00:00:00','1983-08-19 00:00:00',88.22,172,201,51.64,'Intermediate' from dual union all
select 14285,'014285','1982-038U','deb Kosmos-1355','',54347,'Reentered','1983-08-23 00:00:00','1983-08-16 00:00:00',91.47,331,363,65.07,'Intermediate' from dual union all
select 14286,'014286','1982-038V','deb Kosmos-1355','',54347,'Reentered','1983-08-23 00:00:00','1983-08-23 00:00:00',92.03,301,448,65.12,'Intermediate' from dual union all
select 14287,'014287','1983-081C','Star 37E S/N 40094','Star 37E',55838,'Reentered','1991-05-25 00:00:00','1983-09-06 00:00:00',635.32,201,36004,28.65,'GEO Transfer' from dual union all
select 14288,'014288','1983-086A','Fanhui Shi Weixing (05)','Jianbing-1 No. 6',55843,'Reentered','1983-09-03 00:00:00','1983-08-26 00:00:00',89.6,170,340,63.3,'Intermediate' from dual union all
select 14289,'014289','1983-086B','CZ-2C Stage 2','CZ-2C Stage 2',55843,'Reentered','1983-08-30 00:00:00','1983-08-22 00:00:00',89.66,168,348,63.31,'Intermediate' from dual union all
select 1429,'001429','1965-049C','Kosmos-69 sensor cover','',11799,'Reentered','1965-06-30 00:00:00','1965-06-26 00:00:00',91.49,298,398,64.8,'Intermediate' from dual union all
select 14290,'014290','1983-086C','CZ-2C sep motor cover','',55843,'Reentered','1983-08-20 00:00:00','1983-08-19 00:00:00',91.53,165,535,63.01,'Intermediate' from dual union all
select 14291,'014291','1983-080D','Blenda','Sunshade',55837,'Reentered','1983-08-26 00:00:00','1983-08-20 00:00:00',89.55,239,265,82.31,'Intermediate' from dual union all
select 14292,'014292','1983-080E','KDU','KDU',55837,'Reentered','1983-09-11 00:00:00','1983-08-19 00:00:00',89.74,256,268,82.31,'Intermediate' from dual union all
select 14293,'014293','1983-086D','CZ-2C sep motor cover','',55843,'Reentered','1983-08-20 00:00:00','1983-08-20 00:00:00',90.2,195,374,63.25,'Intermediate' from dual union all
select 14294,'014294','1983-086E','CZ-2C sep motor cover','',55843,'Reentered','1983-08-21 00:00:00','1983-08-19 00:00:00',89.54,170,334,63.29,'Intermediate' from dual union all
select 14295,'014295','1983-080F','KDU part','KDU part',55837,'Reentered','1983-08-28 00:00:00','1983-08-20 00:00:00',91.73,262,457,82.31,'Intermediate' from dual union all
select 14296,'014296','1983-086F','CZ-2C sep motor cover','',55843,'Reentered','1983-08-20 00:00:00','1983-08-20 00:00:00',90.42,162,429,62.87,'Intermediate' from dual union all
select 14297,'014297','1983-038G','deb Kosmos-1456','',55795,'Reentered','1998-05-11 00:00:00','1991-03-09 00:00:00',718.98,3436,36978,66.86,'GEO Transfer' from dual union all
select 14298,'014298','1983-082C','Blenda','Sunshade',55839,'Reentered','1983-11-13 00:00:00','1983-09-08 00:00:00',91.9,337,398,72.85,'Intermediate' from dual union all
select 14299,'014299','1983-087A','Kosmos-1493','Zenit-6',55844,'Landed','1983-09-06 00:00:00','1983-08-25 00:00:00',92.25,358,412,72.86,'Intermediate' from dual union all
select 143,'000143','61 Omicron 28','deb r/b Ablestar','',7273,'Reentered','1962-06-16 00:00:00','1961-08-08 00:00:00',97.64,280,1011,66.44,'Intermediate' from dual union all
select 1430,'001430','1965-051A','Tiros 10','Tiros OT1',11801,'In Earth orbit','','1965-11-23 00:00:00',100.64,743,835,98.6,'Sun-Synch' from dual union all
select 14300,'014300','1983-087B','11S510','11S510',55844,'Reentered','1983-08-31 00:00:00','1983-08-25 00:00:00',89.75,186,339,72.85,'Intermediate' from dual union all
select 14301,'014301','1983-038H','deb Kosmos-1456','',55795,'Reentered','1998-05-10 00:00:00','1983-08-26 00:00:00',718.13,814,39558,63.63,'Molniya' from dual union all
select 14302,'014302','1983-082D','KDU','KDU',55839,'Reentered','1984-09-05 00:00:00','1983-09-08 00:00:00',92.19,353,411,72.86,'Intermediate' from dual union all
select 14303,'014303','1983-082E','KDU part','KDU part',55839,'Reentered','1983-08-28 00:00:00','1983-08-25 00:00:00',91.3,324,352,72.88,'Intermediate' from dual union all
select 14304,'014304','1983-087C','Sensor cover','Sensor cover',55844,'Reentered','1983-08-30 00:00:00','1983-08-26 00:00:00',89.26,178,298,72.89,'Intermediate' from dual union all
select 14305,'014305','1983-013D','VA adapter','VA adapter',55770,'Reentered','1983-09-12 00:00:00','1983-08-24 00:00:00',91.05,309,344,51.61,'Intermediate' from dual union all
select 14306,'014306','1983-038J','deb Kosmos-1456','',55795,'Reentered','1997-11-04 00:00:00','1983-08-27 00:00:00',720.64,654,39842,63.66,'Molniya' from dual union all
select 14307,'014307','1983-088A','Raduga','Gran'' No. 24L',55845,'In Earth orbit','','1983-09-24 00:00:00',1436.14,35739,35836,1.26,'Stationary' from dual union all
select 14308,'014308','1983-088B','8S812','8S812',55845,'Reentered','1983-08-28 00:00:00','1983-08-27 00:00:00',88,173,178,51.62,'Intermediate' from dual union all
select 14309,'014309','1983-088C','Perekhodnik','Sredniy perekhodnik',55845,'Reentered','1983-08-26 00:00:00','1983-08-26 00:00:00',88.36,187,200,51.61,'Intermediate' from dual union all
select 1431,'001431','1965-052A','Kosmos-70','DS-A1 No. 7',11802,'Reentered','1966-12-18 00:00:00','1965-08-05 00:00:00',98.12,225,1113,48.76,'Intermediate' from dual union all
select 14310,'014310','1983-088D','SOZ','SOZ',55845,'Reentered','1984-07-06 00:00:00','1983-09-26 00:00:00',649.26,317,36602,47.26,'GEO Transfer' from dual union all
select 14311,'014311','1983-088E','SOZ','SOZ',55845,'Reentered','1986-10-27 00:00:00','1983-09-28 00:00:00',647.93,265,36586,47.24,'GEO Transfer' from dual union all
select 14312,'014312','1983-089A','Challenger (STS-8)','OV-099',55846,'Landed','1983-09-05 00:00:00','1983-09-01 00:00:00',90.59,298,309,28.47,'Intermediate' from dual union all
select 14313,'014313','1983-090A','Molniya-3','Molniya-3 No. 32',55847,'Reentered','2001-05-22 00:00:00','1983-10-01 00:00:00',717.7,477,39874,62.87,'Molniya' from dual union all
select 14314,'014314','1983-090B','11S510','11S510',55847,'Reentered','1983-09-12 00:00:00','1983-09-01 00:00:00',90.67,182,433,62.84,'Intermediate' from dual union all
select 14315,'014315','1983-090C','BOZ','BOZ',55847,'Reentered','1983-09-19 00:00:00','1983-09-06 00:00:00',90.37,204,382,62.84,'Intermediate' from dual union all
select 14316,'014316','1983-091A','Kosmos-1494','Romb',55848,'Reentered','1985-09-26 00:00:00','1983-09-30 00:00:00',93.44,343,543,50.68,'Intermediate' from dual union all
select 14317,'014317','1983-091B','S3M','S3M',55848,'Reentered','1985-05-24 00:00:00','1983-10-01 00:00:00',93.34,340,536,50.68,'Intermediate' from dual;
insert into SATELLITE
select 14318,'014318','1983-089B','Insat 1B','Insat 1B',55846,'In Earth orbit','','1983-11-14 00:00:00',1436.06,35778,35793,.09,'Stationary' from dual union all
select 14319,'014319','1983-090D','Blok-ML','Blok-ML',55847,'Reentered','2002-01-16 00:00:00','1983-10-07 00:00:00',731.37,494,40529,62.93,'Molniya' from dual union all
select 1432,'001432','1965-052B','S1','S1',11802,'Reentered','1966-06-06 00:00:00','1965-08-07 00:00:00',97.95,229,1092,48.76,'Intermediate' from dual union all
select 14320,'014320','1983-092A','Kosmos-1495','Fram',55849,'Landed','1983-09-16 00:00:00','1983-09-05 00:00:00',88.95,212,234,82.32,'Intermediate' from dual union all
select 14321,'014321','1983-092B','11S510','11S510',55849,'Reentered','1983-09-06 00:00:00','1983-09-05 00:00:00',88.28,176,203,82.33,'Intermediate' from dual union all
select 14322,'014322','1983-087D','Blenda','Sunshade',55844,'Reentered','1983-09-18 00:00:00','1983-09-06 00:00:00',92.38,371,412,72.6,'Intermediate' from dual union all
select 14323,'014323','1983-087E','KDU part','KDU part',55844,'Reentered','1983-12-06 00:00:00','1983-09-22 00:00:00',91.91,340,397,72.85,'Intermediate' from dual union all
select 14324,'014324','1983-087F','KDU','KDU',55844,'Reentered','1984-10-10 00:00:00','1983-09-23 00:00:00',92.19,357,406,72.85,'Intermediate' from dual union all
select 14325,'014325','1983-087G','KDU part','KDU part',55844,'Reentered','1983-10-29 00:00:00','1983-09-22 00:00:00',92,350,396,72.85,'Intermediate' from dual union all
select 14326,'014326','1983-093A','Kosmos-1496','Oktan No. 251',55850,'Landed','1983-10-19 00:00:00','1983-09-14 00:00:00',89.71,167,354,67.18,'Intermediate' from dual union all
select 14327,'014327','1983-093B','11S510','11S510',55850,'Reentered','1983-09-11 00:00:00','1983-09-09 00:00:00',89.18,164,304,67.18,'Intermediate' from dual union all
select 14328,'014328','1983-094A','Satcom 2R','RCA-G',55851,'In Earth orbit','','1983-10-10 00:00:00',1436.09,35764,35809,.03,'Stationary' from dual union all
select 14329,'014329','1983-094B','Delta 172','Delta SSPS AJ10-118K',55851,'Reentered','1999-04-16 00:00:00','1990-01-30 00:00:00',119.78,284,3058,25.52,'Intermediate' from dual union all
select 1433,'001433','1965-051B','Altair 2 X-258C4 RH-84','X-258C4 RH-84',11801,'In Earth orbit','','1966-10-02 00:00:00',100.69,745,838,98.62,'Sun-Synch' from dual union all
select 14330,'014330','1983-095A','Kosmos-1497','Zenit-6',55852,'Landed','1983-09-23 00:00:00','1983-09-11 00:00:00',92.24,355,414,72.86,'Intermediate' from dual union all
select 14331,'014331','1983-095B','11S510','11S510',55852,'Reentered','1983-09-19 00:00:00','1983-09-11 00:00:00',89.95,192,352,72.84,'Intermediate' from dual union all
select 14332,'014332','1983-094C','Star 37E S/N 40098','Star 37E',55851,'Reentered','1984-05-21 00:00:00','1983-10-15 00:00:00',616.99,198,35061,24.04,'GEO Transfer' from dual union all
select 14333,'014333','1983-088F','Blok DM','11S86 No. 40L',55845,'In Earth orbit','','1983-10-01 00:00:00',1475.17,36485,36611,1.32,'Drift GEO' from dual union all
select 14334,'014334','1983-096A','Kosmos-1498','Resurs-F1 17F41 No. 24',55853,'Landed','1983-09-28 00:00:00','1983-09-21 00:00:00',89.78,257,270,82.31,'Intermediate' from dual union all
select 14335,'014335','1983-096B','11S510','11S510',55853,'Reentered','1983-09-19 00:00:00','1983-09-16 00:00:00',88.97,200,248,82.32,'Intermediate' from dual union all
select 14336,'014336','1983-096C','Sensor cover','Sensor cover',55853,'Reentered','1983-09-19 00:00:00','1983-09-16 00:00:00',88.92,198,245,82.31,'Intermediate' from dual union all
select 14337,'014337','1983-092C','Blenda','Sunshade',55849,'Reentered','1983-09-19 00:00:00','1983-09-17 00:00:00',87.91,162,181,82.32,'Intermediate' from dual union all
select 14338,'014338','1983-092D','KDU','KDU',55849,'Reentered','1983-09-22 00:00:00','1983-09-16 00:00:00',88.84,210,225,82.32,'Intermediate' from dual union all
select 14339,'014339','1983-097A','Kosmos-1499','Zenit-6',55854,'Landed','1983-10-01 00:00:00','1983-09-19 00:00:00',92.25,355,415,72.85,'Intermediate' from dual union all
select 1434,'001434','1965-052C','deb Kosmos-70','',11802,'Reentered','1965-08-13 00:00:00','1965-07-11 00:00:00',95.67,255,847,48.84,'Intermediate' from dual union all
select 14340,'014340','1983-097B','11S510','11S510',55854,'Reentered','1983-09-26 00:00:00','1983-09-19 00:00:00',89.88,192,345,72.84,'Intermediate' from dual union all
select 14341,'014341','1983-097C','Sensor cover','Sensor cover',55854,'Reentered','1983-09-19 00:00:00','1983-09-18 00:00:00',92.25,356,413,72.85,'Intermediate' from dual union all
select 14342,'014342','1982-038W','deb Kosmos-1355','',54347,'Reentered','1983-11-20 00:00:00','1983-11-19 00:00:00',88.18,180,189,65.11,'Intermediate' from dual union all
select 14343,'014343','1982-115AF','deb Blok-L','',54424,'Reentered','1985-04-02 00:00:00','1983-09-22 00:00:00',94.19,365,594,62.85,'Intermediate' from dual union all
select 14344,'014344','1982-037C','deb Kosmos [-1486] [UNID]        ','Strela-2M debris',54346,'In Earth orbit','','1983-09-18 00:00:00',101.27,793,844,74.03,'Intermediate' from dual union all
select 14345,'014345','1979-089C','deb Kosmos-1140','Strela-2M debris',47573,'In Earth orbit','','1983-09-19 00:00:00',100.74,781,806,74.04,'Intermediate' from dual union all
select 14346,'014346','1981-041D','deb Kosmos-1269','Strela-2M debris 2 (high',52564,'In Earth orbit','','1983-09-22 00:00:00',100.57,776,796,74.06,'Intermediate' from dual union all
select 14347,'014347','1983-082F','KDU part','KDU part',55839,'Reentered','1983-11-13 00:00:00','1983-09-08 00:00:00',94.19,386,573,72.87,'Intermediate' from dual union all
select 14348,'014348','1980-089BF','deb Kosmos-1220','',50299,'Reentered','1984-10-24 00:00:00','1985-09-26 00:00:00',90.2,270,299,64.97,'Intermediate' from dual union all
select 14349,'014349','1980-089BG','deb Kosmos-1220','',50299,'Reentered','1984-03-04 00:00:00','1983-06-01 00:00:00',96.67,510,689,64.94,'Intermediate' from dual union all
select 1435,'001435','1965-048D','deb NNS O-4','',11798,'In Earth orbit','','1966-12-24 00:00:00',106.89,1026,1140,90,'Polar' from dual union all
select 14350,'014350','1980-089BH','deb Kosmos-1220','',50299,'Reentered','1988-05-27 00:00:00','1988-05-24 00:00:00',90.71,292,327,64.9,'Intermediate' from dual union all
select 14351,'014351','1980-089BJ','deb Kosmos-1220','',50299,'Reentered','1985-11-09 00:00:00','1985-04-12 00:00:00',93.54,416,480,64.91,'Intermediate' from dual union all
select 14352,'014352','1980-089BK','deb Kosmos-1220','',50299,'Reentered','1985-04-15 00:00:00','1983-09-20 00:00:00',96.82,508,705,64.97,'Intermediate' from dual union all
select 14353,'014353','1980-089BL','deb Kosmos-1220','',50299,'Reentered','1988-07-11 00:00:00','1988-07-11 00:00:00',88.76,203,224,64.94,'Intermediate' from dual union all
select 14354,'014354','1980-089BM','deb Kosmos-1220','',50299,'Reentered','1989-02-07 00:00:00','1989-02-05 00:00:00',90.13,272,289,64.95,'Intermediate' from dual union all
select 14355,'014355','1980-089BN','deb Kosmos-1220','',50299,'Reentered','1989-02-07 00:00:00','1989-02-06 00:00:00',89.47,244,252,64.91,'Intermediate' from dual union all
select 14356,'014356','1981-053HP','deb Kosmos-1275','',52576,'Reentered','1999-02-28 00:00:00','1993-08-07 00:00:00',97.66,620,673,82.81,'Intermediate' from dual union all
select 14357,'014357','1981-053HQ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',103.69,911,955,82.86,'Intermediate' from dual union all
select 14358,'014358','1981-053HR','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-03 00:00:00',105.47,831,1202,82.83,'Intermediate' from dual union all
select 14359,'014359','1981-053HS','deb Kosmos-1275','',52576,'Reentered','1989-07-07 00:00:00','1989-07-06 00:00:00',92,338,408,82.91,'Intermediate' from dual union all
select 1436,'001436','1965-020AW','deb Blok Kosmos','',11770,'Reentered','1965-05-20 00:00:00','1965-07-05 00:00:00',103.49,235,1613,56,'Intermediate' from dual union all
select 14360,'014360','1981-053HT','deb Kosmos-1275','',52576,'Reentered','2003-05-13 00:00:00','1983-09-20 00:00:00',104.3,847,1076,82.95,'Intermediate' from dual union all
select 14361,'014361','1981-053HU','deb Kosmos-1275','',52576,'Reentered','2001-03-10 00:00:00','1993-07-29 00:00:00',101.46,812,844,82.93,'Intermediate' from dual union all
select 14362,'014362','1981-053HV','deb Kosmos-1275','',52576,'Reentered','1992-03-13 00:00:00','1992-03-12 00:00:00',90.01,253,297,82.8,'Intermediate' from dual union all
select 14363,'014363','1981-053HW','deb Kosmos-1275','',52576,'Reentered','1989-08-20 00:00:00','1989-08-20 00:00:00',89.11,221,240,83.2,'Intermediate' from dual union all
select 14364,'014364','1981-053HX','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',104.06,917,984,82.79,'Intermediate' from dual union all
select 14365,'014365','1983-098A','Galaxy 2','Galaxy II',55855,'In Earth orbit','','1983-10-23 00:00:00',1436.06,35783,35789,.08,'Stationary' from dual union all
select 14366,'014366','1983-098B','Delta 173','Delta SSPS AJ10-118K',55855,'Reentered','1983-10-30 00:00:00','1983-09-29 00:00:00',90.86,278,355,26.64,'Intermediate' from dual union all
select 14367,'014367','1983-095C','Blenda','Sunshade',55852,'Reentered','1983-12-14 00:00:00','1983-10-09 00:00:00',91.87,337,395,72.85,'Intermediate' from dual union all
select 14368,'014368','1983-095D','KDU','KDU',55852,'Reentered','1984-10-08 00:00:00','1983-10-09 00:00:00',92.18,353,410,72.85,'Intermediate' from dual union all
select 14369,'014369','1983-098C','PAM-D','Star 48  S/N 10028-8',55855,'Reentered','1986-12-31 00:00:00','1983-10-22 00:00:00',644.25,204,36459,23.43,'GEO Transfer' from dual union all
select 1437,'001437','1965-020AX','deb Blok Kosmos','',11770,'Reentered','1965-10-28 00:00:00','1965-03-15 00:00:00',100.98,290,1321,56.2,'Intermediate' from dual union all
select 14370,'014370','1983-095E','KDU part','KDU part',55852,'Reentered','1983-11-14 00:00:00','1983-10-09 00:00:00',91.9,344,392,72.85,'Intermediate' from dual union all
select 14371,'014371','1983-083D','Kosmos-1489 AO part','Oktan AO part',55840,'Reentered','1983-09-24 00:00:00','1983-09-24 00:00:00',88.07,141,217,64.72,'Intermediate' from dual union all
select 14372,'014372','1983-099A','Kosmos-1500','Okean-OE NKhM No. 1',55856,'In Earth orbit','','1983-11-06 00:00:00',97.69,631,666,82.54,'Intermediate' from dual union all
select 14373,'014373','1983-099B','S5M','S5M',55856,'In Earth orbit','','1990-07-02 00:00:00',97.55,625,658,82.54,'Intermediate' from dual union all
select 14374,'014374','1983-096D','KDU','KDU',55853,'Reentered','1983-10-16 00:00:00','1983-09-28 00:00:00',89.69,251,267,82.31,'Intermediate' from dual union all
select 14375,'014375','1983-096E','Blenda','Sunshade',55853,'Reentered','1983-10-04 00:00:00','1983-09-28 00:00:00',89.51,240,261,82.31,'Intermediate' from dual union all
select 14376,'014376','1983-096F','KDU part','KDU part',55853,'Reentered','1983-10-05 00:00:00','1983-09-28 00:00:00',91.74,250,470,82.35,'Intermediate' from dual union all
select 14377,'014377','1983-100A','Ekran','Ekran No. 25L',55857,'In Earth orbit','','1983-10-31 00:00:00',1435.96,35779,35789,.32,'Stationary' from dual union all
select 14378,'014378','1983-100B','8S812','8S812',55857,'Reentered','1983-10-02 00:00:00','1983-10-01 00:00:00',87.97,172,176,51.6,'Intermediate' from dual union all
select 14379,'014379','1983-100C','Perekhodnik','Sredniy perekhodnik',55857,'Reentered','1983-09-30 00:00:00','1983-09-29 00:00:00',88.34,189,196,51.61,'Intermediate' from dual union all
select 1438,'001438','1965-020AY','S3 large debris','RCS 0.20',11770,'Reentered','1980-11-03 00:00:00','1965-07-05 00:00:00',108.33,490,1808,56.02,'Intermediate' from dual union all
select 14380,'014380','1983-101A','Kosmos-1501','Romb',55858,'Reentered','1989-05-26 00:00:00','1983-10-30 00:00:00',94.35,465,509,82.94,'Intermediate' from dual union all
select 14381,'014381','1983-101B','S3M','S3M',55858,'Reentered','1989-03-30 00:00:00','1988-06-14 00:00:00',93.29,422,450,82.93,'Intermediate' from dual union all
select 14382,'014382','1983-097D','Blenda','Sunshade',55854,'Reentered','1983-12-20 00:00:00','1983-10-23 00:00:00',91.68,331,383,72.84,'Intermediate' from dual union all
select 14383,'014383','1983-097E','KDU part','KDU part',55854,'Reentered','1983-10-09 00:00:00','1983-10-01 00:00:00',92.15,341,419,73.02,'Intermediate' from dual union all
select 14384,'014384','1983-097F','KDU','KDU',55854,'Reentered','1984-11-04 00:00:00','1983-10-20 00:00:00',92.16,353,408,72.84,'Intermediate' from dual union all
select 14385,'014385','1983-097G','KDU part','KDU part',55854,'Reentered','1983-11-20 00:00:00','1983-10-21 00:00:00',91.73,338,381,72.84,'Intermediate' from dual union all
select 14386,'014386','1982-033AG','Garbage bag','',54342,'Reentered','1983-11-16 00:00:00','1983-10-04 00:00:00',91.35,331,350,51.62,'Intermediate' from dual union all
select 14387,'014387','1982-033AH','Garbage bag','',54342,'Reentered','1984-02-19 00:00:00','1983-10-06 00:00:00',91.27,329,345,51.63,'Intermediate' from dual union all
select 14388,'014388','1982-033AJ','Garbage bag','',54342,'Reentered','1983-11-01 00:00:00','1983-10-07 00:00:00',91.1,320,337,51.63,'Intermediate' from dual union all
select 14389,'014389','1983-100D','SOZ','SOZ',55857,'Reentered','1984-06-16 00:00:00','1983-10-30 00:00:00',629.49,291,35614,47.34,'GEO Transfer' from dual union all
select 1439,'001439','1965-020AZ','S3 large debris','RCS 0.23',11770,'Reentered','1979-01-16 00:00:00','1965-07-05 00:00:00',108.48,453,1860,55.82,'Intermediate' from dual union all
select 14390,'014390','1983-100E','SOZ','SOZ',55857,'Reentered','1984-04-27 00:00:00','1983-10-31 00:00:00',628.03,240,35590,47.32,'GEO Transfer' from dual union all
select 14391,'014391','1983-097H','KDU part','KDU part',55854,'Reentered','1984-01-04 00:00:00','1983-10-17 00:00:00',94.03,388,556,72.84,'Intermediate' from dual union all
select 14392,'014392','1982-033AK','Garbage bag','',54342,'Reentered','1984-03-15 00:00:00','1983-10-06 00:00:00',91.38,331,354,51.63,'Intermediate' from dual union all
select 14393,'014393','1982-033AL','Garbage bag','',54342,'Reentered','1983-10-29 00:00:00','1983-10-04 00:00:00',91.13,300,360,51.65,'Intermediate' from dual union all
select 14394,'014394','1983-100F','Blok DM','11S86 No. 50L',55857,'In Earth orbit','','1983-11-13 00:00:00',1430.24,35563,35781,.77,'Drift GEO' from dual union all
select 14395,'014395','1983-102A','Kosmos-1502','Taifun-1B',55859,'Reentered','1985-08-29 00:00:00','1983-11-04 00:00:00',92.26,362,408,65.84,'Intermediate' from dual union all
select 14396,'014396','1983-102B','S3M','S3M',55859,'Reentered','1984-06-01 00:00:00','1983-11-04 00:00:00',92.11,347,409,65.84,'Intermediate' from dual union all
select 14397,'014397','1983-033F','deb Rohini RS-D-2','',55790,'Reentered','1984-02-04 00:00:00','1983-08-24 00:00:00',95.67,375,727,46.59,'Intermediate' from dual union all
select 14398,'014398','1982-051C','deb Kosmos (-1371) [UNID]','Strela-2M debris',54360,'In Earth orbit','','1983-10-11 00:00:00',101.07,806,813,74.06,'Intermediate' from dual union all
select 14399,'014399','1981-053HY','deb Kosmos-1275','',52576,'Reentered','1992-02-03 00:00:00','1992-02-03 00:00:00',88.78,201,227,83.24,'Intermediate' from dual union all
select 144,'000144','61 Omicron 29','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-06 00:00:00',104.51,936,1006,66.48,'Intermediate' from dual union all
select 1440,'001440','1965-051C','Tiros 10 despin weight','Despin weight',11801,'Reentered','1995-02-14 00:00:00','1966-01-14 00:00:00',99.31,614,838,98.48,'Sun-Synch' from dual union all
select 14400,'014400','1981-053HZ','deb Kosmos-1275','',52576,'Reentered','1990-03-14 00:00:00','1990-03-13 00:00:00',90.43,279,313,83.22,'Intermediate' from dual union all
select 14401,'014401','1983-103A','Kosmos-1503','Strela-2M',55860,'In Earth orbit','','1983-11-13 00:00:00',100.83,787,809,74.05,'Intermediate' from dual union all
select 14402,'014402','1983-103B','S3M','S3M',55860,'In Earth orbit','','1991-03-28 00:00:00',100.58,766,806,74.05,'Intermediate' from dual union all
select 14403,'014403','1983-104A','Kosmos-1504','Yantar''-4K2 No. 3',55861,'Landed','1983-12-06 00:00:00','1983-10-21 00:00:00',89.11,164,297,64.88,'Intermediate' from dual union all
select 14404,'014404','1983-104B','11S510','11S510',55861,'Reentered','1983-10-17 00:00:00','1983-10-16 00:00:00',88.24,155,220,64.88,'Intermediate' from dual union all
select 14405,'014405','1981-053JA','deb Kosmos-1275','',52576,'In Earth orbit','','1983-10-25 00:00:00',105.76,966,1094,83.27,'Intermediate' from dual union all
select 14406,'014406','1981-053JB','deb Kosmos-1275','',52576,'In Earth orbit','','1983-10-19 00:00:00',104.95,961,1024,82.86,'Intermediate' from dual union all
select 14407,'014407','1981-053JC','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-29 00:00:00',104.02,901,996,82.78,'Intermediate' from dual;
insert into SATELLITE
select 14408,'014408','1981-053JD','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',106.12,949,1145,82.97,'Intermediate' from dual union all
select 14409,'014409','1981-053JE','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',106.36,955,1160,83.04,'Intermediate' from dual union all
select 1441,'001441','1965-053A','Kosmos-71','Strela-1',11803,'Reentered','1970-08-11 00:00:00','1965-08-29 00:00:00',95.25,517,544,56.05,'Intermediate' from dual union all
select 14410,'014410','1968-091CL','deb Kosmos-249','',16818,'In Earth orbit','','1993-07-31 00:00:00',109.69,706,1719,62.24,'Intermediate' from dual union all
select 14411,'014411','1968-091CM','deb Kosmos-249','',16818,'Reentered','2000-06-11 00:00:00','1983-10-18 00:00:00',109.33,710,1682,62.54,'Intermediate' from dual union all
select 14412,'014412','1968-091CN','deb Kosmos-249','',16818,'In Earth orbit','','1993-08-06 00:00:00',113.72,750,2045,62.29,'Intermediate' from dual union all
select 14413,'014413','1968-091CP','deb Kosmos-249','',16818,'Reentered','1988-08-24 00:00:00','1988-08-22 00:00:00',90.4,283,306,62.22,'Intermediate' from dual union all
select 14414,'014414','1968-091CQ','deb Kosmos-249','',16818,'Reentered','1993-10-15 00:00:00','1992-10-12 00:00:00',96.03,462,674,62.26,'Intermediate' from dual union all
select 14415,'014415','1968-091CR','deb Kosmos-249','',16818,'Reentered','1986-04-12 00:00:00','1983-09-26 00:00:00',106.25,516,1589,62.33,'Intermediate' from dual union all
select 14416,'014416','1968-091CS','deb Kosmos-249  [*]','',16818,'Reentered','2004-04-12 00:00:00','1993-07-03 00:00:00',107.31,857,1348,68.22,'Intermediate' from dual union all
select 14417,'014417','1968-091CT','deb Kosmos-249  [*]','',16818,'In Earth orbit','','1993-08-06 00:00:00',106.47,510,1617,62.29,'Intermediate' from dual union all
select 14418,'014418','1968-097DZ','deb Kosmos-252','',16824,'Reentered','1988-09-30 00:00:00','1988-09-26 00:00:00',91.46,318,375,62.13,'Intermediate' from dual union all
select 14419,'014419','1979-062F','SOZ','SOZ',47546,'Reentered','1980-03-10 00:00:00','1979-07-06 00:00:00',630.55,160,35800,46.6,'GEO Transfer' from dual union all
select 1442,'001442','1965-053B','Kosmos-72','Strela-1',11803,'Reentered','1979-08-24 00:00:00','1965-08-17 00:00:00',95.9,541,584,56.06,'Intermediate' from dual union all
select 14420,'014420','1968-097EB','deb Kosmos-252','',16824,'In Earth orbit','','1993-08-01 00:00:00',104.57,543,1405,62.29,'Intermediate' from dual union all
select 14421,'014421','1983-105A','INTELSAT 507','INTELSAT V F7',55862,'In Earth orbit','','1984-01-04 00:00:00',1436.11,35672,35902,.24,'Stationary' from dual union all
select 14422,'014422','1983-106A','Progress-18','Progress 7K-TG No. 118',55863,'Deorbited','1983-11-16 00:00:00','1983-10-27 00:00:00',91.25,328,344,51.63,'Intermediate' from dual union all
select 14423,'014423','1983-105B','Ariane H8 L7','Ariane H8 S/N T7',55862,'Reentered','1987-05-04 00:00:00','1983-11-23 00:00:00',504.08,133,29073,8.43,'Highly Ellip' from dual union all
select 14424,'014424','1983-106B','11S510','11S510',55863,'Reentered','1983-10-22 00:00:00','1983-10-21 00:00:00',88.55,178,228,51.61,'Intermediate' from dual union all
select 14425,'014425','1983-107A','Kosmos-1505','Zenit-6',55864,'Landed','1983-11-04 00:00:00','1983-10-23 00:00:00',92.25,356,413,72.88,'Intermediate' from dual union all
select 14426,'014426','1983-107B','11S510','11S510',55864,'Reentered','1983-10-30 00:00:00','1983-10-23 00:00:00',89.66,193,323,72.88,'Intermediate' from dual union all
select 14427,'014427','1964-006E','deb Elektron-1','',10292,'Reentered','1990-01-03 00:00:00','1984-02-29 00:00:00',142.82,397,4949,60.8,'Medium' from dual union all
select 14428,'014428','1964-006F','deb Elektron-1','',10292,'Reentered','1989-02-02 00:00:00','1984-02-29 00:00:00',142.89,403,4948,60.8,'Medium' from dual union all
select 14429,'014429','1968-091CU','deb Kosmos-249','',16818,'Reentered','1989-12-12 00:00:00','1989-12-11 00:00:00',90.33,268,314,62.03,'Intermediate' from dual union all
select 1443,'001443','1965-053C','Kosmos-73','Strela-1',11803,'Reentered','1974-03-20 00:00:00','1965-08-29 00:00:00',95.57,538,555,56.06,'Intermediate' from dual union all
select 14430,'014430','1968-091CV','deb Kosmos-249','',16818,'In Earth orbit','','1993-08-07 00:00:00',113.26,759,1994,62.24,'Intermediate' from dual union all
select 14431,'014431','1980-089BP','deb Kosmos-1220','',50299,'Reentered','1989-03-18 00:00:00','1989-03-16 00:00:00',90.64,302,310,64.95,'Intermediate' from dual union all
select 14432,'014432','1980-089BQ','deb Kosmos-1220','',50299,'Reentered','1989-03-02 00:00:00','1989-03-02 00:00:00',88.54,194,211,64.95,'Intermediate' from dual union all
select 14433,'014433','1980-089BR','deb Kosmos-1220','',50299,'Reentered','1984-06-27 00:00:00','1985-11-08 00:00:00',89.64,244,270,64.91,'Intermediate' from dual union all
select 14434,'014434','1982-115AG','deb Blok-L','',54424,'Reentered','1985-11-16 00:00:00','1985-11-16 00:00:00',86.38,93,96,62.8,'Intermediate' from dual union all
select 14435,'014435','1982-038X','deb Kosmos-1355','',54347,'Reentered','1983-11-12 00:00:00','1983-11-10 00:00:00',88.16,177,189,64.99,'Intermediate' from dual union all
select 14436,'014436','1982-038Y','deb Kosmos-1355','',54347,'Reentered','1983-12-23 00:00:00','1983-12-21 00:00:00',88.39,190,200,64.96,'Intermediate' from dual union all
select 14437,'014437','1981-028BS','deb Kosmos-1260','',52551,'Reentered','1989-01-24 00:00:00','1989-01-24 00:00:00',89.27,228,249,64.98,'Intermediate' from dual union all
select 14438,'014438','1981-028BT','deb Kosmos-1260','',52551,'Reentered','1988-10-12 00:00:00','1988-10-12 00:00:00',90.05,261,293,65.09,'Intermediate' from dual union all
select 14439,'014439','1981-053JF','deb Kosmos-1275','',52576,'Reentered','1993-01-14 00:00:00','1993-01-13 00:00:00',92.18,349,414,82.36,'Intermediate' from dual union all
select 1444,'001444','1965-053D','Kosmos-74','Strela-1',11803,'Reentered','1979-12-13 00:00:00','1965-08-29 00:00:00',96.22,542,612,56.04,'Intermediate' from dual union all
select 14440,'014440','1981-053JG','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',106.38,965,1153,82.92,'Intermediate' from dual union all
select 14441,'014441','1981-053JH','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',104.35,924,1003,83.13,'Intermediate' from dual union all
select 14442,'014442','1977-121BE','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-04 00:00:00',106.4,1057,1063,65.39,'Intermediate' from dual union all
select 14443,'014443','1977-121BF','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-01 00:00:00',101.81,692,997,65.63,'Intermediate' from dual union all
select 14444,'014444','1977-121BG','deb Kosmos-970','',42249,'In Earth orbit','','1983-10-25 00:00:00',106.52,1010,1121,65.8,'Intermediate' from dual union all
select 14445,'014445','1977-121BH','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-05 00:00:00',104.59,866,1085,66.48,'Intermediate' from dual union all
select 14446,'014446','1971-015CQ','Unknown deb                       ','',24833,'In Earth orbit','','1983-10-25 00:00:00',110.81,570,1958,65.62,'Intermediate' from dual union all
select 14447,'014447','1971-015CR','deb Kosmos-397','',24833,'Reentered','1999-12-26 00:00:00','1983-10-25 00:00:00',106.96,564,1608,65.92,'Intermediate' from dual union all
select 14448,'014448','1971-015CS','deb Kosmos-397','',24833,'Reentered','1999-07-22 00:00:00','1993-08-04 00:00:00',99.53,511,962,65.96,'Intermediate' from dual union all
select 14449,'014449','1980-089BS','deb Kosmos-1220','',50299,'Reentered','1989-07-03 00:00:00','1989-05-22 00:00:00',92.49,377,417,65.55,'Intermediate' from dual union all
select 1445,'001445','1965-053E','Kosmos-75','Strela-1',11803,'Reentered','1979-09-28 00:00:00','1965-10-11 00:00:00',96.5,546,636,56.03,'Intermediate' from dual union all
select 14450,'014450','1983-108A','Kosmos-1506','Tsikada',55865,'In Earth orbit','','1983-12-26 00:00:00',104.73,949,1015,82.93,'Intermediate' from dual union all
select 14451,'014451','1983-108B','S3M','S3M',55865,'In Earth orbit','','1990-03-31 00:00:00',104.59,949,1001,82.93,'Intermediate' from dual union all
select 14452,'014452','1983-109A','Meteor-2','Meteor-2 No. 10',55866,'In Earth orbit','','1983-11-29 00:00:00',101.28,751,888,81.16,'Intermediate' from dual union all
select 14453,'014453','1983-109B','8A92ME','8A92ME',55866,'In Earth orbit','','1990-03-30 00:00:00',101.25,740,895,81.17,'Intermediate' from dual union all
select 14454,'014454','1983-109C','deb Meteor-2','',55866,'In Earth orbit','','1983-12-04 00:00:00',101.23,745,889,81.16,'Intermediate' from dual union all
select 14455,'014455','1983-110A','Kosmos-1507','US-PM1',55867,'Reentered','1987-08-19 00:00:00','1983-11-28 00:00:00',93.28,429,441,65.06,'Intermediate' from dual union all
select 14456,'014456','1983-110B','11S692','11S692',55867,'Reentered','1983-10-29 00:00:00','1983-10-29 00:00:00',89.16,110,355,65.05,'Intermediate' from dual union all
select 14457,'014457','1978-026FX','deb Delta 139','',45221,'Reentered','1991-12-26 00:00:00','1991-12-26 00:00:00',90.29,284,293,98.14,'Sun-Synch' from dual union all
select 14458,'014458','1978-026FY','deb Delta 139','',45221,'In Earth orbit','','1993-08-04 00:00:00',102.75,841,937,99.15,'Sun-Synch' from dual union all
select 14459,'014459','1978-026FZ','deb Delta 139','',45221,'Reentered','1990-10-08 00:00:00','1990-10-07 00:00:00',90.96,312,332,98.51,'Sun-Synch' from dual union all
select 1446,'001446','1965-054D','UR-500 GO part','Fairing',11804,'Reentered','1965-07-31 00:00:00','1965-07-16 00:00:00',94.32,180,792,63.4,'Intermediate' from dual union all
select 14460,'014460','1978-026GA','deb Delta 139','',45221,'In Earth orbit','','1993-08-01 00:00:00',106.39,862,1257,98.81,'Sun-Synch' from dual union all
select 14461,'014461','1978-026GB','deb Delta 139','',45221,'In Earth orbit','','1983-11-01 00:00:00',103.49,825,1023,98.76,'Sun-Synch' from dual union all
select 14462,'014462','1978-026GC','deb Delta 139','',45221,'Reentered','1988-05-05 00:00:00','1988-05-03 00:00:00',92.07,363,390,98.06,'Sun-Synch' from dual union all
select 14463,'014463','1978-026GD','deb Delta 139','',45221,'In Earth orbit','','1993-08-08 00:00:00',103.12,798,1014,98.99,'Sun-Synch' from dual union all
select 14464,'014464','1978-026GE','deb Delta 139','',45221,'In Earth orbit','','1993-07-30 00:00:00',103.57,881,974,99.03,'Sun-Synch' from dual union all
select 14465,'014465','1978-026GF','deb Delta 139','',45221,'In Earth orbit','','1993-08-07 00:00:00',104.1,839,1065,98.86,'Sun-Synch' from dual union all
select 14466,'014466','1970-025NS','deb Agena D','',21502,'In Earth orbit','','1993-08-01 00:00:00',102.85,816,971,100,'Sun-Synch' from dual union all
select 14467,'014467','1970-025NT','deb Agena D','',21502,'In Earth orbit','','1993-08-01 00:00:00',106.08,1043,1047,99.95,'Sun-Synch' from dual union all
select 14468,'014468','1970-025NU','deb Agena D','',21502,'Reentered','1992-01-31 00:00:00','1992-01-30 00:00:00',89.07,209,247,99.94,'Retrograde' from dual union all
select 14469,'014469','1970-025NV','deb Agena D','',21502,'In Earth orbit','','1993-08-07 00:00:00',110.45,1038,1456,101.68,'Sun-Synch' from dual union all
select 1447,'001447','1965-055A','OPS 8411','Project 770 Group 1-A',11805,'Reentered Attached','1968-12-18 00:00:00','1965-08-23 00:00:00',94.42,469,512,70.19,'Intermediate' from dual union all
select 14470,'014470','1973-086GP','deb Delta 98','[Ex UNID 095, now ID]',30688,'In Earth orbit','','1993-08-03 00:00:00',112.99,1255,1473,101.77,'Sun-Synch' from dual union all
select 14471,'014471','1976-077EN','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',117.73,1476,1682,102.48,'Sun-Synch' from dual union all
select 14472,'014472','1976-077EP','deb Delta 126','',39029,'In Earth orbit','','1993-08-02 00:00:00',118.41,1485,1733,102.7,'Sun-Synch' from dual union all
select 14473,'014473','1973-086GQ','deb Delta 98','',30688,'In Earth orbit','','1993-08-08 00:00:00',118.26,1402,1804,102.12,'Sun-Synch' from dual union all
select 14474,'014474','1974-089EQ','deb Delta 104','',33692,'In Earth orbit','','1983-11-01 00:00:00',120.04,1452,1914,100.77,'Medium' from dual union all
select 14475,'014475','1974-089ER','deb Delta 104','',33692,'In Earth orbit','','1983-11-01 00:00:00',115.37,1380,1564,101.05,'Sun-Synch' from dual union all
select 14476,'014476','1972-058JD','deb Delta 89','',27990,'Reentered','1991-04-23 00:00:00','1991-03-27 00:00:00',93.13,417,439,98.61,'Sun-Synch' from dual union all
select 14477,'014477','1983-022C','NOAA 8 solar array band','',55779,'Reentered','1991-05-21 00:00:00','1983-08-09 00:00:00',101.21,810,822,98.75,'Sun-Synch' from dual union all
select 14478,'014478','1970-025NW','deb Agena D','',21502,'In Earth orbit','','1993-08-03 00:00:00',119.67,1095,2238,100.71,'Sun-Synch' from dual union all
select 14479,'014479','1970-025NX','deb Agena D','',21502,'Reentered','1990-01-04 00:00:00','1990-01-01 00:00:00',92.03,370,379,99.8,'Sun-Synch' from dual union all
select 1448,'001448','1965-053F','S3','S3',11803,'Reentered','1981-11-21 00:00:00','1965-08-27 00:00:00',96.57,545,643,56.08,'Intermediate' from dual union all
select 14480,'014480','1973-086GR','deb Delta 98','',30688,'In Earth orbit','','1993-08-07 00:00:00',113.84,1299,1507,102.76,'Sun-Synch' from dual union all
select 14481,'014481','1983-107C','KDU','KDU',55864,'Reentered','1984-12-09 00:00:00','1983-11-20 00:00:00',92.17,356,405,72.88,'Intermediate' from dual union all
select 14482,'014482','1983-107D','Blenda','Sunshade',55864,'Reentered','1984-01-03 00:00:00','1983-11-24 00:00:00',91.8,342,383,72.89,'Intermediate' from dual union all
select 14483,'014483','1983-111A','Kosmos-1508','Vektor',55868,'In Earth orbit','','1983-12-15 00:00:00',108.98,397,1963,82.93,'Intermediate' from dual union all
select 14484,'014484','1983-111B','S3M','S3M',55868,'Reentered','2015-07-25 00:00:00','1990-07-05 00:00:00',106.71,375,1774,82.92,'Intermediate' from dual union all
select 14485,'014485','1982-033AM','Garbage bag','',54342,'Reentered','1983-11-17 00:00:00','1983-11-13 00:00:00',91.01,316,333,51.63,'Intermediate' from dual union all
select 14486,'014486','1982-033AN','Garbage bag','',54342,'Reentered','1983-11-18 00:00:00','1983-11-13 00:00:00',91.01,318,331,51.63,'Intermediate' from dual union all
select 14487,'014487','1982-033AP','Garbage bag','',54342,'Reentered','1983-11-17 00:00:00','1983-11-13 00:00:00',90.95,317,325,51.63,'Intermediate' from dual union all
select 14488,'014488','1983-106C','Model VLF part','Model VLF',55863,'Reentered','1983-11-20 00:00:00','1983-11-16 00:00:00',90.71,308,311,51.62,'Intermediate' from dual union all
select 14489,'014489','1983-106D','Model VLF part','Model VLF',55863,'Reentered','1983-11-22 00:00:00','1983-11-16 00:00:00',91.01,317,331,51.62,'Intermediate' from dual union all
select 1449,'001449','1965-053G','deb Blok Kosmos','',11803,'Reentered','1967-05-03 00:00:00','1965-08-21 00:00:00',95.15,510,542,56.05,'Intermediate' from dual union all
select 14490,'014490','1983-112A','Kosmos-1509','Zenit-6',55869,'Landed','1983-12-01 00:00:00','1983-11-19 00:00:00',89.65,225,290,72.86,'Intermediate' from dual union all
select 14491,'014491','1977-065FG','deb Delta 132','',42193,'Reentered','2005-08-26 00:00:00','1993-08-05 00:00:00',102.8,501,1282,29.11,'Intermediate' from dual union all
select 14492,'014492','1977-065FH','deb Delta 132','',42193,'Reentered','1999-05-20 00:00:00','1983-11-17 00:00:00',109.7,570,1856,28.8,'Intermediate' from dual union all
select 14493,'014493','1977-065FJ','deb Delta 132','',42193,'In Earth orbit','','1993-08-04 00:00:00',106.99,519,1656,28.95,'Intermediate' from dual union all
select 14494,'014494','1977-065FK','deb Delta 132','',42193,'Reentered','1988-07-09 00:00:00','1988-07-09 00:00:00',89.7,243,277,29.1,'Intermediate' from dual union all
select 14495,'014495','1977-065FL','deb Delta 132','',42193,'Reentered','1989-06-10 00:00:00','1983-12-01 00:00:00',100.44,444,1115,29.1,'Intermediate' from dual union all
select 14496,'014496','1977-065FM','deb Delta 132','',42193,'In Earth orbit','','1983-11-16 00:00:00',111.69,699,1910,29.2,'Intermediate' from dual union all
select 14497,'014497','1977-065FN','deb Delta 132','',42193,'In Earth orbit','','1993-08-08 00:00:00',120.1,974,2397,29.3,'Medium' from dual union all
select 14498,'014498','1983-112B','11S510','11S510',55869,'Reentered','1983-11-22 00:00:00','1983-11-19 00:00:00',88.87,186,251,72.85,'Intermediate' from dual;
insert into SATELLITE
select 14499,'014499','61 Omicron 265','deb r/b Ablestar','',7273,'Reentered','1991-03-20 00:00:00','1991-03-19 00:00:00',91.59,340,366,66.17,'Intermediate' from dual union all
select 145,'000145','61 Omicron 30','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-16 00:00:00',103.82,882,997,66.92,'Intermediate' from dual union all
select 1450,'001450','1965-054C','UR-500 GO part','Fairing',11804,'Reentered','1965-07-24 00:00:00','1965-07-17 00:00:00',91.96,179,563,63.42,'Intermediate' from dual union all
select 14500,'014500','61 Omicron 266','deb r/b Ablestar','[UNID096] RCS 0.02',7273,'Reentered','1989-05-06 00:00:00','1989-05-06 00:00:00',89.64,249,265,66.64,'Intermediate' from dual union all
select 14501,'014501','1976-126BN','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-03 00:00:00',107.4,526,1687,65.78,'Intermediate' from dual union all
select 14502,'014502','1976-126BP','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-08 00:00:00',110.65,559,1955,65.82,'Intermediate' from dual union all
select 14503,'014503','1976-126BQ','deb Kosmos-886','',39078,'In Earth orbit','','1993-07-29 00:00:00',117.57,748,2395,65.85,'Intermediate' from dual union all
select 14504,'014504','1981-053JJ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',104,918,977,83.2,'Intermediate' from dual union all
select 14505,'014505','1967-014J','deb Diademe 2','[UNID097] RCS 0.02',15269,'Reentered','2002-01-19 00:00:00','1983-11-17 00:00:00',110.56,581,1924,38.84,'Intermediate' from dual union all
select 14506,'014506','1983-113A','DMSP F-7','DMSP Block 5D-2 S-7',55870,'In Earth orbit','','1983-12-26 00:00:00',101.34,813,832,98.74,'Sun-Synch' from dual union all
select 14507,'014507','1982-033AQ','Garbage bag','',54342,'Reentered','1983-12-24 00:00:00','1983-11-20 00:00:00',91.02,320,330,51.62,'Intermediate' from dual union all
select 14508,'014508','1982-033AR','Garbage bag','',54342,'Reentered','1983-12-04 00:00:00','1983-11-20 00:00:00',90.99,316,331,51.63,'Intermediate' from dual union all
select 14509,'014509','1976-077EQ','deb Delta 126','',39029,'In Earth orbit','','1993-07-25 00:00:00',118.85,1503,1756,102.75,'Sun-Synch' from dual union all
select 1451,'001451','1965-054B','8S811','8S811',11804,'Reentered','1965-09-18 00:00:00','1965-07-17 00:00:00',92.4,168,617,63.46,'Intermediate' from dual union all
select 14510,'014510','1976-077ER','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',120.08,1504,1865,102.37,'Medium' from dual union all
select 14511,'014511','1977-121BJ','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-07 00:00:00',109.11,1083,1289,64.68,'Intermediate' from dual union all
select 14512,'014512','1977-012K','deb Tansei 3','',42140,'In Earth orbit','','1983-11-19 00:00:00',133.81,794,3781,65.75,'Medium' from dual union all
select 14513,'014513','1983-087H','KDU part','KDU part',55844,'Reentered','1983-12-15 00:00:00','1983-11-22 00:00:00',92.29,344,430,72.84,'Intermediate' from dual union all
select 14514,'014514','1976-039D','Star 37D yo weight','',38991,'Reentered','1993-12-08 00:00:00','1992-09-12 00:00:00',101.25,269,1367,109.95,'Retrograde' from dual union all
select 14515,'014515','1965-027M','deb SNAPSHOT','',11777,'In Earth orbit','','1993-08-06 00:00:00',111.31,1265,1309,90.27,'Polar' from dual union all
select 14516,'014516','1983-114A','Molniya-1','Molniya-1 No. 48',55871,'Reentered','2001-10-30 00:00:00','1983-12-23 00:00:00',717.66,484,39865,62.85,'Molniya' from dual union all
select 14517,'014517','1983-114B','11S510','11S510',55871,'Reentered','1983-12-15 00:00:00','1983-11-30 00:00:00',90.69,202,415,62.82,'Intermediate' from dual union all
select 14518,'014518','1983-114C','BOZ','BOZ',55871,'Reentered','1983-12-10 00:00:00','1983-11-30 00:00:00',90.36,186,399,62.82,'Intermediate' from dual union all
select 14519,'014519','1983-062C','Soyuz T-9 BO','Bitovoy otsek',55819,'Reentered','1984-03-17 00:00:00','1983-11-23 00:00:00',91.03,317,334,51.63,'Intermediate' from dual union all
select 1452,'001452','1965-055B','Ferret sensor cover','',11805,'Reentered','1966-07-05 00:00:00','1965-08-20 00:00:00',94.29,463,506,70.16,'Intermediate' from dual union all
select 14520,'014520','1983-114D','Blok-ML','Blok-ML No. 71637-454',55871,'Reentered','2001-01-24 00:00:00','1983-12-26 00:00:00',699.21,487,38948,62.79,'Molniya' from dual union all
select 14521,'014521','1983-115A','Kosmos-1510','Musson No. 14',55872,'In Earth orbit','','1983-12-26 00:00:00',116.02,1480,1524,73.62,'Intermediate' from dual union all
select 14522,'014522','1983-115B','S5M','S5M',55872,'In Earth orbit','','1984-03-30 00:00:00',115.98,1480,1520,73.62,'Intermediate' from dual union all
select 14523,'014523','1983-116A','Columbia (STS-9)','OV-102',55873,'Landed','1983-12-08 00:00:00','1983-11-30 00:00:00',89.42,241,251,57.02,'Intermediate' from dual union all
select 14524,'014524','1983-089C','PAM-D','Star 48  S/N 10025-3',55846,'In Earth orbit','','1983-11-28 00:00:00',658.99,214,37200,23.91,'GEO Transfer' from dual union all
select 14525,'014525','1983-051C','deb Delta','',55808,'Reentered','1986-04-07 00:00:00','1983-09-26 00:00:00',102.45,326,1424,72.48,'Intermediate' from dual union all
select 14526,'014526','1983-051D','deb Delta','',55808,'Reentered','1989-03-06 00:00:00','1983-07-04 00:00:00',103.76,329,1543,72.5,'Intermediate' from dual union all
select 14527,'014527','1970-091AS','deb Kosmos-375','',21568,'Reentered','1990-08-29 00:00:00','1990-08-29 00:00:00',89.41,235,256,62.9,'Intermediate' from dual union all
select 14528,'014528','1963-047Q','Centaur AC-2 blanket part','[UNID098] RCS 0.06',9278,'Reentered','2004-04-06 00:00:00','1993-08-03 00:00:00',105.77,651,1410,29.57,'Intermediate' from dual union all
select 14529,'014529','1963-047R','Centaur AC-2 blanket part','[UNID099] RCS 0.10',9278,'Reentered','1991-10-15 00:00:00','1991-10-14 00:00:00',92.01,345,402,30.08,'Intermediate' from dual union all
select 1453,'001453','1965-056B','8K78I','8K78I',11806,'Reentered','1965-07-20 00:00:00','1965-07-18 00:00:00',88.31,147,235,64.75,'Intermediate' from dual union all
select 14530,'014530','1983-117A','Kosmos-1511','Oktan No. 249',55874,'Landed','1984-01-13 00:00:00','1983-12-07 00:00:00',89.67,168,349,67.14,'Intermediate' from dual union all
select 14531,'014531','1983-117B','11S510','11S510',55874,'Reentered','1983-12-05 00:00:00','1983-12-02 00:00:00',89.28,166,312,67.14,'Intermediate' from dual union all
select 14532,'014532','1983-118A','Gorizont','Gorizont No. 18L',55875,'In Earth orbit','','1983-12-31 00:00:00',1436.01,35775,35795,1.34,'Stationary' from dual union all
select 14533,'014533','1983-118B','8S812','8S812',55875,'Reentered','1983-12-02 00:00:00','1983-12-01 00:00:00',88.21,182,189,51.63,'Intermediate' from dual union all
select 14534,'014534','1983-118C','Perekhodnik','Sredniy perekhodnik',55875,'Reentered','1983-12-01 00:00:00','1983-11-30 00:00:00',88.15,177,189,51.62,'Intermediate' from dual union all
select 14535,'014535','1983-112C','KDU','KDU',55869,'Reentered','1983-12-27 00:00:00','1983-12-02 00:00:00',89.58,225,283,72.85,'Intermediate' from dual union all
select 14536,'014536','1983-112D','Blenda','Sunshade',55869,'Reentered','1983-12-05 00:00:00','1983-12-01 00:00:00',89.42,225,267,72.85,'Intermediate' from dual union all
select 14537,'014537','1983-118D','SOZ','SOZ',55875,'Reentered','1986-05-10 00:00:00','1984-01-01 00:00:00',634.17,312,35834,47.27,'GEO Transfer' from dual union all
select 14538,'014538','1983-118E','SOZ','SOZ',55875,'Reentered','1988-09-23 00:00:00','1984-01-06 00:00:00',632.5,252,35808,47.18,'GEO Transfer' from dual union all
select 14539,'014539','1983-112E','KDU part','KDU part',55869,'Reentered','1983-12-03 00:00:00','1983-12-02 00:00:00',91.34,238,443,72.83,'Intermediate' from dual union all
select 1454,'001454','1965-056A','Zond-3','3MV-4 No. 3',11806,'Deep Space','1965-07-19 00:00:00','',null,null,null,null,'' from dual union all
select 14540,'014540','1983-112F','KDU part','KDU part',55869,'Reentered','1983-12-07 00:00:00','1983-12-02 00:00:00',91.31,241,437,72.86,'Intermediate' from dual union all
select 14541,'014541','1983-112G','KDU part','KDU part',55869,'Reentered','1983-12-02 00:00:00','1983-12-02 00:00:00',88.81,201,231,72.92,'Intermediate' from dual union all
select 14542,'014542','1983-119A','Kosmos-1512','Zenit-6',55876,'Landed','1983-12-21 00:00:00','1983-12-09 00:00:00',92.25,354,416,72.86,'Intermediate' from dual union all
select 14543,'014543','1983-119B','11S510','11S510',55876,'Reentered','1983-12-17 00:00:00','1983-12-09 00:00:00',89.93,194,348,72.86,'Intermediate' from dual union all
select 14544,'014544','1983-119C','Sensor cover','Sensor cover',55876,'Reentered','1983-12-09 00:00:00','1983-12-08 00:00:00',89.53,183,320,72.87,'Intermediate' from dual union all
select 14545,'014545','1983-119D','Sensor cover','Sensor cover',55876,'Reentered','1983-12-09 00:00:00','1983-12-08 00:00:00',89.31,177,304,72.85,'Intermediate' from dual union all
select 14546,'014546','1983-120A','Kosmos-1513','Parus',55877,'In Earth orbit','','1984-01-08 00:00:00',104.87,960,1017,82.94,'Intermediate' from dual union all
select 14547,'014547','1983-120B','S3M','S3M',55877,'In Earth orbit','','1991-03-28 00:00:00',104.63,942,1013,82.93,'Intermediate' from dual union all
select 14548,'014548','1983-118F','Blok DM No. 41L','11S86 No. 41L',55875,'In Earth orbit','','1993-05-31 00:00:00',1435.63,35567,35987,7.12,'Inclined GEO' from dual union all
select 14549,'014549','1983-121A','Kosmos-1514','Bion No. 6',55878,'Landed','1983-12-19 00:00:00','1983-12-16 00:00:00',89.2,212,258,82.31,'Intermediate' from dual union all
select 1455,'001455','1965-055C','Ferret sensor cover','',11805,'Reentered','1966-07-24 00:00:00','1965-08-16 00:00:00',94.51,474,516,70.19,'Intermediate' from dual union all
select 14550,'014550','1983-121B','11S510','11S510',55878,'Reentered','1983-12-19 00:00:00','1983-12-16 00:00:00',88.9,192,249,82.31,'Intermediate' from dual union all
select 14551,'014551','1983-122A','Kosmos-1515','Tselina-D',55879,'In Earth orbit','','1984-01-14 00:00:00',97.7,635,663,82.53,'Intermediate' from dual union all
select 14552,'014552','1983-122B','S5M','S5M',55879,'In Earth orbit','','1991-03-27 00:00:00',97.49,626,652,82.52,'Intermediate' from dual union all
select 14553,'014553','1983-113B','DMSP F-7 solar array band','',55870,'Reentered','1991-09-02 00:00:00','1983-12-20 00:00:00',101.33,818,826,98.74,'Sun-Synch' from dual union all
select 14554,'014554','1983-113C','DMSP F-7 solar array band','',55870,'Reentered','1991-10-02 00:00:00','1991-09-22 00:00:00',93.41,424,459,98.57,'Sun-Synch' from dual union all
select 14555,'014555','1981-053JK','deb Kosmos-1275','',52576,'Reentered','2000-06-30 00:00:00','1984-02-20 00:00:00',103.03,856,947,83.14,'Intermediate' from dual union all
select 14556,'014556','1981-053JL','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',106.55,954,1179,83.01,'Intermediate' from dual union all
select 14557,'014557','1981-053JM','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',104.27,921,1000,82.82,'Intermediate' from dual union all
select 14558,'014558','1981-053JN','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-29 00:00:00',103.56,910,944,82.95,'Intermediate' from dual union all
select 14559,'014559','1978-026GG','deb Delta 139','',45221,'In Earth orbit','','1993-08-07 00:00:00',101.78,760,926,99.02,'Sun-Synch' from dual union all
select 1456,'001456','1965-056C','BOZ','BOZ',11806,'Reentered','1965-07-21 00:00:00','1965-07-18 00:00:00',88.31,147,235,64.75,'Intermediate' from dual union all
select 14560,'014560','1983-107E','KDU part','KDU part',55864,'Reentered','1984-02-01 00:00:00','1983-11-20 00:00:00',91.82,335,393,72.88,'Intermediate' from dual union all
select 14561,'014561','1983-107F','KDU part','KDU part',55864,'Reentered','1984-02-26 00:00:00','1983-11-20 00:00:00',94.11,385,566,72.89,'Intermediate' from dual union all
select 14562,'014562','1983-095F','KDU part','KDU part',55852,'Reentered','1983-12-27 00:00:00','1983-10-10 00:00:00',94.11,384,567,72.85,'Intermediate' from dual union all
select 14563,'014563','1977-065FP','deb Delta 132','',42193,'In Earth orbit','','1993-08-07 00:00:00',105.96,515,1564,29.16,'Intermediate' from dual union all
select 14564,'014564','1977-065FQ','deb Delta 132','',42193,'In Earth orbit','','1984-02-25 00:00:00',117.42,764,2366,28.68,'Intermediate' from dual union all
select 14565,'014565','1977-065FR','deb Delta 132','',42193,'In Earth orbit','','1993-08-06 00:00:00',117,811,2281,29.23,'Intermediate' from dual union all
select 14566,'014566','1983-011D','deb Tenma','[UNID100] RCS 0.06',55768,'Reentered','1987-08-31 00:00:00','1983-10-14 00:00:00',94.63,468,534,31.3,'Intermediate' from dual union all
select 14567,'014567','1983-011E','ASTRO-B despin weight','',55768,'Reentered','1984-10-08 00:00:00','1983-07-05 00:00:00',94.04,453,492,31.69,'Intermediate' from dual union all
select 14568,'014568','1983-001C','deb S5M','',55758,'In Earth orbit','','1984-02-22 00:00:00',104.43,945,990,82.91,'Intermediate' from dual union all
select 14569,'014569','1982-087D','Kiku-4 despin weight','',54396,'In Earth orbit','','1984-02-28 00:00:00',106.28,963,1146,44.85,'Intermediate' from dual union all
select 1457,'001457','1965-057A','OPS 5543','CORONA J-22',11807,'Reentered Attached','1965-08-18 00:00:00','1965-07-21 00:00:00',91.1,185,472,85.06,'Polar' from dual union all
select 14570,'014570','1983-123A','Molniya-3','Molniya-3 No. 35',55880,'Reentered','1993-08-18 00:00:00','1984-01-21 00:00:00',717.96,610,39754,62.85,'Molniya' from dual union all
select 14571,'014571','1983-123B','11S510','11S510',55880,'Reentered','1984-02-14 00:00:00','1983-12-28 00:00:00',92.4,212,573,62.84,'Intermediate' from dual union all
select 14572,'014572','1983-123C','BOZ','BOZ',55880,'Reentered','1984-02-04 00:00:00','1983-12-28 00:00:00',92.38,194,589,62.85,'Intermediate' from dual union all
select 14573,'014573','1983-119E','Blenda','Sunshade',55876,'Reentered','1984-02-16 00:00:00','1984-01-09 00:00:00',91.92,344,393,72.86,'Intermediate' from dual union all
select 14574,'014574','1983-119F','KDU part','KDU part',55876,'Reentered','1983-12-30 00:00:00','1983-12-22 00:00:00',93.08,391,459,72.79,'Intermediate' from dual union all
select 14575,'014575','1983-119G','KDU','KDU',55876,'Reentered','1985-04-28 00:00:00','1984-01-06 00:00:00',92.21,354,412,72.86,'Intermediate' from dual union all
select 14576,'014576','1983-119H','KDU part','KDU part',55876,'Reentered','1984-03-20 00:00:00','1984-01-10 00:00:00',91.91,337,400,72.86,'Intermediate' from dual union all
select 14577,'014577','1983-034C','Kosmos-1453 SS 1','ESO',55791,'Reentered','1984-03-25 00:00:00','1983-12-21 00:00:00',94.17,462,495,74.03,'Intermediate' from dual union all
select 14578,'014578','1983-034D','Kosmos-1453 SS 2','ESO',55791,'Reentered','1984-04-01 00:00:00','1983-12-22 00:00:00',94.32,462,510,74.03,'Intermediate' from dual union all
select 14579,'014579','1983-119J','KDU part','KDU part',55876,'Reentered','1984-07-09 00:00:00','1984-01-08 00:00:00',92.2,352,413,72.86,'Intermediate' from dual union all
select 1458,'001458','1965-058A','Vela 3A','Vela 3A',11808,'Deep Space','1972-01-01 00:00:00','1965-08-27 00:00:00',6678.91,106366,115838,35.34,'Very High' from dual union all
select 14580,'014580','1983-119K','KDU part','KDU part',55876,'Reentered','1983-12-28 00:00:00','1983-12-22 00:00:00',92.36,373,408,72.91,'Intermediate' from dual union all
select 14581,'014581','1983-119L','KDU part','KDU part',55876,'Reentered','1984-06-26 00:00:00','1984-02-02 00:00:00',92.09,338,416,72.93,'Intermediate' from dual union all
select 14582,'014582','1983-123D','Blok-ML','Blok-ML No. 71629-519',55880,'Reentered','1994-04-24 00:00:00','1984-02-21 00:00:00',732.57,598,40484,62.89,'Molniya' from dual union all
select 14583,'014583','1983-124A','Kosmos-1516','Siluet No. 3',55881,'Landed','1984-02-09 00:00:00','1984-01-03 00:00:00',89.2,203,267,64.89,'Intermediate' from dual union all
select 14584,'014584','1983-124B','11S510','11S510',55881,'Reentered','1983-12-31 00:00:00','1983-12-29 00:00:00',88.66,180,237,64.89,'Intermediate' from dual union all
select 14585,'014585','1983-125A','Kosmos-1517','BOR-4 No. 405',55882,'Landed','1983-12-27 00:00:00','1983-12-27 00:00:00',88.48,179,220,50.66,'Intermediate' from dual union all
select 14586,'014586','1983-125B','S3M','S3M',55882,'Reentered','1983-12-29 00:00:00','1983-12-28 00:00:00',88.24,170,205,50.66,'Intermediate' from dual union all
select 14587,'014587','1983-126A','Kosmos-1518','Oko',55883,'Reentered','1998-09-19 00:00:00','1984-01-28 00:00:00',718.35,615,39767,63.01,'Molniya' from dual union all
select 14588,'014588','1983-126B','11S510','11S510',55883,'Reentered','1984-02-08 00:00:00','1984-01-04 00:00:00',91.71,212,504,62.83,'Intermediate' from dual;
insert into SATELLITE
select 14589,'014589','1983-126C','BOZ','BOZ',55883,'Reentered','1984-02-04 00:00:00','1984-01-04 00:00:00',92,195,551,62.87,'Intermediate' from dual union all
select 1459,'001459','1965-058B','Vela 3B','Vela 3B',11808,'Deep Space','1969-09-01 00:00:00','1965-08-25 00:00:00',6712.67,101714,121281,34.94,'Very High' from dual union all
select 14590,'014590','1983-127A','Kosmos-1519','Uragan No. 14L',55884,'In Earth orbit','','1984-01-29 00:00:00',675.71,19103,19156,64.77,'Medium' from dual union all
select 14591,'014591','1983-127B','Kosmos-1520','Uragan No. 15L',55884,'In Earth orbit','','1984-01-28 00:00:00',675.69,19089,19169,64.77,'Medium' from dual union all
select 14592,'014592','1983-127C','Kosmos-1521','Uragan GVM',55884,'In Earth orbit','','1984-02-03 00:00:00',673.39,19008,19134,64.77,'Medium' from dual union all
select 14593,'014593','1983-127D','8S812','8S812',55884,'Reentered','1983-12-31 00:00:00','1983-12-30 00:00:00',87.94,165,179,51.62,'Intermediate' from dual union all
select 14594,'014594','1983-127E','Perekhodnik','Sredniy perekhodnik',55884,'Reentered','1983-12-29 00:00:00','1983-12-29 00:00:00',88.38,186,202,51.62,'Intermediate' from dual union all
select 14595,'014595','1983-127F','Blok DM2','11S861 No. 4L',55884,'In Earth orbit','','1992-07-07 00:00:00',673.07,18972,19154,66.37,'Medium' from dual union all
select 14596,'014596','1983-126D','Blok-2BL','Blok-2BL No. 71623-649',55883,'Reentered','1998-09-02 00:00:00','1984-02-20 00:00:00',705.43,629,39115,63.12,'Molniya' from dual union all
select 14597,'014597','1983-034E','Kosmos-1453 SS 3','ESO',55791,'Reentered','1984-06-08 00:00:00','1983-12-29 00:00:00',94.29,466,503,74.04,'Intermediate' from dual union all
select 14598,'014598','1983-034F','Kosmos-1453 SS 4','ESO',55791,'Reentered','1984-12-01 00:00:00','1983-12-30 00:00:00',94.27,464,503,74.03,'Intermediate' from dual union all
select 14599,'014599','1982-088C','deb Kosmos-1405','',54397,'Reentered','1984-01-03 00:00:00','1984-01-03 00:00:00',87.84,156,179,64.89,'Intermediate' from dual union all
select 146,'000146','61 Omicron 31','deb r/b Ablestar','',7273,'Reentered','1969-07-07 00:00:00','1961-08-09 00:00:00',99.86,527,977,66.84,'Intermediate' from dual union all
select 1460,'001460','1965-058C','ERS 17','ORS 3',11808,'Reentered','1968-07-01 00:00:00','1965-10-22 00:00:00',2595.16,566,111790,36.88,'Very High' from dual union all
select 14600,'014600','1982-088D','deb Kosmos-1405','',54397,'Reentered','1984-01-12 00:00:00','1984-01-12 00:00:00',88.31,186,196,65.04,'Intermediate' from dual union all
select 14601,'014601','1982-088E','deb Kosmos-1405','',54397,'Reentered','1984-01-03 00:00:00','1984-01-02 00:00:00',89.92,250,290,65.07,'Intermediate' from dual union all
select 14602,'014602','1982-088F','deb Kosmos-1405','',54397,'Reentered','1984-01-01 00:00:00','1983-12-29 00:00:00',91.32,226,453,65.07,'Intermediate' from dual union all
select 14603,'014603','1982-088G','deb Kosmos-1405','',54397,'Reentered','1984-01-08 00:00:00','1984-01-08 00:00:00',88.71,203,219,64.88,'Intermediate' from dual union all
select 14604,'014604','1982-088H','deb Kosmos-1405','',54397,'Reentered','1984-01-05 00:00:00','1984-01-04 00:00:00',89.03,226,228,65.04,'Intermediate' from dual union all
select 14605,'014605','1982-088J','deb Kosmos-1405','',54397,'Reentered','1984-02-17 00:00:00','1984-02-17 00:00:00',88.05,170,186,64.95,'Intermediate' from dual union all
select 14606,'014606','1982-088K','deb Kosmos-1405','',54397,'Reentered','1984-01-03 00:00:00','1984-01-03 00:00:00',87.68,154,166,64.97,'Intermediate' from dual union all
select 14607,'014607','1983-127G','SOZ','SOZ',55884,'In Earth orbit','','1984-01-28 00:00:00',338.94,399,19051,52.07,'Highly Ellip' from dual union all
select 14608,'014608','1983-127H','SOZ','SOZ',55884,'In Earth orbit','','1984-02-24 00:00:00',339.27,425,19046,52,'Highly Ellip' from dual union all
select 14609,'014609','1983-113D','DMSP F-7 OLS Cover','',55870,'Reentered','1992-06-19 00:00:00','1992-06-19 00:00:00',87.82,156,177,98.52,'Sun-Synch' from dual union all
select 1461,'001461','1965-038E','deb DAPP','',11788,'Reentered','1989-03-20 00:00:00','1965-07-16 00:00:00',100.95,543,1065,98.67,'Sun-Synch' from dual union all
select 14610,'014610','1983-113E','DMSP F-7 OLS Cooler Cover','',55870,'Reentered','1999-11-27 00:00:00','1984-02-26 00:00:00',101.33,812,832,98.75,'Sun-Synch' from dual union all
select 14611,'014611','1984-001A','Kosmos-1522','Strela-1M',57055,'In Earth orbit','','1984-03-17 00:00:00',115.47,1461,1493,74.01,'Intermediate' from dual union all
select 14612,'014612','1984-001B','Kosmos-1523','Strela-1M',57055,'In Earth orbit','','1992-05-21 00:00:00',114.4,1396,1460,74,'Intermediate' from dual union all
select 14613,'014613','1984-001C','Kosmos-1524','Strela-1M',57055,'In Earth orbit','','1992-05-16 00:00:00',114.59,1412,1462,74,'Intermediate' from dual union all
select 14614,'014614','1984-001D','Kosmos-1525','Strela-1M',57055,'In Earth orbit','','1992-05-18 00:00:00',114.74,1427,1461,74,'Intermediate' from dual union all
select 14615,'014615','1984-001E','Kosmos-1526','Strela-1M',57055,'In Earth orbit','','1992-05-18 00:00:00',114.92,1443,1461,74.01,'Intermediate' from dual union all
select 14616,'014616','1984-001F','Kosmos-1527','Strela-1M',57055,'In Earth orbit','','1992-05-17 00:00:00',115.11,1459,1462,74,'Intermediate' from dual union all
select 14617,'014617','1984-001G','Kosmos-1528','Strela-1M',57055,'In Earth orbit','','1992-05-17 00:00:00',115.29,1461,1477,74.01,'Intermediate' from dual union all
select 14618,'014618','1984-001H','Kosmos-1529','Strela-1M',57055,'In Earth orbit','','1992-05-31 00:00:00',115.67,1461,1511,74.01,'Intermediate' from dual union all
select 14619,'014619','1984-001J','S3M','S3M',57055,'In Earth orbit','','1991-03-01 00:00:00',117.55,1469,1673,74,'Intermediate' from dual union all
select 1462,'001462','1965-038F','deb DAPP','',11788,'Reentered','1980-04-23 00:00:00','1965-07-23 00:00:00',98.94,550,865,98.58,'Sun-Synch' from dual union all
select 14620,'014620','1981-070J','DE-1 boom cover','Star 37 despin weight',52593,'In Earth orbit','','1993-04-06 00:00:00',394.15,525,22334,88.82,'Highly Ellip' from dual union all
select 14621,'014621','1981-070K','DE-1 boom cover','',52593,'In Earth orbit','','1993-03-25 00:00:00',396.81,568,22451,88.74,'Highly Ellip' from dual union all
select 14622,'014622','1984-002A','Kosmos-1530','Zenit-6',57056,'Landed','1984-01-25 00:00:00','1984-01-13 00:00:00',92.25,355,415,72.83,'Intermediate' from dual union all
select 14623,'014623','1984-002B','11S510','11S510',57056,'Reentered','1984-01-25 00:00:00','1984-01-13 00:00:00',89.8,191,339,72.83,'Intermediate' from dual union all
select 14624,'014624','1984-003A','Kosmos-1531','Parus',57057,'In Earth orbit','','1984-02-10 00:00:00',105.02,980,1010,82.92,'Intermediate' from dual union all
select 14625,'014625','1984-003B','S3M','S3M',57057,'In Earth orbit','','1990-03-31 00:00:00',104.85,970,1005,82.93,'Intermediate' from dual union all
select 14626,'014626','1981-053JP','deb Kosmos-1275','',52576,'Reentered','1985-05-10 00:00:00','1985-05-10 00:00:00',91.05,311,342,82.95,'Intermediate' from dual union all
select 14627,'014627','1981-053JQ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',104.3,924,999,82.79,'Intermediate' from dual union all
select 14628,'014628','1981-053JR','deb Kosmos-1275','',52576,'Reentered','1988-01-25 00:00:00','1988-08-23 00:00:00',1449.98,35944,36172,1.45,'Drift GEO' from dual union all
select 14629,'014629','1981-053JS','deb Kosmos-1275','',52576,'Reentered','2011-03-21 00:00:00','1993-08-02 00:00:00',100.61,739,836,82.98,'Intermediate' from dual union all
select 1463,'001463','1965-021F','deb DAPP','',11771,'Reentered','1979-05-18 00:00:00','1965-07-16 00:00:00',98.62,530,856,99.01,'Sun-Synch' from dual union all
select 14630,'014630','1981-053JT','deb Kosmos-1275','',52576,'Reentered','1988-11-06 00:00:00','1988-11-04 00:00:00',91.43,326,364,82.71,'Intermediate' from dual union all
select 14631,'014631','1983-075E','Kosmos-1484 part','',55832,'In Earth orbit','','1983-11-28 00:00:00',97.17,601,645,97.91,'Sun-Synch' from dual union all
select 14632,'014632','1977-002E','deb Meteor-2','',42130,'In Earth orbit','','1984-03-02 00:00:00',102.92,857,937,81.28,'Intermediate' from dual union all
select 14633,'014633','1967-014K','deb Diademe 2','',15269,'Reentered','1991-08-30 00:00:00','1991-08-27 00:00:00',91.3,300,377,39.38,'Intermediate' from dual union all
select 14634,'014634','1984-004A','Kosmos-1532','Kobal''t No. 806',57058,'Landed','1984-02-26 00:00:00','1984-01-20 00:00:00',89.36,162,324,67.13,'Intermediate' from dual union all
select 14635,'014635','1984-004B','11S510','11S510',57058,'Reentered','1984-01-18 00:00:00','1984-01-15 00:00:00',89.26,162,314,67.13,'Intermediate' from dual union all
select 14636,'014636','1982-088L','deb Kosmos-1405','',54397,'Reentered','1984-01-08 00:00:00','1983-12-21 00:00:00',91.05,316,337,64.98,'Intermediate' from dual union all
select 14637,'014637','1982-088M','deb Kosmos-1405','',54397,'Reentered','1984-01-10 00:00:00','1983-12-21 00:00:00',91.19,311,355,65,'Intermediate' from dual union all
select 14638,'014638','1982-088N','deb Kosmos-1405','',54397,'Reentered','1983-12-31 00:00:00','1983-12-21 00:00:00',91.58,325,380,65.05,'Intermediate' from dual union all
select 14639,'014639','1982-088P','deb Kosmos-1405','',54397,'Reentered','1984-01-01 00:00:00','1983-12-21 00:00:00',92.86,299,531,64.93,'Intermediate' from dual union all
select 1464,'001464','1965-059A','Kosmos-76','DS-P1-Yu No. 3',11809,'Reentered','1966-03-16 00:00:00','1965-08-26 00:00:00',92.09,255,499,48.78,'Intermediate' from dual union all
select 14640,'014640','1982-088Q','deb Kosmos-1405','',54397,'Reentered','1983-12-28 00:00:00','1983-12-21 00:00:00',91.79,273,452,64.94,'Intermediate' from dual union all
select 14641,'014641','1982-088R','deb Kosmos-1405','',54397,'Reentered','1984-01-01 00:00:00','1983-12-21 00:00:00',91.65,326,385,65.06,'Intermediate' from dual union all
select 14642,'014642','1982-088S','deb Kosmos-1405','',54397,'Reentered','1983-12-30 00:00:00','1983-12-21 00:00:00',91.67,278,434,65.11,'Intermediate' from dual union all
select 14643,'014643','1982-088T','deb Kosmos-1405','',54397,'Reentered','1984-01-04 00:00:00','1983-12-21 00:00:00',91.01,316,333,64.95,'Intermediate' from dual union all
select 14644,'014644','1982-088U','deb Kosmos-1405','',54397,'Reentered','1983-12-30 00:00:00','1983-12-21 00:00:00',90.94,294,348,64.95,'Intermediate' from dual union all
select 14645,'014645','1982-088V','deb Kosmos-1405','',54397,'Reentered','1983-12-31 00:00:00','1983-12-21 00:00:00',91.77,326,397,65.05,'Intermediate' from dual union all
select 14646,'014646','1982-088W','deb Kosmos-1405','',54397,'Reentered','1984-01-13 00:00:00','1983-12-22 00:00:00',91.46,314,379,65.07,'Intermediate' from dual union all
select 14647,'014647','1982-088X','deb Kosmos-1405','',54397,'Reentered','1984-01-06 00:00:00','1983-12-22 00:00:00',91.94,328,412,65.1,'Intermediate' from dual union all
select 14648,'014648','1982-088Y','deb Kosmos-1405','',54397,'Reentered','1984-02-21 00:00:00','1983-12-22 00:00:00',90.96,310,333,65.01,'Intermediate' from dual union all
select 14649,'014649','1982-088Z','deb Kosmos-1405','',54397,'Reentered','1984-01-04 00:00:00','1983-12-22 00:00:00',91.05,297,356,64.97,'Intermediate' from dual union all
select 1465,'001465','1965-059B','S1','S1',11809,'Reentered','1965-12-04 00:00:00','1965-08-22 00:00:00',91.91,256,480,48.78,'Intermediate' from dual union all
select 14650,'014650','1982-088AA','deb Kosmos-1405','',54397,'Reentered','1984-01-09 00:00:00','1983-12-22 00:00:00',91.47,328,366,65.04,'Intermediate' from dual union all
select 14651,'014651','1982-088AB','deb Kosmos-1405','',54397,'Reentered','1984-01-05 00:00:00','1983-12-22 00:00:00',90.8,298,330,65,'Intermediate' from dual union all
select 14652,'014652','1982-088AC','deb Kosmos-1405 antenna','RCS 15m2',54397,'Reentered','1984-02-27 00:00:00','1983-12-22 00:00:00',91.04,307,345,65.01,'Intermediate' from dual union all
select 14653,'014653','1982-088AD','deb Kosmos-1405','',54397,'Reentered','1984-01-28 00:00:00','1983-12-28 00:00:00',91.11,313,345,65.03,'Intermediate' from dual union all
select 14654,'014654','1982-088AE','deb Kosmos-1405','',54397,'Reentered','1984-01-14 00:00:00','1984-01-03 00:00:00',90.53,286,315,65.08,'Intermediate' from dual union all
select 14655,'014655','1982-088AF','deb Kosmos-1405 solar array','RCS 84m2',54397,'Reentered','1984-02-03 00:00:00','1984-02-01 00:00:00',89.22,230,242,65,'Intermediate' from dual union all
select 14656,'014656','1982-088AG','deb Kosmos-1405','',54397,'Reentered','1984-02-19 00:00:00','1983-12-28 00:00:00',91.14,320,341,64.95,'Intermediate' from dual union all
select 14657,'014657','1982-088AH','deb Kosmos-1405','',54397,'Reentered','1984-01-31 00:00:00','1984-01-29 00:00:00',88.37,191,196,64.95,'Intermediate' from dual union all
select 14658,'014658','1982-088AJ','deb Kosmos-1405','',54397,'Reentered','1984-01-21 00:00:00','1984-01-05 00:00:00',90.5,277,322,65.11,'Intermediate' from dual union all
select 14659,'014659','1984-005A','Yuri 2A','BS 2A',57059,'In Earth orbit','','1984-02-26 00:00:00',1436.06,35784,35788,.31,'Stationary' from dual union all
select 1466,'001466','1965-054A','Proton','N-4 No. 1',11804,'Reentered','1965-10-11 00:00:00','1965-08-19 00:00:00',91.53,177,523,63.46,'Intermediate' from dual union all
select 14660,'014660','1984-005B','N-II Stage 2 S/N N-12(F)','AJ10-118FJ',57059,'Reentered','1984-11-21 00:00:00','1984-02-23 00:00:00',109.22,198,2184,29.96,'Intermediate' from dual union all
select 14661,'014661','1984-002C','Blenda','Sunshade',57056,'Reentered','1984-02-04 00:00:00','1984-01-25 00:00:00',92.19,330,434,72.79,'Intermediate' from dual union all
select 14662,'014662','1984-002D','KDU part','KDU part',57056,'Reentered','1984-04-11 00:00:00','1984-02-10 00:00:00',91.92,337,400,72.83,'Intermediate' from dual union all
select 14663,'014663','1984-002E','KDU part','KDU part',57056,'Reentered','1984-03-16 00:00:00','1984-02-11 00:00:00',92.03,349,400,72.83,'Intermediate' from dual union all
select 14664,'014664','1984-002F','KDU part','KDU part',57056,'Reentered','1984-03-26 00:00:00','1984-02-10 00:00:00',94.14,381,573,72.84,'Intermediate' from dual union all
select 14665,'014665','1984-005C','Star 37E S/N 40097','Star 37E',57059,'Reentered','1992-06-30 00:00:00','1984-02-24 00:00:00',633.39,193,35913,28.33,'GEO Transfer' from dual union all
select 14666,'014666','1984-006A','Kosmos-1533','Zenit-6',57060,'Landed','1984-02-09 00:00:00','1984-01-28 00:00:00',92.16,347,414,70.37,'Intermediate' from dual union all
select 14667,'014667','1984-006B','11S510','11S510',57060,'Reentered','1984-02-13 00:00:00','1984-02-02 00:00:00',89.78,214,313,70.37,'Intermediate' from dual union all
select 14668,'014668','1984-007A','Kosmos-1534','Vektor',57061,'Reentered','1990-09-20 00:00:00','1984-02-25 00:00:00',94.43,465,517,65.84,'Intermediate' from dual union all
select 14669,'014669','1984-007B','S3M','S3M',57061,'Reentered','1989-06-30 00:00:00','1988-06-13 00:00:00',93.6,441,461,65.82,'Intermediate' from dual union all
select 1467,'001467','1965-060A','Pegasus 3','Pegasus 3',11810,'Reentered','1969-08-04 00:00:00','1965-09-03 00:00:00',95.15,516,536,28.87,'Intermediate' from dual union all
select 14670,'014670','1984-008A','Shiyan Weixing','STW-1',57062,'In Earth orbit','','1984-02-28 00:00:00',162.63,401,6581,36.08,'Medium' from dual union all
select 14671,'014671','1984-008B','CZ-3 Stage 3','CZ-3 Stage 3',57062,'Reentered','1984-02-08 00:00:00','1984-01-31 00:00:00',92.03,291,458,31.04,'Intermediate' from dual union all
select 14672,'014672','1984-008C','deb CZ-3','',57062,'Reentered','1984-01-30 00:00:00','1984-01-29 00:00:00',92.35,353,427,31.12,'Intermediate' from dual union all
select 14673,'014673','1983-119M','KDU part','KDU part',55876,'Reentered','1984-03-30 00:00:00','1984-01-11 00:00:00',94.11,388,563,72.86,'Intermediate' from dual union all
select 14674,'014674','1983-107G','KDU part','KDU part',55864,'Reentered','1984-02-01 00:00:00','1984-02-01 00:00:00',86.88,119,121,72.82,'Intermediate' from dual union all
select 14675,'014675','1984-009A','OPS 0441','VORTEX 11',57063,'In Earth orbit','','1984-04-01 00:00:00',1436.11,35784,35790,.1,'Stationary' from dual union all
select 14676,'014676','1984-009B','Titan 34D-10 Stage 2','LR91AJ11 34D-10',57063,'Reentered','1984-02-27 00:00:00','1984-01-31 00:00:00',96.36,146,1023,29.4,'Intermediate' from dual union all
select 14677,'014677','1984-009C','Transtage D-10','Transtage D-10',57063,'In Earth orbit','','1984-04-01 00:00:00',1431.67,35600,35800,2,'Drift GEO' from dual union all
select 14678,'014678','1984-008D','deb CZ-3','',57062,'Reentered','1984-02-15 00:00:00','1984-01-30 00:00:00',91.84,211,518,31,'Intermediate' from dual;
insert into SATELLITE
select 14679,'014679','1984-010A','Kosmos-1535','Parus',57064,'In Earth orbit','','1984-03-05 00:00:00',104.82,955,1017,82.96,'Intermediate' from dual union all
select 1468,'001468','1965-060B','Apollo BP-9A','Apollo BP-9A CM',11810,'Reentered','1975-11-22 00:00:00','1965-09-03 00:00:00',95.2,521,536,28.87,'Intermediate' from dual union all
select 14680,'014680','1984-010B','S3M','S3M',57064,'In Earth orbit','','1991-03-28 00:00:00',104.67,952,1005,82.95,'Intermediate' from dual union all
select 14681,'014681','1984-011A','Challenger (STS 41-B)','OV-099',57065,'Landed','1984-02-11 00:00:00','1984-02-05 00:00:00',90.79,305,322,28.47,'Intermediate' from dual union all
select 14682,'014682','1982-038Z','deb Kosmos-1355','',54347,'Reentered','1984-02-13 00:00:00','1984-02-12 00:00:00',88.74,207,218,65.03,'Intermediate' from dual union all
select 14683,'014683','1982-038AA','deb Kosmos-1355','',54347,'Reentered','1984-02-14 00:00:00','1984-02-05 00:00:00',90.53,293,308,65.04,'Intermediate' from dual union all
select 14684,'014684','1982-038AB','deb Kosmos-1355','',54347,'Reentered','1984-02-11 00:00:00','1984-02-08 00:00:00',89.37,182,305,65,'Intermediate' from dual union all
select 14685,'014685','1982-038AC','deb Kosmos-1355','',54347,'Reentered','1984-02-05 00:00:00','1984-02-03 00:00:00',89.9,258,280,64.92,'Intermediate' from dual union all
select 14686,'014686','1982-038AD','deb Kosmos-1355','',54347,'Reentered','1984-02-14 00:00:00','1984-02-13 00:00:00',88.82,210,223,65.03,'Intermediate' from dual union all
select 14687,'014687','1982-038AE','deb Kosmos-1355','',54347,'Reentered','1984-02-05 00:00:00','1984-02-04 00:00:00',89.02,167,285,65.03,'Intermediate' from dual union all
select 14688,'014688','1984-011B','Westar 6','Westar 6',57065,'Landed Attached','1984-11-16 00:00:00','1984-03-04 00:00:00',99.86,296,1208,27.71,'Intermediate' from dual union all
select 14689,'014689','1984-011C','IRT','IRT',57065,'Reentered','1984-02-11 00:00:00','1984-02-05 00:00:00',90.03,269,283,28.46,'Intermediate' from dual union all
select 1469,'001469','1965-061A','Kosmos-77','Zenit-4 No. 13',11811,'Landed','1965-08-11 00:00:00','1965-08-06 00:00:00',89.28,199,278,51.8,'Intermediate' from dual union all
select 14690,'014690','1984-012A','OPS 8737','PARCAE 7',57066,'In Earth orbit','','1990-05-23 00:00:00',107.43,1003,1213,63.44,'Intermediate' from dual union all
select 14691,'014691','1984-012B','MSD plume shield','',57066,'In Earth orbit','','1997-11-21 00:00:00',106.99,817,1357,63.41,'Intermediate' from dual union all
select 14692,'014692','1984-011D','Palapa B2','Palapa B2',57065,'Landed Attached','1984-11-16 00:00:00','1984-03-04 00:00:00',99.19,266,1173,28.19,'Intermediate' from dual union all
select 14693,'014693','1984-011E','PAM-D','Star 48  S/N 10030-3 (PB',57065,'Reentered','2000-09-22 00:00:00','1990-01-29 00:00:00',97.52,270,1010,28.15,'Intermediate' from dual union all
select 14694,'014694','1984-011F','PAM-D','Star 48  S/N 10029-3 (WU',57065,'Reentered','2014-12-23 00:00:00','1990-01-26 00:00:00',98.93,300,1115,27.68,'Intermediate' from dual union all
select 14695,'014695','1984-011G','deb IRT','',57065,'Reentered','1984-02-09 00:00:00','1984-02-06 00:00:00',90.02,269,282,28.39,'Intermediate' from dual union all
select 14696,'014696','1984-011H','deb IRT','',57065,'Reentered','1984-02-12 00:00:00','1984-02-06 00:00:00',90.17,273,293,28.52,'Intermediate' from dual union all
select 14697,'014697','1982-038AF','deb Kosmos-1355','',54347,'Reentered','1984-02-03 00:00:00','1984-02-01 00:00:00',89.95,248,296,65,'Intermediate' from dual union all
select 14698,'014698','1984-011J','deb PAM-D','',57065,'Reentered','1984-02-09 00:00:00','1984-02-08 00:00:00',92.04,272,477,28.58,'Intermediate' from dual union all
select 14699,'014699','1984-013A','Kosmos-1536','Tselina-D',57067,'In Earth orbit','','1984-03-09 00:00:00',97.71,633,666,82.54,'Intermediate' from dual union all
select 147,'000147','61 Omicron 32','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-04 00:00:00',103.6,867,991,66.59,'Intermediate' from dual union all
select 1470,'001470','1965-061B','11A57I','11A57I',11811,'Reentered','1965-08-08 00:00:00','1965-08-04 00:00:00',89.07,189,267,51.81,'Intermediate' from dual union all
select 14700,'014700','1984-013B','S5M','S5M',57067,'In Earth orbit','','1991-03-27 00:00:00',97.52,623,657,82.54,'Intermediate' from dual union all
select 14701,'014701','1984-014A','Soyuz T-10','Soyuz 7K-ST No. 15L',57068,'Landed','1984-04-11 00:00:00','1984-03-09 00:00:00',90.55,301,302,51.63,'Intermediate' from dual union all
select 14702,'014702','1984-014B','11S510','11S510',57068,'Reentered','1984-02-10 00:00:00','1984-02-09 00:00:00',88.23,187,187,51.6,'Intermediate' from dual union all
select 14703,'014703','1983-091C','Kosmos-1494 SS 1','ESO',55848,'Reentered','1984-04-07 00:00:00','1984-02-09 00:00:00',93.11,338,515,50.67,'Intermediate' from dual union all
select 14704,'014704','1983-091D','deb Kosmos-1494','Dispenser cover',55848,'Reentered','1984-05-06 00:00:00','1984-02-08 00:00:00',93.06,338,511,50.67,'Intermediate' from dual union all
select 14705,'014705','1983-091E','Kosmos-1494 SS 2','ESO',55848,'Reentered','1984-04-05 00:00:00','1984-02-09 00:00:00',92.99,336,506,50.68,'Intermediate' from dual union all
select 14706,'014706','1983-091F','Kosmos-1494 SS 3','ESO',55848,'Reentered','1984-04-09 00:00:00','1984-02-08 00:00:00',92.96,334,505,50.65,'Intermediate' from dual union all
select 14707,'014707','1983-091G','Kosmos-1494 SS 4','ESO',55848,'Reentered','1984-04-07 00:00:00','1984-02-09 00:00:00',93.12,337,518,50.67,'Intermediate' from dual union all
select 14708,'014708','1983-091H','Kosmos-1494 SS 5','ESO',55848,'Reentered','1984-04-06 00:00:00','1984-02-09 00:00:00',93.16,339,519,50.67,'Intermediate' from dual union all
select 14709,'014709','1983-091J','Kosmos-1494 SS 6','ESO',55848,'Reentered','1984-04-08 00:00:00','1984-02-09 00:00:00',93.15,338,520,50.67,'Intermediate' from dual union all
select 1471,'001471','1965-062A','OPS 5698','GAMBIT SV 971',11812,'Reentered','1965-08-07 00:00:00','1965-08-03 00:00:00',89.06,149,307,107.47,'Retrograde' from dual union all
select 14710,'014710','1983-091K','Kosmos-1494 SS 7','ESO',55848,'Reentered','1984-04-09 00:00:00','1984-02-09 00:00:00',93.13,338,518,50.68,'Intermediate' from dual union all
select 14711,'014711','1983-091L','Kosmos-1494 SS 8','ESO',55848,'Reentered','1984-04-05 00:00:00','1984-02-09 00:00:00',93.01,336,507,50.68,'Intermediate' from dual union all
select 14712,'014712','1984-006C','Blenda','Sunshade',57060,'Reentered','1984-05-05 00:00:00','1984-02-25 00:00:00',91.71,333,383,70.37,'Intermediate' from dual union all
select 14713,'014713','1984-006D','KDU','KDU',57060,'Reentered','1984-04-11 00:00:00','1984-02-25 00:00:00',92.1,346,409,70.36,'Intermediate' from dual union all
select 14714,'014714','1984-006E','KDU part','KDU part',57060,'Reentered','1984-04-11 00:00:00','1984-02-25 00:00:00',91.74,328,391,70.36,'Intermediate' from dual union all
select 14715,'014715','1965-027N','deb SNAPSHOT','',11777,'In Earth orbit','','1993-08-06 00:00:00',111.39,1271,1310,90.27,'Polar' from dual union all
select 14716,'014716','1965-027P','deb SNAPSHOT','',11777,'In Earth orbit','','1993-08-04 00:00:00',111.14,1260,1298,90.27,'Polar' from dual union all
select 14717,'014717','1965-027Q','deb SNAPSHOT','',11777,'In Earth orbit','','1993-08-07 00:00:00',111.34,1267,1310,90.27,'Polar' from dual union all
select 14718,'014718','1983-091M','Kosmos-1494 SS 9','ESO',55848,'Reentered','1984-04-06 00:00:00','1984-02-11 00:00:00',93.04,336,511,50.69,'Intermediate' from dual union all
select 14719,'014719','1983-091N','Kosmos-1494 SS 10','ESO',55848,'Reentered','1984-04-08 00:00:00','1984-02-12 00:00:00',93.1,337,516,50.67,'Intermediate' from dual union all
select 1472,'001472','1965-062B','MAGNUM','P-11 No. 4403',11812,'Reentered','1968-06-17 00:00:00','1965-09-30 00:00:00',94.7,502,507,107.35,'Retrograde' from dual union all
select 14720,'014720','1983-091P','Kosmos-1494 SS 11','ESO',55848,'Reentered','1984-04-06 00:00:00','1984-02-12 00:00:00',93.06,338,511,50.67,'Intermediate' from dual union all
select 14721,'014721','1983-091Q','Kosmos-1494 SS 12','ESO',55848,'Reentered','1984-04-04 00:00:00','1984-02-11 00:00:00',92.89,334,499,50.69,'Intermediate' from dual union all
select 14722,'014722','1984-015A','Ohzora','EXOS-C',57069,'Reentered','1989-07-19 00:00:00','1984-03-15 00:00:00',96.83,356,858,74.6,'Intermediate' from dual union all
select 14723,'014723','1984-015B','M-3A','M-3A',57069,'Reentered','1988-04-20 00:00:00','1984-03-15 00:00:00',96.85,355,860,74.64,'Intermediate' from dual union all
select 14724,'014724','1984-015C','deb Ohzora','',57069,'Reentered','1988-12-26 00:00:00','1984-03-16 00:00:00',96.93,355,868,74.59,'Intermediate' from dual union all
select 14725,'014725','1984-016A','Raduga','Gran'' No. 25L',57070,'In Earth orbit','','1984-03-16 00:00:00',1435.86,35705,35858,1.19,'Stationary' from dual union all
select 14726,'014726','1984-016B','8S812','8S812',57070,'Reentered','1984-02-17 00:00:00','1984-02-16 00:00:00',88.15,180,186,51.62,'Intermediate' from dual union all
select 14727,'014727','1984-016C','Perekhodnik','Sredniy perekhodnik',57070,'Reentered','1984-02-15 00:00:00','1984-02-15 00:00:00',87.51,142,161,51.61,'Intermediate' from dual union all
select 14728,'014728','1984-012C','JD1','SSU',57066,'In Earth orbit','','1990-05-24 00:00:00',107.44,1018,1199,63.43,'Intermediate' from dual union all
select 14729,'014729','1984-012D','JD2','SSU',57066,'In Earth orbit','','1990-05-24 00:00:00',107.44,1009,1207,63.44,'Intermediate' from dual union all
select 1473,'001473','1965-053H','deb Blok Kosmos','',11803,'Reentered','1969-03-13 00:00:00','1965-08-25 00:00:00',96.68,542,657,56.03,'Intermediate' from dual union all
select 14730,'014730','1983-034G','Kosmos-1453 SS 5','ESO',55791,'Reentered','1984-05-21 00:00:00','1984-02-15 00:00:00',94.5,461,528,73.99,'Intermediate' from dual union all
select 14731,'014731','1983-034H','Kosmos-1453 SS 6','ESO',55791,'Reentered','1984-04-10 00:00:00','1984-02-15 00:00:00',93.94,452,483,74.06,'Intermediate' from dual union all
select 14732,'014732','1983-101C','Kosmos-1501 SS 1','ESO',55858,'Reentered','1984-04-27 00:00:00','1984-02-15 00:00:00',94.25,461,504,82.91,'Intermediate' from dual union all
select 14733,'014733','1983-101D','Kosmos-1501 SS 2','ESO',55858,'Reentered','1984-04-28 00:00:00','1984-02-16 00:00:00',94.32,463,508,82.96,'Intermediate' from dual union all
select 14734,'014734','1981-108E','deb Kosmos-1317','',52631,'In Earth orbit','','1988-04-16 00:00:00',713.61,3439,36710,65.31,'GEO Transfer' from dual union all
select 14735,'014735','1981-108F','deb Kosmos-1317','',52631,'In Earth orbit','','1988-07-13 00:00:00',714.76,3726,36480,65.15,'GEO Transfer' from dual union all
select 14736,'014736','1981-108G','deb Kosmos-1317','',52631,'In Earth orbit','','1993-02-24 00:00:00',719.91,5327,35133,68.55,'GEO Transfer' from dual union all
select 14737,'014737','1984-017A','Kosmos-1537','Resurs-F1 17F41 No. 27',57071,'Landed','1984-03-01 00:00:00','1984-02-18 00:00:00',89.81,259,271,82.35,'Intermediate' from dual union all
select 14738,'014738','1984-017B','11S510','11S510',57071,'Reentered','1984-02-23 00:00:00','1984-02-18 00:00:00',89.12,201,260,82.35,'Intermediate' from dual union all
select 14739,'014739','1981-053JU','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',104.78,933,1036,83.02,'Intermediate' from dual union all
select 1474,'001474','1965-062C','Agena D 4818','Agena D',11812,'Reentered','1965-08-06 00:00:00','1965-08-03 00:00:00',89.06,149,307,107.47,'Retrograde' from dual union all
select 14740,'014740','1981-053JV','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-06 00:00:00',105.81,936,1128,83.15,'Intermediate' from dual union all
select 14741,'014741','1981-053JW','deb Kosmos-1275','',52576,'In Earth orbit','','1984-03-02 00:00:00',107.92,986,1275,82.98,'Intermediate' from dual union all
select 14742,'014742','1984-016D','SOZ','SOZ',57070,'Reentered','1987-03-10 00:00:00','1984-03-18 00:00:00',635.96,329,35909,47.28,'GEO Transfer' from dual union all
select 14743,'014743','1984-016E','SOZ','SOZ',57070,'Reentered','1986-10-27 00:00:00','1984-03-17 00:00:00',634.64,276,35894,47.23,'GEO Transfer' from dual union all
select 14744,'014744','1983-091R','Kosmos-1494 SS 13','ESO',55848,'Reentered','1984-04-04 00:00:00','1984-02-17 00:00:00',92.76,331,489,50.68,'Intermediate' from dual union all
select 14745,'014745','1983-091S','Kosmos-1494 SS 14','ESO',55848,'Reentered','1984-04-02 00:00:00','1984-02-19 00:00:00',92.71,331,484,50.68,'Intermediate' from dual union all
select 14746,'014746','1983-091T','Kosmos-1494 SS 15','ESO',55848,'Reentered','1984-04-04 00:00:00','1984-02-16 00:00:00',92.78,332,490,50.68,'Intermediate' from dual union all
select 14747,'014747','1983-091U','Kosmos-1494 SS 16','ESO',55848,'Reentered','1984-04-05 00:00:00','1984-02-19 00:00:00',92.75,332,486,50.67,'Intermediate' from dual union all
select 14748,'014748','1983-091V','Kosmos-1494 SS 17','ESO',55848,'Reentered','1984-04-10 00:00:00','1984-02-19 00:00:00',92.77,332,489,50.67,'Intermediate' from dual union all
select 14749,'014749','1983-091W','Kosmos-1494 SS 18','ESO',55848,'Reentered','1984-04-06 00:00:00','1984-02-18 00:00:00',92.78,332,490,50.67,'Intermediate' from dual union all
select 1475,'001475','1965-038G','deb DAPP','',11788,'Reentered','1988-02-19 00:00:00','1965-08-04 00:00:00',100.13,552,978,98.61,'Sun-Synch' from dual union all
select 14750,'014750','1983-091X','Kosmos-1494 SS 19','ESO',55848,'Reentered','1984-04-05 00:00:00','1984-02-17 00:00:00',92.84,332,496,50.66,'Intermediate' from dual union all
select 14751,'014751','1983-091Y','Kosmos-1494 SS 20','ESO',55848,'Reentered','1984-04-06 00:00:00','1984-02-20 00:00:00',92.77,332,489,50.66,'Intermediate' from dual union all
select 14752,'014752','1983-091Z','Kosmos-1494 SS 21','ESO',55848,'Reentered','1984-04-09 00:00:00','1984-02-18 00:00:00',92.86,333,496,50.68,'Intermediate' from dual union all
select 14753,'014753','1983-091AA','Kosmos-1494 SS 22','ESO',55848,'Reentered','1984-04-08 00:00:00','1984-02-18 00:00:00',92.86,333,496,50.68,'Intermediate' from dual union all
select 14754,'014754','1983-091AB','Kosmos-1494 SS 23','ESO',55848,'Reentered','1984-04-10 00:00:00','1984-02-19 00:00:00',92.96,335,503,50.66,'Intermediate' from dual union all
select 14755,'014755','1983-091AC','Kosmos-1494 SS 24','ESO',55848,'Reentered','1984-04-10 00:00:00','1984-02-19 00:00:00',92.95,334,504,50.66,'Intermediate' from dual union all
select 14756,'014756','1965-108Y','deb Transtage 8','',11858,'In Earth orbit','','1983-12-08 00:00:00',415.52,243,23892,26.68,'Highly Ellip' from dual union all
select 14757,'014757','1984-018A','Progress-19','Progress 7K-TG No. 120',57072,'Deorbited','1984-04-01 00:00:00','1984-03-05 00:00:00',90.6,303,304,51.63,'Intermediate' from dual union all
select 14758,'014758','1984-018B','11S510','11S510',57072,'Reentered','1984-02-23 00:00:00','1984-02-22 00:00:00',88.04,162,193,51.62,'Intermediate' from dual union all
select 14759,'014759','1984-019A','Kosmos-1538','Strela-2M',57073,'In Earth orbit','','1984-03-22 00:00:00',100.73,777,810,74.05,'Intermediate' from dual union all
select 1476,'001476','1965-020BA','deb Blok Kosmos','',11770,'Reentered','1967-09-21 00:00:00','1965-08-07 00:00:00',105.66,308,1743,56.12,'Intermediate' from dual union all
select 14760,'014760','1984-019B','S3M','S3M',57073,'In Earth orbit','','1991-03-28 00:00:00',100.57,767,805,74.03,'Intermediate' from dual union all
select 14761,'014761','1984-012E','JD1 cover','SSU cover',57066,'Reentered','1984-10-18 00:00:00','1997-11-22 00:00:00',107.06,797,1384,63.42,'Intermediate' from dual union all
select 14762,'014762','1984-006F','KDU part','KDU part',57060,'Reentered','1984-04-14 00:00:00','1984-02-27 00:00:00',93.97,370,568,70.36,'Intermediate' from dual union all
select 14763,'014763','1984-020A','Kosmos-1539','Kobal''t No. 805',57074,'Landed','1984-04-09 00:00:00','1984-03-06 00:00:00',88.94,165,280,67.15,'Intermediate' from dual union all
select 14764,'014764','1984-020B','11S510','11S510',57074,'Reentered','1984-03-03 00:00:00','1984-03-01 00:00:00',89.08,160,298,67.15,'Intermediate' from dual union all
select 14765,'014765','1984-011K','deb PAM-D','',57065,'Reentered','1984-02-28 00:00:00','1984-02-09 00:00:00',91.74,270,449,28.47,'Intermediate' from dual union all
select 14766,'014766','1984-011L','deb PAM-D','',57065,'Reentered','1984-02-28 00:00:00','1984-02-09 00:00:00',91.74,270,449,28.47,'Intermediate' from dual union all
select 14767,'014767','1984-011M','deb PAM-D','',57065,'Reentered','1984-02-28 00:00:00','1984-02-09 00:00:00',91.74,270,449,28.47,'Intermediate' from dual union all
select 14768,'014768','1984-011N','deb PAM-D','',57065,'Reentered','1984-02-28 00:00:00','1984-02-09 00:00:00',91.74,270,449,28.47,'Intermediate' from dual;
insert into SATELLITE
select 14769,'014769','1984-011P','deb PAM-D','',57065,'Reentered','1984-02-28 00:00:00','1984-02-09 00:00:00',91.74,270,449,28.47,'Intermediate' from dual union all
select 1477,'001477','1965-020BB','deb Blok Kosmos','',11770,'In Earth orbit','','1992-01-31 00:00:00',111.9,830,1798,55.49,'Intermediate' from dual union all
select 14770,'014770','1984-011Q','deb PAM-D','',57065,'Reentered','1984-02-28 00:00:00','1984-02-09 00:00:00',90.52,160,439,28.47,'Intermediate' from dual union all
select 14771,'014771','1984-011R','deb PAM-D','',57065,'Reentered','1984-02-28 00:00:00','1984-02-09 00:00:00',92.04,300,450,28.47,'Intermediate' from dual union all
select 14772,'014772','1984-011S','deb PAM-D','',57065,'Reentered','1984-02-28 00:00:00','1984-02-09 00:00:00',92.04,300,450,28.47,'Intermediate' from dual union all
select 14773,'014773','1984-011T','deb PAM-D','',57065,'Reentered','1984-02-28 00:00:00','1984-02-09 00:00:00',92.04,300,450,28.47,'Intermediate' from dual union all
select 14774,'014774','1984-017C','Blenda','Sunshade',57071,'Reentered','1984-03-02 00:00:00','1984-03-01 00:00:00',89.61,244,266,82.34,'Intermediate' from dual union all
select 14775,'014775','1984-017D','KDU part','KDU part',57071,'Reentered','1984-03-18 00:00:00','1984-03-01 00:00:00',89.65,248,267,82.34,'Intermediate' from dual union all
select 14776,'014776','1984-017E','KDU part','KDU part',57071,'Reentered','1984-03-02 00:00:00','1984-03-01 00:00:00',90.41,258,332,82.19,'Intermediate' from dual union all
select 14777,'014777','1984-017F','KDU part','KDU part',57071,'Reentered','1984-03-08 00:00:00','1984-03-01 00:00:00',91.65,257,453,82.35,'Intermediate' from dual union all
select 14778,'014778','1984-017G','KDU','KDU',57071,'Reentered','1984-05-25 00:00:00','1984-03-26 00:00:00',91.55,318,384,82.46,'Intermediate' from dual union all
select 14779,'014779','1984-017H','KDU part','KDU part',57071,'Reentered','1984-03-04 00:00:00','1984-03-02 00:00:00',91.64,249,461,82.35,'Intermediate' from dual union all
select 1478,'001478','1965-020BC','S3 large debris','RCS 0.26',11770,'In Earth orbit','','1992-01-29 00:00:00',109.73,626,1803,56.08,'Intermediate' from dual union all
select 14780,'014780','1984-021A','Landsat 5','Landsat D''',57075,'In Earth orbit','','1984-04-01 00:00:00',98.72,694,701,98.25,'Sun-Synch' from dual union all
select 14781,'014781','1984-021B','UoSAT-OSCAR-11','UoSAT 2',57075,'In Earth orbit','','1984-03-31 00:00:00',98.49,677,696,98.25,'Sun-Synch' from dual union all
select 14782,'014782','1984-021C','Delta 174','Delta SSPS AJ10-118K',57075,'Reentered','1992-01-21 00:00:00','1988-06-08 00:00:00',96.23,482,674,100.08,'Sun-Synch' from dual union all
select 14783,'014783','1984-022A','Kosmos-1540','Geizer No. 12L',57076,'In Earth orbit','','1984-04-01 00:00:00',1435.98,35735,35834,1.39,'Stationary' from dual union all
select 14784,'014784','1984-022B','8S812','8S812',57076,'Reentered','1984-03-03 00:00:00','1984-03-02 00:00:00',88.13,180,184,51.63,'Intermediate' from dual union all
select 14785,'014785','1984-022C','Perekhodnik','Sredniy perekhodnik',57076,'Reentered','1984-03-03 00:00:00','1984-03-02 00:00:00',87.18,127,142,51.61,'Intermediate' from dual union all
select 14786,'014786','1984-023A','INTELSAT 508','INTELSAT V F8',57077,'In Earth orbit','','1984-04-09 00:00:00',1443.99,35927,35954,.32,'Drift GEO' from dual union all
select 14787,'014787','1984-023B','Ariane H8 L8','Ariane H8 S/N T8',57077,'Reentered','1993-04-06 00:00:00','1984-11-04 00:00:00',554.06,297,31639,10.95,'Highly Ellip' from dual union all
select 14788,'014788','1984-022D','SOZ','SOZ',57076,'Reentered','1986-10-27 00:00:00','1984-04-06 00:00:00',635.38,252,35955,47.41,'GEO Transfer' from dual union all
select 14789,'014789','1984-022E','SOZ','SOZ',57076,'Reentered','1990-02-10 00:00:00','1984-04-01 00:00:00',636.86,313,35971,47.49,'GEO Transfer' from dual union all
select 1479,'001479','1965-020BD','deb Blok Kosmos','',11770,'In Earth orbit','','1992-01-30 00:00:00',114.84,810,2087,56.04,'Intermediate' from dual union all
select 14790,'014790','1984-024A','Kosmos-1541','Oko',57078,'In Earth orbit','','1984-04-06 00:00:00',717.72,614,39738,63,'Molniya' from dual union all
select 14791,'014791','1984-024B','11S510','11S510',57078,'Reentered','1984-04-06 00:00:00','1984-03-13 00:00:00',91.6,214,493,62.87,'Intermediate' from dual union all
select 14792,'014792','1984-024C','BOZ','BOZ',57078,'Reentered','1984-03-31 00:00:00','1984-03-14 00:00:00',91.71,192,525,62.9,'Intermediate' from dual union all
select 14793,'014793','1984-025A','Kosmos-1542','Zenit-6',57079,'Landed','1984-03-21 00:00:00','1984-03-09 00:00:00',92.17,348,414,70.35,'Intermediate' from dual union all
select 14794,'014794','1984-025B','11S510','11S510',57079,'Reentered','1984-03-21 00:00:00','1984-03-11 00:00:00',89.87,218,317,70.36,'Intermediate' from dual union all
select 14795,'014795','1984-012F','JD3','SSU',57066,'In Earth orbit','','1990-05-20 00:00:00',107.44,1024,1192,63.43,'Intermediate' from dual union all
select 14796,'014796','1984-024D','Blok-ML','Blok-ML No. 71633-457',57078,'In Earth orbit','','1984-04-11 00:00:00',709.69,663,39292,62.87,'Molniya' from dual union all
select 14797,'014797','1984-026A','Kosmos-1543','Efir No. 1',57080,'Landed','1984-04-05 00:00:00','1984-03-17 00:00:00',90.59,216,391,62.82,'Intermediate' from dual union all
select 14798,'014798','1984-026B','11S510','11S510',57080,'Reentered','1984-03-27 00:00:00','1984-03-17 00:00:00',90.05,209,345,62.8,'Intermediate' from dual union all
select 14799,'014799','1976-126BR','deb Kosmos-886','',39078,'In Earth orbit','','1984-03-13 00:00:00',123.43,1098,2570,65.82,'Medium' from dual union all
select 148,'000148','61 Omicron 33','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-06 00:00:00',104.7,824,1137,65.75,'Intermediate' from dual union all
select 1480,'001480','1965-020BE','deb Blok Kosmos','',11770,'In Earth orbit','','1992-01-30 00:00:00',114.58,744,2129,56.09,'Intermediate' from dual union all
select 14800,'014800','1976-126BS','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-07 00:00:00',125.91,1410,2477,65.78,'Medium' from dual union all
select 14801,'014801','1975-112D','deb Kosmos-783 [UNID101]','Strela-2M debris',36559,'In Earth orbit','','1984-03-12 00:00:00',100.97,795,815,74.08,'Intermediate' from dual union all
select 14802,'014802','1977-059C','deb Kosmos-923 [UNID]','Strela-2M debris',42187,'In Earth orbit','','1984-03-15 00:00:00',100.77,784,807,74.08,'Intermediate' from dual union all
select 14803,'014803','1978-019D','deb Kosmos-990 [UNID]','Strela-2M debris',45214,'In Earth orbit','','1984-03-14 00:00:00',100.27,765,778,74.06,'Intermediate' from dual union all
select 14804,'014804','1978-063D','deb Kosmos-1023','Strela-2M debris',45258,'Reentered','2007-06-10 00:00:00','1980-07-03 00:00:00',100.37,762,790,74.05,'Intermediate' from dual union all
select 14805,'014805','1979-078C','deb Kosmos-1125','Strela-2M debris',47562,'In Earth orbit','','1984-03-16 00:00:00',100.4,773,781,74.08,'Intermediate' from dual union all
select 14806,'014806','1979-078D','deb Kosmos-1125 [UNID]','Strela-2M debris',47562,'In Earth orbit','','1984-03-14 00:00:00',101.11,807,815,74.05,'Intermediate' from dual union all
select 14807,'014807','1979-089D','deb Kosmos-1140 [UNID]','Strela-2M debris',47573,'In Earth orbit','','1980-10-24 00:00:00',106.27,503,1605,62.82,'Intermediate' from dual union all
select 14808,'014808','1980-056C','deb Kosmos-1190 [UNID]','Strela-2M debris',50266,'In Earth orbit','','1984-03-15 00:00:00',101.33,805,839,74.05,'Intermediate' from dual union all
select 14809,'014809','1980-056D','deb Kosmos-1190 [UNID]','Strela-2M debris',50266,'In Earth orbit','','1984-03-15 00:00:00',101.17,802,827,74.05,'Intermediate' from dual union all
select 1481,'001481','1965-020BF','S3 large debris','RCS 0.29',11770,'Reentered','1979-05-23 00:00:00','1965-08-07 00:00:00',108.72,488,1848,56.19,'Intermediate' from dual union all
select 14810,'014810','1981-084D','deb Kosmos-1302 [UNID]','Strela-2M debris',52607,'In Earth orbit','','1984-03-16 00:00:00',101.23,800,834,73.97,'Intermediate' from dual union all
select 14811,'014811','1982-037D','deb Kosmos-1354','Strela-2M debris',54346,'In Earth orbit','','1984-03-12 00:00:00',101.35,802,844,74.03,'Intermediate' from dual union all
select 14812,'014812','1983-031C','deb Kosmos-1452','Strela-2M debris 2 (high',55788,'In Earth orbit','','1984-03-12 00:00:00',101.35,806,840,74.07,'Intermediate' from dual union all
select 14813,'014813','1983-079C','deb Kosmos (-1486) [UNID]','Strela-2M debris 2 (high',55836,'In Earth orbit','','1984-03-12 00:00:00',101.36,805,842,74.05,'Intermediate' from dual union all
select 14814,'014814','1970-028C','deb Tsiklon','[UNID102] RCS 0.0',21505,'Reentered','2005-10-15 00:00:00','1984-03-15 00:00:00',99.82,740,760,74.03,'Intermediate' from dual union all
select 14815,'014815','1976-061D','deb Kosmos-836 [UNID103]','Strela-2M debris',39013,'Reentered','2012-09-24 00:00:00','1984-03-13 00:00:00',100.35,761,790,74.13,'Intermediate' from dual union all
select 14816,'014816','1976-098C','deb Kosmos-858 [UNID]','Strela-2M debris',39050,'In Earth orbit','','1984-03-13 00:00:00',101.01,790,823,74.05,'Intermediate' from dual union all
select 14817,'014817','1976-098D','deb Kosmos-858 [UNID]','Strela-2M debris',39050,'In Earth orbit','','1983-06-01 00:00:00',606.46,278,34433,10.71,'GEO Transfer' from dual union all
select 14818,'014818','1977-059D','deb Kosmos-923 [UNID]','Strela-2M debris',42187,'In Earth orbit','','1984-03-13 00:00:00',100.52,773,794,74.08,'Intermediate' from dual union all
select 14819,'014819','1984-027A','Kosmos-1544','Tselina-D',57081,'In Earth orbit','','1984-04-14 00:00:00',97.69,632,665,82.54,'Intermediate' from dual union all
select 1482,'001482','1965-020BG','deb Blok Kosmos','',11770,'Reentered','1966-01-09 00:00:00','1965-08-08 00:00:00',100.85,211,1386,55.98,'Intermediate' from dual union all
select 14820,'014820','1984-027B','S5M','S5M',57081,'In Earth orbit','','1990-07-03 00:00:00',97.54,625,657,82.54,'Intermediate' from dual union all
select 14821,'014821','1984-028A','Ekran','Ekran No. 26L',57082,'In Earth orbit','','1984-04-15 00:00:00',1435.93,35775,35792,.1,'Stationary' from dual union all
select 14822,'014822','1984-028B','8S812','8S812',57082,'Reentered','1984-03-18 00:00:00','1984-03-17 00:00:00',88.23,185,189,51.63,'Intermediate' from dual union all
select 14823,'014823','1984-028C','Perekhodnik','Sredniy perekhodnik',57082,'Reentered','1984-03-17 00:00:00','1984-03-16 00:00:00',87.96,152,195,51.65,'Intermediate' from dual union all
select 14824,'014824','1984-011U','deb PAM-D','',57065,'Reentered','1990-02-03 00:00:00','1984-03-04 00:00:00',99.4,303,1157,27.44,'Intermediate' from dual union all
select 14825,'014825','1984-029A','Molniya-1','Molniya-1 No. 51',57083,'Reentered','1994-11-04 00:00:00','1984-04-15 00:00:00',717.76,615,39739,62.89,'Molniya' from dual union all
select 14826,'014826','1984-029B','11S510','11S510',57083,'Reentered','1984-04-26 00:00:00','1984-03-23 00:00:00',92.5,212,583,62.84,'Intermediate' from dual union all
select 14827,'014827','1984-029C','BOZ','BOZ',57083,'Reentered','1984-04-20 00:00:00','1984-03-23 00:00:00',92.45,199,590,62.84,'Intermediate' from dual union all
select 14828,'014828','1984-028D','SOZ','SOZ',57082,'In Earth orbit','','1984-04-15 00:00:00',628.57,348,35510,47.46,'GEO Transfer' from dual union all
select 14829,'014829','1984-028E','SOZ','SOZ',57082,'Reentered','1986-10-27 00:00:00','1984-04-15 00:00:00',627.08,290,35491,47.39,'GEO Transfer' from dual union all
select 1483,'001483','1965-020BH','deb Blok Kosmos','',11770,'Reentered','1968-02-27 00:00:00','1965-08-08 00:00:00',107.92,373,1888,56.18,'Intermediate' from dual union all
select 14830,'014830','1984-029D','Blok-2BL','Blok-2BL No. 54 (654)',57083,'Reentered','1994-09-22 00:00:00','1984-04-15 00:00:00',731.04,605,40402,62.93,'Molniya' from dual union all
select 14831,'014831','1980-089BT','deb Kosmos-1220','',50299,'Reentered','1984-04-24 00:00:00','1984-04-22 00:00:00',89.34,230,254,64.91,'Intermediate' from dual union all
select 14832,'014832','1980-089BU','deb Kosmos-1220','[No TLE]',50299,'Reentered','1985-09-28 00:00:00','1984-03-31 00:00:00',96.08,510,632,65,'Intermediate' from dual union all
select 14833,'014833','1980-089BV','deb Kosmos-1220','',50299,'Reentered','1985-09-09 00:00:00','1985-09-09 00:00:00',89.07,220,236,64.94,'Intermediate' from dual union all
select 14834,'014834','1970-089CX','deb Kosmos-374','',21566,'Reentered','1984-05-08 00:00:00','1984-05-07 00:00:00',88.3,175,206,62.8,'Intermediate' from dual union all
select 14835,'014835','1970-089CY','deb Kosmos-374','',21566,'Reentered','1984-04-21 00:00:00','1984-08-09 00:00:00',93.97,326,612,62.71,'Intermediate' from dual union all
select 14836,'014836','1970-089CZ','deb Kosmos-374','',21566,'Reentered','1989-04-07 00:00:00','1989-04-07 00:00:00',89.76,242,284,62.82,'Intermediate' from dual union all
select 14837,'014837','1981-089J','deb Kosmos-1306','',52612,'Reentered','1989-04-08 00:00:00','1989-04-06 00:00:00',90.36,282,303,64.96,'Intermediate' from dual union all
select 14838,'014838','1970-091AT','deb Kosmos-375','',21568,'Reentered','1989-11-01 00:00:00','1989-11-01 00:00:00',89.71,253,268,62.23,'Intermediate' from dual union all
select 14839,'014839','1977-065FS','deb Delta 132','',42193,'In Earth orbit','','1993-08-04 00:00:00',122.01,925,2617,29,'Medium' from dual union all
select 1484,'001484','1965-020BJ','deb Blok Kosmos','',11770,'Reentered','1968-10-24 00:00:00','1965-08-08 00:00:00',107.56,381,1847,56.06,'Intermediate' from dual union all
select 14840,'014840','1968-097EC','deb Kosmos-252','',16824,'Reentered','1988-01-26 00:00:00','1988-01-26 00:00:00',89.11,217,244,62.11,'Intermediate' from dual union all
select 14841,'014841','1963-047S','Centaur AC-2 blanket part','[UNID104] RCS 0.0',9278,'Reentered','1990-01-22 00:00:00','1978-11-12 00:00:00',115.97,640,2359,30.58,'Intermediate' from dual union all
select 14842,'014842','1965-082UK','deb Transtage 4','',11832,'Reentered','1989-09-11 00:00:00','1989-09-09 00:00:00',91.23,313,357,31.06,'Intermediate' from dual union all
select 14843,'014843','1965-082UL','deb Transtage 4','',11832,'Reentered','1988-04-27 00:00:00','1988-04-23 00:00:00',90.91,312,327,32.49,'Intermediate' from dual union all
select 14844,'014844','1966-013N','deb Diapason','',13420,'Reentered','1988-11-08 00:00:00','1983-11-02 00:00:00',100.23,447,1091,34.02,'Intermediate' from dual union all
select 14845,'014845','1966-013P','deb Diapason','',13420,'Reentered','1989-04-23 00:00:00','1983-07-05 00:00:00',102.36,459,1282,34.02,'Intermediate' from dual union all
select 14846,'014846','1969-064Y','Large Unknown GTO Debris','[UNID105] RCS 5.2',18331,'Reentered','1986-01-29 00:00:00','1986-01-20 00:00:00',136,206,4559,26.9,'Medium' from dual union all
select 14847,'014847','1984-025C','Blenda','Sunshade',57079,'Reentered','1984-05-05 00:00:00','1984-03-21 00:00:00',92.16,347,414,70.35,'Intermediate' from dual union all
select 14848,'014848','1984-025D','KDU','KDU',57079,'Reentered','1985-10-13 00:00:00','1984-04-08 00:00:00',92.04,345,405,70.35,'Intermediate' from dual union all
select 14849,'014849','1984-030A','Kosmos-1545','Zenit-6',57084,'Landed','1984-04-05 00:00:00','1984-03-28 00:00:00',92.26,356,415,72.84,'Intermediate' from dual union all
select 1485,'001485','1965-020BK','deb Blok Kosmos','',11770,'Reentered','1968-04-15 00:00:00','1965-08-08 00:00:00',107.2,405,1790,55.93,'Intermediate' from dual union all
select 14850,'014850','1984-030B','11S510','11S510',57084,'Reentered','1984-03-29 00:00:00','1984-03-23 00:00:00',89.82,191,341,72.83,'Intermediate' from dual union all
select 14851,'014851','1984-025E','KDU part','KDU part',57079,'Reentered','1984-05-29 00:00:00','1984-04-06 00:00:00',93.73,363,551,70.35,'Intermediate' from dual union all
select 14852,'014852','1984-025F','KDU part','KDU part',57079,'Reentered','1984-05-29 00:00:00','1984-04-06 00:00:00',91.57,324,379,70.35,'Intermediate' from dual union all
select 14853,'014853','1983-034J','Kosmos-1453 SS 7','ESO',55791,'Reentered','1984-05-30 00:00:00','1984-03-21 00:00:00',94.12,453,499,74.11,'Intermediate' from dual union all
select 14854,'014854','1983-034K','Kosmos-1453 SS 8','ESO',55791,'Reentered','1984-06-06 00:00:00','1984-03-21 00:00:00',94.24,463,500,74.05,'Intermediate' from dual union all
select 14855,'014855','1984-030C','Sensor cover','Sensor cover',57084,'Reentered','1984-03-29 00:00:00','1984-03-25 00:00:00',89.6,186,324,72.84,'Intermediate' from dual union all
select 14856,'014856','1984-025G','KDU part','KDU part',57079,'Reentered','1984-06-05 00:00:00','1984-05-13 00:00:00',91.24,332,338,70.41,'Intermediate' from dual union all
select 14857,'014857','1982-033AS','Garbage bag','',54342,'Reentered','1984-03-29 00:00:00','1984-03-24 00:00:00',90.3,285,294,51.62,'Intermediate' from dual union all
select 14858,'014858','1982-033AT','Garbage bag','',54342,'Reentered','1984-03-27 00:00:00','1984-03-25 00:00:00',90.31,286,294,51.64,'Intermediate' from dual;
insert into SATELLITE
select 14859,'014859','1982-033AU','Garbage bag','',54342,'Reentered','1984-04-04 00:00:00','1984-03-25 00:00:00',90.36,287,298,51.56,'Intermediate' from dual union all
select 1486,'001486','1965-020BL','deb Blok Kosmos','',11770,'Reentered','1967-06-10 00:00:00','1965-08-07 00:00:00',106.23,334,1770,56.15,'Intermediate' from dual union all
select 14860,'014860','1982-033AV','Garbage bag','',54342,'Reentered','1984-04-03 00:00:00','1984-03-25 00:00:00',90.41,292,298,51.62,'Intermediate' from dual union all
select 14861,'014861','1982-033AW','Garbage bag','',54342,'Reentered','1984-04-11 00:00:00','1984-03-25 00:00:00',90.43,293,299,51.63,'Intermediate' from dual union all
select 14862,'014862','1982-033AX','Garbage bag','',54342,'Reentered','1984-04-13 00:00:00','1984-03-25 00:00:00',90.34,274,309,51.61,'Intermediate' from dual union all
select 14863,'014863','1982-033AY','Garbage bag','',54342,'Reentered','1984-04-03 00:00:00','1984-03-25 00:00:00',90.33,289,293,51.64,'Intermediate' from dual union all
select 14864,'014864','1982-033AZ','Garbage bag','',54342,'Reentered','1984-04-14 00:00:00','1984-03-26 00:00:00',90.27,248,327,51.61,'Intermediate' from dual union all
select 14865,'014865','1975-094E','deb Kosmos-773 [UNID]','Strela-2M debris',36541,'In Earth orbit','','1984-03-25 00:00:00',100.71,787,797,74.04,'Intermediate' from dual union all
select 14866,'014866','1979-060C','deb Kosmos-1110 [UNID]','Strela-2M debris',47544,'In Earth orbit','','1984-03-25 00:00:00',100.4,774,780,74.06,'Intermediate' from dual union all
select 14867,'014867','1984-031A','Kosmos-1546','Oko-S F1',57085,'In Earth orbit','','1984-04-28 00:00:00',1448.24,35951,36096,1.23,'Drift GEO' from dual union all
select 14868,'014868','1984-031B','8S812','8S812',57085,'Reentered','1984-03-31 00:00:00','1984-03-30 00:00:00',88.07,175,183,51.62,'Intermediate' from dual union all
select 14869,'014869','1984-031C','Perekhodnik','Sredniy perekhodnik',57085,'Reentered','1984-03-29 00:00:00','1984-03-29 00:00:00',88.49,192,208,51.61,'Intermediate' from dual union all
select 1487,'001487','1965-020BM','deb Blok Kosmos','',11770,'Reentered','1967-04-18 00:00:00','1965-08-08 00:00:00',106.1,361,1730,56.04,'Intermediate' from dual union all
select 14870,'014870','1982-033BA','Garbage bag','',54342,'Reentered','1984-04-27 00:00:00','1984-04-09 00:00:00',90.12,233,328,51.6,'Intermediate' from dual union all
select 14871,'014871','1982-033BB','Garbage bag','',54342,'Reentered','1984-04-15 00:00:00','1984-04-03 00:00:00',90.3,252,327,51.62,'Intermediate' from dual union all
select 14872,'014872','1984-032A','Soyuz T-11','Soyuz 7K-ST No. 17L',57086,'Landed','1984-10-02 00:00:00','1984-06-04 00:00:00',91.43,333,357,51.62,'Intermediate' from dual union all
select 14873,'014873','1984-032B','11S510','11S510',57086,'Reentered','1984-04-05 00:00:00','1984-04-04 00:00:00',88.46,187,210,51.63,'Intermediate' from dual union all
select 14874,'014874','1972-058JE','deb Delta 89','',27990,'Reentered','1989-10-05 00:00:00','1989-10-05 00:00:00',88.38,188,201,98.66,'Sun-Synch' from dual union all
select 14875,'014875','1972-058JF','deb Delta 89','',27990,'Reentered','1992-02-21 00:00:00','1992-02-17 00:00:00',91.6,344,363,98.46,'Sun-Synch' from dual union all
select 14876,'014876','1972-058JG','deb Delta 89','',27990,'Reentered','1986-12-18 00:00:00','1986-12-17 00:00:00',89.46,243,253,98.37,'Sun-Synch' from dual union all
select 14877,'014877','1972-058JH','deb Delta 89','',27990,'Reentered','1992-02-06 00:00:00','1992-02-04 00:00:00',91.86,361,371,98.54,'Sun-Synch' from dual union all
select 14878,'014878','1972-058JJ','deb Delta 89','',27990,'Reentered','1989-08-16 00:00:00','1989-08-15 00:00:00',90.39,276,312,98.76,'Sun-Synch' from dual union all
select 14879,'014879','1974-089ES','deb Delta 104','',33692,'In Earth orbit','','1993-08-04 00:00:00',118.54,1442,1788,101.29,'Sun-Synch' from dual union all
select 1488,'001488','1965-020BN','deb Blok Kosmos','',11770,'Reentered','1967-11-14 00:00:00','1965-08-08 00:00:00',105.27,272,1743,56.1,'Intermediate' from dual union all
select 14880,'014880','1974-089ET','deb Delta 104','',33692,'In Earth orbit','','1984-04-11 00:00:00',117.13,1426,1678,101.19,'Sun-Synch' from dual union all
select 14881,'014881','1974-089EU','deb Delta 104','',33692,'In Earth orbit','','1993-08-04 00:00:00',115.97,1448,1551,102.15,'Sun-Synch' from dual union all
select 14882,'014882','1975-004HG','deb Delta 107','',36451,'Reentered','1989-07-02 00:00:00','1989-07-01 00:00:00',90.79,298,329,98.7,'Sun-Synch' from dual union all
select 14883,'014883','1973-086GS','deb Delta 98','',30688,'In Earth orbit','','1993-07-02 00:00:00',120.21,1488,1893,101.99,'Medium' from dual union all
select 14884,'014884','1984-033A','Kosmos-1547','Oko',57087,'In Earth orbit','','1984-05-04 00:00:00',717.74,642,39711,62.91,'Molniya' from dual union all
select 14885,'014885','1984-033B','11S510','11S510',57087,'Reentered','1984-05-14 00:00:00','1984-04-13 00:00:00',91.86,213,518,62.8,'Intermediate' from dual union all
select 14886,'014886','1984-033C','BOZ','BOZ',57087,'Reentered','1984-05-01 00:00:00','1984-04-11 00:00:00',91.94,191,548,62.88,'Intermediate' from dual union all
select 14887,'014887','1984-031D','SOZ','SOZ',57085,'Reentered','1985-06-25 00:00:00','1984-04-28 00:00:00',639.47,307,36111,47.34,'GEO Transfer' from dual union all
select 14888,'014888','1984-031E','SOZ','SOZ',57085,'Reentered','1986-10-27 00:00:00','1984-05-01 00:00:00',638.22,251,36103,47.25,'GEO Transfer' from dual union all
select 14889,'014889','1984-030D','Blenda','Sunshade',57084,'Reentered','1984-04-15 00:00:00','1984-04-06 00:00:00',90.44,223,370,72.83,'Intermediate' from dual union all
select 1489,'001489','1965-020BP','deb Blok Kosmos','',11770,'Reentered','1966-10-26 00:00:00','1965-08-08 00:00:00',104.53,292,1653,56.1,'Intermediate' from dual union all
select 14890,'014890','1984-030E','KDU','KDU',57084,'Reentered','1984-05-04 00:00:00','1984-04-06 00:00:00',90.77,225,400,72.82,'Intermediate' from dual union all
select 14891,'014891','1984-030F','KDU part','KDU part',57084,'Reentered','1984-05-29 00:00:00','1984-04-10 00:00:00',91.29,198,478,72.55,'Intermediate' from dual union all
select 14892,'014892','1984-030G','KDU part','KDU part',57084,'Reentered','1984-04-11 00:00:00','1984-04-10 00:00:00',88.87,186,251,72.82,'Intermediate' from dual union all
select 14893,'014893','1984-030H','KDU part','KDU part',57084,'Reentered','1984-04-08 00:00:00','1984-04-07 00:00:00',88.81,126,306,73.32,'Intermediate' from dual union all
select 14894,'014894','1984-033D','Blok-2BL','Blok-2BL No. 50',57087,'In Earth orbit','','1984-05-06 00:00:00',706.62,645,39158,62.95,'Molniya' from dual union all
select 14895,'014895','1984-030J','KDU part','KDU part',57084,'Reentered','1984-05-10 00:00:00','1984-04-06 00:00:00',92.79,372,450,72.86,'Intermediate' from dual union all
select 14896,'014896','1984-030K','KDU part','KDU part',57084,'Reentered','1984-04-25 00:00:00','1984-04-06 00:00:00',92.92,382,453,72.82,'Intermediate' from dual union all
select 14897,'014897','1984-034A','Challenger (STS 41-C)','OV-099',57088,'Landed','1984-04-13 00:00:00','1984-04-08 00:00:00',94.18,477,480,28.51,'Intermediate' from dual union all
select 14898,'014898','1984-034B','LDEF','LDEF',57088,'Landed Attached','1990-01-20 00:00:00','1984-05-06 00:00:00',94.14,476,478,28.51,'Intermediate' from dual union all
select 14899,'014899','1984-035A','Shiyan Tongbu Tongxin Wx.','STW',57089,'In Earth orbit','','1984-05-08 00:00:00',1436.14,35777,35797,.65,'Stationary' from dual union all
select 149,'000149','61 Omicron 34','deb r/b Ablestar','',7273,'Reentered','1979-03-03 00:00:00','1961-10-28 00:00:00',102.5,769,985,67.12,'Intermediate' from dual union all
select 1490,'001490','1965-020BQ','deb Blok Kosmos','',11770,'Reentered','1966-01-03 00:00:00','1965-08-08 00:00:00',102.62,275,1491,55.75,'Intermediate' from dual union all
select 14900,'014900','1984-035B','CZ-3 Stage 3','CZ-3 Stage 3',57089,'In Earth orbit','','1984-05-09 00:00:00',630.05,439,35495,31.03,'GEO Transfer' from dual union all
select 14901,'014901','1984-020C','Koba''lt AO-PO part','',57074,'Reentered','1984-04-10 00:00:00','1984-04-09 00:00:00',89.18,173,295,67.13,'Intermediate' from dual union all
select 14902,'014902','1984-036A','Kosmos-1548','Kobal''t No. 807',57090,'Landed','1984-05-25 00:00:00','1984-04-17 00:00:00',89.58,182,326,67.15,'Intermediate' from dual union all
select 14903,'014903','1984-036B','11S510','11S510',57090,'Reentered','1984-04-13 00:00:00','1984-04-12 00:00:00',88.76,156,271,67.14,'Intermediate' from dual union all
select 14904,'014904','1976-077ES','deb Delta 126','',39029,'In Earth orbit','','1992-08-07 00:00:00',116.63,1488,1570,102.81,'Sun-Synch' from dual union all
select 14905,'014905','1976-077ET','deb Delta 126','',39029,'In Earth orbit','','1980-09-09 00:00:00',917.17,166,49618,26.5,'Highly Ellip' from dual union all
select 14906,'014906','1977-121BK','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-07 00:00:00',105.48,930,1104,65.95,'Intermediate' from dual union all
select 14907,'014907','1977-121BL','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-01 00:00:00',106.6,1024,1114,66.04,'Intermediate' from dual union all
select 14908,'014908','1978-026GH','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',103.9,852,1034,98.86,'Sun-Synch' from dual union all
select 14909,'014909','1978-026GJ','deb Delta 139','',45221,'In Earth orbit','','1980-11-18 00:00:00',607.55,194,34574,26.49,'GEO Transfer' from dual union all
select 1491,'001491','1965-020BR','deb Blok Kosmos','',11770,'Reentered','1968-09-10 00:00:00','1965-08-08 00:00:00',99.03,268,1157,56.13,'Intermediate' from dual union all
select 14910,'014910','1978-026GK','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',102.41,860,885,99.12,'Sun-Synch' from dual union all
select 14911,'014911','1978-026GL','deb Delta 139','',45221,'Reentered','2002-04-15 00:00:00','1993-08-06 00:00:00',100.8,714,879,99.05,'Sun-Synch' from dual union all
select 14912,'014912','1978-026GM','deb Delta 139','',45221,'Reentered','2004-10-01 00:00:00','1993-08-01 00:00:00',100.95,751,857,99.02,'Sun-Synch' from dual union all
select 14913,'014913','1978-026GN','deb Delta 139','',45221,'In Earth orbit','','1993-08-04 00:00:00',101.99,771,935,98.99,'Sun-Synch' from dual union all
select 14914,'014914','1978-026GP','deb Delta 139','',45221,'In Earth orbit','','1984-04-14 00:00:00',101.97,805,899,98.71,'Sun-Synch' from dual union all
select 14915,'014915','1978-026GQ','deb Delta 139','',45221,'Reentered','2001-01-23 00:00:00','1980-10-24 00:00:00',268.07,396,14391,19.74,'Highly Ellip' from dual union all
select 14916,'014916','1984-014C','Soyuz T-10 BO part','Soyuz BO part',57068,'Reentered','1984-05-26 00:00:00','1984-04-11 00:00:00',89.98,251,296,51.62,'Intermediate' from dual union all
select 14917,'014917','1984-014D','Soyuz T-10 BO part','Soyuz BO part',57068,'Reentered','1984-05-15 00:00:00','1984-04-15 00:00:00',90.05,212,342,51.61,'Intermediate' from dual union all
select 14918,'014918','1984-014E','Soyuz T-10 BO','Bitovoy otsek',57068,'Reentered','1984-05-20 00:00:00','1984-04-11 00:00:00',90.21,279,291,51.62,'Intermediate' from dual union all
select 14919,'014919','1976-067BG','deb Kosmos-839','',39019,'In Earth orbit','','1980-10-23 00:00:00',104.52,903,1041,99.81,'Sun-Synch' from dual union all
select 1492,'001492','1965-020BS','deb Blok Kosmos','',11770,'Reentered','1965-11-23 00:00:00','1965-08-08 00:00:00',100.73,270,1317,56.08,'Intermediate' from dual union all
select 14920,'014920','1980-030AS','deb Kosmos-1174','',50240,'Reentered','1989-12-27 00:00:00','1989-12-27 00:00:00',90.28,259,318,66.19,'Intermediate' from dual union all
select 14921,'014921','1980-030AT','deb Kosmos-1174','',50240,'Reentered','1988-10-20 00:00:00','1988-10-18 00:00:00',91.23,304,365,66.21,'Intermediate' from dual union all
select 14922,'014922','1980-030AU','deb Kosmos-1174','',50240,'Reentered','1989-01-17 00:00:00','1989-01-16 00:00:00',90.79,296,331,66.16,'Intermediate' from dual union all
select 14923,'014923','1980-030AV','deb Kosmos-1174','',50240,'Reentered','1989-02-03 00:00:00','1989-02-03 00:00:00',89.57,241,266,66.2,'Intermediate' from dual union all
select 14924,'014924','1981-028BU','deb Kosmos-1260','',52551,'Reentered','1986-09-19 00:00:00','1986-09-19 00:00:00',88,168,183,64.97,'Intermediate' from dual union all
select 14925,'014925','1981-053JX','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',103.72,898,971,82.87,'Intermediate' from dual union all
select 14926,'014926','1981-053JY','deb Kosmos-1275','',52576,'Reentered','1990-11-12 00:00:00','1990-11-11 00:00:00',90.61,301,308,82.78,'Intermediate' from dual union all
select 14927,'014927','1983-051E','deb Delta','',55808,'Reentered','1989-03-21 00:00:00','1983-08-24 00:00:00',103.49,329,1518,72.48,'Intermediate' from dual union all
select 14928,'014928','1983-075F','deb Blok E','',55832,'In Earth orbit','','1984-04-10 00:00:00',97.45,609,665,97.99,'Sun-Synch' from dual union all
select 14929,'014929','1984-002G','KDU','KDU',57056,'Reentered','1985-06-21 00:00:00','1984-04-11 00:00:00',91.92,344,393,72.83,'Intermediate' from dual union all
select 1493,'001493','1965-020BT','S3 large debris','RCS 1.49',11770,'Reentered','1966-11-04 00:00:00','1965-08-07 00:00:00',104.98,288,1699,56.22,'Intermediate' from dual union all
select 14930,'014930','1984-037A','DSP F11','DSP 12',57091,'In Earth orbit','','1984-06-13 00:00:00',1436.27,35780,35800,1,'Stationary' from dual union all
select 14931,'014931','1984-037B','Transtage D-11','Transtage D-11',57091,'In Earth orbit','','1984-06-13 00:00:00',1441.89,35800,36000,2,'Drift GEO' from dual union all
select 14932,'014932','1984-038A','Progress-20','Progress 7K-TG No. 121',57092,'Deorbited','1984-05-07 00:00:00','1984-04-17 00:00:00',89.55,236,269,51.59,'Intermediate' from dual union all
select 14933,'014933','1984-038B','11S510','11S510',57092,'Reentered','1984-04-17 00:00:00','1984-04-16 00:00:00',88.64,180,235,51.59,'Intermediate' from dual union all
select 14934,'014934','59 Alpha 4','deb Vanguard II','',5525,'Reentered','2007-10-23 00:00:00','1993-08-04 00:00:00',111.18,526,2036,32.88,'Intermediate' from dual union all
select 14935,'014935','1984-039A','OPS 8424','GAMBIT-3 Mission 4354',57093,'Reentered Attached','1984-08-13 00:00:00','1984-04-17 00:00:00',88.11,127,235,96.4,'Sun-Synch' from dual union all
select 14936,'014936','1983-034L','Kosmos-1453 SS 9','ESO',55791,'Reentered','1984-09-05 00:00:00','1984-04-19 00:00:00',94.17,456,501,73.96,'Intermediate' from dual union all
select 14937,'014937','1983-034M','Kosmos-1453 SS 10','ESO',55791,'Reentered','1984-08-19 00:00:00','1984-04-19 00:00:00',94.11,455,496,74.1,'Intermediate' from dual union all
select 14938,'014938','1984-040A','Kosmos-1549','Zenit-6',57094,'Landed','1984-05-03 00:00:00','1984-04-21 00:00:00',92.26,357,414,72.89,'Intermediate' from dual union all
select 14939,'014939','1984-040B','11S510','11S510',57094,'Reentered','1984-04-28 00:00:00','1984-04-21 00:00:00',89.87,195,341,72.89,'Intermediate' from dual union all
select 1494,'001494','1965-020BU','deb Blok Kosmos','',11770,'Reentered','1965-09-04 00:00:00','1965-03-15 00:00:00',95.32,194,875,56.1,'Intermediate' from dual union all
select 14940,'014940','1984-041A','Gorizont','Gorizont No. 19L',57095,'In Earth orbit','','1984-05-22 00:00:00',1436.32,35777,35805,1.37,'Stationary' from dual union all
select 14941,'014941','1984-041B','8S812','8S812',57095,'Reentered','1984-04-24 00:00:00','1984-04-23 00:00:00',88.03,172,182,51.62,'Intermediate' from dual union all
select 14942,'014942','1984-041C','Perekhodnik','Sredniy perekhodnik',57095,'Reentered','1984-04-22 00:00:00','1984-04-22 00:00:00',87.81,142,191,51.6,'Intermediate' from dual union all
select 14943,'014943','1984-041D','Blok DM No. 46L','11S86 No. 46L',57095,'In Earth orbit','','1993-06-01 00:00:00',1459.99,36188,36318,6.96,'Inclined Drift' from dual union all
select 14944,'014944','1984-041E','SOZ','SOZ',57095,'Reentered','1990-12-09 00:00:00','1984-06-01 00:00:00',643.92,293,36353,47.28,'GEO Transfer' from dual union all
select 14945,'014945','1984-041F','SOZ','SOZ',57095,'Reentered','1986-10-27 00:00:00','1984-05-26 00:00:00',642.43,243,36327,47.26,'GEO Transfer' from dual union all
select 14946,'014946','1982-033BC','D-7-3 EVA-1 debris','',54342,'Reentered','1984-05-17 00:00:00','1984-05-02 00:00:00',90.2,270,299,51.62,'Intermediate' from dual union all
select 14947,'014947','1983-101E','Kosmos-1501 SS 3','ESO',55858,'Reentered','1984-09-05 00:00:00','1984-04-30 00:00:00',94.13,461,492,82.88,'Intermediate' from dual union all
select 14948,'014948','1984-022F','Blok DM','11S86 No. 43L',57076,'In Earth orbit','','1993-06-09 00:00:00',1441.86,35812,35987,7.92,'Inclined Drift' from dual;
insert into SATELLITE
select 14949,'014949','1982-033BD','D-7-3 EVA-1 debris','',54342,'Reentered','1984-06-16 00:00:00','1984-04-28 00:00:00',90.52,277,322,51.62,'Intermediate' from dual union all
select 1495,'001495','1965-020BV','deb Blok Kosmos','',11770,'Reentered','2004-10-14 00:00:00','1992-01-29 00:00:00',103.43,606,1236,55.61,'Intermediate' from dual union all
select 14950,'014950','1977-117C','deb Blok E','(shadow of Blok E)',42245,'In Earth orbit','','1980-10-24 00:00:00',102.75,843,935,70.31,'Intermediate' from dual union all
select 14951,'014951','1984-031F','Blok DM','11S86 No. 45L',57085,'In Earth orbit','','1984-05-02 00:00:00',1448.21,35949,36098,1.24,'Drift GEO' from dual union all
select 14952,'014952','1984-040C','Blenda','Sunshade',57094,'Reentered','1984-08-29 00:00:00','1984-05-20 00:00:00',91.9,341,395,72.89,'Intermediate' from dual union all
select 14953,'014953','1984-040D','KDU','KDU',57094,'Reentered','1986-05-14 00:00:00','1984-05-23 00:00:00',92.18,356,407,72.89,'Intermediate' from dual union all
select 14954,'014954','1984-040E','KDU part','KDU part',57094,'Reentered','1984-06-23 00:00:00','1984-05-20 00:00:00',91.85,345,386,72.89,'Intermediate' from dual union all
select 14955,'014955','1984-040F','KDU part','KDU part',57094,'Reentered','1984-05-07 00:00:00','1984-05-03 00:00:00',94.59,280,718,73.34,'Intermediate' from dual union all
select 14956,'014956','1984-040G','KDU part','KDU part',57094,'Reentered','1984-10-13 00:00:00','1984-05-19 00:00:00',94.28,386,582,72.89,'Intermediate' from dual union all
select 14957,'014957','1982-033BE','EVA debris','',54342,'Reentered','1984-05-13 00:00:00','1984-05-05 00:00:00',90.38,270,316,51.62,'Intermediate' from dual union all
select 14958,'014958','1982-033BF','EVA debris','',54342,'Reentered','1984-05-14 00:00:00','1984-05-05 00:00:00',90.31,269,311,51.62,'Intermediate' from dual union all
select 14959,'014959','61 Omicron 267','deb r/b Ablestar','',7273,'Reentered','1991-01-03 00:00:00','1991-01-01 00:00:00',90.85,307,326,66.83,'Intermediate' from dual union all
select 1496,'001496','1965-020BW','deb Blok Kosmos','',11770,'Reentered','1966-09-15 00:00:00','1965-08-08 00:00:00',104.21,307,1608,56.11,'Intermediate' from dual union all
select 14960,'014960','1982-033BG','D-7-3 EVA-4 debris','',54342,'Reentered','1984-06-21 00:00:00','1984-05-05 00:00:00',90.47,279,317,51.62,'Intermediate' from dual union all
select 14961,'014961','1984-042A','Progress-21','Progress 7K-TG No. 116',57096,'Deorbited','1984-05-26 00:00:00','1984-05-09 00:00:00',89.7,243,277,51.64,'Intermediate' from dual union all
select 14962,'014962','1984-042B','11S510','11S510',57096,'Reentered','1984-05-10 00:00:00','1984-05-09 00:00:00',88.51,178,224,51.63,'Intermediate' from dual union all
select 14963,'014963','1983-101F','Kosmos-1501 SS 4','ESO',55858,'Reentered','1984-09-09 00:00:00','1984-05-10 00:00:00',94.03,452,492,83,'Intermediate' from dual union all
select 14964,'014964','1984-012G','JD2 cover','SSU cover',57066,'Reentered','1984-10-18 00:00:00','1997-11-22 00:00:00',107.42,829,1386,63.44,'Intermediate' from dual union all
select 14965,'014965','1984-043A','Kosmos-1550','Parus',57097,'In Earth orbit','','1992-06-23 00:00:00',104.94,973,1011,82.97,'Intermediate' from dual union all
select 14966,'014966','1984-043B','S3M','S3M',57097,'In Earth orbit','','1991-03-25 00:00:00',104.84,975,999,82.97,'Intermediate' from dual union all
select 14967,'014967','1984-044A','Kosmos-1551','Zenit-6',57098,'Landed','1984-05-23 00:00:00','1984-05-13 00:00:00',89.19,212,257,72.87,'Intermediate' from dual union all
select 14968,'014968','1984-044B','11S510','11S510',57098,'Reentered','1984-05-17 00:00:00','1984-05-13 00:00:00',88.88,189,250,72.85,'Intermediate' from dual union all
select 14969,'014969','1984-012H','JD3 cover','SSU cover',57066,'Reentered','1984-10-18 00:00:00','1997-11-22 00:00:00',107.05,824,1356,63.44,'Intermediate' from dual union all
select 1497,'001497','1965-020BX','deb Blok Kosmos','',11770,'Reentered','1967-05-28 00:00:00','1965-08-07 00:00:00',106.01,360,1724,55.89,'Intermediate' from dual union all
select 14970,'014970','1984-011V','deb PAM-D','',57065,'Reentered','1987-07-16 00:00:00','1984-06-30 00:00:00',95.22,341,718,27.9,'Intermediate' from dual union all
select 14971,'014971','1984-045A','Kosmos-1552','Yantar''-4KS1',57099,'Deorbited','1984-11-03 00:00:00','1984-06-13 00:00:00',89.71,218,303,64.93,'Intermediate' from dual union all
select 14972,'014972','1984-045B','11S510','11S510',57099,'Reentered','1984-05-19 00:00:00','1984-05-16 00:00:00',89.15,178,287,64.94,'Intermediate' from dual union all
select 14973,'014973','1984-046A','Kosmos-1553','Tsikada',57100,'In Earth orbit','','1984-06-21 00:00:00',104.79,962,1008,82.94,'Intermediate' from dual union all
select 14974,'014974','1984-046B','S3M','S3M',57100,'In Earth orbit','','1991-03-27 00:00:00',104.63,943,1012,82.93,'Intermediate' from dual union all
select 14975,'014975','1982-033BH','D-7-3 EVA-4 debris','',54342,'Reentered','1984-05-24 00:00:00','1984-05-19 00:00:00',90.41,279,310,51.62,'Intermediate' from dual union all
select 14976,'014976','1982-033BJ','D-7-3 EVA-4 debris','',54342,'Reentered','1984-05-26 00:00:00','1984-05-19 00:00:00',90.31,270,310,51.63,'Intermediate' from dual union all
select 14977,'014977','1984-047A','Kosmos-1554','Uragan No. 16L',57101,'In Earth orbit','','1984-06-20 00:00:00',675.71,19079,19180,64.81,'Medium' from dual union all
select 14978,'014978','1984-047B','Kosmos-1555','Uragan No. 17L',57101,'In Earth orbit','','1984-06-23 00:00:00',675.71,19107,19152,64.83,'Medium' from dual union all
select 14979,'014979','1984-047C','Kosmos-1556','Uragan GVM',57101,'In Earth orbit','','1984-06-18 00:00:00',676.31,19129,19160,64.81,'Medium' from dual union all
select 1498,'001498','1965-020BY','deb Blok Kosmos','',11770,'Reentered','1966-05-06 00:00:00','1965-08-08 00:00:00',102.92,263,1530,56.11,'Intermediate' from dual union all
select 14980,'014980','1984-047D','8S812','8S812',57101,'Reentered','1984-05-21 00:00:00','1984-05-20 00:00:00',88.14,178,187,51.63,'Intermediate' from dual union all
select 14981,'014981','1984-047E','Perekhodnik','Sredniy perekhodnik',57101,'Reentered','1984-05-20 00:00:00','1984-05-19 00:00:00',88.42,189,204,51.61,'Intermediate' from dual union all
select 14982,'014982','1984-048A','Kosmos-1557','Fram',57102,'Landed','1984-06-04 00:00:00','1984-05-24 00:00:00',88.96,210,237,82.33,'Intermediate' from dual union all
select 14983,'014983','1984-048B','11S510','11S510',57102,'Reentered','1984-05-26 00:00:00','1984-05-24 00:00:00',88.48,195,204,82.34,'Intermediate' from dual union all
select 14984,'014984','1984-047F','Blok DM2','11S861 No. 3L',57101,'In Earth orbit','','1984-06-19 00:00:00',675.51,19090,19159,64.82,'Medium' from dual union all
select 14985,'014985','1984-049A','Spacenet F1','Spacenet F1',57103,'In Earth orbit','','1984-06-22 00:00:00',1436.09,35785,35788,.05,'Stationary' from dual union all
select 14986,'014986','1984-044C','Blenda','Sunshade',57098,'Reentered','1984-05-24 00:00:00','1984-05-23 00:00:00',89.1,207,252,72.86,'Intermediate' from dual union all
select 14987,'014987','1984-044D','KDU part','KDU part',57098,'Reentered','1984-05-25 00:00:00','1984-05-23 00:00:00',88.89,198,242,72.86,'Intermediate' from dual union all
select 14988,'014988','1984-044E','KDU','KDU',57098,'Reentered','1984-05-31 00:00:00','1984-05-23 00:00:00',89.03,211,242,72.87,'Intermediate' from dual union all
select 14989,'014989','1984-049B','Ariane H8 V9','Ariane H8 S/N T9',57103,'Reentered','1986-03-17 00:00:00','1984-06-23 00:00:00',545.14,150,31305,6.99,'Highly Ellip' from dual union all
select 1499,'001499','1965-020BZ','deb Blok Kosmos','',11770,'Reentered','1966-09-07 00:00:00','1965-08-07 00:00:00',104.16,306,1604,56.07,'Intermediate' from dual union all
select 14990,'014990','1984-044F','KDU part','KDU part',57098,'Reentered','1984-05-27 00:00:00','1984-05-23 00:00:00',90.98,243,403,72.88,'Intermediate' from dual union all
select 14991,'014991','1984-036C','Kosmos-1548 part','Solar panel',57090,'Reentered','1984-05-25 00:00:00','1984-05-25 00:00:00',88.48,159,240,67.14,'Intermediate' from dual union all
select 14992,'014992','1984-036D','Kosmos-1548 part','Solar panel',57090,'Reentered','1984-05-25 00:00:00','1984-05-25 00:00:00',88.35,155,231,67.14,'Intermediate' from dual union all
select 14993,'014993','1984-050A','Kosmos-1558','Kobal''t No. 808',57104,'Landed','1984-07-08 00:00:00','1984-06-01 00:00:00',89.2,165,305,67.15,'Intermediate' from dual union all
select 14994,'014994','1984-050B','11S510','11S510',57104,'Reentered','1984-05-28 00:00:00','1984-05-27 00:00:00',88.48,156,243,67.16,'Intermediate' from dual union all
select 14995,'014995','1982-033BK','Progress part','',54342,'Reentered','1984-06-01 00:00:00','1984-05-28 00:00:00',91.24,323,347,51.6,'Intermediate' from dual union all
select 14996,'014996','1984-051A','Progress-22','Progress 7K-TG No. 122',57105,'Deorbited','1984-07-16 00:00:00','1984-06-04 00:00:00',91.42,333,356,51.61,'Intermediate' from dual union all
select 14997,'014997','1984-051B','11S510','11S510',57105,'Reentered','1984-05-30 00:00:00','1984-05-29 00:00:00',88.54,186,219,51.63,'Intermediate' from dual union all
select 14998,'014998','1984-052A','Kosmos-1559','Strela-1M',57106,'In Earth orbit','','1984-06-27 00:00:00',115.77,1470,1511,74.01,'Intermediate' from dual union all
select 14999,'014999','1984-052B','Kosmos-1560','Strela-1M',57106,'In Earth orbit','','1984-07-19 00:00:00',115.57,1471,1492,74.01,'Intermediate' from dual union all
select 15,'000015','59 Delta 1','Explorer 6','NASA S-2',5527,'In Earth orbit','1961-07-01 00:00:00','1959-09-08 00:00:00',763.29,250,42327,46.95,'Highly Ellip' from dual union all
select 150,'000150','61 Omicron 35','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-17 00:00:00',105.76,970,1089,66.23,'Intermediate' from dual union all
select 1500,'001500','61 Omicron 207','deb r/b Ablestar','',7273,'Reentered','2013-06-28 00:00:00','1965-08-07 00:00:00',103.28,848,980,66.77,'Intermediate' from dual union all
select 15000,'015000','1984-052C','Kosmos-1561','Strela-1M',57106,'In Earth orbit','','1991-05-04 00:00:00',115.39,1462,1485,74.01,'Intermediate' from dual union all
select 15001,'015001','1984-052D','Kosmos-1562','Strela-1M',57106,'In Earth orbit','','1991-12-12 00:00:00',115.21,1453,1476,74.01,'Intermediate' from dual union all
select 15002,'015002','1984-052E','Kosmos-1563','Strela-1M',57106,'In Earth orbit','','1991-12-11 00:00:00',115.03,1438,1476,74.01,'Intermediate' from dual union all
select 15003,'015003','1984-052F','Kosmos-1564','Strela-1M',57106,'In Earth orbit','','1991-06-01 00:00:00',114.88,1424,1475,74.01,'Intermediate' from dual union all
select 15004,'015004','1984-052G','Kosmos-1565','Strela-1M',57106,'In Earth orbit','','1991-03-20 00:00:00',114.71,1408,1476,74.01,'Intermediate' from dual union all
select 15005,'015005','1984-052H','Kosmos-1566','Strela-1M',57106,'In Earth orbit','','1991-12-14 00:00:00',114.54,1394,1475,74.01,'Intermediate' from dual union all
select 15006,'015006','1984-052J','S3M','S3M',57106,'In Earth orbit','','1991-12-15 00:00:00',117.68,1476,1678,74.01,'Intermediate' from dual union all
select 15007,'015007','1983-034N','Kosmos-1453 SS 11','ESO',55791,'Reentered','1984-11-02 00:00:00','1984-06-01 00:00:00',94.06,453,493,74.04,'Intermediate' from dual union all
select 15008,'015008','1983-034P','Kosmos-1453 SS 12','ESO',55791,'Reentered','1984-11-22 00:00:00','1984-05-30 00:00:00',94.1,456,494,74.07,'Intermediate' from dual union all
select 15009,'015009','1984-053A','Kosmos-1567','US-PM1',57107,'Deorbited','1988-04-03 00:00:00','1984-06-29 00:00:00',93.28,430,440,65.03,'Intermediate' from dual union all
select 1501,'001501','61 Omicron 208','deb r/b Ablestar','',7273,'Reentered','1979-01-23 00:00:00','1965-08-07 00:00:00',100.25,563,978,66.35,'Intermediate' from dual union all
select 15010,'015010','1984-053B','11S692','11S692',57107,'Reentered','1984-05-31 00:00:00','1984-05-31 00:00:00',88.96,109,337,65,'Intermediate' from dual union all
select 15011,'015011','1984-054A','Kosmos-1568','Zenit-6',57108,'Landed','1984-06-14 00:00:00','1984-06-03 00:00:00',92.25,355,415,72.84,'Intermediate' from dual union all
select 15012,'015012','1984-054B','11S510','11S510',57108,'Reentered','1984-06-11 00:00:00','1984-06-02 00:00:00',89.98,196,350,72.8,'Intermediate' from dual union all
select 15013,'015013','1983-034Q','Kosmos-1453 SS 13','ESO',55791,'Reentered','1984-12-12 00:00:00','1984-06-02 00:00:00',94.25,464,501,74.06,'Intermediate' from dual union all
select 15014,'015014','1983-034R','Kosmos-1453 SS 14','ESO',55791,'Reentered','1984-12-03 00:00:00','1984-06-02 00:00:00',94.21,461,499,74.01,'Intermediate' from dual union all
select 15015,'015015','1983-034S','Kosmos-1453 SS 15','ESO',55791,'Reentered','1984-12-22 00:00:00','1984-06-01 00:00:00',94.21,462,498,74.05,'Intermediate' from dual union all
select 15016,'015016','1983-034T','Kosmos-1453 SS 16','ESO',55791,'Reentered','1984-11-14 00:00:00','1984-06-01 00:00:00',94.15,459,496,74.02,'Intermediate' from dual union all
select 15017,'015017','1983-034U','Kosmos-1453 SS 17','ESO',55791,'Reentered','1984-11-06 00:00:00','1984-05-30 00:00:00',94.1,456,494,73.99,'Intermediate' from dual union all
select 15018,'015018','1983-034V','Kosmos-1453 SS 18','ESO',55791,'Reentered','1984-10-24 00:00:00','1984-06-01 00:00:00',94,449,492,74.01,'Intermediate' from dual union all
select 15019,'015019','1983-034W','Kosmos-1453 SS 19','ESO',55791,'Reentered','1984-10-19 00:00:00','1984-06-02 00:00:00',93.97,447,490,74.05,'Intermediate' from dual union all
select 1502,'001502','1965-063B','SEV','Scout Adapter E-1P',11813,'In Earth orbit','','1966-02-10 00:00:00',122.21,1134,2425,69.24,'Medium' from dual union all
select 15020,'015020','1983-034X','Kosmos-1453 SS 20','ESO',55791,'Reentered','1984-10-14 00:00:00','1984-06-01 00:00:00',93.95,446,490,74,'Intermediate' from dual union all
select 15021,'015021','1984-054C','Sensor cover','Sensor cover',57108,'Reentered','1984-06-06 00:00:00','1984-06-03 00:00:00',89.18,178,289,72.83,'Intermediate' from dual union all
select 15022,'015022','1984-054D','Sensor cover','Sensor cover',57108,'Reentered','1984-06-06 00:00:00','1984-06-03 00:00:00',89.2,179,291,72.82,'Intermediate' from dual union all
select 15023,'015023','1984-048C','KDU','KDU',57102,'Reentered','1984-06-11 00:00:00','1984-06-04 00:00:00',88.83,206,228,82.33,'Intermediate' from dual union all
select 15024,'015024','1984-048D','Blenda','Sunshade',57102,'Reentered','1984-06-06 00:00:00','1984-06-04 00:00:00',88.68,205,213,82.33,'Intermediate' from dual union all
select 15025,'015025','1984-048E','PTDU cover','KDU part',57102,'Reentered','1984-06-06 00:00:00','1984-06-04 00:00:00',90.62,220,390,82.33,'Intermediate' from dual union all
select 15026,'015026','1984-048F','PTDU cover/','KDU part',57102,'Reentered','1984-06-06 00:00:00','1984-06-04 00:00:00',90.56,215,388,82.33,'Intermediate' from dual union all
select 15027,'015027','1984-055A','Kosmos-1569','Oko',57109,'Reentered','2001-05-07 00:00:00','1984-07-06 00:00:00',717.88,612,39748,63.05,'Molniya' from dual union all
select 15028,'015028','1984-055B','11S510','11S510',57109,'Reentered','1984-07-28 00:00:00','1984-06-07 00:00:00',92.48,215,578,62.81,'Intermediate' from dual union all
select 15029,'015029','1984-055C','BOZ','BOZ',57109,'Reentered','1984-07-10 00:00:00','1984-06-07 00:00:00',92.53,195,602,62.8,'Intermediate' from dual union all
select 1503,'001503','1965-064A','Surveyor D-2','Surveyor SD-2',11814,'Deep Space','1965-08-12 00:00:00','1965-08-11 00:00:00',44184.07,166,815085,28.56,'Cislunar' from dual union all
select 15030,'015030','1984-055D','Blok-2BL','Blok-2BL No. 55',57109,'Reentered','1999-09-28 00:00:00','1984-07-06 00:00:00',706.86,607,39208,63,'Molniya' from dual union all
select 15031,'015031','1984-056A','Kosmos-1570','Strela-2M',57110,'In Earth orbit','','1984-07-12 00:00:00',100.86,789,810,74.07,'Intermediate' from dual union all
select 15032,'015032','1984-056B','S3M','S3M',57110,'In Earth orbit','','1990-03-23 00:00:00',100.68,778,804,74.07,'Intermediate' from dual union all
select 15033,'015033','1984-056C','deb Kosmos- 1570','Strela-2M debris 2 (high',57110,'In Earth orbit','','1984-07-10 00:00:00',101.32,808,835,74.08,'Intermediate' from dual union all
select 15034,'015034','1984-057A','INTELSAT 509','INTELSAT V F9',57111,'Deorbited','1984-10-24 00:00:00','1984-07-09 00:00:00',99.04,227,1198,28.71,'Intermediate' from dual union all
select 15035,'015035','1984-057B','Centaur AC-62','Centaur D-1AR  AC-62',57111,'Reentered','1984-06-11 00:00:00','1984-06-10 00:00:00',98.51,130,1245,29.17,'Intermediate' from dual union all
select 15036,'015036','1984-058A','Kosmos-1571','Oblik',57112,'Landed','1984-06-26 00:00:00','1984-06-18 00:00:00',92.17,348,414,70.02,'Intermediate' from dual union all
select 15037,'015037','1984-058B','11S510','11S510',57112,'Reentered','1984-06-24 00:00:00','1984-06-12 00:00:00',90.16,207,358,70,'Intermediate' from dual;
insert into SATELLITE
select 15038,'015038','1984-058C','Sensor cover','Sensor cover',57112,'Reentered','1984-06-16 00:00:00','1984-06-13 00:00:00',89.11,191,270,70.03,'Intermediate' from dual union all
select 15039,'015039','1984-059A','GPS 9','Navstar GPS SVN 9',57113,'In Earth orbit','','1984-07-14 00:00:00',717.94,20009,20354,62.54,'Medium' from dual union all
select 1504,'001504','1965-065A','NRL adapter','Adapter',11815,'In Earth orbit','','1965-12-14 00:00:00',108.09,1087,1190,90.03,'Polar' from dual union all
select 15040,'015040','1984-059B','SGS-II','Star 48  S/N 10014-9',57113,'Reentered','1994-05-11 00:00:00','1984-07-14 00:00:00',367.63,560,20683,62.37,'Highly Ellip' from dual union all
select 15041,'015041','1984-054E','Blenda','Sunshade',57108,'Reentered','1984-08-08 00:00:00','1984-07-01 00:00:00',91.85,344,387,72.83,'Intermediate' from dual union all
select 15042,'015042','1984-054F','KDU part','KDU part',57108,'Reentered','1984-11-01 00:00:00','1984-07-01 00:00:00',91.93,336,403,72.83,'Intermediate' from dual union all
select 15043,'015043','1984-054G','KDU','KDU',57108,'Reentered','1986-08-22 00:00:00','1984-07-04 00:00:00',92.2,354,411,72.83,'Intermediate' from dual union all
select 15044,'015044','1984-054H','KDU part','KDU part',57108,'Reentered','1984-06-16 00:00:00','1984-06-15 00:00:00',92.33,341,437,72.62,'Intermediate' from dual union all
select 15045,'015045','1984-054J','KDU part','KDU part',57108,'Reentered','1984-06-20 00:00:00','1984-06-14 00:00:00',92.31,360,416,72.64,'Intermediate' from dual union all
select 15046,'015046','1984-060A','Kosmos-1572','Resurs-F1 17F41 No. 26',57114,'Landed','1984-06-29 00:00:00','1984-06-17 00:00:00',89.81,259,271,82.35,'Intermediate' from dual union all
select 15047,'015047','1984-060B','11S510','11S510',57114,'Reentered','1984-06-22 00:00:00','1984-06-15 00:00:00',89.22,213,259,82.35,'Intermediate' from dual union all
select 15048,'015048','1984-054K','KDU part','KDU part',57108,'Reentered','1984-12-04 00:00:00','1984-07-01 00:00:00',94.06,384,562,72.83,'Intermediate' from dual union all
select 15049,'015049','1984-054L','KDU part','KDU part',57108,'Reentered','1984-09-10 00:00:00','1984-07-01 00:00:00',94.25,380,585,72.8,'Intermediate' from dual union all
select 1505,'001505','1965-066A','Kosmos-78','Zenit-2 No. 30',11816,'Landed','1965-08-22 00:00:00','1965-08-18 00:00:00',89.53,199,303,68.97,'Intermediate' from dual union all
select 15050,'015050','1984-054M','KDU part','KDU part',57108,'Reentered','1984-07-13 00:00:00','1984-06-15 00:00:00',95.06,382,660,72.89,'Intermediate' from dual union all
select 15051,'015051','1984-061A','Kosmos-1573','Zenit-6',57115,'Landed','1984-06-28 00:00:00','1984-06-21 00:00:00',89.91,230,310,72.87,'Intermediate' from dual union all
select 15052,'015052','1984-061B','11S510','11S510',57115,'Reentered','1984-06-24 00:00:00','1984-06-19 00:00:00',89.18,194,273,72.9,'Intermediate' from dual union all
select 15053,'015053','1984-047G','SOZ','SOZ',57101,'In Earth orbit','','1984-06-21 00:00:00',340.26,402,19132,52.09,'Highly Ellip' from dual union all
select 15054,'015054','1984-047H','SOZ','SOZ',57101,'In Earth orbit','','1984-06-19 00:00:00',340.63,421,19136,51.99,'Highly Ellip' from dual union all
select 15055,'015055','1984-062A','Kosmos-1574','Nadezhda 11F643N No. 531',57116,'In Earth orbit','','1984-08-08 00:00:00',104.86,967,1009,82.96,'Intermediate' from dual union all
select 15056,'015056','1984-062B','S3M','S3M',57116,'In Earth orbit','','1990-07-01 00:00:00',104.72,964,999,82.95,'Intermediate' from dual union all
select 15057,'015057','1984-063A','Raduga','Gran'' No. 27L',57117,'In Earth orbit','','1984-08-05 00:00:00',1436.04,35763,35808,1.18,'Stationary' from dual union all
select 15058,'015058','1984-063B','8S812','8S812',57117,'Reentered','1984-06-24 00:00:00','1984-06-22 00:00:00',88.22,183,190,51.6,'Intermediate' from dual union all
select 15059,'015059','1984-063C','Perekhodnik','Sredniy perekhodnik',57117,'Reentered','1984-06-22 00:00:00','1984-06-22 00:00:00',88.26,184,193,51.6,'Intermediate' from dual union all
select 1506,'001506','1965-063A','EGRS V','Secor Type I S/N 2',11813,'In Earth orbit','','1965-09-25 00:00:00',122.22,1136,2424,69.24,'Medium' from dual union all
select 15060,'015060','1984-064A','Kosmos-1575','Resurs-F1 17F41 No. 48',57118,'Landed','1984-07-07 00:00:00','1984-06-29 00:00:00',89.79,256,272,82.33,'Intermediate' from dual union all
select 15061,'015061','1984-064B','11S510','11S510',57118,'Reentered','1984-06-28 00:00:00','1984-06-23 00:00:00',89.17,212,255,82.3,'Intermediate' from dual union all
select 15062,'015062','1984-054N','KDU part','KDU part',57108,'Reentered','1984-08-22 00:00:00','1984-07-01 00:00:00',91.92,343,395,72.84,'Intermediate' from dual union all
select 15063,'015063','1984-065A','USA 2','HEXAGON SV-19',57119,'Deorbited','1984-10-18 00:00:00','1984-06-27 00:00:00',88.8,169,262,96.4,'Sun-Synch' from dual union all
select 15064,'015064','1984-065B','Titan 34D-4 Stage 2','LR91AJ11 34D-4',57119,'Reentered','1984-06-27 00:00:00','1984-06-26 00:00:00',88.55,163,243,96.4,'Sun-Synch' from dual union all
select 15065,'015065','1984-058D','Blenda','Sunshade',57112,'Reentered','1984-10-20 00:00:00','1984-07-11 00:00:00',91.84,329,401,70.02,'Intermediate' from dual union all
select 15066,'015066','1984-058E','KDU part','KDU part',57112,'Reentered','1984-11-25 00:00:00','1984-07-11 00:00:00',94.16,376,579,70.02,'Intermediate' from dual union all
select 15067,'015067','1984-058F','KDU','KDU',57112,'Reentered','1986-04-17 00:00:00','1984-06-27 00:00:00',92.15,347,413,70,'Intermediate' from dual union all
select 15068,'015068','1984-058G','KDU part','KDU part',57112,'Reentered','1984-07-01 00:00:00','1984-06-26 00:00:00',92.26,349,422,70.05,'Intermediate' from dual union all
select 15069,'015069','1984-058H','KDU part','KDU part',57112,'Reentered','1984-09-22 00:00:00','1984-07-11 00:00:00',91.97,343,400,70.02,'Intermediate' from dual union all
select 1507,'001507','1965-066B','8A92E','8A92E',11816,'Reentered','1965-09-03 00:00:00','1965-08-14 00:00:00',89.78,228,300,69,'Intermediate' from dual union all
select 15070,'015070','1984-066A','Kosmos-1576','Kobal''t No. 810',57120,'Landed','1984-08-24 00:00:00','1984-07-03 00:00:00',89.76,184,342,67.15,'Intermediate' from dual union all
select 15071,'015071','1984-065C','USA 3','P-11 No. 4434',57119,'In Earth orbit','','1995-08-20 00:00:00',97.89,650,666,95.91,'Sun-Synch' from dual union all
select 15072,'015072','1984-058J','KDU part','KDU part',57112,'Reentered','1984-09-08 00:00:00','1984-07-11 00:00:00',94.51,374,616,70.02,'Intermediate' from dual union all
select 15073,'015073','1984-066B','11S510','11S510',57120,'Reentered','1984-06-29 00:00:00','1984-06-26 00:00:00',89.61,169,341,67.1,'Intermediate' from dual union all
select 15074,'015074','1984-066C','Kosmos-1576 part','Kobal''t AO part',57120,'Reentered','1984-07-01 00:00:00','1984-06-27 00:00:00',88.48,149,249,67.12,'Intermediate' from dual union all
select 15075,'015075','1984-063D','SOZ','SOZ',57117,'Reentered','1986-10-27 00:00:00','1984-08-07 00:00:00',617.75,258,35040,47.1,'GEO Transfer' from dual union all
select 15076,'015076','1984-063E','SOZ','SOZ',57117,'Reentered','1999-10-03 00:00:00','1984-08-04 00:00:00',619.1,316,35052,47.19,'GEO Transfer' from dual union all
select 15077,'015077','1984-067A','Kosmos-1577','Parus',57121,'In Earth orbit','','1984-08-12 00:00:00',104.78,957,1012,82.96,'Intermediate' from dual union all
select 15078,'015078','1984-067B','S3M','S3M',57121,'In Earth orbit','','1991-03-25 00:00:00',104.63,960,995,82.96,'Intermediate' from dual union all
select 15079,'015079','1984-061C','Blenda','Sunshade',57115,'Reentered','1984-07-01 00:00:00','1984-06-28 00:00:00',89.12,225,237,72.87,'Intermediate' from dual union all
select 1508,'001508','1965-065B','Ablestar 020','AJ10-104D 020',11815,'In Earth orbit','','1965-10-27 00:00:00',107.86,1095,1161,90.03,'Polar' from dual union all
select 15080,'015080','1984-068A','Kosmos-1578','Taifun-1B',57122,'Reentered','1993-01-10 00:00:00','1984-08-02 00:00:00',104.38,295,1636,50.7,'Intermediate' from dual union all
select 15081,'015081','1984-068B','S3M','S3M',57122,'Reentered','1990-04-03 00:00:00','1990-02-01 00:00:00',92.79,245,577,50.64,'Intermediate' from dual union all
select 15082,'015082','1984-061D','KDU','KDU',57115,'Reentered','1984-07-15 00:00:00','1984-06-28 00:00:00',89.28,224,254,72.87,'Intermediate' from dual union all
select 15083,'015083','1984-061E','KDU part','KDU part',57115,'Reentered','1984-06-30 00:00:00','1984-06-28 00:00:00',89.17,219,247,72.87,'Intermediate' from dual union all
select 15084,'015084','1984-061F','KDU part','KDU part',57115,'Reentered','1984-06-30 00:00:00','1984-06-28 00:00:00',89.12,202,260,72.86,'Intermediate' from dual union all
select 15085,'015085','1984-069A','Kosmos-1579','US-A reactor',57123,'In Earth orbit','','1984-08-06 00:00:00',89.63,250,262,65,'Intermediate' from dual union all
select 15086,'015086','1984-061G','KDU part','KDU part',57115,'Reentered','1984-07-05 00:00:00','1984-06-30 00:00:00',90.95,238,405,72.87,'Intermediate' from dual union all
select 15087,'015087','1984-060C','KDU','KDU',57114,'Reentered','1984-07-30 00:00:00','1984-06-29 00:00:00',89.78,257,270,82.35,'Intermediate' from dual union all
select 15088,'015088','1984-060D','Blenda','Sunshade',57114,'Reentered','1984-07-09 00:00:00','1984-06-29 00:00:00',89.65,245,270,82.35,'Intermediate' from dual union all
select 15089,'015089','1984-060E','KDU part','KDU part',57114,'Reentered','1984-07-13 00:00:00','1984-06-29 00:00:00',91.79,260,465,82.34,'Intermediate' from dual union all
select 1509,'001509','1965-066C','deb Blok-E','',11816,'Reentered','1965-08-17 00:00:00','1965-08-18 00:00:00',89.31,200,281,69,'Intermediate' from dual union all
select 15090,'015090','1984-070A','Kosmos-1580','Oblik No. 793',57124,'Landed','1984-07-13 00:00:00','1984-07-01 00:00:00',89.47,227,270,62.8,'Intermediate' from dual union all
select 15091,'015091','1984-070B','11S510','11S510',57124,'Reentered','1984-07-25 00:00:00','1984-06-30 00:00:00',90.29,242,336,62.8,'Intermediate' from dual union all
select 15092,'015092','1984-070C','Sensor cover','Sensor cover',57124,'Reentered','1984-07-06 00:00:00','1984-07-02 00:00:00',89.43,223,270,62.78,'Intermediate' from dual union all
select 15093,'015093','1984-070D','Sensor cover','Sensor cover',57124,'Reentered','1984-07-05 00:00:00','1984-07-01 00:00:00',90.1,233,325,62.83,'Intermediate' from dual union all
select 15094,'015094','1984-070E','Sensor cover','Sensor cover',57124,'Reentered','1984-07-06 00:00:00','1984-07-01 00:00:00',90.19,234,334,62.78,'Intermediate' from dual union all
select 15095,'015095','1984-071A','Kosmos-1581','Oko',57125,'In Earth orbit','','1984-08-06 00:00:00',717.74,680,39673,62.95,'Molniya' from dual union all
select 15096,'015096','1984-071B','11S510','11S510',57125,'Reentered','1984-08-31 00:00:00','1984-07-04 00:00:00',92.75,211,608,62.8,'Intermediate' from dual union all
select 15097,'015097','1984-071C','BOZ','BOZ',57125,'Reentered','1984-08-10 00:00:00','1984-07-04 00:00:00',92.72,192,624,62.8,'Intermediate' from dual union all
select 15098,'015098','1984-071D','Blok-2BL','Blok-2BL No. 56',57125,'In Earth orbit','','1984-08-27 00:00:00',705.61,714,39039,62.93,'Molniya' from dual union all
select 15099,'015099','1984-072A','Meteor-2','Meteor-2 No. 16',57126,'In Earth orbit','','1984-08-05 00:00:00',104.07,942,959,82.53,'Intermediate' from dual union all
select 151,'000151','61 Omicron 36','deb r/b Ablestar','',7273,'Reentered','1967-12-02 00:00:00','1961-07-31 00:00:00',99.3,476,975,66.99,'Intermediate' from dual union all
select 1510,'001510','1965-065C','Dodecapole 2','Dodecapole 2',11815,'In Earth orbit','','1965-09-21 00:00:00',108.07,1085,1190,90.03,'Polar' from dual union all
select 15100,'015100','1984-072B','S5M','S5M',57126,'In Earth orbit','','1991-03-25 00:00:00',104.04,942,957,82.53,'Intermediate' from dual union all
select 15101,'015101','1984-064C','KDU','KDU',57118,'Reentered','1984-08-10 00:00:00','1984-07-06 00:00:00',89.87,257,279,82.34,'Intermediate' from dual union all
select 15102,'015102','1984-064D','Blenda','Sunshade',57118,'Reentered','1984-07-18 00:00:00','1984-07-06 00:00:00',89.79,253,276,82.34,'Intermediate' from dual union all
select 15103,'015103','1984-064E','KDU part','KDU part',57118,'Reentered','1984-07-12 00:00:00','1984-07-07 00:00:00',91.7,255,461,82.33,'Intermediate' from dual union all
select 15104,'015104','1984-064F','KDU part','KDU part',57118,'Reentered','1984-07-20 00:00:00','1984-07-07 00:00:00',91.81,257,470,82.33,'Intermediate' from dual union all
select 15105,'015105','1982-033BL','Garbage bag','',54342,'Reentered','1984-08-07 00:00:00','1984-07-12 00:00:00',91.08,307,349,51.61,'Intermediate' from dual union all
select 15106,'015106','1982-033BM','Garbage bag','',54342,'Reentered','1984-08-18 00:00:00','1984-07-13 00:00:00',90.99,297,350,51.61,'Intermediate' from dual union all
select 15107,'015107','1982-033BN','Solar panel cover','Solar panel cover',54342,'Reentered','1984-08-14 00:00:00','1984-07-12 00:00:00',91.14,308,353,51.59,'Intermediate' from dual union all
select 15108,'015108','1983-101G','Kosmos-1501 SS 5','ESO',55858,'Reentered','1985-03-01 00:00:00','1984-07-13 00:00:00',94.15,456,499,83.01,'Intermediate' from dual union all
select 15109,'015109','1982-033BP','Solar panel cover','Solar panel cover',54342,'Reentered','1984-08-16 00:00:00','1984-07-13 00:00:00',91.18,298,367,51.61,'Intermediate' from dual union all
select 1511,'001511','1965-065D','NRL adapter','Adapter',11815,'In Earth orbit','','1966-01-18 00:00:00',108.09,1087,1190,90,'Polar' from dual union all
select 15110,'015110','1983-101H','Kosmos-1501 SS 6','ESO',55858,'Reentered','1985-02-04 00:00:00','1984-07-12 00:00:00',94.17,456,501,82.91,'Intermediate' from dual union all
select 15111,'015111','1982-033BQ','Garbage bag','',54342,'Reentered','1984-07-27 00:00:00','1984-07-13 00:00:00',91.19,311,355,51.59,'Intermediate' from dual union all
select 15112,'015112','1982-033BR','Garbage bag','',54342,'Reentered','1984-07-30 00:00:00','1984-07-13 00:00:00',91.18,307,357,51.59,'Intermediate' from dual union all
select 15113,'015113','1982-033BS','Garbage bag','',54342,'Reentered','1984-08-31 00:00:00','1984-07-13 00:00:00',91.12,307,352,51.59,'Intermediate' from dual union all
select 15114,'015114','1982-033BT','Garbage bag','',54342,'Reentered','1984-08-26 00:00:00','1984-07-13 00:00:00',91.01,307,342,51.59,'Intermediate' from dual union all
select 15115,'015115','1984-070F','Blenda','Sunshade',57124,'Reentered','1984-07-15 00:00:00','1984-07-13 00:00:00',89.35,221,264,62.81,'Intermediate' from dual union all
select 15116,'015116','1984-070G','KDU','KDU',57124,'Reentered','1984-08-01 00:00:00','1984-07-14 00:00:00',89.34,223,261,62.8,'Intermediate' from dual union all
select 15117,'015117','1984-070H','KDU part','KDU part',57124,'Reentered','1984-07-16 00:00:00','1984-07-14 00:00:00',88.93,213,231,62.8,'Intermediate' from dual union all
select 15118,'015118','1984-070J','KDU part','KDU part',57124,'Reentered','1984-07-18 00:00:00','1984-07-14 00:00:00',90.9,236,402,62.8,'Intermediate' from dual union all
select 15119,'015119','1984-073A','Soyuz T-12','Soyuz 7K-ST No. 18L',57127,'Landed','1984-07-29 00:00:00','1984-07-22 00:00:00',91.39,332,354,51.6,'Intermediate' from dual union all
select 1512,'001512','1965-065E','Tempsat 1','Tempsat 1',11815,'In Earth orbit','','1966-01-22 00:00:00',108.1,1085,1193,90.02,'Polar' from dual union all
select 15120,'015120','1984-073B','11S510','11S510',57127,'Reentered','1984-07-20 00:00:00','1984-07-18 00:00:00',88.56,191,216,51.6,'Intermediate' from dual union all
select 15121,'015121','1984-074A','Kosmos-1582','Resurs-F1 17F41 No. 49',57128,'Landed','1984-08-02 00:00:00','1984-07-21 00:00:00',89.81,252,278,82.35,'Intermediate' from dual union all
select 15122,'015122','1984-074B','11S510','11S510',57128,'Reentered','1984-07-27 00:00:00','1984-07-20 00:00:00',89.27,212,265,82.4,'Intermediate' from dual union all
select 15123,'015123','1984-075A','Kosmos-1583','Oblik No. 306',57129,'Landed','1984-08-08 00:00:00','1984-08-06 00:00:00',92.27,357,414,72.89,'Intermediate' from dual union all
select 15124,'015124','1984-075B','11S510','11S510',57129,'Reentered','1984-08-07 00:00:00','1984-07-25 00:00:00',89.96,197,348,72.9,'Intermediate' from dual union all
select 15125,'015125','1984-075C','Sensor cover','Sensor cover',57129,'Reentered','1984-08-01 00:00:00','1984-07-25 00:00:00',89.58,182,326,72.8,'Intermediate' from dual union all
select 15126,'015126','1984-075D','Sensor cover','Sensor cover',57129,'Reentered','1984-07-27 00:00:00','1984-07-25 00:00:00',90.07,192,364,72.9,'Intermediate' from dual union all
select 15127,'015127','1982-033BU','EVA debris','',54342,'Reentered','1984-10-13 00:00:00','1984-08-06 00:00:00',91.36,332,351,51.6,'Intermediate' from dual;
insert into SATELLITE
select 15128,'015128','1982-033BV','EVA debris','',54342,'Reentered','1984-08-09 00:00:00','1984-08-06 00:00:00',89.66,160,356,51.6,'Intermediate' from dual union all
select 15129,'015129','1982-033BW','EVA debris','',54342,'Reentered','1984-10-09 00:00:00','1984-08-06 00:00:00',91.36,330,353,51.62,'Intermediate' from dual union all
select 1513,'001513','1965-067A','OPS 7208','CORONA J-23',11817,'Reentered Attached','1965-10-11 00:00:00','1965-09-18 00:00:00',90.32,200,381,70.02,'Intermediate' from dual union all
select 15130,'015130','1982-033BX','EVA debris','',54342,'Reentered','1984-09-22 00:00:00','1984-08-06 00:00:00',91.3,329,348,51.62,'Intermediate' from dual union all
select 15131,'015131','1984-076A','Kosmos-1584','Oblik No. 307 (Priroda)',57130,'Landed','1984-08-10 00:00:00','1984-08-06 00:00:00',89.81,179,352,82.34,'Intermediate' from dual union all
select 15132,'015132','1984-076B','11S510','11S510',57130,'Reentered','1984-07-29 00:00:00','1984-07-27 00:00:00',88.62,182,231,82.3,'Intermediate' from dual union all
select 15133,'015133','1982-033BY','Garbage bag','',54342,'Reentered','1984-08-18 00:00:00','1984-08-06 00:00:00',90.54,283,319,51.63,'Intermediate' from dual union all
select 15134,'015134','1982-033BZ','Garbage bag','',54342,'Reentered','1984-08-04 00:00:00','1984-07-30 00:00:00',91.47,335,359,51.6,'Intermediate' from dual union all
select 15135,'015135','1982-033CA','Garbage bag','',54342,'Reentered','1984-09-27 00:00:00','1984-08-06 00:00:00',91.19,322,344,51.6,'Intermediate' from dual union all
select 15136,'015136','1984-073C','Soyuz T-12 BO','Bitovoy otsek',57127,'Reentered','1985-08-28 00:00:00','1984-07-29 00:00:00',91.63,341,368,51.6,'Intermediate' from dual union all
select 15137,'015137','1982-033CB','Garbage bag','',54342,'Reentered','1984-10-08 00:00:00','1984-08-06 00:00:00',91.38,331,354,51.6,'Intermediate' from dual union all
select 15138,'015138','1982-033CC','Garbage bag','',54342,'Reentered','1984-08-18 00:00:00','1984-08-07 00:00:00',91.06,308,346,51.62,'Intermediate' from dual union all
select 15139,'015139','1984-028F','Blok DM','11S86 No. 44L',57082,'In Earth orbit','','1993-05-31 00:00:00',1419.67,35385,35544,8.24,'Inclined Drift' from dual union all
select 1514,'001514','1965-065F','NNS O-5','NNS O-5',11815,'In Earth orbit','','1965-09-18 00:00:00',108.11,1087,1192,90.02,'Polar' from dual union all
select 15140,'015140','1984-073D','Soyuz T-12 BO part','Soyuz BO part',57127,'Reentered','1984-08-02 00:00:00','1984-07-29 00:00:00',91.6,334,372,51.62,'Intermediate' from dual union all
select 15141,'015141','1983-066F','Blok DM No. 39L','11S86 No. 39L',55823,'In Earth orbit','','1993-05-31 00:00:00',1475.11,36492,36602,7.78,'Inclined Drift' from dual union all
select 15142,'015142','1984-077A','Kosmos-1585','Kobal''t',57131,'Landed','1984-09-28 00:00:00','1984-08-07 00:00:00',89.2,167,303,64.73,'Intermediate' from dual union all
select 15143,'015143','1984-077B','11S510','11S510',57131,'Reentered','1984-08-03 00:00:00','1984-07-31 00:00:00',89.16,173,292,64.7,'Intermediate' from dual union all
select 15144,'015144','1984-078A','Gorizont','Gorizont No. 20L',57132,'In Earth orbit','','1984-09-01 00:00:00',1436.04,35772,35799,1.43,'Stationary' from dual union all
select 15145,'015145','1984-078B','8S812','8S812',57132,'Reentered','1984-08-03 00:00:00','1984-08-02 00:00:00',88.22,181,192,51.6,'Intermediate' from dual union all
select 15146,'015146','1984-078C','Perekhodnik','Sredniy perekhodnik',57132,'Reentered','1984-08-02 00:00:00','1984-08-02 00:00:00',88.2,182,189,51.6,'Intermediate' from dual union all
select 15147,'015147','1984-079A','Kosmos-1586','Oko',57133,'Reentered','2016-01-06 00:00:00','1984-09-01 00:00:00',717.74,626,39727,63.05,'Molniya' from dual union all
select 15148,'015148','1984-079B','11S510','11S510',57133,'Reentered','1984-09-19 00:00:00','1984-08-03 00:00:00',92.14,213,546,62.8,'Intermediate' from dual union all
select 15149,'015149','1984-079C','BOZ','BOZ',57133,'Reentered','1984-09-07 00:00:00','1984-08-03 00:00:00',92.5,195,600,62.9,'Intermediate' from dual union all
select 1515,'001515','1965-065G','Navspasur Rod','Long Rod',11815,'In Earth orbit','','1965-09-20 00:00:00',108.04,1083,1190,90.02,'Polar' from dual union all
select 15150,'015150','1984-074C','KDU','KDU',57128,'Reentered','1984-09-09 00:00:00','1984-08-06 00:00:00',89.77,257,270,82.35,'Intermediate' from dual union all
select 15151,'015151','1984-074D','Blenda','Sunshade',57128,'Reentered','1984-08-14 00:00:00','1984-08-06 00:00:00',89.46,242,254,82.35,'Intermediate' from dual union all
select 15152,'015152','1984-080A','Himawari 3','GMS 3',57134,'In Earth orbit','','1984-09-07 00:00:00',1436.14,35782,35793,1.76,'Stationary' from dual union all
select 15153,'015153','1984-080B','N-II Stage 2 S/N N-13(F)','AJ10-118FJ',57134,'Reentered','1984-10-11 00:00:00','1984-08-06 00:00:00',102.01,167,1541,30.53,'Intermediate' from dual union all
select 15154,'015154','1984-074E','KDU part','KDU part',57128,'Reentered','1984-08-15 00:00:00','1984-08-07 00:00:00',91.53,253,446,82.35,'Intermediate' from dual union all
select 15155,'015155','1984-074F','KDU part','KDU part',57128,'Reentered','1984-08-18 00:00:00','1984-08-06 00:00:00',91.59,258,448,82.36,'Intermediate' from dual union all
select 15156,'015156','1984-079D','Blok-2BL','Blok-2BL No. 59',57133,'Reentered','2009-11-06 00:00:00','1984-09-01 00:00:00',705.79,619,39143,63.06,'Molniya' from dual union all
select 15157,'015157','1984-080C','Star 37E S/N 40100','Star 37E',57134,'Reentered','1994-04-29 00:00:00','1993-03-30 00:00:00',177.57,173,8000,28.83,'Medium' from dual union all
select 15158,'015158','1984-081A','ECS 2','ECS 2',57135,'In Earth orbit','','1984-09-13 00:00:00',1436.04,35746,35825,.04,'Stationary' from dual union all
select 15159,'015159','1984-081B','Telecom 1A','Telecom 1A',57135,'In Earth orbit','','1984-09-03 00:00:00',1436.09,35778,35795,.01,'Stationary' from dual union all
select 1516,'001516','1965-068A','Gemini V','Gemini SC5',11818,'Landed','1965-08-29 00:00:00','1965-08-23 00:00:00',89.5,170,330,32.59,'Intermediate' from dual union all
select 15160,'015160','1984-078D','SOZ','SOZ',57132,'Reentered','1987-08-12 00:00:00','1984-08-07 00:00:00',635.41,353,35857,47.4,'GEO Transfer' from dual union all
select 15161,'015161','1984-078E','SOZ','SOZ',57132,'Reentered','1986-10-27 00:00:00','1984-08-31 00:00:00',634.11,276,35867,47.28,'GEO Transfer' from dual union all
select 15162,'015162','1984-080D','deb N-2 stage 2','',57134,'Reentered','1984-08-12 00:00:00','1984-08-06 00:00:00',91.69,165,549,30.35,'Intermediate' from dual union all
select 15163,'015163','1984-082A','Kosmos-1587','Oblik No. 294',57136,'Landed','1984-08-31 00:00:00','1984-08-13 00:00:00',90.04,195,358,72.86,'Intermediate' from dual union all
select 15164,'015164','1984-082B','11S510','11S510',57136,'Reentered','1984-08-18 00:00:00','1984-08-07 00:00:00',89.98,197,350,72.9,'Intermediate' from dual union all
select 15165,'015165','1984-081C','Ariane H10 V10','Ariane H10 S/N T12',57135,'Reentered','1990-04-06 00:00:00','1984-08-07 00:00:00',623.15,204,35374,7,'GEO Transfer' from dual union all
select 15166,'015166','1984-081D','Sylda 4400 No. 1','Sylda 4400',57135,'In Earth orbit','','1984-09-27 00:00:00',584.12,266,33272,6.87,'Highly Ellip' from dual union all
select 15167,'015167','1984-083A','Kosmos-1588','US-PM1',57137,'Reentered','1988-02-17 00:00:00','1984-09-06 00:00:00',93.28,427,443,65.03,'Intermediate' from dual union all
select 15168,'015168','1984-083B','11S692','11S692',57137,'Reentered','1984-08-08 00:00:00','1984-08-08 00:00:00',88.3,105,275,65,'Intermediate' from dual union all
select 15169,'015169','1984-075E','Blenda','Sunshade',57129,'Reentered','1985-02-01 00:00:00','1984-08-23 00:00:00',94.18,388,569,72.89,'Intermediate' from dual union all
select 1517,'001517','1965-068B','Titan GT-5 Stage 2','LR91AJ5 GT-5 65-12560',11818,'Reentered','1965-08-24 00:00:00','1965-08-21 00:00:00',89.38,160,327,32.59,'Intermediate' from dual union all
select 15170,'015170','1984-075F','KDU part','KDU part',57129,'Reentered','1984-12-19 00:00:00','1984-08-23 00:00:00',92,342,404,72.89,'Intermediate' from dual union all
select 15171,'015171','1984-084A','Kosmos-1589','Musson No. 15',57138,'In Earth orbit','','1990-10-01 00:00:00',115.92,1492,1503,82.59,'Intermediate' from dual union all
select 15172,'015172','1984-084B','S5M','S5M',57138,'In Earth orbit','','1993-08-06 00:00:00',115.88,1490,1501,82.6,'Intermediate' from dual union all
select 15173,'015173','1984-075G','KDU','KDU',57129,'Reentered','1986-11-30 00:00:00','1984-08-23 00:00:00',92.22,354,413,72.89,'Intermediate' from dual union all
select 15174,'015174','1982-033CD','EVA debris','',54342,'Reentered','1984-09-14 00:00:00','1984-08-09 00:00:00',91.62,339,369,51.61,'Intermediate' from dual union all
select 15175,'015175','1982-033CE','EVA debris','',54342,'Reentered','1984-08-31 00:00:00','1984-08-22 00:00:00',90.94,312,330,51.58,'Intermediate' from dual union all
select 15176,'015176','1982-033CF','EVA debris','',54342,'Reentered','1984-12-06 00:00:00','1984-08-08 00:00:00',91.64,340,369,51.61,'Intermediate' from dual union all
select 15177,'015177','1982-033CG','EVA debris','',54342,'Reentered','1984-09-22 00:00:00','1984-08-09 00:00:00',91.63,340,369,51.6,'Intermediate' from dual union all
select 15178,'015178','1984-075H','KDU part','KDU part',57129,'Reentered','1984-10-26 00:00:00','1984-08-23 00:00:00',92.12,351,406,72.89,'Intermediate' from dual union all
select 15179,'015179','1984-075J','KDU part','KDU part',57129,'Reentered','1984-10-25 00:00:00','1984-08-23 00:00:00',94.4,388,591,72.9,'Intermediate' from dual union all
select 1518,'001518','1965-068C','REP','Rendezvous Evaluation Po',11818,'Reentered','1965-08-27 00:00:00','1965-08-21 00:00:00',89.57,169,338,32.59,'Intermediate' from dual union all
select 15180,'015180','1984-075K','KDU part','KDU part',57129,'Reentered','1984-09-07 00:00:00','1984-08-10 00:00:00',92.75,332,486,72.87,'Intermediate' from dual union all
select 15181,'015181','1984-078F','Blok DM No. 48L','11S86 No. 48L',57132,'In Earth orbit','','1993-06-07 00:00:00',1436.24,35697,35882,6.52,'Inclined GEO' from dual union all
select 15182,'015182','1984-085A','Molniya-1','Molniya-1 No. 53',57139,'Reentered','2002-12-31 00:00:00','1984-09-09 00:00:00',735.31,470,40746,62.85,'Molniya' from dual union all
select 15183,'015183','1984-085B','11S510','11S510',57139,'Reentered','1984-09-11 00:00:00','1984-08-11 00:00:00',91.13,211,449,62.8,'Intermediate' from dual union all
select 15184,'015184','1984-085C','BOZ','BOZ',57139,'Reentered','1984-09-05 00:00:00','1984-08-11 00:00:00',90.99,200,446,62.8,'Intermediate' from dual union all
select 15185,'015185','1984-076C','KDU','KDU',57130,'Reentered','1984-08-25 00:00:00','1984-08-10 00:00:00',89.7,177,343,82.33,'Intermediate' from dual union all
select 15186,'015186','1984-076D','Blenda','Sunshade',57130,'Reentered','1984-08-13 00:00:00','1984-08-10 00:00:00',89.53,178,325,82.33,'Intermediate' from dual union all
select 15187,'015187','1984-076E','KDU part','KDU part',57130,'Reentered','1984-08-11 00:00:00','1984-08-10 00:00:00',89.48,175,323,82.34,'Intermediate' from dual union all
select 15188,'015188','1984-085D','Blok-ML','Blok-ML No. 165',57139,'Reentered','2002-12-20 00:00:00','1984-09-09 00:00:00',731.02,476,40530,62.83,'Molniya' from dual union all
select 15189,'015189','1982-033CH','Garbage bag','',54342,'Reentered','1984-10-13 00:00:00','1984-08-11 00:00:00',91.63,342,367,51.6,'Intermediate' from dual union all
select 1519,'001519','1965-068D','REP Cover','',11818,'Reentered','1965-08-26 00:00:00','1965-08-21 00:00:00',89.38,160,327,32.59,'Intermediate' from dual union all
select 15190,'015190','1982-033CJ','Garbage bag','',54342,'Reentered','1984-10-18 00:00:00','1984-08-11 00:00:00',91.6,340,367,51.6,'Intermediate' from dual union all
select 15191,'015191','1984-076F','KDU part','KDU part',57130,'Reentered','1984-08-14 00:00:00','1984-08-11 00:00:00',91.26,168,505,82.37,'Intermediate' from dual union all
select 15192,'015192','1982-033CK','Garbage bag','',54342,'Reentered','1986-05-21 00:00:00','1983-07-11 00:00:00',89.63,246,266,65.11,'Intermediate' from dual union all
select 15193,'015193','1984-086A','Progress-23','Progress 7K-TG No. 124',57140,'Deorbited','1984-08-28 00:00:00','1984-08-16 00:00:00',91.03,293,358,51.63,'Intermediate' from dual union all
select 15194,'015194','1984-086B','11S510','11S510',57140,'Reentered','1984-08-17 00:00:00','1984-08-15 00:00:00',88.51,178,224,51.6,'Intermediate' from dual union all
select 15195,'015195','1982-033CL','Garbage bag','',54342,'Reentered','1984-11-12 00:00:00','1984-08-14 00:00:00',91.62,340,368,51.6,'Intermediate' from dual union all
select 15196,'015196','1982-033CM','Garbage bag','',54342,'Reentered','1984-10-04 00:00:00','1984-08-14 00:00:00',91.58,341,364,51.6,'Intermediate' from dual union all
select 15197,'015197','1984-087A','Kosmos-1590','Resurs-F1 17F41 No. 50',57141,'Landed','1984-08-30 00:00:00','1984-08-18 00:00:00',89.81,261,270,82.36,'Intermediate' from dual union all
select 15198,'015198','1984-087B','11S510','11S510',57141,'Reentered','1984-08-23 00:00:00','1984-08-16 00:00:00',89.15,211,254,82.3,'Intermediate' from dual union all
select 15199,'015199','1984-088A','Charge Composition Explorer','AMPTE-CCE',57142,'In Earth orbit','','1984-09-18 00:00:00',939.24,1108,49675,4.84,'Highly Ellip' from dual union all
select 152,'000152','61 Omicron 37','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-16 00:00:00',103.4,785,1054,66.58,'Intermediate' from dual union all
select 1520,'001520','1965-065H','Calsphere','Calsphere',11815,'In Earth orbit','','1966-01-03 00:00:00',108.1,1083,1194,90.04,'Polar' from dual union all
select 15200,'015200','1984-088B','Ion Release Module','AMPTE-IRM',57142,'Reentered','1987-12-08 00:00:00','1985-01-06 00:00:00',2655.63,896,113396,27.73,'Very High' from dual union all
select 15201,'015201','1984-088C','UK Subsatellite','AMPTE-UKS',57142,'Reentered','1988-12-08 00:00:00','1985-02-02 00:00:00',2659.58,1127,113291,27.79,'Very High' from dual union all
select 15202,'015202','1984-088D','Delta 175','Delta SSPS AJ10-118K',57142,'In Earth orbit','','1984-09-16 00:00:00',134.24,544,4069,28.9,'Medium' from dual union all
select 15203,'015203','1983-101J','Kosmos-1501 SS 7','ESO',55858,'Reentered','1986-07-29 00:00:00','1984-08-17 00:00:00',94.14,456,498,82.93,'Intermediate' from dual union all
select 15204,'015204','1983-101K','Kosmos-1501 SS 8','ESO',55858,'Reentered','1986-09-04 00:00:00','1984-08-17 00:00:00',94.17,459,497,82.93,'Intermediate' from dual union all
select 15205,'015205','1984-088E','Delta 175 part','',57142,'In Earth orbit','','1984-09-16 00:00:00',133.63,549,4011,28.72,'Medium' from dual union all
select 15206,'015206','1984-088F','Star 37E S/N 40099','Star 37E',57142,'In Earth orbit','','1984-09-22 00:00:00',925.9,552,49628,28.57,'Highly Ellip' from dual union all
select 15207,'015207','1982-033CN','Garbage bag','',54342,'Reentered','1984-10-03 00:00:00','1984-08-14 00:00:00',91.57,338,366,51.6,'Intermediate' from dual union all
select 15208,'015208','1982-033CP','Garbage bag','',54342,'Reentered','1984-12-05 00:00:00','1984-08-14 00:00:00',91.6,338,369,51.6,'Intermediate' from dual union all
select 15209,'015209','1982-033CQ','Garbage bag','',54342,'Reentered','1984-10-25 00:00:00','1984-08-20 00:00:00',91.51,336,362,51.6,'Intermediate' from dual union all
select 1521,'001521','1965-065J','NRL adapter','Adapter',11815,'In Earth orbit','','1966-01-21 00:00:00',108.09,1090,1186,90.02,'Polar' from dual union all
select 15210,'015210','1982-033CR','Garbage bag','',54342,'Reentered','1984-09-15 00:00:00','1984-08-20 00:00:00',91.32,330,348,51.6,'Intermediate' from dual union all
select 15211,'015211','1982-033CS','Garbage bag','',54342,'Reentered','1984-10-06 00:00:00','1984-08-20 00:00:00',91.45,335,357,51.61,'Intermediate' from dual union all
select 15212,'015212','1982-033CT','Garbage bag','',54342,'Reentered','1984-09-21 00:00:00','1984-08-20 00:00:00',91.4,336,351,51.6,'Intermediate' from dual union all
select 15213,'015213','1982-033CU','Garbage bag','',54342,'Reentered','1984-10-08 00:00:00','1984-08-23 00:00:00',91.58,269,436,51.6,'Intermediate' from dual union all
select 15214,'015214','1984-089A','Molniya-1','Molniya-1 No. 54',57143,'In Earth orbit','','1984-09-23 00:00:00',717.72,447,39905,62.87,'Molniya' from dual union all
select 15215,'015215','1984-089B','11S510','11S510',57143,'Reentered','1984-09-17 00:00:00','1984-08-24 00:00:00',90.88,209,427,62.8,'Intermediate' from dual union all
select 15216,'015216','1984-089C','BOZ','BOZ',57143,'Reentered','1984-09-13 00:00:00','1984-08-24 00:00:00',90.99,196,451,62.8,'Intermediate' from dual union all
select 15217,'015217','1982-033CV','Solar panel part','Unknown 39 sq m piece',54342,'Reentered','1984-09-08 00:00:00','1984-08-24 00:00:00',91.79,300,425,51.53,'Intermediate' from dual;
insert into SATELLITE
select 15218,'015218','1982-033CW','Garbage bag','',54342,'Reentered','1984-09-19 00:00:00','1984-08-07 00:00:00',91.32,296,383,51.61,'Intermediate' from dual union all
select 15219,'015219','1984-090A','Ekran','Ekran No. 27L',57144,'In Earth orbit','','1984-09-30 00:00:00',1435.98,35767,35802,.39,'Stationary' from dual union all
select 1522,'001522','1965-065L','deb NNS O-5','',11815,'In Earth orbit','','1965-09-17 00:00:00',108.11,1087,1192,90.04,'Polar' from dual union all
select 15220,'015220','1984-090B','8S812','8S812',57144,'Reentered','1984-08-27 00:00:00','1984-08-25 00:00:00',88.21,181,191,51.6,'Intermediate' from dual union all
select 15221,'015221','1984-090C','Perekhodnik','Sredniy perekhodnik',57144,'Reentered','1984-08-25 00:00:00','1984-08-25 00:00:00',88.17,180,188,51.6,'Intermediate' from dual union all
select 15222,'015222','1984-066D','Kosmos-1576 part','Solar panel',57120,'Reentered','1984-08-25 00:00:00','1984-08-25 00:00:00',87.74,151,174,67.14,'Intermediate' from dual union all
select 15223,'015223','1984-089D','Blok-ML','Blok-ML No. 323',57143,'In Earth orbit','','1984-10-01 00:00:00',739.07,448,40952,62.84,'Molniya' from dual union all
select 15224,'015224','1984-090D','SOZ','SOZ',57144,'Reentered','1985-04-18 00:00:00','1984-08-29 00:00:00',628.28,284,35559,47.4,'GEO Transfer' from dual union all
select 15225,'015225','1984-090E','SOZ','SOZ',57144,'Reentered','1986-10-27 00:00:00','1984-09-25 00:00:00',628.9,309,35565,47.35,'GEO Transfer' from dual union all
select 15226,'015226','1984-091A','USA 4','QUASAR 5 (or JUMPSEAT 7)',57145,'In Earth orbit','','1984-09-05 00:00:00',704.42,379,39315,63.3,'Molniya' from dual union all
select 15227,'015227','1984-091B','Agena D','Agena D',57145,'In Earth orbit','','1984-09-05 00:00:00',704.42,379,39315,63.3,'Molniya' from dual union all
select 15228,'015228','1984-087C','Blenda','Sunshade',57141,'Reentered','1984-09-12 00:00:00','1984-08-30 00:00:00',91.89,260,475,82.35,'Intermediate' from dual union all
select 15229,'015229','1984-087D','KDU part','KDU part',57141,'Reentered','1984-09-09 00:00:00','1984-08-30 00:00:00',89.82,257,275,82.34,'Intermediate' from dual union all
select 1523,'001523','1965-069A','Kosmos-79','Zenit-4 No. 7',11819,'Landed','1965-09-02 00:00:00','1965-08-28 00:00:00',89.94,205,338,64.9,'Intermediate' from dual union all
select 15230,'015230','1984-087E','KDU part','KDU part',57141,'Reentered','1984-09-05 00:00:00','1984-08-30 00:00:00',92.25,259,510,82.36,'Intermediate' from dual union all
select 15231,'015231','1984-087F','KDU','KDU',57141,'Reentered','1984-10-02 00:00:00','1984-08-30 00:00:00',89.83,258,275,82.35,'Intermediate' from dual union all
select 15232,'015232','1984-092A','Kosmos-1591','Resurs-F1 17F41 No. 51',57146,'Landed','1984-09-13 00:00:00','1984-09-01 00:00:00',89.81,259,271,82.33,'Intermediate' from dual union all
select 15233,'015233','1984-092B','11S510','11S510',57146,'Reentered','1984-09-05 00:00:00','1984-08-30 00:00:00',89.22,209,263,82.3,'Intermediate' from dual union all
select 15234,'015234','1984-093A','Discovery (STS 41-D)','OV-103',57147,'Landed','1984-09-05 00:00:00','1984-09-01 00:00:00',90.77,296,329,28.46,'Intermediate' from dual union all
select 15235,'015235','1984-093B','SBS 4','SBS D',57147,'In Earth orbit','','1984-09-29 00:00:00',1436.06,35784,35788,.5,'Stationary' from dual union all
select 15236,'015236','1984-093C','Leasat 2','Leasat 2',57147,'In Earth orbit','','1984-09-29 00:00:00',1436.06,35776,35796,3.41,'Inclined GEO' from dual union all
select 15237,'015237','1984-093D','Telstar 302','Telstar 3C',57147,'In Earth orbit','','1984-10-02 00:00:00',1436.44,35427,36160,.16,'Stationary' from dual union all
select 15238,'015238','1984-082C','Blenda','Sunshade',57136,'Reentered','1984-11-18 00:00:00','1984-09-05 00:00:00',92.22,353,414,72.87,'Intermediate' from dual union all
select 15239,'015239','1984-082D','KDU part','KDU part',57136,'Reentered','1984-09-15 00:00:00','1984-08-31 00:00:00',92.67,359,451,72.89,'Intermediate' from dual union all
select 1524,'001524','1965-069B','11A57I','11A57I',11819,'Reentered','1965-09-07 00:00:00','1965-08-27 00:00:00',89.6,199,311,64.9,'Intermediate' from dual union all
select 15240,'015240','1984-082E','KDU part','KDU part',57136,'Reentered','1984-12-27 00:00:00','1984-09-08 00:00:00',94.71,390,620,72.89,'Intermediate' from dual union all
select 15241,'015241','1984-082F','KDU part','KDU part',57136,'Reentered','1985-01-26 00:00:00','1984-09-06 00:00:00',92.05,342,409,72.87,'Intermediate' from dual union all
select 15242,'015242','1984-082G','KDU','KDU',57136,'Reentered','1986-11-26 00:00:00','1984-09-05 00:00:00',92.24,355,414,72.87,'Intermediate' from dual union all
select 15243,'015243','1984-082H','KDU part','KDU part',57136,'Reentered','1985-03-06 00:00:00','1984-09-06 00:00:00',94.25,388,577,72.87,'Intermediate' from dual union all
select 15244,'015244','1984-093E','Leasat PKM','TU-882',57147,'In Earth orbit','','1984-10-01 00:00:00',278.68,314,15196,27.23,'Highly Ellip' from dual union all
select 15245,'015245','1984-093F','PAM-D','Star 48  S/N 10035-3',57147,'In Earth orbit','','1984-10-13 00:00:00',629.93,318,35610,22.95,'GEO Transfer' from dual union all
select 15246,'015246','1984-093G','PAM-D','Star 48  S/N 10036-3',57147,'In Earth orbit','','1984-09-29 00:00:00',669.48,328,37617,25.22,'GEO Transfer' from dual union all
select 15247,'015247','1982-033CX','Garbage bag','Long lived object',54342,'Reentered','1986-05-13 00:00:00','1984-09-02 00:00:00',91.96,369,373,51.64,'Intermediate' from dual union all
select 15248,'015248','1982-033CY','Garbage bag','',54342,'Reentered','1984-12-12 00:00:00','1984-09-01 00:00:00',92.03,373,376,51.65,'Intermediate' from dual union all
select 15249,'015249','1982-033CZ','Garbage bag','',54342,'Reentered','1984-12-26 00:00:00','1984-09-15 00:00:00',91.85,351,380,51.63,'Intermediate' from dual union all
select 1525,'001525','1965-069C','deb Kosmos-79','',11819,'Reentered','1965-08-26 00:00:00','1965-08-26 00:00:00',89.28,204,274,64.9,'Intermediate' from dual union all
select 15250,'015250','1982-033DA','Garbage bag','',54342,'Reentered','1984-12-12 00:00:00','1984-09-01 00:00:00',92.25,324,446,51.64,'Intermediate' from dual union all
select 15251,'015251','1982-033DB','Garbage bag','',54342,'Reentered','1984-10-16 00:00:00','1984-09-08 00:00:00',92.31,255,521,51.63,'Intermediate' from dual union all
select 15252,'015252','1982-033DC','Garbage bag','',54342,'Reentered','1984-12-08 00:00:00','1984-09-04 00:00:00',91.9,364,372,51.65,'Intermediate' from dual union all
select 15253,'015253','1982-033DD','Garbage bag','',54342,'Reentered','1984-09-25 00:00:00','1984-09-01 00:00:00',91.97,371,372,51.62,'Intermediate' from dual union all
select 15254,'015254','1982-033DE','Garbage bag','',54342,'Reentered','1984-11-10 00:00:00','1984-09-01 00:00:00',91.44,334,357,51.6,'Intermediate' from dual union all
select 15255,'015255','1982-033DF','Garbage bag','',54342,'Reentered','1984-11-01 00:00:00','1984-09-01 00:00:00',92,373,373,51.65,'Intermediate' from dual union all
select 15256,'015256','1982-033DG','Garbage bag','',54342,'Reentered','1984-12-08 00:00:00','1984-09-03 00:00:00',91.59,335,371,51.6,'Intermediate' from dual union all
select 15257,'015257','1984-094A','Kosmos-1592','Oblik No. 794',57148,'Landed','1984-09-18 00:00:00','1984-09-06 00:00:00',89.63,225,287,72.88,'Intermediate' from dual union all
select 15258,'015258','1984-094B','11S510','11S510',57148,'Reentered','1984-09-15 00:00:00','1984-09-04 00:00:00',89.97,198,347,72.9,'Intermediate' from dual union all
select 15259,'015259','1984-095A','Kosmos-1593','Uragan No. 18L',57149,'In Earth orbit','','1984-10-11 00:00:00',675.71,19090,19169,64.79,'Medium' from dual union all
select 1526,'001526','1965-069D','deb Kosmos-79','',11819,'Reentered','1965-08-31 00:00:00','1965-08-26 00:00:00',89.87,191,345,64.91,'Intermediate' from dual union all
select 15260,'015260','1984-095B','Kosmos-1594','Uragan No. 19L',57149,'In Earth orbit','','1984-10-13 00:00:00',677.18,19139,19194,64.75,'Medium' from dual union all
select 15261,'015261','1984-095C','Kosmos-1595','Uragan GVM',57149,'In Earth orbit','','1984-10-09 00:00:00',675.69,19078,19180,64.72,'Medium' from dual union all
select 15262,'015262','1984-095D','8S812','8S812',57149,'Reentered','1984-09-06 00:00:00','1984-09-05 00:00:00',88.03,173,181,51.6,'Intermediate' from dual union all
select 15263,'015263','1984-095E','Perekhodnik','Sredniy perekhodnik',57149,'Reentered','1984-09-05 00:00:00','1984-09-05 00:00:00',88,173,178,51.6,'Intermediate' from dual union all
select 15264,'015264','1984-095F','Blok DM2','11S861 No. 5L',57149,'In Earth orbit','','1985-10-20 00:00:00',675.89,19084,19184,64.73,'Medium' from dual union all
select 15265,'015265','1984-095G','SOZ','SOZ',57149,'In Earth orbit','','1984-10-07 00:00:00',340.2,377,19153,52.07,'Highly Ellip' from dual union all
select 15266,'015266','1984-095H','SOZ','SOZ',57149,'In Earth orbit','','1984-10-08 00:00:00',340.55,397,19155,51.97,'Highly Ellip' from dual union all
select 15267,'015267','1984-096A','Kosmos-1596','Oko',57150,'In Earth orbit','','1984-10-07 00:00:00',717.84,654,39704,62.93,'Molniya' from dual union all
select 15268,'015268','1984-096B','11S510','11S510',57150,'Reentered','1984-10-25 00:00:00','1984-09-07 00:00:00',92.19,211,553,62.8,'Intermediate' from dual union all
select 15269,'015269','1984-096C','BOZ','BOZ',57150,'Reentered','1984-10-09 00:00:00','1984-09-08 00:00:00',92.45,189,601,62.8,'Intermediate' from dual union all
select 1527,'001527','1965-069E','deb Kosmos-79','',11819,'Reentered','1965-08-31 00:00:00','1965-08-26 00:00:00',89.6,195,315,64.89,'Intermediate' from dual union all
select 15270,'015270','1984-096D','Blok-2BL','Blok-2BL No. 51',57150,'In Earth orbit','','1984-10-07 00:00:00',703.23,652,38983,62.96,'Molniya' from dual union all
select 15271,'015271','1984-097A','GPS 10','Navstar GPS SVN 10',57151,'In Earth orbit','','1984-10-08 00:00:00',717.98,19961,20404,63.27,'Medium' from dual union all
select 15272,'015272','1984-097B','SGS-II','Star 48  S/N 10023-9',57151,'In Earth orbit','','1984-10-08 00:00:00',369.21,555,20785,63.19,'Highly Ellip' from dual union all
select 15273,'015273','1982-033DH','Garbage bag','',54342,'Reentered','1984-10-03 00:00:00','1984-09-09 00:00:00',92.04,367,382,51.62,'Intermediate' from dual union all
select 15274,'015274','1982-033DJ','Garbage bag','',54342,'Reentered','1984-12-24 00:00:00','1984-09-09 00:00:00',91.9,363,373,51.65,'Intermediate' from dual union all
select 15275,'015275','1982-033DK','Garbage bag','',54342,'Reentered','1984-12-11 00:00:00','1984-09-09 00:00:00',91.9,363,373,51.64,'Intermediate' from dual union all
select 15276,'015276','1982-033DL','Garbage bag','',54342,'Reentered','1984-11-29 00:00:00','1984-09-10 00:00:00',91.96,370,372,51.65,'Intermediate' from dual union all
select 15277,'015277','1983-101L','Kosmos-1501 SS 9','ESO',55858,'Reentered','1986-10-13 00:00:00','1984-09-12 00:00:00',94.16,457,499,82.93,'Intermediate' from dual union all
select 15278,'015278','1983-101M','Kosmos-1501 SS 10','ESO',55858,'Reentered','1986-09-26 00:00:00','1984-09-12 00:00:00',94.12,454,498,82.93,'Intermediate' from dual union all
select 15279,'015279','1984-098A','Fanhui Shi Weixing (06)','Jianbing-1 No. 7',57152,'Reentered','1984-09-29 00:00:00','1984-09-19 00:00:00',89.92,172,369,67.94,'Intermediate' from dual union all
select 1528,'001528','1963-038F','deb Transit VBN-2','[was UNID011] RCS 0.0',9269,'Reentered','1980-01-15 00:00:00','1965-08-30 00:00:00',102.01,817,891,98.71,'Sun-Synch' from dual union all
select 15280,'015280','1984-098B','CZ-2C Stage 2','CZ-2C Stage 2',57152,'Reentered','1984-09-25 00:00:00','1984-09-14 00:00:00',90.01,172,378,67.95,'Intermediate' from dual union all
select 15281,'015281','1982-033DM','Garbage bag','',54342,'Reentered','1985-01-11 00:00:00','1984-09-12 00:00:00',92.1,352,402,51.64,'Intermediate' from dual union all
select 15282,'015282','1982-033DN','Garbage bag','',54342,'Reentered','1985-01-03 00:00:00','1984-09-12 00:00:00',92,372,374,51.64,'Intermediate' from dual union all
select 15283,'015283','1982-033DP','Garbage bag','',54342,'Reentered','1985-02-14 00:00:00','1984-09-12 00:00:00',91.96,367,375,51.65,'Intermediate' from dual union all
select 15284,'015284','1982-033DQ','Garbage bag','',54342,'Reentered','1984-12-04 00:00:00','1984-09-17 00:00:00',91.83,328,400,51.7,'Intermediate' from dual union all
select 15285,'015285','1984-098C','CZ-2C sep motor cover','',57152,'Reentered','1984-09-13 00:00:00','1984-09-12 00:00:00',90.15,174,390,67.91,'Intermediate' from dual union all
select 15286,'015286','1984-098D','CZ-2C sep motor cover','',57152,'Reentered','1984-09-13 00:00:00','1984-09-13 00:00:00',89.26,153,323,68.17,'Intermediate' from dual union all
select 15287,'015287','1984-099A','Kosmos-1597','Fram',57153,'Landed','1984-09-26 00:00:00','1984-09-15 00:00:00',89.02,210,243,82.34,'Intermediate' from dual union all
select 15288,'015288','1984-099B','11S510','11S510',57153,'Reentered','1984-09-19 00:00:00','1984-09-13 00:00:00',89,211,240,82.3,'Intermediate' from dual union all
select 15289,'015289','1984-092C','Blenda','Sunshade',57146,'Reentered','1984-09-15 00:00:00','1984-09-13 00:00:00',89.89,261,277,82.38,'Intermediate' from dual union all
select 1529,'001529','1965-051D','Tiros 10 despin weight','Despin weight',11801,'In Earth orbit','','1968-12-10 00:00:00',101.98,821,884,98.59,'Sun-Synch' from dual union all
select 15290,'015290','1984-092D','KDU','KDU',57146,'Reentered','1984-10-14 00:00:00','1984-09-13 00:00:00',89.87,265,271,82.3,'Intermediate' from dual union all
select 15291,'015291','1984-092E','KDU part','KDU part',57146,'Reentered','1984-09-23 00:00:00','1984-09-13 00:00:00',89.75,257,268,82.29,'Intermediate' from dual union all
select 15292,'015292','1984-100A','Kosmos-1598','Parus',57154,'In Earth orbit','','1985-10-18 00:00:00',104.94,969,1015,82.95,'Intermediate' from dual union all
select 15293,'015293','1984-100B','S3M','S3M',57154,'In Earth orbit','','1991-03-25 00:00:00',104.8,968,1003,82.94,'Intermediate' from dual union all
select 15294,'015294','1984-092F','KDU part','KDU part',57146,'Reentered','1984-09-28 00:00:00','1984-09-14 00:00:00',91.86,266,465,82.32,'Intermediate' from dual union all
select 15295,'015295','1984-092G','KDU part','KDU part',57146,'Reentered','1984-09-29 00:00:00','1984-09-14 00:00:00',91.92,266,472,82.32,'Intermediate' from dual union all
select 15296,'015296','1984-092H','KDU part','KDU part',57146,'Reentered','1984-09-15 00:00:00','1984-09-14 00:00:00',91.42,258,431,82.32,'Intermediate' from dual union all
select 15297,'015297','1982-033DR','Garbage bag','',54342,'Reentered','1984-10-04 00:00:00','1984-09-15 00:00:00',91.96,330,412,51.63,'Intermediate' from dual union all
select 15298,'015298','1984-094C','Blenda','Sunshade',57148,'Reentered','1984-09-26 00:00:00','1984-09-18 00:00:00',89.9,223,316,72.87,'Intermediate' from dual union all
select 15299,'015299','1984-094D','KDU','KDU',57148,'Reentered','1984-10-24 00:00:00','1984-09-18 00:00:00',90.08,224,332,72.88,'Intermediate' from dual union all
select 153,'000153','61 Omicron 38','deb r/b Ablestar','',7273,'Reentered','1981-04-22 00:00:00','1961-09-25 00:00:00',103.25,847,978,67.05,'Intermediate' from dual union all
select 1530,'001530','1965-020CA','deb Blok Kosmos','',11770,'Reentered','1969-07-04 00:00:00','1965-08-30 00:00:00',108.04,391,1882,55.98,'Intermediate' from dual union all
select 15300,'015300','1984-094E','KDU part','KDU part',57148,'Reentered','1984-10-02 00:00:00','1984-09-18 00:00:00',92,258,488,72.88,'Intermediate' from dual union all
select 15301,'015301','1984-094F','KDU part','KDU part',57148,'Reentered','1984-09-23 00:00:00','1984-09-18 00:00:00',92.29,259,514,72.9,'Intermediate' from dual union all
select 15302,'015302','1984-094G','KDU part','KDU part',57148,'Reentered','1984-09-23 00:00:00','1984-09-19 00:00:00',89.99,220,327,72.88,'Intermediate' from dual union all
select 15303,'015303','1982-033DS','Garbage bag','',54342,'Reentered','1984-10-20 00:00:00','1984-09-25 00:00:00',91.55,238,464,51.64,'Intermediate' from dual union all
select 15304,'015304','1982-033DT','Garbage bag','',54342,'Reentered','1984-09-26 00:00:00','1984-09-21 00:00:00',91.88,363,371,51.65,'Intermediate' from dual union all
select 15305,'015305','1982-033DU','Garbage bag','',54342,'Reentered','1985-12-09 00:00:00','1984-09-21 00:00:00',92.01,372,375,51.63,'Intermediate' from dual union all
select 15306,'015306','1982-033DV','Garbage bag','',54342,'Reentered','1984-12-20 00:00:00','1984-09-21 00:00:00',92.07,358,395,51.64,'Intermediate' from dual union all
select 15307,'015307','1982-033DW','Garbage bag','',54342,'Reentered','1985-01-11 00:00:00','1984-09-21 00:00:00',92.04,357,393,51.62,'Intermediate' from dual;
insert into SATELLITE
select 15308,'015308','1984-101A','Galaxy 3','Galaxy III',57155,'In Earth orbit','','1984-10-24 00:00:00',1436.09,35719,35853,.13,'Stationary' from dual union all
select 15309,'015309','1984-101B','Delta 176','Delta SSPS AJ10-118K',57155,'Reentered','1984-11-19 00:00:00','1984-09-22 00:00:00',91.01,270,379,27.1,'Intermediate' from dual union all
select 1531,'001531','1965-020CB','deb Blok Kosmos','',11770,'Reentered','1967-03-05 00:00:00','1965-08-30 00:00:00',105.66,319,1732,56.16,'Intermediate' from dual union all
select 15310,'015310','1984-101C','PAM-D','Star 48  S/N 10032-8',57155,'Reentered','1985-06-12 00:00:00','1984-09-24 00:00:00',651.52,181,36853,23.2,'GEO Transfer' from dual union all
select 15311,'015311','1982-033DX','Garbage bag','',54342,'Reentered','1984-10-13 00:00:00','1984-09-23 00:00:00',91.28,327,348,51.8,'Intermediate' from dual union all
select 15312,'015312','1982-033DY','Garbage bag','',54342,'Reentered','1985-01-11 00:00:00','1984-09-23 00:00:00',91.97,366,377,51.63,'Intermediate' from dual union all
select 15313,'015313','1982-033DZ','Garbage bag','Long lived object',54342,'Reentered','1986-01-11 00:00:00','1984-09-24 00:00:00',91.92,367,370,51.65,'Intermediate' from dual union all
select 15314,'015314','1982-033EA','Garbage bag','',54342,'Reentered','1985-01-10 00:00:00','1984-09-24 00:00:00',91.9,357,379,51.65,'Intermediate' from dual union all
select 15315,'015315','1982-033EB','Garbage bag','',54342,'Reentered','1984-11-25 00:00:00','1984-09-24 00:00:00',92.51,331,465,51.55,'Intermediate' from dual union all
select 15316,'015316','1982-033EC','Garbage bag','',54342,'Reentered','1985-02-11 00:00:00','1984-09-25 00:00:00',91.95,363,378,51.63,'Intermediate' from dual union all
select 15317,'015317','1982-033ED','Garbage bag','',54342,'Reentered','1984-12-08 00:00:00','1984-09-24 00:00:00',92.04,316,434,51.63,'Intermediate' from dual union all
select 15318,'015318','1984-102A','Kosmos-1599','Kobal''t No. 811',57156,'Landed','1984-11-20 00:00:00','1984-10-02 00:00:00',89.26,175,301,67.13,'Intermediate' from dual union all
select 15319,'015319','1984-102B','11S510','11S510',57156,'Reentered','1984-09-27 00:00:00','1984-09-25 00:00:00',88.63,170,244,67.1,'Intermediate' from dual union all
select 1532,'001532','1965-020CC','deb Blok Kosmos','',11770,'Reentered','1966-01-11 00:00:00','1965-08-30 00:00:00',105.01,300,1690,55.96,'Intermediate' from dual union all
select 15320,'015320','1984-099C','KDU','KDU',57153,'Reentered','1984-10-04 00:00:00','1984-09-26 00:00:00',88.75,196,230,82.31,'Intermediate' from dual union all
select 15321,'015321','1984-099D','Blenda','Sunshade',57153,'Reentered','1984-09-27 00:00:00','1984-09-26 00:00:00',88.62,200,213,82.31,'Intermediate' from dual union all
select 15322,'015322','1984-099E','PTDU cover','KDU part',57153,'Reentered','1984-09-29 00:00:00','1984-09-26 00:00:00',90.65,214,399,82.33,'Intermediate' from dual union all
select 15323,'015323','1984-099F','PTDU cover','KDU part',57153,'Reentered','1984-09-28 00:00:00','1984-09-26 00:00:00',90.64,217,395,82.36,'Intermediate' from dual union all
select 15324,'015324','1984-103A','Kosmos-1600','Oblik',57157,'Landed','1984-10-11 00:00:00','1984-09-29 00:00:00',92.18,348,415,69.98,'Intermediate' from dual union all
select 15325,'015325','1984-103B','11S510','11S510',57157,'Reentered','1984-10-14 00:00:00','1984-09-27 00:00:00',90.31,206,374,70,'Intermediate' from dual union all
select 15326,'015326','1984-104A','Kosmos-1601','Romb',57158,'Reentered','1989-11-29 00:00:00','1984-10-27 00:00:00',94.49,472,516,65.84,'Intermediate' from dual union all
select 15327,'015327','1984-069B','Kosmos-1579 RLS','US-A RLS',57123,'Reentered','1984-09-30 00:00:00','1984-09-28 00:00:00',89.12,219,243,65,'Intermediate' from dual union all
select 15328,'015328','1984-069C','Kosmos-1579 DU','US-A DU',57123,'Reentered','1984-11-05 00:00:00','1984-09-28 00:00:00',89.55,243,262,65,'Intermediate' from dual union all
select 15329,'015329','1984-104B','S3M','S3M',57158,'Reentered','1989-09-09 00:00:00','1988-06-12 00:00:00',93.93,438,495,65.82,'Intermediate' from dual union all
select 1533,'001533','1965-020CD','deb Blok Kosmos','',11770,'Reentered','1965-12-27 00:00:00','1965-08-29 00:00:00',99.22,233,1209,56.07,'Intermediate' from dual union all
select 15330,'015330','1984-069D','Kosmos-1579 reactor core','US-A reactor core',57123,'In Earth orbit','','1991-11-15 00:00:00',103.6,903,954,65.05,'Intermediate' from dual union all
select 15331,'015331','1984-105A','Kosmos-1602','Okean-OE NKhM No. 2',57159,'In Earth orbit','','1985-10-18 00:00:00',97.72,631,669,82.53,'Intermediate' from dual union all
select 15332,'015332','1984-105B','S5M','S5M',57159,'In Earth orbit','','1991-03-27 00:00:00',97.53,623,657,82.53,'Intermediate' from dual union all
select 15333,'015333','1984-106A','Kosmos-1603','Tselina-2',57160,'In Earth orbit','','1984-10-29 00:00:00',101.97,850,854,71.01,'Intermediate' from dual union all
select 15334,'015334','1984-106B','Blok DM2','11S861 No. 6L',57160,'In Earth orbit','','1985-10-25 00:00:00',101.97,846,857,71,'Intermediate' from dual union all
select 15335,'015335','1984-106C','SOZ','SOZ',57160,'In Earth orbit','','1991-04-27 00:00:00',101.47,815,842,66.55,'Intermediate' from dual union all
select 15336,'015336','1984-106D','8S812','8S812',57160,'Reentered','1984-10-01 00:00:00','1984-09-28 00:00:00',88.28,187,192,51.6,'Intermediate' from dual union all
select 15337,'015337','1984-106E','Perekhodnik','Sredniy perekhodnik',57160,'Reentered','1984-09-29 00:00:00','1984-09-28 00:00:00',88.2,179,192,51.6,'Intermediate' from dual union all
select 15338,'015338','1984-106F','SOZ','SOZ',57160,'In Earth orbit','','1984-10-28 00:00:00',101.81,836,853,66.56,'Intermediate' from dual union all
select 15339,'015339','61 Omicron 268','deb r/b Ablestar','',7273,'Reentered','1987-12-07 00:00:00','1984-12-26 00:00:00',98.98,615,805,66.7,'Intermediate' from dual union all
select 1534,'001534','1965-020CE','deb Blok Kosmos','',11770,'Reentered','1971-05-05 00:00:00','1965-09-16 00:00:00',106,297,1786,56.11,'Intermediate' from dual union all
select 15340,'015340','1965-027R','deb Snapshot','',11777,'In Earth orbit','','1993-08-06 00:00:00',111.31,1265,1309,90.29,'Polar' from dual union all
select 15341,'015341','1966-005K','NNS O-7 insulation','',13412,'Reentered','1989-12-10 00:00:00','1987-08-13 00:00:00',104.73,840,1124,89.8,'Polar' from dual union all
select 15342,'015342','1971-015CT','deb Kosmos-397','',24833,'Reentered','1992-12-23 00:00:00','1992-09-30 00:00:00',94.66,434,571,66.53,'Intermediate' from dual union all
select 15343,'015343','1971-015CU','deb Kosmos-397','',24833,'Reentered','1989-09-17 00:00:00','1987-08-18 00:00:00',100.68,530,1052,65.8,'Intermediate' from dual union all
select 15344,'015344','1971-120F','deb Meteor 10','',24938,'Reentered','1998-05-28 00:00:00','1984-10-01 00:00:00',101.71,821,859,81.26,'Intermediate' from dual union all
select 15345,'015345','1984-032C','Soyuz T-11 BO','Bitovoy otsek',57086,'Reentered','1986-12-12 00:00:00','1984-10-04 00:00:00',91.95,368,373,51.6,'Intermediate' from dual union all
select 15346,'015346','1984-032D','Soyuz T-11 BO part','Soyuz BO part',57086,'Reentered','1984-12-05 00:00:00','1984-10-04 00:00:00',91.91,366,370,51.6,'Intermediate' from dual union all
select 15347,'015347','1984-012J','deb NOSS','',57066,'In Earth orbit','','1984-10-01 00:00:00',107.15,830,1359,63.44,'Intermediate' from dual union all
select 15348,'015348','1984-012K','deb NOSS','',57066,'In Earth orbit','','1984-10-01 00:00:00',107.15,830,1359,63.44,'Intermediate' from dual union all
select 15349,'015349','1984-012L','deb NOSS','',57066,'In Earth orbit','','1984-10-01 00:00:00',107.15,830,1359,63.44,'Intermediate' from dual union all
select 1535,'001535','1965-020CF','deb Blok Kosmos','',11770,'Reentered','1965-12-14 00:00:00','1965-08-29 00:00:00',100.64,277,1301,56.11,'Intermediate' from dual union all
select 15350,'015350','1984-107A','Kosmos-1604','Oko',57161,'In Earth orbit','','1984-11-03 00:00:00',717.74,626,39727,62.9,'Molniya' from dual union all
select 15351,'015351','1984-107B','11S510','11S510',57161,'Reentered','1984-12-02 00:00:00','1984-10-06 00:00:00',92.51,215,581,62.9,'Intermediate' from dual union all
select 15352,'015352','1984-107C','BOZ','BOZ',57161,'Reentered','1984-11-07 00:00:00','1984-10-06 00:00:00',92.42,191,596,62.9,'Intermediate' from dual union all
select 15353,'015353','1984-108A','Challenger (STS 41-G)','OV-099',57162,'Landed','1984-10-13 00:00:00','1984-10-07 00:00:00',89.61,250,260,57,'Intermediate' from dual union all
select 15354,'015354','1984-108B','ERBS','ERBS',57162,'In Earth orbit','','1988-06-13 00:00:00',96.73,596,609,57,'Intermediate' from dual union all
select 15355,'015355','1984-107D','Blok-2BL','Blok-2BL No. 57',57161,'In Earth orbit','','1984-11-04 00:00:00',708.05,623,39251,62.9,'Molniya' from dual union all
select 15356,'015356','1984-103C','KDU','KDU',57157,'Reentered','1986-10-29 00:00:00','1984-10-27 00:00:00',92.14,347,412,69.98,'Intermediate' from dual union all
select 15357,'015357','1984-103D','Blenda','Sunshade',57157,'Reentered','1985-03-23 00:00:00','1984-10-11 00:00:00',92.02,335,412,69.98,'Intermediate' from dual union all
select 15358,'015358','1984-103E','KDU part','KDU part',57157,'Reentered','1985-05-01 00:00:00','1984-10-11 00:00:00',94.14,377,577,69.99,'Intermediate' from dual union all
select 15359,'015359','1984-109A','Kosmos-1605','Parus',57163,'In Earth orbit','','1984-11-10 00:00:00',104.79,951,1019,82.94,'Intermediate' from dual union all
select 1536,'001536','1965-020CG','deb Blok Kosmos','',11770,'Reentered','1966-04-26 00:00:00','1965-08-30 00:00:00',102.38,268,1474,56.07,'Intermediate' from dual union all
select 15360,'015360','1984-109B','S3M','S3M',57163,'In Earth orbit','','1991-03-25 00:00:00',104.68,949,1010,82.94,'Intermediate' from dual union all
select 15361,'015361','1984-103F','KDU part','KDU part',57157,'Reentered','1984-12-17 00:00:00','1984-10-12 00:00:00',94.66,374,630,70,'Intermediate' from dual union all
select 15362,'015362','1984-110A','Nova 3','Nova 3',57164,'In Earth orbit','','1985-11-05 00:00:00',108.93,1153,1202,90.05,'Polar' from dual union all
select 15363,'015363','1984-110B','Star 20 E-35','Star 20 E-35',57164,'Reentered','1988-07-26 00:00:00','1984-10-15 00:00:00',98.6,322,1062,89.8,'Polar' from dual union all
select 15364,'015364','1984-103G','KDU part','KDU part',57157,'Reentered','1985-01-08 00:00:00','1984-10-12 00:00:00',90,264,285,70,'Intermediate' from dual union all
select 15365,'015365','1982-033EE','Garbage bag','',54342,'Reentered','1984-12-05 00:00:00','1984-10-12 00:00:00',92.61,356,449,51.7,'Intermediate' from dual union all
select 15366,'015366','1982-033EF','Garbage bag','',54342,'Reentered','1985-01-12 00:00:00','1984-10-14 00:00:00',91.8,336,390,51.65,'Intermediate' from dual union all
select 15367,'015367','1982-033EG','Garbage bag','',54342,'Reentered','1985-03-03 00:00:00','1984-10-14 00:00:00',92.11,351,405,51.65,'Intermediate' from dual union all
select 15368,'015368','1982-033EH','Garbage bag','',54342,'Reentered','1985-02-11 00:00:00','1984-10-14 00:00:00',91.9,366,370,51.64,'Intermediate' from dual union all
select 15369,'015369','1984-111A','Kosmos-1606','Tselina-D',57165,'In Earth orbit','','1991-03-28 00:00:00',97.2,608,641,82.5,'Intermediate' from dual union all
select 1537,'001537','1965-020CH','deb Blok Kosmos','',11770,'Reentered','1966-08-17 00:00:00','1965-08-30 00:00:00',103.6,242,1615,56.17,'Intermediate' from dual union all
select 15370,'015370','1984-111B','S5M','S5M',57165,'In Earth orbit','','1991-03-27 00:00:00',97.47,622,654,82.52,'Intermediate' from dual union all
select 15371,'015371','1984-110C','Nova 3 despin weight','',57164,'Reentered','1987-09-20 00:00:00','1984-10-13 00:00:00',98.63,321,1066,90.81,'Polar' from dual union all
select 15372,'015372','1984-110D','Nova 3 despin weight','',57164,'Reentered','1987-07-04 00:00:00','1984-10-21 00:00:00',98.35,319,1041,89.43,'Polar' from dual union all
select 15373,'015373','1984-103H','KDU part','KDU part',57157,'Reentered','1984-10-26 00:00:00','1984-10-21 00:00:00',92.63,340,467,69.92,'Intermediate' from dual union all
select 15374,'015374','1983-101N','Kosmos-1501 SS 11','ESO',55858,'Reentered','1985-07-11 00:00:00','1984-10-22 00:00:00',94.17,459,497,82.94,'Intermediate' from dual union all
select 15375,'015375','1983-101P','Kosmos-1501 SS 12','ESO',55858,'Reentered','1985-07-05 00:00:00','1984-10-22 00:00:00',94.08,449,499,82.94,'Intermediate' from dual union all
select 15376,'015376','1983-034Y','Kosmos-1453 SS 21','ESO',55791,'Reentered','1985-07-15 00:00:00','1984-10-27 00:00:00',94.2,456,504,74.03,'Intermediate' from dual union all
select 15377,'015377','1983-034Z','Kosmos-1453 SS 22','ESO',55791,'Reentered','1985-05-22 00:00:00','1984-10-27 00:00:00',93.9,452,479,74.03,'Intermediate' from dual union all
select 15378,'015378','1984-112A','Kosmos-1607','US-A reactor',57166,'In Earth orbit','','1984-11-30 00:00:00',89.63,249,264,65.02,'Intermediate' from dual union all
select 15379,'015379','1984-045C','Kosmos-1552 AO part','Terilen AO part',57099,'Reentered','1984-11-04 00:00:00','1984-11-04 00:00:00',89.37,205,282,64.93,'Intermediate' from dual union all
select 1538,'001538','1965-020CJ','deb Blok Kosmos','',11770,'Reentered','1974-12-03 00:00:00','1965-08-30 00:00:00',107.51,407,1816,56.1,'Intermediate' from dual union all
select 15380,'015380','1984-045D','Kosmos-1552 AO part','Terilen AO part',57099,'Reentered','1984-11-05 00:00:00','1984-11-03 00:00:00',89.57,210,297,64.89,'Intermediate' from dual union all
select 15381,'015381','1984-045E','Kosmos-1552 AO part','Terilen AO part',57099,'Reentered','1984-11-05 00:00:00','1984-11-04 00:00:00',90.06,207,347,64.94,'Intermediate' from dual union all
select 15382,'015382','1984-113A','Discovery (STS 51-A)','OV-103',57167,'Landed','1984-11-16 00:00:00','1984-11-10 00:00:00',90.71,306,313,28.47,'Intermediate' from dual union all
select 15383,'015383','1984-113B','Anik D2','Anik D2',57167,'In Earth orbit','','1984-12-09 00:00:00',1436.06,35779,35793,1.78,'Stationary' from dual union all
select 15384,'015384','1984-113C','Leasat 1','Leasat 1',57167,'In Earth orbit','','1984-12-17 00:00:00',1436.06,35726,35846,3.41,'Inclined GEO' from dual union all
select 15385,'015385','1984-114A','Spacenet F2','Spacenet F2',57168,'In Earth orbit','','1984-12-13 00:00:00',1436.01,35778,35792,.18,'Stationary' from dual union all
select 15386,'015386','1984-114B','Marecs 2','Marecs B2',57168,'In Earth orbit','','1984-12-21 00:00:00',1436.27,35752,35828,3.05,'Inclined GEO' from dual union all
select 15387,'015387','1984-113D','PAM-D','Star 48  S/N 10038-3',57167,'In Earth orbit','','1984-12-08 00:00:00',637.48,326,35990,25.84,'GEO Transfer' from dual union all
select 15388,'015388','1984-114C','Ariane H10 V11','Ariane H10 S/N T13',57168,'In Earth orbit','','1984-12-11 00:00:00',637.4,327,35985,6.97,'GEO Transfer' from dual union all
select 15389,'015389','1984-114D','Sylda 4400 No. 2','Sylda 4400',57168,'Reentered','1986-10-27 00:00:00','1984-11-18 00:00:00',624.58,223,35429,6.93,'GEO Transfer' from dual union all
select 1539,'001539','1965-020CK','deb Blok Kosmos','',11770,'Reentered','1966-09-26 00:00:00','1965-08-30 00:00:00',103.7,278,1588,56.12,'Intermediate' from dual union all
select 15390,'015390','1984-113E','Leasat PKM','TU-882',57167,'In Earth orbit','','1984-12-18 00:00:00',278.82,313,15206,27.11,'Highly Ellip' from dual union all
select 15391,'015391','1984-115A','NATO 3D','NATO 3D',57169,'In Earth orbit','','1984-12-16 00:00:00',1426.09,35571,35610,5.91,'Inclined Drift' from dual union all
select 15392,'015392','1984-115B','Delta 177','Delta SSPU DSV-3P-4B',57169,'In Earth orbit','','1990-02-18 00:00:00',115.79,671,2312,21.5,'Equatorial' from dual union all
select 15393,'015393','1984-116A','Kosmos-1608','Siluet No. 4',57170,'Landed','1984-12-17 00:00:00','1984-11-21 00:00:00',89.36,208,277,69.96,'Intermediate' from dual union all
select 15394,'015394','1984-116B','11S510','11S510',57170,'Reentered','1984-11-17 00:00:00','1984-11-15 00:00:00',88.8,193,238,70,'Intermediate' from dual union all
select 15395,'015395','1984-117A','Kosmos-1609','Oblik No. 308',57171,'Landed','1984-11-28 00:00:00','1984-11-16 00:00:00',92.25,357,413,72.85,'Intermediate' from dual union all
select 15396,'015396','1984-117B','11S510','11S510',57171,'Reentered','1984-11-25 00:00:00','1984-11-15 00:00:00',89.94,196,347,72.9,'Intermediate' from dual union all
select 15397,'015397','1984-116C','Kosmos-1608 part','Kometa part',57170,'Reentered','1984-11-16 00:00:00','1984-11-15 00:00:00',88.69,180,240,69.98,'Intermediate' from dual union all
select 15398,'015398','1984-118A','Kosmos-1610','Parus',57172,'In Earth orbit','','1993-04-28 00:00:00',104.87,964,1013,82.95,'Intermediate' from dual;
insert into SATELLITE
select 15399,'015399','1984-118B','S3M','S3M',57172,'In Earth orbit','','1990-03-30 00:00:00',104.73,958,1006,82.95,'Intermediate' from dual union all
select 154,'000154','61 Omicron 39','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-16 00:00:00',103.61,854,1005,66.64,'Intermediate' from dual union all
select 1540,'001540','1965-020CL','deb Blok Kosmos','',11770,'Reentered','1968-02-27 00:00:00','1965-08-30 00:00:00',106.84,341,1820,56.15,'Intermediate' from dual union all
select 15400,'015400','1984-117C','Sensor cover','Sensor cover',57171,'Reentered','1984-11-17 00:00:00','1984-11-15 00:00:00',89.88,199,338,72.8,'Intermediate' from dual union all
select 15401,'015401','1982-033EJ','Garbage bag','',54342,'Reentered','1985-01-04 00:00:00','1984-11-15 00:00:00',91.56,330,373,51.6,'Intermediate' from dual union all
select 15402,'015402','1984-115C','Star 37E S/N 40102','Star 37E',57169,'In Earth orbit','','1984-12-14 00:00:00',649.21,428,36488,22.84,'GEO Transfer' from dual union all
select 15403,'015403','1984-119A','Kosmos-1611','Kobal''t',57173,'Landed','1985-01-11 00:00:00','1984-11-28 00:00:00',89.75,169,356,64.75,'Intermediate' from dual union all
select 15404,'015404','1984-119B','11S510','11S510',57173,'Reentered','1984-11-25 00:00:00','1984-11-21 00:00:00',89.35,183,302,64.8,'Intermediate' from dual union all
select 15405,'015405','1984-119C','Kosmos-1611 part','Kobal''t part',57173,'Reentered','1984-11-22 00:00:00','1984-11-21 00:00:00',89.41,179,312,64.8,'Intermediate' from dual union all
select 15406,'015406','1984-120A','Kosmos-1612','Meteor-3 No. 1',57174,'Reentered','1985-01-31 00:00:00','1984-11-27 00:00:00',98.4,135,1229,82.6,'Intermediate' from dual union all
select 15407,'015407','1984-120B','S5M','S5M',57174,'Reentered','1984-12-02 00:00:00','1984-11-27 00:00:00',98.34,130,1229,82.6,'Intermediate' from dual union all
select 15408,'015408','1984-117D','Blenda','Sunshade',57171,'Reentered','1985-07-06 00:00:00','1984-11-28 00:00:00',94.51,400,590,72.9,'Intermediate' from dual union all
select 15409,'015409','1984-117E','KDU','KDU',57171,'Reentered','1987-04-26 00:00:00','1984-12-15 00:00:00',92.22,355,412,72.85,'Intermediate' from dual union all
select 1541,'001541','1965-020CM','deb Blok Kosmos','',11770,'Reentered','1966-03-20 00:00:00','1966-01-30 00:00:00',95,278,759,56.11,'Intermediate' from dual union all
select 15410,'015410','1984-117F','KDU part','KDU part',57171,'Reentered','1985-05-01 00:00:00','1984-11-28 00:00:00',92.34,355,424,72.8,'Intermediate' from dual union all
select 15411,'015411','1984-117G','KDU part','KDU part',57171,'Reentered','1984-12-01 00:00:00','1984-11-28 00:00:00',91.53,301,398,72.8,'Intermediate' from dual union all
select 15412,'015412','1984-117H','KDU part','KDU part',57171,'Reentered','1985-04-02 00:00:00','1984-11-28 00:00:00',92.49,368,426,72.9,'Intermediate' from dual union all
select 15413,'015413','1984-117J','KDU part','KDU part',57171,'Reentered','1985-03-08 00:00:00','1984-11-29 00:00:00',94.88,400,626,72.9,'Intermediate' from dual union all
select 15414,'015414','1984-121A','Kosmos-1613','Oblik No. 797',57175,'Landed','1984-12-24 00:00:00','1984-12-07 00:00:00',89.92,195,345,72.83,'Intermediate' from dual union all
select 15415,'015415','1984-121B','11S510','11S510',57175,'Reentered','1984-12-12 00:00:00','1984-11-29 00:00:00',90.22,208,362,72.8,'Intermediate' from dual union all
select 15416,'015416','1984-121C','Sensor cover','Sensor cover',57175,'Reentered','1984-12-06 00:00:00','1984-11-29 00:00:00',91.14,207,453,72.8,'Intermediate' from dual union all
select 15417,'015417','1984-121D','Sensor cover','Sensor cover',57175,'Reentered','1984-12-01 00:00:00','1984-11-30 00:00:00',89.77,202,324,72.9,'Intermediate' from dual union all
select 15418,'015418','1984-117K','KDU part','KDU part',57171,'Reentered','1984-12-30 00:00:00','1984-11-30 00:00:00',92.43,309,479,72.7,'Intermediate' from dual union all
select 15419,'015419','1984-117L','KDU part','KDU part',57171,'Reentered','1985-02-22 00:00:00','1984-11-30 00:00:00',93.81,411,510,72,'Intermediate' from dual union all
select 1542,'001542','1965-020CN','deb Blok Kosmos','',11770,'Reentered','1966-07-31 00:00:00','1966-01-26 00:00:00',100.98,319,1291,56.09,'Intermediate' from dual union all
select 15420,'015420','1982-033EK','Garbage bag','',54342,'Reentered','1985-01-24 00:00:00','1984-12-01 00:00:00',91.78,337,387,51.6,'Intermediate' from dual union all
select 15421,'015421','1984-117M','KDU part','KDU part',57171,'Reentered','1984-12-11 00:00:00','1984-12-04 00:00:00',92.28,270,503,72.7,'Intermediate' from dual union all
select 15422,'015422','1977-038C','CANYON apogee motor','',42166,'In Earth orbit','','2010-04-20 00:00:00',1407.17,29053,41384,10.42,'Near-synch' from dual union all
select 15423,'015423','1984-122A','USA 6','KENNEN 6',57176,'Deorbited','1994-10-20 00:00:00','1989-08-09 00:00:00',97.22,330,922,97.98,'Sun-Synch' from dual union all
select 15424,'015424','1984-122B','Titan 34D-6 Stage 2','LR91AJ11 34D-6',57176,'Reentered','1984-12-23 00:00:00','1984-12-04 00:00:00',94.1,300,650,97.1,'Sun-Synch' from dual union all
select 15425,'015425','1983-101Q','Kosmos-1501 SS 13','ESO',55858,'Reentered','1987-02-01 00:00:00','1984-12-11 00:00:00',94.12,456,496,82.93,'Intermediate' from dual union all
select 15426,'015426','1983-101R','Kosmos-1501 SS 14','ESO',55858,'Reentered','1987-01-07 00:00:00','1984-12-11 00:00:00',94.09,453,496,82.94,'Intermediate' from dual union all
select 15427,'015427','1984-123A','NOAA 9','NOAA F',57177,'In Earth orbit','','1985-01-16 00:00:00',102.02,843,866,98.93,'Sun-Synch' from dual union all
select 15428,'015428','1984-117N','KDU part','KDU part',57171,'Reentered','1984-12-24 00:00:00','1984-12-13 00:00:00',91.91,352,385,72.7,'Intermediate' from dual union all
select 15429,'015429','1984-124A','Molniya-1','Molniya-1 No. 55',57178,'In Earth orbit','','1985-01-14 00:00:00',717.72,470,39882,62.86,'Molniya' from dual union all
select 1543,'001543','1965-020CP','deb Blok Kosmos','',11770,'Reentered','1968-09-19 00:00:00','1965-12-18 00:00:00',105.85,438,1631,56.09,'Intermediate' from dual union all
select 15430,'015430','1984-124B','11S510','11S510',57178,'Reentered','1985-01-11 00:00:00','1984-12-15 00:00:00',90.82,227,403,62.8,'Intermediate' from dual union all
select 15431,'015431','1984-124C','BOZ','BOZ',57178,'Reentered','1985-01-20 00:00:00','1984-12-15 00:00:00',91.26,221,452,62.8,'Intermediate' from dual union all
select 15432,'015432','1984-125A','Vega-1','5VK No. 901',57179,'Deep Space','1984-12-15 00:00:00','',null,null,null,null,'' from dual union all
select 15433,'015433','1984-125B','SOZ','SOZ',57179,'Reentered','1984-12-16 00:00:00','1984-12-15 00:00:00',88.45,180,216,51.6,'Intermediate' from dual union all
select 15434,'015434','1984-125C','SOZ','SOZ',57179,'Reentered','1984-12-16 00:00:00','1984-12-15 00:00:00',88.44,178,217,51.5,'Intermediate' from dual union all
select 15435,'015435','1984-124D','deb BOZ','',57178,'Reentered','1985-01-21 00:00:00','1984-12-16 00:00:00',91.41,233,455,62.8,'Intermediate' from dual union all
select 15436,'015436','1984-124E','deb BOZ','',57178,'Reentered','1985-01-12 00:00:00','1984-12-15 00:00:00',91.36,228,455,62.8,'Intermediate' from dual union all
select 15437,'015437','1984-124F','deb BOZ','',57178,'Reentered','1985-01-09 00:00:00','1984-12-16 00:00:00',91.32,227,452,62.8,'Intermediate' from dual union all
select 15438,'015438','1984-124G','deb BOZ','',57178,'Reentered','1984-12-20 00:00:00','1984-12-15 00:00:00',91.06,215,438,62.8,'Intermediate' from dual union all
select 15439,'015439','1984-124H','Blok-ML','Blok-ML No. 166',57178,'In Earth orbit','','1985-01-14 00:00:00',733.45,474,40651,62.81,'Molniya' from dual union all
select 1544,'001544','1965-020CQ','deb Blok Kosmos','',11770,'Reentered','1967-02-22 00:00:00','1965-08-30 00:00:00',105.49,314,1721,56.19,'Intermediate' from dual union all
select 15440,'015440','1984-123B','NOAA 9 solar array band','',57177,'Reentered','1999-03-16 00:00:00','1985-10-18 00:00:00',101.94,846,854,98.93,'Sun-Synch' from dual union all
select 15441,'015441','1984-123C','NOAA 9 solar array band','',57177,'Reentered','1992-11-13 00:00:00','1992-10-01 00:00:00',94.1,472,478,98.98,'Sun-Synch' from dual union all
select 15442,'015442','1984-126A','Kosmos-1614','BOR-4 No. 406',57180,'Landed','1984-12-19 00:00:00','1984-12-19 00:00:00',88.43,172,222,50.7,'Intermediate' from dual union all
select 15443,'015443','1984-126B','S3M','S3M',57180,'Reentered','1984-12-21 00:00:00','1984-12-19 00:00:00',88.67,186,232,50.7,'Intermediate' from dual union all
select 15444,'015444','1984-126C','deb Kosmos-1614','',57180,'Reentered','1984-12-20 00:00:00','1984-12-19 00:00:00',88.54,194,211,50.7,'Intermediate' from dual union all
select 15445,'015445','1984-126D','deb Kosmos-1614','',57180,'Reentered','1984-12-20 00:00:00','1984-12-19 00:00:00',88.54,194,211,50.7,'Intermediate' from dual union all
select 15446,'015446','1984-127A','Kosmos-1615','Taifun-1B',57181,'Reentered','1990-04-15 00:00:00','1985-01-27 00:00:00',93.97,438,500,65.82,'Intermediate' from dual union all
select 15447,'015447','1984-125D','Blok D-1 No. 11L','11S824M No. 11L',57179,'Deep Space','1984-12-15 00:00:00','',null,null,null,null,'' from dual union all
select 15448,'015448','1984-127B','S3M','S3M',57181,'Reentered','1989-04-09 00:00:00','1988-06-11 00:00:00',93.22,399,466,65.81,'Intermediate' from dual union all
select 15449,'015449','1984-128A','Vega-2','5VK No. 902',57182,'Deep Space','1984-12-21 00:00:00','',null,null,null,null,'' from dual union all
select 1545,'001545','1965-020CR','deb Blok Kosmos','',11770,'Reentered','1966-01-24 00:00:00','1965-08-30 00:00:00',102.54,290,1468,56.03,'Intermediate' from dual union all
select 15450,'015450','1984-128B','Blok D-1 No. 12L','11S824M No. 12L',57182,'Deep Space','1984-12-21 00:00:00','',null,null,null,null,'' from dual union all
select 15451,'015451','1984-128C','SOZ','SOZ',57182,'Reentered','1984-12-23 00:00:00','1984-12-21 00:00:00',88.22,171,202,51.5,'Intermediate' from dual union all
select 15452,'015452','1984-128D','SOZ','SOZ',57182,'Reentered','1984-12-23 00:00:00','1984-12-21 00:00:00',88.23,171,203,51.5,'Intermediate' from dual union all
select 15453,'015453','1984-129A','DSP F12','DSP 6R',57183,'In Earth orbit','','1984-12-31 00:00:00',1436.32,35777,35805,.1,'Stationary' from dual union all
select 15454,'015454','1984-129B','Transtage D-13','Transtage D-13',57183,'In Earth orbit','','1984-12-31 00:00:00',1431.67,35600,35800,2,'Drift GEO' from dual union all
select 15455,'015455','1984-129C','Titan 34D-13 Stage 2','LR91AJ11 34D-13',57183,'Reentered','1985-01-17 00:00:00','1984-12-23 00:00:00',88.43,126,268,28.7,'Intermediate' from dual union all
select 15456,'015456','1984-121E','KDU','KDU',57175,'Reentered','1987-05-05 00:00:00','1984-12-30 00:00:00',92.24,355,414,72.84,'Intermediate' from dual union all
select 15457,'015457','1984-121F','Blenda','Sunshade',57175,'Reentered','1985-04-13 00:00:00','1984-12-24 00:00:00',92.51,367,429,72.8,'Intermediate' from dual union all
select 15458,'015458','1984-121G','KDU part','KDU part',57175,'Reentered','1985-07-05 00:00:00','1984-12-24 00:00:00',92.34,355,424,72.8,'Intermediate' from dual union all
select 15459,'015459','1984-121H','KDU part','KDU part',57175,'Reentered','1985-10-05 00:00:00','1984-12-24 00:00:00',93.56,312,586,72.8,'Intermediate' from dual union all
select 1546,'001546','1965-020CS','deb Blok Kosmos','',11770,'Reentered','1966-08-16 00:00:00','1965-08-30 00:00:00',104.61,317,1636,56.1,'Intermediate' from dual union all
select 15460,'015460','1984-121J','KDU part','KDU part',57175,'Reentered','1985-05-12 00:00:00','1984-12-25 00:00:00',94.73,403,608,72.8,'Intermediate' from dual union all
select 15461,'015461','1984-121K','KDU part','KDU part',57175,'Reentered','1985-01-11 00:00:00','1984-12-30 00:00:00',92.97,368,472,72.8,'Intermediate' from dual union all
select 15462,'015462','1984-121L','KDU part','KDU part',57175,'Reentered','1985-01-08 00:00:00','1984-12-30 00:00:00',91.81,306,421,73.1,'Intermediate' from dual union all
select 15463,'015463','1984-121M','KDU part','KDU part',57175,'Reentered','1985-01-21 00:00:00','1984-12-31 00:00:00',92.33,367,411,72.6,'Intermediate' from dual union all
select 15464,'015464','1985-001A','Sakigake','MS-T5',58224,'Deep Space','1985-01-08 00:00:00','1985-01-07 00:00:00',1993.08,250,117930,31.1,'Earth Escape' from dual union all
select 15465,'015465','1985-001B','KM-P','KM-P',58224,'Deep Space','1985-01-08 00:00:00','1985-01-07 00:00:00',1993.08,250,117930,31.1,'Earth Escape' from dual union all
select 15466,'015466','1985-001C','M-3B','M-3B',58224,'Reentered','1985-05-22 00:00:00','1985-01-08 00:00:00',98.52,215,1161,31.1,'Intermediate' from dual union all
select 15467,'015467','1985-002A','Kosmos-1616','Kobal''t',58225,'Landed','1985-03-04 00:00:00','1985-01-16 00:00:00',89.1,185,275,64.89,'Intermediate' from dual union all
select 15468,'015468','1985-002B','11S510','11S510',58225,'Reentered','1985-01-15 00:00:00','1985-01-09 00:00:00',89.95,184,360,64.9,'Intermediate' from dual union all
select 15469,'015469','1985-003A','Kosmos-1617','Strela-3',58226,'In Earth orbit','','1985-10-18 00:00:00',114.08,1412,1416,82.61,'Intermediate' from dual union all
select 1547,'001547','1965-020CT','deb Blok Kosmos','',11770,'Reentered','1966-07-24 00:00:00','1965-08-30 00:00:00',102.92,297,1496,56.25,'Intermediate' from dual union all
select 15470,'015470','1985-003B','Kosmos-1618','Strela-3',58226,'In Earth orbit','','1993-04-25 00:00:00',114.01,1406,1415,82.59,'Intermediate' from dual union all
select 15471,'015471','1985-003C','Kosmos-1619','Strela-3',58226,'In Earth orbit','','1993-03-28 00:00:00',113.73,1382,1414,82.6,'Intermediate' from dual union all
select 15472,'015472','1985-003D','Kosmos-1620','Strela-3',58226,'In Earth orbit','','1993-03-26 00:00:00',113.81,1389,1414,82.6,'Intermediate' from dual union all
select 15473,'015473','1985-003E','Kosmos-1621','Strela-3',58226,'In Earth orbit','','1993-03-21 00:00:00',113.88,1394,1415,82.61,'Intermediate' from dual union all
select 15474,'015474','1985-003F','Kosmos-1622','Strela-3',58226,'In Earth orbit','','1993-03-26 00:00:00',113.93,1399,1415,82.6,'Intermediate' from dual union all
select 15475,'015475','1985-003G','S5M','S5M',58226,'In Earth orbit','','1993-03-25 00:00:00',114.72,1413,1473,82.61,'Intermediate' from dual union all
select 15476,'015476','1985-004A','Molniya-3','Molniya-3 No. 36',58227,'Reentered','1994-12-05 00:00:00','1985-02-16 00:00:00',717.8,638,39718,62.89,'Molniya' from dual union all
select 15477,'015477','1985-004B','11S510','11S510',58227,'Reentered','1985-03-27 00:00:00','1985-01-16 00:00:00',92.64,227,581,62.8,'Intermediate' from dual union all
select 15478,'015478','1985-004C','BOZ','BOZ',58227,'Reentered','1985-03-21 00:00:00','1985-01-16 00:00:00',93.08,213,637,62.8,'Intermediate' from dual union all
select 15479,'015479','1985-005A','Kosmos-1623','Oblik',58228,'Landed','1985-01-30 00:00:00','1985-01-18 00:00:00',92.19,349,415,69.99,'Intermediate' from dual union all
select 1548,'001548','1965-020CU','deb Blok Kosmos','',11770,'Reentered','1978-05-27 00:00:00','1965-09-22 00:00:00',107.58,408,1822,56.06,'Intermediate' from dual union all
select 15480,'015480','1985-005B','11S510','11S510',58228,'Reentered','1985-02-05 00:00:00','1985-01-16 00:00:00',90.53,222,378,70,'Intermediate' from dual union all
select 15481,'015481','1985-004D','Blok-ML','Blok-ML No. 324',58227,'Reentered','1995-04-24 00:00:00','1985-11-19 00:00:00',732.08,616,40442,63.1,'Molniya' from dual union all
select 15482,'015482','1985-006A','Kosmos-1624','Strela-2M',58229,'In Earth orbit','','1985-10-18 00:00:00',100.77,784,807,74.04,'Intermediate' from dual union all
select 15483,'015483','1985-006B','S3M','S3M',58229,'In Earth orbit','','1991-03-27 00:00:00',100.55,770,800,74.03,'Intermediate' from dual union all
select 15484,'015484','1985-007A','Gorizont','Gorizont No. 21L',58230,'In Earth orbit','','1985-02-17 00:00:00',1436.32,35741,35841,1.44,'Stationary' from dual union all
select 15485,'015485','1985-007B','8S812','8S812',58230,'Reentered','1985-01-20 00:00:00','1985-01-18 00:00:00',88.48,197,202,51.6,'Intermediate' from dual union all
select 15486,'015486','1985-007C','Perekhodnik','Sredniy perekhodnik',58230,'Reentered','1985-01-19 00:00:00','1985-01-18 00:00:00',88.66,192,225,51.5,'Intermediate' from dual union all
select 15487,'015487','1985-007D','Blok DM No. 52L','11S86 No. 52L',58230,'In Earth orbit','','1993-06-06 00:00:00',1397.66,34960,35102,5.83,'Inclined Drift' from dual union all
select 15488,'015488','1985-007E','SOZ','SOZ',58230,'Reentered','1985-09-21 00:00:00','1985-01-18 00:00:00',617.8,270,35031,47.4,'GEO Transfer' from dual;
insert into SATELLITE
select 15489,'015489','1985-007F','SOZ','SOZ',58230,'Reentered','1996-04-20 00:00:00','1993-03-29 00:00:00',362,163,20732,46.83,'Highly Ellip' from dual union all
select 1549,'001549','1965-020CV','deb Blok Kosmos','',11770,'In Earth orbit','','1965-08-30 00:00:00',115.09,786,2133,56.22,'Intermediate' from dual union all
select 15490,'015490','1985-006C','deb Kosmos-1624','Strela-2M debris 1',58229,'In Earth orbit','','1985-02-16 00:00:00',100.97,802,808,74.05,'Intermediate' from dual union all
select 15491,'015491','1985-006D','deb Kosmos-1624','Strela-2M debris 2 (high',58229,'In Earth orbit','','1985-10-21 00:00:00',101.31,806,835,74.05,'Intermediate' from dual union all
select 15492,'015492','1985-008A','Kosmos-1625','US-PM',58231,'Reentered','1985-01-25 00:00:00','1985-01-24 00:00:00',89.34,115,368,65.01,'Intermediate' from dual union all
select 15493,'015493','1985-008B','11S692','11S692',58231,'Reentered','1985-01-24 00:00:00','1985-01-24 00:00:00',89.49,123,376,65,'Intermediate' from dual union all
select 15494,'015494','1985-009A','Kosmos-1626','Tselina-D',58232,'In Earth orbit','','1987-10-18 00:00:00',97.64,629,663,82.53,'Intermediate' from dual union all
select 15495,'015495','1985-009B','S5M','S5M',58232,'In Earth orbit','','1987-10-29 00:00:00',97.62,630,660,82.53,'Intermediate' from dual union all
select 15496,'015496','1985-010A','Discovery (STS 51-C)','OV-103',58233,'Landed','1985-01-27 00:00:00','1985-01-26 00:00:00',91.63,334,374,28.47,'Intermediate' from dual union all
select 15497,'015497','1985-009C','deb Kosmos-1626','',58232,'Reentered','1989-02-02 00:00:00','1985-02-01 00:00:00',97.62,629,661,82.52,'Intermediate' from dual union all
select 15498,'015498','1985-005C','PTDU cover','KDU part',58228,'Reentered','1985-11-09 00:00:00','1985-01-30 00:00:00',94.73,390,621,70,'Intermediate' from dual union all
select 15499,'015499','1985-005D','KDU','KDU',58228,'Reentered','1987-03-31 00:00:00','1985-02-15 00:00:00',92.15,348,412,69.98,'Intermediate' from dual union all
select 155,'000155','61 Omicron 40','deb r/b Ablestar','',7273,'In Earth orbit','','1962-03-26 00:00:00',103.32,858,974,66.88,'Intermediate' from dual union all
select 1550,'001550','1965-020CW','deb Blok Kosmos','',11770,'Reentered','1970-07-08 00:00:00','1965-08-30 00:00:00',106.21,305,1797,56.11,'Intermediate' from dual union all
select 15500,'015500','1985-005E','Blenda','Sunshade',58228,'Reentered','1985-07-22 00:00:00','1985-02-02 00:00:00',92.01,336,410,69.99,'Intermediate' from dual union all
select 15501,'015501','1985-005F','PTDU cover','KDU part',58228,'Reentered','1985-05-19 00:00:00','1985-02-02 00:00:00',94.6,378,621,69.99,'Intermediate' from dual union all
select 15502,'015502','1984-112B','Kosmos-1607 DU','US-A DU',57166,'Reentered','1985-03-28 00:00:00','1985-02-04 00:00:00',89.55,243,262,65.02,'Intermediate' from dual union all
select 15503,'015503','1984-112C','Kosmos-1607 reactor core','US-A reactor core',57166,'In Earth orbit','','1991-11-01 00:00:00',103.79,910,966,65,'Intermediate' from dual union all
select 15504,'015504','1984-112D','deb Kosmos-1607 reactor core','',57166,'Reentered','1985-10-01 00:00:00','1985-02-02 00:00:00',103.47,857,989,94.99,'Polar' from dual union all
select 15505,'015505','1985-011A','Kosmos-1627','Parus',58234,'In Earth orbit','','1985-10-18 00:00:00',104.85,957,1017,82.92,'Intermediate' from dual union all
select 15506,'015506','1985-011B','S3M','S3M',58234,'In Earth orbit','','1991-03-28 00:00:00',104.72,956,1006,82.91,'Intermediate' from dual union all
select 15507,'015507','1984-112E','US-A RLS','US-A RLS',57166,'Reentered','1985-02-07 00:00:00','1985-02-02 00:00:00',89.4,234,256,65.02,'Intermediate' from dual union all
select 15508,'015508','1984-112F','deb Kosmos-1607','',57166,'Reentered','1985-07-03 00:00:00','1985-02-01 00:00:00',100.54,655,914,64.95,'Intermediate' from dual union all
select 15509,'015509','1985-005G','KDU part','KDU part',58228,'Reentered','1985-05-26 00:00:00','1985-02-04 00:00:00',92.17,348,414,69.99,'Intermediate' from dual union all
select 1551,'001551','1965-020CX','deb Blok Kosmos','',11770,'Reentered','1969-12-19 00:00:00','1965-08-30 00:00:00',106.07,304,1785,56.14,'Intermediate' from dual union all
select 15510,'015510','1985-005H','KDU part','KDU part',58228,'Reentered','1985-02-26 00:00:00','1985-02-04 00:00:00',92,344,402,69.99,'Intermediate' from dual union all
select 15511,'015511','1984-112G','deb Kosmos-1607','',57166,'Reentered','1985-03-05 00:00:00','1985-02-04 00:00:00',98.83,509,897,65.09,'Intermediate' from dual union all
select 15512,'015512','1983-101S','Kosmos-1501 SS 15','ESO',55858,'Reentered','1985-10-06 00:00:00','1985-02-04 00:00:00',93.98,445,494,82.92,'Intermediate' from dual union all
select 15513,'015513','1983-101T','Kosmos-1501 SS 16','ESO',55858,'Reentered','1986-01-03 00:00:00','1985-02-04 00:00:00',94.2,462,498,82.95,'Intermediate' from dual union all
select 15514,'015514','1985-012A','Kosmos-1628','Oblik No. 309',58235,'Landed','1985-02-20 00:00:00','1985-02-08 00:00:00',92.25,355,415,72.85,'Intermediate' from dual union all
select 15515,'015515','1985-012B','11S510','11S510',58235,'Reentered','1985-02-21 00:00:00','1985-02-06 00:00:00',90.15,192,372,72.86,'Intermediate' from dual union all
select 15516,'015516','1985-013A','Meteor-2','Meteor-2 No. 13',58236,'In Earth orbit','','1985-10-18 00:00:00',104.05,939,960,82.53,'Intermediate' from dual union all
select 15517,'015517','1985-013B','S5M','S5M',58236,'In Earth orbit','','1991-03-25 00:00:00',104,936,960,82.54,'Intermediate' from dual union all
select 15518,'015518','1968-091CW','deb Kosmos-249','',16818,'Reentered','1988-04-01 00:00:00','1987-11-11 00:00:00',107.12,319,1868,62.04,'Intermediate' from dual union all
select 15519,'015519','1968-097EF','deb Kosmos-252','',16824,'In Earth orbit','','1993-08-06 00:00:00',111.24,549,2019,62.37,'Intermediate' from dual union all
select 1552,'001552','1965-020CY','deb Blok Kosmos','',11770,'Reentered','1967-03-20 00:00:00','1965-08-30 00:00:00',106.14,336,1759,55.92,'Intermediate' from dual union all
select 15520,'015520','1970-025NY','deb AGENA D','',21502,'Reentered','1989-06-06 00:00:00','1984-08-11 00:00:00',99.66,621,864,99.97,'Sun-Synch' from dual union all
select 15521,'015521','1974-083C','deb Blok E','',33686,'In Earth orbit','','1980-10-24 00:00:00',99.19,689,751,70.03,'Intermediate' from dual union all
select 15522,'015522','1974-089EV','deb Delta 104','',33692,'Reentered','1988-09-04 00:00:00','1983-02-10 00:00:00',111.11,955,1600,104.74,'Retrograde' from dual union all
select 15523,'015523','1976-067BH','deb Kosmos-839','',39019,'Reentered','1990-01-03 00:00:00','1993-08-05 00:00:00',115.45,894,2058,65.93,'Intermediate' from dual union all
select 15524,'015524','1982-033EL','Salyut-7 debris','',54342,'Reentered','1985-03-17 00:00:00','1985-02-11 00:00:00',90.99,314,333,51.65,'Intermediate' from dual union all
select 15525,'015525','1982-033EM','Salyut-7 debris','',54342,'Reentered','1985-04-10 00:00:00','1985-02-06 00:00:00',92.48,349,443,51.77,'Intermediate' from dual union all
select 15526,'015526','1982-033EN','Salyut-7 debris','',54342,'Reentered','1985-12-08 00:00:00','1985-02-06 00:00:00',91.66,356,356,51.64,'Intermediate' from dual union all
select 15527,'015527','1982-033EP','Salyut-7 debris','',54342,'Reentered','1985-03-28 00:00:00','1985-02-06 00:00:00',90.83,310,321,51.64,'Intermediate' from dual union all
select 15528,'015528','1982-109D','deb Kosmos (-1420) [UNID]','Strela-2M debris 2 (high',54418,'In Earth orbit','','1985-10-20 00:00:00',100.96,793,816,74.04,'Intermediate' from dual union all
select 15529,'015529','1965-027S','deb SNAPSHOT','',11777,'In Earth orbit','','1993-08-07 00:00:00',111.25,1261,1308,90.27,'Polar' from dual union all
select 1553,'001553','1965-020CZ','deb Blok Kosmos','',11770,'Reentered','1966-07-23 00:00:00','1965-08-30 00:00:00',103.16,266,1551,55.95,'Intermediate' from dual union all
select 15530,'015530','1965-027T','deb SNAPSHOT','',11777,'In Earth orbit','','1993-08-02 00:00:00',111.34,1262,1314,90.28,'Polar' from dual union all
select 15531,'015531','1967-014L','Diademe','',15269,'Reentered','1991-08-19 00:00:00','1987-08-13 00:00:00',103.04,546,1259,39.5,'Intermediate' from dual union all
select 15532,'015532','1982-033EQ','Salyut-7 debris','',54342,'Reentered','1985-02-15 00:00:00','1985-02-08 00:00:00',89.57,240,267,51.65,'Intermediate' from dual union all
select 15533,'015533','1982-033ER','Salyut-7 debris','',54342,'Reentered','1985-03-01 00:00:00','1985-02-08 00:00:00',90.37,267,319,51.63,'Intermediate' from dual union all
select 15534,'015534','1968-091CX','deb Kosmos-249','[UNID106] RCS 0.01',16818,'In Earth orbit','','1993-08-05 00:00:00',104.85,542,1433,62.62,'Intermediate' from dual union all
select 15535,'015535','1968-091CY','deb Kosmos-249','',16818,'In Earth orbit','','1993-08-04 00:00:00',106.42,517,1604,62.41,'Intermediate' from dual union all
select 15536,'015536','1968-097EG','deb Kosmos-252','',16824,'Reentered','1993-07-04 00:00:00','1992-10-11 00:00:00',95.84,440,678,62.3,'Intermediate' from dual union all
select 15537,'015537','1970-025NZ','deb AGENA D','',21502,'Reentered','1988-12-20 00:00:00','1984-11-03 00:00:00',101.69,818,860,99.13,'Sun-Synch' from dual union all
select 15538,'015538','1971-015CV','deb Kosmos-397','',24833,'Reentered','2000-12-27 00:00:00','1993-08-06 00:00:00',101.52,525,1137,65.71,'Intermediate' from dual union all
select 15539,'015539','1972-058JK','deb Delta 89','',27990,'Reentered','1991-07-22 00:00:00','1991-07-03 00:00:00',92.48,387,406,98.72,'Sun-Synch' from dual union all
select 1554,'001554','1965-020DA','deb Blok Kosmos','',11770,'Reentered','1967-01-06 00:00:00','1965-08-30 00:00:00',104.25,267,1652,56.11,'Intermediate' from dual union all
select 15540,'015540','1976-077EU','deb Delta 126','',39029,'In Earth orbit','','1985-10-20 00:00:00',124.04,1467,2255,102.64,'Medium' from dual union all
select 15541,'015541','1981-053JZ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',104.3,936,988,82.89,'Intermediate' from dual union all
select 15542,'015542','1981-053KA','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',105.42,953,1075,83.01,'Intermediate' from dual union all
select 15543,'015543','1985-010B','USA 8','ORION 1',58233,'In Earth orbit','','1985-01-25 00:00:00',1436.19,35704,35873,0,'Stationary' from dual union all
select 15544,'015544','1985-010C','IUS-11 SRM-1','Orbus 21',58233,'In Earth orbit','','1985-01-25 00:00:00',612.22,341,34670,28.4,'GEO Transfer' from dual union all
select 15545,'015545','1985-010D','IUS-11 SRM-2','Orbus 6E',58233,'In Earth orbit','','2003-03-03 00:00:00',1422.24,35513,35517,12.7,'Inclined Drift' from dual union all
select 15546,'015546','1985-014A','USA 9','QUASAR 6',58237,'In Earth orbit','','1985-02-08 00:00:00',712.62,400,39700,63,'Molniya' from dual union all
select 15547,'015547','1985-014B','Agena D','Agena D',58237,'In Earth orbit','','1985-02-08 00:00:00',710.05,356,39617,63,'Molniya' from dual union all
select 15548,'015548','1969-082KM','deb Agena D','',18349,'Reentered','1989-03-19 00:00:00','1984-03-04 00:00:00',98.66,645,745,70.08,'Intermediate' from dual union all
select 15549,'015549','1969-082KN','deb Agena D','',18349,'Reentered','1989-08-31 00:00:00','1985-02-10 00:00:00',98.96,685,733,70.11,'Intermediate' from dual union all
select 1555,'001555','1965-020DB','deb Blok Kosmos','',11770,'Reentered','1968-02-12 00:00:00','1965-09-07 00:00:00',106.65,371,1771,56,'Intermediate' from dual union all
select 15550,'015550','1969-082KP','deb Agena D','',18349,'Reentered','1989-04-20 00:00:00','1984-12-25 00:00:00',98.89,691,721,70.24,'Intermediate' from dual union all
select 15551,'015551','1969-082KQ','deb Agena D','',18349,'Reentered','1999-02-28 00:00:00','1993-08-07 00:00:00',97.69,622,675,70.43,'Intermediate' from dual union all
select 15552,'015552','1969-082KR','deb Agena D','',18349,'Reentered','2001-11-08 00:00:00','1993-08-06 00:00:00',101.21,731,901,70.59,'Intermediate' from dual union all
select 15553,'015553','1969-082KS','deb Agena D','',18349,'Reentered','2001-04-15 00:00:00','1985-10-18 00:00:00',102.83,816,969,70.55,'Intermediate' from dual union all
select 15554,'015554','1969-082KT','deb Agena D','',18349,'Reentered','1990-01-20 00:00:00','1985-02-02 00:00:00',99.21,681,761,69.65,'Intermediate' from dual union all
select 15555,'015555','1976-126BT','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-06 00:00:00',115.58,700,2264,65.78,'Intermediate' from dual union all
select 15556,'015556','1984-011W','deb PAM-D','',57065,'Reentered','1985-04-24 00:00:00','1985-02-10 00:00:00',91.55,249,453,28.19,'Intermediate' from dual union all
select 15557,'015557','1984-011X','deb PAM-D','',57065,'Reentered','1985-06-28 00:00:00','1985-02-08 00:00:00',92.31,258,518,28.24,'Intermediate' from dual union all
select 15558,'015558','61 Omicron 269','deb r/b Ablestar','',7273,'In Earth orbit','','1985-10-19 00:00:00',102.91,859,934,65.59,'Intermediate' from dual union all
select 15559,'015559','1977-121BM','deb Kosmos-970','',42249,'In Earth orbit','','1993-08-06 00:00:00',105.73,932,1126,65.58,'Intermediate' from dual union all
select 1556,'001556','1965-020DC','deb Blok Kosmos','',11770,'Reentered','1967-05-02 00:00:00','1965-08-30 00:00:00',105.41,283,1745,56.23,'Intermediate' from dual union all
select 15560,'015560','1985-015A','Arabsat 1A','Arabsat 1A',58238,'In Earth orbit','','1985-04-26 00:00:00',1436.04,35756,35815,.5,'Stationary' from dual union all
select 15561,'015561','1985-015B','Brasilsat 1','SBTS 1',58238,'In Earth orbit','','1985-03-10 00:00:00',1436.21,35785,35793,.07,'Stationary' from dual union all
select 15562,'015562','1985-015C','Ariane H10 V12','Ariane H10 S/N T14',58238,'In Earth orbit','','1985-03-10 00:00:00',638.63,301,36074,6.93,'GEO Transfer' from dual union all
select 15563,'015563','1985-015D','Sylda 4400 No. 3','Sylda 4400',58238,'Reentered','1986-04-05 00:00:00','1985-02-10 00:00:00',631.25,217,35779,7.13,'GEO Transfer' from dual union all
select 15564,'015564','1984-104C','Dispenser cover','Romb dispenser cover',57158,'Reentered','1985-11-14 00:00:00','1984-12-21 00:00:00',93.8,424,497,65.87,'Intermediate' from dual union all
select 15565,'015565','1984-104D','Dispenser cover','Romb dispenser cover',57158,'Reentered','1985-11-11 00:00:00','1984-12-18 00:00:00',93.8,424,497,65.82,'Intermediate' from dual union all
select 15566,'015566','1984-104E','Dispenser cover','Romb dispenser cover',57158,'Reentered','1985-12-15 00:00:00','1984-12-18 00:00:00',93.89,430,500,65.85,'Intermediate' from dual union all
select 15567,'015567','1984-104F','Dispenser cover','Romb dispenser cover',57158,'Reentered','1985-12-20 00:00:00','1984-12-27 00:00:00',93.87,429,499,65.8,'Intermediate' from dual union all
select 15568,'015568','1985-012C','Blenda','Sunshade',58235,'Reentered','1985-09-12 00:00:00','1985-02-28 00:00:00',92.03,342,407,72.85,'Intermediate' from dual union all
select 15569,'015569','1985-012D','KDU','KDU',58235,'Reentered','1987-07-14 00:00:00','1985-02-28 00:00:00',92.22,355,412,72.85,'Intermediate' from dual union all
select 1557,'001557','1965-020DD','deb Blok Kosmos','',11770,'Reentered','1967-02-12 00:00:00','1965-08-30 00:00:00',109.97,545,1906,56.09,'Intermediate' from dual union all
select 15570,'015570','1985-012E','KDU part','KDU part',58235,'Reentered','1985-05-31 00:00:00','1985-02-23 00:00:00',92.23,354,414,72.85,'Intermediate' from dual union all
select 15571,'015571','1985-012F','KDU part','KDU part',58235,'Reentered','1985-07-28 00:00:00','1985-02-23 00:00:00',92.23,354,414,72.85,'Intermediate' from dual union all
select 15572,'015572','1985-012G','PTDU cover','KDU part',58235,'Reentered','1986-01-09 00:00:00','1985-03-02 00:00:00',94.41,388,592,72.86,'Intermediate' from dual union all
select 15573,'015573','1985-012H','PTDU cover','KDU part',58235,'Reentered','1985-03-26 00:00:00','1985-02-23 00:00:00',92.64,349,459,72.91,'Intermediate' from dual union all
select 15574,'015574','1985-016A','Kosmos-1629','Oko-S F2',58239,'In Earth orbit','','1985-03-29 00:00:00',1436.24,35785,35793,1.2,'Stationary' from dual union all
select 15575,'015575','1985-016B','8S812','8S812',58239,'Reentered','1985-02-24 00:00:00','1985-02-23 00:00:00',87.9,168,174,51.64,'Intermediate' from dual union all
select 15576,'015576','1985-016C','Perekhodnik','Sredniy perekhodnik',58239,'Reentered','1985-02-22 00:00:00','1985-02-21 00:00:00',88.09,174,185,51.63,'Intermediate' from dual union all
select 15577,'015577','1985-016D','SOZ','SOZ',58239,'Reentered','1986-10-27 00:00:00','1985-02-22 00:00:00',639.84,335,36102,47.42,'GEO Transfer' from dual union all
select 15578,'015578','1985-016E','SOZ','SOZ',58239,'Reentered','1986-10-27 00:00:00','1985-02-27 00:00:00',638.88,286,36102,47.41,'GEO Transfer' from dual;
insert into SATELLITE
select 15579,'015579','1985-012J','KDU part','KDU part',58235,'Reentered','1985-03-14 00:00:00','1985-02-26 00:00:00',91.98,343,401,72.86,'Intermediate' from dual union all
select 1558,'001558','1965-020DE','deb Blok Kosmos','',11770,'Reentered','1968-06-12 00:00:00','1965-08-30 00:00:00',107.1,354,1831,56.17,'Intermediate' from dual union all
select 15580,'015580','1985-012K','KDU part','KDU part',58235,'Reentered','1985-03-18 00:00:00','1985-02-25 00:00:00',92.03,350,398,72.85,'Intermediate' from dual union all
select 15581,'015581','1985-016F','Blok DM','11S86 No. 49L',58239,'In Earth orbit','','1993-06-07 00:00:00',1448.54,35905,36155,6.29,'Inclined Drift' from dual union all
select 15582,'015582','1985-017A','Kosmos-1630','Kobal''t',58240,'Landed','1985-04-23 00:00:00','1985-03-06 00:00:00',89.63,175,337,64.89,'Intermediate' from dual union all
select 15583,'015583','1985-017B','11S510','11S510',58240,'Reentered','1985-03-05 00:00:00','1985-02-27 00:00:00',89.47,174,322,64.91,'Intermediate' from dual union all
select 15584,'015584','1985-018A','Kosmos-1631','Vektor',58241,'Reentered','1990-12-08 00:00:00','1985-10-18 00:00:00',94.41,464,516,65.85,'Intermediate' from dual union all
select 15585,'015585','1985-018B','S3M','S3M',58241,'Reentered','1989-10-20 00:00:00','1985-02-27 00:00:00',94.34,462,512,65.86,'Intermediate' from dual union all
select 15586,'015586','1983-101U','Kosmos-1501 SS 17','ESO',55858,'Reentered','1985-12-23 00:00:00','1985-03-01 00:00:00',94.19,455,504,82.93,'Intermediate' from dual union all
select 15587,'015587','1983-101V','Kosmos-1501 SS 18','ESO',55858,'Reentered','1985-11-20 00:00:00','1985-03-01 00:00:00',93.96,454,483,82.93,'Intermediate' from dual union all
select 15588,'015588','1983-101W','Kosmos-1501 SS 19','ESO',55858,'Reentered','1985-12-20 00:00:00','1985-03-01 00:00:00',94.2,455,504,82.93,'Intermediate' from dual union all
select 15589,'015589','1985-019A','Kosmos-1632','Oblik No. 311',58242,'Landed','1985-03-15 00:00:00','1985-03-03 00:00:00',89.57,209,298,72.87,'Intermediate' from dual union all
select 1559,'001559','1965-020DF','deb Blok Kosmos','',11770,'Reentered','1967-04-18 00:00:00','1965-09-05 00:00:00',105.1,303,1695,56.08,'Intermediate' from dual union all
select 15590,'015590','1985-019B','11S510','11S510',58242,'Reentered','1985-03-04 00:00:00','1985-03-01 00:00:00',88.64,192,223,72.87,'Intermediate' from dual union all
select 15591,'015591','1985-012L','KDU part','KDU part',58235,'Reentered','1985-04-11 00:00:00','1985-03-05 00:00:00',92.81,381,444,73.63,'Intermediate' from dual union all
select 15592,'015592','1985-020A','Kosmos-1633','Tselina-D',58243,'In Earth orbit','','1985-10-18 00:00:00',97.65,636,657,82.54,'Intermediate' from dual union all
select 15593,'015593','1985-020B','S5M','S5M',58243,'In Earth orbit','','1991-03-28 00:00:00',97.45,629,645,82.54,'Intermediate' from dual union all
select 15594,'015594','1983-101X','Kosmos-1501 SS 20','ESO',55858,'Reentered','1985-11-02 00:00:00','1985-03-06 00:00:00',93.94,454,480,82.93,'Intermediate' from dual union all
select 15595,'015595','1985-021A','Geosat 1','Geosat',58244,'In Earth orbit','','1990-01-29 00:00:00',100.57,784,787,108.04,'Retrograde' from dual union all
select 15596,'015596','1985-021B','OIS','Star 27 S/N 0056',58244,'In Earth orbit','','1985-11-05 00:00:00',100.58,757,815,108.04,'Retrograde' from dual union all
select 15597,'015597','1985-022A','Kosmos-1634','Parus',58245,'In Earth orbit','','1985-10-18 00:00:00',104.8,958,1013,82.94,'Intermediate' from dual union all
select 15598,'015598','1985-022B','S3M','S3M',58245,'In Earth orbit','','1985-04-15 00:00:00',104.69,958,1001,82.94,'Intermediate' from dual union all
select 15599,'015599','1981-106D','SA Veneri-13','4V-1 SA',52629,'Deep Space Attached','1981-10-30 00:00:00','1982-03-03 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 156,'000156','61 Omicron 41','deb r/b Ablestar','',7273,'Reentered','1998-09-25 00:00:00','1961-09-18 00:00:00',105.85,967,1102,67.02,'Intermediate' from dual union all
select 1560,'001560','1965-020DG','deb Blok Kosmos','',11770,'Reentered','1965-10-13 00:00:00','1965-03-15 00:00:00',98.4,281,1083,56,'Intermediate' from dual union all
select 15600,'015600','1981-110D','SA Veneri-14','4V-1 SA',52633,'Deep Space Attached','1981-11-04 00:00:00','1982-03-03 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 15601,'015601','1983-044C','deb Kosmos-1461','',55801,'Reentered','1985-03-23 00:00:00','1985-03-14 00:00:00',99.68,657,830,64.7,'Intermediate' from dual union all
select 15602,'015602','1983-044D','deb Kosmos-1461','',55801,'Reentered','1985-03-15 00:00:00','1985-03-14 00:00:00',94.36,341,634,65.14,'Intermediate' from dual union all
select 15603,'015603','1983-044E','deb Kosmos-1461','',55801,'Reentered','1986-10-29 00:00:00','1985-03-14 00:00:00',98.86,544,865,65.22,'Intermediate' from dual union all
select 15604,'015604','1983-044F','deb Kosmos-1461','Medium debris, RCS 1m2',55801,'Reentered','1985-04-22 00:00:00','1985-03-13 00:00:00',96.74,370,836,65.22,'Intermediate' from dual union all
select 15605,'015605','1983-044G','deb Kosmos-1461','',55801,'Reentered','1988-09-04 00:00:00','1985-03-13 00:00:00',100.73,666,921,65.01,'Intermediate' from dual union all
select 15606,'015606','1983-044H','deb Kosmos-1461','',55801,'Reentered','1988-10-22 00:00:00','1985-03-13 00:00:00',99.42,557,905,64.94,'Intermediate' from dual union all
select 15607,'015607','1984-104G','Kosmos-1601 SS 1','ESO',57158,'Reentered','1988-03-22 00:00:00','1985-03-15 00:00:00',94.47,466,520,65.84,'Intermediate' from dual union all
select 15608,'015608','1984-104H','Kosmos-1601 SS 2','ESO',57158,'Reentered','1988-03-03 00:00:00','1985-03-15 00:00:00',94.45,464,520,65.85,'Intermediate' from dual union all
select 15609,'015609','1985-019C','KDU','KDU',58242,'Reentered','1985-03-26 00:00:00','1985-03-15 00:00:00',89.01,210,241,72.89,'Intermediate' from dual union all
select 1561,'001561','1965-020DH','deb Blok Kosmos','',11770,'Reentered','1967-04-12 00:00:00','1965-08-30 00:00:00',105.73,341,1716,55.92,'Intermediate' from dual union all
select 15610,'015610','1985-019D','Blenda','Sunshade',58242,'Reentered','1985-03-17 00:00:00','1985-03-15 00:00:00',88.82,197,236,72.86,'Intermediate' from dual union all
select 15611,'015611','1985-019E','PTDU cover','KDU part',58242,'Reentered','1985-03-24 00:00:00','1985-03-15 00:00:00',91.18,233,432,72.87,'Intermediate' from dual union all
select 15612,'015612','1985-019F','PTDU cover','KDU part',58242,'Reentered','1985-03-20 00:00:00','1985-03-15 00:00:00',91.26,240,433,72.87,'Intermediate' from dual union all
select 15613,'015613','1985-021C','GEOSAT 1 restraint cable','',58244,'Reentered','1992-09-20 00:00:00','1992-09-17 00:00:00',90.24,276,297,108.48,'Retrograde' from dual union all
select 15614,'015614','1985-021D','GEOSAT 1 restraint cable','',58244,'In Earth orbit','','1985-11-05 00:00:00',100.13,746,784,108.24,'Retrograde' from dual union all
select 15615,'015615','1985-021E','GEOSAT 1 despin cable','',58244,'In Earth orbit','','1985-10-26 00:00:00',100.93,758,848,107.84,'Retrograde' from dual union all
select 15616,'015616','1985-021F','GEOSAT 1 despin cable','',58244,'In Earth orbit','','1985-10-23 00:00:00',101.59,743,925,107.58,'Retrograde' from dual union all
select 15617,'015617','1985-023A','Kosmos-1635','Strela-1M',58246,'In Earth orbit','','1985-10-23 00:00:00',115.82,1473,1513,74.05,'Intermediate' from dual union all
select 15618,'015618','1985-023B','Kosmos-1636','Strela-1M',58246,'In Earth orbit','','1993-03-26 00:00:00',115.63,1474,1494,74.05,'Intermediate' from dual union all
select 15619,'015619','1985-023C','Kosmos-1637','Strela-1M',58246,'In Earth orbit','','1993-03-29 00:00:00',115.46,1465,1488,74.06,'Intermediate' from dual union all
select 1562,'001562','1965-020DJ','deb Blok Kosmos','',11770,'Reentered','1966-02-22 00:00:00','1965-08-30 00:00:00',99.56,223,1253,56.16,'Intermediate' from dual union all
select 15620,'015620','1985-023D','Kosmos-1638','Strela-1M',58246,'In Earth orbit','','1993-03-24 00:00:00',115.27,1456,1480,74.05,'Intermediate' from dual union all
select 15621,'015621','1985-023E','Kosmos-1639','Strela-1M',58246,'In Earth orbit','','1993-03-29 00:00:00',115.11,1441,1480,74.05,'Intermediate' from dual union all
select 15622,'015622','1985-023F','Kosmos-1640','Strela-1M',58246,'In Earth orbit','','1993-03-30 00:00:00',114.94,1427,1479,74.05,'Intermediate' from dual union all
select 15623,'015623','1985-023G','Kosmos-1641','Strela-1M',58246,'In Earth orbit','','1993-03-12 00:00:00',114.79,1412,1480,74.05,'Intermediate' from dual union all
select 15624,'015624','1985-023H','Kosmos-1642','Strela-1M',58246,'In Earth orbit','','1993-03-18 00:00:00',114.61,1398,1477,74.05,'Intermediate' from dual union all
select 15625,'015625','1985-023J','S3M','S3M',58246,'In Earth orbit','','1991-02-16 00:00:00',118.05,1476,1711,74.05,'Intermediate' from dual union all
select 15626,'015626','1985-024A','Ekran','Ekran No. 28L',58247,'In Earth orbit','','1985-04-21 00:00:00',1435.96,35778,35790,.36,'Stationary' from dual union all
select 15627,'015627','1985-024B','8S812','8S812',58247,'Reentered','1985-03-25 00:00:00','1985-03-22 00:00:00',88.5,191,210,51.6,'Intermediate' from dual union all
select 15628,'015628','1985-024C','Perekhodnik','Sredniy perekhodnik',58247,'Reentered','1985-03-23 00:00:00','1985-03-22 00:00:00',88.56,190,217,51.6,'Intermediate' from dual union all
select 15629,'015629','1985-025A','INTELSAT 510','INTELSAT VA F10',58248,'In Earth orbit','','1985-04-26 00:00:00',1442.81,35909,35927,.14,'Drift GEO' from dual union all
select 1563,'001563','1965-020DK','deb Blok Kosmos','',11770,'Reentered','1969-10-28 00:00:00','1965-09-25 00:00:00',105.81,293,1772,56.06,'Intermediate' from dual union all
select 15630,'015630','1985-024D','Blok DM','11S86 No. 53L',58247,'In Earth orbit','','1985-04-22 00:00:00',1422.49,35471,35569,.4,'Drift GEO' from dual union all
select 15631,'015631','1985-025B','Centaur AC-63','Centaur D-1AR AC-63',58248,'Reentered','1995-11-11 00:00:00','1993-03-26 00:00:00',305.92,195,17128,22.91,'Highly Ellip' from dual union all
select 15632,'015632','1985-024E','SOZ','SOZ',58247,'Reentered','1986-08-07 00:00:00','1985-03-22 00:00:00',629.47,315,35589,47.61,'GEO Transfer' from dual union all
select 15633,'015633','1985-024F','SOZ','SOZ',58247,'Reentered','1986-01-20 00:00:00','1985-03-23 00:00:00',628.01,262,35567,47.44,'GEO Transfer' from dual union all
select 15634,'015634','1985-026A','Kosmos-1643','Terilen',58249,'Deorbited','1985-10-18 00:00:00','1985-04-24 00:00:00',89.33,225,257,64.77,'Intermediate' from dual union all
select 15635,'015635','1985-026B','11S510','11S510',58249,'Reentered','1985-03-29 00:00:00','1985-03-25 00:00:00',88.87,178,260,64.77,'Intermediate' from dual union all
select 15636,'015636','1985-027A','Kosmos-1644','Oblik',58250,'Landed','1985-04-17 00:00:00','1985-04-05 00:00:00',92.17,348,414,70.35,'Intermediate' from dual union all
select 15637,'015637','1985-027B','11S510','11S510',58250,'Reentered','1985-04-22 00:00:00','1985-04-03 00:00:00',90.23,206,366,70.34,'Intermediate' from dual union all
select 15638,'015638','1985-027C','Sensor cover','Sensor cover',58250,'Reentered','1985-04-07 00:00:00','1985-04-04 00:00:00',90.34,204,378,70.37,'Intermediate' from dual union all
select 15639,'015639','1985-027D','Sensor cover','Sensor cover',58250,'Reentered','1985-04-06 00:00:00','1985-04-04 00:00:00',89.75,201,324,70.37,'Intermediate' from dual union all
select 1564,'001564','1965-020DL','deb Blok Kosmos','',11770,'Reentered','1981-09-10 00:00:00','1965-08-29 00:00:00',109.47,540,1865,56.18,'Intermediate' from dual union all
select 15640,'015640','1985-027E','Sensor cover','Sensor cover',58250,'Reentered','1985-04-07 00:00:00','1985-04-04 00:00:00',90.14,194,369,70.21,'Intermediate' from dual union all
select 15641,'015641','1985-028A','Discovery (STS 51-D)','OV-103',58251,'Landed','1985-04-19 00:00:00','1985-04-14 00:00:00',92.3,314,461,28.53,'Intermediate' from dual union all
select 15642,'015642','1985-028B','Anik C1','Anik C1',58251,'In Earth orbit','','1985-05-13 00:00:00',1436.04,35778,35793,2.85,'Stationary' from dual union all
select 15643,'015643','1985-028C','Leasat 3','Leasat 3',58251,'In Earth orbit','','1985-11-20 00:00:00',1435.83,35594,35969,3.19,'Inclined GEO' from dual union all
select 15644,'015644','1985-028D','PAM-D','Star 48  S/N 10040-3',58251,'In Earth orbit','','1985-05-13 00:00:00',626.23,330,35407,22.87,'GEO Transfer' from dual union all
select 15645,'015645','1985-029A','Kosmos-1645','Foton 34KS No. 1L',58252,'Landed','1985-04-29 00:00:00','1985-04-18 00:00:00',90.54,214,388,62.83,'Intermediate' from dual union all
select 15646,'015646','1985-029B','11S510','11S510',58252,'Reentered','1985-05-10 00:00:00','1985-04-16 00:00:00',90.44,215,377,62.82,'Intermediate' from dual union all
select 15647,'015647','1985-027F','Blenda','Sunshade',58250,'Reentered','1986-01-07 00:00:00','1985-04-17 00:00:00',94.25,377,588,70.35,'Intermediate' from dual union all
select 15648,'015648','1985-027G','KDU part','KDU part',58250,'Reentered','1985-10-15 00:00:00','1985-04-17 00:00:00',92.01,336,410,70.35,'Intermediate' from dual union all
select 15649,'015649','1985-027H','KDU part','KDU part',58250,'Reentered','1985-04-26 00:00:00','1985-04-17 00:00:00',91.71,301,416,70.65,'Intermediate' from dual union all
select 1565,'001565','1965-020DM','deb Blok Kosmos','',11770,'Reentered','1967-02-02 00:00:00','1965-08-30 00:00:00',104.45,282,1656,56.09,'Intermediate' from dual union all
select 15650,'015650','1985-027J','KDU','KDU',58250,'Reentered','1987-07-09 00:00:00','1985-04-17 00:00:00',92.1,349,406,70.37,'Intermediate' from dual union all
select 15651,'015651','1985-027K','KDU part','KDU part',58250,'Reentered','1985-09-26 00:00:00','1985-04-17 00:00:00',94.81,381,638,70.35,'Intermediate' from dual union all
select 15652,'015652','1985-027L','KDU part','KDU part',58250,'Reentered','1985-06-14 00:00:00','1985-04-17 00:00:00',95.82,382,735,70.36,'Intermediate' from dual union all
select 15653,'015653','1985-030A','Kosmos-1646','US-PM1',58253,'Reentered','1988-05-12 00:00:00','1985-05-18 00:00:00',93.28,428,442,65.05,'Intermediate' from dual union all
select 15654,'015654','1985-030B','11S692','11S692',58253,'Reentered','1985-04-19 00:00:00','1985-04-19 00:00:00',89.47,111,385,65.03,'Intermediate' from dual union all
select 15655,'015655','1985-031A','Kosmos-1647','Kobal''t No. 503',58254,'Landed','1985-06-11 00:00:00','1985-04-26 00:00:00',89.35,169,316,67.13,'Intermediate' from dual union all
select 15656,'015656','1985-031B','11S510','11S510',58254,'Reentered','1985-04-23 00:00:00','1985-04-19 00:00:00',89.3,167,313,67.14,'Intermediate' from dual union all
select 15657,'015657','1985-027M','KDU part','KDU part',58250,'Reentered','1985-08-07 00:00:00','1985-04-20 00:00:00',92.18,349,414,70.35,'Intermediate' from dual union all
select 15658,'015658','1985-027N','KDU part','KDU part',58250,'Reentered','1985-06-16 00:00:00','1985-04-24 00:00:00',93.23,363,503,69.51,'Intermediate' from dual union all
select 15659,'015659','1985-032A','Kosmos-1648','Oblik No. 313',58255,'Landed','1985-05-06 00:00:00','1985-04-27 00:00:00',90.04,227,326,82.33,'Intermediate' from dual union all
select 1566,'001566','1965-020DN','deb Blok Kosmos','',11770,'Reentered','1979-12-08 00:00:00','1965-09-08 00:00:00',109.89,504,1940,55.73,'Intermediate' from dual union all
select 15660,'015660','1985-032B','11S510','11S510',58255,'Reentered','1985-04-27 00:00:00','1985-04-25 00:00:00',88.66,177,240,82.36,'Intermediate' from dual union all
select 15661,'015661','1985-033A','Prognoz-10-IK','SO-M No. 510',58256,'Deep Space','1985-04-27 00:00:00','1985-12-15 00:00:00',5783.81,5974,194737,76.76,'Deep Space' from dual union all
select 15662,'015662','1985-033B','11S510','11S510',58256,'Reentered','1985-06-20 00:00:00','1985-04-28 00:00:00',91.28,227,448,64.98,'Intermediate' from dual union all
select 15663,'015663','1985-033C','BOZ','BOZ',58256,'Reentered','1985-06-28 00:00:00','1985-04-26 00:00:00',91.53,227,473,64.96,'Intermediate' from dual union all
select 15664,'015664','1985-033D','Blok SO-L','Blok SO-L No. 71634-804',58256,'Deep Space','1985-04-27 00:00:00','1985-04-26 00:00:00',5784.99,420,200320,64.99,'Deep Space' from dual union all
select 15665,'015665','1985-034A','Challenger (STS 51-B)','OV-099',58257,'Landed','1985-05-06 00:00:00','1985-05-01 00:00:00',91.57,345,359,56.99,'Intermediate' from dual union all
select 15666,'015666','1985-034B','Nusat','Nusat',58257,'Reentered','1986-12-15 00:00:00','1985-05-31 00:00:00',91.48,339,355,56.99,'Intermediate' from dual union all
select 15667,'015667','1985-029C','KB electropitaniya','KBE',58252,'Reentered','1985-06-23 00:00:00','1985-04-30 00:00:00',90.38,214,373,62.83,'Intermediate' from dual union all
select 15668,'015668','1985-029D','KBE part','KBE part',58252,'Reentered','1985-05-13 00:00:00','1985-04-30 00:00:00',90.18,210,357,62.83,'Intermediate' from dual;
insert into SATELLITE
select 15669,'015669','1985-029E','PTDU cover','KBE part',58252,'Reentered','1985-06-04 00:00:00','1985-04-30 00:00:00',92.15,213,546,62.83,'Intermediate' from dual union all
select 1567,'001567','1965-020DP','deb Blok Kosmos','',11770,'Reentered','1974-09-12 00:00:00','1966-05-06 00:00:00',109.19,453,1926,55.82,'Intermediate' from dual union all
select 15670,'015670','1985-029F','PTDU cover','KBE part',58252,'Reentered','1985-05-19 00:00:00','1985-04-30 00:00:00',94.43,321,661,62.82,'Intermediate' from dual union all
select 15671,'015671','1985-029G','KBE part','KBE part',58252,'Reentered','1985-05-01 00:00:00','1985-04-29 00:00:00',90.32,215,365,62.83,'Intermediate' from dual union all
select 15672,'015672','1985-032C','Blenda','Sunshade',58255,'Reentered','1985-05-10 00:00:00','1985-05-06 00:00:00',88.78,205,224,82.33,'Intermediate' from dual union all
select 15673,'015673','1985-032D','KDU part','KDU part',58255,'Reentered','1985-05-15 00:00:00','1985-05-06 00:00:00',91.05,241,411,82.33,'Intermediate' from dual union all
select 15674,'015674','1985-032E','KDU','KDU',58255,'Reentered','1985-05-21 00:00:00','1985-05-06 00:00:00',89.16,215,251,82.33,'Intermediate' from dual union all
select 15675,'015675','1985-032F','KDU part','KDU part',58255,'Reentered','1985-05-11 00:00:00','1985-05-06 00:00:00',91.08,238,418,82.34,'Intermediate' from dual union all
select 15676,'015676','1985-032G','KDU part','KDU part',58255,'Reentered','1985-05-09 00:00:00','1985-05-06 00:00:00',88.71,197,224,82.33,'Intermediate' from dual union all
select 15677,'015677','1985-035A','Gstar 1','Gstar 1A',58258,'In Earth orbit','','1985-06-07 00:00:00',1436.04,35782,35789,.3,'Stationary' from dual union all
select 15678,'015678','1985-035B','Telecom 1B','Telecom 1B',58258,'In Earth orbit','','1985-06-09 00:00:00',1436.09,35779,35794,.01,'Stationary' from dual union all
select 15679,'015679','1985-035C','Ariane H10 V13','Ariane H10 S/N T15',58258,'Reentered','2002-03-27 00:00:00','1985-06-13 00:00:00',641.61,235,36293,7.21,'GEO Transfer' from dual union all
select 1568,'001568','1965-020DQ','deb Blok Kosmos','',11770,'Reentered','1966-05-01 00:00:00','1965-08-29 00:00:00',102.59,277,1486,56.13,'Intermediate' from dual union all
select 15680,'015680','1985-035D','Sylda 4400 No. 4','Sylda 4400',58258,'In Earth orbit','','1985-06-09 00:00:00',613.89,216,34882,7.19,'GEO Transfer' from dual union all
select 15681,'015681','1983-044J','deb Kosmos-1461','',55801,'Reentered','1988-08-18 00:00:00','1985-05-01 00:00:00',99.25,559,887,65.07,'Intermediate' from dual union all
select 15682,'015682','1983-044K','deb Kosmos-1461','',55801,'Reentered','1989-02-18 00:00:00','1985-05-01 00:00:00',99.93,638,873,65.05,'Intermediate' from dual union all
select 15683,'015683','1983-044L','deb Kosmos-1461','',55801,'Reentered','1988-08-24 00:00:00','1985-05-01 00:00:00',99.27,552,896,65.06,'Intermediate' from dual union all
select 15684,'015684','1983-044M','deb Kosmos-1461','',55801,'Reentered','1989-10-06 00:00:00','1985-05-01 00:00:00',100.54,653,915,65.01,'Intermediate' from dual union all
select 15685,'015685','1983-044N','deb Kosmos-1461','',55801,'Reentered','1990-12-16 00:00:00','1985-05-01 00:00:00',100.64,651,926,65.35,'Intermediate' from dual union all
select 15686,'015686','1983-044P','deb Kosmos-1461','',55801,'Reentered','1988-10-22 00:00:00','1985-05-01 00:00:00',99.72,597,893,65.02,'Intermediate' from dual union all
select 15687,'015687','1983-044Q','deb Kosmos-1461','',55801,'Reentered','1989-03-27 00:00:00','1985-05-01 00:00:00',102.86,797,991,64.47,'Intermediate' from dual union all
select 15688,'015688','1983-044R','deb Kosmos-1461','',55801,'Reentered','1987-07-04 00:00:00','1985-05-01 00:00:00',98.95,555,861,64.98,'Intermediate' from dual union all
select 15689,'015689','1983-044S','deb Kosmos-1461','',55801,'Reentered','1987-07-24 00:00:00','1985-05-01 00:00:00',98.84,556,851,64.3,'Intermediate' from dual union all
select 1569,'001569','1965-020DR','deb Blok Kosmos','',11770,'Reentered','1966-07-18 00:00:00','1965-09-01 00:00:00',102.31,279,1457,56.05,'Intermediate' from dual union all
select 15690,'015690','1983-044T','deb Kosmos-1461','Medium debris, RCS 1m2',55801,'Reentered','1989-02-01 00:00:00','1985-05-01 00:00:00',101.92,753,946,64.69,'Intermediate' from dual union all
select 15691,'015691','1983-044U','deb Kosmos-1461','',55801,'Reentered','1988-04-07 00:00:00','1985-05-01 00:00:00',99.96,622,892,65.29,'Intermediate' from dual union all
select 15692,'015692','1983-044V','deb Kosmos-1461','',55801,'Reentered','1989-01-17 00:00:00','1985-05-01 00:00:00',99.41,575,885,65.07,'Intermediate' from dual union all
select 15693,'015693','1984-063F','Blok DM','11S86 No. 47L',57117,'In Earth orbit','','1993-06-07 00:00:00',1394.09,34885,35036,6.58,'Inclined Drift' from dual union all
select 15694,'015694','1985-036A','Kosmos-1649','Oblik No. 314',58259,'Landed','1985-05-29 00:00:00','1985-05-17 00:00:00',92.24,355,414,72.85,'Intermediate' from dual union all
select 15695,'015695','1985-036B','11S510','11S510',58259,'Reentered','1985-05-28 00:00:00','1985-05-15 00:00:00',90.07,195,361,72.83,'Intermediate' from dual union all
select 15696,'015696','1985-036C','Sensor cover','Sensor cover',58259,'Reentered','1985-05-18 00:00:00','1985-05-15 00:00:00',90.14,191,372,72.82,'Intermediate' from dual union all
select 15697,'015697','1985-037A','Kosmos-1650','Uragan No. 20L',58260,'In Earth orbit','','1985-06-24 00:00:00',675.71,19087,19172,64.82,'Medium' from dual union all
select 15698,'015698','1985-037B','Kosmos-1651','Uragan No. 21L',58260,'In Earth orbit','','1985-06-21 00:00:00',675.71,19122,19137,64.8,'Medium' from dual union all
select 15699,'015699','1985-037C','Kosmos-1652','Uragan GVM',58260,'In Earth orbit','','1985-07-22 00:00:00',675.81,19119,19145,64.8,'Medium' from dual union all
select 157,'000157','61 Omicron 42','deb r/b Ablestar','',7273,'Reentered','1981-01-08 00:00:00','1961-09-13 00:00:00',100.35,555,996,66.83,'Intermediate' from dual union all
select 1570,'001570','1965-070A','Kosmos-80','Strela-1',11820,'In Earth orbit','','1965-10-09 00:00:00',114.94,1356,1549,56.05,'Intermediate' from dual union all
select 15700,'015700','1985-037D','8S812','8S812',58260,'Reentered','1985-05-20 00:00:00','1985-05-18 00:00:00',88.23,178,196,51.62,'Intermediate' from dual union all
select 15701,'015701','1985-037E','Perekhodnik','Sredniy perekhodnik',58260,'Reentered','1985-05-18 00:00:00','1985-05-17 00:00:00',88.39,187,203,51.61,'Intermediate' from dual union all
select 15702,'015702','1985-037F','Blok DM2','11S861 No. 7L',58260,'In Earth orbit','','1985-10-22 00:00:00',675.02,19063,19161,64.8,'Medium' from dual union all
select 15703,'015703','1983-044W','deb Kosmos-1461','',55801,'Reentered','1989-02-17 00:00:00','1985-05-20 00:00:00',101.94,732,969,64.76,'Intermediate' from dual union all
select 15704,'015704','1983-044X','deb Kosmos-1461','',55801,'Reentered','1989-03-09 00:00:00','1985-05-20 00:00:00',98.79,542,859,64.5,'Intermediate' from dual union all
select 15705,'015705','1983-044Y','deb Kosmos-1461','',55801,'Reentered','1989-04-28 00:00:00','1985-05-20 00:00:00',99.34,569,886,65.04,'Intermediate' from dual union all
select 15706,'015706','1983-044Z','deb Kosmos-1461','',55801,'Reentered','1989-03-26 00:00:00','1985-05-20 00:00:00',99.55,604,871,65.02,'Intermediate' from dual union all
select 15707,'015707','1983-044AA','Kosmos-1461 major debris','',55801,'Reentered','1999-12-27 00:00:00','1993-08-05 00:00:00',96.13,513,634,65.02,'Intermediate' from dual union all
select 15708,'015708','1983-044AB','deb Kosmos-1461','',55801,'Reentered','1989-10-21 00:00:00','1985-08-14 00:00:00',99.31,571,881,65.02,'Intermediate' from dual union all
select 15709,'015709','1983-044AC','deb Kosmos-1461','',55801,'Reentered','1989-03-08 00:00:00','1985-05-14 00:00:00',99.28,555,894,65.03,'Intermediate' from dual union all
select 1571,'001571','1965-070B','Kosmos-81','Strela-1',11820,'In Earth orbit','','1965-10-11 00:00:00',115.32,1386,1554,56.05,'Intermediate' from dual union all
select 15710,'015710','1983-044AD','deb Kosmos-1461','',55801,'Reentered','1989-08-05 00:00:00','1985-05-14 00:00:00',99.33,562,892,65.04,'Intermediate' from dual union all
select 15711,'015711','1983-044AE','deb Kosmos-1461','',55801,'Reentered','1987-08-15 00:00:00','1985-05-16 00:00:00',99.31,558,894,65.32,'Intermediate' from dual union all
select 15712,'015712','1983-044AF','deb Kosmos-1461','',55801,'Reentered','1988-12-16 00:00:00','1985-05-17 00:00:00',101.76,693,991,65.48,'Intermediate' from dual union all
select 15713,'015713','1983-044AG','deb Kosmos-1461','',55801,'Reentered','1991-08-17 00:00:00','1985-05-16 00:00:00',99.37,571,886,65.05,'Intermediate' from dual union all
select 15714,'015714','1985-037G','SOZ','SOZ',58260,'In Earth orbit','','1985-07-17 00:00:00',340.63,398,19159,51.91,'Highly Ellip' from dual union all
select 15715,'015715','1985-037H','SOZ','SOZ',58260,'In Earth orbit','','1985-07-17 00:00:00',340.31,378,19159,52.02,'Highly Ellip' from dual union all
select 15716,'015716','1983-044AH','deb Kosmos-1461','',55801,'Reentered','1989-05-23 00:00:00','1985-05-21 00:00:00',100.73,666,921,64.92,'Intermediate' from dual union all
select 15717,'015717','1983-044AJ','deb Kosmos-1461','',55801,'Reentered','1989-06-03 00:00:00','1985-05-20 00:00:00',100.33,634,915,64.96,'Intermediate' from dual union all
select 15718,'015718','1983-044AK','deb Kosmos-1461','',55801,'Reentered','1990-09-02 00:00:00','1985-05-21 00:00:00',99.12,539,895,65.05,'Intermediate' from dual union all
select 15719,'015719','1983-044AL','deb Kosmos-1461','',55801,'Reentered','1991-03-22 00:00:00','1985-05-21 00:00:00',99.38,573,885,65.03,'Intermediate' from dual union all
select 1572,'001572','1965-070C','Kosmos-82','Strela-1',11820,'In Earth orbit','','1967-08-28 00:00:00',115.66,1412,1559,56.07,'Intermediate' from dual union all
select 15720,'015720','1983-044AM','deb Kosmos-1461','',55801,'Reentered','1989-03-20 00:00:00','1985-05-20 00:00:00',99.06,539,888,65.05,'Intermediate' from dual union all
select 15721,'015721','1983-044AN','deb Kosmos-1461','',55801,'Reentered','1987-04-15 00:00:00','1985-06-04 00:00:00',99.36,571,885,65.03,'Intermediate' from dual union all
select 15722,'015722','1983-044AP','deb Kosmos-1461','',55801,'Reentered','1990-04-09 00:00:00','1985-05-20 00:00:00',99.23,548,896,65,'Intermediate' from dual union all
select 15723,'015723','1983-044AQ','deb Kosmos-1461','',55801,'Reentered','1989-02-19 00:00:00','1985-05-20 00:00:00',99.81,622,876,64.87,'Intermediate' from dual union all
select 15724,'015724','1983-044AR','deb Kosmos-1461','',55801,'Reentered','1989-06-26 00:00:00','1985-05-20 00:00:00',99.24,557,888,65.03,'Intermediate' from dual union all
select 15725,'015725','1983-044AS','deb Kosmos-1461','',55801,'Reentered','1985-06-06 00:00:00','1985-05-21 00:00:00',95.56,323,769,64.37,'Intermediate' from dual union all
select 15726,'015726','1983-044AT','deb Kosmos-1461','',55801,'Reentered','1989-03-05 00:00:00','1985-05-20 00:00:00',101.52,682,980,65.05,'Intermediate' from dual union all
select 15727,'015727','1983-044AU','deb Kosmos-1461','',55801,'Reentered','1989-03-16 00:00:00','1985-05-16 00:00:00',99.51,582,889,65.02,'Intermediate' from dual union all
select 15728,'015728','1983-044AV','deb Kosmos-1461','',55801,'Reentered','1989-11-28 00:00:00','1985-05-20 00:00:00',99.36,570,886,65.03,'Intermediate' from dual union all
select 15729,'015729','1983-044AW','deb Kosmos-1461','',55801,'Reentered','1990-07-18 00:00:00','1985-05-20 00:00:00',99.28,556,893,65.07,'Intermediate' from dual union all
select 1573,'001573','1965-070D','Kosmos-83','Strela-1',11820,'In Earth orbit','','1965-10-08 00:00:00',116.04,1440,1566,56.05,'Intermediate' from dual union all
select 15730,'015730','1983-044AX','deb Kosmos-1461','',55801,'Reentered','1985-06-11 00:00:00','1985-05-18 00:00:00',98.91,521,893,65.03,'Intermediate' from dual union all
select 15731,'015731','1983-044AY','deb Kosmos-1461','',55801,'Reentered','1990-04-29 00:00:00','1985-05-20 00:00:00',99.18,550,888,65.07,'Intermediate' from dual union all
select 15732,'015732','1985-038A','Kosmos-1653','Resurs-F1 17F41 No. 52',58261,'Landed','1985-06-05 00:00:00','1985-05-25 00:00:00',89.8,258,272,82.33,'Intermediate' from dual union all
select 15733,'015733','1985-038B','11S510','11S510',58261,'Reentered','1985-06-02 00:00:00','1985-05-25 00:00:00',89.17,203,263,82.33,'Intermediate' from dual union all
select 15734,'015734','1985-039A','Kosmos-1654','Kobal''t',58262,'Reentered','1985-08-07 00:00:00','1985-07-24 00:00:00',89.22,177,294,64.69,'Intermediate' from dual union all
select 15735,'015735','1985-039B','11S510','11S510',58262,'Reentered','1985-05-29 00:00:00','1985-05-26 00:00:00',88.97,164,284,64.87,'Intermediate' from dual union all
select 15736,'015736','1983-044AZ','deb Kosmos-1461','',55801,'Reentered','1989-06-26 00:00:00','1985-05-28 00:00:00',99.9,611,897,65,'Intermediate' from dual union all
select 15737,'015737','1983-044BA','deb Kosmos-1461','',55801,'Reentered','1988-09-14 00:00:00','1985-06-10 00:00:00',100.64,613,964,65.42,'Intermediate' from dual union all
select 15738,'015738','1985-040A','Molniya-3','Molniya-3 No. 39',58263,'In Earth orbit','','1985-07-29 00:00:00',717.66,520,39828,62.83,'Molniya' from dual union all
select 15739,'015739','1985-040B','11S510','11S510',58263,'Reentered','1985-07-04 00:00:00','1985-05-29 00:00:00',91.01,202,447,62.8,'Intermediate' from dual union all
select 1574,'001574','1965-070E','Kosmos-84','Strela-1',11820,'In Earth orbit','','1965-10-15 00:00:00',116.42,1468,1571,56.05,'Intermediate' from dual union all
select 15740,'015740','1985-040C','BOZ','BOZ',58263,'Reentered','1985-06-25 00:00:00','1985-06-05 00:00:00',90.57,195,410,62.82,'Intermediate' from dual union all
select 15741,'015741','1985-040D','Blok-ML','Blok-ML No. 327',58263,'In Earth orbit','','1985-07-29 00:00:00',732.28,527,40541,62.83,'Molniya' from dual union all
select 15742,'015742','1985-036D','Blenda','Sunshade',58259,'Reentered','1986-03-19 00:00:00','1985-07-14 00:00:00',94.15,386,569,72.85,'Intermediate' from dual union all
select 15743,'015743','1985-036E','KDU part','KDU part',58259,'Reentered','1985-08-10 00:00:00','1985-07-02 00:00:00',92.5,369,425,71.96,'Intermediate' from dual union all
select 15744,'015744','1985-036F','KDU part','KDU part',58259,'Reentered','1985-06-29 00:00:00','1985-05-29 00:00:00',92.18,348,415,72.84,'Intermediate' from dual union all
select 15745,'015745','1985-036G','KDU part','KDU part',58259,'Reentered','1985-10-01 00:00:00','1985-07-14 00:00:00',91.81,343,384,72.85,'Intermediate' from dual union all
select 15746,'015746','1985-036H','KDU','KDU',58259,'Reentered','1985-12-17 00:00:00','1985-07-14 00:00:00',91.85,336,394,72.84,'Intermediate' from dual union all
select 15747,'015747','1985-036J','KDU part','KDU part',58259,'Reentered','1985-06-03 00:00:00','1985-05-29 00:00:00',91.81,333,393,72.87,'Intermediate' from dual union all
select 15748,'015748','1985-040E','deb Blok-I','',58263,'Reentered','1985-06-17 00:00:00','1985-06-05 00:00:00',91.41,210,478,62.83,'Intermediate' from dual union all
select 15749,'015749','1985-040F','deb Blok-I','',58263,'Reentered','1985-05-31 00:00:00','1985-05-30 00:00:00',91.08,209,447,62.81,'Intermediate' from dual union all
select 1575,'001575','1965-070F','S3','S3',11820,'In Earth orbit','','1967-07-05 00:00:00',114.58,1357,1516,56.12,'Intermediate' from dual union all
select 15750,'015750','1985-040G','deb Blok-I','',58263,'Reentered','1985-06-01 00:00:00','1985-05-29 00:00:00',91.94,212,527,62.77,'Intermediate' from dual union all
select 15751,'015751','1985-041A','Kosmos-1655','Tsikada',58264,'In Earth orbit','','1985-06-29 00:00:00',105.05,977,1017,82.95,'Intermediate' from dual union all
select 15752,'015752','1985-041B','S3M','S3M',58264,'In Earth orbit','','1991-03-25 00:00:00',104.93,973,1010,82.95,'Intermediate' from dual union all
select 15753,'015753','1985-036K','KDU part','KDU part',58259,'Reentered','1985-06-14 00:00:00','1985-05-30 00:00:00',93.49,289,602,72.68,'Intermediate' from dual union all
select 15754,'015754','1985-036L','KDU part','KDU part',58259,'Reentered','1987-10-26 00:00:00','1985-07-14 00:00:00',92.19,355,409,72.85,'Intermediate' from dual union all
select 15755,'015755','1985-042A','Kosmos-1656','Tselina-2',58265,'In Earth orbit','','1985-06-30 00:00:00',101.57,807,858,71.11,'Intermediate' from dual union all
select 15756,'015756','1983-079D','deb Kosmos-1486','Strela-2M debris 1',55836,'In Earth orbit','','1985-02-26 00:00:00',100.67,778,802,74.07,'Intermediate' from dual union all
select 15757,'015757','1984-056D','deb Kosmos-1570','Strela-2M debris 1',57110,'Reentered','1991-04-14 00:00:00','1985-02-21 00:00:00',100.68,776,806,74.06,'Intermediate' from dual union all
select 15758,'015758','1968-091CZ','deb Kosmos-249','',16818,'Reentered','2001-01-18 00:00:00','1991-05-20 00:00:00',101.77,506,1179,62.26,'Intermediate' from dual;
insert into SATELLITE
select 15759,'015759','1980-030AW','deb Kosmos-1174','',50240,'Reentered','1988-02-14 00:00:00','1985-02-25 00:00:00',104.46,381,1558,66.12,'Intermediate' from dual union all
select 1576,'001576','58 Beta 3','Vanguard TV4 clamp','Adapter clamp',4793,'In Earth orbit','','1965-09-02 00:00:00',132.71,663,3818,34.21,'Medium' from dual union all
select 15760,'015760','61 Omicron 270','deb r/b (Ablestar)','',7273,'Reentered','1987-08-12 00:00:00','1985-05-30 00:00:00',106.28,531,1578,62.76,'Intermediate' from dual union all
select 15761,'015761','1971-015CW','deb Kosmos-397','',24833,'Reentered','1986-12-16 00:00:00','1984-12-18 00:00:00',96.63,460,735,65.76,'Intermediate' from dual union all
select 15762,'015762','1976-126BU','deb Kosmos-886','',39078,'In Earth orbit','','1993-12-31 00:00:00',114.68,841,2041,65.66,'Intermediate' from dual union all
select 15763,'015763','1980-089BW','deb Kosmos-1220','',50299,'Reentered','1988-08-19 00:00:00','1985-02-18 00:00:00',96.82,516,697,64.98,'Intermediate' from dual union all
select 15764,'015764','1973-081C','NNS O-20 despin weight','',30683,'In Earth orbit','','1993-08-17 00:00:00',105.8,891,1173,90.48,'Polar' from dual union all
select 15765,'015765','1985-036M','KDU part','KDU part',58259,'Reentered','1985-06-17 00:00:00','1985-05-30 00:00:00',92.01,340,407,71.97,'Intermediate' from dual union all
select 15766,'015766','1981-053KB','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',103.28,907,921,82.96,'Intermediate' from dual union all
select 15767,'015767','1981-053KC','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-08 00:00:00',103.52,910,940,82.92,'Intermediate' from dual union all
select 15768,'015768','1981-053KD','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',104.49,925,1016,83.11,'Intermediate' from dual union all
select 15769,'015769','1981-043C','deb Blok E','',52566,'In Earth orbit','','1993-08-17 00:00:00',102.41,828,918,81.27,'Intermediate' from dual union all
select 1577,'001577','1965-065K','SURCAL 5','SURCAL',11815,'In Earth orbit','','1965-09-26 00:00:00',108.11,1083,1196,90.03,'Polar' from dual union all
select 15770,'015770','1985-042B','8S812','8S812',58265,'Reentered','1985-06-03 00:00:00','1985-05-30 00:00:00',88.36,187,200,51.62,'Intermediate' from dual union all
select 15771,'015771','1985-042C','Perekhodnik','Sredniy perekhodnik',58265,'Reentered','1985-05-31 00:00:00','1985-05-30 00:00:00',88,168,183,51.61,'Intermediate' from dual union all
select 15772,'015772','1985-042D','Blok DM2','11S861 No. 8L',58265,'In Earth orbit','','1991-03-25 00:00:00',101.48,802,856,71.12,'Intermediate' from dual union all
select 15773,'015773','1985-042E','SOZ','SOZ',58265,'In Earth orbit','','1991-04-18 00:00:00',101.15,788,838,66.59,'Intermediate' from dual union all
select 15774,'015774','1985-042F','SOZ','SOZ',58265,'In Earth orbit','','1991-04-19 00:00:00',101.28,800,839,66.57,'Intermediate' from dual union all
select 15775,'015775','1968-091DA','deb Kosmos-249','',16818,'In Earth orbit','','1991-09-09 00:00:00',113.6,785,1999,62.32,'Intermediate' from dual union all
select 15776,'015776','1968-091DB','deb Kosmos-249','',16818,'Reentered','1986-04-06 00:00:00','1985-05-16 00:00:00',113.28,755,2000,62.28,'Intermediate' from dual union all
select 15777,'015777','1971-015CX','deb Kosmos-397','',24833,'Reentered','1991-07-31 00:00:00','1991-07-04 00:00:00',94.04,417,527,65.6,'Intermediate' from dual union all
select 15778,'015778','1978-026GR','deb Delta 139','',45221,'Reentered','2001-09-09 00:00:00','1993-08-08 00:00:00',96.85,526,690,98.82,'Sun-Synch' from dual union all
select 15779,'015779','1978-026GS','deb Delta 139','',45221,'Reentered','2009-04-27 00:00:00','1993-07-30 00:00:00',101.23,753,881,99.02,'Sun-Synch' from dual union all
select 1578,'001578','1965-071A','Kosmos-85','Zenit-4 No. 8',11821,'Landed','1965-09-17 00:00:00','1965-09-09 00:00:00',89.38,203,285,64.95,'Intermediate' from dual union all
select 15780,'015780','1981-053KE','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-29 00:00:00',104.73,934,1029,82.82,'Intermediate' from dual union all
select 15781,'015781','1980-030AX','deb Kosmos-1174','',50240,'Reentered','2000-11-04 00:00:00','1993-08-07 00:00:00',103.52,586,1263,65.77,'Intermediate' from dual union all
select 15782,'015782','1970-089DA','deb Kosmos-374','',21566,'Reentered','1991-12-08 00:00:00','1991-12-04 00:00:00',91.18,304,361,62.53,'Intermediate' from dual union all
select 15783,'015783','61 Omicron 271','deb r/b (Ablestar)','',7273,'In Earth orbit','','1993-08-06 00:00:00',100.89,733,868,67.05,'Intermediate' from dual union all
select 15784,'015784','1979-060D','deb Kosmos-1110','Strela-2M debris',47544,'In Earth orbit','','1993-08-16 00:00:00',99.84,748,754,74.01,'Intermediate' from dual union all
select 15785,'015785','1984-019C','deb Kosmos-1538','Strela-2M debris',57073,'In Earth orbit','','1985-02-21 00:00:00',100.83,786,809,74.04,'Intermediate' from dual union all
select 15786,'015786','1964-006G','deb Elektron-1','',10292,'Reentered','2000-08-06 00:00:00','2000-08-05 00:00:00',89.29,202,277,60.6,'Intermediate' from dual union all
select 15787,'015787','1971-015CY','deb Kosmos-397','',24833,'Reentered','1990-05-29 00:00:00','1985-05-30 00:00:00',112.36,579,2092,65.7,'Intermediate' from dual union all
select 15788,'015788','1978-026GT','deb Delta 139','',45221,'In Earth orbit','','1993-08-02 00:00:00',104.46,867,1072,98.87,'Sun-Synch' from dual union all
select 15789,'015789','1973-086GT','Unknown deb                       ','',30688,'In Earth orbit','','1993-08-07 00:00:00',120.8,1330,2104,103.33,'Medium' from dual union all
select 1579,'001579','1965-071B','11A57I','11A57I',11821,'Reentered','1965-09-18 00:00:00','1965-09-11 00:00:00',89.25,196,279,64.91,'Intermediate' from dual union all
select 15790,'015790','1985-038C','Blenda','Sunshade',58261,'Reentered','1985-06-26 00:00:00','1985-06-05 00:00:00',91.82,267,461,82.33,'Intermediate' from dual union all
select 15791,'015791','1985-038D','KDU part','KDU part',58261,'Reentered','1985-06-06 00:00:00','1985-06-05 00:00:00',90.25,256,318,82.38,'Intermediate' from dual union all
select 15792,'015792','1985-038E','KDU','KDU',58261,'Reentered','1985-07-13 00:00:00','1985-06-05 00:00:00',89.84,263,271,82.33,'Intermediate' from dual union all
select 15793,'015793','1985-038F','KDU part','KDU part',58261,'Reentered','1985-06-18 00:00:00','1985-06-05 00:00:00',89.72,252,270,82.32,'Intermediate' from dual union all
select 15794,'015794','1985-038G','KDU part','KDU part',58261,'Reentered','1985-06-06 00:00:00','1985-06-05 00:00:00',89.31,217,264,82.35,'Intermediate' from dual union all
select 15795,'015795','1985-040H','deb Blok-I','',58263,'Reentered','1985-06-23 00:00:00','1985-06-05 00:00:00',90.75,209,413,62.81,'Intermediate' from dual union all
select 15796,'015796','1983-044BB','deb Kosmos-1461','',55801,'Reentered','1989-06-14 00:00:00','1985-06-06 00:00:00',100.13,621,909,64.99,'Intermediate' from dual union all
select 15797,'015797','1983-044BC','deb Kosmos-1461','',55801,'Reentered','1989-02-04 00:00:00','1985-06-09 00:00:00',111.54,1056,1539,65.05,'Intermediate' from dual union all
select 15798,'015798','1983-044BD','deb Kosmos-1461','Medium debris, RCS 1m2',55801,'Reentered','1987-05-26 00:00:00','1985-06-27 00:00:00',99.29,564,885,65.03,'Intermediate' from dual union all
select 15799,'015799','1983-044BE','deb Kosmos-1461','',55801,'Reentered','1990-11-15 00:00:00','1985-06-06 00:00:00',100.38,647,907,64.95,'Intermediate' from dual union all
select 158,'000158','61 Omicron 43','deb r/b Ablestar','',7273,'In Earth orbit','','1961-10-02 00:00:00',114.62,947,1930,67.07,'Intermediate' from dual union all
select 1580,'001580','1965-072A','OPS 8068','DAPP 13',11822,'In Earth orbit','','1965-10-17 00:00:00',101.91,648,1051,98.66,'Sun-Synch' from dual union all
select 15800,'015800','1983-044BF','deb Kosmos-1461','',55801,'Reentered','1989-04-18 00:00:00','1985-06-14 00:00:00',99.7,594,895,65.03,'Intermediate' from dual union all
select 15801,'015801','1983-044BG','deb Kosmos-1461','',55801,'Reentered','1990-12-01 00:00:00','1985-06-05 00:00:00',99.4,573,887,65.05,'Intermediate' from dual union all
select 15802,'015802','1983-044BH','deb Kosmos-1461','',55801,'Reentered','1988-02-28 00:00:00','1985-05-31 00:00:00',99.29,564,885,65.02,'Intermediate' from dual union all
select 15803,'015803','1983-044BJ','deb Kosmos-1461','',55801,'Reentered','1988-10-21 00:00:00','1985-06-13 00:00:00',98.52,475,901,64.88,'Intermediate' from dual union all
select 15804,'015804','1985-043A','Soyuz T-13','Soyuz 7K-ST No. 19L',58266,'Landed','1985-09-26 00:00:00','1985-07-14 00:00:00',91.66,354,358,51.64,'Intermediate' from dual union all
select 15805,'015805','1985-043B','11S510','11S510',58266,'Reentered','1985-06-08 00:00:00','1985-06-07 00:00:00',88.24,182,193,51.63,'Intermediate' from dual union all
select 15806,'015806','1985-044A','Kosmos-1657','Resurs-F1 17F41 No. 54',58267,'Landed','1985-06-21 00:00:00','1985-06-09 00:00:00',89.8,257,273,82.26,'Intermediate' from dual union all
select 15807,'015807','1985-044B','11S510','11S510',58267,'Reentered','1985-06-11 00:00:00','1985-06-09 00:00:00',88.59,173,237,82.26,'Intermediate' from dual union all
select 15808,'015808','1985-045A','Kosmos-1658','Oko',58268,'Reentered','2005-11-12 00:00:00','1985-08-11 00:00:00',717.78,629,39726,63.12,'Molniya' from dual union all
select 15809,'015809','1985-045B','11S510','11S510',58268,'Reentered','1985-08-24 00:00:00','1985-07-11 00:00:00',91.11,214,444,62.8,'Intermediate' from dual union all
select 1581,'001581','1965-072B','DAPP despin weight','',11822,'Reentered','1967-02-25 00:00:00','1965-10-14 00:00:00',101.6,664,1005,98.73,'Sun-Synch' from dual union all
select 15810,'015810','1985-045C','BOZ','BOZ',58268,'Reentered','1985-07-30 00:00:00','1985-06-19 00:00:00',92.23,196,572,62.86,'Intermediate' from dual union all
select 15811,'015811','1985-045D','Blok-2BL','Blok-2BL No. 61',58268,'Reentered','2002-03-18 00:00:00','1985-08-13 00:00:00',709.3,629,39307,63.18,'Molniya' from dual union all
select 15812,'015812','1983-044BK','deb Kosmos-1461','',55801,'Reentered','1988-03-04 00:00:00','1985-06-13 00:00:00',104.19,842,1070,64.66,'Intermediate' from dual union all
select 15813,'015813','1983-044BL','deb Kosmos-1461','',55801,'Reentered','1985-10-22 00:00:00','1985-06-13 00:00:00',97.48,427,850,63.97,'Intermediate' from dual union all
select 15814,'015814','1983-044BM','deb Kosmos-1461','',55801,'Reentered','1990-06-03 00:00:00','1990-06-03 00:00:00',88.11,166,195,64.93,'Intermediate' from dual union all
select 15815,'015815','1983-044BN','deb Kosmos-1461','',55801,'Reentered','1992-04-29 00:00:00','1992-04-29 00:00:00',88.64,196,219,64.96,'Intermediate' from dual union all
select 15816,'015816','1983-044BP','deb Kosmos-1461','',55801,'Reentered','1989-06-10 00:00:00','1989-06-09 00:00:00',90.36,276,309,64.98,'Intermediate' from dual union all
select 15817,'015817','1983-044BQ','deb Kosmos-1461','',55801,'Reentered','1989-04-27 00:00:00','1985-05-28 00:00:00',99.63,595,887,65,'Intermediate' from dual union all
select 15818,'015818','1985-046A','Kosmos-1659','Oblik No. 317',58269,'Landed','1985-06-27 00:00:00','1985-06-15 00:00:00',92.26,356,415,72.9,'Intermediate' from dual union all
select 15819,'015819','1985-046B','11S510','11S510',58269,'Reentered','1985-06-29 00:00:00','1985-06-21 00:00:00',89.2,186,283,72.89,'Intermediate' from dual union all
select 1582,'001582','1965-072C','DAPP despin weight','',11822,'Reentered','1967-04-06 00:00:00','1965-10-10 00:00:00',101.91,644,1055,98.66,'Sun-Synch' from dual union all
select 15820,'015820','1985-046C','Sensor cover','Sensor cover',58269,'Reentered','1985-06-15 00:00:00','1985-06-13 00:00:00',89.55,196,309,72.89,'Intermediate' from dual union all
select 15821,'015821','1985-047A','Kosmos-1660','Musson No. 17',58270,'In Earth orbit','','1985-10-19 00:00:00',116.06,1483,1524,73.62,'Intermediate' from dual union all
select 15822,'015822','1985-047B','S5M','S5M',58270,'In Earth orbit','','1993-08-17 00:00:00',116,1481,1520,73.62,'Intermediate' from dual union all
select 15823,'015823','1985-048A','Discovery (STS 51-G)','OV-103',58271,'Landed','1985-06-24 00:00:00','1985-06-19 00:00:00',91.88,354,380,28.47,'Intermediate' from dual union all
select 15824,'015824','1985-048B','Morelos 1','Morelos 1',58271,'In Earth orbit','','1985-08-22 00:00:00',1436.04,35779,35792,.04,'Stationary' from dual union all
select 15825,'015825','1985-048C','Arabsat 1B','Arabsat 1B',58271,'In Earth orbit','','1985-09-10 00:00:00',1436.01,35746,35824,.06,'Stationary' from dual union all
select 15826,'015826','1985-048D','Telstar 303','Telstar 3D',58271,'In Earth orbit','','1985-08-19 00:00:00',1436.04,35781,35790,.06,'Stationary' from dual union all
select 15827,'015827','1985-049A','Kosmos-1661','Oko',58272,'In Earth orbit','','1985-08-17 00:00:00',717.92,666,39696,62.96,'Molniya' from dual union all
select 15828,'015828','1985-049B','11S510','11S510',58272,'Reentered','1985-09-01 00:00:00','1985-06-18 00:00:00',92.25,217,553,62.86,'Intermediate' from dual union all
select 15829,'015829','1985-049C','BOZ','BOZ',58272,'Reentered','1985-09-28 00:00:00','1985-07-18 00:00:00',92.07,217,535,62.82,'Intermediate' from dual union all
select 1583,'001583','1965-072D','FW4S S/N 20032','FW4S',11822,'In Earth orbit','','1966-01-24 00:00:00',101.92,651,1049,98.63,'Sun-Synch' from dual union all
select 15830,'015830','1985-049D','Blok-2BL','Blok-2BL No. 60',58272,'In Earth orbit','','1985-08-25 00:00:00',724.89,679,40026,63,'Molniya' from dual union all
select 15831,'015831','1985-048E','Spartan 1','Spartan 101',58271,'Landed Attached','1985-06-24 00:00:00','1985-06-20 00:00:00',91.98,354,390,28.47,'Intermediate' from dual union all
select 15832,'015832','1985-048F','PAM-D','Star 48B S/N 10039-19',58271,'In Earth orbit','','1985-08-17 00:00:00',635.75,344,35883,25.12,'GEO Transfer' from dual union all
select 15833,'015833','1985-050A','Kosmos-1662','Romb',58273,'Reentered','1989-11-16 00:00:00','1985-07-19 00:00:00',94.49,474,514,65.83,'Intermediate' from dual union all
select 15834,'015834','1985-050B','S3M','S3M',58273,'Reentered','1989-10-10 00:00:00','1985-09-05 00:00:00',94.42,464,517,65.84,'Intermediate' from dual union all
select 15835,'015835','1985-050C','Dispenser cover','Romb dispenser cover',58273,'Reentered','1986-10-26 00:00:00','1985-07-24 00:00:00',94.1,439,511,65.86,'Intermediate' from dual union all
select 15836,'015836','1985-048G','PAM-D','Star 48B S/N 10033-17',58271,'In Earth orbit','','1985-08-19 00:00:00',628.59,347,35512,26.43,'GEO Transfer' from dual union all
select 15837,'015837','1985-048H','PAM-D','Star 48B S/N 10034-19',58271,'In Earth orbit','','1985-08-17 00:00:00',661.73,350,37203,25.17,'GEO Transfer' from dual union all
select 15838,'015838','1985-051A','Progress-24','Progress 7K-TG No. 125',58274,'Deorbited','1985-07-15 00:00:00','1985-06-28 00:00:00',91.67,354,359,51.65,'Intermediate' from dual union all
select 15839,'015839','1985-051B','11S510','11S510',58274,'Reentered','1985-06-24 00:00:00','1985-06-22 00:00:00',88.4,176,215,51.63,'Intermediate' from dual union all
select 1584,'001584','1965-073A','Kosmos-86','Strela-1',11823,'In Earth orbit','','1965-10-19 00:00:00',115.04,1277,1637,56.06,'Intermediate' from dual union all
select 15840,'015840','1985-052A','Kosmos-1663','Resurs-F1 17F41 No. 55',58275,'Landed','1985-07-05 00:00:00','1985-06-23 00:00:00',89.81,258,273,82.33,'Intermediate' from dual union all
select 15841,'015841','1985-052B','11S510','11S510',58275,'Reentered','1985-06-30 00:00:00','1985-06-21 00:00:00',89.21,214,256,82.34,'Intermediate' from dual union all
select 15842,'015842','1985-053A','Sep Motor Cover','Sep Motor Cover',58276,'Reentered','1985-06-28 00:00:00','1985-06-21 00:00:00',89.84,194,340,64.41,'Intermediate' from dual union all
select 15843,'015843','1985-044C','KDU','KDU',58267,'Reentered','1985-06-29 00:00:00','1985-06-21 00:00:00',89.84,261,273,82.25,'Intermediate' from dual union all
select 15844,'015844','1985-044D','Blenda','Sunshade',58267,'Reentered','1985-07-03 00:00:00','1985-06-21 00:00:00',89.69,248,270,82.25,'Intermediate' from dual union all
select 15845,'015845','1985-052C','Sensor cover','Sensor cover',58275,'Reentered','1985-06-24 00:00:00','1985-06-22 00:00:00',89.06,201,255,82.34,'Intermediate' from dual union all
select 15846,'015846','1985-052D','Sensor cover','Sensor cover',58275,'Reentered','1985-06-24 00:00:00','1985-06-22 00:00:00',89.16,207,258,82.31,'Intermediate' from dual union all
select 15847,'015847','1985-053B','Sep Motor Cover','Sep Motor Cover',58276,'Reentered','1985-06-24 00:00:00','1985-06-21 00:00:00',89.01,178,274,64.46,'Intermediate' from dual union all
select 15848,'015848','1985-044E','KDU part','KDU part',58267,'Reentered','1985-07-12 00:00:00','1985-06-21 00:00:00',91.72,257,461,82.26,'Intermediate' from dual;
insert into SATELLITE
select 15849,'015849','1985-044F','KDU part','KDU part',58267,'Reentered','1985-06-24 00:00:00','1985-06-21 00:00:00',90.79,256,371,82.2,'Intermediate' from dual union all
select 1585,'001585','1965-073B','Kosmos-87','Strela-1',11823,'In Earth orbit','','1965-10-23 00:00:00',115.45,1304,1648,56.06,'Intermediate' from dual union all
select 15850,'015850','1985-053C','Sep Motor Cover','Sep Motor Cover',58276,'Reentered','1985-06-27 00:00:00','1985-06-21 00:00:00',89.63,188,325,64.58,'Intermediate' from dual union all
select 15851,'015851','1985-039C','deb Kosmos-1654','',58262,'Reentered','1985-08-14 00:00:00','1985-06-21 00:00:00',98.25,194,1156,64.4,'Intermediate' from dual union all
select 15852,'015852','1985-044G','KDU part','KDU part',58267,'Reentered','1985-08-18 00:00:00','1985-07-07 00:00:00',89.65,251,264,82.26,'Intermediate' from dual union all
select 15853,'015853','1985-039D','deb Kosmos-1654','',58262,'Reentered','1985-06-27 00:00:00','1985-06-21 00:00:00',90.07,173,383,65.3,'Intermediate' from dual union all
select 15854,'015854','1985-039E','deb Kosmos-1654','',58262,'Reentered','1985-08-18 00:00:00','1985-06-30 00:00:00',90.58,187,419,64.86,'Intermediate' from dual union all
select 15855,'015855','1985-039F','deb Kosmos-1654','',58262,'Reentered','1985-07-07 00:00:00','1985-06-24 00:00:00',90.66,187,427,64.85,'Intermediate' from dual union all
select 15856,'015856','1984-128E','SA Vega-2','5VK SA',57182,'Deep Space Attached','1984-12-21 00:00:00','1985-06-14 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 15857,'015857','1984-128F','AZ Vega-2','Aerostatniy Zond',57182,'Deep Space Attached','1984-12-21 00:00:00','1985-06-14 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 15858,'015858','1984-125E','SA Vega-1','5VK SA',57179,'Deep Space Attached','1984-12-15 00:00:00','1985-06-11 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 15859,'015859','1984-125F','AZ Vega-1','Aerostatniy Zond',57179,'Deep Space Attached','1984-12-15 00:00:00','1985-06-11 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 1586,'001586','1965-073C','Kosmos-88','Strela-1',11823,'In Earth orbit','','1965-10-25 00:00:00',115.82,1325,1661,56.07,'Intermediate' from dual union all
select 15860,'015860','1985-054A','Kosmos-1664','Oblik No. 318',58277,'Landed','1985-07-05 00:00:00','1985-06-28 00:00:00',90.54,224,378,72.84,'Intermediate' from dual union all
select 15861,'015861','1985-054B','11S510','11S510',58277,'Reentered','1985-07-10 00:00:00','1985-06-28 00:00:00',89.97,192,354,72.84,'Intermediate' from dual union all
select 15862,'015862','1983-101Y','Kosmos-1501 SS 21','ESO',55858,'Reentered','1986-03-20 00:00:00','1985-06-27 00:00:00',94.09,456,492,82.95,'Intermediate' from dual union all
select 15863,'015863','1983-101Z','Kosmos-1501 SS 22','ESO',55858,'Reentered','1986-03-05 00:00:00','1985-06-27 00:00:00',94,450,491,82.92,'Intermediate' from dual union all
select 15864,'015864','1983-101AA','Kosmos-1501 SS 23','ESO',55858,'Reentered','1986-02-26 00:00:00','1985-06-28 00:00:00',94,449,492,82.92,'Intermediate' from dual union all
select 15865,'015865','1983-101AB','Kosmos-1501 SS 24','ESO',55858,'Reentered','1986-03-25 00:00:00','1985-06-27 00:00:00',94.1,458,492,82.95,'Intermediate' from dual union all
select 15866,'015866','1985-054C','Sensor cover','Sensor cover',58277,'Reentered','1985-06-30 00:00:00','1985-06-29 00:00:00',89.07,169,287,72.83,'Intermediate' from dual union all
select 15867,'015867','1985-046D','Blenda','Sunshade',58269,'Reentered','1985-10-23 00:00:00','1985-08-12 00:00:00',91.83,344,385,72.9,'Intermediate' from dual union all
select 15868,'015868','1985-046E','KDU','KDU',58269,'Reentered','1987-11-11 00:00:00','1985-08-15 00:00:00',92.2,356,409,72.9,'Intermediate' from dual union all
select 15869,'015869','1985-046F','KDU part','KDU part',58269,'Reentered','1985-12-25 00:00:00','1985-08-14 00:00:00',91.84,337,393,72.89,'Intermediate' from dual union all
select 1587,'001587','1965-073D','Kosmos-89','Strela-1',11823,'In Earth orbit','','1965-10-19 00:00:00',116.23,1349,1674,56.05,'Intermediate' from dual union all
select 15870,'015870','1985-046G','KDU part','KDU part',58269,'Reentered','1985-11-27 00:00:00','1985-08-12 00:00:00',94.31,379,592,72.89,'Intermediate' from dual union all
select 15871,'015871','1985-046H','KDU part','KDU part',58269,'Reentered','1986-03-30 00:00:00','1985-08-12 00:00:00',94.13,386,567,72.9,'Intermediate' from dual union all
select 15872,'015872','1985-046J','KDU part','KDU part',58269,'Reentered','1985-07-14 00:00:00','1985-06-28 00:00:00',93.13,372,484,72.95,'Intermediate' from dual union all
select 15873,'015873','1985-055A','INTELSAT 511','INTELSAT VA F11',58278,'In Earth orbit','','1985-09-06 00:00:00',1436.06,35782,35790,.18,'Stationary' from dual union all
select 15874,'015874','1985-055B','Centaur AC-64','Centaur D-1AR AC-64',58278,'In Earth orbit','','1985-09-02 00:00:00',602.61,305,34205,23.36,'GEO Transfer' from dual union all
select 15875,'015875','1985-056A','Giotto','Giotto',58279,'Deep Space','1985-07-04 00:00:00','1985-07-02 00:00:00',631.51,202,35807,6.99,'GEO Transfer' from dual union all
select 15876,'015876','1985-056B','Ariane H8 V14','Ariane H8 S/N T10',58279,'In Earth orbit','','1986-04-14 00:00:00',623.62,381,35221,7.62,'GEO Transfer' from dual union all
select 15877,'015877','1985-057A','Kosmos-1665','Oblik No. 319',58280,'Landed','1985-07-17 00:00:00','1985-07-05 00:00:00',89.62,222,290,72.87,'Intermediate' from dual union all
select 15878,'015878','1985-057B','11S510','11S510',58280,'Reentered','1985-07-10 00:00:00','1985-07-06 00:00:00',88.97,190,258,72.86,'Intermediate' from dual union all
select 15879,'015879','1985-057C','Sensor cover','Sensor cover',58280,'Reentered','1985-08-25 00:00:00','1985-07-12 00:00:00',89.22,174,298,72.92,'Intermediate' from dual union all
select 1588,'001588','1965-073E','Kosmos-90','Strela-1',11823,'In Earth orbit','','1966-02-19 00:00:00',116.64,1377,1682,56.08,'Intermediate' from dual union all
select 15880,'015880','1985-052E','KDU','KDU',58275,'Reentered','1985-08-18 00:00:00','1985-07-05 00:00:00',89.84,259,275,82.33,'Intermediate' from dual union all
select 15881,'015881','1985-052F','Blenda','Sunshade',58275,'Reentered','1985-07-19 00:00:00','1985-07-05 00:00:00',89.57,242,264,82.38,'Intermediate' from dual union all
select 15882,'015882','1985-052G','KDU part','KDU part',58275,'Reentered','1985-07-06 00:00:00','1985-07-06 00:00:00',89.4,218,272,82.31,'Intermediate' from dual union all
select 15883,'015883','1985-052H','KDU part','KDU part',58275,'Reentered','1985-07-27 00:00:00','1985-07-05 00:00:00',91.88,260,474,82.33,'Intermediate' from dual union all
select 15884,'015884','1985-052J','KDU part','KDU part',58275,'Reentered','1985-07-16 00:00:00','1985-07-05 00:00:00',92.35,260,520,82.32,'Intermediate' from dual union all
select 15885,'015885','1985-054D','Blenda','Sunshade',58277,'Reentered','1985-07-16 00:00:00','1985-07-05 00:00:00',91.3,264,413,72.84,'Intermediate' from dual union all
select 15886,'015886','1985-054E','KDU','KDU',58277,'Reentered','1985-07-20 00:00:00','1985-07-06 00:00:00',89.2,208,262,72.84,'Intermediate' from dual union all
select 15887,'015887','1985-054F','KDU part','KDU part',58277,'Reentered','1985-07-07 00:00:00','1985-07-06 00:00:00',88.95,198,248,72.84,'Intermediate' from dual union all
select 15888,'015888','1985-054G','KDU part','KDU part',58277,'Reentered','1985-07-08 00:00:00','1985-07-06 00:00:00',88.85,197,239,72.83,'Intermediate' from dual union all
select 15889,'015889','1985-058A','Kosmos-1666','Tselina-D',58281,'In Earth orbit','','1991-03-28 00:00:00',97.22,611,641,82.5,'Intermediate' from dual union all
select 1589,'001589','1965-073F','S3','S3',11823,'In Earth orbit','','1966-06-10 00:00:00',116.79,1375,1698,56.07,'Intermediate' from dual union all
select 15890,'015890','1985-058B','S5M','S5M',58281,'In Earth orbit','','1991-03-27 00:00:00',97.52,623,657,82.53,'Intermediate' from dual union all
select 15891,'015891','1985-059A','Kosmos-1667','Bion No. 7',58282,'Landed','1985-07-17 00:00:00','1985-07-12 00:00:00',89.28,210,268,82.35,'Intermediate' from dual union all
select 15892,'015892','1985-059B','11S510','11S510',58282,'Reentered','1985-07-19 00:00:00','1985-07-12 00:00:00',88.99,200,250,82.35,'Intermediate' from dual union all
select 15893,'015893','1985-039G','deb Kosmos-1654','',58262,'Reentered','1985-06-23 00:00:00','1985-06-22 00:00:00',88.77,172,256,64.9,'Intermediate' from dual union all
select 15894,'015894','1985-039H','deb Kosmos-1654','',58262,'Reentered','1985-06-23 00:00:00','1985-06-22 00:00:00',88.77,167,261,64.8,'Intermediate' from dual union all
select 15895,'015895','1985-039J','deb Kosmos-1654','',58262,'Reentered','1985-06-23 00:00:00','1985-06-22 00:00:00',87.74,119,207,64.7,'Intermediate' from dual union all
select 15896,'015896','1985-039K','deb Kosmos-1654','',58262,'Reentered','1985-06-24 00:00:00','1985-06-22 00:00:00',88.64,183,231,64.8,'Intermediate' from dual union all
select 15897,'015897','1985-039L','deb Kosmos-1654','',58262,'Reentered','1985-06-24 00:00:00','1985-06-22 00:00:00',91.57,186,517,65.2,'Intermediate' from dual union all
select 15898,'015898','1985-039M','deb Kosmos-1654','',58262,'Reentered','1985-06-25 00:00:00','1985-06-22 00:00:00',91.62,145,563,65.2,'Intermediate' from dual union all
select 15899,'015899','1985-039N','deb Kosmos-1654','',58262,'Reentered','1985-06-25 00:00:00','1985-06-22 00:00:00',90.04,187,366,63.6,'Intermediate' from dual union all
select 159,'000159','61 Omicron 44','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-30 00:00:00',111.26,983,1586,65.48,'Intermediate' from dual union all
select 1590,'001590','1965-073G','S3 part','',11823,'In Earth orbit','','1966-01-18 00:00:00',116.5,1373,1674,56.08,'Intermediate' from dual union all
select 15900,'015900','1985-039P','deb Kosmos-1654','',58262,'Reentered','1985-06-25 00:00:00','1985-06-22 00:00:00',90.76,183,441,65.5,'Intermediate' from dual union all
select 15901,'015901','1985-039Q','deb Kosmos-1654','',58262,'Reentered','1985-06-27 00:00:00','1985-06-22 00:00:00',89.15,177,288,64.7,'Intermediate' from dual union all
select 15902,'015902','1985-039R','deb Kosmos-1654','',58262,'Reentered','1985-06-28 00:00:00','1985-06-22 00:00:00',93.15,291,567,65.1,'Intermediate' from dual union all
select 15903,'015903','1985-039S','deb Kosmos-1654','',58262,'Reentered','1985-06-29 00:00:00','1985-06-22 00:00:00',87.85,155,182,64.8,'Intermediate' from dual union all
select 15904,'015904','1985-039T','deb Kosmos-1654','',58262,'Reentered','1985-06-30 00:00:00','1985-06-22 00:00:00',90.42,184,407,65.3,'Intermediate' from dual union all
select 15905,'015905','1985-039U','deb Kosmos-1654','',58262,'Reentered','1985-07-06 00:00:00','1985-06-22 00:00:00',90.99,224,423,65,'Intermediate' from dual union all
select 15906,'015906','1985-060A','Kosmos-1668','Oblik',58283,'Landed','1985-07-29 00:00:00','1985-07-17 00:00:00',89.59,229,280,70.38,'Intermediate' from dual union all
select 15907,'015907','1985-060B','11S510','11S510',58283,'Reentered','1985-07-24 00:00:00','1985-07-17 00:00:00',89.06,203,253,70.39,'Intermediate' from dual union all
select 15908,'015908','1985-060C','Sensor cover','Sensor cover',58283,'Reentered','1985-07-16 00:00:00','1985-07-16 00:00:00',88.21,182,189,70.41,'Intermediate' from dual union all
select 15909,'015909','1985-061A','Molniya-3','Molniya-3 No. 37',58284,'Reentered','2002-09-21 00:00:00','1985-09-15 00:00:00',717.7,508,39843,62.84,'Molniya' from dual union all
select 1591,'001591','1965-073H','S3 part','',11823,'In Earth orbit','','1965-10-20 00:00:00',116.67,1372,1690,56.04,'Intermediate' from dual union all
select 15910,'015910','1985-061B','11S510','11S510',58284,'Reentered','1985-08-22 00:00:00','1985-07-24 00:00:00',90.59,210,397,62.82,'Intermediate' from dual union all
select 15911,'015911','1985-061C','BOZ','BOZ',58284,'Reentered','1985-08-23 00:00:00','1985-07-24 00:00:00',90.78,203,423,62.81,'Intermediate' from dual union all
select 15912,'015912','1985-057D','Blenda','Sunshade',58280,'Reentered','1985-08-04 00:00:00','1985-07-17 00:00:00',91.71,253,464,72.87,'Intermediate' from dual union all
select 15913,'015913','1985-057E','KDU part','KDU part',58280,'Reentered','1985-07-25 00:00:00','1985-07-17 00:00:00',89.58,227,280,72.88,'Intermediate' from dual union all
select 15914,'015914','1985-057F','KDU part','KDU part',58280,'Reentered','1985-07-22 00:00:00','1985-07-17 00:00:00',89.74,225,299,72.86,'Intermediate' from dual union all
select 15915,'015915','1985-057G','KDU','KDU',58280,'Reentered','1985-09-02 00:00:00','1985-08-02 00:00:00',89.4,217,273,72.86,'Intermediate' from dual union all
select 15916,'015916','1985-061D','Blok-ML','Blok-ML No. 76714-522',58284,'Reentered','2004-03-08 00:00:00','1985-09-16 00:00:00',737.8,522,40816,62.82,'Molniya' from dual union all
select 15917,'015917','1985-057H','KDU part','KDU part',58280,'Reentered','1985-07-25 00:00:00','1985-07-19 00:00:00',91.3,248,429,72.85,'Intermediate' from dual union all
select 15918,'015918','1985-062A','Kosmos-1669','Progress 7K-TG No. 126',58285,'Deorbited','1985-08-30 00:00:00','1985-08-20 00:00:00',91.62,353,355,51.64,'Intermediate' from dual union all
select 15919,'015919','1985-062B','11S510','11S510',58285,'Reentered','1985-07-23 00:00:00','1985-07-21 00:00:00',88.3,174,207,51.63,'Intermediate' from dual union all
select 1592,'001592','1965-020DS','deb Blok Kosmos','',11770,'Reentered','1970-10-27 00:00:00','1965-09-20 00:00:00',107.56,417,1811,55.91,'Intermediate' from dual union all
select 15920,'015920','1985-059C','KB electropitaniya','KBE',58282,'Reentered','1985-07-27 00:00:00','1985-07-21 00:00:00',90.74,230,392,82.37,'Intermediate' from dual union all
select 15921,'015921','1985-060D','KDU','KDU',58283,'Reentered','1985-09-03 00:00:00','1985-07-29 00:00:00',89.56,226,280,70.39,'Intermediate' from dual union all
select 15922,'015922','1985-060E','Blenda','Sunshade',58283,'Reentered','1985-08-04 00:00:00','1985-07-29 00:00:00',89.4,227,263,70.38,'Intermediate' from dual union all
select 15923,'015923','1985-060F','KDU part','KDU part',58283,'Reentered','1985-08-02 00:00:00','1985-07-29 00:00:00',90.05,232,321,70.31,'Intermediate' from dual union all
select 15924,'015924','1985-060G','KDU part','KDU part',58283,'Reentered','1985-08-01 00:00:00','1985-07-29 00:00:00',89.49,225,274,70.39,'Intermediate' from dual union all
select 15925,'015925','1985-063A','Challenger (STS 51-F)','OV-099',58286,'Landed','1985-08-06 00:00:00','1985-07-31 00:00:00',90.83,311,320,49.48,'Intermediate' from dual union all
select 15926,'015926','1985-060H','KDU part','KDU part',58283,'Reentered','1985-08-09 00:00:00','1985-07-29 00:00:00',91.4,246,441,70.4,'Intermediate' from dual union all
select 15927,'015927','1985-060J','KDU part','KDU part',58283,'Reentered','1985-07-31 00:00:00','1985-07-29 00:00:00',92.31,231,545,70.25,'Intermediate' from dual union all
select 15928,'015928','1985-060K','KDU part','KDU part',58283,'Reentered','1985-08-02 00:00:00','1985-07-30 00:00:00',90.85,240,393,70.4,'Intermediate' from dual union all
select 15929,'015929','1985-063B','PDP','PDP',58286,'Landed Attached','1985-08-06 00:00:00','1985-08-01 00:00:00',90.82,310,320,49.48,'Intermediate' from dual union all
select 1593,'001593','1965-020DT','deb Blok Kosmos','',11770,'Reentered','1970-09-28 00:00:00','1965-09-19 00:00:00',107.55,418,1808,56.06,'Intermediate' from dual union all
select 15930,'015930','1985-064A','Kosmos-1670','US-A reactor',58287,'In Earth orbit','','1985-10-18 00:00:00',89.62,249,263,65.02,'Intermediate' from dual union all
select 15931,'015931','1985-065A','Kosmos-1671','Oblik No. 320',58288,'Landed','1985-08-16 00:00:00','1985-08-04 00:00:00',90.45,230,364,72.88,'Intermediate' from dual union all
select 15932,'015932','1985-065B','11S510','11S510',58288,'Reentered','1985-08-10 00:00:00','1985-08-04 00:00:00',88.98,192,257,72.87,'Intermediate' from dual union all
select 15933,'015933','1982-033ES','EVA debris','',54342,'Reentered','1985-09-19 00:00:00','1985-08-03 00:00:00',91.62,352,355,51.65,'Intermediate' from dual union all
select 15934,'015934','1982-033ET','EVA debris','',54342,'Reentered','1985-08-24 00:00:00','1985-08-03 00:00:00',91.54,349,352,51.65,'Intermediate' from dual union all
select 15935,'015935','1985-066A','NNS O-24','NNS O-24',58289,'In Earth orbit','','1985-10-27 00:00:00',107.97,1005,1261,89.83,'Polar' from dual union all
select 15936,'015936','1985-066B','NNS O-30','NNS O-30',58289,'In Earth orbit','','1991-03-24 00:00:00',107.93,1003,1258,89.9,'Polar' from dual union all
select 15937,'015937','1982-033EU','EVA debris','',54342,'Reentered','1985-08-12 00:00:00','1985-08-03 00:00:00',91.48,345,350,51.64,'Intermediate' from dual union all
select 15938,'015938','1985-066C','Star 20 E-37','Star 20 E-37',58289,'In Earth orbit','','1991-05-06 00:00:00',107.94,1002,1261,89.91,'Polar' from dual;
insert into SATELLITE
select 15939,'015939','1982-033EV','EVA debris','',54342,'Reentered','1985-09-25 00:00:00','1985-08-05 00:00:00',91.58,351,353,51.64,'Intermediate' from dual union all
select 1594,'001594','1965-020DU','deb Blok Kosmos','',11770,'Reentered','1966-12-19 00:00:00','1965-09-19 00:00:00',106.02,319,1766,56.09,'Intermediate' from dual union all
select 15940,'015940','1985-067A','Kosmos-1672','Resurs-F1 17F41 No. 57',58290,'Landed','1985-08-21 00:00:00','1985-08-09 00:00:00',89.8,258,272,82.34,'Intermediate' from dual union all
select 15941,'015941','1985-067B','11S510','11S510',58290,'Reentered','1985-08-11 00:00:00','1985-08-09 00:00:00',88.55,173,232,82.34,'Intermediate' from dual union all
select 15942,'015942','1985-068A','Kosmos-1673','Siluet No. 5',58291,'Landed','1985-09-19 00:00:00','1985-08-15 00:00:00',89.21,204,267,64.79,'Intermediate' from dual union all
select 15943,'015943','1985-068B','11S510','11S510',58291,'Reentered','1985-08-14 00:00:00','1985-08-10 00:00:00',88.9,192,249,64.8,'Intermediate' from dual union all
select 15944,'015944','1985-069A','Kosmos-1674','Tselina-D',58292,'In Earth orbit','','1985-10-18 00:00:00',97.67,630,665,82.53,'Intermediate' from dual union all
select 15945,'015945','1985-069B','S5M','S5M',58292,'In Earth orbit','','1991-03-27 00:00:00',97.47,621,655,82.53,'Intermediate' from dual union all
select 15946,'015946','1985-070A','Raduga','Gran'' No. 26L',58293,'In Earth orbit','','1985-10-08 00:00:00',1436.06,35745,35827,1.17,'Stationary' from dual union all
select 15947,'015947','1985-070B','8S812','8S812',58293,'Reentered','1985-08-11 00:00:00','1985-08-10 00:00:00',88.07,174,184,51.62,'Intermediate' from dual union all
select 15948,'015948','1985-070C','Perekhodnik','Sredniy perekhodnik',58293,'Reentered','1985-08-09 00:00:00','1985-08-09 00:00:00',88.44,188,207,51.62,'Intermediate' from dual union all
select 15949,'015949','1985-068C','Kosmos-1673 part','Kometa part',58291,'Reentered','1985-08-10 00:00:00','1985-08-09 00:00:00',89.13,188,275,64.8,'Intermediate' from dual union all
select 1595,'001595','1965-020DV','deb Blok Kosmos','',11770,'Reentered','1972-04-06 00:00:00','1965-09-20 00:00:00',109,592,1768,55.73,'Intermediate' from dual union all
select 15950,'015950','1985-066D','SOOS 1 adapter','',58289,'In Earth orbit','','1991-04-26 00:00:00',107.03,967,1212,89.9,'Polar' from dual union all
select 15951,'015951','1985-066E','SOOS 1 adapter','',58289,'In Earth orbit','','1991-05-24 00:00:00',107.14,974,1215,89.86,'Polar' from dual union all
select 15952,'015952','1985-071A','Kosmos-1675','Oko',58294,'In Earth orbit','','1985-10-11 00:00:00',717.88,675,39684,62.89,'Molniya' from dual union all
select 15953,'015953','1985-071B','11S510','11S510',58294,'Reentered','1985-10-23 00:00:00','1985-09-12 00:00:00',91.41,213,475,62.86,'Intermediate' from dual union all
select 15954,'015954','1985-071C','BOZ','BOZ',58294,'Reentered','1985-09-27 00:00:00','1985-08-20 00:00:00',92.19,195,569,62.87,'Intermediate' from dual union all
select 15955,'015955','1985-071D','Blok-2BL','Blok-2BL No. 77324-658',58294,'Reentered','2001-08-23 00:00:00','1985-10-14 00:00:00',708.23,669,39214,62.85,'Molniya' from dual union all
select 15956,'015956','1985-070D','SOZ','SOZ',58293,'Reentered','1986-12-31 00:00:00','1985-10-08 00:00:00',647.31,240,36579,47.03,'GEO Transfer' from dual union all
select 15957,'015957','1985-070E','SOZ','SOZ',58293,'Reentered','1986-10-27 00:00:00','1985-10-10 00:00:00',648.64,294,36593,47.12,'GEO Transfer' from dual union all
select 15958,'015958','1985-071E','BOZ part','',58294,'Reentered','1985-10-01 00:00:00','1985-08-23 00:00:00',92.27,212,560,62.85,'Intermediate' from dual union all
select 15959,'015959','1985-072A','Kosmos-1676','Kobal''t No. 828',58295,'Landed','1985-10-14 00:00:00','1985-08-23 00:00:00',89.6,164,346,67.16,'Intermediate' from dual union all
select 1596,'001596','1965-020DW','deb Blok Kosmos','',11770,'Reentered','1967-10-29 00:00:00','1965-09-23 00:00:00',106.72,366,1784,56.24,'Intermediate' from dual union all
select 15960,'015960','1985-072B','11S510','11S510',58295,'Reentered','1985-08-22 00:00:00','1985-08-18 00:00:00',89.11,164,297,67.16,'Intermediate' from dual union all
select 15961,'015961','1985-065C','Blenda','Sunshade',58288,'Reentered','1985-09-10 00:00:00','1985-08-16 00:00:00',92.2,252,513,72.89,'Intermediate' from dual union all
select 15962,'015962','1985-065D','KDU','KDU',58288,'Reentered','1985-09-15 00:00:00','1985-08-16 00:00:00',89.81,227,304,72.88,'Intermediate' from dual union all
select 15963,'015963','1985-070F','Blok DM','11S86 No. 55L',58293,'In Earth orbit','','1985-10-08 00:00:00',1472.33,36446,36540,1.19,'Drift GEO' from dual union all
select 15964,'015964','1985-065E','KDU part','KDU part',58288,'Reentered','1985-09-03 00:00:00','1985-08-17 00:00:00',92.61,253,551,72.94,'Intermediate' from dual union all
select 15965,'015965','1985-065F','KDU part','KDU part',58288,'Reentered','1985-08-21 00:00:00','1985-08-17 00:00:00',89.59,220,288,72.87,'Intermediate' from dual union all
select 15966,'015966','1985-065G','KDU part','KDU part',58288,'Reentered','1985-08-25 00:00:00','1985-08-17 00:00:00',89.58,225,283,72.88,'Intermediate' from dual union all
select 15967,'015967','1985-073A','Suisei','PLANET-A',58296,'Deep Space','1985-08-19 00:00:00','1985-08-18 00:00:00',1569.46,217,102446,31.18,'Earth Escape' from dual union all
select 15968,'015968','1985-073B','M-3B','M-3B',58296,'Reentered','1985-11-13 00:00:00','1985-08-19 00:00:00',100.71,212,1373,31.18,'Intermediate' from dual union all
select 15969,'015969','1985-073C','KM-P','KM-P',58296,'Deep Space','1985-08-19 00:00:00','1985-08-18 00:00:00',1569.46,217,102446,31.18,'Earth Escape' from dual union all
select 1597,'001597','1965-020DX','deb Blok Kosmos','',11770,'Reentered','1988-12-28 00:00:00','1965-09-20 00:00:00',110.71,580,1939,56.08,'Intermediate' from dual union all
select 15970,'015970','1982-033EW','EVA debris','',54342,'Reentered','1986-01-15 00:00:00','1985-08-18 00:00:00',91.54,349,352,51.64,'Intermediate' from dual union all
select 15971,'015971','1985-067C','Blenda','Sunshade',58290,'Reentered','1985-08-28 00:00:00','1985-08-21 00:00:00',89.87,262,274,82.34,'Intermediate' from dual union all
select 15972,'015972','1985-067D','KDU','KDU',58290,'Reentered','1985-10-15 00:00:00','1985-09-07 00:00:00',89.65,251,264,82.33,'Intermediate' from dual union all
select 15973,'015973','1985-067E','KDU part','KDU part',58290,'Reentered','1985-09-15 00:00:00','1985-08-21 00:00:00',91.93,263,475,82.32,'Intermediate' from dual union all
select 15974,'015974','1985-067F','KDU part','KDU part',58290,'Reentered','1985-09-03 00:00:00','1985-08-22 00:00:00',89.66,247,269,82.34,'Intermediate' from dual union all
select 15975,'015975','1985-067G','KDU part','KDU part',58290,'Reentered','1985-08-22 00:00:00','1985-08-22 00:00:00',89.14,226,238,82.33,'Intermediate' from dual union all
select 15976,'015976','1985-067H','KDU part','KDU part',58290,'Reentered','1985-08-24 00:00:00','1985-08-22 00:00:00',91.55,257,445,82.3,'Intermediate' from dual union all
select 15977,'015977','1985-074A','Molniya-1','Molniya-1 No. 61',58297,'Reentered','2000-03-22 00:00:00','1985-10-31 00:00:00',717.7,744,39607,62.85,'Molniya' from dual union all
select 15978,'015978','1985-074B','11S510','11S510',58297,'Reentered','1985-11-04 00:00:00','1985-09-22 00:00:00',91.75,209,512,62.85,'Intermediate' from dual union all
select 15979,'015979','1985-074C','BOZ','BOZ',58297,'Reentered','1985-10-22 00:00:00','1985-09-24 00:00:00',91.44,195,496,62.84,'Intermediate' from dual union all
select 1598,'001598','1965-020DY','deb Blok Kosmos','',11770,'Reentered','1976-04-12 00:00:00','1965-09-20 00:00:00',108.25,464,1828,55.88,'Intermediate' from dual union all
select 15980,'015980','1984-104J','Kosmos-1601 SS 3','ESO',57158,'Reentered','1988-05-25 00:00:00','1985-08-22 00:00:00',94.54,478,515,65.83,'Intermediate' from dual union all
select 15981,'015981','1984-104K','Kosmos-1601 SS 4','ESO',57158,'Reentered','1988-03-20 00:00:00','1985-08-22 00:00:00',94.36,469,507,65.83,'Intermediate' from dual union all
select 15982,'015982','1984-104L','Kosmos-1601 SS 5','ESO',57158,'Reentered','1988-03-09 00:00:00','1985-08-23 00:00:00',94.31,467,504,65.84,'Intermediate' from dual union all
select 15983,'015983','1985-074D','Blok-ML','Blok-ML No. 71626-468',58297,'Reentered','2000-04-20 00:00:00','1985-10-21 00:00:00',732.45,736,40340,62.84,'Molniya' from dual union all
select 15984,'015984','1984-104M','Kosmos-1601 SS 6','ESO',57158,'Reentered','1988-05-04 00:00:00','1985-08-23 00:00:00',94.49,473,515,65.84,'Intermediate' from dual union all
select 15985,'015985','1984-104N','Kosmos-1601 SS 7','ESO',57158,'Reentered','1988-04-03 00:00:00','1985-08-24 00:00:00',94.35,471,504,65.84,'Intermediate' from dual union all
select 15986,'015986','1985-075A','Kosmos-1677','US-A reactor',58298,'In Earth orbit','','1985-10-18 00:00:00',89.62,249,263,65,'Intermediate' from dual union all
select 15987,'015987','1984-104P','Kosmos-1601 SS 8','ESO',57158,'Reentered','1988-05-07 00:00:00','1985-08-24 00:00:00',94.51,475,515,65.84,'Intermediate' from dual union all
select 15988,'015988','1984-104Q','Kosmos-1601 SS 9','ESO',57158,'Reentered','1988-05-20 00:00:00','1985-08-25 00:00:00',94.54,475,518,65.83,'Intermediate' from dual union all
select 15989,'015989','1984-104R','Kosmos-1601 SS 10','ESO',57158,'Reentered','1988-03-17 00:00:00','1985-08-25 00:00:00',94.31,465,505,65.85,'Intermediate' from dual union all
select 1599,'001599','1965-020DZ','deb Blok Kosmos','',11770,'Reentered','1971-01-16 00:00:00','1965-09-20 00:00:00',108.38,463,1841,55.95,'Intermediate' from dual union all
select 15990,'015990','1984-104S','Kosmos-1601 SS 11','ESO',57158,'Reentered','1988-04-21 00:00:00','1985-08-27 00:00:00',94.42,472,509,65.86,'Intermediate' from dual union all
select 15991,'015991','1984-104T','Kosmos-1601 SS 12','ESO',57158,'Reentered','1988-04-17 00:00:00','1985-08-27 00:00:00',94.43,471,511,65.82,'Intermediate' from dual union all
select 15992,'015992','1985-076A','Discovery (STS 51-I)','OV-103',58299,'Landed','1985-09-03 00:00:00','1985-08-29 00:00:00',92,351,395,28.55,'Intermediate' from dual union all
select 15993,'015993','1985-076B','Aussat K1','Aussat K1',58299,'In Earth orbit','','1985-11-18 00:00:00',1436.09,35778,35795,.03,'Stationary' from dual union all
select 15994,'015994','1985-076C','ASC 1','ASC 1',58299,'In Earth orbit','','1985-11-10 00:00:00',1436.01,35775,35795,.01,'Stationary' from dual union all
select 15995,'015995','1985-076D','Leasat 4','Leasat 4',58299,'In Earth orbit','','1985-11-06 00:00:00',1436.16,35786,35790,3.2,'Inclined GEO' from dual union all
select 15996,'015996','1985-076E','PAM-D','Star 48B S/N 10041-17',58299,'In Earth orbit','','1985-10-28 00:00:00',642.24,388,36172,26.22,'GEO Transfer' from dual union all
select 15997,'015997','1985-077A','Kosmos-1678','Resurs-F1 17F41 No. 53',58300,'Landed','1985-09-12 00:00:00','1985-08-31 00:00:00',89.8,258,272,82.33,'Intermediate' from dual union all
select 15998,'015998','1985-077B','11S510','11S510',58300,'Reentered','1985-09-02 00:00:00','1985-08-31 00:00:00',88.68,174,244,82.32,'Intermediate' from dual union all
select 15999,'015999','1985-078A','Kosmos-1679','Kobal''t',58301,'Landed','1985-10-18 00:00:00','1985-09-05 00:00:00',89.3,169,310,64.86,'Intermediate' from dual union all
select 16,'000016','58 Beta 1','GRC 33-KS-2800','GRC 33-KS-2800  144-79-2',4793,'In Earth orbit','','1959-09-30 00:00:00',138.48,652,4324,34.28,'Medium' from dual union all
select 160,'000160','61 Pi','Discoverer 26','CORONA C''-6',7275,'Reentered Attached','1961-12-05 00:00:00','1961-08-10 00:00:00',94.01,229,713,82.94,'Intermediate' from dual union all
select 1600,'001600','1965-020EA','deb Blok Kosmos','',11770,'Reentered','1978-03-10 00:00:00','1965-09-20 00:00:00',108.39,492,1812,56.1,'Intermediate' from dual union all
select 16000,'016000','1985-078B','11S510','11S510',58301,'Reentered','1985-09-04 00:00:00','1985-08-31 00:00:00',89.2,169,301,64.87,'Intermediate' from dual union all
select 16001,'016001','1985-076F','Leasat PKM','TU-882',58299,'In Earth orbit','','1985-10-27 00:00:00',284.87,381,15546,27.37,'Highly Ellip' from dual union all
select 16002,'016002','1985-077C','Sensor cover','Sensor cover',58300,'Reentered','1985-09-06 00:00:00','1985-09-01 00:00:00',89.44,241,253,82.31,'Intermediate' from dual union all
select 16003,'016003','1984-104U','Kosmos-1601 SS 13','ESO',57158,'Reentered','1988-05-06 00:00:00','1985-08-31 00:00:00',94.49,473,515,65.85,'Intermediate' from dual union all
select 16004,'016004','1984-104V','Kosmos-1601 SS 14','ESO',57158,'Reentered','1988-03-30 00:00:00','1985-08-31 00:00:00',94.37,469,508,65.83,'Intermediate' from dual union all
select 16005,'016005','1984-104W','Kosmos-1601 SS 15','ESO',57158,'Reentered','1988-03-26 00:00:00','1985-08-31 00:00:00',94.33,468,505,65.84,'Intermediate' from dual union all
select 16006,'016006','1984-104X','Kosmos-1601 SS 16','ESO',57158,'Reentered','1988-04-15 00:00:00','1985-08-31 00:00:00',94.43,472,510,65.82,'Intermediate' from dual union all
select 16007,'016007','1985-076G','PAM-D','Star 48B S/N 10042-17',58299,'In Earth orbit','','1985-10-31 00:00:00',638.2,399,35954,26.84,'GEO Transfer' from dual union all
select 16008,'016008','1985-050D','Dispenser cover','Romb dispenser cover',58273,'Reentered','1986-11-15 00:00:00','1985-07-31 00:00:00',94.16,445,510,65.85,'Intermediate' from dual union all
select 16009,'016009','1985-050E','Dispenser cover','Romb dispenser cover',58273,'Reentered','1986-11-03 00:00:00','1985-09-01 00:00:00',94.03,436,508,65.81,'Intermediate' from dual union all
select 1601,'001601','1965-020EB','deb Blok Kosmos','',11770,'Reentered','1967-01-17 00:00:00','1965-09-18 00:00:00',104.11,269,1637,56.08,'Intermediate' from dual union all
select 16010,'016010','1985-050F','Dispenser cover','Romb dispenser cover',58273,'Reentered','1986-11-28 00:00:00','1985-08-03 00:00:00',94.17,445,512,65.8,'Intermediate' from dual union all
select 16011,'016011','1985-079A','Kosmos-1680','Strela-2M',58302,'In Earth orbit','','1985-10-18 00:00:00',100.76,783,807,74.06,'Intermediate' from dual union all
select 16012,'016012','1985-079B','S3M','S3M',58302,'In Earth orbit','','1991-03-28 00:00:00',100.54,773,795,74.04,'Intermediate' from dual union all
select 16013,'016013','1985-076H','EVA screwdriver','',58299,'Reentered','1986-02-28 00:00:00','1985-09-06 00:00:00',92.18,312,451,28.53,'Intermediate' from dual union all
select 16014,'016014','1985-050G','Kosmos-1662 SS 1','ESO',58273,'Reentered','1986-12-22 00:00:00','1985-09-05 00:00:00',94.59,473,525,65.83,'Intermediate' from dual union all
select 16015,'016015','1985-050H','Kosmos-1662 SS 2','ESO',58273,'Reentered','1986-10-16 00:00:00','1985-09-05 00:00:00',94.38,469,509,65.84,'Intermediate' from dual union all
select 16016,'016016','1985-050J','Kosmos-1662 SS 3','ESO',58273,'Reentered','1986-10-09 00:00:00','1985-09-05 00:00:00',94.36,469,507,65.84,'Intermediate' from dual union all
select 16017,'016017','1985-050K','Kosmos-1662 SS 4','ESO',58273,'Reentered','1987-01-19 00:00:00','1985-09-05 00:00:00',94.59,471,527,65.83,'Intermediate' from dual union all
select 16018,'016018','1985-080A','Kosmos-1681','Fram',58303,'Landed','1985-09-19 00:00:00','1985-09-08 00:00:00',88.92,218,225,82.32,'Intermediate' from dual union all
select 16019,'016019','1985-080B','11S510','11S510',58303,'Reentered','1985-09-10 00:00:00','1985-09-08 00:00:00',88.42,185,208,82.32,'Intermediate' from dual union all
select 1602,'001602','1965-074A','OPS 7221','CORONA J-24',11824,'Reentered Attached','1965-10-11 00:00:00','1965-09-29 00:00:00',89.84,172,361,80.03,'Intermediate' from dual union all
select 16020,'016020','1985-066F','NNS O-30 despin weight','',58289,'In Earth orbit','','1991-05-05 00:00:00',107.6,1008,1224,90.23,'Polar' from dual union all
select 16021,'016021','1985-077D','Blenda','Sunshade',58300,'Reentered','1985-09-15 00:00:00','1985-09-12 00:00:00',90.23,251,321,82.36,'Intermediate' from dual union all
select 16022,'016022','1985-077E','KDU part','KDU part',58300,'Reentered','1985-10-03 00:00:00','1985-09-29 00:00:00',90.12,229,332,82.3,'Intermediate' from dual union all
select 16023,'016023','1985-077F','KDU','KDU',58300,'Reentered','1985-10-25 00:00:00','1985-09-12 00:00:00',89.87,263,273,82.32,'Intermediate' from dual union all
select 16024,'016024','1985-077G','KDU part','KDU part',58300,'Reentered','1985-09-14 00:00:00','1985-09-12 00:00:00',89.68,253,265,82.33,'Intermediate' from dual union all
select 16025,'016025','1985-077H','KDU part','KDU part',58300,'Reentered','1985-09-14 00:00:00','1985-09-12 00:00:00',89.55,240,265,82.32,'Intermediate' from dual union all
select 16026,'016026','1985-077J','KDU part','KDU part',58300,'Reentered','1985-09-27 00:00:00','1985-09-12 00:00:00',92.4,268,517,82.3,'Intermediate' from dual union all
select 16027,'016027','1979-017E','P78-1 debris','',47501,'Reentered','1986-01-02 00:00:00','1985-09-14 00:00:00',92.07,212,541,97.28,'Sun-Synch' from dual;
insert into SATELLITE
select 16028,'016028','1979-017F','P78-1 debris','',47501,'Reentered','1988-04-03 00:00:00','1985-09-14 00:00:00',95.06,506,537,97.68,'Sun-Synch' from dual union all
select 16029,'016029','1979-017G','P78-1 debris','',47501,'Reentered','1989-11-11 00:00:00','1985-09-14 00:00:00',95.9,521,604,97.8,'Sun-Synch' from dual union all
select 1603,'001603','1965-075A','Kosmos-91','Zenit-4 No. 9',11825,'Landed','1965-10-01 00:00:00','1965-09-23 00:00:00',89.78,204,324,64.98,'Intermediate' from dual union all
select 16030,'016030','1979-017H','P78-1 debris','',47501,'Reentered','1989-03-23 00:00:00','1985-09-14 00:00:00',96.83,524,690,97.65,'Sun-Synch' from dual union all
select 16031,'016031','1979-017J','P78-1 debris','',47501,'Reentered','1986-01-01 00:00:00','1985-09-14 00:00:00',94.79,480,537,97.72,'Sun-Synch' from dual union all
select 16032,'016032','1979-017K','P78-1 debris','',47501,'Reentered','1987-08-16 00:00:00','1985-09-14 00:00:00',94.26,417,549,97.53,'Sun-Synch' from dual union all
select 16033,'016033','1979-017L','P78-1 debris','',47501,'Reentered','1988-04-04 00:00:00','1985-09-14 00:00:00',94.77,491,524,97.62,'Sun-Synch' from dual union all
select 16034,'016034','1979-017M','P78-1 debris','',47501,'Reentered','1986-06-27 00:00:00','1985-09-14 00:00:00',93.81,299,622,97.72,'Sun-Synch' from dual union all
select 16035,'016035','1979-017N','P78-1 debris','',47501,'Reentered','1988-10-31 00:00:00','1985-09-16 00:00:00',96.13,512,635,98.18,'Sun-Synch' from dual union all
select 16036,'016036','1979-017P','P78-1 debris','',47501,'Reentered','1990-05-12 00:00:00','1985-09-15 00:00:00',96.06,523,617,98.08,'Sun-Synch' from dual union all
select 16037,'016037','1979-017Q','P78-1 debris','',47501,'Reentered','1985-10-08 00:00:00','1985-09-14 00:00:00',92.44,261,528,97.37,'Sun-Synch' from dual union all
select 16038,'016038','1979-017R','P78-1 debris','',47501,'Reentered','1988-12-27 00:00:00','1985-09-15 00:00:00',95.37,499,574,97.58,'Sun-Synch' from dual union all
select 16039,'016039','1979-017S','P78-1 debris','',47501,'Reentered','1988-08-09 00:00:00','1985-09-15 00:00:00',95.67,515,587,97.68,'Sun-Synch' from dual union all
select 1604,'001604','1965-075B','11A57I','11A57I',11825,'Reentered','1965-10-04 00:00:00','1965-09-25 00:00:00',89.51,204,297,64.97,'Intermediate' from dual union all
select 16040,'016040','1979-017T','P78-1 debris','',47501,'Reentered','1987-02-23 00:00:00','1985-09-15 00:00:00',94.86,490,534,97.57,'Sun-Synch' from dual union all
select 16041,'016041','1979-017U','P78-1 debris','',47501,'Reentered','1989-08-20 00:00:00','1985-09-15 00:00:00',95.54,505,585,97.66,'Sun-Synch' from dual union all
select 16042,'016042','1979-017V','P78-1 debris','',47501,'Reentered','1988-07-29 00:00:00','1985-09-14 00:00:00',96.79,518,692,97.69,'Sun-Synch' from dual union all
select 16043,'016043','1979-017W','P78-1 debris','',47501,'Reentered','1985-12-18 00:00:00','1985-09-16 00:00:00',95.81,510,606,97.7,'Sun-Synch' from dual union all
select 16044,'016044','1979-017X','P78-1 debris','',47501,'Reentered','1989-05-20 00:00:00','1985-09-16 00:00:00',95.96,520,609,97.76,'Sun-Synch' from dual union all
select 16045,'016045','1979-017Y','P78-1 debris','',47501,'Reentered','1989-05-23 00:00:00','1985-09-16 00:00:00',96.66,502,695,97.71,'Sun-Synch' from dual union all
select 16046,'016046','1982-033EX','Garbage bag','',54342,'Reentered','1985-11-24 00:00:00','1985-09-17 00:00:00',91.49,337,359,51.64,'Intermediate' from dual union all
select 16047,'016047','1982-033EY','Garbage bag','',54342,'Reentered','1986-02-02 00:00:00','1985-09-17 00:00:00',91.46,337,356,51.62,'Intermediate' from dual union all
select 16048,'016048','1982-033EZ','Garbage bag','',54342,'Reentered','1986-01-01 00:00:00','1985-09-17 00:00:00',91.4,335,352,51.64,'Intermediate' from dual union all
select 16049,'016049','1982-033FA','Garbage bag','',54342,'Reentered','1985-12-24 00:00:00','1985-09-17 00:00:00',91.35,330,352,51.62,'Intermediate' from dual union all
select 1605,'001605','1965-075C','deb Kosmos-91','',11825,'Reentered','1965-09-29 00:00:00','1965-09-24 00:00:00',91.52,322,376,64.8,'Intermediate' from dual union all
select 16050,'016050','1982-033FB','Garbage bag','',54342,'Reentered','1985-10-25 00:00:00','1985-09-16 00:00:00',91.1,326,331,51.64,'Intermediate' from dual union all
select 16051,'016051','1985-081A','Soyuz T-14','Soyuz 7K-ST No. 20L',58304,'Landed','1985-11-21 00:00:00','1985-10-17 00:00:00',91.68,357,357,51.63,'Intermediate' from dual union all
select 16052,'016052','1985-081B','11S510','11S510',58304,'Reentered','1985-09-20 00:00:00','1985-09-18 00:00:00',88.26,183,194,51.62,'Intermediate' from dual union all
select 16053,'016053','1982-033FC','Garbage bag','',54342,'Reentered','1986-05-31 00:00:00','1985-09-20 00:00:00',91.4,336,351,51.61,'Intermediate' from dual union all
select 16054,'016054','1985-082A','Kosmos-1682','US-PM1',58305,'Reentered','1988-05-17 00:00:00','1985-10-20 00:00:00',93.28,429,441,65.02,'Intermediate' from dual union all
select 16055,'016055','1985-082B','11S692','11S692',58305,'Reentered','1985-09-19 00:00:00','1985-09-19 00:00:00',89.73,115,408,64.99,'Intermediate' from dual union all
select 16056,'016056','1985-083A','Kosmos-1683','Oblik No. 316',58306,'Landed','1985-10-04 00:00:00','1985-09-26 00:00:00',92.25,356,413,72.86,'Intermediate' from dual union all
select 16057,'016057','1985-083B','11S510','11S510',58306,'Reentered','1985-10-03 00:00:00','1985-09-21 00:00:00',89.92,193,348,72.86,'Intermediate' from dual union all
select 16058,'016058','1985-080C','PTDU cover','KDU part',58303,'Reentered','1985-09-23 00:00:00','1985-09-19 00:00:00',90.76,212,412,82.32,'Intermediate' from dual union all
select 16059,'016059','1985-080D','KDU','KDU',58303,'Reentered','1985-09-26 00:00:00','1985-09-19 00:00:00',88.73,209,215,82.33,'Intermediate' from dual union all
select 1606,'001606','1965-075D','deb Kosmos-91','',11825,'Reentered','1965-09-29 00:00:00','1965-09-24 00:00:00',91.79,328,397,64.9,'Intermediate' from dual union all
select 16060,'016060','1985-080E','Blenda','Sunshade',58303,'Reentered','1985-09-20 00:00:00','1985-09-19 00:00:00',88.57,198,210,82.32,'Intermediate' from dual union all
select 16061,'016061','1985-071F','BOZ part','',58294,'Reentered','1985-09-30 00:00:00','1985-09-22 00:00:00',90,198,350,62.84,'Intermediate' from dual union all
select 16062,'016062','1982-033FD','Garbage bag','',54342,'Reentered','1986-01-03 00:00:00','1985-09-22 00:00:00',91.37,334,350,51.64,'Intermediate' from dual union all
select 16063,'016063','1982-033FE','Garbage bag','',54342,'Reentered','1985-09-30 00:00:00','1985-09-23 00:00:00',91.25,328,344,51.63,'Intermediate' from dual union all
select 16064,'016064','1985-084A','Kosmos-1684','Oko',58307,'In Earth orbit','','1985-11-24 00:00:00',717.64,613,39735,62.96,'Molniya' from dual union all
select 16065,'016065','1985-084B','11S510','11S510',58307,'Reentered','1985-11-21 00:00:00','1985-10-18 00:00:00',91.31,209,469,62.84,'Intermediate' from dual union all
select 16066,'016066','1985-084C','BOZ','BOZ',58307,'Reentered','1985-10-25 00:00:00','1985-10-01 00:00:00',91.93,182,557,62.86,'Intermediate' from dual union all
select 16067,'016067','1984-104Y','Kosmos-1601 SS 17','ESO',57158,'Reentered','1988-03-17 00:00:00','1985-09-24 00:00:00',94.3,464,506,65.83,'Intermediate' from dual union all
select 16068,'016068','1984-104Z','Kosmos-1601 SS 18','ESO',57158,'Reentered','1988-04-07 00:00:00','1985-09-26 00:00:00',94.36,470,506,65.85,'Intermediate' from dual union all
select 16069,'016069','1984-104AA','Kosmos-1601 SS 19','ESO',57158,'Reentered','1988-05-08 00:00:00','1985-09-25 00:00:00',94.49,480,508,65.83,'Intermediate' from dual union all
select 1607,'001607','1965-075E','deb Kosmos-91','',11825,'Reentered','1965-09-29 00:00:00','1965-09-24 00:00:00',89.87,205,331,65,'Intermediate' from dual union all
select 16070,'016070','1985-084D','Blok-2BL','Blok-2BL No. 77326-663',58307,'In Earth orbit','','1985-11-24 00:00:00',705.99,616,39156,62.96,'Molniya' from dual union all
select 16071,'016071','1979-017Z','P78-1 debris','',47501,'Reentered','1987-07-22 00:00:00','1985-09-24 00:00:00',108.07,510,1765,97.12,'Sun-Synch' from dual union all
select 16072,'016072','1979-017AA','P78-1 debris','',47501,'Reentered','1988-09-07 00:00:00','1985-09-24 00:00:00',99.21,497,945,98.73,'Sun-Synch' from dual union all
select 16073,'016073','1979-017AB','P78-1 debris','',47501,'Reentered','1989-05-01 00:00:00','1985-09-24 00:00:00',99.12,506,927,96.77,'Sun-Synch' from dual union all
select 16074,'016074','1979-017AC','P78-1 debris','',47501,'Reentered','1989-01-12 00:00:00','1985-09-24 00:00:00',99.02,506,918,96.81,'Sun-Synch' from dual union all
select 16075,'016075','1979-017AD','P78-1 debris','',47501,'Reentered','1986-07-07 00:00:00','1985-09-24 00:00:00',95.46,400,681,96.98,'Sun-Synch' from dual union all
select 16076,'016076','1979-017AE','P78-1 debris','',47501,'Reentered','1986-12-02 00:00:00','1985-09-24 00:00:00',95.5,471,615,97.01,'Sun-Synch' from dual union all
select 16077,'016077','1984-104AB','Kosmos-1601 SS 20','ESO',57158,'Reentered','1988-06-11 00:00:00','1985-09-24 00:00:00',94.54,485,508,65.86,'Intermediate' from dual union all
select 16078,'016078','1979-017AF','P78-1 debris','',47501,'Reentered','1988-07-16 00:00:00','1985-09-25 00:00:00',96.54,522,664,97.49,'Sun-Synch' from dual union all
select 16079,'016079','1979-017AG','P78-1 debris','',47501,'Reentered','1988-04-08 00:00:00','1985-10-17 00:00:00',96.69,525,676,97.74,'Sun-Synch' from dual union all
select 1608,'001608','1964-083H','deb NNS O-2','',10369,'In Earth orbit','','1965-10-01 00:00:00',106.29,1025,1085,89.98,'Polar' from dual union all
select 16080,'016080','1979-017AH','P78-1 debris','',47501,'Reentered','1988-09-24 00:00:00','1985-09-25 00:00:00',96.11,517,628,98.11,'Sun-Synch' from dual union all
select 16081,'016081','1979-017AJ','P78-1 debris','',47501,'Reentered','1988-11-30 00:00:00','1985-09-25 00:00:00',95.83,518,599,98.02,'Sun-Synch' from dual union all
select 16082,'016082','1979-017AK','P78-1 debris','',47501,'Reentered','1988-07-09 00:00:00','1985-09-24 00:00:00',95.78,515,598,97.65,'Sun-Synch' from dual union all
select 16083,'016083','1979-017AL','P78-1 debris','',47501,'Reentered','1991-02-27 00:00:00','1985-09-26 00:00:00',95.57,516,577,97.61,'Sun-Synch' from dual union all
select 16084,'016084','1979-017AM','P78-1 debris','',47501,'Reentered','2000-04-14 00:00:00','1993-08-08 00:00:00',93.6,439,463,97.86,'Sun-Synch' from dual union all
select 16085,'016085','1979-017AN','P78-1 debris','',47501,'Reentered','2002-12-06 00:00:00','1993-08-09 00:00:00',94.2,474,486,97.82,'Sun-Synch' from dual union all
select 16086,'016086','1979-017AP','P78-1 debris','',47501,'Reentered','1988-09-19 00:00:00','1985-09-30 00:00:00',95.2,524,533,98.1,'Sun-Synch' from dual union all
select 16087,'016087','1979-017AQ','P78-1 debris','',47501,'Reentered','1990-03-28 00:00:00','1985-10-02 00:00:00',95.75,524,586,97.72,'Sun-Synch' from dual union all
select 16088,'016088','1985-085A','Kosmos-1685','Oblik No. 331',58308,'Landed','1985-10-10 00:00:00','1985-09-28 00:00:00',92.25,355,415,72.86,'Intermediate' from dual union all
select 16089,'016089','1985-085B','11S510','11S510',58308,'Reentered','1985-10-08 00:00:00','1985-09-28 00:00:00',89.77,194,333,72.85,'Intermediate' from dual union all
select 1609,'001609','1965-076A','OPS 7208','GAMBIT SV 972',11826,'Reentered','1965-10-05 00:00:00','1965-10-01 00:00:00',88.71,158,264,95.6,'Sun-Synch' from dual union all
select 16090,'016090','1985-084E','deb Kosmos-1684','',58307,'Reentered','1985-11-16 00:00:00','1985-10-01 00:00:00',92.47,212,579,62.83,'Intermediate' from dual union all
select 16091,'016091','1982-033FF','Garbage bag','',54342,'Reentered','1986-01-20 00:00:00','1985-09-26 00:00:00',91.44,335,356,51.63,'Intermediate' from dual union all
select 16092,'016092','1982-033FG','Garbage bag','',54342,'Reentered','1985-12-22 00:00:00','1985-09-26 00:00:00',91.45,336,356,51.62,'Intermediate' from dual union all
select 16093,'016093','1982-033FH','Garbage bag','',54342,'Reentered','1986-02-26 00:00:00','1985-09-26 00:00:00',91.41,336,351,51.63,'Intermediate' from dual union all
select 16094,'016094','1982-033FJ','Garbage bag','',54342,'Reentered','1986-02-11 00:00:00','1985-09-26 00:00:00',91.38,336,349,51.64,'Intermediate' from dual union all
select 16095,'016095','1985-086A','Kosmos-1686','TKS-M No. 16501',58309,'Reentered Attached','1991-02-07 00:00:00','1985-10-27 00:00:00',91.67,338,375,51.58,'Intermediate' from dual union all
select 16096,'016096','1985-086B','8S812','8S812',58309,'Reentered','1985-10-02 00:00:00','1985-09-29 00:00:00',88.64,162,252,51.62,'Intermediate' from dual union all
select 16097,'016097','1985-043C','Soyuz T-13 BO','Bitovoy otsek',58266,'Reentered','1986-01-21 00:00:00','1985-09-27 00:00:00',91.36,337,346,51.63,'Intermediate' from dual union all
select 16098,'016098','1982-033FK','Garbage bag','',54342,'Reentered','1985-10-03 00:00:00','1985-09-27 00:00:00',90.53,222,378,51.66,'Intermediate' from dual union all
select 16099,'016099','1982-033FL','Garbage bag','',54342,'Reentered','1985-09-30 00:00:00','1985-09-28 00:00:00',90.16,206,359,51.64,'Intermediate' from dual union all
select 161,'000161','61 Omicron 45','deb r/b Ablestar','',7273,'Reentered','1981-08-29 00:00:00','1961-09-21 00:00:00',101.34,663,982,66.85,'Intermediate' from dual union all
select 1610,'001610','1965-077A','Luna-7','E-6 No. 11',11827,'Deep Space','1965-10-07 00:00:00','1965-10-04 00:00:00',21549.85,286,500000,64.76,'Cislunar' from dual union all
select 16100,'016100','1982-033FM','Garbage bag','',54342,'Reentered','1985-12-18 00:00:00','1985-09-28 00:00:00',91.38,336,349,51.64,'Intermediate' from dual union all
select 16101,'016101','1985-087A','INTELSAT 512','INTELSAT VA F12',58310,'In Earth orbit','','1985-12-04 00:00:00',1436.06,35752,35820,.08,'Stationary' from dual union all
select 16102,'016102','1985-087B','Centaur AC-65','Centaur D-1AR AC-65',58310,'Reentered','2012-04-20 00:00:00','1985-11-30 00:00:00',603.61,315,34247,22.95,'GEO Transfer' from dual union all
select 16103,'016103','1985-088A','Kosmos-1687','Oko',58311,'In Earth orbit','','1985-11-30 00:00:00',717.86,810,39549,62.9,'Molniya' from dual union all
select 16104,'016104','1985-088B','11S510','11S510',58311,'Reentered','1985-11-28 00:00:00','1985-10-01 00:00:00',92.33,210,568,62.89,'Intermediate' from dual union all
select 16105,'016105','1985-088C','BOZ','BOZ',58311,'Reentered','1985-11-03 00:00:00','1985-10-08 00:00:00',91.89,185,550,62.89,'Intermediate' from dual union all
select 16106,'016106','1985-088D','Blok-2BL','Blok-2BL No. 76752-662',58311,'In Earth orbit','','1985-12-03 00:00:00',703.6,671,38982,62.99,'Molniya' from dual union all
select 16107,'016107','1985-089A','Kosmos-1688','Romb',58312,'Reentered','1988-07-02 00:00:00','1985-11-01 00:00:00',93.47,346,543,50.68,'Intermediate' from dual union all
select 16108,'016108','1985-089B','S3M','S3M',58312,'Reentered','1988-06-19 00:00:00','1985-12-01 00:00:00',93.34,341,535,50.68,'Intermediate' from dual union all
select 16109,'016109','1984-104AC','Kosmos-1601 SS 21','ESO',57158,'Reentered','1988-04-13 00:00:00','1985-10-02 00:00:00',94.4,473,506,65.86,'Intermediate' from dual union all
select 1611,'001611','1965-077B','8K78I','8K78I',11827,'Reentered','1965-10-04 00:00:00','1965-10-04 00:00:00',88.64,129,285,64.76,'Intermediate' from dual union all
select 16110,'016110','1985-090A','Kosmos-1689','Resurs-O1 No. 1L',58313,'Reentered','2001-01-14 00:00:00','1985-11-02 00:00:00',96.98,573,655,97.97,'Sun-Synch' from dual union all
select 16111,'016111','1985-090B','8A92ME','8A92ME',58313,'In Earth orbit','','1990-04-01 00:00:00',96.88,568,650,97.77,'Sun-Synch' from dual union all
select 16112,'016112','1985-091A','Molniya-3','Molniya-3 No. 38',58314,'Reentered','2001-02-22 00:00:00','1985-12-09 00:00:00',717.64,585,39763,62.96,'Molniya' from dual union all
select 16113,'016113','1985-091B','11S510','11S510',58314,'Reentered','1985-12-15 00:00:00','1985-10-03 00:00:00',92.61,212,592,62.75,'Intermediate' from dual union all
select 16114,'016114','1985-091C','BOZ','BOZ',58314,'Reentered','1985-11-16 00:00:00','1985-10-18 00:00:00',91.75,188,532,62.81,'Intermediate' from dual union all
select 16115,'016115','1985-092A','Atlantis (STS 51-J)','OV-104',58315,'Landed','1985-10-07 00:00:00','1985-10-05 00:00:00',94.52,476,515,28.52,'Intermediate' from dual union all
select 16116,'016116','1985-092B','DSCS III B-4','DSCS III B-4',58315,'In Earth orbit','','2008-03-09 00:00:00',1454.7,36119,36181,10.37,'Inclined Drift' from dual union all
select 16117,'016117','1985-092C','DSCS III B-5','DSCS III B-5',58315,'In Earth orbit','','2008-11-20 00:00:00',1449.8,36036,36073,10.72,'Inclined Drift' from dual;
insert into SATELLITE
select 16118,'016118','1985-092D','IUS-12 SRM-1','Orbus 21',58315,'In Earth orbit','','1985-10-03 00:00:00',636.58,470,35800,27,'GEO Transfer' from dual union all
select 16119,'016119','1985-092E','IUS-12 SRM-2','Orbus 6E',58315,'In Earth orbit','','2009-04-20 00:00:00',1424.56,35319,35802,13.94,'Inclined Drift' from dual union all
select 1612,'001612','1965-077C','BOZ','BOZ',11827,'Reentered','1965-10-05 00:00:00','1965-10-04 00:00:00',88.64,129,285,64.76,'Intermediate' from dual union all
select 16120,'016120','1985-083C','Blenda','Sunshade',58306,'Reentered','1986-03-28 00:00:00','1985-11-20 00:00:00',91.79,335,390,72.86,'Intermediate' from dual union all
select 16121,'016121','1985-083D','KDU','KDU',58306,'Reentered','1987-12-26 00:00:00','1985-11-20 00:00:00',92.17,354,408,72.87,'Intermediate' from dual union all
select 16122,'016122','1982-033FN','Garbage bag','',54342,'Reentered','1986-10-03 00:00:00','1985-10-04 00:00:00',91.35,335,347,51.63,'Intermediate' from dual union all
select 16123,'016123','1985-083E','KDU part','KDU part',58306,'Reentered','1985-12-24 00:00:00','1985-10-20 00:00:00',94.64,384,619,72.87,'Intermediate' from dual union all
select 16124,'016124','1985-083F','KDU part','KDU part',58306,'Reentered','1986-05-22 00:00:00','1985-11-18 00:00:00',94,385,556,72.87,'Intermediate' from dual union all
select 16125,'016125','1985-091D','Blok-ML','Blok-ML',58314,'Reentered','2001-02-20 00:00:00','1985-12-04 00:00:00',734,662,40490,63.02,'Molniya' from dual union all
select 16126,'016126','1985-083G','KDU part','KDU part',58306,'Reentered','1985-10-24 00:00:00','1985-10-20 00:00:00',92.39,377,406,72.89,'Intermediate' from dual union all
select 16127,'016127','1985-083H','KDU part','KDU part',58306,'Reentered','1986-03-04 00:00:00','1985-11-19 00:00:00',91.86,344,387,72.87,'Intermediate' from dual union all
select 16128,'016128','1985-086C','Pion-K cover','TKS-M sensor cover',58309,'Reentered','1986-02-01 00:00:00','1985-11-27 00:00:00',91.1,325,332,51.63,'Intermediate' from dual union all
select 16129,'016129','1985-093A','GPS 11','Navstar GPS SVN 11',58316,'In Earth orbit','','1985-12-08 00:00:00',717.84,19823,20535,63.36,'Medium' from dual union all
select 1613,'001613','1965-078A','OV1-2','OV1-2S',11828,'In Earth orbit','','1966-01-29 00:00:00',125.67,413,3453,144.28,'Medium' from dual union all
select 16130,'016130','1979-017AR','P78-1 debris','',47501,'Reentered','1985-10-10 00:00:00','1985-10-07 00:00:00',89.96,221,324,97.36,'Sun-Synch' from dual union all
select 16131,'016131','1979-017AS','P78-1 debris','',47501,'Reentered','1985-12-15 00:00:00','1985-10-04 00:00:00',92.62,257,548,97.82,'Sun-Synch' from dual union all
select 16132,'016132','1979-017AT','P78-1 debris','',47501,'Reentered','1985-10-24 00:00:00','1985-10-15 00:00:00',93.3,341,531,98.03,'Sun-Synch' from dual union all
select 16133,'016133','1979-017AU','P78-1 debris','',47501,'Reentered','1987-11-27 00:00:00','1985-10-13 00:00:00',94.09,425,523,97.87,'Sun-Synch' from dual union all
select 16134,'016134','1985-085C','Blenda','Sunshade',58308,'Reentered','1986-02-08 00:00:00','1985-11-25 00:00:00',91.96,354,388,72.88,'Intermediate' from dual union all
select 16135,'016135','1985-085D','KDU','KDU',58308,'Reentered','1987-11-24 00:00:00','1985-11-28 00:00:00',92.16,355,406,72.86,'Intermediate' from dual union all
select 16136,'016136','1985-085E','KDU part','KDU part',58308,'Reentered','1985-12-23 00:00:00','1985-11-05 00:00:00',91.68,331,383,72.87,'Intermediate' from dual union all
select 16137,'016137','1985-093B','SGS-II','Star 48  S/N 10022-9',58316,'In Earth orbit','','1985-12-11 00:00:00',368.27,580,20702,63.04,'Highly Ellip' from dual union all
select 16138,'016138','1985-094A','Kosmos-1690','Strela-3',58317,'In Earth orbit','','1985-11-16 00:00:00',113.76,1382,1415,82.61,'Intermediate' from dual union all
select 16139,'016139','1985-094B','Kosmos-1695','Strela-3',58317,'In Earth orbit','','1992-06-02 00:00:00',114.06,1412,1413,82.61,'Intermediate' from dual union all
select 1614,'001614','1965-076B','Agena D 4821','Agena D',11826,'Reentered','1965-10-03 00:00:00','1965-10-01 00:00:00',88.71,158,264,95.6,'Sun-Synch' from dual union all
select 16140,'016140','1985-094C','Kosmos-1692','Strela-3',58317,'In Earth orbit','','1992-06-01 00:00:00',113.83,1389,1416,82.59,'Intermediate' from dual union all
select 16141,'016141','1985-094D','Kosmos-1693','Strela-3',58317,'In Earth orbit','','1992-05-31 00:00:00',113.88,1394,1415,82.61,'Intermediate' from dual union all
select 16142,'016142','1985-094E','Kosmos-1694','Strela-3',58317,'In Earth orbit','','1992-06-01 00:00:00',113.94,1399,1416,82.6,'Intermediate' from dual union all
select 16143,'016143','1985-094F','Kosmos-1691','Strela-3',58317,'In Earth orbit','','1992-06-01 00:00:00',114.02,1406,1416,82.6,'Intermediate' from dual union all
select 16144,'016144','1985-094G','S5M','S5M',58317,'In Earth orbit','','1993-08-15 00:00:00',114.73,1417,1470,82.6,'Intermediate' from dual union all
select 16145,'016145','1985-085F','KDU part','KDU part',58308,'Reentered','1986-04-30 00:00:00','1985-11-28 00:00:00',93.78,373,546,72.86,'Intermediate' from dual union all
select 16146,'016146','1985-085G','KDU part','KDU part',58308,'Reentered','1986-04-15 00:00:00','1985-12-09 00:00:00',91.73,333,386,72.86,'Intermediate' from dual union all
select 16147,'016147','1979-017AV','P78-1 debris','',47501,'Reentered','1987-06-07 00:00:00','1985-10-09 00:00:00',95.36,517,555,97.83,'Sun-Synch' from dual union all
select 16148,'016148','1979-017AW','P78-1 debris','',47501,'Reentered','1985-12-20 00:00:00','1985-10-29 00:00:00',94.87,468,557,97.76,'Sun-Synch' from dual union all
select 16149,'016149','1979-017AX','P78-1 debris','',47501,'Reentered','1987-04-19 00:00:00','1985-09-28 00:00:00',95.24,523,538,97.42,'Sun-Synch' from dual union all
select 1615,'001615','1965-079A','OPS 5325','CORONA JX-28',11829,'Reentered Attached','1965-10-29 00:00:00','1965-10-16 00:00:00',89.58,200,308,75.02,'Intermediate' from dual union all
select 16150,'016150','1979-017AY','P78-1 debris','',47501,'Reentered','1987-10-13 00:00:00','1985-09-23 00:00:00',95.53,499,590,97.43,'Sun-Synch' from dual union all
select 16151,'016151','1979-017AZ','P78-1 debris','',47501,'Reentered','1988-05-10 00:00:00','1985-10-09 00:00:00',94.54,464,529,97.62,'Sun-Synch' from dual union all
select 16152,'016152','1979-017BA','P78-1 debris','',47501,'Reentered','1986-10-28 00:00:00','1985-10-09 00:00:00',94.91,470,559,97.17,'Sun-Synch' from dual union all
select 16153,'016153','1979-017BB','P78-1 debris','',47501,'Reentered','1986-01-21 00:00:00','1985-10-09 00:00:00',94.23,376,587,97.13,'Sun-Synch' from dual union all
select 16154,'016154','1979-017BC','P78-1 debris','',47501,'Reentered','1988-03-24 00:00:00','1985-10-09 00:00:00',94.86,480,544,97.4,'Sun-Synch' from dual union all
select 16155,'016155','1979-017BD','P78-1 debris','',47501,'Reentered','1988-06-19 00:00:00','1985-10-09 00:00:00',96.33,523,643,97.44,'Sun-Synch' from dual union all
select 16156,'016156','1979-017BE','P78-1 debris','',47501,'Reentered','1987-12-21 00:00:00','1985-10-09 00:00:00',96.07,521,619,97.53,'Sun-Synch' from dual union all
select 16157,'016157','1979-017BF','P78-1 debris','',47501,'Reentered','1988-06-27 00:00:00','1985-10-09 00:00:00',95.7,520,585,97.66,'Sun-Synch' from dual union all
select 16158,'016158','1979-017BG','P78-1 debris','',47501,'Reentered','1988-10-12 00:00:00','1985-10-09 00:00:00',96.37,506,664,97.64,'Sun-Synch' from dual union all
select 16159,'016159','1985-085H','KDU part','KDU part',58308,'Reentered','1986-02-24 00:00:00','1985-11-25 00:00:00',91.84,344,385,72.86,'Intermediate' from dual union all
select 1616,'001616','1965-078B','OV1-2P','X-258',11828,'In Earth orbit','','1966-01-15 00:00:00',125.6,413,3447,144.29,'Medium' from dual union all
select 16160,'016160','1985-085J','KDU part','KDU part',58308,'Reentered','1986-10-16 00:00:00','1985-11-25 00:00:00',92.09,354,400,72.86,'Intermediate' from dual union all
select 16161,'016161','1985-085K','KDU part','KDU part',58308,'Reentered','1985-10-18 00:00:00','1985-10-10 00:00:00',92.28,354,419,72.84,'Intermediate' from dual union all
select 16162,'016162','1985-085L','KDU part','KDU part',58308,'Reentered','1985-11-07 00:00:00','1985-10-10 00:00:00',92.5,332,463,72.69,'Intermediate' from dual union all
select 16163,'016163','1982-033FP','Garbage or K-1686 part','',54342,'Reentered','1986-02-09 00:00:00','1985-10-10 00:00:00',91.75,356,364,51.63,'Intermediate' from dual union all
select 16164,'016164','1982-033FQ','Garbage or K-1686 part','',54342,'Reentered','1986-02-17 00:00:00','1985-10-10 00:00:00',91.71,356,361,51.63,'Intermediate' from dual union all
select 16165,'016165','1982-033FR','Garbage or K-1686 part','',54342,'Reentered','1986-01-08 00:00:00','1985-10-10 00:00:00',91.67,354,359,51.62,'Intermediate' from dual union all
select 16166,'016166','1982-033FS','Garbage or K-1686 part','',54342,'Reentered','1987-01-15 00:00:00','1985-10-10 00:00:00',91.65,354,357,51.63,'Intermediate' from dual union all
select 16167,'016167','1982-033FT','Garbage or K-1686 part','',54342,'Reentered','1986-02-19 00:00:00','1985-10-10 00:00:00',91.65,355,356,51.64,'Intermediate' from dual union all
select 16168,'016168','1982-033FU','Garbage or K-1686 part','',54342,'Reentered','1986-01-24 00:00:00','1985-10-10 00:00:00',91.59,350,356,51.64,'Intermediate' from dual union all
select 16169,'016169','1985-095A','Kosmos-1696','Oblik',58318,'Landed','1985-10-30 00:00:00','1985-10-19 00:00:00',89.59,229,280,70.37,'Intermediate' from dual union all
select 1617,'001617','1965-073J','S3 part','',11823,'In Earth orbit','','1965-10-20 00:00:00',117.51,1381,1757,56.15,'Intermediate' from dual union all
select 16170,'016170','1985-095B','11S510','11S510',58318,'Reentered','1985-10-23 00:00:00','1985-10-18 00:00:00',89,200,251,70.38,'Intermediate' from dual union all
select 16171,'016171','1982-033FV','Garbage or K-1686 part','',54342,'Reentered','1986-01-19 00:00:00','1985-10-19 00:00:00',91.7,354,362,51.63,'Intermediate' from dual union all
select 16172,'016172','1982-033FW','Kosmos-1686 sensor cover','TKS-M main sensor cover',54342,'Reentered','1986-02-27 00:00:00','1985-10-27 00:00:00',91.67,354,359,51.64,'Intermediate' from dual union all
select 16173,'016173','1982-033FX','Garbage or K-1686 part','',54342,'Reentered','1986-02-05 00:00:00','1985-10-19 00:00:00',91.6,351,356,51.63,'Intermediate' from dual union all
select 16174,'016174','1982-033FY','Garbage or K-1686 part','',54342,'Reentered','1986-02-16 00:00:00','1985-10-20 00:00:00',91.6,349,358,51.62,'Intermediate' from dual union all
select 16175,'016175','1982-033FZ','Garbage or K-1686 part','',54342,'Reentered','1986-02-07 00:00:00','1985-10-20 00:00:00',91.69,354,361,51.6,'Intermediate' from dual union all
select 16176,'016176','1982-033GA','Garbage or K-1686 part','',54342,'Reentered','1985-12-31 00:00:00','1985-10-20 00:00:00',91.62,340,368,51.64,'Intermediate' from dual union all
select 16177,'016177','1985-096A','Fanhui Shi Weixing (07)','Jianbing-1 No. 8',58319,'Reentered','1985-11-07 00:00:00','1985-10-29 00:00:00',89.6,169,341,62.98,'Intermediate' from dual union all
select 16178,'016178','1985-096B','CZ-2C Stage 2','CZ-2C Stage 2',58319,'Reentered','1985-11-03 00:00:00','1985-10-23 00:00:00',89.92,170,371,62.97,'Intermediate' from dual union all
select 16179,'016179','1985-096C','CZ-2C sep motor cover','',58319,'Reentered','1985-10-24 00:00:00','1985-10-21 00:00:00',89.49,165,334,62.98,'Intermediate' from dual union all
select 1618,'001618','1965-073K','S3 part','',11823,'In Earth orbit','','1965-10-20 00:00:00',117.69,1391,1764,56.19,'Intermediate' from dual union all
select 16180,'016180','1985-096D','CZ-2C sep motor cover','',58319,'Reentered','1985-10-24 00:00:00','1985-10-22 00:00:00',89.65,167,347,62.94,'Intermediate' from dual union all
select 16181,'016181','1985-097A','Kosmos-1697','EPN No. 03.694',58320,'In Earth orbit','','1985-11-22 00:00:00',101.96,848,855,70.99,'Intermediate' from dual union all
select 16182,'016182','1985-097B','11S772','11S772',58320,'In Earth orbit','','1985-12-11 00:00:00',101.79,836,850,71.01,'Intermediate' from dual union all
select 16183,'016183','1985-098A','Kosmos-1698','Oko',58321,'In Earth orbit','','1986-01-03 00:00:00',717.98,651,39714,62.91,'Molniya' from dual union all
select 16184,'016184','1985-098B','11S510','11S510',58321,'Reentered','1986-01-07 00:00:00','1985-11-21 00:00:00',91.62,214,494,62.81,'Intermediate' from dual union all
select 16185,'016185','1985-098C','BOZ','BOZ',58321,'Reentered','1985-12-14 00:00:00','1985-10-29 00:00:00',92.4,197,588,62.86,'Intermediate' from dual union all
select 16186,'016186','1985-098D','Blok-2BL','Blok-2BL No. 71626-664',58321,'In Earth orbit','','1985-12-23 00:00:00',707.85,648,39216,62.94,'Molniya' from dual union all
select 16187,'016187','1985-099A','Molniya-1','Molniya-1 11F658 No. 73',58322,'Reentered','1999-02-13 00:00:00','1986-01-07 00:00:00',717.78,596,39759,63.15,'Molniya' from dual union all
select 16188,'016188','1985-099B','11S510','11S510',58322,'Reentered','1986-03-24 00:00:00','1985-11-22 00:00:00',93.11,230,624,62.96,'Intermediate' from dual union all
select 16189,'016189','1985-099C','BOZ','BOZ',58322,'Reentered','1986-03-21 00:00:00','1985-11-22 00:00:00',93.31,221,653,62.96,'Intermediate' from dual union all
select 1619,'001619','1965-080B','BOZ','BOZ',11830,'Reentered','1965-11-08 00:00:00','1965-10-24 00:00:00',90.61,183,426,64.88,'Intermediate' from dual union all
select 16190,'016190','1985-099D','deb BOZ','',58322,'Reentered','1986-02-12 00:00:00','1985-11-21 00:00:00',93.2,227,636,62.95,'Intermediate' from dual union all
select 16191,'016191','1985-100A','Meteor-3','Meteor-3 No. 2',58323,'In Earth orbit','','1985-11-24 00:00:00',109.89,1208,1236,82.55,'Intermediate' from dual union all
select 16192,'016192','1985-075B','Kosmos-1677 DU','US-A DU',58298,'Reentered','1985-12-14 00:00:00','1985-10-23 00:00:00',89.55,246,258,65,'Intermediate' from dual union all
select 16193,'016193','1985-075C','Kosmos-1677 RLS','US-A RLS',58298,'Reentered','1985-10-27 00:00:00','1985-10-23 00:00:00',89.4,238,252,64.99,'Intermediate' from dual union all
select 16194,'016194','1985-100B','S5M','S5M',58323,'In Earth orbit','','1990-06-29 00:00:00',110.21,1224,1249,82.56,'Intermediate' from dual union all
select 16195,'016195','1985-064B','Kosmos-1670 RLS','US-A RLS',58287,'Reentered','1985-10-27 00:00:00','1985-10-24 00:00:00',89.31,235,246,65.02,'Intermediate' from dual union all
select 16196,'016196','1985-064C','Kosmos-1670 DU','US-A DU',58287,'Reentered','1985-12-08 00:00:00','1985-10-24 00:00:00',89.54,246,258,65.01,'Intermediate' from dual union all
select 16197,'016197','1985-099E','Blok-ML','Blok-ML No. 71632-464',58322,'Reentered','1999-11-22 00:00:00','1986-02-20 00:00:00',698.07,579,38799,63.23,'Molniya' from dual union all
select 16198,'016198','1985-101A','Kosmos-1699','Kobal''t No. 504',58324,'Landed','1985-12-23 00:00:00','1985-11-01 00:00:00',89.11,160,301,67.14,'Intermediate' from dual union all
select 16199,'016199','1985-102A','Kosmos-1700','Al''tair No. 11L',58325,'In Earth orbit','','1985-12-25 00:00:00',1436.14,35767,35808,1.3,'Stationary' from dual union all
select 162,'000162','61 Rho 1','Tiros 3','NASA A-3',7277,'In Earth orbit','','1961-08-16 00:00:00',100.4,734,821,47.9,'Intermediate' from dual union all
select 1620,'001620','1965-081A','OGO 2','OGO C',11831,'Reentered','1981-09-17 00:00:00','1966-01-23 00:00:00',104.33,418,1509,87.37,'Polar' from dual union all
select 16200,'016200','1985-102B','8S812','8S812',58325,'Reentered','1985-10-28 00:00:00','1985-10-27 00:00:00',88.09,176,184,51.62,'Intermediate' from dual union all
select 16201,'016201','1985-102C','Perekhodnik','Sredniy perekhodnik',58325,'Reentered','1985-10-26 00:00:00','1985-10-25 00:00:00',88.58,192,217,51.62,'Intermediate' from dual union all
select 16202,'016202','1982-055D','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-04 00:00:00',103.55,858,995,65.83,'Intermediate' from dual union all
select 16203,'016203','1982-055E','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-01 00:00:00',103.72,878,991,65.84,'Intermediate' from dual union all
select 16204,'016204','1982-055F','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-05 00:00:00',104.4,943,990,65.84,'Intermediate' from dual union all
select 16205,'016205','1982-055G','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-08 00:00:00',105.2,959,1049,65.83,'Intermediate' from dual union all
select 16206,'016206','1982-055H','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-01 00:00:00',105,979,1010,65.84,'Intermediate' from dual union all
select 16207,'016207','1982-055J','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-02 00:00:00',106.1,986,1105,65.84,'Intermediate' from dual;
insert into SATELLITE
select 16208,'016208','1982-055K','deb Kosmos-1375','',54364,'In Earth orbit','','1993-07-29 00:00:00',104.69,952,1008,65.84,'Intermediate' from dual union all
select 16209,'016209','1982-055L','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-02 00:00:00',104.47,956,983,65.83,'Intermediate' from dual union all
select 1621,'001621','1965-080A','Molniya-1','Molniya-1 11F67 No. 4',11830,'Reentered','1967-03-17 00:00:00','1966-02-04 00:00:00',716.18,560,39716,65.19,'GEO Transfer' from dual union all
select 16210,'016210','1982-055M','deb Kosmos-1375','',54364,'In Earth orbit','','1993-07-29 00:00:00',104.79,959,1011,65.83,'Intermediate' from dual union all
select 16211,'016211','1982-055N','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-01 00:00:00',105.2,980,1028,65.84,'Intermediate' from dual union all
select 16212,'016212','1985-101B','11S510','11S510',58324,'Reentered','1985-10-30 00:00:00','1985-10-27 00:00:00',89.12,161,301,67.14,'Intermediate' from dual union all
select 16213,'016213','1985-064D','deb Kosmos-1670 ROCh','',58287,'Reentered','1988-02-22 00:00:00','1985-10-25 00:00:00',104.23,901,1016,64.91,'Intermediate' from dual union all
select 16214,'016214','1985-102D','Blok DM2','11S861 No. 9L',58325,'In Earth orbit','','1985-12-26 00:00:00',1431.19,35611,35770,1.33,'Drift GEO' from dual union all
select 16215,'016215','1985-102E','SOZ','SOZ',58325,'Reentered','1986-10-27 00:00:00','1985-12-26 00:00:00',632.73,279,35793,47.21,'GEO Transfer' from dual union all
select 16216,'016216','1985-102F','SOZ','SOZ',58325,'Reentered','1986-05-18 00:00:00','1985-11-30 00:00:00',631.66,240,35777,47.28,'GEO Transfer' from dual union all
select 16217,'016217','1982-033GB','Garbage or K-1686 part','',54342,'Reentered','1986-08-08 00:00:00','1985-10-31 00:00:00',91.67,355,358,51.64,'Intermediate' from dual union all
select 16218,'016218','1982-033GC','Garbage or K-1686 part','',54342,'Reentered','1986-04-15 00:00:00','1985-10-29 00:00:00',91.66,354,358,51.64,'Intermediate' from dual union all
select 16219,'016219','1985-075D','Kosmos-1677 reactor part','',58298,'Reentered','1985-11-28 00:00:00','1985-10-27 00:00:00',100.93,653,953,64.7,'Intermediate' from dual union all
select 1622,'001622','1965-080C','deb Blok-I','',11830,'Reentered','1965-10-22 00:00:00','1965-10-15 00:00:00',93.68,266,644,64.4,'Intermediate' from dual union all
select 16220,'016220','1985-103A','Molniya-1','Molniya-1 No. 56',58326,'Reentered','2002-08-02 00:00:00','1985-12-31 00:00:00',717.74,563,39790,62.81,'Molniya' from dual union all
select 16221,'016221','1985-103B','11S510','11S510',58326,'Reentered','1985-12-04 00:00:00','1985-11-06 00:00:00',90.77,208,417,62.85,'Intermediate' from dual union all
select 16222,'016222','1985-103C','BOZ','BOZ',58326,'Reentered','1985-11-18 00:00:00','1985-11-04 00:00:00',90.56,185,419,62.83,'Intermediate' from dual union all
select 16223,'016223','1985-103D','Blok-ML','Blok-ML',58326,'Reentered','2003-01-13 00:00:00','1985-12-28 00:00:00',701.06,686,38841,62.86,'Molniya' from dual union all
select 16224,'016224','1985-103E','deb BOZ','',58326,'Reentered','1985-11-20 00:00:00','1985-11-06 00:00:00',90.7,204,414,62.82,'Intermediate' from dual union all
select 16225,'016225','1985-103F','deb BOZ','',58326,'Reentered','1985-11-29 00:00:00','1985-11-04 00:00:00',91.13,216,444,62.83,'Intermediate' from dual union all
select 16226,'016226','1985-103G','deb BOZ','',58326,'Reentered','1985-12-01 00:00:00','1985-11-04 00:00:00',91.18,217,448,62.83,'Intermediate' from dual union all
select 16227,'016227','1985-095C','Blenda','Sunshade',58318,'Reentered','1985-11-07 00:00:00','1985-10-30 00:00:00',89.37,225,262,70.38,'Intermediate' from dual union all
select 16228,'016228','1985-095D','KDU','KDU',58318,'Reentered','1985-12-01 00:00:00','1985-10-30 00:00:00',89.76,162,363,70.36,'Intermediate' from dual union all
select 16229,'016229','1985-028E','Leasat PKM','TU-882',58251,'In Earth orbit','','1985-10-28 00:00:00',288.65,315,15864,27.03,'Highly Ellip' from dual union all
select 1623,'001623','1965-080D','8K78I','8K78I',11830,'Reentered','1965-11-16 00:00:00','1965-10-21 00:00:00',91.21,197,471,64.84,'Intermediate' from dual union all
select 16230,'016230','1985-104A','Challenger (STS 61-A)','OV-099',58327,'Landed','1985-11-06 00:00:00','1985-11-01 00:00:00',91.06,322,332,56.98,'Intermediate' from dual union all
select 16231,'016231','1985-104B','GLOMR','GLOMR',58327,'Reentered','1986-12-26 00:00:00','1985-11-30 00:00:00',90.96,311,332,56.99,'Intermediate' from dual union all
select 16232,'016232','1985-095E','KDU part','KDU part',58318,'Reentered','1985-11-07 00:00:00','1985-10-30 00:00:00',90.41,236,354,70.45,'Intermediate' from dual union all
select 16233,'016233','1985-095F','KDU part','KDU part',58318,'Reentered','1985-11-07 00:00:00','1985-10-30 00:00:00',91.31,246,432,70.39,'Intermediate' from dual union all
select 16234,'016234','1985-095G','KDU part','KDU part',58318,'Reentered','1985-11-03 00:00:00','1985-11-02 00:00:00',88.56,190,217,70.37,'Intermediate' from dual union all
select 16235,'016235','1985-105A','Kosmos-1701','Oko',58328,'Reentered','2001-05-11 00:00:00','1986-01-09 00:00:00',718,692,39674,63.12,'Molniya' from dual union all
select 16236,'016236','1985-105B','11S510','11S510',58328,'Reentered','1986-01-11 00:00:00','1985-12-09 00:00:00',91.14,204,456,62.82,'Intermediate' from dual union all
select 16237,'016237','1985-105C','BOZ','BOZ',58328,'Reentered','1985-12-24 00:00:00','1985-11-16 00:00:00',92.25,191,579,62.87,'Intermediate' from dual union all
select 16238,'016238','1982-033GD','Garbage bag','',54342,'Reentered','1986-05-27 00:00:00','1985-11-09 00:00:00',91.78,348,375,51.57,'Intermediate' from dual union all
select 16239,'016239','1982-033GE','Garbage bag','',54342,'Reentered','1985-12-03 00:00:00','1985-11-09 00:00:00',91.64,329,381,51.61,'Intermediate' from dual union all
select 1624,'001624','1965-082A','Transtage 4 debris','',11832,'Reentered','1972-07-27 00:00:00','1966-01-23 00:00:00',99.97,730,785,32.31,'Intermediate' from dual union all
select 16240,'016240','1982-033GF','Garbage bag','',54342,'Reentered','1986-11-16 00:00:00','1985-11-12 00:00:00',91.63,353,355,51.63,'Intermediate' from dual union all
select 16241,'016241','1982-033GG','Garbage bag','',54342,'Reentered','1986-02-22 00:00:00','1985-11-09 00:00:00',91.6,351,356,51.64,'Intermediate' from dual union all
select 16242,'016242','1982-033GH','Garbage bag','',54342,'Reentered','1986-04-09 00:00:00','1985-11-09 00:00:00',91.56,340,363,51.63,'Intermediate' from dual union all
select 16243,'016243','1985-105D','Blok-2BL','Blok-2BL No. 66',58328,'Reentered','2001-01-11 00:00:00','1986-01-10 00:00:00',706.24,687,39097,63.13,'Molniya' from dual union all
select 16244,'016244','1982-033GJ','Garbage bag','',54342,'Reentered','1986-03-09 00:00:00','1985-11-10 00:00:00',91.64,352,358,51.64,'Intermediate' from dual union all
select 16245,'016245','1982-033GK','Garbage bag','',54342,'Reentered','1985-12-05 00:00:00','1985-11-10 00:00:00',91.66,354,358,51.62,'Intermediate' from dual union all
select 16246,'016246','1982-033GL','Garbage bag','',54342,'Reentered','1986-03-15 00:00:00','1985-11-11 00:00:00',91.7,356,360,51.63,'Intermediate' from dual union all
select 16247,'016247','1985-106A','Kosmos-1702','Oblik No. 321',58329,'Landed','1985-11-27 00:00:00','1985-11-15 00:00:00',92.24,355,414,72.87,'Intermediate' from dual union all
select 16248,'016248','1985-106B','11S510','11S510',58329,'Reentered','1985-11-28 00:00:00','1985-11-20 00:00:00',89.51,186,315,72.87,'Intermediate' from dual union all
select 16249,'016249','1982-033GM','Garbage bag','',54342,'Reentered','1986-02-09 00:00:00','1985-11-16 00:00:00',91.7,354,362,51.63,'Intermediate' from dual union all
select 1625,'001625','1965-081B','Agena D 6801','Agena D 6801',11831,'Reentered','1987-11-27 00:00:00','1966-01-21 00:00:00',104.27,419,1502,87.38,'Polar' from dual union all
select 16250,'016250','1985-107A','Raduga','Gran'' No. 28L',58330,'In Earth orbit','','1986-01-18 00:00:00',1435.96,35779,35789,1.18,'Stationary' from dual union all
select 16251,'016251','1985-107B','8S812','8S812',58330,'Reentered','1985-11-18 00:00:00','1985-11-17 00:00:00',87.98,172,176,51.63,'Intermediate' from dual union all
select 16252,'016252','1985-107C','Perekhodnik','Sredniy perekhodnik',58330,'Reentered','1985-11-16 00:00:00','1985-11-15 00:00:00',88.42,161,232,51.64,'Intermediate' from dual union all
select 16253,'016253','1982-033GN','Garbage bag','',54342,'Reentered','1986-12-03 00:00:00','1985-11-16 00:00:00',91.69,356,359,51.63,'Intermediate' from dual union all
select 16254,'016254','1985-107D','SOZ','SOZ',58330,'Reentered','1986-10-27 00:00:00','1986-01-23 00:00:00',649.26,272,36647,47.13,'GEO Transfer' from dual union all
select 16255,'016255','1985-107E','SOZ','SOZ',58330,'Reentered','1986-07-19 00:00:00','1986-01-17 00:00:00',647.27,223,36594,47.05,'GEO Transfer' from dual union all
select 16256,'016256','1985-050L','Kosmos-1662 SS 5','ESO',58273,'Reentered','1987-03-25 00:00:00','1985-11-20 00:00:00',94.57,473,522,65.82,'Intermediate' from dual union all
select 16257,'016257','1985-050M','Kosmos-1662 SS 6','ESO',58273,'Reentered','1986-12-01 00:00:00','1985-11-20 00:00:00',94.34,458,516,65.85,'Intermediate' from dual union all
select 16258,'016258','1982-033GP','Garbage bag','',54342,'Reentered','1986-01-18 00:00:00','1985-11-20 00:00:00',91.58,350,355,51.64,'Intermediate' from dual union all
select 16259,'016259','1982-033GQ','Garbage bag','',54342,'Reentered','1986-02-17 00:00:00','1985-11-20 00:00:00',91.55,348,354,51.63,'Intermediate' from dual union all
select 1626,'001626','1965-083A','Kosmos-92','Zenit-4 No. 16',11833,'Landed','1965-10-24 00:00:00','1965-10-16 00:00:00',89.86,201,334,65,'Intermediate' from dual union all
select 16260,'016260','1982-033GR','Garbage bag','',54342,'Reentered','1986-06-07 00:00:00','1985-11-21 00:00:00',91.52,346,353,51.63,'Intermediate' from dual union all
select 16261,'016261','1985-081C','Soyuz T-14 BO','Bitovoy otsek',58304,'Reentered','1986-12-29 00:00:00','1985-11-21 00:00:00',91.56,349,354,51.63,'Intermediate' from dual union all
select 16262,'016262','1985-108A','Kosmos-1703','Tselina-D',58331,'In Earth orbit','','1985-12-22 00:00:00',97.71,633,666,82.51,'Intermediate' from dual union all
select 16263,'016263','1985-108B','S5M','S5M',58331,'In Earth orbit','','1991-03-27 00:00:00',97.53,623,657,82.5,'Intermediate' from dual union all
select 16264,'016264','1985-094H','deb Kosmos-1695','',58317,'Reentered','1990-03-23 00:00:00','1985-12-08 00:00:00',114.16,1395,1440,82.62,'Intermediate' from dual union all
select 16265,'016265','1985-094J','deb Kosmos-1695','',58317,'Reentered','1989-11-13 00:00:00','1985-12-17 00:00:00',114.12,1344,1487,82.6,'Intermediate' from dual union all
select 16266,'016266','1985-094K','deb Kosmos-1695','',58317,'In Earth orbit','','1993-08-15 00:00:00',114.01,1398,1423,82.61,'Intermediate' from dual union all
select 16267,'016267','1985-094L','deb Kosmos-1695','',58317,'In Earth orbit','','1993-08-06 00:00:00',112.78,1287,1422,82.77,'Intermediate' from dual union all
select 16268,'016268','1985-094M','deb Kosmos-1695','',58317,'In Earth orbit','','1993-08-08 00:00:00',114.94,1392,1514,82.72,'Intermediate' from dual union all
select 16269,'016269','1985-094N','deb Kosmos-1695','',58317,'In Earth orbit','','1992-12-26 00:00:00',114.11,1404,1426,82.63,'Intermediate' from dual union all
select 1627,'001627','1965-083B','11A57I','11A57I',11833,'Reentered','1965-10-29 00:00:00','1965-10-19 00:00:00',89.59,209,299,64.95,'Intermediate' from dual union all
select 16270,'016270','1985-094P','deb Kosmos-1695','',58317,'In Earth orbit','','1993-07-25 00:00:00',113.73,1194,1602,82.73,'Intermediate' from dual union all
select 16271,'016271','1985-094Q','deb Kosmos-1695','',58317,'In Earth orbit','','1993-08-03 00:00:00',114.02,1405,1417,82.6,'Intermediate' from dual union all
select 16272,'016272','1985-094R','deb Kosmos-1695','',58317,'In Earth orbit','','1993-08-07 00:00:00',113.42,1351,1416,82.62,'Intermediate' from dual union all
select 16273,'016273','1985-109A','Atlantis (STS 61-B)','OV-104',58332,'Landed','1985-12-03 00:00:00','1985-11-29 00:00:00',92.03,368,381,28.47,'Intermediate' from dual union all
select 16274,'016274','1985-109B','Morelos 2','Morelos 2',58332,'In Earth orbit','','1986-01-26 00:00:00',1436.06,35773,35799,2.81,'Stationary' from dual union all
select 16275,'016275','1985-109C','Aussat K2','Aussat K2',58332,'In Earth orbit','','1986-01-28 00:00:00',1436.09,35781,35792,.01,'Stationary' from dual union all
select 16276,'016276','1985-109D','Satcom K2','Satcom K2',58332,'In Earth orbit','','1986-01-26 00:00:00',1436.14,35778,35797,.02,'Stationary' from dual union all
select 16277,'016277','1985-109E','OET','OEX Target',58332,'Reentered','1987-03-02 00:00:00','1986-02-23 00:00:00',91.85,360,371,28.47,'Intermediate' from dual union all
select 16278,'016278','1979-017BH','P78-1 debris','',47501,'Reentered','1988-11-13 00:00:00','1985-11-27 00:00:00',96.36,524,645,97.67,'Sun-Synch' from dual union all
select 16279,'016279','1979-017BJ','P78-1 debris','',47501,'Reentered','1987-04-15 00:00:00','1985-11-30 00:00:00',94.58,371,626,97.22,'Sun-Synch' from dual union all
select 1628,'001628','1965-083C','deb Kosmos-92','',11833,'Reentered','1965-10-25 00:00:00','1965-10-16 00:00:00',89.84,204,330,65.1,'Intermediate' from dual union all
select 16280,'016280','1979-017BK','P78-1 debris','',47501,'Reentered','1989-03-10 00:00:00','1985-11-27 00:00:00',95.1,511,536,97.39,'Sun-Synch' from dual union all
select 16281,'016281','1979-017BL','P78-1 debris','',47501,'Reentered','1987-11-13 00:00:00','1985-11-27 00:00:00',94.48,424,563,96.98,'Sun-Synch' from dual union all
select 16282,'016282','1979-017BM','P78-1 debris','',47501,'Reentered','1988-01-28 00:00:00','1985-11-26 00:00:00',96.34,515,652,97.59,'Sun-Synch' from dual union all
select 16283,'016283','1979-017BN','P78-1 debris','',47501,'Reentered','1987-11-10 00:00:00','1985-10-15 00:00:00',95.43,520,559,97.42,'Sun-Synch' from dual union all
select 16284,'016284','1979-017BP','P78-1 debris','',47501,'Reentered','1988-05-25 00:00:00','1985-11-27 00:00:00',96.59,501,690,97.68,'Sun-Synch' from dual union all
select 16285,'016285','1985-106C','Blenda','Sunshade',58329,'Reentered','1986-05-31 00:00:00','1985-12-16 00:00:00',91.99,340,404,72.86,'Intermediate' from dual union all
select 16286,'016286','1985-106D','KDU','KDU',58329,'Reentered','1988-01-28 00:00:00','1985-12-14 00:00:00',92.21,355,410,72.87,'Intermediate' from dual union all
select 16287,'016287','1985-106E','KDU part','KDU part',58329,'Reentered','1985-12-13 00:00:00','1985-11-27 00:00:00',92.8,358,466,72.86,'Intermediate' from dual union all
select 16288,'016288','1985-106F','KDU part','KDU part',58329,'Reentered','1986-08-10 00:00:00','1985-12-13 00:00:00',94.28,388,580,72.87,'Intermediate' from dual union all
select 16289,'016289','1985-106G','KDU part','KDU part',58329,'Reentered','1986-04-07 00:00:00','1985-12-13 00:00:00',94.54,388,604,72.84,'Intermediate' from dual union all
select 1629,'001629','1965-084A','Kosmos-93','DS-U2-V No. 1',11834,'Reentered','1966-01-03 00:00:00','1966-01-02 00:00:00',88.15,158,208,48.34,'Intermediate' from dual union all
select 16290,'016290','1985-106H','KDU part','KDU part',58329,'Reentered','1986-03-19 00:00:00','1985-12-13 00:00:00',92.16,353,408,72.87,'Intermediate' from dual union all
select 16291,'016291','1985-110A','Kosmos-1704','Parus',58333,'In Earth orbit','','1985-12-28 00:00:00',104.84,964,1010,82.94,'Intermediate' from dual union all
select 16292,'016292','1985-110B','S3M','S3M',58333,'In Earth orbit','','1991-03-28 00:00:00',104.63,957,998,82.95,'Intermediate' from dual union all
select 16293,'016293','1985-109F','PAM-D','Star 48B S/N 10031-17',58332,'In Earth orbit','','1986-01-26 00:00:00',650.81,328,36670,25.56,'GEO Transfer' from dual union all
select 16294,'016294','1985-109G','PAM-D','Star 48B S/N 10043-17',58332,'In Earth orbit','','1986-01-31 00:00:00',647.72,333,36507,25.81,'GEO Transfer' from dual union all
select 16295,'016295','1985-109H','PAM-D2','Star 63D',58332,'In Earth orbit','','1986-01-29 00:00:00',626.42,344,35403,26.05,'GEO Transfer' from dual union all
select 16296,'016296','1985-111A','Kosmos-1705','Oblik No. 322',58334,'Landed','1985-12-17 00:00:00','1985-12-05 00:00:00',92.25,356,413,72.87,'Intermediate' from dual union all
select 16297,'016297','1985-111B','11S510','11S510',58334,'Reentered','1985-12-19 00:00:00','1985-12-10 00:00:00',89.54,190,314,72.86,'Intermediate' from dual union all
select 16298,'016298','1979-017BQ','P78-1 debris','',47501,'Reentered','1986-02-15 00:00:00','1985-12-03 00:00:00',93.47,379,510,97.8,'Sun-Synch' from dual;
insert into SATELLITE
select 16299,'016299','1979-017BR','P78-1 debris','',47501,'Reentered','1986-05-04 00:00:00','1985-12-03 00:00:00',93.95,414,522,97.41,'Sun-Synch' from dual union all
select 163,'000163','61 Sigma 1','Midas 3','Midas ETS-II F1',7278,'In Earth orbit','','1961-08-15 00:00:00',161.47,3354,3535,91.13,'Medium' from dual union all
select 1630,'001630','1965-084B','S1M','S1M',11834,'Reentered','1965-11-17 00:00:00','1965-10-21 00:00:00',91.58,208,497,48.36,'Intermediate' from dual union all
select 16300,'016300','1979-017BS','P78-1 debris','',47501,'Reentered','1988-07-12 00:00:00','1985-12-03 00:00:00',94.43,460,522,97.92,'Sun-Synch' from dual union all
select 16301,'016301','1979-017BT','P78-1 debris','',47501,'Reentered','1989-02-15 00:00:00','1985-09-21 00:00:00',95.11,510,537,97.63,'Sun-Synch' from dual union all
select 16302,'016302','1979-017BU','P78-1 debris','',47501,'Reentered','1987-04-23 00:00:00','1985-12-03 00:00:00',95.06,513,529,97.63,'Sun-Synch' from dual union all
select 16303,'016303','1979-017BV','P78-1 debris','',47501,'Reentered','1989-02-04 00:00:00','1985-12-03 00:00:00',94.9,496,532,98.03,'Sun-Synch' from dual union all
select 16304,'016304','1979-017BW','P78-1 debris','',47501,'Reentered','1986-05-20 00:00:00','1985-12-03 00:00:00',94.61,484,516,97.69,'Sun-Synch' from dual union all
select 16305,'016305','1985-111C','Sensor cover','Sensor cover',58334,'Reentered','1985-12-06 00:00:00','1985-12-05 00:00:00',88.88,174,265,72.87,'Intermediate' from dual union all
select 16306,'016306','1985-112A','Kosmos-1706','Kobal''t No. 837',58335,'Landed','1986-02-09 00:00:00','1986-01-10 00:00:00',89.34,175,308,67.14,'Intermediate' from dual union all
select 16307,'016307','1985-112B','11S510','11S510',58335,'Reentered','1985-12-16 00:00:00','1985-12-13 00:00:00',89.01,161,291,67.15,'Intermediate' from dual union all
select 16308,'016308','1979-017BX','P78-1 debris','',47501,'Reentered','1999-12-24 00:00:00','1993-08-11 00:00:00',92.92,411,425,97.76,'Sun-Synch' from dual union all
select 16309,'016309','1979-017BY','P78-1 debris','',47501,'Reentered','1989-12-27 00:00:00','1985-12-11 00:00:00',95.42,510,568,97.65,'Sun-Synch' from dual union all
select 1631,'001631','1965-084C','deb Kosmos-93','',11834,'Reentered','1965-10-25 00:00:00','1965-10-21 00:00:00',91.25,203,469,48.4,'Intermediate' from dual union all
select 16310,'016310','1979-017BZ','P78-1 debris','',47501,'Reentered','1989-08-25 00:00:00','1985-12-11 00:00:00',95.15,521,531,97.73,'Sun-Synch' from dual union all
select 16311,'016311','1979-017CA','P78-1 debris','',47501,'Reentered','2000-11-07 00:00:00','1993-08-13 00:00:00',93.66,444,464,97.8,'Sun-Synch' from dual union all
select 16312,'016312','1979-017CB','P78-1 debris','',47501,'Reentered','1989-08-24 00:00:00','1985-12-11 00:00:00',94.88,497,528,97.92,'Sun-Synch' from dual union all
select 16313,'016313','1979-017CC','P78-1 debris','',47501,'Reentered','1989-12-21 00:00:00','1985-12-11 00:00:00',94.74,480,532,97.88,'Sun-Synch' from dual union all
select 16314,'016314','1979-017CD','P78-1 debris','',47501,'Reentered','1988-12-11 00:00:00','1985-12-12 00:00:00',95,507,531,97.91,'Sun-Synch' from dual union all
select 16315,'016315','1979-017CE','P78-1 debris','',47501,'Reentered','1986-10-03 00:00:00','1986-09-30 00:00:00',90.3,280,299,97.57,'Sun-Synch' from dual union all
select 16316,'016316','1979-017CF','P78-1 debris','',47501,'Reentered','1988-12-29 00:00:00','1985-12-11 00:00:00',97.73,503,798,98.62,'Sun-Synch' from dual union all
select 16317,'016317','1979-017CG','P78-1 debris','',47501,'Reentered','1989-03-13 00:00:00','1985-12-17 00:00:00',97,506,725,97.7,'Sun-Synch' from dual union all
select 16318,'016318','1979-017CH','P78-1 debris','',47501,'Reentered','1986-10-30 00:00:00','1985-09-21 00:00:00',98.84,518,889,98.32,'Sun-Synch' from dual union all
select 16319,'016319','1979-017CJ','P78-1 debris','',47501,'Reentered','1987-07-10 00:00:00','1985-09-20 00:00:00',95.06,390,653,96.84,'Sun-Synch' from dual union all
select 1632,'001632','1965-084D','deb Kosmos-93','',11834,'Reentered','1965-10-26 00:00:00','1965-10-21 00:00:00',91.07,191,464,48.7,'Intermediate' from dual union all
select 16320,'016320','1979-017CK','P78-1 debris','',47501,'Reentered','1979-04-24 00:00:00','1985-09-21 00:00:00',95.74,488,621,98.68,'Sun-Synch' from dual union all
select 16321,'016321','1979-017CL','P78-1 debris','',47501,'Reentered','1988-04-13 00:00:00','1985-09-24 00:00:00',95.19,524,532,98.35,'Sun-Synch' from dual union all
select 16322,'016322','1979-017CM','P78-1 debris','',47501,'Reentered','1987-12-17 00:00:00','1985-12-11 00:00:00',95.62,518,580,97.7,'Sun-Synch' from dual union all
select 16323,'016323','1979-017CN','P78-1 debris','',47501,'Reentered','1988-01-03 00:00:00','1985-12-11 00:00:00',95.04,507,534,97.74,'Sun-Synch' from dual union all
select 16324,'016324','1979-017CP','P78-1 debris','',47501,'Reentered','1987-03-14 00:00:00','1985-12-13 00:00:00',94.83,458,562,96.96,'Sun-Synch' from dual union all
select 16325,'016325','1979-017CQ','P78-1 debris','',47501,'Reentered','1986-11-21 00:00:00','1985-12-13 00:00:00',94.13,444,508,97.09,'Sun-Synch' from dual union all
select 16326,'016326','1985-113A','Kosmos-1707','Tselina-D',58336,'In Earth orbit','','1986-01-11 00:00:00',97.7,633,665,82.54,'Intermediate' from dual union all
select 16327,'016327','1985-113B','S5M','S5M',58336,'In Earth orbit','','1991-03-27 00:00:00',97.5,623,655,82.54,'Intermediate' from dual union all
select 16328,'016328','1985-114A','ITV 1 Canister','ITV 1',58337,'Reentered','1989-05-11 00:00:00','1986-01-13 00:00:00',95.47,312,771,37.07,'Intermediate' from dual union all
select 16329,'016329','1985-114B','ITV 2 Balloon','ITV 2',58337,'Reentered','1987-08-09 00:00:00','1987-01-21 00:00:00',94.87,310,715,37.07,'Intermediate' from dual union all
select 1633,'001633','1965-020EC','deb Blok Kosmos','',11770,'Reentered','1981-01-31 00:00:00','1965-03-15 00:00:00',113.46,606,2165,56.4,'Intermediate' from dual union all
select 16330,'016330','1985-114C','Star 20 E-32','Star 20 E-32',58337,'Reentered','1988-04-11 00:00:00','1986-01-14 00:00:00',95.44,313,767,37.08,'Intermediate' from dual union all
select 16331,'016331','1985-115A','Kosmos-1708','Resurs-F1 17F41 No. 56',58338,'Landed','1985-12-27 00:00:00','1985-12-15 00:00:00',89.77,255,272,82.28,'Intermediate' from dual union all
select 16332,'016332','1985-115B','11S510','11S510',58338,'Reentered','1985-12-17 00:00:00','1985-12-15 00:00:00',88.75,176,250,82.27,'Intermediate' from dual union all
select 16333,'016333','1979-017CR','P78-1 debris','',47501,'Reentered','1988-11-14 00:00:00','1985-12-11 00:00:00',94.92,495,535,97.71,'Sun-Synch' from dual union all
select 16334,'016334','1979-017CS','P78-1 debris','',47501,'Reentered','1987-01-06 00:00:00','1985-12-13 00:00:00',94.84,500,522,97.07,'Sun-Synch' from dual union all
select 16335,'016335','1979-017CT','P78-1 debris','',47501,'Reentered','1986-09-22 00:00:00','1985-12-10 00:00:00',94.08,388,559,97.24,'Sun-Synch' from dual union all
select 16336,'016336','1979-017CU','P78-1 debris','',47501,'Reentered','1989-01-19 00:00:00','1985-12-13 00:00:00',94.87,497,527,98.06,'Sun-Synch' from dual union all
select 16337,'016337','1979-017CV','P78-1 debris','',47501,'Reentered','1986-06-08 00:00:00','1985-12-13 00:00:00',94,390,550,98.04,'Sun-Synch' from dual union all
select 16338,'016338','1979-017CW','P78-1 debris','',47501,'Reentered','1986-03-03 00:00:00','1985-12-11 00:00:00',94.81,498,521,97.65,'Sun-Synch' from dual union all
select 16339,'016339','1985-107F','Blok DM','11S86 No. 56L',58330,'In Earth orbit','','1986-07-19 00:00:00',1476.92,36495,36669,.72,'Drift GEO' from dual union all
select 1634,'001634','1965-020ED','deb Blok Kosmos','',11770,'In Earth orbit','','1966-01-13 00:00:00',116.3,810,2219,56.2,'Intermediate' from dual union all
select 16340,'016340','1985-115C','MKA Subsatellite','',58338,'Reentered','1985-12-22 00:00:00','1985-12-16 00:00:00',89.61,250,260,82.26,'Intermediate' from dual union all
select 16341,'016341','1979-017CX','P78-1 debris','',47501,'Reentered','1988-09-30 00:00:00','1985-12-16 00:00:00',95.31,514,554,97.85,'Sun-Synch' from dual union all
select 16342,'016342','1979-017CY','P78-1 debris','',47501,'Reentered','1986-03-25 00:00:00','1985-12-16 00:00:00',93.76,380,536,97.86,'Sun-Synch' from dual union all
select 16343,'016343','1979-017CZ','P78-1 debris','',47501,'Reentered','1991-09-10 00:00:00','1991-09-10 00:00:00',87.54,147,159,97.86,'Sun-Synch' from dual union all
select 16344,'016344','1979-017DA','P78-1 debris','',47501,'Reentered','1987-05-01 00:00:00','1987-05-01 00:00:00',89.22,221,250,97.85,'Sun-Synch' from dual union all
select 16345,'016345','1979-017DB','P78-1 debris','',47501,'Reentered','1987-07-08 00:00:00','1987-07-07 00:00:00',89.15,222,243,97.63,'Sun-Synch' from dual union all
select 16346,'016346','1979-017DC','P78-1 debris','',47501,'Reentered','1988-05-06 00:00:00','1985-12-17 00:00:00',94.92,483,547,97.69,'Sun-Synch' from dual union all
select 16347,'016347','1979-017DD','P78-1 debris','',47501,'Reentered','1988-04-06 00:00:00','1985-12-16 00:00:00',94.83,480,540,97.68,'Sun-Synch' from dual union all
select 16348,'016348','1979-017DE','P78-1 debris','',47501,'Reentered','1988-11-23 00:00:00','1996-05-30 00:00:00',104.8,959,1011,82.94,'Intermediate' from dual union all
select 16349,'016349','1979-017DF','P78-1 debris','',47501,'Reentered','1987-07-10 00:00:00','1985-09-22 00:00:00',95.55,517,574,97.71,'Sun-Synch' from dual union all
select 1635,'001635','1965-020EE','deb Blok Kosmos','',11770,'Reentered','1967-10-20 00:00:00','1966-01-28 00:00:00',106.71,437,1712,56.23,'Intermediate' from dual union all
select 16350,'016350','1979-017DG','P78-1 debris','',47501,'Reentered','1986-07-26 00:00:00','1985-12-17 00:00:00',93.12,349,505,97.84,'Sun-Synch' from dual union all
select 16351,'016351','1979-017DH','P78-1 debris','',47501,'Reentered','1986-05-08 00:00:00','1985-12-17 00:00:00',93.21,341,523,97.75,'Sun-Synch' from dual union all
select 16352,'016352','1979-017DJ','P78-1 debris','',47501,'Reentered','1986-05-12 00:00:00','1985-12-14 00:00:00',96.08,513,629,97.64,'Sun-Synch' from dual union all
select 16353,'016353','1985-111D','Blenda','Sunshade',58334,'Reentered','1986-01-08 00:00:00','1985-12-18 00:00:00',92.25,353,417,72.87,'Intermediate' from dual union all
select 16354,'016354','1985-111E','KDU part','KDU part',58334,'Reentered','1986-01-10 00:00:00','1985-12-20 00:00:00',91.99,344,400,73,'Intermediate' from dual union all
select 16355,'016355','1985-111F','KDU part','KDU part',58334,'Reentered','1986-03-07 00:00:00','1986-01-02 00:00:00',95.91,388,738,72.87,'Intermediate' from dual union all
select 16356,'016356','1985-111G','KDU part','KDU part',58334,'Reentered','1986-04-04 00:00:00','1986-01-02 00:00:00',92.18,353,410,72.87,'Intermediate' from dual union all
select 16357,'016357','1985-111H','KDU part','KDU part',58334,'Reentered','1986-10-13 00:00:00','1986-01-02 00:00:00',94.31,388,583,72.87,'Intermediate' from dual union all
select 16358,'016358','1985-111J','KDU part','KDU part',58334,'Reentered','1985-12-22 00:00:00','1985-12-17 00:00:00',92.43,363,425,72.81,'Intermediate' from dual union all
select 16359,'016359','1985-111K','KDU part','KDU part',58334,'Reentered','1986-01-12 00:00:00','1985-12-18 00:00:00',92,339,406,72.91,'Intermediate' from dual union all
select 1636,'001636','1965-085A','Kosmos-94','Zenit-4 No. 15',11835,'Landed','1965-11-05 00:00:00','1965-10-28 00:00:00',89.26,205,271,65,'Intermediate' from dual union all
select 16360,'016360','1979-017DK','P78-1 debris','',47501,'Reentered','1987-10-10 00:00:00','1985-12-19 00:00:00',94.9,509,519,97.65,'Sun-Synch' from dual union all
select 16361,'016361','1979-017DL','P78-1 debris','',47501,'Reentered','1986-11-15 00:00:00','1985-12-26 00:00:00',94.99,499,538,97.79,'Sun-Synch' from dual union all
select 16362,'016362','1979-017DM','P78-1 debris','',47501,'Reentered','1986-01-25 00:00:00','1986-01-24 00:00:00',88.52,189,214,97.99,'Sun-Synch' from dual union all
select 16363,'016363','1979-017DN','P78-1 debris','',47501,'Reentered','1987-08-08 00:00:00','1985-12-19 00:00:00',94.93,501,530,97.75,'Sun-Synch' from dual union all
select 16364,'016364','1979-017DP','P78-1 debris','',47501,'Reentered','1988-10-03 00:00:00','1985-12-16 00:00:00',94.88,489,537,97.66,'Sun-Synch' from dual union all
select 16365,'016365','1979-017DQ','P78-1 debris','',47501,'Reentered','1986-03-20 00:00:00','1985-12-18 00:00:00',93.49,376,515,97.83,'Sun-Synch' from dual union all
select 16366,'016366','1979-017DR','P78-1 debris','',47501,'Reentered','1987-09-02 00:00:00','1985-12-19 00:00:00',94.66,479,525,98.13,'Sun-Synch' from dual union all
select 16367,'016367','1979-017DS','P78-1 debris','',47501,'Reentered','1987-10-12 00:00:00','1985-12-16 00:00:00',94.13,411,542,97.72,'Sun-Synch' from dual union all
select 16368,'016368','1985-116A','Kosmos-1709','Parus',58339,'In Earth orbit','','1986-01-19 00:00:00',104.85,962,1013,82.94,'Intermediate' from dual union all
select 16369,'016369','1985-116B','S3M','S3M',58339,'In Earth orbit','','1990-03-30 00:00:00',104.68,955,1004,82.94,'Intermediate' from dual union all
select 1637,'001637','1965-086A','OPS 2155','CORONA J-25',11836,'Reentered Attached','1965-11-17 00:00:00','1965-10-31 00:00:00',90.57,173,431,74.97,'Intermediate' from dual union all
select 16370,'016370','1985-111L','KDU part','KDU part',58334,'Reentered','1986-07-02 00:00:00','1986-01-02 00:00:00',92.04,343,406,72.87,'Intermediate' from dual union all
select 16371,'016371','1985-111M','KDU part','KDU part',58334,'Reentered','1986-01-24 00:00:00','1985-12-19 00:00:00',92.19,354,409,72.86,'Intermediate' from dual union all
select 16372,'016372','1985-111N','KDU part','KDU part',58334,'Reentered','1986-01-14 00:00:00','1985-12-19 00:00:00',92.2,355,409,72.87,'Intermediate' from dual union all
select 16373,'016373','1985-111P','KDU','KDU',58334,'Reentered','1988-02-15 00:00:00','1986-01-03 00:00:00',92.23,355,413,72.87,'Intermediate' from dual union all
select 16374,'016374','1985-111Q','KDU part','KDU part',58334,'Reentered','1986-02-22 00:00:00','1986-01-02 00:00:00',93.09,387,464,71.98,'Intermediate' from dual union all
select 16375,'016375','1982-055P','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-07 00:00:00',104.42,945,990,65.84,'Intermediate' from dual union all
select 16376,'016376','1982-055Q','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-06 00:00:00',104.1,932,973,65.83,'Intermediate' from dual union all
select 16377,'016377','1982-055R','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-07 00:00:00',103.92,910,977,65.83,'Intermediate' from dual union all
select 16378,'016378','1982-055S','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-02 00:00:00',104.43,943,993,65.83,'Intermediate' from dual union all
select 16379,'016379','1982-055T','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-07 00:00:00',104.53,952,992,65.84,'Intermediate' from dual union all
select 1638,'001638','1965-085B','11A57I','11A57I',11835,'Reentered','1965-11-03 00:00:00','1965-10-29 00:00:00',89.07,202,255,64.97,'Intermediate' from dual union all
select 16380,'016380','1982-055U','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-07 00:00:00',105.61,987,1059,65.84,'Intermediate' from dual union all
select 16381,'016381','1982-055V','deb Kosmos-1375','',54364,'In Earth orbit','','1993-07-30 00:00:00',105.63,975,1073,65.84,'Intermediate' from dual union all
select 16382,'016382','1982-055W','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-06 00:00:00',103.21,876,945,65.83,'Intermediate' from dual union all
select 16383,'016383','1982-055X','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-07 00:00:00',105.23,972,1038,65.84,'Intermediate' from dual union all
select 16384,'016384','1982-055Y','deb Kosmos-1375','',54364,'In Earth orbit','','1993-07-29 00:00:00',105.34,983,1038,65.84,'Intermediate' from dual union all
select 16385,'016385','1982-055Z','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-01 00:00:00',104.8,971,999,65.83,'Intermediate' from dual union all
select 16386,'016386','1982-055AA','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-01 00:00:00',105.16,977,1027,65.84,'Intermediate' from dual union all
select 16387,'016387','1982-055AB','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-04 00:00:00',104.03,905,993,65.84,'Intermediate' from dual union all
select 16388,'016388','1985-073D','M-3B adapter','',58296,'Reentered','1986-05-22 00:00:00','1985-09-18 00:00:00',100.5,212,1352,31.24,'Intermediate' from dual;
insert into SATELLITE
select 16389,'016389','1985-097C','Sep Motor Cover','Sep Motor Cover',58320,'In Earth orbit','','1993-08-16 00:00:00',104.77,842,1126,71.01,'Intermediate' from dual union all
select 1639,'001639','1965-086B','LPARL experiment cover','',11836,'Reentered','1965-11-01 00:00:00','1965-10-28 00:00:00',92.2,163,602,74.9,'Intermediate' from dual union all
select 16390,'016390','1985-097D','Sep Motor Cover','Sep Motor Cover',58320,'In Earth orbit','','1993-08-16 00:00:00',105.08,843,1154,70.97,'Intermediate' from dual union all
select 16391,'016391','1985-097E','Sep Motor Cover','Sep Motor Cover',58320,'In Earth orbit','','1986-01-17 00:00:00',104.86,842,1133,71.01,'Intermediate' from dual union all
select 16392,'016392','1985-097F','Sep Motor Cover','Sep Motor Cover',58320,'In Earth orbit','','1985-11-23 00:00:00',105.03,844,1148,70.98,'Intermediate' from dual union all
select 16393,'016393','1985-117A','Molniya-3','Molniya-3 No. 40',58340,'In Earth orbit','','1986-03-01 00:00:00',717.62,537,39810,62.85,'Molniya' from dual union all
select 16394,'016394','1985-117B','11S510','11S510',58340,'Reentered','1986-01-30 00:00:00','1985-12-31 00:00:00',90.84,203,429,62.86,'Intermediate' from dual union all
select 16395,'016395','1985-117C','BOZ','BOZ',58340,'Reentered','1986-01-21 00:00:00','1985-12-31 00:00:00',90.75,190,433,62.85,'Intermediate' from dual union all
select 16396,'016396','1985-118A','Kosmos-1710','Uragan No. 22L',58341,'In Earth orbit','','1986-03-02 00:00:00',675.71,19109,19150,64.84,'Medium' from dual union all
select 16397,'016397','1985-118B','Kosmos-1711','Uragan No. 23L',58341,'In Earth orbit','','1986-03-02 00:00:00',675.69,19103,19155,64.85,'Medium' from dual union all
select 16398,'016398','1985-118C','Kosmos-1712','Uragan GVM',58341,'In Earth orbit','','1986-03-23 00:00:00',676.29,19139,19149,64.82,'Medium' from dual union all
select 16399,'016399','1985-118D','8S812','8S812',58341,'Reentered','1985-12-27 00:00:00','1985-12-26 00:00:00',90.34,152,431,64.79,'Intermediate' from dual union all
select 164,'000164','61 Sigma 2','Midas  Fairing','Agena Fairing',7278,'Reentered','1961-07-25 00:00:00','1961-07-15 00:00:00',117.23,129,2984,90.8,'Polar' from dual union all
select 1640,'001640','1965-082B','LCS 2','LCS 2',11832,'Reentered','1982-08-25 00:00:00','1965-12-27 00:00:00',98.97,658,761,32.17,'Intermediate' from dual union all
select 16400,'016400','1985-117D','deb Blok-I','',58340,'Reentered','1986-01-16 00:00:00','1985-12-31 00:00:00',91.74,206,513,62.85,'Intermediate' from dual union all
select 16401,'016401','1985-117E','deb Blok-I','',58340,'Reentered','1986-01-20 00:00:00','1986-01-06 00:00:00',91.07,235,420,62.82,'Intermediate' from dual union all
select 16402,'016402','1985-117F','Blok-ML','Blok-ML No. 77346-528',58340,'In Earth orbit','','1986-02-23 00:00:00',732.63,526,40559,62.89,'Molniya' from dual union all
select 16403,'016403','1985-118E','Perekhodnik','Sredniy perekhodnik',58341,'Reentered','1985-12-25 00:00:00','1985-12-25 00:00:00',87.39,130,161,64.83,'Intermediate' from dual union all
select 16404,'016404','1985-118F','Blok DM2','11S861 No. 11L',58341,'In Earth orbit','','1986-02-23 00:00:00',675.51,19119,19130,64.85,'Medium' from dual union all
select 16405,'016405','1985-118G','deb Proton','',58341,'Reentered','1985-12-27 00:00:00','1985-12-26 00:00:00',89.77,161,366,64.64,'Intermediate' from dual union all
select 16406,'016406','1985-118H','deb Proton','',58341,'Reentered','1985-12-27 00:00:00','1985-12-26 00:00:00',90.52,160,439,64.92,'Intermediate' from dual union all
select 16407,'016407','1985-118J','deb Proton','',58341,'Reentered','1985-12-28 00:00:00','1985-12-26 00:00:00',90.54,157,444,64.81,'Intermediate' from dual union all
select 16408,'016408','1985-119A','Meteor-2','Meteor-2 No. 14',58342,'In Earth orbit','','1986-01-25 00:00:00',104.05,938,962,82.54,'Intermediate' from dual union all
select 16409,'016409','1985-119B','S5M','S5M',58342,'In Earth orbit','','1986-01-25 00:00:00',104.02,939,958,82.54,'Intermediate' from dual union all
select 1641,'001641','1965-082C','OV2-1','OV2-1',11832,'In Earth orbit','','1966-01-17 00:00:00',99.82,734,766,32.27,'Intermediate' from dual union all
select 16410,'016410','1985-117G','deb Blok-I','',58340,'Reentered','1986-01-19 00:00:00','1985-12-31 00:00:00',90.88,202,434,62.84,'Intermediate' from dual union all
select 16411,'016411','1985-117H','deb Blok-I','',58340,'Reentered','1986-01-17 00:00:00','1985-12-31 00:00:00',90.82,200,430,62.85,'Intermediate' from dual union all
select 16412,'016412','1985-117J','deb Blok-I','',58340,'Reentered','1986-01-22 00:00:00','1985-12-31 00:00:00',90.96,205,439,62.83,'Intermediate' from dual union all
select 16413,'016413','1979-017DT','P78-1 debris','',47501,'Reentered','1989-08-11 00:00:00','1985-09-16 00:00:00',95.16,508,545,97.73,'Sun-Synch' from dual union all
select 16414,'016414','1979-017DU','P78-1 debris','',47501,'Reentered','1988-10-14 00:00:00','1985-12-20 00:00:00',95.32,516,553,97.94,'Sun-Synch' from dual union all
select 16415,'016415','1979-017DV','P78-1 debris','',47501,'Reentered','1986-03-01 00:00:00','1985-12-27 00:00:00',93.2,365,498,97.85,'Sun-Synch' from dual union all
select 16416,'016416','1979-017DW','P78-1 debris','',47501,'Reentered','1987-12-11 00:00:00','1985-12-26 00:00:00',95.01,506,533,97.2,'Sun-Synch' from dual union all
select 16417,'016417','1979-017DX','P78-1 debris','',47501,'Reentered','1987-05-30 00:00:00','1985-12-26 00:00:00',94.96,502,532,97.74,'Sun-Synch' from dual union all
select 16418,'016418','1979-017DY','P78-1 debris','',47501,'Reentered','1989-01-23 00:00:00','1985-12-24 00:00:00',95.11,510,537,97.73,'Sun-Synch' from dual union all
select 16419,'016419','1979-017DZ','P78-1 debris','',47501,'Reentered','1986-01-01 00:00:00','1985-12-21 00:00:00',92.64,334,474,97.7,'Sun-Synch' from dual union all
select 1642,'001642','1965-082D','Transtage 4 large debris','RCS 2.4',11832,'In Earth orbit','','1966-01-15 00:00:00',99.01,686,737,32.07,'Intermediate' from dual union all
select 16420,'016420','1979-017EA','P78-1 debris','',47501,'Reentered','1986-09-02 00:00:00','1985-12-26 00:00:00',93.13,335,521,97.92,'Sun-Synch' from dual union all
select 16421,'016421','1979-017EB','P78-1 debris','',47501,'Reentered','1986-08-26 00:00:00','1985-12-25 00:00:00',94.44,474,508,97.63,'Sun-Synch' from dual union all
select 16422,'016422','1979-017EC','P78-1 debris','',47501,'Reentered','1987-08-08 00:00:00','1985-12-25 00:00:00',95.08,506,539,97.67,'Sun-Synch' from dual union all
select 16423,'016423','1979-017ED','P78-1 debris','',47501,'Reentered','1986-06-19 00:00:00','1985-12-24 00:00:00',93.1,352,501,98,'Sun-Synch' from dual union all
select 16424,'016424','1979-017EE','P78-1 debris','',47501,'Reentered','1986-04-17 00:00:00','1985-12-23 00:00:00',93.49,347,544,97.99,'Sun-Synch' from dual union all
select 16425,'016425','1979-017EF','P78-1 debris','',47501,'Reentered','1986-09-27 00:00:00','1985-12-24 00:00:00',94.74,493,519,97.73,'Sun-Synch' from dual union all
select 16426,'016426','1985-115D','Blenda','Sunshade',58338,'Reentered','1985-12-30 00:00:00','1985-12-27 00:00:00',90.33,251,331,82.31,'Intermediate' from dual union all
select 16427,'016427','1985-115E','KDU','KDU',58338,'Reentered','1986-01-29 00:00:00','1985-12-27 00:00:00',89.72,253,269,82.27,'Intermediate' from dual union all
select 16428,'016428','1985-115F','KDU part','KDU part',58338,'Reentered','1985-12-28 00:00:00','1985-12-27 00:00:00',89.64,246,267,82.28,'Intermediate' from dual union all
select 16429,'016429','1985-120A','Kosmos-1713','Efir No. 2',58343,'Landed','1986-01-22 00:00:00','1986-01-03 00:00:00',90.64,216,395,62.82,'Intermediate' from dual union all
select 1643,'001643','1965-082E','deb Transtage 4','',11832,'Reentered','1989-12-25 00:00:00','1966-01-16 00:00:00',100.1,723,804,32.44,'Intermediate' from dual union all
select 16430,'016430','1985-120B','11S510','11S510',58343,'Reentered','1986-01-27 00:00:00','1986-01-03 00:00:00',90.31,211,369,62.8,'Intermediate' from dual union all
select 16431,'016431','1985-115G','KDU part','KDU part',58338,'Reentered','1986-01-04 00:00:00','1985-12-27 00:00:00',92.12,258,499,82.31,'Intermediate' from dual union all
select 16432,'016432','1985-115H','KDU part','KDU part',58338,'Reentered','1986-01-15 00:00:00','1985-12-27 00:00:00',91.8,259,466,82.27,'Intermediate' from dual union all
select 16433,'016433','1985-115J','KDU part','KDU part',58338,'Reentered','1986-01-07 00:00:00','1985-12-28 00:00:00',89.6,244,266,82.27,'Intermediate' from dual union all
select 16434,'016434','1985-121A','Kosmos-1714','Tselina-2 No. 3L',58344,'Reentered Attached','1986-02-27 00:00:00','1986-01-28 00:00:00',93.16,160,699,70.98,'Intermediate' from dual union all
select 16435,'016435','1985-121B','deb Kosmos-1714','',58344,'Reentered','1986-01-11 00:00:00','1985-12-30 00:00:00',94.33,164,809,70.99,'Intermediate' from dual union all
select 16436,'016436','1985-121C','deb Kosmos-1714','',58344,'Reentered','1986-01-08 00:00:00','1985-12-30 00:00:00',94.1,165,784,71,'Intermediate' from dual union all
select 16437,'016437','1985-121D','Sep Motor Cover','Sep Motor Cover',58344,'Reentered','1990-07-08 00:00:00','1986-01-29 00:00:00',97.66,443,851,70.98,'Intermediate' from dual union all
select 16438,'016438','1985-121E','Sep Motor Cover','Sep Motor Cover',58344,'Reentered','1990-04-22 00:00:00','1986-01-28 00:00:00',97.43,424,848,70.97,'Intermediate' from dual union all
select 16439,'016439','1985-121F','Sep Motor Cover','Sep Motor Cover',58344,'Reentered','1990-04-04 00:00:00','1986-01-29 00:00:00',97.41,421,849,71.01,'Intermediate' from dual union all
select 1644,'001644','1965-082F','deb Transtage 4','',11832,'Reentered','1980-08-14 00:00:00','1966-01-26 00:00:00',99.98,737,779,32.51,'Intermediate' from dual union all
select 16440,'016440','1985-121G','Sep Motor Cover','Sep Motor Cover',58344,'Reentered','1989-11-16 00:00:00','1986-01-27 00:00:00',97.31,411,849,71.01,'Intermediate' from dual union all
select 16441,'016441','1982-033GS','Garbage bag','',54342,'Reentered','1986-03-10 00:00:00','1986-01-05 00:00:00',91.54,320,381,51.56,'Intermediate' from dual union all
select 16442,'016442','1983-022D','NOAA 8 battery debris','NOAA E battery',55779,'Reentered','1986-02-09 00:00:00','1986-01-16 00:00:00',99.66,503,982,98.71,'Sun-Synch' from dual union all
select 16443,'016443','1983-022E','NOAA 8 battery debris','NOAA E battery',55779,'Reentered','1986-01-16 00:00:00','1986-01-06 00:00:00',100.62,564,1012,98.73,'Sun-Synch' from dual union all
select 16444,'016444','1983-022F','NOAA 8 battery debris','NOAA E battery',55779,'Reentered','1986-01-20 00:00:00','1986-01-06 00:00:00',101.39,674,975,98.59,'Sun-Synch' from dual union all
select 16445,'016445','1985-118K','SOZ','SOZ',58341,'In Earth orbit','','1986-02-23 00:00:00',340.45,422,19123,64.92,'Highly Ellip' from dual union all
select 16446,'016446','1985-118L','SOZ','SOZ',58341,'In Earth orbit','','1986-02-28 00:00:00',340.45,426,19120,64.81,'Highly Ellip' from dual union all
select 16447,'016447','1986-001A','Kosmos-1715','Oblik No. 323',59194,'Landed','1986-01-22 00:00:00','1986-01-11 00:00:00',89.58,226,282,72.81,'Intermediate' from dual union all
select 16448,'016448','1986-001B','11S510','11S510',59194,'Reentered','1986-01-16 00:00:00','1986-01-10 00:00:00',89.02,189,264,72.81,'Intermediate' from dual union all
select 16449,'016449','1986-002A','Kosmos-1716','Strela-1M',59195,'In Earth orbit','','1986-02-13 00:00:00',115.48,1464,1491,73.99,'Intermediate' from dual union all
select 1645,'001645','1965-082G','deb Transtage 4','',11832,'Reentered','1979-05-04 00:00:00','1966-01-22 00:00:00',99.26,671,775,32.56,'Intermediate' from dual union all
select 16450,'016450','1986-002B','Kosmos-1717','Strela-1M',59195,'In Earth orbit','','1986-02-12 00:00:00',115.86,1475,1514,73.99,'Intermediate' from dual union all
select 16451,'016451','1986-002C','Kosmos-1718','Strela-1M',59195,'In Earth orbit','','1986-02-12 00:00:00',115.66,1475,1496,73.99,'Intermediate' from dual union all
select 16452,'016452','1986-002D','Kosmos-1719','Strela-1M',59195,'In Earth orbit','','1986-02-12 00:00:00',115.32,1456,1484,73.99,'Intermediate' from dual union all
select 16453,'016453','1986-002E','Kosmos-1720','Strela-1M',59195,'In Earth orbit','','1986-02-12 00:00:00',115.15,1441,1484,73.99,'Intermediate' from dual union all
select 16454,'016454','1986-002F','Kosmos-1721','Strela-1M',59195,'In Earth orbit','','1986-02-12 00:00:00',114.99,1427,1483,73.99,'Intermediate' from dual union all
select 16455,'016455','1986-002G','Kosmos-1722','Strela-1M',59195,'In Earth orbit','','1986-02-12 00:00:00',114.83,1413,1482,73.99,'Intermediate' from dual union all
select 16456,'016456','1986-002H','Kosmos-1723','Strela-1M',59195,'In Earth orbit','','1986-02-12 00:00:00',114.67,1400,1481,73.99,'Intermediate' from dual union all
select 16457,'016457','1986-002J','S3M','S3M',59195,'In Earth orbit','','1986-02-09 00:00:00',117.94,1480,1697,73.99,'Intermediate' from dual union all
select 16458,'016458','1979-017EG','P78-1 debris','',47501,'Reentered','1988-06-12 00:00:00','1985-09-22 00:00:00',95.57,515,578,97.68,'Sun-Synch' from dual union all
select 16459,'016459','1979-017EH','P78-1 debris','',47501,'Reentered','1986-10-17 00:00:00','1985-09-22 00:00:00',94.76,482,531,97.84,'Sun-Synch' from dual union all
select 1646,'001646','1965-082H','Transtage 4 large debris','RCS 2.1',11832,'In Earth orbit','','1965-11-19 00:00:00',99.62,734,747,32.29,'Intermediate' from dual union all
select 16460,'016460','1979-017EJ','P78-1 debris','',47501,'Reentered','1989-05-19 00:00:00','1985-12-17 00:00:00',95.3,516,551,97.89,'Sun-Synch' from dual union all
select 16461,'016461','1979-017EK','P78-1 debris','',47501,'Reentered','1986-04-21 00:00:00','1985-11-29 00:00:00',95.37,505,568,97.62,'Sun-Synch' from dual union all
select 16462,'016462','1979-017EL','P78-1 debris','',47501,'Reentered','1986-01-28 00:00:00','1985-12-17 00:00:00',92.66,335,475,97.97,'Sun-Synch' from dual union all
select 16463,'016463','1979-017EM','P78-1 debris','',47501,'Reentered','1986-03-06 00:00:00','1985-09-21 00:00:00',94.41,419,561,97.75,'Sun-Synch' from dual union all
select 16464,'016464','1979-017EN','P78-1 debris','',47501,'Reentered','1986-01-22 00:00:00','1985-11-04 00:00:00',92.22,260,507,97.84,'Sun-Synch' from dual union all
select 16465,'016465','1979-017EP','P78-1 debris','',47501,'Reentered','1986-03-06 00:00:00','1986-01-15 00:00:00',92.97,358,482,97.84,'Sun-Synch' from dual union all
select 16466,'016466','1979-017EQ','P78-1 debris','',47501,'Reentered','1988-10-28 00:00:00','1986-01-17 00:00:00',94.96,498,536,97.91,'Sun-Synch' from dual union all
select 16467,'016467','1979-017ER','P78-1 debris','',47501,'Reentered','1989-01-30 00:00:00','1986-01-11 00:00:00',94.5,433,556,97.8,'Sun-Synch' from dual union all
select 16468,'016468','1979-017ES','P78-1 debris','',47501,'Reentered','1988-07-02 00:00:00','1986-01-25 00:00:00',94.48,441,546,97.71,'Sun-Synch' from dual union all
select 16469,'016469','1979-017ET','P78-1 debris','',47501,'Reentered','1986-02-04 00:00:00','1986-01-08 00:00:00',92.64,350,458,97.92,'Sun-Synch' from dual union all
select 1647,'001647','1965-082J','deb Transtage 4','',11832,'Reentered','1989-11-17 00:00:00','1966-01-20 00:00:00',101,724,887,32.12,'Intermediate' from dual union all
select 16470,'016470','1979-017EU','P78-1 debris','',47501,'Reentered','1988-10-29 00:00:00','1986-01-17 00:00:00',95.28,499,566,97.73,'Sun-Synch' from dual union all
select 16471,'016471','1979-017EV','P78-1 debris','',47501,'Reentered','1987-10-22 00:00:00','1985-12-10 00:00:00',96.55,496,691,98.47,'Sun-Synch' from dual union all
select 16472,'016472','1979-017EW','P78-1 debris','',47501,'Reentered','1989-08-02 00:00:00','1986-01-18 00:00:00',95.25,511,550,97.8,'Sun-Synch' from dual union all
select 16473,'016473','1979-017EX','P78-1 debris','',47501,'Reentered','1986-05-19 00:00:00','1986-05-04 00:00:00',93.73,404,509,97.74,'Sun-Synch' from dual union all
select 16474,'016474','1979-017EY','P78-1 debris','',47501,'Reentered','1988-07-09 00:00:00','1986-01-10 00:00:00',95.47,516,567,98.05,'Sun-Synch' from dual union all
select 16475,'016475','1979-017EZ','P78-1 debris','',47501,'Reentered','1987-10-14 00:00:00','1985-12-17 00:00:00',94.97,493,542,97.67,'Sun-Synch' from dual union all
select 16476,'016476','1979-017FA','P78-1 debris','',47501,'Reentered','1988-01-24 00:00:00','1986-01-15 00:00:00',95.95,510,619,97.67,'Sun-Synch' from dual union all
select 16477,'016477','1979-017FB','P78-1 debris','',47501,'Reentered','1986-04-29 00:00:00','1986-01-10 00:00:00',93.87,381,546,98,'Sun-Synch' from dual union all
select 16478,'016478','1979-017FC','P78-1 debris','',47501,'Reentered','1989-11-06 00:00:00','1985-12-15 00:00:00',97.67,523,772,97.56,'Sun-Synch' from dual;
insert into SATELLITE
select 16479,'016479','1979-017FD','P78-1 debris','',47501,'Reentered','1986-04-13 00:00:00','1986-01-11 00:00:00',93.78,395,524,97.82,'Sun-Synch' from dual union all
select 1648,'001648','1965-082K','deb Transtage 4','',11832,'Reentered','1968-03-02 00:00:00','1966-01-26 00:00:00',98.62,632,754,32.16,'Intermediate' from dual union all
select 16480,'016480','1979-017FE','P78-1 debris','',47501,'Reentered','2000-12-17 00:00:00','1993-08-10 00:00:00',93.64,446,460,97.8,'Sun-Synch' from dual union all
select 16481,'016481','1986-003A','Columbia (STS 61-C)','OV-102',59196,'Landed','1986-01-18 00:00:00','1986-01-14 00:00:00',91.22,323,345,28.47,'Intermediate' from dual union all
select 16482,'016482','1986-003B','Satcom K1','Satcom K1',59196,'In Earth orbit','','1986-03-16 00:00:00',1436.14,35781,35794,.02,'Stationary' from dual union all
select 16483,'016483','1986-003C','PAM-D2','Star 63D',59196,'In Earth orbit','','1986-08-14 00:00:00',631.45,351,35655,27.31,'GEO Transfer' from dual union all
select 16484,'016484','1979-017FF','P78-1 debris','',47501,'Reentered','1987-03-22 00:00:00','1985-12-17 00:00:00',95.05,508,533,97.58,'Sun-Synch' from dual union all
select 16485,'016485','1979-017FG','P78-1 debris','',47501,'Reentered','1989-10-27 00:00:00','1985-12-17 00:00:00',95.56,523,569,97.47,'Sun-Synch' from dual union all
select 16486,'016486','1979-017FH','P78-1 debris','',47501,'Reentered','1988-01-21 00:00:00','1986-01-17 00:00:00',95.76,515,596,97.71,'Sun-Synch' from dual union all
select 16487,'016487','1979-017FJ','P78-1 debris','',47501,'Reentered','1986-12-11 00:00:00','1985-12-17 00:00:00',95.74,505,604,97.69,'Sun-Synch' from dual union all
select 16488,'016488','1979-017FK','P78-1 debris','',47501,'Reentered','1989-02-01 00:00:00','1986-01-18 00:00:00',95.23,516,544,97.8,'Sun-Synch' from dual union all
select 16489,'016489','1979-017FL','P78-1 debris','',47501,'Reentered','1989-04-17 00:00:00','1986-01-16 00:00:00',95.28,507,558,97.7,'Sun-Synch' from dual union all
select 1649,'001649','1965-082L','deb Transtage 4','',11832,'Reentered','1979-10-16 00:00:00','1965-12-25 00:00:00',99.06,659,769,33.22,'Intermediate' from dual union all
select 16490,'016490','1986-004A','Kosmos-1724','Kobal''t No. 824',59197,'Landed','1986-03-15 00:00:00','1986-01-22 00:00:00',89.14,162,302,67.15,'Intermediate' from dual union all
select 16491,'016491','1986-004B','11S510','11S510',59197,'Reentered','1986-01-21 00:00:00','1986-01-17 00:00:00',89.08,163,295,67.15,'Intermediate' from dual union all
select 16492,'016492','1986-004C','Kosmos-1724 part','Kobal''t part',59197,'Reentered','1986-01-16 00:00:00','1986-01-15 00:00:00',89.17,166,301,67.11,'Intermediate' from dual union all
select 16493,'016493','1986-005A','Kosmos-1725','Parus',59198,'In Earth orbit','','1986-02-18 00:00:00',104.84,971,1003,82.94,'Intermediate' from dual union all
select 16494,'016494','1986-005B','S3M','S3M',59198,'In Earth orbit','','1986-02-16 00:00:00',104.71,968,994,82.93,'Intermediate' from dual union all
select 16495,'016495','1986-006A','Kosmos-1726','Tselina-D',59199,'In Earth orbit','','1986-02-16 00:00:00',97.66,630,664,82.53,'Intermediate' from dual union all
select 16496,'016496','1986-006B','S5M','S5M',59199,'In Earth orbit','','1986-02-19 00:00:00',97.63,629,662,82.53,'Intermediate' from dual union all
select 16497,'016497','1986-007A','Raduga','Gran'' No. 29L',59200,'In Earth orbit','','1986-03-27 00:00:00',1436.24,35786,35793,1.13,'Stationary' from dual union all
select 16498,'016498','1986-007B','8S812','8S812',59200,'Reentered','1986-01-20 00:00:00','1986-01-19 00:00:00',88.11,179,183,51.61,'Intermediate' from dual union all
select 16499,'016499','1986-007C','Perekhodnik','Sredniy perekhodnik',59200,'Reentered','1986-01-18 00:00:00','1986-01-17 00:00:00',88.35,190,196,51.61,'Intermediate' from dual union all
select 165,'000165','61 Rho 2','Altair','Altair X-248-A5 S/N 3014',7277,'Reentered','2014-02-18 00:00:00','1961-08-14 00:00:00',100.35,734,817,47.9,'Intermediate' from dual union all
select 1650,'001650','1965-082M','deb Transtage 4','',11832,'Reentered','1979-10-18 00:00:00','1966-01-02 00:00:00',100.49,740,824,32.6,'Intermediate' from dual union all
select 16500,'016500','1986-007D','SOZ','SOZ',59200,'Reentered','1987-08-12 00:00:00','1986-03-22 00:00:00',648.99,304,36601,47.13,'GEO Transfer' from dual union all
select 16501,'016501','1986-007E','SOZ','SOZ',59200,'Reentered','1986-09-30 00:00:00','1986-01-25 00:00:00',648.03,302,36554,47.24,'GEO Transfer' from dual union all
select 16502,'016502','1983-022G','NOAA 8 battery debris','',55779,'Reentered','1988-02-21 00:00:00','1986-01-24 00:00:00',99.17,633,805,98.65,'Sun-Synch' from dual union all
select 16503,'016503','1983-022H','NOAA 8 battery debris','',55779,'Reentered','1989-02-19 00:00:00','1987-08-20 00:00:00',99.98,741,775,98.86,'Sun-Synch' from dual union all
select 16504,'016504','1983-022J','NOAA 8 battery debris','',55779,'Reentered','1986-11-13 00:00:00','1986-01-21 00:00:00',100.03,710,810,98.51,'Sun-Synch' from dual union all
select 16505,'016505','1986-001C','Blenda','Sunshade',59194,'Reentered','1986-01-25 00:00:00','1986-01-22 00:00:00',89.34,222,262,72.8,'Intermediate' from dual union all
select 16506,'016506','1986-001D','KDU part','KDU part',59194,'Reentered','1986-01-27 00:00:00','1986-01-23 00:00:00',89.03,218,236,72.81,'Intermediate' from dual union all
select 16507,'016507','1986-001E','KDU','KDU',59194,'Reentered','1986-01-31 00:00:00','1986-01-22 00:00:00',91.23,229,441,72.8,'Intermediate' from dual union all
select 16508,'016508','1985-120C','PTDU cover','Efir part',58343,'Reentered','1986-01-30 00:00:00','1986-01-22 00:00:00',92.49,219,575,62.79,'Intermediate' from dual union all
select 16509,'016509','1985-120D','PTDU cover','Efir part',58343,'Reentered','1986-02-03 00:00:00','1986-01-22 00:00:00',92.05,215,536,62.78,'Intermediate' from dual union all
select 1651,'001651','1965-082N','deb Transtage 4','',11832,'Reentered','1991-01-27 00:00:00','1966-01-19 00:00:00',102.68,725,1046,32.13,'Intermediate' from dual union all
select 16510,'016510','1986-008A','Kosmos-1727','Tsikada',59201,'In Earth orbit','','1986-02-26 00:00:00',104.86,960,1016,82.95,'Intermediate' from dual union all
select 16511,'016511','1986-008B','S3M','S3M',59201,'In Earth orbit','','1986-02-23 00:00:00',104.76,963,1004,82.94,'Intermediate' from dual union all
select 16512,'016512','1986-009A','Kosmos-1728','Oblik',59202,'Landed','1986-02-11 00:00:00','1986-01-30 00:00:00',89.47,225,272,69.97,'Intermediate' from dual union all
select 16513,'016513','1986-009B','11S510','11S510',59202,'Reentered','1986-02-06 00:00:00','1986-01-30 00:00:00',89.1,201,259,69.97,'Intermediate' from dual union all
select 16514,'016514','1986-009C','Sensor cover','Sensor cover',59202,'Reentered','1986-01-31 00:00:00','1986-01-29 00:00:00',89.26,203,272,69.98,'Intermediate' from dual union all
select 16515,'016515','1979-017FM','P78-1 debris','',47501,'Reentered','1988-12-28 00:00:00','1986-01-17 00:00:00',96.3,521,641,97.66,'Sun-Synch' from dual union all
select 16516,'016516','1979-017FN','P78-1 debris','',47501,'Reentered','1987-08-25 00:00:00','1986-01-17 00:00:00',94.26,439,527,97.74,'Sun-Synch' from dual union all
select 16517,'016517','1979-017FP','P78-1 debris','',47501,'Reentered','1986-04-13 00:00:00','1985-09-23 00:00:00',95.2,508,549,97.73,'Sun-Synch' from dual union all
select 16518,'016518','1979-017FQ','P78-1 debris','',47501,'Reentered','1988-10-12 00:00:00','1985-12-17 00:00:00',94.34,450,523,97.72,'Sun-Synch' from dual union all
select 16519,'016519','1979-017FR','P78-1 debris','',47501,'Reentered','1986-12-20 00:00:00','1986-01-15 00:00:00',94.74,442,570,97.81,'Sun-Synch' from dual union all
select 1652,'001652','1965-082P','deb Transtage 4','',11832,'Reentered','1979-03-15 00:00:00','1968-11-23 00:00:00',100.53,721,847,33.25,'Intermediate' from dual union all
select 16520,'016520','1979-017FS','P78-1 debris','',47501,'Reentered','1988-03-16 00:00:00','1985-11-30 00:00:00',95.29,517,549,97.53,'Sun-Synch' from dual union all
select 16521,'016521','1979-017FT','P78-1 debris','',47501,'Reentered','1987-02-14 00:00:00','1986-01-12 00:00:00',94.5,411,578,97.83,'Sun-Synch' from dual union all
select 16522,'016522','1979-017FU','P78-1 debris','',47501,'Reentered','1987-10-21 00:00:00','1986-04-14 00:00:00',95.06,500,543,97.7,'Sun-Synch' from dual union all
select 16523,'016523','1979-017FV','P78-1 debris','',47501,'Reentered','1988-01-30 00:00:00','1985-12-15 00:00:00',95.43,516,563,97.67,'Sun-Synch' from dual union all
select 16524,'016524','1979-017FW','P78-1 debris','',47501,'Reentered','1986-10-16 00:00:00','1985-12-15 00:00:00',94.85,500,523,97.8,'Sun-Synch' from dual union all
select 16525,'016525','1979-017FX','P78-1 debris','',47501,'Reentered','1986-11-07 00:00:00','1985-12-17 00:00:00',94.51,477,512,98.07,'Sun-Synch' from dual union all
select 16526,'016526','1986-010A','Shiyong Tongbu Tongxin Wx.','DFH-2',59203,'In Earth orbit','','1986-04-04 00:00:00',1435.98,35782,35787,.02,'Stationary' from dual union all
select 16527,'016527','1986-011A','Kosmos-1729','Oko',59204,'In Earth orbit','','1986-04-08 00:00:00',717.42,667,39670,63,'Molniya' from dual union all
select 16528,'016528','1986-010B','CZ-3 Stage 3','CZ-3 Stage 3',59203,'In Earth orbit','','1986-04-02 00:00:00',630.96,425,35556,30.8,'GEO Transfer' from dual union all
select 16529,'016529','1986-011B','BOZ','BOZ',59204,'Reentered','1986-03-10 00:00:00','1986-02-08 00:00:00',92.26,187,583,62.88,'Intermediate' from dual union all
select 1653,'001653','1965-082Q','deb Transtage 4','',11832,'In Earth orbit','','1991-05-04 00:00:00',98.88,673,738,32.12,'Intermediate' from dual union all
select 16530,'016530','1986-011C','11S510','11S510',59204,'Reentered','1986-03-24 00:00:00','1986-02-08 00:00:00',91.96,211,531,62.84,'Intermediate' from dual union all
select 16531,'016531','1986-011D','BOZ part','',59204,'Reentered','1986-03-31 00:00:00','1986-02-08 00:00:00',92.8,219,604,62.86,'Intermediate' from dual union all
select 16532,'016532','1986-011E','BOZ part','',59204,'Reentered','1986-03-13 00:00:00','1986-02-11 00:00:00',92.23,205,562,62.84,'Intermediate' from dual union all
select 16533,'016533','1986-011F','Blok-2BL','Blok-2BL No. 76746-665',59204,'In Earth orbit','','1986-04-03 00:00:00',705.73,662,39097,62.97,'Molniya' from dual union all
select 16534,'016534','1979-017FY','P78-1 debris','',47501,'Reentered','1989-02-20 00:00:00','1985-12-17 00:00:00',96.12,524,622,97.77,'Sun-Synch' from dual union all
select 16535,'016535','1979-017FZ','P78-1 debris','',47501,'Reentered','1987-03-13 00:00:00','1985-12-17 00:00:00',95.06,513,529,97.79,'Sun-Synch' from dual union all
select 16536,'016536','1979-017GA','P78-1 debris','',47501,'Reentered','1986-05-11 00:00:00','1985-12-15 00:00:00',93.95,415,521,97.76,'Sun-Synch' from dual union all
select 16537,'016537','1979-017GB','P78-1 debris','',47501,'Reentered','1988-01-29 00:00:00','1985-12-17 00:00:00',95.93,504,623,97.78,'Sun-Synch' from dual union all
select 16538,'016538','1979-017GC','P78-1 debris','',47501,'Reentered','1988-01-09 00:00:00','1986-02-03 00:00:00',94.77,485,530,97.73,'Sun-Synch' from dual union all
select 16539,'016539','1979-017GD','P78-1 debris','',47501,'Reentered','1986-04-11 00:00:00','1985-12-17 00:00:00',93.9,391,539,97.87,'Sun-Synch' from dual union all
select 1654,'001654','1965-082R','deb Transtage 4','',11832,'Reentered','1979-10-16 00:00:00','1966-01-14 00:00:00',99.27,699,749,32.45,'Intermediate' from dual union all
select 16540,'016540','1986-012A','Kosmos-1730','Oblik No. 324',59205,'Landed','1986-02-13 00:00:00','1986-02-06 00:00:00',89.59,226,283,72.85,'Intermediate' from dual union all
select 16541,'016541','1986-012B','11S510','11S510',59205,'Reentered','1986-02-11 00:00:00','1986-02-06 00:00:00',89.32,192,289,72.86,'Intermediate' from dual union all
select 16542,'016542','1979-017GE','P78-1 debris','',47501,'Reentered','1986-10-25 00:00:00','1985-12-17 00:00:00',95.04,502,539,97.95,'Sun-Synch' from dual union all
select 16543,'016543','1979-017GF','P78-1 debris','',47501,'Reentered','1987-04-20 00:00:00','1985-12-17 00:00:00',95.57,515,578,97.98,'Sun-Synch' from dual union all
select 16544,'016544','1964-006H','deb Elektron-1','',10292,'Reentered','1990-04-17 00:00:00','1990-03-27 00:00:00',90.93,222,419,62.78,'Intermediate' from dual union all
select 16545,'016545','1964-006J','deb Elektron-1','',10292,'Reentered','2001-08-24 00:00:00','1983-02-28 00:00:00',137.26,386,4486,60.77,'Medium' from dual union all
select 16546,'016546','1964-006K','deb Elektron-1','',10292,'Reentered','2001-04-26 00:00:00','2001-02-24 00:00:00',95.29,320,746,60.69,'Intermediate' from dual union all
select 16547,'016547','1964-006L','deb Elektron-1','',10292,'Reentered','2001-04-21 00:00:00','1985-02-02 00:00:00',134.36,390,4234,60.78,'Medium' from dual union all
select 16548,'016548','1964-006M','deb Elektron-1','',10292,'Reentered','1990-01-03 00:00:00','',135.57,388,4340,60.78,'Medium' from dual union all
select 16549,'016549','1979-017GG','P78-1 debris','',47501,'Reentered','1988-12-24 00:00:00','1986-01-07 00:00:00',95.23,518,542,97.9,'Sun-Synch' from dual union all
select 1655,'001655','1965-082S','deb Transtage 4','',11832,'Reentered','1971-04-19 00:00:00','1966-01-19 00:00:00',98.56,602,778,31.81,'Intermediate' from dual union all
select 16550,'016550','1979-017GH','P78-1 debris','',47501,'Reentered','1990-01-02 00:00:00','1985-11-30 00:00:00',95.3,515,552,97.8,'Sun-Synch' from dual union all
select 16551,'016551','1979-017GJ','P78-1 debris','',47501,'Reentered','1996-12-21 00:00:00','1993-08-11 00:00:00',92.72,398,418,97.82,'Sun-Synch' from dual union all
select 16552,'016552','1979-017GK','P78-1 debris','',47501,'Reentered','1987-09-18 00:00:00','1986-01-17 00:00:00',95.37,511,561,97.65,'Sun-Synch' from dual union all
select 16553,'016553','1979-017GL','P78-1 debris','',47501,'Reentered','1988-02-24 00:00:00','1986-01-27 00:00:00',95.3,495,572,97.7,'Sun-Synch' from dual union all
select 16554,'016554','1979-017GM','P78-1 debris','',47501,'Reentered','1987-05-27 00:00:00','1986-01-17 00:00:00',95.11,509,539,97.94,'Sun-Synch' from dual union all
select 16555,'016555','1979-017GN','P78-1 debris','',47501,'Reentered','1987-07-29 00:00:00','1986-01-17 00:00:00',95.44,509,571,97.68,'Sun-Synch' from dual union all
select 16556,'016556','1979-017GP','P78-1 debris','',47501,'Reentered','1986-02-12 00:00:00','1986-01-16 00:00:00',92.62,341,465,97.88,'Sun-Synch' from dual union all
select 16557,'016557','1979-017GQ','P78-1 debris','',47501,'Reentered','1986-06-20 00:00:00','1986-01-17 00:00:00',94.19,438,521,97.74,'Sun-Synch' from dual union all
select 16558,'016558','1979-017GR','P78-1 debris','',47501,'Reentered','1987-08-12 00:00:00','1986-02-06 00:00:00',95.27,508,555,97.72,'Sun-Synch' from dual union all
select 16559,'016559','1979-017GS','P78-1 debris','',47501,'Reentered','1986-04-30 00:00:00','1986-03-05 00:00:00',94.94,504,528,97.69,'Sun-Synch' from dual union all
select 1656,'001656','1965-082T','deb Transtage 4','',11832,'Reentered','1981-11-25 00:00:00','1966-01-15 00:00:00',101.73,732,950,32.04,'Intermediate' from dual union all
select 16560,'016560','1979-017GT','P78-1 debris','',47501,'Reentered','1987-02-24 00:00:00','1986-01-17 00:00:00',95.05,505,537,97.71,'Sun-Synch' from dual union all
select 16561,'016561','1979-017GU','P78-1 debris','',47501,'Reentered','1988-01-16 00:00:00','1986-01-17 00:00:00',94.97,501,534,97.86,'Sun-Synch' from dual union all
select 16562,'016562','1979-017GV','P78-1 debris','',47501,'Reentered','1986-06-25 00:00:00','1986-02-06 00:00:00',94.56,461,533,97.72,'Sun-Synch' from dual union all
select 16563,'016563','1979-017GW','P78-1 debris','',47501,'Reentered','1986-08-26 00:00:00','1986-01-16 00:00:00',94.54,465,528,97.68,'Sun-Synch' from dual union all
select 16564,'016564','1979-017GX','P78-1 debris','',47501,'Reentered','2004-05-09 00:00:00','1993-08-13 00:00:00',94.17,474,483,97.81,'Sun-Synch' from dual union all
select 16565,'016565','1979-017GY','P78-1 debris','',47501,'Reentered','1988-05-11 00:00:00','1986-02-08 00:00:00',95.13,508,542,97.75,'Sun-Synch' from dual union all
select 16566,'016566','1979-017GZ','P78-1 debris','',47501,'Reentered','1988-10-10 00:00:00','1986-02-08 00:00:00',94.99,505,532,97.93,'Sun-Synch' from dual union all
select 16567,'016567','1979-017HA','P78-1 debris','',47501,'Reentered','1989-11-22 00:00:00','1986-01-15 00:00:00',95.18,523,532,97.79,'Sun-Synch' from dual union all
select 16568,'016568','1979-017HB','P78-1 debris','',47501,'Reentered','1988-06-01 00:00:00','1986-04-07 00:00:00',94.93,508,522,97.64,'Sun-Synch' from dual;
insert into SATELLITE
select 16569,'016569','1979-017HC','P78-1 debris','',47501,'Reentered','1987-11-15 00:00:00','1986-01-22 00:00:00',94.99,508,528,97.89,'Sun-Synch' from dual union all
select 1657,'001657','1965-082U','deb Transtage 4','',11832,'Reentered','1970-06-17 00:00:00','1966-01-09 00:00:00',97.34,527,736,32.17,'Intermediate' from dual union all
select 16570,'016570','1979-017HD','P78-1 debris','',47501,'Reentered','1992-12-06 00:00:00','1992-09-12 00:00:00',90.79,310,316,97.78,'Sun-Synch' from dual union all
select 16571,'016571','1979-017HE','P78-1 debris','',47501,'Reentered','1988-04-28 00:00:00','1986-01-18 00:00:00',94.51,459,531,97.83,'Sun-Synch' from dual union all
select 16572,'016572','1979-017HF','P78-1 debris','',47501,'Reentered','1988-07-15 00:00:00','1986-01-17 00:00:00',95.63,516,583,97.88,'Sun-Synch' from dual union all
select 16573,'016573','1979-017HG','P78-1 debris','',47501,'Reentered','1987-12-12 00:00:00','1986-01-18 00:00:00',95.55,513,578,97.67,'Sun-Synch' from dual union all
select 16574,'016574','1979-017HH','P78-1 debris','',47501,'Reentered','1988-10-20 00:00:00','1986-02-06 00:00:00',95.32,511,558,97.7,'Sun-Synch' from dual union all
select 16575,'016575','1979-017HJ','P78-1 debris','',47501,'Reentered','1987-07-17 00:00:00','1986-01-17 00:00:00',95.04,509,532,97.78,'Sun-Synch' from dual union all
select 16576,'016576','1979-017HK','P78-1 debris','',47501,'Reentered','1988-08-08 00:00:00','1986-01-17 00:00:00',95.11,511,537,97.88,'Sun-Synch' from dual union all
select 16577,'016577','1979-017HL','P78-1 debris','',47501,'Reentered','1986-11-05 00:00:00','1986-02-11 00:00:00',94.26,468,498,97.91,'Sun-Synch' from dual union all
select 16578,'016578','1979-017HM','P78-1 debris','',47501,'Reentered','1986-03-27 00:00:00','1986-01-20 00:00:00',93.62,398,506,97.81,'Sun-Synch' from dual union all
select 16579,'016579','1979-017HN','P78-1 debris','',47501,'Reentered','1986-03-27 00:00:00','1986-01-17 00:00:00',95.51,518,569,97.73,'Sun-Synch' from dual union all
select 1658,'001658','1965-082V','deb Transtage 4','',11832,'Reentered','1970-05-08 00:00:00','1966-01-17 00:00:00',97.17,508,739,32.41,'Intermediate' from dual union all
select 16580,'016580','1979-017HP','P78-1 debris','',47501,'Reentered','1987-09-09 00:00:00','1986-02-11 00:00:00',95.29,510,556,97.66,'Sun-Synch' from dual union all
select 16581,'016581','1979-017HQ','P78-1 debris','',47501,'Reentered','1989-01-17 00:00:00','1986-01-17 00:00:00',95.43,508,571,97.62,'Sun-Synch' from dual union all
select 16582,'016582','1979-017HR','P78-1 debris','',47501,'Reentered','1989-01-15 00:00:00','1986-01-17 00:00:00',95.36,503,569,97.67,'Sun-Synch' from dual union all
select 16583,'016583','1979-017HS','P78-1 debris','',47501,'Reentered','1987-05-06 00:00:00','1986-01-17 00:00:00',94.98,501,535,97.7,'Sun-Synch' from dual union all
select 16584,'016584','1979-017HT','P78-1 debris','',47501,'Reentered','1988-09-09 00:00:00','1986-01-17 00:00:00',95.08,507,538,98.1,'Sun-Synch' from dual union all
select 16585,'016585','1979-017HU','P78-1 debris','',47501,'Reentered','1986-07-23 00:00:00','1986-04-14 00:00:00',93.18,418,443,97.61,'Sun-Synch' from dual union all
select 16586,'016586','1979-017HV','P78-1 debris','',47501,'Reentered','1986-02-10 00:00:00','1986-02-06 00:00:00',92.11,362,393,97.91,'Sun-Synch' from dual union all
select 16587,'016587','1979-017HW','P78-1 debris','',47501,'Reentered','1989-01-22 00:00:00','1986-01-27 00:00:00',95.39,511,564,97.76,'Sun-Synch' from dual union all
select 16588,'016588','1979-017HX','P78-1 debris','',47501,'Reentered','1988-07-01 00:00:00','1986-01-18 00:00:00',95.4,504,572,97.68,'Sun-Synch' from dual union all
select 16589,'016589','1986-013A','Kosmos-1731','Yantar''-4KS1M',59206,'Deorbited','1986-10-03 00:00:00','1986-03-09 00:00:00',89.47,238,258,64.76,'Intermediate' from dual union all
select 1659,'001659','1965-082W','deb Transtage 4','',11832,'Reentered','1969-11-20 00:00:00','1966-01-24 00:00:00',99.31,632,820,33.62,'Intermediate' from dual union all
select 16590,'016590','1986-013B','11S510','11S510',59206,'Reentered','1986-02-09 00:00:00','1986-02-08 00:00:00',88.69,176,244,64.78,'Intermediate' from dual union all
select 16591,'016591','1986-014A','USA-15','PARCAE 8',59207,'In Earth orbit','','1990-04-27 00:00:00',107.44,1080,1137,63.43,'Intermediate' from dual union all
select 16592,'016592','1986-014B','MSD plume shield','',59207,'In Earth orbit','','1997-11-22 00:00:00',107.01,869,1308,63.37,'Intermediate' from dual union all
select 16593,'016593','1986-015A','Kosmos-1732','Musson No. 16',59208,'In Earth orbit','','1986-03-13 00:00:00',116.03,1479,1526,73.61,'Intermediate' from dual union all
select 16594,'016594','1986-015B','S5M','S5M',59208,'In Earth orbit','','1986-03-13 00:00:00',115.98,1478,1521,73.61,'Intermediate' from dual union all
select 16595,'016595','1986-009D','KDU','KDU',59202,'Reentered','1986-02-26 00:00:00','1986-02-12 00:00:00',89.23,220,253,69.97,'Intermediate' from dual union all
select 16596,'016596','1986-009E','Blenda','Sunshade',59202,'Reentered','1986-02-18 00:00:00','1986-02-12 00:00:00',91.15,235,427,69.97,'Intermediate' from dual union all
select 16597,'016597','1986-016A','Yuri 2B','BS 2b',59209,'In Earth orbit','','1986-05-15 00:00:00',1435.81,35747,35815,.71,'Stationary' from dual union all
select 16598,'016598','1986-016B','N-II Stage 2 S/N N-14(F)','AJ10-118FJ',59209,'Reentered','1987-04-19 00:00:00','1986-03-14 00:00:00',109.08,199,2170,29.96,'Intermediate' from dual union all
select 16599,'016599','1986-009F','KDU part','KDU part',59202,'Reentered','1986-02-15 00:00:00','1986-02-12 00:00:00',88.93,210,234,69.96,'Intermediate' from dual union all
select 166,'000166','61 Rho 3','Tiros 3 despin weight','Despin weight',7277,'Reentered','1992-01-19 00:00:00','1961-08-19 00:00:00',98.87,605,805,47.93,'Intermediate' from dual union all
select 1660,'001660','1965-082X','deb Transtage 4','',11832,'Reentered','1969-09-23 00:00:00','1966-01-29 00:00:00',98.35,598,762,32.53,'Intermediate' from dual union all
select 16600,'016600','1986-016C','Star 37E S/N 40104','Star 37E',59209,'Reentered','2003-11-02 00:00:00','1986-04-14 00:00:00',635.03,199,35991,28.16,'GEO Transfer' from dual union all
select 16601,'016601','1986-016D','deb N-2 Stage 2','',59209,'Reentered','1986-02-22 00:00:00','1986-02-13 00:00:00',94.23,199,763,30.26,'Intermediate' from dual union all
select 16602,'016602','1986-012C','KDU','KDU',59205,'Reentered','1986-03-17 00:00:00','1986-02-13 00:00:00',89.82,224,308,72.86,'Intermediate' from dual union all
select 16603,'016603','1986-012D','Blenda','Sunshade',59205,'Reentered','1986-02-22 00:00:00','1986-02-13 00:00:00',89.67,226,291,72.84,'Intermediate' from dual union all
select 16604,'016604','1986-012E','KDU part','KDU part',59205,'Reentered','1986-03-02 00:00:00','1986-02-13 00:00:00',91.66,241,471,72.83,'Intermediate' from dual union all
select 16605,'016605','1985-050N','Kosmos-1662 SS 7','ESO',58273,'Reentered','1987-05-04 00:00:00','1986-02-14 00:00:00',94.49,470,518,65.91,'Intermediate' from dual union all
select 16606,'016606','1985-050P','Kosmos-1662 SS 8','ESO',58273,'Reentered','1987-04-27 00:00:00','1986-02-14 00:00:00',94.42,464,517,65.77,'Intermediate' from dual union all
select 16607,'016607','1985-050Q','Kosmos-1662 SS 9','ESO',58273,'Reentered','1987-05-07 00:00:00','1986-02-18 00:00:00',94.47,470,516,65.9,'Intermediate' from dual union all
select 16608,'016608','1985-050R','Kosmos-1662 SS 10','ESO',58273,'Reentered','1987-04-08 00:00:00','1986-02-18 00:00:00',94.42,464,517,65.77,'Intermediate' from dual union all
select 16609,'016609','1986-017A','Mir','DOS 7 (17K No. 127-01)',59210,'Deorbited','2001-03-23 00:00:00','1986-03-21 00:00:00',91.23,332,338,51.62,'Intermediate' from dual union all
select 1661,'001661','1965-082Y','deb Transtage 4','',11832,'Reentered','1971-05-15 00:00:00','1966-01-11 00:00:00',98.41,644,722,32.75,'Intermediate' from dual union all
select 16610,'016610','1986-017B','8S812','8S812',59210,'Reentered','1986-02-24 00:00:00','1986-02-21 00:00:00',88.7,165,256,51.63,'Intermediate' from dual union all
select 16611,'016611','1986-018A','Kosmos-1733','Tselina-D',59211,'In Earth orbit','','1986-03-21 00:00:00',97.66,631,662,82.52,'Intermediate' from dual union all
select 16612,'016612','1986-018B','S5M','S5M',59211,'In Earth orbit','','1986-03-31 00:00:00',97.63,630,661,82.52,'Intermediate' from dual union all
select 16613,'016613','1986-019A','SPOT 1','SPOT',59212,'In Earth orbit','','1986-03-24 00:00:00',101.4,824,826,98.73,'Sun-Synch' from dual union all
select 16614,'016614','1986-019B','Viking','Viking',59212,'In Earth orbit','','1986-04-24 00:00:00',261.65,819,13527,98.74,'Medium' from dual union all
select 16615,'016615','1986-019C','Ariane H8 V16','Ariane H8 S/N T11',59212,'In Earth orbit','','1986-03-24 00:00:00',101.24,803,831,98.71,'Sun-Synch' from dual union all
select 16616,'016616','1986-019D','Ariane H8 clamp ring','Payload clamp ring',59212,'In Earth orbit','','1986-03-25 00:00:00',101.45,821,833,98.73,'Sun-Synch' from dual union all
select 16617,'016617','1984-083C','deb Kosmos-1588','',57137,'Reentered','1986-07-13 00:00:00','1986-02-28 00:00:00',93.57,422,477,64.91,'Intermediate' from dual union all
select 16618,'016618','1986-020A','Kosmos-1734','Kobal''t No. 838',59213,'Landed','1986-04-26 00:00:00','1986-03-05 00:00:00',89.2,159,311,67.12,'Intermediate' from dual union all
select 16619,'016619','1986-020B','11S510','11S510',59213,'Reentered','1986-03-03 00:00:00','1986-02-28 00:00:00',89.14,161,303,67.12,'Intermediate' from dual union all
select 1662,'001662','1965-082Z','Transtage 4 large debris','RCS 2.2',11832,'Reentered','1989-08-31 00:00:00','1966-01-21 00:00:00',100.36,696,856,32.09,'Intermediate' from dual union all
select 16620,'016620','1986-021A','Kosmos-1735','US-PM',59214,'Reentered','1988-11-17 00:00:00','1986-03-29 00:00:00',92.77,403,418,65.04,'Intermediate' from dual union all
select 16621,'016621','1986-021B','11S692','11S692',59214,'Reentered','1986-02-27 00:00:00','1986-02-27 00:00:00',89.46,113,383,65.01,'Intermediate' from dual union all
select 16622,'016622','1986-014C','USA-16 cover','SSU cover',59207,'In Earth orbit','','1997-11-22 00:00:00',107.43,870,1346,63.44,'Intermediate' from dual union all
select 16623,'016623','1986-014D','USA-17 cover','SSU cover',59207,'In Earth orbit','','1990-05-07 00:00:00',107.44,1039,1178,63.38,'Intermediate' from dual union all
select 16624,'016624','1986-014E','USA-16','SSU',59207,'In Earth orbit','','1992-11-25 00:00:00',107.43,1038,1178,63.43,'Intermediate' from dual union all
select 16625,'016625','1986-014F','USA-17','SSU',59207,'In Earth orbit','','1992-11-19 00:00:00',107.43,1038,1178,63.43,'Intermediate' from dual union all
select 16626,'016626','1984-083D','deb Kosmos-1588','',57137,'Reentered','1986-03-27 00:00:00','1986-02-28 00:00:00',94.52,405,586,64.66,'Intermediate' from dual union all
select 16627,'016627','1984-083E','deb Kosmos-1588','',57137,'Reentered','1986-10-11 00:00:00','1986-02-24 00:00:00',94.5,412,577,64.76,'Intermediate' from dual union all
select 16628,'016628','1984-083F','deb Kosmos-1588','',57137,'Reentered','1986-03-08 00:00:00','1986-02-27 00:00:00',92.84,392,436,65.06,'Intermediate' from dual union all
select 16629,'016629','1984-083G','deb Kosmos-1588','',57137,'Reentered','1986-04-04 00:00:00','1986-03-24 00:00:00',93.22,377,488,64.64,'Intermediate' from dual union all
select 1663,'001663','1965-082AA','deb Transtage 4','',11832,'Reentered','2002-05-04 00:00:00','1992-01-31 00:00:00',101.35,658,988,32.4,'Intermediate' from dual union all
select 16630,'016630','1986-014G','USA-18 cover','SSU cover',59207,'In Earth orbit','','1997-11-22 00:00:00',107.42,878,1337,63.44,'Intermediate' from dual union all
select 16631,'016631','1986-014H','USA-18','SSU',59207,'In Earth orbit','','1990-05-03 00:00:00',107.44,1015,1202,63.46,'Intermediate' from dual union all
select 16632,'016632','1984-083H','deb Kosmos-1588','',57137,'Reentered','1986-04-04 00:00:00','1986-03-08 00:00:00',92.41,377,408,65.04,'Intermediate' from dual union all
select 16633,'016633','1984-083J','deb Kosmos-1588','',57137,'Reentered','1988-05-30 00:00:00','1986-03-08 00:00:00',93.04,408,439,65.03,'Intermediate' from dual union all
select 16634,'016634','1984-083K','deb Kosmos-1588 solar array','RCS 69m2',57137,'Reentered','1986-04-29 00:00:00','1986-03-07 00:00:00',93.02,411,433,65.03,'Intermediate' from dual union all
select 16635,'016635','1984-083L','deb Kosmos-1588','',57137,'Reentered','1986-04-21 00:00:00','1986-03-08 00:00:00',92.49,379,415,65.04,'Intermediate' from dual union all
select 16636,'016636','1984-083M','deb Kosmos-1588','',57137,'Reentered','1986-04-24 00:00:00','1986-03-09 00:00:00',92.72,391,425,65.03,'Intermediate' from dual union all
select 16637,'016637','1984-083N','deb Kosmos-1588','',57137,'Reentered','1986-04-26 00:00:00','1986-03-09 00:00:00',93.08,414,437,65,'Intermediate' from dual union all
select 16638,'016638','1984-083P','deb Kosmos-1588','',57137,'Reentered','1986-05-17 00:00:00','1986-03-10 00:00:00',92.81,402,423,65,'Intermediate' from dual union all
select 16639,'016639','1984-083Q','deb Kosmos-1588','',57137,'Reentered','1987-12-01 00:00:00','1986-04-11 00:00:00',92.96,408,431,65.03,'Intermediate' from dual union all
select 1664,'001664','1965-082AB','deb Transtage 4','',11832,'Reentered','1998-07-15 00:00:00','1991-05-24 00:00:00',96.33,569,597,32.39,'Intermediate' from dual union all
select 16640,'016640','1984-083R','deb Kosmos-1588','',57137,'Reentered','1986-04-05 00:00:00','1986-03-10 00:00:00',92.12,348,409,65.05,'Intermediate' from dual union all
select 16641,'016641','1984-083S','deb Kosmos-1588','',57137,'Reentered','1986-03-31 00:00:00','1986-03-09 00:00:00',92.33,371,407,65,'Intermediate' from dual union all
select 16642,'016642','1984-083T','deb Kosmos-1588','',57137,'Reentered','1986-04-04 00:00:00','1986-03-10 00:00:00',92.27,356,416,65.04,'Intermediate' from dual union all
select 16643,'016643','1986-022A','Soyuz T-15','Soyuz 7K-ST No. 21L',59215,'Landed','1986-07-16 00:00:00','1986-04-12 00:00:00',91.49,337,359,51.62,'Intermediate' from dual union all
select 16644,'016644','1986-022B','11S510','11S510',59215,'Reentered','1986-03-16 00:00:00','1986-03-15 00:00:00',88.38,180,209,51.62,'Intermediate' from dual union all
select 16645,'016645','1986-023A','Progress-25','Progress 7K-TG No. 134',59216,'Deorbited','1986-04-21 00:00:00','1986-03-26 00:00:00',91.47,337,356,51.63,'Intermediate' from dual union all
select 16646,'016646','1986-023B','11S510','11S510',59216,'Reentered','1986-03-22 00:00:00','1986-03-21 00:00:00',88.36,171,216,51.63,'Intermediate' from dual union all
select 16647,'016647','1986-024A','Kosmos-1736','US-A reactor',59217,'In Earth orbit','','1986-04-20 00:00:00',89.63,250,262,64.99,'Intermediate' from dual union all
select 16648,'016648','1986-025A','Kosmos-1737','US-PM1',59218,'Deorbited','1986-12-03 00:00:00','1986-04-24 00:00:00',93.02,416,429,73.36,'Intermediate' from dual union all
select 16649,'016649','1986-026A','Gstar 2','Gstar 2',59219,'In Earth orbit','','1986-05-31 00:00:00',1436.04,35780,35791,.02,'Stationary' from dual union all
select 1665,'001665','1965-082AC','deb Transtage 4','',11832,'Reentered','1981-01-28 00:00:00','1966-01-20 00:00:00',100.15,712,820,31.83,'Intermediate' from dual union all
select 16650,'016650','1986-026B','Brasilsat 2','SBTS 2',59219,'In Earth orbit','','1986-06-15 00:00:00',1436.09,35760,35813,.06,'Stationary' from dual union all
select 16651,'016651','1984-083U','deb Kosmos-1588','',57137,'Reentered','1986-04-13 00:00:00','1986-04-01 00:00:00',91.66,341,371,65.02,'Intermediate' from dual union all
select 16652,'016652','1984-083V','deb Kosmos-1588','',57137,'Reentered','1986-05-13 00:00:00','1986-03-27 00:00:00',92.94,385,452,65.02,'Intermediate' from dual union all
select 16653,'016653','1984-083W','deb Kosmos-1588','',57137,'Reentered','1986-04-20 00:00:00','1986-03-28 00:00:00',91.77,333,390,65.02,'Intermediate' from dual union all
select 16654,'016654','1984-083X','deb Kosmos-1588','',57137,'Reentered','1986-03-25 00:00:00','1986-03-31 00:00:00',91.28,294,381,65.02,'Intermediate' from dual union all
select 16655,'016655','1984-083Y','deb Kosmos-1588','',57137,'Reentered','1986-07-08 00:00:00','1986-03-28 00:00:00',92.66,390,420,65.05,'Intermediate' from dual union all
select 16656,'016656','1984-083Z','deb Kosmos-1588','',57137,'Reentered','1986-05-16 00:00:00','1986-03-28 00:00:00',92.59,388,415,65,'Intermediate' from dual union all
select 16657,'016657','1986-026C','Ariane H10 V17','Ariane H10 S/N T17',59219,'In Earth orbit','','1986-05-29 00:00:00',656.53,466,36823,7.11,'GEO Transfer' from dual union all
select 16658,'016658','1986-026D','Sylda 4400 No. 6','Sylda 4400',59219,'Reentered','1987-02-06 00:00:00','1986-05-29 00:00:00',615.18,233,34932,7,'GEO Transfer' from dual;
insert into SATELLITE
select 16659,'016659','1986-025B','11S692','11S692',59218,'Reentered','1986-03-26 00:00:00','1986-03-25 00:00:00',89.57,77,429,73.35,'ERROR - Unexpected value "TA"' from dual union all
select 1666,'001666','1965-082AD','deb Transtage 4','',11832,'Reentered','1979-06-16 00:00:00','1966-01-15 00:00:00',100.64,730,848,32.03,'Intermediate' from dual union all
select 16660,'016660','1979-017HY','P78-1 debris','',47501,'Reentered','1989-02-02 00:00:00','1986-03-31 00:00:00',95.23,518,542,97.66,'Sun-Synch' from dual union all
select 16661,'016661','1979-017HZ','P78-1 debris','',47501,'Reentered','1987-10-26 00:00:00','1986-01-17 00:00:00',95.45,519,562,97.89,'Sun-Synch' from dual union all
select 16662,'016662','1979-017JA','P78-1 debris','',47501,'Reentered','1986-10-30 00:00:00','1986-10-29 00:00:00',89.52,236,265,97.64,'Sun-Synch' from dual union all
select 16663,'016663','1979-017JB','P78-1 debris','',47501,'Reentered','1988-08-31 00:00:00','1985-12-17 00:00:00',95.25,514,547,97.72,'Sun-Synch' from dual union all
select 16664,'016664','1984-083AA','deb Kosmos-1588','',57137,'Reentered','1986-04-11 00:00:00','1986-03-30 00:00:00',92.2,363,402,65.01,'Intermediate' from dual union all
select 16665,'016665','1984-083AB','deb Kosmos-1588','',57137,'Reentered','1986-04-20 00:00:00','1986-04-02 00:00:00',91.65,344,367,65.06,'Intermediate' from dual union all
select 16666,'016666','1984-083AC','deb Kosmos-1588','',57137,'Reentered','1986-04-11 00:00:00','1986-04-02 00:00:00',93.18,374,487,64.64,'Intermediate' from dual union all
select 16667,'016667','1986-027A','Kosmos-1738','Geizer No. 13L',59220,'In Earth orbit','','1986-06-06 00:00:00',1435.98,35739,35830,1.32,'Stationary' from dual union all
select 16668,'016668','1986-027B','8S812','8S812',59220,'Reentered','1986-04-08 00:00:00','1986-04-06 00:00:00',88.23,178,196,51.59,'Intermediate' from dual union all
select 16669,'016669','1986-027C','Perekhodnik','Sredniy perekhodnik',59220,'Reentered','1986-04-05 00:00:00','1986-04-04 00:00:00',88.33,177,207,51.59,'Intermediate' from dual union all
select 1667,'001667','1965-082AE','deb Transtage 4','',11832,'Reentered','1982-04-02 00:00:00','1965-12-30 00:00:00',99.89,738,769,32.1,'Intermediate' from dual union all
select 16670,'016670','1986-027D','SOZ','SOZ',59220,'Reentered','1986-10-27 00:00:00','1986-06-10 00:00:00',648.27,217,36651,47.19,'GEO Transfer' from dual union all
select 16671,'016671','1986-027E','SOZ','SOZ',59220,'Reentered','1986-10-27 00:00:00','1986-06-08 00:00:00',650.01,278,36679,47.25,'GEO Transfer' from dual union all
select 16672,'016672','1984-083AD','deb Kosmos-1588','',57137,'Reentered','1986-05-31 00:00:00','1986-04-05 00:00:00',93.55,401,496,64.96,'Intermediate' from dual union all
select 16673,'016673','1984-083AE','deb Kosmos-1588','',57137,'Reentered','1989-04-18 00:00:00','1985-07-31 00:00:00',99.51,586,885,65.03,'Intermediate' from dual union all
select 16674,'016674','1984-083AF','deb Kosmos-1588','',57137,'Reentered','1986-04-20 00:00:00','1986-04-04 00:00:00',92.32,358,419,65.07,'Intermediate' from dual union all
select 16675,'016675','1984-083AG','deb Kosmos-1588','',57137,'Reentered','1986-06-06 00:00:00','1986-04-04 00:00:00',92.6,387,417,65.03,'Intermediate' from dual union all
select 16676,'016676','1986-027F','Blok DM','11S86 No. 59L',59220,'In Earth orbit','','1986-06-27 00:00:00',1473.98,36396,36654,1.29,'Drift GEO' from dual union all
select 16677,'016677','1986-028A','Kosmos-1739','Kobal''t',59221,'Landed','1986-06-07 00:00:00','1986-04-16 00:00:00',89.06,166,290,64.88,'Intermediate' from dual union all
select 16678,'016678','1986-028B','11S510','11S510',59221,'Reentered','1986-04-14 00:00:00','1986-04-11 00:00:00',89.1,168,292,64.89,'Intermediate' from dual union all
select 16679,'016679','1986-029A','Kosmos-1740','Oblik No. 326',59222,'Landed','1986-04-28 00:00:00','1986-04-17 00:00:00',92.25,355,415,72.88,'Intermediate' from dual union all
select 1668,'001668','1965-082AF','Transtage 4 payload truss','RCS 4.2',11832,'In Earth orbit','','1991-05-10 00:00:00',98.8,686,717,32.07,'Intermediate' from dual union all
select 16680,'016680','1986-029B','11S510','11S510',59222,'Reentered','1986-04-27 00:00:00','1986-04-17 00:00:00',89.87,192,343,72.87,'Intermediate' from dual union all
select 16681,'016681','1986-030A','Kosmos-1741','Strela-2M',59223,'In Earth orbit','','1986-05-18 00:00:00',100.78,781,811,74.03,'Intermediate' from dual union all
select 16682,'016682','1986-030B','S3M','S3M',59223,'In Earth orbit','','1986-05-23 00:00:00',100.67,777,804,74.03,'Intermediate' from dual union all
select 16683,'016683','1986-031A','Molniya-3','Molniya-3 No. 43',59224,'Reentered','1999-02-25 00:00:00','1986-06-18 00:00:00',717.78,583,39772,62.96,'Molniya' from dual union all
select 16684,'016684','1986-031B','11S510','11S510',59224,'Reentered','1986-07-04 00:00:00','1986-05-19 00:00:00',91.39,213,473,62.83,'Intermediate' from dual union all
select 16685,'016685','1986-031C','BOZ','BOZ',59224,'Reentered','1986-06-12 00:00:00','1986-04-25 00:00:00',92.47,196,596,62.84,'Intermediate' from dual union all
select 16686,'016686','1986-031D','Blok-ML','Blok-ML',59224,'Reentered','1999-06-28 00:00:00','1986-07-31 00:00:00',733.55,554,40576,63.05,'Molniya' from dual union all
select 16687,'016687','1986-032A','Progress-26','Progress 7K-TG No. 136',59225,'Deorbited','1986-06-23 00:00:00','1986-05-23 00:00:00',91.24,328,343,51.61,'Intermediate' from dual union all
select 16688,'016688','1986-032B','11S510','11S510',59225,'Reentered','1986-04-26 00:00:00','1986-04-25 00:00:00',88.33,170,213,51.63,'Intermediate' from dual union all
select 16689,'016689','1984-083AH','deb Kosmos-1588','Medium debris, RCS 1m2',57137,'Reentered','1986-05-28 00:00:00','1986-04-17 00:00:00',92.17,367,395,65.02,'Intermediate' from dual union all
select 1669,'001669','1965-082AG','deb Transtage 4','',11832,'Reentered','1971-08-15 00:00:00','1966-01-24 00:00:00',99.36,643,813,32.6,'Intermediate' from dual union all
select 16690,'016690','1984-083AJ','deb Kosmos-1588','',57137,'Reentered','1986-07-02 00:00:00','1986-04-14 00:00:00',92.56,387,413,65.01,'Intermediate' from dual union all
select 16691,'016691','1984-083AK','deb Kosmos-1588','',57137,'Reentered','1986-05-21 00:00:00','1986-04-25 00:00:00',91.91,360,377,65,'Intermediate' from dual union all
select 16692,'016692','1986-020C','Kosmos-1734 AO part','Kobal''t AO part',59213,'Reentered','1986-04-27 00:00:00','1986-04-27 00:00:00',89.4,167,323,67.12,'Intermediate' from dual union all
select 16693,'016693','1986-020D','Kosmos-1734 AO part','Kobal''t AO part',59213,'Reentered','1986-04-30 00:00:00','1986-04-28 00:00:00',89.07,165,292,67.1,'Intermediate' from dual union all
select 16694,'016694','1986-029C','KDU part','KDU part',59222,'Reentered','1986-12-14 00:00:00','1986-05-15 00:00:00',92,339,406,72.87,'Intermediate' from dual union all
select 16695,'016695','1986-029D','KDU part','KDU part',59222,'Reentered','1986-08-17 00:00:00','1986-05-16 00:00:00',93.12,388,467,72.15,'Intermediate' from dual union all
select 16696,'016696','1986-029E','Blenda','Sunshade',59222,'Reentered','1987-03-18 00:00:00','1986-05-15 00:00:00',94.31,385,586,72.88,'Intermediate' from dual union all
select 16697,'016697','1986-029F','KDU part','KDU part',59222,'Reentered','1986-07-31 00:00:00','1986-05-15 00:00:00',95.59,385,710,72.88,'Intermediate' from dual union all
select 16698,'016698','1986-029G','KDU part','KDU part',59222,'Reentered','1986-10-07 00:00:00','1986-05-15 00:00:00',92.16,352,409,72.88,'Intermediate' from dual union all
select 16699,'016699','1986-029H','KDU part','KDU part',59222,'Reentered','1986-05-06 00:00:00','1986-04-29 00:00:00',92.3,353,422,72.9,'Intermediate' from dual union all
select 167,'000167','61 Rho 4','Tiros 3 despin weight','Despin weight',7277,'In Earth orbit','','1961-08-13 00:00:00',102,802,905,47.89,'Intermediate' from dual union all
select 1670,'001670','1965-082AH','deb Transtage 4','',11832,'Reentered','1989-02-25 00:00:00','1966-01-21 00:00:00',98.7,636,757,32.09,'Intermediate' from dual union all
select 16700,'016700','1986-029J','KDU part','KDU part',59222,'Reentered','1986-05-11 00:00:00','1986-04-29 00:00:00',91.95,344,397,72.74,'Intermediate' from dual union all
select 16701,'016701','1986-029K','KDU part','KDU part',59222,'Reentered','1986-05-07 00:00:00','1986-04-28 00:00:00',92.74,351,466,72.86,'Intermediate' from dual union all
select 16702,'016702','1984-083AL','deb Kosmos-1588','',57137,'Reentered','1986-07-17 00:00:00','1986-04-18 00:00:00',92.41,378,408,65.02,'Intermediate' from dual union all
select 16703,'016703','1984-083AM','deb Kosmos-1588','',57137,'Reentered','1986-08-02 00:00:00','1986-04-14 00:00:00',92.28,374,399,65.01,'Intermediate' from dual union all
select 16704,'016704','1984-083AN','deb Kosmos-1588','',57137,'Reentered','1986-06-30 00:00:00','1986-04-29 00:00:00',92.19,368,396,65.03,'Intermediate' from dual union all
select 16705,'016705','1984-083AP','deb Kosmos-1588','',57137,'Reentered','1986-06-01 00:00:00','1986-04-29 00:00:00',91.94,357,383,65.02,'Intermediate' from dual union all
select 16706,'016706','1984-083AQ','deb Kosmos-1588','',57137,'Reentered','1986-05-21 00:00:00','1986-04-29 00:00:00',91.73,347,372,65.02,'Intermediate' from dual union all
select 16707,'016707','1984-083AR','deb Kosmos-1588','Medium debris, RCS 1m2',57137,'Reentered','1986-06-13 00:00:00','1986-04-29 00:00:00',91.88,353,381,65.03,'Intermediate' from dual union all
select 16708,'016708','1984-083AS','deb Kosmos-1588','',57137,'Reentered','1986-06-19 00:00:00','1986-04-29 00:00:00',92.14,366,393,65.02,'Intermediate' from dual union all
select 16709,'016709','1984-083AT','deb Kosmos-1588','',57137,'Reentered','1986-07-16 00:00:00','1986-04-29 00:00:00',92.3,373,402,65.02,'Intermediate' from dual union all
select 1671,'001671','1965-082AJ','deb Transtage 4','',11832,'In Earth orbit','','1991-05-03 00:00:00',99.67,697,789,32.17,'Intermediate' from dual union all
select 16710,'016710','1984-083AU','deb Kosmos-1588','',57137,'Reentered','1986-05-31 00:00:00','1986-04-29 00:00:00',91.88,354,380,65.03,'Intermediate' from dual union all
select 16711,'016711','1984-083AV','deb Kosmos-1588','',57137,'Reentered','1986-05-23 00:00:00','1986-04-29 00:00:00',91.68,345,369,65.03,'Intermediate' from dual union all
select 16712,'016712','1984-083AW','deb Kosmos-1588','Medium debris, RCS 1m2',57137,'Reentered','1986-06-19 00:00:00','1986-04-29 00:00:00',92.01,361,386,65.03,'Intermediate' from dual union all
select 16713,'016713','1984-083AX','deb Kosmos-1588','',57137,'Reentered','1986-05-13 00:00:00','1986-04-29 00:00:00',91.39,334,352,65,'Intermediate' from dual union all
select 16714,'016714','1986-029L','KDU','KDU',59222,'Reentered','1988-04-18 00:00:00','1986-05-15 00:00:00',92.21,355,410,72.88,'Intermediate' from dual union all
select 16715,'016715','1970-025PA','deb AGENA D','',21502,'In Earth orbit','','1993-08-01 00:00:00',106.15,1043,1054,99.92,'Sun-Synch' from dual union all
select 16716,'016716','1986-029M','KDU part','KDU part',59222,'Reentered','1986-05-08 00:00:00','1986-05-02 00:00:00',91.9,343,392,72.88,'Intermediate' from dual union all
select 16717,'016717','1986-033A','Kosmos-1742','Oblik No. 329',59226,'Landed','1986-05-28 00:00:00','1986-05-17 00:00:00',92.25,353,417,72.88,'Intermediate' from dual union all
select 16718,'016718','1986-033B','11S510','11S510',59226,'Reentered','1986-05-29 00:00:00','1986-05-21 00:00:00',89.44,189,305,72.87,'Intermediate' from dual union all
select 16719,'016719','1986-034A','Kosmos-1743','Tselina-D',59227,'In Earth orbit','','1986-06-15 00:00:00',97.68,631,665,82.56,'Intermediate' from dual union all
select 1672,'001672','1965-082AK','deb Transtage 4','',11832,'In Earth orbit','','1991-05-22 00:00:00',101.42,702,950,32.36,'Intermediate' from dual union all
select 16720,'016720','1986-034B','S5M','S5M',59227,'In Earth orbit','','1986-06-16 00:00:00',97.67,630,665,82.56,'Intermediate' from dual union all
select 16721,'016721','1982-033GT','Garbage bag','',54342,'Reentered','1986-08-06 00:00:00','1986-05-19 00:00:00',91.4,335,352,51.62,'Intermediate' from dual union all
select 16722,'016722','1986-035A','Soyuz TM','Soyuz 7K-STM No. 51',59228,'Landed','1986-05-30 00:00:00','1986-05-25 00:00:00',91.51,341,357,51.64,'Intermediate' from dual union all
select 16723,'016723','1986-035B','11S510','11S510',59228,'Reentered','1986-05-24 00:00:00','1986-05-23 00:00:00',88.14,178,187,51.61,'Intermediate' from dual union all
select 16724,'016724','1986-036A','Kosmos-1744','Foton 34KS No. 2L',59229,'Landed','1986-06-04 00:00:00','1986-05-23 00:00:00',90.4,218,370,62.82,'Intermediate' from dual union all
select 16725,'016725','1986-036B','11S510','11S510',59229,'Reentered','1986-06-24 00:00:00','1986-05-30 00:00:00',90.08,214,343,62.8,'Intermediate' from dual union all
select 16726,'016726','1979-017JC','P78-1 debris','',47501,'Reentered','1988-11-20 00:00:00','1986-04-14 00:00:00',95.45,463,618,97.66,'Sun-Synch' from dual union all
select 16727,'016727','1986-037A','Kosmos-1745','Parus',59230,'In Earth orbit','','1986-06-22 00:00:00',104.86,965,1011,82.96,'Intermediate' from dual union all
select 16728,'016728','1986-037B','S3M','S3M',59230,'In Earth orbit','','1986-06-22 00:00:00',104.72,961,1002,82.95,'Intermediate' from dual union all
select 16729,'016729','1986-038A','Ekran','Ekran No. 30L',59231,'In Earth orbit','','1986-08-07 00:00:00',1436.16,35773,35803,.17,'Stationary' from dual union all
select 1673,'001673','1965-082AL','deb Transtage 4','',11832,'Reentered','2002-01-19 00:00:00','1991-05-24 00:00:00',98.2,632,714,32.02,'Intermediate' from dual union all
select 16730,'016730','1986-038B','8S812','8S812',59231,'Reentered','1986-05-26 00:00:00','1986-05-25 00:00:00',88.07,174,184,51.61,'Intermediate' from dual union all
select 16731,'016731','1986-038C','Perekhodnik','Sredniy perekhodnik',59231,'Reentered','1986-05-24 00:00:00','1986-05-24 00:00:00',88.42,186,207,51.6,'Intermediate' from dual union all
select 16732,'016732','1986-038D','Blok DM','11S86 No. 64L',59231,'In Earth orbit','','1986-07-23 00:00:00',1420.48,35406,35555,.23,'Drift GEO' from dual union all
select 16733,'016733','1986-038E','SOZ','SOZ',59231,'In Earth orbit','','1986-08-25 00:00:00',627.9,245,35578,47.02,'GEO Transfer' from dual union all
select 16734,'016734','1986-038F','SOZ','SOZ',59231,'Reentered','1986-12-28 00:00:00','1986-08-12 00:00:00',626.29,208,35532,47,'GEO Transfer' from dual union all
select 16735,'016735','1986-039A','Meteor-2','Meteor-2 No. 18',59232,'In Earth orbit','','1986-07-03 00:00:00',104.05,939,960,82.54,'Intermediate' from dual union all
select 16736,'016736','1986-039B','S5M','S5M',59232,'In Earth orbit','','1986-06-27 00:00:00',104.02,940,957,82.53,'Intermediate' from dual union all
select 16737,'016737','1986-040A','Kosmos-1746','Resurs-F1 17F41 No. 58',59233,'Landed','1986-06-12 00:00:00','1986-06-04 00:00:00',89.8,258,272,82.33,'Intermediate' from dual union all
select 16738,'016738','1986-040B','11S510','11S510',59233,'Reentered','1986-06-02 00:00:00','1986-05-30 00:00:00',88.74,175,250,82.33,'Intermediate' from dual union all
select 16739,'016739','1986-033C','KDU part','KDU part',59226,'Reentered','1987-09-05 00:00:00','1986-06-13 00:00:00',95.16,387,666,72.88,'Intermediate' from dual union all
select 1674,'001674','1965-082AM','deb Transtage 4','',11832,'Reentered','1969-03-18 00:00:00','1966-01-24 00:00:00',97.72,560,740,32.27,'Intermediate' from dual union all
select 16740,'016740','1986-033D','Blenda','Sunshade',59226,'Reentered','1986-12-29 00:00:00','1986-06-13 00:00:00',92.04,337,413,72.87,'Intermediate' from dual union all
select 16741,'016741','1986-033E','KDU part','KDU part',59226,'Reentered','1986-11-17 00:00:00','1986-06-13 00:00:00',95.35,386,685,72.87,'Intermediate' from dual union all
select 16742,'016742','1986-033F','KDU','KDU',59226,'Reentered','1988-04-25 00:00:00','1986-06-13 00:00:00',92.25,356,413,72.87,'Intermediate' from dual union all
select 16743,'016743','1986-033G','KDU part','KDU part',59226,'Reentered','1986-06-28 00:00:00','1986-05-28 00:00:00',92.22,352,415,72.88,'Intermediate' from dual union all
select 16744,'016744','1986-033H','KDU part','KDU part',59226,'Reentered','1986-10-31 00:00:00','1986-06-13 00:00:00',92.2,350,414,72.88,'Intermediate' from dual union all
select 16745,'016745','1986-041A','Kosmos-1747','Oblik',59234,'Landed','1986-06-12 00:00:00','1986-06-03 00:00:00',89.15,210,255,70.35,'Intermediate' from dual union all
select 16746,'016746','1986-041B','11S510','11S510',59234,'Reentered','1986-06-25 00:00:00','1986-06-05 00:00:00',90.12,202,358,70.36,'Intermediate' from dual union all
select 16747,'016747','1986-040C','MKA subsatelite','',59233,'Reentered','1986-06-08 00:00:00','1986-05-30 00:00:00',89.64,252,262,82.3,'Intermediate' from dual union all
select 16748,'016748','1986-035C','Soyuz TM BO','Bitovoy otsek',59228,'Reentered','1987-03-22 00:00:00','1986-06-21 00:00:00',91.18,327,338,51.62,'Intermediate' from dual;
insert into SATELLITE
select 16749,'016749','1982-033GU','EVA debris','',54342,'Reentered','1986-07-08 00:00:00','1986-06-03 00:00:00',91.13,327,333,51.62,'Intermediate' from dual union all
select 1675,'001675','1965-082AN','deb Transtage 4','',11832,'Reentered','1979-04-08 00:00:00','1966-01-06 00:00:00',99.55,676,798,33.23,'Intermediate' from dual union all
select 16750,'016750','1982-033GV','EVA debris','',54342,'Reentered','1986-06-19 00:00:00','1986-06-03 00:00:00',91.05,322,331,51.62,'Intermediate' from dual union all
select 16751,'016751','1982-033GW','EVA debris','',54342,'Reentered','1986-10-22 00:00:00','1986-06-03 00:00:00',91.24,331,340,51.62,'Intermediate' from dual union all
select 16752,'016752','1982-033GX','EVA debris','',54342,'Reentered','1986-07-21 00:00:00','1986-06-02 00:00:00',91.2,331,336,51.62,'Intermediate' from dual union all
select 16753,'016753','1986-036C','KBE part','KBE part',59229,'Reentered','1986-06-05 00:00:00','1986-06-04 00:00:00',90.25,217,357,62.51,'Intermediate' from dual union all
select 16754,'016754','1986-036D','KB electropitaniya','KBE',59229,'Reentered','1986-08-18 00:00:00','1986-08-08 00:00:00',88.85,191,245,62.81,'Intermediate' from dual union all
select 16755,'016755','1986-036E','KBE part','KBE part',59229,'Reentered','1986-06-21 00:00:00','1986-06-05 00:00:00',90.09,213,345,62.82,'Intermediate' from dual union all
select 16756,'016756','1986-036F','PTDU cover','KBE part',59229,'Reentered','1986-07-25 00:00:00','1986-06-20 00:00:00',91.7,297,419,62.85,'Intermediate' from dual union all
select 16757,'016757','1986-036G','PTDU cover','KBE part',59229,'Reentered','1986-06-07 00:00:00','1986-06-04 00:00:00',91,217,431,62.82,'Intermediate' from dual union all
select 16758,'016758','1986-042A','Kosmos-1748','Strela-1M',59235,'In Earth orbit','','1986-08-08 00:00:00',115.14,1454,1469,74.02,'Intermediate' from dual union all
select 16759,'016759','1986-042B','Kosmos-1749','Strela-1M',59235,'In Earth orbit','','1986-08-08 00:00:00',114.47,1394,1469,74.01,'Intermediate' from dual union all
select 1676,'001676','1965-082AP','deb Transtage 4','',11832,'Reentered','2004-11-07 00:00:00','1992-01-30 00:00:00',97.42,624,647,32.34,'Intermediate' from dual union all
select 16760,'016760','1986-042C','Kosmos-1750','Strela-1M',59235,'In Earth orbit','','1986-08-08 00:00:00',114.65,1409,1470,74.01,'Intermediate' from dual union all
select 16761,'016761','1986-042D','Kosmos-1751','Strela-1M',59235,'In Earth orbit','','1986-08-08 00:00:00',115.68,1468,1505,74.02,'Intermediate' from dual union all
select 16762,'016762','1986-042E','Kosmos-1752','Strela-1M',59235,'In Earth orbit','','1986-08-08 00:00:00',115.48,1468,1486,74.01,'Intermediate' from dual union all
select 16763,'016763','1986-042F','Kosmos-1753','Strela-1M',59235,'In Earth orbit','','1986-08-08 00:00:00',115.31,1462,1477,74.02,'Intermediate' from dual union all
select 16764,'016764','1986-042G','Kosmos-1754','Strela-1M',59235,'In Earth orbit','','1986-08-08 00:00:00',114.96,1438,1470,74.02,'Intermediate' from dual union all
select 16765,'016765','1986-042H','Kosmos-1755','Strela-1M',59235,'In Earth orbit','','1986-08-08 00:00:00',114.81,1424,1470,74.02,'Intermediate' from dual union all
select 16766,'016766','1986-042J','S3M','S3M',59235,'In Earth orbit','','1986-08-22 00:00:00',117.69,1472,1683,74.01,'Intermediate' from dual union all
select 16767,'016767','1986-043A','Kosmos-1756','Kobal''t',59236,'Landed','1986-08-04 00:00:00','1986-06-13 00:00:00',89.34,169,315,64.87,'Intermediate' from dual union all
select 16768,'016768','1986-043B','11S510','11S510',59236,'Reentered','1986-06-13 00:00:00','1986-06-08 00:00:00',89.31,169,312,64.89,'Intermediate' from dual union all
select 16769,'016769','1986-044A','Gorizont','Gorizont No. 24L',59237,'In Earth orbit','','1986-08-10 00:00:00',1436.09,35765,35808,1.29,'Stationary' from dual union all
select 1677,'001677','1965-082AQ','deb Transtage 4','',11832,'Reentered','1990-06-21 00:00:00','1990-06-17 00:00:00',89.8,261,269,32.15,'Intermediate' from dual union all
select 16770,'016770','1986-044B','8S812','8S812',59237,'Reentered','1986-06-12 00:00:00','1986-06-11 00:00:00',88.15,179,186,51.62,'Intermediate' from dual union all
select 16771,'016771','1986-044C','Perekhodnik','Sredniy perekhodnik',59237,'Reentered','1986-06-11 00:00:00','1986-06-10 00:00:00',88.34,181,204,51.61,'Intermediate' from dual union all
select 16772,'016772','1986-045A','Kosmos-1757','Oblik No. 332 (Priroda)',59238,'Landed','1986-06-25 00:00:00','1986-06-13 00:00:00',90.01,161,389,82.35,'Intermediate' from dual union all
select 16773,'016773','1986-045B','11S510','11S510',59238,'Reentered','1986-06-13 00:00:00','1986-06-12 00:00:00',88.23,173,200,82.33,'Intermediate' from dual union all
select 16774,'016774','1983-044BR','deb Kosmos-1461','',55801,'Reentered','1989-04-11 00:00:00','1985-07-31 00:00:00',99.3,572,879,65.03,'Intermediate' from dual union all
select 16775,'016775','1983-044BS','deb Kosmos-1461','',55801,'Reentered','1989-03-04 00:00:00','1985-06-22 00:00:00',99.33,568,886,65.02,'Intermediate' from dual union all
select 16776,'016776','1983-044BT','deb Kosmos-1461','',55801,'Reentered','1989-12-16 00:00:00','1985-07-31 00:00:00',99.29,569,881,65.03,'Intermediate' from dual union all
select 16777,'016777','1983-044BU','deb Kosmos-1461','',55801,'Reentered','1989-02-12 00:00:00','1986-04-18 00:00:00',99.16,589,847,65.03,'Intermediate' from dual union all
select 16778,'016778','1983-044BV','deb Kosmos-1461','',55801,'Reentered','1989-03-29 00:00:00','1985-07-30 00:00:00',99.34,576,879,65.03,'Intermediate' from dual union all
select 16779,'016779','1983-044BW','deb Kosmos-1461','',55801,'Reentered','1989-01-13 00:00:00','1986-05-30 00:00:00',99.08,583,847,65.02,'Intermediate' from dual union all
select 1678,'001678','1965-082AR','deb Transtage 4','',11832,'Reentered','1980-05-30 00:00:00','1966-02-12 00:00:00',101.1,732,890,32.03,'Intermediate' from dual union all
select 16780,'016780','1983-044BX','deb Kosmos-1461','',55801,'Reentered','1990-07-02 00:00:00','1990-07-02 00:00:00',88.34,183,202,64.94,'Intermediate' from dual union all
select 16781,'016781','1983-044BY','deb Kosmos-1461','',55801,'Reentered','1988-12-10 00:00:00','1986-04-19 00:00:00',99.09,584,847,65.03,'Intermediate' from dual union all
select 16782,'016782','1983-044BZ','deb Kosmos-1461','Medium debris, RCS 1m2',55801,'Reentered','1990-06-22 00:00:00','1990-06-22 00:00:00',88,164,186,64.97,'Intermediate' from dual union all
select 16783,'016783','1986-040D','KDU part','KDU part',59233,'Reentered','1986-06-13 00:00:00','1986-06-11 00:00:00',88.92,207,236,82.3,'Intermediate' from dual union all
select 16784,'016784','1986-040E','KDU part','KDU part',59233,'Reentered','1986-06-13 00:00:00','1986-06-11 00:00:00',89.39,236,252,82.33,'Intermediate' from dual union all
select 16785,'016785','1986-040F','Blenda','Sunshade',59233,'Reentered','1986-06-14 00:00:00','1986-06-11 00:00:00',89.75,241,284,82.36,'Intermediate' from dual union all
select 16786,'016786','1986-040G','KDU part','KDU part',59233,'Reentered','1986-06-13 00:00:00','1986-06-11 00:00:00',89.78,255,273,82.3,'Intermediate' from dual union all
select 16787,'016787','1986-040H','KDU','KDU',59233,'Reentered','1986-07-07 00:00:00','1986-06-11 00:00:00',91.71,261,456,82.34,'Intermediate' from dual union all
select 16788,'016788','1986-040J','KDU part','KDU part',59233,'Reentered','1986-06-23 00:00:00','1986-06-11 00:00:00',92,263,483,82.34,'Intermediate' from dual union all
select 16789,'016789','1986-044D','SOZ','SOZ',59237,'Reentered','1990-03-07 00:00:00','1986-08-21 00:00:00',648.66,286,36602,47.06,'GEO Transfer' from dual union all
select 1679,'001679','1965-082AS','deb Transtage 4','',11832,'Reentered','1980-05-24 00:00:00','1966-02-08 00:00:00',100.68,677,905,32.21,'Intermediate' from dual union all
select 16790,'016790','1986-044E','SOZ','SOZ',59237,'Reentered','1987-08-12 00:00:00','1986-08-24 00:00:00',647.89,240,36609,46.99,'GEO Transfer' from dual union all
select 16791,'016791','1986-046A','Kosmos-1758','Tselina-D',59239,'In Earth orbit','','1986-08-07 00:00:00',97.71,628,671,82.48,'Intermediate' from dual union all
select 16792,'016792','1986-046B','S5M','S5M',59239,'In Earth orbit','','1986-08-20 00:00:00',97.68,627,669,82.48,'Intermediate' from dual union all
select 16793,'016793','1986-041C','Blenda','Sunshade',59234,'Reentered','1986-06-22 00:00:00','1986-06-13 00:00:00',90.93,243,398,70.36,'Intermediate' from dual union all
select 16794,'016794','1986-041D','KDU','KDU',59234,'Reentered','1986-06-25 00:00:00','1986-06-13 00:00:00',89.04,216,239,70.35,'Intermediate' from dual union all
select 16795,'016795','1986-041E','KDU part','KDU part',59234,'Reentered','1986-06-14 00:00:00','1986-06-13 00:00:00',88.93,210,234,70.35,'Intermediate' from dual union all
select 16796,'016796','1986-041F','KDU part','KDU part',59234,'Reentered','1986-06-15 00:00:00','1986-06-13 00:00:00',88.79,205,225,70.35,'Intermediate' from dual union all
select 16797,'016797','1986-044F','Blok DM No. 60L','11S86 No. 60L',59237,'In Earth orbit','','1986-08-17 00:00:00',1474.29,36503,36559,1.28,'Drift GEO' from dual union all
select 16798,'016798','1986-047A','Kosmos-1759','Parus',59240,'In Earth orbit','','1986-08-08 00:00:00',104.8,967,1003,82.92,'Intermediate' from dual union all
select 16799,'016799','1986-047B','S3M','S3M',59240,'In Earth orbit','','1986-07-18 00:00:00',104.68,965,994,82.92,'Intermediate' from dual union all
select 168,'000168','61 Tau 1','Vostok-2','Vostok-3A No. 4',7279,'Landed','1961-08-07 00:00:00','1961-08-06 00:00:00',88.39,172,218,64.87,'Intermediate' from dual union all
select 1680,'001680','1965-082AT','Transtage 4 large debris','RCS 3.0',11832,'In Earth orbit','','1991-05-11 00:00:00',98.81,684,720,32.26,'Intermediate' from dual union all
select 16800,'016800','1986-048A','Kosmos-1760','Oblik',59241,'Landed','1986-07-03 00:00:00','1986-06-21 00:00:00',92.16,347,414,69.99,'Intermediate' from dual union all
select 16801,'016801','1986-048B','11S510','11S510',59241,'Reentered','1986-07-19 00:00:00','1986-06-22 00:00:00',90.31,207,373,69.99,'Intermediate' from dual union all
select 16802,'016802','1986-049A','Molniya-3','Molniya-3 No. 44',59242,'Reentered','1996-11-10 00:00:00','1986-08-24 00:00:00',717.78,666,39689,62.89,'Molniya' from dual union all
select 16803,'016803','1986-049B','11S510','11S510',59242,'Reentered','1986-09-27 00:00:00','1986-08-08 00:00:00',91.38,215,469,62.84,'Intermediate' from dual union all
select 16804,'016804','1986-049C','BOZ','BOZ',59242,'Reentered','1986-10-01 00:00:00','1986-08-07 00:00:00',91.62,208,500,62.84,'Intermediate' from dual union all
select 16805,'016805','1986-049D','Blok-ML','Blok-ML No. 71614-521',59242,'Reentered','1996-12-21 00:00:00','1986-08-23 00:00:00',734.28,653,40513,62.9,'Molniya' from dual union all
select 16806,'016806','1986-024B','Kosmos-1736 DU','US-A DU',59217,'Reentered','1986-09-01 00:00:00','1986-06-21 00:00:00',89.56,243,263,64.99,'Intermediate' from dual union all
select 16807,'016807','1986-024C','Kosmos-1736 RLS','US-A RLS',59217,'Reentered','1986-06-27 00:00:00','1986-06-21 00:00:00',89.47,235,262,64.99,'Intermediate' from dual union all
select 16808,'016808','1986-024D','deb Kosmos-1736 DU','',59217,'Reentered','1986-06-26 00:00:00','1986-06-21 00:00:00',89.57,236,271,64.97,'Intermediate' from dual union all
select 16809,'016809','1986-024E','Kosmos-1736 reactor core','US-A reactor core',59217,'In Earth orbit','','1986-06-21 00:00:00',104.23,924,993,64.99,'Intermediate' from dual union all
select 1681,'001681','1965-082AU','deb Transtage 4','',11832,'Reentered','1991-08-13 00:00:00','1991-05-19 00:00:00',92.64,393,415,32.16,'Intermediate' from dual union all
select 16810,'016810','1982-033GY','Garbage bag','',54342,'Reentered','1986-09-05 00:00:00','1986-08-22 00:00:00',90.21,257,313,51.6,'Intermediate' from dual union all
select 16811,'016811','1982-033GZ','Garbage bag','',54342,'Reentered','1986-08-31 00:00:00','1986-08-05 00:00:00',90.45,291,303,51.62,'Intermediate' from dual union all
select 16812,'016812','1982-033HA','Garbage bag','',54342,'Reentered','1986-09-09 00:00:00','1986-06-24 00:00:00',91.21,329,339,51.62,'Intermediate' from dual union all
select 16813,'016813','1982-033HB','Garbage bag','',54342,'Reentered','1987-01-27 00:00:00','1986-07-02 00:00:00',91.27,332,342,51.62,'Intermediate' from dual union all
select 16814,'016814','1982-033HC','Garbage bag','',54342,'Reentered','1987-02-20 00:00:00','1986-08-22 00:00:00',91.12,329,330,51.62,'Intermediate' from dual union all
select 16815,'016815','1982-033HD','Garbage bag','',54342,'Reentered','1986-10-10 00:00:00','1986-08-07 00:00:00',90.83,315,316,51.64,'Intermediate' from dual union all
select 16816,'016816','1986-045C','Blenda','Sunshade',59238,'Reentered','1986-06-27 00:00:00','1986-06-25 00:00:00',89.62,152,360,82.33,'Intermediate' from dual union all
select 16817,'016817','1986-045D','KDU','KDU',59238,'Reentered','1986-07-05 00:00:00','1986-06-25 00:00:00',89.88,164,373,82.34,'Intermediate' from dual union all
select 16818,'016818','1986-045E','KDU part','KDU part',59238,'Reentered','1986-06-27 00:00:00','1986-06-25 00:00:00',89.69,164,355,82.33,'Intermediate' from dual union all
select 16819,'016819','1986-045F','KDU part','KDU part',59238,'Reentered','1986-07-09 00:00:00','1986-06-25 00:00:00',91.89,243,491,82.34,'Intermediate' from dual union all
select 1682,'001682','1965-082AV','deb Transtage 4','',11832,'In Earth orbit','','1991-05-24 00:00:00',99.61,683,797,32.41,'Intermediate' from dual union all
select 16820,'016820','1986-045G','KDU part','KDU part',59238,'Reentered','1986-06-29 00:00:00','1986-06-25 00:00:00',96.92,281,942,82.35,'Intermediate' from dual union all
select 16821,'016821','1982-033HE','Garbage bag','',54342,'Reentered','1986-08-27 00:00:00','1986-08-07 00:00:00',90.29,257,321,51.62,'Intermediate' from dual union all
select 16822,'016822','1982-033HF','Garbage bag','',54342,'Reentered','1986-10-08 00:00:00','1986-08-07 00:00:00',90.83,311,320,51.63,'Intermediate' from dual union all
select 16823,'016823','1986-024F','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-10-23 00:00:00','1986-08-24 00:00:00',99.54,587,887,65,'Intermediate' from dual union all
select 16824,'016824','1986-024G','deb Kosmos-1736 ROCh','',59217,'Reentered','1987-08-10 00:00:00','1986-08-08 00:00:00',103.07,697,1111,65.02,'Intermediate' from dual union all
select 16825,'016825','1986-024H','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-08-31 00:00:00','1986-06-27 00:00:00',99.74,551,941,65.02,'Intermediate' from dual union all
select 16826,'016826','1986-024J','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-12-04 00:00:00','1986-11-23 00:00:00',96.26,486,673,65.05,'Intermediate' from dual union all
select 16827,'016827','1986-024K','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-10-31 00:00:00','1986-08-07 00:00:00',100.02,686,833,65.07,'Intermediate' from dual union all
select 16828,'016828','1986-024L','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-12-16 00:00:00','1986-08-06 00:00:00',99.65,670,814,65.02,'Intermediate' from dual union all
select 16829,'016829','1986-024M','deb Kosmos-1736 ROCh','',59217,'Reentered','1987-07-26 00:00:00','1986-08-21 00:00:00',103.35,594,1240,65.08,'Intermediate' from dual union all
select 1683,'001683','1965-082AW','deb Transtage 4','',11832,'Reentered','1966-07-11 00:00:00','1966-01-29 00:00:00',97.22,512,740,32.7,'Intermediate' from dual union all
select 16830,'016830','1986-024N','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-07-04 00:00:00','1986-06-27 00:00:00',96.95,400,826,65.07,'Intermediate' from dual union all
select 16831,'016831','1986-024P','deb Kosmos-1736 ROCh','',59217,'Reentered','1987-03-18 00:00:00','1986-06-28 00:00:00',102.68,763,1007,65.02,'Intermediate' from dual union all
select 16832,'016832','1986-024Q','deb Kosmos-1736 ROCh','',59217,'Reentered','1987-08-15 00:00:00','1986-08-14 00:00:00',102.38,759,984,65.02,'Intermediate' from dual union all
select 16833,'016833','1986-024R','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-11-06 00:00:00','1986-09-12 00:00:00',99.04,510,916,65.1,'Intermediate' from dual union all
select 16834,'016834','1986-024S','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-11-04 00:00:00','1986-08-21 00:00:00',100.31,661,886,65.03,'Intermediate' from dual union all
select 16835,'016835','1986-024T','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-10-23 00:00:00','1986-06-27 00:00:00',95.44,382,698,65.01,'Intermediate' from dual union all
select 16836,'016836','1986-024U','deb Kosmos-1736 ROCh','',59217,'Reentered','1987-03-06 00:00:00','1986-08-15 00:00:00',102.64,633,1135,65.05,'Intermediate' from dual union all
select 16837,'016837','1986-024V','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-07-14 00:00:00','1986-06-29 00:00:00',95.02,421,619,65,'Intermediate' from dual union all
select 16838,'016838','1986-024W','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-09-10 00:00:00','1986-06-27 00:00:00',98.67,493,898,65,'Intermediate' from dual;
insert into SATELLITE
select 16839,'016839','1986-024X','deb Kosmos-1736 ROCh','',59217,'Reentered','1987-02-28 00:00:00','1986-08-04 00:00:00',102.96,664,1134,65.02,'Intermediate' from dual union all
select 1684,'001684','1965-082AX','deb Transtage 4','',11832,'In Earth orbit','','1991-05-09 00:00:00',101.85,678,1015,32.22,'Intermediate' from dual union all
select 16840,'016840','1986-024Y','deb Kosmos-1736 ROCh','',59217,'Reentered','1987-02-24 00:00:00','1986-08-22 00:00:00',100.54,776,793,65.01,'Intermediate' from dual union all
select 16841,'016841','1986-024Z','deb Kosmos-1736 ROCh','',59217,'Reentered','1988-01-22 00:00:00','1986-08-04 00:00:00',102.59,862,901,65.01,'Intermediate' from dual union all
select 16842,'016842','1986-024AA','deb Kosmos-1736 ROCh','Large RCS',59217,'Reentered','1986-11-15 00:00:00','1986-08-09 00:00:00',100.46,678,883,65.07,'Intermediate' from dual union all
select 16843,'016843','1979-017JD','P78-1 debris','',47501,'Reentered','1988-07-23 00:00:00','1986-08-08 00:00:00',95.06,501,542,97.77,'Sun-Synch' from dual union all
select 16844,'016844','1986-048C','KDU','KDU',59241,'Reentered','1988-03-24 00:00:00','1986-08-15 00:00:00',92.16,351,410,69.99,'Intermediate' from dual union all
select 16845,'016845','1986-048D','PTDU cover','KDU part',59241,'Reentered','1987-03-23 00:00:00','1986-08-08 00:00:00',94.02,374,569,69.99,'Intermediate' from dual union all
select 16846,'016846','1986-048E','Blenda','Sunshade',59241,'Reentered','1987-03-17 00:00:00','1986-08-09 00:00:00',91.92,335,403,69.99,'Intermediate' from dual union all
select 16847,'016847','1986-048F','KDU deb','',59241,'Reentered','1986-07-27 00:00:00','1986-08-09 00:00:00',92.94,372,465,69.93,'Intermediate' from dual union all
select 16848,'016848','1986-048G','PTDU cover','KDU part',59241,'Reentered','1986-12-26 00:00:00','1986-08-09 00:00:00',94.81,390,629,70,'Intermediate' from dual union all
select 16849,'016849','1986-050A','Kosmos-1761','Oko',59243,'In Earth orbit','','1986-09-03 00:00:00',717.92,646,39716,62.94,'Molniya' from dual union all
select 1685,'001685','1965-082AY','deb Transtage 4','',11832,'Reentered','2014-04-01 00:00:00','1992-01-30 00:00:00',97.62,634,656,32.3,'Intermediate' from dual union all
select 16850,'016850','1986-050B','11S510','11S510',59243,'Reentered','1986-09-19 00:00:00','1986-08-08 00:00:00',91.45,210,482,62.83,'Intermediate' from dual union all
select 16851,'016851','1986-050C','BOZ','BOZ',59243,'Reentered','1986-08-16 00:00:00','1986-08-08 00:00:00',89.87,173,363,62.84,'Intermediate' from dual union all
select 16852,'016852','1982-033HG','Garbage bag','',54342,'Reentered','1986-11-21 00:00:00','1986-08-25 00:00:00',90.95,319,324,51.62,'Intermediate' from dual union all
select 16853,'016853','1982-033HH','Garbage bag','',54342,'Reentered','1986-09-06 00:00:00','1986-08-05 00:00:00',90.71,267,351,51.64,'Intermediate' from dual union all
select 16854,'016854','1986-050D','Blok-2BL','Blok-2BL No. 71646-667',59243,'In Earth orbit','','1986-09-09 00:00:00',710.05,713,39260,62.91,'Molniya' from dual union all
select 16855,'016855','1986-051A','Kosmos-1762','Resurs-F1 14F40 No. 59',59244,'Landed','1986-07-24 00:00:00','1986-07-11 00:00:00',89.81,258,273,82.55,'Intermediate' from dual union all
select 16856,'016856','1986-051B','11S510','11S510',59244,'Reentered','1986-07-15 00:00:00','1986-07-10 00:00:00',88.96,183,264,82.55,'Intermediate' from dual union all
select 16857,'016857','1986-024AB','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-11-14 00:00:00','1986-08-07 00:00:00',96.97,472,756,65,'Intermediate' from dual union all
select 16858,'016858','1986-024AC','deb Kosmos-1736 ROCh','Large RCS',59217,'Reentered','1987-02-28 00:00:00','1986-08-05 00:00:00',101.59,792,876,65.02,'Intermediate' from dual union all
select 16859,'016859','1986-024AD','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-12-16 00:00:00','1986-08-22 00:00:00',100.17,625,909,65.08,'Intermediate' from dual union all
select 1686,'001686','1965-082AZ','deb Transtage 4','',11832,'Reentered','1997-02-27 00:00:00','1992-01-31 00:00:00',95.99,557,576,32.44,'Intermediate' from dual union all
select 16860,'016860','1986-052A','Kosmos-1763','Strela-2M',59245,'In Earth orbit','','1986-08-21 00:00:00',100.47,756,805,74.04,'Intermediate' from dual union all
select 16861,'016861','1986-053A','Kosmos-1764','Kobal''t',59246,'Landed','1986-09-11 00:00:00','1986-08-07 00:00:00',89.54,170,334,64.9,'Intermediate' from dual union all
select 16862,'016862','1986-053B','11S510','11S510',59246,'Reentered','1986-07-26 00:00:00','1986-08-07 00:00:00',89.63,174,339,64.91,'Intermediate' from dual union all
select 16863,'016863','1986-022C','Soyuz T-15 BO','Bitovoy otsek',59215,'Reentered','1987-07-25 00:00:00','1986-08-07 00:00:00',91.36,325,358,51.62,'Intermediate' from dual union all
select 16864,'016864','1986-052B','S3M','S3M',59245,'In Earth orbit','','1986-08-22 00:00:00',100.47,756,805,74.03,'Intermediate' from dual union all
select 16865,'016865','1986-052C','deb Kosmos-1763 (large RCS)','',59245,'In Earth orbit','','1986-08-21 00:00:00',100.55,758,812,74.04,'Intermediate' from dual union all
select 16866,'016866','1986-052D','deb Kosmos-1763 (large RCS)','',59245,'Reentered','2016-01-03 00:00:00','1986-08-23 00:00:00',100.52,757,810,74.03,'Intermediate' from dual union all
select 16867,'016867','1986-052E','deb Kosmos-1763','',59245,'In Earth orbit','','1986-08-24 00:00:00',100.5,758,806,74.04,'Intermediate' from dual union all
select 16868,'016868','1986-024AE','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-11-14 00:00:00','1986-08-08 00:00:00',101.27,691,946,65.02,'Intermediate' from dual union all
select 16869,'016869','1980-104F','SOZ','SOZ',50314,'Reentered','1986-10-26 00:00:00','1983-05-22 00:00:00',351.47,171,20068,46.64,'Highly Ellip' from dual union all
select 1687,'001687','1965-082BA','deb Transtage 4','',11832,'Reentered','2003-02-12 00:00:00','1992-01-31 00:00:00',98.66,650,740,32.03,'Intermediate' from dual union all
select 16870,'016870','1986-007F','Blok DM','11S86 No. 57L',59200,'In Earth orbit','','1986-07-11 00:00:00',1472.33,36369,36617,.83,'Drift GEO' from dual union all
select 16871,'016871','1986-051C','Blenda','Sunshade',59244,'Reentered','1986-08-21 00:00:00','1986-08-08 00:00:00',91.01,250,399,82.54,'Intermediate' from dual union all
select 16872,'016872','1986-051D','KDU part','KDU part',59244,'Reentered','1986-08-03 00:00:00','1986-07-24 00:00:00',89.61,241,270,82.54,'Intermediate' from dual union all
select 16873,'016873','1986-051E','KDU part','KDU part',59244,'Reentered','1986-07-31 00:00:00','1986-07-24 00:00:00',89.75,252,273,82.56,'Intermediate' from dual union all
select 16874,'016874','1986-054A','Kosmos-1765','Oblik No. 334',59247,'Landed','1986-08-07 00:00:00','1986-08-02 00:00:00',92.25,356,413,72.86,'Intermediate' from dual union all
select 16875,'016875','1986-054B','11S510','11S510',59247,'Reentered','1986-08-08 00:00:00','1986-08-07 00:00:00',88.18,163,206,72.85,'Intermediate' from dual union all
select 16876,'016876','1986-051F','KDU','KDU',59244,'Reentered','1986-09-22 00:00:00','1986-08-09 00:00:00',89.6,247,263,82.54,'Intermediate' from dual union all
select 16877,'016877','1979-017JE','P78-1 debris','',47501,'Reentered','1988-03-04 00:00:00','1986-04-12 00:00:00',94.53,464,528,97.72,'Sun-Synch' from dual union all
select 16878,'016878','1979-017JF','P78-1 debris','',47501,'Reentered','2001-11-29 00:00:00','1993-08-09 00:00:00',93.88,456,473,97.87,'Sun-Synch' from dual union all
select 16879,'016879','1979-017JG','P78-1 debris','',47501,'Reentered','1987-03-11 00:00:00','1986-08-07 00:00:00',94.08,460,488,97.74,'Sun-Synch' from dual union all
select 1688,'001688','1965-082BB','deb Transtage 4','',11832,'Reentered','1981-05-01 00:00:00','1966-01-23 00:00:00',102.64,735,1032,32.59,'Intermediate' from dual union all
select 16880,'016880','1986-051G','KDU part','KDU part',59244,'Reentered','1986-08-05 00:00:00','1986-07-26 00:00:00',91.88,259,474,82.56,'Intermediate' from dual union all
select 16881,'016881','1986-055A','Kosmos-1766','Okean-O1 NKhM No. 3',59248,'In Earth orbit','','1987-01-29 00:00:00',97.71,632,667,82.53,'Intermediate' from dual union all
select 16882,'016882','1986-055B','S5M','S5M',59248,'In Earth orbit','','1986-09-02 00:00:00',97.69,632,665,82.52,'Intermediate' from dual union all
select 16883,'016883','1986-056A','Kosmos-1767','EPN No. 03.695',59249,'Reentered','1986-08-16 00:00:00','1986-08-08 00:00:00',88.2,180,190,64.88,'Intermediate' from dual union all
select 16884,'016884','1986-056B','11S772','11S772',59249,'Reentered','1986-08-04 00:00:00','1986-07-30 00:00:00',88.42,190,203,64.87,'Intermediate' from dual union all
select 16885,'016885','1986-057A','Molniya-1','Molniya-1 No. 59',59250,'Reentered','2000-01-28 00:00:00','1986-09-30 00:00:00',717.7,599,39752,62.85,'Molniya' from dual union all
select 16886,'016886','1986-057B','11S510','11S510',59250,'Reentered','1986-10-19 00:00:00','1986-08-29 00:00:00',91.73,209,510,62.81,'Intermediate' from dual union all
select 16887,'016887','1986-057C','BOZ','BOZ',59250,'Reentered','1986-10-05 00:00:00','1986-08-30 00:00:00',91.57,196,508,62.81,'Intermediate' from dual union all
select 16888,'016888','1986-056C','Sep Motor Cover','Sep Motor Cover',59249,'Reentered','1986-08-18 00:00:00','1986-08-08 00:00:00',90.29,190,388,64.96,'Intermediate' from dual union all
select 16889,'016889','1986-057D','Blok-ML','Blok-ML',59250,'Reentered','1996-10-14 00:00:00','1986-09-29 00:00:00',732.79,616,40477,62.86,'Molniya' from dual union all
select 1689,'001689','1965-082BC','deb Transtage 4','',11832,'Reentered','1978-12-29 00:00:00','1966-01-30 00:00:00',100.08,727,798,32.64,'Intermediate' from dual union all
select 16890,'016890','1986-058A','Kosmos-1768','Resurs-F1 14F40 No. 60',59251,'Landed','1986-08-16 00:00:00','1986-08-07 00:00:00',89.8,258,272,82.58,'Intermediate' from dual union all
select 16891,'016891','1986-058B','11S510','11S510',59251,'Reentered','1986-08-07 00:00:00','1986-08-02 00:00:00',88.83,180,254,82.57,'Intermediate' from dual union all
select 16892,'016892','1986-056D','Sep Motor Cover','Sep Motor Cover',59249,'Reentered','1986-08-11 00:00:00','1986-08-08 00:00:00',89.42,178,314,64.8,'Intermediate' from dual union all
select 16893,'016893','1986-056E','Sep Motor Cover','Sep Motor Cover',59249,'Reentered','1986-08-12 00:00:00','1986-08-08 00:00:00',89.53,178,325,64.96,'Intermediate' from dual union all
select 16894,'016894','1986-056F','Sep Motor Cover','Sep Motor Cover',59249,'Reentered','1986-08-20 00:00:00','1986-08-08 00:00:00',90.44,193,400,64.8,'Intermediate' from dual union all
select 16895,'016895','1986-059A','Kosmos-1769','US-PM1',59252,'Deorbited','1988-02-18 00:00:00','1986-09-03 00:00:00',93.28,429,441,65.02,'Intermediate' from dual union all
select 16896,'016896','1986-059B','11S692','11S692',59252,'Reentered','1986-08-04 00:00:00','',89.25,110,365,65,'Intermediate' from dual union all
select 16897,'016897','1986-060A','Kosmos-1770','Terilen',59253,'Deorbited','1987-02-02 00:00:00','1986-09-05 00:00:00',89.88,224,313,64.75,'Intermediate' from dual union all
select 16898,'016898','1986-060B','11S510','11S510',59253,'Reentered','1986-08-12 00:00:00','1986-08-08 00:00:00',88.9,178,263,64.77,'Intermediate' from dual union all
select 16899,'016899','1986-054C','KDU part','KDU part',59247,'Reentered','1986-11-06 00:00:00','1986-08-23 00:00:00',92.17,352,410,72.85,'Intermediate' from dual union all
select 169,'000169','61 Tau 2','8K72KE','8K72KE',7279,'Reentered','1961-08-09 00:00:00','1961-08-08 00:00:00',88.2,180,190,64.91,'Intermediate' from dual union all
select 1690,'001690','1965-082BD','deb Transtage 4','',11832,'Reentered','1981-06-11 00:00:00','1965-11-15 00:00:00',98.43,626,742,32.35,'Intermediate' from dual union all
select 16900,'016900','1986-054D','KDU part','KDU part',59247,'Reentered','1987-05-14 00:00:00','1986-08-23 00:00:00',94.41,389,591,72.86,'Intermediate' from dual union all
select 16901,'016901','1986-054E','KDU part','KDU part',59247,'Reentered','1986-09-23 00:00:00','1986-08-23 00:00:00',92.2,368,397,72.72,'Intermediate' from dual union all
select 16902,'016902','1986-054F','Blenda','Sunshade',59247,'Reentered','1987-03-20 00:00:00','1986-08-23 00:00:00',92.04,342,408,72.85,'Intermediate' from dual union all
select 16903,'016903','1986-054G','KDU part','KDU part',59247,'Reentered','1986-09-18 00:00:00','1986-08-20 00:00:00',92.06,351,400,72.8,'Intermediate' from dual union all
select 16904,'016904','1986-054H','KDU part','KDU part',59247,'Reentered','1986-09-14 00:00:00','1986-08-15 00:00:00',92.14,359,400,72.94,'Intermediate' from dual union all
select 16905,'016905','1986-054J','KDU','KDU',59247,'Reentered','1988-05-12 00:00:00','1986-08-23 00:00:00',92.23,355,413,72.85,'Intermediate' from dual union all
select 16906,'016906','1986-054K','KDU part','KDU part',59247,'Reentered','1986-09-11 00:00:00','1986-08-14 00:00:00',92.07,351,402,72.86,'Intermediate' from dual union all
select 16907,'016907','1986-054L','KDU part','KDU part',59247,'Reentered','1986-09-14 00:00:00','1986-08-15 00:00:00',92.06,346,406,72.85,'Intermediate' from dual union all
select 16908,'016908','1986-061A','Ajisai','EGP',59254,'In Earth orbit','','1986-09-23 00:00:00',115.71,1479,1496,50.01,'Intermediate' from dual union all
select 16909,'016909','1986-061B','Fuji-OSCAR-12','JAS-1',59254,'In Earth orbit','','1986-09-15 00:00:00',115.7,1479,1496,50.02,'Intermediate' from dual union all
select 1691,'001691','1965-082BE','deb Transtage 4','',11832,'Reentered','1971-03-12 00:00:00','1966-01-08 00:00:00',98.3,601,754,32.35,'Intermediate' from dual union all
select 16910,'016910','1986-061C','Jindai','MABES',59254,'In Earth orbit','','1986-09-23 00:00:00',116.83,1483,1594,50.01,'Intermediate' from dual union all
select 16911,'016911','1986-058C','KDU','KDU',59251,'Reentered','1986-10-19 00:00:00','1986-09-01 00:00:00',89.71,256,265,82.58,'Intermediate' from dual union all
select 16912,'016912','1986-058D','Blenda','Sunshade',59251,'Reentered','1986-09-16 00:00:00','1986-08-23 00:00:00',91.56,263,440,82.58,'Intermediate' from dual union all
select 16913,'016913','1986-058E','KDU part','KDU part',59251,'Reentered','1986-08-24 00:00:00','1986-08-21 00:00:00',89.42,240,252,82.57,'Intermediate' from dual union all
select 16914,'016914','1986-058F','KDU part','KDU part',59251,'Reentered','1986-08-17 00:00:00','1986-08-02 00:00:00',88.96,187,260,82.58,'Intermediate' from dual union all
select 16915,'016915','1986-058G','KDU part','KDU part',59251,'Reentered','1986-08-29 00:00:00','1986-08-21 00:00:00',89.5,240,260,82.57,'Intermediate' from dual union all
select 16916,'016916','1986-058H','KDU part','KDU part',59251,'Reentered','1986-08-31 00:00:00','1986-08-20 00:00:00',92.16,260,501,82.55,'Intermediate' from dual union all
select 16917,'016917','1986-062A','Kosmos-1771','US-A reactor',59255,'In Earth orbit','','1986-09-20 00:00:00',89.63,250,262,65,'Intermediate' from dual union all
select 16918,'016918','1986-063A','Kosmos-1772','Oblik No. 336',59256,'Landed','1986-09-03 00:00:00','1986-08-23 00:00:00',92.25,355,415,72.86,'Intermediate' from dual union all
select 16919,'016919','1986-063B','11S510','11S510',59256,'Reentered','1986-09-02 00:00:00','1986-08-23 00:00:00',89.61,194,317,72.85,'Intermediate' from dual union all
select 1692,'001692','1965-082BF','deb Transtage 4','',11832,'Reentered','1970-09-22 00:00:00','1966-01-28 00:00:00',98.29,616,737,33.27,'Intermediate' from dual union all
select 16920,'016920','1986-064A','Kosmos-1773','Kobal''t',59257,'Landed','1986-10-21 00:00:00','1986-09-03 00:00:00',89.33,171,312,64.87,'Intermediate' from dual union all
select 16921,'016921','1986-064B','11S510','11S510',59257,'Reentered','1986-09-02 00:00:00','1986-08-30 00:00:00',88.96,165,281,64.88,'Intermediate' from dual union all
select 16922,'016922','1986-065A','Kosmos-1774','Oko',59258,'Reentered','2010-11-02 00:00:00','1986-09-29 00:00:00',717.52,623,39719,63.06,'Molniya' from dual union all
select 16923,'016923','1986-065B','11S510','11S510',59258,'Reentered','1986-10-22 00:00:00','1986-09-04 00:00:00',91.71,210,507,62.84,'Intermediate' from dual union all
select 16924,'016924','1986-065C','BOZ','BOZ',59258,'Reentered','1986-10-01 00:00:00','1986-09-04 00:00:00',91.88,185,548,62.87,'Intermediate' from dual union all
select 16925,'016925','1986-065D','Blok-2BL','Blok-2BL No. 76711-669',59258,'Reentered','2006-12-28 00:00:00','1986-09-27 00:00:00',707,693,39129,63.14,'Molniya' from dual union all
select 16926,'016926','1986-066A','Kosmos-1775','Oblik',59259,'Landed','1986-09-17 00:00:00','1986-09-05 00:00:00',92.16,347,414,70.36,'Intermediate' from dual union all
select 16927,'016927','1986-066B','11S510','11S510',59259,'Reentered','1986-09-25 00:00:00','1986-09-10 00:00:00',89.98,202,345,70.36,'Intermediate' from dual union all
select 16928,'016928','1986-067A','Kosmos-1776','Romb',59260,'Reentered','1989-12-15 00:00:00','1986-10-03 00:00:00',94.5,473,516,74.02,'Intermediate' from dual;
insert into SATELLITE
select 16929,'016929','1986-067B','S3M','S3M',59260,'Reentered','1989-10-26 00:00:00','1986-10-03 00:00:00',94.42,465,516,74.04,'Intermediate' from dual union all
select 1693,'001693','1965-082BG','deb Transtage 4','',11832,'Reentered','2006-01-02 00:00:00','1992-01-31 00:00:00',97.46,619,656,32.46,'Intermediate' from dual union all
select 16930,'016930','1986-063C','Blenda','Sunshade',59256,'Reentered','1987-04-09 00:00:00','1986-09-29 00:00:00',91.98,341,403,72.86,'Intermediate' from dual union all
select 16931,'016931','1986-063D','KDU','KDU',59256,'Reentered','1988-05-13 00:00:00','1986-09-20 00:00:00',92.22,355,412,72.86,'Intermediate' from dual union all
select 16932,'016932','1986-063E','KDU part','KDU part',59256,'Reentered','1987-01-07 00:00:00','1986-09-20 00:00:00',92.19,354,409,72.86,'Intermediate' from dual union all
select 16933,'016933','1986-063F','KDU part','KDU part',59256,'Reentered','1987-06-01 00:00:00','1986-09-29 00:00:00',94.2,387,573,72.86,'Intermediate' from dual union all
select 16934,'016934','1986-068A','Molniya-1','Molniya-1 No. 57',59261,'Reentered','2000-06-20 00:00:00','1986-10-07 00:00:00',717.74,623,39730,62.91,'Molniya' from dual union all
select 16935,'016935','1986-068B','BOZ','BOZ',59261,'Reentered','1986-10-12 00:00:00','1986-09-12 00:00:00',92.24,184,584,62.84,'Intermediate' from dual union all
select 16936,'016936','1986-068C','11S510','11S510',59261,'Reentered','1986-11-06 00:00:00','1986-10-10 00:00:00',91.05,204,449,62.78,'Intermediate' from dual union all
select 16937,'016937','1986-069A','USA 19','Payload Adapter System',59262,'Reentered','1986-09-28 00:00:00','1986-09-12 00:00:00',93.56,207,691,39.06,'Intermediate' from dual union all
select 16938,'016938','1986-069B','Delta 180 Sensor Module','DM 43 Sensor Module',59262,'Reentered Attached','1986-11-25 00:00:00','1986-10-05 00:00:00',92.09,215,538,22.78,'Equatorial' from dual union all
select 16939,'016939','1986-068D','Blok-ML','Blok-ML',59261,'Reentered','2001-07-19 00:00:00','1986-10-18 00:00:00',731.24,624,40393,62.99,'Molniya' from dual union all
select 1694,'001694','1965-082BH','deb Transtage 4','',11832,'Reentered','1970-10-26 00:00:00','1966-01-23 00:00:00',98.2,578,768,33.24,'Intermediate' from dual union all
select 16940,'016940','1986-069C','deb PAS','',59262,'Reentered','1986-09-24 00:00:00','1986-09-09 00:00:00',91.52,213,486,38.79,'Intermediate' from dual union all
select 16941,'016941','1986-069D','deb PAS','',59262,'Reentered','1987-04-14 00:00:00','1986-09-10 00:00:00',103.25,214,1611,39.19,'Intermediate' from dual union all
select 16942,'016942','1986-069E','deb PAS','',59262,'Reentered','1986-11-10 00:00:00','1986-09-10 00:00:00',126.09,192,3711,37.88,'Medium' from dual union all
select 16943,'016943','1986-069F','deb PAS','',59262,'Reentered','1986-12-11 00:00:00','1986-09-09 00:00:00',96.65,217,980,39.39,'Intermediate' from dual union all
select 16944,'016944','1986-069G','deb PAS','',59262,'Reentered','1986-10-23 00:00:00','1986-09-09 00:00:00',97.08,215,1022,40,'Intermediate' from dual union all
select 16945,'016945','1986-069H','deb PAS','',59262,'Reentered','1986-10-31 00:00:00','1986-09-08 00:00:00',110.83,212,2318,39.47,'Intermediate' from dual union all
select 16946,'016946','1986-069J','deb Delta 180','',59262,'Reentered','1987-03-30 00:00:00','1986-09-05 00:00:00',102.58,216,1545,23.03,'Equatorial' from dual union all
select 16947,'016947','1986-069K','deb Delta 180','',59262,'Reentered','1986-09-16 00:00:00','1986-09-09 00:00:00',90.25,222,351,22.68,'Equatorial' from dual union all
select 16948,'016948','1986-069L','deb Delta 180','',59262,'Reentered','1986-11-15 00:00:00','1986-09-08 00:00:00',91.46,208,485,22.67,'Equatorial' from dual union all
select 16949,'016949','1986-069M','deb Delta 180','',59262,'Reentered','1986-09-23 00:00:00','1986-09-09 00:00:00',92.27,209,563,22.99,'Equatorial' from dual union all
select 1695,'001695','1965-082BJ','deb Transtage 4','',11832,'Reentered','1989-03-28 00:00:00','1966-01-27 00:00:00',101,728,884,32.36,'Intermediate' from dual union all
select 16950,'016950','1986-069N','deb PAS','',59262,'Reentered','1986-09-13 00:00:00','1986-09-09 00:00:00',91.99,182,563,37.12,'Intermediate' from dual union all
select 16951,'016951','1986-069P','deb PAS','',59262,'Reentered','1986-10-21 00:00:00','1986-09-09 00:00:00',99.06,200,1228,39.02,'Intermediate' from dual union all
select 16952,'016952','1986-070A','Kosmos-1777','Strela-2M',59263,'In Earth orbit','','1986-10-10 00:00:00',100.74,776,812,73.99,'Intermediate' from dual union all
select 16953,'016953','1986-070B','S3M','S3M',59263,'In Earth orbit','','1986-10-10 00:00:00',100.57,778,793,74.02,'Intermediate' from dual union all
select 16954,'016954','1984-104AD','Kosmos-1601 SS 22','ESO',57158,'Reentered','1988-08-06 00:00:00','1986-09-10 00:00:00',94.45,461,523,65.84,'Intermediate' from dual union all
select 16955,'016955','1984-104AE','Kosmos-1601 SS 23','ESO',57158,'Reentered','1988-06-18 00:00:00','1986-09-18 00:00:00',94.28,461,506,65.84,'Intermediate' from dual union all
select 16956,'016956','1985-050S','Kosmos-1662 SS 11','ESO',58273,'Reentered','1987-10-07 00:00:00','1986-09-11 00:00:00',94.48,480,506,65.83,'Intermediate' from dual union all
select 16957,'016957','1985-050T','Kosmos-1662 SS 12','ESO',58273,'Reentered','1987-09-15 00:00:00','1986-09-11 00:00:00',94.45,480,504,65.83,'Intermediate' from dual union all
select 16958,'016958','1985-050U','Kosmos-1662 SS 13','ESO',58273,'Reentered','1987-09-09 00:00:00','1986-09-11 00:00:00',94.37,480,497,65.84,'Intermediate' from dual union all
select 16959,'016959','1985-050V','Kosmos-1662 SS 14','ESO',58273,'Reentered','1988-05-26 00:00:00','1986-09-11 00:00:00',94.41,481,499,65.83,'Intermediate' from dual union all
select 1696,'001696','1965-082BK','deb Transtage 4','',11832,'Reentered','1968-01-18 00:00:00','1968-01-08 00:00:00',92.45,387,402,32.17,'Intermediate' from dual union all
select 16960,'016960','1985-050W','Kosmos-1662 SS 15','ESO',58273,'Reentered','1987-09-09 00:00:00','1986-09-12 00:00:00',94.38,480,498,65.84,'Intermediate' from dual union all
select 16961,'016961','1986-071A','Kosmos-1778','Uragan No. 24L',59264,'In Earth orbit','','1986-10-16 00:00:00',675.71,19117,19142,64.81,'Medium' from dual union all
select 16962,'016962','1986-071B','Kosmos-1779','Uragan No. 25L',59264,'In Earth orbit','','1986-10-16 00:00:00',675.69,19112,19146,64.81,'Medium' from dual union all
select 16963,'016963','1986-071C','Kosmos-1780','Uragan No. 26L',59264,'In Earth orbit','','1986-10-17 00:00:00',675.71,19104,19155,64.8,'Medium' from dual union all
select 16964,'016964','1986-071D','8S812','8S812',59264,'Reentered','1986-09-16 00:00:00','1986-09-16 00:00:00',90.42,163,428,64.8,'Intermediate' from dual union all
select 16965,'016965','1986-071E','Perekhodnik','Sredniy perekhodnik',59264,'Reentered','1986-09-16 00:00:00','1986-09-16 00:00:00',90.42,163,428,64.8,'Intermediate' from dual union all
select 16966,'016966','1986-072A','Kosmos-1781','Oblik',59265,'Landed','1986-10-01 00:00:00','1986-09-20 00:00:00',92.16,348,413,70.36,'Intermediate' from dual union all
select 16967,'016967','1986-072B','11S510','11S510',59265,'Reentered','1986-10-07 00:00:00','1986-09-24 00:00:00',89.89,200,338,70.35,'Intermediate' from dual union all
select 16968,'016968','1986-071F','Blok DM2','11S861 No. 10L',59264,'In Earth orbit','','1986-10-16 00:00:00',675.18,19088,19144,64.8,'Medium' from dual union all
select 16969,'016969','1986-073A','NOAA 10','NOAA G',59266,'In Earth orbit','','1986-10-19 00:00:00',101.22,807,825,98.75,'Sun-Synch' from dual union all
select 1697,'001697','1965-082BL','Transtage 4 large debris','RCS 2.9',11832,'Reentered','1978-01-08 00:00:00','1966-01-19 00:00:00',99.24,668,777,32.67,'Intermediate' from dual union all
select 16970,'016970','1986-066C','KDU','KDU',59259,'Reentered','1988-04-12 00:00:00','1986-10-07 00:00:00',92.14,348,411,70.36,'Intermediate' from dual union all
select 16971,'016971','1986-066D','Blenda','Sunshade',59259,'Reentered','1987-03-01 00:00:00','1986-10-03 00:00:00',91.91,333,404,70.36,'Intermediate' from dual union all
select 16972,'016972','1986-066E','KDU part','KDU part',59259,'Reentered','1986-12-20 00:00:00','1986-10-03 00:00:00',94.21,376,585,70.35,'Intermediate' from dual union all
select 16973,'016973','1986-066F','KDU part','KDU part',59259,'Reentered','1987-04-02 00:00:00','1986-10-03 00:00:00',94.02,374,569,70.37,'Intermediate' from dual union all
select 16974,'016974','1986-072C','Sensor cover','Sensor cover',59265,'Reentered','1986-09-20 00:00:00','1986-09-19 00:00:00',89.21,192,279,70.38,'Intermediate' from dual union all
select 16975,'016975','1986-066G','KDU part','KDU part',59259,'Reentered','1986-09-27 00:00:00','1986-09-18 00:00:00',92.24,356,413,70.22,'Intermediate' from dual union all
select 16976,'016976','1986-066H','KDU part','KDU part',59259,'Reentered','1986-10-07 00:00:00','1986-09-17 00:00:00',92.1,341,414,70.23,'Intermediate' from dual union all
select 16977,'016977','1986-066J','KDU part','KDU part',59259,'Reentered','1986-10-02 00:00:00','1986-09-17 00:00:00',90.43,209,383,70.36,'Intermediate' from dual union all
select 16978,'016978','1986-066K','KDU part','KDU part',59259,'Reentered','1986-10-10 00:00:00','1986-09-18 00:00:00',92.2,339,426,70.61,'Intermediate' from dual union all
select 16979,'016979','1986-066L','KDU part','KDU part',59259,'Reentered','1986-12-21 00:00:00','1986-10-03 00:00:00',92.04,343,406,70.36,'Intermediate' from dual union all
select 1698,'001698','1965-082BM','deb Transtage 4','',11832,'Reentered','1972-05-09 00:00:00','1968-11-15 00:00:00',98.06,638,694,32.18,'Intermediate' from dual union all
select 16980,'016980','1985-050X','Kosmos-1662 SS 16','ESO',58273,'Reentered','1987-09-12 00:00:00','1986-09-25 00:00:00',94.36,478,498,65.83,'Intermediate' from dual union all
select 16981,'016981','1985-050Y','Kosmos-1662 SS 17','ESO',58273,'Reentered','1987-09-29 00:00:00','1986-09-25 00:00:00',94.46,481,504,65.84,'Intermediate' from dual union all
select 16982,'016982','1986-073B','NOAA 10 solar array band','',59266,'Reentered','1996-10-03 00:00:00','1986-10-17 00:00:00',101.22,811,822,98.75,'Sun-Synch' from dual union all
select 16983,'016983','1986-073C','NOAA 10 solar array band','',59266,'Reentered','1991-08-04 00:00:00','1986-10-17 00:00:00',101.25,812,824,98.75,'Sun-Synch' from dual union all
select 16984,'016984','1986-071G','SOZ','SOZ',59264,'Reentered','1993-01-29 00:00:00','1986-10-18 00:00:00',340.34,409,19130,64.76,'Highly Ellip' from dual union all
select 16985,'016985','1986-071H','SOZ','SOZ',59264,'Reentered','1992-11-26 00:00:00','1986-10-17 00:00:00',340.34,413,19126,64.85,'Highly Ellip' from dual union all
select 16986,'016986','1986-074A','Kosmos-1782','Tselina-D',59267,'In Earth orbit','','1986-10-30 00:00:00',97.71,634,665,82.52,'Intermediate' from dual union all
select 16987,'016987','1986-074B','S5M','S5M',59267,'In Earth orbit','','1986-10-30 00:00:00',97.69,634,663,82.52,'Intermediate' from dual union all
select 16988,'016988','1986-072D','KDU','KDU',59265,'Reentered','1988-04-09 00:00:00','1986-10-17 00:00:00',92.12,347,410,70.36,'Intermediate' from dual union all
select 16989,'016989','1986-072E','KDU part','KDU part',59265,'Reentered','1987-05-20 00:00:00','1986-10-20 00:00:00',91.93,335,404,70.36,'Intermediate' from dual union all
select 1699,'001699','1965-082BN','deb Transtage 4','',11832,'Reentered','1966-11-21 00:00:00','1966-11-15 00:00:00',91.78,336,388,32.8,'Intermediate' from dual union all
select 16990,'016990','1986-072F','KDU part','KDU part',59265,'Reentered','1987-05-19 00:00:00','1986-10-17 00:00:00',94.06,375,571,70.36,'Intermediate' from dual union all
select 16991,'016991','1986-072G','Blenda','Sunshade',59265,'Reentered','1986-12-29 00:00:00','1986-10-17 00:00:00',92.02,345,402,70.36,'Intermediate' from dual union all
select 16992,'016992','1986-072H','KDU part','KDU part',59265,'Reentered','1987-01-13 00:00:00','1986-10-19 00:00:00',94.79,375,642,70.37,'Intermediate' from dual union all
select 16993,'016993','1986-075A','Kosmos-1783','Oko',59268,'In Earth orbit','','1986-11-02 00:00:00',358.1,612,20041,62.82,'Highly Ellip' from dual union all
select 16994,'016994','1986-075B','11S510','11S510',59268,'Reentered','1986-11-22 00:00:00','1986-10-10 00:00:00',92.1,207,547,62.8,'Intermediate' from dual union all
select 16995,'016995','1986-075C','BOZ','BOZ',59268,'Reentered','1986-11-08 00:00:00','1986-10-11 00:00:00',92.12,190,567,62.82,'Intermediate' from dual union all
select 16996,'016996','1986-075D','Blok-2BL','Blok-2BL No. 77331-671',59268,'In Earth orbit','','1986-11-03 00:00:00',357.03,610,19976,62.74,'Highly Ellip' from dual union all
select 16997,'016997','1976-077EV','deb Delta 126','',39029,'Reentered','1997-10-07 00:00:00','1993-08-11 00:00:00',115.43,1186,1764,102.95,'Sun-Synch' from dual union all
select 16998,'016998','1986-075E','deb Kosmos-1783','',59268,'Reentered','1986-11-12 00:00:00','1986-10-10 00:00:00',92.38,207,576,62.81,'Intermediate' from dual union all
select 16999,'016999','1976-077EW','deb Delta 126','',39029,'Reentered','1988-08-01 00:00:00','1986-10-04 00:00:00',109.24,737,1647,102.36,'Sun-Synch' from dual union all
select 17,'000017','59 Delta 2','Altair','Altair X-248',5527,'In Earth orbit','1961-06-30 00:00:00','1961-01-08 00:00:00',718.92,197,40214,47.1,'GEO Transfer' from dual union all
select 170,'000170','61 Upsilon','Explorer 12','NASA S-3',7281,'Reentered','1963-08-31 00:00:00','1961-09-20 00:00:00',1593.48,457,77170,33.43,'Very High' from dual union all
select 1700,'001700','1965-082BP','deb Transtage 4','',11832,'Reentered','1970-04-03 00:00:00','1966-01-07 00:00:00',97.97,585,739,32.7,'Intermediate' from dual union all
select 17000,'017000','1976-077EX','deb Delta 126','',39029,'In Earth orbit','','1993-08-04 00:00:00',128.3,1456,2641,102.52,'Medium' from dual union all
select 17001,'017001','1986-076A','Fanhui Shi Weixing (08)','Jianbing-1 No. 9',59269,'Reentered','1986-10-23 00:00:00','1986-10-13 00:00:00',89.71,171,349,56.95,'Intermediate' from dual union all
select 17002,'017002','1986-076B','CZ-2C Stage 2','CZ-2C Stage 2',59269,'Reentered','1986-10-20 00:00:00','1986-10-08 00:00:00',89.91,171,369,56.96,'Intermediate' from dual union all
select 17003,'017003','1986-077A','Kosmos-1784','Siluet No. 6',59270,'Landed','1986-11-11 00:00:00','1986-10-13 00:00:00',89.18,206,262,64.75,'Intermediate' from dual union all
select 17004,'017004','1986-077B','11S510','11S510',59270,'Reentered','1986-10-12 00:00:00','1986-10-10 00:00:00',88.63,181,233,64.76,'Intermediate' from dual union all
select 17005,'017005','1986-076C','CZ-2C sep motor cover','',59269,'Reentered','1986-10-08 00:00:00','1986-10-07 00:00:00',89.36,166,320,56.92,'Intermediate' from dual union all
select 17006,'017006','1986-067C','Dispenser cover','Romb dispenser cover',59260,'Reentered','1987-10-30 00:00:00','1986-10-07 00:00:00',94.14,441,513,74.04,'Intermediate' from dual union all
select 17007,'017007','1986-067D','Dispenser cover','Romb dispenser cover',59260,'Reentered','1987-10-14 00:00:00','1986-10-07 00:00:00',94.04,433,512,74.01,'Intermediate' from dual union all
select 17008,'017008','1986-067E','Dispenser cover','Romb dispenser cover',59260,'Reentered','1987-11-08 00:00:00','1986-10-06 00:00:00',94.18,445,513,73.98,'Intermediate' from dual union all
select 17009,'017009','1986-067F','Dispenser cover','Romb dispenser cover',59260,'Reentered','1987-10-24 00:00:00','1986-10-06 00:00:00',94.11,438,513,74.06,'Intermediate' from dual union all
select 1701,'001701','1965-087A','Proton-2','N-4 No. 2',11837,'Reentered','1966-02-06 00:00:00','1966-02-01 00:00:00',88.77,151,276,63.44,'Intermediate' from dual union all
select 17010,'017010','1983-044CA','deb Kosmos-1461','',55801,'Reentered','1989-03-22 00:00:00','1986-10-08 00:00:00',98.83,553,853,65.02,'Intermediate' from dual union all
select 17011,'017011','1983-044CB','deb Kosmos-1461','',55801,'Reentered','1989-01-03 00:00:00','1986-10-08 00:00:00',98.88,554,857,65.08,'Intermediate' from dual union all
select 17012,'017012','1983-044CC','deb Kosmos-1461','',55801,'Reentered','1989-06-22 00:00:00','1986-10-08 00:00:00',99.29,583,866,65.08,'Intermediate' from dual union all
select 17013,'017013','1983-044CD','deb Kosmos-1461','',55801,'Reentered','1990-04-25 00:00:00','1986-10-08 00:00:00',99.22,576,866,64.98,'Intermediate' from dual union all
select 17014,'017014','1983-044CE','deb Kosmos-1461','',55801,'Reentered','1989-02-27 00:00:00','1986-10-07 00:00:00',98.76,548,850,64.99,'Intermediate' from dual union all
select 17015,'017015','1983-044CF','deb Kosmos-1461','',55801,'Reentered','1988-12-22 00:00:00','1986-10-08 00:00:00',98.99,579,842,64.99,'Intermediate' from dual union all
select 17016,'017016','1983-044CG','deb Kosmos-1461','',55801,'Reentered','1989-05-13 00:00:00','1986-10-08 00:00:00',100.01,648,870,65.04,'Intermediate' from dual union all
select 17017,'017017','1983-044CH','deb Kosmos-1461','',55801,'Reentered','1989-08-03 00:00:00','1986-10-08 00:00:00',100.34,666,884,65.07,'Intermediate' from dual;
insert into SATELLITE
select 17018,'017018','1976-067BJ','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-09 00:00:00',116.3,1005,2024,66.01,'Intermediate' from dual union all
select 17019,'017019','1986-069Q','deb PAS','',59262,'Reentered','1987-03-20 00:00:00','1986-10-09 00:00:00',99.59,209,1269,39.51,'Intermediate' from dual union all
select 1702,'001702','1965-087B','8S811','8S811',11837,'Reentered','1966-01-04 00:00:00','1966-01-02 00:00:00',88.32,149,234,63.44,'Intermediate' from dual union all
select 17020,'017020','1986-069R','deb PAS','',59262,'Reentered','1986-10-29 00:00:00','1986-10-08 00:00:00',92.01,208,538,39.45,'Intermediate' from dual union all
select 17021,'017021','1986-069S','deb PAS','',59262,'Reentered','1986-10-26 00:00:00','1986-10-08 00:00:00',92.86,206,624,39.45,'Intermediate' from dual union all
select 17022,'017022','1986-069T','deb PAS','',59262,'Reentered','1986-10-31 00:00:00','1986-10-08 00:00:00',92.44,206,583,39.65,'Intermediate' from dual union all
select 17023,'017023','1974-089EW','deb Delta 104','',33692,'In Earth orbit','','1993-08-04 00:00:00',114.06,1372,1454,101.46,'Sun-Synch' from dual union all
select 17024,'017024','1977-065FT','deb Delta 132','',42193,'In Earth orbit','','1993-07-16 00:00:00',118.01,864,2319,29.44,'Intermediate' from dual union all
select 17025,'017025','1981-053KF','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',105.2,942,1065,82.83,'Intermediate' from dual union all
select 17026,'017026','1981-053KG','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-29 00:00:00',105.37,946,1077,83.14,'Intermediate' from dual union all
select 17027,'017027','1978-026GU','deb Delta 139','',45221,'Reentered','2015-04-23 00:00:00','1993-07-31 00:00:00',100.27,744,799,98.66,'Sun-Synch' from dual union all
select 17028,'017028','1986-076D','FSW RV part','',59269,'Reentered','1986-10-12 00:00:00','1986-10-11 00:00:00',90.21,165,405,56.93,'Intermediate' from dual union all
select 17029,'017029','1971-015CZ','deb Kosmos-397','',24833,'Reentered','1988-09-24 00:00:00','1987-08-11 00:00:00',96.44,445,732,65.95,'Intermediate' from dual union all
select 1703,'001703','1965-087C','UR-500 GO part','Fairing',11837,'Reentered','1965-11-16 00:00:00','1965-11-02 00:00:00',94.56,383,612,63.5,'Intermediate' from dual union all
select 17030,'017030','1986-024AF','deb Kosmos-1736 ROCh','',59217,'Reentered','1986-11-13 00:00:00','1986-10-11 00:00:00',98.31,530,825,65.07,'Intermediate' from dual union all
select 17031,'017031','1986-078A','Kosmos-1785','Oko',59271,'Reentered','2002-02-28 00:00:00','1986-11-14 00:00:00',718.08,634,39736,63.1,'Molniya' from dual union all
select 17032,'017032','1986-078B','BOZ','BOZ',59271,'Reentered','1986-11-18 00:00:00','1986-10-22 00:00:00',92.04,188,561,62.85,'Intermediate' from dual union all
select 17033,'017033','1986-078C','11S510','11S510',59271,'Reentered','1986-12-09 00:00:00','1986-10-22 00:00:00',92.11,209,547,62.84,'Intermediate' from dual union all
select 17034,'017034','1986-062B','Kosmos-1771 RLS','US-A RLS',59255,'Reentered','1986-10-20 00:00:00','1986-10-15 00:00:00',89.46,237,258,65,'Intermediate' from dual union all
select 17035,'017035','1986-062C','Kosmos-1771 reactor core','US-A reactor core',59255,'In Earth orbit','','1986-10-19 00:00:00',103.87,913,970,65,'Intermediate' from dual union all
select 17036,'017036','1986-062D','Kosmos-1771 DU','US-A DU',59255,'Reentered','1986-11-29 00:00:00','1986-10-15 00:00:00',89.57,244,262,65,'Intermediate' from dual union all
select 17037,'017037','1986-078D','Blok-2BL','Blok-2BL No. 76711-668',59271,'Reentered','2015-03-30 00:00:00','1986-11-14 00:00:00',707.61,685,39167,63.35,'Molniya' from dual union all
select 17038,'017038','1986-079A','Molniya-3','Molniya-3 No. 41',59272,'Reentered','1999-10-15 00:00:00','1986-11-19 00:00:00',717.64,612,39736,62.92,'Molniya' from dual union all
select 17039,'017039','1986-079B','11S510','11S510',59272,'Reentered','1986-12-22 00:00:00','1986-11-19 00:00:00',91.04,207,445,62.8,'Intermediate' from dual union all
select 1704,'001704','1965-087D','UR-500 GO part','Fairing',11837,'Reentered','1965-11-08 00:00:00','1965-11-02 00:00:00',94.04,185,760,63.4,'Intermediate' from dual union all
select 17040,'017040','1986-079C','BOZ','BOZ',59272,'Reentered','1986-12-12 00:00:00','1986-10-27 00:00:00',92.41,196,590,62.84,'Intermediate' from dual union all
select 17041,'017041','1986-079D','Blok-ML','Blok-ML No. 71629-459',59272,'Reentered','1999-07-22 00:00:00','1986-11-27 00:00:00',699.41,624,38821,62.86,'Molniya' from dual union all
select 17042,'017042','1986-080A','Kosmos-1786','Taifun-1B',59273,'Reentered','1988-03-06 00:00:00','1986-11-25 00:00:00',112.65,186,2511,64.89,'Intermediate' from dual union all
select 17043,'017043','1986-080B','11S772','11S772',59273,'Reentered','1988-04-08 00:00:00','1986-10-22 00:00:00',113.05,196,2538,64.9,'Intermediate' from dual union all
select 17044,'017044','1986-081A','Kosmos-1787','Oblik',59274,'Landed','1986-11-04 00:00:00','1986-10-24 00:00:00',89.59,229,280,69.98,'Intermediate' from dual union all
select 17045,'017045','1986-081B','11S510','11S510',59274,'Reentered','1986-10-30 00:00:00','1986-10-24 00:00:00',88.91,200,242,69.99,'Intermediate' from dual union all
select 17046,'017046','1986-082A','Raduga','Gran'' No. 30L',59275,'In Earth orbit','','1986-11-25 00:00:00',1435.96,35743,35825,1.29,'Stationary' from dual union all
select 17047,'017047','1986-082B','8S812','8S812',59275,'Reentered','1986-10-28 00:00:00','1986-10-27 00:00:00',88.12,180,183,51.61,'Intermediate' from dual union all
select 17048,'017048','1986-082C','Perekhodnik','Sredniy perekhodnik',59275,'Reentered','1986-10-26 00:00:00','1986-10-25 00:00:00',88.36,188,199,51.62,'Intermediate' from dual union all
select 17049,'017049','1986-072J','KDU part','KDU part',59265,'Reentered','1986-12-01 00:00:00','1986-10-26 00:00:00',93.02,373,472,69.6,'Intermediate' from dual union all
select 1705,'001705','1965-087E','deb Proton-2','',11837,'Reentered','1965-11-08 00:00:00','1965-11-02 00:00:00',92.01,191,556,63.5,'Intermediate' from dual union all
select 17050,'017050','1986-083A','Kosmos-1788','Vektor',59276,'Reentered','1991-01-21 00:00:00','1986-11-26 00:00:00',94.44,466,517,65.83,'Intermediate' from dual union all
select 17051,'017051','1986-083B','S3M','S3M',59276,'Reentered','1990-01-05 00:00:00','1986-11-26 00:00:00',94.41,466,514,65.84,'Intermediate' from dual union all
select 17052,'017052','1986-082D','SOZ','SOZ',59275,'Reentered','1987-07-17 00:00:00','1986-11-27 00:00:00',649.76,251,36693,47.18,'GEO Transfer' from dual union all
select 17053,'017053','1986-082E','SOZ','SOZ',59275,'Reentered','1991-04-21 00:00:00','1986-11-27 00:00:00',651.07,301,36710,47.21,'GEO Transfer' from dual union all
select 17054,'017054','1986-084A','Kosmos-1789','Resurs-F1 14F40 No. 61',59277,'Landed','1986-11-14 00:00:00','1986-11-02 00:00:00',91.16,322,341,82.58,'Intermediate' from dual union all
select 17055,'017055','1986-084B','11S510','11S510',59277,'Reentered','1986-11-04 00:00:00','1986-11-02 00:00:00',88.74,175,250,82.58,'Intermediate' from dual union all
select 17056,'017056','1986-085A','Kosmos-1790','Oblik No. 350',59278,'Landed','1986-11-18 00:00:00','1986-11-06 00:00:00',89.58,227,280,72.86,'Intermediate' from dual union all
select 17057,'017057','1986-085B','11S510','11S510',59278,'Reentered','1986-11-09 00:00:00','1986-11-06 00:00:00',88.97,185,263,72.86,'Intermediate' from dual union all
select 17058,'017058','1986-081C','Blenda','Sunshade',59274,'Reentered','1986-11-20 00:00:00','1986-11-04 00:00:00',91.9,267,468,70.01,'Intermediate' from dual union all
select 17059,'017059','1985-050Z','Kosmos-1662 SS 18','ESO',58273,'Reentered','1988-06-20 00:00:00','1986-11-04 00:00:00',94.43,478,504,65.83,'Intermediate' from dual union all
select 1706,'001706','1965-088A','Kosmos-95','DS-U2-V No. 2',11838,'Reentered','1966-01-18 00:00:00','1966-01-17 00:00:00',88.01,147,205,48.44,'Intermediate' from dual union all
select 17060,'017060','1985-050AA','Kosmos-1662 SS 19','ESO',58273,'Reentered','1988-06-12 00:00:00','1986-11-04 00:00:00',94.38,478,500,65.83,'Intermediate' from dual union all
select 17061,'017061','1986-081D','KDU part','KDU part',59274,'Reentered','1986-11-17 00:00:00','1986-11-04 00:00:00',89.96,226,319,69.99,'Intermediate' from dual union all
select 17062,'017062','1986-081E','KDU','KDU',59274,'Reentered','1986-12-31 00:00:00','1986-11-21 00:00:00',89.83,221,312,69.98,'Intermediate' from dual union all
select 17063,'017063','1986-081F','KDU part','KDU part',59274,'Reentered','1986-11-09 00:00:00','1986-11-04 00:00:00',90.01,224,326,69.99,'Intermediate' from dual union all
select 17064,'017064','1986-081G','KDU part','KDU part',59274,'Reentered','1986-11-08 00:00:00','1986-11-05 00:00:00',91.47,261,433,70.03,'Intermediate' from dual union all
select 17065,'017065','1986-082F','Blok DM','11S86 No. 62L',59275,'In Earth orbit','','1986-11-26 00:00:00',1475.35,36432,36671,1.28,'Drift GEO' from dual union all
select 17066,'017066','1986-086A','Kosmos-1791','Tsikada',59279,'In Earth orbit','','1986-12-13 00:00:00',104.76,954,1013,82.95,'Intermediate' from dual union all
select 17067,'017067','1986-086B','S3M','S3M',59279,'In Earth orbit','','1986-12-26 00:00:00',104.63,951,1004,82.94,'Intermediate' from dual union all
select 17068,'017068','1986-087A','Kosmos-1792','Kobal''t',59280,'Landed','1987-01-05 00:00:00','1986-11-20 00:00:00',89.27,168,309,64.9,'Intermediate' from dual union all
select 17069,'017069','1986-087B','11S510','11S510',59280,'Reentered','1986-11-19 00:00:00','1986-11-15 00:00:00',89.25,169,305,64.92,'Intermediate' from dual union all
select 1707,'001707','1965-088B','S1M','S1M',11838,'Reentered','1965-11-28 00:00:00','1965-11-05 00:00:00',91.58,218,487,48.4,'Intermediate' from dual union all
select 17070,'017070','1986-088A','Polar BEAR P87-1','NNS O-17',59281,'In Earth orbit','','1986-12-14 00:00:00',104.91,961,1020,89.56,'Polar' from dual union all
select 17071,'017071','1986-088B','Star 20 E-38','Star 20 E-38',59281,'In Earth orbit','','1986-12-21 00:00:00',104.89,961,1018,89.55,'Polar' from dual union all
select 17072,'017072','1986-084C','KDU','KDU',59277,'Reentered','1987-08-27 00:00:00','1986-11-30 00:00:00',91.08,316,340,82.58,'Intermediate' from dual union all
select 17073,'017073','1986-084D','Blenda','Sunshade',59277,'Reentered','1986-12-19 00:00:00','1986-11-14 00:00:00',91.13,319,341,82.58,'Intermediate' from dual union all
select 17074,'017074','1986-084E','KDU part','KDU part',59277,'Reentered','1987-01-07 00:00:00','1987-02-06 00:00:00',93.59,365,536,82.63,'Intermediate' from dual union all
select 17075,'017075','1986-084F','KDU part','KDU part',59277,'Reentered','1986-11-19 00:00:00','1986-11-14 00:00:00',90.99,314,333,82.57,'Intermediate' from dual union all
select 17076,'017076','1986-084G','KDU part','KDU part',59277,'Reentered','1987-01-03 00:00:00','1986-12-01 00:00:00',90.57,289,316,82.57,'Intermediate' from dual union all
select 17077,'017077','1986-084H','KDU part','KDU part',59277,'Reentered','1987-02-21 00:00:00','1986-11-30 00:00:00',92.83,333,494,82.57,'Intermediate' from dual union all
select 17078,'017078','1986-089A','Molniya-1','Molniya-1 No. 60',59282,'In Earth orbit','','1986-12-16 00:00:00',717.68,482,39867,62.87,'Molniya' from dual union all
select 17079,'017079','1986-089B','11S510','11S510',59282,'Reentered','1986-12-19 00:00:00','1986-11-22 00:00:00',90.75,208,414,62.85,'Intermediate' from dual union all
select 1708,'001708','1965-088C','deb Kosmos-95','',11838,'Reentered','1965-11-11 00:00:00','1965-11-05 00:00:00',94.24,211,753,48.4,'Intermediate' from dual union all
select 17080,'017080','1986-089C','BOZ','BOZ',59282,'Reentered','1986-12-28 00:00:00','1986-11-22 00:00:00',91.01,212,436,62.82,'Intermediate' from dual union all
select 17081,'017081','1986-089D','Blok-ML','Blok-ML No. 76758-474',59282,'In Earth orbit','','1986-12-31 00:00:00',735.84,565,40677,62.84,'Molniya' from dual union all
select 17082,'017082','1986-085C','Blenda','Sunshade',59278,'Reentered','1986-11-21 00:00:00','1986-11-18 00:00:00',89.39,222,267,72.86,'Intermediate' from dual union all
select 17083,'017083','1986-090A','Gorizont','Gorizont No. 22L',59283,'In Earth orbit','','1986-12-18 00:00:00',1436.04,35778,35793,1.36,'Stationary' from dual union all
select 17084,'017084','1986-090B','8S812','8S812',59283,'Reentered','1986-11-21 00:00:00','1986-11-20 00:00:00',87.95,170,176,51.63,'Intermediate' from dual union all
select 17085,'017085','1986-090C','Perekhodnik','Sredniy perekhodnik',59283,'Reentered','1986-11-19 00:00:00','1986-11-18 00:00:00',88.28,156,223,51.75,'Intermediate' from dual union all
select 17086,'017086','1986-085D','KDU part','KDU part',59278,'Reentered','1986-11-26 00:00:00','1986-11-18 00:00:00',91.44,243,448,72.87,'Intermediate' from dual union all
select 17087,'017087','1986-085E','KDU part','KDU part',59278,'Reentered','1986-11-23 00:00:00','1986-11-19 00:00:00',89.17,218,249,72.86,'Intermediate' from dual union all
select 17088,'017088','1986-085F','KDU part','KDU part',59278,'Reentered','1986-11-24 00:00:00','1986-11-18 00:00:00',91.55,245,457,72.88,'Intermediate' from dual union all
select 17089,'017089','1986-085G','KDU','KDU',59278,'Reentered','1986-12-10 00:00:00','1986-11-19 00:00:00',89.4,222,268,72.86,'Intermediate' from dual union all
select 1709,'001709','1965-082BQ','deb Transtage 4','',11832,'Reentered','1982-04-07 00:00:00','1966-01-12 00:00:00',99.68,701,786,32.58,'Intermediate' from dual union all
select 17090,'017090','1982-055AC','deb Kosmos-1375','',54364,'In Earth orbit','','1993-07-30 00:00:00',104.1,909,996,65.84,'Intermediate' from dual union all
select 17091,'017091','1983-044CJ','deb Kosmos-1461','',55801,'Reentered','1988-03-09 00:00:00','1986-11-12 00:00:00',98.66,604,786,65,'Intermediate' from dual union all
select 17092,'017092','1984-015D','deb Ohzora','',57069,'Reentered','1988-01-01 00:00:00','1984-12-20 00:00:00',95.6,344,752,74.68,'Intermediate' from dual union all
select 17093,'017093','1965-027U','deb Snapshot','',11777,'In Earth orbit','','1980-12-12 00:00:00',111.48,1271,1318,90.25,'Polar' from dual union all
select 17094,'017094','1979-017JH','deb P78-1','',47501,'Reentered','2001-05-29 00:00:00','1993-08-11 00:00:00',93.91,449,483,97.81,'Sun-Synch' from dual union all
select 17095,'017095','1979-017JJ','deb P78-1','',47501,'Reentered','1988-11-13 00:00:00','1986-11-19 00:00:00',95.73,512,596,97.77,'Sun-Synch' from dual union all
select 17096,'017096','1983-044CK','deb Kosmos-1461','',55801,'Reentered','1989-04-17 00:00:00','1986-11-18 00:00:00',99.07,564,865,65.05,'Intermediate' from dual union all
select 17097,'017097','1983-044CL','deb Kosmos-1461','',55801,'Reentered','1989-04-30 00:00:00','1986-11-18 00:00:00',99.01,564,859,65,'Intermediate' from dual union all
select 17098,'017098','1983-044CM','deb Kosmos-1461','',55801,'Reentered','1990-10-21 00:00:00','1986-11-19 00:00:00',99.29,582,868,65.03,'Intermediate' from dual union all
select 17099,'017099','1983-044CN','deb Kosmos-1461','',55801,'Reentered','1990-03-28 00:00:00','1986-11-19 00:00:00',99.26,581,865,65.02,'Intermediate' from dual union all
select 171,'000171','61 Omicron 46','deb r/b Ablestar','',7273,'Reentered','1962-01-29 00:00:00','1961-08-12 00:00:00',97.69,320,976,66.78,'Intermediate' from dual union all
select 1710,'001710','1965-082BR','deb Transtage 4','',11832,'Reentered','1973-03-01 00:00:00','1966-01-22 00:00:00',99.02,677,747,32.7,'Intermediate' from dual union all
select 17100,'017100','1979-017JK','deb P78-1','',47501,'Reentered','1988-09-22 00:00:00','1986-11-19 00:00:00',95.4,506,569,97.91,'Sun-Synch' from dual union all
select 17101,'017101','1979-017JL','deb P78-1','',47501,'Reentered','1989-03-28 00:00:00','1986-11-18 00:00:00',95.35,517,554,97.93,'Sun-Synch' from dual union all
select 17102,'017102','1982-055AD','deb Kosmos-1375','',54364,'In Earth orbit','','1993-07-28 00:00:00',105.83,982,1085,65.84,'Intermediate' from dual union all
select 17103,'017103','1982-055AE','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-02 00:00:00',105.03,977,1014,65.84,'Intermediate' from dual union all
select 17104,'017104','1982-055AF','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-04 00:00:00',104.99,975,1012,65.83,'Intermediate' from dual union all
select 17105,'017105','1982-055AG','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-06 00:00:00',105.09,975,1022,65.83,'Intermediate' from dual union all
select 17106,'017106','1982-055AH','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-02 00:00:00',104.4,950,983,65.83,'Intermediate' from dual union all
select 17107,'017107','1982-055AJ','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-08 00:00:00',103.94,922,968,65.83,'Intermediate' from dual;
insert into SATELLITE
select 17108,'017108','1982-055AK','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-07 00:00:00',104.84,978,996,65.83,'Intermediate' from dual union all
select 17109,'017109','1982-033HJ','Salyut-7 debris','',54342,'Reentered','1987-03-29 00:00:00','1986-02-07 00:00:00',92.76,345,475,51.56,'Intermediate' from dual union all
select 1711,'001711','1965-082BS','deb Transtage 4','',11832,'Reentered','1971-03-20 00:00:00','1966-01-16 00:00:00',98.55,631,748,32.48,'Intermediate' from dual union all
select 17110,'017110','1986-019E','deb Ariane V16','',59212,'Reentered','2002-04-10 00:00:00','1993-08-01 00:00:00',100.18,724,810,98.92,'Sun-Synch' from dual union all
select 17111,'017111','1986-019F','deb Ariane V16','',59212,'Reentered','1989-01-07 00:00:00','1986-11-19 00:00:00',99.16,634,803,99,'Sun-Synch' from dual union all
select 17112,'017112','1986-019G','deb Ariane V16','',59212,'Reentered','1990-11-14 00:00:00','1986-11-19 00:00:00',99.59,675,803,98.73,'Sun-Synch' from dual union all
select 17113,'017113','1986-019H','deb Ariane V16','',59212,'Reentered','1988-08-26 00:00:00','1986-11-19 00:00:00',99.81,688,811,98.51,'Sun-Synch' from dual union all
select 17114,'017114','1986-019J','deb Ariane V16','',59212,'Reentered','1989-04-05 00:00:00','1986-11-19 00:00:00',99.87,693,812,98.5,'Sun-Synch' from dual union all
select 17115,'017115','1986-019K','deb Ariane V16','',59212,'Reentered','1989-01-17 00:00:00','1986-11-19 00:00:00',99.93,704,807,98.86,'Sun-Synch' from dual union all
select 17116,'017116','1986-019L','deb Ariane V16','',59212,'Reentered','2002-03-12 00:00:00','1993-08-05 00:00:00',100,714,802,99.16,'Sun-Synch' from dual union all
select 17117,'017117','1986-019M','deb Ariane V16','',59212,'In Earth orbit','','1993-08-09 00:00:00',102.18,769,954,98.78,'Sun-Synch' from dual union all
select 17118,'017118','1986-019N','deb Ariane V16','',59212,'In Earth orbit','','1993-08-05 00:00:00',103.13,773,1041,99,'Sun-Synch' from dual union all
select 17119,'017119','1986-019P','deb Ariane V16','',59212,'Reentered','1990-07-02 00:00:00','1990-07-02 00:00:00',87.97,167,181,98.86,'Retrograde' from dual union all
select 1712,'001712','1965-082BT','deb Transtage 4','',11832,'Reentered','1981-09-16 00:00:00','1966-01-30 00:00:00',102.34,730,1008,32.23,'Intermediate' from dual union all
select 17120,'017120','1986-019Q','deb Ariane V16','',59212,'Reentered','2001-10-29 00:00:00','1993-08-09 00:00:00',99.54,706,768,99.11,'Sun-Synch' from dual union all
select 17121,'017121','1986-019R','deb Ariane V16','',59212,'In Earth orbit','','1993-07-31 00:00:00',101.6,785,884,98.52,'Sun-Synch' from dual union all
select 17122,'017122','1986-019S','Ariane V16 major debris','',59212,'In Earth orbit','','1993-08-03 00:00:00',100.89,791,811,98.36,'Sun-Synch' from dual union all
select 17123,'017123','1986-019T','deb Ariane V16','',59212,'Reentered','2001-12-10 00:00:00','1993-08-07 00:00:00',99.72,720,771,98.3,'Sun-Synch' from dual union all
select 17124,'017124','1986-019U','deb Ariane V16','',59212,'Reentered','2000-02-14 00:00:00','1993-08-04 00:00:00',98.14,662,678,98.49,'Sun-Synch' from dual union all
select 17125,'017125','1986-090D','Blok DM No. 58L','11S86 No. 58L',59283,'In Earth orbit','','1986-12-21 00:00:00',1434.81,35721,35802,1.37,'Drift GEO' from dual union all
select 17126,'017126','1986-019V','deb Ariane V16','',59212,'Reentered','1987-03-16 00:00:00','1986-11-15 00:00:00',97.46,472,803,98.66,'Sun-Synch' from dual union all
select 17127,'017127','1986-019W','deb Ariane V16','',59212,'Reentered','1987-07-25 00:00:00','1986-11-15 00:00:00',97.8,504,802,99.19,'Sun-Synch' from dual union all
select 17128,'017128','1986-019X','deb Ariane V16','',59212,'In Earth orbit','','1986-11-20 00:00:00',103.63,800,1061,99.02,'Sun-Synch' from dual union all
select 17129,'017129','1986-019Y','Ariane V16 major debris','',59212,'In Earth orbit','','1986-11-20 00:00:00',101.48,802,856,98.7,'Sun-Synch' from dual union all
select 1713,'001713','1965-082BU','deb Transtage 4','',11832,'Reentered','1969-01-02 00:00:00','1966-01-30 00:00:00',104.92,649,1333,32.57,'Intermediate' from dual union all
select 17130,'017130','1986-019Z','deb Ariane V16','',59212,'Reentered','2015-01-23 00:00:00','1986-11-18 00:00:00',101.68,804,873,98.73,'Sun-Synch' from dual union all
select 17131,'017131','1986-019AA','Ariane V16 major debris','',59212,'Reentered','2002-12-08 00:00:00','1993-08-01 00:00:00',99.47,712,755,98.36,'Sun-Synch' from dual union all
select 17132,'017132','1986-019AB','deb Ariane V16','',59212,'Reentered','1988-12-28 00:00:00','1986-11-20 00:00:00',101.62,785,886,98.65,'Sun-Synch' from dual union all
select 17133,'017133','1986-019AC','deb Ariane V16','',59212,'Reentered','1988-09-04 00:00:00','1988-09-03 00:00:00',91.31,309,369,98.89,'Sun-Synch' from dual union all
select 17134,'017134','1986-091A','Kosmos-1793','Oko',59284,'Reentered','2011-05-15 00:00:00','1986-12-20 00:00:00',717.8,639,39717,62.96,'Molniya' from dual union all
select 17135,'017135','1986-091B','11S510','11S510',59284,'Reentered','1987-01-27 00:00:00','1986-12-20 00:00:00',91.1,206,451,62.84,'Intermediate' from dual union all
select 17136,'017136','1986-091C','BOZ','BOZ',59284,'Reentered','1986-12-23 00:00:00','1986-11-27 00:00:00',91.91,181,555,62.88,'Intermediate' from dual union all
select 17137,'017137','1985-050AB','Kosmos-1662 SS 20','ESO',58273,'Reentered','1987-10-29 00:00:00','1986-11-20 00:00:00',94.41,476,504,65.89,'Intermediate' from dual union all
select 17138,'017138','1986-092A','Kosmos-1794','Strela-1M',59285,'In Earth orbit','','1986-12-21 00:00:00',115.6,1466,1500,74.01,'Intermediate' from dual union all
select 17139,'017139','1986-092B','Kosmos-1795','Strela-1M',59285,'In Earth orbit','','1986-12-21 00:00:00',115.4,1466,1482,74.01,'Intermediate' from dual union all
select 1714,'001714','1965-082BV','deb Transtage 4','',11832,'Reentered','1970-08-09 00:00:00','1966-01-14 00:00:00',99.52,706,765,32.92,'Intermediate' from dual union all
select 17140,'017140','1986-092C','Kosmos-1796','Strela-1M',59285,'In Earth orbit','','1986-12-21 00:00:00',115.24,1455,1478,74.02,'Intermediate' from dual union all
select 17141,'017141','1986-092D','Kosmos-1797','Strela-1M',59285,'In Earth orbit','','1986-12-21 00:00:00',115.05,1444,1472,74.01,'Intermediate' from dual union all
select 17142,'017142','1986-092E','Kosmos-1798','Strela-1M',59285,'In Earth orbit','','1986-12-21 00:00:00',114.89,1429,1472,74.01,'Intermediate' from dual union all
select 17143,'017143','1986-092F','Kosmos-1799','Strela-1M',59285,'In Earth orbit','','1986-12-21 00:00:00',114.72,1414,1471,74.01,'Intermediate' from dual union all
select 17144,'017144','1986-092G','Kosmos-1800','Strela-1M',59285,'In Earth orbit','','1986-12-21 00:00:00',114.56,1399,1471,74.01,'Intermediate' from dual union all
select 17145,'017145','1986-092H','Kosmos-1801','Strela-1M',59285,'In Earth orbit','','1986-12-21 00:00:00',114.4,1386,1471,74.01,'Intermediate' from dual union all
select 17146,'017146','1986-092J','S3M','S3M',59285,'In Earth orbit','','1986-12-28 00:00:00',117.66,1474,1678,74.01,'Intermediate' from dual union all
select 17147,'017147','1986-091D','Blok-2BL','Blok-2BL',59284,'In Earth orbit','','1986-12-20 00:00:00',705.89,635,39131,62.94,'Molniya' from dual union all
select 17148,'017148','1986-090E','SOZ','SOZ',59283,'Reentered','1988-08-12 00:00:00','1987-01-08 00:00:00',633.9,290,35842,47.17,'GEO Transfer' from dual union all
select 17149,'017149','1986-090F','SOZ','SOZ',59283,'Reentered','1987-05-09 00:00:00','1986-11-25 00:00:00',632.96,277,35807,47.29,'GEO Transfer' from dual union all
select 1715,'001715','1965-082BW','Transtage 4 large debris','RCS 1.4',11832,'Reentered','1993-02-04 00:00:00','1992-01-29 00:00:00',94.87,505,520,31.87,'Intermediate' from dual union all
select 17150,'017150','1986-019AD','deb Ariane V16','',59212,'Reentered','1990-06-12 00:00:00','1990-06-11 00:00:00',91.2,323,344,98.47,'Sun-Synch' from dual union all
select 17151,'017151','1986-019AE','deb Ariane V16','',59212,'Reentered','1989-05-04 00:00:00','1986-11-23 00:00:00',101.02,783,830,98.56,'Sun-Synch' from dual union all
select 17152,'017152','1986-019AF','deb Ariane V16','',59212,'Reentered','2001-03-05 00:00:00','1993-08-06 00:00:00',99.59,712,766,98.56,'Sun-Synch' from dual union all
select 17153,'017153','1986-019AG','deb Ariane V16','',59212,'In Earth orbit','','1993-08-06 00:00:00',100.61,771,803,98.46,'Sun-Synch' from dual union all
select 17154,'017154','1986-019AH','deb Ariane V16','',59212,'Reentered','2001-11-02 00:00:00','1993-08-04 00:00:00',100.38,728,826,98.64,'Sun-Synch' from dual union all
select 17155,'017155','1986-019AJ','deb Ariane V16','',59212,'In Earth orbit','','1993-08-02 00:00:00',101.44,755,898,98.97,'Sun-Synch' from dual union all
select 17156,'017156','1986-019AK','deb Ariane V16','',59212,'Reentered','1989-12-07 00:00:00','1986-11-23 00:00:00',101.5,803,856,98.48,'Sun-Synch' from dual union all
select 17157,'017157','1986-019AL','deb Ariane V16','',59212,'Reentered','1988-12-16 00:00:00','1987-08-19 00:00:00',100.77,770,821,98.64,'Sun-Synch' from dual union all
select 17158,'017158','1985-050AC','Kosmos-1662 SS 21','ESO',58273,'Reentered','1987-10-27 00:00:00','1986-11-23 00:00:00',94.41,477,503,65.77,'Intermediate' from dual union all
select 17159,'017159','1986-093A','Kosmos-1802','Parus',59286,'In Earth orbit','','1986-12-25 00:00:00',104.96,962,1024,82.93,'Intermediate' from dual union all
select 1716,'001716','1965-082BX','Transtage 4 large debris','RCS 2.3',11832,'In Earth orbit','','1992-01-25 00:00:00',99.58,705,772,32.19,'Intermediate' from dual union all
select 17160,'017160','1986-093B','S3M','S3M',59286,'In Earth orbit','','1986-12-25 00:00:00',104.86,956,1019,82.93,'Intermediate' from dual union all
select 17161,'017161','1984-104AF','Kosmos-1601 SS 24','ESO',57158,'Reentered','1988-05-15 00:00:00','1985-11-09 00:00:00',94.46,478,507,65.84,'Intermediate' from dual union all
select 17162,'017162','1984-015E','deb Ohzora','',57069,'Reentered','1987-09-08 00:00:00','1984-12-14 00:00:00',96.23,391,764,75.15,'Intermediate' from dual union all
select 17163,'017163','1983-044CP','deb Kosmos-1461','',55801,'Reentered','1989-06-13 00:00:00','1986-11-25 00:00:00',99.18,575,863,65.07,'Intermediate' from dual union all
select 17164,'017164','1985-066G','NNS O-30 despin weight','',58289,'In Earth orbit','','1985-09-07 00:00:00',108.34,991,1309,89.34,'Polar' from dual union all
select 17165,'017165','1979-017JM','deb P78-1','',47501,'Reentered','1989-01-27 00:00:00','1986-11-25 00:00:00',94.58,479,518,97.76,'Sun-Synch' from dual union all
select 17166,'017166','1979-017JN','deb P78-1','',47501,'Reentered','1987-10-18 00:00:00','1986-11-26 00:00:00',94.64,486,516,97.84,'Sun-Synch' from dual union all
select 17167,'017167','1979-017JP','deb P78-1','',47501,'Reentered','1987-12-03 00:00:00','1986-11-25 00:00:00',94.9,491,537,97.62,'Sun-Synch' from dual union all
select 17168,'017168','1985-094S','deb Kosmos-1695','',58317,'In Earth orbit','','1985-11-26 00:00:00',114.05,1408,1416,82.6,'Intermediate' from dual union all
select 17169,'017169','1970-025PB','deb Agena D','',21502,'Reentered','1992-02-28 00:00:00','1992-02-28 00:00:00',91.12,313,346,99.84,'Retrograde' from dual union all
select 1717,'001717','1965-082BY','deb Transtage 4','',11832,'Reentered','1969-02-02 00:00:00','1969-01-16 00:00:00',91.84,353,377,32.14,'Intermediate' from dual union all
select 17170,'017170','1970-025PC','deb Agena D','',21502,'Reentered','1995-07-23 00:00:00','1993-09-14 00:00:00',96.27,547,613,99.86,'Sun-Synch' from dual union all
select 17171,'017171','1983-044CQ','deb Kosmos-1461','',55801,'Reentered','1989-03-17 00:00:00','1986-11-25 00:00:00',99.3,589,862,65.02,'Intermediate' from dual union all
select 17172,'017172','1979-017JQ','deb P78-1','',47501,'Reentered','1987-08-15 00:00:00','1986-11-24 00:00:00',94.16,464,492,97.75,'Sun-Synch' from dual union all
select 17173,'017173','1979-017JR','deb P78-1','',47501,'Reentered','1987-05-06 00:00:00','1986-11-25 00:00:00',93.74,445,470,97.74,'Sun-Synch' from dual union all
select 17174,'017174','1979-017JS','deb P78-1','',47501,'Reentered','1987-06-23 00:00:00','1986-11-25 00:00:00',93.92,455,478,97.76,'Sun-Synch' from dual union all
select 17175,'017175','1986-054M','KDU part','KDU part',59247,'Reentered','1987-02-03 00:00:00','1986-08-23 00:00:00',94.8,389,629,72.85,'Intermediate' from dual union all
select 17176,'017176','1967-092G','NNS O-14 insulation','',15347,'Reentered','1991-10-03 00:00:00','1986-09-06 00:00:00',106.62,1035,1106,89.25,'Polar' from dual union all
select 17177,'017177','1986-094A','Kosmos-1803','Musson No. 18',59287,'In Earth orbit','','1987-01-01 00:00:00',115.98,1497,1503,82.6,'Intermediate' from dual union all
select 17178,'017178','1986-094B','S5M','S5M',59287,'In Earth orbit','','1987-01-02 00:00:00',115.92,1496,1499,82.6,'Intermediate' from dual union all
select 17179,'017179','1986-095A','Kosmos-1804','Oblik',59288,'Landed','1986-12-18 00:00:00','1986-12-06 00:00:00',92.16,347,414,69.99,'Intermediate' from dual union all
select 1718,'001718','1965-082BZ','deb Transtage 4','',11832,'Reentered','1981-06-07 00:00:00','1981-06-01 00:00:00',91,316,332,32.14,'Intermediate' from dual union all
select 17180,'017180','1986-095B','11S510','11S510',59288,'Reentered','1986-12-31 00:00:00','1986-12-11 00:00:00',90.41,198,391,70,'Intermediate' from dual union all
select 17181,'017181','1986-096A','FLTSATCOM F7','FLTSATCOM F7',59289,'In Earth orbit','','1987-01-05 00:00:00',1436.01,35548,36021,5.21,'Inclined GEO' from dual union all
select 17182,'017182','1986-096B','Centaur AC-66','Centaur D-1AR AC-66',59289,'Reentered','1988-03-13 00:00:00','1987-01-04 00:00:00',579.77,143,33165,28.41,'Highly Ellip' from dual union all
select 17183,'017183','1986-019AM','deb Ariane V16','',59212,'Reentered','1989-11-08 00:00:00','1986-12-08 00:00:00',103.78,790,1085,98.58,'Sun-Synch' from dual union all
select 17184,'017184','1986-019AN','deb Ariane V16','',59212,'Reentered','2001-11-10 00:00:00','1993-08-02 00:00:00',100.24,721,819,98.88,'Sun-Synch' from dual union all
select 17185,'017185','1986-019AP','deb Ariane V16','',59212,'Reentered','1989-03-11 00:00:00','1986-12-08 00:00:00',99.52,675,797,99.01,'Sun-Synch' from dual union all
select 17186,'017186','1986-019AQ','deb Ariane V16','',59212,'Reentered','1988-12-28 00:00:00','1987-08-17 00:00:00',100.67,768,812,98.7,'Sun-Synch' from dual union all
select 17187,'017187','1986-019AR','deb Ariane V16','',59212,'Reentered','1989-01-23 00:00:00','1986-12-08 00:00:00',99.45,663,802,98.99,'Sun-Synch' from dual union all
select 17188,'017188','1986-019AS','deb Ariane V16','',59212,'Reentered','1988-11-11 00:00:00','1986-12-08 00:00:00',104.03,745,1153,99.14,'Sun-Synch' from dual union all
select 17189,'017189','1986-019AT','deb Ariane V16','',59212,'Reentered','1998-01-10 00:00:00','1993-08-08 00:00:00',98.13,617,722,98.12,'Sun-Synch' from dual union all
select 1719,'001719','1965-082CA','deb Transtage 4','',11832,'Reentered','1981-03-03 00:00:00','1981-03-01 00:00:00',90.69,294,323,31.95,'Intermediate' from dual union all
select 17190,'017190','1986-019AU','deb Ariane V16','',59212,'Reentered','1989-02-20 00:00:00','1986-12-08 00:00:00',99.61,678,802,98.89,'Sun-Synch' from dual union all
select 17191,'017191','1986-097A','Kosmos-1805','Tselina-R',59290,'In Earth orbit','','1987-01-09 00:00:00',97.68,634,662,82.51,'Intermediate' from dual union all
select 17192,'017192','1986-097B','S5M','S5M',59290,'In Earth orbit','','1987-01-09 00:00:00',97.65,633,660,82.51,'Intermediate' from dual union all
select 17193,'017193','1986-019AV','deb Ariane V16','',59212,'Reentered','1989-03-22 00:00:00','1986-12-10 00:00:00',98.98,617,803,99.03,'Sun-Synch' from dual union all
select 17194,'017194','1986-019AW','deb Ariane V16','',59212,'Reentered','1989-02-16 00:00:00','1986-12-10 00:00:00',100.18,734,801,98.73,'Sun-Synch' from dual union all
select 17195,'017195','1986-019AX','deb Ariane V16','',59212,'Reentered','1991-04-01 00:00:00','1986-12-10 00:00:00',100.38,736,818,98.54,'Sun-Synch' from dual union all
select 17196,'017196','1986-019AY','deb Ariane V16','',59212,'Reentered','1989-01-16 00:00:00','1986-12-10 00:00:00',101.07,784,835,98.55,'Sun-Synch' from dual union all
select 17197,'017197','1986-019AZ','deb Ariane V16','',59212,'Reentered','1989-02-21 00:00:00','1986-12-10 00:00:00',101.79,781,906,98.66,'Sun-Synch' from dual union all
select 17198,'017198','1986-019BA','deb Ariane V16','',59212,'Reentered','2002-05-11 00:00:00','1993-08-08 00:00:00',101.22,744,889,98.89,'Sun-Synch' from dual;
insert into SATELLITE
select 17199,'017199','1986-019BB','deb Ariane V16','',59212,'In Earth orbit','','1992-05-13 00:00:00',101.46,759,897,98.52,'Sun-Synch' from dual union all
select 172,'000172','61 Omicron 47','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-04 00:00:00',107.23,980,1216,67.06,'Intermediate' from dual union all
select 1720,'001720','1965-082CB','deb Transtage 4','',11832,'Reentered','1967-03-04 00:00:00','1967-02-17 00:00:00',94.87,483,541,32.36,'Intermediate' from dual union all
select 17200,'017200','1986-019BC','deb Ariane V16','',59212,'Reentered','1991-07-09 00:00:00','1986-12-10 00:00:00',100.62,756,820,98.54,'Sun-Synch' from dual union all
select 17201,'017201','1986-019BD','deb Ariane V16','',59212,'Reentered','2000-04-21 00:00:00','1993-08-06 00:00:00',98.43,672,695,99.16,'Sun-Synch' from dual union all
select 17202,'017202','1986-019BE','deb Ariane V16','',59212,'Reentered','1989-03-04 00:00:00','1986-12-10 00:00:00',102.77,795,985,98.29,'Sun-Synch' from dual union all
select 17203,'017203','1986-019BF','deb Ariane V16','',59212,'In Earth orbit','','1993-08-07 00:00:00',102.94,790,1005,98.81,'Sun-Synch' from dual union all
select 17204,'017204','1986-019BG','deb Ariane V16','',59212,'Reentered','1991-05-18 00:00:00','1986-12-10 00:00:00',103.35,795,1039,98.15,'Sun-Synch' from dual union all
select 17205,'017205','1986-019BH','deb Ariane V16','',59212,'Reentered','1989-03-27 00:00:00','1986-12-10 00:00:00',102.8,798,985,98.33,'Sun-Synch' from dual union all
select 17206,'017206','1986-019BJ','deb Ariane V16','',59212,'Reentered','2000-06-24 00:00:00','1993-08-07 00:00:00',98.4,672,693,98.54,'Sun-Synch' from dual union all
select 17207,'017207','1986-019BK','deb Ariane V16','',59212,'Reentered','1989-01-14 00:00:00','1986-12-10 00:00:00',101.14,789,837,98.55,'Sun-Synch' from dual union all
select 17208,'017208','1986-019BL','deb Ariane V16','',59212,'Reentered','2000-10-28 00:00:00','1993-08-03 00:00:00',98.84,677,730,99.02,'Sun-Synch' from dual union all
select 17209,'017209','1986-019BM','deb Ariane V16','',59212,'Reentered','1991-04-29 00:00:00','1986-12-10 00:00:00',100.29,729,816,98.54,'Sun-Synch' from dual union all
select 1721,'001721','1965-082CC','deb Transtage 4','',11832,'Reentered','2000-07-01 00:00:00','1992-01-30 00:00:00',97.33,606,656,32.54,'Intermediate' from dual union all
select 17210,'017210','1986-019BN','deb Ariane V16','',59212,'Reentered','1989-02-27 00:00:00','1986-12-10 00:00:00',100.15,731,801,98.81,'Sun-Synch' from dual union all
select 17211,'017211','1986-019BP','deb Ariane V16','',59212,'Reentered','1989-04-21 00:00:00','1986-12-10 00:00:00',100.42,755,802,98.76,'Sun-Synch' from dual union all
select 17212,'017212','1986-019BQ','deb Ariane V16','',59212,'In Earth orbit','','1993-08-01 00:00:00',101.15,770,857,98.46,'Sun-Synch' from dual union all
select 17213,'017213','1986-098A','Kosmos-1806','Oko',59291,'In Earth orbit','','1987-01-12 00:00:00',718,634,39732,62.86,'Molniya' from dual union all
select 17214,'017214','1986-098B','11S510','11S510',59291,'Reentered','1987-02-23 00:00:00','1987-01-11 00:00:00',91.25,208,463,62.79,'Intermediate' from dual union all
select 17215,'017215','1986-098C','BOZ','BOZ',59291,'Reentered','1987-02-10 00:00:00','1987-01-11 00:00:00',91.28,191,483,62.83,'Intermediate' from dual union all
select 17216,'017216','1986-098D','Blok-2BL','Blok-2BL',59291,'In Earth orbit','','1987-01-11 00:00:00',705.87,629,39137,62.84,'Molniya' from dual union all
select 17217,'017217','1986-099A','Kosmos-1807','Kobal''t No. 509',59292,'Landed','1987-01-23 00:00:00','1986-12-23 00:00:00',89.18,160,308,67.14,'Intermediate' from dual union all
select 17218,'017218','1986-099B','11S510','11S510',59292,'Reentered','1986-12-22 00:00:00','1986-12-18 00:00:00',89.25,163,311,67.14,'Intermediate' from dual union all
select 17219,'017219','1986-019BR','deb Ariane V16','',59212,'In Earth orbit','','1993-08-08 00:00:00',101.04,767,849,98.35,'Sun-Synch' from dual union all
select 1722,'001722','1965-082CD','deb Transtage 4','',11832,'In Earth orbit','','1992-01-30 00:00:00',99.23,698,746,32.42,'Intermediate' from dual union all
select 17220,'017220','1986-019BS','deb Ariane V16','',59212,'Reentered','1988-09-23 00:00:00','1986-12-17 00:00:00',101,776,836,98.61,'Sun-Synch' from dual union all
select 17221,'017221','1986-019BT','deb Ariane V16','',59212,'Reentered','1989-06-19 00:00:00','1986-12-17 00:00:00',100.95,769,839,98.67,'Sun-Synch' from dual union all
select 17222,'017222','1986-019BU','deb Ariane V16','',59212,'Reentered','1990-05-25 00:00:00','1986-12-17 00:00:00',104.6,798,1154,98.65,'Sun-Synch' from dual union all
select 17223,'017223','1986-019BV','deb Ariane V16','',59212,'Reentered','1992-03-16 00:00:00','1992-03-16 00:00:00',89.97,257,288,98.39,'Sun-Synch' from dual union all
select 17224,'017224','1986-019BW','deb Ariane V16','',59212,'Reentered','1989-03-18 00:00:00','1986-12-17 00:00:00',100.04,714,807,98.75,'Sun-Synch' from dual union all
select 17225,'017225','1986-019BX','deb Ariane V16','',59212,'Reentered','1988-01-16 00:00:00','1986-12-17 00:00:00',97.46,489,785,98.95,'Sun-Synch' from dual union all
select 17226,'017226','1986-019BY','deb Ariane V16','',59212,'Reentered','1989-12-09 00:00:00','1986-12-17 00:00:00',101.06,788,830,98.55,'Sun-Synch' from dual union all
select 17227,'017227','1986-019BZ','deb Ariane V16','',59212,'Reentered','1987-05-19 00:00:00','1986-12-17 00:00:00',99.65,683,800,98.9,'Sun-Synch' from dual union all
select 17228,'017228','1986-019CA','deb Ariane V16','',59212,'Reentered','1991-09-13 00:00:00','1991-09-13 00:00:00',89.5,232,268,98.57,'Sun-Synch' from dual union all
select 17229,'017229','1986-019CB','deb Ariane V16','',59212,'Reentered','1989-06-03 00:00:00','1986-12-17 00:00:00',100.68,784,797,98.67,'Sun-Synch' from dual union all
select 1723,'001723','1965-082CE','deb Transtage 4','',11832,'Reentered','1978-11-06 00:00:00','1978-10-30 00:00:00',91.57,337,367,32.12,'Intermediate' from dual union all
select 17230,'017230','1986-019CC','deb Ariane V16','',59212,'Reentered','1989-09-06 00:00:00','1986-12-17 00:00:00',101.04,779,837,98.5,'Sun-Synch' from dual union all
select 17231,'017231','1986-019CD','deb Ariane V16','',59212,'Reentered','1988-09-25 00:00:00','1986-12-17 00:00:00',98.68,594,798,99.05,'Sun-Synch' from dual union all
select 17232,'017232','1986-019CE','deb Ariane V16','',59212,'Reentered','1988-10-27 00:00:00','1986-12-17 00:00:00',99.03,634,791,99.01,'Sun-Synch' from dual union all
select 17233,'017233','1986-019CF','deb Ariane V16','',59212,'In Earth orbit','','1993-08-03 00:00:00',100.36,750,802,98.29,'Sun-Synch' from dual union all
select 17234,'017234','1986-019CG','deb Ariane V16','',59212,'Reentered','1988-08-10 00:00:00','1986-12-17 00:00:00',99.87,708,797,98.58,'Sun-Synch' from dual union all
select 17235,'017235','1986-019CH','deb Ariane V16','',59212,'In Earth orbit','','1986-12-16 00:00:00',101.06,788,830,98.55,'Sun-Synch' from dual union all
select 17236,'017236','1986-019CJ','deb Ariane V16','',59212,'Reentered','1989-08-20 00:00:00','1986-12-17 00:00:00',101.14,787,839,98.57,'Sun-Synch' from dual union all
select 17237,'017237','1986-019CK','deb Ariane V16','',59212,'Reentered','1989-11-09 00:00:00','1986-12-17 00:00:00',100.98,781,830,98.33,'Sun-Synch' from dual union all
select 17238,'017238','1986-019CL','deb Ariane V16','',59212,'Reentered','1988-12-21 00:00:00','1986-12-16 00:00:00',101.51,763,898,99.27,'Sun-Synch' from dual union all
select 17239,'017239','1986-100A','Kosmos-1808','Parus',59293,'In Earth orbit','','1987-01-16 00:00:00',105.03,972,1020,82.94,'Intermediate' from dual union all
select 1724,'001724','1965-082CF','deb Transtage 4','',11832,'Reentered','1968-04-26 00:00:00','1965-10-15 00:00:00',97.73,557,744,32.5,'Intermediate' from dual union all
select 17240,'017240','1986-100B','S3M','S3M',59293,'In Earth orbit','','1987-01-16 00:00:00',104.9,967,1013,82.93,'Intermediate' from dual union all
select 17241,'017241','1986-101A','Kosmos-1809','AUOS-Z 501 Ionozond-E',59294,'In Earth orbit','','1987-01-18 00:00:00',104.14,943,966,82.53,'Intermediate' from dual union all
select 17242,'017242','1986-101B','S5M','S5M',59294,'In Earth orbit','','1987-01-17 00:00:00',104.12,947,960,82.56,'Intermediate' from dual union all
select 17243,'017243','1986-095C','KDU part','KDU part',59288,'Reentered','1987-09-04 00:00:00','1987-01-05 00:00:00',94.08,375,573,69.99,'Intermediate' from dual union all
select 17244,'017244','1986-095D','KDU','KDU',59288,'Reentered','1988-04-13 00:00:00','1987-01-05 00:00:00',92.12,345,412,69.99,'Intermediate' from dual union all
select 17245,'017245','1986-095E','Blenda','Sunshade',59288,'Reentered','1987-05-26 00:00:00','1987-01-05 00:00:00',91.9,331,405,69.99,'Intermediate' from dual union all
select 17246,'017246','1986-095F','KDU part','KDU part',59288,'Reentered','1987-04-15 00:00:00','1987-01-04 00:00:00',92.03,343,406,69.99,'Intermediate' from dual union all
select 17247,'017247','1985-114D','ITV 2 Canister cover','',58337,'Reentered','1987-03-02 00:00:00','1986-12-18 00:00:00',95.1,310,736,37.07,'Intermediate' from dual union all
select 17248,'017248','1985-082C','deb Kosmos-1682','',58305,'Reentered','1986-12-27 00:00:00','1986-12-21 00:00:00',92.72,361,455,65.03,'Intermediate' from dual union all
select 17249,'017249','1985-082D','deb Kosmos-1682','',58305,'Reentered','1986-12-29 00:00:00','1986-12-19 00:00:00',92.92,378,458,65.01,'Intermediate' from dual union all
select 1725,'001725','1965-082CG','deb Transtage 4','',11832,'Reentered','1997-04-11 00:00:00','1992-01-30 00:00:00',97.83,592,717,32.28,'Intermediate' from dual union all
select 17250,'017250','1985-082E','deb Kosmos-1682','',58305,'Reentered','1987-01-13 00:00:00','1986-12-19 00:00:00',93.52,411,482,64.9,'Intermediate' from dual union all
select 17251,'017251','1985-082F','deb Kosmos-1682','',58305,'Reentered','1987-01-13 00:00:00','1986-12-19 00:00:00',93.3,410,463,65.02,'Intermediate' from dual union all
select 17252,'017252','1986-095G','KDU part','KDU part',59288,'Reentered','1987-04-08 00:00:00','1987-01-26 00:00:00',93.59,366,535,69.99,'Intermediate' from dual union all
select 17253,'017253','1986-026E','deb Ariane','[UNID107] RCS 1.5',59219,'In Earth orbit','','1986-12-07 00:00:00',561,488,31820,8.01,'Highly Ellip' from dual union all
select 17254,'017254','1986-026F','deb Ariane','[UNID108] RCS 0.7',59219,'In Earth orbit','','1986-05-10 00:00:00',556.8,534,31549,8.75,'Highly Ellip' from dual union all
select 17255,'017255','1985-056C','deb Ariane V14','',58279,'In Earth orbit','','1987-06-14 00:00:00',598.54,313,33984,8.46,'Highly Ellip' from dual union all
select 17256,'017256','1981-058E','deb Kosmos-1278','',52581,'Reentered','2000-07-06 00:00:00','1985-01-19 00:00:00',719.08,1939,38480,66.24,'GEO Transfer' from dual union all
select 17257,'017257','1985-114E','ITV 2 Canister','',58337,'Reentered','1987-02-23 00:00:00','1986-12-23 00:00:00',94.96,313,721,37.07,'Intermediate' from dual union all
select 17258,'017258','1986-067G','Kosmos-1776 SS 1','ESO',59260,'Reentered','1987-12-09 00:00:00','1986-12-24 00:00:00',94.48,474,512,74.02,'Intermediate' from dual union all
select 17259,'017259','1986-067H','Kosmos-1776 SS 2','ESO',59260,'Reentered','1987-11-29 00:00:00','1986-12-25 00:00:00',94.49,475,513,74.09,'Intermediate' from dual union all
select 1726,'001726','1965-089A','Explorer 29','GEOS A',11839,'In Earth orbit','','1966-01-17 00:00:00',120.31,1117,2273,59.38,'Medium' from dual union all
select 17260,'017260','1986-067J','Kosmos-1776 SS 3','ESO',59260,'Reentered','1987-11-23 00:00:00','1986-12-25 00:00:00',94.46,472,513,73.96,'Intermediate' from dual union all
select 17261,'017261','1986-067K','Kosmos-1776 SS 4','ESO',59260,'Reentered','1987-11-14 00:00:00','1986-12-24 00:00:00',94.41,469,511,74.03,'Intermediate' from dual union all
select 17262,'017262','1986-102A','Kosmos-1810','Yantar''-4KS1M',59295,'Deorbited','1987-09-11 00:00:00','1987-01-25 00:00:00',89.51,245,256,64.75,'Intermediate' from dual union all
select 17263,'017263','1986-102B','11S510','11S510',59295,'Reentered','1986-12-31 00:00:00','1986-12-28 00:00:00',88.77,175,253,64.76,'Intermediate' from dual union all
select 17264,'017264','1986-103A','Molniya-1','Molniya-1 No. 62',59296,'Reentered','2006-02-01 00:00:00','1987-01-25 00:00:00',717.62,488,39859,62.82,'Molniya' from dual union all
select 17265,'017265','1986-103B','11S510','11S510',59296,'Reentered','1987-02-02 00:00:00','1987-01-02 00:00:00',90.88,202,434,62.81,'Intermediate' from dual union all
select 17266,'017266','1986-103C','BOZ','BOZ',59296,'Reentered','1987-01-21 00:00:00','1987-01-02 00:00:00',90.67,186,429,62.8,'Intermediate' from dual union all
select 17267,'017267','1986-103D','Blok-ML','Blok-ML No. 77328-475',59296,'Reentered','2005-12-06 00:00:00','1987-01-28 00:00:00',698.81,395,39020,62.77,'Molniya' from dual union all
select 17268,'017268','1986-101C','deb S5M','',59294,'In Earth orbit','','1987-01-18 00:00:00',104.03,926,972,82.62,'Intermediate' from dual union all
select 17269,'017269','1986-101D','deb S5M','',59294,'In Earth orbit','','1987-01-18 00:00:00',104.44,955,982,82.65,'Intermediate' from dual union all
select 1727,'001727','1965-090A','Agena D 4822','Agena D',11840,'Reentered','1965-11-11 00:00:00','1966-11-08 00:00:00',88.71,145,277,93.88,'Polar' from dual union all
select 17270,'017270','1986-101E','deb S5M','',59294,'In Earth orbit','','1987-02-08 00:00:00',104.29,960,962,82.44,'Intermediate' from dual union all
select 17271,'017271','1986-101F','deb S5M','',59294,'In Earth orbit','','1987-01-17 00:00:00',103.81,916,962,82.42,'Intermediate' from dual union all
select 17272,'017272','1986-101G','deb S5M','',59294,'In Earth orbit','','1987-01-17 00:00:00',103.82,929,950,82.5,'Intermediate' from dual union all
select 17273,'017273','1986-101H','deb S5M','',59294,'In Earth orbit','','1987-01-18 00:00:00',103.77,925,949,82.54,'Intermediate' from dual union all
select 17274,'017274','1986-101J','deb S5M','',59294,'In Earth orbit','','1987-01-17 00:00:00',104.55,948,999,82.51,'Intermediate' from dual union all
select 17275,'017275','1985-082G','deb Kosmos-1682','',58305,'Reentered','1987-03-01 00:00:00','1986-12-31 00:00:00',93.13,391,465,65.07,'Intermediate' from dual union all
select 17276,'017276','1986-103E','deb BOZ','',59296,'Reentered','1987-01-27 00:00:00','1987-01-03 00:00:00',90.99,204,443,62.8,'Intermediate' from dual union all
select 17277,'017277','1985-082H','deb Kosmos-1682','',58305,'Reentered','1987-05-19 00:00:00','1987-01-04 00:00:00',92.76,374,446,64.95,'Intermediate' from dual union all
select 17278,'017278','1985-082J','deb Kosmos-1682','',58305,'Reentered','1987-03-11 00:00:00','1987-01-03 00:00:00',92.8,366,458,65.02,'Intermediate' from dual union all
select 17279,'017279','1985-082K','deb Kosmos-1682','',58305,'Reentered','1987-05-11 00:00:00','1987-01-03 00:00:00',93.17,378,481,65.02,'Intermediate' from dual union all
select 1728,'001728','1965-090B','OPS 6232','GAMBIT SV 973',11840,'Reentered','1965-11-09 00:00:00','1966-11-08 00:00:00',88.88,155,284,93.9,'Polar' from dual union all
select 17280,'017280','1985-082L','deb Kosmos-1682','',58305,'Reentered','1987-04-27 00:00:00','1987-01-03 00:00:00',92.89,368,465,65.06,'Intermediate' from dual union all
select 17281,'017281','1985-082M','deb Kosmos-1682','',58305,'Reentered','1987-05-27 00:00:00','1987-01-03 00:00:00',93.85,391,535,65.14,'Intermediate' from dual union all
select 17282,'017282','1985-082N','deb Kosmos-1682','',58305,'Reentered','1987-03-12 00:00:00','1987-03-12 00:00:00',88.52,194,209,64.98,'Intermediate' from dual union all
select 17283,'017283','1985-082P','deb Kosmos-1682','',58305,'Reentered','1987-07-13 00:00:00','1987-01-01 00:00:00',93.06,380,469,65,'Intermediate' from dual union all
select 17284,'017284','1985-082Q','deb Kosmos-1682','',58305,'Reentered','1987-01-07 00:00:00','1987-01-02 00:00:00',91.43,315,375,65.05,'Intermediate' from dual union all
select 17285,'017285','1985-082R','deb Kosmos-1682','',58305,'Reentered','1987-03-01 00:00:00','1987-01-04 00:00:00',92.74,375,443,65.03,'Intermediate' from dual union all
select 17286,'017286','1985-082S','deb Kosmos-1682','',58305,'Reentered','1987-03-07 00:00:00','1987-01-03 00:00:00',92.91,389,446,65.11,'Intermediate' from dual union all
select 17287,'017287','1985-082T','deb Kosmos-1682','',58305,'Reentered','1987-03-08 00:00:00','1987-03-07 00:00:00',88.89,208,232,64.99,'Intermediate' from dual union all
select 17288,'017288','1985-082U','deb Kosmos-1682','',58305,'Reentered','1987-01-21 00:00:00','1987-01-03 00:00:00',92.71,386,429,65.07,'Intermediate' from dual;
insert into SATELLITE
select 17289,'017289','1985-082V','deb Kosmos-1682','',58305,'Reentered','1987-07-19 00:00:00','1987-01-03 00:00:00',93.14,380,476,65.01,'Intermediate' from dual union all
select 1729,'001729','1965-089B','Altair 2 X-258C4','X-258C4',11839,'In Earth orbit','','1966-01-03 00:00:00',120.29,1113,2275,59.41,'Medium' from dual union all
select 17290,'017290','1987-001A','Meteor-2','Meteor-2 No. 17',60091,'In Earth orbit','','1987-02-04 00:00:00',104.07,941,960,82.47,'Intermediate' from dual union all
select 17291,'017291','1987-001B','S5M','S5M',60091,'In Earth orbit','','1987-02-04 00:00:00',104.03,941,956,82.47,'Intermediate' from dual union all
select 17292,'017292','1987-002A','Kosmos-1811','Kobal''t',60092,'Landed','1987-02-13 00:00:00','1987-01-16 00:00:00',89.34,200,284,64.85,'Intermediate' from dual union all
select 17293,'017293','1987-002B','11S510','11S510',60092,'Reentered','1987-01-16 00:00:00','1987-01-11 00:00:00',89.29,169,310,64.87,'Intermediate' from dual union all
select 17294,'017294','1987-002C','Kosmos-1811 part','Kobal''t part',60092,'Reentered','1987-01-11 00:00:00','1987-01-10 00:00:00',89.71,172,349,64.85,'Intermediate' from dual union all
select 17295,'017295','1987-003A','Kosmos-1812','Tselina-D',60093,'In Earth orbit','','1987-02-16 00:00:00',97.7,633,665,82.53,'Intermediate' from dual union all
select 17296,'017296','1987-003B','S5M','S5M',60093,'In Earth orbit','','1987-02-13 00:00:00',97.67,632,663,82.53,'Intermediate' from dual union all
select 17297,'017297','1987-004A','Kosmos-1813','Oblik No. 337',60094,'Reentered','1989-03-13 00:00:00','1987-02-25 00:00:00',92.27,359,413,72.82,'Intermediate' from dual union all
select 17298,'017298','1987-004B','11S510','11S510',60094,'Reentered','1987-02-01 00:00:00','1987-01-22 00:00:00',89.54,190,314,72.81,'Intermediate' from dual union all
select 17299,'017299','1987-005A','Progress-27','Progress 7K-TG No. 135',60095,'Deorbited','1987-02-25 00:00:00','1987-01-25 00:00:00',91.1,314,342,51.61,'Intermediate' from dual union all
select 173,'000173','61 Phi 1','Ranger 1','NASA P-32',7274,'Reentered','1961-08-30 00:00:00','1961-08-23 00:00:00',90.77,179,446,32.88,'Intermediate' from dual union all
select 1730,'001730','1965-091A','Venera-2','3MV-4 No. 4',11841,'Deep Space','1965-11-14 00:00:00','1965-11-12 00:00:00',89.7,205,315,51.88,'Intermediate' from dual union all
select 17300,'017300','1987-005B','11S510','11S510',60095,'Reentered','1987-01-20 00:00:00','1987-01-18 00:00:00',88.55,178,228,51.62,'Intermediate' from dual union all
select 17301,'017301','1986-063G','KDU debris','KDU debris',59256,'Reentered','1987-03-19 00:00:00','1987-01-19 00:00:00',93.38,370,510,72.85,'Intermediate' from dual union all
select 17302,'017302','1983-058E','deb Ariane','',55815,'Reentered','1990-07-10 00:00:00','1985-09-05 00:00:00',590.41,173,33697,6.19,'Highly Ellip' from dual union all
select 17303,'017303','1987-006A','Kosmos-1814','Strela-2M',60096,'In Earth orbit','','1987-02-20 00:00:00',100.65,770,808,74.06,'Intermediate' from dual union all
select 17304,'017304','1987-006B','S3M','S3M',60096,'In Earth orbit','','1987-02-20 00:00:00',100.55,766,804,74.06,'Intermediate' from dual union all
select 17305,'017305','1986-019CM','deb Ariane V16','',59212,'Reentered','1990-03-31 00:00:00','1987-01-21 00:00:00',105.5,774,1262,98.37,'Sun-Synch' from dual union all
select 17306,'017306','1986-019CN','deb Ariane V16','',59212,'Reentered','1988-09-24 00:00:00','1987-01-20 00:00:00',100.44,740,819,98.91,'Sun-Synch' from dual union all
select 17307,'017307','1986-019CP','deb Ariane V16','',59212,'Reentered','1992-03-06 00:00:00','1991-11-03 00:00:00',98.62,630,755,97.74,'Sun-Synch' from dual union all
select 17308,'017308','1986-019CQ','deb Ariane V16','',59212,'Reentered','1989-11-27 00:00:00','1987-01-21 00:00:00',105.75,783,1276,98.37,'Sun-Synch' from dual union all
select 17309,'017309','1986-019CR','deb Ariane V16','',59212,'Reentered','1989-06-15 00:00:00','1987-01-21 00:00:00',105.84,728,1339,98.57,'Sun-Synch' from dual union all
select 1731,'001731','1965-091B','11S510','11S510',11841,'Reentered','1965-11-17 00:00:00','1965-11-15 00:00:00',89.5,217,283,51.88,'Intermediate' from dual union all
select 17310,'017310','1986-019CS','deb Ariane V16','',59212,'Reentered','1989-04-08 00:00:00','1987-01-21 00:00:00',106.83,707,1453,98.39,'Sun-Synch' from dual union all
select 17311,'017311','1986-019CT','deb Ariane V16','',59212,'Reentered','2003-02-01 00:00:00','1993-08-05 00:00:00',99.19,716,724,98.34,'Sun-Synch' from dual union all
select 17312,'017312','1986-019CU','deb Ariane V16','',59212,'Reentered','1989-10-19 00:00:00','1987-01-21 00:00:00',101.27,796,842,98.71,'Sun-Synch' from dual union all
select 17313,'017313','1986-019CV','deb Ariane V16','',59212,'Reentered','1989-10-18 00:00:00','1987-01-21 00:00:00',103.96,768,1123,98.57,'Sun-Synch' from dual union all
select 17314,'017314','1986-019CW','deb Ariane V16','',59212,'Reentered','1992-01-06 00:00:00','1992-01-06 00:00:00',89.47,238,258,98.76,'Sun-Synch' from dual union all
select 17315,'017315','1986-019CX','deb Ariane V16','',59212,'Reentered','1992-01-23 00:00:00','1992-01-23 00:00:00',90.69,291,326,98.47,'Sun-Synch' from dual union all
select 17316,'017316','1986-019CY','deb Ariane V16','',59212,'In Earth orbit','','1993-08-03 00:00:00',100.1,753,774,98.29,'Sun-Synch' from dual union all
select 17317,'017317','1986-019CZ','deb Ariane V16','',59212,'Reentered','1987-06-20 00:00:00','1987-01-21 00:00:00',97.08,487,751,98.84,'Sun-Synch' from dual union all
select 17318,'017318','1986-019DA','deb Ariane V16','',59212,'Reentered','1987-10-28 00:00:00','1987-01-21 00:00:00',97.93,552,768,99.27,'Sun-Synch' from dual union all
select 17319,'017319','1986-019DB','deb Ariane V16','',59212,'Reentered','1988-11-09 00:00:00','1987-01-21 00:00:00',99.42,640,822,99.26,'Sun-Synch' from dual union all
select 1732,'001732','1965-091C','BOZ','BOZ',11841,'Reentered','1965-11-25 00:00:00','1965-11-15 00:00:00',88.68,203,216,51.85,'Intermediate' from dual union all
select 17320,'017320','1986-019DC','deb Ariane V16','',59212,'Reentered','1989-02-19 00:00:00','1987-01-21 00:00:00',99.7,672,817,98.34,'Sun-Synch' from dual union all
select 17321,'017321','1986-019DD','deb Ariane V16','',59212,'Reentered','1989-08-01 00:00:00','1987-01-21 00:00:00',99.88,698,808,99.11,'Sun-Synch' from dual union all
select 17322,'017322','1986-019DE','deb Ariane V16','',59212,'Reentered','1988-10-26 00:00:00','1987-01-21 00:00:00',101.02,712,901,98.59,'Sun-Synch' from dual union all
select 17323,'017323','1986-019DF','deb Ariane V16','',59212,'Reentered','1999-07-15 00:00:00','1993-08-06 00:00:00',97.41,616,654,99.06,'Sun-Synch' from dual union all
select 17324,'017324','1986-019DG','deb Ariane V16','',59212,'Reentered','2013-10-01 00:00:00','1993-08-08 00:00:00',100.31,744,803,98.55,'Sun-Synch' from dual union all
select 17325,'017325','1985-056D','deb Ariane V14','',58279,'In Earth orbit','','1985-09-02 00:00:00',623.73,270,35338,7.03,'GEO Transfer' from dual union all
select 17326,'017326','1987-007A','Kosmos-1815','Romb',60097,'Reentered','1988-11-15 00:00:00','1987-02-22 00:00:00',93.49,344,547,50.67,'Intermediate' from dual union all
select 17327,'017327','1987-007B','S3M','S3M',60097,'Reentered','1988-10-06 00:00:00','1987-02-23 00:00:00',93.38,338,541,50.67,'Intermediate' from dual union all
select 17328,'017328','1987-008A','Molniya-3','Molniya-3 No. 42',60098,'In Earth orbit','','1987-02-23 00:00:00',717.74,461,39892,62.85,'Molniya' from dual union all
select 17329,'017329','1987-008B','11S510','11S510',60098,'Reentered','1987-03-01 00:00:00','1987-01-29 00:00:00',90.71,209,410,62.79,'Intermediate' from dual union all
select 1733,'001733','1965-092A','Venera-3','3MV-3 No. 1',11842,'Deep Space','1965-11-18 00:00:00','1965-11-16 00:00:00',89.63,213,300,51.85,'Intermediate' from dual union all
select 17330,'017330','1987-008C','BOZ','BOZ',60098,'Reentered','1987-02-25 00:00:00','1987-01-29 00:00:00',90.82,197,433,62.83,'Intermediate' from dual union all
select 17331,'017331','1983-058F','deb Ariane V14','',55815,'Reentered','1990-05-16 00:00:00','1987-02-06 00:00:00',547.81,244,31355,7.5,'Highly Ellip' from dual union all
select 17332,'017332','1985-056E','deb Ariane V14','',58279,'Reentered','2003-06-19 00:00:00','1986-04-30 00:00:00',622.36,410,35127,7.42,'GEO Transfer' from dual union all
select 17333,'017333','1987-008D','Blok-ML','Blok-ML No. 77352-525',60098,'In Earth orbit','','1987-02-22 00:00:00',730.79,471,40524,62.86,'Molniya' from dual union all
select 17334,'017334','1986-067L','Kosmos-1776 SS 5','ESO',59260,'Reentered','1987-12-04 00:00:00','1987-01-31 00:00:00',94.55,480,513,74.12,'Intermediate' from dual union all
select 17335,'017335','1986-067M','Kosmos-1776 SS 6','ESO',59260,'Reentered','1987-11-09 00:00:00','1987-01-28 00:00:00',94.38,464,514,74.05,'Intermediate' from dual union all
select 17336,'017336','1986-019DH','deb Ariane V16','',59212,'Reentered','1989-11-10 00:00:00','1987-01-27 00:00:00',101.91,777,921,97.97,'Sun-Synch' from dual union all
select 17337,'017337','1986-019DJ','deb Ariane V16','',59212,'Reentered','1989-03-06 00:00:00','1987-01-27 00:00:00',102.28,789,945,98.6,'Sun-Synch' from dual union all
select 17338,'017338','1986-019DK','deb Ariane V16','',59212,'Reentered','1989-09-27 00:00:00','1987-01-27 00:00:00',102.61,782,982,97.87,'Sun-Synch' from dual union all
select 17339,'017339','1986-019DL','deb Ariane V16','',59212,'Reentered','1990-04-20 00:00:00','1987-01-27 00:00:00',103.04,780,1025,98.11,'Sun-Synch' from dual union all
select 1734,'001734','1965-092B','11S510','11S510',11842,'Reentered','1965-11-26 00:00:00','1965-11-24 00:00:00',89.39,203,286,51.86,'Intermediate' from dual union all
select 17340,'017340','1986-019DM','deb Ariane V16','',59212,'Reentered','1988-11-04 00:00:00','1987-01-27 00:00:00',103.21,758,1063,99.87,'Sun-Synch' from dual union all
select 17341,'017341','1986-019DN','deb Ariane V16','',59212,'Reentered','1991-01-01 00:00:00','1987-01-27 00:00:00',103.28,793,1035,97.89,'Sun-Synch' from dual union all
select 17342,'017342','1986-019DP','deb Ariane V16','',59212,'Reentered','1998-10-17 00:00:00','1993-08-06 00:00:00',98.17,625,717,98.19,'Sun-Synch' from dual union all
select 17343,'017343','1986-019DQ','deb Ariane V16','',59212,'Reentered','1991-12-27 00:00:00','1991-12-26 00:00:00',90.5,285,313,98.53,'Sun-Synch' from dual union all
select 17344,'017344','1986-019DR','deb Ariane V16','',59212,'Reentered','1989-01-19 00:00:00','1987-01-27 00:00:00',100.95,773,835,98.64,'Sun-Synch' from dual union all
select 17345,'017345','1986-019DS','deb Ariane V16','',59212,'Reentered','1989-04-16 00:00:00','1987-01-27 00:00:00',100.43,759,798,98.77,'Sun-Synch' from dual union all
select 17346,'017346','1986-019DT','deb Ariane V16','',59212,'Reentered','1989-05-11 00:00:00','1987-01-27 00:00:00',100.8,784,809,98.65,'Sun-Synch' from dual union all
select 17347,'017347','1986-019DU','deb Ariane V16','',59212,'Reentered','1987-11-15 00:00:00','1987-01-27 00:00:00',98.8,645,758,98.47,'Sun-Synch' from dual union all
select 17348,'017348','1986-019DV','deb Ariane V16','',59212,'Reentered','1988-10-12 00:00:00','1987-01-27 00:00:00',100.62,743,833,99.94,'Sun-Synch' from dual union all
select 17349,'017349','1986-019DW','deb Ariane V16','',59212,'Reentered','1989-02-13 00:00:00','1987-01-27 00:00:00',101.3,718,923,98.65,'Sun-Synch' from dual union all
select 1735,'001735','1965-092C','BOZ','BOZ',11842,'Reentered','1965-12-03 00:00:00','1965-11-17 00:00:00',89.56,213,293,51.85,'Intermediate' from dual union all
select 17350,'017350','1986-019DX','deb Ariane V16','',59212,'Reentered','1990-02-25 00:00:00','1987-01-27 00:00:00',102.02,789,920,98.59,'Sun-Synch' from dual union all
select 17351,'017351','1986-019DY','deb Ariane V16','',59212,'Reentered','1990-02-28 00:00:00','1987-01-27 00:00:00',102.43,786,962,97.9,'Sun-Synch' from dual union all
select 17352,'017352','1986-019DZ','deb Ariane V16','',59212,'Reentered','1989-06-30 00:00:00','1987-01-27 00:00:00',102.26,776,955,97.84,'Sun-Synch' from dual union all
select 17353,'017353','1986-019EA','deb Ariane V16','[UNID147] RCS 0.0',59212,'Lost','','1993-08-06 00:00:00',99.62,623,858,98.73,'Sun-Synch' from dual union all
select 17354,'017354','1986-019EB','deb Ariane V16','',59212,'Reentered','1989-02-17 00:00:00','1987-01-27 00:00:00',101.21,798,834,97.87,'Sun-Synch' from dual union all
select 17355,'017355','1986-019EC','deb Ariane V16','',59212,'Reentered','1989-12-03 00:00:00','1987-01-27 00:00:00',101.21,798,834,98.58,'Sun-Synch' from dual union all
select 17356,'017356','1986-067N','Kosmos-1776 SS 7','ESO',59260,'Reentered','1987-12-27 00:00:00','1987-01-31 00:00:00',94.55,480,513,74,'Intermediate' from dual union all
select 17357,'017357','1986-067P','Kosmos-1776 SS 8','ESO',59260,'Reentered','1987-11-22 00:00:00','1987-01-29 00:00:00',94.4,465,513,73.93,'Intermediate' from dual union all
select 17358,'017358','1984-106G','deb SOZ','',57160,'Reentered','1993-01-03 00:00:00','1992-10-06 00:00:00',103.23,719,1103,66.96,'Intermediate' from dual union all
select 17359,'017359','1987-009A','Kosmos-1816','Tsikada',60099,'In Earth orbit','','1987-03-02 00:00:00',104.85,964,1011,82.92,'Intermediate' from dual union all
select 1736,'001736','1965-092D','Blok-L','Blok-L',11842,'Deep Space','1965-11-18 00:00:00','1965-11-16 00:00:00',89.63,213,300,51.85,'Intermediate' from dual union all
select 17360,'017360','1987-009B','S3M','S3M',60099,'In Earth orbit','','1987-03-02 00:00:00',104.7,953,1008,82.92,'Intermediate' from dual union all
select 17361,'017361','1987-004C','Blenda','Sunshade',60094,'Reentered','1987-06-27 00:00:00','1987-03-18 00:00:00',91.8,313,413,72.77,'Intermediate' from dual union all
select 17362,'017362','1987-004D','deb Kosmos- 1813','',60094,'Reentered','1987-02-01 00:00:00','1987-01-29 00:00:00',92.14,347,412,72.84,'Intermediate' from dual union all
select 17363,'017363','1987-004E','deb Kosmos- 1813','',60094,'Reentered','1987-04-25 00:00:00','1987-02-14 00:00:00',92.99,300,542,72.83,'Intermediate' from dual union all
select 17364,'017364','1987-004F','deb Kosmos- 1813','',60094,'Reentered','1987-06-14 00:00:00','1987-03-07 00:00:00',91.12,273,386,72.81,'Intermediate' from dual union all
select 17365,'017365','1987-010A','Kosmos-1817','Ekran-M No. 11L',60100,'Reentered','1987-03-18 00:00:00','1987-01-31 00:00:00',89.15,212,253,51.58,'Intermediate' from dual union all
select 17366,'017366','1987-010B','8S812','8S812',60100,'Reentered','1987-02-02 00:00:00','1987-01-31 00:00:00',88.15,180,186,51.62,'Intermediate' from dual union all
select 17367,'017367','1987-010C','Perekhodnik','Sredniy perekhodnik',60100,'Reentered','1987-01-31 00:00:00','1987-01-31 00:00:00',87.64,154,162,51.62,'Intermediate' from dual union all
select 17368,'017368','1987-010D','Blok DM2','11S861 No. 17L',60100,'Reentered','1987-02-13 00:00:00','1987-02-04 00:00:00',88.98,207,241,51.58,'Intermediate' from dual union all
select 17369,'017369','1987-011A','Kosmos-1818','Plazma-A No. 1',60101,'In Earth orbit','','1987-03-04 00:00:00',100.75,788,800,65.01,'Intermediate' from dual union all
select 1737,'001737','1965-092E','deb Blok-I','',11842,'Reentered','1965-11-16 00:00:00','1965-11-16 00:00:00',89.53,213,290,51.85,'Intermediate' from dual union all
select 17370,'017370','1987-011B','deb Kosmos- 1818','',60101,'Reentered','1987-02-02 00:00:00','1987-02-02 00:00:00',87.81,94,239,65.01,'Intermediate' from dual union all
select 17371,'017371','1986-019ED','deb Ariane V16','',59212,'Reentered','1989-01-09 00:00:00','1987-02-01 00:00:00',99.54,689,785,99.07,'Sun-Synch' from dual union all
select 17372,'017372','1986-019EE','deb Ariane V16','',59212,'Reentered','1989-01-27 00:00:00','1987-02-02 00:00:00',101.28,772,867,98.58,'Sun-Synch' from dual union all
select 17373,'017373','1986-019EF','deb Ariane V16','',59212,'Reentered','1989-01-01 00:00:00','1987-02-02 00:00:00',99.68,694,793,98.88,'Sun-Synch' from dual union all
select 17374,'017374','1986-019EG','deb Ariane V16','',59212,'Reentered','1990-01-05 00:00:00','1991-01-05 00:00:00',91.27,315,359,97.91,'Sun-Synch' from dual union all
select 17375,'017375','1986-019EH','deb Ariane V16','',59212,'Reentered','1988-12-09 00:00:00','1987-02-02 00:00:00',102.83,766,1019,97.83,'Sun-Synch' from dual union all
select 17376,'017376','1986-019EJ','deb Ariane V16','',59212,'Reentered','1990-08-24 00:00:00','1990-08-23 00:00:00',91.02,307,343,98.14,'Sun-Synch' from dual union all
select 17377,'017377','1986-019EK','deb Ariane V16','',59212,'Reentered','1988-02-15 00:00:00','1987-02-02 00:00:00',99.75,596,898,99.62,'Sun-Synch' from dual union all
select 17378,'017378','1986-019EL','deb Ariane V16','',59212,'Reentered','1989-04-09 00:00:00','1987-02-02 00:00:00',100.5,766,799,98.71,'Sun-Synch' from dual;
insert into SATELLITE
select 17379,'017379','1986-019EM','deb Ariane V16','',59212,'Reentered','1990-12-04 00:00:00','1987-02-02 00:00:00',102.36,783,958,99.23,'Sun-Synch' from dual union all
select 1738,'001738','1965-093A','SR 8','Solar Explorer A',11843,'In Earth orbit','','1966-04-19 00:00:00',100.77,693,898,59.71,'Intermediate' from dual union all
select 17380,'017380','1986-019EN','deb Ariane V16','',59212,'Reentered','1990-09-10 00:00:00','1987-02-02 00:00:00',103.02,798,1005,97.94,'Sun-Synch' from dual union all
select 17381,'017381','1986-019EP','deb Ariane V16','',59212,'Reentered','1988-10-05 00:00:00','1987-02-02 00:00:00',98.6,589,795,99.05,'Sun-Synch' from dual union all
select 17382,'017382','1986-019EQ','deb Ariane V16','',59212,'Reentered','1989-12-19 00:00:00','1987-02-02 00:00:00',101.69,784,894,98.57,'Sun-Synch' from dual union all
select 17383,'017383','1986-019ER','deb Ariane V16','',59212,'Reentered','1989-03-08 00:00:00','1987-02-02 00:00:00',101.76,775,909,98.25,'Sun-Synch' from dual union all
select 17384,'017384','1986-019ES','deb Ariane V16','',59212,'Reentered','1989-02-16 00:00:00','1987-02-02 00:00:00',100.17,742,792,98.78,'Sun-Synch' from dual union all
select 17385,'017385','1986-019ET','deb Ariane V16','',59212,'Reentered','1989-04-24 00:00:00','1987-02-02 00:00:00',100.44,752,807,98.62,'Sun-Synch' from dual union all
select 17386,'017386','1986-019EU','deb Ariane V16','',59212,'Reentered','1989-03-01 00:00:00','1987-02-02 00:00:00',101.95,723,979,97.96,'Sun-Synch' from dual union all
select 17387,'017387','1986-019EV','deb Ariane V16','',59212,'Reentered','1989-03-05 00:00:00','1987-02-02 00:00:00',101.95,734,968,99.15,'Sun-Synch' from dual union all
select 17388,'017388','1986-019EW','deb Ariane V16','',59212,'Reentered','1988-09-26 00:00:00','1987-02-02 00:00:00',98.65,597,792,98.98,'Sun-Synch' from dual union all
select 17389,'017389','1986-019EX','deb Ariane V16','',59212,'Reentered','1988-10-05 00:00:00','1987-02-02 00:00:00',98.7,599,793,99.04,'Sun-Synch' from dual union all
select 1739,'001739','1965-093B','Altair 2 X-258E6 RH-125','X-258E6',11843,'In Earth orbit','','1966-02-12 00:00:00',100.74,714,874,59.73,'Intermediate' from dual union all
select 17390,'017390','1986-019EY','deb Ariane V16','',59212,'Reentered','1989-11-10 00:00:00','1987-02-02 00:00:00',100.74,732,856,98.24,'Sun-Synch' from dual union all
select 17391,'017391','1986-019EZ','deb Ariane V16','',59212,'Reentered','1989-07-09 00:00:00','1987-02-02 00:00:00',100.84,773,824,98.41,'Sun-Synch' from dual union all
select 17392,'017392','1986-019FA','deb Ariane V16','',59212,'Reentered','1992-05-13 00:00:00','1992-04-08 00:00:00',93.97,462,476,99.08,'Sun-Synch' from dual union all
select 17393,'017393','1986-019FB','deb Ariane V16','',59212,'Reentered','1988-12-29 00:00:00','1987-02-02 00:00:00',103.48,758,1089,99.51,'Sun-Synch' from dual union all
select 17394,'017394','1986-019FC','deb Ariane V16','',59212,'Reentered','1988-12-19 00:00:00','1987-02-02 00:00:00',99.09,640,791,99.02,'Sun-Synch' from dual union all
select 17395,'017395','1986-019FD','deb Ariane V16','',59212,'Reentered','1989-01-17 00:00:00','1987-02-02 00:00:00',99.39,660,799,98.99,'Sun-Synch' from dual union all
select 17396,'017396','1986-019FE','deb Ariane V16','',59212,'Reentered','1989-04-16 00:00:00','1987-02-02 00:00:00',100.2,738,798,98.86,'Sun-Synch' from dual union all
select 17397,'017397','1986-019FF','deb Ariane V16','',59212,'Reentered','1989-04-14 00:00:00','1987-02-02 00:00:00',100.91,732,872,99.03,'Sun-Synch' from dual union all
select 17398,'017398','1986-060C','Kosmos-1770 AO part','Terilen AO part',59253,'Reentered','1987-02-05 00:00:00','1987-02-02 00:00:00',90.33,204,377,64.74,'Intermediate' from dual union all
select 17399,'017399','1987-011C','Kosmos-1818 sensor cover','',60101,'Reentered','1990-07-29 00:00:00','1987-03-03 00:00:00',100.72,787,799,65.01,'Intermediate' from dual union all
select 174,'000174','61 Phi 2','Agena B 6001','Agena B 6001',7274,'Reentered','1961-09-03 00:00:00','1961-08-27 00:00:00',90.25,176,397,32.88,'Intermediate' from dual union all
select 1740,'001740','1965-088D','deb Kosmos-95','',11838,'Reentered','1966-01-06 00:00:00','1966-01-03 00:00:00',88.6,188,222,48.4,'Intermediate' from dual union all
select 17400,'017400','1987-011D','Kosmos-1818 reactor shield','Thermal shield',60101,'Reentered','1990-05-04 00:00:00','1987-03-03 00:00:00',100.72,787,799,65.01,'Intermediate' from dual union all
select 17401,'017401','1987-004G','deb Kosmos-1813','',60094,'Reentered','1987-06-04 00:00:00','1987-06-03 00:00:00',88.83,200,234,72.79,'Intermediate' from dual union all
select 17402,'017402','1987-004H','deb Kosmos-1813','',60094,'Reentered','1988-12-30 00:00:00','1987-08-21 00:00:00',93.36,345,532,72.8,'Intermediate' from dual union all
select 17403,'017403','1987-004J','deb Kosmos-1813','',60094,'Reentered','1987-02-24 00:00:00','1987-03-17 00:00:00',89.61,200,311,72.75,'Intermediate' from dual union all
select 17404,'017404','1987-004K','deb Kosmos-1813','',60094,'Reentered','1988-12-29 00:00:00','1987-08-21 00:00:00',94.64,362,641,72.8,'Intermediate' from dual union all
select 17405,'017405','1987-004L','deb Kosmos-1813','',60094,'Reentered','1988-08-24 00:00:00','1987-08-07 00:00:00',92.87,376,455,72.8,'Intermediate' from dual union all
select 17406,'017406','1987-004M','deb Kosmos-1813','',60094,'Reentered','1988-11-07 00:00:00','1987-08-21 00:00:00',93.58,373,526,72.8,'Intermediate' from dual union all
select 17407,'017407','1987-004N','deb Kosmos-1813','',60094,'Reentered','1988-11-16 00:00:00','1987-08-21 00:00:00',92.85,375,453,72.8,'Intermediate' from dual union all
select 17408,'017408','1987-004P','deb Kosmos-1813','',60094,'Reentered','1989-01-22 00:00:00','1987-08-21 00:00:00',96.54,306,880,72.8,'Intermediate' from dual union all
select 17409,'017409','1987-004Q','deb Kosmos-1813','',60094,'Reentered','1987-02-22 00:00:00','1987-02-20 00:00:00',89.32,197,284,72.71,'Intermediate' from dual union all
select 1741,'001741','1965-088E','deb Kosmos-95','',11838,'Reentered','1965-12-13 00:00:00','1965-11-05 00:00:00',90.78,206,420,48.4,'Intermediate' from dual union all
select 17410,'017410','1987-004R','deb Kosmos-1813','',60094,'Reentered','1987-03-28 00:00:00','1987-02-03 00:00:00',90.96,248,396,72.82,'Intermediate' from dual union all
select 17411,'017411','1987-004S','deb Kosmos-1813','',60094,'Reentered','1987-02-20 00:00:00','1987-02-11 00:00:00',90.56,266,338,72.78,'Intermediate' from dual union all
select 17412,'017412','1987-004T','deb Kosmos-1813 part','Oblik debris',60094,'Reentered','1988-01-07 00:00:00','1987-02-03 00:00:00',92.35,368,412,72.81,'Intermediate' from dual union all
select 17413,'017413','1987-004U','deb Kosmos-1813','',60094,'Reentered','1987-07-27 00:00:00','1987-07-27 00:00:00',88.12,171,191,72.8,'Intermediate' from dual union all
select 17414,'017414','1987-004V','deb Kosmos-1813','',60094,'Reentered','1989-01-17 00:00:00','1987-08-21 00:00:00',98.94,388,1028,72.8,'Intermediate' from dual union all
select 17415,'017415','1987-004W','deb Kosmos-1813','',60094,'Reentered','1989-06-13 00:00:00','1987-02-03 00:00:00',91.88,333,400,72.82,'Intermediate' from dual union all
select 17416,'017416','1987-004X','deb Kosmos-1813','',60094,'Reentered','1988-06-02 00:00:00','1987-08-21 00:00:00',92.63,375,432,72.8,'Intermediate' from dual union all
select 17417,'017417','1987-004Y','deb Kosmos-1813','',60094,'Reentered','1988-01-18 00:00:00','1987-08-22 00:00:00',94.71,328,682,72.8,'Intermediate' from dual union all
select 17418,'017418','1987-004Z','deb Kosmos-1813','',60094,'Reentered','1987-03-06 00:00:00','1987-02-03 00:00:00',90.78,243,382,72.81,'Intermediate' from dual union all
select 17419,'017419','1987-004AA','deb Kosmos-1813','',60094,'Reentered','1987-08-31 00:00:00','1987-02-03 00:00:00',91.73,323,396,72.81,'Intermediate' from dual union all
select 1742,'001742','1965-094A','Kosmos-96','3MV-4 No. 6',11844,'Reentered Attached','1965-12-09 00:00:00','1965-11-23 00:00:00',89.21,209,262,51.9,'Intermediate' from dual union all
select 17420,'017420','1987-004AB','deb Kosmos-1813','',60094,'Reentered','1987-12-10 00:00:00','1987-02-03 00:00:00',92.65,319,490,72.81,'Intermediate' from dual union all
select 17421,'017421','1986-019FG','deb Ariane V16','',59212,'Reentered','1989-02-10 00:00:00','1987-02-03 00:00:00',99.67,688,798,98.86,'Sun-Synch' from dual union all
select 17422,'017422','1986-019FH','deb Ariane V16','',59212,'Reentered','1988-07-23 00:00:00','1987-02-03 00:00:00',100.69,747,836,98.77,'Sun-Synch' from dual union all
select 17423,'017423','1986-019FJ','deb Ariane V16','',59212,'In Earth orbit','','1993-08-07 00:00:00',103.61,789,1069,98.96,'Sun-Synch' from dual union all
select 17424,'017424','1986-019FK','deb Ariane V16','',59212,'Reentered','1989-11-25 00:00:00','1987-02-03 00:00:00',101,766,846,98.58,'Sun-Synch' from dual union all
select 17425,'017425','1986-010C','deb CZ-3 Stage 3','',59203,'In Earth orbit','','1998-02-01 00:00:00',585.54,362,33251,30.51,'Highly Ellip' from dual union all
select 17426,'017426','1986-019FM','deb Ariane V16','',59212,'Reentered','2014-11-11 00:00:00','1993-08-02 00:00:00',99.85,731,772,98.31,'Sun-Synch' from dual union all
select 17427,'017427','1986-019FN','deb Ariane V16','',59212,'Reentered','1989-12-26 00:00:00','1987-02-03 00:00:00',101.07,789,830,98.23,'Sun-Synch' from dual union all
select 17428,'017428','1986-019FP','deb Ariane V16','',59212,'Reentered','1990-01-06 00:00:00','1987-02-03 00:00:00',101.21,782,850,98.63,'Sun-Synch' from dual union all
select 17429,'017429','1986-019FQ','deb Ariane V16','',59212,'In Earth orbit','','1993-08-02 00:00:00',101.54,762,902,99.01,'Sun-Synch' from dual union all
select 1743,'001743','1965-094B','11S510','11S510',11844,'Reentered','1965-12-04 00:00:00','1965-11-23 00:00:00',89.72,220,302,51.9,'Intermediate' from dual union all
select 17430,'017430','1986-019FR','deb Ariane V16','',59212,'Reentered','1988-08-10 00:00:00','1987-02-03 00:00:00',98.54,585,793,99.13,'Sun-Synch' from dual union all
select 17431,'017431','1986-019FS','deb Ariane V16','',59212,'Reentered','1989-03-25 00:00:00','1987-02-03 00:00:00',100.71,785,800,98.66,'Sun-Synch' from dual union all
select 17432,'017432','1986-019FT','deb Ariane V16','',59212,'Reentered','1988-03-13 00:00:00','1987-02-03 00:00:00',97.65,506,787,99.03,'Sun-Synch' from dual union all
select 17433,'017433','1986-019FU','deb Ariane V16','',59212,'Reentered','1989-08-20 00:00:00','1987-02-03 00:00:00',100.08,720,805,98.9,'Sun-Synch' from dual union all
select 17434,'017434','1986-019FV','deb Ariane V16','',59212,'Reentered','1989-04-01 00:00:00','1987-02-03 00:00:00',100.42,703,854,98.81,'Sun-Synch' from dual union all
select 17435,'017435','1986-019FW','deb Ariane V16','',59212,'Reentered','1989-12-31 00:00:00','1987-08-13 00:00:00',101.71,770,909,97.08,'Sun-Synch' from dual union all
select 17436,'017436','1986-019FX','deb Ariane V16','',59212,'Reentered','1989-02-06 00:00:00','1987-02-03 00:00:00',100.97,788,821,98.65,'Sun-Synch' from dual union all
select 17437,'017437','1986-019FY','deb Ariane V16','',59212,'Reentered','1988-10-19 00:00:00','1987-02-03 00:00:00',100.41,776,780,98.68,'Sun-Synch' from dual union all
select 17438,'017438','1986-019FZ','deb Ariane V16','',59212,'Reentered','1991-04-15 00:00:00','1987-02-03 00:00:00',103.81,920,958,97.97,'Sun-Synch' from dual union all
select 17439,'017439','1986-019GA','deb Ariane V16','',59212,'Reentered','1989-04-10 00:00:00','1987-02-03 00:00:00',98.99,624,797,98.89,'Sun-Synch' from dual union all
select 1744,'001744','1965-055D','Ferret sensor cover','',11805,'Reentered','1966-04-16 00:00:00','1966-01-30 00:00:00',93.72,439,474,70.14,'Intermediate' from dual union all
select 17440,'017440','1986-019GB','deb Ariane V16','',59212,'Reentered','1987-10-29 00:00:00','1987-02-03 00:00:00',98.8,603,800,99.38,'Sun-Synch' from dual union all
select 17441,'017441','1986-019GC','deb Ariane V16','',59212,'Reentered','1988-11-10 00:00:00','1987-08-16 00:00:00',100.61,637,938,99.45,'Sun-Synch' from dual union all
select 17442,'017442','1986-019GD','deb Ariane V16','',59212,'Reentered','1989-12-05 00:00:00','1987-08-16 00:00:00',101.48,808,849,98.34,'Sun-Synch' from dual union all
select 17443,'017443','1986-019GE','deb Ariane V16','',59212,'Reentered','1989-11-15 00:00:00','1987-08-16 00:00:00',101.94,729,972,98.79,'Sun-Synch' from dual union all
select 17444,'017444','1986-019GF','deb Ariane V16','',59212,'Reentered','1991-02-03 00:00:00','1987-08-16 00:00:00',103.87,789,1093,97.97,'Sun-Synch' from dual union all
select 17445,'017445','1986-019GG','deb Ariane V16','',59212,'Reentered','1990-07-24 00:00:00','1987-08-16 00:00:00',102.7,774,999,98.45,'Sun-Synch' from dual union all
select 17446,'017446','1986-019GH','deb Ariane V16','',59212,'Reentered','1989-10-20 00:00:00','1987-08-15 00:00:00',100.62,774,801,98.78,'Sun-Synch' from dual union all
select 17447,'017447','1986-019GJ','deb Ariane V16','',59212,'Reentered','1988-07-20 00:00:00','1987-08-15 00:00:00',97.92,549,770,99.1,'Sun-Synch' from dual union all
select 17448,'017448','1986-019GK','deb Ariane V16','',59212,'Reentered','1988-09-09 00:00:00','1987-08-15 00:00:00',97.93,543,776,98.96,'Sun-Synch' from dual union all
select 17449,'017449','1986-019GL','deb Ariane V16','',59212,'Reentered','1989-06-17 00:00:00','1987-02-03 00:00:00',101.06,781,837,98.54,'Sun-Synch' from dual union all
select 1745,'001745','1965-055E','Ferret sensor cover','',11805,'Reentered','1966-03-23 00:00:00','1966-02-01 00:00:00',93.51,433,459,70.17,'Intermediate' from dual union all
select 17450,'017450','1986-019GM','deb Ariane V16','',59212,'Reentered','1989-08-18 00:00:00','1987-02-03 00:00:00',102.86,775,1013,97.81,'Sun-Synch' from dual union all
select 17451,'017451','1986-019GN','deb Ariane V16','',59212,'Reentered','1989-11-19 00:00:00','1987-02-03 00:00:00',103.77,774,1100,98.97,'Sun-Synch' from dual union all
select 17452,'017452','1986-019GP','deb Ariane V16','',59212,'Reentered','1989-02-27 00:00:00','1987-02-03 00:00:00',100.15,733,798,98.81,'Sun-Synch' from dual union all
select 17453,'017453','1986-019GQ','deb Ariane V16','',59212,'Reentered','1989-01-18 00:00:00','',100.46,702,859,99.13,'Sun-Synch' from dual union all
select 17454,'017454','1986-019GR','deb Ariane V16','',59212,'Reentered','1989-06-12 00:00:00','',100.71,779,806,98.68,'Sun-Synch' from dual union all
select 17455,'017455','1986-019GS','deb Ariane V16','',59212,'Reentered','1991-07-20 00:00:00','',103.25,789,1036,98.03,'Sun-Synch' from dual union all
select 17456,'017456','1986-019GT','deb Ariane V16','',59212,'Reentered','1988-12-23 00:00:00','',99.89,710,796,98.82,'Sun-Synch' from dual union all
select 17457,'017457','1986-019GU','deb Ariane V16','',59212,'Reentered','1990-05-19 00:00:00','',101.96,790,913,98.53,'Sun-Synch' from dual union all
select 17458,'017458','1986-019GV','deb Ariane V16','',59212,'Reentered','1990-09-29 00:00:00','',103.46,805,1040,97.91,'Sun-Synch' from dual union all
select 17459,'017459','1986-019GW','deb Ariane V16','',59212,'Reentered','2014-09-24 00:00:00','1993-08-07 00:00:00',100.12,739,790,98.34,'Sun-Synch' from dual union all
select 1746,'001746','1965-082CH','deb Transtage 4','',11832,'Reentered','1978-05-03 00:00:00','1978-03-26 00:00:00',93.51,446,447,32.41,'Intermediate' from dual union all
select 17460,'017460','1986-019GX','deb Ariane V16','',59212,'Reentered','1989-03-16 00:00:00','',102.23,749,979,98.38,'Sun-Synch' from dual union all
select 17461,'017461','1986-019GY','deb Ariane V16','',59212,'Reentered','1990-11-17 00:00:00','1987-02-03 00:00:00',100.83,785,811,98.68,'Sun-Synch' from dual union all
select 17462,'017462','1986-019GZ','deb Ariane V16','',59212,'Reentered','1988-10-15 00:00:00','1987-02-03 00:00:00',101.11,753,870,98.68,'Sun-Synch' from dual union all
select 17463,'017463','1986-019HA','deb Ariane V16','',59212,'Reentered','1988-10-20 00:00:00','1987-02-03 00:00:00',98.96,619,799,99.02,'Sun-Synch' from dual union all
select 17464,'017464','1986-019HB','deb Ariane V16','',59212,'Reentered','1990-04-22 00:00:00','1987-02-03 00:00:00',102.56,802,958,98.68,'Sun-Synch' from dual union all
select 17465,'017465','1986-019HC','deb Ariane V16','',59212,'Reentered','1990-01-12 00:00:00','',103.08,787,1022,98.15,'Sun-Synch' from dual union all
select 17466,'017466','1986-019HD','deb Ariane V16','',59212,'Reentered','1994-05-21 00:00:00','1993-09-15 00:00:00',94.97,515,520,98.21,'Sun-Synch' from dual union all
select 17467,'017467','1986-019HE','deb Ariane V16','',59212,'Reentered','1988-09-19 00:00:00','',100.76,778,812,98.7,'Sun-Synch' from dual union all
select 17468,'017468','1986-019HF','deb Ariane V16','',59212,'Reentered','1988-11-07 00:00:00','',100.07,705,819,98.82,'Sun-Synch' from dual;
insert into SATELLITE
select 17469,'017469','1986-019HG','deb Ariane V16','',59212,'Reentered','1989-01-31 00:00:00','',102.17,666,1057,98.82,'Sun-Synch' from dual union all
select 1747,'001747','1965-082CJ','deb Transtage 4','',11832,'Reentered','1980-08-19 00:00:00','1980-08-16 00:00:00',91.6,348,358,31.89,'Intermediate' from dual union all
select 17470,'017470','1986-019HH','deb Ariane V16','',59212,'Reentered','1988-10-16 00:00:00','',100,687,830,98.42,'Sun-Synch' from dual union all
select 17471,'017471','1986-019HJ','deb Ariane V16','',59212,'Reentered','2001-06-15 00:00:00','1993-08-06 00:00:00',99.8,714,784,98.54,'Sun-Synch' from dual union all
select 17472,'017472','1986-019HK','deb Ariane V16','',59212,'Reentered','1991-04-29 00:00:00','',103.79,785,1091,98.03,'Sun-Synch' from dual union all
select 17473,'017473','1986-019HL','deb Ariane V16','',59212,'Reentered','1989-11-26 00:00:00','',101.16,796,831,98.59,'Sun-Synch' from dual union all
select 17474,'017474','1986-019HM','deb Ariane V16','',59212,'Reentered','1988-10-26 00:00:00','',100.05,749,773,98.76,'Sun-Synch' from dual union all
select 17475,'017475','1986-019HN','deb Ariane V16','',59212,'Reentered','1988-12-02 00:00:00','',100.08,705,819,98.7,'Sun-Synch' from dual union all
select 17476,'017476','1986-019HP','deb Ariane V16','',59212,'Reentered','1989-08-26 00:00:00','',101.7,781,898,98.61,'Sun-Synch' from dual union all
select 17477,'017477','1986-019HQ','deb Ariane V16','',59212,'Reentered','1989-07-13 00:00:00','1987-02-03 00:00:00',101.06,757,861,98.57,'Sun-Synch' from dual union all
select 17478,'017478','1986-019HR','deb Ariane V16','',59212,'In Earth orbit','','1993-08-01 00:00:00',101.35,791,854,98.4,'Sun-Synch' from dual union all
select 17479,'017479','1986-019HS','deb Ariane V16','',59212,'Reentered','1989-07-16 00:00:00','',101.31,792,850,98.55,'Sun-Synch' from dual union all
select 1748,'001748','1965-082CK','deb Transtage 4','',11832,'Reentered','1991-05-30 00:00:00','1965-11-24 00:00:00',106.88,736,1428,32.24,'Intermediate' from dual union all
select 17480,'017480','1987-012A','Ginga','ASTRO-C',60102,'Reentered','1991-11-01 00:00:00','1987-03-09 00:00:00',96.5,509,673,31.09,'Intermediate' from dual union all
select 17481,'017481','1987-012B','M-3B','M-3B',60102,'Reentered','1997-06-28 00:00:00','1987-04-29 00:00:00',96.88,514,705,31.08,'Intermediate' from dual union all
select 17482,'017482','1987-013A','Soyuz TM-2','Soyuz 7K-STM No. 52',60103,'Landed','1987-07-30 00:00:00','1987-03-09 00:00:00',91.66,344,367,51.62,'Intermediate' from dual union all
select 17483,'017483','1987-013B','11S510','11S510',60103,'Reentered','1987-02-08 00:00:00','1987-02-06 00:00:00',88.36,191,196,51.62,'Intermediate' from dual union all
select 17484,'017484','1987-014A','Kosmos-1819','Oblik No. 352',60104,'Landed','1987-02-18 00:00:00','1987-02-09 00:00:00',89.13,209,254,72.84,'Intermediate' from dual union all
select 17485,'017485','1987-014B','11S510','11S510',60104,'Reentered','1987-02-10 00:00:00','1987-02-09 00:00:00',88.07,168,190,72.84,'Intermediate' from dual union all
select 17486,'017486','1987-014C','Sensor cover','Sensor cover',60104,'Reentered','1987-02-07 00:00:00','1987-02-07 00:00:00',87.61,132,181,72.84,'Intermediate' from dual union all
select 17487,'017487','1987-014D','Sensor cover','Sensor cover',60104,'Reentered','1987-02-08 00:00:00','1987-02-07 00:00:00',88.58,185,224,72.83,'Intermediate' from dual union all
select 17488,'017488','1987-004AC','deb Kosmos-1813','',60094,'Reentered','1988-10-05 00:00:00','1987-02-09 00:00:00',96.56,386,802,72.8,'Intermediate' from dual union all
select 17489,'017489','1987-004AD','deb Kosmos-1813','',60094,'Reentered','1988-09-22 00:00:00','1987-02-09 00:00:00',101.48,350,1307,72.8,'Intermediate' from dual union all
select 1749,'001749','1965-082CL','deb Transtage 4','',11832,'Reentered','1979-12-09 00:00:00','1979-12-04 00:00:00',90.94,311,330,31.9,'Intermediate' from dual union all
select 17490,'017490','1987-004AE','deb Kosmos-1813','',60094,'Reentered','1987-03-22 00:00:00','1987-02-09 00:00:00',91.33,237,443,72.8,'Intermediate' from dual union all
select 17491,'017491','1987-004AF','deb Kosmos-1813','',60094,'Reentered','1987-10-07 00:00:00','1987-02-09 00:00:00',91.88,332,402,72.81,'Intermediate' from dual union all
select 17492,'017492','1987-004AG','deb Kosmos-1813','',60094,'Reentered','1988-11-17 00:00:00','1987-02-09 00:00:00',94.43,388,594,72.8,'Intermediate' from dual union all
select 17493,'017493','1987-004AH','deb Kosmos-1813','',60094,'Reentered','1988-11-03 00:00:00','1987-02-09 00:00:00',93.78,387,531,72.8,'Intermediate' from dual union all
select 17494,'017494','1987-004AJ','deb Kosmos-1813','',60094,'Reentered','1988-09-03 00:00:00','1987-02-09 00:00:00',94.07,351,596,72.8,'Intermediate' from dual union all
select 17495,'017495','1987-004AK','deb Kosmos-1813','',60094,'Reentered','1988-12-07 00:00:00','1987-02-09 00:00:00',93.42,352,532,72.81,'Intermediate' from dual union all
select 17496,'017496','1987-004AL','deb Kosmos-1813','',60094,'Reentered','1987-04-14 00:00:00','1987-02-09 00:00:00',93.06,311,537,72.81,'Intermediate' from dual union all
select 17497,'017497','1987-004AM','deb Kosmos-1813','',60094,'Reentered','1987-03-27 00:00:00','1987-02-09 00:00:00',91.6,322,385,72.8,'Intermediate' from dual union all
select 17498,'017498','1987-004AN','deb Kosmos-1813','',60094,'Reentered','1988-12-05 00:00:00','1987-02-09 00:00:00',101.23,771,863,72.84,'Intermediate' from dual union all
select 17499,'017499','1987-004AP','deb Kosmos-1813','',60094,'Reentered','1988-04-22 00:00:00','1987-02-09 00:00:00',92.55,384,415,72.8,'Intermediate' from dual union all
select 175,'000175','61 Omicron 48','deb r/b Ablestar','',7273,'Reentered','1999-03-12 00:00:00','1961-09-03 00:00:00',106.42,1061,1061,66.82,'Intermediate' from dual union all
select 1750,'001750','1965-082CM','deb Transtage 4','',11832,'Reentered','1970-02-08 00:00:00','1970-01-29 00:00:00',91.84,356,374,32.54,'Intermediate' from dual union all
select 17500,'017500','1987-004AQ','deb Kosmos-1813','',60094,'Reentered','1990-01-30 00:00:00','1987-02-09 00:00:00',97.32,387,874,72.8,'Intermediate' from dual union all
select 17501,'017501','1987-004AR','deb Kosmos-1813','',60094,'Reentered','1988-10-25 00:00:00','1987-02-11 00:00:00',92.89,369,464,72.82,'Intermediate' from dual union all
select 17502,'017502','1987-004AS','deb Kosmos-1813','',60094,'Reentered','1988-12-31 00:00:00','1987-02-11 00:00:00',100.11,374,1153,72.81,'Intermediate' from dual union all
select 17503,'017503','1987-004AT','deb Kosmos-1813','',60094,'Reentered','1987-09-16 00:00:00','1987-08-21 00:00:00',90.87,309,326,72.8,'Intermediate' from dual union all
select 17504,'017504','1987-004AU','deb Kosmos-1813','',60094,'Reentered','1987-09-02 00:00:00','1987-08-21 00:00:00',89.7,247,273,72.8,'Intermediate' from dual union all
select 17505,'017505','1987-004AV','deb Kosmos-1813','',60094,'Reentered','1987-10-27 00:00:00','1987-08-21 00:00:00',89.72,248,274,72.8,'Intermediate' from dual union all
select 17506,'017506','1987-015A','USA 21','JUMPSEAT 8 (or QUASAR 7',60105,'In Earth orbit','','1987-03-01 00:00:00',717.88,610,39750,63.4,'Molniya' from dual union all
select 17507,'017507','1987-015B','Agena D 1524','Agena D',60105,'In Earth orbit','','1987-03-01 00:00:00',701.32,290,39250,63.4,'Molniya' from dual union all
select 17508,'017508','1987-004AW','deb Kosmos-1813 part','Oblik debris',60094,'Reentered','1987-04-14 00:00:00','1987-02-11 00:00:00',92.23,383,385,72.8,'Intermediate' from dual union all
select 17509,'017509','1987-004AX','deb Kosmos-1813','',60094,'Reentered','1987-08-11 00:00:00','1987-02-11 00:00:00',92.06,357,395,72.8,'Intermediate' from dual union all
select 1751,'001751','1965-082CN','deb Transtage 4','',11832,'Reentered','1979-10-21 00:00:00','1979-10-13 00:00:00',91.78,351,372,32.71,'Intermediate' from dual union all
select 17510,'017510','1987-004AY','deb Kosmos-1813','',60094,'Reentered','1987-06-30 00:00:00','1987-02-11 00:00:00',91.93,341,398,72.81,'Intermediate' from dual union all
select 17511,'017511','1987-004AZ','deb Kosmos-1813','',60094,'Reentered','1987-08-20 00:00:00','1987-02-11 00:00:00',92.05,341,410,72.81,'Intermediate' from dual union all
select 17512,'017512','1987-004BA','deb Kosmos-1813','',60094,'Reentered','1987-06-03 00:00:00','1987-02-11 00:00:00',92.44,371,418,72.81,'Intermediate' from dual union all
select 17513,'017513','1987-004BB','deb Kosmos-1813','',60094,'Reentered','1987-06-21 00:00:00','1987-02-11 00:00:00',93.63,374,530,72.79,'Intermediate' from dual union all
select 17514,'017514','1987-004BC','deb Kosmos-1813','',60094,'Reentered','1987-06-22 00:00:00','1987-02-11 00:00:00',91.59,310,396,72.81,'Intermediate' from dual union all
select 17515,'017515','1987-004BD','deb Kosmos-1813','',60094,'Reentered','1987-05-17 00:00:00','1987-02-11 00:00:00',92.19,354,409,72.82,'Intermediate' from dual union all
select 17516,'017516','1987-004BE','deb Kosmos-1813','',60094,'Reentered','1988-09-17 00:00:00','1987-02-11 00:00:00',92.2,361,404,72.81,'Intermediate' from dual union all
select 17517,'017517','1987-004BF','deb Kosmos-1813','',60094,'Reentered','1987-11-22 00:00:00','1987-02-11 00:00:00',91.69,307,408,72.82,'Intermediate' from dual union all
select 17518,'017518','1987-004BG','deb Kosmos-1813','',60094,'Reentered','1987-04-27 00:00:00','1987-02-11 00:00:00',91.72,330,387,72.8,'Intermediate' from dual union all
select 17519,'017519','1987-004BH','deb Kosmos-1813','',60094,'Reentered','1987-08-16 00:00:00','1987-02-11 00:00:00',91.8,320,405,72.81,'Intermediate' from dual union all
select 1752,'001752','1965-082CP','deb Transtage 4','',11832,'Reentered','1980-12-08 00:00:00','1966-01-24 00:00:00',98.12,598,740,32.03,'Intermediate' from dual union all
select 17520,'017520','1987-004BJ','deb Kosmos-1813','',60094,'Reentered','1987-05-05 00:00:00','1987-03-04 00:00:00',91.73,353,366,72.8,'Intermediate' from dual union all
select 17521,'017521','1987-004BK','deb Kosmos-1813','',60094,'Reentered','1988-07-12 00:00:00','1987-02-11 00:00:00',91.74,318,402,72.8,'Intermediate' from dual union all
select 17522,'017522','1987-004BL','deb Kosmos-1813','',60094,'Reentered','1987-11-29 00:00:00','1987-03-04 00:00:00',92.51,337,459,72.81,'Intermediate' from dual union all
select 17523,'017523','1987-016A','Kosmos-1820','EPN No. 03.695',60106,'Reentered','1987-03-06 00:00:00','1987-02-23 00:00:00',88.45,171,225,64.83,'Intermediate' from dual union all
select 17524,'017524','1987-016B','11S772','11S772',60106,'Reentered','1987-02-22 00:00:00','1987-02-16 00:00:00',88.61,177,235,64.84,'Intermediate' from dual union all
select 17525,'017525','1987-017A','Kosmos-1821','Parus',60107,'In Earth orbit','','1987-03-20 00:00:00',104.88,962,1015,82.92,'Intermediate' from dual union all
select 17526,'017526','1987-017B','S3M','S3M',60107,'In Earth orbit','','1987-03-22 00:00:00',104.71,951,1011,82.91,'Intermediate' from dual union all
select 17527,'017527','1987-018A','Momo','MOS-1',60108,'In Earth orbit','','1987-04-13 00:00:00',103.22,910,912,99.09,'Sun-Synch' from dual union all
select 17528,'017528','1987-018B','N-II Stage 2 S/N N-16(F)','AJ10-118FJ',60108,'In Earth orbit','','1987-03-23 00:00:00',100.23,636,903,97.46,'Sun-Synch' from dual union all
select 17529,'017529','1987-014E','Blenda','Sunshade',60104,'Reentered','1987-03-03 00:00:00','1987-02-20 00:00:00',91.49,240,456,72.84,'Intermediate' from dual union all
select 1753,'001753','1965-082CQ','deb Transtage 4','',11832,'Reentered','1990-04-30 00:00:00','1966-01-08 00:00:00',101.46,716,940,32.21,'Intermediate' from dual union all
select 17530,'017530','1987-014F','KDU','KDU',60104,'Reentered','1987-03-24 00:00:00','1987-02-20 00:00:00',89.71,225,296,72.84,'Intermediate' from dual union all
select 17531,'017531','1987-014G','KDU part','KDU part',60104,'Reentered','1987-02-23 00:00:00','1987-02-19 00:00:00',89.65,223,292,72.83,'Intermediate' from dual union all
select 17532,'017532','1987-014H','KDU part','KDU part',60104,'Reentered','1987-02-26 00:00:00','1987-02-20 00:00:00',89.41,220,271,72.84,'Intermediate' from dual union all
select 17533,'017533','1987-019A','Kosmos-1822','Oblik No. 354',60109,'Landed','1987-03-05 00:00:00','1987-02-22 00:00:00',89.64,228,286,72.85,'Intermediate' from dual union all
select 17534,'017534','1987-019B','11S510','11S510',60109,'Reentered','1987-03-01 00:00:00','1987-02-23 00:00:00',88.94,184,261,72.85,'Intermediate' from dual union all
select 17535,'017535','1987-020A','Kosmos-1823','Musson No. 19',60110,'In Earth orbit','','1987-03-23 00:00:00',116.02,1479,1525,73.61,'Intermediate' from dual union all
select 17536,'017536','1987-020B','S5M','S5M',60110,'In Earth orbit','','1987-03-23 00:00:00',115.97,1478,1521,73.61,'Intermediate' from dual union all
select 17537,'017537','1986-067Q','Kosmos-1776 SS 9','ESO',59260,'Reentered','1987-11-24 00:00:00','1987-02-25 00:00:00',94.42,469,512,74.04,'Intermediate' from dual union all
select 17538,'017538','1986-067R','Kosmos-1776 SS 10','ESO',59260,'Reentered','1987-12-28 00:00:00','1987-02-24 00:00:00',94.52,473,518,74.02,'Intermediate' from dual union all
select 17539,'017539','1986-019HT','deb Ariane V16','',59212,'Reentered','1989-07-20 00:00:00','1987-02-25 00:00:00',101,789,823,98.4,'Sun-Synch' from dual union all
select 1754,'001754','1965-082CR','deb Transtage 4','',11832,'Reentered','1968-08-04 00:00:00','1966-01-26 00:00:00',98.01,585,742,32.03,'Intermediate' from dual union all
select 17540,'017540','1986-019HU','deb Ariane V16','',59212,'Reentered','1990-12-20 00:00:00','1987-02-25 00:00:00',103.13,786,1028,97.5,'Sun-Synch' from dual union all
select 17541,'017541','1986-019HV','deb Ariane V16','',59212,'Reentered','1988-10-19 00:00:00','1987-02-25 00:00:00',98.82,594,810,99.05,'Sun-Synch' from dual union all
select 17542,'017542','1986-019HW','deb Ariane V16','',59212,'Reentered','1988-12-14 00:00:00','1987-02-25 00:00:00',100.72,778,808,98.69,'Sun-Synch' from dual union all
select 17543,'017543','1986-019HX','deb Ariane V16','',59212,'In Earth orbit','','1993-08-09 00:00:00',102.66,787,981,98.67,'Sun-Synch' from dual union all
select 17544,'017544','1986-019HY','deb Ariane V16','',59212,'Reentered','1989-03-13 00:00:00','',103.13,818,996,99.3,'Sun-Synch' from dual union all
select 17545,'017545','1986-019HZ','deb Ariane V16','',59212,'Reentered','1989-12-13 00:00:00','1987-02-25 00:00:00',101.07,781,838,98.44,'Sun-Synch' from dual union all
select 17546,'017546','1986-019JA','deb Ariane V16','',59212,'Reentered','1989-07-21 00:00:00','1987-02-25 00:00:00',100.5,762,803,98.79,'Sun-Synch' from dual union all
select 17547,'017547','1986-019JB','deb Ariane V16','',59212,'Reentered','1989-12-15 00:00:00','1987-02-25 00:00:00',102.7,787,986,97.91,'Sun-Synch' from dual union all
select 17548,'017548','1986-019JC','deb Ariane V16','',59212,'Reentered','1989-02-15 00:00:00','1987-02-25 00:00:00',101.17,792,837,98.63,'Sun-Synch' from dual union all
select 17549,'017549','1986-019JD','deb Ariane V16','',59212,'Reentered','1989-04-23 00:00:00','1987-02-25 00:00:00',100.38,754,800,98.76,'Sun-Synch' from dual union all
select 1755,'001755','1965-082CS','deb Transtage 4','',11832,'Reentered','1981-04-30 00:00:00','1981-04-18 00:00:00',92.77,409,412,31.94,'Intermediate' from dual union all
select 17550,'017550','1986-019JE','deb Ariane V16','',59212,'Reentered','1988-10-09 00:00:00','1987-02-25 00:00:00',100.31,734,813,98.68,'Sun-Synch' from dual union all
select 17551,'017551','1986-019JF','deb Ariane V16','',59212,'Reentered','1999-04-05 00:00:00','1992-11-28 00:00:00',97.7,644,653,98.46,'Sun-Synch' from dual union all
select 17552,'017552','1986-019JG','deb Ariane V16','',59212,'Reentered','1989-09-07 00:00:00','1987-02-25 00:00:00',101.08,769,851,98.58,'Sun-Synch' from dual union all
select 17553,'017553','1986-019JH','deb Ariane V16','',59212,'Reentered','1990-03-17 00:00:00','1987-02-25 00:00:00',103.87,793,1090,98.12,'Sun-Synch' from dual union all
select 17554,'017554','1986-019JJ','deb Ariane V16','',59212,'Reentered','1989-10-22 00:00:00','1987-02-25 00:00:00',101.42,783,869,98.66,'Sun-Synch' from dual union all
select 17555,'017555','1986-019JK','deb Ariane V16','',59212,'Reentered','1993-11-19 00:00:00','1992-09-03 00:00:00',96.1,546,598,98.02,'Sun-Synch' from dual union all
select 17556,'017556','1986-019JL','deb Ariane V16','',59212,'Reentered','1990-01-14 00:00:00','1987-02-25 00:00:00',101.55,777,888,98.54,'Sun-Synch' from dual union all
select 17557,'017557','1986-019JM','deb Ariane V16','',59212,'Reentered','1990-12-22 00:00:00','1987-02-25 00:00:00',100.13,716,814,98.87,'Sun-Synch' from dual union all
select 17558,'017558','1986-019JN','deb Ariane V16','',59212,'Reentered','1988-08-31 00:00:00','1987-02-25 00:00:00',98.55,578,801,99.07,'Sun-Synch' from dual;
insert into SATELLITE
select 17559,'017559','1987-021A','Kosmos-1824','Kobal''t No. 506',60111,'Landed','1987-04-22 00:00:00','1987-03-06 00:00:00',89.17,160,306,67.14,'Intermediate' from dual union all
select 1756,'001756','1965-082CT','deb Transtage 4','',11832,'Reentered','1987-09-13 00:00:00','1966-01-23 00:00:00',102.76,714,1064,32.14,'Intermediate' from dual union all
select 17560,'017560','1987-021B','11S510','11S510',60111,'Reentered','1987-03-04 00:00:00','1987-03-01 00:00:00',88.81,156,276,67.14,'Intermediate' from dual union all
select 17561,'017561','1987-022A','GOES 7','GOES H',60112,'In Earth orbit','','1987-03-31 00:00:00',1436.16,35775,35801,.05,'Stationary' from dual union all
select 17562,'017562','1987-022B','Delta 179','Delta SSPS AJ10-118K',60112,'In Earth orbit','','1987-03-28 00:00:00',128.83,252,3891,21.75,'Medium' from dual union all
select 17563,'017563','1987-022C','Star 37E S/N 40105','Star 37E',60112,'In Earth orbit','','1987-03-28 00:00:00',732.69,269,40819,17.9,'GEO Transfer' from dual union all
select 17564,'017564','1987-023A','Progress-28','Progress 7K-TG No. 137',60113,'Deorbited','1987-03-28 00:00:00','1987-03-10 00:00:00',91.76,353,369,51.61,'Intermediate' from dual union all
select 17565,'017565','1987-023B','11S510','11S510',60113,'Reentered','1987-03-06 00:00:00','1987-03-06 00:00:00',87.29,136,144,51.64,'Intermediate' from dual union all
select 17566,'017566','1987-024A','Kosmos-1825','Tselina-D',60114,'In Earth orbit','','1987-04-17 00:00:00',97.67,630,665,82.54,'Intermediate' from dual union all
select 17567,'017567','1987-024B','S5M','S5M',60114,'In Earth orbit','','1987-04-02 00:00:00',97.65,631,662,82.54,'Intermediate' from dual union all
select 17568,'017568','1987-019C','KDU part','KDU part',60109,'Reentered','1987-03-11 00:00:00','1987-03-06 00:00:00',91.51,237,460,72.88,'Intermediate' from dual union all
select 17569,'017569','1987-019D','KDU part','KDU part',60109,'Reentered','1987-03-17 00:00:00','1987-03-06 00:00:00',91.43,240,450,72.85,'Intermediate' from dual union all
select 1757,'001757','1965-082CU','deb Transtage 4','',11832,'Reentered','1981-05-05 00:00:00','1981-05-05 00:00:00',90.49,291,306,32.29,'Intermediate' from dual union all
select 17570,'017570','1987-019E','KDU','KDU',60109,'Reentered','1987-04-10 00:00:00','1987-03-06 00:00:00',89.56,225,281,72.86,'Intermediate' from dual union all
select 17571,'017571','1987-019F','Blenda','Sunshade',60109,'Reentered','1987-03-08 00:00:00','1987-03-06 00:00:00',89.15,210,255,72.87,'Intermediate' from dual union all
select 17572,'017572','1987-019G','KDU part','KDU part',60109,'Reentered','1987-03-11 00:00:00','1987-03-06 00:00:00',89.23,218,255,72.85,'Intermediate' from dual union all
select 17573,'017573','1987-019H','KDU part','KDU part',60109,'Reentered','1987-03-06 00:00:00','1987-03-05 00:00:00',89.02,171,281,72.88,'Intermediate' from dual union all
select 17574,'017574','1986-019JP','deb Ariane V16','',59212,'Reentered','1991-03-02 00:00:00','1987-03-09 00:00:00',102.91,797,996,98.4,'Sun-Synch' from dual union all
select 17575,'017575','1986-019JQ','deb Ariane V16','',59212,'Reentered','1989-01-12 00:00:00','1987-03-09 00:00:00',101.26,770,867,98.61,'Sun-Synch' from dual union all
select 17576,'017576','1986-019JR','deb Ariane V16','',59212,'Reentered','1988-11-13 00:00:00','1987-03-09 00:00:00',99.97,708,806,98.72,'Sun-Synch' from dual union all
select 17577,'017577','1987-025A','Kosmos-1826','Oblik No. 356',60115,'Landed','1987-03-25 00:00:00','1987-03-16 00:00:00',92.25,356,413,72.88,'Intermediate' from dual union all
select 17578,'017578','1987-025B','11S510','11S510',60115,'Reentered','1987-03-25 00:00:00','1987-03-13 00:00:00',89.98,192,355,72.87,'Intermediate' from dual union all
select 17579,'017579','1987-025C','Sensor cover','Sensor cover',60115,'Reentered','1987-03-17 00:00:00','1987-03-13 00:00:00',89.69,184,335,72.87,'Intermediate' from dual union all
select 1758,'001758','1965-082CV','deb Transtage 4','',11832,'Reentered','1973-01-13 00:00:00','1966-01-09 00:00:00',97.95,568,753,31.87,'Intermediate' from dual union all
select 17580,'017580','1987-025D','Sensor cover','Sensor cover',60115,'Reentered','1987-03-16 00:00:00','1987-03-13 00:00:00',89.23,180,293,72.87,'Intermediate' from dual union all
select 17581,'017581','1987-025E','Sensor cover','Sensor cover',60115,'Reentered','1987-03-17 00:00:00','1987-03-13 00:00:00',89.31,181,300,72.86,'Intermediate' from dual union all
select 17582,'017582','1987-026A','Kosmos-1827','Strela-3',60116,'In Earth orbit','','1987-04-12 00:00:00',113.85,1396,1411,82.57,'Intermediate' from dual union all
select 17583,'017583','1987-026B','Kosmos-1828','Strela-3',60116,'In Earth orbit','','1987-04-13 00:00:00',113.73,1385,1411,82.57,'Intermediate' from dual union all
select 17584,'017584','1987-026C','Kosmos-1829','Strela-3',60116,'In Earth orbit','','1987-04-12 00:00:00',114.06,1411,1415,82.57,'Intermediate' from dual union all
select 17585,'017585','1987-026D','Kosmos-1830','Strela-3',60116,'In Earth orbit','','1987-04-13 00:00:00',113.99,1408,1411,82.56,'Intermediate' from dual union all
select 17586,'017586','1987-026E','Kosmos-1831','Strela-3',60116,'In Earth orbit','','1987-04-12 00:00:00',113.81,1392,1410,82.56,'Intermediate' from dual union all
select 17587,'017587','1987-026F','Kosmos-1832','Strela-3',60116,'In Earth orbit','','1987-04-12 00:00:00',113.92,1402,1411,82.57,'Intermediate' from dual union all
select 17588,'017588','1987-026G','S5M','S5M',60116,'In Earth orbit','','1987-04-12 00:00:00',114.68,1411,1471,82.57,'Intermediate' from dual union all
select 17589,'017589','1987-027A','Kosmos-1833','EPN No. 03.694',60117,'In Earth orbit','','1987-03-18 00:00:00',101.92,848,852,71,'Intermediate' from dual union all
select 1759,'001759','1965-082CW','deb Transtage 4','',11832,'Reentered','1980-10-21 00:00:00','1966-02-05 00:00:00',98.99,678,743,32.14,'Intermediate' from dual union all
select 17590,'017590','1987-027B','11S772','11S772',60117,'In Earth orbit','','1987-05-02 00:00:00',101.77,839,846,71,'Intermediate' from dual union all
select 17591,'017591','1986-019JS','deb Ariane V16','',59212,'Reentered','1989-08-07 00:00:00','1987-03-17 00:00:00',101.06,774,844,98.54,'Sun-Synch' from dual union all
select 17592,'017592','1986-019JT','deb Ariane V16','',59212,'Reentered','1991-04-28 00:00:00','1987-03-17 00:00:00',101.79,801,886,98.7,'Sun-Synch' from dual union all
select 17593,'017593','1986-019JU','deb Ariane V16','',59212,'Reentered','1991-02-15 00:00:00','1987-03-17 00:00:00',103.61,790,1069,98.1,'Sun-Synch' from dual union all
select 17594,'017594','1986-019JV','deb Ariane V16','',59212,'Reentered','1989-05-19 00:00:00','1987-03-17 00:00:00',101.03,774,841,98.56,'Sun-Synch' from dual union all
select 17595,'017595','1986-019JW','deb Ariane V16','',59212,'Reentered','1990-01-20 00:00:00','1987-03-17 00:00:00',102.96,808,989,97.84,'Sun-Synch' from dual union all
select 17596,'017596','1986-019JX','deb Ariane V16','',59212,'Reentered','1989-01-15 00:00:00','1987-03-17 00:00:00',99.74,681,812,98.94,'Sun-Synch' from dual union all
select 17597,'017597','1986-019JY','deb Ariane V16','',59212,'Reentered','1989-04-26 00:00:00','1987-03-17 00:00:00',100.57,728,844,98.47,'Sun-Synch' from dual union all
select 17598,'017598','1986-019JZ','deb Ariane V16','',59212,'Reentered','1988-07-13 00:00:00','1987-03-17 00:00:00',100.46,728,833,98.68,'Sun-Synch' from dual union all
select 17599,'017599','1986-019KA','deb Ariane V16','',59212,'Reentered','1989-03-14 00:00:00','1987-03-17 00:00:00',100.33,708,841,98.63,'Sun-Synch' from dual union all
select 176,'000176','61 Omicron 49','deb r/b Ablestar','',7273,'Reentered','1982-02-12 00:00:00','1961-08-19 00:00:00',104.14,897,1012,66.82,'Intermediate' from dual union all
select 1760,'001760','1965-082CX','deb Transtage 4','',11832,'Reentered','1971-03-31 00:00:00','1966-02-08 00:00:00',99.58,721,756,32.45,'Intermediate' from dual union all
select 17600,'017600','1986-019KB','deb Ariane V16','',59212,'Reentered','1988-10-21 00:00:00','1987-03-17 00:00:00',99.95,709,803,98.72,'Sun-Synch' from dual union all
select 17601,'017601','1986-019KC','deb Ariane V16','',59212,'Reentered','1989-02-26 00:00:00','1987-03-17 00:00:00',99.89,700,807,98.89,'Sun-Synch' from dual union all
select 17602,'017602','1986-019KD','deb Ariane V16','',59212,'Reentered','1989-04-08 00:00:00','1987-03-17 00:00:00',102.66,796,973,99.1,'Sun-Synch' from dual union all
select 17603,'017603','1986-019KE','deb Ariane V16','',59212,'Reentered','1990-02-07 00:00:00','1987-03-17 00:00:00',102.85,790,996,97.93,'Sun-Synch' from dual union all
select 17604,'017604','1986-019KF','deb Ariane V16','',59212,'Reentered','1989-12-03 00:00:00','1987-03-17 00:00:00',103.06,800,1006,97.87,'Sun-Synch' from dual union all
select 17605,'017605','1986-019KG','deb Ariane V16','',59212,'Reentered','1989-02-02 00:00:00','1987-03-17 00:00:00',101.41,797,854,98.45,'Sun-Synch' from dual union all
select 17606,'017606','1986-019KH','deb Ariane V16','',59212,'Reentered','1989-05-23 00:00:00','1987-03-17 00:00:00',100.8,779,814,98.2,'Sun-Synch' from dual union all
select 17607,'017607','1986-019KJ','deb Ariane V16','',59212,'Reentered','1989-10-30 00:00:00','1987-03-17 00:00:00',101.19,798,832,98.45,'Sun-Synch' from dual union all
select 17608,'017608','1986-019KK','deb Ariane V16','',59212,'Reentered','2017-09-27 00:00:00','1993-08-07 00:00:00',101.75,755,927,98.79,'Sun-Synch' from dual union all
select 17609,'017609','1986-019KL','deb Ariane V16','',59212,'Reentered','1989-02-16 00:00:00','1987-03-17 00:00:00',99.74,687,806,98.73,'Sun-Synch' from dual union all
select 1761,'001761','1965-082CY','deb Transtage 4','',11832,'Reentered','1989-11-02 00:00:00','1989-11-01 00:00:00',90.46,278,317,32.43,'Intermediate' from dual union all
select 17610,'017610','1986-019KM','deb Ariane V16','',59212,'Reentered','1988-09-19 00:00:00','1987-03-17 00:00:00',100.68,770,812,98.47,'Sun-Synch' from dual union all
select 17611,'017611','1987-028A','Raduga','Gran'' No. 31L',60118,'In Earth orbit','','1987-04-18 00:00:00',1435.93,35762,35805,1.24,'Stationary' from dual union all
select 17612,'017612','1987-028B','8S812','8S812',60118,'Reentered','1987-03-22 00:00:00','1987-03-21 00:00:00',87.9,170,172,51.61,'Intermediate' from dual union all
select 17613,'017613','1987-028C','Perekhodnik','Sredniy perekhodnik',60118,'Reentered','1987-03-19 00:00:00','1987-03-19 00:00:00',88.44,188,207,51.6,'Intermediate' from dual union all
select 17614,'017614','1981-053KH','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-30 00:00:00',104.42,922,1012,82.79,'Intermediate' from dual union all
select 17615,'017615','1981-053KJ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-08 00:00:00',104.18,916,995,82.87,'Intermediate' from dual union all
select 17616,'017616','1981-053KK','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-29 00:00:00',106.36,959,1157,83.04,'Intermediate' from dual union all
select 17617,'017617','1981-053KL','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',106.64,935,1207,82.86,'Intermediate' from dual union all
select 17618,'017618','1981-053KM','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',104.22,907,1009,82.86,'Intermediate' from dual union all
select 17619,'017619','1981-053KN','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',103.46,900,945,82.95,'Intermediate' from dual union all
select 1762,'001762','1965-082CZ','deb Transtage 4','',11832,'Reentered','1968-01-22 00:00:00','1965-10-15 00:00:00',99.17,676,762,32.4,'Intermediate' from dual union all
select 17620,'017620','1981-053KP','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-01 00:00:00',103.63,903,958,83.19,'Intermediate' from dual union all
select 17621,'017621','1981-053KQ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',104.07,904,998,82.81,'Intermediate' from dual union all
select 17622,'017622','1981-053KR','deb Kosmos-1275','',52576,'Reentered','2003-01-10 00:00:00','1993-08-03 00:00:00',101.14,772,854,82.76,'Intermediate' from dual union all
select 17623,'017623','1981-053KS','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',104.78,932,1037,82.81,'Intermediate' from dual union all
select 17624,'017624','1981-053KT','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-29 00:00:00',103.01,879,923,82.88,'Intermediate' from dual union all
select 17625,'017625','1981-053KU','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',104.53,934,1010,82.8,'Intermediate' from dual union all
select 17626,'017626','1981-053KV','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',106.59,953,1185,82.86,'Intermediate' from dual union all
select 17627,'017627','1981-053KW','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-31 00:00:00',104.41,931,1002,82.94,'Intermediate' from dual union all
select 17628,'017628','1981-053KX','deb Kosmos-1275','',52576,'Reentered','2000-12-16 00:00:00','1993-08-02 00:00:00',100.02,731,788,82.75,'Intermediate' from dual union all
select 17629,'017629','1981-053KY','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-03 00:00:00',106.81,940,1218,83,'Intermediate' from dual union all
select 1763,'001763','1965-082DA','deb Transtage 4','',11832,'Reentered','1972-10-08 00:00:00','1966-01-26 00:00:00',98.93,647,768,32.23,'Intermediate' from dual union all
select 17630,'017630','1981-053KZ','deb Kosmos-1275','',52576,'Reentered','2000-03-30 00:00:00','1993-08-06 00:00:00',100.62,774,801,82.93,'Intermediate' from dual union all
select 17631,'017631','1981-053LA','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-09 00:00:00',106.52,960,1171,82.94,'Intermediate' from dual union all
select 17632,'017632','1981-053LB','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-03 00:00:00',106.85,958,1204,83.09,'Intermediate' from dual union all
select 17633,'017633','1981-053LC','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-02 00:00:00',107.24,931,1267,83.09,'Intermediate' from dual union all
select 17634,'017634','1981-053LD','deb Kosmos-1275','',52576,'Reentered','2003-12-29 00:00:00','1993-08-07 00:00:00',100.98,764,846,83.15,'Intermediate' from dual union all
select 17635,'017635','1981-053LE','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',104.15,919,991,82.84,'Intermediate' from dual union all
select 17636,'017636','1981-053LF','deb Kosmos-1275','',52576,'Reentered','2004-10-09 00:00:00','1993-08-04 00:00:00',102.45,832,918,82.98,'Intermediate' from dual union all
select 17637,'017637','1981-053LG','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-08 00:00:00',106.9,957,1210,83.07,'Intermediate' from dual union all
select 17638,'017638','1981-053LH','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',106.46,947,1179,83,'Intermediate' from dual union all
select 17639,'017639','1981-053LJ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-08 00:00:00',107.16,950,1240,83,'Intermediate' from dual union all
select 1764,'001764','1965-082DB','deb Transtage 4','',11832,'Reentered','1977-09-28 00:00:00','1977-09-09 00:00:00',91.66,347,365,32.86,'Intermediate' from dual union all
select 17640,'017640','1981-053LK','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',103.42,906,935,82.94,'Intermediate' from dual union all
select 17641,'017641','1981-053LL','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',104.13,904,1004,82.77,'Intermediate' from dual union all
select 17642,'017642','1981-053LM','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-05 00:00:00',102.99,862,939,82.88,'Intermediate' from dual union all
select 17643,'017643','1983-044CR','deb Kosmos-1461','',55801,'Reentered','1989-04-08 00:00:00','1987-03-19 00:00:00',99.86,619,885,64.92,'Intermediate' from dual union all
select 17644,'017644','1983-044CS','deb Kosmos-1461','',55801,'Reentered','1988-12-18 00:00:00','1987-03-20 00:00:00',98.85,559,849,65,'Intermediate' from dual union all
select 17645,'017645','1983-044CT','deb Kosmos-1461','',55801,'Reentered','1989-04-11 00:00:00','',99.48,587,881,65,'Intermediate' from dual union all
select 17646,'017646','1983-044CU','deb Kosmos-1461','',55801,'Reentered','1989-01-25 00:00:00','1987-03-20 00:00:00',99.51,612,859,64.94,'Intermediate' from dual union all
select 17647,'017647','1983-044CV','deb Kosmos-1461','',55801,'Reentered','1989-02-14 00:00:00','1987-03-20 00:00:00',99.05,569,858,65.02,'Intermediate' from dual union all
select 17648,'017648','61 Omicron 272','deb r/b Ablestar','',7273,'Reentered','1990-02-12 00:00:00','1993-09-21 00:00:00',104.13,869,1039,65.73,'Intermediate' from dual;
insert into SATELLITE
select 17649,'017649','61 Omicron 273','deb r/b Ablestar','',7273,'Reentered','2014-06-28 00:00:00','1993-08-08 00:00:00',102.69,833,939,66.65,'Intermediate' from dual union all
select 1765,'001765','1965-082DC','deb Transtage 4','',11832,'Reentered','1973-06-09 00:00:00','1973-05-08 00:00:00',94.28,482,486,32.04,'Intermediate' from dual union all
select 17650,'017650','61 Omicron 274','deb r/b Ablestar','',7273,'Reentered','2007-08-09 00:00:00','1993-08-09 00:00:00',105.11,739,1260,66.01,'Intermediate' from dual union all
select 17651,'017651','1987-022D','deb r/b Delta','[1961 OMI 275]',60112,'Reentered','1987-08-16 00:00:00','1987-03-28 00:00:00',128.83,252,3891,21.75,'Medium' from dual union all
select 17652,'017652','1987-022E','deb r/b Delta','[1961 OMI 276]',60112,'Reentered','1988-01-18 00:00:00','1987-03-28 00:00:00',128.83,252,3891,21.75,'Medium' from dual union all
select 17653,'017653','61 Omicron 277','deb r/b Ablestar','',7273,'Reentered','1990-08-11 00:00:00','1990-08-08 00:00:00',91.97,357,386,66.37,'Intermediate' from dual union all
select 17654,'017654','1980-089BX','deb Kosmos-1220','',50299,'Reentered','1988-12-24 00:00:00','1987-03-20 00:00:00',97.02,535,696,64.98,'Intermediate' from dual union all
select 17655,'017655','1980-089BY','deb Kosmos-1220','',50299,'Reentered','1989-08-12 00:00:00','1987-03-20 00:00:00',98.1,564,772,64.97,'Intermediate' from dual union all
select 17656,'017656','1980-089BZ','deb Kosmos-1220','',50299,'Reentered','1989-01-21 00:00:00','1987-03-20 00:00:00',97.18,539,709,64.98,'Intermediate' from dual union all
select 17657,'017657','1980-089CA','deb Kosmos-1220','',50299,'Reentered','1989-02-20 00:00:00','1987-03-20 00:00:00',97.18,539,709,64.98,'Intermediate' from dual union all
select 17658,'017658','1969-082KU','deb Agena D','',18349,'In Earth orbit','','1993-08-05 00:00:00',103.66,887,977,70.35,'Intermediate' from dual union all
select 17659,'017659','1969-082KV','deb Agena D','',18349,'Reentered','1991-10-26 00:00:00','1991-10-10 00:00:00',93.8,450,471,70.29,'Intermediate' from dual union all
select 1766,'001766','1965-082DD','deb Transtage 4','',11832,'Reentered','1971-04-03 00:00:00','1966-01-23 00:00:00',98.21,612,735,31.95,'Intermediate' from dual union all
select 17660,'017660','1969-082KW','deb Agena D','',18349,'Reentered','2003-11-16 00:00:00','1993-08-08 00:00:00',101.43,787,866,70.48,'Intermediate' from dual union all
select 17661,'017661','1969-082KX','deb Agena D','',18349,'Reentered','1991-12-20 00:00:00','1991-12-16 00:00:00',91.86,352,380,70.58,'Intermediate' from dual union all
select 17662,'017662','1971-015DA','deb Kosmos-397','',24833,'Reentered','1990-04-24 00:00:00','1987-03-20 00:00:00',102.41,545,1201,65.86,'Intermediate' from dual union all
select 17663,'017663','1971-015DB','deb Kosmos-397','',24833,'Reentered','2014-05-04 00:00:00','1993-08-05 00:00:00',104.69,544,1415,65.7,'Intermediate' from dual union all
select 17664,'017664','1971-015DC','deb Kosmos-397','',24833,'Reentered','1989-06-05 00:00:00','1987-03-20 00:00:00',101.84,521,1170,65.69,'Intermediate' from dual union all
select 17665,'017665','1963-053K','deb Explorer 19','',9284,'Reentered','2005-04-08 00:00:00','1964-02-11 00:00:00',115.89,596,2396,78.58,'Intermediate' from dual union all
select 17666,'017666','1965-027V','deb Snapshot','',11777,'In Earth orbit','','1993-08-07 00:00:00',111.32,1264,1311,90.28,'Polar' from dual union all
select 17667,'017667','1968-091DC','deb Kosmos-249','',16818,'Reentered','1992-06-14 00:00:00','1992-06-11 00:00:00',90.3,265,314,62.23,'Intermediate' from dual union all
select 17668,'017668','1968-097EH','deb Kosmos-252','',16824,'Reentered','1992-02-11 00:00:00','1992-02-10 00:00:00',89.15,216,249,62.17,'Intermediate' from dual union all
select 17669,'017669','1970-089DB','deb Kosmos-374','',21566,'Reentered','1991-07-06 00:00:00','1991-05-21 00:00:00',106.39,516,1603,53,'Intermediate' from dual union all
select 1767,'001767','1965-082DE','deb Transtage 4','',11832,'Reentered','1971-02-19 00:00:00','1966-01-18 00:00:00',97.73,554,746,32.28,'Intermediate' from dual union all
select 17670,'017670','1976-126BV','deb Kosmos-886','',39078,'Reentered','2003-06-27 00:00:00','1993-08-08 00:00:00',104.16,493,1418,65.72,'Intermediate' from dual union all
select 17671,'017671','1981-028BV','deb Kosmos-1260','',52551,'Reentered','1989-12-26 00:00:00','1987-03-28 00:00:00',99.3,708,742,65,'Intermediate' from dual union all
select 17672,'017672','1986-019KN','deb Ariane V16','',59212,'Reentered','1991-02-25 00:00:00','1987-03-20 00:00:00',103.62,804,1055,97.88,'Sun-Synch' from dual union all
select 17673,'017673','1986-019KP','deb Ariane V16','',59212,'Reentered','1988-05-02 00:00:00','1987-03-20 00:00:00',98.93,574,840,99.28,'Sun-Synch' from dual union all
select 17674,'017674','1986-019KQ','deb Ariane V16','',59212,'Reentered','1988-09-19 00:00:00','1987-03-20 00:00:00',99.63,665,817,98.51,'Sun-Synch' from dual union all
select 17675,'017675','1986-019KR','deb Ariane V16','',59212,'Reentered','1989-03-02 00:00:00','1987-03-20 00:00:00',104.54,822,1124,97.86,'Sun-Synch' from dual union all
select 17676,'017676','1986-019KS','deb Ariane V16','',59212,'Reentered','1989-04-29 00:00:00','1987-03-20 00:00:00',101.54,792,872,98.65,'Sun-Synch' from dual union all
select 17677,'017677','1986-019KT','deb Ariane V16','',59212,'Reentered','1988-05-04 00:00:00','1987-03-20 00:00:00',97.97,530,794,99.13,'Sun-Synch' from dual union all
select 17678,'017678','1986-019KU','deb Ariane V16','',59212,'Reentered','1988-11-18 00:00:00','1987-03-20 00:00:00',99.45,667,797,98.61,'Sun-Synch' from dual union all
select 17679,'017679','1986-019KV','deb Ariane V16','',59212,'Reentered','1989-03-28 00:00:00','1987-03-20 00:00:00',100.21,740,797,98.84,'Sun-Synch' from dual union all
select 1768,'001768','1965-082DF','deb Transtage 4','',11832,'Reentered','1970-11-27 00:00:00','1966-01-21 00:00:00',98.89,628,784,31.92,'Intermediate' from dual union all
select 17680,'017680','1986-019KW','deb Ariane V16','',59212,'Reentered','1989-01-18 00:00:00','1987-03-20 00:00:00',100.34,734,816,98.8,'Sun-Synch' from dual union all
select 17681,'017681','1986-019KX','deb Ariane V16','',59212,'Reentered','1988-10-31 00:00:00','1987-03-20 00:00:00',98.87,604,806,99.02,'Sun-Synch' from dual union all
select 17682,'017682','1986-019KY','deb Ariane V16','',59212,'Reentered','1989-03-31 00:00:00','1987-03-20 00:00:00',102.61,795,969,97.89,'Sun-Synch' from dual union all
select 17683,'017683','1986-019KZ','deb Ariane V16','',59212,'Reentered','1989-02-25 00:00:00','1987-03-20 00:00:00',99.98,718,798,98.82,'Sun-Synch' from dual union all
select 17684,'017684','1986-019LA','deb Ariane V16','',59212,'Reentered','1989-05-08 00:00:00','1987-03-20 00:00:00',100.23,741,798,98.37,'Sun-Synch' from dual union all
select 17685,'017685','1986-019LB','deb Ariane V16','',59212,'Reentered','1989-08-13 00:00:00','1987-03-20 00:00:00',100.58,722,851,98.65,'Sun-Synch' from dual union all
select 17686,'017686','1986-019LC','deb Ariane V16','',59212,'Reentered','1987-12-11 00:00:00','1987-03-20 00:00:00',96.99,510,720,99.42,'Sun-Synch' from dual union all
select 17687,'017687','1986-019LD','deb Ariane V16','',59212,'Reentered','1989-03-14 00:00:00','1987-03-20 00:00:00',101.84,791,901,99.35,'Sun-Synch' from dual union all
select 17688,'017688','1986-019LE','deb Ariane V16','',59212,'Reentered','1990-04-28 00:00:00','1987-03-20 00:00:00',103.31,806,1024,97.89,'Sun-Synch' from dual union all
select 17689,'017689','1986-019LF','deb Ariane V16','',59212,'Reentered','1988-01-17 00:00:00','1987-03-20 00:00:00',98.48,581,791,98.85,'Sun-Synch' from dual union all
select 1769,'001769','1965-082DG','deb Transtage 4','',11832,'Reentered','1981-11-07 00:00:00','1981-11-07 00:00:00',90.74,291,331,32.04,'Intermediate' from dual union all
select 17690,'017690','1986-019LG','deb Ariane V16','',59212,'Reentered','1989-06-08 00:00:00','1987-03-20 00:00:00',101.26,777,860,98.56,'Sun-Synch' from dual union all
select 17691,'017691','1986-019LH','deb Ariane V16','',59212,'Reentered','1989-01-14 00:00:00','1987-03-20 00:00:00',99.63,689,792,98.93,'Sun-Synch' from dual union all
select 17692,'017692','1986-019LJ','deb Ariane V16','',59212,'Reentered','1989-02-17 00:00:00','1987-03-20 00:00:00',100.05,721,801,98.89,'Sun-Synch' from dual union all
select 17693,'017693','1986-019LK','deb Ariane V16','',59212,'Reentered','1988-12-29 00:00:00','',98.7,599,793,99,'Sun-Synch' from dual union all
select 17694,'017694','1986-019LL','deb Ariane V16','',59212,'Reentered','1989-03-30 00:00:00','1987-03-20 00:00:00',100.89,783,819,98.4,'Sun-Synch' from dual union all
select 17695,'017695','1986-019LM','deb Ariane V16','',59212,'Reentered','1989-09-05 00:00:00','1987-03-20 00:00:00',107.4,808,1405,98.82,'Sun-Synch' from dual union all
select 17696,'017696','1986-019LN','deb Ariane V16','',59212,'Reentered','1989-12-03 00:00:00','1987-03-20 00:00:00',102.08,794,921,98.18,'Sun-Synch' from dual union all
select 17697,'017697','1986-019LP','deb Ariane V16','',59212,'Reentered','1989-05-19 00:00:00','1987-03-20 00:00:00',101.02,795,819,98.62,'Sun-Synch' from dual union all
select 17698,'017698','1986-019LQ','deb Ariane V16','',59212,'Reentered','1992-02-11 00:00:00','1992-02-08 00:00:00',91.17,313,351,98.46,'Sun-Synch' from dual union all
select 17699,'017699','1986-019LR','deb Ariane V16','',59212,'Reentered','1990-01-06 00:00:00','1987-03-20 00:00:00',104.07,761,1141,98.06,'Sun-Synch' from dual union all
select 177,'000177','61 Omicron 50','deb r/b Ablestar','',7273,'In Earth orbit','','1962-03-07 00:00:00',102.66,787,981,66.66,'Intermediate' from dual union all
select 1770,'001770','1965-082DH','Transtage 4 large debris','RCS 1.1',11832,'Reentered','1979-09-10 00:00:00','1966-01-12 00:00:00',97.97,582,742,32.25,'Intermediate' from dual union all
select 17700,'017700','1986-019LS','deb Ariane V16','',59212,'Reentered','1988-11-12 00:00:00','1987-03-20 00:00:00',101.02,794,819,98.94,'Sun-Synch' from dual union all
select 17701,'017701','1986-019LT','deb Ariane V16','',59212,'Reentered','1987-11-05 00:00:00','1987-03-20 00:00:00',97.79,549,756,99.29,'Sun-Synch' from dual union all
select 17702,'017702','1986-019LU','deb Ariane V16','',59212,'In Earth orbit','','1993-08-05 00:00:00',100.5,776,789,98.47,'Sun-Synch' from dual union all
select 17703,'017703','1986-019LV','deb Ariane V16','',59212,'Reentered','2001-09-16 00:00:00','1993-07-31 00:00:00',99.91,721,788,98.53,'Sun-Synch' from dual union all
select 17704,'017704','1986-019LW','deb Ariane V16','',59212,'Reentered','1990-01-18 00:00:00','1987-03-20 00:00:00',100.85,788,810,98.57,'Sun-Synch' from dual union all
select 17705,'017705','1987-028D','Blok DM','11S86 No. 63L',60118,'In Earth orbit','','1987-04-21 00:00:00',1441.91,35799,36002,1.27,'Drift GEO' from dual union all
select 17706,'017706','1987-029A','Palapa B2P','Palapa B2P',60119,'In Earth orbit','','1987-04-22 00:00:00',1436.16,35769,35807,.04,'Stationary' from dual union all
select 17707,'017707','1987-029B','Delta 182','Delta SSPS AJ10-118K',60119,'Reentered','1987-04-23 00:00:00','1987-03-27 00:00:00',90.37,248,338,26.85,'Intermediate' from dual union all
select 17708,'017708','1987-029C','PAM-D','Star 48B S/N 10050-17',60119,'Reentered','1989-12-18 00:00:00','1987-04-19 00:00:00',647.23,210,36605,24.66,'GEO Transfer' from dual union all
select 17709,'017709','1987-028E','SOZ','SOZ',60118,'Reentered','1991-11-30 00:00:00','1987-04-19 00:00:00',636.1,299,35946,47.35,'GEO Transfer' from dual union all
select 1771,'001771','1965-094C','deb Kosmos-96','',11844,'Reentered','1965-11-28 00:00:00','1965-11-23 00:00:00',89.21,210,260,51.8,'Intermediate' from dual union all
select 17710,'017710','1987-028F','SOZ','SOZ',60118,'Reentered','1987-09-30 00:00:00','1987-04-19 00:00:00',635.08,246,35947,47.28,'GEO Transfer' from dual union all
select 17711,'017711','61 Omicron 278','deb r/b Ablestar','',7273,'Reentered','1989-12-29 00:00:00','1987-03-23 00:00:00',103.77,521,1352,65.81,'Intermediate' from dual union all
select 17712,'017712','61 Omicron 279','deb r/b Ablestar','',7273,'Reentered','1988-08-01 00:00:00','1987-03-23 00:00:00',97.62,494,796,66.25,'Intermediate' from dual union all
select 17713,'017713','61 Omicron 280','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-06 00:00:00',103.52,871,979,66.21,'Intermediate' from dual union all
select 17714,'017714','61 Omicron 281','deb r/b Ablestar','',7273,'Reentered','1990-01-15 00:00:00','1987-03-23 00:00:00',97.06,546,690,67.12,'Intermediate' from dual union all
select 17715,'017715','1970-025PD','deb Agena D','',21502,'Reentered','2002-09-15 00:00:00','1993-08-06 00:00:00',103.09,879,930,99.36,'Sun-Synch' from dual union all
select 17716,'017716','1970-025PE','deb Agena D','',21502,'In Earth orbit','','1993-08-04 00:00:00',103.89,906,978,99.48,'Sun-Synch' from dual union all
select 17717,'017717','1970-025PF','deb Agena D','',21502,'In Earth orbit','','1993-08-08 00:00:00',101.05,697,920,100.03,'Sun-Synch' from dual union all
select 17718,'017718','1970-025PG','deb Agena D','',21502,'Reentered','1989-12-22 00:00:00','1987-03-23 00:00:00',103.3,861,969,100.2,'Sun-Synch' from dual union all
select 17719,'017719','1970-025PH','deb Agena D','',21502,'In Earth orbit','','1993-08-07 00:00:00',104.73,863,1101,99.39,'Sun-Synch' from dual union all
select 1772,'001772','1965-094D','deb Kosmos-96','',11844,'Reentered','1965-11-28 00:00:00','1965-11-23 00:00:00',89.54,223,281,51.8,'Intermediate' from dual union all
select 17720,'017720','1970-025PJ','deb Agena D','',21502,'In Earth orbit','','1993-08-02 00:00:00',104.77,942,1026,99.6,'Sun-Synch' from dual union all
select 17721,'017721','1970-025PK','deb Agena D','',21502,'Reentered','1989-08-17 00:00:00','1987-03-23 00:00:00',102.34,841,898,99.91,'Sun-Synch' from dual union all
select 17722,'017722','1965-027W','deb Snapshot','',11777,'In Earth orbit','','1993-08-08 00:00:00',111.26,1259,1311,90.27,'Polar' from dual union all
select 17723,'017723','1967-048C','deb NNS O-13 insulation','',15303,'Reentered','1992-02-03 00:00:00','1986-04-14 00:00:00',106.76,1058,1095,89.7,'Polar' from dual union all
select 17724,'017724','1969-082KY','deb Agena D','',18349,'Reentered','1989-07-03 00:00:00','1987-03-23 00:00:00',98.49,635,738,70.05,'Intermediate' from dual union all
select 17725,'017725','1970-089DC','deb Kosmos [*}','',21566,'In Earth orbit','','1971-08-25 00:00:00',111.99,518,2119,62.78,'Intermediate' from dual union all
select 17726,'017726','1971-015DD','deb Kosmos-397','',24833,'Reentered','2000-11-22 00:00:00','1993-08-09 00:00:00',101.13,526,1099,65.66,'Intermediate' from dual union all
select 17727,'017727','1972-058JL','deb Delta 89','',27990,'Reentered','2012-06-05 00:00:00','1993-08-06 00:00:00',98.89,593,819,98.48,'Sun-Synch' from dual union all
select 17728,'017728','1974-089EX','deb Delta 104','',33692,'In Earth orbit','','1993-08-08 00:00:00',116.74,1447,1622,101.49,'Sun-Synch' from dual union all
select 17729,'017729','1975-004HH','deb Delta 107','',36451,'Reentered','1992-05-04 00:00:00','1992-04-07 00:00:00',92.09,371,383,97.74,'Sun-Synch' from dual union all
select 1773,'001773','1965-094E','deb Kosmos-96','',11844,'Reentered','1965-11-29 00:00:00','1965-11-23 00:00:00',90.01,220,330,51.8,'Intermediate' from dual union all
select 17730,'017730','1975-004HJ','deb Delta 107','',36451,'Reentered','1989-05-09 00:00:00','1987-03-23 00:00:00',101.06,722,896,98.69,'Sun-Synch' from dual union all
select 17731,'017731','1977-121BN','deb Kosmos-970','',42249,'Reentered','1991-10-15 00:00:00','1991-08-26 00:00:00',105.18,930,1076,66,'Intermediate' from dual union all
select 17732,'017732','1977-121BP','deb Kosmos','[UNID109] RCS 0.0',42249,'In Earth orbit','','1978-01-20 00:00:00',106.62,871,1270,65.83,'Intermediate' from dual union all
select 17733,'017733','1983-044CW','deb Kosmos-1461','',55801,'Reentered','1988-12-22 00:00:00','1987-03-23 00:00:00',99.02,563,861,65,'Intermediate' from dual union all
select 17734,'017734','1983-044CX','deb Kosmos-1461','',55801,'Reentered','1989-09-13 00:00:00','1987-03-23 00:00:00',100.29,648,897,65.03,'Intermediate' from dual union all
select 17735,'017735','1983-044CY','deb Kosmos-1461','',55801,'Reentered','1989-05-11 00:00:00','1987-03-23 00:00:00',99.65,606,877,65.01,'Intermediate' from dual union all
select 17736,'017736','1983-044CZ','deb Kosmos-1461','',55801,'Reentered','1988-01-22 00:00:00','1987-03-23 00:00:00',97.99,540,786,65.11,'Intermediate' from dual union all
select 17737,'017737','1983-044DA','deb Kosmos-1461','',55801,'Reentered','1988-11-16 00:00:00','1987-03-23 00:00:00',98.54,532,846,65.05,'Intermediate' from dual union all
select 17738,'017738','1983-044DB','deb Kosmos-1461','',55801,'Reentered','1989-10-02 00:00:00','1987-03-23 00:00:00',99.28,573,876,65.02,'Intermediate' from dual;
insert into SATELLITE
select 17739,'017739','1983-044DC','deb Kosmos-1461','',55801,'Reentered','1991-02-13 00:00:00','1987-03-23 00:00:00',99.28,573,876,65.02,'Intermediate' from dual union all
select 1774,'001774','1965-094F','deb Kosmos-96','',11844,'Reentered','1965-11-28 00:00:00','1965-11-23 00:00:00',90.01,220,330,51.8,'Intermediate' from dual union all
select 17740,'017740','1983-044DD','deb Kosmos-1461','',55801,'Reentered','1989-10-15 00:00:00','1987-03-23 00:00:00',99.68,613,874,65,'Intermediate' from dual union all
select 17741,'017741','1983-044DE','deb Kosmos-1461','',55801,'Reentered','1989-05-06 00:00:00','1987-03-23 00:00:00',99.67,609,877,64.97,'Intermediate' from dual union all
select 17742,'017742','1983-044DF','deb Kosmos-1461','',55801,'Reentered','1988-11-03 00:00:00','1987-03-23 00:00:00',98.71,548,845,65,'Intermediate' from dual union all
select 17743,'017743','1983-044DG','deb Kosmos-1461','',55801,'Reentered','1988-04-02 00:00:00','1987-03-23 00:00:00',98.08,536,798,65.07,'Intermediate' from dual union all
select 17744,'017744','1983-044DH','deb Kosmos-1461','',55801,'Reentered','1990-03-22 00:00:00','1987-03-23 00:00:00',99.23,570,874,65,'Intermediate' from dual union all
select 17745,'017745','1983-044DJ','deb Kosmos-1461','',55801,'Reentered','1989-08-30 00:00:00','1987-03-23 00:00:00',98.75,529,869,65,'Intermediate' from dual union all
select 17746,'017746','1983-044DK','deb Kosmos-1461','',55801,'Reentered','1989-10-12 00:00:00','1987-03-23 00:00:00',100.36,662,890,64.92,'Intermediate' from dual union all
select 17747,'017747','1983-044DL','deb Kosmos-1461','',55801,'Reentered','1989-04-10 00:00:00','1987-03-23 00:00:00',99.49,594,875,65.03,'Intermediate' from dual union all
select 17748,'017748','1983-044DM','deb Kosmos-1461','',55801,'Reentered','1989-04-01 00:00:00','1987-03-23 00:00:00',99.56,604,872,65.01,'Intermediate' from dual union all
select 17749,'017749','1983-044DN','deb Kosmos-1461','',55801,'Reentered','1989-04-17 00:00:00','1987-03-23 00:00:00',99.45,588,877,65,'Intermediate' from dual union all
select 1775,'001775','1965-094G','deb Kosmos-96','',11844,'Reentered','1965-12-08 00:00:00','1965-11-23 00:00:00',90.11,224,335,51.8,'Intermediate' from dual union all
select 17750,'017750','1983-044DP','deb Kosmos-1461','',55801,'Reentered','1989-07-04 00:00:00','1987-03-23 00:00:00',98.66,535,855,65.03,'Intermediate' from dual union all
select 17751,'017751','1983-044DQ','deb Kosmos-1461','',55801,'Reentered','1989-06-05 00:00:00','1987-03-23 00:00:00',99.13,570,865,65.02,'Intermediate' from dual union all
select 17752,'017752','1983-044DR','deb Kosmos-1461','',55801,'Reentered','1989-12-27 00:00:00','1987-03-23 00:00:00',99.22,572,871,65.02,'Intermediate' from dual union all
select 17753,'017753','1983-044DS','deb Kosmos-1461','',55801,'Reentered','1989-05-14 00:00:00','1987-03-23 00:00:00',98.81,534,870,65.07,'Intermediate' from dual union all
select 17754,'017754','1985-079C','deb Kosmos-1680','Strela-2M debris (high)',58302,'In Earth orbit','','1986-04-14 00:00:00',101.27,801,837,74.05,'Intermediate' from dual union all
select 17755,'017755','1982-116D','deb Meteor-2','',54425,'In Earth orbit','','1986-04-14 00:00:00',101.96,800,903,81.26,'Intermediate' from dual union all
select 17756,'017756','1987-004BM','deb Kosmos-1813','',60094,'Reentered','1989-03-18 00:00:00','1987-03-23 00:00:00',92.51,362,434,72.81,'Intermediate' from dual union all
select 17757,'017757','1987-004BN','deb Kosmos-1813','',60094,'Reentered','1987-10-10 00:00:00','1987-03-23 00:00:00',96.47,361,819,72.79,'Intermediate' from dual union all
select 17758,'017758','1987-004BP','deb Kosmos-1813','',60094,'Reentered','1987-10-19 00:00:00','1987-03-23 00:00:00',92.94,374,463,72.8,'Intermediate' from dual union all
select 17759,'017759','1987-004BQ','deb Kosmos-1813','',60094,'Reentered','1987-09-13 00:00:00','1987-03-23 00:00:00',93.05,357,491,72.79,'Intermediate' from dual union all
select 1776,'001776','1965-094H','deb Kosmos-96','',11844,'Reentered','1965-12-08 00:00:00','1965-11-23 00:00:00',89.81,224,306,51.9,'Intermediate' from dual union all
select 17760,'017760','1987-004BR','deb Kosmos-1813','',60094,'Reentered','1987-12-29 00:00:00','1987-03-23 00:00:00',92.51,363,432,72.8,'Intermediate' from dual union all
select 17761,'017761','1987-004BS','deb Kosmos-1813','',60094,'Reentered','1988-01-09 00:00:00','1987-03-23 00:00:00',92.7,365,449,72.81,'Intermediate' from dual union all
select 17762,'017762','1987-004BT','deb Kosmos-1813','',60094,'Reentered','1989-01-17 00:00:00','1987-03-23 00:00:00',94.73,366,645,72.81,'Intermediate' from dual union all
select 17763,'017763','1978-026GV','deb Delta 139','',45221,'Reentered','1991-05-26 00:00:00','1991-05-25 00:00:00',91.49,330,365,98.68,'Sun-Synch' from dual union all
select 17764,'017764','1978-026GW','deb Delta 139','',45221,'Reentered','1989-06-05 00:00:00','1993-08-08 00:00:00',102.27,831,902,99.01,'Sun-Synch' from dual union all
select 17765,'017765','1978-026GX','deb Delta 139','',45221,'In Earth orbit','','1993-08-01 00:00:00',103.26,881,945,99.16,'Sun-Synch' from dual union all
select 17766,'017766','1978-026GY','deb Delta 139','',45221,'Reentered','1991-09-06 00:00:00','1991-09-05 00:00:00',90.98,307,339,98.74,'Sun-Synch' from dual union all
select 17767,'017767','1978-026GZ','deb Delta 139','',45221,'Reentered','1989-08-12 00:00:00','1987-03-23 00:00:00',99.8,682,816,98.91,'Sun-Synch' from dual union all
select 17768,'017768','1978-026HA','deb Delta 139','',45221,'In Earth orbit','','1993-07-31 00:00:00',100.82,730,865,98.55,'Sun-Synch' from dual union all
select 17769,'017769','1978-026HB','deb Delta 139','',45221,'Reentered','2000-05-18 00:00:00','1993-08-06 00:00:00',101.06,703,915,98.71,'Sun-Synch' from dual union all
select 1777,'001777','1965-095A','Kosmos-97','DS-U2-M No. 1',11845,'Reentered','1967-04-02 00:00:00','1966-01-30 00:00:00',107.66,217,2020,48.45,'Intermediate' from dual union all
select 17770,'017770','1979-017JT','deb P78-1','',47501,'Reentered','1989-02-26 00:00:00','1987-03-23 00:00:00',94.93,490,541,97.66,'Sun-Synch' from dual union all
select 17771,'017771','1979-017JU','deb P78-1','',47501,'Reentered','1988-07-13 00:00:00','1987-03-23 00:00:00',94.78,493,522,97.73,'Sun-Synch' from dual union all
select 17772,'017772','1979-017JV','deb P78-1','',47501,'Reentered','1989-03-21 00:00:00','1987-03-23 00:00:00',94.87,505,520,97.81,'Sun-Synch' from dual union all
select 17773,'017773','1979-017JW','deb P78-1','',47501,'Reentered','1988-01-27 00:00:00','1987-03-23 00:00:00',93.79,434,486,97.73,'Sun-Synch' from dual union all
select 17774,'017774','1979-017JX','deb P78-1','',47501,'Reentered','1989-04-26 00:00:00','1987-03-23 00:00:00',95.32,495,574,97.64,'Sun-Synch' from dual union all
select 17775,'017775','1979-017JY','deb P78-1','',47501,'Reentered','1988-08-06 00:00:00','1987-03-23 00:00:00',94.19,452,507,97.72,'Sun-Synch' from dual union all
select 17776,'017776','1979-017JZ','deb P78-1','',47501,'Reentered','1988-06-25 00:00:00','1987-03-23 00:00:00',95.37,505,568,98.11,'Sun-Synch' from dual union all
select 17777,'017777','1979-017KA','deb P78-1','',47501,'Reentered','1988-12-16 00:00:00','1987-03-23 00:00:00',95.62,511,587,97.98,'Sun-Synch' from dual union all
select 17778,'017778','1979-017KB','deb P78-1','',47501,'Reentered','1988-09-16 00:00:00','1987-03-23 00:00:00',94.82,507,512,97.94,'Sun-Synch' from dual union all
select 17779,'017779','1979-017KC','deb P78-1','',47501,'Reentered','1987-06-05 00:00:00','1987-03-23 00:00:00',93.55,440,457,97.66,'Sun-Synch' from dual union all
select 1778,'001778','1965-096A','Asterix','A-1',11846,'In Earth orbit','','1966-01-29 00:00:00',108.65,527,1801,34.24,'Intermediate' from dual union all
select 17780,'017780','1979-017KD','deb P78-1','',47501,'Reentered','1988-06-17 00:00:00','1987-03-23 00:00:00',95.42,505,573,98.17,'Sun-Synch' from dual union all
select 17781,'017781','1979-017KE','deb P78-1','',47501,'Reentered','1991-11-10 00:00:00','1991-11-03 00:00:00',89.12,224,238,97.9,'Sun-Synch' from dual union all
select 17782,'017782','1979-017KF','deb P78-1','',47501,'Reentered','1988-02-21 00:00:00','1987-03-23 00:00:00',94.95,491,542,98,'Sun-Synch' from dual union all
select 17783,'017783','1979-017KG','deb P78-1','',47501,'Reentered','1988-08-26 00:00:00','1987-03-23 00:00:00',94.99,502,535,97.84,'Sun-Synch' from dual union all
select 17784,'017784','1979-017KH','deb P78-1','',47501,'Reentered','1988-06-04 00:00:00','1987-03-23 00:00:00',94.69,491,517,97.75,'Sun-Synch' from dual union all
select 17785,'017785','1987-025F','KDU part','KDU part',60115,'Reentered','1987-06-30 00:00:00','1987-04-10 00:00:00',92.1,351,404,72.86,'Intermediate' from dual union all
select 17786,'017786','1987-025G','KDU','KDU',60115,'Reentered','1988-08-30 00:00:00','1987-04-10 00:00:00',92.22,355,412,72.87,'Intermediate' from dual union all
select 17787,'017787','1987-025H','KDU part','KDU part',60115,'Reentered','1987-04-08 00:00:00','1987-03-25 00:00:00',92.01,321,426,73.31,'Intermediate' from dual union all
select 17788,'017788','1987-025J','KDU part','KDU part',60115,'Reentered','1987-09-16 00:00:00','1987-04-10 00:00:00',91.99,340,404,72.87,'Intermediate' from dual union all
select 17789,'017789','1987-025K','Blenda','Sunshade',60115,'Reentered','1987-11-21 00:00:00','1987-04-10 00:00:00',94.2,388,572,72.87,'Intermediate' from dual union all
select 1779,'001779','1965-095B','S1M','S1M',11845,'Reentered','1967-02-21 00:00:00','1966-01-28 00:00:00',107.25,214,1985,48.48,'Intermediate' from dual union all
select 17790,'017790','1987-025L','KDU part','KDU part',60115,'Reentered','1987-05-03 00:00:00','1987-03-25 00:00:00',92.96,337,502,73.42,'Intermediate' from dual union all
select 17791,'017791','1987-025M','KDU part','KDU part',60115,'Reentered','1987-04-18 00:00:00','1987-03-25 00:00:00',93.2,313,549,72.42,'Intermediate' from dual union all
select 17792,'017792','1981-053LN','deb Kosmos-1275','',52576,'In Earth orbit','','1993-07-29 00:00:00',104,930,965,82.96,'Intermediate' from dual union all
select 17793,'017793','1981-053LP','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-08 00:00:00',109.16,956,1420,83.14,'Intermediate' from dual union all
select 17794,'017794','1982-055AL','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-07 00:00:00',104.85,941,1034,65.83,'Intermediate' from dual union all
select 17795,'017795','1982-055AM','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-03 00:00:00',105.92,984,1091,65.83,'Intermediate' from dual union all
select 17796,'017796','1982-055AN','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-07 00:00:00',103.49,883,965,65.83,'Intermediate' from dual union all
select 17797,'017797','1982-055AP','deb Kosmos-1375','',54364,'In Earth orbit','','1993-07-31 00:00:00',104.52,970,974,65.83,'Intermediate' from dual union all
select 17798,'017798','1982-055AQ','deb Kosmos-1375','',54364,'In Earth orbit','','1993-07-27 00:00:00',105.99,984,1098,65.84,'Intermediate' from dual union all
select 17799,'017799','1982-055AR','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-06 00:00:00',104.1,930,975,65.84,'Intermediate' from dual union all
select 178,'000178','61 Omicron 51','deb r/b Ablestar','',7273,'In Earth orbit','','1961-09-23 00:00:00',103.82,873,1006,66.87,'Intermediate' from dual union all
select 1780,'001780','1965-097A','Kosmos-98','Zenit-2 No. 31',11847,'Landed','1965-12-05 00:00:00','1965-11-28 00:00:00',92.06,205,546,65.03,'Intermediate' from dual union all
select 17800,'017800','1982-055AS','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-04 00:00:00',105.19,977,1030,65.83,'Intermediate' from dual union all
select 17801,'017801','1982-055AT','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-01 00:00:00',103.45,888,956,65.83,'Intermediate' from dual union all
select 17802,'017802','1982-055AU','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-02 00:00:00',105,973,1016,65.83,'Intermediate' from dual union all
select 17803,'017803','1982-055AV','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-06 00:00:00',104.65,951,1005,65.83,'Intermediate' from dual union all
select 17804,'017804','1982-055AW','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-07 00:00:00',104.45,959,978,65.83,'Intermediate' from dual union all
select 17805,'017805','1982-055AX','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-04 00:00:00',104.37,952,978,65.83,'Intermediate' from dual union all
select 17806,'017806','1982-055AY','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-07 00:00:00',104.37,923,1006,65.84,'Intermediate' from dual union all
select 17807,'017807','1982-055AZ','deb Kosmos-1375','',54364,'In Earth orbit','','1993-08-09 00:00:00',105.16,965,1039,65.83,'Intermediate' from dual union all
select 17808,'017808','1986-019LX','deb Ariane V16','',59212,'Reentered','1988-03-01 00:00:00','1987-03-28 00:00:00',97.6,515,773,99.3,'Sun-Synch' from dual union all
select 17809,'017809','1986-019LY','deb Ariane V16','',59212,'Reentered','1989-08-27 00:00:00','1987-03-28 00:00:00',100.96,748,861,98.56,'Sun-Synch' from dual union all
select 1781,'001781','1965-097B','8A92E','8A92E',11847,'Reentered','1966-01-23 00:00:00','1965-11-28 00:00:00',92.19,198,566,65.03,'Intermediate' from dual union all
select 17810,'017810','1986-019LZ','deb Ariane V16','',59212,'Reentered','1989-01-19 00:00:00','1987-03-28 00:00:00',101.09,794,826,98.43,'Sun-Synch' from dual union all
select 17811,'017811','1986-019MA','deb Ariane V16','',59212,'Reentered','1988-01-22 00:00:00','1987-03-28 00:00:00',100.98,754,857,98.61,'Sun-Synch' from dual union all
select 17812,'017812','1986-019MB','deb Ariane V16','',59212,'Reentered','1988-07-05 00:00:00','1987-03-28 00:00:00',99.86,728,776,98.86,'Sun-Synch' from dual union all
select 17813,'017813','1986-019MC','deb Ariane V16','',59212,'Reentered','1991-02-15 00:00:00','1987-08-15 00:00:00',104.21,801,1114,98.03,'Sun-Synch' from dual union all
select 17814,'017814','1986-019MD','deb Ariane V16','',59212,'Reentered','1988-09-30 00:00:00','1987-03-28 00:00:00',100.86,759,840,98.66,'Sun-Synch' from dual union all
select 17815,'017815','1986-019ME','deb Ariane V16','',59212,'In Earth orbit','','1993-08-04 00:00:00',101.02,782,832,98.37,'Sun-Synch' from dual union all
select 17816,'017816','1986-019MF','deb Ariane V16','',59212,'Reentered','1989-03-27 00:00:00','1987-03-28 00:00:00',101.06,804,814,98.69,'Sun-Synch' from dual union all
select 17817,'017817','1986-019MG','deb Ariane V16','',59212,'Reentered','1988-08-31 00:00:00','1987-03-28 00:00:00',99.23,603,841,99.33,'Sun-Synch' from dual union all
select 17818,'017818','1985-050AD','Kosmos-1662 SS 22','ESO',58273,'Reentered','1988-07-25 00:00:00','1987-03-26 00:00:00',94.4,467,511,65.84,'Intermediate' from dual union all
select 17819,'017819','1985-050AE','Kosmos-1662 SS 23','ESO',58273,'Reentered','1988-07-12 00:00:00','1987-03-26 00:00:00',94.37,465,512,65.84,'Intermediate' from dual union all
select 1782,'001782','1965-095C','deb Kosmos-97','',11845,'Reentered','1966-01-21 00:00:00','1965-11-27 00:00:00',110.57,212,2294,48.4,'Intermediate' from dual union all
select 17820,'017820','1986-017C','Model VLF part','Model VLF',59210,'Reentered','1987-05-07 00:00:00','1987-04-02 00:00:00',91.5,344,353,51.63,'Intermediate' from dual union all
select 17821,'017821','1986-017D','Model VLF part','Model VLF',59210,'Reentered','1987-05-06 00:00:00','1987-04-04 00:00:00',91.54,342,359,51.62,'Intermediate' from dual union all
select 17822,'017822','1979-017KJ','deb P78-1','',47501,'Reentered','1988-04-29 00:00:00','1987-03-28 00:00:00',94.56,497,498,98.03,'Sun-Synch' from dual union all
select 17823,'017823','1979-017KK','deb P78-1','',47501,'Reentered','1988-11-12 00:00:00','1987-03-28 00:00:00',94.86,496,528,97.79,'Sun-Synch' from dual union all
select 17824,'017824','1979-017KL','deb P78-1','',47501,'Reentered','1991-05-04 00:00:00','1987-03-28 00:00:00',95.36,512,560,97.83,'Sun-Synch' from dual union all
select 17825,'017825','1979-017KM','deb P78-1','',47501,'Reentered','1991-10-22 00:00:00','1991-10-21 00:00:00',88.14,177,188,97.87,'Sun-Synch' from dual union all
select 17826,'017826','1979-017KN','deb P78-1','',47501,'Reentered','1989-05-21 00:00:00','1987-03-28 00:00:00',95.19,508,548,97.73,'Sun-Synch' from dual union all
select 17827,'017827','1979-017KP','deb P78-1','',47501,'Reentered','1988-10-07 00:00:00','1987-03-28 00:00:00',94.93,504,526,97.92,'Sun-Synch' from dual union all
select 17828,'017828','1979-017KQ','deb P78-1','',47501,'Reentered','1987-08-15 00:00:00','1987-03-28 00:00:00',94.19,440,519,99.03,'Sun-Synch' from dual;
insert into SATELLITE
select 17829,'017829','1979-017KR','deb P78-1','',47501,'Reentered','1988-02-12 00:00:00','1987-03-28 00:00:00',92.32,319,458,97.81,'Sun-Synch' from dual union all
select 1783,'001783','1965-095D','deb Kosmos-97','',11845,'Reentered','1966-01-21 00:00:00','1965-11-27 00:00:00',108.45,192,2118,48.4,'Intermediate' from dual union all
select 17830,'017830','1979-017KS','deb P78-1','',47501,'Reentered','1988-09-19 00:00:00','1987-03-28 00:00:00',95.02,487,552,97.67,'Sun-Synch' from dual union all
select 17831,'017831','1979-017KT','deb P78-1','',47501,'Reentered','1988-10-05 00:00:00','1987-03-28 00:00:00',94.83,508,513,97.69,'Sun-Synch' from dual union all
select 17832,'017832','1979-017KU','deb P78-1','',47501,'Reentered','1989-02-13 00:00:00','1987-03-28 00:00:00',95.36,512,560,97.76,'Sun-Synch' from dual union all
select 17833,'017833','1983-044DT','deb Kosmos-1461','',55801,'Reentered','1989-05-11 00:00:00','1987-03-28 00:00:00',99.05,554,872,65.06,'Intermediate' from dual union all
select 17834,'017834','1983-044DU','deb Kosmos-1461','',55801,'Reentered','1989-05-22 00:00:00','1987-03-28 00:00:00',99.58,606,871,64.96,'Intermediate' from dual union all
select 17835,'017835','1983-044DV','deb Kosmos-1461','',55801,'Reentered','1989-01-19 00:00:00','1987-03-28 00:00:00',98.78,549,852,65,'Intermediate' from dual union all
select 17836,'017836','1983-044DW','deb Kosmos-1461','',55801,'Reentered','1989-04-10 00:00:00','1987-03-28 00:00:00',99.45,596,869,65.03,'Intermediate' from dual union all
select 17837,'017837','1983-044DX','deb Kosmos-1461','',55801,'Reentered','1989-03-29 00:00:00','1987-03-28 00:00:00',99.23,579,864,65.02,'Intermediate' from dual union all
select 17838,'017838','1976-067BK','deb Kosmos-839','',39019,'In Earth orbit','','1993-07-19 00:00:00',116.34,999,2034,65.99,'Intermediate' from dual union all
select 17839,'017839','1984-011Y','deb PAM-D','',57065,'Reentered','1989-04-12 00:00:00','1984-03-05 00:00:00',99.61,280,1200,28.36,'Intermediate' from dual union all
select 1784,'001784','1965-088F','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-11 00:00:00','1965-11-05 00:00:00',90.2,194,375,48.4,'Intermediate' from dual union all
select 17840,'017840','1985-082W','deb Kosmos-1682','',58305,'Reentered','1987-08-11 00:00:00','1987-01-24 00:00:00',93.08,377,474,65.01,'Intermediate' from dual union all
select 17841,'017841','1985-082X','deb Kosmos-1682','',58305,'Reentered','1988-04-19 00:00:00','1987-01-01 00:00:00',93.16,392,466,65.03,'Intermediate' from dual union all
select 17842,'017842','1986-030C','deb Kosmos-1741','[UNID]',59223,'In Earth orbit','','1986-05-22 00:00:00',101.32,805,838,74.03,'Intermediate' from dual union all
select 17843,'017843','1986-030D','deb Kosmos-1741','[UNID]',59223,'In Earth orbit','','1986-05-19 00:00:00',101.31,806,835,74.04,'Intermediate' from dual union all
select 17844,'017844','1986-101K','deb S5M','',59294,'In Earth orbit','','1987-01-21 00:00:00',103.81,923,955,82.51,'Intermediate' from dual union all
select 17845,'017845','1987-030A','Kvant','TsM-e 37Ke No. 010',60120,'Reentered Attached','2001-03-23 00:00:00','1987-04-30 00:00:00',91.6,343,364,51.64,'Intermediate' from dual union all
select 17846,'017846','1987-030B','8S812','8S812',60120,'Reentered','1987-04-04 00:00:00','1987-04-02 00:00:00',88.54,160,245,51.64,'Intermediate' from dual union all
select 17847,'017847','1987-031A','Kosmos-1834','US-PM',60121,'Reentered','1988-10-14 00:00:00','1987-05-08 00:00:00',92.76,403,417,65.04,'Intermediate' from dual union all
select 17848,'017848','1987-031B','11S692','11S692',60121,'Reentered','1987-04-08 00:00:00','1987-04-08 00:00:00',89.37,115,372,65.02,'Intermediate' from dual union all
select 17849,'017849','1987-032A','Kosmos-1835','Kobal''t',60122,'Landed','1987-06-04 00:00:00','1987-04-16 00:00:00',89.71,174,347,64.76,'Intermediate' from dual union all
select 1785,'001785','1965-088G','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-16 00:00:00','1965-11-05 00:00:00',93.17,195,665,48.4,'Intermediate' from dual union all
select 17850,'017850','1987-032B','11S510','11S510',60122,'Reentered','1987-04-14 00:00:00','1987-04-11 00:00:00',89.3,169,310,64.77,'Intermediate' from dual union all
select 17851,'017851','1987-030C','FSB','FSB No. 16601',60120,'Reentered','1988-08-25 00:00:00','1987-05-19 00:00:00',92.39,380,403,51.63,'Intermediate' from dual union all
select 17852,'017852','1987-004BU','deb Kosmos-1813','',60094,'Reentered','1988-08-14 00:00:00','1987-04-15 00:00:00',94.94,382,650,72.79,'Intermediate' from dual union all
select 17853,'017853','1987-004BV','deb Kosmos-1813','',60094,'Reentered','1987-07-09 00:00:00','1987-04-15 00:00:00',91.39,317,369,72.8,'Intermediate' from dual union all
select 17854,'017854','1987-004BW','deb Kosmos-1813','',60094,'Reentered','1987-10-11 00:00:00','1987-04-15 00:00:00',92.28,355,418,72.79,'Intermediate' from dual union all
select 17855,'017855','1987-004BX','deb Kosmos-1813','',60094,'Reentered','1987-12-31 00:00:00','1987-04-15 00:00:00',91.51,305,393,72.81,'Intermediate' from dual union all
select 17856,'017856','1987-004BY','deb Kosmos-1813','',60094,'Reentered','1987-04-13 00:00:00','1987-04-15 00:00:00',91.51,341,357,72.79,'Intermediate' from dual union all
select 17857,'017857','1987-004BZ','deb Kosmos-1813','',60094,'Reentered','1987-10-31 00:00:00','1987-04-15 00:00:00',91.43,306,384,72.81,'Intermediate' from dual union all
select 17858,'017858','1987-004CA','deb Kosmos-1813','',60094,'Reentered','1987-09-12 00:00:00','1987-04-15 00:00:00',91.78,341,383,72.82,'Intermediate' from dual union all
select 17859,'017859','1987-004CB','deb Kosmos-1813','',60094,'Reentered','1987-05-09 00:00:00','1987-04-15 00:00:00',90.26,262,313,72.81,'Intermediate' from dual union all
select 1786,'001786','1965-088H','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-09 00:00:00','1965-11-05 00:00:00',92.71,203,611,48.4,'Intermediate' from dual union all
select 17860,'017860','1987-004CC','deb Kosmos-1813','',60094,'Reentered','1987-04-15 00:00:00','1987-04-15 00:00:00',91.36,316,367,72.78,'Intermediate' from dual union all
select 17861,'017861','1987-004CD','deb Kosmos-1813','',60094,'Reentered','1987-09-11 00:00:00','1987-04-15 00:00:00',91.37,304,380,72.81,'Intermediate' from dual union all
select 17862,'017862','1987-004CE','deb Kosmos-1813','',60094,'Reentered','1987-09-14 00:00:00','1987-04-15 00:00:00',91.76,328,394,72.8,'Intermediate' from dual union all
select 17863,'017863','1987-004CF','deb Kosmos-1813','',60094,'Reentered','1987-08-28 00:00:00','1987-04-15 00:00:00',91.68,330,384,72.82,'Intermediate' from dual union all
select 17864,'017864','1987-004CG','deb Kosmos-1813','',60094,'Reentered','1987-10-06 00:00:00','1987-04-15 00:00:00',91.88,345,389,72.8,'Intermediate' from dual union all
select 17865,'017865','1987-004CH','deb Kosmos-1813','',60094,'Reentered','1987-05-21 00:00:00','1987-04-15 00:00:00',90.3,249,330,72.8,'Intermediate' from dual union all
select 17866,'017866','1987-004CJ','deb Kosmos-1813','',60094,'Reentered','1987-07-11 00:00:00','1987-04-15 00:00:00',91.03,296,355,72.8,'Intermediate' from dual union all
select 17867,'017867','1987-004CK','deb Kosmos-1813','',60094,'Reentered','1987-08-13 00:00:00','1987-04-15 00:00:00',91.75,342,379,72.81,'Intermediate' from dual union all
select 17868,'017868','1987-004CL','deb Kosmos-1813','',60094,'Reentered','1987-05-24 00:00:00','1987-04-15 00:00:00',90.47,261,335,72.81,'Intermediate' from dual union all
select 17869,'017869','1987-004CM','deb Kosmos-1813','',60094,'Reentered','1987-07-13 00:00:00','1987-04-15 00:00:00',91.42,308,381,72.81,'Intermediate' from dual union all
select 1787,'001787','1965-088J','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-09 00:00:00','1965-11-05 00:00:00',91.03,202,449,48.4,'Intermediate' from dual union all
select 17870,'017870','1987-004CN','deb Kosmos-1813','',60094,'Reentered','1987-06-02 00:00:00','1987-04-15 00:00:00',91.5,324,373,72.8,'Intermediate' from dual union all
select 17871,'017871','1987-004CP','deb Kosmos-1813','',60094,'Reentered','1987-10-15 00:00:00','1987-04-15 00:00:00',91.31,305,373,72.81,'Intermediate' from dual union all
select 17872,'017872','1976-092F','Blok DM','11S86 No. 5L',39044,'In Earth orbit','','1977-07-23 00:00:00',1435.88,35706,35859,.28,'Stationary' from dual union all
select 17873,'017873','1979-035E','Blok DM','11S86 No. 15L',47519,'In Earth orbit','','1980-06-23 00:00:00',1437.9,35708,35936,.46,'Drift GEO' from dual union all
select 17874,'017874','1984-016F','Blok DM','11S86 No. 42L',57070,'In Earth orbit','','1984-08-23 00:00:00',1436.57,35257,36335,.84,'Drift GEO' from dual union all
select 17875,'017875','1984-090F','Blok DM','11S86 No. 51L',57144,'In Earth orbit','','1984-11-04 00:00:00',1422.03,35431,35591,.31,'Drift GEO' from dual union all
select 17876,'017876','1987-033A','Kosmos-1836','Terilen',60123,'Deorbited','1987-12-02 00:00:00','1987-05-17 00:00:00',89.37,226,261,64.79,'Intermediate' from dual union all
select 17877,'017877','1987-033B','11S510','11S510',60123,'Reentered','1987-04-20 00:00:00','1987-04-18 00:00:00',88.66,170,247,64.81,'Intermediate' from dual union all
select 17878,'017878','1987-034A','Progress-29','Progress 7K-TG No. 127',60124,'Deorbited','1987-05-11 00:00:00','1987-04-28 00:00:00',91.59,342,363,51.63,'Intermediate' from dual union all
select 17879,'017879','1987-034B','11S510','11S510',60124,'Reentered','1987-04-24 00:00:00','1987-04-23 00:00:00',88.3,175,206,51.63,'Intermediate' from dual union all
select 1788,'001788','1965-088K','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-19 00:00:00','1965-11-05 00:00:00',91.51,210,487,48.4,'Intermediate' from dual union all
select 17880,'017880','1987-035A','Kosmos-1837','Oblik No. 357',60125,'Landed','1987-04-28 00:00:00','1987-04-24 00:00:00',89.21,225,246,82.32,'Intermediate' from dual union all
select 17881,'017881','1987-035B','11S510','11S510',60125,'Reentered','1987-04-24 00:00:00','1987-04-23 00:00:00',88.38,175,214,82.32,'Intermediate' from dual union all
select 17882,'017882','1987-004CQ','deb Kosmos-1813','',60094,'Reentered','1988-05-04 00:00:00','1987-04-22 00:00:00',92.46,381,410,72.79,'Intermediate' from dual union all
select 17883,'017883','1987-004CR','deb Kosmos-1813','',60094,'Reentered','1987-12-05 00:00:00','1987-04-22 00:00:00',91.57,320,384,72.81,'Intermediate' from dual union all
select 17884,'017884','1987-004CS','deb Kosmos-1813 part','Oblik debris',60094,'Reentered','1988-08-03 00:00:00','1987-04-22 00:00:00',92.15,352,407,72.81,'Intermediate' from dual union all
select 17885,'017885','1987-004CT','deb Kosmos-1813','',60094,'Reentered','1988-01-26 00:00:00','1987-04-22 00:00:00',92.11,378,378,72.8,'Intermediate' from dual union all
select 17886,'017886','1987-004CU','deb Kosmos-1813','',60094,'Reentered','1988-09-12 00:00:00','1987-04-22 00:00:00',92.18,338,424,72.81,'Intermediate' from dual union all
select 17887,'017887','1987-004CV','deb Kosmos-1813','',60094,'Reentered','1987-12-29 00:00:00','1987-04-22 00:00:00',91.27,297,377,72.81,'Intermediate' from dual union all
select 17888,'017888','1987-004CW','deb Kosmos-1813','',60094,'Reentered','1987-08-17 00:00:00','1987-04-22 00:00:00',91.26,296,377,72.81,'Intermediate' from dual union all
select 17889,'017889','1987-004CX','deb Kosmos-1813','',60094,'Reentered','1987-08-13 00:00:00','1987-04-22 00:00:00',91.57,326,377,72.79,'Intermediate' from dual union all
select 1789,'001789','1965-088L','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-28 00:00:00','1965-11-05 00:00:00',91.49,212,484,48.4,'Intermediate' from dual union all
select 17890,'017890','1987-004CY','deb Kosmos-1813','',60094,'Reentered','1987-08-25 00:00:00','1987-04-22 00:00:00',91.56,325,378,72.79,'Intermediate' from dual union all
select 17891,'017891','1987-004CZ','deb Kosmos-1813','',60094,'Reentered','1987-11-25 00:00:00','1987-04-22 00:00:00',91.74,328,391,72.8,'Intermediate' from dual union all
select 17892,'017892','1987-004DA','deb Kosmos-1813','',60094,'Reentered','1988-04-02 00:00:00','1987-04-22 00:00:00',91.93,340,398,72.81,'Intermediate' from dual union all
select 17893,'017893','1987-004DB','deb Kosmos-1813','',60094,'Reentered','1987-10-14 00:00:00','1987-04-22 00:00:00',91.72,332,386,72.81,'Intermediate' from dual union all
select 17894,'017894','1987-004DC','deb Kosmos-1813','',60094,'Reentered','1987-09-24 00:00:00','1987-04-22 00:00:00',91.72,332,386,72.81,'Intermediate' from dual union all
select 17895,'017895','1987-004DD','deb Kosmos-1813','',60094,'Reentered','1987-10-05 00:00:00','1987-04-22 00:00:00',96.42,385,790,72.8,'Intermediate' from dual union all
select 17896,'017896','1987-004DE','deb Kosmos-1813','',60094,'Reentered','1988-08-26 00:00:00','1987-04-22 00:00:00',91.31,296,381,72.81,'Intermediate' from dual union all
select 17897,'017897','1987-004DF','deb Kosmos-1813','',60094,'Reentered','1987-09-11 00:00:00','1987-04-22 00:00:00',91.29,295,381,72.81,'Intermediate' from dual union all
select 17898,'017898','1987-004DG','deb Kosmos-1813','',60094,'Reentered','1988-09-15 00:00:00','1987-04-22 00:00:00',100.52,358,1209,72.86,'Intermediate' from dual union all
select 17899,'017899','1987-004DH','deb Kosmos-1813','',60094,'Reentered','1988-11-05 00:00:00','1987-04-22 00:00:00',94.73,384,627,72.8,'Intermediate' from dual union all
select 179,'000179','61 Omicron 52','deb r/b Ablestar','',7273,'Reentered','2000-03-03 00:00:00','1961-10-10 00:00:00',106.25,974,1132,66.9,'Intermediate' from dual union all
select 1790,'001790','1965-088M','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-12 00:00:00','1965-11-05 00:00:00',91.19,205,461,48.4,'Intermediate' from dual union all
select 17900,'017900','1987-004DJ','deb Kosmos-1813','',60094,'Reentered','1989-10-25 00:00:00','1987-04-22 00:00:00',96.91,384,838,72.8,'Intermediate' from dual union all
select 17901,'017901','1987-004DK','deb Kosmos-1813','',60094,'Reentered','1988-07-05 00:00:00','1987-04-22 00:00:00',97.48,376,901,72.82,'Intermediate' from dual union all
select 17902,'017902','1987-036A','Kosmos-1838','Uragan No. 30L',60126,'Reentered','1991-05-15 00:00:00','1987-05-25 00:00:00',311.14,188,17476,64.89,'Highly Ellip' from dual union all
select 17903,'017903','1987-036B','Kosmos-1839','Uragan No. 31L',60126,'Reentered','1991-05-08 00:00:00','1987-05-27 00:00:00',311,187,17467,64.84,'Highly Ellip' from dual union all
select 17904,'017904','1987-036C','Kosmos-1840','Uragan No. 32L',60126,'Reentered','1991-09-14 00:00:00','1987-05-24 00:00:00',311.44,190,17494,64.84,'Highly Ellip' from dual union all
select 17905,'017905','1987-036D','8S812','8S812',60126,'Reentered','1987-04-25 00:00:00','1987-04-25 00:00:00',86.32,88,96,64.81,'Intermediate' from dual union all
select 17906,'017906','1987-036E','Perekhodnik','Sredniy perekhodnik',60126,'Reentered','1987-04-27 00:00:00','1987-04-26 00:00:00',90.67,129,485,64.77,'Intermediate' from dual union all
select 17907,'017907','1987-037A','Kosmos-1841','Foton 34KS No. 3L',60127,'Landed','1987-05-08 00:00:00','1987-04-27 00:00:00',90.46,216,379,62.84,'Intermediate' from dual union all
select 17908,'017908','1987-037B','11S510','11S510',60127,'Reentered','1987-05-22 00:00:00','1987-05-01 00:00:00',90.21,213,357,62.82,'Intermediate' from dual union all
select 17909,'017909','1987-036F','SOZ','SOZ',60126,'Reentered','1987-08-12 00:00:00','1987-05-07 00:00:00',286.39,174,15853,65.01,'Highly Ellip' from dual union all
select 1791,'001791','1965-088N','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-07 00:00:00','1965-11-05 00:00:00',93.6,211,690,48.4,'Intermediate' from dual union all
select 17910,'017910','1987-036G','Blok DM2','11S861 No. 18L',60126,'Reentered','1991-04-12 00:00:00','1987-05-24 00:00:00',313.06,177,17612,64.85,'Highly Ellip' from dual union all
select 17911,'017911','1987-038A','Kosmos-1842','Tselina-D',60128,'In Earth orbit','','1987-05-27 00:00:00',97.72,633,666,82.51,'Intermediate' from dual union all
select 17912,'017912','1987-038B','S5M','S5M',60128,'In Earth orbit','','1987-05-27 00:00:00',97.69,632,665,82.5,'Intermediate' from dual union all
select 17913,'017913','1987-036H','SOZ','SOZ',60126,'Reentered','1991-11-22 00:00:00','1987-07-09 00:00:00',231.03,196,11991,64.76,'Medium' from dual union all
select 17914,'017914','1987-035C','KDU','KDU',60125,'Reentered','1987-05-18 00:00:00','1987-04-29 00:00:00',89.15,222,243,82.32,'Intermediate' from dual union all
select 17915,'017915','1987-035D','Blenda','Sunshade',60125,'Reentered','1987-04-30 00:00:00','1987-04-28 00:00:00',88.91,214,228,82.32,'Intermediate' from dual union all
select 17916,'017916','1987-035E','KDU part','KDU part',60125,'Reentered','1987-05-02 00:00:00','1987-04-28 00:00:00',89.06,216,240,82.32,'Intermediate' from dual union all
select 17917,'017917','1987-035F','KDU part','KDU part',60125,'Reentered','1987-05-08 00:00:00','1987-04-29 00:00:00',91.02,239,411,82.32,'Intermediate' from dual union all
select 17918,'017918','1987-035G','KDU part','KDU part',60125,'Reentered','1987-05-02 00:00:00','1987-04-29 00:00:00',91.37,234,450,82.35,'Intermediate' from dual;
insert into SATELLITE
select 17919,'017919','1987-004DL','deb Kosmos-1813','',60094,'Reentered','1989-01-22 00:00:00','1987-04-29 00:00:00',98.18,374,969,72.82,'Intermediate' from dual union all
select 1792,'001792','1965-088P','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-16 00:00:00','1965-11-05 00:00:00',91.12,193,465,48.4,'Intermediate' from dual union all
select 17920,'017920','1987-004DM','deb Kosmos-1813','',60094,'Reentered','1988-08-03 00:00:00','1987-04-29 00:00:00',93.56,317,581,72.84,'Intermediate' from dual union all
select 17921,'017921','1987-004DN','deb Kosmos-1813','',60094,'Reentered','1987-07-19 00:00:00','1987-04-29 00:00:00',93.08,315,536,72.86,'Intermediate' from dual union all
select 17922,'017922','1987-004DP','deb Kosmos-1813','',60094,'Reentered','1987-09-01 00:00:00','1987-04-29 00:00:00',94.3,308,662,72.78,'Intermediate' from dual union all
select 17923,'017923','1987-004DQ','deb Kosmos-1813','',60094,'Reentered','1987-08-25 00:00:00','1987-04-29 00:00:00',92.59,362,441,72.8,'Intermediate' from dual union all
select 17924,'017924','1987-004DR','deb Kosmos-1813','',60094,'Reentered','1988-10-14 00:00:00','1987-04-29 00:00:00',96.38,291,880,72.81,'Intermediate' from dual union all
select 17925,'017925','1987-004DS','deb Kosmos-1813','',60094,'Reentered','1989-06-19 00:00:00','1987-04-29 00:00:00',96.74,371,835,72.82,'Intermediate' from dual union all
select 17926,'017926','1987-004DT','deb Kosmos-1813','',60094,'Reentered','1987-06-21 00:00:00','1987-04-29 00:00:00',92.8,355,469,72.81,'Intermediate' from dual union all
select 17927,'017927','1987-004DU','deb Kosmos-1813','',60094,'Reentered','1987-09-22 00:00:00','1987-04-29 00:00:00',92.64,368,439,72.81,'Intermediate' from dual union all
select 17928,'017928','1987-004DV','deb Kosmos-1813','',60094,'Reentered','1987-06-11 00:00:00','1987-04-29 00:00:00',91.68,342,372,72.81,'Intermediate' from dual union all
select 17929,'017929','1987-004DW','deb Kosmos-1813','',60094,'Reentered','1987-10-10 00:00:00','1987-04-29 00:00:00',92.38,355,428,72.8,'Intermediate' from dual union all
select 1793,'001793','1965-088Q','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-07 00:00:00','1965-11-05 00:00:00',91.94,210,530,48.4,'Intermediate' from dual union all
select 17930,'017930','1987-004DX','deb Kosmos-1813','',60094,'Reentered','1988-01-10 00:00:00','1987-04-29 00:00:00',92.87,300,530,72.82,'Intermediate' from dual union all
select 17931,'017931','1987-004DY','deb Kosmos-1813','',60094,'Reentered','1988-09-13 00:00:00','1987-04-29 00:00:00',92.26,357,414,72.81,'Intermediate' from dual union all
select 17932,'017932','1987-004DZ','deb Kosmos-1813','',60094,'Reentered','1988-02-27 00:00:00','1987-04-29 00:00:00',92.72,375,441,72.82,'Intermediate' from dual union all
select 17933,'017933','1987-004EA','deb Kosmos-1813','',60094,'Reentered','1987-09-07 00:00:00','1987-04-28 00:00:00',93,323,520,72.81,'Intermediate' from dual union all
select 17934,'017934','1987-004EB','deb Kosmos-1813','',60094,'Reentered','1987-09-06 00:00:00','1987-04-29 00:00:00',92.63,353,454,72.8,'Intermediate' from dual union all
select 17935,'017935','1987-004EC','deb Kosmos-1813','',60094,'Reentered','1987-06-27 00:00:00','1987-04-29 00:00:00',92,354,392,72.81,'Intermediate' from dual union all
select 17936,'017936','1987-004ED','deb Kosmos-1813','',60094,'Reentered','1987-11-28 00:00:00','1987-04-29 00:00:00',92.38,360,423,72.81,'Intermediate' from dual union all
select 17937,'017937','1987-004EE','deb Kosmos-1813','',60094,'Reentered','1987-10-06 00:00:00','1987-04-29 00:00:00',92.45,353,437,72.81,'Intermediate' from dual union all
select 17938,'017938','1987-004EF','deb Kosmos-1813','',60094,'Reentered','1987-11-29 00:00:00','1987-04-29 00:00:00',92.33,346,432,72.81,'Intermediate' from dual union all
select 17939,'017939','1979-087C','Blok DM','11S86 No. 23L',47571,'In Earth orbit','','1983-01-13 00:00:00',1433.05,35523,35931,2.16,'Drift GEO' from dual union all
select 1794,'001794','1965-088R','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-28 00:00:00','1965-11-05 00:00:00',93.23,199,667,48.4,'Intermediate' from dual union all
select 17940,'017940','1987-039A','Kosmos-1843','Oblik',60129,'Landed','1987-05-19 00:00:00','1987-05-07 00:00:00',92.17,347,415,70.37,'Intermediate' from dual union all
select 17941,'017941','1987-039B','11S510','11S510',60129,'Reentered','1987-05-13 00:00:00','1987-05-07 00:00:00',89.18,202,266,70.37,'Intermediate' from dual union all
select 17942,'017942','1986-019MH','deb Ariane V16','',59212,'Reentered','1989-04-13 00:00:00','1987-05-06 00:00:00',100.57,665,907,98.57,'Sun-Synch' from dual union all
select 17943,'017943','1986-019MJ','deb Ariane V16','',59212,'Reentered','1989-04-23 00:00:00','1987-05-06 00:00:00',100.8,771,822,98.34,'Sun-Synch' from dual union all
select 17944,'017944','1986-019MK','deb Ariane V16','',59212,'Reentered','1989-09-11 00:00:00','1987-05-06 00:00:00',100.88,760,841,98.29,'Sun-Synch' from dual union all
select 17945,'017945','1986-019ML','deb Ariane V16','',59212,'Reentered','1989-03-17 00:00:00','1987-05-06 00:00:00',100.08,727,798,98.83,'Sun-Synch' from dual union all
select 17946,'017946','1986-019MM','deb Ariane V16','',59212,'Reentered','1987-12-07 00:00:00','1987-05-06 00:00:00',98.18,589,755,98.53,'Sun-Synch' from dual union all
select 17947,'017947','1986-019MN','deb Ariane V16','',59212,'Reentered','1988-11-06 00:00:00','1987-05-06 00:00:00',99.11,632,801,99.18,'Sun-Synch' from dual union all
select 17948,'017948','1986-019MP','deb Ariane V16','',59212,'Reentered','1988-04-29 00:00:00','1987-05-06 00:00:00',99.52,696,776,98.88,'Sun-Synch' from dual union all
select 17949,'017949','1986-019MQ','deb Ariane V16','',59212,'Reentered','1988-08-14 00:00:00','1987-05-06 00:00:00',99.32,672,781,98.82,'Sun-Synch' from dual union all
select 1795,'001795','1965-088S','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-08 00:00:00','1965-11-05 00:00:00',93.01,205,639,48.4,'Intermediate' from dual union all
select 17950,'017950','1986-019MR','deb Ariane V16','',59212,'Reentered','1990-01-29 00:00:00','1987-05-06 00:00:00',101.34,761,884,98.17,'Sun-Synch' from dual union all
select 17951,'017951','1986-019MS','deb Ariane V16','',59212,'Reentered','1988-12-11 00:00:00','1987-05-06 00:00:00',99.03,638,787,99.05,'Sun-Synch' from dual union all
select 17952,'017952','1986-019MT','deb Ariane V16','',59212,'Reentered','1989-01-07 00:00:00','1987-05-06 00:00:00',100.72,784,802,98.63,'Sun-Synch' from dual union all
select 17953,'017953','1986-019MU','deb Ariane V16','',59212,'Reentered','1989-01-05 00:00:00','1987-05-06 00:00:00',99.37,650,807,99.03,'Sun-Synch' from dual union all
select 17954,'017954','1986-019MV','deb Ariane V16','',59212,'Reentered','1988-05-04 00:00:00','1987-05-06 00:00:00',100.08,728,796,98.93,'Sun-Synch' from dual union all
select 17955,'017955','1986-019MW','deb Ariane V16','',59212,'Reentered','1988-10-15 00:00:00','1987-05-06 00:00:00',100.49,768,796,98.17,'Sun-Synch' from dual union all
select 17956,'017956','1986-019MX','deb Ariane V16','',59212,'Reentered','1989-11-11 00:00:00','1987-05-06 00:00:00',101.27,788,850,98.54,'Sun-Synch' from dual union all
select 17957,'017957','1986-019MY','deb Ariane V16','',59212,'Reentered','1989-11-02 00:00:00','1987-05-06 00:00:00',100.65,770,808,98.76,'Sun-Synch' from dual union all
select 17958,'017958','1986-019MZ','deb Ariane V16','',59212,'Reentered','1989-08-27 00:00:00','1987-05-06 00:00:00',101.11,781,842,98.65,'Sun-Synch' from dual union all
select 17959,'017959','1986-019NA','deb Ariane V16','',59212,'Reentered','1989-08-24 00:00:00','1987-05-06 00:00:00',100.92,770,834,98.55,'Sun-Synch' from dual union all
select 1796,'001796','1965-088T','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-15 00:00:00','1965-11-05 00:00:00',90.63,209,402,48.4,'Intermediate' from dual union all
select 17960,'017960','1986-019NB','deb Ariane V16','',59212,'Reentered','1989-08-21 00:00:00','1987-05-06 00:00:00',101.19,785,845,98.59,'Sun-Synch' from dual union all
select 17961,'017961','1986-019NC','deb Ariane V16','',59212,'Reentered','1989-01-03 00:00:00','1987-05-06 00:00:00',101.08,789,831,98.63,'Sun-Synch' from dual union all
select 17962,'017962','1987-036J','deb Blok DM-2','Large debris (RCS 4.1)',60126,'Reentered','1987-10-28 00:00:00','1987-06-14 00:00:00',215.79,207,10871,64.84,'Medium' from dual union all
select 17963,'017963','1987-039C','Sensor cover','Sensor cover',60129,'Reentered','1987-05-07 00:00:00','1987-05-06 00:00:00',88.69,180,240,70.39,'Intermediate' from dual union all
select 17964,'017964','1987-037C','KBE part','KBE part',60127,'Reentered','1987-05-09 00:00:00','1987-05-08 00:00:00',90.72,202,418,62.94,'Intermediate' from dual union all
select 17965,'017965','1987-037D','KB electropitaniya','KBE',60127,'Reentered','1987-07-04 00:00:00','1987-05-24 00:00:00',89.99,212,336,62.83,'Intermediate' from dual union all
select 17966,'017966','1987-037E','KBE part','KBE part',60127,'Reentered','1987-05-21 00:00:00','1987-05-08 00:00:00',90.19,214,354,62.84,'Intermediate' from dual union all
select 17967,'017967','1987-037F','PTDU cover','KBE part',60127,'Reentered','1987-05-24 00:00:00','1987-05-09 00:00:00',92.38,322,461,62.85,'Intermediate' from dual union all
select 17968,'017968','1987-037G','PTDU cover','KBE part',60127,'Reentered','1987-06-11 00:00:00','1987-05-09 00:00:00',92.12,314,443,62.84,'Intermediate' from dual union all
select 17969,'017969','1987-040A','Gorizont','Gorizont No. 23L',60130,'In Earth orbit','','1987-06-13 00:00:00',1436.19,35781,35796,.91,'Stationary' from dual union all
select 1797,'001797','1965-088U','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-11 00:00:00','1965-11-05 00:00:00',90.86,207,427,48.4,'Intermediate' from dual union all
select 17970,'017970','1987-040B','8S812','8S812',60130,'Reentered','1987-05-14 00:00:00','1987-05-13 00:00:00',87.88,165,175,51.62,'Intermediate' from dual union all
select 17971,'017971','1987-040C','Perekhodnik','Sredniy perekhodnik',60130,'Reentered','1987-05-12 00:00:00','1987-05-11 00:00:00',88.49,191,208,51.61,'Intermediate' from dual union all
select 17972,'017972','1987-040D','Blok DM No. 61L','11S86 No. 61L',60130,'In Earth orbit','','1987-06-10 00:00:00',1397.81,34970,35098,.92,'Drift GEO' from dual union all
select 17973,'017973','1987-041A','Kosmos-1844','Tselina-2',60131,'In Earth orbit','','1987-06-13 00:00:00',101.94,848,852,71.01,'Intermediate' from dual union all
select 17974,'017974','1987-041B','11S772','11S772',60131,'In Earth orbit','','1987-10-01 00:00:00',101.73,830,852,71.01,'Intermediate' from dual union all
select 17975,'017975','1987-042A','Kosmos-1845','Oblik',60132,'Landed','1987-05-27 00:00:00','1987-05-15 00:00:00',92.17,347,415,70.39,'Intermediate' from dual union all
select 17976,'017976','1987-042B','11S510','11S510',60132,'Reentered','1987-06-01 00:00:00','1987-05-20 00:00:00',89.78,200,328,70.39,'Intermediate' from dual union all
select 17977,'017977','1986-019ND','deb Ariane V16','',59212,'Reentered','1989-01-04 00:00:00','1987-05-14 00:00:00',100.9,784,819,98.56,'Sun-Synch' from dual union all
select 17978,'017978','1986-019NE','deb Ariane V16','',59212,'Reentered','1989-09-09 00:00:00','',100.04,727,794,98.71,'Sun-Synch' from dual union all
select 17979,'017979','1986-019NF','deb Ariane V16','',59212,'Reentered','1988-02-03 00:00:00','1987-05-14 00:00:00',97.49,530,748,99.17,'Sun-Synch' from dual union all
select 1798,'001798','1965-088V','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-28 00:00:00','1965-11-05 00:00:00',91.15,196,466,48.4,'Intermediate' from dual union all
select 17980,'017980','1986-019NG','deb Ariane V16','',59212,'Reentered','1989-03-17 00:00:00','1987-05-15 00:00:00',97.49,530,748,99.17,'Sun-Synch' from dual union all
select 17981,'017981','1986-019NH','deb Ariane V16','',59212,'Reentered','2003-02-10 00:00:00','1993-08-03 00:00:00',101.29,742,898,98.77,'Sun-Synch' from dual union all
select 17982,'017982','1986-019NJ','deb Ariane V16','',59212,'Reentered','1989-03-16 00:00:00','',100.4,759,796,98.48,'Sun-Synch' from dual union all
select 17983,'017983','1986-019NK','deb Ariane V16','',59212,'Reentered','1991-05-12 00:00:00','',103.67,787,1078,98.01,'Sun-Synch' from dual union all
select 17984,'017984','1986-019NL','deb Ariane V16','',59212,'Reentered','1989-04-21 00:00:00','',100.8,789,804,98.68,'Sun-Synch' from dual union all
select 17985,'017985','1986-019NM','deb Ariane V16','',59212,'Reentered','1987-10-11 00:00:00','1987-05-15 00:00:00',97.79,541,764,99.58,'Sun-Synch' from dual union all
select 17986,'017986','1986-019NN','deb Ariane V16','',59212,'Reentered','1988-11-02 00:00:00','1987-05-14 00:00:00',100.77,772,819,98.84,'Sun-Synch' from dual union all
select 17987,'017987','1986-019NP','deb Ariane V16','',59212,'Reentered','1988-11-02 00:00:00','',100.78,755,837,98.76,'Sun-Synch' from dual union all
select 17988,'017988','1986-019NQ','deb Ariane V16','',59212,'Reentered','2001-04-08 00:00:00','1993-08-07 00:00:00',99.63,710,772,98.51,'Sun-Synch' from dual union all
select 17989,'017989','1986-019NR','deb Ariane V16','',59212,'Reentered','1989-03-22 00:00:00','',101.13,787,838,98.64,'Sun-Synch' from dual union all
select 1799,'001799','1965-088W','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-28 00:00:00','1965-11-05 00:00:00',90.93,215,426,48.4,'Intermediate' from dual union all
select 17990,'017990','1986-019NS','deb Ariane V16','',59212,'Reentered','1988-10-10 00:00:00','',98.71,602,791,99.03,'Sun-Synch' from dual union all
select 17991,'017991','1986-019NT','deb Ariane V16','',59212,'Reentered','1988-11-30 00:00:00','1987-05-15 00:00:00',100.89,764,838,98.61,'Sun-Synch' from dual union all
select 17992,'017992','1986-019NU','deb Ariane V16','',59212,'Reentered','1989-07-03 00:00:00','',101.02,794,819,98.59,'Sun-Synch' from dual union all
select 17993,'017993','1986-019NV','deb Ariane V16','',59212,'Reentered','1989-06-03 00:00:00','',100.93,803,803,98.67,'Sun-Synch' from dual union all
select 17994,'017994','1986-019NW','deb Ariane V16','',59212,'Reentered','1989-01-10 00:00:00','1987-05-15 00:00:00',101.05,761,855,98.71,'Sun-Synch' from dual union all
select 17995,'017995','1986-019NX','deb Ariane V16','',59212,'Reentered','1989-10-20 00:00:00','',101.68,804,873,98.72,'Sun-Synch' from dual union all
select 17996,'017996','1986-019NY','deb Ariane V16','',59212,'Reentered','1990-03-26 00:00:00','1987-05-15 00:00:00',102.5,780,974,98.26,'Sun-Synch' from dual union all
select 17997,'017997','1987-043A','USA-22','PARCAE 9',60133,'In Earth orbit','','1992-09-01 00:00:00',107.41,955,1259,63.42,'Intermediate' from dual union all
select 17998,'017998','1987-043B','LIPS 3','LIPS 3',60133,'In Earth orbit','','1997-11-22 00:00:00',107.51,916,1307,63.41,'Intermediate' from dual union all
select 17999,'017999','1987-044A','Progress-30','Progress 7K-TG No. 128',60134,'Deorbited','1987-07-19 00:00:00','1987-06-19 00:00:00',91.6,341,366,51.63,'Intermediate' from dual union all
select 18,'000018','59 Epsilon 1','Discoverer 5','CORONA C-2',5528,'Reentered Attached','1959-09-28 00:00:00','1959-08-14 00:00:00',94.07,215,732,80,'Intermediate' from dual union all
select 180,'000180','61 Chi','Explorer 13','NASA S-55A',7262,'Reentered Attached','1961-08-28 00:00:00','1961-08-26 00:00:00',97.61,125,1164,37.7,'Intermediate' from dual union all
select 1800,'001800','1965-088X','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-28 00:00:00','1965-11-05 00:00:00',93.59,217,684,48.4,'Intermediate' from dual union all
select 18000,'018000','1987-044B','11S510','11S510',60134,'Reentered','1987-05-22 00:00:00','1987-05-21 00:00:00',88.16,169,198,51.62,'Intermediate' from dual union all
select 18001,'018001','1987-039D','Blenda','Sunshade',60129,'Reentered','1987-08-28 00:00:00','1987-06-08 00:00:00',91.95,340,401,70.37,'Intermediate' from dual union all
select 18002,'018002','1987-039E','KDU','KDU',60129,'Reentered','1988-07-11 00:00:00','1987-06-04 00:00:00',92.12,346,410,70.37,'Intermediate' from dual union all
select 18003,'018003','1987-039F','KDU part','KDU part',60129,'Reentered','1987-11-06 00:00:00','1987-06-04 00:00:00',91.91,331,405,70.37,'Intermediate' from dual union all
select 18004,'018004','1987-045A','Kosmos-1846','Resurs-F1 14F40 No. 104',60135,'Landed','1987-06-04 00:00:00','1987-05-23 00:00:00',91.18,323,342,82.35,'Intermediate' from dual union all
select 18005,'018005','1987-045B','11S510','11S510',60135,'Reentered','1987-05-26 00:00:00','1987-05-23 00:00:00',88.81,177,255,82.35,'Intermediate' from dual union all
select 18006,'018006','1987-045C','Sensor cover','Sensor cover',60135,'Reentered','1987-05-22 00:00:00','1987-05-22 00:00:00',88.27,163,215,82.34,'Intermediate' from dual union all
select 18007,'018007','1987-043C','USA-23 cover','SSU cover',60133,'In Earth orbit','','1997-11-22 00:00:00',107.42,899,1316,63.44,'Intermediate' from dual;
insert into SATELLITE
select 18008,'018008','1987-043D','USA-24 cover','SSU cover',60133,'In Earth orbit','','1997-11-22 00:00:00',107.43,899,1317,63.44,'Intermediate' from dual union all
select 18009,'018009','1987-043E','USA-23','SSU',60133,'In Earth orbit','','1990-09-15 00:00:00',107.44,1076,1140,63.44,'Intermediate' from dual union all
select 1801,'001801','1965-088Y','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-26 00:00:00','1965-11-05 00:00:00',93.2,214,649,48.4,'Intermediate' from dual union all
select 18010,'018010','1987-043F','USA-24','SSU',60133,'In Earth orbit','','1993-11-18 00:00:00',107.43,1077,1139,63.43,'Intermediate' from dual union all
select 18011,'018011','1987-046A','Kosmos-1847','Kobal''t No. 510',60136,'Landed','1987-07-22 00:00:00','1987-06-03 00:00:00',89.71,173,348,67.15,'Intermediate' from dual union all
select 18012,'018012','1987-046B','11S510','11S510',60136,'Reentered','1987-05-30 00:00:00','1987-05-28 00:00:00',89.06,160,296,67.15,'Intermediate' from dual union all
select 18013,'018013','1987-042C','Blenda','Sunshade',60132,'Reentered','1987-10-25 00:00:00','1987-06-13 00:00:00',91.89,331,404,70.39,'Intermediate' from dual union all
select 18014,'018014','1987-042D','KDU','KDU',60132,'Reentered','1988-08-16 00:00:00','1987-06-12 00:00:00',92.13,347,411,70.39,'Intermediate' from dual union all
select 18015,'018015','1987-042E','KDU part','KDU part',60132,'Reentered','1987-11-24 00:00:00','1987-06-12 00:00:00',94.07,375,572,70.39,'Intermediate' from dual union all
select 18016,'018016','1987-042F','KDU part','KDU part',60132,'Reentered','1987-06-22 00:00:00','1987-05-27 00:00:00',92.06,344,407,70.34,'Intermediate' from dual union all
select 18017,'018017','1987-047A','Kosmos-1848','Oblik No. 364',60137,'Landed','1987-06-11 00:00:00','1987-05-30 00:00:00',92.25,356,413,72.85,'Intermediate' from dual union all
select 18018,'018018','1987-047B','11S510','11S510',60137,'Reentered','1987-06-12 00:00:00','1987-06-04 00:00:00',89.5,188,312,72.84,'Intermediate' from dual union all
select 18019,'018019','1987-042G','KDU part','KDU part',60132,'Reentered','1987-12-04 00:00:00','1987-06-12 00:00:00',93.48,373,516,70.39,'Intermediate' from dual union all
select 1802,'001802','1965-088Z','deb Blok Kosmos R-12','',11838,'Reentered','1965-11-28 00:00:00','1965-11-05 00:00:00',91.38,203,482,48.4,'Intermediate' from dual union all
select 18020,'018020','1987-042H','KDU part','KDU part',60132,'Reentered','1987-09-14 00:00:00','1987-06-12 00:00:00',94.23,376,587,70.38,'Intermediate' from dual union all
select 18021,'018021','1987-047C','Sensor cover','Sensor cover',60137,'Reentered','1987-05-30 00:00:00','1987-05-29 00:00:00',89.11,181,280,72.85,'Intermediate' from dual union all
select 18022,'018022','1987-039G','PTDU cover','',60129,'Reentered','1987-11-22 00:00:00','1987-06-04 00:00:00',94.1,378,572,70.37,'Intermediate' from dual union all
select 18023,'018023','1987-042J','KDU part','KDU part',60132,'Reentered','1987-09-20 00:00:00','1987-06-12 00:00:00',92.02,342,405,70.38,'Intermediate' from dual union all
select 18024,'018024','1987-043G','USA-25 cover','SSU cover',60133,'In Earth orbit','','1997-11-22 00:00:00',107.43,908,1307,63.44,'Intermediate' from dual union all
select 18025,'018025','1987-043H','USA-25','SSU',60133,'In Earth orbit','','1993-12-04 00:00:00',107.43,1031,1185,63.42,'Intermediate' from dual union all
select 18026,'018026','1986-067S','Kosmos-1776 SS 11','ESO',59260,'Reentered','1988-01-13 00:00:00','1987-06-05 00:00:00',94.41,468,512,74.01,'Intermediate' from dual union all
select 18027,'018027','1986-067T','Kosmos-1776 SS 12','ESO',59260,'Reentered','1988-02-07 00:00:00','1987-06-03 00:00:00',94.5,477,512,74.04,'Intermediate' from dual union all
select 18028,'018028','1987-033C','Kosmos-1836 part','Terilen part',60123,'Reentered','1987-06-20 00:00:00','1987-06-03 00:00:00',89.58,230,278,64.8,'Intermediate' from dual union all
select 18029,'018029','1987-004EG','deb Kosmos-1813 part','Oblik debris',60094,'Reentered','1987-06-18 00:00:00','1987-06-03 00:00:00',89.54,229,275,72.8,'Intermediate' from dual union all
select 1803,'001803','1965-097C','deb Kosmos-98','',11847,'Reentered','1965-12-08 00:00:00','1965-11-27 00:00:00',92.09,205,549,65,'Intermediate' from dual union all
select 18030,'018030','1987-004EH','deb Kosmos-1813','',60094,'Reentered','1987-06-13 00:00:00','1987-06-03 00:00:00',89.33,221,262,72.79,'Intermediate' from dual union all
select 18031,'018031','1987-004EJ','deb Kosmos-1813','',60094,'Reentered','1987-09-22 00:00:00','1987-06-03 00:00:00',91.54,324,376,72.81,'Intermediate' from dual union all
select 18032,'018032','1987-004EK','deb Kosmos-1813','',60094,'Reentered','1987-09-29 00:00:00','1987-06-03 00:00:00',99.71,386,1103,72.76,'Intermediate' from dual union all
select 18033,'018033','1987-004EL','deb Kosmos-1813','',60094,'Reentered','1987-10-31 00:00:00','1987-06-03 00:00:00',91.35,313,368,72.81,'Intermediate' from dual union all
select 18034,'018034','1987-004EM','deb Kosmos-1813','',60094,'Reentered','1987-09-30 00:00:00','1987-06-03 00:00:00',91.56,329,374,72.81,'Intermediate' from dual union all
select 18035,'018035','1987-004EN','deb Kosmos-1813','',60094,'Reentered','1987-07-12 00:00:00','1987-06-03 00:00:00',90.54,280,321,72.8,'Intermediate' from dual union all
select 18036,'018036','1987-004EP','deb Kosmos-1813','',60094,'Reentered','1988-10-10 00:00:00','1987-06-03 00:00:00',92.49,354,440,72.81,'Intermediate' from dual union all
select 18037,'018037','1987-004EQ','deb Kosmos-1813','',60094,'Reentered','1988-01-09 00:00:00','1987-06-03 00:00:00',92.59,346,457,72.81,'Intermediate' from dual union all
select 18038,'018038','1987-004ER','deb Kosmos-1813','',60094,'Reentered','1987-10-29 00:00:00','1987-06-03 00:00:00',91.5,303,394,72.81,'Intermediate' from dual union all
select 18039,'018039','1987-004ES','deb Kosmos-1813','',60094,'Reentered','1989-01-22 00:00:00','1987-06-03 00:00:00',97.47,330,946,72.83,'Intermediate' from dual union all
select 1804,'001804','1965-098A','Alouette 2','Alouette 2',11848,'In Earth orbit','','1966-01-27 00:00:00',121.41,502,2986,79.85,'Medium' from dual union all
select 18040,'018040','1987-004ET','deb Kosmos-1813','',60094,'Reentered','1987-09-15 00:00:00','1987-06-03 00:00:00',92.43,360,428,72.8,'Intermediate' from dual union all
select 18041,'018041','1987-004EU','deb Kosmos-1813','',60094,'Reentered','1987-09-02 00:00:00','1987-06-03 00:00:00',94.14,357,597,72.8,'Intermediate' from dual union all
select 18042,'018042','1987-004EV','deb Kosmos-1813','',60094,'Reentered','1988-11-01 00:00:00','1987-06-03 00:00:00',97.61,384,905,72.78,'Intermediate' from dual union all
select 18043,'018043','1987-004EW','deb Kosmos-1813','',60094,'Reentered','1987-08-06 00:00:00','1987-06-03 00:00:00',91.16,316,347,72.82,'Intermediate' from dual union all
select 18044,'018044','1987-004EX','deb Kosmos-1813','',60094,'Reentered','1988-03-31 00:00:00','1987-06-03 00:00:00',94.48,300,686,72.83,'Intermediate' from dual union all
select 18045,'018045','1987-004EY','deb Kosmos-1813','',60094,'Reentered','1988-07-31 00:00:00','1987-06-03 00:00:00',97.03,371,862,72.81,'Intermediate' from dual union all
select 18046,'018046','1987-004EZ','deb Kosmos-1813','',60094,'Reentered','1989-03-15 00:00:00','1987-06-03 00:00:00',95.09,388,657,72.78,'Intermediate' from dual union all
select 18047,'018047','1987-004FA','deb Kosmos-1813','',60094,'Reentered','1987-10-23 00:00:00','1987-06-03 00:00:00',94.22,365,597,72.78,'Intermediate' from dual union all
select 18048,'018048','1987-004FB','deb Kosmos-1813','',60094,'Reentered','1987-08-17 00:00:00','1987-06-03 00:00:00',90.18,243,324,72.81,'Intermediate' from dual union all
select 18049,'018049','1987-004FC','deb Kosmos-1813','',60094,'Reentered','1988-10-23 00:00:00','1987-06-03 00:00:00',95.26,309,754,72.86,'Intermediate' from dual union all
select 1805,'001805','1965-096B','P064','P064',11846,'In Earth orbit','','1966-01-11 00:00:00',108.75,529,1808,34.25,'Intermediate' from dual union all
select 18050,'018050','1987-004FD','deb Kosmos-1813','',60094,'Reentered','1988-10-18 00:00:00','1987-06-03 00:00:00',92.68,374,438,72.82,'Intermediate' from dual union all
select 18051,'018051','1987-004FE','deb Kosmos-1813','',60094,'Reentered','1987-07-24 00:00:00','1987-06-03 00:00:00',90.68,284,331,72.8,'Intermediate' from dual union all
select 18052,'018052','1987-004FF','deb Kosmos-1813','',60094,'Reentered','1987-07-17 00:00:00','1987-06-03 00:00:00',90.27,252,324,72.8,'Intermediate' from dual union all
select 18053,'018053','1987-004FG','deb Kosmos-1813','',60094,'Reentered','1987-06-29 00:00:00','1987-06-03 00:00:00',90.19,243,325,72.82,'Intermediate' from dual union all
select 18054,'018054','1987-004FH','deb Kosmos-1813','',60094,'Reentered','1987-10-24 00:00:00','1987-06-03 00:00:00',91.48,331,363,72.81,'Intermediate' from dual union all
select 18055,'018055','1987-004FJ','deb Kosmos-1813','',60094,'Reentered','1987-08-09 00:00:00','1987-06-03 00:00:00',91.21,325,343,72.8,'Intermediate' from dual union all
select 18056,'018056','1987-004FK','deb Kosmos-1813','',60094,'Reentered','1987-10-02 00:00:00','1987-06-03 00:00:00',91.37,336,348,72.81,'Intermediate' from dual union all
select 18057,'018057','1987-004FL','deb Kosmos-1813','',60094,'Reentered','1988-09-24 00:00:00','1987-06-03 00:00:00',92.33,349,429,72.81,'Intermediate' from dual union all
select 18058,'018058','1987-004FM','deb Kosmos-1813','',60094,'Reentered','1988-09-19 00:00:00','1987-06-03 00:00:00',92.43,362,426,72.81,'Intermediate' from dual union all
select 18059,'018059','1987-004FN','deb Kosmos-1813','',60094,'Reentered','1987-09-03 00:00:00','1987-06-03 00:00:00',91.18,320,345,72.81,'Intermediate' from dual union all
select 1806,'001806','1965-098B','Explorer 31','DME-A',11848,'In Earth orbit','','1966-01-21 00:00:00',121.41,505,2983,79.82,'Medium' from dual union all
select 18060,'018060','1987-004FP','deb Kosmos-1813','',60094,'Reentered','1987-07-23 00:00:00','1987-06-03 00:00:00',90.69,289,328,72.8,'Intermediate' from dual union all
select 18061,'018061','1987-004FQ','deb Kosmos-1813','',60094,'Reentered','1987-07-23 00:00:00','1987-06-03 00:00:00',90.63,280,331,72.8,'Intermediate' from dual union all
select 18062,'018062','1987-004FR','deb Kosmos-1813','',60094,'Reentered','1987-10-20 00:00:00','1987-06-03 00:00:00',90.97,282,363,72.81,'Intermediate' from dual union all
select 18063,'018063','1987-004FS','deb Kosmos-1813','',60094,'Reentered','1987-09-09 00:00:00','1987-06-03 00:00:00',91.39,309,377,72.8,'Intermediate' from dual union all
select 18064,'018064','1987-004FT','deb Kosmos-1813','',60094,'Reentered','1987-09-13 00:00:00','1987-06-03 00:00:00',92.22,333,434,72.8,'Intermediate' from dual union all
select 18065,'018065','1987-004FU','deb Kosmos-1813','',60094,'Reentered','1987-09-11 00:00:00','1987-06-03 00:00:00',90.83,289,342,72.81,'Intermediate' from dual union all
select 18066,'018066','1987-004FV','deb Kosmos-1813','',60094,'Reentered','1987-08-04 00:00:00','1987-06-03 00:00:00',91.35,315,367,72.8,'Intermediate' from dual union all
select 18067,'018067','1987-004FW','deb Kosmos-1813','',60094,'Reentered','1988-03-27 00:00:00','1987-06-03 00:00:00',92.82,339,487,72.8,'Intermediate' from dual union all
select 18068,'018068','1987-004FX','deb Kosmos-1813','',60094,'Reentered','1988-02-25 00:00:00','1987-06-03 00:00:00',92.65,286,523,72.81,'Intermediate' from dual union all
select 18069,'018069','1987-004FY','deb Kosmos-1813','',60094,'Reentered','1987-08-29 00:00:00','1987-06-03 00:00:00',92.7,337,477,72.79,'Intermediate' from dual union all
select 1807,'001807','1965-098C','Agena B 6102','Agena B 6102',11848,'In Earth orbit','','1966-01-25 00:00:00',121.34,502,2980,79.83,'Medium' from dual union all
select 18070,'018070','1987-004FZ','deb Kosmos-1813','',60094,'Reentered','1988-02-12 00:00:00','1987-06-03 00:00:00',92.45,323,467,72.8,'Intermediate' from dual union all
select 18071,'018071','1987-004GA','deb Kosmos-1813','',60094,'Reentered','1988-03-10 00:00:00','1987-06-03 00:00:00',91.65,324,387,72.81,'Intermediate' from dual union all
select 18072,'018072','1987-004GB','deb Kosmos-1813','',60094,'Reentered','1987-09-06 00:00:00','1987-06-03 00:00:00',91.63,301,408,72.8,'Intermediate' from dual union all
select 18073,'018073','1987-004GC','deb Kosmos-1813','',60094,'Reentered','1987-09-19 00:00:00','1987-06-03 00:00:00',91.52,297,402,72.79,'Intermediate' from dual union all
select 18074,'018074','1987-004GD','deb Kosmos-1813','',60094,'Reentered','1987-10-03 00:00:00','1987-06-03 00:00:00',91.78,351,372,72.8,'Intermediate' from dual union all
select 18075,'018075','1987-004GE','deb Kosmos-1813','',60094,'Reentered','1987-08-12 00:00:00','1987-06-03 00:00:00',90.93,306,335,72.8,'Intermediate' from dual union all
select 18076,'018076','1987-004GF','deb Kosmos-1813','',60094,'Reentered','1988-04-27 00:00:00','1987-06-03 00:00:00',91.71,326,391,72.81,'Intermediate' from dual union all
select 18077,'018077','1987-004GG','deb Kosmos-1813','',60094,'Reentered','1987-09-19 00:00:00','1987-06-03 00:00:00',91.53,311,389,72.81,'Intermediate' from dual union all
select 18078,'018078','1987-004GH','deb Kosmos-1813','',60094,'Reentered','1988-02-16 00:00:00','1987-06-03 00:00:00',92.2,333,432,72.8,'Intermediate' from dual union all
select 18079,'018079','1987-004GJ','deb Kosmos-1813','',60094,'Reentered','1987-07-19 00:00:00','1987-06-03 00:00:00',92.44,376,413,72.8,'Intermediate' from dual union all
select 1808,'001808','1965-098D','DME-A adapter petal','Adapter petal',11848,'Reentered','2002-09-23 00:00:00','1966-01-31 00:00:00',121.33,499,2981,79.88,'Medium' from dual union all
select 18080,'018080','1987-004GK','deb Kosmos-1813','',60094,'Reentered','1987-06-28 00:00:00','1987-06-03 00:00:00',89.83,236,297,72.8,'Intermediate' from dual union all
select 18081,'018081','1987-004GL','deb Kosmos-1813','',60094,'Reentered','1987-09-03 00:00:00','1987-06-03 00:00:00',91.35,319,363,72.81,'Intermediate' from dual union all
select 18082,'018082','1987-004GM','deb Kosmos-1813','',60094,'Reentered','1988-01-15 00:00:00','1987-06-03 00:00:00',92.39,353,431,72.81,'Intermediate' from dual union all
select 18083,'018083','1987-048A','Kosmos-1849','Oko',60138,'Reentered','2003-02-03 00:00:00','1987-07-04 00:00:00',717.88,645,39715,62.91,'Molniya' from dual union all
select 18084,'018084','1987-048B','11S510','11S510',60138,'Reentered','1987-06-30 00:00:00','1987-06-11 00:00:00',91.74,176,544,62.86,'Intermediate' from dual union all
select 18085,'018085','1987-048C','BOZ','BOZ',60138,'Reentered','1987-08-15 00:00:00','1987-07-04 00:00:00',91.65,210,501,62.82,'Intermediate' from dual union all
select 18086,'018086','1987-048D','Blok-2BL','Blok-2BL No. 76728-674',60138,'Reentered','2002-09-19 00:00:00','1987-07-04 00:00:00',706.2,642,39140,62.89,'Molniya' from dual union all
select 18087,'018087','1987-045D','Blenda','Sunshade',60135,'Reentered','1987-06-11 00:00:00','1987-06-04 00:00:00',94.81,334,685,82.39,'Intermediate' from dual union all
select 18088,'018088','1987-045E','KDU part','KDU part',60135,'Reentered','1987-07-21 00:00:00','1987-06-19 00:00:00',93.45,328,559,82.31,'Intermediate' from dual union all
select 18089,'018089','1987-045F','KDU part','KDU part',60135,'Reentered','1987-09-12 00:00:00','1987-06-28 00:00:00',93.02,334,511,82.34,'Intermediate' from dual union all
select 1809,'001809','1965-099B','8K78I','8K78I',11849,'Reentered','1965-12-05 00:00:00','1965-12-03 00:00:00',88.55,185,221,51.87,'Intermediate' from dual union all
select 18090,'018090','1987-045G','KDU part','KDU part',60135,'Reentered','1987-06-14 00:00:00','1987-06-05 00:00:00',91.81,259,468,82.49,'Intermediate' from dual union all
select 18091,'018091','1987-045H','KDU part','KDU part',60135,'Reentered','1987-08-04 00:00:00','1987-06-26 00:00:00',90.66,294,320,82.34,'Intermediate' from dual union all
select 18092,'018092','1987-045J','KDU part','KDU part',60135,'Reentered','1987-07-15 00:00:00','1987-06-06 00:00:00',91.04,312,340,82.33,'Intermediate' from dual union all
select 18093,'018093','1987-045K','KDU part','KDU part',60135,'Reentered','1987-07-19 00:00:00','1987-06-04 00:00:00',91.16,321,342,82.35,'Intermediate' from dual union all
select 18094,'018094','1987-045L','KDU','KDU',60135,'Reentered','1988-01-05 00:00:00','1987-06-20 00:00:00',91.11,317,340,82.35,'Intermediate' from dual union all
select 18095,'018095','1987-049A','Kosmos-1850','Strela-2M',60139,'In Earth orbit','','1987-07-11 00:00:00',100.76,783,807,74.04,'Intermediate' from dual union all
select 18096,'018096','1987-049B','S3M','S3M',60139,'In Earth orbit','','1987-07-09 00:00:00',100.65,775,804,74.05,'Intermediate' from dual union all
select 18097,'018097','1987-047D','KDU part','KDU part',60137,'Reentered','1987-06-25 00:00:00','1987-06-11 00:00:00',92.4,355,430,72.86,'Intermediate' from dual union all
select 18098,'018098','1987-047E','KDU part','KDU part',60137,'Reentered','1987-11-05 00:00:00','1987-06-27 00:00:00',92.01,342,405,72.85,'Intermediate' from dual;
insert into SATELLITE
select 18099,'018099','1987-047F','KDU part','KDU part',60137,'Reentered','1987-08-13 00:00:00','1987-06-27 00:00:00',93.77,375,543,72.86,'Intermediate' from dual union all
select 181,'000181','61 Psi','Discoverer 29','CORONA C''''''-1',7276,'Reentered Attached','1961-09-09 00:00:00','1961-09-04 00:00:00',91.01,138,511,82.04,'Intermediate' from dual union all
select 1810,'001810','1965-099A','Luna-8','E-6 No. 12',11849,'Deep Space','1965-12-06 00:00:00','1965-12-03 00:00:00',88.55,185,221,51.87,'Intermediate' from dual union all
select 18100,'018100','1987-047G','Blenda','Sunshade',60137,'Reentered','1987-11-18 00:00:00','1987-06-27 00:00:00',94.19,387,571,72.85,'Intermediate' from dual union all
select 18101,'018101','1987-047H','KDU part','KDU part',60137,'Reentered','1987-09-19 00:00:00','1987-06-19 00:00:00',92.16,353,408,72.85,'Intermediate' from dual union all
select 18102,'018102','1987-047J','KDU','KDU',60137,'Reentered','1988-09-29 00:00:00','1987-06-28 00:00:00',92.23,357,411,72.85,'Intermediate' from dual union all
select 18103,'018103','1987-050A','Kosmos-1851','Oko',60140,'In Earth orbit','','1987-07-16 00:00:00',717.84,620,39738,62.87,'Molniya' from dual union all
select 18104,'018104','1987-050B','11S510','11S510',60140,'Reentered','1987-08-15 00:00:00','1987-07-12 00:00:00',91.04,213,439,62.78,'Intermediate' from dual union all
select 18105,'018105','1987-050C','BOZ','BOZ',60140,'Reentered','1987-07-22 00:00:00','1987-06-20 00:00:00',91.8,186,540,62.82,'Intermediate' from dual union all
select 18106,'018106','1987-050D','Blok-2BL','Blok-2BL No. 71658-672',60140,'In Earth orbit','','1987-07-12 00:00:00',707.33,618,39220,62.81,'Molniya' from dual union all
select 18107,'018107','1987-047K','KDU part','KDU part',60137,'Reentered','1987-07-18 00:00:00','1987-06-13 00:00:00',92.27,373,399,72.67,'Intermediate' from dual union all
select 18108,'018108','1987-047L','KDU part','KDU part',60137,'Reentered','1987-09-19 00:00:00','1987-06-28 00:00:00',91.44,328,363,72.84,'Intermediate' from dual union all
select 18109,'018109','1987-047M','KDU part','KDU part',60137,'Reentered','1987-09-20 00:00:00','1987-06-27 00:00:00',92.14,353,405,72.85,'Intermediate' from dual union all
select 1811,'001811','1965-099C','BOZ','BOZ',11849,'Reentered','1965-12-06 00:00:00','1965-12-04 00:00:00',88.27,169,209,51.87,'Intermediate' from dual union all
select 18110,'018110','1987-047N','KDU part','KDU part',60137,'Reentered','1987-06-27 00:00:00','1987-06-13 00:00:00',92.17,354,408,72.85,'Intermediate' from dual union all
select 18111,'018111','1987-040E','SOZ','SOZ',60130,'Reentered','1990-03-01 00:00:00','1987-06-15 00:00:00',618.34,315,35014,47.55,'GEO Transfer' from dual union all
select 18112,'018112','1987-040F','SOZ','SOZ',60130,'Reentered','1991-03-19 00:00:00','1987-06-19 00:00:00',619.56,375,35017,47.63,'GEO Transfer' from dual union all
select 18113,'018113','1987-051A','Kosmos-1852','Strela-1M',60141,'In Earth orbit','','1987-07-16 00:00:00',115.68,1473,1500,74,'Intermediate' from dual union all
select 18114,'018114','1987-051B','Kosmos-1853','Strela-1M',60141,'In Earth orbit','','1987-07-16 00:00:00',115.47,1472,1482,74,'Intermediate' from dual union all
select 18115,'018115','1987-051C','Kosmos-1854','Strela-1M',60141,'In Earth orbit','','1987-07-16 00:00:00',115.32,1459,1481,74,'Intermediate' from dual union all
select 18116,'018116','1987-051D','Kosmos-1855','Strela-1M',60141,'In Earth orbit','','1987-07-16 00:00:00',115.14,1447,1477,74,'Intermediate' from dual union all
select 18117,'018117','1987-051E','Kosmos-1856','Strela-1M',60141,'In Earth orbit','','1987-07-16 00:00:00',114.98,1432,1476,74,'Intermediate' from dual union all
select 18118,'018118','1987-051F','Kosmos-1857','Strela-1M',60141,'In Earth orbit','','1987-07-16 00:00:00',114.82,1418,1477,74,'Intermediate' from dual union all
select 18119,'018119','1987-051G','Kosmos-1858','Strela-1M',60141,'In Earth orbit','','1987-09-26 00:00:00',114.66,1402,1478,74,'Intermediate' from dual union all
select 1812,'001812','1965-100A','Gemini VII','Gemini SC7',11850,'Landed','1965-12-18 00:00:00','1965-12-09 00:00:00',90.54,299,303,28.9,'Intermediate' from dual union all
select 18120,'018120','1987-051H','Kosmos-1859','Strela-1M',60141,'In Earth orbit','','1987-09-26 00:00:00',114.48,1387,1476,74,'Intermediate' from dual union all
select 18121,'018121','1987-051J','S3M','S3M',60141,'In Earth orbit','','1987-10-04 00:00:00',117.81,1480,1684,74.01,'Intermediate' from dual union all
select 18122,'018122','1987-052A','Kosmos-1860','US-A ROCh',60142,'In Earth orbit','','1987-07-18 00:00:00',89.62,249,263,64.99,'Intermediate' from dual union all
select 18123,'018123','1987-053A','DMSP F-8','DMSP Block 5D-2 S-9',60143,'In Earth orbit','','1987-11-20 00:00:00',101.9,838,860,98.83,'Sun-Synch' from dual union all
select 18124,'018124','1987-047P','KDU part','KDU part',60137,'Reentered','1987-08-02 00:00:00','1987-06-28 00:00:00',92.65,359,450,72.36,'Intermediate' from dual union all
select 18125,'018125','1987-047Q','KDU part','KDU part',60137,'Reentered','1987-07-17 00:00:00','1987-06-21 00:00:00',91.98,358,386,72.89,'Intermediate' from dual union all
select 18126,'018126','1987-047R','KDU part','KDU part',60137,'Reentered','1987-07-10 00:00:00','1987-06-21 00:00:00',91.83,330,398,72.92,'Intermediate' from dual union all
select 18127,'018127','1987-053B','DMSP F-8 solar array band','',60143,'Reentered','2001-08-31 00:00:00','1987-09-25 00:00:00',101.89,837,860,98.83,'Sun-Synch' from dual union all
select 18128,'018128','1987-053C','DMSP F-8 solar array band','',60143,'Reentered','1999-11-04 00:00:00','1987-10-04 00:00:00',101.88,841,855,98.84,'Sun-Synch' from dual union all
select 18129,'018129','1987-054A','Kosmos-1861','Tsikada/RS-10/RS-11',60144,'In Earth orbit','','1987-07-26 00:00:00',104.95,984,1001,82.93,'Intermediate' from dual union all
select 1813,'001813','1965-100B','Titan GT-7 Stage 2','LR91AJ5 GT-7 65-12562',11850,'Reentered','1965-12-07 00:00:00','1965-12-05 00:00:00',89.28,160,318,28.89,'Intermediate' from dual union all
select 18130,'018130','1987-054B','S3M','S3M',60144,'In Earth orbit','','1987-10-02 00:00:00',104.84,979,994,82.93,'Intermediate' from dual union all
select 18131,'018131','1987-054C','deb Kosmos- 1861','',60144,'In Earth orbit','','1987-09-27 00:00:00',105.17,980,1024,82.93,'Intermediate' from dual union all
select 18132,'018132','1986-019NZ','deb Ariane V16','',59212,'Reentered','1988-02-03 00:00:00','1987-07-01 00:00:00',97.63,540,750,99.3,'Sun-Synch' from dual union all
select 18133,'018133','1986-019PA','deb Ariane V16','',59212,'Reentered','1988-11-02 00:00:00','1987-07-01 00:00:00',101.23,704,930,98.77,'Sun-Synch' from dual union all
select 18134,'018134','1986-019PB','deb Ariane V16','',59212,'Reentered','1989-02-12 00:00:00','1987-07-01 00:00:00',102.68,764,1007,98.96,'Sun-Synch' from dual union all
select 18135,'018135','1986-019PC','deb Ariane V16','',59212,'Reentered','1988-10-16 00:00:00','1987-07-01 00:00:00',99.52,690,782,98.74,'Sun-Synch' from dual union all
select 18136,'018136','1986-019PD','deb Ariane V16','',59212,'Reentered','1988-10-05 00:00:00','1987-07-01 00:00:00',100.26,736,806,98.41,'Sun-Synch' from dual union all
select 18137,'018137','1986-019PE','deb Ariane V16','',59212,'Reentered','1988-11-03 00:00:00','1987-07-01 00:00:00',99.59,702,776,98.71,'Sun-Synch' from dual union all
select 18138,'018138','1986-019PF','deb Ariane V16','',59212,'Reentered','1989-03-14 00:00:00','1987-07-01 00:00:00',100.02,717,802,98.74,'Sun-Synch' from dual union all
select 18139,'018139','1986-019PG','deb Ariane V16','',59212,'Reentered','1989-01-06 00:00:00','1987-07-01 00:00:00',100.82,754,841,98.66,'Sun-Synch' from dual union all
select 1814,'001814','1965-101A','FR-1','FR-1',11851,'In Earth orbit','','1966-01-21 00:00:00',99.87,746,759,75.88,'Intermediate' from dual union all
select 18140,'018140','1986-019PH','deb Ariane V16','',59212,'Reentered','1988-10-23 00:00:00','1987-07-01 00:00:00',101.59,723,945,99.16,'Sun-Synch' from dual union all
select 18141,'018141','1986-019PJ','deb Ariane V16','',59212,'Reentered','1988-09-26 00:00:00','1987-07-01 00:00:00',101.79,736,951,98.81,'Sun-Synch' from dual union all
select 18142,'018142','1986-019PK','deb Ariane V16','',59212,'Reentered','1987-11-10 00:00:00','1987-07-01 00:00:00',96.25,453,705,98.94,'Sun-Synch' from dual union all
select 18143,'018143','1986-019PL','deb Ariane V16','',59212,'Reentered','1989-01-19 00:00:00','1987-07-01 00:00:00',100.97,767,843,98.66,'Sun-Synch' from dual union all
select 18144,'018144','1986-019PM','deb Ariane V16','',59212,'Reentered','1989-10-14 00:00:00','1987-07-01 00:00:00',103.35,805,1029,97.84,'Sun-Synch' from dual union all
select 18145,'018145','1986-019PN','deb Ariane V16','',59212,'Reentered','1991-06-15 00:00:00','1987-07-01 00:00:00',103.95,799,1092,97.89,'Sun-Synch' from dual union all
select 18146,'018146','1986-019PP','deb Ariane V16','',59212,'Reentered','1990-03-27 00:00:00','1987-09-29 00:00:00',101.62,784,887,98.66,'Sun-Synch' from dual union all
select 18147,'018147','1986-019PQ','deb Ariane V16','',59212,'Reentered','1989-08-14 00:00:00','1987-07-01 00:00:00',103.39,807,1031,99.23,'Sun-Synch' from dual union all
select 18148,'018148','1986-019PR','deb Ariane V16','',59212,'Reentered','1989-12-12 00:00:00','1987-07-01 00:00:00',101.26,793,844,98.48,'Sun-Synch' from dual union all
select 18149,'018149','1986-019PS','deb Ariane V16','',59212,'Reentered','1988-12-14 00:00:00','1987-07-01 00:00:00',100.82,754,841,98.62,'Sun-Synch' from dual union all
select 1815,'001815','1965-101B','Altair 2 X-258E6 RH-126','X-258E6',11851,'In Earth orbit','','1966-01-14 00:00:00',100.01,751,767,75.87,'Intermediate' from dual union all
select 18150,'018150','1986-019PT','deb Ariane V16','',59212,'Reentered','1990-02-20 00:00:00','1987-07-01 00:00:00',101.71,771,909,98.63,'Sun-Synch' from dual union all
select 18151,'018151','1986-019PU','deb Ariane V16','',59212,'Reentered','1989-09-19 00:00:00','1987-07-01 00:00:00',101.34,771,873,98.6,'Sun-Synch' from dual union all
select 18152,'018152','1987-055A','Kosmos-1862','Tselina-D',60145,'In Earth orbit','','1987-09-26 00:00:00',97.7,630,667,82.51,'Intermediate' from dual union all
select 18153,'018153','1987-055B','S5M','S5M',60145,'In Earth orbit','','1987-09-28 00:00:00',97.67,630,665,82.51,'Intermediate' from dual union all
select 18154,'018154','1987-053D','DMSP F-8 OLS Cover','',60143,'Reentered','1999-04-11 00:00:00','1987-09-25 00:00:00',101.89,838,859,98.83,'Sun-Synch' from dual union all
select 18155,'018155','1987-056A','Kosmos-1863','Oblik No. 359',60146,'Landed','1987-07-18 00:00:00','1987-07-06 00:00:00',92.26,356,415,72.88,'Intermediate' from dual union all
select 18156,'018156','1987-056B','11S510','11S510',60146,'Reentered','1987-07-18 00:00:00','1987-07-06 00:00:00',89.82,194,338,72.87,'Intermediate' from dual union all
select 18157,'018157','1987-056C','Sensor cover','Sensor cover',60146,'Reentered','1987-07-08 00:00:00','1987-07-07 00:00:00',88.94,180,265,72.86,'Intermediate' from dual union all
select 18158,'018158','1987-056D','Sensor cover','Sensor cover',60146,'Reentered','1987-07-07 00:00:00','1987-07-06 00:00:00',88.89,179,260,72.86,'Intermediate' from dual union all
select 18159,'018159','1987-053E','DMSP F-8 OLS cooler cover','',60143,'Reentered','2001-11-10 00:00:00','1987-10-04 00:00:00',101.89,839,858,98.83,'Sun-Synch' from dual union all
select 1816,'001816','1965-102A','OPS 7249','CORONA JX-27',11852,'Reentered Attached','1965-12-26 00:00:00','1965-12-10 00:00:00',90.72,183,437,80,'Intermediate' from dual union all
select 18160,'018160','1987-057A','Kosmos-1864','Parus',60147,'In Earth orbit','','1987-09-26 00:00:00',104.75,958,1008,82.93,'Intermediate' from dual union all
select 18161,'018161','1987-057B','S3M','S3M',60147,'In Earth orbit','','1987-09-26 00:00:00',104.64,958,997,82.93,'Intermediate' from dual union all
select 18162,'018162','1987-058A','Kosmos-1865','Siluet No. 7',60148,'Landed','1987-08-14 00:00:00','1987-07-15 00:00:00',89.2,207,263,64.75,'Intermediate' from dual union all
select 18163,'018163','1987-058B','11S510','11S510',60148,'Reentered','1987-07-16 00:00:00','1987-07-10 00:00:00',89.23,188,285,64.76,'Intermediate' from dual union all
select 18164,'018164','1986-019PV','deb Ariane V16','',59212,'Reentered','1989-06-26 00:00:00','1987-07-08 00:00:00',100.94,783,824,98.63,'Sun-Synch' from dual union all
select 18165,'018165','1986-019PW','deb Ariane V16','',59212,'Reentered','1987-08-29 00:00:00','1987-07-08 00:00:00',96.33,496,670,99.67,'Sun-Synch' from dual union all
select 18166,'018166','1986-019PX','deb Ariane V16','',59212,'Reentered','1989-02-26 00:00:00','1987-07-08 00:00:00',102.7,765,1008,98.89,'Sun-Synch' from dual union all
select 18167,'018167','1986-019PY','deb Ariane V16','',59212,'Reentered','1989-03-21 00:00:00','1987-07-08 00:00:00',100.17,741,793,98.81,'Sun-Synch' from dual union all
select 18168,'018168','1986-019PZ','deb Ariane V16','',59212,'Reentered','1989-03-13 00:00:00','1987-07-08 00:00:00',100.26,741,801,98.63,'Sun-Synch' from dual union all
select 18169,'018169','1986-019QA','deb Ariane V16','',59212,'Reentered','1990-04-18 00:00:00','1987-07-08 00:00:00',101.88,796,900,98.57,'Sun-Synch' from dual union all
select 1817,'001817','1965-103A','Kosmos-99','Zenit-2 No. 32',11853,'Landed','1965-12-18 00:00:00','1965-12-10 00:00:00',89.62,203,309,65,'Intermediate' from dual union all
select 18170,'018170','1986-019QB','deb Ariane V16','',59212,'Reentered','1987-11-10 00:00:00','1987-07-08 00:00:00',96.84,521,694,99.2,'Sun-Synch' from dual union all
select 18171,'018171','1986-019QC','deb Ariane V16','',59212,'Reentered','1989-10-21 00:00:00','1987-07-08 00:00:00',100.94,769,837,98.57,'Sun-Synch' from dual union all
select 18172,'018172','1986-019QD','deb Ariane V16','',59212,'Reentered','1989-11-12 00:00:00','1987-07-08 00:00:00',101.15,781,846,98.6,'Sun-Synch' from dual union all
select 18173,'018173','1986-019QE','deb Ariane V16','',59212,'Reentered','1989-05-25 00:00:00','1987-07-08 00:00:00',101.23,763,871,98.65,'Sun-Synch' from dual union all
select 18174,'018174','1986-019QF','deb Ariane V16','',59212,'Reentered','1989-08-09 00:00:00','1987-07-08 00:00:00',100.97,792,818,98.64,'Sun-Synch' from dual union all
select 18175,'018175','1986-019QG','deb Ariane V16','',59212,'Reentered','1988-01-19 00:00:00','1987-07-08 00:00:00',97.88,558,757,98.87,'Sun-Synch' from dual union all
select 18176,'018176','1986-019QH','deb Ariane V16','',59212,'Reentered','1989-10-27 00:00:00','1987-07-08 00:00:00',101.06,773,845,98.54,'Sun-Synch' from dual union all
select 18177,'018177','1986-019QJ','deb Ariane V16','',59212,'Reentered','1989-10-12 00:00:00','1987-07-08 00:00:00',101.72,739,942,98.59,'Sun-Synch' from dual union all
select 18178,'018178','1986-019QK','deb Ariane V16','',59212,'Reentered','1990-03-26 00:00:00','1987-07-08 00:00:00',103.09,793,1017,98.09,'Sun-Synch' from dual union all
select 18179,'018179','1986-019QL','deb Ariane V16','',59212,'Reentered','1989-01-14 00:00:00','1987-07-08 00:00:00',100.83,765,831,98.62,'Sun-Synch' from dual union all
select 1818,'001818','1965-103B','8A92E','8A92E',11853,'Reentered','1965-12-28 00:00:00','1965-12-28 00:00:00',88,161,190,64.98,'Intermediate' from dual union all
select 18180,'018180','1986-019QM','deb Ariane V16','',59212,'Reentered','1989-07-24 00:00:00','1987-07-08 00:00:00',100.95,772,836,98.76,'Sun-Synch' from dual union all
select 18181,'018181','1986-019QN','deb Ariane V16','',59212,'Reentered','1989-07-22 00:00:00','1987-07-08 00:00:00',101.12,764,860,98.58,'Sun-Synch' from dual union all
select 18182,'018182','1986-019QP','deb Ariane V16','',59212,'Reentered','1989-01-13 00:00:00','1987-07-08 00:00:00',100.92,740,865,98.6,'Sun-Synch' from dual union all
select 18183,'018183','1986-019QQ','deb Ariane V16','',59212,'Reentered','1991-04-27 00:00:00','1987-07-08 00:00:00',100.95,782,826,98.58,'Sun-Synch' from dual union all
select 18184,'018184','1987-059A','Kosmos-1866','Kobal''t No. 511',60149,'Reentered','1987-11-06 00:00:00','1987-07-14 00:00:00',89.51,163,338,67.15,'Intermediate' from dual union all
select 18185,'018185','1987-059B','11S510','11S510',60149,'Reentered','1987-07-15 00:00:00','1987-07-11 00:00:00',89.42,163,329,67.15,'Intermediate' from dual union all
select 18186,'018186','1987-059C','deb Kosmos- 1866','',60149,'Reentered','1987-07-10 00:00:00','1987-07-10 00:00:00',89.28,162,316,67.14,'Intermediate' from dual union all
select 18187,'018187','1987-060A','Kosmos-1867','Plazma-A No. 2',60150,'In Earth orbit','','1987-09-25 00:00:00',100.74,787,800,65.01,'Intermediate' from dual union all
select 18188,'018188','1987-060B','11S692','11S692',60150,'Reentered','1987-07-11 00:00:00','1987-07-10 00:00:00',89.83,111,422,65,'Intermediate' from dual;
insert into SATELLITE
select 18189,'018189','1987-047S','KDU part','KDU part',60137,'Reentered','1987-07-12 00:00:00','1987-07-08 00:00:00',90.25,282,292,72.84,'Intermediate' from dual union all
select 1819,'001819','1965-082DJ','deb Transtage 4','',11832,'Reentered','1979-06-20 00:00:00','1979-05-17 00:00:00',93.71,446,466,31.55,'Intermediate' from dual union all
select 18190,'018190','1987-047T','KDU part','KDU part',60137,'Reentered','1987-07-12 00:00:00','1987-07-05 00:00:00',90.57,299,306,72.63,'Intermediate' from dual union all
select 18191,'018191','1987-060C','Kosmos-1867 reactor shield','Thermal shield',60150,'Reentered','1990-02-11 00:00:00','1987-10-04 00:00:00',100.74,787,800,65,'Intermediate' from dual union all
select 18192,'018192','1987-061A','Kosmos-1868','Taifun-1B',60151,'Reentered','1989-03-02 00:00:00','1987-09-27 00:00:00',94.38,282,696,74.01,'Intermediate' from dual union all
select 18193,'018193','1987-061B','S3M','S3M',60151,'Reentered','1988-08-30 00:00:00','1987-09-25 00:00:00',94.23,280,682,74.01,'Intermediate' from dual union all
select 18194,'018194','1986-019QR','deb Ariane V16','',59212,'Reentered','1988-05-24 00:00:00','1987-07-15 00:00:00',99.71,731,759,98.73,'Sun-Synch' from dual union all
select 18195,'018195','1986-019QS','deb Ariane V16','',59212,'Reentered','1988-12-10 00:00:00','1987-07-15 00:00:00',104.39,723,1209,98.11,'Sun-Synch' from dual union all
select 18196,'018196','1986-019QT','deb Ariane V16','',59212,'Reentered','2004-11-04 00:00:00','1991-10-20 00:00:00',100.95,763,845,98.57,'Sun-Synch' from dual union all
select 18197,'018197','1986-019QU','deb Ariane V16','',59212,'Reentered','1988-11-02 00:00:00','1987-07-15 00:00:00',101.01,722,890,98.71,'Sun-Synch' from dual union all
select 18198,'018198','1986-019QV','deb Ariane V16','',59212,'Reentered','1988-07-01 00:00:00','1987-07-15 00:00:00',100.67,717,864,98.7,'Sun-Synch' from dual union all
select 18199,'018199','1986-019QW','deb Ariane V16','',59212,'Reentered','1990-02-27 00:00:00','1987-07-15 00:00:00',101.02,785,829,98.58,'Sun-Synch' from dual union all
select 182,'000182','61 Omega 1','Discoverer 30','CORONA C''''''-2',7272,'Reentered Attached','1961-12-11 00:00:00','1961-10-13 00:00:00',91.69,231,484,82.66,'Intermediate' from dual union all
select 1820,'001820','1965-082DK','deb Transtage 4','',11832,'Reentered','1981-03-30 00:00:00','1981-03-29 00:00:00',91.12,310,348,33.13,'Intermediate' from dual union all
select 18200,'018200','1986-019QX','deb Ariane V16','',59212,'Reentered','1988-10-04 00:00:00','1987-07-15 00:00:00',99.22,648,795,98.77,'Sun-Synch' from dual union all
select 18201,'018201','1986-019QY','deb Ariane V16','',59212,'Reentered','1988-12-23 00:00:00','1987-07-15 00:00:00',100.92,751,853,98.71,'Sun-Synch' from dual union all
select 18202,'018202','1986-019QZ','deb Ariane V16','',59212,'Reentered','1989-11-11 00:00:00','1987-07-15 00:00:00',103.41,772,1068,98,'Sun-Synch' from dual union all
select 18203,'018203','1986-019RA','deb Ariane V16','',59212,'Reentered','1989-05-07 00:00:00','1987-07-15 00:00:00',101.22,800,833,98.58,'Sun-Synch' from dual union all
select 18204,'018204','1986-019RB','deb Ariane V16','',59212,'Reentered','1987-11-22 00:00:00','1987-07-15 00:00:00',98.38,632,731,98.97,'Sun-Synch' from dual union all
select 18205,'018205','1986-019RC','deb Ariane V16','',59212,'Reentered','1988-11-24 00:00:00','1987-07-15 00:00:00',103.01,699,1103,99.32,'Sun-Synch' from dual union all
select 18206,'018206','1986-019RD','deb Ariane V16','',59212,'Reentered','1990-03-31 00:00:00','1987-07-15 00:00:00',102.59,779,984,98.21,'Sun-Synch' from dual union all
select 18207,'018207','1986-019RE','deb Ariane V16','',59212,'Reentered','1989-05-20 00:00:00','1987-07-15 00:00:00',100.76,764,826,98.61,'Sun-Synch' from dual union all
select 18208,'018208','1986-019RF','deb Ariane V16','',59212,'Reentered','2000-07-25 00:00:00','1991-11-03 00:00:00',99.33,716,737,98.47,'Sun-Synch' from dual union all
select 18209,'018209','1986-019RG','deb Ariane V16','',59212,'Reentered','1988-10-25 00:00:00','1987-07-15 00:00:00',100.56,757,814,98.72,'Sun-Synch' from dual union all
select 1821,'001821','1965-082DL','deb Transtage 4','',11832,'Reentered','1967-05-05 00:00:00','1967-04-23 00:00:00',94.79,494,523,33.22,'Intermediate' from dual union all
select 18210,'018210','1986-019RH','deb Ariane V16','',59212,'Reentered','1989-03-09 00:00:00','1987-07-15 00:00:00',100.3,732,814,98.25,'Sun-Synch' from dual union all
select 18211,'018211','1986-019RJ','deb Ariane V16','',59212,'Reentered','1989-10-11 00:00:00','1987-07-15 00:00:00',101.04,779,837,98.47,'Sun-Synch' from dual union all
select 18212,'018212','1986-019RK','deb Ariane V16','',59212,'Reentered','2002-04-13 00:00:00','1991-10-11 00:00:00',100.44,747,811,98.51,'Sun-Synch' from dual union all
select 18213,'018213','1986-019RL','deb Ariane V16','',59212,'Reentered','1989-04-06 00:00:00','1987-07-15 00:00:00',101.68,769,908,98.73,'Sun-Synch' from dual union all
select 18214,'018214','1987-062A','Kosmos-1869','Okean-O1 NKhM No. 4',60152,'In Earth orbit','','1987-09-26 00:00:00',97.71,631,667,82.52,'Intermediate' from dual union all
select 18215,'018215','1987-062B','S5M','S5M',60152,'In Earth orbit','','1987-10-03 00:00:00',97.69,630,666,82.52,'Intermediate' from dual union all
select 18216,'018216','1987-056E','KDU part','KDU part',60146,'Reentered','1987-08-20 00:00:00','1987-07-18 00:00:00',92.27,357,414,72.88,'Intermediate' from dual union all
select 18217,'018217','1987-056F','Blenda','Sunshade',60146,'Reentered','1987-11-13 00:00:00','1987-09-25 00:00:00',91.47,325,369,72.87,'Intermediate' from dual union all
select 18218,'018218','1987-056G','KDU part','KDU part',60146,'Reentered','1988-01-23 00:00:00','1987-09-26 00:00:00',93.83,384,540,72.88,'Intermediate' from dual union all
select 18219,'018219','1987-056H','KDU part','KDU part',60146,'Reentered','1987-10-08 00:00:00','1987-09-27 00:00:00',90.66,296,318,72.86,'Intermediate' from dual union all
select 1822,'001822','1965-082DM','Transtage 4 main debris','Transtage 4',11832,'Reentered','2003-12-02 00:00:00','1966-01-01 00:00:00',98.6,628,756,32.16,'Intermediate' from dual union all
select 18220,'018220','1987-056J','KDU part','KDU part',60146,'Reentered','1987-08-09 00:00:00','1987-07-18 00:00:00',92.43,316,471,72.98,'Intermediate' from dual union all
select 18221,'018221','1987-056K','KDU part','KDU part',60146,'Reentered','1987-08-23 00:00:00','1987-07-19 00:00:00',92.21,354,412,72.88,'Intermediate' from dual union all
select 18222,'018222','1987-063A','Soyuz TM-3','Soyuz 7K-STM No. 53',60153,'Landed','1987-12-29 00:00:00','1987-08-24 00:00:00',91.16,307,356,51.63,'Intermediate' from dual union all
select 18223,'018223','1987-063B','11S510','11S510',60153,'Reentered','1987-07-24 00:00:00','1987-07-22 00:00:00',88.47,199,199,51.62,'Intermediate' from dual union all
select 18224,'018224','1987-056L','KDU','KDU',60146,'Reentered','1988-10-07 00:00:00','1987-09-25 00:00:00',92.16,354,407,72.87,'Intermediate' from dual union all
select 18225,'018225','1987-064A','Kosmos-1870','Mech-K No. 304',60154,'Deorbited','1989-07-29 00:00:00','1987-09-01 00:00:00',89.46,242,254,71.93,'Intermediate' from dual union all
select 18226,'018226','1987-064B','8S812','8S812',60154,'Reentered','1987-07-27 00:00:00','1987-07-25 00:00:00',88.23,158,216,71.94,'Intermediate' from dual union all
select 18227,'018227','1987-064C','Kosmos-1870 adapter','Adapter',60154,'Reentered','1987-07-26 00:00:00','1987-07-25 00:00:00',87.72,139,185,71.93,'Intermediate' from dual union all
select 18228,'018228','1986-019RM','deb Ariane V16','',59212,'Reentered','1989-07-19 00:00:00','1987-09-26 00:00:00',100.95,772,836,98.54,'Sun-Synch' from dual union all
select 18229,'018229','1987-012C','Ginga despin weight','',60102,'Reentered','1988-11-25 00:00:00','1987-03-19 00:00:00',96.64,530,666,31.12,'Intermediate' from dual union all
select 1823,'001823','1965-082DN','deb Transtage 4','',11832,'Reentered','1974-04-21 00:00:00','1965-12-23 00:00:00',97.49,543,735,32.39,'Intermediate' from dual union all
select 18230,'018230','1987-004GN','deb Kosmos-1813','',60094,'Reentered','1987-08-24 00:00:00','1987-07-28 00:00:00',89.74,235,289,72.8,'Intermediate' from dual union all
select 18231,'018231','1986-019RN','deb Ariane V16','',59212,'In Earth orbit','','1987-03-07 00:00:00',261.55,791,13548,98.78,'Medium' from dual union all
select 18232,'018232','1976-077EY','deb Delta 126','',39029,'In Earth orbit','','1977-12-31 00:00:00',114.04,1315,1509,102.92,'Sun-Synch' from dual union all
select 18233,'018233','1980-089CB','deb Kosmos-1220','',50299,'Reentered','1989-02-02 00:00:00','1987-09-26 00:00:00',96.98,522,707,64.94,'Intermediate' from dual union all
select 18234,'018234','1986-019RP','deb Ariane V16','',59212,'Reentered','1989-11-05 00:00:00','1987-09-25 00:00:00',101.02,773,841,98.56,'Sun-Synch' from dual union all
select 18235,'018235','1987-004GP','deb Kosmos-1813','',60094,'Reentered','1987-10-25 00:00:00','1987-09-26 00:00:00',90.1,258,301,72.8,'Intermediate' from dual union all
select 18236,'018236','1987-004GQ','deb Kosmos-1813','',60094,'Reentered','1987-11-04 00:00:00','1987-09-25 00:00:00',90.75,299,324,72.81,'Intermediate' from dual union all
select 18237,'018237','1983-044DY','deb Kosmos-1461','',55801,'Reentered','1988-12-21 00:00:00','1985-07-31 00:00:00',99.3,570,881,65.03,'Intermediate' from dual union all
select 18238,'018238','1987-012D','Ginga despin weight','',60102,'Reentered','1989-10-08 00:00:00','1987-05-04 00:00:00',96.6,510,682,31.09,'Intermediate' from dual union all
select 18239,'018239','1987-052B','Kosmos-1860 RLS','US-A RLS antenna',60142,'Reentered','1987-08-02 00:00:00','1987-07-29 00:00:00',89.51,239,262,64.98,'Intermediate' from dual union all
select 1824,'001824','1965-082DP','deb Transtage 4','',11832,'Reentered','1967-07-11 00:00:00','1966-01-01 00:00:00',97.72,600,700,32.5,'Intermediate' from dual union all
select 18240,'018240','1987-052C','Kosmos-1860 DU','US-A DU',60142,'Reentered','1987-09-16 00:00:00','1987-07-28 00:00:00',89.56,244,262,64.98,'Intermediate' from dual union all
select 18241,'018241','1987-052D','Kosmos-1860 reactor core','US-A reactor core',60142,'In Earth orbit','','1987-09-28 00:00:00',103.69,900,966,65.01,'Intermediate' from dual union all
select 18242,'018242','1987-013C','Soyuz TM-2 BO','Bitovoy otsek',60103,'Reentered','1988-01-16 00:00:00','1987-09-25 00:00:00',90.88,298,338,51.63,'Intermediate' from dual union all
select 18243,'018243','1983-044DZ','deb Kosmos-1461','',55801,'Reentered','1989-09-07 00:00:00','1987-07-31 00:00:00',98.9,542,871,65.01,'Intermediate' from dual union all
select 18244,'018244','1983-044EA','deb Kosmos-1461','',55801,'Reentered','1989-09-13 00:00:00','1987-07-31 00:00:00',100.3,641,905,65.01,'Intermediate' from dual union all
select 18245,'018245','1983-044EB','deb Kosmos-1461','',55801,'Reentered','1988-11-29 00:00:00','1987-07-31 00:00:00',98.71,553,841,65.02,'Intermediate' from dual union all
select 18246,'018246','1983-044EC','deb Kosmos-1461','',55801,'Reentered','1989-02-13 00:00:00','1987-07-31 00:00:00',98.88,555,856,65.02,'Intermediate' from dual union all
select 18247,'018247','1983-044ED','deb Kosmos-1461','',55801,'Reentered','1989-02-12 00:00:00','1987-07-31 00:00:00',98.8,550,853,65.03,'Intermediate' from dual union all
select 18248,'018248','1983-044EE','deb Kosmos-1461','',55801,'Reentered','1989-11-21 00:00:00','1987-07-31 00:00:00',99.15,562,873,65.02,'Intermediate' from dual union all
select 18249,'018249','1983-044EF','deb Kosmos-1461','',55801,'Reentered','1988-12-21 00:00:00','1987-07-31 00:00:00',98.51,543,831,65.03,'Intermediate' from dual union all
select 1825,'001825','1965-082DQ','deb Transtage 4','',11832,'Reentered','1989-07-03 00:00:00','1989-06-28 00:00:00',91.23,322,348,32.6,'Intermediate' from dual union all
select 18250,'018250','1979-017KV','deb P78-1','',47501,'Reentered','1987-09-30 00:00:00','1987-07-31 00:00:00',93.13,421,435,97.7,'Sun-Synch' from dual union all
select 18251,'018251','1979-017KW','deb P78-1','',47501,'Reentered','1987-09-11 00:00:00','1987-07-31 00:00:00',92.94,410,427,97.7,'Sun-Synch' from dual union all
select 18252,'018252','1979-017KX','deb P78-1','',47501,'Reentered','1989-10-07 00:00:00','1987-07-31 00:00:00',95.35,515,556,97.81,'Sun-Synch' from dual union all
select 18253,'018253','1979-017KY','deb P78-1','',47501,'Reentered','1987-11-29 00:00:00','1987-07-31 00:00:00',93.74,449,466,97.77,'Sun-Synch' from dual union all
select 18254,'018254','1979-017KZ','deb P78-1','',47501,'Reentered','1987-10-28 00:00:00','1987-07-31 00:00:00',92.88,399,433,97.73,'Sun-Synch' from dual union all
select 18255,'018255','1979-017LA','deb P78-1','',47501,'Reentered','1990-06-23 00:00:00','1990-06-21 00:00:00',87.7,156,166,97.76,'Sun-Synch' from dual union all
select 18256,'018256','1969-082KZ','deb Agena D','',18349,'Reentered','1989-09-07 00:00:00','1987-07-31 00:00:00',98.48,639,733,70.09,'Intermediate' from dual union all
select 18257,'018257','1987-006C','deb Kosmos                        ','[UNID110] RCS 0.01',60096,'In Earth orbit','','1986-04-14 00:00:00',100.87,792,808,74.05,'Intermediate' from dual union all
select 18258,'018258','1965-082UM','deb Transtage','',11832,'Reentered','1989-02-04 00:00:00','1985-07-31 00:00:00',95.2,526,530,32.42,'Intermediate' from dual union all
select 18259,'018259','1987-065A','Kosmos-1871','EPN No. 03.695',60155,'Reentered','1987-08-10 00:00:00','1987-08-03 00:00:00',88.11,171,191,97.02,'Sun-Synch' from dual union all
select 1826,'001826','1965-082DR','deb Transtage 4','',11832,'Reentered','1966-03-22 00:00:00','1966-02-17 00:00:00',92.96,329,510,32.59,'Intermediate' from dual union all
select 18260,'018260','1987-065B','11S772','11S772',60155,'Reentered','1987-08-03 00:00:00','1987-08-02 00:00:00',87.81,152,181,97.01,'Sun-Synch' from dual union all
select 18261,'018261','1986-019RQ','deb Ariane V16','',59212,'Reentered','1989-02-19 00:00:00','1987-08-07 00:00:00',100.27,753,789,98.2,'Sun-Synch' from dual union all
select 18262,'018262','1986-019RR','deb Ariane V16','',59212,'Reentered','1989-03-21 00:00:00','1987-08-06 00:00:00',103.67,806,1058,99.14,'Sun-Synch' from dual union all
select 18263,'018263','1986-019RS','deb Ariane V16','',59212,'Reentered','1989-02-18 00:00:00','',100,720,797,98.71,'Sun-Synch' from dual union all
select 18264,'018264','1986-019RT','deb Ariane V16','',59212,'Reentered','1988-11-14 00:00:00','1987-08-05 00:00:00',100.54,768,801,98.73,'Sun-Synch' from dual union all
select 18265,'018265','1986-019RU','deb Ariane V16','',59212,'Reentered','1989-02-12 00:00:00','1987-08-04 00:00:00',101.15,783,844,98.71,'Sun-Synch' from dual union all
select 18266,'018266','1986-019RV','deb Ariane V16','',59212,'Reentered','1988-12-11 00:00:00','1987-08-08 00:00:00',100.81,764,830,98.66,'Sun-Synch' from dual union all
select 18267,'018267','1986-019RW','deb Ariane V16','',59212,'Reentered','1989-01-05 00:00:00','1987-08-08 00:00:00',101.3,783,857,98.62,'Sun-Synch' from dual union all
select 18268,'018268','1965-027X','deb Snapshot','',11777,'In Earth orbit','','1986-04-13 00:00:00',111.47,1269,1320,90.28,'Polar' from dual union all
select 18269,'018269','1965-027Y','deb Snapshot','',11777,'In Earth orbit','','1985-07-31 00:00:00',111.48,1273,1317,90.29,'Polar' from dual union all
select 1827,'001827','1965-082DS','deb Transtage 4','',11832,'Reentered','1980-12-30 00:00:00','1966-01-24 00:00:00',97.94,586,734,32.36,'Intermediate' from dual union all
select 18270,'018270','1965-027Z','deb Snapshot','',11777,'In Earth orbit','','1987-07-29 00:00:00',111.47,1272,1317,90.3,'Polar' from dual union all
select 18271,'018271','1965-027AA','deb Snapshot','',11777,'In Earth orbit','','1987-08-01 00:00:00',111.48,1273,1317,90.3,'Polar' from dual union all
select 18272,'018272','1976-067BL','deb Kosmos-839','[post Jul 1999]',39019,'In Earth orbit','','1999-11-23 00:00:00',116.39,975,2061,65.85,'Intermediate' from dual union all
select 18273,'018273','1987-004GR','deb Kosmos-1813','',60094,'Reentered','1990-07-16 00:00:00','1990-07-14 00:00:00',88.91,215,227,72.76,'Intermediate' from dual union all
select 18274,'018274','1986-052F','deb Kosmos-1763                  ','[UNID]',59245,'In Earth orbit','','1986-07-29 00:00:00',100.9,766,837,74.07,'Intermediate' from dual union all
select 18275,'018275','1981-053LQ','deb Kosmos-1275','',52576,'In Earth orbit','','1991-10-05 00:00:00',106.48,947,1181,82.94,'Intermediate' from dual union all
select 18276,'018276','1987-054D','deb Kosmos-1861','[UNID111] RCS 0.01',60144,'In Earth orbit','','1991-10-02 00:00:00',104.89,980,999,82.93,'Intermediate' from dual union all
select 18277,'018277','1971-003C','deb Blok E','',24821,'Reentered','1996-07-28 00:00:00','1987-08-03 00:00:00',96.68,533,666,81.21,'Intermediate' from dual union all
select 18278,'018278','1982-055BA','deb Kosmos-1375','',54364,'In Earth orbit','','1991-10-09 00:00:00',104.59,949,1001,65.83,'Intermediate' from dual;
insert into SATELLITE
select 18279,'018279','1982-055BB','deb Kosmos-1375','',54364,'In Earth orbit','','1991-10-04 00:00:00',105.39,959,1067,65.83,'Intermediate' from dual union all
select 1828,'001828','1965-082DT','deb Transtage 4','',11832,'Reentered','1979-01-02 00:00:00','1966-01-12 00:00:00',98.59,627,756,32.16,'Intermediate' from dual union all
select 18280,'018280','1985-082Y','deb Kosmos-1682','',58305,'Reentered','1988-05-31 00:00:00','1987-08-01 00:00:00',93.22,345,520,64.97,'Intermediate' from dual union all
select 18281,'018281','1975-025D','deb Kosmos-724 reactor','',36472,'Reentered','1988-10-05 00:00:00','1986-05-14 00:00:00',102.98,876,924,65.58,'Intermediate' from dual union all
select 18282,'018282','1985-094T','deb Kosmos-1695','',58317,'In Earth orbit','','1985-11-24 00:00:00',114.17,1410,1426,82.64,'Intermediate' from dual union all
select 18283,'018283','1987-066A','Progress-31','Progress 7K-TG No. 138',60156,'Deorbited','1987-09-23 00:00:00','1987-08-11 00:00:00',91.2,308,359,51.63,'Intermediate' from dual union all
select 18284,'018284','1987-066B','11S510','11S510',60156,'Reentered','1987-08-07 00:00:00','1987-08-05 00:00:00',88.36,177,210,51.64,'Intermediate' from dual union all
select 18285,'018285','1987-059D','deb Kosmos-1866','',60149,'Reentered','1987-08-19 00:00:00','1987-08-02 00:00:00',90.15,208,356,67.26,'Intermediate' from dual union all
select 18286,'018286','1987-059E','deb Kosmos- 1866','',60149,'Reentered','1987-08-28 00:00:00','1987-08-03 00:00:00',91.94,223,517,67.59,'Intermediate' from dual union all
select 18287,'018287','1987-059F','deb Kosmos- 1866','',60149,'Reentered','1987-08-18 00:00:00','1987-08-01 00:00:00',90.3,227,351,67.4,'Intermediate' from dual union all
select 18288,'018288','1987-059G','deb Kosmos- 1866','',60149,'Reentered','1987-09-17 00:00:00','1987-08-12 00:00:00',98.09,219,1116,67.14,'Intermediate' from dual union all
select 18289,'018289','1986-019RX','deb Ariane V16','',59212,'Reentered','1988-10-02 00:00:00','1987-08-04 00:00:00',102.68,767,1004,98.89,'Sun-Synch' from dual union all
select 1829,'001829','1965-082DU','deb Transtage 4','',11832,'In Earth orbit','','1992-01-27 00:00:00',98.68,667,725,32.3,'Intermediate' from dual union all
select 18290,'018290','1986-019RY','deb Ariane V16','[UNID112] RCS 0.0',59212,'In Earth orbit','','1991-10-08 00:00:00',100.21,695,841,97.92,'Sun-Synch' from dual union all
select 18291,'018291','1986-019RZ','deb Ariane V16','',59212,'Reentered','1988-10-23 00:00:00','1987-08-03 00:00:00',100.35,760,790,98.78,'Sun-Synch' from dual union all
select 18292,'018292','1986-019SA','deb Ariane V16','',59212,'Reentered','1988-12-10 00:00:00','1987-08-03 00:00:00',100.96,728,881,99.58,'Sun-Synch' from dual union all
select 18293,'018293','1986-019SB','deb Ariane V16','',59212,'Reentered','1989-08-15 00:00:00','1987-08-03 00:00:00',100.02,708,811,98.99,'Sun-Synch' from dual union all
select 18294,'018294','1986-019SC','deb Ariane V16','',59212,'Reentered','1989-02-04 00:00:00','1987-08-03 00:00:00',99.51,676,795,98.94,'Sun-Synch' from dual union all
select 18295,'018295','1986-019SD','deb Ariane V16','',59212,'Reentered','1989-09-09 00:00:00','1987-08-03 00:00:00',101.99,816,890,98.77,'Sun-Synch' from dual union all
select 18296,'018296','1986-019SE','deb Ariane V16','',59212,'Reentered','1989-03-10 00:00:00','1987-08-03 00:00:00',100.47,719,843,98.68,'Sun-Synch' from dual union all
select 18297,'018297','1986-019SF','deb Ariane V16','',59212,'Reentered','1991-03-26 00:00:00','1987-08-03 00:00:00',98.24,593,756,98.84,'Sun-Synch' from dual union all
select 18298,'018298','1986-019SG','deb Ariane V16','',59212,'Reentered','1989-04-19 00:00:00','1987-08-03 00:00:00',101.06,718,900,99.08,'Sun-Synch' from dual union all
select 18299,'018299','1986-019SH','deb Ariane V16','',59212,'Reentered','1989-01-07 00:00:00','1987-08-03 00:00:00',100.77,771,819,98.59,'Sun-Synch' from dual union all
select 183,'000183','61 A Alpha 1','Mercury MA-4','Mercury SC8A',7249,'Landed','1961-09-13 00:00:00','1961-09-13 00:00:00',88.38,149,240,32.54,'Intermediate' from dual union all
select 1830,'001830','1965-082DV','deb Transtage 4','',11832,'Reentered','1991-06-18 00:00:00','1966-01-06 00:00:00',99.95,725,788,32.28,'Intermediate' from dual union all
select 18300,'018300','1986-019SJ','deb Ariane V16','',59212,'Reentered','1989-11-22 00:00:00','1987-08-03 00:00:00',101.54,785,878,98.54,'Sun-Synch' from dual union all
select 18301,'018301','1978-026HC','deb Delta 139','',45221,'Reentered','1988-10-02 00:00:00','1987-08-03 00:00:00',96.05,549,590,98.67,'Sun-Synch' from dual union all
select 18302,'018302','1978-026HD','deb Delta 139','',45221,'In Earth orbit','','1991-10-05 00:00:00',105.1,806,1193,98.87,'Sun-Synch' from dual union all
select 18303,'018303','1979-017LB','deb P78-1','',47501,'Reentered','1988-01-30 00:00:00','1987-08-03 00:00:00',93.52,428,466,97.72,'Sun-Synch' from dual union all
select 18304,'018304','1979-017LC','deb P78-1','',47501,'Reentered','1989-06-22 00:00:00','1987-08-03 00:00:00',94.12,447,505,97.76,'Sun-Synch' from dual union all
select 18305,'018305','1987-004GS','deb Kosmos-1813','',60094,'Reentered','1988-10-17 00:00:00','1987-08-03 00:00:00',99.71,361,1128,72.88,'Intermediate' from dual union all
select 18306,'018306','1987-067A','Fanhui Shi Weixing (09)','Jianbing-1 No. 10',60157,'Reentered','1987-08-23 00:00:00','1987-08-12 00:00:00',89.79,170,359,62.95,'Intermediate' from dual union all
select 18307,'018307','1987-067B','CZ-2C Stage 2','CZ-2C Stage 2',60157,'Reentered','1987-08-21 00:00:00','1987-08-12 00:00:00',89.51,167,334,62.95,'Intermediate' from dual union all
select 18308,'018308','1987-059H','deb Kosmos- 1866','',60149,'Reentered','1987-09-20 00:00:00','1987-08-18 00:00:00',92.3,228,547,67.2,'Intermediate' from dual union all
select 18309,'018309','1987-059J','deb Kosmos- 1866','',60149,'Reentered','1987-10-16 00:00:00','1987-08-17 00:00:00',105.9,186,1887,67.03,'Intermediate' from dual union all
select 1831,'001831','1965-082DW','deb Transtage 4','',11832,'Reentered','1967-11-20 00:00:00','1967-11-15 00:00:00',91.29,333,343,32.43,'Intermediate' from dual union all
select 18310,'018310','1987-059K','deb Kosmos- 1866','',60149,'Reentered','1987-10-12 00:00:00','1987-08-08 00:00:00',104.11,233,1673,67.57,'Intermediate' from dual union all
select 18311,'018311','1987-059L','deb Kosmos- 1866','',60149,'Reentered','1987-10-10 00:00:00','1987-08-17 00:00:00',91.35,225,457,67.15,'Intermediate' from dual union all
select 18312,'018312','1987-068A','Meteor-2','Meteor-2 No. 21',60158,'In Earth orbit','','1987-09-17 00:00:00',104.08,943,960,82.56,'Intermediate' from dual union all
select 18313,'018313','1987-068B','S5M','S5M',60158,'In Earth orbit','','1987-09-17 00:00:00',104.04,942,957,82.56,'Intermediate' from dual union all
select 18314,'018314','1987-069A','Kosmos-1872','Oblik No. 360',60159,'Landed','1987-08-30 00:00:00','1987-08-21 00:00:00',90.83,247,384,72.87,'Intermediate' from dual union all
select 18315,'018315','1987-069B','11S510','11S510',60159,'Reentered','1987-08-26 00:00:00','1987-08-21 00:00:00',89.18,189,278,72.84,'Intermediate' from dual union all
select 18316,'018316','1987-070A','Kiku 5','ETS-5',60160,'In Earth orbit','','1987-09-26 00:00:00',1436.06,35781,35791,.11,'Stationary' from dual union all
select 18317,'018317','1987-070B','H-I Stage 2 S/N H-17(F)','LE-5',60160,'Reentered','1987-08-28 00:00:00','1987-08-27 00:00:00',88.31,167,215,30.39,'Intermediate' from dual union all
select 18318,'018318','1987-071A','Kosmos-1873','EPN No. 03.695',60161,'Reentered','1987-09-14 00:00:00','1987-09-04 00:00:00',88.5,172,229,64.82,'Intermediate' from dual union all
select 18319,'018319','1987-071B','11S772','11S772',60161,'Reentered','1987-09-02 00:00:00','1987-08-30 00:00:00',88.26,168,209,64.83,'Intermediate' from dual union all
select 1832,'001832','1965-082DX','deb Transtage 4','',11832,'Reentered','1999-12-18 00:00:00','1992-01-31 00:00:00',95.78,538,575,32.05,'Intermediate' from dual union all
select 18320,'018320','1987-070C','UM129A','UM129A',60160,'Reentered','1988-06-12 00:00:00','1987-09-26 00:00:00',649.3,228,36693,27.84,'GEO Transfer' from dual union all
select 18321,'018321','1987-069C','KDU','KDU',60159,'Reentered','1987-09-10 00:00:00','1987-08-30 00:00:00',89.13,218,245,72.86,'Intermediate' from dual union all
select 18322,'018322','1987-069D','Blenda','Sunshade',60159,'Reentered','1987-09-01 00:00:00','1987-08-30 00:00:00',88.96,210,237,72.86,'Intermediate' from dual union all
select 18323,'018323','1987-069E','KDU part','KDU part',60159,'Reentered','1987-08-31 00:00:00','1987-08-30 00:00:00',88.9,209,232,72.87,'Intermediate' from dual union all
select 18324,'018324','1987-069F','KDU part','KDU part',60159,'Reentered','1987-09-07 00:00:00','1987-08-31 00:00:00',90.83,243,388,72.86,'Intermediate' from dual union all
select 18325,'018325','1987-069G','KDU part','KDU part',60159,'Reentered','1987-09-01 00:00:00','1987-08-31 00:00:00',90.2,230,339,72.88,'Intermediate' from dual union all
select 18326,'018326','1987-072A','Kosmos-1874','Oblik No. 365',60162,'Landed','1987-09-17 00:00:00','1987-09-05 00:00:00',89.62,225,287,72.86,'Intermediate' from dual union all
select 18327,'018327','1987-072B','11S510','11S510',60162,'Reentered','1987-09-11 00:00:00','1987-09-05 00:00:00',89.26,189,287,72.86,'Intermediate' from dual union all
select 18328,'018328','1987-073A','Ekran','Ekran No. 29L',60163,'In Earth orbit','','1987-10-03 00:00:00',1435.88,35769,35796,.37,'Stationary' from dual union all
select 18329,'018329','1987-073B','8S812','8S812',60163,'Reentered','1987-09-06 00:00:00','1987-09-05 00:00:00',88.13,179,185,51.62,'Intermediate' from dual union all
select 1833,'001833','1965-082DY','deb Transtage 4','',11832,'In Earth orbit','','1992-01-31 00:00:00',100.31,692,855,32.23,'Intermediate' from dual union all
select 18330,'018330','1987-073C','Perekhodnik','Sredniy perekhodnik',60163,'Reentered','1987-09-04 00:00:00','1987-09-04 00:00:00',88.07,174,184,51.6,'Intermediate' from dual union all
select 18331,'018331','1987-073D','Blok DM','11S86 No. 65L',60163,'In Earth orbit','','1987-10-23 00:00:00',1420.3,35395,35559,.37,'Drift GEO' from dual union all
select 18332,'018332','1987-073E','SOZ','SOZ',60163,'Reentered','2003-01-26 00:00:00','1988-02-14 00:00:00',621.03,184,35284,46.81,'GEO Transfer' from dual union all
select 18333,'018333','1987-073F','SOZ','SOZ',60163,'Reentered','1988-07-05 00:00:00','1987-10-03 00:00:00',627.12,252,35531,47.29,'GEO Transfer' from dual union all
select 18334,'018334','1987-074A','Kosmos-1875','Strela-3',60164,'In Earth orbit','','1987-10-14 00:00:00',113.73,1387,1409,82.57,'Intermediate' from dual union all
select 18335,'018335','1987-074B','Kosmos-1876','Strela-3',60164,'In Earth orbit','','1987-10-11 00:00:00',114.05,1410,1415,82.58,'Intermediate' from dual union all
select 18336,'018336','1987-074C','Kosmos-1877','Strela-3',60164,'In Earth orbit','','1987-10-09 00:00:00',113.99,1409,1410,82.57,'Intermediate' from dual union all
select 18337,'018337','1987-074D','Kosmos-1878','Strela-3',60164,'In Earth orbit','','1987-10-10 00:00:00',113.91,1402,1409,82.57,'Intermediate' from dual union all
select 18338,'018338','1987-074E','Kosmos-1879','Strela-3',60164,'In Earth orbit','','1987-10-10 00:00:00',113.85,1397,1410,82.58,'Intermediate' from dual union all
select 18339,'018339','1987-074F','Kosmos-1880','Strela-3',60164,'In Earth orbit','','1987-10-07 00:00:00',113.81,1393,1410,82.57,'Intermediate' from dual union all
select 1834,'001834','1965-082DZ','deb Transtage 4','',11832,'Reentered','1967-07-10 00:00:00','1967-07-02 00:00:00',91.26,315,357,33.01,'Intermediate' from dual union all
select 18340,'018340','1987-074G','S5M','S5M',60164,'In Earth orbit','','1987-10-31 00:00:00',114.69,1409,1474,82.57,'Intermediate' from dual union all
select 18341,'018341','1987-075A','Fanhui Shi Weixing (10)','Jianbing-1A No. 1',60165,'Reentered','1987-10-04 00:00:00','1987-09-16 00:00:00',89.51,204,297,62.99,'Intermediate' from dual union all
select 18342,'018342','1987-075B','CZ-2C Stage 2','CZ-2C Stage 2',60165,'Reentered','1987-09-24 00:00:00','1987-09-16 00:00:00',89.12,198,264,62.99,'Intermediate' from dual union all
select 18343,'018343','1987-076A','Kosmos-1881','Terilen',60166,'Deorbited','1988-03-30 00:00:00','1987-10-11 00:00:00',89.5,229,271,64.76,'Intermediate' from dual union all
select 18344,'018344','1987-076B','11S510','11S510',60166,'Reentered','1987-09-15 00:00:00','1987-09-13 00:00:00',88.54,168,237,64.76,'Intermediate' from dual union all
select 18345,'018345','1986-102C','Kosmos-1810 AO part','Terilen part',59295,'Reentered','1987-09-11 00:00:00','1987-09-11 00:00:00',89.58,177,331,64.77,'Intermediate' from dual union all
select 18346,'018346','1986-102D','Kosmos-1810 AO part','Terilen part',59295,'Reentered','1987-09-12 00:00:00','1987-09-11 00:00:00',89.23,207,266,64.7,'Intermediate' from dual union all
select 18347,'018347','1986-102E','Kosmos-1810 AO part','Terilen part',59295,'Reentered','1987-09-12 00:00:00','1987-09-12 00:00:00',87.74,142,184,64.7,'Intermediate' from dual union all
select 18348,'018348','1987-077A','Kosmos-1882','Resurs-F1 14F40 No. 107',60167,'Landed','1987-10-06 00:00:00','1987-09-22 00:00:00',89.82,258,274,82.33,'Intermediate' from dual union all
select 18349,'018349','1987-077B','11S510','11S510',60167,'Reentered','1987-09-17 00:00:00','1987-09-17 00:00:00',87.87,158,181,82.32,'Intermediate' from dual union all
select 1835,'001835','1965-082EA','deb Transtage 4','',11832,'In Earth orbit','','1992-01-31 00:00:00',97.93,642,678,32.39,'Intermediate' from dual union all
select 18350,'018350','1987-078A','Aussat K3','Aussat K3',60168,'In Earth orbit','','1987-10-17 00:00:00',1436.11,35779,35795,.02,'Stationary' from dual union all
select 18351,'018351','1987-078B','Eutelsat I F4','ECS 4',60168,'In Earth orbit','','1987-10-23 00:00:00',1436.04,35763,35808,.08,'Stationary' from dual union all
select 18352,'018352','1987-078C','Ariane H10 V19','Ariane H10 S/N T19',60168,'Reentered','1991-06-21 00:00:00','1987-10-18 00:00:00',643.57,246,36382,6.87,'GEO Transfer' from dual union all
select 18353,'018353','1987-078D','Sylda 4400 No. 7','Sylda 4400',60168,'Reentered','1988-10-08 00:00:00','1987-10-18 00:00:00',602.1,226,34257,6.9,'GEO Transfer' from dual union all
select 18354,'018354','1986-059C','deb Kosmos-1769','',59252,'Reentered','1987-10-10 00:00:00','1987-09-30 00:00:00',92,297,449,65.03,'Intermediate' from dual union all
select 18355,'018355','1987-079A','Kosmos-1883','Uragan No. 33L',60169,'In Earth orbit','','1987-10-16 00:00:00',675.71,19109,19150,64.87,'Medium' from dual union all
select 18356,'018356','1987-079B','Kosmos-1884','Uragan No. 34L',60169,'In Earth orbit','','1987-10-16 00:00:00',675.71,19109,19150,64.89,'Medium' from dual union all
select 18357,'018357','1987-079C','Kosmos-1885','Uragan No. 35L',60169,'In Earth orbit','','1987-10-18 00:00:00',675.69,19106,19152,64.89,'Medium' from dual union all
select 18358,'018358','1987-079D','Perekhodnik','Sredniy perekhodnik',60169,'Reentered','1987-09-16 00:00:00','1987-09-16 00:00:00',87.29,136,144,64.82,'Intermediate' from dual union all
select 18359,'018359','1987-079E','8S812','8S812',60169,'Reentered','1987-09-16 00:00:00','1987-09-16 00:00:00',86.96,118,129,64.83,'Intermediate' from dual union all
select 1836,'001836','1965-082EB','deb Transtage 4','',11832,'Reentered','1993-04-17 00:00:00','1992-01-31 00:00:00',94.52,482,509,32.21,'Intermediate' from dual union all
select 18360,'018360','1987-079F','Blok DM2','11S861 No. 26L',60169,'In Earth orbit','','1991-09-03 00:00:00',674.72,19073,19136,65.76,'Medium' from dual union all
select 18361,'018361','1987-080A','NNS O-27','NNS O-27',60170,'In Earth orbit','','1987-11-16 00:00:00',107.26,1017,1182,90.32,'Polar' from dual union all
select 18362,'018362','1987-080B','NNS O-29','NNS O-29',60170,'In Earth orbit','','1987-10-18 00:00:00',107.27,1015,1186,90.32,'Polar' from dual union all
select 18363,'018363','1987-080C','Star 20 E-43','Star 20 E-43',60170,'In Earth orbit','','1987-10-23 00:00:00',107.28,1016,1186,90.32,'Polar' from dual union all
select 18364,'018364','1987-078E','deb Ariane V19','',60168,'Reentered','1998-03-20 00:00:00','1997-08-10 00:00:00',308.09,193,17271,7.22,'Highly Ellip' from dual union all
select 18365,'018365','1987-080E','SOOS 2 adapter','',60170,'In Earth orbit','','1987-10-20 00:00:00',107.22,1016,1180,90.31,'Polar' from dual union all
select 18366,'018366','1987-081A','Kosmos-1886','Kobal''t No. 853',60171,'Landed','1987-11-02 00:00:00','1987-09-24 00:00:00',89.29,160,319,67.14,'Intermediate' from dual union all
select 18367,'018367','1987-081B','11S510','11S510',60171,'Reentered','1987-09-22 00:00:00','1987-09-19 00:00:00',89.28,162,316,67.14,'Intermediate' from dual union all
select 18368,'018368','1987-072C','KDU','KDU',60162,'Reentered','1987-10-12 00:00:00','1987-09-17 00:00:00',89.53,223,280,72.86,'Intermediate' from dual;
insert into SATELLITE
select 18369,'018369','1987-072D','KDU part','KDU part',60162,'Reentered','1987-09-19 00:00:00','1987-09-17 00:00:00',91.13,229,431,72.87,'Intermediate' from dual union all
select 1837,'001837','1965-082EC','deb Transtage 4','',11832,'Reentered','1968-05-01 00:00:00','1968-04-24 00:00:00',91.39,337,348,32.2,'Intermediate' from dual union all
select 18370,'018370','1987-072E','Blenda','Sunshade',60162,'Reentered','1987-09-20 00:00:00','1987-09-17 00:00:00',89.37,218,269,72.85,'Intermediate' from dual union all
select 18371,'018371','1987-072F','KDU part','[UNID113]',60162,'Reentered','1987-09-25 00:00:00','1987-09-18 00:00:00',91.35,236,446,72.84,'Intermediate' from dual union all
select 18372,'018372','1987-075C','FSW RV part','',60165,'Reentered','1987-09-23 00:00:00','1987-09-18 00:00:00',89.54,197,306,62.94,'Intermediate' from dual union all
select 18373,'018373','1987-075D','FSW RV part','',60165,'Reentered','1987-09-20 00:00:00','1987-09-18 00:00:00',88.76,183,243,63,'Intermediate' from dual union all
select 18374,'018374','1987-079G','SOZ','SOZ',60169,'In Earth orbit','','1987-10-19 00:00:00',340.14,407,19118,64.95,'Highly Ellip' from dual union all
select 18375,'018375','1987-079H','SOZ','SOZ',60169,'In Earth orbit','','1987-10-16 00:00:00',340.12,406,19118,64.83,'Highly Ellip' from dual union all
select 18376,'018376','1987-082A','Progress-32','Progress 7K-TG No. 139',60172,'Deorbited','1987-11-19 00:00:00','1987-10-02 00:00:00',91,298,350,51.64,'Intermediate' from dual union all
select 18377,'018377','1987-082B','11S510','11S510',60172,'Reentered','1987-09-26 00:00:00','1987-09-25 00:00:00',88.56,181,226,51.61,'Intermediate' from dual union all
select 18378,'018378','1983-044EG','deb Kosmos-1461','',55801,'Reentered','1989-07-02 00:00:00','1986-10-24 00:00:00',100.34,661,889,65.04,'Intermediate' from dual union all
select 18379,'018379','1983-044EH','deb Kosmos-1461','',55801,'Reentered','1989-06-04 00:00:00','1988-02-29 00:00:00',99.91,625,884,65.06,'Intermediate' from dual union all
select 1838,'001838','1965-102B','Fairing','Fairing S/N 029-MT13',11852,'Reentered','1965-12-15 00:00:00','1965-12-11 00:00:00',90.62,180,430,80,'Intermediate' from dual union all
select 18380,'018380','1987-083A','Kosmos-1887','Bion No. 8',60173,'Landed','1987-10-12 00:00:00','1987-10-01 00:00:00',90.49,215,383,62.82,'Intermediate' from dual union all
select 18381,'018381','1987-083B','11S510','11S510',60173,'Reentered','1987-10-19 00:00:00','1987-10-06 00:00:00',90.05,208,346,62.8,'Intermediate' from dual union all
select 18382,'018382','1986-059D','deb Kosmos-1769','',59252,'Reentered','1987-12-14 00:00:00','1987-09-30 00:00:00',92.01,307,439,65.01,'Intermediate' from dual union all
select 18383,'018383','1986-059E','deb Kosmos-1769','',59252,'Reentered','1987-10-07 00:00:00','1987-09-29 00:00:00',90.97,285,359,65.02,'Intermediate' from dual union all
select 18384,'018384','1987-084A','Kosmos-1888','Geizer No. 15L',60174,'In Earth orbit','','1987-10-31 00:00:00',1436.01,35764,35806,1.38,'Stationary' from dual union all
select 18385,'018385','1987-084B','8S812','8S812',60174,'Reentered','1987-10-04 00:00:00','1987-10-03 00:00:00',87.97,171,177,51.62,'Intermediate' from dual union all
select 18386,'018386','1987-084C','Perekhodnik','Sredniy perekhodnik',60174,'Reentered','1987-10-02 00:00:00','1987-10-02 00:00:00',88.28,182,197,51.61,'Intermediate' from dual union all
select 18387,'018387','1987-084D','Blok DM2','11S861 No. 14L',60174,'In Earth orbit','','1987-11-01 00:00:00',1439.59,35744,35966,1.41,'Drift GEO' from dual union all
select 18388,'018388','1987-084E','SOZ','SOZ',60174,'Reentered','1991-07-16 00:00:00','1987-11-03 00:00:00',635.16,243,35954,47.33,'GEO Transfer' from dual union all
select 18389,'018389','1987-077C','Blenda','Sunshade',60167,'Reentered','1987-10-17 00:00:00','1987-10-06 00:00:00',91.52,264,435,82.32,'Intermediate' from dual union all
select 1839,'001839','1965-104A','Gemini VI-A','Gemini SC6',11854,'Landed','1965-12-16 00:00:00','1965-12-15 00:00:00',89.95,270,273,28.9,'Intermediate' from dual union all
select 18390,'018390','1987-077D','KDU part','KDU part',60167,'Reentered','1987-10-12 00:00:00','1987-10-06 00:00:00',89.51,234,267,82.32,'Intermediate' from dual union all
select 18391,'018391','1987-077E','KDU part','KDU part',60167,'Reentered','1987-10-10 00:00:00','1987-10-07 00:00:00',89.58,245,263,82.32,'Intermediate' from dual union all
select 18392,'018392','1987-077F','KDU','KDU',60167,'Reentered','1987-11-05 00:00:00','1987-10-07 00:00:00',89.7,251,269,82.32,'Intermediate' from dual union all
select 18393,'018393','1987-077G','KDU part','KDU part',60167,'Reentered','1987-10-11 00:00:00','1987-10-07 00:00:00',91.42,261,428,82.35,'Intermediate' from dual union all
select 18394,'018394','1987-085A','Kosmos-1889','Oblik',60175,'Landed','1987-10-23 00:00:00','1987-10-12 00:00:00',92.17,348,414,69.98,'Intermediate' from dual union all
select 18395,'018395','1987-085B','11S510','11S510',60175,'Reentered','1987-10-25 00:00:00','1987-10-16 00:00:00',89.6,196,314,69.99,'Intermediate' from dual union all
select 18396,'018396','1987-086A','Kosmos-1890','US-PM',60176,'Reentered','1988-12-26 00:00:00','1987-11-09 00:00:00',92.77,404,417,65.01,'Intermediate' from dual union all
select 18397,'018397','1987-086B','11S692','11S692',60176,'Reentered','1987-10-11 00:00:00','1987-10-10 00:00:00',89.37,112,375,64.99,'Intermediate' from dual union all
select 18398,'018398','1987-083C','KB electropitaniya','KBE',60173,'Reentered','1987-11-08 00:00:00','1987-10-12 00:00:00',90.36,214,371,62.81,'Intermediate' from dual union all
select 18399,'018399','1987-083D','KBE part','KBE part',60173,'Reentered','1987-11-05 00:00:00','1987-10-12 00:00:00',92.37,323,459,62.83,'Intermediate' from dual union all
select 184,'000184','61 A Alpha 2','Atlas D 88D','Atlas D 88D',7249,'Reentered','1961-09-13 00:00:00','1961-09-13 00:00:00',88.31,148,234,32.54,'Intermediate' from dual union all
select 1840,'001840','1965-104B','Titan GT-6 Stage 2','LR91AJ5 GT-6 65-12561',11854,'Reentered','1965-12-16 00:00:00','1965-12-15 00:00:00',88.6,161,250,28.89,'Intermediate' from dual union all
select 18400,'018400','1987-083E','KBE part','KBE part',60173,'Reentered','1987-10-21 00:00:00','1987-10-12 00:00:00',90.15,216,348,62.82,'Intermediate' from dual union all
select 18401,'018401','1987-083F','KBE part','KBE part',60173,'Reentered','1987-10-28 00:00:00','1987-10-13 00:00:00',92.65,328,480,62.92,'Intermediate' from dual union all
select 18402,'018402','1987-087A','Kosmos-1891','Parus',60177,'In Earth orbit','','1987-11-15 00:00:00',104.9,952,1028,82.92,'Intermediate' from dual union all
select 18403,'018403','1987-087B','S3M','S3M',60177,'In Earth orbit','','1987-11-16 00:00:00',104.75,938,1028,82.92,'Intermediate' from dual union all
select 18404,'018404','1986-019SK','deb Ariane V16','',59212,'Reentered','1988-10-05 00:00:00','1987-07-28 00:00:00',100.44,728,830,98.7,'Sun-Synch' from dual union all
select 18405,'018405','1986-019SL','deb Ariane V16','',59212,'Reentered','1988-06-01 00:00:00','1987-10-15 00:00:00',99.66,726,759,98.88,'Sun-Synch' from dual union all
select 18406,'018406','1986-019SM','deb Ariane V16','',59212,'Reentered','1989-01-01 00:00:00','1987-10-16 00:00:00',101.58,792,875,99.01,'Sun-Synch' from dual union all
select 18407,'018407','1987-012E','Ginga interstage part','',60102,'Reentered','1989-08-11 00:00:00','1987-10-15 00:00:00',96.46,493,686,31.32,'Intermediate' from dual union all
select 18408,'018408','1987-012F','Ginga interstage part','',60102,'Reentered','1988-11-22 00:00:00','1987-08-12 00:00:00',96.49,522,659,30.96,'Intermediate' from dual union all
select 18409,'018409','1987-012G','Ginga interstage part','',60102,'Reentered','1989-02-11 00:00:00','1987-06-08 00:00:00',96.44,507,670,31.08,'Intermediate' from dual union all
select 1841,'001841','1965-105A','Pioneer 6','Pioneer A',11855,'Deep Space','1965-12-18 00:00:00','1965-12-16 00:00:00',8627.34,558,291996,30.23,'Earth Escape' from dual union all
select 18410,'018410','1987-041C','Sep Motor Cover','Sep Motor Cover',60131,'In Earth orbit','','1987-08-12 00:00:00',105.09,844,1154,70.99,'Intermediate' from dual union all
select 18411,'018411','1987-041D','Sep Motor Cover','Sep Motor Cover',60131,'In Earth orbit','','1987-10-15 00:00:00',104.91,845,1136,71.04,'Intermediate' from dual union all
select 18412,'018412','1987-041E','Sep Motor Cover','Sep Motor Cover',60131,'In Earth orbit','','1987-10-15 00:00:00',104.91,841,1140,71.03,'Intermediate' from dual union all
select 18413,'018413','1982-055BC','deb Kosmos-1375','',54364,'In Earth orbit','','1991-10-07 00:00:00',105.96,977,1102,65.83,'Intermediate' from dual union all
select 18414,'018414','1982-055BD','deb Kosmos-1375','',54364,'In Earth orbit','','1991-10-01 00:00:00',104.77,976,991,65.83,'Intermediate' from dual union all
select 18415,'018415','1982-055BE','deb Kosmos-1375','',54364,'In Earth orbit','','1991-10-02 00:00:00',105.8,978,1086,65.84,'Intermediate' from dual union all
select 18416,'018416','1987-027C','Sep Motor Cover','Sep Motor Cover',60117,'In Earth orbit','','1987-10-16 00:00:00',104.82,845,1126,71.03,'Intermediate' from dual union all
select 18417,'018417','1987-027D','Sep Motor Cover','Sep Motor Cover',60117,'In Earth orbit','','1987-08-14 00:00:00',105.06,842,1152,70.99,'Intermediate' from dual union all
select 18418,'018418','1970-025PL','deb Agena D','',21502,'In Earth orbit','','1991-11-18 00:00:00',106.2,1049,1052,100.14,'Sun-Synch' from dual union all
select 18419,'018419','1970-025PM','deb Agena D','',21502,'Reentered','1992-01-20 00:00:00','1992-01-19 00:00:00',92.29,363,411,100.48,'Retrograde' from dual union all
select 1842,'001842','1965-105B','Delta 35','DSV-3E-3 S/N 20202',11855,'Reentered','1968-09-06 00:00:00','1966-01-30 00:00:00',100.24,273,1267,30.17,'Intermediate' from dual union all
select 18420,'018420','1987-084F','SOZ','SOZ',60174,'Reentered','1988-06-06 00:00:00','1987-11-09 00:00:00',636.52,294,35973,47.32,'GEO Transfer' from dual union all
select 18421,'018421','1987-088A','Kosmos-1892','Tselina-D',60178,'In Earth orbit','','1987-11-19 00:00:00',97.7,633,665,82.51,'Intermediate' from dual union all
select 18422,'018422','1987-088B','S5M','S5M',60178,'In Earth orbit','','1987-11-19 00:00:00',97.68,632,664,82.51,'Intermediate' from dual union all
select 18423,'018423','1979-017LD','deb P78-1','',47501,'Reentered','1989-03-26 00:00:00','1987-10-20 00:00:00',95.02,512,528,97.72,'Sun-Synch' from dual union all
select 18424,'018424','1979-017LE','deb P78-1','',47501,'Reentered','1992-09-21 00:00:00','1991-11-13 00:00:00',92.24,377,392,97.86,'Sun-Synch' from dual union all
select 18425,'018425','1979-017LF','deb P78-1','',47501,'Reentered','1988-12-01 00:00:00','1987-10-20 00:00:00',94.74,491,521,97.75,'Sun-Synch' from dual union all
select 18426,'018426','1986-088C','P87-1 despin weight','',59281,'In Earth orbit','','1986-12-17 00:00:00',105.27,954,1061,89.13,'Polar' from dual union all
select 18427,'018427','1965-027AB','deb Snapshot','',11777,'In Earth orbit','','1986-04-14 00:00:00',111.38,1266,1314,90.28,'Polar' from dual union all
select 18428,'018428','1976-067BM','deb Kosmos-839','',39019,'In Earth orbit','','1991-10-09 00:00:00',117.94,906,2271,65.72,'Intermediate' from dual union all
select 18429,'018429','1976-077EZ','deb Delta 126','',39029,'In Earth orbit','','1991-09-28 00:00:00',119.47,1473,1842,102.78,'Sun-Synch' from dual union all
select 1843,'001843','1965-106A','Kosmos-100','Meteor 11F614 No. 3',11856,'Reentered','2002-02-15 00:00:00','1966-01-23 00:00:00',97.57,629,655,65,'Intermediate' from dual union all
select 18430,'018430','1985-082Z','deb Kosmos-1682','',58305,'Reentered','1988-10-12 00:00:00','1987-10-20 00:00:00',92.86,354,475,65.01,'Intermediate' from dual union all
select 18431,'018431','1968-097EJ','deb Kosmos-252','',16824,'In Earth orbit','','1991-11-03 00:00:00',110.24,540,1936,62.33,'Intermediate' from dual union all
select 18432,'018432','1987-089A','Kosmos-1893','Kobal''t No. 514',60179,'Landed','1987-12-16 00:00:00','1987-10-29 00:00:00',89.15,159,306,67.16,'Intermediate' from dual union all
select 18433,'018433','1987-089B','11S510','11S510',60179,'Reentered','1987-10-26 00:00:00','1987-10-24 00:00:00',88.9,159,282,67.15,'Intermediate' from dual union all
select 18434,'018434','1986-067U','Kosmos-1776 SS 13','ESO',59260,'Reentered','1988-09-20 00:00:00','1987-10-23 00:00:00',94.44,472,511,74.03,'Intermediate' from dual union all
select 18435,'018435','1986-067V','Kosmos-1776 SS 14','ESO',59260,'Reentered','1988-09-14 00:00:00','1987-10-23 00:00:00',94.41,471,509,74.03,'Intermediate' from dual union all
select 18436,'018436','1987-085C','KDU part','KDU part',60175,'Reentered','1988-02-28 00:00:00','1987-11-08 00:00:00',93.97,372,566,69.98,'Intermediate' from dual union all
select 18437,'018437','1987-085D','Blenda','Sunshade',60175,'Reentered','1988-01-31 00:00:00','1987-11-08 00:00:00',91.8,329,397,69.98,'Intermediate' from dual union all
select 18438,'018438','1987-085E','KDU part','KDU part',60175,'Reentered','1987-11-30 00:00:00','1987-10-23 00:00:00',94.47,376,610,69.98,'Intermediate' from dual union all
select 18439,'018439','1987-085F','KDU part','KDU part',60175,'Reentered','1987-12-23 00:00:00','1987-11-08 00:00:00',91.81,336,390,69.98,'Intermediate' from dual union all
select 1844,'001844','1965-106B','8A92E','8A92E',11856,'Reentered','1999-11-29 00:00:00','1966-01-31 00:00:00',97.71,562,737,65.01,'Intermediate' from dual union all
select 18440,'018440','1987-085G','KDU','KDU',60175,'Reentered','1988-10-10 00:00:00','1987-11-08 00:00:00',92.11,346,410,69.98,'Intermediate' from dual union all
select 18441,'018441','1987-090A','USA 27','Enhanced CRYSTAL 2102',60180,'Deorbited','1992-06-01 00:00:00','1989-07-12 00:00:00',97.33,277,985,97.91,'Sun-Synch' from dual union all
select 18442,'018442','1987-090B','Titan 34D-15 Stage 2','LR91AJ11 34D-15',60180,'Reentered','1987-10-27 00:00:00','1987-10-26 00:00:00',96.28,143,1018,97.8,'Sun-Synch' from dual union all
select 18443,'018443','1987-091A','Kosmos-1894','Oko-S F3',60181,'In Earth orbit','','1987-11-27 00:00:00',1449.36,35917,36175,1.21,'Drift GEO' from dual union all
select 18444,'018444','1987-091B','8S812','8S812',60181,'Reentered','1987-10-31 00:00:00','1987-10-30 00:00:00',87.91,168,175,51.62,'Intermediate' from dual union all
select 18445,'018445','1987-091C','Perekhodnik','Sredniy perekhodnik',60181,'Reentered','1987-10-29 00:00:00','1987-10-28 00:00:00',88.29,182,198,51.6,'Intermediate' from dual union all
select 18446,'018446','1987-091D','Blok DM2','11S861 No. 28L',60181,'In Earth orbit','','1987-11-28 00:00:00',1437.93,35727,35918,1.23,'Drift GEO' from dual union all
select 18447,'018447','1987-091E','SOZ','SOZ',60181,'Reentered','1988-03-09 00:00:00','1987-11-27 00:00:00',634.75,255,35921,47.19,'GEO Transfer' from dual union all
select 18448,'018448','1987-091F','SOZ','SOZ',60181,'Reentered','1988-09-06 00:00:00','1987-11-29 00:00:00',636.08,306,35938,47.24,'GEO Transfer' from dual union all
select 18449,'018449','1987-081C','deb Kosmos- 1886','',60171,'Reentered','1987-11-02 00:00:00','1987-11-02 00:00:00',88.82,169,264,67.12,'Intermediate' from dual union all
select 1845,'001845','1965-100C','Gemini SC7 Equipment Module','Gemini SC7 Adapter Modul',11850,'Reentered','1966-01-27 00:00:00','1965-12-18 00:00:00',90.48,293,304,28.9,'Intermediate' from dual union all
select 18450,'018450','1986-019SN','deb Ariane V16','',59212,'Reentered','1989-02-22 00:00:00','1987-11-04 00:00:00',101.26,768,868,99.36,'Sun-Synch' from dual union all
select 18451,'018451','1986-019SP','deb Ariane V16','',59212,'Reentered','1988-12-29 00:00:00','1987-11-04 00:00:00',100.8,790,802,98.73,'Sun-Synch' from dual union all
select 18452,'018452','1986-019SQ','deb Ariane V16','',59212,'Reentered','1989-04-28 00:00:00','1987-11-04 00:00:00',101.05,769,848,98.56,'Sun-Synch' from dual union all
select 18453,'018453','1986-019SR','deb Ariane V16','',59212,'Reentered','1989-06-29 00:00:00','1987-11-04 00:00:00',101.05,769,848,98.56,'Sun-Synch' from dual union all
select 18454,'018454','1986-019SS','deb Ariane V16','',59212,'Reentered','1991-04-01 00:00:00','1987-11-04 00:00:00',100.94,779,827,98.52,'Sun-Synch' from dual union all
select 18455,'018455','1986-019ST','deb Ariane V16','',59212,'Reentered','1989-04-12 00:00:00','1987-11-04 00:00:00',100.1,729,798,98.79,'Sun-Synch' from dual union all
select 18456,'018456','1986-019SU','deb Ariane V16','',59212,'Reentered','1989-03-23 00:00:00','1987-11-04 00:00:00',102.05,771,941,98.73,'Sun-Synch' from dual union all
select 18457,'018457','1986-019SV','deb Ariane V16','',59212,'Reentered','1989-02-05 00:00:00','1987-11-04 00:00:00',101.09,790,831,98.68,'Sun-Synch' from dual union all
select 18458,'018458','1986-019SW','deb Ariane V16','',59212,'Reentered','1988-10-27 00:00:00','1987-11-04 00:00:00',100.38,742,812,98.82,'Sun-Synch' from dual;
insert into SATELLITE
select 18459,'018459','1986-019SX','deb Ariane V16','',59212,'Reentered','1989-10-11 00:00:00','1987-11-04 00:00:00',101.01,775,837,98.54,'Sun-Synch' from dual union all
select 1846,'001846','1965-107A','Kosmos-101','DS-P1-Yu No. 4',11857,'Reentered','1966-07-12 00:00:00','1966-01-30 00:00:00',92.2,255,510,48.8,'Intermediate' from dual union all
select 18460,'018460','1986-019SY','deb Ariane V16','',59212,'Reentered','1989-05-14 00:00:00','1987-11-04 00:00:00',101.39,782,867,98.68,'Sun-Synch' from dual union all
select 18461,'018461','1986-019SZ','deb Ariane V16','',59212,'Reentered','1999-09-22 00:00:00','1991-10-10 00:00:00',99.64,710,773,98.56,'Sun-Synch' from dual union all
select 18462,'018462','1986-019TA','deb Ariane V16','',59212,'Reentered','1989-01-31 00:00:00','1987-11-04 00:00:00',101.7,833,846,99.08,'Sun-Synch' from dual union all
select 18463,'018463','1976-077FA','deb Delta 126','',39029,'In Earth orbit','','1991-10-10 00:00:00',117.74,1502,1657,102.44,'Sun-Synch' from dual union all
select 18464,'018464','1979-017LG','deb P78-1','[UNID114] RCS 0.0',47501,'Reentered','1990-03-21 00:00:00','1987-11-04 00:00:00',97.89,630,685,97.75,'Sun-Synch' from dual union all
select 18465,'018465','1969-082LA','deb Agena D','',18349,'Reentered','2012-01-24 00:00:00','1991-10-31 00:00:00',99.96,727,787,70.04,'Intermediate' from dual union all
select 18466,'018466','1976-077FB','deb Delta 126','[UNID115] RCS 0.02',39029,'Reentered','2014-01-14 00:00:00','1991-10-04 00:00:00',115.66,1456,1515,103.34,'Sun-Synch' from dual union all
select 18467,'018467','1978-026HE','deb Delta 139','',45221,'In Earth orbit','','1991-10-03 00:00:00',101.72,765,915,98.7,'Sun-Synch' from dual union all
select 18468,'018468','1968-091DD','deb Kosmos-249','',16818,'In Earth orbit','','1991-08-31 00:00:00',113.49,123,2651,97.11,'Sun-Synch' from dual union all
select 18469,'018469','1978-026HF','deb Delta 139','',45221,'In Earth orbit','','1991-10-10 00:00:00',103.28,879,949,99.1,'Sun-Synch' from dual union all
select 1847,'001847','1965-107B','S1','S1',11857,'Reentered','1966-04-16 00:00:00','1966-01-31 00:00:00',91.83,251,478,48.77,'Intermediate' from dual union all
select 18470,'018470','1973-086GU','deb Delta 98','',30688,'In Earth orbit','','1991-10-04 00:00:00',119.34,1506,1796,101.76,'Sun-Synch' from dual union all
select 18471,'018471','1976-067BN','Unknown deb                       ','',39019,'In Earth orbit','','1991-10-02 00:00:00',118.68,818,2425,65.75,'Intermediate' from dual union all
select 18472,'018472','1973-086GV','deb Delta 98','',30688,'In Earth orbit','','1991-10-04 00:00:00',112.74,1341,1364,101.29,'Sun-Synch' from dual union all
select 18473,'018473','1983-044EJ','deb Kosmos-1461','',55801,'Reentered','1989-04-13 00:00:00','1987-11-05 00:00:00',99.65,606,877,65.05,'Intermediate' from dual union all
select 18474,'018474','1983-044EK','deb Kosmos-1461','',55801,'Reentered','1990-08-28 00:00:00','1990-08-27 00:00:00',89.66,242,274,65.03,'Intermediate' from dual union all
select 18475,'018475','1975-004HK','deb Delta 107','',36451,'In Earth orbit','','1991-10-11 00:00:00',99.45,694,771,98.04,'Sun-Synch' from dual union all
select 18476,'018476','1987-041F','Sep Motor Cover','Sep Motor Cover',60131,'In Earth orbit','','1987-11-01 00:00:00',105.18,844,1162,71,'Intermediate' from dual union all
select 18477,'018477','1982-055BF','deb Kosmos-1375','',54364,'In Earth orbit','','1991-10-09 00:00:00',104.37,944,986,65.82,'Intermediate' from dual union all
select 18478,'018478','1968-097EK','deb Kosmos-252','',16824,'Reentered','2010-05-14 00:00:00','1990-09-10 00:00:00',105,501,1487,62.87,'Intermediate' from dual union all
select 18479,'018479','1965-027AC','deb SNAPSHOT','',11777,'In Earth orbit','','1987-11-01 00:00:00',111.46,1272,1316,90.31,'Polar' from dual union all
select 1848,'001848','1965-082EF','deb Transtage 4','',11832,'Reentered','1969-09-17 00:00:00','1966-01-19 00:00:00',98.72,656,739,32.87,'Intermediate' from dual union all
select 18480,'018480','1983-044EL','Kosmos-1461 major debris','',55801,'Reentered','2013-05-17 00:00:00','1991-10-09 00:00:00',98.28,570,783,65.02,'Intermediate' from dual union all
select 18481,'018481','1983-044EM','deb Kosmos-1461','',55801,'Reentered','1988-12-30 00:00:00','1987-11-07 00:00:00',95.86,467,654,65.05,'Intermediate' from dual union all
select 18482,'018482','1983-044EN','deb Kosmos-1461','',55801,'Reentered','1988-03-24 00:00:00','1987-11-06 00:00:00',95.85,466,653,65.08,'Intermediate' from dual union all
select 18483,'018483','1983-044EP','deb Kosmos-1461','',55801,'Reentered','1989-04-23 00:00:00','1987-11-17 00:00:00',98.78,556,845,65.02,'Intermediate' from dual union all
select 18484,'018484','1983-044EQ','deb Kosmos-1461','',55801,'Reentered','1988-10-18 00:00:00','1987-11-08 00:00:00',98.12,526,811,65.05,'Intermediate' from dual union all
select 18485,'018485','1983-044ER','deb Kosmos-1461','',55801,'Reentered','1989-01-19 00:00:00','1987-11-09 00:00:00',98.74,557,840,65.02,'Intermediate' from dual union all
select 18486,'018486','1979-017LH','deb P78-1','',47501,'Reentered','1989-10-30 00:00:00','1987-11-09 00:00:00',95.32,515,554,98.21,'Sun-Synch' from dual union all
select 18487,'018487','1979-017LJ','deb P78-1','',47501,'Reentered','1989-01-28 00:00:00','1987-11-05 00:00:00',94.99,499,538,98.18,'Sun-Synch' from dual union all
select 18488,'018488','1968-097EL','deb Kosmos-252','',16824,'Reentered','1990-07-19 00:00:00','1990-04-09 00:00:00',113.45,823,1947,62.14,'Intermediate' from dual union all
select 18489,'018489','1968-097EM','deb Kosmos-252','',16824,'Reentered','1990-01-03 00:00:00','1987-11-09 00:00:00',114.03,758,2065,62.24,'Intermediate' from dual union all
select 1849,'001849','1965-082EG','deb Transtage 4','',11832,'Reentered','1967-07-19 00:00:00','1966-01-01 00:00:00',97.72,600,700,32.5,'Intermediate' from dual union all
select 18490,'018490','1968-097EN','deb Kosmos-252','',16824,'Reentered','1990-07-25 00:00:00','1990-07-25 00:00:00',92.56,161,639,63.39,'Intermediate' from dual union all
select 18491,'018491','1987-092A','Kosmos-1895','Oblik',60182,'Landed','1987-11-26 00:00:00','1987-11-18 00:00:00',89.61,226,285,70.36,'Intermediate' from dual union all
select 18492,'018492','1987-092B','11S510','11S510',60182,'Reentered','1987-11-25 00:00:00','1987-11-13 00:00:00',90.12,204,357,70.37,'Intermediate' from dual union all
select 18493,'018493','1983-044ES','deb Kosmos-1461','',55801,'Reentered','1988-07-02 00:00:00','1987-11-12 00:00:00',97.64,520,772,65.06,'Intermediate' from dual union all
select 18494,'018494','1977-121BQ','deb Kosmos-970','',42249,'In Earth orbit','','1991-10-08 00:00:00',105.9,952,1121,65.79,'Intermediate' from dual union all
select 18495,'018495','1980-089CC','deb Kosmos-1220','',50299,'Reentered','1989-05-14 00:00:00','1987-11-12 00:00:00',97.73,538,763,64.99,'Intermediate' from dual union all
select 18496,'018496','1973-081D','NNS O-20 despin weight','',30683,'In Earth orbit','','1973-12-02 00:00:00',105.11,893,1107,89.89,'Polar' from dual union all
select 18497,'018497','1971-015DE','deb Kosmos-397','',24833,'Reentered','1989-11-11 00:00:00','1987-11-14 00:00:00',102.93,573,1221,66.24,'Intermediate' from dual union all
select 18498,'018498','61 Omicron 282','deb Ablestar','',7273,'Reentered','1989-11-21 00:00:00','1984-12-18 00:00:00',103.31,570,1261,66.22,'Intermediate' from dual union all
select 18499,'018499','1981-053LR','deb Kosmos-1275','',52576,'Reentered','1992-04-27 00:00:00','1991-10-10 00:00:00',98.05,628,703,83.18,'Intermediate' from dual union all
select 185,'000185','61 Omega 2','deb Discoverer XXX','',7272,'Reentered','1961-09-18 00:00:00','1961-09-15 00:00:00',90.7,219,399,82.58,'Intermediate' from dual union all
select 1850,'001850','1965-082EK','deb Transtage 4','',11832,'Reentered','1970-04-22 00:00:00','1966-01-28 00:00:00',99.23,669,775,32.67,'Intermediate' from dual union all
select 18500,'018500','1975-112E','deb Kosmos-783 [UNID116]','Strela-2M debris',36559,'In Earth orbit','','1985-02-21 00:00:00',101.1,798,823,74.06,'Intermediate' from dual union all
select 18501,'018501','1978-019E','deb Kosmos-990 [UNID]','Strela-2M debris',45214,'In Earth orbit','','1985-02-21 00:00:00',100.76,782,808,74.07,'Intermediate' from dual union all
select 18502,'018502','1982-051D','deb Kosmos (-1371) [UNID]','Strela-2M debris',54360,'In Earth orbit','','1985-03-17 00:00:00',101.07,809,809,74.08,'Intermediate' from dual union all
select 18503,'018503','61 Omicron 283','deb Ablestar','',7273,'Reentered','2015-01-01 00:00:00','1985-03-25 00:00:00',100.3,668,878,66.96,'Intermediate' from dual union all
select 18504,'018504','1976-098E','deb Kosmos-858 [UNID]','Strela-2M debris',39050,'In Earth orbit','','1985-03-26 00:00:00',100.55,768,802,74.06,'Intermediate' from dual union all
select 18505,'018505','1981-053LS','deb Kosmos-1275','',52576,'In Earth orbit','','1991-10-09 00:00:00',105.56,941,1101,83.3,'Intermediate' from dual union all
select 18506,'018506','1985-064E','deb Kosmos-1670 ROCh','',58287,'Reentered','1988-09-28 00:00:00','1986-02-07 00:00:00',103.89,824,1060,64.8,'Intermediate' from dual union all
select 18507,'018507','1982-055BG','deb Kosmos-1375','',54364,'In Earth orbit','','1991-10-02 00:00:00',105.43,989,1040,65.83,'Intermediate' from dual union all
select 18508,'018508','1974-089EY','deb Delta 104','',33692,'Reentered','1989-03-15 00:00:00','1987-11-15 00:00:00',109.24,741,1643,103.5,'Sun-Synch' from dual union all
select 18509,'018509','1982-051E','deb Kosmos (-1371) [UNID]','Strela-2M debris',54360,'In Earth orbit','','1986-09-13 00:00:00',101.04,797,819,74.02,'Intermediate' from dual union all
select 1851,'001851','1965-082EJ','deb Transtage 4','',11832,'Reentered','1966-09-29 00:00:00','1966-08-19 00:00:00',95.9,455,670,32.7,'Intermediate' from dual union all
select 18510,'018510','1982-051F','deb Kosmos (-1371) [UNID]','Strela-2M debris',54360,'In Earth orbit','','1986-09-18 00:00:00',101.08,794,826,74,'Intermediate' from dual union all
select 18511,'018511','1973-086GW','deb Delta 98','',30688,'Reentered','2002-01-25 00:00:00','1991-10-07 00:00:00',112.91,929,1792,102.68,'Sun-Synch' from dual union all
select 18512,'018512','1977-119E','deb Kosmos-968 [UNID]','Strela-2M debris',42247,'In Earth orbit','','1985-01-12 00:00:00',100.56,777,794,74.05,'Intermediate' from dual union all
select 18513,'018513','1970-025PN','deb Agena D','',21502,'In Earth orbit','','1991-11-18 00:00:00',103.57,873,982,99.8,'Sun-Synch' from dual union all
select 18514,'018514','1987-004GT','deb Kosmos-1813','',60094,'Reentered','1987-11-20 00:00:00','1987-11-14 00:00:00',89.13,220,243,72.79,'Intermediate' from dual union all
select 18515,'018515','1987-004GU','deb Kosmos-1813','',60094,'Reentered','1988-01-30 00:00:00','1987-11-13 00:00:00',91.33,304,376,72.8,'Intermediate' from dual union all
select 18516,'018516','1981-053LT','deb Kosmos-1275','',52576,'Reentered','1989-02-26 00:00:00','1987-11-11 00:00:00',101.03,793,822,82.98,'Intermediate' from dual union all
select 18517,'018517','1970-089DD','deb Kosmos-374','',21566,'In Earth orbit','','1991-07-17 00:00:00',108.9,587,1765,62.53,'Intermediate' from dual union all
select 18518,'018518','1976-126BW','deb Kosmos-886','',39078,'In Earth orbit','','1991-10-09 00:00:00',113.45,790,1980,65.88,'Intermediate' from dual union all
select 18519,'018519','1984-019D','deb Kosmos (-1538) [UNID]','Strela-2M debris',57073,'In Earth orbit','','1987-08-12 00:00:00',100.81,783,811,74.06,'Intermediate' from dual union all
select 1852,'001852','1965-082EH','deb Transtage 4','',11832,'Reentered','1968-01-17 00:00:00','1968-01-02 00:00:00',95.54,475,615,32.49,'Intermediate' from dual union all
select 18520,'018520','1983-044ET','deb Kosmos-1461','',55801,'Reentered','1989-06-16 00:00:00','1987-11-12 00:00:00',99.18,576,863,65.05,'Intermediate' from dual union all
select 18521,'018521','1971-015DF','deb Kosmos-397','',24833,'Reentered','1988-11-06 00:00:00','1987-11-14 00:00:00',95.71,380,726,66.03,'Intermediate' from dual union all
select 18522,'018522','1986-019TB','deb Ariane V16','',59212,'Reentered','1988-11-26 00:00:00','1987-11-14 00:00:00',99.4,672,788,98.9,'Sun-Synch' from dual union all
select 18523,'018523','1987-012H','Ginga interstage part','',60102,'Reentered','1988-02-03 00:00:00','1987-08-13 00:00:00',95.7,482,623,31.08,'Intermediate' from dual union all
select 18524,'018524','1987-060D','Kosmos-1867 fairing','',60150,'Reentered','1990-05-14 00:00:00','1987-11-03 00:00:00',100.73,787,800,65,'Intermediate' from dual union all
select 18525,'018525','1986-088D','P87-1 despin weight','',59281,'In Earth orbit','','1987-08-17 00:00:00',104.45,967,971,89.95,'Polar' from dual union all
select 18526,'018526','1987-049C','deb Kosmos-1850','',60139,'In Earth orbit','','1987-11-03 00:00:00',100.92,800,805,74.04,'Intermediate' from dual union all
select 18527,'018527','1987-027E','Sep Motor Cover','Sep Motor Cover',60117,'In Earth orbit','','1987-08-14 00:00:00',104.96,843,1143,70.99,'Intermediate' from dual union all
select 18528,'018528','1979-035F','SOZ','SOZ',47519,'Reentered','1979-12-06 00:00:00','1987-08-23 00:00:00',704.68,775,38932,58.37,'GEO Transfer' from dual union all
select 18529,'018529','1979-005C','deb Blok E','',47489,'Reentered','1988-08-25 00:00:00','1987-08-26 00:00:00',96.35,576,592,97.6,'Sun-Synch' from dual union all
select 1853,'001853','1965-082EL','deb Transtage 4','',11832,'Reentered','1966-11-29 00:00:00','1966-11-19 00:00:00',91.85,278,453,32.72,'Intermediate' from dual union all
select 18530,'018530','1987-080F','SOOS-2 despin weight','',60170,'In Earth orbit','','1987-10-21 00:00:00',106.59,1003,1135,90.38,'Polar' from dual union all
select 18531,'018531','1971-015DG','deb Kosmos-397','',24833,'Reentered','1989-09-23 00:00:00','1987-11-12 00:00:00',108.47,468,1844,65.82,'Intermediate' from dual union all
select 18532,'018532','1978-096D','deb Tiros N','',45291,'Reentered','1988-10-20 00:00:00','1987-11-12 00:00:00',101.64,810,863,99.05,'Sun-Synch' from dual union all
select 18533,'018533','1965-027AD','deb SNAPSHOT','',11777,'In Earth orbit','','1987-11-12 00:00:00',111.45,1271,1315,90.32,'Polar' from dual union all
select 18534,'018534','1965-027AE','deb SNAPSHOT','',11777,'In Earth orbit','','1987-11-12 00:00:00',111.51,1274,1319,90.32,'Polar' from dual union all
select 18535,'018535','1987-093A','Kosmos-1896','Kometa No. 8',60183,'Landed','1987-12-25 00:00:00','1987-11-21 00:00:00',89.15,205,260,64.8,'Intermediate' from dual union all
select 18536,'018536','1987-093B','11S510','11S510',60183,'Reentered','1987-11-20 00:00:00','1987-11-16 00:00:00',89.12,189,273,64.81,'Intermediate' from dual union all
select 18537,'018537','1987-093C','Kosmos-1896 part','Kometa part',60183,'Reentered','1987-11-15 00:00:00','1987-11-14 00:00:00',89.49,188,311,64.82,'Intermediate' from dual union all
select 18538,'018538','1965-027AF','deb SNAPSHOT','',11777,'In Earth orbit','','1991-11-11 00:00:00',111.34,1262,1314,90.29,'Polar' from dual union all
select 18539,'018539','1974-089EZ','deb Delta 104','',33692,'In Earth orbit','','1984-02-18 00:00:00',92.78,332,490,50.67,'Intermediate' from dual union all
select 1854,'001854','1965-082EM','deb Transtage 4','',11832,'Reentered','1966-04-27 00:00:00','1966-03-17 00:00:00',94.38,410,568,32.52,'Intermediate' from dual union all
select 18540,'018540','1976-077FC','deb Delta 126','',39029,'In Earth orbit','','1984-02-18 00:00:00',92.85,333,496,50.69,'Intermediate' from dual union all
select 18541,'018541','1986-019TC','deb Ariane V16','',59212,'Reentered','1989-06-30 00:00:00','1987-10-20 00:00:00',100.81,766,828,98.54,'Sun-Synch' from dual union all
select 18542,'018542','1981-028BW','deb Kosmos-1260','',52551,'Reentered','1988-12-12 00:00:00','1986-04-13 00:00:00',98.17,485,858,64.06,'Intermediate' from dual union all
select 18543,'018543','1963-014DJ','deb West Ford','RCS 0.14',9245,'In Earth orbit','','1987-10-24 00:00:00',163.11,962,6060,85,'Medium' from dual union all
select 18544,'018544','1983-044EU','deb Kosmos-1461','',55801,'Reentered','1989-05-23 00:00:00','1987-08-19 00:00:00',98.16,519,823,65.07,'Intermediate' from dual union all
select 18545,'018545','1986-100C','deb S3M','',59293,'In Earth orbit','','1987-10-23 00:00:00',104.86,969,1006,82.93,'Intermediate' from dual union all
select 18546,'018546','1979-017LK','deb P78-1','',47501,'Reentered','1989-03-13 00:00:00','1987-12-06 00:00:00',94.59,477,521,97.72,'Sun-Synch' from dual union all
select 18547,'018547','1970-025PP','deb Agena D','',21502,'Reentered','1999-08-05 00:00:00','1991-11-04 00:00:00',99.83,706,795,99.64,'Sun-Synch' from dual union all
select 18548,'018548','1971-015DH','deb Kosmos-397','',24833,'Reentered','1988-03-12 00:00:00','1987-12-08 00:00:00',110.09,566,1896,65.71,'Intermediate' from dual;
insert into SATELLITE
select 18549,'018549','1968-091DE','deb Kosmos-249','',16818,'In Earth orbit','','1991-10-29 00:00:00',113.45,776,1994,62.22,'Intermediate' from dual union all
select 1855,'001855','1965-082EN','deb Transtage 4','',11832,'Reentered','1974-09-12 00:00:00','1966-01-29 00:00:00',99.94,742,770,32.02,'Intermediate' from dual union all
select 18550,'018550','1987-027F','Sep Motor Cover','Sep Motor Cover',60117,'In Earth orbit','','1987-10-09 00:00:00',104.71,842,1120,71.03,'Intermediate' from dual union all
select 18551,'018551','1968-091DF','deb Kosmos-249','',16818,'In Earth orbit','','1991-03-12 00:00:00',110.8,607,1920,62.33,'Intermediate' from dual union all
select 18552,'018552','1986-086C','deb Kosmos- 1791','',59279,'In Earth orbit','','1986-12-14 00:00:00',104.58,948,1002,82.94,'Intermediate' from dual union all
select 18553,'018553','1986-019TD','deb Ariane V16','',59212,'Reentered','1989-06-03 00:00:00','1987-12-02 00:00:00',101.02,746,867,98.53,'Sun-Synch' from dual union all
select 18554,'018554','1986-019TE','deb Ariane V16','',59212,'Reentered','1988-08-01 00:00:00','1987-11-30 00:00:00',100.29,754,791,99.04,'Sun-Synch' from dual union all
select 18555,'018555','1986-019TF','deb Ariane V16','',59212,'Reentered','1988-03-19 00:00:00','1987-12-02 00:00:00',98.65,585,803,99.69,'Sun-Synch' from dual union all
select 18556,'018556','1968-097EP','deb Kosmos-252','',16824,'In Earth orbit','','1994-01-17 00:00:00',110.15,709,1758,62.29,'Intermediate' from dual union all
select 18557,'018557','1986-019TG','deb Ariane V16','',59212,'Reentered','1989-03-10 00:00:00','1987-12-03 00:00:00',105.05,785,1209,98.33,'Sun-Synch' from dual union all
select 18558,'018558','1986-019TH','deb Ariane V16','',59212,'Reentered','1989-05-19 00:00:00','1987-11-30 00:00:00',99.41,657,804,99.01,'Sun-Synch' from dual union all
select 18559,'018559','1986-019TJ','deb Ariane V16','',59212,'Reentered','1989-01-20 00:00:00','1987-12-02 00:00:00',100.98,789,822,98.78,'Sun-Synch' from dual union all
select 1856,'001856','1965-082EP','deb Transtage 4','',11832,'Reentered','1967-02-06 00:00:00','1966-01-16 00:00:00',101.09,569,1052,31.68,'Intermediate' from dual union all
select 18560,'018560','1982-055BH','deb Kosmos-1375','',54364,'In Earth orbit','','1991-10-05 00:00:00',105.36,985,1037,65.84,'Intermediate' from dual union all
select 18561,'018561','1987-080G','SOOS-2 adapter','',60170,'In Earth orbit','','1987-10-18 00:00:00',107.26,1017,1182,90.34,'Polar' from dual union all
select 18562,'018562','1987-080H','SOOS-2 despin weight','',60170,'In Earth orbit','','1987-11-19 00:00:00',107.95,992,1272,90.18,'Polar' from dual union all
select 18563,'018563','1986-019TK','deb Ariane V16','',59212,'Reentered','1988-09-26 00:00:00','1987-12-02 00:00:00',99.63,724,758,98.79,'Sun-Synch' from dual union all
select 18564,'018564','61 Omicron 284','deb Ablestar','',7273,'Reentered','2006-12-31 00:00:00','1985-01-22 00:00:00',106.3,541,1570,62.77,'Intermediate' from dual union all
select 18565,'018565','1985-094U','deb Kosmos-1695','[UNID117] RCS 0.0',58317,'In Earth orbit','','1991-09-19 00:00:00',111.67,1202,1405,82.39,'Intermediate' from dual union all
select 18566,'018566','1983-044EV','deb Kosmos-1461','',55801,'Reentered','1989-04-03 00:00:00','1987-12-03 00:00:00',99.45,601,864,65,'Intermediate' from dual union all
select 18567,'018567','1987-085H','KDU part','KDU part',60175,'Reentered','1987-11-27 00:00:00','1987-11-08 00:00:00',92.72,381,435,69.17,'Intermediate' from dual union all
select 18568,'018568','1987-094A','Progress-33','Progress 7K-TG No. 140',60184,'Deorbited','1987-12-19 00:00:00','1987-11-28 00:00:00',91.19,325,341,51.63,'Intermediate' from dual union all
select 18569,'018569','1987-094B','11S510','11S510',60184,'Reentered','1987-11-23 00:00:00','1987-11-22 00:00:00',88.34,172,212,51.65,'Intermediate' from dual union all
select 1857,'001857','1965-082EQ','deb Transtage 4','',11832,'Reentered','2016-09-10 00:00:00','1992-01-30 00:00:00',98.37,643,719,31.91,'Intermediate' from dual union all
select 18570,'018570','1987-095A','TV-SAT','TV-SAT',60185,'In Earth orbit','','1987-12-21 00:00:00',1422.72,35214,35835,.09,'Drift GEO' from dual union all
select 18571,'018571','1987-078F','deb Ariane V19','',60168,'In Earth orbit','','1987-10-17 00:00:00',634.85,276,35905,6.9,'GEO Transfer' from dual union all
select 18572,'018572','1987-092C','Blenda','Sunshade',60182,'Reentered','1987-11-28 00:00:00','1987-11-26 00:00:00',88.89,207,232,70.36,'Intermediate' from dual union all
select 18573,'018573','1987-092D','KDU part','KDU part',60182,'Reentered','1987-11-27 00:00:00','1987-11-26 00:00:00',89.03,213,241,70.36,'Intermediate' from dual union all
select 18574,'018574','1987-092E','KDU','KDU',60182,'Reentered','1987-12-04 00:00:00','1987-11-26 00:00:00',89.1,215,245,70.36,'Intermediate' from dual union all
select 18575,'018575','1987-096A','Kosmos-1897','Al''tair No. 12L',60186,'In Earth orbit','','1987-12-27 00:00:00',1435.96,35748,35820,1.37,'Stationary' from dual union all
select 18576,'018576','1987-096B','8S812','8S812',60186,'Reentered','1987-11-28 00:00:00','1987-11-27 00:00:00',88.08,177,182,51.62,'Intermediate' from dual union all
select 18577,'018577','1987-096C','Perekhodnik','Sredniy perekhodnik',60186,'Reentered','1987-11-27 00:00:00','1987-11-26 00:00:00',88.46,187,210,51.61,'Intermediate' from dual union all
select 18578,'018578','1987-096D','Blok DM2','11S861 No. 29L',60186,'In Earth orbit','','1987-12-28 00:00:00',1431.85,35620,35786,1.39,'Drift GEO' from dual union all
select 18579,'018579','1987-096E','SOZ','SOZ',60186,'Reentered','1989-07-12 00:00:00','1988-03-06 00:00:00',632.38,257,35797,46.95,'GEO Transfer' from dual union all
select 1858,'001858','1965-082ER','deb Transtage 4','',11832,'Reentered','1967-01-03 00:00:00','1966-01-23 00:00:00',96.44,451,726,32.63,'Intermediate' from dual union all
select 18580,'018580','1987-096F','SOZ','SOZ',60186,'Reentered','1988-08-18 00:00:00','1987-12-31 00:00:00',631.72,263,35757,47.22,'GEO Transfer' from dual union all
select 18581,'018581','1987-092F','KDU part','KDU part',60182,'Reentered','1987-11-29 00:00:00','1987-11-27 00:00:00',90.68,212,403,70.36,'Intermediate' from dual union all
select 18582,'018582','1987-092G','KDU part','KDU part',60182,'Reentered','1987-11-27 00:00:00','1987-11-27 00:00:00',89.76,197,328,70.29,'Intermediate' from dual union all
select 18583,'018583','1987-097A','DSP F13','DSP 5R',60187,'In Earth orbit','','1988-02-01 00:00:00',1435.5,35750,35800,0,'Stationary' from dual union all
select 18584,'018584','1987-097B','Transtage D-14','Transtage D-14',60187,'In Earth orbit','','1987-12-01 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 18585,'018585','1987-098A','Kosmos-1898','Strela-2M',60188,'In Earth orbit','','1988-01-02 00:00:00',100.73,777,810,74.01,'Intermediate' from dual union all
select 18586,'018586','1987-098B','S3M','S3M',60188,'In Earth orbit','','1987-12-31 00:00:00',100.63,771,806,74.01,'Intermediate' from dual union all
select 18587,'018587','1987-033D','Kosmos-1836 part','Terilen AO part',60123,'Reentered','1987-12-03 00:00:00','1987-12-02 00:00:00',88.88,189,250,64.79,'Intermediate' from dual union all
select 18588,'018588','1987-033E','Kosmos-1836 part','Terilen AO part',60123,'Reentered','1987-12-05 00:00:00','1987-12-03 00:00:00',90.77,179,446,64.78,'Intermediate' from dual union all
select 18589,'018589','1964-006N','deb unk','[UNID118] RCS 0.1',10292,'In Earth orbit','','1979-02-26 00:00:00',377.52,382,21468,60.55,'Highly Ellip' from dual union all
select 1859,'001859','1965-082ES','deb Transtage 4','',11832,'In Earth orbit','','1992-01-29 00:00:00',100.01,703,814,32.13,'Intermediate' from dual union all
select 18590,'018590','1971-015DJ','deb Kosmos-397','',24833,'Reentered','2000-03-17 00:00:00','1988-10-18 00:00:00',105,507,1482,65.79,'Intermediate' from dual union all
select 18591,'018591','1976-077FD','deb Delta 126','[UNID119] RCS 0.06',39029,'In Earth orbit','','1991-10-02 00:00:00',136.76,1284,3546,101.57,'Medium' from dual union all
select 18592,'018592','1981-053LU','deb Kosmos-1275','',52576,'In Earth orbit','','1991-10-08 00:00:00',103.89,930,955,82.97,'Intermediate' from dual union all
select 18593,'018593','1981-053LV','deb Kosmos-1275','',52576,'Reentered','2009-04-04 00:00:00','1991-10-06 00:00:00',102.04,813,898,82.69,'Intermediate' from dual union all
select 18594,'018594','1968-012E','NNS O-18 insulation','',16739,'Reentered','1991-04-19 00:00:00','1985-03-29 00:00:00',106.72,1024,1125,90.04,'Polar' from dual union all
select 18595,'018595','1987-004GV','deb Kosmos-1813','',60094,'Reentered','1989-02-11 00:00:00','1987-12-13 00:00:00',97.27,337,919,72.83,'Intermediate' from dual union all
select 18596,'018596','1987-004GW','deb Kosmos-1813','',60094,'Reentered','1989-03-05 00:00:00','1987-12-13 00:00:00',95.22,377,682,72.8,'Intermediate' from dual union all
select 18597,'018597','1968-097EQ','deb Kosmos-252 [*}','',16824,'In Earth orbit','','1991-05-27 00:00:00',114.65,850,2029,62.25,'Intermediate' from dual union all
select 18598,'018598','1981-088N','deb Kosmos-1305','',52611,'In Earth orbit','','1985-07-31 00:00:00',251.26,474,13149,63.16,'Medium' from dual union all
select 18599,'018599','1976-077FE','deb Delta 126','[UNID120] RCS 0.04',39029,'In Earth orbit','','1991-10-09 00:00:00',126.83,1232,2736,100.82,'Medium' from dual union all
select 186,'000186','61 A Beta','Discoverer 31','CORONA C''''''-3',7250,'Reentered Attached','1961-10-26 00:00:00','1961-09-25 00:00:00',90.65,233,380,82.7,'Intermediate' from dual union all
select 1860,'001860','1965-082ET','deb Transtage 4','',11832,'Reentered','1969-04-19 00:00:00','1966-01-01 00:00:00',98.24,584,764,33.3,'Intermediate' from dual union all
select 18600,'018600','1976-077FF','deb Delta 126','',39029,'In Earth orbit','','1991-10-07 00:00:00',131.81,1524,2879,102.05,'Medium' from dual union all
select 18601,'018601','1981-053LW','deb Kosmos-1275','',52576,'In Earth orbit','','1991-09-30 00:00:00',104.61,923,1030,82.81,'Intermediate' from dual union all
select 18602,'018602','1968-091DG','deb Kosmos-249','',16818,'In Earth orbit','','1991-10-27 00:00:00',113.14,768,1974,62.26,'Intermediate' from dual union all
select 18603,'018603','62 Kappa 7','West Ford Drag dipole','',8192,'In Earth orbit','','1971-12-13 00:00:00',153.07,2782,3420,86.67,'Medium' from dual union all
select 18604,'018604','1979-017LL','deb P78-1','',47501,'Reentered','1989-04-21 00:00:00','1987-11-02 00:00:00',96.13,517,630,97.8,'Sun-Synch' from dual union all
select 18605,'018605','1978-098C','deb Delta 145','',45293,'Reentered','1988-10-20 00:00:00','1987-11-02 00:00:00',102.81,863,920,99,'Sun-Synch' from dual union all
select 18606,'018606','1976-067BP','deb Kosmos-839','',39019,'In Earth orbit','','1984-12-18 00:00:00',116.59,1001,2054,65.86,'Intermediate' from dual union all
select 18607,'018607','1986-019TL','deb Ariane V16','',59212,'Reentered','1989-03-25 00:00:00','1987-08-12 00:00:00',100.26,735,807,98.9,'Sun-Synch' from dual union all
select 18608,'018608','1985-077K','deb Strela-2M                     ','[UNID121] RCS 0.01',58300,'In Earth orbit','','1987-08-13 00:00:00',100.92,789,816,74.06,'Intermediate' from dual union all
select 18609,'018609','1987-004GX','deb Kosmos-1813','',60094,'Reentered','1988-09-29 00:00:00','1987-11-02 00:00:00',93.89,365,565,72.81,'Intermediate' from dual union all
select 1861,'001861','1965-082ED','deb Transtage 4','',11832,'Reentered','1989-12-22 00:00:00','1989-12-19 00:00:00',90.76,305,319,32.34,'Intermediate' from dual union all
select 18610,'018610','1980-089CD','deb Kosmos-1220','',50299,'Reentered','1988-10-18 00:00:00','1987-05-12 00:00:00',96.44,515,662,64.98,'Intermediate' from dual union all
select 18611,'018611','1986-019TM','deb Ariane V16','',59212,'Reentered','1991-05-11 00:00:00','1987-11-01 00:00:00',101.73,797,885,98.7,'Sun-Synch' from dual union all
select 18612,'018612','1979-017LM','deb P78-1','',47501,'Reentered','1989-08-05 00:00:00','1987-11-01 00:00:00',94.66,486,518,97.79,'Sun-Synch' from dual union all
select 18613,'018613','1971-015DK','deb Kosmos-397','',24833,'Reentered','1989-09-02 00:00:00','1987-11-27 00:00:00',110.58,563,1944,65.64,'Intermediate' from dual union all
select 18614,'018614','1986-019TN','deb Ariane V16','',59212,'Reentered','1990-07-04 00:00:00','1987-10-06 00:00:00',102.71,792,981,97.86,'Sun-Synch' from dual union all
select 18615,'018615','1963-014DK','deb West Ford','RCS 0.06',9245,'In Earth orbit','','1987-12-21 00:00:00',164.93,1944,5225,86.36,'Medium' from dual union all
select 18616,'018616','1969-082LB','deb r/b (AGENA D)','',18349,'In Earth orbit','','1984-01-25 00:00:00',105.1,858,1140,70.14,'Intermediate' from dual union all
select 18617,'018617','1978-064C','deb Seasat 1','Insulation',45259,'Reentered','1988-03-25 00:00:00','1987-11-29 00:00:00',100.28,759,785,108.01,'Retrograde' from dual union all
select 18618,'018618','1968-097ER','deb Kosmos-252','[UNiD122] RCS 0.2',16824,'Reentered','1989-01-02 00:00:00','1982-12-15 00:00:00',174.16,286,7619,62.59,'Medium' from dual union all
select 18619,'018619','1985-030C','deb Kosmos-1646','',58253,'Reentered','1987-12-27 00:00:00','1987-11-20 00:00:00',96.34,404,763,65.04,'Intermediate' from dual union all
select 1862,'001862','1965-082EE','deb Transtage 4','',11832,'Reentered','1968-09-11 00:00:00','1968-09-09 00:00:00',93.9,446,485,32.56,'Intermediate' from dual union all
select 18620,'018620','1985-030D','deb Kosmos-1646','',58253,'Reentered','1988-03-25 00:00:00','1987-12-05 00:00:00',92.45,387,402,65.05,'Intermediate' from dual union all
select 18621,'018621','1985-030E','deb Kosmos-1646','',58253,'Reentered','1987-12-06 00:00:00','1987-12-04 00:00:00',90.02,265,286,65.02,'Intermediate' from dual union all
select 18622,'018622','1985-030F','deb Kosmos-1646','',58253,'Reentered','1988-01-06 00:00:00','1987-12-04 00:00:00',92.32,380,397,65.03,'Intermediate' from dual union all
select 18623,'018623','1985-030G','deb Kosmos-1646','',58253,'Reentered','1988-01-03 00:00:00','1987-12-04 00:00:00',92.15,369,391,65.03,'Intermediate' from dual union all
select 18624,'018624','1985-030H','deb Kosmos-1646','',58253,'Reentered','1987-12-08 00:00:00','1987-12-04 00:00:00',90.68,295,321,65.02,'Intermediate' from dual union all
select 18625,'018625','1987-099A','Kosmos-1899','Oblik',60189,'Landed','1987-12-21 00:00:00','1987-12-09 00:00:00',89.6,229,281,70.38,'Intermediate' from dual union all
select 18626,'018626','1987-099B','11S510','11S510',60189,'Reentered','1987-12-13 00:00:00','1987-12-09 00:00:00',88.93,198,246,70.39,'Intermediate' from dual union all
select 18627,'018627','1987-099C','Sensor cover','',60189,'Reentered','1987-12-08 00:00:00','1987-12-07 00:00:00',88.83,200,234,70.41,'Intermediate' from dual union all
select 18628,'018628','1985-030J','deb Kosmos-1646','',58253,'Reentered','1988-01-15 00:00:00','1987-12-07 00:00:00',92.09,365,389,65.03,'Intermediate' from dual union all
select 18629,'018629','1985-030K','deb Kosmos-1646','',58253,'Reentered','1988-01-15 00:00:00','1987-12-06 00:00:00',91.79,353,372,65.03,'Intermediate' from dual union all
select 1863,'001863','1965-108A','Transtage 8','Transtage 8',11858,'In Earth orbit','','1966-04-29 00:00:00',610.7,283,34649,26.76,'GEO Transfer' from dual union all
select 18630,'018630','1985-030L','deb Kosmos-1646','',58253,'Reentered','1988-01-06 00:00:00','1987-12-08 00:00:00',92.78,358,464,65.06,'Intermediate' from dual union all
select 18631,'018631','1987-100A','Raduga','Gran'' No. 32L',60190,'In Earth orbit','','1988-01-09 00:00:00',1436.11,35786,35788,1.37,'Stationary' from dual union all
select 18632,'018632','1987-100B','8S812','8S812',60190,'Reentered','1987-12-13 00:00:00','1987-12-12 00:00:00',87.87,166,173,51.61,'Intermediate' from dual union all
select 18633,'018633','1987-100C','Perekhodnik','Sredniy perekhodnik',60190,'Reentered','1987-12-11 00:00:00','1987-12-10 00:00:00',88.36,183,204,51.62,'Intermediate' from dual union all
select 18634,'018634','1987-100D','Blok DM2','11S861 No. 30L',60190,'In Earth orbit','','1988-01-12 00:00:00',1392.55,34859,35001,1.43,'Drift GEO' from dual union all
select 18635,'018635','1987-100E','SOZ','SOZ',60190,'Reentered','1991-05-07 00:00:00','1988-01-09 00:00:00',617.48,274,35010,47.24,'GEO Transfer' from dual union all
select 18636,'018636','1987-100F','SOZ','SOZ',60190,'Reentered','1990-07-05 00:00:00','1988-01-09 00:00:00',618.75,329,35021,47.27,'GEO Transfer' from dual union all
select 18637,'018637','1981-053LX','deb Kosmos-1275','',52576,'In Earth orbit','','1985-04-04 00:00:00',105.12,941,1059,82.98,'Intermediate' from dual union all
select 18638,'018638','1981-053LY','deb Kosmos-1275','',52576,'Reentered','1989-09-11 00:00:00','1982-11-17 00:00:00',111.92,920,1709,82.69,'Intermediate' from dual;
insert into SATELLITE
select 18639,'018639','1981-053LZ','deb Kosmos-1275','',52576,'In Earth orbit','','1985-03-26 00:00:00',107.53,971,1254,83.07,'Intermediate' from dual union all
select 1864,'001864','1965-109A','NNS O-6','NNS O-6',11859,'In Earth orbit','','1966-02-15 00:00:00',105.02,907,1084,89.1,'Polar' from dual union all
select 18640,'018640','1981-053MA','deb Kosmos-1275','',52576,'In Earth orbit','','1985-03-29 00:00:00',106.41,969,1152,83.22,'Intermediate' from dual union all
select 18641,'018641','1966-056CH','deb Pageos','RCS 86.0',13463,'Reentered','1989-12-20 00:00:00','1985-03-02 00:00:00',176.51,970,7121,84.64,'Medium' from dual union all
select 18642,'018642','1986-019TP','deb Ariane V16','',59212,'Reentered','1991-05-28 00:00:00','1987-03-09 00:00:00',101.86,802,891,98.7,'Sun-Synch' from dual union all
select 18643,'018643','1977-065FU','deb Delta 132','',42193,'Reentered','1988-11-27 00:00:00','1977-08-14 00:00:00',118.99,646,2626,30.91,'Intermediate' from dual union all
select 18644,'018644','1980-030AY','deb Kosmos-1174','',50240,'Reentered','2003-04-04 00:00:00','1985-01-07 00:00:00',103.39,889,949,67,'Intermediate' from dual union all
select 18645,'018645','1973-086GX','deb Delta 98','',30688,'In Earth orbit','','1986-04-10 00:00:00',116.71,1435,1630,101.99,'Sun-Synch' from dual union all
select 18646,'018646','1979-017LN','deb P78-1','',47501,'Reentered','1988-09-19 00:00:00','1987-12-10 00:00:00',94.24,437,527,97.74,'Sun-Synch' from dual union all
select 18647,'018647','1973-086GY','deb Delta 98','',30688,'In Earth orbit','','1985-05-07 00:00:00',115.56,1423,1539,101.56,'Sun-Synch' from dual union all
select 18648,'018648','1968-097ED','deb Kosmos-252  [*}','',16824,'In Earth orbit','','1969-08-15 00:00:00',115.44,746,2205,62.27,'Intermediate' from dual union all
select 18649,'018649','1978-064D','deb Seasat 1','',45259,'Reentered','1988-03-01 00:00:00','1987-10-12 00:00:00',99.43,708,755,107.92,'Retrograde' from dual union all
select 1865,'001865','1965-109B','Altair 2 X-258E6 RH-129','X-258E6',11859,'In Earth orbit','','1966-02-10 00:00:00',105.01,906,1084,89.11,'Polar' from dual union all
select 18650,'018650','1979-078E','deb Kosmos-1125 [UNID]','Strela-2M debris',47562,'In Earth orbit','','1981-01-20 00:00:00',100.7,785,799,74.09,'Intermediate' from dual union all
select 18651,'018651','61 Omicron 285','deb Ablestar','',7273,'In Earth orbit','','1980-04-17 00:00:00',116.14,922,2093,65.69,'Intermediate' from dual union all
select 18652,'018652','1985-030M','deb Kosmos-1646','',58253,'Reentered','1987-12-30 00:00:00','1987-12-09 00:00:00',91.45,328,363,65.03,'Intermediate' from dual union all
select 18653,'018653','1985-030N','deb Kosmos-1646','',58253,'Reentered','1988-01-20 00:00:00','1987-12-11 00:00:00',93.1,388,465,65.04,'Intermediate' from dual union all
select 18654,'018654','1985-030P','deb Kosmos-1646','',58253,'Reentered','1988-04-02 00:00:00','1987-12-12 00:00:00',92.41,381,405,65.03,'Intermediate' from dual union all
select 18655,'018655','1985-030Q','deb Kosmos-1646','',58253,'Reentered','1988-08-24 00:00:00','1987-12-16 00:00:00',92.37,376,406,65.04,'Intermediate' from dual union all
select 18656,'018656','1985-030R','deb Kosmos-1646','',58253,'Reentered','1987-12-31 00:00:00','1987-12-10 00:00:00',92.17,378,384,65.03,'Intermediate' from dual union all
select 18657,'018657','1985-030S','deb Kosmos-1646','',58253,'Reentered','1988-01-22 00:00:00','1987-12-13 00:00:00',92,363,383,65.03,'Intermediate' from dual union all
select 18658,'018658','1985-030T','deb Kosmos-1646','',58253,'Reentered','1988-02-09 00:00:00','1987-12-11 00:00:00',92.2,372,393,65.03,'Intermediate' from dual union all
select 18659,'018659','1985-030U','deb Kosmos-1646','',58253,'Reentered','1988-01-15 00:00:00','1987-12-09 00:00:00',92.03,365,384,65.03,'Intermediate' from dual union all
select 1866,'001866','1965-110A','OPS 4639','CORONA J-26',11860,'Reentered Attached','1966-01-20 00:00:00','1965-12-26 00:00:00',90.76,178,446,80.01,'Intermediate' from dual union all
select 18660,'018660','1985-030V','deb Kosmos-1646','',58253,'Reentered','1987-12-28 00:00:00','1987-12-11 00:00:00',91.28,316,359,65.03,'Intermediate' from dual union all
select 18661,'018661','1985-030W','deb Kosmos-1646','',58253,'Reentered','1988-02-12 00:00:00','1987-12-10 00:00:00',93.94,399,536,65.08,'Intermediate' from dual union all
select 18662,'018662','1985-030X','deb Kosmos-1646','',58253,'Reentered','1988-01-02 00:00:00','1987-12-09 00:00:00',91.73,348,370,65.02,'Intermediate' from dual union all
select 18663,'018663','1985-030Y','deb Kosmos-1646','',58253,'Reentered','1988-07-28 00:00:00','1987-12-11 00:00:00',92.61,388,417,65.02,'Intermediate' from dual union all
select 18664,'018664','1985-030Z','deb Kosmos-1646','',58253,'Reentered','1987-12-21 00:00:00','1987-12-11 00:00:00',91.62,341,366,65.02,'Intermediate' from dual union all
select 18665,'018665','1987-101A','Kosmos-1900','US-AM reactor',60191,'In Earth orbit','','1988-01-11 00:00:00',89.76,256,269,64.99,'Intermediate' from dual union all
select 18666,'018666','1987-102A','Kosmos-1901','Kobal''t',60192,'Landed','1988-02-03 00:00:00','1987-12-21 00:00:00',89.54,174,330,64.91,'Intermediate' from dual union all
select 18667,'018667','1987-102B','11S510','11S510',60192,'Reentered','1987-12-20 00:00:00','1987-12-16 00:00:00',89.41,169,322,64.91,'Intermediate' from dual union all
select 18668,'018668','1987-103A','Kosmos-1902','Vektor',60193,'Reentered','1988-12-30 00:00:00','1988-01-16 00:00:00',92.28,363,409,65.83,'Intermediate' from dual union all
select 18669,'018669','1987-103B','S3M','S3M',60193,'Reentered','1988-09-21 00:00:00','1988-01-14 00:00:00',92.11,353,403,65.84,'Intermediate' from dual union all
select 1867,'001867','1965-111A','Kosmos-102','4Ya11 (US-A)',11861,'Reentered','1966-01-13 00:00:00','1965-12-27 00:00:00',89.22,203,269,65,'Intermediate' from dual union all
select 18670,'018670','1965-027AG','deb SNAPSHOT','',11777,'In Earth orbit','','1990-05-19 00:00:00',111.46,1267,1320,90.32,'Polar' from dual union all
select 18671,'018671','1973-086GZ','deb Delta 98','',30688,'In Earth orbit','','1985-05-06 00:00:00',116.48,1432,1612,101.93,'Sun-Synch' from dual union all
select 18672,'018672','1966-056CJ','deb Pageos','RCS 1.7',13463,'Reentered','1990-02-13 00:00:00','1987-10-28 00:00:00',180.31,3649,4740,88.87,'Medium' from dual union all
select 18673,'018673','1976-067BQ','deb Kosmos-839','',39019,'Reentered','1988-09-10 00:00:00','1987-10-03 00:00:00',118.42,663,2557,64.65,'Intermediate' from dual union all
select 18674,'018674','1986-019TQ','deb Ariane V16','',59212,'Reentered','1990-03-06 00:00:00','1987-11-03 00:00:00',102.54,778,980,98.46,'Sun-Synch' from dual union all
select 18675,'018675','1965-027AH','deb SNAPSHOT','',11777,'In Earth orbit','','1987-08-11 00:00:00',111.4,1269,1314,90.3,'Polar' from dual union all
select 18676,'018676','1986-019TR','deb Ariane V16','',59212,'Reentered','1990-07-16 00:00:00','1987-11-02 00:00:00',103.45,778,1065,98.04,'Sun-Synch' from dual union all
select 18677,'018677','1976-077FG','deb Delta 126','',39029,'Reentered','2002-12-17 00:00:00','1987-12-08 00:00:00',124.24,1456,2284,104.2,'Medium' from dual union all
select 18678,'018678','1963-014DL','deb West Ford','RCS 0.16',9245,'Reentered','2002-02-04 00:00:00','1987-10-27 00:00:00',164.75,1718,5436,86.4,'Medium' from dual union all
select 18679,'018679','1978-026HG','deb Delta 139','',45221,'Reentered','1991-03-22 00:00:00','1985-05-07 00:00:00',103.76,693,1180,98.3,'Sun-Synch' from dual union all
select 1868,'001868','1965-112A','Kosmos-103','Strela-2',11862,'Reentered','1990-01-02 00:00:00','1966-02-15 00:00:00',96.97,596,632,56.04,'Intermediate' from dual union all
select 18680,'018680','1986-101L','deb S5M','',59294,'In Earth orbit','','1987-01-21 00:00:00',103.85,927,954,82.56,'Intermediate' from dual union all
select 18681,'018681','1987-049D','deb Kosmos-1850','',60139,'In Earth orbit','','1987-11-02 00:00:00',101.33,805,839,74.04,'Intermediate' from dual union all
select 18682,'018682','1987-012J','deb Ginga','',60102,'Reentered','1989-02-09 00:00:00','1987-06-08 00:00:00',96.44,509,668,31.09,'Intermediate' from dual union all
select 18683,'018683','1986-019TS','deb Ariane V16','',59212,'Reentered','1989-01-17 00:00:00','1987-11-03 00:00:00',101.32,757,886,98.18,'Sun-Synch' from dual union all
select 18684,'018684','1986-019TT','deb Ariane V16','',59212,'Reentered','1990-03-22 00:00:00','1987-11-03 00:00:00',102.99,778,1023,98.79,'Sun-Synch' from dual union all
select 18685,'018685','1986-019TU','deb Ariane V16','',59212,'Reentered','1988-12-04 00:00:00','1987-11-02 00:00:00',100.63,721,855,98.61,'Sun-Synch' from dual union all
select 18686,'018686','1964-006P','deb unk','[UNID123] RCS 0.3',10292,'Reentered','1988-10-17 00:00:00','1987-08-24 00:00:00',138.07,271,4671,63.22,'Medium' from dual union all
select 18687,'018687','1987-041G','deb Kosmos- 1844','',60131,'Reentered','1993-04-06 00:00:00','1987-10-28 00:00:00',101.91,846,853,71.02,'Intermediate' from dual union all
select 18688,'018688','1963-014DM','deb West Ford','RCS 0.09',9245,'In Earth orbit','','1967-03-11 00:00:00',166.44,3625,3665,87.31,'Medium' from dual union all
select 18689,'018689','1976-067BR','deb Kosmos-839','',39019,'In Earth orbit','','1985-03-07 00:00:00',124.71,1116,2665,65.82,'Medium' from dual union all
select 1869,'001869','1965-112B','S3','S3',11862,'Reentered','1982-06-15 00:00:00','1966-02-01 00:00:00',97.07,592,645,56.06,'Intermediate' from dual union all
select 18690,'018690','1966-056CK','deb Pageos','RCS 0.4',13463,'Reentered','1989-02-02 00:00:00','1987-11-01 00:00:00',176.46,1258,6829,85.46,'Medium' from dual union all
select 18691,'018691','1965-027AJ','deb SNAPSHOT','',11777,'In Earth orbit','','1987-12-16 00:00:00',111.48,1270,1320,90.31,'Polar' from dual union all
select 18692,'018692','1971-015DL','deb Kosmos-397','',24833,'Reentered','1988-04-23 00:00:00','1987-11-08 00:00:00',110.62,490,2021,64.34,'Intermediate' from dual union all
select 18693,'018693','1974-089FA','deb Delta 104','',33692,'In Earth orbit','','1987-12-03 00:00:00',117.98,1395,1786,101.85,'Sun-Synch' from dual union all
select 18694,'018694','1983-044EW','deb Kosmos-1461','',55801,'Reentered','1988-01-22 00:00:00','1987-12-15 00:00:00',100.64,448,1129,64.79,'Intermediate' from dual union all
select 18695,'018695','1979-017LP','deb P78-1','',47501,'Reentered','1988-01-14 00:00:00','1987-12-07 00:00:00',92.87,390,441,97.78,'Sun-Synch' from dual union all
select 18696,'018696','1987-004GY','deb Kosmos-1813','',60094,'Reentered','1988-03-26 00:00:00','1987-11-30 00:00:00',96.17,333,818,72.84,'Intermediate' from dual union all
select 18697,'018697','1987-098C','deb Kosmos- 1898','Strela-2M debris 1',60188,'In Earth orbit','','1988-01-02 00:00:00',100.97,798,812,74.01,'Intermediate' from dual union all
select 18698,'018698','1987-098D','deb Kosmos- 1898','Strela-2M debris 2 (high',60188,'In Earth orbit','','1988-01-09 00:00:00',101.27,804,834,74,'Intermediate' from dual union all
select 18699,'018699','1987-104A','Soyuz TM-4','Soyuz 7K-STM No. 54',60194,'Landed','1988-06-17 00:00:00','1988-01-21 00:00:00',91.42,332,357,51.63,'Intermediate' from dual union all
select 187,'000187','61 Omega 3','deb Discoverer XXX','',7272,'Reentered','1961-09-28 00:00:00','1961-09-21 00:00:00',90.79,206,421,82.71,'Intermediate' from dual union all
select 1870,'001870','1965-108B','LES 4','LES 4',11858,'Reentered','1977-08-01 00:00:00','1966-02-08 00:00:00',584.9,189,33390,26.51,'Highly Ellip' from dual union all
select 18700,'018700','1987-104B','11S510','11S510',60194,'Reentered','1987-12-23 00:00:00','1987-12-22 00:00:00',88.24,161,214,51.63,'Intermediate' from dual union all
select 18701,'018701','1987-105A','Kosmos-1903','Oko',60195,'In Earth orbit','','1988-01-20 00:00:00',717.72,606,39746,63.08,'Molniya' from dual union all
select 18702,'018702','1987-105B','11S510','11S510',60195,'Reentered','1988-02-09 00:00:00','1987-12-29 00:00:00',91.89,211,524,62.83,'Intermediate' from dual union all
select 18703,'018703','1987-105C','BOZ','BOZ',60195,'Reentered','1988-01-12 00:00:00','1987-12-28 00:00:00',91.74,174,546,62.83,'Intermediate' from dual union all
select 18704,'018704','1987-105D','Blok-2BL','Blok-2BL',60195,'In Earth orbit','','1988-01-22 00:00:00',705.21,606,39127,63.05,'Molniya' from dual union all
select 18705,'018705','1987-099D','KDU','KDU',60189,'Reentered','1988-01-15 00:00:00','1987-12-22 00:00:00',89.49,225,274,70.38,'Intermediate' from dual union all
select 18706,'018706','1987-099E','KDU part','KDU part',60189,'Reentered','1987-12-23 00:00:00','1987-12-22 00:00:00',88.83,192,242,70.4,'Intermediate' from dual union all
select 18707,'018707','1987-099F','KDU part','KDU part',60189,'Reentered','1987-12-22 00:00:00','1987-12-22 00:00:00',89.21,209,262,70.4,'Intermediate' from dual union all
select 18708,'018708','1987-099G','Blenda','Sunshade',60189,'Reentered','1987-12-27 00:00:00','1987-12-22 00:00:00',91.1,243,414,70.4,'Intermediate' from dual union all
select 18709,'018709','1987-106A','Kosmos-1904','Parus',60196,'In Earth orbit','','1988-01-22 00:00:00',104.86,968,1007,82.91,'Intermediate' from dual union all
select 1871,'001871','1965-112C','deb Blok Kosmos','',11862,'Reentered','1967-01-25 00:00:00','1966-02-01 00:00:00',96.54,550,636,56.05,'Intermediate' from dual union all
select 18710,'018710','1987-106B','S3M','S3M',60196,'In Earth orbit','','1988-01-25 00:00:00',104.75,964,1002,82.9,'Intermediate' from dual union all
select 18711,'018711','1987-107A','Kosmos-1905','Oblik',60197,'Landed','1988-01-08 00:00:00','1987-12-27 00:00:00',89.59,229,280,70.35,'Intermediate' from dual union all
select 18712,'018712','1987-107B','11S510','11S510',60197,'Reentered','1988-01-01 00:00:00','1987-12-27 00:00:00',89.06,202,253,70.36,'Intermediate' from dual union all
select 18713,'018713','1987-108A','Kosmos-1906','Resurs-F2 17F42 No. 1',60198,'Reentered','1988-03-13 00:00:00','1988-01-26 00:00:00',89.64,247,267,82.58,'Intermediate' from dual union all
select 18714,'018714','1987-108B','11S510','11S510',60198,'Reentered','1987-12-29 00:00:00','1987-12-28 00:00:00',88.27,167,211,82.6,'Intermediate' from dual union all
select 18715,'018715','1987-109A','Ekran-M','Ekran-M No. 13L',60199,'In Earth orbit','','1988-01-26 00:00:00',1436.04,35720,35851,1.55,'Stationary' from dual union all
select 18716,'018716','1987-109B','8S812','8S812',60199,'Reentered','1987-12-30 00:00:00','1987-12-29 00:00:00',87.94,168,177,51.62,'Intermediate' from dual union all
select 18717,'018717','1987-109C','Perekhodnik','Sredniy perekhodnik',60199,'Reentered','1987-12-28 00:00:00','1987-12-27 00:00:00',88.45,187,208,51.62,'Intermediate' from dual union all
select 18718,'018718','1987-109D','Blok DM2','11S861 No. 31L',60199,'In Earth orbit','','1988-01-26 00:00:00',1428.12,35383,35878,1.58,'Drift GEO' from dual union all
select 18719,'018719','1987-109E','SOZ','SOZ',60199,'Reentered','2000-12-20 00:00:00','1988-02-03 00:00:00',632.05,331,35706,47.25,'GEO Transfer' from dual union all
select 1872,'001872','1965-112D','deb Blok Kosmos','',11862,'Reentered','1967-12-27 00:00:00','1966-02-02 00:00:00',96.59,556,635,56.03,'Intermediate' from dual union all
select 18720,'018720','1987-110A','Kosmos-1907','Oblik No. 362',60200,'Landed','1988-01-12 00:00:00','1987-12-31 00:00:00',92.25,355,415,72.82,'Intermediate' from dual union all
select 18721,'018721','1987-110B','11S510','11S510',60200,'Reentered','1988-01-12 00:00:00','1987-12-31 00:00:00',90.02,195,356,72.82,'Intermediate' from dual union all
select 18722,'018722','1987-109F','SOZ','SOZ',60199,'Reentered','1988-10-26 00:00:00','1988-02-03 00:00:00',630.69,275,35692,47.21,'GEO Transfer' from dual union all
select 18723,'018723','1987-063C','Soyuz TM-3 BO','Bitovoy otsek',60153,'Reentered','1988-06-17 00:00:00','1987-12-29 00:00:00',91.36,329,354,51.62,'Intermediate' from dual union all
select 18724,'018724','1987-110C','Sensor cover','Sensor cover',60200,'Reentered','1988-01-01 00:00:00','1987-12-31 00:00:00',89.65,183,332,72.83,'Intermediate' from dual union all
select 18725,'018725','1987-020C','deb Kosmos-1823','',60110,'In Earth orbit','','1987-12-31 00:00:00',116.48,1464,1581,73.77,'Intermediate' from dual union all
select 18726,'018726','1987-020D','deb Kosmos-1823','',60110,'Reentered','1994-10-05 00:00:00','1987-12-31 00:00:00',117.32,1490,1631,73.53,'Intermediate' from dual union all
select 18727,'018727','1987-020E','deb Kosmos-1823','',60110,'In Earth orbit','','1987-12-31 00:00:00',114.01,1341,1480,73.36,'Intermediate' from dual union all
select 18728,'018728','1987-020F','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-13 00:00:00',116.65,1476,1584,73.62,'Intermediate' from dual;
insert into SATELLITE
select 18729,'018729','1987-020G','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-13 00:00:00',117.77,1480,1682,73.83,'Intermediate' from dual union all
select 1873,'001873','1965-112E','deb Blok Kosmos','',11862,'Reentered','1967-04-21 00:00:00','1966-02-02 00:00:00',96.4,568,604,56.04,'Intermediate' from dual union all
select 18730,'018730','1987-020H','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-09 00:00:00',118.76,1480,1771,73.94,'Intermediate' from dual union all
select 18731,'018731','1987-020J','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-13 00:00:00',115.99,1477,1524,73.62,'Intermediate' from dual union all
select 18732,'018732','1987-020K','deb Kosmos-1823','',60110,'Reentered','1991-03-02 00:00:00','1987-12-31 00:00:00',112.98,1221,1506,74.59,'Intermediate' from dual union all
select 18733,'018733','1987-020L','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-19 00:00:00',113.58,1371,1411,73.46,'Intermediate' from dual union all
select 18734,'018734','1987-020M','deb Kosmos-1823','',60110,'Reentered','1992-11-21 00:00:00','1991-10-16 00:00:00',108.66,1070,1260,73.1,'Intermediate' from dual union all
select 18735,'018735','1987-020N','deb Kosmos-1823','',60110,'Reentered','1989-07-29 00:00:00','1987-12-31 00:00:00',116.54,1495,1555,73.62,'Intermediate' from dual union all
select 18736,'018736','1987-020P','deb Kosmos-1823','',60110,'Reentered','1993-11-05 00:00:00','1991-10-20 00:00:00',110.35,1053,1433,73.35,'Intermediate' from dual union all
select 18737,'018737','1987-020Q','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-07 00:00:00',116,1479,1523,73.6,'Intermediate' from dual union all
select 18738,'018738','1987-020R','deb Kosmos-1823','',60110,'Reentered','1997-12-04 00:00:00','1991-10-21 00:00:00',112.22,1128,1530,73.61,'Intermediate' from dual union all
select 18739,'018739','1987-020S','deb Kosmos-1823','',60110,'Reentered','1991-02-18 00:00:00','1987-12-30 00:00:00',115.16,1402,1523,73.57,'Intermediate' from dual union all
select 1874,'001874','1965-112F','deb Blok Kosmos','',11862,'Reentered','1968-05-01 00:00:00','1966-04-19 00:00:00',96.57,572,617,56.03,'Intermediate' from dual union all
select 18740,'018740','1987-020T','deb Kosmos-1823','',60110,'Reentered','1992-02-27 00:00:00','1991-12-26 00:00:00',104.62,880,1074,73.48,'Intermediate' from dual union all
select 18741,'018741','1987-020U','deb Kosmos-1823','',60110,'Reentered','1989-03-22 00:00:00','1987-12-31 00:00:00',113.82,1310,1494,73.42,'Intermediate' from dual union all
select 18742,'018742','1987-020V','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-01 00:00:00',115.21,1430,1500,73.52,'Intermediate' from dual union all
select 18743,'018743','1987-020W','deb Kosmos-1823','',60110,'Reentered','1989-01-11 00:00:00','1987-12-31 00:00:00',117.1,1489,1612,73.09,'Intermediate' from dual union all
select 18744,'018744','1987-020X','deb Kosmos-1823','',60110,'Reentered','1999-05-07 00:00:00','1991-10-07 00:00:00',109.99,1168,1285,73.26,'Intermediate' from dual union all
select 18745,'018745','1987-020Y','deb Kosmos-1823','',60110,'Reentered','2001-10-29 00:00:00','1991-10-10 00:00:00',113.38,1215,1548,73.52,'Intermediate' from dual union all
select 18746,'018746','1987-020Z','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-12 00:00:00',115.63,1443,1525,73.53,'Intermediate' from dual union all
select 18747,'018747','1987-020AA','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-02 00:00:00',116.23,1479,1544,73.62,'Intermediate' from dual union all
select 18748,'018748','1988-001A','Kosmos-1908','Tselina-D',61250,'In Earth orbit','','1988-02-05 00:00:00',97.7,633,665,82.51,'Intermediate' from dual union all
select 18749,'018749','1988-001B','S5M','S5M',61250,'In Earth orbit','','1988-02-05 00:00:00',97.67,632,663,82.52,'Intermediate' from dual union all
select 1875,'001875','1965-112G','deb Blok Kosmos','',11862,'Reentered','1968-02-17 00:00:00','1966-01-27 00:00:00',96.63,583,612,56.04,'Intermediate' from dual union all
select 18750,'018750','1987-020AB','deb Kosmos-1823','',60110,'Reentered','1990-08-28 00:00:00','1990-08-27 00:00:00',95.68,501,602,73.4,'Intermediate' from dual union all
select 18751,'018751','1987-020AC','deb Kosmos-1823','',60110,'In Earth orbit','','1991-09-28 00:00:00',116.86,1480,1600,73.62,'Intermediate' from dual union all
select 18752,'018752','1987-020AD','deb Kosmos-1823','',60110,'In Earth orbit','','1991-09-26 00:00:00',114.46,1383,1479,73.92,'Intermediate' from dual union all
select 18753,'018753','1987-020AE','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-09 00:00:00',115.81,1477,1507,73.44,'Intermediate' from dual union all
select 18754,'018754','1987-020AF','deb Kosmos-1823','',60110,'In Earth orbit','','1991-09-28 00:00:00',114.14,1356,1477,73.63,'Intermediate' from dual union all
select 18755,'018755','1987-020AG','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-12 00:00:00',116.03,1476,1528,73.68,'Intermediate' from dual union all
select 18756,'018756','1987-020AH','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-02 00:00:00',115.98,1476,1524,73.63,'Intermediate' from dual union all
select 18757,'018757','1987-020AJ','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-03 00:00:00',114.95,1404,1502,73.64,'Intermediate' from dual union all
select 18758,'018758','1987-020AK','deb Kosmos-1823','',60110,'In Earth orbit','','1991-09-30 00:00:00',116.38,1478,1558,73.61,'Intermediate' from dual union all
select 18759,'018759','1987-107C','Blenda','Sunshade',60197,'Reentered','1988-01-12 00:00:00','1988-01-08 00:00:00',89.32,222,260,70.35,'Intermediate' from dual union all
select 1876,'001876','1965-112H','deb Blok Kosmos','',11862,'Reentered','1968-03-12 00:00:00','1966-01-29 00:00:00',96.77,588,620,56.02,'Intermediate' from dual union all
select 18760,'018760','1987-107D','KDU','KDU',60197,'Reentered','1988-02-03 00:00:00','1988-01-08 00:00:00',89.5,227,272,70.36,'Intermediate' from dual union all
select 18761,'018761','1987-107E','KDU part','KDU part',60197,'Reentered','1988-01-15 00:00:00','1988-01-08 00:00:00',91.38,243,442,70.35,'Intermediate' from dual union all
select 18762,'018762','1987-107F','KDU part','KDU part',60197,'Reentered','1988-01-13 00:00:00','1988-01-08 00:00:00',91.81,245,482,70.34,'Intermediate' from dual union all
select 18763,'018763','1987-107G','KDU part','KDU part',60197,'Reentered','1988-01-10 00:00:00','1988-01-09 00:00:00',89.17,215,252,70.35,'Intermediate' from dual union all
select 18764,'018764','1985-042G','deb SOZ','',58265,'In Earth orbit','','1988-01-13 00:00:00',100.43,694,863,66.6,'Intermediate' from dual union all
select 18765,'018765','1985-042H','deb SOZ','',58265,'In Earth orbit','','1988-01-08 00:00:00',101.97,844,859,66.58,'Intermediate' from dual union all
select 18766,'018766','1985-042J','deb SOZ','',58265,'In Earth orbit','','1988-01-09 00:00:00',103.39,857,981,66.59,'Intermediate' from dual union all
select 18767,'018767','1985-042K','deb SOZ','',58265,'In Earth orbit','','1988-01-08 00:00:00',104.95,854,1131,66.6,'Intermediate' from dual union all
select 18768,'018768','1987-110D','Blenda','Sunshade',60200,'Reentered','1988-02-26 00:00:00','1988-01-12 00:00:00',94.9,393,635,72.84,'Intermediate' from dual union all
select 18769,'018769','1987-110E','KDU part','KDU part',60200,'Reentered','1988-05-08 00:00:00','1988-02-03 00:00:00',94.2,387,573,72.82,'Intermediate' from dual union all
select 1877,'001877','1965-112J','deb Blok Kosmos','',11862,'Reentered','1968-03-22 00:00:00','1966-01-27 00:00:00',97.22,609,642,55.98,'Intermediate' from dual union all
select 18770,'018770','1987-110F','KDU','KDU',60200,'Reentered','1988-12-06 00:00:00','1988-02-03 00:00:00',92.19,354,409,72.82,'Intermediate' from dual union all
select 18771,'018771','1987-110G','KDU part','KDU part',60200,'Reentered','1988-04-09 00:00:00','1988-02-03 00:00:00',91.81,335,392,72.81,'Intermediate' from dual union all
select 18772,'018772','1987-110H','KDU part','KDU part',60200,'Reentered','1988-03-30 00:00:00','1988-02-03 00:00:00',92.01,348,399,72.81,'Intermediate' from dual union all
select 18773,'018773','1975-004HL','deb Delta 107','',36451,'Reentered','1995-12-23 00:00:00','1985-03-26 00:00:00',99.03,675,749,97.83,'Sun-Synch' from dual union all
select 18774,'018774','1986-019TV','deb Ariane V16','',59212,'Reentered','1989-04-02 00:00:00','1987-05-19 00:00:00',100.66,772,808,98.57,'Sun-Synch' from dual union all
select 18775,'018775','1978-026HH','deb Delta 139','',45221,'In Earth orbit','','1985-09-13 00:00:00',103.79,884,992,98.86,'Sun-Synch' from dual union all
select 18776,'018776','1979-017LQ','deb P78-1','',47501,'Reentered','1989-04-06 00:00:00','1987-09-25 00:00:00',94.95,502,531,97.75,'Sun-Synch' from dual union all
select 18777,'018777','1985-094V','deb Kosmos-1695','',58317,'In Earth orbit','','1988-01-14 00:00:00',114.07,1412,1414,82.31,'Intermediate' from dual union all
select 18778,'018778','1986-019TW','deb Ariane V16','',59212,'Reentered','1989-09-19 00:00:00','1987-11-02 00:00:00',100.77,763,828,98.58,'Sun-Synch' from dual union all
select 18779,'018779','1978-026HJ','deb Delta 139','',45221,'In Earth orbit','','1987-10-28 00:00:00',104.57,875,1074,99.03,'Sun-Synch' from dual union all
select 1878,'001878','1965-112K','deb Blok Kosmos','',11862,'Reentered','1970-07-02 00:00:00','1966-01-30 00:00:00',97.13,599,644,56.04,'Intermediate' from dual union all
select 18780,'018780','1986-019TX','deb Ariane V16','',59212,'Reentered','1989-05-19 00:00:00','1987-11-02 00:00:00',101.07,797,822,98.66,'Sun-Synch' from dual union all
select 18781,'018781','1986-019TY','deb Ariane V16','',59212,'Reentered','1989-02-21 00:00:00','1987-06-13 00:00:00',99.31,646,806,99.07,'Sun-Synch' from dual union all
select 18782,'018782','1986-019TZ','deb Ariane V16','',59212,'Reentered','1989-01-31 00:00:00','1987-11-01 00:00:00',100.87,775,825,98.68,'Sun-Synch' from dual union all
select 18783,'018783','1965-027AK','deb SNAPSHOT','',11777,'In Earth orbit','','1987-11-29 00:00:00',111.32,1268,1307,90.29,'Polar' from dual union all
select 18784,'018784','1965-027AL','deb SNAPSHOT','',11777,'In Earth orbit','','1987-11-20 00:00:00',111.31,1266,1308,90.3,'Polar' from dual union all
select 18785,'018785','1970-025PQ','deb Agena D','',21502,'In Earth orbit','','1985-07-31 00:00:00',106.41,1058,1063,100.13,'Sun-Synch' from dual union all
select 18786,'018786','1978-096E','deb Tiros N','',45291,'Reentered','1989-02-25 00:00:00','1987-11-04 00:00:00',101.82,819,870,99.06,'Sun-Synch' from dual union all
select 18787,'018787','1986-019UA','deb Ariane V16','',59212,'Reentered','1989-04-18 00:00:00','1987-11-17 00:00:00',105.85,731,1337,99,'Sun-Synch' from dual union all
select 18788,'018788','1988-002A','Kosmos-1909','Strela-3',61251,'In Earth orbit','','1988-02-15 00:00:00',114.04,1411,1413,82.61,'Intermediate' from dual union all
select 18789,'018789','1988-002B','Kosmos-1910','Strela-3',61251,'In Earth orbit','','1988-02-15 00:00:00',113.97,1407,1411,82.6,'Intermediate' from dual union all
select 1879,'001879','1965-082EU','deb Transtage 4','',11832,'Reentered','1966-03-22 00:00:00','1966-02-17 00:00:00',93.23,334,532,32.99,'Intermediate' from dual union all
select 18790,'018790','1988-002C','Kosmos-1911','Strela-3',61251,'In Earth orbit','','1988-02-15 00:00:00',113.91,1400,1412,82.61,'Intermediate' from dual union all
select 18791,'018791','1988-002D','Kosmos-1912','Strela-3',61251,'In Earth orbit','','1988-02-15 00:00:00',113.84,1395,1411,82.61,'Intermediate' from dual union all
select 18792,'018792','1988-002E','Kosmos-1913','Strela-3',61251,'In Earth orbit','','1988-02-15 00:00:00',113.8,1391,1410,82.6,'Intermediate' from dual union all
select 18793,'018793','1988-002F','Kosmos-1914','Strela-3',61251,'In Earth orbit','','1988-02-15 00:00:00',113.72,1384,1410,82.61,'Intermediate' from dual union all
select 18794,'018794','1988-002G','S5M','S5M',61251,'In Earth orbit','','1988-02-15 00:00:00',114.69,1411,1472,82.6,'Intermediate' from dual union all
select 18795,'018795','1988-003A','Progress-34','Progress 7K-TG No. 142',61252,'Deorbited','1988-03-04 00:00:00','1988-01-27 00:00:00',91.41,332,355,51.62,'Intermediate' from dual union all
select 18796,'018796','1988-003B','11S510','11S510',61252,'Reentered','1988-01-23 00:00:00','1988-01-22 00:00:00',88.31,168,214,51.64,'Intermediate' from dual union all
select 18797,'018797','1983-044EX','deb Kosmos-1461','',55801,'Reentered','1989-08-11 00:00:00','1988-01-20 00:00:00',98.94,563,853,65.02,'Intermediate' from dual union all
select 18798,'018798','1970-091AU','deb Kosmos-375','',21568,'Reentered','1989-01-23 00:00:00','1988-01-20 00:00:00',97.7,459,838,62.38,'Intermediate' from dual union all
select 18799,'018799','1985-030AA','deb Kosmos-1646','',58253,'Reentered','1988-07-05 00:00:00','1988-01-20 00:00:00',92.26,342,428,65,'Intermediate' from dual union all
select 188,'000188','61 Sigma 3','Midas 3 solar array cover','Strongback',7278,'In Earth orbit','','1961-09-29 00:00:00',161.11,3355,3505,91.13,'Medium' from dual union all
select 1880,'001880','1965-082EV','deb Transtage 4','',11832,'Reentered','1966-09-11 00:00:00','1966-08-20 00:00:00',92.85,361,468,32.27,'Intermediate' from dual union all
select 18800,'018800','1987-020AL','deb Kosmos-1823','',60110,'In Earth orbit','','1991-09-30 00:00:00',116.29,1477,1550,73.59,'Intermediate' from dual union all
select 18801,'018801','1987-020AM','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-10 00:00:00',113.24,1275,1476,73.14,'Intermediate' from dual union all
select 18802,'018802','1987-020AN','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-10 00:00:00',117.3,1477,1642,73.59,'Intermediate' from dual union all
select 18803,'018803','1987-020AP','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-06 00:00:00',116.94,1478,1609,73.45,'Intermediate' from dual union all
select 18804,'018804','1987-020AQ','deb Kosmos-1823','',60110,'In Earth orbit','','1991-09-20 00:00:00',115.99,1479,1522,73.6,'Intermediate' from dual union all
select 18805,'018805','1987-020AR','deb Kosmos-1823','',60110,'In Earth orbit','','1991-09-18 00:00:00',115.87,1479,1511,73.71,'Intermediate' from dual union all
select 18806,'018806','1987-020AS','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-06 00:00:00',116.83,1473,1604,73.48,'Intermediate' from dual union all
select 18807,'018807','1987-020AT','deb Kosmos-1823','',60110,'In Earth orbit','','1991-10-03 00:00:00',113.36,1255,1506,73.85,'Intermediate' from dual union all
select 18808,'018808','1987-110J','KDU part','KDU part',60200,'Reentered','1988-02-28 00:00:00','1988-02-03 00:00:00',93.04,373,474,71.88,'Intermediate' from dual union all
select 18809,'018809','1988-004A','Kosmos-1915','Oblik No. 363',61253,'Landed','1988-02-09 00:00:00','1988-01-31 00:00:00',90.65,226,387,72.88,'Intermediate' from dual union all
select 1881,'001881','1965-082EW','deb Transtage 4','',11832,'Reentered','1989-01-07 00:00:00','1989-01-05 00:00:00',88.72,208,215,32.06,'Intermediate' from dual union all
select 18810,'018810','1988-004B','11S510','11S510',61253,'Reentered','1988-02-07 00:00:00','1988-01-31 00:00:00',89.62,187,325,72.87,'Intermediate' from dual union all
select 18811,'018811','1970-025PR','deb Agena D','',21502,'Reentered','1989-08-14 00:00:00','1985-07-26 00:00:00',100.74,609,979,102.17,'Sun-Synch' from dual union all
select 18812,'018812','1983-044EY','deb Kosmos-1461','',55801,'Reentered','1991-02-12 00:00:00','1985-08-05 00:00:00',99.62,591,890,65.7,'Intermediate' from dual union all
select 18813,'018813','1986-019UB','deb Ariane V16','',59212,'Reentered','1988-12-02 00:00:00','1987-11-01 00:00:00',98.87,624,786,99.07,'Sun-Synch' from dual union all
select 18814,'018814','1978-026HK','deb Delta 139','',45221,'Reentered','1993-01-08 00:00:00','1987-11-02 00:00:00',101.52,805,857,98.66,'Sun-Synch' from dual union all
select 18815,'018815','1987-004GZ','deb Kosmos-1813','',60094,'Reentered','1989-02-09 00:00:00','1987-08-23 00:00:00',96.29,335,827,72.82,'Intermediate' from dual union all
select 18816,'018816','1978-014G','deb Kyokko','KM part',45209,'Reentered','1988-11-10 00:00:00','1987-10-31 00:00:00',139.69,1073,4007,64.93,'Medium' from dual union all
select 18817,'018817','1986-019UC','deb Ariane V16','',59212,'Reentered','1988-09-18 00:00:00','1987-08-08 00:00:00',101.7,731,948,98.01,'Sun-Synch' from dual union all
select 18818,'018818','1979-017LR','deb P78-1','',47501,'Reentered','1988-11-10 00:00:00','1987-10-11 00:00:00',94.12,474,477,98.1,'Sun-Synch' from dual;
insert into SATELLITE
select 18819,'018819','1985-042L','deb SOZ','',58265,'In Earth orbit','','1988-02-02 00:00:00',103.12,859,954,66.59,'Intermediate' from dual union all
select 1882,'001882','1965-082EX','deb Transtage 4','',11832,'Reentered','1970-07-14 00:00:00','1970-07-04 00:00:00',91.55,344,358,31.98,'Intermediate' from dual union all
select 18820,'018820','1988-005A','Meteor-2','Meteor-2 No. 20',61254,'In Earth orbit','','1988-02-29 00:00:00',104.03,937,961,82.54,'Intermediate' from dual union all
select 18821,'018821','1988-005B','S5M','S5M',61254,'In Earth orbit','','1988-02-29 00:00:00',103.99,937,958,82.54,'Intermediate' from dual union all
select 18822,'018822','1988-006A','DMSP F-9','DMSP Block 5D-2 S-8',61255,'In Earth orbit','','1988-09-02 00:00:00',101.38,819,829,98.74,'Sun-Synch' from dual union all
select 18823,'018823','1988-007A','Kosmos-1916','Kobal''t',61256,'Reentered','1988-02-29 00:00:00','1988-02-11 00:00:00',89.35,163,322,64.88,'Intermediate' from dual union all
select 18824,'018824','1988-007B','11S510','11S510',61256,'Reentered','1988-02-09 00:00:00','1988-02-05 00:00:00',89.47,167,330,64.89,'Intermediate' from dual union all
select 18825,'018825','1987-108C','deb Kosmos- 1906','',60198,'Reentered','1988-02-19 00:00:00','1988-02-08 00:00:00',92.7,221,593,82.38,'Intermediate' from dual union all
select 18826,'018826','1987-108D','deb Kosmos-1906','',60198,'Reentered','1988-02-28 00:00:00','1988-02-08 00:00:00',91.39,240,446,83.03,'Intermediate' from dual union all
select 18827,'018827','1987-108E','deb Kosmos-1906','',60198,'Reentered','1988-02-23 00:00:00','1988-02-06 00:00:00',92.19,241,523,82.9,'Intermediate' from dual union all
select 18828,'018828','1987-108F','deb Kosmos-1906','',60198,'Reentered','1988-04-01 00:00:00','1988-02-08 00:00:00',95.07,244,800,83.14,'Intermediate' from dual union all
select 18829,'018829','1987-108G','deb Kosmos-1906','',60198,'Reentered','1988-03-02 00:00:00','1988-02-07 00:00:00',91.81,200,526,84.03,'Intermediate' from dual union all
select 1883,'001883','1965-082EY','deb Transtage 4','',11832,'In Earth orbit','','1992-01-31 00:00:00',97.43,610,662,32.1,'Intermediate' from dual union all
select 18830,'018830','1987-108H','deb Kosmos-1906','',60198,'Reentered','1988-02-11 00:00:00','1988-02-07 00:00:00',90.71,214,405,84.32,'Intermediate' from dual union all
select 18831,'018831','1987-108J','deb Kosmos-1906','',60198,'Reentered','1988-02-13 00:00:00','1988-02-07 00:00:00',91.43,220,470,82.39,'Intermediate' from dual union all
select 18832,'018832','1987-108K','deb Kosmos-1906','',60198,'Reentered','1988-03-05 00:00:00','1988-02-04 00:00:00',91.05,218,435,83.22,'Intermediate' from dual union all
select 18833,'018833','1987-108L','deb Kosmos-1906','',60198,'Reentered','1988-04-07 00:00:00','1988-02-07 00:00:00',92.2,242,523,81.89,'Intermediate' from dual union all
select 18834,'018834','1987-108M','deb Kosmos-1906','',60198,'Reentered','1988-05-26 00:00:00','1988-02-07 00:00:00',92.79,243,580,82.83,'Intermediate' from dual union all
select 18835,'018835','1987-108N','deb Kosmos-1906','',60198,'Reentered','1988-03-19 00:00:00','1988-02-08 00:00:00',92.15,229,531,82.52,'Intermediate' from dual union all
select 18836,'018836','1987-108P','deb Kosmos-1906','',60198,'Reentered','1988-02-27 00:00:00','1988-02-07 00:00:00',91.22,238,431,83.17,'Intermediate' from dual union all
select 18837,'018837','1987-108Q','deb Kosmos-1906','',60198,'Reentered','1988-06-01 00:00:00','1988-02-08 00:00:00',92.59,243,560,82.89,'Intermediate' from dual union all
select 18838,'018838','1987-108R','deb Kosmos-1906','',60198,'Reentered','1988-02-09 00:00:00','1988-02-07 00:00:00',89.72,217,304,83.02,'Intermediate' from dual union all
select 18839,'018839','1987-108S','Priborniy otsek','PO',60198,'Reentered','1988-03-20 00:00:00','1988-02-06 00:00:00',90.8,215,412,83.37,'Intermediate' from dual union all
select 1884,'001884','1965-082EZ','deb Transtage 4','',11832,'Reentered','1990-10-14 00:00:00','1966-01-29 00:00:00',99.83,729,772,31.92,'Intermediate' from dual union all
select 18840,'018840','1987-108T','deb Kosmos-1906','',60198,'Reentered','1988-03-02 00:00:00','1988-02-07 00:00:00',92.39,209,575,82.32,'Intermediate' from dual union all
select 18841,'018841','1987-108U','deb Kosmos-1906','',60198,'Reentered','1988-02-13 00:00:00','1988-02-07 00:00:00',91.79,236,489,82.8,'Intermediate' from dual union all
select 18842,'018842','1987-108V','deb Kosmos-1906','',60198,'Reentered','1988-02-19 00:00:00','1988-02-08 00:00:00',90.61,231,378,83.63,'Intermediate' from dual union all
select 18843,'018843','1987-108W','deb Kosmos-1906','',60198,'Reentered','1988-02-22 00:00:00','1988-02-07 00:00:00',90.84,229,403,83.14,'Intermediate' from dual union all
select 18844,'018844','1987-108X','deb Kosmos-1906','',60198,'Reentered','1988-03-20 00:00:00','1988-02-08 00:00:00',91.79,243,482,83.15,'Intermediate' from dual union all
select 18845,'018845','1988-006B','DMSP F-9 solar array band','',61255,'Reentered','1992-10-29 00:00:00','1988-03-04 00:00:00',101.35,819,827,98.77,'Sun-Synch' from dual union all
select 18846,'018846','1988-006C','DMSP F-9 solar array band','',61255,'Reentered','1992-03-12 00:00:00','1988-03-04 00:00:00',101.35,820,826,98.77,'Sun-Synch' from dual union all
select 18847,'018847','1988-008A','Delta 181 Canister Cluster','Canister Cluster',61257,'Reentered','1988-03-01 00:00:00','1988-02-15 00:00:00',89.88,218,319,28.58,'Intermediate' from dual union all
select 18848,'018848','1988-008B','Delta 181 Sensor Module','Thrusted V. Sensor Modul',61257,'Reentered Attached','1988-04-02 00:00:00','1988-02-15 00:00:00',90.53,287,314,30.74,'Intermediate' from dual union all
select 18849,'018849','1988-008C','Delta 181 Subsatellite 7','Delta 181 SP2',61257,'Reentered','1988-03-05 00:00:00','1988-02-16 00:00:00',90.04,232,321,28.58,'Intermediate' from dual union all
select 1885,'001885','1965-082FA','deb Transtage 4','',11832,'Reentered','1970-01-30 00:00:00','1966-01-10 00:00:00',97.28,507,749,32.26,'Intermediate' from dual union all
select 18850,'018850','1988-004C','KDU','KDU',61253,'Reentered','1988-04-10 00:00:00','1988-02-26 00:00:00',90.29,220,358,72.87,'Intermediate' from dual union all
select 18851,'018851','1988-004D','Blenda','Sunshade',61253,'Reentered','1988-02-17 00:00:00','1988-02-10 00:00:00',90.35,222,361,72.87,'Intermediate' from dual union all
select 18852,'018852','1988-004E','KDU part','KDU part',61253,'Reentered','1988-02-21 00:00:00','1988-02-12 00:00:00',90.15,218,345,72.87,'Intermediate' from dual union all
select 18853,'018853','1988-004F','KDU part','KDU part',61253,'Reentered','1988-02-23 00:00:00','1988-02-12 00:00:00',92.23,253,515,72.89,'Intermediate' from dual union all
select 18854,'018854','1988-008D','Delta 181 SCRS','Sensor Calib. Ref. Sat.',61257,'Reentered','1988-02-13 00:00:00','1988-02-11 00:00:00',89.08,172,286,28.57,'Intermediate' from dual union all
select 18855,'018855','1986-067W','Kosmos-1776 SS 15','ESO',59260,'Reentered','1988-06-13 00:00:00','1988-02-18 00:00:00',94.36,469,507,73.98,'Intermediate' from dual union all
select 18856,'018856','1986-067X','Kosmos-1776 SS 16','ESO',59260,'Reentered','1988-05-28 00:00:00','1988-02-18 00:00:00',94.34,469,505,74.07,'Intermediate' from dual union all
select 18857,'018857','1988-009A','Kosmos-1919','Uragan No. 36L',61258,'Reentered Attached','1988-02-17 00:00:00','1988-02-17 00:00:00',87.28,132,148,64.82,'Intermediate' from dual union all
select 18858,'018858','1988-009B','8S812','8S812',61258,'Reentered','1988-02-19 00:00:00','1988-02-18 00:00:00',87.59,147,164,64.81,'Intermediate' from dual union all
select 18859,'018859','1988-009C','Perekhodnik','Sredniy perekhodnik',61258,'Reentered','1988-02-20 00:00:00','1988-02-19 00:00:00',89.03,140,314,64.8,'Intermediate' from dual union all
select 1886,'001886','1965-082FB','deb Transtage 4','',11832,'Reentered','1968-03-01 00:00:00','1968-03-02 00:00:00',91.13,322,338,32.08,'Intermediate' from dual union all
select 18860,'018860','1988-010A','Kosmos-1920','Resurs-F1 14F40 No. 106',61259,'Landed','1988-03-09 00:00:00','1988-02-25 00:00:00',91.15,321,341,82.6,'Intermediate' from dual union all
select 18861,'018861','1988-010B','11S510','11S510',61259,'Reentered','1988-02-20 00:00:00','1988-02-19 00:00:00',88.37,174,213,82.61,'Intermediate' from dual union all
select 18862,'018862','1987-108Y','deb Kosmos-1906','',60198,'Reentered','1988-03-13 00:00:00','1988-02-06 00:00:00',90.57,243,362,82.49,'Intermediate' from dual union all
select 18863,'018863','1987-108Z','deb Kosmos-1906','',60198,'Reentered','1988-02-28 00:00:00','1988-02-04 00:00:00',89.96,242,303,82.75,'Intermediate' from dual union all
select 18864,'018864','1977-065FV','deb Delta 132','',42193,'Reentered','2007-01-05 00:00:00','1984-10-19 00:00:00',108.35,529,1772,29.05,'Intermediate' from dual union all
select 18865,'018865','1986-019UD','deb Ariane V16','',59212,'Reentered','1989-11-13 00:00:00','1987-10-24 00:00:00',100.95,752,855,98.56,'Sun-Synch' from dual union all
select 18866,'018866','1970-091AV','deb Kosmos-375','',21568,'In Earth orbit','','1985-02-25 00:00:00',111.69,527,2082,62.66,'Intermediate' from dual union all
select 18867,'018867','1979-017LS','deb P78-1','',47501,'Reentered','1988-10-20 00:00:00','1987-10-31 00:00:00',94.57,471,525,97.74,'Sun-Synch' from dual union all
select 18868,'018868','1983-044EZ','deb Kosmos-1461','',55801,'Reentered','1988-12-26 00:00:00','1988-02-20 00:00:00',98.36,552,809,65.01,'Intermediate' from dual union all
select 18869,'018869','1983-044FA','deb Kosmos-1461','',55801,'Reentered','1988-11-02 00:00:00','1987-08-12 00:00:00',98.53,545,832,65.02,'Intermediate' from dual union all
select 1887,'001887','1965-082FC','deb Transtage 4','',11832,'Reentered','1979-05-23 00:00:00','1979-04-29 00:00:00',91.53,337,363,32.07,'Intermediate' from dual union all
select 18870,'018870','1973-086HA','deb Delta 98','',30688,'In Earth orbit','','1985-05-07 00:00:00',121.23,1448,2024,101.76,'Medium' from dual union all
select 18871,'018871','1979-017LT','deb P78-1','',47501,'Reentered','1990-02-14 00:00:00','1987-11-02 00:00:00',94.43,464,518,97.72,'Sun-Synch' from dual union all
select 18872,'018872','1977-065FW','deb Delta 132','',42193,'Reentered','1998-10-28 00:00:00','1989-04-03 00:00:00',104.3,475,1449,28.87,'Intermediate' from dual union all
select 18873,'018873','1979-017LU','deb P78-1','',47501,'Reentered','1989-04-26 00:00:00','1987-10-31 00:00:00',94.93,488,543,97.72,'Sun-Synch' from dual union all
select 18874,'018874','1965-082UN','deb Transtage','',11832,'Reentered','1990-02-28 00:00:00','1985-07-12 00:00:00',99.17,623,815,31.11,'Intermediate' from dual union all
select 18875,'018875','1988-011A','Kosmos-1921','Oblik',61260,'Landed','1988-03-04 00:00:00','1988-02-21 00:00:00',92.15,346,414,70.02,'Intermediate' from dual union all
select 18876,'018876','1988-011B','11S510','11S510',61260,'Reentered','1988-03-04 00:00:00','1988-02-24 00:00:00',89.78,196,331,70.02,'Intermediate' from dual union all
select 18877,'018877','1988-012A','Sakura 3A','CS-3A',61261,'In Earth orbit','','1988-04-11 00:00:00',1451.54,35742,36435,.11,'Drift GEO' from dual union all
select 18878,'018878','1988-012B','H-I Stage 2 S/N H-18(F)','LE-5',61261,'Reentered','1988-02-26 00:00:00','1988-02-21 00:00:00',89.53,195,307,30.68,'Intermediate' from dual union all
select 18879,'018879','1988-012C','UM129A','UM129A',61261,'Reentered','1998-09-05 00:00:00','1988-03-23 00:00:00',661.36,233,37301,28.25,'GEO Transfer' from dual union all
select 1888,'001888','1965-082FD','deb Transtage 4','',11832,'Reentered','1967-05-09 00:00:00','1967-04-26 00:00:00',92.18,344,419,31.98,'Intermediate' from dual union all
select 18880,'018880','1988-011C','Sensor cover','Sensor cover',61260,'Reentered','1988-02-21 00:00:00','1988-02-20 00:00:00',89.86,200,335,70.04,'Intermediate' from dual union all
select 18881,'018881','1988-013A','Kosmos-1922','Oko',61262,'In Earth orbit','','1988-03-27 00:00:00',718,646,39720,62.93,'Molniya' from dual union all
select 18882,'018882','1988-013B','11S510','11S510',61262,'Reentered','1988-04-05 00:00:00','1988-03-04 00:00:00',92.15,210,550,62.78,'Intermediate' from dual union all
select 18883,'018883','1988-013C','Blok-2BL','Blok-2BL No. 76745-677',61262,'In Earth orbit','','1988-03-29 00:00:00',705.83,645,39119,62.92,'Molniya' from dual union all
select 18884,'018884','1988-013D','BOZ','BOZ',61262,'Reentered','1988-03-25 00:00:00','1988-03-04 00:00:00',92.19,185,579,62.84,'Intermediate' from dual union all
select 18885,'018885','1986-017E','EO-3 EVA-1 debris','',59210,'Reentered','1988-03-08 00:00:00','1988-02-28 00:00:00',91.25,327,345,51.63,'Intermediate' from dual union all
select 18886,'018886','1986-017F','EO-3 EVA-1 debris','',59210,'Reentered','1988-03-21 00:00:00','1988-03-01 00:00:00',91.16,323,340,51.64,'Intermediate' from dual union all
select 18887,'018887','1986-017G','EO-3 EVA-1 debris','',59210,'Reentered','1988-03-02 00:00:00','1988-03-01 00:00:00',89.93,261,281,51.6,'Intermediate' from dual union all
select 18888,'018888','1987-108AA','deb Kosmos-1906','',60198,'Reentered','1988-03-07 00:00:00','1988-03-02 00:00:00',89.54,204,300,82.71,'Intermediate' from dual union all
select 18889,'018889','1986-017H','EO-3 EVA-1 debris','',59210,'Reentered','1988-03-05 00:00:00','1988-03-01 00:00:00',90.48,292,304,51.62,'Intermediate' from dual union all
select 1889,'001889','1965-082FE','deb Transtage 4','',11832,'Reentered','1968-01-09 00:00:00','1966-01-29 00:00:00',97.7,556,742,31.96,'Intermediate' from dual union all
select 18890,'018890','61 Omicron 286','deb r/b Ablestar','',7273,'Reentered','1989-06-30 00:00:00','1986-06-29 00:00:00',102.91,419,1374,66.01,'Intermediate' from dual union all
select 18891,'018891','1976-126BX','deb Kosmos-886','',39078,'Reentered','1990-01-03 00:00:00','1988-03-02 00:00:00',112.45,506,2173,65.82,'Intermediate' from dual union all
select 18892,'018892','1987-108AB','deb Kosmos-1906','',60198,'Reentered','1988-04-02 00:00:00','1988-02-09 00:00:00',90.37,242,344,82.79,'Intermediate' from dual union all
select 18893,'018893','1987-108AC','deb Kosmos-1906','',60198,'Reentered','1988-04-06 00:00:00','1988-02-04 00:00:00',90.1,245,314,82.49,'Intermediate' from dual union all
select 18894,'018894','1987-108AD','deb Kosmos-1906','',60198,'Reentered','1988-03-25 00:00:00','1988-02-04 00:00:00',93.17,242,618,82.89,'Intermediate' from dual union all
select 18895,'018895','1987-108AE','deb Kosmos-1906','',60198,'Reentered','1988-03-12 00:00:00','1988-02-04 00:00:00',89.89,240,298,82.49,'Intermediate' from dual union all
select 18896,'018896','1987-108AF','deb Kosmos-1906','',60198,'Reentered','1988-05-28 00:00:00','1988-02-23 00:00:00',94.92,241,788,83.04,'Intermediate' from dual union all
select 18897,'018897','1987-108AG','deb Kosmos-1906','',60198,'Reentered','1988-03-11 00:00:00','1988-02-25 00:00:00',89.99,227,321,82.75,'Intermediate' from dual union all
select 18898,'018898','1987-108AH','deb Kosmos-1906','',60198,'Reentered','1988-03-30 00:00:00','1988-02-26 00:00:00',91.53,236,464,82.76,'Intermediate' from dual union all
select 18899,'018899','1987-108AJ','deb Kosmos-1906','',60198,'Reentered','1988-04-06 00:00:00','1988-02-25 00:00:00',91.65,237,474,82.49,'Intermediate' from dual union all
select 189,'000189','61 A Gamma 1','Discoverer 32','CORONA C''''''-4',7254,'Reentered Attached','1961-11-13 00:00:00','1961-10-25 00:00:00',90.34,233,349,81.69,'Intermediate' from dual union all
select 1890,'001890','1965-082FF','deb Transtage 4','',11832,'Reentered','1967-03-28 00:00:00','1967-02-23 00:00:00',93.55,387,510,32.46,'Intermediate' from dual union all
select 18900,'018900','1987-108AK','deb Kosmos-1906','',60198,'Reentered','1988-03-12 00:00:00','1988-02-23 00:00:00',91.23,229,441,82.95,'Intermediate' from dual union all
select 18901,'018901','1974-089FB','deb Delta 104','',33692,'Reentered','1989-07-11 00:00:00','1988-02-25 00:00:00',117.64,725,2425,103.44,'Sun-Synch' from dual union all
select 18902,'018902','1981-053MB','deb Kosmos-1275','',52576,'In Earth orbit','','1987-11-02 00:00:00',104.39,951,981,82.97,'Intermediate' from dual union all
select 18903,'018903','1986-019UE','deb Ariane V16','',59212,'Reentered','1989-03-26 00:00:00','1987-11-29 00:00:00',100.78,772,819,98.56,'Sun-Synch' from dual union all
select 18904,'018904','1984-113F','deb Leasat 1 PKM','[UNID124] RCS 0.14',57167,'In Earth orbit','','1987-12-19 00:00:00',273.49,347,14811,26.72,'Highly Ellip' from dual union all
select 18905,'018905','1986-019UF','deb Ariane V16','',59212,'Reentered','1989-09-04 00:00:00','1987-11-03 00:00:00',100.89,771,831,98.62,'Sun-Synch' from dual union all
select 18906,'018906','1973-086HB','deb Delta 98','',30688,'In Earth orbit','','1985-05-07 00:00:00',114.67,1371,1509,102.36,'Sun-Synch' from dual union all
select 18907,'018907','1976-126BY','deb Kosmos-886','',39078,'In Earth orbit','','1985-07-31 00:00:00',112.21,497,2160,65.82,'Intermediate' from dual union all
select 18908,'018908','61 Omicron 287','deb r/b Ablestar','',7273,'In Earth orbit','','1987-11-02 00:00:00',103.05,731,1074,65.74,'Intermediate' from dual;
insert into SATELLITE
select 18909,'018909','1987-004HA','deb Kosmos-1813','',60094,'Reentered','1988-05-01 00:00:00','1988-03-02 00:00:00',91.02,310,340,72.81,'Intermediate' from dual union all
select 1891,'001891','1965-082FG','deb Transtage 4','',11832,'Reentered','1968-03-11 00:00:00','1968-02-22 00:00:00',92.62,403,403,32.02,'Intermediate' from dual union all
select 18910,'018910','1987-020AU','deb Kosmos-1823','',60110,'In Earth orbit','','1988-01-04 00:00:00',115.79,1480,1503,73.68,'Intermediate' from dual union all
select 18911,'018911','1967-014M','deb Explorer IX','[UNID125] RCS 0.01',15269,'In Earth orbit','','1967-07-03 00:00:00',117.77,608,2554,38.87,'Intermediate' from dual union all
select 18912,'018912','1969-070D','deb Tsiklon','[UNID126] RCS 0.04',18337,'Reentered','2014-08-06 00:00:00','1987-11-02 00:00:00',99.4,691,768,74.01,'Intermediate' from dual union all
select 18913,'018913','1974-066D','deb Kosmos-673','[UNID127] RCS 0.01',33669,'Reentered','1989-03-27 00:00:00','1987-11-02 00:00:00',93.85,441,484,81.22,'Intermediate' from dual union all
select 18914,'018914','1986-019UG','deb Ariane V16','',59212,'Reentered','1989-05-30 00:00:00','1987-11-22 00:00:00',100.96,785,824,98.64,'Sun-Synch' from dual union all
select 18915,'018915','1979-017LV','deb P78-1','',47501,'Reentered','1989-12-10 00:00:00','1987-10-23 00:00:00',95.44,535,545,97.85,'Sun-Synch' from dual union all
select 18916,'018916','1988-011D','KDU part','KDU part',61260,'Reentered','1988-05-29 00:00:00','1988-03-20 00:00:00',93.83,372,552,70.01,'Intermediate' from dual union all
select 18917,'018917','1988-011E','Blenda','Sunshade',61260,'Reentered','1988-06-22 00:00:00','1988-03-20 00:00:00',91.85,331,400,70.01,'Intermediate' from dual union all
select 18918,'018918','1988-011F','KDU','KDU',61260,'Reentered','1988-11-24 00:00:00','1988-03-28 00:00:00',92.05,346,405,70.01,'Intermediate' from dual union all
select 18919,'018919','1988-011G','KDU part','KDU part',61260,'Reentered','1988-04-15 00:00:00','1988-03-06 00:00:00',94.92,381,649,70.02,'Intermediate' from dual union all
select 1892,'001892','1965-082FH','deb Transtage 4','',11832,'Reentered','1967-04-16 00:00:00','1967-03-13 00:00:00',93.4,436,445,31.91,'Intermediate' from dual union all
select 18920,'018920','1988-011H','KDU part','KDU part',61260,'Reentered','1988-06-05 00:00:00','1988-03-20 00:00:00',93.35,368,509,70,'Intermediate' from dual union all
select 18921,'018921','1988-011J','KDU part','KDU part',61260,'Reentered','1988-04-19 00:00:00','1988-03-30 00:00:00',91.36,322,360,70.01,'Intermediate' from dual union all
select 18922,'018922','1988-014A','Zhongxing 1','DFH-2A',61263,'In Earth orbit','','1988-04-06 00:00:00',1436.04,35774,35797,.05,'Stationary' from dual union all
select 18923,'018923','1988-014B','CZ-3 Stage 3','CZ-3 Stage 3',61263,'Reentered','1989-07-20 00:00:00','1988-04-07 00:00:00',615.64,182,35007,31.08,'GEO Transfer' from dual union all
select 18924,'018924','1988-010C','KDU part','KDU part',61259,'Reentered','1988-03-11 00:00:00','1988-03-09 00:00:00',90.74,286,336,82.59,'Intermediate' from dual union all
select 18925,'018925','1988-010D','KDU part','KDU part',61259,'Reentered','1988-03-10 00:00:00','1988-03-09 00:00:00',91.06,319,335,82.65,'Intermediate' from dual union all
select 18926,'018926','1988-010E','Blenda','Sunshade',61259,'Reentered','1988-04-06 00:00:00','1988-03-14 00:00:00',90.88,305,330,82.59,'Intermediate' from dual union all
select 18927,'018927','1987-012K','deb Ginga','',60102,'Reentered','1993-04-13 00:00:00','1987-05-05 00:00:00',96.51,514,669,30.85,'Intermediate' from dual union all
select 18928,'018928','1967-014N','deb Diademe 2','',15269,'Reentered','1991-09-20 00:00:00','1984-03-23 00:00:00',103.39,543,1295,39.44,'Intermediate' from dual union all
select 18929,'018929','1967-014P','deb Diademe 2','',15269,'Reentered','1991-11-10 00:00:00','1982-10-30 00:00:00',104.42,556,1379,39.43,'Intermediate' from dual union all
select 1893,'001893','1965-082FJ','deb Transtage 4','',11832,'Reentered','1967-11-02 00:00:00','1966-01-28 00:00:00',96.61,461,731,32.41,'Intermediate' from dual union all
select 18930,'018930','1988-010F','KDU part','KDU part',61259,'Reentered','1988-04-08 00:00:00','1988-03-14 00:00:00',92.85,336,493,82.63,'Intermediate' from dual union all
select 18931,'018931','1988-015A','Kosmos-1923','Oblik No. 366',61264,'Landed','1988-03-22 00:00:00','1988-03-13 00:00:00',90.25,226,348,72.85,'Intermediate' from dual union all
select 18932,'018932','1988-015B','11S510','11S510',61264,'Reentered','1988-03-16 00:00:00','1988-03-15 00:00:00',88.44,170,225,72.85,'Intermediate' from dual union all
select 18933,'018933','1988-010G','KDU','KDU',61259,'Reentered','1988-07-02 00:00:00','1988-03-21 00:00:00',91.03,314,336,82.59,'Intermediate' from dual union all
select 18934,'018934','1988-010H','KDU part','KDU part',61259,'Reentered','1988-03-28 00:00:00','1988-03-14 00:00:00',90.88,308,328,82.59,'Intermediate' from dual union all
select 18935,'018935','1988-010J','KDU part','KDU part',61259,'Reentered','1988-03-26 00:00:00','1988-03-13 00:00:00',90.7,294,323,82.57,'Intermediate' from dual union all
select 18936,'018936','1988-015C','Sensor cover','Sensor cover',61264,'Reentered','1988-03-11 00:00:00','1988-03-10 00:00:00',88.92,185,258,72.84,'Intermediate' from dual union all
select 18937,'018937','1988-016A','Kosmos-1924','Strela-1M',61265,'In Earth orbit','','1988-04-11 00:00:00',115.7,1460,1515,74,'Intermediate' from dual union all
select 18938,'018938','1988-016B','Kosmos-1925','Strela-1M',61265,'In Earth orbit','','1988-04-10 00:00:00',115.49,1459,1497,74,'Intermediate' from dual union all
select 18939,'018939','1988-016C','Kosmos-1926','Strela-1M',61265,'In Earth orbit','','1988-04-11 00:00:00',115.32,1460,1480,74,'Intermediate' from dual union all
select 1894,'001894','1965-082FK','deb Transtage 4','',11832,'Reentered','1969-09-14 00:00:00','1966-01-24 00:00:00',96.76,443,764,32,'Intermediate' from dual union all
select 18940,'018940','1988-016D','Kosmos-1927','Strela-1M',61265,'In Earth orbit','','1988-04-11 00:00:00',115.12,1455,1467,74,'Intermediate' from dual union all
select 18941,'018941','1988-016E','Kosmos-1928','Strela-1M',61265,'In Earth orbit','','1988-04-11 00:00:00',114.94,1445,1461,74,'Intermediate' from dual union all
select 18942,'018942','1988-016F','Kosmos-1929','Strela-1M',61265,'In Earth orbit','','1988-04-11 00:00:00',114.77,1430,1460,74.01,'Intermediate' from dual union all
select 18943,'018943','1988-016G','Kosmos-1930','Strela-1M',61265,'In Earth orbit','','1988-04-11 00:00:00',114.6,1415,1459,74,'Intermediate' from dual union all
select 18944,'018944','1988-016H','Kosmos-1931','Strela-1M',61265,'In Earth orbit','','1988-04-11 00:00:00',114.41,1398,1460,74.01,'Intermediate' from dual union all
select 18945,'018945','1988-016J','S3M','S3M',61265,'In Earth orbit','','1988-04-11 00:00:00',117.65,1461,1689,74.01,'Intermediate' from dual union all
select 18946,'018946','1988-017A','Molniya-1','Molniya-1 No. 65',61266,'In Earth orbit','','1988-04-10 00:00:00',717.8,485,39871,62.99,'Molniya' from dual union all
select 18947,'018947','1988-017B','11S510','11S510',61266,'Reentered','1988-04-07 00:00:00','1988-03-18 00:00:00',90.77,228,397,63.01,'Intermediate' from dual union all
select 18948,'018948','1988-017C','BOZ','BOZ',61266,'Reentered','1988-04-09 00:00:00','1988-03-18 00:00:00',91.27,210,463,63,'Intermediate' from dual union all
select 18949,'018949','1988-017D','Blok-ML','Blok-ML No. 167',61266,'In Earth orbit','','1988-04-15 00:00:00',695.66,460,38798,62.93,'Molniya' from dual union all
select 1895,'001895','1965-082FL','deb Transtage 4','',11832,'Reentered','1966-09-08 00:00:00','1966-08-20 00:00:00',92.63,362,445,32.54,'Intermediate' from dual union all
select 18950,'018950','1987-108AL','deb Kosmos-1906','',60198,'Reentered','1988-03-20 00:00:00','1988-03-14 00:00:00',90.41,213,377,83.01,'Intermediate' from dual union all
select 18951,'018951','1988-018A','Spacenet 3R','Spacenet 3R',61267,'In Earth orbit','','1988-04-11 00:00:00',1436.09,35777,35796,.02,'Stationary' from dual union all
select 18952,'018952','1988-018B','Telecom 1C','Telecom 1C',61267,'In Earth orbit','','1988-04-26 00:00:00',1435.83,35777,35786,.05,'Stationary' from dual union all
select 18953,'018953','1988-018C','Ariane H10 V21','Ariane H10 S/N T21',61267,'In Earth orbit','','1988-04-12 00:00:00',640.33,274,36188,7.01,'GEO Transfer' from dual union all
select 18954,'018954','1988-018D','Sylda 4400 No. 8','Sylda 4400',61267,'Reentered','1989-02-01 00:00:00','1988-04-20 00:00:00',601.26,237,34202,7.01,'GEO Transfer' from dual union all
select 18955,'018955','1988-006D','DMSP F-9 OLS Cover','',61255,'Reentered','1995-06-30 00:00:00','1988-03-13 00:00:00',101.38,819,829,98.76,'Sun-Synch' from dual union all
select 18956,'018956','1988-006E','deb DMSP F-9','',61255,'Reentered','1989-02-02 00:00:00','1988-04-19 00:00:00',101.34,820,825,98.76,'Sun-Synch' from dual union all
select 18957,'018957','1988-019A','Kosmos-1932','US-A reactor',61268,'In Earth orbit','','1988-04-13 00:00:00',89.63,250,262,64.99,'Intermediate' from dual union all
select 18958,'018958','1988-020A','Kosmos-1933','Tselina-D',61269,'In Earth orbit','','1988-04-14 00:00:00',97.67,634,661,82.55,'Intermediate' from dual union all
select 18959,'018959','1988-020B','S5M','S5M',61269,'In Earth orbit','','1988-04-15 00:00:00',97.64,632,660,82.54,'Intermediate' from dual union all
select 1896,'001896','1965-082FM','deb Transtage 4','',11832,'Reentered','1966-09-23 00:00:00','1966-08-18 00:00:00',93.5,380,512,32.39,'Intermediate' from dual union all
select 18960,'018960','1988-021A','IRS-1A','IRS-1A',61270,'In Earth orbit','','1988-04-16 00:00:00',103.12,898,915,99.05,'Sun-Synch' from dual union all
select 18961,'018961','1988-021B','8A92ME','8A92ME',61270,'In Earth orbit','','1988-04-18 00:00:00',102.86,867,920,99.03,'Sun-Synch' from dual union all
select 18962,'018962','1963-014DN','deb West Ford','RCS 0.09',9245,'In Earth orbit','','1968-09-01 00:00:00',165.97,2996,4256,87.08,'Medium' from dual union all
select 18963,'018963','1963-014DP','deb West Ford','RCS 0.08',9245,'Reentered','2000-11-10 00:00:00','1988-02-22 00:00:00',164.72,1792,5360,86.42,'Medium' from dual union all
select 18964,'018964','1965-027AM','deb SNAPSHOT','',11777,'In Earth orbit','','1987-10-23 00:00:00',111.46,1272,1316,90.31,'Polar' from dual union all
select 18965,'018965','1965-027AN','deb SNAPSHOT','',11777,'In Earth orbit','','1988-02-19 00:00:00',111.48,1270,1320,90.31,'Polar' from dual union all
select 18966,'018966','1979-087D','SOZ','SOZ',47571,'Reentered','1980-06-19 00:00:00','1979-10-03 00:00:00',621.64,200,35300,47.8,'GEO Transfer' from dual union all
select 18967,'018967','1965-027AP','deb SNAPSHOT','',11777,'In Earth orbit','','1988-03-20 00:00:00',111.47,1269,1320,90.32,'Polar' from dual union all
select 18968,'018968','1987-020AV','deb Kosmos-1823','',60110,'Reentered','1988-09-23 00:00:00','1988-01-06 00:00:00',112.29,993,1671,74.22,'Intermediate' from dual union all
select 18969,'018969','1987-020AW','deb Kosmos-1823','',60110,'In Earth orbit','','1988-02-27 00:00:00',116.13,1484,1529,73.63,'Intermediate' from dual union all
select 1897,'001897','1965-082FN','deb Transtage 4','',11832,'Reentered','1967-02-07 00:00:00','1967-01-22 00:00:00',91.95,349,392,32.2,'Intermediate' from dual union all
select 18970,'018970','1987-020AX','deb Kosmos-1823','',60110,'Reentered','1990-05-18 00:00:00','1988-03-06 00:00:00',118.37,1401,1815,73.39,'Intermediate' from dual union all
select 18971,'018971','1979-087E','SOZ','SOZ  [TLEs for 63-014DQ]',47571,'Reentered','1980-05-06 00:00:00','1979-10-03 00:00:00',621.64,200,35300,47.8,'GEO Transfer' from dual union all
select 18972,'018972','1987-108AM','deb Kosmos-1906','',60198,'Reentered','1988-05-23 00:00:00','1988-02-04 00:00:00',94.23,244,719,82.97,'Intermediate' from dual union all
select 18973,'018973','1987-108AN','deb Kosmos-1906','',60198,'Reentered','1988-03-20 00:00:00','1988-03-19 00:00:00',88.89,184,256,82.7,'Intermediate' from dual union all
select 18974,'018974','1987-108AP','deb Kosmos-1906','',60198,'Reentered','1988-04-12 00:00:00','1988-03-20 00:00:00',91.35,225,457,82.72,'Intermediate' from dual union all
select 18975,'018975','1983-044FB','deb Kosmos-1461','',55801,'Reentered','1988-05-24 00:00:00','1988-03-20 00:00:00',99.34,674,781,65.57,'Intermediate' from dual union all
select 18976,'018976','1971-015DM','deb Kosmos-397','',24833,'Reentered','1990-01-03 00:00:00','1988-03-17 00:00:00',109.74,568,1862,65.92,'Intermediate' from dual union all
select 18977,'018977','1987-105E','deb Kosmos- 1903','',60195,'Reentered','1988-03-27 00:00:00','1988-03-20 00:00:00',89.39,189,300,62.8,'Intermediate' from dual union all
select 18978,'018978','1963-014DR','deb West Ford','RCS 0.08',9245,'Reentered','1996-05-30 00:00:00','1987-05-01 00:00:00',164.7,1701,5449,86.43,'Medium' from dual union all
select 18979,'018979','1963-014DS','deb West Ford','RCS 0.08',9245,'Reentered','1995-12-15 00:00:00','1988-02-22 00:00:00',164.6,1516,5626,85.98,'Medium' from dual union all
select 1898,'001898','1965-082FP','deb Transtage 4','',11832,'Reentered','1983-11-09 00:00:00','1983-11-06 00:00:00',90.52,290,310,31.9,'Intermediate' from dual union all
select 18980,'018980','1988-022A','Molniya-1','Molniya-1 No. 64',61271,'Reentered','2001-07-04 00:00:00','1988-04-16 00:00:00',717.74,628,39725,62.93,'Molniya' from dual union all
select 18981,'018981','1988-022B','11S510','11S510',61271,'Reentered','1988-04-28 00:00:00','1988-03-25 00:00:00',92.34,210,569,62.83,'Intermediate' from dual union all
select 18982,'018982','1988-022C','BOZ','BOZ',61271,'Reentered','1988-04-10 00:00:00','1988-03-25 00:00:00',92.13,181,577,62.84,'Intermediate' from dual union all
select 18983,'018983','1988-022D','Blok-ML','Blok-ML No. 176',61271,'Reentered','2002-01-23 00:00:00','1988-04-19 00:00:00',731.96,628,40424,62.92,'Molniya' from dual union all
select 18984,'018984','1988-006F','DMSP F-9 OLS Cooler Cover','',61255,'Reentered','2000-10-05 00:00:00','1988-03-17 00:00:00',101.38,819,829,98.76,'Sun-Synch' from dual union all
select 18985,'018985','1988-023A','Kosmos-1934','Parus',61272,'In Earth orbit','','1988-04-22 00:00:00',104.68,951,1007,82.96,'Intermediate' from dual union all
select 18986,'018986','1988-023B','S3M','S3M',61272,'In Earth orbit','','1988-04-22 00:00:00',104.57,953,996,82.96,'Intermediate' from dual union all
select 18987,'018987','1988-015D','KDU','KDU',61264,'Reentered','1988-04-01 00:00:00','1988-03-22 00:00:00',89.27,219,257,72.85,'Intermediate' from dual union all
select 18988,'018988','1988-015E','Blenda','Sunshade',61264,'Reentered','1988-03-24 00:00:00','1988-03-22 00:00:00',89.18,213,255,72.84,'Intermediate' from dual union all
select 18989,'018989','1988-015F','KDU part','KDU part',61264,'Reentered','1988-03-23 00:00:00','1988-03-22 00:00:00',90.31,239,341,73.14,'Intermediate' from dual union all
select 1899,'001899','1965-082FQ','deb Transtage 4','',11832,'Reentered','1967-07-11 00:00:00','1966-01-30 00:00:00',96.9,478,743,32.04,'Intermediate' from dual union all
select 18990,'018990','1988-015G','KDU part','KDU part',61264,'Reentered','1988-03-25 00:00:00','1988-03-22 00:00:00',89,211,240,72.84,'Intermediate' from dual union all
select 18991,'018991','1988-015H','KDU part','KDU part',61264,'Reentered','1988-03-27 00:00:00','1988-03-22 00:00:00',90.94,248,394,72.87,'Intermediate' from dual union all
select 18992,'018992','1988-024A','Progress-35','Progress 7K-TG No. 143',61273,'Deorbited','1988-05-05 00:00:00','1988-03-31 00:00:00',91.15,324,337,51.62,'Intermediate' from dual union all
select 18993,'018993','1988-024B','11S510','11S510',61273,'Reentered','1988-03-26 00:00:00','1988-03-25 00:00:00',88.36,169,217,51.62,'Intermediate' from dual union all
select 18994,'018994','1987-020AY','deb Kosmos-1823','',60110,'Reentered','1989-03-21 00:00:00','1987-12-21 00:00:00',111.42,1118,1466,73.78,'Intermediate' from dual union all
select 18995,'018995','1987-020AZ','deb Kosmos-1823','',60110,'Reentered','1990-02-24 00:00:00','1988-02-29 00:00:00',110.83,944,1586,73.84,'Intermediate' from dual union all
select 18996,'018996','1978-064E','deb Seasat 1','',45259,'Reentered','1988-08-10 00:00:00','1988-03-20 00:00:00',100.22,754,784,107.98,'Retrograde' from dual union all
select 18997,'018997','1977-121BR','deb Kosmos-970','',42249,'Reentered','2010-03-19 00:00:00','1988-02-28 00:00:00',109.94,567,1881,65.74,'Intermediate' from dual union all
select 18998,'018998','1977-121BS','deb Kosmos-970','',42249,'In Earth orbit','','1982-08-31 00:00:00',96.83,469,745,65.03,'Intermediate' from dual union all
select 18999,'018999','1963-014DT','deb West Ford','RCS 0.10',9245,'Reentered','1998-01-07 00:00:00','1988-02-21 00:00:00',164.65,1704,5442,86.3,'Medium' from dual;
insert into SATELLITE
select 19,'000019','59 Zeta','Discoverer 6','CORONA C-3',5538,'Reentered Attached','1959-10-20 00:00:00','1959-08-20 00:00:00',95.16,207,846,84,'Intermediate' from dual union all
select 190,'000190','61 A Gamma 2','deb Discoverer XXXII','Sensor cover',7254,'Reentered','1961-10-25 00:00:00','1961-10-19 00:00:00',89.86,233,301,81.69,'Intermediate' from dual union all
select 1900,'001900','1965-082FR','deb Transtage 4','',11832,'Reentered','1966-11-14 00:00:00','1966-11-10 00:00:00',90.34,270,313,32.52,'Intermediate' from dual union all
select 19000,'019000','1976-077FH','deb Delta 126','',39029,'In Earth orbit','','1985-05-07 00:00:00',117.39,1502,1626,102.02,'Sun-Synch' from dual union all
select 19001,'019001','1963-014DU','deb West Ford','RCS 0.03',9245,'Reentered','1997-01-16 00:00:00','1989-02-05 00:00:00',164.36,1578,5545,85.99,'Medium' from dual union all
select 19002,'019002','1963-014DV','deb West Ford','RCS 0.16',9245,'Reentered','1997-04-14 00:00:00','1974-03-30 00:00:00',164.87,1705,5459,86.19,'Medium' from dual union all
select 19003,'019003','1963-014DW','deb West Ford','RCS 0.07',9245,'Reentered','1998-05-13 00:00:00','1985-07-31 00:00:00',165.36,2055,5148,86.85,'Medium' from dual union all
select 19004,'019004','1988-013E','deb Kosmos- 1922','',61262,'Reentered','1988-04-07 00:00:00','1988-03-05 00:00:00',92.66,221,588,62.83,'Intermediate' from dual union all
select 19005,'019005','1987-020BA','deb Kosmos-1823','',60110,'In Earth orbit','','1988-02-28 00:00:00',116.02,1482,1522,73.87,'Intermediate' from dual union all
select 19006,'019006','1987-020BB','deb Kosmos-1823','',60110,'In Earth orbit','','1988-02-28 00:00:00',115.46,1391,1562,73.52,'Intermediate' from dual union all
select 19007,'019007','1986-019UH','deb Ariane V16','',59212,'Reentered','1988-10-28 00:00:00','1987-10-22 00:00:00',99.61,654,826,98.66,'Sun-Synch' from dual union all
select 19008,'019008','1984-088G','Delta 175 part','',57142,'In Earth orbit','','1986-11-14 00:00:00',133.31,554,3979,28.71,'Medium' from dual union all
select 19009,'019009','1986-019UJ','deb Ariane V16','',59212,'Reentered','1989-08-13 00:00:00','1987-11-21 00:00:00',100.96,787,821,98.54,'Sun-Synch' from dual union all
select 1901,'001901','1965-110B','Fairing','Fairing',11860,'Reentered','1966-01-06 00:00:00','1966-01-03 00:00:00',90.09,178,380,80,'Intermediate' from dual union all
select 19010,'019010','1964-006R','deb Elektron-1','',10292,'Reentered','1990-01-03 00:00:00','1988-03-29 00:00:00',133.91,392,4193,60.77,'Medium' from dual union all
select 19011,'019011','1988-025A','Kosmos-1935','Kobal''t No. 516',61274,'In Earth orbit','1988-04-08 00:00:00','1988-04-01 00:00:00',89.12,157,305,67.16,'Intermediate' from dual union all
select 19012,'019012','1988-025B','11S510','11S510',61274,'Reentered','1988-03-27 00:00:00','1988-03-26 00:00:00',88.73,158,266,67.16,'Intermediate' from dual union all
select 19013,'019013','1988-026A','San Marco 5','San Marco D/L',61275,'Reentered','1988-12-06 00:00:00','1988-04-24 00:00:00',93.16,261,598,3,'Equatorial' from dual union all
select 19014,'019014','1988-026B','Star 20 E-39','Star 20 E-39',61275,'Reentered','1988-06-16 00:00:00','1988-04-05 00:00:00',93.08,261,590,3,'Equatorial' from dual union all
select 19015,'019015','1988-027A','Kosmos-1936','Terilen',61276,'Deorbited','1988-05-18 00:00:00','1988-04-06 00:00:00',89.58,223,285,64.76,'Intermediate' from dual union all
select 19016,'019016','1988-027B','11S510','11S510',61276,'Reentered','1988-04-02 00:00:00','1988-04-01 00:00:00',88.47,171,227,64.77,'Intermediate' from dual union all
select 19017,'019017','1988-028A','Gorizont','Gorizont No. 26L',61277,'In Earth orbit','','1988-05-15 00:00:00',1436.14,35783,35792,1.36,'Stationary' from dual union all
select 19018,'019018','1988-028B','8S812','8S812',61277,'Reentered','1988-04-02 00:00:00','1988-04-01 00:00:00',87.94,169,175,51.62,'Intermediate' from dual union all
select 19019,'019019','1988-028C','Perekhodnik','Sredniy perekhodnik',61277,'Reentered','1988-03-31 00:00:00','1988-03-31 00:00:00',88.2,182,189,51.59,'Intermediate' from dual union all
select 1902,'001902','1965-108C','OSCAR 4','OSCAR 4',11858,'Reentered','1976-04-12 00:00:00','1965-12-21 00:00:00',587.62,162,33561,26.8,'Highly Ellip' from dual union all
select 19020,'019020','1988-028D','Blok DM No. 54L','11S86 No. 54L',61277,'In Earth orbit','','1988-05-08 00:00:00',1472.64,36421,36577,1.37,'Drift GEO' from dual union all
select 19021,'019021','1987-076C','Kosmos-1881 part','Terilen AO part',60166,'Reentered','1988-04-01 00:00:00','1988-03-31 00:00:00',88.5,190,211,64.72,'Intermediate' from dual union all
select 19022,'019022','1988-027C','Kosmos-1936 part','Terilen part',61276,'Reentered','1988-03-31 00:00:00','1988-03-30 00:00:00',87.96,154,193,64.77,'Intermediate' from dual union all
select 19023,'019023','1970-091AW','deb Kosmos-375','',21568,'Reentered','1989-12-27 00:00:00','1984-12-06 00:00:00',102.74,532,1244,62.89,'Intermediate' from dual union all
select 19024,'019024','1971-015DN','Unknown deb                       ','',24833,'In Earth orbit','','1984-12-18 00:00:00',110.07,806,1654,66.28,'Intermediate' from dual union all
select 19025,'019025','1981-053MC','deb Kosmos-1275','',52576,'In Earth orbit','','1985-03-25 00:00:00',105.04,960,1033,82.73,'Intermediate' from dual union all
select 19026,'019026','1981-053MD','deb Kosmos-1275','',52576,'In Earth orbit','','1985-03-26 00:00:00',106.89,969,1197,83.17,'Intermediate' from dual union all
select 19027,'019027','1981-053ME','deb Kosmos-1275','',52576,'In Earth orbit','','1985-03-30 00:00:00',106.17,971,1128,83.04,'Intermediate' from dual union all
select 19028,'019028','1969-082LC','deb r/b (AGENA D)','',18349,'Reentered','2000-03-26 00:00:00','1985-03-06 00:00:00',102.89,809,982,70.58,'Intermediate' from dual union all
select 19029,'019029','1963-014DX','deb West Ford','RCS 0.06',9245,'Reentered','1996-05-15 00:00:00','1986-11-02 00:00:00',165.03,1894,5283,86.56,'Medium' from dual union all
select 1903,'001903','1966-001A','Kosmos-104','Zenit-2 No. 36',13408,'Landed','1966-01-15 00:00:00','1966-01-08 00:00:00',90.24,193,380,65,'Intermediate' from dual union all
select 19030,'019030','1963-014DY','deb West Ford','RCS 0.28',9245,'In Earth orbit','','1967-04-16 00:00:00',166.6,3640,3663,87.34,'Medium' from dual union all
select 19031,'019031','61 Omicron 288','deb r/b Ablestar','',7273,'Reentered','1989-06-26 00:00:00','1988-03-30 00:00:00',109.51,804,1604,67.07,'Intermediate' from dual union all
select 19032,'019032','1963-014DZ','deb West Ford','RCS 0.08',9245,'Reentered','1993-08-27 00:00:00','1988-02-21 00:00:00',164.39,1597,5528,86.12,'Medium' from dual union all
select 19033,'019033','1973-086HC','deb Delta 98','',30688,'In Earth orbit','','1987-11-06 00:00:00',115.74,1464,1514,101.91,'Sun-Synch' from dual union all
select 19034,'019034','1986-019UK','deb Ariane V16','',59212,'Reentered','1988-02-26 00:00:00','1987-08-12 00:00:00',100.95,788,819,98.67,'Sun-Synch' from dual union all
select 19035,'019035','1987-076D','Kosmos-1881 part','Terilen AO part',60166,'Reentered','1988-04-02 00:00:00','1988-04-01 00:00:00',88.64,182,233,64.71,'Intermediate' from dual union all
select 19036,'019036','1988-028E','SOZ','SOZ',61277,'Reentered','1989-02-03 00:00:00','1988-04-30 00:00:00',648.48,306,36573,47.33,'GEO Transfer' from dual union all
select 19037,'019037','1988-028F','SOZ','SOZ',61277,'Reentered','1988-12-10 00:00:00','1988-05-01 00:00:00',647.15,249,36562,47.27,'GEO Transfer' from dual union all
select 19038,'019038','1988-029A','Kosmos-1937','Strela-2M',61278,'In Earth orbit','','1988-05-05 00:00:00',100.62,769,806,74.05,'Intermediate' from dual union all
select 19039,'019039','1988-029B','S3M','S3M',61278,'In Earth orbit','','1988-05-06 00:00:00',100.54,765,804,74.05,'Intermediate' from dual union all
select 1904,'001904','1966-001B','8A92E','8A92E',13408,'Reentered','1966-01-24 00:00:00','1966-01-08 00:00:00',90.29,189,389,65.03,'Intermediate' from dual union all
select 19040,'019040','1988-025C','deb Kosmos- 1935','',61274,'Reentered','1988-04-08 00:00:00','1988-04-08 00:00:00',87.99,146,204,67.16,'Intermediate' from dual union all
select 19041,'019041','1988-030A','Kosmos-1938','Oblik No. 377',61279,'Landed','1988-04-25 00:00:00','1988-04-13 00:00:00',89.62,223,288,72.87,'Intermediate' from dual union all
select 19042,'019042','1988-030B','11S510','11S510',61279,'Reentered','1988-04-16 00:00:00','1988-04-13 00:00:00',88.96,188,259,72.88,'Intermediate' from dual union all
select 19043,'019043','1988-031A','Foton','Foton 34KS No. 4L',61280,'Landed','1988-04-28 00:00:00','1988-04-16 00:00:00',90.4,216,373,62.82,'Intermediate' from dual union all
select 19044,'019044','1988-031B','11S510','11S510',61280,'Reentered','1988-04-29 00:00:00','1988-04-21 00:00:00',89.82,204,328,62.81,'Intermediate' from dual union all
select 19045,'019045','1988-032A','Kosmos-1939','Resurs-O1 No. 2L',61281,'Reentered','2014-10-29 00:00:00','1988-05-20 00:00:00',97.47,618,658,97.96,'Sun-Synch' from dual union all
select 19046,'019046','1988-032B','8A92ME','8A92ME',61281,'In Earth orbit','','1988-05-20 00:00:00',97.69,608,689,97.98,'Sun-Synch' from dual union all
select 19047,'019047','1981-053MF','deb Kosmos-1275','',52576,'In Earth orbit','','1985-05-22 00:00:00',104.95,954,1031,82.8,'Intermediate' from dual union all
select 19048,'019048','1988-029C','deb Kosmos-1937','Strela-2M debris',61278,'In Earth orbit','','1988-04-10 00:00:00',100.74,777,811,74.04,'Intermediate' from dual union all
select 19049,'019049','1966-013Q','deb Diapason','',13420,'Reentered','1989-01-22 00:00:00','1987-08-13 00:00:00',98.86,440,969,34.04,'Intermediate' from dual union all
select 1905,'001905','1966-001C','deb Kosmos-104','',13408,'Reentered','1966-01-12 00:00:00','1966-01-08 00:00:00',89.75,193,332,65,'Intermediate' from dual union all
select 19050,'019050','1986-019UL','deb Ariane V16','',59212,'Reentered','1989-08-26 00:00:00','1987-08-18 00:00:00',99.73,686,806,99.04,'Sun-Synch' from dual union all
select 19051,'019051','1963-014EA','deb West Ford cluster','RCS 0.31',9245,'In Earth orbit','','1987-08-14 00:00:00',166.3,3081,4198,87.23,'Medium' from dual union all
select 19052,'019052','1963-014EB','deb West Ford','RCS 0.05',9245,'Reentered','1997-09-27 00:00:00','1988-02-22 00:00:00',164.61,1643,5500,86.22,'Medium' from dual union all
select 19053,'019053','1975-004HM','deb Delta 107','',36451,'Reentered','1991-12-13 00:00:00','1988-03-26 00:00:00',103.19,904,915,98.83,'Sun-Synch' from dual union all
select 19054,'019054','1977-065FX','deb Delta 132','',42193,'In Earth orbit','','1980-06-16 00:00:00',111.88,756,1871,28.82,'Intermediate' from dual union all
select 19055,'019055','1963-014EC','deb West Ford','RCS 0.04',9245,'Reentered','2001-09-24 00:00:00','1987-08-21 00:00:00',165.58,2417,4804,87.04,'Medium' from dual union all
select 19056,'019056','1963-014ED','deb West Ford','RCS 0.14',9245,'Reentered','1997-09-17 00:00:00','1987-03-18 00:00:00',164.95,1809,5361,86.48,'Medium' from dual union all
select 19057,'019057','1963-014EE','deb West Ford','RCS 0.10',9245,'Reentered','1997-08-26 00:00:00','1988-02-22 00:00:00',164.88,2032,5133,86.7,'Medium' from dual union all
select 19058,'019058','1963-014EF','deb West Ford','RCS 0.19',9245,'Reentered','1989-03-28 00:00:00','1988-02-21 00:00:00',164.56,1632,5507,86.23,'Medium' from dual union all
select 19059,'019059','1963-014EG','deb West Ford','RCS 0.23',9245,'In Earth orbit','','1967-03-12 00:00:00',166.24,3597,3677,87.31,'Medium' from dual union all
select 1906,'001906','1965-112L','deb Blok Kosmos','',11862,'Reentered','1968-10-19 00:00:00','1966-01-30 00:00:00',97.36,624,641,56.28,'Intermediate' from dual union all
select 19060,'019060','1979-017LW','deb P78-1','',47501,'Reentered','1989-03-12 00:00:00','1988-02-29 00:00:00',94.94,502,530,98.39,'Sun-Synch' from dual union all
select 19061,'019061','1979-017LX','deb P78-1','',47501,'Reentered','1988-11-04 00:00:00','1988-04-20 00:00:00',93.44,418,467,97.8,'Sun-Synch' from dual union all
select 19062,'019062','1963-038M','deb Transit VBN-1','was [UNID128]',9269,'Reentered','1991-05-11 00:00:00','1985-07-31 00:00:00',106.62,1044,1097,89.84,'Polar' from dual union all
select 19063,'019063','1987-004HB','deb Kosmos-1813','',60094,'Reentered','1988-05-11 00:00:00','1988-04-20 00:00:00',90.64,236,376,72.8,'Intermediate' from dual union all
select 19064,'019064','1971-015DP','deb Kosmos-397','',24833,'Reentered','1989-02-02 00:00:00','1988-04-07 00:00:00',109.84,550,1889,65.82,'Intermediate' from dual union all
select 19065,'019065','1988-030C','KDU part','KDU part',61279,'Reentered','1988-04-29 00:00:00','1988-04-25 00:00:00',89.17,212,255,72.87,'Intermediate' from dual union all
select 19066,'019066','1988-030D','KDU part','KDU part',61279,'Reentered','1988-04-29 00:00:00','1988-04-25 00:00:00',89.06,205,251,72.87,'Intermediate' from dual union all
select 19067,'019067','1988-030E','KDU','KDU',61279,'Reentered','1988-05-07 00:00:00','1988-04-25 00:00:00',89.27,216,261,72.87,'Intermediate' from dual union all
select 19068,'019068','1988-030F','KDU part','KDU part',61279,'Reentered','1988-04-29 00:00:00','1988-04-26 00:00:00',91.3,235,441,72.88,'Intermediate' from dual union all
select 19069,'019069','1988-030G','Blenda','Sunshade',61279,'Reentered','1988-04-30 00:00:00','1988-04-25 00:00:00',91.15,239,423,72.87,'Intermediate' from dual union all
select 1907,'001907','1965-112M','deb Blok Kosmos','',11862,'Reentered','1968-10-16 00:00:00','1966-02-06 00:00:00',97.41,620,650,56.32,'Intermediate' from dual union all
select 19070,'019070','1988-033A','NNS O-23','NNS O-23',61282,'In Earth orbit','','1988-05-27 00:00:00',108.6,1017,1307,90.36,'Polar' from dual union all
select 19071,'019071','1988-033B','NNS O-32','NNS O-32',61282,'In Earth orbit','','1988-05-27 00:00:00',108.6,1016,1308,90.36,'Polar' from dual union all
select 19072,'019072','1988-033C','Star 20 E-41','Star 20 E-41',61282,'In Earth orbit','','1988-05-27 00:00:00',108.57,1017,1305,90.36,'Polar' from dual union all
select 19073,'019073','1988-034A','Kosmos-1940','Geofizika',61283,'In Earth orbit','','1988-05-28 00:00:00',1436.01,35777,35793,1.18,'Stationary' from dual union all
select 19074,'019074','1988-034B','8S812','8S812',61283,'Reentered','1988-04-28 00:00:00','1988-04-27 00:00:00',88.09,176,184,51.63,'Intermediate' from dual union all
select 19075,'019075','1988-034C','Perekhodnik','Sredniy perekhodnik',61283,'Reentered','1988-04-26 00:00:00','1988-04-26 00:00:00',88.37,184,204,51.62,'Intermediate' from dual union all
select 19076,'019076','1988-034D','Blok DM2','11S861 No. 12L',61283,'In Earth orbit','','1988-06-06 00:00:00',1438.26,35734,35923,1.21,'Drift GEO' from dual union all
select 19077,'019077','1988-033D','SOOS-3 adapter','',61282,'In Earth orbit','','1988-06-04 00:00:00',108.53,1015,1303,90.34,'Polar' from dual union all
select 19078,'019078','1988-033E','SOOS-3 despin weight','',61282,'In Earth orbit','','1988-05-26 00:00:00',107.92,1013,1247,90.58,'Polar' from dual union all
select 19079,'019079','1988-035A','Kosmos-1941','Oblik',61284,'Landed','1988-05-11 00:00:00','1988-04-29 00:00:00',89.29,224,255,70.34,'Intermediate' from dual union all
select 1908,'001908','1965-082FS','deb Transtage 4','',11832,'Reentered','1966-03-23 00:00:00','1966-01-31 00:00:00',93.76,333,583,32.1,'Intermediate' from dual union all
select 19080,'019080','1988-035B','11S510','11S510',61284,'Reentered','1988-05-02 00:00:00','1988-04-29 00:00:00',88.78,195,234,70.34,'Intermediate' from dual union all
select 19081,'019081','1988-035C','Sensor cover','Sensor cover',61284,'Reentered','1988-04-27 00:00:00','1988-04-27 00:00:00',88.36,181,206,70.37,'Intermediate' from dual union all
select 19082,'019082','1988-034E','SOZ','SOZ',61283,'Reentered','1989-01-29 00:00:00','1988-06-18 00:00:00',651.52,274,36760,47.39,'GEO Transfer' from dual union all
select 19083,'019083','1988-034F','SOZ','SOZ',61283,'Reentered','1989-01-08 00:00:00','1988-05-08 00:00:00',650.62,272,36716,47.41,'GEO Transfer' from dual union all
select 19084,'019084','1988-031C','KB electropitaniya','KBE',61280,'Reentered','1988-05-29 00:00:00','1988-04-28 00:00:00',90.22,214,357,62.82,'Intermediate' from dual union all
select 19085,'019085','1988-031D','KBE part','KBE part',61280,'Reentered','1988-05-05 00:00:00','1988-04-28 00:00:00',90.03,209,343,62.81,'Intermediate' from dual union all
select 19086,'019086','1988-031E','PTDU cover','KBE part',61280,'Reentered','1988-05-15 00:00:00','1988-04-29 00:00:00',92.2,313,451,62.83,'Intermediate' from dual union all
select 19087,'019087','1988-035D','Sensor cover','Sensor cover',61284,'Reentered','1988-05-03 00:00:00','1988-04-29 00:00:00',88.77,190,238,70.33,'Intermediate' from dual union all
select 19088,'019088','1988-031F','PTDU cover','KBE part',61280,'Reentered','1988-05-05 00:00:00','1988-04-29 00:00:00',92.26,313,458,62.82,'Intermediate' from dual;
insert into SATELLITE
select 19089,'019089','1988-024C','Progress-35 part','Progress part',61273,'Reentered','1988-05-12 00:00:00','1988-05-05 00:00:00',91.42,328,361,51.62,'Intermediate' from dual union all
select 1909,'001909','1965-082FT','deb Transtage 4','',11832,'Reentered','1966-07-19 00:00:00','1966-07-10 00:00:00',90.97,279,366,32.5,'Intermediate' from dual union all
select 19090,'019090','1988-036A','Ekran','Ekran No. 31L',61285,'In Earth orbit','','1988-06-06 00:00:00',1436.04,35768,35803,.37,'Stationary' from dual union all
select 19091,'019091','1988-036B','8S812','8S812',61285,'Reentered','1988-05-08 00:00:00','1988-05-06 00:00:00',88.38,189,200,51.6,'Intermediate' from dual union all
select 19092,'019092','1988-036C','Perekhodnik','Sredniy perekhodnik',61285,'Reentered','1988-05-06 00:00:00','1988-05-06 00:00:00',88.45,187,208,51.61,'Intermediate' from dual union all
select 19093,'019093','1988-036D','SOZ','SOZ',61285,'Reentered','1990-03-04 00:00:00','1988-06-11 00:00:00',628.92,242,35634,47.34,'GEO Transfer' from dual union all
select 19094,'019094','1988-036E','Blok DM','11S86 No. 66L',61285,'In Earth orbit','','1988-06-06 00:00:00',1424.07,35468,35634,.38,'Drift GEO' from dual union all
select 19095,'019095','1988-036F','SOZ','SOZ',61285,'Reentered','1989-07-16 00:00:00','1988-06-22 00:00:00',630.3,289,35658,47.25,'GEO Transfer' from dual union all
select 19096,'019096','1988-035E','KDU','KDU',61284,'Reentered','1988-05-22 00:00:00','1988-05-12 00:00:00',89.14,215,249,70.34,'Intermediate' from dual union all
select 19097,'019097','1988-035F','Blenda','Sunshade',61284,'Reentered','1988-05-14 00:00:00','1988-05-11 00:00:00',88.92,207,236,70.34,'Intermediate' from dual union all
select 19098,'019098','1988-035G','KDU part','KDU part',61284,'Reentered','1988-05-13 00:00:00','1988-05-11 00:00:00',91.85,242,488,70.35,'Intermediate' from dual union all
select 19099,'019099','1988-035H','KDU part','KDU part',61284,'Reentered','1988-05-16 00:00:00','1988-05-13 00:00:00',90.85,237,395,70.34,'Intermediate' from dual union all
select 191,'000191','61 A Gamma 3','deb Discoverer XXXII','Sensor cover',7254,'Reentered','1961-10-16 00:00:00','1961-10-14 00:00:00',90.12,233,328,81.69,'Intermediate' from dual union all
select 1910,'001910','1965-082FU','deb Transtage 4','',11832,'Reentered','1966-05-09 00:00:00','1966-05-06 00:00:00',90.35,292,292,32.3,'Intermediate' from dual union all
select 19100,'019100','1988-035J','KDU part','KDU part',61284,'Reentered','1988-05-12 00:00:00','1988-05-11 00:00:00',89.02,208,245,70.35,'Intermediate' from dual union all
select 19101,'019101','1983-044FC','deb Kosmos-1461','',55801,'Reentered','1989-05-04 00:00:00','1988-04-18 00:00:00',98.9,585,828,65.01,'Intermediate' from dual union all
select 19102,'019102','1982-051G','deb Kosmos (-1371) [UNID]','Strela-2M debris',54360,'In Earth orbit','','1988-05-02 00:00:00',100.98,792,818,74.06,'Intermediate' from dual union all
select 19103,'019103','1978-064F','deb Seasat 1','',45259,'Reentered','1988-06-11 00:00:00','1988-02-28 00:00:00',100.13,736,794,107.94,'Retrograde' from dual union all
select 19104,'019104','1987-020BC','deb Kosmos-1823','',60110,'Reentered','1989-01-04 00:00:00','1988-02-28 00:00:00',123.22,885,2764,73.66,'Medium' from dual union all
select 19105,'019105','1975-004HN','deb Delta 107','',36451,'In Earth orbit','','1985-04-15 00:00:00',99.92,698,811,98.77,'Sun-Synch' from dual union all
select 19106,'019106','1963-047T','Centaur AC-2 part','Insulation blanket part',9278,'In Earth orbit','','1968-09-24 00:00:00',109.94,684,1764,30.46,'Intermediate' from dual union all
select 19107,'019107','1986-019UM','deb Ariane V16','',59212,'Reentered','1991-05-02 00:00:00','1987-11-30 00:00:00',100.93,768,838,98.55,'Sun-Synch' from dual union all
select 19108,'019108','1963-014EH','deb West Ford','RCS 0.07',9245,'Reentered','2001-12-10 00:00:00','1987-11-12 00:00:00',164.78,1652,5505,86.18,'Medium' from dual union all
select 19109,'019109','1963-014EJ','deb West Ford','RCS 0.12',9245,'In Earth orbit','','1988-04-01 00:00:00',164.98,1900,5273,86.35,'Medium' from dual union all
select 1911,'001911','1965-082FV','deb Transtage 4','',11832,'Reentered','1966-06-21 00:00:00','1966-06-10 00:00:00',90.62,289,321,32.16,'Intermediate' from dual union all
select 19110,'019110','1963-047U','Centaur AC-2 part','Insulation blanket part',9278,'Reentered','1990-02-26 00:00:00','1983-03-13 00:00:00',107.8,605,1645,30.17,'Intermediate' from dual union all
select 19111,'019111','1985-094W','deb Kosmos-1695','',58317,'In Earth orbit','','1986-04-14 00:00:00',114.1,1409,1420,82.61,'Intermediate' from dual union all
select 19112,'019112','1976-126BZ','deb Kosmos-886','',39078,'In Earth orbit','','1979-01-21 00:00:00',113.66,509,2280,65.82,'Intermediate' from dual union all
select 19113,'019113','1965-027AQ','deb SNAPSHOT','',11777,'In Earth orbit','','1987-11-02 00:00:00',111.22,1260,1306,90.31,'Polar' from dual union all
select 19114,'019114','1971-015DQ','deb Kosmos-397','',24833,'Reentered','1989-02-02 00:00:00','1988-04-09 00:00:00',111.06,499,2052,65.82,'Intermediate' from dual union all
select 19115,'019115','1988-037A','Kosmos-1942','Kobal''t No. 839',61286,'Landed','1988-07-04 00:00:00','1988-05-19 00:00:00',89.66,170,346,67.14,'Intermediate' from dual union all
select 19116,'019116','1988-037B','11S510','11S510',61286,'Reentered','1988-05-16 00:00:00','1988-05-16 00:00:00',88.28,145,233,67.13,'Intermediate' from dual union all
select 19117,'019117','1988-038A','Progress-36','Progress 7K-TG No. 144',61287,'Deorbited','1988-06-05 00:00:00','1988-05-22 00:00:00',91.38,328,357,51.62,'Intermediate' from dual union all
select 19118,'019118','1988-038B','11S510','11S510',61287,'Reentered','1988-05-15 00:00:00','1988-05-15 00:00:00',87.6,148,164,51.65,'Intermediate' from dual union all
select 19119,'019119','1988-039A','Kosmos-1943','Tselina-2',61288,'In Earth orbit','','1988-06-14 00:00:00',101.91,847,852,71.01,'Intermediate' from dual union all
select 1912,'001912','1965-082FW','deb Transtage 4','',11832,'Reentered','1966-08-14 00:00:00','1966-08-11 00:00:00',92.9,392,442,31.96,'Intermediate' from dual union all
select 19120,'019120','1988-039B','11S772','11S772',61288,'In Earth orbit','','1988-06-30 00:00:00',101.61,818,851,71.02,'Intermediate' from dual union all
select 19121,'019121','1988-040A','INTELSAT 513','INTELSAT VA F13',61289,'In Earth orbit','','1988-06-17 00:00:00',1436.06,35770,35802,.3,'Stationary' from dual union all
select 19122,'019122','1988-040B','Ariane H10 V23','Ariane H10 S/N T22',61289,'In Earth orbit','','1988-06-18 00:00:00',637.07,477,35818,7.74,'GEO Transfer' from dual union all
select 19123,'019123','1988-041A','Kosmos-1944','Kometa No. 9',61290,'Landed','1988-06-24 00:00:00','1988-05-26 00:00:00',89.21,211,260,64.78,'Intermediate' from dual union all
select 19124,'019124','1988-041B','11S510','11S510',61290,'Reentered','1988-05-24 00:00:00','1988-05-23 00:00:00',88.29,173,207,64.79,'Intermediate' from dual union all
select 19125,'019125','1988-039C','Sep Motor Cover','Sep Motor Cover',61288,'In Earth orbit','','1988-06-14 00:00:00',104.69,844,1116,71.03,'Intermediate' from dual union all
select 19126,'019126','1988-039D','Sep Motor Cover','Sep Motor Cover',61288,'In Earth orbit','','1988-06-14 00:00:00',104.79,845,1125,71.03,'Intermediate' from dual union all
select 19127,'019127','1988-039E','Sep Motor Cover','Sep Motor Cover',61288,'In Earth orbit','','1988-06-14 00:00:00',105.19,845,1162,71,'Intermediate' from dual union all
select 19128,'019128','1988-039F','Sep Motor Cover','Sep Motor Cover',61288,'In Earth orbit','','1988-06-14 00:00:00',105.16,847,1157,70.99,'Intermediate' from dual union all
select 19129,'019129','1988-027D','Kosmos-1936 part','Terilen AO part',61276,'Reentered','1988-05-18 00:00:00','1988-05-18 00:00:00',87.74,134,192,65.01,'Intermediate' from dual union all
select 1913,'001913','1965-082FX','deb Transtage 4','',11832,'Reentered','1979-09-26 00:00:00','1966-04-22 00:00:00',97.53,541,740,32.28,'Intermediate' from dual union all
select 19130,'019130','1988-027E','Kosmos-1936 part','Terilen AO part',61276,'Reentered','1988-05-18 00:00:00','1988-05-18 00:00:00',88.23,164,209,64.65,'Intermediate' from dual union all
select 19131,'019131','1988-042A','Kosmos-1945','Oblik',61291,'Landed','1988-05-31 00:00:00','1988-05-23 00:00:00',90.02,231,320,70.36,'Intermediate' from dual union all
select 19132,'019132','1988-042B','11S510','11S510',61291,'Reentered','1988-06-01 00:00:00','1988-05-23 00:00:00',89.81,200,331,70.36,'Intermediate' from dual union all
select 19133,'019133','1978-100F','deb S5M','',45295,'In Earth orbit','','1988-05-13 00:00:00',119.7,1631,1704,82.42,'Intermediate' from dual union all
select 19134,'019134','1978-100G','deb S5M','',45295,'In Earth orbit','','1988-05-12 00:00:00',119.22,1592,1699,82.42,'Intermediate' from dual union all
select 19135,'019135','1978-100H','deb S5M','',45295,'Reentered','2013-06-14 00:00:00','1988-05-11 00:00:00',119.3,1592,1707,82.41,'Intermediate' from dual union all
select 19136,'019136','1978-100J','deb S5M','',45295,'In Earth orbit','','1988-05-16 00:00:00',112.52,981,1704,82.42,'Intermediate' from dual union all
select 19137,'019137','1978-100K','deb S5M','',45295,'In Earth orbit','','1988-05-16 00:00:00',120.23,1684,1698,82.34,'Medium' from dual union all
select 19138,'019138','1978-100L','deb S5M','',45295,'In Earth orbit','','1988-05-10 00:00:00',117.52,1434,1705,82.33,'Intermediate' from dual union all
select 19139,'019139','1978-100M','deb S5M','',45295,'In Earth orbit','','1988-05-16 00:00:00',123.11,1699,1940,82.15,'Medium' from dual union all
select 1914,'001914','1965-082FY','deb Transtage 4','',11832,'Reentered','1979-12-09 00:00:00','1979-11-08 00:00:00',95.44,519,561,33,'Intermediate' from dual union all
select 19140,'019140','1988-033F','SOOS-3 adapter','',61282,'In Earth orbit','','1988-05-26 00:00:00',108.56,1015,1305,90.36,'Polar' from dual union all
select 19141,'019141','1987-020BD','deb Kosmos-1823','',60110,'In Earth orbit','','1988-04-07 00:00:00',115.14,1433,1491,74.02,'Intermediate' from dual union all
select 19142,'019142','1978-100N','deb S5M','',45295,'Reentered','2009-02-22 00:00:00','1988-05-11 00:00:00',119.35,1598,1706,82.39,'Intermediate' from dual union all
select 19143,'019143','1978-100P','deb S5M','',45295,'In Earth orbit','','1988-05-10 00:00:00',114.73,1181,1706,81.64,'Intermediate' from dual union all
select 19144,'019144','1978-100Q','deb S5M','',45295,'In Earth orbit','','1988-05-19 00:00:00',121.23,1698,1774,82.62,'Medium' from dual union all
select 19145,'019145','1987-020BE','deb Kosmos-1823','',60110,'Reentered','1999-02-25 00:00:00','1993-09-14 00:00:00',108.37,1042,1261,73.28,'Intermediate' from dual union all
select 19146,'019146','1987-020BF','deb Kosmos-1823','',60110,'Reentered','1989-01-21 00:00:00','1987-12-19 00:00:00',115.44,1444,1507,73.75,'Intermediate' from dual union all
select 19147,'019147','1987-020BG','deb Kosmos-1823','',60110,'Reentered','1993-09-23 00:00:00','1988-01-04 00:00:00',116.03,1475,1529,73.51,'Intermediate' from dual union all
select 19148,'019148','1980-089CE','deb Kosmos-1220','',50299,'Reentered','1988-10-16 00:00:00','1988-04-07 00:00:00',97.06,514,721,65.03,'Intermediate' from dual union all
select 19149,'019149','1987-020BH','deb Kosmos-1823','',60110,'Reentered','1991-02-17 00:00:00','1988-04-07 00:00:00',117.21,1316,1795,73.34,'Intermediate' from dual union all
select 1915,'001915','1965-082FZ','deb Transtage 4','',11832,'In Earth orbit','','1992-01-28 00:00:00',99.34,704,751,31.88,'Intermediate' from dual union all
select 19150,'019150','1970-025PS','deb Agena D','',21502,'In Earth orbit','','1988-04-29 00:00:00',108.03,1090,1182,100.37,'Sun-Synch' from dual union all
select 19151,'019151','1987-020BJ','deb Kosmos-1823','',60110,'Reentered','1988-12-01 00:00:00','1988-02-22 00:00:00',112.11,991,1657,73.48,'Intermediate' from dual union all
select 19152,'019152','1987-020BK','deb Kosmos-1823','',60110,'Reentered','1990-03-29 00:00:00','1988-04-16 00:00:00',112.71,942,1761,73.22,'Intermediate' from dual union all
select 19153,'019153','1987-020BL','deb Kosmos-1823','',60110,'Reentered','1988-10-13 00:00:00','1988-02-29 00:00:00',111.98,970,1666,73.53,'Intermediate' from dual union all
select 19154,'019154','1974-089FC','deb Delta 104','',33692,'Reentered','1989-11-30 00:00:00','1987-10-31 00:00:00',112.34,924,1745,100.13,'Sun-Synch' from dual union all
select 19155,'019155','1987-020BM','deb Kosmos-1823','',60110,'Reentered','1989-03-10 00:00:00','1988-05-17 00:00:00',113.26,819,1934,73.13,'Intermediate' from dual union all
select 19156,'019156','1979-017LY','deb P78-1','',47501,'Reentered','1989-03-17 00:00:00','1988-05-18 00:00:00',94.51,489,501,97.96,'Sun-Synch' from dual union all
select 19157,'019157','1963-014EK','deb West Ford','RCS 0.08',9245,'Reentered','1996-05-13 00:00:00','1987-12-21 00:00:00',164.47,1820,5312,86.35,'Medium' from dual union all
select 19158,'019158','1986-019UN','deb Ariane V16','',59212,'Reentered','1989-12-18 00:00:00','1988-04-20 00:00:00',100.97,771,839,98.59,'Sun-Synch' from dual union all
select 19159,'019159','1987-020BN','deb Kosmos-1823','',60110,'Reentered','1988-10-01 00:00:00','1988-03-23 00:00:00',114.89,1326,1575,73.41,'Intermediate' from dual union all
select 1916,'001916','1965-082GA','deb Transtage 4','',11832,'Reentered','1981-02-01 00:00:00','1966-01-26 00:00:00',99.29,693,757,31.94,'Intermediate' from dual union all
select 19160,'019160','1988-019B','Kosmos-1932 DU','US-A DU',61268,'Reentered','1988-06-29 00:00:00','1988-05-23 00:00:00',89.51,241,260,64.99,'Intermediate' from dual union all
select 19161,'019161','1988-019C','Kosmos-1932 RLS','US-A RLS',61268,'Reentered','1988-05-24 00:00:00','1988-05-23 00:00:00',88.24,176,199,64.98,'Intermediate' from dual union all
select 19162,'019162','1988-019D','Kosmos-1932 reactor core','US-A reactor core',61268,'In Earth orbit','','1988-05-23 00:00:00',104.06,923,978,65.04,'Intermediate' from dual union all
select 19163,'019163','1988-043A','Kosmos-1946','Uragan No. 39L',61292,'In Earth orbit','','1988-06-21 00:00:00',675.71,19110,19149,64.91,'Medium' from dual union all
select 19164,'019164','1988-043B','Kosmos-1947','Uragan No. 40L',61292,'In Earth orbit','','1988-06-21 00:00:00',675.69,19114,19144,64.92,'Medium' from dual union all
select 19165,'019165','1988-043C','Kosmos-1948','Uragan No. 41L',61292,'In Earth orbit','','1988-06-21 00:00:00',675.71,19117,19142,64.9,'Medium' from dual union all
select 19166,'019166','1988-043D','8S812','8S812',61292,'Reentered','1988-05-22 00:00:00','1988-05-22 00:00:00',86.76,107,121,64.86,'Intermediate' from dual union all
select 19167,'019167','1988-043E','Perekhodnik','Sredniy perekhodnik',61292,'Reentered','1988-05-21 00:00:00','1988-05-21 00:00:00',87.72,158,166,64.85,'Intermediate' from dual union all
select 19168,'019168','1988-043F','Blok DM2','11S861 No. 13L',61292,'In Earth orbit','','1988-06-21 00:00:00',674.46,19076,19120,64.9,'Medium' from dual union all
select 19169,'019169','1988-043G','SOZ','SOZ',61292,'In Earth orbit','','1988-07-17 00:00:00',340.15,416,19111,64.95,'Highly Ellip' from dual union all
select 1917,'001917','1965-082GB','deb Transtage 4','',11832,'Reentered','1982-04-07 00:00:00','1966-01-24 00:00:00',97.78,561,743,31.9,'Intermediate' from dual union all
select 19170,'019170','1988-043H','SOZ','SOZ',61292,'In Earth orbit','','1988-06-26 00:00:00',340.23,411,19120,64.84,'Highly Ellip' from dual union all
select 19171,'019171','1980-089CF','deb Kosmos-1220','',50299,'Reentered','1989-01-12 00:00:00','1987-11-10 00:00:00',97.33,527,735,65,'Intermediate' from dual union all
select 19172,'019172','1983-044FD','deb Kosmos-1461','',55801,'Reentered','1989-10-27 00:00:00','1988-04-21 00:00:00',98.3,560,795,65.03,'Intermediate' from dual union all
select 19173,'019173','1964-006S','deb Elektron-1','',10292,'Reentered','1999-07-15 00:00:00','1985-01-04 00:00:00',131.26,387,3968,60.79,'Medium' from dual union all
select 19174,'019174','1978-100R','deb S5M','',45295,'In Earth orbit','','1988-05-30 00:00:00',118.01,1471,1712,81.9,'Intermediate' from dual union all
select 19175,'019175','1978-100S','deb S5M','',45295,'In Earth orbit','','1988-05-17 00:00:00',118.64,1554,1685,82.53,'Intermediate' from dual union all
select 19176,'019176','1978-100T','deb S5M','',45295,'In Earth orbit','','1988-05-20 00:00:00',120.78,1594,1838,82.09,'Medium' from dual union all
select 19177,'019177','1980-089CG','deb Kosmos-1220','',50299,'Reentered','1989-01-17 00:00:00','1988-02-29 00:00:00',96.89,494,726,65,'Intermediate' from dual union all
select 19178,'019178','1978-100U','deb S5M','',45295,'In Earth orbit','','1988-05-19 00:00:00',119.38,1606,1701,82.15,'Intermediate' from dual;
insert into SATELLITE
select 19179,'019179','1987-020BP','deb Kosmos-1823','',60110,'Reentered','1988-12-27 00:00:00','1988-04-18 00:00:00',108.21,793,1495,74.4,'Intermediate' from dual union all
select 1918,'001918','1965-082GC','deb Transtage 4','',11832,'Reentered','1978-11-10 00:00:00','1978-09-28 00:00:00',94.04,471,474,31.7,'Intermediate' from dual union all
select 19180,'019180','1987-020BQ','deb Kosmos-1823','',60110,'Reentered','1990-06-14 00:00:00','1988-04-15 00:00:00',112.81,1080,1631,73.36,'Intermediate' from dual union all
select 19181,'019181','1988-033G','SOOS-3 despin weight','',61282,'In Earth orbit','','1988-05-26 00:00:00',109.23,996,1387,90.16,'Polar' from dual union all
select 19182,'019182','1965-082UP','deb Transtage','',11832,'Reentered','1991-02-08 00:00:00','1988-04-04 00:00:00',98.67,640,750,32.03,'Intermediate' from dual union all
select 19183,'019183','1974-089FD','deb Delta 104','',33692,'In Earth orbit','','1977-05-12 00:00:00',111.42,1118,1466,103.59,'Sun-Synch' from dual union all
select 19184,'019184','1987-020BR','deb Kosmos-1823','',60110,'Reentered','1991-02-01 00:00:00','1988-04-09 00:00:00',112.31,1077,1589,73.73,'Intermediate' from dual union all
select 19185,'019185','1965-027AR','deb SNAPSHOT','',11777,'In Earth orbit','','1988-05-05 00:00:00',111.46,1272,1316,90.32,'Polar' from dual union all
select 19186,'019186','1987-020BS','deb Kosmos-1823','',60110,'Reentered','1989-04-23 00:00:00','1988-04-05 00:00:00',113.69,1032,1760,73.18,'Intermediate' from dual union all
select 19187,'019187','1988-022E','deb Blok-I','',61271,'Reentered','1988-07-06 00:00:00','1988-05-18 00:00:00',91.4,221,465,62.82,'Intermediate' from dual union all
select 19188,'019188','1987-020BT','deb Kosmos-1823','',60110,'Reentered','1989-06-26 00:00:00','1988-03-28 00:00:00',114.47,1048,1815,73.14,'Intermediate' from dual union all
select 19189,'019189','1988-044A','Molniya-3','Molniya-3 No. 49',61293,'Reentered','2002-03-15 00:00:00','1988-06-26 00:00:00',717.72,576,39776,62.93,'Molniya' from dual union all
select 1919,'001919','1965-082GD','deb Transtage 4','',11832,'Reentered','1978-12-02 00:00:00','1978-11-24 00:00:00',91.26,326,347,32.21,'Intermediate' from dual union all
select 19190,'019190','1988-044B','Blok-ML','Blok-ML No. 77331-531',61293,'Reentered','2003-01-19 00:00:00','1988-06-27 00:00:00',733.26,570,40546,62.92,'Molniya' from dual union all
select 19191,'019191','1988-044C','11S510','11S510',61293,'Reentered','1988-07-07 00:00:00','1988-06-02 00:00:00',92.01,214,533,62.78,'Intermediate' from dual union all
select 19192,'019192','1988-044D','BOZ','BOZ',61293,'Reentered','1988-06-28 00:00:00','1988-06-02 00:00:00',92.22,191,576,62.82,'Intermediate' from dual union all
select 19193,'019193','1988-045A','Kosmos-1949','US-PU',61294,'Reentered','1990-04-23 00:00:00','1988-06-27 00:00:00',92.77,403,418,65.03,'Intermediate' from dual union all
select 19194,'019194','1988-045B','11S692','11S692',61294,'Reentered','1988-05-28 00:00:00','1988-05-28 00:00:00',89.39,111,377,65,'Intermediate' from dual union all
select 19195,'019195','1988-046A','Kosmos-1950','Musson No. 21',61295,'In Earth orbit','','1988-06-30 00:00:00',116.04,1485,1521,73.6,'Intermediate' from dual union all
select 19196,'019196','1988-046B','S5M','S5M',61295,'In Earth orbit','','1988-06-30 00:00:00',115.99,1484,1517,73.6,'Intermediate' from dual union all
select 19197,'019197','1988-047A','Kosmos-1951','Resurs-F1 14F43 No. 28',61296,'Landed','1988-06-14 00:00:00','1988-06-02 00:00:00',89.83,259,274,82.33,'Intermediate' from dual union all
select 19198,'019198','1988-047B','11S510','11S510',61296,'Reentered','1988-06-03 00:00:00','1988-06-02 00:00:00',87.86,155,183,82.32,'Intermediate' from dual union all
select 19199,'019199','1988-042C','Blenda','Sunshade',61291,'Reentered','1988-06-03 00:00:00','1988-05-31 00:00:00',91.16,250,412,70.31,'Intermediate' from dual union all
select 192,'000192','61 A Delta 1','Midas 4','Midas ETS-II F2',7251,'In Earth orbit','','1961-12-17 00:00:00',165.94,3503,3747,95.9,'Medium' from dual union all
select 1920,'001920','1965-082GE','deb Transtage 4','',11832,'Reentered','1980-02-08 00:00:00','1966-01-28 00:00:00',99.33,675,779,32.65,'Intermediate' from dual union all
select 19200,'019200','1988-042D','KDU part','KDU part',61291,'Reentered','1988-06-04 00:00:00','1988-05-31 00:00:00',90.92,246,394,70.36,'Intermediate' from dual union all
select 19201,'019201','1988-042E','KDU','KDU',61291,'Reentered','1988-06-10 00:00:00','1988-05-31 00:00:00',89.08,212,246,70.36,'Intermediate' from dual union all
select 19202,'019202','1988-042F','KDU part','KDU part',61291,'Reentered','1988-06-01 00:00:00','1988-05-31 00:00:00',88.85,202,233,70.39,'Intermediate' from dual union all
select 19203,'019203','1988-042G','KDU part','KDU part',61291,'Reentered','1988-06-02 00:00:00','1988-05-31 00:00:00',88.75,199,227,70.36,'Intermediate' from dual union all
select 19204,'019204','1988-048A','Soyuz TM-5','Soyuz 7K-STM No. 55',61297,'Landed','1988-09-07 00:00:00','1988-07-07 00:00:00',91.46,345,348,51.62,'Intermediate' from dual union all
select 19205,'019205','1988-048B','11S510','11S510',61297,'Reentered','1988-06-09 00:00:00','1988-06-09 00:00:00',87.58,152,158,51.61,'Intermediate' from dual union all
select 19206,'019206','1988-049A','Kosmos-1952','Oblik',61298,'Landed','1988-06-25 00:00:00','1988-06-14 00:00:00',89.64,229,285,69.98,'Intermediate' from dual union all
select 19207,'019207','1988-049B','11S510','11S510',61298,'Reentered','1988-06-17 00:00:00','1988-06-15 00:00:00',88.53,185,219,69.97,'Intermediate' from dual union all
select 19208,'019208','1988-049C','Sensor cover','Sensor cover',61298,'Reentered','1988-06-12 00:00:00','1988-06-11 00:00:00',88.54,191,213,70,'Intermediate' from dual union all
select 19209,'019209','1988-049D','Sensor cover','Sensor cover',61298,'Reentered','1988-06-12 00:00:00','1988-06-12 00:00:00',88.81,191,241,69.95,'Intermediate' from dual union all
select 1921,'001921','1965-082GF','deb Transtage 4','',11832,'Reentered','1967-02-24 00:00:00','1967-01-25 00:00:00',93.39,404,476,31.9,'Intermediate' from dual union all
select 19210,'019210','1988-050A','Kosmos-1953','Tselina-D',61299,'In Earth orbit','','1988-07-14 00:00:00',97.71,631,667,82.53,'Intermediate' from dual union all
select 19211,'019211','1988-050B','S5M','S5M',61299,'In Earth orbit','','1988-07-15 00:00:00',97.68,630,666,82.53,'Intermediate' from dual union all
select 19212,'019212','1988-047C','KDU','KDU',61296,'Reentered','1988-07-17 00:00:00','1988-06-15 00:00:00',89.73,253,270,82.33,'Intermediate' from dual union all
select 19213,'019213','1988-047D','Blenda','Sunshade',61296,'Reentered','1988-06-19 00:00:00','1988-06-15 00:00:00',89.43,232,261,82.33,'Intermediate' from dual union all
select 19214,'019214','1988-047E','KDU part','KDU part',61296,'Reentered','1988-06-17 00:00:00','1988-06-15 00:00:00',89.49,241,258,82.33,'Intermediate' from dual union all
select 19215,'019215','1988-051A','Meteosat 3','Meteosat P2',61300,'In Earth orbit','','1988-07-16 00:00:00',1434.68,35722,35795,1.5,'Drift GEO' from dual union all
select 19216,'019216','1988-051B','AMSAT-OSCAR-13','AMSAT Phase III-C',61300,'Reentered','1996-12-06 00:00:00','1988-08-24 00:00:00',686.69,2496,36314,57.58,'GEO Transfer' from dual union all
select 19217,'019217','1988-051C','PAS 1','PAS 1',61300,'In Earth orbit','','1988-07-21 00:00:00',1436.11,35779,35795,.05,'Stationary' from dual union all
select 19218,'019218','1988-051D','Ariane H10 V22','Ariane H10 S/N T401',61300,'Reentered','1993-11-10 00:00:00','1988-07-15 00:00:00',631.04,186,35799,9.92,'GEO Transfer' from dual union all
select 19219,'019219','1988-051E','Spelda 10 S/N PF1','Spelda 10',61300,'In Earth orbit','','1988-07-18 00:00:00',636.8,234,36047,9.82,'GEO Transfer' from dual union all
select 1922,'001922','1965-082GG','deb Transtage 4','',11832,'Reentered','1979-03-05 00:00:00','1979-02-23 00:00:00',92.43,374,414,31.71,'Intermediate' from dual union all
select 19220,'019220','1988-051F','Cyclade','Meteosat adapter',61300,'Reentered','2001-11-22 00:00:00','1988-07-16 00:00:00',636.84,229,36054,9.88,'GEO Transfer' from dual union all
select 19221,'019221','1976-126CB','deb Kosmos-886','',39078,'Reentered','1990-01-03 00:00:00','1986-04-13 00:00:00',114.54,711,2158,65.86,'Intermediate' from dual union all
select 19222,'019222','1967-048D','deb NNS O-13 insulation','',15303,'Reentered','1993-01-09 00:00:00','1988-04-19 00:00:00',106.81,1056,1102,89.71,'Polar' from dual union all
select 19223,'019223','1988-052A','Nova 2','Nova 2',61301,'In Earth orbit','','1988-07-16 00:00:00',108.92,1155,1199,90.01,'Polar' from dual union all
select 19224,'019224','1988-052B','Star 20 E-40','Star 20 E-40',61301,'Reentered','1989-03-08 00:00:00','1988-07-16 00:00:00',97.08,310,928,90.13,'Polar' from dual union all
select 19225,'019225','1987-104C','Soyuz TM-4 BO','Bitovoy otsek',60194,'Reentered','1988-11-05 00:00:00','1988-06-19 00:00:00',91.46,344,349,51.62,'Intermediate' from dual union all
select 19226,'019226','1978-100V','deb S5M','',45295,'In Earth orbit','','1988-05-30 00:00:00',120.38,1666,1729,82.54,'Medium' from dual union all
select 19227,'019227','1978-100W','deb S5M','',45295,'In Earth orbit','','1988-05-30 00:00:00',120.2,1672,1708,82.3,'Medium' from dual union all
select 19228,'019228','1978-100X','deb S5M','',45295,'In Earth orbit','','1988-05-10 00:00:00',118.84,1561,1697,82.54,'Intermediate' from dual union all
select 19229,'019229','1978-100Y','deb S5M','',45295,'In Earth orbit','','1988-05-16 00:00:00',120.14,1680,1695,82.35,'Medium' from dual union all
select 1923,'001923','1965-082GH','deb Transtage 4','',11832,'Reentered','1968-06-15 00:00:00','1966-01-26 00:00:00',98.18,563,781,31.76,'Intermediate' from dual union all
select 19230,'019230','1978-100Z','deb S5M','',45295,'In Earth orbit','','1978-12-28 00:00:00',120.31,1682,1708,82.54,'Medium' from dual union all
select 19231,'019231','1978-100AA','deb S5M','',45295,'Reentered','2013-06-22 00:00:00','1988-05-16 00:00:00',118.49,1546,1681,82.56,'Intermediate' from dual union all
select 19232,'019232','1978-100AB','deb S5M','',45295,'In Earth orbit','','1988-05-15 00:00:00',123.4,1678,1987,82.34,'Medium' from dual union all
select 19233,'019233','1978-100AC','deb S5M','',45295,'In Earth orbit','','1988-05-17 00:00:00',119.78,1637,1705,82.57,'Intermediate' from dual union all
select 19234,'019234','1978-100AD','deb S5M','',45295,'In Earth orbit','','1988-05-30 00:00:00',122.32,1686,1883,82.43,'Medium' from dual union all
select 19235,'019235','1970-102G','deb Kosmos-381','[ex UNID129]',21579,'In Earth orbit','','1986-05-21 00:00:00',104.18,953,958,74.03,'Intermediate' from dual union all
select 19236,'019236','1981-065D','Iskra','RK-01',52588,'Reentered','1990-04-16 00:00:00','1988-05-30 00:00:00',97.34,618,645,97.74,'Sun-Synch' from dual union all
select 19237,'019237','1986-019UP','deb Ariane V16','[ex UNID 148]',59212,'Reentered','1988-11-26 00:00:00','1988-02-15 00:00:00',102.55,695,1064,97.86,'Sun-Synch' from dual union all
select 19238,'019238','1970-025PT','deb Agena D','',21502,'In Earth orbit','','1985-05-03 00:00:00',106.85,1044,1118,99.69,'Sun-Synch' from dual union all
select 19239,'019239','1988-044E','deb Blok-I','',61293,'Reentered','1988-07-13 00:00:00','1988-06-07 00:00:00',92.38,223,560,62.81,'Intermediate' from dual union all
select 1924,'001924','1965-082GJ','deb Transtage 4','',11832,'Reentered','1966-06-21 00:00:00','1966-01-06 00:00:00',96.54,450,736,32.71,'Intermediate' from dual union all
select 19240,'019240','1986-019UQ','deb Ariane V16','',59212,'Reentered','1988-11-24 00:00:00','1988-04-08 00:00:00',98.17,594,749,98.99,'Sun-Synch' from dual union all
select 19241,'019241','1985-058C','deb Kosmos-1666','',58281,'Reentered','2012-07-07 00:00:00','1988-05-31 00:00:00',97.65,627,666,82.53,'Intermediate' from dual union all
select 19242,'019242','1965-027AS','deb SNAPSHOT','',11777,'In Earth orbit','','1988-03-01 00:00:00',111.09,1252,1302,90.31,'Polar' from dual union all
select 19243,'019243','1982-055BJ','deb Kosmos-1375','',54364,'Reentered','1989-12-31 00:00:00','1984-12-18 00:00:00',103.26,537,1289,65.66,'Intermediate' from dual union all
select 19244,'019244','1976-126CA','deb Kosmos-886','',39078,'In Earth orbit','','1986-04-21 00:00:00',115.31,888,2051,66.05,'Intermediate' from dual union all
select 19245,'019245','1967-053K','deb Surcal','',15308,'In Earth orbit','','1988-05-24 00:00:00',103.27,907,919,69.98,'Intermediate' from dual union all
select 19246,'019246','1986-019UR','deb Ariane V16','',59212,'Reentered','1989-11-01 00:00:00','1988-04-11 00:00:00',101.01,788,825,98.5,'Sun-Synch' from dual union all
select 19247,'019247','1987-020BU','deb Kosmos-1823','',60110,'Reentered','1988-12-25 00:00:00','1988-03-18 00:00:00',119.21,1177,2114,73.32,'Intermediate' from dual union all
select 19248,'019248','1969-082LD','deb r/b (AGENA D)','',18349,'Reentered','1999-06-29 00:00:00','1987-12-07 00:00:00',100.32,695,853,69.81,'Intermediate' from dual union all
select 19249,'019249','1987-020BV','deb Kosmos-1823','',60110,'Reentered','1988-08-14 00:00:00','1987-12-22 00:00:00',114.57,1409,1462,73.53,'Intermediate' from dual union all
select 1925,'001925','1965-082GK','deb Transtage 4','',11832,'Reentered','1967-10-15 00:00:00','1967-10-08 00:00:00',91.47,338,356,31.97,'Intermediate' from dual union all
select 19250,'019250','1970-025PU','deb Agena D','',21502,'In Earth orbit','','1985-07-30 00:00:00',108.08,952,1324,99.45,'Sun-Synch' from dual union all
select 19251,'019251','1987-104D','Soyuz TM-4 part','Soyuz part',60194,'Reentered','1988-07-05 00:00:00','1988-06-18 00:00:00',91.38,340,345,51.62,'Intermediate' from dual union all
select 19252,'019252','1988-052C','Nova-2 despin weight','',61301,'Reentered','1988-06-18 00:00:00','1988-06-18 00:00:00',97.29,313,945,90.13,'Polar' from dual union all
select 19253,'019253','1987-104E','Soyuz TM-4 part','Soyuz part',60194,'Reentered','1988-06-19 00:00:00','1988-06-18 00:00:00',91.01,323,326,51.63,'Intermediate' from dual union all
select 19254,'019254','1987-104F','Soyuz TM-4 part','Soyuz part',60194,'Reentered','1988-06-27 00:00:00','1988-06-20 00:00:00',91.23,334,336,51.62,'Intermediate' from dual union all
select 19255,'019255','1987-104G','Soyuz TM-4 part','Soyuz part',60194,'Reentered','1988-07-09 00:00:00','1988-06-19 00:00:00',91.38,341,344,51.62,'Intermediate' from dual union all
select 19256,'019256','1988-053A','Kosmos-1954','Strela-2M',61302,'In Earth orbit','','1988-07-22 00:00:00',100.71,779,806,74.05,'Intermediate' from dual union all
select 19257,'019257','1988-053B','S3M','S3M',61302,'In Earth orbit','','1988-07-25 00:00:00',100.6,770,804,74.05,'Intermediate' from dual union all
select 19258,'019258','1988-054A','Kosmos-1955','Kobal''t',61303,'Landed','1988-08-20 00:00:00','1988-06-29 00:00:00',89.96,189,356,64.76,'Intermediate' from dual union all
select 19259,'019259','1988-054B','11S510','11S510',61303,'Reentered','1988-06-27 00:00:00','1988-06-27 00:00:00',88.03,146,208,64.77,'Intermediate' from dual union all
select 1926,'001926','1965-082GL','deb Transtage 4','',11832,'Reentered','1970-05-17 00:00:00','1966-01-26 00:00:00',98.12,576,762,32.12,'Intermediate' from dual union all
select 19260,'019260','1988-053C','deb Kosmos-1954','Strela-2M debris 1',61302,'In Earth orbit','','1988-07-25 00:00:00',101.01,799,813,74.07,'Intermediate' from dual union all
select 19261,'019261','1988-053D','deb Kosmos-1954','Strela-2M debris 2',61302,'In Earth orbit','','1988-07-22 00:00:00',101.03,796,818,74.06,'Intermediate' from dual union all
select 19262,'019262','1988-054C','Kosmos-1955 part','Kobal''t part',61303,'Reentered','1988-06-23 00:00:00','1988-06-22 00:00:00',89.27,167,310,64.76,'Intermediate' from dual union all
select 19263,'019263','1988-055A','Kosmos-1956','Oblik No. 367',61304,'Landed','1988-07-07 00:00:00','1988-06-26 00:00:00',91.53,332,368,82.37,'Intermediate' from dual union all
select 19264,'019264','1988-055B','11S510','11S510',61304,'Reentered','1988-06-25 00:00:00','1988-06-24 00:00:00',88.43,178,216,82.35,'Intermediate' from dual union all
select 19265,'019265','1988-049E','KDU','KDU',61298,'Reentered','1988-07-20 00:00:00','1988-06-27 00:00:00',89.49,225,274,69.98,'Intermediate' from dual union all
select 19266,'019266','1988-049F','Blenda','Sunshade',61298,'Reentered','1988-07-02 00:00:00','1988-06-26 00:00:00',89.04,216,239,69.99,'Intermediate' from dual union all
select 19267,'019267','1988-049G','KDU part','KDU part',61298,'Reentered','1988-06-26 00:00:00','1988-06-25 00:00:00',88.99,209,241,69.97,'Intermediate' from dual union all
select 19268,'019268','1988-049H','KDU part','KDU part',61298,'Reentered','1988-06-28 00:00:00','1988-06-26 00:00:00',88.86,196,241,69.98,'Intermediate' from dual;
insert into SATELLITE
select 19269,'019269','1988-049J','KDU part','KDU part',61298,'Reentered','1988-06-30 00:00:00','1988-06-27 00:00:00',90.91,238,401,69.97,'Intermediate' from dual union all
select 1927,'001927','1965-082GM','deb Transtage 4','',11832,'Reentered','1973-08-03 00:00:00','1966-01-31 00:00:00',99.89,723,784,32.09,'Intermediate' from dual union all
select 19270,'019270','1988-049K','KDU part','KDU part',61298,'Reentered','1988-06-27 00:00:00','1988-06-26 00:00:00',90.57,230,375,69.97,'Intermediate' from dual union all
select 19271,'019271','1986-017J','EO-3 EVA-2 debris','',59210,'Reentered','1988-08-10 00:00:00','1988-07-01 00:00:00',91.44,344,347,51.62,'Intermediate' from dual union all
select 19272,'019272','1986-017K','EO-3 EVA-2 debris','',59210,'Reentered','1988-08-25 00:00:00','1988-07-01 00:00:00',91.35,337,345,51.62,'Intermediate' from dual union all
select 19273,'019273','1988-037C','deb Kosmos-1942','',61286,'Reentered','1988-07-05 00:00:00','1988-07-05 00:00:00',88.38,136,253,67.13,'Intermediate' from dual union all
select 19274,'019274','1988-056A','Okean','Okean-O1 NKhM No. 5',61305,'In Earth orbit','','1988-09-01 00:00:00',97.72,632,668,82.52,'Intermediate' from dual union all
select 19275,'019275','1988-056B','S5M','S5M',61305,'In Earth orbit','','1988-08-06 00:00:00',97.71,633,666,82.52,'Intermediate' from dual union all
select 19276,'019276','1988-057A','Kosmos-1957','Resurs-F1 14F43 No. 29',61306,'Landed','1988-07-21 00:00:00','1988-07-09 00:00:00',89.83,259,274,82.59,'Intermediate' from dual union all
select 19277,'019277','1988-057B','11S510','11S510',61306,'Reentered','1988-07-09 00:00:00','1988-07-08 00:00:00',88.28,175,204,82.59,'Intermediate' from dual union all
select 19278,'019278','1988-055C','Blenda','Sunshade',61304,'Reentered','1988-11-08 00:00:00','1988-07-24 00:00:00',91.4,329,358,82.37,'Intermediate' from dual union all
select 19279,'019279','1988-055D','KDU','KDU',61304,'Reentered','1988-08-23 00:00:00','1988-07-24 00:00:00',90.96,315,329,82.37,'Intermediate' from dual union all
select 1928,'001928','1965-082GN','deb Transtage 4','',11832,'Reentered','1968-02-28 00:00:00','1968-02-07 00:00:00',92.51,376,420,32.49,'Intermediate' from dual union all
select 19280,'019280','1988-055E','KDU part','KDU part',61304,'Reentered','1988-08-31 00:00:00','1988-07-25 00:00:00',93.21,350,514,82.37,'Intermediate' from dual union all
select 19281,'019281','1988-058A','Fobos-1','1F No. 101',61307,'Deep Space','1988-07-08 00:00:00','',null,null,null,null,'' from dual union all
select 19282,'019282','1988-058B','Blok D-2 No. 2L','11S824F No. 2L',61307,'Lost','','1988-07-07 00:00:00',3267.72,2627,130504,50.8,'Very High' from dual union all
select 19283,'019283','1988-058C','SOZ','SOZ',61307,'Reentered','1988-07-08 00:00:00','1988-07-08 00:00:00',88.06,169,188,51.54,'Intermediate' from dual union all
select 19284,'019284','1988-055F','KDU part','KDU part',61304,'Reentered','1988-07-13 00:00:00','1988-07-09 00:00:00',91.1,314,342,82.31,'Intermediate' from dual union all
select 19285,'019285','1988-055G','KDU part','KDU part',61304,'Reentered','1988-08-05 00:00:00','1988-07-07 00:00:00',91.51,329,369,82.37,'Intermediate' from dual union all
select 19286,'019286','1988-055H','KDU part','KDU part',61304,'Reentered','1988-08-05 00:00:00','1988-07-07 00:00:00',93.89,356,574,82.36,'Intermediate' from dual union all
select 19287,'019287','1988-059A','Fobos-2','1F No. 102',61308,'Deep Space','1988-07-13 00:00:00','',null,null,null,null,'' from dual union all
select 19288,'019288','1988-059B','Blok D-2 No. 1L','11S824F No. 1L',61308,'Lost','','1988-07-12 00:00:00',3168.71,169,130000,50.8,'Very High' from dual union all
select 19289,'019289','1988-059C','SOZ','SOZ',61308,'Reentered','1988-07-13 00:00:00','1988-07-12 00:00:00',88.04,162,193,51.51,'Intermediate' from dual union all
select 1929,'001929','1965-082GP','deb Transtage 4','',11832,'Reentered','1982-04-11 00:00:00','1966-01-15 00:00:00',99.63,735,747,32.17,'Intermediate' from dual union all
select 19290,'019290','1970-025PV','deb Agena D','',21502,'In Earth orbit','','1985-05-07 00:00:00',106.69,1052,1095,99.54,'Sun-Synch' from dual union all
select 19291,'019291','1973-086HD','deb Delta 98','',30688,'In Earth orbit','','1985-05-03 00:00:00',115.75,1448,1531,101.67,'Sun-Synch' from dual union all
select 19292,'019292','1988-059D','SOZ','SOZ',61308,'Reentered','1988-07-13 00:00:00','1988-07-13 00:00:00',87.71,151,171,51.55,'Intermediate' from dual union all
select 19293,'019293','61 Omicron 289','deb Ablestar','',7273,'In Earth orbit','','1985-03-09 00:00:00',110.27,983,1495,66.99,'Intermediate' from dual union all
select 19294,'019294','1968-097EE','deb Kosmos-252','',16824,'In Earth orbit','','1987-06-06 00:00:00',114.01,760,2061,62.25,'Intermediate' from dual union all
select 19295,'019295','1978-100AE','deb S5M','',45295,'In Earth orbit','','1988-05-22 00:00:00',120.02,1276,2088,81.99,'Medium' from dual union all
select 19296,'019296','1983-044FE','deb Kosmos-1461','',55801,'Reentered','1989-05-15 00:00:00','1988-03-27 00:00:00',99.48,607,861,64.99,'Intermediate' from dual union all
select 19297,'019297','1976-103F','deb Kosmos-860 reactor','',39055,'Reentered','1998-04-02 00:00:00','1986-04-07 00:00:00',104.26,923,996,64.71,'Intermediate' from dual union all
select 19298,'019298','1988-052D','Nova-2 despin weight','',61301,'Reentered','1988-10-29 00:00:00','1988-07-16 00:00:00',96.31,307,857,90.06,'Polar' from dual union all
select 19299,'019299','1963-014EL','deb West Ford','RCS 0.21',9245,'In Earth orbit','','1987-10-25 00:00:00',166.03,2804,4453,87.13,'Medium' from dual union all
select 193,'000193','61 A Delta 2','Midas Fairing','Midas Fairing',7251,'Reentered','1961-12-05 00:00:00','1961-11-16 00:00:00',107.2,105,2090,94.63,'Polar' from dual union all
select 1930,'001930','1965-082GQ','deb Transtage 4','',11832,'Reentered','1966-09-12 00:00:00','1966-07-19 00:00:00',94.96,447,587,31.98,'Intermediate' from dual union all
select 19300,'019300','61 Omicron 290','deb Ablestar','',7273,'Reentered','2002-03-13 00:00:00','1985-05-05 00:00:00',102.04,814,897,66.91,'Intermediate' from dual union all
select 19301,'019301','1982-055BK','deb Kosmos-1375','',54364,'Reentered','2002-02-27 00:00:00','1988-05-30 00:00:00',104.11,902,1004,65.84,'Intermediate' from dual union all
select 19302,'019302','61 Omicron 291','deb Ablestar','',7273,'Reentered','1993-08-27 00:00:00','1985-02-14 00:00:00',103.91,889,998,66.87,'Intermediate' from dual union all
select 19303,'019303','1983-044FF','deb Kosmos-1461','',55801,'Reentered','1989-12-04 00:00:00','1988-02-28 00:00:00',99.16,571,866,65.02,'Intermediate' from dual union all
select 19304,'019304','61 Omicron 292','deb Ablestar','',7273,'Reentered','2000-02-13 00:00:00','1985-04-19 00:00:00',101.26,765,871,67.08,'Intermediate' from dual union all
select 19305,'019305','1983-044FG','deb Kosmos-1461','',55801,'Reentered','1988-11-18 00:00:00','1988-06-16 00:00:00',97.5,543,736,65.02,'Intermediate' from dual union all
select 19306,'019306','1976-077FJ','deb Delta 126','',39029,'In Earth orbit','','1985-07-27 00:00:00',118.56,1500,1733,102.2,'Sun-Synch' from dual union all
select 19307,'019307','1986-019US','deb Ariane V16','',59212,'Reentered','1990-03-09 00:00:00','1987-11-29 00:00:00',103.28,793,1035,98.05,'Sun-Synch' from dual union all
select 19308,'019308','1983-044FH','deb Kosmos-1461','',55801,'Reentered','1989-05-27 00:00:00','1987-11-21 00:00:00',98.84,558,849,65.04,'Intermediate' from dual union all
select 19309,'019309','1968-097ES','deb Kosmos-252','',16824,'In Earth orbit','','1984-12-19 00:00:00',111.83,543,2079,62.37,'Intermediate' from dual union all
select 1931,'001931','1965-072E','deb DAPP','',11822,'In Earth orbit','','1966-01-09 00:00:00',103.31,655,1176,98.7,'Sun-Synch' from dual union all
select 19310,'019310','1983-044FJ','deb Kosmos-1461','',55801,'Reentered','1989-11-30 00:00:00','1988-04-18 00:00:00',98.74,553,844,65.05,'Intermediate' from dual union all
select 19311,'019311','1975-004HP','deb Delta 107','',36451,'In Earth orbit','','1986-03-31 00:00:00',102.43,733,1015,97.67,'Sun-Synch' from dual union all
select 19312,'019312','1986-019UT','deb Ariane V16','',59212,'Reentered','1989-06-23 00:00:00','1988-06-07 00:00:00',100.69,740,843,98.5,'Sun-Synch' from dual union all
select 19313,'019313','1986-019UU','deb Ariane V16','',59212,'Reentered','1989-02-02 00:00:00','1987-08-10 00:00:00',100.76,753,837,98.75,'Sun-Synch' from dual union all
select 19314,'019314','1977-012L','deb Tansei-3','',42140,'In Earth orbit','','1984-12-20 00:00:00',133.67,643,3920,65.37,'Medium' from dual union all
select 19315,'019315','1986-019UV','deb Ariane V16','',59212,'Reentered','1989-08-06 00:00:00','1987-11-02 00:00:00',101.68,794,883,98.79,'Sun-Synch' from dual union all
select 19316,'019316','1976-077FK','deb Delta 126','',39029,'In Earth orbit','','1985-05-03 00:00:00',119.73,1498,1840,102.18,'Sun-Synch' from dual union all
select 19317,'019317','1968-091DH','deb Kosmos-249','',16818,'In Earth orbit','','1988-03-27 00:00:00',109.42,527,1873,62.84,'Intermediate' from dual union all
select 19318,'019318','1971-015DR','deb Kosmos-397','',24833,'Reentered','2002-08-31 00:00:00','1987-09-26 00:00:00',108.5,468,1847,65.83,'Intermediate' from dual union all
select 19319,'019319','1978-026HL','deb Delta 139','',45221,'Reentered','2002-03-16 00:00:00','1985-08-17 00:00:00',101.61,734,936,98.61,'Sun-Synch' from dual union all
select 1932,'001932','1965-072F','deb DAPP','',11822,'Reentered','2009-08-06 00:00:00','1966-02-20 00:00:00',100.68,645,937,98.63,'Sun-Synch' from dual union all
select 19320,'019320','1988-060A','Kosmos-1958','Vektor',61309,'Reentered','1989-03-21 00:00:00','1988-08-16 00:00:00',92.32,366,411,65.84,'Intermediate' from dual union all
select 19321,'019321','1988-060B','S3M','S3M',61309,'Reentered','1989-01-07 00:00:00','1988-08-18 00:00:00',92.2,355,409,65.84,'Intermediate' from dual union all
select 19322,'019322','1988-061A','Progress-37','Progress 7K-TG No. 145',61310,'Deorbited','1988-08-12 00:00:00','1988-07-25 00:00:00',91.42,342,347,51.62,'Intermediate' from dual union all
select 19323,'019323','1988-061B','11S510','11S510',61310,'Reentered','1988-07-21 00:00:00','1988-07-21 00:00:00',88.09,165,195,51.61,'Intermediate' from dual union all
select 19324,'019324','1988-062A','Kosmos-1959','Parus',61311,'In Earth orbit','','1988-08-19 00:00:00',104.72,957,1006,82.95,'Intermediate' from dual union all
select 19325,'019325','1988-062B','S3M','S3M',61311,'In Earth orbit','','1988-08-19 00:00:00',104.62,957,997,82.95,'Intermediate' from dual union all
select 19326,'019326','1988-057C','Blenda','Sunshade',61306,'Reentered','1988-07-30 00:00:00','1988-07-21 00:00:00',91.82,260,468,82.58,'Intermediate' from dual union all
select 19327,'019327','1988-057D','KDU part','KDU part',61306,'Reentered','1988-07-27 00:00:00','1988-07-22 00:00:00',89.48,234,264,82.59,'Intermediate' from dual union all
select 19328,'019328','1988-057E','KDU part','KDU part',61306,'Reentered','1988-07-24 00:00:00','1988-07-21 00:00:00',89.59,246,263,82.59,'Intermediate' from dual union all
select 19329,'019329','1988-057F','KDU part','KDU part',61306,'Reentered','1988-07-22 00:00:00','1988-07-21 00:00:00',88.45,126,270,82.49,'Intermediate' from dual union all
select 1933,'001933','1965-112N','deb Blok Kosmos','',11862,'Reentered','1967-07-05 00:00:00','1966-01-31 00:00:00',95.98,515,616,56.01,'Intermediate' from dual union all
select 19330,'019330','1988-063A','Insat 1C','Insat 1C',61312,'In Earth orbit','','1988-08-24 00:00:00',1436.11,35765,35809,.15,'Stationary' from dual union all
select 19331,'019331','1988-063B','Eutelsat I F-5','ECS 5',61312,'In Earth orbit','','1988-08-25 00:00:00',1436.04,35773,35798,.05,'Stationary' from dual union all
select 19332,'019332','1988-063C','Ariane H10 V24','Ariane H10 S/N T23',61312,'Reentered','2003-04-30 00:00:00','1988-08-20 00:00:00',637.97,300,36041,7.36,'GEO Transfer' from dual union all
select 19333,'019333','1988-057G','KDU','KDU',61306,'Reentered','1988-08-20 00:00:00','1988-07-22 00:00:00',89.74,253,271,82.59,'Intermediate' from dual union all
select 19334,'019334','1988-057H','KDU part','KDU part',61306,'Reentered','1988-07-23 00:00:00','1988-07-22 00:00:00',91.56,249,454,82.57,'Intermediate' from dual union all
select 19335,'019335','1988-063D','Sylda 4400 No. 9','Sylda 4400',61312,'Reentered','1989-07-12 00:00:00','1988-08-29 00:00:00',603.84,229,34345,7.14,'GEO Transfer' from dual union all
select 19336,'019336','1988-064A','Meteor-3','Meteor-3 No. 3',61313,'In Earth orbit','','1988-08-26 00:00:00',109.35,1185,1208,82.55,'Intermediate' from dual union all
select 19337,'019337','1988-064B','S5M','S5M',61313,'In Earth orbit','','1988-08-29 00:00:00',109.33,1185,1206,82.55,'Intermediate' from dual union all
select 19338,'019338','1988-065A','Kosmos-1960','Romb',61314,'Reentered','1990-04-09 00:00:00','1988-09-01 00:00:00',94.43,468,514,65.84,'Intermediate' from dual union all
select 19339,'019339','1988-065B','S3M','S3M',61314,'Reentered','1990-01-08 00:00:00','1988-08-28 00:00:00',94.33,461,511,65.84,'Intermediate' from dual union all
select 1934,'001934','1965-101C','FR-1 despin weight','',11851,'Reentered','1981-12-10 00:00:00','1966-05-01 00:00:00',99.92,731,779,76.49,'Intermediate' from dual union all
select 19340,'019340','1988-065C','Dispenser cover','Romb dispenser cover',61314,'Reentered','1988-10-19 00:00:00','1988-08-27 00:00:00',93.86,431,496,65.85,'Intermediate' from dual union all
select 19341,'019341','1988-065D','Dispenser cover','Romb dispenser cover',61314,'Reentered','1988-10-19 00:00:00','1988-08-29 00:00:00',93.83,429,495,65.8,'Intermediate' from dual union all
select 19342,'019342','1988-065E','Dispenser cover','Romb dispenser cover',61314,'Reentered','1988-10-18 00:00:00','1988-08-27 00:00:00',93.8,425,496,65.82,'Intermediate' from dual union all
select 19343,'019343','1988-065F','Dispenser cover','Romb dispenser cover',61314,'Reentered','1988-10-18 00:00:00','1988-08-27 00:00:00',93.8,425,496,65.87,'Intermediate' from dual union all
select 19344,'019344','1988-066A','Kosmos-1961','Geizer No. 16L',61315,'In Earth orbit','','1988-08-31 00:00:00',1436.04,35775,35796,1.45,'Stationary' from dual union all
select 19345,'019345','1988-066B','8S812','8S812',61315,'Reentered','1988-08-04 00:00:00','1988-08-03 00:00:00',87.96,171,175,51.62,'Intermediate' from dual union all
select 19346,'019346','1988-066C','Perekhodnik','Sredniy perekhodnik',61315,'Reentered','1988-08-02 00:00:00','1988-08-02 00:00:00',87.93,168,176,51.6,'Intermediate' from dual union all
select 19347,'019347','1988-066D','Blok DM2','11S861 No. 33L',61315,'In Earth orbit','','1988-08-31 00:00:00',1459.57,36111,36379,1.43,'Drift GEO' from dual union all
select 19348,'019348','1988-066E','SOZ','SOZ',61315,'Reentered','2003-10-16 00:00:00','1988-09-05 00:00:00',644.43,338,36334,47.31,'GEO Transfer' from dual union all
select 19349,'019349','1988-066F','SOZ','SOZ',61315,'Reentered','1989-06-02 00:00:00','1988-09-03 00:00:00',643.18,284,36324,47.16,'GEO Transfer' from dual union all
select 1935,'001935','1965-101D','FR-1 despin weight','',11851,'Reentered','1980-11-10 00:00:00','1966-01-10 00:00:00',99.53,693,780,75.24,'Intermediate' from dual union all
select 19350,'019350','1979-017LZ','deb P78-1','',47501,'Reentered','1989-04-07 00:00:00','1988-08-03 00:00:00',94.41,471,509,97.79,'Sun-Synch' from dual union all
select 19351,'019351','1970-025PW','deb Agena D','',21502,'In Earth orbit','','1993-08-02 00:00:00',105.6,1000,1045,100.08,'Sun-Synch' from dual union all
select 19352,'019352','1978-100AF','deb S5M','',45295,'Reentered','1992-03-29 00:00:00','1991-11-04 00:00:00',107.59,925,1306,82.09,'Intermediate' from dual union all
select 19353,'019353','1970-025PX','deb Agena D','',21502,'Reentered','1998-03-06 00:00:00','1993-08-25 00:00:00',99.88,718,788,100.55,'Sun-Synch' from dual union all
select 19354,'019354','1981-053MG','deb Kosmos-1275','',52576,'Reentered','2002-06-14 00:00:00','1993-08-07 00:00:00',101.57,806,860,83.3,'Intermediate' from dual union all
select 19355,'019355','1965-027AT','deb SNAPSHOT','',11777,'In Earth orbit','','1993-08-04 00:00:00',111.34,1268,1309,90.28,'Polar' from dual union all
select 19356,'019356','1986-019UW','deb Ariane V16','',59212,'Reentered','1989-12-03 00:00:00','1988-08-03 00:00:00',102.63,780,987,98.33,'Sun-Synch' from dual union all
select 19357,'019357','1986-019UX','deb Ariane V16','',59212,'Reentered','1990-04-22 00:00:00','1988-08-02 00:00:00',103.37,789,1047,98.03,'Sun-Synch' from dual union all
select 19358,'019358','1969-082LE','deb Agena D','',18349,'Reentered','2002-04-06 00:00:00','1993-08-06 00:00:00',101.07,746,873,70.48,'Intermediate' from dual;
insert into SATELLITE
select 19359,'019359','1978-026HM','deb Delta 139','',45221,'In Earth orbit','','2011-06-06 00:00:00',100.4,760,794,98.3,'Sun-Synch' from dual union all
select 1936,'001936','1965-112P','deb Blok Kosmos','',11862,'Reentered','1969-04-07 00:00:00','1966-02-07 00:00:00',96.88,585,634,56.03,'Intermediate' from dual union all
select 19360,'019360','1973-086HE','deb Delta 98','',30688,'In Earth orbit','','1993-08-07 00:00:00',115.8,1448,1535,101.89,'Sun-Synch' from dual union all
select 19361,'019361','1983-044FK','deb Kosmos-1461','',55801,'Reentered','1989-04-07 00:00:00','1988-08-04 00:00:00',99.03,599,825,64.99,'Intermediate' from dual union all
select 19362,'019362','1968-091DJ','deb Kosmos-249','',16818,'In Earth orbit','','1993-07-02 00:00:00',107.68,557,1681,62.8,'Intermediate' from dual union all
select 19363,'019363','1970-025PY','deb Agena D','',21502,'In Earth orbit','','1993-08-08 00:00:00',106.15,1044,1052,100.05,'Sun-Synch' from dual union all
select 19364,'019364','1978-100AG','deb S5M','',45295,'Reentered','1992-06-26 00:00:00','1992-06-25 00:00:00',92.34,346,433,82.24,'Intermediate' from dual union all
select 19365,'019365','1975-004HQ','deb Delta 107','',36451,'In Earth orbit','','1993-08-08 00:00:00',101.89,722,974,97.67,'Sun-Synch' from dual union all
select 19366,'019366','1966-005L','NNS O-7 insulation','RCS 1.0',13412,'Reentered','1990-04-24 00:00:00','1988-04-02 00:00:00',105.1,851,1148,89.8,'Polar' from dual union all
select 19367,'019367','1971-015DS','Unknown deb                       ','',24833,'In Earth orbit','','1993-08-06 00:00:00',104.6,843,1109,65.71,'Intermediate' from dual union all
select 19368,'019368','1988-067A','FSW-1 RV','FSW-1 RV',61316,'Landed','1988-08-13 00:00:00','1988-08-08 00:00:00',89.57,204,303,63.02,'Intermediate' from dual union all
select 19369,'019369','1988-067B','CZ-2C Stage 2','CZ-2C Stage 2',61316,'Reentered','1988-08-19 00:00:00','1988-08-08 00:00:00',89.42,202,290,63.02,'Intermediate' from dual union all
select 1937,'001937','1965-112Q','deb Blok Kosmos','',11862,'Reentered','2000-10-28 00:00:00','1966-01-30 00:00:00',96.93,578,646,55.93,'Intermediate' from dual union all
select 19370,'019370','1988-067C','CZ-2C sep motor cover','',61316,'Reentered','1988-08-07 00:00:00','1988-08-07 00:00:00',88.09,168,192,62.99,'Intermediate' from dual union all
select 19371,'019371','1988-067D','CZ-2C sep motor cover','',61316,'Reentered','1988-08-07 00:00:00','1988-08-06 00:00:00',89.01,189,263,63.02,'Intermediate' from dual union all
select 19372,'019372','1988-068A','Kosmos-1962','Oblik',61317,'Landed','1988-08-22 00:00:00','1988-08-10 00:00:00',89.65,230,285,70.01,'Intermediate' from dual union all
select 19373,'019373','1988-068B','11S510','11S510',61317,'Reentered','1988-08-13 00:00:00','1988-08-10 00:00:00',88.96,197,250,70.02,'Intermediate' from dual union all
select 19374,'019374','1988-068C','Sensor cover','Sensor cover',61317,'Reentered','1988-08-09 00:00:00','1988-08-08 00:00:00',88.66,192,225,70.03,'Intermediate' from dual union all
select 19375,'019375','1986-067Y','Kosmos-1776 SS 17','ESO',59260,'Reentered','1988-10-18 00:00:00','1988-08-09 00:00:00',94.47,463,523,74.04,'Intermediate' from dual union all
select 19376,'019376','1986-067Z','Kosmos-1776 SS 18','ESO',59260,'Reentered','1988-10-01 00:00:00','1988-08-09 00:00:00',93.95,461,475,74.01,'Intermediate' from dual union all
select 19377,'019377','1988-069A','Molniya-1','Molniya-1 No. 66',61318,'Reentered','1999-12-16 00:00:00','1988-09-12 00:00:00',717.76,571,39783,62.92,'Molniya' from dual union all
select 19378,'019378','1988-069B','11S510','11S510',61318,'Reentered','1988-09-20 00:00:00','1988-08-19 00:00:00',92.2,216,549,62.84,'Intermediate' from dual union all
select 19379,'019379','1988-069C','BOZ','BOZ',61318,'Reentered','1988-09-21 00:00:00','1988-08-19 00:00:00',92.3,210,564,62.84,'Intermediate' from dual union all
select 1938,'001938','1965-096C','deb Asterix','',11846,'Reentered','1967-03-25 00:00:00','1965-12-30 00:00:00',106.21,502,1600,34.25,'Intermediate' from dual union all
select 19380,'019380','1988-069D','Blok-ML','Blok-ML No. 179',61318,'Reentered','1998-05-10 00:00:00','1988-09-11 00:00:00',735.84,487,40755,62.83,'Molniya' from dual union all
select 19381,'019381','1988-067E','Fanhui Shi Weixing (11)','Jianbing-1A No. 2',61316,'Reentered','1988-08-27 00:00:00','1988-08-13 00:00:00',89.43,202,290,63,'Intermediate' from dual union all
select 19382,'019382','1988-067F','FSW-1 RV part','',61316,'Reentered','1988-08-17 00:00:00','1988-08-13 00:00:00',89.11,194,267,63.02,'Intermediate' from dual union all
select 19383,'019383','1988-067G','FSW-1 RV part','',61316,'Reentered','1988-08-19 00:00:00','1988-08-13 00:00:00',89.61,205,306,63.01,'Intermediate' from dual union all
select 19384,'019384','1988-070A','Kosmos-1963','Kobal''t',61319,'Landed','1988-10-02 00:00:00','1988-08-23 00:00:00',89.34,167,317,64.77,'Intermediate' from dual union all
select 19385,'019385','1988-070B','11S510','11S510',61319,'Reentered','1988-08-21 00:00:00','1988-08-18 00:00:00',89.41,170,320,64.78,'Intermediate' from dual union all
select 19386,'019386','1986-017L','Progress part','',59210,'Reentered','1988-11-06 00:00:00','1988-08-17 00:00:00',91.69,347,368,51.62,'Intermediate' from dual union all
select 19387,'019387','1986-017M','Progress part','',59210,'Reentered','1988-09-24 00:00:00','1988-08-17 00:00:00',91.57,340,364,51.63,'Intermediate' from dual union all
select 19388,'019388','1988-070C','Kosmos-1963 part','Kobal''t part',61319,'Reentered','1988-08-17 00:00:00','1988-08-17 00:00:00',88.88,160,279,64.76,'Intermediate' from dual union all
select 19389,'019389','1988-052E','deb Nova-2','',61301,'Reentered','1988-09-15 00:00:00','1988-07-18 00:00:00',95.82,297,820,90.17,'Polar' from dual union all
select 1939,'001939','1966-002A','OPS 7253','GAMBIT SV 974',13409,'Reentered','1966-01-25 00:00:00','1966-01-23 00:00:00',88.53,144,260,93.89,'Polar' from dual union all
select 19390,'019390','1987-020BW','deb Kosmos-1823','',60110,'Reentered','2000-07-22 00:00:00','1993-08-08 00:00:00',111.58,1230,1368,73.34,'Intermediate' from dual union all
select 19391,'019391','1970-025PZ','deb Agena D','',21502,'In Earth orbit','','1993-08-06 00:00:00',104.79,852,1118,98.79,'Sun-Synch' from dual union all
select 19392,'019392','1987-020BX','deb Kosmos-1823','',60110,'Reentered','1990-05-20 00:00:00','1990-05-14 00:00:00',96.94,598,627,74,'Intermediate' from dual union all
select 19393,'019393','1977-121BT','deb Kosmos-970','',42249,'Reentered','1991-09-15 00:00:00','1991-09-13 00:00:00',90.57,272,333,65.62,'Intermediate' from dual union all
select 19394,'019394','1987-020BY','deb Kosmos-1823','',60110,'Reentered','1990-02-23 00:00:00','1988-08-16 00:00:00',116.52,1371,1678,73.49,'Intermediate' from dual union all
select 19395,'019395','1986-019UY','deb Ariane V16','',59212,'Reentered','1989-06-07 00:00:00','1988-08-18 00:00:00',100.75,775,814,98.62,'Sun-Synch' from dual union all
select 19396,'019396','1963-054E','deb Tiros 8','',9285,'In Earth orbit','','1988-06-29 00:00:00',98.75,667,730,58.48,'Intermediate' from dual union all
select 19397,'019397','1988-071A','Gorizont','Gorizont No. 28L',61320,'In Earth orbit','','1988-09-17 00:00:00',1435.96,35748,35820,1.39,'Stationary' from dual union all
select 19398,'019398','1988-071B','8S812','8S812',61320,'Reentered','1988-08-21 00:00:00','1988-08-20 00:00:00',88.02,173,180,51.63,'Intermediate' from dual union all
select 19399,'019399','1988-071C','Perekhodnik','Sredniy perekhodnik',61320,'Reentered','1988-08-19 00:00:00','1988-08-19 00:00:00',87.59,152,159,51.6,'Intermediate' from dual union all
select 194,'000194','61 A Delta 3','Westford','Westford',7251,'In Earth orbit','','1961-12-14 00:00:00',165.54,3503,3715,95.9,'Medium' from dual union all
select 1940,'001940','1966-002B','Agena D 4825','Agena D',13409,'Reentered','1966-01-23 00:00:00','1966-01-21 00:00:00',88.45,151,244,93.89,'Polar' from dual union all
select 19400,'019400','1988-071D','Blok DM2 No. 15L','11S861 No. 15L',61320,'In Earth orbit','','1988-10-02 00:00:00',1432.26,35631,35792,1.37,'Drift GEO' from dual union all
select 19401,'019401','1988-071E','SOZ','SOZ',61320,'Reentered','1989-06-03 00:00:00','1988-09-19 00:00:00',632.79,272,35803,47.31,'GEO Transfer' from dual union all
select 19402,'019402','1988-071F','SOZ','SOZ',61320,'Reentered','1994-09-04 00:00:00','1988-09-24 00:00:00',634.13,327,35817,47.34,'GEO Transfer' from dual union all
select 19403,'019403','1988-068D','Blenda','Sunshade',61317,'Reentered','1988-08-24 00:00:00','1988-08-22 00:00:00',89.46,223,273,70,'Intermediate' from dual union all
select 19404,'019404','1988-068E','KDU part','KDU part',61317,'Reentered','1988-08-26 00:00:00','1988-08-22 00:00:00',89.33,224,259,70.01,'Intermediate' from dual union all
select 19405,'019405','1988-068F','KDU','KDU',61317,'Reentered','1988-09-12 00:00:00','1988-08-22 00:00:00',89.52,226,276,70.01,'Intermediate' from dual union all
select 19406,'019406','1988-068G','KDU part','KDU part',61317,'Reentered','1988-08-27 00:00:00','1988-08-22 00:00:00',91.41,244,444,70.05,'Intermediate' from dual union all
select 19407,'019407','1988-068H','KDU part','KDU part',61317,'Reentered','1988-08-23 00:00:00','1988-08-22 00:00:00',91.45,235,457,70.03,'Intermediate' from dual union all
select 19408,'019408','1986-017N','Garbage bag','',59210,'Reentered','1988-10-16 00:00:00','1988-08-22 00:00:00',91.69,342,373,51.62,'Intermediate' from dual union all
select 19409,'019409','1986-017P','Garbage bag','',59210,'Reentered','1988-10-31 00:00:00','1988-08-22 00:00:00',91.64,341,369,51.62,'Intermediate' from dual union all
select 1941,'001941','1965-108D','LES 3','LES 3',11858,'Reentered','1968-04-06 00:00:00','1966-02-16 00:00:00',573.91,190,32807,26.7,'Highly Ellip' from dual union all
select 19410,'019410','1986-067AA','Kosmos-1776 SS 19','ESO',59260,'Reentered','1988-10-20 00:00:00','1988-08-22 00:00:00',94.37,464,513,74.02,'Intermediate' from dual union all
select 19411,'019411','1986-067AB','Kosmos-1776 SS 20','ESO',59260,'Reentered','1988-10-10 00:00:00','1988-08-22 00:00:00',94.03,459,485,74.03,'Intermediate' from dual union all
select 19412,'019412','1988-072A','Kosmos-1964','Oblik',61321,'Landed','1988-09-07 00:00:00','1988-08-30 00:00:00',89.6,228,281,70,'Intermediate' from dual union all
select 19413,'019413','1988-072B','11S510','11S510',61321,'Reentered','1988-08-28 00:00:00','1988-08-25 00:00:00',88.86,195,241,70.01,'Intermediate' from dual union all
select 19414,'019414','1988-073A','Kosmos-1965','Resurs-F2 17F42 No. 2',61322,'Landed','1988-09-22 00:00:00','1988-08-30 00:00:00',89.79,255,274,82.32,'Intermediate' from dual union all
select 19415,'019415','1988-073B','11S510','11S510',61322,'Reentered','1988-08-25 00:00:00','1988-08-24 00:00:00',88.39,176,214,82.32,'Intermediate' from dual union all
select 19416,'019416','1988-073C','Sensor cover','Sensor cover',61322,'Reentered','1988-08-23 00:00:00','1988-08-23 00:00:00',88.3,173,208,82.34,'Intermediate' from dual union all
select 19417,'019417','1988-072C','Sensor cover','Sensor cover',61321,'Reentered','1988-08-24 00:00:00','1988-08-23 00:00:00',89.36,204,282,70.03,'Intermediate' from dual union all
select 19418,'019418','1988-072D','Sensor cover','Sensor cover',61321,'Reentered','1988-08-24 00:00:00','1988-08-23 00:00:00',88.84,202,232,70.02,'Intermediate' from dual union all
select 19419,'019419','1988-074A','NNS O-25','NNS O-25',61323,'In Earth orbit','','1988-10-10 00:00:00',107.43,1034,1182,89.96,'Polar' from dual union all
select 1942,'001942','61 Omicron 209','deb r/b Ablestar','',7273,'In Earth orbit','','1992-01-28 00:00:00',110.38,965,1524,67.25,'Intermediate' from dual union all
select 19420,'019420','1988-074B','NNS O-31','NNS O-31',61323,'In Earth orbit','','1988-10-07 00:00:00',107.41,1034,1179,89.96,'Polar' from dual union all
select 19421,'019421','1988-074C','Star 20 E-42','Star 20 E-42',61323,'In Earth orbit','','1988-10-31 00:00:00',107.44,1036,1180,89.96,'Polar' from dual union all
select 19422,'019422','1970-025QA','deb Agena D','',21502,'In Earth orbit','','1993-08-07 00:00:00',106.01,1033,1050,100.2,'Sun-Synch' from dual union all
select 19423,'019423','1970-025QB','deb Agena D','',21502,'In Earth orbit','','1993-08-06 00:00:00',103.19,839,979,99.83,'Sun-Synch' from dual union all
select 19424,'019424','1988-069E','deb Blok-I','',61318,'Reentered','1988-09-12 00:00:00','1988-08-23 00:00:00',92.01,221,526,62.83,'Intermediate' from dual union all
select 19425,'019425','1976-126CC','Unknown deb                       ','',39078,'In Earth orbit','','1993-08-06 00:00:00',113.89,1131,1679,66.23,'Intermediate' from dual union all
select 19426,'019426','1983-044FL','deb Kosmos-1461','',55801,'Reentered','1989-03-05 00:00:00','1988-08-24 00:00:00',96.8,511,699,64.97,'Intermediate' from dual union all
select 19427,'019427','1983-044FM','deb Kosmos-1461','',55801,'Reentered','1989-03-03 00:00:00','1988-08-24 00:00:00',97.43,544,727,65.01,'Intermediate' from dual union all
select 19428,'019428','61 Omicron 293','deb Ablestar','',7273,'Reentered','1990-03-08 00:00:00','1988-05-14 00:00:00',98.88,621,790,66.67,'Intermediate' from dual union all
select 19429,'019429','1977-065FY','deb Delta 132','',42193,'In Earth orbit','','1993-08-05 00:00:00',106.5,506,1623,29.11,'Intermediate' from dual union all
select 1943,'001943','61 Omicron 210','deb r/b Ablestar','',7273,'In Earth orbit','','1992-01-31 00:00:00',102.35,816,924,66.85,'Intermediate' from dual union all
select 19430,'019430','61 Omicron 294','deb Ablestar','',7273,'Reentered','1990-05-19 00:00:00','1986-04-23 00:00:00',100.35,633,918,66.61,'Intermediate' from dual union all
select 19431,'019431','1975-004HR','deb Delta 107','',36451,'Reentered','1990-12-14 00:00:00','1988-08-22 00:00:00',98.3,649,706,97.89,'Sun-Synch' from dual union all
select 19432,'019432','1986-074C','deb Kosmos-1782','[UNID130] RCS 0.3',59267,'Reentered','1989-03-13 00:00:00','1987-10-31 00:00:00',98.18,594,750,82.74,'Intermediate' from dual union all
select 19433,'019433','1970-089DE','deb Kosmos-374','',21566,'Reentered','1989-09-25 00:00:00','1988-08-23 00:00:00',107.9,530,1730,62.75,'Intermediate' from dual union all
select 19434,'019434','1988-044F','deb Blok-I','',61293,'Reentered','1988-09-23 00:00:00','1988-07-23 00:00:00',91.96,236,506,62.81,'Intermediate' from dual union all
select 19435,'019435','1986-019UZ','deb Ariane V16','',59212,'Reentered','2000-04-18 00:00:00','1993-08-08 00:00:00',98.39,563,801,98.34,'Sun-Synch' from dual union all
select 19436,'019436','62 A Psi 5','deb Tiros VI','',8152,'Reentered','1992-05-26 00:00:00','1988-07-31 00:00:00',96.66,598,600,58.3,'Intermediate' from dual union all
select 19437,'019437','1987-020BZ','deb Kosmos-1823','',60110,'Reentered','1992-04-18 00:00:00','1992-04-07 00:00:00',99.19,645,794,72.51,'Intermediate' from dual union all
select 19438,'019438','1988-069F','deb Blok-I','',61318,'Reentered','1988-09-15 00:00:00','1988-08-21 00:00:00',92.29,222,552,62.83,'Intermediate' from dual union all
select 19439,'019439','1976-067BS','deb Kosmos-839','',39019,'In Earth orbit','','1989-09-17 00:00:00',116.75,987,2083,65.88,'Intermediate' from dual union all
select 1944,'001944','1965-098E','DME-A adapter petal','Adapter Petal',11848,'Reentered','2001-11-07 00:00:00','1966-01-27 00:00:00',121.31,500,2979,79.83,'Medium' from dual union all
select 19440,'019440','1987-020CA','deb Kosmos-1823','',60110,'In Earth orbit','','1988-05-24 00:00:00',112.96,1235,1490,73.76,'Intermediate' from dual union all
select 19441,'019441','1984-012M','deb NOSS','',57066,'Reentered','1988-08-25 00:00:00','1988-09-01 00:00:00',107.15,830,1359,63.44,'Intermediate' from dual union all
select 19442,'019442','1986-017Q','Garbage bag','',59210,'Reentered','1988-10-05 00:00:00','1988-08-27 00:00:00',91.68,342,372,51.62,'Intermediate' from dual union all
select 19443,'019443','1988-075A','Soyuz TM-6','Soyuz 7K-STM No. 56',61324,'Landed','1988-12-21 00:00:00','1988-09-28 00:00:00',91.44,332,359,51.61,'Intermediate' from dual union all
select 19444,'019444','1988-075B','11S510','11S510',61324,'Reentered','1988-08-31 00:00:00','1988-08-30 00:00:00',88.12,176,187,51.63,'Intermediate' from dual union all
select 19445,'019445','1988-076A','Kosmos-1966','Oko',61325,'Reentered','2005-11-10 00:00:00','1988-09-29 00:00:00',717.84,641,39717,62.95,'Molniya' from dual union all
select 19446,'019446','1988-076B','11S510','11S510',61325,'Reentered','1988-09-30 00:00:00','1988-09-06 00:00:00',91.82,209,519,62.85,'Intermediate' from dual union all
select 19447,'019447','1988-076C','BOZ','BOZ',61325,'Reentered','1988-09-16 00:00:00','1988-09-06 00:00:00',91.48,175,520,62.86,'Intermediate' from dual union all
select 19448,'019448','1988-076D','Blok-2BL','Blok-2BL No. 76',61325,'Reentered','2017-02-07 00:00:00','1988-09-29 00:00:00',705.57,639,39112,62.91,'Molniya' from dual;
insert into SATELLITE
select 19449,'019449','1988-069G','deb Blok-I','',61318,'Reentered','1988-09-16 00:00:00','1988-08-23 00:00:00',92.14,223,536,62.83,'Intermediate' from dual union all
select 1945,'001945','1966-003A','Kosmos-105','Zenit-2 No. 38',13410,'Landed','1966-01-30 00:00:00','1966-01-23 00:00:00',89.64,204,310,65.01,'Intermediate' from dual union all
select 19450,'019450','1978-100AH','deb S5M','',45295,'In Earth orbit','','1988-06-09 00:00:00',120.23,1681,1702,82.55,'Medium' from dual union all
select 19451,'019451','1988-016K','deb S3M','',61265,'In Earth orbit','','1988-06-02 00:00:00',117.63,1459,1689,74,'Intermediate' from dual union all
select 19452,'019452','1987-020CB','deb Kosmos-1823','',60110,'Reentered','1990-02-23 00:00:00','1988-07-08 00:00:00',116.02,1479,1525,73.61,'Intermediate' from dual union all
select 19453,'019453','1982-055BL','deb Kosmos-1375','',54364,'In Earth orbit','','1986-04-13 00:00:00',103.5,857,992,65.83,'Intermediate' from dual union all
select 19454,'019454','1977-065FZ','deb Delta 132','',42193,'Reentered','1999-07-15 00:00:00','1987-10-04 00:00:00',105.95,537,1541,28.41,'Intermediate' from dual union all
select 19455,'019455','1977-065GA','deb Delta 132','',42193,'In Earth orbit','','1980-08-11 00:00:00',114.25,631,2212,28.82,'Intermediate' from dual union all
select 19456,'019456','1987-020CC','deb Kosmos-1823','',60110,'Reentered','1988-11-04 00:00:00','1988-03-09 00:00:00',113.71,800,1994,73.5,'Intermediate' from dual union all
select 19457,'019457','1988-044G','deb Molniya-3','',61293,'Reentered','1988-10-09 00:00:00','1988-07-18 00:00:00',92.24,242,527,62.81,'Intermediate' from dual union all
select 19458,'019458','1988-077A','USA 31','VORTEX 12',61326,'In Earth orbit','','1988-11-01 00:00:00',707.97,470,39400,26.7,'GEO Transfer' from dual union all
select 19459,'019459','1988-077B','Titan 34D-3 Stage 2','LR91AJ11 34D-3',61326,'Reentered','1988-10-01 00:00:00','1988-09-02 00:00:00',100.46,151,1409,29.3,'Intermediate' from dual union all
select 1946,'001946','1966-003B','8A92E','8A92E',13410,'Reentered','1966-02-10 00:00:00','1966-02-01 00:00:00',89.24,203,271,65.01,'Intermediate' from dual union all
select 19460,'019460','1988-078A','USA 32','P-11 No. 5103',61327,'In Earth orbit','','1991-04-27 00:00:00',100.65,785,793,84.99,'Intermediate' from dual union all
select 19461,'019461','1988-078B','Titan 23G-1 Stage 2','LR91AJ5 23G-1',61327,'Reentered','1988-09-08 00:00:00','1991-04-24 00:00:00',89.27,185,291,85,'Polar' from dual union all
select 19462,'019462','1988-079A','Kosmos-1967','Oblik No. 372',61328,'Landed','1988-09-15 00:00:00','1988-09-08 00:00:00',89.55,227,278,72.87,'Intermediate' from dual union all
select 19463,'019463','1988-079B','11S510','11S510',61328,'Reentered','1988-09-15 00:00:00','1988-09-08 00:00:00',89.94,187,356,72.88,'Intermediate' from dual union all
select 19464,'019464','1988-048C','Soyuz TM-5 BO','Bitovoy otsek',61297,'Reentered','1989-01-02 00:00:00','1988-09-06 00:00:00',91.49,334,362,51.62,'Intermediate' from dual union all
select 19465,'019465','1988-079C','Sensor cover','Sensor cover',61328,'Reentered','1988-09-08 00:00:00','1988-09-07 00:00:00',89.3,181,298,72.88,'Intermediate' from dual union all
select 19466,'019466','1988-079D','Sensor cover','Sensor cover',61328,'Reentered','1988-09-08 00:00:00','1988-09-06 00:00:00',89.84,185,348,72.87,'Intermediate' from dual union all
select 19467,'019467','1988-080A','Feng Yun 1A','feng yun yi jia',61329,'In Earth orbit','','1988-10-07 00:00:00',102.81,880,904,99.14,'Sun-Synch' from dual union all
select 19468,'019468','1988-080B','CZ-4 Stage 3','CZ-4 Stage 3',61329,'In Earth orbit','','1988-10-07 00:00:00',102.78,884,897,99.12,'Sun-Synch' from dual union all
select 19469,'019469','1988-072E','Blenda','Sunshade',61321,'Reentered','1988-09-10 00:00:00','1988-09-07 00:00:00',89.3,222,258,70,'Intermediate' from dual union all
select 1947,'001947','1966-003C','deb Kosmos-105','',13410,'Reentered','1966-01-24 00:00:00','1966-01-23 00:00:00',88.14,171,194,65,'Intermediate' from dual union all
select 19470,'019470','1988-072F','KDU part','KDU part',61321,'Reentered','1988-09-11 00:00:00','1988-09-07 00:00:00',91.38,240,445,70,'Intermediate' from dual union all
select 19471,'019471','1988-072G','KDU','KDU',61321,'Reentered','1988-09-25 00:00:00','1988-09-07 00:00:00',89.49,225,274,70,'Intermediate' from dual union all
select 19472,'019472','1988-072H','KDU part','KDU part',61321,'Reentered','1988-09-09 00:00:00','1988-09-07 00:00:00',89.37,220,267,70,'Intermediate' from dual union all
select 19473,'019473','1982-055BM','deb Kosmos-1375','',54364,'In Earth orbit','','1987-11-01 00:00:00',105.17,985,1020,65.83,'Intermediate' from dual union all
select 19474,'019474','1987-004HD','deb Kosmos-1813','',60094,'Reentered','1989-06-26 00:00:00','1988-09-05 00:00:00',102.84,379,1406,72.87,'Intermediate' from dual union all
select 19475,'019475','1976-126CD','deb Kosmos-886','',39078,'In Earth orbit','','1986-04-14 00:00:00',114.1,614,2215,65.84,'Intermediate' from dual union all
select 19476,'019476','1968-091DK','deb Kosmos-249  [*}','',16818,'Reentered','1988-10-01 00:00:00','1988-02-20 00:00:00',125.72,256,3614,62.94,'Medium' from dual union all
select 19477,'019477','1986-019VA','deb Ariane V16','',59212,'Reentered','1989-01-04 00:00:00','1988-07-09 00:00:00',101.1,585,1037,99.54,'Sun-Synch' from dual union all
select 19478,'019478','1981-070L','DE-1 boom cover','',52593,'In Earth orbit','','1983-01-19 00:00:00',410.41,565,23267,89.76,'Highly Ellip' from dual union all
select 19479,'019479','1968-097ET','deb Kosmos-252','',16824,'In Earth orbit','','1988-07-04 00:00:00',108.77,512,1828,62.81,'Intermediate' from dual union all
select 1948,'001948','1965-098F','DME-A adapter petal','Adapter Petal',11848,'In Earth orbit','','1966-01-31 00:00:00',121.4,502,2985,79.88,'Medium' from dual union all
select 19480,'019480','1976-126CE','deb Kosmos-886','',39078,'Reentered','1990-01-03 00:00:00','1988-09-07 00:00:00',111.69,577,2032,65.3,'Intermediate' from dual union all
select 19481,'019481','1988-044H','deb Molniya-3','',61293,'Reentered','1988-10-13 00:00:00','1988-06-29 00:00:00',92.66,244,566,62.82,'Intermediate' from dual union all
select 19482,'019482','61 Omicron 295','deb Ablestar','',7273,'Reentered','2004-03-27 00:00:00','1988-08-03 00:00:00',100.88,698,902,66.36,'Intermediate' from dual union all
select 19483,'019483','1988-081A','Gstar 3','Gstar 3',61330,'In Earth orbit','','1994-04-01 00:00:00',1436.09,35772,35801,5.68,'Inclined GEO' from dual union all
select 19484,'019484','1988-081B','SBS 5','SBS 5',61330,'In Earth orbit','','1988-10-09 00:00:00',1436.06,35785,35787,.04,'Stationary' from dual union all
select 19485,'019485','1988-081C','Ariane H10 V25','Ariane H10 S/N T24',61330,'Reentered','2008-08-02 00:00:00','1988-10-09 00:00:00',653.86,299,36854,6.88,'GEO Transfer' from dual union all
select 19486,'019486','1988-083A','Progress-38','Progress 7K-TG No. 146',61332,'Deorbited','1988-11-23 00:00:00','1988-10-09 00:00:00',91.37,329,355,51.62,'Intermediate' from dual union all
select 19487,'019487','1988-081D','Sylda 4400 No. 10','Sylda 4400',61330,'Reentered','1990-01-03 00:00:00','1988-10-25 00:00:00',607.69,245,34530,6.99,'GEO Transfer' from dual union all
select 19488,'019488','1988-082A','Kosmos-1968','Resurs-F1 14F43 No. 31',61331,'Landed','1988-09-23 00:00:00','1988-09-11 00:00:00',89.83,259,274,82.33,'Intermediate' from dual union all
select 19489,'019489','1988-082B','11S510','11S510',61331,'Reentered','1988-09-11 00:00:00','1988-09-10 00:00:00',88.15,167,199,82.33,'Intermediate' from dual union all
select 1949,'001949','1966-004A','Kosmos-106','DS-P1-I No. 1',13411,'Reentered','1966-11-14 00:00:00','1966-04-22 00:00:00',92.35,278,502,48.37,'Intermediate' from dual union all
select 19490,'019490','1988-077C','Transtage D-3','Transtage D-3',61326,'In Earth orbit','','1988-09-10 00:00:00',708.58,450,39450,26.7,'GEO Transfer' from dual union all
select 19491,'019491','1988-083B','11S510','11S510',61332,'Reentered','1988-09-12 00:00:00','1988-09-11 00:00:00',88.5,179,222,51.62,'Intermediate' from dual union all
select 19492,'019492','1988-079E','KDU part','KDU part',61328,'Reentered','1988-09-16 00:00:00','1988-09-16 00:00:00',88.45,188,208,72.87,'Intermediate' from dual union all
select 19493,'019493','1988-079F','Blenda','Sunshade',61328,'Reentered','1988-09-16 00:00:00','1988-09-15 00:00:00',90.99,229,417,72.79,'Intermediate' from dual union all
select 19494,'019494','1988-079G','KDU','KDU',61328,'Reentered','1988-09-23 00:00:00','1988-09-15 00:00:00',89.08,210,247,72.87,'Intermediate' from dual union all
select 19495,'019495','1988-084A','Kosmos-1969','Kobal''t No. 876',61333,'Landed','1988-11-13 00:00:00','1988-09-22 00:00:00',89.76,177,349,67.14,'Intermediate' from dual union all
select 19496,'019496','1988-084B','11S510','11S510',61333,'Reentered','1988-09-19 00:00:00','1988-09-18 00:00:00',87.73,135,190,67.12,'Intermediate' from dual union all
select 19497,'019497','1988-079H','KDU part','KDU part',61328,'Reentered','1988-09-18 00:00:00','1988-09-15 00:00:00',90.89,216,420,72.88,'Intermediate' from dual union all
select 19498,'019498','1988-079J','KDU part','KDU part',61328,'Reentered','1988-09-18 00:00:00','1988-09-15 00:00:00',90.29,230,348,72.83,'Intermediate' from dual union all
select 19499,'019499','1988-079K','KDU part','KDU part',61328,'Reentered','1988-09-16 00:00:00','1988-09-15 00:00:00',88.06,170,187,72.87,'Intermediate' from dual union all
select 195,'000195','61 A Delta 4','Midas IV solar array cover','Strongback',7251,'In Earth orbit','','1961-12-21 00:00:00',166.36,3522,3762,95.9,'Medium' from dual union all
select 1950,'001950','1966-004B','S1M','S1M',13411,'Reentered','1966-07-16 00:00:00','1966-04-21 00:00:00',91.81,275,451,48.38,'Intermediate' from dual union all
select 19500,'019500','1988-084C','Kosmos-1969 part','Kobal''t part',61333,'Reentered','1988-09-16 00:00:00','1988-09-15 00:00:00',89.02,157,296,67.06,'Intermediate' from dual union all
select 19501,'019501','1988-085A','Kosmos-1970','Uragan No. 42L',61334,'In Earth orbit','','1988-10-18 00:00:00',675.71,19117,19142,64.88,'Medium' from dual union all
select 19502,'019502','1988-085B','Kosmos-1971','Uragan No. 43L',61334,'In Earth orbit','','1988-10-17 00:00:00',675.71,19114,19145,64.87,'Medium' from dual union all
select 19503,'019503','1988-085C','Kosmos-1972','Uragan No. 44L',61334,'In Earth orbit','','1988-10-18 00:00:00',675.69,19117,19141,64.9,'Medium' from dual union all
select 19504,'019504','1988-085D','8S812','8S812',61334,'Reentered','1988-09-16 00:00:00','1988-09-16 00:00:00',86.96,115,133,64.83,'Intermediate' from dual union all
select 19505,'019505','1988-085E','Blok DM2','11S861 No. 43L',61334,'In Earth orbit','','1988-10-17 00:00:00',674.88,19093,19124,64.86,'Medium' from dual union all
select 19506,'019506','1988-065G','Kosmos-1960 SS 1','ESO',61314,'Reentered','1989-01-30 00:00:00','1988-09-19 00:00:00',94.42,468,513,65.81,'Intermediate' from dual union all
select 19507,'019507','1988-065H','Kosmos-1960 SS 2','ESO',61314,'Reentered','1989-01-27 00:00:00','1988-09-19 00:00:00',94.37,464,513,65.87,'Intermediate' from dual union all
select 19508,'019508','1988-086A','Sakura 3B','CS-3B',61335,'In Earth orbit','','1988-11-09 00:00:00',1436.11,35785,35789,.09,'Stationary' from dual union all
select 19509,'019509','1988-086B','H-I Stage 2 S/N H-19(F)','LE-5',61335,'Reentered','1988-09-21 00:00:00','1988-09-19 00:00:00',89.29,187,292,30.72,'Intermediate' from dual union all
select 1951,'001951','1965-098G','DME-A adapter petal','Adapter Petal',11848,'In Earth orbit','','1966-01-23 00:00:00',121.26,501,2974,79.79,'Medium' from dual union all
select 19510,'019510','1986-019VB','deb Ariane V16','',59212,'Reentered','1991-10-25 00:00:00','1988-09-20 00:00:00',98.72,661,734,97.94,'Sun-Synch' from dual union all
select 19511,'019511','1965-027AU','deb SNAPSHOT','',11777,'In Earth orbit','','1988-06-22 00:00:00',111.46,1272,1316,90.3,'Polar' from dual union all
select 19512,'019512','1973-086HF','deb Delta 98','',30688,'In Earth orbit','','1986-04-23 00:00:00',116.65,1507,1554,101.95,'Sun-Synch' from dual union all
select 19513,'019513','1986-019VC','deb Ariane V16','',59212,'Reentered','1989-09-11 00:00:00','1988-07-24 00:00:00',103.8,798,1079,97.95,'Sun-Synch' from dual union all
select 19514,'019514','1977-121BU','deb Kosmos-970','',42249,'Reentered','1989-02-02 00:00:00','1988-05-25 00:00:00',109.8,534,1901,65.79,'Intermediate' from dual union all
select 19515,'019515','1988-074D','SOOS 4 adapter','',61323,'In Earth orbit','','1988-09-25 00:00:00',107.43,1033,1183,89.94,'Polar' from dual union all
select 19516,'019516','1988-074E','SOOS 4 adapter','',61323,'In Earth orbit','','1988-09-26 00:00:00',107.37,1034,1175,89.99,'Polar' from dual union all
select 19517,'019517','1988-019E','Kosmos-1932 reactor part','',61268,'Reentered','1988-11-03 00:00:00','1988-05-21 00:00:00',104.89,904,1075,65,'Intermediate' from dual union all
select 19518,'019518','1965-108Z','deb Transtage 8','',11858,'In Earth orbit','','1988-02-20 00:00:00',519.15,515,29523,26.22,'Highly Ellip' from dual union all
select 19519,'019519','1988-087A','''Ofeq-1','''Ofeq-1',61336,'Reentered','1989-01-14 00:00:00','1988-10-19 00:00:00',97.46,242,1033,142.85,'Retrograde' from dual union all
select 1952,'001952','1966-005A','NNS O-7','NNS O-7',13412,'In Earth orbit','','1966-05-30 00:00:00',105.91,864,1209,89.7,'Polar' from dual union all
select 19520,'019520','1988-087B','AUS-51','AUS-51',61336,'Reentered','1989-01-12 00:00:00','1988-10-19 00:00:00',97.29,239,1019,142.85,'Retrograde' from dual union all
select 19521,'019521','1988-088A','Kosmos-1973','Oblik No. 379',61337,'Landed','1988-10-10 00:00:00','1988-09-30 00:00:00',92.25,356,413,72.86,'Intermediate' from dual union all
select 19522,'019522','1988-088B','11S510','11S510',61337,'Reentered','1988-10-01 00:00:00','1988-09-26 00:00:00',89.41,182,308,72.85,'Intermediate' from dual union all
select 19523,'019523','1988-073D','Blenda','Sunshade',61322,'Reentered','1988-10-04 00:00:00','1988-09-23 00:00:00',91.32,332,347,82.31,'Intermediate' from dual union all
select 19524,'019524','1988-073E','KDU','KDU',61322,'Reentered','1988-10-18 00:00:00','1988-09-23 00:00:00',91.32,330,348,82.32,'Intermediate' from dual union all
select 19525,'019525','1988-073F','Solnechnaya Batareya','Solar panel module',61322,'Reentered','1988-10-13 00:00:00','1988-09-22 00:00:00',93.55,346,550,82.31,'Intermediate' from dual union all
select 19526,'019526','1988-073G','KDU part','KDU part',61322,'Reentered','1988-10-02 00:00:00','1988-09-22 00:00:00',93.88,341,588,82.37,'Intermediate' from dual union all
select 19527,'019527','1988-082C','Blenda','Sunshade',61331,'Reentered','1988-09-28 00:00:00','1988-09-26 00:00:00',90.82,243,387,82.34,'Intermediate' from dual union all
select 19528,'019528','1988-082D','KDU','KDU',61331,'Reentered','1988-10-16 00:00:00','1988-09-26 00:00:00',89.71,254,267,82.33,'Intermediate' from dual union all
select 19529,'019529','1988-082E','KDU part','KDU part',61331,'Reentered','1988-09-27 00:00:00','1988-09-26 00:00:00',88.99,214,236,82.32,'Intermediate' from dual union all
select 1953,'001953','1966-005B','Altair 2 X-258E6 RH-116','X-258E6',13412,'In Earth orbit','','1966-03-28 00:00:00',105.9,862,1210,89.71,'Polar' from dual union all
select 19530,'019530','1988-082F','KDU part','KDU part',61331,'Reentered','1988-09-26 00:00:00','1988-09-25 00:00:00',88.75,205,221,82.32,'Intermediate' from dual union all
select 19531,'019531','1988-089A','NOAA 11','NOAA H',61338,'In Earth orbit','','1988-10-24 00:00:00',102.06,848,865,98.91,'Sun-Synch' from dual union all
select 19532,'019532','1988-089B','NOAA 11 solar array band','',61338,'Reentered','1999-05-23 00:00:00','1988-10-24 00:00:00',102.02,850,859,98.9,'Sun-Synch' from dual union all
select 19533,'019533','1986-017R','Garbage bag','',59210,'Reentered','1988-10-02 00:00:00','1988-09-26 00:00:00',91.34,332,349,51.58,'Intermediate' from dual union all
select 19534,'019534','1988-089C','NOAA 11 solar array band','',61338,'Reentered','1992-06-18 00:00:00','1988-10-24 00:00:00',102.05,850,862,98.9,'Sun-Synch' from dual union all
select 19535,'019535','1988-085F','SOZ','SOZ',61334,'In Earth orbit','','1988-10-21 00:00:00',340.12,413,19112,64.93,'Highly Ellip' from dual union all
select 19536,'019536','1986-017S','Garbage bag','',59210,'Reentered','1988-11-09 00:00:00','1988-09-27 00:00:00',91.35,325,357,51.64,'Intermediate' from dual union all
select 19537,'019537','1988-085G','SOZ','SOZ',61334,'In Earth orbit','','1988-10-17 00:00:00',340.15,411,19115,64.83,'Highly Ellip' from dual union all
select 19538,'019538','1986-017T','Garbage bag','',59210,'Reentered','1988-10-17 00:00:00','1988-09-29 00:00:00',91.19,319,346,51.62,'Intermediate' from dual;
insert into SATELLITE
select 19539,'019539','1986-017U','Garbage bag','',59210,'Reentered','1988-10-01 00:00:00','1988-09-28 00:00:00',91.23,326,343,51.61,'Intermediate' from dual union all
select 1954,'001954','1966-006A','Luna-9','E-6M No. 202 (13)',13413,'Deep Space','1966-02-03 00:00:00','1966-01-31 00:00:00',21545.62,219,500000,51.85,'Cislunar' from dual union all
select 19540,'019540','1986-017V','Garbage bag','',59210,'Reentered','1988-10-23 00:00:00','1988-09-28 00:00:00',91.27,325,349,51.61,'Intermediate' from dual union all
select 19541,'019541','1988-090A','Molniya-3','Molniya-3 No. 51',61339,'Reentered','2002-02-04 00:00:00','1988-10-29 00:00:00',717.82,612,39745,62.89,'Molniya' from dual union all
select 19542,'019542','1988-090B','11S510','11S510',61339,'Reentered','1988-10-29 00:00:00','1988-10-10 00:00:00',91.62,209,499,62.84,'Intermediate' from dual union all
select 19543,'019543','1988-090C','BOZ','BOZ',61339,'Reentered','1988-10-22 00:00:00','1988-10-10 00:00:00',91.55,189,513,62.82,'Intermediate' from dual union all
select 19544,'019544','1988-090D','Blok-ML','Blok-ML',61339,'Reentered','2002-02-14 00:00:00','1988-10-30 00:00:00',698.63,609,38797,62.93,'Molniya' from dual union all
select 19545,'019545','1986-067AC','Kosmos-1776 SS 21','ESO',59260,'Reentered','1988-11-08 00:00:00','1988-09-30 00:00:00',94.28,457,511,74.03,'Intermediate' from dual union all
select 19546,'019546','1986-067AD','Kosmos-1776 SS 22','ESO',59260,'Reentered','1988-10-31 00:00:00','1988-09-29 00:00:00',93.97,457,481,74.02,'Intermediate' from dual union all
select 19547,'019547','1988-091A','Discovery (STS-26R)','OV-103',61340,'Landed','1988-10-03 00:00:00','1988-10-01 00:00:00',90.85,302,330,28.47,'Intermediate' from dual union all
select 19548,'019548','1988-091B','TDRS 3','TDRS C',61340,'In Earth orbit','','1988-10-30 00:00:00',1436.29,35784,35797,.01,'Stationary' from dual union all
select 19549,'019549','1988-091C','IUS-7 SRM-1','Orbus 21',61340,'In Earth orbit','','1988-10-30 00:00:00',632.4,290,35765,26.78,'GEO Transfer' from dual union all
select 1955,'001955','1966-006B','BOZ','BOZ',13413,'Reentered','1966-02-02 00:00:00','1966-01-31 00:00:00',88.35,168,218,51.85,'Intermediate' from dual union all
select 19550,'019550','1988-091D','IUS-7 SRM-2','Orbus 6E',61340,'In Earth orbit','','1988-11-05 00:00:00',1433.23,35667,35793,2.16,'Drift GEO' from dual union all
select 19551,'019551','1988-065J','Kosmos-1960 SS 3','ESO',61314,'Reentered','1989-01-31 00:00:00','1988-10-02 00:00:00',94.3,461,509,65.83,'Intermediate' from dual union all
select 19552,'019552','1988-065K','Kosmos-1960 SS 4','ESO',61314,'Reentered','1989-02-12 00:00:00','1988-10-02 00:00:00',94.42,466,515,65.84,'Intermediate' from dual union all
select 19553,'019553','1987-101B','Kosmos-1900 DU','US-AM DU',60191,'Reentered','1988-10-01 00:00:00','1988-10-01 00:00:00',86.76,108,120,64.94,'Intermediate' from dual union all
select 19554,'019554','1988-092A','Kosmos-1974','Oko',61341,'In Earth orbit','','1988-11-03 00:00:00',718.15,601,39772,62.93,'Molniya' from dual union all
select 19555,'019555','1988-092B','BOZ','BOZ',61341,'Reentered','1988-10-23 00:00:00','1988-10-11 00:00:00',91.67,185,528,62.85,'Intermediate' from dual union all
select 19556,'019556','1988-092C','11S510','11S510',61341,'Reentered','1988-10-28 00:00:00','1988-10-10 00:00:00',91.79,208,517,62.79,'Intermediate' from dual union all
select 19557,'019557','1988-092D','Blok-2BL','Blok-2BL No. 78',61341,'In Earth orbit','','1988-11-03 00:00:00',705.53,601,39148,62.91,'Molniya' from dual union all
select 19558,'019558','1988-086C','UM129A','UM129A',61335,'Reentered','1989-06-16 00:00:00','1988-10-18 00:00:00',667.29,208,37626,28.15,'GEO Transfer' from dual union all
select 19559,'019559','1988-074F','SOOS-4 despin weight','',61323,'In Earth orbit','','1988-09-26 00:00:00',107.34,1030,1177,89.44,'Polar' from dual union all
select 1956,'001956','1965-082GR','deb Transtage 4','',11832,'Reentered','1968-08-12 00:00:00','1966-01-29 00:00:00',97.28,511,746,31.99,'Intermediate' from dual union all
select 19560,'019560','1988-081E','deb Ariane V25','deb H10',61330,'Reentered','1999-02-19 00:00:00','1988-10-10 00:00:00',116.71,924,2142,62.32,'Intermediate' from dual union all
select 19561,'019561','1968-097EU','deb Kosmos-252','',16824,'In Earth orbit','','1983-01-29 00:00:00',107.85,523,1732,62.29,'Intermediate' from dual union all
select 19562,'019562','1987-020CD','deb Kosmos-1823','',60110,'In Earth orbit','','1988-10-05 00:00:00',105.64,933,1116,73.51,'Intermediate' from dual union all
select 19563,'019563','1987-004HC','deb Kosmos-1813','',60094,'Reentered','1989-01-13 00:00:00','1988-09-05 00:00:00',97.42,354,916,72.81,'Intermediate' from dual union all
select 19564,'019564','1988-088C','Blenda','Sunshade',61337,'Reentered','1988-11-22 00:00:00','1988-10-24 00:00:00',93.63,379,526,72.86,'Intermediate' from dual union all
select 19565,'019565','1988-088D','KDU','KDU',61337,'Reentered','1989-03-31 00:00:00','1988-10-23 00:00:00',92.12,351,406,72.86,'Intermediate' from dual union all
select 19566,'019566','1988-088E','KDU part','KDU part',61337,'Reentered','1988-11-20 00:00:00','1988-10-11 00:00:00',92.04,344,405,72.85,'Intermediate' from dual union all
select 19567,'019567','1988-088F','KDU part','KDU part',61337,'Reentered','1988-10-12 00:00:00','1988-10-10 00:00:00',92.04,347,403,72.87,'Intermediate' from dual union all
select 19568,'019568','1986-017W','Garbage bag','',59210,'Reentered','1988-10-22 00:00:00','1988-10-12 00:00:00',90.38,284,303,51.62,'Intermediate' from dual union all
select 19569,'019569','1986-017X','Garbage bag','',59210,'Reentered','1988-10-22 00:00:00','1988-10-10 00:00:00',90.56,287,317,51.62,'Intermediate' from dual union all
select 1957,'001957','1965-082GS','deb Transtage 4','',11832,'Reentered','1979-02-02 00:00:00','1979-01-15 00:00:00',92.53,391,405,32.42,'Intermediate' from dual union all
select 19570,'019570','1986-017Y','Garbage bag','',59210,'Reentered','1988-10-19 00:00:00','1988-10-11 00:00:00',90.4,286,303,51.61,'Intermediate' from dual union all
select 19571,'019571','1986-017Z','Garbage bag','',59210,'Reentered','1988-10-20 00:00:00','1988-10-10 00:00:00',90.48,289,308,51.62,'Intermediate' from dual union all
select 19572,'019572','1988-088G','KDU part','KDU part',61337,'Reentered','1988-11-02 00:00:00','1988-10-11 00:00:00',94.55,392,602,72.87,'Intermediate' from dual union all
select 19573,'019573','1988-093A','Kosmos-1975','Tselina-D',61342,'In Earth orbit','','1989-02-25 00:00:00',97.65,629,664,82.53,'Intermediate' from dual union all
select 19574,'019574','1988-093B','S5M','S5M',61342,'In Earth orbit','','1988-12-25 00:00:00',97.65,628,665,82.54,'Intermediate' from dual union all
select 19575,'019575','1988-088H','KDU part','KDU part',61337,'Reentered','1988-11-01 00:00:00','1988-10-12 00:00:00',92.13,351,407,72.86,'Intermediate' from dual union all
select 19576,'019576','1965-027AV','deb SNAPSHOT','',11777,'In Earth orbit','','1988-01-08 00:00:00',111.25,1262,1306,90.31,'Polar' from dual union all
select 19577,'019577','1988-074G','SOOS-4 despin weight','',61323,'In Earth orbit','','1988-09-26 00:00:00',107.38,1035,1176,90.5,'Polar' from dual union all
select 19578,'019578','1978-100AJ','deb S5M','',45295,'Reentered','1993-05-23 00:00:00','1988-05-20 00:00:00',118.38,1527,1690,82.46,'Intermediate' from dual union all
select 19579,'019579','1978-100AK','deb S5M','',45295,'Reentered','1999-11-28 00:00:00','1988-05-22 00:00:00',118.49,1592,1634,82.54,'Intermediate' from dual union all
select 1958,'001958','1965-082GT','deb Transtage 4','',11832,'Reentered','1979-05-23 00:00:00','1979-04-28 00:00:00',92.27,377,395,31.05,'Intermediate' from dual union all
select 19580,'019580','1978-100AL','deb S5M','',45295,'In Earth orbit','','1988-07-21 00:00:00',119.6,1584,1742,82.31,'Intermediate' from dual union all
select 19581,'019581','1969-082LF','deb Agena D','',18349,'Reentered','2012-12-10 00:00:00','1988-06-25 00:00:00',100.84,754,842,70.09,'Intermediate' from dual union all
select 19582,'019582','1988-094A','Kosmos-1976','Oblik No. 380',61343,'Landed','1988-10-27 00:00:00','1988-10-15 00:00:00',92.24,359,409,72.86,'Intermediate' from dual union all
select 19583,'019583','1988-094B','11S510','11S510',61343,'Reentered','1988-10-21 00:00:00','1988-10-16 00:00:00',89.45,183,312,72.86,'Intermediate' from dual union all
select 19584,'019584','1986-017AA','Garbage bag','',59210,'Reentered','1988-11-05 00:00:00','1988-10-16 00:00:00',91.2,321,345,51.62,'Intermediate' from dual union all
select 19585,'019585','1986-017AB','Garbage bag','',59210,'Reentered','1988-11-26 00:00:00','1988-10-19 00:00:00',91.15,316,346,51.61,'Intermediate' from dual union all
select 19586,'019586','1986-017AC','Garbage bag','',59210,'Reentered','1988-11-08 00:00:00','1988-10-19 00:00:00',91.05,311,342,51.62,'Intermediate' from dual union all
select 19587,'019587','1986-017AD','Garbage bag','',59210,'Reentered','1988-11-04 00:00:00','1988-10-19 00:00:00',91.02,318,332,51.62,'Intermediate' from dual union all
select 19588,'019588','1986-017AE','Garbage bag','',59210,'Reentered','1988-11-05 00:00:00','1988-10-19 00:00:00',91.01,313,335,51.61,'Intermediate' from dual union all
select 19589,'019589','1987-101C','Kosmos-1900 reactor part','',60191,'Reentered','1988-11-03 00:00:00','1988-10-07 00:00:00',100.52,700,867,66.06,'Intermediate' from dual union all
select 1959,'001959','1965-082GU','deb Transtage 4','',11832,'In Earth orbit','','1992-01-28 00:00:00',99.43,678,785,31.79,'Intermediate' from dual union all
select 19590,'019590','1973-086HG','deb Delta 98','',30688,'In Earth orbit','','1985-05-06 00:00:00',116.9,1505,1577,102.01,'Sun-Synch' from dual union all
select 19591,'019591','1988-088J','KDU part','KDU part',61337,'Reentered','1989-02-02 00:00:00','1988-10-24 00:00:00',92.12,352,404,72.86,'Intermediate' from dual union all
select 19592,'019592','1965-108AA','deb Transtage 8','',11858,'In Earth orbit','','1988-08-14 00:00:00',532.05,551,30193,26.62,'Highly Ellip' from dual union all
select 19593,'019593','1965-027AW','deb SNAPSHOT','',11777,'In Earth orbit','','1990-07-13 00:00:00',111.4,1270,1313,90.31,'Polar' from dual union all
select 19594,'019594','1978-026HN','deb Delta 139','',45221,'In Earth orbit','','1986-04-13 00:00:00',106.43,888,1235,98.76,'Sun-Synch' from dual union all
select 19595,'019595','1965-108AB','deb Transtage 8','',11858,'In Earth orbit','','1988-08-19 00:00:00',439.78,376,25181,26.92,'Highly Ellip' from dual union all
select 19596,'019596','1988-095A','Raduga','Gran'' No. 34L',61344,'In Earth orbit','','1988-11-20 00:00:00',1436.01,35760,35810,1.42,'Stationary' from dual union all
select 19597,'019597','1988-095B','8S812','8S812',61344,'Reentered','1988-10-22 00:00:00','1988-10-22 00:00:00',87.63,154,161,51.63,'Intermediate' from dual union all
select 19598,'019598','1988-095C','Perekhodnik','Sredniy perekhodnik',61344,'Reentered','1988-10-21 00:00:00','1988-10-21 00:00:00',87.48,148,152,51.6,'Intermediate' from dual union all
select 19599,'019599','1984-088H','Delta 175 part','',57142,'In Earth orbit','','1988-11-21 00:00:00',133.58,549,4007,28.76,'Medium' from dual union all
select 196,'000196','61 Sigma 4','Midas 3 solar array cover','Strongback',7278,'In Earth orbit','','1961-10-27 00:00:00',161.86,3373,3548,91.15,'Medium' from dual union all
select 1960,'001960','1965-082GV','deb Transtage 4','',11832,'Reentered','1982-03-23 00:00:00','1982-03-22 00:00:00',90.16,270,294,31.86,'Intermediate' from dual union all
select 19600,'019600','1988-095D','SOZ','SOZ',61344,'Reentered','1989-08-12 00:00:00','1988-11-21 00:00:00',647.79,307,36537,47.28,'GEO Transfer' from dual union all
select 19601,'019601','1988-095E','SOZ','SOZ',61344,'Reentered','1989-05-24 00:00:00','1988-11-21 00:00:00',646.21,251,36512,47.23,'GEO Transfer' from dual union all
select 19602,'019602','1988-065L','Kosmos-1960 SS 5','ESO',61314,'Reentered','1989-02-25 00:00:00','1988-10-22 00:00:00',94.44,472,511,65.84,'Intermediate' from dual union all
select 19603,'019603','1988-065M','Kosmos-1960 SS 6','ESO',61314,'Reentered','1989-02-10 00:00:00','1988-10-22 00:00:00',94.21,451,510,65.84,'Intermediate' from dual union all
select 19604,'019604','1986-017AF','EO-3 EVA-3 debris','',59210,'Reentered','1988-11-10 00:00:00','1988-10-21 00:00:00',91.24,309,362,51.61,'Intermediate' from dual union all
select 19605,'019605','1986-017AG','EO-3 EVA-3 debris','',59210,'Reentered','1988-11-06 00:00:00','1988-10-21 00:00:00',91.23,314,356,51.61,'Intermediate' from dual union all
select 19606,'019606','1986-017AH','EO-3 EVA-3 debris','',59210,'Reentered','1989-01-05 00:00:00','1988-10-22 00:00:00',91.41,329,359,51.62,'Intermediate' from dual union all
select 19607,'019607','1986-017AJ','EO-3 EVA-3 debris','',59210,'Reentered','1989-02-22 00:00:00','1988-10-21 00:00:00',91.4,329,358,51.62,'Intermediate' from dual union all
select 19608,'019608','1988-096A','Kosmos-1977','Oko',61345,'In Earth orbit','','1988-11-24 00:00:00',717.82,654,39703,62.91,'Molniya' from dual union all
select 19609,'019609','1988-096B','11S510','11S510',61345,'Reentered','1988-11-25 00:00:00','1988-11-02 00:00:00',91.95,207,534,62.85,'Intermediate' from dual union all
select 1961,'001961','1965-082GW','deb Transtage 4','',11832,'In Earth orbit','','1992-01-27 00:00:00',98.09,659,676,31.88,'Intermediate' from dual union all
select 19610,'019610','1988-096C','BOZ','BOZ',61345,'Reentered','1988-11-09 00:00:00','1988-11-02 00:00:00',91.3,169,508,62.85,'Intermediate' from dual union all
select 19611,'019611','1988-096D','Blok-2BL','Blok-2BL No. 79',61345,'In Earth orbit','','1988-11-24 00:00:00',704.99,653,39069,62.89,'Molniya' from dual union all
select 19612,'019612','1988-097A','Kosmos-1978','Oblik No. 383',61346,'Landed','1988-11-10 00:00:00','1988-10-31 00:00:00',89.59,228,281,72.87,'Intermediate' from dual union all
select 19613,'019613','1988-097B','11S510','11S510',61346,'Reentered','1988-11-03 00:00:00','1988-10-31 00:00:00',89.41,181,310,72.87,'Intermediate' from dual union all
select 19614,'019614','1988-094C','KDU','KDU',61343,'Reentered','1989-04-14 00:00:00','1988-11-13 00:00:00',92.11,354,401,72.86,'Intermediate' from dual union all
select 19615,'019615','1988-094D','Blenda','Sunshade',61343,'Reentered','1988-11-12 00:00:00','1988-10-31 00:00:00',92.09,351,403,72.85,'Intermediate' from dual union all
select 19616,'019616','1988-094E','KDU part','KDU part',61343,'Reentered','1988-12-18 00:00:00','1988-11-13 00:00:00',91.58,329,375,72.86,'Intermediate' from dual union all
select 19617,'019617','1988-094F','KDU part','KDU part',61343,'Reentered','1988-12-10 00:00:00','1988-10-31 00:00:00',94.22,384,578,72.86,'Intermediate' from dual union all
select 19618,'019618','1986-017AK','Garbage bag','',59210,'Reentered','1988-12-07 00:00:00','1988-10-28 00:00:00',91.26,322,351,51.61,'Intermediate' from dual union all
select 19619,'019619','1986-017AL','Garbage bag','',59210,'Reentered','1988-12-08 00:00:00','1988-10-30 00:00:00',91.21,320,348,51.62,'Intermediate' from dual union all
select 1962,'001962','1965-082GX','deb Transtage 4','',11832,'Reentered','2000-01-15 00:00:00','1992-01-31 00:00:00',97.74,602,700,31.89,'Intermediate' from dual union all
select 19620,'019620','1986-017AM','Garbage bag','',59210,'Reentered','1988-11-06 00:00:00','1988-10-30 00:00:00',90.2,278,291,51.61,'Intermediate' from dual union all
select 19621,'019621','1988-098A','TDF 1','TDF 1',61347,'In Earth orbit','','1988-11-27 00:00:00',1436.11,35772,35802,.05,'Stationary' from dual union all
select 19622,'019622','1988-098B','Ariane H10 V26','Ariane H10 S/N T25',61347,'In Earth orbit','','1989-01-15 00:00:00',622.42,257,35283,3.74,'GEO Transfer' from dual union all
select 19623,'019623','1988-094G','KDU part','KDU part',61343,'Reentered','1988-11-16 00:00:00','1988-10-29 00:00:00',94.59,382,616,72.87,'Intermediate' from dual union all
select 19624,'019624','1986-017AN','Garbage bag','',59210,'Reentered','1988-11-06 00:00:00','1988-10-30 00:00:00',90.57,291,314,51.61,'Intermediate' from dual union all
select 19625,'019625','1988-099A','USA 33','Enhanced CRYSTAL 2103',61348,'Deorbited','1996-05-01 00:00:00','1989-05-06 00:00:00',96.54,302,884,97.83,'Sun-Synch' from dual union all
select 19626,'019626','1988-099B','Titan 34D-14 Stage 2','LR91AJ11 34D-14',61348,'Reentered','1988-12-01 00:00:00','1989-05-06 00:00:00',96.29,154,1008,97.9,'Sun-Synch' from dual union all
select 19627,'019627','1988-097C','KDU part','KDU part',61346,'Reentered','1988-11-12 00:00:00','1988-11-11 00:00:00',88.88,205,233,72.87,'Intermediate' from dual union all
select 19628,'019628','1988-097D','Blenda','Sunshade',61346,'Reentered','1988-11-12 00:00:00','1988-11-11 00:00:00',88.89,202,238,72.86,'Intermediate' from dual;
insert into SATELLITE
select 19629,'019629','1988-097E','KDU part','KDU part',61346,'Reentered','1988-11-14 00:00:00','1988-11-11 00:00:00',91.01,232,417,72.86,'Intermediate' from dual union all
select 1963,'001963','1965-082GY','deb Transtage 4','',11832,'Reentered','1995-11-20 00:00:00','1992-01-31 00:00:00',95.1,514,533,32.05,'Intermediate' from dual union all
select 19630,'019630','1988-097F','KDU part','KDU part',61346,'Reentered','1988-11-13 00:00:00','1988-11-10 00:00:00',89.1,208,251,72.86,'Intermediate' from dual union all
select 19631,'019631','1988-097G','KDU','KDU',61346,'Reentered','1988-11-23 00:00:00','1988-11-13 00:00:00',89.2,214,256,72.86,'Intermediate' from dual union all
select 19632,'019632','1986-017AP','Garbage bag','',59210,'Reentered','1988-12-23 00:00:00','1988-11-13 00:00:00',91.17,321,343,51.62,'Intermediate' from dual union all
select 19633,'019633','1986-017AQ','Garbage bag','',59210,'Reentered','1988-12-18 00:00:00','1988-11-14 00:00:00',91.15,319,342,51.62,'Intermediate' from dual union all
select 19634,'019634','1986-017AR','Garbage bag','',59210,'Reentered','1988-12-25 00:00:00','1988-11-13 00:00:00',91.17,314,350,51.61,'Intermediate' from dual union all
select 19635,'019635','1986-017AS','Garbage bag','',59210,'Reentered','1988-12-03 00:00:00','1988-11-14 00:00:00',91.07,314,341,51.63,'Intermediate' from dual union all
select 19636,'019636','1986-017AT','Garbage bag','',59210,'Reentered','1988-12-03 00:00:00','1988-11-16 00:00:00',90.97,307,338,51.62,'Intermediate' from dual union all
select 19637,'019637','1988-100A','Buran OK-1K','Buran OK-1K No. 711 (1K1',61349,'Landed','1988-11-15 00:00:00','1988-11-15 00:00:00',89.52,247,255,51.62,'Intermediate' from dual union all
select 19638,'019638','1977-065GB','Unknown deb                       ','',42193,'In Earth orbit','','1979-07-10 00:00:00',120.23,1022,2361,30.97,'Medium' from dual union all
select 19639,'019639','61 Omicron 296','deb Ablestar','',7273,'Reentered','2004-05-22 00:00:00','1985-03-12 00:00:00',101.78,786,900,66.62,'Intermediate' from dual union all
select 1964,'001964','1965-082GZ','deb Transtage 4','',11832,'In Earth orbit','','1992-01-28 00:00:00',102.51,710,1045,32.19,'Intermediate' from dual union all
select 19640,'019640','1987-020CE','deb Kosmos-1823','',60110,'In Earth orbit','','1988-01-03 00:00:00',115.78,1417,1565,73.25,'Intermediate' from dual union all
select 19641,'019641','1987-020CF','deb Kosmos-1823','',60110,'Reentered','2001-07-05 00:00:00','1988-01-03 00:00:00',116.97,1490,1600,73.74,'Intermediate' from dual union all
select 19642,'019642','1986-019VD','deb Ariane V16','',59212,'Reentered','1989-04-11 00:00:00','1987-10-29 00:00:00',101.1,753,869,98.64,'Sun-Synch' from dual union all
select 19643,'019643','1978-121C','deb Blok E','',45316,'In Earth orbit','','1988-11-03 00:00:00',102.01,803,904,81.24,'Intermediate' from dual union all
select 19644,'019644','1987-020CG','deb Kosmos-1823','',60110,'Reentered','1990-05-25 00:00:00','1988-04-28 00:00:00',117.37,1073,2053,73.4,'Intermediate' from dual union all
select 19645,'019645','1987-020CH','deb Kosmos-1823','',60110,'Reentered','1989-01-01 00:00:00','1988-09-23 00:00:00',115.03,1270,1643,73.3,'Intermediate' from dual union all
select 19646,'019646','1978-100AM','deb S5M','',45295,'In Earth orbit','','1988-05-30 00:00:00',118.88,1565,1697,82.15,'Intermediate' from dual union all
select 19647,'019647','1988-101A','Kosmos-1979','US-PM',61350,'Reentered','1989-12-25 00:00:00','1988-12-18 00:00:00',92.77,404,417,65.02,'Intermediate' from dual union all
select 19648,'019648','1988-101B','11S692','11S692',61350,'Reentered','1988-11-18 00:00:00','1988-11-18 00:00:00',88.81,110,322,64.99,'Intermediate' from dual union all
select 19649,'019649','1988-102A','Kosmos-1980','Tselina-2',61351,'In Earth orbit','','1988-12-23 00:00:00',101.95,848,854,71.01,'Intermediate' from dual union all
select 1965,'001965','1965-082HA','deb Transtage 4','',11832,'Reentered','1970-03-06 00:00:00','1966-01-29 00:00:00',98.13,556,783,31.86,'Intermediate' from dual union all
select 19650,'019650','1988-102B','11S772','11S772',61351,'In Earth orbit','','1988-12-23 00:00:00',101.81,834,854,71,'Intermediate' from dual union all
select 19651,'019651','1988-103A','Kosmos-1981','Oblik No. 389',61352,'Landed','1988-12-08 00:00:00','1988-11-27 00:00:00',89.45,225,270,62.81,'Intermediate' from dual union all
select 19652,'019652','1988-103B','11S510','11S510',61352,'Reentered','1988-12-10 00:00:00','1988-12-01 00:00:00',89.82,223,309,62.8,'Intermediate' from dual union all
select 19653,'019653','1988-103C','Sensor cover','Sensor cover',61352,'Reentered','1988-11-27 00:00:00','1988-11-26 00:00:00',88.59,185,225,62.79,'Intermediate' from dual union all
select 19654,'019654','1988-103D','Sensor cover','Sensor cover',61352,'Reentered','1988-11-28 00:00:00','1988-11-26 00:00:00',89.61,213,298,62.84,'Intermediate' from dual union all
select 19655,'019655','1988-103E','Sensor cover','Sensor cover',61352,'Reentered','1988-11-28 00:00:00','1988-11-27 00:00:00',89.64,213,301,62.78,'Intermediate' from dual union all
select 19656,'019656','1988-102C','Sep Motor Cover','Sep Motor Cover',61351,'In Earth orbit','','1988-12-23 00:00:00',105.24,846,1166,70.98,'Intermediate' from dual union all
select 19657,'019657','1988-102D','Sep Motor Cover','Sep Motor Cover',61351,'In Earth orbit','','1988-12-24 00:00:00',105.21,844,1165,70.99,'Intermediate' from dual union all
select 19658,'019658','1988-102E','Sep Motor Cover','Sep Motor Cover',61351,'In Earth orbit','','1988-12-23 00:00:00',105.01,846,1144,71.02,'Intermediate' from dual union all
select 19659,'019659','1988-102F','Sep Motor Cover','Sep Motor Cover',61351,'In Earth orbit','','1988-12-24 00:00:00',104.82,845,1126,71.03,'Intermediate' from dual union all
select 1966,'001966','1966-006C','11S510','11S510',13413,'Reentered','1966-01-31 00:00:00','1966-01-31 00:00:00',88.36,168,219,51.85,'Intermediate' from dual union all
select 19660,'019660','1988-104A','Soyuz TM-7','Soyuz 7K-STM No. 57',61353,'Landed','1989-04-27 00:00:00','1988-12-26 00:00:00',91.33,326,354,51.62,'Intermediate' from dual union all
select 19661,'019661','1988-104B','11S510','11S510',61353,'Reentered','1988-11-28 00:00:00','1988-11-28 00:00:00',88.07,165,192,51.61,'Intermediate' from dual union all
select 19662,'019662','1988-105A','Kosmos-1982','Oblik',61354,'Landed','1988-12-14 00:00:00','1988-12-04 00:00:00',92.16,347,414,69.99,'Intermediate' from dual union all
select 19663,'019663','1988-105B','11S510','11S510',61354,'Reentered','1988-12-10 00:00:00','1988-12-02 00:00:00',90.11,203,357,70,'Intermediate' from dual union all
select 19664,'019664','1965-027AX','deb SNAPSHOT','',11777,'In Earth orbit','','1988-06-08 00:00:00',111.37,1266,1314,90.31,'Polar' from dual union all
select 19665,'019665','1984-015F','deb Ohzora','',57069,'Reentered','1988-12-11 00:00:00','1988-10-30 00:00:00',94.07,332,614,74.65,'Intermediate' from dual union all
select 19666,'019666','1982-115AH','deb Blok-L','',54424,'Reentered','1989-01-08 00:00:00','1988-11-15 00:00:00',91.51,222,475,62.83,'Intermediate' from dual union all
select 19667,'019667','1970-025QC','deb Agena D','',21502,'In Earth orbit','','1985-07-30 00:00:00',113.04,1088,1644,99.75,'Sun-Synch' from dual union all
select 19668,'019668','1983-044FN','deb Kosmos-1461','',55801,'Reentered','1990-01-16 00:00:00','1988-09-29 00:00:00',99,582,840,65.02,'Intermediate' from dual union all
select 19669,'019669','1988-092E','deb Kosmos- 1974','',61341,'Reentered','1988-12-31 00:00:00','1988-11-13 00:00:00',91.96,234,508,62.83,'Intermediate' from dual union all
select 1967,'001967','1965-030D','Blok-L No. U103-35','Blok-L',11780,'Reentered','1979-06-04 00:00:00','1965-11-26 00:00:00',702.59,1975,37628,64.81,'GEO Transfer' from dual union all
select 19670,'019670','1988-106A','Atlantis (STS-27)','OV-104',61355,'Landed','1988-12-06 00:00:00','1988-12-05 00:00:00',93.55,443,454,56.99,'Intermediate' from dual union all
select 19671,'019671','1988-106B','LACROSSE 1','LACROSSE 1',61355,'Deorbited','1997-03-25 00:00:00','1989-01-05 00:00:00',98.33,647,711,56.98,'Intermediate' from dual union all
select 19672,'019672','1988-107A','Kosmos-1983','Oblik No. 387',61356,'Landed','1988-12-22 00:00:00','1988-12-11 00:00:00',89.08,201,257,62.81,'Intermediate' from dual union all
select 19673,'019673','1988-107B','11S510','11S510',61356,'Reentered','1988-12-14 00:00:00','1988-12-12 00:00:00',88.93,215,229,62.8,'Intermediate' from dual union all
select 19674,'019674','1988-103F','KDU','KDU',61352,'Reentered','1988-12-17 00:00:00','1988-12-09 00:00:00',89.21,216,255,62.81,'Intermediate' from dual union all
select 19675,'019675','1988-103G','KDU part','KDU part',61352,'Reentered','1988-12-09 00:00:00','1988-12-09 00:00:00',88.36,181,206,62.81,'Intermediate' from dual union all
select 19676,'019676','1988-103H','Blenda','Sunshade',61352,'Reentered','1988-12-11 00:00:00','1988-12-09 00:00:00',90.75,231,392,62.83,'Intermediate' from dual union all
select 19677,'019677','1988-107C','Sensor cover','Sensor cover',61356,'Reentered','1988-12-10 00:00:00','1988-12-09 00:00:00',88.91,215,227,62.78,'Intermediate' from dual union all
select 19678,'019678','1988-107D','Sensor cover','Sensor cover',61356,'Reentered','1988-12-10 00:00:00','1988-12-09 00:00:00',89.36,240,246,62.84,'Intermediate' from dual union all
select 19679,'019679','1988-107E','Sensor cover','Sensor cover',61356,'Reentered','1988-12-09 00:00:00','1988-12-09 00:00:00',88.44,183,212,62.8,'Intermediate' from dual union all
select 1968,'001968','1966-007A','OPS 7291','CORONA J-27',13414,'Reentered Attached','1966-02-27 00:00:00','1966-02-09 00:00:00',90.45,178,416,75.03,'Intermediate' from dual union all
select 19680,'019680','1986-017AU','ERA debris','',59210,'Reentered','1988-12-22 00:00:00','1988-12-12 00:00:00',91.4,326,361,51.62,'Intermediate' from dual union all
select 19681,'019681','1986-017AV','ERA debris','',59210,'Reentered','1988-12-28 00:00:00','1988-12-12 00:00:00',91.37,329,355,51.62,'Intermediate' from dual union all
select 19682,'019682','1986-017AW','ERA debris','',59210,'Reentered','1988-12-23 00:00:00','1988-12-11 00:00:00',91.35,326,356,51.62,'Intermediate' from dual union all
select 19683,'019683','1988-108A','Ekran-M','Ekran-M No. 12L',61357,'In Earth orbit','','1989-01-14 00:00:00',1436.04,35768,35803,1.38,'Stationary' from dual union all
select 19684,'019684','1988-108B','8S812','8S812',61357,'Reentered','1988-12-12 00:00:00','1988-12-11 00:00:00',87.76,162,166,51.62,'Intermediate' from dual union all
select 19685,'019685','1988-108C','Perekhodnik','Sredniy perekhodnik',61357,'Reentered','1988-12-11 00:00:00','1988-12-11 00:00:00',87.77,160,169,51.61,'Intermediate' from dual union all
select 19686,'019686','1988-108D','Blok DM2','11S861 No. 19L',61357,'In Earth orbit','','1989-01-12 00:00:00',1418.45,35367,35514,1.4,'Drift GEO' from dual union all
select 19687,'019687','1988-109A','Skynet 4B','Skynet 4B',61358,'In Earth orbit','','1989-01-14 00:00:00',1436.06,35777,35795,3,'Stationary' from dual union all
select 19688,'019688','1988-109B','Astra 1A','Astra 1A',61358,'In Earth orbit','','1989-01-14 00:00:00',1435.7,35759,35799,.1,'Stationary' from dual union all
select 19689,'019689','1988-109C','Ariane H10 V27','Ariane H10 S/N T402',61358,'In Earth orbit','','1989-01-14 00:00:00',644.62,411,36271,6.75,'GEO Transfer' from dual union all
select 1969,'001969','1965-082HB','deb Transtage 4','',11832,'Reentered','1969-09-08 00:00:00','1969-08-28 00:00:00',92.18,368,395,32.04,'Intermediate' from dual union all
select 19690,'019690','1988-109D','Spelda 10 S/N UV1','Spelda 10',61358,'Reentered','1993-10-22 00:00:00','1989-02-05 00:00:00',605.6,222,34444,6.68,'GEO Transfer' from dual union all
select 19691,'019691','1988-108E','SOZ','SOZ',61357,'Reentered','1991-11-13 00:00:00','1989-01-10 00:00:00',628.81,334,35536,47.24,'GEO Transfer' from dual union all
select 19692,'019692','1986-017AX','ERA debris','',59210,'Reentered','1988-12-19 00:00:00','1988-12-12 00:00:00',91.26,321,352,51.62,'Intermediate' from dual union all
select 19693,'019693','1986-017AY','ERA debris','',59210,'Reentered','1988-12-24 00:00:00','1988-12-11 00:00:00',91.37,328,355,51.63,'Intermediate' from dual union all
select 19694,'019694','1988-105C','Blenda','Sunshade',61354,'Reentered','1989-01-20 00:00:00','1988-12-15 00:00:00',91.97,334,409,69.99,'Intermediate' from dual union all
select 19695,'019695','1988-105D','KDU part','KDU part',61354,'Reentered','1989-01-13 00:00:00','1988-12-15 00:00:00',92.12,345,412,69.99,'Intermediate' from dual union all
select 19696,'019696','1988-105E','KDU','KDU',61354,'Reentered','1989-04-27 00:00:00','1989-01-03 00:00:00',91.96,341,401,69.99,'Intermediate' from dual union all
select 19697,'019697','1988-105F','KDU part','KDU part',61354,'Reentered','1989-01-14 00:00:00','1988-12-16 00:00:00',93.97,373,565,69.99,'Intermediate' from dual union all
select 19698,'019698','1988-105G','KDU part','KDU part',61354,'Reentered','1988-12-20 00:00:00','1988-12-16 00:00:00',92.74,340,478,70.03,'Intermediate' from dual union all
select 19699,'019699','1988-108F','SOZ','SOZ',61357,'Reentered','1989-09-06 00:00:00','1989-01-10 00:00:00',627.37,285,35511,47.19,'GEO Transfer' from dual union all
select 197,'000197','61 A Epsilon 1','Discoverer 34','CORONA C''-9',7252,'Reentered Attached','1962-12-07 00:00:00','1961-12-06 00:00:00',96.64,225,971,82.52,'Intermediate' from dual union all
select 1970,'001970','1965-082HC','deb Transtage 4','',11832,'Reentered','1969-10-11 00:00:00','1969-09-29 00:00:00',93.1,419,433,31.76,'Intermediate' from dual union all
select 19700,'019700','1986-017AZ','Garbage bag','',59210,'Reentered','1989-01-19 00:00:00','1988-12-18 00:00:00',91.34,330,351,51.62,'Intermediate' from dual union all
select 19701,'019701','1986-017BA','Garbage bag','',59210,'Reentered','1988-12-28 00:00:00','1988-12-15 00:00:00',91.31,323,355,51.62,'Intermediate' from dual union all
select 19702,'019702','1986-017BB','Garbage bag','',59210,'Reentered','1989-01-27 00:00:00','1988-12-15 00:00:00',91.31,321,356,51.61,'Intermediate' from dual union all
select 19703,'019703','1986-017BC','Garbage bag','',59210,'Reentered','1988-12-22 00:00:00','1988-12-18 00:00:00',90.39,284,304,51.62,'Intermediate' from dual union all
select 19704,'019704','1986-017BD','Garbage bag','',59210,'Reentered','1989-02-28 00:00:00','1988-12-18 00:00:00',91.41,330,358,51.61,'Intermediate' from dual union all
select 19705,'019705','1988-110A','Kosmos-1984','Kobal''t No. 519',61359,'Landed','1989-02-13 00:00:00','1988-12-24 00:00:00',89.03,174,280,62.84,'Intermediate' from dual union all
select 19706,'019706','1988-110B','11S510','11S510',61359,'Reentered','1988-12-20 00:00:00','1988-12-20 00:00:00',88.19,154,216,62.82,'Intermediate' from dual union all
select 19707,'019707','1986-017BE','Garbage bag','',59210,'Reentered','1989-01-23 00:00:00','1988-12-21 00:00:00',91.16,317,346,51.62,'Intermediate' from dual union all
select 19708,'019708','1986-017BF','Garbage bag','',59210,'Reentered','1989-01-20 00:00:00','1988-12-21 00:00:00',91.14,315,346,51.62,'Intermediate' from dual union all
select 19709,'019709','1988-107F','Blenda','Sunshade',61356,'Reentered','1988-12-23 00:00:00','1988-12-22 00:00:00',88.71,195,227,62.8,'Intermediate' from dual union all
select 1971,'001971','1965-082HD','deb Transtage 4','',11832,'Reentered','1970-09-22 00:00:00','1966-01-30 00:00:00',99.68,718,769,32.25,'Intermediate' from dual union all
select 19710,'019710','1988-111A','Zhongxing 2','DFH 2A-2',61360,'In Earth orbit','','1989-01-21 00:00:00',1436.11,35785,35789,.53,'Stationary' from dual union all
select 19711,'019711','1988-113E','Kosmos-1985 SS 4','ESO',61362,'Reentered','1989-03-17 00:00:00','1989-01-23 00:00:00',94.47,487,499,73.54,'Intermediate' from dual union all
select 19712,'019712','1988-113F','Kosmos-1985 SS 5','ESO',61362,'Reentered','1989-03-14 00:00:00','1989-01-24 00:00:00',94.38,482,496,73.59,'Intermediate' from dual union all
select 19713,'019713','1988-112A','Molniya-3','Molniya-3 No. 52',61361,'Reentered','2007-05-27 00:00:00','1989-01-22 00:00:00',717.78,479,39876,62.77,'Molniya' from dual union all
select 19714,'019714','1988-112B','11S510','11S510',61361,'Reentered','1989-01-02 00:00:00','1988-12-28 00:00:00',89.74,193,331,62.78,'Intermediate' from dual union all
select 19715,'019715','1988-112C','BOZ','BOZ',61361,'Reentered','1989-01-02 00:00:00','1988-12-28 00:00:00',90.04,175,378,62.81,'Intermediate' from dual union all
select 19716,'019716','1988-112D','Blok-ML','Blok-ML No. 330',61361,'Reentered','2006-10-10 00:00:00','1989-01-22 00:00:00',696.16,479,38804,62.8,'Molniya' from dual union all
select 19717,'019717','1988-107G','KDU part','KDU part',61356,'Reentered','1988-12-23 00:00:00','1988-12-22 00:00:00',88.62,191,222,62.81,'Intermediate' from dual union all
select 19718,'019718','1988-107H','KDU','KDU',61356,'Reentered','1988-12-28 00:00:00','1988-12-23 00:00:00',88.93,205,239,62.81,'Intermediate' from dual;
insert into SATELLITE
select 19719,'019719','1988-107J','KDU part','KDU part',61356,'Reentered','1988-12-24 00:00:00','1988-12-23 00:00:00',90.45,235,358,62.82,'Intermediate' from dual union all
select 1972,'001972','1965-082HE','deb Transtage 4','',11832,'Reentered','1967-02-02 00:00:00','1967-01-12 00:00:00',94.89,451,576,31.9,'Intermediate' from dual union all
select 19720,'019720','1988-113A','Kosmos-1985','Kol''tso',61362,'Reentered','1992-05-04 00:00:00','1989-01-22 00:00:00',95.13,522,528,73.56,'Intermediate' from dual union all
select 19721,'019721','1988-113B','Kosmos-1985 SS 1','ESO',61362,'Reentered','1989-03-15 00:00:00','1989-01-22 00:00:00',95.14,523,527,73.56,'Intermediate' from dual union all
select 19722,'019722','1988-112E','deb Blok-I','',61361,'Reentered','1989-01-03 00:00:00','1988-12-25 00:00:00',90.43,185,407,62.81,'Intermediate' from dual union all
select 19723,'019723','1988-112F','deb Blok-I','',61361,'Reentered','1988-12-30 00:00:00','1988-12-25 00:00:00',90.43,190,401,62.8,'Intermediate' from dual union all
select 19724,'019724','1988-112G','deb Blok-I','',61361,'Reentered','1988-12-24 00:00:00','1988-12-24 00:00:00',89.45,170,325,62.8,'Intermediate' from dual union all
select 19725,'019725','1988-112H','deb Blok-I','',61361,'Reentered','1989-01-03 00:00:00','1988-12-25 00:00:00',90.78,200,426,62.81,'Intermediate' from dual union all
select 19726,'019726','1988-113C','Kosmos-1985 SS 2','ESO',61362,'Reentered','1989-03-02 00:00:00','1989-02-01 00:00:00',93.65,436,471,73.53,'Intermediate' from dual union all
select 19727,'019727','1988-113D','Kosmos-1985 SS 3','ESO',61362,'Reentered','1989-03-03 00:00:00','1989-01-22 00:00:00',94,453,488,73.59,'Intermediate' from dual union all
select 19728,'019728','1988-114A','Progress-39','Progress 7K-TG No. 147',61363,'Deorbited','1989-02-07 00:00:00','1989-01-02 00:00:00',91.3,325,352,51.63,'Intermediate' from dual union all
select 19729,'019729','1988-114B','11S510','11S510',61363,'Reentered','1988-12-27 00:00:00','1988-12-26 00:00:00',87.44,142,154,51.62,'Intermediate' from dual union all
select 1973,'001973','1965-082HF','deb Transtage 4','',11832,'Reentered','1981-05-05 00:00:00','1981-05-05 00:00:00',91.6,348,358,31.91,'Intermediate' from dual union all
select 19730,'019730','1988-115A','Molniya-1','Molniya-1 No. 63',61364,'Reentered','1998-07-31 00:00:00','1989-01-28 00:00:00',717.72,616,39736,62.84,'Molniya' from dual union all
select 19731,'019731','1988-115B','11S510','11S510',61364,'Reentered','1989-01-21 00:00:00','1989-01-04 00:00:00',91.68,206,508,62.81,'Intermediate' from dual union all
select 19732,'019732','1988-115C','BOZ','BOZ',61364,'Reentered','1989-01-22 00:00:00','1989-01-04 00:00:00',92.22,194,573,62.83,'Intermediate' from dual union all
select 19733,'019733','1988-115D','Blok-ML','Blok-ML No. 182',61364,'Reentered','2001-04-02 00:00:00','1989-01-31 00:00:00',696.98,618,38706,62.87,'Molniya' from dual union all
select 19734,'019734','1988-116A','Kosmos-1986','Kometa No. 10',61365,'Landed','1989-02-11 00:00:00','1989-01-05 00:00:00',89.26,211,265,64.78,'Intermediate' from dual union all
select 19735,'019735','1988-116B','11S510','11S510',61365,'Reentered','1989-01-02 00:00:00','1989-01-02 00:00:00',87.28,129,151,64.77,'Intermediate' from dual union all
select 19736,'019736','1986-017BG','Garbage bag','',59210,'Reentered','1989-02-22 00:00:00','1989-01-02 00:00:00',91.2,321,345,51.62,'Intermediate' from dual union all
select 19737,'019737','1986-017BH','Garbage bag','',59210,'Reentered','1989-02-01 00:00:00','1989-01-02 00:00:00',91.11,321,337,51.62,'Intermediate' from dual union all
select 19738,'019738','1986-017BJ','Garbage bag','',59210,'Reentered','1989-01-22 00:00:00','1989-01-02 00:00:00',91.02,317,333,51.62,'Intermediate' from dual union all
select 19739,'019739','1986-017BK','Garbage bag','',59210,'Reentered','1989-01-17 00:00:00','1989-01-02 00:00:00',90.94,310,332,51.62,'Intermediate' from dual union all
select 1974,'001974','1965-082HG','deb Transtage 4','',11832,'Reentered','1981-03-05 00:00:00','1981-03-03 00:00:00',90.45,290,304,31.73,'Intermediate' from dual union all
select 19740,'019740','1986-017BL','Garbage bag','',59210,'Reentered','1989-03-03 00:00:00','1989-01-01 00:00:00',91.21,319,349,51.62,'Intermediate' from dual union all
select 19741,'019741','1986-017BM','Garbage bag','',59210,'Reentered','1989-01-25 00:00:00','1989-01-02 00:00:00',91.06,316,338,51.62,'Intermediate' from dual union all
select 19742,'019742','1988-115E','deb Blok-I','',61364,'Reentered','1989-01-16 00:00:00','1989-01-04 00:00:00',92.14,201,558,62.81,'Intermediate' from dual union all
select 19743,'019743','1986-017BN','Garbage bag','',59210,'Reentered','1989-03-22 00:00:00','1989-01-02 00:00:00',91.22,325,344,51.62,'Intermediate' from dual union all
select 19744,'019744','1986-017BP','Garbage bag','',59210,'Reentered','1989-03-05 00:00:00','1989-01-02 00:00:00',91.19,321,345,51.62,'Intermediate' from dual union all
select 19745,'019745','1986-017BQ','Garbage bag','',59210,'Reentered','1989-03-05 00:00:00','1989-01-02 00:00:00',91.19,323,343,51.62,'Intermediate' from dual union all
select 19746,'019746','1986-017BR','Garbage bag','',59210,'Reentered','1989-03-05 00:00:00','1989-01-02 00:00:00',91.19,318,348,51.62,'Intermediate' from dual union all
select 19747,'019747','1986-017BS','Garbage bag','',59210,'Reentered','1989-02-11 00:00:00','1989-01-02 00:00:00',91.15,321,341,51.62,'Intermediate' from dual union all
select 19748,'019748','1988-115F','deb Blok-I','',61364,'Reentered','1989-01-25 00:00:00','1989-01-04 00:00:00',92.64,220,588,62.81,'Intermediate' from dual union all
select 19749,'019749','1989-001A','Kosmos-1987','Uragan No. 27L',62278,'In Earth orbit','','1989-02-09 00:00:00',675.69,19116,19142,64.86,'Medium' from dual union all
select 1975,'001975','1965-082HH','deb Transtage 4','',11832,'Reentered','1981-05-23 00:00:00','1981-05-22 00:00:00',90.26,278,297,31.97,'Intermediate' from dual union all
select 19750,'019750','1989-001B','Kosmos-1988','Uragan No. 45L',62278,'In Earth orbit','','1989-02-09 00:00:00',675.71,19118,19141,64.88,'Medium' from dual union all
select 19751,'019751','1989-001C','Kosmos-1989','Etalon PKA No. 1L',62278,'In Earth orbit','','1989-02-09 00:00:00',675.55,19102,19149,64.87,'Medium' from dual union all
select 19752,'019752','1989-001D','8S812','8S812',62278,'Reentered','1989-01-10 00:00:00','1989-01-10 00:00:00',86.87,112,127,64.82,'Intermediate' from dual union all
select 19753,'019753','1989-001E','Etalon adapter','Etalon adapter',62278,'In Earth orbit','','1989-02-09 00:00:00',675.51,19104,19145,64.85,'Medium' from dual union all
select 19754,'019754','1989-001F','Blok DM2','11S861 No. 42L',62278,'In Earth orbit','','1989-02-09 00:00:00',674.7,19067,19141,64.87,'Medium' from dual union all
select 19755,'019755','1989-001G','SOZ','SOZ',62278,'In Earth orbit','','1989-02-10 00:00:00',340.06,407,19114,64.8,'Highly Ellip' from dual union all
select 19756,'019756','1989-002A','Kosmos-1990','Resurs-F2 17F42 No. 3',62279,'Landed','1989-02-11 00:00:00','1989-01-20 00:00:00',89.8,261,269,82.57,'Intermediate' from dual union all
select 19757,'019757','1989-002B','11S510','11S510',62279,'Reentered','1989-01-14 00:00:00','1989-01-13 00:00:00',88.3,172,209,82.57,'Intermediate' from dual union all
select 19758,'019758','1989-003A','Kosmos-1991','Oblik',62280,'Landed','1989-02-01 00:00:00','1989-01-20 00:00:00',92.16,347,414,69.99,'Intermediate' from dual union all
select 19759,'019759','1989-003B','11S510','11S510',62280,'Reentered','1989-01-27 00:00:00','1989-01-23 00:00:00',89.53,191,311,69.98,'Intermediate' from dual union all
select 1976,'001976','1965-082HJ','deb Transtage 4','',11832,'Reentered','1982-04-14 00:00:00','1966-01-30 00:00:00',99.51,703,768,31.89,'Intermediate' from dual union all
select 19760,'019760','1989-003C','Sensor cover','Sensor cover',62280,'Reentered','1989-01-20 00:00:00','1989-01-19 00:00:00',90.02,197,354,70.01,'Intermediate' from dual union all
select 19761,'019761','1989-003D','Sensor cover','Sensor cover',62280,'Reentered','1989-01-19 00:00:00','1989-01-18 00:00:00',90.07,199,357,69.96,'Intermediate' from dual union all
select 19762,'019762','1989-003E','Sensor cover','Sensor cover',62280,'Reentered','1989-01-20 00:00:00','1989-01-19 00:00:00',89.12,184,278,70,'Intermediate' from dual union all
select 19763,'019763','1988-113G','Kosmos-1985 SS 6','ESO',61362,'Reentered','1989-03-15 00:00:00','1989-01-27 00:00:00',94.99,516,521,73.56,'Intermediate' from dual union all
select 19764,'019764','1988-113H','S5M','S5M',61362,'Reentered','2002-08-07 00:00:00','1989-01-26 00:00:00',95.1,520,527,73.58,'Intermediate' from dual union all
select 19765,'019765','1989-004A','Gorizont','Gorizont No. 29L',62281,'In Earth orbit','','1989-02-25 00:00:00',1436.11,35774,35800,1.4,'Stationary' from dual union all
select 19766,'019766','1989-004B','8S812','8S812',62281,'Reentered','1989-01-28 00:00:00','1989-01-27 00:00:00',88.17,179,189,51.63,'Intermediate' from dual union all
select 19767,'019767','1989-004C','Perekhodnik','Sredniy perekhodnik',62281,'Reentered','1989-01-26 00:00:00','1989-01-26 00:00:00',88.43,188,205,51.61,'Intermediate' from dual union all
select 19768,'019768','1989-004D','SOZ','SOZ',62281,'Reentered','1991-07-31 00:00:00','1989-03-03 00:00:00',647.32,277,36543,47.25,'GEO Transfer' from dual union all
select 19769,'019769','1989-005A','Kosmos-1992','Strela-2M',62282,'In Earth orbit','','1989-02-25 00:00:00',100.64,771,807,74.05,'Intermediate' from dual union all
select 1977,'001977','1965-082HK','deb Transtage 4','',11832,'Reentered','1982-07-14 00:00:00','1982-07-12 00:00:00',90.28,278,299,31.26,'Intermediate' from dual union all
select 19770,'019770','1989-005B','S3M','S3M',62282,'In Earth orbit','','1989-02-25 00:00:00',100.49,775,788,74.05,'Intermediate' from dual union all
select 19771,'019771','1989-004E','SOZ','SOZ',62281,'Reentered','1999-02-15 00:00:00','1989-02-26 00:00:00',648.25,333,36534,47.32,'GEO Transfer' from dual union all
select 19772,'019772','1989-006A','INTELSAT 515','INTELSAT VA F15',62283,'In Earth orbit','','1989-02-27 00:00:00',1432.08,35696,35720,.2,'Drift GEO' from dual union all
select 19773,'019773','1989-006B','Ariane H10 V28','Ariane H10 S/N T26',62283,'In Earth orbit','','1989-02-26 00:00:00',638.57,515,35857,8.33,'GEO Transfer' from dual union all
select 19774,'019774','1989-007A','Kosmos-1993','Kobal''t',62284,'Landed','1989-03-27 00:00:00','1989-02-04 00:00:00',89.24,163,311,64.74,'Intermediate' from dual union all
select 19775,'019775','1989-007B','11S510','11S510',62284,'Reentered','1989-02-01 00:00:00','1989-01-30 00:00:00',89.17,164,303,64.76,'Intermediate' from dual union all
select 19776,'019776','1989-004F','Blok DM2 No. 20L','11S861 No. 20L',62281,'In Earth orbit','','1989-02-25 00:00:00',1469.42,36332,36540,1.42,'Drift GEO' from dual union all
select 19777,'019777','1988-095F','Blok DM2','11S861 No. 40L',61344,'In Earth orbit','','1989-01-25 00:00:00',1470.32,36400,36508,1.27,'Drift GEO' from dual union all
select 19778,'019778','1989-003F','KDU part','KDU part',62280,'Reentered','1991-07-31 00:00:00','1989-02-17 00:00:00',92.91,345,490,69.97,'Intermediate' from dual union all
select 19779,'019779','1989-003G','Blenda','Sunshade',62280,'Reentered','1989-02-11 00:00:00','1989-02-02 00:00:00',94.37,373,604,69.99,'Intermediate' from dual union all
select 1978,'001978','1965-082HL','deb Transtage 4','',11832,'Reentered','1980-03-09 00:00:00','1980-03-03 00:00:00',91.74,349,371,32.47,'Intermediate' from dual union all
select 19780,'019780','1989-003H','KDU','KDU',62280,'Reentered','1989-06-18 00:00:00','1989-02-18 00:00:00',91.95,341,400,69.98,'Intermediate' from dual union all
select 19781,'019781','1989-003J','KDU part','KDU part',62280,'Reentered','1989-03-07 00:00:00','1989-02-02 00:00:00',91.96,333,409,69.98,'Intermediate' from dual union all
select 19782,'019782','1989-003K','KDU part','KDU part',62280,'Reentered','1989-02-18 00:00:00','1989-02-02 00:00:00',92.09,344,410,69.98,'Intermediate' from dual union all
select 19783,'019783','1989-008A','Progress-40','Progress 7K-TG No. 148',62285,'Deorbited','1989-03-05 00:00:00','1989-02-17 00:00:00',91.6,345,362,51.62,'Intermediate' from dual union all
select 19784,'019784','1989-008B','11S510','11S510',62285,'Reentered','1989-02-12 00:00:00','1989-02-11 00:00:00',88.29,172,208,51.63,'Intermediate' from dual union all
select 19785,'019785','1989-009A','Kosmos-1994','Strela-3',62286,'In Earth orbit','','1989-03-13 00:00:00',113.91,1396,1416,82.62,'Intermediate' from dual union all
select 19786,'019786','1989-009B','Kosmos-1995','Strela-3',62286,'In Earth orbit','','1989-03-13 00:00:00',114.11,1413,1416,82.62,'Intermediate' from dual union all
select 19787,'019787','1989-009C','Kosmos-1996','Strela-3',62286,'In Earth orbit','','1989-03-13 00:00:00',114.04,1408,1416,82.62,'Intermediate' from dual union all
select 19788,'019788','1989-009D','Kosmos-1997','Strela-3',62286,'In Earth orbit','','1989-03-13 00:00:00',113.96,1401,1416,82.62,'Intermediate' from dual union all
select 19789,'019789','1989-009E','Kosmos-1998','Strela-3',62286,'In Earth orbit','','1989-03-13 00:00:00',113.85,1391,1416,82.61,'Intermediate' from dual union all
select 1979,'001979','1965-082HM','deb Transtage 4','',11832,'Reentered','1967-10-07 00:00:00','1967-09-20 00:00:00',92.16,352,409,31.94,'Intermediate' from dual union all
select 19790,'019790','1989-009F','Kosmos-1999','Strela-3',62286,'In Earth orbit','','1989-03-13 00:00:00',113.79,1385,1416,82.62,'Intermediate' from dual union all
select 19791,'019791','1989-009G','S5M','S5M',62286,'In Earth orbit','','1989-03-13 00:00:00',114.74,1416,1472,82.61,'Intermediate' from dual union all
select 19792,'019792','1989-010A','Kosmos-2000','Oblik No. 386 (Priroda)',62287,'Landed','1989-03-02 00:00:00','1989-02-17 00:00:00',91.9,342,394,82.36,'Intermediate' from dual union all
select 19793,'019793','1989-010B','11S510','11S510',62287,'Reentered','1989-02-12 00:00:00','1989-02-11 00:00:00',88.54,175,230,82.34,'Intermediate' from dual union all
select 19794,'019794','1989-002C','KDU','KDU',62279,'Reentered','1989-02-12 00:00:00','1989-02-11 00:00:00',90.78,184,442,82.56,'Intermediate' from dual union all
select 19795,'019795','1989-002D','Solnechnaya Batareya','Solar panel module',62279,'Reentered','1989-02-11 00:00:00','1989-02-11 00:00:00',88.73,165,258,82.56,'Intermediate' from dual union all
select 19796,'019796','1989-011A','Kosmos-2001','Oko',62288,'Reentered','2008-09-22 00:00:00','1989-03-18 00:00:00',717.86,670,39689,62.84,'Molniya' from dual union all
select 19797,'019797','1989-011B','11S510','11S510',62288,'Reentered','1989-03-11 00:00:00','1989-02-21 00:00:00',91.93,205,534,62.78,'Intermediate' from dual union all
select 19798,'019798','1989-011C','BOZ','BOZ',62288,'Reentered','1989-03-03 00:00:00','1989-02-21 00:00:00',91.49,182,513,62.84,'Intermediate' from dual union all
select 19799,'019799','1989-011D','Blok-2BL','Blok-2BL No. 81',62288,'In Earth orbit','','1989-03-16 00:00:00',705.79,667,39095,62.84,'Molniya' from dual union all
select 198,'000198','61 A Epsilon 2','deb Discoverer XXXIV','',7252,'Reentered','1961-11-30 00:00:00','1961-11-12 00:00:00',94.99,232,805,82.46,'Intermediate' from dual union all
select 1980,'001980','1965-082HN','deb Transtage 4','',11832,'Reentered','1969-05-25 00:00:00','1966-01-30 00:00:00',96.66,463,735,32.4,'Intermediate' from dual union all
select 19800,'019800','1989-012A','Kosmos-2002','Duga-K',62289,'Reentered','1989-10-15 00:00:00','1989-03-16 00:00:00',108.62,185,2141,65.83,'Intermediate' from dual union all
select 19801,'019801','1989-012B','S3M','S3M',62289,'Reentered','1989-07-27 00:00:00','1989-03-16 00:00:00',107.17,179,2013,65.83,'Intermediate' from dual union all
select 19802,'019802','1989-013A','GPS II-1','Navstar GPS SVN 14',62290,'In Earth orbit','','1989-03-16 00:00:00',713.2,19858,20271,55.12,'Medium' from dual union all
select 19803,'019803','1989-013B','Delta 184','Delta SSPS AJ10-118K',62290,'Reentered','1989-03-13 00:00:00','1989-02-21 00:00:00',94.2,182,778,30.32,'Intermediate' from dual union all
select 19804,'019804','1989-013C','Star 48B','Star 48B S/N 10051-18',62290,'Reentered','1989-12-06 00:00:00','1989-03-19 00:00:00',343.08,142,19570,37.32,'Highly Ellip' from dual union all
select 19805,'019805','1989-012C','Kosmos-2002 SS 1','ESO',62289,'Reentered','1989-03-03 00:00:00','1989-02-22 00:00:00',102.93,177,1618,65.8,'Intermediate' from dual union all
select 19806,'019806','1989-012D','Kosmos-2002 SS 2','ESO',62289,'Reentered','1989-03-10 00:00:00','1989-02-21 00:00:00',105.57,182,1861,65.87,'Intermediate' from dual union all
select 19807,'019807','1989-014A','Molniya-1','Molniya-1 11F658 No. 84',62291,'In Earth orbit','','1989-03-18 00:00:00',717.74,469,39884,62.98,'Molniya' from dual union all
select 19808,'019808','1989-014C','BOZ','BOZ',62291,'Reentered','1989-03-01 00:00:00','1989-02-17 00:00:00',91.2,196,471,62.97,'Intermediate' from dual;
insert into SATELLITE
select 19809,'019809','1989-014B','11S510','11S510',62291,'Reentered','1989-03-09 00:00:00','1989-02-22 00:00:00',90.9,222,416,62.95,'Intermediate' from dual union all
select 1981,'001981','1965-082HP','deb Transtage 4','',11832,'Reentered','2014-02-15 00:00:00','1992-01-30 00:00:00',97.34,619,644,32.43,'Intermediate' from dual union all
select 19810,'019810','1989-014D','Blok-ML','Blok-ML No. 171',62291,'In Earth orbit','','1989-03-18 00:00:00',694.55,462,38741,62.97,'Molniya' from dual union all
select 19811,'019811','1988-102G','radar error','(ghost of 102E)',61351,'Spurious','1990-10-01 00:00:00','1990-09-09 00:00:00',104.96,842,1144,71.02,'Intermediate' from dual union all
select 19812,'019812','1986-019VE','deb Ariane V16','',59212,'Reentered','1991-02-11 00:00:00','1987-09-30 00:00:00',101.2,799,832,98.58,'Sun-Synch' from dual union all
select 19813,'019813','1988-102H','radar error','(ghost of 102D)',61351,'Spurious','1990-10-01 00:00:00','1990-09-22 00:00:00',105.18,841,1165,70.98,'Intermediate' from dual union all
select 19814,'019814','1963-014EN','deb West Ford','RCS 0.02',9245,'Reentered','2002-10-29 00:00:00','1988-05-23 00:00:00',165.24,2071,5123,86.77,'Medium' from dual union all
select 19815,'019815','1987-020CJ','deb Kosmos-1823','',60110,'Reentered','1989-08-06 00:00:00','1988-04-15 00:00:00',114.89,1131,1769,73.66,'Intermediate' from dual union all
select 19816,'019816','1986-017BT','Garbage bag','',59210,'Reentered','1989-03-06 00:00:00','1989-01-31 00:00:00',90.59,294,313,51.62,'Intermediate' from dual union all
select 19817,'019817','1970-089DF','deb Kosmos-374','',21566,'Reentered','2003-03-03 00:00:00','1984-12-26 00:00:00',105.34,534,1486,62.27,'Intermediate' from dual union all
select 19818,'019818','1989-015A','Kosmos-2003','Oblik No. 391',62292,'Landed','1989-03-03 00:00:00','1989-02-19 00:00:00',89.47,231,266,62.81,'Intermediate' from dual union all
select 19819,'019819','1989-015B','11S510','11S510',62292,'Reentered','1989-02-23 00:00:00','1989-02-19 00:00:00',89.25,232,243,62.8,'Intermediate' from dual union all
select 1982,'001982','1966-008A','ESSA 1','Tiros OT3',13415,'In Earth orbit','','1966-10-06 00:00:00',100.27,705,838,97.91,'Sun-Synch' from dual union all
select 19820,'019820','1989-015C','Sensor cover','Sensor cover',62292,'Reentered','1989-02-18 00:00:00','1989-02-17 00:00:00',89.1,207,252,62.78,'Intermediate' from dual union all
select 19821,'019821','1989-012E','Kosmos-2002 SS 3','ESO',62289,'Reentered','1989-03-11 00:00:00','1989-02-21 00:00:00',107.1,182,2003,65.86,'Intermediate' from dual union all
select 19822,'019822','1989-016A','Akebono','EXOS-D',62293,'In Earth orbit','','1989-03-23 00:00:00',210.84,279,10433,75.09,'Medium' from dual union all
select 19823,'019823','1989-016B','M-3B','M-3B',62293,'Reentered','1989-04-04 00:00:00','1989-03-03 00:00:00',92.14,262,497,74.57,'Intermediate' from dual union all
select 19824,'019824','1989-016C','KM-D','KM-D',62293,'Reentered','2005-12-17 00:00:00','1989-03-23 00:00:00',210.34,278,10397,75.09,'Medium' from dual union all
select 19825,'019825','1989-012F','Kosmos-2002 SS 4','ESO',62289,'Reentered','1989-03-08 00:00:00','1989-02-21 00:00:00',107.13,182,2006,65.78,'Intermediate' from dual union all
select 19826,'019826','1989-017A','Kosmos-2004','Parus',62294,'In Earth orbit','','1989-03-26 00:00:00',105.01,973,1017,82.95,'Intermediate' from dual union all
select 19827,'019827','1989-017B','S3M','S3M',62294,'In Earth orbit','','1989-03-24 00:00:00',104.9,972,1008,82.95,'Intermediate' from dual union all
select 19828,'019828','1963-014EP','deb West Ford','RCS 0.12',9245,'Reentered','1998-06-05 00:00:00','1985-07-28 00:00:00',165.44,2042,5168,86.84,'Medium' from dual union all
select 19829,'019829','1971-015DT','deb Kosmos-397','',24833,'Reentered','1991-02-27 00:00:00','1985-02-01 00:00:00',105.92,518,1557,65.8,'Intermediate' from dual union all
select 1983,'001983','1966-008B','Altair 2 X-258C4 RH-88','X-258C4 RH-88',13415,'In Earth orbit','','1966-10-15 00:00:00',100.53,706,862,97.9,'Sun-Synch' from dual union all
select 19830,'019830','1978-100AN','deb S5M','',45295,'Reentered','2000-12-20 00:00:00','1988-05-16 00:00:00',118.61,1553,1684,82.56,'Intermediate' from dual union all
select 19831,'019831','1989-005C','deb Kosmos-1992','Strela-2M debris 1',62282,'In Earth orbit','','1989-02-27 00:00:00',100.9,785,818,74.08,'Intermediate' from dual union all
select 19832,'019832','1989-012G','Kosmos-2002 SS 5','ESO',62289,'Reentered','1989-03-10 00:00:00','1989-02-21 00:00:00',106.16,182,1916,65.72,'Intermediate' from dual union all
select 19833,'019833','1987-020CK','deb Kosmos-1823','',60110,'Reentered','1989-12-21 00:00:00','1988-02-22 00:00:00',115.91,1094,1900,73.37,'Intermediate' from dual union all
select 19834,'019834','1972-058JM','deb Delta 89','',27990,'Reentered','1989-08-23 00:00:00','1989-02-23 00:00:00',99.25,709,737,98.47,'Sun-Synch' from dual union all
select 19835,'019835','1982-115AJ','deb Blok-L','',54424,'Reentered','1989-03-12 00:00:00','1989-01-28 00:00:00',92.22,228,539,62.82,'Intermediate' from dual union all
select 19836,'019836','1987-020CL','deb Kosmos-1823','',60110,'Reentered','1989-10-26 00:00:00','1987-12-22 00:00:00',115.79,1470,1513,73.59,'Intermediate' from dual union all
select 19837,'019837','1987-020CM','deb Kosmos-1823','',60110,'Reentered','1989-07-11 00:00:00','1988-03-27 00:00:00',113.88,962,1847,73.27,'Intermediate' from dual union all
select 19838,'019838','1965-048L','deb NNS O-4','was [UNID131] RCS 0.01',11798,'Reentered','1991-07-01 00:00:00','1989-01-07 00:00:00',105.83,1015,1052,90.03,'Polar' from dual union all
select 19839,'019839','1987-020CN','deb Kosmos-1823','',60110,'Reentered','1989-04-01 00:00:00','1989-01-30 00:00:00',108.42,777,1530,73.42,'Intermediate' from dual union all
select 1984,'001984','1965-082HQ','deb Transtage 4','',11832,'Reentered','1974-03-29 00:00:00','1966-02-05 00:00:00',99.56,709,766,32.05,'Intermediate' from dual union all
select 19840,'019840','1987-020CP','deb Kosmos-1823','',60110,'Reentered','1989-11-24 00:00:00','1988-02-22 00:00:00',116.03,1180,1825,74.02,'Intermediate' from dual union all
select 19841,'019841','1989-016D','KM-D nozzle extender','',62293,'Reentered','1989-02-28 00:00:00','1989-02-24 00:00:00',91.89,259,476,74.62,'Intermediate' from dual union all
select 19842,'019842','1989-016E','KM-D igniter','',62293,'Reentered','1989-02-27 00:00:00','1989-02-24 00:00:00',91.67,255,457,74.59,'Intermediate' from dual union all
select 19843,'019843','1989-016F','deb KM-D','',62293,'Reentered','1989-02-25 00:00:00','1989-02-24 00:00:00',90.23,235,337,74.56,'Intermediate' from dual union all
select 19844,'019844','1989-016G','deb KM-D','',62293,'Reentered','1989-02-25 00:00:00','1989-02-24 00:00:00',90.57,244,361,74.53,'Intermediate' from dual union all
select 19845,'019845','1989-016H','deb KM-D','',62293,'Reentered','1989-02-25 00:00:00','1989-02-25 00:00:00',89.93,188,354,74.72,'Intermediate' from dual union all
select 19846,'019846','1989-012H','Kosmos-2002 SS 6','ESO',62289,'Reentered','1989-03-09 00:00:00','1989-02-26 00:00:00',102.94,180,1615,65.92,'Intermediate' from dual union all
select 19847,'019847','1989-012J','Kosmos-2002 SS 7','ESO',62289,'Reentered','1989-03-11 00:00:00','1989-02-26 00:00:00',103.54,175,1677,65.71,'Intermediate' from dual union all
select 19848,'019848','1989-012K','Kosmos-2002 SS 8','ESO',62289,'Reentered','1989-03-11 00:00:00','1989-02-26 00:00:00',103.35,179,1654,65.92,'Intermediate' from dual union all
select 19849,'019849','1989-012L','Kosmos-2002 SS 9','ESO',62289,'Reentered','1989-03-06 00:00:00','1989-02-26 00:00:00',99.63,177,1304,65.76,'Intermediate' from dual union all
select 1985,'001985','1965-082HR','deb Transtage 4','',11832,'Reentered','1968-08-18 00:00:00','1968-08-14 00:00:00',91.47,347,347,31.87,'Intermediate' from dual union all
select 19850,'019850','1989-012M','Kosmos-2002 SS 10','ESO',62289,'Reentered','1989-03-10 00:00:00','1989-02-26 00:00:00',102.58,181,1581,65.77,'Intermediate' from dual union all
select 19851,'019851','1989-018A','Meteor-2','Meteor-2 No. 22',62295,'In Earth orbit','','1989-03-30 00:00:00',104.05,939,960,82.52,'Intermediate' from dual union all
select 19852,'019852','1989-018B','S5M','S5M',62295,'In Earth orbit','','1989-03-31 00:00:00',104.07,938,964,82.52,'Intermediate' from dual union all
select 19853,'019853','1981-053MH','deb Kosmos-1275','',52576,'In Earth orbit','','1985-03-26 00:00:00',106.8,964,1192,82.98,'Intermediate' from dual union all
select 19854,'019854','1987-020CQ','deb Kosmos-1823','',60110,'Reentered','1989-07-23 00:00:00','1988-02-29 00:00:00',117.77,1215,1947,73.43,'Intermediate' from dual union all
select 19855,'019855','1986-019VF','deb Ariane V16','',59212,'Reentered','1989-03-16 00:00:00','1988-10-09 00:00:00',100.31,743,803,98.82,'Sun-Synch' from dual union all
select 19856,'019856','1989-001H','SOZ','SOZ',62278,'In Earth orbit','','1989-02-16 00:00:00',340.06,407,19113,64.9,'Highly Ellip' from dual union all
select 19857,'019857','1985-056P','deb Ariane V14','',58279,'In Earth orbit','','1989-01-28 00:00:00',633.7,671,35451,6.44,'GEO Transfer' from dual union all
select 19858,'019858','1976-077FL','deb Delta 126','',39029,'Reentered','2011-05-02 00:00:00','1991-10-06 00:00:00',123.77,991,2707,102.78,'Medium' from dual union all
select 19859,'019859','1963-014EQ','deb West Ford cluster','RCS 0.04',9245,'Reentered','1996-12-08 00:00:00','1989-04-30 00:00:00',165.39,2165,5041,86.84,'Medium' from dual union all
select 1986,'001986','1965-082HS','deb Transtage 4','',11832,'Reentered','1980-01-08 00:00:00','1966-02-05 00:00:00',101.47,742,915,31.91,'Intermediate' from dual union all
select 19860,'019860','1963-014ER','deb West Ford','RCS 0.11',9245,'Reentered','1998-06-07 00:00:00','1988-02-22 00:00:00',164.66,1759,5388,86.31,'Medium' from dual union all
select 19861,'019861','1987-020CR','deb Kosmos-1823','',60110,'Reentered','1989-09-26 00:00:00','1989-02-28 00:00:00',109.72,1006,1422,73.5,'Intermediate' from dual union all
select 19862,'019862','1989-019A','Kosmos-2005','Kobal''t No. 522',62296,'Landed','1989-04-25 00:00:00','1989-03-09 00:00:00',89.69,170,349,62.8,'Intermediate' from dual union all
select 19863,'019863','1989-019B','11S510','11S510',62296,'Reentered','1989-03-08 00:00:00','1989-03-04 00:00:00',89.39,189,300,62.78,'Intermediate' from dual union all
select 19864,'019864','1989-010C','KDU','KDU',62287,'Reentered','1989-05-30 00:00:00','1989-03-12 00:00:00',91.7,336,380,82.36,'Intermediate' from dual union all
select 19865,'019865','1989-010D','KDU part','KDU part',62287,'Reentered','1989-03-29 00:00:00','1989-03-03 00:00:00',91.67,339,374,82.36,'Intermediate' from dual union all
select 19866,'019866','1989-010E','KDU part','KDU part',62287,'Reentered','1989-03-09 00:00:00','1989-03-02 00:00:00',94.11,376,574,82.36,'Intermediate' from dual union all
select 19867,'019867','1989-010F','Blenda','Sunshade',62287,'Reentered','1989-03-24 00:00:00','1989-03-03 00:00:00',93.86,372,555,82.36,'Intermediate' from dual union all
select 19868,'019868','1989-015D','KDU','KDU',62292,'Reentered','1989-03-08 00:00:00','1989-03-03 00:00:00',88.92,205,238,62.8,'Intermediate' from dual union all
select 19869,'019869','1989-015E','Blenda','Sunshade',62292,'Reentered','1989-03-04 00:00:00','1989-03-03 00:00:00',88.7,195,226,62.81,'Intermediate' from dual union all
select 1987,'001987','1965-082HT','deb Transtage 4','',11832,'Reentered','1969-09-04 00:00:00','1968-11-13 00:00:00',96.96,596,631,32.28,'Intermediate' from dual union all
select 19870,'019870','1989-015F','KDU part','KDU part',62292,'Reentered','1989-03-04 00:00:00','1989-03-03 00:00:00',88.57,192,215,62.81,'Intermediate' from dual union all
select 19871,'019871','1989-008C','Krab antenna','Krab',62285,'Reentered','1989-03-07 00:00:00','1989-03-03 00:00:00',91.6,344,363,51.63,'Intermediate' from dual union all
select 19872,'019872','1989-008D','Krab antenna','Krab',62285,'Reentered','1989-03-07 00:00:00','1989-03-03 00:00:00',91.7,348,368,51.61,'Intermediate' from dual union all
select 19873,'019873','1989-010G','KDU part','KDU part',62287,'Reentered','1989-03-15 00:00:00','1989-03-04 00:00:00',91.71,335,382,82.36,'Intermediate' from dual union all
select 19874,'019874','1989-020A','JCSAT 1','JCSAT 1',62297,'In Earth orbit','','1989-04-18 00:00:00',1436.11,35779,35795,.06,'Stationary' from dual union all
select 19875,'019875','1989-015G','KDU part','',62292,'Reentered','1989-03-05 00:00:00','1989-03-03 00:00:00',90.48,227,370,62.82,'Intermediate' from dual union all
select 19876,'019876','1989-020B','Meteosat 4','MOP 1',62297,'In Earth orbit','','1989-04-09 00:00:00',1435.93,35779,35788,1.15,'Stationary' from dual union all
select 19877,'019877','1989-020C','Ariane H10 V29','Ariane H10 S/N T403',62297,'Reentered','1992-04-21 00:00:00','1989-04-08 00:00:00',628.32,213,35632,7,'GEO Transfer' from dual union all
select 19878,'019878','1989-020D','Spelda 10 S/N UV2','Spelda 10',62297,'Reentered','1992-12-22 00:00:00','1989-04-13 00:00:00',620.39,242,35193,7.01,'GEO Transfer' from dual union all
select 19879,'019879','1968-091DM','deb Kosmos-249 [*}','',16818,'Reentered','1989-04-30 00:00:00','1989-02-14 00:00:00',115.64,255,2714,63.02,'Intermediate' from dual union all
select 1988,'001988','1965-082HU','deb Transtage 4','',11832,'Reentered','1970-11-14 00:00:00','1970-11-07 00:00:00',92.2,372,393,31.64,'Intermediate' from dual union all
select 19880,'019880','1963-014ES','deb West Ford','RCS 0.06',9245,'Reentered','1991-12-16 00:00:00','1989-02-16 00:00:00',164.67,1663,5485,86.19,'Medium' from dual union all
select 19881,'019881','1977-068J','deb Kosmos-931','[UNID132] RCS 0.07',42196,'Reentered','1993-03-01 00:00:00','1979-08-18 00:00:00',689.31,454,38487,60.22,'GEO Transfer' from dual union all
select 19882,'019882','1989-021A','Discovery (STS-29)','OV-103',62298,'Landed','1989-03-18 00:00:00','1989-03-15 00:00:00',90.86,301,333,28.46,'Intermediate' from dual union all
select 19883,'019883','1989-021B','TDRS 4','TDRS D',62298,'In Earth orbit','','1989-04-15 00:00:00',1435.86,35776,35788,.22,'Stationary' from dual union all
select 19884,'019884','1989-021C','IUS-9 SRM-1','Orbus 21',62298,'In Earth orbit','','1989-04-12 00:00:00',631.76,258,35764,26.27,'GEO Transfer' from dual union all
select 19885,'019885','1988-113J','Kosmos-1985 SS 7','ESO',61362,'Reentered','1989-05-04 00:00:00','1989-03-16 00:00:00',95.21,516,542,73.52,'Intermediate' from dual union all
select 19886,'019886','1988-113K','Kosmos-1985 SS 8','ESO',61362,'Reentered','1989-04-26 00:00:00','1989-03-16 00:00:00',94.95,515,518,73.54,'Intermediate' from dual union all
select 19887,'019887','1986-019VG','deb Ariane V16','',59212,'Reentered','1989-05-06 00:00:00','1989-03-02 00:00:00',98.59,679,704,98.47,'Sun-Synch' from dual union all
select 19888,'019888','1987-020CS','deb Kosmos-1823','',60110,'Reentered','1989-08-02 00:00:00','1988-04-28 00:00:00',113.62,937,1849,73.47,'Intermediate' from dual union all
select 19889,'019889','1989-016J','deb KM-D','',62293,'Reentered','1989-03-22 00:00:00','1989-03-13 00:00:00',91.05,241,411,74.52,'Intermediate' from dual union all
select 1989,'001989','1965-082HV','deb Transtage 4','',11832,'Reentered','1970-02-13 00:00:00','1966-02-04 00:00:00',98.14,582,758,31.82,'Intermediate' from dual union all
select 19890,'019890','1987-020CT','deb Kosmos-1823','',60110,'Reentered','1989-10-13 00:00:00','1989-01-31 00:00:00',114.45,861,2000,73.33,'Intermediate' from dual union all
select 19891,'019891','1963-014ET','deb West Ford','RCS 0.19',9245,'In Earth orbit','','1989-02-22 00:00:00',164.76,1950,5205,86.55,'Medium' from dual union all
select 19892,'019892','1986-019VH','deb Ariane V16','',59212,'Reentered','1989-07-22 00:00:00','1989-01-22 00:00:00',99.82,724,776,98.57,'Sun-Synch' from dual union all
select 19893,'019893','1989-022A','Kosmos-2006','Oblik No. 390',62299,'Landed','1989-03-30 00:00:00','1989-03-23 00:00:00',91.94,328,412,62.85,'Intermediate' from dual union all
select 19894,'019894','1989-022B','11S510','11S510',62299,'Reentered','1989-04-03 00:00:00','1989-03-23 00:00:00',90.02,224,326,62.83,'Intermediate' from dual union all
select 19895,'019895','1989-023A','Progress-41','Progress 7K-TG No. 149',62300,'Reentered','1989-04-25 00:00:00','1989-03-23 00:00:00',91.98,358,386,51.63,'Intermediate' from dual union all
select 19896,'019896','1989-023B','11S510','11S510',62300,'Reentered','1989-03-18 00:00:00','1989-03-17 00:00:00',88.7,185,235,51.64,'Intermediate' from dual union all
select 19897,'019897','1989-022C','Sensor cover','Sensor cover',62299,'Reentered','1989-03-20 00:00:00','1989-03-18 00:00:00',90.13,230,332,62.82,'Intermediate' from dual union all
select 19898,'019898','1989-022D','Sensor cover','Sensor cover',62299,'Reentered','1989-03-20 00:00:00','1989-03-18 00:00:00',89.45,211,283,62.87,'Intermediate' from dual union all
select 19899,'019899','1989-022E','Sensor cover','Sensor cover',62299,'Reentered','1989-03-18 00:00:00','1989-03-17 00:00:00',90.26,235,340,62.81,'Intermediate' from dual;
insert into SATELLITE
select 199,'000199','61 A Epsilon 3','deb Discoverer XXXIV','',7252,'Reentered','1961-12-09 00:00:00','1961-11-14 00:00:00',95.8,247,868,82.42,'Intermediate' from dual union all
select 1990,'001990','1965-082HW','deb Transtage 4','',11832,'Reentered','1969-01-09 00:00:00','1966-02-06 00:00:00',98.11,586,751,32.15,'Intermediate' from dual union all
select 19900,'019900','1989-024A','Kosmos-2007','Terilen',62301,'Deorbited','1989-09-22 00:00:00','1989-04-22 00:00:00',89.76,233,293,64.73,'Intermediate' from dual union all
select 19901,'019901','1989-024B','11S510','11S510',62301,'Reentered','1989-03-25 00:00:00','1989-03-24 00:00:00',88.83,178,256,64.76,'Intermediate' from dual union all
select 19902,'019902','1989-025A','Kosmos-2008','Strela-1M',62302,'In Earth orbit','','1989-04-24 00:00:00',114.49,1394,1471,74.02,'Intermediate' from dual union all
select 19903,'019903','1989-025B','Kosmos-2009','Strela-1M',62302,'In Earth orbit','','1989-04-24 00:00:00',114.66,1408,1471,74.01,'Intermediate' from dual union all
select 19904,'019904','1989-025C','Kosmos-2010','Strela-1M',62302,'In Earth orbit','','1989-04-24 00:00:00',114.82,1424,1471,74.02,'Intermediate' from dual union all
select 19905,'019905','1989-025D','Kosmos-2011','Strela-1M',62302,'In Earth orbit','','1989-04-24 00:00:00',115.01,1440,1471,74.01,'Intermediate' from dual union all
select 19906,'019906','1989-025E','Kosmos-2012','Strela-1M',62302,'In Earth orbit','','1989-04-24 00:00:00',115.18,1456,1472,74.01,'Intermediate' from dual union all
select 19907,'019907','1989-025F','Kosmos-2013','Strela-1M',62302,'In Earth orbit','','1989-04-24 00:00:00',115.37,1465,1480,74.02,'Intermediate' from dual union all
select 19908,'019908','1989-025G','Kosmos-2014','Strela-1M',62302,'In Earth orbit','','1989-04-23 00:00:00',115.55,1470,1491,74.01,'Intermediate' from dual union all
select 19909,'019909','1989-025H','Kosmos-2015','Strela-1M',62302,'In Earth orbit','','1989-04-24 00:00:00',115.75,1469,1510,74.02,'Intermediate' from dual union all
select 1991,'001991','1965-082HX','deb Transtage 4','',11832,'Reentered','1968-02-06 00:00:00','1968-01-23 00:00:00',91.21,334,334,31.52,'Intermediate' from dual union all
select 19910,'019910','1989-025J','S3M','S3M',62302,'In Earth orbit','','1989-04-23 00:00:00',117.73,1472,1686,74.01,'Intermediate' from dual union all
select 19911,'019911','1989-026A','Delta Star','Delta Star',62303,'Reentered','1992-06-23 00:00:00','1989-04-23 00:00:00',94.45,483,501,47.68,'Intermediate' from dual union all
select 19912,'019912','1988-113L','Kosmos-1985 SS 9','ESO',61362,'Reentered','1989-04-25 00:00:00','1989-03-27 00:00:00',94.32,480,492,73.55,'Intermediate' from dual union all
select 19913,'019913','1989-021D','IUS-9 SRM-2','Orbus 6E',62298,'In Earth orbit','','1989-04-12 00:00:00',1431.19,35574,35807,2.15,'Drift GEO' from dual union all
select 19914,'019914','1989-022F','PTDU cover','KDU part',62299,'Reentered','1989-04-17 00:00:00','1989-03-30 00:00:00',94.03,343,600,62.85,'Intermediate' from dual union all
select 19915,'019915','1989-022G','Blenda','Sunshade',62299,'Reentered','1989-03-31 00:00:00','1989-03-30 00:00:00',91.77,258,465,62.84,'Intermediate' from dual union all
select 19916,'019916','1989-022H','KDU','KDU',62299,'Reentered','1989-08-25 00:00:00','1989-04-15 00:00:00',91.74,322,397,62.84,'Intermediate' from dual union all
select 19917,'019917','1989-022J','KDU part','KDU part',62299,'Reentered','1989-04-26 00:00:00','1989-03-30 00:00:00',91.78,314,410,62.84,'Intermediate' from dual union all
select 19918,'019918','1989-022K','KDU part','KDU part',62299,'Reentered','1989-04-12 00:00:00','1989-03-31 00:00:00',91.81,325,402,62.84,'Intermediate' from dual union all
select 19919,'019919','1989-027A','Tele-X','Tele-X',62304,'In Earth orbit','','1989-05-04 00:00:00',1436.09,35774,35799,.12,'Stationary' from dual union all
select 1992,'001992','1965-082HY','deb Transtage 4','',11832,'Reentered','1980-11-12 00:00:00','1980-11-11 00:00:00',90.02,269,282,31.89,'Intermediate' from dual union all
select 19920,'019920','1989-027B','Ariane H10 V30','Ariane H10 S/N T27',62304,'In Earth orbit','','1989-05-03 00:00:00',618.98,150,35212,4.02,'GEO Transfer' from dual union all
select 19921,'019921','1989-028A','Kosmos-2016','Parus',62305,'In Earth orbit','','1989-05-04 00:00:00',104.79,957,1013,82.96,'Intermediate' from dual union all
select 19922,'019922','1989-028B','S3M','S3M',62305,'In Earth orbit','','1989-05-06 00:00:00',104.65,954,1002,82.95,'Intermediate' from dual union all
select 19923,'019923','1989-029A','Kosmos-2017','Oblik No. 374',62306,'Landed','1989-04-19 00:00:00','1989-04-08 00:00:00',89.78,230,298,62.81,'Intermediate' from dual union all
select 19924,'019924','1989-029B','11S510','11S510',62306,'Reentered','1989-04-16 00:00:00','1989-04-08 00:00:00',89.33,237,246,62.79,'Intermediate' from dual union all
select 19925,'019925','1989-029C','Sensor cover','Sensor cover',62306,'Reentered','1989-04-07 00:00:00','1989-04-07 00:00:00',88.81,203,229,62.77,'Intermediate' from dual union all
select 19926,'019926','1989-029D','Sensor cover','Sensor cover',62306,'Reentered','1989-04-07 00:00:00','1989-04-07 00:00:00',89.44,235,258,62.83,'Intermediate' from dual union all
select 19927,'019927','1989-029E','Sensor cover','Sensor cover',62306,'Reentered','1989-04-08 00:00:00','1989-04-07 00:00:00',89.56,237,269,62.77,'Intermediate' from dual union all
select 19928,'019928','1989-030A','Raduga','Gran'' No. 33L',62307,'In Earth orbit','','1989-05-14 00:00:00',1436.11,35778,35796,1.37,'Stationary' from dual union all
select 19929,'019929','1989-030B','8S812','8S812',62307,'Reentered','1989-04-16 00:00:00','1989-04-15 00:00:00',88.09,176,184,51.63,'Intermediate' from dual union all
select 1993,'001993','1965-082HZ','deb Transtage 4','',11832,'Reentered','1980-03-30 00:00:00','1966-02-06 00:00:00',98.89,627,785,31.95,'Intermediate' from dual union all
select 19930,'019930','1989-030C','Perekhodnik','Sredniy perekhodnik',62307,'Reentered','1989-04-14 00:00:00','1989-04-14 00:00:00',88.25,178,197,51.62,'Intermediate' from dual union all
select 19931,'019931','1989-030D','Blok DM2','11S861 No. 22L',62307,'In Earth orbit','','1989-05-14 00:00:00',1470.45,36380,36533,1.4,'Drift GEO' from dual union all
select 19932,'019932','1989-030E','SOZ','SOZ',62307,'Reentered','1990-02-06 00:00:00','1989-05-14 00:00:00',648.66,314,36574,47.33,'GEO Transfer' from dual union all
select 19933,'019933','1989-030F','SOZ','SOZ',62307,'Reentered','1989-11-18 00:00:00','1989-05-18 00:00:00',646.99,253,36550,47.26,'GEO Transfer' from dual union all
select 19934,'019934','1989-029F','Blenda','Sunshade',62306,'Reentered','1989-04-22 00:00:00','1989-04-19 00:00:00',88.67,190,228,62.81,'Intermediate' from dual union all
select 19935,'019935','1989-029G','KDU','KDU',62306,'Reentered','1989-04-27 00:00:00','1989-04-19 00:00:00',88.32,167,216,62.8,'Intermediate' from dual union all
select 19936,'019936','1989-029H','KDU part','KDU part',62306,'Reentered','1989-04-19 00:00:00','1989-04-19 00:00:00',88.32,176,207,62.8,'Intermediate' from dual union all
select 19937,'019937','1989-029J','KDU part','KDU part',62306,'Reentered','1989-04-21 00:00:00','1989-04-20 00:00:00',89.98,210,336,62.8,'Intermediate' from dual union all
select 19938,'019938','1989-031A','Kosmos-2018','Kobal''t No. 875',62308,'Landed','1989-06-19 00:00:00','1989-05-20 00:00:00',88.98,166,283,62.8,'Intermediate' from dual union all
select 19939,'019939','1989-031B','11S510','11S510',62308,'Reentered','1989-04-24 00:00:00','1989-04-22 00:00:00',89.16,178,288,62.79,'Intermediate' from dual union all
select 1994,'001994','1963-047J','Centaur AC-2 part','Insulation blanket part',9278,'Reentered','1989-07-30 00:00:00','1965-11-28 00:00:00',108.8,605,1738,30.54,'Intermediate' from dual union all
select 19940,'019940','1989-023C','Progress-41 part','Progress part',62300,'Reentered','1989-05-03 00:00:00','1989-04-21 00:00:00',90.48,190,407,51.63,'Intermediate' from dual union all
select 19941,'019941','1989-032A','Foton','Foton 34KS No. 5L',62309,'Landed','1989-05-11 00:00:00','1989-05-03 00:00:00',90.34,214,368,62.82,'Intermediate' from dual union all
select 19942,'019942','1989-032B','11S510','11S510',62309,'Reentered','1989-05-08 00:00:00','1989-04-29 00:00:00',90.11,208,352,62.81,'Intermediate' from dual union all
select 19943,'019943','1988-113M','Kosmos-1985 SS 10','ESO',61362,'Reentered','1989-06-15 00:00:00','1989-04-27 00:00:00',94.91,509,520,73.55,'Intermediate' from dual union all
select 19944,'019944','1988-113N','Kosmos-1985 SS 11','ESO',61362,'Reentered','1989-06-16 00:00:00','1989-04-27 00:00:00',94.9,510,518,73.55,'Intermediate' from dual union all
select 19945,'019945','1989-005D','deb Kosmos-1992','Strela-2M debris 2 (high',62282,'In Earth orbit','','1989-03-14 00:00:00',101.11,789,834,74.07,'Intermediate' from dual union all
select 19946,'019946','1963-014EU','deb West Ford','RCS 0.21',9245,'In Earth orbit','','1988-04-03 00:00:00',165.22,2024,5168,86.59,'Medium' from dual union all
select 19947,'019947','1983-044FP','deb Kosmos-1461','',55801,'Reentered','1989-05-19 00:00:00','1989-04-05 00:00:00',96.17,514,637,64.96,'Intermediate' from dual union all
select 19948,'019948','1970-025QD','deb Agena D','',21502,'In Earth orbit','','1988-05-20 00:00:00',113.4,1053,1713,99.57,'Sun-Synch' from dual union all
select 19949,'019949','1970-091AX','deb Kosmos-375','',21568,'In Earth orbit','','1989-01-17 00:00:00',108.33,538,1761,62.65,'Intermediate' from dual union all
select 1995,'001995','1965-032C','Explorer 27 despin weight','',11782,'Reentered','1990-01-13 00:00:00','1966-02-05 00:00:00',106.68,834,1312,41.08,'Intermediate' from dual union all
select 19950,'019950','1965-027AY','deb SNAPSHOT','',11777,'In Earth orbit','','1988-04-13 00:00:00',111.48,1273,1317,90.31,'Polar' from dual union all
select 19951,'019951','1985-056Q','deb Ariane V14','',58279,'In Earth orbit','','1989-01-21 00:00:00',633.37,869,35236,7.35,'GEO Transfer' from dual union all
select 19952,'019952','1989-016K','Akebono despin weight','',62293,'Reentered','1998-08-25 00:00:00','1989-03-28 00:00:00',207.49,275,10189,75.72,'Medium' from dual union all
select 19953,'019953','1978-026HP','deb Delta 139','',45221,'Reentered','2004-11-26 00:00:00','1985-07-31 00:00:00',101.69,799,879,98.68,'Sun-Synch' from dual union all
select 19954,'019954','1965-027AZ','deb SNAPSHOT','',11777,'In Earth orbit','','1989-02-25 00:00:00',111.13,1257,1300,90.31,'Polar' from dual union all
select 19955,'019955','1987-020CU','deb Kosmos-1823','',60110,'Reentered','1989-07-17 00:00:00','1989-01-23 00:00:00',112.29,907,1757,73.03,'Intermediate' from dual union all
select 19956,'019956','1989-011E','deb Kosmos- 2001','',62288,'Reentered','1989-04-30 00:00:00','1989-03-25 00:00:00',91.71,229,488,62.81,'Intermediate' from dual union all
select 19957,'019957','1987-020CV','deb Kosmos-1823','',60110,'Reentered','1989-12-21 00:00:00','1989-03-26 00:00:00',113.84,1154,1651,74.01,'Intermediate' from dual union all
select 19958,'019958','1988-113P','Kosmos-1985 SS 12','ESO',61362,'Reentered','1989-06-17 00:00:00','1989-04-29 00:00:00',94.98,511,525,73.55,'Intermediate' from dual union all
select 19959,'019959','1988-113Q','Kosmos-1985 SS 13','ESO',61362,'Reentered','1989-06-17 00:00:00','1989-04-29 00:00:00',94.83,507,514,73.55,'Intermediate' from dual union all
select 1996,'001996','1965-096D','deb Asterix','',11846,'Reentered','2004-11-28 00:00:00','1966-02-05 00:00:00',108.57,522,1800,34.25,'Intermediate' from dual union all
select 19960,'019960','1970-091AY','deb Kosmos-375','',21568,'In Earth orbit','','1984-12-29 00:00:00',111.55,563,2033,62.92,'Intermediate' from dual union all
select 19961,'019961','61 Omicron 297','deb Ablestar','',7273,'In Earth orbit','','1988-03-27 00:00:00',105.1,868,1131,65.65,'Intermediate' from dual union all
select 19962,'019962','1989-016L','Akebono despin weight','',62293,'Reentered','1992-02-01 00:00:00','1989-04-09 00:00:00',206.12,285,10075,75.11,'Medium' from dual union all
select 19963,'019963','1989-016M','deb Akebono','',62293,'Reentered','2002-11-30 00:00:00','1989-04-02 00:00:00',209.84,281,10357,75.22,'Medium' from dual union all
select 19964,'019964','1965-027BA','deb SNAPSHOT','',11777,'In Earth orbit','','1989-04-19 00:00:00',111.43,1268,1317,90.31,'Polar' from dual union all
select 19965,'019965','1977-065GC','deb Delta 132','',42193,'Reentered','1990-10-20 00:00:00','1983-06-03 00:00:00',104.74,550,1415,29.16,'Intermediate' from dual union all
select 19966,'019966','1986-019VJ','deb Ariane V16','',59212,'Reentered','1989-07-06 00:00:00','1989-04-04 00:00:00',98.49,670,703,98.6,'Sun-Synch' from dual union all
select 19967,'019967','1978-026HQ','deb Delta 139','',45221,'Reentered','1993-11-26 00:00:00','1989-06-18 00:00:00',102.45,825,924,98.84,'Sun-Synch' from dual union all
select 19968,'019968','1989-033A','Atlantis (STS-30)','OV-104',62310,'Landed','1989-05-08 00:00:00','1989-05-06 00:00:00',90.8,296,331,28.89,'Intermediate' from dual union all
select 19969,'019969','1989-033B','Magellan','Magellan',62310,'Deep Space','1989-05-05 00:00:00','1989-05-05 00:00:00',181.73,300,34312,28.89,'Earth Escape' from dual union all
select 1997,'001997','1966-009A','OPS 1439','Project 770 Group 3-D',13416,'Reentered Attached','1969-09-26 00:00:00','1966-11-04 00:00:00',94.65,499,504,82.08,'Intermediate' from dual union all
select 19970,'019970','1989-033C','IUS-18 SRM-1','Orbus 21',62310,'In Earth orbit','','1989-06-04 00:00:00',448.43,296,25762,27.93,'Highly Ellip' from dual union all
select 19971,'019971','1989-033D','IUS-18 SRM-2','Orbus 6E',62310,'Deep Space','1989-05-05 00:00:00','1989-05-05 00:00:00',181.73,300,34312,28.89,'Earth Escape' from dual union all
select 19972,'019972','1989-034A','Kosmos-2019','Oblik No. 392',62311,'Landed','1989-05-18 00:00:00','1989-05-07 00:00:00',89.91,186,354,62.85,'Intermediate' from dual union all
select 19973,'019973','1989-034B','11S510','11S510',62311,'Reentered','1989-05-11 00:00:00','1989-05-07 00:00:00',89.2,227,243,62.84,'Intermediate' from dual union all
select 19974,'019974','1989-034C','Sensor cover','Sensor cover',62311,'Reentered','1989-05-06 00:00:00','1989-05-05 00:00:00',88.88,200,239,62.82,'Intermediate' from dual union all
select 19975,'019975','1989-034D','Sensor cover','Sensor cover',62311,'Reentered','1989-05-07 00:00:00','1989-05-06 00:00:00',88.6,199,212,62.82,'Intermediate' from dual union all
select 19976,'019976','1989-035A','USA 37','VORTEX 13',62312,'In Earth orbit','','1989-07-01 00:00:00',1447.01,31000,41000,10,'Near-synch' from dual union all
select 19977,'019977','1989-035B','Titan 34D-16 Stage 2','LR91AJ11 34D-16',62312,'Reentered','1989-06-01 00:00:00','1989-05-10 00:00:00',102.75,208,1570,29.1,'Intermediate' from dual union all
select 19978,'019978','1989-032C','KB electropitaniya','KBE',62309,'Reentered','1989-05-31 00:00:00','1989-05-11 00:00:00',90.13,212,349,62.82,'Intermediate' from dual union all
select 19979,'019979','1989-032D','KBE part','KBE part',62309,'Reentered','1989-05-15 00:00:00','1989-05-12 00:00:00',89.68,200,317,62.82,'Intermediate' from dual union all
select 1998,'001998','1966-010A','Kosmos-107','Zenit-2 No. 34',13417,'Landed','1966-02-18 00:00:00','1966-02-12 00:00:00',89.61,204,306,64.96,'Intermediate' from dual union all
select 19980,'019980','1976-126CF','deb Kosmos-886','',39078,'Reentered','2014-03-28 00:00:00','1991-09-14 00:00:00',109.47,588,1817,65.88,'Intermediate' from dual union all
select 19981,'019981','62 Kappa 8','West Ford deb','[UNID133] RCS 0.15',8192,'Reentered','1970-01-01 00:00:00','1968-12-29 00:00:00',152.81,2751,3429,86.65,'Medium' from dual union all
select 19982,'019982','1987-020CW','deb Kosmos-1823','',60110,'Reentered','1991-03-28 00:00:00','1988-08-09 00:00:00',115.75,1339,1640,73.43,'Intermediate' from dual union all
select 19983,'019983','1989-035C','Transtage D-16','Transtage D-16',62312,'In Earth orbit','','1989-05-12 00:00:00',1447.01,31000,41000,10,'Near-synch' from dual union all
select 19984,'019984','1989-032E','PTDU cover','KBE part',62309,'Reentered','1989-05-18 00:00:00','1989-05-11 00:00:00',92.09,311,442,62.84,'Intermediate' from dual union all
select 19985,'019985','1989-032F','PTDU cover','KBE part',62309,'Reentered','1989-05-15 00:00:00','1989-05-12 00:00:00',91.94,310,430,62.81,'Intermediate' from dual union all
select 19986,'019986','1989-036A','Kosmos-2020','Kobal''t',62313,'Landed','1989-07-15 00:00:00','1989-05-24 00:00:00',88.9,160,280,64.78,'Intermediate' from dual union all
select 19987,'019987','1989-036B','11S510','11S510',62313,'Reentered','1989-05-21 00:00:00','1989-05-19 00:00:00',89.16,165,301,64.8,'Intermediate' from dual union all
select 19988,'019988','1989-034E','KDU','KDU',62311,'Reentered','1989-05-20 00:00:00','1989-05-18 00:00:00',88.7,148,273,62.85,'Intermediate' from dual union all
select 19989,'019989','1989-034F','Blenda','Sunshade',62311,'Reentered','1989-05-19 00:00:00','1989-05-18 00:00:00',89.18,156,312,62.84,'Intermediate' from dual;
insert into SATELLITE
select 1999,'001999','1965-082JA','deb Transtage 4','',11832,'Reentered','1967-04-20 00:00:00','1967-03-21 00:00:00',93.29,402,469,32.33,'Intermediate' from dual union all
select 19990,'019990','1964-006T','deb Elektron-1','',10292,'Reentered','2014-10-28 00:00:00','1980-06-14 00:00:00',152.02,403,5711,60.8,'Medium' from dual union all
select 19991,'019991','1964-006U','deb Elektron-1','',10292,'Reentered','2013-12-21 00:00:00','1980-05-29 00:00:00',151.96,402,5708,60.8,'Medium' from dual union all
select 19992,'019992','1964-006V','deb unk','[UNID134]',10292,'Reentered','1990-03-15 00:00:00','1985-07-31 00:00:00',186.81,386,8508,62.69,'Medium' from dual union all
select 19993,'019993','1964-006W','deb Elektron-1','',10292,'Reentered','2011-11-18 00:00:00','1986-02-07 00:00:00',140.59,402,4755,60.81,'Medium' from dual union all
select 19994,'019994','1964-006X','deb Elektron-1','',10292,'Reentered','2011-04-21 00:00:00','1986-04-13 00:00:00',140.8,403,4772,60.85,'Medium' from dual union all
select 19995,'019995','1964-006Y','deb Elektron-1','',10292,'Reentered','2013-07-02 00:00:00','1986-07-19 00:00:00',141.87,399,4865,60.86,'Medium' from dual union all
select 19996,'019996','1964-006Z','deb Elektron-1','',10292,'Reentered','2009-12-07 00:00:00','1986-04-17 00:00:00',140.35,398,4738,60.81,'Medium' from dual union all
select 19997,'019997','1964-006AA','deb Elektron-1','',10292,'Reentered','2013-07-21 00:00:00','1986-07-20 00:00:00',141.7,402,4849,60.82,'Medium' from dual union all
select 19998,'019998','1964-006AB','deb Elektron-1','',10292,'Reentered','2013-06-10 00:00:00','1980-04-20 00:00:00',153.02,402,5795,60.84,'Medium' from dual union all
select 19999,'019999','1986-019VK','deb Ariane V16','',59212,'Reentered','1989-09-25 00:00:00','1989-04-19 00:00:00',99.62,686,794,98.69,'Sun-Synch' from dual union all
select 2,'000002','57 Alpha 2','1-y ISZ','PS-1',4305,'Reentered','1958-01-04 00:00:00','1957-10-04 00:00:00',96.18,214,938,65.1,'Intermediate' from dual union all
select 20,'000020','59 Eta','Vanguard III','Magne-Ray satellite',5529,'In Earth orbit','','1959-10-27 00:00:00',130.38,521,3758,33.35,'Medium' from dual union all
select 200,'000200','61 A Epsilon 4','deb Discoverer XXXIV','',7252,'Reentered','1961-12-10 00:00:00','1961-11-14 00:00:00',95.53,247,842,82.5,'Intermediate' from dual union all
select 2000,'002000','1966-010B','8A92E','8A92E',13417,'Reentered','1966-02-28 00:00:00','1966-02-18 00:00:00',89.38,213,275,64.96,'Intermediate' from dual union all
select 20000,'020000','1989-037A','Kosmos-2021','Kometa No. 11',62314,'Landed','1989-07-06 00:00:00','1989-05-31 00:00:00',89.41,213,278,69.95,'Intermediate' from dual union all
select 20001,'020001','1989-037B','11S510','11S510',62314,'Reentered','1989-05-27 00:00:00','1989-05-27 00:00:00',87.34,135,151,69.95,'Intermediate' from dual union all
select 20002,'020002','1989-037C','Kosmos-2021 part','Kometa part',62314,'Reentered','1989-05-25 00:00:00','1989-05-24 00:00:00',88.98,180,268,69.96,'Intermediate' from dual union all
select 20003,'020003','1987-020CX','deb Kosmos-1823','',60110,'Reentered','1990-01-28 00:00:00','1989-05-20 00:00:00',116.81,1210,1865,72.83,'Intermediate' from dual union all
select 20004,'020004','1987-020CY','deb Kosmos-1823','',60110,'Reentered','1990-07-10 00:00:00','1989-05-24 00:00:00',112.3,1241,1423,73.64,'Intermediate' from dual union all
select 20005,'020005','1963-014EV','deb West Ford','RCS 0.03',9245,'In Earth orbit','','1993-08-08 00:00:00',165.14,2599,4587,87,'Medium' from dual union all
select 20006,'020006','1989-038A','Resurs-F','Resurs-F1 14F43 No. 45',62315,'Landed','1989-06-17 00:00:00','1989-06-01 00:00:00',89.71,252,269,82.3,'Intermediate' from dual union all
select 20007,'020007','1989-038B','11S510','11S510',62315,'Reentered','1989-05-26 00:00:00','1989-05-26 00:00:00',87.25,131,146,82.28,'Intermediate' from dual union all
select 20008,'020008','1978-026HR','deb Delta 139','',45221,'In Earth orbit','','1993-08-06 00:00:00',104.58,810,1139,98.88,'Sun-Synch' from dual union all
select 20009,'020009','1967-092H','NNS O-14 insulation','',15347,'Reentered','1991-09-12 00:00:00','1991-09-11 00:00:00',91.58,331,374,89.13,'Polar' from dual union all
select 2001,'002001','1966-006D','Blok-L No. U103-32','Blok-L',13413,'Deep Space','1966-02-01 00:00:00','1966-01-31 00:00:00',21545.69,220,499999,51.85,'Cislunar' from dual union all
select 20010,'020010','1978-100AP','deb S5M','',45295,'Reentered','2012-11-05 00:00:00','1993-08-08 00:00:00',116.32,1223,1808,82.22,'Intermediate' from dual union all
select 20011,'020011','1968-091DN','deb Kosmos-249','',16818,'In Earth orbit','','1993-08-04 00:00:00',112.75,747,1959,62.22,'Intermediate' from dual union all
select 20012,'020012','1976-077FM','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',113.8,1336,1466,101.95,'Sun-Synch' from dual union all
select 20013,'020013','1977-121BV','deb Kosmos-970','',42249,'Reentered','1996-05-05 00:00:00','1993-08-23 00:00:00',97.93,540,779,65.96,'Intermediate' from dual union all
select 20014,'020014','1976-067BT','deb Kosmos-839','',39019,'In Earth orbit','','1993-07-02 00:00:00',116.35,968,2066,65.88,'Intermediate' from dual union all
select 20015,'020015','1968-091DP','deb Kosmos-249','',16818,'Reentered','1989-07-24 00:00:00','1989-05-25 00:00:00',115.38,226,2720,63.29,'Intermediate' from dual union all
select 20016,'020016','1978-100AQ','deb S5M','',45295,'In Earth orbit','','1993-07-23 00:00:00',117.83,1270,1897,82.43,'Intermediate' from dual union all
select 20017,'020017','1963-014EW','deb West Ford','RCS 0.03',9245,'Reentered','1999-02-27 00:00:00','1993-07-23 00:00:00',162.93,1016,5990,85.25,'Medium' from dual union all
select 20018,'020018','1963-014EX','deb West Ford','RCS 0.04',9245,'Reentered','1996-07-21 00:00:00','1990-10-20 00:00:00',164.75,1753,5401,86.66,'Medium' from dual union all
select 20019,'020019','1987-020CZ','deb Kosmos-1823','',60110,'Reentered','1989-09-08 00:00:00','1989-05-25 00:00:00',110.23,1099,1376,73.55,'Intermediate' from dual union all
select 2002,'002002','1966-011A','Kosmos-108','DS-U1-G No. 1',13418,'Reentered','1966-11-21 00:00:00','1966-11-08 00:00:00',89.84,190,344,48.84,'Intermediate' from dual union all
select 20020,'020020','1987-020DA','deb Kosmos-1823','',60110,'Reentered','1991-01-17 00:00:00','1989-05-25 00:00:00',113.51,1345,1431,73.54,'Intermediate' from dual union all
select 20021,'020021','1989-016N','deb Akebono','',62293,'Reentered','2000-09-19 00:00:00','1989-04-12 00:00:00',210.5,283,10404,74.49,'Medium' from dual union all
select 20022,'020022','1963-014EY','deb West Ford','RCS 0.03',9245,'Reentered','2004-01-12 00:00:00','1991-04-01 00:00:00',165.62,2413,4811,86.91,'Medium' from dual union all
select 20023,'020023','1963-014EZ','deb West Ford','RCS 0.02',9245,'In Earth orbit','','1993-08-01 00:00:00',163.84,1283,5798,85.8,'Medium' from dual union all
select 20024,'020024','1989-039A','Kosmos-2022','Uragan No. 28L',62316,'In Earth orbit','','1989-07-01 00:00:00',675.69,19123,19135,64.83,'Medium' from dual union all
select 20025,'020025','1989-039B','Kosmos-2023','Uragan No. 29L',62316,'In Earth orbit','','1989-07-01 00:00:00',675.71,19095,19164,64.85,'Medium' from dual union all
select 20026,'020026','1989-039C','Kosmos-2024','Etalon PKA No. 2L',62316,'In Earth orbit','','1989-07-01 00:00:00',675.39,19099,19144,64.84,'Medium' from dual union all
select 20027,'020027','1989-039D','8S812','8S812',62316,'Reentered','1989-05-31 00:00:00','1989-05-31 00:00:00',87.3,131,151,64.83,'Intermediate' from dual union all
select 20028,'020028','1989-039E','Blok DM2','11S861 No. 39L',62316,'In Earth orbit','','1991-10-14 00:00:00',674.5,19057,19141,65.32,'Medium' from dual union all
select 20029,'020029','1978-026HS','deb Delta 139','',45221,'In Earth orbit','','1993-08-04 00:00:00',101.05,753,864,98.64,'Sun-Synch' from dual union all
select 2003,'002003','1966-009B','OPS 1439 sensor cover','',13416,'Reentered','1966-11-18 00:00:00','1966-11-08 00:00:00',91.54,345,356,82.05,'Intermediate' from dual union all
select 20030,'020030','1976-067BU','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-11 00:00:00',118.54,991,2240,65.68,'Intermediate' from dual union all
select 20031,'020031','1965-027BB','deb SNAPSHOT','',11777,'In Earth orbit','','1993-08-08 00:00:00',111.32,1266,1308,90.26,'Polar' from dual union all
select 20032,'020032','1977-121BW','Unknown deb                       ','',42249,'In Earth orbit','','1993-05-13 00:00:00',107.65,530,1706,62.44,'Intermediate' from dual union all
select 20033,'020033','1963-014FA','deb West Ford','RCS 0.03',9245,'Reentered','1994-09-14 00:00:00','1993-06-06 00:00:00',159,574,6114,84.28,'Medium' from dual union all
select 20034,'020034','1989-016P','deb Akebono','',62293,'Reentered','1991-09-04 00:00:00','1989-03-14 00:00:00',209.04,277,10302,75.1,'Medium' from dual union all
select 20035,'020035','1989-040A','Kosmos-2025','Oblik No. 393',62317,'Landed','1989-06-15 00:00:00','1989-06-03 00:00:00',89.83,179,354,62.81,'Intermediate' from dual union all
select 20036,'020036','1989-040B','11S510','11S510',62317,'Reentered','1989-06-08 00:00:00','1989-06-05 00:00:00',88.94,211,234,62.8,'Intermediate' from dual union all
select 20037,'020037','1989-040C','Sensor cover','Sensor cover',62317,'Reentered','1989-06-03 00:00:00','1989-06-01 00:00:00',89.63,242,271,62.78,'Intermediate' from dual union all
select 20038,'020038','1989-040D','Sensor cover','Sensor cover',62317,'Reentered','1989-06-03 00:00:00','1989-06-01 00:00:00',89.54,247,257,62.83,'Intermediate' from dual union all
select 20039,'020039','1989-040E','Sensor cover','Sensor cover',62317,'Reentered','1989-06-02 00:00:00','1989-06-01 00:00:00',89.02,213,240,62.78,'Intermediate' from dual union all
select 2004,'002004','1966-009C','OPS 1439 sensor cover','',13416,'Reentered','1966-11-18 00:00:00','1966-11-09 00:00:00',91.4,336,351,82.06,'Intermediate' from dual union all
select 20040,'020040','1989-041A','Superbird A','Superbird A',62318,'In Earth orbit','','1989-07-09 00:00:00',1436.06,35770,35802,.03,'Stationary' from dual union all
select 20041,'020041','1989-041B','Kopernikus','DFS 1',62318,'In Earth orbit','','1989-07-06 00:00:00',1436.04,35746,35825,.06,'Stationary' from dual union all
select 20042,'020042','1989-041C','Ariane H10 V31','Ariane H10 S/N T404',62318,'Reentered','1996-06-13 00:00:00','1992-02-16 00:00:00',481.39,213,27724,6.61,'Highly Ellip' from dual union all
select 20043,'020043','1989-041D','Spelda 10 S/N UV3','Spelda 10',62318,'Reentered','1992-05-10 00:00:00','1989-06-18 00:00:00',620.08,202,35217,7,'GEO Transfer' from dual union all
select 20044,'020044','1989-039F','Etalon adapter','Etalon adapter',62316,'In Earth orbit','','1993-08-16 00:00:00',675.37,19099,19143,65.48,'Medium' from dual union all
select 20045,'020045','1989-042A','Kosmos-2026','Parus',62319,'In Earth orbit','','1989-07-24 00:00:00',104.71,952,1010,82.94,'Intermediate' from dual union all
select 20046,'020046','1989-042B','S3M','S3M',62319,'In Earth orbit','','1989-07-07 00:00:00',104.6,950,1002,82.94,'Intermediate' from dual union all
select 20047,'020047','1978-026HT','deb Delta 139','',45221,'Reentered','1999-04-11 00:00:00','1992-02-24 00:00:00',95.38,529,545,97.8,'Sun-Synch' from dual union all
select 20048,'020048','1965-027BC','deb SNAPSHOT','',11777,'In Earth orbit','','1993-08-05 00:00:00',111.15,1262,1297,90.27,'Polar' from dual union all
select 20049,'020049','1970-089DG','deb Kosmos-374','',21566,'Reentered','1989-06-15 00:00:00','1989-06-11 00:00:00',99.89,136,1371,63.59,'Intermediate' from dual union all
select 2005,'002005','1966-010C','deb Kosmos-107','',13417,'Reentered','1966-02-12 00:00:00','1966-02-10 00:00:00',89.18,128,340,64.98,'Intermediate' from dual union all
select 20050,'020050','1986-019VL','deb Ariane V16','',59212,'Reentered','1990-04-09 00:00:00','1989-06-09 00:00:00',100.69,759,824,98.77,'Sun-Synch' from dual union all
select 20051,'020051','1976-077FN','deb Delta 126','',39029,'In Earth orbit','','1993-08-05 00:00:00',118.64,1502,1738,102.74,'Sun-Synch' from dual union all
select 20052,'020052','1989-043A','Molniya-3','Molniya-3 No. 45',62320,'Reentered','2001-12-14 00:00:00','1989-07-08 00:00:00',717.74,581,39772,62.91,'Molniya' from dual union all
select 20053,'020053','1989-043B','11S510','11S510',62320,'Reentered','1989-07-05 00:00:00','1989-06-15 00:00:00',91.56,214,489,62.81,'Intermediate' from dual union all
select 20054,'020054','1989-043C','BOZ','BOZ',62320,'Reentered','1989-07-08 00:00:00','1989-06-15 00:00:00',92.17,203,558,62.82,'Intermediate' from dual union all
select 20055,'020055','1989-043D','Blok-ML','Blok-ML No. 332',62320,'Reentered','1999-06-03 00:00:00','1993-08-16 00:00:00',733.37,1317,39804,65.09,'GEO Transfer' from dual union all
select 20056,'020056','1989-038C','Pion','Pion-1',62315,'Reentered','1989-07-23 00:00:00','1989-06-09 00:00:00',89.75,256,269,62.81,'Intermediate' from dual union all
select 20057,'020057','1989-043E','deb Molniya-3','',62320,'Reentered','1989-07-06 00:00:00','1989-06-09 00:00:00',92.9,227,606,62.81,'Intermediate' from dual union all
select 20058,'020058','1989-043F','deb Molniya-3','',62320,'Reentered','1989-06-29 00:00:00','1989-06-10 00:00:00',92.73,218,599,62.81,'Intermediate' from dual union all
select 20059,'020059','1989-043G','deb Molniya-3','',62320,'Reentered','1989-06-30 00:00:00','1989-06-09 00:00:00',92.73,216,601,62.81,'Intermediate' from dual union all
select 2006,'002006','1966-010D','deb Kosmos-107','',13417,'Reentered','1966-02-12 00:00:00','1966-02-10 00:00:00',89.27,127,350,64.98,'Intermediate' from dual union all
select 20060,'020060','1989-038D','Pion','Pion-2',62315,'Reentered','1989-07-24 00:00:00','1989-06-10 00:00:00',89.75,257,268,82.29,'Intermediate' from dual union all
select 20061,'020061','1989-044A','GPS II-2','Navstar GPS SVN 13',62321,'In Earth orbit','','1989-07-11 00:00:00',717.92,19967,20395,54.59,'Medium' from dual union all
select 20062,'020062','1989-044B','Delta 185','Delta SSPS AJ10-118K',62321,'Reentered','1989-07-12 00:00:00','1989-06-17 00:00:00',94.74,180,832,30.31,'Intermediate' from dual union all
select 20063,'020063','1989-044C','Star 48B','Star 48B S/N 10046-18',62321,'Reentered','1991-05-06 00:00:00','1990-02-21 00:00:00',264.75,156,14403,37.38,'Highly Ellip' from dual union all
select 20064,'020064','1989-045A','Kosmos-2027','Vektor',62322,'Reentered','1992-04-14 00:00:00','1990-07-05 00:00:00',93.86,454,472,65.83,'Intermediate' from dual union all
select 20065,'020065','1989-045B','S3M','S3M',62322,'Reentered','1991-03-25 00:00:00','1990-02-12 00:00:00',93.74,436,479,65.84,'Intermediate' from dual union all
select 20066,'020066','1989-046A','DSP F14','DSP 14',62323,'In Earth orbit','','2000-04-11 00:00:00',1436.06,35449,36122,5,'Inclined GEO' from dual union all
select 20067,'020067','1989-046B','Titan 45K-1 Stage 2','LR91AJ11A 45K-1',62323,'Reentered','1989-06-15 00:00:00','1989-06-14 00:00:00',88.16,152,215,28.6,'Intermediate' from dual union all
select 20068,'020068','1989-046C','IUS-8 SRM-1','Orbus 21',62323,'In Earth orbit','','1997-09-11 00:00:00',307.69,255,17184,27.49,'Highly Ellip' from dual union all
select 20069,'020069','1989-046D','IUS-8 SRM-2','Orbus 6E',62323,'In Earth orbit','','1989-07-01 00:00:00',1436.78,35400,36200,2,'Drift GEO' from dual union all
select 2007,'002007','1966-011B','S1','S1',13418,'Reentered','1966-06-01 00:00:00','1966-02-21 00:00:00',95.07,221,822,48.85,'Intermediate' from dual union all
select 20070,'020070','1989-040F','KDU','KDU',62317,'Reentered','1989-06-17 00:00:00','1989-06-15 00:00:00',88.69,143,277,62.8,'Intermediate' from dual union all
select 20071,'020071','1989-040G','KDU part','KDU part',62317,'Reentered','1989-06-15 00:00:00','1989-06-15 00:00:00',87.84,122,214,62.81,'Intermediate' from dual union all
select 20072,'020072','1989-040H','Blenda','Sunshade',62317,'Reentered','1989-07-01 00:00:00','1989-06-15 00:00:00',88.72,141,282,62.8,'Intermediate' from dual union all
select 20073,'020073','1989-047A','Kosmos-2028','Oblik',62324,'Landed','1989-07-06 00:00:00','1989-06-23 00:00:00',89.65,228,287,69.99,'Intermediate' from dual union all
select 20074,'020074','1989-047B','11S510','11S510',62324,'Reentered','1989-06-21 00:00:00','1989-06-18 00:00:00',88.92,193,250,69.99,'Intermediate' from dual union all
select 20075,'020075','1989-047C','Sensor cover','Sensor cover',62324,'Reentered','1989-06-17 00:00:00','1989-06-16 00:00:00',88.92,187,255,70.06,'Intermediate' from dual union all
select 20076,'020076','1989-038E','Blenda','Sunshade',62315,'Reentered','1989-06-20 00:00:00','1989-06-17 00:00:00',89.32,226,256,82.29,'Intermediate' from dual union all
select 20077,'020077','1989-038F','KDU part','KDU part',62315,'Reentered','1989-06-19 00:00:00','1989-06-17 00:00:00',92.01,248,499,82.31,'Intermediate' from dual;
insert into SATELLITE
select 20078,'020078','1989-038G','KDU part','KDU part',62315,'Reentered','1989-06-20 00:00:00','1989-06-17 00:00:00',91.51,255,443,82.29,'Intermediate' from dual union all
select 20079,'020079','1989-038H','KDU','KDU',62315,'Reentered','1989-07-02 00:00:00','1989-06-18 00:00:00',89.54,245,258,82.29,'Intermediate' from dual union all
select 2008,'002008','1966-011C','deb Kosmos-108','',13418,'Reentered','1966-03-27 00:00:00','1966-02-12 00:00:00',95.04,216,825,48.86,'Intermediate' from dual union all
select 20080,'020080','1989-038J','KDU part','KDU part',62315,'Reentered','1989-06-19 00:00:00','1989-06-17 00:00:00',89.16,226,240,82.29,'Intermediate' from dual union all
select 20081,'020081','1989-039G','SOZ','SOZ',62316,'In Earth orbit','','1992-02-17 00:00:00',339.43,608,18872,65.02,'Highly Ellip' from dual union all
select 20082,'020082','1989-039H','SOZ','SOZ',62316,'In Earth orbit','','1992-02-20 00:00:00',339.44,632,18850,65.12,'Highly Ellip' from dual union all
select 20083,'020083','1989-048A','Raduga-1','Globus No. 11',62325,'In Earth orbit','','1989-07-23 00:00:00',1436.09,35763,35810,1.41,'Stationary' from dual union all
select 20084,'020084','1989-048B','8S812','8S812',62325,'Reentered','1989-06-23 00:00:00','1989-06-23 00:00:00',87.98,171,178,51.63,'Intermediate' from dual union all
select 20085,'020085','1989-048C','Perekhodnik','Sredniy perekhodnik',62325,'Reentered','1989-06-22 00:00:00','1989-06-22 00:00:00',88.08,173,186,51.61,'Intermediate' from dual union all
select 20086,'020086','1989-048D','Blok DM2','11S861 No. 16L',62325,'In Earth orbit','','1989-07-24 00:00:00',1470.99,36376,36558,1.4,'Drift GEO' from dual union all
select 20087,'020087','1981-053MJ','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',105.99,948,1134,82.97,'Intermediate' from dual union all
select 20088,'020088','1976-077FP','deb Delta 126','',39029,'In Earth orbit','','1993-08-06 00:00:00',117.46,1501,1633,102.45,'Sun-Synch' from dual union all
select 20089,'020089','1981-053MK','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-04 00:00:00',104.52,926,1017,82.78,'Intermediate' from dual union all
select 2009,'002009','61 A Delta 5','Midas IV solar array cover','Strongback',7251,'In Earth orbit','','1966-01-30 00:00:00',165.69,3500,3730,95.84,'Medium' from dual union all
select 20090,'020090','1971-015DU','deb Kosmos-397','',24833,'In Earth orbit','','1993-08-07 00:00:00',107.48,545,1676,65.7,'Intermediate' from dual union all
select 20091,'020091','1971-015DV','deb Kosmos-397','',24833,'Reentered','1989-06-20 00:00:00','1989-06-18 00:00:00',91.94,325,415,65.6,'Intermediate' from dual union all
select 20092,'020092','1987-020DB','deb Kosmos-1823','',60110,'Reentered','1990-06-13 00:00:00','1990-06-13 00:00:00',93.92,453,480,73.39,'Intermediate' from dual union all
select 20093,'020093','1989-048E','SOZ','SOZ',62325,'Reentered','1990-03-30 00:00:00','1989-12-31 00:00:00',569.31,135,32617,46.8,'Highly Ellip' from dual union all
select 20094,'020094','1989-048F','SOZ','SOZ',62325,'Reentered','2001-02-20 00:00:00','1989-07-26 00:00:00',648.4,334,36541,47.27,'GEO Transfer' from dual union all
select 20095,'020095','1989-049A','Resurs-F','Resurs-F1 14F43 No. 46',62326,'Landed','1989-07-11 00:00:00','1989-06-29 00:00:00',89.83,259,274,82.56,'Intermediate' from dual union all
select 20096,'020096','1989-049B','11S510','11S510',62326,'Reentered','1989-06-29 00:00:00','1989-06-28 00:00:00',87.6,150,162,82.57,'Intermediate' from dual union all
select 20097,'020097','1988-113R','Kosmos-1985 SS 14','ESO',61362,'Reentered','1989-08-26 00:00:00','1989-06-28 00:00:00',94.81,500,519,73.55,'Intermediate' from dual union all
select 20098,'020098','1988-113S','Kosmos-1985 SS 15','ESO',61362,'Reentered','1989-08-25 00:00:00','1989-06-28 00:00:00',94.81,504,515,73.55,'Intermediate' from dual union all
select 20099,'020099','1988-113T','Kosmos-1985 SS 16','ESO',61362,'Reentered','1989-08-28 00:00:00','1989-06-28 00:00:00',94.9,507,521,73.57,'Intermediate' from dual union all
select 201,'000201','61 A Zeta 1','Discoverer 35','CORONA C''-10',7259,'Reentered Attached','1961-12-03 00:00:00','1961-11-25 00:00:00',89.3,233,247,81.63,'Intermediate' from dual union all
select 2010,'002010','61 Omicron 211','deb r/b Ablestar','',7273,'Reentered','1980-05-06 00:00:00','1966-02-09 00:00:00',103.08,783,1026,66.46,'Intermediate' from dual union all
select 20100,'020100','1981-053ML','deb Kosmos-1275','',52576,'In Earth orbit','','1989-07-24 00:00:00',103.04,884,921,82.97,'Intermediate' from dual union all
select 20101,'020101','1964-006AC','deb Elektron-1','',10292,'Reentered','2012-10-23 00:00:00','1993-07-01 00:00:00',125.48,394,3455,60.72,'Medium' from dual union all
select 20102,'020102','1965-108AC','deb Transtage 8','',11858,'In Earth orbit','','1989-07-24 00:00:00',280.62,252,15388,27.06,'Highly Ellip' from dual union all
select 20103,'020103','1989-050A','Nadezhda','Nadezhda 17F118 No. 403',62327,'In Earth orbit','','1989-08-04 00:00:00',104.84,960,1014,82.96,'Intermediate' from dual union all
select 20104,'020104','1989-050B','S3M','S3M',62327,'In Earth orbit','','1990-03-15 00:00:00',104.71,959,1003,82.99,'Intermediate' from dual union all
select 20105,'020105','1989-051A','Kosmos-2029','Oblik No. 394',62328,'Landed','1989-07-19 00:00:00','1989-07-07 00:00:00',91.53,338,361,82.38,'Intermediate' from dual union all
select 20106,'020106','1989-051B','11S510','11S510',62328,'Reentered','1989-07-07 00:00:00','1989-07-07 00:00:00',87.29,135,146,82.32,'Intermediate' from dual union all
select 20107,'020107','1989-052A','Gorizont','Gorizont No. 27L',62329,'In Earth orbit','','1989-08-04 00:00:00',1436.01,35771,35799,1.43,'Stationary' from dual union all
select 20108,'020108','1989-052B','8S812','8S812',62329,'Reentered','1989-07-07 00:00:00','1989-07-07 00:00:00',87.84,166,170,51.62,'Intermediate' from dual union all
select 20109,'020109','1989-052C','Perekhodnik','Sredniy perekhodnik',62329,'Reentered','1989-07-06 00:00:00','1989-07-06 00:00:00',87.37,140,149,51.6,'Intermediate' from dual union all
select 2011,'002011','1965-032D','Explorer 27 despin weight','',11782,'In Earth orbit','','1966-02-12 00:00:00',109,1068,1293,41.17,'Intermediate' from dual union all
select 20110,'020110','1989-052D','Blok DM2 No. 25L','11S861 No. 25L',62329,'In Earth orbit','','1989-08-05 00:00:00',1397.16,34904,35138,1.44,'Drift GEO' from dual union all
select 20111,'020111','1989-047D','KDU','KDU',62324,'Reentered','1989-07-13 00:00:00','1989-07-12 00:00:00',87.81,160,173,69.99,'Intermediate' from dual union all
select 20112,'020112','1989-047E','KDU part','KDU part',62324,'Reentered','1989-07-06 00:00:00','1989-07-06 00:00:00',88.1,171,190,69.98,'Intermediate' from dual union all
select 20113,'020113','1989-047F','KDU part','KDU part',62324,'Reentered','1989-07-07 00:00:00','1989-07-06 00:00:00',88.41,185,207,69.98,'Intermediate' from dual union all
select 20114,'020114','1989-047G','Blenda','Sunshade',62324,'Reentered','1989-07-10 00:00:00','1989-07-06 00:00:00',91.27,235,439,69.99,'Intermediate' from dual union all
select 20115,'020115','1989-052E','SOZ','SOZ',62329,'Reentered','1991-08-15 00:00:00','1989-12-31 00:00:00',587.74,154,33575,46.7,'Highly Ellip' from dual union all
select 20116,'020116','1989-052F','SOZ','SOZ',62329,'Reentered','1998-10-18 00:00:00','1989-08-04 00:00:00',619.19,340,35033,47.28,'GEO Transfer' from dual union all
select 20117,'020117','1989-049C','KDU part','KDU part',62326,'Reentered','1989-07-14 00:00:00','1989-07-11 00:00:00',89.79,257,271,82.56,'Intermediate' from dual union all
select 20118,'020118','1989-049D','KDU part','KDU part',62326,'Reentered','1989-07-15 00:00:00','1989-07-11 00:00:00',89.6,241,268,82.56,'Intermediate' from dual union all
select 20119,'020119','1989-049E','KDU','KDU',62326,'Reentered','1989-08-05 00:00:00','1989-07-11 00:00:00',89.79,257,271,82.56,'Intermediate' from dual union all
select 2012,'002012','1966-012A','OPS 1184','GAMBIT SV 975',13419,'Reentered','1966-02-23 00:00:00','1966-02-17 00:00:00',88.44,147,248,96.35,'Sun-Synch' from dual union all
select 20120,'020120','1989-049F','Blenda','Sunshade',62326,'Reentered','1989-07-18 00:00:00','1989-07-11 00:00:00',91.75,258,463,82.55,'Intermediate' from dual union all
select 20121,'020121','1989-049G','KDU part','KDU part',62326,'Reentered','1989-07-14 00:00:00','1989-07-11 00:00:00',91.8,250,476,82.57,'Intermediate' from dual union all
select 20122,'020122','1989-053A','Olympus','Olympus',62330,'In Earth orbit','','1989-08-12 00:00:00',1436.11,35776,35797,.06,'Stationary' from dual union all
select 20123,'020123','1989-053B','Ariane H10 V32','Ariane H10 S/N T28',62330,'Reentered','1994-03-29 00:00:00','1989-08-11 00:00:00',627.76,202,35614,6.29,'GEO Transfer' from dual union all
select 20124,'020124','1989-054A','Kosmos-2030','Kobal''t No. 523',62331,'Reentered','1989-07-29 00:00:00','1989-07-19 00:00:00',89.53,180,323,67.14,'Intermediate' from dual union all
select 20125,'020125','1989-054B','11S510','11S510',62331,'Reentered','1989-07-16 00:00:00','1989-07-16 00:00:00',87.3,121,161,67.14,'Intermediate' from dual union all
select 20126,'020126','1987-020DC','deb Kosmos-1823','',60110,'Reentered','1991-08-18 00:00:00','1991-08-17 00:00:00',92.38,363,420,72.98,'Intermediate' from dual union all
select 20127,'020127','1988-063E','deb Ariane V24','',61312,'In Earth orbit','','1989-07-24 00:00:00',636.15,416,35832,7.98,'GEO Transfer' from dual union all
select 20128,'020128','1972-011C','deb Kosmos-476','[UNID135]',27943,'Reentered','1990-01-13 00:00:00','1989-07-17 00:00:00',93.35,435,441,81.17,'Intermediate' from dual union all
select 20129,'020129','1989-080E','deb S5M','',62357,'In Earth orbit','','1989-11-03 00:00:00',116.5,560,2487,82.53,'Intermediate' from dual union all
select 2013,'002013','1965-093C','SR 8 expt. cover','',11843,'Reentered','2009-03-06 00:00:00','1966-02-14 00:00:00',100.28,705,839,59.68,'Intermediate' from dual union all
select 20130,'020130','1987-020DD','deb Kosmos-1823','',60110,'Reentered','1990-08-13 00:00:00','1990-08-13 00:00:00',93.74,401,514,72.96,'Intermediate' from dual union all
select 20131,'020131','1965-027BD','deb SNAPSHOT','',11777,'In Earth orbit','','1993-08-08 00:00:00',111.25,1257,1312,90.26,'Polar' from dual union all
select 20132,'020132','1988-098C','deb Ariane V26','',61347,'Reentered','1998-04-22 00:00:00','1989-08-10 00:00:00',583.29,248,33246,3.68,'Highly Ellip' from dual union all
select 20133,'020133','1983-044FQ','deb Kosmos-1461','',55801,'Reentered','1989-07-10 00:00:00','1989-07-10 00:00:00',93.2,402,461,64.98,'Intermediate' from dual union all
select 20134,'020134','1989-055A','Resurs-F','Resurs-F1 14F43 No. 47',62332,'Landed','1989-08-08 00:00:00','1989-07-25 00:00:00',89.79,258,271,82.57,'Intermediate' from dual union all
select 20135,'020135','1989-055B','11S510','11S510',62332,'Reentered','1989-07-20 00:00:00','1989-07-19 00:00:00',87.6,148,164,82.56,'Intermediate' from dual union all
select 20136,'020136','1989-056A','Kosmos-2031','Orlets',62333,'Reentered','1989-09-15 00:00:00','1989-07-25 00:00:00',90.36,231,354,50.55,'Intermediate' from dual union all
select 20137,'020137','1989-056B','11S510','11S510',62333,'Reentered','1989-07-21 00:00:00','1989-07-19 00:00:00',88.65,182,234,50.56,'Intermediate' from dual union all
select 20138,'020138','1989-056C','deb Kosmos-2031','',62333,'Reentered','1989-07-19 00:00:00','1989-07-18 00:00:00',88.8,178,253,50.54,'Intermediate' from dual union all
select 20139,'020139','1989-051C','KDU part','KDU part',62328,'Reentered','1989-08-15 00:00:00','1989-07-19 00:00:00',91.34,336,345,82.36,'Intermediate' from dual union all
select 2014,'002014','1966-012B','OPS 3011','Bluebell 2 Cylinder',13419,'Reentered','1966-02-16 00:00:00','1966-02-15 00:00:00',87.99,150,200,96.5,'Sun-Synch' from dual union all
select 20140,'020140','1989-051D','KDU','KDU',62328,'Reentered','1989-10-14 00:00:00','1989-07-19 00:00:00',91.48,337,358,82.37,'Intermediate' from dual union all
select 20141,'020141','1989-051E','Blenda','Sunshade',62328,'Reentered','1989-08-07 00:00:00','1989-07-19 00:00:00',91.5,329,367,82.36,'Intermediate' from dual union all
select 20142,'020142','1989-051F','PTDU cover','KDU part',62328,'Reentered','1989-08-18 00:00:00','1989-07-19 00:00:00',93.57,352,546,82.37,'Intermediate' from dual union all
select 20143,'020143','1989-051G','PTDU cover','KDU part',62328,'Reentered','1989-08-08 00:00:00','1989-07-19 00:00:00',93.85,358,568,82.39,'Intermediate' from dual union all
select 20144,'020144','1989-051H','KDU part','KDU part',62328,'Reentered','1989-07-23 00:00:00','1989-07-19 00:00:00',91.28,328,347,82.36,'Intermediate' from dual union all
select 20145,'020145','1989-057A','Kosmos-2032','Oblik No. 395',62334,'Landed','1989-08-03 00:00:00','1989-07-23 00:00:00',90.46,232,363,82.31,'Intermediate' from dual union all
select 20146,'020146','1989-057B','11S510','11S510',62334,'Reentered','1989-07-22 00:00:00','1989-07-20 00:00:00',88.64,179,235,82.31,'Intermediate' from dual union all
select 20147,'020147','1989-058A','Kosmos-2033','US-PM',62335,'Reentered','1991-01-06 00:00:00','1989-08-23 00:00:00',92.77,404,417,65.02,'Intermediate' from dual union all
select 20148,'020148','1989-058B','11S692','11S692',62335,'Reentered','1989-07-24 00:00:00','1989-07-24 00:00:00',89.3,112,368,65,'Intermediate' from dual union all
select 20149,'020149','1989-059A','Kosmos-2034','Parus',62336,'In Earth orbit','','1989-08-29 00:00:00',104.91,968,1013,82.94,'Intermediate' from dual union all
select 2015,'002015','1966-012C','OPS 3031','Bluebell 2 Sphere',13419,'Reentered','1966-02-22 00:00:00','1966-02-15 00:00:00',88.64,147,268,96.5,'Sun-Synch' from dual union all
select 20150,'020150','1989-059B','S3M','S3M',62336,'In Earth orbit','','1989-08-29 00:00:00',104.77,963,1005,82.94,'Intermediate' from dual union all
select 20151,'020151','1989-060A','Kosmos-2035','Oblik No. 397',62337,'Landed','1989-08-16 00:00:00','1989-08-04 00:00:00',90.37,230,356,82.57,'Intermediate' from dual union all
select 20152,'020152','1989-060B','11S510','11S510',62337,'Reentered','1989-08-04 00:00:00','1989-08-03 00:00:00',88.44,175,220,82.56,'Intermediate' from dual union all
select 20153,'020153','1963-030H','deb Midas','[UNID136] RCS 0.07',9261,'In Earth orbit','','1991-03-11 00:00:00',162.11,1170,5771,88.7,'Medium' from dual union all
select 20154,'020154','1987-020DE','deb Kosmos-1823','',60110,'Reentered','1990-05-16 00:00:00','1990-05-15 00:00:00',97.46,573,702,73.39,'Intermediate' from dual union all
select 20155,'020155','1987-020DF','deb Kosmos-1823','',60110,'Reentered','1989-10-13 00:00:00','1989-07-29 00:00:00',107.83,712,1541,73.5,'Intermediate' from dual union all
select 20156,'020156','1989-057C','KDU','KDU',62334,'Reentered','1989-08-10 00:00:00','1989-08-03 00:00:00',89.14,228,236,82.28,'Intermediate' from dual union all
select 20157,'020157','1989-057D','Blenda','Sunshade',62334,'Reentered','1989-08-04 00:00:00','1989-08-03 00:00:00',88.73,209,215,82.3,'Intermediate' from dual union all
select 20158,'020158','1989-057E','PTDU cover','KDU part',62334,'Reentered','1989-08-06 00:00:00','1989-08-04 00:00:00',90.87,229,405,82.3,'Intermediate' from dual union all
select 20159,'020159','1989-057F','KDU part','KDU part',62334,'Reentered','1989-08-05 00:00:00','1989-08-04 00:00:00',88.14,170,195,82.3,'Intermediate' from dual union all
select 2016,'002016','1966-013A','Diapason D-1A','Diapason D-1A',13420,'In Earth orbit','','1966-10-31 00:00:00',118.61,502,2735,34.06,'Intermediate' from dual union all
select 20160,'020160','1989-055C','Pion','Pion-3',62332,'Reentered','1989-09-19 00:00:00','1989-08-07 00:00:00',89.75,254,271,82.56,'Intermediate' from dual union all
select 20161,'020161','1989-055D','Pion','Pion-4',62332,'Reentered','1989-09-19 00:00:00','1989-08-07 00:00:00',89.75,254,271,82.56,'Intermediate' from dual union all
select 20162,'020162','1989-055E','KDU part','KDU part',62332,'Reentered','1989-08-09 00:00:00','1989-08-08 00:00:00',90.09,247,311,82.63,'Intermediate' from dual union all
select 20163,'020163','1989-055F','KDU','KDU',62332,'Reentered','1989-08-27 00:00:00','1989-08-08 00:00:00',89.72,252,270,82.56,'Intermediate' from dual union all
select 20164,'020164','1989-061A','Columbia (STS-28R)','OV-102',62338,'Landed','1989-08-13 00:00:00','1989-08-10 00:00:00',90.54,294,308,57,'Intermediate' from dual union all
select 20165,'020165','1989-055G','Blenda','Sunshade',62332,'Reentered','1989-08-13 00:00:00','1989-08-08 00:00:00',91.56,261,442,82.56,'Intermediate' from dual union all
select 20166,'020166','1989-055H','KDU part','KDU part',62332,'Reentered','1989-08-12 00:00:00','1989-08-08 00:00:00',89.48,234,264,82.56,'Intermediate' from dual union all
select 20167,'020167','1989-061B','USA 40','QUASAR 7',62338,'In Earth orbit','','1989-12-02 00:00:00',718.69,1000,39400,63,'Molniya' from dual;
insert into SATELLITE
select 20168,'020168','1989-062A','TV-SAT 2','TV-SAT 2',62339,'In Earth orbit','','1989-09-08 00:00:00',1436.06,35771,35801,.11,'Stationary' from dual union all
select 20169,'020169','1989-062B','Hipparcos','Hipparcos',62339,'In Earth orbit','','1990-08-07 00:00:00',639.19,514,35890,7.33,'GEO Transfer' from dual union all
select 2017,'002017','1966-013B','P064','P064/Equip. bay',13420,'In Earth orbit','','1966-11-02 00:00:00',118.68,503,2741,34.05,'Intermediate' from dual union all
select 20170,'020170','1989-062C','Ariane H10 V33','Ariane H10 S/N T405',62339,'In Earth orbit','','1989-09-08 00:00:00',637.11,371,35926,7.28,'GEO Transfer' from dual union all
select 20171,'020171','1989-062D','Spelda 10 S/N UV4','Spelda 10',62339,'Reentered','1992-03-08 00:00:00','1992-03-08 00:00:00',90.9,136,502,7.19,'Equatorial' from dual union all
select 20172,'020172','1989-061C','USA 41','USA-41',62338,'Reentered','1990-01-01 00:00:00','1989-08-09 00:00:00',90.55,295,308,57.1,'Intermediate' from dual union all
select 20173,'020173','1986-019VM','deb Ariane V16','',59212,'Reentered','1990-07-26 00:00:00','1990-07-24 00:00:00',90.55,270,333,98.58,'Sun-Synch' from dual union all
select 20174,'020174','61 Omicron 298','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-07 00:00:00',109.14,793,1581,67.06,'Intermediate' from dual union all
select 20175,'020175','1989-063A','Resurs-F','Resurs-F2 17F42 No. 4',62340,'Landed','1989-09-14 00:00:00','1989-08-22 00:00:00',89.8,262,268,82.31,'Intermediate' from dual union all
select 20176,'020176','1989-063B','11S510','11S510',62340,'Reentered','1989-08-16 00:00:00','1989-08-16 00:00:00',87.14,127,139,82.3,'Intermediate' from dual union all
select 20177,'020177','1989-060C','Blenda','Sunshade',62337,'Reentered','1989-08-18 00:00:00','1989-08-16 00:00:00',91.03,225,426,82.53,'Intermediate' from dual union all
select 20178,'020178','1989-060D','KDU part','KDU part',62337,'Reentered','1989-08-17 00:00:00','1989-08-16 00:00:00',88.86,215,222,82.58,'Intermediate' from dual union all
select 20179,'020179','1989-060E','KDU part','KDU part',62337,'Reentered','1989-08-17 00:00:00','1989-08-16 00:00:00',88.86,215,222,82.58,'Intermediate' from dual union all
select 2018,'002018','1966-013C','D-1A despin weight','',13420,'Reentered','1967-12-05 00:00:00','1966-11-07 00:00:00',115.25,417,2517,34.09,'Intermediate' from dual union all
select 20180,'020180','1989-060F','KDU','KDU',62337,'Reentered','1989-08-22 00:00:00','1989-08-20 00:00:00',88.37,192,196,82.55,'Intermediate' from dual union all
select 20181,'020181','1988-113U','Kosmos-1985 SS 17','ESO',61362,'Reentered','1989-10-02 00:00:00','1989-08-17 00:00:00',94.76,498,516,73.55,'Intermediate' from dual union all
select 20182,'020182','1988-113V','Kosmos-1985 SS 18','ESO',61362,'Reentered','1989-10-02 00:00:00','1989-08-17 00:00:00',94.75,500,513,73.56,'Intermediate' from dual union all
select 20183,'020183','1988-113W','Kosmos-1985 SS 19','ESO',61362,'Reentered','1989-10-04 00:00:00','1989-08-17 00:00:00',94.75,499,513,73.55,'Intermediate' from dual union all
select 20184,'020184','1988-113X','Kosmos-1985 SS 20','ESO',61362,'Reentered','1989-10-01 00:00:00','1989-08-17 00:00:00',94.74,500,512,73.55,'Intermediate' from dual union all
select 20185,'020185','1989-064A','GPS II-3','Navstar GPS SVN 16',62341,'In Earth orbit','','1989-09-19 00:00:00',717.86,20113,20246,54.92,'Medium' from dual union all
select 20186,'020186','1989-064B','Delta 186','Delta SSPS AJ10-118K',62341,'Reentered','1989-09-09 00:00:00','1989-08-30 00:00:00',91.78,176,548,30.94,'Intermediate' from dual union all
select 20187,'020187','1989-064C','Star 48B','Star 48B S/N 10052-18',62341,'Reentered','1990-06-10 00:00:00','1989-09-17 00:00:00',342.89,138,19562,37.73,'Highly Ellip' from dual union all
select 20188,'020188','1989-065A','Kosmos-2036','Oblik No. 809',62342,'Landed','1989-09-05 00:00:00','1989-08-24 00:00:00',89.77,230,297,62.82,'Intermediate' from dual union all
select 20189,'020189','1989-065B','11S510','11S510',62342,'Reentered','1989-08-29 00:00:00','1989-08-24 00:00:00',89.32,237,245,62.8,'Intermediate' from dual union all
select 2019,'002019','1966-014A','Kosmos-109','Zenit-4 No. 14',13421,'Landed','1966-02-27 00:00:00','1966-02-21 00:00:00',89.42,204,288,64.93,'Intermediate' from dual union all
select 20190,'020190','1989-065C','Sensor cover','Sensor cover',62342,'Reentered','1989-08-23 00:00:00','1989-08-23 00:00:00',88.63,193,220,62.8,'Intermediate' from dual union all
select 20191,'020191','1989-066A','Progress M','Progress 7K-TGM No. 201',62343,'Deorbited','1989-12-01 00:00:00','1989-09-23 00:00:00',92.25,376,394,51.62,'Intermediate' from dual union all
select 20192,'020192','1989-066B','11S510','11S510',62343,'Reentered','1989-08-24 00:00:00','1989-08-23 00:00:00',88.07,170,188,51.63,'Intermediate' from dual union all
select 20193,'020193','1989-067A','Marcopolo 1','BSB R1',62344,'In Earth orbit','','1989-09-27 00:00:00',1436.24,35780,35799,.09,'Stationary' from dual union all
select 20194,'020194','1989-067B','Delta 187','Delta SSPS AJ10-118K',62344,'Reentered','1989-10-22 00:00:00','1989-09-04 00:00:00',92.17,285,477,26.71,'Intermediate' from dual union all
select 20195,'020195','1989-067C','Star 48B','Star 48B S/N 10047-18',62344,'Reentered','1990-10-14 00:00:00','1989-10-25 00:00:00',644.51,271,36404,23.32,'GEO Transfer' from dual union all
select 20196,'020196','1989-068A','Kosmos-2037','Musson No. 20',62345,'In Earth orbit','','1989-09-28 00:00:00',116.08,1484,1525,73.57,'Intermediate' from dual union all
select 20197,'020197','1989-068B','S5M','S5M',62345,'In Earth orbit','','1989-09-28 00:00:00',116.06,1484,1523,73.57,'Intermediate' from dual union all
select 20198,'020198','1987-020DG','deb Kosmos-1823','',60110,'Reentered','1991-03-30 00:00:00','1991-03-30 00:00:00',94.54,460,533,73.41,'Intermediate' from dual union all
select 20199,'020199','1987-020DH','deb Kosmos-1823','',60110,'Reentered','1990-08-25 00:00:00','1990-08-25 00:00:00',94.82,456,564,73.26,'Intermediate' from dual union all
select 202,'000202','61 A Eta 1','Transit 4B','Transit 4B',7253,'In Earth orbit','','1961-12-18 00:00:00',105.77,953,1107,32.43,'Intermediate' from dual union all
select 2020,'002020','1966-013D','D-1A despin weight','',13420,'Reentered','1967-09-11 00:00:00','1966-11-06 00:00:00',113.65,408,2380,34.08,'Intermediate' from dual union all
select 20200,'020200','1977-121BX','deb Kosmos-970','',42249,'Reentered','2002-10-10 00:00:00','1993-01-04 00:00:00',96.67,507,692,65.02,'Intermediate' from dual union all
select 20201,'020201','1987-020DJ','deb Kosmos-1823','',60110,'Reentered','1990-07-17 00:00:00','1990-07-16 00:00:00',93.41,398,484,73.62,'Intermediate' from dual union all
select 20202,'020202','1989-069A','DSCS II F-15','DSCS II E-15',62346,'In Earth orbit','','1989-12-01 00:00:00',1435.5,35750,35800,0,'Stationary' from dual union all
select 20203,'020203','1989-069B','DSCS III A-2','DSCS III A-2',62346,'In Earth orbit','','1989-12-01 00:00:00',1435.5,35750,35800,0,'Stationary' from dual union all
select 20204,'020204','1989-069C','Titan 34D-2 Stage 2','LR91AJ11 34D-2',62346,'Reentered','1989-09-24 00:00:00','1989-09-11 00:00:00',95.46,150,932,28.6,'Intermediate' from dual union all
select 20205,'020205','1989-069D','Transtage D-2','Transtage D-2',62346,'In Earth orbit','','1989-12-01 00:00:00',1431.67,35600,35800,0,'Drift GEO' from dual union all
select 20206,'020206','1989-056D','deb Kosmos-2031','',62333,'Reentered','1989-09-04 00:00:00','1989-09-04 00:00:00',88.52,175,228,50.58,'Intermediate' from dual union all
select 20207,'020207','1989-056E','deb Kosmos-2031','',62333,'Reentered','1989-09-04 00:00:00','1989-09-04 00:00:00',88.71,156,265,50.66,'Intermediate' from dual union all
select 20208,'020208','1989-056F','deb Kosmos-2031','',62333,'Reentered','1989-09-04 00:00:00','1989-09-03 00:00:00',89.37,208,279,50.41,'Intermediate' from dual union all
select 20209,'020209','1989-056G','deb Kosmos-2031','',62333,'Reentered','1989-09-04 00:00:00','1989-09-04 00:00:00',89.53,237,266,50.62,'Intermediate' from dual union all
select 2021,'002021','1966-013E','deb Diapason','',13420,'Reentered','1967-08-25 00:00:00','1966-11-06 00:00:00',113.95,413,2402,34.09,'Intermediate' from dual union all
select 20210,'020210','1989-056H','deb Kosmos-2031','',62333,'Reentered','1989-09-04 00:00:00','1989-09-11 00:00:00',90.59,236,371,50.45,'Intermediate' from dual union all
select 20211,'020211','1989-056J','deb Kosmos-2031','',62333,'Reentered','1989-09-04 00:00:00','1989-09-04 00:00:00',90.46,212,382,50.54,'Intermediate' from dual union all
select 20212,'020212','1989-056K','deb Kosmos-2031','',62333,'Reentered','1989-09-04 00:00:00','1989-09-04 00:00:00',90.43,238,354,50.65,'Intermediate' from dual union all
select 20213,'020213','1989-056L','deb Kosmos-2031','',62333,'Reentered','1989-09-03 00:00:00','1989-09-03 00:00:00',90.24,176,397,50.59,'Intermediate' from dual union all
select 20214,'020214','1989-065D','KDU','KDU',62342,'Reentered','1989-09-10 00:00:00','1989-09-10 00:00:00',87.14,130,136,62.81,'Intermediate' from dual union all
select 20215,'020215','1989-065E','KDU part','KDU part',62342,'Reentered','1989-09-05 00:00:00','1989-09-05 00:00:00',88,168,183,62.82,'Intermediate' from dual union all
select 20216,'020216','1989-065F','Blenda','Sunshade',62342,'Reentered','1989-09-05 00:00:00','1989-09-05 00:00:00',88.15,176,189,62.82,'Intermediate' from dual union all
select 20217,'020217','1989-070A','Himawari 4','GMS 4',62347,'In Earth orbit','','1989-10-06 00:00:00',1436.06,35719,35853,1.6,'Stationary' from dual union all
select 20218,'020218','1989-071A','Soyuz TM-8','Soyuz 7K-STM No. 58',62348,'Landed','1990-02-19 00:00:00','1989-10-07 00:00:00',92.17,373,388,51.62,'Intermediate' from dual union all
select 20219,'020219','1989-071B','11S510','11S510',62348,'Reentered','1989-09-07 00:00:00','1989-09-06 00:00:00',88.33,188,195,51.72,'Intermediate' from dual union all
select 2022,'002022','1966-014B','11A57I','11A57I',13421,'Reentered','1966-02-27 00:00:00','1966-02-22 00:00:00',89.03,198,256,64.95,'Intermediate' from dual union all
select 20220,'020220','1989-072A','USA 45','P-11 No. 5104',62349,'Reentered','1989-09-13 00:00:00','1989-09-06 00:00:00',89.35,185,300,85,'Polar' from dual union all
select 20221,'020221','1989-072B','Titan 23G-2 Stage 2','LR91AJ5 23G-2',62349,'Reentered','1989-09-06 00:00:00','1989-09-06 00:00:00',89.35,185,300,85,'Polar' from dual union all
select 20222,'020222','1989-073A','Resurs-F','Resurs-F1 14F43 No. 48',62350,'Landed','1989-09-22 00:00:00','1989-09-13 00:00:00',89.79,259,270,82.31,'Intermediate' from dual union all
select 20223,'020223','1989-073B','11S510','11S510',62350,'Reentered','1989-09-08 00:00:00','1989-09-07 00:00:00',87.45,138,159,82.31,'Intermediate' from dual union all
select 20224,'020224','1964-006AD','deb Elektron-1','',10292,'Reentered','2001-09-17 00:00:00','1993-08-24 00:00:00',115.24,369,2564,60.74,'Intermediate' from dual union all
select 20225,'020225','1987-020DK','deb Kosmos-1823','',60110,'Reentered','1991-10-12 00:00:00','1991-10-08 00:00:00',96.67,526,673,73.62,'Intermediate' from dual union all
select 20226,'020226','1987-020DL','deb Kosmos-1823','',60110,'Reentered','1991-02-17 00:00:00','1991-02-16 00:00:00',94.92,494,536,72.7,'Intermediate' from dual union all
select 20227,'020227','1987-020DM','deb Kosmos-1823','',60110,'Reentered','1999-07-12 00:00:00','1993-08-21 00:00:00',108.97,1043,1316,73.29,'Intermediate' from dual union all
select 20228,'020228','1987-020DN','deb Kosmos-1823','',60110,'Reentered','1989-11-01 00:00:00','1989-10-02 00:00:00',106.64,820,1322,73.22,'Intermediate' from dual union all
select 20229,'020229','1989-053C','deb Ariane','Olympus solar array cabl',62330,'In Earth orbit','','1993-03-29 00:00:00',636.56,423,35846,6.61,'GEO Transfer' from dual union all
select 2023,'002023','1966-013F','deb Diapason','',13420,'Reentered','1991-07-16 00:00:00','1966-10-21 00:00:00',117.65,485,2666,34.02,'Intermediate' from dual union all
select 20230,'020230','1989-070B','UM129A','UM129A',62347,'In Earth orbit','','1989-10-08 00:00:00',668.75,216,37692,28.58,'GEO Transfer' from dual union all
select 20231,'020231','1989-063C','Blenda','Sunshade',62340,'Reentered','1989-11-19 00:00:00','1989-09-15 00:00:00',88.47,192,206,82.28,'Intermediate' from dual union all
select 20232,'020232','1989-074A','Kosmos-2038','Strela-3',62351,'In Earth orbit','','1989-11-03 00:00:00',113.81,1393,1410,82.57,'Intermediate' from dual union all
select 20233,'020233','1989-074B','Kosmos-2039','Strela-3',62351,'In Earth orbit','','1993-04-29 00:00:00',113.72,1386,1409,82.58,'Intermediate' from dual union all
select 20234,'020234','1989-074C','Kosmos-2040','Strela-3',62351,'In Earth orbit','','1989-10-26 00:00:00',114.06,1409,1417,82.59,'Intermediate' from dual union all
select 20235,'020235','1989-074D','Kosmos-2041','Strela-3',62351,'In Earth orbit','','1989-10-19 00:00:00',113.87,1397,1411,82.58,'Intermediate' from dual union all
select 20236,'020236','1989-074E','Kosmos-2042','Strela-3',62351,'In Earth orbit','','1989-11-03 00:00:00',113.92,1403,1410,82.58,'Intermediate' from dual union all
select 20237,'020237','1989-074F','Kosmos-2043','Strela-3',62351,'In Earth orbit','','1989-10-19 00:00:00',114,1408,1412,82.58,'Intermediate' from dual union all
select 20238,'020238','1989-074G','S5M','S5M',62351,'In Earth orbit','','1989-11-02 00:00:00',114.71,1410,1475,82.59,'Intermediate' from dual union all
select 20239,'020239','1989-063D','Solnechnaya Batareya','Solar panel module',62340,'Reentered','1989-09-16 00:00:00','1989-09-14 00:00:00',91.82,249,479,82.32,'Intermediate' from dual union all
select 2024,'002024','1966-014C','deb Kosmos-109','',13421,'Reentered','1966-02-21 00:00:00','1966-02-19 00:00:00',88.73,188,236,64.9,'Intermediate' from dual union all
select 20240,'020240','1989-063E','KDU part','KDU part',62340,'Reentered','1989-09-19 00:00:00','1989-09-18 00:00:00',89.51,204,297,82.28,'Intermediate' from dual union all
select 20241,'020241','1989-063F','KDU','KDU',62340,'Reentered','1989-09-16 00:00:00','1989-09-15 00:00:00',89.35,235,250,82.3,'Intermediate' from dual union all
select 20242,'020242','1989-075A','Kosmos-2044','Bion No. 9',62352,'Landed','1989-09-29 00:00:00','1989-09-17 00:00:00',89.15,204,261,82.33,'Intermediate' from dual union all
select 20243,'020243','1989-075B','11S510','11S510',62352,'Reentered','1989-09-18 00:00:00','1989-09-18 00:00:00',87.02,119,135,82.3,'Intermediate' from dual union all
select 20244,'020244','1989-076A','Kosmos-2045','Oblik',62353,'Landed','1989-10-02 00:00:00','1989-09-24 00:00:00',90.76,234,390,69.98,'Intermediate' from dual union all
select 20245,'020245','1989-076B','11S510','11S510',62353,'Reentered','1989-09-28 00:00:00','1989-09-25 00:00:00',89,193,257,69.99,'Intermediate' from dual union all
select 20246,'020246','1989-076C','Sensor cover','Sensor cover',62353,'Reentered','1989-09-23 00:00:00','1989-09-22 00:00:00',88.64,182,233,70.01,'Intermediate' from dual union all
select 20247,'020247','1989-076D','Sensor cover','Sensor cover',62353,'Reentered','1989-09-23 00:00:00','1989-09-23 00:00:00',88.8,185,246,69.96,'Intermediate' from dual union all
select 20248,'020248','1989-073C','KDU','KDU',62350,'Reentered','1989-10-12 00:00:00','1989-09-22 00:00:00',89.77,258,269,82.3,'Intermediate' from dual union all
select 20249,'020249','1989-073D','KDU part','KDU part',62350,'Reentered','1989-09-26 00:00:00','1989-09-22 00:00:00',89.48,238,260,82.31,'Intermediate' from dual union all
select 2025,'002025','1966-012D','deb OPS 3031','',13419,'Reentered','1966-02-17 00:00:00','1966-02-16 00:00:00',88.86,147,290,96.35,'Sun-Synch' from dual union all
select 20250,'020250','1989-073E','Blenda','Sunshade',62350,'Reentered','1989-09-28 00:00:00','1989-09-25 00:00:00',91.02,246,404,82.32,'Intermediate' from dual union all
select 20251,'020251','1989-073F','KDU part','KDU part',62350,'Reentered','1989-09-25 00:00:00','1989-09-23 00:00:00',89.49,245,254,82.3,'Intermediate' from dual union all
select 20252,'020252','1989-073G','KDU part','KDU part',62350,'Reentered','1989-09-24 00:00:00','1989-09-23 00:00:00',91.59,253,453,83.2,'Intermediate' from dual union all
select 20253,'020253','1989-077A','FLTSATCOM F8','FLTSATCOM F8',62354,'In Earth orbit','','1989-10-25 00:00:00',1435.93,35781,35786,4.98,'Inclined GEO' from dual union all
select 20254,'020254','1989-077B','Centaur AC-68','Centaur D-1AR AC-68',62354,'Reentered','1991-05-09 00:00:00','1989-10-27 00:00:00',595.04,166,33947,28.46,'Highly Ellip' from dual union all
select 20255,'020255','1989-078A','Molniya-1','Molniya-1 No. 69',62355,'Reentered','2000-11-11 00:00:00','1989-10-27 00:00:00',717.66,648,39701,62.79,'Molniya' from dual union all
select 20256,'020256','1989-078B','11S510','11S510',62355,'Reentered','1989-10-26 00:00:00','1989-10-04 00:00:00',92.24,208,561,62.83,'Intermediate' from dual union all
select 20257,'020257','1989-078C','BOZ','BOZ',62355,'Reentered','1989-10-24 00:00:00','1989-10-04 00:00:00',92.33,201,577,62.83,'Intermediate' from dual;
insert into SATELLITE
select 20258,'020258','1989-078D','Blok-ML','Blok-ML No. 181',62355,'Reentered','2003-10-06 00:00:00','1993-03-23 00:00:00',698.27,659,38729,64.58,'GEO Transfer' from dual union all
select 20259,'020259','1989-079A','Kosmos-2046','US-PU',62356,'Reentered','1991-04-16 00:00:00','1989-10-28 00:00:00',92.77,403,418,65.03,'Intermediate' from dual union all
select 2026,'002026','1966-012E','deb OPS 3031','',13419,'Reentered','1966-02-17 00:00:00','1966-02-16 00:00:00',89.42,180,312,96.6,'Sun-Synch' from dual union all
select 20260,'020260','1989-079B','11S692','11S692',62356,'Reentered','1989-09-28 00:00:00','1989-09-27 00:00:00',88.22,107,266,65.01,'Intermediate' from dual union all
select 20261,'020261','1989-080A','Interkosmos-24','AUOS-Z No. 201',62357,'In Earth orbit','','1989-10-28 00:00:00',115.87,500,2490,82.6,'Intermediate' from dual union all
select 20262,'020262','1989-080C','S5M','S5M',62357,'In Earth orbit','','1989-10-28 00:00:00',115.87,500,2490,82.6,'Intermediate' from dual union all
select 20263,'020263','1989-081A','Gorizont','Gorizont No. 31L',62358,'In Earth orbit','','1989-10-28 00:00:00',1436.06,35782,35790,1.36,'Stationary' from dual union all
select 20264,'020264','1989-081B','8S812','8S812',62358,'Reentered','1989-09-30 00:00:00','1989-09-29 00:00:00',88.23,181,193,51.62,'Intermediate' from dual union all
select 20265,'020265','1989-081C','Perekhodnik','Sredniy perekhodnik',62358,'Reentered','1989-09-29 00:00:00','1989-09-29 00:00:00',87.91,163,180,51.6,'Intermediate' from dual union all
select 20266,'020266','1989-081D','Blok DM2 No. 27L','11S861 No. 27L',62358,'In Earth orbit','','1989-10-28 00:00:00',1431.34,35556,35831,1.39,'Drift GEO' from dual union all
select 20267,'020267','1989-075C','KB electropitaniya','KBE',62352,'Reentered','1989-10-01 00:00:00','1989-10-01 00:00:00',87.31,128,155,82.31,'Intermediate' from dual union all
select 20268,'020268','1989-075D','KBE part','KBE part',62352,'Reentered','1989-09-30 00:00:00','1989-09-29 00:00:00',87.77,150,179,82.33,'Intermediate' from dual union all
select 20269,'020269','1989-075E','KBE part','KBE part',62352,'Reentered','1989-09-29 00:00:00','1989-09-29 00:00:00',89.55,181,324,82.27,'Intermediate' from dual union all
select 2027,'002027','1966-012F','deb OPS 3031','',13419,'Reentered','1966-02-17 00:00:00','1966-02-16 00:00:00',89.18,186,281,95.9,'Sun-Synch' from dual union all
select 20270,'020270','1989-081E','SOZ','SOZ',62358,'Reentered','1990-07-22 00:00:00','1989-11-12 00:00:00',631.35,236,35765,47.27,'GEO Transfer' from dual union all
select 20271,'020271','1989-081F','SOZ','SOZ',62358,'Reentered','1991-09-28 00:00:00','1989-11-14 00:00:00',633.31,290,35812,47.3,'GEO Transfer' from dual union all
select 20272,'020272','1986-067AE','Kosmos-1776 SS 23','ESO',59260,'Reentered','1989-10-05 00:00:00','1989-09-29 00:00:00',92.15,371,389,74,'Intermediate' from dual union all
select 20273,'020273','1986-067AF','Kosmos-1776 SS 24','ESO',59260,'Reentered','1989-10-05 00:00:00','1989-09-29 00:00:00',91.92,359,379,74,'Intermediate' from dual union all
select 20274,'020274','1989-076E','KDU part','KDU part',62353,'Reentered','1989-10-04 00:00:00','1989-10-03 00:00:00',88.63,200,214,69.98,'Intermediate' from dual union all
select 20275,'020275','1989-076F','Blenda','Sunshade',62353,'Reentered','1989-10-05 00:00:00','1989-10-04 00:00:00',90.4,233,356,70,'Intermediate' from dual union all
select 20276,'020276','1989-076G','KDU','KDU',62353,'Reentered','1989-10-09 00:00:00','1989-10-08 00:00:00',88.14,182,183,69.98,'Intermediate' from dual union all
select 20277,'020277','1989-076H','KDU part','KDU part',62353,'Reentered','1989-10-04 00:00:00','1989-10-03 00:00:00',90.71,302,317,70,'Intermediate' from dual union all
select 20278,'020278','1989-076J','KDU part','KDU part',62353,'Reentered','1989-10-03 00:00:00','1989-10-03 00:00:00',88.51,191,211,70,'Intermediate' from dual union all
select 20279,'020279','1989-082A','Kosmos-2047','Kobal''t No. 524',62359,'Landed','1989-11-21 00:00:00','1989-10-10 00:00:00',88.94,169,275,67.13,'Intermediate' from dual union all
select 2028,'002028','1966-012G','deb OPS 3031','',13419,'Reentered','1966-02-18 00:00:00','1966-02-16 00:00:00',88.83,198,235,96.1,'Sun-Synch' from dual union all
select 20280,'020280','1989-082B','11S510','11S510',62359,'Reentered','1989-10-06 00:00:00','1989-10-05 00:00:00',88.11,141,221,67.12,'Intermediate' from dual union all
select 20281,'020281','1989-080B','Magion-2','S2-AK Magion-2',62357,'In Earth orbit','','1989-10-30 00:00:00',115.86,499,2490,82.6,'Intermediate' from dual union all
select 20282,'020282','1989-080D','deb S5M','',62357,'Reentered','2017-11-23 00:00:00','1989-10-16 00:00:00',115.42,498,2451,82.6,'Intermediate' from dual union all
select 20283,'020283','1976-067BW','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-02 00:00:00',116.3,989,2040,65.73,'Intermediate' from dual union all
select 20284,'020284','1985-094X','deb Kosmos-1695','',58317,'In Earth orbit','','1993-08-17 00:00:00',117.35,1388,1736,83.21,'Intermediate' from dual union all
select 20285,'020285','1987-020DP','deb Kosmos-1823','',60110,'In Earth orbit','','1993-08-06 00:00:00',117.15,1482,1623,73.65,'Intermediate' from dual union all
select 20286,'020286','1988-113Y','Kosmos-1985 SS 21','ESO',61362,'Reentered','1989-11-14 00:00:00','1989-10-14 00:00:00',94.63,496,506,73.54,'Intermediate' from dual union all
select 20287,'020287','1988-113Z','Kosmos-1985 SS 22','ESO',61362,'Reentered','1989-11-13 00:00:00','1989-10-14 00:00:00',94.59,495,503,73.54,'Intermediate' from dual union all
select 20288,'020288','1988-113AA','Kosmos-1985 SS 23','ESO',61362,'Reentered','1989-11-12 00:00:00','1989-10-14 00:00:00',94.6,495,503,73.54,'Intermediate' from dual union all
select 20289,'020289','1988-113AB','Kosmos-1985 SS 24','ESO',61362,'Reentered','1989-11-14 00:00:00','1989-10-14 00:00:00',94.56,491,504,73.55,'Intermediate' from dual union all
select 2029,'002029','1966-012H','deb OPS 3031','',13419,'Reentered','1966-02-19 00:00:00','1966-02-16 00:00:00',88.69,188,232,96.3,'Sun-Synch' from dual union all
select 20290,'020290','1988-113AC','Kosmos-1985 SS 25','ESO',61362,'Reentered','1989-11-13 00:00:00','1989-10-15 00:00:00',94.58,493,504,73.55,'Intermediate' from dual union all
select 20291,'020291','1988-113AD','Kosmos-1985 SS 26','ESO',61362,'Reentered','1989-11-13 00:00:00','1989-10-17 00:00:00',94.48,487,499,73.55,'Intermediate' from dual union all
select 20292,'020292','1989-083A','Kosmos-2048','Oblik No. 396',62360,'Landed','1989-10-26 00:00:00','1989-10-19 00:00:00',90.47,238,358,62.81,'Intermediate' from dual union all
select 20293,'020293','1989-083B','11S510','11S510',62360,'Reentered','1989-10-22 00:00:00','1989-10-19 00:00:00',89.2,231,239,62.8,'Intermediate' from dual union all
select 20294,'020294','1989-083C','Sensor cover','Sensor cover',62360,'Reentered','1989-10-18 00:00:00','1989-10-17 00:00:00',89.39,239,250,62.8,'Intermediate' from dual union all
select 20295,'020295','1989-083D','Sensor cover','Sensor cover',62360,'Reentered','1989-10-18 00:00:00','1989-10-17 00:00:00',89.5,236,264,62.83,'Intermediate' from dual union all
select 20296,'020296','1989-083E','Sensor cover','Sensor cover',62360,'Reentered','1989-10-18 00:00:00','1989-10-17 00:00:00',88.87,194,244,62.78,'Intermediate' from dual union all
select 20297,'020297','1989-084A','Atlantis (STS-34)','OV-104',62361,'Landed','1989-10-23 00:00:00','1989-10-20 00:00:00',90.84,299,333,34.32,'Intermediate' from dual union all
select 20298,'020298','1989-084B','Galileo','Galileo',62361,'Deep Space','1989-10-19 00:00:00','1989-10-18 00:00:00',689.16,438,64884,33.95,'Earth Escape' from dual union all
select 20299,'020299','1989-084C','IUS-19 SRM-1','Orbus 21',62361,'Reentered','2006-02-20 00:00:00','1993-08-19 00:00:00',428.38,179,24713,34.29,'Highly Ellip' from dual union all
select 203,'000203','61 A Epsilon 5','deb Discoverer XXXIV','',7252,'Reentered','1961-12-12 00:00:00','1961-11-15 00:00:00',95.81,225,891,82.52,'Intermediate' from dual union all
select 2030,'002030','1966-012J','deb OPS 3031','',13419,'Reentered','1966-02-19 00:00:00','1966-02-16 00:00:00',88.37,128,260,96.2,'Sun-Synch' from dual union all
select 20300,'020300','1989-084D','IUS-19 SRM-2','Orbus 6E',62361,'Deep Space','1989-10-19 00:00:00','1989-10-18 00:00:00',689.16,438,64884,33.95,'Earth Escape' from dual union all
select 20301,'020301','1988-102J','deb Kosmos- 1980','',61351,'In Earth orbit','','1991-04-30 00:00:00',101.91,843,855,71,'Intermediate' from dual union all
select 20302,'020302','1989-085A','GPS II-4','Navstar GPS SVN 19',62362,'In Earth orbit','','1989-11-21 00:00:00',717.9,20081,20280,54.76,'Medium' from dual union all
select 20303,'020303','1989-085B','Delta 188','Delta SSPS AJ10-118K',62362,'In Earth orbit','','1989-11-20 00:00:00',99.47,493,973,35.65,'Intermediate' from dual union all
select 20304,'020304','1989-085C','Star 48B','Star 48B S/N 10053-18',62362,'Reentered','1991-12-05 00:00:00','1989-11-21 00:00:00',345.26,165,19684,37.75,'Highly Ellip' from dual union all
select 20305,'020305','1989-086A','Meteor-3','Meteor-3 No. 4',62363,'In Earth orbit','','1989-11-24 00:00:00',109.42,1187,1213,82.55,'Intermediate' from dual union all
select 20306,'020306','1989-086B','S5M','S5M',62363,'In Earth orbit','','1993-07-15 00:00:00',109.4,1185,1213,82.55,'Intermediate' from dual union all
select 20307,'020307','1989-080F','deb S5M','',62357,'Reentered','2002-01-14 00:00:00','1993-08-24 00:00:00',107.18,422,1771,82.67,'Intermediate' from dual union all
select 20308,'020308','1989-080G','deb S5M','',62357,'In Earth orbit','','1993-08-03 00:00:00',114.48,541,2323,82.66,'Intermediate' from dual union all
select 20309,'020309','1976-077FQ','deb Delta 126','',39029,'In Earth orbit','','1993-08-07 00:00:00',119.43,1491,1820,102.56,'Sun-Synch' from dual union all
select 2031,'002031','1966-012K','deb OPS 3031','',13419,'Reentered','1966-02-20 00:00:00','1966-02-16 00:00:00',88.78,186,243,96.2,'Sun-Synch' from dual union all
select 20310,'020310','1989-083F','KDU','KDU',62360,'Reentered','1989-10-30 00:00:00','1989-10-26 00:00:00',88.96,197,250,62.8,'Intermediate' from dual union all
select 20311,'020311','1989-083G','Blenda','Sunshade',62360,'Reentered','1989-10-29 00:00:00','1989-10-26 00:00:00',90.92,253,387,62.82,'Intermediate' from dual union all
select 20312,'020312','1989-083H','KDU part','KDU part',62360,'Reentered','1989-10-27 00:00:00','1989-10-26 00:00:00',88.49,181,219,62.8,'Intermediate' from dual union all
select 20313,'020313','1989-083J','KDU part','KDU part',62360,'Reentered','1989-10-28 00:00:00','1989-10-27 00:00:00',90.09,225,333,62.8,'Intermediate' from dual union all
select 20314,'020314','1989-083K','KDU part','KDU part',62360,'Reentered','1989-10-27 00:00:00','1989-10-27 00:00:00',87.8,154,178,62,'Intermediate' from dual union all
select 20315,'020315','1989-087A','INTELSAT 602','INTELSAT VI F-2',62364,'In Earth orbit','','1989-11-27 00:00:00',1436.14,35726,35849,.1,'Stationary' from dual union all
select 20316,'020316','1989-087B','Ariane H10 V34','Ariane H10 S/N T406',62364,'In Earth orbit','','1989-11-28 00:00:00',632.09,301,35738,7.04,'GEO Transfer' from dual union all
select 20317,'020317','1989-070C','Star 27','Star 27 TE-M-616-5',62347,'In Earth orbit','','1989-11-01 00:00:00',1457.98,35209,37219,1.62,'Drift GEO' from dual union all
select 20318,'020318','1989-080H','deb S5M','',62357,'Reentered','2010-08-15 00:00:00','1993-08-08 00:00:00',109.71,443,1984,82.5,'Intermediate' from dual union all
select 20319,'020319','1989-046E','DSP 14 Aperture Cover','',62323,'In Earth orbit','','1989-11-30 00:00:00',1421.02,35115,35867,2.8,'Drift GEO' from dual union all
select 2032,'002032','1966-012L','deb OPS 3031','',13419,'Reentered','1966-02-17 00:00:00','1966-02-16 00:00:00',88.53,152,251,96.5,'Sun-Synch' from dual union all
select 20320,'020320','1989-088A','Kosmos-2049','Yantar''-4KS1M',62365,'Deorbited','1990-06-19 00:00:00','1989-12-17 00:00:00',89.62,230,282,64.78,'Intermediate' from dual union all
select 20321,'020321','1989-088B','11S510','11S510',62365,'Reentered','1989-11-19 00:00:00','1989-11-19 00:00:00',87.58,138,172,64.79,'Intermediate' from dual union all
select 20322,'020322','1989-089A','COBE','COBE',62366,'In Earth orbit','','1989-12-19 00:00:00',102.79,886,896,99.02,'Sun-Synch' from dual union all
select 20323,'020323','1989-089B','Delta 189','Delta SSPS AJ10-118K',62366,'In Earth orbit','','1989-12-19 00:00:00',99.97,700,815,97.07,'Sun-Synch' from dual union all
select 20324,'020324','1989-089C','COBE sensor cover','',62366,'In Earth orbit','','1993-03-29 00:00:00',102.38,861,882,98.98,'Sun-Synch' from dual union all
select 20325,'020325','1980-014C','SMM High Gain Antenna','HGA',50224,'Reentered','1989-11-23 00:00:00','1989-11-22 00:00:00',88.86,218,219,28.48,'Intermediate' from dual union all
select 20326,'020326','1970-025QE','deb Agena D','',21502,'In Earth orbit','','1993-07-01 00:00:00',103.7,861,1006,99.94,'Sun-Synch' from dual union all
select 20327,'020327','1977-121BY','deb Kosmos-970 [UNID]','',42249,'In Earth orbit','','1993-02-08 00:00:00',106.59,449,1688,65.8,'Intermediate' from dual union all
select 20328,'020328','1989-089D','COBE dewar cap','Dewar cap',62366,'In Earth orbit','','1993-03-27 00:00:00',102.72,885,890,99.01,'Sun-Synch' from dual union all
select 20329,'020329','1989-090A','Discovery (STS-33R)','OV-103',62367,'Landed','1989-11-28 00:00:00','1989-11-25 00:00:00',92.5,237,558,28.46,'Intermediate' from dual union all
select 2033,'002033','1966-012M','deb OPS 3031','',13419,'Reentered','1966-02-18 00:00:00','1966-02-16 00:00:00',88.47,143,255,96.4,'Sun-Synch' from dual union all
select 20330,'020330','1989-091A','Kosmos-2050','Oko',62368,'In Earth orbit','','1989-12-23 00:00:00',717.92,611,39751,63.02,'Molniya' from dual union all
select 20331,'020331','1989-091B','11S510','11S510',62368,'Reentered','1989-12-18 00:00:00','1989-12-01 00:00:00',91.79,205,520,62.85,'Intermediate' from dual union all
select 20332,'020332','1989-091C','BOZ','BOZ',62368,'Reentered','1989-12-10 00:00:00','1989-12-01 00:00:00',91.36,179,504,62.93,'Intermediate' from dual union all
select 20333,'020333','1989-091D','Blok-2BL','Blok-2BL No. 82',62368,'In Earth orbit','','1993-03-28 00:00:00',705.19,1891,37841,63.49,'Molniya' from dual union all
select 20334,'020334','1989-092A','Kosmos-2051','US-PM',62369,'Deorbited','1991-01-21 00:00:00','1989-12-26 00:00:00',92.78,404,418,64.91,'Intermediate' from dual union all
select 20335,'020335','1989-093A','Kvant-2','TsM-D',62370,'Reentered Attached','2001-03-23 00:00:00','1989-12-26 00:00:00',92.38,390,393,51.62,'Intermediate' from dual union all
select 20336,'020336','1989-093B','8S812','8S812',62370,'Reentered','1989-12-04 00:00:00','1989-11-28 00:00:00',89.43,207,286,51.64,'Intermediate' from dual union all
select 20337,'020337','1989-093C','TsM adapter','TsM adapter',62370,'Reentered','1989-11-27 00:00:00','1989-11-27 00:00:00',89.6,208,302,51.62,'Intermediate' from dual union all
select 20338,'020338','1989-094A','Molniya-3','Molniya-3 No. 46',62371,'Reentered','2000-05-19 00:00:00','1989-12-28 00:00:00',717.82,660,39697,62.84,'Molniya' from dual union all
select 20339,'020339','1989-094B','Blok-ML','Blok-ML No. 336',62371,'Reentered','2000-07-04 00:00:00','1993-03-21 00:00:00',732.14,563,40498,64.63,'GEO Transfer' from dual union all
select 2034,'002034','1966-012N','deb OPS 3031','',13419,'Reentered','1966-02-19 00:00:00','1966-02-16 00:00:00',88.42,142,250,96.4,'Sun-Synch' from dual union all
select 20340,'020340','1989-094C','11S510','11S510',62371,'Reentered','1989-12-22 00:00:00','1989-12-07 00:00:00',91.4,202,485,62.81,'Intermediate' from dual union all
select 20341,'020341','1989-094D','BOZ','BOZ',62371,'Reentered','1989-12-20 00:00:00','1989-12-07 00:00:00',91.75,184,537,62.83,'Intermediate' from dual union all
select 20342,'020342','1988-113AE','Kosmos-1985 SS 27','ESO',61362,'Reentered','1990-01-01 00:00:00','1989-11-29 00:00:00',94.53,488,503,73.55,'Intermediate' from dual union all
select 20343,'020343','1988-113AF','Kosmos-1985 SS 28','ESO',61362,'Reentered','1989-12-31 00:00:00','1989-11-29 00:00:00',94.53,488,503,73.54,'Intermediate' from dual union all
select 20344,'020344','1989-061D','USA 40 PKM','TU-882',62338,'In Earth orbit','','1990-12-11 00:00:00',183.85,504,8160,57,'Medium' from dual union all
select 20345,'020345','1988-113AG','Kosmos-1985 SS 29','ESO',61362,'Reentered','1989-12-31 00:00:00','1989-11-29 00:00:00',94.51,487,503,73.54,'Intermediate' from dual union all
select 20346,'020346','1988-113AH','Kosmos-1985 SS 30','ESO',61362,'Reentered','1989-12-31 00:00:00','1989-11-30 00:00:00',94.48,486,500,73.55,'Intermediate' from dual union all
select 20347,'020347','1989-080J','deb S5M','',62357,'Reentered','2015-08-08 00:00:00','1989-11-30 00:00:00',114.98,500,2409,82.62,'Intermediate' from dual;
insert into SATELLITE
select 20348,'020348','1972-049C','deb Meteor','',27981,'In Earth orbit','','1993-03-30 00:00:00',102.84,856,930,81.24,'Intermediate' from dual union all
select 20349,'020349','1976-077FR','deb Delta 126','',39029,'In Earth orbit','','1993-06-13 00:00:00',127.33,1482,2530,102.56,'Medium' from dual union all
select 2035,'002035','1966-012P','deb OPS 3031','',13419,'Reentered','1966-02-17 00:00:00','1966-02-16 00:00:00',88.25,143,233,96.3,'Sun-Synch' from dual union all
select 20350,'020350','1989-095A','Kosmos-2052','Kobal''t No. 585',62372,'Landed','1990-01-24 00:00:00','1989-12-07 00:00:00',89.58,174,334,67.15,'Intermediate' from dual union all
select 20351,'020351','1989-095B','11S510','11S510',62372,'Reentered','1989-12-03 00:00:00','1989-12-03 00:00:00',87.18,116,154,67.14,'Intermediate' from dual union all
select 20352,'020352','1989-096A','Granat','1AS No. 1',62373,'Deep Space','1989-12-02 00:00:00','1990-01-04 00:00:00',5901.99,1813,201796,53.29,'Deep Space' from dual union all
select 20353,'020353','1989-096B','SOZ','SOZ',62373,'Reentered','1991-03-09 00:00:00','1991-01-24 00:00:00',94.42,202,779,51.49,'Intermediate' from dual union all
select 20354,'020354','1989-096C','Blok D-1 No. 10L','11S824M No. 10L',62373,'Deep Space','1989-12-02 00:00:00','1991-09-16 00:00:00',5774.47,19964,180517,81.14,'Deep Space' from dual union all
select 20355,'020355','1989-090B','USA 48','ORION 2',62367,'In Earth orbit','','2003-05-04 00:00:00',1436.24,34693,36885,8.56,'Inclined GEO' from dual union all
select 20356,'020356','1989-090C','IUS-5 SRM-1','Orbus 21',62367,'In Earth orbit','','1990-11-23 00:00:00',628.5,254,35599,28.4,'GEO Transfer' from dual union all
select 20357,'020357','1989-090D','IUS-5 SRM-2','Orbus 6E',62367,'In Earth orbit','','2003-05-04 00:00:00',1446.88,34593,37402,8.6,'Inclined Drift' from dual union all
select 20358,'020358','1989-096D','SOZ','SOZ',62373,'Reentered','1991-04-15 00:00:00','1990-01-03 00:00:00',106.75,225,1928,51.54,'Intermediate' from dual union all
select 20359,'020359','1989-080K','deb S5M','',62357,'In Earth orbit','','1993-08-07 00:00:00',110.82,489,2040,82.56,'Intermediate' from dual union all
select 2036,'002036','1966-012Q','deb OPS 3031','',13419,'Reentered','1966-02-18 00:00:00','1966-02-16 00:00:00',88.21,161,211,96.5,'Sun-Synch' from dual union all
select 20360,'020360','1970-025QF','deb Agena D','',21502,'In Earth orbit','','1993-08-08 00:00:00',107.23,1064,1133,100.47,'Sun-Synch' from dual union all
select 20361,'020361','1989-097A','GPS II-5','Navstar GPS SVN 17',62374,'In Earth orbit','','1990-01-11 00:00:00',718,20009,20357,54.95,'Medium' from dual union all
select 20362,'020362','1989-097B','Delta 190','Delta SSPS AJ10-118K',62374,'In Earth orbit','','1990-01-11 00:00:00',99.3,490,961,35.65,'Intermediate' from dual union all
select 20363,'020363','1989-097C','Star 48B','Star 48B S/N 10054-18',62374,'Reentered','1990-08-09 00:00:00','1990-01-10 00:00:00',344.64,140,19670,37.69,'Highly Ellip' from dual union all
select 20364,'020364','1979-053D','DSP 11 Aperture Cover','',47537,'In Earth orbit','','1989-12-31 00:00:00',1449.19,34730,37355,7,'Inclined Drift' from dual union all
select 20365,'020365','1988-065N','Kosmos-1960 SS 7','ESO',61314,'Reentered','1990-01-04 00:00:00','1989-12-15 00:00:00',92.25,380,390,65.86,'Intermediate' from dual union all
select 20366,'020366','1988-065P','Kosmos-1960 SS 8','ESO',61314,'Reentered','1990-01-04 00:00:00','1989-12-15 00:00:00',92.25,379,391,65.8,'Intermediate' from dual union all
select 20367,'020367','1989-098A','Raduga','Gran'' No. 36L',62375,'In Earth orbit','','1990-01-15 00:00:00',1436.21,35766,35812,1.44,'Stationary' from dual union all
select 20368,'020368','1989-098B','8S812','8S812',62375,'Reentered','1989-12-17 00:00:00','1989-12-15 00:00:00',88.28,184,194,51.64,'Intermediate' from dual union all
select 20369,'020369','1989-098C','Perekhodnik','Sredniy perekhodnik',62375,'Reentered','1989-12-16 00:00:00','1989-12-15 00:00:00',88.41,168,224,51.64,'Intermediate' from dual union all
select 2037,'002037','1966-012R','deb OPS 3031','',13419,'Reentered','1966-02-20 00:00:00','1966-02-16 00:00:00',88.16,132,235,96.5,'Sun-Synch' from dual union all
select 20370,'020370','1989-098D','Blok DM2','11S861 No. 23L',62375,'In Earth orbit','','1993-05-14 00:00:00',1471.53,36401,36554,1.59,'Drift GEO' from dual union all
select 20371,'020371','1989-098E','SOZ','SOZ',62375,'Reentered','1991-09-04 00:00:00','1990-07-06 00:00:00',624.58,156,35496,46.63,'GEO Transfer' from dual union all
select 20372,'020372','1989-098F','SOZ','SOZ',62375,'Reentered','1991-06-06 00:00:00','1990-05-27 00:00:00',613.76,141,34950,46.71,'GEO Transfer' from dual union all
select 20373,'020373','1989-099A','Progress M-2','Progress 7K-TGM No. 202',62376,'Deorbited','1990-02-09 00:00:00','1989-12-27 00:00:00',92.38,390,393,51.62,'Intermediate' from dual union all
select 20374,'020374','1989-099B','11S510','11S510',62376,'Reentered','1989-12-21 00:00:00','1989-12-21 00:00:00',87.24,137,139,51.59,'Intermediate' from dual union all
select 20375,'020375','1988-065Q','Kosmos-1960 SS 9','ESO',61314,'Reentered','1990-01-10 00:00:00','1989-12-20 00:00:00',92.32,382,394,65.82,'Intermediate' from dual union all
select 20376,'020376','1988-065R','Kosmos-1960 SS 10','ESO',61314,'Reentered','1990-01-06 00:00:00','1989-12-20 00:00:00',92.29,379,395,65.83,'Intermediate' from dual union all
select 20377,'020377','1988-065S','Kosmos-1960 SS 11','ESO',61314,'Reentered','1990-01-10 00:00:00','1989-12-20 00:00:00',92.26,381,390,65.83,'Intermediate' from dual union all
select 20378,'020378','1988-065T','Kosmos-1960 SS 12','ESO',61314,'Reentered','1990-01-07 00:00:00','1989-12-21 00:00:00',92.06,369,383,65.84,'Intermediate' from dual union all
select 20379,'020379','1988-065U','Kosmos-1960 SS 13','ESO',61314,'Reentered','1990-01-07 00:00:00','1989-12-20 00:00:00',92.09,370,384,65.82,'Intermediate' from dual union all
select 2038,'002038','1966-012S','deb OPS 3031','',13419,'Reentered','1966-02-19 00:00:00','1966-02-16 00:00:00',88.4,171,220,97.1,'Sun-Synch' from dual union all
select 20380,'020380','1988-065V','Kosmos-1960 SS 14','ESO',61314,'Reentered','1990-01-09 00:00:00','1989-12-21 00:00:00',92.19,377,387,65.81,'Intermediate' from dual union all
select 20381,'020381','1988-065W','Kosmos-1960 SS 15','ESO',61314,'Reentered','1990-01-09 00:00:00','1989-12-21 00:00:00',92.16,376,385,65.85,'Intermediate' from dual union all
select 20382,'020382','1988-065X','Kosmos-1960 SS 16','ESO',61314,'Reentered','1990-01-10 00:00:00','1989-12-21 00:00:00',92.2,378,386,65.83,'Intermediate' from dual union all
select 20383,'020383','1988-065Y','Kosmos-1960 SS 17','ESO',61314,'Reentered','1990-01-12 00:00:00','1989-12-25 00:00:00',92.03,370,379,65.81,'Intermediate' from dual union all
select 20384,'020384','1988-065Z','Kosmos-1960 SS 18','ESO',61314,'Reentered','1990-01-15 00:00:00','1989-12-25 00:00:00',92.21,376,390,65.82,'Intermediate' from dual union all
select 20385,'020385','1988-065AA','Kosmos-1960 SS 19','ESO',61314,'Reentered','1990-01-13 00:00:00','1989-12-25 00:00:00',92.05,371,380,65.82,'Intermediate' from dual union all
select 20386,'020386','1988-065AB','Kosmos-1960 SS 20','ESO',61314,'Reentered','1990-01-13 00:00:00','1989-12-25 00:00:00',92.09,375,378,65.84,'Intermediate' from dual union all
select 20387,'020387','1988-065AC','Kosmos-1960 SS 21','ESO',61314,'Reentered','1990-01-16 00:00:00','1989-12-25 00:00:00',92.21,375,391,65.82,'Intermediate' from dual union all
select 20388,'020388','1988-065AD','Kosmos-1960 SS 22','ESO',61314,'Reentered','1990-01-16 00:00:00','1989-12-25 00:00:00',92.3,380,395,65.84,'Intermediate' from dual union all
select 20389,'020389','1989-100A','Kosmos-2053','Kol''tso',62377,'Reentered','1997-09-02 00:00:00','1990-01-28 00:00:00',95.12,518,531,73.54,'Intermediate' from dual union all
select 2039,'002039','1966-012T','deb OPS 3031','',13419,'Reentered','1966-02-16 00:00:00','1966-02-16 00:00:00',88.14,145,220,96.5,'Sun-Synch' from dual union all
select 20390,'020390','1989-100B','S5M','S5M',62377,'Reentered','2003-06-05 00:00:00','1990-01-26 00:00:00',95.13,519,531,73.54,'Intermediate' from dual union all
select 20391,'020391','1989-101A','Kosmos-2054','Al''tair No. 14L',62378,'In Earth orbit','','1990-01-26 00:00:00',1435.88,35706,35859,1.44,'Stationary' from dual union all
select 20392,'020392','1989-101B','Perekhodnik','Sredniy perekhodnik',62378,'Reentered','1989-12-28 00:00:00','1989-12-28 00:00:00',87.49,148,153,51.61,'Intermediate' from dual union all
select 20393,'020393','1989-101C','8S812','8S812',62378,'Reentered','1989-12-28 00:00:00','1989-12-29 00:00:00',87.26,135,143,51.62,'Intermediate' from dual union all
select 20394,'020394','1989-101D','Blok DM2','11S861 No. 34L',62378,'In Earth orbit','','1990-01-28 00:00:00',1465.64,36319,36407,1.46,'Drift GEO' from dual union all
select 20395,'020395','1988-065AE','Kosmos-1960 SS 23','ESO',61314,'Reentered','1990-01-15 00:00:00','1989-12-27 00:00:00',92.01,368,379,65.82,'Intermediate' from dual union all
select 20396,'020396','1988-065AF','Kosmos-1960 SS 24','ESO',61314,'Reentered','1990-01-13 00:00:00','1989-12-27 00:00:00',91.9,364,372,65.83,'Intermediate' from dual union all
select 20397,'020397','1989-100C','Kosmos-2053 SS 1','ESO',62377,'Reentered','1990-03-12 00:00:00','1989-12-27 00:00:00',94.82,504,516,73.52,'Intermediate' from dual union all
select 20398,'020398','1989-100D','Kosmos-2053 SS 2','ESO',62377,'Reentered','1990-03-28 00:00:00','1989-12-27 00:00:00',94.78,501,515,73.53,'Intermediate' from dual union all
select 20399,'020399','1989-101E','SOZ','SOZ',62378,'Reentered','2000-09-03 00:00:00','1993-08-18 00:00:00',442.47,202,25511,46.64,'Highly Ellip' from dual union all
select 204,'000204','61 A Eta 3','Ablestar 009','AJ10-104',7253,'In Earth orbit','','1961-12-15 00:00:00',105.63,945,1103,32.41,'Intermediate' from dual union all
select 2040,'002040','1966-012U','deb OPS 3031','',13419,'Reentered','1966-02-18 00:00:00','1966-02-16 00:00:00',88.81,166,265,96.7,'Sun-Synch' from dual union all
select 20400,'020400','1989-101F','SOZ','SOZ',62378,'Reentered','1991-07-31 00:00:00','1989-12-31 00:00:00',644.35,308,36360,47.3,'GEO Transfer' from dual union all
select 20401,'020401','1990-001A','Skynet 4A','Skynet 4A',63243,'In Earth orbit','','1990-01-31 00:00:00',1427.49,35449,35787,3.37,'Inclined Drift' from dual union all
select 20402,'020402','1990-001B','JCSAT 2','JCSAT 2',63243,'In Earth orbit','','1990-02-04 00:00:00',1436.09,35776,35797,.26,'Stationary' from dual union all
select 20403,'020403','1990-001C','Titan CT-1 Stage 2','LR91AJ11 CT-1',63243,'Reentered','1990-01-09 00:00:00','1990-01-05 00:00:00',91.64,178,532,28.6,'Intermediate' from dual union all
select 20404,'020404','1990-001D','Star 63D','Star 63D',63243,'In Earth orbit','','1990-02-05 00:00:00',613.8,299,34794,21.56,'GEO Transfer' from dual union all
select 20405,'020405','1990-001E','Payload Carrier Cover','Extension Module',63243,'Reentered','1990-01-15 00:00:00','1990-01-01 00:00:00',92.54,158,640,28.6,'Intermediate' from dual union all
select 20406,'020406','1990-001F','Orbus 7S','Orbus 7S',63243,'Reentered','2013-09-26 00:00:00','1990-02-05 00:00:00',340.91,303,19272,26.71,'Highly Ellip' from dual union all
select 20407,'020407','1986-017BY','Orlan-DM (No. 7)','large piece (5 sq m RCS)',59210,'Reentered','1990-06-27 00:00:00','1990-06-27 00:00:00',87.53,137,168,51.59,'Intermediate' from dual union all
select 20408,'020408','1989-100E','Kosmos-2053 SS 3','ESO',62377,'Reentered','1990-03-09 00:00:00','1990-01-08 00:00:00',94.49,484,504,73.52,'Intermediate' from dual union all
select 20409,'020409','1990-002A','Columbia (STS-32)','OV-102',63244,'Landed','1990-01-20 00:00:00','1990-01-11 00:00:00',91.08,319,337,28.5,'Intermediate' from dual union all
select 2041,'002041','1965-082JB','deb Transtage 4','',11832,'Reentered','1973-03-16 00:00:00','1966-02-19 00:00:00',99.79,669,827,32.01,'Intermediate' from dual union all
select 20410,'020410','1990-002B','Leasat 5','Syncom IV-5',63244,'In Earth orbit','','1990-02-12 00:00:00',1435.98,35264,36305,3.36,'Inclined GEO' from dual union all
select 20411,'020411','1990-002C','Leasat PKM','TU-882',63244,'In Earth orbit','','1990-02-08 00:00:00',276.64,319,15052,27.19,'Highly Ellip' from dual union all
select 20412,'020412','1983-070F','deb Kosmos-1481','',55827,'In Earth orbit','','1989-12-31 00:00:00',705.77,2828,36933,67.5,'GEO Transfer' from dual union all
select 20413,'020413','1983-020D','Blok D-1 No. 7L','11S824M No. 7L',55777,'Deep Space','1983-03-24 00:00:00','1990-01-10 00:00:00',5826.13,6566,195184,48.34,'Deep Space' from dual union all
select 20414,'020414','1986-017BU','EO-5 EVA-2 debris','',59210,'Reentered','1990-01-24 00:00:00','1990-01-10 00:00:00',92.26,383,388,51.61,'Intermediate' from dual union all
select 20415,'020415','1986-017BV','EO-5 EVA-2 debris','',59210,'Reentered','1990-03-15 00:00:00','1990-01-10 00:00:00',92.29,386,387,51.62,'Intermediate' from dual union all
select 20416,'020416','1986-017BW','EO-5 EVA-2 debris','',59210,'Reentered','1990-03-04 00:00:00','1990-01-16 00:00:00',92.17,380,382,51.62,'Intermediate' from dual union all
select 20417,'020417','1986-017BX','EO-5 EVA-2 debris','',59210,'Reentered','1990-02-07 00:00:00','1990-01-15 00:00:00',92.07,376,377,51.62,'Intermediate' from dual union all
select 20418,'020418','1986-017BZ','EO-5 EVA-2 debris','',59210,'Reentered','1990-03-04 00:00:00','1990-01-15 00:00:00',92.24,383,386,51.63,'Intermediate' from dual union all
select 20419,'020419','1986-017CA','EO-5 EVA-2 debris','',59210,'Reentered','1990-03-01 00:00:00','1990-01-15 00:00:00',92.23,382,386,51.61,'Intermediate' from dual union all
select 2042,'002042','1965-082JC','deb Transtage 4','',11832,'Reentered','1979-11-21 00:00:00','1979-11-09 00:00:00',92.9,412,422,31.45,'Intermediate' from dual union all
select 20420,'020420','1986-017CB','EO-5 EVA-2 debris','',59210,'Reentered','1990-02-11 00:00:00','1990-01-16 00:00:00',92.17,380,382,51.62,'Intermediate' from dual union all
select 20421,'020421','1986-017CC','EO-5 EVA-2 debris','',59210,'Reentered','1990-01-31 00:00:00','1990-01-16 00:00:00',92.05,375,376,51.62,'Intermediate' from dual union all
select 20422,'020422','1988-113AJ','Kosmos-1985 SS 31','ESO',61362,'Reentered','1990-02-28 00:00:00','1990-01-17 00:00:00',94.61,498,502,73.55,'Intermediate' from dual union all
select 20423,'020423','1988-113AK','Kosmos-1985 SS 32','ESO',61362,'Reentered','1990-02-21 00:00:00','1990-01-18 00:00:00',94.26,466,500,73.55,'Intermediate' from dual union all
select 20424,'020424','1988-113AL','Kosmos-1985 SS 33','ESO',61362,'Reentered','1990-02-25 00:00:00','1990-01-18 00:00:00',94.44,485,498,73.55,'Intermediate' from dual union all
select 20425,'020425','1988-113AM','Kosmos-1985 SS 34','ESO',61362,'Reentered','1990-02-25 00:00:00','1990-01-18 00:00:00',94.42,481,500,73.55,'Intermediate' from dual union all
select 20426,'020426','1990-003A','Kosmos-2055','Oblik No. 810',63245,'Landed','1990-01-29 00:00:00','1990-01-19 00:00:00',90.19,248,319,62.83,'Intermediate' from dual union all
select 20427,'020427','1990-003B','11S510','11S510',63245,'Reentered','1990-01-24 00:00:00','1990-01-19 00:00:00',89.34,240,244,62.8,'Intermediate' from dual union all
select 20428,'020428','1990-003C','Sensor cover','Sensor cover',63245,'Reentered','1990-01-19 00:00:00','1990-01-18 00:00:00',89.41,236,255,62.84,'Intermediate' from dual union all
select 20429,'020429','1990-003D','Sensor cover','Sensor cover',63245,'Reentered','1990-01-18 00:00:00','1990-01-18 00:00:00',88.65,196,220,62.79,'Intermediate' from dual union all
select 2043,'002043','1965-082JD','deb Transtage 4','',11832,'Reentered','1968-09-01 00:00:00','1966-02-19 00:00:00',97.21,522,728,32.16,'Intermediate' from dual union all
select 20430,'020430','1988-113AN','Kosmos-1985 SS 35','ESO',61362,'Reentered','1990-02-24 00:00:00','1990-01-19 00:00:00',94.34,474,500,73.55,'Intermediate' from dual union all
select 20431,'020431','1988-113AP','Kosmos-1985 SS 36','ESO',61362,'Reentered','1990-02-18 00:00:00','1990-01-19 00:00:00',94.14,456,498,73.54,'Intermediate' from dual union all
select 20432,'020432','1990-004A','Kosmos-2056','Strela-2M',63246,'In Earth orbit','','1990-02-17 00:00:00',100.71,775,810,74.04,'Intermediate' from dual union all
select 20433,'020433','1990-004B','S3M','S3M',63246,'In Earth orbit','','1990-02-20 00:00:00',100.6,761,813,74.03,'Intermediate' from dual union all
select 20434,'020434','1990-004C','deb Kosmos-2056','Strela-2M debris 2 (high',63246,'In Earth orbit','','1993-08-17 00:00:00',100.76,781,808,74.05,'Intermediate' from dual union all
select 20435,'020435','1990-004D','deb Kosmos-2056','Strela-2M debris 1',63246,'In Earth orbit','','1993-08-16 00:00:00',100.26,755,787,74.03,'Intermediate' from dual union all
select 20436,'020436','1990-005A','SPOT 2','SPOT 2',63247,'In Earth orbit','','1990-02-21 00:00:00',101.39,824,825,98.72,'Sun-Synch' from dual union all
select 20437,'020437','1990-005B','UoSAT-OSCAR-14','UoSAT 3',63247,'In Earth orbit','','1990-02-22 00:00:00',100.77,786,805,98.71,'Sun-Synch' from dual;
insert into SATELLITE
select 20438,'020438','1990-005C','UoSAT-OSCAR-15','UoSAT 4',63247,'In Earth orbit','','1990-02-23 00:00:00',100.76,786,804,98.71,'Sun-Synch' from dual union all
select 20439,'020439','1990-005D','AMSAT-OSCAR-16','Pacsat',63247,'In Earth orbit','','1990-02-21 00:00:00',100.77,787,804,98.72,'Sun-Synch' from dual union all
select 2044,'002044','1965-082JE','deb Transtage 4','',11832,'Reentered','1980-11-20 00:00:00','1980-11-13 00:00:00',92.35,382,398,32.5,'Intermediate' from dual union all
select 20440,'020440','1990-005E','DOVE-OSCAR-17','Dove',63247,'In Earth orbit','','1990-02-23 00:00:00',100.78,787,805,98.72,'Sun-Synch' from dual union all
select 20441,'020441','1990-005F','WEBERSAT-OSCAR-18','Webersat',63247,'In Earth orbit','','1990-02-24 00:00:00',100.8,788,805,98.71,'Sun-Synch' from dual union all
select 20442,'020442','1990-005G','LUSAT-OSCAR-19','Lusat',63247,'In Earth orbit','','1990-02-21 00:00:00',100.82,790,805,98.71,'Sun-Synch' from dual union all
select 20443,'020443','1990-005H','Ariane H10 V35','Ariane H10 S/N T407',63247,'In Earth orbit','','1990-03-14 00:00:00',100.67,782,799,98.61,'Sun-Synch' from dual union all
select 20444,'020444','1990-006A','Molniya-3','Molniya-3 No. 53',63248,'Reentered','2003-06-23 00:00:00','1990-02-22 00:00:00',717.6,592,39754,62.81,'Molniya' from dual union all
select 20445,'020445','1990-006B','11S510','11S510',63248,'Reentered','1990-02-22 00:00:00','1990-01-31 00:00:00',91.86,209,523,62.82,'Intermediate' from dual union all
select 20446,'020446','1990-006C','Blok-ML','Blok-ML No. 334',63248,'Reentered','2002-10-23 00:00:00','1993-08-17 00:00:00',696.72,941,38370,64.87,'GEO Transfer' from dual union all
select 20447,'020447','1990-006D','BOZ','BOZ',63248,'Reentered','1990-02-15 00:00:00','1990-01-30 00:00:00',91.9,187,549,62.83,'Intermediate' from dual union all
select 20448,'020448','1990-007A','Hiten','MUSES-A',63249,'Deep Space','1990-01-25 00:00:00','1990-02-14 00:00:00',16085.12,400,409000,30.7,'Cislunar' from dual union all
select 20449,'020449','1990-007C','M-3B','M-3B',63249,'Reentered','1990-01-29 00:00:00','1990-01-26 00:00:00',87.56,152,155,30.75,'Intermediate' from dual union all
select 2045,'002045','1965-082JF','deb Transtage 4','',11832,'Reentered','1967-10-20 00:00:00','1966-02-18 00:00:00',97.52,544,736,31.69,'Intermediate' from dual union all
select 20450,'020450','1990-008C','Star 48B','Star 48B S/N 10055-18',63250,'Reentered','1994-09-19 00:00:00','1993-03-29 00:00:00',167.79,172,7226,37.66,'Medium' from dual union all
select 20451,'020451','1990-007D','KM-M','KM-M',63249,'Deep Space','1990-01-25 00:00:00','1990-01-24 00:00:00',9788.46,400,290000,30.7,'Deep Space' from dual union all
select 20452,'020452','1990-008A','GPS II-6','Navstar GPS SVN 18',63250,'In Earth orbit','','1990-02-25 00:00:00',717.92,20087,20275,54.64,'Medium' from dual union all
select 20453,'020453','1990-008B','Delta 191','Delta SSPS AJ10-118K',63250,'In Earth orbit','','1990-03-06 00:00:00',102.49,449,1304,35.64,'Intermediate' from dual union all
select 20454,'020454','1987-020DQ','deb Kosmos-1823','',60110,'Reentered','1990-02-21 00:00:00','1990-03-31 00:00:00',96.27,497,663,73.2,'Intermediate' from dual union all
select 20455,'020455','1963-014FB','deb West Ford','RCS 0.07',9245,'Reentered','2007-10-31 00:00:00','1993-08-07 00:00:00',163.68,1201,5867,85.65,'Medium' from dual union all
select 20456,'020456','1977-065GD','deb Delta 132','',42193,'Reentered','2001-01-26 00:00:00','1993-08-06 00:00:00',104.48,587,1353,28.08,'Intermediate' from dual union all
select 20457,'020457','1990-009A','Kosmos-2057','Kobal''t No. 526',63251,'Landed','1990-03-19 00:00:00','1990-02-01 00:00:00',89.39,173,316,62.83,'Intermediate' from dual union all
select 20458,'020458','1990-009B','11S510','11S510',63251,'Reentered','1990-01-30 00:00:00','1990-01-29 00:00:00',88.42,160,233,62.82,'Intermediate' from dual union all
select 20459,'020459','1986-017CD','EO-5 EVA-3 debris','',59210,'Reentered','1990-02-08 00:00:00','1990-02-07 00:00:00',88.99,219,231,51.61,'Intermediate' from dual union all
select 2046,'002046','1965-082JG','deb Transtage 4','',11832,'Reentered','1967-11-15 00:00:00','1966-02-19 00:00:00',97.28,520,737,31.87,'Intermediate' from dual union all
select 20460,'020460','1986-017CE','EO-5 EVA-3 debris','',59210,'Reentered','1990-03-08 00:00:00','1990-02-20 00:00:00',91.56,344,359,51.62,'Intermediate' from dual union all
select 20461,'020461','1986-017CF','EO-5 EVA-3 debris','',59210,'Reentered','1990-01-28 00:00:00','1990-01-28 00:00:00',91.25,326,346,51.61,'Intermediate' from dual union all
select 20462,'020462','1990-003E','KDU','KDU',63245,'Reentered','1990-02-03 00:00:00','1990-02-02 00:00:00',87.32,138,146,62.82,'Intermediate' from dual union all
select 20463,'020463','1990-003F','Blenda','Sunshade',63245,'Reentered','1990-01-31 00:00:00','1990-02-01 00:00:00',88.78,212,216,62.8,'Intermediate' from dual union all
select 20464,'020464','1990-003G','KDU part','KDU part',63245,'Reentered','1990-01-30 00:00:00','1990-01-31 00:00:00',88.64,206,209,62.8,'Intermediate' from dual union all
select 20465,'020465','1990-010A','Kosmos-2058','Tselina-R',63252,'In Earth orbit','','1990-07-05 00:00:00',97.66,629,665,82.5,'Intermediate' from dual union all
select 20466,'020466','1990-010B','S5M','S5M',63252,'In Earth orbit','','1990-07-04 00:00:00',97.65,629,664,82.5,'Intermediate' from dual union all
select 20467,'020467','1989-100F','Kosmos-2053 SS 4','ESO',62377,'Reentered','1990-03-31 00:00:00','1990-02-22 00:00:00',94.56,487,507,73.46,'Intermediate' from dual union all
select 20468,'020468','1989-100G','Kosmos-2053 SS 5','ESO',62377,'Reentered','1990-03-26 00:00:00','1990-02-20 00:00:00',93.78,451,467,73.57,'Intermediate' from dual union all
select 20469,'020469','1990-065T','G-1 Canister','Ba Canister',63307,'In Earth orbit','','1992-07-02 00:00:00',582.36,352,33093,17.78,'Highly Ellip' from dual union all
select 2047,'002047','1965-082JH','deb Transtage 4','',11832,'Reentered','1967-11-16 00:00:00','1966-01-30 00:00:00',104.43,716,1220,32.62,'Intermediate' from dual union all
select 20470,'020470','1963-038N','deb NNS','',9269,'Reentered','2017-01-12 00:00:00','1993-03-25 00:00:00',105.93,1023,1053,89.95,'Polar' from dual union all
select 20471,'020471','1988-093C','deb Kosmos- 1975','',61342,'Reentered','2001-04-06 00:00:00','1993-08-17 00:00:00',96.08,560,581,82.53,'Intermediate' from dual union all
select 20472,'020472','1986-017CG','EO-5 EVA-4 debris','',59210,'Reentered','1990-02-03 00:00:00','1990-02-02 00:00:00',91.81,353,374,51.63,'Intermediate' from dual union all
select 20473,'020473','1990-011A','Zhongxing 3','DFH 2A',63253,'In Earth orbit','','1990-03-06 00:00:00',1436.09,35786,35787,.08,'Stationary' from dual union all
select 20474,'020474','1990-011B','CZ-3 Stage 3','CZ-3 Stage 3',63253,'In Earth orbit','','1990-03-06 00:00:00',625.67,250,35458,30.52,'GEO Transfer' from dual union all
select 20475,'020475','1986-017CH','EO-5 EVA-5 debris','',59210,'Reentered','1990-02-07 00:00:00','1990-02-05 00:00:00',91.63,351,358,51.62,'Intermediate' from dual union all
select 20476,'020476','1990-012A','Kosmos-2059','Duga-K',63254,'Reentered','1990-11-12 00:00:00','1990-03-08 00:00:00',108.71,190,2145,65.83,'Intermediate' from dual union all
select 20477,'020477','1990-012B','S3M','S3M',63254,'Reentered','1990-07-12 00:00:00','1990-03-08 00:00:00',107.28,178,2024,65.84,'Intermediate' from dual union all
select 20478,'020478','1990-013A','Momo-1B','Mos-1b',63255,'In Earth orbit','','1990-03-12 00:00:00',103.23,911,912,99.2,'Sun-Synch' from dual union all
select 20479,'020479','1990-013B','Orizuru','DEBUT',63255,'In Earth orbit','','1990-05-14 00:00:00',112.21,911,1746,99.04,'Sun-Synch' from dual union all
select 2048,'002048','1965-082JJ','deb Transtage 4','',11832,'Reentered','1967-01-27 00:00:00','1966-02-19 00:00:00',98.17,564,779,31.78,'Intermediate' from dual union all
select 20480,'020480','1990-013C','Fuji-OSCAR-20','JAS-1b',63255,'In Earth orbit','','1990-03-16 00:00:00',112.21,912,1744,99.05,'Sun-Synch' from dual union all
select 20481,'020481','1990-012C','Kosmos-2059 SS 1','ESO',63254,'Reentered','1990-03-06 00:00:00','1990-02-16 00:00:00',105.58,190,1854,65.84,'Intermediate' from dual union all
select 20482,'020482','1990-012D','Kosmos-2059 SS 2','ESO',63254,'Reentered','1990-03-24 00:00:00','1990-02-22 00:00:00',105.84,185,1883,65.72,'Intermediate' from dual union all
select 20483,'020483','1990-012E','Kosmos-2059 SS 3','ESO',63254,'Reentered','1990-03-06 00:00:00','1990-02-16 00:00:00',103.8,187,1690,65.78,'Intermediate' from dual union all
select 20484,'020484','1990-012F','Kosmos-2059 SS 4','ESO',63254,'Reentered','1990-03-06 00:00:00','1990-02-17 00:00:00',105.49,186,1849,65.92,'Intermediate' from dual union all
select 20485,'020485','1990-012G','Kosmos-2059 SS 5','ESO',63254,'Reentered','1990-03-05 00:00:00','1990-02-18 00:00:00',104.44,183,1754,65.79,'Intermediate' from dual union all
select 20486,'020486','1990-012H','Kosmos-2059 SS 6','ESO',63254,'Reentered','1990-03-07 00:00:00','1990-02-18 00:00:00',104.53,187,1757,65.77,'Intermediate' from dual union all
select 20487,'020487','1990-012J','Kosmos-2059 SS 7','ESO',63254,'Reentered','1990-02-25 00:00:00','1990-02-22 00:00:00',96.97,170,1057,65.8,'Intermediate' from dual union all
select 20488,'020488','1988-063F','deb Ariane V24','',61312,'In Earth orbit','','1993-08-16 00:00:00',306.47,400,16958,7.38,'Highly Ellip' from dual union all
select 20489,'020489','1963-014FC','deb West Ford','RCS 0.08',9245,'Reentered','1997-10-30 00:00:00','1993-08-07 00:00:00',162.74,942,6049,85.07,'Medium' from dual union all
select 2049,'002049','1965-082JK','deb Transtage 4','',11832,'Reentered','1970-01-14 00:00:00','1966-02-19 00:00:00',99.83,685,816,31.59,'Intermediate' from dual union all
select 20490,'020490','1990-012K','Kosmos-2059 SS 8','ESO',63254,'Reentered','1990-03-06 00:00:00','1990-02-20 00:00:00',103.97,180,1713,65.92,'Intermediate' from dual union all
select 20491,'020491','1990-013D','H-I Stage 2 S/N H-21(F)','LE-5',63255,'In Earth orbit','','1990-03-10 00:00:00',110.52,892,1610,99.06,'Sun-Synch' from dual union all
select 20492,'020492','1990-012L','Kosmos-2059 SS 9','ESO',63254,'Reentered','1990-03-05 00:00:00','1990-02-17 00:00:00',104.53,186,1758,65.69,'Intermediate' from dual union all
select 20493,'020493','1990-012M','Kosmos-2059 SS 10','ESO',63254,'Reentered','1990-02-27 00:00:00','1990-02-26 00:00:00',94.34,164,810,65.69,'Intermediate' from dual union all
select 20494,'020494','1990-014A','Soyuz TM-9','Soyuz 7K-STM No. 60',63256,'Landed','1990-08-09 00:00:00','1990-03-13 00:00:00',92.3,378,396,51.62,'Intermediate' from dual union all
select 20495,'020495','1990-014B','11S510','11S510',63256,'Reentered','1990-02-13 00:00:00','1990-02-12 00:00:00',87.54,150,156,51.57,'Intermediate' from dual union all
select 20496,'020496','1990-015A','LACE','LACE',63257,'Reentered','2000-05-24 00:00:00','1990-04-29 00:00:00',95.36,528,544,43.1,'Intermediate' from dual union all
select 20497,'020497','1990-015B','RME','RME',63257,'Reentered','1992-05-24 00:00:00','1990-04-29 00:00:00',93.79,450,469,43.11,'Intermediate' from dual union all
select 20498,'020498','1990-015C','Delta 192','Delta SSPS AJ10-118K',63257,'Reentered','1990-12-07 00:00:00','1990-03-21 00:00:00',93.45,420,467,41.64,'Intermediate' from dual union all
select 20499,'020499','1990-016A','Raduga','Gran'' No. 35L',63258,'In Earth orbit','','1990-03-17 00:00:00',1436.19,35768,35809,1.41,'Stationary' from dual union all
select 205,'000205','61 A Eta 2','Traac','Traac',7253,'In Earth orbit','','1961-12-18 00:00:00',105.79,934,1129,32.43,'Intermediate' from dual union all
select 2050,'002050','1965-082JL','deb Transtage 4','',11832,'In Earth orbit','','1992-02-03 00:00:00',100.53,705,863,32.26,'Intermediate' from dual union all
select 20500,'020500','1990-016B','8S812','8S812',63258,'Reentered','1990-02-17 00:00:00','1990-02-16 00:00:00',87.96,169,178,51.64,'Intermediate' from dual union all
select 20501,'020501','1990-016C','Perekhodnik','Sredniy perekhodnik',63258,'Reentered','1990-02-17 00:00:00','1990-02-15 00:00:00',87.13,126,139,51.63,'Intermediate' from dual union all
select 20502,'020502','1990-016D','Blok DM2','11S861 No. 41L',63258,'In Earth orbit','','1990-03-18 00:00:00',1439.79,35701,36017,1.44,'Drift GEO' from dual union all
select 20503,'020503','1976-067BX','deb Kosmos-839','',39019,'In Earth orbit','','1993-07-30 00:00:00',111.73,568,2045,65.72,'Intermediate' from dual union all
select 20504,'020504','1963-014FD','deb West Ford','RCS 0.12',9245,'In Earth orbit','','1993-08-03 00:00:00',164.11,1480,5622,85.97,'Medium' from dual union all
select 20505,'020505','1981-053MM','deb Kosmos-1275','',52576,'In Earth orbit','','1993-08-07 00:00:00',103.05,885,921,83.19,'Intermediate' from dual union all
select 20506,'020506','1990-016E','SOZ','SOZ',63258,'Reentered','1990-12-03 00:00:00','1990-03-31 00:00:00',636.21,304,35947,47.3,'GEO Transfer' from dual union all
select 20507,'020507','1990-016F','SOZ','SOZ',63258,'Reentered','1991-09-04 00:00:00','1990-03-31 00:00:00',634.58,250,35917,47.2,'GEO Transfer' from dual union all
select 20508,'020508','1990-017A','Nadezhda','Nadezhda 17F118 No. 405',63259,'In Earth orbit','','1990-03-30 00:00:00',104.86,956,1019,82.95,'Intermediate' from dual union all
select 20509,'020509','1990-017B','S3M','S3M',63259,'In Earth orbit','','1990-06-21 00:00:00',104.76,952,1014,82.95,'Intermediate' from dual union all
select 2051,'002051','1965-082JM','deb Transtage 4','',11832,'Reentered','1981-09-16 00:00:00','1966-02-19 00:00:00',99.16,692,745,32.28,'Intermediate' from dual union all
select 20510,'020510','1990-018A','Okean-O1','Okean-O1 No. 5/NKhM No.',63260,'In Earth orbit','','1990-04-02 00:00:00',97.75,637,666,82.53,'Intermediate' from dual union all
select 20511,'020511','1990-018B','S5M','S5M',63260,'In Earth orbit','','1990-03-30 00:00:00',97.74,636,665,82.53,'Intermediate' from dual union all
select 20512,'020512','1990-019A','Atlantis (STS-36)','OV-104',63261,'Landed','1990-03-04 00:00:00','1990-03-02 00:00:00',89.29,238,241,61.98,'Intermediate' from dual union all
select 20513,'020513','1990-020A','Progress M-3','Progress 7K-TGM No. 203',63262,'Deorbited','1990-04-28 00:00:00','1990-03-08 00:00:00',92.33,378,399,51.61,'Intermediate' from dual union all
select 20514,'020514','1990-020B','11S510','11S510',63262,'Reentered','1990-03-02 00:00:00','1990-03-01 00:00:00',88.2,173,198,51.62,'Intermediate' from dual union all
select 20515,'020515','1989-100H','Kosmos-2053 SS 6','ESO',62377,'Reentered','1990-04-24 00:00:00','1990-03-14 00:00:00',94.81,506,513,73.52,'Intermediate' from dual union all
select 20516,'020516','1990-019B','USA 53','AFP-731 MISTY',63261,'In Earth orbit','','1990-10-16 00:00:00',100.95,800,808,64.99,'Intermediate' from dual union all
select 20517,'020517','1990-019C','deb USA-53','',63261,'Reentered','1990-03-19 00:00:00','1990-03-07 00:00:00',89.77,257,270,62,'Intermediate' from dual union all
select 20518,'020518','1990-019D','deb USA-53','',63261,'Reentered','1990-03-19 00:00:00','1990-03-07 00:00:00',89.77,257,270,62,'Intermediate' from dual union all
select 20519,'020519','1990-019E','deb USA-53','',63261,'Reentered','1990-04-01 00:00:00','1990-03-07 00:00:00',89.77,257,270,62,'Intermediate' from dual union all
select 2052,'002052','1965-082JN','deb Transtage 4','',11832,'Reentered','1972-06-26 00:00:00','1966-02-14 00:00:00',97.31,551,709,32.21,'Intermediate' from dual union all
select 20520,'020520','1990-019F','deb USA-53','',63261,'Reentered','1990-05-01 00:00:00','1990-03-07 00:00:00',89.77,257,270,62,'Intermediate' from dual union all
select 20521,'020521','1990-019G','deb USA-53','',63261,'Reentered','1990-05-01 00:00:00','1990-03-07 00:00:00',89.77,257,270,62,'Intermediate' from dual union all
select 20522,'020522','1989-100J','Kosmos-2053 SS 7','ESO',62377,'Reentered','1990-04-28 00:00:00','1990-03-15 00:00:00',94.95,506,527,73.52,'Intermediate' from dual union all
select 20523,'020523','1990-021A','INTELSAT 603','INTELSAT VI F-3',63263,'In Earth orbit','','1990-09-10 00:00:00',95.73,548,559,28.34,'Intermediate' from dual union all
select 20524,'020524','1990-021B','Orbus 21S','Orbus 21S',63263,'Reentered Attached','1990-03-28 00:00:00','1990-03-16 00:00:00',89.56,167,339,28.6,'Intermediate' from dual union all
select 20525,'020525','1990-022A','Kosmos-2060','US-PM',63264,'Reentered','1991-09-01 00:00:00','1990-04-19 00:00:00',92.76,403,417,65.03,'Intermediate' from dual union all
select 20526,'020526','1990-022B','11S692','11S692',63264,'Reentered','1990-03-15 00:00:00','1990-03-15 00:00:00',87.78,100,230,65.01,'Intermediate' from dual union all
select 20527,'020527','1990-023A','Kosmos-2061','Parus',63265,'In Earth orbit','','1990-04-20 00:00:00',105,972,1017,82.94,'Intermediate' from dual;
insert into SATELLITE
select 20528,'020528','1990-023B','S3M','S3M',63265,'In Earth orbit','','1990-04-20 00:00:00',104.89,971,1008,82.94,'Intermediate' from dual union all
select 20529,'020529','1990-024A','Kosmos-2062','Oblik No. 384',63266,'Landed','1990-04-05 00:00:00','1990-03-24 00:00:00',89.04,209,246,82.32,'Intermediate' from dual union all
select 2053,'002053','1965-082JP','deb Transtage 4','',11832,'Reentered','1973-02-27 00:00:00','1966-02-19 00:00:00',99.05,684,743,32.29,'Intermediate' from dual union all
select 20530,'020530','1990-024B','11S510','11S510',63266,'Reentered','1990-03-23 00:00:00','1990-03-23 00:00:00',87.96,162,184,82.32,'Intermediate' from dual union all
select 20531,'020531','1989-100K','Kosmos-2053 SS 8','ESO',62377,'Reentered','1990-04-30 00:00:00','1990-03-24 00:00:00',94.71,500,509,73.52,'Intermediate' from dual union all
select 20532,'020532','1989-100L','Kosmos-2053 SS 9','ESO',62377,'Reentered','1990-05-07 00:00:00','1990-03-22 00:00:00',94.97,504,531,73.54,'Intermediate' from dual union all
select 20533,'020533','1990-025A','GPS II-7','Navstar GPS SVN 20',63267,'In Earth orbit','','1990-04-30 00:00:00',717.84,20089,20269,55.07,'Medium' from dual union all
select 20534,'020534','1990-025B','Delta 193','Delta SSPS AJ10-118K',63267,'Reentered','1990-04-04 00:00:00','1990-03-29 00:00:00',90.83,176,455,30.23,'Intermediate' from dual union all
select 20535,'020535','1990-025C','Star 48B','Star 48B S/N 10049-18',63267,'Reentered','1994-01-18 00:00:00','1990-04-29 00:00:00',341.05,159,19425,37.59,'Highly Ellip' from dual union all
select 20536,'020536','1990-026A','Kosmos-2063','Oko',63268,'In Earth orbit','','1990-04-26 00:00:00',717.8,627,39729,62.93,'Molniya' from dual union all
select 20537,'020537','1990-026B','11S510','11S510',63268,'Reentered','1990-04-22 00:00:00','1990-04-05 00:00:00',91.66,209,503,62.76,'Intermediate' from dual union all
select 20538,'020538','1990-026C','BOZ','BOZ',63268,'Reentered','1990-04-10 00:00:00','1990-03-29 00:00:00',92.14,179,579,62.82,'Intermediate' from dual union all
select 20539,'020539','1990-026D','Blok-2BL','Blok-2BL No. 85',63268,'Reentered','2013-05-05 00:00:00','1990-04-26 00:00:00',709.28,632,39303,63.11,'Molniya' from dual union all
select 2054,'002054','1965-082JQ','deb Transtage 4','',11832,'Reentered','1967-01-23 00:00:00','1966-02-19 00:00:00',96.45,360,817,31.78,'Intermediate' from dual union all
select 20540,'020540','1990-027A','''Ofeq-2','''Ofeq-2',63269,'Reentered','1990-07-09 00:00:00','1990-05-03 00:00:00',100.54,200,1369,143.2,'Retrograde' from dual union all
select 20541,'020541','1990-027B','AUS-51','AUS-51',63269,'Reentered','1990-06-20 00:00:00','1990-05-03 00:00:00',99.18,197,1242,143.2,'Retrograde' from dual union all
select 20542,'020542','1990-024C','Blenda','Sunshade',63266,'Reentered','1990-04-09 00:00:00','1990-04-06 00:00:00',91.21,245,423,82.33,'Intermediate' from dual union all
select 20543,'020543','1990-024D','KDU','KDU',63266,'Reentered','1990-04-14 00:00:00','1990-04-05 00:00:00',89.54,228,275,82.32,'Intermediate' from dual union all
select 20544,'020544','1990-024E','KDU part','KDU part',63266,'Reentered','1990-04-08 00:00:00','1990-04-06 00:00:00',89.16,217,249,82.32,'Intermediate' from dual union all
select 20545,'020545','1990-024F','PTDU cover','KDU part',63266,'Reentered','1990-04-07 00:00:00','1990-04-05 00:00:00',91.64,241,469,82.34,'Intermediate' from dual union all
select 20546,'020546','1990-028A','Pegsat','Pegsat',63270,'Reentered Attached','1998-11-14 00:00:00','1990-05-06 00:00:00',96.01,460,674,94.12,'Polar' from dual union all
select 20547,'020547','1990-028B','TERCEL','SECS',63270,'In Earth orbit','','1990-08-06 00:00:00',96.26,489,670,94.14,'Polar' from dual union all
select 20548,'020548','1990-024G','KDU part','KDU part',63266,'Reentered','1990-04-08 00:00:00','1990-04-05 00:00:00',89.27,215,261,82.32,'Intermediate' from dual union all
select 20549,'020549','1990-029A','Kosmos-2064','Strela-1M',63271,'In Earth orbit','','1991-12-15 00:00:00',115.45,1463,1489,73.98,'Intermediate' from dual union all
select 2055,'002055','1965-082JR','deb Transtage 4','',11832,'Reentered','1966-06-28 00:00:00','1966-02-19 00:00:00',95.4,366,710,31.72,'Intermediate' from dual union all
select 20550,'020550','1990-029B','Kosmos-2065','Strela-1M',63271,'In Earth orbit','','1991-12-15 00:00:00',115.27,1462,1473,73.98,'Intermediate' from dual union all
select 20551,'020551','1990-029C','Kosmos-2066','Strela-1M',63271,'In Earth orbit','','1991-12-15 00:00:00',114.3,1385,1463,73.98,'Intermediate' from dual union all
select 20552,'020552','1990-029D','Kosmos-2067','Strela-1M',63271,'In Earth orbit','','1991-12-15 00:00:00',114.47,1400,1462,73.98,'Intermediate' from dual union all
select 20553,'020553','1990-029E','Kosmos-2068','Strela-1M',63271,'In Earth orbit','','1991-12-15 00:00:00',114.62,1414,1463,73.98,'Intermediate' from dual union all
select 20554,'020554','1990-029F','Kosmos-2069','Strela-1M',63271,'In Earth orbit','','1991-12-15 00:00:00',114.79,1428,1464,73.98,'Intermediate' from dual union all
select 20555,'020555','1990-029G','Kosmos-2070','Strela-1M',63271,'In Earth orbit','','1991-12-15 00:00:00',114.94,1442,1463,73.98,'Intermediate' from dual union all
select 20556,'020556','1990-029H','Kosmos-2071','Strela-1M',63271,'In Earth orbit','','1991-12-15 00:00:00',115.12,1458,1463,73.98,'Intermediate' from dual union all
select 20557,'020557','1990-029J','S3M','S3M',63271,'In Earth orbit','','1990-07-05 00:00:00',117.76,1463,1698,73.98,'Intermediate' from dual union all
select 20558,'020558','1990-030A','Asiasat 1','Asiasat 1',63272,'In Earth orbit','','1990-05-09 00:00:00',1436.16,35786,35790,.13,'Stationary' from dual union all
select 20559,'020559','1990-030B','CZ-3 Stage 3','CZ-3 Stage 3',63272,'In Earth orbit','','1990-05-13 00:00:00',645.82,256,36487,31.29,'GEO Transfer' from dual union all
select 2056,'002056','1966-012V','deb OPS 3031','',13419,'Reentered','1966-02-20 00:00:00','1966-02-16 00:00:00',88.48,141,258,96.3,'Sun-Synch' from dual union all
select 20560,'020560','1990-031A','POGS','POGS',63273,'In Earth orbit','','1997-11-21 00:00:00',98.48,627,745,89.82,'Polar' from dual union all
select 20561,'020561','1990-031B','TEX','TEX',63273,'In Earth orbit','','1997-11-21 00:00:00',98.44,627,742,89.82,'Polar' from dual union all
select 20562,'020562','1990-031C','SCE','SCE',63273,'In Earth orbit','','1997-11-21 00:00:00',98.43,627,741,89.82,'Polar' from dual union all
select 20563,'020563','1990-031D','Star 20 E-44','Star 20 E-44',63273,'In Earth orbit','','1997-11-21 00:00:00',98.41,628,738,89.82,'Polar' from dual union all
select 20564,'020564','1990-031E','PDD Test Mass','PDD mass',63273,'In Earth orbit','','1997-11-21 00:00:00',98.28,643,710,90.33,'Polar' from dual union all
select 20565,'020565','1990-031F','PDD Test Mass','PDD mass',63273,'In Earth orbit','','1997-11-22 00:00:00',98.1,611,725,89.44,'Polar' from dual union all
select 20566,'020566','1990-032A','Foton','Foton 34KS No. 6L',63274,'Landed','1990-04-27 00:00:00','1990-04-18 00:00:00',90.28,213,364,62.8,'Intermediate' from dual union all
select 20567,'020567','1990-032B','11S510','11S510',63274,'Reentered','1990-04-21 00:00:00','1990-04-19 00:00:00',88.98,185,264,62.78,'Intermediate' from dual union all
select 20568,'020568','1990-033A','Kosmos-2072','Neman',63275,'Deorbited','1990-11-21 00:00:00','1990-05-13 00:00:00',89.65,235,280,64.76,'Intermediate' from dual union all
select 20569,'020569','1990-033B','11S510','11S510',63275,'Reentered','1990-04-16 00:00:00','1990-04-15 00:00:00',87.36,134,154,64.76,'Intermediate' from dual union all
select 2057,'002057','1966-012W','deb OPS 3031','',13419,'Reentered','1966-02-21 00:00:00','1966-02-16 00:00:00',89.45,139,355,96.9,'Sun-Synch' from dual union all
select 20570,'020570','1990-034A','Palapa B2R','Palapa B2R',63276,'In Earth orbit','','1990-05-17 00:00:00',1438.41,35800,35864,.05,'Drift GEO' from dual union all
select 20571,'020571','1990-034B','Delta 194','Delta SSPS AJ10-118K',63276,'In Earth orbit','','1990-05-15 00:00:00',104.18,495,1417,22.75,'Equatorial' from dual union all
select 20572,'020572','1990-034C','Star 48B','Star 48B S/N 10048-18',63276,'Reentered','1997-03-30 00:00:00','1990-05-15 00:00:00',659.7,209,37241,18.93,'GEO Transfer' from dual union all
select 20573,'020573','1990-035A','Kosmos-2073','Oblik No. 811',63277,'Landed','1990-04-28 00:00:00','1990-04-19 00:00:00',89.78,231,296,82.36,'Intermediate' from dual union all
select 20574,'020574','1990-035B','11S510','11S510',63277,'Reentered','1990-04-18 00:00:00','1990-04-18 00:00:00',87.7,144,178,82.35,'Intermediate' from dual union all
select 20575,'020575','1990-031G','deb PDD','',63273,'In Earth orbit','','1997-11-21 00:00:00',98.29,625,729,89.82,'Polar' from dual union all
select 20576,'020576','1990-031H','deb PDD','',63273,'In Earth orbit','','1997-11-21 00:00:00',98.26,625,726,89.81,'Polar' from dual union all
select 20577,'020577','1990-036A','Kosmos-2074','Parus',63278,'In Earth orbit','','1990-07-04 00:00:00',104.82,965,1007,82.95,'Intermediate' from dual union all
select 20578,'020578','1990-036B','S3M','S3M',63278,'In Earth orbit','','1990-05-22 00:00:00',104.71,967,995,82.94,'Intermediate' from dual union all
select 20579,'020579','1990-037A','Discovery (STS-31)','OV-103',63279,'Landed','1990-04-29 00:00:00','1990-04-26 00:00:00',97.02,614,618,28.48,'Intermediate' from dual union all
select 2058,'002058','1966-012X','deb OPS 3031','',13419,'Reentered','1966-02-18 00:00:00','1966-02-16 00:00:00',87.96,141,206,96.3,'Sun-Synch' from dual union all
select 20580,'020580','1990-037B','Hubble Space Telescope','Hubble Space Telescope',63279,'In Earth orbit','','1990-05-24 00:00:00',97,611,620,28.47,'Intermediate' from dual union all
select 20581,'020581','1990-038A','Kosmos-2075','Romb',63280,'Reentered','1992-02-20 00:00:00','1990-05-25 00:00:00',94.53,480,512,74.03,'Intermediate' from dual union all
select 20582,'020582','1990-038B','S3M','S3M',63280,'Reentered','1991-12-30 00:00:00','1990-06-01 00:00:00',94.42,469,512,74.03,'Intermediate' from dual union all
select 20583,'020583','1990-039A','Molniya-1','Molniya-1 No. 71',63281,'Reentered','2005-02-24 00:00:00','1990-05-26 00:00:00',717.76,652,39702,62.8,'Molniya' from dual union all
select 20584,'020584','1990-039B','11S510','11S510',63281,'Reentered','1990-05-26 00:00:00','1990-05-03 00:00:00',91.81,213,514,62.78,'Intermediate' from dual union all
select 20585,'020585','1990-039C','BOZ','BOZ',63281,'Reentered','1990-05-23 00:00:00','1990-05-03 00:00:00',92.22,194,573,62.82,'Intermediate' from dual union all
select 20586,'020586','1990-039D','Blok-ML','Blok-ML No. 177',63281,'Reentered','2002-05-31 00:00:00','1993-03-29 00:00:00',733.02,748,40356,63.97,'Molniya' from dual union all
select 20587,'020587','1990-038C','Dispenser cover','Romb dispenser cover',63280,'Reentered','1990-07-04 00:00:00','1990-05-26 00:00:00',93.38,418,462,74.03,'Intermediate' from dual union all
select 20588,'020588','1990-032C','KB electropitaniya','KBE',63274,'Reentered','1990-05-16 00:00:00','1990-04-27 00:00:00',90.03,209,343,62.8,'Intermediate' from dual union all
select 20589,'020589','1990-032D','KBE part','KBE part',63274,'Reentered','1990-05-01 00:00:00','1990-04-27 00:00:00',89.75,204,320,62.79,'Intermediate' from dual union all
select 2059,'002059','1966-012Y','deb OPS 3031','',13419,'Reentered','1966-02-18 00:00:00','1966-02-16 00:00:00',88.09,153,206,96.6,'Sun-Synch' from dual union all
select 20590,'020590','1990-032E','PTDU cover','KBE part',63274,'Reentered','1990-05-07 00:00:00','1990-04-28 00:00:00',91.97,305,438,62.81,'Intermediate' from dual union all
select 20591,'020591','1990-035C','KDU part','KDU part',63277,'Reentered','1990-04-29 00:00:00','1990-04-28 00:00:00',89.5,228,272,82.38,'Intermediate' from dual union all
select 20592,'020592','1990-035D','KDU part','KDU part',63277,'Reentered','1990-04-29 00:00:00','1990-04-28 00:00:00',89.52,225,277,82.35,'Intermediate' from dual union all
select 20593,'020593','1990-035E','KDU part','KDU part',63277,'Reentered','1990-04-29 00:00:00','1990-04-28 00:00:00',89.5,228,272,82.36,'Intermediate' from dual union all
select 20594,'020594','1990-035F','Blenda','Sunshade',63277,'Reentered','1990-05-01 00:00:00','1990-04-28 00:00:00',89.46,213,283,82.4,'Intermediate' from dual union all
select 20595,'020595','1990-035G','KDU','KDU',63277,'Reentered','1990-05-13 00:00:00','1990-04-28 00:00:00',89.55,224,281,82.37,'Intermediate' from dual union all
select 20596,'020596','1990-040A','Kosmos-2076','Oko',63282,'In Earth orbit','','1990-05-28 00:00:00',717.86,591,39767,63.06,'Molniya' from dual union all
select 20597,'020597','1990-040B','11S510','11S510',63282,'Reentered','1990-05-28 00:00:00','1990-05-09 00:00:00',91.82,207,521,62.82,'Intermediate' from dual union all
select 20598,'020598','1990-040C','BOZ','BOZ',63282,'Reentered','1990-05-13 00:00:00','1990-05-06 00:00:00',90.74,171,451,62.86,'Intermediate' from dual union all
select 20599,'020599','1990-040D','Blok-2BL','Blok-2BL No. 86',63282,'In Earth orbit','','1990-05-30 00:00:00',707.67,589,39266,63.18,'Molniya' from dual union all
select 206,'000206','61 A Theta','Ranger 2','NASA P-33',7258,'Reentered','1961-11-20 00:00:00','1961-11-19 00:00:00',87.66,149,169,33.34,'Intermediate' from dual union all
select 2060,'002060','1966-012Z','deb OPS 3031','',13419,'Reentered','1966-02-18 00:00:00','1966-02-16 00:00:00',89.26,160,316,96.5,'Sun-Synch' from dual union all
select 20600,'020600','1990-040E','deb Blok-I','',63282,'Reentered','1990-05-03 00:00:00','1990-04-28 00:00:00',91.78,206,517,62.84,'Intermediate' from dual union all
select 20601,'020601','1990-035H','PTDU cover','KDU part',63277,'Reentered','1990-05-02 00:00:00','1990-04-28 00:00:00',91.36,243,440,82.36,'Intermediate' from dual union all
select 20602,'020602','1990-041A','Progress-42','Progress 7K-TG No. 150',63283,'Deorbited','1990-05-27 00:00:00','1990-05-15 00:00:00',92.33,381,397,51.61,'Intermediate' from dual union all
select 20603,'020603','1990-041B','11S510','11S510',63283,'Reentered','1990-05-07 00:00:00','1990-05-07 00:00:00',87.77,155,174,51.63,'Intermediate' from dual union all
select 20604,'020604','1990-042A','Kosmos-2077','Kobal''t No. 850',63284,'Landed','1990-07-04 00:00:00','1990-05-14 00:00:00',89.54,166,338,62.84,'Intermediate' from dual union all
select 20605,'020605','1990-042B','11S510','11S510',63284,'Reentered','1990-05-12 00:00:00','1990-05-11 00:00:00',87.77,145,184,62.81,'Intermediate' from dual union all
select 20606,'020606','1990-042C','Kosmos-2077 part','Kobal''t part',63284,'Reentered','1990-05-08 00:00:00','1990-05-08 00:00:00',89.04,176,279,62.79,'Intermediate' from dual union all
select 20607,'020607','1990-043A','MACSAT 1','MACSAT 1',63285,'In Earth orbit','','1993-04-28 00:00:00',98.36,603,758,89.93,'Polar' from dual union all
select 20608,'020608','1990-043B','MACSAT 2','MACSAT 2',63285,'In Earth orbit','','1990-06-12 00:00:00',98.56,610,769,89.89,'Polar' from dual union all
select 20609,'020609','1990-043C','Star 20 E-45','Star 20 E-45',63285,'In Earth orbit','','1990-06-12 00:00:00',98.55,609,770,89.88,'Polar' from dual union all
select 2061,'002061','1966-012AA','deb OPS 3031','',13419,'Reentered','1966-02-19 00:00:00','1966-02-16 00:00:00',91.23,332,338,96.3,'Sun-Synch' from dual union all
select 20610,'020610','1990-043D','MACSAT despin weight','',63285,'Reentered','2001-12-08 00:00:00','1993-08-16 00:00:00',97.33,567,694,89.92,'Polar' from dual union all
select 20611,'020611','1990-043E','MACSAT despin weight','',63285,'Reentered','2001-06-24 00:00:00','1993-08-16 00:00:00',97.15,561,684,89.93,'Polar' from dual union all
select 20612,'020612','1990-043F','MACSAT despin weight','',63285,'Reentered','2001-08-11 00:00:00','1993-08-15 00:00:00',97.13,569,674,89.9,'Polar' from dual union all
select 20613,'020613','1990-043G','MACSAT despin weight','',63285,'Reentered','1992-03-01 00:00:00','1992-02-29 00:00:00',90.3,281,297,89.91,'Polar' from dual union all
select 20614,'020614','1990-043H','MACSAT adapter','',63285,'Reentered','2001-05-25 00:00:00','1993-08-15 00:00:00',97.06,566,670,89.91,'Polar' from dual union all
select 20615,'020615','1990-044A','Kosmos-2078','Kometa No. 12',63286,'Landed','1990-06-28 00:00:00','1990-05-22 00:00:00',89.36,211,274,69.99,'Intermediate' from dual union all
select 20616,'020616','1990-044B','11S510','11S510',63286,'Reentered','1990-05-19 00:00:00','1990-05-19 00:00:00',87.36,137,151,69.99,'Intermediate' from dual union all
select 20617,'020617','1990-044C','Kosmos-2078 part','Kometa part',63286,'Reentered','1990-05-16 00:00:00','1990-05-16 00:00:00',88.64,171,244,70,'Intermediate' from dual;
insert into SATELLITE
select 20618,'020618','1990-007B','Hagoromo','MUSES-A Subsatellite',63249,'Deep Space Attached','1990-01-25 00:00:00','1990-03-18 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 20619,'020619','1990-045A','Kosmos-2079','Uragan No. 46L',63287,'In Earth orbit','','1990-06-18 00:00:00',675.69,19075,19183,64.88,'Medium' from dual union all
select 2062,'002062','1966-012AB','deb OPS 3031','',13419,'Reentered','1966-02-22 00:00:00','1966-02-16 00:00:00',89.36,193,293,96.1,'Sun-Synch' from dual union all
select 20620,'020620','1990-045B','Kosmos-2080','Uragan No. 51L',63287,'In Earth orbit','','1990-06-18 00:00:00',675.69,19107,19151,64.88,'Medium' from dual union all
select 20621,'020621','1990-045C','Kosmos-2081','Uragan No. 52L',63287,'In Earth orbit','','1990-06-19 00:00:00',675.69,19099,19159,64.9,'Medium' from dual union all
select 20622,'020622','1990-045D','8S812','8S812',63287,'Reentered','1990-05-19 00:00:00','1990-05-19 00:00:00',87.06,122,136,64.87,'Intermediate' from dual union all
select 20623,'020623','1990-045E','Blok DM2','11S861 No. 37L',63287,'In Earth orbit','','1990-06-19 00:00:00',674.72,19064,19145,64.88,'Medium' from dual union all
select 20624,'020624','1990-046A','Kosmos-2082','Tselina-2',63288,'In Earth orbit','','1990-06-21 00:00:00',101.96,847,855,71,'Intermediate' from dual union all
select 20625,'020625','1990-046B','11S772','11S772',63288,'In Earth orbit','','1990-06-25 00:00:00',101.88,836,859,71,'Intermediate' from dual union all
select 20626,'020626','1990-046C','Sep Motor Cover','Sep Motor Cover',63288,'In Earth orbit','','1993-08-16 00:00:00',105.08,842,1155,71.02,'Intermediate' from dual union all
select 20627,'020627','1990-046D','Sep Motor Cover','Sep Motor Cover',63288,'In Earth orbit','','1993-08-17 00:00:00',105.21,843,1166,70.97,'Intermediate' from dual union all
select 20628,'020628','1990-046E','Sep Motor Cover','Sep Motor Cover',63288,'In Earth orbit','','1993-08-17 00:00:00',105.11,841,1159,70.98,'Intermediate' from dual union all
select 20629,'020629','1990-046F','Sep Motor Cover','Sep Motor Cover',63288,'In Earth orbit','','1993-08-17 00:00:00',104.98,843,1144,71.03,'Intermediate' from dual union all
select 2063,'002063','1966-012AC','deb OPS 3031','',13419,'Reentered','1966-02-17 00:00:00','1966-02-16 00:00:00',89.4,137,353,96.6,'Sun-Synch' from dual union all
select 20630,'020630','1990-045F','SOZ','SOZ',63287,'In Earth orbit','','1993-08-17 00:00:00',339.65,636,18859,65.31,'Highly Ellip' from dual union all
select 20631,'020631','1990-045G','SOZ','SOZ',63287,'In Earth orbit','','1993-08-16 00:00:00',339.46,647,18836,65.2,'Highly Ellip' from dual union all
select 20632,'020632','1990-047A','Resurs-F','Resurs-F1 14F43 No. 50',63289,'Landed','1990-06-14 00:00:00','1990-06-06 00:00:00',89.77,257,270,82.33,'Intermediate' from dual union all
select 20633,'020633','1990-047B','11S510','11S510',63289,'Reentered','1990-05-31 00:00:00','1990-05-30 00:00:00',87.33,136,149,82.33,'Intermediate' from dual union all
select 20634,'020634','1990-043J','deb MACSAT 1','',63285,'Reentered','1992-09-02 00:00:00','1992-08-31 00:00:00',89.28,228,250,89.85,'Polar' from dual union all
select 20635,'020635','1990-048A','Kristall','TsM-T 77KST No. 17201',63290,'Reentered Attached','2001-03-23 00:00:00','1990-06-30 00:00:00',92.16,373,388,51.61,'Intermediate' from dual union all
select 20636,'020636','1990-048B','8S812','8S812',63290,'Reentered','1990-06-11 00:00:00','1990-06-03 00:00:00',89.45,207,287,51.62,'Intermediate' from dual union all
select 20637,'020637','1989-100M','Kosmos-2053 SS 10','ESO',62377,'Reentered','1990-08-16 00:00:00','1990-05-31 00:00:00',94.88,502,523,73.52,'Intermediate' from dual union all
select 20638,'020638','1990-049A','ROSAT','ROSAT',63291,'Reentered','2011-10-23 00:00:00','1990-07-06 00:00:00',96.12,562,584,52.99,'Intermediate' from dual union all
select 20639,'020639','1990-049B','Delta 195','Delta SSPS AJ10-118K',63291,'Reentered','1992-01-15 00:00:00','1990-07-01 00:00:00',94.71,453,557,52.52,'Intermediate' from dual union all
select 2064,'002064','1966-012AD','deb OPS 3031','',13419,'Reentered','1966-02-22 00:00:00','1966-02-16 00:00:00',89.21,155,316,96.5,'Sun-Synch' from dual union all
select 20640,'020640','1989-100N','Kosmos-2053 SS 11','ESO',62377,'Reentered','1990-08-20 00:00:00','1990-06-07 00:00:00',94.8,501,516,73.53,'Intermediate' from dual union all
select 20641,'020641','1990-050A','USA 59','SLDCOM 1',63292,'In Earth orbit','','2007-01-31 00:00:00',238.96,1239,11516,63.42,'Medium' from dual union all
select 20642,'020642','1990-050E','Titan 45K-4 Stage 2','LR91AJ11A 45K-4',63292,'Reentered','1990-06-25 00:00:00','1990-06-08 00:00:00',90.03,268,284,61,'Intermediate' from dual union all
select 20643,'020643','1990-051A','Insat 1D','Insat 1D',63293,'In Earth orbit','','1990-07-13 00:00:00',1436.11,35769,35804,.14,'Stationary' from dual union all
select 20644,'020644','1990-051B','Delta 196','Delta SSPS AJ10-118K',63293,'Reentered','1990-07-12 00:00:00','1990-06-20 00:00:00',90.84,249,383,27.27,'Intermediate' from dual union all
select 20645,'020645','1990-051C','Star 48B','Star 48B S/N 10056-18',63293,'Reentered','1991-07-26 00:00:00','1990-07-09 00:00:00',683.79,136,38529,27.18,'GEO Transfer' from dual union all
select 20646,'020646','1990-052A','Molniya-3','Molniya-3 No. 47',63294,'Reentered','2006-02-26 00:00:00','1990-07-13 00:00:00',717.7,494,39857,62.85,'Molniya' from dual union all
select 20647,'020647','1990-052B','11S510','11S510',63294,'Reentered','1990-07-03 00:00:00','1990-06-20 00:00:00',90.62,206,404,62.85,'Intermediate' from dual union all
select 20648,'020648','1990-052C','BOZ','BOZ',63294,'Reentered','1990-06-28 00:00:00','1990-06-22 00:00:00',89.96,181,363,62.82,'Intermediate' from dual union all
select 20649,'020649','1990-052D','Blok-ML','Blok-ML',63294,'Reentered','2006-09-13 00:00:00','1993-08-14 00:00:00',733.69,704,40433,63.39,'Molniya' from dual union all
select 2065,'002065','1966-012AE','deb OPS 3031','',13419,'Reentered','1966-02-19 00:00:00','1966-02-16 00:00:00',88.82,181,252,96.1,'Sun-Synch' from dual union all
select 20650,'020650','1963-014FE','deb West Ford','RCS 0.07',9245,'Reentered','1993-10-11 00:00:00','1993-09-10 00:00:00',120.14,328,3047,83.66,'Medium' from dual union all
select 20651,'020651','1990-043K','deb MACSAT 1','',63285,'Reentered','2002-03-11 00:00:00','1993-08-16 00:00:00',97.84,555,756,90.16,'Polar' from dual union all
select 20652,'020652','1967-092J','NNS O-14 insulation','',15347,'Reentered','1991-09-26 00:00:00','1991-09-20 00:00:00',95.4,505,571,89.17,'Polar' from dual union all
select 20653,'020653','1990-047C','Blenda','Sunshade',63289,'Reentered','1990-06-17 00:00:00','1990-06-14 00:00:00',89.5,243,257,82.33,'Intermediate' from dual union all
select 20654,'020654','1990-047D','KDU','KDU',63289,'Reentered','1990-07-07 00:00:00','1990-06-14 00:00:00',89.71,254,267,82.33,'Intermediate' from dual union all
select 20655,'020655','1990-047E','KDU part','KDU part',63289,'Reentered','1990-06-16 00:00:00','1990-06-14 00:00:00',91.51,242,455,82.36,'Intermediate' from dual union all
select 20656,'020656','1990-047F','KDU part','KDU part',63289,'Reentered','1990-06-20 00:00:00','1990-06-14 00:00:00',91.69,258,457,82.31,'Intermediate' from dual union all
select 20657,'020657','1990-053A','Kosmos-2083','Oblik No. 821',63295,'Landed','1990-07-03 00:00:00','1990-06-21 00:00:00',91.64,298,412,82.59,'Intermediate' from dual union all
select 20658,'020658','1990-053B','11S510','11S510',63295,'Reentered','1990-06-21 00:00:00','1990-06-21 00:00:00',87.16,129,138,82.58,'Intermediate' from dual union all
select 20659,'020659','1990-054A','Gorizont','Gorizont No. 30L',63296,'In Earth orbit','','1990-07-23 00:00:00',1436.27,35701,35879,1.45,'Stationary' from dual union all
select 2066,'002066','1966-012AF','deb OPS 3031','',13419,'Reentered','1966-02-19 00:00:00','1966-02-16 00:00:00',88.34,135,249,96.3,'Sun-Synch' from dual union all
select 20660,'020660','1990-054B','8S812','8S812',63296,'Reentered','1990-06-22 00:00:00','1990-06-22 00:00:00',87.72,158,166,51.63,'Intermediate' from dual union all
select 20661,'020661','1990-054C','Perekhodnik','Sredniy perekhodnik',63296,'Reentered','1990-06-21 00:00:00','1990-06-21 00:00:00',88.23,174,199,51.61,'Intermediate' from dual union all
select 20662,'020662','1990-054D','Blok DM No. 67L','11S86 No. 67L',63296,'In Earth orbit','','1993-06-02 00:00:00',1432.56,35687,35748,.98,'Drift GEO' from dual union all
select 20663,'020663','1990-055A','Kosmos-2084','Oko',63297,'In Earth orbit','','1990-07-21 00:00:00',98.15,582,758,62.81,'Intermediate' from dual union all
select 20664,'020664','1990-055B','11S510','11S510',63297,'Reentered','1990-07-21 00:00:00','1990-07-02 00:00:00',91.45,202,490,62.82,'Intermediate' from dual union all
select 20665,'020665','1990-055C','BOZ','BOZ',63297,'Reentered','1990-07-20 00:00:00','1990-07-02 00:00:00',91.8,192,534,62.86,'Intermediate' from dual union all
select 20666,'020666','1990-055D','Blok-2BL','Blok-2BL No. 77345-680',63297,'In Earth orbit','','1990-08-30 00:00:00',97.96,573,750,62.79,'Intermediate' from dual union all
select 20667,'020667','1990-056A','INTELSAT 604','INTELSAT VI F-4',63298,'In Earth orbit','','1990-07-26 00:00:00',1436.24,35692,35887,.33,'Stationary' from dual union all
select 20668,'020668','1990-056B','Titan CT-3 Stage 2','LR91AJ11 CT-3',63298,'Reentered','1990-06-26 00:00:00','1990-06-25 00:00:00',88.19,144,225,28.6,'Intermediate' from dual union all
select 20669,'020669','1990-056C','Orbus 21S','Orbus 21S',63298,'In Earth orbit','','1990-07-24 00:00:00',682.5,258,38342,24.22,'GEO Transfer' from dual union all
select 2067,'002067','1966-012AG','deb OPS 3031','',13419,'Reentered','1966-02-17 00:00:00','1966-02-16 00:00:00',88.24,153,222,96.8,'Sun-Synch' from dual union all
select 20670,'020670','1990-057A','Meteor-2','Meteor-2 No. 23',63299,'In Earth orbit','','1990-07-29 00:00:00',104.05,938,962,82.54,'Intermediate' from dual union all
select 20671,'020671','1990-057B','S5M','S5M',63299,'In Earth orbit','','1990-08-02 00:00:00',104.02,937,960,82.54,'Intermediate' from dual union all
select 20672,'020672','1965-108AD','deb Transtage 8','',11858,'In Earth orbit','','1993-01-27 00:00:00',530.65,601,30067,26.96,'Highly Ellip' from dual union all
select 20673,'020673','1976-126CG','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-07 00:00:00',107.48,553,1668,65.64,'Intermediate' from dual union all
select 20674,'020674','1984-081E','deb Ariane','',57135,'In Earth orbit','','1993-08-23 00:00:00',600.24,737,33649,6.84,'GEO Transfer' from dual union all
select 20675,'020675','1990-053C','Blenda','Sunshade',63295,'Reentered','1990-07-19 00:00:00','1990-07-03 00:00:00',91.59,295,411,82.59,'Intermediate' from dual union all
select 20676,'020676','1990-053D','KDU','KDU',63295,'Reentered','1990-09-30 00:00:00','1990-07-03 00:00:00',91.59,295,411,82.59,'Intermediate' from dual union all
select 20677,'020677','1990-053E','KDU part','KDU part',63295,'Reentered','1990-07-29 00:00:00','1990-07-03 00:00:00',91.41,281,407,82.58,'Intermediate' from dual union all
select 20678,'020678','1990-053F','PTDU cover','KDU part',63295,'Reentered','1990-07-20 00:00:00','1990-07-04 00:00:00',94.2,357,603,82.58,'Intermediate' from dual union all
select 20679,'020679','1990-053G','PTDU cover','KDU part',63295,'Reentered','1990-08-09 00:00:00','1990-07-04 00:00:00',93.49,354,537,82.59,'Intermediate' from dual union all
select 2068,'002068','1966-012AH','deb OPS 3031','',13419,'Reentered','1966-02-17 00:00:00','1966-02-16 00:00:00',88.16,149,218,96.5,'Sun-Synch' from dual union all
select 20680,'020680','1974-089FE','deb Delta 104','',33692,'Reentered','2000-02-29 00:00:00','1993-08-08 00:00:00',104.65,870,1087,101.64,'Sun-Synch' from dual union all
select 20681,'020681','1974-089FF','deb Delta 104','',33692,'Reentered','1990-08-02 00:00:00','1990-08-01 00:00:00',98.6,476,908,102.15,'Retrograde' from dual union all
select 20682,'020682','1990-050B','USA 60','NOSS B-1 SSU-1',63292,'In Earth orbit','','1990-12-05 00:00:00',107.44,1071,1146,63.44,'Intermediate' from dual union all
select 20683,'020683','1990-058A','Gamma','Gamma No. 1L',63300,'Reentered','1992-02-28 00:00:00','1990-08-15 00:00:00',92.98,411,430,51.62,'Intermediate' from dual union all
select 20684,'020684','1990-058B','11S510','11S510',63300,'Reentered','1990-07-12 00:00:00','1990-07-12 00:00:00',88.19,173,197,51.62,'Intermediate' from dual union all
select 20685,'020685','1990-059A','BADR','BADR',63301,'Reentered','1990-12-08 00:00:00','1990-08-16 00:00:00',95.75,201,909,28.49,'Intermediate' from dual union all
select 20686,'020686','1990-059B','CZ-2E Stage 2','CZ-2E Stage 2',63301,'Reentered','1990-10-09 00:00:00','1990-08-19 00:00:00',94.78,199,817,28.48,'Intermediate' from dual union all
select 20687,'020687','1990-060A','Resurs-F','Resurs-F2 17F42 No. 5',63302,'Landed','1990-08-16 00:00:00','1990-07-24 00:00:00',89.73,258,265,82.32,'Intermediate' from dual union all
select 20688,'020688','1990-060B','11S510','11S510',63302,'Reentered','1990-07-19 00:00:00','1990-07-19 00:00:00',87.09,119,141,82.31,'Intermediate' from dual union all
select 20689,'020689','1986-017CJ','EO-6 EVA-1 debris','',59210,'Reentered','1990-09-27 00:00:00','1990-09-27 00:00:00',88.21,173,199,51.59,'Intermediate' from dual union all
select 2069,'002069','1966-012AJ','deb OPS 3031','',13419,'Reentered','1966-02-20 00:00:00','1966-02-16 00:00:00',89.46,179,317,96.9,'Sun-Synch' from dual union all
select 20690,'020690','1986-017CK','EO-6 EVA-1 debris','',59210,'Reentered','1990-08-03 00:00:00','1990-08-02 00:00:00',89.26,201,274,51.54,'Intermediate' from dual union all
select 20691,'020691','1990-050C','USA 61','NOSS B-1 SSU-2',63292,'In Earth orbit','','1990-12-05 00:00:00',107.44,1071,1146,63.44,'Intermediate' from dual union all
select 20692,'020692','1990-050D','USA 62','NOSS B-1 SSU-3',63292,'In Earth orbit','','1990-12-05 00:00:00',107.44,1071,1146,63.44,'Intermediate' from dual union all
select 20693,'020693','1990-061A','Kosmos-2085','Geizer No. 17L',63303,'In Earth orbit','','1990-08-19 00:00:00',1436.01,35773,35797,1.42,'Stationary' from dual union all
select 20694,'020694','1990-061B','8S812','8S812',63303,'Reentered','1990-07-21 00:00:00','1990-07-21 00:00:00',86.92,119,124,51.61,'Intermediate' from dual union all
select 20695,'020695','1990-061C','Perekhodnik','Sredniy perekhodnik',63303,'Reentered','1990-07-20 00:00:00','1990-07-18 00:00:00',88.33,187,197,51.58,'Intermediate' from dual union all
select 20696,'020696','1990-061D','Blok DM2','11S861 No. 24L',63303,'In Earth orbit','','1993-06-07 00:00:00',1437.24,35690,35928,1.02,'Drift GEO' from dual union all
select 20697,'020697','1990-061E','SOZ','SOZ',63303,'Reentered','1991-08-24 00:00:00','1991-08-24 00:00:00',89.86,98,437,46.29,'Intermediate' from dual union all
select 20698,'020698','1990-061F','SOZ','SOZ',63303,'Reentered','2000-07-16 00:00:00','1993-08-24 00:00:00',516.76,290,29617,46.89,'Highly Ellip' from dual union all
select 20699,'020699','1986-017CL','EO-6 EVA-1 debris','',59210,'Reentered','1990-08-21 00:00:00','1990-08-20 00:00:00',88.9,206,234,51.59,'Intermediate' from dual union all
select 207,'000207','61 A Zeta 2','deb Discoverer XXXV','',7259,'Reentered','1961-11-23 00:00:00','1961-11-17 00:00:00',90.78,179,447,81.53,'Intermediate' from dual union all
select 2070,'002070','1966-015A','Kosmos-110','Voskhod 3KV No. 5',13422,'Landed','1966-03-16 00:00:00','1966-02-23 00:00:00',95.35,192,879,51.85,'Intermediate' from dual union all
select 20700,'020700','1986-017CM','EO-6 EVA-1 debris','',59210,'Reentered','1990-08-21 00:00:00','1990-08-20 00:00:00',88.69,203,217,51.59,'Intermediate' from dual union all
select 20701,'020701','1986-017CN','EO-6 EVA-1 debris','',59210,'Reentered','1990-08-03 00:00:00','1990-08-01 00:00:00',89.97,261,285,51.6,'Intermediate' from dual union all
select 20702,'020702','1990-062A','Kosmos-2086','Oblik No. 818',63304,'Landed','1990-08-03 00:00:00','1990-07-22 00:00:00',90.35,237,346,82.33,'Intermediate' from dual union all
select 20703,'020703','1990-062B','11S510','11S510',63304,'Reentered','1990-07-22 00:00:00','1990-07-22 00:00:00',87.37,140,149,82.31,'Intermediate' from dual union all
select 20704,'020704','1990-054E','SOZ','SOZ',63296,'Reentered','2000-04-23 00:00:00','1993-03-31 00:00:00',487.27,290,27978,47.01,'Highly Ellip' from dual union all
select 20705,'020705','1990-063A','TDF 2','TDF 2',63305,'In Earth orbit','','1990-08-25 00:00:00',1436.06,35783,35789,.05,'Stationary' from dual union all
select 20706,'020706','1990-063B','Kopernikus 2','DFS 2',63305,'In Earth orbit','','1990-08-23 00:00:00',1436.04,35778,35793,.04,'Stationary' from dual union all
select 20707,'020707','1990-064A','Kosmos-2087','Oko',63306,'In Earth orbit','','1990-08-23 00:00:00',718.13,660,39712,62.88,'Molniya' from dual;
insert into SATELLITE
select 20708,'020708','1990-064B','11S510','11S510',63306,'Reentered','1990-08-22 00:00:00','1990-08-02 00:00:00',91.63,206,503,62.82,'Intermediate' from dual union all
select 20709,'020709','1990-064C','BOZ','BOZ',63306,'Reentered','1990-08-08 00:00:00','1990-08-02 00:00:00',90.53,164,437,62.86,'Intermediate' from dual union all
select 2071,'002071','1966-015B','11A57I','11A57I',13422,'Reentered','1966-04-29 00:00:00','1966-02-23 00:00:00',95.25,185,877,51.83,'Intermediate' from dual union all
select 20710,'020710','1990-064D','Blok-2BL','Blok-2BL',63306,'In Earth orbit','','1993-08-14 00:00:00',703.96,1958,37713,65.48,'GEO Transfer' from dual union all
select 20711,'020711','1990-054F','SOZ','SOZ',63296,'Reentered','1991-01-02 00:00:00','1990-09-30 00:00:00',620.56,166,35278,46.7,'GEO Transfer' from dual union all
select 20712,'020712','1990-065A','CRRES','CRRES',63307,'In Earth orbit','','1990-08-27 00:00:00',591.61,343,33590,18.22,'Highly Ellip' from dual union all
select 20713,'020713','1990-065B','Centaur AC-69','Centaur I AC-69',63307,'In Earth orbit','','1990-08-25 00:00:00',593.72,339,33705,18.12,'Highly Ellip' from dual union all
select 20714,'020714','1990-064E','deb Kosmos-2087','',63306,'Reentered','1990-08-14 00:00:00','1990-07-31 00:00:00',91.97,204,539,62.85,'Intermediate' from dual union all
select 20715,'020715','1990-064F','deb Kosmos-2087','',63306,'Reentered','1990-08-08 00:00:00','1990-07-26 00:00:00',92.44,192,597,62.86,'Intermediate' from dual union all
select 20716,'020716','1990-064G','deb Blok-I','',63306,'Reentered','1990-08-05 00:00:00','1990-07-26 00:00:00',92.33,183,595,62.86,'Intermediate' from dual union all
select 20717,'020717','1990-063C','Ariane H10 V37','Ariane H10 S/N T409',63305,'In Earth orbit','','1990-08-26 00:00:00',639.39,403,36011,4.05,'GEO Transfer' from dual union all
select 20718,'020718','1990-063D','Spelda 10 S/N UV6','Spelda 10',63305,'In Earth orbit','','1993-08-16 00:00:00',571.92,345,32546,4.38,'Highly Ellip' from dual union all
select 20719,'020719','1986-017CP','EO-6 EVA-2 debris','',59210,'Reentered','1990-09-19 00:00:00','1990-09-19 00:00:00',88.47,195,203,51.59,'Intermediate' from dual union all
select 2072,'002072','1966-012AK','deb OPS 3031','',13419,'Reentered','1966-02-18 00:00:00','1966-02-16 00:00:00',88.91,129,313,96.3,'Sun-Synch' from dual union all
select 20720,'020720','1990-066A','Kosmos-2088','Musson No. 22',63308,'In Earth orbit','','1990-09-03 00:00:00',116.06,1483,1524,73.6,'Intermediate' from dual union all
select 20721,'020721','1990-066B','S5M','S5M',63308,'In Earth orbit','','1992-08-12 00:00:00',116.02,1483,1521,73.59,'Intermediate' from dual union all
select 20722,'020722','1990-067A','Soyuz TM-10','Soyuz 7K-STM No. 61A',63309,'Landed','1990-12-10 00:00:00','1990-09-03 00:00:00',92.2,366,399,51.62,'Intermediate' from dual union all
select 20723,'020723','1990-067B','11S510','11S510',63309,'Reentered','1990-08-02 00:00:00','1990-08-02 00:00:00',87.4,143,149,51.61,'Intermediate' from dual union all
select 20724,'020724','1990-068A','GPS II-8','Navstar GPS SVN 21',63310,'In Earth orbit','','1990-09-03 00:00:00',718,19932,20434,54.71,'Medium' from dual union all
select 20725,'020725','1990-068B','Delta 197','Delta SSPS AJ10-118K',63310,'Reentered','1990-08-15 00:00:00','1990-08-08 00:00:00',90.79,178,449,30.13,'Intermediate' from dual union all
select 20726,'020726','1990-068C','Star 48B','Star 48B S/N 10057-18',63310,'Reentered','1991-03-18 00:00:00','1990-09-02 00:00:00',343.81,146,19612,37.66,'Highly Ellip' from dual union all
select 20727,'020727','1990-062C','Blenda','Sunshade',63304,'Reentered','1990-08-08 00:00:00','1990-08-08 00:00:00',87.58,138,172,82.27,'Intermediate' from dual union all
select 20728,'020728','1990-062D','KDU','KDU',63304,'Reentered','1990-08-16 00:00:00','1990-08-08 00:00:00',89.07,219,238,82.32,'Intermediate' from dual union all
select 20729,'020729','1990-062E','KDU part','KDU part',63304,'Reentered','1990-08-04 00:00:00','1990-08-04 00:00:00',87.71,155,168,82.31,'Intermediate' from dual union all
select 2073,'002073','1966-012AL','deb OPS 3031','',13419,'Reentered','1966-02-17 00:00:00','1966-02-16 00:00:00',88.78,147,281,96.5,'Sun-Synch' from dual union all
select 20730,'020730','1990-062F','KDU part','KDU part',63304,'Reentered','1990-08-06 00:00:00','1990-08-05 00:00:00',87.91,167,175,82.31,'Intermediate' from dual union all
select 20731,'020731','1990-062G','KDU part','KDU part',63304,'Reentered','1990-08-04 00:00:00','1990-08-04 00:00:00',89.7,202,318,82.41,'Intermediate' from dual union all
select 20732,'020732','1990-069A','Kosmos-2089','Kobal''t No. 532',63311,'Landed','1990-10-01 00:00:00','1990-08-11 00:00:00',89.04,181,274,62.84,'Intermediate' from dual union all
select 20733,'020733','1990-069B','11S510','11S510',63311,'Reentered','1990-08-09 00:00:00','1990-08-08 00:00:00',88.42,164,229,62.82,'Intermediate' from dual union all
select 20734,'020734','1990-069C','Kosmos-2089 part','Kobal''t part',63311,'Reentered','1990-08-04 00:00:00','1990-08-04 00:00:00',88.78,172,257,62.81,'Intermediate' from dual union all
select 20735,'020735','1990-070A','Kosmos-2090','Strela-3',63312,'In Earth orbit','','1993-04-20 00:00:00',113.82,1391,1413,82.57,'Intermediate' from dual union all
select 20736,'020736','1990-070B','Kosmos-2091','Strela-3',63312,'In Earth orbit','','1990-09-10 00:00:00',114.07,1413,1414,82.58,'Intermediate' from dual union all
select 20737,'020737','1990-070C','Kosmos-2092','Strela-3',63312,'In Earth orbit','','1993-04-19 00:00:00',114,1407,1413,82.57,'Intermediate' from dual union all
select 20738,'020738','1990-070D','Kosmos-2093','Strela-3',63312,'In Earth orbit','','1993-04-19 00:00:00',113.92,1400,1413,82.58,'Intermediate' from dual union all
select 20739,'020739','1990-070E','Kosmos-2094','Strela-3',63312,'In Earth orbit','','1993-04-19 00:00:00',113.87,1396,1412,82.58,'Intermediate' from dual union all
select 2074,'002074','1966-012AM','deb OPS 3031','',13419,'Reentered','1966-02-18 00:00:00','1966-02-16 00:00:00',89.01,206,246,96.9,'Sun-Synch' from dual union all
select 20740,'020740','1990-070F','Kosmos-2095','Strela-3',63312,'In Earth orbit','','1990-09-10 00:00:00',113.74,1384,1413,82.57,'Intermediate' from dual union all
select 20741,'020741','1990-070G','S5M','S5M',63312,'In Earth orbit','','1991-02-14 00:00:00',114.67,1413,1467,82.58,'Intermediate' from dual union all
select 20742,'020742','1990-071A','Molniya-1','Molniya-1T No. 68',63313,'Reentered','2007-07-06 00:00:00','1990-09-09 00:00:00',717.76,680,39673,62.83,'Molniya' from dual union all
select 20743,'020743','1990-071B','11S510','11S510',63313,'Reentered','1990-09-10 00:00:00','1990-08-23 00:00:00',91.59,206,500,62.82,'Intermediate' from dual union all
select 20744,'020744','1990-071C','BOZ','BOZ',63313,'Reentered','1990-09-02 00:00:00','1990-08-23 00:00:00',91.3,183,494,62.83,'Intermediate' from dual union all
select 20745,'020745','1990-071D','Blok-ML','Blok-ML No. 76731-472',63313,'Reentered','2007-08-19 00:00:00','1993-08-15 00:00:00',732.69,1200,39888,63.6,'Molniya' from dual union all
select 20746,'020746','1990-038D','Kosmos-2075 SS 1','ESO',63280,'Reentered','1990-10-02 00:00:00','1990-08-13 00:00:00',94.67,481,525,73.98,'Intermediate' from dual union all
select 20747,'020747','1990-038E','Kosmos-2075 SS 2','ESO',63280,'Reentered','1990-09-28 00:00:00','1990-08-13 00:00:00',94.56,478,517,74.05,'Intermediate' from dual union all
select 20748,'020748','1990-038F','Kosmos-2075 SS 3','ESO',63280,'Reentered','1990-09-20 00:00:00','1990-08-13 00:00:00',94.12,464,488,74.07,'Intermediate' from dual union all
select 20749,'020749','1990-038G','Kosmos-2075 SS 4','ESO',63280,'Reentered','1990-09-20 00:00:00','1990-08-13 00:00:00',94.25,472,493,73.99,'Intermediate' from dual union all
select 2075,'002075','1966-012AN','deb OPS 3031','',13419,'Reentered','1966-02-17 00:00:00','1966-02-16 00:00:00',89,170,281,96.3,'Sun-Synch' from dual union all
select 20750,'020750','1990-038H','Kosmos-2075 SS 5','ESO',63280,'Reentered','1990-09-22 00:00:00','1990-08-13 00:00:00',94.36,475,501,74.09,'Intermediate' from dual union all
select 20751,'020751','1990-038J','Kosmos-2075 SS 6','ESO',63280,'Reentered','1990-09-25 00:00:00','1990-08-14 00:00:00',94.41,476,504,73.95,'Intermediate' from dual union all
select 20752,'020752','1990-072A','Progress M-4','Progress 7K-TGM No. 204',63314,'Deorbited','1990-09-20 00:00:00','1990-08-22 00:00:00',92.26,368,403,51.61,'Intermediate' from dual union all
select 20753,'020753','1990-072B','11S510','11S510',63314,'Reentered','1990-08-16 00:00:00','1990-08-16 00:00:00',87.61,151,161,51.63,'Intermediate' from dual union all
select 20754,'020754','1990-073A','Resurs-F','Resurs-F1 14F43 No. 49',63315,'Landed','1990-09-01 00:00:00','1990-08-23 00:00:00',89.73,254,269,82.34,'Intermediate' from dual union all
select 20755,'020755','1990-073B','11S510','11S510',63315,'Reentered','1990-08-18 00:00:00','1990-08-16 00:00:00',88.49,178,222,82.3,'Intermediate' from dual union all
select 20756,'020756','1990-060C','KDU','KDU',63302,'Reentered','1990-08-19 00:00:00','1990-08-16 00:00:00',89.6,250,260,82.3,'Intermediate' from dual union all
select 20757,'020757','1990-060D','Blenda','Sunshade',63302,'Reentered','1990-08-18 00:00:00','1990-08-16 00:00:00',89.57,244,262,82.31,'Intermediate' from dual union all
select 20758,'020758','1990-060E','Solnechnaya Batareya','Solar panel module',63302,'Reentered','1990-08-22 00:00:00','1990-08-21 00:00:00',89.18,191,276,82.28,'Intermediate' from dual union all
select 20759,'020759','1990-043L','deb MACSAT','',63285,'Reentered','2000-01-25 00:00:00','1990-08-22 00:00:00',97.89,616,700,89.66,'Polar' from dual union all
select 2076,'002076','1966-012AP','deb OPS 3031','',13419,'Reentered','1966-02-19 00:00:00','1966-02-16 00:00:00',89.17,133,333,96.4,'Sun-Synch' from dual union all
select 20760,'020760','1965-108AE','deb Transtage 8','',11858,'In Earth orbit','','1986-08-02 00:00:00',531.09,638,30054,27.04,'Highly Ellip' from dual union all
select 20761,'020761','1990-060F','KDU part','KDU part',63302,'Reentered','1990-08-18 00:00:00','1990-08-17 00:00:00',90.42,233,358,82.29,'Intermediate' from dual union all
select 20762,'020762','1990-074A','Marcopolo 2','BSB R2',63316,'In Earth orbit','','1990-09-21 00:00:00',1436.14,35717,35858,.3,'Stationary' from dual union all
select 20763,'020763','1990-074B','Delta 198','Delta SSPS AJ10-118K',63316,'In Earth orbit','','1991-03-10 00:00:00',102.62,474,1292,24.77,'Equatorial' from dual union all
select 20764,'020764','1990-074C','Star 48B','Star 48B S/N 10059-18',63316,'In Earth orbit','','1993-08-11 00:00:00',669.62,422,37530,20.58,'GEO Transfer' from dual union all
select 20765,'020765','1990-075A','Kosmos-2096','US-PM',63317,'Deorbited','1992-08-30 00:00:00','1991-03-28 00:00:00',92.77,404,417,65.01,'Intermediate' from dual union all
select 20766,'020766','1990-075B','11S692','11S692',63317,'Reentered','1990-08-24 00:00:00','1990-08-23 00:00:00',88.5,106,295,64.98,'Intermediate' from dual union all
select 20767,'020767','1990-076A','Kosmos-2097','Oko',63318,'Reentered','2014-12-28 00:00:00','1990-09-27 00:00:00',718.11,634,39737,62.99,'Molniya' from dual union all
select 20768,'020768','1990-076B','11S510','11S510',63318,'Reentered','1990-09-26 00:00:00','1990-09-04 00:00:00',92.05,207,544,62.77,'Intermediate' from dual union all
select 20769,'020769','1990-076C','BOZ','BOZ',63318,'Reentered','1990-09-11 00:00:00','1990-09-03 00:00:00',91.03,171,479,62.82,'Intermediate' from dual union all
select 2077,'002077','1966-012AQ','deb OPS 3031','',13419,'Reentered','1966-02-18 00:00:00','1966-02-16 00:00:00',91.54,323,378,96.5,'Sun-Synch' from dual union all
select 20770,'020770','1990-076D','Blok-2BL','Blok-2BL No. 76747-689',63318,'Reentered','2008-10-19 00:00:00','1993-08-16 00:00:00',707.89,1474,38392,65.31,'GEO Transfer' from dual union all
select 20771,'020771','1990-077A','Yuri 3A','BS 3A',63319,'In Earth orbit','','1990-10-25 00:00:00',1436.04,35771,35800,.14,'Stationary' from dual union all
select 20772,'020772','1990-077B','H-I Stage 2 S/N H-22(F)','LE-5',63319,'Reentered','1990-09-05 00:00:00','1990-09-03 00:00:00',89.22,162,310,30.66,'Intermediate' from dual union all
select 20773,'020773','1990-077C','UM129A','UM129A',63319,'Reentered','1991-02-16 00:00:00','1990-09-26 00:00:00',668.28,209,37675,28.78,'GEO Transfer' from dual union all
select 20774,'020774','1990-078A','Kosmos-2098','Vektor',63320,'In Earth orbit','','1990-09-28 00:00:00',109.08,396,1973,82.96,'Intermediate' from dual union all
select 20775,'020775','1990-078B','S3M','S3M',63320,'In Earth orbit','','1991-03-01 00:00:00',108.63,380,1947,82.97,'Intermediate' from dual union all
select 20776,'020776','1990-079A','Skynet 4C','Skynet 4C',63321,'In Earth orbit','','1990-09-30 00:00:00',1436.06,35785,35787,4.31,'Inclined GEO' from dual union all
select 20777,'020777','1990-079B','Eutelsat II F-1','Eutelsat IIF1',63321,'In Earth orbit','','1990-11-14 00:00:00',1382.6,32555,36911,.44,'Near-synch' from dual union all
select 20778,'020778','1990-079C','Ariane H10 V38','Ariane H10 S/N T410',63321,'Reentered','1993-08-28 00:00:00','1993-04-01 00:00:00',270.46,143,14808,7.42,'Highly Ellip' from dual union all
select 20779,'020779','1990-080A','Kosmos-2099','Oblik No. 815',63322,'Landed','1990-09-14 00:00:00','1990-09-02 00:00:00',90.34,237,346,82.32,'Intermediate' from dual union all
select 2078,'002078','1966-012AR','deb OPS 3031','',13419,'Reentered','1966-02-19 00:00:00','1966-02-16 00:00:00',89.36,153,333,96.2,'Sun-Synch' from dual union all
select 20780,'020780','1990-080B','11S510','11S510',63322,'Reentered','1990-09-01 00:00:00','1990-09-01 00:00:00',87.17,128,141,82.28,'Intermediate' from dual union all
select 20781,'020781','1990-079D','Spelda 10 S/N UV8','Spelda 10',63321,'Reentered','1991-09-28 00:00:00','1991-09-22 00:00:00',120.41,138,3260,7.3,'Medium' from dual union all
select 20782,'020782','1990-073C','KDU','KDU',63315,'Reentered','1990-09-21 00:00:00','1990-09-04 00:00:00',89.61,248,263,82.34,'Intermediate' from dual union all
select 20783,'020783','1990-073D','Blenda','Sunshade',63315,'Reentered','1990-09-07 00:00:00','1990-09-04 00:00:00',90.79,240,387,82.33,'Intermediate' from dual union all
select 20784,'020784','1990-073E','KDU part','KDU part',63315,'Reentered','1990-09-05 00:00:00','1990-09-04 00:00:00',88.48,186,213,82.34,'Intermediate' from dual union all
select 20785,'020785','1990-073F','KDU part','KDU part',63315,'Reentered','1990-09-03 00:00:00','1990-09-03 00:00:00',88.12,176,187,82.33,'Intermediate' from dual union all
select 20786,'020786','1990-073G','KDU part','KDU part',63315,'Reentered','1990-09-04 00:00:00','1990-09-04 00:00:00',89.64,183,331,82.38,'Intermediate' from dual union all
select 20787,'020787','1990-073H','KDU part','KDU part',63315,'Reentered','1990-09-03 00:00:00','1990-09-03 00:00:00',88.11,171,191,82.31,'Intermediate' from dual union all
select 20788,'020788','1990-081A','Feng Yun 1B','feng yun yi yi',63323,'In Earth orbit','','1990-10-08 00:00:00',102.8,884,899,98.93,'Sun-Synch' from dual union all
select 20789,'020789','1990-081B','Da Qui Weixing 1','DQ-1A',63323,'Reentered','1991-03-11 00:00:00','1990-10-08 00:00:00',102.39,843,901,98.97,'Sun-Synch' from dual union all
select 2079,'002079','1965-082JS','deb Transtage 4','',11832,'Reentered','1983-04-12 00:00:00','1983-04-08 00:00:00',90.04,272,280,31.83,'Intermediate' from dual union all
select 20790,'020790','1990-081C','Da Qui Weixing 2','DQ-1B',63323,'Reentered','1991-07-24 00:00:00','1990-10-04 00:00:00',102.56,860,900,98.94,'Sun-Synch' from dual union all
select 20791,'020791','1990-081D','CZ-4 Stage 3','CZ-4 Stage 3',63323,'In Earth orbit','','1990-10-05 00:00:00',102.75,882,895,98.95,'Sun-Synch' from dual union all
select 20792,'020792','1990-081E','DQ 1 Canister','Canister',63323,'In Earth orbit','','1990-10-04 00:00:00',102.69,874,897,98.94,'Sun-Synch' from dual union all
select 20793,'020793','1990-081F','DQ 2 Canister','Canister',63323,'In Earth orbit','','1993-08-08 00:00:00',102.53,870,887,98.83,'Sun-Synch' from dual union all
select 20794,'020794','1990-082A','Resurs-F','Resurs-F1 14F43 No. 51',63324,'Landed','1990-09-21 00:00:00','1990-09-09 00:00:00',89.84,260,273,82.57,'Intermediate' from dual union all
select 20795,'020795','1990-082B','11S510','11S510',63324,'Reentered','1990-09-09 00:00:00','1990-09-09 00:00:00',87.34,136,149,82.57,'Intermediate' from dual union all
select 20796,'020796','1990-080C','Sensor cover','Sensor cover',63322,'Reentered','1990-09-10 00:00:00','1990-09-09 00:00:00',88.68,189,230,82.27,'Intermediate' from dual union all
select 20797,'020797','1990-081G','DQ 1 Canister part','',63323,'In Earth orbit','','1990-10-05 00:00:00',102.74,881,896,98.93,'Sun-Synch' from dual union all
select 20798,'020798','1990-081H','DQ 2 Canister part','',63323,'In Earth orbit','','1990-10-08 00:00:00',102.73,881,895,98.94,'Sun-Synch' from dual;
insert into SATELLITE
select 20799,'020799','1977-048G','SVM-5','SVM-5',42176,'In Earth orbit','','1990-09-07 00:00:00',1431.67,34802,36597,9.92,'Inclined Drift' from dual union all
select 208,'000208','61 A Iota 1','Mercury MA-5','Mercury SC9',7255,'Landed','1961-11-29 00:00:00','1961-11-29 00:00:00',88.44,158,237,32.58,'Intermediate' from dual union all
select 2080,'002080','1965-082JT','deb Transtage 4','',11832,'Reentered','1966-09-23 00:00:00','1966-01-31 00:00:00',98.04,618,712,32.1,'Intermediate' from dual union all
select 20800,'020800','1989-020E','Mage 1','Mage 1',62297,'In Earth orbit','','1990-09-10 00:00:00',1433.58,35226,36249,.72,'Drift GEO' from dual union all
select 20801,'020801','1978-062D','SVM-5','SVM-5',45257,'In Earth orbit','','1990-09-09 00:00:00',1451.11,32064,40096,9.3,'Near-synch' from dual union all
select 20802,'020802','1989-100P','Kosmos-2053 SS 12','ESO',62377,'Reentered','1990-10-30 00:00:00','1990-10-27 00:00:00',90.98,312,334,73.5,'Intermediate' from dual union all
select 20803,'020803','1989-100Q','Kosmos-2053 SS 13','ESO',62377,'Reentered','1990-10-30 00:00:00','1990-10-27 00:00:00',90.83,305,325,73.5,'Intermediate' from dual union all
select 20804,'020804','1990-083A','Kosmos-2100','Parus',63325,'In Earth orbit','','1990-10-14 00:00:00',104.84,961,1012,82.94,'Intermediate' from dual union all
select 20805,'020805','1990-083B','S3M','S3M',63325,'In Earth orbit','','1991-02-14 00:00:00',104.71,957,1004,82.93,'Intermediate' from dual union all
select 20806,'020806','1990-080D','KDU','KDU',63322,'Reentered','1990-09-23 00:00:00','1990-09-17 00:00:00',89.13,213,250,82.31,'Intermediate' from dual union all
select 20807,'020807','1990-080E','Blenda','Sunshade',63322,'Reentered','1990-09-16 00:00:00','1990-09-16 00:00:00',88.14,175,190,82.34,'Intermediate' from dual union all
select 20808,'020808','1990-080F','KDU part','KDU part',63322,'Reentered','1990-09-15 00:00:00','1990-09-15 00:00:00',90.03,212,340,82.32,'Intermediate' from dual union all
select 20809,'020809','1990-080G','KDU part','KDU part',63322,'Reentered','1990-09-17 00:00:00','1990-09-16 00:00:00',89.83,217,316,82.3,'Intermediate' from dual union all
select 2081,'002081','1965-082JU','deb Transtage 4','',11832,'Reentered','1966-11-08 00:00:00','1966-10-31 00:00:00',90.14,270,292,32.17,'Intermediate' from dual union all
select 20810,'020810','1990-080H','KDU part','KDU part',63322,'Reentered','1990-09-16 00:00:00','1990-09-15 00:00:00',88.31,179,202,82.32,'Intermediate' from dual union all
select 20811,'020811','1990-065C','G-13A Canister','Large Ba Canister',63307,'In Earth orbit','','1990-09-14 00:00:00',591.5,344,33583,18.26,'Highly Ellip' from dual union all
select 20812,'020812','1990-072C','Fokus experiment','Fokus part',63314,'Reentered','1990-10-16 00:00:00','1990-09-20 00:00:00',91.66,340,372,51.61,'Intermediate' from dual union all
select 20813,'020813','1990-084A','Molniya-3','Molniya-3 No. 54',63326,'Reentered','2009-07-08 00:00:00','1990-10-20 00:00:00',717.78,489,39866,62.85,'Molniya' from dual union all
select 20814,'020814','1990-084B','11S510','11S510',63326,'Reentered','1990-10-03 00:00:00','1990-09-25 00:00:00',90.18,202,365,62.84,'Intermediate' from dual union all
select 20815,'020815','1990-084C','BOZ','BOZ',63326,'Reentered','1990-10-01 00:00:00','1990-09-25 00:00:00',90.1,185,374,62.83,'Intermediate' from dual union all
select 20816,'020816','1990-084D','Blok-ML','Blok-ML',63326,'Reentered','2009-08-20 00:00:00','1993-08-15 00:00:00',731.71,1317,39723,63.11,'Molniya' from dual union all
select 20817,'020817','1990-082C','KDU','KDU',63324,'Reentered','1990-10-13 00:00:00','1990-09-23 00:00:00',89.63,249,264,82.57,'Intermediate' from dual union all
select 20818,'020818','1990-082D','KDU part','KDU part',63324,'Reentered','1990-09-23 00:00:00','1990-09-23 00:00:00',88.36,186,201,82.57,'Intermediate' from dual union all
select 20819,'020819','1990-082E','Blenda','Sunshade',63324,'Reentered','1990-09-25 00:00:00','1990-09-25 00:00:00',89.79,217,312,82.58,'Intermediate' from dual union all
select 2082,'002082','1965-082JV','deb Transtage 4','',11832,'Reentered','1979-06-16 00:00:00','1966-01-30 00:00:00',100.28,739,805,31.52,'Intermediate' from dual union all
select 20820,'020820','1990-082F','KDU part','KDU part',63324,'Reentered','1990-09-25 00:00:00','1990-09-24 00:00:00',88.48,183,216,82.57,'Intermediate' from dual union all
select 20821,'020821','1989-100R','Kosmos-2053 SS 14','ESO',62377,'Reentered','1990-11-16 00:00:00','1990-11-09 00:00:00',92.19,374,390,73.5,'Intermediate' from dual union all
select 20822,'020822','1989-100S','Kosmos-2053 SS 15','ESO',62377,'Reentered','1990-11-21 00:00:00','1990-11-19 00:00:00',90.28,279,298,73.48,'Intermediate' from dual union all
select 20823,'020823','1989-100T','Kosmos-2053 SS 16','ESO',62377,'Reentered','1990-11-16 00:00:00','1990-11-08 00:00:00',92.24,380,389,73.51,'Intermediate' from dual union all
select 20824,'020824','1990-085A','Progress M-5','Progress 7K-TGM No. 206',63327,'Deorbited','1990-11-28 00:00:00','1990-10-31 00:00:00',92.36,370,411,51.61,'Intermediate' from dual union all
select 20825,'020825','1990-085B','11S510','11S510',63327,'Reentered','1990-09-28 00:00:00','1990-09-28 00:00:00',87.99,165,184,51.59,'Intermediate' from dual union all
select 20826,'020826','1990-086A','Meteor-2','Meteor-2 No. 25',63328,'In Earth orbit','','1990-10-31 00:00:00',104.09,943,961,82.53,'Intermediate' from dual union all
select 20827,'020827','1990-086B','S5M','S5M',63328,'In Earth orbit','','1991-02-28 00:00:00',104.07,942,959,82.52,'Intermediate' from dual union all
select 20828,'020828','1990-087A','Kosmos-2101','Orlets',63329,'Exploded','1990-11-30 00:00:00','1990-10-31 00:00:00',89.7,217,303,64.76,'Intermediate' from dual union all
select 20829,'020829','1990-087B','11S510','11S510',63329,'Reentered','1990-10-04 00:00:00','1990-10-01 00:00:00',89.07,170,287,64.78,'Intermediate' from dual union all
select 2083,'002083','1965-082JW','deb Transtage 4','',11832,'Reentered','1989-06-20 00:00:00','1989-06-20 00:00:00',90.71,304,315,31,'Intermediate' from dual union all
select 20830,'020830','1990-088A','GPS II-9','Navstar GPS SVN 15',63330,'In Earth orbit','','1990-10-31 00:00:00',717.94,19972,20391,54.9,'Medium' from dual union all
select 20831,'020831','1990-088B','Delta 199','Delta SSPS AJ10-118K',63330,'Reentered','1990-10-10 00:00:00','1990-10-09 00:00:00',88.67,154,264,30.83,'Intermediate' from dual union all
select 20832,'020832','1990-088C','Star 48B','Star 48B S/N 10060-18',63330,'Reentered','1991-06-18 00:00:00','1990-11-04 00:00:00',345.51,156,19708,37.68,'Highly Ellip' from dual union all
select 20833,'020833','1965-098K','Alouette 2 part','',11848,'Reentered','1992-02-11 00:00:00','1990-11-25 00:00:00',116.3,510,2519,79.81,'Intermediate' from dual union all
select 20834,'020834','1989-080L','deb S5M','',62357,'Reentered','2017-04-10 00:00:00','1990-10-21 00:00:00',113.96,498,2319,82.55,'Intermediate' from dual union all
select 20835,'020835','1975-011F','SVM-5','SVM-5',36458,'In Earth orbit','','1984-06-03 00:00:00',1460.6,35873,36656,6.12,'Inclined Drift' from dual union all
select 20836,'020836','1974-060F','Blok DM','11S86 No. 2L',33663,'In Earth orbit','','1977-07-14 00:00:00',1436.04,35675,35896,2.41,'Stationary' from dual union all
select 20837,'020837','1981-057F','Mage 1','Mage 1',52580,'In Earth orbit','','1990-10-14 00:00:00',1449.08,35651,36430,7.37,'Inclined Drift' from dual union all
select 20838,'020838','1990-089A','Fanhui Shi Weixing (12)','Jianbing-1A No. 3',63331,'Reentered','1990-10-23 00:00:00','1990-10-12 00:00:00',89.44,203,291,56.97,'Intermediate' from dual union all
select 20839,'020839','1990-089B','CZ-2C Stage 2','CZ-2C Stage 2',63331,'Reentered','1990-10-15 00:00:00','1990-10-10 00:00:00',89.16,196,270,56.98,'Intermediate' from dual union all
select 2084,'002084','1965-082JX','deb Transtage 4','',11832,'Reentered','1981-08-29 00:00:00','1981-08-26 00:00:00',90.85,312,321,32.02,'Intermediate' from dual union all
select 20840,'020840','1990-089C','CZ-2C sep motor cover','',63331,'Reentered','1990-10-07 00:00:00','1990-10-05 00:00:00',89.17,196,271,56.96,'Intermediate' from dual union all
select 20841,'020841','1990-090A','Discovery (STS-41)','OV-103',63332,'Landed','1990-10-10 00:00:00','1990-10-08 00:00:00',90.36,283,302,28.47,'Intermediate' from dual union all
select 20842,'020842','1990-090B','Ulysses','Ulysses',63332,'Deep Space','1990-10-06 00:00:00','1990-10-06 00:00:00',28.19,310,19204,28.47,'Earth Escape' from dual union all
select 20843,'020843','1990-090C','IUS-17 SRM-1','Orbus 21',63332,'In Earth orbit','','1990-11-06 00:00:00',563.76,317,32139,28.53,'Highly Ellip' from dual union all
select 20844,'020844','1990-090D','IUS-17 SRM-2','Orbus 6E',63332,'Deep Space','1990-10-07 00:00:00','1990-10-06 00:00:00',903.67,309,74990,28.47,'Earth Escape' from dual union all
select 20845,'020845','1990-090E','PAM-S','Star 48B S/N 10058-24',63332,'Deep Space','1990-10-06 00:00:00','1990-10-06 00:00:00',28.19,310,19204,28.47,'Earth Escape' from dual union all
select 20846,'020846','1990-081J','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-04 00:00:00',105.91,876,1198,99.02,'Sun-Synch' from dual union all
select 20847,'020847','1990-081K','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-03-27 00:00:00',101.34,786,858,98.85,'Sun-Synch' from dual union all
select 20848,'020848','1990-081L','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-07 00:00:00',105.49,841,1194,98.97,'Sun-Synch' from dual union all
select 20849,'020849','1990-081M','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-08 00:00:00',103.13,873,941,98.85,'Sun-Synch' from dual union all
select 2085,'002085','1966-008C','ESSA 1 despin weight','',13415,'Reentered','2002-08-28 00:00:00','1966-10-30 00:00:00',99.21,687,754,97.77,'Sun-Synch' from dual union all
select 20850,'020850','1990-081N','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-04 00:00:00',101.27,759,878,98.81,'Sun-Synch' from dual union all
select 20851,'020851','1990-081P','deb CZ-4 Stage 3','',63323,'Reentered','2001-10-14 00:00:00','1993-08-05 00:00:00',99.86,695,809,98.83,'Sun-Synch' from dual union all
select 20852,'020852','1990-081Q','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-07-31 00:00:00',102.61,868,897,98.85,'Sun-Synch' from dual union all
select 20853,'020853','1990-081R','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-07 00:00:00',102.39,844,900,98.84,'Sun-Synch' from dual union all
select 20854,'020854','1990-081S','deb CZ-4 Stage 3','',63323,'Reentered','2000-03-25 00:00:00','1993-08-07 00:00:00',99.04,650,776,98.88,'Sun-Synch' from dual union all
select 20855,'020855','1990-081T','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-07-30 00:00:00',102.32,850,886,98.82,'Sun-Synch' from dual union all
select 20856,'020856','1990-081U','deb CZ-4 Stage 3','',63323,'Reentered','1997-07-07 00:00:00','1993-08-08 00:00:00',96.55,540,647,98.86,'Sun-Synch' from dual union all
select 20857,'020857','1990-081V','deb CZ-4 Stage 3','',63323,'Reentered','2013-11-20 00:00:00','1993-07-31 00:00:00',101.23,786,848,98.86,'Sun-Synch' from dual union all
select 20858,'020858','1990-081W','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-05 00:00:00',101.6,799,870,98.83,'Sun-Synch' from dual union all
select 20859,'020859','1990-081X','deb CZ-4 Stage 3','',63323,'Reentered','2015-08-20 00:00:00','1993-08-07 00:00:00',101.17,776,852,98.85,'Sun-Synch' from dual union all
select 2086,'002086','1965-109C','NNS O-6 despin weight','',11859,'Reentered','2004-02-17 00:00:00','1966-01-27 00:00:00',103.74,891,980,89.18,'Polar' from dual union all
select 20860,'020860','1990-081Y','deb CZ-4 Stage 3','',63323,'Reentered','2014-07-27 00:00:00','1993-08-06 00:00:00',100.35,701,849,98.74,'Sun-Synch' from dual union all
select 20861,'020861','1990-081Z','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-02 00:00:00',102.22,835,893,98.84,'Sun-Synch' from dual union all
select 20862,'020862','1990-081AA','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-08 00:00:00',106.38,874,1244,99,'Sun-Synch' from dual union all
select 20863,'020863','1990-081AB','deb CZ-4 Stage 3','',63323,'Reentered','1991-01-10 00:00:00','1991-01-09 00:00:00',88.45,179,217,98.9,'Sun-Synch' from dual union all
select 20864,'020864','1990-081AC','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-03 00:00:00',103.66,871,993,98.88,'Sun-Synch' from dual union all
select 20865,'020865','1990-081AD','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-06 00:00:00',101.28,785,854,98.84,'Sun-Synch' from dual union all
select 20866,'020866','1990-081AE','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-03 00:00:00',103.05,847,959,98.87,'Sun-Synch' from dual union all
select 20867,'020867','1990-081AF','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-06 00:00:00',102.97,822,977,98.87,'Sun-Synch' from dual union all
select 20868,'020868','1990-081AG','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-05 00:00:00',104.82,853,1119,98.97,'Sun-Synch' from dual union all
select 20869,'020869','1990-081AH','deb CZ-4 Stage 3','',63323,'Reentered','2000-07-31 00:00:00','1993-08-07 00:00:00',99.48,673,795,98.87,'Sun-Synch' from dual union all
select 2087,'002087','61 Omicron 212','deb r/b Ablestar','',7273,'Reentered','1979-03-13 00:00:00','1979-02-20 00:00:00',93.74,439,476,65.73,'Intermediate' from dual union all
select 20870,'020870','1990-081AJ','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-06 00:00:00',102.71,860,913,98.85,'Sun-Synch' from dual union all
select 20871,'020871','1990-081AK','deb CZ-4 Stage 3','',63323,'Reentered','2003-08-18 00:00:00','1993-08-01 00:00:00',100.08,695,830,98.81,'Sun-Synch' from dual union all
select 20872,'020872','1990-091A','SBS 6','SBS 6',63333,'In Earth orbit','','1990-12-03 00:00:00',1436.09,35777,35796,.05,'Stationary' from dual union all
select 20873,'020873','1990-091B','Galaxy 6','Westar 6S',63333,'In Earth orbit','','1990-11-14 00:00:00',1436.09,35781,35792,.07,'Stationary' from dual union all
select 20874,'020874','1990-091C','Ariane H10 V39','Ariane H10 S/N T411',63333,'Reentered','2016-09-20 00:00:00','1993-08-16 00:00:00',583.89,265,33261,7.6,'Highly Ellip' from dual union all
select 20875,'020875','1990-091D','Spelda 10 S/N UV9','Spelda 10',63333,'Reentered','1991-10-02 00:00:00','1990-11-03 00:00:00',634.6,209,35959,7.11,'GEO Transfer' from dual union all
select 20876,'020876','1990-081AL','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',102.37,852,890,98.94,'Sun-Synch' from dual union all
select 20877,'020877','1990-081AM','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',102.48,842,910,98.92,'Sun-Synch' from dual union all
select 20878,'020878','1990-081AN','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',102.66,853,916,98.92,'Sun-Synch' from dual union all
select 20879,'020879','1990-081AP','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',104.31,869,1056,98.97,'Sun-Synch' from dual union all
select 2088,'002088','1965-093D','SR 8 expt. cover','',11843,'In Earth orbit','','1966-11-03 00:00:00',101.38,716,931,59.72,'Intermediate' from dual union all
select 20880,'020880','1990-081AQ','deb CZ-4 Stage 3','',63323,'Reentered','2001-11-12 00:00:00','1990-10-18 00:00:00',100.29,659,886,98.93,'Sun-Synch' from dual union all
select 20881,'020881','1990-081AR','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',102.69,806,966,98.97,'Sun-Synch' from dual union all
select 20882,'020882','1990-081AS','deb CZ-4 Stage 3','',63323,'Reentered','2006-12-13 00:00:00','1990-10-18 00:00:00',101.92,811,889,98.94,'Sun-Synch' from dual union all
select 20883,'020883','1990-081AT','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',102.57,866,894,98.94,'Sun-Synch' from dual union all
select 20884,'020884','1990-081AU','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',103.38,886,951,98.92,'Sun-Synch' from dual union all
select 20885,'020885','1990-081AV','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',103.8,888,989,98.93,'Sun-Synch' from dual union all
select 20886,'020886','1990-081AW','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-17 00:00:00',106.73,884,1267,98.99,'Sun-Synch' from dual union all
select 20887,'020887','1990-081AX','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',102.33,846,891,98.94,'Sun-Synch' from dual union all
select 20888,'020888','1990-081AY','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',104.71,743,1218,98.93,'Sun-Synch' from dual;
insert into SATELLITE
select 20889,'020889','1990-081AZ','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',103.87,871,1012,98.93,'Sun-Synch' from dual union all
select 2089,'002089','1966-015C','deb Kosmos-110','',13422,'Reentered','1966-03-05 00:00:00','1966-02-22 00:00:00',94.42,165,816,51.9,'Intermediate' from dual union all
select 20890,'020890','1990-081BA','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',102.69,875,897,98.93,'Sun-Synch' from dual union all
select 20891,'020891','1990-081BB','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',105.92,871,1203,99.01,'Sun-Synch' from dual union all
select 20892,'020892','1990-081BC','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',105.55,881,1160,98.96,'Sun-Synch' from dual union all
select 20893,'020893','1990-081BD','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',102.62,878,888,98.94,'Sun-Synch' from dual union all
select 20894,'020894','1990-081BE','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',102.31,848,887,98.94,'Sun-Synch' from dual union all
select 20895,'020895','1990-081BF','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',102.13,831,888,98.94,'Sun-Synch' from dual union all
select 20896,'020896','1990-081BG','deb CZ-4 Stage 3','',63323,'Reentered','2000-11-13 00:00:00','1990-10-18 00:00:00',102.19,838,886,98.93,'Sun-Synch' from dual union all
select 20897,'020897','1990-081BH','deb CZ-4 Stage 3','',63323,'Reentered','1993-01-21 00:00:00','1990-10-18 00:00:00',100.06,637,886,98.93,'Sun-Synch' from dual union all
select 20898,'020898','1990-081BJ','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',101.79,791,896,98.93,'Sun-Synch' from dual union all
select 20899,'020899','1990-081BK','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',102.98,888,911,98.94,'Sun-Synch' from dual union all
select 209,'000209','61 A Iota 2','Atlas 93D','Atlas D 93D',7255,'Reentered','1961-11-30 00:00:00','1961-11-29 00:00:00',87.43,130,165,32.58,'Intermediate' from dual union all
select 2090,'002090','1966-015D','deb Kosmos-110','',13422,'Reentered','1966-03-05 00:00:00','1966-02-22 00:00:00',93.26,180,689,51.8,'Intermediate' from dual union all
select 20900,'020900','1990-081BL','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',103.89,877,1008,98.9,'Sun-Synch' from dual union all
select 20901,'020901','1990-081BM','deb CZ-4 Stage 3','',63323,'Reentered','2014-05-30 00:00:00','1990-10-18 00:00:00',101.58,778,889,98.93,'Sun-Synch' from dual union all
select 20902,'020902','1990-081BN','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',104.18,888,1024,98.95,'Sun-Synch' from dual union all
select 20903,'020903','1990-081BP','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',101.72,792,889,98.91,'Sun-Synch' from dual union all
select 20904,'020904','1990-081BQ','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-17 00:00:00',102.54,848,910,98.94,'Sun-Synch' from dual union all
select 20905,'020905','1990-081BR','deb CZ-4 Stage 3','',63323,'Reentered','2016-09-24 00:00:00','1990-10-18 00:00:00',102.59,874,889,98.93,'Sun-Synch' from dual union all
select 20906,'020906','1990-081BS','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',104.42,888,1047,98.95,'Sun-Synch' from dual union all
select 20907,'020907','1990-081BT','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-17 00:00:00',102.41,858,888,98.94,'Sun-Synch' from dual union all
select 20908,'020908','1990-081BU','deb CZ-4 Stage 3','',63323,'Reentered','1990-11-29 00:00:00','1990-10-17 00:00:00',104.43,889,1047,98.95,'Sun-Synch' from dual union all
select 20909,'020909','1990-092A','Kosmos-2102','Kobal''t No. 528',63334,'Landed','1990-12-12 00:00:00','1990-10-23 00:00:00',89.81,187,344,62.85,'Intermediate' from dual union all
select 2091,'002091','1966-016A','ESSA 2','Tiros OT2',13423,'In Earth orbit','','1966-10-24 00:00:00',113.46,1357,1413,100.96,'Sun-Synch' from dual union all
select 20910,'020910','1990-092B','11S510','11S510',63334,'Reentered','1990-10-20 00:00:00','1990-10-18 00:00:00',89.15,174,291,62.83,'Intermediate' from dual union all
select 20911,'020911','1989-100U','Kosmos-2053 SS 17','ESO',62377,'Reentered','1990-11-17 00:00:00','1990-10-17 00:00:00',93.99,462,478,73.53,'Intermediate' from dual union all
select 20912,'020912','1990-081BV','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',102.38,855,888,98.94,'Sun-Synch' from dual union all
select 20913,'020913','1990-081BW','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-18 00:00:00',102.87,811,978,98.97,'Sun-Synch' from dual union all
select 20914,'020914','1990-081BX','deb CZ-4 Stage 3','',63323,'Reentered','2001-10-02 00:00:00','1990-10-18 00:00:00',99.26,561,886,98.95,'Sun-Synch' from dual union all
select 20915,'020915','1990-081BY','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1990-10-20 00:00:00',102.45,854,896,98.96,'Sun-Synch' from dual union all
select 20916,'020916','1990-081BZ','deb CZ-4 Stage 3','',63323,'Reentered','1991-02-02 00:00:00','1990-10-23 00:00:00',102.84,844,942,98.98,'Sun-Synch' from dual union all
select 20917,'020917','1986-017CQ','EO-7 EVA-1 debris','(Kvant-2 hatch)',59210,'Reentered','1990-10-31 00:00:00','1990-10-30 00:00:00',91.74,347,373,51.6,'Intermediate' from dual union all
select 20918,'020918','1990-093A','INMARSAT II F-1','INMARSAT II F-1',63335,'In Earth orbit','','1991-02-19 00:00:00',1436.01,35701,35869,2.41,'Stationary' from dual union all
select 20919,'020919','1990-093B','Delta 200','Delta SSPS AJ10-118K',63335,'Reentered','2000-03-26 00:00:00','1990-11-29 00:00:00',100.31,377,1170,24.75,'Equatorial' from dual union all
select 2092,'002092','1965-098H','Alouette sep clamp','',11848,'In Earth orbit','','1966-11-02 00:00:00',121.36,506,2978,79.86,'Medium' from dual union all
select 20920,'020920','1990-093C','Star 48B','Star 48B S/N 10062-18',63335,'Reentered','1991-03-03 00:00:00','1991-02-24 00:00:00',611.47,128,34844,23.28,'GEO Transfer' from dual union all
select 20921,'020921','1986-017CR','EO-7 EVA-1 debris','(Kvant-2 hatch)',59210,'Reentered','1990-11-09 00:00:00','1990-11-09 00:00:00',89.79,258,271,51.6,'Intermediate' from dual union all
select 20922,'020922','1986-017CS','EO-7 EVA-1 debris','(Kvant-2 hatch)',59210,'Reentered','1990-11-08 00:00:00','1990-11-08 00:00:00',89.76,238,288,51.6,'Intermediate' from dual union all
select 20923,'020923','1990-094A','Gorizont 32','Gorizont No. 32L',63336,'In Earth orbit','','1990-12-03 00:00:00',1436.11,35779,35795,1.37,'Stationary' from dual union all
select 20924,'020924','1990-094B','Perekhodnik','Sredniy perekhodnik',63336,'Reentered','1990-11-04 00:00:00','1990-11-04 00:00:00',87.72,157,167,51.65,'Intermediate' from dual union all
select 20925,'020925','1990-094C','8S812','8S812',63336,'Reentered','1990-11-05 00:00:00','1990-11-05 00:00:00',87.42,146,148,51.66,'Intermediate' from dual union all
select 20926,'020926','1990-094D','Blok DM2 No. 35L','11S861 No. 35L',63336,'In Earth orbit','','1991-09-19 00:00:00',1427.59,35463,35777,.69,'Drift GEO' from dual union all
select 20927,'020927','1990-094E','SOZ','SOZ',63336,'Reentered','1995-03-04 00:00:00','1991-06-01 00:00:00',602.06,155,34326,46.67,'GEO Transfer' from dual union all
select 20928,'020928','1990-094F','SOZ','SOZ',63336,'Reentered','1991-05-02 00:00:00','1990-12-06 00:00:00',631.1,249,35739,47.18,'GEO Transfer' from dual union all
select 20929,'020929','1990-095A','DSP F15','DSP 15',63337,'In Earth orbit','','2000-03-23 00:00:00',1436.09,35770,35803,4,'Inclined GEO' from dual union all
select 2093,'002093','1966-017A','Kosmos-111','E-6S No. 204 ISL',13424,'Reentered Attached','1966-03-03 00:00:00','1966-03-01 00:00:00',88.25,182,193,51.8,'Intermediate' from dual union all
select 20930,'020930','1990-095B','Titan 45K-6 Stage 2','LR91AJ11A 45K-6',63337,'Reentered','1990-11-13 00:00:00','1990-11-13 00:00:00',87.49,144,157,28.62,'Intermediate' from dual union all
select 20931,'020931','1990-095C','IUS-6 SRM-1','Orbus 21',63337,'In Earth orbit','2000-01-01 00:00:00','1990-11-13 00:00:00',630.55,160,35800,28.5,'GEO Transfer' from dual union all
select 20932,'020932','1990-095D','IUS-6 SRM-2','Orbus 6E',63337,'In Earth orbit','','1990-11-30 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 20933,'020933','1990-096A','Kosmos-2103','US-PM',63338,'Reentered','1991-04-03 00:00:00','1991-03-03 00:00:00',91.71,350,367,65.01,'Intermediate' from dual union all
select 20934,'020934','1990-096B','11S692','11S692',63338,'Reentered','1990-11-14 00:00:00','1990-11-14 00:00:00',88.14,107,258,64.97,'Intermediate' from dual union all
select 20935,'020935','1990-097A','Atlantis (STS-38)','OV-104',63339,'Landed','1990-11-20 00:00:00','1990-11-17 00:00:00',89.79,260,269,28.47,'Intermediate' from dual union all
select 20936,'020936','1990-098A','Kosmos-2104','Oblik No. 813',63340,'Landed','1990-12-04 00:00:00','1990-11-23 00:00:00',89.62,231,281,62.81,'Intermediate' from dual union all
select 20937,'020937','1990-098B','11S510','11S510',63340,'Reentered','1990-12-05 00:00:00','1990-11-26 00:00:00',89.77,222,305,62.79,'Intermediate' from dual union all
select 20938,'020938','1990-098C','Sensor cover','Sensor cover',63340,'Reentered','1990-11-19 00:00:00','1990-11-19 00:00:00',88.65,169,247,62.78,'Intermediate' from dual union all
select 20939,'020939','1990-098D','Sensor cover','Sensor cover',63340,'Reentered','1990-11-20 00:00:00','1990-11-20 00:00:00',88.46,171,226,62.77,'Intermediate' from dual union all
select 2094,'002094','1966-017B','11S510','11S510',13424,'Reentered','1966-03-01 00:00:00','1966-03-01 00:00:00',88.25,182,193,51.8,'Intermediate' from dual union all
select 20940,'020940','1990-098E','Sensor cover','Sensor cover',63340,'Reentered','1990-11-19 00:00:00','1990-11-19 00:00:00',88.5,178,223,62.82,'Intermediate' from dual union all
select 20941,'020941','1990-099A','Kosmos-2105','Oko',63341,'Reentered','2008-01-21 00:00:00','1990-12-20 00:00:00',717.6,595,39751,63.27,'Molniya' from dual union all
select 20942,'020942','1990-099B','11S510','11S510',63341,'Reentered','1990-12-23 00:00:00','1990-11-28 00:00:00',91.93,210,529,62.98,'Intermediate' from dual union all
select 20943,'020943','1990-099C','BOZ','BOZ',63341,'Reentered','1990-12-07 00:00:00','1990-11-28 00:00:00',91.12,176,483,63.01,'Intermediate' from dual union all
select 20944,'020944','1990-099D','Blok-2BL','Blok-2BL',63341,'Reentered','2007-07-08 00:00:00','1991-09-18 00:00:00',707.41,890,38952,64.36,'GEO Transfer' from dual union all
select 20945,'020945','1990-100A','Satcom C1','Satcom C1',63342,'In Earth orbit','','1990-12-25 00:00:00',1436.01,35775,35795,.05,'Stationary' from dual union all
select 20946,'020946','1990-100B','Gstar 4','Gstar 4',63342,'In Earth orbit','','1990-12-23 00:00:00',1436.06,35782,35790,.04,'Stationary' from dual union all
select 20947,'020947','1990-100C','Ariane H10 V40','Ariane H10 S/N T413',63342,'In Earth orbit','','1991-09-14 00:00:00',621.2,288,35189,7.37,'GEO Transfer' from dual union all
select 20948,'020948','1990-100D','Sylda 4400 No. 11','Sylda 4400',63342,'Reentered','1991-11-06 00:00:00','1991-09-19 00:00:00',268.96,160,14688,6.94,'Highly Ellip' from dual union all
select 20949,'020949','1990-101A','Molniya-1','Molniya-1T No. 70',63343,'Reentered','2005-08-27 00:00:00','1990-12-25 00:00:00',717.74,592,39761,62.9,'Molniya' from dual union all
select 2095,'002095','1966-017C','BOZ','BOZ',13424,'Reentered','1966-03-02 00:00:00','1966-03-01 00:00:00',88.25,182,193,51.8,'Intermediate' from dual union all
select 20950,'020950','1990-101B','11S510','11S510',63343,'Reentered','1990-12-26 00:00:00','1990-12-04 00:00:00',91.79,209,515,62.81,'Intermediate' from dual union all
select 20951,'020951','1990-101C','BOZ','BOZ',63343,'Reentered','1990-12-12 00:00:00','1990-12-04 00:00:00',91.13,172,488,62.84,'Intermediate' from dual union all
select 20952,'020952','1990-101D','Blok-ML','Blok-ML',63343,'Reentered','2004-12-15 00:00:00','1991-09-10 00:00:00',730.73,530,40462,63.56,'Molniya' from dual union all
select 20953,'020953','1990-102A','Gorizont 33','Gorizont No. 33L',63344,'In Earth orbit','','1990-12-24 00:00:00',1435.96,35760,35808,1.38,'Stationary' from dual union all
select 20954,'020954','1990-102B','8S812','8S812',63344,'Reentered','1990-11-25 00:00:00','1990-11-25 00:00:00',87.16,132,136,51.61,'Intermediate' from dual union all
select 20955,'020955','1990-102C','Perekhodnik','Sredniy perekhodnik',63344,'Reentered','1990-11-24 00:00:00','1990-11-24 00:00:00',87.6,151,161,51.61,'Intermediate' from dual union all
select 20956,'020956','1990-098F','KDU part','KDU part',63340,'Reentered','1990-12-07 00:00:00','1990-12-06 00:00:00',89.63,213,300,62.81,'Intermediate' from dual union all
select 20957,'020957','1990-102E','SOZ','SOZ',63344,'Reentered','1991-07-01 00:00:00','1991-06-11 00:00:00',627.16,163,35622,46.59,'GEO Transfer' from dual union all
select 20958,'020958','1990-102F','SOZ','SOZ',63344,'Reentered','1991-05-02 00:00:00','1991-06-27 00:00:00',584.52,225,33334,46.65,'Highly Ellip' from dual union all
select 20959,'020959','1990-103A','GPS IIA-1','Navstar GPS SVN 23',63345,'In Earth orbit','','1990-12-30 00:00:00',717.98,20072,20293,54.91,'Medium' from dual union all
select 2096,'002096','1966-016B','Altair 2 X-258C4 RH-92','X-258C4 RH-92',13423,'In Earth orbit','','1966-10-28 00:00:00',113.48,1354,1418,100.91,'Sun-Synch' from dual union all
select 20960,'020960','1990-103B','Delta 201','Delta SSPS AJ10-118K',63345,'Reentered','2001-10-01 00:00:00','1991-03-12 00:00:00',96.74,495,711,21.42,'Equatorial' from dual union all
select 20961,'020961','1990-103C','Star 48B','Star 48B S/N 10063-18',63345,'Reentered','1992-09-30 00:00:00','1991-09-20 00:00:00',218.39,147,11123,34.59,'Medium' from dual union all
select 20962,'020962','1975-100F','SVM-5','SVM-5',36547,'In Earth orbit','','1989-06-06 00:00:00',1412.55,34068,36581,9.76,'Inclined Drift' from dual union all
select 20963,'020963','1990-097B','USA 67','QUASAR 8',63339,'In Earth orbit','','1990-12-23 00:00:00',1436.27,35780,35800,0,'Stationary' from dual union all
select 20964,'020964','1990-097C','USA 67 PKM','TU-882',63339,'Reentered','2000-01-01 00:00:00','1997-11-20 00:00:00',121.14,179,3285,25.93,'Medium' from dual union all
select 20965,'020965','1990-097D','PAM-D','Star 48',63339,'Reentered','2000-01-01 00:00:00','1990-11-20 00:00:00',634.25,350,35800,26,'GEO Transfer' from dual union all
select 20966,'020966','1990-104A','Kosmos-2106','Kol''tso',63346,'Reentered','2000-04-07 00:00:00','1991-03-28 00:00:00',95,509,529,82.51,'Intermediate' from dual union all
select 20967,'020967','1990-104B','S5M','S5M',63346,'Reentered','2007-01-08 00:00:00','1990-12-31 00:00:00',95.15,517,534,82.51,'Intermediate' from dual union all
select 20968,'020968','1990-081CA','deb CZ-4 Stage 3','',63323,'Reentered','2000-04-09 00:00:00','1993-08-07 00:00:00',99.86,715,788,98.85,'Sun-Synch' from dual union all
select 20969,'020969','1990-081CB','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-02 00:00:00',102.04,811,900,98.84,'Sun-Synch' from dual union all
select 2097,'002097','1963-038G','deb Transit VBN-1','',9269,'In Earth orbit','','1967-02-26 00:00:00',107.28,1073,1128,89.9,'Polar' from dual union all
select 20970,'020970','1990-081CC','deb CZ-4 Stage 3','',63323,'Reentered','2003-10-24 00:00:00','1993-08-08 00:00:00',101.55,820,845,98.89,'Sun-Synch' from dual union all
select 20971,'020971','1990-081CD','deb CZ-4 Stage 3','',63323,'Reentered','2016-06-13 00:00:00','1993-08-03 00:00:00',101.7,826,853,98.86,'Sun-Synch' from dual union all
select 20972,'020972','1990-081CE','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-07 00:00:00',102.05,840,871,98.84,'Sun-Synch' from dual union all
select 20973,'020973','1990-081CF','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-08-07 00:00:00',101.39,774,874,98.83,'Sun-Synch' from dual union all
select 20974,'020974','1990-081CG','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1993-07-30 00:00:00',102.13,844,875,98.84,'Sun-Synch' from dual union all
select 20975,'020975','1990-104C','Kosmos-2106 SS 1','ESO',63346,'Reentered','1991-02-21 00:00:00','1990-12-29 00:00:00',94.23,475,488,82.53,'Intermediate' from dual union all
select 20976,'020976','1990-104D','Kosmos-2106 SS 2','ESO',63346,'Reentered','1991-02-23 00:00:00','1990-12-30 00:00:00',94.27,478,489,82.48,'Intermediate' from dual union all
select 20977,'020977','1990-104E','Kosmos-2106 SS 3','ESO',63346,'Reentered','1991-03-06 00:00:00','1990-12-30 00:00:00',94.66,494,511,82.47,'Intermediate' from dual union all
select 20978,'020978','1990-105A','DMSP F-10','DMSP Block 5D-2 S-10',63347,'In Earth orbit','','1990-12-31 00:00:00',100.65,732,846,98.86,'Sun-Synch' from dual;
insert into SATELLITE
select 20979,'020979','1990-105B','DMSP F-10 solar array band','',63347,'Reentered','1998-10-28 00:00:00','1993-08-16 00:00:00',97.68,614,682,98.78,'Sun-Synch' from dual union all
select 2098,'002098','1965-084E','deb Kosmos-93','',11834,'Reentered','1968-07-06 00:00:00','1965-11-30 00:00:00',93.92,412,521,48.45,'Intermediate' from dual union all
select 20980,'020980','1990-106A','Columbia (STS-35)','OV-102',63348,'Landed','1990-12-11 00:00:00','1990-12-04 00:00:00',91.57,346,358,28.47,'Intermediate' from dual union all
select 20981,'020981','1990-107A','Soyuz TM-11','Soyuz 7K-STM No. 61',63349,'Landed','1991-05-26 00:00:00','1991-01-01 00:00:00',92.21,366,399,51.61,'Intermediate' from dual union all
select 20982,'020982','1990-107B','11S510','11S510',63349,'Reentered','1990-12-04 00:00:00','1990-12-03 00:00:00',87.64,154,162,51.63,'Intermediate' from dual union all
select 20983,'020983','1990-105C','deb Star 37S','',63347,'Reentered','1991-02-06 00:00:00','1990-12-26 00:00:00',98.31,546,810,98.86,'Sun-Synch' from dual union all
select 20984,'020984','1990-105D','deb Star 37S','',63347,'Reentered','1992-08-04 00:00:00','1992-08-04 00:00:00',87.61,145,167,98.84,'Retrograde' from dual union all
select 20985,'020985','1990-108A','Kosmos-2107','US-PM',63350,'Reentered','1992-04-05 00:00:00','1991-03-03 00:00:00',92.76,403,417,65.01,'Intermediate' from dual union all
select 20986,'020986','1990-108B','11S692','11S692',63350,'Reentered','1990-12-04 00:00:00','1990-12-04 00:00:00',87.45,102,195,64.98,'Intermediate' from dual union all
select 20987,'020987','1990-105E','deb Star 37S','',63347,'Reentered','1991-02-23 00:00:00','1990-12-26 00:00:00',98.32,562,795,98.83,'Sun-Synch' from dual union all
select 20988,'020988','1990-105F','deb Star 37S','',63347,'Reentered','1991-06-02 00:00:00','1991-01-01 00:00:00',98.8,571,832,98.84,'Sun-Synch' from dual union all
select 20989,'020989','1990-105G','deb Star 37S','',63347,'Reentered','1991-02-16 00:00:00','1990-12-26 00:00:00',98.35,576,784,98.84,'Sun-Synch' from dual union all
select 2099,'002099','1966-018A','OPS 3488','CORONA J-29',13425,'Reentered Attached','1966-03-29 00:00:00','1966-03-09 00:00:00',90.74,181,441,75.04,'Intermediate' from dual union all
select 20990,'020990','1990-105H','deb Star 37S','',63347,'Reentered','1991-01-10 00:00:00','1990-12-09 00:00:00',98.28,539,814,98.92,'Sun-Synch' from dual union all
select 20991,'020991','1990-105J','deb Star 37S','',63347,'Reentered','1991-09-20 00:00:00','1991-01-01 00:00:00',97.68,465,831,98.92,'Sun-Synch' from dual union all
select 20992,'020992','1990-105K','deb Star 37S','',63347,'Reentered','1991-03-02 00:00:00','1990-12-30 00:00:00',98.31,538,818,98.88,'Sun-Synch' from dual union all
select 20993,'020993','1990-105L','deb Star 37S','',63347,'Reentered','1991-09-02 00:00:00','1991-09-02 00:00:00',90.29,266,312,98.84,'Sun-Synch' from dual union all
select 20994,'020994','1990-098G','KDU','KDU',63340,'Reentered','1990-12-10 00:00:00','1990-12-06 00:00:00',88.63,188,226,62.81,'Intermediate' from dual union all
select 20995,'020995','1990-098H','KDU part','KDU part',63340,'Reentered','1990-12-05 00:00:00','1990-12-04 00:00:00',88.75,191,234,62.81,'Intermediate' from dual union all
select 20996,'020996','1990-098J','Blenda','Sunshade',63340,'Reentered','1990-12-14 00:00:00','1990-12-04 00:00:00',88.36,186,201,62.79,'Intermediate' from dual union all
select 20997,'020997','1990-105P','deb Star 37S','',63347,'Reentered','1991-01-05 00:00:00','1990-12-09 00:00:00',98.06,531,801,98.81,'Sun-Synch' from dual union all
select 20998,'020998','1990-105M','deb Star 37S','',63347,'Reentered','1991-01-30 00:00:00','1990-12-27 00:00:00',97.69,523,774,98.88,'Sun-Synch' from dual union all
select 20999,'020999','1990-105N','deb Star 37S','',63347,'Reentered','1991-02-02 00:00:00','1990-12-31 00:00:00',97.73,534,767,98.8,'Sun-Synch' from dual union all
select 21,'000021','59 Theta 1','AMS Luna-3','E-2A',5536,'Deep Space','1960-04-29 00:00:00','1959-10-05 00:00:00',21563.33,499,500000,55,'Cislunar' from dual union all
select 210,'000210','61 Omicron 53','deb r/b Ablestar','',7273,'In Earth orbit','','1961-12-30 00:00:00',105.5,972,1064,66.49,'Intermediate' from dual union all
select 2100,'002100','1965-082JY','deb Transtage 4','',11832,'Reentered','1968-12-13 00:00:00','1966-11-05 00:00:00',96.74,491,715,32.44,'Intermediate' from dual union all
select 21000,'021000','1990-109A','Kosmos-2108','Kobal''t No. 529',63351,'Landed','1991-01-28 00:00:00','1990-12-11 00:00:00',89.52,181,321,62.8,'Intermediate' from dual union all
select 21001,'021001','1990-109B','11S510','11S510',63351,'Reentered','1990-12-09 00:00:00','1990-12-09 00:00:00',87.44,134,162,62.76,'Intermediate' from dual union all
select 21002,'021002','1990-098K','KDU part','KDU part',63340,'Reentered','1990-12-06 00:00:00','1990-12-05 00:00:00',88.9,190,251,62.75,'Intermediate' from dual union all
select 21003,'021003','1990-087C','deb Kosmos-2101','',63329,'Reentered','1990-12-08 00:00:00','1990-12-07 00:00:00',90.43,189,403,64.73,'Intermediate' from dual union all
select 21004,'021004','1990-087D','deb Kosmos-2101','',63329,'Reentered','1990-12-06 00:00:00','1990-12-05 00:00:00',91.16,178,485,64.84,'Intermediate' from dual union all
select 21005,'021005','1990-087E','deb Kosmos-2101','',63329,'Reentered','1990-12-10 00:00:00','1990-12-09 00:00:00',90.95,177,466,64.52,'Intermediate' from dual union all
select 21006,'021006','1990-110A','Kosmos-2109','Uragan No. 47L',63352,'In Earth orbit','','1991-01-09 00:00:00',675.71,18969,19290,64.83,'Medium' from dual union all
select 21007,'021007','1990-110B','Kosmos-2110','Uragan No. 48L',63352,'In Earth orbit','','1991-01-07 00:00:00',675.71,19027,19232,64.86,'Medium' from dual union all
select 21008,'021008','1990-110C','Kosmos-2111','Uragan No. 49L',63352,'In Earth orbit','','1991-01-08 00:00:00',675.73,19109,19151,64.84,'Medium' from dual union all
select 21009,'021009','1990-110D','8S812','8S812',63352,'Reentered','1990-12-08 00:00:00','1990-12-08 00:00:00',87,121,130,64.8,'Intermediate' from dual union all
select 2101,'002101','1965-082JZ','deb Transtage 4','',11832,'Reentered','1980-12-19 00:00:00','1980-12-17 00:00:00',91.27,318,356,32.74,'Intermediate' from dual union all
select 21010,'021010','1990-110E','Perekhodnik','Sredniy perekhodnik',63352,'Reentered','1990-12-09 00:00:00','1990-12-09 00:00:00',88.76,138,289,64.94,'Intermediate' from dual union all
select 21011,'021011','1990-110F','Blok DM2','11S861 No. 47L',63352,'In Earth orbit','','1991-01-08 00:00:00',675.22,19097,19137,64.83,'Medium' from dual union all
select 21012,'021012','1990-110G','SOZ','SOZ',63352,'In Earth orbit','','1991-01-08 00:00:00',340.41,417,19126,64.91,'Highly Ellip' from dual union all
select 21013,'021013','1990-110H','SOZ','SOZ',63352,'In Earth orbit','','1991-01-08 00:00:00',340.42,418,19126,64.78,'Highly Ellip' from dual union all
select 21014,'021014','1990-111A','Kosmos-2112','Strela-2M',63353,'In Earth orbit','','1991-10-27 00:00:00',100.65,769,810,74.04,'Intermediate' from dual union all
select 21015,'021015','1990-111B','S3M','S3M',63353,'In Earth orbit','','1991-03-28 00:00:00',100.56,767,804,74.04,'Intermediate' from dual union all
select 21016,'021016','1990-112A','Raduga','Gran'' No. 37L',63354,'In Earth orbit','','1991-01-20 00:00:00',1436.09,35695,35878,1.39,'Stationary' from dual union all
select 21017,'021017','1990-112B','8S812','8S812',63354,'Reentered','1990-12-22 00:00:00','1990-12-20 00:00:00',88.13,174,189,51.66,'Intermediate' from dual union all
select 21018,'021018','1990-112C','Perekhodnik','Sredniy perekhodnik',63354,'Reentered','1990-12-21 00:00:00','1990-12-20 00:00:00',88.33,186,198,51.65,'Intermediate' from dual union all
select 21019,'021019','1990-112D','Blok DM2','11S861 No. 48L',63354,'In Earth orbit','','1991-01-22 00:00:00',1439.79,35741,35977,1.42,'Drift GEO' from dual union all
select 2102,'002102','1965-082KA','deb Transtage 4','',11832,'Reentered','1969-04-15 00:00:00','1966-11-07 00:00:00',98.53,639,737,32.53,'Intermediate' from dual union all
select 21020,'021020','1989-100V','Kosmos-2053 SS 18','ESO',62377,'Reentered','1991-02-06 00:00:00','1990-12-21 00:00:00',94.63,498,504,73.53,'Intermediate' from dual union all
select 21021,'021021','1989-100W','radar error','',62377,'Spurious','1990-12-21 00:00:00','1990-12-20 00:00:00',94.64,496,507,73.54,'Intermediate' from dual union all
select 21022,'021022','1989-100X','Kosmos-2053 SS 19','ESO',62377,'Reentered','1991-02-07 00:00:00','1990-12-21 00:00:00',94.66,494,511,73.53,'Intermediate' from dual union all
select 21023,'021023','1989-100Y','Kosmos-2053 SS 20','ESO',62377,'Reentered','1991-02-08 00:00:00','1990-12-20 00:00:00',94.62,491,510,73.56,'Intermediate' from dual union all
select 21024,'021024','1990-112E','SOZ','SOZ',63354,'Reentered','1991-07-31 00:00:00','1991-01-22 00:00:00',634.6,273,35895,47.17,'GEO Transfer' from dual union all
select 21025,'021025','1990-112F','SOZ','SOZ',63354,'Reentered','2000-07-02 00:00:00','1991-09-13 00:00:00',583.27,158,33335,46.53,'Highly Ellip' from dual union all
select 21026,'021026','1990-113A','Kosmos-2113','Neman',63355,'Deorbited','1991-06-11 00:00:00','1991-01-20 00:00:00',89.06,202,253,64.76,'Intermediate' from dual union all
select 21027,'021027','1990-113B','11S510','11S510',63355,'Reentered','1990-12-22 00:00:00','1990-12-21 00:00:00',89.06,180,276,64.76,'Intermediate' from dual union all
select 21028,'021028','1990-114A','Kosmos-2114','Strela-3',63356,'In Earth orbit','','1991-01-23 00:00:00',114.04,1409,1415,82.57,'Intermediate' from dual union all
select 21029,'021029','1990-114B','Kosmos-2115','Strela-3',63356,'In Earth orbit','','1991-01-24 00:00:00',113.97,1407,1411,82.57,'Intermediate' from dual union all
select 2103,'002103','1965-082KB','deb Transtage 4','',11832,'Reentered','1967-11-27 00:00:00','1966-02-28 00:00:00',98.06,588,744,32.6,'Intermediate' from dual union all
select 21030,'021030','1990-114C','Kosmos-2116','Strela-3',63356,'In Earth orbit','','1991-01-23 00:00:00',113.91,1402,1409,82.57,'Intermediate' from dual union all
select 21031,'021031','1990-114D','Kosmos-2117','Strela-3',63356,'In Earth orbit','','1991-01-23 00:00:00',113.84,1396,1410,82.58,'Intermediate' from dual union all
select 21032,'021032','1990-114E','Kosmos-2118','Strela-3',63356,'In Earth orbit','','1991-01-23 00:00:00',113.8,1392,1409,82.56,'Intermediate' from dual union all
select 21033,'021033','1990-114F','Kosmos-2119','Strela-3',63356,'In Earth orbit','','1991-01-23 00:00:00',113.72,1385,1410,82.57,'Intermediate' from dual union all
select 21034,'021034','1990-114G','S5M','S5M',63356,'In Earth orbit','','1991-02-01 00:00:00',114.68,1409,1473,82.56,'Intermediate' from dual union all
select 21035,'021035','1990-115A','Kosmos-2120','Oblik No. 816',63357,'Landed','1991-01-17 00:00:00','1991-01-02 00:00:00',90.25,234,340,82.58,'Intermediate' from dual union all
select 21036,'021036','1990-115B','11S510','11S510',63357,'Reentered','1990-12-28 00:00:00','1990-12-26 00:00:00',88.38,175,214,82.57,'Intermediate' from dual union all
select 21037,'021037','1990-115C','Sensor cover','Sensor cover',63357,'Reentered','1990-12-28 00:00:00','1990-12-26 00:00:00',88.42,168,225,82.58,'Intermediate' from dual union all
select 21038,'021038','1990-116A','Raduga-1','Globus No. 12',63358,'In Earth orbit','','1991-02-06 00:00:00',1436.09,35769,35804,1.38,'Stationary' from dual union all
select 21039,'021039','1990-116B','8S812','8S812',63358,'Reentered','1990-12-29 00:00:00','1990-12-27 00:00:00',88.29,187,193,51.63,'Intermediate' from dual union all
select 2104,'002104','1966-019A','TDA 3','TDA 3',13426,'Reentered Attached','1967-09-15 00:00:00','1966-11-04 00:00:00',92.29,382,392,28.87,'Intermediate' from dual union all
select 21040,'021040','1990-116C','Perekhodnik','Sredniy perekhodnik',63358,'Reentered','1990-12-28 00:00:00','1990-12-27 00:00:00',88.37,184,204,51.62,'Intermediate' from dual union all
select 21041,'021041','1990-116D','Blok DM2','11S861 No. 46L',63358,'In Earth orbit','','1991-02-15 00:00:00',1470.14,36308,36593,1.38,'Drift GEO' from dual union all
select 21042,'021042','1989-100Z','Kosmos-2053 SS 21','ESO',62377,'Reentered','1991-02-12 00:00:00','1990-12-28 00:00:00',94.63,492,510,73.53,'Intermediate' from dual union all
select 21043,'021043','1989-100AA','Kosmos-2053 SS 22','ESO',62377,'Reentered','1991-02-12 00:00:00','1990-12-28 00:00:00',94.67,496,510,73.53,'Intermediate' from dual union all
select 21044,'021044','1990-116E','SOZ','SOZ',63358,'Reentered','1991-09-26 00:00:00','1991-01-13 00:00:00',646.5,283,36495,47.3,'GEO Transfer' from dual union all
select 21045,'021045','1990-116F','SOZ','SOZ',63358,'Reentered','1995-05-06 00:00:00','1992-02-20 00:00:00',356.75,157,20412,46.51,'Highly Ellip' from dual union all
select 21046,'021046','1990-102D','Blok DM2 No. 45L','11S861 No. 45L',63344,'In Earth orbit','','1990-12-29 00:00:00',1471.3,36381,36565,1.39,'Drift GEO' from dual union all
select 21047,'021047','1991-001A','NATO 4A','NATO 4A',64170,'In Earth orbit','','1991-02-09 00:00:00',1410.82,34919,35662,4.25,'Inclined Drift' from dual union all
select 21048,'021048','1991-001B','Delta 202','Delta SSPS AJ10-118K',64170,'In Earth orbit','','1991-02-27 00:00:00',121.55,783,2718,18.46,'Medium' from dual union all
select 21049,'021049','1991-001C','Star 48B','Star 48B S/N 10064-18',64170,'In Earth orbit','','1992-02-20 00:00:00',635.76,782,35446,25.22,'GEO Transfer' from dual union all
select 2105,'002105','1966-020A','Gemini VIII','Gemini SC8',13427,'Landed','1966-03-17 00:00:00','1966-03-16 00:00:00',89.81,261,270,28.9,'Intermediate' from dual union all
select 21050,'021050','1986-017CT','EO-8 EVA-1 debris','',59210,'Reentered','1991-02-04 00:00:00','1991-01-10 00:00:00',91.95,353,388,51.62,'Intermediate' from dual union all
select 21051,'021051','1986-017CU','EO-8 EVA-1 debris','',59210,'Reentered','1991-02-05 00:00:00','1991-01-13 00:00:00',91.95,353,388,51.61,'Intermediate' from dual union all
select 21052,'021052','1969-069C','JPL SR-28-3','JPL SR-28-3',18336,'In Earth orbit','','1983-01-13 00:00:00',1466.69,35802,36965,7.73,'Inclined Drift' from dual union all
select 21053,'021053','1991-002A','Progress M-6','Progress 7K-TGM No. 205',64171,'Deorbited','1991-03-15 00:00:00','1991-02-14 00:00:00',92,358,388,51.61,'Intermediate' from dual union all
select 21054,'021054','1991-002B','11S510','11S510',64171,'Reentered','1991-01-15 00:00:00','1991-01-15 00:00:00',87.4,142,150,51.62,'Intermediate' from dual union all
select 21055,'021055','1991-003A','Italsat 1','Italsat 1',64172,'In Earth orbit','','1991-02-14 00:00:00',1436.11,35728,35845,.1,'Stationary' from dual union all
select 21056,'021056','1991-003B','Eutelsat II F-2','Eutelsat II F-2',64172,'In Earth orbit','','1991-02-19 00:00:00',1435.98,35773,35796,.06,'Stationary' from dual union all
select 21057,'021057','1991-003C','Ariane H10 V41','Ariane H10 S/N T412',64172,'Reentered','1998-05-08 00:00:00','1992-02-20 00:00:00',599.21,254,34078,6.68,'Highly Ellip' from dual union all
select 21058,'021058','1991-003D','Spelda 10 S/N UV10','Spelda 10',64172,'Reentered','1995-03-09 00:00:00','1992-02-17 00:00:00',509.62,238,29275,6.89,'Highly Ellip' from dual union all
select 21059,'021059','1991-004A','Kosmos-2121','Oblik No. 817',64173,'Landed','1991-02-10 00:00:00','1991-01-24 00:00:00',89.79,231,298,82.59,'Intermediate' from dual union all
select 2106,'002106','1966-020B','Titan GT-8 Stage 2','LR91AJ5 GT-8 65-12563',13427,'Reentered','1966-03-17 00:00:00','1966-03-16 00:00:00',88.94,160,285,29,'Intermediate' from dual union all
select 21060,'021060','1991-004B','11S510','11S510',64173,'Reentered','1991-01-19 00:00:00','1991-01-19 00:00:00',87.26,126,152,82.57,'Intermediate' from dual union all
select 21061,'021061','1990-115D','KDU part','KDU part',63357,'Reentered','1991-01-18 00:00:00','1991-01-17 00:00:00',88.5,194,207,82.55,'Intermediate' from dual union all
select 21062,'021062','1990-115E','KDU part','KDU part',63357,'Reentered','1991-01-19 00:00:00','1991-01-17 00:00:00',88.76,205,222,82.55,'Intermediate' from dual union all
select 21063,'021063','1990-115F','KDU','KDU',63357,'Reentered','1991-01-19 00:00:00','1991-01-24 00:00:00',88.12,177,186,82.56,'Intermediate' from dual union all
select 21064,'021064','1989-100AB','Kosmos-2053 SS 23','ESO',62377,'Reentered','1991-02-12 00:00:00','1991-01-28 00:00:00',93.4,429,453,73.52,'Intermediate' from dual union all
select 21065,'021065','1991-005A','Kosmos-2122','US-PU',64174,'Deorbited','1993-03-28 00:00:00','1991-02-18 00:00:00',92.76,403,417,65.01,'Intermediate' from dual union all
select 21066,'021066','1991-005B','11S692','11S692',64174,'Reentered','1991-01-19 00:00:00','1991-01-18 00:00:00',88.03,104,249,64.99,'Intermediate' from dual union all
select 21067,'021067','1991-005C','deb Kosmos-2122','',64174,'Reentered','1991-05-30 00:00:00','1991-02-18 00:00:00',92.55,392,407,65.01,'Intermediate' from dual;
insert into SATELLITE
select 21068,'021068','1990-115G','Blenda','Sunshade',63357,'Reentered','1991-01-21 00:00:00','1991-01-19 00:00:00',90.05,223,331,82.56,'Intermediate' from dual union all
select 21069,'021069','1990-104G','Kosmos-2106 SS 5','ESO',63346,'Reentered','1991-03-10 00:00:00','1991-01-23 00:00:00',95.05,506,536,82.51,'Intermediate' from dual union all
select 2107,'002107','1966-021A','Kosmos-112','Zenit-2 No. 37',13428,'Landed','1966-03-25 00:00:00','1966-03-17 00:00:00',92.27,207,565,72.1,'Intermediate' from dual union all
select 21070,'021070','1990-104F','Kosmos-2106 SS 4','ESO',63346,'Reentered','1991-03-10 00:00:00','1991-01-24 00:00:00',94.99,504,533,82.5,'Intermediate' from dual union all
select 21071,'021071','1990-105Q','deb Star 37S','',63347,'Reentered','1991-04-09 00:00:00','1991-01-27 00:00:00',98.45,567,803,98.85,'Sun-Synch' from dual union all
select 21072,'021072','1990-105R','deb Star 37S','',63347,'Reentered','1991-03-18 00:00:00','1991-01-28 00:00:00',97.8,542,765,98.82,'Sun-Synch' from dual union all
select 21073,'021073','1990-105S','DMSP F-10 solar array band','',63347,'Reentered','1999-07-02 00:00:00','1991-01-27 00:00:00',100.36,708,844,98.85,'Sun-Synch' from dual union all
select 21074,'021074','1990-105T','deb Star 37S','',63347,'Reentered','1991-03-09 00:00:00','1991-01-27 00:00:00',97.85,540,771,98.73,'Sun-Synch' from dual union all
select 21075,'021075','1990-105U','deb Star 37S','',63347,'Reentered','1991-02-09 00:00:00','1991-01-26 00:00:00',95.38,423,651,98.78,'Sun-Synch' from dual union all
select 21076,'021076','1990-105V','deb Star 37S','',63347,'Reentered','1991-02-21 00:00:00','1991-01-27 00:00:00',97.13,525,718,98.72,'Sun-Synch' from dual union all
select 21077,'021077','1990-105W','deb Star 37S','',63347,'Reentered','1991-05-19 00:00:00','1991-01-27 00:00:00',98.39,557,807,98.84,'Sun-Synch' from dual union all
select 21078,'021078','1990-105X','deb Star 37S','',63347,'Reentered','1991-04-12 00:00:00','1991-01-25 00:00:00',98.91,582,832,98.86,'Sun-Synch' from dual union all
select 21079,'021079','1990-105Y','deb Star 37S','',63347,'Reentered','1991-04-12 00:00:00','1991-01-25 00:00:00',98.26,565,786,98.86,'Sun-Synch' from dual union all
select 2108,'002108','1966-021B','8A92E','8A92E',13428,'Reentered','1966-05-17 00:00:00','1966-03-17 00:00:00',92.27,207,565,72.1,'Intermediate' from dual union all
select 21080,'021080','1990-105Z','deb DMSP F-10 OLS Cover','',63347,'Reentered','2001-10-01 00:00:00','1991-01-27 00:00:00',100.61,729,846,98.86,'Sun-Synch' from dual union all
select 21081,'021081','1990-104H','Kosmos-2106 SS 6','ESO',63346,'Reentered','1991-03-10 00:00:00','1991-01-24 00:00:00',95,509,529,82.51,'Intermediate' from dual union all
select 21082,'021082','1986-017CV','EO-8 EVA-2 debris (Strela)','',59210,'Reentered','1991-03-18 00:00:00','1991-01-30 00:00:00',91.98,356,388,51.61,'Intermediate' from dual union all
select 21083,'021083','1990-081CJ','deb CZ-4 Stage 3','',63323,'Reentered','1999-01-06 00:00:00','1995-01-13 00:00:00',96.58,546,644,98.89,'Sun-Synch' from dual union all
select 21084,'021084','1986-017CW','EO-8 EVA-2 debris (Strela)','',59210,'Reentered','1991-03-05 00:00:00','1991-01-30 00:00:00',91.95,354,387,51.61,'Intermediate' from dual union all
select 21085,'021085','1986-017CX','EO-8 EVA-2 debris (Strela)','',59210,'Reentered','1991-03-01 00:00:00','1991-01-30 00:00:00',91.95,353,388,51.6,'Intermediate' from dual union all
select 21086,'021086','1986-017CY','EO-8 EVA-2 debris (Strela)','',59210,'Reentered','1991-02-24 00:00:00','1991-01-30 00:00:00',91.94,353,387,51.61,'Intermediate' from dual union all
select 21087,'021087','1991-006A','Informator-1','Informator No. 1',64175,'In Earth orbit','','1991-03-01 00:00:00',104.76,958,1008,82.95,'Intermediate' from dual union all
select 21088,'021088','1991-006B','S3M','S3M',64175,'In Earth orbit','','1991-03-01 00:00:00',104.64,958,997,82.95,'Intermediate' from dual union all
select 21089,'021089','1991-007A','Kosmos-2123','Tsikada/RS-12/RS-13',64176,'In Earth orbit','','1991-03-08 00:00:00',104.79,964,1006,82.92,'Intermediate' from dual union all
select 2109,'002109','1966-022A','OPS 0879','GAMBIT SV 976',13429,'Reentered','1966-03-24 00:00:00','1966-03-19 00:00:00',88.97,147,301,101,'Retrograde' from dual union all
select 21090,'021090','1991-007B','S3M','S3M',64176,'In Earth orbit','','1991-03-08 00:00:00',104.69,964,996,82.93,'Intermediate' from dual union all
select 21091,'021091','1991-007C','deb Kosmos-2123','',64176,'In Earth orbit','','1991-10-25 00:00:00',104.64,957,999,82.93,'Intermediate' from dual union all
select 21092,'021092','1991-008A','Kosmos-2124','Kobal''t No. 530',64177,'Landed','1991-04-07 00:00:00','1991-02-14 00:00:00',88.87,174,264,62.84,'Intermediate' from dual union all
select 21093,'021093','1991-008B','11S510','11S510',64177,'Reentered','1991-02-11 00:00:00','1991-02-08 00:00:00',89.42,183,309,62.83,'Intermediate' from dual union all
select 21094,'021094','1991-008C','deb Kosmos-2124','',64177,'Reentered','1991-02-08 00:00:00','1991-02-08 00:00:00',88.46,160,237,62.8,'Intermediate' from dual union all
select 21095,'021095','1991-004C','KDU','KDU',64173,'Reentered','1991-02-21 00:00:00','1991-02-14 00:00:00',89.16,217,249,82.59,'Intermediate' from dual union all
select 21096,'021096','1991-004D','Blenda','Sunshade',64173,'Reentered','1991-02-12 00:00:00','1991-02-11 00:00:00',88.6,198,213,82.59,'Intermediate' from dual union all
select 21097,'021097','1991-004E','KDU part','KDU part',64173,'Reentered','1991-02-12 00:00:00','1991-02-10 00:00:00',89.31,214,266,82.59,'Intermediate' from dual union all
select 21098,'021098','1991-004F','PTDU cover','KDU part',64173,'Reentered','1991-02-14 00:00:00','1991-02-12 00:00:00',90.58,228,378,82.58,'Intermediate' from dual union all
select 21099,'021099','1991-004G','PTDU cover','KDU part',64173,'Reentered','1991-02-11 00:00:00','1991-02-11 00:00:00',90.75,230,393,82.59,'Intermediate' from dual union all
select 211,'000211','61 Omicron 54','deb r/b Ablestar','',7273,'Reentered','2002-10-17 00:00:00','1961-11-25 00:00:00',102.64,713,1054,66.57,'Intermediate' from dual union all
select 2110,'002110','1966-021C','deb Kosmos-112','',13428,'Reentered','1966-03-25 00:00:00','1966-03-18 00:00:00',91.11,203,455,72,'Intermediate' from dual union all
select 21100,'021100','1991-009A','Kosmos-2125','Strela-1M',64178,'In Earth orbit','','1991-03-14 00:00:00',115.21,1457,1473,74.03,'Intermediate' from dual union all
select 21101,'021101','1991-009B','Kosmos-2126','Strela-1M',64178,'In Earth orbit','','1991-03-14 00:00:00',115.57,1466,1496,74.03,'Intermediate' from dual union all
select 21102,'021102','1991-009C','Kosmos-2127','Strela-1M',64178,'In Earth orbit','','1991-03-14 00:00:00',115.37,1467,1477,74.03,'Intermediate' from dual union all
select 21103,'021103','1991-009D','Kosmos-2128','Strela-1M',64178,'In Earth orbit','','1991-03-14 00:00:00',115.03,1446,1467,74.03,'Intermediate' from dual union all
select 21104,'021104','1991-009E','Kosmos-2129','Strela-1M',64178,'In Earth orbit','','1991-03-14 00:00:00',114.85,1430,1467,74.03,'Intermediate' from dual union all
select 21105,'021105','1991-009F','Kosmos-2130','Strela-1M',64178,'In Earth orbit','','1991-03-14 00:00:00',114.55,1401,1469,74.03,'Intermediate' from dual union all
select 21106,'021106','1991-009G','Kosmos-2131','Strela-1M',64178,'In Earth orbit','','1991-03-14 00:00:00',114.39,1388,1468,74.03,'Intermediate' from dual union all
select 21107,'021107','1991-009H','Kosmos-2132','Strela-1M',64178,'In Earth orbit','','1991-03-14 00:00:00',114.7,1416,1467,74.03,'Intermediate' from dual union all
select 21108,'021108','1991-009J','S3M','S3M',64178,'In Earth orbit','','1991-03-14 00:00:00',118.08,1489,1701,74.07,'Intermediate' from dual union all
select 21109,'021109','1991-009K','deb Kosmos R-14','',64178,'In Earth orbit','','1991-03-14 00:00:00',118.26,1462,1744,74.03,'Intermediate' from dual union all
select 2111,'002111','1966-021D','deb Kosmos-112','',13428,'Reentered','1966-03-25 00:00:00','1966-03-18 00:00:00',91.1,190,467,72.1,'Intermediate' from dual union all
select 21110,'021110','1991-009L','deb Kosmos R-14','',64178,'In Earth orbit','','1991-03-14 00:00:00',119.45,1465,1848,74.05,'Intermediate' from dual union all
select 21111,'021111','1991-010A','Kosmos-2133','Oko-1 No. 7120',64179,'In Earth orbit','','1991-03-17 00:00:00',1436.06,35761,35811,2.25,'Stationary' from dual union all
select 21112,'021112','1991-010B','8S812','8S812',64179,'Reentered','1991-02-16 00:00:00','1991-02-16 00:00:00',87.64,156,160,51.64,'Intermediate' from dual union all
select 21113,'021113','1991-010C','Perekhodnik','Sredniy perekhodnik',64179,'Reentered','1991-02-14 00:00:00','1991-02-14 00:00:00',88.25,187,189,51.62,'Intermediate' from dual union all
select 21114,'021114','1991-010D','SOZ','SOZ',64179,'Reentered','2007-01-30 00:00:00','1992-02-20 00:00:00',428.33,162,24727,46.51,'Highly Ellip' from dual union all
select 21115,'021115','1991-009M','deb Kosmos R-14','',64178,'In Earth orbit','','1991-03-18 00:00:00',118.47,1464,1760,74.04,'Intermediate' from dual union all
select 21116,'021116','1991-011A','Kosmos-2134','Kometa No. 13',64180,'Landed','1991-04-01 00:00:00','1991-02-22 00:00:00',89.21,211,260,64.73,'Intermediate' from dual union all
select 21117,'021117','1991-011B','11S510','11S510',64180,'Reentered','1991-02-19 00:00:00','1991-02-19 00:00:00',87.58,139,171,64.74,'Intermediate' from dual union all
select 21118,'021118','1991-012A','Molniya-1','Molniya-1T No. 73',64181,'In Earth orbit','','1991-03-18 00:00:00',717.88,438,39922,62.81,'Molniya' from dual union all
select 21119,'021119','1991-012B','11S510','11S510',64181,'Reentered','1991-02-26 00:00:00','1991-02-19 00:00:00',90.18,197,369,62.82,'Intermediate' from dual union all
select 2112,'002112','1966-022B','Agena D 4823','Agena D',13429,'Reentered','1966-03-23 00:00:00','1966-03-19 00:00:00',88.22,137,236,100.9,'Retrograde' from dual union all
select 21120,'021120','1991-012C','BOZ','BOZ',64181,'Reentered','1991-02-22 00:00:00','1991-02-21 00:00:00',88.99,155,295,62.8,'Intermediate' from dual union all
select 21121,'021121','1991-012D','Blok-ML','Blok-ML No. 185',64181,'In Earth orbit','','1991-03-17 00:00:00',700.66,432,39075,62.73,'Molniya' from dual union all
select 21122,'021122','1991-010E','SOZ','SOZ',64179,'Reentered','2002-08-18 00:00:00','1991-03-16 00:00:00',636.06,336,35907,47.35,'GEO Transfer' from dual union all
select 21123,'021123','1991-011C','Kosmos-2134 part','Kometa part',64180,'Reentered','1991-02-16 00:00:00','1991-02-16 00:00:00',88.38,157,232,64.74,'Intermediate' from dual union all
select 21124,'021124','1990-105AA','deb Star 37S','',63347,'Reentered','1991-03-11 00:00:00','1991-01-03 00:00:00',98.44,564,804,98.88,'Sun-Synch' from dual union all
select 21125,'021125','1990-105AB','deb Star 37S','',63347,'Reentered','1991-08-26 00:00:00','1991-08-02 00:00:00',94.43,457,525,98.81,'Sun-Synch' from dual union all
select 21126,'021126','1964-053C','deb Blok-E','',10339,'In Earth orbit','','1993-08-16 00:00:00',98.89,645,767,65.08,'Intermediate' from dual union all
select 21127,'021127','1990-105AC','deb Star 37S','',63347,'Reentered','1991-04-08 00:00:00','1990-12-31 00:00:00',98.76,577,822,98.87,'Sun-Synch' from dual union all
select 21128,'021128','1976-067BY','deb Kosmos-839','',39019,'In Earth orbit','','1993-08-06 00:00:00',116.51,982,2066,66.03,'Intermediate' from dual union all
select 21129,'021129','1991-010F','Blok DM2','11S861 No. 38L',64179,'In Earth orbit','','1991-03-16 00:00:00',1438.57,35758,35912,2.29,'Drift GEO' from dual union all
select 2113,'002113','1966-018B','Fairing','Fairing',13425,'Reentered','1966-03-22 00:00:00','1966-03-19 00:00:00',89.56,185,321,75,'Intermediate' from dual union all
select 21130,'021130','1991-013A','Kosmos-2135','Parus',64182,'In Earth orbit','','1991-10-28 00:00:00',104.51,923,1020,82.83,'Intermediate' from dual union all
select 21131,'021131','1991-013B','S3M','S3M',64182,'In Earth orbit','','1991-03-28 00:00:00',104.4,923,1010,82.82,'Intermediate' from dual union all
select 21132,'021132','1991-014A','Raduga','Gran'' No. 38L',64183,'In Earth orbit','','1991-03-30 00:00:00',1436.34,35783,35800,1.45,'Stationary' from dual union all
select 21133,'021133','1991-014B','8S812','8S812',64183,'Reentered','1991-03-02 00:00:00','1991-03-01 00:00:00',88,172,179,51.66,'Intermediate' from dual union all
select 21134,'021134','1991-014C','Perekhodnik','Sredniy perekhodnik',64183,'Reentered','1991-02-28 00:00:00','1991-02-28 00:00:00',88.37,184,204,51.64,'Intermediate' from dual union all
select 21135,'021135','1991-014D','Blok DM2','11S861 No. 49L',64183,'In Earth orbit','','1991-03-31 00:00:00',1392.15,34824,35019,1.47,'Drift GEO' from dual union all
select 21136,'021136','1991-009N','deb S3M','',64178,'In Earth orbit','','1991-10-27 00:00:00',120.38,1462,1934,73.99,'Medium' from dual union all
select 21137,'021137','1991-009P','deb S3M','',64178,'In Earth orbit','','1991-10-26 00:00:00',118.21,1479,1722,74.04,'Intermediate' from dual union all
select 21138,'021138','1991-009Q','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',117.15,1380,1726,74.08,'Intermediate' from dual union all
select 21139,'021139','1991-015A','Astra 1B','Astra 1B',64184,'In Earth orbit','','1991-04-01 00:00:00',1424.1,35467,35636,.16,'Drift GEO' from dual union all
select 2114,'002114','1966-023A','Kosmos-113','Zenit-4 No. 17',13430,'Landed','1966-03-29 00:00:00','1966-03-21 00:00:00',89.7,207,312,64.9,'Intermediate' from dual union all
select 21140,'021140','1991-015B','Meteosat 5','MOP 2',64184,'In Earth orbit','','1991-04-01 00:00:00',1435.73,35771,35788,1.15,'Stationary' from dual union all
select 21141,'021141','1991-015C','Ariane H10 V42','Ariane H10 S/N T414',64184,'Reentered','2002-02-10 00:00:00','1991-04-04 00:00:00',636.12,246,36000,7.02,'GEO Transfer' from dual union all
select 21142,'021142','1991-015D','Spelda 10 S/N UV11','Spelda 10',64184,'Reentered','1995-11-23 00:00:00','1991-04-02 00:00:00',617.07,225,35038,7.07,'GEO Transfer' from dual union all
select 21143,'021143','1991-016A','Kosmos-2136','Oblik No. 818',64185,'Landed','1991-03-20 00:00:00','1991-03-08 00:00:00',90.16,251,314,62.85,'Intermediate' from dual union all
select 21144,'021144','1991-016B','11S510','11S510',64185,'Reentered','1991-03-17 00:00:00','1991-03-08 00:00:00',90.04,248,304,62.84,'Intermediate' from dual union all
select 21145,'021145','1991-016C','Sensor cover','Sensor cover',64185,'Reentered','1991-03-08 00:00:00','1991-03-08 00:00:00',88.01,162,190,62.81,'Intermediate' from dual union all
select 21146,'021146','1991-016D','Sensor cover','',64185,'Reentered','1991-03-09 00:00:00','1991-03-09 00:00:00',88.2,159,211,62.8,'Intermediate' from dual union all
select 21147,'021147','1991-017A','USA 69','ONYX 2',64186,'Deorbited','2011-03-26 00:00:00','1991-05-11 00:00:00',98.22,672,676,68.03,'Intermediate' from dual union all
select 21148,'021148','1991-017B','Titan 45K-5 Stage 2','LR91AJ11A 45K-5',64186,'Reentered','1999-01-09 00:00:00','1991-04-07 00:00:00',95.53,420,669,67.98,'Intermediate' from dual union all
select 21149,'021149','1991-018A','INMARSAT II F-2','INMARSAT II F-2',64187,'In Earth orbit','','1991-04-18 00:00:00',1436.19,35766,35811,2.64,'Stationary' from dual union all
select 2115,'002115','1966-020C','Gemini SC8 Equipment Module','Gemini SC8 Adapter Modul',13427,'Reentered','1966-04-18 00:00:00','1966-03-17 00:00:00',90.42,292,299,29.02,'Intermediate' from dual union all
select 21150,'021150','1991-018B','Delta 203','Delta SSPS AJ10-118K',64187,'Reentered','2003-05-26 00:00:00','1992-02-20 00:00:00',99.87,405,1100,24.97,'Equatorial' from dual union all
select 21151,'021151','1991-018C','Star 48B','Star 48B S/N 10066-18',64187,'In Earth orbit','','1992-02-18 00:00:00',599.63,185,34169,23.01,'Highly Ellip' from dual union all
select 21152,'021152','1991-019A','Nadezhda','Nadezhda 17F118 No. 409',64188,'In Earth orbit','','1991-04-11 00:00:00',104.84,957,1017,82.92,'Intermediate' from dual union all
select 21153,'021153','1991-019B','S3M','S3M',64188,'In Earth orbit','','1991-04-21 00:00:00',104.73,955,1008,82.92,'Intermediate' from dual union all
select 21154,'021154','1991-009R','deb S3M','',64178,'In Earth orbit','','1991-10-12 00:00:00',119.95,1520,1838,74.28,'Intermediate' from dual union all
select 21155,'021155','1991-009S','deb S3M','',64178,'In Earth orbit','','1993-08-01 00:00:00',117.02,1433,1661,73.92,'Intermediate' from dual union all
select 21156,'021156','1991-009T','deb S3M','',64178,'In Earth orbit','','1991-10-05 00:00:00',116.71,1416,1650,73.91,'Intermediate' from dual union all
select 21157,'021157','1991-009U','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',117.63,1435,1714,73.95,'Intermediate' from dual;
insert into SATELLITE
select 21158,'021158','1991-009V','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',118.63,1505,1734,74.09,'Intermediate' from dual union all
select 21159,'021159','1991-009W','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',117.17,1439,1669,73.94,'Intermediate' from dual union all
select 2116,'002116','1966-023B','11A57I','11A57I',13430,'Reentered','1966-03-30 00:00:00','1966-03-21 00:00:00',89.7,207,312,64.9,'Intermediate' from dual union all
select 21160,'021160','1991-009X','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',117.57,1467,1677,74.04,'Intermediate' from dual union all
select 21161,'021161','1991-009Y','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',118.08,1489,1701,74.07,'Intermediate' from dual union all
select 21162,'021162','1991-009Z','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',117.27,1449,1668,73.97,'Intermediate' from dual union all
select 21163,'021163','1991-009AA','deb S3M','',64178,'In Earth orbit','','1991-10-12 00:00:00',117.08,1438,1662,73.96,'Intermediate' from dual union all
select 21164,'021164','1991-009AB','deb S3M','',64178,'In Earth orbit','','1993-07-29 00:00:00',117.84,1484,1684,74.04,'Intermediate' from dual union all
select 21165,'021165','1991-009AC','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',118.81,1499,1756,74.16,'Intermediate' from dual union all
select 21166,'021166','1991-009AD','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',118.12,1498,1695,74.06,'Intermediate' from dual union all
select 21167,'021167','1991-009AE','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',117.14,1443,1662,73.91,'Intermediate' from dual union all
select 21168,'021168','1991-009AF','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',117.32,1402,1719,73.9,'Intermediate' from dual union all
select 21169,'021169','1991-009AG','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',117.76,1475,1685,73.99,'Intermediate' from dual union all
select 2117,'002117','1966-023C','deb Kosmos-113','',13430,'Reentered','1966-03-22 00:00:00','1966-03-21 00:00:00',89.34,192,292,65.1,'Intermediate' from dual union all
select 21170,'021170','1991-009AH','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',118.72,1502,1745,74.11,'Intermediate' from dual union all
select 21171,'021171','1991-009AJ','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',117.87,1430,1741,73.95,'Intermediate' from dual union all
select 21172,'021172','1991-009AK','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',117.3,1453,1666,74.03,'Intermediate' from dual union all
select 21173,'021173','1991-009AL','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',118.11,1487,1705,74.11,'Intermediate' from dual union all
select 21174,'021174','1991-009AM','deb S3M','',64178,'In Earth orbit','','1991-10-12 00:00:00',120.12,1458,1915,74.1,'Medium' from dual union all
select 21175,'021175','1991-009AN','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',117.21,1438,1673,73.93,'Intermediate' from dual union all
select 21176,'021176','1991-009AP','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',118.23,1436,1767,74.06,'Intermediate' from dual union all
select 21177,'021177','1991-009AQ','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',118.67,1503,1740,74.1,'Intermediate' from dual union all
select 21178,'021178','1991-009AR','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',118.39,1496,1722,74.1,'Intermediate' from dual union all
select 21179,'021179','1991-009AS','deb S3M','',64178,'In Earth orbit','','1991-10-06 00:00:00',118.39,1511,1706,73.98,'Intermediate' from dual union all
select 2118,'002118','1966-008D','ESSA 1 despin weight','',13415,'In Earth orbit','','1966-09-12 00:00:00',101.33,695,949,98.05,'Sun-Synch' from dual union all
select 21180,'021180','1991-009AT','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',117.32,1473,1648,74.07,'Intermediate' from dual union all
select 21181,'021181','1991-009AU','deb S3M','',64178,'In Earth orbit','','1991-10-09 00:00:00',117.89,1418,1754,73.98,'Intermediate' from dual union all
select 21182,'021182','1991-009AV','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',117.99,1467,1715,74.02,'Intermediate' from dual union all
select 21183,'021183','1991-009AW','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',119.44,1481,1831,74.18,'Intermediate' from dual union all
select 21184,'021184','1991-009AX','deb S3M','',64178,'In Earth orbit','','1991-10-11 00:00:00',118.12,1490,1703,74.02,'Intermediate' from dual union all
select 21185,'021185','1991-009AY','deb S3M','',64178,'In Earth orbit','','1991-10-02 00:00:00',118.42,1446,1774,74.02,'Intermediate' from dual union all
select 21186,'021186','1991-009AZ','deb S3M','',64178,'In Earth orbit','','1991-10-12 00:00:00',118.76,1506,1745,74.09,'Intermediate' from dual union all
select 21187,'021187','1991-009BA','deb S3M','',64178,'In Earth orbit','','1991-10-10 00:00:00',119.12,1496,1787,74.2,'Intermediate' from dual union all
select 21188,'021188','1991-020A','Progress M-7','Progress 7K-TGM No. 208',64189,'Deorbited','1991-05-07 00:00:00','1991-03-28 00:00:00',92.07,365,388,51.61,'Intermediate' from dual union all
select 21189,'021189','1991-020B','11S510','11S510',64189,'Reentered','1991-03-20 00:00:00','1991-03-20 00:00:00',87.71,157,166,51.65,'Intermediate' from dual union all
select 2119,'002119','1966-024A','NNS O-8','NNS O-8',13431,'In Earth orbit','','1966-09-20 00:00:00',105.29,892,1123,89.72,'Polar' from dual union all
select 21190,'021190','1991-021A','Kosmos-2137','Taifun-1B',64190,'Reentered','1995-04-03 00:00:00','1991-04-26 00:00:00',93.91,441,491,65.84,'Intermediate' from dual union all
select 21191,'021191','1991-021B','S3M','S3M',64190,'Reentered','1992-06-27 00:00:00','1991-04-26 00:00:00',93.77,432,485,65.84,'Intermediate' from dual union all
select 21192,'021192','1991-016E','Blenda','Sunshade',64185,'Reentered','1991-03-22 00:00:00','1991-03-20 00:00:00',87.81,152,181,62.85,'Intermediate' from dual union all
select 21193,'021193','1991-016F','KDU','KDU',64185,'Reentered','1991-03-24 00:00:00','1991-03-21 00:00:00',88.51,184,218,62.85,'Intermediate' from dual union all
select 21194,'021194','1991-016G','KDU part','KDU part',64185,'Reentered','1991-03-21 00:00:00','1991-03-20 00:00:00',90.49,223,374,62.83,'Intermediate' from dual union all
select 21195,'021195','1991-016H','KDU part','KDU part',64185,'Reentered','1991-03-22 00:00:00','1991-03-21 00:00:00',89.07,158,298,62.89,'Intermediate' from dual union all
select 21196,'021196','1991-022A','Molniya-3','Molniya-3 No. 55',64191,'In Earth orbit','','1991-04-21 00:00:00',717.74,453,39899,62.87,'Molniya' from dual union all
select 21197,'021197','1991-022B','11S510','11S510',64191,'Reentered','1991-04-02 00:00:00','1991-03-28 00:00:00',90.03,192,360,62.83,'Intermediate' from dual union all
select 21198,'021198','1991-022C','BOZ','BOZ',64191,'Reentered','1991-03-28 00:00:00','1991-03-28 00:00:00',88.14,143,222,62.81,'Intermediate' from dual union all
select 21199,'021199','1991-022D','Blok-ML','Blok-ML No. 339',64191,'In Earth orbit','','1991-04-22 00:00:00',700.22,490,38995,62.86,'Molniya' from dual union all
select 212,'000212','61 Omicron 55','deb r/b Ablestar','',7273,'Reentered','1988-08-11 00:00:00','1962-01-08 00:00:00',102.66,659,1110,68.06,'Intermediate' from dual union all
select 2120,'002120','1966-024B','Altair 2 X-258E6 RH-118','X-258E6',13431,'In Earth orbit','','1966-09-23 00:00:00',105.3,893,1124,89.73,'Polar' from dual union all
select 21200,'021200','1991-022E','deb Blok-I','',64191,'Reentered','1991-03-29 00:00:00','1991-03-28 00:00:00',89.16,171,295,62.81,'Intermediate' from dual union all
select 21201,'021201','1991-014E','SOZ','SOZ',64183,'Reentered','1991-09-10 00:00:00','1991-04-01 00:00:00',618.09,285,35031,47.5,'GEO Transfer' from dual union all
select 21202,'021202','1991-014F','SOZ','SOZ',64183,'Reentered','1991-08-10 00:00:00','1991-04-01 00:00:00',616.24,231,34989,47.5,'GEO Transfer' from dual union all
select 21203,'021203','1991-023A','Kosmos-2138','Kobal''t No. 536',64192,'Landed','1991-05-24 00:00:00','1991-04-02 00:00:00',89.89,187,351,67.15,'Intermediate' from dual union all
select 21204,'021204','1991-023B','11S510','11S510',64192,'Reentered','1991-03-29 00:00:00','1991-03-28 00:00:00',88.62,150,263,67.16,'Intermediate' from dual union all
select 21205,'021205','1989-100AC','Kosmos-2053 SS 24','ESO',62377,'Reentered','1991-04-24 00:00:00','1991-04-14 00:00:00',93.17,423,437,73.53,'Intermediate' from dual union all
select 21206,'021206','1989-100AD','Kosmos-2053 SS 25','ESO',62377,'Reentered','1991-04-23 00:00:00','1991-04-14 00:00:00',93.06,418,431,73.51,'Intermediate' from dual union all
select 21207,'021207','1989-100AE','Kosmos-2053 SS 26','ESO',62377,'Reentered','1991-04-27 00:00:00','1991-04-13 00:00:00',93.46,435,453,73.52,'Intermediate' from dual union all
select 21208,'021208','1991-009BB','deb S3M','',64178,'In Earth orbit','','1991-09-10 00:00:00',117.11,1432,1669,73.96,'Intermediate' from dual union all
select 21209,'021209','1991-009BC','deb S3M','',64178,'In Earth orbit','','1991-09-09 00:00:00',117.69,1470,1684,74.04,'Intermediate' from dual union all
select 2121,'002121','1966-025A','OV1-4','OV1-4S',13432,'In Earth orbit','','1967-01-01 00:00:00',104.07,888,1014,144.52,'Retrograde' from dual union all
select 21210,'021210','1991-009BD','deb S3M','',64178,'In Earth orbit','','1991-09-11 00:00:00',118.29,1494,1715,74.1,'Intermediate' from dual union all
select 21211,'021211','1991-009BE','deb S3M','',64178,'In Earth orbit','','1991-09-08 00:00:00',118.19,1438,1761,74.09,'Intermediate' from dual union all
select 21212,'021212','1991-009BF','deb S3M','',64178,'In Earth orbit','','1991-09-07 00:00:00',119.32,1463,1838,74.16,'Intermediate' from dual union all
select 21213,'021213','1991-024A','Almaz-1','Mech-KU No. 305',64193,'Deorbited','1992-10-17 00:00:00','1991-05-01 00:00:00',89.99,267,281,72.7,'Intermediate' from dual union all
select 21214,'021214','1991-024B','8S812','8S812',64193,'Reentered','1991-04-02 00:00:00','1991-04-01 00:00:00',88.26,157,219,72.71,'Intermediate' from dual union all
select 21215,'021215','1990-105AD','deb Star 37S','',63347,'Reentered','1991-04-08 00:00:00','1991-04-05 00:00:00',93.42,408,476,98.82,'Sun-Synch' from dual union all
select 21216,'021216','1991-025A','Kosmos-2139','Uragan No. 50L',64194,'In Earth orbit','','1991-05-04 00:00:00',675.71,19112,19147,64.82,'Medium' from dual union all
select 21217,'021217','1991-025B','Kosmos-2140','Uragan No. 53L',64194,'In Earth orbit','','1991-05-04 00:00:00',675.69,19104,19154,64.81,'Medium' from dual union all
select 21218,'021218','1991-025C','Kosmos-2141','Uragan No. 54L',64194,'In Earth orbit','','1991-05-04 00:00:00',675.71,19108,19151,64.8,'Medium' from dual union all
select 21219,'021219','1991-025D','8S812','8S812',64194,'Reentered','1991-04-04 00:00:00','1991-04-04 00:00:00',87.02,118,136,64.82,'Intermediate' from dual union all
select 2122,'002122','1966-025B','OV1-5','OV1-5S',13432,'In Earth orbit','','1966-11-09 00:00:00',105.62,993,1054,144.65,'Retrograde' from dual union all
select 21220,'021220','1991-025F','SOZ','SOZ',64194,'In Earth orbit','','1992-02-16 00:00:00',339.54,404,19084,64.99,'Highly Ellip' from dual union all
select 21221,'021221','1991-025E','Blok DM2','11S861 No. 53L',64194,'In Earth orbit','','1992-02-15 00:00:00',675.49,19112,19136,64.96,'Medium' from dual union all
select 21222,'021222','1991-026A','Anik E2','Anik E2',64195,'In Earth orbit','','1991-05-06 00:00:00',1435.22,35732,35807,.1,'Drift GEO' from dual union all
select 21223,'021223','1991-026B','Ariane H10 V43','Ariane H10 S/N T416',64195,'In Earth orbit','','1992-02-20 00:00:00',635.34,417,35789,3.57,'GEO Transfer' from dual union all
select 21224,'021224','1991-027A','Atlantis (STS-37)','OV-104',64196,'Landed','1991-04-11 00:00:00','1991-04-07 00:00:00',93.61,445,458,28.46,'Intermediate' from dual union all
select 21225,'021225','1991-027B','Compton Observatory','Gamma Ray Observatory',64196,'Deorbited','2000-06-04 00:00:00','1991-05-05 00:00:00',93.54,441,455,28.47,'Intermediate' from dual union all
select 21226,'021226','1991-025G','SOZ','SOZ',64194,'In Earth orbit','','1991-05-07 00:00:00',340.03,397,19121,64.75,'Highly Ellip' from dual union all
select 21227,'021227','1991-028A','ASC 2','ASC 2',64197,'In Earth orbit','','1991-05-13 00:00:00',1436.06,35773,35799,.23,'Stationary' from dual union all
select 21228,'021228','1991-028B','Delta 204','Delta SSPS AJ10-118K',64197,'In Earth orbit','','1991-05-13 00:00:00',115.52,556,2402,24.02,'Equatorial' from dual union all
select 21229,'021229','1991-028C','Star 48B','Star 48B S/N 10067-18',64197,'In Earth orbit','','1991-05-17 00:00:00',655.43,1351,35882,22.59,'GEO Transfer' from dual union all
select 2123,'002123','1966-025C','OV1-5P','X-258',13432,'In Earth orbit','','1966-10-27 00:00:00',105.62,985,1062,144.66,'Retrograde' from dual union all
select 21230,'021230','1991-029A','Kosmos-2142','Parus',64198,'In Earth orbit','','1991-05-17 00:00:00',104.92,964,1018,82.96,'Intermediate' from dual union all
select 21231,'021231','1991-029B','S3M','S3M',64198,'In Earth orbit','','1991-05-18 00:00:00',104.75,957,1009,82.95,'Intermediate' from dual union all
select 21232,'021232','1991-030A','Meteor-3','Meteor-3 No. 6',64199,'In Earth orbit','','1991-05-25 00:00:00',109.41,1186,1213,82.55,'Intermediate' from dual union all
select 21233,'021233','1991-030B','S5M','S5M',64199,'In Earth orbit','','1992-02-16 00:00:00',109.4,1187,1211,82.54,'Intermediate' from dual union all
select 21234,'021234','1991-009CT','deb S3M','',64178,'In Earth orbit','','1991-04-16 00:00:00',116.96,1444,1644,74,'Intermediate' from dual union all
select 21235,'021235','1986-017CZ','EO-8 EVA-4 debris','',59210,'Reentered','1991-08-25 00:00:00','1991-08-16 00:00:00',88.77,209,219,51.6,'Intermediate' from dual union all
select 21236,'021236','1986-017DA','EO-8 EVA-4 debris','',59210,'Reentered','1991-06-24 00:00:00','1991-04-26 00:00:00',91.94,365,375,51.6,'Intermediate' from dual union all
select 21237,'021237','1986-017DB','EO-8 EVA-4 debris','',59210,'Reentered','1991-06-15 00:00:00','1991-04-27 00:00:00',91.9,365,370,51.61,'Intermediate' from dual union all
select 21238,'021238','1986-017DC','EO-8 EVA-4 debris','',59210,'Reentered','1991-06-07 00:00:00','1991-04-27 00:00:00',91.9,364,372,51.61,'Intermediate' from dual union all
select 21239,'021239','1986-017DD','EO-8 EVA-4 debris','',59210,'Reentered','1991-06-25 00:00:00','1991-04-28 00:00:00',91.93,364,374,51.6,'Intermediate' from dual union all
select 2124,'002124','1966-025D','OV1-4P','X-258',13432,'In Earth orbit','','1966-10-26 00:00:00',104.05,890,1010,144.52,'Retrograde' from dual union all
select 21240,'021240','1986-017DE','EO-8 EVA-4 debris','',59210,'Reentered','1991-11-11 00:00:00','1991-09-13 00:00:00',90.81,313,316,51.63,'Intermediate' from dual union all
select 21241,'021241','1986-017DF','EO-8 EVA-4 debris','',59210,'Reentered','1991-06-09 00:00:00','1991-04-28 00:00:00',91.92,363,375,51.61,'Intermediate' from dual union all
select 21242,'021242','1991-031A','Discovery (STS-39)','OV-103',64200,'Landed','1991-05-06 00:00:00','1991-04-30 00:00:00',89.57,247,260,57,'Intermediate' from dual union all
select 21243,'021243','1986-017DG','EVA debris','',59210,'Reentered','1991-06-07 00:00:00','1991-04-29 00:00:00',91.84,359,371,51.61,'Intermediate' from dual union all
select 21244,'021244','1991-031B','IBSS/SPAS','SPAS-2',64200,'Landed Attached','1991-05-06 00:00:00','1991-05-02 00:00:00',89.49,242,257,56.99,'Intermediate' from dual union all
select 21245,'021245','1991-031D','CRO-C','CRO-CO',64200,'Reentered','1991-05-14 00:00:00','1991-05-07 00:00:00',89.02,218,235,56.99,'Intermediate' from dual union all
select 21246,'021246','1991-031E','CRO-B','CRO-BU',64200,'Reentered','1991-05-12 00:00:00','1991-05-03 00:00:00',89.49,241,258,57,'Intermediate' from dual union all
select 21247,'021247','1991-031F','CRO-A','CRO-AM',64200,'Reentered','1991-05-13 00:00:00','1991-05-07 00:00:00',89.3,229,250,56.99,'Intermediate' from dual;
insert into SATELLITE
select 21248,'021248','1986-017DH','EO-8 EVA-4 debris','',59210,'Reentered','1991-08-21 00:00:00','1991-05-10 00:00:00',91.81,358,368,51.6,'Intermediate' from dual union all
select 21249,'021249','1991-009BG','deb S3M','',64178,'In Earth orbit','','1991-05-09 00:00:00',117.87,1486,1685,74.07,'Intermediate' from dual union all
select 2125,'002125','1966-026A','OPS 0340','DAPP 15',13433,'In Earth orbit','','1966-10-19 00:00:00',100.49,627,937,98.59,'Sun-Synch' from dual union all
select 21250,'021250','1991-009BH','deb S3M','',64178,'In Earth orbit','','1991-05-09 00:00:00',118.24,1487,1717,74.06,'Intermediate' from dual union all
select 21251,'021251','1991-009BJ','deb S3M','',64178,'In Earth orbit','','1991-05-09 00:00:00',116.66,1409,1653,73.96,'Intermediate' from dual union all
select 21252,'021252','1991-009BK','deb S3M','',64178,'In Earth orbit','','1991-05-09 00:00:00',117.66,1474,1678,74.09,'Intermediate' from dual union all
select 21253,'021253','1991-009BL','deb S3M','',64178,'In Earth orbit','','1991-05-09 00:00:00',118.56,1498,1735,74.11,'Intermediate' from dual union all
select 21254,'021254','1991-009BM','deb S3M','',64178,'In Earth orbit','','1991-05-09 00:00:00',116.2,1361,1659,73.89,'Intermediate' from dual union all
select 21255,'021255','1990-111C','deb Kosmos-2112','Strela-2M debris 1',63353,'In Earth orbit','','1991-05-09 00:00:00',100.86,787,812,74.05,'Intermediate' from dual union all
select 21256,'021256','1991-009BN','deb S3M','',64178,'In Earth orbit','','1991-05-09 00:00:00',121.68,1554,1958,73.95,'Medium' from dual union all
select 21257,'021257','1991-007D','deb Kosmos-2123','',64176,'In Earth orbit','','1991-05-08 00:00:00',104.86,961,1015,82.93,'Intermediate' from dual union all
select 21258,'021258','1990-104K','Kosmos-2106 SS 7','ESO',63346,'Reentered','1991-07-15 00:00:00','1991-05-12 00:00:00',94.99,507,530,82.5,'Intermediate' from dual union all
select 21259,'021259','1990-104L','Kosmos-2106 SS 8','ESO',63346,'Reentered','1991-07-10 00:00:00','1991-05-12 00:00:00',94.85,498,525,82.5,'Intermediate' from dual union all
select 2126,'002126','1966-027A','Luna-10','E-6S No. 206 ISL',13434,'Deep Space','1966-04-03 00:00:00','1966-03-31 00:00:00',21544.43,200,500000,52,'Cislunar' from dual union all
select 21260,'021260','1990-104M','Kosmos-2106 SS 9','ESO',63346,'Reentered','1991-07-10 00:00:00','1991-05-12 00:00:00',94.82,498,521,82.51,'Intermediate' from dual union all
select 21261,'021261','1990-104N','Kosmos-2106 SS 10','ESO',63346,'Reentered','1991-07-10 00:00:00','1991-05-13 00:00:00',94.81,498,521,82.51,'Intermediate' from dual union all
select 21262,'021262','1991-031C','USA 70','MPEC satellite',64200,'Reentered','1991-07-01 00:00:00','1991-05-06 00:00:00',89.68,253,265,57,'Intermediate' from dual union all
select 21263,'021263','1991-032A','NOAA 12','NOAA D',64201,'In Earth orbit','','1991-06-13 00:00:00',101.29,810,829,98.73,'Sun-Synch' from dual union all
select 21264,'021264','1990-104P','Kosmos-2106 SS 11','ESO',63346,'Reentered','1991-07-21 00:00:00','1991-05-15 00:00:00',94.95,510,523,82.5,'Intermediate' from dual union all
select 21265,'021265','1990-104Q','Kosmos-2106 SS 12','ESO',63346,'Reentered','1991-07-14 00:00:00','1991-05-15 00:00:00',94.84,497,525,82.5,'Intermediate' from dual union all
select 21266,'021266','1990-104R','Kosmos-2106 SS 13','ESO',63346,'Reentered','1991-07-20 00:00:00','1991-05-15 00:00:00',94.97,510,525,82.51,'Intermediate' from dual union all
select 21267,'021267','1991-032B','NOAA 12 solar array band','',64201,'Reentered','2001-12-26 00:00:00','1992-02-20 00:00:00',100.88,795,805,98.72,'Sun-Synch' from dual union all
select 21268,'021268','1975-052D','deb Delta 111','',36499,'In Earth orbit','','1991-06-04 00:00:00',110.41,1091,1400,99.83,'Sun-Synch' from dual union all
select 21269,'021269','1975-052E','deb Delta 111','',36499,'In Earth orbit','','1991-05-29 00:00:00',109.01,1088,1274,99.69,'Sun-Synch' from dual union all
select 2127,'002127','1966-027B','BOZ','BOZ',13434,'Reentered','1966-04-04 00:00:00','1966-03-31 00:00:00',88.99,200,250,52,'Intermediate' from dual union all
select 21270,'021270','1975-052F','deb Delta 111','',36499,'In Earth orbit','','1991-05-29 00:00:00',115.28,1094,1843,99.44,'Sun-Synch' from dual union all
select 21271,'021271','1975-052G','deb Delta 111','',36499,'In Earth orbit','','1991-05-29 00:00:00',110.86,1064,1468,99.62,'Sun-Synch' from dual union all
select 21272,'021272','1975-052H','deb Delta 111','',36499,'In Earth orbit','','1991-05-31 00:00:00',106.43,1027,1095,99.43,'Sun-Synch' from dual union all
select 21273,'021273','1975-052J','deb Delta 111','',36499,'In Earth orbit','','1991-05-17 00:00:00',107.74,1095,1149,99.75,'Sun-Synch' from dual union all
select 21274,'021274','1975-052K','deb Delta 111','',36499,'Reentered','1997-10-17 00:00:00','1991-05-31 00:00:00',106.43,1027,1095,99.43,'Sun-Synch' from dual union all
select 21275,'021275','1975-052L','deb Delta 111','',36499,'In Earth orbit','','1991-05-29 00:00:00',106.68,1047,1099,99.46,'Sun-Synch' from dual union all
select 21276,'021276','1975-052M','deb Delta 111','',36499,'In Earth orbit','','1991-05-19 00:00:00',107.9,1095,1165,99.74,'Sun-Synch' from dual union all
select 21277,'021277','1975-052N','deb Delta 111','',36499,'In Earth orbit','','1991-05-28 00:00:00',109.35,1094,1300,99.69,'Sun-Synch' from dual union all
select 21278,'021278','1975-052P','deb Delta 111','',36499,'In Earth orbit','','1991-05-29 00:00:00',112.92,1057,1664,99.82,'Sun-Synch' from dual union all
select 21279,'021279','1975-052Q','deb Delta 111','',36499,'In Earth orbit','','1991-05-29 00:00:00',106.27,1012,1095,99.49,'Sun-Synch' from dual union all
select 2128,'002128','1966-027C','11S510','11S510',13434,'Reentered','1966-04-02 00:00:00','1966-03-31 00:00:00',88.99,200,250,52,'Intermediate' from dual union all
select 21280,'021280','1975-052R','deb Delta 111','',36499,'In Earth orbit','','1991-05-29 00:00:00',106.36,1020,1096,99.48,'Sun-Synch' from dual union all
select 21281,'021281','1975-052S','deb Delta 111','',36499,'In Earth orbit','','1991-05-29 00:00:00',108.06,1094,1180,99.75,'Sun-Synch' from dual union all
select 21282,'021282','1975-052T','deb Delta 111','',36499,'In Earth orbit','','1991-05-19 00:00:00',108.8,1095,1248,99.7,'Sun-Synch' from dual union all
select 21283,'021283','1975-052U','deb Delta 111','',36499,'In Earth orbit','','1991-06-05 00:00:00',111.14,1095,1463,99.73,'Sun-Synch' from dual union all
select 21284,'021284','1975-052V','deb Delta 111','',36499,'In Earth orbit','','1991-06-03 00:00:00',108.34,1090,1210,99.7,'Sun-Synch' from dual union all
select 21285,'021285','1975-052W','deb Delta 111','',36499,'In Earth orbit','','1991-06-02 00:00:00',107.44,1094,1123,99.68,'Sun-Synch' from dual union all
select 21286,'021286','1975-052X','deb Delta 111','',36499,'In Earth orbit','','1992-01-30 00:00:00',108.03,1095,1177,99.67,'Sun-Synch' from dual union all
select 21287,'021287','1975-052Y','deb Delta 111','',36499,'In Earth orbit','','1992-01-30 00:00:00',109.29,1095,1293,99.68,'Sun-Synch' from dual union all
select 21288,'021288','1975-052Z','deb Delta 111','',36499,'In Earth orbit','','1991-05-30 00:00:00',112.98,1089,1638,99.65,'Sun-Synch' from dual union all
select 21289,'021289','1975-052AA','deb Delta 111','',36499,'In Earth orbit','','1991-06-05 00:00:00',111.05,1087,1463,99.91,'Sun-Synch' from dual union all
select 2129,'002129','1966-026B','FW4S S/N 20036','FW4S',13433,'Reentered','2002-05-10 00:00:00','1966-10-27 00:00:00',100.49,627,937,98.59,'Sun-Synch' from dual union all
select 21290,'021290','1975-052AB','deb Delta 111','',36499,'In Earth orbit','','1991-06-03 00:00:00',115.93,1091,1905,99.5,'Sun-Synch' from dual union all
select 21291,'021291','1975-052AC','deb Delta 111','',36499,'In Earth orbit','','1991-05-29 00:00:00',104.8,874,1097,99.89,'Sun-Synch' from dual union all
select 21292,'021292','1975-052AD','deb Delta 111','',36499,'In Earth orbit','','1991-05-29 00:00:00',106.47,889,1237,100.01,'Sun-Synch' from dual union all
select 21293,'021293','1975-052AE','deb Delta 111','',36499,'In Earth orbit','','1991-05-30 00:00:00',111.69,1093,1516,99.88,'Sun-Synch' from dual union all
select 21294,'021294','1975-052AF','deb Delta 111','',36499,'In Earth orbit','','1991-06-05 00:00:00',111.46,1092,1496,99.83,'Sun-Synch' from dual union all
select 21295,'021295','1975-052AG','deb Delta 111','',36499,'In Earth orbit','','1991-05-29 00:00:00',107.52,1095,1128,100.08,'Sun-Synch' from dual union all
select 21296,'021296','1975-052AH','deb Delta 111','',36499,'In Earth orbit','','1991-06-01 00:00:00',110.22,1052,1422,99.6,'Sun-Synch' from dual union all
select 21297,'021297','1975-052AJ','deb Delta 111','',36499,'In Earth orbit','','1991-06-02 00:00:00',108.22,1090,1199,99.74,'Sun-Synch' from dual union all
select 21298,'021298','1991-032C','NOAA 12 solar array band','',64201,'Reentered','2002-03-28 00:00:00','1992-02-19 00:00:00',100.89,796,806,98.71,'Sun-Synch' from dual union all
select 21299,'021299','1991-033A','Kosmos-2143','Strela-3',64202,'In Earth orbit','','1991-09-05 00:00:00',113.94,1399,1416,82.57,'Intermediate' from dual union all
select 213,'000213','61 A Kappa 1','Discoverer 36','CORONA C''''''-5',7256,'Reentered Attached','1962-03-08 00:00:00','1962-01-11 00:00:00',91.22,223,446,81.18,'Intermediate' from dual union all
select 2130,'002130','1966-027D','Blok-L No. N103-42','Blok-L',13434,'Deep Space','1966-04-03 00:00:00','1966-03-31 00:00:00',21544.43,200,500000,52,'Cislunar' from dual union all
select 21300,'021300','1991-033B','Kosmos-2144','Strela-3',64202,'In Earth orbit','','1991-09-05 00:00:00',114.08,1411,1417,82.58,'Intermediate' from dual union all
select 21301,'021301','1991-033C','Kosmos-2145','Strela-3',64202,'In Earth orbit','','1991-09-05 00:00:00',114.01,1405,1416,82.57,'Intermediate' from dual union all
select 21302,'021302','1991-033D','Kosmos-2146','Strela-3',64202,'In Earth orbit','','1991-09-05 00:00:00',113.89,1394,1416,82.58,'Intermediate' from dual union all
select 21303,'021303','1991-033E','Kosmos-2147','Strela-3',64202,'In Earth orbit','','1991-09-05 00:00:00',113.83,1389,1416,82.57,'Intermediate' from dual union all
select 21304,'021304','1991-033F','Kosmos-2148','Strela-3',64202,'In Earth orbit','','1991-09-05 00:00:00',113.76,1382,1415,82.58,'Intermediate' from dual union all
select 21305,'021305','1991-033G','S5M','S5M',64202,'In Earth orbit','','1991-09-07 00:00:00',114.76,1416,1473,82.57,'Intermediate' from dual union all
select 21306,'021306','1990-104S','Kosmos-2106 SS 14','ESO',63346,'Reentered','1991-07-21 00:00:00','1991-05-16 00:00:00',94.94,509,523,82.51,'Intermediate' from dual union all
select 21307,'021307','1990-104T','Kosmos-2106 SS 15','ESO',63346,'Reentered','1991-07-22 00:00:00','1991-05-18 00:00:00',94.94,509,523,82.5,'Intermediate' from dual union all
select 21308,'021308','1990-104U','Kosmos-2106 SS 16','ESO',63346,'Reentered','1991-07-15 00:00:00','1991-05-17 00:00:00',94.85,498,525,82.51,'Intermediate' from dual union all
select 21309,'021309','1990-104V','Kosmos-2106 SS 17','ESO',63346,'Reentered','1991-07-15 00:00:00','1991-05-17 00:00:00',94.83,497,523,82.5,'Intermediate' from dual union all
select 2131,'002131','1966-027E','Otdelyaemiy otsek No. 1','Otdelyaemiy otsek No. 1',13434,'Deep Space Attached','1966-04-03 00:00:00','1966-03-31 00:00:00',21544.43,200,500000,52,'Cislunar' from dual union all
select 21310,'021310','1990-104W','radar-error','',63346,'Spurious','1991-05-19 00:00:00','1991-05-17 00:00:00',94.85,501,522,82.5,'Intermediate' from dual union all
select 21311,'021311','1991-034A','Soyuz TM-12','Soyuz 7K-STM No. 62',64203,'Landed','1991-10-10 00:00:00','1991-06-18 00:00:00',92.37,390,392,51.61,'Intermediate' from dual union all
select 21312,'021312','1991-034B','11S510','11S510',64203,'Reentered','1991-05-19 00:00:00','1991-05-18 00:00:00',88.42,194,199,51.63,'Intermediate' from dual union all
select 21313,'021313','1991-035A','Resurs-F','Resurs-F2 17F42 No. 6',64204,'Landed','1991-06-20 00:00:00','1991-05-28 00:00:00',89.11,226,235,82.32,'Intermediate' from dual union all
select 21314,'021314','1991-035B','11S510','11S510',64204,'Reentered','1991-05-23 00:00:00','1991-05-22 00:00:00',88.48,176,222,82.32,'Intermediate' from dual union all
select 21315,'021315','1991-036A','Kosmos-2149','Kobal''t No. 521',64205,'Landed','1991-07-04 00:00:00','1991-05-31 00:00:00',90,185,364,67.13,'Intermediate' from dual union all
select 21316,'021316','1991-036B','11S510','11S510',64205,'Reentered','1991-05-27 00:00:00','1991-05-27 00:00:00',87.23,116,158,67.12,'Intermediate' from dual union all
select 21317,'021317','1975-052AK','deb Delta 111','',36499,'In Earth orbit','','1991-05-31 00:00:00',106.43,1027,1095,99.43,'Sun-Synch' from dual union all
select 21318,'021318','1975-052AL','deb Delta 111','',36499,'In Earth orbit','','1991-06-04 00:00:00',106.31,802,1310,99.58,'Sun-Synch' from dual union all
select 21319,'021319','1975-052AM','deb Delta 111','',36499,'In Earth orbit','','1991-06-04 00:00:00',110.64,1069,1444,99.69,'Sun-Synch' from dual union all
select 2132,'002132','1966-027F','Otdelyaemiy otsek No. 2','Otdelyaemiy otsek No. 2',13434,'Deep Space Attached','1966-04-03 00:00:00','1966-03-31 00:00:00',21544.43,200,500000,52,'Cislunar' from dual union all
select 21320,'021320','1975-052AN','deb Delta 111','',36499,'In Earth orbit','','1991-09-10 00:00:00',105.29,909,1107,99.76,'Sun-Synch' from dual union all
select 21321,'021321','1975-052AP','deb Delta 111','',36499,'Reentered','2011-11-07 00:00:00','1991-06-06 00:00:00',105.8,966,1098,99.26,'Sun-Synch' from dual union all
select 21322,'021322','1975-052AQ','deb Delta 111','',36499,'Reentered','2011-06-13 00:00:00','1991-06-03 00:00:00',105.86,955,1114,99.24,'Sun-Synch' from dual union all
select 21323,'021323','1975-052AR','deb Delta 111','',36499,'Reentered','2002-04-18 00:00:00','1991-06-05 00:00:00',107.19,804,1390,99.34,'Sun-Synch' from dual union all
select 21324,'021324','1975-052AS','deb Delta 111','',36499,'In Earth orbit','','1991-06-04 00:00:00',111.57,1033,1564,99.59,'Sun-Synch' from dual union all
select 21325,'021325','1975-052AT','deb Delta 111','',36499,'In Earth orbit','','1991-06-04 00:00:00',106.29,950,1160,99.8,'Sun-Synch' from dual union all
select 21326,'021326','1975-052AU','deb Delta 111','',36499,'In Earth orbit','','1992-02-06 00:00:00',107.6,1092,1140,99.67,'Sun-Synch' from dual union all
select 21327,'021327','1975-052AV','deb Delta 111','',36499,'In Earth orbit','','1992-02-08 00:00:00',111.74,1092,1522,99.67,'Sun-Synch' from dual union all
select 21328,'021328','1975-052AW','deb Delta 111','',36499,'Reentered','2014-04-30 00:00:00','1991-06-05 00:00:00',104.33,827,1100,99.73,'Sun-Synch' from dual union all
select 21329,'021329','1975-052AX','deb Delta 111','',36499,'In Earth orbit','','1991-06-04 00:00:00',107.38,830,1380,99.73,'Sun-Synch' from dual union all
select 2133,'002133','1966-028A','Kosmos-114','Zenit-4 No. 19',13435,'Landed','1966-04-14 00:00:00','1966-04-06 00:00:00',90.05,201,352,72.9,'Intermediate' from dual union all
select 21330,'021330','1975-052AY','deb Delta 111','',36499,'In Earth orbit','','1991-06-03 00:00:00',108.37,913,1390,99.84,'Sun-Synch' from dual union all
select 21331,'021331','1975-052AZ','deb Delta 111','',36499,'In Earth orbit','','1991-06-04 00:00:00',107.9,1095,1165,99.74,'Sun-Synch' from dual union all
select 21332,'021332','1991-009CP','deb S3M','',64178,'In Earth orbit','','1992-04-14 00:00:00',118.76,1511,1740,74.19,'Intermediate' from dual union all
select 21333,'021333','1975-052BB','deb Delta 111','',36499,'In Earth orbit','','1991-05-29 00:00:00',96.61,521,671,98.82,'Sun-Synch' from dual union all
select 21334,'021334','1975-052BC','deb Delta 111','',36499,'Reentered','2000-12-04 00:00:00','1991-06-05 00:00:00',104.9,799,1181,98.99,'Sun-Synch' from dual union all
select 21335,'021335','1975-052BD','deb Delta 111','',36499,'In Earth orbit','','1991-06-05 00:00:00',108.87,1094,1255,99.14,'Sun-Synch' from dual union all
select 21336,'021336','1975-052BE','deb Delta 111','',36499,'In Earth orbit','','1991-06-05 00:00:00',110.35,1094,1391,99.14,'Sun-Synch' from dual union all
select 21337,'021337','1975-052BF','deb Delta 111','',36499,'Reentered','1998-12-05 00:00:00','1991-06-04 00:00:00',104.15,817,1093,99.36,'Sun-Synch' from dual;
insert into SATELLITE
select 21338,'021338','1975-052BG','deb Delta 111','',36499,'In Earth orbit','','1992-02-18 00:00:00',107.98,1087,1180,99.49,'Sun-Synch' from dual union all
select 21339,'021339','1975-052BH','deb Delta 111','',36499,'In Earth orbit','','1992-02-14 00:00:00',113.27,1080,1673,99.54,'Sun-Synch' from dual union all
select 2134,'002134','1966-028B','11A57I','11A57I',13435,'Reentered','1966-04-18 00:00:00','1966-04-06 00:00:00',90.05,201,352,72.9,'Intermediate' from dual union all
select 21340,'021340','1975-052BJ','deb Delta 111','',36499,'Reentered','2013-12-24 00:00:00','1991-06-05 00:00:00',113.95,936,1880,99.55,'Sun-Synch' from dual union all
select 21341,'021341','1975-052BK','deb Delta 111','',36499,'In Earth orbit','','1992-02-17 00:00:00',114.66,1041,1839,99.72,'Sun-Synch' from dual union all
select 21342,'021342','1975-052BL','deb Delta 111','',36499,'In Earth orbit','','1991-06-04 00:00:00',103.7,772,1095,100.03,'Sun-Synch' from dual union all
select 21343,'021343','1975-052BM','deb Delta 111','',36499,'In Earth orbit','','1992-02-18 00:00:00',106.9,956,1211,99,'Sun-Synch' from dual union all
select 21344,'021344','1975-052BN','deb Delta 111','',36499,'Reentered','2005-03-02 00:00:00','1991-06-03 00:00:00',105.75,961,1097,99.35,'Sun-Synch' from dual union all
select 21345,'021345','1975-052BP','deb Delta 111','',36499,'In Earth orbit','','1991-06-04 00:00:00',110.37,1094,1394,99.36,'Sun-Synch' from dual union all
select 21346,'021346','1975-052BQ','deb Delta 111','',36499,'In Earth orbit','','1992-02-17 00:00:00',112.4,956,1717,99.34,'Sun-Synch' from dual union all
select 21347,'021347','1975-052BR','deb Delta 111','',36499,'In Earth orbit','','1992-02-17 00:00:00',112.57,955,1735,99.45,'Sun-Synch' from dual union all
select 21348,'021348','1975-052BS','deb Delta 111','',36499,'Reentered','2014-09-25 00:00:00','1991-06-05 00:00:00',106.37,1020,1097,99.43,'Sun-Synch' from dual union all
select 21349,'021349','1975-052BT','deb Delta 111','',36499,'Reentered','1991-06-29 00:00:00','1991-06-05 00:00:00',96.78,394,815,99.41,'Sun-Synch' from dual union all
select 2135,'002135','1966-028C','deb Kosmos-114','',13435,'Reentered','1966-04-10 00:00:00','1966-04-06 00:00:00',89.62,195,317,73,'Intermediate' from dual union all
select 21350,'021350','1975-052BU','deb Delta 111','',36499,'Reentered','2017-12-08 00:00:00','1991-06-05 00:00:00',106.45,1032,1093,99.43,'Sun-Synch' from dual union all
select 21351,'021351','1975-052BV','deb Delta 111','',36499,'In Earth orbit','','1991-09-09 00:00:00',108.22,1095,1194,99.7,'Sun-Synch' from dual union all
select 21352,'021352','1975-052BW','deb Delta 111','',36499,'In Earth orbit','','1991-06-06 00:00:00',111.07,1015,1537,99.49,'Sun-Synch' from dual union all
select 21353,'021353','1975-052BX','deb Delta 111','',36499,'In Earth orbit','','1991-06-05 00:00:00',116.19,1094,1925,99.53,'Sun-Synch' from dual union all
select 21354,'021354','1975-052BY','deb Delta 111','',36499,'In Earth orbit','','1991-06-03 00:00:00',110.19,1035,1435,99.51,'Sun-Synch' from dual union all
select 21355,'021355','1975-052BZ','deb Delta 111','',36499,'In Earth orbit','','1991-06-05 00:00:00',109.62,1095,1323,99.63,'Sun-Synch' from dual union all
select 21356,'021356','1975-052CA','deb Delta 111','',36499,'In Earth orbit','','1991-06-01 00:00:00',108.41,1083,1223,99.6,'Sun-Synch' from dual union all
select 21357,'021357','1975-052CB','deb Delta 111','',36499,'Reentered','1992-08-18 00:00:00','1991-06-05 00:00:00',101.79,597,1090,99.59,'Sun-Synch' from dual union all
select 21358,'021358','1975-052CC','deb Delta 111','',36499,'In Earth orbit','','1991-06-03 00:00:00',110.01,1094,1361,99.6,'Sun-Synch' from dual union all
select 21359,'021359','1975-052CD','deb Delta 111','',36499,'In Earth orbit','','1991-06-05 00:00:00',104.7,843,1118,99.61,'Sun-Synch' from dual union all
select 2136,'002136','1966-029A','OPS 1612','CORONA J-30',13436,'Reentered Attached','1966-04-26 00:00:00','1966-04-07 00:00:00',89.69,197,321,75.1,'Intermediate' from dual union all
select 21360,'021360','1975-052CE','deb Delta 111','',36499,'In Earth orbit','','1991-06-01 00:00:00',108.37,1092,1211,99.7,'Sun-Synch' from dual union all
select 21361,'021361','1975-052CF','deb Delta 111','',36499,'In Earth orbit','','1991-06-03 00:00:00',121.25,1070,2404,99.6,'Medium' from dual union all
select 21362,'021362','1975-052CG','deb Delta 111','',36499,'In Earth orbit','','1991-06-04 00:00:00',118.05,1058,2129,99.54,'Sun-Synch' from dual union all
select 21363,'021363','1975-052CH','deb Delta 111','',36499,'In Earth orbit','','1991-06-02 00:00:00',110.02,1095,1361,99.66,'Sun-Synch' from dual union all
select 21364,'021364','1975-052CJ','deb Delta 111','',36499,'In Earth orbit','','1991-06-01 00:00:00',111.48,1094,1495,99.66,'Sun-Synch' from dual union all
select 21365,'021365','1975-052CK','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',111.05,1096,1454,99.66,'Sun-Synch' from dual union all
select 21366,'021366','1975-052CL','deb Delta 111','',36499,'In Earth orbit','','1991-08-29 00:00:00',107.46,1093,1125,99.7,'Sun-Synch' from dual union all
select 21367,'021367','1975-052CM','deb Delta 111','',36499,'In Earth orbit','','1991-09-09 00:00:00',108.37,1095,1207,99.69,'Sun-Synch' from dual union all
select 21368,'021368','1975-052CN','deb Delta 111','',36499,'In Earth orbit','','1991-09-10 00:00:00',106.02,986,1099,99.69,'Sun-Synch' from dual union all
select 21369,'021369','1975-052CP','deb Delta 111','',36499,'In Earth orbit','','1991-06-05 00:00:00',105.39,892,1134,99.7,'Sun-Synch' from dual union all
select 2137,'002137','1966-028D','deb Kosmos-114','',13435,'Reentered','1966-04-10 00:00:00','1966-04-06 00:00:00',89.88,200,337,73,'Intermediate' from dual union all
select 21370,'021370','1975-052CQ','deb Delta 111','',36499,'In Earth orbit','','1991-09-10 00:00:00',108.5,1096,1219,99.76,'Sun-Synch' from dual union all
select 21371,'021371','1975-052CR','deb Delta 111','',36499,'In Earth orbit','','1991-05-29 00:00:00',109.66,1094,1328,99.7,'Sun-Synch' from dual union all
select 21372,'021372','1975-052CS','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',108.14,1093,1189,99.77,'Sun-Synch' from dual union all
select 21373,'021373','1975-052CT','deb Delta 111','',36499,'In Earth orbit','','1991-09-10 00:00:00',108.28,1090,1205,99.77,'Sun-Synch' from dual union all
select 21374,'021374','1975-052CU','deb Delta 111','',36499,'In Earth orbit','','1991-09-09 00:00:00',106.95,869,1301,100.05,'Sun-Synch' from dual union all
select 21375,'021375','1975-052CV','deb Delta 111','',36499,'Reentered','2012-03-01 00:00:00','1991-06-01 00:00:00',109.23,823,1559,99.99,'Sun-Synch' from dual union all
select 21376,'021376','1975-052CW','deb Delta 111','',36499,'Reentered','2002-10-19 00:00:00','1991-06-04 00:00:00',109,778,1582,100.05,'Sun-Synch' from dual union all
select 21377,'021377','1975-052CX','deb Delta 111','',36499,'In Earth orbit','','1991-06-03 00:00:00',109.98,1026,1426,100.11,'Sun-Synch' from dual union all
select 21378,'021378','1975-052CY','deb Delta 111','',36499,'Reentered','2003-01-07 00:00:00','1991-05-31 00:00:00',104.75,764,1202,100,'Sun-Synch' from dual union all
select 21379,'021379','1975-052CZ','deb Delta 111','',36499,'Reentered','1999-10-14 00:00:00','1991-06-05 00:00:00',103.95,724,1166,99.06,'Sun-Synch' from dual union all
select 2138,'002138','1966-028E','deb Kosmos-114','',13435,'Reentered','1966-04-09 00:00:00','1966-04-06 00:00:00',89.94,201,342,72.9,'Intermediate' from dual union all
select 21380,'021380','1975-052DA','deb Delta 111','',36499,'In Earth orbit','','1991-06-04 00:00:00',106.43,1027,1095,99.41,'Sun-Synch' from dual union all
select 21381,'021381','1975-052DB','deb Delta 111','',36499,'Reentered','2014-10-16 00:00:00','1991-06-05 00:00:00',106.76,1060,1094,99.43,'Sun-Synch' from dual union all
select 21382,'021382','1975-052DC','deb Delta 111','',36499,'In Earth orbit','','1991-06-05 00:00:00',115.45,1052,1900,99.56,'Sun-Synch' from dual union all
select 21383,'021383','1975-052DD','deb Delta 111','',36499,'In Earth orbit','','1991-05-30 00:00:00',113.37,1094,1669,99.67,'Sun-Synch' from dual union all
select 21384,'021384','1975-052DE','deb Delta 111','',36499,'In Earth orbit','','1991-05-29 00:00:00',111.81,1074,1546,99.64,'Sun-Synch' from dual union all
select 21385,'021385','1975-052DF','deb Delta 111','',36499,'Reentered','2011-09-30 00:00:00','1991-05-27 00:00:00',96.59,526,665,98.83,'Sun-Synch' from dual union all
select 21386,'021386','1975-052DG','deb Delta 111','',36499,'In Earth orbit','','1991-05-30 00:00:00',109.44,1049,1352,99.56,'Sun-Synch' from dual union all
select 21387,'021387','1975-052DH','deb Delta 111','',36499,'In Earth orbit','','1991-06-05 00:00:00',107.4,1071,1142,99.38,'Sun-Synch' from dual union all
select 21388,'021388','1975-052DJ','deb Delta 111','',36499,'In Earth orbit','','1991-09-08 00:00:00',106.8,1025,1132,99.43,'Sun-Synch' from dual union all
select 21389,'021389','1975-052DK','deb Delta 111','',36499,'Reentered','2000-08-27 00:00:00','1991-06-03 00:00:00',102.84,680,1106,99.55,'Sun-Synch' from dual union all
select 2139,'002139','1966-030A','Surveyor M-2','Surveyor Model 2 (SM-2)',13437,'Reentered','1966-05-05 00:00:00','1966-04-08 00:00:00',89.69,175,343,30.7,'Intermediate' from dual union all
select 21390,'021390','1975-052DL','deb Delta 111','',36499,'Reentered','2005-05-11 00:00:00','1991-06-06 00:00:00',95.83,555,563,98.76,'Sun-Synch' from dual union all
select 21391,'021391','1975-052DM','deb Delta 111','',36499,'Reentered','1991-06-05 00:00:00','1991-06-05 00:00:00',89.91,229,311,100.27,'Retrograde' from dual union all
select 21392,'021392','1991-037A','Satcom C-5','Satcom C-5',64206,'In Earth orbit','','1991-07-03 00:00:00',1435.96,35770,35798,.06,'Stationary' from dual union all
select 21393,'021393','1991-037B','Delta 205','Delta SSPS AJ10-118K',64206,'In Earth orbit','','1992-02-18 00:00:00',112.81,398,2314,24.99,'Equatorial' from dual union all
select 21394,'021394','1991-037C','Star 48B','Star 48B S/N 10045A-18',64206,'In Earth orbit','','1991-09-11 00:00:00',648.6,1460,35425,24.5,'GEO Transfer' from dual union all
select 21395,'021395','1991-038A','Progress M-8','Progress 7K-TGM No. 207',64207,'Deorbited','1991-08-16 00:00:00','1991-06-30 00:00:00',92.4,391,394,51.6,'Intermediate' from dual union all
select 21396,'021396','1991-038B','11S510','11S510',64207,'Reentered','1991-05-31 00:00:00','1991-05-31 00:00:00',88.01,161,190,51.64,'Intermediate' from dual union all
select 21397,'021397','1991-039A','Okean','Okean-O1 No. 6/NKhM No.',64208,'In Earth orbit','','1991-07-04 00:00:00',97.69,632,665,82.53,'Intermediate' from dual union all
select 21398,'021398','1991-039B','S5M','S5M',64208,'In Earth orbit','','1991-08-30 00:00:00',97.68,631,665,82.53,'Intermediate' from dual union all
select 21399,'021399','1991-040A','Columbia (STS-40)','OV-102',64209,'Landed','1991-06-14 00:00:00','1991-06-07 00:00:00',90.25,278,296,39.01,'Intermediate' from dual union all
select 214,'000214','61 A Kappa 2','OSCAR 1','OSCAR 1',7256,'Reentered','1962-01-31 00:00:00','1961-12-24 00:00:00',91.35,235,447,81.21,'Intermediate' from dual union all
select 2140,'002140','1966-005C','NNS O-7 despin weight','Transit high deb',13412,'In Earth orbit','','1966-10-31 00:00:00',107.86,866,1390,89.92,'Polar' from dual union all
select 21400,'021400','1991-009BP','deb S3M','',64178,'In Earth orbit','','1991-09-11 00:00:00',116.5,1377,1670,73.98,'Intermediate' from dual union all
select 21401,'021401','1991-009BQ','deb S3M','',64178,'In Earth orbit','','1991-09-09 00:00:00',118.38,1491,1726,74.13,'Intermediate' from dual union all
select 21402,'021402','1991-009BR','deb S3M','',64178,'In Earth orbit','','1991-09-09 00:00:00',118.25,1492,1713,74.03,'Intermediate' from dual union all
select 21403,'021403','1991-009BS','deb S3M','',64178,'In Earth orbit','','1991-09-10 00:00:00',117.07,1436,1663,73.93,'Intermediate' from dual union all
select 21404,'021404','1991-009BT','deb S3M','',64178,'In Earth orbit','','1991-09-07 00:00:00',116.08,1357,1652,73.97,'Intermediate' from dual union all
select 21405,'021405','1991-009BU','deb S3M','',64178,'In Earth orbit','','1991-09-12 00:00:00',118.21,1417,1784,73.96,'Intermediate' from dual union all
select 21406,'021406','1991-009BV','deb S3M','',64178,'In Earth orbit','','1991-09-11 00:00:00',117.07,1437,1661,73.91,'Intermediate' from dual union all
select 21407,'021407','1991-009BW','deb S3M','',64178,'In Earth orbit','','1991-09-06 00:00:00',116.95,1407,1681,73.86,'Intermediate' from dual union all
select 21408,'021408','1991-009BX','deb S3M','',64178,'In Earth orbit','','1991-09-13 00:00:00',118.52,1450,1778,74.03,'Intermediate' from dual union all
select 21409,'021409','1986-017DJ','Garbage bag','',59210,'Reentered','1991-07-24 00:00:00','1991-07-15 00:00:00',90.24,270,303,51.59,'Intermediate' from dual union all
select 2141,'002141','1966-005D','NNS O-7 part','',13412,'In Earth orbit','','1966-03-06 00:00:00',104.44,844,1093,89.75,'Polar' from dual union all
select 21410,'021410','1986-017DK','Garbage bag','',59210,'Reentered','1991-07-14 00:00:00','1991-06-12 00:00:00',91.9,348,387,51.61,'Intermediate' from dual union all
select 21411,'021411','1986-017DL','Garbage bag','',59210,'Reentered','1991-08-24 00:00:00','1991-08-23 00:00:00',88.04,145,210,51.61,'Intermediate' from dual union all
select 21412,'021412','1986-017DM','Garbage bag','',59210,'Reentered','1991-08-11 00:00:00','1991-08-09 00:00:00',88.98,219,230,51.59,'Intermediate' from dual union all
select 21413,'021413','1986-017DN','Garbage bag','',59210,'Reentered','1991-08-28 00:00:00','1991-08-28 00:00:00',87.83,164,171,51.57,'Intermediate' from dual union all
select 21414,'021414','1986-017DP','Garbage bag','',59210,'Reentered','1991-10-20 00:00:00','1991-09-12 00:00:00',91.01,315,333,51.6,'Intermediate' from dual union all
select 21415,'021415','1986-017DQ','Garbage bag','',59210,'Reentered','1991-09-02 00:00:00','1991-09-01 00:00:00',88.52,198,204,51.58,'Intermediate' from dual union all
select 21416,'021416','1986-017DR','Garbage bag','',59210,'Reentered','1991-08-14 00:00:00','1991-08-13 00:00:00',87.88,42,298,51.64,'ERROR - Unexpected value "TA"' from dual union all
select 21417,'021417','1986-017DS','Garbage bag','',59210,'Reentered','1991-10-30 00:00:00','1991-09-12 00:00:00',91.01,315,333,51.63,'Intermediate' from dual union all
select 21418,'021418','1991-041A','Kosmos-2150','Strela-2M',64210,'In Earth orbit','','1991-09-05 00:00:00',100.76,782,808,74.04,'Intermediate' from dual union all
select 21419,'021419','1991-041B','S3M','S3M',64210,'In Earth orbit','','1991-09-02 00:00:00',100.67,777,804,74.04,'Intermediate' from dual union all
select 2142,'002142','1966-031A','OAO 1','OAO A1',13438,'In Earth orbit','','1967-01-20 00:00:00',100.85,794,804,35.03,'Intermediate' from dual union all
select 21420,'021420','1991-041C','deb Kosmos-2150','Strela-2M debris 2 (high',64210,'In Earth orbit','','1991-07-11 00:00:00',101.01,801,812,74.07,'Intermediate' from dual union all
select 21421,'021421','1986-017DT','Garbage bag','',59210,'Reentered','1991-08-08 00:00:00','1991-08-07 00:00:00',88.18,176,192,51.59,'Intermediate' from dual union all
select 21422,'021422','1991-042A','Kosmos-2151','Tselina-R',64211,'In Earth orbit','','1991-07-14 00:00:00',97.7,635,663,82.52,'Intermediate' from dual union all
select 21423,'021423','1991-042B','S5M','S5M',64211,'In Earth orbit','','1991-07-16 00:00:00',97.68,634,662,82.5,'Intermediate' from dual union all
select 21424,'021424','1986-017DU','Garbage bag','',59210,'Reentered','1991-07-11 00:00:00','1991-06-13 00:00:00',91.27,305,369,51.57,'Intermediate' from dual union all
select 21425,'021425','1986-017DV','MAK-1','MAK 1',59210,'Reentered','1991-10-18 00:00:00','1991-10-18 00:00:00',87.42,143,151,51.58,'Intermediate' from dual union all
select 21426,'021426','1991-043A','Molniya-1','Molniya-1T No. 74',64212,'In Earth orbit','','1991-07-18 00:00:00',717.62,452,39895,62.9,'Molniya' from dual union all
select 21427,'021427','1991-043B','11S510','11S510',64212,'Reentered','1991-07-02 00:00:00','1991-06-24 00:00:00',89.86,198,337,62.83,'Intermediate' from dual;
insert into SATELLITE
select 21428,'021428','1991-043C','BOZ','BOZ',64212,'Reentered','1991-06-27 00:00:00','1991-06-24 00:00:00',89.2,166,304,62.82,'Intermediate' from dual union all
select 21429,'021429','1991-043D','Blok-ML','Blok-ML No. 186',64212,'In Earth orbit','','1991-07-18 00:00:00',732.45,449,40627,62.9,'Molniya' from dual union all
select 2143,'002143','1966-030B','Centaur AC-8','Centaur 3D',13437,'Reentered','1966-04-17 00:00:00','1966-04-08 00:00:00',89.69,175,343,30.7,'Intermediate' from dual union all
select 21430,'021430','1975-052DN','deb Delta 111','',36499,'Reentered','2012-08-25 00:00:00','1991-08-30 00:00:00',121.55,1063,2438,99.62,'Medium' from dual union all
select 21431,'021431','1975-052DP','deb Delta 111','',36499,'Reentered','1991-12-11 00:00:00','1991-09-12 00:00:00',119.24,1248,2046,101.15,'Sun-Synch' from dual union all
select 21432,'021432','1975-052DQ','deb Delta 111','',36499,'Reentered','1992-03-03 00:00:00','1991-09-11 00:00:00',100.21,547,990,99.51,'Sun-Synch' from dual union all
select 21433,'021433','1975-052DR','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',107.06,1047,1134,99.82,'Sun-Synch' from dual union all
select 21434,'021434','1975-052DS','deb Delta 111','',36499,'In Earth orbit','','1991-09-11 00:00:00',118.53,1089,2141,99.44,'Sun-Synch' from dual union all
select 21435,'021435','1975-052DT','deb Delta 111','',36499,'Reentered','2012-11-02 00:00:00','1991-09-11 00:00:00',113.6,1102,1682,100.14,'Sun-Synch' from dual union all
select 21436,'021436','1975-052DU','deb Delta 111','',36499,'Reentered','1991-12-19 00:00:00','1991-09-12 00:00:00',112.66,1229,1469,99.38,'Sun-Synch' from dual union all
select 21437,'021437','1975-052DV','deb Delta 111','',36499,'In Earth orbit','','1991-09-11 00:00:00',107.98,1091,1176,99.74,'Sun-Synch' from dual union all
select 21438,'021438','1975-052DW','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',108.49,1095,1219,99.6,'Sun-Synch' from dual union all
select 21439,'021439','1975-052DX','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',114.16,973,1862,100.06,'Sun-Synch' from dual union all
select 2144,'002144','1966-031B','Agena D 6703','Agena D 6703',13438,'In Earth orbit','','1966-05-26 00:00:00',100.81,785,809,35.03,'Intermediate' from dual union all
select 21440,'021440','1975-052DY','deb Delta 111','',36499,'Reentered','2005-07-31 00:00:00','1991-09-13 00:00:00',111.31,997,1576,100.28,'Sun-Synch' from dual union all
select 21441,'021441','1975-052DZ','deb Delta 111','',36499,'In Earth orbit','','1991-09-13 00:00:00',112.27,1094,1568,99.64,'Sun-Synch' from dual union all
select 21442,'021442','1975-052EA','deb Delta 111','',36499,'In Earth orbit','','1991-09-13 00:00:00',115.09,1103,1816,99.63,'Sun-Synch' from dual union all
select 21443,'021443','1975-052EB','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',108.25,1094,1198,99.71,'Sun-Synch' from dual union all
select 21444,'021444','1975-052EC','deb Delta 111','',36499,'In Earth orbit','','1991-09-11 00:00:00',111.67,1095,1512,99.66,'Sun-Synch' from dual union all
select 21445,'021445','1975-052ED','deb Delta 111','',36499,'In Earth orbit','','1991-09-11 00:00:00',108.47,1094,1218,99.68,'Sun-Synch' from dual union all
select 21446,'021446','1975-052EE','deb Delta 111','',36499,'In Earth orbit','','1991-09-13 00:00:00',113.38,1094,1670,99.66,'Sun-Synch' from dual union all
select 21447,'021447','1975-052EF','deb Delta 111','',36499,'Reentered','1994-05-21 00:00:00','1991-09-11 00:00:00',117.99,1106,2076,100.26,'Sun-Synch' from dual union all
select 21448,'021448','1975-052EG','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',109.3,1095,1294,99.61,'Sun-Synch' from dual union all
select 21449,'021449','1975-052EH','deb Delta 111','',36499,'Reentered','1991-08-28 00:00:00','1991-06-18 00:00:00',109.88,946,1496,100.4,'Sun-Synch' from dual union all
select 2145,'002145','1966-031C','OAO 1 battery debris','',13438,'Reentered','1981-10-31 00:00:00','1966-05-16 00:00:00',100.84,792,805,35.03,'Intermediate' from dual union all
select 21450,'021450','1975-052EJ','deb Delta 111','',36499,'In Earth orbit','','1991-09-13 00:00:00',108.14,1095,1187,99.73,'Sun-Synch' from dual union all
select 21451,'021451','1975-052EK','deb Delta 111','',36499,'Reentered','1991-10-29 00:00:00','1991-09-11 00:00:00',111.47,786,1802,99,'Sun-Synch' from dual union all
select 21452,'021452','1975-052EL','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',118.07,1088,2100,99.76,'Sun-Synch' from dual union all
select 21453,'021453','1975-052EM','deb Delta 111','',36499,'In Earth orbit','','1991-09-13 00:00:00',108.82,1078,1267,99.81,'Sun-Synch' from dual union all
select 21454,'021454','1975-052EN','deb Delta 111','',36499,'Reentered','1992-02-28 00:00:00','1991-06-17 00:00:00',128.43,1462,2646,101.29,'Medium' from dual union all
select 21455,'021455','1975-052EP','deb Delta 111','',36499,'Reentered','1991-08-20 00:00:00','1991-06-18 00:00:00',120.64,792,2627,100.48,'Medium' from dual union all
select 21456,'021456','1975-052EQ','deb Delta 111','',36499,'Reentered','1991-09-30 00:00:00','1991-09-12 00:00:00',106.09,749,1342,101.22,'Sun-Synch' from dual union all
select 21457,'021457','1975-052ER','deb Delta 111','',36499,'In Earth orbit','','1991-09-11 00:00:00',109.69,948,1477,99.93,'Sun-Synch' from dual union all
select 21458,'021458','1975-052ES','deb Delta 111','',36499,'In Earth orbit','','1991-09-10 00:00:00',115.98,1055,1945,99.78,'Sun-Synch' from dual union all
select 21459,'021459','1975-052ET','deb Delta 111','',36499,'In Earth orbit','','1991-09-13 00:00:00',107.67,1079,1158,99.79,'Sun-Synch' from dual union all
select 2146,'002146','1966-032A','OPS 0910','GAMBIT SV 977',13439,'Reentered','1966-04-26 00:00:00','1966-04-20 00:00:00',89.94,145,398,116.95,'Retrograde' from dual union all
select 21460,'021460','1975-052EU','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',108.55,1091,1229,99.61,'Sun-Synch' from dual union all
select 21461,'021461','1975-052EV','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',109.3,1043,1346,99.61,'Sun-Synch' from dual union all
select 21462,'021462','1975-052EW','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',114.99,981,1929,99.74,'Sun-Synch' from dual union all
select 21463,'021463','1975-052EX','deb Delta 111','',36499,'In Earth orbit','','1991-08-30 00:00:00',106.67,952,1193,100.05,'Sun-Synch' from dual union all
select 21464,'021464','1975-052EY','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',112.67,1119,1580,99.97,'Sun-Synch' from dual union all
select 21465,'021465','1975-052EZ','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',107.26,1046,1154,99.65,'Sun-Synch' from dual union all
select 21466,'021466','1975-052FA','deb Delta 111','',36499,'In Earth orbit','','1991-09-10 00:00:00',109.28,1089,1298,99.68,'Sun-Synch' from dual union all
select 21467,'021467','1975-052FB','deb Delta 111','',36499,'In Earth orbit','','1991-09-11 00:00:00',106.98,1067,1107,99.77,'Sun-Synch' from dual union all
select 21468,'021468','1975-052FC','deb Delta 111','',36499,'Reentered','1991-11-12 00:00:00','1991-09-12 00:00:00',112.05,896,1746,100.41,'Sun-Synch' from dual union all
select 21469,'021469','1975-052FD','deb Delta 111','',36499,'In Earth orbit','','1991-09-13 00:00:00',113.38,1093,1671,99.65,'Sun-Synch' from dual union all
select 2147,'002147','1966-033A','Kosmos-115','Zenit-2 No. 35',13440,'Landed','1966-04-28 00:00:00','1966-04-22 00:00:00',89.23,187,285,65.03,'Intermediate' from dual union all
select 21470,'021470','1975-052FE','deb Delta 111','',36499,'In Earth orbit','','1992-02-02 00:00:00',107.04,1014,1166,99.86,'Sun-Synch' from dual union all
select 21471,'021471','1975-052FF','deb Delta 111','',36499,'In Earth orbit','','1992-01-23 00:00:00',106.77,877,1278,100.09,'Sun-Synch' from dual union all
select 21472,'021472','1975-052FG','deb Delta 111','',36499,'In Earth orbit','','1992-01-30 00:00:00',107.08,886,1297,100.04,'Sun-Synch' from dual union all
select 21473,'021473','1975-052FH','deb Delta 111','',36499,'In Earth orbit','','1991-09-13 00:00:00',106.94,883,1287,100.07,'Sun-Synch' from dual union all
select 21474,'021474','1975-052FJ','deb Delta 111','',36499,'Reentered','1991-07-05 00:00:00','1991-06-27 00:00:00',103.92,607,1281,99.95,'Sun-Synch' from dual union all
select 21475,'021475','1975-052FK','deb Delta 111','',36499,'In Earth orbit','','1991-09-05 00:00:00',106.47,995,1131,99.79,'Sun-Synch' from dual union all
select 21476,'021476','1986-017DW','Garbage bag','',59210,'Reentered','1991-09-08 00:00:00','1991-09-08 00:00:00',87.76,163,164,51.58,'Intermediate' from dual union all
select 21477,'021477','1986-017DX','Garbage bag','',59210,'Reentered','1991-08-12 00:00:00','1991-08-11 00:00:00',88.53,191,213,51.59,'Intermediate' from dual union all
select 21478,'021478','1986-017DY','Garbage bag','',59210,'Reentered','1991-08-19 00:00:00','1991-08-18 00:00:00',88.05,152,204,51.58,'Intermediate' from dual union all
select 21479,'021479','1991-035C','KDU','KDU',64204,'Reentered','1991-06-21 00:00:00','1991-06-21 00:00:00',88.15,174,192,82.3,'Intermediate' from dual union all
select 2148,'002148','1966-032B','Agena D 4827','Agena D',13439,'Reentered','1966-04-22 00:00:00','1966-04-20 00:00:00',89,139,312,116.96,'Retrograde' from dual union all
select 21480,'021480','1991-035D','Blenda','Sunshade',64204,'Reentered','1991-06-20 00:00:00','1991-06-20 00:00:00',88.01,168,184,82.29,'Intermediate' from dual union all
select 21481,'021481','1986-017DZ','Garbage bag','',59210,'Reentered','1991-12-16 00:00:00','1991-09-13 00:00:00',91.72,355,362,51.66,'Intermediate' from dual union all
select 21482,'021482','1986-017EA','Garbage bag','',59210,'Reentered','1992-01-07 00:00:00','1992-01-07 00:00:00',87.58,155,155,51.61,'Intermediate' from dual union all
select 21483,'021483','1986-017EB','Garbage bag','',59210,'Reentered','1991-09-13 00:00:00','1991-09-12 00:00:00',88.15,178,188,51.58,'Intermediate' from dual union all
select 21484,'021484','1986-017EC','Garbage bag','',59210,'Reentered','1991-07-12 00:00:00','1991-06-19 00:00:00',91.24,299,372,51.59,'Intermediate' from dual union all
select 21485,'021485','1991-035E','Solnechnaya Batareya','Solar panel module',64204,'Reentered','1991-06-23 00:00:00','1991-06-23 00:00:00',87.99,159,191,82.28,'Intermediate' from dual union all
select 21486,'021486','1986-017ED','Garbage bag','',59210,'Reentered','1991-08-05 00:00:00','1991-08-04 00:00:00',88.51,195,206,51.63,'Intermediate' from dual union all
select 21487,'021487','1986-017EE','EO-9 EVA-1 debris','',59210,'Reentered','1991-07-06 00:00:00','1991-07-06 00:00:00',88.89,218,222,51.58,'Intermediate' from dual union all
select 21488,'021488','1986-017EF','EO-9 EVA-1 debris','',59210,'Reentered','1992-04-11 00:00:00','1991-09-09 00:00:00',92.09,376,378,51.62,'Intermediate' from dual union all
select 21489,'021489','1986-017EG','EO-9 EVA-1 debris','',59210,'Reentered','1991-09-23 00:00:00','1991-09-19 00:00:00',89.51,246,254,51.6,'Intermediate' from dual union all
select 2149,'002149','1966-033B','8A92E','8A92E',13440,'Reentered','1966-04-30 00:00:00','1966-04-22 00:00:00',89.28,215,263,65.03,'Intermediate' from dual union all
select 21490,'021490','1986-017EH','EO-9 EVA-1 debris','',59210,'Reentered','1991-09-15 00:00:00','1991-09-14 00:00:00',88.59,181,229,51.6,'Intermediate' from dual union all
select 21491,'021491','1975-052FL','deb Delta 111','',36499,'Reentered','2001-11-10 00:00:00','1991-09-09 00:00:00',104.99,851,1136,99.02,'Sun-Synch' from dual union all
select 21492,'021492','1975-052FM','deb Delta 111','',36499,'In Earth orbit','','1991-08-30 00:00:00',108.12,1094,1186,99.68,'Sun-Synch' from dual union all
select 21493,'021493','1975-052FN','deb Delta 111','',36499,'In Earth orbit','','1991-09-04 00:00:00',120.7,1099,2326,99.7,'Medium' from dual union all
select 21494,'021494','1975-052FP','deb Delta 111','',36499,'Reentered','1991-08-05 00:00:00','1991-08-04 00:00:00',98.81,491,912,101.39,'Sun-Synch' from dual union all
select 21495,'021495','1975-052FQ','deb Delta 111','',36499,'In Earth orbit','','1991-08-15 00:00:00',106.1,500,1591,100.69,'Sun-Synch' from dual union all
select 21496,'021496','1975-052FR','deb Delta 111','',36499,'In Earth orbit','','1991-09-09 00:00:00',112.49,1074,1607,99.79,'Sun-Synch' from dual union all
select 21497,'021497','1975-052FS','deb Delta 111','',36499,'In Earth orbit','','1991-09-09 00:00:00',108.1,1094,1183,99.74,'Sun-Synch' from dual union all
select 21498,'021498','1975-052FT','deb Delta 111','',36499,'In Earth orbit','','1991-09-13 00:00:00',111.69,1095,1514,99.65,'Sun-Synch' from dual union all
select 21499,'021499','1975-052FU','deb Delta 111','',36499,'In Earth orbit','','1991-09-09 00:00:00',110.34,1097,1388,99.68,'Sun-Synch' from dual union all
select 215,'000215','61 A Kappa 3','Discoverer XXXVI inst cover','',7256,'Reentered','1961-12-19 00:00:00','1961-12-14 00:00:00',91.22,235,434,81.21,'Intermediate' from dual union all
select 2150,'002150','1966-034A','OV3-1','OV3-1',13441,'In Earth orbit','','1966-05-24 00:00:00',151.68,350,5736,82.45,'Medium' from dual union all
select 21500,'021500','1975-052FV','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',103.3,715,1115,100.06,'Sun-Synch' from dual union all
select 21501,'021501','1975-052FW','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',115.74,1049,1929,99.74,'Sun-Synch' from dual union all
select 21502,'021502','1975-052FX','deb Delta 111','',36499,'Reentered','1991-11-12 00:00:00','1991-09-12 00:00:00',110.61,921,1589,100.3,'Sun-Synch' from dual union all
select 21503,'021503','1975-052FY','deb Delta 111','',36499,'Reentered','1992-08-04 00:00:00','1991-09-12 00:00:00',105.61,855,1191,100.04,'Sun-Synch' from dual union all
select 21504,'021504','1975-052FZ','deb Delta 111','',36499,'In Earth orbit','','1991-09-06 00:00:00',115.65,1046,1923,99.79,'Sun-Synch' from dual union all
select 21505,'021505','1975-052GA','deb Delta 111','',36499,'Reentered','1991-10-24 00:00:00','1991-09-12 00:00:00',108.06,1039,1235,100.89,'Sun-Synch' from dual union all
select 21506,'021506','1975-052GB','deb Delta 111','',36499,'In Earth orbit','','1991-09-13 00:00:00',110.23,1095,1380,99.6,'Sun-Synch' from dual union all
select 21507,'021507','1975-052GC','deb Delta 111','',36499,'Reentered','1991-08-10 00:00:00','1991-08-25 00:00:00',107.34,741,1467,99.52,'Sun-Synch' from dual union all
select 21508,'021508','1975-052GD','deb Delta 111','',36499,'Reentered','1991-09-03 00:00:00','1991-08-28 00:00:00',103.21,766,1055,99.81,'Sun-Synch' from dual union all
select 21509,'021509','1975-052GE','deb Delta 111','',36499,'Reentered','1991-08-29 00:00:00','1991-08-29 00:00:00',98.12,554,784,99.25,'Sun-Synch' from dual union all
select 2151,'002151','1966-035A','Molniya-1','Molniya-1 11F67 No. 6',13442,'Reentered','1973-06-11 00:00:00','1966-06-05 00:00:00',709.77,404,39555,64.82,'GEO Transfer' from dual union all
select 21510,'021510','1975-052GF','deb Delta 111','',36499,'Reentered','1991-10-31 00:00:00','1991-09-08 00:00:00',108.77,1098,1242,99.03,'Sun-Synch' from dual union all
select 21511,'021511','1975-052GG','deb Delta 111','',36499,'Reentered','1991-07-10 00:00:00','1991-07-10 00:00:00',97.2,552,698,100.1,'Sun-Synch' from dual union all
select 21512,'021512','1975-052GH','deb Delta 111','',36499,'In Earth orbit','','1991-09-11 00:00:00',107.76,1095,1152,99.73,'Sun-Synch' from dual union all
select 21513,'021513','1975-052GJ','deb Delta 111','',36499,'Reentered','1992-01-04 00:00:00','1991-09-12 00:00:00',118.88,1371,1891,99.9,'Sun-Synch' from dual union all
select 21514,'021514','1975-052GK','deb Delta 111','',36499,'In Earth orbit','','1991-07-19 00:00:00',111.25,752,1817,100.12,'Sun-Synch' from dual union all
select 21515,'021515','1975-052GL','deb Delta 111','',36499,'In Earth orbit','','1991-09-11 00:00:00',111.2,1068,1496,99.68,'Sun-Synch' from dual union all
select 21516,'021516','1975-052GM','deb Delta 111','',36499,'In Earth orbit','','1991-09-08 00:00:00',108.31,1095,1203,99.72,'Sun-Synch' from dual union all
select 21517,'021517','1975-052GN','deb Delta 111','',36499,'Reentered','1991-09-29 00:00:00','1991-07-29 00:00:00',120.3,811,2578,100.64,'Medium' from dual;
insert into SATELLITE
select 21518,'021518','1975-052GP','deb Delta 111','',36499,'Reentered','1991-10-29 00:00:00','1991-09-03 00:00:00',108.69,1075,1257,99.98,'Sun-Synch' from dual union all
select 21519,'021519','1975-052GQ','deb Delta 111','',36499,'Reentered','1991-11-13 00:00:00','1991-09-02 00:00:00',109.95,978,1471,100.3,'Sun-Synch' from dual union all
select 2152,'002152','1966-036A','Kosmos-116','DS-P1-Yu No. 6',13443,'Reentered','1966-12-03 00:00:00','1966-05-26 00:00:00',91.87,287,446,48.37,'Intermediate' from dual union all
select 21520,'021520','1975-052GR','deb Delta 111','',36499,'Reentered','1991-11-21 00:00:00','1991-09-10 00:00:00',114.69,1207,1676,99.19,'Sun-Synch' from dual union all
select 21521,'021521','1975-052GS','deb Delta 111','',36499,'Reentered','1999-06-19 00:00:00','1991-09-10 00:00:00',114.45,1065,1796,100.66,'Sun-Synch' from dual union all
select 21522,'021522','1975-052GT','deb Delta 111','',36499,'Reentered','1991-11-10 00:00:00','1991-09-02 00:00:00',111.2,1065,1498,99.45,'Sun-Synch' from dual union all
select 21523,'021523','1975-052GU','deb Delta 111','',36499,'In Earth orbit','','1991-09-09 00:00:00',124.73,1197,2586,99.87,'Medium' from dual union all
select 21524,'021524','1991-044A','Resurs-F','Resurs-F1 14F43 No. 52',64213,'Landed','1991-07-21 00:00:00','1991-07-05 00:00:00',89.75,255,270,82.34,'Intermediate' from dual union all
select 21525,'021525','1991-044B','11S510','11S510',64213,'Reentered','1991-06-30 00:00:00','1991-06-29 00:00:00',87.24,131,145,82.33,'Intermediate' from dual union all
select 21526,'021526','1986-017EJ','EO-9 EVA-2 debris','',59210,'Reentered','1992-04-05 00:00:00','1991-09-08 00:00:00',92.1,377,378,51.64,'Intermediate' from dual union all
select 21527,'021527','1991-045A','REX','REX',64214,'In Earth orbit','','1991-07-29 00:00:00',101.35,771,874,89.58,'Polar' from dual union all
select 21528,'021528','1991-045B','Star 20 E-54','Star 20 E-54',64214,'In Earth orbit','','1991-07-29 00:00:00',101.26,773,864,89.61,'Polar' from dual union all
select 21529,'021529','1991-045C','REX despin weight','',64214,'In Earth orbit','','1992-02-20 00:00:00',101.32,771,871,89.58,'Polar' from dual union all
select 2153,'002153','1965-098J','DME-A adapter sep clamp','ISIS-X part',11848,'In Earth orbit','','1966-03-09 00:00:00',121.3,496,2981,79.81,'Medium' from dual union all
select 21530,'021530','1986-017EK','EO-9 EVA-2 debris','',59210,'Reentered','1991-09-01 00:00:00','1991-09-01 00:00:00',87.81,164,169,51.56,'Intermediate' from dual union all
select 21531,'021531','1986-017EL','EO-9 EVA-2 debris','',59210,'Reentered','1991-09-09 00:00:00','1991-09-09 00:00:00',88.43,193,201,51.58,'Intermediate' from dual union all
select 21532,'021532','1991-045D','REX despin weight','',64214,'In Earth orbit','','1992-02-19 00:00:00',101.3,765,876,89.55,'Polar' from dual union all
select 21533,'021533','1991-046A','Gorizont','Gorizont No. 34L',64215,'In Earth orbit','','1991-08-02 00:00:00',1436.04,35687,35884,1.44,'Stationary' from dual union all
select 21534,'021534','1991-046B','8S812','8S812',64215,'Reentered','1991-07-04 00:00:00','1991-07-03 00:00:00',87.94,168,177,51.62,'Intermediate' from dual union all
select 21535,'021535','1991-046C','Perekhodnik','Sredniy perekhodnik',64215,'Reentered','1991-07-03 00:00:00','1991-07-02 00:00:00',87.91,165,178,51.61,'Intermediate' from dual union all
select 21536,'021536','1991-046D','Blok DM2 No. 50L','11S861 No. 50L',64215,'In Earth orbit','','1991-08-01 00:00:00',1426.77,35550,35658,1.47,'Drift GEO' from dual union all
select 21537,'021537','1989-100AF','Kosmos-2053 SS 27','ESO',62377,'Reentered','1991-08-14 00:00:00','1991-08-14 00:00:00',88.83,205,229,73.5,'Intermediate' from dual union all
select 21538,'021538','1991-046E','SOZ','SOZ',64215,'Reentered','2000-08-22 00:00:00','1992-02-19 00:00:00',595.93,167,33993,46.63,'Highly Ellip' from dual union all
select 21539,'021539','1975-052GV','deb Delta 111','',36499,'Reentered','1991-08-31 00:00:00','1991-08-31 00:00:00',94.25,447,518,100.94,'Retrograde' from dual union all
select 2154,'002154','1966-008E','Altair 2 yo weight','',13415,'In Earth orbit','','1966-05-02 00:00:00',100.28,714,830,97.81,'Sun-Synch' from dual union all
select 21540,'021540','1989-100AG','Kosmos-2053 SS 28','ESO',62377,'Reentered','1991-08-13 00:00:00','1991-08-13 00:00:00',89.28,227,250,73.49,'Intermediate' from dual union all
select 21541,'021541','1991-046F','SOZ','SOZ',64215,'Reentered','1992-02-26 00:00:00','1991-08-04 00:00:00',629.22,269,35622,47.25,'GEO Transfer' from dual union all
select 21542,'021542','1975-052GW','deb Delta 111','',36499,'In Earth orbit','','1991-09-13 00:00:00',107.97,1094,1172,99.74,'Sun-Synch' from dual union all
select 21543,'021543','1975-052GX','deb Delta 111','',36499,'In Earth orbit','','1991-09-10 00:00:00',110.39,1101,1388,99.73,'Sun-Synch' from dual union all
select 21544,'021544','1975-052GY','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',105.58,786,1258,100.04,'Sun-Synch' from dual union all
select 21545,'021545','1975-052GZ','deb Delta 111','',36499,'In Earth orbit','','1991-09-13 00:00:00',109.19,1011,1368,99.63,'Sun-Synch' from dual union all
select 21546,'021546','1975-052HA','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',118.14,1090,2105,99.53,'Sun-Synch' from dual union all
select 21547,'021547','1975-052HB','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',107.11,885,1301,100.07,'Sun-Synch' from dual union all
select 21548,'021548','1975-052HC','deb Delta 111','',36499,'In Earth orbit','','1991-09-13 00:00:00',111.32,1096,1479,99.74,'Sun-Synch' from dual union all
select 21549,'021549','1975-052HD','deb Delta 111','',36499,'In Earth orbit','','1991-09-11 00:00:00',110.65,1097,1417,99.8,'Sun-Synch' from dual union all
select 2155,'002155','1966-035B','BOZ','BOZ',13442,'Reentered','1966-05-09 00:00:00','1966-04-27 00:00:00',91.01,175,474,64.86,'Intermediate' from dual union all
select 21550,'021550','1975-052HE','deb Delta 111','',36499,'In Earth orbit','','1991-09-06 00:00:00',116.02,1063,1941,99.83,'Sun-Synch' from dual union all
select 21551,'021551','1975-052HF','deb Delta 111','',36499,'In Earth orbit','','1991-08-30 00:00:00',108.93,1096,1259,99.68,'Sun-Synch' from dual union all
select 21552,'021552','1991-047A','GPS IIA-2','Navstar GPS SVN 24',64216,'In Earth orbit','','1991-08-14 00:00:00',717.86,20085,20274,55.18,'Medium' from dual union all
select 21553,'021553','1991-047B','LOSAT-X','LOSAT-X',64216,'Reentered','1991-11-15 00:00:00','1991-08-04 00:00:00',92.43,387,401,39.97,'Intermediate' from dual union all
select 21554,'021554','1991-047C','Delta 206','Delta SSPS AJ10-118K',64216,'Reentered','1991-10-29 00:00:00','1991-08-04 00:00:00',92.23,381,387,37.48,'Intermediate' from dual union all
select 21555,'021555','1991-047D','Star 48B','Star 48B S/N 10065-18',64216,'Reentered','1995-09-11 00:00:00','1991-08-04 00:00:00',350.45,187,19988,34.57,'Highly Ellip' from dual union all
select 21556,'021556','1986-017EM','Garbage bag','',59210,'Reentered','1991-09-20 00:00:00','1991-09-19 00:00:00',88.5,195,206,51.59,'Intermediate' from dual union all
select 21557,'021557','1986-017EN','Garbage bag','',59210,'Reentered','1991-08-14 00:00:00','1991-08-13 00:00:00',88.32,178,204,51.55,'Intermediate' from dual union all
select 21558,'021558','1991-048A','Kosmos-2152','Oblik No. 819',64217,'Landed','1991-07-23 00:00:00','1991-07-11 00:00:00',90.35,235,349,82.34,'Intermediate' from dual union all
select 21559,'021559','1991-048B','11S510','11S510',64217,'Reentered','1991-07-10 00:00:00','1991-07-09 00:00:00',88.44,173,222,82.35,'Intermediate' from dual union all
select 2156,'002156','1966-035C','11S510','11S510',13442,'Reentered','1966-05-08 00:00:00','1966-04-28 00:00:00',90.8,184,444,64.9,'Intermediate' from dual union all
select 21560,'021560','1991-049A','Kosmos-2153','Neman',64218,'Deorbited','1992-03-13 00:00:00','1991-08-09 00:00:00',89.69,235,283,64.87,'Intermediate' from dual union all
select 21561,'021561','1991-049B','11S510','11S510',64218,'Reentered','1991-07-13 00:00:00','1991-07-13 00:00:00',87.6,144,168,64.87,'Intermediate' from dual union all
select 21562,'021562','1975-052HG','deb Delta 111','',36499,'Reentered','1991-08-07 00:00:00','1991-07-26 00:00:00',116.3,520,2509,100.92,'Sun-Synch' from dual union all
select 21563,'021563','1975-052HH','deb Delta 111','',36499,'Reentered','1991-10-09 00:00:00','1991-10-03 00:00:00',102.25,627,1104,98.99,'Sun-Synch' from dual union all
select 21564,'021564','1975-052HJ','deb Delta 111','',36499,'Reentered','1991-10-03 00:00:00','1991-09-12 00:00:00',112.58,824,1867,100.51,'Sun-Synch' from dual union all
select 21565,'021565','1975-052HK','deb Delta 111','',36499,'In Earth orbit','','1991-09-05 00:00:00',113.48,1091,1681,99.84,'Sun-Synch' from dual union all
select 21566,'021566','1975-052HL','deb Delta 111','',36499,'In Earth orbit','','1991-08-30 00:00:00',112.43,1086,1591,99.59,'Sun-Synch' from dual union all
select 21567,'021567','1975-052HM','deb Delta 111','',36499,'In Earth orbit','','1991-08-29 00:00:00',106.88,881,1283,100.08,'Sun-Synch' from dual union all
select 21568,'021568','1975-052HN','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',107.12,884,1303,100.08,'Sun-Synch' from dual union all
select 21569,'021569','1975-052HP','deb Delta 111','',36499,'Reentered','1991-11-16 00:00:00','1991-09-10 00:00:00',110.89,1089,1447,99.03,'Sun-Synch' from dual union all
select 2157,'002157','1966-035D','deb Blok-I','',13442,'Reentered','1966-04-29 00:00:00','1966-04-26 00:00:00',90.49,176,422,65.05,'Intermediate' from dual union all
select 21570,'021570','1975-052HQ','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',121.29,1057,2420,99.52,'Medium' from dual union all
select 21571,'021571','1986-017EP','EO-9 EVA-3 debris (Sofora)','',59210,'Reentered','1991-09-11 00:00:00','1991-09-11 00:00:00',88.39,192,198,51.58,'Intermediate' from dual union all
select 21572,'021572','1986-017EQ','EO-9 EVA-3 debris (Sofora)','',59210,'Reentered','1991-08-27 00:00:00','1991-08-26 00:00:00',88.58,200,208,51.59,'Intermediate' from dual union all
select 21573,'021573','1986-017ER','EO-9 EVA-3 debris (Sofora)','',59210,'Reentered','1991-09-06 00:00:00','1991-09-04 00:00:00',89.72,258,263,51.6,'Intermediate' from dual union all
select 21574,'021574','1991-050A','ERS-1','ERS-1',64219,'In Earth orbit','','1991-08-16 00:00:00',100.38,776,778,98.52,'Sun-Synch' from dual union all
select 21575,'021575','1991-050B','UoSAT-OSCAR-22','UoSAT 5',64219,'In Earth orbit','','1991-08-18 00:00:00',100.29,768,777,98.54,'Sun-Synch' from dual union all
select 21576,'021576','1991-050C','Orbcomm-X','Orbcomm-X',64219,'In Earth orbit','','1992-02-15 00:00:00',100.25,766,774,98.52,'Sun-Synch' from dual union all
select 21577,'021577','1991-050D','Tubsat','Tubsat',64219,'In Earth orbit','','1991-08-25 00:00:00',100.26,765,777,98.54,'Sun-Synch' from dual union all
select 21578,'021578','1991-050E','SARA','SARA',64219,'In Earth orbit','','1991-08-16 00:00:00',100.31,770,777,98.54,'Sun-Synch' from dual union all
select 21579,'021579','1986-017ES','EO-9 EVA-3 debris (Sofora)','',59210,'Reentered','1991-08-29 00:00:00','1991-07-17 00:00:00',92.31,385,391,51.6,'Intermediate' from dual union all
select 2158,'002158','1965-082KC','deb Transtage 4','',11832,'Reentered','1966-10-27 00:00:00','1966-04-23 00:00:00',97.77,563,741,31.71,'Intermediate' from dual union all
select 21580,'021580','1991-051A','Microsat 1','Microsat 1',64220,'Reentered','1992-01-23 00:00:00','1991-08-16 00:00:00',92.6,356,448,82.02,'Intermediate' from dual union all
select 21581,'021581','1991-051B','Microsat 2','Microsat 2',64220,'Reentered','1992-01-23 00:00:00','1991-09-03 00:00:00',92.34,348,431,82.03,'Intermediate' from dual union all
select 21582,'021582','1991-051C','Microsat 3','Microsat 3',64220,'Reentered','1992-01-24 00:00:00','1991-09-03 00:00:00',92.32,348,429,82.02,'Intermediate' from dual union all
select 21583,'021583','1991-051D','Microsat 4','Microsat 4',64220,'Reentered','1992-01-23 00:00:00','1991-09-03 00:00:00',92.33,349,429,82.02,'Intermediate' from dual union all
select 21584,'021584','1991-051E','Microsat 5','Microsat 5',64220,'Reentered','1992-01-24 00:00:00','1991-09-01 00:00:00',92.39,350,434,82.02,'Intermediate' from dual union all
select 21585,'021585','1991-051F','Microsat 6','Microsat 6',64220,'Reentered','1992-01-25 00:00:00','1991-09-03 00:00:00',92.31,348,428,82.02,'Intermediate' from dual union all
select 21586,'021586','1991-051G','Microsat 7','Microsat 7',64220,'Reentered','1992-01-23 00:00:00','1991-09-03 00:00:00',92.3,348,427,82.02,'Intermediate' from dual union all
select 21587,'021587','1991-051H','HAPS','HAPS',64220,'Reentered','1991-11-08 00:00:00','1991-09-04 00:00:00',92.36,344,437,82.03,'Intermediate' from dual union all
select 21588,'021588','1990-065D','G-13B Canister','Large Ba Canister',63307,'In Earth orbit','','1992-01-27 00:00:00',589.01,336,33460,17.97,'Highly Ellip' from dual union all
select 21589,'021589','1990-065E','G-14A Canister','Large Ba Canister',63307,'In Earth orbit','','1992-02-22 00:00:00',581.93,288,33134,17.7,'Highly Ellip' from dual union all
select 2159,'002159','1966-036B','S1M','S1M',13443,'Reentered','1966-08-15 00:00:00','1966-05-28 00:00:00',91.47,282,412,48.37,'Intermediate' from dual union all
select 21590,'021590','1990-065F','G-14B Canister','Large Ca Canister',63307,'In Earth orbit','','1992-02-22 00:00:00',584.44,303,33252,19.96,'Highly Ellip' from dual union all
select 21591,'021591','1990-065G','G-5A Canister','Li Canister',63307,'In Earth orbit','','1992-01-29 00:00:00',598.33,577,33709,18.21,'Highly Ellip' from dual union all
select 21592,'021592','1990-065H','G-7A Canister (LI)','[UNID139] RCS 0.30',63307,'In Earth orbit','','1992-01-29 00:00:00',558.7,485,31700,17.75,'Highly Ellip' from dual union all
select 21593,'021593','1990-065J','G-2 Canister','Ba Canister',63307,'In Earth orbit','','1992-01-16 00:00:00',571.84,239,32648,17.48,'Highly Ellip' from dual union all
select 21594,'021594','1990-065K','G-10A Canister','Large Ba Canister',63307,'In Earth orbit','','1992-01-28 00:00:00',593.17,443,33572,16.6,'Highly Ellip' from dual union all
select 21595,'021595','1986-017ET','EO-9 EVA-4 debris (Sofora)','',59210,'Reentered','1991-09-07 00:00:00','1991-09-05 00:00:00',89.18,231,237,51.59,'Intermediate' from dual union all
select 21596,'021596','1986-017EU','EO-9 EVA-4 debris (Sofora)','',59210,'Reentered','1991-08-31 00:00:00','1991-08-31 00:00:00',87.66,157,161,51.57,'Intermediate' from dual union all
select 21597,'021597','1975-052HR','deb Delta 111','',36499,'Reentered','1993-03-16 00:00:00','1991-09-09 00:00:00',122.5,1156,2429,101.11,'Medium' from dual union all
select 21598,'021598','1975-052HS','deb Delta 111','',36499,'In Earth orbit','','1991-09-12 00:00:00',112.45,1063,1616,99.58,'Sun-Synch' from dual union all
select 21599,'021599','1975-052HT','deb Delta 111','',36499,'In Earth orbit','','1991-09-11 00:00:00',114.78,1074,1817,99.84,'Sun-Synch' from dual union all
select 216,'000216','61 Omicron 56','deb r/b Ablestar','',7273,'Reentered','1989-11-13 00:00:00','1961-12-18 00:00:00',101.68,669,1007,66.88,'Intermediate' from dual union all
select 2160,'002160','1966-035E','deb Blok-I','',13442,'Reentered','1966-04-29 00:00:00','1966-04-26 00:00:00',89.94,153,390,64.89,'Intermediate' from dual union all
select 21600,'021600','1975-052HU','deb Delta 111','',36499,'Reentered','1991-11-10 00:00:00','1991-09-13 00:00:00',111.66,949,1657,101.3,'Sun-Synch' from dual union all
select 21601,'021601','1975-052HV','deb Delta 111','',36499,'In Earth orbit','','1991-09-11 00:00:00',115.56,1045,1917,99.81,'Sun-Synch' from dual union all
select 21602,'021602','1975-052HW','deb Delta 111','',36499,'In Earth orbit','','1991-09-13 00:00:00',108.7,1095,1239,99.84,'Sun-Synch' from dual union all
select 21603,'021603','1986-017EV','EO-9 EVA-4 debris','',59210,'Reentered','1991-10-05 00:00:00','1991-10-03 00:00:00',88.62,201,212,51.58,'Intermediate' from dual union all
select 21604,'021604','1991-044C','Blenda','Sunshade',64213,'Reentered','1991-07-21 00:00:00','1991-07-21 00:00:00',89.17,228,238,82.33,'Intermediate' from dual union all
select 21605,'021605','1991-044D','KDU','KDU',64213,'Reentered','1991-08-10 00:00:00','1991-07-24 00:00:00',89.53,243,260,82.33,'Intermediate' from dual union all
select 21606,'021606','1991-044E','KDU part','KDU part',64213,'Reentered','1991-07-23 00:00:00','1991-07-23 00:00:00',88.67,200,218,82.33,'Intermediate' from dual union all
select 21607,'021607','1991-044F','KDU part','KDU part',64213,'Reentered','1991-07-25 00:00:00','1991-07-24 00:00:00',88.53,193,211,82.32,'Intermediate' from dual;
insert into SATELLITE
select 21608,'021608','1991-044G','KDU part','KDU part',64213,'Reentered','1991-07-28 00:00:00','1991-07-25 00:00:00',90.65,234,379,82.33,'Intermediate' from dual union all
select 21609,'021609','1991-044H','KDU part','KDU part',64213,'Reentered','1991-07-23 00:00:00','1991-07-22 00:00:00',90.61,232,376,82.32,'Intermediate' from dual union all
select 2161,'002161','1966-013G','deb Diapason','',13420,'Reentered','2015-08-12 00:00:00','1966-04-26 00:00:00',119.51,518,2800,34.12,'Intermediate' from dual union all
select 21610,'021610','1991-050F','Ariane H10 V44','Ariane H10 S/N T415',64219,'In Earth orbit','','1991-08-17 00:00:00',100.43,776,782,98.46,'Sun-Synch' from dual union all
select 21611,'021611','1991-052A','Resurs-F','Resurs-F1 14F43 No. 53',64221,'Landed','1991-08-08 00:00:00','1991-07-30 00:00:00',89.69,254,265,82.33,'Intermediate' from dual union all
select 21612,'021612','1991-052B','11S510','11S510',64221,'Reentered','1991-07-25 00:00:00','1991-07-24 00:00:00',88.27,173,204,82.33,'Intermediate' from dual union all
select 21613,'021613','1991-048C','Blenda','Sunshade',64217,'Reentered','1991-07-27 00:00:00','1991-07-25 00:00:00',90.37,229,357,82.34,'Intermediate' from dual union all
select 21614,'021614','1991-048D','KDU','KDU',64217,'Reentered','1991-07-31 00:00:00','1991-07-29 00:00:00',88.38,192,197,82.34,'Intermediate' from dual union all
select 21615,'021615','1991-048E','KDU part','KDU part',64217,'Reentered','1991-07-25 00:00:00','1991-07-25 00:00:00',89.68,205,313,82.33,'Intermediate' from dual union all
select 21616,'021616','1991-048F','KDU part','KDU part',64217,'Reentered','1991-07-24 00:00:00','1991-07-24 00:00:00',88.39,180,210,82.35,'Intermediate' from dual union all
select 21617,'021617','1991-048G','KDU part','KDU part',64217,'Reentered','1991-07-26 00:00:00','1991-07-24 00:00:00',88.87,213,225,82.35,'Intermediate' from dual union all
select 21618,'021618','1986-017EW','EO-9 EVA-5 debris (Sofora)','RCS 0.3',59210,'Reentered','1991-08-15 00:00:00','1991-07-29 00:00:00',91.93,362,377,51.6,'Intermediate' from dual union all
select 21619,'021619','1986-017EX','EO-9 EVA-5 debris (Sofora)','RCS 0.3 Long lived',59210,'Reentered','1991-12-18 00:00:00','1991-09-12 00:00:00',91.97,369,374,51.6,'Intermediate' from dual union all
select 2162,'002162','1966-026C','DAPP despin weight','',13433,'Reentered','1967-02-11 00:00:00','1966-05-03 00:00:00',99.92,656,853,98.52,'Sun-Synch' from dual union all
select 21620,'021620','1987-100G','deb SOZ','[UNID140] RCS 1.2',60190,'Reentered','1994-10-08 00:00:00','1992-01-20 00:00:00',250.59,172,13405,46.52,'Highly Ellip' from dual union all
select 21621,'021621','1964-006AE','deb Unknown                       ','[UNID141] RCS 0.3',10292,'Reentered','1991-08-06 00:00:00','1991-07-24 00:00:00',108.64,299,2029,60.46,'Intermediate' from dual union all
select 21622,'021622','1987-036K','deb Unknown','[UNID142] RCS 0.02',60126,'Reentered','2015-04-04 00:00:00','1991-09-10 00:00:00',145.08,1517,4018,65.01,'Medium' from dual union all
select 21623,'021623','1987-036L','deb Unknown (Oko)','[UNID143] RCS 0.02',60126,'In Earth orbit','','1992-01-18 00:00:00',216.72,470,10677,62.86,'Medium' from dual union all
select 21624,'021624','1986-017EY','EO-9 EVA-6 debris (Sofora)','RCS 0.4',59210,'Reentered','1991-08-19 00:00:00','1991-08-19 00:00:00',88.6,199,212,51.57,'Intermediate' from dual union all
select 21625,'021625','1986-017EZ','Orlan-DMA No. 10','RCS 0.5 long lived',59210,'Reentered','1991-12-06 00:00:00','1991-09-09 00:00:00',91.87,362,370,51.62,'Intermediate' from dual union all
select 21626,'021626','1986-017FA','EO-9 EVA-6 debris (Sofora)','RCS 0.5',59210,'Reentered','1991-10-03 00:00:00','1991-09-26 00:00:00',90.27,282,294,51.59,'Intermediate' from dual union all
select 21627,'021627','1986-017FB','EO-9 EVA-6 debris (Sofora)','RCS 0.5 long lived',59210,'Reentered','1991-12-03 00:00:00','1991-09-12 00:00:00',91.84,364,366,51.6,'Intermediate' from dual union all
select 21628,'021628','1986-017FC','EO-9 EVA-6 debris (Sofora)','RCS 0.5',59210,'Reentered','1991-10-07 00:00:00','1991-10-02 00:00:00',89.74,257,267,51.59,'Intermediate' from dual union all
select 21629,'021629','1986-017FD','EO-9 EVA-6 debris (Sofora)','RCS 0.2',59210,'Reentered','1991-09-30 00:00:00','1991-09-26 00:00:00',89.73,254,269,51.59,'Intermediate' from dual union all
select 2163,'002163','1966-037A','Kosmos-117','Zenit-2 No. 39',13444,'Landed','1966-05-14 00:00:00','1966-05-08 00:00:00',89.5,204,296,64.95,'Intermediate' from dual union all
select 21630,'021630','1991-053A','Molniya-1','Molniya-1T No. 76',64222,'Reentered','2004-10-09 00:00:00','1991-08-31 00:00:00',717.8,618,39738,62.89,'Molniya' from dual union all
select 21631,'021631','1991-053B','11S510','11S510',64222,'Reentered','1991-09-02 00:00:00','1991-08-12 00:00:00',92.28,208,565,62.82,'Intermediate' from dual union all
select 21632,'021632','1991-053C','BOZ','BOZ',64222,'Reentered','1991-09-03 00:00:00','1991-08-10 00:00:00',92.3,206,569,62.81,'Intermediate' from dual union all
select 21633,'021633','1991-053D','Blok-ML','Blok-ML No. 76745-480',64222,'Reentered','2005-03-16 00:00:00','1992-01-21 00:00:00',733.26,553,40563,63.24,'Molniya' from dual union all
select 21634,'021634','1990-065L','G-6A Canister','Li Canister',63307,'In Earth orbit','','1992-01-19 00:00:00',588.67,292,33486,17.56,'Highly Ellip' from dual union all
select 21635,'021635','1990-065M','G-8A Canister','Large Ba Canister',63307,'In Earth orbit','','1992-01-12 00:00:00',588.21,285,33469,17.74,'Highly Ellip' from dual union all
select 21636,'021636','1990-065N','G-8B Canister','Large Ba Canister',63307,'In Earth orbit','','1992-01-19 00:00:00',622.34,403,35133,17.4,'GEO Transfer' from dual union all
select 21637,'021637','1990-065P','G-9A Canister','Large Ba Canister',63307,'In Earth orbit','','1992-01-16 00:00:00',564.04,289,32182,17.69,'Highly Ellip' from dual union all
select 21638,'021638','1991-054A','Atlantis (STS-43)','OV-104',64223,'Landed','1991-08-11 00:00:00','1991-08-04 00:00:00',90.82,299,331,28.47,'Intermediate' from dual union all
select 21639,'021639','1991-054B','TDRS 5','TDRS-E',64223,'In Earth orbit','','1991-09-06 00:00:00',1435.91,35779,35787,.1,'Stationary' from dual union all
select 2164,'002164','1966-037B','8A92E','8A92E',13444,'Reentered','1966-05-21 00:00:00','1966-05-13 00:00:00',89.2,209,261,64.95,'Intermediate' from dual union all
select 21640,'021640','1991-054C','IUS-15 SRM-1','Orbus 21',64223,'In Earth orbit','','1991-09-01 00:00:00',632.69,287,35783,26.98,'GEO Transfer' from dual union all
select 21641,'021641','1991-054D','IUS-15 SRM-2','Orbus 6E',64223,'In Earth orbit','','1991-09-02 00:00:00',1436.37,35645,35939,1.39,'Stationary' from dual union all
select 21642,'021642','1991-054E','deb IUS SRM-1','',64223,'Reentered','1992-09-22 00:00:00','1991-09-15 00:00:00',618.59,270,35072,27.04,'GEO Transfer' from dual union all
select 21643,'021643','1991-052C','Blenda','Sunshade',64221,'Reentered','1991-08-12 00:00:00','1991-08-11 00:00:00',88.44,186,209,82.31,'Intermediate' from dual union all
select 21644,'021644','1991-052D','KDU part','KDU part',64221,'Reentered','1991-08-10 00:00:00','1991-08-10 00:00:00',88.43,189,205,82.33,'Intermediate' from dual union all
select 21645,'021645','1991-052E','KDU','KDU',64221,'Reentered','1991-08-28 00:00:00','1991-08-18 00:00:00',89.22,227,245,82.32,'Intermediate' from dual union all
select 21646,'021646','1991-052F','KDU part','KDU part',64221,'Reentered','1991-08-15 00:00:00','1991-08-11 00:00:00',90.92,239,401,82.33,'Intermediate' from dual union all
select 21647,'021647','1991-052G','KDU part','KDU part',64221,'Reentered','1991-08-11 00:00:00','1991-08-10 00:00:00',89.95,213,331,82.32,'Intermediate' from dual union all
select 21648,'021648','1989-101G','Blok DM2 part','',62378,'In Earth orbit','','1991-10-08 00:00:00',1471.61,36330,36628,.05,'Drift GEO' from dual union all
select 21649,'021649','1986-017FE','Garbage bag','',59210,'Reentered','1991-08-29 00:00:00','1991-08-28 00:00:00',89.1,226,234,51.59,'Intermediate' from dual union all
select 2165,'002165','1966-037C','deb Kosmos-117','',13444,'Reentered','1966-05-07 00:00:00','1966-05-06 00:00:00',88.58,176,233,64.97,'Intermediate' from dual union all
select 21650,'021650','1986-017FF','Garbage bag','',59210,'Reentered','1991-08-13 00:00:00','1991-08-12 00:00:00',90.15,273,291,51.6,'Intermediate' from dual union all
select 21651,'021651','1986-017FG','Garbage bag','',59210,'Reentered','1991-08-12 00:00:00','1991-08-11 00:00:00',89.55,246,258,51.56,'Intermediate' from dual union all
select 21652,'021652','1986-017FH','Garbage bag','',59210,'Reentered','1991-08-21 00:00:00','1991-08-21 00:00:00',88.42,190,203,51.56,'Intermediate' from dual union all
select 21653,'021653','1991-055A','INTELSAT 605','INTELSAT VI F-5',64224,'In Earth orbit','','1991-09-13 00:00:00',1436.11,35756,35818,.03,'Stationary' from dual union all
select 21654,'021654','1991-055B','Ariane H10 V45','Ariane H10 S/N T417',64224,'Reentered','2012-01-04 00:00:00','1991-09-15 00:00:00',635.86,266,35967,7.08,'GEO Transfer' from dual union all
select 21655,'021655','1991-056A','Meteor-3','Meteor-3 No. 5/TOMS',64225,'In Earth orbit','','1991-09-16 00:00:00',109.34,1186,1207,82.55,'Intermediate' from dual union all
select 21656,'021656','1991-056B','S5M','S5M',64225,'In Earth orbit','','1992-01-04 00:00:00',109.33,1187,1205,82.55,'Intermediate' from dual union all
select 21657,'021657','1987-036M','deb SOZ','[UNID144] RCS 0.04',60126,'In Earth orbit','','1992-01-19 00:00:00',149.69,985,4937,64.81,'Medium' from dual union all
select 21658,'021658','1982-055BN','deb Kosmos-1375','',54364,'Reentered','2001-09-24 00:00:00','1991-10-09 00:00:00',103.64,541,1321,65.67,'Intermediate' from dual union all
select 21659,'021659','1963-014FF','deb West Ford','RCS 0.12',9245,'Reentered','1994-10-02 00:00:00','1991-10-03 00:00:00',162.91,1018,5987,86.09,'Medium' from dual union all
select 2166,'002166','1966-037D','deb Kosmos-117','',13444,'Reentered','1966-05-08 00:00:00','1966-05-06 00:00:00',89.37,192,295,65.01,'Intermediate' from dual union all
select 21660,'021660','1963-014FG','deb West Ford','RCS 0.06',9245,'In Earth orbit','','1991-09-09 00:00:00',164.23,1521,5591,86.01,'Medium' from dual union all
select 21661,'021661','1986-017FJ','Naduvaniy gazovoy ballon','Balloon subsatellite',59210,'Reentered','1991-08-29 00:00:00','1991-08-29 00:00:00',88.34,187,198,51.58,'Intermediate' from dual union all
select 21662,'021662','1991-057A','Progress M-9','Progress 7K-TGM No. 210',64226,'Deorbited','1991-09-30 00:00:00','1991-08-28 00:00:00',92.3,379,396,51.6,'Intermediate' from dual union all
select 21663,'021663','1991-057B','11S510','11S510',64226,'Reentered','1991-08-22 00:00:00','1991-08-21 00:00:00',88.48,184,215,51.62,'Intermediate' from dual union all
select 21664,'021664','1991-058A','Resurs-F','Resurs-F2 17F42 No. 7',64227,'Landed','1991-09-20 00:00:00','1991-08-28 00:00:00',89.08,224,233,82.32,'Intermediate' from dual union all
select 21665,'021665','1991-058B','11S510','11S510',64227,'Reentered','1991-08-23 00:00:00','1991-08-23 00:00:00',87.27,129,150,82.3,'Intermediate' from dual union all
select 21666,'021666','1991-059A','Kosmos-2154','Parus',64228,'In Earth orbit','','1991-09-22 00:00:00',104.9,972,1008,82.91,'Intermediate' from dual union all
select 21667,'021667','1991-059B','S3M','S3M',64228,'In Earth orbit','','1991-10-02 00:00:00',104.78,965,1004,82.91,'Intermediate' from dual union all
select 21668,'021668','1991-060A','Yuri 3B','BS 3B',64229,'In Earth orbit','','1991-09-25 00:00:00',1436.09,35774,35799,.17,'Stationary' from dual union all
select 21669,'021669','1991-060B','H-I Stage 2 S/N H-23(F)','LE-5',64229,'Reentered','1991-09-07 00:00:00','1991-08-27 00:00:00',92.41,171,615,30.58,'Intermediate' from dual union all
select 2167,'002167','1966-034B','FW4S S/N 20031A','FW4S',13441,'Reentered','2000-11-13 00:00:00','1966-05-28 00:00:00',151.63,351,5731,82.45,'Medium' from dual union all
select 21670,'021670','1991-060C','UM129A','UM129A',64229,'Reentered','1992-05-01 00:00:00','1992-01-28 00:00:00',638.59,155,36218,28.63,'GEO Transfer' from dual union all
select 21671,'021671','1975-052HX','deb Delta 111','',36499,'Reentered','1991-10-27 00:00:00','1991-10-05 00:00:00',105.55,838,1202,100.82,'Sun-Synch' from dual union all
select 21672,'021672','1975-052HY','deb Delta 111','',36499,'Reentered','1991-12-24 00:00:00','1991-10-04 00:00:00',114.24,1151,1690,99.01,'Sun-Synch' from dual union all
select 21673,'021673','1975-052HZ','deb Delta 111','',36499,'Reentered','1991-11-11 00:00:00','1991-09-14 00:00:00',110.83,1210,1320,99.85,'Sun-Synch' from dual union all
select 21674,'021674','1975-052JA','deb Delta 111','',36499,'In Earth orbit','','1991-09-25 00:00:00',103.64,770,1092,100.12,'Sun-Synch' from dual union all
select 21675,'021675','1975-052JB','deb Delta 111','',36499,'Reentered','1991-12-01 00:00:00','1991-09-09 00:00:00',113.51,956,1820,99.12,'Sun-Synch' from dual union all
select 21676,'021676','1975-052JC','deb Delta 111','',36499,'Reentered','1992-02-25 00:00:00','1991-09-21 00:00:00',123.51,933,2742,101.91,'Medium' from dual union all
select 21677,'021677','1975-052JD','deb Delta 111','',36499,'In Earth orbit','','1991-10-02 00:00:00',113.47,1010,1762,99.88,'Sun-Synch' from dual union all
select 21678,'021678','1975-052JE','deb Delta 111','',36499,'In Earth orbit','','1991-10-01 00:00:00',111.15,972,1588,99.95,'Sun-Synch' from dual union all
select 21679,'021679','1975-052JF','deb Delta 111','',36499,'Reentered','1991-10-19 00:00:00','1991-10-06 00:00:00',103.38,683,1154,99.2,'Sun-Synch' from dual union all
select 2168,'002168','1966-038A','Kosmos-118','Meteor 11F614 No. 4',13445,'Reentered','1988-11-23 00:00:00','1966-06-17 00:00:00',97.1,586,654,65.01,'Intermediate' from dual union all
select 21680,'021680','1975-052JG','deb Delta 111','',36499,'In Earth orbit','','1991-09-25 00:00:00',113.38,1085,1678,99.56,'Sun-Synch' from dual union all
select 21681,'021681','1975-052JH','deb Delta 111','',36499,'Reentered','1991-09-06 00:00:00','1991-09-08 00:00:00',95.85,502,618,98.67,'Sun-Synch' from dual union all
select 21682,'021682','1975-052JJ','deb Delta 111','',36499,'In Earth orbit','','1991-09-25 00:00:00',124.93,1079,2722,99.62,'Medium' from dual union all
select 21683,'021683','1975-052JK','deb Delta 111','',36499,'Reentered','1992-10-17 00:00:00','1991-09-18 00:00:00',112.52,1212,1472,103.29,'Sun-Synch' from dual union all
select 21684,'021684','1975-052JL','deb Delta 111','',36499,'Reentered','1991-08-28 00:00:00','1991-08-28 00:00:00',99.56,605,871,99.77,'Sun-Synch' from dual union all
select 21685,'021685','1975-052JM','deb Delta 111','',36499,'Reentered','1991-11-01 00:00:00','1991-09-11 00:00:00',108.63,1030,1297,99.81,'Sun-Synch' from dual union all
select 21686,'021686','1975-052JN','deb Delta 111','',36499,'In Earth orbit','','1991-10-06 00:00:00',106.62,855,1286,100,'Sun-Synch' from dual union all
select 21687,'021687','1975-052JP','deb Delta 111','',36499,'Reentered','1991-11-08 00:00:00','1991-09-03 00:00:00',122.08,905,2643,96.56,'Medium' from dual union all
select 21688,'021688','1991-061A','IRS-1B','IRS-1B',64230,'In Earth orbit','','1991-09-29 00:00:00',103.12,893,920,99.14,'Sun-Synch' from dual union all
select 21689,'021689','1991-061B','8A92ME','8A92ME',64230,'In Earth orbit','','1991-10-02 00:00:00',102.83,868,917,99.26,'Sun-Synch' from dual union all
select 2169,'002169','1966-038B','8A92E','8A92E',13445,'Reentered','1989-07-12 00:00:00','1966-06-18 00:00:00',96.99,551,679,65,'Intermediate' from dual union all
select 21690,'021690','1990-105AE','DMSP F-10 OLS Cooler Cover','',63347,'Reentered','2000-09-06 00:00:00','1991-10-05 00:00:00',100.01,706,812,98.87,'Sun-Synch' from dual union all
select 21691,'021691','1991-045E','REX despin weight','',64214,'In Earth orbit','','1991-12-19 00:00:00',100.55,771,799,89.85,'Polar' from dual union all
select 21692,'021692','1965-108AF','deb Transtage 8','',11858,'In Earth orbit','','1991-12-18 00:00:00',461.88,517,26313,27.36,'Highly Ellip' from dual union all
select 21693,'021693','1963-014FH','deb West Ford','RCS 0.03',9245,'Reentered','1996-11-03 00:00:00','1991-12-05 00:00:00',162.77,999,5995,85.17,'Medium' from dual union all
select 21694,'021694','1991-062A','Yohkoh','SOLAR-A',64231,'Reentered','2005-09-12 00:00:00','1991-10-06 00:00:00',97.82,521,787,31.35,'Intermediate' from dual union all
select 21695,'021695','1991-062B','M-3B','M-3B',64231,'Reentered','2014-05-02 00:00:00','1991-10-03 00:00:00',97.74,518,784,31.34,'Intermediate' from dual union all
select 21696,'021696','1991-062C','SOLAR-A despin weight','',64231,'Reentered','1993-06-27 00:00:00','1991-09-29 00:00:00',97.67,519,775,31.36,'Intermediate' from dual union all
select 21697,'021697','1991-062D','SOLAR-A despin weight','',64231,'Reentered','1993-08-04 00:00:00','1991-09-30 00:00:00',97.67,518,776,31.33,'Intermediate' from dual union all
select 21698,'021698','1991-062E','deb Yohkoh','',64231,'Reentered','1993-05-11 00:00:00','1991-09-29 00:00:00',97.67,519,775,31.34,'Intermediate' from dual;
insert into SATELLITE
select 21699,'021699','1991-062F','deb Yohkoh','',64231,'Reentered','1994-02-23 00:00:00','1991-09-29 00:00:00',97.69,519,778,31.36,'Intermediate' from dual union all
select 217,'000217','61 A Lambda 1','Samos 5 Reflector','Samos 5 Reflector',7257,'Reentered','1962-08-14 00:00:00','1962-01-26 00:00:00',94.81,225,794,89.64,'Polar' from dual union all
select 2170,'002170','1966-037E','deb Kosmos-117','',13444,'Reentered','1966-05-16 00:00:00','1966-05-14 00:00:00',89.2,199,271,64.95,'Intermediate' from dual union all
select 21700,'021700','1991-063A','Discovery (STS-48)','OV-103',64232,'Landed','1991-09-18 00:00:00','1991-09-14 00:00:00',96.04,563,575,56.98,'Intermediate' from dual union all
select 21701,'021701','1991-063B','UARS','UARS',64232,'Reentered','2011-09-24 00:00:00','1991-10-28 00:00:00',96.19,573,580,56.98,'Intermediate' from dual union all
select 21702,'021702','1991-064A','Kosmos-2155','Oko-S F4',64233,'In Earth orbit','','1991-10-14 00:00:00',1435.96,35763,35805,1.26,'Stationary' from dual union all
select 21703,'021703','1991-064B','Blok DM2','11S861 No. 54L',64233,'In Earth orbit','','1991-10-14 00:00:00',1441.45,35875,35908,1.27,'Drift GEO' from dual union all
select 21704,'021704','1991-064C','Perekhodnik','Sredniy perekhodnik',64233,'Reentered','1991-09-14 00:00:00','1991-09-14 00:00:00',87.09,129,132,51.65,'Intermediate' from dual union all
select 21705,'021705','1991-064D','8S812','8S812',64233,'Reentered','1991-09-15 00:00:00','1991-09-15 00:00:00',86.98,123,127,51.65,'Intermediate' from dual union all
select 21706,'021706','1991-065A','Molniya-3','Molniya-3 No. 48',64234,'In Earth orbit','','1991-10-17 00:00:00',717.72,493,39859,62.86,'Molniya' from dual union all
select 21707,'021707','1991-065B','11S510','11S510',64234,'Reentered','1991-09-30 00:00:00','1991-09-19 00:00:00',90.77,208,417,62.84,'Intermediate' from dual union all
select 21708,'021708','1991-065C','BOZ','BOZ',64234,'Reentered','1991-09-25 00:00:00','1991-09-19 00:00:00',90.87,203,432,62.82,'Intermediate' from dual union all
select 21709,'021709','1991-065D','Blok-ML','Blok-ML No. 338',64234,'Reentered','2010-12-06 00:00:00','1991-12-22 00:00:00',733.26,608,40508,62.84,'Molniya' from dual union all
select 2171,'002171','1966-039A','OPS 1950','GAMBIT SV 978',13446,'Reentered','1966-05-21 00:00:00','1966-05-15 00:00:00',89.38,131,357,110.5,'Retrograde' from dual union all
select 21710,'021710','1991-065E','deb Blok-I','',64234,'Reentered','1991-09-27 00:00:00','1991-09-19 00:00:00',90.56,178,425,62.82,'Intermediate' from dual union all
select 21711,'021711','1991-041D','deb Kosmos-2150','Strela-2M debris 1',64210,'In Earth orbit','','1991-10-02 00:00:00',100.75,782,807,74.03,'Intermediate' from dual union all
select 21712,'021712','1991-045F','REX despin weight','',64214,'In Earth orbit','','1991-10-06 00:00:00',102.04,746,964,89.31,'Polar' from dual union all
select 21713,'021713','1991-066A','Kosmos-2156','Kobal''t No. 538',64235,'Landed','1991-11-17 00:00:00','1991-09-26 00:00:00',89.59,180,329,67.13,'Intermediate' from dual union all
select 21714,'021714','1991-066B','11S510','11S510',64235,'Reentered','1991-09-22 00:00:00','1991-09-22 00:00:00',87.67,129,190,67.12,'Intermediate' from dual union all
select 21715,'021715','1991-058C','KDU','KDU',64227,'Reentered','1991-09-21 00:00:00','1991-09-21 00:00:00',88.13,179,185,82.31,'Intermediate' from dual union all
select 21716,'021716','1991-058D','Blenda','Sunshade',64227,'Reentered','1991-09-21 00:00:00','1991-09-21 00:00:00',89.68,195,322,82.3,'Intermediate' from dual union all
select 21717,'021717','1991-058E','Solnechnaya Batareya','Solar panel module',64227,'Reentered','1991-09-22 00:00:00','1991-09-21 00:00:00',89.43,186,306,82.27,'Intermediate' from dual union all
select 21718,'021718','1991-065F','deb Blok-I','',64234,'Reentered','1991-09-27 00:00:00','1991-09-21 00:00:00',90.65,204,409,62.82,'Intermediate' from dual union all
select 21719,'021719','1991-065G','deb Blok-I','',64234,'Reentered','1991-09-25 00:00:00','1991-09-21 00:00:00',90.22,196,375,62.8,'Intermediate' from dual union all
select 2172,'002172','1966-039B','LEIGE','P-11 No. 4404',13446,'Reentered','1970-10-27 00:00:00','1966-06-13 00:00:00',95.33,517,553,109.95,'Retrograde' from dual union all
select 21720,'021720','1991-065H','deb Blok-I','',64234,'Reentered','1991-09-25 00:00:00','1991-09-22 00:00:00',90.03,185,367,62.82,'Intermediate' from dual union all
select 21721,'021721','1970-025QG','deb Agena D','',21502,'In Earth orbit','','1991-10-04 00:00:00',106.53,1063,1069,100.09,'Sun-Synch' from dual union all
select 21722,'021722','1991-009BY','deb S3M','',64178,'In Earth orbit','','1991-09-27 00:00:00',122.36,1483,2090,74.05,'Medium' from dual union all
select 21723,'021723','1971-015DW','deb Kosmos-397','',24833,'In Earth orbit','','1991-09-28 00:00:00',107.09,559,1624,65.72,'Intermediate' from dual union all
select 21724,'021724','1991-009BZ','deb S3M','',64178,'In Earth orbit','','1991-09-30 00:00:00',118.06,1426,1762,73.98,'Intermediate' from dual union all
select 21725,'021725','1987-036N','deb Blok DM-2','',60126,'In Earth orbit','','1991-10-03 00:00:00',150.75,1361,4649,64.84,'Medium' from dual union all
select 21726,'021726','1991-067A','Anik E1','Anik E1',64236,'In Earth orbit','','1991-10-27 00:00:00',1436.06,35773,35799,.04,'Stationary' from dual union all
select 21727,'021727','1991-067B','Ariane H10 V46','Ariane H10 S/N T418',64236,'In Earth orbit','','1992-01-20 00:00:00',638.77,422,35960,3.86,'GEO Transfer' from dual union all
select 21728,'021728','1991-068A','Kosmos-2157','Strela-3',64237,'In Earth orbit','','1991-10-28 00:00:00',114.01,1406,1415,82.58,'Intermediate' from dual union all
select 21729,'021729','1991-068B','Kosmos-2158','Strela-3',64237,'In Earth orbit','','1991-12-24 00:00:00',113.93,1405,1409,82.58,'Intermediate' from dual union all
select 2173,'002173','1966-040A','Nimbus 2','Nimbus C',13447,'In Earth orbit','','1966-06-23 00:00:00',108.11,1101,1178,100.3,'Sun-Synch' from dual union all
select 21730,'021730','1991-068C','Kosmos-2159','Strela-3',64237,'In Earth orbit','','1991-10-29 00:00:00',113.76,1390,1408,82.58,'Intermediate' from dual union all
select 21731,'021731','1991-068D','Kosmos-2160','Strela-3',64237,'In Earth orbit','','1991-12-24 00:00:00',113.88,1400,1409,82.58,'Intermediate' from dual union all
select 21732,'021732','1991-068E','Kosmos-2161','Strela-3',64237,'In Earth orbit','','1991-12-24 00:00:00',113.83,1396,1409,82.58,'Intermediate' from dual union all
select 21733,'021733','1991-068F','Kosmos-2162','Strela-3',64237,'In Earth orbit','','1991-12-24 00:00:00',114.07,1407,1420,82.58,'Intermediate' from dual union all
select 21734,'021734','1991-068G','S5M','S5M',64237,'In Earth orbit','','1991-10-30 00:00:00',114.76,1408,1481,82.58,'Intermediate' from dual union all
select 21735,'021735','1991-069A','Soyuz TM-13','Soyuz 7K-STM No. 63',64238,'Landed','1992-03-25 00:00:00','1991-11-02 00:00:00',92.44,390,399,51.6,'Intermediate' from dual union all
select 21736,'021736','1991-069B','11S510','11S510',64238,'Reentered','1991-10-03 00:00:00','1991-10-03 00:00:00',88.2,176,195,51.65,'Intermediate' from dual union all
select 21737,'021737','1991-070A','Foton','Foton 34KS No. 7L',64239,'Landed','1991-10-20 00:00:00','1991-10-15 00:00:00',90.39,211,377,62.81,'Intermediate' from dual union all
select 21738,'021738','1991-070B','11S510','11S510',64239,'Reentered','1991-10-16 00:00:00','1991-10-07 00:00:00',90.35,213,371,62.8,'Intermediate' from dual union all
select 21739,'021739','1991-064E','SOZ','SOZ',64233,'Reentered','1992-04-09 00:00:00','1991-11-22 00:00:00',645.78,162,36579,47.3,'GEO Transfer' from dual union all
select 2174,'002174','1966-040B','Agena B 6202','Agena B 6202',13447,'In Earth orbit','','1967-01-31 00:00:00',107.87,1085,1172,100.3,'Sun-Synch' from dual union all
select 21740,'021740','1991-064F','SOZ','SOZ',64233,'Reentered','1992-05-09 00:00:00','1991-10-21 00:00:00',650.76,291,36704,47.5,'GEO Transfer' from dual union all
select 21741,'021741','1991-071A','Kosmos-2163','Orlets',64240,'Exploded','1991-12-06 00:00:00','1991-10-17 00:00:00',89.96,207,338,64.78,'Intermediate' from dual union all
select 21742,'021742','1991-071B','11S510','11S510',64240,'Reentered','1991-10-12 00:00:00','1991-10-11 00:00:00',88.62,157,256,64.79,'Intermediate' from dual union all
select 21743,'021743','1991-072A','Kosmos-2164','Taifun-1B',64241,'Reentered','1992-12-12 00:00:00','1991-12-29 00:00:00',93.92,278,654,74,'Intermediate' from dual union all
select 21744,'021744','1991-072B','S3M','S3M',64241,'Reentered','1992-03-14 00:00:00','1991-11-12 00:00:00',93.84,275,650,73.98,'Intermediate' from dual union all
select 21745,'021745','1991-072C','deb S3M','',64241,'Reentered','1991-10-25 00:00:00','1991-10-20 00:00:00',93.04,291,555,73.94,'Intermediate' from dual union all
select 21746,'021746','1991-073A','Progress M-10','Progress 7K-TGM No. 211',64242,'Deorbited','1992-01-20 00:00:00','1992-01-20 00:00:00',92.09,376,378,51.6,'Intermediate' from dual union all
select 21747,'021747','1991-073B','11S510','11S510',64242,'Reentered','1991-10-18 00:00:00','1991-10-18 00:00:00',87.58,150,160,51.65,'Intermediate' from dual union all
select 21748,'021748','1963-014FJ','deb West Ford','RCS 0.11',9245,'Reentered','1996-06-18 00:00:00','1991-10-23 00:00:00',163.12,997,6026,85.26,'Medium' from dual union all
select 21749,'021749','1968-097EV','deb Kosmos-252','',16824,'Reentered','1992-03-29 00:00:00','1991-11-06 00:00:00',97.34,430,833,61.73,'Intermediate' from dual union all
select 2175,'002175','1966-039C','Agena D 4824','Agena D',13446,'Reentered','1966-05-17 00:00:00','1966-05-15 00:00:00',88.78,105,324,110.68,'Retrograde' from dual union all
select 21750,'021750','1991-009CQ','deb S3M','',64178,'In Earth orbit','','1991-04-03 00:00:00',117.57,1473,1670,74.07,'Intermediate' from dual union all
select 21751,'021751','1988-085H','deb SOZ','',61334,'Reentered','2002-07-30 00:00:00','1992-01-20 00:00:00',222.89,642,10956,64.58,'Medium' from dual union all
select 21752,'021752','1983-127J','deb SOZ','',55884,'In Earth orbit','','1992-01-17 00:00:00',230.93,593,11588,53,'Medium' from dual union all
select 21753,'021753','1983-127K','deb SOZ','',55884,'Reentered','2003-11-09 00:00:00','1992-01-19 00:00:00',270.47,556,14395,52.07,'Medium' from dual union all
select 21754,'021754','1975-052JQ','deb Delta 111','',36499,'In Earth orbit','','1991-10-25 00:00:00',108.16,1083,1201,99.74,'Sun-Synch' from dual union all
select 21755,'021755','1991-070C','KBE part','KBE part',64239,'Reentered','1991-10-25 00:00:00','1991-10-23 00:00:00',89,184,267,62.8,'Intermediate' from dual union all
select 21756,'021756','1991-070D','PTDU cover','KBE part',64239,'Reentered','1991-10-25 00:00:00','1991-10-23 00:00:00',91.75,306,415,62.81,'Intermediate' from dual union all
select 21757,'021757','1991-070E','KBE part','KBE part',64239,'Reentered','1991-10-27 00:00:00','1991-10-23 00:00:00',91.5,297,400,62.81,'Intermediate' from dual union all
select 21758,'021758','1991-070F','KB electropitaniya','KBE',64239,'Reentered','1991-11-09 00:00:00','1991-10-23 00:00:00',90.05,205,349,62.8,'Intermediate' from dual union all
select 21759,'021759','1991-074A','Gorizont','Gorizont No. 35L',64243,'In Earth orbit','','1991-11-22 00:00:00',1436.01,35776,35794,1.42,'Stationary' from dual union all
select 2176,'002176','1966-041A','NNS O-9','NNS O-9',13448,'In Earth orbit','','1966-06-19 00:00:00',103.39,857,981,90,'Polar' from dual union all
select 21760,'021760','1991-074B','8S812','8S812',64243,'Reentered','1991-10-25 00:00:00','1991-10-24 00:00:00',88.21,183,189,51.64,'Intermediate' from dual union all
select 21761,'021761','1991-074C','Perekhodnik','Sredniy perekhodnik',64243,'Reentered','1991-10-24 00:00:00','1991-10-24 00:00:00',88.11,178,184,51.64,'Intermediate' from dual union all
select 21762,'021762','1991-074D','Blok DM2 No. 55L','11S861 No. 55L',64243,'In Earth orbit','','1991-11-26 00:00:00',1444.19,35904,35985,1.43,'Drift GEO' from dual union all
select 21763,'021763','1991-074E','SOZ','SOZ',64243,'Reentered','1992-08-02 00:00:00','1992-01-17 00:00:00',634.83,239,35941,47.18,'GEO Transfer' from dual union all
select 21764,'021764','1991-074F','SOZ','SOZ',64243,'Reentered','1992-04-16 00:00:00','1991-11-24 00:00:00',635.22,234,35966,47.38,'GEO Transfer' from dual union all
select 21765,'021765','1991-075A','INTELSAT 601','INTELSAT VI F-1',64244,'In Earth orbit','','1991-12-04 00:00:00',1436.19,35680,35897,.04,'Stationary' from dual union all
select 21766,'021766','1991-075B','Ariane H10 V47','Ariane H10 S/N T419',64244,'Reentered','2014-10-20 00:00:00','1992-01-11 00:00:00',631.6,273,35741,6.85,'GEO Transfer' from dual union all
select 21767,'021767','1989-100AH','Kosmos-2053 SS 29','ESO',62377,'Reentered','1991-11-23 00:00:00','1991-11-20 00:00:00',91.2,322,345,73.51,'Intermediate' from dual union all
select 21768,'021768','1989-100AJ','Kosmos-2053 SS 30','ESO',62377,'Reentered','1991-11-22 00:00:00','1991-11-19 00:00:00',90.98,311,335,73.51,'Intermediate' from dual union all
select 21769,'021769','1989-100AK','Kosmos-2053 SS 31','ESO',62377,'Reentered','1991-11-21 00:00:00','1991-11-19 00:00:00',90.97,311,334,73.5,'Intermediate' from dual union all
select 2177,'002177','1966-026D','DAPP despin weight','',13433,'In Earth orbit','','1966-05-18 00:00:00',102.36,630,1111,98.62,'Sun-Synch' from dual union all
select 21770,'021770','1989-100AL','Kosmos-2053 SS 32','ESO',62377,'Reentered','1991-11-22 00:00:00','1991-11-19 00:00:00',91.2,319,348,73.51,'Intermediate' from dual union all
select 21771,'021771','1989-100AM','Kosmos-2053 SS 33','ESO',62377,'Reentered','1991-11-22 00:00:00','1991-11-20 00:00:00',91.13,317,343,73.52,'Intermediate' from dual union all
select 21772,'021772','1989-100AN','Kosmos-2053 SS 34','ESO',62377,'Reentered','1991-11-29 00:00:00','1991-11-19 00:00:00',92.44,382,407,73.5,'Intermediate' from dual union all
select 21773,'021773','1989-100AP','Kosmos-2053 SS 35','ESO',62377,'Reentered','1991-11-27 00:00:00','1991-11-19 00:00:00',92.21,368,398,73.49,'Intermediate' from dual union all
select 21774,'021774','1989-100AQ','Kosmos-2053 SS 36','ESO',62377,'Reentered','1991-12-06 00:00:00','1991-11-21 00:00:00',92.86,403,427,73.53,'Intermediate' from dual union all
select 21775,'021775','1991-076A','USA 72','SLDCOM 2',64245,'In Earth orbit','','2007-02-17 00:00:00',260.2,2632,11613,63.36,'Medium' from dual union all
select 21776,'021776','1991-076B','Titan 45K-8 Stage 2','LR91AJ11A 45K-8',64245,'Reentered','1993-01-01 00:00:00','1991-12-08 00:00:00',93.48,300,590,63.53,'Intermediate' from dual union all
select 21777,'021777','1990-065Q','G-9B Canister','Large Ba Canister',63307,'In Earth orbit','','1991-12-25 00:00:00',600.53,811,33590,18.48,'GEO Transfer' from dual union all
select 21778,'021778','1990-065R','G-3 Canister','Ba Canister',63307,'In Earth orbit','','1992-01-12 00:00:00',610.37,296,34619,17.55,'GEO Transfer' from dual union all
select 21779,'021779','1991-077A','Kosmos-2165','Strela-3',64246,'In Earth orbit','','1991-12-15 00:00:00',113.85,1394,1413,82.61,'Intermediate' from dual union all
select 2178,'002178','1966-026E','deb DAPP','',13433,'Reentered','1983-01-06 00:00:00','1966-05-18 00:00:00',98.71,628,766,98.61,'Sun-Synch' from dual union all
select 21780,'021780','1991-077B','Kosmos-2166','Strela-3',64246,'In Earth orbit','','1991-12-12 00:00:00',113.99,1407,1411,82.6,'Intermediate' from dual union all
select 21781,'021781','1991-077C','Kosmos-2167','Strela-3',64246,'In Earth orbit','','1991-12-12 00:00:00',113.91,1400,1412,82.6,'Intermediate' from dual union all
select 21782,'021782','1991-077D','Kosmos-2168','Strela-3',64246,'In Earth orbit','','1991-12-12 00:00:00',113.8,1390,1411,82.6,'Intermediate' from dual union all
select 21783,'021783','1991-077E','Kosmos-2169','Strela-3',64246,'In Earth orbit','','1991-12-12 00:00:00',113.72,1383,1412,82.6,'Intermediate' from dual union all
select 21784,'021784','1991-077F','Kosmos-2170','Strela-3',64246,'In Earth orbit','','1991-12-12 00:00:00',114.06,1412,1413,82.6,'Intermediate' from dual union all
select 21785,'021785','1991-077G','S5M','S5M',64246,'In Earth orbit','','1991-12-22 00:00:00',114.73,1412,1475,82.59,'Intermediate' from dual union all
select 21786,'021786','1991-062G','deb Yohkoh','',64231,'Reentered','1993-06-09 00:00:00','1991-09-30 00:00:00',97.39,455,813,31.22,'Intermediate' from dual union all
select 21787,'021787','1991-078A','Kosmos-2171','Kobal''t No. 539',64247,'Landed','1992-01-17 00:00:00','1991-11-27 00:00:00',89.2,180,290,62.81,'Intermediate' from dual union all
select 21788,'021788','1991-078B','11S510','11S510',64247,'Reentered','1991-11-24 00:00:00','1991-11-22 00:00:00',89.2,182,287,62.81,'Intermediate' from dual;
insert into SATELLITE
select 21789,'021789','1991-079A','Kosmos-2172','Geizer No. 18L',64248,'In Earth orbit','','1991-12-22 00:00:00',1436.01,35776,35794,1.37,'Stationary' from dual union all
select 2179,'002179','1966-026F','deb DAPP','',13433,'Reentered','1993-04-20 00:00:00','1966-05-18 00:00:00',100.21,630,907,98.63,'Sun-Synch' from dual union all
select 21790,'021790','1991-079B','8S812','8S812',64248,'Reentered','1991-11-24 00:00:00','1991-11-24 00:00:00',87.48,148,152,51.63,'Intermediate' from dual union all
select 21791,'021791','1991-079C','Perekhodnik','Sredniy perekhodnik',64248,'Reentered','1991-11-23 00:00:00','1991-11-23 00:00:00',87.5,148,154,51.62,'Intermediate' from dual union all
select 21792,'021792','1991-079D','Blok DM2','11S861 No. 51L',64248,'In Earth orbit','','1991-12-27 00:00:00',1460.14,36219,36293,1.38,'Drift GEO' from dual union all
select 21793,'021793','1991-079E','SOZ','SOZ',64248,'Reentered','1992-06-23 00:00:00','1991-12-29 00:00:00',641.56,254,36271,47.16,'GEO Transfer' from dual union all
select 21794,'021794','1991-079F','SOZ','SOZ',64248,'Reentered','1996-10-15 00:00:00','1991-12-29 00:00:00',643.47,305,36318,47.24,'GEO Transfer' from dual union all
select 21795,'021795','1991-080A','Atlantis (STS-44)','OV-104',64249,'Landed','1991-12-01 00:00:00','1991-11-26 00:00:00',91.82,361,367,28.46,'Intermediate' from dual union all
select 21796,'021796','1991-081A','Kosmos-2172','Parus',64250,'In Earth orbit','','1992-01-14 00:00:00',104.74,946,1019,82.95,'Intermediate' from dual union all
select 21797,'021797','1991-081B','S3M','S3M',64250,'In Earth orbit','','1992-01-14 00:00:00',104.61,946,1007,82.96,'Intermediate' from dual union all
select 21798,'021798','1991-082A','DMSP F-11','DMSP Block 5D-2 S-12',64251,'In Earth orbit','','1991-12-30 00:00:00',101.89,839,858,98.93,'Sun-Synch' from dual union all
select 21799,'021799','1991-076C','USA 74','NOSS B-2 SSU-1',64245,'In Earth orbit','','1991-12-20 00:00:00',107.43,1052,1164,63.42,'Intermediate' from dual union all
select 218,'000218','61 A Lambda 2','S.T. LOW','AFTRACK 18 (S.T. LOW)',7257,'Reentered Attached','1961-12-31 00:00:00','1961-12-22 00:00:00',93.38,230,650,89.7,'Polar' from dual union all
select 2180,'002180','1966-041B','Altair 2 X-258E8 RH-110','X-258E8',13448,'In Earth orbit','','1966-07-06 00:00:00',103.41,854,986,90,'Polar' from dual union all
select 21800,'021800','1991-082B','DMSP F-11 solar array band','deb Star 37S',64251,'Reentered','2011-09-17 00:00:00','1992-01-19 00:00:00',101.81,840,849,98.91,'Sun-Synch' from dual union all
select 21801,'021801','1991-082C','DMSP F-11 solar array band','deb Star 37S',64251,'Reentered','2013-11-12 00:00:00','1991-12-28 00:00:00',101.86,842,852,98.92,'Sun-Synch' from dual union all
select 21802,'021802','1991-062H','deb Yohkoh','',64231,'Reentered','2000-11-15 00:00:00','1991-12-21 00:00:00',97.94,568,752,31.5,'Intermediate' from dual union all
select 21803,'021803','1991-083A','Eutelsat II F-3','Eutelsat II F-3',64252,'In Earth orbit','','1992-01-07 00:00:00',1436.04,35782,35789,.03,'Stationary' from dual union all
select 21804,'021804','1991-083B','Centaur AC-102','Centaur II(3) AC-102',64252,'In Earth orbit','','1992-01-29 00:00:00',753.44,845,41254,16.65,'Highly Ellip' from dual union all
select 21805,'021805','1991-080B','DSP F16','DSP F16',64249,'In Earth orbit','','2000-02-29 00:00:00',1436.09,35766,35807,3.97,'Inclined GEO' from dual union all
select 21806,'021806','1991-080C','IUS-14 SRM-1','Orbus 21',64249,'In Earth orbit','2000-01-01 00:00:00','1991-11-25 00:00:00',630.94,180,35800,26.5,'GEO Transfer' from dual union all
select 21807,'021807','1991-080D','IUS-14 SRM-2','Orbus 6E',64249,'In Earth orbit','','1991-11-25 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 21808,'021808','1991-076D','USA 76','NOSS B-2 SSU-2',64245,'In Earth orbit','','1991-12-20 00:00:00',107.44,1041,1176,63.43,'Intermediate' from dual union all
select 21809,'021809','1991-076E','USA 77','NOSS B-2 SSU-3',64245,'In Earth orbit','','1991-12-20 00:00:00',107.43,1051,1165,63.43,'Intermediate' from dual union all
select 2181,'002181','1966-042A','OPS 1778','CORONA J-33',13449,'Reentered Attached','1966-06-09 00:00:00','1966-05-31 00:00:00',88.89,175,265,66.03,'Intermediate' from dual union all
select 21810,'021810','1986-017FK','Garbage bag','',59210,'Reentered','1992-02-28 00:00:00','1991-12-17 00:00:00',92.12,371,386,51.61,'Intermediate' from dual union all
select 21811,'021811','1986-017FL','Garbage bag','',59210,'Reentered','1992-06-17 00:00:00','1991-12-13 00:00:00',92.21,378,387,51.6,'Intermediate' from dual union all
select 21812,'021812','1986-017FM','Garbage bag','',59210,'Reentered','1992-05-31 00:00:00','1991-12-21 00:00:00',92.12,375,382,51.6,'Intermediate' from dual union all
select 21813,'021813','1991-084A','Telecom 2A','Telecom 2A',64253,'In Earth orbit','','1992-01-15 00:00:00',1436.11,35771,35803,.05,'Stationary' from dual union all
select 21814,'021814','1991-084B','INMARSAT II F-3','INMARSAT II F-3',64253,'In Earth orbit','','1992-08-03 00:00:00',1436.04,35781,35790,2.24,'Stationary' from dual union all
select 21815,'021815','1991-084C','Ariane H10 V48','Ariane H10 S/N T420',64253,'In Earth orbit','','1992-01-21 00:00:00',644.84,381,36312,3.73,'GEO Transfer' from dual union all
select 21816,'021816','1991-085A','Kosmos-2174','Kometa No. 14',64254,'Landed','1992-01-30 00:00:00','1991-12-24 00:00:00',89.27,210,267,64.86,'Intermediate' from dual union all
select 21817,'021817','1991-085B','11S510','11S510',64254,'Reentered','1991-12-22 00:00:00','1991-12-22 00:00:00',87.34,134,152,64.86,'Intermediate' from dual union all
select 21818,'021818','1991-084D','Spelda 10 S/N UV12','Spelda 10',64253,'In Earth orbit','','1992-01-28 00:00:00',632.48,323,35736,3.81,'GEO Transfer' from dual union all
select 21819,'021819','1991-086A','Interkosmos-25','AUOS-Z No. 301',64255,'In Earth orbit','','1992-01-29 00:00:00',121.62,437,3070,82.57,'Medium' from dual union all
select 2182,'002182','1966-043A','Kosmos-119','DS-U2-I No. 1',13450,'Reentered Attached','1966-11-30 00:00:00','1966-06-24 00:00:00',98.87,208,1202,48.39,'Intermediate' from dual union all
select 21820,'021820','1991-086B','S5M','S5M',64255,'In Earth orbit','','1992-01-25 00:00:00',121.61,437,3069,82.57,'Medium' from dual union all
select 21821,'021821','1991-087A','Raduga','Gran'' No. 39L',64256,'In Earth orbit','','1992-01-19 00:00:00',1436.14,35763,35812,1.46,'Stationary' from dual union all
select 21822,'021822','1991-087B','8S812','8S812',64256,'Reentered','1991-12-21 00:00:00','1991-12-21 00:00:00',87.41,144,149,51.65,'Intermediate' from dual union all
select 21823,'021823','1991-087C','Perekhodnik','Sredniy perekhodnik',64256,'Reentered','1991-12-20 00:00:00','1991-12-19 00:00:00',87.79,162,169,51.64,'Intermediate' from dual union all
select 21824,'021824','1991-087D','Blok DM2','11S861 No. 52L',64256,'In Earth orbit','','1992-01-20 00:00:00',1469.01,36347,36510,1.44,'Drift GEO' from dual union all
select 21825,'021825','1991-082D','DMSP F-11 OLS Cover','',64251,'Reentered','2015-04-07 00:00:00','1992-01-05 00:00:00',101.85,838,855,98.93,'Sun-Synch' from dual union all
select 21826,'021826','1991-086C','IK-25 instrument cover','',64255,'In Earth orbit','','1992-01-29 00:00:00',121.35,437,3046,82.61,'Medium' from dual union all
select 21827,'021827','1991-086D','IK-25 instrument cover','',64255,'In Earth orbit','','1992-01-29 00:00:00',121.49,437,3058,82.52,'Medium' from dual union all
select 21828,'021828','1991-087E','SOZ','SOZ',64256,'Reentered','1992-08-13 00:00:00','1992-01-25 00:00:00',645.6,264,36468,47.15,'GEO Transfer' from dual union all
select 21829,'021829','1991-087F','SOZ','SOZ',64256,'Reentered','2000-10-08 00:00:00','1992-01-26 00:00:00',647.25,318,36498,47.2,'GEO Transfer' from dual union all
select 2183,'002183','1966-044A','Explorer 32','AE-B',13451,'Reentered','1985-02-22 00:00:00','1966-06-28 00:00:00',115.99,283,2718,64.67,'Intermediate' from dual union all
select 21830,'021830','1986-017FN','Garbage bag','',59210,'Reentered','1992-07-01 00:00:00','1991-12-23 00:00:00',92.15,377,383,51.6,'Intermediate' from dual union all
select 21831,'021831','1986-017FP','Garbage bag','',59210,'Reentered','1992-06-17 00:00:00','1991-12-23 00:00:00',92.14,375,384,51.6,'Intermediate' from dual union all
select 21832,'021832','1986-017FQ','Garbage bag','',59210,'Reentered','1992-06-04 00:00:00','1991-12-23 00:00:00',92.15,376,383,51.6,'Intermediate' from dual union all
select 21833,'021833','1991-088A','Zhongxing 4','DFH 2A-4',64257,'In Earth orbit','','1992-01-28 00:00:00',613.91,196,34903,31.52,'GEO Transfer' from dual union all
select 21834,'021834','1991-088B','CZ-3 Stage 3','CZ-3 Stage 3',64257,'Reentered','1993-04-12 00:00:00','1992-01-28 00:00:00',111.33,217,2359,31.06,'Intermediate' from dual union all
select 21835,'021835','1991-086E','Magion-3','S2-AP',64255,'In Earth orbit','','1992-01-18 00:00:00',121.63,437,3071,82.56,'Medium' from dual union all
select 21836,'021836','1991-082E','DMSP F-11 OLS Cooler Cover','',64251,'In Earth orbit','','1992-01-28 00:00:00',101.84,837,855,98.93,'Sun-Synch' from dual union all
select 21837,'021837','1970-025QH','deb Agena D','',21502,'Reentered','2001-09-28 00:00:00','1992-01-26 00:00:00',100.49,680,884,100.03,'Sun-Synch' from dual union all
select 21838,'021838','1970-025QJ','deb Agena D','',21502,'Reentered','2001-10-04 00:00:00','1992-01-28 00:00:00',100.49,680,884,100.02,'Sun-Synch' from dual union all
select 21839,'021839','1970-025QK','deb Agena D','',21502,'Reentered','2001-10-13 00:00:00','1992-01-27 00:00:00',100.52,677,890,100.09,'Sun-Synch' from dual union all
select 2184,'002184','1966-044B','FW4D','FW4D S/N 20033 (FW4S)',13451,'Reentered','1969-06-19 00:00:00','1966-06-25 00:00:00',115.75,277,2702,64.66,'Intermediate' from dual union all
select 21840,'021840','1970-025QL','deb Agena D','',21502,'Reentered','2001-04-03 00:00:00','1992-01-28 00:00:00',100.25,674,867,100.04,'Sun-Synch' from dual union all
select 21841,'021841','1970-025QM','deb Agena D','',21502,'Reentered','2001-04-19 00:00:00','1992-01-28 00:00:00',100.3,679,867,100.03,'Sun-Synch' from dual union all
select 21842,'021842','1991-039C','deb Okean','',64208,'Reentered','2007-02-24 00:00:00','1992-01-28 00:00:00',97.53,624,656,82.53,'Intermediate' from dual union all
select 21843,'021843','1991-078C','Kosmos-2171 part','Kobal''t AO part',64247,'Reentered','1992-01-18 00:00:00','1992-01-18 00:00:00',88.03,152,202,62.77,'Intermediate' from dual union all
select 21844,'021844','1992-001A','Kosmos-2175','Kobal''t No. 534',64713,'Landed','1992-03-20 00:00:00','1992-01-28 00:00:00',89.59,183,326,67.13,'Intermediate' from dual union all
select 21845,'021845','1992-001B','11S510','11S510',64713,'Reentered','1992-01-24 00:00:00','1992-01-23 00:00:00',88.68,150,269,67.13,'Intermediate' from dual union all
select 21846,'021846','1992-002A','Discovery (STS-42)','OV-103',64714,'Landed','1992-01-30 00:00:00','1992-01-24 00:00:00',90.47,292,304,56.97,'Intermediate' from dual union all
select 21847,'021847','1992-003A','Kosmos-2176','Oko',64715,'Reentered','2012-01-17 00:00:00','1992-02-23 00:00:00',717.84,633,39725,62.9,'Molniya' from dual union all
select 21848,'021848','1992-003B','11S510','11S510',64715,'Reentered','1992-02-19 00:00:00','1992-02-02 00:00:00',91.8,204,521,62.75,'Intermediate' from dual union all
select 21849,'021849','1992-003C','BOZ','BOZ',64715,'Reentered','1992-02-05 00:00:00','1992-01-27 00:00:00',92.03,172,576,62.81,'Intermediate' from dual union all
select 2185,'002185','1966-045A','Surveyor 1','Surveyor SC-1',13452,'Deep Space','1966-06-02 00:00:00','1966-05-30 00:00:00',28785.21,168,609332,30.05,'Cislunar' from dual union all
select 21850,'021850','1992-003D','Blok-2BL','Blok-2BL No. 77323-692',64715,'Reentered','2006-12-13 00:00:00','1992-02-23 00:00:00',706.12,631,39147,62.87,'Molniya' from dual union all
select 21851,'021851','1992-004A','Progress M-11','Progress 7K-TGM No. 212',64716,'Deorbited','1992-03-13 00:00:00','1992-02-03 00:00:00',92.24,375,394,51.6,'Intermediate' from dual union all
select 21852,'021852','1992-004B','11S510','11S510',64716,'Reentered','1992-01-26 00:00:00','1992-01-26 00:00:00',87.45,144,153,51.61,'Intermediate' from dual union all
select 21853,'021853','1992-005A','Kosmos-2177','Uragan No. 68L',64717,'In Earth orbit','','1992-02-29 00:00:00',675.69,19111,19147,64.79,'Medium' from dual union all
select 21854,'021854','1992-005B','Kosmos-2178','Uragan No. 69L',64717,'In Earth orbit','','1992-02-28 00:00:00',675.71,19088,19171,64.79,'Medium' from dual union all
select 21855,'021855','1992-005C','Kosmos-2179','Uragan No. 71L',64717,'In Earth orbit','','1992-02-28 00:00:00',675.71,19111,19148,64.79,'Medium' from dual union all
select 21856,'021856','1992-005D','8S812','8S812',64717,'Reentered','1992-01-30 00:00:00','1992-01-30 00:00:00',87.16,128,140,64.83,'Intermediate' from dual union all
select 21857,'021857','1992-005E','Perekhodnik','Sredniy perekhodnik',64717,'Reentered','1992-01-30 00:00:00','1992-01-30 00:00:00',87.04,118,138,64.85,'Intermediate' from dual union all
select 21858,'021858','1992-005F','Blok DM2','11S861 No. 59L',64717,'In Earth orbit','','1992-02-29 00:00:00',675.35,19112,19129,64.78,'Medium' from dual union all
select 21859,'021859','1988-098D','deb Ariane V26','',61347,'Reentered','1992-06-27 00:00:00','1992-02-05 00:00:00',136.11,240,4534,3.66,'Medium' from dual union all
select 2186,'002186','1966-046A','ATDA','TDA 4',13453,'Reentered','1966-07-11 00:00:00','1966-06-10 00:00:00',90.25,286,287,28.87,'Intermediate' from dual union all
select 21860,'021860','1983-127L','deb SOZ','',55884,'Reentered','1992-08-18 00:00:00','1992-02-01 00:00:00',271.39,434,14580,51.68,'Highly Ellip' from dual union all
select 21861,'021861','1991-009CA','deb S3M','',64178,'In Earth orbit','','1992-01-30 00:00:00',117.71,1470,1686,74.03,'Intermediate' from dual union all
select 21862,'021862','1992-005G','SOZ','SOZ',64717,'Reentered','1997-12-23 00:00:00','1992-02-29 00:00:00',340.36,413,19127,64.77,'Highly Ellip' from dual union all
select 21863,'021863','1992-005H','SOZ','SOZ',64717,'Reentered','1997-10-20 00:00:00','1992-03-02 00:00:00',340.34,413,19126,64.84,'Highly Ellip' from dual union all
select 21864,'021864','1990-038K','Kosmos-2075 SS 7','ESO',63280,'Reentered','1992-02-08 00:00:00','1992-02-06 00:00:00',90.01,267,283,74,'Intermediate' from dual union all
select 21865,'021865','1990-038L','Kosmos-2075 SS 8','ESO',63280,'Reentered','1992-02-07 00:00:00','1992-02-06 00:00:00',89.7,255,265,73.91,'Intermediate' from dual union all
select 21866,'021866','1990-038M','Kosmos-2075 SS 9','ESO',63280,'Reentered','1992-02-06 00:00:00','1992-02-06 00:00:00',89.8,249,281,73.99,'Intermediate' from dual union all
select 21867,'021867','1992-007A','Fuyo 1','ERS-1',64719,'Reentered','2001-12-03 00:00:00','1992-03-15 00:00:00',96.07,569,572,97.71,'Sun-Synch' from dual union all
select 21868,'021868','1992-007B','H-I Stage 2 S/N H-24(F)','LE-5',64719,'Reentered','1995-02-28 00:00:00','1992-03-15 00:00:00',94.5,441,547,97.69,'Sun-Synch' from dual union all
select 21869,'021869','1990-038N','Kosmos-2075 SS 10','ESO',63280,'Reentered','1992-02-13 00:00:00','1992-02-11 00:00:00',89.82,261,271,73.99,'Intermediate' from dual union all
select 2187,'002187','1966-045B','Centaur AC-10','Centaur 1D',13452,'Deep Space','1966-06-02 00:00:00','1966-05-30 00:00:00',19396.79,170,465348,30.06,'Cislunar' from dual union all
select 21870,'021870','1990-038P','Kosmos-2075 SS 11','ESO',63280,'Reentered','1992-02-11 00:00:00','1992-02-11 00:00:00',89.58,248,260,73.97,'Intermediate' from dual union all
select 21871,'021871','1990-038Q','Kosmos-2075 SS 12','ESO',63280,'Reentered','1992-02-13 00:00:00','1992-02-11 00:00:00',89.69,259,260,73.99,'Intermediate' from dual union all
select 21872,'021872','1990-038R','Kosmos-2075 SS 13','ESO',63280,'Reentered','1992-02-12 00:00:00','1992-02-11 00:00:00',89.5,242,258,73.98,'Intermediate' from dual union all
select 21873,'021873','1992-006A','USA 78','DSCS III B-14',64718,'In Earth orbit','','1992-04-01 00:00:00',1435.5,35750,35800,0,'Stationary' from dual union all
select 21874,'021874','1992-006B','Centaur AC-101','Centaur II(3) AC-101',64718,'Reentered','1998-06-18 00:00:00','1992-02-11 00:00:00',628.77,173,35695,28.5,'GEO Transfer' from dual union all
select 21875,'021875','1992-008A','Kosmos-2180','Parus',64720,'In Earth orbit','','1992-03-18 00:00:00',104.86,961,1015,82.93,'Intermediate' from dual union all
select 21876,'021876','1992-008B','S3M','S3M',64720,'In Earth orbit','','1992-03-21 00:00:00',104.77,960,1007,82.93,'Intermediate' from dual union all
select 21877,'021877','1992-006C','IABS-1','IABS-1',64718,'In Earth orbit','','1992-02-20 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 21878,'021878','1985-066H','NNS O-30 gravity boom part','',58289,'In Earth orbit','','1992-02-25 00:00:00',107.89,1003,1256,89.91,'Polar' from dual;
insert into SATELLITE
select 21879,'021879','1986-017FR','EO-10 EVA-1 debris','',59210,'Reentered','1992-03-25 00:00:00','1992-02-24 00:00:00',91.96,365,377,51.6,'Intermediate' from dual union all
select 2188,'002188','1966-046B','Atlas SLV-3 5304','Atlas SLV-3 5304',13453,'Exploded','1966-06-22 00:00:00','1966-06-08 00:00:00',90,269,280,28.81,'Intermediate' from dual union all
select 21880,'021880','1986-017FS','EO-10 EVA-1 debris','',59210,'Reentered','1992-03-22 00:00:00','1992-02-25 00:00:00',91.86,357,374,51.59,'Intermediate' from dual union all
select 21881,'021881','1986-017FT','EO-10 EVA-1 debris','',59210,'Reentered','1992-03-29 00:00:00','1992-02-26 00:00:00',92.01,363,384,51.59,'Intermediate' from dual union all
select 21882,'021882','1986-017FU','EO-10 EVA-1 debris','',59210,'Reentered','1993-01-21 00:00:00','1992-02-23 00:00:00',92.1,369,385,51.6,'Intermediate' from dual union all
select 21883,'021883','1986-017FV','EO-10 EVA-1 debris','',59210,'Reentered','1992-02-25 00:00:00','1992-02-24 00:00:00',90.59,298,309,51.58,'Intermediate' from dual union all
select 21884,'021884','1986-017FW','EO-10 EVA-1 debris','',59210,'Reentered','1993-01-18 00:00:00','1992-02-26 00:00:00',92.06,366,386,51.6,'Intermediate' from dual union all
select 21885,'021885','1986-017FX','EO-10 EVA-1 debris','',59210,'Reentered','1992-04-21 00:00:00','1992-02-25 00:00:00',92,364,382,51.6,'Intermediate' from dual union all
select 21886,'021886','1986-017FY','EO-10 EVA-1 debris','',59210,'Reentered','1992-03-26 00:00:00','1992-02-25 00:00:00',91.95,363,378,51.6,'Intermediate' from dual union all
select 21887,'021887','1986-017FZ','EO-10 EVA-1 debris','',59210,'Reentered','1992-06-29 00:00:00','1992-02-26 00:00:00',92,366,380,51.6,'Intermediate' from dual union all
select 21888,'021888','1986-017GA','EO-10 EVA-1 debris','',59210,'Reentered','1992-04-04 00:00:00','1992-02-23 00:00:00',92.02,365,383,51.61,'Intermediate' from dual union all
select 21889,'021889','1986-017GB','EO-10 EVA-1 debris','',59210,'Reentered','1992-06-09 00:00:00','1992-02-24 00:00:00',92.01,367,380,51.5,'Intermediate' from dual union all
select 2189,'002189','1966-046C','deb ATDA','',13453,'Reentered','1966-06-05 00:00:00','1966-06-02 00:00:00',90.62,264,345,28.87,'Intermediate' from dual union all
select 21890,'021890','1992-009A','GPS IIA-3','Navstar GPS SVN 25',64721,'In Earth orbit','','1992-03-27 00:00:00',717.9,20018,20343,54.7,'Medium' from dual union all
select 21891,'021891','1992-009B','Delta 207','Delta SSPS AJ10-118K',64721,'In Earth orbit','','1992-03-25 00:00:00',98.21,628,719,20,'Equatorial' from dual union all
select 21892,'021892','1992-009C','Star 48B','Star 48B S/N 10061-18',64721,'Reentered','1998-04-12 00:00:00','1992-03-24 00:00:00',350.73,186,20007,34.65,'Highly Ellip' from dual union all
select 21893,'021893','1992-010A','Superbird B1','Superbird B1',64722,'In Earth orbit','','1992-04-06 00:00:00',1436.06,35773,35799,.01,'Stationary' from dual union all
select 21894,'021894','1992-010B','Arabsat 1C','Arabsat 1C',64722,'In Earth orbit','','1992-03-27 00:00:00',1436.04,35774,35797,.08,'Stationary' from dual union all
select 21895,'021895','1992-010C','Ariane H10 V49','Ariane H10 S/N T421',64722,'Reentered','1998-05-02 00:00:00','1992-03-27 00:00:00',614.6,232,34903,6.98,'GEO Transfer' from dual union all
select 21896,'021896','1986-017GC','Garbage bag','',59210,'Reentered','1992-05-17 00:00:00','1992-03-09 00:00:00',91.73,351,368,51.6,'Intermediate' from dual union all
select 21897,'021897','1992-011A','Molniya-1','Molniya-1T No. 75',64723,'Reentered','2007-07-01 00:00:00','1992-04-03 00:00:00',717.66,637,39712,62.83,'Molniya' from dual union all
select 21898,'021898','1992-011B','11S510','11S510',64723,'Reentered','1992-03-28 00:00:00','1992-03-11 00:00:00',91.5,213,484,62.8,'Intermediate' from dual union all
select 21899,'021899','1992-011C','BOZ','BOZ',64723,'Reentered','1992-03-19 00:00:00','1992-03-11 00:00:00',91.3,177,500,62.81,'Intermediate' from dual union all
select 219,'000219','61 A Lambda 3','Samos 5','SAMOS E-5 No. 2',7257,'Reentered','1962-01-09 00:00:00','1961-12-30 00:00:00',89.12,187,275,89.67,'Polar' from dual union all
select 2190,'002190','1966-046D','deb ATDA','',13453,'Reentered','1966-06-05 00:00:00','1966-06-02 00:00:00',89.97,268,278,28.85,'Intermediate' from dual union all
select 21900,'021900','1992-011D','Blok-ML','Blok-ML No. 76719-484',64723,'Reentered','2006-09-26 00:00:00','1992-04-03 00:00:00',698.37,637,38756,62.8,'Molniya' from dual union all
select 21901,'021901','1986-017GD','Garbage bag','',59210,'Reentered','1992-05-10 00:00:00','1992-03-09 00:00:00',91.71,351,365,51.59,'Intermediate' from dual union all
select 21902,'021902','1992-012A','Kosmos-2181','Tsikada',64724,'In Earth orbit','','1992-04-08 00:00:00',104.95,972,1013,82.95,'Intermediate' from dual union all
select 21903,'021903','1992-012B','S3M','S3M',64724,'In Earth orbit','','1992-04-08 00:00:00',104.79,962,1008,82.94,'Intermediate' from dual union all
select 21904,'021904','1991-015E','Mage 1','Mage 1',64184,'In Earth orbit','','1992-03-12 00:00:00',1437.8,35165,36475,.72,'Drift GEO' from dual union all
select 21905,'021905','1991-086F','deb IK-25','',64255,'In Earth orbit','','1992-03-19 00:00:00',121.85,441,3086,82.61,'Medium' from dual union all
select 21906,'021906','1992-013A','Galaxy 5','Galaxy V',64725,'In Earth orbit','','1992-04-13 00:00:00',1434.22,35693,35807,.13,'Drift GEO' from dual union all
select 21907,'021907','1992-013B','Centaur AC-72','Centaur I AC-72',64725,'In Earth orbit','','1992-04-13 00:00:00',638.34,1096,35264,19.67,'GEO Transfer' from dual union all
select 21908,'021908','1992-014A','Soyuz TM-14','Soyuz 7K-STM No. 64',64726,'Landed','1992-08-10 00:00:00','1992-04-16 00:00:00',92.26,375,396,51.6,'Intermediate' from dual union all
select 21909,'021909','1992-014B','11S510','11S510',64726,'Reentered','1992-03-19 00:00:00','1992-03-18 00:00:00',88.49,181,219,51.66,'Intermediate' from dual union all
select 2191,'002191','1966-047A','Gemini IX-A','Gemini SC9',13454,'Landed','1966-06-06 00:00:00','1966-06-06 00:00:00',89.97,272,274,28.86,'Intermediate' from dual union all
select 21910,'021910','1975-052JR','deb Delta 111','',36499,'In Earth orbit','','1992-03-19 00:00:00',112.84,1093,1620,99.64,'Sun-Synch' from dual union all
select 21911,'021911','1990-081CH','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1992-03-20 00:00:00',101.55,787,878,98.85,'Sun-Synch' from dual union all
select 21912,'021912','1988-023C','Kosmos-1934 collision debris','',61272,'In Earth orbit','','1992-03-19 00:00:00',104.62,948,1006,82.96,'Intermediate' from dual union all
select 21913,'021913','1978-100AR','deb S5M','',45295,'In Earth orbit','','1992-03-19 00:00:00',120.28,1688,1698,82.54,'Medium' from dual union all
select 21914,'021914','1975-052JS','deb Delta 111','',36499,'In Earth orbit','','1992-03-19 00:00:00',118.84,1117,2141,99.46,'Sun-Synch' from dual union all
select 21915,'021915','1992-015A','Atlantis (STS-45)','OV-104',64727,'Landed','1992-04-02 00:00:00','1992-03-26 00:00:00',90.42,291,299,57,'Intermediate' from dual union all
select 21916,'021916','1990-050F','deb USA 59-62','',63292,'In Earth orbit','','1992-04-01 00:00:00',129.34,1228,2960,63.5,'Medium' from dual union all
select 21917,'021917','1990-050G','deb USA 59-62','',63292,'In Earth orbit','','1992-04-01 00:00:00',125.21,1224,2601,63.4,'Medium' from dual union all
select 21918,'021918','1991-009CB','deb S3M','',64178,'In Earth orbit','','1992-03-24 00:00:00',117.98,1493,1688,74.08,'Intermediate' from dual union all
select 21919,'021919','1991-009CC','deb S3M','',64178,'In Earth orbit','','1992-03-31 00:00:00',122.69,1507,2095,74.06,'Medium' from dual union all
select 2192,'002192','1966-048A','OPS 1577','GAMBIT SV 979',13455,'Deorbited','1966-06-09 00:00:00','1966-06-04 00:00:00',88.8,143,288,87.01,'Polar' from dual union all
select 21920,'021920','1992-016A','Kosmos-2182','Kobal''t No. 542',64728,'Landed','1992-05-30 00:00:00','1992-04-08 00:00:00',89.59,177,332,67.16,'Intermediate' from dual union all
select 21921,'021921','1992-016B','11S510','11S510',64728,'Reentered','1992-04-04 00:00:00','1992-04-03 00:00:00',87.58,130,180,67.15,'Intermediate' from dual union all
select 21922,'021922','1992-017A','Gorizont','Gorizont No. 36L',64729,'In Earth orbit','','1992-05-04 00:00:00',1435.98,35774,35795,1.36,'Stationary' from dual union all
select 21923,'021923','1992-017B','8S812','8S812',64729,'Reentered','1992-04-04 00:00:00','1992-04-04 00:00:00',87.08,123,136,51.61,'Intermediate' from dual union all
select 21924,'021924','1992-017C','Perekhodnik','Sredniy perekhodnik',64729,'Reentered','1992-04-02 00:00:00','1992-04-02 00:00:00',87.35,136,151,51.63,'Intermediate' from dual union all
select 21925,'021925','1992-017D','Blok DM2 No. 57L','11S861 No. 57L',64729,'In Earth orbit','','1992-05-02 00:00:00',1424.51,35464,35655,1.4,'Drift GEO' from dual union all
select 21926,'021926','1992-017E','SOZ','SOZ',64729,'Reentered','1992-10-23 00:00:00','1992-05-05 00:00:00',628.81,243,35627,47.26,'GEO Transfer' from dual union all
select 21927,'021927','1992-017F','SOZ','SOZ',64729,'Reentered','1992-11-04 00:00:00','1992-05-02 00:00:00',630.55,301,35659,47.33,'GEO Transfer' from dual union all
select 21928,'021928','1992-018A','Kosmos-2183','Neman',64730,'Deorbited','1993-02-16 00:00:00','1992-05-08 00:00:00',89.62,233,279,64.86,'Intermediate' from dual union all
select 21929,'021929','1992-018B','11S510','11S510',64730,'Reentered','1992-04-10 00:00:00','1992-04-09 00:00:00',88.35,166,220,64.89,'Intermediate' from dual union all
select 2193,'002193','1966-047B','Titan GT-9 Stage 2','LR91AJ5 GT-9 65-12564',13454,'Reentered','1966-06-04 00:00:00','1966-06-04 00:00:00',87.47,135,164,28.86,'Intermediate' from dual union all
select 21930,'021930','1992-019A','GPS IIA-4','Navstar GPS SVN 28',64731,'In Earth orbit','','1992-05-12 00:00:00',717.94,19979,20384,55.14,'Medium' from dual union all
select 21931,'021931','1992-019B','Delta 208','Delta SSPS AJ10-118K',64731,'Reentered','2013-12-15 00:00:00','1992-05-10 00:00:00',97.14,532,712,21.18,'Equatorial' from dual union all
select 21932,'021932','1992-019C','Star 48B','Star 48B S/N 10068-18',64731,'Reentered','1998-08-16 00:00:00','1992-05-10 00:00:00',352.43,187,20112,34.67,'Highly Ellip' from dual union all
select 21933,'021933','1975-052JT','deb Delta 111','',36499,'In Earth orbit','','1992-04-11 00:00:00',109.19,1059,1319,99.28,'Sun-Synch' from dual union all
select 21934,'021934','1975-052JU','deb Delta 111','',36499,'In Earth orbit','','1992-04-11 00:00:00',111.55,1068,1527,99.62,'Sun-Synch' from dual union all
select 21935,'021935','1983-127M','deb SOZ','',55884,'In Earth orbit','','1992-04-13 00:00:00',319.75,578,17644,52.41,'Highly Ellip' from dual union all
select 21936,'021936','1991-009CD','deb S3M','',64178,'In Earth orbit','','1992-04-11 00:00:00',119.97,1467,1892,74.07,'Intermediate' from dual union all
select 21937,'021937','1992-020A','Kosmos-2184','Parus',64732,'In Earth orbit','','1992-06-07 00:00:00',104.9,965,1015,82.93,'Intermediate' from dual union all
select 21938,'021938','1992-020B','S3M','S3M',64732,'In Earth orbit','','1992-05-16 00:00:00',104.77,963,1005,82.93,'Intermediate' from dual union all
select 21939,'021939','1992-021A','Telecom 2B','Telecom 2B',64733,'In Earth orbit','','1992-05-22 00:00:00',1436.01,35771,35799,.05,'Stationary' from dual union all
select 2194,'002194','1966-048B','Agena D 4826','Agena D',13455,'Reentered','1966-06-09 00:00:00','1966-06-05 00:00:00',88.62,132,281,86.97,'Polar' from dual union all
select 21940,'021940','1992-021B','INMARSAT II F-4','INMARSAT II F-4',64733,'In Earth orbit','','1992-05-22 00:00:00',1436.11,35768,35806,1.95,'Stationary' from dual union all
select 21941,'021941','1992-021C','Ariane H10+ V50','Ariane H10+ S/N T423',64733,'In Earth orbit','','1992-05-15 00:00:00',641.87,356,36185,3.98,'GEO Transfer' from dual union all
select 21942,'021942','1992-021D','Spelda 10 S/N UV13','Spelda 10',64733,'In Earth orbit','','1992-05-16 00:00:00',633.99,328,35809,4.03,'GEO Transfer' from dual union all
select 21943,'021943','1991-009CE','deb S3M','',64178,'In Earth orbit','','1992-04-16 00:00:00',118.12,1488,1705,73.95,'Intermediate' from dual union all
select 21944,'021944','1980-032C','SVS yo weight','',50242,'Reentered','2015-05-01 00:00:00','1992-04-20 00:00:00',232,473,11785,62.83,'Medium' from dual union all
select 21945,'021945','1965-048M','deb NNS O-4','',11798,'Reentered','2001-09-30 00:00:00','1992-04-21 00:00:00',105.9,964,1108,90.09,'Polar' from dual union all
select 21946,'021946','1992-022A','Progress M-12','Progress 7K-TGM No. 213',64734,'Deorbited','1992-06-28 00:00:00','1992-05-20 00:00:00',92.41,371,415,51.6,'Intermediate' from dual union all
select 21947,'021947','1992-022B','11S510','11S510',64734,'Reentered','1992-04-21 00:00:00','1992-04-20 00:00:00',87.72,155,168,51.67,'Intermediate' from dual union all
select 21948,'021948','1990-065S','G-4 Canister','Ba Canister',63307,'In Earth orbit','','1992-04-20 00:00:00',571.97,277,32617,17.45,'Highly Ellip' from dual union all
select 21949,'021949','1992-023A','USA 81','P-11 No. 5105',64735,'In Earth orbit','','1992-06-15 00:00:00',100.74,791,797,85.02,'Polar' from dual union all
select 2195,'002195','1966-049A','OGO 3','OGO B',13456,'Reentered','1981-09-15 00:00:00','1966-10-04 00:00:00',2914.63,904,121519,36.92,'Very High' from dual union all
select 21950,'021950','1992-023B','Titan 23G-3 Stage 2','LR91AJ5 23G-3',64735,'Reentered','1992-04-26 00:00:00','1992-04-26 00:00:00',87.7,147,175,85,'Polar' from dual union all
select 21951,'021951','1992-024A','Resurs-F','Resurs-F2 17F42 No. 8',64736,'Landed','1992-05-29 00:00:00','1992-05-06 00:00:00',89.1,223,236,82.09,'Intermediate' from dual union all
select 21952,'021952','1992-024B','11S510','11S510',64736,'Reentered','1992-05-01 00:00:00','1992-05-01 00:00:00',87.44,139,157,82.07,'Intermediate' from dual union all
select 21953,'021953','1992-025A','Kosmos-2185','Kometa No. 15',64737,'Landed','1992-06-11 00:00:00','1992-05-06 00:00:00',89.41,212,279,69.97,'Intermediate' from dual union all
select 21954,'021954','1992-025B','11S510','11S510',64737,'Reentered','1992-05-03 00:00:00','1992-05-03 00:00:00',87.35,131,156,69.97,'Intermediate' from dual union all
select 21955,'021955','1991-009CF','deb S3M','',64178,'In Earth orbit','','1992-05-08 00:00:00',90.48,258,339,28.34,'Intermediate' from dual union all
select 21956,'021956','1991-076F','deb USA 74','',64245,'In Earth orbit','','1992-05-01 00:00:00',106.08,1039,1051,63.42,'Intermediate' from dual union all
select 21957,'021957','1976-067BV','deb Kosmos-839','',39019,'In Earth orbit','','1992-05-10 00:00:00',117.05,978,2119,65.8,'Intermediate' from dual union all
select 21958,'021958','1991-009CG','deb S3M','',64178,'In Earth orbit','','1992-05-07 00:00:00',118.36,1501,1714,73.97,'Intermediate' from dual union all
select 21959,'021959','1991-009CH','deb S3M','',64178,'In Earth orbit','','1992-05-09 00:00:00',118.27,1501,1706,73.96,'Intermediate' from dual union all
select 2196,'002196','1966-050A','Kosmos-120','Zenit-2 No. 41',13457,'Landed','1966-06-16 00:00:00','1966-06-10 00:00:00',89.36,200,286,51.79,'Intermediate' from dual union all
select 21960,'021960','1985-118M','deb SOZ','',58341,'Reentered','1999-12-31 00:00:00','1992-05-12 00:00:00',332.94,852,18217,64.88,'Highly Ellip' from dual union all
select 21961,'021961','1991-086H','deb IK-25','',64255,'Reentered','2011-09-03 00:00:00','1994-09-09 00:00:00',118.26,433,2773,82.53,'Intermediate' from dual union all
select 21962,'021962','1977-054E','deb Molniya-1 (37)','',42182,'Reentered','1992-06-19 00:00:00','1992-05-11 00:00:00',663.67,140,37511,63.39,'GEO Transfer' from dual union all
select 21963,'021963','1992-026A','Endeavour (STS-49)','OV-105',64738,'Landed','1992-05-16 00:00:00','1992-05-09 00:00:00',90.77,275,350,28.33,'Intermediate' from dual union all
select 21964,'021964','1992-027A','Palapa B4','Palapa B4',64739,'In Earth orbit','','1992-06-17 00:00:00',1436.16,35777,35798,.16,'Stationary' from dual union all
select 21965,'021965','1992-027B','Delta 209','Delta SSPS AJ10-118K',64739,'In Earth orbit','','1992-06-13 00:00:00',119.22,502,2790,19.7,'Equatorial' from dual union all
select 21966,'021966','1992-027C','Star 48B','Star 48B S/N 10073-18',64739,'In Earth orbit','','1992-06-14 00:00:00',701.26,2816,36721,22.73,'GEO Transfer' from dual union all
select 21967,'021967','1992-028B','ASLV-D3 AS-4','AS-4',64740,'Reentered','1992-06-24 00:00:00','1992-05-27 00:00:00',91.11,250,408,46.03,'Intermediate' from dual union all
select 21968,'021968','1992-028A','SROSS-C','SROSS-C',64740,'Reentered','1992-07-14 00:00:00','1992-05-27 00:00:00',91.16,250,412,46.04,'Intermediate' from dual;
insert into SATELLITE
select 21969,'021969','1975-052JV','deb Delta 111','',36499,'In Earth orbit','','1992-05-25 00:00:00',108.36,1061,1241,98.98,'Sun-Synch' from dual union all
select 2197,'002197','1966-050B','11A57I','11A57I',13457,'Reentered','1966-06-12 00:00:00','1966-06-10 00:00:00',88.87,177,261,51.8,'Intermediate' from dual union all
select 21970,'021970','61 Omicron 299','deb r/b Ablestar','',7273,'In Earth orbit','','1992-05-26 00:00:00',102.49,442,1310,66.03,'Intermediate' from dual union all
select 21971,'021971','1965-082UQ','deb Transtage 4','',11832,'Reentered','1994-05-24 00:00:00','1992-05-26 00:00:00',96.6,551,641,32.51,'Intermediate' from dual union all
select 21972,'021972','1975-052JW','deb Delta 111','',36499,'In Earth orbit','','1992-05-21 00:00:00',111.63,1041,1563,99.61,'Sun-Synch' from dual union all
select 21973,'021973','1992-029A','Kosmos-2186','Kobal''t No. 540',64741,'Landed','1992-07-24 00:00:00','1992-06-04 00:00:00',89.58,178,330,62.85,'Intermediate' from dual union all
select 21974,'021974','1992-029B','11S510','11S510',64741,'Reentered','1992-06-03 00:00:00','1992-06-01 00:00:00',88.76,171,256,62.83,'Intermediate' from dual union all
select 21975,'021975','1992-024C','KDU','KDU',64736,'Reentered','1992-05-30 00:00:00','1992-05-29 00:00:00',88.73,201,223,82.07,'Intermediate' from dual union all
select 21976,'021976','1992-030A','Kosmos-2187','Strela-1M',64742,'In Earth orbit','','1992-07-11 00:00:00',114.68,1402,1480,74,'Intermediate' from dual union all
select 21977,'021977','1992-030B','Kosmos-2188','Strela-1M',64742,'In Earth orbit','','1992-07-11 00:00:00',114.5,1388,1478,74,'Intermediate' from dual union all
select 21978,'021978','1992-030C','Kosmos-2189','Strela-1M',64742,'In Earth orbit','','1992-07-11 00:00:00',114.83,1417,1478,74,'Intermediate' from dual union all
select 21979,'021979','1992-030D','Kosmos-2190','Strela-1M',64742,'In Earth orbit','','1992-07-11 00:00:00',115,1431,1479,74,'Intermediate' from dual union all
select 2198,'002198','1966-050C','deb Kosmos-120','',13457,'Reentered','1966-06-11 00:00:00','1966-06-09 00:00:00',88.93,179,265,51.74,'Intermediate' from dual union all
select 21980,'021980','1992-030E','Kosmos-2191','Strela-1M',64742,'In Earth orbit','','1992-07-11 00:00:00',115.7,1473,1502,74,'Intermediate' from dual union all
select 21981,'021981','1992-030F','Kosmos-2192','Strela-1M',64742,'In Earth orbit','','1992-07-11 00:00:00',115.5,1472,1484,74,'Intermediate' from dual union all
select 21982,'021982','1992-030G','Kosmos-2193','Strela-1M',64742,'In Earth orbit','','1992-07-11 00:00:00',115.15,1446,1479,74,'Intermediate' from dual union all
select 21983,'021983','1992-030H','Kosmos-2194','Strela-1M',64742,'In Earth orbit','','1992-07-11 00:00:00',115.34,1458,1484,74,'Intermediate' from dual union all
select 21984,'021984','1992-030J','S3M','S3M',64742,'In Earth orbit','','1992-07-05 00:00:00',117.82,1482,1684,74,'Intermediate' from dual union all
select 21985,'021985','1975-052JX','deb Delta 111','',36499,'In Earth orbit','','1992-06-10 00:00:00',107.79,1086,1162,99.72,'Sun-Synch' from dual union all
select 21986,'021986','61 Omicron 300','deb r/b Ablestar','',7273,'In Earth orbit','','1992-06-09 00:00:00',102.81,824,960,66.88,'Intermediate' from dual union all
select 21987,'021987','1992-031A','EUVE','EUVE',64743,'Reentered','2002-01-31 00:00:00','1992-07-11 00:00:00',95.04,514,527,28.43,'Intermediate' from dual union all
select 21988,'021988','1992-031B','Delta 210','Delta SSPS AJ10-118K',64743,'Reentered','1992-10-16 00:00:00','1992-07-07 00:00:00',92.31,277,499,30.21,'Intermediate' from dual union all
select 21989,'021989','1992-032A','INTELSAT K','INTELSAT K',64744,'In Earth orbit','','1992-07-19 00:00:00',1436.04,35775,35796,.17,'Stationary' from dual union all
select 2199,'002199','1966-050D','deb Kosmos-120','',13457,'Reentered','1966-06-09 00:00:00','1966-06-08 00:00:00',88.62,186,227,51.8,'Intermediate' from dual union all
select 21990,'021990','1992-032B','Centaur AC-105','Centaur IIA(4N) AC-105',64744,'Reentered','2002-05-30 00:00:00','1992-07-11 00:00:00',624.22,238,35395,26.59,'GEO Transfer' from dual union all
select 21991,'021991','1990-104X','Kosmos-2106 SS 18','ESO',63346,'Reentered','1992-10-07 00:00:00','1992-06-10 00:00:00',94.41,486,493,82.49,'Intermediate' from dual union all
select 21992,'021992','1990-104Y','Kosmos-2106 SS 19','ESO',63346,'Reentered','1992-09-17 00:00:00','1992-06-10 00:00:00',94.22,470,492,82.5,'Intermediate' from dual union all
select 21993,'021993','1990-104Z','Kosmos-2106 SS 20','ESO',63346,'Reentered','1992-09-17 00:00:00','1992-06-11 00:00:00',94.22,469,493,82.5,'Intermediate' from dual union all
select 21994,'021994','1990-104AA','Kosmos-2106 SS 21','ESO',63346,'Reentered','1992-10-07 00:00:00','1992-06-15 00:00:00',94.41,486,493,82.52,'Intermediate' from dual union all
select 21995,'021995','1990-104AB','Kosmos-2106 SS 22','ESO',63346,'Reentered','1992-09-23 00:00:00','1992-06-15 00:00:00',94.2,468,492,82.5,'Intermediate' from dual union all
select 21996,'021996','1990-104AC','Kosmos-2106 SS 23','ESO',63346,'Reentered','1992-09-22 00:00:00','1992-06-16 00:00:00',94.19,467,491,82.5,'Intermediate' from dual union all
select 21997,'021997','1978-100AS','deb S5M','',45295,'In Earth orbit','','1992-06-24 00:00:00',120,1435,1927,82.32,'Medium' from dual union all
select 21998,'021998','1992-033A','Resurs-F','Resurs-F1 14F43 No. 55',64745,'Landed','1992-07-09 00:00:00','1992-06-30 00:00:00',89.07,228,229,82.32,'Intermediate' from dual union all
select 21999,'021999','1992-033B','11S510','11S510',64745,'Reentered','1992-06-25 00:00:00','1992-06-24 00:00:00',88.22,172,201,82.32,'Intermediate' from dual union all
select 22,'000022','59 Iota 1','Explorer 7','NASA S-1A',5531,'In Earth orbit','','1959-11-17 00:00:00',101.36,560,1087,50.27,'Intermediate' from dual union all
select 220,'000220','61 Omicron 57','deb r/b Ablestar','',7273,'Reentered','1966-02-21 00:00:00','1962-01-16 00:00:00',98.06,392,940,67.08,'Intermediate' from dual union all
select 2200,'002200','1966-051A','FTV 1351','Midas RTS-I F1',13458,'Reentered Attached','1966-12-03 00:00:00','1966-07-09 00:00:00',120.62,172,3246,90.02,'Medium' from dual union all
select 22000,'022000','1992-034A','Columbia (STS-50)','OV-102',64746,'Landed','1992-07-09 00:00:00','1992-06-27 00:00:00',90.55,299,304,28.47,'Intermediate' from dual union all
select 22001,'022001','1990-104AD','Kosmos-2106 SS 24','ESO',63346,'Reentered','1992-10-02 00:00:00','1992-01-22 00:00:00',90.54,299,303,56.99,'Intermediate' from dual union all
select 22002,'022002','1990-104AE','Kosmos-2106 SS 25','ESO',63346,'Reentered','1992-10-06 00:00:00','1992-05-08 00:00:00',90.8,290,337,28.33,'Intermediate' from dual union all
select 22003,'022003','1990-104AF','Kosmos-2106 SS 26','ESO',63346,'Reentered','1992-10-06 00:00:00','1992-05-08 00:00:00',90.94,290,352,28.33,'Intermediate' from dual union all
select 22004,'022004','1992-035A','Progress M-13','Progress 7K-TGM No. 214',64747,'Deorbited','1992-07-24 00:00:00','1992-07-07 00:00:00',92.53,387,410,51.6,'Intermediate' from dual union all
select 22005,'022005','1992-035B','11S510','11S510',64747,'Reentered','1992-07-02 00:00:00','1992-07-01 00:00:00',88.44,180,214,51.62,'Intermediate' from dual union all
select 22006,'022006','1992-036A','Kosmos-2195','Parus',64748,'In Earth orbit','','1992-08-04 00:00:00',104.76,957,1010,82.93,'Intermediate' from dual union all
select 22007,'022007','1992-036B','S3M','S3M',64748,'In Earth orbit','','1992-08-03 00:00:00',104.6,952,1000,82.93,'Intermediate' from dual union all
select 22008,'022008','1978-014H','deb Kyokko','',45209,'Reentered','1992-08-02 00:00:00','1992-07-07 00:00:00',122.32,611,2958,64.34,'Medium' from dual union all
select 22009,'022009','1992-037A','USA 82','DSCS III B-12',64749,'In Earth orbit','','1996-02-27 00:00:00',1436.09,35718,35855,.05,'Stationary' from dual union all
select 2201,'002201','1966-052A','OV3-4','OV3-4 PHASR',13459,'In Earth orbit','','1966-07-11 00:00:00',143.16,643,4730,40.83,'Medium' from dual union all
select 22010,'022010','1992-037B','Centaur AC-103','Centaur II(3) AC-103',64749,'Reentered','2002-06-17 00:00:00','1992-07-02 00:00:00',633.27,300,35800,26.1,'GEO Transfer' from dual union all
select 22011,'022011','1992-037C','IABS-2','IABS-2',64749,'In Earth orbit','','1992-07-30 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 22012,'022012','1992-038A','SAMPEX','SAMPEX',64750,'Reentered','2012-11-13 00:00:00','1992-08-11 00:00:00',96.72,514,689,81.67,'Intermediate' from dual union all
select 22013,'022013','1992-038B','Star 20 E-48','Star 20 E-48',64750,'Reentered','2002-01-28 00:00:00','1992-08-03 00:00:00',96.7,514,687,81.67,'Intermediate' from dual union all
select 22014,'022014','1992-039A','GPS IIA-5','Navstar GPS SVN 26',64751,'In Earth orbit','','1992-08-06 00:00:00',717.92,19959,20403,55.03,'Medium' from dual union all
select 22015,'022015','1992-039B','Delta 211','Delta SSPS AJ10-118K',64751,'In Earth orbit','','1992-08-11 00:00:00',97.63,567,724,20.72,'Equatorial' from dual union all
select 22016,'022016','1992-039C','Star 48B','Star 48B S/N 10069-18',64751,'Reentered','1999-11-23 00:00:00','1992-08-06 00:00:00',353.52,175,20192,34.73,'Highly Ellip' from dual union all
select 22017,'022017','1992-040A','Kosmos-2196','Oko',64752,'Reentered','2016-09-27 00:00:00','1992-08-07 00:00:00',718.31,616,39765,63.04,'Molniya' from dual union all
select 22018,'022018','1992-040B','11S510','11S510',64752,'Reentered','1992-08-15 00:00:00','1992-07-16 00:00:00',91.83,209,520,62.78,'Intermediate' from dual union all
select 22019,'022019','1992-040C','BOZ','BOZ',64752,'Reentered','1992-07-26 00:00:00','1992-07-16 00:00:00',91.17,173,491,62.82,'Intermediate' from dual union all
select 2202,'002202','1966-051C','ERS 16','ORS 2',13458,'Reentered','1967-03-12 00:00:00','1966-07-10 00:00:00',122.35,176,3396,90.01,'Medium' from dual union all
select 22020,'022020','1992-040D','Blok-2BL','Blok-2BL',64752,'Reentered','2015-06-28 00:00:00','1992-08-15 00:00:00',705.77,628,39133,62.96,'Molniya' from dual union all
select 22021,'022021','1992-040E','deb Blok-I','',64752,'Reentered','1992-07-14 00:00:00','1992-07-13 00:00:00',90.94,245,397,62.38,'Intermediate' from dual union all
select 22022,'022022','1992-040F','deb Blok-I','',64752,'Reentered','1992-08-07 00:00:00','1992-07-17 00:00:00',91.9,206,530,62.82,'Intermediate' from dual union all
select 22023,'022023','1986-017GE','EO-11 EVA-1 debris','',59210,'Reentered','1992-12-12 00:00:00','1992-07-13 00:00:00',92.49,385,408,51.59,'Intermediate' from dual union all
select 22024,'022024','1986-017GF','EO-11 EVA-1 debris','',59210,'Reentered','1993-05-04 00:00:00','1992-07-13 00:00:00',92.5,386,409,51.6,'Intermediate' from dual union all
select 22025,'022025','1991-009CJ','deb S3M','',64178,'In Earth orbit','','1992-07-09 00:00:00',117.02,1433,1661,73.94,'Intermediate' from dual union all
select 22026,'022026','1992-033C','Blenda','Sunshade',64745,'Reentered','1992-07-12 00:00:00','1992-07-12 00:00:00',88.44,167,227,82.28,'Intermediate' from dual union all
select 22027,'022027','1992-041A','Insat 2A','Insat 2A',64753,'In Earth orbit','','1992-08-09 00:00:00',1436.06,35770,35802,.17,'Stationary' from dual union all
select 22028,'022028','1992-041B','Eutelsat II F-4','Eutelsat II F-4',64753,'In Earth orbit','','1992-08-10 00:00:00',1436.11,35773,35801,.2,'Stationary' from dual union all
select 22029,'022029','1992-033D','KDU part','KDU part',64745,'Reentered','1992-07-11 00:00:00','1992-07-10 00:00:00',87.44,141,155,82.3,'Intermediate' from dual union all
select 2203,'002203','1965-020EF','deb Blok Kosmos','',11770,'Reentered','1967-04-20 00:00:00','1966-06-11 00:00:00',101.76,288,1395,55.94,'Intermediate' from dual union all
select 22030,'022030','1992-033E','KDU','KDU',64745,'Reentered','1992-07-21 00:00:00','1992-07-14 00:00:00',88.6,204,207,82.31,'Intermediate' from dual union all
select 22031,'022031','1992-033F','KDU part','KDU part',64745,'Reentered','1992-07-10 00:00:00','1992-07-10 00:00:00',87.65,152,165,82.34,'Intermediate' from dual union all
select 22032,'022032','1992-041C','Ariane H10 V51','Ariane H10 S/N T422',64753,'In Earth orbit','','1992-08-09 00:00:00',635.18,252,35946,7.27,'GEO Transfer' from dual union all
select 22033,'022033','1992-041D','Spelda 10 S/N UV15','Spelda 10',64753,'Reentered','2000-05-01 00:00:00','1992-08-11 00:00:00',620.27,242,35187,7.09,'GEO Transfer' from dual union all
select 22034,'022034','1992-042A','Kosmos-2197','Strela-3',64754,'In Earth orbit','','1992-08-13 00:00:00',113.91,1397,1414,82.59,'Intermediate' from dual union all
select 22035,'022035','1992-042B','Kosmos-2198','Strela-3',64754,'In Earth orbit','','1992-08-14 00:00:00',114.05,1410,1415,82.59,'Intermediate' from dual union all
select 22036,'022036','1992-042C','Kosmos-2199','Gonets-D',64754,'In Earth orbit','','1992-08-13 00:00:00',114.21,1414,1424,82.59,'Intermediate' from dual union all
select 22037,'022037','1992-042D','Kosmos-2200','Strela-3',64754,'In Earth orbit','','1992-08-14 00:00:00',114,1405,1414,82.59,'Intermediate' from dual union all
select 22038,'022038','1992-042E','Kosmos-2201','Gonets-D',64754,'In Earth orbit','','1992-08-14 00:00:00',114.14,1413,1420,82.59,'Intermediate' from dual union all
select 22039,'022039','1992-042F','Kosmos-2202','Strela-3',64754,'In Earth orbit','','1992-08-14 00:00:00',114.06,1409,1417,82.59,'Intermediate' from dual union all
select 2204,'002204','1965-020EG','deb Blok Kosmos','',11770,'Reentered','1966-10-17 00:00:00','1966-06-11 00:00:00',97.83,291,1019,56.06,'Intermediate' from dual union all
select 22040,'022040','1992-042G','S5M','S5M',64754,'In Earth orbit','','1992-08-24 00:00:00',114.7,1411,1473,82.6,'Intermediate' from dual union all
select 22041,'022041','1992-043A','Gorizont','Gorizont No. 37L',64755,'In Earth orbit','','1992-08-13 00:00:00',1435.88,35770,35795,1.45,'Stationary' from dual union all
select 22042,'022042','1992-043B','8S812','8S812',64755,'Reentered','1992-07-17 00:00:00','1992-07-16 00:00:00',88.02,173,180,51.66,'Intermediate' from dual union all
select 22043,'022043','1992-043C','Perekhodnik','Sredniy perekhodnik',64755,'Reentered','1992-07-15 00:00:00','1992-07-15 00:00:00',88.4,188,203,51.66,'Intermediate' from dual union all
select 22044,'022044','1992-043D','Blok DM2 No. 60L','11S861 No. 60L',64755,'In Earth orbit','','1992-08-17 00:00:00',1471.84,36375,36592,1.46,'Drift GEO' from dual union all
select 22045,'022045','1986-017GG','EO-11 EVA-1 debris','',59210,'Reentered','1993-05-04 00:00:00','1992-03-23 00:00:00',90.4,289,300,57,'Intermediate' from dual union all
select 22046,'022046','1986-017GH','EO-11 EVA-1 debris','',59210,'Reentered','1992-10-19 00:00:00','1992-08-03 00:00:00',92.26,373,397,51.6,'Intermediate' from dual union all
select 22047,'022047','1992-043E','SOZ','SOZ',64755,'Reentered','1993-06-18 00:00:00','1992-08-18 00:00:00',647.25,285,36531,47.17,'GEO Transfer' from dual union all
select 22048,'022048','1992-043F','SOZ','SOZ',64755,'Reentered','2001-07-23 00:00:00','1992-08-14 00:00:00',648.56,343,36540,47.25,'GEO Transfer' from dual union all
select 22049,'022049','1992-044A','Geotail','Geotail',64756,'Deep Space','1992-07-25 00:00:00','1992-10-19 00:00:00',2474.82,3906,104552,22.35,'Very High' from dual union all
select 2205,'002205','1966-051B','EGRS VI','Secor Type II S/N 7',13458,'Reentered','1967-07-06 00:00:00','1966-07-09 00:00:00',123.23,171,3479,90.05,'Medium' from dual union all
select 22050,'022050','1992-044B','Delta 212','Delta SSPS AJ10-118K',64756,'Reentered','1993-03-16 00:00:00','1992-08-24 00:00:00',101.16,215,1413,27.45,'Intermediate' from dual union all
select 22051,'022051','1992-044C','Star 48B','Star 48B S/N 10076-18',64756,'Deep Space','1992-07-25 00:00:00','1992-07-24 00:00:00',13367.24,184,360211,28.7,'Cislunar' from dual union all
select 22052,'022052','1992-045A','Kosmos-2203','Kobal''t No. 543',64757,'Landed','1992-09-22 00:00:00','1992-08-23 00:00:00',89.76,173,353,62.8,'Intermediate' from dual union all
select 22053,'022053','1992-045B','11S510','11S510',64757,'Reentered','1992-08-01 00:00:00','1992-07-26 00:00:00',89.28,188,290,62.8,'Intermediate' from dual union all
select 22054,'022054','1992-046A','Soyuz TM-15','Soyuz 7K-STM No. 65',64758,'Landed','1993-02-01 00:00:00','1992-08-27 00:00:00',92.67,403,408,51.62,'Intermediate' from dual union all
select 22055,'022055','1992-046B','11S510','11S510',64758,'Reentered','1992-07-29 00:00:00','1992-07-29 00:00:00',87.5,150,152,51.63,'Intermediate' from dual union all
select 22056,'022056','1992-047A','Kosmos-2204','Uragan No. 56L',64759,'In Earth orbit','','1992-08-30 00:00:00',675.71,19117,19142,64.88,'Medium' from dual union all
select 22057,'022057','1992-047B','Kosmos-2205','Uragan No. 72L',64759,'In Earth orbit','','1992-08-30 00:00:00',675.69,19119,19139,64.86,'Medium' from dual;
insert into SATELLITE
select 22058,'022058','1992-047C','Kosmos-2206','Uragan No. 74L',64759,'In Earth orbit','','1992-08-29 00:00:00',675.69,19102,19156,64.86,'Medium' from dual union all
select 22059,'022059','1992-047D','8S812','8S812',64759,'Reentered','1992-07-30 00:00:00','1992-07-30 00:00:00',87.25,131,146,64.83,'Intermediate' from dual union all
select 2206,'002206','1966-052B','FW4S S/N 20039','FW4S',13459,'In Earth orbit','','1966-07-13 00:00:00',143.16,643,4730,40.81,'Medium' from dual union all
select 22060,'022060','1992-047E','Perekhodnik','Sredniy perekhodnik',64759,'Reentered','1995-10-03 00:00:00','1992-07-30 00:00:00',87.55,149,158,64.82,'Intermediate' from dual union all
select 22061,'022061','1992-047F','Blok DM2','11S861 No. 63L',64759,'In Earth orbit','','1992-08-29 00:00:00',675,19096,19127,64.84,'Medium' from dual union all
select 22062,'022062','1992-048A','Kosmos-2207','Oblik No. 830',64760,'Landed','1992-08-13 00:00:00','1992-08-01 00:00:00',89.99,232,316,82.33,'Intermediate' from dual union all
select 22063,'022063','1992-048B','11S510','11S510',64760,'Reentered','1992-08-01 00:00:00','1992-07-31 00:00:00',88.37,176,212,82.34,'Intermediate' from dual union all
select 22064,'022064','1992-049A','Atlantis (STS-46)','OV-104',64761,'Landed','1992-08-08 00:00:00','1992-08-02 00:00:00',93.11,421,432,28.47,'Intermediate' from dual union all
select 22065,'022065','1992-049B','Eureca 1','Eureca 1',64761,'Landed Attached','1993-07-01 00:00:00','1992-09-25 00:00:00',94.65,500,504,28.46,'Intermediate' from dual union all
select 22066,'022066','1992-047G','SOZ','SOZ',64759,'In Earth orbit','','1992-09-01 00:00:00',340.3,405,19130,64.93,'Highly Ellip' from dual union all
select 22067,'022067','1992-047H','SOZ','SOZ',64759,'Reentered','2000-12-19 00:00:00','1992-08-31 00:00:00',340.25,402,19131,64.82,'Highly Ellip' from dual union all
select 22068,'022068','1992-050A','Molniya-1','Molniya-1T No. 72',64762,'Reentered','2008-04-04 00:00:00','1992-09-05 00:00:00',717.74,661,39692,62.87,'Molniya' from dual union all
select 22069,'022069','1992-050B','11S510','11S510',64762,'Reentered','1992-09-20 00:00:00','1992-08-14 00:00:00',92.18,213,550,62.82,'Intermediate' from dual union all
select 2207,'002207','1966-053A','GGTS','GGTS',13460,'In Earth orbit','','1966-12-16 00:00:00',1333.76,33661,33857,.43,'Medium' from dual union all
select 22070,'022070','1992-050C','BOZ','BOZ',64762,'Reentered','1992-08-29 00:00:00','1992-08-29 00:00:00',87.5,129,173,62.8,'Intermediate' from dual union all
select 22071,'022071','1992-050D','Blok-ML','Blok-ML',64762,'Reentered','2008-03-13 00:00:00','1992-09-06 00:00:00',733.34,660,40460,62.87,'Molniya' from dual union all
select 22072,'022072','1992-051A','Fanhui Shi Weixing (13)','Jianbing-1B No. 1',64763,'Reentered','1992-09-01 00:00:00','1992-08-16 00:00:00',89.55,170,335,63.07,'Intermediate' from dual union all
select 22073,'022073','1992-051B','CZ-2D Stage 2','CZ-2D Stage 2',64763,'Reentered','1992-08-16 00:00:00','1992-08-13 00:00:00',88.68,160,258,63.06,'Intermediate' from dual union all
select 22074,'022074','1992-051C','CZ-2D sep motor cover','',64763,'Reentered','1992-08-09 00:00:00','1992-08-09 00:00:00',87.63,136,179,63.1,'Intermediate' from dual union all
select 22075,'022075','1992-051D','CZ-2D sep motor cover','',64763,'Reentered','1992-08-10 00:00:00','1992-08-09 00:00:00',88.49,151,249,63.1,'Intermediate' from dual union all
select 22076,'022076','1992-052A','Topex-Poseidon','Topex-Poseidon',64764,'In Earth orbit','','1992-09-10 00:00:00',112.38,1331,1340,66.04,'Intermediate' from dual union all
select 22077,'022077','1992-052B','Kitsat-OSCAR-23','Kitsat A',64764,'In Earth orbit','','1992-09-09 00:00:00',112.69,1301,1400,66.07,'Intermediate' from dual union all
select 22078,'022078','1992-052C','S80/T','S80/T',64764,'In Earth orbit','','1992-09-11 00:00:00',111.92,1302,1328,66.08,'Intermediate' from dual union all
select 22079,'022079','1992-052D','Ariane H10 V52','Ariane H10 S/N T424',64764,'In Earth orbit','','1992-09-30 00:00:00',111.94,1306,1326,66.08,'Intermediate' from dual union all
select 2208,'002208','1966-034C','OV3-1 despin weight','',13441,'Reentered','2012-01-09 00:00:00','1974-06-30 00:00:00',146.39,363,5283,82.33,'Medium' from dual union all
select 22080,'022080','1992-053A','Kosmos-2208','Strela-2M',64765,'In Earth orbit','','1992-09-12 00:00:00',100.81,787,807,74.05,'Intermediate' from dual union all
select 22081,'022081','1992-053B','S3M','S3M',64765,'In Earth orbit','','1992-09-11 00:00:00',100.68,777,805,74.04,'Intermediate' from dual union all
select 22082,'022082','1992-048C','KDU','KDU',64760,'Reentered','1992-09-30 00:00:00','1992-09-01 00:00:00',89.94,228,315,82.33,'Intermediate' from dual union all
select 22083,'022083','1992-048D','Blenda','Sunshade',64760,'Reentered','1992-08-23 00:00:00','1992-08-23 00:00:00',87.53,139,165,82.3,'Intermediate' from dual union all
select 22084,'022084','1992-048E','KDU part','KDU part',64760,'Reentered','1992-08-18 00:00:00','1992-08-18 00:00:00',87.87,155,183,82.32,'Intermediate' from dual union all
select 22085,'022085','1992-048F','KDU part','KDU part',64760,'Reentered','1992-08-29 00:00:00','1992-08-29 00:00:00',88.47,162,236,82.3,'Intermediate' from dual union all
select 22086,'022086','1992-048G','KDU part','KDU part',64760,'Reentered','1992-08-21 00:00:00','1992-08-21 00:00:00',88.48,94,305,82.22,'Intermediate' from dual union all
select 22087,'022087','1992-054A','Optus B1','Optus B1',64766,'In Earth orbit','','1992-09-12 00:00:00',1436.21,35678,35899,.28,'Stationary' from dual union all
select 22088,'022088','1992-054B','CZ-2E Stage 2','CZ-2E Stage 2',64766,'Reentered','1992-12-12 00:00:00','1992-09-13 00:00:00',95.96,202,928,27.99,'Intermediate' from dual union all
select 22089,'022089','1992-054C','Star 63F','Star 63F',64766,'In Earth orbit','','1992-09-12 00:00:00',664.3,377,37305,23.38,'GEO Transfer' from dual union all
select 2209,'002209','1966-034D','OV3-1 despin weight','',13441,'Reentered','1989-12-04 00:00:00','1966-06-16 00:00:00',150.9,338,5684,82.47,'Medium' from dual union all
select 22090,'022090','1992-055A','Progress M-14','Progress 7K-TGM No. 209',64767,'Deorbited','1992-10-21 00:00:00','1992-09-14 00:00:00',92.66,403,407,51.62,'Intermediate' from dual union all
select 22091,'022091','1992-055B','11S510','11S510',64767,'Reentered','1992-08-18 00:00:00','1992-08-17 00:00:00',88.17,172,196,51.62,'Intermediate' from dual union all
select 22092,'022092','1992-055C','11S510 part','11S510 part',64767,'Reentered','1992-08-17 00:00:00','1992-08-16 00:00:00',88.55,169,237,51.61,'Intermediate' from dual union all
select 22093,'022093','1992-056A','Resurs-F','Resurs-F1 14F43 No. 54',64768,'Landed','1992-09-04 00:00:00','1992-08-26 00:00:00',89.06,222,234,82.57,'Intermediate' from dual union all
select 22094,'022094','1992-056B','11S510','11S510',64768,'Reentered','1992-08-21 00:00:00','1992-08-21 00:00:00',87.65,150,167,82.57,'Intermediate' from dual union all
select 22095,'022095','1992-050E','deb Blok-I','',64762,'Reentered','1992-09-08 00:00:00','1992-09-01 00:00:00',90.4,196,393,62.81,'Intermediate' from dual union all
select 22096,'022096','1992-057A','Satcom C4','Satcom C-4',64769,'In Earth orbit','','1992-10-06 00:00:00',1436.06,35764,35808,.11,'Stationary' from dual union all
select 22097,'022097','1992-057B','Delta 213','Delta SSPS AJ10-118K',64769,'In Earth orbit','','1992-10-12 00:00:00',131.84,1768,2638,25.19,'Medium' from dual union all
select 22098,'022098','1992-057C','Star 48B','Star 48B S/N 10071-18',64769,'In Earth orbit','','1992-10-27 00:00:00',661.89,1799,35762,20.31,'GEO Transfer' from dual union all
select 22099,'022099','1992-056C','Pion-Germes-1','Pion-Germes-1',64768,'Reentered','1992-09-25 00:00:00','1992-09-01 00:00:00',89.06,223,233,82.56,'Intermediate' from dual union all
select 221,'000221','62 Alpha 1','Ranger 3','NASA P-34',8160,'Deep Space','1962-01-28 00:00:00','',null,null,null,null,'' from dual union all
select 2210,'002210','1966-054A','Kosmos-121','Zenit-4 No. 20',13461,'Landed','1966-06-25 00:00:00','1966-06-21 00:00:00',89.78,203,325,72.85,'Intermediate' from dual union all
select 22100,'022100','1992-056D','Pion-Germes-2','Pion-Germes-2',64768,'Reentered','1992-09-24 00:00:00','1992-09-02 00:00:00',89.02,221,232,82.57,'Intermediate' from dual union all
select 22101,'022101','1988-063G','deb Ariane V24','',61312,'In Earth orbit','','1992-09-14 00:00:00',290.48,384,15916,7.88,'Highly Ellip' from dual union all
select 22102,'022102','1992-056E','KDU','KDU',64768,'Reentered','1992-09-13 00:00:00','1992-09-07 00:00:00',88.72,205,218,82.56,'Intermediate' from dual union all
select 22103,'022103','1992-056F','Blenda','Sunshade',64768,'Reentered','1992-09-07 00:00:00','1992-09-07 00:00:00',88.98,176,273,82.57,'Intermediate' from dual union all
select 22104,'022104','1992-056G','KDU part','KDU part',64768,'Reentered','1992-09-06 00:00:00','1992-09-06 00:00:00',88.78,177,251,82.55,'Intermediate' from dual union all
select 22105,'022105','1992-056H','KDU part','KDU part',64768,'Reentered','1992-09-07 00:00:00','1992-09-06 00:00:00',89.19,194,275,82.5,'Intermediate' from dual union all
select 22106,'022106','1986-017GJ','EO-12 EVA-2 debris (VDU)','',59210,'Reentered','1993-02-26 00:00:00','1992-09-16 00:00:00',92.62,400,406,51.62,'Intermediate' from dual union all
select 22107,'022107','1986-017GK','EO-12 EVA-2 debris (VDU)','',59210,'Reentered','1992-12-10 00:00:00','1992-09-15 00:00:00',92.57,398,403,51.63,'Intermediate' from dual union all
select 22108,'022108','1992-058A','GPS IIA-6','Navstar GPS SVN 27',64770,'In Earth orbit','','1992-10-11 00:00:00',717.98,19914,20451,54.77,'Medium' from dual union all
select 22109,'022109','1992-058B','Delta 214','Delta SSPS AJ10-118K',64770,'In Earth orbit','','1993-02-16 00:00:00',98.56,657,723,19.79,'Equatorial' from dual union all
select 2211,'002211','1966-054B','11A57I','11A57I',13461,'Reentered','1966-06-26 00:00:00','1966-06-19 00:00:00',89.46,189,307,72.85,'Intermediate' from dual union all
select 22110,'022110','1992-058C','Star 48B','Star 48B S/N 10070-18',64770,'Reentered','1996-12-10 00:00:00','1992-10-10 00:00:00',351.95,182,20087,34.69,'Highly Ellip' from dual union all
select 22111,'022111','1986-017GL','EO-12 EVA-2 debris (VDU)','',59210,'Reentered','1992-09-23 00:00:00','1992-09-13 00:00:00',92.12,372,385,51.62,'Intermediate' from dual union all
select 22112,'022112','1992-059A','Kosmos-2209','Oko-S F5',64771,'In Earth orbit','','1992-10-10 00:00:00',1436.06,35767,35805,1.28,'Stationary' from dual union all
select 22113,'022113','1992-059B','8S812','8S812',64771,'Reentered','1992-09-13 00:00:00','1992-09-13 00:00:00',87.24,135,141,51.64,'Intermediate' from dual union all
select 22114,'022114','1992-059C','Perekhodnik','Sredniy perekhodnik',64771,'Reentered','1992-09-11 00:00:00','1992-09-11 00:00:00',88.06,172,185,51.64,'Intermediate' from dual union all
select 22115,'022115','1992-059D','Blok DM2','11S861 No. 61L',64771,'In Earth orbit','','1992-10-10 00:00:00',1442.89,35890,35949,1.3,'Drift GEO' from dual union all
select 22116,'022116','1992-060A','Hispasat 1A','Hispasat 1A',64772,'In Earth orbit','','1992-10-16 00:00:00',1436.09,35778,35795,.06,'Stationary' from dual union all
select 22117,'022117','1992-060B','Satcom C3','Satcom C-3',64772,'In Earth orbit','','1992-10-12 00:00:00',1436.09,35775,35798,.08,'Stationary' from dual union all
select 22118,'022118','1992-060C','Ariane H10+ V53','Ariane H10+ S/N T425',64772,'Reentered','1993-10-20 00:00:00','1992-12-31 00:00:00',625.76,215,35498,6.81,'GEO Transfer' from dual union all
select 22119,'022119','1992-060D','Spelda 10 S/N UV16','Spelda 10',64772,'Reentered','1994-03-23 00:00:00','1992-10-11 00:00:00',624.27,229,35407,6.93,'GEO Transfer' from dual union all
select 2212,'002212','1966-054C','deb Kosmos-121','',13461,'Reentered','1966-06-19 00:00:00','1966-06-18 00:00:00',88.61,132,280,72.98,'Intermediate' from dual union all
select 22120,'022120','1992-061A','Endeavour (STS-47)','OV-105',64773,'Landed','1992-09-20 00:00:00','1992-09-14 00:00:00',90.59,298,309,57,'Intermediate' from dual union all
select 22121,'022121','1986-017GM','EO-12 EVA-3 debris (VDU)','',59210,'Reentered','1993-10-28 00:00:00','1992-09-16 00:00:00',92.65,403,406,51.63,'Intermediate' from dual union all
select 22122,'022122','1986-017GN','EO-12 EVA-3 debris (VDU)','',59210,'Reentered','1992-12-02 00:00:00','1992-09-13 00:00:00',92.57,399,402,51.63,'Intermediate' from dual union all
select 22123,'022123','1986-017GP','EO-12 EVA-3 debris (VDU)','',59210,'Reentered','1995-09-18 00:00:00','1992-09-13 00:00:00',92.69,405,408,51.63,'Intermediate' from dual union all
select 22124,'022124','1986-017GQ','EO-12 EVA-3 debris (VDU)','',59210,'Reentered','1992-11-03 00:00:00','1992-09-16 00:00:00',92.67,404,407,51.62,'Intermediate' from dual union all
select 22125,'022125','1992-059E','SOZ','SOZ',64771,'Reentered','1993-03-31 00:00:00','1993-03-02 00:00:00',478.05,111,27638,46.84,'Highly Ellip' from dual union all
select 22126,'022126','1992-059F','SOZ','SOZ',64771,'Reentered','1993-05-07 00:00:00','1992-10-10 00:00:00',650.83,297,36702,47.45,'GEO Transfer' from dual union all
select 22127,'022127','1986-017GR','EO-12 EVA-3 debris (VDU)','',59210,'Reentered','1995-06-14 00:00:00','1992-09-14 00:00:00',92.69,400,413,51.69,'Intermediate' from dual union all
select 22128,'022128','1986-017GS','EO-12 EVA-3 debris (VDU)','',59210,'Reentered','1995-06-09 00:00:00','1992-09-13 00:00:00',92.66,404,406,51.64,'Intermediate' from dual union all
select 22129,'022129','1986-017GT','VDU container','Large piece (10 sq m RCS',59210,'Reentered','1992-11-17 00:00:00','1992-09-16 00:00:00',92.63,401,406,51.62,'Intermediate' from dual union all
select 2213,'002213','1966-054D','deb Kosmos-121','',13461,'Reentered','1966-06-20 00:00:00','1966-06-18 00:00:00',89.25,194,281,72.94,'Intermediate' from dual union all
select 22130,'022130','1986-017GU','EO-12 EVA-4 debris (Kurs)','',59210,'Reentered','1992-11-01 00:00:00','1992-09-16 00:00:00',92.59,399,404,51.63,'Intermediate' from dual union all
select 22131,'022131','1986-017GV','EO-12 EVA-4 debris (Kurs)','',59210,'Reentered','1992-11-12 00:00:00','1992-09-20 00:00:00',92.56,398,402,51.63,'Intermediate' from dual union all
select 22132,'022132','1991-009CR','deb S3M','',64178,'In Earth orbit','','1991-04-18 00:00:00',118.36,1495,1720,74.03,'Intermediate' from dual union all
select 22133,'022133','1992-062A','Kosmos-2210','Kobal''t No. 533',64774,'Landed','1992-11-20 00:00:00','1992-09-29 00:00:00',89.53,172,331,67.16,'Intermediate' from dual union all
select 22134,'022134','1992-062B','11S510','11S510',64774,'Reentered','1992-09-26 00:00:00','1992-09-24 00:00:00',89.06,156,300,67.15,'Intermediate' from dual union all
select 22135,'022135','1992-045C','Kosmos-2203 part','Kobal''t AO part',64757,'Reentered','1992-09-23 00:00:00','1992-09-22 00:00:00',89.23,193,280,62.81,'Intermediate' from dual union all
select 22136,'022136','1992-063A','Mars Observer','Mars Observer',64775,'Deep Space','1992-09-26 00:00:00','1992-09-25 00:00:00',986.83,287,78710,29.3,'Earth Escape' from dual union all
select 22137,'022137','1992-063B','Titan CT-4 Stage 2','LR91AJ11 CT-4',64775,'Reentered','1992-10-01 00:00:00','1992-09-28 00:00:00',90.18,138,429,29.38,'Intermediate' from dual union all
select 22138,'022138','1992-063C','TOS ''USS Thomas O. Paine''','Orbus 21',64775,'Deep Space','1992-09-26 00:00:00','1992-09-25 00:00:00',986.83,287,78710,29.3,'Earth Escape' from dual union all
select 22139,'022139','1979-057D','deb NOAA 6','',47541,'Reentered','1993-05-12 00:00:00','1992-10-03 00:00:00',100.73,782,805,98.6,'Sun-Synch' from dual union all
select 2214,'002214','1966-054E','deb Kosmos-121','',13461,'Reentered','1966-06-20 00:00:00','1966-06-18 00:00:00',89.69,198,321,72.85,'Intermediate' from dual union all
select 22140,'022140','1984-106H','deb SOZ','',57160,'Reentered','1992-12-06 00:00:00','1992-10-06 00:00:00',102.95,739,1058,66.46,'Intermediate' from dual union all
select 22141,'022141','1984-106J','deb SOZ','',57160,'Reentered','1992-10-11 00:00:00','1992-10-06 00:00:00',97.49,547,731,66.88,'Intermediate' from dual union all
select 22142,'022142','1984-106K','deb SOZ','',57160,'Reentered','1992-10-10 00:00:00','1992-10-05 00:00:00',97.57,591,694,66.77,'Intermediate' from dual union all
select 22143,'022143','1984-106L','deb SOZ','',57160,'Reentered','1992-10-11 00:00:00','1992-10-06 00:00:00',102.06,779,934,67.47,'Intermediate' from dual union all
select 22144,'022144','1984-106M','deb SOZ','',57160,'Reentered','1992-10-19 00:00:00','1992-10-06 00:00:00',102.39,713,1031,67,'Intermediate' from dual union all
select 22145,'022145','1984-106N','deb SOZ','',57160,'Reentered','1992-11-05 00:00:00','1992-10-06 00:00:00',103.12,647,1166,66.62,'Intermediate' from dual union all
select 22146,'022146','1984-106P','deb SOZ','',57160,'Reentered','1993-03-27 00:00:00','1992-10-06 00:00:00',103.94,749,1141,66.33,'Intermediate' from dual union all
select 22147,'022147','1984-106Q','deb SOZ','',57160,'Reentered','1992-10-08 00:00:00','1992-10-06 00:00:00',98.08,611,723,66.53,'Intermediate' from dual;
insert into SATELLITE
select 22148,'022148','1984-106R','deb SOZ','',57160,'Reentered','1992-11-13 00:00:00','1992-10-06 00:00:00',103.03,783,1021,66.6,'Intermediate' from dual union all
select 22149,'022149','1984-106S','deb SOZ','',57160,'Reentered','1992-10-11 00:00:00','1992-10-06 00:00:00',101.09,595,1026,67.15,'Intermediate' from dual union all
select 2215,'002215','1966-053B','OPS 9311','IDCSP 1',13460,'In Earth orbit','','1966-12-16 00:00:00',1334.51,33648,33900,.33,'Medium' from dual union all
select 22150,'022150','1984-106T','deb SOZ','',57160,'Reentered','1992-10-14 00:00:00','1992-10-05 00:00:00',100.08,661,863,66.78,'Intermediate' from dual union all
select 22151,'022151','1984-106U','deb SOZ','',57160,'Reentered','1992-10-14 00:00:00','1992-10-06 00:00:00',103.21,625,1196,67.1,'Intermediate' from dual union all
select 22152,'022152','1984-106V','deb SOZ','',57160,'Reentered','1992-11-16 00:00:00','1992-10-05 00:00:00',102.53,766,990,66.76,'Intermediate' from dual union all
select 22153,'022153','1984-106W','deb SOZ','',57160,'Reentered','1992-10-23 00:00:00','1992-10-06 00:00:00',102.99,727,1073,66.37,'Intermediate' from dual union all
select 22154,'022154','1984-106X','deb SOZ','',57160,'Reentered','1992-10-20 00:00:00','1992-10-06 00:00:00',100.38,598,956,66.95,'Intermediate' from dual union all
select 22155,'022155','1984-106Y','deb SOZ','',57160,'Reentered','1992-10-22 00:00:00','1992-10-06 00:00:00',99.5,578,892,66.96,'Intermediate' from dual union all
select 22156,'022156','1984-106Z','deb SOZ','',57160,'Reentered','1992-10-14 00:00:00','1992-10-06 00:00:00',102.16,803,919,67.35,'Intermediate' from dual union all
select 22157,'022157','1984-106AA','deb SOZ','',57160,'Reentered','1992-10-10 00:00:00','1992-10-05 00:00:00',98.38,594,768,67.01,'Intermediate' from dual union all
select 22158,'022158','1984-106AB','deb SOZ','',57160,'Reentered','1992-11-20 00:00:00','1992-10-06 00:00:00',101.28,611,1028,66.81,'Intermediate' from dual union all
select 22159,'022159','1984-106AC','deb SOZ','',57160,'Reentered','1992-10-11 00:00:00','1992-10-06 00:00:00',97.3,488,770,66.93,'Intermediate' from dual union all
select 2216,'002216','1966-053C','OPS 9312','IDCSP 2',13460,'In Earth orbit','','1967-01-20 00:00:00',1335.26,33671,33907,.51,'Medium' from dual union all
select 22160,'022160','1984-106AD','deb SOZ','',57160,'Reentered','1992-10-13 00:00:00','1992-10-06 00:00:00',102.3,542,1193,67.11,'Intermediate' from dual union all
select 22161,'022161','1992-064A','Freja','Freja',64776,'In Earth orbit','','1992-11-08 00:00:00',108.95,596,1760,63.01,'Intermediate' from dual union all
select 22162,'022162','1992-064B','Fanhui Shi Weixing (14)','Jianbing-1A No. 4',64776,'Reentered','1992-10-31 00:00:00','1992-10-13 00:00:00',89.64,215,299,63.01,'Intermediate' from dual union all
select 22163,'022163','1992-064C','CZ-2C Stage 2','CZ-2C Stage 2',64776,'Reentered','1992-10-20 00:00:00','1992-10-09 00:00:00',89.55,209,296,63,'Intermediate' from dual union all
select 22164,'022164','1992-064D','Freja upper adapter','',64776,'Reentered','1992-10-09 00:00:00','1992-10-07 00:00:00',89.2,200,269,63,'Intermediate' from dual union all
select 22165,'022165','1992-064E','CZ-2C sep motor cover','',64776,'Reentered','1992-10-08 00:00:00','1992-10-07 00:00:00',89.22,195,277,63.01,'Intermediate' from dual union all
select 22166,'022166','1992-064F','CZ-2C sep motor cover','',64776,'Reentered','1992-10-09 00:00:00','1992-10-08 00:00:00',89.92,185,356,63.3,'Intermediate' from dual union all
select 22167,'022167','1992-064G','CZ-2C sep motor cover','',64776,'Reentered','1992-10-08 00:00:00','1992-10-07 00:00:00',90.36,188,397,63.25,'Intermediate' from dual union all
select 22168,'022168','1992-064H','CZ-2C sep motor cover','',64776,'Reentered','1992-10-10 00:00:00','1992-10-09 00:00:00',88.35,159,227,62.74,'Intermediate' from dual union all
select 22169,'022169','1992-064J','Freja adapter part','',64776,'Reentered','1992-10-09 00:00:00','1992-10-09 00:00:00',89.17,180,287,62.81,'Intermediate' from dual union all
select 2217,'002217','1966-053D','OPS 9313','IDCSP 3',13460,'In Earth orbit','','1969-06-01 00:00:00',1336.38,33702,33921,2.66,'Medium' from dual union all
select 22170,'022170','1992-064K','Freja adapter part','',64776,'Reentered','1992-10-09 00:00:00','1992-10-09 00:00:00',88.36,171,216,62.68,'Intermediate' from dual union all
select 22171,'022171','1990-050H','deb USA-59','',63292,'In Earth orbit','','1992-10-10 00:00:00',124.08,1225,2500,63.4,'Medium' from dual union all
select 22172,'022172','1967-034H','NNS O-12 insulation','',15289,'Reentered','2001-10-31 00:00:00','1992-10-13 00:00:00',106.24,1032,1073,90.12,'Polar' from dual union all
select 22173,'022173','1992-065A','Foton','Foton 34KS No. 8L',64777,'Landed','1992-10-24 00:00:00','1992-10-15 00:00:00',90.2,217,352,62.81,'Intermediate' from dual union all
select 22174,'022174','1992-065B','11S510','11S510',64777,'Reentered','1992-10-22 00:00:00','1992-10-12 00:00:00',89.88,212,325,62.8,'Intermediate' from dual union all
select 22175,'022175','1992-066A','Kopernikus 3','DFS 3',64778,'In Earth orbit','','1992-11-13 00:00:00',1435.88,35775,35790,.02,'Stationary' from dual union all
select 22176,'022176','1992-066B','Delta 215','Delta SSPS AJ10-118K',64778,'In Earth orbit','','1993-02-25 00:00:00',132.64,1406,3068,25.08,'Medium' from dual union all
select 22177,'022177','1992-066C','Star 48B','Star 48B S/N 10078-18',64778,'In Earth orbit','','1992-11-12 00:00:00',658.32,1526,35854,19.8,'GEO Transfer' from dual union all
select 22178,'022178','1992-067A','Molniya-3','Molniya-3 No. 50',64779,'In Earth orbit','','1992-11-13 00:00:00',717.74,495,39858,62.82,'Molniya' from dual union all
select 22179,'022179','1992-067B','11S510','11S510',64779,'Reentered','1992-10-28 00:00:00','1992-10-19 00:00:00',90.04,205,348,62.82,'Intermediate' from dual union all
select 2218,'002218','1966-053E','OPS 9314','IDCSP 4',13460,'In Earth orbit','','1967-01-21 00:00:00',1338.52,33715,33994,.52,'Medium' from dual union all
select 22180,'022180','1992-067C','BOZ','BOZ',64779,'Reentered','1992-10-24 00:00:00','1992-10-19 00:00:00',90.09,176,382,62.81,'Intermediate' from dual union all
select 22181,'022181','1992-067D','Blok-ML','Blok-ML No. 76753-541',64779,'In Earth orbit','','1992-11-14 00:00:00',733.61,492,40640,62.85,'Molniya' from dual union all
select 22182,'022182','1992-068A','Kosmos-2211','Strela-3',64780,'In Earth orbit','','1992-12-21 00:00:00',113.93,1401,1413,82.59,'Intermediate' from dual union all
select 22183,'022183','1992-068B','Kosmos-2212','Strela-3',64780,'In Earth orbit','','1992-11-21 00:00:00',114.02,1407,1414,82.6,'Intermediate' from dual union all
select 22184,'022184','1992-068C','Kosmos-2213','Strela-3',64780,'In Earth orbit','','1992-12-30 00:00:00',114.06,1412,1413,82.6,'Intermediate' from dual union all
select 22185,'022185','1992-068D','Kosmos-2214','Strela-3',64780,'In Earth orbit','','1992-12-28 00:00:00',114.16,1412,1423,82.59,'Intermediate' from dual union all
select 22186,'022186','1992-068E','Kosmos-2215','Strela-3',64780,'In Earth orbit','','1992-11-22 00:00:00',114.22,1412,1427,82.6,'Intermediate' from dual union all
select 22187,'022187','1992-068F','Kosmos-2216','Strela-3',64780,'In Earth orbit','','1992-11-20 00:00:00',114.06,1411,1415,82.6,'Intermediate' from dual union all
select 22188,'022188','1992-068G','S5M','S5M',64780,'In Earth orbit','','1992-11-22 00:00:00',114.74,1409,1479,82.6,'Intermediate' from dual union all
select 22189,'022189','1992-069A','Kosmos-2217','Oko',64781,'Reentered','2010-11-06 00:00:00','1992-11-20 00:00:00',718,646,39720,62.98,'Molniya' from dual union all
select 2219,'002219','1966-053F','OPS 9315','IDCSP 5',13460,'In Earth orbit','','1968-10-05 00:00:00',1340.92,33570,34235,1.66,'Medium' from dual union all
select 22190,'022190','1992-069B','11S510','11S510',64781,'Reentered','1992-11-25 00:00:00','1992-10-28 00:00:00',92.15,210,550,62.82,'Intermediate' from dual union all
select 22191,'022191','1992-069C','BOZ','BOZ',64781,'Reentered','1992-11-07 00:00:00','1992-10-28 00:00:00',91.54,177,524,62.85,'Intermediate' from dual union all
select 22192,'022192','1992-069D','Blok-2BL','Blok-2BL No. 77347-690',64781,'In Earth orbit','','1992-11-22 00:00:00',709.91,664,39302,63.19,'Molniya' from dual union all
select 22193,'022193','1992-067E','deb Blok-I','',64779,'Reentered','1992-10-28 00:00:00','1992-10-21 00:00:00',90.34,202,380,62.81,'Intermediate' from dual union all
select 22194,'022194','1992-070A','Columbia (STS-52)','OV-102',64782,'Landed','1992-11-01 00:00:00','1992-10-24 00:00:00',90.29,285,292,28.47,'Intermediate' from dual union all
select 22195,'022195','1992-070B','Lageos 2','Lageos 2',64782,'In Earth orbit','','1992-11-24 00:00:00',222.44,5616,5949,52.65,'Medium' from dual union all
select 22196,'022196','1992-070D','IRIS','IRIS',64782,'In Earth orbit','','1993-02-16 00:00:00',152.52,291,5864,41.18,'Medium' from dual union all
select 22197,'022197','1992-070E','Lageos Apogee Stage','MAGE 1SC/LAS',64782,'In Earth orbit','','1993-02-25 00:00:00',222.41,5615,5949,52.66,'Medium' from dual union all
select 22198,'022198','1991-009CS','deb S3M','',64178,'In Earth orbit','','1992-11-21 00:00:00',117.06,1434,1663,73.92,'Intermediate' from dual union all
select 22199,'022199','1992-065C','KB electropitaniya','KBE',64777,'Reentered','1992-11-20 00:00:00','1992-10-26 00:00:00',89.99,214,334,62.81,'Intermediate' from dual union all
select 222,'000222','62 Alpha 2','Agena B 6003','Agena B 6003',8160,'Deep Space','1962-01-28 00:00:00','1962-01-26 00:00:00',91.6,200,507,29.4,'Intermediate' from dual union all
select 2220,'002220','1966-053G','OPS 9316','IDCSP 6',13460,'In Earth orbit','','1966-11-03 00:00:00',1343.89,33678,34246,.25,'Medium' from dual union all
select 22200,'022200','1992-065D','KBE part','KBE part',64777,'Reentered','1992-10-31 00:00:00','1992-10-26 00:00:00',89.64,207,306,62.81,'Intermediate' from dual union all
select 22201,'022201','1992-065E','PTDU cover','KBE part',64777,'Reentered','1992-10-30 00:00:00','1992-10-26 00:00:00',91.52,286,413,62.79,'Intermediate' from dual union all
select 22202,'022202','1992-065F','PTDU cover','KBE part',64777,'Reentered','1992-11-03 00:00:00','1992-10-25 00:00:00',91.8,294,432,62.83,'Intermediate' from dual union all
select 22203,'022203','1992-071A','Progress M-15','Progress 7K-TGM No. 215',64783,'Deorbited','1993-02-07 00:00:00','1992-11-30 00:00:00',92.5,396,399,51.62,'Intermediate' from dual union all
select 22204,'022204','1992-071B','11S510','11S510',64783,'Reentered','1992-10-28 00:00:00','1992-10-28 00:00:00',88.19,177,193,51.61,'Intermediate' from dual union all
select 22205,'022205','1992-072A','Galaxy 7','Galaxy VII',64784,'In Earth orbit','','1992-11-28 00:00:00',1436.06,35782,35790,.1,'Stationary' from dual union all
select 22206,'022206','1992-072B','Ariane H10+ V54','Ariane H10+ S/N T427',64784,'Reentered','1994-08-26 00:00:00','1993-03-02 00:00:00',445.89,196,25715,7.09,'Highly Ellip' from dual union all
select 22207,'022207','1992-073A','Kosmos-2218','Parus',64785,'In Earth orbit','','1992-12-31 00:00:00',104.92,966,1016,82.92,'Intermediate' from dual union all
select 22208,'022208','1992-073B','S3M','S3M',64785,'In Earth orbit','','1992-12-23 00:00:00',104.76,959,1008,82.92,'Intermediate' from dual union all
select 22209,'022209','1986-017GW','Garbage bag','',59210,'Reentered','1993-04-09 00:00:00','1992-11-03 00:00:00',92.38,389,394,51.63,'Intermediate' from dual union all
select 2221,'002221','1966-053H','OPS 9317','IDCSP 7',13460,'In Earth orbit','','1966-12-18 00:00:00',1347.54,33702,34368,.41,'Medium' from dual union all
select 22210,'022210','1992-074A','Ekran-M','Ekran-M No. 15L',64786,'In Earth orbit','','1992-11-29 00:00:00',1436.06,35764,35808,1.45,'Stationary' from dual union all
select 22211,'022211','1992-074B','8S812','8S812',64786,'Reentered','1992-11-01 00:00:00','1992-11-01 00:00:00',87.65,149,168,51.66,'Intermediate' from dual union all
select 22212,'022212','1992-074C','Perekhodnik','Sredniy perekhodnik',64786,'Reentered','1992-10-31 00:00:00','1992-10-31 00:00:00',87.79,162,169,51.65,'Intermediate' from dual union all
select 22213,'022213','1992-074D','Blok DM2','11S861 No. 58L',64786,'In Earth orbit','','1993-04-11 00:00:00',1423.61,35469,35615,1.17,'Drift GEO' from dual union all
select 22214,'022214','1992-070C','CTA','Canadian Target Assembly',64782,'Reentered','1992-11-01 00:00:00','1992-10-31 00:00:00',88.71,207,215,28.46,'Intermediate' from dual union all
select 22215,'022215','1992-074E','SOZ','SOZ',64786,'Reentered','1994-03-19 00:00:00','1992-11-29 00:00:00',629.18,290,35599,47.37,'GEO Transfer' from dual union all
select 22216,'022216','1992-074F','SOZ','SOZ',64786,'Reentered','1993-06-03 00:00:00','1992-11-29 00:00:00',627.7,243,35569,47.34,'GEO Transfer' from dual union all
select 22217,'022217','1992-075A','Resurs-500','Resurs-500 No. 1',64787,'Landed','1992-11-22 00:00:00','1992-11-17 00:00:00',88.66,179,238,82.57,'Intermediate' from dual union all
select 22218,'022218','1992-075B','11S510','11S510',64787,'Reentered','1992-11-18 00:00:00','1992-11-17 00:00:00',88.33,177,207,82.58,'Intermediate' from dual union all
select 22219,'022219','1992-076A','Kosmos-2219','Tselina-2',64788,'In Earth orbit','','1992-12-17 00:00:00',101.96,848,855,71.01,'Intermediate' from dual union all
select 2222,'002222','1966-053J','Transtage 11','Transtage 11/ICDSP truss',13460,'In Earth orbit','','1975-07-18 00:00:00',1351.67,33720,34515,7.58,'Medium' from dual union all
select 22220,'022220','1992-076B','11S772','11S772',64788,'In Earth orbit','','1992-12-17 00:00:00',101.75,831,852,71,'Intermediate' from dual union all
select 22221,'022221','1992-076C','Sep Motor Cover','Sep Motor Cover',64788,'In Earth orbit','','1992-12-20 00:00:00',104.71,844,1118,71.04,'Intermediate' from dual union all
select 22222,'022222','1992-076D','Sep Motor Cover','Sep Motor Cover',64788,'In Earth orbit','','1992-12-18 00:00:00',105.25,845,1168,71,'Intermediate' from dual union all
select 22223,'022223','1992-076E','Sep Motor Cover','Sep Motor Cover',64788,'In Earth orbit','','1992-12-19 00:00:00',104.85,843,1132,71.03,'Intermediate' from dual union all
select 22224,'022224','1992-076F','Sep Motor Cover','Sep Motor Cover',64788,'In Earth orbit','','1992-12-21 00:00:00',104.9,845,1135,70.99,'Intermediate' from dual union all
select 22225,'022225','1986-017GX','MAK-2','MAK 2',59210,'Reentered','1993-04-01 00:00:00','1992-11-29 00:00:00',92.37,389,392,51.63,'Intermediate' from dual union all
select 22226,'022226','1992-077A','Kosmos-2220','Kobal''t No. 544',64789,'Landed','1993-01-18 00:00:00','1992-11-27 00:00:00',89.37,151,336,67.13,'Intermediate' from dual union all
select 22227,'022227','1992-077B','11S510','11S510',64789,'Reentered','1992-11-24 00:00:00','1992-11-23 00:00:00',88.71,153,269,67.13,'Intermediate' from dual union all
select 22228,'022228','1986-017GY','Garbage bag','',59210,'Reentered','1992-12-08 00:00:00','1992-11-30 00:00:00',91.57,346,358,51.62,'Intermediate' from dual union all
select 22229,'022229','1992-078A','MSTI-1','MSTI-1',64790,'Reentered','1993-07-18 00:00:00','1992-12-21 00:00:00',91.68,308,406,96.75,'Sun-Synch' from dual union all
select 2223,'002223','1966-016C','ESSA 2 despin weight','',13423,'In Earth orbit','','1966-06-17 00:00:00',111.9,1242,1386,100.91,'Sun-Synch' from dual union all
select 22230,'022230','1992-078B','Star 20 E-47','Star 20 E-47',64790,'Reentered','1993-02-09 00:00:00','1992-12-21 00:00:00',92.11,324,432,96.76,'Sun-Synch' from dual union all
select 22231,'022231','1992-079A','GPS IIA-7','Navstar GPS SVN 32',64791,'In Earth orbit','','1992-12-23 00:00:00',717.96,20074,20290,54.84,'Medium' from dual union all
select 22232,'022232','1992-079B','Delta 216','Delta SSPS AJ10-118K',64791,'Reentered','2013-07-31 00:00:00','1992-12-22 00:00:00',97.17,527,720,21.22,'Equatorial' from dual union all
select 22233,'022233','1992-079C','Star 48B','Star 48B S/N 10072-18',64791,'Reentered','2001-03-24 00:00:00','1992-12-22 00:00:00',351.63,181,20068,34.64,'Highly Ellip' from dual union all
select 22234,'022234','1992-075C','KDU','KDU',64787,'Reentered','1992-11-30 00:00:00','1992-11-23 00:00:00',89.18,181,286,82.58,'Intermediate' from dual union all
select 22235,'022235','1992-075D','Blenda','Sunshade',64787,'Reentered','1992-11-28 00:00:00','1992-11-24 00:00:00',91.17,287,377,82.58,'Intermediate' from dual union all
select 22236,'022236','1992-080A','Kosmos-2221','Tselina-D',64792,'In Earth orbit','','1993-03-12 00:00:00',97.71,635,663,82.51,'Intermediate' from dual union all
select 22237,'022237','1992-080B','S5M','S5M',64792,'In Earth orbit','','1993-03-13 00:00:00',97.69,634,663,82.51,'Intermediate' from dual;
insert into SATELLITE
select 22238,'022238','1992-081A','Kosmos-2222','Oko',64793,'In Earth orbit','','1992-12-25 00:00:00',717.96,640,39724,62.89,'Molniya' from dual union all
select 22239,'022239','1992-081B','11S510','11S510',64793,'Reentered','1992-12-28 00:00:00','1992-12-02 00:00:00',91.86,208,524,62.79,'Intermediate' from dual union all
select 2224,'002224','1966-016D','ESSA 2 despin weight','',13423,'In Earth orbit','','1966-06-17 00:00:00',115.05,1349,1566,101.05,'Sun-Synch' from dual union all
select 22240,'022240','1992-081C','BOZ','BOZ',64793,'Reentered','1992-12-14 00:00:00','1992-12-02 00:00:00',91.5,175,522,62.79,'Intermediate' from dual union all
select 22241,'022241','1992-081D','Blok-2BL','Blok-2BL No. 76719-683',64793,'In Earth orbit','','1992-12-27 00:00:00',707.77,650,39210,63.08,'Molniya' from dual union all
select 22242,'022242','1992-081E','deb Blok-I','',64793,'Reentered','1992-12-15 00:00:00','1992-12-02 00:00:00',91.82,194,533,62.8,'Intermediate' from dual union all
select 22243,'022243','1992-081F','deb Blok-I','',64793,'Reentered','1992-11-29 00:00:00','1992-11-28 00:00:00',90.62,160,450,62.8,'Intermediate' from dual union all
select 22244,'022244','1992-081G','deb Blok-I','',64793,'Reentered','1992-12-09 00:00:00','1992-11-29 00:00:00',91.85,183,547,62.79,'Intermediate' from dual union all
select 22245,'022245','1992-082A','Gorizont','Gorizont No. 38L',64794,'In Earth orbit','','1993-01-10 00:00:00',1436.09,35776,35797,1.37,'Stationary' from dual union all
select 22246,'022246','1992-082B','8S812','8S812',64794,'Reentered','1992-11-30 00:00:00','1992-11-29 00:00:00',87.93,170,173,51.65,'Intermediate' from dual union all
select 22247,'022247','1992-082C','Perekhodnik','Sredniy perekhodnik',64794,'Reentered','1992-11-28 00:00:00','1992-11-27 00:00:00',88.12,176,187,51.64,'Intermediate' from dual union all
select 22248,'022248','1992-082D','Blok DM2 No. 65L','11S861 No. 65L',64794,'In Earth orbit','','1992-12-30 00:00:00',1468.96,36373,36482,1.4,'Drift GEO' from dual union all
select 22249,'022249','1992-082E','SOZ','SOZ',64794,'Reentered','1993-07-14 00:00:00','1992-12-30 00:00:00',645.99,267,36485,47.18,'GEO Transfer' from dual union all
select 2225,'002225','1966-041C','NNS O-9 despin weight','',13448,'Reentered','2012-04-14 00:00:00','1966-07-02 00:00:00',101.21,771,861,89.89,'Polar' from dual union all
select 22250,'022250','1992-082F','SOZ','SOZ',64794,'Reentered','2001-10-02 00:00:00','1992-12-31 00:00:00',647.58,322,36510,47.21,'GEO Transfer' from dual union all
select 22251,'022251','1992-083A','USA 86','Enhanced CRYSTAL 2104',64795,'Deorbited','2000-06-05 00:00:00','1993-07-02 00:00:00',97.48,264,1013,97.88,'Sun-Synch' from dual union all
select 22252,'022252','1992-083B','Titan 45K-3 Stage 2','LR91AJ11A 45K-3',64795,'Reentered','1995-01-01 00:00:00','1992-11-28 00:00:00',96.34,256,911,97.7,'Sun-Synch' from dual union all
select 22253,'022253','1992-084A','Superbird A1','Superbird A1',64796,'In Earth orbit','','1993-01-01 00:00:00',1436.06,35774,35798,.06,'Stationary' from dual union all
select 22254,'022254','1992-084B','Ariane H10+ V55','Ariane H10+ S/N T428',64796,'Reentered','2015-05-17 00:00:00','1993-02-26 00:00:00',632.52,256,35804,6.71,'GEO Transfer' from dual union all
select 22255,'022255','1992-085A','Molniya-3','Molniya-3 No. 56',64797,'Reentered','2008-11-08 00:00:00','1993-01-02 00:00:00',717.72,408,39944,62.86,'Molniya' from dual union all
select 22256,'022256','1992-085B','11S510','11S510',64797,'Reentered','1992-12-14 00:00:00','1992-12-04 00:00:00',90.28,206,371,62.78,'Intermediate' from dual union all
select 22257,'022257','1992-085C','BOZ','BOZ',64797,'Reentered','1992-12-11 00:00:00','1992-12-04 00:00:00',90.57,178,427,62.82,'Intermediate' from dual union all
select 22258,'022258','1992-085D','Blok-ML','Blok-ML No. 76723-540',64797,'Reentered','2007-11-23 00:00:00','1993-01-03 00:00:00',697.79,405,38959,62.91,'Molniya' from dual union all
select 22259,'022259','1992-086A','Discovery (STS-53)','OV-103',64798,'Landed','1992-12-09 00:00:00','1992-12-04 00:00:00',91,317,331,57,'Intermediate' from dual union all
select 2226,'002226','1965-109D','NNS O-6 despin weight','Transit high deb',11859,'In Earth orbit','','1966-06-17 00:00:00',107.34,904,1303,89.09,'Polar' from dual union all
select 22260,'022260','1992-087A','Kosmos-2223','Neman',64799,'Deorbited','1993-12-16 00:00:00','1993-01-08 00:00:00',89.78,239,289,64.65,'Intermediate' from dual union all
select 22261,'022261','1992-087B','11S510','11S510',64799,'Reentered','1992-12-12 00:00:00','1992-12-11 00:00:00',88.32,161,222,64.67,'Intermediate' from dual union all
select 22262,'022262','1992-085E','deb Blok-I','',64797,'Reentered','1992-12-15 00:00:00','1992-12-10 00:00:00',89.83,190,343,62.82,'Intermediate' from dual union all
select 22263,'022263','1989-061E','deb USA-40','',62338,'In Earth orbit','','1998-09-11 00:00:00',182,563,7958,56.93,'Medium' from dual union all
select 22264,'022264','1989-061F','deb USA-40','',62338,'In Earth orbit','','1998-11-17 00:00:00',181.48,553,7927,56.78,'Medium' from dual union all
select 22265,'022265','1989-061G','deb USA-40','',62338,'In Earth orbit','','1998-08-25 00:00:00',176.25,435,7635,57.2,'Medium' from dual union all
select 22266,'022266','1984-080E','Star 27 S/N 0062','Star 27 TE-M-616-5',57134,'In Earth orbit','','1992-12-01 00:00:00',1443.55,35338,36526,5.74,'Inclined Drift' from dual union all
select 22267,'022267','1989-061H','deb USA-40','',62338,'In Earth orbit','','1998-08-24 00:00:00',181.99,513,8007,57.21,'Medium' from dual union all
select 22268,'022268','1989-061J','deb USA-40','',62338,'In Earth orbit','','1998-08-08 00:00:00',181.05,509,7938,57,'Medium' from dual union all
select 22269,'022269','1992-088A','Kosmos-2224','Oko-1 No. 7121',64800,'In Earth orbit','','1993-01-20 00:00:00',1448.08,35972,36070,2.23,'Drift GEO' from dual union all
select 2227,'002227','1966-055A','OPS 1599','CORONA J-31',13462,'Reentered Attached','1966-07-14 00:00:00','1966-06-29 00:00:00',89.98,193,354,80.09,'Intermediate' from dual union all
select 22270,'022270','1992-088B','8S812','8S812',64800,'Reentered','1992-12-19 00:00:00','1992-12-18 00:00:00',88.15,181,185,51.63,'Intermediate' from dual union all
select 22271,'022271','1992-088C','Perekhodnik','Sredniy perekhodnik',64800,'Reentered','1992-12-18 00:00:00','1992-12-17 00:00:00',88.26,178,199,51.62,'Intermediate' from dual union all
select 22272,'022272','1992-088D','Blok DM2','11S861 No. 64L',64800,'In Earth orbit','','1993-03-20 00:00:00',1438.95,35803,35882,2.13,'Drift GEO' from dual union all
select 22273,'022273','1992-088E','SOZ','SOZ',64800,'Reentered','2004-06-27 00:00:00','1993-01-18 00:00:00',635.61,335,35885,47.29,'GEO Transfer' from dual union all
select 22274,'022274','1992-088F','SOZ','SOZ',64800,'Reentered','2006-05-04 00:00:00','1993-01-17 00:00:00',634.13,279,35865,47.23,'GEO Transfer' from dual union all
select 22275,'022275','1992-089A','GPS IIA-8','Navstar GPS SVN 29',64801,'In Earth orbit','','1993-01-25 00:00:00',717.94,20037,20326,54.77,'Medium' from dual union all
select 22276,'022276','1992-089B','Delta 217','Delta SSPS AJ10-118K',64801,'In Earth orbit','','1993-01-20 00:00:00',98.18,611,733,20.29,'Equatorial' from dual union all
select 22277,'022277','1992-089C','Star 48B','Star 48B S/N 10074-18',64801,'Reentered','2013-07-22 00:00:00','1993-01-17 00:00:00',352.68,85,20230,34.89,'Highly Ellip' from dual union all
select 22278,'022278','1992-090A','Star 63F','Star 63F',64802,'Reentered','1995-06-29 00:00:00','1993-01-20 00:00:00',96.94,207,1018,28.14,'Intermediate' from dual union all
select 22279,'022279','1992-090B','CZ-2E Stage 2','CZ-2E Stage 2',64802,'Reentered','1993-06-19 00:00:00','1993-01-21 00:00:00',96.36,205,963,28.13,'Intermediate' from dual union all
select 2228,'002228','1966-046E','deb Atlas 5304','',13453,'Reentered','1966-06-30 00:00:00','1966-06-21 00:00:00',89.5,232,268,28.58,'Intermediate' from dual union all
select 22280,'022280','1992-091A','Kosmos-2225','Don',64803,'Deorbited','1993-02-18 00:00:00','1992-12-29 00:00:00',89.53,209,294,64.9,'Intermediate' from dual union all
select 22281,'022281','1992-091B','11S510','11S510',64803,'Reentered','1992-12-26 00:00:00','1992-12-24 00:00:00',88.74,160,265,64.91,'Intermediate' from dual union all
select 22282,'022282','1992-092A','Kosmos-2226','Musson No. 23',64804,'In Earth orbit','','1993-01-21 00:00:00',116.01,1478,1525,73.62,'Intermediate' from dual union all
select 22283,'022283','1992-092B','S5M','S5M',64804,'In Earth orbit','','1993-03-19 00:00:00',115.35,1425,1518,73.62,'Intermediate' from dual union all
select 22284,'022284','1992-093A','Kosmos-2227','Tselina-2',64805,'In Earth orbit','','1993-01-25 00:00:00',101.95,848,854,71.02,'Intermediate' from dual union all
select 22285,'022285','1992-093B','11S772','11S772',64805,'In Earth orbit','','1993-01-31 00:00:00',101.84,843,848,71.02,'Intermediate' from dual union all
select 22286,'022286','1992-094A','Kosmos-2228','Tselina-D',64806,'In Earth orbit','','1993-01-24 00:00:00',97.72,632,668,82.52,'Intermediate' from dual union all
select 22287,'022287','1992-094B','S5M','S5M',64806,'In Earth orbit','','1993-03-30 00:00:00',97.7,630,667,82.53,'Intermediate' from dual union all
select 22288,'022288','1992-093C','deb Kosmos-2227','',64805,'Reentered','1999-10-01 00:00:00','1993-01-25 00:00:00',101.77,837,848,71.02,'Intermediate' from dual union all
select 22289,'022289','1992-093D','deb Kosmos-2227','',64805,'In Earth orbit','','1993-01-25 00:00:00',100.86,751,848,70.84,'Intermediate' from dual union all
select 2229,'002229','1966-046F','deb Atlas 5304','',13453,'Reentered','1966-06-30 00:00:00','1966-06-20 00:00:00',89.41,213,278,28.75,'Intermediate' from dual union all
select 22290,'022290','1992-093E','Sep Motor Cover','Sep Motor Cover',64805,'In Earth orbit','','1993-01-25 00:00:00',104.91,845,1136,71.05,'Intermediate' from dual union all
select 22291,'022291','1992-093F','Sep Motor Cover','Sep Motor Cover',64805,'In Earth orbit','','1993-01-24 00:00:00',105.11,844,1155,71.01,'Intermediate' from dual union all
select 22292,'022292','1992-093G','Sep Motor Cover','Sep Motor Cover',64805,'In Earth orbit','','1993-01-25 00:00:00',105.2,847,1160,71.01,'Intermediate' from dual union all
select 22293,'022293','1992-093H','Sep Motor Cover','Sep Motor Cover',64805,'In Earth orbit','','1993-01-24 00:00:00',104.93,845,1138,71.06,'Intermediate' from dual union all
select 22294,'022294','1992-093J','deb 11S772','',64805,'In Earth orbit','','1993-01-25 00:00:00',102.18,788,936,71.06,'Intermediate' from dual union all
select 22295,'022295','1992-093K','deb 11S772','',64805,'In Earth orbit','','1993-01-25 00:00:00',102.58,807,955,71.01,'Intermediate' from dual union all
select 22296,'022296','1992-093L','deb 11S772','',64805,'In Earth orbit','','1993-01-28 00:00:00',103.03,817,987,70.87,'Intermediate' from dual union all
select 22297,'022297','1992-093M','deb 11S772','',64805,'In Earth orbit','','1993-01-26 00:00:00',102.84,815,971,70.95,'Intermediate' from dual union all
select 22298,'022298','1992-093N','deb 11S772','',64805,'Reentered','2001-12-24 00:00:00','1993-01-25 00:00:00',103.48,840,1007,70.83,'Intermediate' from dual union all
select 22299,'022299','1991-005D','deb Kosmos-2122','',64174,'Reentered','1993-01-02 00:00:00','1992-12-30 00:00:00',92.42,382,404,64.93,'Intermediate' from dual union all
select 223,'000223','61 Omicron 58','deb r/b Ablestar','',7273,'In Earth orbit','','1962-01-30 00:00:00',103.93,892,997,66.87,'Intermediate' from dual union all
select 2230,'002230','1966-046G','deb Atlas 5304','',13453,'Reentered','1966-06-30 00:00:00','1966-06-20 00:00:00',89.47,222,275,28.79,'Intermediate' from dual union all
select 22300,'022300','1992-095A','Kosmos-2229','Bion No. 10',64807,'Landed','1993-01-10 00:00:00','1992-12-31 00:00:00',90.41,217,373,62.81,'Intermediate' from dual union all
select 22301,'022301','1992-095B','11S510','11S510',64807,'Reentered','1993-01-16 00:00:00','1993-01-08 00:00:00',89.67,205,312,62.79,'Intermediate' from dual union all
select 22302,'022302','1992-095C','KB electropitaniya','KBE',64807,'Reentered','1993-02-11 00:00:00','1993-01-11 00:00:00',90.23,214,358,62.8,'Intermediate' from dual union all
select 22303,'022303','1992-095D','KBE part','KBE part',64807,'Reentered','1993-01-18 00:00:00','1993-01-11 00:00:00',89.95,207,337,62.81,'Intermediate' from dual union all
select 22304,'022304','1992-095E','KBE part','KBE part',64807,'Reentered','1993-01-29 00:00:00','1993-01-11 00:00:00',92.15,315,445,62.8,'Intermediate' from dual union all
select 22305,'022305','1992-095F','KBE part','KBE part',64807,'Reentered','1993-01-18 00:00:00','1993-01-11 00:00:00',92.44,316,472,62.77,'Intermediate' from dual union all
select 22306,'022306','1992-095G','KBE part','KBE part',64807,'Reentered','1993-01-12 00:00:00','1993-01-11 00:00:00',90.15,225,339,62.36,'Intermediate' from dual union all
select 22307,'022307','1993-001A','Kosmos-2230','Tsikada',65103,'In Earth orbit','','1993-02-11 00:00:00',104.89,972,1007,82.94,'Intermediate' from dual union all
select 22308,'022308','1993-001B','S3M','S3M',65103,'In Earth orbit','','1993-03-28 00:00:00',104.72,959,1004,82.93,'Intermediate' from dual union all
select 22309,'022309','1993-002A','Molniya-1','Molniya-1T No. 78',65104,'Reentered','2005-11-15 00:00:00','1993-02-13 00:00:00',717.7,599,39752,62.9,'Molniya' from dual union all
select 2231,'002231','1966-046H','deb Atlas 5304','',13453,'Reentered','1966-06-30 00:00:00','1966-06-20 00:00:00',92.1,243,511,28.77,'Intermediate' from dual union all
select 22310,'022310','1993-002B','11S510','11S510',65104,'Reentered','1993-03-04 00:00:00','1993-01-20 00:00:00',92.36,214,567,62.83,'Intermediate' from dual union all
select 22311,'022311','1993-002C','BOZ','BOZ',65104,'Reentered','1993-02-05 00:00:00','1993-01-20 00:00:00',91.71,176,541,62.81,'Intermediate' from dual union all
select 22312,'022312','1993-002D','Blok-ML','Blok-ML',65104,'Reentered','2006-04-04 00:00:00','1993-03-22 00:00:00',731.92,574,40476,62.98,'Molniya' from dual union all
select 22313,'022313','1993-003A','Endeavour (STS-54)','OV-105',65105,'Landed','1993-01-19 00:00:00','1993-01-15 00:00:00',90.6,300,307,28.47,'Intermediate' from dual union all
select 22314,'022314','1993-003B','TDRS-6','TDRS F',65105,'In Earth orbit','','1993-02-15 00:00:00',1436.06,35778,35793,.07,'Stationary' from dual union all
select 22315,'022315','1993-003C','IUS-13 SRM-1','Orbus 21',65105,'In Earth orbit','','1993-02-12 00:00:00',632.92,306,35776,26.8,'GEO Transfer' from dual union all
select 22316,'022316','1993-003D','IUS-13 SRM-2','Orbus 6E',65105,'In Earth orbit','','1993-03-13 00:00:00',1437.88,35519,36124,1.86,'Drift GEO' from dual union all
select 22317,'022317','1993-004A','Kosmos-2231','Kobal''t No. 537',65106,'Landed','1993-03-25 00:00:00','1993-02-18 00:00:00',89.72,174,348,67.12,'Intermediate' from dual union all
select 22318,'022318','1993-004B','11S510','11S510',65106,'Reentered','1993-01-23 00:00:00','1993-01-21 00:00:00',89.06,154,302,67.13,'Intermediate' from dual union all
select 22319,'022319','1993-005A','Soyuz TM-16','Soyuz 7K-STM No. 101',65107,'Landed','1993-07-22 00:00:00','1993-02-23 00:00:00',92.34,387,392,51.62,'Intermediate' from dual union all
select 2232,'002232','1966-046J','deb Atlas 5304','',13453,'Reentered','1966-06-30 00:00:00','1966-06-21 00:00:00',89.55,193,312,28.72,'Intermediate' from dual union all
select 22320,'022320','1993-005B','11S510','11S510',65107,'Reentered','1993-01-26 00:00:00','1993-01-26 00:00:00',87.54,149,157,51.62,'Intermediate' from dual union all
select 22321,'022321','1993-006A','Kosmos-2232','Oko',65108,'In Earth orbit','','1993-02-25 00:00:00',717.64,614,39734,62.76,'Molniya' from dual union all
select 22322,'022322','1993-006B','11S510','11S510',65108,'Reentered','1993-03-09 00:00:00','1993-02-02 00:00:00',92.24,214,555,62.8,'Intermediate' from dual union all
select 22323,'022323','1993-006C','BOZ','BOZ',65108,'Reentered','1993-02-14 00:00:00','1993-02-02 00:00:00',91.48,176,519,62.84,'Intermediate' from dual union all
select 22324,'022324','1993-006D','Blok-2BL','Blok-2BL',65108,'In Earth orbit','','1993-02-27 00:00:00',706.82,616,39197,62.78,'Molniya' from dual union all
select 22325,'022325','1993-006E','Ullage rocket','Ullage rocket',65108,'Reentered','1993-02-28 00:00:00','1993-02-05 00:00:00',92.05,211,539,62.82,'Intermediate' from dual union all
select 22326,'022326','1992-093P','deb 11S772','',64805,'In Earth orbit','','1993-01-28 00:00:00',104.45,845,1093,70.66,'Intermediate' from dual union all
select 22327,'022327','1992-093Q','deb 11S772','',64805,'In Earth orbit','','1993-01-28 00:00:00',99.75,638,856,71.13,'Intermediate' from dual;
insert into SATELLITE
select 22328,'022328','1992-093R','deb 11S772','',64805,'Reentered','2012-10-18 00:00:00','1993-01-28 00:00:00',102.24,835,895,70.54,'Intermediate' from dual union all
select 22329,'022329','1992-093S','deb 11S772','',64805,'In Earth orbit','','1993-01-28 00:00:00',100.32,677,870,70.92,'Intermediate' from dual union all
select 2233,'002233','1966-046K','deb Atlas 5304','',13453,'Reentered','1966-06-21 00:00:00','1966-06-19 00:00:00',88.17,108,260,28.7,'Intermediate' from dual union all
select 22330,'022330','1992-093T','deb 11S772','',64805,'In Earth orbit','','1993-01-28 00:00:00',103.03,811,992,70.42,'Intermediate' from dual union all
select 22331,'022331','1992-093U','deb 11S772','',64805,'In Earth orbit','','1993-01-28 00:00:00',102.61,825,940,70.92,'Intermediate' from dual union all
select 22332,'022332','1992-093V','deb 11S772','',64805,'In Earth orbit','','1993-01-27 00:00:00',102.89,840,951,70.58,'Intermediate' from dual union all
select 22333,'022333','1992-093W','deb 11S772','',64805,'In Earth orbit','','1993-01-26 00:00:00',100.24,670,870,71.01,'Intermediate' from dual union all
select 22334,'022334','1992-093X','deb 11S772','',64805,'Reentered','1999-12-11 00:00:00','1993-01-28 00:00:00',100.8,675,918,70.65,'Intermediate' from dual union all
select 22335,'022335','1992-093Y','deb 11S772','',64805,'In Earth orbit','','1993-01-26 00:00:00',103.81,846,1032,71.17,'Intermediate' from dual union all
select 22336,'022336','1992-093Z','deb 11S772','',64805,'In Earth orbit','','1993-01-28 00:00:00',103.89,846,1039,71.13,'Intermediate' from dual union all
select 22337,'022337','1992-093AA','deb 11S772','',64805,'In Earth orbit','','1993-01-28 00:00:00',104.11,844,1062,71.28,'Intermediate' from dual union all
select 22338,'022338','1992-093AB','deb 11S772','',64805,'In Earth orbit','','1993-01-28 00:00:00',100.08,673,852,70.63,'Intermediate' from dual union all
select 22339,'022339','1992-093AC','deb 11S772','',64805,'In Earth orbit','','1993-01-28 00:00:00',100.65,695,884,70.81,'Intermediate' from dual union all
select 2234,'002234','1966-046L','deb Atlas 5304','',13453,'Reentered','1966-06-23 00:00:00','1966-06-19 00:00:00',88.83,187,246,28.76,'Intermediate' from dual union all
select 22340,'022340','1992-093AD','deb 11S772','',64805,'In Earth orbit','','1993-01-28 00:00:00',103.36,843,991,70.88,'Intermediate' from dual union all
select 22341,'022341','1992-093AE','deb 11S772','',64805,'In Earth orbit','','1993-01-26 00:00:00',103.91,844,1043,70.75,'Intermediate' from dual union all
select 22342,'022342','1992-093AF','deb 11S772','',64805,'In Earth orbit','','1993-01-28 00:00:00',103.81,845,1033,70.7,'Intermediate' from dual union all
select 22343,'022343','1992-093AG','deb 11S772','',64805,'In Earth orbit','','1993-01-28 00:00:00',102.96,820,978,70.97,'Intermediate' from dual union all
select 22344,'022344','1992-093AH','deb 11S772','',64805,'In Earth orbit','','1993-01-28 00:00:00',103.75,842,1029,71.1,'Intermediate' from dual union all
select 22345,'022345','1992-093AJ','deb 11S772','',64805,'Reentered','2004-06-26 00:00:00','1993-01-28 00:00:00',102.13,781,937,70.56,'Intermediate' from dual union all
select 22346,'022346','1992-093AK','deb 11S772','',64805,'Reentered','2000-01-23 00:00:00','1993-01-31 00:00:00',100.01,625,893,70.76,'Intermediate' from dual union all
select 22347,'022347','1992-093AL','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',105.24,840,1172,70.84,'Intermediate' from dual union all
select 22348,'022348','1992-093AM','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',102.99,816,985,71.26,'Intermediate' from dual union all
select 22349,'022349','1992-093AN','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',104.7,837,1124,70.83,'Intermediate' from dual union all
select 2235,'002235','1966-046M','deb Atlas 5304','',13453,'Reentered','1966-06-23 00:00:00','1966-06-19 00:00:00',88.8,178,253,28.73,'Intermediate' from dual union all
select 22350,'022350','1992-093AP','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',102.59,831,932,70.91,'Intermediate' from dual union all
select 22351,'022351','1992-093AQ','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.15,825,991,70.91,'Intermediate' from dual union all
select 22352,'022352','1992-093AR','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',102.88,839,951,70.89,'Intermediate' from dual union all
select 22353,'022353','1992-093AS','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.35,816,1018,70.98,'Intermediate' from dual union all
select 22354,'022354','1992-093AT','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.59,820,1037,71.15,'Intermediate' from dual union all
select 22355,'022355','1992-093AU','deb 11S772','',64805,'Reentered','2011-11-07 00:00:00','1993-01-31 00:00:00',102.26,832,900,71.24,'Intermediate' from dual union all
select 22356,'022356','1992-093AV','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.5,845,1004,70.71,'Intermediate' from dual union all
select 22357,'022357','1992-093AW','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',99.95,658,855,70.89,'Intermediate' from dual union all
select 22358,'022358','1992-093AX','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',104.11,845,1061,71.22,'Intermediate' from dual union all
select 22359,'022359','1992-093AY','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',104.72,836,1127,70.76,'Intermediate' from dual union all
select 2236,'002236','1966-046N','deb Atlas 5304','',13453,'Reentered','1966-06-22 00:00:00','1966-06-19 00:00:00',88.49,178,222,28.73,'Intermediate' from dual union all
select 22360,'022360','1992-093AZ','deb 11S772','',64805,'Reentered','1999-11-20 00:00:00','1993-02-01 00:00:00',99.52,596,876,70.82,'Intermediate' from dual union all
select 22361,'022361','1992-093BA','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',104.76,839,1128,70.77,'Intermediate' from dual union all
select 22362,'022362','1992-093BB','deb 11S772','',64805,'In Earth orbit','','1993-02-01 00:00:00',105.39,846,1179,70.9,'Intermediate' from dual union all
select 22363,'022363','1992-093BC','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.78,824,1050,71.04,'Intermediate' from dual union all
select 22364,'022364','1992-093BD','deb 11S772','',64805,'Reentered','2002-08-30 00:00:00','1993-01-31 00:00:00',101.81,719,969,71.12,'Intermediate' from dual union all
select 22365,'022365','1992-093BE','deb 11S772','',64805,'Reentered','1999-12-09 00:00:00','1993-02-01 00:00:00',97.67,446,849,71.13,'Intermediate' from dual union all
select 22366,'022366','1992-093BF','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.96,809,1083,71.14,'Intermediate' from dual union all
select 22367,'022367','1992-093BG','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',104.26,821,1098,70.9,'Intermediate' from dual union all
select 22368,'022368','1992-093BH','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',100.01,666,851,71.25,'Intermediate' from dual union all
select 22369,'022369','1992-093BJ','deb 11S772','',64805,'In Earth orbit','','1993-02-02 00:00:00',103.33,843,990,70.89,'Intermediate' from dual union all
select 2237,'002237','1966-046P','deb Atlas 5304','',13453,'Reentered','1966-06-23 00:00:00','1966-06-18 00:00:00',89.55,192,313,29.8,'Intermediate' from dual union all
select 22370,'022370','1992-093BK','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',102.95,811,986,70.98,'Intermediate' from dual union all
select 22371,'022371','1992-093BL','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.58,845,1011,71.17,'Intermediate' from dual union all
select 22372,'022372','1992-093BM','deb 11S772','',64805,'Reentered','2002-01-21 00:00:00','1993-01-31 00:00:00',99.76,637,858,71.12,'Intermediate' from dual union all
select 22373,'022373','1992-093BN','deb 11S772','',64805,'Reentered','2000-11-29 00:00:00','1993-01-31 00:00:00',101.79,711,976,70.37,'Intermediate' from dual union all
select 22374,'022374','1992-093BP','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',104.79,836,1134,70.79,'Intermediate' from dual union all
select 22375,'022375','1992-093BQ','deb 11S772','',64805,'In Earth orbit','','1993-02-02 00:00:00',102.95,833,964,70.82,'Intermediate' from dual union all
select 22376,'022376','1992-093BR','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',102.74,831,946,70.93,'Intermediate' from dual union all
select 22377,'022377','1992-093BS','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',102.75,846,931,71.09,'Intermediate' from dual union all
select 22378,'022378','1992-093BT','deb 11S772','',64805,'Reentered','2002-10-06 00:00:00','1993-01-31 00:00:00',99.83,649,852,71.29,'Intermediate' from dual union all
select 22379,'022379','1992-093BU','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.43,845,997,70.71,'Intermediate' from dual union all
select 2238,'002238','1966-046Q','deb Atlas 5304','',13453,'Reentered','1966-06-22 00:00:00','1966-06-21 00:00:00',88.9,129,312,28.91,'Intermediate' from dual union all
select 22380,'022380','1992-093BV','deb 11S772','',64805,'Reentered','2000-12-09 00:00:00','1993-01-31 00:00:00',101.25,718,918,70.65,'Intermediate' from dual union all
select 22381,'022381','1992-093BW','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.05,844,962,70.55,'Intermediate' from dual union all
select 22382,'022382','1992-093BX','deb 11S772','',64805,'In Earth orbit','','1993-01-30 00:00:00',104.52,838,1106,70.87,'Intermediate' from dual union all
select 22383,'022383','1992-093BY','deb 11S772','',64805,'Reentered','2013-11-02 00:00:00','1993-02-01 00:00:00',104.12,843,1063,70.71,'Intermediate' from dual union all
select 22384,'022384','1992-093BZ','deb 11S772','',64805,'Reentered','2000-10-29 00:00:00','1993-01-31 00:00:00',101.57,760,906,70.73,'Intermediate' from dual union all
select 22385,'022385','1992-093CA','deb 11S772','',64805,'Reentered','2001-10-29 00:00:00','1993-02-01 00:00:00',100.94,707,900,70.82,'Intermediate' from dual union all
select 22386,'022386','1992-093CB','deb 11S772','',64805,'Reentered','1999-11-15 00:00:00','1993-02-01 00:00:00',97.81,456,852,70.86,'Intermediate' from dual union all
select 22387,'022387','1992-093CC','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.5,831,1018,70.9,'Intermediate' from dual union all
select 22388,'022388','1992-093CD','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.49,827,1021,70.96,'Intermediate' from dual union all
select 22389,'022389','1992-093CE','deb 11S772','',64805,'Reentered','2003-10-27 00:00:00','1993-01-31 00:00:00',99.69,634,854,71.16,'Intermediate' from dual union all
select 2239,'002239','1966-046R','deb Atlas 5304','',13453,'Reentered','1966-06-22 00:00:00','1966-06-20 00:00:00',88.9,202,239,28.77,'Intermediate' from dual union all
select 22390,'022390','1992-093CF','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.14,846,968,71.18,'Intermediate' from dual union all
select 22391,'022391','1992-093CG','deb 11S772','',64805,'Reentered','2013-06-17 00:00:00','1993-02-01 00:00:00',102.61,805,959,71.36,'Intermediate' from dual union all
select 22392,'022392','1992-093CH','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',99.65,630,853,70.9,'Intermediate' from dual union all
select 22393,'022393','1992-093CJ','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.6,821,1037,71.13,'Intermediate' from dual union all
select 22394,'022394','1992-093CK','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.15,844,972,71.29,'Intermediate' from dual union all
select 22395,'022395','1992-093CL','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.91,843,1044,71.17,'Intermediate' from dual union all
select 22396,'022396','1992-093CM','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',102.35,845,895,70.67,'Intermediate' from dual union all
select 22397,'022397','1992-093CN','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',102.38,839,904,70.82,'Intermediate' from dual union all
select 22398,'022398','1992-093CP','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.33,821,1012,70.96,'Intermediate' from dual union all
select 22399,'022399','1992-093CQ','deb 11S772','',64805,'Reentered','2003-10-26 00:00:00','1993-01-31 00:00:00',99.95,650,862,71,'Intermediate' from dual union all
select 224,'000224','61 Omicron 59','deb r/b Ablestar','',7273,'In Earth orbit','','1962-01-30 00:00:00',104.16,892,1019,66.71,'Intermediate' from dual union all
select 2240,'002240','1966-046S','deb Atlas 5304','',13453,'Reentered','1966-06-23 00:00:00','1966-06-19 00:00:00',90.71,232,387,28.07,'Intermediate' from dual union all
select 22400,'022400','1992-093CR','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',104.15,834,1075,71.24,'Intermediate' from dual union all
select 22401,'022401','1992-093CS','deb 11S772','',64805,'Reentered','2003-06-29 00:00:00','1993-01-31 00:00:00',99.67,628,858,71.14,'Intermediate' from dual union all
select 22402,'022402','1992-093CT','deb 11S772','',64805,'Reentered','2000-07-09 00:00:00','1993-01-31 00:00:00',100.84,695,902,70.65,'Intermediate' from dual union all
select 22403,'022403','1992-093CU','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.19,843,976,70.9,'Intermediate' from dual union all
select 22404,'022404','1992-093CV','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',99.95,643,869,71.07,'Intermediate' from dual union all
select 22405,'022405','1992-093CW','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',100.43,681,876,71.04,'Intermediate' from dual union all
select 22406,'022406','1992-093CX','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',104.63,841,1113,70.76,'Intermediate' from dual union all
select 22407,'022407','1992-093CY','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',104.62,835,1119,70.87,'Intermediate' from dual union all
select 22408,'022408','1992-093CZ','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.69,820,1046,71.1,'Intermediate' from dual union all
select 22409,'022409','1992-093DA','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',99.79,640,857,71.14,'Intermediate' from dual union all
select 2241,'002241','1966-046T','deb Atlas 5304','',13453,'Reentered','1966-06-22 00:00:00','1966-06-20 00:00:00',90.38,253,334,27.99,'Intermediate' from dual union all
select 22410,'022410','1992-093DB','deb 11S772','',64805,'In Earth orbit','','1993-02-15 00:00:00',103.43,842,1000,71.09,'Intermediate' from dual union all
select 22411,'022411','1992-093DC','deb 11S772','',64805,'Reentered','2015-07-03 00:00:00','1993-01-31 00:00:00',104.06,843,1057,70.67,'Intermediate' from dual union all
select 22412,'022412','1992-093DD','deb 11S772','',64805,'Reentered','2000-04-24 00:00:00','1993-01-31 00:00:00',103.44,829,1014,71.03,'Intermediate' from dual union all
select 22413,'022413','1992-093DE','deb 11S772','',64805,'Reentered','2003-06-28 00:00:00','1993-01-31 00:00:00',99.8,643,854,71.13,'Intermediate' from dual union all
select 22414,'022414','1992-093DF','deb 11S772','',64805,'Reentered','2015-01-07 00:00:00','1993-01-31 00:00:00',99.67,631,854,71.14,'Intermediate' from dual union all
select 22415,'022415','1992-093DG','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.12,807,1006,71.13,'Intermediate' from dual union all
select 22416,'022416','1992-093DH','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',102.66,826,943,70.96,'Intermediate' from dual union all
select 22417,'022417','1992-093DJ','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.18,846,972,71.19,'Intermediate' from dual;
insert into SATELLITE
select 22418,'022418','1992-093DK','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.59,820,1037,71.14,'Intermediate' from dual union all
select 22419,'022419','1992-093DL','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.55,846,1007,71.18,'Intermediate' from dual union all
select 2242,'002242','1966-046U','deb Atlas 5304','',13453,'Reentered','1966-06-27 00:00:00','1966-06-20 00:00:00',89.47,238,258,28.28,'Intermediate' from dual union all
select 22420,'022420','1992-093DM','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',100.17,675,859,71.01,'Intermediate' from dual union all
select 22421,'022421','1992-093DN','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',102.63,808,959,71.04,'Intermediate' from dual union all
select 22422,'022422','1992-093DP','deb 11S772','',64805,'Reentered','1994-12-31 00:00:00','1993-01-31 00:00:00',103.37,846,990,71.19,'Intermediate' from dual union all
select 22423,'022423','1992-093DQ','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.3,842,988,71.28,'Intermediate' from dual union all
select 22424,'022424','1992-093DR','deb 11S772','',64805,'Reentered','2001-06-21 00:00:00','1993-01-31 00:00:00',99.86,650,854,71.13,'Intermediate' from dual union all
select 22425,'022425','1992-093DS','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.65,846,1017,71.18,'Intermediate' from dual union all
select 22426,'022426','1992-093DT','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',102.92,772,1021,71.31,'Intermediate' from dual union all
select 22427,'022427','1992-093DU','deb 11S772','',64805,'Reentered','2002-02-15 00:00:00','1993-01-31 00:00:00',102.39,824,920,70.64,'Intermediate' from dual union all
select 22428,'022428','1992-093DV','deb 11S772','',64805,'Reentered','1999-03-31 00:00:00','1993-01-31 00:00:00',99.39,607,852,71.28,'Intermediate' from dual union all
select 22429,'022429','1992-093DW','deb 11S772','',64805,'Reentered','2017-11-24 00:00:00','1993-02-01 00:00:00',99.9,651,856,71.13,'Intermediate' from dual union all
select 2243,'002243','1966-046V','deb Atlas 5304','',13453,'Reentered','1966-06-21 00:00:00','1966-06-20 00:00:00',89.53,221,282,27.63,'Intermediate' from dual union all
select 22430,'022430','1992-093DX','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.52,838,1012,71.23,'Intermediate' from dual union all
select 22431,'022431','1992-093DY','deb 11S772','',64805,'Reentered','2014-03-21 00:00:00','1993-01-31 00:00:00',104.28,841,1081,70.92,'Intermediate' from dual union all
select 22432,'022432','1992-093DZ','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.58,843,1012,70.84,'Intermediate' from dual union all
select 22433,'022433','1992-093EA','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.24,809,1015,71.19,'Intermediate' from dual union all
select 22434,'022434','1992-093EB','deb 11S772','',64805,'Reentered','1999-11-16 00:00:00','1993-01-31 00:00:00',100.18,680,855,71.28,'Intermediate' from dual union all
select 22435,'022435','1992-093EC','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.62,826,1034,71.37,'Intermediate' from dual union all
select 22436,'022436','1992-093ED','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',100.81,728,866,71.52,'Intermediate' from dual union all
select 22437,'022437','1992-093EE','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',101.47,786,871,71.35,'Intermediate' from dual union all
select 22438,'022438','1992-093EF','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.21,819,1002,71.36,'Intermediate' from dual union all
select 22439,'022439','1992-093EG','deb 11S772','',64805,'Reentered','1998-12-13 00:00:00','1993-01-30 00:00:00',98.94,571,845,71.41,'Intermediate' from dual union all
select 2244,'002244','1966-046W','deb Atlas 5304','',13453,'Reentered','1966-06-23 00:00:00','1966-06-18 00:00:00',89.56,223,283,29.47,'Intermediate' from dual union all
select 22440,'022440','1992-093EH','deb 11S772','',64805,'Reentered','1999-02-24 00:00:00','1993-01-31 00:00:00',99.11,585,848,71.38,'Intermediate' from dual union all
select 22441,'022441','1992-093EJ','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.18,842,976,71.2,'Intermediate' from dual union all
select 22442,'022442','1992-093EK','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.86,805,1076,71.14,'Intermediate' from dual union all
select 22443,'022443','1992-093EL','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',103.64,844,1018,71.13,'Intermediate' from dual union all
select 22444,'022444','1992-093EM','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',102.86,832,955,71.32,'Intermediate' from dual union all
select 22445,'022445','1992-093EN','deb 11S772','',64805,'In Earth orbit','','1993-01-31 00:00:00',102.62,831,935,71.28,'Intermediate' from dual union all
select 22446,'022446','1993-007A','GPS IIA-9','Navstar GPS SVN 22',65109,'In Earth orbit','','1993-03-05 00:00:00',717.9,20007,20354,54.87,'Medium' from dual union all
select 22447,'022447','1993-007B','Delta 218','Delta SSPS AJ10-118K',65109,'In Earth orbit','','1993-03-29 00:00:00',97.56,558,726,20.86,'Equatorial' from dual union all
select 22448,'022448','1993-007C','Star 48B','Star 48B S/N 10075-18',65109,'Reentered','2002-09-11 00:00:00','1993-03-31 00:00:00',346.54,179,19751,34.75,'Highly Ellip' from dual union all
select 22449,'022449','1992-071C','Znamya-2','Znamya-2',64783,'Reentered','1993-02-05 00:00:00','1993-02-04 00:00:00',92.37,389,392,51.62,'Intermediate' from dual union all
select 2245,'002245','1966-046X','deb Atlas 5304','',13453,'Reentered','1966-06-22 00:00:00','1966-06-20 00:00:00',88.66,186,231,28.78,'Intermediate' from dual union all
select 22450,'022450','1992-093EP','deb 11S772','',64805,'Reentered','1997-09-17 00:00:00','1993-02-10 00:00:00',99.23,589,855,70.74,'Intermediate' from dual union all
select 22451,'022451','1992-093EQ','deb 11S772','',64805,'In Earth orbit','','1993-02-09 00:00:00',103.5,825,1024,71.14,'Intermediate' from dual union all
select 22452,'022452','1992-093ER','deb 11S772','',64805,'Reentered','2001-01-21 00:00:00','1993-02-08 00:00:00',99.83,647,854,70.98,'Intermediate' from dual union all
select 22453,'022453','1992-093ES','deb 11S772','',64805,'In Earth orbit','','1993-02-09 00:00:00',104.61,838,1115,70.81,'Intermediate' from dual union all
select 22454,'022454','1992-093ET','deb 11S772','',64805,'In Earth orbit','','1993-02-05 00:00:00',103.3,833,997,70.92,'Intermediate' from dual union all
select 22455,'022455','1992-093EU','deb 11S772','',64805,'In Earth orbit','','1993-02-09 00:00:00',104.49,839,1102,70.89,'Intermediate' from dual union all
select 22456,'022456','1992-093EV','deb 11S772','',64805,'In Earth orbit','','1993-02-08 00:00:00',104.62,836,1118,71.3,'Intermediate' from dual union all
select 22457,'022457','1992-093EW','deb 11S772','',64805,'Reentered','2000-04-01 00:00:00','1993-02-08 00:00:00',101,694,918,70.68,'Intermediate' from dual union all
select 22458,'022458','1992-093EX','deb 11S772','',64805,'In Earth orbit','','1993-02-08 00:00:00',102.98,845,955,70.85,'Intermediate' from dual union all
select 22459,'022459','1992-093EY','deb 11S772','',64805,'In Earth orbit','','1993-02-09 00:00:00',103.99,831,1063,71.21,'Intermediate' from dual union all
select 2246,'002246','1966-046Y','deb Atlas 5304','',13453,'Reentered','1966-06-21 00:00:00','1966-06-19 00:00:00',88.28,140,239,28.84,'Intermediate' from dual union all
select 22460,'022460','1992-093EZ','deb 11S772','',64805,'Reentered','1993-06-24 00:00:00','1993-02-09 00:00:00',97.57,450,835,70.55,'Intermediate' from dual union all
select 22461,'022461','1992-093FA','deb 11S772','',64805,'Reentered','1996-07-24 00:00:00','1993-02-10 00:00:00',99.18,577,862,70.79,'Intermediate' from dual union all
select 22462,'022462','1992-093FB','deb 11S772','',64805,'Reentered','2012-12-24 00:00:00','1993-02-08 00:00:00',103.7,844,1023,70.75,'Intermediate' from dual union all
select 22463,'022463','1992-093FC','deb 11S772','',64805,'Reentered','2013-01-13 00:00:00','1993-02-04 00:00:00',103.69,829,1037,70.39,'Intermediate' from dual union all
select 22464,'022464','1992-093FD','deb 11S772','',64805,'Reentered','2001-04-01 00:00:00','1993-02-09 00:00:00',99.9,651,856,71,'Intermediate' from dual union all
select 22465,'022465','1992-093FE','deb 11S772','',64805,'Reentered','2001-01-08 00:00:00','1993-02-09 00:00:00',101.06,771,847,71.26,'Intermediate' from dual union all
select 22466,'022466','1992-093FF','deb 11S772','',64805,'Reentered','2001-03-16 00:00:00','1993-02-06 00:00:00',101.66,783,892,70.71,'Intermediate' from dual union all
select 22467,'022467','1992-093FG','deb 11S772','',64805,'Reentered','2001-08-24 00:00:00','1993-02-08 00:00:00',102.2,843,883,70.85,'Intermediate' from dual union all
select 22468,'022468','1992-093FH','deb 11S772','',64805,'In Earth orbit','','1993-02-09 00:00:00',104.58,848,1101,71.2,'Intermediate' from dual union all
select 22469,'022469','1992-093FJ','deb 11S772','',64805,'Reentered','2005-04-15 00:00:00','1993-02-09 00:00:00',103.11,845,967,71.19,'Intermediate' from dual union all
select 2247,'002247','1966-046Z','deb Atlas 5304','',13453,'Reentered','1966-06-21 00:00:00','1966-06-19 00:00:00',90.69,239,378,27.73,'Intermediate' from dual union all
select 22470,'022470','1992-093FK','deb 11S772','',64805,'Reentered','2000-05-20 00:00:00','1993-02-09 00:00:00',100.89,748,854,71.43,'Intermediate' from dual union all
select 22471,'022471','1992-093FL','deb 11S772','',64805,'In Earth orbit','','1993-02-09 00:00:00',103.4,815,1023,71.16,'Intermediate' from dual union all
select 22472,'022472','1992-093FM','deb 11S772','',64805,'Reentered','2003-10-23 00:00:00','1993-02-10 00:00:00',99.5,624,845,71.25,'Intermediate' from dual union all
select 22473,'022473','1992-093FN','deb 11S772','',64805,'In Earth orbit','','1993-02-09 00:00:00',104.33,838,1088,71.34,'Intermediate' from dual union all
select 22474,'022474','1992-093FP','deb 11S772','',64805,'In Earth orbit','','1993-02-09 00:00:00',105.51,846,1191,71.33,'Intermediate' from dual union all
select 22475,'022475','1992-093FQ','deb 11S772','',64805,'In Earth orbit','','1993-02-09 00:00:00',102.1,804,913,71.34,'Intermediate' from dual union all
select 22476,'022476','1992-093FR','deb 11S772','',64805,'In Earth orbit','','1993-02-08 00:00:00',110.56,845,1660,71.57,'Intermediate' from dual union all
select 22477,'022477','1992-093FS','deb 11S772','',64805,'In Earth orbit','','1993-02-09 00:00:00',100.63,695,882,70.89,'Intermediate' from dual union all
select 22478,'022478','1992-093FT','deb 11S772','',64805,'In Earth orbit','','1993-02-06 00:00:00',102.26,795,937,71.31,'Intermediate' from dual union all
select 22479,'022479','1992-093FU','deb 11S772','',64805,'Reentered','1996-05-04 00:00:00','1993-02-08 00:00:00',98.68,566,826,71.41,'Intermediate' from dual union all
select 2248,'002248','1966-046AA','deb Atlas 5304','',13453,'Reentered','1966-06-23 00:00:00','1966-06-18 00:00:00',92.36,227,554,27.28,'Intermediate' from dual union all
select 22480,'022480','1992-093FV','deb 11S772','',64805,'In Earth orbit','','1993-02-09 00:00:00',103.39,818,1020,71.14,'Intermediate' from dual union all
select 22481,'022481','1992-093FW','deb 11S772','',64805,'In Earth orbit','','1993-02-09 00:00:00',104.23,846,1071,71.3,'Intermediate' from dual union all
select 22482,'022482','1992-093FX','deb 11S772','',64805,'Reentered','2000-04-20 00:00:00','1993-02-08 00:00:00',101.09,741,880,71.46,'Intermediate' from dual union all
select 22483,'022483','1992-093FY','deb 11S772','',64805,'Reentered','1993-02-22 00:00:00','1993-02-09 00:00:00',93.64,342,564,71.09,'Intermediate' from dual union all
select 22484,'022484','1992-093FZ','deb 11S772','',64805,'Reentered','2001-11-13 00:00:00','1993-02-07 00:00:00',99.75,640,854,71.14,'Intermediate' from dual union all
select 22485,'022485','1992-093GA','deb 11S772','',64805,'Reentered','2015-10-30 00:00:00','1993-02-09 00:00:00',104.32,839,1086,70.49,'Intermediate' from dual union all
select 22486,'022486','1990-104AG','Kosmos-2106 SS 27','ESO',63346,'Reentered','1993-04-05 00:00:00','1993-02-08 00:00:00',94.02,468,474,82.5,'Intermediate' from dual union all
select 22487,'022487','1993-008A','Kosmos-2233','Parus',65110,'In Earth orbit','','1993-03-31 00:00:00',104.71,953,1009,82.95,'Intermediate' from dual union all
select 22488,'022488','1993-008B','S3M','S3M',65110,'In Earth orbit','','1993-03-30 00:00:00',104.58,952,998,82.95,'Intermediate' from dual union all
select 22489,'022489','1993-009A','Orbcomm CDS-1','Orbcomm OXP-1',65111,'In Earth orbit','','1993-03-28 00:00:00',100,725,792,24.96,'Equatorial' from dual union all
select 2249,'002249','1966-046AB','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',90.29,238,340,28.41,'Intermediate' from dual union all
select 22490,'022490','1993-009B','SCD-1','SCD-1',65111,'In Earth orbit','','1993-03-12 00:00:00',99.95,724,789,24.97,'Equatorial' from dual union all
select 22491,'022491','1993-009C','Orion 38','Orion 38',65111,'In Earth orbit','','1993-03-30 00:00:00',99.73,706,785,24.98,'Equatorial' from dual union all
select 22492,'022492','1992-093GB','deb 11S772','',64805,'Reentered','1999-01-02 00:00:00','1993-02-15 00:00:00',99.04,569,856,70.99,'Intermediate' from dual union all
select 22493,'022493','1992-093GC','deb 11S772','',64805,'In Earth orbit','','1993-02-14 00:00:00',100.85,702,896,71.04,'Intermediate' from dual union all
select 22494,'022494','1992-093GD','deb 11S772','',64805,'In Earth orbit','','1993-02-15 00:00:00',105.57,844,1199,70.82,'Intermediate' from dual union all
select 22495,'022495','1992-093GE','deb 11S772','',64805,'In Earth orbit','','1993-02-15 00:00:00',104,923,973,71.22,'Intermediate' from dual union all
select 22496,'022496','1992-093GF','deb 11S772','',64805,'In Earth orbit','','1993-02-15 00:00:00',103.23,845,977,71.21,'Intermediate' from dual union all
select 22497,'022497','1992-093GG','deb 11S772','',64805,'Reentered','1999-05-22 00:00:00','1993-02-15 00:00:00',99.68,641,846,71.12,'Intermediate' from dual union all
select 22498,'022498','1992-093GH','deb 11S772','',64805,'Reentered','2015-10-21 00:00:00','1993-02-14 00:00:00',102.67,805,965,71.28,'Intermediate' from dual union all
select 22499,'022499','1992-093GJ','deb 11S772','',64805,'In Earth orbit','','1993-02-14 00:00:00',103.86,835,1047,71.43,'Intermediate' from dual union all
select 225,'000225','61 Omicron 60','deb r/b Ablestar','',7273,'In Earth orbit','','1962-02-06 00:00:00',103.1,782,1028,66.81,'Intermediate' from dual union all
select 2250,'002250','1966-046AC','deb Atlas 5304','',13453,'Reentered','1966-06-29 00:00:00','1966-06-19 00:00:00',90.08,239,318,28.57,'Intermediate' from dual union all
select 22500,'022500','1992-093GK','deb 11S772','',64805,'In Earth orbit','','1993-02-14 00:00:00',101.86,794,900,70.85,'Intermediate' from dual union all
select 22501,'022501','1992-093GL','deb 11S772','',64805,'Reentered','2013-11-04 00:00:00','1993-02-15 00:00:00',103.21,829,992,70.99,'Intermediate' from dual union all
select 22502,'022502','1992-093GM','deb 11S772','',64805,'In Earth orbit','','1993-02-15 00:00:00',104.04,831,1068,70.82,'Intermediate' from dual union all
select 22503,'022503','1992-093GN','deb 11S772','',64805,'In Earth orbit','','1993-02-13 00:00:00',102.98,841,959,71.2,'Intermediate' from dual union all
select 22504,'022504','1992-093GP','deb 11S772','',64805,'In Earth orbit','','1993-02-14 00:00:00',103.53,827,1024,71.34,'Intermediate' from dual union all
select 22505,'022505','1992-093GQ','deb 11S772','',64805,'Reentered','2011-04-14 00:00:00','1993-02-13 00:00:00',103.87,839,1044,70.84,'Intermediate' from dual union all
select 22506,'022506','1992-093GR','deb 11S772','',64805,'Reentered','2016-03-10 00:00:00','1993-02-13 00:00:00',103.67,842,1023,70.69,'Intermediate' from dual union all
select 22507,'022507','1992-093GS','deb 11S772','',64805,'Reentered','2000-06-16 00:00:00','1993-02-13 00:00:00',103.69,834,1031,71.03,'Intermediate' from dual;
insert into SATELLITE
select 22508,'022508','1992-093GT','deb 11S772','',64805,'Reentered','1999-05-12 00:00:00','1993-02-15 00:00:00',101.01,737,876,71,'Intermediate' from dual union all
select 22509,'022509','1992-093GU','deb 11S772','',64805,'In Earth orbit','','1993-02-13 00:00:00',103.5,817,1032,71.15,'Intermediate' from dual union all
select 2251,'002251','1966-046AD','deb Atlas 5304','',13453,'Reentered','1966-06-25 00:00:00','1966-06-19 00:00:00',90.48,207,390,28,'Intermediate' from dual union all
select 22510,'022510','1992-093GV','deb 11S772','',64805,'Reentered','1998-03-03 00:00:00','1993-02-14 00:00:00',100.14,672,859,70.97,'Intermediate' from dual union all
select 22511,'022511','1992-093GW','deb 11S772','',64805,'Reentered','2003-10-14 00:00:00','1993-02-15 00:00:00',103.23,838,985,70.56,'Intermediate' from dual union all
select 22512,'022512','1993-010A','Kosmos-2234','Uragan No. 73L',65112,'In Earth orbit','','1993-03-19 00:00:00',675.71,19106,19153,64.83,'Medium' from dual union all
select 22513,'022513','1993-010B','Kosmos-2235','Uragan No. 59L',65112,'In Earth orbit','','1993-03-19 00:00:00',675.69,19111,19147,64.83,'Medium' from dual union all
select 22514,'022514','1993-010C','Kosmos-2236','Uragan No. 57L',65112,'In Earth orbit','','1993-03-21 00:00:00',675.71,19096,19163,64.85,'Medium' from dual union all
select 22515,'022515','1993-010D','8S812','8S812',65112,'Reentered','1993-02-18 00:00:00','1993-02-18 00:00:00',87.05,121,136,64.84,'Intermediate' from dual union all
select 22516,'022516','1993-010E','Perekhodnik','Sredniy perekhodnik',65112,'Reentered','1993-02-18 00:00:00','1993-02-18 00:00:00',87.5,130,172,64.86,'Intermediate' from dual union all
select 22517,'022517','1993-010F','Blok DM2','11S861 No. 66L',65112,'In Earth orbit','','1993-03-21 00:00:00',674.74,19076,19134,64.84,'Medium' from dual union all
select 22518,'022518','1992-086B','USA 89','QUASAR 9',64798,'In Earth orbit','','1993-02-01 00:00:00',718.69,1000,39400,63,'Molniya' from dual union all
select 22519,'022519','1992-086C','USA 89 PKM','TU-882',64798,'In Earth orbit','','1997-11-21 00:00:00',170.79,372,7265,56.94,'Medium' from dual union all
select 2252,'002252','1966-046AE','deb Atlas 5304','',13453,'Reentered','1966-06-23 00:00:00','1966-06-19 00:00:00',89.28,220,258,28.78,'Intermediate' from dual union all
select 22520,'022520','1992-091C','Kosmos-2225 debris','[FR6 part]',64803,'Reentered','1993-02-23 00:00:00','1993-02-22 00:00:00',90.83,175,456,64.58,'Intermediate' from dual union all
select 22521,'022521','1993-011A','Asuka','ASTRO-D',65113,'Reentered','2001-03-02 00:00:00','1993-03-25 00:00:00',96.51,537,646,31.1,'Intermediate' from dual union all
select 22522,'022522','1993-011B','M-3B','M-3B',65113,'Reentered','2003-12-10 00:00:00','1993-03-25 00:00:00',96.06,526,614,31.1,'Intermediate' from dual union all
select 22523,'022523','1993-011C','Asuka despin weight','',65113,'Reentered','1994-12-20 00:00:00','1993-03-23 00:00:00',95.82,522,595,31.1,'Intermediate' from dual union all
select 22524,'022524','1993-010G','SOZ','SOZ',65112,'Reentered','1998-09-05 00:00:00','1993-03-22 00:00:00',340.26,403,19131,64.88,'Highly Ellip' from dual union all
select 22525,'022525','1992-091D','Kosmos-2225 debris','[FR6 part]',64803,'Reentered','1993-02-25 00:00:00','1993-02-23 00:00:00',91.32,204,475,65.16,'Intermediate' from dual union all
select 22526,'022526','1992-091E','Kosmos-2225 debris','[FR6 part]',64803,'Reentered','1993-02-22 00:00:00','1993-02-20 00:00:00',90.58,206,400,64.62,'Intermediate' from dual union all
select 22527,'022527','1992-091F','Kosmos-2225 debris','[FR6 part]',64803,'Reentered','1993-02-27 00:00:00','1993-02-23 00:00:00',92.26,208,563,64.61,'Intermediate' from dual union all
select 22528,'022528','1993-010H','SOZ','SOZ',65112,'Reentered','1998-11-18 00:00:00','1993-03-22 00:00:00',340.25,402,19131,64.85,'Highly Ellip' from dual union all
select 22529,'022529','1992-091G','Kosmos-2225 debris','[FR6 part]',64803,'Reentered','1993-02-24 00:00:00','1993-02-23 00:00:00',90.83,194,436,64.79,'Intermediate' from dual union all
select 2253,'002253','1966-056A','Pageos','Pageos A  (RCS 1.5)',13463,'Reentered','2016-09-02 00:00:00','1966-07-24 00:00:00',181.4,3978,4496,87.01,'Medium' from dual union all
select 22530,'022530','1993-012A','Progress M-16','Progress 7K-TGM No. 216',65114,'Deorbited','1993-03-27 00:00:00','1993-03-01 00:00:00',92.33,387,391,51.62,'Intermediate' from dual union all
select 22531,'022531','1993-012B','11S510','11S510',65114,'Reentered','1993-02-23 00:00:00','1993-02-22 00:00:00',88.37,177,211,51.64,'Intermediate' from dual union all
select 22532,'022532','1970-025QN','deb Agena D','',21502,'In Earth orbit','','1993-02-23 00:00:00',106.61,1064,1075,100,'Sun-Synch' from dual union all
select 22533,'022533','1970-025QP','deb Agena D','',21502,'In Earth orbit','','1993-02-26 00:00:00',106.61,1069,1071,100.01,'Sun-Synch' from dual union all
select 22534,'022534','1993-011D','Asuka despin weight','',65113,'Reentered','1999-11-25 00:00:00','1993-03-22 00:00:00',96.27,549,611,30.87,'Intermediate' from dual union all
select 22535,'022535','1992-070F','deb IRIS','',64782,'Reentered','2005-01-08 00:00:00','1993-02-28 00:00:00',151.69,294,5794,41.19,'Medium' from dual union all
select 22536,'022536','1978-094C','deb Kosmos-1043','',45289,'Reentered','1993-03-11 00:00:00','1993-02-24 00:00:00',93.28,433,437,81.17,'Intermediate' from dual union all
select 22537,'022537','1992-093GX','deb 11S772','',64805,'In Earth orbit','','1993-02-24 00:00:00',103.93,844,1045,71.18,'Intermediate' from dual union all
select 22538,'022538','1992-093GY','deb 11S772','',64805,'Reentered','2003-10-30 00:00:00','1993-02-24 00:00:00',103.66,820,1043,71.08,'Intermediate' from dual union all
select 22539,'022539','1992-093GZ','deb 11S772','',64805,'Reentered','1998-09-25 00:00:00','1993-02-26 00:00:00',98.85,553,855,71.11,'Intermediate' from dual union all
select 2254,'002254','1966-057A','Kosmos-122','Meteor 11F614 No. 5',13464,'Reentered','1989-11-14 00:00:00','1966-07-29 00:00:00',97.06,583,652,64.98,'Intermediate' from dual union all
select 22540,'022540','1992-093HA','deb 11S772','',64805,'Reentered','2000-08-14 00:00:00','1993-02-24 00:00:00',100.36,679,873,71.01,'Intermediate' from dual union all
select 22541,'022541','1992-093HB','deb 11S772','',64805,'Reentered','1998-09-10 00:00:00','1993-02-26 00:00:00',99.95,659,854,70.99,'Intermediate' from dual union all
select 22542,'022542','1992-093HC','deb 11S772','',64805,'Reentered','2000-07-15 00:00:00','1993-02-25 00:00:00',100.76,710,879,70.77,'Intermediate' from dual union all
select 22543,'022543','1992-093HD','deb 11S772','',64805,'In Earth orbit','','1993-02-24 00:00:00',103.95,835,1056,71.34,'Intermediate' from dual union all
select 22544,'022544','1992-093HE','deb 11S772','',64805,'In Earth orbit','','1993-02-28 00:00:00',104.61,805,1148,71.27,'Intermediate' from dual union all
select 22545,'022545','1992-093HF','deb 11S772','',64805,'In Earth orbit','','1993-02-28 00:00:00',109.36,848,1547,71.56,'Intermediate' from dual union all
select 22546,'022546','1992-093HG','deb 11S772','',64805,'In Earth orbit','','1993-02-28 00:00:00',103.95,829,1061,71.37,'Intermediate' from dual union all
select 22547,'022547','1992-093HH','deb 11S772','',64805,'In Earth orbit','','1993-03-09 00:00:00',103.6,824,1034,71.47,'Intermediate' from dual union all
select 22548,'022548','1992-093HJ','deb 11S772','',64805,'Reentered','1993-05-19 00:00:00','1993-03-09 00:00:00',96.98,496,733,71.39,'Intermediate' from dual union all
select 22549,'022549','1992-093HK','deb 11S772','',64805,'In Earth orbit','','1993-03-07 00:00:00',103.76,835,1038,71.23,'Intermediate' from dual union all
select 2255,'002255','1966-056B','Agena D 6311','Agena D 6311',13463,'In Earth orbit','','1966-10-20 00:00:00',181.22,4202,4258,86.94,'Medium' from dual union all
select 22550,'022550','1992-093HL','deb 11S772','',64805,'Reentered','2013-12-16 00:00:00','1993-03-15 00:00:00',103.27,769,1057,71.21,'Intermediate' from dual union all
select 22551,'022551','1992-093HM','deb 11S772','',64805,'In Earth orbit','','1993-03-15 00:00:00',103.32,844,988,70.87,'Intermediate' from dual union all
select 22552,'022552','1992-093HN','deb 11S772','',64805,'In Earth orbit','','1993-03-15 00:00:00',102.81,844,940,71.1,'Intermediate' from dual union all
select 22553,'022553','1992-093HP','deb 11S772','',64805,'Reentered','2016-01-14 00:00:00','1993-03-15 00:00:00',102,809,898,71.45,'Intermediate' from dual union all
select 22554,'022554','1992-093HQ','deb 11S772','',64805,'In Earth orbit','','1993-03-16 00:00:00',103.6,831,1027,71.4,'Intermediate' from dual union all
select 22555,'022555','1992-093HR','deb 11S772','',64805,'Reentered','2014-04-21 00:00:00','1993-03-16 00:00:00',103.99,839,1056,70.54,'Intermediate' from dual union all
select 22556,'022556','1992-093HS','deb 11S772','',64805,'Reentered','2002-02-08 00:00:00','1993-03-16 00:00:00',102.5,817,937,70.68,'Intermediate' from dual union all
select 22557,'022557','1993-013A','Raduga','Gran'' No. 42L',65115,'In Earth orbit','','1993-04-24 00:00:00',1436.16,35769,35807,1.43,'Stationary' from dual union all
select 22558,'022558','1993-013B','8S812','8S812',65115,'Reentered','1993-03-27 00:00:00','1993-03-26 00:00:00',88,173,178,51.66,'Intermediate' from dual union all
select 22559,'022559','1993-013C','Perekhodnik','Sredniy perekhodnik',65115,'Reentered','1993-03-25 00:00:00','1993-03-25 00:00:00',88.13,175,188,51.65,'Intermediate' from dual union all
select 2256,'002256','1966-056C','Pageos canister half','Pageos canister half',13463,'In Earth orbit','','1966-09-16 00:00:00',181.34,4192,4277,86.92,'Medium' from dual union all
select 22560,'022560','1974-089FG','deb Delta 104','',33692,'Reentered','2001-09-30 00:00:00','1993-04-24 00:00:00',104.65,867,1090,101.68,'Sun-Synch' from dual union all
select 22561,'022561','1993-014A','EKA-1','EKA-1',65116,'In Earth orbit','','1993-04-25 00:00:00',101.42,682,970,75.76,'Intermediate' from dual union all
select 22562,'022562','1993-014B','Start','Start',65116,'In Earth orbit','','1993-04-24 00:00:00',101.03,682,933,75.77,'Intermediate' from dual union all
select 22563,'022563','1993-015A','UHF F/O F1','UFO F1',65117,'In Earth orbit','','1993-07-04 00:00:00',1450.88,36062,36089,27.24,'Near-synch' from dual union all
select 22564,'022564','1993-015B','Centaur AC-74','Centaur I AC-74',65117,'Reentered','2000-01-06 00:00:00','1993-04-25 00:00:00',191.57,215,9044,27.43,'Medium' from dual union all
select 22565,'022565','1993-016A','Kosmos-2237','Tselina-2',65118,'In Earth orbit','','1993-04-25 00:00:00',101.94,848,852,71.02,'Intermediate' from dual union all
select 22566,'022566','1993-016B','11S772','11S772',65118,'In Earth orbit','','1993-04-25 00:00:00',101.84,837,855,71.01,'Intermediate' from dual union all
select 22567,'022567','1993-014C','deb Start','deb Start',65116,'In Earth orbit','','1993-04-25 00:00:00',101.54,683,981,75.76,'Intermediate' from dual union all
select 22568,'022568','1993-014D','deb Start','deb Start',65116,'In Earth orbit','','1993-04-25 00:00:00',101.53,682,981,75.75,'Intermediate' from dual union all
select 22569,'022569','1993-013E','SOZ','SOZ',65115,'Reentered','1993-10-21 00:00:00','1993-04-24 00:00:00',647.64,310,36526,47.24,'GEO Transfer' from dual union all
select 2257,'002257','1966-057B','8A92E','8A92E',13464,'Reentered','1984-01-20 00:00:00','1966-10-25 00:00:00',97.04,529,705,64.99,'Intermediate' from dual union all
select 22570,'022570','1993-013F','SOZ','SOZ',65115,'Reentered','1993-09-29 00:00:00','1993-04-27 00:00:00',646.21,263,36500,47.24,'GEO Transfer' from dual union all
select 22571,'022571','1992-093HT','deb 11S772','',64805,'In Earth orbit','','1993-03-22 00:00:00',105.04,831,1162,71.36,'Intermediate' from dual union all
select 22572,'022572','1992-093HU','deb 11S772','',64805,'Reentered','2004-04-21 00:00:00','1993-03-26 00:00:00',103.59,839,1018,70.75,'Intermediate' from dual union all
select 22573,'022573','1992-093HV','deb 11S772','',64805,'Reentered','2012-06-08 00:00:00','1993-03-27 00:00:00',103.47,824,1022,71.02,'Intermediate' from dual union all
select 22574,'022574','1992-093HW','deb 11S772','',64805,'In Earth orbit','','1993-03-26 00:00:00',104.04,817,1081,71.08,'Intermediate' from dual union all
select 22575,'022575','1993-016C','Sep Motor Cover','Sep Motor Cover',65118,'In Earth orbit','','1993-06-13 00:00:00',105.06,842,1152,71,'Intermediate' from dual union all
select 22576,'022576','1993-016D','Sep Motor Cover','Sep Motor Cover',65118,'In Earth orbit','','1993-06-14 00:00:00',105.02,845,1146,71,'Intermediate' from dual union all
select 22577,'022577','1993-016E','Sep Motor Cover','Sep Motor Cover',65118,'In Earth orbit','','1993-04-29 00:00:00',100,670,847,71.04,'Intermediate' from dual union all
select 22578,'022578','1993-016F','Sep Motor Cover','Sep Motor Cover',65118,'In Earth orbit','','1993-04-28 00:00:00',99.91,662,847,71.07,'Intermediate' from dual union all
select 22579,'022579','1993-016G','deb 11S772','',65118,'In Earth orbit','','1993-06-18 00:00:00',100.13,680,849,71.03,'Intermediate' from dual union all
select 2258,'002258','1966-058A','Explorer 33','AIMP D',13465,'Deep Space','1966-07-02 00:00:00','1966-10-12 00:00:00',25214,79527,477379,21.66,'Cislunar' from dual union all
select 22580,'022580','1993-016H','deb 11S772','',65118,'Reentered','2000-05-18 00:00:00','1993-04-25 00:00:00',100.29,695,850,71.18,'Intermediate' from dual union all
select 22581,'022581','1993-017A','GPS IIA-10','Navstar GPS SVN 31',65119,'In Earth orbit','','1993-05-02 00:00:00',717.96,20076,20288,54.91,'Medium' from dual union all
select 22582,'022582','1993-017B','SEDS 1','SEDS 1 End Mass/Tether',65119,'Reentered','1993-03-30 00:00:00','1993-03-30 00:00:00',93.61,184,719,36,'Intermediate' from dual union all
select 22583,'022583','1993-017C','Delta 219','Delta SSPS AJ10-118K',65119,'Reentered','1999-05-05 00:00:00','1993-04-29 00:00:00',100.84,305,1292,36.19,'Intermediate' from dual union all
select 22584,'022584','1993-017D','Star 48B','Star 48B S/N 10077-18',65119,'Reentered','1999-02-10 00:00:00','1993-04-29 00:00:00',352.59,184,20125,34.86,'Highly Ellip' from dual union all
select 22585,'022585','1993-018A','Kosmos-2238','US-PM',65120,'Reentered','1994-12-10 00:00:00','1993-06-21 00:00:00',92.77,404,417,65.03,'Intermediate' from dual union all
select 22586,'022586','1993-018B','11S692','11S692',65120,'Reentered','1993-03-31 00:00:00','1993-03-30 00:00:00',92.83,396,431,65.01,'Intermediate' from dual union all
select 22587,'022587','1993-011E','deb Asuka','',65113,'Reentered','1998-03-22 00:00:00','1993-03-29 00:00:00',95.97,524,607,31.11,'Intermediate' from dual union all
select 22588,'022588','1993-019A','Progress M-17','Progress 7K-TGM No. 217',65121,'Deorbited','1994-03-03 00:00:00','1993-05-04 00:00:00',92.37,391,391,51.62,'Intermediate' from dual union all
select 22589,'022589','1993-019B','11S510','11S510',65121,'Reentered','1993-04-01 00:00:00','1993-03-31 00:00:00',88.33,178,206,51.64,'Intermediate' from dual union all
select 2259,'002259','1966-058B','Delta 39','DSV-3E-3 S/N 20207',13465,'Reentered','1966-11-17 00:00:00','1966-08-02 00:00:00',99.92,181,1329,28.82,'Intermediate' from dual union all
select 22590,'022590','1993-020A','Kosmos-2239','Parus',65122,'In Earth orbit','','1993-06-15 00:00:00',104.73,964,1000,82.93,'Intermediate' from dual union all
select 22591,'022591','1993-020B','S3M','S3M',65122,'In Earth orbit','','1993-05-01 00:00:00',104.62,964,990,82.93,'Intermediate' from dual union all
select 22592,'022592','1993-021A','Kosmos-2240','Kobal''t No. 541',65123,'Landed','1993-06-07 00:00:00','1993-05-02 00:00:00',89.93,189,353,62.84,'Intermediate' from dual union all
select 22593,'022593','1993-021B','11S510','11S510',65123,'Reentered','1993-04-07 00:00:00','1993-04-07 00:00:00',87.86,149,189,62.82,'Intermediate' from dual union all
select 22594,'022594','1993-022A','Kosmos-2241','Oko',65124,'In Earth orbit','','1993-05-06 00:00:00',717.76,663,39691,62.96,'Molniya' from dual union all
select 22595,'022595','1993-022B','11S510','11S510',65124,'Reentered','1993-05-10 00:00:00','1993-04-13 00:00:00',91.92,205,533,62.77,'Intermediate' from dual union all
select 22596,'022596','1993-022C','BOZ','BOZ',65124,'Reentered','1993-04-22 00:00:00','1993-04-13 00:00:00',91.42,172,517,62.82,'Intermediate' from dual union all
select 22597,'022597','1993-022D','Blok-2BL','Blok-2BL',65124,'Reentered','2009-02-11 00:00:00','1993-05-11 00:00:00',703.15,663,38968,62.97,'Molniya' from dual union all
select 22598,'022598','1992-093HX','deb 11S772','',64805,'In Earth orbit','','1993-04-08 00:00:00',103.39,844,994,70.65,'Intermediate' from dual;
insert into SATELLITE
select 22599,'022599','1993-014E','deb Start','',65116,'In Earth orbit','','1993-04-26 00:00:00',101.41,682,969,75.77,'Intermediate' from dual union all
select 226,'000226','62 Beta 1','Tiros 4','NASA A-9',8185,'In Earth orbit','','1962-03-13 00:00:00',100.38,716,838,48.29,'Intermediate' from dual union all
select 2260,'002260','1966-058C','FW4D','FW4D S/N 00001',13465,'Deep Space','1966-07-03 00:00:00','1966-07-01 00:00:00',47898.87,293,860741,28.9,'Cislunar' from dual union all
select 22600,'022600','1993-016J','deb 11S772','',65118,'In Earth orbit','','1993-05-02 00:00:00',104.3,843,1081,70.91,'Intermediate' from dual union all
select 22601,'022601','1993-016K','deb 11S772','',65118,'In Earth orbit','','1993-04-27 00:00:00',104.61,845,1108,71.05,'Intermediate' from dual union all
select 22602,'022602','1993-016L','deb 11S772','',65118,'In Earth orbit','','1993-05-10 00:00:00',105,845,1143,71.04,'Intermediate' from dual union all
select 22603,'022603','1993-016M','deb 11S772','',65118,'In Earth orbit','','1993-04-27 00:00:00',104.49,854,1088,71,'Intermediate' from dual union all
select 22604,'022604','1993-016N','deb 11S772','',65118,'In Earth orbit','','1993-05-02 00:00:00',104.65,854,1103,71,'Intermediate' from dual union all
select 22605,'022605','1993-016P','deb 11S772','',65118,'In Earth orbit','','1993-04-28 00:00:00',103.85,842,1038,71,'Intermediate' from dual union all
select 22606,'022606','1993-016Q','deb 11S772','',65118,'In Earth orbit','','1993-05-03 00:00:00',104.35,846,1082,71,'Intermediate' from dual union all
select 22607,'022607','1993-016R','deb 11S772','',65118,'In Earth orbit','','1993-05-02 00:00:00',104.7,853,1108,71,'Intermediate' from dual union all
select 22608,'022608','1993-016S','deb 11S772','',65118,'In Earth orbit','','1993-05-02 00:00:00',104.04,854,1045,71,'Intermediate' from dual union all
select 22609,'022609','1993-016T','deb 11S772','',65118,'In Earth orbit','','1993-05-10 00:00:00',104.23,853,1064,70.99,'Intermediate' from dual union all
select 2261,'002261','1966-055B','Fairing','Fairing',13462,'Reentered','1966-07-04 00:00:00','1966-07-03 00:00:00',89.25,182,293,80.07,'Intermediate' from dual union all
select 22610,'022610','1993-016U','deb 11S772','',65118,'In Earth orbit','','1993-05-10 00:00:00',105.33,852,1167,71,'Intermediate' from dual union all
select 22611,'022611','1993-016V','deb 11S772','',65118,'In Earth orbit','','1993-04-27 00:00:00',104.76,855,1112,71,'Intermediate' from dual union all
select 22612,'022612','1993-016W','deb 11S772','',65118,'In Earth orbit','','1993-04-25 00:00:00',103.87,854,1029,71,'Intermediate' from dual union all
select 22613,'022613','1993-016X','deb 11S772','',65118,'In Earth orbit','','1993-04-25 00:00:00',105.25,853,1160,71,'Intermediate' from dual union all
select 22614,'022614','1993-016Y','deb 11S772','',65118,'In Earth orbit','','1993-05-03 00:00:00',104.16,855,1056,71.01,'Intermediate' from dual union all
select 22615,'022615','1993-016Z','deb 11S772','',65118,'In Earth orbit','','1993-04-25 00:00:00',104.37,855,1075,71.01,'Intermediate' from dual union all
select 22616,'022616','1993-016AA','deb 11S772','',65118,'In Earth orbit','','1993-05-05 00:00:00',104.99,854,1134,71,'Intermediate' from dual union all
select 22617,'022617','1993-016AB','deb 11S772','',65118,'In Earth orbit','','1993-04-25 00:00:00',104.36,855,1074,71,'Intermediate' from dual union all
select 22618,'022618','1993-016AC','deb 11S772','',65118,'In Earth orbit','','1993-04-29 00:00:00',104.49,851,1090,71,'Intermediate' from dual union all
select 22619,'022619','1993-016AD','deb 11S772','',65118,'In Earth orbit','','1993-04-30 00:00:00',104.89,852,1127,71,'Intermediate' from dual union all
select 2262,'002262','1966-058D','Delta attach clamp','',13465,'Reentered','1966-09-07 00:00:00','1966-08-14 00:00:00',96.51,203,979,28.81,'Intermediate' from dual union all
select 22620,'022620','1993-016AE','deb 11S772','',65118,'In Earth orbit','','1993-04-30 00:00:00',104.1,854,1051,71,'Intermediate' from dual union all
select 22621,'022621','1993-023A','Discovery (STS-56)','OV-103',65125,'Landed','1993-04-17 00:00:00','1993-04-10 00:00:00',90.41,292,298,57,'Intermediate' from dual union all
select 22622,'022622','1986-017GZ','Garbage bag','',59210,'Reentered','1993-10-15 00:00:00','1993-04-11 00:00:00',92.38,388,395,51.63,'Intermediate' from dual union all
select 22623,'022623','1993-023B','Spartan 201','Spartan 201',65125,'Landed Attached','1993-04-17 00:00:00','1993-04-11 00:00:00',90.41,292,298,57,'Intermediate' from dual union all
select 22624,'022624','1993-013D','Blok DM-2 No. 67L','11S861 No. 67L',65115,'In Earth orbit','','1993-04-24 00:00:00',1472.98,36092,36919,1.76,'Drift GEO' from dual union all
select 22625,'022625','1989-089AW','deb COBE','',62366,'Reentered','1997-12-25 00:00:00','1993-07-05 00:00:00',102.48,849,902,99,'Sun-Synch' from dual union all
select 22626,'022626','1993-024A','Kosmos-2242','Tselina-R',65126,'In Earth orbit','','1993-05-18 00:00:00',97.72,633,666,82.53,'Intermediate' from dual union all
select 22627,'022627','1993-024B','S5M','S5M',65126,'In Earth orbit','','1993-06-21 00:00:00',97.7,630,667,82.53,'Intermediate' from dual union all
select 22628,'022628','1993-011F','deb Asuka','',65113,'Reentered','1998-02-10 00:00:00','1993-04-18 00:00:00',95.89,523,600,31.1,'Intermediate' from dual union all
select 22629,'022629','1993-022E','Ullage rocket','Ullage rocket',65124,'Reentered','1993-04-29 00:00:00','1993-04-17 00:00:00',91.6,200,507,62.81,'Intermediate' from dual union all
select 2263,'002263','1966-058E','Delta attach clamp','',13465,'Reentered','1966-08-29 00:00:00','1966-07-08 00:00:00',100.76,181,1409,28.76,'Intermediate' from dual union all
select 22630,'022630','1993-022F','Ullage rocket','Ullage rocket',65124,'Reentered','1993-05-05 00:00:00','1993-04-18 00:00:00',91.84,205,525,62.81,'Intermediate' from dual union all
select 22631,'022631','1986-017HA','EO-13 EVA-1 debris (MSB)','',59210,'Reentered','1993-07-25 00:00:00','1993-04-20 00:00:00',92.32,385,392,51.63,'Intermediate' from dual union all
select 22632,'022632','1986-017HB','EO-13 EVA-1 debris (MSB)','',59210,'Reentered','1993-04-25 00:00:00','1993-04-21 00:00:00',92.17,381,381,51.62,'Intermediate' from dual union all
select 22633,'022633','1993-025A','Molniya-3','Molniya-3 No. 57',65127,'Reentered','2004-01-25 00:00:00','1993-05-21 00:00:00',717.92,643,39719,62.85,'Molniya' from dual union all
select 22634,'022634','1993-025B','11S510','11S510',65127,'Reentered','1993-05-30 00:00:00','1993-04-29 00:00:00',91.8,208,518,62.8,'Intermediate' from dual union all
select 22635,'022635','1993-025C','BOZ','BOZ',65127,'Reentered','1993-05-12 00:00:00','1993-04-29 00:00:00',91.69,177,538,62.82,'Intermediate' from dual union all
select 22636,'022636','1993-025D','Blok-ML','Blok-ML No. 76712-542',65127,'Reentered','2004-11-15 00:00:00','1993-05-21 00:00:00',735,672,40529,62.83,'Molniya' from dual union all
select 22637,'022637','1986-017HC','EO-13 EVA-1 debris (MSB)','',59210,'Reentered','1994-03-02 00:00:00','1993-04-25 00:00:00',92.38,391,391,51.62,'Intermediate' from dual union all
select 22638,'022638','1993-026A','Alexis','Alexis',65128,'In Earth orbit','','1993-09-18 00:00:00',100.68,747,835,69.93,'Intermediate' from dual union all
select 22639,'022639','1993-026B','Orbcomm CDS-2','VSUME',65128,'In Earth orbit','','1994-03-19 00:00:00',100.77,751,840,69.92,'Intermediate' from dual union all
select 2264,'002264','1966-046AF','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',90.03,255,297,28.74,'Intermediate' from dual union all
select 22640,'022640','1993-027A','Columbia (STS-55)','OV-102',65129,'Landed','1993-05-06 00:00:00','1993-04-28 00:00:00',90.52,295,305,28.46,'Intermediate' from dual union all
select 22641,'022641','1993-028A','Kosmos-2243','Kometa No. 16',65130,'Reentered','1993-05-06 00:00:00','1993-04-29 00:00:00',88.77,190,238,70.35,'Intermediate' from dual union all
select 22642,'022642','1993-028B','11S510','11S510',65130,'Reentered','1993-05-01 00:00:00','1993-04-30 00:00:00',88.14,168,197,70.36,'Intermediate' from dual union all
select 22643,'022643','1993-029A','Kosmos-2244','US-PM',65131,'Reentered','1995-03-18 00:00:00','1993-05-31 00:00:00',92.76,403,417,65.03,'Intermediate' from dual union all
select 22644,'022644','1993-029B','11S692','11S692',65131,'Reentered','1993-04-28 00:00:00','1993-04-28 00:00:00',88.07,103,255,65.01,'Intermediate' from dual union all
select 22645,'022645','1993-016AF','deb 11S772','',65118,'In Earth orbit','','1993-05-08 00:00:00',104.91,849,1131,71,'Intermediate' from dual union all
select 22646,'022646','1993-030A','Kosmos-2245','Strela-3',65132,'In Earth orbit','','1993-06-11 00:00:00',113.93,1397,1416,82.59,'Intermediate' from dual union all
select 22647,'022647','1993-030B','Kosmos-2246','Strela-3',65132,'In Earth orbit','','1993-06-11 00:00:00',113.96,1400,1417,82.58,'Intermediate' from dual union all
select 22648,'022648','1993-030C','Kosmos-2247','Strela-3',65132,'In Earth orbit','','1993-06-10 00:00:00',114,1403,1417,82.58,'Intermediate' from dual union all
select 22649,'022649','1993-030D','Kosmos-2248','Strela-3',65132,'In Earth orbit','','1993-06-12 00:00:00',113.99,1402,1417,82.58,'Intermediate' from dual union all
select 2265,'002265','1966-046AG','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',92.96,331,507,29.1,'Intermediate' from dual union all
select 22650,'022650','1993-030E','Kosmos-2249','Strela-3',65132,'In Earth orbit','','1993-06-19 00:00:00',114,1403,1417,82.59,'Intermediate' from dual union all
select 22651,'022651','1993-030F','Kosmos-2250','Strela-3',65132,'In Earth orbit','','1993-06-10 00:00:00',113.96,1400,1417,82.58,'Intermediate' from dual union all
select 22652,'022652','1993-030G','S5M','S5M',65132,'In Earth orbit','','1993-06-17 00:00:00',114.01,1404,1417,82.59,'Intermediate' from dual union all
select 22653,'022653','1993-031A','Astra 1C','Astra 1C',65133,'In Earth orbit','','1993-06-26 00:00:00',1435.98,35729,35840,.01,'Stationary' from dual union all
select 22654,'022654','1993-031B','Arsene-OSCAR-24','Arsene',65133,'In Earth orbit','','1993-06-20 00:00:00',1012.62,17185,36866,1.12,'Medium' from dual union all
select 22655,'022655','1993-031C','Ariane H10 V56','Ariane H10 S/N T426',65133,'In Earth orbit','','1993-06-14 00:00:00',627.24,222,35567,5.18,'GEO Transfer' from dual union all
select 22656,'022656','1993-031D','Cyclade','Cyclade',65133,'Reentered','2007-01-02 00:00:00','1993-06-15 00:00:00',626.87,223,35547,5.19,'GEO Transfer' from dual union all
select 22657,'022657','1993-032A','GPS IIA-11','Navstar GPS SVN 37',65134,'In Earth orbit','','1993-06-14 00:00:00',717.88,20033,20327,54.95,'Medium' from dual union all
select 22658,'022658','1993-032B','Delta 220','Delta SSPS AJ10-118K',65134,'Reentered','2002-12-08 00:00:00','1993-06-14 00:00:00',96.72,472,732,21.92,'Equatorial' from dual union all
select 22659,'022659','1993-032C','Star 48B','Star 48B S/N 10079-18',65134,'Reentered','2001-01-12 00:00:00','1993-06-13 00:00:00',352.56,191,20116,34.94,'Highly Ellip' from dual union all
select 2266,'002266','1966-046AH','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',94.69,216,792,29.1,'Intermediate' from dual union all
select 22660,'022660','1992-093HY','deb 11S772','',64805,'Reentered','2002-01-18 00:00:00','1993-05-13 00:00:00',101.04,732,884,70.99,'Intermediate' from dual union all
select 22661,'022661','1993-016AG','deb 11S772','',65118,'Reentered','1999-11-08 00:00:00','1993-05-17 00:00:00',100.29,681,863,71.14,'Intermediate' from dual union all
select 22662,'022662','1992-093HZ','deb 11S772','',64805,'Reentered','2015-02-26 00:00:00','1993-05-18 00:00:00',104.11,840,1065,70.53,'Intermediate' from dual union all
select 22663,'022663','1993-033A','Resurs-F','Resurs-F2 17F42 No. 9',65135,'Landed','1993-06-20 00:00:00','1993-05-28 00:00:00',89.08,225,233,82.57,'Intermediate' from dual union all
select 22664,'022664','1993-033B','11S510','11S510',65135,'Reentered','1993-05-24 00:00:00','1993-05-24 00:00:00',87.39,135,156,82.57,'Intermediate' from dual union all
select 22665,'022665','1992-093JA','deb 11S772','',64805,'In Earth orbit','','1993-06-06 00:00:00',102.92,842,952,71.08,'Intermediate' from dual union all
select 22666,'022666','1993-034A','Progress M-18','Progress 7K-TGM No. 218',65136,'Deorbited','1993-07-04 00:00:00','1993-05-31 00:00:00',92.33,388,390,51.62,'Intermediate' from dual union all
select 22667,'022667','1993-034B','11S510','11S510',65136,'Reentered','1993-05-25 00:00:00','1993-05-24 00:00:00',87.35,140,147,51.6,'Intermediate' from dual union all
select 22668,'022668','1988-077D','deb Transtage D-3','',61326,'In Earth orbit','','1993-06-01 00:00:00',702.53,400,39200,26.7,'GEO Transfer' from dual union all
select 22669,'022669','1988-077E','deb Transtage D-3','',61326,'In Earth orbit','','1993-06-01 00:00:00',702.53,400,39200,26.7,'GEO Transfer' from dual union all
select 2267,'002267','1966-046AJ','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',92.47,235,557,28.3,'Intermediate' from dual union all
select 22670,'022670','1992-072C','Ariane V54 main debris','',64784,'In Earth orbit','','1993-05-24 00:00:00',481.69,345,27609,7.19,'Highly Ellip' from dual union all
select 22671,'022671','1993-035A','Molniya-1','Molniya-1T No. 81',65137,'In Earth orbit','','1993-06-26 00:00:00',717.68,460,39890,62.83,'Molniya' from dual union all
select 22672,'022672','1993-035B','11S510','11S510',65137,'Reentered','1993-06-16 00:00:00','1993-06-03 00:00:00',90.21,202,368,62.83,'Intermediate' from dual union all
select 22673,'022673','1993-035C','BOZ','BOZ',65137,'Reentered','1993-06-06 00:00:00','1993-06-01 00:00:00',89.79,175,353,62.82,'Intermediate' from dual union all
select 22674,'022674','1993-035D','Blok-ML','Blok-ML No. 76724-489',65137,'Reentered','2011-07-29 00:00:00','1993-07-05 00:00:00',732.98,471,40630,62.85,'Molniya' from dual union all
select 22675,'022675','1993-036A','Kosmos-2251','Strela-2M',65138,'In Earth orbit','','1993-07-23 00:00:00',100.73,781,806,74.04,'Intermediate' from dual union all
select 22676,'022676','1993-036B','S3M','S3M',65138,'In Earth orbit','','1993-07-24 00:00:00',100.63,776,800,74.05,'Intermediate' from dual union all
select 22677,'022677','1986-017HD','EO-13 EVA-2 debris (MSB)','',59210,'Reentered','1993-08-09 00:00:00','1993-06-19 00:00:00',92.39,388,395,51.62,'Intermediate' from dual union all
select 22678,'022678','1986-017HE','EO-13 EVA-2 debris (MSB)','',59210,'Reentered','1993-06-30 00:00:00','1993-06-23 00:00:00',92.06,372,380,51.62,'Intermediate' from dual union all
select 22679,'022679','1986-017HF','EO-13 EVA-2 debris (MSB)','',59210,'Reentered','1993-11-01 00:00:00','1993-06-25 00:00:00',92.41,390,395,51.62,'Intermediate' from dual union all
select 2268,'002268','1966-046AK','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',89.52,227,274,28.3,'Intermediate' from dual union all
select 22680,'022680','1986-017HG','EO-13 EVA-2 debris (MSB)','',59210,'Reentered','1993-06-27 00:00:00','1993-06-24 00:00:00',91.57,344,360,51.6,'Intermediate' from dual union all
select 22681,'022681','1993-033C','Blenda','Sunshade',65135,'Reentered','1993-06-22 00:00:00','1993-06-20 00:00:00',88.71,198,224,82.59,'Intermediate' from dual union all
select 22682,'022682','1993-033D','Solnechnaya Batareya','Solar panel module',65135,'Reentered','1993-06-22 00:00:00','1993-06-21 00:00:00',88.72,205,218,82.56,'Intermediate' from dual union all
select 22683,'022683','1989-089E','deb COBE','',62366,'Reentered','1998-10-23 00:00:00','1993-07-15 00:00:00',102.48,850,902,99.01,'Sun-Synch' from dual union all
select 22684,'022684','1993-037A','Endeavour (STS-57)','OV-105',65139,'Landed','1993-07-01 00:00:00','1993-06-23 00:00:00',93.35,409,467,28.46,'Intermediate' from dual union all
select 22685,'022685','1993-033E','PTDU cover','PTDU cover',65135,'Reentered','1993-06-25 00:00:00','1993-06-22 00:00:00',90.47,222,373,82.55,'Intermediate' from dual union all
select 22686,'022686','1993-033F','PTDU cover','PTDU cover',65135,'Reentered','1993-06-23 00:00:00','1993-06-21 00:00:00',90.19,196,372,82.56,'Intermediate' from dual union all
select 22687,'022687','1993-038A','Kosmos-2252','Strela-3',65140,'In Earth orbit','','1993-07-30 00:00:00',114,1404,1415,82.58,'Intermediate' from dual union all
select 22688,'022688','1993-038B','Kosmos-2253','Strela-3',65140,'In Earth orbit','','1993-08-02 00:00:00',114.15,1409,1424,82.59,'Intermediate' from dual;
insert into SATELLITE
select 22689,'022689','1993-038C','Kosmos-2254','Strela-3',65140,'In Earth orbit','','1993-07-30 00:00:00',113.84,1393,1412,82.58,'Intermediate' from dual union all
select 2269,'002269','1966-046AL','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',89.4,169,321,28.3,'Intermediate' from dual union all
select 22690,'022690','1993-038D','Kosmos-2255','Strela-3',65140,'In Earth orbit','','1993-08-05 00:00:00',114,1406,1413,82.59,'Intermediate' from dual union all
select 22691,'022691','1993-038E','Kosmos-2256','Strela-3',65140,'In Earth orbit','','1993-08-05 00:00:00',113.94,1402,1412,82.57,'Intermediate' from dual union all
select 22692,'022692','1993-038F','Kosmos-2257','Strela-3',65140,'In Earth orbit','','1993-07-25 00:00:00',114.08,1409,1419,82.58,'Intermediate' from dual union all
select 22693,'022693','1993-038G','S5M','S5M',65140,'In Earth orbit','','1993-07-31 00:00:00',114.76,1409,1480,82.59,'Intermediate' from dual union all
select 22694,'022694','1993-039A','Galaxy 4H','Galaxy IVH',65141,'In Earth orbit','','1993-10-21 00:00:00',1429.5,35531,35784,.33,'Drift GEO' from dual union all
select 22695,'022695','1989-089F','deb COBE','',62366,'Reentered','1999-02-22 00:00:00','1993-08-01 00:00:00',102.14,849,871,99,'Sun-Synch' from dual union all
select 22696,'022696','1993-040A','Resurs-F','Resurs-F1 14F43 No. 57',65142,'Landed','1993-07-12 00:00:00','1993-07-02 00:00:00',89,216,235,82.59,'Intermediate' from dual union all
select 22697,'022697','1993-040B','11S510','11S510',65142,'Reentered','1993-06-27 00:00:00','1993-06-27 00:00:00',87.47,140,159,82.58,'Intermediate' from dual union all
select 22698,'022698','1993-041A','RADCAL','RADCAL',65143,'In Earth orbit','','1993-08-04 00:00:00',101.3,754,887,89.56,'Polar' from dual union all
select 22699,'022699','1993-041B','Star 20 E-55','Star 20 E-55',65143,'In Earth orbit','','1993-07-31 00:00:00',101.35,756,890,89.56,'Polar' from dual union all
select 227,'000227','62 Beta 2','Tiros 4 despin weight','Despin weight',8185,'In Earth orbit','','1962-04-15 00:00:00',101.38,708,939,48.16,'Intermediate' from dual union all
select 2270,'002270','1966-046AM','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',89.43,171,322,28.34,'Intermediate' from dual union all
select 22700,'022700','1993-042A','GPS IIA-12','Navstar GPS SVN 39',65144,'In Earth orbit','','1993-07-27 00:00:00',717.96,20120,20244,54.73,'Medium' from dual union all
select 22701,'022701','1993-042B','Delta 221','Delta SSPS AJ10-118K',65144,'Reentered','1993-08-30 00:00:00','1993-07-26 00:00:00',93.19,185,677,25.71,'Intermediate' from dual union all
select 22702,'022702','1993-042C','Star 48B','Star 48B S/N 10081-18',65144,'Reentered','2000-02-05 00:00:00','1993-08-05 00:00:00',350.72,184,20008,34.76,'Highly Ellip' from dual union all
select 22703,'022703','1986-017HH','EO-13 EVA-2 debris (MSB)','',59210,'Reentered','1993-11-06 00:00:00','1993-06-29 00:00:00',92.31,384,392,51.62,'Intermediate' from dual union all
select 22704,'022704','1993-043A','Soyuz TM-17','Soyuz 7K-STM No. 66',65145,'Landed','1994-01-14 00:00:00','1993-07-31 00:00:00',92.36,388,393,51.62,'Intermediate' from dual union all
select 22705,'022705','1993-043B','11S510','11S510',65145,'Reentered','1993-07-03 00:00:00','1993-07-03 00:00:00',87.14,131,135,51.61,'Intermediate' from dual union all
select 22706,'022706','1993-041C','RADCAL despin weight','',65143,'In Earth orbit','','1993-07-25 00:00:00',100.75,772,817,89.3,'Polar' from dual union all
select 22707,'022707','1993-041D','RADCAL despin weight','',65143,'In Earth orbit','','1993-07-26 00:00:00',101.82,727,963,89.86,'Polar' from dual union all
select 22708,'022708','1993-034C','Progress M-18 part','Progress part',65136,'Reentered','1994-02-11 00:00:00','1993-07-07 00:00:00',92.28,380,393,51.63,'Intermediate' from dual union all
select 22709,'022709','1993-044A','Kosmos-2258','US-PM',65146,'Reentered','1995-06-08 00:00:00','1993-08-06 00:00:00',92.76,403,417,65.05,'Intermediate' from dual union all
select 2271,'002271','1966-046AN','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',95.17,243,811,29.8,'Intermediate' from dual union all
select 22710,'022710','1993-044B','11S692','11S692',65146,'Reentered','1993-07-07 00:00:00','1993-07-07 00:00:00',88.4,105,286,65.02,'Intermediate' from dual union all
select 22711,'022711','1993-040C','KDU','KDU',65142,'Reentered','1993-07-27 00:00:00','1993-07-12 00:00:00',88.97,219,228,82.59,'Intermediate' from dual union all
select 22712,'022712','1993-040D','PTDU cover','PTDU cover',65142,'Reentered','1993-07-17 00:00:00','1993-07-13 00:00:00',90.88,222,414,82.58,'Intermediate' from dual union all
select 22713,'022713','1993-040E','PTDU cover','PTDU cover',65142,'Reentered','1993-07-18 00:00:00','1993-07-13 00:00:00',90.58,222,384,82.59,'Intermediate' from dual union all
select 22714,'022714','1993-040F','Blenda','Sunshade',65142,'Reentered','1993-07-14 00:00:00','1993-07-13 00:00:00',88.49,198,202,82.58,'Intermediate' from dual union all
select 22715,'022715','1993-040G','KDU part','KDU part',65142,'Reentered','1993-07-14 00:00:00','1993-07-13 00:00:00',88.19,181,189,82.59,'Intermediate' from dual union all
select 22716,'022716','1993-045A','Kosmos-2259','Kobal''t No. 535',65147,'Landed','1993-07-25 00:00:00','1993-07-16 00:00:00',89.56,164,342,67.13,'Intermediate' from dual union all
select 22717,'022717','1993-045B','11S510','11S510',65147,'Reentered','1993-07-19 00:00:00','1993-07-16 00:00:00',89.17,159,307,67.12,'Intermediate' from dual union all
select 22718,'022718','1989-061K','deb USA-40','',62338,'In Earth orbit','','1998-09-13 00:00:00',181.58,500,7987,56.99,'Medium' from dual union all
select 22719,'022719','1993-046A','USA 93','DSCS III B-9',65148,'In Earth orbit','','1993-09-01 00:00:00',1435.5,35750,35800,0,'Stationary' from dual union all
select 2272,'002272','1966-046AP','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',92.63,201,606,28.36,'Intermediate' from dual union all
select 22720,'022720','1993-046B','Centaur AC-104','Centaur II(3) AC-104',65148,'Reentered','1998-01-03 00:00:00','1993-07-19 00:00:00',629.14,173,35714,26.5,'GEO Transfer' from dual union all
select 22721,'022721','1993-047A','Kosmos-2260','Oblik No. 831 (Resurs-T)',65149,'Landed','1993-08-05 00:00:00','1993-07-24 00:00:00',89.87,240,296,82.29,'Intermediate' from dual union all
select 22722,'022722','1993-047B','11S510','11S510',65149,'Reentered','1993-07-24 00:00:00','1993-07-23 00:00:00',88.23,174,199,82.28,'Intermediate' from dual union all
select 22723,'022723','1993-048A','Hispasat 1B','Hispasat 1B',65150,'In Earth orbit','','1993-10-03 00:00:00',1436.11,35777,35797,.04,'Stationary' from dual union all
select 22724,'022724','1993-048B','Insat 2B','Insat 2B',65150,'In Earth orbit','','1993-09-14 00:00:00',1436.11,35773,35801,.14,'Stationary' from dual union all
select 22725,'022725','1993-048C','Ariane H10+ V58','Ariane H10+ S/N T430',65150,'In Earth orbit','','1993-08-22 00:00:00',643.31,315,36300,6.99,'GEO Transfer' from dual union all
select 22726,'022726','1993-048D','Spelda 30 S/N UV18','Mini-Spelda',65150,'Reentered','2000-05-20 00:00:00','1993-08-28 00:00:00',624.97,233,35439,7.12,'GEO Transfer' from dual union all
select 22727,'022727','1981-059D','deb NOAA 7','',52582,'Reentered','1994-09-11 00:00:00','1993-08-01 00:00:00',101.54,829,835,98.89,'Sun-Synch' from dual union all
select 22728,'022728','1981-059E','deb NOAA 7','',52582,'Reentered','1994-03-15 00:00:00','1993-08-01 00:00:00',101.78,831,855,98.9,'Sun-Synch' from dual union all
select 22729,'022729','1993-049A','Molniya-3','Molniya-3 No. 58',65151,'Reentered','2013-12-31 00:00:00','1993-09-05 00:00:00',717.74,449,39904,62.83,'Molniya' from dual union all
select 2273,'002273','1966-046AQ','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',91.05,253,400,28.72,'Intermediate' from dual union all
select 22730,'022730','1993-049B','11S510','11S510',65151,'Reentered','1993-08-30 00:00:00','1993-08-11 00:00:00',90.44,210,383,62.82,'Intermediate' from dual union all
select 22731,'022731','1993-049C','BOZ','BOZ',65151,'Reentered','1993-08-17 00:00:00','1993-08-08 00:00:00',90.11,178,382,62.8,'Intermediate' from dual union all
select 22732,'022732','1993-049D','Blok-ML','Blok-ML No. 77324-543',65151,'Reentered','2013-01-15 00:00:00','1993-09-08 00:00:00',699.55,445,39007,62.85,'Molniya' from dual union all
select 22733,'022733','1993-047C','KDU','KDU',65149,'Reentered','1993-09-14 00:00:00','1993-08-05 00:00:00',89.8,239,291,82.29,'Intermediate' from dual union all
select 22734,'022734','1993-047D','Blenda','Sunshade',65149,'Reentered','1993-08-12 00:00:00','1993-08-05 00:00:00',89.66,237,278,82.34,'Intermediate' from dual union all
select 22735,'022735','1993-047E','PTDU cover','PTDU cover',65149,'Reentered','1993-08-18 00:00:00','1993-08-05 00:00:00',91.71,249,468,82.3,'Intermediate' from dual union all
select 22736,'022736','1993-047F','KDU part','KDU part',65149,'Reentered','1993-08-09 00:00:00','1993-08-06 00:00:00',89.65,235,280,82.29,'Intermediate' from dual union all
select 22737,'022737','1993-047G','PTDU cover','PTDU cover',65149,'Reentered','1993-08-15 00:00:00','1993-08-05 00:00:00',92.06,257,495,82.33,'Intermediate' from dual union all
select 22738,'022738','1993-046C','IABS-3','IABS',65148,'In Earth orbit','','1993-07-30 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 22739,'022739','1993-050A','NOAA 13','NOAA I',65152,'In Earth orbit','','1993-09-08 00:00:00',102.05,849,863,98.91,'Sun-Synch' from dual union all
select 2274,'002274','1966-046AR','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',90.84,201,431,28.64,'Intermediate' from dual union all
select 22740,'022740','1993-050B','NOAA 13 solar array band','',65152,'Reentered','2009-12-16 00:00:00','1993-09-08 00:00:00',102.09,854,862,98.91,'Sun-Synch' from dual union all
select 22741,'022741','1993-051A','Kosmos-2261','Oko',65153,'Reentered','2012-12-31 00:00:00','1993-09-10 00:00:00',717.7,626,39725,62.91,'Molniya' from dual union all
select 22742,'022742','1993-051B','11S510','11S510',65153,'Reentered','1993-10-06 00:00:00','1993-08-19 00:00:00',92.09,216,538,62.83,'Intermediate' from dual union all
select 22743,'022743','1993-051C','BOZ','BOZ',65153,'Reentered','1993-09-04 00:00:00','1993-08-17 00:00:00',91.6,180,527,62.87,'Intermediate' from dual union all
select 22744,'022744','1993-051D','Blok-2BL','Blok-2BL No. 77353-694',65153,'In Earth orbit','','1993-09-12 00:00:00',707.75,628,39231,62.94,'Molniya' from dual union all
select 22745,'022745','1993-052A','Progress M-19','Progress 7K-TGM No. 219',65154,'Deorbited','1993-10-13 00:00:00','1993-09-09 00:00:00',92.31,385,391,51.62,'Intermediate' from dual union all
select 22746,'022746','1993-052B','11S510','11S510',65154,'Reentered','1993-08-13 00:00:00','1993-08-12 00:00:00',87.78,158,172,51.64,'Intermediate' from dual union all
select 22747,'022747','1989-089G','deb COBE','',62366,'Reentered','1999-01-15 00:00:00','1993-08-19 00:00:00',102.98,847,953,98.85,'Sun-Synch' from dual union all
select 22748,'022748','1989-089H','deb COBE','',62366,'Reentered','1997-12-20 00:00:00','1993-08-26 00:00:00',102.52,862,894,98.98,'Sun-Synch' from dual union all
select 22749,'022749','1989-089J','deb COBE','',62366,'Reentered','1997-12-13 00:00:00','1993-08-26 00:00:00',102.53,861,896,98.99,'Sun-Synch' from dual union all
select 2275,'002275','1966-046AS','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',91.11,233,425,28.76,'Intermediate' from dual union all
select 22750,'022750','1989-089K','deb COBE','',62366,'Reentered','1998-02-17 00:00:00','1993-08-27 00:00:00',102.54,863,895,98.99,'Sun-Synch' from dual union all
select 22751,'022751','1989-089L','deb COBE','',62366,'Reentered','1998-10-06 00:00:00','1993-08-27 00:00:00',102.55,869,889,98.98,'Sun-Synch' from dual union all
select 22752,'022752','1989-089M','deb COBE','',62366,'Reentered','1998-05-06 00:00:00','1993-08-23 00:00:00',102.48,860,891,99,'Sun-Synch' from dual union all
select 22753,'022753','1989-089N','deb COBE','',62366,'Reentered','1998-03-05 00:00:00','1993-08-25 00:00:00',102.48,857,895,98.98,'Sun-Synch' from dual union all
select 22754,'022754','1989-089P','deb COBE','',62366,'Reentered','1997-04-28 00:00:00','1993-08-27 00:00:00',102.48,831,921,99,'Sun-Synch' from dual union all
select 22755,'022755','1989-089Q','deb COBE','',62366,'Reentered','1997-06-19 00:00:00','1993-08-25 00:00:00',102.53,868,889,99,'Sun-Synch' from dual union all
select 22756,'022756','1989-089R','deb COBE','',62366,'Reentered','1998-04-05 00:00:00','1993-08-25 00:00:00',102.4,853,892,99,'Sun-Synch' from dual union all
select 22757,'022757','1989-089S','deb COBE','',62366,'Reentered','1997-06-23 00:00:00','1993-08-16 00:00:00',102.36,813,928,98.97,'Sun-Synch' from dual union all
select 22758,'022758','1989-089T','deb COBE','',62366,'Reentered','1997-08-12 00:00:00','1993-08-22 00:00:00',102.3,856,878,98.97,'Sun-Synch' from dual union all
select 22759,'022759','1989-089U','deb COBE','',62366,'Reentered','1997-08-29 00:00:00','1993-08-26 00:00:00',102.42,843,904,99,'Sun-Synch' from dual union all
select 2276,'002276','1966-046AT','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-21 00:00:00',89.91,235,305,28.69,'Intermediate' from dual union all
select 22760,'022760','1989-089V','deb COBE','',62366,'Reentered','1997-03-06 00:00:00','1993-08-26 00:00:00',102.48,843,909,99.01,'Sun-Synch' from dual union all
select 22761,'022761','1989-089W','deb COBE','',62366,'Reentered','1998-06-14 00:00:00','1993-08-25 00:00:00',102.43,865,882,99.01,'Sun-Synch' from dual union all
select 22762,'022762','1989-089X','deb COBE','',62366,'Reentered','1996-12-12 00:00:00','1993-08-25 00:00:00',102.28,834,900,99.01,'Sun-Synch' from dual union all
select 22763,'022763','1989-089Y','deb COBE','',62366,'Reentered','1997-09-30 00:00:00','1993-08-25 00:00:00',102.39,794,950,99.02,'Sun-Synch' from dual union all
select 22764,'022764','1989-089Z','deb COBE','',62366,'Reentered','1996-11-10 00:00:00','1993-08-25 00:00:00',102.32,833,904,99,'Sun-Synch' from dual union all
select 22765,'022765','1989-089AA','deb COBE','',62366,'Reentered','1997-01-20 00:00:00','1993-08-26 00:00:00',102.36,854,886,98.99,'Sun-Synch' from dual union all
select 22766,'022766','1989-089AB','deb COBE','',62366,'Reentered','1998-05-04 00:00:00','1993-08-26 00:00:00',102.35,839,900,99,'Sun-Synch' from dual union all
select 22767,'022767','1989-089AC','deb COBE','',62366,'Reentered','1998-04-07 00:00:00','1993-08-26 00:00:00',102.55,872,887,98.99,'Sun-Synch' from dual union all
select 22768,'022768','1989-089AD','deb COBE','',62366,'Reentered','1998-07-11 00:00:00','1993-08-26 00:00:00',102.37,843,898,99.01,'Sun-Synch' from dual union all
select 22769,'022769','1989-089AE','deb COBE','',62366,'Reentered','1998-06-21 00:00:00','1993-08-25 00:00:00',102.38,860,883,98.98,'Sun-Synch' from dual union all
select 2277,'002277','1966-046AU','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-21 00:00:00',90.65,172,441,28.43,'Intermediate' from dual union all
select 22770,'022770','1989-089AF','deb COBE','',62366,'Reentered','1998-02-12 00:00:00','1993-08-26 00:00:00',102.33,857,881,99.02,'Sun-Synch' from dual union all
select 22771,'022771','1989-089AG','deb COBE','',62366,'Reentered','1997-10-05 00:00:00','1993-08-26 00:00:00',102.37,852,890,99.02,'Sun-Synch' from dual union all
select 22772,'022772','1989-089AH','deb COBE','',62366,'Reentered','1998-04-21 00:00:00','1993-08-25 00:00:00',102.42,853,894,99.01,'Sun-Synch' from dual union all
select 22773,'022773','1989-089AJ','deb COBE','',62366,'Reentered','1998-01-31 00:00:00','1993-08-25 00:00:00',102.54,864,894,99,'Sun-Synch' from dual union all
select 22774,'022774','1989-089AK','deb COBE','',62366,'Reentered','1998-04-06 00:00:00','1993-08-23 00:00:00',102.34,842,897,99.03,'Sun-Synch' from dual union all
select 22775,'022775','1989-089AL','deb COBE','',62366,'Reentered','1998-09-06 00:00:00','1993-08-26 00:00:00',102.41,843,902,99.01,'Sun-Synch' from dual union all
select 22776,'022776','1989-089AM','deb COBE','',62366,'Reentered','1997-09-27 00:00:00','1993-08-26 00:00:00',102.35,835,905,99.01,'Sun-Synch' from dual union all
select 22777,'022777','1993-053A','Resurs-F','Resurs-F1 14F43 No. 56',65155,'Landed','1993-09-10 00:00:00','1993-08-31 00:00:00',89.06,222,234,82.59,'Intermediate' from dual union all
select 22778,'022778','1993-053B','11S510','11S510',65155,'Reentered','1993-08-27 00:00:00','1993-08-26 00:00:00',87.98,161,188,82.58,'Intermediate' from dual;
insert into SATELLITE
select 22779,'022779','1993-054A','GPS IIA-13','Navstar GPS SVN 35',65156,'In Earth orbit','','1993-10-01 00:00:00',718,20109,20257,54.89,'Medium' from dual union all
select 2278,'002278','1966-046AV','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-21 00:00:00',90.68,216,400,28.79,'Intermediate' from dual union all
select 22780,'022780','1993-054B','Delta 222','Delta SSPS AJ10-118K',65156,'Reentered','2002-04-05 00:00:00','1993-09-30 00:00:00',96.38,452,719,22.04,'Equatorial' from dual union all
select 22781,'022781','1993-054C','Star 48B','Star 48B S/N 10082-18',65156,'Reentered','2007-01-18 00:00:00','1993-09-30 00:00:00',352.46,193,20108,34.81,'Highly Ellip' from dual union all
select 22782,'022782','1993-055A','Meteor-2','Meteor-2 No. 24',65157,'In Earth orbit','','1993-10-04 00:00:00',104.11,937,968,82.55,'Intermediate' from dual union all
select 22783,'022783','1993-055B','Temisat','Temisat',65157,'In Earth orbit','','1993-09-30 00:00:00',104.09,936,968,82.55,'Intermediate' from dual union all
select 22784,'022784','1993-055C','S5M','S5M',65157,'In Earth orbit','','1993-09-30 00:00:00',104.1,937,968,82.55,'Intermediate' from dual union all
select 22785,'022785','1993-026C','deb Alexis','',65128,'In Earth orbit','','1999-05-25 00:00:00',100.91,743,860,69.94,'Intermediate' from dual union all
select 22786,'022786','1993-026D','deb Alexis','',65128,'In Earth orbit','','1999-05-25 00:00:00',100.97,750,860,69.94,'Intermediate' from dual union all
select 22787,'022787','1993-056A','UHF F/O F2','UFO F2',65158,'In Earth orbit','','1993-10-07 00:00:00',1436.09,35129,36444,5.09,'Inclined GEO' from dual union all
select 22788,'022788','1993-056B','Centaur AC-75','Centaur I AC-75',65158,'Reentered','2016-03-15 00:00:00','1993-10-04 00:00:00',272.38,223,14859,26.97,'Highly Ellip' from dual union all
select 22789,'022789','1993-057A','Kosmos-2262','Don',65159,'Exploded','1993-12-18 00:00:00','1993-10-07 00:00:00',89.61,207,304,64.89,'Intermediate' from dual union all
select 2279,'002279','1966-046AW','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',88.49,100,300,28.66,'Intermediate' from dual union all
select 22790,'022790','1993-057B','11S510','11S510',65159,'Reentered','1993-09-11 00:00:00','1993-09-10 00:00:00',88.29,157,223,64.9,'Intermediate' from dual union all
select 22791,'022791','1993-053C','Blenda','Sunshade',65155,'Reentered','1993-09-12 00:00:00','1993-09-12 00:00:00',87.98,166,183,82.58,'Intermediate' from dual union all
select 22792,'022792','1993-053D','KDU','KDU',65155,'Reentered','1993-09-24 00:00:00','1993-09-15 00:00:00',88.77,209,219,82.58,'Intermediate' from dual union all
select 22793,'022793','1993-053E','PTDU cover','PTDU cover',65155,'Reentered','1993-09-15 00:00:00','1993-09-14 00:00:00',87.97,153,195,82.58,'Intermediate' from dual union all
select 22794,'022794','1993-053F','PTDU cover','PTDU cover',65155,'Reentered','1993-09-13 00:00:00','1993-09-13 00:00:00',89.4,183,306,82.58,'Intermediate' from dual union all
select 22795,'022795','1993-058A','Discovery (STS-51)','OV-103',65160,'Landed','1993-09-22 00:00:00','1993-09-14 00:00:00',90.29,271,307,28.46,'Intermediate' from dual union all
select 22796,'022796','1993-058B','ACTS','ACTS',65160,'In Earth orbit','','1993-10-14 00:00:00',1474.42,26536,46530,1.44,'Near-synch' from dual union all
select 22797,'022797','1993-058D','TOS','Orbus 21',65160,'In Earth orbit','','1993-10-13 00:00:00',715.96,323,39942,15.44,'GEO Transfer' from dual union all
select 22798,'022798','1993-058C','ORFEUS-SPAS','ASTRO-SPAS',65160,'Landed Attached','1993-09-22 00:00:00','1993-09-14 00:00:00',90.6,266,341,28.46,'Intermediate' from dual union all
select 22799,'022799','1993-058E','deb TOS','',65160,'Reentered','1993-09-16 00:00:00','1993-09-15 00:00:00',89.72,243,279,28.47,'Intermediate' from dual union all
select 228,'000228','62 Beta 3','Tiros 4 despin weight','Despin weight',8185,'In Earth orbit','','1962-03-12 00:00:00',99.49,706,762,48.42,'Intermediate' from dual union all
select 2280,'002280','1966-046AX','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',90.52,223,377,28.19,'Intermediate' from dual union all
select 22800,'022800','1993-058F','deb TOS','',65160,'Reentered','1993-09-16 00:00:00','1993-09-15 00:00:00',90.21,256,314,28.63,'Intermediate' from dual union all
select 22801,'022801','1993-050C','NOAA 13 solar array band','',65152,'In Earth orbit','','1993-09-14 00:00:00',102.04,853,858,98.91,'Sun-Synch' from dual union all
select 22802,'022802','1993-059A','Kosmos-2263','Tselina-2',65161,'In Earth orbit','','1993-10-17 00:00:00',101.95,847,854,71,'Intermediate' from dual union all
select 22803,'022803','1993-059B','11S772','11S772',65161,'In Earth orbit','','1993-10-16 00:00:00',101.7,826,853,70.98,'Intermediate' from dual union all
select 22804,'022804','1993-059C','Sep Motor Cover','Sep Motor Cover',65161,'In Earth orbit','','1993-10-19 00:00:00',105.15,843,1160,70.98,'Intermediate' from dual union all
select 22805,'022805','1993-059D','Sep Motor Cover','Sep Motor Cover',65161,'In Earth orbit','','1993-10-19 00:00:00',105.23,845,1165,70.97,'Intermediate' from dual union all
select 22806,'022806','1993-059E','Sep Motor Cover','Sep Motor Cover',65161,'In Earth orbit','','1993-10-19 00:00:00',105,844,1144,71.02,'Intermediate' from dual union all
select 22807,'022807','1993-059F','Sep Motor Cover','Sep Motor Cover',65161,'In Earth orbit','','1993-10-19 00:00:00',105.1,845,1154,71.02,'Intermediate' from dual union all
select 22808,'022808','1993-060A','Kosmos-2264','US-PM',65162,'Reentered','1995-08-07 00:00:00','1993-10-20 00:00:00',92.77,403,418,65.02,'Intermediate' from dual union all
select 22809,'022809','1993-060B','11S692','11S692',65162,'Reentered','1993-09-17 00:00:00','1993-09-17 00:00:00',89.08,112,346,64.99,'Intermediate' from dual union all
select 2281,'002281','1966-046AY','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',90.3,209,370,28.53,'Intermediate' from dual union all
select 22810,'022810','1986-017HJ','EO-14 EVA-1 debris (Ripana)','',59210,'Reentered','1993-10-11 00:00:00','1993-09-23 00:00:00',92.03,373,376,51.62,'Intermediate' from dual union all
select 22811,'022811','1986-017HK','EO-14 EVA-1 debris (Ripana)','',59210,'Reentered','1994-02-24 00:00:00','1993-09-24 00:00:00',92.28,384,389,51.61,'Intermediate' from dual union all
select 22812,'022812','1992-093JB','deb 11S772','',64805,'In Earth orbit','','1993-09-23 00:00:00',104.36,840,1088,70.69,'Intermediate' from dual union all
select 22813,'022813','1991-076G','deb USA 74','',64245,'In Earth orbit','','1993-09-30 00:00:00',107.58,1050,1180,63.43,'Intermediate' from dual union all
select 22814,'022814','1991-076H','deb USA 74','',64245,'In Earth orbit','','1993-09-30 00:00:00',107.58,1050,1180,63.43,'Intermediate' from dual union all
select 22815,'022815','1986-017HL','EO-14 EVA-2 debris (Ripana)','',59210,'Reentered','1994-01-22 00:00:00','1993-09-23 00:00:00',92.27,384,388,51.62,'Intermediate' from dual union all
select 22816,'022816','1992-093JC','deb 11S772','',64805,'In Earth orbit','','1993-09-20 00:00:00',104.85,838,1136,71.49,'Intermediate' from dual union all
select 22817,'022817','1986-017HM','EO-14 EVA-2 debris (Ripana)','',59210,'Reentered','1994-01-15 00:00:00','1993-09-26 00:00:00',92.6,385,418,51.64,'Intermediate' from dual union all
select 22818,'022818','1988-077F','deb Transtage D-3','',61326,'In Earth orbit','','1993-09-30 00:00:00',702.53,400,39200,26.7,'GEO Transfer' from dual union all
select 22819,'022819','1986-017HN','Ripana section','',59210,'Reentered','1994-03-20 00:00:00','1993-09-25 00:00:00',92.28,386,387,51.62,'Intermediate' from dual union all
select 2282,'002282','1966-046AZ','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',89.32,203,279,28.36,'Intermediate' from dual union all
select 22820,'022820','1989-089AN','deb COBE','',62366,'Reentered','1997-12-13 00:00:00','1993-09-26 00:00:00',102.32,855,882,99,'Sun-Synch' from dual union all
select 22821,'022821','1993-011G','deb Asuka','',65113,'Reentered','1998-09-19 00:00:00','1993-09-24 00:00:00',95.52,470,618,31.38,'Intermediate' from dual union all
select 22822,'022822','1993-011H','deb Asuka','',65113,'Reentered','2000-02-09 00:00:00','1993-09-24 00:00:00',95.75,493,616,30.89,'Intermediate' from dual union all
select 22823,'022823','1993-061A','SPOT 3','SPOT 3',65163,'In Earth orbit','','1993-10-31 00:00:00',101.4,824,826,98.74,'Sun-Synch' from dual union all
select 22824,'022824','1993-061B','Stella','Stella',65163,'In Earth orbit','','1993-10-28 00:00:00',100.89,797,805,98.68,'Sun-Synch' from dual union all
select 22825,'022825','1993-061C','AMRAD-OSCAR-27','Eyesat 1',65163,'In Earth orbit','','1993-11-01 00:00:00',100.86,793,805,98.68,'Sun-Synch' from dual union all
select 22826,'022826','1993-061D','Healthsat 2','Healthsat 2',65163,'In Earth orbit','','1993-11-01 00:00:00',100.84,792,805,98.68,'Sun-Synch' from dual union all
select 22827,'022827','1993-061E','ITAMSat-OSCAR-26','ITAMsat',65163,'In Earth orbit','','1993-11-01 00:00:00',100.85,792,806,98.68,'Sun-Synch' from dual union all
select 22828,'022828','1993-061F','Kitsat-OSCAR-25','KITSAT-2',65163,'In Earth orbit','','1993-11-07 00:00:00',100.85,794,804,98.67,'Sun-Synch' from dual union all
select 22829,'022829','1993-061G','POSAT-OSCAR-28','Posat 1',65163,'In Earth orbit','','1993-10-26 00:00:00',100.83,790,806,98.67,'Sun-Synch' from dual union all
select 2283,'002283','1966-046BA','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',88.49,100,300,28.65,'Intermediate' from dual union all
select 22830,'022830','1993-061H','Ariane H10 V59','Ariane H10 S/N T431',65163,'In Earth orbit','','1993-11-01 00:00:00',100.82,789,806,98.58,'Sun-Synch' from dual union all
select 22831,'022831','1986-017HP','EO-14 EVA-3 debris','',59210,'Reentered','1993-10-20 00:00:00','1993-09-29 00:00:00',92.24,383,386,51.62,'Intermediate' from dual union all
select 22832,'022832','1986-017HQ','EO-14 EVA-3 debris','',59210,'Reentered','1993-10-19 00:00:00','1993-09-30 00:00:00',92.14,377,381,51.62,'Intermediate' from dual union all
select 22833,'022833','1986-017HR','EO-14 EVA-3 debris','',59210,'Reentered','1993-11-01 00:00:00','1993-09-30 00:00:00',92.24,382,387,51.62,'Intermediate' from dual union all
select 22834,'022834','1986-017HS','Ripana section','',59210,'Reentered','1994-09-03 00:00:00','1993-10-01 00:00:00',92.29,385,389,51.62,'Intermediate' from dual union all
select 22835,'022835','1986-017HT','Ripana section','',59210,'Reentered','1994-04-22 00:00:00','1993-09-30 00:00:00',92.25,378,392,51.62,'Intermediate' from dual union all
select 22836,'022836','1993-062A','Raduga','Gran'' No. 41L',65164,'In Earth orbit','','1993-10-31 00:00:00',1436.01,35763,35807,1.43,'Stationary' from dual union all
select 22837,'022837','1993-062B','8S812','8S812',65164,'Reentered','1993-09-30 00:00:00','1993-09-30 00:00:00',88.3,189,192,51.65,'Intermediate' from dual union all
select 22838,'022838','1993-062C','Perekhodnik','Sredniy perekhodnik',65164,'Reentered','1993-10-01 00:00:00','1993-10-01 00:00:00',88.24,186,188,51.67,'Intermediate' from dual union all
select 22839,'022839','1993-062D','Blok DM-2 No. 36L','11S861 No. 36L',65164,'In Earth orbit','','1993-11-10 00:00:00',1437.82,35749,35892,1.43,'Drift GEO' from dual union all
select 2284,'002284','1966-046BB','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-21 00:00:00',89.49,239,260,28.67,'Intermediate' from dual union all
select 22840,'022840','1991-009CK','deb S3M','',64178,'In Earth orbit','','1993-10-23 00:00:00',116.72,1390,1677,73.96,'Intermediate' from dual union all
select 22841,'022841','1991-009CL','deb S3M','',64178,'In Earth orbit','','1993-10-25 00:00:00',117.76,1477,1684,74.04,'Intermediate' from dual union all
select 22842,'022842','1991-009CM','deb S3M','',64178,'In Earth orbit','','1993-10-24 00:00:00',116.64,1417,1643,73.89,'Intermediate' from dual union all
select 22843,'022843','1991-009CN','deb S3M','',64178,'In Earth orbit','','1993-10-17 00:00:00',116.77,1344,1728,74.08,'Intermediate' from dual union all
select 22844,'022844','1991-062J','deb Yohkoh','',64231,'Reentered','1995-06-30 00:00:00','1993-10-06 00:00:00',94.52,423,568,31.56,'Intermediate' from dual union all
select 22845,'022845','1991-086G','deb Interkosmos-25','',64255,'Reentered','2016-04-08 00:00:00','1993-10-03 00:00:00',119.05,427,2850,82.51,'Intermediate' from dual union all
select 22846,'022846','1992-093JD','deb 11S772','',64805,'Reentered','2001-11-04 00:00:00','1994-08-01 00:00:00',101.81,788,901,70.77,'Intermediate' from dual union all
select 22847,'022847','1992-038C','SAMPEX despin weight','',64750,'Reentered','1999-11-24 00:00:00','1993-10-06 00:00:00',96.84,484,731,81.66,'Intermediate' from dual union all
select 22848,'022848','1992-038D','SAMPEX despin weight','',64750,'Reentered','1998-01-09 00:00:00','1993-10-07 00:00:00',95.13,484,565,81.68,'Intermediate' from dual union all
select 22849,'022849','1992-089D','Star 48B yo weight','',64801,'In Earth orbit','','1993-07-26 00:00:00',337.31,180,19166,35.03,'Highly Ellip' from dual union all
select 2285,'002285','1966-046BC','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-21 00:00:00',89.38,225,263,28.57,'Intermediate' from dual union all
select 22850,'022850','1992-021E','deb Ariane V50','',64733,'In Earth orbit','','1993-10-05 00:00:00',640.89,914,35577,3.28,'GEO Transfer' from dual union all
select 22851,'022851','1992-021F','Ariane V50 main debris','RCS 42 sq m',64733,'Reentered','2002-08-29 00:00:00','1993-10-04 00:00:00',624.76,237,35424,3.36,'GEO Transfer' from dual union all
select 22852,'022852','1989-089AP','deb COBE','',62366,'Reentered','1998-05-14 00:00:00','1994-06-04 00:00:00',102.05,819,893,98.93,'Sun-Synch' from dual union all
select 22853,'022853','1989-089AQ','deb COBE','',62366,'Reentered','1997-09-07 00:00:00','1994-08-01 00:00:00',101.24,804,831,99.04,'Sun-Synch' from dual union all
select 22854,'022854','1989-089AR','deb COBE','',62366,'Reentered','1996-07-23 00:00:00','1994-07-13 00:00:00',101.13,786,838,99.01,'Sun-Synch' from dual union all
select 22855,'022855','1989-089AS','deb COBE','',62366,'Reentered','1998-03-25 00:00:00','1994-08-06 00:00:00',101.6,797,871,99.03,'Sun-Synch' from dual union all
select 22856,'022856','1989-089AT','deb COBE','',62366,'Reentered','1999-01-01 00:00:00','1994-08-01 00:00:00',101.83,809,882,98.96,'Sun-Synch' from dual union all
select 22857,'022857','1989-089AU','deb COBE','',62366,'Reentered','1998-04-03 00:00:00','1994-05-21 00:00:00',101.92,842,858,98.99,'Sun-Synch' from dual union all
select 22858,'022858','1989-089AV','deb COBE','',62366,'Reentered','1997-10-18 00:00:00','1994-08-01 00:00:00',101.33,811,832,99.05,'Sun-Synch' from dual union all
select 22859,'022859','1993-063A','Jian Bing 93','Jianbing-1A No. 5',65165,'Reentered','1993-10-28 00:00:00','1993-10-15 00:00:00',89.34,204,279,56.95,'Intermediate' from dual union all
select 2286,'002286','1966-046BD','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',89.87,234,302,28.69,'Intermediate' from dual union all
select 22860,'022860','1993-063B','CZ-2C Stage 2','CZ-2C Stage 2',65165,'Reentered','1993-10-22 00:00:00','1993-10-10 00:00:00',89.41,206,284,56.95,'Intermediate' from dual union all
select 22861,'022861','1993-063C','CZ-2C sep motor cover','',65165,'Reentered','1993-10-12 00:00:00','1993-10-10 00:00:00',90.81,197,432,56.73,'Intermediate' from dual union all
select 22862,'022862','1993-063D','CZ-2C sep motor cover','',65165,'Reentered','1993-10-11 00:00:00','1993-10-10 00:00:00',90.61,199,410,56.66,'Intermediate' from dual union all
select 22863,'022863','1993-063E','CZ-2C sep motor cover','',65165,'Reentered','1993-10-11 00:00:00','1993-10-10 00:00:00',90.08,193,364,57.25,'Intermediate' from dual union all
select 22864,'022864','1993-063F','CZ-2C sep motor cover','',65165,'Reentered','1993-10-10 00:00:00','1993-10-09 00:00:00',90.8,202,425,57.19,'Intermediate' from dual union all
select 22865,'022865','1993-063G','CZ-2C part','',65165,'Reentered','1993-10-09 00:00:00','1993-10-08 00:00:00',89.19,199,270,56.95,'Intermediate' from dual union all
select 22866,'022866','1986-017HU','Garbage bag','',59210,'Reentered','1994-03-03 00:00:00','1994-01-13 00:00:00',91.19,327,338,51.61,'Intermediate' from dual union all
select 22867,'022867','1993-064A','Progress M-20','Progress 7K-TGM No. 220',65166,'Deorbited','1993-11-21 00:00:00','1993-10-18 00:00:00',92.39,387,397,51.62,'Intermediate' from dual union all
select 22868,'022868','1993-064B','11S510','11S510',65166,'Reentered','1993-10-13 00:00:00','1993-10-12 00:00:00',88.5,184,216,51.65,'Intermediate' from dual;
insert into SATELLITE
select 22869,'022869','1993-065A','Columbia (STS-58)','OV-102',65167,'Landed','1993-11-01 00:00:00','1993-10-20 00:00:00',90.21,281,289,39.02,'Intermediate' from dual union all
select 2287,'002287','1966-046BE','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-20 00:00:00',89.99,242,306,28.62,'Intermediate' from dual union all
select 22870,'022870','1993-063H','FSW-1 RV','FSW-1 RV',65165,'Reentered','1996-03-12 00:00:00','1993-11-07 00:00:00',117.72,180,2977,56.54,'Intermediate' from dual union all
select 22871,'022871','1993-066A','INTELSAT 701','INTELSAT 701',65168,'In Earth orbit','','1993-11-29 00:00:00',1436.09,35657,35916,.03,'Stationary' from dual union all
select 22872,'022872','1986-017HV','EO-14 EVA-4 debris','',59210,'Reentered','1993-12-15 00:00:00','1993-11-16 00:00:00',91.83,363,365,51.6,'Intermediate' from dual union all
select 22873,'022873','1986-017HW','EO-14 EVA-4 debris','',59210,'Reentered','1994-05-26 00:00:00','1993-11-17 00:00:00',92.2,380,385,51.62,'Intermediate' from dual union all
select 22874,'022874','1986-017HX','EO-14 EVA-4 debris','',59210,'Reentered','1993-12-26 00:00:00','1993-11-12 00:00:00',91.74,358,362,51.62,'Intermediate' from dual union all
select 22875,'022875','1993-067A','Kosmos-2265','Taifun-1B',65169,'Reentered','2003-08-11 00:00:00','1993-11-25 00:00:00',103.61,291,1568,82.94,'Intermediate' from dual union all
select 22876,'022876','1993-067B','S3M','S3M',65169,'Reentered','2000-03-02 00:00:00','1993-11-28 00:00:00',103.45,286,1558,82.94,'Intermediate' from dual union all
select 22877,'022877','1993-068A','GPS IIA-14','Navstar GPS SVN 34',65170,'In Earth orbit','','1993-11-25 00:00:00',717.96,20104,20260,55.08,'Medium' from dual union all
select 22878,'022878','1993-068B','Delta 223','Delta SSPS AJ10-118K',65170,'Reentered','1999-09-18 00:00:00','1993-11-26 00:00:00',94.42,451,530,35.15,'Intermediate' from dual union all
select 22879,'022879','1993-068C','Star 48B','Star 48B S/N 10083-18',65170,'Reentered','2004-01-20 00:00:00','1993-11-28 00:00:00',350.33,173,19995,34.98,'Highly Ellip' from dual union all
select 2288,'002288','1966-046BF','deb Atlas 5304','',13453,'Reentered','1966-07-04 00:00:00','1966-06-21 00:00:00',90.03,237,315,28.69,'Intermediate' from dual union all
select 22880,'022880','1993-069A','Gorizont','Gorizont No. 40L',65171,'In Earth orbit','','1993-11-27 00:00:00',1435.96,35767,35800,1.4,'Stationary' from dual union all
select 22881,'022881','1993-069B','8S812','8S812',65171,'Reentered','1993-10-31 00:00:00','1993-10-31 00:00:00',87.08,127,133,51.65,'Intermediate' from dual union all
select 22882,'022882','1993-069C','Perekhodnik','Sredniy perekhodnik',65171,'Reentered','1993-10-29 00:00:00','1993-10-29 00:00:00',87.54,151,155,51.65,'Intermediate' from dual union all
select 22883,'022883','1993-069D','Blok DM-2 No. 72L','11S861 No. 72L',65171,'In Earth orbit','','1994-01-01 00:00:00',1433.2,35713,35747,1.36,'Drift GEO' from dual union all
select 22884,'022884','1993-069E','SOZ','SOZ',65171,'Reentered','1994-05-28 00:00:00','1993-11-30 00:00:00',631.62,233,35782,47.27,'GEO Transfer' from dual union all
select 22885,'022885','1993-069F','SOZ','SOZ',65171,'Reentered','1994-11-08 00:00:00','1993-11-28 00:00:00',633.24,293,35805,47.35,'GEO Transfer' from dual union all
select 22886,'022886','1986-017HY','Orlan-DMA No. 14','',59210,'Reentered','1994-07-01 00:00:00','1993-11-18 00:00:00',92.29,383,390,51.62,'Intermediate' from dual union all
select 22887,'022887','1986-017HZ','EO-14 EVA-5 debris','',59210,'Reentered','1993-11-21 00:00:00','1993-11-01 00:00:00',92.29,385,389,51.62,'Intermediate' from dual union all
select 22888,'022888','1993-070A','Kosmos-2266','Parus',65172,'In Earth orbit','','1993-12-14 00:00:00',104.77,949,1019,82.95,'Intermediate' from dual union all
select 22889,'022889','1993-070B','S3M','S3M',65172,'In Earth orbit','','1993-12-02 00:00:00',104.59,942,1009,82.95,'Intermediate' from dual union all
select 2289,'002289','1966-059A','Saturn S-IVB-203','Saturn S-IVB-203',13466,'Exploded','1966-07-05 00:00:00','1966-07-05 00:00:00',88.47,184,214,31.98,'Intermediate' from dual union all
select 22890,'022890','1983-075G','deb Kosmos-1484','',55832,'Reentered','2000-02-06 00:00:00','1993-11-17 00:00:00',96.41,546,627,97.49,'Sun-Synch' from dual union all
select 22891,'022891','1983-075H','deb Kosmos-1484','',55832,'Reentered','1994-03-12 00:00:00','1993-11-17 00:00:00',93.97,402,536,97.6,'Sun-Synch' from dual union all
select 22892,'022892','1983-075J','deb Kosmos-1484','',55832,'Reentered','1996-03-26 00:00:00','1993-11-17 00:00:00',104.46,539,1400,98.51,'Sun-Synch' from dual union all
select 22893,'022893','1983-075K','deb Kosmos-1484','',55832,'Reentered','1994-07-09 00:00:00','1993-11-17 00:00:00',97.09,541,698,97.35,'Sun-Synch' from dual union all
select 22894,'022894','1983-075L','deb Kosmos-1484','',55832,'Reentered','1994-02-26 00:00:00','1993-11-18 00:00:00',93.81,393,529,98.13,'Sun-Synch' from dual union all
select 22895,'022895','1983-075M','deb Kosmos-1484','',55832,'Reentered','1995-02-05 00:00:00','1993-11-17 00:00:00',95.49,482,603,98.47,'Sun-Synch' from dual union all
select 22896,'022896','1983-075N','deb Kosmos-1484','',55832,'Reentered','1999-01-17 00:00:00','1993-11-18 00:00:00',96.8,524,687,98.34,'Sun-Synch' from dual union all
select 22897,'022897','1983-075P','deb Kosmos-1484','',55832,'Reentered','1999-11-21 00:00:00','1993-11-17 00:00:00',98.47,542,829,98.17,'Sun-Synch' from dual union all
select 22898,'022898','1983-075Q','deb Kosmos-1484','',55832,'Reentered','1993-11-12 00:00:00','1993-11-04 00:00:00',93.5,414,477,97.81,'Sun-Synch' from dual union all
select 22899,'022899','1983-075R','deb Kosmos-1484','',55832,'Reentered','1994-04-01 00:00:00','1993-11-18 00:00:00',98.5,529,845,99,'Sun-Synch' from dual union all
select 229,'000229','62 Beta 4','Altair 3020','Altair X-248-A5 SV-177',8185,'Reentered','2002-08-10 00:00:00','1962-03-12 00:00:00',100.33,714,835,48.27,'Intermediate' from dual union all
select 2290,'002290','1966-060A','Proton-3','N-4 No. 4',13467,'Reentered','1966-09-16 00:00:00','1966-08-11 00:00:00',91.17,174,489,63.45,'Intermediate' from dual union all
select 22900,'022900','1983-075S','deb Kosmos-1484','',55832,'Reentered','1993-12-06 00:00:00','1993-11-18 00:00:00',106.29,423,1687,97.96,'Sun-Synch' from dual union all
select 22901,'022901','1983-075T','deb Kosmos-1484','',55832,'Reentered','1993-11-19 00:00:00','1993-11-16 00:00:00',90.83,304,327,97.54,'Sun-Synch' from dual union all
select 22902,'022902','1983-075U','deb Kosmos-1484','',55832,'Reentered','1999-07-04 00:00:00','1993-11-17 00:00:00',97.22,536,716,98.39,'Sun-Synch' from dual union all
select 22903,'022903','1983-075V','deb Kosmos-1484','',55832,'Reentered','1994-01-19 00:00:00','1993-11-18 00:00:00',105.61,508,1538,97.65,'Sun-Synch' from dual union all
select 22904,'022904','1993-071A','Kosmos-2267','Neman',65173,'Deorbited','1994-12-28 00:00:00','1993-12-05 00:00:00',89.8,242,288,70.38,'Intermediate' from dual union all
select 22905,'022905','1993-071B','11S510','11S510',65173,'Reentered','1993-11-08 00:00:00','1993-11-07 00:00:00',87.6,148,164,70.39,'Intermediate' from dual union all
select 22906,'022906','1993-066B','Ariane H10 V60','Ariane H10 S/N T432',65168,'Reentered','1994-04-17 00:00:00','1993-11-22 00:00:00',637.79,148,36184,6.9,'GEO Transfer' from dual union all
select 22907,'022907','1993-072A','Rimsat G1','Gorizont No. 41L',65174,'In Earth orbit','','1993-12-19 00:00:00',1436.21,35763,35815,1.38,'Stationary' from dual union all
select 22908,'022908','1993-072B','8S812','8S812',65174,'Reentered','1993-11-21 00:00:00','1993-11-21 00:00:00',87.27,136,142,51.65,'Intermediate' from dual union all
select 22909,'022909','1993-072C','Perekhodnik','Sredniy perekhodnik',65174,'Reentered','1993-11-19 00:00:00','1993-11-19 00:00:00',87.85,167,170,51.65,'Intermediate' from dual union all
select 2291,'002291','1966-059B','AS-203 Nosecone','Subcritical Cryo LN2 Exp',13466,'Reentered','1966-07-13 00:00:00','1966-07-05 00:00:00',88.47,184,214,31.98,'Intermediate' from dual union all
select 22910,'022910','1993-072D','Blok DM-2 No. 85L','11S861 No. 85L',65174,'In Earth orbit','','1993-12-21 00:00:00',1395.76,34925,35061,1.43,'Drift GEO' from dual union all
select 22911,'022911','1993-073A','Solidaridad 1','Solidaridad 1',65175,'In Earth orbit','','1993-12-23 00:00:00',1436.06,35755,35817,.07,'Stationary' from dual union all
select 22912,'022912','1993-073B','Meteosat 6','MOP 3',65175,'In Earth orbit','','1993-12-20 00:00:00',1435.32,35769,35773,1.29,'Drift GEO' from dual union all
select 22913,'022913','1993-073C','Ariane H10+ V61','Ariane H10+ S/N T433',65175,'In Earth orbit','','1993-12-21 00:00:00',637.54,194,36124,6.65,'GEO Transfer' from dual union all
select 22914,'022914','1993-073D','Spelda 10 S/N UV17','Spelda 10',65175,'Reentered','1998-07-08 00:00:00','1993-12-26 00:00:00',614.45,207,34920,6.79,'GEO Transfer' from dual union all
select 22915,'022915','1993-074A','USA 97','DSCS III B-10',65176,'In Earth orbit','','1993-12-06 00:00:00',1436.21,35789,35789,0,'Stationary' from dual union all
select 22916,'022916','1993-074B','IABS-4','IABS',65176,'In Earth orbit','','1993-12-30 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 22917,'022917','1993-075A','Endeavour (STS-61)','OV-105',65177,'Landed','1993-12-13 00:00:00','1993-12-04 00:00:00',96.38,579,592,28.47,'Intermediate' from dual union all
select 22918,'022918','1983-075W','deb Kosmos-1484','',55832,'Reentered','1995-07-11 00:00:00','1993-12-02 00:00:00',95.81,537,579,97.53,'Sun-Synch' from dual union all
select 22919,'022919','1988-023D','Kosmos-1934 collision debris','',61272,'In Earth orbit','','1993-12-06 00:00:00',96.43,585,590,28.47,'Intermediate' from dual union all
select 2292,'002292','1966-060D','UR-500 GO part','Fairing',13467,'Reentered','1966-07-11 00:00:00','1966-07-08 00:00:00',91.23,187,482,63.41,'Intermediate' from dual union all
select 22920,'022920','1990-037C','HST SA I +V2 array','SA I array',63279,'Reentered','1998-10-28 00:00:00','1993-12-06 00:00:00',96.4,583,590,28.47,'Intermediate' from dual union all
select 22921,'022921','1993-076A','NATO 4B','NATO 4B',65178,'In Earth orbit','','1994-01-09 00:00:00',1440.92,35117,36645,4.13,'Inclined Drift' from dual union all
select 22922,'022922','1993-076B','Delta 224','Delta SSPS AJ10-118K',65178,'Reentered','2002-11-18 00:00:00','1994-01-10 00:00:00',109.64,314,2107,23.05,'Equatorial' from dual union all
select 22923,'022923','1993-076C','Star 48B','Star 48B S/N 10084-18',65178,'In Earth orbit','','1994-01-07 00:00:00',642.71,747,35837,23.18,'GEO Transfer' from dual union all
select 22924,'022924','1993-074C','Centaur AC-106','Centaur II(3) AC-106',65176,'Reentered','1999-12-05 00:00:00','1993-11-29 00:00:00',625.44,184,35512,26.5,'GEO Transfer' from dual union all
select 22925,'022925','1993-072E','SOZ','SOZ',65174,'Reentered','2001-01-06 00:00:00','1993-12-27 00:00:00',619.08,315,35052,47.3,'GEO Transfer' from dual union all
select 22926,'022926','1993-072F','SOZ','SOZ',65174,'Reentered','1994-07-16 00:00:00','1993-12-20 00:00:00',617.69,257,35038,47.3,'GEO Transfer' from dual union all
select 22927,'022927','1993-077A','Telstar 401','Telstar 401',65179,'In Earth orbit','','1994-01-16 00:00:00',1436.11,35784,35790,.05,'Stationary' from dual union all
select 22928,'022928','1993-077B','Centaur AC-108','Centaur IIA(4N) AC-108',65179,'Reentered','1998-12-17 00:00:00','1994-01-18 00:00:00',609.41,170,34695,23.83,'GEO Transfer' from dual union all
select 22929,'022929','1992-087C','Kosmos-2223 part','Neman AO part',64799,'Reentered','1993-12-19 00:00:00','1993-12-18 00:00:00',88.72,174,249,64.59,'Intermediate' from dual union all
select 2293,'002293','1966-060B','8S811','8S811',13467,'Reentered','1966-08-21 00:00:00','1966-07-13 00:00:00',92.1,184,571,63.47,'Intermediate' from dual union all
select 22930,'022930','1993-078A','DirecTV 1','DBS 1',65180,'In Earth orbit','','1994-01-18 00:00:00',1436.06,35771,35801,.02,'Stationary' from dual union all
select 22931,'022931','1993-078B','Thaicom 1','Thaicom 1',65180,'In Earth orbit','','1994-01-17 00:00:00',1436.09,35763,35810,.07,'Stationary' from dual union all
select 22932,'022932','1993-078C','Ariane H10+ V62','Ariane H10+ S/N T434',65180,'Reentered','1994-01-27 00:00:00','1993-12-27 00:00:00',559.93,136,32115,4.24,'Highly Ellip' from dual union all
select 22933,'022933','1993-078D','Spelda 30 S/N UV23','Mini Spelda',65180,'Reentered','2001-08-04 00:00:00','1994-01-18 00:00:00',628.01,217,35612,3.83,'GEO Transfer' from dual union all
select 22934,'022934','1983-075X','deb Kosmos-1484','',55832,'Reentered','1995-07-11 00:00:00','1993-12-21 00:00:00',95.85,536,583,97.51,'Sun-Synch' from dual union all
select 22935,'022935','1983-075Y','deb Kosmos-1484','',55832,'Reentered','1999-08-04 00:00:00','1993-12-17 00:00:00',97.11,552,688,97.46,'Sun-Synch' from dual union all
select 22936,'022936','1983-075Z','deb Kosmos-1484','',55832,'Reentered','1995-06-02 00:00:00','1993-12-21 00:00:00',95.89,538,586,97.51,'Sun-Synch' from dual union all
select 22937,'022937','1983-075AA','deb Kosmos-1484','',55832,'Reentered','1994-11-29 00:00:00','1993-12-20 00:00:00',95.63,529,570,97.52,'Sun-Synch' from dual union all
select 22938,'022938','1983-075AB','deb Kosmos-1484','',55832,'Reentered','1995-05-09 00:00:00','1993-12-21 00:00:00',95.85,535,585,97.5,'Sun-Synch' from dual union all
select 22939,'022939','1983-075AC','deb Kosmos-1484','',55832,'Reentered','1995-05-30 00:00:00','1993-12-21 00:00:00',95.83,536,582,97.5,'Sun-Synch' from dual union all
select 2294,'002294','1966-060C','UR-500 GO part','Fairing',13467,'Reentered','1966-07-19 00:00:00','1966-07-08 00:00:00',92.01,180,567,63.45,'Intermediate' from dual union all
select 22940,'022940','1983-075AD','deb Kosmos-1484','',55832,'Reentered','1995-08-22 00:00:00','1993-12-20 00:00:00',95.83,537,581,97.51,'Sun-Synch' from dual union all
select 22941,'022941','1983-075AE','deb Kosmos-1484','',55832,'Reentered','1995-04-07 00:00:00','1993-12-21 00:00:00',95.8,533,582,97.49,'Sun-Synch' from dual union all
select 22942,'022942','1983-075AF','deb Kosmos-1484','',55832,'Reentered','1995-11-13 00:00:00','1993-12-28 00:00:00',95.78,532,580,97.5,'Sun-Synch' from dual union all
select 22943,'022943','1983-075AG','deb Kosmos-1484','',55832,'Reentered','1998-04-28 00:00:00','1993-12-21 00:00:00',97.92,545,774,97.36,'Sun-Synch' from dual union all
select 22944,'022944','1983-075AH','deb Kosmos-1484','',55832,'Reentered','1995-06-23 00:00:00','1993-12-21 00:00:00',95.88,536,587,97.51,'Sun-Synch' from dual union all
select 22945,'022945','1983-075AJ','deb Kosmos-1484','',55832,'Reentered','1995-06-04 00:00:00','1993-12-21 00:00:00',95.87,536,586,97.51,'Sun-Synch' from dual union all
select 22946,'022946','1983-075AK','deb Kosmos-1484','',55832,'Reentered','1995-08-09 00:00:00','1993-12-21 00:00:00',95.6,510,586,98.17,'Sun-Synch' from dual union all
select 22947,'022947','1983-075AL','deb Kosmos-1484','',55832,'Reentered','1994-12-02 00:00:00','1993-12-21 00:00:00',95.48,523,561,97.5,'Sun-Synch' from dual union all
select 22948,'022948','1983-075AM','deb Kosmos-1484','',55832,'Reentered','1999-06-24 00:00:00','1993-12-20 00:00:00',97.37,530,736,98.33,'Sun-Synch' from dual union all
select 22949,'022949','1993-079A','Molniya-1T','Molniya-1T',65181,'In Earth orbit','','1994-01-21 00:00:00',704.38,448,39244,62.85,'Molniya' from dual union all
select 2295,'002295','1966-061A','Kosmos-123','DS-P1-Yu No. 5',13468,'Reentered','1966-12-10 00:00:00','1966-08-10 00:00:00',91.97,253,489,48.78,'Intermediate' from dual union all
select 22950,'022950','1993-079B','11S510','11S510',65181,'Reentered','1994-01-10 00:00:00','1993-12-29 00:00:00',90.06,204,351,62.79,'Intermediate' from dual union all
select 22951,'022951','1993-079C','BOZ','BOZ',65181,'Reentered','1994-01-05 00:00:00','1993-12-29 00:00:00',90.03,181,370,62.79,'Intermediate' from dual union all
select 22952,'022952','1993-079D','Blok-ML','Blok-ML',65181,'In Earth orbit','','1994-01-21 00:00:00',699.09,441,38988,62.83,'Molniya' from dual union all
select 22953,'022953','1993-057C','deb Kosmos-2262','',65159,'Reentered','1993-12-30 00:00:00','1993-12-29 00:00:00',88.01,154,198,64.87,'Intermediate' from dual union all
select 22954,'022954','1986-017JA','Garbage bag','',59210,'Reentered','1994-12-26 00:00:00','1994-01-07 00:00:00',92.21,379,386,51.62,'Intermediate' from dual union all
select 22955,'022955','1993-062E','SOZ','SOZ',65164,'Reentered','1994-08-15 00:00:00','1994-01-08 00:00:00',633.97,233,35903,47.07,'GEO Transfer' from dual union all
select 22956,'022956','1993-062F','SOZ','SOZ',65164,'Reentered','1994-05-16 00:00:00','1994-01-08 00:00:00',626.37,169,35575,47.04,'GEO Transfer' from dual union all
select 22957,'022957','1994-001A','Soyuz TM-18','Soyuz 7K-STM No. 67',65397,'Landed','1994-07-09 00:00:00','1994-02-07 00:00:00',92.29,384,390,51.62,'Intermediate' from dual union all
select 22958,'022958','1994-001B','11S510','11S510',65397,'Reentered','1994-01-11 00:00:00','1994-01-10 00:00:00',88.08,173,186,51.62,'Intermediate' from dual;
insert into SATELLITE
select 22959,'022959','1983-075AN','deb Kosmos-1484','',55832,'Reentered','1995-04-04 00:00:00','1994-01-10 00:00:00',95.68,531,572,97.51,'Sun-Synch' from dual union all
select 2296,'002296','1966-061B','S1','S1',13468,'Reentered','1966-10-02 00:00:00','1966-08-10 00:00:00',91.52,246,453,48.81,'Intermediate' from dual union all
select 22960,'022960','1983-075AP','deb Kosmos-1484','',55832,'Reentered','1995-03-22 00:00:00','1994-01-10 00:00:00',95.69,532,572,97.52,'Sun-Synch' from dual union all
select 22961,'022961','1983-075AQ','deb Kosmos-1484','',55832,'Reentered','1995-05-01 00:00:00','1994-01-10 00:00:00',95.7,532,573,97.52,'Sun-Synch' from dual union all
select 22962,'022962','1983-075AR','deb Kosmos-1484','',55832,'Reentered','1995-04-22 00:00:00','1994-01-11 00:00:00',95.75,533,577,97.49,'Sun-Synch' from dual union all
select 22963,'022963','1994-002A','Gals 1','Gals No. 11L',65398,'In Earth orbit','','1994-02-20 00:00:00',1435.96,35776,35792,.19,'Stationary' from dual union all
select 22964,'022964','1994-002B','8S812','8S812',65398,'Reentered','1994-01-22 00:00:00','1994-01-22 00:00:00',86.94,119,127,51.65,'Intermediate' from dual union all
select 22965,'022965','1994-002C','Perekhodnik','Sredniy perekhodnik',65398,'Reentered','1994-01-21 00:00:00','1994-01-21 00:00:00',87.21,134,139,51.63,'Intermediate' from dual union all
select 22966,'022966','1994-002D','Blok DM-2M No. 3L','11S861-01 No. 3L',65398,'In Earth orbit','','1994-02-20 00:00:00',1435.75,35469,36091,.19,'Stationary' from dual union all
select 22967,'022967','1994-002E','SOZ','SOZ',65398,'Reentered','1994-10-31 00:00:00','1994-02-19 00:00:00',637.77,269,36062,47.36,'GEO Transfer' from dual union all
select 22968,'022968','1994-002F','SOZ','SOZ',65398,'Reentered','2000-08-29 00:00:00','1994-02-24 00:00:00',639.1,313,36086,47.33,'GEO Transfer' from dual union all
select 22969,'022969','1994-003A','Meteor-3','Meteor-3 No. 7',65399,'In Earth orbit','','1994-02-26 00:00:00',109.34,1185,1207,82.56,'Intermediate' from dual union all
select 2297,'002297','1966-059C','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-12 00:00:00','1966-07-09 00:00:00',90.6,165,443,33.4,'Intermediate' from dual union all
select 22970,'022970','1994-003B','S5M','S5M',65399,'In Earth orbit','','1994-03-01 00:00:00',109.34,1184,1209,82.56,'Intermediate' from dual union all
select 22971,'022971','1994-003C','Tubsat-B','Tubsat-B',65399,'In Earth orbit','','1994-02-27 00:00:00',109.34,1185,1207,82.57,'Intermediate' from dual union all
select 22972,'022972','1989-089AX','deb COBE','',62366,'Reentered','1996-11-10 00:00:00','1994-08-04 00:00:00',100.92,753,852,98.84,'Sun-Synch' from dual union all
select 22973,'022973','1994-004A','Clementine 1','DSPSE',65400,'Deep Space','1994-02-06 00:00:00','1994-02-01 00:00:00',3125.41,256,128608,66.87,'Very High' from dual union all
select 22974,'022974','1994-004B','Titan 23G-11 Stage 2','LR91AJ5 23G-11',65400,'Reentered','1994-02-11 00:00:00','1994-02-01 00:00:00',89.63,233,280,66.99,'Intermediate' from dual union all
select 22975,'022975','1994-005A','Progress M-21','Progress 7K-TGM No. 221',65401,'Deorbited','1994-03-23 00:00:00','1994-02-07 00:00:00',92.29,384,390,51.62,'Intermediate' from dual union all
select 22976,'022976','1994-005B','11S510','11S510',65401,'Reentered','1994-01-29 00:00:00','1994-01-29 00:00:00',86.95,120,127,51.61,'Intermediate' from dual union all
select 22977,'022977','1994-006A','Discovery (STS-60)','OV-103',65402,'Landed','1994-02-11 00:00:00','1994-02-05 00:00:00',91.57,346,358,56.99,'Intermediate' from dual union all
select 22978,'022978','1994-007A','Ryusei','OREX',65403,'Landed','1994-02-04 00:00:00','1994-02-03 00:00:00',93.64,448,457,30.51,'Intermediate' from dual union all
select 22979,'022979','1994-007B','Myojo','VEP',65403,'In Earth orbit','','1994-03-06 00:00:00',641.97,483,36063,28.61,'GEO Transfer' from dual union all
select 2298,'002298','1966-059D','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-12 00:00:00','1966-07-09 00:00:00',89.59,195,314,32.23,'Intermediate' from dual union all
select 22980,'022980','1994-007C','H-II Stage 2','H-II Stage 2',65403,'In Earth orbit','','1994-08-01 00:00:00',580.74,296,33063,28.59,'Highly Ellip' from dual union all
select 22981,'022981','1994-008A','Raduga-1','Globus No. 13L',65404,'In Earth orbit','','1994-03-08 00:00:00',1435.88,35772,35793,1.45,'Stationary' from dual union all
select 22982,'022982','1994-008B','8S812','8S812',65404,'Reentered','1994-02-05 00:00:00','1994-02-07 00:00:00',87.71,160,162,51.64,'Intermediate' from dual union all
select 22983,'022983','1994-008C','Perekhodnik','Sredniy perekhodnik',65404,'Reentered','1994-02-05 00:00:00','1994-02-06 00:00:00',87.29,135,146,51.59,'Intermediate' from dual union all
select 22984,'022984','1994-008D','Blok DM-2 No. 83L','11S861 No. 83L',65404,'In Earth orbit','','1994-03-20 00:00:00',1469.83,36361,36528,1.45,'Drift GEO' from dual union all
select 22985,'022985','1994-008E','SOZ','SOZ',65404,'Reentered','1994-11-28 00:00:00','1994-03-13 00:00:00',646.35,281,36488,47.31,'GEO Transfer' from dual union all
select 22986,'022986','1994-008F','SOZ','SOZ',65404,'Reentered','2001-02-15 00:00:00','1994-03-11 00:00:00',647.6,325,36509,47.37,'GEO Transfer' from dual union all
select 22987,'022987','1994-004C','ISA','Interstage Adapter Satel',65400,'Reentered','1994-06-08 00:00:00','1994-02-24 00:00:00',3075.56,396,126957,65.81,'Very High' from dual union all
select 22988,'022988','1994-009A','Milstar 1','Milstar DFS-1',65405,'In Earth orbit','','1999-03-09 00:00:00',1436.09,35777,35796,.01,'Stationary' from dual union all
select 22989,'022989','1994-009B','Centaur TC-12','Centaur TC-12',65405,'In Earth orbit','','1994-02-08 00:00:00',1434.81,35733,35790,12,'Inclined Drift' from dual union all
select 2299,'002299','1966-059E','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-11 00:00:00','1966-07-08 00:00:00',89.8,181,349,32.17,'Intermediate' from dual union all
select 22990,'022990','1994-006B','ODERACS A','ODERACS',65402,'Reentered','1994-10-02 00:00:00','1994-03-05 00:00:00',91.3,327,349,56.99,'Intermediate' from dual union all
select 22991,'022991','1994-006C','ODERACS B','ODERACS',65402,'Reentered','1994-10-04 00:00:00','1994-03-06 00:00:00',91.32,328,351,56.99,'Intermediate' from dual union all
select 22992,'022992','1994-006D','ODERACS C','ODERACS',65402,'Reentered','1995-01-01 00:00:00','1994-02-17 00:00:00',91.35,329,352,56.99,'Intermediate' from dual union all
select 22993,'022993','1994-006E','ODERACS D','ODERACS',65402,'Reentered','1995-01-01 00:00:00','1994-02-17 00:00:00',91.35,329,352,56.99,'Intermediate' from dual union all
select 22994,'022994','1994-006F','ODERACS E','ODERACS',65402,'Reentered','1995-03-03 00:00:00','1994-03-07 00:00:00',91.41,334,354,56.99,'Intermediate' from dual union all
select 22995,'022995','1994-006G','ODERACS F','ODERACS',65402,'Reentered','1995-02-24 00:00:00','1994-03-07 00:00:00',91.42,334,355,56.99,'Intermediate' from dual union all
select 22996,'022996','1994-010A','Shi Jian 4','shijian sihao',65406,'In Earth orbit','','1994-03-10 00:00:00',637.4,179,36133,28.53,'GEO Transfer' from dual union all
select 22997,'022997','1994-010C','CZ-3A Stage 3','CZ-3A Stage 3',65406,'Reentered','2007-01-01 00:00:00','1994-03-10 00:00:00',636.25,193,36060,27.68,'GEO Transfer' from dual union all
select 22998,'022998','1994-006H','BREMSAT','BREMSAT',65402,'Reentered','1995-02-12 00:00:00','1994-03-07 00:00:00',91.48,339,355,56.99,'Intermediate' from dual union all
select 22999,'022999','1994-011A','Kosmos-2268','Strela-3',65407,'In Earth orbit','','1994-03-17 00:00:00',114.18,1412,1425,82.57,'Intermediate' from dual union all
select 23,'000023','59 Iota 2','Cluster 13 Assembly 4','RTV Motor',5531,'Reentered','1989-07-16 00:00:00','1959-11-17 00:00:00',101.31,554,1088,50.3,'Intermediate' from dual union all
select 230,'000230','61 Omicron 61','deb r/b Ablestar','',7273,'In Earth orbit','','1962-02-07 00:00:00',103.57,854,1000,66.87,'Intermediate' from dual union all
select 2300,'002300','1966-059F','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-12 00:00:00','1966-07-09 00:00:00',89.62,211,301,32.05,'Intermediate' from dual union all
select 23000,'023000','1994-011B','Kosmos-2269','Strela-3',65407,'In Earth orbit','','1994-03-17 00:00:00',114.11,1410,1419,82.56,'Intermediate' from dual union all
select 23001,'023001','1994-011C','Kosmos-2270','Strela-3',65407,'In Earth orbit','','1994-03-15 00:00:00',114.03,1406,1416,82.57,'Intermediate' from dual union all
select 23002,'023002','1994-011D','Kosmos-2271','Strela-3',65407,'In Earth orbit','','1994-04-05 00:00:00',113.97,1403,1415,82.56,'Intermediate' from dual union all
select 23003,'023003','1994-011E','Kosmos-2272','Strela-3',65407,'In Earth orbit','','1994-03-17 00:00:00',113.9,1395,1416,82.57,'Intermediate' from dual union all
select 23004,'023004','1994-011F','Kosmos-2273','Strela-3',65407,'In Earth orbit','','1994-03-16 00:00:00',114.03,1407,1416,82.58,'Intermediate' from dual union all
select 23005,'023005','1994-011G','S5M','S5M',65407,'In Earth orbit','','1994-03-17 00:00:00',114.72,1411,1474,82.57,'Intermediate' from dual union all
select 23006,'023006','1985-004E','deb Molniya-3','',58227,'Reentered','1994-12-11 00:00:00','1994-02-20 00:00:00',717.31,415,39917,64.44,'GEO Transfer' from dual union all
select 23007,'023007','1993-036C','deb Kosmos-2251','Strela-2M debris 1',65138,'In Earth orbit','','1994-02-17 00:00:00',100.8,785,808,74.03,'Intermediate' from dual union all
select 23008,'023008','1992-093JE','deb 11S772','',64805,'Reentered','1999-12-17 00:00:00','1994-02-16 00:00:00',101.34,814,831,71.02,'Intermediate' from dual union all
select 23009,'023009','1994-010B','Kua Fu 1','DFH-3 mockup',65406,'Reentered','2002-02-07 00:00:00','1994-03-10 00:00:00',635.69,178,36046,28.51,'GEO Transfer' from dual union all
select 2301,'002301','1966-059G','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-11 00:00:00','1966-07-09 00:00:00',89.49,174,324,32.19,'Intermediate' from dual union all
select 23010,'023010','1994-012A','Raduga','Gran'' No. 40L',65408,'In Earth orbit','','1994-03-20 00:00:00',1436.32,35755,35827,1.45,'Stationary' from dual union all
select 23011,'023011','1994-012B','8S812','8S812',65408,'Reentered','1994-02-21 00:00:00','1994-02-20 00:00:00',87.18,130,140,51.66,'Intermediate' from dual union all
select 23012,'023012','1994-012C','Perekhodnik','Sredniy perekhodnik',65408,'Reentered','1994-02-19 00:00:00','1994-02-18 00:00:00',87.77,159,170,51.64,'Intermediate' from dual union all
select 23013,'023013','1994-012D','Blok DM-2 No. 73L','11S861 No. 73L',65408,'In Earth orbit','','1994-03-31 00:00:00',1467.75,36281,36527,1.43,'Drift GEO' from dual union all
select 23014,'023014','1994-012E','SOZ','SOZ',65408,'Reentered','1994-11-11 00:00:00','1994-03-30 00:00:00',646.13,252,36507,47.28,'GEO Transfer' from dual union all
select 23015,'023015','1994-012F','SOZ','SOZ',65408,'Reentered','1995-08-25 00:00:00','1994-03-24 00:00:00',647.52,322,36508,47.37,'GEO Transfer' from dual union all
select 23016,'023016','1994-013A','Galaxy 1R','Galaxy 1RR',65409,'In Earth orbit','','1994-03-24 00:00:00',1436.01,35774,35796,.05,'Stationary' from dual union all
select 23017,'023017','1994-013B','Delta 225','Delta SSPS AJ10-118K',65409,'Reentered','1994-04-18 00:00:00','1994-02-27 00:00:00',116.76,140,2931,26.54,'Intermediate' from dual union all
select 23018,'023018','1994-013C','Star 48B','Star 48B S/N 10086-18',65409,'In Earth orbit','','1994-03-21 00:00:00',711.53,2900,37146,25.53,'GEO Transfer' from dual union all
select 23019,'023019','1994-014A','Koronas-I','AUOS-SM-KI-IK No. 801',65410,'Reentered','2001-03-04 00:00:00','1994-04-01 00:00:00',94.76,486,528,82.49,'Intermediate' from dual union all
select 2302,'002302','1966-059H','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-10 00:00:00','1966-07-09 00:00:00',89.23,99,374,31.98,'Intermediate' from dual union all
select 23020,'023020','1994-014B','S5M','S5M',65410,'Reentered','2005-02-13 00:00:00','1994-04-01 00:00:00',94.76,486,528,82.49,'Intermediate' from dual union all
select 23021,'023021','1994-014C','deb Koronas-I','',65410,'Reentered','1995-04-08 00:00:00','1994-04-02 00:00:00',93.96,455,482,82.5,'Intermediate' from dual union all
select 23022,'023022','1994-014D','deb Koronas-I','',65410,'Reentered','1996-03-21 00:00:00','1994-04-03 00:00:00',94.49,478,510,82.5,'Intermediate' from dual union all
select 23023,'023023','1994-014E','deb Koronas-I','',65410,'Reentered','1997-02-07 00:00:00','1994-04-03 00:00:00',94.81,475,544,82.5,'Intermediate' from dual union all
select 23024,'023024','1994-014F','deb Koronas-I','',65410,'Reentered','1996-07-06 00:00:00','1994-04-03 00:00:00',94.58,481,516,82.48,'Intermediate' from dual union all
select 23025,'023025','1994-015A','Columbia (STS-62)','OV-102',65411,'Landed','1994-03-18 00:00:00','1994-03-06 00:00:00',90.53,296,305,39.01,'Intermediate' from dual union all
select 23026,'023026','1994-014G','deb Koronas-I','',65410,'Reentered','1996-12-08 00:00:00','1994-04-01 00:00:00',94.71,484,526,82.51,'Intermediate' from dual union all
select 23027,'023027','1994-016A','GPS IIA-15','Navstar GPS SVN 36',65412,'In Earth orbit','','1994-04-15 00:00:00',716.69,19986,20315,54.91,'Medium' from dual union all
select 23028,'023028','1994-016B','Delta 226','Delta SSPS AJ10-118K',65412,'Reentered','1994-05-08 00:00:00','1994-03-17 00:00:00',91.34,340,341,32.35,'Intermediate' from dual union all
select 23029,'023029','1994-016C','Star 48B','Star 48B S/N 10085-18',65412,'Reentered','2001-03-27 00:00:00','1994-04-10 00:00:00',351.17,182,20038,34.89,'Highly Ellip' from dual union all
select 2303,'002303','1966-059J','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-12 00:00:00','1966-07-11 00:00:00',88.14,111,254,31.96,'Intermediate' from dual union all
select 23030,'023030','1994-017A','STEP M0','P90-5',65413,'In Earth orbit','','1994-04-24 00:00:00',95.85,554,566,105.02,'Retrograde' from dual union all
select 23031,'023031','1994-017B','DARPASAT','DARPASAT',65413,'Reentered','2002-05-01 00:00:00','1997-11-22 00:00:00',96.72,564,639,105.02,'Retrograde' from dual union all
select 23032,'023032','1994-017C','Orion 38','Orion 38',65413,'Reentered','2000-01-01 00:00:00','1997-11-21 00:00:00',95.47,524,559,105.01,'Retrograde' from dual union all
select 23033,'023033','1994-018A','Kosmos-2274','Kobal''t No. 545',65414,'Landed','1994-05-21 00:00:00','1994-04-16 00:00:00',89.47,175,321,67.12,'Intermediate' from dual union all
select 23034,'023034','1994-018B','11S510','11S510',65414,'Reentered','1994-03-21 00:00:00','1994-03-20 00:00:00',88.22,141,232,67.12,'Intermediate' from dual union all
select 23035,'023035','1994-019A','Progress M-22','Progress 7K-TGM No. 222',65415,'Deorbited','1994-05-23 00:00:00','1994-04-21 00:00:00',92.36,380,401,51.64,'Intermediate' from dual union all
select 23036,'023036','1994-019B','11S510','11S510',65415,'Reentered','1994-03-24 00:00:00','1994-03-23 00:00:00',88.13,172,192,51.64,'Intermediate' from dual union all
select 23037,'023037','1988-077G','deb Transtage D-3','',61326,'In Earth orbit','','1994-04-01 00:00:00',702.53,400,39200,26.7,'GEO Transfer' from dual union all
select 23038,'023038','1988-077H','deb Transtage D-3','',61326,'In Earth orbit','','1994-04-01 00:00:00',702.53,400,39200,26.7,'GEO Transfer' from dual union all
select 23039,'023039','1988-077J','deb Transtage D-3','',61326,'In Earth orbit','','1994-04-01 00:00:00',702.53,400,39200,26.7,'GEO Transfer' from dual union all
select 2304,'002304','1966-059K','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-11 00:00:00','1966-07-11 00:00:00',88.19,138,232,32.13,'Intermediate' from dual union all
select 23040,'023040','1988-077K','deb Transtage D-3','',61326,'In Earth orbit','','1994-04-01 00:00:00',702.53,400,39200,26.7,'GEO Transfer' from dual union all
select 23041,'023041','1988-077L','deb Transtage D-3','',61326,'In Earth orbit','','1994-04-01 00:00:00',702.53,400,39200,26.7,'GEO Transfer' from dual union all
select 23042,'023042','1994-020A','Endeavour (STS-59)','OV-105',65416,'Landed','1994-04-20 00:00:00','1994-04-11 00:00:00',88.86,211,226,56.99,'Intermediate' from dual union all
select 23043,'023043','1994-021A','Kosmos-2275','Uragan No. 58L',65417,'In Earth orbit','','1994-05-14 00:00:00',675.69,19116,19142,64.81,'Medium' from dual union all
select 23044,'023044','1994-021B','Kosmos-2276','Uragan No. 60L',65417,'In Earth orbit','','1994-05-14 00:00:00',675.71,19059,19200,64.8,'Medium' from dual union all
select 23045,'023045','1994-021C','Kosmos-2277','Uragan No. 61L',65417,'In Earth orbit','','1994-05-14 00:00:00',675.65,19110,19146,64.81,'Medium' from dual union all
select 23046,'023046','1994-021D','8S812','8S812',65417,'Reentered','1994-04-11 00:00:00','1994-04-11 00:00:00',87.78,157,173,64.86,'Intermediate' from dual union all
select 23047,'023047','1994-021E','Perekhodnik','Sredniy perekhodnik',65417,'Reentered','1994-04-11 00:00:00','1994-04-11 00:00:00',86.8,115,117,64.84,'Intermediate' from dual;
insert into SATELLITE
select 23048,'023048','1994-021F','Blok DM2 No. 70L','11S861 No. 70L',65417,'In Earth orbit','','1994-05-13 00:00:00',675.04,19099,19126,64.8,'Medium' from dual union all
select 23049,'023049','1994-021G','SOZ','SOZ',65417,'Reentered','2000-09-04 00:00:00','1994-05-11 00:00:00',340.25,405,19128,64.8,'Highly Ellip' from dual union all
select 2305,'002305','1966-059L','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-11 00:00:00','1966-07-11 00:00:00',88.19,163,207,32.11,'Intermediate' from dual union all
select 23050,'023050','1994-021H','SOZ','SOZ',65417,'Reentered','2000-07-25 00:00:00','1994-05-11 00:00:00',340.25,400,19133,64.88,'Highly Ellip' from dual union all
select 23051,'023051','1994-022A','GOES 8','GOES I',65418,'In Earth orbit','','1994-05-14 00:00:00',1435.88,35634,35930,.43,'Stationary' from dual union all
select 23052,'023052','1994-022B','Centaur AC-73','Centaur I AC-73',65418,'Reentered','1994-09-28 00:00:00','1994-05-14 00:00:00',739.85,148,41290,27.16,'GEO Transfer' from dual union all
select 23053,'023053','1989-089AY','deb COBE','',62366,'Reentered','1996-12-10 00:00:00','1994-04-19 00:00:00',101.57,771,895,99,'Sun-Synch' from dual union all
select 23054,'023054','1989-089AZ','deb COBE','',62366,'Reentered','1995-09-15 00:00:00','1994-04-19 00:00:00',101.14,767,859,98.92,'Sun-Synch' from dual union all
select 23055,'023055','1989-089BA','deb COBE','',62366,'Reentered','1998-01-28 00:00:00','1994-04-18 00:00:00',102.43,861,887,98.98,'Sun-Synch' from dual union all
select 23056,'023056','1989-089BB','deb COBE','',62366,'Reentered','1998-11-21 00:00:00','1994-04-18 00:00:00',102.41,855,891,98.98,'Sun-Synch' from dual union all
select 23057,'023057','1989-089BC','deb COBE','',62366,'Reentered','1998-12-11 00:00:00','1994-04-18 00:00:00',102.49,862,890,98.98,'Sun-Synch' from dual union all
select 23058,'023058','1989-089BD','deb COBE','',62366,'Reentered','1999-01-17 00:00:00','1994-04-18 00:00:00',102.39,836,908,98.97,'Sun-Synch' from dual union all
select 23059,'023059','1989-089BE','deb COBE','',62366,'Reentered','1997-04-14 00:00:00','1994-04-19 00:00:00',102.32,794,943,98.94,'Sun-Synch' from dual union all
select 2306,'002306','1966-059M','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-11 00:00:00','1966-07-10 00:00:00',88.37,97,291,31.95,'Intermediate' from dual union all
select 23060,'023060','1989-089BF','deb COBE','',62366,'Reentered','1999-10-07 00:00:00','1994-04-18 00:00:00',102.18,849,874,99,'Sun-Synch' from dual union all
select 23061,'023061','1989-089BG','deb COBE','',62366,'Reentered','1998-06-02 00:00:00','1994-04-19 00:00:00',102.37,850,892,98.98,'Sun-Synch' from dual union all
select 23062,'023062','1989-089BH','deb COBE','',62366,'Reentered','1998-05-14 00:00:00','1994-04-18 00:00:00',102.35,848,892,98.99,'Sun-Synch' from dual union all
select 23063,'023063','1989-089BJ','deb COBE','',62366,'Reentered','1998-05-29 00:00:00','1994-04-15 00:00:00',102.27,827,906,98.97,'Sun-Synch' from dual union all
select 23064,'023064','1989-089BK','deb COBE','',62366,'Reentered','1997-05-03 00:00:00','1994-04-18 00:00:00',102.08,853,862,98.96,'Sun-Synch' from dual union all
select 23065,'023065','1989-089BL','deb COBE','',62366,'Reentered','1998-09-24 00:00:00','1994-04-19 00:00:00',102.28,848,886,98.98,'Sun-Synch' from dual union all
select 23066,'023066','1989-089BM','deb COBE','',62366,'Reentered','1999-01-19 00:00:00','1994-04-19 00:00:00',102.37,862,880,98.98,'Sun-Synch' from dual union all
select 23067,'023067','1989-089BN','deb COBE','',62366,'Reentered','1999-01-24 00:00:00','1994-04-18 00:00:00',102.42,835,912,99,'Sun-Synch' from dual union all
select 23068,'023068','1989-089BP','deb COBE','',62366,'Reentered','1998-11-18 00:00:00','1994-04-16 00:00:00',102.26,850,882,98.98,'Sun-Synch' from dual union all
select 23069,'023069','1989-089BQ','deb COBE','',62366,'Reentered','1998-06-24 00:00:00','1994-04-18 00:00:00',102.14,843,877,98.97,'Sun-Synch' from dual union all
select 2307,'002307','1966-059N','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-19 00:00:00','1966-07-08 00:00:00',91.92,136,602,32.24,'Intermediate' from dual union all
select 23070,'023070','1989-089BR','deb COBE','',62366,'Reentered','1997-02-09 00:00:00','1994-04-17 00:00:00',102.18,846,877,98.99,'Sun-Synch' from dual union all
select 23071,'023071','1989-089BS','deb COBE','',62366,'Reentered','2001-08-06 00:00:00','1994-04-15 00:00:00',107.56,1065,1163,99.01,'Sun-Synch' from dual union all
select 23072,'023072','1989-089BT','deb COBE','',62366,'Reentered','1995-02-18 00:00:00','1994-04-18 00:00:00',102.2,836,889,98.97,'Sun-Synch' from dual union all
select 23073,'023073','1989-089BU','deb COBE','',62366,'Reentered','1996-05-12 00:00:00','1994-04-19 00:00:00',101.43,798,855,98.98,'Sun-Synch' from dual union all
select 23074,'023074','1989-089BV','deb COBE','',62366,'Reentered','1998-04-07 00:00:00','1994-04-18 00:00:00',102.04,831,879,99,'Sun-Synch' from dual union all
select 23075,'023075','1989-089BW','deb COBE','',62366,'Reentered','1999-02-27 00:00:00','1994-04-18 00:00:00',102.46,867,884,98.97,'Sun-Synch' from dual union all
select 23076,'023076','1989-089BX','deb COBE','',62366,'Reentered','1999-06-30 00:00:00','1994-04-18 00:00:00',102.48,863,889,98.98,'Sun-Synch' from dual union all
select 23077,'023077','1989-089BY','deb COBE','',62366,'Reentered','1998-03-01 00:00:00','1994-04-18 00:00:00',102.31,838,898,98.98,'Sun-Synch' from dual union all
select 23078,'023078','1989-089BZ','deb COBE','',62366,'Reentered','1998-12-20 00:00:00','1994-04-18 00:00:00',102.37,862,880,98.99,'Sun-Synch' from dual union all
select 23079,'023079','1989-089CA','deb COBE','',62366,'Reentered','1998-11-06 00:00:00','1994-04-19 00:00:00',102.39,857,887,98.97,'Sun-Synch' from dual union all
select 2308,'002308','1966-059P','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-14 00:00:00','1966-07-09 00:00:00',89.25,222,253,32.74,'Intermediate' from dual union all
select 23080,'023080','1989-089CB','deb COBE','',62366,'Reentered','1997-10-11 00:00:00','1994-04-19 00:00:00',102.39,865,879,98.98,'Sun-Synch' from dual union all
select 23081,'023081','1989-089CC','deb COBE','',62366,'Reentered','1998-05-10 00:00:00','1994-04-18 00:00:00',102.37,850,892,98.98,'Sun-Synch' from dual union all
select 23082,'023082','1989-089CD','deb COBE','',62366,'Reentered','1998-04-06 00:00:00','1994-04-19 00:00:00',102.41,838,908,98.98,'Sun-Synch' from dual union all
select 23083,'023083','1989-089CE','deb COBE','',62366,'Reentered','1999-01-18 00:00:00','1994-04-17 00:00:00',102.41,856,890,98.98,'Sun-Synch' from dual union all
select 23084,'023084','1989-089CF','deb COBE','',62366,'Reentered','1998-11-06 00:00:00','1994-04-19 00:00:00',102.44,853,896,98.98,'Sun-Synch' from dual union all
select 23085,'023085','1989-089CG','deb COBE','',62366,'Reentered','1998-06-26 00:00:00','1994-04-16 00:00:00',102.26,846,885,98.99,'Sun-Synch' from dual union all
select 23086,'023086','1989-089CH','deb COBE','',62366,'Reentered','1995-07-26 00:00:00','1994-04-15 00:00:00',101.32,757,886,98.97,'Sun-Synch' from dual union all
select 23087,'023087','1994-023A','Kosmos-2278','Tselina-2',65419,'In Earth orbit','','1994-05-23 00:00:00',101.96,848,855,71,'Intermediate' from dual union all
select 23088,'023088','1994-023B','11S772','11S772',65419,'In Earth orbit','','1994-05-23 00:00:00',101.84,842,849,71,'Intermediate' from dual union all
select 23089,'023089','1994-023C','Sep Motor Cover','Sep Motor Cover',65419,'In Earth orbit','','1994-05-23 00:00:00',104.96,845,1140,71.03,'Intermediate' from dual union all
select 2309,'002309','1966-059Q','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-15 00:00:00','1966-07-09 00:00:00',89.44,191,303,32.06,'Intermediate' from dual union all
select 23090,'023090','1994-023D','Sep Motor Cover','Sep Motor Cover',65419,'In Earth orbit','','1994-05-23 00:00:00',104.86,845,1131,71.03,'Intermediate' from dual union all
select 23091,'023091','1994-023E','Sep Motor Cover','Sep Motor Cover',65419,'In Earth orbit','','1994-05-24 00:00:00',105,845,1143,70.98,'Intermediate' from dual union all
select 23092,'023092','1994-024A','Kosmos-2279','Parus',65420,'In Earth orbit','','1994-05-26 00:00:00',104.72,956,1006,82.95,'Intermediate' from dual union all
select 23093,'023093','1994-024B','S3M','S3M',65420,'In Earth orbit','','1994-05-27 00:00:00',104.6,957,994,82.94,'Intermediate' from dual union all
select 23094,'023094','1994-023F','Sep Motor Cover','Sep Motor Cover',65419,'In Earth orbit','','1994-05-23 00:00:00',104.98,844,1142,70.99,'Intermediate' from dual union all
select 23095,'023095','1994-025A','Kosmos-2280','Neman',65421,'Deorbited','1995-03-10 00:00:00','1994-05-28 00:00:00',90.04,238,315,70.38,'Intermediate' from dual union all
select 23096,'023096','1994-025B','11S510','11S510',65421,'Reentered','1994-05-02 00:00:00','1994-05-02 00:00:00',87.75,151,176,70.39,'Intermediate' from dual union all
select 23097,'023097','1994-026A','USA 103','TRUMPET 1',65422,'In Earth orbit','','1999-08-19 00:00:00',717.8,1628,38728,63.06,'Molniya' from dual union all
select 23098,'023098','1994-026B','Centaur TC-10','Centaur TC-10',65422,'Reentered','2000-05-18 00:00:00','1994-05-03 00:00:00',717.74,791,39562,64.4,'GEO Transfer' from dual union all
select 23099,'023099','1994-027A','SROSS-C2','SROSS-C2',65423,'Reentered','2001-07-12 00:00:00','1994-06-04 00:00:00',98.26,431,920,46.05,'Intermediate' from dual union all
select 231,'000231','61 Omicron 62','deb r/b Ablestar','',7273,'Reentered','1965-02-13 00:00:00','1962-02-07 00:00:00',97.58,376,910,66.83,'Intermediate' from dual union all
select 2310,'002310','1966-059R','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-19 00:00:00','1966-07-09 00:00:00',91.89,180,555,31.88,'Intermediate' from dual union all
select 23100,'023100','1994-027B','ASLV-D4 AS-4','AS-4',65423,'Reentered','2003-10-06 00:00:00','1994-06-03 00:00:00',98.3,433,921,46.05,'Intermediate' from dual union all
select 23101,'023101','1994-028A','MSTI-2','MSTI-2',65424,'Reentered','1998-11-28 00:00:00','1994-06-08 00:00:00',93.11,418,436,97.11,'Sun-Synch' from dual union all
select 23102,'023102','1994-028B','Star 20 E-51','Star 20 E-51',65424,'Reentered','1995-03-20 00:00:00','1994-06-12 00:00:00',92.65,354,455,96.84,'Sun-Synch' from dual union all
select 23103,'023103','1994-017D','deb P90-5','',65413,'In Earth orbit','','1997-11-17 00:00:00',95.97,542,589,105.03,'Retrograde' from dual union all
select 23104,'023104','1992-094C','deb Kosmos-2228','',64806,'Reentered','2015-06-20 00:00:00','1994-05-15 00:00:00',97.7,629,669,82.53,'Intermediate' from dual union all
select 23105,'023105','1994-029A','STEP M2','STEP 2',65425,'In Earth orbit','','1994-06-21 00:00:00',99.01,604,819,81.95,'Intermediate' from dual union all
select 23106,'023106','1994-029B','HAPS','HAPS',65425,'In Earth orbit','','1994-06-19 00:00:00',98.79,582,820,81.96,'Intermediate' from dual union all
select 23107,'023107','1993-061J','deb Ariane V59','',65163,'Reentered','2000-10-19 00:00:00','1994-06-06 00:00:00',100.74,784,803,98.57,'Sun-Synch' from dual union all
select 23108,'023108','1994-030A','Rimsat G2','Gorizont No. 42L',65426,'In Earth orbit','','1994-06-19 00:00:00',1436.14,35783,35792,1.26,'Stationary' from dual union all
select 23109,'023109','1994-030B','8S812','8S812',65426,'Reentered','1994-05-23 00:00:00','1994-05-22 00:00:00',87.79,161,170,51.65,'Intermediate' from dual union all
select 2311,'002311','1966-059S','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-16 00:00:00','1966-07-09 00:00:00',90.02,175,376,32.84,'Intermediate' from dual union all
select 23110,'023110','1994-030C','Perekhodnik','Sredniy perekhodnik',65426,'Reentered','1994-05-20 00:00:00','1994-05-20 00:00:00',87.59,153,158,51.65,'Intermediate' from dual union all
select 23111,'023111','1994-030D','Blok DM2 No. 71L','11S861 No. 71L',65426,'In Earth orbit','','1994-06-21 00:00:00',1386.72,34624,35005,1.31,'Drift GEO' from dual union all
select 23112,'023112','1994-018C','Kosmos-2274 part','Kobal''t AO part',65414,'Reentered','1994-05-23 00:00:00','1994-05-22 00:00:00',87.88,101,239,67.08,'Intermediate' from dual union all
select 23113,'023113','1994-018D','Kosmos-2274 part','Kobal''t AO part',65414,'Reentered','1994-05-23 00:00:00','1994-05-22 00:00:00',88.65,159,257,67.16,'Intermediate' from dual union all
select 23114,'023114','1994-031A','Progress M-23','Progress 7K-TGM No. 223',65427,'Deorbited','1994-07-02 00:00:00','1994-05-30 00:00:00',92.51,397,399,51.65,'Intermediate' from dual union all
select 23115,'023115','1994-031B','11S510','11S510',65427,'Reentered','1994-05-24 00:00:00','1994-05-23 00:00:00',87.94,165,179,51.61,'Intermediate' from dual union all
select 23116,'023116','1994-030E','SOZ','SOZ',65426,'Reentered','1994-12-31 00:00:00','1994-06-22 00:00:00',616.7,246,34998,47.31,'GEO Transfer' from dual union all
select 23117,'023117','1994-030F','SOZ','SOZ',65426,'Reentered','2001-09-29 00:00:00','1994-06-21 00:00:00',617.96,306,35003,47.35,'GEO Transfer' from dual union all
select 23118,'023118','1993-073E','Mage 1','Mage 1',65175,'In Earth orbit','','1994-03-09 00:00:00',1440.28,35590,36147,1.16,'Drift GEO' from dual union all
select 23119,'023119','1994-032A','Kosmos-2281','Oblik No. 832 (Priroda)',65428,'Landed','1994-06-29 00:00:00','1994-06-14 00:00:00',89.79,236,293,82.58,'Intermediate' from dual union all
select 2312,'002312','1966-059T','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-15 00:00:00','1966-07-09 00:00:00',89.75,220,305,32.5,'Intermediate' from dual union all
select 23120,'023120','1994-032B','11S510','11S510',65428,'Reentered','1994-06-10 00:00:00','1994-06-09 00:00:00',88.11,165,197,82.58,'Intermediate' from dual union all
select 23121,'023121','1994-032C','Sensor cover','Sensor cover',65428,'Reentered','1994-06-18 00:00:00','1994-06-12 00:00:00',89.62,234,278,82.6,'Intermediate' from dual union all
select 23122,'023122','1994-033A','Foton','Foton 34KS No. 9',65429,'Landed','1994-07-02 00:00:00','1994-06-21 00:00:00',90.29,220,358,62.81,'Intermediate' from dual union all
select 23123,'023123','1994-033B','11S510','11S510',65429,'Reentered','1994-07-10 00:00:00','1994-06-21 00:00:00',89.99,221,327,62.8,'Intermediate' from dual union all
select 23124,'023124','1994-034A','INTELSAT 702','INTELSAT 702',65430,'In Earth orbit','','1994-07-18 00:00:00',1436.11,35738,35836,.04,'Stationary' from dual union all
select 23125,'023125','1994-034B','STRV 1A','STRV 1A',65430,'In Earth orbit','','1994-07-17 00:00:00',633.51,272,35839,7,'GEO Transfer' from dual union all
select 23126,'023126','1994-034C','STRV 1B','STRV 1B',65430,'In Earth orbit','','1994-07-23 00:00:00',634.6,272,35896,7,'GEO Transfer' from dual union all
select 23127,'023127','1994-034D','Ariane H10+ V64','Ariane H10+ S/N T436',65430,'In Earth orbit','','1994-07-17 00:00:00',653.31,455,36670,7.02,'GEO Transfer' from dual union all
select 23128,'023128','1994-017E','deb P90-5','',65413,'In Earth orbit','','1994-03-31 00:00:00',95.65,550,550,105,'Retrograde' from dual union all
select 23129,'023129','1994-017F','deb P90-5','',65413,'In Earth orbit','','1994-03-31 00:00:00',95.65,550,550,105,'Retrograde' from dual union all
select 2313,'002313','1966-059U','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-15 00:00:00','1966-07-09 00:00:00',89.87,174,362,32.07,'Intermediate' from dual union all
select 23130,'023130','1988-077M','deb Transtage D-3','',61326,'In Earth orbit','','1994-07-01 00:00:00',702.53,400,39200,26.7,'GEO Transfer' from dual union all
select 23131,'023131','1989-061L','deb USA-40','',62338,'In Earth orbit','','1998-07-29 00:00:00',181.73,535,7965,57.06,'Medium' from dual union all
select 23132,'023132','1994-035A','UHF F/O F3','UFO F3',65431,'In Earth orbit','','1994-07-25 00:00:00',1431.98,35059,36352,5.11,'Inclined Drift' from dual union all
select 23133,'023133','1994-035B','Centaur AC-76','Centaur I AC-76',65431,'Reentered','2001-04-05 00:00:00','1994-07-24 00:00:00',281.96,210,15520,26.95,'Highly Ellip' from dual union all
select 23134,'023134','1994-032D','PTDU cover','PTDU cover',65428,'Reentered','1994-07-25 00:00:00','1994-06-29 00:00:00',91.92,267,470,82.59,'Intermediate' from dual union all
select 23135,'023135','1994-032E','PTDU cover','PTDU cover',65428,'Reentered','1994-07-19 00:00:00','1994-06-29 00:00:00',92.35,271,509,82.58,'Intermediate' from dual union all
select 23136,'023136','1994-032F','KDU','KDU',65428,'Reentered','1994-07-06 00:00:00','1994-06-29 00:00:00',89.97,232,314,82.58,'Intermediate' from dual union all
select 23137,'023137','1994-032G','Blenda','Sunshade',65428,'Reentered','1994-07-11 00:00:00','1994-06-30 00:00:00',89.8,231,299,82.59,'Intermediate' from dual;
insert into SATELLITE
select 23138,'023138','1994-032H','KDU part','KDU part',65428,'Reentered','1994-08-30 00:00:00','1994-07-07 00:00:00',89.9,231,308,82.58,'Intermediate' from dual union all
select 23139,'023139','1994-036A','Soyuz TM-19','Soyuz 7K-STM No. 68',65432,'Landed','1994-11-04 00:00:00','1994-07-31 00:00:00',92.48,396,397,51.65,'Intermediate' from dual union all
select 2314,'002314','1966-059V','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-16 00:00:00','1966-07-10 00:00:00',89.76,161,365,31.82,'Intermediate' from dual union all
select 23140,'023140','1994-036B','11S510','11S510',65432,'Reentered','1994-07-03 00:00:00','1994-07-02 00:00:00',88.11,178,184,51.62,'Intermediate' from dual union all
select 23141,'023141','1994-033C','PTDU cover','PTDU cover',65429,'Reentered','1994-07-16 00:00:00','1994-07-03 00:00:00',92.09,307,447,62.81,'Intermediate' from dual union all
select 23142,'023142','1994-033D','PTDU cover','PTDU cover',65429,'Reentered','1994-08-06 00:00:00','1994-07-03 00:00:00',92.03,308,441,62.81,'Intermediate' from dual union all
select 23143,'023143','1994-033E','KB electropitaniya','KBE',65429,'Reentered','1994-09-01 00:00:00','1994-07-14 00:00:00',89.97,216,330,62.81,'Intermediate' from dual union all
select 23144,'023144','1994-033F','KBE part','KBE part',65429,'Reentered','1994-07-15 00:00:00','1994-07-03 00:00:00',89.95,213,331,62.81,'Intermediate' from dual union all
select 23145,'023145','1994-037A','Fanhui Shi Weixing (16)','Jianbing-1B No. 2',65433,'Reentered','1994-09-13 00:00:00','1994-08-02 00:00:00',90.08,207,350,62.95,'Intermediate' from dual union all
select 23146,'023146','1994-037B','CZ-2D Stage 2','CZ-2D Stage 2',65433,'Reentered','1994-07-13 00:00:00','1994-07-05 00:00:00',89.42,170,322,62.95,'Intermediate' from dual union all
select 23147,'023147','1994-037C','CZ-2D sep motor cover','',65433,'Reentered','1994-07-04 00:00:00','1994-07-03 00:00:00',89.4,208,282,62.97,'Intermediate' from dual union all
select 23148,'023148','1983-075AS','deb Kosmos-1484','',55832,'Reentered','1994-10-18 00:00:00','1994-07-06 00:00:00',93.64,440,466,97.46,'Sun-Synch' from dual union all
select 23149,'023149','1983-075AT','deb Kosmos-1484','',55832,'Reentered','1994-11-11 00:00:00','1994-06-29 00:00:00',94.23,475,488,97.58,'Sun-Synch' from dual union all
select 2315,'002315','1966-059W','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-16 00:00:00','1966-07-08 00:00:00',89.96,194,351,32.02,'Intermediate' from dual union all
select 23150,'023150','1983-075AU','deb Kosmos-1484','',55832,'Reentered','1995-08-27 00:00:00','1994-07-06 00:00:00',94.29,473,495,97.5,'Sun-Synch' from dual union all
select 23151,'023151','1983-075AV','deb Kosmos-1484','',55832,'Reentered','1994-12-21 00:00:00','1994-07-06 00:00:00',94.95,498,535,97.52,'Sun-Synch' from dual union all
select 23152,'023152','1983-075AW','deb Kosmos-1484','',55832,'Reentered','1995-07-14 00:00:00','1994-07-06 00:00:00',94.96,497,537,97.49,'Sun-Synch' from dual union all
select 23153,'023153','1983-075AX','deb Kosmos-1484','',55832,'Reentered','1996-07-07 00:00:00','1994-07-05 00:00:00',95,501,537,97.49,'Sun-Synch' from dual union all
select 23154,'023154','1983-075AY','deb Kosmos-1484','',55832,'Reentered','1995-02-27 00:00:00','1994-07-05 00:00:00',94.74,490,522,97.48,'Sun-Synch' from dual union all
select 23155,'023155','1983-075AZ','deb Kosmos-1484','',55832,'Reentered','1995-04-17 00:00:00','1994-07-05 00:00:00',94.97,497,537,97.48,'Sun-Synch' from dual union all
select 23156,'023156','1983-075BA','deb Kosmos-1484','',55832,'Reentered','1995-07-04 00:00:00','1994-07-06 00:00:00',94.92,474,556,97.52,'Sun-Synch' from dual union all
select 23157,'023157','1983-075BB','deb Kosmos-1484','',55832,'Reentered','1995-06-06 00:00:00','1994-07-05 00:00:00',95.05,503,539,97.48,'Sun-Synch' from dual union all
select 23158,'023158','1989-061M','deb USA-40','',62338,'In Earth orbit','','1998-12-23 00:00:00',169.27,364,7152,56.87,'Medium' from dual union all
select 23159,'023159','1989-061N','deb USA-40','',62338,'In Earth orbit','','1998-09-06 00:00:00',181.08,512,7936,56.96,'Medium' from dual union all
select 2316,'002316','1966-059X','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-16 00:00:00','1966-07-08 00:00:00',89.93,201,341,32.28,'Intermediate' from dual union all
select 23160,'023160','1989-061P','deb USA-40','',62338,'In Earth orbit','','1998-09-07 00:00:00',182.28,516,8027,57,'Medium' from dual union all
select 23161,'023161','1989-061Q','deb USA-40','',62338,'In Earth orbit','','1998-07-06 00:00:00',180.9,521,7914,56.63,'Medium' from dual union all
select 23162,'023162','1989-061R','deb USA-40','',62338,'In Earth orbit','','1998-10-29 00:00:00',182.34,535,8012,56.77,'Medium' from dual union all
select 23163,'023163','1989-061S','deb USA-40','',62338,'In Earth orbit','','1998-09-10 00:00:00',181.54,514,7970,56.97,'Medium' from dual union all
select 23164,'023164','1989-061T','deb USA-40','',62338,'In Earth orbit','','1998-10-12 00:00:00',182.45,534,8022,56.94,'Medium' from dual union all
select 23165,'023165','1989-061U','deb USA-40','',62338,'In Earth orbit','','1998-10-13 00:00:00',182.05,522,8003,56.99,'Medium' from dual union all
select 23166,'023166','1989-061V','deb USA-40','',62338,'In Earth orbit','','1998-07-06 00:00:00',183.39,531,8098,56.62,'Medium' from dual union all
select 23167,'023167','1989-061W','deb USA-40','',62338,'In Earth orbit','','1998-09-27 00:00:00',180.96,508,7932,56.52,'Medium' from dual union all
select 23168,'023168','1994-038A','Kosmos-2282','Oko-1 No. 7123',65434,'In Earth orbit','','1994-08-05 00:00:00',1435.96,35748,35820,2.29,'Stationary' from dual union all
select 23169,'023169','1994-038B','8S812','8S812',65434,'Reentered','1994-07-09 00:00:00','1994-07-09 00:00:00',87.7,160,162,51.64,'Intermediate' from dual union all
select 2317,'002317','1966-059Y','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-17 00:00:00','1966-07-11 00:00:00',90.45,102,492,31.94,'Intermediate' from dual union all
select 23170,'023170','1994-038C','Perekhodnik','Sredniy perekhodnik',65434,'Reentered','1994-07-08 00:00:00','1994-07-07 00:00:00',88.35,188,198,51.64,'Intermediate' from dual union all
select 23171,'023171','1994-038D','Blok DM2 No. 68L','11S861 No. 68L',65434,'In Earth orbit','','1994-08-06 00:00:00',1448.29,35940,36110,2.29,'Drift GEO' from dual union all
select 23172,'023172','1994-038E','SOZ','SOZ',65434,'Reentered','2001-08-05 00:00:00','1994-08-05 00:00:00',638.51,280,36089,47.78,'GEO Transfer' from dual union all
select 23173,'023173','1994-039A','Columbia (STS-65)','OV-102',65435,'Landed','1994-07-23 00:00:00','1994-07-15 00:00:00',90.52,298,302,28.47,'Intermediate' from dual union all
select 23174,'023174','1994-038F','SOZ','SOZ',65434,'Reentered','2003-01-24 00:00:00','1994-08-08 00:00:00',640.01,343,36103,47.85,'GEO Transfer' from dual union all
select 23175,'023175','1994-040A','PAS 2','Panamsat K1',65436,'In Earth orbit','','1994-08-08 00:00:00',1436.16,35633,35943,.03,'Stationary' from dual union all
select 23176,'023176','1994-040B','BS-3N','BS-3N',65436,'In Earth orbit','','1994-08-12 00:00:00',1436.24,35785,35793,.18,'Stationary' from dual union all
select 23177,'023177','1994-040C','Ariane H10+ V65','Ariane H10+ S/N T437',65436,'In Earth orbit','','1994-08-10 00:00:00',653.29,375,36749,7.61,'GEO Transfer' from dual union all
select 23178,'023178','1994-040D','Spelda 30 S/N UV24','Mini Spelda',65436,'Reentered','2003-05-17 00:00:00','1994-08-07 00:00:00',627.51,233,35570,7.07,'GEO Transfer' from dual union all
select 23179,'023179','1994-041A','Nadezhda','Nadezhda 17F118 No. 104',65437,'In Earth orbit','','1994-08-13 00:00:00',104.65,953,1003,82.94,'Intermediate' from dual union all
select 2318,'002318','1966-059Z','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-18 00:00:00','1966-07-11 00:00:00',90.98,206,440,32.05,'Intermediate' from dual union all
select 23180,'023180','1994-041B','S3M','S3M',65437,'In Earth orbit','','1994-08-13 00:00:00',104.54,953,993,82.94,'Intermediate' from dual union all
select 23181,'023181','1994-037D','FSW-2 RV part','',65433,'Reentered','1994-07-18 00:00:00','1994-07-18 00:00:00',91.41,131,557,62.83,'Intermediate' from dual union all
select 23182,'023182','1994-042A','Kosmos-2283','Kobal''t No. 548',65438,'Landed','1994-09-29 00:00:00','1994-08-19 00:00:00',89.42,179,313,67.1,'Intermediate' from dual union all
select 23183,'023183','1994-042B','11S510','11S510',65438,'Reentered','1994-07-26 00:00:00','1994-07-22 00:00:00',89.12,166,296,67.1,'Intermediate' from dual union all
select 23184,'023184','1994-042C','Kosmos-2283 part','Kobal''t part',65438,'Reentered','1994-07-22 00:00:00','1994-07-21 00:00:00',89.1,166,294,67.08,'Intermediate' from dual union all
select 23185,'023185','1994-043A','APSTAR 1','APSTAR 1',65439,'In Earth orbit','','1994-08-20 00:00:00',1436.06,35777,35795,.02,'Stationary' from dual union all
select 23186,'023186','1994-043B','CZ-3 Stage 3','CZ-3 Stage 3',65439,'Reentered','1999-06-08 00:00:00','1994-08-20 00:00:00',753.3,191,41902,26.88,'Highly Ellip' from dual union all
select 23187,'023187','1994-044A','Kosmos-2284','Kometa No. 17',65440,'Landed','1994-09-11 00:00:00','1994-08-05 00:00:00',89.35,211,274,70.38,'Intermediate' from dual union all
select 23188,'023188','1994-044B','11S510','11S510',65440,'Reentered','1994-08-04 00:00:00','1994-07-31 00:00:00',88.83,191,243,70.39,'Intermediate' from dual union all
select 23189,'023189','1994-045A','Kosmos-2285','Obzor No. 1',65441,'In Earth orbit','','1994-09-01 00:00:00',104.96,974,1012,74.03,'Intermediate' from dual union all
select 2319,'002319','1966-059AA','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-20 00:00:00','1966-07-15 00:00:00',89.88,202,335,31.86,'Intermediate' from dual union all
select 23190,'023190','1994-045B','S3M','S3M',65441,'In Earth orbit','','1994-09-09 00:00:00',104.84,971,1003,74.02,'Intermediate' from dual union all
select 23191,'023191','1994-046A','APEX','APEX',65442,'In Earth orbit','','1994-09-02 00:00:00',114.93,362,2543,69.97,'Intermediate' from dual union all
select 23192,'023192','1994-047A','DirecTV 2','DBS 2',65443,'In Earth orbit','','1994-09-02 00:00:00',1436.06,35776,35796,.04,'Stationary' from dual union all
select 23193,'023193','1994-047B','Centaur AC-107','Centaur IIA(4N) AC-107',65443,'Reentered','1998-04-22 00:00:00','1994-09-06 00:00:00',696.66,220,39088,26.95,'GEO Transfer' from dual union all
select 23194,'023194','1994-048A','Kosmos-2286','Oko',65444,'In Earth orbit','','1994-09-04 00:00:00',717.56,589,39755,62.92,'Molniya' from dual union all
select 23195,'023195','1994-048B','11S510','11S510',65444,'Reentered','1994-10-08 00:00:00','1994-09-04 00:00:00',91.13,214,446,62.83,'Intermediate' from dual union all
select 23196,'023196','1994-048C','BOZ','BOZ',65444,'Reentered','1994-09-05 00:00:00','1994-08-12 00:00:00',91.72,183,535,62.83,'Intermediate' from dual union all
select 23197,'023197','1994-048D','Blok-2BL','Blok-2BL No. 71612-695',65444,'In Earth orbit','','1994-09-05 00:00:00',706.42,587,39206,62.9,'Molniya' from dual union all
select 23198,'023198','1994-046B','Orion 38','',65442,'In Earth orbit','','1994-09-08 00:00:00',114.93,363,2542,69.97,'Intermediate' from dual union all
select 23199,'023199','1994-049A','Brasilsat B1','Brasilsat B1',65445,'In Earth orbit','','1994-09-10 00:00:00',1438.44,35789,35876,.85,'Drift GEO' from dual union all
select 232,'000232','61 Omicron 63','deb r/b Ablestar','',7273,'In Earth orbit','','1962-02-07 00:00:00',103.78,884,991,66.83,'Intermediate' from dual union all
select 2320,'002320','1966-059AB','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-19 00:00:00','1966-07-11 00:00:00',91.7,123,593,31.87,'Intermediate' from dual union all
select 23200,'023200','1994-049B','Turksat 1B','Turksat 1B',65445,'In Earth orbit','','1994-09-13 00:00:00',1435.96,35778,35790,.04,'Stationary' from dual union all
select 23201,'023201','1994-049C','Ariane H10+ V66','Ariane H10+ S/N T438',65445,'In Earth orbit','','1994-09-10 00:00:00',658.17,364,37008,7,'GEO Transfer' from dual union all
select 23202,'023202','1994-049D','Spelda 10 S/N UV20','Spelda 10',65445,'Reentered','1998-10-10 00:00:00','1994-09-10 00:00:00',624.1,232,35395,7.05,'GEO Transfer' from dual union all
select 23203,'023203','1994-050A','Kosmos-2287','Uragan No. 67L',65446,'In Earth orbit','','1994-09-10 00:00:00',675.71,19112,19147,64.84,'Medium' from dual union all
select 23204,'023204','1994-050B','Kosmos-2288','Uragan No. 70L',65446,'In Earth orbit','','1994-09-10 00:00:00',675.71,19091,19168,64.83,'Medium' from dual union all
select 23205,'023205','1994-050C','Kosmos-2289','Uragan No. 75L',65446,'In Earth orbit','','1994-09-10 00:00:00',675.71,19121,19138,64.83,'Medium' from dual union all
select 23206,'023206','1994-050D','8S812','8S812',65446,'Reentered','1994-08-12 00:00:00','1994-08-11 00:00:00',87.65,135,182,64.8,'Intermediate' from dual union all
select 23207,'023207','1994-050E','Perekhodnik','Sredniy perekhodnik',65446,'Reentered','1994-08-11 00:00:00','1994-08-11 00:00:00',87.66,145,173,64.85,'Intermediate' from dual union all
select 23208,'023208','1994-050F','Blok DM2 No. 74L','11S861 No. 74L',65446,'In Earth orbit','','1994-09-10 00:00:00',674.68,19074,19133,64.84,'Medium' from dual union all
select 23209,'023209','1994-050G','SOZ','SOZ',65446,'Reentered','2000-07-10 00:00:00','1994-09-10 00:00:00',340.26,415,19119,64.9,'Highly Ellip' from dual union all
select 2321,'002321','1966-059AC','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-18 00:00:00','1966-07-11 00:00:00',91.29,179,497,31.9,'Intermediate' from dual union all
select 23210,'023210','1994-050H','SOZ','SOZ',65446,'Reentered','2000-09-26 00:00:00','1994-09-11 00:00:00',340.26,416,19118,64.82,'Highly Ellip' from dual union all
select 23211,'023211','1994-051A','Molniya-3','Molniya-3 No. 60',65447,'Reentered','2009-02-10 00:00:00','1994-09-23 00:00:00',717.9,608,39753,62.83,'Molniya' from dual union all
select 23212,'023212','1994-051B','11S510','11S510',65447,'Reentered','1994-10-23 00:00:00','1994-09-22 00:00:00',91.24,210,461,62.83,'Intermediate' from dual union all
select 23213,'023213','1994-051C','BOZ','BOZ',65447,'Reentered','1994-09-26 00:00:00','1994-08-30 00:00:00',92.17,186,576,62.81,'Intermediate' from dual union all
select 23214,'023214','1994-051D','Blok-ML','Blok-ML',65447,'Reentered','2008-02-19 00:00:00','1994-09-25 00:00:00',698.51,605,38795,62.87,'Molniya' from dual union all
select 23215,'023215','1994-052A','Progress M-24','Progress 7K-TGM No. 224',65448,'Deorbited','1994-10-04 00:00:00','1994-09-01 00:00:00',92.46,394,397,51.65,'Intermediate' from dual union all
select 23216,'023216','1994-052B','11S510','11S510',65448,'Reentered','1994-08-27 00:00:00','1994-08-26 00:00:00',88.31,180,202,51.62,'Intermediate' from dual union all
select 23217,'023217','1994-051E','Ullage rocket','Ullage rocket',65447,'Reentered','1994-08-31 00:00:00','1994-08-25 00:00:00',92.33,207,571,62.76,'Intermediate' from dual union all
select 23218,'023218','1994-053A','Kosmos-2290','Orlets-2',65449,'Deorbited','1995-04-04 00:00:00','1994-09-25 00:00:00',89.76,203,322,64.81,'Intermediate' from dual union all
select 23219,'023219','1994-053B','11S772','11S772',65449,'Reentered','1994-09-14 00:00:00','1994-09-02 00:00:00',89.18,205,263,64.85,'Intermediate' from dual union all
select 2322,'002322','1966-062A','OPS 1850','GAMBIT SV 980',13469,'Deorbited','1966-07-20 00:00:00','1966-07-15 00:00:00',88.54,140,264,95.54,'Sun-Synch' from dual union all
select 23220,'023220','1994-053C','Sep Motor Cover','Sep Motor Cover',65449,'Reentered','1994-09-18 00:00:00','1994-09-02 00:00:00',91.54,202,499,64.75,'Intermediate' from dual union all
select 23221,'023221','1994-053D','Sep Motor Cover','Sep Motor Cover',65449,'Reentered','1994-09-17 00:00:00','1994-09-02 00:00:00',91.38,204,481,64.87,'Intermediate' from dual union all
select 23222,'023222','1994-053E','Sep Motor Cover','Sep Motor Cover',65449,'Reentered','1994-09-24 00:00:00','1994-09-08 00:00:00',91.26,204,469,64.88,'Intermediate' from dual union all
select 23223,'023223','1994-054A','USA 105','MERCURY 14',65450,'In Earth orbit','','2008-04-07 00:00:00',1436.09,35592,35981,4.6,'Inclined GEO' from dual union all
select 23224,'023224','1994-053F','deb Kosmos-2290','',65449,'Reentered','1994-08-31 00:00:00','1994-08-28 00:00:00',88.9,187,253,64.8,'Intermediate' from dual union all
select 23225,'023225','1994-053G','deb Kosmos-2290','',65449,'Reentered','1994-09-24 00:00:00','1994-09-02 00:00:00',89.57,237,270,64.83,'Intermediate' from dual union all
select 23226,'023226','1994-053H','Sep Motor Cover','Sep Motor Cover',65449,'Reentered','1994-09-23 00:00:00','1994-09-02 00:00:00',91.8,207,519,64.74,'Intermediate' from dual union all
select 23227,'023227','1994-055A','Optus B3','Optus B3',65451,'In Earth orbit','','1994-09-28 00:00:00',1435.98,35646,35923,.76,'Stationary' from dual;
insert into SATELLITE
select 23228,'023228','1994-055B','CZ-2E Stage 2','CZ-2E Stage 2',65451,'Reentered','1994-12-27 00:00:00','1994-09-26 00:00:00',96.34,187,979,27.87,'Intermediate' from dual union all
select 23229,'023229','1994-055C','Star 63F S/N 005','Star 63F',65451,'In Earth orbit','','1994-09-27 00:00:00',697.72,377,38984,24.02,'GEO Transfer' from dual union all
select 2323,'002323','1966-062B','Agena D 4828','Agena D',13469,'Reentered','1966-07-14 00:00:00','1966-07-13 00:00:00',87.91,128,215,95.54,'Sun-Synch' from dual union all
select 23230,'023230','1994-056A','Kiku-6','ETS-6',65452,'In Earth orbit','','1994-09-27 00:00:00',845.89,7798,38702,13.11,'Highly Ellip' from dual union all
select 23231,'023231','1994-056B','H-II Stage 2','H-II Stage 2',65452,'Reentered','1995-04-02 00:00:00','1994-09-27 00:00:00',551.72,170,31640,28.64,'Highly Ellip' from dual union all
select 23232,'023232','1991-015H','deb Ariane V42','',64184,'In Earth orbit','','1994-09-02 00:00:00',500.46,302,28703,7.23,'Highly Ellip' from dual union all
select 23233,'023233','1994-057A','DMSP F-12','DMSP Block 5D-2 S-11',65453,'In Earth orbit','','1994-09-28 00:00:00',101.92,841,859,98.92,'Sun-Synch' from dual union all
select 23234,'023234','1994-057B','DMSP F-12 solar array band','',65453,'Reentered','2015-04-10 00:00:00','1994-09-30 00:00:00',101.94,844,857,98.91,'Sun-Synch' from dual union all
select 23235,'023235','1994-057C','DMSP F-12 solar array band','',65453,'Reentered','2015-03-13 00:00:00','1994-10-01 00:00:00',101.94,844,857,98.91,'Sun-Synch' from dual union all
select 23236,'023236','1991-015F','deb Ariane V42','',64184,'Reentered','2004-09-28 00:00:00','1994-08-31 00:00:00',500.01,231,28749,6.26,'Highly Ellip' from dual union all
select 23237,'023237','1992-021G','deb Ariane V50','',64733,'In Earth orbit','','1994-08-31 00:00:00',640.35,886,35577,3.72,'GEO Transfer' from dual union all
select 23238,'023238','1992-021H','deb Ariane V50','',64733,'In Earth orbit','','1994-09-02 00:00:00',641.54,964,35560,3.88,'GEO Transfer' from dual union all
select 23239,'023239','1992-021J','deb Ariane V50','',64733,'In Earth orbit','','1994-08-31 00:00:00',643.47,1051,35572,3.73,'GEO Transfer' from dual union all
select 2324,'002324','1966-063A','PasComSat','OV1-8 PasComSat',13470,'Reentered','1978-01-04 00:00:00','1966-08-15 00:00:00',105.14,955,1047,144.22,'Retrograde' from dual union all
select 23240,'023240','1992-021K','deb Ariane V50','',64733,'In Earth orbit','','1994-08-30 00:00:00',643.92,1061,35585,3.56,'GEO Transfer' from dual union all
select 23241,'023241','1992-021L','deb Ariane V50','',64733,'In Earth orbit','','1994-09-02 00:00:00',641.52,871,35652,3.68,'GEO Transfer' from dual union all
select 23242,'023242','1992-021M','deb Ariane V50','',64733,'In Earth orbit','','1994-08-30 00:00:00',641.52,934,35589,3.79,'GEO Transfer' from dual union all
select 23243,'023243','1989-027C','deb Ariane V30','',62304,'In Earth orbit','','1994-08-31 00:00:00',643.47,1051,35572,3.73,'GEO Transfer' from dual union all
select 23244,'023244','1992-021N','deb Ariane V50','',64733,'In Earth orbit','','1994-08-31 00:00:00',649.62,1340,35597,4.5,'GEO Transfer' from dual union all
select 23245,'023245','1992-021P','deb Ariane V50','',64733,'In Earth orbit','','1994-09-02 00:00:00',642.84,1003,35588,3.77,'GEO Transfer' from dual union all
select 23246,'023246','1991-015G','deb Ariane V42','',64184,'Reentered','2007-09-06 00:00:00','1994-08-30 00:00:00',499.44,240,28708,6.84,'Highly Ellip' from dual union all
select 23247,'023247','1994-054B','Centaur TC-11','Centaur TC-11',65450,'In Earth orbit','','1994-09-01 00:00:00',1429.12,35500,35800,2,'Drift GEO' from dual union all
select 23248,'023248','1994-056C','LAPS','LAPS',65452,'In Earth orbit','','1994-09-30 00:00:00',846.04,7819,38688,13.11,'Highly Ellip' from dual union all
select 23249,'023249','1994-058A','Telstar 402','Telstar 402',65454,'Reentered','2004-11-13 00:00:00','1994-10-09 00:00:00',627.61,206,35602,6.64,'GEO Transfer' from dual union all
select 2325,'002325','1966-064A','Kosmos-124','Zenit-2 No. 42',13471,'Landed','1966-07-22 00:00:00','1966-07-16 00:00:00',89.39,204,284,51.78,'Intermediate' from dual union all
select 23250,'023250','1994-057D','DMSP F-12 OLS Cover','',65453,'In Earth orbit','','1994-09-28 00:00:00',101.94,840,861,98.92,'Sun-Synch' from dual union all
select 23251,'023251','1994-059A','Discovery (STS-64)','OV-103',65455,'Landed','1994-09-20 00:00:00','1994-09-11 00:00:00',89.68,253,265,57.01,'Intermediate' from dual union all
select 23252,'023252','1986-017JB','EO-16 EVA-1 debris','',59210,'Reentered','1995-03-31 00:00:00','1994-09-11 00:00:00',92.46,394,397,51.65,'Intermediate' from dual union all
select 23253,'023253','1994-059B','Spartan 201','Spartan 201',65455,'Landed Attached','1994-09-20 00:00:00','1994-09-13 00:00:00',89.67,252,265,57,'Intermediate' from dual union all
select 23254,'023254','1990-081CK','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1994-09-16 00:00:00',101.1,750,872,98.76,'Sun-Synch' from dual union all
select 23255,'023255','1990-081CL','deb CZ-4 Stage 3','',63323,'Reentered','2014-05-31 00:00:00','1994-09-16 00:00:00',100.98,761,850,98.81,'Sun-Synch' from dual union all
select 23256,'023256','1990-081CM','deb CZ-4 Stage 3','',63323,'Reentered','2014-03-03 00:00:00','1994-09-15 00:00:00',102.93,849,946,98.85,'Sun-Synch' from dual union all
select 23257,'023257','1990-081CN','deb CZ-4 Stage 3','',63323,'Reentered','2011-10-08 00:00:00','1994-09-15 00:00:00',100.77,739,852,98.81,'Sun-Synch' from dual union all
select 23258,'023258','1990-081CP','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1994-09-10 00:00:00',102.17,841,881,98.77,'Sun-Synch' from dual union all
select 23259,'023259','1990-081CQ','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1994-09-16 00:00:00',102.12,846,871,98.79,'Sun-Synch' from dual union all
select 2326,'002326','1966-064B','11A57I','11A57I',13471,'Reentered','1966-07-19 00:00:00','1966-07-16 00:00:00',88.95,169,277,51.78,'Intermediate' from dual union all
select 23260,'023260','1990-081CR','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1994-09-15 00:00:00',101.55,812,852,98.81,'Sun-Synch' from dual union all
select 23261,'023261','1990-081CS','deb CZ-4 Stage 3','',63323,'In Earth orbit','','1994-09-16 00:00:00',102,825,882,98.82,'Sun-Synch' from dual union all
select 23262,'023262','1990-081CT','deb CZ-4 Stage 3','',63323,'Reentered','1998-07-08 00:00:00','1994-09-16 00:00:00',98.33,670,688,98.96,'Sun-Synch' from dual union all
select 23263,'023263','1991-009CU','deb S3M','',64178,'In Earth orbit','','1994-09-12 00:00:00',119.1,1513,1767,74.13,'Intermediate' from dual union all
select 23264,'023264','1991-009CV','deb S3M','',64178,'In Earth orbit','','1994-09-16 00:00:00',119.01,1443,1830,74.08,'Intermediate' from dual union all
select 23265,'023265','1991-009CW','deb S3M','',64178,'In Earth orbit','','1994-09-05 00:00:00',117.66,1447,1705,74.01,'Intermediate' from dual union all
select 23266,'023266','1986-017JC','EO-16 EVA-2 debris','',59210,'Reentered','1995-03-02 00:00:00','1994-09-18 00:00:00',92.44,393,396,51.65,'Intermediate' from dual union all
select 23267,'023267','1994-060A','Kosmos-2291','Geizer No. 19L',65456,'In Earth orbit','','1994-10-22 00:00:00',1436.06,35762,35810,1.4,'Stationary' from dual union all
select 23268,'023268','1994-060B','8S812','8S812',65456,'Reentered','1994-09-25 00:00:00','1994-09-24 00:00:00',87.71,157,166,51.61,'Intermediate' from dual union all
select 23269,'023269','1994-060C','Perekhodnik','Sredniy perekhodnik',65456,'Reentered','1994-09-22 00:00:00','1994-09-21 00:00:00',88.36,187,200,51.6,'Intermediate' from dual union all
select 2327,'002327','1966-063B','OV1-8 Container Half','Container Half',13470,'In Earth orbit','','1966-10-29 00:00:00',105.24,1000,1012,144.25,'Retrograde' from dual union all
select 23270,'023270','1994-060D','Blok DM2 No. 86L','11S861 No. 86L',65456,'In Earth orbit','','1994-11-06 00:00:00',1439.33,35790,35910,1.39,'Drift GEO' from dual union all
select 23271,'023271','1994-060E','SOZ','SOZ',65456,'Reentered','1995-07-29 00:00:00','1994-10-21 00:00:00',635.92,296,35940,47.45,'GEO Transfer' from dual union all
select 23272,'023272','1994-060F','SOZ','SOZ',65456,'Reentered','1995-05-19 00:00:00','1994-10-21 00:00:00',634.71,244,35930,47.39,'GEO Transfer' from dual union all
select 23273,'023273','1994-017G','deb P90-5','',65413,'In Earth orbit','','1994-03-31 00:00:00',95.65,550,550,105,'Retrograde' from dual union all
select 23274,'023274','1994-017H','deb P90-5','',65413,'In Earth orbit','','1994-03-31 00:00:00',95.65,550,550,105,'Retrograde' from dual union all
select 23275,'023275','1992-093JF','deb 11S772','',64805,'In Earth orbit','','1995-07-01 00:00:00',103.54,843,1009,71.25,'Intermediate' from dual union all
select 23276,'023276','1994-017J','deb P90-5','',65413,'In Earth orbit','','1994-03-31 00:00:00',95.65,550,550,105,'Retrograde' from dual union all
select 23277,'023277','1994-057E','DMSP F-12 OLS Cooler Cover','',65453,'In Earth orbit','','1994-09-30 00:00:00',101.92,841,859,98.92,'Sun-Synch' from dual union all
select 23278,'023278','1994-061A','Kosmos-2292','Vektor',65457,'In Earth orbit','','1994-10-27 00:00:00',108.91,400,1953,82.99,'Intermediate' from dual union all
select 23279,'023279','1994-061B','S3M','S3M',65457,'In Earth orbit','','1994-10-28 00:00:00',108.75,395,1943,82.99,'Intermediate' from dual union all
select 2328,'002328','1966-063C','OV1-8P','X-258',13470,'In Earth orbit','','1966-10-25 00:00:00',105.24,1001,1011,144.25,'Retrograde' from dual union all
select 23280,'023280','1994-022C','Centaur AC-73 (duplicate)','',65418,'Spurious','1994-09-29 00:00:00','1994-09-28 00:00:00',91.74,100,620,26.96,'Intermediate' from dual union all
select 23281,'023281','1994-061C','deb Kosmos-2292','',65457,'Reentered','2001-04-08 00:00:00','1994-10-27 00:00:00',108.47,365,1947,83,'Intermediate' from dual union all
select 23282,'023282','1992-093JG','deb 11S772','',64805,'In Earth orbit','','1994-09-28 00:00:00',103.8,841,1036,71.18,'Intermediate' from dual union all
select 23283,'023283','1994-042D','Kosmos-2283 part','Kobal''t AO part',65438,'Reentered','1994-09-30 00:00:00','1994-09-29 00:00:00',89.19,178,291,67.1,'Intermediate' from dual union all
select 23284,'023284','1994-042E','Kosmos-2283 part','Kobal''t AO part',65438,'Reentered','1994-09-30 00:00:00','1994-09-29 00:00:00',89.11,167,294,67.08,'Intermediate' from dual union all
select 23285,'023285','1994-062A','Endeavour (STS-68)','OV-105',65458,'Landed','1994-10-11 00:00:00','1994-10-02 00:00:00',88.85,212,224,57.01,'Intermediate' from dual union all
select 23286,'023286','1994-061D','deb Kosmos-2292','',65457,'Reentered','2011-04-03 00:00:00','1994-10-27 00:00:00',109.62,474,1945,83.08,'Intermediate' from dual union all
select 23287,'023287','1994-051F','Ullage rocket','Ullage rocket',65447,'Reentered','1994-10-10 00:00:00','1994-10-03 00:00:00',89.83,181,352,62.78,'Intermediate' from dual union all
select 23288,'023288','1994-063A','Soyuz TM-20','Soyuz 7K-STM No. 69',65459,'Landed','1995-03-22 00:00:00','1994-11-03 00:00:00',92.42,392,395,51.65,'Intermediate' from dual union all
select 23289,'023289','1994-063B','11S510','11S510',65459,'Reentered','1994-10-06 00:00:00','1994-10-05 00:00:00',88.45,189,207,51.64,'Intermediate' from dual union all
select 2329,'002329','1966-063D','OV1-8 Container Half','Container Half',13470,'In Earth orbit','','1966-11-07 00:00:00',105.41,1010,1018,144.26,'Retrograde' from dual union all
select 23290,'023290','1988-098E','deb Ariane V26','',61347,'Reentered','1994-11-04 00:00:00','1994-09-05 00:00:00',102.51,254,1501,3.95,'Equatorial' from dual union all
select 23291,'023291','1985-118N','deb SOZ','',58341,'Reentered','2000-09-02 00:00:00','1995-06-09 00:00:00',311.54,882,16808,64.15,'Highly Ellip' from dual union all
select 23292,'023292','1985-118P','deb SOZ','',58341,'In Earth orbit','','1995-06-29 00:00:00',336.84,334,18983,64.98,'Highly Ellip' from dual union all
select 23293,'023293','1985-118Q','deb SOZ','',58341,'In Earth orbit','','1995-06-29 00:00:00',341.4,560,19046,65.55,'Highly Ellip' from dual union all
select 23294,'023294','1985-118R','deb SOZ','',58341,'In Earth orbit','','1995-06-29 00:00:00',336.3,573,18709,65.34,'Highly Ellip' from dual union all
select 23295,'023295','1985-118S','deb SOZ','',58341,'Reentered','1999-11-19 00:00:00','1995-06-24 00:00:00',311.2,268,17400,64.62,'Highly Ellip' from dual union all
select 23296,'023296','1985-118T','deb SOZ','',58341,'Reentered','2014-04-08 00:00:00','1995-06-29 00:00:00',337.3,533,18813,64.5,'Highly Ellip' from dual union all
select 23297,'023297','1985-118U','deb SOZ','',58341,'In Earth orbit','','1995-06-13 00:00:00',319.65,822,17394,66.44,'Highly Ellip' from dual union all
select 23298,'023298','1985-118V','deb SOZ','',58341,'Reentered','2003-11-21 00:00:00','1995-06-14 00:00:00',321.41,682,17648,64.82,'Highly Ellip' from dual union all
select 23299,'023299','1992-039D','Star 48 yo weight','',64751,'Reentered','2006-02-21 00:00:00','1995-07-01 00:00:00',300.54,239,16730,34.4,'Highly Ellip' from dual union all
select 233,'000233','61 Omicron 64','deb r/b Ablestar','',7273,'In Earth orbit','','1962-02-08 00:00:00',103.6,865,993,66.88,'Intermediate' from dual union all
select 2330,'002330','1966-064C','deb Kosmos-124','',13471,'Reentered','1966-07-17 00:00:00','1966-07-15 00:00:00',88.89,166,274,51.77,'Intermediate' from dual union all
select 23300,'023300','1992-058D','deb Delta 214','',64770,'Reentered','1994-12-10 00:00:00','1994-11-26 00:00:00',93.87,437,490,19.76,'Equatorial' from dual union all
select 23301,'023301','1992-072D','deb Ariane V54','',64784,'In Earth orbit','','1995-06-27 00:00:00',483.27,484,27559,7.35,'Highly Ellip' from dual union all
select 23302,'023302','1992-093JH','deb 11S772','',64805,'In Earth orbit','','1995-07-01 00:00:00',103.52,822,1028,71.13,'Intermediate' from dual union all
select 23303,'023303','1993-059G','deb 11S772','',65161,'In Earth orbit','','1995-07-01 00:00:00',100.69,760,822,71.13,'Intermediate' from dual union all
select 23304,'023304','1993-016AH','deb 11S772','',65118,'Reentered','2015-02-12 00:00:00','1995-06-30 00:00:00',102.68,799,972,71,'Intermediate' from dual union all
select 23305,'023305','1994-064A','INTELSAT 703','INTELSAT 703',65460,'In Earth orbit','','1994-11-08 00:00:00',1436.5,35663,35926,.19,'Drift GEO' from dual union all
select 23306,'023306','1994-064B','Centaur AC-111','Centaur IIA(4N) AC-111',65460,'Reentered','1995-05-11 00:00:00','1994-11-05 00:00:00',659.94,174,37288,26.34,'GEO Transfer' from dual union all
select 23307,'023307','1968-066K','deb Explorer 40','',16793,'Reentered','2002-11-16 00:00:00','1994-10-06 00:00:00',107.15,638,1552,80.68,'Intermediate' from dual union all
select 23308,'023308','1974-105C','deb Kosmos-700','',33708,'In Earth orbit','','1994-10-07 00:00:00',104.6,960,992,82.95,'Intermediate' from dual union all
select 23309,'023309','1975-052JY','deb Delta 111','',36499,'In Earth orbit','','1994-10-07 00:00:00',113.55,1095,1683,99.69,'Sun-Synch' from dual union all
select 2331,'002331','1966-064D','deb Kosmos-124','',13471,'Reentered','1966-07-16 00:00:00','1966-07-15 00:00:00',88.86,170,266,51.79,'Intermediate' from dual union all
select 23310,'023310','1975-052JZ','deb Delta 111','',36499,'In Earth orbit','','1994-10-07 00:00:00',111.85,1059,1565,99.93,'Sun-Synch' from dual union all
select 23311,'023311','1985-118W','deb SOZ','',58341,'Reentered','1999-08-01 00:00:00','1994-10-05 00:00:00',317.4,379,17691,64.69,'Highly Ellip' from dual union all
select 23312,'023312','1992-027D','deb Delta','',64739,'Reentered','1996-10-22 00:00:00','1994-10-07 00:00:00',114.23,595,2246,19.7,'Equatorial' from dual union all
select 23313,'023313','1994-065A','Solidaridad 2','Solidaridad 2',65461,'In Earth orbit','','1994-11-15 00:00:00',1436.06,35777,35795,.05,'Stationary' from dual union all
select 23314,'023314','1994-065B','Thaicom 2','Thaicom 2',65461,'In Earth orbit','','1994-12-14 00:00:00',1436.09,35778,35795,.01,'Stationary' from dual union all
select 23315,'023315','1994-065C','Ariane H10+ V68','Ariane H10+ S/N T440',65461,'Reentered','1996-01-22 00:00:00','1994-11-09 00:00:00',612.8,155,34886,3.97,'GEO Transfer' from dual union all
select 23316,'023316','1994-065D','Spelda 30 S/N UV25','MiniSpelda',65461,'Reentered','1998-11-24 00:00:00','1994-11-09 00:00:00',625.92,215,35506,3.83,'GEO Transfer' from dual union all
select 23317,'023317','1994-066A','Okean','Okean-O1 No. 7/NKhM No.',65462,'In Earth orbit','','1994-11-10 00:00:00',97.68,630,666,82.54,'Intermediate' from dual;
insert into SATELLITE
select 23318,'023318','1994-066B','S5M','S5M',65462,'In Earth orbit','','1994-11-10 00:00:00',97.67,630,665,82.54,'Intermediate' from dual union all
select 23319,'023319','1994-067A','Ekspress-2','Ekspress No. 11L',65463,'In Earth orbit','','1994-11-12 00:00:00',1435.83,35777,35786,.16,'Stationary' from dual union all
select 2332,'002332','1966-064E','deb Kosmos-124','',13471,'Reentered','1966-07-19 00:00:00','1966-07-16 00:00:00',88.98,160,289,51.75,'Intermediate' from dual union all
select 23320,'023320','1994-067B','8S812','8S812',65463,'Reentered','1994-10-16 00:00:00','1994-10-15 00:00:00',88.11,181,181,51.61,'Intermediate' from dual union all
select 23321,'023321','1994-067C','Perekhodnik','Sredniy perekhodnik',65463,'Reentered','1994-10-14 00:00:00','1994-10-13 00:00:00',88.48,189,210,51.61,'Intermediate' from dual union all
select 23322,'023322','1994-067D','Blok DM-2M No. 1L','11S861-01 No. 1L',65463,'In Earth orbit','','1994-12-04 00:00:00',1437.82,35782,35859,.15,'Drift GEO' from dual union all
select 23323,'023323','1994-068A','IRS-P2','IRS-P2',65464,'In Earth orbit','','1994-11-15 00:00:00',101.27,818,820,98.69,'Sun-Synch' from dual union all
select 23324,'023324','1994-068B','PSLV-D2 PS4','PS4',65464,'In Earth orbit','','1994-11-17 00:00:00',101.71,798,882,98.7,'Sun-Synch' from dual union all
select 23325,'023325','1994-067E','SOZ','SOZ',65463,'Reentered','1995-08-12 00:00:00','1994-11-12 00:00:00',635.01,295,35894,47.45,'GEO Transfer' from dual union all
select 23326,'023326','1994-067F','SOZ','SOZ',65463,'Reentered','1995-05-12 00:00:00','1994-11-12 00:00:00',633.12,227,35865,47.41,'GEO Transfer' from dual union all
select 23327,'023327','1994-069A','Elektro','Elektro No. 1L',65465,'In Earth orbit','','1994-12-04 00:00:00',1441.68,35863,35929,1.21,'Drift GEO' from dual union all
select 23328,'023328','1994-069B','8S812','8S812',65465,'Reentered','1994-11-03 00:00:00','1994-11-02 00:00:00',87.88,169,170,51.66,'Intermediate' from dual union all
select 23329,'023329','1994-069C','Perekhodnik','Sredniy perekhodnik',65465,'Reentered','1994-11-01 00:00:00','1994-10-31 00:00:00',88.42,184,209,51.65,'Intermediate' from dual union all
select 2333,'002333','1965-082KD','deb Transtage 4','',11832,'Reentered','1967-09-29 00:00:00','1967-08-31 00:00:00',95.05,475,566,32.54,'Intermediate' from dual union all
select 23330,'023330','1994-069D','Blok DM2 No. 56L','11S861 No. 56L',65465,'In Earth orbit','','1994-12-04 00:00:00',1438.64,35752,35920,1.25,'Drift GEO' from dual union all
select 23331,'023331','1994-070A','Astra 1D','Astra 1D',65466,'In Earth orbit','','1994-12-13 00:00:00',1435.83,35742,35821,.08,'Stationary' from dual union all
select 23332,'023332','1994-070B','Ariane H10+ V69','Ariane H10+ S/N T441',65466,'Reentered','2004-01-20 00:00:00','1994-12-06 00:00:00',558.59,222,31957,6.9,'Highly Ellip' from dual union all
select 23333,'023333','1994-071A','Wind','WIND',65467,'Deep Space','1994-11-02 00:00:00','1994-11-01 00:00:00',19700.41,186,470310,28.75,'Cislunar' from dual union all
select 23334,'023334','1994-071B','Delta 227','Delta SSPS AJ10-118K',65467,'Reentered','1995-10-20 00:00:00','1994-12-01 00:00:00',116.59,179,2876,27.68,'Intermediate' from dual union all
select 23335,'023335','1991-086J','deb Interkosmos-25','',64255,'Reentered','2011-10-15 00:00:00','1994-11-14 00:00:00',118.18,431,2768,82.56,'Intermediate' from dual union all
select 23336,'023336','1994-072A','Kosmos-2293','US-PM',65468,'Deorbited','1996-05-13 00:00:00','1994-12-02 00:00:00',92.76,403,417,65.03,'Intermediate' from dual union all
select 23337,'023337','1994-072B','11S692','11S692',65468,'Reentered','1994-11-02 00:00:00','1994-11-02 00:00:00',88.97,111,337,64.99,'Intermediate' from dual union all
select 23338,'023338','1994-069E','SOZ','SOZ',65465,'Reentered','1995-05-11 00:00:00','1994-12-01 00:00:00',634.6,245,35923,47.38,'GEO Transfer' from dual union all
select 23339,'023339','1994-069F','SOZ','SOZ',65465,'Reentered','1995-07-06 00:00:00','1994-12-01 00:00:00',635.98,299,35940,47.41,'GEO Transfer' from dual union all
select 2334,'002334','1965-020EH','deb Blok Kosmos','',11770,'In Earth orbit','','1992-02-02 00:00:00',110.69,783,1734,55.7,'Intermediate' from dual union all
select 23340,'023340','1994-073A','Atlantis (STS-66)','OV-104',65469,'Landed','1994-11-14 00:00:00','1994-11-05 00:00:00',90.57,294,311,56.99,'Intermediate' from dual union all
select 23341,'023341','1994-073B','CRISTA-SPAS','CRISTA-SPAS',65469,'Landed Attached','1994-11-14 00:00:00','1994-11-05 00:00:00',90.55,294,308,57,'Intermediate' from dual union all
select 23342,'023342','1994-074A','Resurs-O1','Resurs-O1 No. 3L',65470,'In Earth orbit','','1994-12-05 00:00:00',97.96,661,662,98.05,'Sun-Synch' from dual union all
select 23343,'023343','1994-074B','11S772','11S772',65470,'In Earth orbit','','1994-12-04 00:00:00',97.91,652,666,98.05,'Sun-Synch' from dual union all
select 23344,'023344','1994-074C','Sep Motor Cover','Sep Motor Cover',65470,'In Earth orbit','','1994-12-04 00:00:00',99.91,645,864,98.3,'Sun-Synch' from dual union all
select 23345,'023345','1994-074D','Sep Motor Cover','Sep Motor Cover',65470,'In Earth orbit','','1994-12-07 00:00:00',101.22,661,972,97.87,'Sun-Synch' from dual union all
select 23346,'023346','1994-074E','Sep Motor Cover','Sep Motor Cover',65470,'In Earth orbit','','1994-12-04 00:00:00',101.07,662,957,98.29,'Sun-Synch' from dual union all
select 23347,'023347','1994-074F','Sep Motor Cover','Sep Motor Cover',65470,'In Earth orbit','','1994-12-04 00:00:00',100.05,647,875,97.82,'Sun-Synch' from dual union all
select 23348,'023348','1994-075A','Progress M-25','Progress 7K-TGM No. 225',65471,'Deorbited','1995-02-16 00:00:00','1994-12-11 00:00:00',92.4,391,394,51.65,'Intermediate' from dual union all
select 23349,'023349','1994-075B','11S510','11S510',65471,'Reentered','1994-11-13 00:00:00','1994-11-12 00:00:00',88.36,175,212,51.63,'Intermediate' from dual union all
select 2335,'002335','1965-065M','deb NNS O-5                       ','[UNID012] RCS 0.0',11815,'Reentered','1981-04-13 00:00:00','1973-04-28 00:00:00',106.9,1024,1142,89.98,'Polar' from dual union all
select 23350,'023350','1994-071C','Star 48B','Star 48B',65467,'Deep Space','1994-11-02 00:00:00','1994-11-01 00:00:00',19700.41,186,470310,28.75,'Cislunar' from dual union all
select 23351,'023351','1989-061X','deb USA-40','',62338,'In Earth orbit','','1998-09-23 00:00:00',150.59,342,5654,56.96,'Medium' from dual union all
select 23352,'023352','1989-061Y','deb USA-40','',62338,'In Earth orbit','','1998-12-04 00:00:00',182.26,376,8165,56.95,'Medium' from dual union all
select 23353,'023353','1989-061Z','deb USA-40','',62338,'In Earth orbit','','1998-10-28 00:00:00',182.36,518,8031,56.97,'Medium' from dual union all
select 23354,'023354','1989-061AA','deb USA-40','',62338,'In Earth orbit','','1998-10-08 00:00:00',181.78,505,7999,57.27,'Medium' from dual union all
select 23355,'023355','1989-061AB','deb USA-40','',62338,'In Earth orbit','','1998-09-11 00:00:00',182.85,515,8071,57.01,'Medium' from dual union all
select 23356,'023356','1989-061AC','deb USA-40','',62338,'In Earth orbit','','1998-10-14 00:00:00',181.5,526,7956,56.94,'Medium' from dual union all
select 23357,'023357','1989-061AD','deb USA-40','',62338,'In Earth orbit','','1998-08-10 00:00:00',182.3,511,8032,57.41,'Medium' from dual union all
select 23358,'023358','1989-061AE','deb USA-40','',62338,'In Earth orbit','','1998-10-29 00:00:00',182.55,516,8048,56.98,'Medium' from dual union all
select 23359,'023359','1989-061AF','deb USA-40','',62338,'In Earth orbit','','1998-11-28 00:00:00',181.8,572,7933,56.87,'Medium' from dual union all
select 2336,'002336','1966-044C','Explorer 32 despin weight','',13451,'Reentered','1969-08-31 00:00:00','1966-07-14 00:00:00',117.77,335,2827,64.59,'Intermediate' from dual union all
select 23360,'023360','1989-061AG','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23361,'023361','1989-061AH','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23362,'023362','1989-061AJ','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23363,'023363','1989-061AK','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23364,'023364','1989-061AL','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23365,'023365','1989-061AM','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23366,'023366','1989-061AN','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23367,'023367','1989-061AP','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23368,'023368','1989-061AQ','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23369,'023369','1989-061AR','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 2337,'002337','1966-063E','OV1-8P control assembly','',13470,'In Earth orbit','','1966-10-21 00:00:00',105.31,1005,1013,144.25,'Retrograde' from dual union all
select 23370,'023370','1989-061AS','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23371,'023371','1989-061AT','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23372,'023372','1989-061AU','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23373,'023373','1989-061AV','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23374,'023374','1989-061AW','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23375,'023375','1989-061AX','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23376,'023376','1989-061AY','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23377,'023377','1989-061AZ','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23378,'023378','1989-061BA','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23379,'023379','1989-061BB','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 2338,'002338','1965-053J','deb Blok Kosmos','',11803,'Reentered','1967-11-12 00:00:00','1966-04-04 00:00:00',95.3,526,541,56.08,'Intermediate' from dual union all
select 23380,'023380','1989-061BC','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23381,'023381','1989-061BD','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23382,'023382','1989-061BE','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23383,'023383','1989-061BF','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23384,'023384','1989-061BG','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23385,'023385','1989-061BH','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23386,'023386','1989-061BJ','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23387,'023387','1989-061BK','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23388,'023388','1989-061BL','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23389,'023389','1989-061BM','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 2339,'002339','1965-020EJ','deb Blok Kosmos','',11770,'Reentered','1966-11-12 00:00:00','1966-11-06 00:00:00',89.64,257,257,56.08,'Intermediate' from dual union all
select 23390,'023390','1989-061BN','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23391,'023391','1989-061BP','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23392,'023392','1989-061BQ','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23393,'023393','1989-061BR','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23394,'023394','1989-061BS','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23395,'023395','1989-061BT','deb USA-40','',62338,'In Earth orbit','','1994-10-31 00:00:00',183.88,532,8135,57,'Medium' from dual union all
select 23396,'023396','1994-076A','Kosmos-2294','Uragan No. 62L',65472,'In Earth orbit','','1994-12-20 00:00:00',675.69,19050,19208,64.89,'Medium' from dual union all
select 23397,'023397','1994-076B','Kosmos-2295','Uragan No. 63L',65472,'In Earth orbit','','1994-12-21 00:00:00',675.71,19103,19156,64.88,'Medium' from dual union all
select 23398,'023398','1994-076C','Kosmos-2296','Uragan No. 64L',65472,'In Earth orbit','','1994-12-20 00:00:00',675.71,19123,19136,64.87,'Medium' from dual union all
select 23399,'023399','1994-076D','8S812','8S812',65472,'Reentered','1994-11-20 00:00:00','1994-11-20 00:00:00',87.39,139,152,64.83,'Intermediate' from dual union all
select 234,'000234','61 Omicron 65','deb r/b Ablestar','',7273,'In Earth orbit','','1962-02-23 00:00:00',103.69,869,997,66.88,'Intermediate' from dual union all
select 2340,'002340','1965-082KE','deb Transtage 4','',11832,'Reentered','1973-07-11 00:00:00','1966-07-16 00:00:00',99.21,686,755,32.03,'Intermediate' from dual union all
select 23400,'023400','1994-076E','Perekhodnik','Sredniy perekhodnik',65472,'Reentered','1994-11-20 00:00:00','1994-11-20 00:00:00',87.2,126,146,64.81,'Intermediate' from dual union all
select 23401,'023401','1994-076F','Blok DM2 No. 97L','11S861 No. 97L',65472,'In Earth orbit','','1994-12-22 00:00:00',674.28,19056,19131,64.88,'Medium' from dual union all
select 23402,'023402','1994-076G','SOZ','SOZ',65472,'In Earth orbit','','1995-01-12 00:00:00',339.96,419,19096,64.95,'Highly Ellip' from dual union all
select 23403,'023403','1994-076H','SOZ','SOZ',65472,'In Earth orbit','','1994-12-20 00:00:00',339.96,410,19104,64.78,'Highly Ellip' from dual union all
select 23404,'023404','1994-077A','Kosmos-2297','Tselina-2',65473,'In Earth orbit','','1994-12-24 00:00:00',101.95,848,854,71,'Intermediate' from dual union all
select 23405,'023405','1994-077B','11S772','11S772',65473,'In Earth orbit','','1994-12-24 00:00:00',101.8,841,846,70.97,'Intermediate' from dual union all
select 23406,'023406','1994-077C','Sep Motor Cover','Sep Motor Cover',65473,'In Earth orbit','','1994-12-26 00:00:00',105.02,845,1146,71.03,'Intermediate' from dual union all
select 23407,'023407','1994-077D','Sep Motor Cover','Sep Motor Cover',65473,'In Earth orbit','','1994-12-24 00:00:00',104.99,845,1143,71.03,'Intermediate' from dual;
insert into SATELLITE
select 23408,'023408','1994-077E','Sep Motor Cover','Sep Motor Cover',65473,'In Earth orbit','','1994-12-26 00:00:00',105.18,844,1162,70.98,'Intermediate' from dual union all
select 23409,'023409','1994-077F','deb Zenit','Slow decay',65473,'In Earth orbit','','1994-12-24 00:00:00',103.24,844,980,70.88,'Intermediate' from dual union all
select 2341,'002341','1966-059AD','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-15 00:00:00','1966-07-09 00:00:00',89.36,190,296,33.02,'Intermediate' from dual union all
select 23410,'023410','1994-077G','Sep Motor Cover','Sep Motor Cover',65473,'In Earth orbit','','1994-12-27 00:00:00',105.38,847,1177,70.98,'Intermediate' from dual union all
select 23411,'023411','1994-078A','Geo-IK','Musson No. 24',65474,'In Earth orbit','','1994-12-30 00:00:00',116.04,1479,1527,73.61,'Intermediate' from dual union all
select 23412,'023412','1994-078B','S5M','S5M',65474,'In Earth orbit','','1995-01-01 00:00:00',116.01,1479,1524,73.61,'Intermediate' from dual union all
select 23413,'023413','1994-079A','Orion 1','Orion 1',65475,'In Earth orbit','','1994-12-31 00:00:00',1436.09,35776,35797,.03,'Stationary' from dual union all
select 23414,'023414','1994-079B','Centaur AC-110','Centaur IIA(4N) AC-110',65475,'Reentered','1999-05-31 00:00:00','1995-01-04 00:00:00',2881,599,120782,26.59,'Very High' from dual union all
select 23415,'023415','1994-080A','DFH-3','DFH-3',65476,'In Earth orbit','','1994-12-29 00:00:00',1425.88,35181,35992,.26,'Drift GEO' from dual union all
select 23416,'023416','1994-080B','CZ-3A Stage 3','CZ-3A Stage 3',65476,'Reentered','2003-10-13 00:00:00','1994-12-29 00:00:00',634.27,190,35961,27.83,'GEO Transfer' from dual union all
select 23417,'023417','1994-077H','deb Kosmos-2297','Rapid decay',65473,'Reentered','2000-04-10 00:00:00','1994-12-25 00:00:00',101.7,828,850,71,'Intermediate' from dual union all
select 23418,'023418','1994-077J','deb Kosmos-2297','Slow decay',65473,'In Earth orbit','','2006-04-04 00:00:00',102.43,817,931,71.01,'Intermediate' from dual union all
select 23419,'023419','1994-077K','deb Kosmos-2297','Rapid decay',65473,'Reentered','2001-02-15 00:00:00','1994-12-24 00:00:00',101.78,839,847,71,'Intermediate' from dual union all
select 2342,'002342','1966-059AE','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-20 00:00:00','1966-07-15 00:00:00',89.9,113,426,31.89,'Intermediate' from dual union all
select 23420,'023420','1994-081A','Molniya-1T','Molniya-1T',65477,'In Earth orbit','','1995-01-13 00:00:00',717.62,460,39887,62.82,'Molniya' from dual union all
select 23421,'023421','1994-081B','11S510','11S510',65477,'Reentered','1995-01-14 00:00:00','1994-12-21 00:00:00',90.66,205,408,62.83,'Intermediate' from dual union all
select 23422,'023422','1994-081C','BOZ','BOZ',65477,'Reentered','1994-12-30 00:00:00','1994-12-21 00:00:00',90.15,179,385,62.8,'Intermediate' from dual union all
select 23423,'023423','1994-081D','Blok-ML','Blok-ML',65477,'Reentered','2012-10-17 00:00:00','1995-01-24 00:00:00',700.7,467,39042,62.79,'Molniya' from dual union all
select 23424,'023424','1977-044E','DMSP F-2 OLS Cooler Cover','',42172,'Reentered','1995-01-01 00:00:00','1995-01-01 00:00:00',101.03,780,834,99,'Sun-Synch' from dual union all
select 23425,'023425','1991-017C','deb USA-69','',64186,'Reentered','1995-01-01 00:00:00','1995-01-01 00:00:00',98.05,658,673,67.99,'Intermediate' from dual union all
select 23426,'023426','1994-082A','Luch','Al''tair No. 13L',65478,'In Earth orbit','','1995-01-15 00:00:00',1435.88,35755,35810,2.53,'Stationary' from dual union all
select 23427,'023427','1994-082B','8S812','8S812',65478,'Reentered','1994-12-19 00:00:00','1994-12-18 00:00:00',88.01,175,177,51.66,'Intermediate' from dual union all
select 23428,'023428','1994-082C','Perekhodnik','Sredniy perekhodnik',65478,'Reentered','1994-12-17 00:00:00','1994-12-16 00:00:00',88.4,188,203,51.65,'Intermediate' from dual union all
select 23429,'023429','1994-082D','Blok DM2 No. 75L','11S861 No. 75L',65478,'In Earth orbit','','1995-01-19 00:00:00',1428.69,35516,35767,2.55,'Drift GEO' from dual union all
select 2343,'002343','1966-059AF','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-20 00:00:00','1966-07-16 00:00:00',89.7,194,326,31.85,'Intermediate' from dual union all
select 23430,'023430','1994-082E','SOZ','SOZ',65478,'Reentered','2001-10-27 00:00:00','1995-01-19 00:00:00',632.65,333,35735,47.3,'GEO Transfer' from dual union all
select 23431,'023431','1994-083A','Kosmos-2298','Strela-2M',65479,'In Earth orbit','','1995-01-23 00:00:00',100.82,785,810,74.03,'Intermediate' from dual union all
select 23432,'023432','1994-083B','S3M','S3M',65479,'In Earth orbit','','1995-01-19 00:00:00',100.64,779,799,74.03,'Intermediate' from dual union all
select 23433,'023433','1994-083C','deb Kosmos-2298','Strela-2M debris 1',65479,'In Earth orbit','','1995-01-21 00:00:00',100.91,793,811,74.01,'Intermediate' from dual union all
select 23434,'023434','1994-083D','deb Kosmos-2298','Strela-2M debris 2 (high',65479,'In Earth orbit','','1995-01-19 00:00:00',101.17,805,824,74.03,'Intermediate' from dual union all
select 23435,'023435','1994-084A','DSP F17','DSP 17',65480,'In Earth orbit','','1995-03-01 00:00:00',1435.5,35750,35800,0,'Stationary' from dual union all
select 23436,'023436','1994-084B','Titan 45K-14 Stage 2','LR91AJ11A 45K-14',65480,'Reentered','1994-12-24 00:00:00','1994-12-22 00:00:00',88.16,152,215,28.6,'Intermediate' from dual union all
select 23437,'023437','1994-084C','IUS-20 SRM-1','Orbus 21',65480,'In Earth orbit','2000-01-01 00:00:00','1994-12-23 00:00:00',630.94,180,35800,26.5,'GEO Transfer' from dual union all
select 23438,'023438','1994-084D','IUS-20 SRM-2','Orbus 6E',65480,'In Earth orbit','','1995-03-01 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 23439,'023439','1994-085A','RS-15 Radio-ROSTO','Radio-ROSTO',65481,'In Earth orbit','','1995-01-25 00:00:00',127.69,1881,2163,64.82,'Medium' from dual union all
select 2344,'002344','1966-059AG','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-20 00:00:00','1966-07-15 00:00:00',89.83,139,394,31.86,'Intermediate' from dual union all
select 23440,'023440','1994-085B','Briz-K','Briz-K',65481,'In Earth orbit','','1995-01-29 00:00:00',127.74,1866,2182,64.84,'Medium' from dual union all
select 23441,'023441','1994-086A','Kosmos-2299','Strela-3',65482,'In Earth orbit','','1995-01-29 00:00:00',113.95,1401,1415,82.57,'Intermediate' from dual union all
select 23442,'023442','1994-086B','Kosmos-2300','Strela-3',65482,'In Earth orbit','','1995-01-30 00:00:00',114.04,1409,1415,82.55,'Intermediate' from dual union all
select 23443,'023443','1994-086C','Kosmos-2301','Strela-3',65482,'In Earth orbit','','1995-01-29 00:00:00',114.1,1414,1415,82.56,'Intermediate' from dual union all
select 23444,'023444','1994-086D','Kosmos-2302','Strela-3',65482,'In Earth orbit','','1995-01-29 00:00:00',114.18,1414,1423,82.56,'Intermediate' from dual union all
select 23445,'023445','1994-086E','Kosmos-2303','Strela-3',65482,'In Earth orbit','','1995-01-29 00:00:00',114.24,1414,1428,82.57,'Intermediate' from dual union all
select 23446,'023446','1994-086F','Kosmos-2304','Strela-3',65482,'In Earth orbit','','1995-01-30 00:00:00',114.1,1412,1417,82.56,'Intermediate' from dual union all
select 23447,'023447','1994-086G','S5M','S5M',65482,'In Earth orbit','','1995-01-29 00:00:00',114.69,1413,1470,82.56,'Intermediate' from dual union all
select 23448,'023448','1994-087A','Raduga','Gran'' No. 43L',65483,'In Earth orbit','','1995-01-27 00:00:00',1436.21,35785,35793,1.42,'Stationary' from dual union all
select 23449,'023449','1994-087B','8S812','8S812',65483,'Reentered','1994-12-31 00:00:00','1994-12-30 00:00:00',88.01,174,177,51.65,'Intermediate' from dual union all
select 2345,'002345','1966-059AH','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-18 00:00:00','1966-07-15 00:00:00',89.75,252,273,32,'Intermediate' from dual union all
select 23450,'023450','1994-087C','Perekhodnik','Sredniy perekhodnik',65483,'Reentered','1994-12-29 00:00:00','1994-12-28 00:00:00',88.12,178,185,51.64,'Intermediate' from dual union all
select 23451,'023451','1994-087D','Blok DM2 No. 62L','11S861 No. 62L',65483,'In Earth orbit','','1995-02-03 00:00:00',1439.1,35792,35898,1.43,'Drift GEO' from dual union all
select 23452,'023452','1994-082F','SOZ','SOZ',65478,'Reentered','1995-10-04 00:00:00','1995-01-18 00:00:00',631.45,277,35729,47.26,'GEO Transfer' from dual union all
select 23453,'023453','1994-088A','Kosmos-2305','Neman',65484,'Deorbited','1995-12-18 00:00:00','1995-01-28 00:00:00',89.99,240,307,64.9,'Intermediate' from dual union all
select 23454,'023454','1994-088B','11S510','11S510',65484,'Reentered','1995-01-03 00:00:00','1994-12-31 00:00:00',88.74,173,252,64.92,'Intermediate' from dual union all
select 23455,'023455','1994-089A','NOAA 14','NOAA J',65485,'In Earth orbit','','1995-01-29 00:00:00',102.01,848,859,98.89,'Sun-Synch' from dual union all
select 23456,'023456','1993-071C','Kosmos-2267 part','Neman AO part',65173,'Reentered','1994-12-31 00:00:00','1994-12-30 00:00:00',88.7,168,253,70.32,'Intermediate' from dual union all
select 23457,'023457','1994-089B','NOAA-14 solar array band','',65485,'In Earth orbit','','1995-01-29 00:00:00',101.99,849,857,98.89,'Sun-Synch' from dual union all
select 23458,'023458','1994-089C','NOAA-14 solar array band','',65485,'Reentered','2011-11-02 00:00:00','1995-01-29 00:00:00',101.98,848,856,98.88,'Sun-Synch' from dual union all
select 23459,'023459','1994-087E','SOZ','SOZ',65483,'Reentered','1996-05-27 00:00:00','1995-01-29 00:00:00',634.91,279,35905,47.22,'GEO Transfer' from dual union all
select 2346,'002346','1966-059AJ','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-19 00:00:00','1966-07-14 00:00:00',90.09,174,384,31.9,'Intermediate' from dual union all
select 23460,'023460','1994-087F','SOZ','SOZ',65483,'Reentered','2002-02-17 00:00:00','1995-01-29 00:00:00',636.12,331,35915,47.27,'GEO Transfer' from dual union all
select 23461,'023461','1995-001A','INTELSAT 704','INTELSAT 704',65662,'In Earth orbit','','1995-02-21 00:00:00',1433.66,35687,35791,.05,'Drift GEO' from dual union all
select 23462,'023462','1995-001B','Centaur AC-113','Centaur IIA(1N) AC-113',65662,'Reentered','1996-06-27 00:00:00','1995-02-09 00:00:00',726.8,160,40639,26.57,'GEO Transfer' from dual union all
select 23463,'023463','1995-002A','Tsikada','Tsikada',65663,'In Earth orbit','','1995-02-24 00:00:00',104.94,964,1019,82.93,'Intermediate' from dual union all
select 23464,'023464','1995-002B','Astrid','Astrid',65663,'In Earth orbit','','1995-02-23 00:00:00',105.01,965,1025,82.93,'Intermediate' from dual union all
select 23465,'023465','1995-002C','FAISAT','FAISAT',65663,'In Earth orbit','','1995-02-25 00:00:00',104.98,967,1019,82.93,'Intermediate' from dual union all
select 23466,'023466','1995-002D','S3M','S3M',65663,'In Earth orbit','','1995-02-25 00:00:00',104.84,964,1010,82.92,'Intermediate' from dual union all
select 23467,'023467','1995-003A','UHF F/O F4','UFO F4 EHF',65664,'In Earth orbit','','1995-03-08 00:00:00',1435.91,35646,35920,5.11,'Inclined GEO' from dual union all
select 23468,'023468','1995-003B','Centaur AC-112','Centaur II(3) AC-112',65664,'In Earth orbit','','1995-03-02 00:00:00',477.56,288,27433,26.9,'Highly Ellip' from dual union all
select 23469,'023469','1995-004A','Discovery (STS-63)','OV-103',65665,'Landed','1995-02-11 00:00:00','1995-02-05 00:00:00',91.6,332,375,51.65,'Intermediate' from dual union all
select 2347,'002347','1966-059AK','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-19 00:00:00','1966-07-16 00:00:00',89.73,223,300,31.96,'Intermediate' from dual union all
select 23470,'023470','1995-004B','Spartan-204','Spartan-204',65665,'Landed Attached','1995-02-11 00:00:00','1995-02-08 00:00:00',92.31,388,388,51.65,'Intermediate' from dual union all
select 23471,'023471','1995-004C','ODERACS 2A','ODERACS 6 inch sphere',65665,'Reentered','1996-03-13 00:00:00','1995-03-05 00:00:00',91.24,325,346,51.66,'Intermediate' from dual union all
select 23472,'023472','1995-004D','ODERACS 2B','ODERACS 4 inch sphere',65665,'Reentered','1995-09-29 00:00:00','1995-03-06 00:00:00',91.1,317,340,51.65,'Intermediate' from dual union all
select 23473,'023473','1995-004E','ODERACS 2C','ODERACS 2 inch sphere',65665,'Reentered','1996-02-07 00:00:00','1995-02-04 00:00:00',91.13,320,340,51.65,'Intermediate' from dual union all
select 23474,'023474','1995-004F','ODERACS 2D','ODERACS dipole',65665,'Reentered','1995-03-02 00:00:00','1995-02-11 00:00:00',91.03,316,335,51.64,'Intermediate' from dual union all
select 23475,'023475','1995-004G','ODERACS 2E','ODERACS dipole',65665,'Reentered','1995-02-27 00:00:00','1995-02-11 00:00:00',91.11,320,338,51.64,'Intermediate' from dual union all
select 23476,'023476','1983-075BC','deb Kosmos-1484','',55832,'Reentered','1996-11-26 00:00:00','1995-03-28 00:00:00',94.83,486,535,98.32,'Sun-Synch' from dual union all
select 23477,'023477','1995-005A','Progress M-26','Progress 7K-TGM No. 226',65666,'Deorbited','1995-03-15 00:00:00','1995-02-23 00:00:00',92.43,391,397,51.65,'Intermediate' from dual union all
select 23478,'023478','1995-005B','11S510','11S510',65666,'Reentered','1995-02-17 00:00:00','1995-02-16 00:00:00',88.35,181,205,51.62,'Intermediate' from dual union all
select 23479,'023479','1994-085C','deb Briz','',65481,'In Earth orbit','','1995-02-15 00:00:00',126.28,1863,2057,65.01,'Medium' from dual union all
select 2348,'002348','1966-065A','TDA 1A','TDA 1A',13472,'Reentered Attached','1966-12-29 00:00:00','1966-10-31 00:00:00',90.7,301,316,28.86,'Intermediate' from dual union all
select 23480,'023480','1994-085D','deb Briz','',65481,'In Earth orbit','','1995-02-15 00:00:00',127.08,1822,2168,64.68,'Medium' from dual union all
select 23481,'023481','1971-120G','deb Meteor','',24938,'Reentered','2001-09-30 00:00:00','1992-07-29 00:00:00',101.03,802,813,81.23,'Intermediate' from dual union all
select 23482,'023482','1994-085E','deb Briz','',65481,'In Earth orbit','','1995-02-15 00:00:00',128.16,1816,2269,64.82,'Medium' from dual union all
select 23483,'023483','1994-085F','deb Briz','',65481,'In Earth orbit','','1995-02-15 00:00:00',128.23,1874,2216,64.73,'Medium' from dual union all
select 23484,'023484','1994-085G','deb Briz','',65481,'In Earth orbit','','1995-02-15 00:00:00',128.61,1858,2266,64.8,'Medium' from dual union all
select 23485,'023485','1994-085H','deb Briz','',65481,'In Earth orbit','','1995-02-15 00:00:00',127.96,1854,2213,64.65,'Medium' from dual union all
select 23486,'023486','1994-085J','deb Briz','',65481,'In Earth orbit','','1995-02-14 00:00:00',128.2,1878,2210,64.9,'Medium' from dual union all
select 23487,'023487','1994-085K','deb Briz','',65481,'In Earth orbit','','1995-02-14 00:00:00',127.39,1858,2159,64.94,'Medium' from dual union all
select 23488,'023488','1994-085L','deb Briz','',65481,'In Earth orbit','','1995-02-12 00:00:00',128.06,1844,2232,64.96,'Medium' from dual union all
select 23489,'023489','1994-085M','deb Briz','',65481,'In Earth orbit','','1995-02-15 00:00:00',127.87,1867,2192,64.94,'Medium' from dual union all
select 2349,'002349','1966-066A','Gemini X','Gemini SC10',13473,'Landed','1966-07-21 00:00:00','1966-07-19 00:00:00',95.19,299,757,28.8,'Intermediate' from dual union all
select 23490,'023490','1994-085N','deb Briz','',65481,'In Earth orbit','','1995-02-15 00:00:00',128.51,1937,2178,64.84,'Medium' from dual union all
select 23491,'023491','1994-085P','deb Briz','',65481,'In Earth orbit','','1995-02-14 00:00:00',128,1833,2238,64.88,'Medium' from dual union all
select 23492,'023492','1994-085Q','deb Briz','',65481,'In Earth orbit','','1995-02-14 00:00:00',127.32,1851,2160,64.95,'Medium' from dual union all
select 23493,'023493','1994-085R','deb Briz','',65481,'In Earth orbit','','1995-02-15 00:00:00',127.26,1871,2135,64.95,'Medium' from dual union all
select 23494,'023494','1994-085S','deb Briz','',65481,'In Earth orbit','','1995-02-14 00:00:00',127.92,1729,2335,64.95,'Medium' from dual union all
select 23495,'023495','1994-085T','deb Briz','',65481,'In Earth orbit','','1995-02-15 00:00:00',127.32,1846,2165,65.02,'Medium' from dual union all
select 23496,'023496','1994-085U','deb Briz','',65481,'In Earth orbit','','1995-02-12 00:00:00',127.79,1802,2250,64.82,'Medium' from dual union all
select 23497,'023497','1995-006A','Foton','Foton 34KS No. 10',65667,'Landed','1995-03-03 00:00:00','1995-02-24 00:00:00',90.31,217,363,62.81,'Intermediate' from dual union all
select 23498,'023498','1995-006B','11S510','11S510',65667,'Reentered','1995-03-12 00:00:00','1995-02-23 00:00:00',90.06,214,341,62.8,'Intermediate' from dual;
insert into SATELLITE
select 23499,'023499','1995-006C','Sensor cover','Sensor cover',65667,'Reentered','1995-02-18 00:00:00','1995-02-18 00:00:00',89.95,204,339,62.77,'Intermediate' from dual union all
select 235,'000235','61 Omicron 66','deb r/b Ablestar','',7273,'In Earth orbit','','1962-02-08 00:00:00',103.62,783,1077,66.98,'Intermediate' from dual union all
select 2350,'002350','1966-066B','Titan GT-10 Stage 2','LR91AJ5 GT-10 65-12565',13473,'Reentered','1966-07-20 00:00:00','1966-07-19 00:00:00',87.88,147,193,28.86,'Intermediate' from dual union all
select 23500,'023500','1995-007A','Endeavour (STS-67)','OV-105',65668,'Landed','1995-03-18 00:00:00','1995-03-09 00:00:00',91.55,343,358,28.47,'Intermediate' from dual union all
select 23501,'023501','1995-008A','Kosmos-2306','Romb',65669,'Reentered','2000-10-30 00:00:00','1995-04-01 00:00:00',94.45,466,518,65.85,'Intermediate' from dual union all
select 23502,'023502','1995-008B','S3M','S3M',65669,'Reentered','2000-06-18 00:00:00','1995-04-02 00:00:00',94.36,461,514,65.85,'Intermediate' from dual union all
select 23503,'023503','1995-006D','PTDU cover','PTDU cover',65667,'Reentered','1995-03-14 00:00:00','1995-03-03 00:00:00',92.5,313,482,62.83,'Intermediate' from dual union all
select 23504,'023504','1995-006E','PTDU cover','PTDU cover',65667,'Reentered','1995-04-01 00:00:00','1995-03-03 00:00:00',92.11,306,449,62.8,'Intermediate' from dual union all
select 23505,'023505','1995-006F','KBE part','KBE part',65667,'Reentered','1995-03-14 00:00:00','1995-03-03 00:00:00',90.06,215,339,62.8,'Intermediate' from dual union all
select 23506,'023506','1995-006G','KB electropitaniya','KBE',65667,'Reentered','1995-04-15 00:00:00','1995-03-03 00:00:00',90.24,218,355,62.8,'Intermediate' from dual union all
select 23507,'023507','1995-008C','Dispenser cover','Romb dispenser cover',65669,'Reentered','1997-10-11 00:00:00','1995-04-02 00:00:00',94.76,475,539,65.83,'Intermediate' from dual union all
select 23508,'023508','1995-008D','Dispenser cover','Romb dispenser cover',65669,'Reentered','1996-06-30 00:00:00','1995-04-02 00:00:00',94.06,436,510,65.81,'Intermediate' from dual union all
select 23509,'023509','1995-008E','Dispenser cover','Romb dispenser cover',65669,'Reentered','1996-04-24 00:00:00','1995-04-01 00:00:00',93.96,428,509,65.82,'Intermediate' from dual union all
select 2351,'002351','1966-067A','Kosmos-125','US-A',13474,'Reentered','1966-08-02 00:00:00','1966-07-22 00:00:00',89.07,200,257,64.99,'Intermediate' from dual union all
select 23510,'023510','1995-008F','Dispenser cover','Romb dispenser cover',65669,'Reentered','1996-04-17 00:00:00','1995-04-01 00:00:00',93.96,436,500,65.85,'Intermediate' from dual union all
select 23511,'023511','1995-009A','Kosmos-2307','Uragan No. 65L',65670,'In Earth orbit','','1995-04-07 00:00:00',675.69,19112,19146,64.78,'Medium' from dual union all
select 23512,'023512','1995-009B','Kosmos-2308','Uragan No. 66L',65670,'In Earth orbit','','1995-04-07 00:00:00',675.71,19100,19159,64.79,'Medium' from dual union all
select 23513,'023513','1995-009C','Kosmos-2309','Uragan No. 67L',65670,'In Earth orbit','','1995-04-06 00:00:00',675.69,19117,19141,64.79,'Medium' from dual union all
select 23514,'023514','1995-009D','8S812','8S812',65670,'Reentered','1995-03-11 00:00:00','1995-03-07 00:00:00',87.03,119,136,64.82,'Intermediate' from dual union all
select 23515,'023515','1995-009E','Perekhodnik','Sredniy perekhodnik',65670,'Reentered','1995-03-07 00:00:00','1995-03-07 00:00:00',87.15,125,141,64.8,'Intermediate' from dual union all
select 23516,'023516','1995-009F','Blok DM2 No. 76L','11S861 No. 76L',65670,'In Earth orbit','','1995-04-15 00:00:00',675.06,19080,19146,64.79,'Medium' from dual union all
select 23517,'023517','1995-009G','SOZ','SOZ',65670,'Reentered','2001-08-06 00:00:00','1995-04-08 00:00:00',340.23,395,19137,64.8,'Highly Ellip' from dual union all
select 23518,'023518','1995-009H','SOZ','SOZ',65670,'Reentered','2001-06-08 00:00:00','1995-04-07 00:00:00',340.19,392,19137,64.86,'Highly Ellip' from dual union all
select 23519,'023519','1995-010A','Soyuz TM-21','Soyuz 7K-STM No. 70',65671,'Landed','1995-09-11 00:00:00','1995-04-13 00:00:00',92.4,389,396,51.64,'Intermediate' from dual union all
select 2352,'002352','1966-059AL','deb Saturn S-IVB-203','',13466,'Reentered','1966-07-22 00:00:00','1966-07-20 00:00:00',89.33,206,277,31.99,'Intermediate' from dual union all
select 23520,'023520','1995-010B','11S510','11S510',65671,'Reentered','1995-03-16 00:00:00','1995-03-15 00:00:00',88.49,189,211,51.63,'Intermediate' from dual union all
select 23521,'023521','1995-011A','SFU','SFU',65672,'Landed Attached','1996-01-20 00:00:00','1995-04-17 00:00:00',94.22,470,492,28.46,'Intermediate' from dual union all
select 23522,'023522','1995-011B','Himawari-5','GMS 5',65672,'In Earth orbit','','1995-04-21 00:00:00',1436.04,35783,35787,1.02,'Stationary' from dual union all
select 23523,'023523','1995-011C','H-II Stage 2','H-II Stage 2',65672,'In Earth orbit','','1995-04-17 00:00:00',582.89,287,33186,25.55,'Highly Ellip' from dual union all
select 23524,'023524','1995-011D','GMS AKM','Star 27',65672,'In Earth orbit','','1995-04-19 00:00:00',1422.16,34610,36417,1.02,'Drift GEO' from dual union all
select 23525,'023525','1995-011E','5/4D Lower Fairing','GMS/SFU Adapter',65672,'Reentered','1995-04-09 00:00:00','1995-03-25 00:00:00',90.79,301,326,28.46,'Intermediate' from dual union all
select 23526,'023526','1995-012A','Kosmos-2310','Parus',65673,'In Earth orbit','','1995-04-21 00:00:00',105,979,1010,82.94,'Intermediate' from dual union all
select 23527,'023527','1995-012B','S3M','S3M',65673,'In Earth orbit','','1995-04-26 00:00:00',104.88,977,1000,82.93,'Intermediate' from dual union all
select 23528,'023528','1995-013A','INTELSAT 705','INTELSAT 705',65674,'In Earth orbit','','1995-04-24 00:00:00',1436.04,35619,35952,.08,'Stationary' from dual union all
select 23529,'023529','1995-013B','Centaur AC-115','Centaur IIA(1N) AC-115',65674,'Reentered','1999-02-06 00:00:00','1995-04-22 00:00:00',707.12,183,39645,26.64,'GEO Transfer' from dual union all
select 2353,'002353','1966-024C','NNS O-8 despin weight','was [UNID013] RCS 0.0',13431,'In Earth orbit','','1966-06-29 00:00:00',105.45,894,1137,89.38,'Polar' from dual union all
select 23530,'023530','1995-014A','Kosmos-2311','Kobal''t No. 547',65675,'Landed','1995-05-31 00:00:00','1995-04-21 00:00:00',90,176,373,67.14,'Intermediate' from dual union all
select 23531,'023531','1995-014B','11S510','11S510',65675,'Reentered','1995-03-27 00:00:00','1995-03-24 00:00:00',89.12,165,297,67.15,'Intermediate' from dual union all
select 23532,'023532','1994-017K','deb P90-5','',65413,'Reentered','1995-03-31 00:00:00','1994-03-31 00:00:00',95.65,550,550,105,'Retrograde' from dual union all
select 23533,'023533','1995-015A','DMSP F-13','DMSP Block 5D-2 S-13',65676,'In Earth orbit','','1995-04-23 00:00:00',101.92,845,855,98.83,'Sun-Synch' from dual union all
select 23534,'023534','1995-015B','DMSP F-13 solar array band','',65676,'Reentered','2015-10-07 00:00:00','1995-04-26 00:00:00',101.9,846,852,98.83,'Sun-Synch' from dual union all
select 23535,'023535','1995-015C','DMSP F-13 solar array band','',65676,'In Earth orbit','','1995-04-29 00:00:00',101.91,842,857,98.84,'Sun-Synch' from dual union all
select 23536,'023536','1995-016A','Brasilsat B2','Brasilsat B2',65677,'In Earth orbit','','1995-04-28 00:00:00',1436.09,35768,35804,.08,'Stationary' from dual union all
select 23537,'023537','1995-016B','Hot Bird 1','Eutelsat HB1',65677,'In Earth orbit','','1995-05-08 00:00:00',1435.93,35775,35792,.04,'Stationary' from dual union all
select 23538,'023538','1995-016C','Ariane H10+ V71','Ariane H10+ S/N T443',65677,'In Earth orbit','','1995-04-28 00:00:00',653.17,302,36816,6.75,'GEO Transfer' from dual union all
select 23539,'023539','1995-016D','Spelda 10 S/N UV21','Spelda 10',65677,'Reentered','1998-12-29 00:00:00','1995-04-29 00:00:00',624.62,231,35423,7.06,'GEO Transfer' from dual union all
select 2354,'002354','1966-066C','Gemini SC10 EM debris','',13473,'Reentered','1966-08-14 00:00:00','1966-07-23 00:00:00',91.26,290,383,28.85,'Intermediate' from dual union all
select 23540,'023540','1983-075BD','deb Kosmos-1484','',55832,'Reentered','1999-07-08 00:00:00','1995-03-29 00:00:00',98.53,520,857,97.05,'Sun-Synch' from dual union all
select 23541,'023541','1994-053J','deb Kosmos-2290','',65449,'Reentered','1995-05-23 00:00:00','1995-04-02 00:00:00',92.01,183,564,64.8,'Intermediate' from dual union all
select 23542,'023542','1994-053K','deb Kosmos-2290','',65449,'Reentered','1995-04-30 00:00:00','1995-04-02 00:00:00',91.94,183,557,64.81,'Intermediate' from dual union all
select 23543,'023543','1994-053L','deb Kosmos-2290','',65449,'Reentered','1995-04-21 00:00:00','1995-04-02 00:00:00',91.87,183,549,64.81,'Intermediate' from dual union all
select 23544,'023544','1994-053M','deb Kosmos-2290','',65449,'Reentered','1995-04-16 00:00:00','1995-04-02 00:00:00',91.87,180,553,64.8,'Intermediate' from dual union all
select 23545,'023545','1995-017A','Orbcomm F1','Orbcomm FM1',65678,'In Earth orbit','','1995-05-03 00:00:00',99.64,733,749,69.97,'Intermediate' from dual union all
select 23546,'023546','1995-017B','Orbcomm F2','Orbcomm FM2',65678,'In Earth orbit','','1995-05-04 00:00:00',99.62,733,748,69.99,'Intermediate' from dual union all
select 23547,'023547','1995-017C','OrbView-1','Microlab 1',65678,'In Earth orbit','','1995-05-03 00:00:00',99.62,733,748,69.98,'Intermediate' from dual union all
select 23548,'023548','1995-017D','Orion 38','Orion 38',65678,'In Earth orbit','','1995-05-03 00:00:00',99.65,735,749,69.99,'Intermediate' from dual union all
select 23549,'023549','1995-018A','''Ofeq-3','''Ofeq-3',65679,'Reentered','2000-10-24 00:00:00','1995-05-05 00:00:00',95.54,365,724,143.37,'Retrograde' from dual union all
select 2355,'002355','1966-066D','S-13 bracket','EVA debris',13473,'Reentered','1966-09-18 00:00:00','1966-08-18 00:00:00',91.94,365,375,28.86,'Intermediate' from dual union all
select 23550,'023550','1995-018B','AUS-51','AUS-51',65679,'Reentered','1995-08-25 00:00:00','1995-05-05 00:00:00',93.63,241,664,143.37,'Retrograde' from dual union all
select 23551,'023551','1983-075BE','deb Kosmos-1484','',55832,'Reentered','1997-10-21 00:00:00','1995-04-05 00:00:00',95.85,539,580,97.47,'Sun-Synch' from dual union all
select 23552,'023552','1994-017L','deb P90-5','',65413,'In Earth orbit','','1994-03-31 00:00:00',95.65,550,550,105,'Retrograde' from dual union all
select 23553,'023553','1995-019A','AMSC 1','AMSC 1',65680,'In Earth orbit','','1995-05-11 00:00:00',1435.63,35759,35796,.11,'Stationary' from dual union all
select 23554,'023554','1995-019B','Centaur AC-114','Centaur IIA(4N) AC-114',65680,'Reentered','2005-03-30 00:00:00','1995-05-08 00:00:00',711.32,229,39807,26.49,'GEO Transfer' from dual union all
select 23555,'023555','1995-020A','Progress M-27','Progress 7K-TGM No. 227',65681,'Deorbited','1995-05-23 00:00:00','1995-04-16 00:00:00',92.39,388,395,51.64,'Intermediate' from dual union all
select 23556,'023556','1995-020B','11S510','11S510',65681,'Reentered','1995-04-11 00:00:00','1995-04-10 00:00:00',88.46,185,212,51.65,'Intermediate' from dual union all
select 23557,'023557','1986-017JD','Garbage bag (KBO)','',59210,'Reentered','1996-07-08 00:00:00','1995-04-18 00:00:00',92.32,382,394,51.65,'Intermediate' from dual union all
select 23558,'023558','1986-017JE','GFZ-1','GFZ-1',59210,'Reentered','1999-06-23 00:00:00','1995-05-10 00:00:00',92.31,383,393,51.65,'Intermediate' from dual union all
select 23559,'023559','1986-017JF','GFZ-1 cover','',59210,'Reentered','1996-05-19 00:00:00','1995-04-20 00:00:00',92.32,384,393,51.64,'Intermediate' from dual union all
select 2356,'002356','1966-066E','ELSS chest pack','EVA debris',13473,'Reentered','1967-04-18 00:00:00','1966-11-07 00:00:00',92.25,363,407,28.85,'Intermediate' from dual union all
select 23560,'023560','1995-021A','ERS-2','ERS-2',65682,'In Earth orbit','','1995-05-21 00:00:00',100.53,783,785,98.55,'Sun-Synch' from dual union all
select 23561,'023561','1995-021B','Ariane H10+ V72','Ariane H10+ S/N T444',65682,'In Earth orbit','','1995-05-25 00:00:00',100.46,775,786,98.48,'Sun-Synch' from dual union all
select 23562,'023562','1967-048E','deb NNS O-13 insulation','',15303,'Reentered','2005-10-22 00:00:00','1995-04-21 00:00:00',106.72,1052,1097,89.57,'Polar' from dual union all
select 23563,'023563','1991-047E','deb Star 48','[UNID145]',64216,'Reentered','1999-11-01 00:00:00','1997-11-10 00:00:00',147.39,179,5551,37.59,'Medium' from dual union all
select 23564,'023564','1994-085V','deb Briz','',65481,'In Earth orbit','','1995-05-10 00:00:00',126.91,1818,2157,65.02,'Medium' from dual union all
select 23565,'023565','1986-017JG','EO-18 EVA-1 debris','',59210,'Reentered','1995-05-19 00:00:00','1995-05-13 00:00:00',92.19,381,383,51.65,'Intermediate' from dual union all
select 23566,'023566','1986-017JH','EO-18 EVA-1 debris','',59210,'Reentered','1996-11-17 00:00:00','1995-05-13 00:00:00',92.37,388,394,51.65,'Intermediate' from dual union all
select 23567,'023567','1995-022A','USA 110','ORION 3',65683,'In Earth orbit','','1995-07-01 00:00:00',1435.5,35750,35800,0,'Stationary' from dual union all
select 23568,'023568','1995-022B','Centaur TC-17','Centaur TC-17',65683,'In Earth orbit','','1995-07-01 00:00:00',1431.67,35600,35800,0,'Drift GEO' from dual union all
select 23569,'023569','1995-011F','IRTS Sun Shield','',65672,'Reentered','1995-07-12 00:00:00','1995-05-16 00:00:00',93.81,450,472,28.45,'Intermediate' from dual union all
select 2357,'002357','1966-066F','Gemini SC10 Equipment Module','Gemini SC10 Adapter Modu',13473,'Reentered','1966-10-15 00:00:00','1966-08-10 00:00:00',91.19,296,370,28.87,'Intermediate' from dual union all
select 23570,'023570','1995-011G','IRTS cover','',65672,'Reentered','1997-05-21 00:00:00','1995-05-14 00:00:00',94.12,462,490,28.46,'Intermediate' from dual union all
select 23571,'023571','1995-023A','INTELSAT 706','INTELSAT 706',65684,'In Earth orbit','','1995-06-25 00:00:00',1435.93,35725,35842,.01,'Stationary' from dual union all
select 23572,'023572','1995-023B','Ariane H10-3 V73','Ariane H10-3 S/N T445',65684,'Reentered','1995-11-13 00:00:00','1995-06-17 00:00:00',576.81,139,33012,7.23,'Highly Ellip' from dual union all
select 23573,'023573','1986-017JJ','EO-18 EVA-2 debris','',59210,'Reentered','1995-10-14 00:00:00','1995-05-18 00:00:00',92.48,394,399,51.64,'Intermediate' from dual union all
select 23574,'023574','1994-017M','deb P90-5','',65413,'Reentered','1998-08-13 00:00:00','1998-08-11 00:00:00',90.66,292,321,104.92,'Retrograde' from dual union all
select 23575,'023575','1994-085W','deb Briz','',65481,'Reentered','2000-01-05 00:00:00','1995-05-17 00:00:00',132.81,1461,3029,64.52,'Medium' from dual union all
select 23576,'023576','1994-017N','deb P90-5','',65413,'In Earth orbit','','1997-11-22 00:00:00',95.5,516,570,104.94,'Retrograde' from dual union all
select 23577,'023577','1994-017P','deb P90-5','',65413,'In Earth orbit','','1997-11-21 00:00:00',96.69,567,634,104.97,'Retrograde' from dual union all
select 23578,'023578','1994-085X','deb Briz','',65481,'Reentered','1999-05-01 00:00:00','1995-05-19 00:00:00',128.79,733,3407,64.72,'Medium' from dual union all
select 23579,'023579','1995-024A','Spektr','TsM-O No. 17301',65685,'Reentered Attached','2001-03-23 00:00:00','1995-06-19 00:00:00',92.47,392,400,51.65,'Intermediate' from dual union all
select 2358,'002358','1966-066G','Gemini SC10 EM debris','',13473,'Reentered','1966-08-12 00:00:00','1966-07-25 00:00:00',90.87,287,348,28.85,'Intermediate' from dual union all
select 23580,'023580','1995-024B','8S812','8S812',65685,'Reentered','1995-06-09 00:00:00','1995-05-27 00:00:00',89.32,206,275,51.67,'Intermediate' from dual union all
select 23581,'023581','1995-025A','GOES 9','GOES J',65686,'In Earth orbit','','1995-06-22 00:00:00',1436.09,35769,35804,.24,'Stationary' from dual union all
select 23582,'023582','1995-025B','Centaur AC-77','Centaur I AC-77',65686,'Reentered','1995-11-13 00:00:00','1995-06-22 00:00:00',721.86,147,40409,27.07,'GEO Transfer' from dual union all
select 23583,'023583','1995-020C','Progress part','Progress part',65681,'Reentered','1996-06-25 00:00:00','1995-05-23 00:00:00',92.49,393,401,51.65,'Intermediate' from dual union all
select 23584,'023584','1995-026A','Kosmos-2312','Oko',65687,'Reentered','2013-09-23 00:00:00','1995-06-23 00:00:00',717.9,653,39708,62.91,'Molniya' from dual union all
select 23585,'023585','1995-026B','11S510','11S510',65687,'Reentered','1995-08-08 00:00:00','1995-06-23 00:00:00',91.54,209,492,62.82,'Intermediate' from dual union all
select 23586,'023586','1995-026C','BOZ','BOZ',65687,'Reentered','1995-06-22 00:00:00','1995-05-31 00:00:00',91.94,182,558,62.87,'Intermediate' from dual union all
select 23587,'023587','1995-026D','Blok-2BL','Blok-2BL',65687,'In Earth orbit','','1995-06-23 00:00:00',708.17,745,39135,63.11,'Molniya' from dual union all
select 23588,'023588','1995-020D','Progress part','Progress part',65681,'Reentered','1997-03-19 00:00:00','1995-05-25 00:00:00',92.5,389,405,51.66,'Intermediate' from dual;
insert into SATELLITE
select 23589,'023589','1995-027A','UHF F/O F5','UHF F/O F5-EHF',65688,'In Earth orbit','','1995-07-01 00:00:00',1436.27,35060,36520,5.04,'Inclined GEO' from dual union all
select 2359,'002359','1963-014J','deb West Ford','RCS 0.16',9245,'In Earth orbit','','1966-03-30 00:00:00',166.4,3542,3744,87.32,'Medium' from dual union all
select 23590,'023590','1995-027B','Centaur AC-116','Centaur II(3) AC-116',65688,'In Earth orbit','','1995-07-01 00:00:00',459.76,275,26434,26.9,'Highly Ellip' from dual union all
select 23591,'023591','1995-014C','Kosmos-2311 part','Kobal''t AO part',65675,'Reentered','1995-06-01 00:00:00','1995-06-01 00:00:00',88.7,167,254,67.07,'Intermediate' from dual union all
select 23592,'023592','1995-014D','Kosmos-2311 part','Kobal''t AO part',65675,'Reentered','1995-06-01 00:00:00','1995-06-01 00:00:00',88.82,170,263,67.12,'Intermediate' from dual union all
select 23593,'023593','1995-011H','IRTS Sun Shield part','',65672,'Reentered','1995-11-21 00:00:00','1995-06-01 00:00:00',93.45,441,446,28.45,'Intermediate' from dual union all
select 23594,'023594','1995-015D','DMSP F-13 OLS Cover','',65676,'Reentered','2015-01-31 00:00:00','1995-06-03 00:00:00',101.91,847,852,98.83,'Sun-Synch' from dual union all
select 23595,'023595','1995-015E','DMSP F-13 OLS Cooler Cover','',65676,'In Earth orbit','','1995-06-03 00:00:00',101.9,842,855,98.83,'Sun-Synch' from dual union all
select 23596,'023596','1995-028A','Kosmos-2313','US-PM',65689,'Reentered','1997-07-11 00:00:00','1995-07-08 00:00:00',92.76,403,417,65.04,'Intermediate' from dual union all
select 23597,'023597','1995-028B','11S692','11S692',65689,'Reentered','1995-06-08 00:00:00','1995-06-08 00:00:00',89.38,113,375,65,'Intermediate' from dual union all
select 23598,'023598','1995-029A','DirecTV 3','DBS 3',65690,'In Earth orbit','','1995-07-15 00:00:00',1436.04,35762,35809,.08,'Stationary' from dual union all
select 23599,'023599','1995-029B','Ariane H10-3 V74','Ariane H10-3 S/N T446',65690,'Reentered','2008-01-13 00:00:00','1995-07-11 00:00:00',571.77,223,32660,7.27,'Highly Ellip' from dual union all
select 236,'000236','61 Omicron 67','deb r/b Ablestar','',7273,'Reentered','1969-05-13 00:00:00','1962-02-07 00:00:00',99.39,498,960,67.38,'Intermediate' from dual union all
select 2360,'002360','1963-014K','deb West Ford','RCS 0.10',9245,'Reentered','2007-06-27 00:00:00','1992-02-01 00:00:00',165.33,2947,4254,87.38,'Medium' from dual union all
select 23600,'023600','1995-030A','Atlantis (STS-71)','OV-104',65691,'Landed','1995-07-07 00:00:00','1995-06-29 00:00:00',92.36,382,399,51.65,'Intermediate' from dual union all
select 23601,'023601','1995-031A','Kosmos-2314','Kobal''t No. 546',65692,'Landed','1995-09-06 00:00:00','1995-07-03 00:00:00',89.68,168,350,67.13,'Intermediate' from dual union all
select 23602,'023602','1995-031B','11S510','11S510',65692,'Reentered','1995-07-04 00:00:00','1995-06-30 00:00:00',89.32,163,319,67.13,'Intermediate' from dual union all
select 23603,'023603','1995-032A','Kosmos-2315','Nadezhda 17F118',65693,'In Earth orbit','','1995-08-04 00:00:00',104.93,970,1012,82.9,'Intermediate' from dual union all
select 23604,'023604','1995-032B','S3M','S3M',65693,'In Earth orbit','','1995-08-04 00:00:00',104.82,968,1004,82.9,'Intermediate' from dual union all
select 23605,'023605','1995-033A','Helios 1A','Helios 1A',65694,'In Earth orbit','','1995-08-06 00:00:00',98.35,679,681,98.09,'Sun-Synch' from dual union all
select 23606,'023606','1995-033B','CERISE','CERISE',65694,'In Earth orbit','','1995-08-06 00:00:00',98.15,665,676,98.07,'Sun-Synch' from dual union all
select 23607,'023607','1995-033C','UPM/SAT 1','UPM/LBSAT',65694,'In Earth orbit','','1995-08-07 00:00:00',98.14,664,675,98.07,'Sun-Synch' from dual union all
select 23608,'023608','1995-033D','Ariane H10-3 V75','Ariane H10-3 S/N T447',65694,'In Earth orbit','','1995-08-06 00:00:00',98.09,646,689,98.05,'Sun-Synch' from dual union all
select 23609,'023609','1995-034A','USA 112','TRUMPET 2',65695,'In Earth orbit','','1999-07-11 00:00:00',717.78,1575,38780,63.41,'Molniya' from dual union all
select 2361,'002361','1963-014L','deb West Ford cluster','RCS 0.64',9245,'In Earth orbit','','1992-02-02 00:00:00',166.24,3188,4086,87.3,'Medium' from dual union all
select 23610,'023610','1995-034B','Centaur TC-8','Centaur TC-8',65695,'In Earth orbit','','1995-07-10 00:00:00',718.69,1350,39050,63.41,'Molniya' from dual union all
select 23611,'023611','1994-017Q','deb P90-5','',65413,'In Earth orbit','','1994-03-31 00:00:00',95.65,550,550,105,'Retrograde' from dual union all
select 23612,'023612','1995-035A','Discovery (STS-70)','OV-103',65696,'Landed','1995-07-22 00:00:00','1995-07-15 00:00:00',90.53,286,315,28.47,'Intermediate' from dual union all
select 23613,'023613','1995-035B','TDRS 7','TDRS G',65696,'In Earth orbit','','1995-08-12 00:00:00',1436.14,35784,35790,.03,'Stationary' from dual union all
select 23614,'023614','1995-035C','IUS-26 SRM-1','Orbus 21',65696,'In Earth orbit','','1995-08-12 00:00:00',632.36,301,35752,26.23,'GEO Transfer' from dual union all
select 23615,'023615','1995-035D','IUS-26 SRM-2','Orbus 6S',65696,'In Earth orbit','','1995-08-13 00:00:00',1432.89,35693,35755,2.07,'Drift GEO' from dual union all
select 23616,'023616','1992-047J','deb SOZ','',64759,'In Earth orbit','','1995-07-18 00:00:00',332.74,544,18511,64.85,'Highly Ellip' from dual union all
select 23617,'023617','1995-036A','Progress M-28','Progress 7K-TGM No. 228',65697,'Deorbited','1995-09-04 00:00:00','1995-07-27 00:00:00',92.46,393,397,51.65,'Intermediate' from dual union all
select 23618,'023618','1995-036B','11S510','11S510',65697,'Reentered','1995-07-23 00:00:00','1995-07-22 00:00:00',88,166,185,51.64,'Intermediate' from dual union all
select 23619,'023619','1988-077N','deb Transtage D-3','',61326,'In Earth orbit','','1995-08-01 00:00:00',702.53,400,39200,26.7,'GEO Transfer' from dual union all
select 2362,'002362','1963-014M','deb West Ford','RCS 0.16',9245,'In Earth orbit','','1992-01-24 00:00:00',165.13,2033,5151,86.51,'Medium' from dual union all
select 23620,'023620','1995-037A','Kosmos-2316','Uragan No. 80L',65698,'In Earth orbit','','1995-08-23 00:00:00',675.71,19082,19177,64.84,'Medium' from dual union all
select 23621,'023621','1995-037B','Kosmos-2317','Uragan No. 81L',65698,'In Earth orbit','','1995-08-24 00:00:00',675.71,19084,19175,64.87,'Medium' from dual union all
select 23622,'023622','1995-037C','Kosmos-2318','Uragan No. 85L',65698,'In Earth orbit','','1995-08-23 00:00:00',675.71,19036,19223,64.85,'Medium' from dual union all
select 23623,'023623','1995-037D','8S812','8S812',65698,'Reentered','1995-07-25 00:00:00','1995-07-25 00:00:00',87.13,127,138,64.85,'Intermediate' from dual union all
select 23624,'023624','1995-037E','Perekhodnik','Sredniy perekhodnik',65698,'Reentered','1995-07-25 00:00:00','1995-07-24 00:00:00',87.5,146,156,64.82,'Intermediate' from dual union all
select 23625,'023625','1995-037F','Blok DM2 No. 77L','11S861 No. 77L',65698,'In Earth orbit','','1995-08-23 00:00:00',674.7,19086,19122,64.85,'Medium' from dual union all
select 23626,'023626','1995-037G','deb Proton','',65698,'Reentered','1995-07-26 00:00:00','1995-07-25 00:00:00',90.94,155,487,64.86,'Intermediate' from dual union all
select 23627,'023627','1995-037H','deb Proton','',65698,'Reentered','1995-07-27 00:00:00','1995-07-27 00:00:00',89.1,102,358,64.98,'Intermediate' from dual union all
select 23628,'023628','1995-038A','USA 113','DSCS III B-7',65699,'In Earth orbit','','1995-09-01 00:00:00',1435.5,35750,35800,0,'Stationary' from dual union all
select 23629,'023629','1995-038B','Centaur AC-118','Centaur IIA(4) AC-118',65699,'In Earth orbit','','1995-07-31 00:00:00',629.97,228,35702,26.5,'GEO Transfer' from dual union all
select 2363,'002363','1963-014N','deb West Ford','RCS 0.15',9245,'Reentered','2000-07-23 00:00:00','1992-02-01 00:00:00',163.8,1263,5815,85.65,'Medium' from dual union all
select 23630,'023630','1995-037J','SOZ','SOZ',65698,'Reentered','2001-09-01 00:00:00','1995-08-23 00:00:00',340.15,413,19114,64.82,'Highly Ellip' from dual union all
select 23631,'023631','1995-037K','SOZ','SOZ',65698,'Reentered','2001-11-02 00:00:00','1995-08-23 00:00:00',340.17,413,19114,64.88,'Highly Ellip' from dual union all
select 23632,'023632','1995-039A','Interbol-1','SO-M2 No. 511',65700,'Deep Space','1995-08-03 00:00:00','1995-09-02 00:00:00',5459.55,1442,191214,63.28,'Deep Space' from dual union all
select 23633,'023633','1995-039B','11S510','11S510',65700,'Reentered','1996-04-14 00:00:00','1995-09-01 00:00:00',94.27,232,735,62.83,'Intermediate' from dual union all
select 23634,'023634','1995-039C','BOZ','BOZ',65700,'Reentered','1995-11-24 00:00:00','1995-09-01 00:00:00',93.96,205,732,62.8,'Intermediate' from dual union all
select 23635,'023635','1995-039D','Blok SO-L','Blok SO-L No. 71634-805',65700,'Deep Space','1995-08-03 00:00:00','1997-04-09 00:00:00',5498.19,17150,176474,70.89,'Deep Space' from dual union all
select 23636,'023636','1995-040A','PAS 4','Panamsat K3',65701,'In Earth orbit','','1995-09-21 00:00:00',1434.45,35726,35783,.11,'Drift GEO' from dual union all
select 23637,'023637','1995-040B','Ariane H10-3 V76','Ariane H10-3 S/N T448 (1',65701,'In Earth orbit','','1995-09-04 00:00:00',659.35,617,36815,4.26,'GEO Transfer' from dual union all
select 23638,'023638','1995-039E','Ullage rocket','Ullage rocket',65700,'Reentered','1995-08-21 00:00:00','1995-08-10 00:00:00',93.63,227,678,62.81,'Intermediate' from dual union all
select 23639,'023639','1995-041A','Mugunghwa 1','Koreasat 1',65702,'In Earth orbit','','1995-09-04 00:00:00',1436.01,35777,35793,.06,'Stationary' from dual union all
select 2364,'002364','1963-014P','deb West Ford cluster','RCS 0.60',9245,'In Earth orbit','','1992-02-01 00:00:00',166.2,3183,4088,87.43,'Medium' from dual union all
select 23640,'023640','1995-041B','Delta 228','Delta SSPS AJ10-118K',65702,'In Earth orbit','','1995-09-04 00:00:00',108.45,937,1374,26.68,'Intermediate' from dual union all
select 23641,'023641','1995-041C','Star 48B','Star 48B',65702,'In Earth orbit','','1995-09-05 00:00:00',532.52,1370,29400,20.53,'Highly Ellip' from dual union all
select 23642,'023642','1995-042A','Molniya-3','Molniya-3 No. 59',65703,'In Earth orbit','','1995-09-08 00:00:00',717.92,448,39914,62.86,'Molniya' from dual union all
select 23643,'023643','1995-042B','11S510','11S510',65703,'Reentered','1995-09-05 00:00:00','1995-08-16 00:00:00',90.2,204,365,62.82,'Intermediate' from dual union all
select 23644,'023644','1995-042C','BOZ','BOZ',65703,'Reentered','1995-08-25 00:00:00','1995-08-16 00:00:00',90.09,175,382,62.82,'Intermediate' from dual union all
select 23645,'023645','1995-042D','Blok-ML','Blok-ML',65703,'In Earth orbit','','1995-09-08 00:00:00',735.04,517,40686,62.84,'Molniya' from dual union all
select 23646,'023646','1995-039F','Magion-4','Magion-4',65700,'Deep Space','1995-08-03 00:00:00','1996-10-31 00:00:00',5469.2,14776,178121,71.39,'Deep Space' from dual union all
select 23647,'023647','1981-038C','deb USAF satellite','',52561,'In Earth orbit','','1995-04-01 00:00:00',690.49,1000,38000,63.3,'Molniya' from dual union all
select 23648,'023648','1995-038C','IABS-5','IABS',65699,'In Earth orbit','','1995-08-05 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 23649,'023649','1995-043A','JCSAT 3','JCSAT 3',65704,'In Earth orbit','','1995-09-30 00:00:00',1435.96,35740,35828,.06,'Stationary' from dual union all
select 2365,'002365','1963-014Q','deb West Ford','RCS 0.26',9245,'In Earth orbit','','1992-02-02 00:00:00',166.25,3367,3908,87.27,'Medium' from dual union all
select 23650,'023650','1995-017E','deb Orbcomm FM1','',65678,'Reentered','2000-08-19 00:00:00','1995-09-05 00:00:00',99.58,729,747,69.99,'Intermediate' from dual union all
select 23651,'023651','1995-044A','N-STAR a','N-STAR a',65705,'In Earth orbit','','1995-10-04 00:00:00',1436.16,35780,35795,.02,'Stationary' from dual union all
select 23652,'023652','1995-044B','Ariane H10-3 V77','Ariane H10-3 S/N T449',65705,'Reentered','1996-01-13 00:00:00','1995-10-07 00:00:00',624.06,129,35496,6.65,'GEO Transfer' from dual union all
select 23653,'023653','1995-045A','Kosmos-2319','Geizer No. 20L',65706,'In Earth orbit','','1995-09-29 00:00:00',1436.04,35745,35826,1.55,'Stationary' from dual union all
select 23654,'023654','1995-045B','8S812','8S812',65706,'Reentered','1995-09-03 00:00:00','1995-09-01 00:00:00',88.12,179,184,51.64,'Intermediate' from dual union all
select 23655,'023655','1995-045C','Perekhodnik','Sredniy perekhodnik',65706,'Reentered','1995-08-31 00:00:00','1995-08-30 00:00:00',88.48,193,205,51.65,'Intermediate' from dual union all
select 23656,'023656','1995-045D','Blok DM2 No. 78L','11S861 No. 78L',65706,'In Earth orbit','','1995-10-12 00:00:00',1438.23,35746,35910,1.47,'Drift GEO' from dual union all
select 23657,'023657','1995-046A','Sich-1','Okean-O1 No. 8/NKhM 10',65707,'In Earth orbit','','1995-10-01 00:00:00',97.71,630,669,82.53,'Intermediate' from dual union all
select 23658,'023658','1995-043B','Centaur AC-117','Centaur IIA(1N) AC-117',65704,'Reentered','1996-02-12 00:00:00','1995-11-29 00:00:00',1639.6,219,79144,23.03,'Very High' from dual union all
select 23659,'023659','1995-046C','S5M','S5M',65707,'In Earth orbit','','1995-09-30 00:00:00',97.7,630,667,82.53,'Intermediate' from dual union all
select 2366,'002366','1963-014R','deb West Ford','RCS 0.11',9245,'In Earth orbit','','1992-01-29 00:00:00',166.25,3483,3792,87.33,'Medium' from dual union all
select 23660,'023660','1995-045E','SOZ','SOZ',65706,'Reentered','1996-06-06 00:00:00','1995-09-29 00:00:00',634.17,259,35887,47.41,'GEO Transfer' from dual union all
select 23661,'023661','1995-045F','SOZ','SOZ',65706,'Reentered','2002-05-08 00:00:00','1995-09-30 00:00:00',635.41,317,35893,47.45,'GEO Transfer' from dual union all
select 23662,'023662','1986-017JK','Garbage bag','',59210,'Reentered','1996-02-07 00:00:00','1995-08-31 00:00:00',92.37,390,392,51.65,'Intermediate' from dual union all
select 23663,'023663','1995-045G','deb Proton','',65706,'Reentered','1996-09-07 00:00:00','1995-10-06 00:00:00',90.35,184,399,51.6,'Intermediate' from dual union all
select 23664,'023664','1995-045H','deb Proton  (error)','',65706,'Reentered','1995-09-02 00:00:00','1995-09-01 00:00:00',92,193,553,51.69,'Intermediate' from dual union all
select 23665,'023665','1995-047A','Soyuz TM-22','Soyuz 7K-STM No. 71',65708,'Landed','1996-02-29 00:00:00','1995-10-03 00:00:00',92.43,392,396,51.65,'Intermediate' from dual union all
select 23666,'023666','1995-047B','11S510','11S510',65708,'Reentered','1995-09-06 00:00:00','1995-09-05 00:00:00',88.11,178,184,51.63,'Intermediate' from dual union all
select 23667,'023667','1995-048A','Endeavour (STS-69)','OV-105',65709,'Landed','1995-09-18 00:00:00','1995-09-09 00:00:00',92,369,377,28.47,'Intermediate' from dual union all
select 23668,'023668','1995-048B','Spartan 201','Spartan 201',65709,'Landed Attached','1995-09-18 00:00:00','1995-09-09 00:00:00',92.05,369,382,28.47,'Intermediate' from dual union all
select 23669,'023669','1995-048C','Wake Shield Facility','WSF',65709,'Landed Attached','1995-09-18 00:00:00','1995-09-11 00:00:00',92.56,396,404,28.47,'Intermediate' from dual union all
select 2367,'002367','1963-014S','deb West Ford','RCS 0.16',9245,'In Earth orbit','','1992-02-01 00:00:00',164.26,1549,5566,86.02,'Medium' from dual union all
select 23670,'023670','1995-049A','Telstar 402R','Telstar 402R',65710,'In Earth orbit','','1995-10-24 00:00:00',1436.06,35758,35813,.16,'Stationary' from dual union all
select 23671,'023671','1995-049B','Ariane H10-3 V78','Ariane H10-3 S/N T450',65710,'In Earth orbit','','1995-10-24 00:00:00',656.16,399,36871,7.13,'GEO Transfer' from dual union all
select 23672,'023672','1995-050A','Resurs-F2','Resurs-F2 17F42 No. 10',65711,'Landed','1995-10-26 00:00:00','1995-10-03 00:00:00',89.13,228,235,82.32,'Intermediate' from dual union all
select 23673,'023673','1995-050B','11S510','11S510',65711,'Reentered','1995-09-29 00:00:00','1995-09-28 00:00:00',88.25,169,207,82.32,'Intermediate' from dual union all
select 23674,'023674','1995-051A','Kosmos-2320','Neman',65712,'Deorbited','1996-09-28 00:00:00','1995-10-29 00:00:00',89.75,238,287,64.91,'Intermediate' from dual union all
select 23675,'023675','1995-051B','11S510','11S510',65712,'Reentered','1995-10-03 00:00:00','1995-10-01 00:00:00',88.54,170,235,64.93,'Intermediate' from dual union all
select 23676,'023676','1995-052A','Kosmos-2321','Parus',65713,'Reentered','1997-08-21 00:00:00','1995-11-05 00:00:00',94.95,258,775,82.94,'Intermediate' from dual union all
select 23677,'023677','1995-052B','S3M','S3M',65713,'Reentered','1997-03-13 00:00:00','1995-11-05 00:00:00',94.81,249,770,82.94,'Intermediate' from dual union all
select 23678,'023678','1995-053A','Progress M-29','Progress 7K-TGM No. 229',65714,'Deorbited','1995-12-19 00:00:00','1995-11-07 00:00:00',92.41,391,395,51.64,'Intermediate' from dual;
insert into SATELLITE
select 23679,'023679','1995-053B','11S510','11S510',65714,'Reentered','1995-10-10 00:00:00','1995-10-09 00:00:00',88.49,188,211,51.68,'Intermediate' from dual union all
select 2368,'002368','1966-068A','Kosmos-126','Zenit-4 No. 21',13475,'Landed','1966-08-06 00:00:00','1966-08-05 00:00:00',89.96,207,338,51.77,'Intermediate' from dual union all
select 23680,'023680','1995-054A','Luch-1','Gelios No. 11L',65715,'In Earth orbit','','1995-11-10 00:00:00',1436.01,35757,35813,3.02,'Inclined GEO' from dual union all
select 23681,'023681','1995-054B','8S812','8S812',65715,'Reentered','1995-10-14 00:00:00','1995-10-13 00:00:00',87.6,153,159,51.65,'Intermediate' from dual union all
select 23682,'023682','1995-054C','Perekhodnik','Sredniy perekhodnik',65715,'Reentered','1996-10-11 00:00:00','1995-10-12 00:00:00',87.6,155,157,51.6,'Intermediate' from dual union all
select 23683,'023683','1995-054D','Blok DM2 No.79L','11S861 No. 79L',65715,'In Earth orbit','','1995-11-27 00:00:00',1436.78,35729,35871,2.99,'Drift GEO' from dual union all
select 23684,'023684','1995-054E','SOZ','SOZ',65715,'Reentered','1996-05-20 00:00:00','1995-11-12 00:00:00',634.46,251,35910,47.39,'GEO Transfer' from dual union all
select 23685,'023685','1995-054F','SOZ','SOZ',65715,'Reentered','1996-08-02 00:00:00','1995-11-12 00:00:00',635.73,305,35921,47.47,'GEO Transfer' from dual union all
select 23686,'023686','1995-055A','Astra 1E','Astra 1E',65716,'In Earth orbit','','1995-12-06 00:00:00',1436.09,35780,35793,.01,'Stationary' from dual union all
select 23687,'023687','1995-055B','Ariane H10-3 V79','Ariane H10-3 S/N T451',65716,'In Earth orbit','','1995-11-18 00:00:00',653.96,476,36682,4.66,'GEO Transfer' from dual union all
select 23688,'023688','1995-056A','Columbia (STS-73)','OV-102',65717,'Landed','1995-11-05 00:00:00','1995-10-27 00:00:00',89.86,262,273,39.01,'Intermediate' from dual union all
select 23689,'023689','1986-017JL','EO-20 EVA-1 debris','',59210,'Reentered','1996-10-11 00:00:00','1995-10-21 00:00:00',92.43,391,397,51.64,'Intermediate' from dual union all
select 2369,'002369','1966-068B','11A57I','11A57I',13475,'Reentered','1966-08-10 00:00:00','1966-07-30 00:00:00',89.82,203,329,51.79,'Intermediate' from dual union all
select 23690,'023690','1986-017JM','EO-20 EVA-1 debris','(long lived)',59210,'Reentered','1999-06-23 00:00:00','1995-10-21 00:00:00',92.45,392,398,51.65,'Intermediate' from dual union all
select 23691,'023691','1986-017JN','EO-20 EVA-1 debris','',59210,'Reentered','1997-11-12 00:00:00','1995-10-21 00:00:00',92.46,392,399,51.64,'Intermediate' from dual union all
select 23692,'023692','1986-017JP','EO-20 EVA-1 debris','',59210,'Reentered','1997-03-14 00:00:00','1995-10-21 00:00:00',92.46,391,400,51.65,'Intermediate' from dual union all
select 23693,'023693','1986-017JQ','EO-20 EVA-1 debris','',59210,'Reentered','1997-12-20 00:00:00','1995-10-21 00:00:00',92.47,393,399,51.65,'Intermediate' from dual union all
select 23694,'023694','1986-017JR','EO-20 EVA-1 debris','',59210,'Reentered','1997-04-02 00:00:00','1995-10-21 00:00:00',92.5,393,402,51.65,'Intermediate' from dual union all
select 23695,'023695','1986-017JS','EO-20 EVA-1 debris','',59210,'Reentered','1997-01-26 00:00:00','1995-10-21 00:00:00',92.51,393,402,51.65,'Intermediate' from dual union all
select 23696,'023696','1995-057A','UHF F/O F6','UFO F6-EHF',65718,'In Earth orbit','','1995-11-22 00:00:00',1435.96,34979,36589,5.06,'Inclined GEO' from dual union all
select 23697,'023697','1995-057B','Centaur AC-119','Centaur II(3) AC-119',65718,'In Earth orbit','','1995-11-21 00:00:00',477.79,267,27467,26.97,'Highly Ellip' from dual union all
select 23698,'023698','1986-017JT','EO-20 EVA-1 debris','',59210,'Reentered','1996-01-26 00:00:00','1995-10-24 00:00:00',92.36,389,392,51.65,'Intermediate' from dual union all
select 23699,'023699','1995-050C','Solnechnaya Batareya','Solar panel module',65711,'Reentered','1995-11-02 00:00:00','1995-10-26 00:00:00',89.76,253,272,82.32,'Intermediate' from dual union all
select 237,'000237','61 Omicron 68','deb r/b Ablestar','',7273,'Reentered','1969-04-12 00:00:00','1968-11-11 00:00:00',95.26,430,633,67.3,'Intermediate' from dual union all
select 2370,'002370','1966-068C','deb Kosmos-126','',13475,'Reentered','1966-07-30 00:00:00','1966-07-28 00:00:00',89.56,205,301,51.8,'Intermediate' from dual union all
select 23700,'023700','1995-050D','Blenda','Sunshade',65711,'Reentered','1995-10-30 00:00:00','1995-10-26 00:00:00',89.77,253,274,82.32,'Intermediate' from dual union all
select 23701,'023701','1995-050E','PTDU cover','PTDU cover',65711,'Reentered','1995-11-07 00:00:00','1995-10-26 00:00:00',92.17,270,492,82.31,'Intermediate' from dual union all
select 23702,'023702','1995-050F','PTDU cover','PTDU cover',65711,'Reentered','1995-11-10 00:00:00','1995-10-26 00:00:00',91.67,270,443,82.31,'Intermediate' from dual union all
select 23703,'023703','1986-017JU','EO-20 EVA-1 debris','',59210,'Reentered','1997-12-11 00:00:00','1995-10-31 00:00:00',92.45,390,399,51.65,'Intermediate' from dual union all
select 23704,'023704','1995-058A','Kosmos-2322','Tselina-2',65719,'In Earth orbit','','1995-11-30 00:00:00',101.92,848,852,71.02,'Intermediate' from dual union all
select 23705,'023705','1995-058B','11S772','11S772',65719,'In Earth orbit','','1995-11-30 00:00:00',101.81,837,851,71.01,'Intermediate' from dual union all
select 23706,'023706','1995-058C','Sep Motor Cover','Sep Motor Cover',65719,'In Earth orbit','','1995-11-30 00:00:00',104.96,844,1142,71,'Intermediate' from dual union all
select 23707,'023707','1995-058D','Sep Motor Cover','Sep Motor Cover',65719,'In Earth orbit','','1995-11-30 00:00:00',105.08,847,1150,71.01,'Intermediate' from dual union all
select 23708,'023708','1995-058E','Sep Motor Cover','Sep Motor Cover',65719,'In Earth orbit','','1995-11-30 00:00:00',104.77,846,1121,71.07,'Intermediate' from dual union all
select 23709,'023709','1995-058F','Sep Motor Cover','Sep Motor Cover',65719,'In Earth orbit','','1995-11-30 00:00:00',104.93,844,1139,71.04,'Intermediate' from dual union all
select 2371,'002371','61 A Delta 6','West Ford package fragment','',7251,'In Earth orbit','','1965-11-30 00:00:00',165.64,3228,3998,95.84,'Medium' from dual union all
select 23710,'023710','1995-059A','Radarsat','Radarsat',65720,'In Earth orbit','','1995-12-04 00:00:00',100.67,789,792,98.58,'Sun-Synch' from dual union all
select 23711,'023711','1995-059B','Delta 229','Delta SSPS AJ10-118K',65720,'In Earth orbit','','1995-12-04 00:00:00',109.72,934,1493,100.64,'Sun-Synch' from dual union all
select 23712,'023712','1995-060A','Milstar 2','Milstar DFS-2',65721,'In Earth orbit','','1996-07-18 00:00:00',1436.09,35778,35795,3.53,'Inclined GEO' from dual union all
select 23713,'023713','1995-060B','Centaur TC-13','Centaur TC-13',65721,'In Earth orbit','','1995-11-14 00:00:00',1434.81,35733,35790,9.85,'Inclined Drift' from dual union all
select 23714,'023714','1995-061A','Atlantis (STS-74)','OV-104',65722,'Landed','1995-11-20 00:00:00','1995-11-14 00:00:00',91.93,345,394,51.64,'Intermediate' from dual union all
select 23715,'023715','1995-062A','ISO','ISO',65723,'In Earth orbit','','1995-12-17 00:00:00',1437.13,1110,70504,5.06,'Near-synch' from dual union all
select 23716,'023716','1995-062B','Ariane H10-3 V80','Ariane H10-3 S/N T452',65723,'In Earth orbit','','1997-03-14 00:00:00',1516,927,73746,.46,'Very High' from dual union all
select 23717,'023717','1995-063A','Gals 2','Gals No. 12L',65724,'In Earth orbit','','1995-12-17 00:00:00',1436.27,35782,35798,.15,'Stationary' from dual union all
select 23718,'023718','1995-063B','8S812','8S812',65724,'Reentered','1995-11-21 00:00:00','1995-11-19 00:00:00',88.11,179,183,51.62,'Intermediate' from dual union all
select 23719,'023719','1995-063C','Perekhodnik','Sredniy perekhodnik',65724,'Reentered','1995-11-18 00:00:00','1995-11-17 00:00:00',88.37,187,200,51.62,'Intermediate' from dual union all
select 2372,'002372','1963-014T','deb West Ford','RCS 0.24',9245,'In Earth orbit','','1992-02-02 00:00:00',166.44,3407,3883,87.35,'Medium' from dual union all
select 23720,'023720','1995-063D','Blok DM2 No. 94L','11S861 No. 94L',65724,'In Earth orbit','','1995-12-29 00:00:00',1437.75,35662,35976,.14,'Drift GEO' from dual union all
select 23721,'023721','1995-063E','SOZ','SOZ',65724,'Reentered','1996-06-13 00:00:00','1995-12-17 00:00:00',633.82,241,35887,47.34,'GEO Transfer' from dual union all
select 23722,'023722','1995-063F','SOZ','SOZ',65724,'Reentered','1997-12-18 00:00:00','1995-12-19 00:00:00',635.12,298,35897,47.35,'GEO Transfer' from dual union all
select 23723,'023723','1995-064A','Asiasat 2','Asiasat 2',65725,'In Earth orbit','','1996-01-18 00:00:00',1436.09,35779,35794,.19,'Stationary' from dual union all
select 23724,'023724','1995-064B','CZ-2E Stage 2','CZ-2E Stage 2',65725,'Reentered','1995-12-05 00:00:00','1995-11-30 00:00:00',89.14,184,280,28.01,'Intermediate' from dual union all
select 23725,'023725','1995-064C','EPKM','FG-46',65725,'In Earth orbit','','1995-12-28 00:00:00',615.63,223,34965,25.53,'GEO Transfer' from dual union all
select 23726,'023726','1995-065A','SOHO','SOHO',65726,'Deep Space','1995-12-02 00:00:00','1995-12-02 00:00:00',70318.62,177,1115746,29.72,'Cislunar' from dual union all
select 23727,'023727','1995-065B','Centaur AC-121','Centaur IIA(4N) AC-121',65726,'Deep Space','1995-12-02 00:00:00','1995-12-02 00:00:00',87081.21,201,1288630,28.12,'Cislunar' from dual union all
select 23728,'023728','1995-066A','USA 116','Enhanced CRYSTAL 2105',65727,'Deorbited','2008-11-18 00:00:00','1996-04-16 00:00:00',98.99,409,1012,97.75,'Sun-Synch' from dual union all
select 23729,'023729','1995-066B','Titan 45K-15 Stage 2','LR91AJ11A 45K-15',65727,'Reentered','2000-01-01 00:00:00','1995-12-05 00:00:00',97.42,265,1005,97.8,'Sun-Synch' from dual union all
select 2373,'002373','1963-014U','deb West Ford large cluster','RCS 0.24',9245,'Reentered','1993-06-23 00:00:00','1992-02-01 00:00:00',162.11,837,6104,84.92,'Medium' from dual union all
select 23730,'023730','1995-067A','Telecom 2C','Telecom 2C',65728,'In Earth orbit','','1996-01-09 00:00:00',1436.01,35777,35793,.1,'Stationary' from dual union all
select 23731,'023731','1995-067B','Insat 2C','Insat 2C',65728,'In Earth orbit','','1996-01-24 00:00:00',1435.78,35756,35805,.02,'Stationary' from dual union all
select 23732,'023732','1995-067C','Ariane H10-3 V81','Ariane H10-3 S/N T453',65728,'In Earth orbit','','1996-01-12 00:00:00',654.82,276,36926,6.67,'GEO Transfer' from dual union all
select 23733,'023733','1995-067D','Spelda 30 S/N UV26','Mini-Spelda',65728,'Reentered','2002-01-13 00:00:00','1996-01-12 00:00:00',625.11,222,35456,6.86,'GEO Transfer' from dual union all
select 23734,'023734','1995-068A','Kosmos-2323','Uragan No. 82L',65729,'In Earth orbit','','1996-01-14 00:00:00',675.71,19083,19176,64.84,'Medium' from dual union all
select 23735,'023735','1995-068B','Kosmos-2324','Uragan No. 78L',65729,'In Earth orbit','','1996-01-17 00:00:00',675.65,19114,19142,64.83,'Medium' from dual union all
select 23736,'023736','1995-068C','Kosmos-2325','Uragan No. 76L',65729,'In Earth orbit','','1996-01-13 00:00:00',675.71,19113,19146,64.83,'Medium' from dual union all
select 23737,'023737','1995-068D','8S812','8S812',65729,'Reentered','1995-12-15 00:00:00','1995-12-14 00:00:00',87.59,151,159,64.82,'Intermediate' from dual union all
select 23738,'023738','1995-068E','Perekhodnik','Sredniy perekhodnik',65729,'Reentered','1995-12-14 00:00:00','1995-12-14 00:00:00',87.68,141,179,64.81,'Intermediate' from dual union all
select 23739,'023739','1995-068F','Blok DM2 No. 80L','11S861 No. 80L',65729,'In Earth orbit','','1996-01-13 00:00:00',674.84,19085,19130,64.83,'Medium' from dual union all
select 2374,'002374','1963-014V','deb West Ford','RCS 0.14',9245,'Reentered','2002-07-18 00:00:00','1992-02-01 00:00:00',163.8,1230,5847,85.54,'Medium' from dual union all
select 23740,'023740','1995-068G','SOZ','SOZ',65729,'Reentered','2001-11-23 00:00:00','1996-01-14 00:00:00',340.09,403,19120,64.83,'Highly Ellip' from dual union all
select 23741,'023741','1995-069A','Galaxy 3R','Galaxy 3R',65730,'In Earth orbit','','1996-01-15 00:00:00',1436.16,35763,35813,.06,'Stationary' from dual union all
select 23742,'023742','1995-069B','Centaur AC-120','Centaur IIA(4N) AC-120',65730,'Reentered','2002-03-16 00:00:00','1996-01-18 00:00:00',591.48,196,33730,26.86,'Highly Ellip' from dual union all
select 23743,'023743','1995-068H','SOZ','SOZ',65729,'Reentered','2001-10-13 00:00:00','1996-01-14 00:00:00',340.09,408,19115,64.88,'Highly Ellip' from dual union all
select 23744,'023744','1995-070A','Progress M-30','Progress 7K-TGM No. 230',65731,'Deorbited','1996-02-22 00:00:00','1996-01-18 00:00:00',92.42,390,397,51.64,'Intermediate' from dual union all
select 23745,'023745','1995-070B','11S510','11S510',65731,'Reentered','1995-12-19 00:00:00','1995-12-18 00:00:00',88.44,187,208,51.64,'Intermediate' from dual union all
select 23746,'023746','1994-088C','Kosmos-2305 part','Neman AO part',65484,'Reentered','1995-12-22 00:00:00','1995-12-19 00:00:00',89.11,168,293,64.89,'Intermediate' from dual union all
select 23747,'023747','1994-088D','Kosmos-2305 part','Neman AO part',65484,'Reentered','1995-12-22 00:00:00','1995-12-19 00:00:00',88.27,155,223,64.92,'Intermediate' from dual union all
select 23748,'023748','1995-071A','Kosmos-2326','US-PU/Konus-A',65732,'Deorbited','1997-11-08 00:00:00','1996-01-19 00:00:00',92.76,406,414,65.02,'Intermediate' from dual union all
select 23749,'023749','1995-059C','deb Delta','',65720,'Reentered','2002-04-01 00:00:00','1996-02-05 00:00:00',109.69,932,1492,100.64,'Sun-Synch' from dual union all
select 2375,'002375','1963-014W','deb West Ford','RCS 0.13',9245,'In Earth orbit','','1992-02-03 00:00:00',165.09,2016,5165,86.46,'Medium' from dual union all
select 23750,'023750','1995-071B','11S692','11S692',65732,'Reentered','1995-12-20 00:00:00','1995-12-20 00:00:00',89.56,112,394,65,'Intermediate' from dual union all
select 23751,'023751','1995-072A','IRS-1C','IRS-1C',65733,'In Earth orbit','','1996-01-28 00:00:00',101.28,819,820,98.71,'Sun-Synch' from dual union all
select 23752,'023752','1995-072B','Skipper','Skipper',65733,'In Earth orbit','','1996-01-30 00:00:00',101.05,804,813,98.56,'Sun-Synch' from dual union all
select 23753,'023753','1995-072C','Blok-2BL','Blok-2BL',65733,'In Earth orbit','','1996-01-27 00:00:00',101.04,805,810,98.6,'Sun-Synch' from dual union all
select 23754,'023754','1995-073A','Echostar I','Echostar 1',65734,'In Earth orbit','','1996-01-29 00:00:00',1436.09,35764,35809,.09,'Stationary' from dual union all
select 23755,'023755','1995-073B','CZ-2E Stage 2','CZ-2E Stage 2',65734,'Reentered','1996-01-06 00:00:00','1995-12-30 00:00:00',89.2,189,280,28.01,'Intermediate' from dual union all
select 23756,'023756','1995-073C','EPKM','FG-46',65734,'In Earth orbit','','1996-02-08 00:00:00',616.94,215,35041,24.22,'GEO Transfer' from dual union all
select 23757,'023757','1995-074A','Rossi X-ray Timing Explorer','XTE',65735,'Reentered','2018-04-30 00:00:00','1996-01-29 00:00:00',96.13,564,583,22.98,'Equatorial' from dual union all
select 23758,'023758','1995-074B','Delta 230','Delta SSPS AJ10-118K',65735,'Reentered','1996-01-23 00:00:00','1996-01-07 00:00:00',91.55,174,528,24.95,'Equatorial' from dual union all
select 23759,'023759','1995-074C','deb XTE','',65735,'Reentered','1996-01-19 00:00:00','1996-01-17 00:00:00',92.55,387,412,22.82,'Equatorial' from dual union all
select 2376,'002376','1966-069A','OPS 3014','GAMBIT-3 Mission 4301',13476,'Reentered Attached','1966-08-06 00:00:00','1966-07-31 00:00:00',88.52,158,245,94.12,'Polar' from dual union all
select 23760,'023760','1995-074D','deb XTE','',65735,'Reentered','1996-01-05 00:00:00','1996-01-01 00:00:00',94.96,473,561,22.93,'Equatorial' from dual union all
select 23761,'023761','1995-074E','deb XTE','',65735,'Reentered','1996-01-05 00:00:00','1995-12-31 00:00:00',95.52,463,625,23.1,'Equatorial' from dual union all
select 23762,'023762','1996-001A','Endeavour (STS-72)','OV-105',65889,'Landed','1996-01-20 00:00:00','1996-01-13 00:00:00',93.82,458,465,28.45,'Intermediate' from dual union all
select 23763,'023763','1996-001B','OAST-Flyer','Spartan 206',65889,'Landed Attached','1996-01-20 00:00:00','1996-01-14 00:00:00',90.66,303,311,28.45,'Intermediate' from dual union all
select 23764,'023764','1996-002A','PAS 3R','Panamsat K4',65890,'In Earth orbit','','1996-02-11 00:00:00',1436.01,35774,35796,.07,'Stationary' from dual union all
select 23765,'023765','1996-002B','Measat 1','HS376 Measat 1',65890,'In Earth orbit','','1996-02-13 00:00:00',1436.06,35769,35802,.19,'Stationary' from dual union all
select 23766,'023766','1996-002C','Ariane H10-3 V82','Ariane H10-3 S/N T454',65890,'In Earth orbit','','1996-02-11 00:00:00',653.68,252,36892,6.86,'GEO Transfer' from dual union all
select 23767,'023767','1996-002D','Spelda 30 S/N UV27','Spelda 30',65890,'Reentered','2002-04-04 00:00:00','1996-02-13 00:00:00',625.55,221,35481,6.89,'GEO Transfer' from dual union all
select 23768,'023768','1996-003A','Mugunghwa 2','Koreasat 2',65891,'In Earth orbit','','1996-02-25 00:00:00',1436.27,35786,35794,.01,'Stationary' from dual;
insert into SATELLITE
select 23769,'023769','1996-003B','Delta 231','Delta SSPS AJ10-118K',65891,'Reentered','2014-10-30 00:00:00','1996-02-13 00:00:00',103.53,384,1467,25.32,'Intermediate' from dual union all
select 2377,'002377','1963-014X','deb West Ford','RCS 0.18',9245,'In Earth orbit','','1992-02-02 00:00:00',166,3038,4216,87.23,'Medium' from dual union all
select 23770,'023770','1996-003C','Star 48B','Star 48B',65891,'In Earth orbit','','1996-02-14 00:00:00',655.25,1385,35839,21.09,'GEO Transfer' from dual union all
select 23771,'023771','1995-011J','SFU solar panel 1','',65672,'Reentered','1998-10-18 00:00:00','1996-01-14 00:00:00',94.07,466,481,28.45,'Intermediate' from dual union all
select 23772,'023772','1995-011K','SFU solar panel 2','',65672,'Reentered','1998-11-28 00:00:00','1996-01-14 00:00:00',133.67,521,4043,46.72,'Medium' from dual union all
select 23773,'023773','1996-004A','Kosmos-2327','Parus',65892,'In Earth orbit','','1996-02-15 00:00:00',104.8,951,1020,82.98,'Intermediate' from dual union all
select 23774,'023774','1996-004B','S3M','S3M',65892,'In Earth orbit','','1996-02-16 00:00:00',104.68,951,1007,82.97,'Intermediate' from dual union all
select 23775,'023775','1996-005A','Gorizont','Gorizont No. 43L',65893,'In Earth orbit','','1996-02-25 00:00:00',1436.19,35761,35816,1.45,'Stationary' from dual union all
select 23776,'023776','1996-005B','8S812','8S812',65893,'Reentered','1996-01-26 00:00:00','1996-01-25 00:00:00',88.29,186,194,51.63,'Intermediate' from dual union all
select 23777,'023777','1996-005C','Perekhodnik','Sredniy perekhodnik',65893,'Reentered','1996-01-29 00:00:00','1996-01-26 00:00:00',87.87,167,172,51.63,'Intermediate' from dual union all
select 23778,'023778','1996-005D','Blok DM2 No. 81L','11S861 No. 81L',65893,'In Earth orbit','','1996-02-26 00:00:00',1474.21,36500,36559,1.46,'Drift GEO' from dual union all
select 23779,'023779','1996-006A','Palapa C1','Palapa C1',65894,'In Earth orbit','','1996-03-02 00:00:00',1436.16,35493,36083,.05,'Stationary' from dual union all
select 2378,'002378','1963-014Y','deb West Ford','RCS 0.24',9245,'In Earth orbit','','1992-01-30 00:00:00',166.28,3400,3877,87.32,'Medium' from dual union all
select 23780,'023780','1996-006B','Centaur AC-126','Centaur IIA(4N) AC-126',65894,'Reentered','1996-02-03 00:00:00','1996-02-03 00:00:00',1926.93,152,89681,21.99,'Very High' from dual union all
select 23781,'023781','1996-007A','N-Star b','N-Star b',65895,'In Earth orbit','','1996-03-06 00:00:00',1436.11,35489,36085,.05,'Stationary' from dual union all
select 23782,'023782','1996-007B','Ariane H10-3 V83','Ariane H10-3 S/N T455',65895,'Reentered','2000-06-07 00:00:00','1996-03-06 00:00:00',655.71,210,37037,6.81,'GEO Transfer' from dual union all
select 23783,'023783','1986-017JV','EO-20 EVA-3 debris','',59210,'Reentered','1997-09-11 00:00:00','1996-02-09 00:00:00',92.38,389,394,51.64,'Intermediate' from dual union all
select 23784,'023784','1996-008A','NEAR Shoemaker','NEAR',65896,'Deep Space','1996-02-18 00:00:00','1996-02-17 00:00:00',314.85,184,43602,28.5,'Earth Escape' from dual union all
select 23785,'023785','1996-008B','Delta 232','Delta SSPS AJ10-118K',65896,'Reentered','1998-01-17 00:00:00','1996-03-18 00:00:00',156.73,175,6328,26.02,'Medium' from dual union all
select 23786,'023786','1996-008C','Star 48B','Star 48B',65896,'Deep Space','1996-02-18 00:00:00','1996-02-17 00:00:00',314.85,184,43602,28.5,'Earth Escape' from dual union all
select 23787,'023787','1996-009A','Gonets-D1 No. 319','Gonets-D1 No. 1 (13)',65897,'In Earth orbit','','1996-03-20 00:00:00',113.92,1400,1413,82.58,'Intermediate' from dual union all
select 23788,'023788','1996-009B','Gonets-D1 No. 320','Gonets-D1 No. 2 (14)',65897,'In Earth orbit','','1996-03-20 00:00:00',114,1407,1413,82.57,'Intermediate' from dual union all
select 23789,'023789','1996-009C','Gonets-D1 No. 321','Gonets-D1 No. 3 (15)',65897,'In Earth orbit','','1996-03-21 00:00:00',114.05,1409,1416,82.58,'Intermediate' from dual union all
select 2379,'002379','1963-014Z','deb West Ford','RCS 0.11',9245,'In Earth orbit','','1992-01-23 00:00:00',165.54,2362,4856,86.78,'Medium' from dual union all
select 23790,'023790','1996-009D','Kosmos-2328','Strela-3',65897,'In Earth orbit','','1996-03-20 00:00:00',114.05,1411,1414,82.58,'Intermediate' from dual union all
select 23791,'023791','1996-009E','Kosmos-2329','Strela-3',65897,'In Earth orbit','','1996-03-21 00:00:00',114.14,1411,1422,82.58,'Intermediate' from dual union all
select 23792,'023792','1996-009F','Kosmos-2330','Strela-3',65897,'In Earth orbit','','1996-03-20 00:00:00',114.21,1411,1428,82.58,'Intermediate' from dual union all
select 23793,'023793','1996-009G','S5M','S5M',65897,'In Earth orbit','','1996-03-21 00:00:00',114.72,1408,1477,82.57,'Intermediate' from dual union all
select 23794,'023794','1996-010A','Raduga','Gran'' No. 44L',65898,'Reentered','2004-05-06 00:00:00','1996-03-20 00:00:00',645.39,223,36498,48.52,'GEO Transfer' from dual union all
select 23795,'023795','1996-010B','8S812','8S812',65898,'Reentered','1996-02-22 00:00:00','1996-02-21 00:00:00',88.06,175,182,51.62,'Intermediate' from dual union all
select 23796,'023796','1996-010C','Perekhodnik','Sredniy perekhodnik',65898,'Reentered','1996-02-20 00:00:00','1996-02-19 00:00:00',88.04,174,181,51.61,'Intermediate' from dual union all
select 23797,'023797','1996-010D','Blok DM2 No. 82L','11S861 No. 82L',65898,'Reentered','1996-08-21 00:00:00','1996-03-20 00:00:00',645,231,36470,48.49,'GEO Transfer' from dual union all
select 23798,'023798','1996-011A','Soyuz TM-23','Soyuz 7K-STM No. 72',65899,'Landed','1996-09-02 00:00:00','1996-03-23 00:00:00',92.42,389,398,51.65,'Intermediate' from dual union all
select 23799,'023799','1996-011B','11S510','11S510',65899,'Reentered','1996-02-24 00:00:00','1996-02-23 00:00:00',88.26,181,195,51.61,'Intermediate' from dual union all
select 238,'000238','61 Omicron 69','deb r/b Ablestar','',7273,'Reentered','1969-12-07 00:00:00','1962-02-10 00:00:00',99.79,526,970,67.08,'Intermediate' from dual union all
select 2380,'002380','1963-014AA','deb West Ford','RCS 0.15',9245,'In Earth orbit','','1992-02-03 00:00:00',165.98,2915,4338,87.21,'Medium' from dual union all
select 23800,'023800','1996-005E','SOZ','SOZ',65893,'Reentered','1996-12-16 00:00:00','1996-02-24 00:00:00',647.76,277,36565,47.32,'GEO Transfer' from dual union all
select 23801,'023801','1996-012A','Columbia (STS-75)','OV-102',65900,'Landed','1996-03-09 00:00:00','1996-02-29 00:00:00',90.36,289,296,28.46,'Intermediate' from dual union all
select 23802,'023802','1996-013A','Polar','Polar',65901,'In Earth orbit','','1996-03-25 00:00:00',1051.45,5157,50591,85.93,'Highly Ellip' from dual union all
select 23803,'023803','1996-013B','Delta 233','Delta SSPS AJ10-118K',65901,'Reentered','1996-05-02 00:00:00','1996-03-25 00:00:00',94.08,179,769,85.91,'Polar' from dual union all
select 23804,'023804','1996-013C','Star 48B','Star 48B',65901,'Reentered','1996-12-02 00:00:00','1996-04-08 00:00:00',935.4,211,50399,86.09,'Highly Ellip' from dual union all
select 23805,'023805','1996-012B','TSS-1R','TSS-1R',65900,'Reentered','1996-03-20 00:00:00','1996-02-29 00:00:00',91.72,315,402,28.47,'Intermediate' from dual union all
select 23806,'023806','1981-038D','deb USAF satellite','',52561,'In Earth orbit','','1996-01-31 00:00:00',691.87,960,38109,63.2,'Molniya' from dual union all
select 23807,'023807','1981-038E','deb USAF satellite','',52561,'In Earth orbit','','1996-01-31 00:00:00',687.03,1209,37618,63.3,'GEO Transfer' from dual union all
select 23808,'023808','1988-077P','deb Transtage D-3','',61326,'In Earth orbit','','1996-03-01 00:00:00',702.53,400,39200,26.7,'GEO Transfer' from dual union all
select 23809,'023809','1988-077Q','deb Transtage D-3','',61326,'In Earth orbit','','1996-03-01 00:00:00',702.53,400,39200,26.7,'GEO Transfer' from dual union all
select 2381,'002381','1963-014AB','deb West Ford','RCS 0.22',9245,'In Earth orbit','','1992-02-01 00:00:00',166.18,3269,4000,87.31,'Medium' from dual union all
select 23810,'023810','1988-077R','deb Transtage D-3','',61326,'In Earth orbit','','1996-03-01 00:00:00',702.53,400,39200,26.7,'GEO Transfer' from dual union all
select 23811,'023811','1983-078C','deb USAF satellite','',55835,'In Earth orbit','','1996-03-01 00:00:00',690.49,1000,38000,63.6,'Molniya' from dual union all
select 23812,'023812','1983-078D','deb USAF satellite','',55835,'In Earth orbit','','1996-03-01 00:00:00',690.49,1000,38000,63.6,'Molniya' from dual union all
select 23813,'023813','1996-005F','SOZ','SOZ',65893,'Reentered','2004-03-07 00:00:00','1996-02-29 00:00:00',648.89,324,36576,47.36,'GEO Transfer' from dual union all
select 23814,'023814','1996-014A','REX II','REX II',65902,'In Earth orbit','','1996-04-08 00:00:00',101.24,803,831,89.94,'Polar' from dual union all
select 23815,'023815','1996-014B','Orion 38','Orion 38',65902,'In Earth orbit','','1996-04-08 00:00:00',101.21,801,831,89.95,'Polar' from dual union all
select 23816,'023816','1996-015A','INTELSAT 707','INTELSAT 707',65903,'In Earth orbit','','1996-04-16 00:00:00',1436.14,35784,35790,.05,'Stationary' from dual union all
select 23817,'023817','1996-015B','Ariane H10-3 V84','Ariane H10-3 S/N T456',65903,'Reentered','2000-10-19 00:00:00','1996-04-13 00:00:00',651.48,206,36826,7,'GEO Transfer' from dual union all
select 23818,'023818','1996-016A','Kosmos-2331','Kobal''t No. 549',65904,'Landed','1996-06-11 00:00:00','1996-04-13 00:00:00',89.6,179,331,67.13,'Intermediate' from dual union all
select 23819,'023819','1996-016B','11S510','11S510',65904,'Reentered','1996-03-20 00:00:00','1996-03-16 00:00:00',89.31,160,320,67.13,'Intermediate' from dual union all
select 2382,'002382','1966-066H','S-12 experiment','EVA debris',13473,'Reentered','1966-10-20 00:00:00','1966-08-10 00:00:00',92.1,373,382,28.86,'Intermediate' from dual union all
select 23820,'023820','1966-077D','deb FTV 1352','',13484,'In Earth orbit','','1996-03-15 00:00:00',165.28,1271,5926,88.3,'Medium' from dual union all
select 23821,'023821','1966-077E','deb FTV 1352','',13484,'In Earth orbit','','1996-03-13 00:00:00',164.66,1026,6120,88.57,'Medium' from dual union all
select 23822,'023822','1986-017JW','EO-21 EVA-1 debris','',59210,'Reentered','1996-03-24 00:00:00','1996-03-16 00:00:00',92.32,384,393,51.65,'Intermediate' from dual union all
select 23823,'023823','1986-017JX','EO-21 EVA-1 debris','',59210,'Reentered','1996-03-25 00:00:00','1996-03-16 00:00:00',92.23,383,385,51.64,'Intermediate' from dual union all
select 23824,'023824','1996-010E','SOZ','SOZ',65898,'Reentered','2000-11-23 00:00:00','1996-03-21 00:00:00',649.72,455,36487,48.15,'GEO Transfer' from dual union all
select 23825,'023825','1986-017JY','EO-21 EVA-1 debris','',59210,'Reentered','1997-03-10 00:00:00','1996-03-17 00:00:00',92.42,390,397,51.65,'Intermediate' from dual union all
select 23826,'023826','1986-017JZ','EO-21 EVA-1 debris','',59210,'Reentered','1996-03-24 00:00:00','1996-03-17 00:00:00',92.19,378,386,51.64,'Intermediate' from dual union all
select 23827,'023827','1996-017A','IRS-P3','IRS-P3',65905,'In Earth orbit','','1996-04-20 00:00:00',101.55,816,849,98.79,'Sun-Synch' from dual union all
select 23828,'023828','1996-017B','PSLV-D3 PS4','PS4',65905,'In Earth orbit','','1996-04-20 00:00:00',101.38,802,846,98.8,'Sun-Synch' from dual union all
select 23829,'023829','1966-077F','deb FTV 1352','',13484,'In Earth orbit','','1996-03-21 00:00:00',166.18,1631,5638,88.6,'Medium' from dual union all
select 2383,'002383','1966-066J','GT-10 umbilical bag','EVA debris',13473,'Reentered','1967-01-17 00:00:00','1966-11-11 00:00:00',91.59,350,356,28.87,'Intermediate' from dual union all
select 23830,'023830','1966-077G','deb FTV 1352','',13484,'In Earth orbit','','1996-03-20 00:00:00',166.69,2510,4800,89.33,'Medium' from dual union all
select 23831,'023831','1996-018A','Atlantis (STS-76)','OV-104',65906,'Landed','1996-03-31 00:00:00','1996-03-24 00:00:00',92.39,389,395,51.65,'Intermediate' from dual union all
select 23832,'023832','1966-077H','deb FTV 1352','',13484,'In Earth orbit','','1996-03-24 00:00:00',164.62,1038,6106,88.29,'Medium' from dual union all
select 23833,'023833','1996-019A','GPS IIA-16','Navstar GPS SVN 33',65907,'In Earth orbit','','1996-04-27 00:00:00',717.96,20080,20284,54.7,'Medium' from dual union all
select 23834,'023834','1996-019B','Delta 234','Delta SSPS AJ10-118K',65907,'Reentered','2000-04-27 00:00:00','1996-04-27 00:00:00',94.62,450,550,35.07,'Intermediate' from dual union all
select 23835,'023835','1996-019C','Star 48B','Star 48B',65907,'Reentered','2001-09-29 00:00:00','1996-04-27 00:00:00',352.67,199,20115,34.87,'Highly Ellip' from dual union all
select 23836,'023836','1978-007D','deb Kosmos-985','',45202,'In Earth orbit','','1988-07-31 00:00:00',104.63,937,1018,82.94,'Intermediate' from dual union all
select 23837,'023837','1966-077J','deb FTV 1352','',13484,'In Earth orbit','','1996-03-28 00:00:00',167.31,3198,4162,89.67,'Medium' from dual union all
select 23838,'023838','1966-077K','deb FTV 1352','',13484,'In Earth orbit','','1996-03-15 00:00:00',166.49,2042,5252,88.98,'Medium' from dual union all
select 23839,'023839','1996-020A','INMARSAT III F-1','INMARSAT III F-1',65908,'In Earth orbit','','1996-05-03 00:00:00',1424.51,35546,35573,2.64,'Drift GEO' from dual union all
select 2384,'002384','1965-082KF','deb Transtage 4','',11832,'Reentered','1973-09-29 00:00:00','1966-07-31 00:00:00',99.9,733,774,32.38,'Intermediate' from dual union all
select 23840,'023840','1996-020B','Centaur AC-122','Centaur IIA(4) AC-122',65908,'In Earth orbit','','1996-05-03 00:00:00',642.28,1032,35530,21.9,'GEO Transfer' from dual union all
select 23841,'023841','1966-077L','deb FTV 1352','',13484,'In Earth orbit','','1996-04-04 00:00:00',165.24,1756,5438,89.1,'Medium' from dual union all
select 23842,'023842','1996-021A','Astra 1F','Astra 1F',65909,'In Earth orbit','','1996-05-08 00:00:00',1436.09,35776,35797,.03,'Stationary' from dual union all
select 23843,'023843','1996-021B','Perekhodnik','Sredniy perekhodnik',65909,'Reentered','1996-04-11 00:00:00','1996-04-10 00:00:00',88.44,194,201,51.6,'Intermediate' from dual union all
select 23844,'023844','1996-021C','8S812','8S812',65909,'Reentered','1996-04-14 00:00:00','1996-04-10 00:00:00',88.55,189,217,51.6,'Intermediate' from dual union all
select 23845,'023845','1996-021D','Blok DM3 No. 1L','Blok DM3 No. 1L',65909,'In Earth orbit','','1996-05-11 00:00:00',876.15,11971,35933,6.95,'Medium' from dual union all
select 23846,'023846','1996-022A','M-SAT 1','MSAT-1',65910,'In Earth orbit','','1996-05-24 00:00:00',1436.06,35776,35796,.05,'Stationary' from dual union all
select 23847,'023847','1996-022B','Ariane H10-3 V85','Ariane H10-3 S/N T457',65910,'In Earth orbit','','1996-05-20 00:00:00',633.29,287,35814,7,'GEO Transfer' from dual union all
select 23848,'023848','1996-023A','Priroda','TsM-I 77KSI No. 17401',65911,'Reentered Attached','2001-03-23 00:00:00','1996-05-24 00:00:00',92.4,389,396,51.65,'Intermediate' from dual union all
select 23849,'023849','1996-023B','8S812','8S812',65911,'Reentered','1996-05-19 00:00:00','1996-04-30 00:00:00',89.52,209,293,51.66,'Intermediate' from dual union all
select 2385,'002385','1965-082KG','deb Transtage 4','',11832,'Reentered','1989-04-16 00:00:00','1968-11-14 00:00:00',99.71,719,771,32.4,'Intermediate' from dual union all
select 23850,'023850','1996-023C','deb Priroda','',65911,'Reentered','1996-05-13 00:00:00','1996-05-01 00:00:00',92.58,209,593,51.68,'Intermediate' from dual union all
select 23851,'023851','1996-024A','MSX','MSX',65912,'In Earth orbit','','1996-05-25 00:00:00',103.01,896,906,99.38,'Sun-Synch' from dual union all
select 23852,'023852','1996-024B','Delta 235','Delta SSPS AJ10-118K',65912,'Reentered','1997-01-22 00:00:00','1996-05-24 00:00:00',95.29,221,845,96.58,'Sun-Synch' from dual union all
select 23853,'023853','1996-025A','Kosmos-2332','Taifun-1B',65913,'Reentered','2005-01-28 00:00:00','1996-05-24 00:00:00',103.59,295,1562,82.96,'Intermediate' from dual union all
select 23854,'023854','1996-025B','S3M','S3M',65913,'Reentered','2001-01-04 00:00:00','1996-05-24 00:00:00',103.49,289,1558,82.96,'Intermediate' from dual union all
select 23855,'023855','1996-026A','USA 118','MERCURY 15',65914,'In Earth orbit','','1996-05-01 00:00:00',1436.27,35780,35800,.1,'Stationary' from dual union all
select 23856,'023856','1986-017KF','EO-21 EVA-4 debris','',59210,'Reentered','1996-10-11 00:00:00','1996-06-01 00:00:00',92.37,388,394,51.65,'Intermediate' from dual union all
select 23857,'023857','1996-027A','BeppoSAX','SAX',65915,'Reentered','2003-04-29 00:00:00','1996-06-10 00:00:00',96.53,581,604,3.94,'Equatorial' from dual union all
select 23858,'023858','1996-027B','Centaur AC-78','Centaur I AC-78',65915,'Reentered','2000-06-09 00:00:00','1996-06-10 00:00:00',95.12,462,586,3.99,'Equatorial' from dual;
insert into SATELLITE
select 23859,'023859','1996-024C','SPIRIT III Cover','SPIRIT III Cover',65912,'In Earth orbit','','1996-05-25 00:00:00',103.11,842,969,99.37,'Sun-Synch' from dual union all
select 2386,'002386','1966-024D','NNS O-8 despin weight','',13431,'In Earth orbit','','2012-06-27 00:00:00',104.39,869,1062,90.15,'Polar' from dual union all
select 23860,'023860','1996-028A','Progress M-31','Progress 7K-TGM No. 231',65916,'Deorbited','1996-08-01 00:00:00','1996-06-05 00:00:00',92.4,389,396,51.65,'Intermediate' from dual union all
select 23861,'023861','1996-028B','11S510','11S510',65916,'Reentered','1996-05-08 00:00:00','1996-05-07 00:00:00',88.26,174,203,51.62,'Intermediate' from dual union all
select 23862,'023862','1996-029D','USA 122','SLDCOM 4',65917,'In Earth orbit','','2007-01-23 00:00:00',260.46,2653,11610,63.47,'Medium' from dual union all
select 23863,'023863','1996-029G','Titan 45K-22 Stage 2','LR91AJ11A 45K-22',65917,'Reentered','1999-03-12 00:00:00','1996-06-15 00:00:00',93.78,276,643,63.41,'Intermediate' from dual union all
select 23864,'023864','1996-030A','Palapa C2','Palapa C2',65918,'In Earth orbit','','1996-06-16 00:00:00',1436.11,35785,35789,.07,'Stationary' from dual union all
select 23865,'023865','1996-030B','Amos 1','Amos-1',65918,'In Earth orbit','','1996-06-16 00:00:00',1436.04,35767,35804,.05,'Stationary' from dual union all
select 23866,'023866','1996-030C','Ariane H10-3 V86','Ariane H10-3 S/N T458',65918,'In Earth orbit','','1996-06-15 00:00:00',638.06,281,36065,4.4,'GEO Transfer' from dual union all
select 23867,'023867','1996-030D','Spelda 30 S/N UV29','Spelda 30',65918,'In Earth orbit','','1996-06-17 00:00:00',634.6,313,35855,4.18,'GEO Transfer' from dual union all
select 23868,'023868','1996-031A','MSTI 3','MSTI 3',65919,'Deorbited','1997-12-11 00:00:00','1996-06-17 00:00:00',93.12,420,435,97.12,'Sun-Synch' from dual union all
select 23869,'023869','1996-031B','Orion 38','Orion 38',65919,'Reentered','1996-10-11 00:00:00','1996-06-16 00:00:00',90.86,291,343,97.08,'Sun-Synch' from dual union all
select 2387,'002387','1965-020EK','deb Blok Kosmos','',11770,'Reentered','1966-10-19 00:00:00','1966-08-15 00:00:00',97.21,301,950,56.02,'Intermediate' from dual union all
select 23870,'023870','1996-032A','Endeavour (STS-77)','OV-105',65920,'Landed','1996-05-29 00:00:00','1996-05-21 00:00:00',90.18,278,289,39.02,'Intermediate' from dual union all
select 23871,'023871','1996-032B','Spartan 207','Spartan 207',65920,'Landed Attached','1996-05-29 00:00:00','1996-05-21 00:00:00',90.26,281,294,39.02,'Intermediate' from dual union all
select 23872,'023872','1996-032C','IAE','Inflatable Antenna Expt.',65920,'Reentered','1996-05-22 00:00:00','1996-05-21 00:00:00',90.23,277,295,38.99,'Intermediate' from dual union all
select 23873,'023873','1986-017KA','EO-21 EVA-2 debris','',59210,'Reentered','1997-07-18 00:00:00','1996-05-21 00:00:00',92.36,387,394,51.65,'Intermediate' from dual union all
select 23874,'023874','1986-017KB','EO-21 EVA-2 debris','',59210,'Reentered','1997-05-26 00:00:00','1996-05-21 00:00:00',92.41,389,397,51.65,'Intermediate' from dual union all
select 23875,'023875','1986-017KC','EO-21 EVA-2 debris','',59210,'Reentered','1997-05-17 00:00:00','1996-05-22 00:00:00',92.38,388,395,51.65,'Intermediate' from dual union all
select 23876,'023876','1996-032D','PAMS','PAMS',65920,'Reentered','1996-10-26 00:00:00','1996-06-18 00:00:00',90.06,274,281,39.02,'Intermediate' from dual union all
select 23877,'023877','1996-033A','Galaxy 9','Galaxy 9',65921,'In Earth orbit','','1996-06-23 00:00:00',1436.04,35777,35794,.04,'Stationary' from dual union all
select 23878,'023878','1996-033B','Delta 236','Delta SSPS AJ10-118K',65921,'Reentered','2003-01-17 00:00:00','1996-06-23 00:00:00',104.16,337,1574,25.83,'Intermediate' from dual union all
select 23879,'023879','1996-033C','Star 48B','Star 48B',65921,'In Earth orbit','','1996-06-23 00:00:00',681.96,1886,36687,23.09,'GEO Transfer' from dual union all
select 2388,'002388','1966-066K','Hasselblad camera','EVA debris',13473,'Reentered','1967-02-26 00:00:00','1966-10-26 00:00:00',92.03,371,378,28.88,'Intermediate' from dual union all
select 23880,'023880','1996-034A','Gorizont','Gorizont No. 44L',65922,'In Earth orbit','','1996-06-24 00:00:00',1436.06,35722,35849,1.43,'Stationary' from dual union all
select 23881,'023881','1996-034B','8S812','8S812',65922,'Reentered','1996-05-28 00:00:00','1996-05-27 00:00:00',87.93,165,179,51.62,'Intermediate' from dual union all
select 23882,'023882','1996-034C','Perekhodnik','Sredniy perekhodnik',65922,'Reentered','1996-05-26 00:00:00','1996-05-25 00:00:00',88.23,182,192,51.61,'Intermediate' from dual union all
select 23883,'023883','1996-034D','Blok DM2 No. 100L','11S861 No. 100L',65922,'In Earth orbit','','1996-06-24 00:00:00',1470.12,36345,36555,1.46,'Drift GEO' from dual union all
select 23884,'023884','1986-017KD','EO-21 EVA-3 debris','',59210,'Reentered','1997-05-15 00:00:00','1996-05-25 00:00:00',92.4,388,396,51.65,'Intermediate' from dual union all
select 23885,'023885','1986-017KE','EO-21 EVA-3 debris','',59210,'Reentered','1996-12-23 00:00:00','1996-05-25 00:00:00',92.41,381,405,51.62,'Intermediate' from dual union all
select 23886,'023886','1996-034E','SOZ','SOZ',65922,'Reentered','1996-12-19 00:00:00','1996-06-28 00:00:00',646.64,250,36535,47.25,'GEO Transfer' from dual union all
select 23887,'023887','1996-034F','SOZ','SOZ',65922,'Reentered','2000-03-12 00:00:00','1996-06-24 00:00:00',647.89,311,36538,47.33,'GEO Transfer' from dual union all
select 23888,'023888','1986-017KG','EO-21 EVA-4 debris','',59210,'Reentered','1996-10-01 00:00:00','1996-06-01 00:00:00',92.35,386,394,51.65,'Intermediate' from dual union all
select 23889,'023889','1986-017KH','EO-21 EVA-4 debris','',59210,'Reentered','1996-06-11 00:00:00','1996-06-01 00:00:00',91.96,366,376,51.65,'Intermediate' from dual union all
select 2389,'002389','1966-070A','OV3-3','OV3-3',13477,'In Earth orbit','','1966-11-05 00:00:00',136.84,358,4479,81.47,'Medium' from dual union all
select 23890,'023890','1986-017KJ','EO-21 EVA-4 debris','',59210,'Reentered','1997-03-16 00:00:00','1996-06-02 00:00:00',92.34,389,390,51.65,'Intermediate' from dual union all
select 23891,'023891','1986-017KK','EO-21 EVA-4 debris','',59210,'Reentered','1996-12-27 00:00:00','1996-06-01 00:00:00',92.36,385,396,51.65,'Intermediate' from dual union all
select 23892,'023892','1986-017KL','EO-21 EVA-4 debris','',59210,'Reentered','1997-12-04 00:00:00','1996-06-01 00:00:00',92.39,388,395,51.65,'Intermediate' from dual union all
select 23893,'023893','1996-029A','USA 119','NOSS B-4 SSU-1',65917,'In Earth orbit','','1996-06-12 00:00:00',107.43,1051,1165,63.41,'Intermediate' from dual union all
select 23894,'023894','1986-017KM','EO-21 EVA-5 debris','',59210,'Reentered','1997-03-18 00:00:00','1996-06-03 00:00:00',92.36,385,396,51.65,'Intermediate' from dual union all
select 23895,'023895','1986-017KN','EO-21 EVA-5 debris','',59210,'Reentered','1997-02-28 00:00:00','1996-06-03 00:00:00',92.38,386,397,51.65,'Intermediate' from dual union all
select 23896,'023896','1986-017KP','EO-21 EVA-5 debris','(long lived)',59210,'Reentered','1998-03-26 00:00:00','1996-06-03 00:00:00',92.4,383,402,51.65,'Intermediate' from dual union all
select 23897,'023897','1986-017KQ','EO-21 EVA-5 debris','',59210,'Reentered','1997-11-04 00:00:00','1996-06-03 00:00:00',92.38,387,396,51.65,'Intermediate' from dual union all
select 23898,'023898','1986-017KR','EO-21 EVA-5 debris','',59210,'Reentered','1997-05-01 00:00:00','1996-06-08 00:00:00',92.33,382,396,51.65,'Intermediate' from dual union all
select 23899,'023899','1986-017KS','EO-21 EVA-5 debris','',59210,'Reentered','1997-12-10 00:00:00','1996-06-08 00:00:00',92.37,385,397,51.65,'Intermediate' from dual union all
select 239,'000239','61 Omicron 70','deb r/b Ablestar','',7273,'Reentered','1967-06-17 00:00:00','1962-02-23 00:00:00',98.63,438,949,67,'Intermediate' from dual union all
select 2390,'002390','1965-082KH','deb Transtage 4','',11832,'Reentered','1966-10-27 00:00:00','1966-08-21 00:00:00',96.37,551,618,32.16,'Intermediate' from dual union all
select 23900,'023900','1986-017KT','EO-21 EVA-5 debris','',59210,'Reentered','1997-06-01 00:00:00','1996-06-08 00:00:00',92.35,385,395,51.65,'Intermediate' from dual union all
select 23901,'023901','1986-017KU','EO-21 EVA-5 debris','',59210,'Reentered','1997-06-26 00:00:00','1996-06-08 00:00:00',92.36,385,396,51.65,'Intermediate' from dual union all
select 23902,'023902','1986-017KV','EO-21 EVA-5 debris','',59210,'Reentered','1997-11-18 00:00:00','1996-06-08 00:00:00',92.38,387,396,51.65,'Intermediate' from dual union all
select 23903,'023903','1986-017KW','EO-21 EVA-5 debris','',59210,'Reentered','1996-10-31 00:00:00','1996-06-08 00:00:00',92.39,386,398,51.65,'Intermediate' from dual union all
select 23904,'023904','1986-017KX','EO-21 EVA-5 debris','(long lived)',59210,'Reentered','1998-06-26 00:00:00','1996-06-09 00:00:00',92.4,389,396,51.65,'Intermediate' from dual union all
select 23905,'023905','1978-064G','deb Seasat 1','',45259,'Reentered','1998-08-20 00:00:00','1996-06-08 00:00:00',99.3,713,738,108.02,'Retrograde' from dual union all
select 23906,'023906','1983-063E','deb P83-1','',55820,'Reentered','2011-12-02 00:00:00','1996-06-10 00:00:00',100.55,755,814,82.02,'Intermediate' from dual union all
select 23907,'023907','1996-029B','USA 120','NOSS B-4 SSU-2',65917,'In Earth orbit','','1996-06-14 00:00:00',107.43,1050,1166,63.41,'Intermediate' from dual union all
select 23908,'023908','1996-029C','USA 121','NOSS B-4 SSU-3',65917,'In Earth orbit','','1996-06-13 00:00:00',107.43,1053,1162,63.41,'Intermediate' from dual union all
select 23909,'023909','1996-016C','Kosmos-2331 AO part','Kobal''t AO part',65904,'Reentered','1996-06-12 00:00:00','1996-06-11 00:00:00',88.47,152,246,67.12,'Intermediate' from dual union all
select 2391,'002391','1966-071A','Kosmos-127','Zenit-4 No. 22',13478,'Landed','1966-08-16 00:00:00','1966-08-11 00:00:00',89.17,202,264,51.83,'Intermediate' from dual union all
select 23910,'023910','1986-017KY','EO-21 EVA-6 debris','(long lived)',59210,'Reentered','1998-05-04 00:00:00','1996-06-14 00:00:00',92.41,389,397,51.65,'Intermediate' from dual union all
select 23911,'023911','1986-017KZ','EO-21 EVA-6 debris','',59210,'Reentered','1997-04-30 00:00:00','1996-06-14 00:00:00',92.35,384,396,51.65,'Intermediate' from dual union all
select 23912,'023912','1986-017LA','EO-21 EVA-6 debris','',59210,'Reentered','1997-09-28 00:00:00','1996-06-15 00:00:00',92.36,385,396,51.65,'Intermediate' from dual union all
select 23913,'023913','1986-017LB','EO-21 EVA-6 debris','',59210,'Reentered','1996-11-06 00:00:00','1996-06-14 00:00:00',92.36,385,396,51.65,'Intermediate' from dual union all
select 23914,'023914','1986-017LC','EO-21 EVA-6 debris','',59210,'Reentered','1997-07-10 00:00:00','1996-06-15 00:00:00',92.44,394,395,51.64,'Intermediate' from dual union all
select 23915,'023915','1996-035A','INTELSAT 709','INTELSAT 709',65923,'In Earth orbit','','1996-07-15 00:00:00',1436.14,35779,35796,.03,'Stationary' from dual union all
select 23916,'023916','1996-035B','Ariane H10-3 V87','Ariane H10-3 S/N T459',65923,'Reentered','1996-09-30 00:00:00','1996-07-17 00:00:00',635.08,129,36064,7.04,'GEO Transfer' from dual union all
select 23917,'023917','1986-017LD','EO-21 EVA-6 debris','',59210,'Reentered','1997-03-03 00:00:00','1996-06-17 00:00:00',92.37,387,395,51.65,'Intermediate' from dual union all
select 23918,'023918','1986-017LE','EO-21 EVA-6 debris','',59210,'Reentered','1997-01-11 00:00:00','1996-06-17 00:00:00',92.37,387,395,51.65,'Intermediate' from dual union all
select 23919,'023919','1986-017LF','EO-21 EVA-6 debris','',59210,'Reentered','1996-08-04 00:00:00','1996-06-17 00:00:00',92.38,388,395,51.65,'Intermediate' from dual union all
select 2392,'002392','1966-071B','11A57I','11A57I',13478,'Reentered','1966-08-12 00:00:00','1966-08-10 00:00:00',88.74,178,247,51.83,'Intermediate' from dual union all
select 23920,'023920','1986-017LG','EO-21 EVA-6 debris','(long lived)',59210,'Reentered','1998-06-09 00:00:00','1996-06-17 00:00:00',92.39,388,395,51.65,'Intermediate' from dual union all
select 23921,'023921','1986-017LH','EO-21 EVA-6 debris','',59210,'Reentered','1997-11-29 00:00:00','1996-06-18 00:00:00',92.39,388,395,51.65,'Intermediate' from dual union all
select 23922,'023922','1986-017LJ','EO-21 EVA-6 debris','',59210,'Reentered','1997-11-27 00:00:00','1996-06-17 00:00:00',92.37,388,394,51.65,'Intermediate' from dual union all
select 23923,'023923','1986-017LK','EO-21 EVA-6 debris','',59210,'Reentered','1997-02-19 00:00:00','1996-06-17 00:00:00',92.38,388,395,51.65,'Intermediate' from dual union all
select 23924,'023924','1986-017LL','EO-21 EVA-6 debris','',59210,'Reentered','1996-10-16 00:00:00','1996-06-17 00:00:00',92.34,360,419,51.65,'Intermediate' from dual union all
select 23925,'023925','1986-017LM','EO-21 EVA-6 debris','',59210,'Reentered','1997-03-01 00:00:00','1996-06-18 00:00:00',92.36,386,395,51.66,'Intermediate' from dual union all
select 23926,'023926','1986-017LN','EO-21 EVA-6 debris','',59210,'Reentered','1996-09-14 00:00:00','1996-06-18 00:00:00',92.34,387,392,51.65,'Intermediate' from dual union all
select 23927,'023927','1986-017LP','EO-21 EVA-6 debris','',59210,'Reentered','1996-09-04 00:00:00','1996-06-17 00:00:00',92.36,388,393,51.66,'Intermediate' from dual union all
select 23928,'023928','1986-017LQ','EO-21 EVA-6 debris','',59210,'Reentered','1996-12-11 00:00:00','1996-06-18 00:00:00',92.4,389,396,51.65,'Intermediate' from dual union all
select 23929,'023929','1986-017LR','EO-21 EVA-6 debris','',59210,'Reentered','1997-11-08 00:00:00','1996-06-18 00:00:00',92.41,389,397,51.65,'Intermediate' from dual union all
select 2393,'002393','1966-072A','OPS 1545','CORONA J-32',13479,'Reentered Attached','1966-09-11 00:00:00','1966-08-16 00:00:00',89.15,189,276,100.11,'Retrograde' from dual union all
select 23930,'023930','1978-064H','deb Seasat 1','',45259,'Reentered','1998-08-02 00:00:00','1996-06-19 00:00:00',99.33,716,737,108,'Retrograde' from dual union all
select 23931,'023931','1996-036A','Columbia (STS-78)','OV-102',65924,'Landed','1996-07-07 00:00:00','1996-06-27 00:00:00',89.89,264,274,39.01,'Intermediate' from dual union all
select 23932,'023932','1986-017LS','EO-21 EVA-6 debris','',59210,'Reentered','1996-11-23 00:00:00','1996-06-22 00:00:00',182,468,8053,56.99,'Medium' from dual union all
select 23933,'023933','61 A Delta 7','West Ford package fragment','',7251,'In Earth orbit','','1996-06-20 00:00:00',165.42,2747,4461,95.88,'Medium' from dual union all
select 23934,'023934','1986-017LT','EO-21 EVA-6 debris','',59210,'Reentered','1997-05-26 00:00:00','1996-06-22 00:00:00',167.34,143,7218,28.26,'Medium' from dual union all
select 23935,'023935','1978-096F','deb Tiros-N','',45291,'Reentered','1999-12-18 00:00:00','1996-06-26 00:00:00',101.27,808,829,98.89,'Sun-Synch' from dual union all
select 23936,'023936','1996-029E','USA 123','TIPS Ralph',65917,'In Earth orbit','','1996-07-07 00:00:00',107.42,1055,1160,63.42,'Intermediate' from dual union all
select 23937,'023937','1996-029F','USA 124','TIPS Norton/TIPS Tether',65917,'In Earth orbit','','1996-06-20 00:00:00',105.58,1020,1024,63.41,'Intermediate' from dual union all
select 23938,'023938','1996-029H','deb USA 119','',65917,'In Earth orbit','','1997-11-21 00:00:00',109.22,1151,1231,63.43,'Intermediate' from dual union all
select 23939,'023939','1996-029J','deb USA 119','',65917,'In Earth orbit','','1996-07-31 00:00:00',109.2,1150,1230,63.43,'Intermediate' from dual union all
select 2394,'002394','1966-073A','Lunar Orbiter I','Lunar Orbiter A',13480,'Deep Space','1966-08-11 00:00:00','1966-08-10 00:00:00',12587.92,196,345551,29.61,'Cislunar' from dual union all
select 23940,'023940','1996-037A','Earth Probe TOMS','TOMS-EP',65925,'In Earth orbit','','1996-08-01 00:00:00',94.65,493,511,97.44,'Sun-Synch' from dual union all
select 23941,'023941','1996-037B','Orion 38','Orion 38',65925,'Reentered','2000-09-05 00:00:00','1996-08-01 00:00:00',97.52,341,939,97.38,'Sun-Synch' from dual union all
select 23942,'023942','1994-029C','deb HAPS','',65425,'Reentered','1996-07-03 00:00:00','1996-07-01 00:00:00',91.69,317,398,81.85,'Intermediate' from dual union all
select 23943,'023943','1996-039A','Apstar 1A','Apstar 1A',65927,'In Earth orbit','','1996-08-02 00:00:00',1436.09,35779,35794,.03,'Stationary' from dual union all
select 23944,'023944','1996-039B','CZ-3 Stage 3','CZ-3 Stage 3',65927,'Reentered','2009-09-12 00:00:00','1996-08-02 00:00:00',758.95,233,42134,27.17,'Highly Ellip' from dual union all
select 23945,'023945','1996-038A','USA 125','QUASAR 10',65926,'In Earth orbit','2011-01-01 00:00:00','2005-02-10 00:00:00',717.62,1020,39327,64.13,'GEO Transfer' from dual union all
select 23946,'023946','1996-038B','Titan 45K-2 Stage 2','LR91AJ11A 45K-2',65926,'Reentered','1996-10-15 00:00:00','1996-08-20 00:00:00',90.06,276,279,55,'Intermediate' from dual union all
select 23947,'023947','1996-038C','USA 125 PKM','TU-882',65926,'In Earth orbit','','1999-12-16 00:00:00',263.23,383,14072,55.36,'Highly Ellip' from dual union all
select 23948,'023948','1996-040A','Arabsat IIA','Arabsat 2A',65928,'In Earth orbit','','1996-08-09 00:00:00',1436.01,35779,35790,.05,'Stationary' from dual;
insert into SATELLITE
select 23949,'023949','1996-040B','Turksat 1C','Turksat 1C',65928,'In Earth orbit','','1996-08-08 00:00:00',1435.93,35770,35797,.04,'Stationary' from dual union all
select 2395,'002395','1966-073B','Agena D 6630','Agena D 6630',13480,'Deep Space','1966-08-11 00:00:00','1966-08-10 00:00:00',12587.92,196,345551,29.61,'Cislunar' from dual union all
select 23950,'023950','1996-040C','Ariane H10-3 V89','Ariane H10-3 S/N T460',65928,'In Earth orbit','','1996-08-10 00:00:00',658.99,301,37113,7.11,'GEO Transfer' from dual union all
select 23951,'023951','1996-040D','Spelda 40 S/N SP UV31','Spelda 40 (SMS)',65928,'Reentered','2001-03-26 00:00:00','1996-08-08 00:00:00',624.43,227,35417,7.08,'GEO Transfer' from dual union all
select 23952,'023952','1966-077M','deb FTV 1352','',13484,'In Earth orbit','','1996-07-10 00:00:00',165.11,2248,4935,89.4,'Medium' from dual union all
select 23953,'023953','1996-041A','GPS IIA-17','Navstar GPS SVN 40',65929,'In Earth orbit','','1996-08-15 00:00:00',717.9,20134,20227,55.04,'Medium' from dual union all
select 23954,'023954','1996-041B','Delta 237','Delta SSPS AJ10-118K',65929,'Reentered','2000-10-08 00:00:00','1996-08-15 00:00:00',94.88,462,564,35.25,'Intermediate' from dual union all
select 23955,'023955','1996-041C','Star 48B','Star 48B',65929,'Reentered','2004-07-15 00:00:00','1996-08-15 00:00:00',353.92,190,20202,35.01,'Highly Ellip' from dual union all
select 23956,'023956','1994-029D','deb HAPS','',65425,'Reentered','1996-09-04 00:00:00','1996-07-16 00:00:00',93.59,371,529,81.27,'Intermediate' from dual union all
select 23957,'023957','1994-029E','deb HAPS','',65425,'Reentered','1996-07-21 00:00:00','1996-07-16 00:00:00',90.27,240,336,82.19,'Intermediate' from dual union all
select 23958,'023958','1994-029F','deb HAPS','',65425,'Reentered','1996-09-28 00:00:00','1996-07-16 00:00:00',94.14,362,592,81.49,'Intermediate' from dual union all
select 23959,'023959','1994-029G','deb HAPS','',65425,'Reentered','1996-09-12 00:00:00','1996-07-17 00:00:00',101.61,309,1361,82.06,'Intermediate' from dual union all
select 2396,'002396','1966-074A','OPS 1832','GAMBIT SV 981',13481,'Deorbited','1966-08-24 00:00:00','1966-08-18 00:00:00',89.41,148,342,93.22,'Polar' from dual union all
select 23960,'023960','1994-029H','deb HAPS','',65425,'Reentered','1996-08-01 00:00:00','1996-07-16 00:00:00',94.5,374,615,82.03,'Intermediate' from dual union all
select 23961,'023961','1994-029J','deb HAPS','',65425,'Reentered','1996-09-21 00:00:00','1996-07-17 00:00:00',93.27,344,525,80.63,'Intermediate' from dual union all
select 23962,'023962','1994-029K','deb HAPS','',65425,'Reentered','1996-08-24 00:00:00','1996-07-17 00:00:00',92.47,329,462,81.35,'Intermediate' from dual union all
select 23963,'023963','1994-029L','deb HAPS','',65425,'Reentered','1996-12-19 00:00:00','1996-07-17 00:00:00',93.64,330,575,82.22,'Intermediate' from dual union all
select 23964,'023964','1994-029M','deb HAPS','',65425,'Reentered','1997-02-26 00:00:00','1996-07-17 00:00:00',94.6,376,623,81.69,'Intermediate' from dual union all
select 23965,'023965','1994-029N','deb HAPS','',65425,'Reentered','1996-09-27 00:00:00','1996-07-17 00:00:00',94.07,389,558,82.36,'Intermediate' from dual union all
select 23966,'023966','1988-098F','deb Ariane V26','',61347,'In Earth orbit','','1996-07-20 00:00:00',539.11,341,30787,4.12,'Highly Ellip' from dual union all
select 23967,'023967','1996-042A','UHF F/O F7','UFO F7-EHF',65930,'In Earth orbit','','1996-08-28 00:00:00',1435.7,34918,36640,5.04,'Inclined GEO' from dual union all
select 23968,'023968','1996-042B','Centaur AC-125','Centaur II(3) AC-125',65930,'In Earth orbit','','1996-08-24 00:00:00',473.55,296,27198,26.87,'Highly Ellip' from dual union all
select 23969,'023969','1994-029P','deb HAPS','',65425,'Reentered','1997-01-03 00:00:00','1996-07-26 00:00:00',97.23,550,703,81.82,'Intermediate' from dual union all
select 2397,'002397','1966-074B','SAMPAN I','P-11 No. 4405',13481,'Reentered','1970-03-05 00:00:00','1966-11-04 00:00:00',94.89,509,518,93.17,'Polar' from dual union all
select 23970,'023970','1994-029Q','deb HAPS','',65425,'Reentered','1998-09-05 00:00:00','1996-07-27 00:00:00',103.55,520,1332,83,'Intermediate' from dual union all
select 23971,'023971','1994-029R','deb HAPS','',65425,'Reentered','2000-10-12 00:00:00','1996-07-27 00:00:00',104.73,579,1385,81.03,'Intermediate' from dual union all
select 23972,'023972','1994-029S','deb HAPS','',65425,'Reentered','1999-01-23 00:00:00','1996-07-27 00:00:00',103.04,592,1213,82.58,'Intermediate' from dual union all
select 23973,'023973','1994-029T','deb HAPS','',65425,'Reentered','1999-02-24 00:00:00','1996-07-27 00:00:00',103.54,596,1256,82.69,'Intermediate' from dual union all
select 23974,'023974','1994-029U','deb HAPS','',65425,'Reentered','1999-01-16 00:00:00','1996-07-27 00:00:00',102.97,587,1212,82.6,'Intermediate' from dual union all
select 23975,'023975','1994-029V','deb HAPS','',65425,'In Earth orbit','','1996-07-27 00:00:00',105.54,582,1458,82.52,'Intermediate' from dual union all
select 23976,'023976','1994-029W','deb HAPS','',65425,'Reentered','2001-02-01 00:00:00','1996-07-27 00:00:00',116.67,592,2471,81.64,'Intermediate' from dual union all
select 23977,'023977','1994-029X','deb HAPS','',65425,'Reentered','1999-11-02 00:00:00','1996-07-27 00:00:00',104.64,568,1388,81.01,'Intermediate' from dual union all
select 23978,'023978','1994-029Y','deb HAPS','',65425,'Reentered','1998-02-20 00:00:00','1996-07-27 00:00:00',107.31,514,1691,82.99,'Intermediate' from dual union all
select 23979,'023979','1994-029Z','deb HAPS','',65425,'Reentered','1997-07-24 00:00:00','1996-07-27 00:00:00',99.36,543,913,82.11,'Intermediate' from dual union all
select 2398,'002398','1966-075A','Pioneer 7','Pioneer B',13482,'Deep Space','1966-08-26 00:00:00','1966-08-17 00:00:00',18492.83,369,476421,42.04,'Earth Escape' from dual union all
select 23980,'023980','1994-029AA','deb HAPS','',65425,'Reentered','1999-04-03 00:00:00','1996-07-27 00:00:00',97.93,468,852,82.51,'Intermediate' from dual union all
select 23981,'023981','1994-029AB','deb HAPS','',65425,'Reentered','1996-08-06 00:00:00','1996-07-27 00:00:00',94.24,407,557,81.82,'Intermediate' from dual union all
select 23982,'023982','1994-029AC','deb HAPS','',65425,'Reentered','2001-01-21 00:00:00','1996-07-27 00:00:00',108.14,564,1718,82.78,'Intermediate' from dual union all
select 23983,'023983','1994-029AD','deb HAPS','',65425,'Reentered','1997-09-03 00:00:00','1996-07-27 00:00:00',102.4,564,1181,80.89,'Intermediate' from dual union all
select 23984,'023984','1994-029AE','deb HAPS','',65425,'Reentered','1996-10-26 00:00:00','1996-07-27 00:00:00',93,309,534,81.13,'Intermediate' from dual union all
select 23985,'023985','1994-029AF','deb HAPS','',65425,'Reentered','1997-11-26 00:00:00','1996-07-27 00:00:00',101.15,598,1029,82.07,'Intermediate' from dual union all
select 23986,'023986','1994-029AG','deb HAPS','',65425,'In Earth orbit','','1996-07-27 00:00:00',98.14,584,756,82.06,'Intermediate' from dual union all
select 23987,'023987','1994-029AH','deb HAPS','',65425,'Reentered','1997-10-11 00:00:00','1996-07-27 00:00:00',95.47,475,607,81.93,'Intermediate' from dual union all
select 23988,'023988','1994-029AJ','deb HAPS','',65425,'Reentered','1996-09-27 00:00:00','1996-07-27 00:00:00',96.72,439,764,82.06,'Intermediate' from dual union all
select 23989,'023989','1994-029AK','deb HAPS','',65425,'Reentered','2001-06-13 00:00:00','1996-07-27 00:00:00',101.11,475,1148,82.32,'Intermediate' from dual union all
select 2399,'002399','1966-075B','Delta 40','DSV-3E-3 S/N 20204',13482,'Reentered','1967-08-07 00:00:00','1966-11-07 00:00:00',96.33,250,916,32.97,'Intermediate' from dual union all
select 23990,'023990','1994-029AL','deb HAPS','',65425,'Reentered','1997-02-18 00:00:00','1996-07-27 00:00:00',98.93,524,891,81.91,'Intermediate' from dual union all
select 23991,'023991','1994-029AM','deb HAPS','',65425,'Reentered','1997-11-06 00:00:00','1996-07-27 00:00:00',100.06,574,949,82.01,'Intermediate' from dual union all
select 23992,'023992','1994-029AN','deb HAPS','',65425,'Reentered','1998-02-15 00:00:00','1996-07-27 00:00:00',98.43,532,836,82.07,'Intermediate' from dual union all
select 23993,'023993','1994-029AP','deb HAPS','',65425,'Reentered','1999-03-01 00:00:00','1996-07-27 00:00:00',96.79,526,684,81.96,'Intermediate' from dual union all
select 23994,'023994','1995-033E','deb CERISE boom','debris',65694,'Reentered','2017-05-22 00:00:00','1996-07-27 00:00:00',98.13,663,676,98.1,'Sun-Synch' from dual union all
select 23995,'023995','1994-029AQ','deb HAPS','',65425,'Reentered','1999-11-14 00:00:00','1996-07-29 00:00:00',96.25,455,703,81.9,'Intermediate' from dual union all
select 23996,'023996','1994-029AR','deb HAPS','',65425,'Reentered','2000-05-09 00:00:00','1996-07-30 00:00:00',98.63,558,829,81.97,'Intermediate' from dual union all
select 23997,'023997','1994-029AS','deb HAPS','',65425,'Reentered','1999-07-11 00:00:00','1996-07-30 00:00:00',101.11,609,1014,82.32,'Intermediate' from dual union all
select 23998,'023998','1994-029AT','deb HAPS','',65425,'Reentered','1996-11-21 00:00:00','1996-07-29 00:00:00',96.2,431,723,82,'Intermediate' from dual union all
select 23999,'023999','1994-029AU','deb HAPS','',65425,'Reentered','2002-04-13 00:00:00','1996-07-30 00:00:00',100.17,576,958,81.85,'Intermediate' from dual union all
select 24,'000024','59 Kappa','Discoverer 7','CORONA C-4',5532,'Reentered Attached','1959-11-26 00:00:00','1959-11-15 00:00:00',92.75,155,664,81.64,'Intermediate' from dual union all
select 240,'000240','62 Gamma 1','Friendship Seven','Mercury SC13',8190,'Landed','1962-02-20 00:00:00','1962-02-20 00:00:00',88.47,149,249,32.54,'Intermediate' from dual union all
select 2400,'002400','1966-074C','Agena D 4829','Agena D',13481,'Reentered','1966-08-18 00:00:00','1966-08-17 00:00:00',89.15,142,323,93.27,'Polar' from dual union all
select 24000,'024000','1994-029AV','deb HAPS','',65425,'Reentered','2001-06-19 00:00:00','1996-07-29 00:00:00',100.83,606,990,81.97,'Intermediate' from dual union all
select 24001,'024001','1994-029AW','deb HAPS','',65425,'Reentered','1999-07-22 00:00:00','1996-07-30 00:00:00',99.89,603,904,82.07,'Intermediate' from dual union all
select 24002,'024002','1994-029AX','deb HAPS','',65425,'Reentered','1997-11-09 00:00:00','1996-07-29 00:00:00',96.34,519,647,81.98,'Intermediate' from dual union all
select 24003,'024003','1994-029AY','deb HAPS','',65425,'Reentered','1999-07-10 00:00:00','1996-07-29 00:00:00',100.06,601,921,82.01,'Intermediate' from dual union all
select 24004,'024004','1994-029AZ','deb HAPS','',65425,'Reentered','1998-03-03 00:00:00','1996-07-29 00:00:00',97.73,557,744,81.93,'Intermediate' from dual union all
select 24005,'024005','1994-029BA','deb HAPS','',65425,'Reentered','1997-10-03 00:00:00','1996-07-29 00:00:00',99.24,579,866,81.81,'Intermediate' from dual union all
select 24006,'024006','1994-029BB','deb HAPS','',65425,'Reentered','1998-08-28 00:00:00','1996-07-29 00:00:00',98.72,505,890,82.13,'Intermediate' from dual union all
select 24007,'024007','1994-029BC','deb HAPS','',65425,'In Earth orbit','','1996-07-30 00:00:00',97.99,581,745,81.93,'Intermediate' from dual union all
select 24008,'024008','1994-029BD','deb HAPS','',65425,'Reentered','1998-12-07 00:00:00','1996-07-29 00:00:00',98.03,530,799,82.14,'Intermediate' from dual union all
select 24009,'024009','1994-029BE','deb HAPS','',65425,'Reentered','1998-07-14 00:00:00','1996-07-29 00:00:00',99.41,578,882,82.04,'Intermediate' from dual union all
select 2401,'002401','1966-076A','NNS O-10','NNS O-10',13483,'In Earth orbit','','1966-10-27 00:00:00',106.76,1052,1102,88.86,'Polar' from dual union all
select 24010,'024010','1994-029BF','deb HAPS','',65425,'Reentered','1997-03-23 00:00:00','1996-07-29 00:00:00',98.42,394,973,82.29,'Intermediate' from dual union all
select 24011,'024011','1994-029BG','deb HAPS','',65425,'Reentered','1996-12-23 00:00:00','1996-07-31 00:00:00',99.08,504,926,82.11,'Intermediate' from dual union all
select 24012,'024012','1994-029BH','deb HAPS','',65425,'Reentered','1998-10-03 00:00:00','1996-07-31 00:00:00',98.83,607,799,81.72,'Intermediate' from dual union all
select 24013,'024013','1994-029BJ','deb HAPS','',65425,'Reentered','1999-05-11 00:00:00','1996-07-31 00:00:00',97.47,579,697,82.09,'Intermediate' from dual union all
select 24014,'024014','1994-029BK','deb HAPS','',65425,'Reentered','2001-12-10 00:00:00','1996-07-31 00:00:00',100.4,605,950,82.03,'Intermediate' from dual union all
select 24015,'024015','1994-029BL','deb HAPS','',65425,'Reentered','1997-09-20 00:00:00','1996-07-31 00:00:00',99.32,593,860,81.72,'Intermediate' from dual union all
select 24016,'024016','1994-029BM','deb HAPS','',65425,'Reentered','2001-10-01 00:00:00','1996-07-31 00:00:00',101.17,606,1022,81.97,'Intermediate' from dual union all
select 24017,'024017','1994-029BN','deb HAPS','',65425,'Reentered','1997-06-06 00:00:00','1996-07-31 00:00:00',98.74,566,831,81.89,'Intermediate' from dual union all
select 24018,'024018','1994-029BP','deb HAPS','',65425,'Reentered','1996-11-19 00:00:00','1996-07-28 00:00:00',97.12,482,759,82.19,'Intermediate' from dual union all
select 24019,'024019','1994-029BQ','deb HAPS','',65425,'Reentered','1999-01-30 00:00:00','1996-07-31 00:00:00',99.18,599,840,81.93,'Intermediate' from dual union all
select 2402,'002402','1966-075C','FW4D','FW4D S/N 00002',13482,'Deep Space','1966-08-26 00:00:00','1966-08-17 00:00:00',18492.83,369,476421,42.04,'Earth Escape' from dual union all
select 24020,'024020','1994-029BR','deb HAPS','',65425,'Reentered','1997-02-20 00:00:00','1996-07-31 00:00:00',96.71,526,677,81.83,'Intermediate' from dual union all
select 24021,'024021','1994-029BS','deb HAPS','',65425,'Reentered','1997-01-15 00:00:00','1996-07-29 00:00:00',98.18,493,851,82.19,'Intermediate' from dual union all
select 24022,'024022','1994-029BT','deb HAPS','',65425,'Reentered','1996-09-04 00:00:00','1996-07-28 00:00:00',94.92,450,579,81.76,'Intermediate' from dual union all
select 24023,'024023','1994-029BU','deb HAPS','',65425,'Reentered','1997-06-01 00:00:00','1996-07-30 00:00:00',100.06,549,973,82.07,'Intermediate' from dual union all
select 24024,'024024','1994-029BV','deb HAPS','',65425,'Reentered','2000-01-24 00:00:00','1996-07-31 00:00:00',102.48,592,1160,82.27,'Intermediate' from dual union all
select 24025,'024025','1994-029BW','deb HAPS','',65425,'Reentered','2002-05-08 00:00:00','1996-07-31 00:00:00',106.35,615,1500,81.69,'Intermediate' from dual union all
select 24026,'024026','1994-029BX','deb HAPS','',65425,'Reentered','1996-12-30 00:00:00','1996-07-31 00:00:00',97.82,482,827,82.2,'Intermediate' from dual union all
select 24027,'024027','1994-029BY','deb HAPS','',65425,'Reentered','1996-12-15 00:00:00','1996-07-31 00:00:00',98.02,467,861,82.1,'Intermediate' from dual union all
select 24028,'024028','1994-029BZ','deb HAPS','',65425,'Reentered','1996-09-30 00:00:00','1996-07-31 00:00:00',96.16,460,690,81.86,'Intermediate' from dual union all
select 24029,'024029','1994-029CA','deb HAPS','',65425,'Reentered','1997-04-07 00:00:00','1996-07-30 00:00:00',99,532,890,82.05,'Intermediate' from dual union all
select 2403,'002403','1966-077A','FTV 1352','Midas RTS-I F2',13484,'In Earth orbit','','1966-11-05 00:00:00',167.52,3676,3701,90.11,'Medium' from dual union all
select 24030,'024030','1994-029CB','deb HAPS','',65425,'Reentered','1998-10-27 00:00:00','1996-07-31 00:00:00',99.19,525,915,82.39,'Intermediate' from dual union all
select 24031,'024031','1994-029CC','deb HAPS','',65425,'Reentered','1996-11-21 00:00:00','1996-07-29 00:00:00',96.82,517,696,81.81,'Intermediate' from dual union all
select 24032,'024032','1994-029CD','deb HAPS','',65425,'Reentered','2000-09-08 00:00:00','1996-07-31 00:00:00',101.91,592,1106,82.22,'Intermediate' from dual union all
select 24033,'024033','1994-029CE','deb HAPS','',65425,'Reentered','2000-04-28 00:00:00','1996-07-31 00:00:00',98.45,573,797,81.97,'Intermediate' from dual union all
select 24034,'024034','1994-029CF','deb HAPS','',65425,'Reentered','1999-12-12 00:00:00','1996-07-31 00:00:00',98.59,575,808,82,'Intermediate' from dual union all
select 24035,'024035','1994-029CG','deb HAPS','',65425,'Reentered','1998-04-15 00:00:00','1996-07-31 00:00:00',95.58,464,630,81.9,'Intermediate' from dual union all
select 24036,'024036','1994-029CH','deb HAPS','',65425,'Reentered','1997-03-19 00:00:00','1996-07-31 00:00:00',96.87,461,757,81.98,'Intermediate' from dual union all
select 24037,'024037','1994-029CJ','deb HAPS','',65425,'Reentered','1996-09-21 00:00:00','1996-07-31 00:00:00',96.72,425,779,82.19,'Intermediate' from dual;
insert into SATELLITE
select 24038,'024038','1994-029CK','deb HAPS','',65425,'Reentered','1999-01-26 00:00:00','1996-07-31 00:00:00',99.39,565,894,82.09,'Intermediate' from dual union all
select 24039,'024039','1994-029CL','deb HAPS','',65425,'Reentered','1997-12-08 00:00:00','1996-07-31 00:00:00',96.19,524,629,82.44,'Intermediate' from dual union all
select 2404,'002404','1966-070B','FW4S S/N 30105','FW4S',13477,'Reentered','1992-01-03 00:00:00','1966-10-26 00:00:00',136.78,359,4473,81.48,'Medium' from dual union all
select 24040,'024040','1994-029CM','deb HAPS','',65425,'Reentered','1997-05-30 00:00:00','1996-07-30 00:00:00',98.58,587,795,81.7,'Intermediate' from dual union all
select 24041,'024041','1994-029CN','deb HAPS','',65425,'Reentered','1998-04-06 00:00:00','1996-07-31 00:00:00',95.88,513,610,82.2,'Intermediate' from dual union all
select 24042,'024042','1994-029CP','deb HAPS','',65425,'Reentered','1999-05-06 00:00:00','1996-07-31 00:00:00',97.84,526,785,82.01,'Intermediate' from dual union all
select 24043,'024043','1994-029CQ','deb HAPS','',65425,'Reentered','2000-08-03 00:00:00','1996-07-31 00:00:00',99.44,569,895,82.31,'Intermediate' from dual union all
select 24044,'024044','1994-029CR','deb HAPS','',65425,'Reentered','1997-09-22 00:00:00','1996-07-30 00:00:00',99.81,561,938,82.02,'Intermediate' from dual union all
select 24045,'024045','1994-029CS','deb HAPS','',65425,'Reentered','1996-12-28 00:00:00','1996-07-30 00:00:00',98.05,492,839,82.15,'Intermediate' from dual union all
select 24046,'024046','1994-029CT','deb HAPS','',65425,'Reentered','1997-08-02 00:00:00','1996-07-30 00:00:00',98.99,582,839,81.74,'Intermediate' from dual union all
select 24047,'024047','1994-029CU','deb HAPS','',65425,'Reentered','1996-11-07 00:00:00','1996-07-30 00:00:00',96.11,500,645,82.09,'Intermediate' from dual union all
select 24048,'024048','1994-029CV','deb HAPS','',65425,'Reentered','1996-11-22 00:00:00','1996-07-29 00:00:00',96.94,503,722,81.91,'Intermediate' from dual union all
select 24049,'024049','1994-029CW','deb HAPS','',65425,'Reentered','1997-09-02 00:00:00','1996-07-30 00:00:00',99.81,558,941,81.98,'Intermediate' from dual union all
select 2405,'002405','1966-075D','Delta 40 attach clamp','',13482,'Reentered','1967-02-26 00:00:00','1966-11-06 00:00:00',95.27,217,847,32.98,'Intermediate' from dual union all
select 24050,'024050','1994-029CX','deb HAPS','',65425,'Reentered','1997-01-30 00:00:00','1996-07-31 00:00:00',97.11,541,700,81.85,'Intermediate' from dual union all
select 24051,'024051','1994-029CY','deb HAPS','',65425,'In Earth orbit','','1996-07-31 00:00:00',106.74,589,1563,81.31,'Intermediate' from dual union all
select 24052,'024052','1994-029CZ','deb HAPS','',65425,'Reentered','1997-04-15 00:00:00','1996-07-31 00:00:00',98.61,528,857,81.9,'Intermediate' from dual union all
select 24053,'024053','1994-029DA','deb HAPS','',65425,'Reentered','1997-03-04 00:00:00','1996-07-29 00:00:00',98.98,516,904,82.03,'Intermediate' from dual union all
select 24054,'024054','1994-029DB','deb HAPS','',65425,'Reentered','2000-02-14 00:00:00','1996-07-31 00:00:00',99.82,596,904,82.04,'Intermediate' from dual union all
select 24055,'024055','1994-029DC','deb HAPS','',65425,'Reentered','1997-09-13 00:00:00','1996-07-31 00:00:00',99,582,840,81.81,'Intermediate' from dual union all
select 24056,'024056','1994-029DD','deb HAPS','',65425,'Reentered','1996-11-10 00:00:00','1996-07-31 00:00:00',97.68,482,813,82.27,'Intermediate' from dual union all
select 24057,'024057','1994-029DE','deb HAPS','',65425,'Reentered','1997-10-25 00:00:00','1996-07-31 00:00:00',96.65,427,769,81.53,'Intermediate' from dual union all
select 24058,'024058','1994-029DF','deb HAPS','',65425,'Reentered','1998-10-17 00:00:00','1996-08-01 00:00:00',101.04,605,1011,81.61,'Intermediate' from dual union all
select 24059,'024059','1994-029DG','deb HAPS','',65425,'Reentered','1998-09-01 00:00:00','1996-08-01 00:00:00',96.9,532,689,82.03,'Intermediate' from dual union all
select 2406,'002406','1966-078A','Luna-11','E-6LF No. 101',13485,'Deep Space','1966-08-26 00:00:00','1966-08-24 00:00:00',21544.43,200,500000,51.8,'Cislunar' from dual union all
select 24060,'024060','1994-029DH','deb HAPS','',65425,'Reentered','1996-11-13 00:00:00','1996-07-29 00:00:00',96.89,486,734,81.88,'Intermediate' from dual union all
select 24061,'024061','1994-029DJ','deb HAPS','',65425,'Reentered','1997-02-10 00:00:00','1996-08-01 00:00:00',98.45,497,873,81.97,'Intermediate' from dual union all
select 24062,'024062','1994-029DK','deb HAPS','',65425,'Reentered','2000-04-07 00:00:00','1996-08-01 00:00:00',100.23,601,937,82.03,'Intermediate' from dual union all
select 24063,'024063','1994-029DL','deb HAPS','',65425,'Reentered','1996-10-21 00:00:00','1996-08-01 00:00:00',95.59,504,591,82.27,'Intermediate' from dual union all
select 24064,'024064','1994-029DM','deb HAPS','',65425,'Reentered','1999-11-23 00:00:00','1996-07-31 00:00:00',98.49,610,763,81.96,'Intermediate' from dual union all
select 24065,'024065','1994-029DN','deb HAPS','',65425,'Reentered','1998-02-18 00:00:00','1996-08-01 00:00:00',102.09,570,1145,82.19,'Intermediate' from dual union all
select 24066,'024066','1994-029DP','deb HAPS','',65425,'Reentered','1997-11-15 00:00:00','1996-07-31 00:00:00',101.32,581,1062,82.01,'Intermediate' from dual union all
select 24067,'024067','1994-029DQ','deb HAPS','',65425,'Reentered','1996-12-12 00:00:00','1996-08-01 00:00:00',97.57,500,785,81.93,'Intermediate' from dual union all
select 24068,'024068','1994-029DR','deb HAPS','',65425,'Reentered','2001-09-08 00:00:00','1996-08-01 00:00:00',101.04,604,1012,81.99,'Intermediate' from dual union all
select 24069,'024069','1994-029DS','deb HAPS','',65425,'Reentered','1998-11-29 00:00:00','1996-08-01 00:00:00',103.42,590,1251,82.16,'Intermediate' from dual union all
select 2407,'002407','1966-078B','11S510','11S510',13485,'Reentered','1966-08-25 00:00:00','1966-08-25 00:00:00',88.16,177,189,51.9,'Intermediate' from dual union all
select 24070,'024070','1994-029DT','deb HAPS','',65425,'Reentered','2002-12-20 00:00:00','1996-08-01 00:00:00',101.61,609,1061,82.15,'Intermediate' from dual union all
select 24071,'024071','1996-043A','Progress M-32','Progress 7K-TGM No. 232',65931,'Deorbited','1996-11-20 00:00:00','1996-08-30 00:00:00',92.21,375,391,51.65,'Intermediate' from dual union all
select 24072,'024072','1996-043B','11S510','11S510',65931,'Reentered','1996-08-03 00:00:00','1996-08-02 00:00:00',88.21,176,196,51.6,'Intermediate' from dual union all
select 24073,'024073','1994-029DU','deb HAPS','',65425,'Reentered','1997-04-13 00:00:00','1996-07-31 00:00:00',99.17,518,920,82.03,'Intermediate' from dual union all
select 24074,'024074','1994-029DV','deb HAPS','',65425,'Reentered','1997-03-20 00:00:00','1996-08-01 00:00:00',98.67,573,818,81.66,'Intermediate' from dual union all
select 24075,'024075','1994-029DW','deb HAPS','',65425,'Reentered','1999-04-11 00:00:00','1996-08-01 00:00:00',98.59,595,788,81.89,'Intermediate' from dual union all
select 24076,'024076','1994-029DX','deb HAPS','',65425,'Reentered','2009-03-22 00:00:00','1996-08-01 00:00:00',99.19,602,838,81.92,'Intermediate' from dual union all
select 24077,'024077','1994-029DY','deb HAPS','',65425,'Reentered','1998-11-06 00:00:00','1996-08-01 00:00:00',96.84,573,642,81.88,'Intermediate' from dual union all
select 24078,'024078','1994-029DZ','deb HAPS','',65425,'Reentered','1998-03-07 00:00:00','1996-08-01 00:00:00',96.17,407,744,82.06,'Intermediate' from dual union all
select 24079,'024079','1994-029EA','deb HAPS','',65425,'Reentered','1998-03-29 00:00:00','1996-08-01 00:00:00',96.57,437,751,82.31,'Intermediate' from dual union all
select 2408,'002408','1966-078C','BOZ','BOZ',13485,'Reentered','1966-08-26 00:00:00','1966-08-25 00:00:00',88.43,193,201,51.84,'Intermediate' from dual union all
select 24080,'024080','1994-029EB','deb HAPS','',65425,'Reentered','2014-08-11 00:00:00','1996-08-01 00:00:00',97.79,554,752,81.94,'Intermediate' from dual union all
select 24081,'024081','1994-029EC','deb HAPS','',65425,'Reentered','2002-03-19 00:00:00','1996-08-01 00:00:00',98.07,599,734,82.08,'Intermediate' from dual union all
select 24082,'024082','1994-029ED','deb HAPS','',65425,'In Earth orbit','','1996-08-01 00:00:00',98.8,602,801,81.93,'Intermediate' from dual union all
select 24083,'024083','1994-029EE','deb HAPS','',65425,'Reentered','1998-05-03 00:00:00','1996-08-01 00:00:00',97.39,573,695,81.82,'Intermediate' from dual union all
select 24084,'024084','1994-029EF','deb HAPS','',65425,'Reentered','1999-12-08 00:00:00','1996-08-01 00:00:00',97.61,576,713,81.84,'Intermediate' from dual union all
select 24085,'024085','1994-029EG','deb HAPS','',65425,'Reentered','2000-07-12 00:00:00','1996-08-01 00:00:00',97.68,580,716,81.86,'Intermediate' from dual union all
select 24086,'024086','1994-029EH','deb HAPS','',65425,'Reentered','1999-01-05 00:00:00','1996-08-01 00:00:00',97.88,555,760,81.93,'Intermediate' from dual union all
select 24087,'024087','1994-029EJ','deb HAPS','',65425,'Reentered','1998-11-10 00:00:00','1996-08-01 00:00:00',98.28,539,814,82.03,'Intermediate' from dual union all
select 24088,'024088','1994-029EK','deb HAPS','',65425,'Reentered','1999-06-20 00:00:00','1996-08-01 00:00:00',98.52,561,814,82.07,'Intermediate' from dual union all
select 24089,'024089','1994-029EL','deb HAPS','',65425,'Reentered','2000-12-30 00:00:00','1996-08-01 00:00:00',98.63,568,819,81.99,'Intermediate' from dual union all
select 2409,'002409','1966-079A','Kosmos-128','Zenit-4 No. 23',13486,'Landed','1966-09-04 00:00:00','1966-08-27 00:00:00',89.82,213,318,64.99,'Intermediate' from dual union all
select 24090,'024090','1994-029EM','deb HAPS','',65425,'Reentered','1998-10-29 00:00:00','1996-08-01 00:00:00',96.08,490,652,81.96,'Intermediate' from dual union all
select 24091,'024091','1994-029EN','deb HAPS','',65425,'In Earth orbit','','1996-08-01 00:00:00',98.16,580,761,82.01,'Intermediate' from dual union all
select 24092,'024092','1994-029EP','deb HAPS','',65425,'Reentered','2016-02-15 00:00:00','1996-08-01 00:00:00',101.09,591,1029,82.08,'Intermediate' from dual union all
select 24093,'024093','1994-029EQ','deb HAPS','',65425,'Reentered','2000-06-13 00:00:00','1996-08-01 00:00:00',96.8,496,715,81.86,'Intermediate' from dual union all
select 24094,'024094','1994-029ER','deb HAPS','',65425,'Reentered','2015-04-19 00:00:00','1996-08-01 00:00:00',97.77,554,750,81.95,'Intermediate' from dual union all
select 24095,'024095','1994-029ES','deb HAPS','',65425,'Reentered','2003-02-06 00:00:00','1996-08-01 00:00:00',97.84,576,735,81.88,'Intermediate' from dual union all
select 24096,'024096','1994-029ET','deb HAPS','',65425,'Reentered','1999-07-06 00:00:00','1996-08-01 00:00:00',99.43,572,891,82.05,'Intermediate' from dual union all
select 24097,'024097','1994-029EU','deb HAPS','',65425,'In Earth orbit','','1996-08-01 00:00:00',99.61,597,883,82.02,'Intermediate' from dual union all
select 24098,'024098','1994-029EV','deb HAPS','',65425,'Reentered','1998-05-30 00:00:00','1996-08-01 00:00:00',97.2,542,708,81.94,'Intermediate' from dual union all
select 24099,'024099','1994-029EW','deb HAPS','',65425,'Reentered','2001-04-07 00:00:00','1996-08-01 00:00:00',97.94,564,757,81.92,'Intermediate' from dual union all
select 241,'000241','62 Gamma 2','Atlas D 109D','Atlas D 109D',8190,'Reentered','1962-02-20 00:00:00','1962-02-20 00:00:00',88.39,148,242,32.54,'Intermediate' from dual union all
select 2410,'002410','1966-079B','11A57I','11A57I',13486,'Reentered','1966-09-07 00:00:00','1966-08-28 00:00:00',90.02,209,342,64.98,'Intermediate' from dual union all
select 24100,'024100','1994-029EX','deb HAPS','',65425,'Reentered','2000-10-23 00:00:00','1996-08-01 00:00:00',97.85,577,735,81.89,'Intermediate' from dual union all
select 24101,'024101','1994-029EY','deb HAPS','',65425,'Reentered','1998-12-31 00:00:00','1996-08-01 00:00:00',97.94,589,732,81.87,'Intermediate' from dual union all
select 24102,'024102','1994-029EZ','deb HAPS','',65425,'In Earth orbit','','1996-08-01 00:00:00',97.98,594,731,81.88,'Intermediate' from dual union all
select 24103,'024103','1994-029FA','deb HAPS','',65425,'Reentered','2002-09-24 00:00:00','1996-08-03 00:00:00',97.85,563,749,81.93,'Intermediate' from dual union all
select 24104,'024104','1994-029FB','deb HAPS','',65425,'Reentered','1998-08-20 00:00:00','1996-08-03 00:00:00',96.53,559,626,82.11,'Intermediate' from dual union all
select 24105,'024105','1994-029FC','deb HAPS','',65425,'Reentered','1996-12-18 00:00:00','1996-08-03 00:00:00',96.42,535,640,81.89,'Intermediate' from dual union all
select 24106,'024106','1994-029FD','deb HAPS','',65425,'Reentered','2001-09-26 00:00:00','1996-08-03 00:00:00',98.85,597,811,81.96,'Intermediate' from dual union all
select 24107,'024107','1994-029FE','deb HAPS','',65425,'Reentered','2001-11-03 00:00:00','1996-08-03 00:00:00',104.99,611,1377,81.71,'Intermediate' from dual union all
select 24108,'024108','1994-029FF','deb HAPS','',65425,'Reentered','2011-12-17 00:00:00','1996-08-03 00:00:00',98.08,534,800,82.04,'Intermediate' from dual union all
select 24109,'024109','1994-029FG','deb HAPS','',65425,'Reentered','1999-12-17 00:00:00','1996-08-02 00:00:00',98.34,566,792,81.99,'Intermediate' from dual union all
select 2411,'002411','1966-077B','EGRS VII','Secor Type II S/N 8',13484,'In Earth orbit','','1966-11-08 00:00:00',167.5,3675,3700,90.11,'Medium' from dual union all
select 24110,'024110','1994-029FH','deb HAPS','',65425,'Reentered','1999-12-23 00:00:00','1996-08-02 00:00:00',97.18,582,666,81.9,'Intermediate' from dual union all
select 24111,'024111','1994-029FJ','deb HAPS','',65425,'Reentered','2000-01-20 00:00:00','1996-08-03 00:00:00',97.67,584,711,82.08,'Intermediate' from dual union all
select 24112,'024112','1994-029FK','deb HAPS','',65425,'Reentered','2000-03-25 00:00:00','1996-08-02 00:00:00',98.72,568,827,82.01,'Intermediate' from dual union all
select 24113,'024113','1994-029FL','deb HAPS','',65425,'Reentered','2000-04-27 00:00:00','1996-08-03 00:00:00',99.58,567,910,81.9,'Intermediate' from dual union all
select 24114,'024114','1994-029FM','deb HAPS','',65425,'Reentered','2015-08-05 00:00:00','1996-08-03 00:00:00',97.99,591,735,82.07,'Intermediate' from dual union all
select 24115,'024115','1994-029FN','deb HAPS','',65425,'Reentered','2015-11-13 00:00:00','1996-08-03 00:00:00',98.09,600,735,82.09,'Intermediate' from dual union all
select 24116,'024116','1994-029FP','deb HAPS','',65425,'Reentered','2000-11-13 00:00:00','1996-08-03 00:00:00',98.19,542,802,82.04,'Intermediate' from dual union all
select 24117,'024117','1994-029FQ','deb HAPS','',65425,'Reentered','2000-11-22 00:00:00','1996-08-03 00:00:00',100.08,600,925,82.03,'Intermediate' from dual union all
select 24118,'024118','1994-029FR','deb HAPS','',65425,'Reentered','1999-08-09 00:00:00','1996-08-03 00:00:00',99.02,605,819,81.82,'Intermediate' from dual union all
select 24119,'024119','1994-029FS','deb HAPS','',65425,'Reentered','1998-01-19 00:00:00','1996-08-03 00:00:00',97.32,483,777,82.1,'Intermediate' from dual union all
select 2412,'002412','1966-077C','ERS 15','ERS 15',13484,'In Earth orbit','','1966-11-05 00:00:00',167.61,3684,3700,90.11,'Medium' from dual union all
select 24120,'024120','1994-029FT','deb HAPS','',65425,'Reentered','1997-12-11 00:00:00','1996-08-03 00:00:00',100.02,616,903,82.45,'Intermediate' from dual union all
select 24121,'024121','1994-029FU','deb HAPS','',65425,'Reentered','1999-09-21 00:00:00','1996-08-03 00:00:00',100.42,610,946,81.32,'Intermediate' from dual union all
select 24122,'024122','1994-029FV','deb HAPS','',65425,'Reentered','2001-10-10 00:00:00','1996-08-03 00:00:00',100.54,584,984,81.77,'Intermediate' from dual union all
select 24123,'024123','1994-029FW','deb HAPS','',65425,'Reentered','2004-10-16 00:00:00','1996-08-03 00:00:00',101.67,600,1076,82.02,'Intermediate' from dual union all
select 24124,'024124','1994-029FX','deb HAPS','',65425,'Reentered','2003-04-01 00:00:00','1996-08-03 00:00:00',102.4,588,1157,82.29,'Intermediate' from dual union all
select 24125,'024125','1994-029FY','deb HAPS','',65425,'Reentered','1998-09-03 00:00:00','1996-08-03 00:00:00',103.69,520,1346,83,'Intermediate' from dual union all
select 24126,'024126','1994-029FZ','deb HAPS','',65425,'In Earth orbit','','1996-08-03 00:00:00',104.54,612,1334,81.85,'Intermediate' from dual union all
select 24127,'024127','1994-029GA','deb HAPS','',65425,'Reentered','1998-07-31 00:00:00','1996-08-03 00:00:00',104.48,514,1427,83.01,'Intermediate' from dual;
insert into SATELLITE
select 24128,'024128','1994-029GB','deb HAPS','',65425,'Reentered','1999-10-31 00:00:00','1996-08-03 00:00:00',105.03,605,1387,82.03,'Intermediate' from dual union all
select 24129,'024129','1994-029GC','deb HAPS','',65425,'In Earth orbit','','1996-08-03 00:00:00',106.91,555,1612,81.49,'Intermediate' from dual union all
select 2413,'002413','1966-076B','Altair 2 X-258E6 RH-136','X-258RHE6',13483,'In Earth orbit','','1966-10-05 00:00:00',106.76,1050,1104,88.85,'Polar' from dual union all
select 24130,'024130','1994-029GD','deb HAPS','',65425,'Reentered','2002-03-06 00:00:00','1996-08-03 00:00:00',101.76,611,1072,81.92,'Intermediate' from dual union all
select 24131,'024131','1994-029GE','deb HAPS','',65425,'Reentered','2012-11-21 00:00:00','1996-08-03 00:00:00',100.61,607,968,82.06,'Intermediate' from dual union all
select 24132,'024132','1994-029GF','deb HAPS','',65425,'Reentered','2002-01-23 00:00:00','1996-08-03 00:00:00',101.68,612,1065,81.89,'Intermediate' from dual union all
select 24133,'024133','1994-029GG','deb HAPS','',65425,'Reentered','2001-12-29 00:00:00','1996-08-03 00:00:00',101.45,609,1046,81.95,'Intermediate' from dual union all
select 24134,'024134','1994-029GH','deb HAPS','',65425,'Reentered','2002-01-05 00:00:00','1996-08-03 00:00:00',101.54,612,1052,81.89,'Intermediate' from dual union all
select 24135,'024135','1994-029GJ','deb HAPS','',65425,'Reentered','2002-03-12 00:00:00','1996-08-03 00:00:00',101.68,613,1064,81.87,'Intermediate' from dual union all
select 24136,'024136','1994-029GK','deb HAPS','',65425,'Reentered','2002-04-04 00:00:00','1996-08-03 00:00:00',101.73,613,1069,81.88,'Intermediate' from dual union all
select 24137,'024137','1994-029GL','deb HAPS','',65425,'Reentered','1999-09-05 00:00:00','1996-08-03 00:00:00',102.37,602,1140,82.06,'Intermediate' from dual union all
select 24138,'024138','1994-029GM','deb HAPS','',65425,'In Earth orbit','','1996-08-03 00:00:00',102.44,603,1146,81.58,'Intermediate' from dual union all
select 24139,'024139','1994-029GN','deb HAPS','',65425,'Reentered','2000-03-29 00:00:00','1996-08-03 00:00:00',97.8,591,715,82.08,'Intermediate' from dual union all
select 2414,'002414','1966-080A','TDA 6','TDA 6',13487,'Reentered Attached','1966-12-30 00:00:00','1966-11-08 00:00:00',90.76,309,315,28.83,'Intermediate' from dual union all
select 24140,'024140','1994-029GP','deb HAPS','',65425,'Reentered','2000-12-26 00:00:00','1996-08-03 00:00:00',97.86,587,726,82.09,'Intermediate' from dual union all
select 24141,'024141','1994-029GQ','deb HAPS','',65425,'Reentered','1999-05-06 00:00:00','1996-08-03 00:00:00',100.68,607,975,81.76,'Intermediate' from dual union all
select 24142,'024142','1994-029GR','deb HAPS','',65425,'Reentered','1998-11-09 00:00:00','1996-08-03 00:00:00',103.61,593,1266,82,'Intermediate' from dual union all
select 24143,'024143','1994-029GS','deb HAPS','',65425,'Reentered','2000-03-23 00:00:00','1996-08-03 00:00:00',98.76,562,837,82.22,'Intermediate' from dual union all
select 24144,'024144','1994-029GT','deb HAPS','',65425,'Reentered','1998-04-06 00:00:00','1996-08-03 00:00:00',96.95,601,624,81.7,'Intermediate' from dual union all
select 24145,'024145','1994-029GU','deb HAPS','',65425,'Reentered','1999-08-04 00:00:00','1996-08-03 00:00:00',97.64,580,712,81.87,'Intermediate' from dual union all
select 24146,'024146','1994-029GV','deb HAPS','',65425,'Reentered','2001-10-01 00:00:00','1996-08-03 00:00:00',99.63,565,917,81.31,'Intermediate' from dual union all
select 24147,'024147','1994-029GW','deb HAPS','',65425,'Reentered','2000-06-22 00:00:00','1996-08-03 00:00:00',98.42,610,756,82.07,'Intermediate' from dual union all
select 24148,'024148','1994-029GX','deb HAPS','',65425,'Reentered','2000-02-04 00:00:00','1996-08-07 00:00:00',98.7,598,794,82.13,'Intermediate' from dual union all
select 24149,'024149','1994-029GY','deb HAPS','',65425,'Reentered','2001-11-24 00:00:00','1996-08-07 00:00:00',101.12,605,1019,81.97,'Intermediate' from dual union all
select 2415,'002415','1966-081A','Gemini XI','Gemini SC11',13488,'Landed','1966-09-15 00:00:00','1966-09-14 00:00:00',101.57,298,1368,28.83,'Intermediate' from dual union all
select 24150,'024150','1994-029GZ','deb HAPS','',65425,'Reentered','2000-04-02 00:00:00','1996-08-06 00:00:00',98.74,568,828,81.97,'Intermediate' from dual union all
select 24151,'024151','1994-029HA','deb HAPS','',65425,'In Earth orbit','','1996-08-07 00:00:00',99.75,604,890,82.03,'Intermediate' from dual union all
select 24152,'024152','1994-029HB','deb HAPS','',65425,'Reentered','1999-11-01 00:00:00','1996-08-07 00:00:00',98.21,528,819,82.09,'Intermediate' from dual union all
select 24153,'024153','1994-029HC','deb HAPS','',65425,'Reentered','2000-04-25 00:00:00','1996-08-06 00:00:00',98.67,572,818,82.01,'Intermediate' from dual union all
select 24154,'024154','1994-029HD','deb HAPS','',65425,'Reentered','2000-12-29 00:00:00','1996-08-06 00:00:00',99.93,603,908,82.01,'Intermediate' from dual union all
select 24155,'024155','1994-029HE','deb HAPS','',65425,'Reentered','2002-01-10 00:00:00','1996-08-07 00:00:00',101.58,609,1058,81.93,'Intermediate' from dual union all
select 24156,'024156','1994-029HF','deb HAPS','',65425,'Reentered','2003-02-25 00:00:00','1996-08-07 00:00:00',103.45,595,1249,82.21,'Intermediate' from dual union all
select 24157,'024157','1994-029HG','deb HAPS','',65425,'Reentered','1996-11-11 00:00:00','1996-08-06 00:00:00',96.72,512,692,81.8,'Intermediate' from dual union all
select 24158,'024158','1994-029HH','deb HAPS','',65425,'Reentered','2001-07-02 00:00:00','1996-08-07 00:00:00',97.05,527,707,81.92,'Intermediate' from dual union all
select 24159,'024159','1994-029HJ','deb HAPS','',65425,'Reentered','1999-01-08 00:00:00','1996-08-07 00:00:00',97.37,575,691,82.29,'Intermediate' from dual union all
select 2416,'002416','1966-081B','Titan GT-11 Stage 2','LR91AJ5 GT-11 65-12566',13488,'Reentered','1966-09-13 00:00:00','1966-09-12 00:00:00',87.82,144,190,28.8,'Intermediate' from dual union all
select 24160,'024160','1994-029HK','deb HAPS','',65425,'Reentered','1996-10-18 00:00:00','1996-08-06 00:00:00',96.56,449,739,82.28,'Intermediate' from dual union all
select 24161,'024161','1994-029HL','deb HAPS','',65425,'Reentered','2014-04-19 00:00:00','1996-08-07 00:00:00',97.75,574,729,81.86,'Intermediate' from dual union all
select 24162,'024162','1994-029HM','deb HAPS','',65425,'Reentered','1999-06-07 00:00:00','1996-08-07 00:00:00',97.73,466,835,81.6,'Intermediate' from dual union all
select 24163,'024163','1994-029HN','deb HAPS','',65425,'In Earth orbit','','1996-08-07 00:00:00',98.24,602,747,82.08,'Intermediate' from dual union all
select 24164,'024164','1994-029HP','deb HAPS','',65425,'Reentered','2000-12-01 00:00:00','1996-08-07 00:00:00',100.73,566,1021,81.71,'Intermediate' from dual union all
select 24165,'024165','1994-029HQ','deb HAPS','',65425,'In Earth orbit','','1996-08-07 00:00:00',104.65,614,1343,82.21,'Intermediate' from dual union all
select 24166,'024166','1994-029HR','deb HAPS','',65425,'Reentered','2002-11-09 00:00:00','1996-08-07 00:00:00',105.4,481,1545,81.16,'Intermediate' from dual union all
select 24167,'024167','1994-029HS','deb HAPS','',65425,'Reentered','1997-11-23 00:00:00','1996-08-07 00:00:00',97.77,421,882,82.28,'Intermediate' from dual union all
select 24168,'024168','1994-029HT','deb HAPS','',65425,'Reentered','1999-10-12 00:00:00','1996-08-06 00:00:00',98.14,591,749,82.08,'Intermediate' from dual union all
select 24169,'024169','1994-029HU','deb HAPS','',65425,'Reentered','1998-07-25 00:00:00','1996-08-07 00:00:00',96.98,513,716,81.93,'Intermediate' from dual union all
select 2417,'002417','1966-081C','Gemini SC11 Equipment Module','Gemini SC11 Adapter Modu',13488,'Reentered','1966-10-08 00:00:00','1966-09-15 00:00:00',90.18,275,292,28.3,'Intermediate' from dual union all
select 24170,'024170','1994-029HV','deb HAPS','',65425,'Reentered','1997-01-05 00:00:00','1996-08-05 00:00:00',96.53,546,639,82.14,'Intermediate' from dual union all
select 24171,'024171','1994-029HW','deb HAPS','',65425,'Reentered','1998-09-27 00:00:00','1996-08-07 00:00:00',97.92,400,919,82.29,'Intermediate' from dual union all
select 24172,'024172','1994-029HX','deb HAPS','',65425,'Reentered','2001-09-20 00:00:00','1996-08-07 00:00:00',98.72,539,856,82.14,'Intermediate' from dual union all
select 24173,'024173','1994-029HY','deb HAPS','',65425,'Reentered','1999-07-02 00:00:00','1996-08-07 00:00:00',99.97,503,1012,81.73,'Intermediate' from dual union all
select 24174,'024174','1994-029HZ','deb HAPS','',65425,'Reentered','2001-10-14 00:00:00','1996-08-06 00:00:00',100.67,611,970,81.79,'Intermediate' from dual union all
select 24175,'024175','1994-029JA','deb HAPS','',65425,'Reentered','2000-06-23 00:00:00','1996-08-07 00:00:00',103.01,610,1192,81.97,'Intermediate' from dual union all
select 24176,'024176','1994-029JB','deb HAPS','',65425,'Reentered','2001-01-27 00:00:00','1996-08-07 00:00:00',103.5,544,1304,81.86,'Intermediate' from dual union all
select 24177,'024177','1994-029JC','deb HAPS','',65425,'Reentered','1997-07-16 00:00:00','1996-08-06 00:00:00',99.58,533,944,82.03,'Intermediate' from dual union all
select 24178,'024178','1994-029JD','deb HAPS','',65425,'Reentered','1998-12-23 00:00:00','1996-08-07 00:00:00',99.18,535,904,82.26,'Intermediate' from dual union all
select 24179,'024179','1994-029JE','deb HAPS','',65425,'Reentered','2002-01-30 00:00:00','1996-08-07 00:00:00',101.65,613,1060,81.86,'Intermediate' from dual union all
select 2418,'002418','1966-082A','OPS 6026','DAPP 1416',13489,'In Earth orbit','','1966-11-03 00:00:00',100.81,694,900,98.49,'Sun-Synch' from dual union all
select 24180,'024180','1994-029JF','deb HAPS','',65425,'Reentered','1999-04-27 00:00:00','1996-08-07 00:00:00',103.52,591,1259,82.15,'Intermediate' from dual union all
select 24181,'024181','1994-029JG','deb HAPS','',65425,'Reentered','2000-12-31 00:00:00','1996-08-07 00:00:00',98.75,611,786,82.22,'Intermediate' from dual union all
select 24182,'024182','1994-029JH','deb HAPS','',65425,'Reentered','1997-07-10 00:00:00','1996-08-07 00:00:00',98.85,568,840,81.78,'Intermediate' from dual union all
select 24183,'024183','1994-029JJ','deb HAPS','',65425,'Reentered','1999-09-10 00:00:00','1996-08-07 00:00:00',99.65,580,904,82.15,'Intermediate' from dual union all
select 24184,'024184','1994-029JK','deb HAPS','',65425,'Reentered','2001-08-23 00:00:00','1996-08-07 00:00:00',100.02,605,914,82.01,'Intermediate' from dual union all
select 24185,'024185','1994-029JL','deb HAPS','',65425,'Reentered','1997-11-14 00:00:00','1996-08-06 00:00:00',100.82,556,1038,82.18,'Intermediate' from dual union all
select 24186,'024186','1994-029JM','deb HAPS','',65425,'Reentered','1998-08-26 00:00:00','1996-08-07 00:00:00',102.12,479,1239,82.27,'Intermediate' from dual union all
select 24187,'024187','1994-029JN','deb HAPS','',65425,'Reentered','2000-04-29 00:00:00','1996-08-07 00:00:00',97.87,591,723,82.13,'Intermediate' from dual union all
select 24188,'024188','1994-029JP','deb HAPS','',65425,'Reentered','1998-04-08 00:00:00','1996-08-07 00:00:00',99.08,595,835,81.75,'Intermediate' from dual union all
select 24189,'024189','1994-029JQ','deb HAPS','',65425,'Reentered','2000-08-04 00:00:00','1996-08-07 00:00:00',98.61,612,773,81.91,'Intermediate' from dual union all
select 2419,'002419','1966-083A','OPS 1686','GAMBIT SV 982',13490,'Deorbited','1966-09-23 00:00:00','1966-09-17 00:00:00',89.31,148,333,93.98,'Polar' from dual union all
select 24190,'024190','1994-029JR','deb HAPS','',65425,'Reentered','2000-08-19 00:00:00','1996-08-07 00:00:00',97.97,561,763,81.93,'Intermediate' from dual union all
select 24191,'024191','1994-029JS','deb HAPS','',65425,'Reentered','2002-02-07 00:00:00','1996-08-07 00:00:00',101.53,608,1055,81.96,'Intermediate' from dual union all
select 24192,'024192','1994-029JT','deb HAPS','',65425,'Reentered','1997-08-21 00:00:00','1996-08-07 00:00:00',98.17,539,803,82.04,'Intermediate' from dual union all
select 24193,'024193','1994-029JU','deb HAPS','',65425,'Reentered','1999-07-03 00:00:00','1996-08-06 00:00:00',98.08,534,800,82.06,'Intermediate' from dual union all
select 24194,'024194','1994-029JV','deb HAPS','',65425,'Reentered','1998-07-24 00:00:00','1996-08-07 00:00:00',97.39,541,727,81.89,'Intermediate' from dual union all
select 24195,'024195','1994-029JW','deb HAPS','',65425,'Reentered','2002-05-09 00:00:00','1996-08-07 00:00:00',105.21,479,1530,81.73,'Intermediate' from dual union all
select 24196,'024196','1994-029JX','deb HAPS','',65425,'Reentered','1999-11-07 00:00:00','1996-08-07 00:00:00',100.71,600,985,82.03,'Intermediate' from dual union all
select 24197,'024197','1994-029JY','deb HAPS','',65425,'In Earth orbit','','1996-08-07 00:00:00',98.62,605,781,82.1,'Intermediate' from dual union all
select 24198,'024198','1994-029JZ','deb HAPS','',65425,'Reentered','2002-03-09 00:00:00','1996-08-07 00:00:00',101.75,613,1070,81.87,'Intermediate' from dual union all
select 24199,'024199','1994-029KA','deb HAPS','',65425,'Reentered','2001-12-16 00:00:00','1996-08-07 00:00:00',110.85,607,1925,81.63,'Intermediate' from dual union all
select 242,'000242','62 Delta','FTV 2301','Project 102 Group 0-D',8187,'Reentered Attached','1962-03-09 00:00:00','1962-03-03 00:00:00',88.38,169,220,81.94,'Intermediate' from dual union all
select 2420,'002420','1966-083B','FANION II','P-11 No. 4406',13490,'Reentered','1968-05-09 00:00:00','1966-10-23 00:00:00',94.15,466,488,94.02,'Polar' from dual union all
select 24200,'024200','1994-029KB','deb HAPS','',65425,'Reentered','1996-11-22 00:00:00','1996-08-06 00:00:00',97.14,478,766,82.25,'Intermediate' from dual union all
select 24201,'024201','1994-029KC','deb HAPS','',65425,'Reentered','1998-08-08 00:00:00','1996-08-07 00:00:00',102.88,509,1281,82.98,'Intermediate' from dual union all
select 24202,'024202','1994-029KD','deb HAPS','',65425,'Reentered','1997-08-11 00:00:00','1996-08-07 00:00:00',98.31,564,792,81.84,'Intermediate' from dual union all
select 24203,'024203','1994-029KE','deb HAPS','',65425,'Reentered','1999-06-14 00:00:00','1996-08-07 00:00:00',98.01,590,737,82.07,'Intermediate' from dual union all
select 24204,'024204','1994-029KF','deb HAPS','',65425,'Reentered','2001-04-28 00:00:00','1996-08-07 00:00:00',99.88,514,992,81.98,'Intermediate' from dual union all
select 24205,'024205','1994-029KG','deb HAPS','',65425,'Reentered','1997-07-02 00:00:00','1996-08-07 00:00:00',96.86,424,793,82.26,'Intermediate' from dual union all
select 24206,'024206','1994-029KH','deb HAPS','',65425,'Reentered','1999-09-04 00:00:00','1996-08-07 00:00:00',103.46,604,1241,81.53,'Intermediate' from dual union all
select 24207,'024207','1994-029KJ','deb HAPS','',65425,'Reentered','2001-10-30 00:00:00','1996-08-07 00:00:00',98.51,613,762,81.81,'Intermediate' from dual union all
select 24208,'024208','1996-044A','Italsat F2','Italsat F2',65932,'In Earth orbit','','1996-09-07 00:00:00',1414.43,34928,35795,.08,'Drift GEO' from dual union all
select 24209,'024209','1996-044B','Telecom 2D','Telecom 2D',65932,'In Earth orbit','','1996-09-07 00:00:00',1436.06,35774,35798,.45,'Stationary' from dual union all
select 2421,'002421','1966-081D','D-15 imager cover','',13488,'Reentered','1966-09-20 00:00:00','1966-09-13 00:00:00',89.63,239,273,28.8,'Intermediate' from dual union all
select 24210,'024210','1996-044C','Ariane H10-3 V90','Ariane H10-3 S/N T461',65932,'Reentered','1996-08-10 00:00:00','1996-08-10 00:00:00',631.04,215,35770,5.43,'GEO Transfer' from dual union all
select 24211,'024211','1996-044D','Spelda 40 S/N UV33','Spelda 40 (SMS)',65932,'Reentered','2000-06-01 00:00:00','1996-09-08 00:00:00',628.03,232,35598,5.56,'GEO Transfer' from dual union all
select 24212,'024212','1994-029KK','deb HAPS','',65425,'Reentered','2001-12-22 00:00:00','1996-08-10 00:00:00',101.05,605,1011,81.98,'Intermediate' from dual union all
select 24213,'024213','1994-029KL','deb HAPS','',65425,'In Earth orbit','','1996-08-10 00:00:00',107.41,556,1658,81.49,'Intermediate' from dual union all
select 24214,'024214','1994-029KM','deb HAPS','',65425,'Reentered','1998-05-15 00:00:00','1996-08-10 00:00:00',99.67,567,919,82.09,'Intermediate' from dual union all
select 24215,'024215','1994-029KN','deb HAPS','',65425,'Reentered','1999-07-13 00:00:00','1996-08-09 00:00:00',97.7,535,763,82.16,'Intermediate' from dual union all
select 24216,'024216','1994-029KP','deb HAPS','',65425,'Reentered','2000-07-10 00:00:00','1996-08-10 00:00:00',98.52,596,780,82.13,'Intermediate' from dual union all
select 24217,'024217','1994-029KQ','deb HAPS','',65425,'Reentered','2001-03-17 00:00:00','1996-08-10 00:00:00',109.92,576,1870,82.02,'Intermediate' from dual;
insert into SATELLITE
select 24218,'024218','1994-029KR','deb HAPS','',65425,'Reentered','1998-11-21 00:00:00','1996-08-10 00:00:00',99,598,824,81.67,'Intermediate' from dual union all
select 24219,'024219','1994-029KS','deb HAPS','',65425,'Reentered','1996-10-13 00:00:00','1996-08-09 00:00:00',96.01,487,648,81.88,'Intermediate' from dual union all
select 2422,'002422','1966-082B','Burner II No. 1','Star 37B',13489,'In Earth orbit','','1966-10-29 00:00:00',100.82,695,900,98.45,'Sun-Synch' from dual union all
select 24220,'024220','1994-029KT','deb HAPS','',65425,'Reentered','1998-02-05 00:00:00','1996-08-10 00:00:00',100.27,493,1050,82.99,'Intermediate' from dual union all
select 24221,'024221','1994-029KU','deb HAPS','',65425,'Reentered','2000-12-20 00:00:00','1996-08-10 00:00:00',105.91,613,1461,81.69,'Intermediate' from dual union all
select 24222,'024222','1994-029KV','deb HAPS','',65425,'Reentered','1997-07-13 00:00:00','1996-08-10 00:00:00',99.29,543,907,82.03,'Intermediate' from dual union all
select 24223,'024223','1994-029KW','deb HAPS','',65425,'Reentered','2012-02-28 00:00:00','1996-08-10 00:00:00',101.47,609,1047,81.94,'Intermediate' from dual union all
select 24224,'024224','1994-029KX','deb HAPS','',65425,'Reentered','1999-02-25 00:00:00','1996-08-10 00:00:00',98.13,588,751,81.89,'Intermediate' from dual union all
select 24225,'024225','1994-029KY','deb HAPS','',65425,'Reentered','1999-12-25 00:00:00','1996-08-10 00:00:00',98.39,567,797,81.98,'Intermediate' from dual union all
select 24226,'024226','1994-029KZ','deb HAPS','',65425,'Reentered','1996-08-21 00:00:00','1996-08-08 00:00:00',91.65,304,407,82.14,'Intermediate' from dual union all
select 24227,'024227','1994-029LA','deb HAPS','',65425,'In Earth orbit','','1996-08-10 00:00:00',99.42,582,880,82.05,'Intermediate' from dual union all
select 24228,'024228','1994-029LB','deb HAPS','',65425,'Reentered','1999-02-14 00:00:00','1996-08-10 00:00:00',98.31,605,751,81.81,'Intermediate' from dual union all
select 24229,'024229','1994-029LC','deb HAPS','',65425,'Reentered','1999-05-12 00:00:00','1996-08-10 00:00:00',97.7,581,717,81.82,'Intermediate' from dual union all
select 2423,'002423','1966-083C','Agena D 4832','Agena D',13490,'Reentered','1966-09-18 00:00:00','1966-09-16 00:00:00',89.03,132,321,93.9,'Polar' from dual union all
select 24230,'024230','1994-029LD','deb HAPS','',65425,'Reentered','2000-03-06 00:00:00','1996-08-10 00:00:00',99.89,600,907,82.03,'Intermediate' from dual union all
select 24231,'024231','1994-029LE','deb HAPS','',65425,'Reentered','2006-10-01 00:00:00','1996-08-10 00:00:00',97.72,583,717,82.09,'Intermediate' from dual union all
select 24232,'024232','1994-029LF','deb HAPS','',65425,'Reentered','1998-10-28 00:00:00','1996-08-10 00:00:00',97.48,463,814,82.09,'Intermediate' from dual union all
select 24233,'024233','1994-029LG','deb HAPS','',65425,'Reentered','2000-03-16 00:00:00','1996-08-10 00:00:00',98.82,611,794,81.82,'Intermediate' from dual union all
select 24234,'024234','1994-029LH','deb HAPS','',65425,'Reentered','2000-05-19 00:00:00','1996-08-10 00:00:00',98.42,590,776,82.28,'Intermediate' from dual union all
select 24235,'024235','1994-029LJ','deb HAPS','',65425,'Reentered','1999-10-18 00:00:00','1996-08-10 00:00:00',99.64,582,901,82.07,'Intermediate' from dual union all
select 24236,'024236','1994-029LK','deb HAPS','',65425,'In Earth orbit','','1996-08-10 00:00:00',106.02,537,1548,81.45,'Intermediate' from dual union all
select 24237,'024237','1994-029LL','deb HAPS','',65425,'Reentered','1998-12-04 00:00:00','1996-08-10 00:00:00',97.32,486,775,81.96,'Intermediate' from dual union all
select 24238,'024238','1994-029LM','deb HAPS','',65425,'Reentered','2000-09-30 00:00:00','1996-08-10 00:00:00',101.68,611,1066,82.06,'Intermediate' from dual union all
select 24239,'024239','1994-029LN','deb HAPS','',65425,'In Earth orbit','','1996-08-10 00:00:00',102.06,604,1109,81.54,'Intermediate' from dual union all
select 2424,'002424','1966-083D','OPS 6874 sensor cover','',13490,'Reentered','1966-10-30 00:00:00','1966-09-16 00:00:00',94.32,474,497,94,'Polar' from dual union all
select 24240,'024240','1994-029LP','deb HAPS','',65425,'Reentered','2000-01-19 00:00:00','1996-08-10 00:00:00',101.31,609,1033,81.39,'Intermediate' from dual union all
select 24241,'024241','1994-029LQ','deb HAPS','',65425,'Reentered','1997-01-09 00:00:00','1996-08-10 00:00:00',96.6,546,646,81.81,'Intermediate' from dual union all
select 24242,'024242','1994-029LR','deb HAPS','',65425,'Reentered','2002-03-05 00:00:00','1996-08-11 00:00:00',102.09,613,1103,81.82,'Intermediate' from dual union all
select 24243,'024243','1994-029LS','deb HAPS','',65425,'Reentered','2015-04-25 00:00:00','1996-08-11 00:00:00',100.35,604,947,82.02,'Intermediate' from dual union all
select 24244,'024244','1994-029LT','deb HAPS','',65425,'Reentered','1997-10-26 00:00:00','1996-08-11 00:00:00',100.44,560,999,82.07,'Intermediate' from dual union all
select 24245,'024245','1994-029LU','deb HAPS','',65425,'Reentered','1997-11-30 00:00:00','1996-08-11 00:00:00',97.33,491,771,82.36,'Intermediate' from dual union all
select 24246,'024246','1994-029LV','deb HAPS','',65425,'Reentered','1996-11-24 00:00:00','1996-08-11 00:00:00',96.41,550,624,81.79,'Intermediate' from dual union all
select 24247,'024247','1994-029LW','deb HAPS','',65425,'Reentered','2001-06-19 00:00:00','1996-08-10 00:00:00',99.36,612,844,82.18,'Intermediate' from dual union all
select 24248,'024248','1994-029LX','deb HAPS','',65425,'Reentered','1999-06-28 00:00:00','1996-08-10 00:00:00',97.81,546,762,82.08,'Intermediate' from dual union all
select 24249,'024249','1994-029LY','deb HAPS','',65425,'Reentered','2000-10-18 00:00:00','1996-08-11 00:00:00',100.5,611,953,82.19,'Intermediate' from dual union all
select 2425,'002425','1966-084A','Surveyor 2','Surveyor SC-2',13491,'Deep Space','1966-09-22 00:00:00','1966-09-20 00:00:00',42326.1,165,791708,33.41,'Cislunar' from dual union all
select 24250,'024250','1994-029LZ','deb HAPS','',65425,'Reentered','1998-10-08 00:00:00','1996-08-11 00:00:00',100.3,590,956,82.04,'Intermediate' from dual union all
select 24251,'024251','1994-029MA','deb HAPS','',65425,'Reentered','1996-09-02 00:00:00','1996-08-11 00:00:00',94.55,383,610,82.14,'Intermediate' from dual union all
select 24252,'024252','1994-029MB','deb HAPS','',65425,'Reentered','1996-12-17 00:00:00','1996-08-11 00:00:00',95.27,443,621,81.82,'Intermediate' from dual union all
select 24253,'024253','1994-029MC','deb HAPS','',65425,'Reentered','1997-09-03 00:00:00','1996-08-11 00:00:00',96.11,523,622,81.67,'Intermediate' from dual union all
select 24254,'024254','1994-029MD','deb HAPS','',65425,'Reentered','2001-10-10 00:00:00','1996-08-11 00:00:00',100.43,608,950,82.06,'Intermediate' from dual union all
select 24255,'024255','1994-029ME','deb HAPS','',65425,'Reentered','1998-09-11 00:00:00','1996-08-11 00:00:00',99.17,444,994,82.41,'Intermediate' from dual union all
select 24256,'024256','1994-029MF','deb HAPS','',65425,'Reentered','2003-10-28 00:00:00','1996-08-11 00:00:00',100.23,604,935,82.02,'Intermediate' from dual union all
select 24257,'024257','1994-029MG','deb HAPS','',65425,'Reentered','1997-01-08 00:00:00','1996-08-11 00:00:00',97.05,480,755,82.25,'Intermediate' from dual union all
select 24258,'024258','1994-029MH','deb HAPS','',65425,'Reentered','2000-11-02 00:00:00','1996-08-11 00:00:00',101.38,576,1071,81.84,'Intermediate' from dual union all
select 24259,'024259','1994-029MJ','deb HAPS','',65425,'Reentered','2001-12-05 00:00:00','1996-08-11 00:00:00',102.16,612,1110,81.81,'Intermediate' from dual union all
select 2426,'002426','1966-084B','Centaur AC-7','Centaur 3D',13491,'Deep Space','1966-09-22 00:00:00','1966-09-20 00:00:00',26607.01,168,577531,33.41,'Cislunar' from dual union all
select 24260,'024260','1994-029MK','deb HAPS','',65425,'Reentered','1999-10-04 00:00:00','1996-08-11 00:00:00',96.6,495,697,81.9,'Intermediate' from dual union all
select 24261,'024261','1994-029ML','deb HAPS','',65425,'Reentered','2002-02-10 00:00:00','1996-08-11 00:00:00',98.99,610,811,81.71,'Intermediate' from dual union all
select 24262,'024262','1994-029MM','deb HAPS','',65425,'Reentered','2000-07-01 00:00:00','1996-08-11 00:00:00',99.12,555,879,82.33,'Intermediate' from dual union all
select 24263,'024263','1994-029MN','deb HAPS','',65425,'Reentered','2004-01-11 00:00:00','1996-08-11 00:00:00',97.89,594,722,82.09,'Intermediate' from dual union all
select 24264,'024264','1994-029MP','deb HAPS','',65425,'Reentered','1998-12-01 00:00:00','1996-08-11 00:00:00',98.37,520,842,82.2,'Intermediate' from dual union all
select 24265,'024265','1994-029MQ','deb HAPS','',65425,'Reentered','1997-08-17 00:00:00','1996-08-11 00:00:00',101.67,534,1142,82.18,'Intermediate' from dual union all
select 24266,'024266','1994-029MR','deb HAPS','',65425,'Reentered','1999-11-13 00:00:00','1996-08-11 00:00:00',97.87,568,746,81.88,'Intermediate' from dual union all
select 24267,'024267','1994-029MS','deb HAPS','',65425,'Reentered','1999-11-16 00:00:00','1996-08-11 00:00:00',98.39,553,810,82.05,'Intermediate' from dual union all
select 24268,'024268','1994-029MT','deb HAPS','',65425,'Reentered','2003-10-13 00:00:00','1996-08-11 00:00:00',98.84,610,797,81.87,'Intermediate' from dual union all
select 24269,'024269','1994-029MU','deb HAPS','',65425,'Reentered','1998-11-25 00:00:00','1996-08-11 00:00:00',101.64,597,1076,81.48,'Intermediate' from dual union all
select 2427,'002427','1966-085A','OPS 1703','CORONA J-36',13492,'Reentered Attached','1966-10-12 00:00:00','1966-09-21 00:00:00',90.82,188,442,85.13,'Polar' from dual union all
select 24270,'024270','1994-029MV','deb HAPS','',65425,'Reentered','1999-03-04 00:00:00','1996-08-11 00:00:00',99.96,595,919,82,'Intermediate' from dual union all
select 24271,'024271','1994-029MW','deb HAPS','',65425,'Reentered','1997-10-10 00:00:00','1996-08-10 00:00:00',99.38,566,892,82.23,'Intermediate' from dual union all
select 24272,'024272','1964-026H','deb Transit VC','',10312,'Reentered','1999-08-31 00:00:00','1996-08-12 00:00:00',101.53,784,878,90.52,'Polar' from dual union all
select 24273,'024273','1996-045A','Molniya-1T','Molniya-1T',65933,'Reentered','2012-04-07 00:00:00','1996-09-13 00:00:00',717.76,495,39859,62.87,'Molniya' from dual union all
select 24274,'024274','1996-045B','11S510','11S510',65933,'Reentered','1996-09-21 00:00:00','1996-08-21 00:00:00',90.79,203,424,62.8,'Intermediate' from dual union all
select 24275,'024275','1996-045C','BOZ','BOZ',65933,'Reentered','1996-09-01 00:00:00','1996-08-21 00:00:00',90.47,177,419,62.81,'Intermediate' from dual union all
select 24276,'024276','1996-045D','Blok-ML','Blok-ML No. 77316-491',65933,'Reentered','2013-07-17 00:00:00','1996-09-13 00:00:00',732.22,499,40566,62.85,'Molniya' from dual union all
select 24277,'024277','1996-046A','Midori','ADEOS',65934,'In Earth orbit','','1996-09-16 00:00:00',100.86,799,799,98.62,'Sun-Synch' from dual union all
select 24278,'024278','1996-046B','Fuji-OSCAR-29','JAS-2',65934,'In Earth orbit','','1996-09-17 00:00:00',106.44,800,1324,98.58,'Sun-Synch' from dual union all
select 24279,'024279','1996-046C','H-II Stage 2','H-II Stage 2',65934,'In Earth orbit','','1996-09-17 00:00:00',106.95,860,1311,98.76,'Sun-Synch' from dual union all
select 2428,'002428','1965-058D','Agena D 1803','[UNID001 1966-000A] RCS',11808,'Reentered','1977-09-17 00:00:00','1965-07-21 00:00:00',2591.3,194,112038,34.4,'Very High' from dual union all
select 24280,'024280','1996-047A','Soyuz TM-24','Soyuz 7K-STM No. 73',65935,'Landed','1997-03-02 00:00:00','1996-09-17 00:00:00',92.18,375,388,51.65,'Intermediate' from dual union all
select 24281,'024281','1996-047B','11S510','11S510',65935,'Reentered','1996-08-21 00:00:00','1996-08-19 00:00:00',88.48,185,214,51.62,'Intermediate' from dual union all
select 24282,'024282','1996-048A','Zhongxing 7','Zhongxing 7',65936,'In Earth orbit','','1996-11-22 00:00:00',1350.07,21666,46505,26.31,'Medium' from dual union all
select 24283,'024283','1996-048B','CZ-3 Stage 3','CZ-3 Stage 3',65936,'Reentered','2003-07-03 00:00:00','1996-09-17 00:00:00',307.98,208,17250,27.28,'Highly Ellip' from dual union all
select 24284,'024284','1995-058G','Zenit part','Slow decay',65719,'In Earth orbit','','1996-08-19 00:00:00',101.81,834,854,70.87,'Intermediate' from dual union all
select 24285,'024285','1996-049A','FAST','FAST',65937,'In Earth orbit','','1996-09-20 00:00:00',133.1,351,4164,83,'Medium' from dual union all
select 24286,'024286','1996-049B','Orion 38','Orion 38',65937,'In Earth orbit','','1996-09-20 00:00:00',133.15,352,4167,82.99,'Medium' from dual union all
select 24287,'024287','1986-017LU','Garbage bag','',59210,'Reentered','1996-09-14 00:00:00','1996-08-27 00:00:00',91.28,335,340,51.65,'Intermediate' from dual union all
select 24288,'024288','1994-029MX','deb HAPS','',65425,'Reentered','1996-09-06 00:00:00','1996-08-28 00:00:00',92.64,374,434,81.87,'Intermediate' from dual union all
select 24289,'024289','1994-029MY','deb HAPS','',65425,'Reentered','1996-09-07 00:00:00','1996-08-28 00:00:00',93.73,397,517,82.12,'Intermediate' from dual union all
select 2429,'002429','1963-014DG','deb West Ford (1966-00B)','[UNID002 1966-000B]',9245,'Reentered','1979-01-01 00:00:00','1967-05-27 00:00:00',163.1,903,6118,85.24,'Medium' from dual union all
select 24290,'024290','1994-029MZ','deb HAPS','',65425,'Reentered','1996-09-08 00:00:00','1996-08-28 00:00:00',94.07,457,490,81.84,'Intermediate' from dual union all
select 24291,'024291','1996-050A','Victor','Microsatelite',65938,'Reentered','1999-11-12 00:00:00','1996-09-28 00:00:00',98.64,236,1152,62.79,'Intermediate' from dual union all
select 24292,'024292','1996-050B','Magion-5','Magion-5',65938,'In Earth orbit','','1996-09-29 00:00:00',347.37,777,19205,62.8,'Highly Ellip' from dual union all
select 24293,'024293','1996-050C','Interbol-2','SO-M2 No. 512',65938,'In Earth orbit','','1996-09-28 00:00:00',347.31,771,19207,62.79,'Highly Ellip' from dual union all
select 24294,'024294','1996-050D','11S510','11S510',65938,'Reentered','1998-07-26 00:00:00','1996-09-28 00:00:00',97.82,234,1074,62.76,'Intermediate' from dual union all
select 24295,'024295','1996-050E','BOZ','BOZ',65938,'Reentered','1997-07-20 00:00:00','1996-09-28 00:00:00',97.91,208,1109,62.81,'Intermediate' from dual union all
select 24296,'024296','1996-050F','Blok SO-L','Blok SO-L No. 71644-806',65938,'In Earth orbit','','1996-09-28 00:00:00',346.94,770,19185,62.76,'Highly Ellip' from dual union all
select 24297,'024297','1996-051A','Kosmos-2333','Tselina-2',65939,'In Earth orbit','','1996-10-04 00:00:00',101.92,848,852,71.01,'Intermediate' from dual union all
select 24298,'024298','1996-051B','11S772','11S772',65939,'In Earth orbit','','1996-10-04 00:00:00',101.94,841,859,71.02,'Intermediate' from dual union all
select 24299,'024299','1996-051C','Sep Motor Cover','Sep Motor Cover',65939,'In Earth orbit','','1996-10-04 00:00:00',104.84,847,1127,71.04,'Intermediate' from dual union all
select 243,'000243','61 Omicron 71','deb r/b Ablestar','',7273,'In Earth orbit','','1962-02-21 00:00:00',105.23,974,1037,66.47,'Intermediate' from dual union all
select 2430,'002430','1963-014DH','deb West Ford (1966-00C)','[UNID003 1966-000C]',9245,'Reentered','1979-01-01 00:00:00','1966-09-19 00:00:00',163.04,738,6278,85.27,'Medium' from dual union all
select 24300,'024300','1996-051D','Sep Motor Cover','Sep Motor Cover',65939,'In Earth orbit','','1996-10-04 00:00:00',104.93,845,1138,71.03,'Intermediate' from dual union all
select 24301,'024301','1996-051E','Sep Motor Cover','Sep Motor Cover',65939,'In Earth orbit','','1996-10-04 00:00:00',105.1,844,1155,71,'Intermediate' from dual union all
select 24302,'024302','1996-051F','Sep Motor Cover','Sep Motor Cover',65939,'In Earth orbit','','1996-10-04 00:00:00',105.18,847,1159,71.04,'Intermediate' from dual union all
select 24303,'024303','1996-051G','deb Zenit','',65939,'In Earth orbit','','1996-10-04 00:00:00',102.32,816,921,71.1,'Intermediate' from dual union all
select 24304,'024304','1996-052A','Kosmos-2334','Parus',65940,'In Earth orbit','','1996-10-05 00:00:00',104.87,967,1009,82.94,'Intermediate' from dual union all
select 24305,'024305','1996-052B','MEXICO-OSCAR-30','UNAMSAT-B',65940,'In Earth orbit','','1996-10-05 00:00:00',104.86,966,1010,82.94,'Intermediate' from dual union all
select 24306,'024306','1996-052C','S3M','S3M',65940,'In Earth orbit','','1996-10-06 00:00:00',104.76,968,999,82.93,'Intermediate' from dual union all
select 24307,'024307','1996-053A','INMARSAT III F-2','INMARSAT III F-2',65941,'In Earth orbit','','1996-10-06 00:00:00',1449.36,36039,36053,2.67,'Drift GEO' from dual;
insert into SATELLITE
select 24308,'024308','1996-053B','8S812','8S812',65941,'Reentered','1996-09-16 00:00:00','1996-09-08 00:00:00',88.84,216,219,51.6,'Intermediate' from dual union all
select 24309,'024309','1996-053C','Perekhodnik','Sredniy perekhodnik',65941,'Reentered','1996-09-09 00:00:00','1996-09-08 00:00:00',88.75,206,220,51.59,'Intermediate' from dual union all
select 2431,'002431','1963-014AC','deb West Ford','[UNID004 1966-000D] RCS',9245,'In Earth orbit','','1992-02-01 00:00:00',165.95,2910,4341,87.28,'Medium' from dual union all
select 24310,'024310','1995-041D','deb Delta','',65702,'Reentered','2003-06-14 00:00:00','1996-09-16 00:00:00',108.54,917,1402,26.68,'Intermediate' from dual union all
select 24311,'024311','1971-120H','deb Meteor','',24938,'Reentered','2001-09-14 00:00:00','1996-09-06 00:00:00',100.62,776,800,81.22,'Intermediate' from dual union all
select 24312,'024312','1966-077N','deb FTV 1352','',13484,'In Earth orbit','','1996-09-06 00:00:00',165.49,1534,5680,88.54,'Medium' from dual union all
select 24313,'024313','1996-055A','Echostar II','Echostar II',65943,'In Earth orbit','','1996-10-25 00:00:00',1436.27,35771,35809,0,'Stationary' from dual union all
select 24314,'024314','1996-055B','Ariane H10-3 V91','Ariane H10-3 S/N T462',65943,'Reentered','2000-04-02 00:00:00','1996-10-11 00:00:00',645.33,212,36506,7.27,'GEO Transfer' from dual union all
select 24315,'024315','1996-054A','GE 1','GE 1',65942,'In Earth orbit','','1996-10-09 00:00:00',1436.11,35784,35790,.01,'Stationary' from dual union all
select 24316,'024316','1996-054B','Centaur AC-123','Centaur IIA(1N) AC-123',65942,'Reentered','1997-03-17 00:00:00','1996-09-09 00:00:00',1061.51,192,55992,25.12,'Highly Ellip' from dual union all
select 24317,'024317','1996-049C','deb Pegasus','',65937,'Reentered','2001-12-03 00:00:00','1996-09-23 00:00:00',133.15,347,4172,83,'Medium' from dual union all
select 24318,'024318','1996-049D','deb Pegasus','',65937,'Reentered','2001-12-21 00:00:00','1996-09-23 00:00:00',133.16,346,4174,82.99,'Medium' from dual union all
select 24319,'024319','1996-049E','deb Pegasus','',65937,'Reentered','1999-06-01 00:00:00','1996-09-20 00:00:00',132.95,348,4154,83,'Medium' from dual union all
select 2432,'002432','1963-049G','deb Transit VBN-2','',9280,'In Earth orbit','','1966-10-14 00:00:00',107.11,1067,1119,89.94,'Polar' from dual union all
select 24320,'024320','1996-056A','GPS IIA-18','Navstar GPS SVN 30',65944,'In Earth orbit','','1996-10-17 00:00:00',717.94,20058,20305,54.71,'Medium' from dual union all
select 24321,'024321','1996-056C','Star 48B','Star 48B',65944,'Reentered','2000-08-15 00:00:00','1996-10-12 00:00:00',352.17,177,20106,35.01,'Highly Ellip' from dual union all
select 24322,'024322','1994-029NA','deb HAPS','',65425,'Reentered','1997-05-27 00:00:00','1996-09-13 00:00:00',96.08,435,707,82.09,'Intermediate' from dual union all
select 24323,'024323','1996-056B','Delta 238','Delta SSPS AJ-10-118K',65944,'Reentered','2001-07-28 00:00:00','1996-10-12 00:00:00',95.43,470,609,35.54,'Intermediate' from dual union all
select 24324,'024324','1996-057A','Atlantis (STS-79)','OV-104',65945,'Landed','1996-09-26 00:00:00','1996-09-18 00:00:00',90,251,297,51.65,'Intermediate' from dual union all
select 24325,'024325','1996-049F','deb Pegasus','',65937,'Reentered','1999-03-17 00:00:00','1996-09-22 00:00:00',132.73,346,4137,83,'Medium' from dual union all
select 24326,'024326','1994-029NB','deb HAPS','',65425,'Reentered','1997-01-11 00:00:00','1996-09-16 00:00:00',94.18,403,555,82.15,'Intermediate' from dual union all
select 24327,'024327','1994-029NC','deb HAPS','',65425,'Reentered','1996-10-01 00:00:00','1996-09-18 00:00:00',93.61,407,496,82.45,'Intermediate' from dual union all
select 24328,'024328','1994-029ND','deb HAPS','',65425,'Reentered','1996-10-01 00:00:00','1996-09-17 00:00:00',93.96,454,483,81.8,'Intermediate' from dual union all
select 24329,'024329','1994-029NE','deb HAPS','',65425,'Reentered','1998-02-27 00:00:00','1996-09-20 00:00:00',96.84,535,680,81.93,'Intermediate' from dual union all
select 2433,'002433','1966-086A','OPS 4096','GAMBIT-3 Mission 4302',13493,'Reentered Attached','1966-10-07 00:00:00','1966-09-30 00:00:00',88.96,151,295,93.98,'Polar' from dual union all
select 24330,'024330','1994-029NF','deb HAPS','',65425,'Reentered','1996-12-24 00:00:00','1996-09-20 00:00:00',96.63,518,677,81.95,'Intermediate' from dual union all
select 24331,'024331','1994-029NG','deb HAPS','',65425,'Reentered','2000-02-06 00:00:00','1996-09-20 00:00:00',97.65,612,681,81.7,'Intermediate' from dual union all
select 24332,'024332','1994-029NH','deb HAPS','',65425,'Reentered','1997-04-04 00:00:00','1996-09-20 00:00:00',97.32,579,682,81.79,'Intermediate' from dual union all
select 24333,'024333','1994-029NJ','deb HAPS','',65425,'Reentered','1997-01-27 00:00:00','1996-09-20 00:00:00',97.3,517,742,81.99,'Intermediate' from dual union all
select 24334,'024334','1994-029NK','deb HAPS','',65425,'Reentered','1997-08-22 00:00:00','1996-09-20 00:00:00',97.89,528,788,81.97,'Intermediate' from dual union all
select 24335,'024335','1994-029NL','deb HAPS','',65425,'Reentered','1997-10-17 00:00:00','1996-09-20 00:00:00',98.12,428,910,82.26,'Intermediate' from dual union all
select 24336,'024336','1994-029NM','deb HAPS','',65425,'Reentered','1997-10-31 00:00:00','1996-09-20 00:00:00',100.37,561,992,82.17,'Intermediate' from dual union all
select 24337,'024337','1994-029NN','deb HAPS','',65425,'Reentered','2001-06-11 00:00:00','1996-09-20 00:00:00',100.83,607,989,81.97,'Intermediate' from dual union all
select 24338,'024338','1994-029NP','deb HAPS','',65425,'Reentered','1997-11-19 00:00:00','1996-09-20 00:00:00',100.57,548,1024,82.18,'Intermediate' from dual union all
select 24339,'024339','1994-029NQ','deb HAPS','',65425,'Reentered','2001-12-11 00:00:00','1996-09-20 00:00:00',111.52,592,2002,81.64,'Intermediate' from dual union all
select 2434,'002434','1966-085B','Fairing','Fairing',13492,'Reentered','1966-10-08 00:00:00','1966-09-30 00:00:00',90.82,188,442,85.13,'Polar' from dual union all
select 24340,'024340','1994-029NR','deb HAPS','',65425,'In Earth orbit','','1996-09-23 00:00:00',113.2,581,2166,81.6,'Intermediate' from dual union all
select 24341,'024341','1994-029NS','deb HAPS','',65425,'Reentered','2000-09-28 00:00:00','1996-09-20 00:00:00',99.18,602,837,81.95,'Intermediate' from dual union all
select 24342,'024342','1994-029NT','deb HAPS','',65425,'Reentered','2000-04-08 00:00:00','1996-09-20 00:00:00',98.62,567,819,81.99,'Intermediate' from dual union all
select 24343,'024343','1994-029NU','deb HAPS','',65425,'Reentered','2000-03-02 00:00:00','1996-09-20 00:00:00',98.73,603,792,82.09,'Intermediate' from dual union all
select 24344,'024344','1994-029NV','deb HAPS','',65425,'Reentered','1997-12-02 00:00:00','1996-09-23 00:00:00',98.74,554,843,82.1,'Intermediate' from dual union all
select 24345,'024345','1994-029NW','deb HAPS','',65425,'Reentered','1999-01-04 00:00:00','1996-09-23 00:00:00',96.39,559,613,82.26,'Intermediate' from dual union all
select 24346,'024346','1994-029NX','deb HAPS','',65425,'Reentered','1997-11-06 00:00:00','1996-09-23 00:00:00',100.62,558,1018,82.05,'Intermediate' from dual union all
select 24347,'024347','1994-029NY','deb HAPS','',65425,'Reentered','2002-06-14 00:00:00','1996-09-23 00:00:00',99.72,589,901,82.07,'Intermediate' from dual union all
select 24348,'024348','1994-029NZ','deb HAPS','',65425,'Reentered','2000-08-04 00:00:00','1996-09-23 00:00:00',99.95,577,936,82.09,'Intermediate' from dual union all
select 24349,'024349','1994-029PA','deb HAPS','',65425,'Reentered','1999-10-23 00:00:00','1996-09-23 00:00:00',101.5,600,1060,82.07,'Intermediate' from dual union all
select 2435,'002435','1966-087A','ESSA 3','TOS A',13494,'In Earth orbit','','1966-11-07 00:00:00',114.58,1385,1488,101.01,'Sun-Synch' from dual union all
select 24350,'024350','1994-029PB','deb HAPS','',65425,'Reentered','1998-07-11 00:00:00','1996-09-23 00:00:00',96.9,543,678,81.93,'Intermediate' from dual union all
select 24351,'024351','1994-029PC','deb HAPS','',65425,'Reentered','1996-11-16 00:00:00','1996-09-22 00:00:00',96.49,529,652,81.81,'Intermediate' from dual union all
select 24352,'024352','1994-029PD','deb HAPS','',65425,'Reentered','1997-03-06 00:00:00','1996-09-22 00:00:00',97.4,556,713,81.8,'Intermediate' from dual union all
select 24353,'024353','1994-029PE','deb HAPS','',65425,'Reentered','1998-03-31 00:00:00','1996-09-19 00:00:00',102.63,547,1219,80.84,'Intermediate' from dual union all
select 24354,'024354','1994-029PF','deb HAPS','',65425,'Reentered','1998-05-12 00:00:00','1996-09-23 00:00:00',99.52,587,884,81.32,'Intermediate' from dual union all
select 24355,'024355','1994-029PG','deb HAPS','',65425,'Reentered','1999-10-27 00:00:00','1996-09-23 00:00:00',104.42,597,1338,81.9,'Intermediate' from dual union all
select 24356,'024356','1994-029PH','deb HAPS','',65425,'Reentered','1997-01-20 00:00:00','1996-09-23 00:00:00',97.34,494,769,82.18,'Intermediate' from dual union all
select 24357,'024357','1994-029PJ','deb HAPS','',65425,'Reentered','1997-04-17 00:00:00','1996-09-23 00:00:00',98.5,530,844,82.33,'Intermediate' from dual union all
select 24358,'024358','1994-029PK','deb HAPS','',65425,'Reentered','2000-03-28 00:00:00','1996-09-23 00:00:00',106.38,600,1518,81.51,'Intermediate' from dual union all
select 24359,'024359','1994-029PL','deb HAPS','',65425,'Reentered','2001-04-26 00:00:00','1996-09-23 00:00:00',99.58,614,862,81.86,'Intermediate' from dual union all
select 2436,'002436','1966-087B','Altair 2 X-258C4 RH-89','X-258C4 RH-89',13494,'In Earth orbit','','1966-11-10 00:00:00',114.57,1383,1489,101.04,'Sun-Synch' from dual union all
select 24360,'024360','1994-029PM','deb HAPS','',65425,'Reentered','1997-04-22 00:00:00','1996-09-23 00:00:00',98.71,505,889,82.12,'Intermediate' from dual union all
select 24361,'024361','1994-029PN','deb HAPS','',65425,'Reentered','2001-01-30 00:00:00','1996-09-23 00:00:00',98.44,549,820,82.17,'Intermediate' from dual union all
select 24362,'024362','1994-029PP','deb HAPS','',65425,'Reentered','1997-05-13 00:00:00','1996-09-23 00:00:00',97.86,557,756,81.86,'Intermediate' from dual union all
select 24363,'024363','1994-029PQ','deb HAPS','',65425,'Reentered','1996-12-21 00:00:00','1996-09-23 00:00:00',96.66,525,673,81.86,'Intermediate' from dual union all
select 24364,'024364','1994-029PR','deb HAPS','',65425,'Reentered','1998-04-08 00:00:00','1996-09-22 00:00:00',96.37,478,691,82.17,'Intermediate' from dual union all
select 24365,'024365','1994-029PS','deb HAPS','',65425,'Reentered','1996-11-09 00:00:00','1996-09-23 00:00:00',95.5,536,550,81.8,'Intermediate' from dual union all
select 24366,'024366','1994-029PT','deb HAPS','',65425,'Reentered','1997-08-05 00:00:00','1996-09-23 00:00:00',97.23,565,688,81.83,'Intermediate' from dual union all
select 24367,'024367','1994-029PU','deb HAPS','',65425,'Reentered','1996-12-02 00:00:00','1996-09-23 00:00:00',96.67,524,675,81.85,'Intermediate' from dual union all
select 24368,'024368','1994-029PV','deb HAPS','',65425,'Reentered','2000-01-23 00:00:00','1996-09-23 00:00:00',97.74,529,772,82.14,'Intermediate' from dual union all
select 24369,'024369','1994-029PW','deb HAPS','',65425,'Reentered','1999-03-24 00:00:00','1996-09-23 00:00:00',104.42,586,1349,81.95,'Intermediate' from dual union all
select 2437,'002437','1966-088A','OGCh','OGCh No. 05L',13495,'Reentered','1966-11-11 00:00:00','1966-09-18 00:00:00',96.78,163,1046,49.63,'Intermediate' from dual union all
select 24370,'024370','1994-029PX','deb HAPS','',65425,'Reentered','1997-02-01 00:00:00','1996-09-23 00:00:00',97.04,510,724,82.25,'Intermediate' from dual union all
select 24371,'024371','1994-029PY','deb HAPS','',65425,'Reentered','1996-12-04 00:00:00','1996-09-22 00:00:00',97.07,477,760,82.26,'Intermediate' from dual union all
select 24372,'024372','1994-029PZ','deb HAPS','',65425,'Reentered','1999-12-19 00:00:00','1996-09-23 00:00:00',98.42,602,765,82.08,'Intermediate' from dual union all
select 24373,'024373','1994-029QA','deb HAPS','',65425,'Reentered','1999-08-26 00:00:00','1996-09-23 00:00:00',98.44,570,799,82.13,'Intermediate' from dual union all
select 24374,'024374','1994-029QB','deb HAPS','',65425,'Reentered','1999-08-01 00:00:00','1996-09-23 00:00:00',98.41,612,754,81.85,'Intermediate' from dual union all
select 24375,'024375','1994-029QC','deb HAPS','',65425,'Reentered','2002-10-25 00:00:00','1996-09-24 00:00:00',112.26,581,2080,81.7,'Intermediate' from dual union all
select 24376,'024376','1994-029QD','deb HAPS','',65425,'Reentered','1997-04-30 00:00:00','1996-09-23 00:00:00',97.92,514,805,82.02,'Intermediate' from dual union all
select 24377,'024377','1994-029QE','deb HAPS','',65425,'Reentered','2000-06-09 00:00:00','1996-09-23 00:00:00',99.06,579,849,82.02,'Intermediate' from dual union all
select 24378,'024378','1994-029QF','deb HAPS','',65425,'In Earth orbit','','1996-09-24 00:00:00',99.87,608,897,82.03,'Intermediate' from dual union all
select 24379,'024379','1994-029QG','deb HAPS','',65425,'Reentered','1998-10-03 00:00:00','1996-09-21 00:00:00',97.42,611,659,82.15,'Intermediate' from dual union all
select 2438,'002438','1966-088B','8S692','8S692',13495,'Reentered','1967-03-04 00:00:00','1966-11-06 00:00:00',97.17,274,973,49.26,'Intermediate' from dual union all
select 24380,'024380','1994-029QH','deb HAPS','',65425,'Reentered','1997-10-16 00:00:00','1996-09-23 00:00:00',99.75,569,924,82.04,'Intermediate' from dual union all
select 24381,'024381','1994-029QJ','deb HAPS','',65425,'Reentered','1998-02-22 00:00:00','1996-09-22 00:00:00',100.52,560,1007,82.14,'Intermediate' from dual union all
select 24382,'024382','1994-029QK','deb HAPS','',65425,'Reentered','1997-11-04 00:00:00','1996-09-23 00:00:00',100.58,543,1030,82.2,'Intermediate' from dual union all
select 24383,'024383','1994-029QL','deb HAPS','',65425,'Reentered','1997-11-14 00:00:00','1996-09-23 00:00:00',100.66,559,1020,82.2,'Intermediate' from dual union all
select 24384,'024384','1994-029QM','deb HAPS','',65425,'Reentered','1998-06-15 00:00:00','1996-09-23 00:00:00',101.05,577,1040,81.47,'Intermediate' from dual union all
select 24385,'024385','1994-029QN','deb HAPS','',65425,'Reentered','2001-03-28 00:00:00','1996-09-24 00:00:00',101.22,500,1133,82.32,'Intermediate' from dual union all
select 24386,'024386','1994-029QP','deb HAPS','',65425,'Reentered','1997-12-22 00:00:00','1996-09-22 00:00:00',101.29,548,1092,82.12,'Intermediate' from dual union all
select 24387,'024387','1994-029QQ','deb HAPS','',65425,'Reentered','1998-02-27 00:00:00','1996-09-23 00:00:00',101.6,557,1112,82.13,'Intermediate' from dual union all
select 24388,'024388','1994-029QR','deb HAPS','',65425,'Reentered','2000-01-29 00:00:00','1996-09-23 00:00:00',97.98,564,761,81.91,'Intermediate' from dual union all
select 24389,'024389','1994-029QS','deb HAPS','',65425,'Reentered','2000-03-22 00:00:00','1996-09-23 00:00:00',98.06,585,747,82.14,'Intermediate' from dual union all
select 2439,'002439','1966-088C','deb OGCh','',13495,'Reentered','1966-10-10 00:00:00','1966-09-18 00:00:00',91.91,214,523,49.8,'Intermediate' from dual union all
select 24390,'024390','1994-029QT','deb HAPS','',65425,'Reentered','1997-06-21 00:00:00','1996-09-23 00:00:00',98.42,569,797,81.81,'Intermediate' from dual union all
select 24391,'024391','1994-029QU','deb HAPS','',65425,'Reentered','1997-05-12 00:00:00','1996-09-23 00:00:00',98.38,548,815,81.95,'Intermediate' from dual union all
select 24392,'024392','1994-029QV','deb HAPS','',65425,'Reentered','1997-04-29 00:00:00','1996-09-23 00:00:00',98.36,532,829,82.09,'Intermediate' from dual union all
select 24393,'024393','1994-029QW','deb HAPS','',65425,'Reentered','1997-04-15 00:00:00','1996-09-23 00:00:00',98.73,520,876,81.99,'Intermediate' from dual union all
select 24394,'024394','1994-029QX','deb HAPS','',65425,'Reentered','1997-06-15 00:00:00','1996-09-23 00:00:00',98.64,422,966,82.24,'Intermediate' from dual union all
select 24395,'024395','1994-029QY','deb HAPS','',65425,'Reentered','1997-08-07 00:00:00','1996-09-23 00:00:00',99.07,561,868,82.02,'Intermediate' from dual union all
select 24396,'024396','1994-029QZ','deb HAPS','',65425,'Reentered','1998-10-14 00:00:00','1996-09-23 00:00:00',99.7,594,895,81.61,'Intermediate' from dual union all
select 24397,'024397','1994-029RA','deb HAPS','',65425,'Reentered','1997-12-04 00:00:00','1996-09-23 00:00:00',100.67,553,1028,82.2,'Intermediate' from dual union all
select 24398,'024398','1994-029RB','deb HAPS','',65425,'Reentered','1998-08-31 00:00:00','1996-09-23 00:00:00',101.03,587,1028,81.53,'Intermediate' from dual;
insert into SATELLITE
select 24399,'024399','1994-029RC','deb HAPS','',65425,'Reentered','1998-03-12 00:00:00','1996-09-23 00:00:00',101.84,537,1155,82.22,'Intermediate' from dual union all
select 244,'000244','61 Omicron 72','deb r/b Ablestar','',7273,'In Earth orbit','','1962-02-21 00:00:00',103.88,880,1004,66.8,'Intermediate' from dual union all
select 2440,'002440','1966-088D','deb OGCh','',13495,'Reentered','1966-10-18 00:00:00','1966-09-18 00:00:00',93.89,186,744,49.79,'Intermediate' from dual union all
select 24400,'024400','1994-029RD','deb HAPS','',65425,'Reentered','1997-05-29 00:00:00','1996-09-23 00:00:00',98.49,549,824,81.92,'Intermediate' from dual union all
select 24401,'024401','1994-029RE','deb HAPS','',65425,'Reentered','1997-11-09 00:00:00','1996-09-22 00:00:00',101.22,550,1083,82.09,'Intermediate' from dual union all
select 24402,'024402','1994-029RF','deb HAPS','',65425,'Reentered','1998-07-23 00:00:00','1996-09-23 00:00:00',95.56,478,613,81.98,'Intermediate' from dual union all
select 24403,'024403','1994-029RG','deb HAPS','',65425,'Reentered','1997-12-13 00:00:00','1996-09-23 00:00:00',96.09,535,608,81.99,'Intermediate' from dual union all
select 24404,'024404','1994-029RH','deb HAPS','',65425,'Reentered','1996-10-08 00:00:00','1996-09-23 00:00:00',94.07,444,502,82.2,'Intermediate' from dual union all
select 24405,'024405','1994-029RJ','deb HAPS','',65425,'Reentered','1999-02-04 00:00:00','1996-09-23 00:00:00',97.17,569,678,81.98,'Intermediate' from dual union all
select 24406,'024406','1994-029RK','deb HAPS','',65425,'Reentered','1997-05-01 00:00:00','1996-09-23 00:00:00',97.68,554,742,81.83,'Intermediate' from dual union all
select 24407,'024407','1994-029RL','deb HAPS','',65425,'Reentered','1996-11-16 00:00:00','1996-09-23 00:00:00',97.25,494,760,82.06,'Intermediate' from dual union all
select 24408,'024408','1994-029RM','deb HAPS','',65425,'Reentered','1998-05-15 00:00:00','1996-09-23 00:00:00',96.88,565,654,82.05,'Intermediate' from dual union all
select 24409,'024409','1994-029RN','deb HAPS','',65425,'Reentered','1996-10-20 00:00:00','1996-09-23 00:00:00',95.21,466,592,81.93,'Intermediate' from dual union all
select 2441,'002441','1966-088E','deb OGCh adapter','',13495,'Reentered','1967-06-23 00:00:00','1966-11-08 00:00:00',99.93,290,1221,50.28,'Intermediate' from dual union all
select 24410,'024410','1994-029RP','deb HAPS','',65425,'Reentered','1996-12-01 00:00:00','1996-09-23 00:00:00',96.47,524,656,81.86,'Intermediate' from dual union all
select 24411,'024411','1994-029RQ','deb HAPS','',65425,'Reentered','1997-01-10 00:00:00','1996-09-23 00:00:00',97.78,501,804,81.99,'Intermediate' from dual union all
select 24412,'024412','1994-029RR','deb HAPS','',65425,'Reentered','1996-11-03 00:00:00','1996-09-23 00:00:00',95.61,467,629,82.17,'Intermediate' from dual union all
select 24413,'024413','1994-029RS','deb HAPS','',65425,'Reentered','1996-10-13 00:00:00','1996-09-23 00:00:00',95,447,591,82.21,'Intermediate' from dual union all
select 24414,'024414','1994-029RT','deb HAPS','',65425,'Reentered','1999-08-12 00:00:00','1996-09-23 00:00:00',98.12,599,739,82.1,'Intermediate' from dual union all
select 24415,'024415','1994-029RU','deb HAPS','',65425,'Reentered','1997-01-10 00:00:00','1996-09-23 00:00:00',97.35,498,766,82.16,'Intermediate' from dual union all
select 24416,'024416','1994-029RV','deb HAPS','',65425,'Reentered','1996-11-20 00:00:00','1996-09-23 00:00:00',96.82,476,736,82.18,'Intermediate' from dual union all
select 24417,'024417','1994-029RW','deb HAPS','',65425,'Reentered','1998-03-21 00:00:00','1996-09-23 00:00:00',101.82,560,1129,82.07,'Intermediate' from dual union all
select 24418,'024418','1994-029RX','deb HAPS','',65425,'Reentered','1997-11-23 00:00:00','1996-09-23 00:00:00',98.38,583,780,81.77,'Intermediate' from dual union all
select 24419,'024419','1986-017LV','Garbage bag','',59210,'Reentered','1998-05-22 00:00:00','1996-09-25 00:00:00',92.31,385,391,51.65,'Intermediate' from dual union all
select 2442,'002442','1966-088F','deb OGCh','',13495,'Reentered','1966-09-25 00:00:00','1966-09-18 00:00:00',93.89,186,744,49.7,'Intermediate' from dual union all
select 24420,'024420','1994-029RY','deb HAPS','',65425,'Reentered','1997-04-10 00:00:00','1996-09-26 00:00:00',97.18,597,651,82.09,'Intermediate' from dual union all
select 24421,'024421','1994-029RZ','deb HAPS','',65425,'Reentered','1997-06-04 00:00:00','1996-09-26 00:00:00',98.84,531,876,81.99,'Intermediate' from dual union all
select 24422,'024422','1994-029SA','deb HAPS','',65425,'Reentered','1997-08-27 00:00:00','1996-09-26 00:00:00',99.11,530,903,81.95,'Intermediate' from dual union all
select 24423,'024423','1994-029SB','deb HAPS','',65425,'Reentered','1998-05-07 00:00:00','1996-09-26 00:00:00',101.5,579,1080,81.01,'Intermediate' from dual union all
select 24424,'024424','1994-029SC','deb HAPS','',65425,'Reentered','1997-03-06 00:00:00','1996-09-25 00:00:00',97.91,520,798,82.06,'Intermediate' from dual union all
select 24425,'024425','1994-029SD','deb HAPS','',65425,'Reentered','1998-08-30 00:00:00','1996-09-25 00:00:00',99.28,557,892,82.39,'Intermediate' from dual union all
select 24426,'024426','1994-029SE','deb STEP M2','',65425,'Reentered','1996-12-23 00:00:00','1996-09-25 00:00:00',96.66,545,653,81.81,'Intermediate' from dual union all
select 24427,'024427','1994-029SF','deb HAPS','',65425,'Reentered','1998-03-04 00:00:00','1996-09-26 00:00:00',98.66,596,794,81.86,'Intermediate' from dual union all
select 24428,'024428','1994-029SG','deb HAPS','',65425,'Reentered','1999-04-26 00:00:00','1996-09-26 00:00:00',107.52,547,1677,82.78,'Intermediate' from dual union all
select 24429,'024429','1994-029SH','deb HAPS','',65425,'Reentered','2000-12-30 00:00:00','1996-09-26 00:00:00',100,606,911,82.01,'Intermediate' from dual union all
select 2443,'002443','1966-088G','deb OGCh','',13495,'Reentered','1966-09-20 00:00:00','1966-09-18 00:00:00',93.03,239,607,50.3,'Intermediate' from dual union all
select 24430,'024430','1994-029SJ','deb HAPS','',65425,'Reentered','1999-03-09 00:00:00','1996-09-26 00:00:00',94.7,395,614,82.2,'Intermediate' from dual union all
select 24431,'024431','1994-029SK','deb HAPS','',65425,'Reentered','1996-12-03 00:00:00','1996-09-25 00:00:00',96.44,538,639,81.88,'Intermediate' from dual union all
select 24432,'024432','1994-029SL','deb HAPS','',65425,'Reentered','1996-12-24 00:00:00','1996-09-25 00:00:00',96.57,535,654,81.87,'Intermediate' from dual union all
select 24433,'024433','1994-029SM','deb HAPS','',65425,'Reentered','1997-03-19 00:00:00','1996-09-26 00:00:00',96.86,505,712,82.34,'Intermediate' from dual union all
select 24434,'024434','1994-029SN','deb HAPS','',65425,'Reentered','1996-10-12 00:00:00','1996-09-25 00:00:00',94.85,437,585,82.18,'Intermediate' from dual union all
select 24435,'024435','1996-058A','Ekspress-6','Ekspress No. 12L',65946,'In Earth orbit','','1996-10-26 00:00:00',1436.06,35779,35793,.17,'Stationary' from dual union all
select 24436,'024436','1996-058B','8S812','8S812',65946,'Reentered','1996-09-29 00:00:00','1996-09-28 00:00:00',88.03,174,179,51.63,'Intermediate' from dual union all
select 24437,'024437','1996-058C','Perekhodnik','Sredniy perekhodnik',65946,'Reentered','1996-09-27 00:00:00','1996-09-27 00:00:00',88.19,181,189,51.62,'Intermediate' from dual union all
select 24438,'024438','1996-058D','Blok DM-2M No. 2L','11S861-01 No. 2L',65946,'In Earth orbit','','1996-11-01 00:00:00',1437.26,35784,35835,.2,'Drift GEO' from dual union all
select 24439,'024439','1994-029SV','deb HAPS','',65425,'Reentered','1996-12-14 00:00:00','1996-09-25 00:00:00',96.87,502,715,82.02,'Intermediate' from dual union all
select 2444,'002444','1966-088H','deb OGCh','',13495,'Reentered','1966-09-25 00:00:00','1966-09-18 00:00:00',94.17,138,819,49.6,'Intermediate' from dual union all
select 24440,'024440','1994-029SW','deb HAPS','',65425,'Reentered','1999-08-14 00:00:00','1996-09-26 00:00:00',103.79,585,1291,82.18,'Intermediate' from dual union all
select 24441,'024441','1994-029SX','deb HAPS','',65425,'Reentered','2001-03-20 00:00:00','1996-09-26 00:00:00',105.17,591,1413,82.3,'Intermediate' from dual union all
select 24442,'024442','1994-029SY','deb HAPS','',65425,'Reentered','1997-06-26 00:00:00','1996-09-26 00:00:00',98.14,576,764,81.76,'Intermediate' from dual union all
select 24443,'024443','1994-029SZ','deb HAPS','',65425,'Reentered','1997-05-07 00:00:00','1996-09-26 00:00:00',98.02,558,770,81.81,'Intermediate' from dual union all
select 24444,'024444','1994-029TA','deb HAPS','',65425,'Reentered','1997-03-19 00:00:00','1996-09-25 00:00:00',97.9,549,767,81.94,'Intermediate' from dual union all
select 24445,'024445','1994-029TB','deb HAPS','',65425,'Reentered','1999-10-02 00:00:00','1996-09-25 00:00:00',103.67,599,1266,81.98,'Intermediate' from dual union all
select 24446,'024446','1994-029TC','deb HAPS','',65425,'Reentered','1997-10-25 00:00:00','1996-09-26 00:00:00',100.07,562,962,82,'Intermediate' from dual union all
select 24447,'024447','1994-029SP','deb STEP M2','',65425,'Reentered','1999-04-16 00:00:00','1996-09-26 00:00:00',107.03,579,1599,81.63,'Intermediate' from dual union all
select 24448,'024448','1994-029SQ','deb HAPS','',65425,'Reentered','1997-09-06 00:00:00','1996-09-26 00:00:00',98.98,530,890,82,'Intermediate' from dual union all
select 24449,'024449','1994-029SR','deb HAPS','',65425,'Reentered','1997-10-17 00:00:00','1996-09-26 00:00:00',100,545,972,82.09,'Intermediate' from dual union all
select 2445,'002445','1966-088J','deb OGCh','',13495,'Reentered','1966-09-24 00:00:00','1966-09-18 00:00:00',94.48,138,848,49.6,'Intermediate' from dual union all
select 24450,'024450','1994-029SS','deb HAPS','',65425,'Reentered','1997-10-04 00:00:00','1996-09-26 00:00:00',100.13,548,982,82.02,'Intermediate' from dual union all
select 24451,'024451','1994-029ST','deb HAPS','',65425,'Reentered','1997-01-31 00:00:00','1996-09-26 00:00:00',97.15,558,687,81.78,'Intermediate' from dual union all
select 24452,'024452','1994-029SU','deb HAPS','',65425,'Reentered','1997-01-15 00:00:00','1996-09-26 00:00:00',96.76,533,674,81.85,'Intermediate' from dual union all
select 24453,'024453','1994-029TD','deb HAPS','',65425,'Reentered','1997-08-13 00:00:00','1996-09-26 00:00:00',99.4,543,917,82.18,'Intermediate' from dual union all
select 24454,'024454','1996-058E','SOZ','SOZ',65946,'Reentered','1997-07-26 00:00:00','1996-10-29 00:00:00',634.58,293,35874,47.52,'GEO Transfer' from dual union all
select 24455,'024455','1995-058H','Zenit part','',65719,'Reentered','2001-01-22 00:00:00','1996-09-30 00:00:00',101.71,831,849,71.01,'Intermediate' from dual union all
select 24456,'024456','1995-071C','deb Kosmos-2326','',65732,'Reentered','1998-10-09 00:00:00','1996-10-03 00:00:00',92.79,403,420,65.02,'Intermediate' from dual union all
select 24457,'024457','1994-029TE','deb HAPS','',65425,'Reentered','1996-10-04 00:00:00','1996-10-04 00:00:00',89.41,236,255,82.12,'Intermediate' from dual union all
select 24458,'024458','1994-029TF','deb HAPS','',65425,'Reentered','2000-12-31 00:00:00','1996-10-07 00:00:00',98.43,611,757,81.82,'Intermediate' from dual union all
select 24459,'024459','1994-029TG','deb HAPS','',65425,'Reentered','1997-06-08 00:00:00','1996-10-07 00:00:00',99.11,488,945,82.16,'Intermediate' from dual union all
select 2446,'002446','1966-088K','deb OGCh','',13495,'Reentered','1966-10-03 00:00:00','1966-09-18 00:00:00',93.1,131,722,49.8,'Intermediate' from dual union all
select 24460,'024460','1994-029TH','deb HAPS','',65425,'Reentered','1997-03-05 00:00:00','1996-10-07 00:00:00',95.55,502,589,81.7,'Intermediate' from dual union all
select 24461,'024461','1994-029TJ','deb HAPS','',65425,'Reentered','1996-12-12 00:00:00','1996-10-06 00:00:00',96.65,488,709,82.16,'Intermediate' from dual union all
select 24462,'024462','1994-029TK','deb HAPS','',65425,'Reentered','1996-12-04 00:00:00','1996-10-06 00:00:00',96.2,514,640,81.84,'Intermediate' from dual union all
select 24463,'024463','1994-029TL','deb HAPS','',65425,'Reentered','1998-12-22 00:00:00','1996-10-07 00:00:00',104.11,562,1344,82.26,'Intermediate' from dual union all
select 24464,'024464','1994-029TM','deb HAPS','',65425,'Reentered','1997-08-05 00:00:00','1996-10-06 00:00:00',99.08,560,870,82.14,'Intermediate' from dual union all
select 24465,'024465','1994-029TN','deb HAPS','',65425,'Reentered','1997-08-20 00:00:00','1996-10-06 00:00:00',100.28,555,989,82.43,'Intermediate' from dual union all
select 24466,'024466','1994-029TP','deb HAPS','',65425,'Reentered','2001-10-18 00:00:00','1996-10-07 00:00:00',101.02,609,1005,81.94,'Intermediate' from dual union all
select 24467,'024467','1994-029TQ','deb HAPS','',65425,'Reentered','1996-12-17 00:00:00','1996-10-07 00:00:00',96.36,537,632,81.67,'Intermediate' from dual union all
select 24468,'024468','1994-029TR','deb HAPS','',65425,'Reentered','1996-12-04 00:00:00','1996-10-07 00:00:00',95.96,497,633,81.96,'Intermediate' from dual union all
select 24469,'024469','1994-029TS','deb HAPS','',65425,'Reentered','1997-01-14 00:00:00','1996-10-06 00:00:00',96.76,545,661,81.69,'Intermediate' from dual union all
select 2447,'002447','1966-088L','deb OGCh','',13495,'Reentered','1966-10-03 00:00:00','1966-09-18 00:00:00',98.97,180,1239,49.2,'Intermediate' from dual union all
select 24470,'024470','1994-029TT','deb HAPS','',65425,'Reentered','1997-03-22 00:00:00','1996-10-07 00:00:00',97.56,515,769,82.01,'Intermediate' from dual union all
select 24471,'024471','1994-029TU','deb HAPS','',65425,'Reentered','1996-12-30 00:00:00','1996-10-07 00:00:00',97.11,507,733,81.96,'Intermediate' from dual union all
select 24472,'024472','1994-029TV','deb HAPS','',65425,'Reentered','1997-07-15 00:00:00','1996-10-07 00:00:00',98.02,556,772,81.85,'Intermediate' from dual union all
select 24473,'024473','1994-029TW','deb HAPS','',65425,'Reentered','1997-05-25 00:00:00','1996-10-08 00:00:00',98.07,539,794,82.13,'Intermediate' from dual union all
select 24474,'024474','1994-029TX','deb HAPS','',65425,'Reentered','1997-01-24 00:00:00','1996-10-08 00:00:00',97.31,501,758,82.06,'Intermediate' from dual union all
select 24475,'024475','1994-029TY','deb HAPS','',65425,'Reentered','1998-12-28 00:00:00','1996-10-07 00:00:00',105.91,547,1527,82.19,'Intermediate' from dual union all
select 24476,'024476','1994-029TZ','deb HAPS','',65425,'Reentered','1998-01-06 00:00:00','1996-10-07 00:00:00',105.87,452,1619,81.68,'Intermediate' from dual union all
select 24477,'024477','1994-029UA','deb HAPS','',65425,'Reentered','2002-09-04 00:00:00','1996-10-07 00:00:00',112.08,563,2082,82.08,'Intermediate' from dual union all
select 24478,'024478','1994-029UB','deb HAPS','',65425,'Reentered','1997-09-10 00:00:00','1996-10-07 00:00:00',99.81,551,948,82.01,'Intermediate' from dual union all
select 24479,'024479','1994-029UC','deb HAPS','',65425,'Reentered','1996-11-17 00:00:00','1996-10-07 00:00:00',95.62,497,601,81.91,'Intermediate' from dual union all
select 2448,'002448','1966-088M','deb OGCh','',13495,'Reentered','1966-10-01 00:00:00','1966-09-18 00:00:00',94.15,132,822,49.7,'Intermediate' from dual union all
select 24480,'024480','1994-029UD','deb HAPS','',65425,'Reentered','1996-12-18 00:00:00','1996-10-07 00:00:00',97.07,494,742,82.01,'Intermediate' from dual union all
select 24481,'024481','1994-029UE','deb HAPS','',65425,'Reentered','2000-03-10 00:00:00','1996-10-07 00:00:00',101.66,600,1075,81.81,'Intermediate' from dual union all
select 24482,'024482','1994-029UF','deb HAPS','',65425,'Reentered','1996-10-13 00:00:00','1996-10-06 00:00:00',93.39,410,471,81.75,'Intermediate' from dual union all
select 24483,'024483','1994-029UG','deb HAPS','',65425,'Reentered','1997-03-03 00:00:00','1996-10-07 00:00:00',97.33,514,748,81.96,'Intermediate' from dual union all
select 24484,'024484','1994-029UH','deb HAPS','',65425,'Reentered','1997-01-05 00:00:00','1996-10-07 00:00:00',97.09,492,747,82.17,'Intermediate' from dual union all
select 24485,'024485','1994-029UJ','deb HAPS','',65425,'Reentered','1997-09-03 00:00:00','1996-10-07 00:00:00',99.2,549,892,82.02,'Intermediate' from dual union all
select 24486,'024486','1994-029UK','deb HAPS','',65425,'Reentered','1997-12-06 00:00:00','1996-10-07 00:00:00',95.49,478,606,82.24,'Intermediate' from dual union all
select 24487,'024487','1994-029UL','deb HAPS','',65425,'In Earth orbit','','1996-10-07 00:00:00',98.58,583,798,81.97,'Intermediate' from dual union all
select 24488,'024488','1994-029UM','deb HAPS','',65425,'Reentered','1999-07-17 00:00:00','1996-10-06 00:00:00',97.73,559,742,81.91,'Intermediate' from dual;
insert into SATELLITE
select 24489,'024489','1994-029UN','deb HAPS','',65425,'Reentered','1998-08-26 00:00:00','1996-10-07 00:00:00',96.07,474,667,81.9,'Intermediate' from dual union all
select 2449,'002449','1966-088N','deb OGCh','',13495,'Reentered','1966-09-30 00:00:00','1966-09-18 00:00:00',94.66,157,848,49.7,'Intermediate' from dual union all
select 24490,'024490','1994-029UP','deb HAPS','',65425,'Reentered','1999-06-27 00:00:00','1996-10-07 00:00:00',97.36,575,690,82.13,'Intermediate' from dual union all
select 24491,'024491','1994-029UQ','deb HAPS','',65425,'Reentered','1999-09-14 00:00:00','1996-10-07 00:00:00',97.8,555,752,81.83,'Intermediate' from dual union all
select 24492,'024492','1994-029UR','deb HAPS','',65425,'Reentered','1997-10-15 00:00:00','1996-10-07 00:00:00',100.07,601,922,82.19,'Intermediate' from dual union all
select 24493,'024493','1994-029US','deb HAPS','',65425,'Reentered','2002-05-31 00:00:00','1996-10-07 00:00:00',100.01,608,910,82,'Intermediate' from dual union all
select 24494,'024494','1994-029UT','deb HAPS','',65425,'Reentered','1997-12-02 00:00:00','1996-10-07 00:00:00',97.81,577,731,81.83,'Intermediate' from dual union all
select 24495,'024495','1994-029UU','deb HAPS','',65425,'Reentered','1999-08-13 00:00:00','1996-10-07 00:00:00',98.04,531,799,82.01,'Intermediate' from dual union all
select 24496,'024496','1994-029UV','deb HAPS','',65425,'Reentered','1997-03-09 00:00:00','1996-10-07 00:00:00',97.48,534,743,82.04,'Intermediate' from dual union all
select 24497,'024497','1994-029UW','deb HAPS','',65425,'Reentered','1997-03-24 00:00:00','1996-10-07 00:00:00',97.68,515,781,81.99,'Intermediate' from dual union all
select 24498,'024498','1994-029UX','deb HAPS','',65425,'Reentered','1997-05-03 00:00:00','1996-10-07 00:00:00',98.59,508,875,82.14,'Intermediate' from dual union all
select 24499,'024499','1994-029UY','deb HAPS','',65425,'Reentered','1997-11-16 00:00:00','1996-10-06 00:00:00',97.03,464,769,82.15,'Intermediate' from dual union all
select 245,'000245','61 Omicron 73','deb r/b Ablestar','',7273,'In Earth orbit','','1962-02-21 00:00:00',103.73,869,1000,66.09,'Intermediate' from dual union all
select 2450,'002450','1966-088P','deb OGCh','',13495,'Reentered','1966-09-26 00:00:00','1966-09-18 00:00:00',93.88,133,796,49.7,'Intermediate' from dual union all
select 24500,'024500','1994-029UZ','deb HAPS','',65425,'Reentered','2002-07-28 00:00:00','1996-10-07 00:00:00',102.28,615,1119,81.82,'Intermediate' from dual union all
select 24501,'024501','1994-029VA','deb HAPS','',65425,'Reentered','2014-09-22 00:00:00','1996-10-07 00:00:00',98.68,566,826,82.09,'Intermediate' from dual union all
select 24502,'024502','1994-029VB','deb HAPS','',65425,'Reentered','1996-11-09 00:00:00','1996-10-06 00:00:00',95.07,479,565,81.86,'Intermediate' from dual union all
select 24503,'024503','1996-058F','SOZ','SOZ',65946,'Reentered','1997-04-30 00:00:00','1996-10-26 00:00:00',632.79,229,35846,47.46,'GEO Transfer' from dual union all
select 24504,'024504','1994-029VC','deb HAPS','',65425,'Reentered','1996-10-09 00:00:00','1996-10-08 00:00:00',91.87,352,381,82.18,'Intermediate' from dual union all
select 24505,'024505','1994-029VD','deb HAPS','',65425,'Reentered','1996-10-13 00:00:00','1996-10-06 00:00:00',93.55,395,502,82.03,'Intermediate' from dual union all
select 24506,'024506','1994-029VE','deb HAPS','',65425,'Reentered','1996-10-27 00:00:00','1996-10-06 00:00:00',93.97,406,532,82.04,'Intermediate' from dual union all
select 24507,'024507','1994-029VF','deb HAPS','',65425,'Reentered','1996-10-21 00:00:00','1996-10-08 00:00:00',94.26,462,504,81.86,'Intermediate' from dual union all
select 24508,'024508','1994-029VG','deb HAPS','',65425,'Reentered','1997-01-10 00:00:00','1996-10-05 00:00:00',94.44,437,546,81.29,'Intermediate' from dual union all
select 24509,'024509','1994-029VH','deb HAPS','',65425,'Reentered','1996-10-21 00:00:00','1996-10-06 00:00:00',94.52,452,539,81.71,'Intermediate' from dual union all
select 2451,'002451','1966-088Q','deb OGCh','',13495,'Reentered','1966-10-02 00:00:00','1966-09-18 00:00:00',91.19,127,539,49.6,'Intermediate' from dual union all
select 24510,'024510','1994-029VJ','deb HAPS','',65425,'Reentered','1997-04-13 00:00:00','1996-10-06 00:00:00',94.99,469,568,82.38,'Intermediate' from dual union all
select 24511,'024511','1994-029VK','deb HAPS','',65425,'Reentered','1997-10-11 00:00:00','1996-10-08 00:00:00',95.04,447,594,82.13,'Intermediate' from dual union all
select 24512,'024512','1994-029VL','deb HAPS','',65425,'Reentered','1996-11-04 00:00:00','1996-10-07 00:00:00',95.25,459,603,81.85,'Intermediate' from dual union all
select 24513,'024513','1994-029VM','deb HAPS','',65425,'Reentered','1997-09-09 00:00:00','1996-10-08 00:00:00',95.39,443,632,81.83,'Intermediate' from dual union all
select 24514,'024514','1994-029VN','deb HAPS','',65425,'Reentered','1996-11-13 00:00:00','1996-10-08 00:00:00',95.55,510,580,81.86,'Intermediate' from dual union all
select 24515,'024515','1994-029VP','deb HAPS','',65425,'Reentered','2001-10-24 00:00:00','1996-10-09 00:00:00',98.72,565,830,82.19,'Intermediate' from dual union all
select 24516,'024516','1994-029VQ','deb HAPS','',65425,'Reentered','1997-01-13 00:00:00','1996-10-09 00:00:00',96.72,523,681,81.78,'Intermediate' from dual union all
select 24517,'024517','1994-029VR','deb HAPS','',65425,'Reentered','2001-10-30 00:00:00','1996-10-09 00:00:00',101.11,607,1016,81.97,'Intermediate' from dual union all
select 24518,'024518','1994-029VS','deb HAPS','',65425,'Reentered','1998-01-14 00:00:00','1996-10-07 00:00:00',100.43,569,989,81.7,'Intermediate' from dual union all
select 24519,'024519','1994-029VT','deb HAPS','',65425,'Reentered','1997-01-12 00:00:00','1996-10-07 00:00:00',97.45,501,773,82.03,'Intermediate' from dual union all
select 2452,'002452','1966-088R','deb OGCh','',13495,'Reentered','1966-09-22 00:00:00','1966-09-18 00:00:00',94.68,152,855,49.8,'Intermediate' from dual union all
select 24520,'024520','1994-029VU','deb HAPS','',65425,'Reentered','1998-11-18 00:00:00','1996-10-09 00:00:00',97.88,584,731,82.17,'Intermediate' from dual union all
select 24521,'024521','1994-029VV','deb HAPS','',65425,'Reentered','2002-07-09 00:00:00','1996-10-07 00:00:00',98.48,603,769,82.1,'Intermediate' from dual union all
select 24522,'024522','1994-029VW','deb HAPS','',65425,'Reentered','1999-12-29 00:00:00','1996-10-09 00:00:00',98.45,568,802,81.99,'Intermediate' from dual union all
select 24523,'024523','1994-029VX','deb HAPS','',65425,'Reentered','1997-04-02 00:00:00','1996-10-09 00:00:00',97.84,512,799,82.06,'Intermediate' from dual union all
select 24524,'024524','1994-029VY','deb HAPS','',65425,'Reentered','1998-02-14 00:00:00','1996-10-09 00:00:00',102.55,535,1224,81.78,'Intermediate' from dual union all
select 24525,'024525','1994-029VZ','deb HAPS','',65425,'Reentered','1997-04-04 00:00:00','1996-10-07 00:00:00',97.77,546,758,81.87,'Intermediate' from dual union all
select 24526,'024526','1994-029WA','deb HAPS','',65425,'Reentered','1997-12-21 00:00:00','1996-10-08 00:00:00',101.19,563,1067,82.07,'Intermediate' from dual union all
select 24527,'024527','1994-029WB','deb HAPS','',65425,'Reentered','1997-11-19 00:00:00','1996-10-07 00:00:00',102.42,560,1187,82.06,'Intermediate' from dual union all
select 24528,'024528','1994-029WC','deb HAPS','',65425,'Reentered','2000-02-18 00:00:00','1996-10-09 00:00:00',98.62,570,816,82.01,'Intermediate' from dual union all
select 24529,'024529','1994-029WD','deb HAPS','',65425,'In Earth orbit','','1996-10-09 00:00:00',98.39,567,797,81.98,'Intermediate' from dual union all
select 2453,'002453','1966-088S','deb OGCh','',13495,'Reentered','1966-10-04 00:00:00','1966-09-18 00:00:00',96.14,187,961,49.5,'Intermediate' from dual union all
select 24530,'024530','1994-029WE','deb HAPS','',65425,'Reentered','1996-12-02 00:00:00','1996-10-06 00:00:00',96.46,520,659,81.81,'Intermediate' from dual union all
select 24531,'024531','1994-029WF','deb HAPS','',65425,'Reentered','1999-11-20 00:00:00','1996-10-09 00:00:00',104.53,599,1345,81.68,'Intermediate' from dual union all
select 24532,'024532','1994-029WG','deb HAPS','',65425,'Reentered','1999-01-21 00:00:00','1996-10-09 00:00:00',97.3,597,662,81.92,'Intermediate' from dual union all
select 24533,'024533','1994-029WH','deb HAPS','',65425,'Reentered','1998-10-07 00:00:00','1996-10-09 00:00:00',99.07,589,840,81.83,'Intermediate' from dual union all
select 24534,'024534','1994-029WJ','deb HAPS','',65425,'Reentered','1999-11-18 00:00:00','1996-10-09 00:00:00',97.7,481,817,81.5,'Intermediate' from dual union all
select 24535,'024535','1994-029WK','deb HAPS','',65425,'Reentered','1998-09-16 00:00:00','1996-10-09 00:00:00',103.13,533,1280,82.38,'Intermediate' from dual union all
select 24536,'024536','1994-029WL','deb HAPS','',65425,'Reentered','1998-01-09 00:00:00','1996-10-09 00:00:00',98.49,551,822,82.07,'Intermediate' from dual union all
select 24537,'024537','1994-029WM','deb HAPS','',65425,'Reentered','1997-11-21 00:00:00','1996-10-09 00:00:00',99.18,552,887,82.11,'Intermediate' from dual union all
select 24538,'024538','1994-029WN','deb HAPS','',65425,'Reentered','1996-12-05 00:00:00','1996-10-09 00:00:00',96.64,490,706,82.04,'Intermediate' from dual union all
select 24539,'024539','1994-029WP','deb HAPS','',65425,'Reentered','1997-01-02 00:00:00','1996-10-09 00:00:00',96.37,520,650,81.83,'Intermediate' from dual union all
select 2454,'002454','1966-088T','deb OGCh','',13495,'Reentered','1966-10-02 00:00:00','1966-09-18 00:00:00',92.14,190,569,51.1,'Intermediate' from dual union all
select 24540,'024540','1994-029WQ','deb HAPS','',65425,'Reentered','2000-04-10 00:00:00','1996-10-09 00:00:00',99.38,465,992,82.42,'Intermediate' from dual union all
select 24541,'024541','1994-029WR','deb HAPS','',65425,'Reentered','1996-10-15 00:00:00','1996-09-25 00:00:00',92.76,364,455,82.33,'Intermediate' from dual union all
select 24542,'024542','1994-029WS','deb HAPS','',65425,'Reentered','1999-01-28 00:00:00','1996-10-08 00:00:00',99.44,606,858,81.79,'Intermediate' from dual union all
select 24543,'024543','1994-029WT','deb HAPS','',65425,'Reentered','1997-11-10 00:00:00','1996-10-09 00:00:00',98.44,590,779,82.2,'Intermediate' from dual union all
select 24544,'024544','1994-029WU','deb HAPS','',65425,'Reentered','1997-02-09 00:00:00','1996-10-07 00:00:00',97.39,461,807,82.1,'Intermediate' from dual union all
select 24545,'024545','1994-029WV','deb HAPS','',65425,'Reentered','1999-04-27 00:00:00','1996-10-11 00:00:00',96.06,499,640,82.01,'Intermediate' from dual union all
select 24546,'024546','1994-029WW','deb HAPS','',65425,'Reentered','1996-12-13 00:00:00','1996-10-11 00:00:00',96.03,478,659,82.02,'Intermediate' from dual union all
select 24547,'024547','1994-029WX','deb HAPS','',65425,'In Earth orbit','','1996-10-11 00:00:00',98.79,568,834,82.05,'Intermediate' from dual union all
select 24548,'024548','1994-029WY','deb HAPS','',65425,'Reentered','1998-03-24 00:00:00','1996-10-11 00:00:00',96.76,597,610,82.05,'Intermediate' from dual union all
select 24549,'024549','1994-029WZ','deb HAPS','',65425,'Reentered','1997-06-18 00:00:00','1996-10-10 00:00:00',97.95,547,775,82.12,'Intermediate' from dual union all
select 2455,'002455','1966-088U','deb OGCh','',13495,'Reentered','1966-10-04 00:00:00','1966-09-18 00:00:00',94.81,150,869,49.4,'Intermediate' from dual union all
select 24550,'024550','1994-029XA','deb HAPS','',65425,'Reentered','1997-10-06 00:00:00','1996-10-11 00:00:00',103.99,522,1373,82.28,'Intermediate' from dual union all
select 24551,'024551','1994-029XB','deb HAPS','',65425,'Reentered','1997-05-07 00:00:00','1996-10-09 00:00:00',98.73,525,871,82.07,'Intermediate' from dual union all
select 24552,'024552','1994-029XC','deb HAPS','',65425,'Reentered','1998-06-23 00:00:00','1996-10-11 00:00:00',97.36,571,694,82.05,'Intermediate' from dual union all
select 24553,'024553','1994-029XD','deb HAPS','',65425,'Reentered','1998-01-01 00:00:00','1996-10-11 00:00:00',100.7,564,1020,82.09,'Intermediate' from dual union all
select 24554,'024554','1994-029XE','deb HAPS','',65425,'Reentered','1998-05-13 00:00:00','1996-10-11 00:00:00',101.68,528,1149,81.66,'Intermediate' from dual union all
select 24555,'024555','1994-029XF','deb HAPS','',65425,'Reentered','1997-06-10 00:00:00','1996-10-11 00:00:00',98.66,550,840,81.78,'Intermediate' from dual union all
select 24556,'024556','1994-029XG','deb HAPS','',65425,'Reentered','1997-04-15 00:00:00','1996-10-10 00:00:00',98.17,549,794,81.66,'Intermediate' from dual union all
select 24557,'024557','1994-029XH','deb HAPS','',65425,'Reentered','1997-11-05 00:00:00','1996-10-11 00:00:00',100.02,565,954,82,'Intermediate' from dual union all
select 24558,'024558','1994-029XJ','deb HAPS','',65425,'Reentered','1996-10-12 00:00:00','1996-10-09 00:00:00',92.53,376,421,82.21,'Intermediate' from dual union all
select 24559,'024559','1994-029XK','deb HAPS','',65425,'Reentered','1997-06-24 00:00:00','1996-10-11 00:00:00',98.72,506,889,82.05,'Intermediate' from dual union all
select 2456,'002456','1966-088V','deb OGCh','',13495,'Reentered','1966-09-28 00:00:00','1966-09-18 00:00:00',89.13,161,301,49.5,'Intermediate' from dual union all
select 24560,'024560','1994-029XL','deb HAPS','',65425,'Reentered','1997-12-07 00:00:00','1996-10-10 00:00:00',100.53,589,978,82.08,'Intermediate' from dual union all
select 24561,'024561','1994-029XM','deb HAPS','',65425,'Reentered','1998-09-21 00:00:00','1996-10-10 00:00:00',99.21,538,904,82.3,'Intermediate' from dual union all
select 24562,'024562','1994-029XN','deb HAPS','',65425,'Reentered','1996-11-24 00:00:00','1996-10-09 00:00:00',95.59,492,603,81.82,'Intermediate' from dual union all
select 24563,'024563','1994-029XP','deb HAPS','',65425,'Reentered','1997-05-30 00:00:00','1996-10-10 00:00:00',96.32,388,777,82.32,'Intermediate' from dual union all
select 24564,'024564','1994-029XQ','deb HAPS','',65425,'Reentered','1997-09-17 00:00:00','1996-10-11 00:00:00',100.28,534,1009,82.19,'Intermediate' from dual union all
select 24565,'024565','1994-029XR','deb HAPS','',65425,'Reentered','1999-02-01 00:00:00','1996-10-11 00:00:00',96.36,485,684,82.15,'Intermediate' from dual union all
select 24566,'024566','1994-029XS','deb HAPS','',65425,'Reentered','2000-10-22 00:00:00','1996-10-12 00:00:00',101.66,610,1064,82.1,'Intermediate' from dual union all
select 24567,'024567','1994-029XT','deb HAPS','',65425,'Reentered','1998-01-18 00:00:00','1996-10-12 00:00:00',101.77,566,1119,81.78,'Intermediate' from dual union all
select 24568,'024568','1994-029XU','deb HAPS','',65425,'Reentered','1997-05-20 00:00:00','1996-10-12 00:00:00',98.95,539,878,82.05,'Intermediate' from dual union all
select 24569,'024569','1994-029XV','deb HAPS','',65425,'Reentered','1997-01-15 00:00:00','1996-10-12 00:00:00',97.43,516,755,82.07,'Intermediate' from dual union all
select 2457,'002457','1966-088W','deb OGCh','',13495,'Reentered','1966-09-26 00:00:00','1966-09-18 00:00:00',88.54,143,262,49.1,'Intermediate' from dual union all
select 24570,'024570','1994-029XW','deb HAPS','',65425,'Reentered','1997-01-03 00:00:00','1996-10-12 00:00:00',95.67,455,647,81.93,'Intermediate' from dual union all
select 24571,'024571','1994-029XX','deb HAPS','',65425,'Reentered','1997-10-18 00:00:00','1996-10-12 00:00:00',99.44,556,908,81.74,'Intermediate' from dual union all
select 24572,'024572','1994-029XY','deb HAPS','',65425,'Reentered','1996-12-02 00:00:00','1996-10-12 00:00:00',95.84,487,632,81.88,'Intermediate' from dual union all
select 24573,'024573','1994-029XZ','deb HAPS','',65425,'Reentered','1997-11-01 00:00:00','1996-10-12 00:00:00',101.17,533,1096,82.43,'Intermediate' from dual union all
select 24574,'024574','1994-029YA','deb HAPS','',65425,'Reentered','1998-04-11 00:00:00','1996-10-11 00:00:00',96.05,528,611,81.91,'Intermediate' from dual union all
select 24575,'024575','1994-029YB','deb HAPS','',65425,'Reentered','1997-12-15 00:00:00','1996-10-12 00:00:00',101.35,547,1099,82.22,'Intermediate' from dual union all
select 24576,'024576','1994-029YC','deb HAPS','',65425,'Reentered','1997-04-18 00:00:00','1996-10-12 00:00:00',97.8,557,749,81.7,'Intermediate' from dual union all
select 24577,'024577','1994-029YD','deb HAPS','',65425,'Reentered','1997-10-23 00:00:00','1996-10-11 00:00:00',99.7,540,949,82.11,'Intermediate' from dual union all
select 24578,'024578','1994-029YE','deb HAPS','',65425,'Reentered','1997-01-30 00:00:00','1996-10-12 00:00:00',96.98,540,689,81.86,'Intermediate' from dual;
insert into SATELLITE
select 24579,'024579','1994-029YF','deb HAPS','',65425,'Reentered','2000-04-24 00:00:00','1996-10-11 00:00:00',106.24,597,1508,82.04,'Intermediate' from dual union all
select 2458,'002458','1966-088X','deb OGCh','',13495,'Reentered','1966-09-30 00:00:00','1966-09-18 00:00:00',91.46,135,558,49.4,'Intermediate' from dual union all
select 24580,'024580','1994-029YG','deb HAPS','',65425,'Reentered','1998-07-10 00:00:00','1996-10-11 00:00:00',107.62,534,1700,82.24,'Intermediate' from dual union all
select 24581,'024581','1994-029YH','deb HAPS','',65425,'In Earth orbit','','1996-10-12 00:00:00',112.07,583,2061,81.62,'Intermediate' from dual union all
select 24582,'024582','1994-029YJ','deb HAPS','',65425,'Reentered','1999-07-29 00:00:00','1996-10-12 00:00:00',97.63,577,714,82.02,'Intermediate' from dual union all
select 24583,'024583','1994-029YK','deb HAPS','',65425,'Reentered','1998-06-29 00:00:00','1996-10-11 00:00:00',95.37,469,604,81.42,'Intermediate' from dual union all
select 24584,'024584','1994-029YL','deb HAPS','',65425,'Reentered','1996-12-02 00:00:00','1996-10-09 00:00:00',95.91,509,616,81.83,'Intermediate' from dual union all
select 24585,'024585','1994-029YM','deb HAPS','',65425,'Reentered','2001-11-15 00:00:00','1996-10-10 00:00:00',100.67,499,1082,81.85,'Intermediate' from dual union all
select 24586,'024586','1994-029YN','deb HAPS','',65425,'Reentered','1999-10-30 00:00:00','1996-10-12 00:00:00',98.03,598,730,81.84,'Intermediate' from dual union all
select 24587,'024587','1994-029YP','deb HAPS','',65425,'Reentered','1998-09-16 00:00:00','1996-10-11 00:00:00',99.55,560,915,82.39,'Intermediate' from dual union all
select 24588,'024588','1994-029YQ','deb HAPS','',65425,'Reentered','1997-12-01 00:00:00','1996-10-12 00:00:00',101.3,553,1088,82.12,'Intermediate' from dual union all
select 24589,'024589','1994-029YR','deb HAPS','',65425,'Reentered','1999-06-24 00:00:00','1996-10-12 00:00:00',99.68,445,1042,82.36,'Intermediate' from dual union all
select 2459,'002459','1966-088Y','deb OGCh','',13495,'Reentered','1966-10-03 00:00:00','1966-09-18 00:00:00',96.33,206,960,49.4,'Intermediate' from dual union all
select 24590,'024590','1994-029YS','deb HAPS','',65425,'Reentered','1999-10-31 00:00:00','1996-10-12 00:00:00',98.74,612,785,81.87,'Intermediate' from dual union all
select 24591,'024591','1994-029YT','deb HAPS','',65425,'Reentered','1997-01-02 00:00:00','1996-10-12 00:00:00',96.5,525,657,81.85,'Intermediate' from dual union all
select 24592,'024592','1994-029YU','deb HAPS','',65425,'Reentered','1997-02-19 00:00:00','1996-10-12 00:00:00',97.52,518,762,82.05,'Intermediate' from dual union all
select 24593,'024593','1994-029YV','deb HAPS','',65425,'Reentered','1996-11-22 00:00:00','1996-10-12 00:00:00',95.82,515,602,81.84,'Intermediate' from dual union all
select 24594,'024594','1994-029YW','deb HAPS','',65425,'In Earth orbit','','1996-10-12 00:00:00',107.05,557,1624,81.49,'Intermediate' from dual union all
select 24595,'024595','1994-029YX','deb HAPS','',65425,'Reentered','1996-12-28 00:00:00','1996-10-12 00:00:00',95.51,485,602,81.89,'Intermediate' from dual union all
select 24596,'024596','1994-029YY','deb HAPS','',65425,'Reentered','1997-07-05 00:00:00','1996-10-14 00:00:00',98.81,520,884,82.06,'Intermediate' from dual union all
select 24597,'024597','1994-029YZ','deb HAPS','',65425,'Reentered','1996-11-07 00:00:00','1996-10-15 00:00:00',95.95,453,676,82.19,'Intermediate' from dual union all
select 24598,'024598','1994-029ZA','deb HAPS','',65425,'Reentered','1999-05-03 00:00:00','1996-10-15 00:00:00',98.01,598,729,82.1,'Intermediate' from dual union all
select 24599,'024599','1994-029ZB','deb HAPS','',65425,'Reentered','2002-08-03 00:00:00','1996-10-15 00:00:00',98.63,495,892,81.91,'Intermediate' from dual union all
select 246,'000246','61 Omicron 74','deb r/b Ablestar','',7273,'Reentered','1994-05-10 00:00:00','1962-02-21 00:00:00',104.12,902,1005,66.85,'Intermediate' from dual union all
select 2460,'002460','1966-088Z','deb OGCh','',13495,'Reentered','1966-09-22 00:00:00','1966-09-18 00:00:00',96.28,147,1014,49.4,'Intermediate' from dual union all
select 24600,'024600','1994-029ZC','deb HAPS','',65425,'Reentered','1997-03-31 00:00:00','1996-10-15 00:00:00',98.75,474,924,82.14,'Intermediate' from dual union all
select 24601,'024601','1994-029ZD','deb HAPS','',65425,'Reentered','1997-06-04 00:00:00','1996-10-15 00:00:00',96.23,458,697,82.06,'Intermediate' from dual union all
select 24602,'024602','1994-029ZE','deb HAPS','',65425,'In Earth orbit','','1996-10-15 00:00:00',111.09,568,1986,81.48,'Intermediate' from dual union all
select 24603,'024603','1994-029ZF','deb HAPS','',65425,'Reentered','1998-11-29 00:00:00','1996-10-15 00:00:00',97.58,565,721,81.87,'Intermediate' from dual union all
select 24604,'024604','1994-029ZG','deb HAPS','',65425,'Reentered','1997-07-16 00:00:00','1996-10-15 00:00:00',100.27,523,1020,82.25,'Intermediate' from dual union all
select 24605,'024605','1994-029ZH','deb HAPS','',65425,'Reentered','1999-04-21 00:00:00','1996-10-14 00:00:00',98.09,589,746,81.51,'Intermediate' from dual union all
select 24606,'024606','1994-029ZJ','deb HAPS','',65425,'Reentered','2013-02-26 00:00:00','1996-10-13 00:00:00',103.95,612,1278,81.83,'Intermediate' from dual union all
select 24607,'024607','1994-029ZK','deb HAPS','',65425,'Reentered','1997-10-18 00:00:00','1996-10-15 00:00:00',99.43,558,904,81.97,'Intermediate' from dual union all
select 24608,'024608','1994-029ZL','deb HAPS','',65425,'Reentered','1997-09-16 00:00:00','1996-10-15 00:00:00',99.36,545,911,82,'Intermediate' from dual union all
select 24609,'024609','1994-029ZM','deb HAPS','',65425,'Reentered','1999-06-23 00:00:00','1996-10-15 00:00:00',103.4,583,1256,82.13,'Intermediate' from dual union all
select 2461,'002461','1966-088AA','deb OGCh','',13495,'Reentered','1966-09-24 00:00:00','1966-09-18 00:00:00',92.97,119,721,49.5,'Intermediate' from dual union all
select 24610,'024610','1994-029ZN','deb HAPS','',65425,'Reentered','2001-12-31 00:00:00','1996-10-15 00:00:00',98.15,608,732,81.99,'Intermediate' from dual union all
select 24611,'024611','1994-029ZP','deb HAPS','',65425,'Reentered','1997-04-12 00:00:00','1996-10-16 00:00:00',97.87,517,797,81.99,'Intermediate' from dual union all
select 24612,'024612','1994-029ZQ','deb HAPS','',65425,'Reentered','2000-03-16 00:00:00','1996-10-16 00:00:00',107.48,593,1628,81.5,'Intermediate' from dual union all
select 24613,'024613','1994-029ZR','deb HAPS','',65425,'Reentered','1997-04-10 00:00:00','1996-10-15 00:00:00',98.65,515,874,82.12,'Intermediate' from dual union all
select 24614,'024614','1994-029ZS','deb HAPS','',65425,'Reentered','2000-02-01 00:00:00','1996-10-14 00:00:00',98.03,601,728,82.08,'Intermediate' from dual union all
select 24615,'024615','1994-029ZT','deb HAPS','',65425,'Reentered','1998-04-13 00:00:00','1996-10-16 00:00:00',95.62,396,702,82.2,'Intermediate' from dual union all
select 24616,'024616','1994-029ZU','deb HAPS','',65425,'Reentered','1997-02-22 00:00:00','1996-10-16 00:00:00',97.32,492,769,82.11,'Intermediate' from dual union all
select 24617,'024617','1994-029ZV','deb HAPS','',65425,'Reentered','1997-04-09 00:00:00','1996-10-16 00:00:00',97.42,561,710,81.78,'Intermediate' from dual union all
select 24618,'024618','1994-029ZW','deb HAPS','',65425,'Reentered','2000-03-26 00:00:00','1996-10-15 00:00:00',104.71,578,1384,82.23,'Intermediate' from dual union all
select 24619,'024619','1994-029ZX','deb HAPS','',65425,'Reentered','1997-04-04 00:00:00','1996-10-16 00:00:00',98.05,523,808,81.82,'Intermediate' from dual union all
select 2462,'002462','1966-088AB','deb OGCh','',13495,'Reentered','1966-09-27 00:00:00','1966-09-18 00:00:00',95.53,172,917,49.7,'Intermediate' from dual union all
select 24620,'024620','1994-029ZY','deb HAPS','',65425,'Reentered','2000-01-04 00:00:00','1996-10-15 00:00:00',98.72,572,823,82.1,'Intermediate' from dual union all
select 24621,'024621','1994-029ZZ','deb HAPS','',65425,'Reentered','1997-10-05 00:00:00','1996-10-16 00:00:00',99.84,540,962,81.97,'Intermediate' from dual union all
select 24622,'024622','1994-029AAA','deb HAPS','',65425,'Reentered','1997-10-23 00:00:00','1996-10-16 00:00:00',98.78,509,892,82.32,'Intermediate' from dual union all
select 24623,'024623','1994-029AAB','deb HAPS','',65425,'Reentered','1997-09-30 00:00:00','1996-10-14 00:00:00',88.75,210,216,39.01,'Intermediate' from dual union all
select 24624,'024624','1994-029AAC','deb HAPS','',65425,'Reentered','1999-03-31 00:00:00','1996-10-17 00:00:00',105.84,600,1468,83,'Intermediate' from dual union all
select 24625,'024625','1994-029AAD','deb HAPS','',65425,'Reentered','1998-04-25 00:00:00','1996-10-17 00:00:00',101.45,556,1099,81.76,'Intermediate' from dual union all
select 24626,'024626','1994-029AAE','deb HAPS','',65425,'Reentered','2014-09-26 00:00:00','1996-10-17 00:00:00',99.58,602,874,82.02,'Intermediate' from dual union all
select 24627,'024627','1994-029AAF','deb HAPS','',65425,'Reentered','2000-03-06 00:00:00','1996-10-16 00:00:00',98.77,564,836,82,'Intermediate' from dual union all
select 24628,'024628','1994-029AAG','deb HAPS','',65425,'Reentered','2001-10-26 00:00:00','1996-10-17 00:00:00',100.11,604,924,82.01,'Intermediate' from dual union all
select 24629,'024629','1994-029AAH','deb HAPS','',65425,'Reentered','1997-11-15 00:00:00','1996-10-17 00:00:00',102.2,540,1185,82.24,'Intermediate' from dual union all
select 2463,'002463','1966-088AC','deb OGCh','',13495,'Reentered','1966-09-26 00:00:00','1966-09-18 00:00:00',97.33,190,1071,50.4,'Intermediate' from dual union all
select 24630,'024630','1994-029AAJ','deb HAPS','',65425,'Reentered','1997-11-10 00:00:00','1996-10-15 00:00:00',100.62,569,1007,82.03,'Intermediate' from dual union all
select 24631,'024631','1994-029AAK','deb HAPS','',65425,'Reentered','2002-11-11 00:00:00','1996-10-17 00:00:00',109.64,602,1819,82.1,'Intermediate' from dual union all
select 24632,'024632','1994-029AAL','deb HAPS','',65425,'Reentered','1999-10-20 00:00:00','1996-10-17 00:00:00',97.62,540,750,81.91,'Intermediate' from dual union all
select 24633,'024633','1994-029AAM','deb HAPS','',65425,'Reentered','1998-05-21 00:00:00','1996-10-17 00:00:00',103.63,503,1358,81.42,'Intermediate' from dual union all
select 24634,'024634','1996-059A','Fanhui Shi Weixing (17)','Jianbing-1B No. 3',65947,'Reentered','1996-12-03 00:00:00','1996-10-27 00:00:00',89.6,170,340,63.04,'Intermediate' from dual union all
select 24635,'024635','1996-059B','CZ-2D Stage 2','CZ-2D Stage 2',65947,'Reentered','1996-10-29 00:00:00','1996-10-22 00:00:00',89.38,169,319,63.04,'Intermediate' from dual union all
select 24636,'024636','1990-031J','deb Stacksat','',63273,'Reentered','2002-06-03 00:00:00','1996-11-01 00:00:00',91.57,347,357,89.75,'Polar' from dual union all
select 24637,'024637','1991-076J','deb USA 77','',64245,'In Earth orbit','','1996-10-30 00:00:00',107.47,1050,1170,63.42,'Intermediate' from dual union all
select 24638,'024638','1991-076K','deb USA 77','',64245,'In Earth orbit','','1996-10-30 00:00:00',107.47,1050,1170,63.42,'Intermediate' from dual union all
select 24639,'024639','1978-096G','deb Tiros-N','',45291,'Reentered','1998-08-11 00:00:00','1996-10-24 00:00:00',101.26,809,828,98.88,'Sun-Synch' from dual union all
select 2464,'002464','1966-088AD','deb OGCh','',13495,'Reentered','1966-09-27 00:00:00','1966-09-18 00:00:00',95.99,170,963,49.8,'Intermediate' from dual union all
select 24640,'024640','1996-060A','Molniya-3','Molniya-3 No. 62',65948,'Reentered','2007-10-18 00:00:00','1996-11-24 00:00:00',717.76,629,39725,62.86,'Molniya' from dual union all
select 24641,'024641','1996-060B','11S510','11S510',65948,'Reentered','1997-01-11 00:00:00','1996-11-24 00:00:00',91.34,211,470,62.82,'Intermediate' from dual union all
select 24642,'024642','1996-060C','BOZ','BOZ',65948,'Reentered','1996-11-26 00:00:00','1996-11-01 00:00:00',92.01,182,565,62.81,'Intermediate' from dual union all
select 24643,'024643','1996-060D','Blok-2BL','Blok-2BL No. 71612-697',65948,'Reentered','2008-02-18 00:00:00','1996-12-15 00:00:00',732.75,641,40450,62.88,'Molniya' from dual union all
select 24644,'024644','1996-059C','FSW-2 RV part','',65947,'Reentered','1996-11-04 00:00:00','1996-11-04 00:00:00',90.65,147,466,62.88,'Intermediate' from dual union all
select 24645,'024645','1996-061A','HETE','HETE',65949,'Reentered Attached','2002-04-07 00:00:00','1996-12-04 00:00:00',95.05,488,554,37.98,'Intermediate' from dual union all
select 24646,'024646','1996-049G','deb Pegasus','',65937,'Reentered','2002-09-24 00:00:00','1996-11-25 00:00:00',133.02,359,4149,82.99,'Medium' from dual union all
select 24647,'024647','1966-077P','deb FTV 1352','',13484,'In Earth orbit','','1996-12-08 00:00:00',167.2,3021,4330,89.57,'Medium' from dual union all
select 24648,'024648','1996-062A','Mars Global Surveyor','MGS',65950,'Deep Space','1996-11-08 00:00:00','1996-11-07 00:00:00',1436.43,184,97283,28.46,'Earth Escape' from dual union all
select 24649,'024649','1996-062B','Delta 239','Delta SSPS AJ-10-118K',65950,'Reentered','1998-06-21 00:00:00','1996-12-07 00:00:00',136.12,184,4590,25.31,'Medium' from dual union all
select 2465,'002465','1966-088AE','deb OGCh','',13495,'Reentered','1966-09-25 00:00:00','1966-09-18 00:00:00',93.9,131,800,49.8,'Intermediate' from dual union all
select 24650,'024650','1996-062C','Star 48B','Star 48B',65950,'Deep Space','1996-11-08 00:00:00','1996-11-07 00:00:00',1436.43,184,97283,28.46,'Earth Escape' from dual union all
select 24651,'024651','1996-062D','deb Delta','',65950,'Reentered','1997-10-25 00:00:00','1996-12-11 00:00:00',135.82,235,4514,25.24,'Medium' from dual union all
select 24652,'024652','1996-063A','Arabsat IIB','Arabsat 2B',65951,'In Earth orbit','','1996-12-23 00:00:00',1432.82,35721,35724,.04,'Drift GEO' from dual union all
select 24653,'024653','1996-063B','Measat 2','Measat 2',65951,'In Earth orbit','','1996-12-14 00:00:00',1436.11,35777,35797,.02,'Stationary' from dual union all
select 24654,'024654','1996-063C','Ariane H10-3 V92','Ariane H10-3 S/N T463',65951,'In Earth orbit','','1996-12-13 00:00:00',646.03,360,36394,3.86,'GEO Transfer' from dual union all
select 24655,'024655','1996-063D','Spelda 40 S/N UV32','Spelda 40',65951,'Reentered','2003-08-21 00:00:00','1996-12-19 00:00:00',631.62,264,35751,3.9,'GEO Transfer' from dual union all
select 24656,'024656','1996-064A','Blok D-2 No. 3L','11S824F No. 3L',65952,'Reentered','1996-11-18 00:00:00','1996-11-16 00:00:00',87.42,139,155,51.53,'Intermediate' from dual union all
select 24657,'024657','1986-017LY','EO-22 EVA-1 debris','',59210,'Reentered','1997-02-17 00:00:00','1996-12-05 00:00:00',92.09,370,384,51.65,'Intermediate' from dual union all
select 24658,'024658','1986-017LW','EO-22 EVA-1 debris','',59210,'Reentered','1997-03-08 00:00:00','1996-12-05 00:00:00',92.12,369,388,51.65,'Intermediate' from dual union all
select 24659,'024659','1986-017LX','EO-22 EVA-1 debris','',59210,'Reentered','1997-02-20 00:00:00','1996-12-04 00:00:00',92.15,372,388,51.65,'Intermediate' from dual union all
select 2466,'002466','1966-088AF','deb OGCh','',13495,'Reentered','1966-09-28 00:00:00','1966-09-18 00:00:00',94.81,190,829,49.5,'Intermediate' from dual union all
select 24660,'024660','1996-065A','Columbia (STS-80)','OV-102',65953,'Landed','1996-12-07 00:00:00','1996-11-26 00:00:00',91.57,346,358,28.47,'Intermediate' from dual union all
select 24661,'024661','1996-065B','ORFEUS-SPAS','ASTRO-SPAS',65953,'Landed Attached','1996-12-07 00:00:00','1996-11-26 00:00:00',91.57,346,358,28.47,'Intermediate' from dual union all
select 24662,'024662','1996-065C','Wake Shield Facility','WSF',65953,'Landed Attached','1996-12-07 00:00:00','1996-11-25 00:00:00',91.58,347,358,28.47,'Intermediate' from dual union all
select 24663,'024663','1996-066A','Progress M-33','Progress 7K-TGM No. 233',65954,'Deorbited','1997-03-12 00:00:00','1996-12-21 00:00:00',92.14,371,388,51.65,'Intermediate' from dual union all
select 24664,'024664','1996-066B','11S510','11S510',65954,'Reentered','1996-11-22 00:00:00','1996-11-21 00:00:00',88.31,175,206,51.66,'Intermediate' from dual union all
select 24665,'024665','1996-067A','Hot Bird 2','Eutelsat HB2',65955,'In Earth orbit','','1996-12-22 00:00:00',1435.96,35776,35792,.09,'Stationary' from dual union all
select 24666,'024666','1996-067B','Centaur AC-124','Centaur IIA(1N) AC-124',65955,'Reentered','1997-04-07 00:00:00','1996-12-21 00:00:00',613.2,154,34908,23.87,'GEO Transfer' from dual union all
select 24667,'024667','1996-068A','Sagan Memorial Station','MPF Lander',65956,'Deep Space Attached','1996-12-04 00:00:00','1996-12-04 00:00:00',1347.4,184,93761,28.78,'Earth Escape' from dual union all
select 24668,'024668','1996-068B','Delta 240','Delta SSPS AJ-10-118K',65956,'Reentered','1997-11-28 00:00:00','1997-01-03 00:00:00',117.37,178,2948,36.4,'Intermediate' from dual;
insert into SATELLITE
select 24669,'024669','1996-068C','Star 48B','Star 48B',65956,'Deep Space','1996-12-04 00:00:00','1996-12-04 00:00:00',1347.4,184,93761,28.78,'Earth Escape' from dual union all
select 2467,'002467','1966-088AG','deb OGCh','',13495,'Reentered','1966-09-26 00:00:00','1966-09-18 00:00:00',94.58,174,823,49.6,'Intermediate' from dual union all
select 24670,'024670','1996-069A','Kosmos-2335','US-PU',65957,'Reentered','1999-01-01 00:00:00','1997-01-10 00:00:00',92.74,401,416,65.04,'Intermediate' from dual union all
select 24671,'024671','1996-069B','11S692','11S692',65957,'Reentered','1996-12-12 00:00:00','1996-12-11 00:00:00',89.25,113,362,65.01,'Intermediate' from dual union all
select 24672,'024672','1986-017LZ','EO-22 EVA-2 debris','',59210,'Reentered','1997-03-06 00:00:00','1996-12-12 00:00:00',92.2,369,395,51.66,'Intermediate' from dual union all
select 24673,'024673','1986-017MA','EO-22 EVA-2 debris','',59210,'Reentered','1997-02-01 00:00:00','1996-12-12 00:00:00',92.09,368,385,51.65,'Intermediate' from dual union all
select 24674,'024674','1996-070A','INMARSAT III F-3','INMARSAT III F3',65958,'In Earth orbit','','1997-01-18 00:00:00',1425.32,35457,35694,2.61,'Drift GEO' from dual union all
select 24675,'024675','1996-070B','Centaur AC-129','Centaur IIA(4) AC-129',65958,'In Earth orbit','','1997-01-17 00:00:00',647.58,1016,35817,22.59,'GEO Transfer' from dual union all
select 24676,'024676','1966-077Q','deb FTV 1352','',13484,'In Earth orbit','','1996-12-18 00:00:00',167.21,2961,4391,89.54,'Medium' from dual union all
select 24677,'024677','1996-071A','Kosmos-2336','Parus',65959,'In Earth orbit','','1997-01-19 00:00:00',105.02,978,1013,82.94,'Intermediate' from dual union all
select 24678,'024678','1996-071B','S3M','S3M',65959,'In Earth orbit','','1997-01-19 00:00:00',104.89,971,1008,82.93,'Intermediate' from dual union all
select 24679,'024679','1994-085Y','deb Briz','',65481,'In Earth orbit','','1996-12-22 00:00:00',127.19,1848,2152,64.98,'Medium' from dual union all
select 2468,'002468','1966-088AH','deb OGCh','',13495,'Reentered','1966-09-22 00:00:00','1966-09-18 00:00:00',98.68,166,1226,50.3,'Intermediate' from dual union all
select 24680,'024680','1996-072A','USA 129','Enhanced CRYSTAL 2106',65960,'Deorbited','2014-05-01 00:00:00','1997-01-28 00:00:00',97.77,264,1040,97.87,'Sun-Synch' from dual union all
select 24681,'024681','1996-072B','Titan 45K-13 Stage 2','LR91AJ11A',65960,'Reentered','1997-01-30 00:00:00','1996-12-30 00:00:00',95.17,156,898,97.89,'Sun-Synch' from dual union all
select 24682,'024682','1994-029AAQ','deb HAPS','',65425,'In Earth orbit','','1997-03-18 00:00:00',102.93,616,1179,82.14,'Intermediate' from dual union all
select 24683,'024683','1994-029AAR','deb HAPS','',65425,'Reentered','1997-03-18 00:00:00','1997-03-17 00:00:00',91.44,331,360,81.95,'Intermediate' from dual union all
select 24684,'024684','1994-029AAS','deb HAPS','',65425,'Reentered','1997-10-19 00:00:00','1997-03-18 00:00:00',98.79,570,832,82,'Intermediate' from dual union all
select 24685,'024685','1994-029AAT','deb HAPS','',65425,'Reentered','1997-03-13 00:00:00','1997-02-03 00:00:00',96.62,384,810,81.98,'Intermediate' from dual union all
select 24686,'024686','1994-029AAU','deb HAPS','',65425,'Reentered','2000-04-16 00:00:00','1997-03-19 00:00:00',99.31,462,990,82.42,'Intermediate' from dual union all
select 24687,'024687','1994-029AAV','deb HAPS','',65425,'Reentered','1997-09-21 00:00:00','1997-03-18 00:00:00',97.09,563,676,81.75,'Intermediate' from dual union all
select 24688,'024688','1994-029AAW','deb HAPS','',65425,'Reentered','1997-03-01 00:00:00','1997-03-01 00:00:00',90.66,296,318,81.92,'Intermediate' from dual union all
select 24689,'024689','1994-029AAX','deb HAPS','',65425,'Reentered','1997-05-06 00:00:00','1997-03-18 00:00:00',95.53,478,611,82.01,'Intermediate' from dual union all
select 2469,'002469','1966-088AJ','deb OGCh','',13495,'Reentered','1966-09-24 00:00:00','1966-09-18 00:00:00',95.22,116,943,49.4,'Intermediate' from dual union all
select 24690,'024690','1994-029AAY','deb HAPS','',65425,'Reentered','1997-10-17 00:00:00','1997-03-18 00:00:00',99.16,561,876,82.09,'Intermediate' from dual union all
select 24691,'024691','1994-029AAZ','deb HAPS','',65425,'Reentered','1998-10-20 00:00:00','1997-03-19 00:00:00',100.12,574,954,82.37,'Intermediate' from dual union all
select 24692,'024692','1994-029ABA','deb HAPS','',65425,'Reentered','1997-12-29 00:00:00','1997-03-19 00:00:00',100.09,566,960,82.24,'Intermediate' from dual union all
select 24693,'024693','1994-029ABB','deb HAPS','',65425,'Reentered','1999-04-05 00:00:00','1997-03-18 00:00:00',99.44,580,884,81.77,'Intermediate' from dual union all
select 24694,'024694','1994-029ABC','deb HAPS','',65425,'Reentered','2000-05-16 00:00:00','1997-03-18 00:00:00',97.91,592,726,82.08,'Intermediate' from dual union all
select 24695,'024695','1994-029ABD','deb HAPS','',65425,'Reentered','1999-01-28 00:00:00','1997-03-19 00:00:00',99.16,575,862,82.15,'Intermediate' from dual union all
select 24696,'024696','1994-029ABE','deb HAPS','',65425,'Reentered','2002-08-21 00:00:00','1997-03-18 00:00:00',98.55,567,812,82.06,'Intermediate' from dual union all
select 24697,'024697','1994-029ABF','deb HAPS','',65425,'Reentered','1997-11-27 00:00:00','1997-03-19 00:00:00',99.02,561,863,82.13,'Intermediate' from dual union all
select 24698,'024698','1994-029ABG','deb HAPS','',65425,'Reentered','1998-10-30 00:00:00','1997-03-18 00:00:00',100.07,554,970,82.03,'Intermediate' from dual union all
select 24699,'024699','1994-029ABH','deb HAPS','',65425,'Reentered','1998-01-23 00:00:00','1997-03-16 00:00:00',99.58,568,909,81.83,'Intermediate' from dual union all
select 247,'000247','62 Epsilon 1','Discoverer 38','CORONA M-1',8188,'Reentered Attached','1962-03-21 00:00:00','1962-03-07 00:00:00',90.15,207,356,82.23,'Intermediate' from dual union all
select 2470,'002470','1966-088AK','deb OGCh','',13495,'Reentered','1966-09-24 00:00:00','1966-09-18 00:00:00',93.25,190,677,50.2,'Intermediate' from dual union all
select 24700,'024700','1995-062C','ISO Cryo Cover','',65723,'In Earth orbit','','1996-12-15 00:00:00',1435.09,1080,70454,1.37,'Near-synch' from dual union all
select 24701,'024701','1996-073A','Bion No. 11','Bion No. 11',65961,'Landed','1997-01-07 00:00:00','1996-12-26 00:00:00',90.46,217,378,62.8,'Intermediate' from dual union all
select 24702,'024702','1996-073B','11S510','11S510',65961,'Reentered','1997-01-28 00:00:00','1996-12-31 00:00:00',90.23,215,357,62.79,'Intermediate' from dual union all
select 24703,'024703','1994-085Z','deb Briz','',65481,'In Earth orbit','','1996-12-27 00:00:00',128.51,1880,2235,64.73,'Medium' from dual union all
select 24704,'024704','1966-005M','NNS O-7 insulation','',13412,'Reentered','2001-09-18 00:00:00','1997-01-03 00:00:00',105.48,839,1195,89.95,'Polar' from dual union all
select 24705,'024705','1965-109E','NNS O-6 insulation','',11859,'In Earth orbit','','1997-01-03 00:00:00',104.64,893,1063,89.08,'Polar' from dual union all
select 24706,'024706','1996-073C','KB electropitaniya','KBE',65961,'Reentered','1997-03-10 00:00:00','1997-01-23 00:00:00',90.06,212,343,62.8,'Intermediate' from dual union all
select 24707,'024707','1996-073D','PTDU cover','PTDU cover',65961,'Reentered','1997-03-02 00:00:00','1997-01-24 00:00:00',92.02,304,444,62.81,'Intermediate' from dual union all
select 24708,'024708','1996-073E','PTDU cover','PTDU cover',65961,'Reentered','1997-02-25 00:00:00','1997-01-23 00:00:00',91.46,293,400,62.81,'Intermediate' from dual union all
select 24709,'024709','1996-073F','KBE part','KBE part',65961,'Reentered','1997-01-23 00:00:00','1997-01-07 00:00:00',90.2,215,354,62.8,'Intermediate' from dual union all
select 2471,'002471','1966-088AL','deb OGCh','',13495,'Reentered','1966-09-25 00:00:00','1966-09-18 00:00:00',96.97,138,1090,49.5,'Intermediate' from dual union all
select 24710,'024710','1996-073G','KBE part','KBE part',65961,'Reentered','1997-01-10 00:00:00','1997-01-07 00:00:00',90.35,227,357,62.75,'Intermediate' from dual union all
select 24711,'024711','1997-001A','Atlantis (STS-81)','OV-104',66081,'Landed','1997-01-22 00:00:00','1997-01-14 00:00:00',90.01,253,297,51.65,'Intermediate' from dual union all
select 24712,'024712','1986-026G','deb Ariane V17','',59219,'In Earth orbit','','1997-01-17 00:00:00',587.19,391,33309,8.66,'Highly Ellip' from dual union all
select 24713,'024713','1997-002A','GE 2','GE 2',66082,'In Earth orbit','','1997-03-07 00:00:00',1435.81,35770,35792,.03,'Stationary' from dual union all
select 24714,'024714','1997-002B','Nahuel 1A','Nahuel 1A',66082,'In Earth orbit','','1997-03-01 00:00:00',1436.09,35776,35797,.06,'Stationary' from dual union all
select 24715,'024715','1997-002C','Ariane H10-3 V93','Ariane H10-3 S/N T465',66082,'In Earth orbit','','1997-03-03 00:00:00',636.31,258,35998,7.38,'GEO Transfer' from dual union all
select 24716,'024716','1997-002D','Spelda 30 S/N SP UV34','Spelda 30',66082,'Reentered','2004-06-30 00:00:00','1997-03-30 00:00:00',620.02,239,35177,6.83,'GEO Transfer' from dual union all
select 24717,'024717','1997-003A','Soyuz TM-25','Soyuz 7K-STM No. 74',66083,'Landed','1997-08-14 00:00:00','1997-03-13 00:00:00',92.26,378,393,51.65,'Intermediate' from dual union all
select 24718,'024718','1997-003B','11S510','11S510',66083,'Reentered','1997-02-13 00:00:00','1997-02-12 00:00:00',88.22,172,201,51.62,'Intermediate' from dual union all
select 24719,'024719','1997-004A','Discovery (STS-82)','OV-103',66084,'Landed','1997-02-21 00:00:00','1997-02-13 00:00:00',96.5,589,593,28.47,'Intermediate' from dual union all
select 2472,'002472','1966-088AM','deb OGCh','',13495,'Reentered','1966-09-25 00:00:00','1966-09-18 00:00:00',95.91,156,970,49.5,'Intermediate' from dual union all
select 24720,'024720','1997-005A','Haruka','MUSES-B',66085,'In Earth orbit','','1997-03-14 00:00:00',379.57,574,21401,31.34,'Highly Ellip' from dual union all
select 24721,'024721','1997-005B','KM-V1','KM-V1',66085,'In Earth orbit','','1997-03-14 00:00:00',373.72,236,21381,31.32,'Highly Ellip' from dual union all
select 24722,'024722','1991-003E','deb Ariane V41','',64172,'In Earth orbit','','1997-02-11 00:00:00',543.33,459,30898,7.68,'Highly Ellip' from dual union all
select 24723,'024723','1991-003F','deb Ariane V41','',64172,'In Earth orbit','','1997-02-13 00:00:00',545.27,605,30857,6.88,'Highly Ellip' from dual union all
select 24724,'024724','1991-003G','deb Ariane V41','',64172,'In Earth orbit','','1997-02-13 00:00:00',542.41,448,30859,6.9,'Highly Ellip' from dual union all
select 24725,'024725','1997-006A','Kosmos-2337','Strela-3',66086,'In Earth orbit','','1997-03-16 00:00:00',113.93,1402,1412,82.6,'Intermediate' from dual union all
select 24726,'024726','1997-006B','Kosmos-2338','Strela-3',66086,'In Earth orbit','','1997-03-16 00:00:00',114.02,1409,1413,82.6,'Intermediate' from dual union all
select 24727,'024727','1997-006C','Kosmos-2339','Strela-3',66086,'In Earth orbit','','1997-03-16 00:00:00',114.06,1411,1415,82.61,'Intermediate' from dual union all
select 24728,'024728','1997-006D','Gonets-D1 No. 322','Gonets-D1 No. 4 (01)',66086,'In Earth orbit','','1997-03-16 00:00:00',114.23,1412,1429,82.61,'Intermediate' from dual union all
select 24729,'024729','1997-006E','Gonets-D1 No. 323','Gonets-D1 No. 5 (16)',66086,'In Earth orbit','','1997-03-16 00:00:00',114.16,1412,1423,82.6,'Intermediate' from dual union all
select 2473,'002473','1966-088AN','deb OGCh','',13495,'Reentered','1966-09-27 00:00:00','1966-09-18 00:00:00',97.82,136,1172,49.8,'Intermediate' from dual union all
select 24730,'024730','1997-006F','Gonets-D1 No. 324','Gonets-D1 No. 6',66086,'In Earth orbit','','1997-03-16 00:00:00',114.07,1412,1414,82.6,'Intermediate' from dual union all
select 24731,'024731','1997-006G','S5M','S5M',66086,'In Earth orbit','','1997-03-16 00:00:00',114.71,1410,1475,82.6,'Intermediate' from dual union all
select 24732,'024732','1997-007A','JCSAT R','JCSAT 4',66087,'In Earth orbit','','1997-03-19 00:00:00',1436.06,35780,35792,.03,'Stationary' from dual union all
select 24733,'024733','1997-007B','Centaur AC-127','Centaur IIA(1N) AC-127',66087,'Reentered','1997-02-25 00:00:00','1997-02-19 00:00:00',2051.33,195,94007,23.43,'Very High' from dual union all
select 24734,'024734','1994-029AAN','deb HAPS','',65425,'Reentered','1997-02-06 00:00:00','1997-02-06 00:00:00',91.51,332,366,82.05,'Intermediate' from dual union all
select 24735,'024735','1994-029AAP','deb HAPS','',65425,'Reentered','1997-01-31 00:00:00','1997-01-31 00:00:00',91.1,316,341,81.74,'Intermediate' from dual union all
select 24736,'024736','1996-010F','SOZ','SOZ',65898,'Reentered','2002-02-02 00:00:00','1997-02-16 00:00:00',603.53,274,34284,47.66,'GEO Transfer' from dual union all
select 24737,'024737','1997-008A','DSP F18','DSP 20',66088,'In Earth orbit','','1997-02-25 00:00:00',1436.27,35780,35800,.1,'Stationary' from dual union all
select 24738,'024738','1997-008B','Titan 4B-24 Stage 2','LR91AJ11A',66088,'Reentered','1997-06-26 00:00:00','1997-02-24 00:00:00',94.03,185,759,28.62,'Intermediate' from dual union all
select 24739,'024739','1997-008C','IUS-4 SRM-1','Orbus 21',66088,'In Earth orbit','','1997-02-24 00:00:00',631.04,185,35800,28,'GEO Transfer' from dual union all
select 2474,'002474','1966-088AP','deb OGCh','',13495,'Reentered','1966-09-23 00:00:00','1966-09-18 00:00:00',97.82,136,1172,49.7,'Intermediate' from dual union all
select 24740,'024740','1997-008D','IUS-4 SRM-2','Orbus 6E',66088,'In Earth orbit','','1997-02-25 00:00:00',1431.67,35600,35800,2,'Drift GEO' from dual union all
select 24741,'024741','1991-003H','deb Ariane V41','',64172,'In Earth orbit','','1997-02-27 00:00:00',546.92,648,30903,6.75,'Highly Ellip' from dual union all
select 24742,'024742','1997-009A','INTELSAT 801','INTELSAT 801',66089,'In Earth orbit','','1997-04-04 00:00:00',1435.98,35770,35799,.02,'Stationary' from dual union all
select 24743,'024743','1997-009B','Ariane H10-3 V94','Ariane H10-3 S/N T464',66089,'Reentered','2000-12-12 00:00:00','1997-03-31 00:00:00',632.13,181,35860,6.86,'GEO Transfer' from dual union all
select 24744,'024744','1997-010A','Zeya','Mozhaets 2',66090,'Reentered','1999-10-25 00:00:00','1997-04-03 00:00:00',94.03,466,477,97.28,'Sun-Synch' from dual union all
select 24745,'024745','1997-010B','Start','DS',66090,'Reentered','2001-01-12 00:00:00','1997-04-03 00:00:00',94.02,467,476,97.28,'Sun-Synch' from dual union all
select 24746,'024746','1997-010C','Start DS cover','Start',66090,'Reentered','1998-06-05 00:00:00','1997-04-03 00:00:00',93.45,420,467,97.27,'Sun-Synch' from dual union all
select 24747,'024747','1997-010D','deb Start','',66090,'Reentered','1998-05-23 00:00:00','1997-04-03 00:00:00',93.58,434,466,97.28,'Sun-Synch' from dual union all
select 24748,'024748','1997-011A','Tempo 2','Tempo 2',66091,'In Earth orbit','','1997-04-07 00:00:00',1436.04,35753,35818,.55,'Stationary' from dual union all
select 24749,'024749','1997-011B','Centaur AC-128','Centaur IIA(1N) AC-128',66091,'Reentered','1997-11-19 00:00:00','1997-04-07 00:00:00',360.44,159,20639,25.75,'Highly Ellip' from dual union all
select 2475,'002475','1966-088AQ','deb OGCh','',13495,'Reentered','1966-09-23 00:00:00','1966-09-18 00:00:00',90.99,132,515,49.4,'Intermediate' from dual union all
select 24750,'024750','1991-003J','Ariane V41 large debris','RCS 14 sq m',64172,'Reentered','1997-12-23 00:00:00','1997-03-11 00:00:00',406.87,222,23399,6.8,'Highly Ellip' from dual union all
select 24751,'024751','1991-003K','deb Ariane V41','',64172,'In Earth orbit','','1997-03-31 00:00:00',540.82,358,30863,7.09,'Highly Ellip' from dual union all
select 24752,'024752','1991-006C','deb Informator-1','',64175,'In Earth orbit','','1997-04-02 00:00:00',104.61,957,996,82.93,'Intermediate' from dual union all
select 24753,'024753','1997-012A','DMSP F-14','DMSP Block 5D-2 S-14',66092,'In Earth orbit','','1997-05-04 00:00:00',101.89,842,855,98.94,'Sun-Synch' from dual union all
select 24754,'024754','1979-104C','deb Ariane L01','',47588,'Reentered','2018-03-05 00:00:00','1997-03-25 00:00:00',357.56,418,20201,17.7,'Highly Ellip' from dual union all
select 24755,'024755','1997-013A','Columbia (STS-83)','OV-102',66093,'Landed','1997-04-08 00:00:00','1997-04-06 00:00:00',90.53,298,303,28.47,'Intermediate' from dual union all
select 24756,'024756','1997-012B','DMSP F-14 solar array band','',66092,'In Earth orbit','','1997-05-04 00:00:00',101.89,845,852,98.94,'Sun-Synch' from dual union all
select 24757,'024757','1997-014A','Progress M-34','Progress 7K-TGM No. 234',66094,'Deorbited','1997-07-02 00:00:00','1997-05-06 00:00:00',92.34,383,396,51.65,'Intermediate' from dual union all
select 24758,'024758','1997-014B','11S510','11S510',66094,'Reentered','1997-04-09 00:00:00','1997-04-08 00:00:00',88.03,168,186,51.64,'Intermediate' from dual;
insert into SATELLITE
select 24759,'024759','1994-029ABJ','deb HAPS','',65425,'Reentered','2012-03-03 00:00:00','1997-04-08 00:00:00',112.94,580,2144,82.02,'Intermediate' from dual union all
select 2476,'002476','1966-088AR','deb OGCh','',13495,'Reentered','1966-09-27 00:00:00','1966-09-18 00:00:00',96.38,160,1011,49.8,'Intermediate' from dual union all
select 24760,'024760','1994-029ABK','deb HAPS','',65425,'Reentered','2002-09-23 00:00:00','1997-04-07 00:00:00',101.55,598,1067,82.64,'Intermediate' from dual union all
select 24761,'024761','1997-015A','Kosmos-2340','Oko',66095,'In Earth orbit','','1997-05-10 00:00:00',717.9,559,39801,62.94,'Molniya' from dual union all
select 24762,'024762','1997-015B','11S510','11S510',66095,'Reentered','1997-06-13 00:00:00','1997-05-09 00:00:00',90.65,223,390,62.84,'Intermediate' from dual union all
select 24763,'024763','1997-015C','BOZ','BOZ',66095,'Reentered','1997-05-18 00:00:00','1997-04-16 00:00:00',91.51,198,500,62.86,'Intermediate' from dual union all
select 24764,'024764','1997-015D','Blok-2BL','Blok-2BL 77342-698',66095,'In Earth orbit','','1997-05-09 00:00:00',708.72,581,39326,63.05,'Molniya' from dual union all
select 24765,'024765','1997-012C','DMSP F-14 solar array band','',66092,'Reentered','2016-12-22 00:00:00','1997-05-04 00:00:00',101.91,846,853,98.94,'Sun-Synch' from dual union all
select 24766,'024766','1994-029ABL','deb HAPS','',65425,'Reentered','2001-12-31 00:00:00','1997-04-14 00:00:00',101.34,611,1034,82.08,'Intermediate' from dual union all
select 24767,'024767','1994-029ABM','deb HAPS','',65425,'Reentered','2001-07-14 00:00:00','1997-04-14 00:00:00',102.03,608,1102,81.15,'Intermediate' from dual union all
select 24768,'024768','1997-016A','Thaicom 3','Thaicom 3',66096,'In Earth orbit','','1997-05-18 00:00:00',1439.41,35587,36116,.06,'Drift GEO' from dual union all
select 24769,'024769','1997-016B','BSAT 1a','BSAT 1a',66096,'In Earth orbit','','1997-05-16 00:00:00',1436.14,35745,35829,.11,'Stationary' from dual union all
select 2477,'002477','1966-088AS','deb OGCh','',13495,'Reentered','1966-09-30 00:00:00','1966-09-18 00:00:00',95.69,159,944,49.5,'Intermediate' from dual union all
select 24770,'024770','1997-016C','Ariane H10-3 V95','Ariane H10-3 S/N T467',66096,'Reentered','2012-02-22 00:00:00','1997-05-18 00:00:00',640.48,257,36213,7.07,'GEO Transfer' from dual union all
select 24771,'024771','1997-016D','Spelda 30 S/N UV35','Spelda 30',66096,'Reentered','2000-06-19 00:00:00','1997-05-17 00:00:00',624.58,227,35425,7.05,'GEO Transfer' from dual union all
select 24772,'024772','1997-017A','Kosmos-2341','Parus',66097,'In Earth orbit','','1997-05-17 00:00:00',105.02,977,1013,82.92,'Intermediate' from dual union all
select 24773,'024773','1997-017B','S3M','S3M',66097,'In Earth orbit','','1997-05-17 00:00:00',104.88,975,1003,82.92,'Intermediate' from dual union all
select 24774,'024774','1994-029ABN','deb HAPS','',65425,'Reentered','1997-03-17 00:00:00','1997-03-15 00:00:00',91.36,327,355,81.88,'Intermediate' from dual union all
select 24775,'024775','1994-029ABP','deb HAPS','',65425,'Reentered','1997-04-02 00:00:00','1997-01-29 00:00:00',94.95,503,530,81.8,'Intermediate' from dual union all
select 24776,'024776','1994-029ABQ','deb HAPS','',65425,'Reentered','1997-04-08 00:00:00','1996-12-22 00:00:00',95.54,489,601,81.66,'Intermediate' from dual union all
select 24777,'024777','1997-012D','DMSP F-14 OLS Cover','',66092,'In Earth orbit','','1997-05-04 00:00:00',101.89,842,855,98.94,'Sun-Synch' from dual union all
select 24778,'024778','1997-012E','DMSP F-14 OLS Cooler Cover','',66092,'In Earth orbit','','1997-05-04 00:00:00',101.9,842,855,98.95,'Sun-Synch' from dual union all
select 24779,'024779','1997-018A','Minisat-01','Minisat-01',66098,'Reentered','2002-02-26 00:00:00','1997-05-21 00:00:00',96.09,562,581,150.97,'Retrograde' from dual union all
select 2478,'002478','1966-088AT','deb OGCh','',13495,'Reentered','1966-09-27 00:00:00','1966-09-18 00:00:00',97.52,181,1099,50.2,'Intermediate' from dual union all
select 24780,'024780','1997-018B','Orion 38','Orion 38',66098,'Reentered','2002-05-20 00:00:00','1997-05-21 00:00:00',96.02,554,582,150.97,'Retrograde' from dual union all
select 24781,'024781','1995-008G','Kosmos-2306 SS 1','ESO',65669,'Reentered','1998-11-27 00:00:00','1997-04-22 00:00:00',94.45,469,514,65.86,'Intermediate' from dual union all
select 24782,'024782','1995-008H','Kosmos-2306 SS 2','ESO',65669,'Reentered','1998-11-13 00:00:00','1997-04-22 00:00:00',94.38,463,514,65.84,'Intermediate' from dual union all
select 24783,'024783','1995-008J','Kosmos-2306 SS 3','ESO',65669,'Reentered','1998-10-20 00:00:00','1997-04-22 00:00:00',94.28,453,515,65.86,'Intermediate' from dual union all
select 24784,'024784','1995-008K','Kosmos-2306 SS 4','ESO',65669,'Reentered','1998-10-03 00:00:00','1997-04-22 00:00:00',94.22,448,513,65.83,'Intermediate' from dual union all
select 24785,'024785','1995-008L','Kosmos-2306 SS 5','ESO',65669,'Reentered','1998-11-02 00:00:00','1997-04-22 00:00:00',94.32,458,513,65.87,'Intermediate' from dual union all
select 24786,'024786','1997-019A','GOES 10','GOES K',66099,'In Earth orbit','','1997-05-25 00:00:00',1436.14,35774,35801,.42,'Stationary' from dual union all
select 24787,'024787','1997-019B','Centaur AC-79','Centaur I AC-79',66099,'Reentered','1998-04-21 00:00:00','1997-05-25 00:00:00',744.69,173,41501,27.26,'GEO Transfer' from dual union all
select 24788,'024788','1994-029ABR','deb HAPS','',65425,'Reentered','2002-04-13 00:00:00','1997-04-25 00:00:00',102.22,611,1116,81.82,'Intermediate' from dual union all
select 24789,'024789','1995-008M','Kosmos-2306 SS 6','ESO',65669,'Reentered','1998-11-03 00:00:00','1997-04-26 00:00:00',94.32,458,513,65.83,'Intermediate' from dual union all
select 2479,'002479','1966-088AU','deb OGCh','',13495,'Reentered','1966-10-03 00:00:00','1966-09-18 00:00:00',95.4,138,938,49.2,'Intermediate' from dual union all
select 24790,'024790','1994-029ABS','deb HAPS','',65425,'Reentered','1997-04-08 00:00:00','1997-05-01 00:00:00',98.66,560,829,82,'Intermediate' from dual union all
select 24791,'024791','1994-029ABT','deb HAPS','',65425,'Reentered','1997-04-12 00:00:00','1997-03-11 00:00:00',92.48,362,430,81.77,'Intermediate' from dual union all
select 24792,'024792','1997-020A','Iridium 8','Iridium SV008',66100,'Reentered','2017-11-24 00:00:00','1997-06-04 00:00:00',100.38,774,779,86.4,'Polar' from dual union all
select 24793,'024793','1997-020B','Iridium 7','Iridium SV007',66100,'In Earth orbit','','1997-06-04 00:00:00',100.4,773,781,86.4,'Polar' from dual union all
select 24794,'024794','1997-020C','Iridium 6','Iridium SV006',66100,'Reentered','2017-12-23 00:00:00','1997-06-06 00:00:00',100.4,773,781,86.41,'Polar' from dual union all
select 24795,'024795','1997-020D','Iridium 5','Iridium SV005',66100,'In Earth orbit','','1997-06-04 00:00:00',100.38,769,785,86.4,'Polar' from dual union all
select 24796,'024796','1997-020E','Iridium 4','Iridium SV004',66100,'In Earth orbit','','1997-06-04 00:00:00',100.4,771,784,86.4,'Polar' from dual union all
select 24797,'024797','1997-020F','Delta 242','Delta SSPS AJ-10-118K',66100,'In Earth orbit','','1997-06-04 00:00:00',100.28,593,950,86.9,'Polar' from dual union all
select 24798,'024798','1997-021A','Zhongxing 6','DFH-3',66101,'In Earth orbit','','1997-06-10 00:00:00',1436.19,35785,35792,.27,'Stationary' from dual union all
select 24799,'024799','1997-021B','CZ-3A Stage 3','CZ-3A Stage 3',66101,'Reentered','2006-12-24 00:00:00','1997-06-12 00:00:00',631.76,271,35751,29.17,'GEO Transfer' from dual union all
select 248,'000248','62 Epsilon 2','Discoverer 38 camera door','',8188,'Reentered','1962-03-03 00:00:00','1962-02-28 00:00:00',88.93,207,237,82.23,'Intermediate' from dual union all
select 2480,'002480','1966-088AV','deb OGCh','',13495,'Reentered','1966-09-25 00:00:00','1966-09-18 00:00:00',95.36,125,947,49.9,'Intermediate' from dual union all
select 24800,'024800','1997-022A','Kosmos-2342','Oko',66102,'In Earth orbit','','1997-06-13 00:00:00',717.8,565,39791,62.85,'Molniya' from dual union all
select 24801,'024801','1997-022B','11S510','11S510',66102,'Reentered','1997-08-08 00:00:00','1997-06-13 00:00:00',91.19,224,442,62.77,'Intermediate' from dual union all
select 24802,'024802','1997-022C','BOZ','BOZ',66102,'Reentered','1997-06-25 00:00:00','1997-05-21 00:00:00',91.51,197,501,62.81,'Intermediate' from dual union all
select 24803,'024803','1997-022D','Blok-2BL','Blok-2BL No. 71621-699',66102,'In Earth orbit','','1997-06-13 00:00:00',705.07,564,39162,62.84,'Molniya' from dual union all
select 24804,'024804','1997-023A','Atlantis (STS-84)','OV-104',66103,'Landed','1997-05-24 00:00:00','1997-05-17 00:00:00',92.33,382,396,51.65,'Intermediate' from dual union all
select 24805,'024805','1997-024A','Kosmos-2343','Don',66104,'Deorbited','1997-09-18 00:00:00','1997-06-14 00:00:00',89.92,214,327,64.86,'Intermediate' from dual union all
select 24806,'024806','1997-024B','11S510','11S510',66104,'Reentered','1997-05-20 00:00:00','1997-05-17 00:00:00',88.98,172,276,64.87,'Intermediate' from dual union all
select 24807,'024807','1967-092K','NNS O-14 insulation','',15347,'Reentered','2004-06-26 00:00:00','1997-05-18 00:00:00',106.47,1027,1100,89.25,'Polar' from dual union all
select 24808,'024808','1997-025A','Thor II','Thor 2A',66105,'In Earth orbit','','1997-06-19 00:00:00',1436.14,35757,35818,.01,'Stationary' from dual union all
select 24809,'024809','1997-025B','Delta 243','Delta SSPS AJ-10-118K',66105,'In Earth orbit','','1997-06-19 00:00:00',108.45,646,1665,26.29,'Intermediate' from dual union all
select 2481,'002481','1966-089A','FTV 1353','Midas RTS-I F3',13496,'In Earth orbit','','1966-11-04 00:00:00',167.57,3675,3705,90.19,'Medium' from dual union all
select 24810,'024810','1997-025C','Star 48B','Star 48B',66105,'In Earth orbit','','1997-06-20 00:00:00',666.3,1321,36463,19.77,'GEO Transfer' from dual union all
select 24811,'024811','1965-048N','deb NNS O-4','',11798,'Reentered','2003-03-10 00:00:00','1997-05-22 00:00:00',106.24,980,1125,90.13,'Polar' from dual union all
select 24812,'024812','1997-026A','Telstar 5','Telstar 5',66106,'In Earth orbit','','1997-07-03 00:00:00',1436.06,35775,35797,.04,'Stationary' from dual union all
select 24813,'024813','1997-026B','8S812','8S812',66106,'Reentered','1997-06-01 00:00:00','1997-05-26 00:00:00',88.66,203,214,51.59,'Intermediate' from dual union all
select 24814,'024814','1997-026C','Perekhodnik','Sredniy perekhodnik',66106,'Reentered','1997-05-27 00:00:00','1997-05-26 00:00:00',88.57,202,206,51.58,'Intermediate' from dual union all
select 24815,'024815','1997-026D','Blok DM4 No. 1L','Blok DM4 No. 1L',66106,'In Earth orbit','','1997-07-07 00:00:00',758.39,6592,35747,17.54,'Highly Ellip' from dual union all
select 24816,'024816','1995-071D','deb Kosmos-2326','',65732,'Reentered','1998-12-11 00:00:00','1997-05-29 00:00:00',92.79,406,417,65.01,'Intermediate' from dual union all
select 24817,'024817','1994-029ABU','deb HAPS','',65425,'In Earth orbit','','1997-05-30 00:00:00',104.83,604,1369,81.71,'Intermediate' from dual union all
select 24818,'024818','1977-087C','deb S3M','',42215,'In Earth orbit','','1997-06-02 00:00:00',104.69,956,1004,82.97,'Intermediate' from dual union all
select 24819,'024819','1997-027A','INMARSAT III F-4','INMARSAT III F4',66107,'In Earth orbit','','1997-07-03 00:00:00',1448.39,36026,36028,.26,'Drift GEO' from dual union all
select 2482,'002482','1966-088AW','deb OGCh','',13495,'Reentered','1966-10-18 00:00:00','1966-09-18 00:00:00',95.1,174,873,50.1,'Intermediate' from dual union all
select 24820,'024820','1997-027B','Insat 2D','Insat 2D',66107,'In Earth orbit','','1997-07-03 00:00:00',1436.04,35695,35876,.14,'Stationary' from dual union all
select 24821,'024821','1997-027C','Ariane H10-3 V97','Ariane H10-3 S/N T468',66107,'Reentered','1997-06-04 00:00:00','1997-07-03 00:00:00',627.47,225,35576,7.1,'GEO Transfer' from dual union all
select 24822,'024822','1997-027D','Spelda 30 S/N UV36','Spelda 30',66107,'Reentered','2011-10-21 00:00:00','1997-07-03 00:00:00',627.39,226,35571,7.11,'GEO Transfer' from dual union all
select 24823,'024823','1994-029ABV','deb HAPS','',65425,'Reentered','1997-09-07 00:00:00','1997-06-04 00:00:00',96.33,518,648,81.78,'Intermediate' from dual union all
select 24824,'024824','1994-029ABW','deb HAPS','',65425,'Reentered','1997-05-24 00:00:00','1997-05-15 00:00:00',92.95,382,456,81.93,'Intermediate' from dual union all
select 24825,'024825','1994-029ABX','deb HAPS','',65425,'Reentered','1997-05-12 00:00:00','1997-03-30 00:00:00',94.57,450,545,82.06,'Intermediate' from dual union all
select 24826,'024826','1994-029ABY','deb HAPS','',65425,'Reentered','1997-05-23 00:00:00','1997-05-14 00:00:00',93.15,376,482,82.01,'Intermediate' from dual union all
select 24827,'024827','1997-028A','Kosmos-2344','Arkon 11F664 No. 1',66108,'In Earth orbit','','1997-07-06 00:00:00',129.94,1509,2731,63.42,'Medium' from dual union all
select 24828,'024828','1997-028B','8S812','8S812',66108,'Reentered','1997-06-07 00:00:00','1997-06-06 00:00:00',87.57,141,168,64.83,'Intermediate' from dual union all
select 24829,'024829','1997-028C','Blok DM-5 No. 1L','17S40 No. 1L',66108,'In Earth orbit','','1997-07-06 00:00:00',130.05,1506,2744,63.42,'Medium' from dual union all
select 2483,'002483','1966-088AX','deb OGCh','',13495,'Reentered','1966-10-08 00:00:00','1966-09-18 00:00:00',93.85,166,760,49.1,'Intermediate' from dual union all
select 24830,'024830','1997-028D','SOZ','SOZ',66108,'Reentered','1998-09-29 00:00:00','1997-07-07 00:00:00',111.96,182,2452,64.56,'Intermediate' from dual union all
select 24831,'024831','1994-061E','deb Kosmos-2292','',65457,'Reentered','2000-03-06 00:00:00','1997-06-06 00:00:00',106.89,396,1770,83.44,'Intermediate' from dual union all
select 24832,'024832','1997-028E','SOZ','SOZ',66108,'Reentered','1999-11-02 00:00:00','1997-07-06 00:00:00',112.39,202,2471,64.55,'Intermediate' from dual union all
select 24833,'024833','1997-028F','deb Kosmos-2344','',66108,'In Earth orbit','','1997-07-28 00:00:00',130.14,1508,2750,63.41,'Medium' from dual union all
select 24834,'024834','1997-029A','Feng Yun 2A','Feng Yun Er Jia',66109,'In Earth orbit','','1997-07-10 00:00:00',1436.32,35782,35799,1.17,'Stationary' from dual union all
select 24835,'024835','1997-029B','CZ-3 Stage 3','CZ-3 Stage 3',66109,'Reentered','1997-06-10 00:00:00','1997-06-10 00:00:00',634.93,226,35959,28.56,'GEO Transfer' from dual union all
select 24836,'024836','1997-030A','Iridium 14','Iridium SV014',66110,'In Earth orbit','','1997-07-18 00:00:00',100.38,775,779,86.4,'Polar' from dual union all
select 24837,'024837','1997-030B','Iridium 12','Iridium SV012',66110,'In Earth orbit','','1997-07-18 00:00:00',100.4,773,781,86.39,'Polar' from dual union all
select 24838,'024838','1997-030C','Iridium 9','Iridium SV009',66110,'Reentered','2003-03-11 00:00:00','1997-07-18 00:00:00',100.38,774,779,86.39,'Polar' from dual union all
select 24839,'024839','1997-030D','Iridium 10','Iridium SV010',66110,'In Earth orbit','','1997-07-18 00:00:00',100.37,775,778,86.41,'Polar' from dual union all
select 2484,'002484','1966-088AY','OGCh long lived debris','',13495,'Reentered','1967-02-09 00:00:00','1966-11-08 00:00:00',99.6,259,1220,50.06,'Intermediate' from dual union all
select 24840,'024840','1997-030E','Iridium 13','Iridium SV013',66110,'Reentered','2018-04-29 00:00:00','1997-07-18 00:00:00',100.38,773,780,86.39,'Polar' from dual union all
select 24841,'024841','1997-030F','Iridium 16','Iridium SV016',66110,'In Earth orbit','','1997-07-18 00:00:00',100.38,775,779,86.4,'Polar' from dual union all
select 24842,'024842','1997-030G','Iridium 11','Iridium SV011',66110,'In Earth orbit','','1997-07-18 00:00:00',100.38,775,779,86.4,'Polar' from dual union all
select 24843,'024843','1997-028G','Kosmos-2344 sensor cover','',66108,'In Earth orbit','','1997-07-28 00:00:00',129.79,1513,2714,63.41,'Medium' from dual union all
select 24844,'024844','1994-029ABZ','deb HAPS','',65425,'Reentered','1997-10-27 00:00:00','1997-06-24 00:00:00',98.77,510,890,82.13,'Intermediate' from dual union all
select 24845,'024845','1986-017MB','Spektr debris (spurious)','',59210,'Reentered','1997-06-25 00:00:00','1997-06-25 00:00:00',91.46,340,353,51.64,'Intermediate' from dual union all
select 24846,'024846','1997-031A','INTELSAT 802','INTELSAT 802',66111,'In Earth orbit','','1997-07-25 00:00:00',1436.09,35774,35799,.03,'Stationary' from dual union all
select 24847,'024847','1997-031B','Ariane H10-3 V96','Ariane H10-3 S/N T466',66111,'Reentered','2003-03-03 00:00:00','1997-07-25 00:00:00',631.16,193,35798,7.02,'GEO Transfer' from dual union all
select 24848,'024848','1986-017MC','Spektr debris (spurious)','',59210,'Reentered','1997-06-22 00:00:00','1997-06-22 00:00:00',88.17,179,189,51.64,'Intermediate' from dual;
insert into SATELLITE
select 24849,'024849','1997-032A','Columbia (STS-94)','OV-102',66112,'Landed','1997-07-17 00:00:00','1997-07-08 00:00:00',90.53,298,303,28.47,'Intermediate' from dual union all
select 2485,'002485','1966-088AZ','OGCh Retro','OGCh Retro',13495,'Reentered','1966-11-12 00:00:00','1966-11-05 00:00:00',91.82,240,488,49.18,'Intermediate' from dual union all
select 24850,'024850','1994-029ACA','deb HAPS','',65425,'Reentered','1999-01-18 00:00:00','1997-07-02 00:00:00',103.35,568,1266,82.26,'Intermediate' from dual union all
select 24851,'024851','1997-033A','Progress M-35','Progress 7K-TGM No. 235',66113,'Deorbited','1997-10-08 00:00:00','1997-08-04 00:00:00',92.31,385,391,51.65,'Intermediate' from dual union all
select 24852,'024852','1997-033B','11S510','11S510',66113,'Reentered','1997-07-08 00:00:00','1997-07-07 00:00:00',88.03,167,187,51.61,'Intermediate' from dual union all
select 24853,'024853','1995-028C','deb Kosmos-2313','',65689,'Reentered','1997-07-30 00:00:00','1997-07-07 00:00:00',92.42,274,513,64.87,'Intermediate' from dual union all
select 24854,'024854','1995-028D','deb Kosmos-2313','',65689,'Reentered','1997-07-18 00:00:00','1997-07-07 00:00:00',91.83,264,465,64.9,'Intermediate' from dual union all
select 24855,'024855','1995-028E','deb Kosmos-2313','',65689,'Reentered','1997-09-25 00:00:00','1997-07-07 00:00:00',93.19,275,586,64.88,'Intermediate' from dual union all
select 24856,'024856','1995-028F','deb Kosmos-2313','',65689,'Reentered','1997-07-25 00:00:00','1997-07-07 00:00:00',93.76,269,648,65.08,'Intermediate' from dual union all
select 24857,'024857','1995-028G','deb Kosmos-2313','',65689,'Reentered','1997-09-23 00:00:00','1997-07-07 00:00:00',92.97,277,563,64.89,'Intermediate' from dual union all
select 24858,'024858','1995-028H','deb Kosmos-2313','',65689,'Reentered','1997-07-24 00:00:00','1997-07-07 00:00:00',91.66,271,441,64.92,'Intermediate' from dual union all
select 24859,'024859','1995-028J','deb Kosmos-2313','',65689,'Reentered','1997-07-16 00:00:00','1997-07-08 00:00:00',91.48,264,430,64.91,'Intermediate' from dual union all
select 2486,'002486','1966-088BA','deb OGCh','',13495,'Reentered','1966-10-21 00:00:00','1966-09-18 00:00:00',96.3,224,939,49.3,'Intermediate' from dual union all
select 24860,'024860','1995-028K','deb Kosmos-2313','',65689,'Reentered','1997-07-08 00:00:00','1997-07-07 00:00:00',88.78,207,222,64.98,'Intermediate' from dual union all
select 24861,'024861','1995-028L','deb Kosmos-2313','',65689,'Reentered','1997-07-29 00:00:00','1997-07-08 00:00:00',91.06,268,385,64.94,'Intermediate' from dual union all
select 24862,'024862','1995-028M','deb Kosmos-2313','',65689,'Reentered','1997-07-07 00:00:00','1997-07-06 00:00:00',87.91,154,189,65.01,'Intermediate' from dual union all
select 24863,'024863','1995-028N','deb Kosmos-2313','',65689,'Reentered','1997-07-12 00:00:00','1997-07-07 00:00:00',90.72,254,366,64.99,'Intermediate' from dual union all
select 24864,'024864','1995-028P','deb Kosmos-2313','',65689,'Reentered','1997-07-11 00:00:00','1997-07-07 00:00:00',91.13,260,400,64.89,'Intermediate' from dual union all
select 24865,'024865','1992-086D','deb USA 89 PKM','',64798,'In Earth orbit','','1997-07-08 00:00:00',169.69,350,7200,57,'Medium' from dual union all
select 24866,'024866','1996-051H','deb Zenit','',65939,'In Earth orbit','','1997-07-15 00:00:00',102.26,819,913,71.12,'Intermediate' from dual union all
select 24867,'024867','1992-086E','deb USA 89 PKM','',64798,'In Earth orbit','','1997-07-08 00:00:00',169.69,350,7200,57,'Medium' from dual union all
select 24868,'024868','1994-029ACG','deb HAPS','',65425,'Reentered','1997-09-23 00:00:00','1997-07-08 00:00:00',92.58,363,439,83.02,'Intermediate' from dual union all
select 24869,'024869','1997-034A','Iridium 15','Iridium SV015',66114,'In Earth orbit','','1997-08-08 00:00:00',100.38,774,779,86.4,'Polar' from dual union all
select 2487,'002487','1966-088BB','deb OGCh','',13495,'Reentered','1966-10-18 00:00:00','1966-09-18 00:00:00',96.05,213,925,49,'Intermediate' from dual union all
select 24870,'024870','1997-034B','Iridium 17','Iridium SV017',66114,'In Earth orbit','','1997-08-08 00:00:00',100.38,773,780,86.4,'Polar' from dual union all
select 24871,'024871','1997-034C','Iridium 20','Iridium SV020',66114,'In Earth orbit','','1997-08-08 00:00:00',100.37,774,778,86.4,'Polar' from dual union all
select 24872,'024872','1997-034D','Iridium 18','Iridium SV018',66114,'In Earth orbit','','1997-08-08 00:00:00',100.38,772,782,86.4,'Polar' from dual union all
select 24873,'024873','1997-034E','Iridium 21','Iridium SV021',66114,'In Earth orbit','','1997-08-08 00:00:00',97.42,626,645,86.4,'Polar' from dual union all
select 24874,'024874','1997-034F','Delta 244','Delta SSPS AJ-10-118K',66114,'Reentered','1998-03-30 00:00:00','1997-08-08 00:00:00',93.41,266,617,82.73,'Intermediate' from dual union all
select 24875,'024875','1996-051J','deb Zenit','',65939,'Reentered','2009-03-13 00:00:00','1997-07-16 00:00:00',102.14,815,905,71.16,'Intermediate' from dual union all
select 24876,'024876','1997-035A','GPS IIR-2','Navstar GPS SVN 43',66115,'In Earth orbit','','1997-08-22 00:00:00',718.08,20123,20247,54.93,'Medium' from dual union all
select 24877,'024877','1997-035B','Delta 245','Delta SSPS AJ-10-118K',66115,'Reentered','1998-01-02 00:00:00','1997-08-22 00:00:00',99.43,195,1268,37.68,'Intermediate' from dual union all
select 24878,'024878','1997-035C','Star 48B','Star 48B',66115,'Reentered','2000-07-23 00:00:00','1997-08-22 00:00:00',354.44,187,20238,38.98,'Highly Ellip' from dual union all
select 24879,'024879','1996-051K','deb Zenit','',65939,'Reentered','2000-04-15 00:00:00','1997-07-23 00:00:00',101.48,770,888,71.1,'Intermediate' from dual union all
select 2488,'002488','1966-088BC','deb OGCh','',13495,'Reentered','1966-10-10 00:00:00','1966-09-18 00:00:00',90.67,172,443,49.1,'Intermediate' from dual union all
select 24880,'024880','1997-036A','Superbird 3','Superbird C',66116,'In Earth orbit','','1997-08-28 00:00:00',1436.04,35767,35804,.01,'Stationary' from dual union all
select 24881,'024881','1997-036B','Centaur AC-133','Centaur IIA(1N) AC-133',66116,'Reentered','1998-07-12 00:00:00','1997-07-31 00:00:00',1911.12,223,89048,25.63,'Very High' from dual union all
select 24882,'024882','1997-034G','deb Iridium 21','',66114,'Reentered','1997-09-10 00:00:00','1997-08-08 00:00:00',96.85,552,664,86.3,'Polar' from dual union all
select 24883,'024883','1997-037A','OrbView-2','Seastar',66117,'In Earth orbit','','1997-08-31 00:00:00',98.61,685,700,98.21,'Sun-Synch' from dual union all
select 24884,'024884','1997-037B','Orion 38','Orion 38',66117,'Reentered','1997-09-29 00:00:00','1997-08-08 00:00:00',90.54,293,309,98.28,'Sun-Synch' from dual union all
select 24885,'024885','1997-034H','deb Iridium 21','',66114,'Reentered','1998-02-10 00:00:00','1997-08-08 00:00:00',97.39,619,648,86.38,'Polar' from dual union all
select 24886,'024886','1997-038A','Soyuz TM-26','Soyuz 7K-STM No. 75',66118,'Landed','1998-02-19 00:00:00','1997-09-04 00:00:00',92.3,384,391,51.65,'Intermediate' from dual union all
select 24887,'024887','1997-038B','11S510','11S510',66118,'Reentered','1997-08-08 00:00:00','1997-08-07 00:00:00',88.24,175,199,51.62,'Intermediate' from dual union all
select 24888,'024888','1997-038C','Soyuz TM-26 part','Soyuz part',66118,'Reentered','1997-08-08 00:00:00','1997-08-06 00:00:00',88.99,180,270,51.61,'Intermediate' from dual union all
select 24889,'024889','1997-039A','Discovery (STS-85)','OV-103',66119,'Landed','1997-08-19 00:00:00','1997-08-09 00:00:00',90.4,289,300,56.99,'Intermediate' from dual union all
select 2489,'002489','1966-090A','OPS 2055','GAMBIT SV 983',13497,'Deorbited','1966-10-20 00:00:00','1966-10-13 00:00:00',88.91,155,287,90.96,'Polar' from dual union all
select 24890,'024890','1997-039B','CRISTA-SPAS','ASTRO-SPAS',66119,'Landed Attached','1997-08-19 00:00:00','1997-08-09 00:00:00',90.4,290,299,57,'Intermediate' from dual union all
select 24891,'024891','1997-040A','PAS 6','PAS 6',66120,'In Earth orbit','','1997-09-11 00:00:00',1436.06,35785,35787,.05,'Stationary' from dual union all
select 24892,'024892','1997-040B','Ariane H10-3 V98','Ariane H10-3 S/N T470',66120,'In Earth orbit','','1997-09-08 00:00:00',656.51,273,37015,6.82,'GEO Transfer' from dual union all
select 24893,'024893','1988-032C','deb Blok E','',61281,'In Earth orbit','','1997-08-08 00:00:00',97.13,586,657,97.63,'Sun-Synch' from dual union all
select 24894,'024894','1997-041A','Kosmos-2345','Oko-S F6',66121,'In Earth orbit','','1997-09-13 00:00:00',1436.04,34296,37275,1.28,'Inclined GEO' from dual union all
select 24895,'024895','1997-041B','8S812','8S812',66121,'Reentered','1997-08-16 00:00:00','1997-08-15 00:00:00',87.95,161,185,53.17,'Intermediate' from dual union all
select 24896,'024896','1997-041C','Perekhodnik','Sredniy perekhodnik',66121,'Reentered','1997-08-15 00:00:00','1997-08-15 00:00:00',87.66,145,173,53.17,'Intermediate' from dual union all
select 24897,'024897','1997-041D','Blok DM-2 No. 88L','11S861 No. 88L',66121,'In Earth orbit','','1997-09-15 00:00:00',1439.79,34301,37417,1.31,'Drift GEO' from dual union all
select 24898,'024898','1997-034J','deb Iridium 21','',66114,'Reentered','1997-11-04 00:00:00','1997-08-11 00:00:00',97.16,589,657,86.35,'Polar' from dual union all
select 24899,'024899','1997-041E','SOZ','SOZ',66121,'Reentered','1998-05-21 00:00:00','1997-09-30 00:00:00',622.71,295,35260,48.75,'GEO Transfer' from dual union all
select 249,'000249','62 Epsilon 3','Discoverer 38 camera door','',8188,'Reentered','1962-03-03 00:00:00','1962-02-28 00:00:00',89.08,207,251,82.23,'Intermediate' from dual union all
select 2490,'002490','1966-090B','Agena D 4830','Agena D',13497,'Reentered','1966-10-21 00:00:00','1966-10-17 00:00:00',88.88,181,258,90.88,'Polar' from dual union all
select 24900,'024900','1997-041F','SOZ','SOZ',66121,'Reentered','1998-03-21 00:00:00','1997-09-13 00:00:00',621.51,260,35232,48.76,'GEO Transfer' from dual union all
select 24901,'024901','1997-042A','Agila 2','Mabuhay',66122,'In Earth orbit','','1997-09-18 00:00:00',1436.11,35780,35794,.24,'Stationary' from dual union all
select 24902,'024902','1997-042B','CZ-3B Stage 3','CZ-3B Stage 3',66122,'Reentered','1997-12-17 00:00:00','1997-09-18 00:00:00',803.45,149,44352,23.31,'Highly Ellip' from dual union all
select 24903,'024903','1997-043A','Iridium 26','Iridium SV026',66123,'In Earth orbit','','1997-09-20 00:00:00',100.38,774,779,86.41,'Polar' from dual union all
select 24904,'024904','1997-043B','Iridium 25','Iridium SV025',66123,'Reentered','2018-05-14 00:00:00','1997-09-20 00:00:00',100.4,775,780,86.4,'Polar' from dual union all
select 24905,'024905','1997-043C','Iridium 24','Iridium SV024',66123,'In Earth orbit','','1997-09-20 00:00:00',100.38,774,779,86.4,'Polar' from dual union all
select 24906,'024906','1997-043D','Iridium 23','Iridium SV023',66123,'Reentered','2018-03-28 00:00:00','1997-09-20 00:00:00',100.38,775,779,86.39,'Polar' from dual union all
select 24907,'024907','1997-043E','Iridium 22','Iridium SV022',66123,'In Earth orbit','','1997-09-20 00:00:00',100.38,776,778,86.4,'Polar' from dual union all
select 24908,'024908','1997-043F','Delta 246','Delta SSPS AJ-10-118K',66123,'Reentered','1997-10-15 00:00:00','1997-08-28 00:00:00',91.92,209,529,82.63,'Intermediate' from dual union all
select 24909,'024909','1997-044A','Lewis','SSTI/Lewis',66124,'Reentered','1997-09-28 00:00:00','1997-08-30 00:00:00',90.63,294,316,97.56,'Sun-Synch' from dual union all
select 2491,'002491','1966-091A','Kosmos-129','Zenit-2 No. 33',13498,'Landed','1966-10-21 00:00:00','1966-10-14 00:00:00',89.42,180,312,64.65,'Intermediate' from dual union all
select 24910,'024910','1997-044B','OAM','OAM',66124,'Reentered','1997-10-13 00:00:00','1997-08-30 00:00:00',90.42,289,302,97.57,'Sun-Synch' from dual union all
select 24911,'024911','1981-070M','DE-1 boom cover','',52593,'In Earth orbit','','1997-08-24 00:00:00',410.03,559,23250,88.64,'Highly Ellip' from dual union all
select 24912,'024912','1997-045A','ACE','Advanced Composition Exp',66125,'Deep Space','1997-08-26 00:00:00','1997-08-25 00:00:00',86411.19,177,1281969,28.7,'Cislunar' from dual union all
select 24913,'024913','1997-045B','Delta 247','Delta SSPS AJ-10-118K',66125,'Deep Space','1997-08-26 00:00:00','1997-08-25 00:00:00',46267.29,172,840904,28.8,'Cislunar' from dual union all
select 24914,'024914','1981-059F','deb NOAA 7','',52582,'In Earth orbit','','1997-08-27 00:00:00',101.97,830,874,98.73,'Sun-Synch' from dual union all
select 24915,'024915','1981-059G','deb NOAA 7','',52582,'Reentered','1998-10-01 00:00:00','1997-08-27 00:00:00',101.72,831,850,98.72,'Sun-Synch' from dual union all
select 24916,'024916','1997-046A','PAS 5','PAS 5',66126,'In Earth orbit','','1997-10-01 00:00:00',1436.04,35780,35791,.01,'Stationary' from dual union all
select 24917,'024917','1997-046B','8S812','8S812',66126,'Reentered','1997-08-30 00:00:00','1997-08-28 00:00:00',87.94,144,201,51.61,'Intermediate' from dual union all
select 24918,'024918','1997-046C','Perekhodnik','Sredniy perekhodnik',66126,'Reentered','1997-08-28 00:00:00','1997-08-28 00:00:00',88.2,164,207,51.59,'Intermediate' from dual union all
select 24919,'024919','1997-046D','Blok DM3 No. 3L','Blok DM3 No. 3L',66126,'In Earth orbit','','1997-09-27 00:00:00',800.78,8454,35920,14.62,'Medium' from dual union all
select 2492,'002492','1966-091B','8A92E','8A92E',13498,'Reentered','1966-10-23 00:00:00','1966-10-15 00:00:00',89.5,217,283,64.58,'Intermediate' from dual union all
select 24920,'024920','1997-047A','FORTE','FORTE',66127,'In Earth orbit','','1997-09-29 00:00:00',101.22,800,833,69.95,'Intermediate' from dual union all
select 24921,'024921','1997-047B','Orion 38','Orion 38',66127,'In Earth orbit','','1997-09-28 00:00:00',101.17,800,828,69.97,'Intermediate' from dual union all
select 24922,'024922','1997-043G','deb Iridium  23','',66123,'Reentered','1997-08-31 00:00:00','1997-08-29 00:00:00',94.01,445,497,86.63,'Polar' from dual union all
select 24923,'024923','1997-043H','deb Iridium  23','',66123,'Reentered','1997-09-04 00:00:00','1997-08-29 00:00:00',95,502,536,86.67,'Polar' from dual union all
select 24924,'024924','1997-043J','deb Iridium  23','',66123,'Reentered','1997-09-20 00:00:00','1997-08-29 00:00:00',95.47,529,554,86.69,'Polar' from dual union all
select 24925,'024925','1997-048A','Iridium MFS 1','Iridium MFS 1',66128,'In Earth orbit','','1997-10-01 00:00:00',97.24,624,629,86.33,'Polar' from dual union all
select 24926,'024926','1997-048B','Iridium MFS 2','Iridium MFS 2',66128,'In Earth orbit','','1997-10-01 00:00:00',97.27,624,632,86.33,'Polar' from dual union all
select 24927,'024927','1997-048C','Smart Dispenser','CZ-2C SD',66128,'Reentered','1998-01-06 00:00:00','1997-10-01 00:00:00',92.42,225,562,86.23,'Polar' from dual union all
select 24928,'024928','1997-048D','CZ-2C retro cover','',66128,'Reentered','1997-09-06 00:00:00','1997-09-03 00:00:00',93.81,177,745,86.03,'Polar' from dual union all
select 24929,'024929','1997-048E','CZ-2C retro cover','',66128,'Reentered','1997-09-07 00:00:00','1997-09-03 00:00:00',94.16,177,779,86.07,'Polar' from dual union all
select 2493,'002493','1965-082KJ','deb Transtage 4','',11832,'Reentered','1973-02-16 00:00:00','1973-02-11 00:00:00',93.02,418,427,31.86,'Intermediate' from dual union all
select 24930,'024930','1997-048F','CZ-2C Stage 2','CZ-2C Stage 2',66128,'Reentered','1997-10-13 00:00:00','1997-09-08 00:00:00',92.47,177,615,86.34,'Polar' from dual union all
select 24931,'024931','1997-049A','Hot Bird 3','Eutelsat HB3',66129,'In Earth orbit','','1997-10-04 00:00:00',1435.98,35692,35877,.49,'Stationary' from dual union all
select 24932,'024932','1997-049B','Meteosat 7','MTP 1',66129,'In Earth orbit','','1997-10-02 00:00:00',1436.06,35779,35793,1.74,'Stationary' from dual union all
select 24933,'024933','1997-049C','Ariane H10-3 V99','Ariane H10-3 S/N T471',66129,'Reentered','1997-09-04 00:00:00','1997-09-04 00:00:00',631.18,212,35780,6.96,'GEO Transfer' from dual union all
select 24934,'024934','1997-049D','Spelda 10 S/N UV28','Spelda 10',66129,'Reentered','2004-04-21 00:00:00','1997-10-02 00:00:00',625.94,223,35499,7.03,'GEO Transfer' from dual union all
select 24935,'024935','1981-059H','deb NOAA 7','',52582,'Reentered','1998-09-22 00:00:00','1997-09-02 00:00:00',101.68,828,849,98.72,'Sun-Synch' from dual union all
select 24936,'024936','1997-050A','GE 3','GE 3',66130,'In Earth orbit','','1997-10-04 00:00:00',1436.09,35783,35790,.02,'Stationary' from dual union all
select 24937,'024937','1997-050B','Centaur AC-146','Centaur IIA (1N) AC-146',66130,'In Earth orbit','','1997-10-04 00:00:00',791.67,304,43635,19.03,'Highly Ellip' from dual union all
select 24938,'024938','1992-047K','deb SOZ','',64759,'Reentered','2015-10-30 00:00:00','1997-09-04 00:00:00',339.03,661,18795,64.99,'Highly Ellip' from dual;
insert into SATELLITE
select 24939,'024939','1994-029ACB','deb HAPS','',65425,'Reentered','1997-12-23 00:00:00','1997-09-08 00:00:00',98.33,493,865,82.03,'Intermediate' from dual union all
select 2494,'002494','1966-090C','SGLS cover','',13497,'Reentered','1966-10-16 00:00:00','1966-10-13 00:00:00',88.92,160,283,90.9,'Polar' from dual union all
select 24940,'024940','1994-029ACC','deb HAPS','',65425,'Reentered','1998-03-29 00:00:00','1997-09-08 00:00:00',98.85,509,899,81.68,'Intermediate' from dual union all
select 24941,'024941','1994-029ACD','deb HAPS','',65425,'Reentered','2011-11-30 00:00:00','1997-09-08 00:00:00',102.24,612,1118,81.81,'Intermediate' from dual union all
select 24942,'024942','1994-029ACE','deb HAPS','',65425,'Reentered','1998-12-05 00:00:00','1997-09-08 00:00:00',106.43,568,1555,82.46,'Intermediate' from dual union all
select 24943,'024943','1986-017MD','EO-24 EVA-2 debris (Spektr)','',59210,'Reentered','1997-11-25 00:00:00','1997-09-11 00:00:00',91.07,324,331,51.64,'Intermediate' from dual union all
select 24944,'024944','1997-051A','Iridium 29','Iridium SV029',66131,'In Earth orbit','','1997-10-14 00:00:00',100.38,776,778,86.4,'Polar' from dual union all
select 24945,'024945','1997-051B','Iridium 32','Iridium SV032',66131,'In Earth orbit','','1997-10-15 00:00:00',100.38,776,778,86.4,'Polar' from dual union all
select 24946,'024946','1997-051C','Iridium 33','Iridium SV033',66131,'In Earth orbit','','1997-10-14 00:00:00',100.38,774,779,86.39,'Polar' from dual union all
select 24947,'024947','1997-051D','Iridium 27','Iridium SV027',66131,'Reentered','2002-02-01 00:00:00','1997-10-14 00:00:00',95.78,554,559,86.64,'Polar' from dual union all
select 24948,'024948','1997-051E','Iridium 28','Iridium SV028',66131,'In Earth orbit','','1997-10-14 00:00:00',100.38,776,778,86.4,'Polar' from dual union all
select 24949,'024949','1997-051F','Iridium 30','Iridium SV030',66131,'Reentered','2017-09-28 00:00:00','1997-10-14 00:00:00',100.38,774,779,86.39,'Polar' from dual union all
select 2495,'002495','1966-090D','deb Agena D','',13497,'Reentered','1966-10-16 00:00:00','1966-10-13 00:00:00',89.23,182,291,90.9,'Polar' from dual union all
select 24950,'024950','1997-051G','Iridium 31','Iridium SV031',66131,'In Earth orbit','','1997-10-14 00:00:00',100.38,776,778,86.4,'Polar' from dual union all
select 24951,'024951','1997-051H','8S812','8S812',66131,'Reentered','1997-09-14 00:00:00','1997-09-14 00:00:00',87.33,110,175,72.42,'Intermediate' from dual union all
select 24952,'024952','1986-017ME','EO-24 EVA-2 debris (Spektr)','',59210,'Reentered','1997-12-27 00:00:00','1997-09-16 00:00:00',91.26,333,340,51.65,'Intermediate' from dual union all
select 24953,'024953','1997-052A','Kosmos-2346','Parus',66132,'In Earth orbit','','1997-10-23 00:00:00',104.41,939,995,82.92,'Intermediate' from dual union all
select 24954,'024954','1997-052B','FAISAT-2V','FAISAT-2V',66132,'In Earth orbit','','1997-10-23 00:00:00',104.4,938,995,82.92,'Intermediate' from dual union all
select 24955,'024955','1997-052C','S3M','S3M',66132,'In Earth orbit','','1997-10-23 00:00:00',104.29,938,984,82.91,'Intermediate' from dual union all
select 24956,'024956','1966-040C','Nimbus 2 debris','',13447,'In Earth orbit','','1997-09-23 00:00:00',107.96,1091,1173,100.48,'Sun-Synch' from dual union all
select 24957,'024957','1997-053A','INTELSAT 803','INTELSAT 803',66133,'In Earth orbit','','1997-10-23 00:00:00',1436.09,35773,35799,.09,'Stationary' from dual union all
select 24958,'024958','1997-058C','Spoutnik-40','RS-17',66138,'Reentered','1998-05-21 00:00:00','1997-11-04 00:00:00',92.13,376,382,51.66,'Intermediate' from dual union all
select 24959,'024959','1966-077R','deb FTV 1352','',13484,'In Earth orbit','','1997-09-24 00:00:00',167.27,3005,4352,89.62,'Medium' from dual union all
select 2496,'002496','1963-014AD','deb West Ford','RCS 0.16',9245,'Reentered','2001-07-20 00:00:00','1992-01-14 00:00:00',165.26,2317,4878,86.39,'Medium' from dual union all
select 24960,'024960','1997-054A','Molniya-1T','Molniya-1T No. 98',66134,'In Earth orbit','','1997-10-24 00:00:00',717.76,486,39868,62.85,'Molniya' from dual union all
select 24961,'024961','1997-054B','11S510','11S510',66134,'Reentered','1997-10-14 00:00:00','1997-10-01 00:00:00',90.14,204,359,62.79,'Intermediate' from dual union all
select 24962,'024962','1997-054C','BOZ','BOZ',66134,'Reentered','1997-10-09 00:00:00','1997-10-01 00:00:00',90.24,179,394,62.81,'Intermediate' from dual union all
select 24963,'024963','1997-054D','Blok-ML','Blok-ML No. 76716-492',66134,'Reentered','2016-09-30 00:00:00','1997-10-24 00:00:00',731.43,482,40544,62.85,'Molniya' from dual union all
select 24964,'024964','1997-055A','Atlantis (STS-86)','OV-104',66135,'Landed','1997-10-06 00:00:00','1997-09-28 00:00:00',92.28,384,389,51.65,'Intermediate' from dual union all
select 24965,'024965','1997-056A','Iridium 19','Iridium SV019',66136,'Reentered','2018-04-19 00:00:00','1997-10-27 00:00:00',100.37,775,778,86.39,'Polar' from dual union all
select 24966,'024966','1997-056B','Iridium 35','Iridium SV037',66136,'In Earth orbit','','1997-10-27 00:00:00',100.38,775,779,86.4,'Polar' from dual union all
select 24967,'024967','1997-056C','Iridium 36','Iridium SV036',66136,'In Earth orbit','','1997-10-27 00:00:00',95.95,562,567,86.71,'Polar' from dual union all
select 24968,'024968','1997-056D','Iridium 37','Iridium SV035',66136,'Reentered','2018-05-26 00:00:00','1997-10-27 00:00:00',100.38,776,778,86.39,'Polar' from dual union all
select 24969,'024969','1997-056E','Iridium 34','Iridium SV034',66136,'Reentered','2018-01-08 00:00:00','1997-10-27 00:00:00',100.38,775,779,86.4,'Polar' from dual union all
select 2497,'002497','1963-014AE','deb West Ford large cluster','RCS 0.01',9245,'Reentered','1981-03-30 00:00:00','1981-02-23 00:00:00',159.83,772,5984,85,'Medium' from dual union all
select 24970,'024970','1997-056F','Delta 248','Delta SSPS AJ-10-118K',66136,'Reentered','1998-04-17 00:00:00','1997-10-27 00:00:00',92.6,276,528,83.61,'Intermediate' from dual union all
select 24971,'024971','1997-057A','IRS-1D','IRS-1D',66137,'In Earth orbit','','1997-10-30 00:00:00',100.48,738,825,98.62,'Sun-Synch' from dual union all
select 24972,'024972','1997-057B','PSLV-D3 PS4','PS4',66137,'Reentered','1999-11-04 00:00:00','1997-10-29 00:00:00',95.87,306,816,98.64,'Sun-Synch' from dual union all
select 24973,'024973','1994-029ACF','deb HAPS','',65425,'In Earth orbit','','1997-09-29 00:00:00',103.06,591,1216,82.05,'Intermediate' from dual union all
select 24974,'024974','1992-053C','deb Kosmos-2208','Strela-2M debris 2 (high',64765,'In Earth orbit','','1997-10-02 00:00:00',101.31,802,840,74.06,'Intermediate' from dual union all
select 24975,'024975','1994-029ACH','deb HAPS','',65425,'Reentered','1997-12-12 00:00:00','1997-10-02 00:00:00',94.7,497,512,81.98,'Intermediate' from dual union all
select 24976,'024976','1994-029ACJ','deb HAPS','',65425,'Reentered','2001-08-16 00:00:00','1997-10-03 00:00:00',105.14,573,1428,82.07,'Intermediate' from dual union all
select 24977,'024977','1994-029ACK','deb HAPS','',65425,'Reentered','2002-10-11 00:00:00','1997-10-03 00:00:00',116.21,566,2455,81.57,'Intermediate' from dual union all
select 24978,'024978','1994-029ACL','deb HAPS','',65425,'In Earth orbit','','1997-10-03 00:00:00',105.31,515,1503,81.49,'Intermediate' from dual union all
select 24979,'024979','1994-029ACM','deb HAPS','',65425,'Reentered','1999-03-07 00:00:00','1997-10-02 00:00:00',101.81,587,1102,81.53,'Intermediate' from dual union all
select 2498,'002498','1966-052C','OV3-4 despin weight','',13459,'In Earth orbit','','1966-09-09 00:00:00',141.25,584,4628,40.63,'Medium' from dual union all
select 24980,'024980','1994-029ACN','deb HAPS','',65425,'In Earth orbit','','1997-10-02 00:00:00',104.21,604,1310,81.61,'Intermediate' from dual union all
select 24981,'024981','1994-029ACP','deb HAPS','',65425,'Reentered','2002-03-03 00:00:00','1997-10-03 00:00:00',105.06,609,1386,81.67,'Intermediate' from dual union all
select 24982,'024982','1994-029ACQ','deb HAPS','',65425,'Reentered','2000-06-24 00:00:00','1997-10-03 00:00:00',104.96,606,1380,81.66,'Intermediate' from dual union all
select 24983,'024983','1994-029ACR','deb HAPS','',65425,'Reentered','1999-09-17 00:00:00','1997-10-03 00:00:00',106.12,565,1529,82.26,'Intermediate' from dual union all
select 24984,'024984','1994-029ACS','deb HAPS','',65425,'Reentered','2000-09-10 00:00:00','1997-10-03 00:00:00',106.7,611,1537,81.63,'Intermediate' from dual union all
select 24985,'024985','1994-029ACT','deb HAPS','',65425,'In Earth orbit','','1997-10-03 00:00:00',108.55,572,1748,81.53,'Intermediate' from dual union all
select 24986,'024986','1994-029ACU','deb HAPS','',65425,'Reentered','1999-12-07 00:00:00','1997-10-03 00:00:00',106.54,592,1540,82.88,'Intermediate' from dual union all
select 24987,'024987','1994-029ACV','deb HAPS','',65425,'Reentered','1998-08-26 00:00:00','1997-10-03 00:00:00',105.37,568,1456,81.08,'Intermediate' from dual union all
select 24988,'024988','1994-029ACW','deb HAPS','',65425,'Reentered','2002-06-28 00:00:00','1997-10-02 00:00:00',105.4,576,1451,82.11,'Intermediate' from dual union all
select 24989,'024989','1994-029ACX','deb HAPS','',65425,'Reentered','1999-03-01 00:00:00','1997-10-02 00:00:00',103.71,582,1286,82,'Intermediate' from dual union all
select 2499,'002499','1963-014AF','deb West Ford','',9245,'Reentered','1977-04-12 00:00:00','1966-10-13 00:00:00',165.37,1988,5216,86.69,'Medium' from dual union all
select 24990,'024990','1987-079J','deb SOZ','',60169,'In Earth orbit','','1997-10-03 00:00:00',325.39,278,18308,64.64,'Highly Ellip' from dual union all
select 24991,'024991','1994-029ACY','deb HAPS','',65425,'Reentered','1998-05-16 00:00:00','1997-10-02 00:00:00',105.97,536,1543,82.39,'Intermediate' from dual union all
select 24992,'024992','1994-029ACZ','deb HAPS','',65425,'Reentered','1998-07-10 00:00:00','1997-10-02 00:00:00',103.43,512,1330,82.14,'Intermediate' from dual union all
select 24993,'024993','1994-029ADA','deb HAPS','',65425,'Reentered','1999-11-07 00:00:00','1997-10-03 00:00:00',106.74,579,1573,82.6,'Intermediate' from dual union all
select 24994,'024994','1994-029ADB','deb HAPS','',65425,'Reentered','1997-12-04 00:00:00','1997-10-03 00:00:00',97.86,492,821,80.47,'Intermediate' from dual union all
select 24995,'024995','1994-029ADC','deb HAPS','',65425,'Reentered','1999-03-31 00:00:00','1997-10-03 00:00:00',98.96,431,986,81.73,'Intermediate' from dual union all
select 24996,'024996','1994-029ADD','deb HAPS','',65425,'In Earth orbit','','1997-10-03 00:00:00',105.03,604,1388,82.18,'Intermediate' from dual union all
select 24997,'024997','1994-029ADE','deb HAPS','',65425,'Reentered','1998-04-11 00:00:00','1997-10-03 00:00:00',100.46,526,1035,81.1,'Intermediate' from dual union all
select 24998,'024998','1994-029ADF','deb HAPS','',65425,'Reentered','2000-06-01 00:00:00','1997-10-03 00:00:00',98.49,583,790,81.95,'Intermediate' from dual union all
select 24999,'024999','1994-029ADG','deb HAPS','',65425,'Reentered','1997-11-20 00:00:00','1997-10-03 00:00:00',97.19,524,725,82.08,'Intermediate' from dual union all
select 25,'000025','59 Lambda','Discoverer 8','CORONA C-5',5533,'Reentered Attached','1960-03-08 00:00:00','1959-12-22 00:00:00',100.64,184,1393,80.65,'Intermediate' from dual union all
select 250,'000250','61 Omicron 75','deb r/b Ablestar','',7273,'Reentered','1981-09-13 00:00:00','1962-02-28 00:00:00',101.6,666,1002,67.04,'Intermediate' from dual union all
select 2500,'002500','1963-014AG','deb West Ford','',9245,'Reentered','1974-11-19 00:00:00','1974-11-09 00:00:00',156.61,450,6042,83.61,'Medium' from dual union all
select 25000,'025000','1968-081G','Transtage 5 debris','Large debris 0.7m2',16808,'In Earth orbit','','1997-09-27 00:00:00',1425.24,35064,36084,13.34,'Inclined Drift' from dual union all
select 25001,'025001','1968-081H','Transtage 5 debris','',16808,'In Earth orbit','','1997-09-27 00:00:00',1415.73,35059,35715,13.32,'Inclined Drift' from dual union all
select 25002,'025002','1997-058A','Progress M-36','Progress 7K-TGM No. 237',66138,'Deorbited','1997-12-19 00:00:00','1997-11-04 00:00:00',92.26,381,390,51.65,'Intermediate' from dual union all
select 25003,'025003','1997-058B','11S510','11S510',66138,'Reentered','1997-10-07 00:00:00','1997-10-06 00:00:00',88.39,181,208,51.65,'Intermediate' from dual union all
select 25004,'025004','1997-059A','Echostar III','Echostar 3',66139,'In Earth orbit','','1997-11-05 00:00:00',1436.04,35775,35796,.01,'Stationary' from dual union all
select 25005,'025005','1997-059B','Centaur AC-135','Centaur IIA(1N) AC-135',66139,'Reentered','1998-03-15 00:00:00','1997-11-05 00:00:00',672.75,178,37932,25.9,'GEO Transfer' from dual union all
select 25006,'025006','1997-060A','Foton','Foton 34KS No. 11',66140,'Landed','1997-10-23 00:00:00','1997-10-11 00:00:00',90.42,218,373,62.81,'Intermediate' from dual union all
select 25007,'025007','1997-060B','11S510','11S510',66140,'Reentered','1997-10-31 00:00:00','1997-10-16 00:00:00',90.02,208,342,62.8,'Intermediate' from dual union all
select 25008,'025008','1997-061A','Cassini','Cassini',66141,'Deep Space','1997-10-15 00:00:00','1997-10-15 00:00:00',614.84,267,60927,28.63,'Earth Escape' from dual union all
select 25009,'025009','1997-061B','Centaur TC-21','Centaur TC-21',66141,'Deep Space','1997-10-15 00:00:00','1997-10-15 00:00:00',614.84,267,60927,28.63,'Earth Escape' from dual union all
select 2501,'002501','1966-092A','Molniya-1','Molniya-1 11F67 No. 7',13499,'Reentered','1968-09-11 00:00:00','1966-12-01 00:00:00',718.11,464,39906,64.68,'GEO Transfer' from dual union all
select 25010,'025010','1997-062A','Apstar 2R','Apstar 2R',66142,'In Earth orbit','','1997-11-16 00:00:00',1436.09,35776,35797,.1,'Stationary' from dual union all
select 25011,'025011','1997-062B','CZ-3B Stage 3','CZ-3B Stage 3',66142,'Reentered','1998-03-18 00:00:00','1997-11-16 00:00:00',870.94,142,47522,23.77,'Highly Ellip' from dual union all
select 25012,'025012','1987-079K','deb SOZ','',60169,'Reentered','1997-10-31 00:00:00','1997-10-17 00:00:00',141.29,237,4979,65.31,'Medium' from dual union all
select 25013,'025013','1997-063A','STEP M4','STEP M4',66143,'Reentered','2001-03-31 00:00:00','1997-11-21 00:00:00',93.92,432,501,44.97,'Intermediate' from dual union all
select 25014,'025014','1997-063B','Orion 38','Orion 38',66143,'Reentered','1999-11-30 00:00:00','1997-11-21 00:00:00',93.88,432,496,44.98,'Intermediate' from dual union all
select 25015,'025015','1997-060C','PTDU cover','PTDU cover',66140,'Reentered','1997-11-17 00:00:00','1997-10-24 00:00:00',92.12,317,440,62.81,'Intermediate' from dual union all
select 25016,'025016','1997-060D','PTDU cover','PTDU cover',66140,'Reentered','1997-11-04 00:00:00','1997-10-24 00:00:00',92.41,326,460,62.79,'Intermediate' from dual union all
select 25017,'025017','1997-064A','USA 133','ONYX 3',66144,'Deorbited','2011-10-08 00:00:00','1997-11-24 00:00:00',98.2,671,675,57.01,'Intermediate' from dual union all
select 25018,'025018','1997-064B','Titan 4A-18 Stage 2','LR91AJ11A 4A-18',66144,'Reentered','2002-03-26 00:00:00','1997-11-23 00:00:00',95.59,429,666,57.01,'Intermediate' from dual union all
select 25019,'025019','1997-065A','USA 134','DSCS III B-13',66145,'In Earth orbit','','1997-11-01 00:00:00',1436.27,35780,35800,0,'Stationary' from dual union all
select 2502,'002502','1966-093A','Kosmos-130','Zenit-4 No. 24',13500,'Landed','1966-10-28 00:00:00','1966-10-23 00:00:00',89.72,208,314,64.95,'Intermediate' from dual union all
select 25020,'025020','1997-065B','Centaur AC-131','Centaur IIA(4) AC-131',66145,'Reentered','1998-09-27 00:00:00','1997-11-24 00:00:00',614.41,191,34934,26.3,'GEO Transfer' from dual union all
select 25021,'025021','1997-065C','IABS-6','IABS',66145,'In Earth orbit','','1997-11-25 00:00:00',1424.81,35532,35599,.03,'Drift GEO' from dual union all
select 25022,'025022','1987-079L','deb SOZ','',60169,'Reentered','1997-10-30 00:00:00','1997-10-26 00:00:00',105.85,222,1847,64.76,'Intermediate' from dual union all
select 25023,'025023','1997-066A','TEAMSAT','TEAMSAT',66146,'In Earth orbit','','1997-12-01 00:00:00',467.72,530,26633,7.86,'Highly Ellip' from dual union all
select 25024,'025024','1997-066B','Maqsat-B','Maqsat-B',66146,'In Earth orbit','','1997-11-29 00:00:00',466.63,526,26575,7.84,'Highly Ellip' from dual union all
select 25025,'025025','1997-066C','YES','Young Engineers Satellit',66146,'In Earth orbit','','1997-12-02 00:00:00',467.82,547,26622,7.84,'Highly Ellip' from dual union all
select 25026,'025026','1997-066D','SPELTRA','SPELTRA',66146,'In Earth orbit','','1997-11-29 00:00:00',467.61,528,26629,7.85,'Highly Ellip' from dual union all
select 25027,'025027','1992-053D','deb Kosmos-2208','Strela-2M debris 1',64765,'In Earth orbit','','1997-11-01 00:00:00',100.92,800,805,74.03,'Intermediate' from dual;
insert into SATELLITE
select 25028,'025028','1993-036D','deb Kosmos-2251','Strela-2M debris 2 (high',65138,'In Earth orbit','','1997-11-03 00:00:00',101.03,798,817,74.05,'Intermediate' from dual union all
select 25029,'025029','1987-020DR','deb Kosmos-1823','',60110,'Reentered','1997-11-06 00:00:00','1997-11-05 00:00:00',93.79,423,497,72.87,'Intermediate' from dual union all
select 2503,'002503','1966-092B','BOZ','BOZ',13499,'Reentered','1966-11-10 00:00:00','1966-11-09 00:00:00',88.68,165,254,64.78,'Intermediate' from dual union all
select 25030,'025030','1997-067A','GPS IIA-19','Navstar GPS SVN 38',66147,'In Earth orbit','','1997-12-13 00:00:00',717.9,19912,20449,54.9,'Medium' from dual union all
select 25031,'025031','1997-067B','Delta 249','Delta SSPS AJ-10-118K',66147,'Reentered','2000-10-26 00:00:00','1997-12-06 00:00:00',94.81,459,560,35.2,'Intermediate' from dual union all
select 25032,'025032','1997-067C','Star 48B','Star 48B',66147,'Reentered','2000-11-10 00:00:00','1997-12-06 00:00:00',352.75,186,20133,34.99,'Highly Ellip' from dual union all
select 25033,'025033','1989-085D','Star 48 yo weight','',62362,'Reentered','2000-05-10 00:00:00','1997-11-07 00:00:00',166.09,178,7084,37.4,'Medium' from dual union all
select 25034,'025034','1997-068A','USA 136','TRUMPET 3',66148,'In Earth orbit','','1998-03-18 00:00:00',709.61,1211,38740,64.74,'GEO Transfer' from dual union all
select 25035,'025035','1997-068B','Centaur TC-16','Centaur TC-16',66148,'Reentered','2004-06-01 00:00:00','1998-03-18 00:00:00',709.61,1211,38740,64.74,'GEO Transfer' from dual union all
select 25036,'025036','1986-017MF','EO-24 EVA-5 debris (MSB)','',59210,'Reentered','1998-02-18 00:00:00','1997-11-08 00:00:00',92.22,377,390,51.65,'Intermediate' from dual union all
select 25037,'025037','1986-017MG','EO-24 EVA-5 debris (MSB)','',59210,'Reentered','1998-02-24 00:00:00','1997-11-09 00:00:00',92.21,376,390,51.66,'Intermediate' from dual union all
select 25038,'025038','1986-017MH','EO-24 EVA-5 debris (MSB)','',59210,'Reentered','1998-02-09 00:00:00','1997-11-09 00:00:00',92.2,377,388,51.66,'Intermediate' from dual union all
select 25039,'025039','1997-069A','Iridium 43','Iridium SV043',66149,'Reentered','2018-02-11 00:00:00','1997-12-09 00:00:00',100.4,776,779,86.4,'Polar' from dual union all
select 2504,'002504','1966-093B','11A57I','11A57I',13500,'Reentered','1966-10-29 00:00:00','1966-10-22 00:00:00',89.48,206,291,64.94,'Intermediate' from dual union all
select 25040,'025040','1997-069B','Iridium 41','Iridium SV041',66149,'In Earth orbit','','1997-12-09 00:00:00',100.37,775,778,86.4,'Polar' from dual union all
select 25041,'025041','1997-069C','Iridium 40','Iridium SV040',66149,'In Earth orbit','','1997-12-09 00:00:00',100.21,767,770,86.42,'Polar' from dual union all
select 25042,'025042','1997-069D','Iridium 39','Iridium SV039',66149,'In Earth orbit','','1997-12-09 00:00:00',100.4,776,779,86.4,'Polar' from dual union all
select 25043,'025043','1997-069E','Iridium 38','Iridium SV038',66149,'In Earth orbit','','1997-12-09 00:00:00',100.23,768,771,86.42,'Polar' from dual union all
select 25044,'025044','1997-069F','Delta 250','Delta SSPS AJ-10-118K',66149,'Reentered','1998-04-26 00:00:00','1997-12-10 00:00:00',93.06,252,597,83.56,'Intermediate' from dual union all
select 25045,'025045','1997-070A','Kupon','K95K',66150,'In Earth orbit','','1997-12-14 00:00:00',1436.09,35775,35798,.01,'Stationary' from dual union all
select 25046,'025046','1997-070B','Perekhodnik','Sredniy perekhodnik',66150,'Reentered','1997-11-13 00:00:00','1997-11-13 00:00:00',87.93,167,177,51.63,'Intermediate' from dual union all
select 25047,'025047','1997-070C','8S812','8S812',66150,'Reentered','1997-11-15 00:00:00','1997-11-13 00:00:00',88.23,184,189,51.62,'Intermediate' from dual union all
select 25048,'025048','1997-070D','Blok DM-2M No. 8L','11S861-01 No. 8L',66150,'In Earth orbit','','1998-01-04 00:00:00',1445.24,35909,36022,.08,'Drift GEO' from dual union all
select 25049,'025049','1997-071A','Sirius 2','Sirius 2',66151,'In Earth orbit','','1998-01-04 00:00:00',1436.06,35783,35789,.04,'Stationary' from dual union all
select 2505,'002505','1966-092C','11S510','11S510',13499,'Reentered','1966-11-02 00:00:00','1966-10-21 00:00:00',90.74,200,422,64.82,'Intermediate' from dual union all
select 25050,'025050','1997-071B','Cakrawarta 1','Indostar 1',66151,'In Earth orbit','','1997-12-20 00:00:00',1437.39,35756,35868,.19,'Drift GEO' from dual union all
select 25051,'025051','1997-071C','Ariane H10-3 V102','Ariane H10-3 S/N T469',66151,'Reentered','1998-08-06 00:00:00','1997-12-12 00:00:00',640.13,180,36272,6.91,'GEO Transfer' from dual union all
select 25052,'025052','1997-071D','Spelda 10','Spelda 10',66151,'Reentered','1998-10-21 00:00:00','1997-12-16 00:00:00',623.19,213,35367,6.99,'GEO Transfer' from dual union all
select 25053,'025053','1997-070E','SOZ','SOZ',66150,'In Earth orbit','','1997-12-12 00:00:00',638.26,290,36066,47.5,'GEO Transfer' from dual union all
select 25054,'025054','1997-070F','SOZ','SOZ',66150,'In Earth orbit','','1997-12-12 00:00:00',636.78,232,36048,47.46,'GEO Transfer' from dual union all
select 25055,'025055','1978-064J','deb Seasat','',45259,'Reentered','1999-06-03 00:00:00','1997-11-13 00:00:00',100.83,741,854,71,'Intermediate' from dual union all
select 25056,'025056','1964-026J','deb Transit VC','',10312,'Reentered','2000-04-18 00:00:00','1997-11-11 00:00:00',101,770,842,90.51,'Polar' from dual union all
select 25057,'025057','1990-036C','deb Parus [UNID]','',63278,'In Earth orbit','','1997-11-16 00:00:00',104.53,956,989,82.94,'Intermediate' from dual union all
select 25058,'025058','1965-098L','Alouette 2 part','',11848,'Reentered','1999-12-15 00:00:00','1997-11-16 00:00:00',117.65,498,2653,79.81,'Intermediate' from dual union all
select 25059,'025059','1997-072A','Resurs F-1M','Resurs F-1M',66152,'Landed','1997-12-13 00:00:00','1997-11-25 00:00:00',89.01,209,243,82.33,'Intermediate' from dual union all
select 2506,'002506','1966-088BD','deb OGCh','',13495,'Reentered','1966-10-21 00:00:00','1966-09-18 00:00:00',92.57,196,605,49.7,'Intermediate' from dual union all
select 25060,'025060','1997-072B','11S510','11S510',66152,'Reentered','1997-11-21 00:00:00','1997-11-20 00:00:00',88.1,165,196,82.33,'Intermediate' from dual union all
select 25061,'025061','1997-073A','Columbia (STS-87)','OV-102',66153,'Landed','1997-12-05 00:00:00','1997-11-26 00:00:00',90.09,276,282,28.47,'Intermediate' from dual union all
select 25062,'025062','1997-073B','Spartan 201','Spartan 201',66153,'Landed Attached','1997-12-05 00:00:00','1997-11-24 00:00:00',90.11,277,283,28.47,'Intermediate' from dual union all
select 25063,'025063','1997-074A','TRMM','TRMM',66154,'Reentered','2015-06-16 00:00:00','1997-12-27 00:00:00',91.45,345,347,34.99,'Intermediate' from dual union all
select 25064,'025064','1997-074B','Hikoboshi','ETS-7',66154,'Reentered','2015-11-13 00:00:00','1997-12-27 00:00:00',95.57,545,548,34.97,'Intermediate' from dual union all
select 25065,'025065','1997-074C','H-II Stage 2','H-II Stage 2',66154,'Reentered','1999-01-27 00:00:00','1997-12-27 00:00:00',93.42,366,518,34.54,'Intermediate' from dual union all
select 25066,'025066','1997-074D','4/4D Lower Fairing','TRMM Adapter',66154,'Reentered','1998-02-26 00:00:00','1997-12-27 00:00:00',91.71,352,365,35,'Intermediate' from dual union all
select 25067,'025067','1997-075A','JCSAT 1B','JCSAT 5',66155,'In Earth orbit','','1998-01-01 00:00:00',1436.09,35785,35788,.08,'Stationary' from dual union all
select 25068,'025068','1997-075B','Equator-S','Equator-S',66155,'In Earth orbit','','1998-01-04 00:00:00',1336.65,564,67070,3.75,'Highly Ellip' from dual union all
select 25069,'025069','1997-075C','Ariane H10-3 V103','Ariane H10-3 S/N T473',66155,'Reentered','2000-11-07 00:00:00','1998-01-04 00:00:00',640.15,176,36277,3.88,'GEO Transfer' from dual union all
select 2507,'002507','1966-088BE','deb OGCh','',13495,'Reentered','1966-10-21 00:00:00','1966-09-18 00:00:00',90.5,200,399,49.4,'Intermediate' from dual union all
select 25070,'025070','1997-075D','Cyclade','Cyclade',66155,'Reentered','2001-08-27 00:00:00','1998-01-04 00:00:00',624.55,221,35429,3.91,'GEO Transfer' from dual union all
select 25071,'025071','1997-076A','Astra 1G','Astra 1G',66156,'In Earth orbit','','1998-01-05 00:00:00',1436.19,35170,36407,.04,'Inclined GEO' from dual union all
select 25072,'025072','1997-076B','8S812','8S812',66156,'Reentered','1997-12-05 00:00:00','1997-12-04 00:00:00',87.98,152,196,51.59,'Intermediate' from dual union all
select 25073,'025073','1997-076C','Perekhodnik','Sredniy perekhodnik',66156,'Reentered','1997-12-04 00:00:00','1997-12-03 00:00:00',88.4,188,203,51.56,'Intermediate' from dual union all
select 25074,'025074','1997-076D','Blok DM3 No. 2L','Blok DM3 No. 2L',66156,'In Earth orbit','','1998-01-03 00:00:00',837.45,10174,35931,12.42,'Medium' from dual union all
select 25075,'025075','1994-029ADH','deb HAPS','',65425,'Reentered','1997-12-09 00:00:00','1997-12-02 00:00:00',92.95,413,425,81.58,'Intermediate' from dual union all
select 25076,'025076','1996-049H','deb Pegasus','',65937,'Reentered','2001-02-09 00:00:00','1997-12-04 00:00:00',130.18,324,3937,82.96,'Medium' from dual union all
select 25077,'025077','1997-077A','Iridium 42','Iridium SV042',66157,'In Earth orbit','','1998-01-07 00:00:00',100.13,764,765,86.42,'Polar' from dual union all
select 25078,'025078','1997-077B','Iridium 44','Iridium SV044',66157,'In Earth orbit','','1998-01-07 00:00:00',100.38,776,778,86.4,'Polar' from dual union all
select 25079,'025079','1997-077C','CZ-2C Stage 2','CZ-2C Stage 2',66157,'Reentered','1998-01-20 00:00:00','1997-12-15 00:00:00',92.47,176,616,86.29,'Polar' from dual union all
select 2508,'002508','1966-094A','Luna-12','E-6LF No. 102',13501,'Deep Space','1966-10-25 00:00:00','1966-10-22 00:00:00',21544.43,200,500000,51.8,'Cislunar' from dual union all
select 25080,'025080','1997-077D','Smart Dispenser','CZ-2C SD',66157,'Reentered','1997-12-12 00:00:00','1997-12-10 00:00:00',92.9,210,624,86.23,'Polar' from dual union all
select 25081,'025081','1997-077E','CZ-2C retro cover','',66157,'Reentered','1997-12-12 00:00:00','1997-12-10 00:00:00',92.81,174,651,86.57,'Polar' from dual union all
select 25082,'025082','1997-077F','CZ-2C retro cover','',66157,'Reentered','1997-12-12 00:00:00','1997-12-10 00:00:00',93.65,174,732,86.54,'Polar' from dual union all
select 25083,'025083','1997-077G','CZ-2C retro cover','',66157,'Reentered','1997-12-13 00:00:00','1997-12-10 00:00:00',92.87,178,653,86.29,'Polar' from dual union all
select 25084,'025084','1997-077H','CZ-2C retro cover','',66157,'Reentered','1997-12-12 00:00:00','1997-12-10 00:00:00',92.96,173,665,85.99,'Polar' from dual union all
select 25085,'025085','1991-009CX','deb S3M','',64178,'In Earth orbit','','1997-12-08 00:00:00',118.08,1446,1744,74.11,'Intermediate' from dual union all
select 25086,'025086','1997-078A','Galaxy 8i','Galaxy 8-i',66158,'In Earth orbit','','1998-01-08 00:00:00',1436.19,35784,35793,.54,'Stationary' from dual union all
select 25087,'025087','1997-078B','Centaur AC-149','Centaur IIA(1N) AC-149',66158,'Reentered','1998-06-03 00:00:00','1998-01-15 00:00:00',905.76,179,49085,26.65,'Highly Ellip' from dual union all
select 25088,'025088','1997-079A','Kosmos-2347','US-PU',66159,'Reentered','1999-12-11 00:00:00','1998-01-08 00:00:00',92.78,404,418,65.03,'Intermediate' from dual union all
select 25089,'025089','1997-079B','11S692','11S692',66159,'Reentered','1997-12-09 00:00:00','1997-12-09 00:00:00',88.91,110,332,65,'Intermediate' from dual union all
select 2509,'002509','1966-094B','BOZ','BOZ',13501,'Reentered','1966-10-24 00:00:00','1966-10-22 00:00:00',88.6,199,212,51.92,'Intermediate' from dual union all
select 25090,'025090','1993-032D','Star 48 yo weight','',65134,'Reentered','2009-12-15 00:00:00','1997-11-18 00:00:00',322.75,214,18202,34.84,'Highly Ellip' from dual union all
select 25091,'025091','1997-072C','KDU','KDU',66152,'Reentered','1997-12-22 00:00:00','1997-12-14 00:00:00',88.87,206,232,82.33,'Intermediate' from dual union all
select 25092,'025092','1997-072D','Blenda','Sunshade',66152,'Reentered','1997-12-14 00:00:00','1997-12-14 00:00:00',88.52,195,208,82.34,'Intermediate' from dual union all
select 25093,'025093','1997-072E','PTDU cover','PTDU cover',66152,'Reentered','1997-12-17 00:00:00','1997-12-13 00:00:00',91.08,220,435,82.35,'Intermediate' from dual union all
select 25094,'025094','1997-072F','PTDU cover','PTDU cover',66152,'Reentered','1997-12-18 00:00:00','1997-12-14 00:00:00',90.85,220,412,82.33,'Intermediate' from dual union all
select 25095,'025095','1997-080A','Kosmos-2348','Kobal''t No. 552',66160,'Landed','1998-04-14 00:00:00','1998-01-14 00:00:00',89.2,167,303,67.14,'Intermediate' from dual union all
select 25096,'025096','1997-080B','11S510','11S510',66160,'Reentered','1997-12-20 00:00:00','1997-12-17 00:00:00',89.26,160,316,67.15,'Intermediate' from dual union all
select 25097,'025097','1966-077S','deb FTV 1352','',13484,'In Earth orbit','','1997-12-15 00:00:00',165.92,1901,5347,89.04,'Medium' from dual union all
select 25098,'025098','1979-104D','deb Ariane L01','',47588,'In Earth orbit','','1997-08-17 00:00:00',423.23,457,24133,17.49,'Highly Ellip' from dual union all
select 25099,'025099','1990-103D','Star 48 yo weight','',63345,'Reentered','1999-08-28 00:00:00','1997-12-18 00:00:00',185.8,171,8645,34.51,'Medium' from dual union all
select 251,'000251','62 Epsilon 4','Discoverer 38 camera door','',8188,'Reentered','1962-03-07 00:00:00','1962-03-04 00:00:00',90.92,207,433,82.23,'Intermediate' from dual union all
select 2510,'002510','1966-094C','11S510','11S510',13501,'Reentered','1966-10-23 00:00:00','1966-10-22 00:00:00',88.18,174,195,51.89,'Intermediate' from dual union all
select 25100,'025100','1997-058D','X-Mir Inspector','Inspector',66138,'Reentered','1998-11-02 00:00:00','1997-12-18 00:00:00',92.21,378,387,51.66,'Intermediate' from dual union all
select 25101,'025101','1967-065D','P67-1 despin weight','',15320,'In Earth orbit','','1997-12-19 00:00:00',171.83,3744,3977,90.19,'Medium' from dual union all
select 25102,'025102','1997-081A','Progress M-37','Progress 7K-TGM No 236',66161,'Deorbited','1998-03-16 00:00:00','1998-01-19 00:00:00',92.2,378,386,51.65,'Intermediate' from dual union all
select 25103,'025103','1997-081B','11S510','11S510',66161,'Reentered','1997-12-22 00:00:00','1997-12-21 00:00:00',88.47,181,217,51.6,'Intermediate' from dual union all
select 25104,'025104','1997-082A','Iridium 45','Iridium SV045',66162,'In Earth orbit','','1998-01-19 00:00:00',100.38,775,779,86.4,'Polar' from dual union all
select 25105,'025105','1997-082B','Iridium 46','Iridium SV046',66162,'In Earth orbit','','1998-01-19 00:00:00',100.38,775,779,86.4,'Polar' from dual union all
select 25106,'025106','1997-082C','Iridium 47','Iridium SV047',66162,'In Earth orbit','','1998-01-19 00:00:00',100.38,775,779,86.4,'Polar' from dual union all
select 25107,'025107','1997-082D','Iridium 48','Iridium SV048',66162,'Reentered','2001-05-05 00:00:00','1998-01-19 00:00:00',100.4,775,780,86.4,'Polar' from dual union all
select 25108,'025108','1997-082E','Iridium 49','Iridium SV049',66162,'Reentered','2018-02-13 00:00:00','1998-01-19 00:00:00',100.38,775,779,86.4,'Polar' from dual union all
select 25109,'025109','1997-082F','Delta 251','Delta SSPS AJ-10-118K',66162,'Reentered','1998-06-24 00:00:00','1998-01-19 00:00:00',93.15,258,599,83.61,'Intermediate' from dual union all
select 2511,'002511','1966-056D','Pageos canister half','Pageos canister half',13463,'In Earth orbit','','1980-01-01 00:00:00',181.49,4219,4262,87.02,'Medium' from dual union all
select 25110,'025110','1997-083A','INTELSAT 804','INTELSAT 804',66163,'In Earth orbit','','1998-01-21 00:00:00',1435.88,35776,35789,.13,'Stationary' from dual union all
select 25111,'025111','1997-083B','Ariane H10-3 V104','Ariane H10-3 S/N T474',66163,'Reentered','2004-05-03 00:00:00','1998-01-21 00:00:00',644.98,191,36509,6.77,'GEO Transfer' from dual union all
select 25112,'025112','1997-084A','Orbcomm A1','Orbcomm FM5',66164,'In Earth orbit','','1998-01-22 00:00:00',101.27,814,824,45.03,'Intermediate' from dual union all
select 25113,'025113','1997-084B','Orbcomm A2','Orbcomm FM6',66164,'In Earth orbit','','1998-01-22 00:00:00',101.26,813,824,45.02,'Intermediate' from dual union all
select 25114,'025114','1997-084C','Orbcomm A3','Orbcomm FM7',66164,'In Earth orbit','','1998-01-24 00:00:00',101.27,815,823,45.02,'Intermediate' from dual union all
select 25115,'025115','1997-084D','Orbcomm A4','Orbcomm FM8',66164,'In Earth orbit','','1998-01-22 00:00:00',101.27,816,822,45.02,'Intermediate' from dual union all
select 25116,'025116','1997-084E','Orbcomm A5','Orbcomm FM9',66164,'In Earth orbit','','1998-01-22 00:00:00',101.27,816,822,45.03,'Intermediate' from dual union all
select 25117,'025117','1997-084F','Orbcomm A6','Orbcomm FM10',66164,'In Earth orbit','','1998-01-22 00:00:00',101.31,818,823,45.03,'Intermediate' from dual;
insert into SATELLITE
select 25118,'025118','1997-084G','Orbcomm A7','Orbcomm FM11',66164,'In Earth orbit','','1998-01-22 00:00:00',101.3,817,823,45.03,'Intermediate' from dual union all
select 25119,'025119','1997-084H','Orbcomm A8','Orbcomm FM12',66164,'In Earth orbit','','1998-01-22 00:00:00',101.3,818,822,45.03,'Intermediate' from dual union all
select 2512,'002512','1966-095A','Surveyor Model 3','Surveyor SM-3',13502,'Deep Space','1966-10-27 00:00:00','1966-10-29 00:00:00',19682.62,165,470040,29.67,'Cislunar' from dual union all
select 25120,'025120','1997-084J','Orion 38','Orion 38',66164,'Reentered','1998-12-28 00:00:00','1998-01-22 00:00:00',94.63,289,712,44.97,'Intermediate' from dual union all
select 25121,'025121','1997-084K','HAPS','HAPS',66164,'Reentered','2001-09-01 00:00:00','1998-01-22 00:00:00',96.98,407,821,42.68,'Intermediate' from dual union all
select 25122,'025122','1967-065E','P67-1 despin weight','',15320,'In Earth orbit','','1997-12-23 00:00:00',172.45,3869,3901,90.02,'Medium' from dual union all
select 25123,'025123','1997-085A','EarlyBird','EarlyBird',66165,'Reentered','2000-07-27 00:00:00','1998-01-23 00:00:00',94.25,479,486,97.3,'Sun-Synch' from dual union all
select 25124,'025124','1997-085B','Start','Start/DS',66165,'Reentered','2000-12-12 00:00:00','1998-01-23 00:00:00',94.23,479,484,97.3,'Sun-Synch' from dual union all
select 25125,'025125','1997-085C','Start DS cover','',66165,'Reentered','1999-03-31 00:00:00','1998-01-23 00:00:00',94.21,476,485,97.3,'Sun-Synch' from dual union all
select 25126,'025126','1997-086A','Asiasat 3','Asiasat 3',66166,'Deep Space','1998-04-24 00:00:00','1998-05-17 00:00:00',22254.26,35645,475760,18.22,'Cislunar' from dual union all
select 25127,'025127','1997-086B','8S812','8S812',66166,'Reentered','1997-12-27 00:00:00','1997-12-26 00:00:00',87.99,149,201,51.61,'Intermediate' from dual union all
select 25128,'025128','1997-086C','Perekhodnik','Sredniy perekhodnik',66166,'Reentered','1997-12-25 00:00:00','1997-12-25 00:00:00',88.38,178,211,51.59,'Intermediate' from dual union all
select 25129,'025129','1997-086D','Blok DM3 No. 5L','Blok DM3 No. 5L',66166,'Reentered','2000-10-07 00:00:00','1998-01-23 00:00:00',636.27,273,35981,51.54,'GEO Transfer' from dual union all
select 2513,'002513','1966-095B','Centaur D AC-9','Centaur 8D',13502,'Deep Space','1966-10-27 00:00:00','1966-10-29 00:00:00',13197.8,164,357071,29.66,'Cislunar' from dual union all
select 25130,'025130','1980-028F','deb Kosmos-1172','',50238,'Reentered','1997-12-24 00:00:00','1997-12-24 00:00:00',94.29,71,897,61.72,'ERROR - Unexpected value "TA"' from dual union all
select 25131,'025131','1998-001A','Lunar Prospector','LP',66437,'Deep Space','1998-01-07 00:00:00','1998-01-07 00:00:00',13016.67,197,353645,29.2,'Cislunar' from dual union all
select 25132,'025132','1998-001B','OAM','OAM',66437,'Reentered','1998-01-07 00:00:00','1998-01-07 00:00:00',87.87,141,198,29.25,'Intermediate' from dual union all
select 25133,'025133','1986-017MJ','EO-24 EVA-6 debris','',59210,'Reentered','1998-02-22 00:00:00','1998-01-09 00:00:00',92.16,376,385,51.68,'Intermediate' from dual union all
select 25134,'025134','1998-002A','Skynet 4D','Skynet 4D',66438,'In Earth orbit','','1998-02-12 00:00:00',1433.84,35672,35813,4.15,'Inclined Drift' from dual union all
select 25135,'025135','1998-002B','Delta 252','Delta SSPS AJ-10-118K',66438,'In Earth orbit','','1998-02-09 00:00:00',105.75,764,1294,26.88,'Intermediate' from dual union all
select 25136,'025136','1998-002C','Star 48B','Star 48B',66438,'In Earth orbit','','1998-02-09 00:00:00',654.96,1165,36044,23.58,'GEO Transfer' from dual union all
select 25137,'025137','1998-002D','deb Delta','',66438,'Reentered','2000-04-12 00:00:00','1998-02-10 00:00:00',99.55,335,1140,28.56,'Intermediate' from dual union all
select 25138,'025138','1998-002E','deb Delta','',66438,'In Earth orbit','','1998-02-09 00:00:00',105.78,767,1295,26.88,'Intermediate' from dual union all
select 25139,'025139','1998-002F','deb Delta','',66438,'Reentered','1999-05-31 00:00:00','1998-02-09 00:00:00',99.47,332,1135,28.57,'Intermediate' from dual union all
select 2514,'002514','1966-096A','INTELSAT II F-1','INTELSAT II F-1',13503,'In Earth orbit','','1967-07-17 00:00:00',717.62,3171,37176,16.9,'GEO Transfer' from dual union all
select 25140,'025140','1986-017MK','EO-24 EVA-7 debris','',59210,'Reentered','1998-04-06 00:00:00','1998-01-14 00:00:00',92.16,377,384,51.66,'Intermediate' from dual union all
select 25141,'025141','1997-082G','deb Iridium 47','',66162,'Reentered','1998-02-17 00:00:00','1998-01-15 00:00:00',96.59,583,608,86.52,'Polar' from dual union all
select 25142,'025142','1986-017ML','EO-24 EVA-7 debris','',59210,'Reentered','1998-04-13 00:00:00','1998-01-17 00:00:00',92.19,378,386,51.66,'Intermediate' from dual union all
select 25143,'025143','1998-003A','Endeavour (STS-89)','OV-105',66439,'Landed','1998-01-31 00:00:00','1998-01-25 00:00:00',92.19,377,387,51.66,'Intermediate' from dual union all
select 25144,'025144','1987-079M','deb SOZ','',60169,'Reentered','1998-04-25 00:00:00','1998-01-26 00:00:00',128.63,370,3755,67.21,'Medium' from dual union all
select 25145,'025145','1978-016E','deb Centaur AC-44','',45211,'Reentered','1998-01-27 00:00:00','1998-01-23 00:00:00',97.95,248,1074,26.23,'Intermediate' from dual union all
select 25146,'025146','1998-004A','Soyuz TM-27','Soyuz 7K-STM No. 76',66440,'Landed','1998-08-25 00:00:00','1998-02-28 00:00:00',92.16,377,384,51.66,'Intermediate' from dual union all
select 25147,'025147','1998-004B','11S510','11S510',66440,'Reentered','1998-01-31 00:00:00','1998-01-30 00:00:00',88.39,184,205,51.6,'Intermediate' from dual union all
select 25148,'025148','1998-005A','USA 137','QUASAR 11',66441,'Reentered','2010-01-01 00:00:00','1998-01-30 00:00:00',718.59,795,39600,63.4,'Molniya' from dual union all
select 25149,'025149','1998-005B','Centaur AC-109','Centaur IIA AC-109',66441,'Reentered','2010-01-01 00:00:00','1998-01-30 00:00:00',718.59,795,39600,63.4,'Molniya' from dual union all
select 2515,'002515','1966-096B','Delta 42','DSV-3E-3 S/N 20206',13503,'Reentered','1967-02-09 00:00:00','1966-11-26 00:00:00',92.18,240,522,28.73,'Intermediate' from dual union all
select 25150,'025150','1992-072E','deb Ariane V54','',64784,'Reentered','1998-08-08 00:00:00','1998-02-04 00:00:00',238.16,182,12516,7.62,'Medium' from dual union all
select 25151,'025151','1992-072F','deb Ariane V54','',64784,'Reentered','2004-05-25 00:00:00','1997-11-30 00:00:00',438.13,364,25097,6.63,'Highly Ellip' from dual union all
select 25152,'025152','1998-006A','Brasilsat B3','Brasilsat B3',66442,'In Earth orbit','','1998-03-24 00:00:00',1436.34,35784,35798,.01,'Stationary' from dual union all
select 25153,'025153','1998-006B','INMARSAT III F-5','INMARSAT 3 F5',66442,'In Earth orbit','','1998-03-06 00:00:00',1436.85,35783,35820,2.89,'Drift GEO' from dual union all
select 25154,'025154','1998-006C','Ariane H10-3 V105','Ariane H10-3 S/N T476',66442,'Reentered','2004-03-05 00:00:00','1998-03-07 00:00:00',642.79,204,36384,6.78,'GEO Transfer' from dual union all
select 25155,'025155','1998-006D','Spelda 10 S/N UV30','Spelda 10',66442,'Reentered','2001-12-05 00:00:00','1998-03-07 00:00:00',623.4,230,35361,6.92,'GEO Transfer' from dual union all
select 25156,'025156','1963-049N','deb Transit VBN-2','',9280,'In Earth orbit','','1998-02-06 00:00:00',106.96,1060,1112,90.1,'Polar' from dual union all
select 25157,'025157','1998-007A','GFO','GFO',66443,'In Earth orbit','','1998-03-12 00:00:00',101.47,781,875,107.99,'Retrograde' from dual union all
select 25158,'025158','1998-007B','Orbcomm G1','Orbcomm FM3',66443,'In Earth orbit','','1998-03-12 00:00:00',101.46,782,874,108,'Retrograde' from dual union all
select 25159,'025159','1998-007C','Orbcomm G2','Orbcomm FM4',66443,'In Earth orbit','','1998-03-12 00:00:00',100.67,777,804,108,'Retrograde' from dual union all
select 2516,'002516','1966-052D','OV3-4 despin weight','',13459,'In Earth orbit','','1966-09-29 00:00:00',145.28,702,4850,41.01,'Medium' from dual union all
select 25160,'025160','1998-007D','Orion 38','Orion 38',66443,'In Earth orbit','','1998-03-14 00:00:00',101.43,782,871,107.99,'Retrograde' from dual union all
select 25161,'025161','1998-007E','DPAF','Dual Payload Attach Fitt',66443,'In Earth orbit','','1998-03-15 00:00:00',101.47,782,875,107.99,'Retrograde' from dual union all
select 25162,'025162','1998-008A','Globalstar FM1','Globalstar FM1',66444,'In Earth orbit','','1998-03-17 00:00:00',110.65,1248,1266,52.01,'Intermediate' from dual union all
select 25163,'025163','1998-008B','Globalstar FM4','Globalstar FM4',66444,'In Earth orbit','','1998-03-17 00:00:00',110.4,1234,1257,52,'Intermediate' from dual union all
select 25164,'025164','1998-008C','Globalstar FM2','Globalstar FM2',66444,'In Earth orbit','','1998-03-19 00:00:00',110.6,1249,1260,52,'Intermediate' from dual union all
select 25165,'025165','1998-008D','Globalstar FM3','Globalstar FM3',66444,'In Earth orbit','','1998-03-16 00:00:00',110.61,1248,1262,51.99,'Intermediate' from dual union all
select 25166,'025166','1998-008E','Delta 253','Delta SSPS AJ-10-118K',66444,'Reentered','1999-03-16 00:00:00','1998-03-16 00:00:00',99.38,244,1214,52.73,'Intermediate' from dual union all
select 25167,'025167','1998-009A','Kosmos-2349','Kometa No. 19',66445,'Landed','1998-04-02 00:00:00','1998-02-24 00:00:00',89.32,210,272,70.37,'Intermediate' from dual union all
select 25168,'025168','1998-009B','11S510','11S510',66445,'Reentered','1998-02-21 00:00:00','1998-02-19 00:00:00',88.62,181,232,70.38,'Intermediate' from dual union all
select 25169,'025169','1998-010A','Iridium 52','Iridium SV052',66446,'In Earth orbit','','1998-03-20 00:00:00',100.4,774,780,86.39,'Polar' from dual union all
select 2517,'002517','1966-097A','OV3-2','OV3-2',13504,'Reentered','1971-09-29 00:00:00','1966-12-07 00:00:00',104.1,316,1588,81.97,'Intermediate' from dual union all
select 25170,'025170','1998-010B','Iridium 56','Iridium SV056',66446,'In Earth orbit','','1998-03-21 00:00:00',100.4,775,780,86.39,'Polar' from dual union all
select 25171,'025171','1998-010C','Iridium 54','Iridium SV054',66446,'In Earth orbit','','1998-03-27 00:00:00',98.61,690,695,86.54,'Polar' from dual union all
select 25172,'025172','1998-010D','Iridium 50','Iridium SV050',66446,'In Earth orbit','','1998-03-20 00:00:00',100.37,774,778,86.4,'Polar' from dual union all
select 25173,'025173','1998-010E','Iridium 53','Iridium SV053',66446,'In Earth orbit','','1998-03-20 00:00:00',100.38,775,779,86.39,'Polar' from dual union all
select 25174,'025174','1998-010F','Delta 254','Delta SSPS AJ-10-118K',66446,'Reentered','1998-08-27 00:00:00','1998-03-21 00:00:00',93.17,262,598,83.66,'Intermediate' from dual union all
select 25175,'025175','1998-011A','Kakehashi','COMETS',66447,'In Earth orbit','','1998-06-01 00:00:00',319.2,472,17715,30,'Highly Ellip' from dual union all
select 25176,'025176','1998-011B','H-II Stage 2','H-II Stage 2',66447,'Reentered','2000-02-01 00:00:00','1998-02-21 00:00:00',106.51,247,1883,30.04,'Intermediate' from dual union all
select 25177,'025177','1987-068C','deb S5M','',60158,'Reentered','1998-05-16 00:00:00','1998-02-23 00:00:00',103.03,754,1050,82.43,'Intermediate' from dual union all
select 25178,'025178','1987-068D','deb S5M','',60158,'Reentered','1998-07-07 00:00:00','1998-02-24 00:00:00',104.15,931,979,82.37,'Intermediate' from dual union all
select 25179,'025179','1987-068E','deb S5M','',60158,'Reentered','1998-06-29 00:00:00','1998-02-24 00:00:00',104.18,902,1009,82.37,'Intermediate' from dual union all
select 2518,'002518','1966-087C','ESSA 3 despin weight','',13494,'In Earth orbit','','1966-11-05 00:00:00',115.91,1433,1561,100.83,'Sun-Synch' from dual union all
select 25180,'025180','1987-068F','deb S5M','',60158,'Reentered','2000-09-10 00:00:00','1998-02-24 00:00:00',101.11,656,967,82.54,'Intermediate' from dual union all
select 25181,'025181','1987-068G','deb S5M','',60158,'Reentered','1999-01-24 00:00:00','1998-02-24 00:00:00',104.23,927,990,82.38,'Intermediate' from dual union all
select 25182,'025182','1987-068H','deb S5M','',60158,'Reentered','1999-04-26 00:00:00','1998-02-25 00:00:00',104.19,935,978,82.45,'Intermediate' from dual union all
select 25183,'025183','1987-068J','deb S5M','',60158,'Reentered','1998-06-30 00:00:00','1998-02-24 00:00:00',104.22,928,987,82.39,'Intermediate' from dual union all
select 25184,'025184','1987-068K','deb S5M','',60158,'Reentered','1999-06-20 00:00:00','1998-02-23 00:00:00',104.24,938,980,82.55,'Intermediate' from dual union all
select 25185,'025185','1987-068L','deb S5M','',60158,'Reentered','1999-06-12 00:00:00','1998-02-24 00:00:00',104.61,951,1001,82.6,'Intermediate' from dual union all
select 25186,'025186','1987-068M','deb S5M','',60158,'Reentered','1999-03-09 00:00:00','1998-02-24 00:00:00',105.3,960,1056,82.57,'Intermediate' from dual union all
select 25187,'025187','1987-068N','deb S5M','',60158,'In Earth orbit','','1998-02-24 00:00:00',103.93,936,953,82.56,'Intermediate' from dual union all
select 25188,'025188','1987-068P','deb S5M','',60158,'Reentered','1999-05-14 00:00:00','1998-02-24 00:00:00',104.32,956,969,82.59,'Intermediate' from dual union all
select 25189,'025189','1987-068Q','deb S5M','',60158,'Reentered','2006-11-03 00:00:00','1998-02-24 00:00:00',105.9,947,1125,82.16,'Intermediate' from dual union all
select 2519,'002519','1966-097B','FW4S S/N 30201','FW4S',13504,'Reentered','1970-01-28 00:00:00','1966-11-27 00:00:00',104.09,317,1587,81.98,'Intermediate' from dual union all
select 25190,'025190','1987-068R','deb S5M','',60158,'Reentered','1999-02-03 00:00:00','1998-02-24 00:00:00',103.74,883,988,82.58,'Intermediate' from dual union all
select 25191,'025191','1987-068S','deb S5M','',60158,'Reentered','2002-01-20 00:00:00','1998-02-24 00:00:00',102.24,778,952,82.41,'Intermediate' from dual union all
select 25192,'025192','1987-068T','deb S5M','',60158,'Reentered','1999-01-08 00:00:00','1998-02-25 00:00:00',103.35,873,960,82.57,'Intermediate' from dual union all
select 25193,'025193','1987-068U','deb S5M','',60158,'Reentered','1999-01-28 00:00:00','1998-02-25 00:00:00',104.53,951,994,82.6,'Intermediate' from dual union all
select 25194,'025194','1987-068V','deb S5M','',60158,'Reentered','2003-01-12 00:00:00','1998-02-24 00:00:00',100.48,584,979,82.06,'Intermediate' from dual union all
select 25195,'025195','1987-068W','deb S5M','',60158,'Reentered','1998-05-18 00:00:00','1998-02-24 00:00:00',103.24,785,1039,82.44,'Intermediate' from dual union all
select 25196,'025196','1987-068X','deb S5M','',60158,'Reentered','1998-05-02 00:00:00','1998-02-24 00:00:00',102.83,747,1038,82.44,'Intermediate' from dual union all
select 25197,'025197','1987-068Y','deb S5M','',60158,'Reentered','1999-03-06 00:00:00','1998-02-24 00:00:00',104.1,933,972,82.55,'Intermediate' from dual union all
select 25198,'025198','1987-068Z','deb S5M','',60158,'Reentered','1999-07-13 00:00:00','1998-02-24 00:00:00',104.47,957,983,82.61,'Intermediate' from dual union all
select 25199,'025199','1987-068AA','deb S5M','',60158,'In Earth orbit','','1998-02-25 00:00:00',105.55,942,1099,82.37,'Intermediate' from dual union all
select 252,'000252','61 Omicron 76','deb r/b Ablestar','',7273,'In Earth orbit','','1962-03-09 00:00:00',105.57,972,1070,67.09,'Intermediate' from dual union all
select 2520,'002520','1966-089B','EGRS VIII','Secor Type II S/N 9',13496,'In Earth orbit','','1966-12-09 00:00:00',167.64,3682,3704,90.2,'Medium' from dual union all
select 25200,'025200','1987-068AB','deb S5M','',60158,'Reentered','2001-10-26 00:00:00','1998-02-24 00:00:00',101.15,661,965,82.5,'Intermediate' from dual union all
select 25201,'025201','1987-068AC','deb S5M','',60158,'Reentered','1998-07-05 00:00:00','1998-02-24 00:00:00',104.11,911,994,82.38,'Intermediate' from dual union all
select 25202,'025202','1987-068AD','deb S5M','',60158,'Reentered','1999-07-28 00:00:00','1998-02-24 00:00:00',100.93,637,969,82.06,'Intermediate' from dual union all
select 25203,'025203','1987-068AE','deb S5M','',60158,'In Earth orbit','','1998-02-25 00:00:00',102.28,783,951,82.27,'Intermediate' from dual union all
select 25204,'025204','1987-068AF','deb S5M','',60158,'In Earth orbit','','1998-02-25 00:00:00',105.45,910,1121,82.44,'Intermediate' from dual union all
select 25205,'025205','1987-068AG','deb S5M','',60158,'In Earth orbit','','1998-02-24 00:00:00',103.95,934,957,82.53,'Intermediate' from dual union all
select 25206,'025206','1987-068AH','deb S5M','',60158,'Reentered','1999-05-08 00:00:00','1998-02-25 00:00:00',104.61,955,997,82.56,'Intermediate' from dual union all
select 25207,'025207','1987-068AJ','deb S5M','',60158,'Reentered','1999-03-29 00:00:00','1998-02-24 00:00:00',104.49,951,991,82.55,'Intermediate' from dual;
insert into SATELLITE
select 25208,'025208','1987-068AK','deb S5M','',60158,'Reentered','1999-11-11 00:00:00','1998-02-24 00:00:00',104.48,953,988,82.55,'Intermediate' from dual union all
select 25209,'025209','1987-068AL','deb S5M','',60158,'Reentered','1999-05-23 00:00:00','1998-02-25 00:00:00',103.99,932,963,82.5,'Intermediate' from dual union all
select 2521,'002521','1966-070C','OV3-3 despin weight','',13477,'Reentered','1974-06-30 00:00:00','1966-09-30 00:00:00',134.42,290,4339,81.46,'Medium' from dual union all
select 25210,'025210','1987-068AM','deb S5M','',60158,'Reentered','1998-12-23 00:00:00','1998-02-24 00:00:00',103.28,856,972,82.55,'Intermediate' from dual union all
select 25211,'025211','1987-068AN','deb S5M','',60158,'Reentered','1999-02-22 00:00:00','1998-02-25 00:00:00',103.71,887,981,82.37,'Intermediate' from dual union all
select 25212,'025212','1987-068AP','deb S5M','',60158,'Reentered','1999-05-02 00:00:00','1998-02-24 00:00:00',103.99,931,963,82.47,'Intermediate' from dual union all
select 25213,'025213','1987-068AQ','deb S5M','',60158,'Reentered','1999-03-11 00:00:00','1998-02-24 00:00:00',104.08,945,958,82.53,'Intermediate' from dual union all
select 25214,'025214','1987-068AR','deb S5M','',60158,'In Earth orbit','','1998-02-24 00:00:00',104,937,959,82.55,'Intermediate' from dual union all
select 25215,'025215','1987-068AS','deb S5M','',60158,'Reentered','1998-06-12 00:00:00','1998-02-24 00:00:00',103.62,884,976,82.38,'Intermediate' from dual union all
select 25216,'025216','1987-068AT','deb S5M','',60158,'Reentered','1999-07-31 00:00:00','1998-02-24 00:00:00',105.29,954,1062,82.56,'Intermediate' from dual union all
select 25217,'025217','1987-068AU','deb S5M','',60158,'Reentered','1999-02-17 00:00:00','1998-02-24 00:00:00',99.4,510,949,82,'Intermediate' from dual union all
select 25218,'025218','1987-068AV','deb S5M','',60158,'Reentered','1999-04-14 00:00:00','1998-02-24 00:00:00',104.48,955,986,82.55,'Intermediate' from dual union all
select 25219,'025219','1987-068AW','deb S5M','',60158,'Reentered','1998-07-09 00:00:00','1998-02-24 00:00:00',104.55,961,985,82.57,'Intermediate' from dual union all
select 2522,'002522','1963-014AH','deb West Ford cluster','RCS 2.5',9245,'Reentered','1978-01-18 00:00:00','1966-10-25 00:00:00',165.73,2302,4931,86.84,'Medium' from dual union all
select 25220,'025220','1987-068AX','deb S5M','',60158,'Reentered','2000-01-30 00:00:00','1998-02-24 00:00:00',104.28,952,969,82.57,'Intermediate' from dual union all
select 25221,'025221','1987-068AY','deb S5M','',60158,'In Earth orbit','','1998-02-24 00:00:00',106.7,946,1202,82.3,'Intermediate' from dual union all
select 25222,'025222','1987-068AZ','deb S5M','',60158,'Reentered','1999-05-01 00:00:00','1998-02-24 00:00:00',99.82,541,959,82.22,'Intermediate' from dual union all
select 25223,'025223','1987-068BA','deb S5M','',60158,'Reentered','2000-04-11 00:00:00','1998-02-24 00:00:00',104.15,942,968,82.49,'Intermediate' from dual union all
select 25224,'025224','1987-068BB','deb S5M','',60158,'Reentered','2001-01-10 00:00:00','1998-02-25 00:00:00',104.62,952,1002,82.55,'Intermediate' from dual union all
select 25225,'025225','1987-068BC','deb S5M','',60158,'In Earth orbit','','1998-02-24 00:00:00',105.24,942,1070,82.55,'Intermediate' from dual union all
select 25226,'025226','1987-068BD','deb S5M','',60158,'In Earth orbit','','1998-02-24 00:00:00',105.24,942,1070,82.54,'Intermediate' from dual union all
select 25227,'025227','1987-068BE','deb S5M','',60158,'In Earth orbit','','1998-02-24 00:00:00',105.24,942,1070,82.54,'Intermediate' from dual union all
select 25228,'025228','1987-068BF','deb S5M','',60158,'Reentered','1999-10-12 00:00:00','1998-02-23 00:00:00',99.2,457,983,82.06,'Intermediate' from dual union all
select 25229,'025229','1987-068BG','deb S5M','',60158,'Reentered','1999-01-11 00:00:00','1998-02-25 00:00:00',103.72,897,971,82.55,'Intermediate' from dual union all
select 2523,'002523','1966-098A','OPS 2070','GAMBIT SV 984',13505,'Deorbited','1966-11-10 00:00:00','1966-11-03 00:00:00',89.14,159,305,91,'Polar' from dual union all
select 25230,'025230','1987-068BH','deb S5M','',60158,'Reentered','1999-01-15 00:00:00','1998-02-24 00:00:00',103.61,882,977,82.55,'Intermediate' from dual union all
select 25231,'025231','1987-068BJ','deb S5M','',60158,'Reentered','1998-11-11 00:00:00','1998-02-23 00:00:00',102.98,745,1055,82.44,'Intermediate' from dual union all
select 25232,'025232','1987-068BK','deb S5M','',60158,'Reentered','1998-12-19 00:00:00','1998-02-24 00:00:00',103.54,875,977,82.54,'Intermediate' from dual union all
select 25233,'025233','1998-012A','SNOE','SNOE',66448,'Reentered','2003-12-13 00:00:00','1998-03-31 00:00:00',95.79,535,579,97.75,'Sun-Synch' from dual union all
select 25234,'025234','1998-012B','T1','BATSAT',66448,'Reentered','2000-10-09 00:00:00','1998-03-29 00:00:00',95.78,534,578,97.75,'Sun-Synch' from dual union all
select 25235,'025235','1998-012C','Orion 38','Orion 38',66448,'Reentered','2000-05-25 00:00:00','1998-03-29 00:00:00',95.81,536,580,97.75,'Sun-Synch' from dual union all
select 25236,'025236','1998-012D','DPAF','DPAF',66448,'Reentered','2003-03-10 00:00:00','1998-03-29 00:00:00',95.76,532,579,97.75,'Sun-Synch' from dual union all
select 25237,'025237','1998-013A','Hot Bird 4','Eutelsat HB4',66449,'In Earth orbit','','1998-03-31 00:00:00',1435.93,35758,35809,.03,'Stationary' from dual union all
select 25238,'025238','1998-013B','Ariane H10-3 V106','Ariane H10-3 S/N T475',66449,'Reentered','1998-04-08 00:00:00','1998-03-15 00:00:00',306.99,118,17274,6.87,'Highly Ellip' from dual union all
select 25239,'025239','1998-014A','INTELSAT 806','INTELSAT 806',66450,'In Earth orbit','','1998-04-01 00:00:00',1436.11,35783,35790,.11,'Stationary' from dual union all
select 2524,'002524','1966-099A','MOL Canister','OV4-3',13506,'Reentered Attached','1967-01-09 00:00:00','1966-12-04 00:00:00',90.08,277,280,32.84,'Intermediate' from dual union all
select 25240,'025240','1998-014B','Centaur AC-151','Centaur IIA(1N) AC-151',66450,'Reentered','2003-04-27 00:00:00','1998-03-30 00:00:00',582,158,33268,22.96,'Highly Ellip' from dual union all
select 25241,'025241','1998-010G','deb Iridium 53','',66446,'Reentered','1998-03-05 00:00:00','1998-03-02 00:00:00',95.21,503,555,86.55,'Polar' from dual union all
select 25242,'025242','1998-010H','deb Iridium 53','',66446,'Reentered','1998-03-14 00:00:00','1998-03-02 00:00:00',96.29,572,590,86.56,'Polar' from dual union all
select 25243,'025243','1987-068BL','deb S5M','',60158,'Reentered','1999-01-07 00:00:00','1998-03-01 00:00:00',103.36,868,967,82.59,'Intermediate' from dual union all
select 25244,'025244','1987-068BM','deb S5M','',60158,'Reentered','1999-06-13 00:00:00','1998-03-02 00:00:00',104.44,957,979,82.56,'Intermediate' from dual union all
select 25245,'025245','1987-068BN','deb S5M','',60158,'Reentered','1999-05-01 00:00:00','1998-03-01 00:00:00',104.35,953,975,82.62,'Intermediate' from dual union all
select 25246,'025246','1987-068BP','deb S5M','',60158,'In Earth orbit','','1998-03-01 00:00:00',102.9,839,952,82.25,'Intermediate' from dual union all
select 25247,'025247','1987-068BQ','deb S5M','',60158,'Reentered','1999-01-24 00:00:00','1998-03-01 00:00:00',103.93,915,974,82.42,'Intermediate' from dual union all
select 25248,'025248','1987-068BR','deb S5M','',60158,'Reentered','1999-04-27 00:00:00','1998-03-02 00:00:00',104.15,951,959,82.53,'Intermediate' from dual union all
select 25249,'025249','1987-068BS','deb S5M','',60158,'Reentered','2000-05-29 00:00:00','1998-03-01 00:00:00',104.23,947,970,82.49,'Intermediate' from dual union all
select 2525,'002525','1966-098B','Agena D 4831','Agena D',13505,'Reentered','1966-11-16 00:00:00','1966-11-08 00:00:00',89.61,200,311,90.96,'Polar' from dual union all
select 25250,'025250','1987-068BT','deb S5M','',60158,'Reentered','1998-06-05 00:00:00','1998-03-01 00:00:00',100.27,598,945,82.26,'Intermediate' from dual union all
select 25251,'025251','1987-068BU','deb S5M','',60158,'Reentered','1998-11-14 00:00:00','1998-03-01 00:00:00',98.59,443,940,82.62,'Intermediate' from dual union all
select 25252,'025252','1987-068BV','deb S5M','',60158,'Reentered','1999-06-01 00:00:00','1998-03-02 00:00:00',104.54,964,981,82.53,'Intermediate' from dual union all
select 25253,'025253','1978-068C','deb Centaur AC-41','',45263,'In Earth orbit','','1998-01-08 00:00:00',640.25,877,35581,21.47,'GEO Transfer' from dual union all
select 25254,'025254','1993-059H','deb 11S772','',65161,'In Earth orbit','','1998-03-10 00:00:00',101.68,825,851,70.99,'Intermediate' from dual union all
select 25255,'025255','1981-043D','deb Blok E','',52566,'In Earth orbit','','1998-03-12 00:00:00',102.39,827,917,81.27,'Intermediate' from dual union all
select 25256,'025256','1998-015A','Progress M-38','Progress 7K-TGM No 240',66451,'Deorbited','1998-05-16 00:00:00','1998-04-15 00:00:00',92.07,374,379,51.66,'Intermediate' from dual union all
select 25257,'025257','1998-015B','11S510','11S510',66451,'Reentered','1998-03-16 00:00:00','1998-03-15 00:00:00',88.6,186,225,51.63,'Intermediate' from dual union all
select 25258,'025258','1998-016A','UHF F/O F8','UFO F8',66452,'In Earth orbit','','1998-04-15 00:00:00',1435.91,35474,36092,6.04,'Inclined GEO' from dual union all
select 25259,'025259','1998-016B','Centaur AC-132','Centaur II(3) AC-132',66452,'Reentered','2007-10-24 00:00:00','1998-04-15 00:00:00',412.1,224,23708,26.99,'Highly Ellip' from dual union all
select 2526,'002526','1966-099B','OV4-1R','OV4-1R',13506,'Reentered','1967-01-05 00:00:00','1966-12-05 00:00:00',89.95,272,272,32.84,'Intermediate' from dual union all
select 25260,'025260','1998-017A','SPOT-4','SPOT-4',66453,'In Earth orbit','','1998-04-23 00:00:00',101.39,824,825,98.74,'Sun-Synch' from dual union all
select 25261,'025261','1998-017B','Ariane H10-3 V107','Ariane H10-3 S/N T477',66453,'In Earth orbit','','1998-04-23 00:00:00',100.78,794,798,98.65,'Sun-Synch' from dual union all
select 25262,'025262','1998-018A','Iridium 51','Iridium SV051',66454,'In Earth orbit','','1998-04-24 00:00:00',100.4,775,780,86.4,'Polar' from dual union all
select 25263,'025263','1998-018B','Iridium 61','Iridium SV061',66454,'In Earth orbit','','1998-04-24 00:00:00',98.65,693,696,86.54,'Polar' from dual union all
select 25264,'025264','1998-018C','CZ-2C Stage 2','CZ-2C Stage 2',66454,'Reentered','1998-04-24 00:00:00','1998-04-01 00:00:00',92.29,174,600,86.38,'Polar' from dual union all
select 25265,'025265','1998-018D','Smart Dispenser','CZ-2C SD',66454,'Reentered','1998-05-10 00:00:00','1998-04-02 00:00:00',92.38,194,589,86.31,'Polar' from dual union all
select 25266,'025266','1998-018E','CZ-2 retro cover','',66454,'Reentered','1998-03-29 00:00:00','1998-03-27 00:00:00',91.81,170,557,86.68,'Polar' from dual union all
select 25267,'025267','1998-018F','CZ-2 retro cover','',66454,'Reentered','1998-03-30 00:00:00','1998-03-27 00:00:00',93.51,177,716,86.1,'Polar' from dual union all
select 25268,'025268','1998-018G','CZ-2 retro cover','',66454,'Reentered','1998-03-30 00:00:00','1998-03-27 00:00:00',93.62,177,727,86.12,'Polar' from dual union all
select 25269,'025269','1998-018H','CZ-2 retro cover','',66454,'Reentered','1998-03-28 00:00:00','1998-03-28 00:00:00',90.96,165,479,86.63,'Polar' from dual union all
select 2527,'002527','1966-099C','OV1-6S','OV1-6S',13506,'Reentered','1966-12-31 00:00:00','1966-11-10 00:00:00',90.32,289,291,32.84,'Intermediate' from dual union all
select 25270,'025270','1968-055E','deb Star 37D','',16782,'In Earth orbit','','1998-03-12 00:00:00',147.37,778,4950,120.78,'Medium' from dual union all
select 25271,'025271','1994-029ADJ','deb Pegasus','',65425,'Reentered','2004-11-24 00:00:00','1998-03-27 00:00:00',104.98,509,1478,81.82,'Intermediate' from dual union all
select 25272,'025272','1998-019A','Iridium 55','Iridium SV055',66455,'In Earth orbit','','1998-04-29 00:00:00',100.38,774,779,86.39,'Polar' from dual union all
select 25273,'025273','1998-019B','Iridium 57','Iridium SV057',66455,'In Earth orbit','','1998-04-29 00:00:00',100.38,775,779,86.4,'Polar' from dual union all
select 25274,'025274','1998-019C','Iridium 58','Iridium SV058',66455,'In Earth orbit','','1998-04-29 00:00:00',100.38,776,778,86.39,'Polar' from dual union all
select 25275,'025275','1998-019D','Iridium 59','Iridium SV059',66455,'In Earth orbit','','1998-04-29 00:00:00',100.4,775,780,86.39,'Polar' from dual union all
select 25276,'025276','1998-019E','Iridium 60','Iridium SV060',66455,'In Earth orbit','','1998-04-29 00:00:00',100.38,773,780,86.39,'Polar' from dual union all
select 25277,'025277','1998-019F','Delta 255','Delta SSPS AJ-10-118K',66455,'Reentered','1998-05-11 00:00:00','1998-04-06 00:00:00',92.3,192,583,83.01,'Intermediate' from dual union all
select 25278,'025278','1964-006AF','deb Blok E','',10292,'Reentered','1998-07-11 00:00:00','1998-03-30 00:00:00',273.32,125,15020,56.24,'Highly Ellip' from dual union all
select 25279,'025279','1989-062E','Ariane H10 V33 main deb','H10 deb [RCS 30.6 sq m]',62339,'In Earth orbit','','1998-03-28 00:00:00',616.26,368,34853,7.22,'GEO Transfer' from dual union all
select 2528,'002528','1966-099D','OV4-1T','OV4-1T',13506,'Reentered','1967-01-11 00:00:00','1966-12-04 00:00:00',90.27,276,300,32.84,'Intermediate' from dual union all
select 25280,'025280','1998-020A','TRACE','TRACE',66456,'In Earth orbit','','1998-05-02 00:00:00',97.09,597,642,97.8,'Sun-Synch' from dual union all
select 25281,'025281','1998-020B','Orion 38','Orion 38',66456,'In Earth orbit','','1998-05-02 00:00:00',96.96,597,630,97.8,'Sun-Synch' from dual union all
select 25282,'025282','1989-062F','deb Ariane V33','H10 deb',62339,'In Earth orbit','','1998-02-12 00:00:00',616.16,404,34812,7.36,'GEO Transfer' from dual union all
select 25283,'025283','1986-017MM','EO-25 EVA-2 debris','',59210,'Reentered','1998-04-30 00:00:00','1998-04-03 00:00:00',92.05,372,379,51.66,'Intermediate' from dual union all
select 25284,'025284','1993-014F','Start DS cover','EKA deb',65116,'In Earth orbit','','1998-04-06 00:00:00',101.41,681,970,75.77,'Intermediate' from dual union all
select 25285,'025285','1998-021A','Iridium 62','Iridium SV062',66457,'In Earth orbit','','1998-05-07 00:00:00',100.38,776,778,86.4,'Polar' from dual union all
select 25286,'025286','1998-021B','Iridium 63','Iridium SV063',66457,'In Earth orbit','','1998-05-07 00:00:00',100.4,775,780,86.39,'Polar' from dual union all
select 25287,'025287','1998-021C','Iridium 64','Iridium SV064',66457,'In Earth orbit','','1998-05-07 00:00:00',100.38,775,779,86.4,'Polar' from dual union all
select 25288,'025288','1998-021D','Iridium 65','Iridium SV065',66457,'In Earth orbit','','1998-05-07 00:00:00',100.38,776,778,86.4,'Polar' from dual union all
select 25289,'025289','1998-021E','Iridium 66','Iridium SV066',66457,'In Earth orbit','','1998-05-07 00:00:00',100.38,772,782,86.4,'Polar' from dual union all
select 2529,'002529','1965-034D','LES 2 AKM','Star 13A/LES adapter',11784,'In Earth orbit','','1965-11-29 00:00:00',309.77,2781,14793,32.1,'Medium' from dual union all
select 25290,'025290','1998-021F','Iridium 67','Iridium SV067',66457,'In Earth orbit','','1998-05-07 00:00:00',100.38,773,780,86.4,'Polar' from dual union all
select 25291,'025291','1998-021G','Iridium 68','Iridium SV068',66457,'Reentered','2018-06-06 00:00:00','1998-05-07 00:00:00',100.38,775,779,86.4,'Polar' from dual union all
select 25292,'025292','1986-017MP','EO-25 EVA-3 deb (Spektr SB)','',59210,'Reentered','1998-07-30 00:00:00','1998-04-08 00:00:00',92.06,373,379,51.66,'Intermediate' from dual union all
select 25293,'025293','1986-017MN','EO-25 EVA-3 deb (Spektr SB)','',59210,'Reentered','1998-06-03 00:00:00','1998-04-07 00:00:00',92.09,373,381,51.66,'Intermediate' from dual union all
select 25294,'025294','1986-017MQ','VDU','VDU No. 1',59210,'Reentered','1998-12-02 00:00:00','1998-04-11 00:00:00',92.07,373,379,51.66,'Intermediate' from dual union all
select 25295,'025295','1997-080C','Kosmos-2348 AO part','Kobal''t AO part',66160,'Reentered','1998-04-14 00:00:00','1998-04-14 00:00:00',87.66,127,191,67.07,'Intermediate' from dual union all
select 25296,'025296','1997-080D','Kosmos-2348 AO part','Kobal''t AO part',66160,'Reentered','1998-04-14 00:00:00','1998-04-14 00:00:00',88.71,154,268,67.11,'Intermediate' from dual union all
select 25297,'025297','1998-022A','Columbia (STS-90)','OV-102',66458,'Landed','1998-05-03 00:00:00','1998-04-24 00:00:00',89.83,252,281,39,'Intermediate' from dual union all
select 25298,'025298','1998-019G','Iridium part','Iridium part',66455,'Reentered','1998-04-30 00:00:00','1998-04-20 00:00:00',95.58,534,560,86.62,'Polar' from dual;
insert into SATELLITE
select 25299,'025299','1998-019H','Iridium part','Iridium part',66455,'Reentered','1998-04-24 00:00:00','1998-04-20 00:00:00',94.7,499,510,86.61,'Polar' from dual union all
select 253,'000253','61 Omicron 77','deb r/b Ablestar','',7273,'In Earth orbit','','1962-03-01 00:00:00',103.02,818,985,66.82,'Intermediate' from dual union all
select 2530,'002530','1963-014AJ','deb West Ford','RCS 0.13',9245,'In Earth orbit','','1992-01-29 00:00:00',165.9,2797,4450,87.22,'Medium' from dual union all
select 25300,'025300','1987-062C','deb Kosmos-1869 part','',60152,'Reentered','2015-07-01 00:00:00','1998-04-22 00:00:00',97.18,617,631,82.51,'Intermediate' from dual union all
select 25301,'025301','1986-017MR','EO-25 EVA-6 debris','(VDU-2 tools)',59210,'Reentered','1998-06-10 00:00:00','1998-04-23 00:00:00',92.04,372,378,51.67,'Intermediate' from dual union all
select 25302,'025302','1986-017MS','EO-25 EVA-6 debris','(VDU-2 tools)',59210,'Reentered','1998-08-28 00:00:00','1998-04-23 00:00:00',92.05,372,379,51.66,'Intermediate' from dual union all
select 25303,'025303','1986-017MT','EO-25 EVA-6 debris','(VDU-2 tools)',59210,'Reentered','1998-06-12 00:00:00','1998-04-23 00:00:00',92.03,371,378,51.66,'Intermediate' from dual union all
select 25304,'025304','1986-017MU','EO-25 EVA-6 debris','(VDU-2 tools)',59210,'Reentered','1998-05-31 00:00:00','1998-04-23 00:00:00',92.03,371,378,51.66,'Intermediate' from dual union all
select 25305,'025305','1994-029ADK','deb HAPS','',65425,'Reentered','2000-10-20 00:00:00','1998-04-24 00:00:00',92.05,374,377,51.65,'Intermediate' from dual union all
select 25306,'025306','1998-023A','Globalstar FM14','Globalstar M014',66459,'In Earth orbit','','1998-05-24 00:00:00',114.06,1394,1432,52.03,'Intermediate' from dual union all
select 25307,'025307','1998-023B','Globalstar FM6','Globalstar M006',66459,'In Earth orbit','','1998-05-24 00:00:00',114.02,1396,1426,52.02,'Intermediate' from dual union all
select 25308,'025308','1998-023C','Globalstar FM15','Globalstar M015',66459,'In Earth orbit','','1998-05-24 00:00:00',114.06,1397,1429,52.02,'Intermediate' from dual union all
select 25309,'025309','1998-023D','Globalstar FM8','Globalstar M008',66459,'In Earth orbit','','1998-05-24 00:00:00',114.06,1403,1423,52.03,'Intermediate' from dual union all
select 2531,'002531','1963-014AK','deb West Ford','RCS 0.17',9245,'In Earth orbit','','1992-02-02 00:00:00',165.52,3145,4071,87.05,'Medium' from dual union all
select 25310,'025310','1998-023E','Delta 256','Delta SSPS AJ-10-118K',66459,'Reentered','2000-09-17 00:00:00','1998-05-24 00:00:00',100.14,298,1233,52.92,'Intermediate' from dual union all
select 25311,'025311','1998-024A','Nilesat 101','Nilesat 101',66460,'In Earth orbit','','1998-06-17 00:00:00',1436.06,35779,35793,.02,'Stationary' from dual union all
select 25312,'025312','1998-024B','BSAT 1b','BSAT 1b',66460,'In Earth orbit','','1998-05-28 00:00:00',1436.11,35770,35804,.16,'Stationary' from dual union all
select 25313,'025313','1998-024C','Ariane H10-3 V108','Ariane H10-3 S/N T478',66460,'Reentered','2001-02-01 00:00:00','1998-05-31 00:00:00',644.45,196,36477,6.93,'GEO Transfer' from dual union all
select 25314,'025314','1998-024D','Spelda 30 S/N UV37','Spelda 30',66460,'Reentered','2001-02-18 00:00:00','1998-05-29 00:00:00',622.32,226,35309,6.98,'GEO Transfer' from dual union all
select 25315,'025315','1998-025A','Kosmos-2350','Oko-1 No. 7122',66461,'In Earth orbit','','1998-05-29 00:00:00',1436.21,35775,35803,2.23,'Stationary' from dual union all
select 25316,'025316','1998-025B','8S812','8S812',66461,'Reentered','1998-05-02 00:00:00','1998-05-01 00:00:00',87.75,162,165,51.62,'Intermediate' from dual union all
select 25317,'025317','1998-025C','Perekhodnik','Sredniy perekhodnik',66461,'Reentered','1998-04-30 00:00:00','1998-04-29 00:00:00',88.2,181,190,51.6,'Intermediate' from dual union all
select 25318,'025318','1998-025D','Blok DM-2 No. 96L','11S861 No. 96L',66461,'In Earth orbit','','1998-05-31 00:00:00',1439.54,35726,35982,2.25,'Drift GEO' from dual union all
select 25319,'025319','1998-026A','Iridium 69','Iridium SV069',66462,'In Earth orbit','','1998-06-02 00:00:00',100.4,775,780,86.4,'Polar' from dual union all
select 2532,'002532','1963-014AL','deb West Ford','RCS 0.14',9245,'Reentered','1987-10-17 00:00:00','2000-09-20 00:00:00',165.97,2900,4352,87.22,'Medium' from dual union all
select 25320,'025320','1998-026B','Iridium 71','Iridium SV071',66462,'In Earth orbit','','1998-06-01 00:00:00',100.38,774,779,86.39,'Polar' from dual union all
select 25321,'025321','1998-026C','CZ-2C Stage 2','CZ-2C Stage 2',66462,'Reentered','1998-06-09 00:00:00','1998-05-09 00:00:00',92.42,177,609,86.34,'Polar' from dual union all
select 25322,'025322','1998-026D','Smart Dispenser','CZ-2C SD',66462,'Reentered','1998-08-27 00:00:00','1998-06-01 00:00:00',92.14,202,556,86.26,'Polar' from dual union all
select 25323,'025323','1998-026E','CZ-2C retro cover','',66462,'Reentered','1998-05-05 00:00:00','1998-05-03 00:00:00',94.03,173,771,86.63,'Polar' from dual union all
select 25324,'025324','1998-026F','CZ-2C retro cover','',66462,'Reentered','1998-05-06 00:00:00','1998-05-02 00:00:00',94.56,171,823,86.41,'Polar' from dual union all
select 25325,'025325','1998-026G','CZ-2C retro cover','',66462,'Reentered','1998-05-10 00:00:00','1998-05-02 00:00:00',94.46,154,831,86.34,'Polar' from dual union all
select 25326,'025326','1998-026H','CZ-2C retro cover','',66462,'Reentered','1998-05-05 00:00:00','1998-05-05 00:00:00',91.3,167,510,86.07,'Polar' from dual union all
select 25327,'025327','1998-027A','Kosmos-2351','Oko',66463,'In Earth orbit','','1998-06-06 00:00:00',717.8,544,39812,62.99,'Molniya' from dual union all
select 25328,'025328','1998-027B','11S510','11S510',66463,'Reentered','1998-06-25 00:00:00','1998-05-14 00:00:00',91.44,228,463,62.85,'Intermediate' from dual union all
select 25329,'025329','1998-027C','BOZ','BOZ',66463,'Reentered','1998-06-02 00:00:00','1998-05-14 00:00:00',91.18,197,467,62.87,'Intermediate' from dual union all
select 2533,'002533','1963-014AM','deb West Ford','',9245,'Reentered','1977-01-03 00:00:00','1973-08-20 00:00:00',165.24,2065,5129,86.19,'Medium' from dual union all
select 25330,'025330','1998-027D','Blok-ML','Blok-ML No. 71616-546',66463,'In Earth orbit','','1998-06-06 00:00:00',707.69,499,39357,63.03,'Molniya' from dual union all
select 25331,'025331','1998-028A','Echostar IV','Echostar 4',66464,'In Earth orbit','','1998-06-06 00:00:00',1436.01,35775,35795,.03,'Stationary' from dual union all
select 25332,'025332','1998-028B','8S812','8S812',66464,'Reentered','1998-05-09 00:00:00','1998-05-08 00:00:00',88.25,158,218,51.61,'Intermediate' from dual union all
select 25333,'025333','1998-028C','Perekhodnik','Sredniy perekhodnik',66464,'Reentered','1998-05-09 00:00:00','1998-05-08 00:00:00',88.31,173,208,51.6,'Intermediate' from dual union all
select 25334,'025334','1998-028D','Blok DM3 No. 7L','Blok DM3 No. 7L',66464,'In Earth orbit','','1998-06-07 00:00:00',790.43,8173,35708,15.42,'Medium' from dual union all
select 25335,'025335','1994-029ADL','deb HAPS','',65425,'Reentered','1998-11-12 00:00:00','1998-05-08 00:00:00',101.04,546,1070,82.65,'Intermediate' from dual union all
select 25336,'025336','1998-029A','USA 139','ORION 4',66465,'In Earth orbit','','1998-05-11 00:00:00',1436.27,35780,35800,.1,'Stationary' from dual union all
select 25337,'025337','1998-029B','Centaur TC-18','Centaur TC-18',66465,'In Earth orbit','','1998-05-11 00:00:00',1436.27,35780,35800,.1,'Stationary' from dual union all
select 25338,'025338','1998-030A','NOAA 15','NOAA K',66466,'In Earth orbit','','1998-06-12 00:00:00',101.2,808,823,98.71,'Sun-Synch' from dual union all
select 25339,'025339','1996-053D','Blok DM1 No. 1L','Blok DM1 No. 1L',65941,'In Earth orbit','','1998-05-12 00:00:00',1467.96,36181,36635,1.45,'Drift GEO' from dual union all
select 2534,'002534','1966-100A','Lunar Orbiter 2','Lunar Orbiter B',13507,'Deep Space','1966-11-08 00:00:00','1966-11-06 00:00:00',13285.54,185,358688,28.6,'Cislunar' from dual union all
select 25340,'025340','1998-031A','Progress M-39','Progress 7K-TGM No 238',66467,'Deorbited','1998-10-29 00:00:00','1998-06-13 00:00:00',92,367,378,51.66,'Intermediate' from dual union all
select 25341,'025341','1998-031B','11S510','11S510',66467,'Reentered','1998-05-17 00:00:00','1998-05-16 00:00:00',87.99,167,183,51.62,'Intermediate' from dual union all
select 25342,'025342','1998-032A','Iridium 70','Iridium SV070',66468,'In Earth orbit','','1998-06-16 00:00:00',100.4,776,779,86.39,'Polar' from dual union all
select 25343,'025343','1998-032B','Iridium 72','Iridium SV072',66468,'Reentered','2018-05-14 00:00:00','1998-06-16 00:00:00',100.38,776,778,86.4,'Polar' from dual union all
select 25344,'025344','1998-032C','Iridium 73','Iridium SV073',66468,'In Earth orbit','','1998-06-16 00:00:00',100.4,776,779,86.4,'Polar' from dual union all
select 25345,'025345','1998-032D','Iridium 74','Iridium SV074',66468,'Deorbited','2017-06-11 00:00:00','1998-06-16 00:00:00',100.4,776,779,86.39,'Polar' from dual union all
select 25346,'025346','1998-032E','Iridium 75','Iridium SV075',66468,'In Earth orbit','','1998-06-16 00:00:00',98.63,692,695,86.55,'Polar' from dual union all
select 25347,'025347','1998-032F','Delta 257','Delta SSPS AJ-10-118K',66468,'Reentered','1999-05-15 00:00:00','1998-06-16 00:00:00',93.97,317,621,84.15,'Intermediate' from dual union all
select 25348,'025348','1998-032G','deb Iridium','',66468,'Reentered','1998-06-03 00:00:00','1998-05-25 00:00:00',96.38,547,624,86.57,'Polar' from dual union all
select 25349,'025349','1996-026B','Centaur TC-15','Centaur TC-15',65914,'In Earth orbit','','1996-05-01 00:00:00',1436.27,35780,35800,.1,'Stationary' from dual union all
select 2535,'002535','1966-100B','Agena D 6631','Agena D 6631',13507,'Deep Space','1966-11-08 00:00:00','1966-11-06 00:00:00',12798.18,128,349600,29.6,'Cislunar' from dual union all
select 25350,'025350','1998-008F','deb Delta','',66444,'Reentered','1998-06-30 00:00:00','1998-05-21 00:00:00',107.9,569,1690,52.14,'Intermediate' from dual union all
select 25351,'025351','1998-027E','BOZ ullage motor','Ullage motor',66463,'Reentered','1998-06-13 00:00:00','1998-05-21 00:00:00',91.22,226,443,62.87,'Intermediate' from dual union all
select 25352,'025352','1998-027F','BOZ ullage motor','Ullage motor',66463,'Reentered','1998-06-06 00:00:00','1998-05-21 00:00:00',90.86,216,418,62.87,'Intermediate' from dual union all
select 25353,'025353','1997-049E','Mage 1','Mage 1',66129,'In Earth orbit','','1998-05-23 00:00:00',1430.85,35349,36019,1.24,'Drift GEO' from dual union all
select 25354,'025354','1998-033A','Zhongwei 1','Chinastar 1',66469,'In Earth orbit','','1998-06-29 00:00:00',1436.06,33423,38148,.11,'Synchronous' from dual union all
select 25355,'025355','1998-033B','CZ-3B Stage 3','CZ-3B Stage 3',66469,'Reentered','1998-10-08 00:00:00','1998-09-17 00:00:00',1769.25,178,83979,22.39,'Very High' from dual union all
select 25356,'025356','1998-034A','Discovery (STS-91)','OV-103',66470,'Landed','1998-06-12 00:00:00','1998-06-04 00:00:00',91.69,342,373,51.66,'Intermediate' from dual union all
select 25357,'025357','1994-029ADM','deb HAPS','',65425,'Reentered','1998-09-23 00:00:00','1998-06-05 00:00:00',99.79,507,989,82.38,'Intermediate' from dual union all
select 25358,'025358','1998-035A','Thor III','Thor 3',66471,'In Earth orbit','','1998-07-13 00:00:00',1436.06,35753,35819,.13,'Stationary' from dual union all
select 25359,'025359','1998-035B','Delta 258','Delta SSPS AJ-10-118K',66471,'In Earth orbit','','1998-07-17 00:00:00',111.25,890,1679,26.39,'Intermediate' from dual union all
select 2536,'002536','1966-101A','8S692','8S692',13508,'Reentered','1966-11-29 00:00:00','1966-11-09 00:00:00',94.16,138,818,49.57,'Intermediate' from dual union all
select 25360,'025360','1998-035C','Star 48B','Star 48B',66471,'In Earth orbit','','1998-07-17 00:00:00',682.14,1470,37112,19.41,'GEO Transfer' from dual union all
select 25361,'025361','1994-029ADN','deb HAPS','',65425,'Reentered','2001-07-05 00:00:00','1998-06-06 00:00:00',99.49,590,879,82.09,'Intermediate' from dual union all
select 25362,'025362','1994-029ADP','deb HAPS','',65425,'Reentered','1998-12-29 00:00:00','1998-06-09 00:00:00',96.66,543,655,82.03,'Intermediate' from dual union all
select 25363,'025363','1998-036A','Kosmos-2352','Strela-3',66472,'In Earth orbit','','1998-07-15 00:00:00',118.02,1310,1874,82.59,'Intermediate' from dual union all
select 25364,'025364','1998-036B','Kosmos-2353','Strela-3',66472,'In Earth orbit','','1998-07-15 00:00:00',117.86,1300,1870,82.59,'Intermediate' from dual union all
select 25365,'025365','1998-036C','Kosmos-2354','Strela-3',66472,'In Earth orbit','','1998-07-15 00:00:00',117.95,1306,1872,82.58,'Intermediate' from dual union all
select 25366,'025366','1998-036D','Kosmos-2355','Strela-3',66472,'In Earth orbit','','1998-07-15 00:00:00',117.86,1302,1868,82.59,'Intermediate' from dual union all
select 25367,'025367','1998-036E','Kosmos-2356','Strela-3',66472,'In Earth orbit','','1998-07-15 00:00:00',117.79,1297,1867,82.58,'Intermediate' from dual union all
select 25368,'025368','1998-036F','Kosmos-2357','Strela-3',66472,'In Earth orbit','','1998-07-15 00:00:00',117.71,1293,1863,82.58,'Intermediate' from dual union all
select 25369,'025369','1998-036G','S5M','S5M',66472,'In Earth orbit','','1998-07-15 00:00:00',118.54,1340,1891,82.59,'Intermediate' from dual union all
select 2537,'002537','1966-102A','OPS 1866','CORONA J-38',13509,'Reentered Attached','1966-11-29 00:00:00','1966-11-15 00:00:00',89.26,177,299,100.08,'Retrograde' from dual union all
select 25370,'025370','1987-068BW','deb S5M','',60158,'Reentered','1998-07-04 00:00:00','1998-06-13 00:00:00',97.38,537,730,82.34,'Intermediate' from dual union all
select 25371,'025371','1998-037A','INTELSAT 805','INTELSAT 805',66473,'In Earth orbit','','1998-07-19 00:00:00',1436.14,35774,35801,.07,'Stationary' from dual union all
select 25372,'025372','1998-037B','Centaur AC-153','Centaur IIA AC-153',66473,'Reentered','2009-10-29 00:00:00','1998-08-04 00:00:00',616.01,197,35011,23.15,'GEO Transfer' from dual union all
select 25373,'025373','1998-038A','Kosmos-2358','Kobal''t No. 550',66474,'Landed','1998-10-22 00:00:00','1998-07-24 00:00:00',89.6,175,335,67.12,'Intermediate' from dual union all
select 25374,'025374','1998-038B','11S510','11S510',66474,'Reentered','1998-06-28 00:00:00','1998-06-26 00:00:00',89.15,163,301,67.12,'Intermediate' from dual union all
select 25375,'025375','1998-038C','deb Kosmos-2358','',66474,'Reentered','1998-06-25 00:00:00','1998-06-25 00:00:00',88.42,146,247,67.1,'Intermediate' from dual union all
select 25376,'025376','1998-039A','Kosmos-2359','Neman',66475,'Deorbited','1999-07-12 00:00:00','1998-07-25 00:00:00',89.92,237,304,64.91,'Intermediate' from dual union all
select 25377,'025377','1998-039B','11S510','11S510',66475,'Reentered','1998-06-29 00:00:00','1998-06-27 00:00:00',88.74,174,251,64.93,'Intermediate' from dual union all
select 25378,'025378','1994-029ADQ','deb HAPS','',65425,'Reentered','1998-09-11 00:00:00','1998-06-14 00:00:00',99.28,541,908,82.28,'Intermediate' from dual union all
select 25379,'025379','1998-040A','Molniya-3','Molniya-3 No. 61',66476,'Reentered','2011-02-02 00:00:00','1998-07-31 00:00:00',717.68,464,39886,62.81,'Molniya' from dual union all
select 2538,'002538','1966-101B','OGCh Adapter','',13508,'Reentered','1966-11-22 00:00:00','1966-11-07 00:00:00',91.49,134,561,49.41,'Intermediate' from dual union all
select 25380,'025380','1998-040B','11S510','11S510',66476,'Reentered','1998-07-18 00:00:00','1998-07-08 00:00:00',89.82,202,330,62.76,'Intermediate' from dual union all
select 25381,'025381','1998-040C','BOZ','BOZ',66476,'Reentered','1998-07-12 00:00:00','1998-07-03 00:00:00',90.56,178,425,62.79,'Intermediate' from dual union all
select 25382,'025382','1998-040D','Blok-ML','Blok-ML No. 71647-548',66476,'Reentered','2011-05-19 00:00:00','1998-07-31 00:00:00',731.88,464,40584,62.8,'Molniya' from dual union all
select 25383,'025383','1998-041A','Nozomi','Planet B',66477,'Deep Space','1998-07-04 00:00:00','1998-07-19 00:00:00',22646.75,1421,516130,25.4,'Cislunar' from dual union all
select 25384,'025384','1998-041B','M-34','M-34',66477,'Reentered','1998-07-08 00:00:00','1998-07-06 00:00:00',89.61,143,367,31.12,'Intermediate' from dual union all
select 25385,'025385','1998-041D','KM adapter','Adapter',66477,'Reentered','1998-07-05 00:00:00','1998-07-04 00:00:00',90.53,188,413,31.14,'Intermediate' from dual union all
select 25386,'025386','1998-041E','KM adapter','Adapter',66477,'Reentered','1998-07-31 00:00:00','1998-07-11 00:00:00',90.9,160,477,31.15,'Intermediate' from dual union all
select 25387,'025387','1998-040E','BOZ ullage motor','',66476,'Reentered','1998-07-16 00:00:00','1998-07-09 00:00:00',90.09,195,363,62.78,'Intermediate' from dual union all
select 25388,'025388','1998-041C','KM-V1','KM-V1',66477,'Deep Space','1998-07-04 00:00:00','1998-07-22 00:00:00',27724.01,1916,592196,23.88,'Cislunar' from dual;
insert into SATELLITE
select 25389,'025389','1998-042A','Tubsat-N','Tubsat-N',66478,'Reentered','2002-04-22 00:00:00','1998-08-06 00:00:00',96.4,400,772,78.9,'Intermediate' from dual union all
select 2539,'002539','1966-101C','deb OGCh','',13508,'Reentered','1966-11-15 00:00:00','1966-11-06 00:00:00',101.25,123,1512,49.22,'Intermediate' from dual union all
select 25390,'025390','1998-042B','Tubsat-N1','Tubsat-N1',66478,'Reentered','2000-10-21 00:00:00','1998-08-06 00:00:00',96.34,400,767,78.9,'Intermediate' from dual union all
select 25391,'025391','1998-042C','Shtil''-1 Nose Cone','Shtil''-1 instr. package',66478,'Reentered','2014-05-08 00:00:00','1998-08-06 00:00:00',96.76,401,806,78.96,'Intermediate' from dual union all
select 25392,'025392','1998-042D','Shtil''-1 Payload Capsule','',66478,'Reentered','2001-04-09 00:00:00','1998-08-06 00:00:00',96.39,400,772,78.9,'Intermediate' from dual union all
select 25393,'025393','1998-042E','Shtil''-1 Payload Plate','Adapter',66478,'Reentered','2000-03-31 00:00:00','1998-08-06 00:00:00',96.33,399,767,78.9,'Intermediate' from dual union all
select 25394,'025394','1998-043A','Resurs-O1','Resurs-O1 No. 4L',66479,'In Earth orbit','','1998-08-10 00:00:00',101.23,816,818,98.8,'Sun-Synch' from dual union all
select 25395,'025395','1998-043B','Fasat-Bravo','Fasat-Bravo',66479,'In Earth orbit','','1998-08-10 00:00:00',101.24,816,819,98.8,'Sun-Synch' from dual union all
select 25396,'025396','1998-043C','Thai-OSCAR-31','Thai Puht',66479,'In Earth orbit','','1998-08-10 00:00:00',101.23,816,818,98.8,'Sun-Synch' from dual union all
select 25397,'025397','1998-043D','Gurwin-OSCAR-32','Techsat 1B',66479,'In Earth orbit','','1998-08-10 00:00:00',101.22,815,817,98.8,'Sun-Synch' from dual union all
select 25398,'025398','1998-043E','WESTPAC','WESTPAC',66479,'In Earth orbit','','1998-08-10 00:00:00',101.24,817,817,98.8,'Sun-Synch' from dual union all
select 25399,'025399','1998-043F','SAFIR-2','SAFIR-2',66479,'In Earth orbit','','1998-08-10 00:00:00',101.21,815,817,98.8,'Sun-Synch' from dual union all
select 254,'000254','61 Omicron 78','deb r/b Ablestar','',7273,'In Earth orbit','','1962-03-09 00:00:00',102.78,793,988,66.87,'Intermediate' from dual union all
select 2540,'002540','1966-101D','deb OGCh','',13508,'Reentered','1966-11-13 00:00:00','1966-11-07 00:00:00',100.1,106,1421,49.47,'Intermediate' from dual union all
select 25400,'025400','1998-043G','11S772','11S772',66479,'In Earth orbit','','1998-08-10 00:00:00',101.09,804,817,98.79,'Sun-Synch' from dual union all
select 25401,'025401','1998-043H','Sep Motor Cover','Sep Motor Cover',66479,'In Earth orbit','','1998-08-10 00:00:00',103.49,801,1047,98.54,'Sun-Synch' from dual union all
select 25402,'025402','1998-043J','Sep Motor Cover','Sep Motor Cover',66479,'In Earth orbit','','1998-08-10 00:00:00',103.33,801,1031,99.03,'Sun-Synch' from dual union all
select 25403,'025403','1998-043K','Sep Motor Cover','Sep Motor Cover',66479,'In Earth orbit','','1998-08-10 00:00:00',104.65,817,1139,98.59,'Sun-Synch' from dual union all
select 25404,'025404','1998-044A','Xinnuo 1','xinnuo yihao',66480,'In Earth orbit','','1998-08-17 00:00:00',1436.06,35779,35793,.06,'Stationary' from dual union all
select 25405,'025405','1998-044B','CZ-3B Stage 3','CZ-3B Stage 3',66480,'In Earth orbit','','1998-08-17 00:00:00',639.25,590,35817,18.85,'GEO Transfer' from dual union all
select 25406,'025406','1998-045A','Kosmos-2360','Tselina-2',66481,'In Earth orbit','','1998-08-27 00:00:00',101.94,846,854,71.01,'Intermediate' from dual union all
select 25407,'025407','1998-045B','11S772','11S772',66481,'In Earth orbit','','1998-08-27 00:00:00',101.76,835,849,70.99,'Intermediate' from dual union all
select 25408,'025408','1967-092L','NNS O-14 insulation','',15347,'Reentered','2007-05-02 00:00:00','1998-07-28 00:00:00',106.31,1019,1093,89.24,'Polar' from dual union all
select 25409,'025409','1998-045C','Sep Motor Cover','Sep Motor Cover',66481,'In Earth orbit','','1998-08-27 00:00:00',104.89,845,1134,71.08,'Intermediate' from dual union all
select 2541,'002541','1966-101E','deb OGCh','',13508,'Reentered','1966-11-29 00:00:00','1966-11-07 00:00:00',98.73,195,1201,49.17,'Intermediate' from dual union all
select 25410,'025410','1998-045D','Sep Motor Cover','Sep Motor Cover',66481,'In Earth orbit','','1998-08-27 00:00:00',105.16,846,1158,70.98,'Intermediate' from dual union all
select 25411,'025411','1998-045E','Sep Motor Cover','Sep Motor Cover',66481,'In Earth orbit','','1998-08-27 00:00:00',105.19,845,1162,70.99,'Intermediate' from dual union all
select 25412,'025412','1998-045F','Sep Motor Cover','Sep Motor Cover',66481,'In Earth orbit','','1998-08-27 00:00:00',104.77,845,1122,71.08,'Intermediate' from dual union all
select 25413,'025413','1998-046A','Orbcomm B5','Orbcomm FM17',66482,'In Earth orbit','','1998-09-01 00:00:00',101.26,813,824,45,'Intermediate' from dual union all
select 25414,'025414','1998-046B','Orbcomm B6','Orbcomm FM18',66482,'In Earth orbit','','1998-09-01 00:00:00',101.26,813,824,45,'Intermediate' from dual union all
select 25415,'025415','1998-046C','Orbcomm B7','Orbcomm FM19',66482,'In Earth orbit','','1998-09-01 00:00:00',101.27,814,824,45,'Intermediate' from dual union all
select 25416,'025416','1998-046D','Orbcomm B8','Orbcomm FM20',66482,'In Earth orbit','','1998-09-01 00:00:00',101.28,814,825,45,'Intermediate' from dual union all
select 25417,'025417','1998-046E','Orbcomm B4','Orbcomm FM16',66482,'In Earth orbit','','1998-09-01 00:00:00',101.31,817,824,45,'Intermediate' from dual union all
select 25418,'025418','1998-046F','Orbcomm B3','Orbcomm FM15',66482,'In Earth orbit','','1998-09-01 00:00:00',101.32,819,824,45,'Intermediate' from dual union all
select 25419,'025419','1998-046G','Orbcomm B2','Orbcomm FM14',66482,'In Earth orbit','','1998-09-01 00:00:00',101.32,816,827,45,'Intermediate' from dual union all
select 2542,'002542','1966-101F','deb OGCh','',13508,'Reentered','1966-11-21 00:00:00','1966-11-05 00:00:00',98.38,179,1184,49.5,'Intermediate' from dual union all
select 25420,'025420','1998-046H','Orbcomm B1','Orbcomm FM13',66482,'In Earth orbit','','1998-09-02 00:00:00',101.32,816,827,45,'Intermediate' from dual union all
select 25421,'025421','1998-046J','Orion 38','Orion 38',66482,'Reentered','1999-01-04 00:00:00','1998-09-01 00:00:00',94.03,253,691,44.94,'Intermediate' from dual union all
select 25422,'025422','1998-046K','HAPS','HAPS',66482,'Reentered','2000-12-15 00:00:00','1998-09-02 00:00:00',96.69,382,819,42.58,'Intermediate' from dual union all
select 25423,'025423','1996-051L','deb Zenit','',65939,'Reentered','2000-04-15 00:00:00','1999-08-18 00:00:00',99.01,676,747,71,'Intermediate' from dual union all
select 25424,'025424','1997-074E','Orihime','ETS-7 Target',66154,'Reentered Attached','2015-11-13 00:00:00','1998-08-08 00:00:00',95.5,542,544,34.97,'Intermediate' from dual union all
select 25425,'025425','1994-029ADR','deb HAPS','',65425,'Reentered','1998-12-27 00:00:00','1998-08-08 00:00:00',102.26,514,1218,81.59,'Intermediate' from dual union all
select 25426,'025426','1994-029ADS','deb HAPS','',65425,'Reentered','1999-03-22 00:00:00','1998-08-08 00:00:00',98.97,569,850,82.15,'Intermediate' from dual union all
select 25427,'025427','1994-029ADT','deb HAPS','',65425,'Reentered','1999-04-23 00:00:00','1998-08-07 00:00:00',97.07,575,662,82.08,'Intermediate' from dual union all
select 25428,'025428','1994-029ADU','deb HAPS','',65425,'Reentered','2001-10-01 00:00:00','1998-08-06 00:00:00',97.62,576,714,82.04,'Intermediate' from dual union all
select 25429,'025429','1998-047A','Soyuz TM-28','Soyuz 7K-STM No. 77',66483,'Landed','1999-02-28 00:00:00','1998-09-13 00:00:00',91.84,360,370,51.66,'Intermediate' from dual union all
select 2543,'002543','1966-101G','OGCh','OGCh No. 06L',13508,'Reentered','1967-05-06 00:00:00','1966-12-03 00:00:00',94.82,141,879,49.6,'Intermediate' from dual union all
select 25430,'025430','1998-047B','11S510','11S510',66483,'Reentered','1998-08-15 00:00:00','1998-08-14 00:00:00',88.33,179,204,51.62,'Intermediate' from dual union all
select 25431,'025431','1998-048A','Iridium 3','Iridium SV078',66484,'Reentered','2018-02-08 00:00:00','1998-09-18 00:00:00',100.4,776,779,86.39,'Polar' from dual union all
select 25432,'025432','1998-048B','Iridium 76','Iridium SV076',66484,'In Earth orbit','','1998-09-18 00:00:00',100.38,775,779,86.39,'Polar' from dual union all
select 25433,'025433','1998-048C','CZ-2C Stage 2','CZ-2C Stage 2',66484,'Reentered','1998-09-21 00:00:00','1998-08-26 00:00:00',92.36,176,605,86.36,'Polar' from dual union all
select 25434,'025434','1998-048D','Smart Dispenser','CZ-2C SD',66484,'Reentered','1998-08-24 00:00:00','1998-08-21 00:00:00',92.65,194,615,86.32,'Polar' from dual union all
select 25435,'025435','1998-048E','CZ-2 retro cover','',66484,'Reentered','1998-08-24 00:00:00','1998-08-21 00:00:00',93.92,177,755,86.05,'Polar' from dual union all
select 25436,'025436','1998-048F','CZ-2 retro cover','',66484,'Reentered','1998-08-24 00:00:00','1998-08-21 00:00:00',93.9,175,756,86.64,'Polar' from dual union all
select 25437,'025437','1998-048G','CZ-2 retro cover','',66484,'Reentered','1998-08-24 00:00:00','1998-08-21 00:00:00',93.81,180,742,86.09,'Polar' from dual union all
select 25438,'025438','1998-048H','CZ-2 retro cover','',66484,'Reentered','1998-08-23 00:00:00','1998-08-21 00:00:00',93.09,174,677,86.71,'Polar' from dual union all
select 25439,'025439','1986-017MV','Soyuz TM-27 debris','',59210,'Reentered','1998-09-20 00:00:00','1998-08-25 00:00:00',91.86,361,371,51.66,'Intermediate' from dual union all
select 2544,'002544','1966-101H','deb OGCh','',13508,'Reentered','1966-11-29 00:00:00','1966-11-09 00:00:00',93.09,172,680,49.38,'Intermediate' from dual union all
select 25440,'025440','1994-029ADV','deb HAPS','',65425,'Reentered','1999-01-14 00:00:00','1998-08-22 00:00:00',97.47,555,721,81.75,'Intermediate' from dual union all
select 25441,'025441','1994-029ADW','deb HAPS','',65425,'Reentered','1998-12-31 00:00:00','1998-08-19 00:00:00',96.47,546,633,81.78,'Intermediate' from dual union all
select 25442,'025442','1994-029ADX','deb HAPS','',65425,'In Earth orbit','','1998-08-25 00:00:00',102.94,594,1201,81.8,'Intermediate' from dual union all
select 25443,'025443','1994-029ADY','deb HAPS','',65425,'Reentered','1998-12-29 00:00:00','1998-08-25 00:00:00',100.65,557,1022,81.78,'Intermediate' from dual union all
select 25444,'025444','1987-068BX','deb HAPS','',60158,'In Earth orbit','','1998-08-24 00:00:00',103.88,929,955,82.55,'Intermediate' from dual union all
select 25445,'025445','1994-029ADZ','deb HAPS','',65425,'In Earth orbit','','1998-08-23 00:00:00',99.83,605,896,82,'Intermediate' from dual union all
select 25446,'025446','1994-029AEA','deb HAPS','',65425,'Reentered','1999-09-18 00:00:00','1998-08-25 00:00:00',97.3,534,724,82.13,'Intermediate' from dual union all
select 25447,'025447','1994-029AEB','deb HAPS','',65425,'Reentered','1999-09-15 00:00:00','1998-08-25 00:00:00',100.57,563,1008,82.02,'Intermediate' from dual union all
select 25448,'025448','1994-029AEC','deb HAPS','',65425,'Reentered','2014-05-17 00:00:00','1998-08-25 00:00:00',123.25,611,3041,82.33,'Medium' from dual union all
select 25449,'025449','1994-029AED','deb HAPS','',65425,'In Earth orbit','','1998-08-25 00:00:00',121.45,606,2886,82.19,'Medium' from dual union all
select 2545,'002545','1966-101J','deb OGCh','',13508,'Reentered','1966-11-17 00:00:00','1966-11-11 00:00:00',93.72,154,759,49.53,'Intermediate' from dual union all
select 25450,'025450','1987-068BY','deb S5M','',60158,'Reentered','2013-06-05 00:00:00','1998-08-25 00:00:00',102.63,802,965,82.28,'Intermediate' from dual union all
select 25451,'025451','1987-068BZ','deb S5M','',60158,'Reentered','2001-02-02 00:00:00','1998-08-25 00:00:00',102.14,788,932,82.56,'Intermediate' from dual union all
select 25452,'025452','1987-068CA','deb S5M','',60158,'Reentered','1999-03-17 00:00:00','1998-08-22 00:00:00',102.86,826,961,82.5,'Intermediate' from dual union all
select 25453,'025453','1987-068CB','deb S5M','',60158,'Reentered','1999-08-14 00:00:00','1998-08-18 00:00:00',104.67,792,1165,82.51,'Intermediate' from dual union all
select 25454,'025454','1987-068CC','deb S5M','',60158,'In Earth orbit','','1998-08-25 00:00:00',105.73,937,1121,82.19,'Intermediate' from dual union all
select 25455,'025455','1987-068CD','deb S5M','',60158,'Reentered','1999-01-12 00:00:00','1998-08-25 00:00:00',102.5,814,940,82.4,'Intermediate' from dual union all
select 25456,'025456','1987-068CE','deb S5M','',60158,'Reentered','1999-05-27 00:00:00','1998-08-26 00:00:00',104.02,913,984,82.53,'Intermediate' from dual union all
select 25457,'025457','1987-068CF','deb S5M','',60158,'Reentered','1998-11-07 00:00:00','1998-08-24 00:00:00',100.81,664,929,82.43,'Intermediate' from dual union all
select 25458,'025458','1987-068CG','deb S5M','',60158,'Reentered','1998-12-03 00:00:00','1998-08-25 00:00:00',100.5,765,800,82.5,'Intermediate' from dual union all
select 25459,'025459','1987-068CH','deb S5M','',60158,'Reentered','1999-06-13 00:00:00','1998-08-23 00:00:00',103.67,819,1045,82.54,'Intermediate' from dual union all
select 2546,'002546','1966-101K','deb OGCh','',13508,'Reentered','1966-11-08 00:00:00','1966-11-07 00:00:00',90.94,136,505,49.88,'Intermediate' from dual union all
select 25460,'025460','1998-049A','ST-1','Singapore-Taiwan 1',66485,'In Earth orbit','','1998-09-24 00:00:00',1436.06,35755,35817,.02,'Stationary' from dual union all
select 25461,'025461','1998-049B','Ariane H10-3 V109','Ariane H10-3 S/N T481',66485,'Reentered','1998-09-20 00:00:00','1998-09-07 00:00:00',658.03,267,37098,3.98,'GEO Transfer' from dual union all
select 25462,'025462','1998-050A','Astra 2A','Astra 2A',66486,'In Earth orbit','','1998-09-29 00:00:00',1434.61,35732,35783,.03,'Drift GEO' from dual union all
select 25463,'025463','1998-050B','8S812','8S812',66486,'Reentered','1998-09-01 00:00:00','1998-08-31 00:00:00',87.77,145,184,51.6,'Intermediate' from dual union all
select 25464,'025464','1998-050C','Perekhodnik','Sredniy perekhodnik',66486,'Reentered','1998-08-31 00:00:00','1998-08-30 00:00:00',87.93,161,182,51.59,'Intermediate' from dual union all
select 25465,'025465','1998-050D','Blok DM3 No. 9L','Blok DM3 No. 9L',66486,'In Earth orbit','','1998-10-01 00:00:00',787.61,7796,35950,15.61,'Medium' from dual union all
select 25466,'025466','1998-050E','deb Proton','',66486,'Reentered','1998-09-05 00:00:00','1998-09-03 00:00:00',91.08,203,453,51.57,'Intermediate' from dual union all
select 25467,'025467','1998-051A','Iridium 82','Iridium SV082',66487,'In Earth orbit','','1998-10-08 00:00:00',98.89,704,708,86.52,'Polar' from dual union all
select 25468,'025468','1998-051B','Iridium 81','Iridium SV081',66487,'In Earth orbit','','1998-10-08 00:00:00',100.38,776,778,86.4,'Polar' from dual union all
select 25469,'025469','1998-051C','Iridium 80','Iridium SV080',66487,'In Earth orbit','','1998-10-08 00:00:00',100.38,776,778,86.4,'Polar' from dual union all
select 2547,'002547','1966-101L','deb OGCh','',13508,'Reentered','1966-11-12 00:00:00','1966-11-08 00:00:00',98.91,161,1252,49.68,'Intermediate' from dual union all
select 25470,'025470','1998-051D','Iridium 79','Iridium SV079',66487,'Deorbited','2000-11-29 00:00:00','1998-10-08 00:00:00',95.09,497,549,86.02,'Polar' from dual union all
select 25471,'025471','1998-051E','Iridium 77','Iridium SV077',66487,'Reentered','2017-09-22 00:00:00','1998-10-08 00:00:00',98.89,704,708,86.52,'Polar' from dual union all
select 25472,'025472','1998-051F','Delta 260','Delta SSPS AJ-10-118K',66487,'Reentered','1999-03-15 00:00:00','1998-10-08 00:00:00',92.71,311,504,83.43,'Intermediate' from dual union all
select 25473,'025473','1998-052A','PAS 7','PAS 7',66488,'In Earth orbit','','1998-10-17 00:00:00',1436.09,35777,35796,.04,'Stationary' from dual union all
select 25474,'025474','1998-052B','Ariane H10-3 V110','Ariane H10-3 S/N T479',66488,'Reentered','1998-10-26 00:00:00','1998-10-02 00:00:00',688.27,89,38800,6.93,'GEO Transfer' from dual union all
select 25475,'025475','1998-053A','Orbcomm C1','Orbcomm FM21',66489,'In Earth orbit','','1998-10-25 00:00:00',101.21,810,822,45.01,'Intermediate' from dual union all
select 25476,'025476','1998-053B','Orbcomm C2','Orbcomm FM22',66489,'In Earth orbit','','1998-10-25 00:00:00',101.26,814,822,45.02,'Intermediate' from dual union all
select 25477,'025477','1998-053C','Orbcomm C3','Orbcomm FM23',66489,'In Earth orbit','','1998-10-25 00:00:00',101.26,813,824,45.01,'Intermediate' from dual union all
select 25478,'025478','1998-053D','Orbcomm C4','Orbcomm FM24',66489,'In Earth orbit','','1998-10-25 00:00:00',101.21,810,822,45.01,'Intermediate' from dual;
insert into SATELLITE
select 25479,'025479','1998-053E','Orbcomm C5','Orbcomm FM25',66489,'In Earth orbit','','1998-10-25 00:00:00',101.14,806,820,45.02,'Intermediate' from dual union all
select 2548,'002548','1966-101M','deb OGCh','',13508,'Reentered','1966-11-29 00:00:00','1966-11-09 00:00:00',104.35,195,1733,49.42,'Intermediate' from dual union all
select 25480,'025480','1998-053F','Orbcomm C6','Orbcomm FM26',66489,'In Earth orbit','','1998-10-25 00:00:00',101.15,807,819,45.02,'Intermediate' from dual union all
select 25481,'025481','1998-053G','Orbcomm C7','Orbcomm FM27',66489,'In Earth orbit','','1998-10-25 00:00:00',101.15,807,819,45.02,'Intermediate' from dual union all
select 25482,'025482','1998-053H','Orbcomm C8','Orbcomm FM28',66489,'In Earth orbit','','1998-10-26 00:00:00',101.21,811,821,45.02,'Intermediate' from dual union all
select 25483,'025483','1998-053J','Orion 38','Orion 38',66489,'Reentered','1998-11-07 00:00:00','1998-09-30 00:00:00',91.33,251,428,44.97,'Intermediate' from dual union all
select 25484,'025484','1998-053K','HAPS','HAPS',66489,'In Earth orbit','','1998-10-28 00:00:00',100.14,717,814,44.54,'Intermediate' from dual union all
select 25485,'025485','1998-054A','Molniya-1T','Molniya-1T No. 99',66490,'In Earth orbit','','1998-10-28 00:00:00',717.66,449,39900,62.88,'Molniya' from dual union all
select 25486,'025486','1998-054B','11S510','11S510',66490,'Reentered','1998-10-14 00:00:00','1998-10-05 00:00:00',89.95,201,343,62.82,'Intermediate' from dual union all
select 25487,'025487','1998-054C','BOZ','BOZ',66490,'Reentered','1998-10-10 00:00:00','1998-09-30 00:00:00',90.66,186,427,62.83,'Intermediate' from dual union all
select 25488,'025488','1998-054D','Blok-ML','Blok-ML No. 71647-493',66490,'In Earth orbit','','1998-10-28 00:00:00',733.22,453,40661,62.87,'Molniya' from dual union all
select 25489,'025489','1998-055A','STEX','STEX',66491,'In Earth orbit','','1998-11-02 00:00:00',98.89,700,712,84.99,'Intermediate' from dual union all
select 2549,'002549','1966-101N','deb OGCh','',13508,'Reentered','1966-11-08 00:00:00','1966-11-07 00:00:00',90.81,124,504,49.6,'Intermediate' from dual union all
select 25490,'025490','1998-055B','Orion 38','Orion 38',66491,'In Earth orbit','','1998-11-02 00:00:00',98.43,671,697,85.08,'Polar' from dual union all
select 25491,'025491','1998-056A','Eutelsat W2','Eutelsat W2',66492,'In Earth orbit','','1998-11-05 00:00:00',1433.58,35723,35752,.01,'Drift GEO' from dual union all
select 25492,'025492','1998-056B','Sirius 3','Sirius 3',66492,'In Earth orbit','','1998-11-06 00:00:00',1436.06,35763,35809,.03,'Stationary' from dual union all
select 25493,'025493','1998-056C','Ariane H10-3 V111','Ariane H10-3 S/N T482',66492,'Reentered','2012-07-08 00:00:00','1998-11-04 00:00:00',649.38,210,36714,6.72,'GEO Transfer' from dual union all
select 25494,'025494','1998-056D','Spelda 30 S/N UV40','Spelda 30',66492,'Reentered','1999-11-18 00:00:00','1998-11-07 00:00:00',620.39,227,35208,6.96,'GEO Transfer' from dual union all
select 25495,'025495','1998-057A','Hot Bird 5','Eutelsat HB5',66493,'In Earth orbit','','1998-11-12 00:00:00',1436.04,35763,35808,.04,'Stationary' from dual union all
select 25496,'025496','1998-057B','Centaur AC-134','Centaur IIA AC-134',66493,'Reentered','1999-04-14 00:00:00','1998-11-09 00:00:00',607.63,178,34594,24.92,'GEO Transfer' from dual union all
select 25497,'025497','1985-056F','deb Ariane V14','',58279,'Reentered','2000-09-18 00:00:00','1998-10-15 00:00:00',173.2,289,7540,7.72,'Medium' from dual union all
select 25498,'025498','1985-056G','deb Ariane V14','',58279,'Reentered','2002-03-11 00:00:00','1998-10-15 00:00:00',182.23,325,8213,7.15,'Medium' from dual union all
select 25499,'025499','1998-025E','SOZ','SOZ',66461,'Reentered','1998-12-19 00:00:00','1998-10-16 00:00:00',593.36,127,33898,47.33,'Highly Ellip' from dual union all
select 255,'000255','62 Zeta 1','OSO 1','NASA S-16',8207,'Reentered','1981-10-08 00:00:00','1962-04-19 00:00:00',96.13,546,601,32.83,'Intermediate' from dual union all
select 2550,'002550','1966-101P','deb OGCh','',13508,'Reentered','1966-11-24 00:00:00','1966-11-11 00:00:00',92.73,146,670,49.51,'Intermediate' from dual union all
select 25500,'025500','1998-025F','SOZ','SOZ',66461,'Reentered','1999-10-23 00:00:00','1998-10-15 00:00:00',630.59,179,35783,47.4,'GEO Transfer' from dual union all
select 25501,'025501','1998-058A','UHF F/O F9','UFO F9',66494,'In Earth orbit','','1998-11-21 00:00:00',1436.09,34812,36761,6.02,'Inclined GEO' from dual union all
select 25502,'025502','1998-058B','Centaur AC-130','Centaur IIA AC-130',66494,'In Earth orbit','','1998-11-23 00:00:00',447.13,280,25703,26.96,'Highly Ellip' from dual union all
select 25503,'025503','1998-059A','Maqsat-3','Maqsat-3',66495,'In Earth orbit','','1998-11-23 00:00:00',641.22,1005,35503,7.12,'GEO Transfer' from dual union all
select 25504,'025504','1998-060A','SCD-2','SCD-2',66496,'In Earth orbit','','1998-11-23 00:00:00',99.92,742,768,25,'Equatorial' from dual union all
select 25505,'025505','1991-039D','deb Okean','',64208,'Reentered','2014-06-30 00:00:00','1998-10-22 00:00:00',97.47,622,654,82.53,'Intermediate' from dual union all
select 25506,'025506','1998-060B','Orion 38','Orion 38',66496,'In Earth orbit','','1998-11-25 00:00:00',99.83,733,767,25,'Equatorial' from dual union all
select 25507,'025507','1998-060C','deb Pegasus','',66496,'Reentered','1999-10-29 00:00:00','1998-11-22 00:00:00',99.79,732,765,24.98,'Equatorial' from dual union all
select 25508,'025508','1998-061A','Deep Space 1','Deep Space 1',66497,'Deep Space','1998-10-25 00:00:00','1998-10-24 00:00:00',7537.97,184,267640,28.5,'Earth Escape' from dual union all
select 25509,'025509','1998-061B','SEDSAT-OSCAR-33','SEDSAT',66497,'In Earth orbit','','1998-11-23 00:00:00',101.12,547,1077,31.44,'Intermediate' from dual union all
select 2551,'002551','1966-101Q','deb OGCh','',13508,'Reentered','1966-11-08 00:00:00','1966-11-03 00:00:00',93.03,166,680,49.42,'Intermediate' from dual union all
select 25510,'025510','1998-061C','Delta 261','Delta SSPS AJ-10-118K',66497,'In Earth orbit','','1998-11-23 00:00:00',101.07,542,1077,31.44,'Intermediate' from dual union all
select 25511,'025511','1998-061D','Star 37FM','Star 37FM',66497,'Deep Space','1998-10-25 00:00:00','1998-10-24 00:00:00',7537.97,184,267640,28.5,'Earth Escape' from dual union all
select 25512,'025512','1998-062A','Progress M-40','Progress 7K-TGM No 239',66498,'Deorbited','1999-02-05 00:00:00','1998-11-24 00:00:00',91.63,349,360,51.66,'Intermediate' from dual union all
select 25513,'025513','1998-062B','11S510','11S510',66498,'Reentered','1998-10-26 00:00:00','1998-10-25 00:00:00',88.4,179,212,51.65,'Intermediate' from dual union all
select 25514,'025514','1990-002D','deb PKM','',63244,'Reentered','1998-08-12 00:00:00','1998-05-26 00:00:00',97.95,302,1020,26.56,'Intermediate' from dual union all
select 25515,'025515','1998-063A','Afristar','Afristar',66499,'In Earth orbit','','1998-11-27 00:00:00',1436.01,35768,35802,.03,'Stationary' from dual union all
select 25516,'025516','1998-063B','GE 5','GE 5',66499,'In Earth orbit','','1998-11-27 00:00:00',1431.39,35495,35894,.18,'Drift GEO' from dual union all
select 25517,'025517','1998-063C','Ariane H10-3 V113','Ariane H10-3 S/N T482',66499,'Reentered','1998-10-31 00:00:00','1998-10-29 00:00:00',630.24,199,35745,6.5,'GEO Transfer' from dual union all
select 25518,'025518','1998-063D','Spelda 40 S/N UV38','Spelda 40',66499,'Reentered','1999-09-06 00:00:00','1998-11-28 00:00:00',610.3,201,34710,6.45,'GEO Transfer' from dual union all
select 25519,'025519','1998-064A','Discovery (STS-95)','OV-103',66500,'Landed','1998-11-07 00:00:00','1998-10-31 00:00:00',95.75,550,560,28.47,'Intermediate' from dual union all
select 2552,'002552','1966-101R','deb OGCh','',13508,'Reentered','1966-11-06 00:00:00','1966-11-02 00:00:00',98.83,168,1238,49.36,'Intermediate' from dual union all
select 25520,'025520','1998-064B','PANSAT-Oscar-34','PANSAT',66500,'In Earth orbit','','1998-11-28 00:00:00',95.72,541,565,28.39,'Intermediate' from dual union all
select 25521,'025521','1998-064C','Spartan 201','Spartan 201',66500,'Landed Attached','1998-11-07 00:00:00','1998-11-02 00:00:00',95.74,549,560,28.47,'Intermediate' from dual union all
select 25522,'025522','1998-065A','PAS 8','PAS 8',66501,'In Earth orbit','','1998-12-29 00:00:00',1436.11,35774,35800,.07,'Stationary' from dual union all
select 25523,'025523','1998-065B','8S812','8S812',66501,'Reentered','1998-11-06 00:00:00','1998-11-05 00:00:00',88.05,169,187,51.6,'Intermediate' from dual union all
select 25524,'025524','1998-065C','Perekhodnik','Sredniy perekhodnik',66501,'Reentered','1998-11-04 00:00:00','1998-11-04 00:00:00',88.01,175,177,51.59,'Intermediate' from dual union all
select 25525,'025525','1998-065D','Blok DM3 No. 10L','Blok DM3 No. 10L',66501,'In Earth orbit','','1998-12-10 00:00:00',765.28,6795,35878,17.36,'Highly Ellip' from dual union all
select 25526,'025526','1966-040D','Nimbus 2 debris','',13447,'In Earth orbit','','1998-11-04 00:00:00',107.65,1082,1154,100.58,'Sun-Synch' from dual union all
select 25527,'025527','1998-066A','Iridium 2','Iridium SV087',66502,'In Earth orbit','','1998-12-08 00:00:00',95.14,512,539,85.4,'Polar' from dual union all
select 25528,'025528','1998-066B','Iridium 86','Iridium SV086',66502,'In Earth orbit','','1998-12-08 00:00:00',98.85,701,707,86.52,'Polar' from dual union all
select 25529,'025529','1998-066C','Iridium 85','Iridium SV085',66502,'Reentered','2000-12-30 00:00:00','1998-12-06 00:00:00',95.12,514,535,86.01,'Polar' from dual union all
select 2553,'002553','1966-101S','deb OGCh','',13508,'Reentered','1966-11-08 00:00:00','1966-11-03 00:00:00',98.59,138,1245,49.6,'Intermediate' from dual union all
select 25530,'025530','1998-066D','Iridium 84','Iridium SV084',66502,'In Earth orbit','','1998-12-07 00:00:00',98.83,701,705,86.51,'Polar' from dual union all
select 25531,'025531','1998-066E','Iridium 83','Iridium SV083',66502,'In Earth orbit','','1998-12-06 00:00:00',100.4,776,779,86.39,'Polar' from dual union all
select 25532,'025532','1998-066F','Delta 262','Delta SSPS AJ-10-118K',66502,'Reentered','1999-01-07 00:00:00','1998-12-06 00:00:00',91.25,236,436,82.46,'Intermediate' from dual union all
select 25533,'025533','1998-062C','Spoutnik-41','RS-18',66498,'Reentered','1999-01-11 00:00:00','1998-11-24 00:00:00',91.44,339,351,51.66,'Intermediate' from dual union all
select 25534,'025534','1986-017MW','EO-26 EVA-2 debris','',59210,'Reentered','1998-12-04 00:00:00','1998-11-16 00:00:00',91.42,340,349,51.66,'Intermediate' from dual union all
select 25535,'025535','1986-017MX','EO-26 EVA-2 debris','',59210,'Reentered','1998-11-25 00:00:00','1998-11-16 00:00:00',91.19,328,337,51.66,'Intermediate' from dual union all
select 25536,'025536','1986-017MY','EO-26 EVA-2 debris','',59210,'Reentered','1998-11-17 00:00:00','1998-11-16 00:00:00',90.42,291,299,51.66,'Intermediate' from dual union all
select 25537,'025537','1986-017MZ','EO-26 EVA-2 debris','',59210,'Reentered','1998-11-23 00:00:00','1998-11-16 00:00:00',91.01,321,328,51.66,'Intermediate' from dual union all
select 25538,'025538','1988-109E','deb Ariane V27','',61358,'In Earth orbit','','1998-11-09 00:00:00',655.88,594,36662,6.88,'GEO Transfer' from dual union all
select 25539,'025539','1988-109F','deb Ariane V27','',61358,'In Earth orbit','','1998-11-18 00:00:00',623.25,434,35149,6.92,'GEO Transfer' from dual union all
select 2554,'002554','1966-101T','deb OGCh','',13508,'Reentered','1966-11-07 00:00:00','1966-11-04 00:00:00',95.01,129,910,49.65,'Intermediate' from dual union all
select 25540,'025540','1988-109G','deb Ariane V27','',61358,'In Earth orbit','','1998-11-18 00:00:00',637.07,385,35910,6.63,'GEO Transfer' from dual union all
select 25541,'025541','1988-109H','deb Ariane V27','',61358,'In Earth orbit','','1998-11-16 00:00:00',636.06,602,35641,6.94,'GEO Transfer' from dual union all
select 25542,'025542','1988-109J','deb Ariane V27','',61358,'In Earth orbit','','1998-11-15 00:00:00',635.78,460,35769,6.88,'GEO Transfer' from dual union all
select 25543,'025543','1988-109K','deb Ariane V27','',61358,'In Earth orbit','','1998-11-17 00:00:00',636.43,461,35801,6.85,'GEO Transfer' from dual union all
select 25544,'025544','1998-067A','Zarya','ISS FGB 77KM No. 17501',66503,'In Earth orbit','','1998-12-20 00:00:00',92.47,393,399,51.59,'Intermediate' from dual union all
select 25545,'025545','1998-067B','8S812','8S812',66503,'Reentered','1998-11-27 00:00:00','1998-11-22 00:00:00',89.29,176,303,51.61,'Intermediate' from dual union all
select 25546,'025546','1998-068A','BONUM-1','BONUM-1',66504,'In Earth orbit','','1998-12-23 00:00:00',1436.01,35783,35787,.07,'Stationary' from dual union all
select 25547,'025547','1998-068B','Delta 263','Delta SSPS AJ-10-118K',66504,'Reentered','2000-10-07 00:00:00','1998-12-22 00:00:00',103.01,274,1528,25.76,'Intermediate' from dual union all
select 25548,'025548','1998-068C','Star 48B','Star 48B',66504,'In Earth orbit','','1998-12-25 00:00:00',669.25,1301,36632,19.54,'GEO Transfer' from dual union all
select 25549,'025549','1998-069A','Endeavour (STS-88)','OV-105',66505,'Landed','1998-12-16 00:00:00','1998-12-06 00:00:00',90.47,211,385,51.6,'Intermediate' from dual union all
select 2555,'002555','1966-101U','deb OGCh','',13508,'Reentered','1966-11-20 00:00:00','1966-11-12 00:00:00',93.2,131,731,49.54,'Intermediate' from dual union all
select 25550,'025550','1998-069B','SAC-A','SAC-A',66505,'Reentered','1999-10-25 00:00:00','1999-01-03 00:00:00',92.29,378,395,51.58,'Intermediate' from dual union all
select 25551,'025551','1998-069C','Mightysat 1','Mightysat 1',66505,'Reentered','1999-11-21 00:00:00','1999-01-04 00:00:00',92.26,380,391,51.58,'Intermediate' from dual union all
select 25552,'025552','1964-049G','deb Kosmos-41','',10335,'Reentered','2001-01-24 00:00:00','1998-12-04 00:00:00',704.89,966,38751,67.65,'GEO Transfer' from dual union all
select 25553,'025553','1964-049H','deb Kosmos-41','',10335,'Reentered','2001-01-02 00:00:00','1998-12-01 00:00:00',701.4,962,38582,67.67,'GEO Transfer' from dual union all
select 25554,'025554','1964-049J','deb Kosmos-41','',10335,'Reentered','2001-04-01 00:00:00','1998-12-04 00:00:00',701.5,1032,38517,67.87,'GEO Transfer' from dual union all
select 25555,'025555','1964-049K','deb Kosmos-41','',10335,'Reentered','2001-08-31 00:00:00','1998-12-03 00:00:00',704.68,1143,38564,68.15,'GEO Transfer' from dual union all
select 25556,'025556','1964-049L','deb Kosmos-41','',10335,'Reentered','2001-12-11 00:00:00','1998-12-01 00:00:00',701.48,1282,38266,68.33,'GEO Transfer' from dual union all
select 25557,'025557','1964-049M','deb Kosmos-41','',10335,'Reentered','2001-09-21 00:00:00','1998-12-04 00:00:00',702.69,1244,38364,68.42,'GEO Transfer' from dual union all
select 25558,'025558','1998-070A','Satmex 5','Satmex 5',66506,'In Earth orbit','','1999-01-06 00:00:00',1436.14,35770,35805,.06,'Stationary' from dual union all
select 25559,'025559','1998-070B','Ariane H10-3 V114','Ariane H10-3 S/N T485',66506,'Reentered','2004-07-13 00:00:00','1999-01-06 00:00:00',370,196,21193,7.25,'Highly Ellip' from dual union all
select 2556,'002556','1966-101V','deb OGCh','',13508,'Reentered','1966-11-08 00:00:00','1966-11-03 00:00:00',93.68,130,779,49.62,'Intermediate' from dual union all
select 25560,'025560','1998-071A','SWAS','SWAS',66507,'In Earth orbit','','1999-01-07 00:00:00',97.6,638,649,69.9,'Intermediate' from dual union all
select 25561,'025561','1998-071B','Orion 38','Orion 38',66507,'In Earth orbit','','1999-01-07 00:00:00',97.57,638,647,69.92,'Intermediate' from dual union all
select 25562,'025562','1998-071C','deb Pegasus','',66507,'Reentered','1999-03-18 00:00:00','1999-01-08 00:00:00',96.83,596,618,69.93,'Intermediate' from dual union all
select 25563,'025563','1990-111D','deb Kosmos-2112','Strela-2M debris 2 (high',63353,'In Earth orbit','','1998-12-07 00:00:00',100.9,785,818,74.05,'Intermediate' from dual union all
select 25564,'025564','1998-069D','EVA debris','Wire carrier',66505,'Reentered','1999-04-08 00:00:00','1999-01-03 00:00:00',92.04,371,379,51.6,'Intermediate' from dual union all
select 25565,'025565','1998-069E','EVA debris','WIF Socket',66505,'Reentered','1999-03-24 00:00:00','1999-01-03 00:00:00',91.99,368,377,51.6,'Intermediate' from dual union all
select 25566,'025566','1994-029AEE','deb HAPS','',65425,'Reentered','2017-04-11 00:00:00','1998-12-10 00:00:00',122.63,613,2984,82.33,'Medium' from dual union all
select 25567,'025567','1998-072A','Nadezhda','Nadezhda 17F118',66508,'In Earth orbit','','1999-01-11 00:00:00',105,976,1013,82.95,'Intermediate' from dual union all
select 25568,'025568','1998-072B','Astrid-2','Astrid 2',66508,'In Earth orbit','','1999-01-09 00:00:00',105.02,978,1013,82.95,'Intermediate' from dual;
insert into SATELLITE
select 25569,'025569','1998-072C','S3M','S3M',66508,'In Earth orbit','','1999-01-09 00:00:00',104.87,972,1004,82.94,'Intermediate' from dual union all
select 2557,'002557','1966-101W','deb OGCh','',13508,'Reentered','1966-11-08 00:00:00','1966-11-03 00:00:00',104.46,187,1752,49.4,'Intermediate' from dual union all
select 25570,'025570','1998-067C','EVA debris','Trunnion pin cover',66503,'Reentered','1998-12-14 00:00:00','1998-12-10 00:00:00',92.26,379,392,51.6,'Intermediate' from dual union all
select 25571,'025571','1998-073A','Mars Climate Orbiter','MCO',66509,'Deep Space','1998-12-12 00:00:00','1998-12-11 00:00:00',1158.89,187,86038,28.31,'Earth Escape' from dual union all
select 25572,'025572','1998-073B','Delta 264','Delta SSPS AJ-10-118K',66509,'Reentered','1998-12-30 00:00:00','1998-12-18 00:00:00',93.73,158,756,23.9,'Equatorial' from dual union all
select 25573,'025573','1998-073C','Star 48B','Star 48B',66509,'Deep Space','1998-12-12 00:00:00','1998-12-11 00:00:00',1158.89,187,86038,28.31,'Earth Escape' from dual union all
select 25574,'025574','1988-063H','deb Ariane V24','',61312,'Reentered','2002-10-30 00:00:00','1998-12-07 00:00:00',384.51,256,22020,7.46,'Highly Ellip' from dual union all
select 25575,'025575','1998-069F','Unity','ISS Node 1',66505,'In Earth orbit','','1998-12-10 00:00:00',92.42,387,400,51.59,'Intermediate' from dual union all
select 25576,'025576','1965-109F','NNS O-6 insulation','',11859,'Reentered','2001-01-25 00:00:00','1998-12-17 00:00:00',104.02,856,1041,89.07,'Polar' from dual union all
select 25577,'025577','1998-074A','Iridium 20A','Iridium SV088',66510,'In Earth orbit','','1999-01-18 00:00:00',98.77,687,713,86.55,'Polar' from dual union all
select 25578,'025578','1998-074B','Iridium 11A','Iridium SV089',66510,'In Earth orbit','','1999-01-18 00:00:00',98.74,685,712,86.52,'Polar' from dual union all
select 25579,'025579','1998-074C','CZ-2C Stage 2','CZ-2C Stage 2',66510,'Reentered','1999-01-20 00:00:00','1998-12-26 00:00:00',92.28,174,599,86.36,'Polar' from dual union all
select 2558,'002558','1966-101X','deb OGCh','',13508,'Reentered','1966-11-07 00:00:00','1966-11-03 00:00:00',92.84,137,691,49.5,'Intermediate' from dual union all
select 25580,'025580','1998-074D','Smart Dispenser','CZ-2C SD',66510,'Reentered','1999-04-02 00:00:00','1999-01-18 00:00:00',92.35,208,572,86.33,'Polar' from dual union all
select 25581,'025581','1998-074E','CZ-2 retro cover','',66510,'Reentered','1998-12-23 00:00:00','1998-12-21 00:00:00',92.87,171,659,86.05,'Polar' from dual union all
select 25582,'025582','1998-074F','CZ-2 retro cover','',66510,'Reentered','1998-12-22 00:00:00','1998-12-21 00:00:00',92.59,169,634,86.71,'Polar' from dual union all
select 25583,'025583','1998-074G','CZ-2 retro cover','',66510,'Reentered','1998-12-23 00:00:00','1998-12-21 00:00:00',93.07,170,679,86.6,'Polar' from dual union all
select 25584,'025584','1998-074H','CZ-2 retro cover','',66510,'Reentered','1998-12-24 00:00:00','1998-12-21 00:00:00',93.67,175,734,86.12,'Polar' from dual union all
select 25585,'025585','1998-075A','PAS 6B','PAS 6B',66511,'In Earth orbit','','1999-01-22 00:00:00',1436.11,35779,35795,.03,'Stationary' from dual union all
select 25586,'025586','1998-075B','Ariane H10-3 V115','Ariane H10-3 S/N T480',66511,'In Earth orbit','','1999-01-21 00:00:00',626.31,225,35516,7.24,'GEO Transfer' from dual union all
select 25587,'025587','1988-077S','deb Transtage D-3','',61326,'In Earth orbit','','1998-12-31 00:00:00',719.67,391,40057,26.7,'GEO Transfer' from dual union all
select 25588,'025588','1983-078E','deb OPS 7304','',55835,'In Earth orbit','','1998-12-31 00:00:00',711.42,831,39210,63.5,'Molniya' from dual union all
select 25589,'025589','1991-017D','deb Titan','',64186,'Reentered','1999-01-01 00:00:00','1998-12-31 00:00:00',93.22,380,484,68,'Intermediate' from dual union all
select 2559,'002559','1966-101Y','deb OGCh','',13508,'Reentered','1966-11-09 00:00:00','1966-11-03 00:00:00',102.12,104,1614,49.61,'Intermediate' from dual union all
select 25590,'025590','1998-076A','Kosmos-2361','Parus',66512,'In Earth orbit','','1999-01-25 00:00:00',104.92,969,1013,82.93,'Intermediate' from dual union all
select 25591,'025591','1991-076L','deb USA 74-77','[probably spurious]',64245,'In Earth orbit','','1998-12-31 00:00:00',107.04,900,1280,63.4,'Intermediate' from dual union all
select 25592,'025592','1998-076B','S3M','S3M',66512,'In Earth orbit','','1999-01-25 00:00:00',104.85,968,1006,82.93,'Intermediate' from dual union all
select 25593,'025593','1998-077A','Kosmos-2362','Uragan No. 86L',66513,'In Earth orbit','','1999-01-29 00:00:00',684.63,19125,19582,64.82,'Medium' from dual union all
select 25594,'025594','1998-077B','Kosmos-2363','Uragan No. 84L',66513,'In Earth orbit','','1999-01-29 00:00:00',675.71,19104,19155,64.81,'Medium' from dual union all
select 25595,'025595','1998-077C','Kosmos-2364','Uragan No. 79L',66513,'In Earth orbit','','1999-02-01 00:00:00',675.71,19126,19133,64.83,'Medium' from dual union all
select 25596,'025596','1998-077D','8S812','8S812',66513,'Reentered','1998-12-31 00:00:00','1998-12-31 00:00:00',87.01,123,130,64.84,'Intermediate' from dual union all
select 25597,'025597','1998-077E','Perekhodnik','Sredniy perekhodnik',66513,'Reentered','1998-12-31 00:00:00','1998-12-31 00:00:00',90.82,155,474,64.84,'Intermediate' from dual union all
select 25598,'025598','1998-077F','Blok DM-2 No. 92L','11S861 No. 92L',66513,'In Earth orbit','','1999-02-03 00:00:00',674.74,19084,19126,64.83,'Medium' from dual union all
select 25599,'025599','1998-077G','deb Proton','',66513,'Reentered','1999-01-01 00:00:00','1998-12-31 00:00:00',90.68,155,460,64.84,'Intermediate' from dual union all
select 256,'000256','62 Eta 1','Samos 6','SAMOS E-5 No. 3',8189,'Reentered','1963-06-07 00:00:00','1962-04-10 00:00:00',93.54,241,655,90.89,'Polar' from dual union all
select 2560,'002560','1966-101Z','deb OGCh','',13508,'Reentered','1966-11-07 00:00:00','1966-11-03 00:00:00',101.48,114,1543,49.36,'Intermediate' from dual union all
select 25600,'025600','1998-077H','SOZ','SOZ',66513,'Reentered','2005-10-19 00:00:00','1999-01-30 00:00:00',340.06,396,19125,64.81,'Highly Ellip' from dual union all
select 25601,'025601','1998-077J','SOZ','SOZ',66513,'Reentered','2005-08-09 00:00:00','1999-01-29 00:00:00',340.11,395,19129,64.88,'Highly Ellip' from dual union all
select 25602,'025602','1998-077K','deb Proton','',66513,'Reentered','1998-12-31 00:00:00','1998-12-31 00:00:00',89.5,148,352,64.68,'Intermediate' from dual union all
select 25603,'025603','1998-077L','deb Proton','',66513,'Reentered','1998-12-31 00:00:00','1998-12-31 00:00:00',89.28,144,334,65.04,'Intermediate' from dual union all
select 25604,'025604','1998-077M','deb Proton','',66513,'Reentered','1998-12-31 00:00:00','1998-12-31 00:00:00',89.94,147,396,64.85,'Intermediate' from dual union all
select 25605,'025605','1999-001A','Mars Polar Lander','MPL',66682,'Deep Space Attached','1999-01-04 00:00:00','1999-01-03 00:00:00',1121.31,185,84446,28.43,'Earth Escape' from dual union all
select 25606,'025606','1999-001B','Star 48B','Star 48B',66682,'Deep Space','1999-01-04 00:00:00','1999-01-03 00:00:00',1121.31,185,84446,28.43,'Earth Escape' from dual union all
select 25607,'025607','1999-001C','Delta 265','Delta SSPS AJ-10-118K',66682,'Reentered','1999-03-23 00:00:00','1999-02-02 00:00:00',93.22,220,645,25.84,'Intermediate' from dual union all
select 25608,'025608','1992-086F','deb USA 89 PKM','',64798,'In Earth orbit','','1999-01-31 00:00:00',102.78,890,891,57,'Intermediate' from dual union all
select 25609,'025609','1996-029K','deb USA 119','',65917,'In Earth orbit','','1999-01-31 00:00:00',107.41,1053,1161,63.4,'Intermediate' from dual union all
select 2561,'002561','1966-101AA','deb OGCh','',13508,'Reentered','1966-11-07 00:00:00','1966-11-03 00:00:00',95.28,126,939,49.35,'Intermediate' from dual union all
select 25610,'025610','1996-029L','deb USA 119','',65917,'In Earth orbit','','1999-01-31 00:00:00',107.37,1050,1160,63.4,'Intermediate' from dual union all
select 25611,'025611','1997-029C','FG-36 AKM','Shanxi AKM',66109,'In Earth orbit','','1999-01-04 00:00:00',1399.66,34101,36040,.33,'Drift GEO' from dual union all
select 25612,'025612','1966-040E','Nimbus 2 debris','',13447,'In Earth orbit','','1999-01-12 00:00:00',107.89,1090,1168,100.43,'Sun-Synch' from dual union all
select 25613,'025613','1966-040F','Nimbus 2 debris','',13447,'In Earth orbit','','1999-01-13 00:00:00',107.94,1100,1162,100.38,'Sun-Synch' from dual union all
select 25614,'025614','1966-040G','Nimbus 2 debris','',13447,'In Earth orbit','','1999-01-13 00:00:00',107.87,1089,1167,100.49,'Sun-Synch' from dual union all
select 25615,'025615','1998-055C','ATEX','ATEX',66491,'In Earth orbit','','1999-05-27 00:00:00',99.94,746,766,85,'Polar' from dual union all
select 25616,'025616','1999-002A','Formosat-1','ROCSAT-1',66683,'In Earth orbit','','1999-03-04 00:00:00',96.58,586,604,34.99,'Intermediate' from dual union all
select 25617,'025617','1999-002B','OAM','OAM',66683,'Reentered','1999-01-29 00:00:00','1999-01-28 00:00:00',90.29,119,459,34.67,'Intermediate' from dual union all
select 25618,'025618','1999-003A','Stardust','Stardust',66684,'Deep Space','1999-02-08 00:00:00','1999-02-07 00:00:00',314.7,224,43632,28.49,'Earth Escape' from dual union all
select 25619,'025619','1999-003B','Delta 266','Delta SSPS AJ-10-118K',66684,'In Earth orbit','','1999-03-09 00:00:00',164.09,292,6808,22.51,'Medium' from dual union all
select 2562,'002562','1966-101AB','deb OGCh','',13508,'Reentered','1966-11-08 00:00:00','1966-11-03 00:00:00',94.55,125,869,49.53,'Intermediate' from dual union all
select 25620,'025620','1999-003C','Star 37FM','Star 37FM',66684,'Deep Space','1999-02-08 00:00:00','1999-02-07 00:00:00',314.7,224,43632,28.49,'Earth Escape' from dual union all
select 25621,'025621','1999-004A','Globalstar FM23','Globalstar FM23',66685,'In Earth orbit','','1999-03-11 00:00:00',113.94,1405,1409,52,'Intermediate' from dual union all
select 25622,'025622','1999-004B','Globalstar FM40','Globalstar FM40',66685,'In Earth orbit','','1999-03-11 00:00:00',113.66,1388,1400,51.98,'Intermediate' from dual union all
select 25623,'025623','1999-004C','Globalstar FM36','Globalstar FM36',66685,'In Earth orbit','','1999-03-11 00:00:00',114.06,1410,1416,52.02,'Intermediate' from dual union all
select 25624,'025624','1999-004D','Globalstar FM38','Globalstar FM38',66685,'In Earth orbit','','1999-03-11 00:00:00',114.06,1412,1413,52,'Intermediate' from dual union all
select 25625,'025625','1999-004E','11S510','11S510',66685,'Reentered','1999-08-16 00:00:00','1999-03-11 00:00:00',95.2,234,822,51.96,'Intermediate' from dual union all
select 25626,'025626','1999-005A','Telstar 6','Telstar 6',66686,'In Earth orbit','','1999-03-17 00:00:00',1436.06,35776,35796,.04,'Stationary' from dual union all
select 25627,'025627','1999-005B','8S812','8S812',66686,'Reentered','1999-02-17 00:00:00','1999-02-16 00:00:00',87.65,136,181,51.6,'Intermediate' from dual union all
select 25628,'025628','1999-005C','Perekhodnik','Sredniy perekhodnik',66686,'Reentered','1999-02-15 00:00:00','1999-02-15 00:00:00',87.69,148,173,51.59,'Intermediate' from dual union all
select 25629,'025629','1999-005D','Blok DM3 No. 4L','Blok DM3 No. 4L',66686,'In Earth orbit','','1999-03-18 00:00:00',759.51,6617,35776,17.31,'Highly Ellip' from dual union all
select 2563,'002563','1966-101AC','deb OGCh','',13508,'Reentered','1966-11-08 00:00:00','1966-11-04 00:00:00',93.76,129,788,49.59,'Intermediate' from dual union all
select 25630,'025630','1999-006A','JCSAT 4A','JCSAT 6',66687,'In Earth orbit','','1999-03-18 00:00:00',1436.57,35785,35807,.01,'Drift GEO' from dual union all
select 25631,'025631','1999-006B','Centaur AC-152','Centaur IIA AC-152',66687,'Reentered','1999-03-17 00:00:00','1999-02-16 00:00:00',2129.02,261,96624,22.77,'Very High' from dual union all
select 25632,'025632','1999-007A','Soyuz TM-29','Soyuz 7K-STM No. 78',66688,'Landed','1999-08-28 00:00:00','1999-03-22 00:00:00',91.51,341,357,51.66,'Intermediate' from dual union all
select 25633,'025633','1999-007B','11S510','11S510',66688,'Reentered','1999-02-21 00:00:00','1999-02-20 00:00:00',88.36,182,205,51.64,'Intermediate' from dual union all
select 25634,'025634','1999-008A','ARGOS','P91-1',66689,'In Earth orbit','','1999-03-25 00:00:00',101.64,828,845,98.73,'Sun-Synch' from dual union all
select 25635,'025635','1999-008B','Orsted','Orsted',66689,'In Earth orbit','','1999-03-25 00:00:00',99.92,646,864,96.48,'Sun-Synch' from dual union all
select 25636,'025636','1999-008C','Sunsat-OSCAR-35','Sunsat',66689,'In Earth orbit','','1999-03-25 00:00:00',99.93,646,864,96.48,'Sun-Synch' from dual union all
select 25637,'025637','1999-008D','Delta 267','Delta SSPS AJ-10-118K',66689,'In Earth orbit','','1999-03-25 00:00:00',99.9,644,864,96.48,'Sun-Synch' from dual union all
select 25638,'025638','1999-009A','Arabsat 3A','Arabsat 3A',66690,'In Earth orbit','','1999-03-28 00:00:00',1436.01,35773,35797,.07,'Stationary' from dual union all
select 25639,'025639','1999-009B','Skynet 4E','Skynet 4E',66690,'In Earth orbit','','1999-03-28 00:00:00',1416.06,31657,39130,3.35,'Near-synch' from dual union all
select 2564,'002564','1966-101AD','deb OGCh','',13508,'Reentered','1966-11-09 00:00:00','1966-11-07 00:00:00',90.33,184,398,49.79,'Intermediate' from dual union all
select 25640,'025640','1999-009C','Ariane H10-3 V116','Ariane H10-3 S/N T484',66690,'Reentered','2013-02-11 00:00:00','1999-03-30 00:00:00',654.98,243,36967,7.01,'GEO Transfer' from dual union all
select 25641,'025641','1999-009D','Spelda 30 S/N UV39','Spelda 40',66690,'Reentered','2003-09-04 00:00:00','1999-03-28 00:00:00',622.8,231,35329,6.95,'GEO Transfer' from dual union all
select 25642,'025642','1999-010A','Raduga-1','Globus No. 15L',66691,'In Earth orbit','','1999-03-30 00:00:00',1436.01,35757,35813,1.43,'Stationary' from dual union all
select 25643,'025643','1999-010B','8S812','8S812',66691,'Reentered','1999-03-03 00:00:00','1999-03-01 00:00:00',87.97,172,176,51.64,'Intermediate' from dual union all
select 25644,'025644','1999-010C','Perekhodnik','Sredniy perekhodnik',66691,'Reentered','1999-03-01 00:00:00','1999-02-28 00:00:00',88.09,176,184,51.63,'Intermediate' from dual union all
select 25645,'025645','1999-010D','Blok DM-2 No. 84L','11S861 No. 84L',66691,'In Earth orbit','','1999-03-30 00:00:00',1468.68,36336,36508,1.47,'Drift GEO' from dual union all
select 25646,'025646','1999-011A','WIRE','WIRE',66692,'Reentered','2011-05-10 00:00:00','1999-04-04 00:00:00',95.97,539,591,97.53,'Sun-Synch' from dual union all
select 25647,'025647','1999-011B','Orion 38','Orion 38',66692,'Reentered','2007-09-06 00:00:00','1999-04-08 00:00:00',95.93,540,587,97.55,'Sun-Synch' from dual union all
select 25648,'025648','1999-011C','WIRE telescope cover','Telescope cover',66692,'Reentered','2001-09-24 00:00:00','1999-04-04 00:00:00',95.94,539,589,97.52,'Sun-Synch' from dual union all
select 25649,'025649','1999-012A','Globalstar FM22','Globalstar FM22',66693,'In Earth orbit','','1999-04-14 00:00:00',114.06,1412,1413,52.01,'Intermediate' from dual union all
select 2565,'002565','1966-103A','TDA 7A','TDA 7A',13510,'Reentered Attached','1966-12-23 00:00:00','1966-11-18 00:00:00',89.92,251,290,28.85,'Intermediate' from dual union all
select 25650,'025650','1999-012B','Globalstar FM41','Globalstar FM41',66693,'In Earth orbit','','1999-04-14 00:00:00',114.06,1411,1415,52.01,'Intermediate' from dual union all
select 25651,'025651','1999-012C','Globalstar FM46','Globalstar FM46',66693,'In Earth orbit','','1999-04-14 00:00:00',114.06,1411,1415,52.01,'Intermediate' from dual union all
select 25652,'025652','1999-012D','Globalstar FM37','Globalstar FM37',66693,'In Earth orbit','','1999-04-14 00:00:00',114.02,1410,1411,52.01,'Intermediate' from dual union all
select 25653,'025653','1999-012E','11S510','11S510',66693,'Reentered','1999-09-10 00:00:00','1999-04-14 00:00:00',95.27,231,833,51.98,'Intermediate' from dual union all
select 25654,'025654','1999-012F','Ikar No. 2','Ikar 50KS',66693,'Deorbited','1999-03-16 00:00:00','1999-03-15 00:00:00',103.46,895,950,51.97,'Intermediate' from dual union all
select 25655,'025655','1988-032D','deb Blok E','',61281,'In Earth orbit','','1999-03-15 00:00:00',97.06,585,651,97.68,'Sun-Synch' from dual union all
select 25656,'025656','1983-127N','deb SOZ','',55884,'Reentered','1999-07-11 00:00:00','1999-03-15 00:00:00',147.11,574,5132,50.63,'Medium' from dual union all
select 25657,'025657','1999-013A','Asiasat 3S','Asiasat 3S',66694,'In Earth orbit','','1999-04-21 00:00:00',1436.04,35766,35805,.05,'Stationary' from dual union all
select 25658,'025658','1999-013B','8S812','8S812',66694,'Reentered','1999-03-22 00:00:00','1999-03-22 00:00:00',87.69,144,177,51.59,'Intermediate' from dual;
insert into SATELLITE
select 25659,'025659','1999-013C','Perekhodnik','Sredniy perekhodnik',66694,'Reentered','1999-03-21 00:00:00','1999-03-21 00:00:00',87.14,126,140,51.59,'Intermediate' from dual union all
select 2566,'002566','1966-104A','Gemini XII','Gemini SC12',13511,'Landed','1966-11-15 00:00:00','1966-11-14 00:00:00',89.94,252,290,28.84,'Intermediate' from dual union all
select 25660,'025660','1999-013D','Blok DM3 No. 12L','Blok DM3 No. 12L',66694,'In Earth orbit','','1999-04-22 00:00:00',823.21,9501,35935,13.11,'Medium' from dual union all
select 25661,'025661','1999-014A','DemoSat','DemoSat',66695,'In Earth orbit','','1999-04-29 00:00:00',645.02,628,36074,1.31,'GEO Transfer' from dual union all
select 25662,'025662','1999-014B','Blok DM-SL No. 1TL','Blok DM-SL No. 1TL',66695,'In Earth orbit','','1999-04-28 00:00:00',642.45,571,36000,1.29,'GEO Transfer' from dual union all
select 25663,'025663','1994-029AEF','deb HAPS','',65425,'Reentered','2000-01-18 00:00:00','1999-02-15 00:00:00',99.36,577,879,82.21,'Intermediate' from dual union all
select 25664,'025664','1999-015A','Progress M-41','Progress 7K-TGM No 241',66696,'Deorbited','1999-07-17 00:00:00','1999-05-02 00:00:00',91.5,336,360,51.66,'Intermediate' from dual union all
select 25665,'025665','1999-015B','11S510','11S510',66696,'Reentered','1999-04-04 00:00:00','1999-04-03 00:00:00',88.07,166,192,51.63,'Intermediate' from dual union all
select 25666,'025666','1999-016A','Insat 2E','Insat 2E',66697,'In Earth orbit','','1999-05-02 00:00:00',1436.06,35766,35806,.05,'Stationary' from dual union all
select 25667,'025667','1999-016B','Ariane H10-3 V117','Ariane H10-3 S/N T486',66697,'In Earth orbit','','1999-05-08 00:00:00',657.12,270,37049,4.02,'GEO Transfer' from dual union all
select 25668,'025668','1992-060E','Ariane V53 large debris','RCS 11 sq m',64772,'Reentered','2015-03-28 00:00:00','1999-04-07 00:00:00',552.32,256,31586,7.25,'Highly Ellip' from dual union all
select 25669,'025669','1999-017A','DSP F19','DSP 19',66698,'In Earth orbit','','2006-08-17 00:00:00',618.17,590,34730,29.29,'GEO Transfer' from dual union all
select 2567,'002567','1966-104B','Titan GT-12 Stage 2','LR91AJ5 GT-12 65-12567',13511,'Reentered','1966-11-12 00:00:00','1966-11-11 00:00:00',88.9,164,277,28.91,'Intermediate' from dual union all
select 25670,'025670','1999-017B','Titan 4B-24 Stage 2','LR91AJ11A  (K-32)',66698,'Reentered','1999-07-02 00:00:00','1999-05-09 00:00:00',92.74,185,633,28.63,'Intermediate' from dual union all
select 25671,'025671','1999-017C','IUS-21 SRM-1','Orbus 21',66698,'In Earth orbit','','1999-04-09 00:00:00',633.66,250,35870,28.5,'GEO Transfer' from dual union all
select 25672,'025672','1999-017D','IUS-21 SRM-2','Orbus 6E',66698,'In Earth orbit','','1999-04-09 00:00:00',633.66,250,35870,28.5,'GEO Transfer' from dual union all
select 25673,'025673','1999-018A','Eutelsat W3','Eutelsat W3',66699,'In Earth orbit','','1999-05-12 00:00:00',1432.46,35694,35736,.02,'Drift GEO' from dual union all
select 25674,'025674','1999-018B','Centaur AC-154','Centaur IIA AC-154',66699,'Reentered','2001-09-27 00:00:00','1999-05-16 00:00:00',808.61,200,44546,19.71,'Highly Ellip' from dual union all
select 25675,'025675','1999-017E','IUS debris','',66698,'In Earth orbit','','1999-04-09 00:00:00',633.66,250,35870,28.5,'GEO Transfer' from dual union all
select 25676,'025676','1999-019A','Globalstar FM45','Globalstar FM45',66700,'In Earth orbit','','1999-10-10 00:00:00',114.06,1412,1413,52,'Intermediate' from dual union all
select 25677,'025677','1999-019B','Globalstar FM19','Globalstar FM19',66700,'In Earth orbit','','1999-10-10 00:00:00',114.06,1412,1413,52,'Intermediate' from dual union all
select 25678,'025678','1999-019C','Globalstar FM44','Globalstar FM44',66700,'In Earth orbit','','1999-09-03 00:00:00',114.07,1412,1414,52,'Intermediate' from dual union all
select 25679,'025679','1999-019D','Globalstar FM42','Globalstar FM42',66700,'In Earth orbit','','1999-05-15 00:00:00',114.06,1412,1413,52,'Intermediate' from dual union all
select 2568,'002568','1966-105A','Kosmos-131','Zenit-4 No. 25',13512,'Landed','1966-11-20 00:00:00','1966-11-14 00:00:00',89.86,201,334,72.86,'Intermediate' from dual union all
select 25680,'025680','1999-019E','11S510','11S510',66700,'Reentered','1999-10-14 00:00:00','1999-05-15 00:00:00',95.41,231,846,51.95,'Intermediate' from dual union all
select 25681,'025681','1999-019F','Ikar No. 3','Ikar 50KS No. M15000-03',66700,'Deorbited','1999-04-16 00:00:00','1999-04-16 00:00:00',103.52,898,952,51.94,'Intermediate' from dual union all
select 25682,'025682','1999-020A','Landsat 7','Landsat 7',66701,'In Earth orbit','','1999-05-15 00:00:00',98.61,687,698,98.22,'Sun-Synch' from dual union all
select 25683,'025683','1999-020B','Delta 268','Delta SSPS AJ-10-118K',66701,'Reentered','1999-05-30 00:00:00','1999-04-22 00:00:00',93.01,174,669,107.57,'Retrograde' from dual union all
select 25684,'025684','1985-118X','deb SOZ','',58341,'In Earth orbit','','1999-04-14 00:00:00',305.62,447,16856,64.64,'Highly Ellip' from dual union all
select 25685,'025685','1999-015C','Sputnik-99','Sputnik-99',66696,'Reentered','1999-07-29 00:00:00','1999-05-02 00:00:00',91.26,328,345,51.66,'Intermediate' from dual union all
select 25686,'025686','1986-017NA','EO-27 EVA-1 debris','',59210,'Reentered','1999-07-10 00:00:00','1999-04-18 00:00:00',91.38,335,350,51.66,'Intermediate' from dual union all
select 25687,'025687','1986-017NB','EO-27 EVA-1 debris','',59210,'Reentered','1999-06-28 00:00:00','1999-04-18 00:00:00',91.4,337,350,51.66,'Intermediate' from dual union all
select 25688,'025688','1986-017NC','EO-27 EVA-1 debris','',59210,'Reentered','1999-04-26 00:00:00','1999-04-18 00:00:00',91.13,322,338,51.66,'Intermediate' from dual union all
select 25689,'025689','1986-017ND','EO-27 EVA-1 debris','',59210,'Reentered','1999-04-21 00:00:00','1999-04-18 00:00:00',90.8,309,319,51.66,'Intermediate' from dual union all
select 2569,'002569','1966-105B','11A57I','11A57I',13512,'Reentered','1966-11-22 00:00:00','1966-11-17 00:00:00',89.26,206,270,72.85,'Intermediate' from dual union all
select 25690,'025690','1986-017NE','EO-27 EVA-1 debris','',59210,'Reentered','1999-06-08 00:00:00','1999-04-18 00:00:00',91.38,336,349,51.67,'Intermediate' from dual union all
select 25691,'025691','1986-017NF','EO-27 EVA-1 debris','',59210,'Reentered','1999-09-02 00:00:00','1999-04-18 00:00:00',91.4,336,351,51.66,'Intermediate' from dual union all
select 25692,'025692','1986-017NG','EO-27 EVA-1 debris','',59210,'Reentered','1999-05-12 00:00:00','1999-04-19 00:00:00',91.24,325,346,51.66,'Intermediate' from dual union all
select 25693,'025693','1999-021A','UOSAT-OSCAR-36','UoSAT-12',66702,'In Earth orbit','','1999-05-22 00:00:00',97.72,647,653,64.55,'Intermediate' from dual union all
select 25694,'025694','1999-021B','Dnepr plume shield','GDS',66702,'Reentered','2002-12-12 00:00:00','1999-05-21 00:00:00',97.62,637,652,64.56,'Intermediate' from dual union all
select 25695,'025695','1999-021C','R-36M2 DS','R-36M2 DS',66702,'In Earth orbit','','1999-05-23 00:00:00',105.15,600,1403,64.56,'Intermediate' from dual union all
select 25696,'025696','1989-100AR','S5M debris','',62377,'Reentered','1999-05-09 00:00:00','1999-04-28 00:00:00',93.67,444,465,73.41,'Intermediate' from dual union all
select 25697,'025697','1989-100AS','S5M debris','',62377,'Reentered','1999-05-21 00:00:00','1999-04-28 00:00:00',94.79,468,549,73.54,'Intermediate' from dual union all
select 25698,'025698','1989-100AT','S5M debris','',62377,'Reentered','1999-05-12 00:00:00','1999-04-28 00:00:00',93.85,458,468,73.44,'Intermediate' from dual union all
select 25699,'025699','1989-100AU','S5M debris','',62377,'Reentered','1999-05-02 00:00:00','1999-04-28 00:00:00',90.94,249,393,74.85,'Intermediate' from dual union all
select 257,'000257','62 Zeta 2','Altair','Altair X-248-A5',8207,'Reentered','1970-05-03 00:00:00','1962-04-06 00:00:00',96.23,573,583,32.81,'Intermediate' from dual union all
select 2570,'002570','1966-101AE','deb OGCh','',13508,'Reentered','1966-11-22 00:00:00','1966-11-09 00:00:00',90.43,124,467,49.61,'Intermediate' from dual union all
select 25700,'025700','1989-100AV','S5M debris','',62377,'Reentered','1999-06-14 00:00:00','1999-04-27 00:00:00',96.52,545,639,73.19,'Intermediate' from dual union all
select 25701,'025701','1989-100AW','S5M debris','',62377,'Reentered','1999-06-12 00:00:00','1999-04-27 00:00:00',94.63,471,530,73.55,'Intermediate' from dual union all
select 25702,'025702','1989-100AX','S5M debris','',62377,'Reentered','1999-08-12 00:00:00','1999-04-28 00:00:00',93.97,456,482,73.52,'Intermediate' from dual union all
select 25703,'025703','1989-100AY','S5M debris','',62377,'Reentered','1999-07-06 00:00:00','1999-04-28 00:00:00',94.35,474,501,73.52,'Intermediate' from dual union all
select 25704,'025704','1989-100AZ','S5M debris','',62377,'Reentered','1999-07-04 00:00:00','1999-04-28 00:00:00',94.86,483,541,73.48,'Intermediate' from dual union all
select 25705,'025705','1989-100BA','S5M debris','',62377,'Reentered','1999-05-06 00:00:00','1999-04-27 00:00:00',91.72,271,447,75.19,'Intermediate' from dual union all
select 25706,'025706','1989-100BB','S5M debris','',62377,'Reentered','1999-05-06 00:00:00','1999-04-27 00:00:00',92.7,371,443,73.43,'Intermediate' from dual union all
select 25707,'025707','1989-100BC','S5M debris','',62377,'Reentered','1999-05-12 00:00:00','1999-04-27 00:00:00',94.29,467,501,74.91,'Intermediate' from dual union all
select 25708,'025708','1989-100BD','S5M debris','',62377,'Reentered','1999-05-30 00:00:00','1999-04-27 00:00:00',93.72,436,476,73.53,'Intermediate' from dual union all
select 25709,'025709','1989-100BE','S5M debris','',62377,'Reentered','2000-04-01 00:00:00','1999-04-29 00:00:00',94.64,474,529,73.65,'Intermediate' from dual union all
select 2571,'002571','1966-101AF','deb OGCh','',13508,'Reentered','1966-11-29 00:00:00','1966-11-09 00:00:00',93.52,136,758,49.55,'Intermediate' from dual union all
select 25710,'025710','1989-100BF','S5M debris','',62377,'Reentered','1999-06-26 00:00:00','1999-04-27 00:00:00',95.21,477,581,73.54,'Intermediate' from dual union all
select 25711,'025711','1989-100BG','S5M debris','',62377,'Reentered','1999-06-30 00:00:00','1999-04-28 00:00:00',93.88,450,479,73.54,'Intermediate' from dual union all
select 25712,'025712','1989-100BH','S5M debris','',62377,'Reentered','1999-09-16 00:00:00','1999-04-29 00:00:00',93.79,422,498,74.18,'Intermediate' from dual union all
select 25713,'025713','1989-100BJ','S5M debris','',62377,'Reentered','1999-06-03 00:00:00','1999-04-27 00:00:00',95.67,464,638,73.99,'Intermediate' from dual union all
select 25714,'025714','1989-100BK','S5M debris','',62377,'Reentered','1999-08-07 00:00:00','1999-04-29 00:00:00',93.99,457,483,73.53,'Intermediate' from dual union all
select 25715,'025715','1989-100BL','S5M debris','',62377,'Reentered','1999-05-15 00:00:00','1999-04-28 00:00:00',95.17,447,607,73.91,'Intermediate' from dual union all
select 25716,'025716','1989-100BM','S5M debris','',62377,'Reentered','1999-06-10 00:00:00','1999-04-28 00:00:00',93.94,458,477,73.51,'Intermediate' from dual union all
select 25717,'025717','1989-100BN','S5M debris','',62377,'Reentered','1999-10-07 00:00:00','1999-04-28 00:00:00',93.9,446,485,73.56,'Intermediate' from dual union all
select 25718,'025718','1989-100BP','S5M debris','',62377,'Reentered','1999-06-17 00:00:00','1999-04-28 00:00:00',94.31,456,515,73.69,'Intermediate' from dual union all
select 25719,'025719','1989-100BQ','S5M debris','',62377,'Reentered','1999-07-01 00:00:00','1999-04-28 00:00:00',93.93,453,480,73.58,'Intermediate' from dual union all
select 2572,'002572','1966-075E','Delta 40 attach clamp','',13482,'Reentered','1967-03-09 00:00:00','1966-09-29 00:00:00',96.4,254,918,32.99,'Intermediate' from dual union all
select 25720,'025720','1989-100BR','S5M debris','',62377,'Reentered','2001-02-17 00:00:00','1999-04-28 00:00:00',95.6,471,625,73.78,'Intermediate' from dual union all
select 25721,'025721','1999-022A','ABRIXAS','ABRIXAS',66703,'Reentered','2017-10-31 00:00:00','1999-05-28 00:00:00',96.23,554,601,48.45,'Intermediate' from dual union all
select 25722,'025722','1999-022B','Megsat-0','Megsat-0',66703,'Reentered','2003-11-04 00:00:00','1999-05-28 00:00:00',96.16,549,601,48.46,'Intermediate' from dual union all
select 25723,'025723','1999-022C','S3M','S3M',66703,'In Earth orbit','','1999-06-01 00:00:00',96.28,555,606,48.45,'Intermediate' from dual union all
select 25724,'025724','1999-023A','Milstar 3','Milstar DFS-3',66704,'In Earth orbit','','1999-06-06 00:00:00',153.61,1094,5152,28.2,'Medium' from dual union all
select 25725,'025725','1999-023B','Centaur TC-14','Centaur TC-14',66704,'In Earth orbit','','1999-06-08 00:00:00',148.94,703,5156,28.29,'Medium' from dual union all
select 25726,'025726','1991-003L','deb Ariane V41','',64172,'Reentered','2002-08-02 00:00:00','1999-05-01 00:00:00',500.97,292,28741,6.67,'Highly Ellip' from dual union all
select 25727,'025727','1999-024A','Orion 3','Orion 3',66705,'In Earth orbit','','1999-06-04 00:00:00',102.33,421,1316,29.06,'Intermediate' from dual union all
select 25728,'025728','1999-024B','Delta 269','DCSS F02',66705,'Reentered','1999-06-05 00:00:00','1999-05-12 00:00:00',99.23,160,1284,29.5,'Intermediate' from dual union all
select 25729,'025729','1999-024C','deb Delta III','',66705,'Reentered','1999-05-07 00:00:00','1999-05-06 00:00:00',95.07,147,897,29.62,'Intermediate' from dual union all
select 2573,'002573','1965-082KK','deb Transtage 4','',11832,'Reentered','1971-03-15 00:00:00','1971-03-14 00:00:00',92.83,394,432,32.39,'Intermediate' from dual union all
select 25730,'025730','1999-025A','Feng Yun 1C','Feng Yun Yi Beng',66706,'In Earth orbit','','1999-06-09 00:00:00',102.09,849,866,98.79,'Sun-Synch' from dual union all
select 25731,'025731','1999-025B','Shi Jian 5','shijian wuhao',66706,'In Earth orbit','','1999-06-09 00:00:00',102.05,844,867,98.79,'Sun-Synch' from dual union all
select 25732,'025732','1999-025C','CZ-4B Stage 3','CZ-4 Stage 3',66706,'In Earth orbit','','1999-06-09 00:00:00',101.66,811,864,98.74,'Sun-Synch' from dual union all
select 25733,'025733','1999-025D','Adapter','',66706,'In Earth orbit','','1999-06-09 00:00:00',102.07,846,868,98.79,'Sun-Synch' from dual union all
select 25734,'025734','1992-072G','deb Ariane V54','',64784,'Reentered','2001-03-27 00:00:00','2001-03-18 00:00:00',103.32,176,1656,6.96,'Equatorial' from dual union all
select 25735,'025735','1999-026A','TERRIERS','TERRIERS',66707,'In Earth orbit','','1999-06-17 00:00:00',95.58,540,554,97.72,'Sun-Synch' from dual union all
select 25736,'025736','1999-026B','MUBLCOM','MUBLCOM',66707,'In Earth orbit','','1999-06-17 00:00:00',100.33,772,777,97.72,'Sun-Synch' from dual union all
select 25737,'025737','1999-026C','Orion 38','Orion 38',66707,'Reentered','2000-09-24 00:00:00','1999-06-17 00:00:00',94.1,404,546,97.68,'Sun-Synch' from dual union all
select 25738,'025738','1999-026D','HAPS','HAPS',66707,'Reentered','2001-10-31 00:00:00','1999-06-17 00:00:00',95.76,392,719,97.08,'Sun-Synch' from dual union all
select 25739,'025739','1999-026E','PAF','Payload Adapter Fitting',66707,'Reentered','2000-07-31 00:00:00','1999-06-17 00:00:00',95.53,537,552,97.72,'Sun-Synch' from dual union all
select 2574,'002574','1965-082KL','deb Transtage 4','',11832,'Reentered','1968-12-03 00:00:00','1968-09-07 00:00:00',94.33,465,508,31.75,'Intermediate' from dual union all
select 25740,'025740','1999-027A','Nimiq 1','Telesat DTH-1',66708,'In Earth orbit','','1999-06-20 00:00:00',1436.09,35778,35795,.02,'Stationary' from dual union all
select 25741,'025741','1999-027B','8S812','8S812',66708,'Reentered','1999-05-23 00:00:00','1999-05-22 00:00:00',87.54,133,173,51.6,'Intermediate' from dual union all
select 25742,'025742','1999-027C','Perekhodnik','Sredniy perekhodnik',66708,'Reentered','1999-05-21 00:00:00','1999-05-21 00:00:00',87.82,121,212,51.6,'Intermediate' from dual union all
select 25743,'025743','1999-027D','Blok DM3 No. 11L','Blok DM3 No. 11L',66708,'In Earth orbit','','1999-06-19 00:00:00',766.63,7004,35734,16.43,'Highly Ellip' from dual union all
select 25744,'025744','1999-028A','USA 144','MISTY 2',66709,'In Earth orbit','','2005-10-20 00:00:00',148.46,2682,3137,63.44,'Medium' from dual union all
select 25745,'025745','1999-028B','Titan 4B-12 Stage 2','LR91AJ11A',66709,'Reentered','1999-06-01 00:00:00','1999-05-24 00:00:00',89.48,202,296,63.39,'Intermediate' from dual union all
select 25746,'025746','1999-028C','USA 144 part','',66709,'Reentered','1999-08-01 00:00:00','1999-06-08 00:00:00',92.21,376,390,63.41,'Intermediate' from dual union all
select 25747,'025747','1999-028D','USA 144 part','',66709,'Reentered','1999-08-01 00:00:00','1999-10-15 00:00:00',90.89,264,372,63.4,'Intermediate' from dual union all
select 25748,'025748','1999-028E','USA 144 part','',66709,'Reentered','1999-08-01 00:00:00','1999-05-24 00:00:00',89.5,200,300,63.4,'Intermediate' from dual;
insert into SATELLITE
select 25749,'025749','1999-028F','USA 144 part','',66709,'Reentered','1999-08-01 00:00:00','1999-05-24 00:00:00',89.5,200,300,63.4,'Intermediate' from dual union all
select 2575,'002575','1966-101AG','deb OGCh','',13508,'Reentered','1966-11-11 00:00:00','1966-11-10 00:00:00',88.09,28,332,48.35,'ERROR - Unexpected value "TA"' from dual union all
select 25750,'025750','1994-029AEG','deb HAPS','',65425,'In Earth orbit','','1999-05-09 00:00:00',102.23,606,1123,82.45,'Intermediate' from dual union all
select 25751,'025751','1999-028G','USA 144 part','',66709,'Reentered','1999-08-01 00:00:00','1999-05-24 00:00:00',89.5,200,300,63.4,'Intermediate' from dual union all
select 25752,'025752','1999-028H','USA 144 part','',66709,'Reentered','2000-02-20 00:00:00','2000-02-19 00:00:00',87.74,153,172,63.37,'Intermediate' from dual union all
select 25753,'025753','1999-028J','USA 144 part','',66709,'Reentered','2000-02-01 00:00:00','1999-05-24 00:00:00',89.5,200,300,63.4,'Intermediate' from dual union all
select 25754,'025754','1999-028K','USA 144 part','',66709,'Reentered','2000-02-01 00:00:00','1999-05-24 00:00:00',89.5,200,300,63.4,'Intermediate' from dual union all
select 25755,'025755','1999-028L','USA 144 part','',66709,'Reentered','2000-02-01 00:00:00','1999-05-24 00:00:00',89.5,200,300,63.4,'Intermediate' from dual union all
select 25756,'025756','1999-029A','Uribyol-3','KITSAT-3',66710,'In Earth orbit','','1999-06-25 00:00:00',99.33,716,737,98.38,'Sun-Synch' from dual union all
select 25757,'025757','1999-029B','DLR-TUBSAT','DLR-TUBSAT',66710,'In Earth orbit','','1999-06-25 00:00:00',99.24,713,732,98.28,'Sun-Synch' from dual union all
select 25758,'025758','1999-029C','Oceansat','IRS-P4',66710,'In Earth orbit','','1999-06-25 00:00:00',99.34,716,739,98.38,'Sun-Synch' from dual union all
select 25759,'025759','1999-029D','PSLV-D3 PS4','PS4',66710,'In Earth orbit','','1999-06-25 00:00:00',99.31,716,736,98.38,'Sun-Synch' from dual union all
select 2576,'002576','1966-101AH','deb OGCh','',13508,'Reentered','1966-11-13 00:00:00','1966-11-12 00:00:00',90.5,140,459,49.23,'Intermediate' from dual union all
select 25760,'025760','1999-030A','Discovery (STS-96)','OV-103',66711,'Landed','1999-06-06 00:00:00','1999-05-29 00:00:00',92.2,380,385,51.59,'Intermediate' from dual union all
select 25761,'025761','1999-010E','SOZ','SOZ',66691,'Reentered','1999-09-02 00:00:00','1999-05-27 00:00:00',641.52,192,36331,47.36,'GEO Transfer' from dual union all
select 25762,'025762','1999-010F','SOZ','SOZ',66691,'Reentered','1999-10-06 00:00:00','1999-05-26 00:00:00',646.62,248,36536,47.43,'GEO Transfer' from dual union all
select 25763,'025763','1979-104E','deb Ariane L01','',47588,'Reentered','1999-05-16 00:00:00','1999-05-16 00:00:00',88.07,127,231,17.93,'Equatorial' from dual union all
select 25764,'025764','1979-104F','deb Ariane L01','',47588,'Reentered','1999-08-07 00:00:00','1999-06-02 00:00:00',109.58,283,2132,19.05,'Equatorial' from dual union all
select 25765,'025765','1979-104G','deb Ariane L01','',47588,'In Earth orbit','','1999-06-02 00:00:00',386.19,412,21966,19.58,'Highly Ellip' from dual union all
select 25766,'025766','1979-104H','deb Ariane L01','',47588,'In Earth orbit','','1999-06-02 00:00:00',414.36,557,23509,20.38,'Highly Ellip' from dual union all
select 25767,'025767','1979-104J','deb Ariane L01','',47588,'In Earth orbit','','1999-05-29 00:00:00',271.6,402,14626,17.91,'Highly Ellip' from dual union all
select 25768,'025768','1979-104K','deb Ariane L01','',47588,'In Earth orbit','','1999-06-01 00:00:00',314.23,590,17275,17.83,'Highly Ellip' from dual union all
select 25769,'025769','1999-030B','Starshine','Starshine',66711,'Reentered','2000-02-18 00:00:00','1999-06-26 00:00:00',92.23,375,393,51.59,'Intermediate' from dual union all
select 2577,'002577','1966-104C','Gemini XII waste bag','',13511,'Reentered','1966-11-15 00:00:00','1966-11-13 00:00:00',89.62,249,263,28.84,'Intermediate' from dual union all
select 25770,'025770','1999-031A','Globalstar M025','Globalstar M025',66712,'In Earth orbit','','1999-07-11 00:00:00',114.06,1413,1413,52,'Intermediate' from dual union all
select 25771,'025771','1999-031B','Globalstar M049','Globalstar M049',66712,'In Earth orbit','','1999-07-10 00:00:00',114.06,1412,1413,52,'Intermediate' from dual union all
select 25772,'025772','1999-031C','Globalstar M047','Globalstar M047',66712,'In Earth orbit','','1999-07-10 00:00:00',114.07,1413,1414,52,'Intermediate' from dual union all
select 25773,'025773','1999-031D','Globalstar M052','Globalstar M052',66712,'In Earth orbit','','1999-07-10 00:00:00',114.06,1412,1413,51.99,'Intermediate' from dual union all
select 25774,'025774','1999-031E','Delta 270','Delta SSPS AJ-10-118K',66712,'In Earth orbit','','1999-07-12 00:00:00',107,807,1369,51.54,'Intermediate' from dual union all
select 25775,'025775','1999-017F','IUS debris','',66698,'In Earth orbit','','1999-04-10 00:00:00',631.91,250,35780,27,'GEO Transfer' from dual union all
select 25776,'025776','1979-009C','Star 37N S/N 14006','Star 37N',47493,'Reentered','2014-09-27 00:00:00','1999-06-08 00:00:00',249.28,287,13197,23.96,'Medium' from dual union all
select 25777,'025777','1999-032A','Iridium 14A','Iridium SV092',66713,'In Earth orbit','','1999-07-11 00:00:00',98.98,708,712,86.52,'Polar' from dual union all
select 25778,'025778','1999-032B','Iridium 21A','Iridium SV093',66713,'Reentered','2018-05-24 00:00:00','1999-07-11 00:00:00',98.98,709,711,86.51,'Polar' from dual union all
select 25779,'025779','1999-032C','CZ-2C Stage 2','CZ-2C Stage 2',66713,'Reentered','1999-07-13 00:00:00','1999-06-18 00:00:00',92.4,175,609,86.38,'Polar' from dual union all
select 2578,'002578','1966-104D','Gemini XII EVA debris','',13511,'Reentered','1966-11-15 00:00:00','1966-11-13 00:00:00',89.66,250,266,28.81,'Intermediate' from dual union all
select 25780,'025780','1999-032D','Smart Dispenser','CZ-2C SD',66713,'Reentered','1999-07-25 00:00:00','1999-06-18 00:00:00',92.61,206,599,86.38,'Polar' from dual union all
select 25781,'025781','1999-032E','CZ-2C Stage 2 motor cover','',66713,'Reentered','1999-06-15 00:00:00','1999-06-13 00:00:00',93.75,174,742,86.08,'Polar' from dual union all
select 25782,'025782','1999-032F','CZ-2C Stage 2 motor cover','',66713,'Reentered','1999-06-15 00:00:00','1999-06-13 00:00:00',93.71,175,737,86.61,'Polar' from dual union all
select 25783,'025783','1999-032G','CZ-2C Stage 2 motor cover','',66713,'Reentered','1999-06-13 00:00:00','1999-06-13 00:00:00',94.1,173,777,86.69,'Polar' from dual union all
select 25784,'025784','1999-032H','CZ-2C Stage 2 motor cover','',66713,'Reentered','1999-06-15 00:00:00','1999-06-13 00:00:00',93.92,175,758,86.13,'Polar' from dual union all
select 25785,'025785','1999-033A','Astra 1H','Astra 1H',66714,'In Earth orbit','','1999-07-18 00:00:00',1436.06,35771,35801,.04,'Stationary' from dual union all
select 25786,'025786','1999-033B','8S812','8S812',66714,'Reentered','1999-06-19 00:00:00','1999-06-19 00:00:00',87.48,132,168,51.6,'Intermediate' from dual union all
select 25787,'025787','1999-033C','Perekhodnik','Sredniy perekhodnik',66714,'Reentered','1999-06-19 00:00:00','1999-06-18 00:00:00',87.12,123,141,51.59,'Intermediate' from dual union all
select 25788,'025788','1999-033D','Blok DM3 No. 8L','Blok DM3 No. 8L',66714,'In Earth orbit','','1999-07-18 00:00:00',774.95,7335,35804,16.28,'Highly Ellip' from dual union all
select 25789,'025789','1999-034A','QuikScat','QuikScat',66715,'In Earth orbit','','1999-07-20 00:00:00',100.98,804,807,98.62,'Sun-Synch' from dual union all
select 2579,'002579','1966-104E','Gemini XII EVA debris','',13511,'Reentered','1966-11-16 00:00:00','1966-11-14 00:00:00',89.65,241,274,28.86,'Intermediate' from dual union all
select 25790,'025790','1999-034B','Titan 23G Stage 2','LR91AJ5 23G-12',66715,'Reentered','1999-12-20 00:00:00','1999-07-20 00:00:00',95.04,246,794,98.66,'Sun-Synch' from dual union all
select 25791,'025791','1999-035A','FUSE','Far UV Spec. Explorer',66716,'In Earth orbit','','1999-07-24 00:00:00',100.06,753,770,24.98,'Equatorial' from dual union all
select 25792,'025792','1999-035B','Delta 271','Delta SSPS AJ-10-118K',66716,'Reentered','1999-08-04 00:00:00','1999-07-01 00:00:00',95.06,171,872,19.06,'Equatorial' from dual union all
select 25793,'025793','61 Sigma 6','Midas 3 debris','',7278,'In Earth orbit','','1999-06-23 00:00:00',161.23,3311,3559,91.17,'Medium' from dual union all
select 25794,'025794','1980-089CH','deb Kosmos-1220','',50299,'Reentered','1999-06-28 00:00:00','1999-06-27 00:00:00',88.79,200,229,64.91,'Intermediate' from dual union all
select 25795,'025795','1980-089CJ','deb Kosmos-1220','',50299,'In Earth orbit','','1999-06-27 00:00:00',97.38,531,736,65,'Intermediate' from dual union all
select 25796,'025796','1980-089CK','deb Kosmos-1220','',50299,'Reentered','2000-01-16 00:00:00','1999-06-27 00:00:00',96.04,503,635,64.98,'Intermediate' from dual union all
select 25797,'025797','1989-001J','deb SOZ','',62278,'Reentered','1999-06-28 00:00:00','1999-05-28 00:00:00',116.33,433,2599,64.86,'Intermediate' from dual union all
select 25798,'025798','1989-001K','deb SOZ','',62278,'In Earth orbit','','1999-06-28 00:00:00',269.42,653,14226,64.99,'Medium' from dual union all
select 25799,'025799','1989-001L','deb SOZ','',62278,'Reentered','2014-02-10 00:00:00','1999-05-12 00:00:00',298.54,838,15997,65.34,'Highly Ellip' from dual union all
select 258,'000258','62 Eta 2','Samos 6 Reflector','Samos 6 Reflector',8189,'Reentered','1962-03-31 00:00:00','1962-03-18 00:00:00',89.93,219,322,90.85,'Polar' from dual union all
select 2580,'002580','1966-076C','NNS O-10 despin weight','',13483,'In Earth orbit','','1966-10-11 00:00:00',105.33,934,1086,89.14,'Polar' from dual union all
select 25800,'025800','1990-110J','deb SOZ','[UNID]',63352,'In Earth orbit','','1999-06-26 00:00:00',343.53,764,18976,65.12,'Highly Ellip' from dual union all
select 25801,'025801','1991-010G','deb SOZ','',64179,'Reentered','2009-04-26 00:00:00','1999-06-02 00:00:00',531.4,295,30414,47.29,'Highly Ellip' from dual union all
select 25802,'025802','1991-010H','deb SOZ','',64179,'Reentered','2004-10-07 00:00:00','1999-06-22 00:00:00',571.47,433,32434,46.99,'Highly Ellip' from dual union all
select 25803,'025803','1987-079N','deb SOZ','',60169,'Lost','','1999-06-28 00:00:00',296.41,468,16227,65.29,'Highly Ellip' from dual union all
select 25804,'025804','1987-079P','deb SOZ','',60169,'In Earth orbit','','1999-06-28 00:00:00',291.65,461,15918,65.2,'Highly Ellip' from dual union all
select 25805,'025805','1987-079Q','deb SOZ','',60169,'In Earth orbit','','1999-06-18 00:00:00',311.03,440,17217,65.01,'Highly Ellip' from dual union all
select 25806,'025806','1987-079R','deb SOZ','',60169,'In Earth orbit','','1999-06-28 00:00:00',289.8,402,15854,64.72,'Highly Ellip' from dual union all
select 25807,'025807','1987-079S','deb SOZ','',60169,'In Earth orbit','','1999-06-28 00:00:00',329.74,488,18377,65.32,'Highly Ellip' from dual union all
select 25808,'025808','1987-079T','deb SOZ','',60169,'In Earth orbit','','1999-06-27 00:00:00',326.15,480,18155,65.31,'Highly Ellip' from dual union all
select 25809,'025809','1987-079U','deb SOZ','',60169,'In Earth orbit','','1999-06-23 00:00:00',288.41,451,15713,65.06,'Highly Ellip' from dual union all
select 2581,'002581','1966-101AJ','deb OGCh','',13508,'Reentered','1966-11-23 00:00:00','1966-11-14 00:00:00',94.26,107,859,49.78,'Intermediate' from dual union all
select 25810,'025810','1987-079V','deb SOZ','',60169,'In Earth orbit','','1999-06-23 00:00:00',281.33,386,15302,64.64,'Highly Ellip' from dual union all
select 25811,'025811','1989-101H','deb SOZ','',62378,'Reentered','2012-10-01 00:00:00','1999-06-07 00:00:00',488.38,553,27777,46.4,'Highly Ellip' from dual union all
select 25812,'025812','1989-101J','deb SOZ','',62378,'Reentered','2013-05-12 00:00:00','1999-06-28 00:00:00',490.22,555,27878,46.4,'Highly Ellip' from dual union all
select 25813,'025813','1989-101K','deb SOZ','',62378,'Reentered','2000-09-12 00:00:00','1999-06-27 00:00:00',219.29,238,11097,46.17,'Medium' from dual union all
select 25814,'025814','1989-101L','deb SOZ','',62378,'Reentered','2010-02-17 00:00:00','1999-06-26 00:00:00',488.38,532,27798,46.44,'Highly Ellip' from dual union all
select 25815,'025815','1989-101M','deb SOZ','',62378,'Reentered','2012-02-18 00:00:00','1999-05-25 00:00:00',488.45,545,27789,46.31,'Highly Ellip' from dual union all
select 25816,'025816','1989-101N','deb SOZ','',62378,'Reentered','2008-02-05 00:00:00','1999-06-28 00:00:00',382.64,290,21872,46.42,'Highly Ellip' from dual union all
select 25817,'025817','1989-101P','deb SOZ','',62378,'Reentered','1999-09-05 00:00:00','1999-06-22 00:00:00',215.68,1160,9911,46.13,'Medium' from dual union all
select 25818,'025818','1989-101Q','deb SOZ','',62378,'In Earth orbit','','1999-06-21 00:00:00',224.69,1237,10493,47.41,'Medium' from dual union all
select 25819,'025819','1994-027C','AS-4 debris','',65423,'Reentered','1999-08-26 00:00:00','1999-06-28 00:00:00',94.66,397,607,46.02,'Intermediate' from dual union all
select 2582,'002582','1966-101AK','deb OGCh','',13508,'Reentered','1966-11-14 00:00:00','1966-11-09 00:00:00',90.31,223,356,49.18,'Intermediate' from dual union all
select 25820,'025820','1998-042F','deb Shtil''','',66478,'Reentered','1999-07-24 00:00:00','1999-06-26 00:00:00',92.4,332,453,78.87,'Intermediate' from dual union all
select 25821,'025821','1984-023C','deb Ariane L8','',57077,'In Earth orbit','','1999-06-28 00:00:00',566.55,1099,31506,8.31,'Highly Ellip' from dual union all
select 25822,'025822','1984-023D','deb Ariane L8','',57077,'In Earth orbit','','1999-06-28 00:00:00',567.94,1014,31665,8.12,'Highly Ellip' from dual union all
select 25823,'025823','1984-023E','deb Ariane L8','',57077,'In Earth orbit','','1999-06-20 00:00:00',555.59,759,31259,8.26,'Highly Ellip' from dual union all
select 25824,'025824','1984-023F','deb Ariane L8','',57077,'In Earth orbit','','1999-06-28 00:00:00',561.95,829,31530,7.61,'Highly Ellip' from dual union all
select 25825,'025825','1966-040H','Nimbus 2 debris','',13447,'In Earth orbit','','1999-06-28 00:00:00',107.95,1083,1181,100.36,'Sun-Synch' from dual union all
select 25826,'025826','1966-077T','deb FTV 1352/Agena D','',13484,'In Earth orbit','','1999-06-27 00:00:00',164.81,1437,5722,88.56,'Medium' from dual union all
select 25827,'025827','1966-077U','deb FTV 1352/Agena D','',13484,'In Earth orbit','','1999-06-28 00:00:00',166.36,2334,4950,89.4,'Medium' from dual union all
select 25828,'025828','1966-077V','deb FTV 1352/Agena D','',13484,'In Earth orbit','','1999-06-28 00:00:00',164.76,1810,5345,89.1,'Medium' from dual union all
select 25829,'025829','1966-077W','deb FTV 1352/Agena D','',13484,'In Earth orbit','','1999-06-28 00:00:00',166.33,2395,4886,89.53,'Medium' from dual union all
select 2583,'002583','1966-101AL','deb OGCh','',13508,'Reentered','1966-11-14 00:00:00','1966-11-07 00:00:00',88.31,47,335,49.5,'ERROR - Unexpected value "TA"' from dual union all
select 25830,'025830','1970-067F','NNS O-19 insulation','',21544,'Reentered','2003-10-24 00:00:00','1999-06-28 00:00:00',106.25,922,1184,90.02,'Polar' from dual union all
select 25831,'025831','1978-100AT','deb S5M','',45295,'In Earth orbit','','1999-06-30 00:00:00',120.27,1683,1703,82.54,'Medium' from dual union all
select 25832,'025832','1978-100AU','deb S5M','',45295,'In Earth orbit','','1999-06-28 00:00:00',120.24,1684,1699,82.55,'Medium' from dual union all
select 25833,'025833','1978-100AV','deb S5M','',45295,'Reentered','2015-03-21 00:00:00','1999-06-27 00:00:00',116.5,1248,1798,82.22,'Intermediate' from dual union all
select 25834,'025834','1978-100AW','deb S5M','',45295,'In Earth orbit','','1999-06-27 00:00:00',116.24,1295,1729,82.26,'Intermediate' from dual union all
select 25835,'025835','1979-058J','deb Kosmos-1109','',47542,'Reentered','2012-11-17 00:00:00','1999-06-22 00:00:00',726.47,2371,38411,67.12,'GEO Transfer' from dual union all
select 25836,'025836','1979-058K','deb Kosmos-1109','',47542,'Reentered','2012-02-13 00:00:00','1999-06-21 00:00:00',726.8,2608,38191,67.52,'GEO Transfer' from dual union all
select 25837,'025837','1979-104L','deb Ariane L01','',47588,'In Earth orbit','','1999-06-28 00:00:00',270.35,323,14621,18,'Highly Ellip' from dual union all
select 25838,'025838','1981-071G','deb Kosmos-1285','',52594,'In Earth orbit','','1999-06-24 00:00:00',726.47,2445,38337,66.67,'GEO Transfer' from dual;
insert into SATELLITE
select 25839,'025839','1981-071H','deb Kosmos-1285','',52594,'In Earth orbit','','1999-06-14 00:00:00',726.09,2513,38251,66.74,'GEO Transfer' from dual union all
select 2584,'002584','1966-101AM','deb OGCh','',13508,'Reentered','1966-11-12 00:00:00','1966-11-07 00:00:00',89.55,113,392,49.81,'Intermediate' from dual union all
select 25840,'025840','1981-071J','deb Kosmos-1285','',52594,'In Earth orbit','','1999-06-24 00:00:00',727.23,2367,38453,66.54,'GEO Transfer' from dual union all
select 25841,'025841','1981-071K','deb Kosmos-1285','',52594,'In Earth orbit','','1999-06-25 00:00:00',727.21,2496,38323,66.55,'GEO Transfer' from dual union all
select 25842,'025842','1983-072C','deb SGS yo weight','',55829,'In Earth orbit','','1999-06-24 00:00:00',367.44,1857,19374,65.38,'Highly Ellip' from dual union all
select 25843,'025843','1989-062G','deb Ariane V33','H10 large debris',62339,'Reentered','2015-11-05 00:00:00','1999-06-22 00:00:00',531.29,357,30346,6.83,'Highly Ellip' from dual union all
select 25844,'025844','1989-062H','deb Ariane V33','H10 deb',62339,'In Earth orbit','','1999-06-28 00:00:00',603.86,350,34225,7.55,'GEO Transfer' from dual union all
select 25845,'025845','1988-098G','deb Ariane V26','',61347,'Reentered','2006-06-12 00:00:00','1999-06-30 00:00:00',409.71,232,23558,4.39,'Highly Ellip' from dual union all
select 25846,'025846','1998-067D','EVA Tether','',66503,'Reentered','1999-08-18 00:00:00','1999-07-05 00:00:00',91.57,350,354,51.59,'Intermediate' from dual union all
select 25847,'025847','1999-036A','Molniya-3','Molniya-3 No. 63',66717,'In Earth orbit','','1999-08-07 00:00:00',717.78,470,39885,62.87,'Molniya' from dual union all
select 25848,'025848','1999-036B','11S510','11S510',66717,'Reentered','1999-07-24 00:00:00','1999-07-15 00:00:00',89.73,202,321,62.81,'Intermediate' from dual union all
select 25849,'025849','1999-036C','BOZ','BOZ',66717,'Reentered','1999-07-21 00:00:00','1999-07-10 00:00:00',90.65,186,426,62.81,'Intermediate' from dual union all
select 2585,'002585','1966-101AN','deb OGCh','',13508,'Reentered','1966-11-14 00:00:00','1966-11-07 00:00:00',90.32,118,463,49.62,'Intermediate' from dual union all
select 25850,'025850','1999-036D','Blok-ML','Blok-ML No. 77347-547',66717,'In Earth orbit','','1999-08-12 00:00:00',732.34,475,40596,62.86,'Molniya' from dual union all
select 25851,'025851','1999-037A','Globalstar M035','Globalstar M035',66718,'In Earth orbit','','1999-08-09 00:00:00',114.06,1413,1413,52,'Intermediate' from dual union all
select 25852,'025852','1999-037B','Globalstar M032','Globalstar M032',66718,'In Earth orbit','','1999-08-09 00:00:00',114.06,1413,1413,52,'Intermediate' from dual union all
select 25853,'025853','1999-037C','Globalstar M051','Globalstar M051',66718,'In Earth orbit','','1999-08-09 00:00:00',114.06,1412,1413,52,'Intermediate' from dual union all
select 25854,'025854','1999-037D','Globalstar M030','Globalstar M030',66718,'In Earth orbit','','1999-08-09 00:00:00',114.06,1413,1413,52,'Intermediate' from dual union all
select 25855,'025855','1999-037E','Delta 272','Delta SSPS AJ-10-118K',66718,'Reentered','2003-05-16 00:00:00','1999-08-09 00:00:00',101.78,332,1354,52.18,'Intermediate' from dual union all
select 25856,'025856','1997-005C','KM-V1 adapter part','',66085,'Reentered','1999-07-17 00:00:00','1999-07-12 00:00:00',102.15,186,1535,31.28,'Intermediate' from dual union all
select 25857,'025857','1997-005D','KM-V1 adapter part','',66085,'Reentered','1999-07-25 00:00:00','1999-07-12 00:00:00',103.41,167,1673,31.19,'Intermediate' from dual union all
select 25858,'025858','1999-038A','Progress M-42','Progress 7K-TGM No 242',66719,'Deorbited','2000-02-02 00:00:00','1999-08-15 00:00:00',91.41,340,348,51.66,'Intermediate' from dual union all
select 25859,'025859','1999-038B','11S510','11S510',66719,'Reentered','1999-07-19 00:00:00','1999-07-18 00:00:00',88.18,173,196,51.64,'Intermediate' from dual union all
select 2586,'002586','1966-101AP','deb OGCh','',13508,'Reentered','1966-11-12 00:00:00','1966-11-06 00:00:00',92.22,93,674,49.49,'Intermediate' from dual union all
select 25860,'025860','1999-039A','Okean-O','Okean-O No. 1',66720,'In Earth orbit','','1999-08-16 00:00:00',97.97,660,664,98.05,'Sun-Synch' from dual union all
select 25861,'025861','1999-039B','11S772','11S772',66720,'In Earth orbit','','1999-08-16 00:00:00',97.74,640,662,98.05,'Sun-Synch' from dual union all
select 25862,'025862','1999-039C','Sep Motor Cover','Sep Motor Cover',66720,'In Earth orbit','','1999-08-16 00:00:00',100.6,653,920,97.77,'Sun-Synch' from dual union all
select 25863,'025863','1999-039D','Sep Motor Cover','Sep Motor Cover',66720,'In Earth orbit','','1999-08-16 00:00:00',100.44,652,907,98.32,'Sun-Synch' from dual union all
select 25864,'025864','1999-039E','Sep Motor Cover','Sep Motor Cover',66720,'In Earth orbit','','1999-08-16 00:00:00',100.8,659,934,98.33,'Sun-Synch' from dual union all
select 25865,'025865','1999-039F','Sep Motor Cover','Sep Motor Cover',66720,'In Earth orbit','','1999-08-16 00:00:00',101.05,659,958,97.8,'Sun-Synch' from dual union all
select 25866,'025866','1999-040A','Columbia (STS-93)','OV-102',66721,'Landed','1999-07-28 00:00:00','1999-07-25 00:00:00',90.2,272,297,28.47,'Intermediate' from dual union all
select 25867,'025867','1999-040B','Chandra X-ray Observatory','AXAF',66721,'In Earth orbit','','1999-08-27 00:00:00',3808.91,9998,138826,28.56,'Very High' from dual union all
select 25868,'025868','1999-040C','IUS-27 SRM-1','Orbus 21',66721,'In Earth orbit','','1999-08-22 00:00:00',257.88,272,13812,28.47,'Highly Ellip' from dual union all
select 25869,'025869','1999-040D','IUS-27 SRM-2','Orbus 6S',66721,'In Earth orbit','','1999-08-23 00:00:00',1481.82,435,72919,28.64,'Near-synch' from dual union all
select 2587,'002587','1966-101AQ','deb OGCh','',13508,'Reentered','1966-11-16 00:00:00','1966-11-09 00:00:00',99.01,144,1279,49.72,'Intermediate' from dual union all
select 25870,'025870','1986-017NH','EO-27 EVA-2 debris','',59210,'Reentered','1999-07-28 00:00:00','1999-07-24 00:00:00',91.39,341,345,51.66,'Intermediate' from dual union all
select 25871,'025871','1986-017NJ','EO-27 EVA-2 debris','',59210,'Reentered','1999-07-25 00:00:00','1999-07-24 00:00:00',91.49,346,350,51.67,'Intermediate' from dual union all
select 25872,'025872','1999-041A','Globalstar M048','Globalstar M048',66722,'In Earth orbit','','1999-08-24 00:00:00',114.06,1412,1413,52,'Intermediate' from dual union all
select 25873,'025873','1999-041B','Globalstar M026','Globalstar M026',66722,'In Earth orbit','','1999-08-24 00:00:00',114.06,1413,1413,52.01,'Intermediate' from dual union all
select 25874,'025874','1999-041C','Globalstar M043','Globalstar M043',66722,'In Earth orbit','','1999-08-24 00:00:00',114.06,1413,1413,52,'Intermediate' from dual union all
select 25875,'025875','1999-041D','Globalstar M028','Globalstar M028',66722,'In Earth orbit','','1999-08-24 00:00:00',114.06,1412,1413,52,'Intermediate' from dual union all
select 25876,'025876','1999-041E','Delta 273','Delta SSPS AJ-10-118K',66722,'In Earth orbit','','1999-08-24 00:00:00',103.65,499,1364,51.78,'Intermediate' from dual union all
select 25877,'025877','1986-017NK','Reflektor','',59210,'Reentered','1999-07-31 00:00:00','1999-07-28 00:00:00',91.38,340,345,51.66,'Intermediate' from dual union all
select 25878,'025878','1986-017NL','EO-27 EVA-3 debris','',59210,'Reentered','1999-08-03 00:00:00','1999-07-28 00:00:00',90.92,317,323,51.66,'Intermediate' from dual union all
select 25879,'025879','1986-017NM','EO-27 EVA-3 debris','',59210,'Reentered','1999-08-02 00:00:00','1999-07-29 00:00:00',91.18,330,335,51.67,'Intermediate' from dual union all
select 2588,'002588','1966-104F','Gemini XII ELSS','',13511,'Reentered','1966-12-04 00:00:00','1966-11-20 00:00:00',89.88,258,279,28.84,'Intermediate' from dual union all
select 25880,'025880','1999-042A','Telkom 1','Telkom 1',66723,'In Earth orbit','','1999-09-11 00:00:00',1436.06,35779,35793,.02,'Stationary' from dual union all
select 25881,'025881','1999-042B','Ariane H10-3 V118','Ariane H10-3 S/N T487',66723,'Reentered','2014-03-10 00:00:00','1999-09-11 00:00:00',655.43,242,36991,7.02,'GEO Transfer' from dual union all
select 25882,'025882','1970-034D','DFH-1 part','',21511,'In Earth orbit','','1999-08-10 00:00:00',108.15,432,1851,68.41,'Intermediate' from dual union all
select 25883,'025883','1999-043A','Globalstar M024','Globalstar M024',66724,'In Earth orbit','','1999-09-16 00:00:00',114.06,1412,1413,52.01,'Intermediate' from dual union all
select 25884,'025884','1999-043B','Globalstar M027','Globalstar M027',66724,'In Earth orbit','','1999-09-16 00:00:00',114.06,1412,1413,52.01,'Intermediate' from dual union all
select 25885,'025885','1999-043C','Globalstar M054','Globalstar M054',66724,'In Earth orbit','','1999-09-16 00:00:00',114.06,1412,1413,52.01,'Intermediate' from dual union all
select 25886,'025886','1999-043D','Globalstar M053','Globalstar M053',66724,'In Earth orbit','','1999-09-16 00:00:00',114.06,1412,1413,52.01,'Intermediate' from dual union all
select 25887,'025887','1999-043E','Delta 274','Delta SSPS AJ-10-118K',66724,'Reentered','2002-09-21 00:00:00','1999-09-16 00:00:00',101.6,319,1349,52.31,'Intermediate' from dual union all
select 25888,'025888','1987-068CJ','deb S5M','',60158,'Reentered','1999-01-12 00:00:00','1999-08-08 00:00:00',95.06,488,555,82.64,'Intermediate' from dual union all
select 25889,'025889','1999-044A','Kosmos-2365','Kobal''t No. 553',66725,'Landed','1999-12-15 00:00:00','1999-09-17 00:00:00',89.72,184,338,67.12,'Intermediate' from dual union all
select 2589,'002589','1966-104G','Gemini SC12 Equipment Module','Gemini SC12 Adapter Modu',13511,'Reentered','1966-12-01 00:00:00','1966-11-18 00:00:00',89.84,251,283,28.85,'Intermediate' from dual union all
select 25890,'025890','1999-044B','11S510','11S510',66725,'Reentered','1999-08-22 00:00:00','1999-08-20 00:00:00',88.95,158,288,67.14,'Intermediate' from dual union all
select 25891,'025891','1999-044C','Kosmos-2365 part','Kobal''t part',66725,'Reentered','1999-08-19 00:00:00','1999-08-18 00:00:00',89.44,163,331,67.1,'Intermediate' from dual union all
select 25892,'025892','1999-045A','Kosmos-2366','Parus',66726,'In Earth orbit','','1999-09-25 00:00:00',104.79,965,1005,82.93,'Intermediate' from dual union all
select 25893,'025893','1999-045B','S3M','S3M',66726,'In Earth orbit','','1999-09-25 00:00:00',104.65,961,996,82.92,'Intermediate' from dual union all
select 25894,'025894','1999-046A','Mugunghwa 3','Koreasat 3',66727,'In Earth orbit','','1999-10-04 00:00:00',1436.11,35782,35792,.01,'Stationary' from dual union all
select 25895,'025895','1999-046B','Ariane H10-3 V120','Ariane H10-3 S/N T488',66727,'Reentered','2003-06-06 00:00:00','1999-10-04 00:00:00',659.9,220,37240,7.01,'GEO Transfer' from dual union all
select 25896,'025896','1999-047A','Yamal-101','Yamal-100 No. 1',66728,'In Earth orbit','','1999-10-07 00:00:00',1442.02,35502,36303,.04,'Drift GEO' from dual union all
select 25897,'025897','1999-047B','Yamal-102','Yamal-100 No. 2',66728,'In Earth orbit','','1999-10-07 00:00:00',1441.97,35501,36302,.05,'Drift GEO' from dual union all
select 25898,'025898','1999-047C','8S812','8S812',66728,'Reentered','1999-09-07 00:00:00','1999-09-06 00:00:00',87.74,130,195,51.65,'Intermediate' from dual union all
select 25899,'025899','1999-047D','Perekhodnik','Sredniy perekhodnik',66728,'Reentered','1999-09-07 00:00:00','1999-09-06 00:00:00',87.67,140,178,51.64,'Intermediate' from dual union all
select 259,'000259','62 Eta 3','S.T. LOW','AFTRACK 21 (S.T. LOW)',8189,'Reentered Attached','1962-11-03 00:00:00','1962-03-07 00:00:00',90.67,235,380,90.82,'Polar' from dual union all
select 2590,'002590','1966-104H','Gemini XII handrail','',13511,'Reentered','1966-11-18 00:00:00','1966-11-16 00:00:00',89.97,258,288,28.85,'Intermediate' from dual union all
select 25900,'025900','1999-047E','Blok DM-2M No. 4L','11S861-01 No. 4L',66728,'In Earth orbit','','1999-10-16 00:00:00',1438.44,35366,36299,.05,'Drift GEO' from dual union all
select 25901,'025901','1999-047F','Adapter part','',66728,'Reentered','1999-09-07 00:00:00','1999-09-07 00:00:00',90.62,78,532,51.06,'ERROR - Unexpected value "TA"' from dual union all
select 25902,'025902','1999-048A','Foton','Foton 34KS No. 12',66729,'Landed','1999-09-24 00:00:00','1999-09-16 00:00:00',90.42,215,376,62.8,'Intermediate' from dual union all
select 25903,'025903','1999-048B','11S510','11S510',66729,'Reentered','1999-09-25 00:00:00','1999-09-16 00:00:00',89.91,206,334,62.78,'Intermediate' from dual union all
select 25904,'025904','1994-016D','Star 48 yo weight','',65412,'In Earth orbit','','1999-09-15 00:00:00',268.26,199,14601,34.69,'Highly Ellip' from dual union all
select 25905,'025905','1995-058J','Zenit part','',65719,'Reentered','2000-03-09 00:00:00','1999-09-20 00:00:00',99.61,732,748,71.01,'Intermediate' from dual union all
select 25906,'025906','1993-059J','deb 11S772','',65161,'Reentered','2000-04-21 00:00:00','1999-09-08 00:00:00',100.07,761,763,71.02,'Intermediate' from dual union all
select 25907,'025907','1999-049A','Globalstar M058','Globalstar FM58',66730,'In Earth orbit','','1999-10-23 00:00:00',103.59,901,956,51.98,'Intermediate' from dual union all
select 25908,'025908','1999-049B','Globalstar M050','Globalstar FM50',66730,'In Earth orbit','','1999-10-22 00:00:00',114.06,1412,1413,52,'Intermediate' from dual union all
select 25909,'025909','1999-049C','Globalstar M033','Globalstar FM33',66730,'In Earth orbit','','1999-10-22 00:00:00',103.57,899,956,51.99,'Intermediate' from dual union all
select 2591,'002591','1966-104J','Gemini XII EVA tether','',13511,'Reentered','1966-11-18 00:00:00','1966-11-16 00:00:00',89.37,224,263,28.87,'Intermediate' from dual union all
select 25910,'025910','1999-049D','Globalstar M055','Globalstar FM55',66730,'In Earth orbit','','1999-10-22 00:00:00',114.07,1412,1414,52,'Intermediate' from dual union all
select 25911,'025911','1999-049E','11S510','11S510',66730,'Reentered','2000-02-17 00:00:00','1999-10-22 00:00:00',95.28,231,834,51.97,'Intermediate' from dual union all
select 25912,'025912','1999-049F','Ikar No. 4','Ikar 50KS No. M15000-04',66730,'Deorbited','1999-09-23 00:00:00','1999-09-22 00:00:00',103.57,898,957,51.96,'Intermediate' from dual union all
select 25913,'025913','1999-050A','Echostar V','Echostar 5',66731,'In Earth orbit','','1999-10-26 00:00:00',1436.09,35766,35807,.11,'Stationary' from dual union all
select 25914,'025914','1999-050B','Centaur AC-155','Centaur IIA AC-155',66731,'Reentered','1999-11-30 00:00:00','1999-10-23 00:00:00',768.89,102,42745,26.4,'Highly Ellip' from dual union all
select 25915,'025915','1999-048C','PTDU cover','PTDU cover',66729,'Reentered','1999-09-30 00:00:00','1999-09-24 00:00:00',92.76,324,496,62.82,'Intermediate' from dual union all
select 25916,'025916','1999-048D','PTDU cover','PTDU cover',66729,'Reentered','1999-10-11 00:00:00','1999-09-24 00:00:00',92.56,323,477,62.8,'Intermediate' from dual union all
select 25917,'025917','1999-048E','KB electropitaniya','KBE',66729,'Reentered','1999-10-22 00:00:00','1999-09-24 00:00:00',90.27,214,362,62.78,'Intermediate' from dual union all
select 25918,'025918','1999-048F','KBE part','KBE part',66729,'Reentered','1999-10-03 00:00:00','1999-09-24 00:00:00',90.09,211,347,62.8,'Intermediate' from dual union all
select 25919,'025919','1999-051A','Ikonos','Ikonos 2',66732,'In Earth orbit','','1999-10-24 00:00:00',98.34,677,682,98.19,'Sun-Synch' from dual union all
select 2592,'002592','1966-104K','Gemini XII EVA tether','',13511,'Reentered','1966-11-18 00:00:00','1966-11-16 00:00:00',89.44,243,250,28.84,'Intermediate' from dual union all
select 25920,'025920','1999-051B','OAM','OAM',66732,'Reentered','1999-09-28 00:00:00','1999-09-26 00:00:00',91.77,135,587,97.74,'Sun-Synch' from dual union all
select 25921,'025921','1999-051C','Orbus 21D','Orbus 21D',66732,'Reentered','1999-10-25 00:00:00','1999-10-02 00:00:00',90.6,204,404,98.19,'Sun-Synch' from dual union all
select 25922,'025922','1999-052A','Telstar 7','Telstar 7',66733,'In Earth orbit','','1999-10-28 00:00:00',1436.04,35773,35798,.02,'Stationary' from dual union all
select 25923,'025923','1999-052B','Ariane H10-3 V121','Ariane H10-3 S/N T489',66733,'Reentered','1999-12-10 00:00:00','1999-10-25 00:00:00',1237.8,110,63510,6.68,'Highly Ellip' from dual union all
select 25924,'025924','1999-053A','LMI 1','LMI 1',66734,'In Earth orbit','','1999-10-27 00:00:00',1436.06,35780,35792,.06,'Stationary' from dual union all
select 25925,'025925','1999-053B','8S812','8S812',66734,'Reentered','1999-09-28 00:00:00','1999-09-27 00:00:00',88.07,153,205,51.59,'Intermediate' from dual union all
select 25926,'025926','1999-053C','Perekhodnik','Sredniy perekhodnik',66734,'Reentered','1999-09-27 00:00:00','1999-09-27 00:00:00',88.7,190,231,51.58,'Intermediate' from dual union all
select 25927,'025927','1999-053D','Blok DM3 No. 18L','Blok DM3 No. 18L',66734,'In Earth orbit','','1999-11-01 00:00:00',760.48,6519,35922,17.36,'Highly Ellip' from dual union all
select 25928,'025928','1991-015J','deb Ariane V42','',64184,'In Earth orbit','','1999-09-25 00:00:00',506.97,563,28803,7.1,'Highly Ellip' from dual;
insert into SATELLITE
select 25929,'025929','1999-054A','Resurs F-1M','Resurs F-1M',66735,'Landed','1999-10-21 00:00:00','1999-10-05 00:00:00',89.01,222,230,82.32,'Intermediate' from dual union all
select 2593,'002593','1966-104L','Gemini XII EVA debris','',13511,'Reentered','1966-11-18 00:00:00','1966-11-16 00:00:00',89.38,238,249,28.83,'Intermediate' from dual union all
select 25930,'025930','1999-054B','11S510','11S510',66735,'Reentered','1999-09-30 00:00:00','1999-09-29 00:00:00',88.24,173,202,82.33,'Intermediate' from dual union all
select 25931,'025931','1966-077X','deb FTV 1352','',13484,'In Earth orbit','','1999-10-04 00:00:00',165.88,1632,5613,88.78,'Medium' from dual union all
select 25932,'025932','1966-077Y','deb FTV 1352','',13484,'In Earth orbit','','1999-10-02 00:00:00',165.32,1307,5892,88.23,'Medium' from dual union all
select 25933,'025933','1999-055A','GPS IIR-3','Navstar GPS SVN 46',66736,'In Earth orbit','','1999-11-10 00:00:00',717.96,20096,20268,53.04,'Medium' from dual union all
select 25934,'025934','1999-055B','Delta 275','Delta SSPS AJ-10-118K',66736,'Reentered','1999-11-23 00:00:00','1999-10-20 00:00:00',95.52,186,902,33.34,'Intermediate' from dual union all
select 25935,'025935','1999-055C','Star 48B','Star 48B',66736,'Reentered','2011-03-06 00:00:00','1999-11-06 00:00:00',350.97,197,20011,38.88,'Highly Ellip' from dual union all
select 25936,'025936','1986-010D','deb CZ-3 Stage 3','',59203,'In Earth orbit','','1999-10-06 00:00:00',611.85,515,34477,31.36,'GEO Transfer' from dual union all
select 25937,'025937','1999-056A','DirecTV-1R','DirecTV-1R',66737,'In Earth orbit','','1999-11-09 00:00:00',1436.09,35781,35792,.03,'Stationary' from dual union all
select 25938,'025938','1999-056B','Blok DM-SL No. 3L','Blok DM-SL No. 3L',66737,'In Earth orbit','','1999-11-10 00:00:00',678.67,2694,35714,.62,'GEO Transfer' from dual union all
select 25939,'025939','1979-104M','deb Ariane L01','',47588,'Reentered','2000-06-19 00:00:00','1999-10-13 00:00:00',132.18,320,4115,17.95,'Medium' from dual union all
select 2594,'002594','1966-104M','Gemini XII S-13 bracket','',13511,'Reentered','1966-11-18 00:00:00','1966-11-16 00:00:00',89.64,251,263,28.83,'Intermediate' from dual union all
select 25940,'025940','1999-057A','Zi Yuan 1-1','ZY-1/CBERS 1',66738,'In Earth orbit','','1999-11-13 00:00:00',100.31,773,774,98.56,'Sun-Synch' from dual union all
select 25941,'025941','1999-057B','SACI-1','SACI-1',66738,'In Earth orbit','','1999-11-13 00:00:00',99.58,733,744,98.55,'Sun-Synch' from dual union all
select 25942,'025942','1999-057C','CZ-4B Stage 3','CZ-4B Stage 3',66738,'In Earth orbit','','1999-11-13 00:00:00',99.52,727,744,98.55,'Sun-Synch' from dual union all
select 25943,'025943','1999-058A','Globalstar M057','Globalstar M057',66739,'In Earth orbit','','1999-11-17 00:00:00',114.06,1411,1415,52,'Intermediate' from dual union all
select 25944,'025944','1999-058B','Globalstar M059','Globalstar M059',66739,'In Earth orbit','','1999-11-18 00:00:00',114.06,1412,1413,52.01,'Intermediate' from dual union all
select 25945,'025945','1999-058C','Globalstar M056','Globalstar M056',66739,'In Earth orbit','','1999-11-17 00:00:00',103.46,897,948,51.96,'Intermediate' from dual union all
select 25946,'025946','1999-058D','Globalstar M031','Globalstar M031',66739,'In Earth orbit','','1999-11-17 00:00:00',103.41,892,948,51.96,'Intermediate' from dual union all
select 25947,'025947','1999-058E','11S510','11S510',66739,'Reentered','2000-03-04 00:00:00','1999-11-17 00:00:00',95,231,807,51.96,'Intermediate' from dual union all
select 25948,'025948','1999-058F','Ikar No. 5','Ikar 50KS No. A15000-05',66739,'Deorbited','1999-10-20 00:00:00','1999-10-18 00:00:00',103.45,894,950,51.97,'Intermediate' from dual union all
select 25949,'025949','1999-059A','Orion 2','Orion 2',66740,'In Earth orbit','','1999-11-18 00:00:00',1436.01,35777,35793,.04,'Stationary' from dual union all
select 2595,'002595','1966-104N','Gemini XII EVA camera cable','',13511,'Reentered','1966-11-18 00:00:00','1966-11-16 00:00:00',89.55,244,261,28.84,'Intermediate' from dual union all
select 25950,'025950','1999-059B','Ariane H10-3 V122','Ariane H10-3 S/N T490',66740,'Reentered','2000-01-27 00:00:00','1999-11-18 00:00:00',1174.23,145,60837,6.57,'Highly Ellip' from dual union all
select 25951,'025951','1999-054C','PTDU cover','PTDU cover',66735,'Reentered','1999-10-23 00:00:00','1999-10-21 00:00:00',91.01,218,430,82.36,'Intermediate' from dual union all
select 25952,'025952','1999-054D','Blenda','Sunshade',66735,'Reentered','1999-10-22 00:00:00','1999-10-21 00:00:00',88.79,200,229,82.32,'Intermediate' from dual union all
select 25953,'025953','1999-054E','KDU','KDU',66735,'Reentered','1999-10-31 00:00:00','1999-10-22 00:00:00',88.98,212,237,82.32,'Intermediate' from dual union all
select 25954,'025954','1999-060A','GE 4','GE 4',66741,'In Earth orbit','','1999-12-13 00:00:00',1445.57,35959,35985,.05,'Drift GEO' from dual union all
select 25955,'025955','1999-060B','Ariane H10-3 V123','Ariane H10-3 S/N T491',66741,'Reentered','2005-10-15 00:00:00','1999-11-25 00:00:00',662.8,221,37386,5.9,'GEO Transfer' from dual union all
select 25956,'025956','1999-061A','Shenzhou','Shenzhou 1 Fanhui Cang',66742,'Landed','1999-11-20 00:00:00','1999-11-20 00:00:00',91.53,193,507,42.48,'Intermediate' from dual union all
select 25957,'025957','1999-061B','CZ-2F Stage 2','CZ-2F Stage 2',66742,'Reentered','1999-11-27 00:00:00','1999-11-21 00:00:00',89.82,205,327,42.61,'Intermediate' from dual union all
select 25958,'025958','1999-061C','CZ-2F sep motor cover','',66742,'Reentered','1999-11-21 00:00:00','1999-11-20 00:00:00',90.96,186,458,42.72,'Intermediate' from dual union all
select 25959,'025959','1999-061D','CZ-2F sep motor cover','',66742,'Reentered','1999-11-22 00:00:00','1999-11-21 00:00:00',90.4,181,408,42.47,'Intermediate' from dual union all
select 2596,'002596','1966-104P','Gemini XII EVA debris','',13511,'Reentered','1966-11-18 00:00:00','1966-11-16 00:00:00',89.38,149,339,29.05,'Intermediate' from dual union all
select 25960,'025960','1999-061E','Shenzhou 1 Guidao Cang','Shenzhou 1 orbital modul',66742,'Reentered','1999-12-01 00:00:00','1999-11-21 00:00:00',89.63,195,318,42.6,'Intermediate' from dual union all
select 25961,'025961','1999-062A','Globalstar M039','Globalstar M039',66743,'In Earth orbit','','1999-12-23 00:00:00',103.39,897,941,51.96,'Intermediate' from dual union all
select 25962,'025962','1999-062B','Globalstar M034','Globalstar M034',66743,'In Earth orbit','','1999-12-22 00:00:00',114.06,1412,1413,51.99,'Intermediate' from dual union all
select 25963,'025963','1999-062C','Globalstar M029','Globalstar M029',66743,'In Earth orbit','','1999-12-23 00:00:00',103.43,900,942,51.97,'Intermediate' from dual union all
select 25964,'025964','1999-062D','Globalstar M061','Globalstar M061',66743,'In Earth orbit','','1999-12-22 00:00:00',114.07,1412,1414,52,'Intermediate' from dual union all
select 25965,'025965','1999-062E','11S510','11S510',66743,'Reentered','2000-03-19 00:00:00','1999-12-22 00:00:00',94.86,226,798,51.97,'Intermediate' from dual union all
select 25966,'025966','1999-062F','Ikar No. 6','Ikar 50KS',66743,'Deorbited','1999-11-24 00:00:00','1999-11-23 00:00:00',103.41,899,941,51.96,'Intermediate' from dual union all
select 25967,'025967','1999-063A','UHF F/O F10','UFO 10',66744,'In Earth orbit','','1999-12-23 00:00:00',1436.06,34942,36630,6.02,'Inclined GEO' from dual union all
select 25968,'025968','1999-063B','Centaur AC-136','Centaur IIA AC-136',66744,'In Earth orbit','','1999-12-01 00:00:00',454.15,271,26116,26.96,'Highly Ellip' from dual union all
select 25969,'025969','1997-079C','deb Kosmos-2347','',66159,'Reentered','1999-11-29 00:00:00','1999-11-28 00:00:00',91.18,283,382,64.92,'Intermediate' from dual union all
select 2597,'002597','1966-104Q','Gemini XII EVA debris','',13511,'Reentered','1966-11-18 00:00:00','1966-11-16 00:00:00',88.15,87,279,29.05,'Intermediate' from dual union all
select 25970,'025970','1997-079D','deb Kosmos-2347','',66159,'Reentered','1999-12-10 00:00:00','1999-11-30 00:00:00',91.39,291,395,64.99,'Intermediate' from dual union all
select 25971,'025971','1997-079E','deb Kosmos-2347','',66159,'Reentered','1999-11-30 00:00:00','1999-11-29 00:00:00',89.73,230,293,64.96,'Intermediate' from dual union all
select 25972,'025972','1997-079F','deb Kosmos-2347','',66159,'Reentered','1999-11-30 00:00:00','1999-11-29 00:00:00',88.53,169,235,65.02,'Intermediate' from dual union all
select 25973,'025973','1997-079G','deb Kosmos-2347','',66159,'Reentered','1999-11-28 00:00:00','1999-11-26 00:00:00',94.17,334,623,64.85,'Intermediate' from dual union all
select 25974,'025974','1997-079H','deb Kosmos-2347','',66159,'Reentered','2000-01-08 00:00:00','1999-11-30 00:00:00',91.91,310,427,65.2,'Intermediate' from dual union all
select 25975,'025975','1997-079J','deb Kosmos-2347','',66159,'Reentered','1999-12-09 00:00:00','1999-11-30 00:00:00',93.4,345,536,65.02,'Intermediate' from dual union all
select 25976,'025976','1997-079K','deb Kosmos-2347','',66159,'Reentered','1999-11-29 00:00:00','1999-11-28 00:00:00',87.91,151,191,64.99,'Intermediate' from dual union all
select 25977,'025977','1999-064A','Helios 1B','Helios 1B',66745,'In Earth orbit','','1999-12-10 00:00:00',98.18,669,675,98.08,'Sun-Synch' from dual union all
select 25978,'025978','1999-064B','Clementine','Clementine',66745,'In Earth orbit','','1999-12-10 00:00:00',97.84,647,664,98.08,'Sun-Synch' from dual union all
select 25979,'025979','1999-064C','Ariane H10-3 V124','Ariane H10-3 S/N T493',66745,'In Earth orbit','','1999-12-11 00:00:00',97.88,651,664,98.1,'Sun-Synch' from dual union all
select 2598,'002598','1966-101AR','deb OGCh','',13508,'Reentered','1966-11-17 00:00:00','1966-11-08 00:00:00',90.48,123,474,49.69,'Intermediate' from dual union all
select 25980,'025980','1999-065A','Orbcomm D2','Orbcomm FM30',66746,'In Earth orbit','','2000-01-19 00:00:00',101.48,825,833,45.04,'Intermediate' from dual union all
select 25981,'025981','1999-065B','Orbcomm D3','Orbcomm FM31',66746,'In Earth orbit','','2000-01-19 00:00:00',101.48,825,833,45.04,'Intermediate' from dual union all
select 25982,'025982','1999-065C','Orbcomm D4','Orbcomm FM32',66746,'In Earth orbit','','2000-01-20 00:00:00',101.45,823,832,45.03,'Intermediate' from dual union all
select 25983,'025983','1999-065D','Orbcomm D5','Orbcomm FM33',66746,'In Earth orbit','','2000-01-19 00:00:00',101.44,824,830,45.04,'Intermediate' from dual union all
select 25984,'025984','1999-065E','Orbcomm D8','Orbcomm FM36',66746,'In Earth orbit','','2000-01-20 00:00:00',101.45,824,830,45.03,'Intermediate' from dual union all
select 25985,'025985','1999-065F','Orbcomm D7','Orbcomm FM35',66746,'In Earth orbit','','2000-01-19 00:00:00',101.43,824,829,45.04,'Intermediate' from dual union all
select 25986,'025986','1999-065G','Orbcomm D6','Orbcomm FM34',66746,'In Earth orbit','','2000-01-19 00:00:00',101.43,824,829,45.04,'Intermediate' from dual union all
select 25987,'025987','1999-065H','Orion 38','Orion 38',66746,'Reentered','2002-04-19 00:00:00','1999-12-11 00:00:00',95.98,407,725,45,'Intermediate' from dual union all
select 25988,'025988','1999-065J','HAPS','HAPS',66746,'Reentered','2002-02-01 00:00:00','1999-12-11 00:00:00',96.65,371,826,40.98,'Intermediate' from dual union all
select 25989,'025989','1999-066A','XMM-Newton','XMM',66747,'In Earth orbit','','2000-01-09 00:00:00',2872.14,7078,114027,38.49,'Very High' from dual union all
select 2599,'002599','1966-106A','Kosmos-132','Zenit-2 No. 46',13513,'Landed','1966-11-27 00:00:00','1966-11-21 00:00:00',89.17,201,266,64.99,'Intermediate' from dual union all
select 25990,'025990','1999-066B','EPS 504','EPS S/N 506',66747,'In Earth orbit','','2000-01-12 00:00:00',2603.78,789,111844,38.71,'Very High' from dual union all
select 25991,'025991','1999-067A','DMSP F-15','DMSP Block 5D-3 S-15',66748,'In Earth orbit','','1999-12-19 00:00:00',101.8,837,851,98.9,'Sun-Synch' from dual union all
select 25992,'025992','1999-067B','DMSP F-15 solar array band','',66748,'In Earth orbit','','1999-12-19 00:00:00',101.78,838,848,98.9,'Sun-Synch' from dual union all
select 25993,'025993','1999-067C','DMSP F-15 solar array band','',66748,'In Earth orbit','','1999-12-20 00:00:00',101.81,840,849,98.9,'Sun-Synch' from dual union all
select 25994,'025994','1999-068A','Terra','EOS AM-1',66749,'In Earth orbit','','2000-01-17 00:00:00',98.15,655,686,98.24,'Sun-Synch' from dual union all
select 25995,'025995','1999-068B','Centaur AC-141','Centaur IIA C-141',66749,'Reentered','2000-09-21 00:00:00','1999-12-25 00:00:00',94.64,358,645,98.26,'Sun-Synch' from dual union all
select 25996,'025996','1999-069A','Discovery (STS-103)','OV-103',66750,'Landed','1999-12-28 00:00:00','1999-12-21 00:00:00',96.26,570,589,28.47,'Intermediate' from dual union all
select 25997,'025997','1975-052BA','deb Delta 111','',36499,'In Earth orbit','','1999-12-20 00:00:00',103.35,755,1079,99.98,'Sun-Synch' from dual union all
select 25998,'025998','1991-068H','deb S5M','',64237,'In Earth orbit','','1999-12-19 00:00:00',115.05,1419,1496,82.66,'Intermediate' from dual union all
select 25999,'025999','1991-068J','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',114.23,1374,1467,82.7,'Intermediate' from dual union all
select 26,'000026','59 Epsilon 2','SRV 111','SRV',5528,'Reentered','1961-02-11 00:00:00','1960-05-01 00:00:00',102.19,194,1531,78.94,'Intermediate' from dual union all
select 260,'000260','61 Omicron 79','deb r/b Ablestar','',7273,'In Earth orbit','','1962-03-06 00:00:00',103.31,841,989,66.76,'Intermediate' from dual union all
select 2600,'002600','1966-106B','8A92E','8A92E',13513,'Reentered','1966-11-27 00:00:00','1966-11-21 00:00:00',89.19,215,254,65.01,'Intermediate' from dual union all
select 26000,'026000','1991-068K','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',115.74,1453,1525,82.73,'Intermediate' from dual union all
select 26001,'026001','1991-068L','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',113.66,1322,1467,82.23,'Intermediate' from dual union all
select 26002,'026002','1991-068M','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',114.85,1420,1478,82.68,'Intermediate' from dual union all
select 26003,'026003','1991-068N','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',115.6,1447,1518,82.71,'Intermediate' from dual union all
select 26004,'026004','1991-068P','deb S5M','',64237,'In Earth orbit','','1999-12-18 00:00:00',115.12,1403,1519,82.63,'Intermediate' from dual union all
select 26005,'026005','1991-068Q','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',114.25,1378,1464,82.61,'Intermediate' from dual union all
select 26006,'026006','1991-068R','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',114.07,1338,1489,82.61,'Intermediate' from dual union all
select 26007,'026007','1991-068S','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',115.34,1461,1481,82.7,'Intermediate' from dual union all
select 26008,'026008','1991-068T','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',114,1338,1482,82.63,'Intermediate' from dual union all
select 26009,'026009','1991-068U','deb S5M','',64237,'In Earth orbit','','1999-12-21 00:00:00',113.04,1272,1461,82.5,'Intermediate' from dual union all
select 2601,'002601','1966-107A','Kosmos-133','Soyuz 7K-OK No. 2',13514,'Reentered','1966-11-30 00:00:00','1966-11-29 00:00:00',88.41,173,219,51.83,'Intermediate' from dual union all
select 26010,'026010','1991-068V','deb S5M','',64237,'In Earth orbit','','1999-12-15 00:00:00',115.77,1450,1530,82.74,'Intermediate' from dual union all
select 26011,'026011','1991-068W','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',114.27,1378,1467,82.43,'Intermediate' from dual union all
select 26012,'026012','1991-068X','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',115.52,1447,1511,82.74,'Intermediate' from dual union all
select 26013,'026013','1991-068Y','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',114.79,1425,1467,82.72,'Intermediate' from dual union all
select 26014,'026014','1991-068Z','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',115.71,1454,1522,82.75,'Intermediate' from dual union all
select 26015,'026015','1991-068AA','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',114.11,1363,1467,82.48,'Intermediate' from dual union all
select 26016,'026016','1991-068AB','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',112.21,1144,1513,81.91,'Intermediate' from dual union all
select 26017,'026017','1991-068AC','deb S5M','',64237,'In Earth orbit','','1999-12-18 00:00:00',115.81,1442,1542,82.69,'Intermediate' from dual;
insert into SATELLITE
select 26018,'026018','1991-068AD','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',115.61,1470,1497,82.71,'Intermediate' from dual union all
select 26019,'026019','1991-068AE','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',116.61,1314,1743,82.65,'Intermediate' from dual union all
select 2602,'002602','1966-107B','11S510','11S510',13514,'Reentered','1966-11-29 00:00:00','1966-11-29 00:00:00',88.26,170,207,51.82,'Intermediate' from dual union all
select 26020,'026020','1991-068AF','deb S5M','',64237,'In Earth orbit','','1999-12-19 00:00:00',115.4,1438,1509,82.75,'Intermediate' from dual union all
select 26021,'026021','1991-068AG','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',112.09,1146,1499,81.78,'Intermediate' from dual union all
select 26022,'026022','1991-068AH','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',114.27,1369,1476,82.46,'Intermediate' from dual union all
select 26023,'026023','1991-068AJ','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',115.26,1453,1482,82.59,'Intermediate' from dual union all
select 26024,'026024','1991-068AK','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',117.1,1341,1759,82.7,'Intermediate' from dual union all
select 26025,'026025','1991-068AL','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',116.95,1343,1745,82.73,'Intermediate' from dual union all
select 26026,'026026','1991-068AM','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',114.07,1344,1483,82.51,'Intermediate' from dual union all
select 26027,'026027','1991-068AN','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',117.02,1456,1638,83.29,'Intermediate' from dual union all
select 26028,'026028','1991-068AP','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',115.64,1464,1505,82.7,'Intermediate' from dual union all
select 26029,'026029','1991-068AQ','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',114.66,1411,1469,82.57,'Intermediate' from dual union all
select 2603,'002603','1966-108A','Kosmos-134','Zenit-4 No. 26',13515,'Landed','1966-12-11 00:00:00','1966-12-05 00:00:00',89.46,205,291,64.96,'Intermediate' from dual union all
select 26030,'026030','1991-068AU','deb S5M','',64237,'In Earth orbit','','1999-12-20 00:00:00',114.11,1363,1467,82.51,'Intermediate' from dual union all
select 26031,'026031','1988-018E','deb Ariane V21','',61267,'Reentered','2004-06-30 00:00:00','1999-12-20 00:00:00',419.53,295,24077,6.46,'Highly Ellip' from dual union all
select 26032,'026032','1999-070A','Arirang-1','KOMPSAT',66751,'In Earth orbit','','2000-01-20 00:00:00',98.72,687,708,98.27,'Sun-Synch' from dual union all
select 26033,'026033','1999-070B','ACRIMSAT','ACRIMSAT',66751,'In Earth orbit','','2000-01-20 00:00:00',98.86,682,727,98.28,'Sun-Synch' from dual union all
select 26034,'026034','1999-070C','Orion 38','Orion 38',66751,'In Earth orbit','','2000-08-03 00:00:00',98.67,677,713,98.26,'Sun-Synch' from dual union all
select 26035,'026035','1999-070D','APC','Adapter',66751,'In Earth orbit','','1999-12-28 00:00:00',98.86,683,726,98.28,'Sun-Synch' from dual union all
select 26036,'026036','1975-052KA','deb Delta 111','',36499,'In Earth orbit','','1999-12-21 00:00:00',102.81,755,1029,100.01,'Sun-Synch' from dual union all
select 26037,'026037','1987-079W','deb SOZ','',60169,'In Earth orbit','','1999-12-21 00:00:00',332,456,18553,64.77,'Highly Ellip' from dual union all
select 26038,'026038','1999-071A','Galaxy 11','Galaxy 11',66752,'In Earth orbit','','2000-01-21 00:00:00',1438.36,33612,38050,.12,'Near-synch' from dual union all
select 26039,'026039','1999-071B','Ariane H10-3 V125','Ariane H10-3 S/N T492',66752,'Reentered','2014-04-11 00:00:00','1999-12-29 00:00:00',709.91,206,39760,5.55,'GEO Transfer' from dual union all
select 2604,'002604','1966-108B','11A57I','11A57I',13515,'Reentered','1966-12-10 00:00:00','1966-12-05 00:00:00',89.19,198,270,64.98,'Intermediate' from dual union all
select 26040,'026040','1999-072A','Kosmos-2367','US-PU/Konus-A',66753,'Reentered','2002-07-20 00:00:00','2000-01-25 00:00:00',92.76,403,417,65.03,'Intermediate' from dual union all
select 26041,'026041','1999-072B','11S692','11S692',66753,'Reentered','1999-12-26 00:00:00','1999-12-26 00:00:00',89.69,120,399,65,'Intermediate' from dual union all
select 26042,'026042','1999-073A','Kosmos-2368','Oko',66754,'In Earth orbit','','2000-01-26 00:00:00',717.74,577,39776,62.88,'Molniya' from dual union all
select 26043,'026043','1999-073B','11S510','11S510',66754,'Reentered','2000-02-07 00:00:00','2000-01-22 00:00:00',90.68,212,403,62.81,'Intermediate' from dual union all
select 26044,'026044','1999-073C','BOZ','BOZ',66754,'Reentered','2000-01-22 00:00:00','2000-01-22 00:00:00',87.46,130,168,62.82,'Intermediate' from dual union all
select 26045,'026045','1999-073D','Blok-2BL','Blok-2BL',66754,'In Earth orbit','','2000-01-21 00:00:00',705.19,658,39074,62.89,'Molniya' from dual union all
select 26046,'026046','1989-001M','deb SOZ','',62278,'Reentered','2002-04-05 00:00:00','2000-01-20 00:00:00',152.21,593,5537,64.54,'Medium' from dual union all
select 26047,'026047','1989-001N','deb SOZ','',62278,'Reentered','2000-08-13 00:00:00','2000-01-15 00:00:00',131.99,548,3871,64.37,'Medium' from dual union all
select 26048,'026048','1965-108AG','deb Transtage 8','',11858,'In Earth orbit','','2000-01-13 00:00:00',449.59,343,25782,26.56,'Highly Ellip' from dual union all
select 26049,'026049','1965-108AH','deb Transtage 8','',11858,'In Earth orbit','','2000-01-21 00:00:00',273.26,542,14599,27.19,'Highly Ellip' from dual union all
select 2605,'002605','1966-108C','deb Kosmos-134','',13515,'Reentered','1966-12-04 00:00:00','1966-12-04 00:00:00',89.24,186,287,65.06,'Intermediate' from dual union all
select 26050,'026050','1965-108AJ','deb Transtage 8','',11858,'In Earth orbit','','2000-01-20 00:00:00',534.69,1004,29884,26.57,'Highly Ellip' from dual union all
select 26051,'026051','1993-009D','Pegasus Stage 3 part','',65111,'Reentered','2000-12-29 00:00:00','2000-01-22 00:00:00',98.76,673,726,24.96,'Equatorial' from dual union all
select 26052,'026052','2000-001A','USA 148','DSCS III B-8',66903,'In Earth orbit','','2000-03-01 00:00:00',1436.27,35780,35800,1,'Stationary' from dual union all
select 26053,'026053','2000-001B','Centaur AC-138','Centaur IIA(4) AC-138',66903,'In Earth orbit','','2000-03-01 00:00:00',615.93,233,34971,25.97,'GEO Transfer' from dual union all
select 26054,'026054','2000-001C','IABS-9','IABS-9',66903,'In Earth orbit','','2000-01-21 00:00:00',1434.22,35700,35800,1,'Drift GEO' from dual union all
select 26055,'026055','1999-047G','SOZ','SOZ',66728,'Reentered','2000-01-25 00:00:00','2000-01-24 00:00:00',104.6,97,1855,47.49,'Intermediate' from dual union all
select 26056,'026056','2000-002A','Galaxy 10R','Galaxy XR',66904,'In Earth orbit','','2000-02-24 00:00:00',1432.97,35673,35778,.05,'Drift GEO' from dual union all
select 26057,'026057','2000-002B','Ariane H10-3 V126','Ariane H10-3 S/N T495',66904,'Reentered','2004-04-16 00:00:00','2000-03-01 00:00:00',560.79,179,32118,7.12,'Highly Ellip' from dual union all
select 26058,'026058','2000-003A','Zhongxing 22','Feng Huo 1',66905,'In Earth orbit','','2000-02-24 00:00:00',1436.16,35784,35792,.71,'Stationary' from dual union all
select 26059,'026059','2000-003B','CZ-3A Stage 3','CZ-3A Stage 3',66905,'Reentered','2001-03-23 00:00:00','2000-02-01 00:00:00',753.09,190,41893,25.35,'Highly Ellip' from dual union all
select 2606,'002606','1966-109A','OPS 1890','GAMBIT SV 985',13516,'Deorbited','1966-12-14 00:00:00','1966-12-09 00:00:00',89.52,135,367,104.65,'Retrograde' from dual union all
select 26060,'026060','1999-047H','SOZ','SOZ',66728,'Reentered','2000-03-15 00:00:00','2000-01-25 00:00:00',612.66,150,34884,47.82,'GEO Transfer' from dual union all
select 26061,'026061','2000-004A','Weber-OSCAR-39','JAWSAT',66906,'In Earth orbit','','2000-02-27 00:00:00',100.38,751,803,100.23,'Sun-Synch' from dual union all
select 26062,'026062','2000-004B','OCS','Optical Calibration Sphe',66906,'Reentered','2001-03-05 00:00:00','2000-02-26 00:00:00',100.3,748,797,100.23,'Sun-Synch' from dual union all
select 26063,'026063','2000-004C','OPAL-OSCAR-38','OPAL',66906,'In Earth orbit','','2000-02-26 00:00:00',100.4,750,804,100.23,'Sun-Synch' from dual union all
select 26064,'026064','2000-004D','Falconsat-1','Falconsat',66906,'In Earth orbit','','2000-02-26 00:00:00',100.41,750,806,100.23,'Sun-Synch' from dual union all
select 26065,'026065','2000-004E','ASUSAT-OSCAR-37','ASUSAT',66906,'In Earth orbit','','2000-02-27 00:00:00',100.4,750,804,100.23,'Sun-Synch' from dual union all
select 26066,'026066','2000-004F','Orion 38','Orion 38',66906,'In Earth orbit','','2000-02-26 00:00:00',100.34,750,800,100.23,'Sun-Synch' from dual union all
select 26067,'026067','2000-005A','Progress M1-1','Progress 7K-TGM No. 250',66907,'Deorbited','2000-04-26 00:00:00','2000-03-02 00:00:00',91.43,342,347,51.65,'Intermediate' from dual union all
select 26068,'026068','2000-005B','11S510','11S510',66907,'Reentered','2000-02-02 00:00:00','2000-02-01 00:00:00',88.3,181,200,51.62,'Intermediate' from dual union all
select 26069,'026069','2000-006A','Kosmos-2369','Tselina-2',66908,'In Earth orbit','','2000-03-04 00:00:00',101.92,846,854,71.01,'Intermediate' from dual union all
select 2607,'002607','1966-109B','Agena D 4833','Agena D',13516,'Reentered','1966-12-08 00:00:00','1966-12-06 00:00:00',89.12,132,330,104.6,'Retrograde' from dual union all
select 26070,'026070','2000-006B','11S772','11S772',66908,'In Earth orbit','','2000-02-11 00:00:00',101.78,830,856,71.01,'Intermediate' from dual union all
select 26071,'026071','2000-007A','Hispasat 1C','Hispasat 1C',66909,'In Earth orbit','','2000-03-07 00:00:00',1436.06,35770,35802,.06,'Stationary' from dual union all
select 26072,'026072','2000-007B','Centaur AC-158','Centaur IIA AC-158',66909,'In Earth orbit','','2000-02-15 00:00:00',823.76,212,45250,18.76,'Highly Ellip' from dual union all
select 26073,'026073','2000-006C','Sep Motor Cover','Sep Motor Cover',66908,'In Earth orbit','','2000-02-11 00:00:00',104.89,846,1132,71.08,'Intermediate' from dual union all
select 26074,'026074','2000-006D','Sep Motor Cover','Sep Motor Cover',66908,'In Earth orbit','','2000-02-11 00:00:00',105.24,848,1164,70.97,'Intermediate' from dual union all
select 26075,'026075','2000-006E','Sep Motor Cover','Sep Motor Cover',66908,'In Earth orbit','','2000-02-10 00:00:00',104.79,845,1125,71.07,'Intermediate' from dual union all
select 26076,'026076','1999-040E','IUS-27 SRM-1 motor slag','',66721,'In Earth orbit','','2005-07-16 00:00:00',242.51,311,12696,28.47,'Medium' from dual union all
select 26077,'026077','2000-006G','Sep Motor Cover','Sep Motor Cover',66908,'In Earth orbit','','2000-02-11 00:00:00',105.02,844,1147,70.98,'Intermediate' from dual union all
select 26078,'026078','1999-040F','IUS-27 SRM-1 motor slag','',66721,'In Earth orbit','','2005-05-27 00:00:00',243.12,297,12753,28.46,'Medium' from dual union all
select 26079,'026079','2000-004G','Picosat cover','',66906,'In Earth orbit','','2000-03-04 00:00:00',100.4,750,804,100.23,'Sun-Synch' from dual union all
select 2608,'002608','1966-110A','ATS 1','ATS B',13517,'In Earth orbit','','1967-04-03 00:00:00',1436.04,35780,35791,.1,'Stationary' from dual union all
select 26080,'026080','2000-004H','Picosat 1','MEMS 1',66906,'In Earth orbit','','2000-03-04 00:00:00',100.32,749,799,100.23,'Sun-Synch' from dual union all
select 26081,'026081','2000-008A','Globalstar M063','Globalstar 63',66910,'In Earth orbit','','2000-03-09 00:00:00',103.45,913,931,52,'Intermediate' from dual union all
select 26082,'026082','2000-008B','Globalstar M062','Globalstar 62',66910,'In Earth orbit','','2000-03-10 00:00:00',114.06,1412,1413,52,'Intermediate' from dual union all
select 26083,'026083','2000-008C','Globalstar M060','Globalstar 60',66910,'In Earth orbit','','2000-03-09 00:00:00',103.56,923,930,52.01,'Intermediate' from dual union all
select 26084,'026084','2000-008D','Globalstar M064','Globalstar 64',66910,'In Earth orbit','','2000-03-09 00:00:00',103.42,913,927,52,'Intermediate' from dual union all
select 26085,'026085','2000-008E','Delta 276','Delta SSPS AJ-10-118K',66910,'Reentered','2000-05-15 00:00:00','2000-03-13 00:00:00',94.66,219,785,49.55,'Intermediate' from dual union all
select 26086,'026086','2000-009A','Gruzovoy Maket','Dummy satellite',66911,'In Earth orbit','','2000-02-15 00:00:00',96.55,581,606,64.86,'Intermediate' from dual union all
select 26087,'026087','2000-009B','Fregat','Fregat 14S44 No. 1001',66911,'Landed','2000-02-09 00:00:00','2000-02-09 00:00:00',96.5,578,604,64.86,'Intermediate' from dual union all
select 26088,'026088','2000-010A','Endeavour (STS-99)','OV-105',66912,'Landed','2000-02-22 00:00:00','2000-02-13 00:00:00',89.16,224,241,57.01,'Intermediate' from dual union all
select 26089,'026089','2000-011A','Garuda 1','Garuda 1',66913,'In Earth orbit','','2000-03-14 00:00:00',1436.01,35774,35796,2.99,'Stationary' from dual union all
select 2609,'002609','1966-110B','Agena D 6151','Agena D 6151',13517,'Reentered','1974-10-15 00:00:00','1967-01-09 00:00:00',631.08,143,35844,31.09,'GEO Transfer' from dual union all
select 26090,'026090','2000-011B','Blok DM3 No. 15L','Blok DM3 No. 15L',66913,'In Earth orbit','','2000-02-20 00:00:00',755.05,6231,35947,16.69,'Highly Ellip' from dual union all
select 26091,'026091','2000-004J','Thelma','Picosat 7',66906,'In Earth orbit','','2000-03-04 00:00:00',100.4,752,803,100.22,'Sun-Synch' from dual union all
select 26092,'026092','2000-004K','Louise','Picosat 8',66906,'In Earth orbit','','2000-03-04 00:00:00',100.36,748,804,100.22,'Sun-Synch' from dual union all
select 26093,'026093','2000-004L','JAK','Picosat 5',66906,'In Earth orbit','','2000-03-04 00:00:00',100.37,749,804,100.22,'Sun-Synch' from dual union all
select 26094,'026094','2000-004M','STENSAT','Picosat 6',66906,'In Earth orbit','','2000-03-05 00:00:00',100.38,750,804,100.22,'Sun-Synch' from dual union all
select 26095,'026095','2000-012A','Superbird B2','Superbird 4',66914,'In Earth orbit','','2000-03-19 00:00:00',1436.09,35757,35815,.03,'Stationary' from dual union all
select 26096,'026096','2000-012B','Ariane H10-3 V127','Ariane H10-3 S/N T494',66914,'In Earth orbit','','2000-02-28 00:00:00',657.28,309,37018,5.2,'GEO Transfer' from dual union all
select 26097,'026097','2000-004N','Picosat part','',66906,'Reentered','2001-02-07 00:00:00','2000-02-22 00:00:00',100.36,748,804,100.22,'Sun-Synch' from dual union all
select 26098,'026098','2000-013A','Ekspress 6A','Ekspress A No. 2',66915,'In Earth orbit','','2000-04-11 00:00:00',1436.14,35781,35794,.15,'Stationary' from dual union all
select 26099,'026099','2000-013B','8S812','8S812',66915,'Reentered','2000-03-14 00:00:00','2000-03-13 00:00:00',87.59,152,159,51.64,'Intermediate' from dual union all
select 261,'000261','61 Omicron 80','deb r/b Ablestar','',7273,'In Earth orbit','','1962-03-08 00:00:00',103.48,835,1012,66.69,'Intermediate' from dual union all
select 2610,'002610','1966-111A','OV1-9','OV1-9',13518,'In Earth orbit','','1966-12-14 00:00:00',142.21,467,4826,99.13,'Medium' from dual union all
select 26100,'026100','2000-013C','Perekhodnik','Sredniy perekhodnik',66915,'Reentered','2000-03-13 00:00:00','2000-03-12 00:00:00',87.64,156,160,51.61,'Intermediate' from dual union all
select 26101,'026101','2000-013D','Blok DM-2M No. 10L','11S861-01 No. 10L',66915,'In Earth orbit','','2000-04-11 00:00:00',1429.17,35604,35698,.16,'Drift GEO' from dual union all
select 26102,'026102','2000-014A','MTI','MTI',66916,'In Earth orbit','','2000-04-12 00:00:00',96.54,573,613,97.4,'Sun-Synch' from dual union all
select 26103,'026103','2000-014B','Orion 38','Orion 38',66916,'In Earth orbit','','2000-08-03 00:00:00',96.47,572,608,97.4,'Sun-Synch' from dual union all
select 26104,'026104','2000-013E','SOZ','SOZ',66915,'Reentered','2000-10-05 00:00:00','2000-04-11 00:00:00',631.29,290,35708,47.54,'GEO Transfer' from dual union all
select 26105,'026105','2000-013F','SOZ','SOZ',66915,'Reentered','2000-09-03 00:00:00','2000-04-18 00:00:00',629.06,223,35660,47.46,'GEO Transfer' from dual union all
select 26106,'026106','2000-015A','Dumsat','Dumsat',66917,'In Earth orbit','','2000-03-29 00:00:00',320.3,244,18014,64.66,'Highly Ellip' from dual union all
select 26107,'026107','2000-016A','Asiastar','Asiastar',66918,'In Earth orbit','','2000-04-20 00:00:00',1436.14,35781,35794,.01,'Stationary' from dual;
insert into SATELLITE
select 26108,'026108','2000-016B','Insat 3B','Insat 3B',66918,'In Earth orbit','','2000-04-23 00:00:00',1436.06,35743,35829,.01,'Stationary' from dual union all
select 26109,'026109','2000-016C','Sylda 5','Sylda 5',66918,'In Earth orbit','','2000-04-03 00:00:00',638.18,581,35771,7.02,'GEO Transfer' from dual union all
select 2611,'002611','1966-111B','OV1-10','OV1-10',13518,'Reentered','2002-11-30 00:00:00','1968-11-30 00:00:00',98.72,634,761,93.41,'Polar' from dual union all
select 26110,'026110','2000-016D','EPS 505','EPS S/N 505R1',66918,'In Earth orbit','','2000-03-29 00:00:00',634.01,701,35437,7.02,'GEO Transfer' from dual union all
select 26111,'026111','1987-020DS','deb Kosmos-1823','',60110,'In Earth orbit','','2000-03-23 00:00:00',115.99,1478,1522,73.61,'Intermediate' from dual union all
select 26112,'026112','1999-057D','CZ-4 debris','',66738,'Reentered','2000-03-31 00:00:00','2000-03-27 00:00:00',91.85,242,488,98.52,'Sun-Synch' from dual union all
select 26113,'026113','2000-017A','IMAGE','IMAGE',66919,'In Earth orbit','','2000-04-26 00:00:00',856.19,1055,45925,89.83,'Highly Ellip' from dual union all
select 26114,'026114','2000-017B','Delta 277','Delta SSPS AJ-10-118K',66919,'Reentered','2000-05-17 00:00:00','2000-04-04 00:00:00',96.32,189,976,91.33,'Polar' from dual union all
select 26115,'026115','2000-017C','Star 37FM','Star 37FM',66919,'In Earth orbit','','2000-04-03 00:00:00',853.93,1005,45870,89.9,'Highly Ellip' from dual union all
select 26116,'026116','2000-018A','Soyuz TM-30','Soyuz 7K-STM No. 204',66920,'Landed','2000-06-16 00:00:00','2000-05-05 00:00:00',91.96,358,384,51.65,'Intermediate' from dual union all
select 26117,'026117','2000-018B','Blok-I','11S510',66920,'Reentered','2000-04-05 00:00:00','2000-04-04 00:00:00',88.22,172,201,51.65,'Intermediate' from dual union all
select 26118,'026118','1999-057E','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',103.75,681,1190,98.98,'Sun-Synch' from dual union all
select 26119,'026119','1999-057F','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',107.8,729,1521,99.37,'Sun-Synch' from dual union all
select 2612,'002612','1966-112A','Kosmos-135','DS-U2-MP No. 1',13519,'Reentered','1967-04-12 00:00:00','1967-01-11 00:00:00',93.12,251,603,48.41,'Intermediate' from dual union all
select 26120,'026120','1999-057G','CZ-4 debris','',66738,'Reentered','2011-11-05 00:00:00','2000-04-17 00:00:00',97.77,560,744,98.71,'Sun-Synch' from dual union all
select 26121,'026121','1999-057H','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',98.19,604,741,98.69,'Sun-Synch' from dual union all
select 26122,'026122','1999-057J','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',108.49,732,1582,98.94,'Sun-Synch' from dual union all
select 26123,'026123','1999-057K','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',100.11,703,825,98.54,'Sun-Synch' from dual union all
select 26124,'026124','1999-057L','CZ-4 debris','',66738,'Reentered','2003-01-09 00:00:00','2000-04-17 00:00:00',98.21,616,731,98.65,'Sun-Synch' from dual union all
select 26125,'026125','1999-057M','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',103.56,704,1149,98.86,'Sun-Synch' from dual union all
select 26126,'026126','1999-057N','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',100.98,716,895,98.56,'Sun-Synch' from dual union all
select 26127,'026127','1999-057P','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',101.08,691,929,98.1,'Sun-Synch' from dual union all
select 26128,'026128','1999-057Q','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',98.7,657,736,98.64,'Sun-Synch' from dual union all
select 26129,'026129','1999-057R','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',106.87,723,1441,99.1,'Sun-Synch' from dual union all
select 2613,'002613','1966-097C','OV3-2 despin weight','',13504,'Reentered','1967-09-20 00:00:00','1966-12-01 00:00:00',102.32,254,1483,81.96,'Intermediate' from dual union all
select 26130,'026130','1999-057S','CZ-4 debris','',66738,'Reentered','2011-08-06 00:00:00','2000-04-17 00:00:00',101.28,722,917,98.45,'Sun-Synch' from dual union all
select 26131,'026131','1999-057T','CZ-4 debris','',66738,'Reentered','2005-04-05 00:00:00','2000-04-17 00:00:00',98.78,652,748,98.39,'Sun-Synch' from dual union all
select 26132,'026132','1999-057U','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',102.39,732,1011,98.61,'Sun-Synch' from dual union all
select 26133,'026133','1999-057V','CZ-4 debris','',66738,'Reentered','2000-10-25 00:00:00','2000-04-17 00:00:00',96.83,447,767,97.72,'Sun-Synch' from dual union all
select 26134,'026134','1999-057W','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',106.82,723,1436,99.03,'Sun-Synch' from dual union all
select 26135,'026135','1999-057X','CZ-4 debris','',66738,'Reentered','2005-06-07 00:00:00','2000-04-17 00:00:00',98.4,623,742,98.68,'Sun-Synch' from dual union all
select 26136,'026136','1999-057Y','CZ-4 debris','',66738,'Reentered','2000-09-05 00:00:00','2000-04-16 00:00:00',96.08,441,700,98.39,'Sun-Synch' from dual union all
select 26137,'026137','1999-057Z','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',104.22,677,1238,98.81,'Sun-Synch' from dual union all
select 26138,'026138','1999-057AA','CZ-4 debris','',66738,'Reentered','2000-10-06 00:00:00','2000-04-13 00:00:00',96.6,479,712,98.28,'Sun-Synch' from dual union all
select 26139,'026139','1999-057AB','CZ-4 debris','',66738,'Reentered','2012-08-20 00:00:00','2000-04-17 00:00:00',98.61,654,731,98.64,'Sun-Synch' from dual union all
select 2614,'002614','1966-097D','OV3-2 despin weight','',13504,'Reentered','1976-08-20 00:00:00','1966-12-12 00:00:00',105.56,372,1670,81.99,'Intermediate' from dual union all
select 26140,'026140','1999-057AC','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',101.99,657,1049,99.35,'Sun-Synch' from dual union all
select 26141,'026141','1999-057AD','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',99.95,720,793,98.45,'Sun-Synch' from dual union all
select 26142,'026142','1999-057AE','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',104.53,732,1213,98.75,'Sun-Synch' from dual union all
select 26143,'026143','1999-057AF','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',107.41,732,1482,99.02,'Sun-Synch' from dual union all
select 26144,'026144','1999-057AG','CZ-4 debris','',66738,'Reentered','2000-10-22 00:00:00','2000-04-14 00:00:00',97.27,543,712,98.74,'Sun-Synch' from dual union all
select 26145,'026145','1999-057AH','CZ-4 debris','',66738,'Reentered','2012-11-14 00:00:00','2000-04-17 00:00:00',100.77,678,913,98.03,'Sun-Synch' from dual union all
select 26146,'026146','1999-067D','DMSP F-15 OLS Cover','',66748,'In Earth orbit','','2000-04-17 00:00:00',101.68,829,848,98.89,'Sun-Synch' from dual union all
select 26147,'026147','1999-067E','DMSP F-15 OLS Cooler Cover','',66748,'In Earth orbit','','2000-04-17 00:00:00',101.7,833,846,98.88,'Sun-Synch' from dual union all
select 26148,'026148','1999-057AJ','CZ-4 debris','',66738,'Reentered','2002-03-26 00:00:00','2000-04-17 00:00:00',97.83,579,730,98.06,'Sun-Synch' from dual union all
select 26149,'026149','1999-057AK','CZ-4 debris','',66738,'Reentered','2000-09-30 00:00:00','2000-04-13 00:00:00',96.25,458,700,98.5,'Sun-Synch' from dual union all
select 2615,'002615','1966-112B','S1M','S1M',13519,'Reentered','1967-03-15 00:00:00','1967-01-11 00:00:00',92.89,256,577,48.41,'Intermediate' from dual union all
select 26150,'026150','1999-057AL','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',105.57,727,1316,99.08,'Sun-Synch' from dual union all
select 26151,'026151','1999-057AM','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',109.99,732,1721,99.45,'Sun-Synch' from dual union all
select 26152,'026152','1999-057AN','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',108.31,733,1565,99.22,'Sun-Synch' from dual union all
select 26153,'026153','1999-057AP','CZ-4 debris','',66738,'Reentered','2000-12-30 00:00:00','2000-04-16 00:00:00',96.58,464,726,98.34,'Sun-Synch' from dual union all
select 26154,'026154','1999-057AQ','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',108.33,731,1568,99.52,'Sun-Synch' from dual union all
select 26155,'026155','1999-057AR','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',102.2,683,1043,98.75,'Sun-Synch' from dual union all
select 26156,'026156','1999-057AS','CZ-4 debris','',66738,'Reentered','2014-01-11 00:00:00','2000-04-16 00:00:00',97.74,567,735,98.69,'Sun-Synch' from dual union all
select 26157,'026157','1999-057AT','CZ-4 debris','',66738,'Reentered','2015-02-17 00:00:00','2000-04-16 00:00:00',97.79,569,737,98.69,'Sun-Synch' from dual union all
select 26158,'026158','1999-057AU','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',103.11,718,1094,98.76,'Sun-Synch' from dual union all
select 26159,'026159','1999-057AV','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',104.83,714,1259,99.06,'Sun-Synch' from dual union all
select 2616,'002616','1966-112C','deb Kosmos-135','',13519,'Reentered','1966-12-28 00:00:00','1966-12-14 00:00:00',93.2,256,607,48.47,'Intermediate' from dual union all
select 26160,'026160','1999-057AW','CZ-4 debris','',66738,'Reentered','2002-01-27 00:00:00','2000-04-16 00:00:00',97.8,527,779,97.76,'Sun-Synch' from dual union all
select 26161,'026161','1999-057AX','CZ-4 debris','',66738,'Reentered','2001-04-07 00:00:00','2000-04-17 00:00:00',97.28,464,793,99.27,'Sun-Synch' from dual union all
select 26162,'026162','1999-057AY','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',103.78,697,1178,98.98,'Sun-Synch' from dual union all
select 26163,'026163','1999-057AZ','CZ-4 debris','',66738,'Reentered','2001-10-08 00:00:00','2000-04-16 00:00:00',97.53,479,802,97.81,'Sun-Synch' from dual union all
select 26164,'026164','1999-057BA','CZ-4 debris','',66738,'Reentered','2000-11-29 00:00:00','2000-04-14 00:00:00',97.46,556,719,98.75,'Sun-Synch' from dual union all
select 26165,'026165','1999-057BB','CZ-4 debris','',66738,'Reentered','2002-11-01 00:00:00','2000-04-17 00:00:00',99.05,600,827,97.7,'Sun-Synch' from dual union all
select 26166,'026166','1999-057BC','CZ-4 debris','',66738,'Reentered','2009-11-17 00:00:00','2000-04-17 00:00:00',100.38,712,842,98.46,'Sun-Synch' from dual union all
select 26167,'026167','1999-057BD','CZ-4 debris','',66738,'Reentered','2000-11-04 00:00:00','2000-04-14 00:00:00',97.3,552,707,98.5,'Sun-Synch' from dual union all
select 26168,'026168','1999-057BE','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',100.64,710,868,98.44,'Sun-Synch' from dual union all
select 26169,'026169','1999-057BF','CZ-4 debris','',66738,'Reentered','2003-05-19 00:00:00','2000-04-17 00:00:00',97.79,575,731,98.72,'Sun-Synch' from dual union all
select 2617,'002617','1966-112D','deb Kosmos-135','',13519,'Reentered','1966-12-28 00:00:00','1966-12-19 00:00:00',92.27,244,527,48.45,'Intermediate' from dual union all
select 26170,'026170','1999-057BG','CZ-4 debris','',66738,'Reentered','2000-11-13 00:00:00','2000-04-16 00:00:00',96.74,484,722,98.87,'Sun-Synch' from dual union all
select 26171,'026171','1999-057BH','CZ-4 debris','',66738,'Reentered','2000-12-01 00:00:00','2000-04-14 00:00:00',96.95,517,708,98.47,'Sun-Synch' from dual union all
select 26172,'026172','1999-057BJ','CZ-4 debris','',66738,'Reentered','2018-01-10 00:00:00','2000-04-16 00:00:00',97.48,543,734,98.67,'Sun-Synch' from dual union all
select 26173,'026173','1999-057BK','CZ-4 debris','',66738,'Reentered','2001-03-25 00:00:00','2000-04-14 00:00:00',97.48,551,726,98.74,'Sun-Synch' from dual union all
select 26174,'026174','1999-057BL','CZ-4 debris','',66738,'Reentered','2001-10-25 00:00:00','2000-04-16 00:00:00',97.91,594,724,98.57,'Sun-Synch' from dual union all
select 26175,'026175','1999-057BM','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',104.95,731,1254,98.82,'Sun-Synch' from dual union all
select 26176,'026176','1999-057BN','CZ-4 debris','',66738,'Reentered','2001-04-07 00:00:00','2000-04-17 00:00:00',97.13,530,713,98.48,'Sun-Synch' from dual union all
select 26177,'026177','1999-057BP','CZ-4 debris','',66738,'Reentered','2001-03-29 00:00:00','2000-04-18 00:00:00',98.53,635,742,98.34,'Sun-Synch' from dual union all
select 26178,'026178','1999-057BQ','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',100.54,718,851,98.52,'Sun-Synch' from dual union all
select 26179,'026179','1999-057BR','CZ-4 debris','',66738,'Reentered','2002-08-04 00:00:00','2000-04-17 00:00:00',100.53,705,863,98.42,'Sun-Synch' from dual union all
select 2618,'002618','1966-113A','OPS 8968','GAMBIT-3 Mission 4303',13520,'Reentered Attached','1966-12-24 00:00:00','1966-12-17 00:00:00',89.53,136,367,109.56,'Retrograde' from dual union all
select 26180,'026180','1999-057BS','CZ-4 debris','',66738,'Reentered','2002-08-04 00:00:00','2000-04-18 00:00:00',97.99,596,730,98.72,'Sun-Synch' from dual union all
select 26181,'026181','1999-057BT','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',99.19,709,731,98.58,'Sun-Synch' from dual union all
select 26182,'026182','1999-057BU','CZ-4 debris','',66738,'Reentered','2002-02-28 00:00:00','2000-04-18 00:00:00',98.09,596,738,98.71,'Sun-Synch' from dual union all
select 26183,'026183','1999-057BV','CZ-4 debris','',66738,'Reentered','2002-04-23 00:00:00','2000-04-18 00:00:00',98.64,654,734,98.66,'Sun-Synch' from dual union all
select 26184,'026184','1999-057BW','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',100.78,711,881,98.45,'Sun-Synch' from dual union all
select 26185,'026185','1999-057BX','CZ-4 debris','',66738,'Reentered','2004-10-23 00:00:00','2000-04-18 00:00:00',98.87,660,750,98.48,'Sun-Synch' from dual union all
select 26186,'026186','1999-057BY','CZ-4 debris','',66738,'Reentered','2012-09-01 00:00:00','2000-04-18 00:00:00',100.02,724,795,98.45,'Sun-Synch' from dual union all
select 26187,'026187','1999-057BZ','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',98.4,630,734,98.53,'Sun-Synch' from dual union all
select 26188,'026188','1999-057CA','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',101,729,883,98.84,'Sun-Synch' from dual union all
select 26189,'026189','1999-057CB','CZ-4 debris','',66738,'Reentered','2003-10-30 00:00:00','2000-04-18 00:00:00',98.55,648,731,98.64,'Sun-Synch' from dual union all
select 2619,'002619','1966-114D','deb Bios 1 Adapter','',13521,'Reentered','1967-01-10 00:00:00','1966-12-22 00:00:00',90.43,287,305,33.49,'Intermediate' from dual union all
select 26190,'026190','1999-057CC','CZ-4 debris','',66738,'Reentered','2000-12-02 00:00:00','2000-04-18 00:00:00',98.21,631,716,98.65,'Sun-Synch' from dual union all
select 26191,'026191','1999-057CD','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',100.85,713,885,98.47,'Sun-Synch' from dual union all
select 26192,'026192','1999-057CE','CZ-4 debris','',66738,'Reentered','2002-07-14 00:00:00','2000-04-18 00:00:00',98.21,612,735,98.61,'Sun-Synch' from dual union all
select 26193,'026193','1999-057CF','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',99.19,705,735,98.53,'Sun-Synch' from dual union all
select 26194,'026194','1999-057CG','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',99.54,691,783,98.4,'Sun-Synch' from dual union all
select 26195,'026195','1999-057CH','CZ-4 debris','',66738,'Reentered','2013-02-15 00:00:00','2000-04-18 00:00:00',98.3,622,733,98.65,'Sun-Synch' from dual union all
select 26196,'026196','1999-057CJ','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',105.03,729,1262,98.89,'Sun-Synch' from dual union all
select 26197,'026197','1999-057CK','CZ-4 debris','',66738,'Reentered','2003-10-10 00:00:00','2000-04-17 00:00:00',99.1,700,731,98.61,'Sun-Synch' from dual union all
select 26198,'026198','1999-057CL','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',100.43,713,844,98.45,'Sun-Synch' from dual;
insert into SATELLITE
select 26199,'026199','1999-057CM','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',100.74,708,880,98.41,'Sun-Synch' from dual union all
select 262,'000262','61 Omicron 81','deb r/b Ablestar','',7273,'In Earth orbit','','1962-03-11 00:00:00',103.65,810,1053,66.46,'Intermediate' from dual union all
select 2620,'002620','1963-049H','deb Transit VBN-2','',9280,'In Earth orbit','','1966-11-29 00:00:00',106.77,1064,1090,89.94,'Polar' from dual union all
select 26200,'026200','1999-057CN','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',99.68,692,795,98.4,'Sun-Synch' from dual union all
select 26201,'026201','1999-057CP','CZ-4 debris','',66738,'Reentered','2001-12-28 00:00:00','2000-04-18 00:00:00',98.51,642,733,98.49,'Sun-Synch' from dual union all
select 26202,'026202','1999-057CQ','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',100.97,716,893,98.39,'Sun-Synch' from dual union all
select 26203,'026203','1999-057CR','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',101.05,720,897,98.37,'Sun-Synch' from dual union all
select 26204,'026204','1999-057CS','CZ-4 debris','',66738,'Reentered','2001-11-30 00:00:00','2000-04-18 00:00:00',97.31,533,727,98.37,'Sun-Synch' from dual union all
select 26205,'026205','1999-057CT','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',110.5,733,1766,99.18,'Sun-Synch' from dual union all
select 26206,'026206','1999-057CU','CZ-4 debris','',66738,'Reentered','2000-11-08 00:00:00','2000-04-18 00:00:00',97.13,537,706,98.41,'Sun-Synch' from dual union all
select 26207,'026207','1999-057CV','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',101.21,712,920,98.39,'Sun-Synch' from dual union all
select 26208,'026208','1999-057CW','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',99.83,690,810,98.46,'Sun-Synch' from dual union all
select 26209,'026209','1999-057CX','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',99.16,693,744,98.54,'Sun-Synch' from dual union all
select 2621,'002621','1966-111C','OV1-10P','X-258',13518,'In Earth orbit','','1967-01-14 00:00:00',98.8,632,771,93.43,'Polar' from dual union all
select 26210,'026210','1999-057CY','CZ-4 debris','',66738,'Reentered','2001-04-08 00:00:00','2000-04-18 00:00:00',97.55,556,727,98.7,'Sun-Synch' from dual union all
select 26211,'026211','1999-057CZ','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',108.48,733,1580,98.94,'Sun-Synch' from dual union all
select 26212,'026212','1999-057DA','CZ-4 debris','',66738,'Reentered','2003-09-28 00:00:00','2000-04-18 00:00:00',98.98,690,730,98.6,'Sun-Synch' from dual union all
select 26213,'026213','1999-057DB','CZ-4 debris','',66738,'Reentered','2012-09-26 00:00:00','2000-04-18 00:00:00',97.86,575,737,98.69,'Sun-Synch' from dual union all
select 26214,'026214','1999-057DC','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',98.79,665,737,98.59,'Sun-Synch' from dual union all
select 26215,'026215','1999-057DD','CZ-4 debris','',66738,'Reentered','2002-12-13 00:00:00','2000-04-18 00:00:00',98.87,682,728,98.58,'Sun-Synch' from dual union all
select 26216,'026216','1999-057DE','CZ-4 debris','',66738,'Reentered','2002-04-10 00:00:00','2000-04-18 00:00:00',98.73,655,740,98.45,'Sun-Synch' from dual union all
select 26217,'026217','1999-057DF','CZ-4 debris','',66738,'Reentered','2002-09-15 00:00:00','2000-04-18 00:00:00',99.02,697,727,98.54,'Sun-Synch' from dual union all
select 26218,'026218','1999-057DG','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',101.17,719,910,98.69,'Sun-Synch' from dual union all
select 26219,'026219','1999-057DH','CZ-4 debris','',66738,'Reentered','2003-08-25 00:00:00','2000-04-18 00:00:00',98.16,606,736,98.69,'Sun-Synch' from dual union all
select 2622,'002622','1966-111D','OV1-9P','X-258',13518,'In Earth orbit','','1967-02-01 00:00:00',142.21,475,4819,99.14,'Medium' from dual union all
select 26220,'026220','1999-057DJ','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',98.38,618,745,98.64,'Sun-Synch' from dual union all
select 26221,'026221','1999-057DK','CZ-4 debris','',66738,'Reentered','2015-03-08 00:00:00','2000-04-18 00:00:00',100.02,718,801,98.44,'Sun-Synch' from dual union all
select 26222,'026222','1999-057DL','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',100.51,712,854,98.43,'Sun-Synch' from dual union all
select 26223,'026223','1999-057DM','CZ-4 debris','',66738,'Reentered','2001-01-16 00:00:00','2000-04-17 00:00:00',97.61,564,725,98.72,'Sun-Synch' from dual union all
select 26224,'026224','1999-057DN','CZ-4 debris','',66738,'Reentered','2002-10-29 00:00:00','2000-04-18 00:00:00',98.54,646,732,98.62,'Sun-Synch' from dual union all
select 26225,'026225','1999-057DP','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',100.14,729,802,98.5,'Sun-Synch' from dual union all
select 26226,'026226','1999-057DQ','CZ-4 debris','',66738,'Reentered','2006-04-14 00:00:00','2000-04-18 00:00:00',100.02,724,795,98.44,'Sun-Synch' from dual union all
select 26227,'026227','1999-057DR','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',100.56,713,857,98.45,'Sun-Synch' from dual union all
select 26228,'026228','1999-057DS','CZ-4 debris','',66738,'Reentered','2016-03-07 00:00:00','2000-04-18 00:00:00',100.6,713,861,98.46,'Sun-Synch' from dual union all
select 26229,'026229','1999-057DT','CZ-4 debris','',66738,'Reentered','2013-11-10 00:00:00','2000-04-18 00:00:00',100.45,708,851,98.44,'Sun-Synch' from dual union all
select 2623,'002623','1966-114B','Delta 43','DSV-3E-3 S/N 20201',13521,'Reentered','1967-01-22 00:00:00','1966-12-24 00:00:00',90.31,281,298,33.49,'Intermediate' from dual union all
select 26230,'026230','1999-057DU','CZ-4 debris','',66738,'Reentered','2014-06-30 00:00:00','2000-04-18 00:00:00',98.27,614,738,98.7,'Sun-Synch' from dual union all
select 26231,'026231','1999-057DV','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',98.41,622,744,98.7,'Sun-Synch' from dual union all
select 26232,'026232','1999-057DW','CZ-4 debris','',66738,'Reentered','2004-01-30 00:00:00','2000-04-18 00:00:00',99.47,700,766,98.55,'Sun-Synch' from dual union all
select 26233,'026233','1999-057DX','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',99.29,712,738,98.53,'Sun-Synch' from dual union all
select 26234,'026234','1999-057DY','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',99.62,731,749,98.61,'Sun-Synch' from dual union all
select 26235,'026235','1999-057DZ','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',99.82,718,782,98.49,'Sun-Synch' from dual union all
select 26236,'026236','1999-057EA','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',100.51,712,854,98.44,'Sun-Synch' from dual union all
select 26237,'026237','1999-057EB','CZ-4 debris','',66738,'Reentered','2001-11-07 00:00:00','2000-04-18 00:00:00',98.09,605,729,98.91,'Sun-Synch' from dual union all
select 26238,'026238','1999-057EC','CZ-4 debris','',66738,'Reentered','2015-08-05 00:00:00','2000-04-18 00:00:00',99.93,712,799,98.41,'Sun-Synch' from dual union all
select 26239,'026239','1999-057ED','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',100.8,709,884,98.39,'Sun-Synch' from dual union all
select 2624,'002624','1966-115A','Kosmos-136','Zenit-2 No. 47',13522,'Landed','1966-12-27 00:00:00','1966-12-22 00:00:00',89.28,193,285,64.58,'Intermediate' from dual union all
select 26240,'026240','1999-057EE','CZ-4 debris','',66738,'Reentered','2000-06-03 00:00:00','2000-04-17 00:00:00',94.48,294,692,97.67,'Sun-Synch' from dual union all
select 26241,'026241','1999-057EF','CZ-4 debris','',66738,'In Earth orbit','','2000-04-17 00:00:00',100.83,712,883,98.42,'Sun-Synch' from dual union all
select 26242,'026242','1999-057EG','CZ-4 debris','',66738,'Reentered','2002-03-26 00:00:00','2000-04-17 00:00:00',97.9,589,728,98.56,'Sun-Synch' from dual union all
select 26243,'026243','2000-019A','Sesat','SESAT',66921,'In Earth orbit','','2000-05-17 00:00:00',1436.09,35781,35792,.04,'Stationary' from dual union all
select 26244,'026244','2000-019B','8S812','8S812',66921,'Reentered','2000-04-19 00:00:00','2000-04-18 00:00:00',88.1,176,185,51.64,'Intermediate' from dual union all
select 26245,'026245','2000-019C','Perekhodnik','Sredniy perekhodnik',66921,'Reentered','2000-04-18 00:00:00','2000-04-18 00:00:00',87.1,110,151,51.63,'Intermediate' from dual union all
select 26246,'026246','2000-019D','Blok DM-2M No. 9L','11S861-01 No. 9L',66921,'In Earth orbit','','2000-05-26 00:00:00',1446.32,35929,36044,.1,'Drift GEO' from dual union all
select 26247,'026247','1999-057EH','CZ-4 debris','',66738,'Reentered','2002-06-03 00:00:00','2000-04-19 00:00:00',100.47,706,856,98.47,'Sun-Synch' from dual union all
select 26248,'026248','1999-057EJ','CZ-4 debris','',66738,'Reentered','2001-11-07 00:00:00','2000-04-19 00:00:00',98.06,611,721,98.67,'Sun-Synch' from dual union all
select 26249,'026249','1999-057EK','CZ-4 debris','',66738,'Reentered','2001-04-08 00:00:00','2000-04-18 00:00:00',98.33,624,733,98.57,'Sun-Synch' from dual union all
select 2625,'002625','1966-115B','8A92E','8A92E',13522,'Reentered','1966-12-28 00:00:00','1966-12-21 00:00:00',89.31,208,273,64.61,'Intermediate' from dual union all
select 26250,'026250','1999-057EL','CZ-4 debris','',66738,'Reentered','2001-11-06 00:00:00','2000-04-19 00:00:00',99.15,679,757,98.46,'Sun-Synch' from dual union all
select 26251,'026251','1999-057EM','CZ-4 debris','',66738,'Reentered','2004-02-12 00:00:00','2000-04-19 00:00:00',99.1,634,797,98.57,'Sun-Synch' from dual union all
select 26252,'026252','1999-057EN','CZ-4 debris','',66738,'Reentered','2000-12-29 00:00:00','2000-04-19 00:00:00',98.39,645,719,98.66,'Sun-Synch' from dual union all
select 26253,'026253','1999-057EP','CZ-4 debris','',66738,'Reentered','2014-02-10 00:00:00','2000-04-19 00:00:00',99.7,718,771,98.54,'Sun-Synch' from dual union all
select 26254,'026254','1999-057EQ','CZ-4 debris','',66738,'Reentered','2000-06-29 00:00:00','2000-04-18 00:00:00',96.52,503,681,98.77,'Sun-Synch' from dual union all
select 26255,'026255','1999-057ER','CZ-4 debris','',66738,'Reentered','2001-06-12 00:00:00','2000-04-19 00:00:00',98.1,613,722,98.66,'Sun-Synch' from dual union all
select 26256,'026256','1999-057ES','CZ-4 debris','',66738,'Reentered','2001-08-29 00:00:00','2000-04-18 00:00:00',98.1,616,719,97.6,'Sun-Synch' from dual union all
select 26257,'026257','1999-057ET','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',99.21,704,738,98.49,'Sun-Synch' from dual union all
select 26258,'026258','1999-057EU','CZ-4 debris','',66738,'Reentered','2001-04-06 00:00:00','2000-04-19 00:00:00',97.54,556,726,98.71,'Sun-Synch' from dual union all
select 26259,'026259','1999-057EV','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',100.97,713,897,98.38,'Sun-Synch' from dual union all
select 2626,'002626','1966-116A','Luna-13','E-6M No. 205',13523,'Deep Space','1966-12-24 00:00:00','1966-12-21 00:00:00',21544.43,200,500000,51.8,'Cislunar' from dual union all
select 26260,'026260','1999-057EW','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',101,714,897,98.44,'Sun-Synch' from dual union all
select 26261,'026261','1999-057EX','CZ-4 debris','',66738,'Reentered','2003-10-29 00:00:00','2000-04-18 00:00:00',100.57,709,863,98.4,'Sun-Synch' from dual union all
select 26262,'026262','1999-057EY','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',100.71,722,863,98.34,'Sun-Synch' from dual union all
select 26263,'026263','1999-057EZ','CZ-4 debris','',66738,'Reentered','2001-02-01 00:00:00','2000-04-19 00:00:00',100.36,691,861,98.45,'Sun-Synch' from dual union all
select 26264,'026264','1999-057FA','CZ-4 debris','',66738,'Reentered','2009-05-21 00:00:00','2000-04-19 00:00:00',99.23,715,729,98.67,'Sun-Synch' from dual union all
select 26265,'026265','1999-057FB','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',100.85,713,885,98.42,'Sun-Synch' from dual union all
select 26266,'026266','1999-057FC','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',100.93,705,901,98.37,'Sun-Synch' from dual union all
select 26267,'026267','1999-057FD','CZ-4 debris','',66738,'Reentered','2010-06-12 00:00:00','2000-04-19 00:00:00',100.21,724,813,98.44,'Sun-Synch' from dual union all
select 26268,'026268','1999-057FE','CZ-4 debris','',66738,'In Earth orbit','','2000-04-18 00:00:00',99.58,727,750,98.69,'Sun-Synch' from dual union all
select 26269,'026269','1999-057FF','CZ-4 debris','',66738,'Reentered','2002-09-20 00:00:00','2000-04-18 00:00:00',98.95,690,726,98.57,'Sun-Synch' from dual union all
select 2627,'002627','1966-117A','Kosmos-137','DS-U2-D No. 1',13524,'Reentered','1967-11-23 00:00:00','1967-01-24 00:00:00',103.45,221,1623,48.8,'Intermediate' from dual union all
select 26270,'026270','1999-057FG','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',100.76,708,881,98.42,'Sun-Synch' from dual union all
select 26271,'026271','1999-057FH','CZ-4 debris','',66738,'Reentered','2001-04-04 00:00:00','2000-04-19 00:00:00',98.19,619,725,98.33,'Sun-Synch' from dual union all
select 26272,'026272','1999-057FJ','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',100.69,710,873,98.4,'Sun-Synch' from dual union all
select 26273,'026273','1999-057FK','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',98.51,629,746,98.66,'Sun-Synch' from dual union all
select 26274,'026274','1999-057FL','CZ-4 debris','',66738,'Reentered','2000-11-08 00:00:00','2000-04-18 00:00:00',98.13,626,713,98.62,'Sun-Synch' from dual union all
select 26275,'026275','1999-057FM','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',100.76,708,881,98.45,'Sun-Synch' from dual union all
select 26276,'026276','1999-057FN','CZ-4 debris','',66738,'Reentered','2004-03-03 00:00:00','2000-04-18 00:00:00',100.34,713,837,98.46,'Sun-Synch' from dual union all
select 26277,'026277','1999-057FP','CZ-4 debris','',66738,'Reentered','2015-12-23 00:00:00','2000-04-19 00:00:00',100.88,694,907,98.29,'Sun-Synch' from dual union all
select 26278,'026278','1999-057FQ','CZ-4 debris','',66738,'Reentered','2011-12-10 00:00:00','2000-04-19 00:00:00',99.3,693,758,98.53,'Sun-Synch' from dual union all
select 26279,'026279','1999-057FR','CZ-4 debris','',66738,'Reentered','2002-01-27 00:00:00','2000-04-19 00:00:00',98.11,610,727,98.46,'Sun-Synch' from dual union all
select 2628,'002628','1966-116B','11S510','11S510',13523,'Reentered','1966-12-23 00:00:00','1966-12-21 00:00:00',88.29,179,201,51.81,'Intermediate' from dual union all
select 26280,'026280','1999-057FS','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',100.54,712,857,98.45,'Sun-Synch' from dual union all
select 26281,'026281','1999-057FT','CZ-4 debris','',66738,'Reentered','2011-05-20 00:00:00','2000-04-19 00:00:00',100.47,712,850,98.42,'Sun-Synch' from dual union all
select 26282,'026282','1999-057FU','CZ-4 debris','',66738,'Reentered','2001-09-26 00:00:00','2000-04-19 00:00:00',98.78,655,746,98.44,'Sun-Synch' from dual union all
select 26283,'026283','1999-057FV','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',100.7,710,874,98.44,'Sun-Synch' from dual union all
select 26284,'026284','1999-057FW','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',100.44,706,853,98.38,'Sun-Synch' from dual union all
select 26285,'026285','1999-057FX','CZ-4 debris','',66738,'Reentered','2014-10-24 00:00:00','2000-04-19 00:00:00',100.14,728,803,98.51,'Sun-Synch' from dual union all
select 26286,'026286','1999-057FY','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',100.47,713,849,98.41,'Sun-Synch' from dual union all
select 26287,'026287','1999-057FZ','CZ-4 debris','',66738,'Reentered','2009-07-19 00:00:00','2000-04-19 00:00:00',100.45,709,851,98.45,'Sun-Synch' from dual union all
select 26288,'026288','1999-057GA','CZ-4 debris','',66738,'Reentered','2012-06-07 00:00:00','2000-04-19 00:00:00',100.6,714,860,98.42,'Sun-Synch' from dual;
insert into SATELLITE
select 26289,'026289','1999-057GB','CZ-4 debris','',66738,'Reentered','2003-03-04 00:00:00','2000-04-19 00:00:00',99.32,695,758,98.52,'Sun-Synch' from dual union all
select 2629,'002629','1966-116C','BOZ','BOZ',13523,'Reentered','1966-12-22 00:00:00','1966-12-21 00:00:00',87.83,136,199,51.77,'Intermediate' from dual union all
select 26290,'026290','1999-057GC','CZ-4 debris','',66738,'Reentered','2002-05-02 00:00:00','2000-04-18 00:00:00',98.99,693,728,98.57,'Sun-Synch' from dual union all
select 26291,'026291','1999-057GD','CZ-4 debris','',66738,'Reentered','2002-10-25 00:00:00','2000-04-19 00:00:00',98.96,690,727,98.57,'Sun-Synch' from dual union all
select 26292,'026292','1999-057GE','CZ-4 debris','',66738,'Reentered','2001-12-05 00:00:00','2000-04-19 00:00:00',98.43,643,724,98.57,'Sun-Synch' from dual union all
select 26293,'026293','1999-057GF','CZ-4 debris','',66738,'Reentered','2004-03-02 00:00:00','2000-04-19 00:00:00',100.17,704,830,98.43,'Sun-Synch' from dual union all
select 26294,'026294','1999-057GG','CZ-4 debris','',66738,'In Earth orbit','','2000-04-19 00:00:00',100.68,710,872,98.42,'Sun-Synch' from dual union all
select 26295,'026295','1999-057GH','CZ-4 debris','',66738,'Reentered','2001-08-06 00:00:00','2000-04-19 00:00:00',97.92,598,721,98.43,'Sun-Synch' from dual union all
select 26296,'026296','1999-057GJ','CZ-4 debris','',66738,'Reentered','2003-03-11 00:00:00','2000-04-19 00:00:00',99.93,725,786,98.45,'Sun-Synch' from dual union all
select 26297,'026297','1999-057GK','CZ-4 debris','',66738,'Reentered','2015-01-22 00:00:00','2000-04-19 00:00:00',98.65,658,731,98.67,'Sun-Synch' from dual union all
select 26298,'026298','2000-020A','Galaxy 4R','Galaxy IVR',66922,'In Earth orbit','','2000-05-19 00:00:00',1438.51,35780,35888,.01,'Drift GEO' from dual union all
select 26299,'026299','2000-020B','Ariane H10-3 V129','Ariane H10-3 S/N T496',66922,'In Earth orbit','','2000-04-26 00:00:00',560.01,223,32032,7.28,'Highly Ellip' from dual union all
select 263,'000263','61 Omicron 82','deb r/b Ablestar','',7273,'In Earth orbit','','1962-03-05 00:00:00',102.4,741,1004,66.57,'Intermediate' from dual union all
select 2630,'002630','1966-117B','S1','S1',13524,'Reentered','1967-09-25 00:00:00','1967-01-20 00:00:00',103.32,221,1611,48.8,'Intermediate' from dual union all
select 26300,'026300','1999-057GL','CZ-4 debris','',66738,'In Earth orbit','','2000-04-24 00:00:00',100.65,710,869,98.44,'Sun-Synch' from dual union all
select 26301,'026301','2000-021A','Progress M1-2','Progress M1 No. 252',66923,'Deorbited','2000-10-15 00:00:00','2000-05-25 00:00:00',91.97,363,380,51.65,'Intermediate' from dual union all
select 26302,'026302','2000-021B','11S510','11S510',66923,'Reentered','2000-04-27 00:00:00','2000-04-26 00:00:00',88.3,175,206,51.63,'Intermediate' from dual union all
select 26303,'026303','1986-073D','deb NOAA 10','',59266,'Reentered','2001-02-08 00:00:00','2000-05-01 00:00:00',100.94,794,813,98.64,'Sun-Synch' from dual union all
select 26304,'026304','1999-057GM','CZ-4 debris','',66738,'Reentered','2003-05-01 00:00:00','2000-05-03 00:00:00',100.5,710,855,98.47,'Sun-Synch' from dual union all
select 26305,'026305','1999-057GN','CZ-4 debris','',66738,'Reentered','2003-02-17 00:00:00','2000-05-03 00:00:00',100.15,689,843,98.42,'Sun-Synch' from dual union all
select 26306,'026306','1999-057GP','CZ-4 debris','',66738,'In Earth orbit','','2000-05-02 00:00:00',100.56,710,861,98.43,'Sun-Synch' from dual union all
select 26307,'026307','1999-057GQ','CZ-4 debris','',66738,'Reentered','2004-02-06 00:00:00','2000-05-03 00:00:00',100.28,711,833,98.45,'Sun-Synch' from dual union all
select 26308,'026308','1999-057GR','CZ-4 debris','',66738,'Reentered','2011-10-23 00:00:00','2000-05-02 00:00:00',100.65,705,873,98.49,'Sun-Synch' from dual union all
select 26309,'026309','1999-057GS','CZ-4 debris','',66738,'Reentered','2013-06-30 00:00:00','2000-05-02 00:00:00',100.7,707,877,98.39,'Sun-Synch' from dual union all
select 2631,'002631','1966-114C','Bios 1 Adapter','Biosatellite A Adapter',13521,'Reentered','1967-01-10 00:00:00','1966-12-24 00:00:00',90.31,281,298,33.5,'Intermediate' from dual union all
select 26310,'026310','1999-057GT','CZ-4 debris','',66738,'In Earth orbit','','2000-05-02 00:00:00',99.82,700,800,98.5,'Sun-Synch' from dual union all
select 26311,'026311','1999-057GU','CZ-4 debris','',66738,'Reentered','2015-09-27 00:00:00','2000-05-03 00:00:00',100.48,711,851,98.47,'Sun-Synch' from dual union all
select 26312,'026312','1999-057GV','CZ-4 debris','',66738,'Reentered','2002-02-28 00:00:00','2000-05-03 00:00:00',99.55,691,784,98.47,'Sun-Synch' from dual union all
select 26313,'026313','1999-057GW','CZ-4 debris','',66738,'In Earth orbit','','2000-05-03 00:00:00',100.69,711,872,98.45,'Sun-Synch' from dual union all
select 26314,'026314','1999-057GX','CZ-4 debris','',66738,'Reentered','2001-07-28 00:00:00','2000-05-02 00:00:00',97.89,590,726,98.55,'Sun-Synch' from dual union all
select 26315,'026315','1999-057GY','CZ-4 debris','',66738,'Reentered','2002-05-06 00:00:00','2000-05-02 00:00:00',100.86,676,923,98.25,'Sun-Synch' from dual union all
select 26316,'026316','1999-057GZ','CZ-4 debris','',66738,'Reentered','2002-01-11 00:00:00','2000-05-03 00:00:00',99.34,687,768,98.36,'Sun-Synch' from dual union all
select 26317,'026317','1999-057HA','CZ-4 debris','',66738,'Reentered','2002-11-21 00:00:00','2000-05-01 00:00:00',100.24,704,836,98.39,'Sun-Synch' from dual union all
select 26318,'026318','1999-057HB','CZ-4 debris','',66738,'Reentered','2001-01-10 00:00:00','2000-05-03 00:00:00',98.51,660,714,98.58,'Sun-Synch' from dual union all
select 26319,'026319','1999-057HC','CZ-4 debris','',66738,'Reentered','2001-05-08 00:00:00','2000-05-03 00:00:00',97.98,609,716,98.6,'Sun-Synch' from dual union all
select 2632,'002632','1966-114A','Biosatellite 1 Capsule','Biosatellite A SRV',13521,'Reentered','1967-02-15 00:00:00','1967-01-13 00:00:00',90.09,272,286,33.48,'Intermediate' from dual union all
select 26320,'026320','1999-057HD','CZ-4 debris','',66738,'Reentered','2011-05-26 00:00:00','2000-05-01 00:00:00',100.38,696,857,98.42,'Sun-Synch' from dual union all
select 26321,'026321','1999-057HE','CZ-4 debris','',66738,'In Earth orbit','','2000-05-02 00:00:00',100.68,710,872,98.43,'Sun-Synch' from dual union all
select 26322,'026322','1999-057HF','CZ-4 debris','',66738,'Reentered','2003-01-19 00:00:00','2000-05-03 00:00:00',99.46,676,790,98.38,'Sun-Synch' from dual union all
select 26323,'026323','1999-057HG','CZ-4 debris','',66738,'In Earth orbit','','2000-05-03 00:00:00',100.58,711,862,98.42,'Sun-Synch' from dual union all
select 26324,'026324','1999-057HH','CZ-4 debris','',66738,'Reentered','2002-11-30 00:00:00','2000-05-02 00:00:00',98.2,628,718,98.5,'Sun-Synch' from dual union all
select 26325,'026325','1999-057HJ','CZ-4 debris','',66738,'Reentered','2001-01-14 00:00:00','2000-05-03 00:00:00',97.56,568,716,98.69,'Sun-Synch' from dual union all
select 26326,'026326','1999-057HK','CZ-4 debris','',66738,'Reentered','2001-03-25 00:00:00','2000-05-03 00:00:00',98.18,620,724,98.44,'Sun-Synch' from dual union all
select 26327,'026327','1999-057HL','CZ-4 debris','',66738,'Reentered','2001-03-11 00:00:00','2000-05-03 00:00:00',98.15,620,721,98.46,'Sun-Synch' from dual union all
select 26328,'026328','1999-057HM','CZ-4 debris','',66738,'Reentered','2001-02-22 00:00:00','2000-05-03 00:00:00',98.09,611,724,98.45,'Sun-Synch' from dual union all
select 26329,'026329','1999-057HN','CZ-4 debris','',66738,'In Earth orbit','','2000-05-03 00:00:00',100.81,709,885,98.4,'Sun-Synch' from dual union all
select 2633,'002633','1966-105C','deb Kosmos-131','(1966-00D)',13512,'Reentered','1966-12-30 00:00:00','1966-12-22 00:00:00',90.13,188,374,73.18,'Intermediate' from dual union all
select 26330,'026330','1999-057HP','CZ-4 debris','',66738,'Reentered','2008-07-25 00:00:00','2000-05-02 00:00:00',98.56,648,731,98.63,'Sun-Synch' from dual union all
select 26331,'026331','1999-057HQ','CZ-4 debris','',66738,'Reentered','2006-02-10 00:00:00','2000-05-03 00:00:00',100.45,711,849,98.44,'Sun-Synch' from dual union all
select 26332,'026332','1999-057HR','CZ-4 debris','',66738,'Reentered','2015-03-08 00:00:00','2000-05-03 00:00:00',100.48,708,855,98.47,'Sun-Synch' from dual union all
select 26333,'026333','1999-057HS','CZ-4 debris','',66738,'In Earth orbit','','2000-05-03 00:00:00',100.56,709,862,98.43,'Sun-Synch' from dual union all
select 26334,'026334','1999-057HT','CZ-4 debris','',66738,'Reentered','2001-07-18 00:00:00','2000-05-02 00:00:00',98.85,673,735,98.36,'Sun-Synch' from dual union all
select 26335,'026335','1999-057HU','CZ-4 debris','',66738,'Reentered','2001-07-11 00:00:00','2000-05-03 00:00:00',100.46,695,866,98.4,'Sun-Synch' from dual union all
select 26336,'026336','1999-057HV','CZ-4 debris','',66738,'In Earth orbit','','2000-05-03 00:00:00',100.74,717,871,98.49,'Sun-Synch' from dual union all
select 26337,'026337','1999-057HW','CZ-4 debris','',66738,'Reentered','2001-07-11 00:00:00','2000-05-03 00:00:00',98.08,612,722,98.65,'Sun-Synch' from dual union all
select 26338,'026338','1999-057HX','CZ-4 debris','',66738,'In Earth orbit','','2000-05-03 00:00:00',100.61,714,860,98.47,'Sun-Synch' from dual union all
select 26339,'026339','1999-057HY','CZ-4 debris','',66738,'In Earth orbit','','2000-05-03 00:00:00',100.61,709,865,98.44,'Sun-Synch' from dual union all
select 2634,'002634','1966-118A','OPS 1584','MULTIGROUP 1',13525,'Reentered Attached','1969-04-05 00:00:00','1967-01-29 00:00:00',94.36,484,492,75.02,'Intermediate' from dual union all
select 26340,'026340','1999-057HZ','CZ-4 debris','',66738,'Reentered','2000-11-10 00:00:00','2000-05-02 00:00:00',99.43,682,780,98.4,'Sun-Synch' from dual union all
select 26341,'026341','1999-057JA','CZ-4 debris','',66738,'Reentered','2003-11-10 00:00:00','2000-05-03 00:00:00',100.65,708,871,98.46,'Sun-Synch' from dual union all
select 26342,'026342','1999-057JB','CZ-4 debris','',66738,'Reentered','2000-11-13 00:00:00','2000-05-02 00:00:00',100.56,704,867,98.41,'Sun-Synch' from dual union all
select 26343,'026343','1999-057JC','CZ-4 debris','',66738,'Reentered','2004-04-16 00:00:00','2000-05-03 00:00:00',100.7,709,875,98.47,'Sun-Synch' from dual union all
select 26344,'026344','1999-057JD','CZ-4 debris','',66738,'Reentered','2003-05-01 00:00:00','2000-05-03 00:00:00',100.52,706,861,98.43,'Sun-Synch' from dual union all
select 26345,'026345','1999-057JE','CZ-4 debris','',66738,'Reentered','2003-03-08 00:00:00','2000-05-03 00:00:00',100.52,706,861,98.46,'Sun-Synch' from dual union all
select 26346,'026346','1999-057JF','CZ-4 debris','',66738,'Reentered','2009-09-23 00:00:00','2000-05-03 00:00:00',100.57,708,864,98.46,'Sun-Synch' from dual union all
select 26347,'026347','1999-057JG','CZ-4 debris','',66738,'In Earth orbit','','2000-05-02 00:00:00',100.53,726,841,98.43,'Sun-Synch' from dual union all
select 26348,'026348','1999-057JH','CZ-4 debris','',66738,'Reentered','2007-01-19 00:00:00','2000-05-02 00:00:00',100.63,681,896,98.27,'Sun-Synch' from dual union all
select 26349,'026349','1999-057JJ','CZ-4 debris','',66738,'Reentered','2003-02-08 00:00:00','2000-05-03 00:00:00',100.9,695,908,98.33,'Sun-Synch' from dual union all
select 2635,'002635','1966-118B','OPS 1584 sensor cover','',13525,'Reentered','1967-03-23 00:00:00','1967-01-29 00:00:00',93.92,461,472,75,'Intermediate' from dual union all
select 26350,'026350','1999-057JK','CZ-4 debris','',66738,'In Earth orbit','','2000-05-03 00:00:00',98.83,671,735,98.59,'Sun-Synch' from dual union all
select 26351,'026351','1999-057JL','CZ-4 debris','',66738,'In Earth orbit','','2000-05-03 00:00:00',100.27,719,824,98.5,'Sun-Synch' from dual union all
select 26352,'026352','2000-022A','GOES 11','GOES L',66924,'In Earth orbit','','2000-06-08 00:00:00',1436.16,35782,35794,.21,'Stationary' from dual union all
select 26353,'026353','2000-022B','Centaur AC-137','Centaur IIA',66924,'Reentered','2013-11-07 00:00:00','2000-05-13 00:00:00',752.76,248,41819,20.15,'Highly Ellip' from dual union all
select 26354,'026354','2000-023A','Kosmos-2370','Neman',66925,'Deorbited','2001-05-03 00:00:00','2000-06-24 00:00:00',89.88,240,296,64.75,'Intermediate' from dual union all
select 26355,'026355','2000-023B','Blok-I','11S510',66925,'Reentered','2000-05-06 00:00:00','2000-05-06 00:00:00',87.12,121,143,64.78,'Intermediate' from dual union all
select 26356,'026356','2000-024A','DSP F20','DSP 18',66926,'In Earth orbit','','2000-05-09 00:00:00',1443.96,35911,35970,2.9,'Drift GEO' from dual union all
select 26357,'026357','2000-024B','Titan 4B-29 Stage 2','Titan 4 Stage 2',66926,'Reentered','2000-07-28 00:00:00','2000-06-09 00:00:00',92.51,193,603,28.63,'Intermediate' from dual union all
select 26358,'026358','2000-024C','IUS-22 SRM-1','Orbus 21',66926,'In Earth orbit','','2000-05-08 00:00:00',637.24,332,35972,28.6,'GEO Transfer' from dual union all
select 26359,'026359','2000-024D','IUS-22 SRM-2','Orbus 6E',66926,'In Earth orbit','','2000-05-08 00:00:00',1444.32,35924,35971,2.9,'Drift GEO' from dual union all
select 2636,'002636','1966-118C','OPS 1584 sensor cover','',13525,'Reentered','1967-03-23 00:00:00','1967-01-29 00:00:00',93.93,455,479,75.04,'Intermediate' from dual union all
select 26360,'026360','2000-025A','GPS IIR-4','Navstar GPS SVN 51',66927,'In Earth orbit','','2000-06-11 00:00:00',718.02,20133,20234,54.9,'Medium' from dual union all
select 26361,'026361','2000-025B','Delta 278','Delta SSPS AJ-10-118K',66927,'Reentered','2000-08-23 00:00:00','2000-06-10 00:00:00',98.2,193,1153,37.64,'Intermediate' from dual union all
select 26362,'026362','2000-025C','Star 48B','Star 48B',66927,'Reentered','2011-06-22 00:00:00','2000-07-21 00:00:00',347.51,189,19801,39.03,'Highly Ellip' from dual union all
select 26363,'026363','1986-017NN','EO-28 EVA-1 debris','',59210,'Reentered','2000-05-25 00:00:00','2000-05-14 00:00:00',91.85,358,373,51.65,'Intermediate' from dual union all
select 26364,'026364','1986-017NP','EO-28 EVA-1 debris','',59210,'Reentered','2000-08-07 00:00:00','2000-05-13 00:00:00',91.99,365,379,51.66,'Intermediate' from dual union all
select 26365,'026365','2000-026A','IKA-1','EPN 813IP/003 (Iridium)',66928,'In Earth orbit','','2000-05-24 00:00:00',95.65,543,557,86.38,'Polar' from dual union all
select 26366,'026366','2000-026B','IKA-2','EPN 813IP/007 (Iridium)',66928,'In Earth orbit','','2000-05-23 00:00:00',95.6,542,554,86.38,'Polar' from dual union all
select 26367,'026367','2000-026C','Briz-KM No. 72501','Briz-KM 14S45',66928,'Reentered','2000-06-21 00:00:00','2000-05-25 00:00:00',91.38,174,511,86.36,'Polar' from dual union all
select 26368,'026368','2000-027A','Atlantis (STS-101)','OV-104',66929,'Landed','2000-05-29 00:00:00','2000-05-21 00:00:00',91.04,319,333,51.58,'Intermediate' from dual union all
select 26369,'026369','2000-028A','Eutelsat W4','Eutelsat W4',66930,'In Earth orbit','','2000-07-08 00:00:00',1436.21,35600,35978,.01,'Stationary' from dual union all
select 2637,'002637','1965-020EL','deb Blok Kosmos','',11770,'Reentered','1967-02-19 00:00:00','1967-01-04 00:00:00',94.63,246,756,55.97,'Intermediate' from dual union all
select 26370,'026370','2000-028B','Centaur AC-201','Centaur IIIA AC-201',66930,'Reentered','2013-10-27 00:00:00','2000-07-19 00:00:00',812.77,230,44713,20.33,'Highly Ellip' from dual union all
select 26371,'026371','1991-015K','deb Ariane V42','',64184,'Reentered','2000-06-01 00:00:00','2000-06-01 00:00:00',92.56,138,662,7.29,'Equatorial' from dual union all
select 26372,'026372','2000-029A','Gorizont','Gorizont No. 45L',66931,'In Earth orbit','','2000-07-06 00:00:00',1435.98,35783,35786,1.39,'Stationary' from dual union all
select 26373,'026373','2000-029B','Briz-M No. 88502L','Briz-M 14S43 No. 88502',66931,'In Earth orbit','','2000-07-08 00:00:00',1434.17,34689,36809,1.58,'Drift GEO' from dual union all
select 26374,'026374','2000-030A','TSX-5','TSX-5',66932,'In Earth orbit','','2000-06-19 00:00:00',106.26,403,1704,68.95,'Intermediate' from dual union all
select 26375,'026375','2000-030B','Orion 38','Orion 38',66932,'In Earth orbit','','2000-06-19 00:00:00',106.2,403,1698,68.96,'Intermediate' from dual union all
select 26376,'026376','1977-053C','Star 27 S/N 0031','Star 27 TE-M-616-9',42181,'In Earth orbit','','2000-06-09 00:00:00',704.93,19318,20401,62.97,'Medium' from dual union all
select 26377,'026377','2000-029C','Briz-M DTB','Briz-M tank',66931,'Reentered','2008-10-06 00:00:00','2000-06-13 00:00:00',618.86,370,34986,48.76,'GEO Transfer' from dual union all
select 26378,'026378','2000-031A','Ekspress 3A','Ekspress-A No. 3',66933,'In Earth orbit','','2000-08-01 00:00:00',1436.14,35780,35795,.13,'Stationary' from dual;
insert into SATELLITE
select 26379,'026379','2000-031B','8S812','8S812',66933,'Reentered','2000-06-26 00:00:00','2000-06-25 00:00:00',87.78,162,167,51.62,'Intermediate' from dual union all
select 2638,'002638','1963-014AN','deb West Ford','RCS 0.04',9245,'Reentered','1981-12-15 00:00:00','1981-11-24 00:00:00',123.86,325,3381,84.3,'Medium' from dual union all
select 26380,'026380','2000-031C','Perekhodnik','Sredniy perekhodnik',66933,'Reentered','2000-06-25 00:00:00','2000-06-24 00:00:00',87.96,170,176,51.61,'Intermediate' from dual union all
select 26381,'026381','2000-031D','Blok DM-2 No. 89L','11S861 No. 89L',66933,'In Earth orbit','','2000-07-30 00:00:00',1448.06,35959,36082,.15,'Drift GEO' from dual union all
select 26382,'026382','2000-032A','Feng Yun 2B','Feng Yun Er Yi',66934,'In Earth orbit','','2000-07-03 00:00:00',1436.65,35791,35804,1.07,'Drift GEO' from dual union all
select 26383,'026383','2000-032B','CZ-3 Stage 3','CZ-3 Stage 3',66934,'In Earth orbit','','2000-07-04 00:00:00',631.89,235,35794,27.45,'GEO Transfer' from dual union all
select 26384,'026384','2000-033A','Nadezhda','Nadezhda 17F118 No. 701',66935,'In Earth orbit','','2000-07-07 00:00:00',98.66,683,707,98.12,'Sun-Synch' from dual union all
select 26385,'026385','2000-033B','Hangtian Qinghua 1','Tsinghua 1',66935,'In Earth orbit','','2000-07-08 00:00:00',98.68,684,708,98.13,'Sun-Synch' from dual union all
select 26386,'026386','2000-033C','SNAP-1','SNAP-1',66935,'In Earth orbit','','2000-07-07 00:00:00',98.65,683,706,98.13,'Sun-Synch' from dual union all
select 26387,'026387','2000-033D','S3M','S3M',66935,'In Earth orbit','','2000-07-28 00:00:00',98.54,671,707,98.13,'Sun-Synch' from dual union all
select 26388,'026388','2000-034A','TDRS 8','TDRS H',66936,'In Earth orbit','','2000-12-14 00:00:00',1435.96,35669,35899,6.72,'Inclined GEO' from dual union all
select 26389,'026389','2000-034B','Centaur AC-139','Centaur IIA',66936,'In Earth orbit','','2000-07-08 00:00:00',480.34,236,27642,26.99,'Highly Ellip' from dual union all
select 2639,'002639','1967-001A','INTELSAT II F-2','INTELSAT II F-2',15256,'In Earth orbit','','1967-07-18 00:00:00',1436.19,35783,35794,1.69,'Stationary' from dual union all
select 26390,'026390','2000-035A','Sirius FM-1','Sirius CD Radio 1',66937,'In Earth orbit','','2000-07-07 00:00:00',1418.5,23783,47100,63.34,'Near-synch' from dual union all
select 26391,'026391','2000-035B','8S812','8S812',66937,'Reentered','2000-07-01 00:00:00','2000-07-01 00:00:00',87.25,130,147,64.84,'Intermediate' from dual union all
select 26392,'026392','2000-035C','Perekhodnik','Sredniy perekhodnik',66937,'Reentered','2000-07-01 00:00:00','2000-07-01 00:00:00',87.48,136,164,64.83,'Intermediate' from dual union all
select 26393,'026393','2000-035D','Blok DM3 No. 29L','Blok DM3 No. 29L',66937,'In Earth orbit','','2000-07-09 00:00:00',988.6,6084,46906,63.37,'Highly Ellip' from dual union all
select 26394,'026394','2000-036A','Kosmos-2371','Geizer No. 22L',66938,'In Earth orbit','','2000-07-11 00:00:00',1441.58,35872,35916,1.44,'Drift GEO' from dual union all
select 26395,'026395','2000-036B','8S812','8S812',66938,'Reentered','2000-07-07 00:00:00','2000-07-07 00:00:00',86.94,118,128,51.62,'Intermediate' from dual union all
select 26396,'026396','2000-036C','Perekhodnik','Sredniy perekhodnik',66938,'Reentered','2000-07-05 00:00:00','2000-07-05 00:00:00',87.86,167,171,51.61,'Intermediate' from dual union all
select 26397,'026397','2000-036D','Blok DM-2 No. 90L','11S861 No. 90L',66938,'In Earth orbit','','2000-07-18 00:00:00',1404.4,34494,35834,1.52,'Drift GEO' from dual union all
select 26398,'026398','2000-036E','SOZ','SOZ',66938,'Reentered','2012-03-27 00:00:00','2000-07-14 00:00:00',635.88,351,35883,47.35,'GEO Transfer' from dual union all
select 26399,'026399','2000-036F','SOZ','SOZ',66938,'Reentered','2001-03-16 00:00:00','2000-07-21 00:00:00',629.7,300,35616,47.27,'GEO Transfer' from dual union all
select 264,'000264','61 Omicron 83','deb r/b Ablestar','',7273,'In Earth orbit','','1962-03-09 00:00:00',104.22,928,987,66.57,'Intermediate' from dual union all
select 2640,'002640','1967-001B','Delta 44','DSV-3E-3 S/N 20208',15256,'Reentered','1967-04-13 00:00:00','1967-02-14 00:00:00',91.81,240,487,28.75,'Intermediate' from dual union all
select 26400,'026400','2000-037A','Zvezda','ISS SM 77KSM No. 128-01',66939,'In Earth orbit','','2000-07-14 00:00:00',90.98,288,358,51.58,'Intermediate' from dual union all
select 26401,'026401','2000-037B','8S812','8S812',66939,'Reentered','2000-07-17 00:00:00','2000-07-14 00:00:00',88.98,171,278,51.6,'Intermediate' from dual union all
select 26402,'026402','2000-038A','Echostar VI','Echostar 6',66940,'In Earth orbit','','2000-07-21 00:00:00',1393.49,33980,35917,.17,'Drift GEO' from dual union all
select 26403,'026403','2000-038B','Centaur AC-161','Centaur IIA',66940,'Reentered','2000-10-01 00:00:00','2000-07-24 00:00:00',660.24,137,37340,26.49,'GEO Transfer' from dual union all
select 26404,'026404','2000-039A','MITA','MITA-NINA',66941,'Reentered','2001-08-15 00:00:00','2000-07-22 00:00:00',93.46,415,473,87.27,'Polar' from dual union all
select 26405,'026405','2000-039B','CHAMP','CHAMP',66941,'Reentered','2010-09-19 00:00:00','2000-07-23 00:00:00',93.54,421,475,87.27,'Polar' from dual union all
select 26406,'026406','2000-039C','S3M','S3M',66941,'Reentered','2001-08-30 00:00:00','2000-07-22 00:00:00',93.5,417,475,87.27,'Polar' from dual union all
select 26407,'026407','2000-040A','GPS IIR-5','Navstar GPS SVN 44',66942,'In Earth orbit','','2000-07-27 00:00:00',722.98,20184,20427,55.06,'Medium' from dual union all
select 26408,'026408','2000-040B','Delta 279','Delta SSPS AJ-10-118K',66942,'Reentered','2000-10-14 00:00:00','2000-07-23 00:00:00',99.54,194,1280,37.59,'Intermediate' from dual union all
select 26409,'026409','2000-040C','Star 48B','Star 48B',66942,'Reentered','2003-07-27 00:00:00','2000-07-23 00:00:00',356.22,182,20354,38.98,'Highly Ellip' from dual union all
select 2641,'002641','1967-001C','Delta 44 attach clamp','',15256,'Reentered','1967-02-24 00:00:00','1967-01-20 00:00:00',92.42,259,528,28.73,'Intermediate' from dual union all
select 26410,'026410','2000-041A','Cluster Samba','Cluster FM7 (C3)',66943,'In Earth orbit','','2000-07-31 00:00:00',3430.27,17007,120923,90.56,'Very High' from dual union all
select 26411,'026411','2000-041B','Cluster Salsa','Cluster FM6 (C2)',66943,'In Earth orbit','','2000-08-17 00:00:00',3441.99,17313,120961,90.78,'Very High' from dual union all
select 26412,'026412','2000-041C','Fregat','Fregat 14S44 No. 1003 (3',66943,'Deorbited','2000-07-16 00:00:00','2000-07-16 00:00:00',319.61,156,18058,64.83,'Highly Ellip' from dual union all
select 26413,'026413','1992-072H','deb Ariane V54','',64784,'Reentered','2000-09-24 00:00:00','2000-07-18 00:00:00',113.15,240,2503,6.99,'Equatorial' from dual union all
select 26414,'026414','2000-042A','Sindri','Mightysat 2.1',66944,'Reentered','2002-11-12 00:00:00','2000-07-27 00:00:00',95.99,548,585,97.8,'Sun-Synch' from dual union all
select 26415,'026415','2000-042B','Minotaur Stage 4','Orion 38',66944,'Reentered','2014-03-24 00:00:00','2000-07-27 00:00:00',95.94,548,580,97.82,'Sun-Synch' from dual union all
select 26416,'026416','1999-057JM','CZ-4 debris','',66738,'In Earth orbit','','2000-07-26 00:00:00',100.57,712,860,98.4,'Sun-Synch' from dual union all
select 26417,'026417','1999-057JN','CZ-4 debris','',66738,'Reentered','2000-12-12 00:00:00','2000-07-25 00:00:00',98.81,660,744,98.4,'Sun-Synch' from dual union all
select 26418,'026418','1999-057JP','CZ-4 debris','',66738,'Reentered','2000-08-03 00:00:00','2000-07-26 00:00:00',95.32,503,566,98.44,'Sun-Synch' from dual union all
select 26419,'026419','1999-057JQ','CZ-4 debris','',66738,'Reentered','2000-08-10 00:00:00','2000-07-27 00:00:00',95.99,523,610,98.32,'Sun-Synch' from dual union all
select 2642,'002642','1967-002A','OPS 1664','CORONA J-34',15257,'Reentered Attached','1967-02-02 00:00:00','1967-01-22 00:00:00',89.83,174,358,80.07,'Intermediate' from dual union all
select 26420,'026420','1999-057JR','CZ-4 debris','',66738,'Reentered','2000-08-11 00:00:00','2000-07-25 00:00:00',94.03,440,504,98.67,'Sun-Synch' from dual union all
select 26421,'026421','1999-057JS','CZ-4 debris','',66738,'Reentered','2001-01-13 00:00:00','2000-07-27 00:00:00',97.2,576,674,98.67,'Sun-Synch' from dual union all
select 26422,'026422','1999-057JT','CZ-4 debris','',66738,'Reentered','2005-11-08 00:00:00','2000-07-27 00:00:00',100.33,711,838,98.45,'Sun-Synch' from dual union all
select 26423,'026423','1999-057JU','CZ-4 debris','',66738,'In Earth orbit','','2000-07-19 00:00:00',101.07,703,916,98.5,'Sun-Synch' from dual union all
select 26424,'026424','1999-057JV','CZ-4 debris','',66738,'Reentered','2001-05-27 00:00:00','2000-07-27 00:00:00',98.1,628,708,98.46,'Sun-Synch' from dual union all
select 26425,'026425','1999-057JW','CZ-4 debris','',66738,'Reentered','2000-11-04 00:00:00','2000-07-25 00:00:00',96.22,524,631,98.52,'Sun-Synch' from dual union all
select 26426,'026426','1999-057JX','CZ-4 debris','',66738,'Reentered','2001-12-16 00:00:00','2000-07-26 00:00:00',100.06,694,829,98.43,'Sun-Synch' from dual union all
select 26427,'026427','1999-057JY','CZ-4 debris','',66738,'Reentered','2001-11-25 00:00:00','2000-07-24 00:00:00',98.25,571,779,98.24,'Sun-Synch' from dual union all
select 26428,'026428','1999-057JZ','CZ-4 debris','',66738,'Reentered','2002-05-21 00:00:00','2000-07-27 00:00:00',100.14,702,828,98.37,'Sun-Synch' from dual union all
select 26429,'026429','1999-057KA','CZ-4 debris','',66738,'Reentered','2017-07-18 00:00:00','2000-07-26 00:00:00',100.38,708,846,98.45,'Sun-Synch' from dual union all
select 2643,'002643','1967-001D','FW4D','FW4D S/N 00004',15256,'In Earth orbit','','1967-03-11 00:00:00',532.5,361,30408,26.73,'Highly Ellip' from dual union all
select 26430,'026430','1999-057KB','CZ-4 debris','',66738,'In Earth orbit','','2000-07-16 00:00:00',100.89,703,899,98.29,'Sun-Synch' from dual union all
select 26431,'026431','1999-057KC','CZ-4 debris','',66738,'Reentered','2000-11-05 00:00:00','2000-07-27 00:00:00',98.31,626,730,98.48,'Sun-Synch' from dual union all
select 26432,'026432','1999-057KD','CZ-4 debris','',66738,'In Earth orbit','','2000-07-19 00:00:00',102.17,690,1033,98.57,'Sun-Synch' from dual union all
select 26433,'026433','1999-057KE','CZ-4 debris','',66738,'Reentered','2000-12-21 00:00:00','2000-07-25 00:00:00',98.91,664,750,98.45,'Sun-Synch' from dual union all
select 26434,'026434','1999-057KF','CZ-4 debris','',66738,'Reentered','2000-07-28 00:00:00','2000-07-17 00:00:00',92.44,364,425,98.41,'Sun-Synch' from dual union all
select 26435,'026435','1999-057KG','CZ-4 debris','',66738,'Reentered','2000-07-31 00:00:00','2000-07-27 00:00:00',92.38,372,411,98.67,'Sun-Synch' from dual union all
select 26436,'026436','1999-057KH','CZ-4 debris','',66738,'In Earth orbit','','2000-07-25 00:00:00',111.56,733,1864,99.5,'Sun-Synch' from dual union all
select 26437,'026437','1999-057KJ','CZ-4 debris','',66738,'Reentered','2000-07-28 00:00:00','2000-07-27 00:00:00',92.85,386,443,98.45,'Sun-Synch' from dual union all
select 26438,'026438','1999-057KK','CZ-4 debris','',66738,'Reentered','2000-10-31 00:00:00','2000-07-26 00:00:00',99.55,667,808,98.46,'Sun-Synch' from dual union all
select 26439,'026439','1999-057KL','CZ-4 debris','',66738,'Reentered','2000-09-25 00:00:00','2000-07-25 00:00:00',95.87,499,623,98.78,'Sun-Synch' from dual union all
select 2644,'002644','1966-041D','NNS O-9 despin weight','Transit high debris',13448,'In Earth orbit','','1966-06-27 00:00:00',105.64,852,1196,89.93,'Polar' from dual union all
select 26440,'026440','1999-057KM','CZ-4 debris','',66738,'Reentered','2000-08-03 00:00:00','2000-07-27 00:00:00',92.83,382,445,98.75,'Sun-Synch' from dual union all
select 26441,'026441','1999-057KN','CZ-4 debris','',66738,'In Earth orbit','','2000-07-26 00:00:00',100.57,715,856,98.44,'Sun-Synch' from dual union all
select 26442,'026442','1999-057KP','CZ-4 debris','',66738,'In Earth orbit','','2000-07-26 00:00:00',100.23,713,826,98.49,'Sun-Synch' from dual union all
select 26443,'026443','1999-057KQ','CZ-4 debris','',66738,'In Earth orbit','','2000-07-18 00:00:00',100.66,711,869,98.43,'Sun-Synch' from dual union all
select 26444,'026444','1999-057KR','CZ-4 debris','',66738,'In Earth orbit','','2000-07-26 00:00:00',99.7,673,816,98.51,'Sun-Synch' from dual union all
select 26445,'026445','1999-057KS','CZ-4 debris','',66738,'Reentered','2001-10-20 00:00:00','2000-07-27 00:00:00',99.49,690,779,98.48,'Sun-Synch' from dual union all
select 26446,'026446','1999-057KT','CZ-4 debris','',66738,'In Earth orbit','','2000-07-26 00:00:00',100.95,713,895,98.42,'Sun-Synch' from dual union all
select 26447,'026447','1999-057KU','CZ-4 debris','',66738,'Reentered','2001-06-19 00:00:00','2000-07-22 00:00:00',99.97,699,816,98.35,'Sun-Synch' from dual union all
select 26448,'026448','1999-057KV','CZ-4 debris','',66738,'In Earth orbit','','2000-07-25 00:00:00',101.09,708,912,98.39,'Sun-Synch' from dual union all
select 26449,'026449','1999-057KW','CZ-4 debris','',66738,'In Earth orbit','','2000-07-26 00:00:00',100.56,712,859,98.42,'Sun-Synch' from dual union all
select 2645,'002645','1967-003A','OPS 9321','IDCSP 8',15258,'In Earth orbit','','1967-05-28 00:00:00',1329.53,33505,33843,.29,'Medium' from dual union all
select 26450,'026450','1999-057KX','CZ-4 debris','',66738,'Reentered','2001-10-15 00:00:00','2000-07-26 00:00:00',99.96,692,822,98.47,'Sun-Synch' from dual union all
select 26451,'026451','2000-043A','PAS 9','PAS 9',66945,'In Earth orbit','','2000-08-04 00:00:00',1427.33,35444,35786,.07,'Drift GEO' from dual union all
select 26452,'026452','2000-043B','Blok DM-SL No. 4L','Blok DM-SL No. 4L',66945,'In Earth orbit','','2000-08-06 00:00:00',664.36,1911,35775,1.19,'GEO Transfer' from dual union all
select 26453,'026453','1999-057KY','CZ-4 debris','',66738,'Reentered','2000-09-19 00:00:00','2000-08-01 00:00:00',95.26,484,579,98.77,'Sun-Synch' from dual union all
select 26454,'026454','1999-057KZ','CZ-4 debris','',66738,'In Earth orbit','','2000-08-02 00:00:00',100.52,710,857,98.44,'Sun-Synch' from dual union all
select 26455,'026455','1999-057LA','CZ-4 debris','',66738,'Reentered','2000-11-27 00:00:00','2000-07-31 00:00:00',99.22,653,790,98.47,'Sun-Synch' from dual union all
select 26456,'026456','1999-057LB','CZ-4 debris','',66738,'Reentered','2000-09-18 00:00:00','2000-08-02 00:00:00',97.4,594,675,98.42,'Sun-Synch' from dual union all
select 26457,'026457','1999-057LC','CZ-4 debris','',66738,'Reentered','2001-06-21 00:00:00','2000-07-25 00:00:00',98.35,660,700,98.29,'Sun-Synch' from dual union all
select 26458,'026458','1999-057LD','CZ-4 debris','',66738,'Reentered','2001-01-10 00:00:00','2000-07-30 00:00:00',97.36,588,676,98.45,'Sun-Synch' from dual union all
select 26459,'026459','1999-057LE','CZ-4 debris','',66738,'Reentered','2005-05-18 00:00:00','2000-08-02 00:00:00',100.29,708,837,98.45,'Sun-Synch' from dual union all
select 2646,'002646','1967-004A','Kosmos-138','Zenit-2 No. 43',15259,'Landed','1967-01-27 00:00:00','1967-01-21 00:00:00',89.16,190,275,64.56,'Intermediate' from dual union all
select 26460,'026460','2000-032C','FG-36 AKM','Shanxi AKM',66934,'In Earth orbit','','2000-07-29 00:00:00',1438.05,35714,35936,1.08,'Drift GEO' from dual union all
select 26461,'026461','2000-044A','Progress M1-3','Progress M1 No. 251',66946,'Deorbited','2000-11-01 00:00:00','2000-08-13 00:00:00',91.63,347,362,51.58,'Intermediate' from dual union all
select 26462,'026462','2000-044B','11S510','11S510',66946,'Reentered','2000-08-08 00:00:00','2000-08-08 00:00:00',87.34,139,147,51.64,'Intermediate' from dual union all
select 26463,'026463','2000-045A','Cluster Rumba','Cluster FM5 (C1)',66947,'In Earth orbit','','2000-08-17 00:00:00',3431.09,17239,120715,90.89,'Very High' from dual union all
select 26464,'026464','2000-045B','Cluster Tango','Cluster FM8 (C4)',66947,'In Earth orbit','','2000-07-31 00:00:00',3418.19,16808,120767,90.7,'Very High' from dual union all
select 26465,'026465','2000-045C','Fregat No. 1004','Fregat 14S44 No. 1004 (4',66947,'Deorbited','2000-08-09 00:00:00','2000-08-09 00:00:00',299.76,156,17075,64.86,'Suborbital' from dual union all
select 26466,'026466','1987-078G','deb Ariane V19','',60168,'Reentered','2002-04-22 00:00:00','2000-08-15 00:00:00',305.92,329,16994,7.29,'Highly Ellip' from dual union all
select 26467,'026467','1981-058F','deb Kosmos-1278','',52581,'Reentered','2000-08-14 00:00:00','2000-08-14 00:00:00',95.54,90,1000,62.74,'Intermediate' from dual union all
select 26468,'026468','1964-049N','deb Kosmos-41','',10335,'Reentered','2000-09-03 00:00:00','2000-08-16 00:00:00',233.25,152,12195,65.15,'Medium' from dual;
insert into SATELLITE
select 26469,'026469','2000-046A','Brasilsat B4','Brasilsat B4',66948,'In Earth orbit','','2001-02-21 00:00:00',1436.06,35765,35807,.05,'Stationary' from dual union all
select 2647,'002647','1965-073L','Kosmos-89 part','',11823,'In Earth orbit','','1967-01-16 00:00:00',116.13,1353,1661,56.05,'Intermediate' from dual union all
select 26470,'026470','2000-046B','Nilesat 102','Nilesat 102',66948,'In Earth orbit','','2000-09-10 00:00:00',1436.06,35769,35802,.05,'Stationary' from dual union all
select 26471,'026471','2000-046C','Mini-Spelda','Spelda 40',66948,'In Earth orbit','','2000-08-20 00:00:00',632.87,272,35807,2.98,'GEO Transfer' from dual union all
select 26472,'026472','2000-049G','deb Proton','',66951,'Reentered','2000-09-01 00:00:00','2000-08-29 00:00:00',91.74,183,536,51.62,'Intermediate' from dual union all
select 26473,'026473','2000-047A','USA 152','ONYX 4',66949,'In Earth orbit','','2000-08-19 00:00:00',97.21,574,677,68,'Intermediate' from dual union all
select 26474,'026474','2000-047B','Titan 4B-28 Stage 2','Titan 4 Stage 2',66949,'In Earth orbit','','2000-08-21 00:00:00',97.17,572,674,68,'Intermediate' from dual union all
select 26475,'026475','2000-048B','Delta 280','DCSS F03',66950,'Reentered','2009-02-04 00:00:00','2000-08-26 00:00:00',360.78,185,20634,27.61,'Highly Ellip' from dual union all
select 26476,'026476','2000-048A','DM-F3','DM-F3',66950,'In Earth orbit','','2000-08-26 00:00:00',360.73,187,20629,27.63,'Highly Ellip' from dual union all
select 26477,'026477','2000-049A','Raduga-1','Globus No. 16L',66951,'In Earth orbit','','2000-08-30 00:00:00',1470.89,36377,36553,2.21,'Drift GEO' from dual union all
select 26478,'026478','2000-049B','8S812','8S812',66951,'Reentered','2000-08-30 00:00:00','2000-08-29 00:00:00',87.54,152,153,51.63,'Intermediate' from dual union all
select 26479,'026479','2000-049C','Perekhodnik','Sredniy perekhodnik',66951,'Reentered','2000-08-29 00:00:00','2000-08-29 00:00:00',88.13,177,187,51.64,'Intermediate' from dual union all
select 2648,'002648','1967-004B','8A92E','8A92E',15259,'Reentered','1967-01-28 00:00:00','1967-01-23 00:00:00',88.99,209,241,64.57,'Intermediate' from dual union all
select 26480,'026480','2000-049D','Blok DM-2 No. 87L','11S861 No. 87L',66951,'In Earth orbit','','2000-09-14 00:00:00',1466.51,36277,36483,1.51,'Drift GEO' from dual union all
select 26481,'026481','2000-050A','Zi Yuan-2 01 xing','Jianbing-3 01 xing',66952,'Reentered','2016-03-10 00:00:00','2000-09-05 00:00:00',94.26,474,492,97.42,'Sun-Synch' from dual union all
select 26482,'026482','2000-050B','CZ-4B Stage 3','CZ-4B Stage 3',66952,'Reentered','2002-11-01 00:00:00','2000-09-05 00:00:00',94.21,469,492,97.42,'Sun-Synch' from dual union all
select 26483,'026483','2000-051A','Sirius FM-2','Sirius CD Radio 2',66953,'In Earth orbit','','2009-07-01 00:00:00',1435.93,24394,47173,63.73,'Synchronous' from dual union all
select 26484,'026484','2000-051B','8S812','8S812',66953,'Reentered','2000-09-06 00:00:00','2000-09-05 00:00:00',87.6,144,168,64.84,'Intermediate' from dual union all
select 26485,'026485','2000-051C','Perekhodnik','Sredniy perekhodnik',66953,'Reentered','2000-09-06 00:00:00','2000-09-05 00:00:00',87.44,143,152,64.83,'Intermediate' from dual union all
select 26486,'026486','2000-051D','Blok DM3 No. 22L','Blok DM3 No. 22L',66953,'In Earth orbit','','2000-09-08 00:00:00',987.83,6096,46859,63.39,'Highly Ellip' from dual union all
select 26487,'026487','2000-052A','Eutelsat W1','Eutelsat W1',66954,'In Earth orbit','','2001-04-21 00:00:00',1436.04,35770,35801,.05,'Stationary' from dual union all
select 26488,'026488','2000-052B','Ariane H10-3 V132','Ariane H10-3 S/N T4099',66954,'In Earth orbit','','2000-09-07 00:00:00',663.08,356,37265,6.94,'GEO Transfer' from dual union all
select 26489,'026489','2000-053A','Atlantis (STS-106)','OV-104',66955,'Landed','2000-09-20 00:00:00','2000-09-08 00:00:00',89.32,158,324,51.58,'Intermediate' from dual union all
select 2649,'002649','1967-003B','OPS 9322','IDCSP 9',15258,'In Earth orbit','','1967-05-28 00:00:00',1329.92,33520,33844,.29,'Medium' from dual union all
select 26490,'026490','2000-046D','Ariane H10-3 V131','Ariane H10-3 S/N T4098',66948,'In Earth orbit','','2000-08-20 00:00:00',632.87,272,35807,3,'GEO Transfer' from dual union all
select 26491,'026491','1988-081F','deb Ariane V25','deb H10',61330,'Reentered','2000-09-17 00:00:00','2000-09-11 00:00:00',98.39,173,1191,7.43,'Equatorial' from dual union all
select 26492,'026492','2000-053B','EVA debris','',66955,'Reentered','2000-09-20 00:00:00','2000-09-13 00:00:00',91.45,339,352,51.58,'Intermediate' from dual union all
select 26493,'026493','2000-053C','EVA debris','',66955,'Reentered','2000-09-25 00:00:00','2000-09-15 00:00:00',91.37,336,348,51.57,'Intermediate' from dual union all
select 26494,'026494','2000-054A','Astra 2B','Astra 2B',66956,'In Earth orbit','','2001-05-21 00:00:00',1436.01,35748,35822,.08,'Stationary' from dual union all
select 26495,'026495','2000-054B','GE 7','GE 7',66956,'In Earth orbit','','2001-03-26 00:00:00',1436.09,35774,35799,.04,'Stationary' from dual union all
select 26496,'026496','2000-054C','Sylda 5','Sylda 5',66956,'In Earth orbit','','2000-09-15 00:00:00',642.88,576,36017,6.87,'GEO Transfer' from dual union all
select 26497,'026497','2000-054D','EPS 506','EPS S/N 504',66956,'In Earth orbit','','2000-09-15 00:00:00',642.88,576,36017,6.9,'GEO Transfer' from dual union all
select 26498,'026498','1982-055BP','deb Kosmos-1375','',54364,'In Earth orbit','','2000-09-15 00:00:00',104,919,977,65.82,'Intermediate' from dual union all
select 26499,'026499','2000-031E','SOZ','SOZ',66933,'Reentered','2001-01-21 00:00:00','2000-09-18 00:00:00',634.79,216,35962,47.07,'GEO Transfer' from dual union all
select 265,'000265','61 Omicron 84','deb r/b Ablestar','',7273,'Reentered','1978-10-12 00:00:00','1962-03-05 00:00:00',100.6,602,972,66.65,'Intermediate' from dual union all
select 2650,'002650','1967-003C','OPS 9323','IDCSP 10',15258,'In Earth orbit','','1967-05-28 00:00:00',1330.7,33543,33852,.25,'Medium' from dual union all
select 26500,'026500','2000-031F','SOZ','SOZ',66933,'Reentered','2006-04-17 00:00:00','2000-09-19 00:00:00',638.12,263,36086,47.14,'GEO Transfer' from dual union all
select 26501,'026501','2000-049E','SOZ','SOZ',66951,'Reentered','2001-12-30 00:00:00','2000-09-18 00:00:00',647.19,335,36478,47.41,'GEO Transfer' from dual union all
select 26502,'026502','2000-049F','SOZ','SOZ',66951,'Reentered','2001-05-23 00:00:00','2000-09-19 00:00:00',645.88,274,36472,47.38,'GEO Transfer' from dual union all
select 26503,'026503','1981-021E','deb US-A coolant','',52544,'In Earth orbit','','2000-09-19 00:00:00',103.67,908,957,64.96,'Intermediate' from dual union all
select 26504,'026504','1981-021F','deb US-A coolant','',52544,'In Earth orbit','','2000-09-19 00:00:00',103.64,907,955,64.96,'Intermediate' from dual union all
select 26505,'026505','1981-021G','deb US-A coolant','',52544,'In Earth orbit','','2000-09-19 00:00:00',103.66,908,956,64.96,'Intermediate' from dual union all
select 26506,'026506','1981-021H','deb US-A coolant','',52544,'In Earth orbit','','2000-09-19 00:00:00',103.66,908,956,64.96,'Intermediate' from dual union all
select 26507,'026507','1981-021J','deb US-A coolant','',52544,'In Earth orbit','','2000-09-20 00:00:00',103.69,909,957,64.96,'Intermediate' from dual union all
select 26508,'026508','1981-021K','deb US-A coolant','',52544,'In Earth orbit','','2000-09-19 00:00:00',103.66,908,956,64.97,'Intermediate' from dual union all
select 26509,'026509','1981-021L','deb US-A coolant','',52544,'In Earth orbit','','2000-09-20 00:00:00',103.67,908,957,64.96,'Intermediate' from dual union all
select 2651,'002651','1967-003D','OPS 9324','IDCSP 11',15258,'In Earth orbit','','1967-05-28 00:00:00',1332.12,33563,33889,.24,'Medium' from dual union all
select 26510,'026510','1981-021M','deb US-A coolant','',52544,'In Earth orbit','','2000-09-21 00:00:00',103.7,908,959,64.96,'Intermediate' from dual union all
select 26511,'026511','1981-021N','deb US-A coolant','',52544,'In Earth orbit','','2000-09-21 00:00:00',103.7,910,957,64.96,'Intermediate' from dual union all
select 26512,'026512','1981-021P','deb US-A coolant','',52544,'In Earth orbit','','2000-09-19 00:00:00',103.65,907,956,64.96,'Intermediate' from dual union all
select 26513,'026513','1984-069E','deb US-A coolant','',57123,'In Earth orbit','','2000-09-20 00:00:00',103.72,898,971,65.05,'Intermediate' from dual union all
select 26514,'026514','1984-069F','deb US-A coolant','',57123,'In Earth orbit','','2000-09-21 00:00:00',103.75,903,969,65.05,'Intermediate' from dual union all
select 26515,'026515','1984-069G','deb US-A coolant','',57123,'In Earth orbit','','2000-09-21 00:00:00',103.73,902,968,65.05,'Intermediate' from dual union all
select 26516,'026516','1984-069H','deb US-A coolant','',57123,'In Earth orbit','','2000-09-21 00:00:00',103.69,899,967,65.05,'Intermediate' from dual union all
select 26517,'026517','1984-069J','deb US-A coolant','',57123,'In Earth orbit','','2000-09-21 00:00:00',103.73,901,969,65.05,'Intermediate' from dual union all
select 26518,'026518','1984-069K','deb US-A coolant','',57123,'In Earth orbit','','2000-09-21 00:00:00',103.67,899,966,65.05,'Intermediate' from dual union all
select 26519,'026519','1984-069L','deb US-A coolant','',57123,'In Earth orbit','','2000-09-21 00:00:00',103.75,903,969,65.05,'Intermediate' from dual union all
select 2652,'002652','1967-003E','OPS 9325','IDCSP 12',15258,'In Earth orbit','','1967-05-28 00:00:00',1334.53,33591,33958,.27,'Medium' from dual union all
select 26520,'026520','1984-069M','deb US-A coolant','',57123,'In Earth orbit','','2000-09-21 00:00:00',103.74,902,968,65.05,'Intermediate' from dual union all
select 26521,'026521','1984-069N','deb US-A coolant','',57123,'In Earth orbit','','2000-09-20 00:00:00',103.7,899,968,65.05,'Intermediate' from dual union all
select 26522,'026522','1984-069P','deb US-A coolant','',57123,'In Earth orbit','','2000-09-21 00:00:00',103.71,900,968,65.05,'Intermediate' from dual union all
select 26523,'026523','1984-069Q','deb US-A coolant','',57123,'In Earth orbit','','2000-09-10 00:00:00',103.69,900,966,65.05,'Intermediate' from dual union all
select 26524,'026524','1965-108AK','deb Transtage 8','',11858,'In Earth orbit','','2000-09-18 00:00:00',520.82,654,29476,26.58,'Highly Ellip' from dual union all
select 26525,'026525','1965-108AL','deb Transtage 8','',11858,'In Earth orbit','','2000-09-16 00:00:00',516.65,571,29330,26.55,'Highly Ellip' from dual union all
select 26526,'026526','1965-108AM','deb Transtage 8','',11858,'In Earth orbit','','2000-03-18 00:00:00',486.54,586,27641,27.49,'Highly Ellip' from dual union all
select 26527,'026527','1965-108AN','deb Transtage 8','',11858,'In Earth orbit','','2000-08-25 00:00:00',533.81,647,30193,26.76,'Highly Ellip' from dual union all
select 26528,'026528','1965-108AP','deb Transtage 8','',11858,'In Earth orbit','','2000-09-19 00:00:00',538.35,953,30134,27.44,'Highly Ellip' from dual union all
select 26529,'026529','1965-108AQ','deb Transtage 8','',11858,'In Earth orbit','','2000-09-19 00:00:00',543.39,603,30757,26.65,'Highly Ellip' from dual union all
select 2653,'002653','1967-003F','OPS 9326','IDCSP 13',15258,'In Earth orbit','','1967-05-28 00:00:00',1336.48,33592,34035,.14,'Medium' from dual union all
select 26530,'026530','1965-108AR','deb Transtage 8','',11858,'In Earth orbit','','2000-09-21 00:00:00',520.04,482,29605,27.28,'Highly Ellip' from dual union all
select 26531,'026531','1967-048F','deb NNS O-13 insulation','',15303,'Reentered','2012-01-20 00:00:00','2000-09-20 00:00:00',105.7,997,1058,89.58,'Polar' from dual union all
select 26532,'026532','1979-104N','deb Ariane L01','',47588,'In Earth orbit','','2000-08-08 00:00:00',383.82,872,21362,15.64,'Highly Ellip' from dual union all
select 26533,'026533','1979-104P','deb Ariane L01','',47588,'In Earth orbit','','2000-08-22 00:00:00',392.69,1082,21689,16.1,'Highly Ellip' from dual union all
select 26534,'026534','1979-104Q','deb Ariane L01','',47588,'In Earth orbit','','2000-08-04 00:00:00',606.77,471,34256,15.73,'GEO Transfer' from dual union all
select 26535,'026535','1979-104R','deb Ariane L01','',47588,'In Earth orbit','','2000-09-17 00:00:00',441.07,421,25211,20.03,'Highly Ellip' from dual union all
select 26536,'026536','2000-055A','NOAA 16','NOAA L',66957,'In Earth orbit','','2000-09-23 00:00:00',102.04,848,863,98.79,'Sun-Synch' from dual union all
select 26537,'026537','1983-044FR','deb Kosmos-1461','',55801,'Reentered','2001-02-15 00:00:00','2000-09-25 00:00:00',97.44,527,746,65.01,'Intermediate' from dual union all
select 26538,'026538','2000-056A','Kosmos-2372','Yenisey',66958,'Deorbited','2001-04-20 00:00:00','2000-09-26 00:00:00',89.96,210,335,64.78,'Intermediate' from dual union all
select 26539,'026539','2000-056B','11S772','11S772',66958,'Reentered','2000-10-11 00:00:00','2000-09-25 00:00:00',89.96,215,330,64.8,'Intermediate' from dual union all
select 2654,'002654','1967-003G','OPS 9327','IDCSP 14',15258,'In Earth orbit','','1967-05-29 00:00:00',1339.45,33646,34100,.29,'Medium' from dual union all
select 26540,'026540','2000-056C','Sep Motor Cover','Sep Motor Cover',66958,'Reentered','2000-10-14 00:00:00','2000-09-26 00:00:00',92.59,205,598,64.73,'Intermediate' from dual union all
select 26541,'026541','2000-056D','Sep Motor Cover','Sep Motor Cover',66958,'Reentered','2000-10-12 00:00:00','2000-09-26 00:00:00',92.44,208,581,64.84,'Intermediate' from dual union all
select 26542,'026542','2000-056E','Sep Motor Cover','Sep Motor Cover',66958,'Reentered','2000-10-08 00:00:00','2000-09-25 00:00:00',92.17,206,556,64.83,'Intermediate' from dual union all
select 26543,'026543','2000-056F','Sep Motor Cover','Sep Motor Cover',66958,'Reentered','2000-10-11 00:00:00','2000-09-25 00:00:00',92.3,207,567,64.72,'Intermediate' from dual union all
select 26544,'026544','2000-056G','deb 11S772','',66958,'Reentered','2000-09-26 00:00:00','2000-09-26 00:00:00',89.04,185,270,64.78,'Intermediate' from dual union all
select 26545,'026545','2000-057A','Malaysia-OSCAR-46','Tiungsat-1',66959,'In Earth orbit','','2000-10-11 00:00:00',97.46,633,642,64.56,'Intermediate' from dual union all
select 26546,'026546','2000-057B','Megsat-1','Megsat-1',66959,'In Earth orbit','','2000-10-10 00:00:00',97.58,640,645,64.56,'Intermediate' from dual union all
select 26547,'026547','2000-057C','Unisat','Unisat',66959,'In Earth orbit','','2000-10-11 00:00:00',97.55,639,644,64.56,'Intermediate' from dual union all
select 26548,'026548','2000-057D','Saudi-OSCAR-41','Saudisat 1A  (Aprize)',66959,'In Earth orbit','','2000-10-10 00:00:00',97.65,639,654,64.56,'Intermediate' from dual union all
select 26549,'026549','2000-057E','Saudi-OSCAR-42','Saudisat 1B  (Aprize)',66959,'In Earth orbit','','2000-10-10 00:00:00',97.71,639,659,64.56,'Intermediate' from dual union all
select 2655,'002655','1967-003H','OPS 9328','IDCSP 15',15258,'In Earth orbit','','1967-03-27 00:00:00',1343.04,33639,34251,.09,'Medium' from dual union all
select 26550,'026550','2000-057F','R-36M2 DS','15F183 No. 51099801UP',66959,'In Earth orbit','','2000-10-11 00:00:00',104.46,586,1353,64.56,'Intermediate' from dual union all
select 26551,'026551','2000-057G','Dnepr plume shield','GDS',66959,'Reentered','2012-06-10 00:00:00','2000-10-11 00:00:00',97.78,639,666,64.56,'Intermediate' from dual union all
select 26552,'026552','2000-058A','Kosmos-2373','Kometa No. 20',66960,'Landed','2000-11-13 00:00:00','2000-10-11 00:00:00',89.46,211,285,70.34,'Intermediate' from dual union all
select 26553,'026553','2000-058B','11S510','11S510',66960,'Reentered','2000-10-02 00:00:00','2000-09-30 00:00:00',88.86,188,249,70.36,'Intermediate' from dual union all
select 26554,'026554','2000-059A','GE-1A','GE-1A',66961,'In Earth orbit','','2000-10-11 00:00:00',1435.35,35768,35776,.05,'Drift GEO' from dual union all
select 26555,'026555','2000-059B','8S812','8S812',66961,'Reentered','2000-10-03 00:00:00','2000-10-03 00:00:00',87,113,139,51.6,'Intermediate' from dual union all
select 26556,'026556','2000-059C','Perekhodnik','Sredniy perekhodnik',66961,'Reentered','2000-10-03 00:00:00','2000-10-02 00:00:00',87.55,63,244,51.57,'ERROR - Unexpected value "TA"' from dual union all
select 26557,'026557','2000-059D','Blok DM3 No. 13L','Blok DM3 No. 13L',66961,'In Earth orbit','','2000-10-09 00:00:00',771.54,7257,35718,15.98,'Highly Ellip' from dual union all
select 26558,'026558','62 B Gamma 2','Altair','X-248',8167,'Reentered','1987-12-20 00:00:00','1962-10-02 00:00:00',2185.34,281,98530,33.3,'Very High' from dual;
insert into SATELLITE
select 26559,'026559','2000-060A','N-SAT-110','Superbird 5',66962,'In Earth orbit','','2001-02-28 00:00:00',1436.09,35784,35788,.06,'Stationary' from dual union all
select 2656,'002656','1967-005A','Kosmos-139','OGCh No. 07L',15260,'Deorbited','1967-01-25 00:00:00','1967-01-25 00:00:00',87.31,123,160,49.7,'Intermediate' from dual union all
select 26560,'026560','2000-060B','Ariane H10-3 V133','Ariane H10-3 S/N T4097',66962,'Reentered','2004-12-12 00:00:00','2000-10-08 00:00:00',630.65,216,35749,6.95,'GEO Transfer' from dual union all
select 26561,'026561','2000-061A','HETE 2','HETE 2',66963,'In Earth orbit','','2000-10-10 00:00:00',97,595,636,1.95,'Equatorial' from dual union all
select 26562,'026562','2000-061B','Orion 38','Orion 38',66963,'In Earth orbit','','2000-10-10 00:00:00',96.97,594,634,1.96,'Equatorial' from dual union all
select 26563,'026563','2000-062A','Discovery (STS-92)','OV-103',66964,'Landed','2000-10-24 00:00:00','2000-10-13 00:00:00',91.39,311,375,51.57,'Intermediate' from dual union all
select 26564,'026564','2000-063A','Kosmos-2374','Uragan No. 83L',66965,'In Earth orbit','','2000-10-14 00:00:00',675.51,19122,19127,64.85,'Medium' from dual union all
select 26565,'026565','2000-063B','Kosmos-2375','Uragan No. 87L',66965,'In Earth orbit','','2000-10-16 00:00:00',675.47,19119,19128,64.84,'Medium' from dual union all
select 26566,'026566','2000-063C','Kosmos-2376','Uragan No. 88L',66965,'In Earth orbit','','2000-10-15 00:00:00',675.51,19120,19129,64.84,'Medium' from dual union all
select 26567,'026567','2000-063D','Perekhodnik','Sredniy perekhodnik',66965,'Reentered','2000-10-14 00:00:00','2000-10-13 00:00:00',87.29,60,220,64.84,'ERROR - Unexpected value "TA"' from dual union all
select 26568,'026568','2000-063E','8S812','8S812',66965,'Reentered','2000-10-15 00:00:00','2000-10-14 00:00:00',87.43,141,153,64.84,'Intermediate' from dual union all
select 26569,'026569','2000-063F','Blok DM-2 No. 91L','11S861 No. 91L',66965,'In Earth orbit','','2000-10-15 00:00:00',674.74,19084,19126,64.84,'Medium' from dual union all
select 2657,'002657','1967-006A','ESSA 4','TOS B',15261,'In Earth orbit','','1967-02-26 00:00:00',113.42,1326,1441,102,'Sun-Synch' from dual union all
select 26570,'026570','2000-064A','Progress M-43','Progress 7K-TGM No. 243',66966,'Deorbited','2001-01-29 00:00:00','2000-10-17 00:00:00',88.63,186,228,51.64,'Intermediate' from dual union all
select 26571,'026571','2000-064B','11S510','11S510',66966,'Reentered','2000-10-18 00:00:00','2000-10-17 00:00:00',88.12,167,196,51.65,'Intermediate' from dual union all
select 26572,'026572','1998-067E','EVA debris','Depress valve cap',66503,'Reentered','2000-11-01 00:00:00','2000-10-19 00:00:00',91.84,364,366,51.58,'Intermediate' from dual union all
select 26573,'026573','2000-063G','SOZ','SOZ',66965,'Reentered','2007-12-30 00:00:00','2000-10-18 00:00:00',340.2,406,19124,64.79,'Highly Ellip' from dual union all
select 26574,'026574','2000-063H','SOZ','SOZ',66965,'Reentered','2007-07-12 00:00:00','2000-10-18 00:00:00',340.03,400,19118,64.86,'Highly Ellip' from dual union all
select 26575,'026575','2000-065A','USA 153','DSCS III B-11',66967,'In Earth orbit','','2001-01-01 00:00:00',1436.27,35780,35800,1,'Stationary' from dual union all
select 26576,'026576','2000-065B','Centaur AC-140','Centaur IIA AC-140',66967,'Reentered','2004-05-15 00:00:00','2000-10-20 00:00:00',620.68,218,35232,26.04,'GEO Transfer' from dual union all
select 26577,'026577','2000-065C','IABS-8','IABS-8',66967,'In Earth orbit','','2000-10-21 00:00:00',1434.22,35700,35800,1,'Drift GEO' from dual union all
select 26578,'026578','2000-066A','Thuraya 1','Thuraya 1',66968,'In Earth orbit','','2001-01-22 00:00:00',1436.04,35765,35806,6.13,'Inclined GEO' from dual union all
select 26579,'026579','2000-066B','Blok DM-SL No. 6L','Blok DM-SL No. 6L',66968,'Reentered','2004-05-28 00:00:00','2000-10-24 00:00:00',628.4,180,35669,6.35,'GEO Transfer' from dual union all
select 2658,'002658','1967-005B','8S692','8S692',15260,'Reentered','1967-01-25 00:00:00','1967-01-25 00:00:00',87.4,123,168,50.16,'Intermediate' from dual union all
select 26580,'026580','2000-067A','GE 6','GE 6',66969,'In Earth orbit','','2001-04-23 00:00:00',1436.09,35781,35792,.02,'Stationary' from dual union all
select 26581,'026581','2000-067B','8S812','8S812',66969,'Reentered','2000-10-23 00:00:00','2000-10-23 00:00:00',87.05,109,148,51.56,'Intermediate' from dual union all
select 26582,'026582','2000-067C','Perekhodnik','Sredniy perekhodnik',66969,'Reentered','2000-10-22 00:00:00','2000-10-22 00:00:00',87.62,148,166,51.58,'Intermediate' from dual union all
select 26583,'026583','2000-067D','Blok DM3 No. 19L','Blok DM3 No. 19L',66969,'In Earth orbit','','2000-10-23 00:00:00',742.8,5856,35726,18.67,'GEO Transfer' from dual union all
select 26584,'026584','1995-008N','Kosmos-2306 SS 7','ESO',65669,'Reentered','2000-10-25 00:00:00','2000-10-25 00:00:00',88.52,195,208,65.8,'Intermediate' from dual union all
select 26585,'026585','1995-008P','Kosmos-2306 SS 8','ESO',65669,'Reentered','2000-10-28 00:00:00','2000-10-25 00:00:00',89.08,216,242,65.84,'Intermediate' from dual union all
select 26586,'026586','1995-008Q','Kosmos-2306 SS 9','ESO',65669,'Reentered','2000-10-27 00:00:00','2000-10-27 00:00:00',88.12,178,185,65.84,'Intermediate' from dual union all
select 26587,'026587','1995-008R','Kosmos-2306 SS 10','ESO',65669,'Reentered','2000-10-27 00:00:00','2000-10-27 00:00:00',88.71,199,223,65.81,'Intermediate' from dual union all
select 26588,'026588','1995-008S','Kosmos-2306 SS 11','ESO',65669,'Reentered','2000-10-27 00:00:00','2000-10-27 00:00:00',88.22,182,191,65.8,'Intermediate' from dual union all
select 26589,'026589','1995-008T','Kosmos-2306 SS 12','ESO',65669,'Reentered','2000-10-27 00:00:00','2000-10-27 00:00:00',88.06,172,185,65.81,'Intermediate' from dual union all
select 2659,'002659','1967-005C','Kosmos-139 Adapter','OGCh Adapter',15260,'Reentered','1967-01-25 00:00:00','1967-01-25 00:00:00',87.19,111,159,49.92,'Intermediate' from dual union all
select 26590,'026590','2000-068A','Europe*Star 1','EuropeStar FM1',66970,'In Earth orbit','','2001-04-07 00:00:00',1436.06,35771,35801,.02,'Stationary' from dual union all
select 26591,'026591','2000-068B','Ariane H10-3 V134','Ariane H10-3 S/N T4101',66970,'In Earth orbit','','2000-10-29 00:00:00',631.2,175,35818,6.89,'GEO Transfer' from dual union all
select 26592,'026592','1995-008U','Kosmos-2306 SS 13','ESO',65669,'Reentered','2000-10-29 00:00:00','2000-10-29 00:00:00',87.57,102,207,65.77,'Intermediate' from dual union all
select 26593,'026593','1995-008V','Kosmos-2306 SS 14','ESO',65669,'Reentered','2000-10-29 00:00:00','2000-10-29 00:00:00',87.11,125,138,65.82,'Intermediate' from dual union all
select 26594,'026594','1995-008W','Kosmos-2306 SS 15','ESO',65669,'Reentered','2000-10-29 00:00:00','2000-10-29 00:00:00',87.41,133,160,65.8,'Intermediate' from dual union all
select 26595,'026595','1995-008X','Kosmos-2306 SS 16','ESO',65669,'Reentered','2000-10-29 00:00:00','2000-10-29 00:00:00',87.66,157,161,65.8,'Intermediate' from dual union all
select 26596,'026596','1995-008Y','Kosmos-2306 SS 17','ESO',65669,'Reentered','2000-10-29 00:00:00','2000-10-29 00:00:00',87.46,122,176,65.84,'Intermediate' from dual union all
select 26597,'026597','1995-008Z','Kosmos-2306 SS 18','ESO',65669,'Reentered','2000-10-29 00:00:00','2000-10-29 00:00:00',87.88,148,192,65.82,'Intermediate' from dual union all
select 26598,'026598','1995-008AA','Kosmos-2306 SS 19','ESO',65669,'Reentered','2000-10-29 00:00:00','2000-10-29 00:00:00',87.25,124,153,65.82,'Intermediate' from dual union all
select 26599,'026599','2000-069A','Beidou Daohang Shiyan Wei.','yi ke beidou daohang s.w',66971,'In Earth orbit','','2001-03-17 00:00:00',1436.04,35760,35811,.13,'Stationary' from dual union all
select 266,'000266','62 Theta 1','Kosmos-1','DS-2 No. 1',8204,'Reentered','1962-05-25 00:00:00','1962-04-16 00:00:00',93.98,208,731,49.02,'Intermediate' from dual union all
select 2660,'002660','1967-003J','Transtage 13','Transtage 13',15258,'In Earth orbit','','1995-06-08 00:00:00',1329.75,33562,33795,11.39,'Medium' from dual union all
select 26600,'026600','2000-069B','CZ-3A Stage 3','CZ-3A Stage 3',66971,'Reentered','2001-02-19 00:00:00','2000-11-08 00:00:00',750.62,185,41778,25.28,'Highly Ellip' from dual union all
select 26601,'026601','1987-020DT','deb Kosmos-1823','',60110,'In Earth orbit','','2000-12-01 00:00:00',116,1478,1523,73.61,'Intermediate' from dual union all
select 26602,'026602','1996-017C','deb IRS-P3','',65905,'Reentered','2004-02-06 00:00:00','2000-10-30 00:00:00',101.26,817,819,98.6,'Sun-Synch' from dual union all
select 26603,'026603','2000-070A','Soyuz TM-31','Soyuz 7K-STM No. 205',66972,'Landed','2001-05-06 00:00:00','2000-11-11 00:00:00',92.18,378,385,51.57,'Intermediate' from dual union all
select 26604,'026604','2000-070B','Blok-I','11S510',66972,'Reentered','2000-11-01 00:00:00','2000-10-31 00:00:00',88.48,181,217,51.67,'Intermediate' from dual union all
select 26605,'026605','2000-071A','GPS IIR-6','Navstar GPS SVN 41',66973,'In Earth orbit','','2000-11-13 00:00:00',724.28,20177,20498,55.06,'Medium' from dual union all
select 26606,'026606','2000-071B','Delta 281','Delta SSPS AJ-10-118K',66973,'Reentered','2001-02-08 00:00:00','2000-11-13 00:00:00',99.72,195,1295,37.57,'Intermediate' from dual union all
select 26607,'026607','2000-071C','Star 48B','Star 48B',66973,'Reentered','2005-01-13 00:00:00','2000-11-13 00:00:00',353.8,180,20205,39.07,'Highly Ellip' from dual union all
select 26608,'026608','2000-072A','PAS 1R','PAS 1R',66974,'In Earth orbit','','2001-04-28 00:00:00',1436.09,35786,35787,.04,'Stationary' from dual union all
select 26609,'026609','2000-072B','AMSAT-OSCAR-40','AMSAT Phase III-D',66974,'In Earth orbit','','2000-11-20 00:00:00',708.37,613,39277,6.42,'GEO Transfer' from dual union all
select 2661,'002661','1967-006B','Altair 2 X-258C4 RH-91','X-258C4 RH-91',15261,'In Earth orbit','','1967-02-28 00:00:00',113.61,1341,1443,101.98,'Sun-Synch' from dual union all
select 26610,'026610','2000-072C','STRV 1c','STRV 1c',66974,'In Earth orbit','','2000-11-21 00:00:00',707.85,609,39255,6.43,'GEO Transfer' from dual union all
select 26611,'026611','2000-072D','STRV 1d','STRV 1d',66974,'In Earth orbit','','2000-11-20 00:00:00',708.37,613,39277,6.42,'GEO Transfer' from dual union all
select 26612,'026612','2000-072E','EPS 507','EPS S/N 507',66974,'In Earth orbit','','2000-11-20 00:00:00',709.26,617,39317,6.43,'GEO Transfer' from dual union all
select 26613,'026613','2000-072F','ACU 1663SP','ACU adapter',66974,'In Earth orbit','','2000-11-18 00:00:00',710.07,617,39357,6.43,'GEO Transfer' from dual union all
select 26614,'026614','1978-026HU','deb Delta 139','',45221,'In Earth orbit','','2000-12-06 00:00:00',102.85,786,1001,98.6,'Sun-Synch' from dual union all
select 26615,'026615','2000-073A','Progress M1-4','Progress M1 No. 253',66975,'Deorbited','2001-02-08 00:00:00','2000-11-20 00:00:00',92.14,376,382,51.58,'Intermediate' from dual union all
select 26616,'026616','2000-073B','11S510','11S510',66975,'Reentered','2000-11-18 00:00:00','2000-11-17 00:00:00',87.47,145,154,51.64,'Intermediate' from dual union all
select 26617,'026617','2000-074A','QuickBird 1','QuickBird 1',66976,'Reentered Attached','2000-11-21 00:00:00','2000-11-20 00:00:00',91.48,81,613,65.78,'Intermediate' from dual union all
select 26618,'026618','1975-017C','deb JUMPSEAT','',36464,'Reentered','2000-11-23 00:00:00','2000-11-21 00:00:00',98.62,119,1266,63.16,'Intermediate' from dual union all
select 26619,'026619','2000-075A','EO-1','Earth Orbiter 1',66977,'In Earth orbit','','2000-11-21 00:00:00',98.88,682,729,98.28,'Sun-Synch' from dual union all
select 2662,'002662','1967-002B','Fairing','Fairing',15257,'Reentered','1967-01-28 00:00:00','1967-01-27 00:00:00',88.25,141,234,80.06,'Intermediate' from dual union all
select 26620,'026620','2000-075B','SAC-C','SAC C',66977,'In Earth orbit','','2000-11-22 00:00:00',98.71,687,707,98.21,'Sun-Synch' from dual union all
select 26621,'026621','2000-075C','Munin','Munin',66977,'In Earth orbit','','2000-11-22 00:00:00',110.47,697,1800,95.44,'Retrograde' from dual union all
select 26622,'026622','2000-075D','Delta 282','Delta SSPS AJ10-118K',66977,'In Earth orbit','','2000-11-22 00:00:00',110.49,693,1806,95.45,'Retrograde' from dual union all
select 26623,'026623','2000-075E','Delta DPAF','DPAF',66977,'In Earth orbit','','2000-11-23 00:00:00',98.75,691,707,98.21,'Sun-Synch' from dual union all
select 26624,'026624','2000-076A','Anik F1','Anik F1',66978,'In Earth orbit','','2001-03-18 00:00:00',1436.09,35784,35788,.02,'Stationary' from dual union all
select 26625,'026625','2000-076B','Ariane H10-3 V136','Ariane H10-3 S/N T4100',66978,'Reentered','2002-01-25 00:00:00','2000-11-24 00:00:00',678.24,253,38133,5.94,'GEO Transfer' from dual union all
select 26626,'026626','2000-077A','Sirius FM-3','Sirius CD Radio 3',66979,'In Earth orbit','','2001-06-26 00:00:00',1436.09,24455,47118,63.34,'Synchronous' from dual union all
select 26627,'026627','2000-077B','8S812','8S812',66979,'Reentered','2000-12-01 00:00:00','2000-12-01 00:00:00',87.28,125,154,64.84,'Intermediate' from dual union all
select 26628,'026628','2000-077C','Perekhodnik','Sredniy perekhodnik',66979,'Reentered','2000-12-01 00:00:00','2000-12-01 00:00:00',86.87,112,127,64.86,'Intermediate' from dual union all
select 26629,'026629','2000-077D','Blok DM3 No. 17L','Blok DM3 No. 17L',66979,'In Earth orbit','','2000-11-30 00:00:00',994.99,6181,47092,63.41,'Highly Ellip' from dual union all
select 2663,'002663','1964-011D','deb Agena D','',10297,'Reentered','1967-03-01 00:00:00','1966-12-31 00:00:00',93.99,464,476,82.07,'Intermediate' from dual union all
select 26630,'026630','2000-078A','Endeavour (STS-97)','OV-105',66980,'Landed','2000-12-11 00:00:00','2000-12-01 00:00:00',89.79,207,322,51.58,'Intermediate' from dual union all
select 26631,'026631','2000-079A','EROS A1','EROS A1',66981,'Reentered','2016-07-07 00:00:00','2000-12-05 00:00:00',94.55,491,503,97.32,'Sun-Synch' from dual union all
select 26632,'026632','2000-079B','Start','Start',66981,'Reentered','2004-01-17 00:00:00','2000-12-13 00:00:00',94.55,491,503,97.32,'Sun-Synch' from dual union all
select 26633,'026633','1983-070G','deb Kosmos-1481','',55827,'In Earth orbit','','2000-11-20 00:00:00',716.91,1379,38933,66.6,'GEO Transfer' from dual union all
select 26634,'026634','1984-069R','deb US-A coolant','',57123,'In Earth orbit','','2001-01-03 00:00:00',103.66,907,957,65.05,'Intermediate' from dual union all
select 26635,'026635','2000-080A','USA 155','QUASAR 12',66982,'In Earth orbit','','2001-03-01 00:00:00',1436.27,35780,35800,0,'Stationary' from dual union all
select 26636,'026636','2000-080B','Centaur AC-157','Centaur IIA AC-157',66982,'In Earth orbit','','2000-12-06 00:00:00',663.55,267,37378,26.5,'GEO Transfer' from dual union all
select 26637,'026637','1996-029M','deb USA 119','',65917,'Reentered','2007-10-24 00:00:00','2000-12-31 00:00:00',107.39,1091,1121,63.4,'Intermediate' from dual union all
select 26638,'026638','2000-081A','Astra 2D','Astra 2D',66983,'In Earth orbit','','2000-12-28 00:00:00',1441.4,35886,35895,.29,'Drift GEO' from dual union all
select 26639,'026639','2000-081B','GE 8','GE 8',66983,'In Earth orbit','','2000-12-30 00:00:00',1436.24,35784,35795,.05,'Stationary' from dual union all
select 2664,'002664','1967-007A','OPS 4399','GAMBIT SV 986',15262,'Deorbited','1967-02-12 00:00:00','1967-02-04 00:00:00',89.54,137,367,102.94,'Retrograde' from dual union all
select 26640,'026640','2000-081C','LDREX','LDREX',66983,'Reentered','2010-03-21 00:00:00','2001-01-04 00:00:00',627.3,215,35577,1.91,'GEO Transfer' from dual union all
select 26641,'026641','2000-081D','EPS 508','EPS S/N 508',66983,'Reentered','2002-05-31 00:00:00','2000-12-27 00:00:00',635.63,218,36003,1.97,'GEO Transfer' from dual union all
select 26642,'026642','2000-081E','Sylda 5','Sylda 5',66983,'Reentered','2002-06-29 00:00:00','2001-01-01 00:00:00',618.98,210,35152,1.93,'GEO Transfer' from dual union all
select 26643,'026643','2000-082A','Beidou Daohang Shiyan Wei. 2','er ke beidou daohang s.w',66984,'In Earth orbit','','2000-12-30 00:00:00',1436.06,35776,35796,.11,'Stationary' from dual union all
select 26644,'026644','2000-082B','CZ-3A Stage 3','CZ-3A Stage 3',66984,'Reentered','2007-10-17 00:00:00','2000-12-27 00:00:00',749.45,183,41723,25.35,'GEO Transfer' from dual union all
select 26645,'026645','1984-069S','deb US-A coolant','',57123,'In Earth orbit','','2001-01-03 00:00:00',103.67,893,972,65.04,'Intermediate' from dual union all
select 26646,'026646','1984-069T','deb US-A coolant','',57123,'In Earth orbit','','2001-01-02 00:00:00',103.71,894,973,65.05,'Intermediate' from dual union all
select 26647,'026647','1984-069U','deb US-A coolant','',57123,'In Earth orbit','','2001-01-03 00:00:00',103.72,907,962,65.05,'Intermediate' from dual union all
select 26648,'026648','1984-069V','deb US-A coolant','',57123,'In Earth orbit','','2001-01-03 00:00:00',103.69,893,973,65.04,'Intermediate' from dual;
insert into SATELLITE
select 26649,'026649','1999-057LF','CZ-4 debris','',66738,'Reentered','2001-02-04 00:00:00','2000-12-31 00:00:00',95.73,539,569,98.36,'Sun-Synch' from dual union all
select 2665,'002665','1967-007B','Agena D 4834','Agena D',15262,'Reentered','1967-02-04 00:00:00','1967-02-03 00:00:00',88.6,126,285,102.71,'Retrograde' from dual union all
select 26650,'026650','1999-057LG','CZ-4 debris','',66738,'In Earth orbit','','2001-01-04 00:00:00',99.04,700,726,98.51,'Sun-Synch' from dual union all
select 26651,'026651','1999-057LH','CZ-4 debris','',66738,'Reentered','2003-03-21 00:00:00','2001-01-03 00:00:00',100.13,691,839,98.41,'Sun-Synch' from dual union all
select 26652,'026652','1999-057LJ','CZ-4 debris','',66738,'Reentered','2003-11-03 00:00:00','2001-01-04 00:00:00',100.11,692,836,98.4,'Sun-Synch' from dual union all
select 26653,'026653','1999-057LK','CZ-4 debris','',66738,'Reentered','2001-04-28 00:00:00','2001-01-04 00:00:00',96.25,548,610,98.34,'Sun-Synch' from dual union all
select 26654,'026654','1999-057LL','CZ-4 debris','',66738,'In Earth orbit','','2000-12-31 00:00:00',102.06,720,993,98.46,'Sun-Synch' from dual union all
select 26655,'026655','1999-057LM','CZ-4 debris','',66738,'Reentered','2001-01-06 00:00:00','2001-01-04 00:00:00',91.39,331,355,98.63,'Sun-Synch' from dual union all
select 26656,'026656','1999-057LN','CZ-4 debris','',66738,'Reentered','2002-12-17 00:00:00','2001-01-04 00:00:00',99.83,691,810,98.42,'Sun-Synch' from dual union all
select 26657,'026657','1999-057LP','CZ-4 debris','',66738,'Reentered','2000-10-06 00:00:00','2000-09-30 00:00:00',92.68,398,413,98.66,'Sun-Synch' from dual union all
select 26658,'026658','1999-057LQ','CZ-4 debris','',66738,'In Earth orbit','','2001-01-03 00:00:00',100.62,709,867,98.41,'Sun-Synch' from dual union all
select 26659,'026659','1999-057LR','CZ-4 debris','',66738,'Reentered','2002-11-11 00:00:00','2001-01-04 00:00:00',98.6,677,707,98.55,'Sun-Synch' from dual union all
select 2666,'002666','1967-008A','Lunar Orbiter 3','Lunar Orbiter C',15263,'Deep Space','1967-02-08 00:00:00','1967-02-05 00:00:00',16052.78,171,408662,29.8,'Cislunar' from dual union all
select 26660,'026660','1999-057LS','CZ-4 debris','',66738,'Reentered','2002-01-29 00:00:00','2001-01-04 00:00:00',99.54,680,793,98.46,'Sun-Synch' from dual union all
select 26661,'026661','1999-057LT','CZ-4 debris','',66738,'In Earth orbit','','2001-01-03 00:00:00',100.64,709,869,98.4,'Sun-Synch' from dual union all
select 26662,'026662','1999-057LU','CZ-4 debris','',66738,'Reentered','2000-11-27 00:00:00','2000-11-23 00:00:00',93.5,430,462,98.46,'Sun-Synch' from dual union all
select 26663,'026663','1999-057LV','CZ-4 debris','',66738,'Reentered','2001-02-07 00:00:00','2001-01-04 00:00:00',95.75,548,562,98.44,'Sun-Synch' from dual union all
select 26664,'026664','2001-001A','Shenzhou 2','Shenzhou 2 Fanhui Cang',67154,'Landed','2001-01-16 00:00:00','2001-01-12 00:00:00',91.23,330,340,42.57,'Intermediate' from dual union all
select 26665,'026665','2001-001B','CZ-2F Stage 2','CZ-2F Stage 2',67154,'Reentered','2001-01-20 00:00:00','2001-01-12 00:00:00',89.48,191,307,42.58,'Intermediate' from dual union all
select 26666,'026666','2001-002A','Turksat 2A','Eurasiasat 1',67155,'In Earth orbit','','2001-09-10 00:00:00',1436.01,35638,35931,.02,'Stationary' from dual union all
select 26667,'026667','2001-002B','Ariane H10-3 V137','Ariane H10-3 S/N T4102',67155,'Reentered','2001-01-11 00:00:00','2001-01-11 00:00:00',648.97,162,36742,6.51,'GEO Transfer' from dual union all
select 26668,'026668','1964-049P','deb Kosmos-41','',10335,'Reentered','2002-06-10 00:00:00','2000-12-17 00:00:00',711.53,807,39239,66.47,'GEO Transfer' from dual union all
select 26669,'026669','1966-040J','Nimbus 2 debris','',13447,'In Earth orbit','','2001-01-12 00:00:00',107.66,1087,1150,100.36,'Sun-Synch' from dual union all
select 2667,'002667','1967-009A','Kosmos-140','Soyuz No. 3',15264,'Landed','1967-02-09 00:00:00','1967-02-08 00:00:00',88.32,166,217,51.65,'Intermediate' from dual union all
select 26670,'026670','1966-040K','Nimbus 2 debris','',13447,'In Earth orbit','','2001-01-09 00:00:00',107.79,1098,1151,100.33,'Sun-Synch' from dual union all
select 26671,'026671','1966-077Z','deb FTV 1352','',13484,'In Earth orbit','','2001-01-12 00:00:00',166.86,2712,4612,89.56,'Medium' from dual union all
select 26672,'026672','1968-055F','deb Star 37D','',16782,'In Earth orbit','','2001-01-12 00:00:00',134.61,661,3984,120.61,'Medium' from dual union all
select 26673,'026673','1968-055G','deb Star 37D','',16782,'Reentered','2002-03-05 00:00:00','2001-01-11 00:00:00',112.2,562,2094,120.54,'Retrograde' from dual union all
select 26674,'026674','1979-058L','deb Kosmos-1109','',47542,'Reentered','2011-11-15 00:00:00','2001-01-12 00:00:00',727.02,3695,37115,67.68,'GEO Transfer' from dual union all
select 26675,'026675','1980-050E','deb Kosmos-1188','',50260,'In Earth orbit','','2001-01-12 00:00:00',740.75,3268,38214,68.99,'GEO Transfer' from dual union all
select 26676,'026676','1980-050F','deb Kosmos-1188','',50260,'In Earth orbit','','2001-01-06 00:00:00',743.7,3073,38553,68.56,'GEO Transfer' from dual union all
select 26677,'026677','1980-050G','deb Kosmos-1188','',50260,'In Earth orbit','','2001-01-07 00:00:00',745.65,3409,38311,69.24,'GEO Transfer' from dual union all
select 26678,'026678','1980-085E','deb Kosmos-1217','',50295,'In Earth orbit','','2001-01-07 00:00:00',679.87,1052,37416,65.04,'GEO Transfer' from dual union all
select 26679,'026679','1980-085F','deb Kosmos-1217','',50295,'In Earth orbit','','2000-12-28 00:00:00',691.77,1424,37640,65.46,'GEO Transfer' from dual union all
select 2668,'002668','1967-009B','11S510','11S510',15264,'Reentered','1967-02-08 00:00:00','1967-02-07 00:00:00',88.16,158,209,51.7,'Intermediate' from dual union all
select 26680,'026680','1981-071L','deb Kosmos-1285','',52594,'In Earth orbit','','2001-01-11 00:00:00',726.45,2813,37969,67.25,'GEO Transfer' from dual union all
select 26681,'026681','1984-023G','deb Ariane L8','',57077,'In Earth orbit','','2000-11-28 00:00:00',559.65,663,31573,8.61,'Highly Ellip' from dual union all
select 26682,'026682','1985-037J','deb SOZ','',58260,'Reentered','2004-03-15 00:00:00','2001-01-11 00:00:00',261.81,708,13648,51.99,'Medium' from dual union all
select 26683,'026683','1985-056H','deb Ariane V14','',58279,'In Earth orbit','','2001-01-03 00:00:00',646.09,799,35958,6.16,'GEO Transfer' from dual union all
select 26684,'026684','1985-056J','deb Ariane V14','',58279,'In Earth orbit','','2000-12-02 00:00:00',614.72,584,34557,5.54,'GEO Transfer' from dual union all
select 26685,'026685','1985-056K','deb Ariane V14','',58279,'In Earth orbit','','2000-10-21 00:00:00',664.32,1107,36577,7.07,'GEO Transfer' from dual union all
select 26686,'026686','1985-056L','deb Ariane V14','',58279,'Reentered','2012-12-16 00:00:00','2001-01-16 00:00:00',392.79,348,22429,7.66,'Highly Ellip' from dual union all
select 26687,'026687','2001-001C','Shenzhou 2 Guidao Cang','Shenzhou 2 orbital modul',67154,'Reentered','2001-08-24 00:00:00','2001-01-16 00:00:00',91.27,329,345,42.59,'Intermediate' from dual union all
select 26688,'026688','2001-003A','Progress M1-5','Progress M1 No. 254',67156,'Reentered Attached','2001-03-23 00:00:00','2001-01-26 00:00:00',89.89,256,282,51.65,'Intermediate' from dual union all
select 26689,'026689','2001-003B','11S510','11S510',67156,'Reentered','2001-01-25 00:00:00','2001-01-25 00:00:00',87.3,138,144,51.62,'Intermediate' from dual union all
select 2669,'002669','1967-010A','OPS 6073','DAPP 2418',15265,'In Earth orbit','','1967-03-12 00:00:00',101.48,790,868,98.87,'Sun-Synch' from dual union all
select 26690,'026690','2001-004A','GPS IIR-7','Navstar GPS SVN 54',67157,'In Earth orbit','','2001-02-02 00:00:00',718.08,20104,20266,55,'Medium' from dual union all
select 26691,'026691','2001-004B','Delta 283','Delta SSPS AJ-10-118K',67157,'Reentered','2001-03-17 00:00:00','2001-02-02 00:00:00',99.25,169,1276,32.6,'Intermediate' from dual union all
select 26692,'026692','2001-004C','Star 48B','Star 48B',67157,'Reentered','2004-08-26 00:00:00','2001-02-02 00:00:00',355.84,178,20334,38.98,'Highly Ellip' from dual union all
select 26693,'026693','1999-057LW','CZ-4 debris','',66738,'Reentered','2002-11-12 00:00:00','2001-01-30 00:00:00',99.67,688,798,98.41,'Sun-Synch' from dual union all
select 26694,'026694','2001-005A','SICRAL 1','SICRAL 1A',67158,'In Earth orbit','','2001-02-20 00:00:00',1426.85,35445,35766,.07,'Drift GEO' from dual union all
select 26695,'026695','2001-005B','Skynet 4F','Skynet 4F',67158,'In Earth orbit','','2001-12-03 00:00:00',1436.01,35775,35795,3.33,'Inclined GEO' from dual union all
select 26696,'026696','2001-005C','Ariane H10-3 V139','Ariane H10-3 S/N T4103',67158,'In Earth orbit','','2001-02-14 00:00:00',647.52,282,36548,7.17,'GEO Transfer' from dual union all
select 26697,'026697','2001-005D','Spelda','Spelda 40',67158,'Reentered','2005-04-28 00:00:00','2001-02-14 00:00:00',629.6,232,35679,6.99,'GEO Transfer' from dual union all
select 26698,'026698','2001-006A','Atlantis (STS-98)','OV-104',67159,'Landed','2001-02-20 00:00:00','2001-02-09 00:00:00',89.7,183,337,51.3,'Intermediate' from dual union all
select 26699,'026699','1999-057LX','CZ-4 debris','',66738,'Reentered','2002-11-23 00:00:00','2001-02-07 00:00:00',99.86,688,816,98.44,'Sun-Synch' from dual union all
select 267,'000267','62 Theta 2','S1','S1',8204,'Reentered','1962-06-18 00:00:00','1962-04-15 00:00:00',94.55,209,785,49.04,'Intermediate' from dual union all
select 2670,'002670','1967-012A','Kosmos-141','Zenit-4 No. 29',15267,'Landed','1967-02-16 00:00:00','1967-02-13 00:00:00',89.68,203,315,72.87,'Intermediate' from dual union all
select 26700,'026700','2001-006B','Destiny','ISS US Lab Module',67159,'In Earth orbit','','2001-02-14 00:00:00',91.97,365,378,51.6,'Intermediate' from dual union all
select 26701,'026701','1975-052KB','deb Delta 111','',36499,'In Earth orbit','','2001-02-15 00:00:00',106.84,879,1282,99.69,'Sun-Synch' from dual union all
select 26702,'026702','2001-007A','Odin','Odin',67160,'In Earth orbit','','2001-03-02 00:00:00',97.02,611,621,97.83,'Sun-Synch' from dual union all
select 26703,'026703','2001-007B','Start','Start',67160,'In Earth orbit','','2001-03-02 00:00:00',96.97,610,618,97.82,'Sun-Synch' from dual union all
select 26704,'026704','2001-007C','deb Start','',67160,'Reentered','2001-03-08 00:00:00','2001-02-27 00:00:00',93.79,405,515,97.79,'Sun-Synch' from dual union all
select 26705,'026705','2001-007D','deb Start','',67160,'Reentered','2001-03-05 00:00:00','2001-02-24 00:00:00',95.12,470,579,97.79,'Sun-Synch' from dual union all
select 26706,'026706','2001-007E','deb Start','',67160,'Reentered','2001-03-11 00:00:00','2001-02-23 00:00:00',95.25,472,589,97.79,'Sun-Synch' from dual union all
select 26707,'026707','2001-007F','deb Start','',67160,'Reentered','2001-03-11 00:00:00','2001-02-27 00:00:00',95.2,487,569,97.8,'Sun-Synch' from dual union all
select 26708,'026708','2001-007G','deb Start','Start rocket part',67160,'Reentered','2002-12-31 00:00:00','2001-03-02 00:00:00',96.05,528,611,97.82,'Sun-Synch' from dual union all
select 26709,'026709','2001-007H','deb Start','',67160,'Reentered','2001-03-08 00:00:00','2001-02-27 00:00:00',94.22,411,551,97.79,'Sun-Synch' from dual union all
select 2671,'002671','1967-011B','P064','P064',15266,'In Earth orbit','','1967-03-12 00:00:00',104.28,569,1352,39.98,'Intermediate' from dual union all
select 26710,'026710','2000-019E','SOZ','SOZ',66921,'Reentered','2001-02-24 00:00:00','2000-04-30 00:00:00',630.55,180,35780,47.8,'GEO Transfer' from dual union all
select 26711,'026711','2000-019F','SOZ','SOZ',66921,'Reentered','2001-09-20 00:00:00','2001-02-23 00:00:00',621.57,175,35321,47.8,'GEO Transfer' from dual union all
select 26712,'026712','2001-007J','deb Start','',67160,'Reentered','2001-03-08 00:00:00','2001-02-27 00:00:00',93.92,417,516,97.8,'Sun-Synch' from dual union all
select 26713,'026713','2001-008A','Progress M-44','Progress 7K-TGM No. 244',67161,'Deorbited','2001-04-16 00:00:00','2001-03-02 00:00:00',92.12,373,383,51.57,'Intermediate' from dual union all
select 26714,'026714','2001-008B','11S510','11S510',67161,'Reentered','2001-02-28 00:00:00','2001-02-27 00:00:00',87.73,154,171,51.64,'Intermediate' from dual union all
select 26715,'026715','2001-009A','Milstar 4','Milstar DFS-4',67162,'In Earth orbit','','2001-02-28 00:00:00',1435.04,35764,35768,4.5,'Inclined Drift' from dual union all
select 26716,'026716','2001-009B','Centaur TC-22','Centaur TC-22',67162,'In Earth orbit','','2001-02-28 00:00:00',1435.04,35764,35768,4.5,'Inclined Drift' from dual union all
select 26717,'026717','1989-006C','deb Ariane V28','',62283,'In Earth orbit','','2001-03-20 00:00:00',637.07,583,35712,8.54,'GEO Transfer' from dual union all
select 26718,'026718','2001-010A','Discovery (STS-102)','OV-103',67163,'Landed','2001-03-21 00:00:00','2001-03-10 00:00:00',92.05,370,381,51.57,'Intermediate' from dual union all
select 26719,'026719','2001-011A','Eurobird 1','Eurobird',67164,'In Earth orbit','','2001-10-25 00:00:00',1436.04,35757,35813,.03,'Stationary' from dual union all
select 2672,'002672','1967-012B','11A57I','11A57I',15267,'Reentered','1967-02-16 00:00:00','1967-02-11 00:00:00',89.35,198,287,72.85,'Intermediate' from dual union all
select 26720,'026720','2001-011B','BSAT-2a','BSAT-2a',67164,'In Earth orbit','','2001-03-13 00:00:00',1445.83,35703,36250,.22,'Drift GEO' from dual union all
select 26721,'026721','2001-011C','EPS 509','EPS S/N 509',67164,'In Earth orbit','','2001-03-11 00:00:00',641.26,872,35638,1.96,'GEO Transfer' from dual union all
select 26722,'026722','2001-011D','Sylda 5','Sylda 5',67164,'In Earth orbit','','2001-03-11 00:00:00',646.35,880,35890,2.06,'GEO Transfer' from dual union all
select 26723,'026723','2001-010B','PAD','PFR Attachment Device',67163,'Reentered','2001-06-07 00:00:00','2001-03-15 00:00:00',92.01,371,375,51.57,'Intermediate' from dual union all
select 26724,'026724','2001-012A','XM-2','XM Radio-2',67165,'In Earth orbit','','2001-10-29 00:00:00',1436.06,35785,35787,.06,'Stationary' from dual union all
select 26725,'026725','2001-012B','Blok DM-SL No. 5L','Blok DM-SL',67165,'In Earth orbit','','2001-03-20 00:00:00',644.14,904,35753,1.27,'GEO Transfer' from dual union all
select 26726,'026726','1989-006D','deb Ariane V28','',62283,'In Earth orbit','','2001-03-20 00:00:00',636.02,532,35709,8.44,'GEO Transfer' from dual union all
select 26727,'026727','1989-006E','deb Ariane V28','',62283,'In Earth orbit','','2001-03-20 00:00:00',635.59,541,35678,8.64,'GEO Transfer' from dual union all
select 26728,'026728','1989-006F','deb Ariane V28','',62283,'In Earth orbit','','2001-03-18 00:00:00',630.53,437,35522,8.37,'GEO Transfer' from dual union all
select 26729,'026729','1989-006G','deb Ariane V28','',62283,'In Earth orbit','','2001-03-20 00:00:00',633.41,429,35677,8.55,'GEO Transfer' from dual union all
select 2673,'002673','1967-011C','deb Diademe 1 despin weight','',15266,'Reentered','1967-12-13 00:00:00','1967-03-13 00:00:00',103.89,575,1310,39.93,'Intermediate' from dual union all
select 26730,'026730','1999-051D','deb Ikonos','',66732,'Reentered','2001-04-11 00:00:00','2001-03-23 00:00:00',98.07,664,669,98.2,'Sun-Synch' from dual union all
select 26731,'026731','1965-027BE','deb SNAPSHOT','',11777,'In Earth orbit','','2001-03-30 00:00:00',111.43,1266,1318,90.16,'Polar' from dual union all
select 26732,'026732','1965-027BF','deb SNAPSHOT','',11777,'In Earth orbit','','2001-03-29 00:00:00',111.42,1268,1316,90.16,'Polar' from dual union all
select 26733,'026733','1965-027BG','deb SNAPSHOT','',11777,'In Earth orbit','','2001-03-30 00:00:00',111.43,1265,1320,90.16,'Polar' from dual union all
select 26734,'026734','2001-013A','Mars Odyssey','2001 Mars Odyssey',67166,'Deep Space','2001-04-08 00:00:00','2001-04-07 00:00:00',1186.99,184,87212,51.83,'Earth Escape' from dual union all
select 26735,'026735','2001-013B','Star 48B','Star 48B',67166,'Deep Space','2001-04-08 00:00:00','2001-04-07 00:00:00',1186.99,184,87212,51.83,'Earth Escape' from dual union all
select 26736,'026736','2001-014A','Ekran-M','Ekran-M No. 18L',67167,'In Earth orbit','','2001-04-17 00:00:00',1423.31,35393,35679,2.01,'Drift GEO' from dual union all
select 26737,'026737','2001-014B','Briz-M DTB','Briz-M torus tank',67167,'Reentered','2011-04-24 00:00:00','2001-04-16 00:00:00',641.65,912,35618,47.94,'GEO Transfer' from dual union all
select 26738,'026738','2001-014C','Briz-M No. 88503L','Briz-M 14S43 No. 88503',67167,'In Earth orbit','','2001-04-20 00:00:00',1469.86,35663,37226,1.9,'Drift GEO' from dual;
insert into SATELLITE
select 26739,'026739','2001-013C','Delta 284','Delta SSPS AJ-10-118K',67166,'Reentered','2001-07-09 00:00:00','2001-04-17 00:00:00',103.67,175,1690,40.03,'Intermediate' from dual union all
select 2674,'002674','1967-011A','Diademe D-1C','Diademe D-1C',15266,'In Earth orbit','','1967-03-11 00:00:00',104.26,569,1351,39.98,'Intermediate' from dual union all
select 26740,'026740','1994-029AEH','deb HAPS','',65425,'In Earth orbit','','2001-04-14 00:00:00',98.9,565,848,81.87,'Intermediate' from dual union all
select 26741,'026741','1992-072J','deb Ariane V54','',64784,'In Earth orbit','','2001-02-21 00:00:00',494.56,882,27794,8.84,'Highly Ellip' from dual union all
select 26742,'026742','1994-038G','deb SOZ','',65434,'Reentered','2008-08-31 00:00:00','2001-04-12 00:00:00',620.31,1136,34295,47.55,'GEO Transfer' from dual union all
select 26743,'026743','1979-028C','deb Kosmos-1091','',47512,'In Earth orbit','','2011-05-28 00:00:00',104.7,959,1002,82.92,'Intermediate' from dual union all
select 26744,'026744','1997-005E','KM-V1 part','',66085,'Reentered','2002-03-23 00:00:00','2001-04-17 00:00:00',163.54,235,6822,31.29,'Medium' from dual union all
select 26745,'026745','2001-015A','GSAT-1','GSAT-1',67168,'In Earth orbit','','2001-04-25 00:00:00',1384.09,33800,35724,.97,'Drift GEO' from dual union all
select 26746,'026746','2001-015B','GSLV CS-1','GSLV-D1 12KRB stage',67168,'Reentered','2003-01-18 00:00:00','2001-04-26 00:00:00',556.61,167,31906,19.3,'Highly Ellip' from dual union all
select 26747,'026747','2001-016A','Endeavour (STS-100)','OV-105',67169,'Landed','2001-05-01 00:00:00','2001-04-21 00:00:00',91.59,331,375,51.57,'Intermediate' from dual union all
select 26748,'026748','2000-056H','deb Kosmos-2372','',66958,'Reentered','2001-04-21 00:00:00','2001-04-21 00:00:00',88.52,132,271,64.62,'Intermediate' from dual union all
select 26749,'026749','2001-017A','Soyuz TM-32','Soyuz 7K-STM No. 206',67170,'Landed','2001-10-31 00:00:00','2001-04-30 00:00:00',92.38,380,403,51.57,'Intermediate' from dual union all
select 2675,'002675','1967-011D','deb Diademe 1 despin weight','',15266,'Reentered','1968-01-03 00:00:00','1967-11-15 00:00:00',98.31,496,860,39.88,'Intermediate' from dual union all
select 26750,'026750','2001-017B','Blok-I','11S510',67170,'Reentered','2001-04-29 00:00:00','2001-04-28 00:00:00',88.45,182,214,51.61,'Intermediate' from dual union all
select 26751,'026751','1999-057LY','CZ-4 debris','',66738,'Reentered','2002-06-10 00:00:00','2001-05-02 00:00:00',98.85,666,742,98.44,'Sun-Synch' from dual union all
select 26752,'026752','1999-057LZ','CZ-4 debris','',66738,'Reentered','2002-02-24 00:00:00','2001-05-03 00:00:00',98.72,647,748,98.48,'Sun-Synch' from dual union all
select 26753,'026753','1999-057MA','CZ-4 debris','',66738,'Reentered','2003-03-23 00:00:00','2001-05-04 00:00:00',97.92,620,698,98.49,'Sun-Synch' from dual union all
select 26754,'026754','1999-057MB','CZ-4 debris','',66738,'In Earth orbit','','2001-05-03 00:00:00',99.59,665,813,98.47,'Sun-Synch' from dual union all
select 26755,'026755','1999-057MC','CZ-4 debris','',66738,'In Earth orbit','','2001-05-03 00:00:00',100.17,688,846,98.47,'Sun-Synch' from dual union all
select 26756,'026756','1999-057MD','CZ-4 debris','',66738,'In Earth orbit','','2001-05-01 00:00:00',100.4,714,840,98.42,'Sun-Synch' from dual union all
select 26757,'026757','1999-057ME','CZ-4 debris','',66738,'Reentered','2002-03-16 00:00:00','2001-05-02 00:00:00',99.47,668,799,98.46,'Sun-Synch' from dual union all
select 26758,'026758','1999-057MF','CZ-4 debris','',66738,'Reentered','2014-11-08 00:00:00','2001-05-01 00:00:00',98.12,625,713,98.61,'Sun-Synch' from dual union all
select 26759,'026759','1999-057MG','CZ-4 debris','',66738,'Reentered','2013-04-26 00:00:00','2001-05-04 00:00:00',99.89,677,830,98.44,'Sun-Synch' from dual union all
select 2676,'002676','1967-011E','deb Diademe 1','',15266,'Reentered','1967-12-05 00:00:00','1967-03-13 00:00:00',103.66,573,1291,39.96,'Intermediate' from dual union all
select 26760,'026760','1977-068H','deb Kosmos-931','',42196,'In Earth orbit','','2001-05-03 00:00:00',718.35,2307,38076,65.64,'GEO Transfer' from dual union all
select 26761,'026761','2001-018A','XM-1','XM Radio-1',67171,'In Earth orbit','','2002-01-03 00:00:00',1436.06,35785,35787,.03,'Stationary' from dual union all
select 26762,'026762','2001-018B','Blok DM-SL No.  7L','Blok DM-SL',67171,'In Earth orbit','','2001-05-11 00:00:00',643.27,885,35728,1.28,'GEO Transfer' from dual union all
select 26763,'026763','1965-108AS','deb Transtage 8','',11858,'In Earth orbit','','2001-05-12 00:00:00',527.72,646,29862,26.21,'Highly Ellip' from dual union all
select 26764,'026764','1965-108AT','deb Transtage 8','',11858,'In Earth orbit','','2001-05-11 00:00:00',542.17,653,30641,26.47,'Highly Ellip' from dual union all
select 26765,'026765','1965-108AU','deb Transtage 8','',11858,'In Earth orbit','','2001-04-03 00:00:00',536.99,824,30189,27.25,'Highly Ellip' from dual union all
select 26766,'026766','2001-019A','PAS 10','PAS 10',67172,'In Earth orbit','','2002-04-30 00:00:00',1436.06,35769,35802,.01,'Stationary' from dual union all
select 26767,'026767','2001-019B','8S812','8S812',67172,'Reentered','2001-05-16 00:00:00','2001-05-16 00:00:00',87.43,129,166,51.6,'Intermediate' from dual union all
select 26768,'026768','2001-019C','Perekhodnik','Sredniy perekhodnik',67172,'Reentered','2001-05-15 00:00:00','2001-05-15 00:00:00',87.17,128,141,51.59,'Intermediate' from dual union all
select 26769,'026769','2001-019D','Blok DM3 No. 6L','Blok DM3 No. 6L',67172,'In Earth orbit','','2001-05-15 00:00:00',771.15,7014,35942,17.02,'Highly Ellip' from dual union all
select 2677,'002677','1967-011F','deb Diademe 1','',15266,'Reentered','1968-02-12 00:00:00','1967-03-16 00:00:00',103.62,585,1274,40.01,'Intermediate' from dual union all
select 26770,'026770','2001-020A','GeoLITE','GeoLITE',67173,'In Earth orbit','','2001-07-01 00:00:00',1436.27,35780,35800,1,'Stationary' from dual union all
select 26771,'026771','2001-020B','Delta 285','Delta SSPS AJ-10-118K',67173,'Reentered','2001-10-21 00:00:00','2001-05-18 00:00:00',110.27,180,2299,24.26,'Equatorial' from dual union all
select 26772,'026772','2001-020C','Star 48B','Star 48B',67173,'Reentered','2002-11-14 00:00:00','2001-05-18 00:00:00',631.14,188,35802,26.5,'GEO Transfer' from dual union all
select 26773,'026773','2001-021A','Progress M1-6','Progress M1 No. 255',67174,'Deorbited','2001-08-22 00:00:00','2001-05-24 00:00:00',92.27,374,398,51.57,'Intermediate' from dual union all
select 26774,'026774','2001-021B','11S510','11S510',67174,'Reentered','2001-05-22 00:00:00','2001-05-21 00:00:00',88.43,184,210,51.65,'Intermediate' from dual union all
select 26775,'026775','2001-022A','Kosmos-2377','Kobal''t No. 554',67175,'Landed','2001-10-10 00:00:00','2001-05-31 00:00:00',89.57,163,344,67.11,'Intermediate' from dual union all
select 26776,'026776','2001-022B','11S510','11S510',67175,'Reentered','2001-06-02 00:00:00','2001-05-31 00:00:00',88.97,156,292,67.11,'Intermediate' from dual union all
select 26777,'026777','1965-108AV','deb Transtage 8','',11858,'In Earth orbit','','2001-05-31 00:00:00',561.43,1057,31274,26.64,'Highly Ellip' from dual union all
select 26778,'026778','1965-108AW','deb Transtage 8','',11858,'In Earth orbit','','2001-05-30 00:00:00',546.07,739,30766,27.83,'Highly Ellip' from dual union all
select 26779,'026779','1965-108AX','deb Transtage 8','',11858,'In Earth orbit','','2001-05-05 00:00:00',554.18,854,31088,26.9,'Highly Ellip' from dual union all
select 2678,'002678','1967-013A','Kosmos-142','DS-U2-I No. 2',15268,'Reentered','1967-07-06 00:00:00','1967-03-16 00:00:00',98.68,206,1186,48.41,'Intermediate' from dual union all
select 26780,'026780','1965-108AY','deb Transtage 8','',11858,'In Earth orbit','','2001-05-30 00:00:00',556.61,1870,30203,26.43,'Highly Ellip' from dual union all
select 26781,'026781','1965-108AZ','deb Transtage 8','',11858,'In Earth orbit','','2001-05-31 00:00:00',457.6,497,26088,27.3,'Highly Ellip' from dual union all
select 26782,'026782','1965-108BA','deb Transtage 8','',11858,'In Earth orbit','','2001-05-30 00:00:00',260.34,279,13976,26.52,'Highly Ellip' from dual union all
select 26783,'026783','1965-108BB','deb Transtage 8','',11858,'In Earth orbit','','2001-05-29 00:00:00',537.19,1098,29926,27.97,'Highly Ellip' from dual union all
select 26784,'026784','1971-120J','deb Meteor','',24938,'Reentered','2002-01-30 00:00:00','2001-05-31 00:00:00',98.13,656,683,81.24,'Intermediate' from dual union all
select 26785,'026785','1984-035C','deb CZ-3 Stage 3','',57089,'In Earth orbit','','2001-05-13 00:00:00',577.19,442,32729,30.6,'Highly Ellip' from dual union all
select 26786,'026786','1981-016J','deb Kosmos-1247','',52539,'In Earth orbit','','2001-05-15 00:00:00',702.03,2552,37023,66.58,'GEO Transfer' from dual union all
select 26787,'026787','1985-056M','deb Ariane V14','',58279,'In Earth orbit','','2001-05-25 00:00:00',636.37,1001,35258,7.38,'GEO Transfer' from dual union all
select 26788,'026788','1986-010E','deb CZ-3 Stage 3','',59203,'In Earth orbit','','2000-12-21 00:00:00',597.44,476,33763,30.58,'Highly Ellip' from dual union all
select 26789,'026789','1999-055D','Star 48 yo weight','',66736,'Reentered','2017-09-16 00:00:00','2001-05-29 00:00:00',324.71,191,18351,38.77,'Highly Ellip' from dual union all
select 2679,'002679','1967-013B','S1M','S1M',15268,'Reentered','1967-06-15 00:00:00','1967-03-19 00:00:00',98.51,208,1167,48.4,'Intermediate' from dual union all
select 26790,'026790','1992-021Q','deb Ariane V50','',64733,'In Earth orbit','','2001-04-28 00:00:00',645.03,1099,35604,3.84,'GEO Transfer' from dual union all
select 26791,'026791','1992-019D','Star 48 yo weight','',64731,'In Earth orbit','','2001-05-31 00:00:00',258.68,209,13931,34.69,'Highly Ellip' from dual union all
select 26792,'026792','1992-009D','Star 48 yo weight','',64721,'In Earth orbit','','2001-04-10 00:00:00',261.78,240,14115,34.51,'Highly Ellip' from dual union all
select 26793,'026793','1991-047F','Star 48 yo weight','',64216,'Reentered','2005-09-17 00:00:00','2001-05-30 00:00:00',205.02,188,10090,34.74,'Medium' from dual union all
select 26794,'026794','1991-015L','deb Ariane V42','',64184,'In Earth orbit','','2001-05-13 00:00:00',507.22,634,28746,6.83,'Highly Ellip' from dual union all
select 26795,'026795','1990-102G','deb SOZ','',63344,'In Earth orbit','','2001-05-23 00:00:00',172.59,1736,6045,46.13,'Medium' from dual union all
select 26796,'026796','1989-101R','deb SOZ','',62378,'Reentered','2005-02-16 00:00:00','2001-05-30 00:00:00',436.46,222,25142,46.63,'Highly Ellip' from dual union all
select 26797,'026797','1989-101S','deb SOZ','',62378,'Reentered','2001-12-23 00:00:00','2001-06-01 00:00:00',180,162,8202,46.69,'Medium' from dual union all
select 26798,'026798','1989-101T','deb SOZ','',62378,'Reentered','2005-02-10 00:00:00','2001-02-11 00:00:00',373.88,239,21388,46.67,'Highly Ellip' from dual union all
select 26799,'026799','1989-062J','deb Ariane V33','H10 deb',62339,'Reentered','2017-09-14 00:00:00','2001-06-01 00:00:00',539.57,434,30719,7.96,'Highly Ellip' from dual union all
select 268,'000268','61 Omicron 85','deb r/b Ablestar','',7273,'In Earth orbit','','1962-03-28 00:00:00',116.12,973,2040,67.82,'Intermediate' from dual union all
select 2680,'002680','1967-014A','Diademe D-1D','Diademe D-1D',15269,'In Earth orbit','','1967-03-18 00:00:00',110.2,590,1882,39.46,'Intermediate' from dual union all
select 26800,'026800','1989-027D','deb Ariane V30','',62304,'Reentered','2012-07-30 00:00:00','2001-05-31 00:00:00',429.69,248,24721,3.27,'Highly Ellip' from dual union all
select 26801,'026801','1985-118Y','deb SOZ','',58341,'Reentered','2001-03-09 00:00:00','2001-03-14 00:00:00',134.25,466,4148,65.49,'Medium' from dual union all
select 26802,'026802','1985-118Z','deb SOZ','',58341,'In Earth orbit','','2001-04-13 00:00:00',284.02,728,15142,64.05,'Highly Ellip' from dual union all
select 26803,'026803','1985-118AA','deb SOZ','',58341,'In Earth orbit','','2001-06-01 00:00:00',234.42,974,11457,64.9,'Medium' from dual union all
select 26804,'026804','1985-118AB','deb SOZ','',58341,'In Earth orbit','','2001-05-20 00:00:00',268.48,771,14044,65.71,'Medium' from dual union all
select 26805,'026805','1986-026H','deb Ariane V17','',59219,'Reentered','2013-08-17 00:00:00','2001-05-14 00:00:00',418.35,311,23991,8.96,'Highly Ellip' from dual union all
select 26806,'026806','1988-081G','deb Ariane V25','deb H10',61330,'In Earth orbit','','2001-05-15 00:00:00',663.23,1102,36527,6.18,'GEO Transfer' from dual union all
select 26807,'026807','1988-081H','deb Ariane V25','deb H10',61330,'In Earth orbit','','2001-04-06 00:00:00',662.88,1034,36577,6.12,'GEO Transfer' from dual union all
select 26808,'026808','1988-109L','deb Ariane V27','',61358,'In Earth orbit','','2000-10-31 00:00:00',672.44,663,37431,6.69,'GEO Transfer' from dual union all
select 26809,'026809','1988-063J','deb Ariane V24','',61312,'In Earth orbit','','2001-06-07 00:00:00',579.19,358,32919,6.88,'Highly Ellip' from dual union all
select 2681,'002681','1967-014D','Diademe 2 despin weight','',15269,'Reentered','1968-09-16 00:00:00','1967-03-20 00:00:00',108.85,598,1750,39.54,'Intermediate' from dual union all
select 26810,'026810','1988-109M','deb Ariane V27','',61358,'In Earth orbit','','2001-06-03 00:00:00',666.57,810,36988,7.1,'GEO Transfer' from dual union all
select 26811,'026811','1988-109N','deb Ariane V27','',61358,'Reentered','2002-09-07 00:00:00','2001-06-02 00:00:00',437.65,270,25163,6.85,'Highly Ellip' from dual union all
select 26812,'026812','1988-109P','deb Ariane V27','',61358,'In Earth orbit','','2000-12-09 00:00:00',650.26,990,35980,6.88,'GEO Transfer' from dual union all
select 26813,'026813','1989-001P','deb SOZ','',62278,'In Earth orbit','','2001-06-08 00:00:00',200.07,578,9328,64.48,'Medium' from dual union all
select 26814,'026814','1989-001Q','deb SOZ','',62278,'Reentered','2002-07-27 00:00:00','2001-06-08 00:00:00',229.13,596,11455,65.29,'Medium' from dual union all
select 26815,'026815','1965-048P','deb NNS O-4','',11798,'Reentered','2012-11-25 00:00:00','2001-06-12 00:00:00',105.45,953,1078,90.11,'Polar' from dual union all
select 26816,'026816','1966-040L','Nimbus 2 debris','',13447,'In Earth orbit','','2001-06-12 00:00:00',107.95,1090,1174,100.32,'Sun-Synch' from dual union all
select 26817,'026817','1966-076H','NNS O-10 insulation','',13483,'In Earth orbit','','2001-06-09 00:00:00',105.96,1008,1070,88.86,'Polar' from dual union all
select 26818,'026818','2001-023A','Kosmos-2378','Parus',67176,'In Earth orbit','','2001-06-08 00:00:00',104.83,963,1009,82.93,'Intermediate' from dual union all
select 26819,'026819','2001-023B','S3M','S3M',67176,'In Earth orbit','','2001-06-08 00:00:00',104.69,963,997,82.93,'Intermediate' from dual union all
select 2682,'002682','1967-014B','P064','P064',15269,'In Earth orbit','','1967-03-17 00:00:00',110.32,591,1892,39.46,'Intermediate' from dual union all
select 26820,'026820','1969-082LG','deb Agena D','',18349,'Reentered','2002-02-03 00:00:00','2001-06-12 00:00:00',97.83,632,678,70.06,'Intermediate' from dual union all
select 26821,'026821','1970-025QQ','deb Agena D','',21502,'In Earth orbit','','2001-06-12 00:00:00',102.93,819,976,99.99,'Sun-Synch' from dual union all
select 26822,'026822','1975-052KC','deb Delta 111','',36499,'In Earth orbit','','2001-06-05 00:00:00',112.26,1087,1573,99.7,'Sun-Synch' from dual union all
select 26823,'026823','1975-052KD','deb Delta 111','',36499,'In Earth orbit','','2001-06-12 00:00:00',107.45,1069,1149,99.88,'Sun-Synch' from dual union all
select 26824,'026824','2001-024A','INTELSAT 901','IS-901',67177,'In Earth orbit','','2002-03-01 00:00:00',1436.06,35773,35799,.02,'Stationary' from dual union all
select 26825,'026825','2001-024B','Ariane H10-3 V141','Ariane H10-3 S/N T4104',67177,'Reentered','2001-10-09 00:00:00','2001-06-09 00:00:00',611.03,32,34917,7.15,'ERROR - Unexpected value "TA"' from dual union all
select 26826,'026826','1975-052KE','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',111.08,1044,1508,99.58,'Sun-Synch' from dual union all
select 26827,'026827','1975-052KF','deb Delta 111','',36499,'In Earth orbit','','2001-06-08 00:00:00',108.77,1083,1257,99.66,'Sun-Synch' from dual union all
select 26828,'026828','1975-052KG','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',106.74,1008,1144,99.62,'Sun-Synch' from dual;
insert into SATELLITE
select 26829,'026829','1975-052KH','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',110.03,1031,1426,99.65,'Sun-Synch' from dual union all
select 2683,'002683','1967-014E','deb Diademe 2','',15269,'Reentered','1982-11-12 00:00:00','1968-02-29 00:00:00',109.68,585,1839,39.4,'Intermediate' from dual union all
select 26830,'026830','1975-052KJ','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',109.29,1084,1304,99.71,'Sun-Synch' from dual union all
select 26831,'026831','1975-052KK','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',107.51,1050,1173,99.71,'Sun-Synch' from dual union all
select 26832,'026832','1975-052KL','deb Delta 111','',36499,'In Earth orbit','','2001-06-08 00:00:00',118.03,1094,2091,99.63,'Sun-Synch' from dual union all
select 26833,'026833','1975-052KM','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',113.19,1086,1659,99.72,'Sun-Synch' from dual union all
select 26834,'026834','1975-052KN','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',108.04,1094,1179,100.03,'Sun-Synch' from dual union all
select 26835,'026835','1975-052KP','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',105.83,1002,1065,99.51,'Sun-Synch' from dual union all
select 26836,'026836','1975-052KQ','deb Delta 111','',36499,'In Earth orbit','','2001-06-04 00:00:00',108.64,1085,1243,99.83,'Sun-Synch' from dual union all
select 26837,'026837','1975-052KR','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',110.68,1042,1474,99.71,'Sun-Synch' from dual union all
select 26838,'026838','1975-052KS','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',111.73,1056,1557,99.24,'Sun-Synch' from dual union all
select 26839,'026839','1975-052KT','deb Delta 111','',36499,'In Earth orbit','','2001-06-08 00:00:00',108.39,1090,1215,100.07,'Sun-Synch' from dual union all
select 2684,'002684','1967-014C','Diademe 2 despin weight','',15269,'In Earth orbit','','1967-03-17 00:00:00',110.21,590,1883,39.46,'Intermediate' from dual union all
select 26840,'026840','1975-052KU','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',113.34,1089,1671,99.7,'Sun-Synch' from dual union all
select 26841,'026841','1975-052KV','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',112.93,937,1786,99.92,'Sun-Synch' from dual union all
select 26842,'026842','1975-052KW','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',105.84,1001,1067,99.61,'Sun-Synch' from dual union all
select 26843,'026843','1975-052KX','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',109.04,1092,1273,99.67,'Sun-Synch' from dual union all
select 26844,'026844','1975-052KY','deb Delta 111','',36499,'In Earth orbit','','2001-06-08 00:00:00',106.17,1015,1083,99.65,'Sun-Synch' from dual union all
select 26845,'026845','1975-052KZ','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',113.97,1066,1752,99.17,'Sun-Synch' from dual union all
select 26846,'026846','1975-052LA','deb Delta 111','',36499,'In Earth orbit','','2001-06-08 00:00:00',109.93,1093,1354,99.8,'Sun-Synch' from dual union all
select 26847,'026847','1975-052LB','deb Delta 111','',36499,'In Earth orbit','','2001-06-09 00:00:00',108.54,1091,1228,99.76,'Sun-Synch' from dual union all
select 26848,'026848','1975-124C','deb 8A92ME','',36571,'In Earth orbit','','2001-06-09 00:00:00',102.24,841,888,81.26,'Intermediate' from dual union all
select 26849,'026849','1985-118AC','deb SOZ','',58341,'In Earth orbit','','2001-06-06 00:00:00',258.5,861,13266,65.2,'Medium' from dual union all
select 2685,'002685','1967-014F','deb Diademe 2','',15269,'In Earth orbit','','1967-03-19 00:00:00',110.08,582,1879,38.93,'Intermediate' from dual union all
select 26850,'026850','1992-072K','deb Ariane V54','',64784,'In Earth orbit','','2001-06-08 00:00:00',496.91,956,27851,7.36,'Highly Ellip' from dual union all
select 26851,'026851','1992-058E','Star 48 yo weight','',64770,'In Earth orbit','','2001-06-08 00:00:00',253.58,230,13555,34.82,'Highly Ellip' from dual union all
select 26852,'026852','1992-079D','Star 48 yo weight','',64791,'Reentered','2011-06-05 00:00:00','2001-06-16 00:00:00',174.26,183,7730,34.67,'Medium' from dual union all
select 26853,'026853','2001-025A','Astra 2C','Astra 2C',67178,'In Earth orbit','','2002-02-01 00:00:00',1436.06,35768,35804,.08,'Stationary' from dual union all
select 26854,'026854','2001-025B','8S812','8S812',67178,'Reentered','2001-06-18 00:00:00','2001-06-16 00:00:00',88.19,158,212,51.59,'Intermediate' from dual union all
select 26855,'026855','2001-025C','Blok DM3 No. 27L','Blok DM3 No. 27L',67178,'In Earth orbit','','2001-06-16 00:00:00',779.26,7580,35766,16,'Highly Ellip' from dual union all
select 26856,'026856','2001-025D','Perekhodnik','Sredniy perekhodnik',67178,'Reentered','2001-06-16 00:00:00','2001-06-16 00:00:00',87.97,160,188,51.58,'Intermediate' from dual union all
select 26857,'026857','2001-026A','ICO F2','ICO 2',67179,'In Earth orbit','','2001-06-29 00:00:00',351.31,10103,10126,44.95,'Medium' from dual union all
select 26858,'026858','2001-026B','Centaur AC-156','Centaur AC-156',67179,'In Earth orbit','','2001-06-28 00:00:00',346.38,9810,10110,44.86,'Medium' from dual union all
select 26859,'026859','2001-027A','WMAP','Microwave Anisotropy Pro',67180,'Deep Space','2001-07-01 00:00:00','2001-07-10 00:00:00',12866.56,3144,347874,28.31,'Cislunar' from dual union all
select 2686,'002686','1967-015A','OPS 4750','CORONA J-39',15270,'Reentered Attached','1967-03-11 00:00:00','1967-03-02 00:00:00',89.77,174,353,80.03,'Intermediate' from dual union all
select 26860,'026860','2001-027B','Delta 286','Delta SSPS  AJ-10-118K',67180,'Reentered','2001-07-07 00:00:00','2001-07-02 00:00:00',89.47,196,301,26.91,'Intermediate' from dual union all
select 26861,'026861','1978-026HV','deb Delta 139','',45221,'In Earth orbit','','2011-06-09 00:00:00',100.88,727,874,98.49,'Sun-Synch' from dual union all
select 26862,'026862','2001-028A','Atlantis (STS-104)','OV-104',67181,'Landed','2001-07-25 00:00:00','2001-07-13 00:00:00',89.95,237,307,51.57,'Intermediate' from dual union all
select 26863,'026863','2001-029A','Artemis','Artemis',67182,'In Earth orbit','','2003-07-09 00:00:00',1436.01,35780,35790,1.93,'Stationary' from dual union all
select 26864,'026864','2001-029B','BSAT-2b','BSAT-2b',67182,'Reentered','2014-01-28 00:00:00','2001-07-14 00:00:00',317.25,589,17472,2.93,'Highly Ellip' from dual union all
select 26865,'026865','2001-029C','EPS 510','EPS S/N 510',67182,'Reentered','2012-11-22 00:00:00','2001-07-14 00:00:00',316.99,587,17457,2.92,'Highly Ellip' from dual union all
select 26866,'026866','2001-029D','Sylda 5','Sylda 5',67182,'Reentered','2010-10-06 00:00:00','2001-07-13 00:00:00',317.37,589,17480,2.92,'Highly Ellip' from dual union all
select 26867,'026867','2001-030A','Molniya-3','Molniya-3K No. 11',67183,'Reentered','2016-12-19 00:00:00','2001-07-22 00:00:00',735.76,407,40831,62.86,'Molniya' from dual union all
select 26868,'026868','2001-030B','11S510','11S510',67183,'Reentered','2001-08-12 00:00:00','2001-07-22 00:00:00',90.86,214,420,62.82,'Intermediate' from dual union all
select 26869,'026869','2001-030C','BOZ','BOZ',67183,'Reentered','2001-08-01 00:00:00','2001-07-22 00:00:00',90.45,183,410,62.82,'Intermediate' from dual union all
select 2687,'002687','1967-016A','OPS 4204','GAMBIT-3 Mission 4304',15271,'Reentered Attached','1967-03-06 00:00:00','1967-02-26 00:00:00',89.84,135,399,106.99,'Retrograde' from dual union all
select 26870,'026870','2001-030D','Blok-ML','Blok-ML No. 76734-497',67183,'Reentered','2018-03-17 00:00:00','2001-07-22 00:00:00',732.39,405,40668,62.87,'Molniya' from dual union all
select 26871,'026871','2001-031A','GOES 12','GOES M',67184,'In Earth orbit','','2002-04-27 00:00:00',1436.11,35767,35807,.18,'Stationary' from dual union all
select 26872,'026872','2001-031B','Centaur AC-142','Centaur AC-142',67184,'Reentered','2002-01-28 00:00:00','2001-07-25 00:00:00',751.12,250,41737,20.67,'Highly Ellip' from dual union all
select 26873,'026873','2001-032A','Koronas F','AUOS-SM-KF',67185,'Reentered','2005-12-06 00:00:00','2001-08-02 00:00:00',94.78,486,529,82.49,'Intermediate' from dual union all
select 26874,'026874','2001-032B','S5M','S5M',67185,'Reentered','2017-04-27 00:00:00','2001-08-02 00:00:00',94.77,486,528,82.49,'Intermediate' from dual union all
select 26875,'026875','2001-032C','Koronas F part','',67185,'Reentered','2001-12-03 00:00:00','2001-08-02 00:00:00',94.45,478,505,82.48,'Intermediate' from dual union all
select 26876,'026876','2001-032D','Koronas F part','',67185,'Reentered','2001-11-03 00:00:00','2001-08-02 00:00:00',94.54,484,509,82.47,'Intermediate' from dual union all
select 26877,'026877','2001-032E','Koronas F part','',67185,'Reentered','2001-12-05 00:00:00','2001-08-04 00:00:00',95.06,483,560,82.49,'Intermediate' from dual union all
select 26878,'026878','2001-032F','Koronas F part','',67185,'Reentered','2001-10-31 00:00:00','2001-08-03 00:00:00',94.44,478,505,82.48,'Intermediate' from dual union all
select 26879,'026879','2001-030E','deb BOZ','SOZ',67183,'Reentered','2001-08-05 00:00:00','2001-08-04 00:00:00',88.23,152,222,62.82,'Intermediate' from dual union all
select 2688,'002688','1967-011G','deb Diademe 1','',15266,'Reentered','1980-06-22 00:00:00','1967-04-05 00:00:00',104.21,576,1339,39.99,'Intermediate' from dual union all
select 26880,'026880','2001-033A','DSP F21','DSP',67186,'In Earth orbit','','2001-08-07 00:00:00',1435.75,35780,35780,2.9,'Stationary' from dual union all
select 26881,'026881','2001-033B','Titan 4B-31 Stage 2','Titan 4 Stage 2',67186,'Reentered','2001-10-13 00:00:00','2001-08-06 00:00:00',94.52,328,663,28.7,'Intermediate' from dual union all
select 26882,'026882','2001-033C','IUS-16 SRM-1','Orbus 21',67186,'In Earth orbit','','2001-08-06 00:00:00',633.66,330,35790,28.7,'GEO Transfer' from dual union all
select 26883,'026883','2001-033D','IUS-16 SRM-2','Orbus 6E',67186,'In Earth orbit','','2001-08-06 00:00:00',1443.17,35900,35950,3,'Inclined Drift' from dual union all
select 26884,'026884','2001-034A','Genesis','Genesis',67187,'Deep Space','2001-08-09 00:00:00','2001-08-10 00:00:00',80835.73,188,1225640,28.7,'Cislunar' from dual union all
select 26885,'026885','2001-034B','Delta 287','Delta SSPS AJ-10-118K',67187,'Reentered','2002-06-30 00:00:00','2001-08-10 00:00:00',126.98,189,3792,26.88,'Medium' from dual union all
select 26886,'026886','2001-034C','Star 37FM','Star 37FM',67187,'Deep Space','2001-08-09 00:00:00','2001-08-08 00:00:00',80835.05,182,1225640,28.7,'Cislunar' from dual union all
select 26887,'026887','2001-033E','DSP F21 Aperture Cover','',67186,'In Earth orbit','','2001-08-06 00:00:00',1435.5,35750,35800,3,'Stationary' from dual union all
select 26888,'026888','2001-035A','Discovery (STS-105)','OV-103',67188,'Landed','2001-08-22 00:00:00','2001-08-10 00:00:00',88.37,155,232,51.63,'Intermediate' from dual union all
select 26889,'026889','2001-035B','Simplesat','Simplesat',67188,'Reentered','2002-01-30 00:00:00','2001-08-20 00:00:00',92.43,385,403,51.64,'Intermediate' from dual union all
select 2689,'002689','1967-011H','deb Diademe 1','',15266,'Reentered','1990-06-20 00:00:00','1967-03-11 00:00:00',105.08,553,1444,39.98,'Intermediate' from dual union all
select 26890,'026890','2001-036A','Progress M-45','Progress M No. 245',67189,'Deorbited','2001-11-22 00:00:00','2001-08-22 00:00:00',90.64,297,315,51.64,'Intermediate' from dual union all
select 26891,'026891','2001-036B','11S510','11S510',67189,'Reentered','2001-08-23 00:00:00','2001-08-22 00:00:00',88.06,167,190,51.64,'Intermediate' from dual union all
select 26892,'026892','2001-037A','Kosmos-2379','Oko-1 No. 7124',67190,'In Earth orbit','','2002-08-02 00:00:00',1436.01,35767,35802,1.52,'Stationary' from dual union all
select 26893,'026893','2001-037B','8S812','8S812',67190,'Reentered','2001-08-27 00:00:00','2001-08-25 00:00:00',88.34,187,198,51.64,'Intermediate' from dual union all
select 26894,'026894','2001-037C','Perekhodnik','Sredniy perekhodnik',67190,'Reentered','2001-08-25 00:00:00','2001-08-25 00:00:00',88.28,186,193,51.65,'Intermediate' from dual union all
select 26895,'026895','2001-037D','Blok DM-2 No. 99L','11S861 No. 99L',67190,'In Earth orbit','','2001-08-29 00:00:00',1437.34,35729,35893,2.37,'Drift GEO' from dual union all
select 26896,'026896','2001-037E','SOZ','SOZ',67190,'Reentered','2002-06-07 00:00:00','2001-08-26 00:00:00',634.67,292,35880,47.98,'GEO Transfer' from dual union all
select 26897,'026897','2001-037F','SOZ','SOZ',67190,'Reentered','2004-12-29 00:00:00','2001-09-02 00:00:00',635.69,341,35883,48,'GEO Transfer' from dual union all
select 26898,'026898','2001-038A','LRE','LRE',67191,'In Earth orbit','','2001-09-03 00:00:00',639.25,255,36152,28.47,'GEO Transfer' from dual union all
select 26899,'026899','2001-038B','H-2A F1 Stage 2','H-2A Stage 2',67191,'In Earth orbit','','2001-08-31 00:00:00',640.83,279,36209,28.14,'GEO Transfer' from dual union all
select 269,'000269','62 Iota 1','Kosmos-2','1MS No. 1',8191,'Reentered','1963-08-20 00:00:00','1962-05-14 00:00:00',101.58,218,1449,48.95,'Intermediate' from dual union all
select 2690,'002690','1967-011J','deb Diademe 1','',15266,'Reentered','1968-05-08 00:00:00','1967-03-11 00:00:00',103.96,564,1328,39.97,'Intermediate' from dual union all
select 26900,'026900','2001-039A','INTELSAT 902','INTELSAT 902',67192,'In Earth orbit','','2002-10-04 00:00:00',1436.06,35776,35796,.01,'Stationary' from dual union all
select 26901,'026901','2001-039B','Ariane H10-3 V143','Ariane H10-3 S/N T4105',67192,'In Earth orbit','','2001-09-05 00:00:00',658.95,276,37136,7.14,'GEO Transfer' from dual union all
select 26902,'026902','2001-037G','Proton deb','',67190,'Reentered','2001-08-29 00:00:00','2001-08-28 00:00:00',90.48,172,425,51.58,'Intermediate' from dual union all
select 26903,'026903','2001-037H','Proton deb','',67190,'Reentered','2001-08-31 00:00:00','2001-08-29 00:00:00',90.11,172,388,51.65,'Intermediate' from dual union all
select 26904,'026904','2000-042C','Picosat 7','Tethered Picosat No. 23',66944,'Reentered','2002-07-11 00:00:00','2001-09-08 00:00:00',95.13,511,539,97.78,'Sun-Synch' from dual union all
select 26905,'026905','2001-040A','USA 160','INTRUDER 1',67193,'In Earth orbit','','2001-09-09 00:00:00',107.26,1100,1100,63,'Intermediate' from dual union all
select 26906,'026906','2001-040B','Centaur AC-160','Centaur AC-160',67193,'In Earth orbit','','2001-09-09 00:00:00',107.26,1100,1100,63,'Intermediate' from dual union all
select 26907,'026907','2001-040C','USA 160 P/L 2','INTRUDER 2',67193,'In Earth orbit','','2001-09-09 00:00:00',107.26,1100,1100,63,'Intermediate' from dual union all
select 26908,'026908','2001-041A','Progress M-SO1','Progress No. 301',67194,'Deorbited','2001-09-27 00:00:00','2001-09-16 00:00:00',89.54,239,264,51.64,'Intermediate' from dual union all
select 26909,'026909','2001-041B','11S510','11S510',67194,'Reentered','2001-09-16 00:00:00','2001-09-15 00:00:00',87.96,170,176,51.63,'Intermediate' from dual union all
select 2691,'002691','1967-011K','deb Diademe 1','',15266,'Reentered','1967-10-29 00:00:00','1967-03-11 00:00:00',103.69,570,1296,39.97,'Intermediate' from dual union all
select 26910,'026910','1965-082UR','deb Transtage 4','',11832,'Reentered','2002-03-21 00:00:00','2001-09-04 00:00:00',95.46,531,551,32.58,'Intermediate' from dual union all
select 26911,'026911','1981-021Q','deb US-A coolant','',52544,'In Earth orbit','','2001-09-12 00:00:00',103.64,888,973,64.96,'Intermediate' from dual union all
select 26912,'026912','1965-048Q','deb NNS O-4','',11798,'Reentered','2001-10-08 00:00:00','2001-09-15 00:00:00',95.91,541,585,90.08,'Polar' from dual union all
select 26913,'026913','1965-082US','deb Transtage 4','',11832,'Reentered','2012-01-01 00:00:00','2001-09-10 00:00:00',97.4,582,687,31.84,'Intermediate' from dual union all
select 26914,'026914','1985-056N','deb Ariane V14','',58279,'Reentered','2004-05-04 00:00:00','2001-09-17 00:00:00',344.13,393,19385,7.79,'Highly Ellip' from dual union all
select 26915,'026915','1978-083H','deb Kosmos-1030','',45278,'Reentered','2009-05-31 00:00:00','2001-09-19 00:00:00',718.27,1789,38589,64.66,'GEO Transfer' from dual union all
select 26916,'026916','1983-072D','SGS debris','',55829,'Reentered','2016-01-05 00:00:00','2001-09-19 00:00:00',368.54,1475,19824,64.33,'Highly Ellip' from dual union all
select 26917,'026917','1980-050H','deb Kosmos-1188','',50260,'In Earth orbit','','2001-09-21 00:00:00',760.58,3811,38635,70.02,'Highly Ellip' from dual union all
select 26918,'026918','1980-085G','deb Kosmos-1217','',50295,'In Earth orbit','','2001-09-09 00:00:00',700.96,1525,37997,65.49,'GEO Transfer' from dual;
insert into SATELLITE
select 26919,'026919','1980-085H','deb Kosmos-1217','',50295,'In Earth orbit','','2001-09-20 00:00:00',702.31,1651,37938,65.51,'GEO Transfer' from dual union all
select 2692,'002692','1967-011L','deb Diademe 1','',15266,'Reentered','1989-04-21 00:00:00','1967-03-11 00:00:00',103.35,561,1273,39.98,'Intermediate' from dual union all
select 26920,'026920','1983-075BF','deb Kosmos-1484 part','[Sep from 75C]',55832,'Reentered','2012-04-20 00:00:00','2001-09-18 00:00:00',96.01,517,618,97.83,'Sun-Synch' from dual union all
select 26921,'026921','1985-093C','SGS-II yo weight','',58316,'In Earth orbit','','2001-09-19 00:00:00',369.22,1450,19891,64.12,'Highly Ellip' from dual union all
select 26922,'026922','1984-023H','deb Ariane L8','',57077,'In Earth orbit','','2001-07-09 00:00:00',519.58,1241,28821,9.06,'Highly Ellip' from dual union all
select 26923,'026923','1987-080J','NNS O-27 insulation','',60170,'Reentered','2003-11-04 00:00:00','2001-09-04 00:00:00',104,882,1013,90.22,'Polar' from dual union all
select 26924,'026924','1984-023J','deb Ariane L8','',57077,'In Earth orbit','','2001-04-21 00:00:00',570.94,1074,31765,8.49,'Highly Ellip' from dual union all
select 26925,'026925','1991-050G','deb Ariane V44','',64219,'Reentered','2002-01-30 00:00:00','2001-09-21 00:00:00',99.58,717,759,98.21,'Sun-Synch' from dual union all
select 26926,'026926','1984-097C','SGS-II yo weight','',57151,'Reentered','2018-04-08 00:00:00','2001-09-19 00:00:00',365.96,1523,19617,64.65,'Highly Ellip' from dual union all
select 26927,'026927','2001-042A','Atlantic Bird 2','Atlantic Bird 2',67195,'In Earth orbit','','2002-12-06 00:00:00',1436.06,35767,35805,.03,'Stationary' from dual union all
select 26928,'026928','2001-042B','Ariane H10-3 V144','Ariane H10-3 S/N T4106',67195,'Reentered','2002-08-11 00:00:00','2001-09-27 00:00:00',651.38,176,36851,6.9,'GEO Transfer' from dual union all
select 26929,'026929','2001-043A','Starshine-OSCAR-43','Starshine 3',67196,'Reentered','2003-01-21 00:00:00','2001-09-30 00:00:00',94,467,473,67.05,'Intermediate' from dual union all
select 2693,'002693','1967-017A','Kosmos-143','Zenit-2 No. 45',15272,'Landed','1967-03-07 00:00:00','1967-03-01 00:00:00',89.46,202,294,64.98,'Intermediate' from dual union all
select 26930,'026930','2001-043B','Picosat','P97-1',67196,'In Earth orbit','','2001-09-30 00:00:00',100.74,789,799,67,'Intermediate' from dual union all
select 26931,'026931','2001-043C','Navy-OSCAR-44','PCSat',67196,'In Earth orbit','','2001-09-30 00:00:00',100.77,790,800,67.06,'Intermediate' from dual union all
select 26932,'026932','2001-043D','Navy-OSCAR-45','SAPPHIRE',67196,'In Earth orbit','','2001-09-30 00:00:00',100.8,792,801,67.05,'Intermediate' from dual union all
select 26933,'026933','2001-043E','OAM','OAM',67196,'Reentered','2001-10-06 00:00:00','2001-09-30 00:00:00',90.7,215,403,67.21,'Intermediate' from dual union all
select 26934,'026934','2001-044A','USA 161','Enhanced CRYSTAL 2107',67197,'Deorbited','2014-11-20 00:00:00','2001-10-05 00:00:00',97.13,202,1041,97.8,'Sun-Synch' from dual union all
select 26935,'026935','2001-044B','Titan 4B-34 Stage 2','Titan 4B-34 Stage 2',67197,'Reentered','2001-12-10 00:00:00','2001-10-05 00:00:00',97.13,202,1041,97.8,'Sun-Synch' from dual union all
select 26936,'026936','2001-045A','Raduga-1','Globus No. 14L',67198,'In Earth orbit','','2002-10-26 00:00:00',1436.06,35784,35788,.58,'Stationary' from dual union all
select 26937,'026937','2001-045B','8S812','8S812',67198,'Reentered','2001-10-09 00:00:00','2001-10-08 00:00:00',87.97,171,177,51.66,'Intermediate' from dual union all
select 26938,'026938','2001-045C','Perekhodnik','Sredniy perekhodnik',67198,'Reentered','2001-10-07 00:00:00','2001-10-07 00:00:00',87.42,143,151,51.64,'Intermediate' from dual union all
select 26939,'026939','2001-045D','Blok DM2 No. 93L','11S861 No. 93L',67198,'In Earth orbit','','2009-06-30 00:00:00',1435.24,35650,35890,5.68,'Inclined Drift' from dual union all
select 2694,'002694','1967-017B','8A92E','8A92E',15272,'Reentered','1967-03-09 00:00:00','1967-03-01 00:00:00',89.53,216,287,64.99,'Intermediate' from dual union all
select 26940,'026940','2001-045E','SOZ','SOZ',67198,'Reentered','2002-05-31 00:00:00','2001-10-06 00:00:00',633.84,224,35904,48.78,'GEO Transfer' from dual union all
select 26941,'026941','2001-045F','SOZ','SOZ',67198,'Reentered','2002-04-19 00:00:00','2001-10-06 00:00:00',633.84,224,35904,48.78,'GEO Transfer' from dual union all
select 26942,'026942','2001-045G','Proton deb','',67198,'Reentered','2001-10-10 00:00:00','2001-10-08 00:00:00',91.67,179,533,51.67,'Intermediate' from dual union all
select 26943,'026943','1998-067F','Pirs thermal blanket','',66503,'Reentered','2001-10-10 00:00:00','2001-10-10 00:00:00',88.86,208,228,51.61,'Intermediate' from dual union all
select 26944,'026944','1998-067G','Pirs thermal blanket','',66503,'Reentered','2001-10-18 00:00:00','2001-10-10 00:00:00',90.08,255,302,51.58,'Intermediate' from dual union all
select 26945,'026945','1998-067H','Pirs thermal blanket','',66503,'Reentered','2001-10-19 00:00:00','2001-10-11 00:00:00',91.83,359,370,51.64,'Intermediate' from dual union all
select 26946,'026946','1998-067J','Pirs thermal blanket','',66503,'Reentered','2001-10-17 00:00:00','2001-10-11 00:00:00',91.53,341,359,51.64,'Intermediate' from dual union all
select 26947,'026947','2001-045H','Proton deb','',67198,'Reentered','2001-10-10 00:00:00','2001-10-11 00:00:00',89.48,165,333,51.63,'Intermediate' from dual union all
select 26948,'026948','2001-046A','USA 162','QUASAR 13',67199,'In Earth orbit','','2002-01-01 00:00:00',1436.27,35780,35800,0,'Stationary' from dual union all
select 26949,'026949','2001-046B','Centaur AC-162','Centaur IIA AC-162',67199,'In Earth orbit','','2001-10-11 00:00:00',666.85,274,37538,26.5,'GEO Transfer' from dual union all
select 2695,'002695','1967-018A','Kosmos-144','Meteor 11F614 No. 6',15273,'Reentered','1982-09-14 00:00:00','1967-04-01 00:00:00',96.83,581,633,81.2,'Intermediate' from dual union all
select 26950,'026950','1998-067K','EVA debris','',66503,'Reentered','2001-10-12 00:00:00','2001-10-11 00:00:00',90.46,280,314,51.66,'Intermediate' from dual union all
select 26951,'026951','1998-067L','EVA debris','',66503,'Reentered','2001-10-26 00:00:00','2001-10-16 00:00:00',92.43,388,400,51.64,'Intermediate' from dual union all
select 26952,'026952','1998-067M','EVA debris','',66503,'Reentered','2001-10-26 00:00:00','2001-10-16 00:00:00',92.38,385,398,51.64,'Intermediate' from dual union all
select 26953,'026953','2001-047A','QuickBird 2','QuickBird',67200,'Reentered','2015-01-27 00:00:00','2001-10-19 00:00:00',93.85,461,465,97.22,'Sun-Synch' from dual union all
select 26954,'026954','2001-047B','Delta 288','Delta SSPS AJ-10-118K',67200,'Reentered','2001-10-25 00:00:00','2001-10-18 00:00:00',90.66,167,447,108.88,'Retrograde' from dual union all
select 26955,'026955','2001-048A','Soyuz TM-33','Soyuz 7K-STM No. 207',67201,'Landed','2002-05-05 00:00:00','2001-10-23 00:00:00',92.41,387,399,51.64,'Intermediate' from dual union all
select 26956,'026956','2001-048B','11S510','11S510',67201,'Reentered','2001-10-22 00:00:00','2001-10-21 00:00:00',88.25,182,193,51.64,'Intermediate' from dual union all
select 26957,'026957','2001-049A','TES','TES',67202,'In Earth orbit','','2001-10-24 00:00:00',95.95,550,579,97.78,'Sun-Synch' from dual union all
select 26958,'026958','2001-049B','PROBA','PROBA',67202,'In Earth orbit','','2001-10-24 00:00:00',96.98,553,676,97.91,'Sun-Synch' from dual union all
select 26959,'026959','2001-049C','BIRD','BIRD',67202,'In Earth orbit','','2001-10-24 00:00:00',95.96,550,580,97.78,'Sun-Synch' from dual union all
select 2696,'002696','1967-018B','8A92ME','8A92ME',15273,'Reentered','1989-09-01 00:00:00','1967-04-04 00:00:00',96.94,518,707,81.21,'Intermediate' from dual union all
select 26960,'026960','2001-049D','PSLV-C3-PS4','PS4 PSLV-C3',67202,'In Earth orbit','','2001-10-24 00:00:00',96.95,552,674,97.9,'Sun-Synch' from dual union all
select 26961,'026961','1964-049Q','deb Kosmos-41','',10335,'Reentered','2003-05-21 00:00:00','2001-10-23 00:00:00',721.62,861,39683,66.42,'GEO Transfer' from dual union all
select 26962,'026962','1966-040M','Nimbus 2 debris','',13447,'In Earth orbit','','2001-10-15 00:00:00',107.95,1091,1173,100.32,'Sun-Synch' from dual union all
select 26963,'026963','1978-064K','deb Seasat','',45259,'Reentered','2001-11-30 00:00:00','2001-10-19 00:00:00',98.57,675,706,108.01,'Retrograde' from dual union all
select 26964,'026964','1977-047E','deb Kosmos-917','',42175,'In Earth orbit','','2001-10-11 00:00:00',710.7,3427,36578,66.97,'GEO Transfer' from dual union all
select 26965,'026965','1996-041D','Star 48 yo weight','',65929,'Reentered','2007-02-07 00:00:00','2001-09-26 00:00:00',291.26,204,16150,35.04,'Highly Ellip' from dual union all
select 26966,'026966','1994-077L','deb Kosmos-2297','Rapid decay',65473,'Reentered','2002-01-04 00:00:00','2001-10-20 00:00:00',97.75,639,664,70.97,'Intermediate' from dual union all
select 26967,'026967','1993-017E','Star 48 yo weight','',65119,'In Earth orbit','','2001-10-18 00:00:00',260.87,218,14074,34.92,'Highly Ellip' from dual union all
select 26968,'026968','1993-016AJ','deb 11S772','',65118,'In Earth orbit','','2001-10-24 00:00:00',101.45,820,835,71.01,'Intermediate' from dual union all
select 26969,'026969','1993-007D','Star 48 yo weight','',65109,'Reentered','2007-04-21 00:00:00','2001-10-24 00:00:00',221.96,195,11335,34.55,'Medium' from dual union all
select 2697,'002697','1967-019A','Kosmos-145','DS-U2-M No. 2',15274,'Reentered','1968-03-08 00:00:00','1967-04-08 00:00:00',107.3,213,1991,48.45,'Intermediate' from dual union all
select 26970,'026970','2001-050A','Molniya-3','Molniya-3 No. 64',67203,'Reentered','2011-12-06 00:00:00','2001-10-26 00:00:00',736.49,615,40658,62.85,'Molniya' from dual union all
select 26971,'026971','2001-050B','11S510','11S510',67203,'Reentered','2001-11-27 00:00:00','2001-10-27 00:00:00',92.87,214,617,62.76,'Intermediate' from dual union all
select 26972,'026972','2001-050C','BOZ','BOZ',67203,'Reentered','2001-11-10 00:00:00','2001-10-27 00:00:00',92.45,184,606,62.81,'Intermediate' from dual union all
select 26973,'026973','2001-050D','Blok-ML','Blok-ML No. 71623-549',67203,'Reentered','2012-04-23 00:00:00','2001-10-25 00:00:00',733.98,614,40537,62.82,'Molniya' from dual union all
select 26974,'026974','1978-066E','deb Kosmos-1024','',45261,'In Earth orbit','','2001-10-17 00:00:00',704.4,3196,36497,67.27,'GEO Transfer' from dual union all
select 26975,'026975','1978-066F','deb Kosmos-1024','',45261,'In Earth orbit','','2001-10-28 00:00:00',700.24,4055,35431,68.83,'GEO Transfer' from dual union all
select 26976,'026976','1979-058M','deb Kosmos-1109','',47542,'Reentered','2012-08-12 00:00:00','2001-10-28 00:00:00',726.43,3773,37008,67.31,'GEO Transfer' from dual union all
select 26977,'026977','1980-057H','deb Kosmos-1191','',50267,'In Earth orbit','','2001-10-23 00:00:00',721.03,3286,37229,67.68,'GEO Transfer' from dual union all
select 26978,'026978','1980-057J','deb Kosmos-1191','',50267,'In Earth orbit','','2001-10-28 00:00:00',721.23,3343,37182,67.68,'GEO Transfer' from dual union all
select 26979,'026979','1992-093JJ','deb 11S772','',64805,'Reentered','2012-11-13 00:00:00','2001-10-31 00:00:00',102.19,762,963,71.33,'Intermediate' from dual union all
select 2698,'002698','1967-019B','S1M','S1M',15274,'Reentered','1967-12-01 00:00:00','1967-04-04 00:00:00',106.96,211,1961,48.43,'Intermediate' from dual union all
select 26980,'026980','1992-093JK','deb 11S772','',64805,'In Earth orbit','','2001-11-01 00:00:00',102.75,771,1007,71.12,'Intermediate' from dual union all
select 26981,'026981','1992-072L','deb Ariane V54','',64784,'In Earth orbit','','2001-07-08 00:00:00',497.43,1005,27831,9.55,'Highly Ellip' from dual union all
select 26982,'026982','1987-020DU','deb Kosmos-1823','',60110,'In Earth orbit','','2001-11-01 00:00:00',115.98,1478,1521,73.6,'Intermediate' from dual union all
select 26983,'026983','2001-051A','Progress M1-7','Progress M1 No. 256',67204,'Deorbited','2002-03-20 00:00:00','2001-11-28 00:00:00',89.23,229,244,51.64,'Intermediate' from dual union all
select 26984,'026984','2001-051B','11S510','11S510',67204,'Reentered','2001-11-28 00:00:00','2001-11-27 00:00:00',88.39,178,211,51.65,'Intermediate' from dual union all
select 26985,'026985','2001-052A','DirecTV-4S','DirecTV-4S',67205,'In Earth orbit','','2003-01-08 00:00:00',1436.06,35781,35791,.03,'Stationary' from dual union all
select 26986,'026986','2001-052B','Ariane H10-3 V146','Ariane H10-3 S/N T4107',67205,'Reentered','2015-05-12 00:00:00','2001-12-01 00:00:00',658.4,223,37161,6.96,'GEO Transfer' from dual union all
select 26987,'026987','2001-053A','Kosmos-2382','Uragan-M No. 711',67206,'In Earth orbit','','2001-12-05 00:00:00',675.2,19107,19126,64.83,'Medium' from dual union all
select 26988,'026988','2001-053B','Kosmos-2381','Uragan No. 789',67206,'In Earth orbit','','2001-12-05 00:00:00',675.14,19099,19131,64.84,'Medium' from dual union all
select 26989,'026989','2001-053C','Kosmos-2380','Uragan No. 790',67206,'In Earth orbit','','2001-12-05 00:00:00',675.16,19105,19126,64.84,'Medium' from dual union all
select 2699,'002699','1965-082KM','deb Transtage 4','',11832,'Reentered','1989-07-03 00:00:00','1967-03-02 00:00:00',104.42,738,1196,32.4,'Intermediate' from dual union all
select 26990,'026990','2001-053D','8S812','8S812',67206,'Reentered','2001-12-02 00:00:00','2001-12-01 00:00:00',87.41,134,159,64.9,'Intermediate' from dual union all
select 26991,'026991','2001-053E','Perekhodnik','Sredniy perekhodnik',67206,'Reentered','2001-12-01 00:00:00','2001-12-02 00:00:00',87.28,120,160,64.9,'Intermediate' from dual union all
select 26992,'026992','2001-053F','Blok DM-2 No. 101L','11S861 No. 101L',67206,'In Earth orbit','','2001-12-04 00:00:00',674.14,19055,19125,64.84,'Medium' from dual union all
select 26993,'026993','2001-053G','SOZ','SOZ',67206,'Reentered','2008-08-26 00:00:00','2001-12-04 00:00:00',340.01,386,19132,64.83,'Highly Ellip' from dual union all
select 26994,'026994','2001-053H','SOZ','SOZ',67206,'Reentered','2008-07-19 00:00:00','2001-12-05 00:00:00',340,386,19131,64.85,'Highly Ellip' from dual union all
select 26995,'026995','2001-054A','Endeavour (STS-108)','OV-105',67207,'Landed','2001-12-17 00:00:00','2001-12-05 00:00:00',89.04,225,230,51.6,'Intermediate' from dual union all
select 26996,'026996','2001-054B','Starshine 2','Starshine 2',67207,'Reentered','2002-04-26 00:00:00','2001-12-16 00:00:00',92.05,360,391,51.64,'Intermediate' from dual union all
select 26997,'026997','2001-055A','Jason','Jason',67208,'In Earth orbit','','2001-12-09 00:00:00',112.11,1320,1327,66.05,'Intermediate' from dual union all
select 26998,'026998','2001-055B','TIMED','TIMED',67208,'In Earth orbit','','2001-12-10 00:00:00',97.24,626,627,74.08,'Intermediate' from dual union all
select 26999,'026999','2001-055C','Delta 289','Delta SSPS AJ-10-118K',67208,'Reentered','2002-01-01 00:00:00','2001-12-10 00:00:00',92.51,198,598,76.64,'Intermediate' from dual union all
select 27,'000027','60 Alpha 1','Pioneer V','P-2',6317,'Deep Space','1960-03-13 00:00:00','',null,null,null,null,'' from dual union all
select 270,'000270','62 Iota 2','S1','S1',8191,'Reentered','1962-10-06 00:00:00','1962-05-13 00:00:00',100.09,210,1315,48.94,'Intermediate' from dual union all
select 2700,'002700','1965-089C','Explorer 29 despin weight','',11839,'In Earth orbit','','1967-02-12 00:00:00',119.17,1061,2227,59.6,'Intermediate' from dual union all
select 27000,'027000','2001-055D','Delta DPAF','DPAF',67208,'In Earth orbit','','2001-12-09 00:00:00',112.17,1324,1329,66.05,'Intermediate' from dual union all
select 27001,'027001','2001-056A','Meteor-3M No. 1','Meteor-3M 17F45 No. 101',67209,'In Earth orbit','','2001-12-11 00:00:00',105.24,996,1016,99.65,'Sun-Synch' from dual union all
select 27002,'027002','2001-056B','Kompass','Kompass',67209,'In Earth orbit','','2001-12-11 00:00:00',105.12,987,1014,99.64,'Sun-Synch' from dual union all
select 27003,'027003','2001-056C','Badr B','Badr B',67209,'In Earth orbit','','2001-12-11 00:00:00',105.1,985,1014,99.65,'Sun-Synch' from dual union all
select 27004,'027004','2001-056D','Zarkae Al Yamama','Maroc-Tubsat',67209,'In Earth orbit','','2001-12-11 00:00:00',105.1,985,1014,99.65,'Sun-Synch' from dual union all
select 27005,'027005','2001-056E','Reflector','Reflector',67209,'In Earth orbit','','2001-12-11 00:00:00',105.09,984,1014,99.65,'Sun-Synch' from dual union all
select 27006,'027006','2001-056F','11S772','11S772',67209,'In Earth orbit','','2001-12-11 00:00:00',105.05,986,1008,99.65,'Sun-Synch' from dual union all
select 27007,'027007','2001-056G','AR-7018','Agregat rasdeleniya',67209,'In Earth orbit','','2001-12-11 00:00:00',104.96,969,1017,99.67,'Sun-Synch' from dual;
insert into SATELLITE
select 27008,'027008','1999-057MH','CZ-4 debris','',66738,'Reentered','2002-02-27 00:00:00','2001-12-10 00:00:00',94.62,428,573,97.42,'Sun-Synch' from dual union all
select 27009,'027009','1999-072C','deb Kosmos-2367','',66753,'Reentered','2001-12-24 00:00:00','2001-12-10 00:00:00',91.88,361,373,65.01,'Intermediate' from dual union all
select 2701,'002701','1965-048E','deb NNS O-4','',11798,'In Earth orbit','','1967-01-21 00:00:00',106.57,1022,1114,89.96,'Polar' from dual union all
select 27010,'027010','1999-072D','deb Kosmos-2367','',66753,'Reentered','2001-12-17 00:00:00','2001-12-10 00:00:00',91.17,330,333,64.99,'Intermediate' from dual union all
select 27011,'027011','1999-072E','deb Kosmos-2367','',66753,'Reentered','2001-12-11 00:00:00','2001-12-09 00:00:00',90.34,287,296,64.87,'Intermediate' from dual union all
select 27012,'027012','1999-072F','deb Kosmos-2367','',66753,'Reentered','2001-12-13 00:00:00','2001-12-12 00:00:00',90.16,280,285,64.98,'Intermediate' from dual union all
select 27013,'027013','1999-072G','deb Kosmos-2367','',66753,'Reentered','2001-12-23 00:00:00','2001-12-13 00:00:00',92.46,356,434,64.59,'Intermediate' from dual union all
select 27014,'027014','1999-072H','deb Kosmos-2367','',66753,'Reentered','2001-12-13 00:00:00','2001-12-13 00:00:00',88.51,197,205,64.96,'Intermediate' from dual union all
select 27015,'027015','1999-072J','deb Kosmos-2367','',66753,'Reentered','2001-12-26 00:00:00','2001-12-13 00:00:00',92.67,366,445,64.8,'Intermediate' from dual union all
select 27016,'027016','1999-072K','deb Kosmos-2367','',66753,'Reentered','2002-01-20 00:00:00','2001-12-13 00:00:00',97.6,397,891,64.45,'Intermediate' from dual union all
select 27017,'027017','1999-072L','deb Kosmos-2367','',66753,'Reentered','2001-12-22 00:00:00','2001-12-13 00:00:00',91.79,351,373,64.66,'Intermediate' from dual union all
select 27018,'027018','1999-072M','deb Kosmos-2367','',66753,'Reentered','2001-12-24 00:00:00','2001-12-13 00:00:00',91.6,326,381,64.79,'Intermediate' from dual union all
select 27019,'027019','1999-072N','deb Kosmos-2367','',66753,'Reentered','2001-12-11 00:00:00','2001-12-11 00:00:00',88.63,196,217,65.13,'Intermediate' from dual union all
select 2702,'002702','1966-076D','NNS O-10 despin weight','Transit high deb',13483,'In Earth orbit','','1966-12-27 00:00:00',108.41,1085,1222,88.58,'Polar' from dual union all
select 27020,'027020','1999-072P','deb Kosmos-2367','',66753,'Reentered','2001-12-12 00:00:00','2001-12-12 00:00:00',89.06,221,235,64.96,'Intermediate' from dual union all
select 27021,'027021','1999-072Q','deb Kosmos-2367','',66753,'Reentered','2001-12-25 00:00:00','2001-12-13 00:00:00',91.62,352,355,65.01,'Intermediate' from dual union all
select 27022,'027022','1999-072R','deb Kosmos-2367','',66753,'Reentered','2001-12-14 00:00:00','2001-12-13 00:00:00',90.01,272,278,65.05,'Intermediate' from dual union all
select 27023,'027023','1999-072S','deb Kosmos-2367','',66753,'Reentered','2001-12-22 00:00:00','2001-12-13 00:00:00',91.4,343,344,64.99,'Intermediate' from dual union all
select 27024,'027024','1999-072T','deb Kosmos-2367','',66753,'Reentered','2002-01-09 00:00:00','2001-12-13 00:00:00',91.92,361,377,64.98,'Intermediate' from dual union all
select 27025,'027025','1986-026J','deb Ariane V17','',59219,'In Earth orbit','','2001-12-19 00:00:00',611.93,423,34573,6.37,'GEO Transfer' from dual union all
select 27026,'027026','1988-081J','deb Ariane V25','deb H10',61330,'In Earth orbit','','2001-12-19 00:00:00',631.89,515,35514,7.74,'GEO Transfer' from dual union all
select 27027,'027027','1988-081K','deb Ariane V25','deb H10',61330,'In Earth orbit','','2001-11-28 00:00:00',652.35,745,36331,8.1,'GEO Transfer' from dual union all
select 27028,'027028','1989-006H','deb Ariane V28','',62283,'In Earth orbit','','2001-12-15 00:00:00',634.32,548,35606,8.19,'GEO Transfer' from dual union all
select 27029,'027029','1989-006J','deb Ariane V28','',62283,'In Earth orbit','','2001-12-05 00:00:00',635.78,593,35636,8.61,'GEO Transfer' from dual union all
select 2703,'002703','1967-020A','OSO 3','OSO E1',15275,'Reentered','1982-04-04 00:00:00','1967-04-07 00:00:00',95.67,537,565,32.87,'Intermediate' from dual union all
select 27030,'027030','1989-006K','deb Ariane V28','',62283,'In Earth orbit','','2001-12-13 00:00:00',632.03,493,35543,8.41,'GEO Transfer' from dual union all
select 27031,'027031','1989-006L','deb Ariane V28','',62283,'In Earth orbit','','2001-12-20 00:00:00',627.95,455,35371,8.2,'GEO Transfer' from dual union all
select 27032,'027032','1989-006M','deb Ariane V28','',62283,'In Earth orbit','','2001-12-19 00:00:00',634.4,535,35623,8.6,'GEO Transfer' from dual union all
select 27033,'027033','1989-006N','deb Ariane V28','',62283,'In Earth orbit','','2001-12-16 00:00:00',633.22,533,35564,8.48,'GEO Transfer' from dual union all
select 27034,'027034','1989-006P','deb Ariane V28','',62283,'In Earth orbit','','2001-12-20 00:00:00',640.76,733,35751,8.16,'GEO Transfer' from dual union all
select 27035,'027035','1989-006Q','deb Ariane V28','',62283,'In Earth orbit','','2001-12-06 00:00:00',631.86,515,35512,8.38,'GEO Transfer' from dual union all
select 27036,'027036','1989-006R','deb Ariane V28','',62283,'In Earth orbit','','2001-12-19 00:00:00',638.32,630,35729,8.99,'GEO Transfer' from dual union all
select 27037,'027037','1989-006S','deb Ariane V28','',62283,'In Earth orbit','','2001-12-17 00:00:00',635.38,515,35693,8.25,'GEO Transfer' from dual union all
select 27038,'027038','1989-006T','deb Ariane V28','',62283,'In Earth orbit','','2001-12-18 00:00:00',633.2,399,35697,8.53,'GEO Transfer' from dual union all
select 27039,'027039','1989-006U','deb Ariane V28','',62283,'In Earth orbit','','2001-12-18 00:00:00',637.3,583,35724,8.35,'GEO Transfer' from dual union all
select 2704,'002704','1967-020B','Altair 2 X-258E6 RH-140','X-258E6 RH-140',15275,'Reentered','1970-04-19 00:00:00','1967-04-08 00:00:00',95.56,536,556,32.87,'Intermediate' from dual union all
select 27040,'027040','1989-006V','deb Ariane V28','',62283,'In Earth orbit','','2001-12-19 00:00:00',643.61,920,35710,8.15,'GEO Transfer' from dual union all
select 27041,'027041','1989-006W','deb Ariane V28','',62283,'In Earth orbit','','2001-12-19 00:00:00',635.01,466,35723,8.58,'GEO Transfer' from dual union all
select 27042,'027042','1989-006X','deb Ariane V28','',62283,'In Earth orbit','','2001-12-13 00:00:00',640.48,699,35770,8.29,'GEO Transfer' from dual union all
select 27043,'027043','1989-006Y','deb Ariane V28','',62283,'In Earth orbit','','2001-12-17 00:00:00',640.29,711,35749,8.28,'GEO Transfer' from dual union all
select 27044,'027044','1989-006Z','deb Ariane V28','',62283,'In Earth orbit','','2001-12-14 00:00:00',633.72,497,35626,8.29,'GEO Transfer' from dual union all
select 27045,'027045','1989-006AA','deb Ariane V28','',62283,'In Earth orbit','','2001-12-20 00:00:00',639.21,695,35710,8.57,'GEO Transfer' from dual union all
select 27046,'027046','1989-006AB','deb Ariane V28','',62283,'In Earth orbit','','2001-12-13 00:00:00',642.79,806,35782,8.04,'GEO Transfer' from dual union all
select 27047,'027047','1989-062K','deb Ariane V33','H10 deb',62339,'In Earth orbit','','2001-12-09 00:00:00',585.18,398,33196,9.3,'Highly Ellip' from dual union all
select 27048,'027048','1989-062L','deb Ariane V33','H10 deb',62339,'In Earth orbit','','2001-11-24 00:00:00',563.93,368,32097,8.42,'Highly Ellip' from dual union all
select 27049,'027049','1989-062M','deb Ariane V33','H10 deb',62339,'In Earth orbit','','2001-12-05 00:00:00',602.19,333,34155,8.89,'GEO Transfer' from dual union all
select 2705,'002705','1967-021A','Kosmos-146','L-1 No. 2P',15276,'Deep Space','1967-03-11 00:00:00','1967-03-11 00:00:00',15565.01,250,400000,51.51,'Cislunar' from dual union all
select 27050,'027050','1989-062N','deb Ariane V33','H10 deb',62339,'Reentered','2004-01-03 00:00:00','2001-12-19 00:00:00',439.54,289,25254,8.21,'Highly Ellip' from dual union all
select 27051,'027051','1990-065U','G-11A Canister','Ba Canister',63307,'In Earth orbit','','2001-12-20 00:00:00',558.38,333,31835,18.35,'Highly Ellip' from dual union all
select 27052,'027052','1990-065V','G-11B Canister','Ba Canister',63307,'In Earth orbit','','2001-08-11 00:00:00',563.39,337,32099,18.12,'Highly Ellip' from dual union all
select 27053,'027053','2001-057A','Kosmos-2383','US-PU',67210,'Reentered','2004-03-20 00:00:00','2001-12-21 00:00:00',92.75,404,415,65,'Intermediate' from dual union all
select 27054,'027054','2001-057B','11S692','11S692',67210,'Reentered','2001-12-21 00:00:00','2001-12-21 00:00:00',90,145,404,65,'Intermediate' from dual union all
select 27055,'027055','2001-058A','Kosmos-2384','Strela-3',67211,'In Earth orbit','','2002-01-09 00:00:00',114.1,1411,1418,82.53,'Intermediate' from dual union all
select 27056,'027056','2001-058B','Kosmos-2385','Strela-3',67211,'In Earth orbit','','2002-01-09 00:00:00',114.14,1416,1417,82.54,'Intermediate' from dual union all
select 27057,'027057','2001-058C','Kosmos-2386','Strela-3',67211,'In Earth orbit','','2002-01-04 00:00:00',114.01,1404,1417,82.54,'Intermediate' from dual union all
select 27058,'027058','2001-058D','Gonets-D1 No. 207','Gonets-D1 No. 10 (06)',67211,'In Earth orbit','','2002-01-09 00:00:00',114.29,1416,1431,82.54,'Intermediate' from dual union all
select 27059,'027059','2001-058E','Gonets-D1 No. 208','Gonets-D1 No. 11 (07)',67211,'In Earth orbit','','2002-01-09 00:00:00',114.23,1416,1425,82.54,'Intermediate' from dual union all
select 2706,'002706','1967-006C','ESSA 4 despin weight','',15261,'In Earth orbit','','1967-02-26 00:00:00',114.26,1395,1449,102.12,'Sun-Synch' from dual union all
select 27060,'027060','2001-058F','Gonets-D1 No. 209','Gonets-D1 No. 12 (10)',67211,'In Earth orbit','','2002-01-10 00:00:00',114.14,1414,1419,82.54,'Intermediate' from dual union all
select 27061,'027061','2001-058G','S5M','S5M',67211,'In Earth orbit','','2002-01-09 00:00:00',114.82,1415,1480,82.54,'Intermediate' from dual union all
select 27062,'027062','2001-049E','deb PSLV-C3 PS4','',67202,'Reentered','2001-12-30 00:00:00','2001-12-29 00:00:00',89.54,196,308,98.54,'Sun-Synch' from dual union all
select 27063,'027063','2001-049F','deb PSLV-C3 PS4','',67202,'Reentered','2015-06-17 00:00:00','2002-01-14 00:00:00',97,553,677,97.62,'Sun-Synch' from dual union all
select 27064,'027064','2001-049G','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-13 00:00:00','2002-01-15 00:00:00',96.66,542,656,97.78,'Sun-Synch' from dual union all
select 27065,'027065','2001-049H','deb PSLV-C3 PS4','',67202,'Reentered','2003-05-13 00:00:00','2002-01-14 00:00:00',95.83,455,663,97.74,'Sun-Synch' from dual union all
select 27066,'027066','2001-049J','deb PSLV-C3 PS4','',67202,'Reentered','2002-08-16 00:00:00','2002-01-15 00:00:00',96.26,492,667,98.1,'Sun-Synch' from dual union all
select 27067,'027067','2001-049K','deb PSLV-C3 PS4','',67202,'Reentered','2003-01-25 00:00:00','2002-01-14 00:00:00',96.86,553,663,97.81,'Sun-Synch' from dual union all
select 27068,'027068','2001-049L','deb PSLV-C3 PS4','',67202,'Reentered','2013-09-16 00:00:00','2002-01-15 00:00:00',96.46,508,671,97.8,'Sun-Synch' from dual union all
select 27069,'027069','2001-049M','deb PSLV-C3 PS4','',67202,'Reentered','2014-11-11 00:00:00','2002-01-15 00:00:00',97.05,565,669,97.8,'Sun-Synch' from dual union all
select 2707,'002707','1967-006D','ESSA 4 despin weight','',15261,'In Earth orbit','','1967-03-11 00:00:00',112.63,1232,1463,101.86,'Sun-Synch' from dual union all
select 27070,'027070','2001-049N','deb PSLV-C3 PS4','',67202,'Reentered','2003-01-28 00:00:00','2002-01-14 00:00:00',95.51,429,658,98.77,'Sun-Synch' from dual union all
select 27071,'027071','2001-049P','deb PSLV-C3 PS4','',67202,'Reentered','2002-08-01 00:00:00','2002-01-14 00:00:00',94.99,388,649,98.79,'Sun-Synch' from dual union all
select 27072,'027072','2001-049Q','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-19 00:00:00','2002-01-15 00:00:00',95.53,449,640,98.11,'Sun-Synch' from dual union all
select 27073,'027073','2001-049R','deb PSLV-C3 PS4','',67202,'Reentered','2002-06-06 00:00:00','2002-01-14 00:00:00',95.32,416,653,97.44,'Sun-Synch' from dual union all
select 27074,'027074','2001-049S','deb PSLV-C3 PS4','',67202,'Reentered','2002-10-25 00:00:00','2002-01-15 00:00:00',96.27,458,702,97.22,'Sun-Synch' from dual union all
select 27075,'027075','2001-049T','deb PSLV-C3 PS4','',67202,'Reentered','2014-10-03 00:00:00','2002-01-14 00:00:00',99.73,549,943,96.37,'Sun-Synch' from dual union all
select 27076,'027076','2001-049U','deb PSLV-C3 PS4','',67202,'Reentered','2003-04-27 00:00:00','2002-01-14 00:00:00',95.66,410,691,97.17,'Sun-Synch' from dual union all
select 27077,'027077','2001-049V','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-06 00:00:00','2002-01-15 00:00:00',95.52,428,660,97.5,'Sun-Synch' from dual union all
select 27078,'027078','2001-049W','deb PSLV-C3 PS4','',67202,'Reentered','2003-04-14 00:00:00','2002-01-15 00:00:00',96.97,554,673,97.84,'Sun-Synch' from dual union all
select 27079,'027079','2001-049X','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-07 00:00:00','2002-01-15 00:00:00',94.85,370,653,97.58,'Sun-Synch' from dual union all
select 2708,'002708','1966-112E','deb Kosmos-135                    ','[UNID015]',13519,'Reentered','1967-05-28 00:00:00','1967-01-19 00:00:00',94.67,371,635,47.76,'Intermediate' from dual union all
select 27080,'027080','2001-049Y','deb PSLV-C3 PS4','',67202,'Reentered','2002-06-11 00:00:00','2002-01-14 00:00:00',95.66,443,658,97.66,'Sun-Synch' from dual union all
select 27081,'027081','2001-049Z','deb PSLV-C3 PS4','',67202,'Reentered','2002-09-05 00:00:00','2002-01-15 00:00:00',95.96,459,671,96.95,'Sun-Synch' from dual union all
select 27082,'027082','2001-049AA','deb PSLV-C3 PS4','',67202,'Reentered','2003-01-10 00:00:00','2002-01-14 00:00:00',96.46,508,671,97.83,'Sun-Synch' from dual union all
select 27083,'027083','2001-049AB','deb PSLV-C3 PS4','',67202,'Reentered','2004-05-11 00:00:00','2002-01-14 00:00:00',96.64,535,661,97.78,'Sun-Synch' from dual union all
select 27084,'027084','2001-049AC','deb PSLV-C3 PS4','',67202,'Reentered','2003-05-01 00:00:00','2002-01-15 00:00:00',96.59,527,664,97.77,'Sun-Synch' from dual union all
select 27085,'027085','2001-049AD','deb PSLV-C3 PS4','',67202,'Reentered','2012-07-17 00:00:00','2002-01-14 00:00:00',96.74,532,673,97.7,'Sun-Synch' from dual union all
select 27086,'027086','2001-049AE','deb PSLV-C3 PS4','',67202,'Reentered','2002-12-21 00:00:00','2002-01-14 00:00:00',95.61,439,658,98.77,'Sun-Synch' from dual union all
select 27087,'027087','2001-049AF','deb PSLV-C3 PS4','',67202,'Reentered','2009-05-02 00:00:00','2002-01-14 00:00:00',96.49,514,667,97.77,'Sun-Synch' from dual union all
select 27088,'027088','2001-049AG','deb PSLV-C3 PS4','',67202,'Reentered','2013-08-13 00:00:00','2002-01-14 00:00:00',96.65,530,667,97.77,'Sun-Synch' from dual union all
select 27089,'027089','2001-049AH','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',97.2,575,674,97.69,'Sun-Synch' from dual union all
select 2709,'002709','1967-021B','SOZ','SOZ',15276,'Reentered','1967-03-19 00:00:00','1967-03-12 00:00:00',89.58,184,324,51.51,'Intermediate' from dual union all
select 27090,'027090','2001-049AJ','deb PSLV-C3 PS4','',67202,'Reentered','2003-02-11 00:00:00','2002-01-15 00:00:00',97.17,541,706,97.76,'Sun-Synch' from dual union all
select 27091,'027091','2001-049AK','deb PSLV-C3 PS4','',67202,'Reentered','2014-01-10 00:00:00','2002-01-15 00:00:00',96.92,552,671,97.79,'Sun-Synch' from dual union all
select 27092,'027092','2001-049AL','deb PSLV-C3 PS4','',67202,'Reentered','2002-08-23 00:00:00','2002-01-14 00:00:00',95.7,443,662,97.58,'Sun-Synch' from dual union all
select 27093,'027093','2001-049AM','deb PSLV-C3 PS4','',67202,'Reentered','2004-10-27 00:00:00','2002-01-15 00:00:00',96.94,556,668,97.82,'Sun-Synch' from dual union all
select 27094,'027094','2001-049AN','deb PSLV-C3 PS4','',67202,'Reentered','2002-10-07 00:00:00','2002-01-15 00:00:00',96.84,551,664,97.84,'Sun-Synch' from dual union all
select 27095,'027095','2001-049AP','deb PSLV-C3 PS4','',67202,'Reentered','2002-10-16 00:00:00','2002-01-15 00:00:00',96.46,468,710,97.77,'Sun-Synch' from dual union all
select 27096,'027096','2001-049AQ','deb PSLV-C3 PS4','',67202,'Reentered','2003-10-22 00:00:00','2002-01-14 00:00:00',96.4,462,711,97.14,'Sun-Synch' from dual union all
select 27097,'027097','2001-049AR','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-15 00:00:00',97.88,560,755,97.35,'Sun-Synch' from dual union all
select 27098,'027098','2001-049AS','deb PSLV-C3 PS4','',67202,'Reentered','2012-04-21 00:00:00','2002-01-15 00:00:00',96.87,548,670,97.79,'Sun-Synch' from dual;
insert into SATELLITE
select 27099,'027099','2001-049AT','deb PSLV-C3 PS4','',67202,'Reentered','2003-08-28 00:00:00','2002-01-15 00:00:00',96.42,513,662,97.73,'Sun-Synch' from dual union all
select 271,'000271','62 Kappa 1','Midas 5','Midas ETS-II F3',8192,'In Earth orbit','','1962-09-16 00:00:00',152.9,2818,3370,86.69,'Medium' from dual union all
select 2710,'002710','1967-022A','Kosmos-147','Zenit-2 No. 44',15277,'Landed','1967-03-21 00:00:00','1967-03-15 00:00:00',89.33,199,283,64.56,'Intermediate' from dual union all
select 27100,'027100','2001-049AU','deb PSLV-C3 PS4','',67202,'Reentered','2002-11-20 00:00:00','2002-01-14 00:00:00',95.44,423,657,98.84,'Sun-Synch' from dual union all
select 27101,'027101','2001-049AV','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-21 00:00:00','2002-01-15 00:00:00',94.92,389,641,97.76,'Sun-Synch' from dual union all
select 27102,'027102','2001-049AW','deb PSLV-C3 PS4','',67202,'Reentered','2002-11-12 00:00:00','2002-01-15 00:00:00',96.81,548,663,97.84,'Sun-Synch' from dual union all
select 27103,'027103','2001-049AX','deb PSLV-C3 PS4','',67202,'Reentered','2002-12-14 00:00:00','2002-01-14 00:00:00',96.42,512,663,97.56,'Sun-Synch' from dual union all
select 27104,'027104','2001-049AY','deb PSLV-C3 PS4','',67202,'Reentered','2010-10-19 00:00:00','2002-01-14 00:00:00',96.3,491,671,98.86,'Sun-Synch' from dual union all
select 27105,'027105','2001-049AZ','deb PSLV-C3 PS4','',67202,'Reentered','2016-10-29 00:00:00','2002-01-14 00:00:00',97.02,560,672,97.79,'Sun-Synch' from dual union all
select 27106,'027106','2001-049BA','deb PSLV-C3 PS4','',67202,'Reentered','2002-09-02 00:00:00','2002-01-15 00:00:00',96.77,536,671,98.01,'Sun-Synch' from dual union all
select 27107,'027107','2001-049BB','deb PSLV-C3 PS4','',67202,'Reentered','2013-09-16 00:00:00','2002-01-14 00:00:00',96.59,521,670,97.79,'Sun-Synch' from dual union all
select 27108,'027108','2001-049BC','deb PSLV-C3 PS4','',67202,'Reentered','2002-11-30 00:00:00','2002-01-15 00:00:00',96.72,543,661,97.77,'Sun-Synch' from dual union all
select 27109,'027109','2001-049BD','deb PSLV-C3 PS4','',67202,'Reentered','2004-04-08 00:00:00','2002-01-14 00:00:00',96.93,557,666,97.81,'Sun-Synch' from dual union all
select 2711,'002711','1967-022B','8A92E','8A92E',15277,'Reentered','1967-03-22 00:00:00','1967-03-15 00:00:00',89.47,211,286,64.59,'Intermediate' from dual union all
select 27110,'027110','2001-049BE','deb PSLV-C3 PS4','',67202,'Reentered','2002-07-14 00:00:00','2002-01-15 00:00:00',95.43,405,674,97.43,'Sun-Synch' from dual union all
select 27111,'027111','2001-049BF','deb PSLV-C3 PS4','',67202,'Reentered','2003-11-21 00:00:00','2002-01-15 00:00:00',96.78,542,666,97.78,'Sun-Synch' from dual union all
select 27112,'027112','2001-049BG','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-14 00:00:00','2002-01-15 00:00:00',95.2,425,632,98.17,'Sun-Synch' from dual union all
select 27113,'027113','2001-049BH','deb PSLV-C3 PS4','',67202,'Reentered','2003-05-23 00:00:00','2002-01-15 00:00:00',96.93,560,664,97.67,'Sun-Synch' from dual union all
select 27114,'027114','2001-049BJ','deb PSLV-C3 PS4','',67202,'Reentered','2017-03-04 00:00:00','2002-01-14 00:00:00',97.74,616,686,97.72,'Sun-Synch' from dual union all
select 27115,'027115','2001-049BK','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',97.17,567,680,97.66,'Sun-Synch' from dual union all
select 27116,'027116','2001-049BL','deb PSLV-C3 PS4','',67202,'Reentered','2003-12-18 00:00:00','2002-01-15 00:00:00',97.66,625,669,97.96,'Sun-Synch' from dual union all
select 27117,'027117','2001-049BM','deb PSLV-C3 PS4','',67202,'Reentered','2003-01-13 00:00:00','2002-01-15 00:00:00',96.67,537,662,97.84,'Sun-Synch' from dual union all
select 27118,'027118','2001-049BN','deb PSLV-C3 PS4','',67202,'Reentered','2005-07-20 00:00:00','2002-01-15 00:00:00',96.83,547,667,97.81,'Sun-Synch' from dual union all
select 27119,'027119','2001-049BP','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',97.23,580,673,97.78,'Sun-Synch' from dual union all
select 2712,'002712','1967-023A','Kosmos-148','DS-P1-I No. 2',15278,'Reentered','1967-05-07 00:00:00','1967-03-26 00:00:00',91.06,270,384,71,'Intermediate' from dual union all
select 27120,'027120','2001-049BQ','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-13 00:00:00','2002-01-15 00:00:00',95.81,486,630,97.74,'Sun-Synch' from dual union all
select 27121,'027121','2001-049BR','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',98.68,613,778,97.42,'Sun-Synch' from dual union all
select 27122,'027122','2001-049BS','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-29 00:00:00','2002-01-15 00:00:00',98.26,615,736,97.77,'Sun-Synch' from dual union all
select 27123,'027123','2001-049BT','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',98.91,670,744,97.4,'Sun-Synch' from dual union all
select 27124,'027124','2001-049BU','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',98.1,636,699,97.98,'Sun-Synch' from dual union all
select 27125,'027125','2001-049BV','deb PSLV-C3 PS4','',67202,'Reentered','2016-09-29 00:00:00','2002-01-15 00:00:00',98.16,658,684,97.58,'Sun-Synch' from dual union all
select 27126,'027126','2001-049BW','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',97.95,618,704,98.15,'Sun-Synch' from dual union all
select 27127,'027127','2001-049BX','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',97.93,620,700,98.06,'Sun-Synch' from dual union all
select 27128,'027128','2001-049BY','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',98.31,587,769,97.49,'Sun-Synch' from dual union all
select 27129,'027129','2001-049BZ','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',98.82,652,753,98.14,'Sun-Synch' from dual union all
select 2713,'002713','1967-023B','S1M','S1M',15278,'Reentered','1967-04-14 00:00:00','1967-03-23 00:00:00',90.91,274,364,71,'Intermediate' from dual union all
select 27130,'027130','2001-049CA','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',97.99,649,676,97.97,'Sun-Synch' from dual union all
select 27131,'027131','2001-049CB','deb PSLV-C3 PS4','',67202,'Reentered','2004-08-24 00:00:00','2002-01-14 00:00:00',97.8,634,673,97.97,'Sun-Synch' from dual union all
select 27132,'027132','2001-049CC','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',97.09,570,669,97.83,'Sun-Synch' from dual union all
select 27133,'027133','2001-049CD','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',98.8,665,738,97.72,'Sun-Synch' from dual union all
select 27134,'027134','2001-049CE','deb PSLV-C3 PS4','',67202,'Reentered','2012-06-04 00:00:00','2002-01-14 00:00:00',97.29,585,672,97.86,'Sun-Synch' from dual union all
select 27135,'027135','2001-049CF','deb PSLV-C3 PS4','',67202,'Reentered','2011-07-22 00:00:00','2002-01-14 00:00:00',97.6,619,669,97.8,'Sun-Synch' from dual union all
select 27136,'027136','2001-049CG','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',97.79,613,693,97.9,'Sun-Synch' from dual union all
select 27137,'027137','2001-049CH','deb PSLV-C3 PS4','',67202,'Reentered','2013-02-25 00:00:00','2002-01-14 00:00:00',98.14,638,702,97.63,'Sun-Synch' from dual union all
select 27138,'027138','2001-049CJ','deb PSLV-C3 PS4','',67202,'Reentered','2004-12-01 00:00:00','2002-01-15 00:00:00',97.13,574,669,97.76,'Sun-Synch' from dual union all
select 27139,'027139','2001-049CK','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-19 00:00:00','2002-01-15 00:00:00',95.11,411,636,97.57,'Sun-Synch' from dual union all
select 2714,'002714','1967-024A','Kosmos-149','DS-MO No. 1',15279,'Reentered','1967-04-07 00:00:00','1967-03-29 00:00:00',89.44,230,264,48.39,'Intermediate' from dual union all
select 27140,'027140','2001-049CL','deb PSLV-C3 PS4','',67202,'Reentered','2014-12-29 00:00:00','2002-01-14 00:00:00',97.41,592,678,97.91,'Sun-Synch' from dual union all
select 27141,'027141','2001-049CM','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',97.78,607,698,97.96,'Sun-Synch' from dual union all
select 27142,'027142','2001-049CN','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',102.45,653,1096,97.4,'Sun-Synch' from dual union all
select 27143,'027143','2001-049CP','deb PSLV-C3 PS4','',67202,'Reentered','2002-09-30 00:00:00','2002-01-15 00:00:00',95.08,392,653,98.74,'Sun-Synch' from dual union all
select 27144,'027144','2001-049CQ','deb PSLV-C3 PS4','',67202,'Reentered','2005-01-10 00:00:00','2002-01-15 00:00:00',96.88,555,664,97.78,'Sun-Synch' from dual union all
select 27145,'027145','2001-049CR','deb PSLV-C3 PS4','',67202,'Reentered','2003-09-05 00:00:00','2002-01-14 00:00:00',99.54,450,1024,98.66,'Sun-Synch' from dual union all
select 27146,'027146','2001-049CS','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',98.52,639,736,98.03,'Sun-Synch' from dual union all
select 27147,'027147','2001-049CT','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',98.88,669,742,98.17,'Sun-Synch' from dual union all
select 27148,'027148','2001-049CU','deb PSLV-C3 PS4','',67202,'Reentered','2014-12-16 00:00:00','2002-01-14 00:00:00',97.17,575,672,97.72,'Sun-Synch' from dual union all
select 27149,'027149','2001-049CV','deb PSLV-C3 PS4','',67202,'Reentered','2012-10-15 00:00:00','2002-01-14 00:00:00',96.8,537,674,97.86,'Sun-Synch' from dual union all
select 2715,'002715','1967-025A','Kosmos-150','Zenit-4 No. 27',15280,'Landed','1967-03-30 00:00:00','1967-03-24 00:00:00',90.02,199,352,65.64,'Intermediate' from dual union all
select 27150,'027150','2001-049CW','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',97.74,596,706,97.78,'Sun-Synch' from dual union all
select 27151,'027151','2001-049CX','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',97.3,586,672,97.87,'Sun-Synch' from dual union all
select 27152,'027152','2001-049CY','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',98.33,643,715,97.97,'Sun-Synch' from dual union all
select 27153,'027153','2001-049CZ','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',97.81,613,695,97.91,'Sun-Synch' from dual union all
select 27154,'027154','2001-049DA','deb PSLV-C3 PS4','',67202,'Reentered','2010-03-03 00:00:00','2002-01-15 00:00:00',97.86,583,730,98.2,'Sun-Synch' from dual union all
select 27155,'027155','2001-049DB','deb PSLV-C3 PS4','',67202,'Reentered','2015-02-10 00:00:00','2002-01-15 00:00:00',100.42,669,887,97.76,'Sun-Synch' from dual union all
select 27156,'027156','2001-049DC','deb PSLV-C3 PS4','',67202,'Reentered','2004-04-09 00:00:00','2002-01-14 00:00:00',97.24,577,677,97.89,'Sun-Synch' from dual union all
select 27157,'027157','2001-049DD','deb PSLV-C3 PS4','',67202,'Reentered','2006-08-28 00:00:00','2002-01-14 00:00:00',97.83,623,687,97.99,'Sun-Synch' from dual union all
select 27158,'027158','2001-049DE','deb PSLV-C3 PS4','',67202,'Reentered','2012-11-17 00:00:00','2002-01-14 00:00:00',98.47,646,725,97.51,'Sun-Synch' from dual union all
select 27159,'027159','2001-049DF','deb PSLV-C3 PS4','',67202,'Reentered','2009-08-25 00:00:00','2002-01-14 00:00:00',99.05,668,759,97.9,'Sun-Synch' from dual union all
select 2716,'002716','1967-024B','S1M','S1M',15279,'Reentered','1967-03-24 00:00:00','1967-03-22 00:00:00',89.29,222,257,48.45,'Intermediate' from dual union all
select 27160,'027160','2001-049DG','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-14 00:00:00',98.35,643,716,98.02,'Sun-Synch' from dual union all
select 27161,'027161','2001-049DH','deb PSLV-C3 PS4','',67202,'Reentered','2006-05-12 00:00:00','2002-01-14 00:00:00',99.26,647,800,97.79,'Sun-Synch' from dual union all
select 27162,'027162','2001-049DJ','deb PSLV-C3 PS4','',67202,'Reentered','2002-09-18 00:00:00','2002-01-15 00:00:00',95.27,411,653,98.43,'Sun-Synch' from dual union all
select 27163,'027163','2001-049DK','deb PSLV-C3 PS4','',67202,'Reentered','2002-11-24 00:00:00','2002-01-15 00:00:00',95.23,403,656,98.84,'Sun-Synch' from dual union all
select 27164,'027164','2001-049DL','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-28 00:00:00','2002-01-14 00:00:00',96.61,542,651,97.79,'Sun-Synch' from dual union all
select 27165,'027165','2001-049DM','deb PSLV-C3 PS4','',67202,'Reentered','2004-02-12 00:00:00','2002-01-15 00:00:00',97.69,616,680,98.03,'Sun-Synch' from dual union all
select 27166,'027166','1998-067N','Orlan-M glove','',66503,'Reentered','2002-01-16 00:00:00','2002-01-15 00:00:00',91.9,361,375,51.64,'Intermediate' from dual union all
select 27167,'027167','1998-067P','Orlan-M glove','',66503,'Reentered','2002-02-26 00:00:00','2002-01-15 00:00:00',92.44,389,400,51.64,'Intermediate' from dual union all
select 27168,'027168','2002-001A','Milstar 5','Milstar DFS-5',67377,'In Earth orbit','','2006-01-18 00:00:00',1436.09,35758,35815,1.7,'Stationary' from dual union all
select 27169,'027169','2002-001B','Centaur TC-19','Centaur TC-19',67377,'In Earth orbit','','2006-01-20 00:00:00',1436.78,35780,35820,2,'Drift GEO' from dual union all
select 2717,'002717','1967-026A','INTELSAT II F-3','INTELSAT II F-3',15281,'In Earth orbit','','1967-05-01 00:00:00',1436.06,35777,35795,1.31,'Stationary' from dual union all
select 27170,'027170','2001-049DN','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-16 00:00:00','2002-01-17 00:00:00',94.21,372,589,98.82,'Sun-Synch' from dual union all
select 27171,'027171','2001-049DP','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-07 00:00:00','2002-01-17 00:00:00',96.74,552,653,97.73,'Sun-Synch' from dual union all
select 27172,'027172','2001-049DQ','deb PSLV-C3 PS4','',67202,'Reentered','2002-08-17 00:00:00','2002-01-16 00:00:00',96.61,517,676,97.86,'Sun-Synch' from dual union all
select 27173,'027173','2001-049DR','deb PSLV-C3 PS4','',67202,'Reentered','2002-07-19 00:00:00','2002-01-16 00:00:00',96.73,550,655,97.89,'Sun-Synch' from dual union all
select 27174,'027174','2001-049DS','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-25 00:00:00','2002-01-16 00:00:00',96.49,530,651,97.73,'Sun-Synch' from dual union all
select 27175,'027175','2001-049DT','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-25 00:00:00','2002-01-17 00:00:00',96.51,532,650,97.8,'Sun-Synch' from dual union all
select 27176,'027176','2001-049DU','deb PSLV-C3 PS4','',67202,'Reentered','2002-01-23 00:00:00','2002-01-17 00:00:00',92.58,317,485,98.72,'Sun-Synch' from dual union all
select 27177,'027177','2001-049DV','deb PSLV-C3 PS4','',67202,'Reentered','2002-06-10 00:00:00','2002-01-17 00:00:00',96.4,526,647,97.61,'Sun-Synch' from dual union all
select 27178,'027178','2001-049DW','deb PSLV-C3 PS4','',67202,'Reentered','2002-12-10 00:00:00','2002-01-16 00:00:00',96.71,544,659,97.63,'Sun-Synch' from dual union all
select 27179,'027179','2001-049DX','deb PSLV-C3 PS4','',67202,'Reentered','2002-07-06 00:00:00','2002-01-16 00:00:00',96.58,453,737,96.77,'Sun-Synch' from dual union all
select 2718,'002718','1967-025B','11A57I','11A57I',15280,'Reentered','1967-03-30 00:00:00','1967-03-24 00:00:00',89.64,188,326,65.67,'Intermediate' from dual union all
select 27180,'027180','2001-049DY','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-13 00:00:00','2002-01-17 00:00:00',94.81,328,691,96.88,'Sun-Synch' from dual union all
select 27181,'027181','2001-049DZ','deb PSLV-C3 PS4','',67202,'Reentered','2003-06-18 00:00:00','2002-01-16 00:00:00',96.86,552,664,97.82,'Sun-Synch' from dual union all
select 27182,'027182','2001-049EA','deb PSLV-C3 PS4','',67202,'Reentered','2002-01-18 00:00:00','2002-01-15 00:00:00',90.62,248,362,98.9,'Sun-Synch' from dual union all
select 27183,'027183','2001-049EB','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-04 00:00:00','2002-01-16 00:00:00',94.53,377,615,97.7,'Sun-Synch' from dual union all
select 27184,'027184','2001-049EC','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-19 00:00:00','2002-01-17 00:00:00',95.45,437,644,97.79,'Sun-Synch' from dual union all
select 27185,'027185','2001-049ED','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-30 00:00:00','2002-01-16 00:00:00',95.19,410,645,97.75,'Sun-Synch' from dual union all
select 27186,'027186','2001-049EE','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-13 00:00:00','2002-01-17 00:00:00',96.71,548,655,97.73,'Sun-Synch' from dual union all
select 27187,'027187','2001-049EF','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-29 00:00:00','2002-01-16 00:00:00',96.26,510,649,97.81,'Sun-Synch' from dual union all
select 27188,'027188','2001-049EG','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-18 00:00:00','2002-01-16 00:00:00',96.83,562,652,97.8,'Sun-Synch' from dual;
insert into SATELLITE
select 27189,'027189','2001-049EH','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-26 00:00:00','2002-01-16 00:00:00',96.09,502,640,97.77,'Sun-Synch' from dual union all
select 2719,'002719','1967-026B','Delta 47','DSV-3E-3 S/N 20211',15281,'Reentered','1967-07-07 00:00:00','1967-04-22 00:00:00',92.36,241,540,28.77,'Intermediate' from dual union all
select 27190,'027190','2001-049EJ','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-18 00:00:00','2002-01-17 00:00:00',96.03,488,648,97.88,'Sun-Synch' from dual union all
select 27191,'027191','2001-049EK','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-17 00:00:00','2002-01-17 00:00:00',96.39,526,646,97.75,'Sun-Synch' from dual union all
select 27192,'027192','2001-049EL','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-18 00:00:00','2002-01-17 00:00:00',96.64,542,654,97.82,'Sun-Synch' from dual union all
select 27193,'027193','2001-049EM','deb PSLV-C3 PS4','',67202,'Reentered','2002-08-07 00:00:00','2002-01-16 00:00:00',96.83,549,665,97.8,'Sun-Synch' from dual union all
select 27194,'027194','2001-049EN','deb PSLV-C3 PS4','',67202,'Reentered','2002-06-27 00:00:00','2002-01-16 00:00:00',96.31,510,653,97.74,'Sun-Synch' from dual union all
select 27195,'027195','2001-049EP','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-23 00:00:00','2002-01-17 00:00:00',94.73,403,608,98.42,'Sun-Synch' from dual union all
select 27196,'027196','2001-049EQ','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-19 00:00:00','2002-01-17 00:00:00',94.93,378,653,97.83,'Sun-Synch' from dual union all
select 27197,'027197','2001-049ER','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-06 00:00:00','2002-01-16 00:00:00',93.82,319,604,97.88,'Sun-Synch' from dual union all
select 27198,'027198','2001-049ES','deb PSLV-C3 PS4','',67202,'Reentered','2002-09-05 00:00:00','2002-01-16 00:00:00',96.43,508,668,97.79,'Sun-Synch' from dual union all
select 27199,'027199','2001-049ET','deb PSLV-C3 PS4','',67202,'Reentered','2002-01-19 00:00:00','2002-01-16 00:00:00',91,254,394,98.19,'Sun-Synch' from dual union all
select 272,'000272','62 Kappa 2','Midas 5 Fairing','',8192,'Reentered','1962-05-04 00:00:00','1962-04-11 00:00:00',114.07,98,2729,86.64,'Polar' from dual union all
select 2720,'002720','1967-027A','Kosmos-151','Strela-2',15282,'Reentered','1991-05-06 00:00:00','1967-04-29 00:00:00',97.13,596,647,56.06,'Intermediate' from dual union all
select 27200,'027200','2001-049EU','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-11 00:00:00','2002-01-17 00:00:00',93.73,308,606,98.7,'Sun-Synch' from dual union all
select 27201,'027201','2001-049EV','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-14 00:00:00','2002-01-17 00:00:00',94.24,301,663,98.29,'Sun-Synch' from dual union all
select 27202,'027202','2001-049EW','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-15 00:00:00','2002-01-17 00:00:00',95.57,353,740,96.5,'Sun-Synch' from dual union all
select 27203,'027203','2001-049EX','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-04 00:00:00','2002-01-16 00:00:00',96.02,488,647,97.7,'Sun-Synch' from dual union all
select 27204,'027204','2001-049EY','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-18 00:00:00','2002-01-17 00:00:00',95.85,467,653,98.53,'Sun-Synch' from dual union all
select 27205,'027205','2001-049EZ','deb PSLV-C3 PS4','',67202,'Reentered','2002-09-27 00:00:00','2002-01-17 00:00:00',96.13,452,695,97.54,'Sun-Synch' from dual union all
select 27206,'027206','2001-049FA','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-06 00:00:00','2002-01-16 00:00:00',96,504,630,97.81,'Sun-Synch' from dual union all
select 27207,'027207','2001-049FB','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-30 00:00:00','2002-01-17 00:00:00',96.74,560,646,97.81,'Sun-Synch' from dual union all
select 27208,'027208','2001-049FC','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-30 00:00:00','2002-01-17 00:00:00',96.1,498,646,97.72,'Sun-Synch' from dual union all
select 27209,'027209','2001-049FD','deb PSLV-C3 PS4','',67202,'Reentered','2002-01-25 00:00:00','2002-01-16 00:00:00',92.91,332,503,97.78,'Sun-Synch' from dual union all
select 2721,'002721','1967-027B','S3','S3',15282,'Reentered','1988-07-03 00:00:00','1967-04-27 00:00:00',97.15,594,651,56.06,'Intermediate' from dual union all
select 27210,'027210','2001-049FE','deb PSLV-C3 PS4','',67202,'Reentered','2002-08-14 00:00:00','2002-01-17 00:00:00',96.43,524,652,97.77,'Sun-Synch' from dual union all
select 27211,'027211','2001-049FF','deb PSLV-C3 PS4','',67202,'Reentered','2002-09-29 00:00:00','2002-01-16 00:00:00',95.81,461,655,97.8,'Sun-Synch' from dual union all
select 27212,'027212','2001-049FG','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-03 00:00:00','2002-01-17 00:00:00',95.94,513,614,97.79,'Sun-Synch' from dual union all
select 27213,'027213','2001-049FH','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-18 00:00:00','2002-01-16 00:00:00',97.15,564,681,97.79,'Sun-Synch' from dual union all
select 27214,'027214','2001-049FJ','deb PSLV-C3 PS4','',67202,'Reentered','2002-01-29 00:00:00','2002-01-22 00:00:00',94.48,457,530,97.78,'Sun-Synch' from dual union all
select 27215,'027215','2001-049FK','deb PSLV-C3 PS4','',67202,'Reentered','2002-11-22 00:00:00','2002-01-18 00:00:00',97.95,634,688,97.4,'Sun-Synch' from dual union all
select 27216,'027216','2001-049FL','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-21 00:00:00',98.01,629,698,98.15,'Sun-Synch' from dual union all
select 27217,'027217','2001-049FM','deb PSLV-C3 PS4','',67202,'Reentered','2004-04-23 00:00:00','2002-01-22 00:00:00',96.79,546,664,97.73,'Sun-Synch' from dual union all
select 27218,'027218','2001-049FN','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-21 00:00:00',101.79,531,1155,99.04,'Sun-Synch' from dual union all
select 27219,'027219','2001-049FP','deb PSLV-C3 PS4','',67202,'Reentered','2002-08-28 00:00:00','2002-01-22 00:00:00',97.07,582,655,97.85,'Sun-Synch' from dual union all
select 2722,'002722','1967-028A','Kosmos-152','DS-P1-Yu No. 7',15283,'Reentered','1967-08-05 00:00:00','1967-04-24 00:00:00',91.62,263,444,70.98,'Intermediate' from dual union all
select 27220,'027220','2001-049FQ','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-08 00:00:00','2002-01-19 00:00:00',95.33,480,590,97.77,'Sun-Synch' from dual union all
select 27221,'027221','2001-049FR','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-16 00:00:00','2002-01-20 00:00:00',96.52,540,643,97.73,'Sun-Synch' from dual union all
select 27222,'027222','2001-049FS','deb PSLV-C3 PS4','',67202,'Reentered','2012-05-29 00:00:00','2002-01-21 00:00:00',97.09,566,673,97.6,'Sun-Synch' from dual union all
select 27223,'027223','2001-049FT','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-15 00:00:00','2002-01-21 00:00:00',99.26,637,809,97.18,'Sun-Synch' from dual union all
select 27224,'027224','2001-049FU','deb PSLV-C3 PS4','',67202,'Reentered','2002-11-13 00:00:00','2002-01-20 00:00:00',96.63,534,660,97.26,'Sun-Synch' from dual union all
select 27225,'027225','2001-049FV','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-27 00:00:00','2002-01-22 00:00:00',96.7,511,690,97.42,'Sun-Synch' from dual union all
select 27226,'027226','2001-049FW','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-18 00:00:00',98.1,639,697,97.98,'Sun-Synch' from dual union all
select 27227,'027227','2001-049FX','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-05 00:00:00','2002-01-22 00:00:00',95.54,448,642,98.27,'Sun-Synch' from dual union all
select 27228,'027228','2001-049FY','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-27 00:00:00','2002-01-21 00:00:00',94.09,354,595,97.79,'Sun-Synch' from dual union all
select 27229,'027229','2001-049FZ','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-08 00:00:00','2002-01-20 00:00:00',94.98,395,641,97.63,'Sun-Synch' from dual union all
select 2723,'002723','1967-028B','S1M','S1M',15283,'Reentered','1967-05-22 00:00:00','1967-04-03 00:00:00',91.72,270,448,70.99,'Intermediate' from dual union all
select 27230,'027230','2001-049GA','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-26 00:00:00','2002-01-21 00:00:00',96.26,515,644,97.78,'Sun-Synch' from dual union all
select 27231,'027231','2001-049GB','deb PSLV-C3 PS4','',67202,'Reentered','2008-02-13 00:00:00','2002-01-21 00:00:00',96.89,553,667,97.8,'Sun-Synch' from dual union all
select 27232,'027232','2001-049GC','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-19 00:00:00','2002-01-20 00:00:00',95.76,504,607,97.84,'Sun-Synch' from dual union all
select 27233,'027233','2001-049GD','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-03 00:00:00','2002-01-21 00:00:00',95.71,490,616,97.81,'Sun-Synch' from dual union all
select 27234,'027234','2001-049GE','deb PSLV-C3 PS4','',67202,'Reentered','2004-05-07 00:00:00','2002-01-21 00:00:00',96.15,487,662,98.57,'Sun-Synch' from dual union all
select 27235,'027235','2001-049GF','deb PSLV-C3 PS4','',67202,'Reentered','2002-12-12 00:00:00','2002-01-20 00:00:00',96.55,521,666,97.88,'Sun-Synch' from dual union all
select 27236,'027236','2001-049GG','deb PSLV-C3 PS4','',67202,'Reentered','2002-09-18 00:00:00','2002-01-20 00:00:00',96.74,545,660,97.71,'Sun-Synch' from dual union all
select 27237,'027237','2001-049GH','deb PSLV-C3 PS4','',67202,'Reentered','2002-06-17 00:00:00','2002-01-22 00:00:00',96.63,541,654,97.84,'Sun-Synch' from dual union all
select 27238,'027238','2001-049GJ','deb PSLV-C3 PS4','',67202,'Reentered','2002-09-21 00:00:00','2002-01-20 00:00:00',97.78,621,684,97.79,'Sun-Synch' from dual union all
select 27239,'027239','2001-049GK','deb PSLV-C3 PS4','',67202,'Reentered','2004-08-12 00:00:00','2002-01-20 00:00:00',99.45,660,805,97.48,'Sun-Synch' from dual union all
select 2724,'002724','1967-024C','deb S1M','',15279,'Reentered','1967-03-26 00:00:00','1967-03-21 00:00:00',89.03,201,253,48.45,'Intermediate' from dual union all
select 27240,'027240','2001-049GL','deb PSLV-C3 PS4','',67202,'Reentered','2003-04-01 00:00:00','2002-01-21 00:00:00',96.26,515,644,97.78,'Sun-Synch' from dual union all
select 27241,'027241','2001-049GM','deb PSLV-C3 PS4','',67202,'Reentered','2002-09-11 00:00:00','2002-01-20 00:00:00',96.16,496,654,98.6,'Sun-Synch' from dual union all
select 27242,'027242','2001-049GN','deb PSLV-C3 PS4','',67202,'Reentered','2003-07-23 00:00:00','2002-01-21 00:00:00',101.68,659,1017,98.05,'Sun-Synch' from dual union all
select 27243,'027243','2001-049GP','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-20 00:00:00','2002-01-20 00:00:00',95.81,499,617,97.82,'Sun-Synch' from dual union all
select 27244,'027244','2001-049GQ','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-03 00:00:00','2002-01-22 00:00:00',95.55,466,625,97.78,'Sun-Synch' from dual union all
select 27245,'027245','2001-049GR','deb PSLV-C3 PS4','',67202,'Reentered','2002-06-29 00:00:00','2002-01-21 00:00:00',96.83,564,650,97.93,'Sun-Synch' from dual union all
select 27246,'027246','2001-049GS','deb PSLV-C3 PS4','',67202,'Reentered','2003-05-03 00:00:00','2002-01-22 00:00:00',97.12,582,660,97.7,'Sun-Synch' from dual union all
select 27247,'027247','2001-049GT','deb PSLV-C3 PS4','',67202,'Reentered','2002-08-14 00:00:00','2002-01-20 00:00:00',97.37,607,659,97.86,'Sun-Synch' from dual union all
select 27248,'027248','2001-049GU','deb PSLV-C3 PS4','',67202,'Reentered','2002-09-22 00:00:00','2002-01-22 00:00:00',97.83,624,686,98.03,'Sun-Synch' from dual union all
select 27249,'027249','2001-049GV','deb PSLV-C3 PS4','',67202,'Reentered','2003-02-25 00:00:00','2002-01-19 00:00:00',98.32,647,710,97.78,'Sun-Synch' from dual union all
select 2725,'002725','1967-024D','deb S1M','',15279,'Reentered','1967-03-26 00:00:00','1967-03-21 00:00:00',88.5,156,244,48.45,'Intermediate' from dual union all
select 27250,'027250','2001-049GW','deb PSLV-C3 PS4','',67202,'Reentered','2006-02-24 00:00:00','2002-01-18 00:00:00',99.31,646,806,97.77,'Sun-Synch' from dual union all
select 27251,'027251','2001-049GX','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-19 00:00:00','2002-01-22 00:00:00',96.93,576,648,97.74,'Sun-Synch' from dual union all
select 27252,'027252','2001-049GY','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-01 00:00:00','2002-01-22 00:00:00',96.85,567,649,97.72,'Sun-Synch' from dual union all
select 27253,'027253','2001-049GZ','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-15 00:00:00','2002-01-21 00:00:00',96.8,574,637,97.81,'Sun-Synch' from dual union all
select 27254,'027254','2001-049HA','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-13 00:00:00','2002-01-19 00:00:00',97.8,636,671,97.73,'Sun-Synch' from dual union all
select 27255,'027255','2001-049HB','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-21 00:00:00',98.21,627,720,98.07,'Sun-Synch' from dual union all
select 27256,'027256','2001-049HC','deb PSLV-C3 PS4','',67202,'Reentered','2002-01-30 00:00:00','2002-01-21 00:00:00',94.69,436,572,97.65,'Sun-Synch' from dual union all
select 27257,'027257','2001-049HD','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-06 00:00:00','2002-01-18 00:00:00',95.74,506,603,96.39,'Sun-Synch' from dual union all
select 27258,'027258','2001-049HE','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-16 00:00:00','2002-01-19 00:00:00',96.86,570,647,97.63,'Sun-Synch' from dual union all
select 27259,'027259','2001-049HF','deb PSLV-C3 PS4','',67202,'Reentered','2004-02-13 00:00:00','2002-01-20 00:00:00',98.7,664,729,97.2,'Sun-Synch' from dual union all
select 2726,'002726','1967-024E','deb S1M','',15279,'Reentered','1967-03-27 00:00:00','1967-03-24 00:00:00',89.66,245,271,48.4,'Intermediate' from dual union all
select 27260,'027260','2001-049HG','deb PSLV-C3 PS4','',67202,'Reentered','2003-03-20 00:00:00','2002-01-18 00:00:00',98.78,626,774,97.79,'Sun-Synch' from dual union all
select 27261,'027261','2001-049HH','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-14 00:00:00','2002-01-21 00:00:00',93.19,305,557,98.21,'Sun-Synch' from dual union all
select 27262,'027262','2001-049HJ','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-20 00:00:00','2002-01-21 00:00:00',95.16,449,603,97.71,'Sun-Synch' from dual union all
select 27263,'027263','2001-049HK','deb PSLV-C3 PS4','',67202,'Reentered','2002-11-10 00:00:00','2002-01-20 00:00:00',96.61,533,660,97.8,'Sun-Synch' from dual union all
select 27264,'027264','2001-049HL','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-18 00:00:00','2002-01-20 00:00:00',95.1,441,606,97.76,'Sun-Synch' from dual union all
select 27265,'027265','2001-049HM','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-16 00:00:00','2002-01-22 00:00:00',96.72,559,644,97.86,'Sun-Synch' from dual union all
select 27266,'027266','2001-049HN','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-15 00:00:00','2002-01-21 00:00:00',97.42,614,656,97.76,'Sun-Synch' from dual union all
select 27267,'027267','2001-049HP','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-21 00:00:00',98.35,630,730,98.01,'Sun-Synch' from dual union all
select 27268,'027268','2001-049HQ','deb PSLV-C3 PS4','',67202,'Reentered','2002-08-25 00:00:00','2002-01-23 00:00:00',96.26,512,647,97.72,'Sun-Synch' from dual union all
select 27269,'027269','2001-049HR','deb PSLV-C3 PS4','',67202,'Reentered','2002-10-02 00:00:00','2002-01-23 00:00:00',97.22,591,661,97.86,'Sun-Synch' from dual union all
select 2727,'002727','1967-024F','deb S1M','',15279,'Reentered','1967-03-27 00:00:00','1967-03-25 00:00:00',88.81,216,216,48.4,'Intermediate' from dual union all
select 27270,'027270','2001-049HS','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-02 00:00:00','2002-01-23 00:00:00',96.71,559,644,97.85,'Sun-Synch' from dual union all
select 27271,'027271','2001-049HT','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-20 00:00:00','2002-01-24 00:00:00',96.39,526,646,97.82,'Sun-Synch' from dual union all
select 27272,'027272','2001-049HU','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-17 00:00:00','2002-01-24 00:00:00',97.29,600,658,98.36,'Sun-Synch' from dual union all
select 27273,'027273','2001-049HV','deb PSLV-C3 PS4','',67202,'Reentered','2015-01-06 00:00:00','2002-01-23 00:00:00',97.63,622,669,97.79,'Sun-Synch' from dual union all
select 27274,'027274','2001-049HW','deb PSLV-C3 PS4','',67202,'Reentered','2002-01-27 00:00:00','2002-01-24 00:00:00',92.58,355,447,97.6,'Sun-Synch' from dual union all
select 27275,'027275','2001-049HX','deb PSLV-C3 PS4','',67202,'Reentered','2002-10-22 00:00:00','2002-01-23 00:00:00',97.8,619,687,97.69,'Sun-Synch' from dual union all
select 27276,'027276','2001-049HY','deb PSLV-C3 PS4','',67202,'Reentered','2002-01-26 00:00:00','2002-01-24 00:00:00',91.28,306,369,98.68,'Sun-Synch' from dual union all
select 27277,'027277','2001-049HZ','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-06 00:00:00','2002-01-24 00:00:00',94.07,406,541,97.69,'Sun-Synch' from dual union all
select 27278,'027278','2001-049JA','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-06 00:00:00','2002-01-24 00:00:00',94.71,463,547,97.77,'Sun-Synch' from dual;
insert into SATELLITE
select 27279,'027279','2001-049JB','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-29 00:00:00','2002-01-23 00:00:00',96.42,544,631,97.72,'Sun-Synch' from dual union all
select 2728,'002728','1967-024G','deb S1M','',15279,'Reentered','1967-03-27 00:00:00','1967-03-24 00:00:00',89.53,243,260,48.4,'Intermediate' from dual union all
select 27280,'027280','2001-049JC','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-23 00:00:00','2002-01-24 00:00:00',94.58,412,585,98.76,'Sun-Synch' from dual union all
select 27281,'027281','2001-049JD','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-04 00:00:00','2002-01-23 00:00:00',96.43,524,652,97.85,'Sun-Synch' from dual union all
select 27282,'027282','2001-049JE','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-07 00:00:00','2002-01-24 00:00:00',93.84,381,544,97.66,'Sun-Synch' from dual union all
select 27283,'027283','2001-049JF','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-25 00:00:00','2002-01-22 00:00:00',96.41,540,634,97.68,'Sun-Synch' from dual union all
select 27284,'027284','2001-049JG','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-08 00:00:00','2002-01-23 00:00:00',95.74,439,670,97.95,'Sun-Synch' from dual union all
select 27285,'027285','2001-049JH','deb PSLV-C3 PS4','',67202,'Reentered','2002-06-09 00:00:00','2002-01-23 00:00:00',96.71,550,653,97.8,'Sun-Synch' from dual union all
select 27286,'027286','2001-049JJ','deb PSLV-C3 PS4','',67202,'Reentered','2014-12-04 00:00:00','2002-01-23 00:00:00',97.14,572,671,97.77,'Sun-Synch' from dual union all
select 27287,'027287','2001-049JK','deb PSLV-C3 PS4','',67202,'Reentered','2002-08-09 00:00:00','2002-01-23 00:00:00',96.66,545,653,97.79,'Sun-Synch' from dual union all
select 27288,'027288','2001-049JL','deb PSLV-C3 PS4','',67202,'Reentered','2002-01-29 00:00:00','2002-01-23 00:00:00',94.36,451,525,97.78,'Sun-Synch' from dual union all
select 27289,'027289','2001-049JM','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-25 00:00:00','2002-01-24 00:00:00',96.56,537,651,97.77,'Sun-Synch' from dual union all
select 2729,'002729','1967-024H','deb S1M','',15279,'Reentered','1967-04-12 00:00:00','1967-03-22 00:00:00',91.13,249,410,48.25,'Intermediate' from dual union all
select 27290,'027290','2001-049JN','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-19 00:00:00','2002-01-23 00:00:00',96.03,511,626,97.8,'Sun-Synch' from dual union all
select 27291,'027291','2001-049JP','deb PSLV-C3 PS4','',67202,'Reentered','2013-05-13 00:00:00','2002-01-23 00:00:00',96.51,516,667,97.8,'Sun-Synch' from dual union all
select 27292,'027292','2001-049JQ','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-05 00:00:00','2002-01-24 00:00:00',95.95,512,617,97.84,'Sun-Synch' from dual union all
select 27293,'027293','2001-049JR','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-23 00:00:00','2002-01-23 00:00:00',95.89,516,608,97.79,'Sun-Synch' from dual union all
select 27294,'027294','2001-049JS','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-30 00:00:00','2002-01-24 00:00:00',96.9,577,644,97.82,'Sun-Synch' from dual union all
select 27295,'027295','2001-049JT','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-23 00:00:00','2002-01-23 00:00:00',96.6,542,650,97.79,'Sun-Synch' from dual union all
select 27296,'027296','2001-049JU','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-09 00:00:00','2002-01-23 00:00:00',96.04,505,633,97.76,'Sun-Synch' from dual union all
select 27297,'027297','2001-049JV','deb PSLV-C3 PS4','',67202,'Reentered','2007-01-10 00:00:00','2002-01-23 00:00:00',96.51,519,663,97.72,'Sun-Synch' from dual union all
select 27298,'027298','2002-002A','Insat 3C','Insat 3C',67378,'In Earth orbit','','2003-02-15 00:00:00',1436.06,35777,35795,.1,'Stationary' from dual union all
select 27299,'027299','2002-002B','Ariane H10-3 V147','Ariane H10-3 S/N T4108',67378,'In Earth orbit','','2002-01-25 00:00:00',656.18,425,36846,3.98,'GEO Transfer' from dual union all
select 273,'000273','62 Kappa 3','Midas V solar array cover','Strongback',8192,'In Earth orbit','','1962-06-07 00:00:00',152.59,2784,3378,86.65,'Medium' from dual union all
select 2730,'002730','1967-024J','deb S1M','',15279,'Reentered','1967-04-03 00:00:00','1967-03-23 00:00:00',90.92,247,392,48.3,'Intermediate' from dual union all
select 27300,'027300','2001-049JW','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-19 00:00:00','2002-01-24 00:00:00',95.39,481,594,97.85,'Sun-Synch' from dual union all
select 27301,'027301','2001-049JX','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-06 00:00:00','2002-01-24 00:00:00',93.12,340,515,98.95,'Sun-Synch' from dual union all
select 27302,'027302','2001-049JY','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-26 00:00:00','2002-01-24 00:00:00',96.87,569,648,97.86,'Sun-Synch' from dual union all
select 27303,'027303','2001-049JZ','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-13 00:00:00','2002-01-23 00:00:00',96.19,514,638,97.77,'Sun-Synch' from dual union all
select 27304,'027304','2001-049KA','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-25 00:00:00','2002-01-23 00:00:00',96.17,522,629,97.82,'Sun-Synch' from dual union all
select 27305,'027305','2001-049KB','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-22 00:00:00',98.44,647,722,97.98,'Sun-Synch' from dual union all
select 27306,'027306','2001-049KC','deb PSLV-C3 PS4','',67202,'Reentered','2003-12-21 00:00:00','2002-01-23 00:00:00',96.43,516,659,97.78,'Sun-Synch' from dual union all
select 27307,'027307','2001-049KD','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-27 00:00:00','2002-01-24 00:00:00',96.24,524,633,97.78,'Sun-Synch' from dual union all
select 27308,'027308','2001-049KE','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-21 00:00:00','2002-01-23 00:00:00',96.17,519,632,97.81,'Sun-Synch' from dual union all
select 27309,'027309','2001-049KF','deb PSLV-C3 PS4','',67202,'Reentered','2002-11-17 00:00:00','2002-01-23 00:00:00',95.76,453,657,97.79,'Sun-Synch' from dual union all
select 2731,'002731','1967-024K','deb S1M','',15279,'Reentered','1967-04-03 00:00:00','1967-03-22 00:00:00',90.45,252,342,48.33,'Intermediate' from dual union all
select 27310,'027310','2001-049KG','deb PSLV-C3 PS4','',67202,'Reentered','2002-06-19 00:00:00','2002-01-23 00:00:00',95.89,463,661,97.76,'Sun-Synch' from dual union all
select 27311,'027311','2001-049KH','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-18 00:00:00','2002-01-23 00:00:00',95.9,502,623,97.74,'Sun-Synch' from dual union all
select 27312,'027312','2001-049KJ','deb PSLV-C3 PS4','',67202,'Reentered','2002-01-23 00:00:00','2002-01-23 00:00:00',90.06,248,306,98.71,'Sun-Synch' from dual union all
select 27313,'027313','2001-049KK','deb PSLV-C3 PS4','',67202,'Reentered','2002-01-23 00:00:00','2002-01-23 00:00:00',91.06,293,361,98.78,'Sun-Synch' from dual union all
select 27314,'027314','2001-049KL','deb PSLV-C3 PS4','',67202,'Reentered','2002-01-30 00:00:00','2002-01-23 00:00:00',93.08,364,487,98.76,'Sun-Synch' from dual union all
select 27315,'027315','2001-049KM','deb PSLV-C3 PS4','',67202,'Reentered','2004-08-18 00:00:00','2002-01-23 00:00:00',97.77,639,665,97.8,'Sun-Synch' from dual union all
select 27316,'027316','2001-049KN','deb PSLV-C3 PS4','',67202,'Reentered','2002-11-30 00:00:00','2002-01-23 00:00:00',97,568,662,97.9,'Sun-Synch' from dual union all
select 27317,'027317','2001-049KP','deb PSLV-C3 PS4','',67202,'Reentered','2015-04-20 00:00:00','2002-01-23 00:00:00',97.37,592,674,97.62,'Sun-Synch' from dual union all
select 27318,'027318','2001-049KQ','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-05 00:00:00','2002-01-23 00:00:00',95.58,478,616,97.75,'Sun-Synch' from dual union all
select 27319,'027319','2001-049KR','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-22 00:00:00','2002-01-23 00:00:00',96.26,522,637,97.81,'Sun-Synch' from dual union all
select 2732,'002732','1967-024L','deb S1M','',15279,'Reentered','1967-04-09 00:00:00','1967-03-29 00:00:00',90.09,253,305,48.3,'Intermediate' from dual union all
select 27320,'027320','2001-049KS','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-24 00:00:00','2002-01-23 00:00:00',96.08,512,629,97.78,'Sun-Synch' from dual union all
select 27321,'027321','2001-049KT','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-24 00:00:00','2002-01-23 00:00:00',95.56,486,605,97.78,'Sun-Synch' from dual union all
select 27322,'027322','2001-049KU','deb PSLV-C3 PS4','',67202,'Reentered','2006-10-06 00:00:00','2002-01-23 00:00:00',96.6,527,665,97.72,'Sun-Synch' from dual union all
select 27323,'027323','2001-049KV','deb PSLV-C3 PS4','',67202,'Reentered','2002-06-22 00:00:00','2002-01-23 00:00:00',96.73,550,655,97.83,'Sun-Synch' from dual union all
select 27324,'027324','2001-049KW','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-26 00:00:00','2002-01-24 00:00:00',96.81,570,642,97.83,'Sun-Synch' from dual union all
select 27325,'027325','2001-049KX','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-04 00:00:00','2002-01-23 00:00:00',93.55,365,531,97.65,'Sun-Synch' from dual union all
select 27326,'027326','2001-049KY','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-15 00:00:00','2002-01-23 00:00:00',94.68,435,572,97.68,'Sun-Synch' from dual union all
select 27327,'027327','2001-049KZ','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-06 00:00:00','2002-01-23 00:00:00',96.65,563,634,97.78,'Sun-Synch' from dual union all
select 27328,'027328','1998-067Q','EVA decontam towel','',66503,'Reentered','2002-02-06 00:00:00','2002-02-01 00:00:00',91.43,340,349,51.63,'Intermediate' from dual union all
select 27329,'027329','2001-049LA','deb PSLV-C3 PS4','',67202,'Reentered','2002-10-02 00:00:00','2002-02-01 00:00:00',96.37,523,647,97.78,'Sun-Synch' from dual union all
select 2733,'002733','1967-024M','deb S1M','',15279,'Reentered','1967-03-28 00:00:00','1967-03-25 00:00:00',88.93,222,222,48.25,'Intermediate' from dual union all
select 27330,'027330','2001-049LB','deb PSLV-C3 PS4','',67202,'Reentered','2005-06-20 00:00:00','2002-01-29 00:00:00',96.94,561,664,97.78,'Sun-Synch' from dual union all
select 27331,'027331','2001-049LC','deb PSLV-C3 PS4','',67202,'Reentered','2002-12-11 00:00:00','2002-01-31 00:00:00',96.42,518,657,97.72,'Sun-Synch' from dual union all
select 27332,'027332','2001-049LD','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-07 00:00:00','2002-01-30 00:00:00',96.36,533,636,97.79,'Sun-Synch' from dual union all
select 27333,'027333','2001-049LE','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-30 00:00:00','2002-01-30 00:00:00',95.19,417,639,97.91,'Sun-Synch' from dual union all
select 27334,'027334','2001-049LF','deb PSLV-C3 PS4','',67202,'Reentered','2011-11-10 00:00:00','2002-02-01 00:00:00',98.3,662,693,98.05,'Sun-Synch' from dual union all
select 27335,'027335','2001-049LG','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-04 00:00:00','2002-02-01 00:00:00',92.48,372,421,97.77,'Sun-Synch' from dual union all
select 27336,'027336','2001-049LH','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-03 00:00:00','2002-02-01 00:00:00',94.12,388,564,98.71,'Sun-Synch' from dual union all
select 27337,'027337','2001-049LJ','deb PSLV-C3 PS4','',67202,'Reentered','2002-10-01 00:00:00','2002-01-31 00:00:00',97.36,617,648,97.81,'Sun-Synch' from dual union all
select 27338,'027338','2001-049LK','deb PSLV-C3 PS4','',67202,'Reentered','2003-11-10 00:00:00','2002-01-31 00:00:00',96.69,517,683,97.69,'Sun-Synch' from dual union all
select 27339,'027339','2001-049LL','deb PSLV-C3 PS4','',67202,'Reentered','2002-01-30 00:00:00','2002-01-25 00:00:00',93.19,389,473,97.64,'Sun-Synch' from dual union all
select 2734,'002734','1967-024N','deb S1M','',15279,'Reentered','1967-03-28 00:00:00','1967-03-26 00:00:00',88.11,88,274,48.54,'Intermediate' from dual union all
select 27340,'027340','2001-049LM','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-09 00:00:00','2002-02-01 00:00:00',93.67,414,494,97.78,'Sun-Synch' from dual union all
select 27341,'027341','2001-049LN','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-19 00:00:00','2002-01-31 00:00:00',95.11,485,563,97.78,'Sun-Synch' from dual union all
select 27342,'027342','2001-049LP','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-22 00:00:00','2002-02-01 00:00:00',96.78,568,641,97.82,'Sun-Synch' from dual union all
select 27343,'027343','2001-049LQ','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-24 00:00:00','2002-01-27 00:00:00',95.51,490,597,97.8,'Sun-Synch' from dual union all
select 27344,'027344','2001-049LR','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-10 00:00:00','2002-02-01 00:00:00',93.98,426,513,97.76,'Sun-Synch' from dual union all
select 27345,'027345','2001-049LS','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-09 00:00:00','2002-02-01 00:00:00',95.48,491,593,97.65,'Sun-Synch' from dual union all
select 27346,'027346','2001-049LT','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-30 00:00:00','2002-02-01 00:00:00',96.5,553,629,97.81,'Sun-Synch' from dual union all
select 27347,'027347','2001-049LU','deb PSLV-C3 PS4','',67202,'Reentered','2013-11-26 00:00:00','2002-02-01 00:00:00',97.15,572,673,97.77,'Sun-Synch' from dual union all
select 27348,'027348','2001-049LV','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-17 00:00:00','2002-02-01 00:00:00',96.14,526,622,97.82,'Sun-Synch' from dual union all
select 27349,'027349','2001-049LW','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-09 00:00:00','2002-01-31 00:00:00',94.21,379,582,98.74,'Sun-Synch' from dual union all
select 2735,'002735','1965-080E','Blok-L No. U103-37','Blok-L',11830,'Reentered','1967-01-18 00:00:00','1965-10-14 00:00:00',718.69,480,39920,65.2,'GEO Transfer' from dual union all
select 27350,'027350','2001-049LX','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-08 00:00:00','2002-02-01 00:00:00',93.65,420,487,97.76,'Sun-Synch' from dual union all
select 27351,'027351','2001-049LY','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-19 00:00:00','2002-01-31 00:00:00',96.58,560,629,97.83,'Sun-Synch' from dual union all
select 27352,'027352','2001-049LZ','deb PSLV-C3 PS4','',67202,'Reentered','2002-10-29 00:00:00','2002-01-29 00:00:00',98.04,640,690,97.99,'Sun-Synch' from dual union all
select 27353,'027353','2001-049MA','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-01-29 00:00:00',97.88,616,699,97.97,'Sun-Synch' from dual union all
select 27354,'027354','2001-049MB','deb PSLV-C3 PS4','',67202,'Reentered','2002-08-13 00:00:00','2002-01-29 00:00:00',97.09,584,655,97.84,'Sun-Synch' from dual union all
select 27355,'027355','2001-049MC','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-01 00:00:00','2002-02-01 00:00:00',94.77,412,603,98.75,'Sun-Synch' from dual union all
select 27356,'027356','2001-049MD','deb PSLV-C3 PS4','',67202,'Reentered','2002-01-30 00:00:00','2002-01-30 00:00:00',90.08,236,321,98.75,'Sun-Synch' from dual union all
select 27357,'027357','2001-049ME','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-06 00:00:00','2002-01-31 00:00:00',93.61,421,482,97.72,'Sun-Synch' from dual union all
select 27358,'027358','2001-049MF','deb PSLV-C3 PS4','',67202,'Reentered','2002-05-01 00:00:00','2002-02-01 00:00:00',96.7,570,632,97.85,'Sun-Synch' from dual union all
select 27359,'027359','2001-049MG','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-13 00:00:00','2002-02-01 00:00:00',94.59,468,530,97.84,'Sun-Synch' from dual union all
select 2736,'002736','1967-029A','OPS 4779','CORONA J-35',15284,'Reentered Attached','1967-04-17 00:00:00','1967-04-07 00:00:00',89.98,181,366,85.07,'Polar' from dual union all
select 27360,'027360','2001-049MH','deb PSLV-C3 PS4','',67202,'Reentered','2002-06-22 00:00:00','2002-02-01 00:00:00',96.61,551,641,97.79,'Sun-Synch' from dual union all
select 27361,'027361','2001-049MJ','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-21 00:00:00','2002-02-01 00:00:00',96.14,521,627,97.79,'Sun-Synch' from dual union all
select 27362,'027362','2001-049MK','deb PSLV-C3 PS4','',67202,'Reentered','2002-04-20 00:00:00','2002-01-31 00:00:00',96.69,569,632,97.83,'Sun-Synch' from dual union all
select 27363,'027363','2001-049ML','deb PSLV-C3 PS4','',67202,'Reentered','2004-01-14 00:00:00','2002-02-01 00:00:00',96.45,503,675,97.84,'Sun-Synch' from dual union all
select 27364,'027364','2001-049MM','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-17 00:00:00','2002-01-31 00:00:00',95.16,491,562,97.78,'Sun-Synch' from dual union all
select 27365,'027365','2001-049MN','deb PSLV-C3 PS4','',67202,'Reentered','2002-03-25 00:00:00','2002-02-01 00:00:00',95.96,518,611,97.78,'Sun-Synch' from dual union all
select 27366,'027366','2001-049MP','deb PSLV-C3 PS4','',67202,'Reentered','2002-02-20 00:00:00','2002-01-31 00:00:00',94.91,465,564,97.83,'Sun-Synch' from dual union all
select 27367,'027367','2002-003A','Tsubasa','MDS-1',67379,'In Earth orbit','','2002-02-04 00:00:00',635.49,480,35734,28.55,'GEO Transfer' from dual union all
select 27368,'027368','2002-003B','VEP-3','VEP-3/Upper Adapter',67379,'In Earth orbit','','2002-02-04 00:00:00',634.71,483,35691,28.55,'GEO Transfer' from dual;
insert into SATELLITE
select 27369,'027369','2002-003C','H-2A F2 Stage 2','H-2A Stage 2',67379,'In Earth orbit','','2002-02-04 00:00:00',635.86,481,35752,28.54,'GEO Transfer' from dual union all
select 2737,'002737','1967-024P','deb S1M','',15279,'Reentered','1967-03-27 00:00:00','1967-03-31 00:00:00',90.01,270,280,48.2,'Intermediate' from dual union all
select 27370,'027370','2002-004A','RHESSI','HESSI',67380,'In Earth orbit','','2002-02-05 00:00:00',96.65,588,609,38.04,'Intermediate' from dual union all
select 27371,'027371','2002-004B','Orion 38','Orion 38',67380,'In Earth orbit','','2002-02-05 00:00:00',96.6,583,608,38.04,'Intermediate' from dual union all
select 27372,'027372','2002-005A','Iridium 91','Iridium SV091',67381,'In Earth orbit','','2002-02-18 00:00:00',97.44,629,643,86.57,'Polar' from dual union all
select 27373,'027373','2002-005B','Iridium 90','Iridium SV090',67381,'In Earth orbit','','2002-02-13 00:00:00',97.4,627,642,86.57,'Polar' from dual union all
select 27374,'027374','2002-005C','Iridium 94','Iridium SV094',67381,'Reentered','2018-04-18 00:00:00','2002-02-18 00:00:00',97.82,648,660,86.57,'Polar' from dual union all
select 27375,'027375','2002-005D','Iridium 95','Iridium SV095',67381,'In Earth orbit','','2002-02-18 00:00:00',98.19,664,681,86.58,'Polar' from dual union all
select 27376,'027376','2002-005E','Iridium 96','Iridium SV096',67381,'In Earth orbit','','2002-02-13 00:00:00',97.39,626,642,86.58,'Polar' from dual union all
select 27377,'027377','2002-005F','Delta 290','Delta SSPS AJ-10-118K',67381,'Reentered','2002-09-16 00:00:00','2002-02-14 00:00:00',93.97,312,626,84.42,'Intermediate' from dual union all
select 27378,'027378','2002-006A','Echostar VII','Echostar 7',67382,'In Earth orbit','','2003-03-01 00:00:00',1436.06,35780,35792,.05,'Stationary' from dual union all
select 27379,'027379','2002-006B','Centaur AC-204','Centaur IIIB DEC AC-204',67382,'Reentered','2002-09-24 00:00:00','2002-02-21 00:00:00',1085.87,219,57016,22.77,'Highly Ellip' from dual union all
select 2738,'002738','1967-024Q','deb S1M','',15279,'Reentered','1967-03-27 00:00:00','1967-03-31 00:00:00',90.41,195,395,48.3,'Intermediate' from dual union all
select 27380,'027380','2002-007A','INTELSAT 904','INTELSAT 904',67383,'In Earth orbit','','2003-02-07 00:00:00',1436.04,35758,35812,.01,'Stationary' from dual union all
select 27381,'027381','2002-007B','Ariane H10-3 V148','Ariane H10-3 S/N T4110',67383,'Reentered','2002-07-01 00:00:00','2002-02-25 00:00:00',660.95,197,37316,7,'GEO Transfer' from dual union all
select 27382,'027382','2002-008A','Kosmos-2387','Kobal''t No. 555',67384,'Landed','2002-06-27 00:00:00','2002-02-26 00:00:00',89.59,165,344,67.12,'Intermediate' from dual union all
select 27383,'027383','2002-008B','11S510','11S510',67384,'Reentered','2002-02-28 00:00:00','2002-02-26 00:00:00',89.44,162,332,67.12,'Intermediate' from dual union all
select 27384,'027384','1999-057MJ','CZ-4 debris','',66738,'In Earth orbit','','2002-02-24 00:00:00',100.21,690,847,98.33,'Sun-Synch' from dual union all
select 27385,'027385','1999-057MK','CZ-4 debris','',66738,'In Earth orbit','','2002-02-24 00:00:00',99.93,675,835,98.32,'Sun-Synch' from dual union all
select 27386,'027386','2002-009A','Envisat','Envisat PPF',67385,'In Earth orbit','','2002-03-08 00:00:00',100.6,784,790,98.56,'Sun-Synch' from dual union all
select 27387,'027387','2002-009B','EPS 511','EPS S/N 512R1',67385,'In Earth orbit','','2002-03-08 00:00:00',100.34,759,791,98.54,'Sun-Synch' from dual union all
select 27388,'027388','2002-010A','Columbia (STS-109)','OV-102',67386,'Landed','2002-03-12 00:00:00','2002-03-03 00:00:00',92.44,215,573,28.47,'Intermediate' from dual union all
select 27389,'027389','2002-011A','TDRS 9','TDRS I',67387,'In Earth orbit','','2002-11-01 00:00:00',1436.16,35767,35808,8.39,'Inclined GEO' from dual union all
select 2739,'002739','1967-024R','deb S1M','',15279,'Reentered','1967-03-27 00:00:00','1967-03-31 00:00:00',90.35,251,333,48.3,'Intermediate' from dual union all
select 27390,'027390','2002-011B','Centaur AC-143','Centaur AC-143',67387,'In Earth orbit','','2002-03-20 00:00:00',504.85,237,29012,27.01,'Highly Ellip' from dual union all
select 27391,'027391','2002-012A','GRACE 1','GRACE 1',67388,'Reentered','2018-03-10 00:00:00','2002-03-19 00:00:00',94.49,482,506,89.03,'Polar' from dual union all
select 27392,'027392','2002-012B','GRACE 2','GRACE 2',67388,'Reentered','2017-12-24 00:00:00','2002-03-20 00:00:00',94.5,482,507,89.03,'Polar' from dual union all
select 27393,'027393','2002-012C','Briz-KM No. 72505','Briz-KM 14S45',67388,'Reentered','2002-03-27 00:00:00','2002-03-20 00:00:00',90.44,143,450,89.01,'Polar' from dual union all
select 27394,'027394','2001-051C','Kolibri','Kolibri-2000',67204,'Reentered','2002-05-04 00:00:00','2002-03-20 00:00:00',92.28,385,388,51.63,'Intermediate' from dual union all
select 27395,'027395','2002-013A','Progress M1-8','Progress M1 No. 257',67389,'Deorbited','2002-06-25 00:00:00','2002-03-25 00:00:00',92.36,388,393,51.63,'Intermediate' from dual union all
select 27396,'027396','2002-013B','11S510','11S510',67389,'Reentered','2002-03-23 00:00:00','2002-03-22 00:00:00',88.61,185,227,51.61,'Intermediate' from dual union all
select 27397,'027397','2002-014A','Shenzhou 3','Shenzhou 3 Fanhui Cang',67390,'Landed','2002-04-01 00:00:00','2002-03-27 00:00:00',91.2,331,336,42.4,'Intermediate' from dual union all
select 27398,'027398','2002-014B','CZ-2F Stage 2','CZ-2F Stage 2',67390,'Reentered','2002-04-02 00:00:00','2002-03-27 00:00:00',89.5,193,307,42.39,'Intermediate' from dual union all
select 27399,'027399','2002-015A','JCSAT 2A','JCSAT-8',67391,'In Earth orbit','','2003-06-13 00:00:00',1436.11,35776,35797,.05,'Stationary' from dual union all
select 274,'000274','62 Kappa 4','Midas V solar array cover','Strongback',8192,'In Earth orbit','','1962-06-05 00:00:00',153.32,2784,3438,86.65,'Medium' from dual union all
select 2740,'002740','1967-030A','Kosmos-153','Zenit-2 No. 48',15285,'Landed','1967-04-12 00:00:00','1967-04-07 00:00:00',89.2,197,273,64.59,'Intermediate' from dual union all
select 27400,'027400','2002-015B','Astra 3A','Astra 3A',67391,'In Earth orbit','','2003-05-14 00:00:00',1436.06,35780,35792,.05,'Stationary' from dual union all
select 27401,'027401','2002-015C','Ariane H10-3 V149','Ariane H10-3 S/N T4109',67391,'Reentered','2017-04-12 00:00:00','2002-03-31 00:00:00',632.54,255,35807,4.03,'GEO Transfer' from dual union all
select 27402,'027402','2002-015D','Spelda 30 S/N UV43','Spelda 40',67391,'In Earth orbit','','2002-03-31 00:00:00',632.32,270,35780,4.05,'GEO Transfer' from dual union all
select 27403,'027403','2002-016A','INTELSAT 903','INTELSAT 903',67392,'In Earth orbit','','2003-05-09 00:00:00',1436.09,35774,35799,.01,'Stationary' from dual union all
select 27404,'027404','2002-016B','Blok DM3 No. 28L','Blok DM3 No. 28L',67392,'In Earth orbit','','2002-04-01 00:00:00',698.01,3463,35912,25.01,'GEO Transfer' from dual union all
select 27405,'027405','1990-065W','G-5B Canister','Li Canister',63307,'In Earth orbit','','2002-04-08 00:00:00',489.73,281,28125,17.76,'Highly Ellip' from dual union all
select 27406,'027406','1966-040AL','Nimbus 2 debris','',13447,'In Earth orbit','','2004-09-27 00:00:00',107.15,1060,1129,100.3,'Sun-Synch' from dual union all
select 27407,'027407','1999-035C','FUSE insulation','',66716,'Reentered','2004-12-25 00:00:00','2004-09-29 00:00:00',98.83,684,721,24.99,'Equatorial' from dual union all
select 27408,'027408','2002-014C','Shenzhou 3 Guidao Cang','Shenzhou 3 orbital modul',67390,'Reentered','2002-11-12 00:00:00','2002-04-01 00:00:00',91.23,330,340,42.42,'Intermediate' from dual union all
select 27409,'027409','2002-017A','Kosmos-2388','Oko US-KS',67393,'Reentered','2011-09-14 00:00:00','2002-04-18 00:00:00',715.57,519,39727,62.93,'Molniya' from dual union all
select 2741,'002741','1967-010B','Burner II No. 2','Star 37B',15265,'In Earth orbit','','1967-04-02 00:00:00',101.48,791,867,98.81,'Sun-Synch' from dual union all
select 27410,'027410','2002-017B','11S510','11S510',67393,'Reentered','2002-04-29 00:00:00','2002-04-04 00:00:00',91.75,231,490,62.83,'Intermediate' from dual union all
select 27411,'027411','2002-017C','BOZ','BOZ',67393,'Reentered','2002-04-19 00:00:00','2002-04-04 00:00:00',91.58,197,508,62.86,'Intermediate' from dual union all
select 27412,'027412','2002-017D','Blok-2BL','Blok-2BL No. 77314-622',67393,'Reentered','2014-12-18 00:00:00','2002-04-23 00:00:00',701.83,518,39047,62.93,'Molniya' from dual union all
select 27413,'027413','2002-018A','Atlantis (STS-110)','OV-104',67394,'Landed','2002-04-19 00:00:00','2002-04-09 00:00:00',88.29,155,225,51.63,'Intermediate' from dual union all
select 27414,'027414','2002-019A','NSS 7','NSS 7',67395,'In Earth orbit','','2003-04-11 00:00:00',1436.06,35777,35795,0,'Stationary' from dual union all
select 27415,'027415','2002-019B','Ariane H10-3 V150','Ariane H10-3 S/N T4111',67395,'Reentered','2005-03-26 00:00:00','2002-04-26 00:00:00',656.28,199,37077,7,'GEO Transfer' from dual union all
select 27416,'027416','2002-020A','Soyuz TM-34','Soyuz 7K-STM No. 208',67396,'Landed','2002-11-10 00:00:00','2002-05-03 00:00:00',92.36,385,396,51.63,'Intermediate' from dual union all
select 27417,'027417','2002-020B','11S510','11S510',67396,'Reentered','2002-04-26 00:00:00','2002-04-25 00:00:00',88.36,184,202,51.61,'Intermediate' from dual union all
select 27418,'027418','1991-076M','deb USA 76','',64245,'In Earth orbit','','2002-04-30 00:00:00',107.19,898,1296,63.4,'Intermediate' from dual union all
select 27419,'027419','1991-076N','deb USA 77','',64245,'In Earth orbit','','2002-04-30 00:00:00',107.09,900,1283,63.4,'Intermediate' from dual union all
select 2742,'002742','1967-030B','8A92E','8A92E',15285,'Reentered','1967-04-11 00:00:00','1967-04-06 00:00:00',89.23,214,259,64.58,'Intermediate' from dual union all
select 27420,'027420','1990-050J','USA 60 cover','',63292,'In Earth orbit','','2002-04-30 00:00:00',107.42,1051,1164,63.4,'Intermediate' from dual union all
select 27421,'027421','2002-021A','SPOT 5','SPOT 5',67397,'In Earth orbit','','2002-05-15 00:00:00',101.23,816,818,98.75,'Sun-Synch' from dual union all
select 27422,'027422','2002-021B','Ariane H10-3 V151','Ariane H10-3 S/N T4112',67397,'In Earth orbit','','2002-05-06 00:00:00',100.84,791,806,98.72,'Sun-Synch' from dual union all
select 27423,'027423','1972-102C','deb Kosmos-539','impact fragment',28034,'Reentered','2002-06-03 00:00:00','2002-05-06 00:00:00',113.39,1038,1727,74.21,'Intermediate' from dual union all
select 27424,'027424','2002-022A','Aqua','EOS PM-1',67398,'In Earth orbit','','2002-05-14 00:00:00',98.34,673,686,98.18,'Sun-Synch' from dual union all
select 27425,'027425','2002-022B','Delta 291','Delta SSPS AJ-10-118K',67398,'Reentered','2002-07-02 00:00:00','2002-05-15 00:00:00',93.07,213,637,97.4,'Sun-Synch' from dual union all
select 27426,'027426','2002-023A','DirecTV-5','DirecTV-5',67399,'In Earth orbit','','2003-07-05 00:00:00',1436.06,35775,35797,.01,'Stationary' from dual union all
select 27427,'027427','2002-023B','8S812','8S812',67399,'Reentered','2002-05-09 00:00:00','2002-05-08 00:00:00',87.91,144,199,51.6,'Intermediate' from dual union all
select 27428,'027428','2002-023C','Perekhodnik','Sredniy perekhodnik',67399,'Reentered','2002-05-08 00:00:00','2002-05-08 00:00:00',87.27,131,148,51.59,'Intermediate' from dual union all
select 27429,'027429','2002-023D','Blok DM3 No. 21L','Blok DM3 No. 21L',67399,'In Earth orbit','','2002-05-09 00:00:00',755.57,6433,35769,17.66,'Highly Ellip' from dual union all
select 2743,'002743','1967-031A','ATS 2','ATS A',15286,'Reentered','1969-09-02 00:00:00','1967-05-08 00:00:00',214.08,186,10766,28.46,'Medium' from dual union all
select 27430,'027430','2002-024A','Hai Yang 1','haiyang yihao A',67400,'In Earth orbit','','2002-05-28 00:00:00',100.7,792,792,98.8,'Sun-Synch' from dual union all
select 27431,'027431','2002-024B','Feng Yun 1D','feng yun yi ding',67400,'In Earth orbit','','2002-05-17 00:00:00',102.17,851,872,98.8,'Sun-Synch' from dual union all
select 27432,'027432','2002-024C','CZ-4B Stage 3','CZ-4 Stage 3',67400,'In Earth orbit','','2002-05-16 00:00:00',101.86,811,883,98.71,'Sun-Synch' from dual union all
select 27433,'027433','2002-024D','Adapter','',67400,'In Earth orbit','','2002-05-19 00:00:00',102.13,850,868,98.8,'Sun-Synch' from dual union all
select 27434,'027434','2002-025A','''Ofeq-5','''Ofeq-5',67401,'In Earth orbit','','2002-06-05 00:00:00',95.97,367,763,143.46,'Retrograde' from dual union all
select 27435,'027435','2002-025B','AUS-51','AUS-51',67401,'Reentered','2002-08-31 00:00:00','2002-06-05 00:00:00',94.61,259,740,143.47,'Retrograde' from dual union all
select 27436,'027436','2002-026A','Kosmos-2389','Parus',67402,'In Earth orbit','','2002-06-04 00:00:00',104.74,949,1015,82.95,'Intermediate' from dual union all
select 27437,'027437','2002-026B','S3M','S3M',67402,'In Earth orbit','','2002-06-04 00:00:00',104.61,949,1003,82.95,'Intermediate' from dual union all
select 27438,'027438','2002-027A','INTELSAT 905','INTELSAT 905',67403,'In Earth orbit','','2003-06-28 00:00:00',1436.06,35778,35793,.03,'Stationary' from dual union all
select 27439,'027439','2002-027B','Ariane H10-3 V152','Ariane H10-3 S/N T4113',67403,'Reentered','2003-11-01 00:00:00','2002-06-10 00:00:00',658.44,211,37175,6.81,'GEO Transfer' from dual union all
select 2744,'002744','1967-031B','Agena D 6152','Agena D 6152',15286,'Reentered','1968-06-29 00:00:00','1967-05-08 00:00:00',213.27,161,10731,28.47,'Medium' from dual union all
select 27440,'027440','2002-028A','Endeavour (STS-111)','OV-105',67404,'Landed','2002-06-19 00:00:00','2002-06-08 00:00:00',92.25,381,388,51.64,'Intermediate' from dual union all
select 27441,'027441','2002-029A','Ekspress A1R','Ekspress-A No. 4',67405,'In Earth orbit','','2002-06-12 00:00:00',1454,36102,36171,.21,'Drift GEO' from dual union all
select 27442,'027442','2002-029B','8S812 No. 407-01','8S812',67405,'Reentered','2002-06-12 00:00:00','2002-06-12 00:00:00',87.36,140,148,51.63,'Intermediate' from dual union all
select 27443,'027443','2002-029C','Perekhodnik','Sredniy perekhodnik',67405,'Reentered','2002-06-10 00:00:00','2002-06-10 00:00:00',88.12,178,185,51.63,'Intermediate' from dual union all
select 27444,'027444','2002-029D','Blok DM-2M No. 11L','11S861-01 No. 11L',67405,'In Earth orbit','','2002-06-11 00:00:00',1450.7,36006,36138,.24,'Drift GEO' from dual union all
select 27445,'027445','2002-030A','Galaxy 3C','Galaxy III-C',67406,'In Earth orbit','','2003-06-22 00:00:00',1436.09,35785,35788,.05,'Stationary' from dual union all
select 27446,'027446','2002-030B','Blok DM-SL No. 8L','Blok DM-SL',67406,'In Earth orbit','','2002-06-15 00:00:00',747.48,385,41425,.14,'GEO Transfer' from dual union all
select 27447,'027447','2002-029E','SOZ','SOZ',67405,'Reentered','2003-08-24 00:00:00','2002-06-18 00:00:00',640.31,328,36133,47.39,'GEO Transfer' from dual union all
select 27448,'027448','2002-029F','SOZ','SOZ',67405,'Reentered','2002-12-23 00:00:00','2002-06-19 00:00:00',639.14,269,36132,47.39,'GEO Transfer' from dual union all
select 27449,'027449','1999-057ML','CZ-4 debris','',66738,'Reentered','2013-02-03 00:00:00','2002-06-16 00:00:00',97.57,577,707,98.56,'Sun-Synch' from dual union all
select 2745,'002745','1967-032A','SOZ','SOZ',15287,'Reentered','1967-04-10 00:00:00','1967-04-10 00:00:00',88.36,180,207,51.53,'Intermediate' from dual union all
select 27450,'027450','2002-031A','Iridium 97','Iridium SV097',67407,'In Earth orbit','','2002-06-22 00:00:00',98.02,658,670,86.59,'Polar' from dual union all
select 27451,'027451','2002-031B','Iridium 98','Iridium SV098',67407,'In Earth orbit','','2002-06-23 00:00:00',97.98,658,667,86.58,'Polar' from dual union all
select 27452,'027452','2002-031C','Briz-KM No. 72502','Briz-KM 14S45 No. 72502/',67407,'Reentered','2002-11-16 00:00:00','2002-06-23 00:00:00',93.5,232,660,86.58,'Polar' from dual union all
select 27453,'027453','2002-032A','NOAA 17','NOAA M',67408,'In Earth orbit','','2002-06-27 00:00:00',101.16,806,822,98.78,'Sun-Synch' from dual union all
select 27454,'027454','2002-033A','Progress M-46','Progress M No. 246',67409,'Deorbited','2002-10-14 00:00:00','2002-06-27 00:00:00',89.97,269,277,51.63,'Intermediate' from dual union all
select 27455,'027455','2002-033B','11S510','11S510',67409,'Reentered','2002-06-28 00:00:00','2002-06-27 00:00:00',87.72,156,168,51.62,'Intermediate' from dual union all
select 27456,'027456','2002-008C','Kosmos-2387 solar panel','Kobal''t solar panel',67384,'Reentered','2002-06-28 00:00:00','2002-06-27 00:00:00',88.7,162,259,67.09,'Intermediate' from dual union all
select 27457,'027457','2002-034A','Contour','Contour',67410,'Deep Space','2002-08-15 00:00:00','2002-07-10 00:00:00',2427.14,204,106692,29.87,'Very High' from dual union all
select 27458,'027458','2002-034B','Delta 292','Delta SSPS AJ-10-118K',67410,'Reentered','2002-07-06 00:00:00','2002-07-05 00:00:00',88.68,170,249,29.58,'Intermediate' from dual;
insert into SATELLITE
select 27459,'027459','2002-034C','Star 48B','Star 48B',67410,'Reentered','2003-10-15 00:00:00','2002-07-11 00:00:00',2412.25,192,106214,29.96,'Very High' from dual union all
select 2746,'002746','1967-032C','Kosmos-154','L-1 No. 3P',15287,'Reentered Attached','1967-04-19 00:00:00','1967-04-09 00:00:00',88.43,182,211,51.53,'Intermediate' from dual union all
select 27460,'027460','2002-035A','Stellat 5','Stellat 5',67411,'In Earth orbit','','2003-05-30 00:00:00',1436.09,35772,35801,.01,'Stationary' from dual union all
select 27461,'027461','2002-035B','N-Star c','N-Star c',67411,'In Earth orbit','','2002-07-14 00:00:00',1446.52,35816,36165,.05,'Drift GEO' from dual union all
select 27462,'027462','2002-035C','EPS 512','EPS S/N 511R1',67411,'In Earth orbit','','2002-07-14 00:00:00',646.85,621,36175,5.5,'GEO Transfer' from dual union all
select 27463,'027463','2002-035D','Sylda SY-04F','Sylda 5F',67411,'In Earth orbit','','2002-07-12 00:00:00',637.21,593,35709,5.51,'GEO Transfer' from dual union all
select 27464,'027464','2002-036A','Kosmos-2390','Strela-3 No. 37',67412,'In Earth orbit','','2002-07-17 00:00:00',115.7,1468,1506,82.48,'Intermediate' from dual union all
select 27465,'027465','2002-036B','Kosmos-2391','Strela-3 No. 50',67412,'In Earth orbit','','2002-07-18 00:00:00',115.72,1468,1508,82.49,'Intermediate' from dual union all
select 27466,'027466','2002-036C','S3M','S3M',67412,'In Earth orbit','','2002-07-16 00:00:00',115.72,1468,1508,82.49,'Intermediate' from dual union all
select 27467,'027467','1983-127P','deb SOZ','',55884,'Reentered','2005-05-15 00:00:00','2002-07-08 00:00:00',169.9,687,6880,51.95,'Medium' from dual union all
select 27468,'027468','1983-127Q','deb SOZ','',55884,'Reentered','2004-07-11 00:00:00','2002-07-08 00:00:00',169.98,687,6886,51.95,'Medium' from dual union all
select 27469,'027469','2000-047C','Titan 4B deb','',66949,'In Earth orbit','','2002-07-14 00:00:00',96.91,563,659,68,'Intermediate' from dual union all
select 2747,'002747','1967-032B','SOZ','SOZ',15287,'Reentered','1967-04-10 00:00:00','1967-04-10 00:00:00',88.39,180,210,51.53,'Intermediate' from dual union all
select 27470,'027470','2002-037A','Kosmos-2392','Arkon 11F664 No. 2',67413,'In Earth orbit','','2002-07-26 00:00:00',119.89,1512,1840,63.46,'Intermediate' from dual union all
select 27471,'027471','2002-037B','8S812','8S812',67413,'Reentered','2002-07-26 00:00:00','2002-07-26 00:00:00',87.25,132,145,64.86,'Intermediate' from dual union all
select 27472,'027472','2002-037C','Perekhodnik','Sredniy perekhodnik',67413,'Reentered','2002-07-25 00:00:00','2002-07-25 00:00:00',87.36,136,152,64.86,'Intermediate' from dual union all
select 27473,'027473','2002-037D','Blok DM5 No. 2L','17S40 No. 2L',67413,'In Earth orbit','','2002-07-26 00:00:00',119.85,1510,1839,63.46,'Intermediate' from dual union all
select 27474,'027474','2002-037E','SOZ','SOZ',67413,'Reentered','2006-02-28 00:00:00','2002-07-26 00:00:00',116.95,199,2889,63.79,'Intermediate' from dual union all
select 27475,'027475','2002-037F','SOZ','SOZ',67413,'Reentered','2005-04-29 00:00:00','2002-07-26 00:00:00',116.89,193,2889,63.79,'Intermediate' from dual union all
select 27476,'027476','2002-037G','Kosmos-2392 part','',67413,'In Earth orbit','','2002-08-01 00:00:00',119.84,1507,1841,63.46,'Intermediate' from dual union all
select 27477,'027477','2000-055B','NOAA 16 solar array band','',66957,'Reentered','2005-07-15 00:00:00','2002-08-01 00:00:00',99.42,721,741,98.77,'Sun-Synch' from dual union all
select 27478,'027478','61 A Delta 8','West Ford package fragment','',7251,'In Earth orbit','','2002-07-29 00:00:00',166.14,3329,3937,95.84,'Medium' from dual union all
select 27479,'027479','1963-038P','deb NNS','',9269,'In Earth orbit','','2002-07-28 00:00:00',106.87,1050,1114,90.1,'Polar' from dual union all
select 2748,'002748','1967-024S','deb S1M','',15279,'Reentered','1967-03-31 00:00:00','1967-03-31 00:00:00',90.24,258,315,48.3,'Intermediate' from dual union all
select 27480,'027480','1968-055H','deb Star 37D','',16782,'In Earth orbit','','2002-07-27 00:00:00',152.67,861,5307,120.88,'Medium' from dual union all
select 27481,'027481','1968-055J','deb Star 37D','',16782,'In Earth orbit','','2002-07-03 00:00:00',136.34,715,4079,120.7,'Medium' from dual union all
select 27482,'027482','1990-081CU','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2002-07-26 00:00:00',102.36,862,879,98.98,'Sun-Synch' from dual union all
select 27483,'027483','1991-001D','deb Delta','',64170,'Reentered','2013-04-04 00:00:00','2002-07-29 00:00:00',111.36,753,1826,18.39,'Equatorial' from dual union all
select 27484,'027484','1992-041E','deb Ariane V47','',64753,'Reentered','2003-04-27 00:00:00','2002-07-27 00:00:00',419.47,248,24120,7.36,'Highly Ellip' from dual union all
select 27485,'027485','1992-041F','deb Ariane V47','',64753,'Reentered','2003-03-19 00:00:00','2002-07-27 00:00:00',434.3,256,24982,7.22,'Highly Ellip' from dual union all
select 27486,'027486','1992-041G','deb Ariane V47','',64753,'Reentered','2003-07-04 00:00:00','2002-07-25 00:00:00',433.03,274,24890,7.25,'Highly Ellip' from dual union all
select 27487,'027487','1992-041H','deb Ariane V47','',64753,'Reentered','2003-11-10 00:00:00','2002-07-28 00:00:00',442.52,266,25450,7.15,'Highly Ellip' from dual union all
select 27488,'027488','1997-005F','HALCA despin weight','',66085,'Reentered','2003-04-23 00:00:00','2002-07-29 00:00:00',134.4,248,4379,31.23,'Medium' from dual union all
select 27489,'027489','1997-005G','HALCA despin weight','',66085,'In Earth orbit','','2002-04-02 00:00:00',299.99,239,16694,31.41,'Highly Ellip' from dual union all
select 2749,'002749','1967-024T','deb S1M','',15279,'Reentered','1967-04-04 00:00:00','1967-03-31 00:00:00',88.75,201,225,48.3,'Intermediate' from dual union all
select 27490,'027490','1997-035D','Star 48 yo weight','',66115,'Reentered','2002-10-28 00:00:00','2002-07-29 00:00:00',121.99,138,3402,38.9,'Medium' from dual union all
select 27491,'027491','1998-043L','Sep Motor Cover','Sep Motor Cover',66479,'In Earth orbit','','2002-07-29 00:00:00',104.46,817,1122,98.89,'Sun-Synch' from dual union all
select 27492,'027492','1999-057MM','CZ-4 debris','',66738,'In Earth orbit','','2002-07-28 00:00:00',100.32,695,853,98.42,'Sun-Synch' from dual union all
select 27493,'027493','1999-057MN','CZ-4 debris','',66738,'In Earth orbit','','2002-07-28 00:00:00',100.15,703,828,98.35,'Sun-Synch' from dual union all
select 27494,'027494','2002-037H','Kosmos-2392 sensor cover','',67413,'In Earth orbit','','2002-08-15 00:00:00',119.01,1512,1761,63.46,'Intermediate' from dual union all
select 27495,'027495','1983-044FS','deb Kosmos-1461','',55801,'Reentered','2002-12-20 00:00:00','2002-08-16 00:00:00',97.43,540,732,65.01,'Intermediate' from dual union all
select 27496,'027496','2001-049MQ','deb PSLV-C3 PS4','',67202,'Reentered','2002-12-07 00:00:00','2002-08-19 00:00:00',94.8,475,543,97.77,'Sun-Synch' from dual union all
select 27497,'027497','2001-055E','deb TIMED','',67208,'Reentered','2012-02-10 00:00:00','2002-08-19 00:00:00',96.69,597,604,74.07,'Intermediate' from dual union all
select 27498,'027498','2001-049MR','deb PSLV-C3 PS4','',67202,'Reentered','2010-03-30 00:00:00','2002-08-22 00:00:00',96.15,509,640,97.85,'Sun-Synch' from dual union all
select 27499,'027499','2002-038A','Hot Bird 6','Eutelsat HB6',67414,'In Earth orbit','','2004-05-26 00:00:00',1436.01,35734,35836,.04,'Stationary' from dual union all
select 275,'000275','61 Omicron 86','deb r/b Ablestar','',7273,'Reentered','2001-12-01 00:00:00','1962-07-07 00:00:00',102.98,817,983,66.79,'Intermediate' from dual union all
select 2750,'002750','1967-033A','Kosmos-155','Zenit-4 No. 28',15288,'Landed','1967-04-20 00:00:00','1967-04-15 00:00:00',89.23,202,270,51.79,'Intermediate' from dual union all
select 27500,'027500','2002-038B','Centaur AV-001','Centaur AV-001',67414,'In Earth orbit','','2002-08-28 00:00:00',789.74,273,43575,17.98,'Highly Ellip' from dual union all
select 27501,'027501','2002-039A','Echostar VIII','Echostar 8',67415,'In Earth orbit','','2002-08-29 00:00:00',1436.85,35773,35830,.08,'Drift GEO' from dual union all
select 27502,'027502','2001-049MS','deb PSLV-C3 PS4','',67202,'Reentered','2005-11-10 00:00:00','2002-08-23 00:00:00',96.54,561,625,97.8,'Sun-Synch' from dual union all
select 27503,'027503','2001-049MT','deb PSLV-C3 PS4','',67202,'Reentered','2006-11-25 00:00:00','2002-08-26 00:00:00',96.05,518,621,97.8,'Sun-Synch' from dual union all
select 27504,'027504','2002-039B','Blok DM3 No. 20L','Blok DM3 No. 20L',67415,'In Earth orbit','','2002-08-30 00:00:00',713.41,4277,35862,23.27,'GEO Transfer' from dual union all
select 27505,'027505','1998-067R','EVA decontam towel','',66503,'Reentered','2002-09-04 00:00:00','2002-08-26 00:00:00',91.4,335,352,51.63,'Intermediate' from dual union all
select 27506,'027506','1998-067S','EVA decontam towel','',66503,'Reentered','2002-09-03 00:00:00','2002-08-26 00:00:00',91.43,335,355,51.63,'Intermediate' from dual union all
select 27507,'027507','1998-067T','EVA decontam towel','',66503,'Reentered','2002-09-04 00:00:00','2002-08-26 00:00:00',91.52,342,357,51.63,'Intermediate' from dual union all
select 27508,'027508','2002-040A','Atlantic Bird 1','Atlantic Bird 1',67416,'In Earth orbit','','2003-08-31 00:00:00',1436.04,35776,35795,.06,'Stationary' from dual union all
select 27509,'027509','2002-040B','Meteosat 8','MSG 1',67416,'In Earth orbit','','2003-09-12 00:00:00',1436.09,35775,35798,1.21,'Stationary' from dual union all
select 2751,'002751','1967-033B','11A57I','11A57I',15288,'Reentered','1967-04-16 00:00:00','1967-04-14 00:00:00',88.83,177,257,51.78,'Intermediate' from dual union all
select 27510,'027510','2002-040C','EPS 513','EPS S/N 514',67416,'In Earth orbit','','2002-08-30 00:00:00',634.09,563,35579,5.32,'GEO Transfer' from dual union all
select 27511,'027511','2002-040D','Sylda SY-08C','SYLDA 5C',67416,'In Earth orbit','','2002-08-30 00:00:00',641.17,589,35916,5.46,'GEO Transfer' from dual union all
select 27512,'027512','2001-049MU','deb PSLV-C3 PS4','',67202,'Reentered','2004-01-25 00:00:00','2002-09-04 00:00:00',96.46,549,629,97.79,'Sun-Synch' from dual union all
select 27513,'027513','2002-041A','INTELSAT 906','INTELSAT 906',67417,'In Earth orbit','','2003-10-24 00:00:00',1436.09,35776,35797,.01,'Stationary' from dual union all
select 27514,'027514','2002-041B','Ariane H10-3 V154','Ariane H10-3 S/N T4114',67417,'Reentered','2002-12-04 00:00:00','2002-09-06 00:00:00',631.47,179,35828,7.04,'GEO Transfer' from dual union all
select 27515,'027515','2002-042A','USERS','USERS SEM',67418,'Reentered','2007-06-15 00:00:00','2003-05-29 00:00:00',94.23,474,489,30.41,'Intermediate' from dual union all
select 27516,'027516','2002-042B','Kodama','DRTS',67418,'In Earth orbit','','2003-11-19 00:00:00',1436.09,35780,35793,.1,'Stationary' from dual union all
select 27517,'027517','2002-042C','4/4D-LC Adapter','Adapter',67418,'Reentered','2002-12-11 00:00:00','2002-09-10 00:00:00',93.56,442,456,30.41,'Intermediate' from dual union all
select 27518,'027518','2002-042D','H-2A F3 Stage 2','H-2A Stage 2',67418,'In Earth orbit','','2002-09-10 00:00:00',635.96,438,35800,30.4,'GEO Transfer' from dual union all
select 27519,'027519','2002-042E','4/4D-LC Lower Fairing Half','Fairing Half',67418,'Reentered','2002-11-03 00:00:00','2002-09-10 00:00:00',93.5,437,455,30.41,'Intermediate' from dual union all
select 2752,'002752','1967-033C','deb Kosmos-155','',15288,'Reentered','1967-04-13 00:00:00','1967-04-12 00:00:00',88.03,177,177,51.85,'Intermediate' from dual union all
select 27520,'027520','2002-042F','4/4D-LC Lower Fairing Half','Fairing Half',67418,'Reentered','2002-11-12 00:00:00','2002-09-10 00:00:00',93.56,442,456,30.41,'Intermediate' from dual union all
select 27521,'027521','1998-067U','ISS EVA debris','',66503,'Reentered','2002-09-11 00:00:00','2002-09-11 00:00:00',89.19,229,239,51.63,'Intermediate' from dual union all
select 27522,'027522','1998-067V','ISS EVA debris','',66503,'Reentered','2002-09-11 00:00:00','2002-09-11 00:00:00',89.45,242,253,51.63,'Intermediate' from dual union all
select 27523,'027523','1998-067W','ISS EVA debris','',66503,'Reentered','2002-09-12 00:00:00','2002-09-11 00:00:00',90.14,274,289,51.62,'Intermediate' from dual union all
select 27524,'027524','1998-067X','ISS EVA debris','',66503,'Reentered','2002-09-12 00:00:00','2002-09-11 00:00:00',89.93,263,279,51.62,'Intermediate' from dual union all
select 27525,'027525','2002-043A','KALPANA-1','METSAT-1',67419,'In Earth orbit','','2002-09-25 00:00:00',1436.06,35771,35801,.48,'Stationary' from dual union all
select 27526,'027526','2002-043B','PS4','PS4',67419,'Reentered','2012-05-09 00:00:00','2002-09-20 00:00:00',606.48,197,34515,17.77,'GEO Transfer' from dual union all
select 27527,'027527','2002-042G','4/4D-LC Part','',67418,'Reentered','2002-10-02 00:00:00','2002-09-17 00:00:00',92.94,410,427,30.4,'Intermediate' from dual union all
select 27528,'027528','2002-044A','Hispasat 1D','Hispasat 1D',67420,'In Earth orbit','','2003-10-19 00:00:00',1436.09,35774,35799,.04,'Stationary' from dual union all
select 27529,'027529','2002-044B','Centaur AC-159','Centaur IIA AC-159',67420,'Reentered','2003-04-03 00:00:00','2002-09-23 00:00:00',823.65,192,45265,20.93,'Highly Ellip' from dual union all
select 2753,'002753','1967-033D','deb Kosmos-155','',15288,'Reentered','1967-04-16 00:00:00','1967-04-12 00:00:00',88.93,196,247,51.8,'Intermediate' from dual union all
select 27530,'027530','2001-049MV','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-09-20 00:00:00',96.55,529,657,97.82,'Sun-Synch' from dual union all
select 27531,'027531','2002-045A','Progress M1-9','Progress M1 No. 258',67421,'Deorbited','2003-02-01 00:00:00','2002-09-27 00:00:00',90.58,280,326,51.64,'Intermediate' from dual union all
select 27532,'027532','2002-045B','11S510','11S510',67421,'Reentered','2002-09-27 00:00:00','2002-09-26 00:00:00',88.15,175,191,51.62,'Intermediate' from dual union all
select 27533,'027533','2002-036D','Strela-3 adapter','',67412,'In Earth orbit','','2002-09-25 00:00:00',115.69,1466,1508,82.49,'Intermediate' from dual union all
select 27534,'027534','2002-046A','Nadezhda-M','Nadezhda-M 17F118M No. 4',67422,'In Earth orbit','','2002-09-29 00:00:00',104.91,965,1015,82.93,'Intermediate' from dual union all
select 27535,'027535','2002-046B','S3M','S3M',67422,'In Earth orbit','','2002-09-28 00:00:00',104.78,964,1005,82.93,'Intermediate' from dual union all
select 27536,'027536','1999-057MP','CZ-4 debris','',66738,'In Earth orbit','','2002-09-26 00:00:00',100.15,737,795,98.45,'Sun-Synch' from dual union all
select 27537,'027537','2002-047A','Atlantis (STS-112)','OV-104',67423,'Landed','2002-10-18 00:00:00','2002-10-09 00:00:00',89.11,220,241,51.63,'Intermediate' from dual union all
select 27538,'027538','2001-049MW','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-10-11 00:00:00',96.72,546,658,97.79,'Sun-Synch' from dual union all
select 27539,'027539','1998-067Y','ISS EVA debris','',66503,'Reentered','2002-10-25 00:00:00','2002-10-17 00:00:00',91.55,342,360,51.63,'Intermediate' from dual union all
select 2754,'002754','1967-034A','NNS O-12','NNS O-12',15289,'In Earth orbit','','1967-05-16 00:00:00',106.5,1048,1080,90.25,'Polar' from dual union all
select 27540,'027540','2002-048A','Integral','Integral',67424,'Deep Space','2002-10-18 00:00:00','2002-10-28 00:00:00',4032.86,2284,152813,52.16,'Deep Space' from dual union all
select 27541,'027541','2002-048B','8S812','8S812',67424,'Reentered','2002-12-08 00:00:00','2002-10-19 00:00:00',93.11,198,656,51.57,'Intermediate' from dual union all
select 27542,'027542','2002-048C','Blok DM2 No. 5L','17S40 DM2 No. 5L',67424,'Deep Space','2002-10-18 00:00:00','2002-10-28 00:00:00',3903.31,653,150829,52.15,'Deep Space' from dual union all
select 27543,'027543','2002-048D','Perekhodnik','Sredniy perekhodnik',67424,'Reentered','2002-10-29 00:00:00','2002-10-19 00:00:00',92.95,195,642,51.56,'Intermediate' from dual union all
select 27544,'027544','2002-048E','deb Blok DM2','',67424,'Reentered','2002-11-01 00:00:00','2002-10-28 00:00:00',92.86,178,651,51.55,'Intermediate' from dual union all
select 27545,'027545','1965-027BH','deb SNAPSHOT','',11777,'In Earth orbit','','2002-10-28 00:00:00',111.46,1272,1316,90.14,'Polar' from dual union all
select 27546,'027546','2002-036E','Strela-3 adapter part','',67412,'In Earth orbit','','2002-10-27 00:00:00',115.66,1464,1507,82.48,'Intermediate' from dual union all
select 27547,'027547','2002-048F','deb Blok DM2','',67424,'Reentered','2002-10-31 00:00:00','2002-10-29 00:00:00',92.24,175,594,51.6,'Intermediate' from dual union all
select 27548,'027548','2002-048G','deb Blok DM2','',67424,'Reentered','2002-10-29 00:00:00','2002-10-29 00:00:00',89.98,162,385,51.55,'Intermediate' from dual;
insert into SATELLITE
select 27549,'027549','2002-048H','deb Blok DM2','',67424,'Reentered','2002-10-29 00:00:00','2002-10-29 00:00:00',90.58,153,453,51.49,'Intermediate' from dual union all
select 2755,'002755','1967-034B','Altair 2 X-258E6 ABL-134','X-258RHE6',15289,'In Earth orbit','','1967-08-20 00:00:00',106.52,1050,1080,90.25,'Polar' from dual union all
select 27550,'027550','2002-049A','Zi Yuan-2 02 xing','Jianbing-3 02 xing',67425,'Reentered','2015-01-22 00:00:00','2002-10-29 00:00:00',94.13,470,482,97.4,'Sun-Synch' from dual union all
select 27551,'027551','2002-049B','CZ-4B Stage 3','CZ-4B Stage 3',67425,'Reentered','2007-01-06 00:00:00','2002-10-28 00:00:00',94.31,396,575,97.27,'Sun-Synch' from dual union all
select 27552,'027552','2002-050A','Soyuz TMA-1','Soyuz 11F732 No. 211',67426,'Landed','2003-05-04 00:00:00','2002-11-06 00:00:00',92.4,383,402,51.64,'Intermediate' from dual union all
select 27553,'027553','2002-050B','11S510','11S510',67426,'Reentered','2002-11-01 00:00:00','2002-10-31 00:00:00',88.09,175,185,51.62,'Intermediate' from dual union all
select 27554,'027554','2002-051A','Eutelsat W5','Eutelsat W5',67427,'In Earth orbit','','2002-12-04 00:00:00',1436.24,35783,35796,.11,'Stationary' from dual union all
select 27555,'027555','2002-051B','Delta 293','DCSS-4 F01',67427,'In Earth orbit','','2002-11-27 00:00:00',612.51,512,34514,13.45,'GEO Transfer' from dual union all
select 27556,'027556','2002-052A','Endeavour (STS-113)','OV-105',67428,'Landed','2002-12-07 00:00:00','2002-11-26 00:00:00',92.3,378,396,51.64,'Intermediate' from dual union all
select 27557,'027557','2002-053A','Astra 1K','Astra 1K',67429,'Deorbited','2002-12-10 00:00:00','2002-11-27 00:00:00',88.79,142,288,51.59,'Intermediate' from dual union all
select 27558,'027558','2002-053B','Blok DM3 No. 24L','Blok DM3 No. 24L',67429,'Reentered','2002-11-28 00:00:00','2002-11-27 00:00:00',87.87,164,174,51.56,'Intermediate' from dual union all
select 27559,'027559','2002-054A','AlSat-1','DMC Alsat-1',67430,'In Earth orbit','','2002-12-01 00:00:00',99.02,680,743,98.24,'Sun-Synch' from dual union all
select 2756,'002756','1967-035A','Surveyor 3','Surveyor SC-3',15290,'Deep Space','1967-04-17 00:00:00','1967-04-17 00:00:00',21047.89,167,492121,29.9,'Cislunar' from dual union all
select 27560,'027560','2002-054B','Mozhaets-3','Mozhaets',67430,'In Earth orbit','','2002-12-01 00:00:00',99.07,683,746,98.24,'Sun-Synch' from dual union all
select 27561,'027561','2002-054C','S3M','S3M',67430,'In Earth orbit','','2002-11-30 00:00:00',99.11,686,747,98.24,'Sun-Synch' from dual union all
select 27562,'027562','2002-052B','MEPSI 1A','MEPSI',67428,'Reentered','2003-01-31 00:00:00','2002-12-02 00:00:00',92.4,388,396,51.65,'Intermediate' from dual union all
select 27563,'027563','2002-051C','deb Delta 293','deb DCSS-4 F01',67427,'Reentered','2002-11-30 00:00:00','2002-11-27 00:00:00',94.14,376,578,25.75,'Intermediate' from dual union all
select 27564,'027564','2002-051D','deb Delta 293','deb DCSS-4 F01',67427,'Reentered','2002-12-09 00:00:00','2002-12-04 00:00:00',95.69,416,687,25.55,'Intermediate' from dual union all
select 27565,'027565','2002-051E','deb Delta 293','deb DCSS-4 F01',67427,'Reentered','2002-12-06 00:00:00','2002-12-02 00:00:00',95.14,392,658,25.67,'Intermediate' from dual union all
select 27566,'027566','2002-055A','TDRS 10','TDRS J',67431,'In Earth orbit','','2004-01-01 00:00:00',1435.86,35766,35798,6.09,'Inclined GEO' from dual union all
select 27567,'027567','2002-055B','Centaur AC-144','Centaur IIA AC-144',67431,'In Earth orbit','','2002-12-05 00:00:00',545.38,206,31262,26.95,'Highly Ellip' from dual union all
select 27568,'027568','1980-034E','deb US-A coolant','',50244,'In Earth orbit','','2002-12-06 00:00:00',102.69,850,921,64.83,'Intermediate' from dual union all
select 27569,'027569','1981-021R','deb US-A coolant','',52544,'In Earth orbit','','2002-12-06 00:00:00',103.6,891,967,64.96,'Intermediate' from dual union all
select 2757,'002757','1967-036A','ESSA 5','TOS C',15291,'In Earth orbit','','1967-07-05 00:00:00',113.54,1356,1422,101.92,'Sun-Synch' from dual union all
select 27570,'027570','1981-021S','deb US-A coolant','',52544,'In Earth orbit','','2002-12-06 00:00:00',103.6,891,967,64.95,'Intermediate' from dual union all
select 27571,'027571','1981-021T','deb US-A coolant','',52544,'In Earth orbit','','2002-12-06 00:00:00',103.6,891,967,64.96,'Intermediate' from dual union all
select 27572,'027572','1984-069W','deb US-A coolant','',57123,'In Earth orbit','','2002-12-05 00:00:00',103.67,912,953,65.05,'Intermediate' from dual union all
select 27573,'027573','1984-069X','deb US-A coolant','',57123,'In Earth orbit','','2002-12-06 00:00:00',103.61,907,952,65.04,'Intermediate' from dual union all
select 27574,'027574','1975-052LC','deb Delta 111','',36499,'In Earth orbit','','2002-12-06 00:00:00',117.76,1205,1956,99.41,'Sun-Synch' from dual union all
select 27575,'027575','1977-065GE','deb Delta 132','',42193,'In Earth orbit','','2002-12-05 00:00:00',107.1,650,1535,29.13,'Intermediate' from dual union all
select 27576,'027576','1975-052LD','deb Delta 111','',36499,'In Earth orbit','','2002-12-05 00:00:00',105.31,922,1096,99.28,'Sun-Synch' from dual union all
select 27577,'027577','1975-052LE','deb Delta 111','',36499,'In Earth orbit','','2002-12-06 00:00:00',107.32,1041,1164,99.09,'Sun-Synch' from dual union all
select 27578,'027578','1975-052LF','deb Delta 111','',36499,'In Earth orbit','','2002-12-04 00:00:00',105.56,986,1056,99.57,'Sun-Synch' from dual union all
select 27579,'027579','2001-049MX','deb PSLV-C3 PS4','',67202,'Reentered','2013-04-01 00:00:00','2002-12-07 00:00:00',96.36,537,632,97.77,'Sun-Synch' from dual union all
select 2758,'002758','1967-036B','Altair 2 X-258E6 RH-141','X-258E6 RH-141',15291,'In Earth orbit','','1967-06-03 00:00:00',113.56,1358,1421,101.95,'Sun-Synch' from dual union all
select 27580,'027580','2001-049MY','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-11-30 00:00:00',98.06,629,703,98.01,'Sun-Synch' from dual union all
select 27581,'027581','2001-049MZ','deb PSLV-C3 PS4','',67202,'Reentered','2014-08-24 00:00:00','2002-11-28 00:00:00',96.1,510,634,97.75,'Sun-Synch' from dual union all
select 27582,'027582','2001-049NA','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-11-29 00:00:00',98.27,641,711,98.19,'Sun-Synch' from dual union all
select 27583,'027583','2001-049NB','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-12-03 00:00:00',97.7,616,682,97.97,'Sun-Synch' from dual union all
select 27584,'027584','2001-049NC','deb PSLV-C3 PS4','',67202,'Reentered','2004-03-21 00:00:00','2002-12-09 00:00:00',94.98,462,574,97.71,'Sun-Synch' from dual union all
select 27585,'027585','2001-049ND','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-12-07 00:00:00',98.18,621,723,97.84,'Sun-Synch' from dual union all
select 27586,'027586','2001-049NE','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2002-12-09 00:00:00',96.49,527,654,97.8,'Sun-Synch' from dual union all
select 27587,'027587','2001-049NF','deb PSLV-C3 PS4','',67202,'Reentered','2003-08-05 00:00:00','2002-12-09 00:00:00',95.39,513,561,97.78,'Sun-Synch' from dual union all
select 27588,'027588','2001-049NG','deb PSLV-C3 PS4','',67202,'Reentered','2003-05-28 00:00:00','2002-12-09 00:00:00',94.41,452,527,97.71,'Sun-Synch' from dual union all
select 27589,'027589','1985-037K','deb SOZ','',58260,'Reentered','2002-12-13 00:00:00','2002-12-02 00:00:00',142.02,325,4952,51.51,'Medium' from dual union all
select 2759,'002759','1967-037A','Soyuz-1','Soyuz No. 4',15292,'Deorbited','1967-04-24 00:00:00','1967-04-23 00:00:00',88.53,194,210,51.63,'Intermediate' from dual union all
select 27590,'027590','2001-049NH','deb PSLV-C3 PS4','',67202,'Reentered','2015-04-01 00:00:00','2002-12-14 00:00:00',96.6,545,647,97.85,'Sun-Synch' from dual union all
select 27591,'027591','2001-049NJ','deb PSLV-C3 PS4','',67202,'Reentered','2005-07-25 00:00:00','2002-12-13 00:00:00',96.92,600,623,97.97,'Sun-Synch' from dual union all
select 27592,'027592','2001-049NK','deb PSLV-C3 PS4','',67202,'Reentered','2003-01-21 00:00:00','2002-12-14 00:00:00',92.83,387,440,97.69,'Sun-Synch' from dual union all
select 27593,'027593','1988-040C','deb Ariane V23','',61289,'In Earth orbit','','2002-12-12 00:00:00',631.33,465,35535,7.21,'GEO Transfer' from dual union all
select 27594,'027594','1988-040D','deb Ariane V23','',61289,'In Earth orbit','','2002-12-11 00:00:00',633.86,453,35677,7.36,'GEO Transfer' from dual union all
select 27595,'027595','1988-040E','deb Ariane V23','',61289,'In Earth orbit','','2002-12-13 00:00:00',631.72,462,35558,7.19,'GEO Transfer' from dual union all
select 27596,'027596','1986-019VN','deb Ariane V16','',59212,'Reentered','1990-07-26 00:00:00','2002-12-13 00:00:00',100.33,738,811,98.67,'Sun-Synch' from dual union all
select 27597,'027597','2002-056A','Midori-2','ADEOS-2',67432,'In Earth orbit','','2002-12-21 00:00:00',100.98,804,807,98.69,'Sun-Synch' from dual union all
select 27598,'027598','2002-056B','Fedsat','Fedsat',67432,'In Earth orbit','','2002-12-21 00:00:00',100.85,792,806,98.67,'Sun-Synch' from dual union all
select 27599,'027599','2002-056C','WEOS','WEOS',67432,'In Earth orbit','','2002-12-21 00:00:00',100.82,790,805,98.68,'Sun-Synch' from dual union all
select 276,'000276','62 Lambda 1','Discoverer 39','CORONA M-2',8193,'Reentered Attached','1962-05-28 00:00:00','1962-04-19 00:00:00',91.58,201,503,73.5,'Intermediate' from dual union all
select 2760,'002760','1967-037B','11S510','11S510',15292,'Reentered','1967-04-24 00:00:00','1967-04-23 00:00:00',88.56,186,221,51.66,'Intermediate' from dual union all
select 27600,'027600','2002-056D','Micro-LabSat','Mu-Lab-Sat',67432,'In Earth orbit','','2002-12-21 00:00:00',100.78,788,803,98.67,'Sun-Synch' from dual union all
select 27601,'027601','2002-056E','H-2A F4 Stage 2','H-2A Stage 2',67432,'In Earth orbit','','2002-12-22 00:00:00',100.67,737,844,98.73,'Sun-Synch' from dual union all
select 27602,'027602','1989-006AC','deb Ariane V28','',62283,'In Earth orbit','','2002-12-10 00:00:00',641.36,854,35661,7.1,'GEO Transfer' from dual union all
select 27603,'027603','2002-057A','NSS 6','NSS 6',67433,'In Earth orbit','','2002-12-26 00:00:00',1431.77,35668,35736,.06,'Drift GEO' from dual union all
select 27604,'027604','2002-057B','Ariane H10-3 V156','Ariane H10-3 S/N T4115',67433,'In Earth orbit','','2002-12-25 00:00:00',660.45,297,37190,7.2,'GEO Transfer' from dual union all
select 27605,'027605','2002-058A','AATiS-OSCAR-49','Rubin-2',67434,'In Earth orbit','','2002-12-27 00:00:00',97.85,634,678,64.56,'Intermediate' from dual union all
select 27606,'027606','2002-058B','Latinsat-B','Latinsat-B',67434,'In Earth orbit','','2002-12-29 00:00:00',98.07,631,702,64.56,'Intermediate' from dual union all
select 27607,'027607','2002-058C','Saudi-OSCAR-50','Saudisat 1C',67434,'In Earth orbit','','2002-12-27 00:00:00',97.96,633,690,64.56,'Intermediate' from dual union all
select 27608,'027608','2002-058D','Unisat 2','Unisat 2',67434,'In Earth orbit','','2002-12-27 00:00:00',97.74,635,666,64.56,'Intermediate' from dual union all
select 27609,'027609','2002-058E','Trailblazer Dummy','TB Dummy',67434,'In Earth orbit','','2002-12-27 00:00:00',97.55,638,644,64.56,'Intermediate' from dual union all
select 2761,'002761','1967-038A','San Marco 2','San Marco B',15293,'Reentered','1967-10-14 00:00:00','1967-05-30 00:00:00',93.51,210,683,2.9,'Equatorial' from dual union all
select 27610,'027610','2002-058F','R-36M2 DS','15F183 51099801UP',67434,'In Earth orbit','','2002-12-27 00:00:00',100.91,600,1004,64.56,'Intermediate' from dual union all
select 27611,'027611','2002-058G','Dnepr plume shield','GDS',67434,'In Earth orbit','','2002-12-27 00:00:00',97.44,633,640,64.56,'Intermediate' from dual union all
select 27612,'027612','2002-058H','Latinsat-A','Latinsat-A',67434,'In Earth orbit','','2002-12-27 00:00:00',97.55,638,644,64.55,'Intermediate' from dual union all
select 27613,'027613','2002-059A','Kosmos-2393','Oko US-KS',67435,'Reentered','2013-12-21 00:00:00','2002-12-31 00:00:00',704.56,528,39173,62.85,'Molniya' from dual union all
select 27614,'027614','2002-059B','11S510','11S510',67435,'Reentered','2003-02-03 00:00:00','2002-12-31 00:00:00',91.77,225,498,62.8,'Intermediate' from dual union all
select 27615,'027615','2002-059C','BOZ','BOZ',67435,'Reentered','2003-01-15 00:00:00','2002-12-31 00:00:00',91.2,191,475,62.83,'Intermediate' from dual union all
select 27616,'027616','2002-059D','Blok-2BL','Blok-2BL No. 71612-623',67435,'Reentered','2018-05-05 00:00:00','2002-12-31 00:00:00',701.54,529,39022,62.84,'Molniya' from dual union all
select 27617,'027617','2002-060A','Kosmos-2394','Uragan No. 791',67436,'In Earth orbit','','2003-01-01 00:00:00',675.65,19126,19130,64.79,'Medium' from dual union all
select 27618,'027618','2002-060B','Kosmos-2395','Uragan No. 792',67436,'In Earth orbit','','2003-01-01 00:00:00',675.61,19124,19130,64.8,'Medium' from dual union all
select 27619,'027619','2002-060C','Kosmos-2396','Uragan No. 793',67436,'In Earth orbit','','2003-01-01 00:00:00',675.69,19123,19135,64.79,'Medium' from dual union all
select 2762,'002762','1967-039A','Kosmos-156','Meteor 11F614 No. 7',15294,'Reentered','1989-10-23 00:00:00','1967-06-04 00:00:00',96.91,582,640,81.19,'Intermediate' from dual union all
select 27620,'027620','2002-060D','Blok DM-2 No. 95L','11S861 No. 95L',67436,'In Earth orbit','','2003-01-01 00:00:00',674.6,19077,19126,64.79,'Medium' from dual union all
select 27621,'027621','2002-060E','SOZ','SOZ',67436,'Reentered','2009-05-09 00:00:00','2003-01-03 00:00:00',340.15,394,19133,64.84,'Highly Ellip' from dual union all
select 27622,'027622','2002-060F','SOZ','SOZ',67436,'Reentered','2009-08-11 00:00:00','2003-01-03 00:00:00',340.15,391,19135,64.76,'Highly Ellip' from dual union all
select 27623,'027623','2001-049NL','deb PSLV-C3 PS4','',67202,'Reentered','2003-01-07 00:00:00','2003-01-07 00:00:00',88.08,173,186,98.23,'Sun-Synch' from dual union all
select 27624,'027624','1968-055K','deb Star 37D','',16782,'In Earth orbit','','2003-01-25 00:00:00',140.38,741,4398,120.67,'Medium' from dual union all
select 27625,'027625','1988-098H','deb Ariane V26','',61347,'In Earth orbit','','2002-12-29 00:00:00',717.13,969,39354,4.16,'GEO Transfer' from dual union all
select 27626,'027626','1988-098J','deb Ariane V26','',61347,'In Earth orbit','','2002-11-30 00:00:00',522.21,1321,28885,4.04,'Highly Ellip' from dual union all
select 27627,'027627','1988-098K','deb Ariane V26','',61347,'Reentered','2003-07-19 00:00:00','2002-12-29 00:00:00',131.29,214,4144,4.03,'Medium' from dual union all
select 27628,'027628','1987-068CK','deb S5M','',60158,'In Earth orbit','','2002-12-25 00:00:00',103.52,906,944,82.52,'Intermediate' from dual union all
select 27629,'027629','1987-068CL','deb S5M','',60158,'In Earth orbit','','2002-12-28 00:00:00',101.73,817,865,82.29,'Intermediate' from dual union all
select 2763,'002763','1967-039B','8A92ME','8A92ME',15294,'Reentered','1991-10-27 00:00:00','1967-05-30 00:00:00',97.14,538,706,81.2,'Intermediate' from dual union all
select 27630,'027630','2002-061A','Shenzhou 4','Shenzhou 4 Fanhui Cang',67437,'Landed','2003-01-05 00:00:00','2002-12-31 00:00:00',91.21,331,337,42.4,'Intermediate' from dual union all
select 27631,'027631','2002-061B','CZ-2F Stage 2','CZ-2F Stage 2',67437,'Reentered','2003-01-09 00:00:00','2002-12-31 00:00:00',89.63,194,319,42.41,'Intermediate' from dual union all
select 27632,'027632','2002-062A','Nimiq 2','Nimiq 2',67438,'In Earth orbit','','2004-01-15 00:00:00',1436.09,35776,35797,.01,'Stationary' from dual union all
select 27633,'027633','2002-062B','Briz-M No. 88504','Briz-M 14S43 No. 88504',67438,'In Earth orbit','','2003-01-02 00:00:00',783.7,7872,35687,16.41,'Medium' from dual union all
select 27634,'027634','2002-061C','Shenzhou 4 Guidao Cang','Shenzhou 4 orbital modul',67437,'Reentered','2003-09-09 00:00:00','2003-01-10 00:00:00',91.66,348,363,42.42,'Intermediate' from dual union all
select 27635,'027635','1966-077AA','deb FTV 1352','',13484,'In Earth orbit','','2002-12-23 00:00:00',165.33,1427,5774,88.81,'Medium' from dual union all
select 27636,'027636','1966-077AB','deb FTV 1352','',13484,'In Earth orbit','','2003-01-03 00:00:00',165.27,1420,5776,88.74,'Medium' from dual union all
select 27637,'027637','1978-064L','deb Seasat','',45259,'Reentered','2002-12-31 00:00:00','2002-12-31 00:00:00',91.95,350,391,108,'Retrograde' from dual union all
select 27638,'027638','1984-069Y','deb US-A coolant','',57123,'In Earth orbit','','2003-01-03 00:00:00',103.63,908,953,65.05,'Intermediate' from dual;
insert into SATELLITE
select 27639,'027639','1987-068CM','deb S5M','',60158,'In Earth orbit','','2003-01-03 00:00:00',105.08,949,1048,82.52,'Intermediate' from dual union all
select 2764,'002764','1967-035B','Centaur D AC-12','Centaur D AC-12',15290,'Deep Space','1967-04-20 00:00:00','1967-04-20 00:00:00',23045.13,172143,351608,19.5,'Cislunar' from dual union all
select 27640,'027640','2003-001A','Coriolis','P98-2 Coriolis',67625,'In Earth orbit','','2003-01-10 00:00:00',97.07,391,845,98.72,'Sun-Synch' from dual union all
select 27641,'027641','2003-001B','Titan 23G Stage 2','LR91AJ5 23G-4',67625,'Reentered','2003-12-19 00:00:00','2003-01-10 00:00:00',95.68,273,830,98.71,'Sun-Synch' from dual union all
select 27642,'027642','2003-002A','Icesat','Icesat',67626,'Reentered','2010-08-30 00:00:00','2003-02-02 00:00:00',96.55,585,602,94.01,'Polar' from dual union all
select 27643,'027643','2003-002B','Chips','Chipsat',67626,'In Earth orbit','','2003-02-02 00:00:00',96.38,577,594,94.01,'Polar' from dual union all
select 27644,'027644','2003-002C','RH-DPAF Upper Cone','Reduced Height DPAF',67626,'Reentered','2013-12-12 00:00:00','2003-01-31 00:00:00',96.3,571,592,94.01,'Polar' from dual union all
select 27645,'027645','2003-002D','Delta 294','Delta SSPS AJ-10-118K',67626,'Reentered','2003-03-18 00:00:00','2003-02-02 00:00:00',91.68,235,479,97.53,'Sun-Synch' from dual union all
select 27646,'027646','2002-062C','Briz-M DTB','Briz-M torus tank',67438,'In Earth orbit','','2003-01-15 00:00:00',312.1,612,17115,49.51,'Highly Ellip' from dual union all
select 27647,'027647','2003-003A','Columbia (STS-107)','OV-102',67627,'Deorbited','2003-02-01 00:00:00','2003-01-23 00:00:00',90.03,270,282,39,'Intermediate' from dual union all
select 27648,'027648','1984-081F','deb Ariane','',57135,'In Earth orbit','','2003-01-30 00:00:00',685.91,910,37861,5.5,'GEO Transfer' from dual union all
select 27649,'027649','1989-006AD','deb Ariane V28','',62283,'In Earth orbit','','2003-01-25 00:00:00',645.25,1089,35625,7.91,'GEO Transfer' from dual union all
select 2765,'002765','1967-040A','Vela 4A','Vela 4A',15295,'Deep Space','1970-01-01 00:00:00','1967-09-29 00:00:00',2827.88,9193,110534,34.43,'Very High' from dual union all
select 27650,'027650','1989-006AE','deb Ariane V28','',62283,'In Earth orbit','','2003-02-02 00:00:00',629.49,395,35510,7.98,'GEO Transfer' from dual union all
select 27651,'027651','2003-004A','SORCE','SORCE',67628,'In Earth orbit','','2003-02-01 00:00:00',97.3,610,649,39.99,'Intermediate' from dual union all
select 27652,'027652','2003-004B','Orion 38','Orion 38',67628,'In Earth orbit','','2003-01-29 00:00:00',97.34,614,649,40.01,'Intermediate' from dual union all
select 27653,'027653','2001-049NQ','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2003-02-02 00:00:00',97.99,612,714,97.97,'Sun-Synch' from dual union all
select 27654,'027654','2001-049NR','deb PSLV-C3 PS4','',67202,'Reentered','2015-02-07 00:00:00','2003-01-27 00:00:00',96.58,519,671,97.11,'Sun-Synch' from dual union all
select 27655,'027655','2001-049NS','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2003-02-02 00:00:00',100.97,651,959,97.23,'Sun-Synch' from dual union all
select 27656,'027656','2001-049NT','deb PSLV-C3 PS4','',67202,'Reentered','2005-01-17 00:00:00','2003-02-01 00:00:00',95.6,499,597,97.82,'Sun-Synch' from dual union all
select 27657,'027657','2001-049NU','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2003-01-26 00:00:00',98.08,632,702,97.99,'Sun-Synch' from dual union all
select 27658,'027658','1971-024D','deb Isis 2','',24842,'In Earth orbit','','2003-01-31 00:00:00',113.57,1354,1427,88.17,'Polar' from dual union all
select 27659,'027659','1989-062P','deb Ariane V33','H10 deb',62339,'In Earth orbit','','2003-01-31 00:00:00',619.85,742,34665,7.05,'GEO Transfer' from dual union all
select 2766,'002766','1967-040B','Vela 4B','Vela 4B',15295,'Deep Space','1970-01-01 00:00:00','1967-12-05 00:00:00',6672.73,106999,115059,32.56,'Very High' from dual union all
select 27660,'027660','1989-062Q','deb Ariane V33','H10 deb',62339,'In Earth orbit','','2003-02-01 00:00:00',610.14,565,34338,7.3,'GEO Transfer' from dual union all
select 27661,'027661','1991-015M','deb Ariane V42','',64184,'In Earth orbit','','2003-01-29 00:00:00',497.36,422,28410,5.44,'Highly Ellip' from dual union all
select 27662,'027662','1991-015N','deb Ariane V42','',64184,'In Earth orbit','','2003-01-04 00:00:00',436.25,512,24840,6.06,'Highly Ellip' from dual union all
select 27663,'027663','2003-005A','GPS IIR-8','Navstar GPS SVN 56',67629,'In Earth orbit','','2003-02-01 00:00:00',720.7,20155,20344,55.06,'Medium' from dual union all
select 27664,'027664','2003-005B','XSS-10','XSS-10',67629,'In Earth orbit','','2003-02-02 00:00:00',97.93,516,804,39.75,'Intermediate' from dual union all
select 27665,'027665','2003-005C','Delta 295','Delta SSPS AJ-10-118K',67629,'In Earth orbit','','2003-02-01 00:00:00',100.86,793,805,39.58,'Intermediate' from dual union all
select 27666,'027666','2003-005D','Star 48B','Star 48B',67629,'Reentered','2005-02-08 00:00:00','2003-02-02 00:00:00',355.07,182,20282,39.01,'Highly Ellip' from dual union all
select 27667,'027667','1991-075C','Ariane V47 large debris','RCS 16 sq m',64244,'Reentered','2009-08-10 00:00:00','2003-01-30 00:00:00',445.13,203,25664,7.3,'Highly Ellip' from dual union all
select 27668,'027668','1991-075D','deb Ariane V47','',64244,'Reentered','2007-01-08 00:00:00','2003-01-29 00:00:00',398.79,253,22885,7.26,'Highly Ellip' from dual union all
select 27669,'027669','1991-075E','deb Ariane V47','',64244,'In Earth orbit','','2003-02-01 00:00:00',464.58,347,26637,7.58,'Highly Ellip' from dual union all
select 2767,'002767','1967-040C','ERS 18','ERS 18',15295,'Lost','','1967-06-29 00:00:00',2831.11,8989,110838,33.31,'Very High' from dual union all
select 27670,'027670','1991-075F','deb Ariane V47','',64244,'Reentered','2006-10-08 00:00:00','2003-01-28 00:00:00',385.39,328,22001,6.9,'Highly Ellip' from dual union all
select 27671,'027671','1991-075G','deb Ariane V47','',64244,'In Earth orbit','','2003-02-01 00:00:00',308.49,322,17168,7.09,'Highly Ellip' from dual union all
select 27672,'027672','1991-075H','deb Ariane V47','',64244,'Reentered','2005-07-14 00:00:00','2003-01-30 00:00:00',410.23,335,23486,8.02,'Highly Ellip' from dual union all
select 27673,'027673','1991-075J','deb Ariane V47','',64244,'In Earth orbit','','2003-02-02 00:00:00',502.13,637,28461,7.24,'Highly Ellip' from dual union all
select 27674,'027674','1991-075K','deb Ariane V47','',64244,'In Earth orbit','','2003-01-28 00:00:00',458.73,217,26433,7.29,'Highly Ellip' from dual union all
select 27675,'027675','1992-041J','deb Ariane V47','',64753,'Reentered','2003-05-18 00:00:00','2003-02-02 00:00:00',292.73,186,16265,7.16,'Highly Ellip' from dual union all
select 27676,'027676','1999-057MQ','CZ-4 debris','',66738,'In Earth orbit','','2003-01-31 00:00:00',245.28,279,12925,46.33,'Medium' from dual union all
select 27677,'027677','1999-057MR','CZ-4 debris','',66738,'In Earth orbit','','2003-01-27 00:00:00',99.86,693,811,98.19,'Sun-Synch' from dual union all
select 27678,'027678','1999-057MS','CZ-4 debris','',66738,'In Earth orbit','','2003-01-27 00:00:00',100.63,708,868,98.32,'Sun-Synch' from dual union all
select 27679,'027679','1994-029AEJ','deb HAPS','',65425,'In Earth orbit','','2003-02-02 00:00:00',101.96,584,1119,82.02,'Intermediate' from dual union all
select 2768,'002768','1967-040D','ERS 20','OV5-3',15295,'Lost','','1967-06-29 00:00:00',2831.05,8981,110844,33.28,'Very High' from dual union all
select 27680,'027680','1997-008E','DSP F18 Aperture Cover','',66088,'In Earth orbit','','2003-02-04 00:00:00',1446.96,35231,36767,1.43,'Drift GEO' from dual union all
select 27681,'027681','2003-006A','Progress M-47','Progress M No. 247',67630,'Deorbited','2003-08-28 00:00:00','2003-02-03 00:00:00',90.26,271,303,51.64,'Intermediate' from dual union all
select 27682,'027682','2003-006B','11S510','11S510',67630,'Reentered','2003-02-04 00:00:00','2003-02-03 00:00:00',88.43,177,217,51.65,'Intermediate' from dual union all
select 27683,'027683','2003-007A','INTELSAT 907','INTELSAT 907',67631,'In Earth orbit','','2003-02-25 00:00:00',1436.09,35777,35796,.1,'Stationary' from dual union all
select 27684,'027684','2003-007B','Ariane H10-3 V159','Ariane H10-3 S/N T4116',67631,'In Earth orbit','','2003-02-25 00:00:00',657.65,254,37092,7.04,'GEO Transfer' from dual union all
select 27685,'027685','1992-072M','deb Ariane V54','',64784,'In Earth orbit','','2003-03-08 00:00:00',446.04,391,25529,6.39,'Highly Ellip' from dual union all
select 27686,'027686','1993-054D','Star 48 yo weight','',65156,'Reentered','2014-03-19 00:00:00','2003-03-03 00:00:00',243.32,196,12868,34.76,'Medium' from dual union all
select 27687,'027687','1999-057MT','CZ-4 debris','',66738,'In Earth orbit','','2003-03-06 00:00:00',100.32,710,838,98.33,'Sun-Synch' from dual union all
select 27688,'027688','1999-057MU','CZ-4 debris','',66738,'In Earth orbit','','2003-03-07 00:00:00',100.29,700,845,98.39,'Sun-Synch' from dual union all
select 27689,'027689','2001-056H','Sep Motor Cover','Sep Motor Cover',67209,'In Earth orbit','','2003-03-09 00:00:00',108.62,1006,1320,99.84,'Sun-Synch' from dual union all
select 2769,'002769','1967-040E','ERS 27','OV5-1',15295,'Lost','','1967-08-09 00:00:00',2827.3,9110,110599,34.27,'Very High' from dual union all
select 27690,'027690','2001-056J','Sep Motor Cover','Sep Motor Cover',67209,'In Earth orbit','','2003-03-09 00:00:00',108.23,1006,1284,99.9,'Sun-Synch' from dual union all
select 27691,'027691','2003-008A','DSCS III A-3','DSCS III A-3',67632,'In Earth orbit','','2003-03-13 00:00:00',1436.27,35780,35800,1,'Stationary' from dual union all
select 27692,'027692','2003-008B','Delta 296','DCSS-4 F02',67632,'In Earth orbit','','2003-03-11 00:00:00',631.72,233,35787,25.5,'GEO Transfer' from dual union all
select 27693,'027693','2003-008C','IABS-10','IABS-10',67632,'In Earth orbit','','2003-03-15 00:00:00',1436.27,35780,35800,1,'Stationary' from dual union all
select 27694,'027694','2002-051F','deb Delta 293','deb DCSS-4 F01',67427,'Reentered','2003-05-26 00:00:00','2003-03-21 00:00:00',101.39,444,1205,24.18,'Equatorial' from dual union all
select 27695,'027695','2002-051G','deb Delta 293','deb DCSS-4 F01',67427,'Reentered','2006-11-29 00:00:00','2003-03-22 00:00:00',118.96,543,2726,22.36,'Equatorial' from dual union all
select 27696,'027696','2002-051H','deb Delta 293','deb DCSS-4 F01',67427,'Reentered','2003-11-06 00:00:00','2003-03-22 00:00:00',132.13,568,3863,20.91,'Medium' from dual union all
select 27697,'027697','2002-051J','deb Delta 293','deb DCSS-4 F01',67427,'Reentered','2006-01-28 00:00:00','2003-03-22 00:00:00',214.95,506,10510,17.35,'Medium' from dual union all
select 27698,'027698','2003-009A','JSE kougaku-1 gouki','IGS Optical-1',67633,'Reentered','2014-07-18 00:00:00','2003-03-30 00:00:00',94.36,485,491,97.31,'Sun-Synch' from dual union all
select 27699,'027699','2003-009B','JSE reda-1 gouki','IGS Radar-1',67633,'Reentered','2012-07-26 00:00:00','2003-04-01 00:00:00',94.44,489,494,97.31,'Sun-Synch' from dual union all
select 277,'000277','62 Lambda 2','Discoverer 39 camera door','',8193,'Reentered','1962-04-20 00:00:00','1962-04-19 00:00:00',91.58,201,503,73.5,'Intermediate' from dual union all
select 2770,'002770','1967-040F','Transtage 10','Transtage 10',15295,'In Earth orbit','','1967-04-28 00:00:00',2831.17,8588,111242,32.85,'Very High' from dual union all
select 27700,'027700','2003-009C','H-2A F5 Stage 2','H-2A Stage 2',67633,'Reentered','2012-06-03 00:00:00','2003-04-01 00:00:00',94.81,463,555,97.3,'Sun-Synch' from dual union all
select 27701,'027701','2003-009D','4/4D-LC Adapter','Adapter',67633,'Reentered','2004-08-09 00:00:00','2003-03-30 00:00:00',94.31,480,491,97.3,'Sun-Synch' from dual union all
select 27702,'027702','2003-009E','4/4D-LC Lower Fairing Half','Fairing Half',67633,'Reentered','2003-11-07 00:00:00','2003-03-30 00:00:00',94.31,481,490,97.31,'Sun-Synch' from dual union all
select 27703,'027703','2003-009F','4/4D-LC Lower Fairing Half','Fairing Half',67633,'Reentered','2003-11-29 00:00:00','2003-03-30 00:00:00',94.44,489,494,97.3,'Sun-Synch' from dual union all
select 27704,'027704','2003-010A','GPS IIR-9','Navstar GPS SVN 45',67634,'In Earth orbit','','2003-04-03 00:00:00',720.64,20063,20433,54.96,'Medium' from dual union all
select 27705,'027705','2003-010B','Delta 297','Delta/AJ-10-118K No. 297',67634,'Reentered','2003-06-29 00:00:00','2003-04-05 00:00:00',97.77,200,1104,30.81,'Intermediate' from dual union all
select 27706,'027706','2003-010C','Star 48B','Star 48B',67634,'Reentered','2016-09-25 00:00:00','2003-04-04 00:00:00',355.05,191,20272,39,'Highly Ellip' from dual union all
select 27707,'027707','2003-011A','Molniya-1T','Molniya-1T No. 101',67635,'Reentered','2013-05-27 00:00:00','2003-04-05 00:00:00',736.37,624,40644,62.87,'Molniya' from dual union all
select 27708,'027708','2003-011B','BOZ','BOZ',67635,'Reentered','2003-04-23 00:00:00','2003-04-05 00:00:00',92.41,183,603,62.85,'Intermediate' from dual union all
select 27709,'027709','2003-011C','11S510','11S510',67635,'Reentered','2003-05-13 00:00:00','2003-04-05 00:00:00',92.33,214,564,62.88,'Intermediate' from dual union all
select 2771,'002771','1967-038B','FW4S S/N 30202','FW4S',15293,'Reentered','1967-05-29 00:00:00','1967-05-03 00:00:00',93.37,211,668,2.9,'Equatorial' from dual union all
select 27710,'027710','2003-011D','Blok-2BL','Blok-2BL No. 76716-624',67635,'Reentered','2013-09-26 00:00:00','2003-04-04 00:00:00',732.18,623,40440,62.84,'Molniya' from dual union all
select 27711,'027711','2003-012A','Milstar 6','Milstar DFS-6',67636,'In Earth orbit','','2003-04-08 00:00:00',1433.3,35728,35736,4.45,'Inclined Drift' from dual union all
select 27712,'027712','2003-012B','Centaur TC-23','Centaur TC-23',67636,'In Earth orbit','','2003-04-08 00:00:00',1431.7,35696,35705,3.72,'Inclined Drift' from dual union all
select 27713,'027713','2003-003B','Columbia RCC panel 8 debris','',67627,'Reentered','2003-01-19 00:00:00','2003-01-17 00:00:00',90.12,272,289,39,'Intermediate' from dual union all
select 27714,'027714','2003-013A','Insat 3A','Insat 3A',67637,'In Earth orbit','','2004-05-28 00:00:00',1436.06,35776,35796,.03,'Stationary' from dual union all
select 27715,'027715','2003-013B','Galaxy 12','Galaxy XII',67637,'In Earth orbit','','2004-05-12 00:00:00',1438.03,35812,35837,.06,'Drift GEO' from dual union all
select 27716,'027716','2003-013C','EPS 514','EPS S/N 513R1',67637,'In Earth orbit','','2003-04-11 00:00:00',642.77,865,35722,1.9,'GEO Transfer' from dual union all
select 27717,'027717','2003-013D','Sylda 5 SY-09C','Sylda 5C',67637,'In Earth orbit','','2003-04-10 00:00:00',644.58,863,35817,2.02,'GEO Transfer' from dual union all
select 27718,'027718','2003-014A','Asiasat 4','Asiasat 4',67638,'In Earth orbit','','2004-05-12 00:00:00',1436.09,35784,35788,.01,'Stationary' from dual union all
select 27719,'027719','2003-014B','Centaur AC-205','Centaur IIIB SEC AC-205',67638,'Reentered','2004-02-12 00:00:00','2003-04-14 00:00:00',881.38,477,47669,26.97,'Highly Ellip' from dual union all
select 2772,'002772','1967-041A','Lunar Orbiter 4','Lunar Orbiter D',15296,'Deep Space','1967-05-07 00:00:00','1967-05-05 00:00:00',17620.27,334,435515,22.7,'Cislunar' from dual union all
select 27720,'027720','1965-004E','deb Altair 2','',11754,'In Earth orbit','','2003-04-05 00:00:00',119.17,705,2583,96.51,'Retrograde' from dual union all
select 27721,'027721','1965-082UT','deb Transtage 4','',11832,'In Earth orbit','','2003-04-14 00:00:00',100.53,686,882,31.64,'Intermediate' from dual union all
select 27722,'027722','1965-108BC','deb Transtage 8','',11858,'In Earth orbit','','2003-04-08 00:00:00',566.27,1148,31442,26.93,'Highly Ellip' from dual union all
select 27723,'027723','1965-108BD','deb Transtage 8','',11858,'In Earth orbit','','2003-04-12 00:00:00',524.32,645,29677,27.05,'Highly Ellip' from dual union all
select 27724,'027724','1966-077AC','deb FTV 1352','',13484,'In Earth orbit','','2003-04-08 00:00:00',166.13,1776,5489,88.92,'Medium' from dual union all
select 27725,'027725','1969-082LH','deb Agena D','',18349,'In Earth orbit','','2003-04-13 00:00:00',98.79,683,719,70.02,'Intermediate' from dual union all
select 27726,'027726','1975-004HS','deb Delta 107','',36451,'In Earth orbit','','2003-04-13 00:00:00',101.21,736,895,97.84,'Sun-Synch' from dual union all
select 27727,'027727','1976-077FS','deb Delta 126','',39029,'In Earth orbit','','2003-04-14 00:00:00',123.26,1487,2166,102.63,'Medium' from dual union all
select 27728,'027728','1976-077FT','deb Delta 126','',39029,'In Earth orbit','','2003-04-14 00:00:00',122.23,1458,2103,102.51,'Medium' from dual;
insert into SATELLITE
select 27729,'027729','1977-065GF','deb Delta 132','',42193,'In Earth orbit','','2003-04-14 00:00:00',104.68,550,1409,28.66,'Intermediate' from dual union all
select 2773,'002773','1967-042A','Ariel 3','UK-3',15297,'Reentered','1970-12-14 00:00:00','1967-06-07 00:00:00',95.6,496,600,80.17,'Intermediate' from dual union all
select 27730,'027730','1977-065GG','deb Delta 132','',42193,'Reentered','2011-09-09 00:00:00','2003-04-14 00:00:00',97.72,465,835,29.03,'Intermediate' from dual union all
select 27731,'027731','1978-087C','KM part','',45282,'In Earth orbit','','2003-04-12 00:00:00',268.04,280,14505,31.32,'Highly Ellip' from dual union all
select 27732,'027732','1979-104S','deb Ariane L01','',47588,'In Earth orbit','','2003-04-13 00:00:00',744.69,906,40768,15.57,'GEO Transfer' from dual union all
select 27733,'027733','1981-053MN','deb Kosmos-1275','',52576,'In Earth orbit','','2003-04-08 00:00:00',103.23,880,943,83.12,'Intermediate' from dual union all
select 27734,'027734','1981-053MP','deb Kosmos-1275','',52576,'In Earth orbit','','2003-04-14 00:00:00',104.26,891,1029,83.11,'Intermediate' from dual union all
select 27735,'027735','1981-053MQ','deb Kosmos-1275','',52576,'In Earth orbit','','2003-03-29 00:00:00',105.2,926,1082,83.05,'Intermediate' from dual union all
select 27736,'027736','1981-120H','deb S3M','',52643,'In Earth orbit','','2003-04-09 00:00:00',118.52,1571,1658,82.96,'Intermediate' from dual union all
select 27737,'027737','1981-120J','deb S3M','',52643,'In Earth orbit','','2003-04-10 00:00:00',119.2,1628,1662,83,'Intermediate' from dual union all
select 27738,'027738','1981-120K','deb S3M','',52643,'In Earth orbit','','2003-04-14 00:00:00',118.46,1567,1657,82.95,'Intermediate' from dual union all
select 27739,'027739','1984-069Z','deb US-A coolant','',57123,'In Earth orbit','','2003-04-16 00:00:00',103.64,905,957,65.04,'Intermediate' from dual union all
select 2774,'002774','1967-042B','Altair 2 X-258E6 ABL-143','X-258E6',15297,'Reentered','1970-12-05 00:00:00','1967-06-07 00:00:00',95.72,494,613,80.18,'Intermediate' from dual union all
select 27740,'027740','1984-114E','deb Ariane','',57168,'In Earth orbit','','2003-04-16 00:00:00',199.54,279,9587,6.78,'Medium' from dual union all
select 27741,'027741','1985-090C','deb Blok-E','',58313,'In Earth orbit','','2003-04-14 00:00:00',95.16,495,558,97.71,'Sun-Synch' from dual union all
select 27742,'027742','1987-020DV','deb Kosmos-1823','',60110,'In Earth orbit','','2003-04-16 00:00:00',108.22,1017,1272,74.15,'Intermediate' from dual union all
select 27743,'027743','1991-009CY','deb S3M','',64178,'In Earth orbit','','2003-04-15 00:00:00',118.07,1486,1703,74.07,'Intermediate' from dual union all
select 27744,'027744','1991-009CZ','deb S3M','',64178,'In Earth orbit','','2003-04-18 00:00:00',118.17,1459,1739,74.06,'Intermediate' from dual union all
select 27745,'027745','1991-009DA','deb S3M','',64178,'In Earth orbit','','2003-04-18 00:00:00',117,1432,1660,73.92,'Intermediate' from dual union all
select 27746,'027746','1991-009DB','deb S3M','',64178,'In Earth orbit','','2003-04-18 00:00:00',118.64,1481,1759,74.1,'Intermediate' from dual union all
select 27747,'027747','1991-009DC','deb S3M','',64178,'In Earth orbit','','2003-04-17 00:00:00',118.22,1529,1673,73.96,'Intermediate' from dual union all
select 27748,'027748','1991-009DD','deb S3M','',64178,'In Earth orbit','','2003-04-17 00:00:00',118.14,1456,1739,74.06,'Intermediate' from dual union all
select 27749,'027749','1992-093JL','deb 11S772','',64805,'In Earth orbit','','2003-04-17 00:00:00',102.57,787,974,71.11,'Intermediate' from dual union all
select 2775,'002775','1966-087D','ESSA 3 despin weight','',13494,'In Earth orbit','','1966-11-01 00:00:00',113.29,1282,1474,101.24,'Sun-Synch' from dual union all
select 27750,'027750','1992-093JM','deb 11S772','',64805,'In Earth orbit','','2003-04-16 00:00:00',102.23,773,956,71.17,'Intermediate' from dual union all
select 27751,'027751','1992-093JN','deb 11S772','',64805,'In Earth orbit','','2003-04-17 00:00:00',101.29,775,865,71.17,'Intermediate' from dual union all
select 27752,'027752','1993-014G','EKA 1 deb','EKA deb',65116,'In Earth orbit','','2003-04-10 00:00:00',101.13,679,946,75.75,'Intermediate' from dual union all
select 27753,'027753','1993-014H','deb Start','',65116,'Reentered','2012-09-24 00:00:00','2003-04-17 00:00:00',100.48,670,893,75.75,'Intermediate' from dual union all
select 27754,'027754','1993-042D','Star 48 yo weight','',65144,'Reentered','2006-12-10 00:00:00','2003-04-17 00:00:00',184.74,183,8551,34.71,'Medium' from dual union all
select 27755,'027755','1994-029AEK','deb HAPS','',65425,'In Earth orbit','','2003-04-15 00:00:00',110.97,608,1935,81.82,'Intermediate' from dual union all
select 27756,'027756','1994-029AEL','deb HAPS','',65425,'Reentered','2003-05-07 00:00:00','2003-04-18 00:00:00',94.35,436,539,81.92,'Intermediate' from dual union all
select 27757,'027757','1994-029AEM','deb HAPS','',65425,'Reentered','2005-12-11 00:00:00','2003-04-17 00:00:00',97.87,636,678,81.22,'Intermediate' from dual union all
select 27758,'027758','1994-029AEN','deb HAPS','',65425,'Reentered','2004-01-06 00:00:00','2003-04-10 00:00:00',95.28,478,587,81.42,'Intermediate' from dual union all
select 27759,'027759','1994-029AEP','deb HAPS','',65425,'Reentered','2005-05-31 00:00:00','2003-04-18 00:00:00',97.12,513,729,81.74,'Intermediate' from dual union all
select 2776,'002776','1967-027C','deb Kosmos-151','',15282,'Reentered','1967-11-28 00:00:00','1967-04-24 00:00:00',96.79,584,626,56.08,'Intermediate' from dual union all
select 27760,'027760','1994-077M','deb Kosmos-2297','Slow decay',65473,'In Earth orbit','','2003-04-18 00:00:00',100,698,818,70.98,'Intermediate' from dual union all
select 27761,'027761','1996-056D','Star 48 yo weight','',65944,'Reentered','2017-05-21 00:00:00','2003-04-20 00:00:00',246.5,207,13082,34.89,'Medium' from dual union all
select 27762,'027762','1999-057MV','CZ-4 debris','',66738,'In Earth orbit','','2003-04-19 00:00:00',102.24,726,1004,98.45,'Sun-Synch' from dual union all
select 27763,'027763','2000-015B','Fregat part','',66917,'In Earth orbit','','2003-04-18 00:00:00',261.53,394,13943,64.77,'Highly Ellip' from dual union all
select 27764,'027764','2000-025D','Star 48 yo weight','',66927,'In Earth orbit','','2003-04-19 00:00:00',292.88,193,16269,38.9,'Highly Ellip' from dual union all
select 27765,'027765','2000-040D','Star 48 yo weight','',66942,'In Earth orbit','','2003-04-19 00:00:00',272.39,174,14908,38.96,'Highly Ellip' from dual union all
select 27766,'027766','2001-004D','Star 48 yo weight','',67157,'Reentered','2005-10-27 00:00:00','2003-04-20 00:00:00',253.42,159,13616,38.81,'Highly Ellip' from dual union all
select 27767,'027767','2001-049NV','deb PSLV-C3 PS4','',67202,'Reentered','2003-05-21 00:00:00','2003-03-30 00:00:00',92.47,371,421,97.75,'Sun-Synch' from dual union all
select 27768,'027768','2002-054D','deb S3M','',67430,'In Earth orbit','','2003-04-21 00:00:00',99.09,686,744,98.23,'Sun-Synch' from dual union all
select 27769,'027769','2002-054E','deb S3M','',67430,'In Earth orbit','','2003-04-21 00:00:00',98.96,678,740,98.23,'Sun-Synch' from dual union all
select 2777,'002777','1967-034C','NNS O-12 despin weight','',15289,'In Earth orbit','','1967-07-03 00:00:00',104.33,845,1081,90.31,'Polar' from dual union all
select 27770,'027770','1985-037L','deb SOZ','',58260,'Reentered','2002-12-13 00:00:00','2003-01-16 00:00:00',247.68,642,12731,52.94,'Medium' from dual union all
select 27771,'027771','2002-051K','deb Delta 293','deb DCSS-4 F01',67427,'Reentered','2003-04-03 00:00:00','2003-04-01 00:00:00',96.36,435,734,23.3,'Equatorial' from dual union all
select 27772,'027772','1999-031F','deb Globalstar M052','',66712,'In Earth orbit','','2003-04-12 00:00:00',112.41,1310,1365,52.01,'Intermediate' from dual union all
select 27773,'027773','2003-005E','Star 48 yo weight','',67629,'Reentered','2008-05-30 00:00:00','2003-04-19 00:00:00',346.73,158,19784,38.92,'Highly Ellip' from dual union all
select 27774,'027774','2003-011E','deb BOZ','BOZ ullage motor',67635,'Reentered','2003-05-04 00:00:00','2003-04-22 00:00:00',91.35,205,477,62.87,'Intermediate' from dual union all
select 27775,'027775','2003-015A','Kosmos-2397','Oko-1 No. 7126',67639,'In Earth orbit','','2003-04-28 00:00:00',1442.35,35892,35926,2.32,'Drift GEO' from dual union all
select 27776,'027776','2003-015B','8S812','8S812',67639,'Reentered','2003-04-26 00:00:00','2003-04-25 00:00:00',87.51,148,155,51.65,'Intermediate' from dual union all
select 27777,'027777','2003-015C','Perekhodnik','Sredniy perekhodnik',67639,'Reentered','2003-04-25 00:00:00','2003-04-24 00:00:00',87.78,163,167,51.64,'Intermediate' from dual union all
select 27778,'027778','2003-015D','SOZ','SOZ',67639,'Reentered','2003-11-22 00:00:00','2003-05-07 00:00:00',634.67,257,35915,48,'GEO Transfer' from dual union all
select 27779,'027779','2003-015E','SOZ','SOZ',67639,'Reentered','2004-01-08 00:00:00','2003-05-04 00:00:00',635.84,311,35921,48.05,'GEO Transfer' from dual union all
select 2778,'002778','1967-034D','NNS O-12 despin weight','Transit high deb',15289,'In Earth orbit','','1967-05-16 00:00:00',108.74,1053,1284,90.28,'Polar' from dual union all
select 27780,'027780','2003-015F','Blok DM-2 No. 98L','11S861 No. 98L',67639,'In Earth orbit','','2003-05-18 00:00:00',1437.95,35552,36094,2.26,'Drift GEO' from dual union all
select 27781,'027781','2003-016A','Soyuz TMA-2','Soyuz 11F732 No. 212',67640,'Landed','2003-10-28 00:00:00','2003-04-28 00:00:00',90.69,261,355,51.64,'Intermediate' from dual union all
select 27782,'027782','2003-016B','11S510','11S510',67640,'Reentered','2003-04-28 00:00:00','2003-04-27 00:00:00',87.85,163,174,51.65,'Intermediate' from dual union all
select 27783,'027783','2003-017A','GALEX','GALEX',67641,'In Earth orbit','','2003-05-01 00:00:00',98.7,693,700,29,'Intermediate' from dual union all
select 27784,'027784','2003-017B','Orion 38','Orion 38',67641,'In Earth orbit','','2003-04-30 00:00:00',98.64,690,698,29,'Intermediate' from dual union all
select 27785,'027785','1968-063B','Agena D','Agena D',16790,'In Earth orbit','','1968-12-30 00:00:00',1435.4,31680,39866,9.9,'Near-synch' from dual union all
select 27786,'027786','1969-036B','Agena D','Agena D',18303,'In Earth orbit','','1969-12-31 00:00:00',1445.04,32672,39251,10.2,'Near-synch' from dual union all
select 27787,'027787','1970-069B','Agena D','Agena D',21546,'In Earth orbit','','1970-09-30 00:00:00',1441.94,31947,39855,10.3,'Near-synch' from dual union all
select 27788,'027788','1965-089E','deb Explorer 29','',11839,'In Earth orbit','','2003-04-30 00:00:00',118.22,1107,2095,59.39,'Intermediate' from dual union all
select 27789,'027789','1965-089F','deb Explorer 29','',11839,'In Earth orbit','','2003-05-01 00:00:00',119.37,1111,2195,59.37,'Intermediate' from dual union all
select 2779,'002779','1967-043A','OPS 4696','CORONA J-40',15298,'Reentered Attached','1967-07-13 00:00:00','1967-06-08 00:00:00',92.87,179,652,85.04,'Polar' from dual union all
select 27790,'027790','1965-108BE','deb Transtage 8','',11858,'In Earth orbit','','2003-04-18 00:00:00',525.62,733,29660,26.31,'Highly Ellip' from dual union all
select 27791,'027791','1965-108BF','deb Transtage 8','',11858,'In Earth orbit','','2003-05-02 00:00:00',548.11,809,30806,26.63,'Highly Ellip' from dual union all
select 27792,'027792','1965-108BG','deb Transtage 8','',11858,'In Earth orbit','','2003-05-04 00:00:00',524.05,477,29830,26.56,'Highly Ellip' from dual union all
select 27793,'027793','1965-108BH','deb Transtage 8','',11858,'Lost','','2003-02-19 00:00:00',485.94,553,27640,26.49,'Highly Ellip' from dual union all
select 27794,'027794','1965-108BJ','deb Transtage 8','',11858,'In Earth orbit','','2003-03-05 00:00:00',522.99,706,29543,26.47,'Highly Ellip' from dual union all
select 27795,'027795','1979-104T','deb Ariane L01','',47588,'In Earth orbit','','2003-04-26 00:00:00',717.46,707,39632,16.06,'GEO Transfer' from dual union all
select 27796,'027796','1981-031H','deb Kosmos-1261','',52554,'In Earth orbit','','2003-05-03 00:00:00',713.63,2730,37420,67.02,'GEO Transfer' from dual union all
select 27797,'027797','1981-053MR','deb Kosmos-1275','',52576,'In Earth orbit','','2003-04-20 00:00:00',107.09,940,1244,83.14,'Intermediate' from dual union all
select 27798,'027798','1981-071M','deb Kosmos-1285','',52594,'In Earth orbit','','2003-04-11 00:00:00',726.6,1967,38822,65.71,'GEO Transfer' from dual union all
select 27799,'027799','1981-071N','deb Kosmos-1285','',52594,'In Earth orbit','','2003-04-15 00:00:00',728.02,2612,38247,66.98,'GEO Transfer' from dual union all
select 278,'000278','62 Lambda 3','Discoverer 39 camera door','',8193,'Reentered','1962-04-21 00:00:00','1962-04-19 00:00:00',91.58,201,503,73.5,'Intermediate' from dual union all
select 2780,'002780','1967-043B','FANION III','P-11 No. 4408',15298,'Reentered','1993-03-14 00:00:00','1967-06-11 00:00:00',98.34,553,806,84.95,'Intermediate' from dual union all
select 27800,'027800','1981-071P','deb Kosmos-1285','',52594,'In Earth orbit','','2003-04-22 00:00:00',728.61,2976,37912,67.44,'GEO Transfer' from dual union all
select 27801,'027801','1981-071Q','deb Kosmos-1285','',52594,'In Earth orbit','','2003-05-03 00:00:00',727.33,2955,37869,67.58,'GEO Transfer' from dual union all
select 27802,'027802','1981-071R','deb Kosmos-1285','',52594,'In Earth orbit','','2003-04-15 00:00:00',728.76,2280,38615,66.42,'GEO Transfer' from dual union all
select 27803,'027803','1982-055BQ','deb Kosmos-1375','',54364,'In Earth orbit','','2003-05-04 00:00:00',103.47,903,942,65.82,'Intermediate' from dual union all
select 27804,'027804','1991-068AR','deb S5M','',64237,'In Earth orbit','','2003-05-04 00:00:00',114.55,1355,1515,82.65,'Intermediate' from dual union all
select 27805,'027805','1992-047L','deb SOZ','',64759,'In Earth orbit','','2002-12-21 00:00:00',291.72,799,15586,65.23,'Highly Ellip' from dual union all
select 27806,'027806','1999-057MW','CZ-4 debris','',66738,'In Earth orbit','','2003-04-28 00:00:00',100.71,718,867,98.3,'Sun-Synch' from dual union all
select 27807,'027807','2003-018A','GSAT-2','GSAT-2',67642,'In Earth orbit','','2009-07-02 00:00:00',1436.04,35770,35801,.01,'Stationary' from dual union all
select 27808,'027808','2003-018B','GSLV CS-2','GSLV-D2 12KRB stage',67642,'Reentered','2005-02-24 00:00:00','2003-05-16 00:00:00',620.16,155,35268,19.24,'GEO Transfer' from dual union all
select 27809,'027809','2003-019A','Hayabusa','MUSES C',67643,'Deep Space','2003-05-09 00:00:00','2003-05-09 00:00:00',1166.15,206,86362,31.08,'Earth Escape' from dual union all
select 2781,'002781','1967-044A','Kosmos-157','Zenit-2 No. 49',15299,'Landed','1967-05-20 00:00:00','1967-05-16 00:00:00',89.33,200,283,51.25,'Intermediate' from dual union all
select 27810,'027810','2003-019B','KM-V2','KM-V2',67643,'Deep Space','2003-05-09 00:00:00','2003-05-09 00:00:00',1166.15,206,86362,31.08,'Earth Escape' from dual union all
select 27811,'027811','2003-020A','Hellas Sat 2','Hellas Sat 2',67644,'In Earth orbit','','2003-05-21 00:00:00',1428.35,35486,35784,.07,'Drift GEO' from dual union all
select 27812,'027812','2003-020B','Centaur AV-002','Centaur AV-002',67644,'Reentered','2004-12-12 00:00:00','2003-05-22 00:00:00',1658.22,374,79685,17.51,'Very High' from dual union all
select 27813,'027813','2003-021A','Beidou Daohang Shiyan Wei. 3','san ke beidou daohang s.',67645,'In Earth orbit','','2003-06-02 00:00:00',1436.65,35760,35835,.29,'Drift GEO' from dual union all
select 27814,'027814','2003-021B','CZ-3A Stage 3','CZ-3A Stage 3',67645,'Reentered','2008-09-25 00:00:00','2003-05-31 00:00:00',749.72,202,41717,25.23,'GEO Transfer' from dual union all
select 27815,'027815','2002-042H','USERS REM','USERS REM',67418,'Landed','2003-05-29 00:00:00','2002-09-10 00:00:00',93.52,439,455,30.39,'Intermediate' from dual union all
select 27816,'027816','2003-022A','Mars Express','Mars Express',67646,'Deep Space','2003-06-03 00:00:00','2003-06-02 00:00:00',1616.74,340,104357,51.78,'Earth Escape' from dual union all
select 27817,'027817','2003-022B','Fregat','Fregat 14S44 No. 1005',67646,'Deep Space','2003-06-03 00:00:00','2003-06-02 00:00:00',1609.01,341,104066,51.79,'Earth Escape' from dual union all
select 27818,'027818','2003-023A','Kosmos-2398','Parus',67647,'In Earth orbit','','2003-06-07 00:00:00',104.94,970,1014,82.95,'Intermediate' from dual;
insert into SATELLITE
select 27819,'027819','2003-023B','S3M','S3M',67647,'In Earth orbit','','2003-06-06 00:00:00',104.83,970,1003,82.95,'Intermediate' from dual union all
select 2782,'002782','1967-044B','8A92E','8A92E',15299,'Reentered','1967-05-24 00:00:00','1967-05-15 00:00:00',89.44,232,262,51.28,'Intermediate' from dual union all
select 27820,'027820','2003-024A','AMC 9','Americom 9',67648,'In Earth orbit','','2003-11-05 00:00:00',1436.06,35779,35793,.04,'Stationary' from dual union all
select 27821,'027821','2003-024B','Briz-M DTB','Briz-M torus tank',67648,'In Earth orbit','','2003-06-09 00:00:00',276.42,296,15061,49.6,'Highly Ellip' from dual union all
select 27822,'027822','1990-037D','HST insulation debris','',63279,'Reentered','2003-09-05 00:00:00','2003-08-08 00:00:00',95.91,558,568,28.47,'Intermediate' from dual union all
select 27823,'027823','2003-025A','Progress M1-10','Progress M1 No. 259',67649,'Deorbited','2003-10-02 00:00:00','2003-06-09 00:00:00',90.7,304,313,51.63,'Intermediate' from dual union all
select 27824,'027824','2003-025B','11S510','11S510',67649,'Reentered','2003-06-10 00:00:00','2003-06-09 00:00:00',88.18,171,198,51.63,'Intermediate' from dual union all
select 27825,'027825','2003-026A','Thuraya 2','Thuraya 2',67650,'In Earth orbit','','2003-06-20 00:00:00',1435.98,35747,35822,6.26,'Inclined GEO' from dual union all
select 27826,'027826','2003-026B','Blok DM-SL No. 18L','Blok DM-SL',67650,'In Earth orbit','','2003-06-17 00:00:00',648.52,1158,35723,6.24,'GEO Transfer' from dual union all
select 27827,'027827','2003-027A','Spirit Rover','Mars Exploration Rover 2',67651,'Deep Space Attached','2003-06-11 00:00:00','2003-06-10 00:00:00',1586.8,185,103071,28.46,'Earth Escape' from dual union all
select 27828,'027828','2003-027B','Star 48B','Star 48B',67651,'Deep Space','2003-06-11 00:00:00','2003-06-10 00:00:00',1586.8,185,103071,28.46,'Earth Escape' from dual union all
select 27829,'027829','2003-027C','Delta 298','Delta SSPS AJ-10-118K',67651,'Reentered','2004-01-27 00:00:00','2003-06-19 00:00:00',135.13,161,4528,25.7,'Medium' from dual union all
select 2783,'002783','1967-044C','deb Kosmos-157','',15299,'Reentered','1967-05-13 00:00:00','1967-05-13 00:00:00',89.3,200,280,51.25,'Intermediate' from dual union all
select 27830,'027830','2003-028A','BSAT-2c','BSAT-2c',67652,'In Earth orbit','','2003-06-19 00:00:00',1426.21,35381,35805,.13,'Drift GEO' from dual union all
select 27831,'027831','2003-028B','Optus and Defense C1','Optus C1/D',67652,'In Earth orbit','','2003-06-21 00:00:00',1435.14,35699,35837,.08,'Drift GEO' from dual union all
select 27832,'027832','2003-028C','EPS 515','EPS S/N 515',67652,'In Earth orbit','','2003-06-19 00:00:00',640.83,602,35886,7.1,'GEO Transfer' from dual union all
select 27833,'027833','2003-028D','Sylda 5 SY-10C-E','Sylda 5C',67652,'In Earth orbit','','2003-06-20 00:00:00',638.41,602,35762,7.02,'GEO Transfer' from dual union all
select 27834,'027834','2003-029A','Molniya-3','Molniya-3 No. 65',67653,'Reentered','2013-06-15 00:00:00','2003-06-28 00:00:00',719.1,631,39789,62.9,'Molniya' from dual union all
select 27835,'027835','2003-029B','11S510','11S510',67653,'Reentered','2003-07-30 00:00:00','2003-06-26 00:00:00',91.88,208,526,62.85,'Intermediate' from dual union all
select 27836,'027836','2003-029C','BOZ','BOZ',67653,'Reentered','2003-07-13 00:00:00','2003-06-26 00:00:00',91.96,178,563,62.85,'Intermediate' from dual union all
select 27837,'027837','2003-029D','Blok-2BL','Blok-2BL',67653,'Reentered','2013-09-17 00:00:00','2003-06-27 00:00:00',733.14,630,40480,62.9,'Molniya' from dual union all
select 27838,'027838','2003-030A','Orbview-3','Orbview-3',67654,'Deorbited','2011-03-13 00:00:00','2003-07-03 00:00:00',92.47,364,428,97.28,'Sun-Synch' from dual union all
select 27839,'027839','2003-030B','Orion 38','Orion 38',67654,'Reentered','2004-02-22 00:00:00','2003-07-05 00:00:00',92.44,365,424,97.29,'Sun-Synch' from dual union all
select 2784,'002784','1967-044D','deb Kosmos-157','',15299,'Reentered','1967-05-12 00:00:00','1967-05-13 00:00:00',89.3,200,280,51.25,'Intermediate' from dual union all
select 27840,'027840','2003-031A','Briz-KM No. 72503','Briz-KM 14S45/NLS-1,2',67655,'In Earth orbit','','2003-07-08 00:00:00',100.11,697,831,98.72,'Sun-Synch' from dual union all
select 27841,'027841','2003-031B','Mimosa','Mimosa',67655,'Reentered','2011-12-11 00:00:00','2003-07-08 00:00:00',96.28,316,845,96.81,'Sun-Synch' from dual union all
select 27842,'027842','2003-031C','DTUSat','DTUSat',67655,'In Earth orbit','','2003-07-09 00:00:00',101.35,817,829,98.73,'Sun-Synch' from dual union all
select 27843,'027843','2003-031D','MOST','MOST',67655,'In Earth orbit','','2003-07-07 00:00:00',101.4,818,831,98.72,'Sun-Synch' from dual union all
select 27844,'027844','2003-031E','Cubesat-OSCAR-55','Cute-I',67655,'In Earth orbit','','2003-07-07 00:00:00',101.4,819,830,98.72,'Sun-Synch' from dual union all
select 27845,'027845','2003-031F','QuakeSat','QuakeSat',67655,'In Earth orbit','','2003-07-08 00:00:00',101.43,821,832,98.73,'Sun-Synch' from dual union all
select 27846,'027846','2003-031G','AAU-Cubesat','AAU-Cubesat',67655,'In Earth orbit','','2003-07-11 00:00:00',101.36,817,830,98.73,'Sun-Synch' from dual union all
select 27847,'027847','2003-031H','Can X-1','Can X-1',67655,'In Earth orbit','','2003-07-09 00:00:00',101.35,817,829,98.73,'Sun-Synch' from dual union all
select 27848,'027848','2003-031J','Cubesat-OSCAR-57','XI-IV',67655,'In Earth orbit','','2003-07-12 00:00:00',101.4,822,828,98.73,'Sun-Synch' from dual union all
select 27849,'027849','2003-032A','Opportunity Rover','Mars Exploration Rover 1',67656,'Deep Space Attached','2003-07-08 00:00:00','2003-07-08 00:00:00',1254.35,184,89996,29.4,'Earth Escape' from dual union all
select 2785,'002785','1967-044E','deb Kosmos-157','',15299,'Reentered','1967-05-14 00:00:00','1967-05-13 00:00:00',89.3,200,280,51.25,'Intermediate' from dual union all
select 27850,'027850','2003-032B','Star 48B','Star 48B',67656,'Deep Space','2003-07-08 00:00:00','2003-07-08 00:00:00',770.29,233,68660,29.46,'Earth Escape' from dual union all
select 27851,'027851','2003-032C','Delta 299','Delta SSPS AJ-10-118K',67656,'Reentered','2004-07-25 00:00:00','2003-07-17 00:00:00',157.58,164,6408,25.99,'Medium' from dual union all
select 27852,'027852','2003-033A','Rainbow 1','Rainbow 1',67657,'In Earth orbit','','2004-06-13 00:00:00',1436.09,35780,35793,.01,'Stationary' from dual union all
select 27853,'027853','2003-033B','Centaur AV-003','Centaur AV-003',67657,'In Earth orbit','','2003-07-18 00:00:00',689.95,3787,35186,17.61,'GEO Transfer' from dual union all
select 27854,'027854','2003-034A','Echostar IX','Echostar IX',67658,'In Earth orbit','','2003-08-16 00:00:00',1434.38,35555,35951,.05,'Drift GEO' from dual union all
select 27855,'027855','2003-034B','Blok DM-SL No. 10L','Blok DM-SL No. 10L',67658,'In Earth orbit','','2003-08-18 00:00:00',641.01,685,35812,.04,'GEO Transfer' from dual union all
select 27856,'027856','2003-035A','Kosmos-2399','Don',67659,'Exploded','2003-12-09 00:00:00','2003-08-19 00:00:00',89.69,202,317,64.93,'Intermediate' from dual union all
select 27857,'027857','2003-035B','11S510','11S510',67659,'Reentered','2003-08-16 00:00:00','2003-08-14 00:00:00',88.71,157,265,64.95,'Intermediate' from dual union all
select 27858,'027858','2003-036A','SCISAT-1','SCISAT-1',67660,'In Earth orbit','','2003-08-20 00:00:00',97.67,641,654,73.94,'Intermediate' from dual union all
select 27859,'027859','2003-036B','Orion 38','Orion 38',67660,'In Earth orbit','','2003-08-20 00:00:00',97.59,637,650,73.93,'Intermediate' from dual union all
select 2786,'002786','1965-082KN','deb Transtage 4','',11832,'Reentered','1979-02-12 00:00:00','1979-01-20 00:00:00',93.64,442,464,32.61,'Intermediate' from dual union all
select 27860,'027860','1964-049R','deb Kosmos-41','',10335,'Reentered','2004-03-18 00:00:00','2003-08-01 00:00:00',709.65,451,39502,65.13,'GEO Transfer' from dual union all
select 27861,'027861','1964-049S','deb Kosmos-41','',10335,'Reentered','2005-06-19 00:00:00','2003-06-23 00:00:00',721.7,1140,39408,66.43,'GEO Transfer' from dual union all
select 27862,'027862','1964-049T','deb Kosmos-41','',10335,'Reentered','2004-05-23 00:00:00','2003-08-16 00:00:00',719.44,493,39943,65.2,'GEO Transfer' from dual union all
select 27863,'027863','1964-049U','deb Kosmos-41','',10335,'Reentered','2007-07-31 00:00:00','2003-08-14 00:00:00',727.47,1651,39181,68.73,'GEO Transfer' from dual union all
select 27864,'027864','1965-108BK','deb Transtage 8','',11858,'In Earth orbit','','2003-07-12 00:00:00',540.8,864,30356,26.6,'Highly Ellip' from dual union all
select 27865,'027865','1965-108BL','deb Transtage 8','',11858,'In Earth orbit','','2003-03-17 00:00:00',553.88,924,31002,26.78,'Highly Ellip' from dual union all
select 27866,'027866','1965-108BM','deb Transtage 8','',11858,'In Earth orbit','','2003-03-01 00:00:00',552.91,1329,30545,26.86,'Highly Ellip' from dual union all
select 27867,'027867','1965-108BN','deb Transtage 8','',11858,'In Earth orbit','','2003-07-28 00:00:00',540.14,1067,30117,26.46,'Highly Ellip' from dual union all
select 27868,'027868','2003-037A','Kosmos-2400','Strela-3 No. 52',67661,'In Earth orbit','','2003-08-27 00:00:00',115.64,1468,1501,82.47,'Intermediate' from dual union all
select 27869,'027869','2003-037B','Kosmos-2401','Strela-3',67661,'In Earth orbit','','2003-08-26 00:00:00',115.6,1465,1501,82.47,'Intermediate' from dual union all
select 2787,'002787','1965-082KP','deb Transtage 4','',11832,'Reentered','1980-09-19 00:00:00','1980-09-17 00:00:00',91.11,308,350,31.7,'Intermediate' from dual union all
select 27870,'027870','2003-037C','S3M','S3M',67661,'In Earth orbit','','2003-08-27 00:00:00',115.68,1467,1506,82.47,'Intermediate' from dual union all
select 27871,'027871','2003-038A','Spitzer Space Telescope','SIRTF',67662,'Deep Space','2003-08-25 00:00:00','2003-08-25 00:00:00',174840.16,172,2084426,31.5,'Earth Escape' from dual union all
select 27872,'027872','2003-038B','Delta 300','Delta SSPS AJ-10-118K',67662,'Deep Space','2003-08-25 00:00:00','2003-08-25 00:00:00',2788.7,438,144453,31.49,'Earth Escape' from dual union all
select 27873,'027873','2003-039A','Progress M-48','Progress M No. 248',67663,'Deorbited','2004-01-28 00:00:00','2003-09-05 00:00:00',92.17,377,385,51.63,'Intermediate' from dual union all
select 27874,'027874','2003-039B','11S510','11S510',67663,'Reentered','2003-08-31 00:00:00','2003-08-30 00:00:00',87.93,164,180,51.64,'Intermediate' from dual union all
select 27875,'027875','2003-040A','DSCS III B-6','DSCS III B-6',67664,'In Earth orbit','','2003-08-30 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 27876,'027876','2003-040B','Delta 301','DCSS-4 F03',67664,'In Earth orbit','','2003-08-30 00:00:00',627.18,235,35551,25.5,'GEO Transfer' from dual union all
select 27877,'027877','2003-040C','IABS-7','IABS-7',67664,'In Earth orbit','','2003-08-30 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 27878,'027878','1965-108BP','deb Transtage 8','',11858,'In Earth orbit','','2003-05-30 00:00:00',511.52,608,29010,26.52,'Highly Ellip' from dual union all
select 27879,'027879','1965-108BQ','deb Transtage 8','',11858,'In Earth orbit','','2003-08-30 00:00:00',553.43,855,31047,26.61,'Highly Ellip' from dual union all
select 2788,'002788','1965-082KQ','deb Transtage 4','',11832,'Reentered','2003-03-08 00:00:00','1992-02-02 00:00:00',99.41,642,819,32.19,'Intermediate' from dual union all
select 27880,'027880','1966-025J','deb OV1-5','',13432,'In Earth orbit','','2003-09-06 00:00:00',101.4,799,851,144.6,'Retrograde' from dual union all
select 27881,'027881','1976-128C','deb Kosmos-887','',39080,'In Earth orbit','','2003-09-06 00:00:00',104.48,935,1006,82.94,'Intermediate' from dual union all
select 27882,'027882','1977-027F','deb Kosmos-903','deb Oko',42155,'In Earth orbit','','2003-08-15 00:00:00',728.12,4484,36380,66.68,'GEO Transfer' from dual union all
select 27883,'027883','1977-047F','deb Kosmos-917','deb Oko',42175,'In Earth orbit','','2003-07-14 00:00:00',712.54,3158,36938,66.57,'GEO Transfer' from dual union all
select 27884,'027884','1977-047G','deb Kosmos-917','deb Oko',42175,'In Earth orbit','','2003-08-27 00:00:00',714.13,3247,36928,66.71,'GEO Transfer' from dual union all
select 27885,'027885','1977-047H','deb Kosmos-917','deb Oko',42175,'In Earth orbit','','2003-06-29 00:00:00',705.47,3100,36646,66.32,'GEO Transfer' from dual union all
select 27886,'027886','1977-047J','deb Kosmos-917','deb Oko',42175,'In Earth orbit','','2003-07-13 00:00:00',740.65,4709,36768,68.95,'GEO Transfer' from dual union all
select 27887,'027887','1977-047K','deb Kosmos-917','deb Oko',42175,'In Earth orbit','','2003-09-06 00:00:00',727.57,4882,35955,68.67,'GEO Transfer' from dual union all
select 27888,'027888','1977-047L','deb Kosmos-917','deb Oko',42175,'In Earth orbit','','2003-05-11 00:00:00',712.96,3141,36976,66.61,'GEO Transfer' from dual union all
select 27889,'027889','1978-064M','deb Seasat','',45259,'Reentered','2006-03-28 00:00:00','2003-09-07 00:00:00',99.66,732,753,107.99,'Retrograde' from dual union all
select 2789,'002789','1965-082KR','deb Transtage 4','',11832,'In Earth orbit','','1992-02-02 00:00:00',101.35,682,964,32.28,'Intermediate' from dual union all
select 27890,'027890','1978-066G','deb Kosmos-1024','deb Oko',45261,'In Earth orbit','','2003-07-03 00:00:00',709.26,3401,36533,67.34,'GEO Transfer' from dual union all
select 27891,'027891','1978-083J','deb Kosmos-1030','deb Oko',45278,'Reentered','2004-04-30 00:00:00','2003-08-31 00:00:00',722.02,561,40003,62.66,'Molniya' from dual union all
select 27892,'027892','1978-083K','deb Kosmos-1030','deb Oko',45278,'In Earth orbit','','2003-07-13 00:00:00',718.59,1673,38722,64.36,'GEO Transfer' from dual union all
select 27893,'027893','1979-058N','deb Kosmos-1109','deb Oko',47542,'Reentered','2009-11-05 00:00:00','2003-08-09 00:00:00',710.62,2721,37280,68.14,'GEO Transfer' from dual union all
select 27894,'027894','1979-058P','deb Kosmos-1109','deb Oko',47542,'In Earth orbit','','2003-07-09 00:00:00',725.34,3435,37292,68.22,'GEO Transfer' from dual union all
select 27895,'027895','1980-050J','deb Kosmos-1188','deb Oko',50260,'In Earth orbit','','2003-07-14 00:00:00',738.91,5146,36246,69.63,'GEO Transfer' from dual union all
select 27896,'027896','1980-050K','deb Kosmos-1188','deb Oko',50260,'Reentered','2011-09-11 00:00:00','2003-07-08 00:00:00',702.07,3670,35907,67.85,'GEO Transfer' from dual union all
select 27897,'027897','1980-057K','deb Kosmos-1191','deb Oko',50267,'In Earth orbit','','2003-07-22 00:00:00',722.98,3130,37481,67.2,'GEO Transfer' from dual union all
select 27898,'027898','1980-057L','deb Kosmos-1191','deb Oko',50267,'In Earth orbit','','2003-09-08 00:00:00',721.27,3145,37382,67.04,'GEO Transfer' from dual union all
select 27899,'027899','1980-085J','deb Kosmos-1217','',50295,'In Earth orbit','','2003-09-05 00:00:00',710.96,1854,38164,67.25,'GEO Transfer' from dual union all
select 279,'000279','62 Lambda 4','Discoverer 39 camera door','',8193,'Reentered','1962-04-21 00:00:00','1962-04-19 00:00:00',91.58,201,503,73.5,'Intermediate' from dual union all
select 2790,'002790','1965-082KS','deb Transtage 4','',11832,'Reentered','1982-08-13 00:00:00','1967-05-12 00:00:00',100.06,736,787,32.55,'Intermediate' from dual union all
select 27900,'027900','1980-085K','deb Kosmos-1217','',50295,'In Earth orbit','','2003-08-06 00:00:00',709.14,1494,38434,66.51,'GEO Transfer' from dual union all
select 27901,'027901','1981-031J','deb Kosmos-1261','',52554,'In Earth orbit','','2003-06-25 00:00:00',715.98,3461,36805,68.17,'GEO Transfer' from dual union all
select 27902,'027902','1981-031K','deb Kosmos-1261','',52554,'In Earth orbit','','2003-09-03 00:00:00',715.69,3518,36733,68.23,'GEO Transfer' from dual union all
select 27903,'027903','1982-064E','deb Kosmos-1382','deb Oko',54373,'In Earth orbit','','2003-07-07 00:00:00',734.32,3437,37731,67.82,'GEO Transfer' from dual union all
select 27904,'027904','1982-064F','deb Kosmos-1382','deb Oko',54373,'In Earth orbit','','2003-07-20 00:00:00',726.45,3526,37256,68.1,'GEO Transfer' from dual union all
select 27905,'027905','1982-064G','deb Kosmos-1382','deb Oko',54373,'In Earth orbit','','2003-09-04 00:00:00',719.12,3716,36704,68.15,'GEO Transfer' from dual union all
select 27906,'027906','1983-070H','deb Kosmos-1481','',55827,'In Earth orbit','','2003-07-10 00:00:00',702.49,2564,37034,66.41,'GEO Transfer' from dual union all
select 27907,'027907','1983-070J','deb Kosmos-1481','',55827,'In Earth orbit','','2003-07-24 00:00:00',725.34,1606,39121,65.31,'GEO Transfer' from dual union all
select 27908,'027908','1984-069AA','deb US-A coolant','',57123,'In Earth orbit','','2003-08-31 00:00:00',103.62,895,964,65.03,'Intermediate' from dual;
insert into SATELLITE
select 27909,'027909','1984-069AB','deb US-A coolant','',57123,'In Earth orbit','','2003-08-29 00:00:00',103.59,893,963,65.04,'Intermediate' from dual union all
select 2791,'002791','1965-082KT','deb Transtage 4','',11832,'Reentered','1981-04-29 00:00:00','1981-04-10 00:00:00',94.31,463,508,31.89,'Intermediate' from dual union all
select 27910,'027910','1984-069AC','deb US-A coolant','',57123,'In Earth orbit','','2003-08-21 00:00:00',103.62,892,968,65.04,'Intermediate' from dual union all
select 27911,'027911','1984-069AD','deb US-A coolant','',57123,'In Earth orbit','','2003-08-12 00:00:00',103.65,896,966,65.04,'Intermediate' from dual union all
select 27912,'027912','1984-069AE','deb US-A coolant','',57123,'In Earth orbit','','2003-09-02 00:00:00',103.62,892,968,65.04,'Intermediate' from dual union all
select 27913,'027913','1984-069AF','deb US-A coolant','',57123,'In Earth orbit','','2003-08-26 00:00:00',103.63,895,966,65.04,'Intermediate' from dual union all
select 27914,'027914','1987-079X','deb SOZ','',60169,'Reentered','2003-09-16 00:00:00','2003-08-13 00:00:00',339.13,401,19061,65.43,'Highly Ellip' from dual union all
select 27915,'027915','1987-079Y','deb SOZ','',60169,'Reentered','2008-09-25 00:00:00','2003-09-07 00:00:00',344.43,503,19294,65.51,'Highly Ellip' from dual union all
select 27916,'027916','1987-079Z','deb SOZ','',60169,'Reentered','2008-02-24 00:00:00','2003-08-24 00:00:00',346.03,559,19339,65.31,'Highly Ellip' from dual union all
select 27917,'027917','1987-079AA','deb SOZ','',60169,'Reentered','2003-09-02 00:00:00','2003-08-08 00:00:00',338.48,355,19066,65.65,'Highly Ellip' from dual union all
select 27918,'027918','1987-079AB','deb SOZ','',60169,'In Earth orbit','','2003-09-05 00:00:00',348.61,717,19343,65.46,'Highly Ellip' from dual union all
select 27919,'027919','1987-079AC','deb SOZ','',60169,'In Earth orbit','','2003-09-06 00:00:00',347.13,679,19288,65.26,'Highly Ellip' from dual union all
select 2792,'002792','1965-082KU','deb Transtage 4','',11832,'Reentered','1971-12-26 00:00:00','1967-06-07 00:00:00',99.02,698,726,31.86,'Intermediate' from dual union all
select 27920,'027920','1987-079AD','deb SOZ','',60169,'Reentered','2011-05-26 00:00:00','2003-09-04 00:00:00',344.57,509,19296,65.51,'Highly Ellip' from dual union all
select 27921,'027921','1987-079AE','deb SOZ','',60169,'Reentered','2007-08-18 00:00:00','2003-08-24 00:00:00',344.83,500,19322,65.65,'Highly Ellip' from dual union all
select 27922,'027922','1987-079AF','deb SOZ','',60169,'Reentered','2005-01-01 00:00:00','2003-08-26 00:00:00',345.13,545,19295,65.55,'Highly Ellip' from dual union all
select 27923,'027923','1987-079AG','deb SOZ','',60169,'In Earth orbit','','2003-09-05 00:00:00',347.31,638,19340,65.48,'Highly Ellip' from dual union all
select 27924,'027924','1987-079AH','deb SOZ','',60169,'Reentered','2003-09-10 00:00:00','2003-08-12 00:00:00',342.7,406,19282,65.53,'Highly Ellip' from dual union all
select 27925,'027925','1987-079AJ','deb SOZ','',60169,'Reentered','2003-09-17 00:00:00','2003-08-12 00:00:00',342.46,410,19263,65.54,'Highly Ellip' from dual union all
select 27926,'027926','1987-079AK','deb SOZ','',60169,'Reentered','2003-09-15 00:00:00','2003-08-11 00:00:00',344.62,449,19360,65.61,'Highly Ellip' from dual union all
select 27927,'027927','1987-079AL','deb SOZ','',60169,'Reentered','2009-10-13 00:00:00','2003-09-06 00:00:00',335.64,425,18816,65.67,'Highly Ellip' from dual union all
select 27928,'027928','1987-079AM','deb SOZ','',60169,'Reentered','2003-09-09 00:00:00','2003-08-16 00:00:00',344.73,396,19420,65.06,'Highly Ellip' from dual union all
select 27929,'027929','1987-079AN','deb SOZ','',60169,'Reentered','2003-08-20 00:00:00','2003-07-22 00:00:00',360.57,489,20317,65.38,'Highly Ellip' from dual union all
select 2793,'002793','1963-014AP','deb West Ford','',9245,'Reentered','1971-02-25 00:00:00','1971-02-24 00:00:00',97.39,222,1046,83.62,'Intermediate' from dual union all
select 27930,'027930','1987-079AP','deb SOZ','',60169,'Reentered','2004-03-30 00:00:00','2003-08-08 00:00:00',347.23,527,19446,66.17,'Highly Ellip' from dual union all
select 27931,'027931','1987-079AQ','deb SOZ','',60169,'In Earth orbit','','2003-08-16 00:00:00',343.03,422,19286,65.58,'Highly Ellip' from dual union all
select 27932,'027932','1987-079AR','deb SOZ','',60169,'Reentered','2004-03-10 00:00:00','2003-08-25 00:00:00',340.98,582,18996,65.67,'Highly Ellip' from dual union all
select 27933,'027933','1987-079AS','deb SOZ','',60169,'Reentered','2010-07-25 00:00:00','2003-07-28 00:00:00',349.04,690,19397,65.69,'Highly Ellip' from dual union all
select 27934,'027934','1987-079AT','deb SOZ','',60169,'Reentered','2004-04-05 00:00:00','2003-09-06 00:00:00',336.15,358,18915,65.53,'Highly Ellip' from dual union all
select 27935,'027935','1987-079AU','deb SOZ','',60169,'Reentered','2003-10-03 00:00:00','2003-08-12 00:00:00',344.53,439,19364,65.62,'Highly Ellip' from dual union all
select 27936,'027936','1987-079AV','deb SOZ','',60169,'Reentered','2004-05-20 00:00:00','2003-08-26 00:00:00',345.46,595,19267,65.61,'Highly Ellip' from dual union all
select 27937,'027937','2003-041A','USA 171','ORION 5',67665,'In Earth orbit','','2003-09-10 00:00:00',1436.27,35780,35800,2,'Stationary' from dual union all
select 27938,'027938','2003-041B','Centaur TC-20','Centaur TC-20',67665,'In Earth orbit','','2003-09-10 00:00:00',1436.27,35780,35800,2,'Stationary' from dual union all
select 27939,'027939','2003-042A','Mozhaets-4','Mozhaets 4',67666,'In Earth orbit','','2003-09-28 00:00:00',98.44,676,693,98.2,'Sun-Synch' from dual union all
select 2794,'002794','1965-082KV','deb Transtage 4','',11832,'Reentered','1981-02-03 00:00:00','1981-01-31 00:00:00',91.23,331,339,31.43,'Intermediate' from dual union all
select 27940,'027940','2003-042B','S3M','S3M',67666,'In Earth orbit','','2003-09-28 00:00:00',98.42,675,692,98.2,'Sun-Synch' from dual union all
select 27941,'027941','2003-042C','NigeriaSat-1','DMC NigeriaSat-1',67666,'In Earth orbit','','2003-09-28 00:00:00',98.45,676,694,98.2,'Sun-Synch' from dual union all
select 27942,'027942','2003-042D','UK-DMC','DMC BNSCSat-1',67666,'In Earth orbit','','2003-09-28 00:00:00',98.45,676,694,98.2,'Sun-Synch' from dual union all
select 27943,'027943','2003-042E','BILSAT-1','DMC BILSAT-1',67666,'In Earth orbit','','2003-09-28 00:00:00',98.45,677,693,98.2,'Sun-Synch' from dual union all
select 27944,'027944','2003-042F','Larets','Larets',67666,'In Earth orbit','','2003-09-28 00:00:00',98.43,674,694,98.21,'Sun-Synch' from dual union all
select 27945,'027945','2003-042G','STSat-1','KAISTSAT-4',67666,'In Earth orbit','','2003-09-28 00:00:00',98.47,676,695,98.2,'Sun-Synch' from dual union all
select 27946,'027946','2003-043B','EPS 516','EPS S/N 516G/ACU',67667,'In Earth orbit','','2003-09-27 00:00:00',640.7,671,35810,6.97,'GEO Transfer' from dual union all
select 27947,'027947','1999-035D','FUSE insulation','',66716,'Reentered','2004-11-16 00:00:00','2004-10-05 00:00:00',98.11,628,709,24.98,'Equatorial' from dual union all
select 27948,'027948','2003-043A','Eurobird 3','e-Bird',67667,'In Earth orbit','','2004-06-19 00:00:00',1436.01,35782,35788,.01,'Stationary' from dual union all
select 27949,'027949','2003-043C','SMART-1','SMART-1',67667,'Deep Space','2004-07-09 00:00:00','2004-07-09 00:00:00',4682.62,21223,151451,6.85,'Deep Space' from dual union all
select 2795,'002795','1965-082KW','deb Transtage 4','',11832,'Reentered','1978-09-14 00:00:00','1978-08-22 00:00:00',92.64,399,409,32.66,'Intermediate' from dual union all
select 27950,'027950','2003-043D','Sylda 5 SY-13A','Sylda 5A',67667,'In Earth orbit','','2003-09-27 00:00:00',638.18,654,35698,6.91,'GEO Transfer' from dual union all
select 27951,'027951','2003-043E','Insat 3E','Insat 3E',67667,'In Earth orbit','','2004-08-07 00:00:00',1436.04,35759,35811,.04,'Stationary' from dual union all
select 27952,'027952','2003-043F','ACU/ACY adapter','ACU/ACY adapter',67667,'In Earth orbit','','2003-09-27 00:00:00',638.63,656,35719,6.92,'GEO Transfer' from dual union all
select 27953,'027953','2003-044B','Blok DM-SL No. 12L','Blok DM-SL No. 12L',67668,'In Earth orbit','','2003-10-03 00:00:00',673.13,2398,35731,.04,'GEO Transfer' from dual union all
select 27954,'027954','2003-044A','Galaxy 13','Galaxy XIII/Horizons 1',67668,'In Earth orbit','','2004-07-06 00:00:00',1436.09,35777,35796,.01,'Stationary' from dual union all
select 27955,'027955','2003-005F','deb Delta','',67629,'Reentered','2003-12-10 00:00:00','2003-10-05 00:00:00',98.88,563,848,39.58,'Intermediate' from dual union all
select 27956,'027956','1978-064N','deb Seasat','',45259,'Reentered','2003-10-03 00:00:00','2003-10-02 00:00:00',92.64,390,418,107.88,'Retrograde' from dual union all
select 27957,'027957','1987-079AW','deb SOZ','',60169,'Reentered','2005-02-08 00:00:00','2003-08-26 00:00:00',346.38,520,19400,65.55,'Highly Ellip' from dual union all
select 27958,'027958','1987-079AX','deb SOZ','',60169,'In Earth orbit','','2003-10-04 00:00:00',343.37,721,19009,65.43,'Highly Ellip' from dual union all
select 27959,'027959','1987-079AY','deb SOZ','',60169,'In Earth orbit','','2003-10-04 00:00:00',348.72,690,19377,65.69,'Highly Ellip' from dual union all
select 2796,'002796','1963-014AQ','deb West Ford','RCS 0.21',9245,'In Earth orbit','','1992-01-31 00:00:00',165.95,2857,4394,87.19,'Medium' from dual union all
select 27960,'027960','1987-079AZ','deb SOZ','',60169,'Reentered','2003-11-17 00:00:00','2003-08-18 00:00:00',344.27,465,19321,65.58,'Highly Ellip' from dual union all
select 27961,'027961','1987-079BA','deb SOZ','',60169,'Reentered','2004-09-09 00:00:00','2003-08-26 00:00:00',344.76,505,19312,65.69,'Highly Ellip' from dual union all
select 27962,'027962','1987-079BB','deb SOZ','',60169,'Reentered','2014-11-20 00:00:00','2003-10-04 00:00:00',347.59,665,19331,65.58,'Highly Ellip' from dual union all
select 27963,'027963','1987-079BC','deb SOZ','',60169,'In Earth orbit','','2003-10-04 00:00:00',342.45,452,19220,65.22,'Highly Ellip' from dual union all
select 27964,'027964','1988-085J','deb SOZ','',61334,'Reentered','2003-11-22 00:00:00','2003-09-17 00:00:00',338.04,693,18700,65.72,'Highly Ellip' from dual union all
select 27965,'027965','1988-085K','deb SOZ','',61334,'In Earth orbit','','2003-10-04 00:00:00',335.28,649,18568,65.31,'Highly Ellip' from dual union all
select 27966,'027966','1988-085L','deb SOZ','',61334,'Reentered','2003-11-12 00:00:00','2003-09-23 00:00:00',337.09,631,18702,65.66,'Highly Ellip' from dual union all
select 27967,'027967','1988-085M','deb SOZ','',61334,'In Earth orbit','','2003-10-04 00:00:00',339,627,18826,65.79,'Highly Ellip' from dual union all
select 27968,'027968','1988-085N','deb SOZ','',61334,'Reentered','2003-11-18 00:00:00','2003-09-18 00:00:00',336.86,735,18582,65.32,'Highly Ellip' from dual union all
select 27969,'027969','1988-085P','deb SOZ','',61334,'Reentered','2003-11-25 00:00:00','2003-09-29 00:00:00',337.83,750,18630,65.37,'Highly Ellip' from dual union all
select 2797,'002797','1967-027D','deb Kosmos-151','',15282,'Reentered','1970-10-12 00:00:00','1969-01-27 00:00:00',96.05,549,590,56.04,'Intermediate' from dual union all
select 27970,'027970','1988-085Q','deb SOZ','',61334,'Reentered','2003-11-04 00:00:00','2003-09-16 00:00:00',335.09,586,18620,65.7,'Highly Ellip' from dual union all
select 27971,'027971','1988-085R','deb SOZ','',61334,'Reentered','2003-11-29 00:00:00','2003-10-02 00:00:00',330.97,520,18422,65.29,'Highly Ellip' from dual union all
select 27972,'027972','1988-085S','deb SOZ','',61334,'Reentered','2003-11-25 00:00:00','2003-09-26 00:00:00',338.86,758,18687,65.44,'Highly Ellip' from dual union all
select 27973,'027973','1988-085T','deb SOZ','',61334,'Reentered','2003-11-27 00:00:00','2003-10-01 00:00:00',330.01,507,18375,65.31,'Highly Ellip' from dual union all
select 27974,'027974','1988-085U','deb SOZ','',61334,'Reentered','2003-11-20 00:00:00','2003-09-18 00:00:00',336.51,722,18573,65.25,'Highly Ellip' from dual union all
select 27975,'027975','1988-085V','deb SOZ','',61334,'Reentered','2003-10-27 00:00:00','2003-09-16 00:00:00',328.38,435,18342,65.6,'Highly Ellip' from dual union all
select 27976,'027976','1988-085W','deb SOZ','',61334,'In Earth orbit','','2003-10-04 00:00:00',337.86,607,18775,65.59,'Highly Ellip' from dual union all
select 27977,'027977','1988-085X','deb SOZ','',61334,'Reentered','2003-12-30 00:00:00','2003-10-04 00:00:00',338.95,757,18694,65.49,'Highly Ellip' from dual union all
select 27978,'027978','1988-085Y','deb SOZ','',61334,'In Earth orbit','','2003-10-04 00:00:00',337.99,695,18695,65.46,'Highly Ellip' from dual union all
select 27979,'027979','1988-085Z','deb SOZ','',61334,'Reentered','2003-11-06 00:00:00','2003-09-17 00:00:00',336.71,638,18670,65.67,'Highly Ellip' from dual union all
select 2798,'002798','1964-083J','deb VE-5','',10369,'Reentered','1979-01-13 00:00:00','1966-12-16 00:00:00',106.28,1022,1086,89.95,'Polar' from dual union all
select 27980,'027980','1988-085AA','deb SOZ','',61334,'Reentered','2003-10-21 00:00:00','2003-09-10 00:00:00',331.37,487,18482,65.44,'Highly Ellip' from dual union all
select 27981,'027981','1988-085AB','deb SOZ','',61334,'Reentered','2003-11-30 00:00:00','2003-10-04 00:00:00',336.87,660,18659,65.56,'Highly Ellip' from dual union all
select 27982,'027982','1988-085AC','deb SOZ','',61334,'Reentered','2003-10-24 00:00:00','2003-10-02 00:00:00',331.97,556,18451,65.2,'Highly Ellip' from dual union all
select 27983,'027983','1988-085AD','deb SOZ','',61334,'Reentered','2003-11-29 00:00:00','2003-10-04 00:00:00',336.6,620,18682,65.34,'Highly Ellip' from dual union all
select 27984,'027984','1988-085AE','deb SOZ','',61334,'Reentered','2003-11-20 00:00:00','2003-09-20 00:00:00',337.34,704,18645,65.64,'Highly Ellip' from dual union all
select 27985,'027985','1988-085AF','deb SOZ','',61334,'Reentered','2003-11-05 00:00:00','2003-09-24 00:00:00',338.54,590,18835,65.68,'Highly Ellip' from dual union all
select 27986,'027986','1988-085AG','deb SOZ','',61334,'Reentered','2004-01-27 00:00:00','2003-10-04 00:00:00',339.16,637,18826,65.58,'Highly Ellip' from dual union all
select 27987,'027987','1988-085AH','deb SOZ','',61334,'Reentered','2003-11-09 00:00:00','2003-10-01 00:00:00',336.67,564,18742,65.64,'Highly Ellip' from dual union all
select 27988,'027988','1988-085AJ','deb SOZ','',61334,'Reentered','2003-10-16 00:00:00','2003-09-09 00:00:00',334.62,551,18625,65.6,'Highly Ellip' from dual union all
select 27989,'027989','1988-085AK','deb SOZ','',61334,'Reentered','2004-08-30 00:00:00','2003-09-19 00:00:00',333.82,665,18460,65.52,'Highly Ellip' from dual union all
select 2799,'002799','1965-082KX','deb Transtage 4','',11832,'In Earth orbit','','1992-02-02 00:00:00',99.07,680,749,32.05,'Intermediate' from dual union all
select 27990,'027990','1988-085AL','deb SOZ','',61334,'Reentered','2003-11-11 00:00:00','2003-09-30 00:00:00',337.75,579,18796,65.94,'Highly Ellip' from dual union all
select 27991,'027991','1988-085AM','deb SOZ','',61334,'Reentered','2003-11-08 00:00:00','2003-09-26 00:00:00',336.54,610,18688,65.48,'Highly Ellip' from dual union all
select 27992,'027992','1988-085AN','deb SOZ','',61334,'In Earth orbit','','2003-10-04 00:00:00',330.04,636,18248,65.22,'Highly Ellip' from dual union all
select 27993,'027993','1988-085AP','deb SOZ','',61334,'Reentered','2003-11-11 00:00:00','2003-09-17 00:00:00',329.06,411,18410,65.21,'Highly Ellip' from dual union all
select 27994,'027994','1988-085AQ','deb SOZ','',61334,'Reentered','2003-11-08 00:00:00','2003-09-25 00:00:00',326.53,561,18097,65.44,'Highly Ellip' from dual union all
select 27995,'027995','1988-085AR','deb SOZ','',61334,'Reentered','2003-11-09 00:00:00','2003-09-10 00:00:00',334.48,754,18413,65.15,'Highly Ellip' from dual union all
select 27996,'027996','1988-085AS','deb SOZ','',61334,'Reentered','2003-11-06 00:00:00','2003-09-16 00:00:00',327.18,525,18175,65.4,'Highly Ellip' from dual union all
select 27997,'027997','1988-085AT','deb SOZ','',61334,'Reentered','2003-11-22 00:00:00','2003-10-04 00:00:00',338.94,619,18831,65.6,'Highly Ellip' from dual union all
select 27998,'027998','1988-085AU','deb SOZ','',61334,'In Earth orbit','','2003-10-04 00:00:00',336.6,636,18666,65.47,'Highly Ellip' from dual union all
select 27999,'027999','1988-085AV','deb SOZ','',61334,'Reentered','2003-11-13 00:00:00','2003-09-20 00:00:00',339.02,723,18731,65.62,'Highly Ellip' from dual;
insert into SATELLITE
select 28,'000028','60 Beta 1','Altair','Altair X-248',6318,'Reentered','1991-07-02 00:00:00','1960-05-05 00:00:00',99.21,690,752,48.41,'Intermediate' from dual union all
select 280,'000280','62 Mu 1','Ranger 4','Ranger 4',8194,'Deep Space','1962-04-26 00:00:00','',null,null,null,null,'' from dual union all
select 2800,'002800','1966-070D','OV3-3 despin weight','',13477,'In Earth orbit','','1967-05-14 00:00:00',138.91,426,4588,81.45,'Medium' from dual union all
select 28000,'028000','1988-085AW','deb SOZ','',61334,'Reentered','2003-11-23 00:00:00','2003-09-30 00:00:00',331.28,472,18491,65.45,'Highly Ellip' from dual union all
select 28001,'028001','1988-085AX','deb SOZ','',61334,'Reentered','2003-10-15 00:00:00','2003-09-20 00:00:00',335.08,912,18293,65.27,'Highly Ellip' from dual union all
select 28002,'028002','1988-085AY','deb SOZ','',61334,'Reentered','2003-11-08 00:00:00','2003-10-01 00:00:00',329.38,440,18402,65.33,'Highly Ellip' from dual union all
select 28003,'028003','1988-085AZ','deb SOZ','',61334,'Reentered','2003-11-07 00:00:00','2003-09-24 00:00:00',336.65,652,18652,65.51,'Highly Ellip' from dual union all
select 28004,'028004','1988-085BA','deb SOZ','',61334,'In Earth orbit','','2003-10-04 00:00:00',336.41,714,18576,65.46,'Highly Ellip' from dual union all
select 28005,'028005','1988-085BB','deb SOZ','',61334,'Reentered','2003-10-18 00:00:00','2003-09-16 00:00:00',335.96,640,18620,65.43,'Highly Ellip' from dual union all
select 28006,'028006','1988-085BC','deb SOZ','',61334,'Reentered','2003-10-14 00:00:00','2003-09-24 00:00:00',334.84,525,18665,65.46,'Highly Ellip' from dual union all
select 28007,'028007','1988-085BD','deb SOZ','',61334,'Reentered','2003-10-27 00:00:00','2003-09-18 00:00:00',331.83,601,18397,65.51,'Highly Ellip' from dual union all
select 28008,'028008','1988-085BE','deb SOZ','',61334,'Reentered','2003-10-30 00:00:00','2003-09-09 00:00:00',332.88,566,18499,65.26,'Highly Ellip' from dual union all
select 28009,'028009','1988-085BF','deb SOZ','',61334,'Reentered','2003-10-25 00:00:00','2003-09-21 00:00:00',330.93,449,18492,65.48,'Highly Ellip' from dual union all
select 2801,'002801','1967-045A','Kosmos-158','Tsiklon GVM',15300,'In Earth orbit','','1967-07-03 00:00:00',100.43,736,821,74.04,'Intermediate' from dual union all
select 28010,'028010','1988-085BG','deb SOZ','',61334,'Reentered','2003-10-25 00:00:00','2003-09-17 00:00:00',331.76,504,18490,65.25,'Highly Ellip' from dual union all
select 28011,'028011','1988-085BH','deb SOZ','',61334,'Reentered','2003-11-24 00:00:00','2003-10-01 00:00:00',336.29,667,18615,65.33,'Highly Ellip' from dual union all
select 28012,'028012','1988-085BJ','deb SOZ','',61334,'Reentered','2003-11-17 00:00:00','2003-09-18 00:00:00',337.09,796,18537,65.29,'Highly Ellip' from dual union all
select 28013,'028013','1988-085BK','deb SOZ','',61334,'Reentered','2003-10-16 00:00:00','2003-09-25 00:00:00',334.92,548,18647,65.53,'Highly Ellip' from dual union all
select 28014,'028014','1988-085BL','deb SOZ','',61334,'Reentered','2003-10-28 00:00:00','2003-09-29 00:00:00',327.88,412,18333,65.36,'Highly Ellip' from dual union all
select 28015,'028015','1988-085BM','deb SOZ','',61334,'Reentered','2003-10-17 00:00:00','2003-09-17 00:00:00',336.62,662,18641,65.71,'Highly Ellip' from dual union all
select 28016,'028016','1988-085BN','deb SOZ','',61334,'Reentered','2003-11-16 00:00:00','2003-09-13 00:00:00',338.18,741,18660,65.41,'Highly Ellip' from dual union all
select 28017,'028017','1988-085BP','deb SOZ','',61334,'Reentered','2003-10-21 00:00:00','2003-09-16 00:00:00',331.83,483,18514,65.32,'Highly Ellip' from dual union all
select 28018,'028018','1988-085BQ','deb SOZ','',61334,'Reentered','2003-10-18 00:00:00','2003-09-17 00:00:00',330.29,478,18422,65.32,'Highly Ellip' from dual union all
select 28019,'028019','1988-085BR','deb SOZ','',61334,'Reentered','2003-11-02 00:00:00','2003-10-01 00:00:00',329.38,506,18335,65.34,'Highly Ellip' from dual union all
select 2802,'002802','1967-045B','S3M','S3M',15300,'In Earth orbit','','1967-06-14 00:00:00',100.64,733,845,74.01,'Intermediate' from dual union all
select 28020,'028020','1988-085BS','deb SOZ','',61334,'Reentered','2003-10-26 00:00:00','2003-09-24 00:00:00',341.88,1222,18414,65.59,'Highly Ellip' from dual union all
select 28021,'028021','1988-085BT','deb SOZ','',61334,'Reentered','2003-11-25 00:00:00','2003-09-18 00:00:00',337.31,790,18557,65.36,'Highly Ellip' from dual union all
select 28022,'028022','1988-085BU','deb SOZ','',61334,'Reentered','2003-10-27 00:00:00','2003-09-16 00:00:00',331.81,418,18578,65.46,'Highly Ellip' from dual union all
select 28023,'028023','1988-085BV','deb SOZ','',61334,'Reentered','2003-10-29 00:00:00','2003-09-16 00:00:00',331.98,579,18429,65.2,'Highly Ellip' from dual union all
select 28024,'028024','1988-085BW','deb SOZ','',61334,'Reentered','2003-10-26 00:00:00','2003-09-17 00:00:00',336.4,595,18693,65.57,'Highly Ellip' from dual union all
select 28025,'028025','1988-085BX','deb SOZ','',61334,'Reentered','2003-11-19 00:00:00','2003-09-29 00:00:00',336.01,546,18718,65.94,'Highly Ellip' from dual union all
select 28026,'028026','1988-085BY','deb SOZ','',61334,'Reentered','2003-11-02 00:00:00','2003-09-10 00:00:00',330.65,524,18399,65.69,'Highly Ellip' from dual union all
select 28027,'028027','1988-085BZ','deb SOZ','',61334,'Reentered','2003-11-15 00:00:00','2003-09-19 00:00:00',331.78,591,18404,65.23,'Highly Ellip' from dual union all
select 28028,'028028','1988-085CA','deb SOZ','',61334,'Reentered','2003-10-24 00:00:00','2003-09-29 00:00:00',334.39,493,18668,65.76,'Highly Ellip' from dual union all
select 28029,'028029','1988-085CB','deb SOZ','',61334,'Reentered','2003-12-29 00:00:00','2003-09-16 00:00:00',336.97,669,18656,65.31,'Highly Ellip' from dual union all
select 2803,'002803','1967-046C','BOZ','BOZ',15301,'Reentered','1967-06-02 00:00:00','1967-05-24 00:00:00',90.38,198,389,51.79,'Intermediate' from dual union all
select 28030,'028030','1988-085CC','deb SOZ','',61334,'Reentered','2003-11-18 00:00:00','2003-09-11 00:00:00',334.23,678,18472,65.57,'Highly Ellip' from dual union all
select 28031,'028031','1988-085CD','deb SOZ','',61334,'Reentered','2003-10-19 00:00:00','2003-09-16 00:00:00',335.72,637,18609,65.47,'Highly Ellip' from dual union all
select 28032,'028032','1988-085CE','deb SOZ','',61334,'Reentered','2003-10-17 00:00:00','2003-09-18 00:00:00',327.74,422,18315,65.44,'Highly Ellip' from dual union all
select 28033,'028033','1988-085CF','deb SOZ','',61334,'Reentered','2003-11-02 00:00:00','2003-09-15 00:00:00',335.63,879,18361,65.36,'Highly Ellip' from dual union all
select 28034,'028034','1988-085CG','deb SOZ','',61334,'Reentered','2003-12-06 00:00:00','2003-10-05 00:00:00',330.09,469,18417,65.32,'Highly Ellip' from dual union all
select 28035,'028035','1988-085CH','deb SOZ','',61334,'Reentered','2003-10-18 00:00:00','2003-09-16 00:00:00',337.2,716,18624,65.65,'Highly Ellip' from dual union all
select 28036,'028036','1988-085CJ','deb SOZ','',61334,'Reentered','2003-12-07 00:00:00','2003-09-29 00:00:00',337.99,737,18653,65.55,'Highly Ellip' from dual union all
select 28037,'028037','1988-085CK','deb SOZ','',61334,'Reentered','2003-10-29 00:00:00','2003-09-23 00:00:00',331.31,540,18425,65.33,'Highly Ellip' from dual union all
select 28038,'028038','1988-085CL','deb SOZ','',61334,'Reentered','2003-11-14 00:00:00','2003-09-09 00:00:00',332.28,562,18464,65.22,'Highly Ellip' from dual union all
select 28039,'028039','1991-003M','deb Ariane V41','',64172,'In Earth orbit','','2003-10-04 00:00:00',557.94,1148,30996,7.51,'Highly Ellip' from dual union all
select 2804,'002804','1967-046B','11S510','11S510',15301,'Reentered','1967-05-29 00:00:00','1967-05-20 00:00:00',90.54,203,399,51.77,'Intermediate' from dual union all
select 28040,'028040','1991-015P','deb Ariane V42','',64184,'In Earth orbit','','2003-10-03 00:00:00',520.38,1257,28849,7.49,'Highly Ellip' from dual union all
select 28041,'028041','1991-075L','deb Ariane V47','',64244,'In Earth orbit','','2003-10-04 00:00:00',506.01,752,28561,7.05,'Highly Ellip' from dual union all
select 28042,'028042','1992-093JP','deb 11S772','',64805,'In Earth orbit','','2003-09-27 00:00:00',102.49,825,928,70.87,'Intermediate' from dual union all
select 28043,'028043','2003-045A','Shenzhou 5','Shenzhou 5 Fanhui Cang',67669,'Landed','2003-10-15 00:00:00','2003-10-15 00:00:00',91.21,331,337,42.42,'Intermediate' from dual union all
select 28044,'028044','2003-045B','CZ-2F Stage 2','CZ-2F Stage 2',67669,'Reentered','2003-10-25 00:00:00','2003-10-18 00:00:00',89.42,190,301,42.41,'Intermediate' from dual union all
select 28045,'028045','2003-045C','CZ-2F sep motor cover','',67669,'Reentered','2003-10-16 00:00:00','2003-10-15 00:00:00',90.58,184,422,42.6,'Intermediate' from dual union all
select 28046,'028046','2003-045D','CZ-2F sep motor cover','',67669,'Reentered','2003-10-16 00:00:00','2003-10-16 00:00:00',89.61,170,341,42.55,'Intermediate' from dual union all
select 28047,'028047','2003-045E','CZ-2F sep motor cover','',67669,'Reentered','2003-10-21 00:00:00','2003-10-15 00:00:00',91.34,192,489,42.25,'Intermediate' from dual union all
select 28048,'028048','2003-045F','CZ-2F sep motor cover','',67669,'Reentered','2003-10-21 00:00:00','2003-10-16 00:00:00',91,187,461,42.28,'Intermediate' from dual union all
select 28049,'028049','2003-045G','Shenzhou 5 Guidao Cang','Shenzhou 5 orbital modul',67669,'Reentered','2004-05-30 00:00:00','2003-10-18 00:00:00',91.18,331,334,42.43,'Intermediate' from dual union all
select 2805,'002805','1967-046A','Kosmos-159','E-6LS No. 111',15301,'Reentered','1977-11-11 00:00:00','1967-07-25 00:00:00',1173.82,671,60293,51.86,'Highly Ellip' from dual union all
select 28050,'028050','2003-046B','PS4','PS4',67670,'In Earth orbit','','2003-10-17 00:00:00',101.68,802,875,98.74,'Sun-Synch' from dual union all
select 28051,'028051','2003-046A','Resourcesat','IRS-P6',67670,'In Earth orbit','','2003-10-18 00:00:00',101.68,802,875,98.74,'Sun-Synch' from dual union all
select 28052,'028052','2003-047A','Soyuz TMA-3','Soyuz 11F732 No. 213',67671,'Landed','2004-04-30 00:00:00','2003-10-18 00:00:00',89.89,266,272,51.63,'Intermediate' from dual union all
select 28053,'028053','2003-047B','11S510','11S510',67671,'Reentered','2003-10-20 00:00:00','2003-10-18 00:00:00',88.46,186,210,51.65,'Intermediate' from dual union all
select 28054,'028054','2003-048A','DMSP F-16','DMSP Block 5D-3 S-20',67672,'In Earth orbit','','2003-10-18 00:00:00',101.87,843,852,98.93,'Sun-Synch' from dual union all
select 28055,'028055','2003-048B','DMSP F-16 solar array band','',67672,'In Earth orbit','','2003-10-18 00:00:00',101.92,842,858,98.93,'Sun-Synch' from dual union all
select 28056,'028056','2003-048C','DMSP F-16 solar array band','',67672,'In Earth orbit','','2003-10-18 00:00:00',101.86,843,851,98.91,'Sun-Synch' from dual union all
select 28057,'028057','2003-049A','Zi Yuan 1-2','ziyuan yihao 02 xing',67673,'In Earth orbit','','2003-10-21 00:00:00',99.59,730,748,98.55,'Sun-Synch' from dual union all
select 28058,'028058','2003-049B','Chuangxin-1 01 xing','CX-1',67673,'In Earth orbit','','2003-10-28 00:00:00',99.23,686,757,98.53,'Sun-Synch' from dual union all
select 28059,'028059','2003-049C','CZ-4B Stage 3','CZ-4B Stage 3',67673,'In Earth orbit','','2003-10-28 00:00:00',99.62,731,749,98.53,'Sun-Synch' from dual union all
select 2806,'002806','1967-047A','Kosmos-160','OGCh No. 09',15302,'Reentered','1967-05-17 00:00:00','1967-05-18 00:00:00',88.22,139,234,49.61,'Intermediate' from dual union all
select 28060,'028060','2003-050A','SERVIS-1','SERVIS-1',67674,'In Earth orbit','','2003-11-07 00:00:00',105.1,984,1014,99.52,'Sun-Synch' from dual union all
select 28061,'028061','2003-050B','Briz-KM No. 72506','Briz-KM 14S45 No. 72506',67674,'In Earth orbit','','2003-11-06 00:00:00',103.06,797,1010,99.58,'Sun-Synch' from dual union all
select 28062,'028062','1995-064D','EPKM part','',65725,'Reentered','2004-01-15 00:00:00','2003-10-27 00:00:00',238.85,199,12548,25.4,'Medium' from dual union all
select 28063,'028063','1995-064E','EPKM part','',65725,'Reentered','2003-11-21 00:00:00','2003-10-18 00:00:00',140.97,175,5014,25.43,'Medium' from dual union all
select 28064,'028064','1999-057ST','CZ-4 debris','',66738,'Reentered','2015-03-22 00:00:00','2003-10-28 00:00:00',100.23,714,824,98.28,'Sun-Synch' from dual union all
select 28065,'028065','1991-050H','SARA antenna debris','',64219,'Reentered','2007-03-26 00:00:00','2003-11-02 00:00:00',99.07,713,716,98.06,'Sun-Synch' from dual union all
select 28066,'028066','1995-021C','deb ERS-2','',65682,'Reentered','2006-04-16 00:00:00','2003-10-26 00:00:00',96.55,586,601,98.32,'Sun-Synch' from dual union all
select 28067,'028067','1999-057MX','CZ-4 debris','',66738,'In Earth orbit','','2003-11-01 00:00:00',100.27,687,856,98.3,'Sun-Synch' from dual union all
select 28068,'028068','1999-057MY','CZ-4 debris','',66738,'In Earth orbit','','2003-11-01 00:00:00',100.26,700,842,98.3,'Sun-Synch' from dual union all
select 28069,'028069','1999-057MZ','CZ-4 debris','',66738,'In Earth orbit','','2003-11-02 00:00:00',100.06,700,823,98.29,'Sun-Synch' from dual union all
select 2807,'002807','1967-048A','NNS O-13','NNS O-13',15303,'In Earth orbit','','1967-07-05 00:00:00',106.97,1071,1102,89.58,'Polar' from dual union all
select 28070,'028070','1999-057NA','CZ-4 debris','',66738,'In Earth orbit','','2003-10-28 00:00:00',99.93,698,813,98.31,'Sun-Synch' from dual union all
select 28071,'028071','1999-057NB','CZ-4 debris','',66738,'In Earth orbit','','2003-10-23 00:00:00',100.54,706,863,98.21,'Sun-Synch' from dual union all
select 28072,'028072','1999-057NC','CZ-4 debris','',66738,'In Earth orbit','','2003-10-30 00:00:00',100.35,703,848,98.29,'Sun-Synch' from dual union all
select 28073,'028073','1999-057ND','CZ-4 debris','',66738,'In Earth orbit','','2003-10-28 00:00:00',100.07,699,825,98.28,'Sun-Synch' from dual union all
select 28074,'028074','1999-057NE','CZ-4 debris','',66738,'In Earth orbit','','2003-11-01 00:00:00',100.61,712,863,98.3,'Sun-Synch' from dual union all
select 28075,'028075','1999-057NF','CZ-4 debris','',66738,'In Earth orbit','','2003-10-24 00:00:00',100.53,704,863,98.3,'Sun-Synch' from dual union all
select 28076,'028076','2003-051C','CZ-2D motor cover','',67675,'Reentered','2003-11-04 00:00:00','2003-11-03 00:00:00',88.92,177,266,62.92,'Intermediate' from dual union all
select 28077,'028077','2003-051B','CZ-2D Stage 2','CZ-2D Stage 2',67675,'Reentered','2003-11-12 00:00:00','2003-11-05 00:00:00',89.4,187,303,62.98,'Intermediate' from dual union all
select 28078,'028078','2003-051A','Fanhui Shi Weixing (18)','Jianbing-4 01 xing',67675,'Reentered','2003-12-18 00:00:00','2003-11-05 00:00:00',89.74,193,331,62.99,'Intermediate' from dual union all
select 28079,'028079','2003-051D','CZ-2D motor cover','',67675,'Reentered','2003-11-03 00:00:00','2003-11-03 00:00:00',88.39,168,222,63,'Intermediate' from dual union all
select 2808,'002808','1967-046D','deb Blok-I','',15301,'Reentered','1967-05-22 00:00:00','1967-05-17 00:00:00',90.44,204,389,51.76,'Intermediate' from dual union all
select 28080,'028080','2003-048D','DMSP F-16 OLS Cover','',67672,'In Earth orbit','','2003-11-14 00:00:00',101.86,842,852,98.93,'Sun-Synch' from dual union all
select 28081,'028081','2003-048E','DMSP F-16 OLS Cooler Cover','',67672,'In Earth orbit','','2003-11-14 00:00:00',101.86,842,852,98.92,'Sun-Synch' from dual union all
select 28082,'028082','2003-052A','Zhongxing 20','Shentong yi hao 01 xing',67676,'In Earth orbit','','2004-01-13 00:00:00',1436.11,35776,35797,.17,'Stationary' from dual union all
select 28083,'028083','2003-052B','CZ-3A Stage 3','CZ-3A Stage 3',67676,'Reentered','2010-04-12 00:00:00','2004-01-14 00:00:00',716.95,151,40163,24.72,'GEO Transfer' from dual union all
select 28084,'028084','2003-035C','Kosmos-2399 SpK film part','',67659,'Reentered','2003-11-23 00:00:00','2003-11-19 00:00:00',89.3,196,283,64.92,'Intermediate' from dual union all
select 28085,'028085','2003-035D','Kosmos-2399 SpK film part','',67659,'Reentered','2003-11-22 00:00:00','2003-11-19 00:00:00',89.24,193,281,64.92,'Intermediate' from dual union all
select 28086,'028086','2003-035E','Spuskaemaya Kapsula','',67659,'Reentered','2003-11-30 00:00:00','2003-11-19 00:00:00',89.45,200,295,64.92,'Intermediate' from dual union all
select 28087,'028087','2003-035F','Kosmos-2399 SpK film part','',67659,'Reentered','2003-11-26 00:00:00','2003-11-19 00:00:00',89.46,199,297,64.91,'Intermediate' from dual union all
select 28088,'028088','2003-035G','Kosmos-2399 SpK film part','',67659,'Reentered','2003-11-26 00:00:00','2003-11-20 00:00:00',89.28,194,284,64.86,'Intermediate' from dual;
insert into SATELLITE
select 28089,'028089','2003-053A','Yamal-202','Yamal-200 No. 2',67677,'In Earth orbit','','2004-01-28 00:00:00',1436.01,35780,35790,.01,'Stationary' from dual union all
select 2809,'002809','1967-046E','deb Blok-I','',15301,'Reentered','1967-05-22 00:00:00','1967-05-17 00:00:00',89.8,204,326,51.76,'Intermediate' from dual union all
select 28090,'028090','2003-053D','Perekhodnik','Sredniy perekhodnik',67677,'Reentered','2003-11-24 00:00:00','2003-11-24 00:00:00',87.76,133,195,51.66,'Intermediate' from dual union all
select 28091,'028091','2003-053C','8S812','8S812',67677,'Reentered','2003-11-24 00:00:00','2003-11-24 00:00:00',87.23,117,157,51.66,'Intermediate' from dual union all
select 28092,'028092','2003-053F','SOZ','SOZ',67677,'Reentered','2004-09-20 00:00:00','2004-02-06 00:00:00',648.09,247,36612,48.06,'GEO Transfer' from dual union all
select 28093,'028093','2003-053G','SOZ','SOZ',67677,'Reentered','2005-01-23 00:00:00','2004-02-19 00:00:00',651.01,316,36692,48.11,'GEO Transfer' from dual union all
select 28094,'028094','2003-053B','Yamal-201','Yamal-200 No. 1',67677,'In Earth orbit','','2004-01-25 00:00:00',1436.06,35780,35792,.14,'Stationary' from dual union all
select 28095,'028095','2003-054A','USA 173','INTRUDER 3',67678,'In Earth orbit','','2003-12-02 00:00:00',107.4,1013,1199,63.43,'Intermediate' from dual union all
select 28096,'028096','2003-054B','Centaur AC-164','Centaur AC-164',67678,'In Earth orbit','','2003-12-02 00:00:00',107.41,1002,1212,63.68,'Intermediate' from dual union all
select 28097,'028097','2003-054C','USA 173 P/L 2','INTRUDER 4',67678,'In Earth orbit','','2003-12-02 00:00:00',107.4,1011,1202,63.44,'Intermediate' from dual union all
select 28098,'028098','2003-055A','Gruzomaket','GVM 30201-14S135',67679,'Reentered','2015-12-15 00:00:00','2003-12-07 00:00:00',93.73,453,461,67.08,'Intermediate' from dual union all
select 28099,'028099','2003-055B','Strela APB','Strela APB 14S425',67679,'Reentered','2010-03-10 00:00:00','2003-12-07 00:00:00',93.02,389,456,67.08,'Intermediate' from dual union all
select 281,'000281','62 Nu 1','Kosmos-3','2MS No. 1',8195,'Reentered','1962-10-17 00:00:00','1962-06-13 00:00:00',93.15,226,632,49,'Intermediate' from dual union all
select 2810,'002810','1967-047B','8S692','8S692',15302,'Reentered','1967-05-18 00:00:00','1967-05-18 00:00:00',88.22,139,234,49.61,'Intermediate' from dual union all
select 28100,'028100','2003-035H','Kosmos-2399 debris','',67659,'Reentered','2003-12-10 00:00:00','2003-12-09 00:00:00',88.53,163,241,64.87,'Intermediate' from dual union all
select 28101,'028101','2003-035J','Kosmos-2399 debris','',67659,'Reentered','2003-12-10 00:00:00','2003-12-09 00:00:00',89.1,160,300,64.91,'Intermediate' from dual union all
select 28102,'028102','2003-035K','Kosmos-2399 debris','',67659,'Reentered','2003-12-09 00:00:00','2003-12-09 00:00:00',88.93,151,292,64.89,'Intermediate' from dual union all
select 28103,'028103','2003-035L','Kosmos-2399 debris','',67659,'Reentered','2003-12-09 00:00:00','2003-12-09 00:00:00',89.1,172,287,64.89,'Intermediate' from dual union all
select 28104,'028104','2003-035M','Kosmos-2399 debris','',67659,'Reentered','2003-12-10 00:00:00','2003-12-09 00:00:00',92.85,166,663,64.9,'Intermediate' from dual union all
select 28105,'028105','2003-035N','Kosmos-2399 debris','',67659,'Reentered','2003-12-11 00:00:00','2003-12-09 00:00:00',94.76,172,842,64.89,'Intermediate' from dual union all
select 28106,'028106','2003-035P','Kosmos-2399 debris','',67659,'Reentered','2003-12-09 00:00:00','2003-12-09 00:00:00',89.06,161,295,64.89,'Intermediate' from dual union all
select 28107,'028107','2003-035Q','Kosmos-2399 debris','',67659,'Reentered','2003-12-12 00:00:00','2003-12-09 00:00:00',94.21,175,785,64.91,'Intermediate' from dual union all
select 28108,'028108','2003-035R','Kosmos-2399 debris','',67659,'Reentered','2003-12-11 00:00:00','2003-12-09 00:00:00',91.89,171,564,64.91,'Intermediate' from dual union all
select 28109,'028109','2003-035S','Kosmos-2399 debris','',67659,'Reentered','2003-12-12 00:00:00','2003-12-09 00:00:00',94.03,174,770,64.91,'Intermediate' from dual union all
select 2811,'002811','1967-048B','Altair 2 X-258E6 ABL-139','X-258E6',15303,'In Earth orbit','','1967-08-01 00:00:00',106.95,1070,1101,89.58,'Polar' from dual union all
select 28110,'028110','2003-035T','Kosmos-2399 debris','',67659,'Reentered','2003-12-10 00:00:00','2003-12-09 00:00:00',89.41,167,324,64.9,'Intermediate' from dual union all
select 28111,'028111','2003-035U','Kosmos-2399 debris','',67659,'Reentered','2003-12-09 00:00:00','2003-12-09 00:00:00',89.95,160,384,64.9,'Intermediate' from dual union all
select 28112,'028112','2003-056A','Kosmos-2402','Uragan 794',67680,'In Earth orbit','','2004-02-10 00:00:00',675.69,19010,19248,65.06,'Medium' from dual union all
select 28113,'028113','2003-056B','Kosmos-2403','Uragan 795',67680,'In Earth orbit','','2004-02-08 00:00:00',675.69,19018,19240,65.07,'Medium' from dual union all
select 28114,'028114','2003-056C','Kosmos-2404','Uragan-M  No. 11L (701)',67680,'In Earth orbit','','2004-02-08 00:00:00',675.71,19030,19229,65.08,'Medium' from dual union all
select 28115,'028115','2003-056D','Briz-M No. 88506','Briz-M 14S43 No. 88506',67680,'In Earth orbit','','2004-02-09 00:00:00',675.63,18923,19332,65.03,'Medium' from dual union all
select 28116,'028116','2003-056E','Briz-M DTB','Briz-M torus tank',67680,'In Earth orbit','','2004-02-10 00:00:00',337.64,286,19082,53.02,'Highly Ellip' from dual union all
select 28117,'028117','2003-057A','UHF F/O F11','UHF F/O F11',67681,'In Earth orbit','','2003-12-30 00:00:00',1436.27,35780,35800,.01,'Stationary' from dual union all
select 28118,'028118','2003-057B','Centaur AC-203','Centaur IIIB AC-203',67681,'In Earth orbit','','2004-12-31 00:00:00',473.07,174,27293,27.95,'Highly Ellip' from dual union all
select 28119,'028119','2003-053E','Blok DM-2M No. 12L','11S861-01 No. 12L',67677,'In Earth orbit','','2004-01-25 00:00:00',1439.41,34978,36725,.21,'Drift GEO' from dual union all
select 2812,'002812','1967-049A','Kosmos-161','Zenit-4 No. 31',15304,'Landed','1967-05-30 00:00:00','1967-05-24 00:00:00',89.69,199,320,65.65,'Intermediate' from dual union all
select 28120,'028120','2003-035V','Kosmos-2399 debris','',67659,'Reentered','2003-12-10 00:00:00','2003-12-09 00:00:00',90.21,170,400,64.9,'Intermediate' from dual union all
select 28121,'028121','2003-035W','Kosmos-2399 debris','',67659,'Reentered','2003-12-10 00:00:00','2003-12-09 00:00:00',90.21,170,400,64.9,'Intermediate' from dual union all
select 28122,'028122','2003-035X','Kosmos-2399 debris','',67659,'Reentered','2003-12-10 00:00:00','2003-12-09 00:00:00',90.21,170,400,64.9,'Intermediate' from dual union all
select 28123,'028123','2003-035Y','Kosmos-2399 debris','',67659,'Reentered','2003-12-10 00:00:00','2003-12-09 00:00:00',90.21,170,400,64.9,'Intermediate' from dual union all
select 28124,'028124','2003-035Z','Kosmos-2399 debris','',67659,'Reentered','2003-12-10 00:00:00','2003-12-09 00:00:00',90.21,170,400,64.9,'Intermediate' from dual union all
select 28125,'028125','2003-035AA','Kosmos-2399 debris','',67659,'Reentered','2003-12-10 00:00:00','2003-12-09 00:00:00',90.21,170,400,64.9,'Intermediate' from dual union all
select 28126,'028126','2003-035AB','Kosmos-2399 debris','',67659,'Reentered','2003-12-10 00:00:00','2003-12-09 00:00:00',90.21,170,400,64.9,'Intermediate' from dual union all
select 28127,'028127','2003-035AC','Kosmos-2399 debris','',67659,'Reentered','2003-12-10 00:00:00','2003-12-09 00:00:00',90.21,170,400,64.9,'Intermediate' from dual union all
select 28128,'028128','2003-035AD','Kosmos-2399 debris','',67659,'Reentered','2003-12-10 00:00:00','2003-12-09 00:00:00',90.21,170,400,64.9,'Intermediate' from dual union all
select 28129,'028129','2003-058A','GPS IIR-10','Navstar GPS SVN 47',67682,'In Earth orbit','','2004-02-22 00:00:00',717.92,20043,20319,55.1,'Medium' from dual union all
select 2813,'002813','1967-050A','OPS 4321','GAMBIT SV 987',15305,'Deorbited','1967-05-30 00:00:00','1967-05-24 00:00:00',89.15,141,324,91.5,'Polar' from dual union all
select 28130,'028130','2003-058B','Delta 302','Delta SSPS AJ-10-118K',67682,'Reentered','2004-09-16 00:00:00','2003-12-27 00:00:00',98.25,231,1119,29.85,'Intermediate' from dual union all
select 28131,'028131','2003-058C','Star 48B','Star 48B',67682,'Reentered','2011-03-03 00:00:00','2004-02-22 00:00:00',347.35,178,19803,38.92,'Highly Ellip' from dual union all
select 28132,'028132','2003-059A','Amos 2','Amos-2',67683,'In Earth orbit','','2004-03-01 00:00:00',1436.04,35758,35812,.08,'Stationary' from dual union all
select 28133,'028133','2003-059B','Fregat','Fregat 14S44 No. 1006',67683,'In Earth orbit','','2004-03-01 00:00:00',714.72,4403,35801,23.96,'GEO Transfer' from dual union all
select 28134,'028134','2003-060A','Ekspress AM-22','Sesat-2',67684,'In Earth orbit','','2004-03-04 00:00:00',1436.04,35732,35838,.01,'Stationary' from dual union all
select 28135,'028135','2003-060B','8S812','8S812',67684,'Reentered','2003-12-31 00:00:00','2003-12-29 00:00:00',88.07,174,184,51.62,'Intermediate' from dual union all
select 28136,'028136','2003-060C','Perekhodnik','Sredniy perekhodnik',67684,'Reentered','2003-12-30 00:00:00','2003-12-29 00:00:00',87.5,146,156,51.6,'Intermediate' from dual union all
select 28137,'028137','2004-001A','Estrela do Sul 1','Telstar 14',67845,'In Earth orbit','','2004-03-11 00:00:00',1436.09,35777,35796,.03,'Stationary' from dual union all
select 28138,'028138','2004-001B','Blok DM-SL No. 9L','Blok DM-SL No. 9L',67845,'In Earth orbit','','2004-03-15 00:00:00',640.91,658,35834,.15,'GEO Transfer' from dual union all
select 28139,'028139','2003-060D','Blok DM-2M No. 13L','11S861-01 No. 13L',67684,'In Earth orbit','','2004-03-02 00:00:00',1436.85,35725,35878,.18,'Drift GEO' from dual union all
select 2814,'002814','1967-049B','11A57I','11A57I',15304,'Reentered','1967-05-28 00:00:00','1967-05-24 00:00:00',89.43,196,297,65.65,'Intermediate' from dual union all
select 28140,'028140','2003-061A','Tan Ce 1','Double Star DSP-E',67685,'Reentered','2007-10-14 00:00:00','2003-12-29 00:00:00',1619.44,555,78051,28.5,'Very High' from dual union all
select 28141,'028141','2003-061B','SM Y1','CTS',67685,'Reentered','2007-10-14 00:00:00','2003-12-29 00:00:00',1619.44,555,78051,28.5,'Very High' from dual union all
select 28142,'028142','2004-002A','Progress M1-11','Progress M1 No. 260',67846,'Deorbited','2004-06-03 00:00:00','2004-02-02 00:00:00',91.9,363,373,51.63,'Intermediate' from dual union all
select 28143,'028143','2004-002B','11S510','11S510',67846,'Reentered','2004-02-01 00:00:00','2004-01-31 00:00:00',87.49,142,159,51.63,'Intermediate' from dual union all
select 28144,'028144','1976-066D','Star 37C yo weight','',39018,'In Earth orbit','','2004-01-09 00:00:00',234.96,449,12020,24.53,'Medium' from dual union all
select 28145,'028145','1977-018F','Star 37E yo weight','',42146,'In Earth orbit','','2003-12-27 00:00:00',286.77,300,15753,24.42,'Highly Ellip' from dual union all
select 28146,'028146','1978-064P','deb Seasat','',45259,'Reentered','2004-03-14 00:00:00','2004-01-26 00:00:00',98.22,587,761,107.94,'Retrograde' from dual union all
select 28147,'028147','1979-104U','deb Ariane L01','',47588,'In Earth orbit','','2004-01-16 00:00:00',396.16,1215,21765,14.49,'Highly Ellip' from dual union all
select 28148,'028148','1981-059J','deb NOAA 7','',52582,'Reentered','2005-03-19 00:00:00','2004-02-10 00:00:00',101.08,758,862,98.44,'Sun-Synch' from dual union all
select 28149,'028149','1987-079BD','deb SOZ','',60169,'Reentered','2005-03-10 00:00:00','2004-02-05 00:00:00',331.92,755,18249,65.39,'Highly Ellip' from dual union all
select 2815,'002815','1967-021C','SOK','Support cone',15276,'Reentered','1967-03-11 00:00:00','1967-03-10 00:00:00',89.24,180,294,51.44,'Intermediate' from dual union all
select 28150,'028150','1987-079BE','deb SOZ','',60169,'Reentered','2004-06-06 00:00:00','2004-02-05 00:00:00',320.26,768,17488,65.36,'Highly Ellip' from dual union all
select 28151,'028151','1987-079BF','deb SOZ','',60169,'Reentered','2007-06-22 00:00:00','2004-02-06 00:00:00',327.79,718,18021,65.49,'Highly Ellip' from dual union all
select 28152,'028152','1988-085CM','deb SOZ','',61334,'In Earth orbit','','2004-01-17 00:00:00',305.68,475,16832,65.45,'Highly Ellip' from dual union all
select 28153,'028153','1988-085CN','deb SOZ','',61334,'In Earth orbit','','2004-02-04 00:00:00',254.44,569,13277,64.97,'Medium' from dual union all
select 28154,'028154','2004-003A','AMC 10','AMC-10',67847,'In Earth orbit','','2004-04-10 00:00:00',1436.04,35775,35796,.06,'Stationary' from dual union all
select 28155,'028155','2004-003B','Centaur AC-165','Centaur IIA AC-165',67847,'In Earth orbit','','2004-04-06 00:00:00',624.7,239,35419,12.18,'GEO Transfer' from dual union all
select 28156,'028156','2000-024E','DSP F20 Aperture Cover','',66926,'In Earth orbit','','2000-05-09 00:00:00',1443.96,35911,35970,2.9,'Drift GEO' from dual union all
select 28157,'028157','1983-008J','deb SSU','',55765,'In Earth orbit','','2004-02-01 00:00:00',107.26,900,1299,63.43,'Intermediate' from dual union all
select 28158,'028158','2004-004A','DSP F22','DSP F22',67848,'In Earth orbit','','2004-02-15 00:00:00',1445.91,35852,36105,.68,'Drift GEO' from dual union all
select 28159,'028159','2004-004B','Titan 4B-39 Stage 2','Titan 4 Stage 2',67848,'Reentered','2005-01-01 00:00:00','2004-02-15 00:00:00',93.96,181,755,28.6,'Intermediate' from dual union all
select 2816,'002816','1967-050B','Agena D 4838','Agena D',15305,'Reentered','1967-05-27 00:00:00','1967-05-24 00:00:00',89.3,126,354,91.5,'Polar' from dual union all
select 28160,'028160','2004-004C','IUS-10 SRM-1','Orbus 21',67848,'In Earth orbit','','2004-02-15 00:00:00',638.98,375,36018,28,'GEO Transfer' from dual union all
select 28161,'028161','2004-004D','IUS-10 SRM-2','Orbus 6E',67848,'In Earth orbit','','2004-02-15 00:00:00',1435.5,35750,35800,3,'Stationary' from dual union all
select 28162,'028162','1977-005G','deb Delta 128','',42133,'Reentered','2004-02-17 00:00:00','2004-02-09 00:00:00',102.1,528,1189,28.09,'Intermediate' from dual union all
select 28163,'028163','2004-005A','Molniya-1T','Molniya-1T',67849,'Reentered','2016-04-16 00:00:00','2004-05-03 00:00:00',717.8,722,39634,62.86,'Molniya' from dual union all
select 28164,'028164','2004-005B','Blok-ML','Blok-ML No. 71636-495',67849,'Reentered','2016-10-12 00:00:00','2004-05-02 00:00:00',731.1,728,40281,62.86,'Molniya' from dual union all
select 28165,'028165','2004-005C','11S510','11S510',67849,'Reentered','2004-04-11 00:00:00','2004-02-25 00:00:00',92.39,211,573,62.83,'Intermediate' from dual union all
select 28166,'028166','2004-005D','BOZ','BOZ',67849,'Reentered','2004-03-15 00:00:00','2004-02-25 00:00:00',91.82,181,547,62.83,'Intermediate' from dual union all
select 28167,'028167','2004-005E','deb BOZ','BOZ ullage motor',67849,'Reentered','2004-03-22 00:00:00','2004-02-25 00:00:00',92.31,205,571,62.85,'Intermediate' from dual union all
select 28168,'028168','2004-005F','deb BOZ','BOZ ullage motor',67849,'Reentered','2004-03-28 00:00:00','2004-02-25 00:00:00',92.39,210,573,62.84,'Intermediate' from dual union all
select 28169,'028169','2004-006A','Rosetta','Rosetta',67850,'Deep Space','2004-03-02 00:00:00','2004-03-02 00:00:00',938.63,391,76660,5.7,'Earth Escape' from dual union all
select 2817,'002817','1967-051A','IMP 4','IMP F',15306,'Deep Space','1967-05-25 00:00:00','1967-10-27 00:00:00',6358.33,241,214381,67.17,'Deep Space' from dual union all
select 28170,'028170','2004-006B','EPS 518','EPS S/N 520',67850,'Deep Space','2004-03-02 00:00:00','2004-03-02 00:00:00',938.63,391,76660,5.7,'Earth Escape' from dual union all
select 28171,'028171','2001-057C','deb Kosmos-2383','',67210,'Reentered','2004-03-05 00:00:00','2004-03-04 00:00:00',88.87,180,257,64.96,'Intermediate' from dual union all
select 28172,'028172','2001-057D','deb Kosmos-2383','',67210,'Reentered','2004-02-29 00:00:00','2004-02-29 00:00:00',90.49,228,370,65,'Intermediate' from dual union all
select 28173,'028173','2001-057E','deb Kosmos-2383','',67210,'Reentered','2004-03-01 00:00:00','2004-03-01 00:00:00',89.96,205,340,65.01,'Intermediate' from dual union all
select 28174,'028174','2001-057F','deb Kosmos-2383','',67210,'Reentered','2004-03-03 00:00:00','2004-03-01 00:00:00',90.1,213,346,65.01,'Intermediate' from dual union all
select 28175,'028175','2001-057G','deb Kosmos-2383','',67210,'Reentered','2004-03-18 00:00:00','2004-03-07 00:00:00',91.67,219,494,64.85,'Intermediate' from dual union all
select 28176,'028176','2001-057H','deb Kosmos-2383','',67210,'Reentered','2004-03-05 00:00:00','2004-03-05 00:00:00',89.89,204,334,65.22,'Intermediate' from dual union all
select 28177,'028177','2001-057J','deb Kosmos-2383','',67210,'Reentered','2004-03-11 00:00:00','2004-03-07 00:00:00',93.02,234,610,65,'Intermediate' from dual union all
select 28178,'028178','2001-057K','deb Kosmos-2383','',67210,'Reentered','2004-03-03 00:00:00','2004-03-01 00:00:00',90.64,225,387,65.01,'Intermediate' from dual;
insert into SATELLITE
select 28179,'028179','2001-057L','deb Kosmos-2383','',67210,'Reentered','2004-03-01 00:00:00','2004-03-01 00:00:00',89.15,169,296,64.99,'Intermediate' from dual union all
select 2818,'002818','1967-052B','11S510','11S510',15307,'Reentered','1967-06-05 00:00:00','1967-05-26 00:00:00',90.6,197,411,64.85,'Intermediate' from dual union all
select 28180,'028180','2001-057M','deb Kosmos-2383','',67210,'Reentered','2004-03-02 00:00:00','2004-03-02 00:00:00',89.36,193,293,64.97,'Intermediate' from dual union all
select 28181,'028181','2001-057N','deb Kosmos-2383','',67210,'Reentered','2004-03-03 00:00:00','2004-03-03 00:00:00',89.88,211,326,64.99,'Intermediate' from dual union all
select 28182,'028182','2001-057P','deb Kosmos-2383','',67210,'Reentered','2004-03-05 00:00:00','2004-03-07 00:00:00',88.88,184,255,64.97,'Intermediate' from dual union all
select 28183,'028183','2001-057Q','deb Kosmos-2383','',67210,'Reentered','2004-03-03 00:00:00','2004-03-03 00:00:00',89.18,197,270,65.03,'Intermediate' from dual union all
select 28184,'028184','2004-007A','MBSAT','MBSAT',67851,'In Earth orbit','','2004-05-12 00:00:00',1436.16,35743,35833,.01,'Stationary' from dual union all
select 28185,'028185','2004-007B','Centaur AC-202','Centaur IIIA',67851,'Reentered','2011-07-01 00:00:00','2004-05-14 00:00:00',608.78,189,34643,25.13,'GEO Transfer' from dual union all
select 28186,'028186','2004-010D','SOZ','SOZ',67854,'Reentered','2004-09-10 00:00:00','2004-05-30 00:00:00',632.56,200,35863,47.33,'GEO Transfer' from dual union all
select 28187,'028187','2004-008A','Eutelsat W3A','Eutelsat W3A',67852,'In Earth orbit','','2004-05-15 00:00:00',1435.52,35757,35794,.02,'Stationary' from dual union all
select 28188,'028188','2004-008B','Briz-M DTB','Briz-M torus tank',67852,'In Earth orbit','','2004-05-18 00:00:00',284.01,303,15566,49.48,'Highly Ellip' from dual union all
select 28189,'028189','2003-058D','Star 48 yo weight','',67682,'Reentered','2012-04-28 00:00:00','2004-04-04 00:00:00',349.57,183,19937,38.8,'Highly Ellip' from dual union all
select 2819,'002819','1967-052C','BOZ','BOZ',15307,'Reentered','1967-06-15 00:00:00','1967-06-03 00:00:00',90.39,204,384,64.84,'Intermediate' from dual union all
select 28190,'028190','2004-009A','GPS IIR-11','Navstar GPS SVN 59',67853,'In Earth orbit','','2004-05-20 00:00:00',718,20095,20271,55.03,'Medium' from dual union all
select 28191,'028191','2004-009B','Delta 303','Delta SSPS AJ-10-118K',67853,'Reentered','2004-09-22 00:00:00','2004-03-22 00:00:00',98.24,215,1134,30.3,'Intermediate' from dual union all
select 28192,'028192','2004-009C','Star 48B','Star 48B',67853,'Reentered','2007-03-19 00:00:00','2004-05-20 00:00:00',339.43,161,19320,38.89,'Highly Ellip' from dual union all
select 28193,'028193','2004-009D','Star 48 yo weight','',67853,'In Earth orbit','','2004-05-23 00:00:00',345.95,152,19741,38.83,'Highly Ellip' from dual union all
select 28194,'028194','2004-010A','Raduga-1','Globus No. 17L',67854,'In Earth orbit','','2004-05-26 00:00:00',1436.11,35772,35802,1.31,'Stationary' from dual union all
select 28195,'028195','2004-010B','Perekhodnik','Sredniy perekhodnik',67854,'Reentered','2004-03-28 00:00:00','2004-03-27 00:00:00',87.44,144,152,51.66,'Intermediate' from dual union all
select 28196,'028196','2004-010C','8S812','8S812',67854,'Reentered','2004-03-29 00:00:00','2004-03-28 00:00:00',87.65,155,161,51.65,'Intermediate' from dual union all
select 28197,'028197','2004-010E','SOZ','SOZ',67854,'Reentered','2004-10-10 00:00:00','2004-06-02 00:00:00',635.53,249,35967,47.34,'GEO Transfer' from dual union all
select 28198,'028198','2003-060E','SOZ','SOZ',67684,'Reentered','2006-06-28 00:00:00','2004-04-04 00:00:00',635.14,394,35802,47.68,'GEO Transfer' from dual union all
select 28199,'028199','2003-060F','SOZ','SOZ',67684,'Reentered','2006-05-13 00:00:00','2004-04-02 00:00:00',633.78,338,35788,47.64,'GEO Transfer' from dual union all
select 282,'000282','62 Mu 2','Agena B 6004','Agena B 6004',8194,'Deep Space','1962-04-26 00:00:00','',null,null,null,null,'' from dual union all
select 2820,'002820','1967-052D','deb Blok-I','',15307,'Reentered','1967-05-28 00:00:00','1967-05-25 00:00:00',91.22,198,470,64.88,'Intermediate' from dual union all
select 28200,'028200','1966-040N','Nimbus 2 debris','',13447,'In Earth orbit','','2004-04-04 00:00:00',107.88,1088,1169,100.35,'Sun-Synch' from dual union all
select 28201,'028201','1966-040P','Nimbus 2 debris','',13447,'In Earth orbit','','2004-04-03 00:00:00',107.42,1059,1156,100.3,'Sun-Synch' from dual union all
select 28202,'028202','1966-040Q','Nimbus 2 debris','',13447,'In Earth orbit','','2004-04-03 00:00:00',107.25,1068,1130,100.25,'Sun-Synch' from dual union all
select 28203,'028203','1966-040R','Nimbus 2 debris','',13447,'In Earth orbit','','2004-03-26 00:00:00',107.79,1088,1161,100.31,'Sun-Synch' from dual union all
select 28204,'028204','1966-040S','Nimbus 2 debris','',13447,'In Earth orbit','','2004-04-02 00:00:00',107.2,1060,1135,100.25,'Sun-Synch' from dual union all
select 28205,'028205','1966-040T','Nimbus 2 debris','',13447,'In Earth orbit','','2004-04-04 00:00:00',107.98,1096,1171,100.37,'Sun-Synch' from dual union all
select 28206,'028206','1966-040U','Nimbus 2 debris','',13447,'In Earth orbit','','2004-04-04 00:00:00',107.98,1091,1176,100.38,'Sun-Synch' from dual union all
select 28207,'028207','1966-040V','Nimbus 2 debris','',13447,'In Earth orbit','','2004-04-03 00:00:00',107.11,1047,1139,100.32,'Sun-Synch' from dual union all
select 28208,'028208','1966-040W','Nimbus 2 debris','',13447,'In Earth orbit','','2004-04-03 00:00:00',107.17,1057,1134,100.32,'Sun-Synch' from dual union all
select 28209,'028209','1966-040X','Nimbus 2 debris','',13447,'In Earth orbit','','2004-03-31 00:00:00',107.31,1068,1137,100.31,'Sun-Synch' from dual union all
select 2821,'002821','1967-052E','deb Blok-I','',15307,'Reentered','1967-05-28 00:00:00','1967-05-25 00:00:00',90.68,198,418,64.88,'Intermediate' from dual union all
select 28210,'028210','1966-041G','NNS O-9 insulation','',13448,'Reentered','2012-01-02 00:00:00','2004-04-03 00:00:00',96.97,601,627,90.18,'Polar' from dual union all
select 28211,'028211','1981-021U','deb US-A coolant','',52544,'In Earth orbit','','2004-03-25 00:00:00',103.57,893,962,64.96,'Intermediate' from dual union all
select 28212,'028212','1987-079BG','deb SOZ','',60169,'Reentered','2004-10-16 00:00:00','2004-04-02 00:00:00',336.7,679,18629,65.39,'Highly Ellip' from dual union all
select 28213,'028213','1987-079BH','deb SOZ','',60169,'Reentered','2006-12-21 00:00:00','2004-03-24 00:00:00',327.45,762,17956,65.61,'Highly Ellip' from dual union all
select 28214,'028214','1987-079BJ','deb SOZ','',60169,'Reentered','2004-04-02 00:00:00','2004-04-01 00:00:00',115.31,200,2739,65.84,'Intermediate' from dual union all
select 28215,'028215','1987-079BK','deb SOZ','',60169,'Reentered','2004-02-12 00:00:00','2004-04-01 00:00:00',333.96,963,18171,65.64,'Highly Ellip' from dual union all
select 28216,'028216','1987-079BL','deb SOZ','',60169,'Reentered','2006-03-24 00:00:00','2004-03-21 00:00:00',333.95,853,18280,66.03,'Highly Ellip' from dual union all
select 28217,'028217','1997-067D','Star 48 yo weight','',66147,'Reentered','2008-04-24 00:00:00','2004-04-04 00:00:00',176.55,175,7919,35.01,'Medium' from dual union all
select 28218,'028218','2004-011A','Superbird A2','Superbird 6',67855,'In Earth orbit','','2009-06-27 00:00:00',1447.7,35998,36029,4.38,'Inclined Drift' from dual union all
select 28219,'028219','2004-011B','Centaur AC-163','Centaur IIA AC-163',67855,'Reentered','2004-04-19 00:00:00','2004-04-16 00:00:00',2917.41,166,122343,26.3,'Very High' from dual union all
select 2822,'002822','1967-052A','Molniya-1','Molniya-1 11F67 No. 8',15307,'Reentered','1971-11-26 00:00:00','1967-08-03 00:00:00',711.1,500,39525,64.74,'GEO Transfer' from dual union all
select 28220,'028220','2004-012A','Shiyan Weixing 1','Shiyan 1',67856,'In Earth orbit','','2004-04-19 00:00:00',96.82,599,614,97.71,'Sun-Synch' from dual union all
select 28221,'028221','2004-012B','Naxing 1','Nanosatellite 1',67856,'In Earth orbit','','2004-04-19 00:00:00',96.84,600,615,97.71,'Sun-Synch' from dual union all
select 28222,'028222','2004-012C','CZ-2C Stage 2','CZ-2C Stage 2',67856,'In Earth orbit','','2004-04-19 00:00:00',96.29,543,619,97.64,'Sun-Synch' from dual union all
select 28223,'028223','2004-012D','CZ-2C Stage 2 part','',67856,'Reentered','2004-08-21 00:00:00','2004-04-20 00:00:00',94.14,349,605,97.72,'Sun-Synch' from dual union all
select 28224,'028224','2004-012E','CZ-2C Stage 2 motor cover','',67856,'Reentered','2014-11-20 00:00:00','2004-04-21 00:00:00',97.98,597,728,97.56,'Sun-Synch' from dual union all
select 28225,'028225','2004-012F','CZ-2C Stage 2 motor cover','',67856,'In Earth orbit','','2004-04-21 00:00:00',98.48,602,770,97.52,'Sun-Synch' from dual union all
select 28226,'028226','2004-012G','CZ-2C Stage 2 motor cover','',67856,'Reentered','2016-02-27 00:00:00','2004-04-21 00:00:00',98.44,607,762,97.89,'Sun-Synch' from dual union all
select 28227,'028227','2004-012H','CZ-2C Stage 2 motor cover','',67856,'Reentered','2009-02-26 00:00:00','2004-04-21 00:00:00',97.4,602,667,97.62,'Sun-Synch' from dual union all
select 28228,'028228','2004-013A','Soyuz TMA-4','Soyuz 11F732 No. 214',67857,'Landed','2004-10-24 00:00:00','2004-04-20 00:00:00',90.1,262,297,51.63,'Intermediate' from dual union all
select 28229,'028229','2004-013B','11S510','11S510',67857,'Reentered','2004-04-22 00:00:00','2004-04-20 00:00:00',88.21,174,198,51.66,'Intermediate' from dual union all
select 2823,'002823','1967-045C','deb Kosmos-158','',15300,'Reentered','1991-12-21 00:00:00','1967-06-15 00:00:00',100.68,739,843,74.03,'Intermediate' from dual union all
select 28230,'028230','2004-014A','Gravity Probe B','GP-B',67858,'In Earth orbit','','2004-04-21 00:00:00',97.58,641,645,90.01,'Polar' from dual union all
select 28231,'028231','2004-014B','Delta 304','Delta SSPS AJ-10-118K',67858,'Reentered','2004-05-27 00:00:00','2004-04-20 00:00:00',92.69,182,631,94.56,'Polar' from dual union all
select 28232,'028232','2004-012J','CZ-2C Stage 2 part','',67856,'In Earth orbit','','2004-04-21 00:00:00',99.46,608,857,97.9,'Sun-Synch' from dual union all
select 28233,'028233','2004-012K','CZ-2C Stage 2 part','',67856,'Reentered','2014-06-17 00:00:00','2004-04-21 00:00:00',97.86,595,718,97.82,'Sun-Synch' from dual union all
select 28234,'028234','2004-015A','Ekspress AM-11','Ekspress AM-11',67859,'In Earth orbit','','2004-06-27 00:00:00',1436.06,35782,35790,.01,'Stationary' from dual union all
select 28235,'028235','2004-015B','Perekhodnik','Sredniy perekhodnik',67859,'Reentered','2004-04-27 00:00:00','2004-04-27 00:00:00',87.57,154,154,51.61,'Intermediate' from dual union all
select 28236,'028236','2004-015C','8S812','8S812',67859,'Reentered','2004-04-29 00:00:00','2004-04-28 00:00:00',87.63,154,161,51.62,'Intermediate' from dual union all
select 28237,'028237','2002-003D','4/4D-LC Lower Fairing Half','Fairing Half',67379,'In Earth orbit','','2004-05-07 00:00:00',626.15,443,35290,28.71,'GEO Transfer' from dual union all
select 28238,'028238','2004-016A','DirecTV 7S','DirecTV 7S',67860,'In Earth orbit','','2004-07-04 00:00:00',1436.09,35782,35791,.04,'Stationary' from dual union all
select 28239,'028239','2004-016B','Blok DM-SL No. 16L','Blok DM-SL',67860,'Reentered','2004-09-20 00:00:00','2004-07-03 00:00:00',599.08,128,34197,.25,'Highly Ellip' from dual union all
select 2824,'002824','1967-043C','Fairing','Fairing',15298,'Reentered','1967-06-06 00:00:00','1967-05-29 00:00:00',90.78,161,464,85.05,'Polar' from dual union all
select 28240,'028240','2004-015D','Blok DM-2M No. 14L','11S861-01 No. 14L',67859,'In Earth orbit','','2004-06-26 00:00:00',1429.86,35597,35732,.18,'Drift GEO' from dual union all
select 28241,'028241','2004-015E','SOZ','SOZ',67859,'Reentered','2005-08-07 00:00:00','2004-06-29 00:00:00',632.01,333,35702,47.51,'GEO Transfer' from dual union all
select 28242,'028242','2004-015F','SOZ','SOZ',67859,'Reentered','2005-02-21 00:00:00','2004-06-30 00:00:00',630.59,277,35685,47.46,'GEO Transfer' from dual union all
select 28243,'028243','2002-003E','4/4D-LC Lower Fairing Half','Fairing Half',67379,'In Earth orbit','','2004-05-09 00:00:00',629.22,446,35445,28.73,'GEO Transfer' from dual union all
select 28244,'028244','2003-008D','deb Delta 296','deb DCSS-4 F02',67632,'In Earth orbit','','2004-05-01 00:00:00',616.82,250,34999,25.5,'GEO Transfer' from dual union all
select 28245,'028245','2003-008E','deb Delta 296','deb DCSS-4 F02',67632,'In Earth orbit','','2004-05-01 00:00:00',616.82,250,34999,25.5,'GEO Transfer' from dual union all
select 28246,'028246','2003-008F','deb Delta 296','deb DCSS-4 F02',67632,'In Earth orbit','','2004-05-01 00:00:00',616.82,250,34999,25.5,'GEO Transfer' from dual union all
select 28247,'028247','2003-008G','deb Delta 296','deb DCSS-4 F02',67632,'In Earth orbit','','2004-05-01 00:00:00',616.82,250,34999,25.5,'GEO Transfer' from dual union all
select 28248,'028248','1964-083M','deb VE-5','',10369,'In Earth orbit','','2004-05-13 00:00:00',105.81,1008,1057,89.85,'Polar' from dual union all
select 28249,'028249','1990-050K','USA 61 cover','',63292,'In Earth orbit','','2004-05-30 00:00:00',107.26,1050,1149,63.4,'Intermediate' from dual union all
select 2825,'002825','1967-053B','Agena D 2704','Agena D 2704',15308,'In Earth orbit','','1967-07-24 00:00:00',103.4,913,926,69.98,'Intermediate' from dual union all
select 28250,'028250','1990-050L','USA 62 cover','',63292,'In Earth orbit','','2004-05-30 00:00:00',107.26,1050,1149,63.4,'Intermediate' from dual union all
select 28251,'028251','1996-029N','deb USA 119','',65917,'In Earth orbit','','2004-05-20 00:00:00',107.37,1050,1160,63.42,'Intermediate' from dual union all
select 28252,'028252','2004-017A','AMC 11','Americom 11',67861,'In Earth orbit','','2004-07-20 00:00:00',1436.06,35784,35788,.02,'Stationary' from dual union all
select 28253,'028253','2004-017B','Centaur AC-166','Centaur IIA AC-166',67861,'Reentered','2014-06-16 00:00:00','2004-07-19 00:00:00',618.4,203,35129,12.53,'GEO Transfer' from dual union all
select 28254,'028254','2004-018A','Formosat-2','ROCSAT-2',67862,'In Earth orbit','','2004-05-23 00:00:00',99.48,725,743,99.1,'Sun-Synch' from dual union all
select 28255,'028255','2004-018B','Orion 38','Orion 38',67862,'In Earth orbit','','2004-05-22 00:00:00',99.4,719,741,99.11,'Sun-Synch' from dual union all
select 28256,'028256','2004-010F','Blok DM-2 No. 105L','11S861 No. 105L',67854,'In Earth orbit','','2004-05-28 00:00:00',1438.51,35271,36396,1.21,'Drift GEO' from dual union all
select 28257,'028257','1989-001R','deb SOZ','',62278,'Reentered','2004-05-26 00:00:00','2004-05-24 00:00:00',97.37,230,1036,66.03,'Intermediate' from dual union all
select 28258,'028258','1989-001S','deb SOZ','',62278,'Reentered','2007-03-06 00:00:00','2004-05-24 00:00:00',349.31,619,19485,65.38,'Highly Ellip' from dual union all
select 28259,'028259','1989-001T','deb SOZ','',62278,'Reentered','2008-12-23 00:00:00','2004-05-09 00:00:00',342.67,467,19219,65.49,'Highly Ellip' from dual union all
select 2826,'002826','1967-053A','Calsphere IV','NRL PL160',15308,'In Earth orbit','','1967-07-24 00:00:00',103.5,913,936,69.97,'Intermediate' from dual union all
select 28260,'028260','1989-001U','deb SOZ','',62278,'In Earth orbit','','2004-05-23 00:00:00',334.32,461,18696,65.48,'Highly Ellip' from dual union all
select 28261,'028261','2004-019A','Progress M-49','Progress M No. 249',67863,'Deorbited','2004-07-30 00:00:00','2004-05-25 00:00:00',89.32,236,246,51.63,'Intermediate' from dual union all
select 28262,'028262','2004-019B','11S510','11S510',67863,'Reentered','2004-05-28 00:00:00','2004-05-25 00:00:00',88.57,184,223,51.64,'Intermediate' from dual union all
select 28263,'028263','2000-071D','Star 48 yo weight','',66973,'Reentered','2013-12-26 00:00:00','2004-01-16 00:00:00',289.74,198,16055,38.98,'Highly Ellip' from dual union all
select 28264,'028264','2003-010D','Star 48 yo weight','',67634,'Reentered','2016-10-01 00:00:00','2004-05-01 00:00:00',342.64,198,19486,39.19,'Highly Ellip' from dual union all
select 28265,'028265','1966-052E','deb FW4S','',13459,'In Earth orbit','','2004-05-06 00:00:00',139.22,642,4398,40.88,'Medium' from dual union all
select 28266,'028266','1968-055L','deb Star 37D','',16782,'In Earth orbit','','2004-05-25 00:00:00',137.33,706,4173,120.64,'Medium' from dual union all
select 28267,'028267','1978-096H','deb Tiros-N','',45291,'Reentered','2005-05-09 00:00:00','2004-05-24 00:00:00',101.07,768,851,98.51,'Sun-Synch' from dual union all
select 28268,'028268','1980-050L','deb Kosmos-1188','deb Oko',50260,'In Earth orbit','','2004-05-19 00:00:00',743.41,5082,36530,69.79,'GEO Transfer' from dual;
insert into SATELLITE
select 28269,'028269','1981-016K','deb Kosmos-1247','',52539,'In Earth orbit','','2004-04-27 00:00:00',711.22,2001,38030,65.93,'GEO Transfer' from dual union all
select 2827,'002827','1967-054A','Kosmos-162','Zenit-4 No. 30',15309,'Landed','1967-06-09 00:00:00','1967-06-03 00:00:00',89.16,191,274,51.71,'Intermediate' from dual union all
select 28270,'028270','1981-016L','deb Kosmos-1247','',52539,'In Earth orbit','','2004-05-23 00:00:00',711.26,2033,38000,65.65,'GEO Transfer' from dual union all
select 28271,'028271','1984-069AG','deb US-A coolant','',57123,'In Earth orbit','','2004-05-08 00:00:00',103.61,884,975,65.06,'Intermediate' from dual union all
select 28272,'028272','1984-069AH','deb US-A coolant','',57123,'In Earth orbit','','2004-05-06 00:00:00',103.61,885,974,65.04,'Intermediate' from dual union all
select 28273,'028273','1984-069AJ','deb US-A coolant','',57123,'In Earth orbit','','2004-05-21 00:00:00',103.59,883,974,65.05,'Intermediate' from dual union all
select 28274,'028274','1984-069AK','deb US-A coolant','',57123,'In Earth orbit','','2004-05-24 00:00:00',103.6,884,974,65.05,'Intermediate' from dual union all
select 28275,'028275','1988-019F','deb US-A coolant','',61268,'In Earth orbit','','2004-05-18 00:00:00',104.3,940,983,65.04,'Intermediate' from dual union all
select 28276,'028276','1988-085CP','deb SOZ','',61334,'Reentered','2005-01-07 00:00:00','2004-05-12 00:00:00',233.87,555,11836,65.58,'Medium' from dual union all
select 28277,'028277','1991-068AS','deb S5M','',64237,'In Earth orbit','','2004-05-19 00:00:00',112.19,1312,1342,82.6,'Intermediate' from dual union all
select 28278,'028278','1991-068AT','deb S5M','',64237,'In Earth orbit','','2004-05-24 00:00:00',113.21,1303,1445,82.29,'Intermediate' from dual union all
select 28279,'028279','1992-072N','deb Ariane V54','',64784,'In Earth orbit','','2004-05-18 00:00:00',496.41,996,27783,8.14,'Highly Ellip' from dual union all
select 2828,'002828','1967-053C','GGSE 4','POPPY 5 SV2',15308,'In Earth orbit','','1967-07-04 00:00:00',103.42,915,926,69.98,'Intermediate' from dual union all
select 28280,'028280','1992-072P','deb Ariane V54','',64784,'In Earth orbit','','2003-12-21 00:00:00',493.58,870,27751,8.01,'Highly Ellip' from dual union all
select 28281,'028281','1993-014J','deb Start','',65116,'Reentered','2005-11-19 00:00:00','2004-05-17 00:00:00',98.15,574,767,75.74,'Intermediate' from dual union all
select 28282,'028282','1993-014K','deb Start','',65116,'Reentered','2015-02-05 00:00:00','2004-05-21 00:00:00',100.52,664,903,75.75,'Intermediate' from dual union all
select 28283,'028283','1993-014L','deb Start','',65116,'In Earth orbit','','2004-05-24 00:00:00',100.7,673,910,75.77,'Intermediate' from dual union all
select 28284,'028284','1993-014M','deb Start','',65116,'In Earth orbit','','2004-05-24 00:00:00',100.73,676,911,75.75,'Intermediate' from dual union all
select 28285,'028285','1993-014N','deb Start','',65116,'Reentered','2011-11-04 00:00:00','2004-05-18 00:00:00',99.98,642,874,75.75,'Intermediate' from dual union all
select 28286,'028286','1993-014P','deb Start','',65116,'Reentered','2015-02-16 00:00:00','2004-05-21 00:00:00',100.63,671,906,75.76,'Intermediate' from dual union all
select 28287,'028287','1993-014Q','deb Start','',65116,'Reentered','2007-01-05 00:00:00','2004-05-01 00:00:00',98.85,601,807,75.75,'Intermediate' from dual union all
select 28288,'028288','1993-014R','deb Start','',65116,'In Earth orbit','','2004-05-23 00:00:00',100.4,659,896,75.75,'Intermediate' from dual union all
select 28289,'028289','1993-014S','deb Start','',65116,'Reentered','2011-10-03 00:00:00','2004-05-05 00:00:00',99.66,644,841,75.7,'Intermediate' from dual union all
select 2829,'002829','1967-054B','11A57I','11A57I',15309,'Reentered','1967-06-04 00:00:00','1967-06-03 00:00:00',88.64,187,227,51.79,'Intermediate' from dual union all
select 28290,'028290','1993-014T','deb Start','',65116,'In Earth orbit','','2004-04-30 00:00:00',100.61,667,908,75.76,'Intermediate' from dual union all
select 28291,'028291','1993-014U','deb Start','',65116,'In Earth orbit','','2004-05-25 00:00:00',100.57,666,906,75.76,'Intermediate' from dual union all
select 28292,'028292','1993-014V','deb Start','',65116,'In Earth orbit','','2004-05-24 00:00:00',100.76,674,916,75.76,'Intermediate' from dual union all
select 28293,'028293','1995-058K','Zenit part','',65719,'Reentered','2005-07-02 00:00:00','2004-05-25 00:00:00',97.14,619,625,70.99,'Intermediate' from dual union all
select 28294,'028294','1991-082F','deb DMSP F-11','',64251,'Reentered','2005-05-17 00:00:00','2004-05-26 00:00:00',101.15,761,865,98.68,'Sun-Synch' from dual union all
select 28295,'028295','1991-082G','deb DMSP F-11','',64251,'Reentered','2004-10-30 00:00:00','2004-05-26 00:00:00',100.93,626,980,98.65,'Sun-Synch' from dual union all
select 28296,'028296','1991-082H','deb DMSP F-11','',64251,'Reentered','2005-05-05 00:00:00','2004-05-26 00:00:00',101.28,775,864,98.65,'Sun-Synch' from dual union all
select 28297,'028297','1991-082J','deb DMSP F-11','',64251,'Reentered','2011-06-12 00:00:00','2004-05-26 00:00:00',101.41,809,842,98.62,'Sun-Synch' from dual union all
select 28298,'028298','1991-082K','deb DMSP F-11','',64251,'Reentered','2006-01-03 00:00:00','2004-05-26 00:00:00',101.8,780,908,98.67,'Sun-Synch' from dual union all
select 28299,'028299','1991-082L','deb DMSP F-11','',64251,'Reentered','2005-12-18 00:00:00','2004-05-26 00:00:00',101.91,811,887,98.66,'Sun-Synch' from dual union all
select 283,'000283','62 Nu 2','S1','S1',8195,'Reentered','1962-08-05 00:00:00','1962-06-13 00:00:00',92.47,225,567,48.99,'Intermediate' from dual union all
select 2830,'002830','1967-054C','deb Kosmos-162','',15309,'Reentered','1967-06-02 00:00:00','1967-06-01 00:00:00',88.73,212,212,51.87,'Intermediate' from dual union all
select 28300,'028300','1991-082M','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',101.73,832,850,98.68,'Sun-Synch' from dual union all
select 28301,'028301','1991-082N','deb DMSP F-11','',64251,'Reentered','2012-07-19 00:00:00','2004-05-26 00:00:00',101.76,813,870,98.6,'Sun-Synch' from dual union all
select 28302,'028302','1991-082P','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',101.99,830,875,98.75,'Sun-Synch' from dual union all
select 28303,'028303','1991-082Q','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',101.1,789,833,98.72,'Sun-Synch' from dual union all
select 28304,'028304','1991-082R','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',101.72,832,849,98.66,'Sun-Synch' from dual union all
select 28305,'028305','1991-082S','deb DMSP F-11','',64251,'Reentered','2012-02-10 00:00:00','2004-05-26 00:00:00',101.59,813,855,98.23,'Sun-Synch' from dual union all
select 28306,'028306','1991-082T','deb DMSP F-11','',64251,'Reentered','2005-09-21 00:00:00','2004-05-26 00:00:00',101.12,776,848,98.45,'Sun-Synch' from dual union all
select 28307,'028307','1991-082U','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.18,834,890,98.69,'Sun-Synch' from dual union all
select 28308,'028308','1991-082V','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',101.78,833,853,98.7,'Sun-Synch' from dual union all
select 28309,'028309','1991-082W','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',101.33,811,832,98.71,'Sun-Synch' from dual union all
select 2831,'002831','1967-055A','OPS 4360','GAMBIT SV 988',15310,'Reentered','1967-06-12 00:00:00','1967-06-06 00:00:00',88.65,145,270,104.88,'Retrograde' from dual union all
select 28310,'028310','1991-082X','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',101.66,831,844,98.65,'Sun-Synch' from dual union all
select 28311,'028311','1991-082Y','deb DMSP F-11','',64251,'Reentered','2005-06-17 00:00:00','2004-05-26 00:00:00',101.46,692,964,98.47,'Sun-Synch' from dual union all
select 28312,'028312','1991-082Z','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',101.69,831,846,98.66,'Sun-Synch' from dual union all
select 28313,'028313','1991-082AA','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.06,832,881,98.66,'Sun-Synch' from dual union all
select 28314,'028314','1991-082AB','deb DMSP F-11','',64251,'Reentered','2014-09-20 00:00:00','2004-05-26 00:00:00',100.08,675,850,98.97,'Sun-Synch' from dual union all
select 28315,'028315','1991-082AC','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',103.92,832,1056,98.69,'Sun-Synch' from dual union all
select 28316,'028316','1991-082AD','deb DMSP F-11','',64251,'Reentered','2006-01-01 00:00:00','2004-05-26 00:00:00',99.08,580,850,99.01,'Sun-Synch' from dual union all
select 28317,'028317','1991-082AE','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.13,763,956,98.95,'Sun-Synch' from dual union all
select 28318,'028318','1991-082AF','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.34,816,923,98.78,'Sun-Synch' from dual union all
select 28319,'028319','1991-082AG','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.25,809,922,98.6,'Sun-Synch' from dual union all
select 2832,'002832','1967-056A','Kosmos-163','DS-U2-MP No. 2',15311,'Reentered','1967-10-11 00:00:00','1967-07-17 00:00:00',92.56,251,548,48.4,'Intermediate' from dual union all
select 28320,'028320','1991-082AH','deb DMSP F-11','',64251,'Reentered','2017-07-03 00:00:00','2004-05-26 00:00:00',102.33,824,913,98.8,'Sun-Synch' from dual union all
select 28321,'028321','1991-082AJ','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',101.67,832,843,98.66,'Sun-Synch' from dual union all
select 28322,'028322','1991-082AK','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',103.38,832,1004,98.58,'Sun-Synch' from dual union all
select 28323,'028323','1991-082AL','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.91,819,973,98.75,'Sun-Synch' from dual union all
select 28324,'028324','1991-082AM','deb DMSP F-11','',64251,'Reentered','2015-10-23 00:00:00','2004-05-26 00:00:00',101.71,832,848,98.68,'Sun-Synch' from dual union all
select 28325,'028325','1991-082AN','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',103.02,832,971,98.64,'Sun-Synch' from dual union all
select 28326,'028326','1991-082AP','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.62,828,938,98.8,'Sun-Synch' from dual union all
select 28327,'028327','1991-082AQ','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.01,829,879,98.65,'Sun-Synch' from dual union all
select 28328,'028328','1991-082AR','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-10 00:00:00',102.98,824,976,98.8,'Sun-Synch' from dual union all
select 28329,'028329','1991-082AS','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.93,823,971,98.77,'Sun-Synch' from dual union all
select 2833,'002833','1967-056B','S1M','S1M',15311,'Reentered','1967-09-12 00:00:00','1967-07-17 00:00:00',92.23,245,522,48.4,'Intermediate' from dual union all
select 28330,'028330','1991-082AT','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.86,822,966,98.75,'Sun-Synch' from dual union all
select 28331,'028331','1991-082AU','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',100.75,755,833,98.7,'Sun-Synch' from dual union all
select 28332,'028332','1991-082AV','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.49,827,926,98.77,'Sun-Synch' from dual union all
select 28333,'028333','1991-082AW','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',101.7,832,847,98.67,'Sun-Synch' from dual union all
select 28334,'028334','1991-082AX','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.57,812,949,98.66,'Sun-Synch' from dual union all
select 28335,'028335','1991-082AY','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',101.76,833,851,98.64,'Sun-Synch' from dual union all
select 28336,'028336','1991-082AZ','deb DMSP F-11','',64251,'Reentered','2005-05-21 00:00:00','2004-05-26 00:00:00',101.48,784,874,98.49,'Sun-Synch' from dual union all
select 28337,'028337','1991-082BA','deb DMSP F-11','',64251,'Reentered','2005-01-30 00:00:00','2004-05-26 00:00:00',101.75,787,896,98.55,'Sun-Synch' from dual union all
select 28338,'028338','1991-082BB','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',101.73,832,850,98.67,'Sun-Synch' from dual union all
select 28339,'028339','1991-082BC','deb DMSP F-11','',64251,'Reentered','2006-02-27 00:00:00','2004-05-26 00:00:00',101.86,774,920,98.63,'Sun-Synch' from dual union all
select 2834,'002834','1967-053D','GGSE 5','POPPY 5 SV4',15308,'In Earth orbit','','1967-07-14 00:00:00',103.4,913,926,69.98,'Intermediate' from dual union all
select 28340,'028340','1991-082BD','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.08,831,883,98.64,'Sun-Synch' from dual union all
select 28341,'028341','1991-082BE','deb DMSP F-11','',64251,'Reentered','2015-02-04 00:00:00','2004-05-26 00:00:00',102.32,806,931,98.59,'Sun-Synch' from dual union all
select 28342,'028342','1991-082BF','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.83,820,964,98.72,'Sun-Synch' from dual union all
select 28343,'028343','1991-082BG','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.89,826,964,98.81,'Sun-Synch' from dual union all
select 28344,'028344','1991-082BH','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',101.66,832,843,98.65,'Sun-Synch' from dual union all
select 28345,'028345','1991-082BJ','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-25 00:00:00',104.02,833,1064,98.64,'Sun-Synch' from dual union all
select 28346,'028346','1991-082BK','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-25 00:00:00',101.75,833,850,98.68,'Sun-Synch' from dual union all
select 28347,'028347','1991-082BL','deb DMSP F-11','',64251,'Reentered','2005-06-19 00:00:00','2004-05-26 00:00:00',101.75,764,919,98.64,'Sun-Synch' from dual union all
select 28348,'028348','1991-082BM','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',101.76,831,853,98.66,'Sun-Synch' from dual union all
select 28349,'028349','1991-082BN','deb DMSP F-11','',64251,'In Earth orbit','','2004-05-26 00:00:00',102.8,825,957,98.76,'Sun-Synch' from dual union all
select 2835,'002835','1967-055B','Agena D 4837','Agena D',15310,'Reentered','1967-06-06 00:00:00','1967-06-05 00:00:00',90.3,143,435,104.87,'Retrograde' from dual union all
select 28350,'028350','2004-020A','Kosmos-2405','US-PU',67864,'Reentered','2006-06-16 00:00:00','2004-05-28 00:00:00',92.76,404,415,65.03,'Intermediate' from dual union all
select 28351,'028351','2004-020B','11S692','11S692',67864,'Reentered','2004-05-29 00:00:00','2004-05-28 00:00:00',88.4,100,291,65,'Intermediate' from dual union all
select 28352,'028352','2004-021A','Kosmos-2406','Tselina-2',67865,'In Earth orbit','','2004-06-17 00:00:00',102.04,846,865,71,'Intermediate' from dual union all
select 28353,'028353','2004-021B','11S772','11S772',67865,'In Earth orbit','','2004-06-18 00:00:00',101.83,841,850,71,'Intermediate' from dual union all
select 28354,'028354','2004-021C','Sep Motor Cover','Sep Motor Cover',67865,'In Earth orbit','','2004-06-17 00:00:00',105.27,843,1171,70.96,'Intermediate' from dual union all
select 28355,'028355','2004-021D','Sep Motor Cover','Sep Motor Cover',67865,'In Earth orbit','','2004-06-17 00:00:00',105.35,842,1180,70.97,'Intermediate' from dual union all
select 28356,'028356','2004-021E','Sep Motor Cover','Sep Motor Cover',67865,'In Earth orbit','','2004-06-17 00:00:00',105.25,843,1169,71.07,'Intermediate' from dual union all
select 28357,'028357','2004-021F','Sep Motor Cover','Sep Motor Cover',67865,'In Earth orbit','','2004-06-17 00:00:00',104.93,843,1139,71.07,'Intermediate' from dual union all
select 28358,'028358','2004-022A','INTELSAT 10-02','INTELSAT 10-02',67866,'In Earth orbit','','2004-08-19 00:00:00',1437.19,35803,35813,.03,'Drift GEO' from dual;
insert into SATELLITE
select 28359,'028359','2004-022B','Briz-M DTB','Briz-M torus tank',67866,'In Earth orbit','','2004-08-21 00:00:00',246.14,294,12969,49.61,'Medium' from dual union all
select 2836,'002836','1967-057A','Kosmos-164','Zenit-2 No. 50',15312,'Landed','1967-06-14 00:00:00','1967-06-12 00:00:00',89.42,196,295,65.64,'Intermediate' from dual union all
select 28360,'028360','2004-022C','Briz-M No. 88509','Briz-M 14S43 No. 88509',67866,'In Earth orbit','','2004-08-20 00:00:00',714.68,4297,35905,23.67,'GEO Transfer' from dual union all
select 28361,'028361','2004-023A','GPS IIR-12','Navstar GPS SVN 60',67867,'In Earth orbit','','2004-08-23 00:00:00',718,20089,20277,55.07,'Medium' from dual union all
select 28362,'028362','2004-023B','Delta 305','Delta SSPS AJ-10-118K',67867,'Reentered','2004-09-01 00:00:00','2004-06-26 00:00:00',99.46,171,1294,32.92,'Intermediate' from dual union all
select 28363,'028363','2004-023C','Star 48B','Star 48B',67867,'Reentered','2010-06-30 00:00:00','2004-08-23 00:00:00',351.39,185,20049,39.03,'Highly Ellip' from dual union all
select 28364,'028364','2004-024A','Telstar 18','Apstar 5',67868,'In Earth orbit','','2004-08-29 00:00:00',1436.11,35776,35797,.04,'Stationary' from dual union all
select 28365,'028365','2004-024B','Blok DM-SL No. 11L','Blok DM-SL',67868,'In Earth orbit','','2004-09-02 00:00:00',385.72,712,21637,.06,'Highly Ellip' from dual union all
select 28366,'028366','2004-025A','Aprizesat-2','Latinsat D',67869,'In Earth orbit','','2004-06-30 00:00:00',100.31,695,852,98.27,'Sun-Synch' from dual union all
select 28367,'028367','2004-025B','R-36M2 DS','R-36M2 DS',67869,'In Earth orbit','','2004-06-29 00:00:00',103.76,680,1193,98.23,'Sun-Synch' from dual union all
select 28368,'028368','2004-025C','Demeter','Demeter',67869,'In Earth orbit','','2004-06-29 00:00:00',98.74,685,712,98.26,'Sun-Synch' from dual union all
select 28369,'028369','2004-025D','SaudiComsat-1','SaudiComsat-1',67869,'In Earth orbit','','2004-06-30 00:00:00',99.11,699,734,98.27,'Sun-Synch' from dual union all
select 2837,'002837','1967-057B','11A57I','11A57I',15312,'Reentered','1967-06-17 00:00:00','1967-06-10 00:00:00',89.13,187,276,65.66,'Intermediate' from dual union all
select 28370,'028370','2004-025E','SaudiComsat-2','SaudiComsat-2',67869,'In Earth orbit','','2004-06-30 00:00:00',99.44,699,765,98.26,'Sun-Synch' from dual union all
select 28371,'028371','2004-025F','SaudiSat-2','SaudiSat-2',67869,'In Earth orbit','','2004-06-30 00:00:00',98.96,696,722,98.27,'Sun-Synch' from dual union all
select 28372,'028372','2004-025G','Aprizesat-1','Latinsat C',67869,'In Earth orbit','','2004-06-29 00:00:00',99.27,699,749,98.26,'Sun-Synch' from dual union all
select 28373,'028373','2004-025H','UniSat-3','Unisat-3',67869,'In Earth orbit','','2004-06-29 00:00:00',99.6,700,779,98.26,'Sun-Synch' from dual union all
select 28374,'028374','2004-025J','Dnepr plume shield','GDS',67869,'In Earth orbit','','2004-07-06 00:00:00',98.74,685,712,98.27,'Sun-Synch' from dual union all
select 28375,'028375','2004-025K','AMSAT-OSCAR-51','AMSAT-Echo',67869,'In Earth orbit','','2004-06-30 00:00:00',99.79,698,799,98.26,'Sun-Synch' from dual union all
select 28376,'028376','2004-026A','Aura','EOS Chem-1',67870,'In Earth orbit','','2004-07-23 00:00:00',98.3,673,682,98.22,'Sun-Synch' from dual union all
select 28377,'028377','2004-026B','Delta 306','Delta SSPS AJ-10-118K',67870,'Reentered','2004-09-23 00:00:00','2004-07-22 00:00:00',93.14,201,655,103.03,'Retrograde' from dual union all
select 28378,'028378','2004-027A','Anik F2','Anik F2',67871,'In Earth orbit','','2004-09-16 00:00:00',1436.04,35785,35785,.05,'Stationary' from dual union all
select 28379,'028379','2004-027B','EPS 519','EPS S/N 518 +',67871,'In Earth orbit','','2004-09-20 00:00:00',692.69,660,38450,7.03,'GEO Transfer' from dual union all
select 2838,'002838','1967-057C','deb Kosmos-164','',15312,'Reentered','1967-06-09 00:00:00','1967-06-09 00:00:00',87.34,130,156,65.65,'Intermediate' from dual union all
select 28380,'028380','2004-028A','Kosmos-2407','Parus No. 203',67872,'In Earth orbit','','2004-07-30 00:00:00',104.65,951,1005,82.96,'Intermediate' from dual union all
select 28381,'028381','2004-028B','S3M','S3M',67872,'In Earth orbit','','2004-07-31 00:00:00',104.51,950,993,82.95,'Intermediate' from dual union all
select 28382,'028382','2004-029A','Tan Ce 2','Double Star DSP-P',67873,'In Earth orbit','','2004-09-24 00:00:00',695.06,605,38623,89.91,'GEO Transfer' from dual union all
select 28383,'028383','2001-049NM','deb PSLV-C3 PS4','',67202,'Reentered','2005-11-01 00:00:00','2004-08-20 00:00:00',94.08,447,501,97.69,'Sun-Synch' from dual union all
select 28384,'028384','2004-034A','USA 179','QUASAR 14',67878,'In Earth orbit','','2004-10-01 00:00:00',712.62,300,39800,63,'Molniya' from dual union all
select 28385,'028385','2004-034B','Centaur AC-167','Centaur IIA AC-167',67878,'In Earth orbit','','2004-11-02 00:00:00',279.76,277,15305,57.41,'Highly Ellip' from dual union all
select 28386,'028386','1998-067AE','Antenna cover','',66503,'Reentered','2004-10-05 00:00:00','2004-09-05 00:00:00',91.7,355,361,51.64,'Intermediate' from dual union all
select 28387,'028387','1999-035E','FUSE deb','',66716,'Reentered','2004-12-11 00:00:00','2004-09-27 00:00:00',98.85,685,723,24.97,'Equatorial' from dual union all
select 28388,'028388','1981-043E','deb Meteor-2','',52566,'In Earth orbit','','2004-07-29 00:00:00',102.1,821,895,81.27,'Intermediate' from dual union all
select 28389,'028389','1981-043F','deb Meteor-2','',52566,'In Earth orbit','','2004-07-30 00:00:00',102.09,822,894,81.27,'Intermediate' from dual union all
select 2839,'002839','1967-058C','11S510','11S510',15313,'Reentered','1967-06-12 00:00:00','1967-06-12 00:00:00',87.97,158,189,51.8,'Intermediate' from dual union all
select 28390,'028390','2004-030B','Delta 307','Delta SSPS AJ-10-118K',67874,'Reentered','2005-05-02 00:00:00','2004-10-02 00:00:00',159.01,152,6537,32.21,'Medium' from dual union all
select 28391,'028391','2004-030A','Messenger','MESSENGER',67874,'Deep Space','2004-08-03 00:00:00','2004-08-05 00:00:00',631.3,190,61700,32.66,'Earth Escape' from dual union all
select 28392,'028392','2004-030C','Star 48B','Star 48B',67874,'Deep Space','2004-08-03 00:00:00','2004-08-05 00:00:00',631.3,190,61700,32.66,'Earth Escape' from dual union all
select 28393,'028393','2004-031A','Amazonas-1','Amazonas',67875,'In Earth orbit','','2004-10-06 00:00:00',1435.98,35769,35799,.09,'Stationary' from dual union all
select 28394,'028394','2004-031B','Briz-M No. 88508','Briz-M 14S43 No. 88508',67875,'In Earth orbit','','2004-10-14 00:00:00',684.77,3251,35463,9.01,'GEO Transfer' from dual union all
select 28395,'028395','2004-031C','Briz-M DTB','Briz-M torus tank',67875,'In Earth orbit','','2004-10-04 00:00:00',297.25,303,16449,49.41,'Highly Ellip' from dual union all
select 28396,'028396','2004-038A','Kosmos-2410','Kobal''t-M No. 556',67882,'Landed','2005-01-09 00:00:00','2004-10-01 00:00:00',89.94,213,330,67.16,'Intermediate' from dual union all
select 28397,'028397','2004-038B','11S510','11S510',67882,'Reentered','2004-10-01 00:00:00','2004-09-26 00:00:00',89.36,162,324,67.15,'Intermediate' from dual union all
select 28398,'028398','1999-035F','FUSE deb','',66716,'Reentered','2006-08-22 00:00:00','2004-09-26 00:00:00',100.01,739,778,25,'Equatorial' from dual union all
select 28399,'028399','2004-032A','Progress M-50','Progress 7K-TGM No. 350',67876,'Deorbited','2004-12-22 00:00:00','2004-08-18 00:00:00',101.71,767,913,71.4,'Intermediate' from dual union all
select 284,'000284','62 Xi 2','8K72KE','8K72KE',8206,'Reentered','1962-06-17 00:00:00','1962-05-13 00:00:00',90.26,287,288,64.95,'Intermediate' from dual union all
select 2840,'002840','1967-058A','Venera-4','4V-1 No. 310',15313,'Deep Space','1967-06-14 00:00:00','',null,null,null,null,'' from dual union all
select 28400,'028400','2004-032B','11S510','11S510',67876,'Reentered','2004-08-13 00:00:00','2004-08-11 00:00:00',88.59,181,229,51.65,'Intermediate' from dual union all
select 28401,'028401','1978-083L','deb Kosmos-1030','deb Oko',45278,'Reentered','2004-08-20 00:00:00','2004-08-18 00:00:00',106.87,121,2043,61.68,'Intermediate' from dual union all
select 28402,'028402','2004-033A','Fanhui Shi Weixing (19)','Jianbing-2 01 xing',67877,'Reentered','2004-11-07 00:00:00','2004-08-29 00:00:00',91.08,166,490,63,'Intermediate' from dual union all
select 28403,'028403','2004-033B','CZ-2C Stage 2','CZ-2C Stage 2',67877,'Reentered','2004-09-10 00:00:00','2004-08-29 00:00:00',91.03,164,487,63,'Intermediate' from dual union all
select 28404,'028404','2004-033C','CZ-2C Stage 2 motor cover','',67877,'Reentered','2004-08-31 00:00:00','2004-08-30 00:00:00',91.47,164,530,63.3,'Intermediate' from dual union all
select 28405,'028405','2004-033D','CZ-2C Stage 2 motor cover','',67877,'Reentered','2004-08-31 00:00:00','2004-08-31 00:00:00',89.36,150,335,63.21,'Intermediate' from dual union all
select 28406,'028406','1999-035G','FUSE deb','',66716,'Reentered','2005-02-08 00:00:00','2004-09-26 00:00:00',99.33,684,770,24.98,'Equatorial' from dual union all
select 28407,'028407','1999-035H','FUSE deb','',66716,'Reentered','2010-10-14 00:00:00','2004-09-27 00:00:00',99.52,728,744,24.97,'Equatorial' from dual union all
select 28408,'028408','1998-067Z','Antenna cover','',66503,'Reentered','2004-10-12 00:00:00','2004-09-04 00:00:00',91.66,349,363,51.63,'Intermediate' from dual union all
select 28409,'028409','1998-067AA','Antenna cover','',66503,'Reentered','2004-10-12 00:00:00','2004-09-04 00:00:00',91.68,352,362,51.63,'Intermediate' from dual union all
select 2841,'002841','1967-058B','BOZ','BOZ',15313,'Reentered','1967-06-13 00:00:00','1967-06-12 00:00:00',88.3,171,210,51.78,'Intermediate' from dual union all
select 28410,'028410','1998-067AB','Antenna cover','',66503,'Reentered','2004-10-12 00:00:00','2004-09-04 00:00:00',91.68,356,358,51.64,'Intermediate' from dual union all
select 28411,'028411','1998-067AC','Antenna cover','',66503,'Reentered','2004-10-03 00:00:00','2004-09-04 00:00:00',91.69,356,359,51.64,'Intermediate' from dual union all
select 28412,'028412','1998-067AD','Zarya PIG/RRZh1','',66503,'Reentered','2005-01-25 00:00:00','2004-09-04 00:00:00',91.74,356,363,51.63,'Intermediate' from dual union all
select 28413,'028413','2004-035A','Shi Jian 6/1B','shijian liuhao B xing',67879,'In Earth orbit','','2004-09-16 00:00:00',96.63,593,602,97.74,'Sun-Synch' from dual union all
select 28414,'028414','2004-035B','Shi Jian 6/1A','shijian liuhao A xing',67879,'In Earth orbit','','2004-09-16 00:00:00',96.36,577,591,97.74,'Sun-Synch' from dual union all
select 28415,'028415','2004-035C','CZ-4B Stage  3','CZ-4B',67879,'In Earth orbit','','2004-09-15 00:00:00',96.33,529,637,97.63,'Sun-Synch' from dual union all
select 28416,'028416','2004-035D','Adapter','',67879,'Reentered','2017-03-10 00:00:00','2004-09-16 00:00:00',96.6,591,601,97.74,'Sun-Synch' from dual union all
select 28417,'028417','2004-036A','EDUSAT','GSAT-3',67880,'In Earth orbit','','2004-11-22 00:00:00',1436.06,35770,35802,.01,'Stationary' from dual union all
select 28418,'028418','2004-036B','GSLV CS-3','GSLV-F01 12KRB stage',67880,'Reentered','2007-11-24 00:00:00','2004-11-19 00:00:00',619.64,165,35231,19.52,'GEO Transfer' from dual union all
select 28419,'028419','2004-037A','Kosmos-2408','Strela-3 No. 49',67881,'In Earth orbit','','2004-09-30 00:00:00',115.6,1470,1496,82.48,'Intermediate' from dual union all
select 2842,'002842','1967-059A','Kosmos-165','DS-P1-Yu No. 11',15314,'Reentered','1968-01-15 00:00:00','1967-07-18 00:00:00',100.84,198,1399,81.9,'Intermediate' from dual union all
select 28420,'028420','2004-037B','Kosmos-2409','Strela-3 No. 54',67881,'In Earth orbit','','2004-09-30 00:00:00',115.63,1473,1494,82.48,'Intermediate' from dual union all
select 28421,'028421','2004-037C','S3M','S3M',67881,'In Earth orbit','','2004-09-30 00:00:00',115.7,1474,1501,82.48,'Intermediate' from dual union all
select 28422,'028422','1999-035J','FUSE deb','',66716,'Reentered','2004-10-27 00:00:00','2004-09-30 00:00:00',89.9,259,280,51.63,'Intermediate' from dual union all
select 28423,'028423','1999-035K','FUSE deb','',66716,'Reentered','2011-10-17 00:00:00','2004-08-26 00:00:00',100.24,736,804,24.97,'Equatorial' from dual union all
select 28424,'028424','2004-039A','Fanhui Shi Weixing (20)','Jianbing-4 02 xing',67883,'Reentered','2004-11-09 00:00:00','2004-10-04 00:00:00',89.79,206,323,63,'Intermediate' from dual union all
select 28425,'028425','2004-039B','CZ-2D Stage 2','CZ-2D Stage 2',67883,'Reentered','2004-10-10 00:00:00','2004-09-29 00:00:00',89.36,203,283,63,'Intermediate' from dual union all
select 28426,'028426','2004-039C','CZ-2D Stage 2 motor cover','',67883,'Reentered','2004-09-28 00:00:00','2004-09-27 00:00:00',89.17,197,270,62.93,'Intermediate' from dual union all
select 28427,'028427','2004-033E','FSW 19 RV adapter','',67877,'Reentered','2004-10-01 00:00:00','2004-09-29 00:00:00',92.27,149,623,63.02,'Intermediate' from dual union all
select 28428,'028428','2001-049NN','deb PSLV-C3 PS4','',67202,'Reentered','2013-01-23 00:00:00','2004-09-14 00:00:00',96.58,571,619,97.92,'Sun-Synch' from dual union all
select 28429,'028429','2001-049NP','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2004-09-21 00:00:00',97.37,606,660,97.95,'Sun-Synch' from dual union all
select 2843,'002843','1967-059B','S1M','S1M',15314,'Reentered','1967-11-01 00:00:00','1967-07-17 00:00:00',99.94,196,1315,81.91,'Intermediate' from dual union all
select 28430,'028430','2001-049NW','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2004-09-28 00:00:00',97.11,581,660,97.88,'Sun-Synch' from dual union all
select 28431,'028431','2001-049NX','deb PSLV-C3 PS4','',67202,'Reentered','2017-09-15 00:00:00','2004-09-29 00:00:00',97.19,591,658,97.95,'Sun-Synch' from dual union all
select 28432,'028432','2001-049NY','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2004-09-29 00:00:00',99.29,658,791,97.68,'Sun-Synch' from dual union all
select 28433,'028433','2001-049NZ','deb PSLV-C3 PS4','',67202,'Reentered','2011-01-06 00:00:00','2004-09-29 00:00:00',96.89,581,639,97.83,'Sun-Synch' from dual union all
select 28434,'028434','2001-049PA','deb PSLV-C3 PS4','',67202,'Reentered','2011-04-13 00:00:00','2004-09-14 00:00:00',98.06,592,740,97.78,'Sun-Synch' from dual union all
select 28435,'028435','2001-049PB','deb PSLV-C3 PS4','',67202,'Reentered','2014-04-05 00:00:00','2004-09-29 00:00:00',96.25,558,600,97.5,'Sun-Synch' from dual union all
select 28436,'028436','2001-056K','Sep Motor Cover','',67209,'In Earth orbit','','2004-09-28 00:00:00',108.4,1008,1298,99.41,'Sun-Synch' from dual union all
select 28437,'028437','2001-056L','Sep Motor Cover','',67209,'In Earth orbit','','2004-09-28 00:00:00',108.5,1005,1310,99.4,'Sun-Synch' from dual union all
select 28438,'028438','2002-024E','deb FY-1D','',67400,'In Earth orbit','','2004-09-26 00:00:00',101.35,805,840,98.73,'Sun-Synch' from dual union all
select 28439,'028439','2002-049C','deb CZ-4B','',67425,'Reentered','2010-12-11 00:00:00','2004-09-27 00:00:00',96.05,458,680,97.26,'Sun-Synch' from dual union all
select 2844,'002844','1967-058D','deb Blok-I','',15313,'Reentered','1967-06-13 00:00:00','1967-06-12 00:00:00',87.78,149,181,51.77,'Intermediate' from dual union all
select 28440,'028440','2002-049D','deb CZ-4B','',67425,'Reentered','2007-11-25 00:00:00','2004-09-25 00:00:00',95.26,449,614,97.37,'Sun-Synch' from dual union all
select 28441,'028441','2002-049E','deb CZ-4B','',67425,'Reentered','2011-04-22 00:00:00','2004-09-29 00:00:00',96.02,457,679,97.33,'Sun-Synch' from dual union all
select 28442,'028442','2002-049F','deb CZ-4B','',67425,'Reentered','2010-03-31 00:00:00','2004-09-26 00:00:00',95.71,445,661,97.3,'Sun-Synch' from dual union all
select 28443,'028443','62 Beta 5','deb Tiros 4','',8185,'In Earth orbit','','2004-09-25 00:00:00',99.17,662,776,48.28,'Intermediate' from dual union all
select 28444,'028444','2004-040A','Soyuz TMA-5','Soyuz 11F732 No. 215',67884,'Landed','2005-04-24 00:00:00','2004-10-22 00:00:00',91.74,354,366,51.64,'Intermediate' from dual union all
select 28445,'028445','2004-040B','11S510','11S510',67884,'Reentered','2004-10-16 00:00:00','2004-10-15 00:00:00',88.17,178,190,51.65,'Intermediate' from dual union all
select 28446,'028446','2004-041A','AMC 15','AMC-15',67885,'In Earth orbit','','2004-12-13 00:00:00',1433.89,35737,35750,.03,'Drift GEO' from dual union all
select 28447,'028447','2004-041B','Briz-M No. 88510','Briz-M 14S43 No. 88510',67885,'In Earth orbit','','2004-12-19 00:00:00',770.13,7079,35828,18.43,'Highly Ellip' from dual union all
select 28448,'028448','2004-029B','SM Y2','CTS',67873,'In Earth orbit','','2004-10-25 00:00:00',688.77,575,38339,89.9,'GEO Transfer' from dual;
insert into SATELLITE
select 28449,'028449','2004-039D','FSW capsule adapter','',67883,'Reentered','2004-10-17 00:00:00','2004-10-17 00:00:00',88.22,122,250,62.97,'Intermediate' from dual union all
select 2845,'002845','1967-060A','Mariner 5','Mariner 67-2',15315,'Deep Space','1967-06-16 00:00:00','1967-06-14 00:00:00',1650.41,184,105464,30.31,'Earth Escape' from dual union all
select 28450,'028450','2003-051E','FSW capsule adapter','',67675,'Reentered','2003-11-22 00:00:00','2003-11-21 00:00:00',89.98,125,422,62.9,'Intermediate' from dual union all
select 28451,'028451','2004-042A','Feng Yun 2C','FY-2 O4',67886,'In Earth orbit','','2004-12-20 00:00:00',1436.19,35782,35795,.68,'Stationary' from dual union all
select 28452,'028452','2004-042B','CZ-3A Stage 3','CZ-3A Stage 3',67886,'Reentered','2005-10-17 00:00:00','2004-12-19 00:00:00',315.17,128,17798,27.04,'Highly Ellip' from dual union all
select 28453,'028453','2001-034D','Genesis SRC','Sample Return Capsule',67187,'Deep Space Attached','2001-08-09 00:00:00','2001-08-10 00:00:00',80835.73,188,1225640,28.7,'Cislunar' from dual union all
select 28454,'028454','2002-056F','Micro-LabSat RITE Target 1','RITE Target 1',67432,'In Earth orbit','','2003-03-14 00:00:00',100.78,788,803,98.66,'Sun-Synch' from dual union all
select 28455,'028455','2002-056G','Micro-LabSat RITE Target 2','RITE Target 2',67432,'In Earth orbit','','2003-05-14 00:00:00',100.78,788,803,98.65,'Sun-Synch' from dual union all
select 28456,'028456','1983-044FT','deb Kosmos-1461','',55801,'Reentered','2007-11-25 00:00:00','2004-11-19 00:00:00',97.28,545,712,65.01,'Intermediate' from dual union all
select 28457,'028457','1983-113F','deb DMSP F-7','',55870,'Reentered','2004-11-18 00:00:00','2004-10-24 00:00:00',97.23,615,638,98.46,'Sun-Synch' from dual union all
select 28458,'028458','1983-113G','deb DMSP F-7','',55870,'Reentered','2006-05-06 00:00:00','2004-10-25 00:00:00',100.86,782,816,98.87,'Sun-Synch' from dual union all
select 28459,'028459','2004-035E','deb CZ-4B','',67879,'Reentered','2011-11-21 00:00:00','2004-10-26 00:00:00',96.65,597,600,97.69,'Sun-Synch' from dual union all
select 2846,'002846','1967-060B','Agena D 6933','Agena D 6933',15315,'Deep Space','1967-06-16 00:00:00','1967-06-14 00:00:00',1650.41,184,105464,30.31,'Earth Escape' from dual union all
select 28460,'028460','2004-035F','deb CZ-4B','',67879,'Reentered','2011-04-11 00:00:00','2004-10-26 00:00:00',96.42,576,599,97.74,'Sun-Synch' from dual union all
select 28461,'028461','2004-035G','CZ-4B sep motor cover','',67879,'Reentered','2012-02-11 00:00:00','2004-10-24 00:00:00',96.17,485,665,97.71,'Sun-Synch' from dual union all
select 28462,'028462','2004-035H','CZ-4B sep motor cover','',67879,'Reentered','2013-11-27 00:00:00','2004-10-19 00:00:00',96.86,534,683,97.78,'Sun-Synch' from dual union all
select 28463,'028463','2004-043A','Ekspress AM-1','Ekspress AM-1',67887,'In Earth orbit','','2004-12-29 00:00:00',1440.15,35794,35938,.03,'Drift GEO' from dual union all
select 28464,'028464','2004-043B','8S812','8S812',67887,'Reentered','2004-11-01 00:00:00','2004-10-30 00:00:00',88.21,183,189,51.62,'Intermediate' from dual union all
select 28465,'028465','2004-043C','Perekhodnik','Sredniy perekhodnik',67887,'Reentered','2004-10-30 00:00:00','2004-10-30 00:00:00',88.05,171,185,51.62,'Intermediate' from dual union all
select 28466,'028466','2004-043D','Blok DM-2M No. 15L','11S861-01 No. 15L',67887,'In Earth orbit','','2005-01-02 00:00:00',1438.39,35797,35866,.23,'Drift GEO' from dual union all
select 28467,'028467','2004-043E','deb 8S812','',67887,'Reentered','2004-11-05 00:00:00','2004-11-01 00:00:00',91.02,179,471,51.61,'Intermediate' from dual union all
select 28468,'028468','2004-043F','deb 8S812','',67887,'Reentered','2004-11-03 00:00:00','2004-11-01 00:00:00',90.63,177,434,51.61,'Intermediate' from dual union all
select 28469,'028469','2004-043G','deb 8S812','',67887,'Reentered','2004-11-03 00:00:00','2004-11-03 00:00:00',88.21,144,228,51.57,'Intermediate' from dual union all
select 2847,'002847','1967-053E','Timation 1','Timation 1',15308,'In Earth orbit','','1967-07-20 00:00:00',103.4,913,926,69.98,'Intermediate' from dual union all
select 28470,'028470','2004-044A','Zi Yuan 2-3','ziyuan erhao 03 xing',67888,'In Earth orbit','','2004-11-13 00:00:00',94.14,472,482,97.33,'Sun-Synch' from dual union all
select 28471,'028471','2004-044B','CZ-4B Stage 3','CZ-4B Stage 3',67888,'Reentered','2013-01-02 00:00:00','2004-11-08 00:00:00',94.37,397,580,97.2,'Sun-Synch' from dual union all
select 28472,'028472','2004-048A','AMC 16','AMC 16',67892,'In Earth orbit','','2005-02-16 00:00:00',1436.11,35783,35790,.02,'Stationary' from dual union all
select 28473,'028473','2004-048B','Centaur AV-005','Centaur AV-005',67892,'In Earth orbit','','2005-02-17 00:00:00',698.69,4793,34616,18.45,'GEO Transfer' from dual union all
select 28474,'028474','2004-045A','GPS IIR-13','Navstar GPS SVN 61',67889,'In Earth orbit','','2005-01-05 00:00:00',717.94,19936,20427,54.82,'Medium' from dual union all
select 28475,'028475','2004-045B','Delta 308','Delta SSPS AJ-10-118K',67889,'Reentered','2005-01-01 00:00:00','2004-11-09 00:00:00',99.13,170,1265,33.14,'Intermediate' from dual union all
select 28476,'028476','2004-045C','Star 48B','Star 48B',67889,'Reentered','2018-05-23 00:00:00','2005-01-05 00:00:00',346.89,163,19789,38.94,'Highly Ellip' from dual union all
select 28477,'028477','2004-041C','Briz-M DTB','Briz-M torus tank',67885,'In Earth orbit','','2004-12-16 00:00:00',643.37,893,35725,48.97,'GEO Transfer' from dual union all
select 28478,'028478','2004-043H','SOZ','SOZ',67887,'Reentered','2011-03-20 00:00:00','2004-12-29 00:00:00',635.04,354,35837,47.5,'GEO Transfer' from dual union all
select 28479,'028479','2004-046A','Shiyan Weixing 2','Shiyan 2',67890,'In Earth orbit','','2004-11-20 00:00:00',98.82,694,711,98.16,'Sun-Synch' from dual union all
select 2848,'002848','1967-061A','Kosmos-166','DS-U3-S No. 1',15316,'Reentered','1967-10-25 00:00:00','1967-07-20 00:00:00',92.67,277,534,48.43,'Intermediate' from dual union all
select 28480,'028480','2004-046B','CZ-2C Stage 2','CZ-2C Stage 2',67890,'In Earth orbit','','2004-11-21 00:00:00',101.06,705,913,98.05,'Sun-Synch' from dual union all
select 28481,'028481','2004-046C','CZ-2C Stage 2 motor cover','',67890,'In Earth orbit','','2004-11-21 00:00:00',100.88,693,908,98.03,'Sun-Synch' from dual union all
select 28482,'028482','2004-046D','CZ-2C Stage 2 motor cover','',67890,'In Earth orbit','','2004-11-21 00:00:00',101.05,697,920,98.35,'Sun-Synch' from dual union all
select 28483,'028483','2004-046E','CZ-2C Stage 2 motor cover','',67890,'In Earth orbit','','2004-11-20 00:00:00',100.45,686,874,98.05,'Sun-Synch' from dual union all
select 28484,'028484','2004-046F','CZ-2C Stage 2 motor cover','',67890,'In Earth orbit','','2004-11-20 00:00:00',100.65,697,882,98.37,'Sun-Synch' from dual union all
select 28485,'028485','2004-047A','Swift','Swift GRB Explorer',67891,'In Earth orbit','','2004-11-21 00:00:00',96.56,584,604,20.56,'Equatorial' from dual union all
select 28486,'028486','2004-047B','Delta 309','Delta SSPS AJ-10-118K',67891,'Reentered','2004-12-23 00:00:00','2004-11-20 00:00:00',92.48,197,596,19.47,'Equatorial' from dual union all
select 28487,'028487','2004-043J','SOZ','SOZ',67887,'Reentered','2008-04-30 00:00:00','2004-12-30 00:00:00',633.7,298,35824,47.44,'GEO Transfer' from dual union all
select 28488,'028488','2004-046G','Shiyan 2 large debris','',67890,'In Earth orbit','','2004-11-25 00:00:00',98.34,662,697,98.19,'Sun-Synch' from dual union all
select 28489,'028489','1989-052G','deb SOZ','',62329,'Reentered','2005-05-25 00:00:00','2004-12-17 00:00:00',115.86,380,2609,48,'Intermediate' from dual union all
select 2849,'002849','1967-061B','S1M','S1M',15316,'Reentered','1967-10-11 00:00:00','1967-07-19 00:00:00',92.54,278,520,48.43,'Intermediate' from dual union all
select 28490,'028490','1999-057NG','CZ-4 debris','',66738,'Reentered','2008-04-11 00:00:00','2004-12-17 00:00:00',99.94,701,810,98.24,'Sun-Synch' from dual union all
select 28491,'028491','2004-042C','FG-36 AKM','Shanxi AKM',67886,'In Earth orbit','','2004-12-18 00:00:00',1420.94,35397,35582,.71,'Drift GEO' from dual union all
select 28492,'028492','2004-049A','Helios IIA','Helios 2A',67893,'In Earth orbit','','2004-12-21 00:00:00',98.24,666,683,98.08,'Sun-Synch' from dual union all
select 28493,'028493','2004-049B','Nanosat 1','Nanosat',67893,'In Earth orbit','','2004-12-21 00:00:00',97.96,657,666,98.08,'Sun-Synch' from dual union all
select 28494,'028494','2004-049C','Essaim 1','Essaim 1/W23',67893,'In Earth orbit','','2004-12-20 00:00:00',97.93,655,664,98.07,'Sun-Synch' from dual union all
select 28495,'028495','2004-049D','Essaim 2','Essaim 2/W11',67893,'In Earth orbit','','2004-12-20 00:00:00',97.92,654,665,98.08,'Sun-Synch' from dual union all
select 28496,'028496','2004-049E','Essaim 3','Essaim 3/E24',67893,'In Earth orbit','','2004-12-21 00:00:00',97.91,653,665,98.07,'Sun-Synch' from dual union all
select 28497,'028497','2004-049F','Essaim 4','Essaim 4/E12',67893,'In Earth orbit','','2004-12-21 00:00:00',97.9,652,665,98.08,'Sun-Synch' from dual union all
select 28498,'028498','2004-049G','Parasol','Parasol',67893,'In Earth orbit','','2004-12-21 00:00:00',97.9,651,666,98.08,'Sun-Synch' from dual union all
select 28499,'028499','2004-049H','EPS 520','EPS S/N 517',67893,'In Earth orbit','','2004-12-21 00:00:00',97.5,600,679,98.06,'Sun-Synch' from dual union all
select 285,'000285','62 Omicron 1','Ariel 1','UK 1',8197,'Reentered','1976-05-24 00:00:00','1962-06-14 00:00:00',100.86,397,1202,53.86,'Intermediate' from dual union all
select 2850,'002850','1967-062A','OPS 3559','CORONA J-37',15317,'Reentered Attached','1967-07-20 00:00:00','1967-06-23 00:00:00',90,181,367,80.01,'Intermediate' from dual union all
select 28500,'028500','2004-050A','Demosat','HLV-OLDSP',67894,'In Earth orbit','','2005-02-23 00:00:00',1044.64,19000,36452,13.37,'Medium' from dual union all
select 28501,'028501','2004-038C','Kosmos-2410 solar panel','Kobal''t-M part',67882,'Reentered','2005-01-08 00:00:00','2005-01-08 00:00:00',87.83,147,188,67.12,'Intermediate' from dual union all
select 28502,'028502','2004-038D','Kosmos-2410 solar panel','Kobal''t-M part',67882,'Reentered','2005-01-08 00:00:00','2005-01-08 00:00:00',87.94,150,194,67.11,'Intermediate' from dual union all
select 28503,'028503','2004-051A','Progress M-51','Progress 7K-TGM No. 351',67895,'Deorbited','2005-03-09 00:00:00','2004-12-24 00:00:00',90.04,264,289,51.64,'Intermediate' from dual union all
select 28504,'028504','2004-051B','11S510','11S510',67895,'Reentered','2004-12-26 00:00:00','2004-12-24 00:00:00',88.54,183,222,51.65,'Intermediate' from dual union all
select 28505,'028505','2004-052A','Sich-1M','Okean-O1 No. 9',67896,'Reentered','2006-04-15 00:00:00','2004-12-24 00:00:00',93.79,280,640,82.57,'Intermediate' from dual union all
select 28506,'028506','2004-052B','S5M','S5M',67896,'Reentered','2009-07-14 00:00:00','2004-12-24 00:00:00',93.81,282,640,82.56,'Intermediate' from dual union all
select 28507,'028507','2004-052C','MK-1TS Mikron','KS5-MF2',67896,'Reentered','2005-09-30 00:00:00','2005-01-01 00:00:00',93.71,281,631,82.56,'Intermediate' from dual union all
select 28508,'028508','2004-053A','Kosmos-2411','Uragan No. 796',67897,'In Earth orbit','','2005-02-24 00:00:00',675.71,19108,19151,64.83,'Medium' from dual union all
select 28509,'028509','2004-053B','Kosmos-2413','Uragan-M No. 712',67897,'In Earth orbit','','2005-02-25 00:00:00',675.69,19126,19132,64.83,'Medium' from dual union all
select 2851,'002851','1967-062B','SAVANT I','P-11 No. 4409',15317,'Reentered','1968-10-22 00:00:00','1967-07-22 00:00:00',94.74,498,514,80.21,'Intermediate' from dual union all
select 28510,'028510','2004-053C','Kosmos-2412','Uragan No. 797',67897,'In Earth orbit','','2005-02-25 00:00:00',675.71,19121,19138,64.82,'Medium' from dual union all
select 28511,'028511','2004-053D','8S812','8S812',67897,'Reentered','2004-12-26 00:00:00','2004-12-26 00:00:00',87.46,137,161,64.85,'Intermediate' from dual union all
select 28512,'028512','2004-053E','Blok DM-2 No. 104L','11S861 No. 104L',67897,'In Earth orbit','','2005-02-24 00:00:00',675.04,19079,19146,64.82,'Medium' from dual union all
select 28513,'028513','2004-053F','Perekhodnik','Sredniy perekhodnik',67897,'Reentered','2004-12-26 00:00:00','2004-12-27 00:00:00',89.94,157,386,64.99,'Intermediate' from dual union all
select 28514,'028514','2004-053G','SOZ','SOZ',67897,'Reentered','2008-06-26 00:00:00','2005-02-25 00:00:00',340.34,397,19142,64.79,'Highly Ellip' from dual union all
select 28515,'028515','2004-053H','SOZ','SOZ',67897,'Reentered','2008-07-08 00:00:00','2005-02-24 00:00:00',340.36,398,19142,64.84,'Highly Ellip' from dual union all
select 28516,'028516','2003-023C','deb Kosmos-2398','',67647,'Reentered','2005-01-23 00:00:00','2005-01-10 00:00:00',104.74,903,1062,82.82,'Intermediate' from dual union all
select 28517,'028517','2005-001A','DIF','Deep Impact Flyby',68010,'Deep Space','2005-01-13 00:00:00','2005-01-12 00:00:00',1167.94,179,86409,29.73,'Earth Escape' from dual union all
select 28518,'028518','2005-001B','Star 48B','Star 48B',68010,'Deep Space','2005-01-13 00:00:00','2005-01-12 00:00:00',1167.94,179,86409,29.73,'Earth Escape' from dual union all
select 28519,'028519','2005-001C','Delta 311','Delta SSPS AJ-10-118K',68010,'Reentered','2005-06-20 00:00:00','2005-01-14 00:00:00',130.6,153,4145,23.98,'Medium' from dual union all
select 2852,'002852','1967-063A','Kosmos-167','4V-1 No. 311',15318,'Reentered','1967-06-25 00:00:00','1967-06-18 00:00:00',88.98,187,262,51.81,'Intermediate' from dual union all
select 28520,'028520','1987-022F','GOES 7 AKM','Star 27 TE-M-616-5',60112,'In Earth orbit','','2005-01-13 00:00:00',1457.88,31505,40919,11.96,'Near-synch' from dual union all
select 28521,'028521','2005-002A','Kosmos-2414','Parus',68011,'In Earth orbit','','2005-01-27 00:00:00',103.78,909,965,82.95,'Intermediate' from dual union all
select 28522,'028522','2005-002B','S3M','S3M',68011,'In Earth orbit','','2005-01-27 00:00:00',103.66,907,957,82.95,'Intermediate' from dual union all
select 28523,'028523','2005-002C','MGU-250','Universitetskiy/Tatiana',68011,'In Earth orbit','','2005-01-27 00:00:00',103.81,912,966,82.95,'Intermediate' from dual union all
select 28524,'028524','1981-038F','deb USAF satellite','',52561,'In Earth orbit','','2005-02-01 00:00:00',690.49,1000,38000,63.3,'Molniya' from dual union all
select 28525,'028525','1999-017G','DSP F19 debris','',66698,'In Earth orbit','','2006-03-31 00:00:00',618.36,600,34729,29.29,'GEO Transfer' from dual union all
select 28526,'028526','2005-003A','AMC 12','Americom 12',68012,'In Earth orbit','','2005-04-04 00:00:00',1436.27,35783,35796,.07,'Stationary' from dual union all
select 28527,'028527','2005-003B','Briz-M No. 88511','Briz-M',68012,'In Earth orbit','','2005-04-04 00:00:00',745.42,6402,35308,18.54,'GEO Transfer' from dual union all
select 28528,'028528','2005-003C','Briz-M DTB','Briz-M torus tank',68012,'In Earth orbit','','2005-04-04 00:00:00',284.23,313,15570,49.53,'Highly Ellip' from dual union all
select 28529,'028529','1984-012N','deb NOSS','',57066,'In Earth orbit','','2005-03-01 00:00:00',107.15,830,1359,63.44,'Intermediate' from dual union all
select 2853,'002853','1967-063B','BOZ','BOZ',15318,'Reentered','1967-06-21 00:00:00','1967-06-18 00:00:00',89.08,189,269,51.8,'Intermediate' from dual union all
select 28530,'028530','1992-086G','deb USA 89 PKM','',64798,'In Earth orbit','','2005-03-01 00:00:00',169.69,350,7200,57,'Medium' from dual union all
select 28531,'028531','1992-086H','deb USA 89 PKM','',64798,'In Earth orbit','','2005-03-01 00:00:00',169.69,350,7200,57,'Medium' from dual union all
select 28532,'028532','1992-086J','deb USA 89 PKM','',64798,'In Earth orbit','','2005-03-01 00:00:00',169.69,350,7200,57,'Medium' from dual union all
select 28533,'028533','1992-086K','deb USA 89 PKM','',64798,'In Earth orbit','','2005-03-01 00:00:00',169.69,350,7200,57,'Medium' from dual union all
select 28534,'028534','1992-086L','deb USA 89 PKM','',64798,'In Earth orbit','','2005-03-01 00:00:00',169.69,350,7200,57,'Medium' from dual union all
select 28535,'028535','1992-086M','deb USA 89 PKM','',64798,'In Earth orbit','','2005-03-01 00:00:00',169.69,350,7200,57,'Medium' from dual union all
select 28536,'028536','1992-086N','deb USA 89 PKM','',64798,'In Earth orbit','','2005-03-01 00:00:00',169.69,350,7200,57,'Medium' from dual union all
select 28537,'028537','2005-004A','USA 181','INTRUDER 5',68013,'In Earth orbit','','2005-02-07 00:00:00',107.47,1011,1209,63.42,'Intermediate' from dual union all
select 28538,'028538','2005-004B','Centaur AC-206','Centaur IIIB SEC AC-206',68013,'In Earth orbit','','2005-02-07 00:00:00',107.24,994,1204,63.62,'Intermediate' from dual;
insert into SATELLITE
select 28539,'028539','1980-019J','deb OPS 7245','',50229,'In Earth orbit','','2005-03-01 00:00:00',107.04,730,1450,63.48,'Intermediate' from dual union all
select 2854,'002854','1967-063D','11S510','11S510',15318,'Reentered','1967-06-18 00:00:00','1967-06-17 00:00:00',88.12,149,214,51.87,'Intermediate' from dual union all
select 28540,'028540','1980-019K','deb OPS 7245','',50229,'In Earth orbit','','2005-03-01 00:00:00',107.04,730,1450,63.48,'Intermediate' from dual union all
select 28541,'028541','2005-004C','USA-181 P/L 2','INTRUDER 6',68013,'In Earth orbit','','2005-02-07 00:00:00',107.47,1011,1209,63.42,'Intermediate' from dual union all
select 28542,'028542','2005-005A','XTAR-EUR','XTAR-EUR',68014,'In Earth orbit','','2005-04-15 00:00:00',1436.06,35770,35802,.05,'Stationary' from dual union all
select 28543,'028543','2005-005B','Sylda 5 SY-12C','Sylda 5C',68014,'Reentered','2012-12-03 00:00:00','2005-04-14 00:00:00',625.16,268,35414,6.75,'GEO Transfer' from dual union all
select 28544,'028544','2005-005C','Sloshsat-FLEVO','Sloshsat-FLEVO',68014,'In Earth orbit','','2005-04-14 00:00:00',631.2,267,35726,6.76,'GEO Transfer' from dual union all
select 28545,'028545','2005-005D','ESC-A 521','Ariane 5 ESC-A FM-03',68014,'In Earth orbit','','2005-04-16 00:00:00',638.08,255,36092,6.76,'GEO Transfer' from dual union all
select 28546,'028546','2004-050B','Delta 310','DCSS-5 F01',67894,'In Earth orbit','','2005-03-02 00:00:00',1019.02,17880,36452,12.95,'Medium' from dual union all
select 28547,'028547','2005-007C','TNS-0 Nanosputnik','TEKh-42',68016,'Reentered','2005-08-30 00:00:00','2005-03-28 00:00:00',91.62,349,359,51.64,'Intermediate' from dual union all
select 28548,'028548','2000-017D','Star 37 despin weight','',66919,'In Earth orbit','','2005-02-13 00:00:00',853.98,1405,45472,89.87,'Highly Ellip' from dual union all
select 28549,'028549','2000-017E','Star 37 despin weight','',66919,'In Earth orbit','','2005-02-12 00:00:00',851.51,1362,45400,89.73,'Highly Ellip' from dual union all
select 2855,'002855','1967-063E','deb Blok-I','',15318,'Reentered','1967-06-19 00:00:00','1967-06-17 00:00:00',88.03,177,177,51.87,'Intermediate' from dual union all
select 28550,'028550','1965-108BR','deb Transtage 8','',11858,'In Earth orbit','','2004-11-10 00:00:00',524.21,708,29608,27.07,'Highly Ellip' from dual union all
select 28551,'028551','1965-108BS','deb Transtage 8','',11858,'In Earth orbit','','2005-02-14 00:00:00',408.65,541,23186,27.12,'Highly Ellip' from dual union all
select 28552,'028552','1965-108BT','deb Transtage 8','',11858,'In Earth orbit','','2005-01-16 00:00:00',513.77,509,29233,26.37,'Highly Ellip' from dual union all
select 28553,'028553','1965-108BU','deb Transtage 8','',11858,'In Earth orbit','','2005-01-12 00:00:00',530.98,594,30092,26.64,'Highly Ellip' from dual union all
select 28554,'028554','1979-095C','deb Meteor-2','',47579,'In Earth orbit','','2005-02-11 00:00:00',101.71,826,854,81.2,'Intermediate' from dual union all
select 28555,'028555','1979-095D','deb Meteor-2','',47579,'In Earth orbit','','2005-01-25 00:00:00',101.7,830,848,81.2,'Intermediate' from dual union all
select 28556,'028556','1979-104V','deb Ariane L01','',47588,'In Earth orbit','','2005-02-14 00:00:00',736.7,881,40403,16.53,'GEO Transfer' from dual union all
select 28557,'028557','1979-104W','deb Ariane L01','',47588,'In Earth orbit','','2004-12-02 00:00:00',731.32,596,40425,19.24,'GEO Transfer' from dual union all
select 28558,'028558','1981-071S','deb Kosmos-1285','',52594,'In Earth orbit','','2005-02-14 00:00:00',728.51,1905,38978,65.24,'GEO Transfer' from dual union all
select 28559,'028559','1981-071T','deb Kosmos-1285','',52594,'In Earth orbit','','2005-01-28 00:00:00',726.96,2048,38759,65.7,'GEO Transfer' from dual union all
select 2856,'002856','1967-063C','Blok-VL No. Ya716-71','Blok-L',15318,'Reentered','1967-06-26 00:00:00','1967-06-20 00:00:00',88.98,194,255,51.8,'Intermediate' from dual union all
select 28560,'028560','1981-071U','deb Kosmos-1285','',52594,'In Earth orbit','','2005-02-01 00:00:00',727.45,2063,38768,65.7,'GEO Transfer' from dual union all
select 28561,'028561','1981-071V','deb Kosmos-1285','',52594,'In Earth orbit','','2005-01-19 00:00:00',727.41,1919,38910,64.81,'GEO Transfer' from dual union all
select 28562,'028562','1988-040F','deb Ariane V23','',61289,'In Earth orbit','','2005-02-14 00:00:00',628.96,485,35393,7.13,'GEO Transfer' from dual union all
select 28563,'028563','1988-098L','deb Ariane V26','',61347,'In Earth orbit','','2004-11-11 00:00:00',696.16,856,38427,3.56,'GEO Transfer' from dual union all
select 28564,'028564','1989-006AF','deb Ariane V28','',62283,'In Earth orbit','','2005-01-03 00:00:00',641.85,871,35669,8.54,'GEO Transfer' from dual union all
select 28565,'028565','1989-006AG','deb Ariane V28','',62283,'In Earth orbit','','2005-01-20 00:00:00',624.82,518,35146,7.75,'GEO Transfer' from dual union all
select 28566,'028566','1989-006AH','deb Ariane V28','',62283,'In Earth orbit','','2004-12-11 00:00:00',636.87,699,35586,6.66,'GEO Transfer' from dual union all
select 28567,'028567','1989-006AJ','deb Ariane V28','',62283,'In Earth orbit','','2004-12-28 00:00:00',627.82,496,35323,7.16,'GEO Transfer' from dual union all
select 28568,'028568','1989-006AK','deb Ariane V28','',62283,'In Earth orbit','','2005-01-18 00:00:00',643.51,922,35703,7.93,'GEO Transfer' from dual union all
select 28569,'028569','1989-006AL','deb Ariane V28','',62283,'In Earth orbit','','2005-02-11 00:00:00',632.57,578,35486,7.94,'GEO Transfer' from dual union all
select 2857,'002857','1967-063F','deb Blok-I','',15318,'Reentered','1967-06-28 00:00:00','1967-06-17 00:00:00',88.87,206,232,51.8,'Intermediate' from dual union all
select 28570,'028570','1989-006AM','deb Ariane V28','',62283,'In Earth orbit','','2004-12-07 00:00:00',633.78,671,35455,7.01,'GEO Transfer' from dual union all
select 28571,'028571','1989-101U','deb SOZ','',62378,'Reentered','2011-02-27 00:00:00','2005-02-12 00:00:00',488.61,433,27910,46.74,'Highly Ellip' from dual union all
select 28572,'028572','1989-101V','deb SOZ','',62378,'In Earth orbit','','2005-01-29 00:00:00',488.52,476,27862,46.54,'Highly Ellip' from dual union all
select 28573,'028573','1991-003N','deb Ariane V41','',64172,'In Earth orbit','','2005-02-14 00:00:00',545.03,678,30771,7.01,'Highly Ellip' from dual union all
select 28574,'028574','1991-003P','deb Ariane V41','',64172,'In Earth orbit','','2005-02-10 00:00:00',505.81,400,28902,6.77,'Highly Ellip' from dual union all
select 28575,'028575','1991-075M','deb Ariane V47','',64244,'In Earth orbit','','2005-02-15 00:00:00',509.71,846,28672,6.73,'Highly Ellip' from dual union all
select 28576,'028576','1991-075N','deb Ariane V47','',64244,'In Earth orbit','','2005-02-14 00:00:00',507.92,804,28615,6.77,'Highly Ellip' from dual union all
select 28577,'028577','1992-021R','deb Ariane V50','',64733,'In Earth orbit','','2005-02-14 00:00:00',644.25,1010,35653,3.16,'GEO Transfer' from dual union all
select 28578,'028578','1992-047M','deb SOZ','',64759,'Reentered','2016-10-24 00:00:00','2005-02-08 00:00:00',338.21,769,18635,65.05,'Highly Ellip' from dual union all
select 28579,'028579','1992-047N','deb SOZ','',64759,'Reentered','2009-08-28 00:00:00','2005-02-14 00:00:00',289.32,410,15814,65.34,'Highly Ellip' from dual union all
select 2858,'002858','1967-064A','OPS 4282','GAMBIT-3 Mission 4306',15319,'Reentered Attached','1967-06-30 00:00:00','1967-06-22 00:00:00',88.97,128,320,111.4,'Retrograde' from dual union all
select 28580,'028580','1992-047P','deb SOZ','',64759,'Reentered','2012-11-07 00:00:00','2005-02-13 00:00:00',243.05,529,12516,64.97,'Medium' from dual union all
select 28581,'028581','1992-047Q','deb SOZ','',64759,'Reentered','2011-09-21 00:00:00','2005-02-15 00:00:00',261.27,501,13818,64.96,'Medium' from dual union all
select 28582,'028582','1992-047R','deb SOZ','',64759,'In Earth orbit','','2005-02-11 00:00:00',246.05,481,12776,64.93,'Medium' from dual union all
select 28583,'028583','1992-047S','deb SOZ','',64759,'In Earth orbit','','2005-02-11 00:00:00',328.84,607,18200,65.38,'Highly Ellip' from dual union all
select 28584,'028584','1992-047T','deb SOZ','',64759,'In Earth orbit','','2005-02-15 00:00:00',332.94,568,18501,65.03,'Highly Ellip' from dual union all
select 28585,'028585','1992-047U','deb SOZ','',64759,'Reentered','2013-08-14 00:00:00','2005-01-30 00:00:00',319.07,510,17669,64.45,'Highly Ellip' from dual union all
select 28586,'028586','1992-047V','deb SOZ','',64759,'Reentered','2005-09-02 00:00:00','2004-12-18 00:00:00',333.32,670,18423,64.18,'Highly Ellip' from dual union all
select 28587,'028587','1992-047W','deb SOZ','',64759,'Reentered','2005-04-15 00:00:00','2005-02-15 00:00:00',133.22,344,4181,64.97,'Medium' from dual union all
select 28588,'028588','1992-047X','deb SOZ','',64759,'Reentered','2011-11-18 00:00:00','2005-01-26 00:00:00',318.82,588,17575,65.95,'Highly Ellip' from dual union all
select 28589,'028589','1992-047Y','deb SOZ','',64759,'In Earth orbit','','2005-02-15 00:00:00',334.17,592,18555,65.49,'Highly Ellip' from dual union all
select 2859,'002859','1967-042C','UK-3 despin weight','',15297,'Reentered','1968-05-05 00:00:00','1967-06-08 00:00:00',95.55,456,635,80.31,'Intermediate' from dual union all
select 28590,'028590','1992-047Z','deb SOZ','',64759,'In Earth orbit','','2005-01-25 00:00:00',324.66,543,17995,62.86,'Highly Ellip' from dual union all
select 28591,'028591','1974-015C','Burner 2A collision debris','',33618,'In Earth orbit','','2005-02-16 00:00:00',101.43,792,860,99.1,'Sun-Synch' from dual union all
select 28592,'028592','1974-015D','Burner 2A collision debris','',33618,'In Earth orbit','','2005-02-16 00:00:00',100.23,683,856,99.08,'Sun-Synch' from dual union all
select 28593,'028593','1974-015E','Burner 2A collision debris','',33618,'In Earth orbit','','2005-02-15 00:00:00',101.03,754,861,99.1,'Sun-Synch' from dual union all
select 28594,'028594','1979-095E','deb Meteor-2','',47579,'In Earth orbit','','2005-02-21 00:00:00',102.23,854,875,81.2,'Intermediate' from dual union all
select 28595,'028595','1979-095F','deb Meteor-2','',47579,'In Earth orbit','','2005-02-13 00:00:00',102.1,847,870,81.2,'Intermediate' from dual union all
select 28596,'028596','1979-095G','deb Meteor-2','',47579,'In Earth orbit','','2005-02-19 00:00:00',101.77,836,849,81.2,'Intermediate' from dual union all
select 28597,'028597','1981-043G','deb Meteor-2','',52566,'In Earth orbit','','2005-02-15 00:00:00',101.66,803,872,81.26,'Intermediate' from dual union all
select 28598,'028598','1982-116E','deb Meteor-2','',54425,'In Earth orbit','','2005-02-08 00:00:00',101.65,808,866,81.26,'Intermediate' from dual union all
select 28599,'028599','1981-043H','deb Meteor-2','',52566,'In Earth orbit','','2005-02-20 00:00:00',101.19,792,838,81.26,'Intermediate' from dual union all
select 286,'000286','62 Pi','FTV 2401','SAMOS E-6 No. 1',8199,'Reentered Attached','1962-04-28 00:00:00','1962-04-26 00:00:00',88.77,209,219,92,'Polar' from dual union all
select 2860,'002860','1967-042D','UK-3 despin weight','',15297,'Reentered','1968-03-09 00:00:00','1967-06-18 00:00:00',95.07,456,588,79.97,'Intermediate' from dual union all
select 28600,'028600','1981-043J','deb Meteor-2','',52566,'In Earth orbit','','2005-02-20 00:00:00',101.07,787,832,81.25,'Intermediate' from dual union all
select 28601,'028601','1981-043K','deb Meteor-2','',52566,'In Earth orbit','','2005-02-03 00:00:00',101.76,802,882,81.27,'Intermediate' from dual union all
select 28602,'028602','1981-043L','deb Meteor-2','',52566,'In Earth orbit','','2005-02-21 00:00:00',101.77,804,881,81.27,'Intermediate' from dual union all
select 28603,'028603','1985-013C','deb Meteor-2','',58236,'In Earth orbit','','2005-02-11 00:00:00',103.76,918,954,82.51,'Intermediate' from dual union all
select 28604,'028604','1983-022K','NOAA 8 battery debris','',55779,'Reentered','2005-10-17 00:00:00','2005-02-20 00:00:00',99.02,696,728,98.77,'Sun-Synch' from dual union all
select 28605,'028605','2003-048F','deb DMSP 5D-3 F-16','',67672,'Reentered','2015-02-05 00:00:00','2005-02-17 00:00:00',101.53,828,835,98.96,'Sun-Synch' from dual union all
select 28606,'028606','2003-048G','deb DMSP 5D-3 F-16','',67672,'Reentered','2014-06-24 00:00:00','2005-02-10 00:00:00',101.51,822,839,98.9,'Sun-Synch' from dual union all
select 28607,'028607','2003-048H','deb DMSP 5D-3 F-16','',67672,'Reentered','2013-12-14 00:00:00','2005-02-22 00:00:00',101.24,814,821,98.96,'Sun-Synch' from dual union all
select 28608,'028608','1968-055M','RAE-1 despin weight','',16782,'In Earth orbit','','2005-02-22 00:00:00',224.2,5843,5851,120.86,'Medium' from dual union all
select 28609,'028609','1966-040Y','Nimbus 2 debris','',13447,'In Earth orbit','','2005-02-06 00:00:00',107.8,1083,1167,100.41,'Sun-Synch' from dual union all
select 2861,'002861','1967-065A','EGRS IX','Secor Type II S/N 10',15320,'In Earth orbit','','1967-08-12 00:00:00',172.11,3797,3946,89.82,'Medium' from dual union all
select 28610,'028610','1966-040Z','Nimbus 2 debris','',13447,'In Earth orbit','','2005-02-03 00:00:00',107.43,1078,1138,100.41,'Sun-Synch' from dual union all
select 28611,'028611','1966-040AA','Nimbus 2 debris','',13447,'In Earth orbit','','2005-02-20 00:00:00',105.93,1001,1075,100.1,'Sun-Synch' from dual union all
select 28612,'028612','1966-040AB','Nimbus 2 debris','',13447,'In Earth orbit','','2005-02-21 00:00:00',106.66,1037,1107,100.42,'Sun-Synch' from dual union all
select 28613,'028613','1966-040AC','Nimbus 2 debris','',13447,'In Earth orbit','','2005-02-21 00:00:00',106.96,1050,1121,100.41,'Sun-Synch' from dual union all
select 28614,'028614','1966-040AD','Nimbus 2 debris','',13447,'In Earth orbit','','2005-02-02 00:00:00',107.36,1054,1154,100.36,'Sun-Synch' from dual union all
select 28615,'028615','1966-040AE','Nimbus 2 debris','',13447,'In Earth orbit','','2005-02-21 00:00:00',106.72,1038,1111,100.25,'Sun-Synch' from dual union all
select 28616,'028616','1966-040AF','Nimbus 2 debris','',13447,'In Earth orbit','','2005-02-01 00:00:00',107.14,1059,1129,100.31,'Sun-Synch' from dual union all
select 28617,'028617','1966-040AG','Nimbus 2 debris','',13447,'In Earth orbit','','2005-02-20 00:00:00',106.73,1023,1128,100.47,'Sun-Synch' from dual union all
select 28618,'028618','1966-040AH','Nimbus 2 debris','',13447,'In Earth orbit','','2005-02-22 00:00:00',106.64,1041,1101,100.25,'Sun-Synch' from dual union all
select 28619,'028619','1966-040AJ','Nimbus 2 debris','',13447,'In Earth orbit','','2005-02-20 00:00:00',106.73,1045,1105,100.29,'Sun-Synch' from dual union all
select 2862,'002862','1967-066A','OPS 9331','IDCSP 16',15321,'In Earth orbit','','1967-09-02 00:00:00',1309.68,33015,33534,6.88,'Medium' from dual union all
select 28620,'028620','1966-040AK','Nimbus 2 debris','',13447,'In Earth orbit','','2005-02-20 00:00:00',107.96,1089,1176,100.4,'Sun-Synch' from dual union all
select 28621,'028621','1966-040AM','Nimbus 2 debris','',13447,'In Earth orbit','','2005-02-22 00:00:00',104.93,967,1016,100.28,'Sun-Synch' from dual union all
select 28622,'028622','2005-006A','Himawari-6','MTSAT-1R',68015,'In Earth orbit','','2005-05-10 00:00:00',1433.51,35576,35896,.27,'Drift GEO' from dual union all
select 28623,'028623','2005-006B','H-2A F7 Stage 2','H-2A Stage 2',68015,'Reentered','2008-07-18 00:00:00','2005-04-28 00:00:00',583.99,175,33356,28.9,'Highly Ellip' from dual union all
select 28624,'028624','2005-007A','Progress M-52','Progress 7K-TGM No. 352/',68016,'Deorbited','2005-06-15 00:00:00','2005-03-02 00:00:00',91.68,354,359,51.65,'Intermediate' from dual union all
select 28625,'028625','2005-007B','11S510','11S510',68016,'Reentered','2005-03-02 00:00:00','2005-03-02 00:00:00',87.7,155,167,51.64,'Intermediate' from dual union all
select 28626,'028626','2005-008A','XM-3','XM Radio-3',68017,'In Earth orbit','','2005-05-01 00:00:00',1436.09,35785,35788,.06,'Stationary' from dual union all
select 28627,'028627','2005-008B','Blok DM-SL No. 17L','Blok DM-SL',68017,'In Earth orbit','','2005-04-30 00:00:00',673.27,2447,35689,.05,'GEO Transfer' from dual union all
select 28628,'028628','2005-009A','INMARSAT 4 F1','INMARSAT 4 F1',68018,'In Earth orbit','','2005-05-11 00:00:00',1427.41,35549,35684,2.92,'Drift GEO' from dual;
insert into SATELLITE
select 28629,'028629','2005-010A','Ekspress AM-2','Ekspress AM-2',68019,'In Earth orbit','','2005-05-30 00:00:00',1436.09,35782,35791,.03,'Stationary' from dual union all
select 2863,'002863','1967-066B','OPS 9332','IDCSP 17',15321,'In Earth orbit','','1967-09-02 00:00:00',1310.35,33034,33542,6.89,'Medium' from dual union all
select 28630,'028630','2005-010B','8S812','8S812',68019,'Reentered','2005-04-01 00:00:00','2005-03-30 00:00:00',88.1,174,186,51.63,'Intermediate' from dual union all
select 28631,'028631','2005-010C','Perekhodnik','Sredniy perekhodnik',68019,'Reentered','2005-03-31 00:00:00','2005-03-30 00:00:00',88.06,175,182,51.61,'Intermediate' from dual union all
select 28632,'028632','2005-010D','SOZ','SOZ',68019,'In Earth orbit','','2005-05-29 00:00:00',634.5,345,35818,47.55,'GEO Transfer' from dual union all
select 28633,'028633','2005-010E','SOZ','SOZ',68019,'Reentered','2006-09-24 00:00:00','2005-05-29 00:00:00',633.12,290,35802,47.5,'GEO Transfer' from dual union all
select 28634,'028634','2005-010F','Blok DM-2M No. 16L','11S861-01 No. 16L',68019,'In Earth orbit','','2005-05-30 00:00:00',1435.63,35691,35864,.18,'Stationary' from dual union all
select 28635,'028635','1998-067AF','EVA debris (connector cap)','Plastic connector cap',66503,'Reentered','2005-04-10 00:00:00','2005-03-28 00:00:00',91.69,354,361,51.65,'Intermediate' from dual union all
select 28636,'028636','2005-011A','XSS-11','XSS-11',68020,'Reentered','2013-11-12 00:00:00','2005-04-18 00:00:00',102.14,847,873,98.8,'Sun-Synch' from dual union all
select 28637,'028637','2005-011B','Minotaur Stage 4','Orion 38',68020,'In Earth orbit','','2005-04-18 00:00:00',102.05,847,865,98.8,'Sun-Synch' from dual union all
select 28638,'028638','2005-012A','Apstar 6','Apstar 6',68021,'In Earth orbit','','2005-06-11 00:00:00',1436.06,35784,35788,.02,'Stationary' from dual union all
select 28639,'028639','2005-012B','CZ-3B Stage 3','CZ-3B Stage 3',68021,'Reentered','2006-02-26 00:00:00','2005-06-12 00:00:00',503.83,166,29026,25.86,'Highly Ellip' from dual union all
select 2864,'002864','1967-066C','OPS 9333','IDCSP 18',15321,'In Earth orbit','','1967-09-02 00:00:00',1311.67,33092,33536,6.92,'Medium' from dual union all
select 28640,'028640','2005-013A','Soyuz TMA-6','Soyuz 11F732 No. 216',68022,'Landed','2005-10-11 00:00:00','2005-04-17 00:00:00',91.64,350,359,51.65,'Intermediate' from dual union all
select 28641,'028641','2005-013B','11S510','11S510',68022,'Reentered','2005-04-18 00:00:00','2005-04-16 00:00:00',88.32,181,202,51.64,'Intermediate' from dual union all
select 28642,'028642','2005-014A','DART','DART/HAPS',68023,'Reentered','2016-05-07 00:00:00','2005-04-17 00:00:00',99.71,740,750,97.72,'Sun-Synch' from dual union all
select 28643,'028643','2005-014B','Orion 38','Orion 38',68023,'Reentered','2012-01-31 00:00:00','2005-04-17 00:00:00',94.08,401,547,97.74,'Sun-Synch' from dual union all
select 28644,'028644','2005-015A','Spaceway 1','Spaceway 1',68024,'In Earth orbit','','2005-06-28 00:00:00',1435.42,31829,39718,.16,'Near-synch' from dual union all
select 28645,'028645','2005-015B','Blok DM-SL No. 19L','Blok DM-SL',68024,'In Earth orbit','','2005-06-26 00:00:00',593.3,213,33809,.29,'Highly Ellip' from dual union all
select 28646,'028646','2005-016A','USA 182','ONYX 5',68025,'In Earth orbit','','2005-05-04 00:00:00',99.08,712,717,57.01,'Intermediate' from dual union all
select 28647,'028647','2005-016B','Titan 4B-30 Stage 2','Titan 4 Stage 2',68025,'In Earth orbit','','2005-05-07 00:00:00',96.5,476,706,57,'Intermediate' from dual union all
select 28648,'028648','1992-086P','deb USA 89 PKM','',64798,'In Earth orbit','','2005-05-01 00:00:00',169.69,350,7200,57,'Medium' from dual union all
select 28649,'028649','2005-017A','Cartosat-1','Cartosat-1',68026,'In Earth orbit','','2005-05-12 00:00:00',97.12,606,635,97.91,'Sun-Synch' from dual union all
select 2865,'002865','1967-066D','OPS 9334','IDCSP 19/DATS',15321,'In Earth orbit','','1967-09-30 00:00:00',1313.7,33138,33573,6.84,'Medium' from dual union all
select 28650,'028650','2005-017B','VUSat-OSCAR-52','HAMSAT',68026,'In Earth orbit','','2005-05-12 00:00:00',97.23,608,644,97.91,'Sun-Synch' from dual union all
select 28651,'028651','2005-017C','PSLV-C6 PS4','PS4',68026,'In Earth orbit','','2005-05-12 00:00:00',97.97,607,717,97.82,'Sun-Synch' from dual union all
select 28652,'028652','2005-017D','deb PS4','',68026,'Reentered','2005-08-18 00:00:00','2005-05-28 00:00:00',97.17,577,670,97.82,'Sun-Synch' from dual union all
select 28653,'028653','2005-017E','deb PS4','',68026,'Reentered','2006-11-23 00:00:00','2005-05-13 00:00:00',97.52,605,675,97.86,'Sun-Synch' from dual union all
select 28654,'028654','2005-018A','NOAA 18','NOAA N',68027,'In Earth orbit','','2005-05-27 00:00:00',102.05,846,865,98.74,'Sun-Synch' from dual union all
select 28655,'028655','2005-018B','Delta 312','Delta SSPS AJ-10-118K',68027,'Reentered','2005-07-05 00:00:00','2005-05-27 00:00:00',94.04,165,780,103.57,'Retrograde' from dual union all
select 28656,'028656','2004-008C','Briz-M No. 88507','Briz-M 14S43 No. 88507',67852,'In Earth orbit','','2005-05-19 00:00:00',729.73,4103,36840,12.5,'GEO Transfer' from dual union all
select 28657,'028657','1988-077T','deb Transtage D-3','',61326,'In Earth orbit','','2005-05-31 00:00:00',702.31,426,39163,27.2,'GEO Transfer' from dual union all
select 28658,'028658','1988-077U','deb Transtage D-3','',61326,'In Earth orbit','','2005-05-31 00:00:00',702.31,426,39163,27.2,'GEO Transfer' from dual union all
select 28659,'028659','2005-019A','DirecTV-8','DirecTV-8',68028,'In Earth orbit','','2005-07-22 00:00:00',1436.06,35776,35796,.04,'Stationary' from dual union all
select 2866,'002866','1967-066E','LES 5','LES 5',15321,'In Earth orbit','','1967-09-03 00:00:00',1316.15,33171,33639,6.92,'Medium' from dual union all
select 28660,'028660','2005-019B','Briz-M No. 88512','Briz-M 14S43 No. 88512',68028,'In Earth orbit','','2005-07-22 00:00:00',932.79,14830,35662,8.06,'Medium' from dual union all
select 28661,'028661','2005-019C','Briz-M DTB','Briz-M torus tank',68028,'In Earth orbit','','2005-07-22 00:00:00',402.05,340,22993,49.34,'Highly Ellip' from dual union all
select 28662,'028662','1965-089G','deb Explorer 29','',11839,'In Earth orbit','','2004-11-18 00:00:00',118.51,1057,2171,59.4,'Intermediate' from dual union all
select 28663,'028663','1965-108BV','deb Transtage 8','',11858,'In Earth orbit','','2004-08-07 00:00:00',552.43,1151,30697,26.62,'Highly Ellip' from dual union all
select 28664,'028664','1966-077AD','deb FTV 1352','',13484,'In Earth orbit','','2005-05-21 00:00:00',167.45,3562,3809,89.86,'Medium' from dual union all
select 28665,'028665','1968-055N','deb Star 37D','',16782,'In Earth orbit','','2005-04-19 00:00:00',122.05,652,2893,120.64,'Medium' from dual union all
select 28666,'028666','1968-055P','deb Star 37D','',16782,'In Earth orbit','','2005-05-10 00:00:00',138.39,750,4219,120.56,'Medium' from dual union all
select 28667,'028667','1968-055Q','deb Star 37D','',16782,'In Earth orbit','','2005-05-11 00:00:00',140.38,737,4402,120.66,'Medium' from dual union all
select 28668,'028668','1968-055S','deb Star 37D','',16782,'In Earth orbit','','2005-03-17 00:00:00',131.88,708,3700,120.8,'Medium' from dual union all
select 28669,'028669','1968-055R','deb Star 37D','',16782,'In Earth orbit','','2005-05-08 00:00:00',122.06,600,2946,120.48,'Medium' from dual union all
select 2867,'002867','1967-066F','DODGE','DODGE',15321,'In Earth orbit','','1982-12-21 00:00:00',1319.06,33256,33671,4.39,'Medium' from dual union all
select 28670,'028670','1968-055T','deb Star 37D','',16782,'In Earth orbit','','2005-05-14 00:00:00',118.81,634,2621,120.55,'Retrograde' from dual union all
select 28671,'028671','1968-055U','deb Star 37D','',16782,'In Earth orbit','','2005-05-20 00:00:00',128.94,653,3500,120.56,'Medium' from dual union all
select 28672,'028672','1988-074H','NNS O-25 insulation','',61323,'In Earth orbit','','2005-06-06 00:00:00',106.79,1009,1147,89.85,'Polar' from dual union all
select 28673,'028673','1988-074J','NNS O-25 insulation','',61323,'In Earth orbit','','2005-06-01 00:00:00',105.85,978,1091,89.84,'Polar' from dual union all
select 28674,'028674','1988-074K','NNS O-25 insulation','',61323,'In Earth orbit','','2005-03-01 00:00:00',105.36,959,1064,89.83,'Polar' from dual union all
select 28675,'028675','1988-074L','NNS O-25 insulation','',61323,'In Earth orbit','','2005-02-24 00:00:00',106.94,1019,1150,89.84,'Polar' from dual union all
select 28676,'028676','1988-074M','NNS O-25 insulation','',61323,'In Earth orbit','','2005-05-20 00:00:00',107.06,1022,1160,89.86,'Polar' from dual union all
select 28677,'028677','1992-047AA','deb SOZ','',64759,'Reentered','2005-07-22 00:00:00','2005-05-21 00:00:00',126.75,337,3624,63.88,'Medium' from dual union all
select 28678,'028678','1992-047AB','deb SOZ','',64759,'Reentered','2015-09-18 00:00:00','2005-05-24 00:00:00',219.58,567,10789,63.77,'Medium' from dual union all
select 28679,'028679','1992-047AC','deb SOZ','',64759,'In Earth orbit','','2005-05-20 00:00:00',181.26,616,7847,63.37,'Medium' from dual union all
select 2868,'002868','1967-066G','Transtage 14','Transtage 14',15321,'In Earth orbit','','1977-01-21 00:00:00',1319.03,33275,33651,.48,'Medium' from dual union all
select 28680,'028680','1992-047AD','deb SOZ','',64759,'Reentered','2007-12-11 00:00:00','2005-05-12 00:00:00',253.75,995,12802,63.81,'Medium' from dual union all
select 28681,'028681','1992-047AE','deb SOZ','',64759,'Reentered','2005-06-24 00:00:00','2005-05-05 00:00:00',312.2,998,16735,67.95,'Highly Ellip' from dual union all
select 28682,'028682','1992-047AF','deb SOZ','',64759,'In Earth orbit','','2005-05-24 00:00:00',333.99,1408,17727,63.95,'Highly Ellip' from dual union all
select 28683,'028683','1992-047AG','deb SOZ','',64759,'Reentered','2007-03-08 00:00:00','2005-05-19 00:00:00',311.37,867,16812,65.28,'Highly Ellip' from dual union all
select 28684,'028684','1992-047AH','deb SOZ','',64759,'In Earth orbit','','2005-05-24 00:00:00',211.1,342,10390,64.41,'Medium' from dual union all
select 28685,'028685','1992-047AJ','deb SOZ','',64759,'Reentered','2016-05-25 00:00:00','2005-05-24 00:00:00',210.17,314,10349,64.3,'Medium' from dual union all
select 28686,'028686','2005-020A','Foton-M No. 2','Foton 34KS No. 14',68029,'Landed','2005-06-16 00:00:00','2005-06-02 00:00:00',89.99,258,290,62.97,'Intermediate' from dual union all
select 28687,'028687','2005-020B','11S510','11S510',68029,'Reentered','2005-06-24 00:00:00','2005-06-02 00:00:00',89.88,255,282,62.98,'Intermediate' from dual union all
select 28688,'028688','2005-020C','deb 11S510','',68029,'Reentered','2005-06-13 00:00:00','2005-06-02 00:00:00',90.56,255,349,62.96,'Intermediate' from dual union all
select 28689,'028689','2002-037J','deb SOZ','',67413,'Reentered','2005-06-03 00:00:00','2005-06-03 00:00:00',92.29,234,540,63.45,'Intermediate' from dual union all
select 2869,'002869','1967-067A','Kosmos-168','Zenit-2 No. 52',15322,'Landed','1967-07-12 00:00:00','1967-07-05 00:00:00',89.09,198,261,51.8,'Intermediate' from dual union all
select 28690,'028690','2002-037K','deb SOZ','',67413,'Reentered','2005-06-04 00:00:00','2005-06-03 00:00:00',94.02,264,679,63.46,'Intermediate' from dual union all
select 28691,'028691','2002-037L','deb SOZ','',67413,'Reentered','2005-06-04 00:00:00','2005-06-03 00:00:00',93.95,298,638,63.64,'Intermediate' from dual union all
select 28692,'028692','2002-037M','deb SOZ','',67413,'Reentered','2005-06-06 00:00:00','2005-06-02 00:00:00',95.05,266,775,63.8,'Intermediate' from dual union all
select 28693,'028693','2002-037N','deb SOZ','',67413,'Reentered','2005-06-17 00:00:00','2005-06-07 00:00:00',94.9,280,748,63.44,'Intermediate' from dual union all
select 28694,'028694','2005-020D','KB electropitaniya','KBE',68029,'Reentered','2005-07-30 00:00:00','2005-06-16 00:00:00',89.86,255,280,62.97,'Intermediate' from dual union all
select 28695,'028695','2005-020E','KBE part','KBE part',68029,'Reentered','2005-06-28 00:00:00','2005-06-16 00:00:00',90.76,264,360,62.86,'Intermediate' from dual union all
select 28696,'028696','2005-020F','PTDU cover','PTDU cover',68029,'Reentered','2005-07-02 00:00:00','2005-06-16 00:00:00',91.84,267,463,62.98,'Intermediate' from dual union all
select 28697,'028697','2005-020G','PTDU cover','PTDU cover',68029,'Reentered','2005-06-19 00:00:00','2005-06-16 00:00:00',92.32,260,517,62.95,'Intermediate' from dual union all
select 28698,'028698','2005-020H','KBE part','KBE part',68029,'Reentered','2005-06-17 00:00:00','2005-06-16 00:00:00',90.07,259,297,62.88,'Intermediate' from dual union all
select 28699,'028699','2005-020J','KBE part','KBE part',68029,'Reentered','2005-07-02 00:00:00','2005-06-16 00:00:00',92.09,264,490,63.05,'Intermediate' from dual union all
select 287,'000287','62 Xi 1','Kosmos-4','Zenit-2 No. 2',8206,'Landed','1962-04-29 00:00:00','1962-04-26 00:00:00',90.8,298,330,65,'Intermediate' from dual union all
select 2870,'002870','1967-067B','11A57I','11A57I',15322,'Reentered','1967-07-08 00:00:00','1967-07-05 00:00:00',88.77,184,244,51.81,'Intermediate' from dual union all
select 28700,'028700','2005-021A','Progress M-53','Progress 7K-TGM No. 353/',68030,'Deorbited','2005-09-07 00:00:00','2005-06-24 00:00:00',91.52,347,352,51.64,'Intermediate' from dual union all
select 28701,'028701','2005-021B','11S510','11S510',68030,'Reentered','2005-06-19 00:00:00','2005-06-18 00:00:00',88.34,179,205,51.63,'Intermediate' from dual union all
select 28702,'028702','2005-022A','Intelsat IA-8','IA-8',68031,'In Earth orbit','','2005-08-23 00:00:00',1436.01,35784,35785,.03,'Stationary' from dual union all
select 28703,'028703','2005-022B','Blok DM-SL No. 13L','Blok DM-SL',68031,'Reentered','2005-10-20 00:00:00','2005-07-25 00:00:00',613.14,123,34936,.01,'GEO Transfer' from dual union all
select 28704,'028704','2005-023H','Blok DM-2 No. 103L','11S861 No. 103L',68032,'In Earth orbit','','2005-08-24 00:00:00',1428.38,35583,35688,.15,'Drift GEO' from dual union all
select 28705,'028705','2002-037P','deb SOZ','',67413,'Reentered','2005-06-30 00:00:00','2005-06-30 00:00:00',91.52,259,440,63.65,'Intermediate' from dual union all
select 28706,'028706','1990-017C','deb S3M','collision',63259,'In Earth orbit','','2005-06-23 00:00:00',104.52,907,1037,83.09,'Intermediate' from dual union all
select 28707,'028707','2005-023A','Ekspress AM-3','Ekspress AM-3',68032,'In Earth orbit','','2005-08-24 00:00:00',1436.19,35763,35814,.06,'Stationary' from dual union all
select 28708,'028708','2005-023B','8S812','8S812',68032,'Reentered','2005-06-27 00:00:00','2005-06-26 00:00:00',87.94,165,179,51.63,'Intermediate' from dual union all
select 28709,'028709','2005-023C','Perekhodnik','Sredniy perekhodnik',68032,'Reentered','2005-06-25 00:00:00','2005-06-25 00:00:00',88.13,178,186,51.62,'Intermediate' from dual union all
select 2871,'002871','1967-067C','deb Kosmos-168','',15322,'Reentered','1967-07-08 00:00:00','1967-07-04 00:00:00',89.17,201,266,51.75,'Intermediate' from dual union all
select 28710,'028710','2005-023D','deb 8S812','deb 8S812',68032,'Reentered','2005-06-30 00:00:00','2005-06-27 00:00:00',90.69,181,435,51.62,'Intermediate' from dual union all
select 28711,'028711','2005-023E','deb 8S812','deb 8S812',68032,'Reentered','2005-06-30 00:00:00','2005-06-27 00:00:00',91.28,191,483,51.59,'Intermediate' from dual union all
select 28712,'028712','2005-023F','SOZ','SOZ',68032,'Reentered','2006-01-30 00:00:00','2005-08-23 00:00:00',631.37,268,35734,47.51,'GEO Transfer' from dual union all
select 28713,'028713','2005-023G','SOZ','SOZ',68032,'Reentered','2005-12-04 00:00:00','2005-08-24 00:00:00',629.53,211,35696,47.47,'GEO Transfer' from dual union all
select 28714,'028714','2005-023J','deb 8S812','deb 8S812',68032,'Reentered','2005-07-01 00:00:00','2005-06-28 00:00:00',90.58,178,428,51.62,'Intermediate' from dual union all
select 28715,'028715','2002-037Q','deb SOZ','',67413,'Reentered','2005-06-30 00:00:00','2005-06-29 00:00:00',92.45,242,548,63.67,'Intermediate' from dual union all
select 28716,'028716','2002-037R','deb SOZ','',67413,'Reentered','2005-06-30 00:00:00','2005-06-30 00:00:00',94.11,262,688,63.66,'Intermediate' from dual union all
select 28717,'028717','2002-037S','deb SOZ','',67413,'Reentered','2005-06-30 00:00:00','2005-06-30 00:00:00',95.57,265,828,63.65,'Intermediate' from dual union all
select 28718,'028718','2002-037T','deb SOZ','',67413,'Reentered','2005-06-30 00:00:00','2005-06-30 00:00:00',91.77,212,510,63.66,'Intermediate' from dual;
insert into SATELLITE
select 28719,'028719','2002-037U','deb SOZ','',67413,'Reentered','2005-06-30 00:00:00','2005-06-29 00:00:00',92.6,277,526,63.65,'Intermediate' from dual union all
select 2872,'002872','1967-053F','Calsphere III','NRL PL159',15308,'In Earth orbit','','1967-07-01 00:00:00',103.4,914,925,69.98,'Intermediate' from dual union all
select 28720,'028720','2002-037V','deb SOZ','',67413,'Reentered','2005-06-30 00:00:00','2005-06-30 00:00:00',91.88,280,454,63.61,'Intermediate' from dual union all
select 28721,'028721','2002-037W','deb SOZ','',67413,'Reentered','2005-07-02 00:00:00','2005-06-30 00:00:00',94.8,309,709,63.48,'Intermediate' from dual union all
select 28722,'028722','2002-037X','deb SOZ','',67413,'Reentered','2005-07-07 00:00:00','2005-06-30 00:00:00',94.79,303,713,63.57,'Intermediate' from dual union all
select 28723,'028723','2002-037Y','deb SOZ','',67413,'Reentered','2005-06-30 00:00:00','2005-06-30 00:00:00',92.44,277,512,63.66,'Intermediate' from dual union all
select 28724,'028724','2002-037Z','deb SOZ','',67413,'Reentered','2005-07-28 00:00:00','2005-06-30 00:00:00',95.23,253,807,63.74,'Intermediate' from dual union all
select 28725,'028725','2002-037AA','deb SOZ','',67413,'Reentered','2006-06-20 00:00:00','2005-07-01 00:00:00',95.76,263,848,63.69,'Intermediate' from dual union all
select 28726,'028726','2002-037AB','deb SOZ','',67413,'Reentered','2005-08-17 00:00:00','2005-06-30 00:00:00',96.32,270,895,63.7,'Intermediate' from dual union all
select 28727,'028727','2002-037AC','deb SOZ','',67413,'Reentered','2012-02-16 00:00:00','2005-06-30 00:00:00',96.93,419,805,63.72,'Intermediate' from dual union all
select 28728,'028728','2002-037AD','deb SOZ','',67413,'Reentered','2005-08-02 00:00:00','2005-06-30 00:00:00',94.99,238,798,63.73,'Intermediate' from dual union all
select 28729,'028729','2002-037AE','deb SOZ','',67413,'Reentered','2011-04-07 00:00:00','2005-07-01 00:00:00',97.3,297,962,63.23,'Intermediate' from dual union all
select 2873,'002873','1967-053G','NRL PL151','POPPY 5 SV1',15308,'In Earth orbit','','1967-07-14 00:00:00',103.41,913,927,69.98,'Intermediate' from dual union all
select 28730,'028730','2002-037AF','deb SOZ','',67413,'Reentered','2005-09-05 00:00:00','2005-07-01 00:00:00',97.72,269,1031,63.6,'Intermediate' from dual union all
select 28731,'028731','2002-037AG','deb SOZ','',67413,'Reentered','2006-01-07 00:00:00','2005-07-01 00:00:00',98.22,282,1066,63.34,'Intermediate' from dual union all
select 28732,'028732','2002-037AH','deb SOZ','',67413,'Reentered','2005-10-05 00:00:00','2005-07-01 00:00:00',101.23,302,1332,63.39,'Intermediate' from dual union all
select 28733,'028733','2002-037AJ','deb SOZ','',67413,'Reentered','2005-10-07 00:00:00','2005-07-01 00:00:00',94.79,332,685,63.62,'Intermediate' from dual union all
select 28734,'028734','2002-037AK','deb SOZ','',67413,'Reentered','2005-10-03 00:00:00','2005-07-01 00:00:00',94.45,238,746,63.77,'Intermediate' from dual union all
select 28735,'028735','2002-037AL','deb SOZ','',67413,'Reentered','2005-10-03 00:00:00','2005-07-01 00:00:00',100.74,308,1280,63.32,'Intermediate' from dual union all
select 28736,'028736','2002-037AM','deb SOZ','',67413,'Reentered','2005-09-28 00:00:00','2005-07-01 00:00:00',95.21,339,719,63.55,'Intermediate' from dual union all
select 28737,'028737','2005-024A','Shi Jian 7','Tansuo-2',68033,'In Earth orbit','','2005-07-07 00:00:00',95.81,547,568,97.58,'Sun-Synch' from dual union all
select 28738,'028738','2005-024B','CZ-2D Stage 2','CZ-2D Stage 2',68033,'In Earth orbit','','2005-07-07 00:00:00',96,558,576,97.75,'Sun-Synch' from dual union all
select 28739,'028739','2005-024C','CZ-2D Stage 2 motor cover','',68033,'In Earth orbit','','2005-08-05 00:00:00',98.6,555,829,97.47,'Sun-Synch' from dual union all
select 2874,'002874','1967-053H','NRL PL153','POPPY 5 SV3',15308,'In Earth orbit','','1967-07-12 00:00:00',103.41,913,927,69.98,'Intermediate' from dual union all
select 28740,'028740','2005-024D','CZ-2D Stage 2 motor cover','',68033,'In Earth orbit','','2005-08-04 00:00:00',98.99,557,864,97.64,'Sun-Synch' from dual union all
select 28741,'028741','2005-024E','deb SJ-7','',68033,'Reentered','2005-08-23 00:00:00','2005-08-05 00:00:00',91.78,249,475,97.44,'Sun-Synch' from dual union all
select 28742,'028742','2005-024F','CZ-2D Stage 2 motor cover','',68033,'In Earth orbit','','2005-08-05 00:00:00',98.67,555,836,97.64,'Sun-Synch' from dual union all
select 28743,'028743','2002-037AN','deb SOZ','',67413,'Reentered','2005-11-04 00:00:00','2005-07-07 00:00:00',95.48,260,823,63.71,'Intermediate' from dual union all
select 28744,'028744','2002-037AP','deb SOZ','',67413,'Reentered','2005-10-01 00:00:00','2005-07-07 00:00:00',94.49,229,759,63.79,'Intermediate' from dual union all
select 28745,'028745','2002-037AQ','deb SOZ','',67413,'Reentered','2005-08-27 00:00:00','2005-07-07 00:00:00',93.51,221,672,63.94,'Intermediate' from dual union all
select 28746,'028746','2002-037AR','deb SOZ','',67413,'Reentered','2005-08-02 00:00:00','2005-07-07 00:00:00',95.6,300,795,64.05,'Intermediate' from dual union all
select 28747,'028747','2002-037AS','deb SOZ','',67413,'Reentered','2005-07-12 00:00:00','2005-07-07 00:00:00',91.73,238,481,63.71,'Intermediate' from dual union all
select 28748,'028748','2002-037AT','deb SOZ','',67413,'Reentered','2012-04-29 00:00:00','2005-07-08 00:00:00',96.96,290,936,63.3,'Intermediate' from dual union all
select 28749,'028749','2002-037AU','deb SOZ','',67413,'Reentered','2005-07-10 00:00:00','2005-07-07 00:00:00',93.55,200,696,63.74,'Intermediate' from dual union all
select 2875,'002875','1967-068A','Surveyor 4','Surveyor SC-4',15323,'Deep Space','1967-07-16 00:00:00','1967-07-14 00:00:00',25038.8,167,554098,30.56,'Cislunar' from dual union all
select 28750,'028750','2002-037AV','deb SOZ','',67413,'Reentered','2005-12-11 00:00:00','2005-07-07 00:00:00',97.08,249,989,63.94,'Intermediate' from dual union all
select 28751,'028751','2002-037AW','deb SOZ','',67413,'Reentered','2006-04-30 00:00:00','2005-07-08 00:00:00',95.4,284,792,63.4,'Intermediate' from dual union all
select 28752,'028752','2002-037AX','deb SOZ','',67413,'Reentered','2008-12-21 00:00:00','2005-07-07 00:00:00',95.94,320,808,63.11,'Intermediate' from dual union all
select 28753,'028753','2002-037AY','deb SOZ','',67413,'Reentered','2010-01-13 00:00:00','2005-07-07 00:00:00',96.66,295,903,63.09,'Intermediate' from dual union all
select 28754,'028754','2002-037AZ','deb SOZ','',67413,'Reentered','2009-11-10 00:00:00','2005-07-07 00:00:00',98.99,425,996,63.62,'Intermediate' from dual union all
select 28755,'028755','2002-037BA','deb SOZ','',67413,'Reentered','2005-11-25 00:00:00','2005-07-07 00:00:00',95.33,300,770,62.61,'Intermediate' from dual union all
select 28756,'028756','2002-037BB','deb SOZ','',67413,'Reentered','2005-08-03 00:00:00','2005-07-07 00:00:00',93.42,210,674,63.93,'Intermediate' from dual union all
select 28757,'028757','2002-037BC','deb SOZ','',67413,'Reentered','2005-10-09 00:00:00','2005-07-07 00:00:00',99.04,321,1105,63.45,'Intermediate' from dual union all
select 28758,'028758','2002-037BD','deb SOZ','',67413,'Reentered','2005-12-19 00:00:00','2005-07-07 00:00:00',99.21,314,1128,63.24,'Intermediate' from dual union all
select 28759,'028759','2002-037BE','deb SOZ','',67413,'Reentered','2005-07-22 00:00:00','2005-07-07 00:00:00',92.14,291,468,63.4,'Intermediate' from dual union all
select 2876,'002876','1967-065B','Aurora 1','Aurora 1',15320,'In Earth orbit','','1967-08-01 00:00:00',172.11,3797,3946,89.81,'Medium' from dual union all
select 28760,'028760','2002-037BF','deb SOZ','',67413,'Reentered','2005-07-15 00:00:00','2005-07-07 00:00:00',93.86,217,710,63.75,'Intermediate' from dual union all
select 28761,'028761','2002-037BG','deb SOZ','',67413,'Reentered','2005-07-13 00:00:00','2005-07-08 00:00:00',94.23,195,768,64.59,'Intermediate' from dual union all
select 28762,'028762','2002-037BH','deb SOZ','',67413,'Reentered','2005-07-27 00:00:00','2005-07-07 00:00:00',95.37,254,819,63.43,'Intermediate' from dual union all
select 28763,'028763','2002-037BJ','deb SOZ','',67413,'Reentered','2006-05-18 00:00:00','2005-07-07 00:00:00',98.3,275,1080,63.35,'Intermediate' from dual union all
select 28764,'028764','2002-037BK','deb SOZ','',67413,'Reentered','2005-10-24 00:00:00','2005-07-07 00:00:00',99.32,329,1123,63.38,'Intermediate' from dual union all
select 28765,'028765','2002-037BL','deb SOZ','',67413,'Reentered','2005-08-24 00:00:00','2005-07-08 00:00:00',94.03,241,703,63.78,'Intermediate' from dual union all
select 28766,'028766','2002-037BM','deb SOZ','',67413,'Reentered','2005-09-16 00:00:00','2005-07-08 00:00:00',94.28,234,734,63.89,'Intermediate' from dual union all
select 28767,'028767','2002-037BN','deb SOZ','',67413,'Reentered','2005-07-15 00:00:00','2005-07-07 00:00:00',93.27,237,632,63.68,'Intermediate' from dual union all
select 28768,'028768','2002-037BP','deb SOZ','',67413,'Reentered','2005-08-18 00:00:00','2005-07-07 00:00:00',99.47,277,1190,63.57,'Intermediate' from dual union all
select 28769,'028769','2002-037BQ','deb SOZ','',67413,'Reentered','2005-07-10 00:00:00','2005-07-07 00:00:00',93.48,208,681,63.64,'Intermediate' from dual union all
select 2877,'002877','1967-065C','Injection stage','Star 13A/Plume shield',15320,'In Earth orbit','','1967-07-31 00:00:00',172.11,3797,3946,89.82,'Medium' from dual union all
select 28770,'028770','2002-037BR','deb SOZ','',67413,'Reentered','2013-04-27 00:00:00','2005-07-07 00:00:00',95.59,296,799,63.28,'Intermediate' from dual union all
select 28771,'028771','2002-037BS','deb SOZ','',67413,'Reentered','2005-12-28 00:00:00','2005-07-07 00:00:00',97.4,265,1004,63.63,'Intermediate' from dual union all
select 28772,'028772','2002-037BT','deb SOZ','',67413,'Reentered','2005-08-05 00:00:00','2005-07-07 00:00:00',93.89,344,586,63.51,'Intermediate' from dual union all
select 28773,'028773','2005-025A','Suzaku','ASTRO-E2',68034,'In Earth orbit','','2005-07-10 00:00:00',92.43,248,540,31.39,'Intermediate' from dual union all
select 28774,'028774','2005-025B','M-34','M-34',68034,'Reentered','2005-12-23 00:00:00','2005-07-10 00:00:00',92.92,295,541,31.4,'Intermediate' from dual union all
select 28775,'028775','2005-026A','Discovery (STS-114)','OV-103',68035,'Landed','2005-08-09 00:00:00','2005-07-31 00:00:00',91.59,350,356,51.65,'Intermediate' from dual union all
select 28776,'028776','2005-027A','Fanhui Shi Weixing (21)','Jianbing-2 02 xing',68036,'Reentered','2005-10-16 00:00:00','2005-08-04 00:00:00',91.1,166,491,63,'Intermediate' from dual union all
select 28777,'028777','2005-027B','CZ-2C Stage 2','CZ-2C Stage 2',68036,'Reentered','2005-08-16 00:00:00','2005-08-04 00:00:00',90.78,164,462,63,'Intermediate' from dual union all
select 28778,'028778','2005-027C','deb FSW','',68036,'Reentered','2005-08-03 00:00:00','2005-08-02 00:00:00',89.84,157,377,63.02,'Intermediate' from dual union all
select 28779,'028779','2005-027D','deb FSW','',68036,'Reentered','2005-08-03 00:00:00','2005-08-02 00:00:00',91.15,159,503,62.99,'Intermediate' from dual union all
select 2878,'002878','1967-069A','Kosmos-169','OGCh No. 10',15324,'Deorbited','1967-07-17 00:00:00','1967-07-17 00:00:00',87.83,135,200,49.68,'Intermediate' from dual union all
select 28780,'028780','2005-027E','CZ-2C Stage 2 motor cover','',68036,'Reentered','2005-08-04 00:00:00','2005-08-03 00:00:00',92.03,165,584,63.36,'Intermediate' from dual union all
select 28781,'028781','2005-027F','CZ-2C Stage 2 motor cover','',68036,'Reentered','2005-08-04 00:00:00','2005-08-03 00:00:00',92.29,166,608,63.26,'Intermediate' from dual union all
select 28782,'028782','2005-027G','CZ-2C Stage 2 motor cover','',68036,'Reentered','2005-08-04 00:00:00','2005-08-03 00:00:00',92.11,161,595,62.7,'Intermediate' from dual union all
select 28783,'028783','2005-027H','CZ-2C Stage 2 motor cover','',68036,'Reentered','2005-08-04 00:00:00','2005-08-03 00:00:00',92.13,158,599,62.77,'Intermediate' from dual union all
select 28784,'028784','2005-027J','deb FSW','',68036,'Reentered','2005-08-04 00:00:00','2005-08-04 00:00:00',88.59,130,280,62.74,'Intermediate' from dual union all
select 28785,'028785','2005-024G','CZ-2D Stage 2 motor cover','',68033,'In Earth orbit','','2005-08-05 00:00:00',98.66,555,835,97.59,'Sun-Synch' from dual union all
select 28786,'028786','2005-028A','Thaicom 4','iPSTAR 1',68037,'In Earth orbit','','2005-10-11 00:00:00',1436.11,35777,35797,.04,'Stationary' from dual union all
select 28787,'028787','2005-028B','EPS 523','EPS S/N 522 +',68037,'In Earth orbit','','2005-10-10 00:00:00',635.76,553,35675,6.67,'GEO Transfer' from dual union all
select 28788,'028788','2005-029A','Mars Reconnaissance Orbiter','MRO',68038,'Deep Space','2005-08-12 00:00:00','2005-08-12 00:00:00',633.32,205,61820,40.72,'Earth Escape' from dual union all
select 28789,'028789','2005-029B','Centaur AV-007','Centaur AV-007',68038,'Deep Space','2005-08-12 00:00:00','2005-08-12 00:00:00',674.75,116,63838,41.1,'Earth Escape' from dual union all
select 2879,'002879','1967-069B','Kosmos-169 Adapter','OGCh Adapter',15324,'Reentered','1967-07-18 00:00:00','1967-07-18 00:00:00',87.09,110,151,49.68,'Intermediate' from dual union all
select 28790,'028790','2005-030A','Galaxy 14','Galaxy 14',68039,'In Earth orbit','','2009-07-02 00:00:00',1436.06,35776,35796,.02,'Stationary' from dual union all
select 28791,'028791','2005-030B','Fregat','Fregat 14S44 No. 1007',68039,'Reentered','2006-01-20 00:00:00','2005-08-16 00:00:00',1306.09,334,66070,49.24,'Highly Ellip' from dual union all
select 28792,'028792','1998-067AG','Matroshka-R cover','EVA debris',66503,'Reentered','2005-08-23 00:00:00','2005-08-19 00:00:00',91.47,343,350,51.64,'Intermediate' from dual union all
select 28793,'028793','1965-063C','deb FW4S','',11813,'Reentered','2005-09-17 00:00:00','2005-03-15 00:00:00',119.78,1041,2301,69.24,'Intermediate' from dual union all
select 28794,'028794','1965-063D','deb FW4S','',11813,'In Earth orbit','','2005-08-19 00:00:00',121.98,1136,2403,69.21,'Medium' from dual union all
select 28795,'028795','1965-063E','deb FW4S','',11813,'In Earth orbit','','2005-08-20 00:00:00',121.94,1144,2391,69.22,'Medium' from dual union all
select 28796,'028796','1965-063F','deb FW4S','',11813,'In Earth orbit','','2005-07-30 00:00:00',121.06,1134,2323,69.2,'Medium' from dual union all
select 28797,'028797','1965-063G','deb FW4S','',11813,'In Earth orbit','','2005-07-25 00:00:00',120.68,1150,2273,69.26,'Medium' from dual union all
select 28798,'028798','1965-063H','deb FW4S','',11813,'In Earth orbit','','2005-07-04 00:00:00',121.07,1129,2329,69.07,'Medium' from dual union all
select 28799,'028799','1965-063J','deb FW4S','',11813,'In Earth orbit','','2005-08-13 00:00:00',120.84,1126,2311,69.2,'Medium' from dual union all
select 288,'000288','62 Omicron 2','Altair','Altair X-248-A5DM SV-186',8197,'Reentered','1971-04-28 00:00:00','1962-06-15 00:00:00',100.89,398,1204,53.86,'Intermediate' from dual union all
select 2880,'002880','1967-069C','8S692','8S692',15324,'Reentered','1967-07-18 00:00:00','1967-07-17 00:00:00',87.88,125,215,49.68,'Intermediate' from dual union all
select 28800,'028800','1965-063K','deb FW4S','',11813,'In Earth orbit','','2005-07-31 00:00:00',120.32,1084,2307,69.27,'Medium' from dual union all
select 28801,'028801','1965-063L','deb FW4S','',11813,'In Earth orbit','','2005-06-10 00:00:00',120.75,1097,2332,69.23,'Medium' from dual union all
select 28802,'028802','1965-063M','deb FW4S','',11813,'In Earth orbit','','2005-08-13 00:00:00',120.85,1163,2275,69.23,'Medium' from dual union all
select 28803,'028803','1965-063N','deb FW4S','',11813,'In Earth orbit','','2005-08-14 00:00:00',121.45,1151,2341,69.27,'Medium' from dual union all
select 28804,'028804','1965-108BW','deb Transtage 8','',11858,'In Earth orbit','','2005-08-21 00:00:00',543.83,1165,30219,27.17,'Highly Ellip' from dual union all
select 28805,'028805','1965-108BX','deb Transtage 8','',11858,'In Earth orbit','','2005-01-19 00:00:00',525.02,552,29808,26.39,'Highly Ellip' from dual union all
select 28806,'028806','1965-108BY','deb Transtage 8','',11858,'In Earth orbit','','2005-04-03 00:00:00',540.69,1140,30074,28.29,'Highly Ellip' from dual union all
select 28807,'028807','1965-108BZ','deb Transtage 8','',11858,'In Earth orbit','','2005-08-11 00:00:00',560.9,537,31766,25.13,'Highly Ellip' from dual union all
select 28808,'028808','1965-108CA','deb Transtage 8','',11858,'In Earth orbit','','2005-07-14 00:00:00',391.23,299,22384,27.84,'Highly Ellip' from dual;
insert into SATELLITE
select 28809,'028809','2005-031A','Kirari','OICETS',68040,'In Earth orbit','','2005-08-26 00:00:00',96.78,597,612,97.83,'Sun-Synch' from dual union all
select 2881,'002881','1967-069D','deb Kosmos-169','',15324,'Reentered','1967-07-18 00:00:00','1967-07-17 00:00:00',87.88,125,215,49.68,'Intermediate' from dual union all
select 28810,'028810','2005-031B','Reimei','INDEX',68040,'In Earth orbit','','2005-08-26 00:00:00',97.2,602,648,97.83,'Sun-Synch' from dual union all
select 28811,'028811','2005-031C','R-36M2 DS','R-36M2 DS',68040,'In Earth orbit','','2005-08-25 00:00:00',102.9,595,1196,97.81,'Sun-Synch' from dual union all
select 28812,'028812','2005-031D','OICETS adapter','',68040,'In Earth orbit','','2005-08-25 00:00:00',96.91,601,621,97.83,'Sun-Synch' from dual union all
select 28813,'028813','2005-031E','Dnepr plume shield','GDS',68040,'In Earth orbit','','2005-08-25 00:00:00',96.46,572,607,97.83,'Sun-Synch' from dual union all
select 28814,'028814','1965-108CB','deb Transtage 8','',11858,'In Earth orbit','','2005-08-23 00:00:00',554.42,830,31125,26.94,'Highly Ellip' from dual union all
select 28815,'028815','1965-108CC','deb Transtage 8','',11858,'In Earth orbit','','2005-08-20 00:00:00',552.69,930,30932,26.23,'Highly Ellip' from dual union all
select 28816,'028816','1965-108CD','deb Transtage 8','',11858,'In Earth orbit','','2005-03-22 00:00:00',546.83,1342,30204,25.85,'Highly Ellip' from dual union all
select 28817,'028817','1965-108CE','deb Transtage 8','',11858,'In Earth orbit','','2005-08-21 00:00:00',432.05,419,24688,26.37,'Highly Ellip' from dual union all
select 28818,'028818','1965-108CF','deb Transtage 8','',11858,'In Earth orbit','','2005-08-03 00:00:00',559.17,1126,31084,26.95,'Highly Ellip' from dual union all
select 28819,'028819','1965-108CG','deb Transtage 8','',11858,'In Earth orbit','','2005-08-12 00:00:00',532.6,747,30027,27.54,'Highly Ellip' from dual union all
select 2882,'002882','1967-069E','deb Kosmos-169','',15324,'Reentered','1967-07-18 00:00:00','1967-07-18 00:00:00',88.27,146,232,49.68,'Intermediate' from dual union all
select 28820,'028820','1965-108CH','deb Transtage 8','',11858,'In Earth orbit','','2005-05-25 00:00:00',515.78,452,29401,26.58,'Highly Ellip' from dual union all
select 28821,'028821','1965-108CJ','deb Transtage 8','',11858,'In Earth orbit','','2005-02-12 00:00:00',546.42,1345,30179,28.01,'Highly Ellip' from dual union all
select 28822,'028822','2005-032A','Monitor-E No. 1','98M',68041,'In Earth orbit','','2005-08-28 00:00:00',95.31,524,544,97.56,'Sun-Synch' from dual union all
select 28823,'028823','2005-032B','Briz-KM No. 72507','Briz-KM 14S45 No. 72507',68041,'Reentered','2005-09-13 00:00:00','2005-08-28 00:00:00',91.28,148,527,97.76,'Sun-Synch' from dual union all
select 28824,'028824','2005-033A','Fanhui Shi Weixing (22)','Jianbing-4 03 xing',68042,'Reentered','2005-10-16 00:00:00','2005-08-29 00:00:00',89.5,203,297,62.98,'Intermediate' from dual union all
select 28825,'028825','2005-033B','CZ-2D Stage 2 motor cover','',68042,'Reentered','2005-08-30 00:00:00','2005-08-29 00:00:00',89.35,196,289,62.97,'Intermediate' from dual union all
select 28826,'028826','2005-033C','CZ-2D Stage 2 motor cover','',68042,'Reentered','2005-08-30 00:00:00','2005-08-29 00:00:00',89.43,197,296,62.89,'Intermediate' from dual union all
select 28827,'028827','2005-033D','CZ-2D Stage 2','CZ-2D Stage 2',68042,'Reentered','2005-09-12 00:00:00','2005-08-29 00:00:00',89.46,203,293,62.97,'Intermediate' from dual union all
select 28828,'028828','2005-031F','deb OICETS','',68040,'Reentered','2014-08-03 00:00:00','2005-08-29 00:00:00',96.73,593,612,97.83,'Sun-Synch' from dual union all
select 28829,'028829','2005-031G','deb INDEX','',68040,'Reentered','2014-07-28 00:00:00','2005-08-29 00:00:00',97.23,604,649,97.82,'Sun-Synch' from dual union all
select 2883,'002883','1967-068B','Centaur D AC-11','Centaur D AC-11',15323,'Deep Space','1967-07-15 00:00:00','1967-07-14 00:00:00',15955.21,170,406954,30.59,'Cislunar' from dual union all
select 28830,'028830','2005-031H','deb OICETS','',68040,'Reentered','2014-09-10 00:00:00','2005-08-29 00:00:00',96.78,597,612,97.82,'Sun-Synch' from dual union all
select 28831,'028831','2005-031J','deb OICETS','',68040,'Reentered','2014-09-07 00:00:00','2005-08-29 00:00:00',96.8,597,614,97.82,'Sun-Synch' from dual union all
select 28832,'028832','2005-031K','deb INDEX','',68040,'Reentered','2015-02-16 00:00:00','2005-08-28 00:00:00',97.18,599,648,97.83,'Sun-Synch' from dual union all
select 28833,'028833','1965-108CK','deb Transtage 8','',11858,'In Earth orbit','','2005-08-29 00:00:00',544,1179,30214,28.18,'Highly Ellip' from dual union all
select 28834,'028834','1965-108CL','deb Transtage 8','',11858,'In Earth orbit','','2005-06-02 00:00:00',556.32,1163,30894,27.43,'Highly Ellip' from dual union all
select 28835,'028835','1965-108CM','deb Transtage 8','',11858,'In Earth orbit','','2005-06-11 00:00:00',524.92,632,29723,27.42,'Highly Ellip' from dual union all
select 28836,'028836','1965-108CN','deb Transtage 8','',11858,'In Earth orbit','','2005-01-20 00:00:00',539.01,574,30549,26.97,'Highly Ellip' from dual union all
select 28837,'028837','1965-108CP','deb Transtage 8','',11858,'In Earth orbit','','2005-05-11 00:00:00',546.81,1311,30234,26.17,'Highly Ellip' from dual union all
select 28838,'028838','1965-108CQ','deb Transtage 8','',11858,'In Earth orbit','','2005-08-12 00:00:00',534.95,696,30206,26.48,'Highly Ellip' from dual union all
select 28839,'028839','1965-108CR','deb Transtage 8','',11858,'In Earth orbit','','2005-01-29 00:00:00',565.95,946,31627,26.75,'Highly Ellip' from dual union all
select 2884,'002884','1967-070A','Explorer 35','AIMP E',15325,'Deep Space','1967-07-22 00:00:00','1967-07-19 00:00:00',25309,650,557687,29.34,'Cislunar' from dual union all
select 28840,'028840','1965-108CS','deb Transtage 8','',11858,'In Earth orbit','','2005-06-05 00:00:00',568.54,523,32188,26.83,'Highly Ellip' from dual union all
select 28841,'028841','2005-034A','Kosmos-2415','Kometa No. 21',68043,'Landed','2005-10-16 00:00:00','2005-09-09 00:00:00',89.3,202,277,64.88,'Intermediate' from dual union all
select 28842,'028842','2005-034B','11S510','11S510',68043,'Reentered','2005-09-08 00:00:00','2005-09-04 00:00:00',88.97,188,260,64.9,'Intermediate' from dual union all
select 28843,'028843','1978-064Q','deb Seasat','',45259,'Reentered','2006-09-08 00:00:00','2005-08-24 00:00:00',98.78,670,731,108.01,'Retrograde' from dual union all
select 28844,'028844','1991-050J','SARA antenna debris','',64219,'Reentered','2011-09-26 00:00:00','2005-09-07 00:00:00',99.08,709,721,98.1,'Sun-Synch' from dual union all
select 28845,'028845','1993-014W','deb Start','',65116,'Reentered','2011-12-03 00:00:00','2005-08-27 00:00:00',99.48,661,806,75.76,'Intermediate' from dual union all
select 28846,'028846','1993-014X','deb Start','',65116,'Reentered','2012-01-17 00:00:00','2005-09-03 00:00:00',99.05,643,783,75.75,'Intermediate' from dual union all
select 28847,'028847','1993-014Y','deb Start','',65116,'Reentered','2012-12-22 00:00:00','2005-08-31 00:00:00',99.71,659,831,75.75,'Intermediate' from dual union all
select 28848,'028848','1993-014Z','deb Start','',65116,'Reentered','2012-08-27 00:00:00','2005-09-02 00:00:00',99.64,655,828,75.75,'Intermediate' from dual union all
select 28849,'028849','1993-014AA','deb Start','',65116,'Reentered','2010-03-10 00:00:00','2005-08-27 00:00:00',99.04,688,738,75.75,'Intermediate' from dual union all
select 2885,'002885','1967-070B','Delta 50','DSV-3E-3 S/N 20217',15325,'Reentered','1967-08-31 00:00:00','1967-08-07 00:00:00',99.64,155,1328,29.58,'Intermediate' from dual union all
select 28850,'028850','2001-049PC','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2005-09-06 00:00:00',97.37,604,662,97.84,'Sun-Synch' from dual union all
select 28851,'028851','2001-049PD','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2005-09-04 00:00:00',97.79,612,694,97.9,'Sun-Synch' from dual union all
select 28852,'028852','2001-049PE','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2005-08-31 00:00:00',97.97,629,695,97.87,'Sun-Synch' from dual union all
select 28853,'028853','2001-049PF','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2005-09-04 00:00:00',97.75,622,681,97.87,'Sun-Synch' from dual union all
select 28854,'028854','2001-049PG','deb PSLV-C3 PS4','',67202,'Reentered','2005-11-03 00:00:00','2005-09-07 00:00:00',91.99,362,383,97.65,'Sun-Synch' from dual union all
select 28855,'028855','2001-049PH','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2005-08-25 00:00:00',116.24,1444,1580,101.12,'Sun-Synch' from dual union all
select 28856,'028856','2001-049PJ','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2005-08-30 00:00:00',98.09,635,699,97.82,'Sun-Synch' from dual union all
select 28857,'028857','2001-049PK','deb PSLV-C3 PS4','',67202,'Reentered','2014-11-18 00:00:00','2005-09-03 00:00:00',96.68,543,657,97.25,'Sun-Synch' from dual union all
select 28858,'028858','2001-049PL','deb PSLV-C3 PS4','',67202,'Reentered','2017-03-08 00:00:00','2005-09-04 00:00:00',97.34,599,664,97.84,'Sun-Synch' from dual union all
select 28859,'028859','2001-049PM','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2005-09-01 00:00:00',97.34,603,660,97.86,'Sun-Synch' from dual union all
select 2886,'002886','1963-047K','Centaur AC-2 part','Insulation blanket part',9278,'In Earth orbit','','1967-03-30 00:00:00',108.83,667,1679,29.87,'Intermediate' from dual union all
select 28860,'028860','2002-037BU','deb SOZ','',67413,'Reentered','2005-09-18 00:00:00','2005-09-06 00:00:00',92.88,268,564,63.17,'Intermediate' from dual union all
select 28861,'028861','2002-037BV','deb SOZ','',67413,'Reentered','2005-11-04 00:00:00','2005-09-07 00:00:00',95.06,295,748,62.94,'Intermediate' from dual union all
select 28862,'028862','2004-044C','CZ-4B sep motor cover','',67888,'Reentered','2014-02-14 00:00:00','2005-09-04 00:00:00',97.7,481,817,97.35,'Sun-Synch' from dual union all
select 28863,'028863','2004-044D','CZ-4B sep motor cover','',67888,'Reentered','2013-12-16 00:00:00','2005-09-03 00:00:00',97.49,481,797,97.27,'Sun-Synch' from dual union all
select 28864,'028864','2004-044E','CZ-4B sep motor cover','',67888,'Reentered','2014-02-05 00:00:00','2005-09-07 00:00:00',97.79,481,825,97.2,'Sun-Synch' from dual union all
select 28865,'028865','2004-044F','CZ-4B sep motor cover','',67888,'Reentered','2013-06-03 00:00:00','2005-09-04 00:00:00',97.27,480,776,97.33,'Sun-Synch' from dual union all
select 28866,'028866','2005-035A','Progress M-54','Progress 7K-TGM No. 354/',68044,'Deorbited','2006-03-03 00:00:00','2005-09-16 00:00:00',91.53,349,351,51.64,'Intermediate' from dual union all
select 28867,'028867','2005-035B','11S510','11S510',68044,'Reentered','2005-09-11 00:00:00','2005-09-09 00:00:00',88.49,178,222,51.64,'Intermediate' from dual union all
select 28868,'028868','2005-036A','Anik F1R','Anik F1R',68045,'In Earth orbit','','2005-11-08 00:00:00',1436.06,35777,35795,.04,'Stationary' from dual union all
select 28869,'028869','2005-036B','Briz-M No. 88513','Briz-M 14S43 No. 88513',68045,'In Earth orbit','','2005-11-08 00:00:00',683.3,3193,35447,10.14,'GEO Transfer' from dual union all
select 2887,'002887','1965-082KY','deb Transtage 4','',11832,'Reentered','1980-03-21 00:00:00','1967-03-30 00:00:00',99.71,621,869,30.96,'Intermediate' from dual union all
select 28870,'028870','2005-036C','Briz-M DTB','Briz-M torus tank',68045,'In Earth orbit','','2005-11-08 00:00:00',304.98,312,16949,49.4,'Highly Ellip' from dual union all
select 28871,'028871','2005-037A','STP-R1','Streak',68046,'Reentered','2006-06-28 00:00:00','2005-09-25 00:00:00',90.68,296,320,96.34,'Sun-Synch' from dual union all
select 28872,'028872','2005-037B','Minotaur Stage 4','Orion 38',68046,'Reentered','2005-11-29 00:00:00','2005-09-25 00:00:00',90.6,295,313,96.36,'Sun-Synch' from dual union all
select 28873,'028873','2005-037C','deb Minotaur','',68046,'Reentered','2005-10-26 00:00:00','2005-09-26 00:00:00',90.63,292,318,96.34,'Sun-Synch' from dual union all
select 28874,'028874','2005-038A','GPS IIR-14(M)','Navstar GPS SVN 53',68047,'In Earth orbit','','2005-11-25 00:00:00',717.92,20140,20222,55.03,'Medium' from dual union all
select 28875,'028875','2005-038B','Delta 313','Delta SSPS AJ-10-118K',68047,'Reentered','2006-01-30 00:00:00','2005-09-28 00:00:00',99.12,191,1243,35.21,'Intermediate' from dual union all
select 28876,'028876','2005-038C','Star 48B','Star 48B',68047,'In Earth orbit','','2005-11-26 00:00:00',356.22,250,20286,39.48,'Highly Ellip' from dual union all
select 28877,'028877','2005-039A','Soyuz TMA-7','Soyuz 11F732 No. 217',68048,'Landed','2006-04-08 00:00:00','2005-10-03 00:00:00',91.49,347,349,51.64,'Intermediate' from dual union all
select 28878,'028878','2005-039B','11S510','11S510',68048,'Reentered','2005-10-04 00:00:00','2005-10-02 00:00:00',88.41,186,206,51.64,'Intermediate' from dual union all
select 28879,'028879','2005-040A','Shenzhou 6','Shenzhou 6 Fanhui Cang',68049,'Landed','2005-10-16 00:00:00','2005-10-14 00:00:00',91.19,330,336,42.41,'Intermediate' from dual union all
select 2888,'002888','1965-089D','Explorer 29 despin weight','',11839,'In Earth orbit','','1967-07-16 00:00:00',121.33,1157,2324,59.2,'Medium' from dual union all
select 28880,'028880','2005-040B','CZ-2F Stage 2','CZ-2F Stage 2',68049,'Reentered','2005-10-26 00:00:00','2005-10-14 00:00:00',89.62,194,318,42.41,'Intermediate' from dual union all
select 28881,'028881','2005-040C','CZ-2F sep motor cover','',68049,'Reentered','2005-10-12 00:00:00','2005-10-12 00:00:00',91.29,186,490,42.59,'Intermediate' from dual union all
select 28882,'028882','2005-040D','CZ-2F sep motor cover','',68049,'Reentered','2005-10-15 00:00:00','2005-10-13 00:00:00',89.87,210,326,42.54,'Intermediate' from dual union all
select 28883,'028883','2005-040E','Shenzhou 6 Guidao Cang','Shenzhou 6 orbital modul',68049,'Reentered','2008-04-01 00:00:00','2005-10-19 00:00:00',91.26,333,340,42.43,'Intermediate' from dual union all
select 28884,'028884','2005-041A','Galaxy 15','Galaxy 15',68050,'In Earth orbit','','2005-12-13 00:00:00',1436.09,35778,35795,.06,'Stationary' from dual union all
select 28885,'028885','2005-041B','Syracuse 3A','Syracuse 3A',68050,'In Earth orbit','','2005-12-14 00:00:00',1436.04,35769,35802,.02,'Stationary' from dual union all
select 28886,'028886','2005-041C','EPS 524','EPS S/N 519 +',68050,'In Earth orbit','','2005-12-14 00:00:00',642.1,594,35959,7.61,'GEO Transfer' from dual union all
select 28887,'028887','2005-041D','Sylda 5 SY-14E','Sylda 5E',68050,'In Earth orbit','','2005-12-13 00:00:00',638.28,586,35771,6.96,'GEO Transfer' from dual union all
select 28888,'028888','2005-042A','USA 186','Enhanced CRYSTAL 2108',68051,'In Earth orbit','','2005-10-28 00:00:00',97.87,264,1050,97.88,'Sun-Synch' from dual union all
select 28889,'028889','2005-042B','Titan 4B-26 Stage 2','Titan 4B-26 Stage 2',68051,'Reentered','2006-01-01 00:00:00','2005-10-30 00:00:00',95.29,165,901,97.87,'Sun-Synch' from dual union all
select 2889,'002889','1966-005E','NNS O-7 despin weight','Transit high deb',13412,'In Earth orbit','','1967-07-18 00:00:00',109.63,1076,1344,89.5,'Polar' from dual union all
select 28890,'028890','2005-043A','Beijing-1','China-DMC',68052,'In Earth orbit','','2005-11-03 00:00:00',98.62,682,704,98.18,'Sun-Synch' from dual union all
select 28891,'028891','2005-043B','Topsat','Topsat',68052,'In Earth orbit','','2005-11-03 00:00:00',98.65,682,707,98.18,'Sun-Synch' from dual union all
select 28892,'028892','2005-043C','UWE-1','UWE-1',68052,'In Earth orbit','','2005-11-03 00:00:00',98.66,682,708,98.18,'Sun-Synch' from dual union all
select 28893,'028893','2005-043D','Sinah-1','Sina-1',68052,'In Earth orbit','','2005-11-03 00:00:00',98.62,682,704,98.18,'Sun-Synch' from dual union all
select 28894,'028894','2005-043E','Express-OSCAR-53','SSETI Express',68052,'In Earth orbit','','2005-11-03 00:00:00',98.66,682,708,98.18,'Sun-Synch' from dual union all
select 28895,'028895','2005-043F','Cubesat-OSCAR-58','Cubesat XI-V',68052,'In Earth orbit','','2005-11-03 00:00:00',98.66,682,708,98.18,'Sun-Synch' from dual union all
select 28896,'028896','2000-078B','FPP','Floating Potential Probe',66980,'Reentered','2006-02-25 00:00:00','2000-12-07 00:00:00',92.03,369,380,51.6,'Intermediate' from dual union all
select 28897,'028897','2005-043H','NCUBE-2','deb SSETI',68052,'In Earth orbit','','2005-11-03 00:00:00',98.66,682,708,98.18,'Sun-Synch' from dual union all
select 28898,'028898','2005-043G','S3M','S3M',68052,'In Earth orbit','','2005-11-03 00:00:00',98.72,683,712,98.18,'Sun-Synch' from dual union all
select 28899,'028899','2005-044A','INMARSAT 4 F2','INMARSAT 4 F2',68053,'In Earth orbit','','2006-01-07 00:00:00',1443.22,35891,35961,2.85,'Drift GEO' from dual;
insert into SATELLITE
select 289,'000289','62 Xi 3','Kosmos-4 part','',8206,'Reentered','1962-05-03 00:00:00','1962-04-26 00:00:00',90.8,298,330,65,'Intermediate' from dual union all
select 2890,'002890','1967-071A','OPS 1879','MULTIGROUP 2',15326,'Reentered Attached','1969-06-05 00:00:00','1967-08-27 00:00:00',94.26,454,512,75.03,'Intermediate' from dual union all
select 28900,'028900','2005-044B','Blok DM-SL No. 20L','Blok DM-SL',68053,'In Earth orbit','','2006-01-08 00:00:00',630.92,284,35695,2.92,'GEO Transfer' from dual union all
select 28901,'028901','2005-045A','Venus Express','Venus Express',68054,'Deep Space','2005-11-09 00:00:00','2005-11-09 00:00:00',1906.53,334,114954,51.67,'Earth Escape' from dual union all
select 28902,'028902','2005-046A','Telkom 2','Telkom 2',68055,'In Earth orbit','','2006-01-18 00:00:00',1436.11,35779,35795,.05,'Stationary' from dual union all
select 28903,'028903','2005-046B','Spaceway 2','Spaceway F2',68055,'In Earth orbit','','2006-01-16 00:00:00',1437.65,33020,38614,.15,'Near-synch' from dual union all
select 28904,'028904','2005-046C','ESC-A 522','Ariane 5 ESC-A FM-02',68055,'In Earth orbit','','2006-01-16 00:00:00',630.2,408,35534,6.73,'GEO Transfer' from dual union all
select 28905,'028905','2005-046D','Sylda 5 SY-17C','Sylda 5C',68055,'In Earth orbit','','2006-01-16 00:00:00',628.38,261,35587,6.73,'GEO Transfer' from dual union all
select 28906,'028906','2005-047A','Progress M-55','Progress 7K-TGM No. 355/',68056,'Deorbited','2006-06-19 00:00:00','2005-12-24 00:00:00',91.51,342,356,51.65,'Intermediate' from dual union all
select 28907,'028907','2005-047B','11S510','11S510',68056,'Reentered','2005-12-24 00:00:00','2005-12-23 00:00:00',88.26,173,203,51.65,'Intermediate' from dual union all
select 28908,'028908','2005-048A','Kosmos-2416','Rodnik',68057,'In Earth orbit','','2005-12-24 00:00:00',114.72,1438,1448,82.47,'Intermediate' from dual union all
select 28909,'028909','2005-048B','Gonets-M No. 11','Gonets-M No. 11',68057,'In Earth orbit','','2005-12-24 00:00:00',114.7,1437,1447,82.47,'Intermediate' from dual union all
select 2891,'002891','1967-071B','OPS 1879 sensor cover','',15326,'Reentered','1967-08-21 00:00:00','1967-08-02 00:00:00',93.61,442,461,75.02,'Intermediate' from dual union all
select 28910,'028910','2005-048C','S3M','S3M',68057,'In Earth orbit','','2005-12-24 00:00:00',114.8,1437,1455,82.47,'Intermediate' from dual union all
select 28911,'028911','2005-049A','Insat 4A','Insat 4A',68058,'In Earth orbit','','2006-02-20 00:00:00',1436.14,35764,35811,.04,'Stationary' from dual union all
select 28912,'028912','2005-049B','Meteosat 9','MSG 2',68058,'In Earth orbit','','2006-02-21 00:00:00',1436.04,35781,35790,1.73,'Stationary' from dual union all
select 28913,'028913','2005-049C','EPS 525','EPS+ 525',68058,'In Earth orbit','','2006-02-20 00:00:00',640.46,626,35843,3.57,'GEO Transfer' from dual union all
select 28914,'028914','2005-049D','Sylda 5','Sylda 5',68058,'In Earth orbit','','2006-02-20 00:00:00',645.05,629,36075,3.76,'GEO Transfer' from dual union all
select 28915,'028915','2005-050A','Kosmos-2419','Uragan No. 798',68059,'In Earth orbit','','2006-02-23 00:00:00',675.71,19076,19183,64.86,'Medium' from dual union all
select 28916,'028916','2005-050B','Kosmos-2418','Uragan-M No. 714',68059,'In Earth orbit','','2006-02-23 00:00:00',675.69,19078,19180,64.88,'Medium' from dual union all
select 28917,'028917','2005-050C','Kosmos-2417','Uragan-M No. 713',68059,'In Earth orbit','','2006-02-23 00:00:00',675.69,19107,19151,64.85,'Medium' from dual union all
select 28918,'028918','2005-050D','Blok DM-2 No. 106L','11S861 No. 106L',68059,'In Earth orbit','','2006-02-23 00:00:00',674.44,19081,19114,64.87,'Medium' from dual union all
select 28919,'028919','2005-050E','SOZ','SOZ',68059,'In Earth orbit','','2006-02-23 00:00:00',340.17,387,19141,64.81,'Highly Ellip' from dual union all
select 2892,'002892','1967-071C','OPS 1879 sensor cover','',15326,'Reentered','1967-10-16 00:00:00','1967-08-31 00:00:00',93.68,431,479,75.04,'Intermediate' from dual union all
select 28920,'028920','2005-050F','SOZ','SOZ',68059,'In Earth orbit','','2006-02-23 00:00:00',340.15,386,19140,64.94,'Highly Ellip' from dual union all
select 28921,'028921','2005-050G','8S812','8S812',68059,'Reentered','2005-12-25 00:00:00','2005-12-25 00:00:00',87.51,144,159,64.85,'Intermediate' from dual union all
select 28922,'028922','2005-051A','Giove A','Galileo IOV Experiment A',68060,'In Earth orbit','','2006-02-26 00:00:00',846.17,23229,23284,56.05,'Medium' from dual union all
select 28923,'028923','2005-051B','Fregat','Fregat 14S44 No. 1009',68060,'In Earth orbit','','2006-02-27 00:00:00',853.82,23379,23491,56.06,'Medium' from dual union all
select 28924,'028924','2005-052A','AMC 23','Americom 23',68061,'In Earth orbit','','2006-03-01 00:00:00',1436.06,35782,35790,0,'Stationary' from dual union all
select 28925,'028925','2005-052B','Briz-M No. 88514','Briz-M',68061,'In Earth orbit','','2006-06-07 00:00:00',741.67,6310,35217,18.51,'GEO Transfer' from dual union all
select 28926,'028926','2005-052C','Briz-M DTB','Briz-M torus tank',68061,'In Earth orbit','','2006-02-27 00:00:00',283.46,321,15510,49.55,'Highly Ellip' from dual union all
select 28927,'028927','1999-003D','Sample Return Capsule','SRC',66684,'Deep Space Attached','1999-02-08 00:00:00','1999-02-07 00:00:00',314.7,224,43632,28.49,'Earth Escape' from dual union all
select 28928,'028928','2006-001A','New Horizons','New Horizons',68173,'Deep Space','2006-01-19 00:00:00','2006-01-19 00:00:00',21.06,339,18149,29.58,'Earth Escape' from dual union all
select 28929,'028929','2006-001B','Centaur AV-010','Centaur AV-010',68173,'Deep Space','2006-01-20 00:00:00','2006-01-19 00:00:00',127.39,253,29783,31.65,'Earth Escape' from dual union all
select 2893,'002893','1967-072A','OV1-86','OV1-8S',15327,'Reentered','1972-02-22 00:00:00','1967-08-27 00:00:00',95.42,475,602,101.72,'Retrograde' from dual union all
select 28930,'028930','2006-001C','Star 48B','Star 48B',68173,'Deep Space','2006-01-19 00:00:00','2006-01-19 00:00:00',21.06,339,18149,29.58,'Earth Escape' from dual union all
select 28931,'028931','2006-002A','Daichi','ALOS',68174,'In Earth orbit','','2006-01-31 00:00:00',98.74,697,700,98.22,'Sun-Synch' from dual union all
select 28932,'028932','2006-002B','H-2A F8 Stage 2','H-2A Stage 2',68174,'In Earth orbit','','2006-01-31 00:00:00',97.16,547,699,98.21,'Sun-Synch' from dual union all
select 28933,'028933','2005-035C','Orlan-M No. 14','Orlan-M No. 14',68044,'Reentered','2006-09-07 00:00:00','2006-02-04 00:00:00',91.43,339,351,51.65,'Intermediate' from dual union all
select 28934,'028934','2005-035D','Orlan M-14 glove','EVA debris',68044,'Reentered','2006-03-20 00:00:00','2006-02-09 00:00:00',91.34,335,346,51.64,'Intermediate' from dual union all
select 28935,'028935','2006-003A','Echostar X','Echostar 10',68175,'In Earth orbit','','2006-04-18 00:00:00',1436.06,35785,35787,.07,'Stationary' from dual union all
select 28936,'028936','2006-003B','Blok DM-SL No. 14L','Blok DM-SL',68175,'In Earth orbit','','2006-04-17 00:00:00',656.04,1680,35584,.1,'GEO Transfer' from dual union all
select 28937,'028937','2006-004A','Himawari-7','MTSAT-2',68176,'In Earth orbit','','2006-04-20 00:00:00',1436.11,35774,35800,.02,'Stationary' from dual union all
select 28938,'028938','2006-004B','H-2A F9 Stage 2','H-2A Stage 2',68176,'In Earth orbit','','2006-04-19 00:00:00',613.58,218,34864,28.26,'GEO Transfer' from dual union all
select 28939,'028939','2006-005A','Akari','ASTRO-F',68177,'In Earth orbit','','2006-03-21 00:00:00',98.84,696,711,98.2,'Sun-Synch' from dual union all
select 2894,'002894','1967-072B','OV1-6P','X-258',15327,'Reentered','1970-11-15 00:00:00','1967-08-26 00:00:00',95.25,447,615,101.72,'Retrograde' from dual union all
select 28940,'028940','2006-005B','deb M-34','',68177,'Reentered','2006-06-05 00:00:00','2006-02-23 00:00:00',94.76,302,712,98.18,'Sun-Synch' from dual union all
select 28941,'028941','2006-005C','Cubesat-OSCAR-56','CUTE-1.7-APD',68177,'Reentered','2009-10-25 00:00:00','2006-02-23 00:00:00',94.7,298,711,98.19,'Sun-Synch' from dual union all
select 28942,'028942','2006-005D','M-34','M-34',68177,'Reentered','2011-02-02 00:00:00','2006-02-23 00:00:00',94.74,300,712,98.19,'Sun-Synch' from dual union all
select 28943,'028943','2006-006A','Arabsat 4A','Arabsat 4A',68178,'Deorbited','2006-03-24 00:00:00','2006-03-08 00:00:00',274.02,505,14688,51.53,'Highly Ellip' from dual union all
select 28944,'028944','2006-006B','Briz-M No. 88515','Briz-M 14S43 No. 88515',68178,'In Earth orbit','','2006-04-29 00:00:00',274.11,510,14689,51.53,'Highly Ellip' from dual union all
select 28945,'028945','2006-007A','Spainsat','Spainsat',68179,'In Earth orbit','','2006-05-11 00:00:00',1436.04,35762,35809,.01,'Stationary' from dual union all
select 28946,'028946','2006-007B','Hot Bird 7A','Hot Bird 7A',68179,'In Earth orbit','','2006-05-12 00:00:00',1436.06,35777,35795,.01,'Stationary' from dual union all
select 28947,'028947','2006-007C','Sylda 5A','Sylda 5',68179,'In Earth orbit','','2006-05-11 00:00:00',623.44,294,35298,5.08,'GEO Transfer' from dual union all
select 28948,'028948','2006-007D','ESC-A 527','Ariane 5 ESC-A',68179,'In Earth orbit','','2006-05-10 00:00:00',629.47,294,35610,4.94,'GEO Transfer' from dual union all
select 28949,'028949','1965-101E','deb FR-1','',11851,'Reentered','2011-11-16 00:00:00','2006-03-20 00:00:00',96.59,586,605,75.84,'Intermediate' from dual union all
select 2895,'002895','1967-073A','OGO 4','OGO D',15328,'Reentered','1972-08-16 00:00:00','1967-08-28 00:00:00',97.8,411,896,86,'Polar' from dual union all
select 28950,'028950','1993-014AB','deb Start','',65116,'In Earth orbit','','2006-03-20 00:00:00',100.64,678,900,75.76,'Intermediate' from dual union all
select 28951,'028951','1993-014AC','deb Start','',65116,'Reentered','2014-04-11 00:00:00','2006-03-20 00:00:00',100.61,672,903,75.76,'Intermediate' from dual union all
select 28952,'028952','1993-014AD','deb Start','',65116,'Reentered','2013-03-09 00:00:00','2006-03-18 00:00:00',100.32,682,866,75.76,'Intermediate' from dual union all
select 28953,'028953','1993-014AE','deb Start','',65116,'Reentered','2016-09-26 00:00:00','2006-03-20 00:00:00',100.65,674,905,75.76,'Intermediate' from dual union all
select 28954,'028954','1993-014AF','deb Start','',65116,'Reentered','2011-10-18 00:00:00','2006-03-20 00:00:00',99.74,659,833,75.75,'Intermediate' from dual union all
select 28955,'028955','1993-014AG','deb Start','',65116,'In Earth orbit','','2006-03-20 00:00:00',100.61,678,897,75.76,'Intermediate' from dual union all
select 28956,'028956','1993-014AH','deb Start','',65116,'In Earth orbit','','2006-03-19 00:00:00',100.45,673,887,75.76,'Intermediate' from dual union all
select 28957,'028957','1993-014AJ','deb Start','',65116,'Reentered','2012-12-31 00:00:00','2006-03-20 00:00:00',100.21,676,861,75.76,'Intermediate' from dual union all
select 28958,'028958','1993-014AK','deb Start','',65116,'In Earth orbit','','2006-03-12 00:00:00',100.43,670,888,75.77,'Intermediate' from dual union all
select 28959,'028959','1993-014AL','deb Start','',65116,'Reentered','2014-10-26 00:00:00','2006-03-15 00:00:00',100.17,664,870,75.76,'Intermediate' from dual union all
select 2896,'002896','1967-073B','Agena D 6802','Agena D 6802',15328,'Reentered','1974-05-15 00:00:00','1967-08-30 00:00:00',97.73,411,890,86.01,'Polar' from dual union all
select 28960,'028960','1993-014AM','deb Start','',65116,'Reentered','2008-12-24 00:00:00','2006-02-21 00:00:00',97.2,586,664,75.72,'Intermediate' from dual union all
select 28961,'028961','1993-014AN','deb Start','',65116,'Reentered','2017-10-20 00:00:00','2006-03-17 00:00:00',100.63,673,904,75.76,'Intermediate' from dual union all
select 28962,'028962','1993-014AP','deb Start','',65116,'Reentered','2012-11-25 00:00:00','2006-03-20 00:00:00',99.69,651,837,75.76,'Intermediate' from dual union all
select 28963,'028963','1993-014AQ','deb Start','',65116,'Reentered','2015-07-05 00:00:00','2006-03-20 00:00:00',100.67,672,909,75.76,'Intermediate' from dual union all
select 28964,'028964','1993-014AR','deb Start','',65116,'Reentered','2011-11-27 00:00:00','2006-03-07 00:00:00',99.13,645,790,75.75,'Intermediate' from dual union all
select 28965,'028965','1993-014AS','deb Start','',65116,'Reentered','2015-12-16 00:00:00','2006-03-11 00:00:00',100.25,666,874,75.76,'Intermediate' from dual union all
select 28966,'028966','1993-014AT','deb Start','',65116,'Reentered','2012-06-22 00:00:00','2006-01-21 00:00:00',100.02,681,838,75.78,'Intermediate' from dual union all
select 28967,'028967','1993-014AU','deb Start','',65116,'Reentered','2011-12-13 00:00:00','2006-03-20 00:00:00',99.85,666,836,75.75,'Intermediate' from dual union all
select 28968,'028968','1993-014AV','deb Start','',65116,'Reentered','2014-06-11 00:00:00','2006-03-12 00:00:00',100.09,666,860,75.76,'Intermediate' from dual union all
select 28969,'028969','1992-047AK','deb SOZ','',64759,'Reentered','2010-04-12 00:00:00','2006-03-12 00:00:00',222.85,598,10998,64.65,'Medium' from dual union all
select 2897,'002897','1967-072C','OV1-12P','FW4S',15327,'Reentered','1977-05-02 00:00:00','1967-09-01 00:00:00',95.54,537,553,101.62,'Retrograde' from dual union all
select 28970,'028970','1992-047AL','deb SOZ','',64759,'Reentered','2007-09-28 00:00:00','2006-03-20 00:00:00',231.57,583,11643,64.63,'Medium' from dual union all
select 28971,'028971','1992-047AM','deb SOZ','',64759,'Reentered','2011-05-13 00:00:00','2006-03-20 00:00:00',231.96,619,11635,64.45,'Medium' from dual union all
select 28972,'028972','1979-095H','deb Meteor-2','',47579,'In Earth orbit','','2006-03-17 00:00:00',101.39,813,835,81.21,'Intermediate' from dual union all
select 28973,'028973','1979-095J','deb Meteor-2','',47579,'In Earth orbit','','2006-03-21 00:00:00',102.33,859,879,81.21,'Intermediate' from dual union all
select 28974,'028974','1979-095K','deb Meteor-2','',47579,'In Earth orbit','','2006-03-14 00:00:00',102.35,860,879,81.2,'Intermediate' from dual union all
select 28975,'028975','1987-001C','deb Meteor-2','',60091,'In Earth orbit','','2006-03-20 00:00:00',103.89,934,951,82.48,'Intermediate' from dual union all
select 28976,'028976','2003-048J','deb DMSP 5D-3 F-16','',67672,'Reentered','2015-02-17 00:00:00','2006-03-14 00:00:00',101.08,801,819,98.97,'Sun-Synch' from dual union all
select 28977,'028977','1966-040AN','Nimbus 2 debris','',13447,'In Earth orbit','','2006-03-19 00:00:00',107.96,1090,1175,100.45,'Sun-Synch' from dual union all
select 28978,'028978','1966-040AP','Nimbus 2 debris','',13447,'In Earth orbit','','2006-03-21 00:00:00',107.92,1090,1171,100.44,'Sun-Synch' from dual union all
select 28979,'028979','1976-077FU','deb Delta 126','',39029,'In Earth orbit','','2006-03-19 00:00:00',131.56,1505,2876,102,'Medium' from dual union all
select 2898,'002898','1965-082KZ','deb Transtage 4','',11832,'Reentered','1971-06-27 00:00:00','1966-06-27 00:00:00',98.96,669,749,31.91,'Intermediate' from dual union all
select 28980,'028980','2006-008A','ST-5 FWD','Space Technology 5',68180,'In Earth orbit','','2006-03-22 00:00:00',137.02,298,4554,105.62,'Medium' from dual union all
select 28981,'028981','2006-008B','ST-5 MID','Space Technology 5',68180,'In Earth orbit','','2006-03-23 00:00:00',136.99,302,4547,105.61,'Medium' from dual union all
select 28982,'028982','2006-008C','ST-5 AFT','Space Technology 5',68180,'In Earth orbit','','2006-03-23 00:00:00',137.04,303,4551,105.62,'Medium' from dual union all
select 28983,'028983','2006-008D','Orion 38','Orion 38',68180,'In Earth orbit','','2006-03-23 00:00:00',137.19,303,4564,105.63,'Medium' from dual union all
select 28984,'028984','1990-011C','deb CZ-3','',63253,'In Earth orbit','','2006-03-23 00:00:00',467.45,425,26723,29.36,'Highly Ellip' from dual union all
select 28985,'028985','1992-047AN','deb SOZ','',64759,'In Earth orbit','','2006-03-28 00:00:00',223.07,648,10963,64.7,'Medium' from dual union all
select 28986,'028986','1980-073C','deb Meteor-2','',50283,'In Earth orbit','','2006-03-24 00:00:00',102,824,882,81.22,'Intermediate' from dual union all
select 28987,'028987','1980-073D','deb Meteor-2','',50283,'In Earth orbit','','2006-02-21 00:00:00',101.89,821,876,81.23,'Intermediate' from dual union all
select 28988,'028988','1988-005C','deb Meteor-2','',61254,'In Earth orbit','','2006-03-29 00:00:00',104.05,944,956,82.55,'Intermediate' from dual union all
select 28989,'028989','1988-005D','deb Meteor-2','',61254,'In Earth orbit','','2006-03-27 00:00:00',103.8,924,953,82.54,'Intermediate' from dual;
insert into SATELLITE
select 2899,'002899','1965-082LA','deb Transtage 4','',11832,'Reentered','1979-05-06 00:00:00','1979-04-10 00:00:00',92.6,391,413,32.05,'Intermediate' from dual union all
select 28990,'028990','1988-005E','deb Meteor-2','',61254,'In Earth orbit','','2006-03-28 00:00:00',103.82,925,954,82.53,'Intermediate' from dual union all
select 28991,'028991','1988-005F','deb Meteor-2','',61254,'In Earth orbit','','2006-03-29 00:00:00',103.77,931,942,82.53,'Intermediate' from dual union all
select 28992,'028992','1988-005G','deb Meteor-2','',61254,'In Earth orbit','','2006-03-28 00:00:00',104.08,942,961,82.54,'Intermediate' from dual union all
select 28993,'028993','1988-005H','deb Meteor-2','',61254,'In Earth orbit','','2006-03-29 00:00:00',103.99,937,958,82.53,'Intermediate' from dual union all
select 28994,'028994','1988-005J','deb Meteor-2','',61254,'In Earth orbit','','2006-03-28 00:00:00',103.99,935,960,82.55,'Intermediate' from dual union all
select 28995,'028995','1988-005K','deb Meteor-2','',61254,'In Earth orbit','','2006-03-28 00:00:00',103.82,923,956,82.54,'Intermediate' from dual union all
select 28996,'028996','2006-009A','Soyuz TMA-8','Soyuz 11F732 No. 218',68181,'Landed','2006-09-29 00:00:00','2006-03-31 00:00:00',90.26,267,308,51.64,'Intermediate' from dual union all
select 28997,'028997','2006-009B','11S510','11S510',68181,'Reentered','2006-04-02 00:00:00','2006-03-31 00:00:00',88.59,189,221,51.64,'Intermediate' from dual union all
select 28998,'028998','2003-024C','Briz-M No. 88505','Briz-M 14S43 No. 88505',67648,'In Earth orbit','','2006-03-30 00:00:00',789.43,7673,36160,17.16,'Highly Ellip' from dual union all
select 28999,'028999','1980-018D','deb Ayame 2','',50228,'In Earth orbit','','2006-03-30 00:00:00',819.18,9918,35328,8.1,'Medium' from dual union all
select 29,'000029','60 Beta 2','Tiros 1','A-1',6318,'In Earth orbit','','1960-05-05 00:00:00',99.23,690,754,48.41,'Intermediate' from dual union all
select 290,'000290','62 Rho 1','FTV 1125','CORONA M-3',8201,'Reentered Attached','1962-05-26 00:00:00','1962-05-10 00:00:00',90.56,190,414,73.11,'Intermediate' from dual union all
select 2900,'002900','1967-011M','deb Diademe 1','',15266,'Reentered','1980-05-23 00:00:00','1967-03-30 00:00:00',104.23,570,1346,39.96,'Intermediate' from dual union all
select 29000,'029000','1979-007C','SCATHA AIM','FW-5',47491,'In Earth orbit','','2006-03-28 00:00:00',1417.48,27900,42943,17.1,'Near-synch' from dual union all
select 29001,'029001','1979-101D','deb Satcom 3','',47585,'In Earth orbit','','2006-03-29 00:00:00',788.01,8066,35699,9.83,'Medium' from dual union all
select 29002,'029002','1979-101E','deb Satcom 3','',47585,'In Earth orbit','','2006-03-30 00:00:00',791.9,8224,35727,9.35,'Medium' from dual union all
select 29003,'029003','1979-101F','deb Satcom 3','',47585,'In Earth orbit','','2006-03-20 00:00:00',785.46,7975,35668,9.61,'Medium' from dual union all
select 29004,'029004','1979-101G','deb Satcom 3','',47585,'In Earth orbit','','2006-03-28 00:00:00',793.35,8190,35830,9.29,'Medium' from dual union all
select 29005,'029005','59 Eta 2','deb Vanguard 3','',5529,'Reentered','2015-02-04 00:00:00','2006-04-03 00:00:00',124.84,500,3292,33.35,'Medium' from dual union all
select 29006,'029006','62 A Psi 6','deb Tiros VI','',8152,'In Earth orbit','','2006-04-02 00:00:00',97.39,624,644,58.31,'Intermediate' from dual union all
select 29007,'029007','1964-049V','deb Kosmos-41','',10335,'Reentered','2006-09-25 00:00:00','2006-03-26 00:00:00',726.9,365,40439,64.37,'GEO Transfer' from dual union all
select 29008,'029008','1964-038G','deb Elektron-3','',10324,'Lost','','2005-09-23 00:00:00',156.65,407,6089,60.81,'Medium' from dual union all
select 29009,'029009','1965-063P','deb FW4S','',11813,'In Earth orbit','','2005-08-02 00:00:00',121.09,1076,2384,69.19,'Medium' from dual union all
select 2901,'002901','1967-072D','OV1-12','OV1-12S',15327,'Reentered','1980-07-22 00:00:00','1967-09-02 00:00:00',95.57,538,555,101.61,'Retrograde' from dual union all
select 29010,'029010','1965-089H','deb Explorer 29','',11839,'In Earth orbit','','2006-03-28 00:00:00',119.36,1086,2219,59.34,'Intermediate' from dual union all
select 29011,'029011','1965-108CT','deb Transtage 8','',11858,'In Earth orbit','','2006-04-03 00:00:00',417.21,602,23633,27.55,'Highly Ellip' from dual union all
select 29012,'029012','1965-108CU','deb Transtage 8','',11858,'In Earth orbit','','2006-04-02 00:00:00',507.8,604,28808,26.93,'Highly Ellip' from dual union all
select 29013,'029013','1965-108CV','deb Transtage 8','',11858,'In Earth orbit','','2006-04-01 00:00:00',535.74,638,30307,26.25,'Highly Ellip' from dual union all
select 29014,'029014','1977-092K','Ekran 2 debris','',42220,'In Earth orbit','','2006-02-14 00:00:00',1437.8,35736,35904,14.58,'Inclined Drift' from dual union all
select 29015,'029015','1979-104X','deb Ariane L01','',47588,'Reentered','2013-07-07 00:00:00','2006-04-04 00:00:00',126.31,383,3538,18.4,'Medium' from dual union all
select 29016,'029016','1979-104Y','deb Ariane L01','',47588,'Reentered','2014-03-10 00:00:00','2006-04-05 00:00:00',329.6,527,18329,19.05,'Highly Ellip' from dual union all
select 29017,'029017','1979-104Z','deb Ariane L01','',47588,'In Earth orbit','','2005-07-02 00:00:00',736.49,996,40278,16.32,'GEO Transfer' from dual union all
select 29018,'029018','1979-104AA','deb Ariane L01','',47588,'In Earth orbit','','2005-10-27 00:00:00',691.89,959,38111,18.57,'GEO Transfer' from dual union all
select 29019,'029019','1980-034F','deb US-A coolant','',50244,'In Earth orbit','','2006-03-21 00:00:00',102.93,859,936,64.83,'Intermediate' from dual union all
select 2902,'002902','1967-074A','Kosmos-170','OGCh No. 11',15329,'Deorbited','1967-07-31 00:00:00','1967-07-31 00:00:00',87.93,135,209,49.68,'Intermediate' from dual union all
select 29020,'029020','1981-021V','deb US-A coolant','',52544,'In Earth orbit','','2006-02-03 00:00:00',103.53,893,958,64.96,'Intermediate' from dual union all
select 29021,'029021','1984-069AL','deb US-A coolant','',57123,'In Earth orbit','','2006-04-05 00:00:00',103.64,903,959,65.05,'Intermediate' from dual union all
select 29022,'029022','1989-006AN','deb Ariane V28','',62283,'In Earth orbit','','2006-04-05 00:00:00',640.15,827,35626,7.72,'GEO Transfer' from dual union all
select 29023,'029023','1989-006AP','deb Ariane V28','',62283,'In Earth orbit','','2006-04-05 00:00:00',641.09,799,35702,7.76,'GEO Transfer' from dual union all
select 29024,'029024','1991-003Q','deb Ariane V41','',64172,'In Earth orbit','','2006-04-05 00:00:00',500.19,540,28450,7.35,'Highly Ellip' from dual union all
select 29025,'029025','1991-010J','deb SOZ','',64179,'Reentered','2008-11-22 00:00:00','2006-04-06 00:00:00',609.41,257,34607,46.5,'GEO Transfer' from dual union all
select 29026,'029026','1992-021S','deb Ariane V50','',64733,'In Earth orbit','','2005-10-01 00:00:00',644.06,1150,35503,3.57,'GEO Transfer' from dual union all
select 29027,'029027','1992-047AP','deb SOZ','',64759,'In Earth orbit','','2006-04-10 00:00:00',275.13,877,14391,64.7,'Medium' from dual union all
select 29028,'029028','1992-047AQ','deb SOZ','',64759,'Reentered','2013-09-10 00:00:00','2006-04-11 00:00:00',196.05,332,9270,64.33,'Medium' from dual union all
select 29029,'029029','1993-014AW','deb Start','',65116,'In Earth orbit','','2006-03-15 00:00:00',100.55,677,893,75.76,'Intermediate' from dual union all
select 2903,'002903','1967-074B','8S692','8S692',15329,'Reentered','1967-08-01 00:00:00','1967-07-31 00:00:00',87.24,135,141,49.39,'Intermediate' from dual union all
select 29030,'029030','1993-014AX','deb Start','',65116,'In Earth orbit','','2006-03-24 00:00:00',98.49,633,740,75.76,'Intermediate' from dual union all
select 29031,'029031','1993-014AY','deb Start','',65116,'Reentered','2009-04-09 00:00:00','2005-10-24 00:00:00',97.7,605,692,75.74,'Intermediate' from dual union all
select 29032,'029032','1993-014AZ','deb Start','',65116,'In Earth orbit','','2005-07-17 00:00:00',100.5,685,880,75.78,'Intermediate' from dual union all
select 29033,'029033','1994-029AEQ','deb HAPS','',65425,'In Earth orbit','','2006-04-07 00:00:00',101.75,593,1090,81.98,'Intermediate' from dual union all
select 29034,'029034','1979-095L','deb Meteor-2','',47579,'Reentered','2015-05-19 00:00:00','2006-04-07 00:00:00',99.9,742,765,81.2,'Intermediate' from dual union all
select 29035,'029035','1979-095M','deb Meteor-2','',47579,'Reentered','2014-01-18 00:00:00','2006-04-09 00:00:00',99.16,707,729,81.21,'Intermediate' from dual union all
select 29036,'029036','1979-095N','deb Meteor-2','',47579,'In Earth orbit','','2006-04-10 00:00:00',99.93,735,775,81.19,'Intermediate' from dual union all
select 29037,'029037','1999-040G','IUS-27 SRM-1 motor slag','',66721,'In Earth orbit','','2006-04-11 00:00:00',242.53,276,12732,28.45,'Medium' from dual union all
select 29038,'029038','1999-057NH','CZ-4 debris','',66738,'Reentered','2014-09-12 00:00:00','2006-04-09 00:00:00',98.05,635,696,98.85,'Sun-Synch' from dual union all
select 29039,'029039','1999-057NJ','CZ-4 debris','',66738,'In Earth orbit','','2006-04-11 00:00:00',99.6,685,793,98.22,'Sun-Synch' from dual union all
select 2904,'002904','1967-074C','Kosmos-170 Adapter','OGCh Adapter',15329,'Reentered','1967-08-01 00:00:00','1967-08-01 00:00:00',87.64,119,197,49.46,'Intermediate' from dual union all
select 29040,'029040','1999-057NK','CZ-4 debris','',66738,'In Earth orbit','','2006-04-10 00:00:00',100.04,699,822,98.19,'Sun-Synch' from dual union all
select 29041,'029041','2000-017F','Star 37 deb','',66919,'In Earth orbit','','2006-03-31 00:00:00',852.49,1692,45116,89.93,'Highly Ellip' from dual union all
select 29042,'029042','2004-035J','deb CZ-4B','',67879,'In Earth orbit','','2006-04-10 00:00:00',96.54,587,599,97.73,'Sun-Synch' from dual union all
select 29043,'029043','2004-035K','deb CZ-4B','',67879,'In Earth orbit','','2006-04-07 00:00:00',96.56,588,600,97.72,'Sun-Synch' from dual union all
select 29044,'029044','2005-031L','deb OICETS','',68040,'Reentered','2014-08-03 00:00:00','2006-04-11 00:00:00',96.67,591,607,97.85,'Sun-Synch' from dual union all
select 29045,'029045','2006-010A','JCSAT 5A','JCSAT 9',68182,'In Earth orbit','','2006-06-13 00:00:00',1436.09,35784,35788,.03,'Stationary' from dual union all
select 29046,'029046','2006-010B','Blok DM-SL No. 22L','Blok DM-SL',68182,'In Earth orbit','','2006-06-13 00:00:00',656.26,1661,35614,.18,'GEO Transfer' from dual union all
select 29047,'029047','2006-011A','FORMOSAT 3 FM6','FORMOSAT 3 FM6',68183,'In Earth orbit','','2006-04-17 00:00:00',94.94,495,536,72.02,'Intermediate' from dual union all
select 29048,'029048','2006-011B','FORMOSAT 3 FM1','FORMOSAT 3 FM1',68183,'In Earth orbit','','2006-04-17 00:00:00',94.97,495,540,72.02,'Intermediate' from dual union all
select 29049,'029049','2006-011C','FORMOSAT 3 FM5','FORMOSAT 3 FM5',68183,'In Earth orbit','','2006-04-17 00:00:00',94.97,496,539,72.02,'Intermediate' from dual union all
select 2905,'002905','1967-074D','deb R-36','',15329,'Reentered','1967-08-02 00:00:00','1967-07-31 00:00:00',89.89,142,395,49.59,'Intermediate' from dual union all
select 29050,'029050','2006-011D','FORMOSAT 3 FM3','FORMOSAT 3 FM3',68183,'In Earth orbit','','2006-04-17 00:00:00',94.98,496,540,72.02,'Intermediate' from dual union all
select 29051,'029051','2006-011E','FORMOSAT 3 FM4','FORMOSAT 3 FM4',68183,'In Earth orbit','','2006-04-17 00:00:00',94.97,495,540,72.02,'Intermediate' from dual union all
select 29052,'029052','2006-011F','FORMOSAT 3 FM2','FORMOSAT 3 FM2',68183,'In Earth orbit','','2006-04-17 00:00:00',94.97,495,540,72.02,'Intermediate' from dual union all
select 29053,'029053','2006-011G','Minotaur Stage 4','Orion 38',68183,'Reentered','2018-02-03 00:00:00','2006-04-17 00:00:00',94.97,495,540,72.02,'Intermediate' from dual union all
select 29054,'029054','2006-005E','ASTRO-F optics cover','',68177,'In Earth orbit','','2006-04-17 00:00:00',98.91,699,715,98.2,'Sun-Synch' from dual union all
select 29055,'029055','2006-012A','Astra 1KR','Astra 1KR',68184,'In Earth orbit','','2006-07-10 00:00:00',1436.09,35773,35799,.08,'Stationary' from dual union all
select 29056,'029056','2006-012B','Centaur AV-008','Centaur AV-008',68184,'In Earth orbit','','2006-06-20 00:00:00',733.24,6079,35036,24.63,'GEO Transfer' from dual union all
select 29057,'029057','2006-013A','Progress M-56','Progress 7K-TGM No. 356',68185,'Deorbited','2006-09-19 00:00:00','2006-04-25 00:00:00',90.28,246,331,51.64,'Intermediate' from dual union all
select 29058,'029058','2006-013B','11S510','11S510',68185,'Reentered','2006-04-26 00:00:00','2006-04-25 00:00:00',88.38,183,206,51.62,'Intermediate' from dual union all
select 29059,'029059','1965-027BJ','deb SNAPSHOT','',11777,'In Earth orbit','','2006-04-23 00:00:00',111.42,1266,1318,90.15,'Polar' from dual union all
select 2906,'002906','1967-074E','deb R-36','',15329,'Reentered','1967-08-01 00:00:00','1967-07-31 00:00:00',87.23,119,156,49.46,'Intermediate' from dual union all
select 29060,'029060','1965-027BK','deb SNAPSHOT','',11777,'In Earth orbit','','2006-04-25 00:00:00',111.35,1265,1312,90.16,'Polar' from dual union all
select 29061,'029061','1965-027BL','deb SNAPSHOT','',11777,'In Earth orbit','','2006-04-24 00:00:00',111.35,1265,1312,90.17,'Polar' from dual union all
select 29062,'029062','1968-055V','deb Star 37D','',16782,'In Earth orbit','','2005-10-14 00:00:00',130.69,663,3643,120.72,'Medium' from dual union all
select 29063,'029063','1973-088G','deb USAF satellite','',30690,'Reentered','2014-10-30 00:00:00','2006-04-22 00:00:00',101.46,581,1075,96.98,'Sun-Synch' from dual union all
select 29064,'029064','1973-088H','deb USAF satellite','',30690,'In Earth orbit','','2006-04-15 00:00:00',101.53,630,1033,96.92,'Sun-Synch' from dual union all
select 29065,'029065','1978-100AX','deb S5M','',45295,'In Earth orbit','','2006-04-21 00:00:00',120.15,1672,1704,82.54,'Medium' from dual union all
select 29066,'029066','1988-005L','deb Meteor-2','',61254,'In Earth orbit','','2006-04-18 00:00:00',103.76,919,954,82.55,'Intermediate' from dual union all
select 29067,'029067','1988-005M','deb Meteor-2','',61254,'In Earth orbit','','2006-04-23 00:00:00',103.83,923,957,82.53,'Intermediate' from dual union all
select 29068,'029068','1991-082BP','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-25 00:00:00',102.5,811,943,98.81,'Sun-Synch' from dual union all
select 29069,'029069','1991-082BQ','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-25 00:00:00',102.6,807,957,98.69,'Sun-Synch' from dual union all
select 2907,'002907','1967-075A','Lunar Orbiter 5','Lunar Orbiter E',15330,'Deep Space','1967-08-04 00:00:00','1967-08-01 00:00:00',13783.61,187,367917,31.3,'Cislunar' from dual union all
select 29070,'029070','1991-082BR','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-23 00:00:00',101.67,823,853,98.62,'Sun-Synch' from dual union all
select 29071,'029071','1991-082BS','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-14 00:00:00',102.84,815,971,98.78,'Sun-Synch' from dual union all
select 29072,'029072','1991-082BT','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-24 00:00:00',102.73,809,967,98.81,'Sun-Synch' from dual union all
select 29073,'029073','1991-082BU','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-25 00:00:00',101.63,822,850,98.68,'Sun-Synch' from dual union all
select 29074,'029074','1991-082BV','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-25 00:00:00',101.66,826,848,98.62,'Sun-Synch' from dual union all
select 29075,'029075','1991-082BW','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-24 00:00:00',101.38,779,869,98.64,'Sun-Synch' from dual union all
select 29076,'029076','1991-082BX','deb DMSP F-11','',64251,'Reentered','2016-01-01 00:00:00','2006-04-22 00:00:00',101.76,797,887,98.6,'Sun-Synch' from dual union all
select 29077,'029077','1991-082BY','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-25 00:00:00',103.37,825,1011,98.64,'Sun-Synch' from dual union all
select 29078,'029078','1991-082BZ','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-24 00:00:00',101.81,825,864,98.61,'Sun-Synch' from dual;
insert into SATELLITE
select 29079,'029079','2006-014A','EROS B','EROS B',68186,'In Earth orbit','','2006-04-25 00:00:00',94.78,503,512,97.32,'Sun-Synch' from dual union all
select 2908,'002908','1967-075B','Agena D 6634','Agena D 6634',15330,'Deep Space','1967-08-04 00:00:00','1967-08-01 00:00:00',13783.61,187,367917,31.3,'Cislunar' from dual union all
select 29080,'029080','2006-014B','Start','Start',68186,'Reentered','2017-09-03 00:00:00','2006-04-25 00:00:00',94.81,503,516,97.32,'Sun-Synch' from dual union all
select 29081,'029081','1991-082CA','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-24 00:00:00',102.64,802,966,98.77,'Sun-Synch' from dual union all
select 29082,'029082','1991-082CB','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-24 00:00:00',101.5,819,840,98.64,'Sun-Synch' from dual union all
select 29083,'029083','1991-082CC','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-24 00:00:00',101.57,822,844,98.64,'Sun-Synch' from dual union all
select 29084,'029084','1991-082CD','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-25 00:00:00',101.42,814,838,98.65,'Sun-Synch' from dual union all
select 29085,'029085','1991-082CE','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-17 00:00:00',101.67,828,847,98.71,'Sun-Synch' from dual union all
select 29086,'029086','1991-082CF','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-25 00:00:00',101.47,818,838,98.65,'Sun-Synch' from dual union all
select 29087,'029087','1991-082CG','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-25 00:00:00',101.53,822,840,98.58,'Sun-Synch' from dual union all
select 29088,'029088','1991-082CH','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-23 00:00:00',101.62,824,847,98.65,'Sun-Synch' from dual union all
select 29089,'029089','1991-082CJ','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-09 00:00:00',101.48,818,840,98.66,'Sun-Synch' from dual union all
select 2909,'002909','1967-053J','NRL PL150B','NRL PL150B',15308,'In Earth orbit','','1967-08-03 00:00:00',103.41,914,925,69.98,'Intermediate' from dual union all
select 29090,'029090','1991-082CK','deb DMSP F-11','',64251,'In Earth orbit','','2006-04-23 00:00:00',101.7,821,857,98.57,'Sun-Synch' from dual union all
select 29091,'029091','1991-082CL','deb DMSP F-11','',64251,'In Earth orbit','','2006-03-19 00:00:00',102.63,804,963,98.69,'Sun-Synch' from dual union all
select 29092,'029092','2006-015A','Yaogan Weixing 1','Jianbing-5 01 xing',68187,'In Earth orbit','','2006-04-29 00:00:00',97.02,606,626,97.8,'Sun-Synch' from dual union all
select 29093,'029093','2006-015B','CZ-4B Stage 3','CZ-4B Stage 3',68187,'In Earth orbit','','2006-04-29 00:00:00',95.39,451,623,97.85,'Sun-Synch' from dual union all
select 29094,'029094','1992-093JQ','deb 11S772','',64805,'In Earth orbit','','2006-04-27 00:00:00',102.72,760,1015,71.29,'Intermediate' from dual union all
select 29095,'029095','1993-014BA','deb Start','',65116,'Reentered','2012-11-19 00:00:00','2006-04-26 00:00:00',100.31,674,873,75.76,'Intermediate' from dual union all
select 29096,'029096','1993-016AK','deb 11S772','',65118,'In Earth orbit','','2006-04-21 00:00:00',100.77,715,876,70.94,'Intermediate' from dual union all
select 29097,'029097','1993-016AL','deb 11S772','',65118,'In Earth orbit','','2006-04-24 00:00:00',103.36,823,1012,71,'Intermediate' from dual union all
select 29098,'029098','1993-016AM','deb 11S772','',65118,'In Earth orbit','','2006-04-27 00:00:00',102.31,809,927,70.9,'Intermediate' from dual union all
select 29099,'029099','1993-016AN','deb 11S772','',65118,'Reentered','2016-03-04 00:00:00','2006-04-24 00:00:00',100.27,707,836,70.93,'Intermediate' from dual union all
select 291,'000291','62 Rho 2','CORONA camera door','',8201,'Reentered','1962-05-01 00:00:00','1962-05-10 00:00:00',90.56,190,414,73.11,'Intermediate' from dual union all
select 2910,'002910','1967-076A','OPS 4827','CORONA J-42',15331,'Reentered Attached','1967-09-01 00:00:00','1967-08-16 00:00:00',89.8,187,343,79.99,'Intermediate' from dual union all
select 29100,'029100','1993-016AP','deb 11S772','',65118,'In Earth orbit','','2006-04-26 00:00:00',102.55,822,936,70.73,'Intermediate' from dual union all
select 29101,'029101','1993-016AQ','deb 11S772','',65118,'In Earth orbit','','2006-04-26 00:00:00',102.27,748,984,70.94,'Intermediate' from dual union all
select 29102,'029102','1993-016AR','deb 11S772','',65118,'In Earth orbit','','2006-04-26 00:00:00',102.58,769,992,70.95,'Intermediate' from dual union all
select 29103,'029103','1995-058L','Zenit part','',65719,'Reentered','2013-09-13 00:00:00','2006-04-26 00:00:00',101.6,824,845,71.03,'Intermediate' from dual union all
select 29104,'029104','2001-049PN','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2006-04-23 00:00:00',97,585,645,97.81,'Sun-Synch' from dual union all
select 29105,'029105','2004-021G','deb 11S772','',67865,'In Earth orbit','','2006-04-26 00:00:00',105.42,837,1192,70.93,'Intermediate' from dual union all
select 29106,'029106','2005-049E','SEVIRI Cooler Cover','',68058,'In Earth orbit','','2006-04-13 00:00:00',1423.33,35511,35562,1.62,'Drift GEO' from dual union all
select 29107,'029107','2006-016A','Cloudsat','Cloudsat',68188,'In Earth orbit','','2006-04-30 00:00:00',98.54,689,689,98.24,'Sun-Synch' from dual union all
select 29108,'029108','2006-016B','Calipso','Calipso',68188,'In Earth orbit','','2006-04-30 00:00:00',98.52,687,689,98.24,'Sun-Synch' from dual union all
select 29109,'029109','2006-016C','Delta 314','Delta SSPS AJ-10-118K',68188,'Reentered','2006-07-06 00:00:00','2006-04-30 00:00:00',92.91,206,629,102.75,'Retrograde' from dual union all
select 2911,'002911','1967-077A','Kosmos-171','OGCh No. 12',15332,'Deorbited','1967-08-08 00:00:00','1967-08-08 00:00:00',87.02,120,133,49.65,'Intermediate' from dual union all
select 29110,'029110','2006-016D','Delta DPAF','DPAF',68188,'In Earth orbit','','2006-04-30 00:00:00',98.47,683,687,98.24,'Sun-Synch' from dual union all
select 29111,'029111','2006-017A','Kosmos-2420','Kobal''t-M No. 557',68189,'Landed','2006-07-19 00:00:00','2006-05-04 00:00:00',89.49,167,332,67.15,'Intermediate' from dual union all
select 29112,'029112','2006-017B','11S510','11S510',68189,'Reentered','2006-05-08 00:00:00','2006-05-04 00:00:00',89.17,163,304,67.15,'Intermediate' from dual union all
select 29113,'029113','2006-017C','deb Kosmos-2420','',68189,'Reentered','2006-05-06 00:00:00','2006-05-05 00:00:00',88.46,145,252,67.15,'Intermediate' from dual union all
select 29114,'029114','1985-108C','deb S5M','',58331,'Reentered','2006-05-23 00:00:00','2006-05-08 00:00:00',96.41,542,632,82.67,'Intermediate' from dual union all
select 29115,'029115','1985-108D','deb S5M','',58331,'Reentered','2006-05-23 00:00:00','2006-05-08 00:00:00',96.26,535,623,82.68,'Intermediate' from dual union all
select 29116,'029116','1985-108E','deb S5M','',58331,'Reentered','2006-05-22 00:00:00','2006-05-08 00:00:00',96.44,549,628,82.67,'Intermediate' from dual union all
select 29117,'029117','1985-108F','deb S5M','',58331,'Reentered','2006-08-18 00:00:00','2006-05-08 00:00:00',97.37,615,651,82.52,'Intermediate' from dual union all
select 29118,'029118','1985-108G','deb S5M','',58331,'Reentered','2006-07-10 00:00:00','2006-05-08 00:00:00',97.07,575,662,82.62,'Intermediate' from dual union all
select 29119,'029119','1985-108H','deb S5M','',58331,'Reentered','2006-07-15 00:00:00','2006-05-08 00:00:00',97.18,574,674,82.64,'Intermediate' from dual union all
select 2912,'002912','1967-077B','Kosmos-171 Adapter','OGCh Adapter',15332,'Reentered','1967-08-09 00:00:00','1967-08-08 00:00:00',87.5,119,183,49.6,'Intermediate' from dual union all
select 29120,'029120','1985-108J','deb S5M','',58331,'Reentered','2006-06-18 00:00:00','2006-05-08 00:00:00',97.08,569,669,82.66,'Intermediate' from dual union all
select 29121,'029121','1985-108K','deb S5M','',58331,'Reentered','2012-07-11 00:00:00','2006-05-07 00:00:00',96.96,604,623,82.55,'Intermediate' from dual union all
select 29122,'029122','1985-108L','deb S5M','',58331,'Reentered','2006-07-30 00:00:00','2006-05-08 00:00:00',97.09,578,661,82.64,'Intermediate' from dual union all
select 29123,'029123','1985-108M','deb S5M','',58331,'Reentered','2006-10-21 00:00:00','2006-05-08 00:00:00',97.29,615,643,82.54,'Intermediate' from dual union all
select 29124,'029124','1985-108N','deb S5M','',58331,'Reentered','2006-06-05 00:00:00','2006-05-08 00:00:00',96.97,570,658,82.67,'Intermediate' from dual union all
select 29125,'029125','1985-108P','deb S5M','',58331,'Reentered','2006-10-05 00:00:00','2006-05-08 00:00:00',97.33,617,645,82.52,'Intermediate' from dual union all
select 29126,'029126','1985-108Q','deb S5M','',58331,'In Earth orbit','','2006-05-08 00:00:00',97.41,618,652,82.58,'Intermediate' from dual union all
select 29127,'029127','1985-108R','deb S5M','',58331,'Reentered','2006-07-08 00:00:00','2006-05-08 00:00:00',97.42,603,668,82.52,'Intermediate' from dual union all
select 29128,'029128','1985-108S','deb S5M','',58331,'Reentered','2006-06-06 00:00:00','2006-05-08 00:00:00',97.04,602,631,82.53,'Intermediate' from dual union all
select 29129,'029129','1985-108T','deb S5M','',58331,'Reentered','2006-06-05 00:00:00','2006-05-08 00:00:00',96.99,574,656,82.64,'Intermediate' from dual union all
select 2913,'002913','1967-077C','deb R-36','',15332,'Reentered','1967-08-09 00:00:00','1967-08-09 00:00:00',89.04,127,328,49.57,'Intermediate' from dual union all
select 29130,'029130','1985-108U','deb S5M','',58331,'Reentered','2006-08-16 00:00:00','2006-05-08 00:00:00',97.45,612,662,82.55,'Intermediate' from dual union all
select 29131,'029131','1985-108V','deb S5M','',58331,'Reentered','2006-12-15 00:00:00','2006-05-08 00:00:00',96.2,505,649,82.73,'Intermediate' from dual union all
select 29132,'029132','1985-108W','deb S5M','',58331,'Reentered','2006-08-11 00:00:00','2006-05-07 00:00:00',97.35,615,649,82.5,'Intermediate' from dual union all
select 29133,'029133','1985-108X','deb S5M','',58331,'Reentered','2006-06-29 00:00:00','2006-05-08 00:00:00',97.54,603,679,82.51,'Intermediate' from dual union all
select 29134,'029134','1985-108Y','deb S5M','',58331,'Reentered','2006-06-12 00:00:00','2006-05-08 00:00:00',96.8,582,629,82.48,'Intermediate' from dual union all
select 29135,'029135','1985-108Z','deb S5M','',58331,'Reentered','2006-07-05 00:00:00','2006-05-07 00:00:00',97.11,592,648,82.55,'Intermediate' from dual union all
select 29136,'029136','1985-100C','deb Meteor-3','',58323,'In Earth orbit','','2006-05-01 00:00:00',109.17,1181,1196,82.54,'Intermediate' from dual union all
select 29137,'029137','1985-100D','deb Meteor-3','',58323,'In Earth orbit','','2006-04-20 00:00:00',109.16,1180,1196,82.54,'Intermediate' from dual union all
select 29138,'029138','1966-076J','NNS O-10 insulation','',13483,'In Earth orbit','','2006-05-09 00:00:00',106.4,1036,1084,88.86,'Polar' from dual union all
select 29139,'029139','1974-015F','Burner 2A collision debris','',33618,'In Earth orbit','','2006-05-08 00:00:00',101.03,759,856,99.14,'Sun-Synch' from dual union all
select 2914,'002914','1967-078A','Kosmos-172','Zenit-4 No. 34',15333,'Landed','1967-08-17 00:00:00','1967-08-13 00:00:00',89.36,200,286,51.79,'Intermediate' from dual union all
select 29140,'029140','1988-074N','NNS O-25 insulation','',61323,'In Earth orbit','','2006-05-08 00:00:00',105.16,960,1043,89.84,'Polar' from dual union all
select 29141,'029141','1985-108AA','deb S5M','',58331,'Reentered','2006-06-19 00:00:00','2006-05-15 00:00:00',96.54,583,603,82.5,'Intermediate' from dual union all
select 29142,'029142','1985-108AB','deb S5M','',58331,'Reentered','2006-07-07 00:00:00','2006-05-15 00:00:00',96.81,577,635,82.66,'Intermediate' from dual union all
select 29143,'029143','1985-108AC','deb S5M','',58331,'Reentered','2006-06-22 00:00:00','2006-05-15 00:00:00',96.13,554,593,82.52,'Intermediate' from dual union all
select 29144,'029144','1985-108AD','deb S5M','',58331,'Reentered','2006-09-03 00:00:00','2006-05-15 00:00:00',97.44,583,689,82.63,'Intermediate' from dual union all
select 29145,'029145','1985-108AE','deb S5M','',58331,'Reentered','2006-09-14 00:00:00','2006-05-15 00:00:00',97.25,609,646,82.45,'Intermediate' from dual union all
select 29146,'029146','1985-108AF','deb S5M','',58331,'Reentered','2006-06-05 00:00:00','2006-05-15 00:00:00',96.56,584,604,82.49,'Intermediate' from dual union all
select 29147,'029147','1985-108AG','deb S5M','',58331,'Reentered','2006-07-10 00:00:00','2006-05-15 00:00:00',97.07,609,628,82.54,'Intermediate' from dual union all
select 29148,'029148','1985-108AH','deb S5M','',58331,'Reentered','2006-06-13 00:00:00','2006-05-15 00:00:00',96.66,568,629,82.62,'Intermediate' from dual union all
select 29149,'029149','1985-108AJ','deb S5M','',58331,'Reentered','2006-08-16 00:00:00','2006-05-15 00:00:00',96.89,596,624,82.48,'Intermediate' from dual union all
select 2915,'002915','1967-077D','8S692','8S692',15332,'Reentered','1967-08-08 00:00:00','1967-08-08 00:00:00',89.09,139,320,49.5,'Intermediate' from dual union all
select 29150,'029150','1985-108AK','deb S5M','',58331,'Reentered','2006-07-05 00:00:00','2006-05-15 00:00:00',96.95,603,623,82.51,'Intermediate' from dual union all
select 29151,'029151','1985-108AL','deb S5M','',58331,'Reentered','2006-07-21 00:00:00','2006-05-15 00:00:00',97.04,584,650,82.62,'Intermediate' from dual union all
select 29152,'029152','1985-108AM','deb S5M','',58331,'Reentered','2006-07-30 00:00:00','2006-05-15 00:00:00',97.2,610,640,82.53,'Intermediate' from dual union all
select 29153,'029153','1985-108AN','deb S5M','',58331,'Reentered','2006-07-27 00:00:00','2006-05-15 00:00:00',96.97,607,621,82.48,'Intermediate' from dual union all
select 29154,'029154','1985-108AP','deb S5M','',58331,'Reentered','2006-06-28 00:00:00','2006-05-15 00:00:00',96.68,566,634,82.61,'Intermediate' from dual union all
select 29155,'029155','2006-018A','GOES 13','GOES N',68190,'In Earth orbit','','2006-07-26 00:00:00',1436.11,35784,35790,.41,'Stationary' from dual union all
select 29156,'029156','2006-018B','Delta 315','DCSS-4 F04',68190,'In Earth orbit','','2006-07-25 00:00:00',745.79,6587,35141,13.23,'GEO Transfer' from dual union all
select 29157,'029157','2006-019A','COMPASS-2','Vulkan-Kompas-2',68191,'Reentered','2011-12-28 00:00:00','2006-06-03 00:00:00',93.85,402,524,78.89,'Intermediate' from dual union all
select 29158,'029158','2006-019B','Shtil''-1 Payload Capsule','',68191,'Reentered','2009-05-10 00:00:00','2006-06-03 00:00:00',93.45,401,486,78.9,'Intermediate' from dual union all
select 29159,'029159','2006-019C','Shtil''-1 Nose Cone','Shtil''-1 instr. package',68191,'Reentered','2014-11-14 00:00:00','2006-06-03 00:00:00',93.51,401,491,78.9,'Intermediate' from dual union all
select 2916,'002916','1967-077E','deb R-36','',15332,'Reentered','1967-08-09 00:00:00','1967-08-08 00:00:00',88.97,132,316,49.2,'Intermediate' from dual union all
select 29160,'029160','2006-019D','Shtil''-1 Payload Plate','Adapter',68191,'Reentered','2006-10-15 00:00:00','2006-06-03 00:00:00',93.43,399,486,78.87,'Intermediate' from dual union all
select 29161,'029161','2006-019E','deb Shtil''-1','',68191,'Reentered','2007-03-21 00:00:00','2006-06-02 00:00:00',93.51,401,491,78.91,'Intermediate' from dual union all
select 29162,'029162','2006-020A','Satmex 6','Satmex 6',68192,'In Earth orbit','','2006-07-27 00:00:00',1436.14,35775,35800,.01,'Stationary' from dual union all
select 29163,'029163','2006-020B','Thaicom 5','Thaicom 5',68192,'In Earth orbit','','2006-07-26 00:00:00',1436.09,35777,35796,.02,'Stationary' from dual union all
select 29164,'029164','2006-020C','Sylda 5E','Sylda 5',68192,'In Earth orbit','','2006-07-30 00:00:00',628.42,251,35599,7.15,'GEO Transfer' from dual union all
select 29165,'029165','2006-020D','ESC-A 529','Ariane 5 ESC-A',68192,'In Earth orbit','','2006-07-28 00:00:00',628.36,248,35599,7.69,'GEO Transfer' from dual union all
select 29166,'029166','1965-063Q','deb FW4S','',11813,'In Earth orbit','','2006-05-30 00:00:00',120.36,1082,2312,69.21,'Medium' from dual union all
select 29167,'029167','1965-063R','deb FW4S','',11813,'In Earth orbit','','2005-07-02 00:00:00',121.39,1154,2332,69.2,'Medium' from dual union all
select 29168,'029168','1965-063S','deb FW4S','',11813,'In Earth orbit','','2006-05-31 00:00:00',122.04,1067,2477,69.24,'Medium' from dual;
insert into SATELLITE
select 29169,'029169','1965-063T','deb FW4S','',11813,'In Earth orbit','','2005-09-25 00:00:00',121.58,1129,2374,69.21,'Medium' from dual union all
select 2917,'002917','1967-078B','11A57I','11A57I',15333,'Reentered','1967-08-13 00:00:00','1967-08-11 00:00:00',88.85,183,253,51.77,'Intermediate' from dual union all
select 29170,'029170','1965-063U','deb FW4S','',11813,'In Earth orbit','','2005-07-17 00:00:00',121.29,1126,2351,69.24,'Medium' from dual union all
select 29171,'029171','1965-063V','deb FW4S','',11813,'In Earth orbit','','2006-05-28 00:00:00',121.52,1128,2370,69.21,'Medium' from dual union all
select 29172,'029172','1965-063W','deb FW4S','',11813,'In Earth orbit','','2006-05-06 00:00:00',122.01,1138,2404,69.22,'Medium' from dual union all
select 29173,'029173','1965-063X','deb FW4S','',11813,'In Earth orbit','','2005-08-14 00:00:00',120.89,1147,2295,69.23,'Medium' from dual union all
select 29174,'029174','1965-063Y','deb FW4S','',11813,'In Earth orbit','','2005-06-08 00:00:00',120.85,1135,2303,69.26,'Medium' from dual union all
select 29175,'029175','1965-063Z','deb FW4S','',11813,'In Earth orbit','','2006-05-23 00:00:00',120.56,1112,2300,69.27,'Medium' from dual union all
select 29176,'029176','1965-063AA','deb FW4S','',11813,'In Earth orbit','','2006-05-31 00:00:00',120.95,1170,2277,69.25,'Medium' from dual union all
select 29177,'029177','1965-063AB','deb FW4S','',11813,'In Earth orbit','','2006-05-02 00:00:00',121.35,1174,2309,69.21,'Medium' from dual union all
select 29178,'029178','1965-063AC','deb FW4S','',11813,'In Earth orbit','','2006-01-15 00:00:00',121.67,1141,2370,69.2,'Medium' from dual union all
select 29179,'029179','1965-063AD','deb FW4S','',11813,'In Earth orbit','','2005-08-31 00:00:00',121.7,1155,2359,69.19,'Medium' from dual union all
select 2918,'002918','1967-072E','deb OV1','',15327,'Reentered','1967-11-02 00:00:00','1967-08-26 00:00:00',95.19,473,582,101.74,'Retrograde' from dual union all
select 29180,'029180','1967-034J','NNS O-12 insulation','',15289,'In Earth orbit','','2006-05-30 00:00:00',106.13,1043,1052,90.16,'Polar' from dual union all
select 29181,'029181','1968-002E','deb FW4D','',16729,'In Earth orbit','','2006-05-13 00:00:00',111.66,1067,1538,105.82,'Retrograde' from dual union all
select 29182,'029182','1968-002F','deb FW4D','',16729,'In Earth orbit','','2006-05-20 00:00:00',109.76,1015,1417,105.82,'Retrograde' from dual union all
select 29183,'029183','1968-002G','deb FW4D','',16729,'In Earth orbit','','2006-05-30 00:00:00',109.09,978,1392,105.8,'Retrograde' from dual union all
select 29184,'029184','1968-002H','deb FW4D','',16729,'In Earth orbit','','2006-05-13 00:00:00',110.21,1024,1449,105.79,'Retrograde' from dual union all
select 29185,'029185','1968-002J','deb FW4D','',16729,'In Earth orbit','','2006-05-31 00:00:00',109.86,1018,1423,105.77,'Retrograde' from dual union all
select 29186,'029186','1968-002K','deb FW4D','',16729,'In Earth orbit','','2006-04-29 00:00:00',111.56,1062,1535,105.82,'Retrograde' from dual union all
select 29187,'029187','1968-002L','deb FW4D','',16729,'In Earth orbit','','2006-06-01 00:00:00',111.69,1086,1523,105.81,'Retrograde' from dual union all
select 29188,'029188','1968-002M','deb FW4D','',16729,'In Earth orbit','','2006-05-29 00:00:00',108.55,958,1361,105.83,'Retrograde' from dual union all
select 29189,'029189','1968-002N','deb FW4D','',16729,'In Earth orbit','','2006-05-10 00:00:00',110.27,1043,1436,105.81,'Retrograde' from dual union all
select 2919,'002919','1967-079A','OPS 4886','GAMBIT-3 Mission 4307',15334,'Reentered Attached','1967-08-29 00:00:00','1967-08-18 00:00:00',90.29,139,439,111.58,'Retrograde' from dual union all
select 29190,'029190','1968-002P','deb FW4D','',16729,'In Earth orbit','','2006-05-29 00:00:00',109.81,993,1443,105.84,'Retrograde' from dual union all
select 29191,'029191','1998-067AH','Yakor APFR adapter','EVA debris',66503,'Reentered','2006-12-01 00:00:00','2006-06-03 00:00:00',91.35,335,347,51.63,'Intermediate' from dual union all
select 29192,'029192','1971-120K','deb Meteor','',24938,'Reentered','2013-06-26 00:00:00','2006-06-05 00:00:00',99.43,712,751,81.23,'Intermediate' from dual union all
select 29193,'029193','1971-120L','deb Meteor','',24938,'Reentered','2014-06-26 00:00:00','2006-06-05 00:00:00',99.4,721,739,81.24,'Intermediate' from dual union all
select 29194,'029194','1976-103G','deb Kosmos-860 coolant','',39055,'In Earth orbit','','2006-06-05 00:00:00',103.38,873,964,64.69,'Intermediate' from dual union all
select 29195,'029195','1976-103H','deb Kosmos-860 coolant','',39055,'In Earth orbit','','2006-06-01 00:00:00',103.97,900,993,64.7,'Intermediate' from dual union all
select 29196,'029196','1976-103J','deb Kosmos-860 coolant','',39055,'In Earth orbit','','2006-06-04 00:00:00',103.96,899,993,64.7,'Intermediate' from dual union all
select 29197,'029197','1978-047C','SVS yo weight','',45242,'In Earth orbit','','2006-06-06 00:00:00',200.75,579,9379,64.17,'Medium' from dual union all
select 29198,'029198','1979-095P','deb Meteor-2','',47579,'In Earth orbit','','2006-06-06 00:00:00',101.45,820,835,81.21,'Intermediate' from dual union all
select 29199,'029199','1980-073E','deb Meteor-2','',50283,'In Earth orbit','','2006-05-09 00:00:00',102.08,825,889,81.22,'Intermediate' from dual union all
select 292,'000292','62 Sigma 1','FTV 1126','ARGON 5',8202,'Reentered Attached','1963-11-26 00:00:00','1962-06-20 00:00:00',93.75,284,632,82.3,'Intermediate' from dual union all
select 2920,'002920','1967-080A','OPS 7202','DAPP 3419',15335,'In Earth orbit','','1967-09-26 00:00:00',102.16,830,892,99,'Sun-Synch' from dual union all
select 29200,'029200','1980-073F','deb Meteor-2','',50283,'In Earth orbit','','2006-05-29 00:00:00',101.95,815,887,81.22,'Intermediate' from dual union all
select 29201,'029201','1980-073G','deb Meteor-2','',50283,'In Earth orbit','','2006-06-06 00:00:00',102,820,886,81.22,'Intermediate' from dual union all
select 29202,'029202','1981-031L','deb Kosmos-1261','',52554,'In Earth orbit','','2004-02-14 00:00:00',718.65,2220,38178,66.41,'GEO Transfer' from dual union all
select 29203,'029203','1981-037E','deb Kosmos-1266 coolant','',52560,'In Earth orbit','','2006-06-03 00:00:00',103.53,894,957,64.76,'Intermediate' from dual union all
select 29204,'029204','1982-099F','deb Kosmos-1412 coolant','',54408,'In Earth orbit','','2006-05-14 00:00:00',103.81,902,976,64.81,'Intermediate' from dual union all
select 29205,'029205','1985-108AQ','deb S5M','',58331,'Reentered','2006-07-19 00:00:00','2006-06-03 00:00:00',96.32,555,610,82.48,'Intermediate' from dual union all
select 29206,'029206','1985-108AR','deb S5M','',58331,'Reentered','2006-07-16 00:00:00','2006-06-04 00:00:00',96.35,563,604,82.47,'Intermediate' from dual union all
select 29207,'029207','1987-080K','NNS O-27 insulation','',60170,'In Earth orbit','','2006-05-29 00:00:00',105.36,890,1133,90.19,'Polar' from dual union all
select 29208,'029208','1987-080L','NNS O-27 insulation','',60170,'In Earth orbit','','2006-05-28 00:00:00',105.33,910,1109,90.23,'Polar' from dual union all
select 29209,'029209','1987-080M','NNS O-27 insulation','',60170,'In Earth orbit','','2006-06-04 00:00:00',103.63,834,1026,90.26,'Polar' from dual union all
select 2921,'002921','1967-081A','Kosmos-173','DS-P1-Yu No. 8',15336,'Reentered','1967-12-17 00:00:00','1967-09-23 00:00:00',91.86,261,471,71.04,'Intermediate' from dual union all
select 29210,'029210','1987-080N','NNS O-27 insulation','',60170,'In Earth orbit','','2006-06-02 00:00:00',105.2,902,1105,90.25,'Polar' from dual union all
select 29211,'029211','1987-080P','NNS O-27 insulation','',60170,'In Earth orbit','','2006-06-03 00:00:00',104.68,900,1059,90.28,'Polar' from dual union all
select 29212,'029212','1988-033H','NNS O-23 insulation','',61282,'In Earth orbit','','2006-01-05 00:00:00',107.28,986,1216,90.26,'Polar' from dual union all
select 29213,'029213','1999-057NL','CZ-4 debris','',66738,'In Earth orbit','','2006-06-05 00:00:00',97.86,603,710,98.68,'Sun-Synch' from dual union all
select 29214,'029214','1999-057NM','CZ-4 debris','',66738,'Reentered','2011-05-03 00:00:00','2006-06-06 00:00:00',97.19,581,668,98.72,'Sun-Synch' from dual union all
select 29215,'029215','2002-024F','CZ-4B sep motor cover','',67400,'In Earth orbit','','2006-06-05 00:00:00',105.82,847,1219,98.83,'Sun-Synch' from dual union all
select 29216,'029216','2002-024G','CZ-4B sep motor cover','',67400,'In Earth orbit','','2006-06-05 00:00:00',105.43,844,1186,98.88,'Sun-Synch' from dual union all
select 29217,'029217','2005-024H','deb CZ-2D motor cover','',68033,'In Earth orbit','','2006-06-05 00:00:00',98.82,555,850,97.55,'Sun-Synch' from dual union all
select 29218,'029218','1985-108AS','deb S5M','',58331,'Reentered','2006-06-27 00:00:00','2006-06-06 00:00:00',95.76,518,593,82.46,'Intermediate' from dual union all
select 29219,'029219','1985-108AT','deb S5M','',58331,'Reentered','2006-06-26 00:00:00','2006-06-04 00:00:00',96.1,526,618,82.58,'Intermediate' from dual union all
select 2922,'002922','1967-081B','S1M','S1M',15336,'Reentered','1967-10-30 00:00:00','1967-09-27 00:00:00',91.24,268,403,71.02,'Intermediate' from dual union all
select 29220,'029220','1985-108AU','deb S5M','',58331,'Reentered','2006-08-02 00:00:00','2006-06-01 00:00:00',96.99,577,653,82.6,'Intermediate' from dual union all
select 29221,'029221','1985-108AV','deb S5M','',58331,'Reentered','2006-07-11 00:00:00','2006-06-04 00:00:00',96.27,547,613,82.62,'Intermediate' from dual union all
select 29222,'029222','1985-108AW','deb S5M','',58331,'Reentered','2006-08-15 00:00:00','2006-06-04 00:00:00',96.82,587,625,82.37,'Intermediate' from dual union all
select 29223,'029223','1985-108AX','deb S5M','',58331,'Reentered','2006-07-05 00:00:00','2006-06-06 00:00:00',96.09,537,606,82.47,'Intermediate' from dual union all
select 29224,'029224','1985-108AY','deb S5M','',58331,'Reentered','2006-07-17 00:00:00','2006-06-03 00:00:00',96.53,561,623,82.65,'Intermediate' from dual union all
select 29225,'029225','1985-108AZ','deb S5M','',58331,'Reentered','2006-08-11 00:00:00','2006-06-03 00:00:00',96.78,587,622,82.48,'Intermediate' from dual union all
select 29226,'029226','1985-108BA','deb S5M','',58331,'Reentered','2006-07-06 00:00:00','2006-06-06 00:00:00',95.84,532,587,82.51,'Intermediate' from dual union all
select 29227,'029227','1985-108BB','deb S5M','',58331,'Reentered','2006-07-18 00:00:00','2006-06-04 00:00:00',96.4,539,633,82.59,'Intermediate' from dual union all
select 29228,'029228','2006-021A','Resurs-DK','Resurs-DK No. 1',68193,'In Earth orbit','','2006-06-17 00:00:00',89.81,191,340,69.94,'Intermediate' from dual union all
select 29229,'029229','2006-021B','Blok-I','11S510',68193,'Reentered','2006-06-28 00:00:00','2006-06-17 00:00:00',89.64,188,326,69.96,'Intermediate' from dual union all
select 2923,'002923','1967-076B','Fairing','Fairing',15331,'Reentered','1967-08-25 00:00:00','1967-08-23 00:00:00',88.65,165,251,79.94,'Intermediate' from dual union all
select 29230,'029230','2006-022A','Kazsat','Kazsat 78KS',68194,'In Earth orbit','','2006-08-17 00:00:00',1436.11,35780,35794,.04,'Stationary' from dual union all
select 29231,'029231','2006-022B','8S812','8S812',68194,'Reentered','2006-06-18 00:00:00','2006-06-18 00:00:00',88.38,180,209,51.56,'Intermediate' from dual union all
select 29232,'029232','2006-022C','Perekhodnik','Sredniy perekhodnik',68194,'Reentered','2006-06-20 00:00:00','2006-06-19 00:00:00',87.93,146,198,51.57,'Intermediate' from dual union all
select 29233,'029233','2006-022D','Blok DM3 No. 23L','Blok DM3 No. 23L',68194,'In Earth orbit','','2006-08-17 00:00:00',1425.58,35366,35795,.15,'Drift GEO' from dual union all
select 29234,'029234','2006-022E','deb Perekhodnik','',68194,'Reentered','2006-07-03 00:00:00','2006-06-20 00:00:00',91.37,203,481,51.55,'Intermediate' from dual union all
select 29235,'029235','2006-022F','deb Perekhodnik','',68194,'Reentered','2006-07-06 00:00:00','2006-06-20 00:00:00',91.66,219,493,51.52,'Intermediate' from dual union all
select 29236,'029236','2006-023A','Galaxy 16','Galaxy 16',68195,'In Earth orbit','','2006-08-18 00:00:00',1436.06,35777,35795,.03,'Stationary' from dual union all
select 29237,'029237','2006-023B','Blok DM-SL No. 21L','Blok DM-SL',68195,'In Earth orbit','','2006-08-17 00:00:00',667.01,2210,35610,.06,'GEO Transfer' from dual union all
select 29238,'029238','2006-022G','deb Perekhodnik','',68194,'Reentered','2006-07-06 00:00:00','2006-06-25 00:00:00',91.59,211,495,51.55,'Intermediate' from dual union all
select 29239,'029239','2006-022H','deb Perekhodnik','',68194,'Reentered','2006-07-07 00:00:00','2006-06-25 00:00:00',91.31,214,463,51.61,'Intermediate' from dual union all
select 2924,'002924','1967-046F','Blok-L No. Ya716-56','Blok-L',15301,'Reentered','1978-04-14 00:00:00','1967-08-06 00:00:00',1170.96,708,60136,52.39,'Highly Ellip' from dual union all
select 29240,'029240','2006-024A','USA 187','MITEX OSC satellite',68196,'In Earth orbit','','2006-06-24 00:00:00',1436.27,35780,35800,0,'Stationary' from dual union all
select 29241,'029241','2006-024B','USA 188','MITEX Lockheed satellite',68196,'In Earth orbit','','2006-06-24 00:00:00',1436.27,35780,35800,0,'Stationary' from dual union all
select 29242,'029242','2006-024C','USA 189','NRL POTV',68196,'In Earth orbit','','2008-01-24 00:00:00',1384.55,34758,34785,1.21,'Drift GEO' from dual union all
select 29243,'029243','2006-024D','Delta 316','Delta SSPS AJ-10-118K',68196,'In Earth orbit','','2006-06-21 00:00:00',109.46,202,2202,23.1,'Equatorial' from dual union all
select 29244,'029244','2006-024E','Star 48B','Star 48B',68196,'Reentered','2010-01-01 00:00:00','2006-06-21 00:00:00',639.25,184,36223,25.25,'GEO Transfer' from dual union all
select 29245,'029245','2006-025A','Progress M-57','Progress 7K-TGM No. 357',68197,'Deorbited','2007-01-17 00:00:00','2006-07-02 00:00:00',91.35,334,348,51.63,'Intermediate' from dual union all
select 29246,'029246','2006-025B','11S510','11S510',68197,'Reentered','2006-06-27 00:00:00','2006-06-25 00:00:00',88.24,178,196,51.63,'Intermediate' from dual union all
select 29247,'029247','2006-026A','Kosmos-2421','US-PU/Konus-A',68198,'Reentered','2010-08-19 00:00:00','2006-07-02 00:00:00',92.76,404,415,65.05,'Intermediate' from dual union all
select 29248,'029248','2006-026B','11S692','11S692',68198,'Reentered','2006-06-25 00:00:00','2006-06-25 00:00:00',87.93,92,252,65.07,'Intermediate' from dual union all
select 29249,'029249','2006-027A','USA 184','RAVEN 4/HEO-1',68199,'In Earth orbit','','2006-09-09 00:00:00',717.66,1139,39210,63.23,'Molniya' from dual union all
select 2925,'002925','1967-082A','Kosmos-174','Molniya-1Yu 11F67 No. 11',15337,'Reentered','1968-12-30 00:00:00','1967-10-17 00:00:00',714.84,428,39782,64.77,'GEO Transfer' from dual union all
select 29250,'029250','2006-027B','Delta 317','DCSS-4 F05',68199,'In Earth orbit','','2006-07-26 00:00:00',717.8,1167,39189,63.23,'Molniya' from dual union all
select 29251,'029251','2006-028A','Discovery (STS-121)','OV-103',68200,'Landed','2006-07-17 00:00:00','2006-07-06 00:00:00',91.35,334,348,51.63,'Intermediate' from dual union all
select 29252,'029252','2006-029A','Genesis-1','Bigelow Genesis-1',68201,'In Earth orbit','','2006-07-19 00:00:00',95.81,555,560,64.51,'Intermediate' from dual union all
select 29253,'029253','2006-029B','Dnepr plume shield','GDS',68201,'Reentered','2015-02-22 00:00:00','2006-07-19 00:00:00',95.53,533,556,64.51,'Intermediate' from dual union all
select 29254,'029254','2006-032B','Briz-M No. 88516','Briz-M',68204,'In Earth orbit','','2006-10-05 00:00:00',697.52,3799,35552,12.72,'GEO Transfer' from dual union all
select 29255,'029255','61 A Delta 9','West Ford package fragment','',7251,'In Earth orbit','','2006-07-27 00:00:00',165.29,2714,4484,95.84,'Medium' from dual union all
select 29256,'029256','2006-029C','R-36M2 DS','R-36M2 DS',68201,'In Earth orbit','','2006-07-19 00:00:00',108.03,540,1732,64.58,'Intermediate' from dual union all
select 29257,'029257','1998-067AJ','EVA debris (spatula)','',66503,'Reentered','2006-09-29 00:00:00','2006-07-12 00:00:00',91.35,334,348,51.63,'Intermediate' from dual union all
select 29258,'029258','2006-017D','Kosmos-2420 solar panel','Kobal''t-M part',68189,'Reentered','2006-07-20 00:00:00','2006-07-19 00:00:00',88.57,164,244,67.14,'Intermediate' from dual;
insert into SATELLITE
select 29259,'029259','2006-017E','Kosmos-2420 solar panel','Kobal''t-M part',68189,'Reentered','2006-07-20 00:00:00','2006-07-20 00:00:00',87.35,115,172,67.16,'Intermediate' from dual union all
select 2926,'002926','1967-082C','11S510','11S510',15337,'Reentered','1967-09-15 00:00:00','1967-09-08 00:00:00',90.2,192,377,64.81,'Intermediate' from dual union all
select 29260,'029260','2006-030A','Kosmos-2422','Oko US-KS',68202,'In Earth orbit','','2006-09-20 00:00:00',717.96,584,39780,62.9,'Molniya' from dual union all
select 29261,'029261','2006-030B','Blok-ML','Blok-ML No. 76798-496',68202,'In Earth orbit','','2006-09-20 00:00:00',700.74,584,38927,62.86,'Molniya' from dual union all
select 29262,'029262','2006-030C','11S510','11S510',68202,'Reentered','2006-09-30 00:00:00','2006-07-28 00:00:00',91.32,229,449,62.83,'Intermediate' from dual union all
select 29263,'029263','2006-030D','BOZ','BOZ',68202,'Reentered','2006-09-01 00:00:00','2006-07-28 00:00:00',91.54,194,506,62.85,'Intermediate' from dual union all
select 29264,'029264','2006-030E','BOZ ullage motor','Ullage motor',68202,'Reentered','2006-09-25 00:00:00','2006-07-28 00:00:00',92,223,523,62.87,'Intermediate' from dual union all
select 29265,'029265','1967-034K','NNS O-12 insulation','',15289,'In Earth orbit','','2006-08-14 00:00:00',105.64,988,1061,90.16,'Polar' from dual union all
select 29266,'029266','2006-030F','BOZ ullage motor','Ullage motor',68202,'Reentered','2006-09-02 00:00:00','2006-07-28 00:00:00',91.8,212,513,62.86,'Intermediate' from dual union all
select 29267,'029267','2006-030G','BOZ part','',68202,'Reentered','2006-09-13 00:00:00','2006-07-28 00:00:00',91.88,218,516,62.86,'Intermediate' from dual union all
select 29268,'029268','2006-031A','Arirang-2','KOMPSAT-2',68203,'In Earth orbit','','2006-08-04 00:00:00',98.56,677,703,98.14,'Sun-Synch' from dual union all
select 29269,'029269','2006-031B','Briz-KM No. 72504','Briz-KM 14S45',68203,'In Earth orbit','','2006-08-04 00:00:00',96.72,505,699,98.19,'Sun-Synch' from dual union all
select 2927,'002927','1967-082B','BOZ','BOZ',15337,'Reentered','1967-09-19 00:00:00','1967-09-07 00:00:00',90.67,193,422,64.82,'Intermediate' from dual union all
select 29270,'029270','2006-032A','Hot Bird 8','Hot Bird 8',68204,'In Earth orbit','','2006-10-06 00:00:00',1436.04,35762,35809,.07,'Stationary' from dual union all
select 29271,'029271','2006-032C','Briz-M DTB','Briz-M torus tank',68204,'In Earth orbit','','2006-10-05 00:00:00',297.18,301,16446,49.44,'Highly Ellip' from dual union all
select 29272,'029272','2006-033A','JCSAT 3A','JCSAT 10',68205,'In Earth orbit','','2006-10-12 00:00:00',1436.16,35783,35793,.01,'Stationary' from dual union all
select 29273,'029273','2006-033B','Syracuse 3B','Syracuse 3B',68205,'In Earth orbit','','2006-10-11 00:00:00',1436.06,35776,35796,.03,'Stationary' from dual union all
select 29274,'029274','2006-033C','ESC-A 531','Ariane 5 ESC-A',68205,'In Earth orbit','','2006-10-11 00:00:00',628.73,300,35566,5.65,'GEO Transfer' from dual union all
select 29275,'029275','2006-033D','Sylda 5A','Sylda 5',68205,'In Earth orbit','','2006-10-11 00:00:00',627.8,290,35528,6.21,'GEO Transfer' from dual union all
select 29276,'029276','1967-034L','NNS O-12 insulation','',15289,'In Earth orbit','','2006-08-15 00:00:00',106.11,1022,1071,90.19,'Polar' from dual union all
select 29277,'029277','1968-066L','deb Explorer 40','',16793,'In Earth orbit','','2006-08-11 00:00:00',104.91,620,1360,80.64,'Intermediate' from dual union all
select 29278,'029278','1968-066M','deb Explorer 40','',16793,'In Earth orbit','','2006-07-15 00:00:00',110.94,648,1892,80.64,'Intermediate' from dual union all
select 29279,'029279','1968-066N','deb Explorer 40','',16793,'In Earth orbit','','2006-08-08 00:00:00',108.91,655,1698,80.63,'Intermediate' from dual union all
select 2928,'002928','1967-082D','deb BOZ','',15337,'Reentered','1967-09-05 00:00:00','1967-09-01 00:00:00',90.27,198,378,64.84,'Intermediate' from dual union all
select 29280,'029280','1980-089CL','deb Kosmos-1220','[89L]',50299,'Reentered','2006-12-31 00:00:00','2006-08-15 00:00:00',94.49,447,540,64.97,'Intermediate' from dual union all
select 29281,'029281','1980-089CM','deb Kosmos-1220','[89M]',50299,'Reentered','2006-12-23 00:00:00','2006-08-14 00:00:00',94.58,452,545,64.98,'Intermediate' from dual union all
select 29282,'029282','1981-120L','deb S3M','',52643,'In Earth orbit','','2006-08-14 00:00:00',119.61,1649,1678,82.95,'Intermediate' from dual union all
select 29283,'029283','1981-120M','deb S3M','',52643,'In Earth orbit','','2006-08-13 00:00:00',119.22,1620,1672,82.95,'Intermediate' from dual union all
select 29284,'029284','1981-120N','deb S3M','',52643,'In Earth orbit','','2006-08-05 00:00:00',119.75,1645,1695,82.96,'Intermediate' from dual union all
select 29285,'029285','1981-120P','deb S3M','',52643,'In Earth orbit','','2006-08-12 00:00:00',118.67,1583,1660,82.96,'Intermediate' from dual union all
select 29286,'029286','1981-120Q','deb S3M','',52643,'In Earth orbit','','2006-08-14 00:00:00',119.41,1653,1656,82.97,'Intermediate' from dual union all
select 29287,'029287','1981-120R','deb S3M','',52643,'In Earth orbit','','2006-08-12 00:00:00',119.56,1645,1678,82.96,'Intermediate' from dual union all
select 29288,'029288','1981-120S','deb S3M','',52643,'In Earth orbit','','2006-08-13 00:00:00',119.34,1631,1672,82.93,'Intermediate' from dual union all
select 29289,'029289','1981-120T','deb S3M','',52643,'In Earth orbit','','2006-08-15 00:00:00',118.46,1567,1657,82.96,'Intermediate' from dual union all
select 2929,'002929','1965-082LB','deb Transtage 4','',11832,'Reentered','1979-04-23 00:00:00','1979-03-26 00:00:00',91.26,330,343,32.78,'Intermediate' from dual union all
select 29290,'029290','1981-120U','deb S3M','',52643,'In Earth orbit','','2006-08-15 00:00:00',118.96,1614,1655,82.96,'Intermediate' from dual union all
select 29291,'029291','1979-095Q','deb Meteor-2','',47579,'In Earth orbit','','2006-08-16 00:00:00',102.35,866,874,81.21,'Intermediate' from dual union all
select 29292,'029292','1979-095R','deb Meteor-2','',47579,'In Earth orbit','','2006-08-14 00:00:00',102.24,864,866,81.2,'Intermediate' from dual union all
select 29293,'029293','1979-095S','deb Meteor-2','',47579,'In Earth orbit','','2006-08-08 00:00:00',102.23,857,872,81.21,'Intermediate' from dual union all
select 29294,'029294','1971-028E','deb Kosmos-405','',24846,'Reentered','2007-10-28 00:00:00','2006-08-16 00:00:00',95.22,518,540,81.26,'Intermediate' from dual union all
select 29295,'029295','1981-053MS','deb Kosmos-1275','',52576,'In Earth orbit','','2006-07-28 00:00:00',105.43,938,1092,82.96,'Intermediate' from dual union all
select 29296,'029296','1981-071W','deb Kosmos-1285','',52594,'In Earth orbit','','2006-08-13 00:00:00',726.5,1367,39417,64.47,'GEO Transfer' from dual union all
select 29297,'029297','1994-074G','deb Resurs-O1','',65470,'Reentered','2011-11-11 00:00:00','2006-08-16 00:00:00',97.32,629,632,97.85,'Sun-Synch' from dual union all
select 29298,'029298','1994-074H','deb Resurs-O1','',65470,'Reentered','2011-12-05 00:00:00','2006-08-16 00:00:00',97.4,632,637,97.86,'Sun-Synch' from dual union all
select 29299,'029299','1988-005N','deb Meteor-2','',61254,'In Earth orbit','','2006-08-15 00:00:00',103.88,928,956,82.55,'Intermediate' from dual union all
select 293,'000293','62 Sigma 2','ARGON camera door','',8202,'Reentered','1962-07-03 00:00:00','1962-06-11 00:00:00',92.61,272,532,82.2,'Intermediate' from dual union all
select 2930,'002930','1963-049J','deb Transit VBN-2','',9280,'Reentered','1979-10-14 00:00:00','1967-07-31 00:00:00',107.1,1061,1124,89.94,'Polar' from dual union all
select 29300,'029300','1988-005P','deb Meteor-2','',61254,'In Earth orbit','','2006-07-26 00:00:00',103.98,934,960,82.55,'Intermediate' from dual union all
select 29301,'029301','1988-005Q','deb Meteor-2','',61254,'In Earth orbit','','2006-08-11 00:00:00',103.91,937,950,82.53,'Intermediate' from dual union all
select 29302,'029302','1988-005R','deb Meteor-2','',61254,'In Earth orbit','','2006-08-14 00:00:00',103.64,924,937,82.55,'Intermediate' from dual union all
select 29303,'029303','1989-039J','deb SOZ','',62316,'Reentered','2010-05-06 00:00:00','2006-08-17 00:00:00',335.09,821,18384,65.46,'Highly Ellip' from dual union all
select 29304,'029304','1989-039K','deb SOZ','',62316,'Reentered','2007-01-11 00:00:00','2006-08-17 00:00:00',335.16,886,18323,65.56,'Highly Ellip' from dual union all
select 29305,'029305','1989-039L','deb SOZ','',62316,'Reentered','2007-06-14 00:00:00','2006-08-17 00:00:00',331.59,824,18158,65.61,'Highly Ellip' from dual union all
select 29306,'029306','1989-039M','deb SOZ','',62316,'Reentered','2007-04-16 00:00:00','2006-08-17 00:00:00',324.6,864,17671,65.07,'Highly Ellip' from dual union all
select 29307,'029307','1989-039N','deb SOZ','',62316,'Reentered','2009-04-01 00:00:00','2006-08-17 00:00:00',325.78,832,17779,64.91,'Highly Ellip' from dual union all
select 29308,'029308','1989-039P','deb SOZ','',62316,'In Earth orbit','','2006-08-17 00:00:00',334.53,733,18436,65.42,'Highly Ellip' from dual union all
select 29309,'029309','1989-039Q','deb SOZ','',62316,'Reentered','2010-12-14 00:00:00','2006-08-17 00:00:00',331.98,864,18144,65.58,'Highly Ellip' from dual union all
select 2931,'002931','1966-101AS','OGCh Retro','OGCh',13508,'Reentered','1976-04-26 00:00:00','1967-07-10 00:00:00',97.42,504,767,49.13,'Intermediate' from dual union all
select 29310,'029310','1989-039R','deb SOZ','',62316,'Reentered','2012-03-13 00:00:00','2006-08-17 00:00:00',334.89,834,18358,65.51,'Highly Ellip' from dual union all
select 29311,'029311','1989-039S','deb SOZ','',62316,'In Earth orbit','','2006-08-17 00:00:00',332.52,845,18197,65.61,'Highly Ellip' from dual union all
select 29312,'029312','1989-039T','deb SOZ','',62316,'In Earth orbit','','2006-08-17 00:00:00',332.88,828,18237,65.54,'Highly Ellip' from dual union all
select 29313,'029313','1989-039U','deb SOZ','',62316,'Reentered','2006-12-30 00:00:00','2006-08-17 00:00:00',331.86,885,18115,65.65,'Highly Ellip' from dual union all
select 29314,'029314','1989-039V','deb SOZ','',62316,'Reentered','2008-05-27 00:00:00','2006-08-17 00:00:00',336.07,956,18312,64.7,'Highly Ellip' from dual union all
select 29315,'029315','1989-039W','deb SOZ','',62316,'Reentered','2006-12-23 00:00:00','2006-08-17 00:00:00',327.73,585,18150,64.91,'Highly Ellip' from dual union all
select 29316,'029316','1989-039X','deb SOZ','',62316,'Reentered','2006-12-06 00:00:00','2006-08-17 00:00:00',327.32,582,18128,64.95,'Highly Ellip' from dual union all
select 29317,'029317','1989-039Y','deb SOZ','',62316,'In Earth orbit','','2006-08-17 00:00:00',330.68,664,18261,65.05,'Highly Ellip' from dual union all
select 29318,'029318','1989-039Z','deb SOZ','',62316,'In Earth orbit','','2006-08-17 00:00:00',329.17,612,18216,65.05,'Highly Ellip' from dual union all
select 29319,'029319','1989-039AA','deb SOZ','',62316,'Reentered','2006-12-21 00:00:00','2006-08-17 00:00:00',327.32,537,18173,64.91,'Highly Ellip' from dual union all
select 2932,'002932','1965-082LC','deb Transtage 4','',11832,'Reentered','1990-04-26 00:00:00','1990-04-24 00:00:00',89.98,267,280,32.08,'Intermediate' from dual union all
select 29320,'029320','1989-039AB','deb SOZ','',62316,'Reentered','2008-10-03 00:00:00','2006-08-17 00:00:00',328.07,625,18133,65.02,'Highly Ellip' from dual union all
select 29321,'029321','1989-039AC','deb SOZ','',62316,'In Earth orbit','2007-01-01 00:00:00','2006-08-17 00:00:00',328.32,723,18051,65.02,'Highly Ellip' from dual union all
select 29322,'029322','1989-039AD','deb SOZ','',62316,'Reentered','2008-04-11 00:00:00','2006-08-17 00:00:00',327.96,595,18156,64.97,'Highly Ellip' from dual union all
select 29323,'029323','1989-039AE','deb SOZ','',62316,'Lost','','2006-08-17 00:00:00',332.89,630,18435,65.07,'Highly Ellip' from dual union all
select 29324,'029324','1989-039AF','deb SOZ','',62316,'Reentered','2007-01-06 00:00:00','2006-08-17 00:00:00',327.56,583,18142,64.93,'Highly Ellip' from dual union all
select 29325,'029325','1989-039AG','deb SOZ','',62316,'In Earth orbit','','2006-08-17 00:00:00',328.53,603,18183,65.04,'Highly Ellip' from dual union all
select 29326,'029326','1989-039AH','deb SOZ','',62316,'Reentered','2007-02-01 00:00:00','2006-08-17 00:00:00',327.82,572,18170,64.93,'Highly Ellip' from dual union all
select 29327,'029327','1989-039AJ','deb SOZ','',62316,'Reentered','2006-11-04 00:00:00','2006-08-17 00:00:00',328.06,597,18160,64.95,'Highly Ellip' from dual union all
select 29328,'029328','1989-039AK','deb SOZ','',62316,'In Earth orbit','','2006-08-17 00:00:00',336.95,816,18508,65.15,'Highly Ellip' from dual union all
select 29329,'029329','1989-039AL','deb SOZ','',62316,'Reentered','2006-11-22 00:00:00','2006-08-17 00:00:00',327.49,541,18180,64.92,'Highly Ellip' from dual union all
select 2933,'002933','1963-014AR','deb West Ford','RCS 0.08',9245,'In Earth orbit','','1992-01-24 00:00:00',165.97,3157,4095,87.25,'Medium' from dual union all
select 29330,'029330','1989-039AM','deb SOZ','',62316,'Reentered','2008-04-09 00:00:00','2006-08-17 00:00:00',327.29,685,18023,65,'Highly Ellip' from dual union all
select 29331,'029331','1989-039AN','deb SOZ','',62316,'In Earth orbit','','2006-08-17 00:00:00',328.63,776,18018,65.15,'Highly Ellip' from dual union all
select 29332,'029332','1989-039AP','deb SOZ','',62316,'Reentered','2013-01-11 00:00:00','2006-08-17 00:00:00',334.78,760,18426,64.53,'Highly Ellip' from dual union all
select 29333,'029333','1989-039AQ','deb SOZ','',62316,'Reentered','2008-03-28 00:00:00','2006-08-17 00:00:00',337.71,911,18461,65.42,'Highly Ellip' from dual union all
select 29334,'029334','1989-039AR','deb SOZ','',62316,'Reentered','2008-05-13 00:00:00','2006-08-17 00:00:00',333.4,1145,17953,65.61,'Highly Ellip' from dual union all
select 29335,'029335','1989-039AS','deb SOZ','',62316,'Reentered','2006-09-29 00:00:00','2006-08-17 00:00:00',339.4,1073,18405,65.33,'Highly Ellip' from dual union all
select 29336,'029336','1989-039AT','deb SOZ','',62316,'Reentered','2010-05-06 00:00:00','2006-08-17 00:00:00',341.1,1210,18377,65.6,'Highly Ellip' from dual union all
select 29337,'029337','1989-039AU','deb SOZ','',62316,'Reentered','2006-11-28 00:00:00','2006-08-17 00:00:00',334.51,1343,17825,65.4,'Highly Ellip' from dual union all
select 29338,'029338','1989-039AV','deb SOZ','',62316,'Reentered','2009-05-04 00:00:00','2006-08-17 00:00:00',333.35,1040,18054,65.63,'Highly Ellip' from dual union all
select 29339,'029339','1989-039AW','deb SOZ','',62316,'In Earth orbit','','2006-08-17 00:00:00',338.38,1112,18303,65.06,'Highly Ellip' from dual union all
select 2934,'002934','1965-020EM','deb Blok Kosmos','',11770,'In Earth orbit','','1992-02-02 00:00:00',115.37,1184,1761,55.58,'Intermediate' from dual union all
select 29340,'029340','1989-039AX','deb SOZ','',62316,'Reentered','2007-05-19 00:00:00','2006-08-17 00:00:00',340.68,1133,18426,65.65,'Highly Ellip' from dual union all
select 29341,'029341','1989-039AY','deb SOZ','',62316,'Reentered','2007-01-17 00:00:00','2006-08-16 00:00:00',324.71,968,17574,65.09,'Highly Ellip' from dual union all
select 29342,'029342','1989-039AZ','deb SOZ','',62316,'Reentered','2010-06-29 00:00:00','2006-08-16 00:00:00',336.21,966,18311,64.73,'Highly Ellip' from dual union all
select 29343,'029343','1989-039BA','deb SOZ','',62316,'Reentered','2010-10-10 00:00:00','2006-08-17 00:00:00',336.76,986,18326,65.16,'Highly Ellip' from dual union all
select 29344,'029344','1989-039BB','deb SOZ','',62316,'In Earth orbit','','2006-08-16 00:00:00',331.22,784,18174,65.16,'Highly Ellip' from dual union all
select 29345,'029345','1989-039BC','deb SOZ','',62316,'Reentered','2008-02-07 00:00:00','2006-08-16 00:00:00',323.79,908,17575,64.99,'Highly Ellip' from dual union all
select 29346,'029346','1989-039BD','deb SOZ','',62316,'Reentered','2008-05-05 00:00:00','2006-08-16 00:00:00',329.96,904,17975,64.7,'Highly Ellip' from dual union all
select 29347,'029347','1989-039BE','deb SOZ','',62316,'In Earth orbit','','2006-08-16 00:00:00',341.56,810,18806,64.89,'Highly Ellip' from dual union all
select 29348,'029348','1989-039BF','deb SOZ','',62316,'Reentered','2006-12-25 00:00:00','2006-08-16 00:00:00',327.7,645,18089,65.01,'Highly Ellip' from dual;
insert into SATELLITE
select 29349,'029349','2006-034A','Mugunghwa 5','Koreasat 5',68206,'In Earth orbit','','2006-10-27 00:00:00',1436.09,35777,35796,.01,'Stationary' from dual union all
select 2935,'002935','1967-083A','Biosatellite 2','Biosatellite B Adapter',15338,'Reentered','1967-10-04 00:00:00','1967-09-16 00:00:00',90.54,289,313,33.46,'Intermediate' from dual union all
select 29350,'029350','2006-034B','Blok DM-SL No. 23L','Blok DM-SL No. 23L',68206,'In Earth orbit','','2006-10-21 00:00:00',678.75,2923,35488,.11,'GEO Transfer' from dual union all
select 29351,'029351','1983-044FU','deb Kosmos-1461','',55801,'Reentered','2009-11-24 00:00:00','2006-08-22 00:00:00',96.32,522,643,64.98,'Intermediate' from dual union all
select 29352,'029352','1983-044FV','deb Kosmos-1461','',55801,'Reentered','2010-03-22 00:00:00','2006-08-22 00:00:00',96.51,522,661,65,'Intermediate' from dual union all
select 29353,'029353','1983-044FW','deb Kosmos-1461','',55801,'Reentered','2006-12-18 00:00:00','2006-08-20 00:00:00',94.14,451,503,64.98,'Intermediate' from dual union all
select 29354,'029354','1985-108BC','deb S5M','',58331,'Reentered','2006-09-13 00:00:00','2006-08-21 00:00:00',95.33,487,582,82.55,'Intermediate' from dual union all
select 29355,'029355','1987-020DW','deb Kosmos-1823','',60110,'In Earth orbit','','2006-08-19 00:00:00',115.81,1468,1517,73.62,'Intermediate' from dual union all
select 29356,'029356','1987-020DX','deb Kosmos-1823','',60110,'In Earth orbit','','2006-08-27 00:00:00',115.78,1464,1518,73.66,'Intermediate' from dual union all
select 29357,'029357','1987-020DY','deb Kosmos-1823','',60110,'In Earth orbit','','2006-08-26 00:00:00',112.05,1152,1490,73.79,'Intermediate' from dual union all
select 29358,'029358','1987-020DZ','deb Kosmos-1823','',60110,'In Earth orbit','','2006-08-15 00:00:00',110.66,1212,1303,73.86,'Intermediate' from dual union all
select 29359,'029359','1987-020EA','deb Kosmos-1823','',60110,'In Earth orbit','','2006-08-27 00:00:00',116.01,1479,1524,73.61,'Intermediate' from dual union all
select 2936,'002936','1967-083B','Delta 51','DSV-3E-3 S/N 20202',15338,'Reentered','1968-01-19 00:00:00','1967-10-13 00:00:00',91.93,310,428,32.94,'Intermediate' from dual union all
select 29360,'029360','1987-020EB','deb Kosmos-1823','',60110,'In Earth orbit','','2006-08-27 00:00:00',116.25,1469,1556,73.7,'Intermediate' from dual union all
select 29361,'029361','1987-020EC','deb Kosmos-1823','',60110,'In Earth orbit','','2006-08-06 00:00:00',115.84,1471,1516,73.61,'Intermediate' from dual union all
select 29362,'029362','1988-033J','NNS O-23 insulation','',61282,'In Earth orbit','','2006-08-14 00:00:00',108.47,1008,1304,90.26,'Polar' from dual union all
select 29363,'029363','2000-075F','deb EO-1','',66977,'In Earth orbit','','2006-08-26 00:00:00',109.37,689,1706,95.41,'Retrograde' from dual union all
select 29364,'029364','2006-002C','deb H-2A F8','',68174,'Reentered','2006-12-04 00:00:00','2006-09-08 00:00:00',96.51,528,655,98.18,'Sun-Synch' from dual union all
select 29365,'029365','2006-002D','deb H-2A F8','',68174,'Reentered','2006-12-12 00:00:00','2006-09-08 00:00:00',96.74,536,670,98.21,'Sun-Synch' from dual union all
select 29366,'029366','2006-002E','deb H-2A F8','',68174,'Reentered','2006-11-17 00:00:00','2006-09-08 00:00:00',96.16,529,621,98.22,'Sun-Synch' from dual union all
select 29367,'029367','2006-002F','deb H-2A F8','',68174,'Reentered','2006-10-18 00:00:00','2006-09-08 00:00:00',95.6,462,634,98.2,'Sun-Synch' from dual union all
select 29368,'029368','2006-002G','deb H-2A F8','',68174,'Reentered','2006-10-18 00:00:00','2006-09-08 00:00:00',95.82,482,635,98.23,'Sun-Synch' from dual union all
select 29369,'029369','2006-002H','deb H-2A F8','',68174,'Reentered','2006-11-25 00:00:00','2006-09-08 00:00:00',96.66,520,678,98.21,'Sun-Synch' from dual union all
select 2937,'002937','1967-084A','Surveyor 5','Surveyor 5',15339,'Deep Space','1967-09-10 00:00:00','1967-09-08 00:00:00',25585.04,174,562308,29.8,'Cislunar' from dual union all
select 29370,'029370','2006-002J','deb H-2A F8','',68174,'Reentered','2006-11-13 00:00:00','2006-09-08 00:00:00',96.34,501,666,98.21,'Sun-Synch' from dual union all
select 29371,'029371','2006-002K','deb H-2A F8','',68174,'Reentered','2006-10-29 00:00:00','2006-09-08 00:00:00',96.13,494,653,98.2,'Sun-Synch' from dual union all
select 29372,'029372','2006-002L','deb H-2A F8','',68174,'Reentered','2006-11-17 00:00:00','2006-09-08 00:00:00',96.54,509,677,98.21,'Sun-Synch' from dual union all
select 29373,'029373','2006-002M','deb H-2A F8','',68174,'Reentered','2006-11-22 00:00:00','2006-09-08 00:00:00',96.86,547,669,98.2,'Sun-Synch' from dual union all
select 29374,'029374','2006-002N','deb H-2A F8','',68174,'Reentered','2006-11-09 00:00:00','2006-09-08 00:00:00',96.32,502,663,98.21,'Sun-Synch' from dual union all
select 29375,'029375','2006-002P','deb H-2A F8','',68174,'Reentered','2007-01-08 00:00:00','2006-09-08 00:00:00',96.88,526,692,98.22,'Sun-Synch' from dual union all
select 29376,'029376','2006-002Q','deb H-2A F8','',68174,'Reentered','2006-11-20 00:00:00','2006-09-08 00:00:00',96.56,517,671,98.21,'Sun-Synch' from dual union all
select 29377,'029377','2006-002R','deb H-2A F8','',68174,'Reentered','2006-10-10 00:00:00','2006-09-08 00:00:00',95.57,466,627,98.23,'Sun-Synch' from dual union all
select 29378,'029378','2006-002S','deb H-2A F8','',68174,'Reentered','2006-11-23 00:00:00','2006-09-08 00:00:00',96.87,546,672,98.21,'Sun-Synch' from dual union all
select 29379,'029379','2006-002T','deb H-2A F8','',68174,'Reentered','2006-11-20 00:00:00','2006-09-08 00:00:00',96.69,531,670,98.18,'Sun-Synch' from dual union all
select 2938,'002938','1967-084B','Centaur D AC-13','Centaur D AC-13',15339,'Deep Space','1967-09-10 00:00:00','1967-09-08 00:00:00',12627.92,169,346337,29.87,'Cislunar' from dual union all
select 29380,'029380','2006-002U','deb H-2A F8','',68174,'Reentered','2007-01-09 00:00:00','2006-09-08 00:00:00',97.04,543,691,98.2,'Sun-Synch' from dual union all
select 29381,'029381','2006-002V','deb H-2A F8','',68174,'Reentered','2006-10-17 00:00:00','2006-09-08 00:00:00',95.81,477,639,98.21,'Sun-Synch' from dual union all
select 29382,'029382','2006-002W','deb H-2A F8','',68174,'Reentered','2006-10-14 00:00:00','2006-09-08 00:00:00',95.94,504,623,98.16,'Sun-Synch' from dual union all
select 29383,'029383','2006-002X','deb H-2A F8','',68174,'Reentered','2006-11-11 00:00:00','2006-09-08 00:00:00',96.6,538,654,98.2,'Sun-Synch' from dual union all
select 29384,'029384','2006-002Y','deb H-2A F8','',68174,'Reentered','2006-10-26 00:00:00','2006-09-08 00:00:00',96.23,508,648,98.23,'Sun-Synch' from dual union all
select 29385,'029385','2006-035A','Shi Jian 8','shijian bahao yuzhong we',68207,'Reentered','2006-11-01 00:00:00','2006-09-09 00:00:00',90.74,177,445,62.99,'Intermediate' from dual union all
select 29386,'029386','2006-035B','CZ-2C Stage 2','CZ-2C Stage 2',68207,'Reentered','2006-10-03 00:00:00','2006-09-09 00:00:00',90.76,179,445,62.99,'Intermediate' from dual union all
select 29387,'029387','2006-035C','CZ-2C Stage 2 deb','',68207,'Reentered','2006-09-13 00:00:00','2006-09-09 00:00:00',90.65,177,436,62.91,'Intermediate' from dual union all
select 29388,'029388','2006-035D','CZ-2C Stage 2 motor cover','',68207,'Reentered','2006-09-12 00:00:00','2006-09-09 00:00:00',92.68,181,630,63.24,'Intermediate' from dual union all
select 29389,'029389','2006-035E','CZ-2C Stage 2 motor cover','',68207,'Reentered','2006-09-12 00:00:00','2006-09-09 00:00:00',92.99,179,662,62.68,'Intermediate' from dual union all
select 2939,'002939','1967-085A','Kosmos-175','Zenit-4 No. 37',15340,'Landed','1967-09-19 00:00:00','1967-09-13 00:00:00',90.15,202,362,72.88,'Intermediate' from dual union all
select 29390,'029390','2006-035F','CZ-2C Stage 2 motor cover','',68207,'Reentered','2006-09-13 00:00:00','2006-09-09 00:00:00',93.07,175,675,62.73,'Intermediate' from dual union all
select 29391,'029391','2006-036A','Atlantis (STS-115)','OV-104',68208,'Landed','2006-09-21 00:00:00','2006-09-09 00:00:00',89.62,228,283,51.63,'Intermediate' from dual union all
select 29392,'029392','2006-035G','CZ-2C Stage 2 motor cover','',68207,'Reentered','2006-09-12 00:00:00','2006-09-11 00:00:00',91.2,174,492,63.29,'Intermediate' from dual union all
select 29393,'029393','2006-037A','JSE kougaku-2 gouki','IGS Optical-2',68209,'Reentered','2016-10-29 00:00:00','2006-09-14 00:00:00',94.35,484,491,97.31,'Sun-Synch' from dual union all
select 29394,'029394','2006-037B','H-2A F10 Stage 2','H-2A Stage 2',68209,'Reentered','2012-02-16 00:00:00','2006-11-10 00:00:00',93.82,431,492,97.23,'Sun-Synch' from dual union all
select 29395,'029395','2006-037C','deb IGS O-2','',68209,'Reentered','2006-11-07 00:00:00','2006-09-18 00:00:00',93.91,450,482,97.21,'Sun-Synch' from dual union all
select 29396,'029396','2006-037D','deb IGS O-2','',68209,'Reentered','2007-04-26 00:00:00','2006-09-19 00:00:00',93.81,430,491,97.22,'Sun-Synch' from dual union all
select 29397,'029397','2006-039C','deb Kosmos-2423','',68211,'Reentered','2006-09-15 00:00:00','2006-09-14 00:00:00',88.57,160,247,64.91,'Intermediate' from dual union all
select 29398,'029398','2006-038A','Zhongxing 22A','Feng Huo 2',68210,'In Earth orbit','','2006-11-11 00:00:00',1436.16,35755,35821,.29,'Stationary' from dual union all
select 29399,'029399','2006-038B','CZ-3A Stage 3','CZ-3A Stage 3',68210,'Reentered','2006-11-08 00:00:00','2006-09-19 00:00:00',753.22,161,41928,25.32,'Highly Ellip' from dual union all
select 294,'000294','62 Sigma 3','ARGON camera door','',8202,'Reentered','1962-07-13 00:00:00','1962-06-14 00:00:00',92.94,285,551,82.28,'Intermediate' from dual union all
select 2940,'002940','1967-080B','Burner II No. 4','Star 37B',15335,'In Earth orbit','','1967-09-23 00:00:00',102.16,828,894,98.95,'Sun-Synch' from dual union all
select 29400,'029400','2006-040A','Soyuz TMA-9','Soyuz 11F732 No. 219',68212,'Landed','2007-04-21 00:00:00','2006-09-20 00:00:00',91.36,334,349,51.63,'Intermediate' from dual union all
select 29401,'029401','2006-040B','11S510','11S510',68212,'Reentered','2006-09-21 00:00:00','2006-09-19 00:00:00',88.17,176,192,51.66,'Intermediate' from dual union all
select 29402,'029402','2006-039A','Kosmos-2423','Don',68211,'Deorbited','2006-11-17 00:00:00','2006-09-21 00:00:00',89.64,209,305,64.9,'Intermediate' from dual union all
select 29403,'029403','2006-039B','11S510','11S510',68211,'Reentered','2006-09-19 00:00:00','2006-09-16 00:00:00',88.83,166,268,64.91,'Intermediate' from dual union all
select 29404,'029404','1989-039BG','deb SOZ','',62316,'Reentered','2006-12-19 00:00:00','2006-09-15 00:00:00',329.28,786,18048,65.28,'Highly Ellip' from dual union all
select 29405,'029405','1989-039BH','deb SOZ','',62316,'In Earth orbit','','2006-09-15 00:00:00',344.35,1734,18058,65.69,'Highly Ellip' from dual union all
select 29406,'029406','1989-039BJ','deb SOZ','',62316,'Reentered','2009-11-01 00:00:00','2006-09-19 00:00:00',330.31,470,18431,65.45,'Highly Ellip' from dual union all
select 29407,'029407','1989-039BK','deb SOZ','',62316,'In Earth orbit','','2006-09-10 00:00:00',328.31,1134,17639,65.7,'Highly Ellip' from dual union all
select 29408,'029408','1989-039BL','deb SOZ','',62316,'Reentered','2007-01-08 00:00:00','2006-09-19 00:00:00',331.23,822,18138,65.73,'Highly Ellip' from dual union all
select 29409,'029409','1989-039BM','deb SOZ','',62316,'Reentered','2007-01-09 00:00:00','2006-09-19 00:00:00',332.2,958,18064,65.94,'Highly Ellip' from dual union all
select 2941,'002941','1967-085B','11A57I','11A57I',15340,'Reentered','1967-09-23 00:00:00','1967-09-13 00:00:00',89.94,201,342,72.88,'Intermediate' from dual union all
select 29410,'029410','1989-039BN','deb SOZ','',62316,'Reentered','2009-10-26 00:00:00','2006-09-09 00:00:00',332.69,1050,18003,65.67,'Highly Ellip' from dual union all
select 29411,'029411','1989-039BP','deb SOZ','',62316,'Reentered','2006-12-28 00:00:00','2006-09-19 00:00:00',334.34,1626,17532,65.92,'Medium' from dual union all
select 29412,'029412','1989-039BQ','deb SOZ','',62316,'Reentered','2007-01-03 00:00:00','2006-09-12 00:00:00',341.97,979,18663,65.43,'Highly Ellip' from dual union all
select 29413,'029413','1989-039BR','deb SOZ','',62316,'Reentered','2007-01-03 00:00:00','2006-09-19 00:00:00',327.42,748,17967,65.21,'Highly Ellip' from dual union all
select 29414,'029414','1989-039BS','deb SOZ','',62316,'In Earth orbit','','2006-09-19 00:00:00',333.57,692,18417,65.12,'Highly Ellip' from dual union all
select 29415,'029415','1989-039BT','deb SOZ','',62316,'Reentered','2008-05-09 00:00:00','2006-09-19 00:00:00',335.99,1009,18253,65.52,'Highly Ellip' from dual union all
select 29416,'029416','1989-039BU','deb SOZ','',62316,'In Earth orbit','','2006-09-18 00:00:00',334.73,760,18423,64.7,'Highly Ellip' from dual union all
select 29417,'029417','1989-039BV','deb SOZ','',62316,'In Earth orbit','','2006-08-04 00:00:00',333.19,1369,17716,65.01,'Highly Ellip' from dual union all
select 29418,'029418','1989-039BW','deb SOZ','',62316,'In Earth orbit','','2006-07-23 00:00:00',333.87,1292,17835,64.25,'Highly Ellip' from dual union all
select 29419,'029419','2003-037D','deb S3M','',67661,'In Earth orbit','','2006-09-19 00:00:00',115.6,1465,1501,82.47,'Intermediate' from dual union all
select 2942,'002942','1967-086A','Kosmos-176','DS-P1-Yu No. 10',15341,'Reentered','1968-03-03 00:00:00','1967-10-13 00:00:00',100.74,193,1394,81.92,'Intermediate' from dual union all
select 29420,'029420','2003-037E','deb S3M','',67661,'In Earth orbit','','2006-09-14 00:00:00',115.64,1466,1502,82.47,'Intermediate' from dual union all
select 29421,'029421','2003-037F','deb S3M','',67661,'In Earth orbit','','2006-09-19 00:00:00',115.6,1464,1502,82.47,'Intermediate' from dual union all
select 29422,'029422','2004-023D','Star 48 yo weight','',67867,'In Earth orbit','','2006-09-19 00:00:00',311.55,157,17533,38.82,'Highly Ellip' from dual union all
select 29423,'029423','2004-044G','deb CZ-4B','',67888,'Reentered','2012-08-18 00:00:00','2006-09-19 00:00:00',93.95,387,549,97.11,'Sun-Synch' from dual union all
select 29424,'029424','1989-039BX','deb SOZ','',62316,'Reentered','2008-01-17 00:00:00','2006-09-19 00:00:00',332.3,730,18298,65.7,'Highly Ellip' from dual union all
select 29425,'029425','1989-039BY','deb SOZ','',62316,'Reentered','2006-12-04 00:00:00','2006-09-20 00:00:00',329.49,709,18140,65.26,'Highly Ellip' from dual union all
select 29426,'029426','1989-039BZ','deb SOZ','',62316,'Reentered','2006-11-24 00:00:00','2006-09-08 00:00:00',331.69,1249,17740,66.13,'Highly Ellip' from dual union all
select 29427,'029427','1989-039CA','deb SOZ','',62316,'Reentered','2008-03-10 00:00:00','2006-09-20 00:00:00',331.94,886,18118,65.84,'Highly Ellip' from dual union all
select 29428,'029428','1989-039CB','deb SOZ','',62316,'Lost','','2006-09-20 00:00:00',331.73,1152,17840,65.91,'Highly Ellip' from dual union all
select 29429,'029429','1989-039CC','deb SOZ','',62316,'Lost','','2006-09-20 00:00:00',331.86,1004,17996,65.74,'Highly Ellip' from dual union all
select 2943,'002943','1967-085C','deb Kosmos-175','',15340,'Reentered','1967-09-13 00:00:00','1967-09-12 00:00:00',89.62,194,318,72.91,'Intermediate' from dual union all
select 29430,'029430','1989-039CD','deb SOZ','',62316,'In Earth orbit','','2006-09-20 00:00:00',333.32,988,18105,65.73,'Highly Ellip' from dual union all
select 29431,'029431','1989-039CF','deb SOZ','',62316,'In Earth orbit','2007-01-01 00:00:00','2006-09-10 00:00:00',341.26,1433,18163,65.32,'Highly Ellip' from dual union all
select 29432,'029432','1989-039CG','deb SOZ','',62316,'In Earth orbit','','2006-09-17 00:00:00',322.9,1256,17170,65.27,'Highly Ellip' from dual union all
select 29433,'029433','1989-039CH','deb SOZ','',62316,'Reentered','2009-12-25 00:00:00','2006-09-19 00:00:00',332.34,1045,17986,65.94,'Highly Ellip' from dual union all
select 29434,'029434','1989-039CJ','deb SOZ','',62316,'Reentered','2007-01-15 00:00:00','2006-09-15 00:00:00',332.99,902,18169,65.96,'Highly Ellip' from dual union all
select 29435,'029435','1989-039CK','deb SOZ','',62316,'Reentered','2006-11-14 00:00:00','2006-09-11 00:00:00',319.92,1068,17165,65.1,'Highly Ellip' from dual union all
select 29436,'029436','1989-039CE','deb SOZ','',62316,'Reentered','2009-04-01 00:00:00','2006-09-19 00:00:00',331.69,1022,17967,65.98,'Highly Ellip' from dual union all
select 29437,'029437','1989-039CL','deb SOZ','',62316,'Reentered','2008-04-08 00:00:00','2006-09-19 00:00:00',341.61,981,18638,65.43,'Highly Ellip' from dual union all
select 29438,'029438','1989-039CM','deb SOZ','',62316,'Reentered','2007-01-29 00:00:00','2006-09-20 00:00:00',334.68,920,18260,65.98,'Highly Ellip' from dual;
insert into SATELLITE
select 29439,'029439','1989-039CN','deb SOZ','',62316,'In Earth orbit','','2006-09-20 00:00:00',336.98,1157,18169,65.26,'Highly Ellip' from dual union all
select 2944,'002944','1967-086C','deb S1M','',15341,'Reentered','1967-10-04 00:00:00','1967-09-23 00:00:00',98.42,139,1228,82.21,'Intermediate' from dual union all
select 29440,'029440','1989-039CP','deb SOZ','',62316,'Reentered','2009-05-15 00:00:00','2006-09-11 00:00:00',325.76,1123,17487,65.24,'Highly Ellip' from dual union all
select 29441,'029441','1989-039CQ','deb SOZ','',62316,'In Earth orbit','','2006-09-20 00:00:00',324.39,1000,17521,65.27,'Highly Ellip' from dual union all
select 29442,'029442','1989-039CR','deb SOZ','',62316,'Reentered','2008-05-05 00:00:00','2006-09-13 00:00:00',332.03,994,18017,65.32,'Highly Ellip' from dual union all
select 29443,'029443','1989-039CS','deb SOZ','',62316,'In Earth orbit','','2006-09-20 00:00:00',341.77,991,18638,65.04,'Highly Ellip' from dual union all
select 29444,'029444','1981-021W','deb US-A coolant ','[UNID]',52544,'In Earth orbit','','2006-09-19 00:00:00',103.73,910,960,64.96,'Intermediate' from dual union all
select 29445,'029445','1984-035D','deb CZ-3 Stage 3','',57089,'In Earth orbit','','2006-09-20 00:00:00',618.29,517,34809,30.99,'GEO Transfer' from dual union all
select 29446,'029446','1984-035E','deb CZ-3 Stage 3','',57089,'In Earth orbit','','2006-09-21 00:00:00',613.01,525,34527,31.16,'GEO Transfer' from dual union all
select 29447,'029447','1986-010F','deb CZ-3 Stage 3','',59203,'In Earth orbit','','2006-09-12 00:00:00',635.76,490,35738,30.92,'GEO Transfer' from dual union all
select 29448,'029448','1986-010G','deb CZ-3 Stage 3','',59203,'In Earth orbit','','2006-09-15 00:00:00',622.92,439,35127,30.72,'GEO Transfer' from dual union all
select 29449,'029449','1989-039CT','deb SOZ','',62316,'In Earth orbit','','2006-09-22 00:00:00',342.65,1773,17912,66.12,'Medium' from dual union all
select 2945,'002945','1967-086B','S1M','S1M',15341,'Reentered','1967-12-12 00:00:00','1967-10-15 00:00:00',99.08,199,1231,81.82,'Intermediate' from dual union all
select 29450,'029450','1989-039CU','deb SOZ','',62316,'In Earth orbit','','2006-09-22 00:00:00',342.31,1282,18380,66.04,'Highly Ellip' from dual union all
select 29451,'029451','1989-039CV','deb SOZ','',62316,'Reentered','2008-04-15 00:00:00','2006-09-22 00:00:00',333.84,990,18136,65.12,'Highly Ellip' from dual union all
select 29452,'029452','1989-039CW','deb SOZ','',62316,'Reentered','2007-01-06 00:00:00','2006-09-11 00:00:00',331.33,887,18079,65.46,'Highly Ellip' from dual union all
select 29453,'029453','1989-039CX','deb SOZ','',62316,'In Earth orbit','','2006-09-21 00:00:00',331.11,962,17990,66.12,'Highly Ellip' from dual union all
select 29454,'029454','1989-039CY','deb SOZ','',62316,'Reentered','2007-01-28 00:00:00','2006-09-20 00:00:00',324.77,919,17627,65.7,'Highly Ellip' from dual union all
select 29455,'029455','1989-039CZ','deb SOZ','',62316,'Reentered','2010-02-12 00:00:00','2006-09-21 00:00:00',331.7,916,18073,65.93,'Highly Ellip' from dual union all
select 29456,'029456','1989-039DA','deb SOZ','',62316,'Reentered','2007-02-10 00:00:00','2006-09-22 00:00:00',325.47,998,17592,65.3,'Highly Ellip' from dual union all
select 29457,'029457','1989-039DB','deb SOZ','',62316,'Reentered','2006-11-29 00:00:00','2006-09-21 00:00:00',331.91,739,18264,66,'Highly Ellip' from dual union all
select 29458,'029458','1989-039DC','deb SOZ','',62316,'In Earth orbit','','2006-09-18 00:00:00',337.34,1487,17862,65.23,'Highly Ellip' from dual union all
select 29459,'029459','1989-039DD','deb SOZ','',62316,'In Earth orbit','','2006-09-22 00:00:00',333.54,1268,17838,65.95,'Highly Ellip' from dual union all
select 2946,'002946','1967-087A','OPS 5089','CORONA CR-1',15342,'Reentered Attached','1967-10-04 00:00:00','1967-09-26 00:00:00',90,148,401,80.09,'Intermediate' from dual union all
select 29460,'029460','1989-039DE','deb SOZ','',62316,'Reentered','2007-01-03 00:00:00','2006-09-09 00:00:00',332.39,745,18289,65.77,'Highly Ellip' from dual union all
select 29461,'029461','1989-039DF','deb SOZ','',62316,'Reentered','2007-10-15 00:00:00','2006-09-22 00:00:00',336.49,1128,18166,65.19,'Highly Ellip' from dual union all
select 29462,'029462','1989-039DG','deb SOZ','',62316,'In Earth orbit','2011-01-01 00:00:00','2006-09-22 00:00:00',324.58,963,17570,65.38,'Highly Ellip' from dual union all
select 29463,'029463','1989-039DH','deb SOZ','',62316,'In Earth orbit','','2006-09-17 00:00:00',331.14,906,18047,65.48,'Highly Ellip' from dual union all
select 29464,'029464','1989-039DJ','deb SOZ','',62316,'Reentered','2009-01-05 00:00:00','2006-08-31 00:00:00',336.13,1117,18155,64.91,'Highly Ellip' from dual union all
select 29465,'029465','1989-039DK','deb SOZ','',62316,'Reentered','2006-11-12 00:00:00','2006-09-22 00:00:00',327.79,640,18099,65.3,'Highly Ellip' from dual union all
select 29466,'029466','1989-039DL','deb SOZ','',62316,'Reentered','2006-11-20 00:00:00','2006-09-21 00:00:00',328.45,826,17956,65.62,'Highly Ellip' from dual union all
select 29467,'029467','1989-039DM','deb SOZ','',62316,'Reentered','2008-03-03 00:00:00','2006-09-15 00:00:00',327.46,734,17985,65.41,'Highly Ellip' from dual union all
select 29468,'029468','1989-039DN','deb SOZ','',62316,'Reentered','2006-11-23 00:00:00','2006-09-22 00:00:00',327.64,677,18053,65.41,'Highly Ellip' from dual union all
select 29469,'029469','1989-039DP','deb SOZ','',62316,'Reentered','2006-11-26 00:00:00','2006-09-22 00:00:00',327.84,671,18071,65.29,'Highly Ellip' from dual union all
select 2947,'002947','1967-088A','Kosmos-177','Zenit-2 No. 53',15343,'Landed','1967-09-24 00:00:00','1967-09-20 00:00:00',89.18,201,267,51.79,'Intermediate' from dual union all
select 29470,'029470','1989-039DQ','deb SOZ','',62316,'Reentered','2006-12-13 00:00:00','2006-09-13 00:00:00',328.54,922,17866,65.67,'Highly Ellip' from dual union all
select 29471,'029471','1989-039DR','deb SOZ','',62316,'Reentered','2006-11-14 00:00:00','2006-09-17 00:00:00',328.26,834,17936,65.59,'Highly Ellip' from dual union all
select 29472,'029472','1989-039DS','deb SOZ','',62316,'Reentered','2006-12-04 00:00:00','2006-09-22 00:00:00',327.29,608,18100,65.16,'Highly Ellip' from dual union all
select 29473,'029473','1989-039DT','deb SOZ','',62316,'Reentered','2011-09-09 00:00:00','2006-09-11 00:00:00',328.62,937,17856,65.69,'Highly Ellip' from dual union all
select 29474,'029474','1989-039DU','deb SOZ','',62316,'In Earth orbit','2012-01-01 00:00:00','2006-09-21 00:00:00',333.05,1012,18064,65.99,'Highly Ellip' from dual union all
select 29475,'029475','1989-039DV','deb SOZ','',62316,'Reentered','2007-07-09 00:00:00','2006-09-07 00:00:00',333.96,906,18227,65.69,'Highly Ellip' from dual union all
select 29476,'029476','1989-039DW','deb SOZ','',62316,'Reentered','2007-10-14 00:00:00','2006-09-21 00:00:00',324.41,979,17544,65.26,'Highly Ellip' from dual union all
select 29477,'029477','1989-039DX','deb SOZ','',62316,'Reentered','2007-07-16 00:00:00','2006-09-22 00:00:00',327.48,846,17874,65.42,'Highly Ellip' from dual union all
select 29478,'029478','1989-039DY','deb SOZ','',62316,'Reentered','2008-02-19 00:00:00','2006-09-22 00:00:00',329.73,947,17917,65.53,'Highly Ellip' from dual union all
select 29479,'029479','2006-041A','Hinode','SOLAR-B',68213,'In Earth orbit','','2006-09-25 00:00:00',94.11,279,672,98.33,'Sun-Synch' from dual union all
select 2948,'002948','1967-088B','11A57I','11A57I',15343,'Reentered','1967-09-19 00:00:00','1967-09-18 00:00:00',88.56,173,234,51.79,'Intermediate' from dual union all
select 29480,'029480','2006-041B','M-34','M-34',68213,'Reentered','2009-10-20 00:00:00','2006-09-25 00:00:00',94.07,280,666,98.32,'Sun-Synch' from dual union all
select 29481,'029481','2006-041C','deb M-34','',68213,'Reentered','2006-10-11 00:00:00','2006-09-24 00:00:00',94.08,283,665,98.35,'Sun-Synch' from dual union all
select 29482,'029482','2006-041D','SSSAT','SolarSail Subpayload Sat',68213,'Reentered','2006-09-25 00:00:00','2006-09-23 00:00:00',93.12,276,579,98.32,'Sun-Synch' from dual union all
select 29483,'029483','2006-041E','deb M-34','',68213,'Reentered','2006-10-13 00:00:00','2006-09-25 00:00:00',93.77,276,642,98.35,'Sun-Synch' from dual union all
select 29484,'029484','2006-041F','HITSAT-OSCAR-59','HIT-SAT',68213,'Reentered','2008-06-18 00:00:00','2006-09-25 00:00:00',94.07,279,668,98.33,'Sun-Synch' from dual union all
select 29485,'029485','2006-041G','deb M-34','',68213,'Reentered','2006-10-15 00:00:00','2006-09-25 00:00:00',93.69,276,634,98.32,'Sun-Synch' from dual union all
select 29486,'029486','2006-042A','GPS IIR-15(M)','Navstar GPS SVN 52',68214,'In Earth orbit','','2006-11-25 00:00:00',717.9,20018,20343,55,'Medium' from dual union all
select 29487,'029487','2006-042B','Star 48B','Star 48B',68214,'Reentered','2013-12-27 00:00:00','2006-11-26 00:00:00',351.87,187,20077,39.96,'Highly Ellip' from dual union all
select 29488,'029488','2006-042C','Delta 318','Delta SSPS AJ-10-118K',68214,'Reentered','2006-12-23 00:00:00','2006-09-27 00:00:00',98.71,182,1212,33.65,'Intermediate' from dual union all
select 29489,'029489','1989-039DZ','deb SOZ','',62316,'In Earth orbit','','2006-10-25 00:00:00',331.62,1250,17735,65.44,'Highly Ellip' from dual union all
select 2949,'002949','1963-014AS','deb West Ford','RCS 0.22',9245,'In Earth orbit','','1992-02-02 00:00:00',163.92,1500,5587,85.91,'Medium' from dual union all
select 29490,'029490','2006-041H','deb M-34','',68213,'Reentered','2006-10-21 00:00:00','2006-09-26 00:00:00',93.73,278,636,98.37,'Sun-Synch' from dual union all
select 29491,'029491','2006-041J','deb SSSAT','',68213,'Reentered','2006-10-03 00:00:00','2006-09-26 00:00:00',93.18,273,588,98.34,'Sun-Synch' from dual union all
select 29492,'029492','2006-041K','deb SSSAT','',68213,'Reentered','2006-10-02 00:00:00','2006-09-27 00:00:00',92.14,264,495,98.37,'Sun-Synch' from dual union all
select 29493,'029493','2006-037E','deb IGS O-2','',68209,'Reentered','2007-06-29 00:00:00','2006-09-28 00:00:00',93.82,430,492,97.23,'Sun-Synch' from dual union all
select 29494,'029494','2006-043A','DirecTV 9S','DirecTV 9S',68215,'In Earth orbit','','2006-12-13 00:00:00',1433.23,35723,35738,.02,'Drift GEO' from dual union all
select 29495,'029495','2006-043B','Optus D1','Optus D1',68215,'In Earth orbit','','2006-12-14 00:00:00',1436.09,35772,35801,.05,'Stationary' from dual union all
select 29496,'029496','2006-043C','LDREX-2','LDREX-2',68215,'Reentered','2010-09-30 00:00:00','2006-12-14 00:00:00',628.38,267,35581,7.03,'GEO Transfer' from dual union all
select 29497,'029497','2006-043D','ESC-A 533','Ariane 5 ESC-A',68215,'Reentered','2008-07-21 00:00:00','2006-12-13 00:00:00',628.34,257,35589,7.01,'GEO Transfer' from dual union all
select 29498,'029498','2006-043E','Sylda 5A','Sylda 5',68215,'In Earth orbit','','2006-12-14 00:00:00',628.56,249,35608,6.62,'GEO Transfer' from dual union all
select 29499,'029499','2006-044A','METOP A','METOP A',68216,'In Earth orbit','','2006-10-26 00:00:00',101.28,819,820,98.74,'Sun-Synch' from dual union all
select 295,'000295','62 Tau 1','Aurora Seven','Mercury SC18',8203,'Landed','1962-05-24 00:00:00','1962-05-24 00:00:00',88.63,154,260,32.5,'Intermediate' from dual union all
select 2950,'002950','1963-014AT','deb West Ford','RCS 0.06',9245,'Reentered','1996-06-03 00:00:00','1992-01-03 00:00:00',162.77,948,6046,85.21,'Medium' from dual union all
select 29500,'029500','2006-044B','Fregat No. 1011','Fregat 14S44 No. 1011',68216,'Deorbited','2006-10-19 00:00:00','2006-10-19 00:00:00',93.8,99,822,98.74,'Sun-Synch' from dual union all
select 29501,'029501','1991-050K','SARA antenna debris','',64219,'Reentered','2011-08-23 00:00:00','2006-10-19 00:00:00',99.15,708,728,98.09,'Sun-Synch' from dual union all
select 29502,'029502','1991-050L','SARA antenna debris','',64219,'Reentered','2011-12-31 00:00:00','2006-10-19 00:00:00',99.47,725,741,98.09,'Sun-Synch' from dual union all
select 29503,'029503','2006-045A','Progress M-58','Progress 7K-TGM No. 358',68217,'Deorbited','2007-03-28 00:00:00','2006-10-30 00:00:00',91.27,329,345,51.63,'Intermediate' from dual union all
select 29504,'029504','2006-045B','11S510','11S510',68217,'Reentered','2006-10-27 00:00:00','2006-10-25 00:00:00',88.38,179,210,51.63,'Intermediate' from dual union all
select 29505,'029505','2006-046A','Shi Jian 6/2B','shijian liuhao er zu B x',68218,'In Earth orbit','','2006-10-31 00:00:00',96.65,596,601,97.72,'Sun-Synch' from dual union all
select 29506,'029506','2006-046B','Shi Jian 6/2A','shijian liuhao er zu A x',68218,'In Earth orbit','','2006-10-31 00:00:00',96.61,594,599,97.72,'Sun-Synch' from dual union all
select 29507,'029507','2006-046C','CZ-4B Stage  3','CZ-4B',68218,'In Earth orbit','','2006-10-31 00:00:00',96.49,560,621,97.68,'Sun-Synch' from dual union all
select 29508,'029508','2006-046D','Adapter','',68218,'Reentered','2017-04-20 00:00:00','2006-10-31 00:00:00',96.62,595,599,97.71,'Sun-Synch' from dual union all
select 29509,'029509','2006-046E','deb CZ-4B','',68218,'In Earth orbit','','2006-11-09 00:00:00',96.87,573,645,97.62,'Sun-Synch' from dual union all
select 2951,'002951','1967-089A','Kosmos-178','OGCh No. 13',15344,'Deorbited','1967-09-19 00:00:00','1967-09-19 00:00:00',88.97,137,311,49.6,'Intermediate' from dual union all
select 29510,'029510','2006-047A','STEREO A','STEREO Ahead',68219,'Deep Space','2006-10-26 00:00:00','2006-12-25 00:00:00',163132.55,192937,1772266,33.1,'Cislunar' from dual union all
select 29511,'029511','2006-047B','STEREO B','STEREO Behind',68219,'Deep Space','2006-10-26 00:00:00','2006-12-25 00:00:00',57532.27,115890,858689,28.16,'Cislunar' from dual union all
select 29512,'029512','2006-047C','Delta 319','Delta SSPS AJ-10-118K',68219,'Reentered','2007-11-01 00:00:00','2006-11-02 00:00:00',119.3,179,3120,24.56,'Equatorial' from dual union all
select 29513,'029513','2006-047D','Delta attach clamp','',68219,'Reentered','2006-10-26 00:00:00','2006-10-26 00:00:00',118.56,84,3148,28.44,'Intermediate' from dual union all
select 29514,'029514','1989-039EA','deb SOZ','',62316,'Reentered','2008-04-03 00:00:00','2006-11-08 00:00:00',336.87,1331,17987,65.78,'Highly Ellip' from dual union all
select 29515,'029515','1989-039EB','deb SOZ','',62316,'Reentered','2013-05-10 00:00:00','2006-10-25 00:00:00',345.11,1139,18701,65.05,'Highly Ellip' from dual union all
select 29516,'029516','2006-048A','Xinnuo 2','xinnuo erhao',68220,'In Earth orbit','','2006-12-27 00:00:00',1436.09,35668,35905,.25,'Stationary' from dual union all
select 29517,'029517','2006-048B','CZ-3B Stage 3','CZ-3B Stage 3',68220,'In Earth orbit','','2006-12-27 00:00:00',616.53,156,35079,28.41,'GEO Transfer' from dual union all
select 29518,'029518','2004-029C','deb Tan Ce 2','',67873,'Reentered','2012-11-07 00:00:00','2006-11-13 00:00:00',496.37,405,28372,89.2,'Highly Ellip' from dual union all
select 29519,'029519','2004-029D','deb Tan Ce 2','',67873,'In Earth orbit','','2006-11-12 00:00:00',585.94,569,33065,89.44,'Highly Ellip' from dual union all
select 2952,'002952','1967-090A','OPS 4941','GAMBIT-3 Mission 4308',15345,'Reentered Attached','1967-09-30 00:00:00','1967-09-21 00:00:00',89.7,121,399,106.1,'Retrograde' from dual union all
select 29520,'029520','2006-049A','XM-4','XM Radio-4',68221,'In Earth orbit','','2006-12-30 00:00:00',1436.06,35784,35788,.04,'Stationary' from dual union all
select 29521,'029521','2006-049B','Blok DM-SL No. 24L','Blok DM-SL',68221,'In Earth orbit','','2006-12-30 00:00:00',657.91,1294,36065,.24,'GEO Transfer' from dual union all
select 29522,'029522','2006-050A','DMSP F-17','DMSP Block 5D-3 S-17',68222,'In Earth orbit','','2006-11-06 00:00:00',101.87,841,854,98.79,'Sun-Synch' from dual union all
select 29523,'029523','2006-050B','Delta 320','DCSS-4 F06',68222,'Deorbited','2006-11-04 00:00:00','2006-11-04 00:00:00',101.84,830,862,98.79,'Sun-Synch' from dual union all
select 29524,'029524','2006-050C','DMSP F-17 OLS Cover','',68222,'In Earth orbit','','2006-11-07 00:00:00',101.92,847,853,98.78,'Sun-Synch' from dual union all
select 29525,'029525','2006-050D','DMSP F-17 OLS Cooler Cover','',68222,'In Earth orbit','','2006-11-08 00:00:00',101.87,845,850,98.78,'Sun-Synch' from dual union all
select 29526,'029526','2006-051A','Badr 4','Arabsat 4B',68223,'In Earth orbit','','2007-01-07 00:00:00',1436.06,35761,35811,.05,'Stationary' from dual union all
select 29527,'029527','2006-051B','Briz-M No. 88517','Briz-M',68223,'In Earth orbit','','2007-01-08 00:00:00',687.47,3092,35757,14.58,'GEO Transfer' from dual union all
select 29528,'029528','2006-051C','Briz-M DTB','Briz-M torus tank',68223,'In Earth orbit','','2007-01-08 00:00:00',395.33,663,22267,51.44,'Highly Ellip' from dual;
insert into SATELLITE
select 29529,'029529','1989-039EC','deb SOZ','',62316,'Reentered','2007-06-14 00:00:00','2006-11-06 00:00:00',331.67,662,18326,66.03,'Highly Ellip' from dual union all
select 2953,'002953','1967-089B','Kosmos-178 Adapter','OGCh Adapter',15344,'Reentered','1967-09-20 00:00:00','1967-09-19 00:00:00',87.7,150,172,49.6,'Intermediate' from dual union all
select 29530,'029530','1990-065X','G-12A Canister','Ba Canister',63307,'In Earth orbit','','2006-11-07 00:00:00',606.54,303,34412,17.45,'GEO Transfer' from dual union all
select 29531,'029531','1990-065Y','G-12B Canister','Ba Canister',63307,'In Earth orbit','','2006-11-06 00:00:00',601.77,312,34154,17.24,'GEO Transfer' from dual union all
select 29532,'029532','1990-065Z','G-6B Canister','',63307,'In Earth orbit','','2006-11-07 00:00:00',579.22,345,32933,17.78,'Highly Ellip' from dual union all
select 29533,'029533','1990-065AA','G-7B Canister','',63307,'In Earth orbit','','2006-11-06 00:00:00',592.56,366,33617,17.59,'Highly Ellip' from dual union all
select 29534,'029534','2005-038D','Star 48 yo weight','',68047,'In Earth orbit','','2006-11-03 00:00:00',355.6,259,20238,39.51,'Highly Ellip' from dual union all
select 29535,'029535','2006-008E','deb ST-5/Pegasus','',68180,'Reentered','2009-06-26 00:00:00','2006-09-06 00:00:00',129.63,301,3912,105.66,'Medium' from dual union all
select 29536,'029536','2006-015C','CZ-4B sep motor cover','',68187,'In Earth orbit','','2006-11-08 00:00:00',99.96,619,894,97.76,'Sun-Synch' from dual union all
select 29537,'029537','2006-015D','CZ-4B sep motor cover','',68187,'In Earth orbit','','2006-11-08 00:00:00',100.04,619,902,97.85,'Sun-Synch' from dual union all
select 29538,'029538','2006-015E','CZ-4B sep motor cover','',68187,'In Earth orbit','','2006-11-08 00:00:00',99.56,612,863,97.82,'Sun-Synch' from dual union all
select 29539,'029539','2006-015F','CZ-4B sep motor cover','',68187,'In Earth orbit','','2006-11-08 00:00:00',99.49,616,853,97.76,'Sun-Synch' from dual union all
select 2954,'002954','1967-089C','8S692','8S692',15344,'Reentered','1967-09-19 00:00:00','1967-09-19 00:00:00',87.48,137,163,49.6,'Intermediate' from dual union all
select 29540,'029540','2006-050E','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2009-03-27 00:00:00','2006-11-13 00:00:00',99.63,636,846,98.52,'Sun-Synch' from dual union all
select 29541,'029541','2006-050F','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2006-11-16 00:00:00','2006-11-14 00:00:00',92.9,327,507,98.5,'Sun-Synch' from dual union all
select 29542,'029542','2006-050G','DMSP F-17 solar array band','',68222,'Reentered','2007-08-04 00:00:00','2006-11-13 00:00:00',97.98,487,838,98.51,'Sun-Synch' from dual union all
select 29543,'029543','2006-050H','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2012-01-08 00:00:00','2006-11-14 00:00:00',101.57,817,849,98.83,'Sun-Synch' from dual union all
select 29544,'029544','2006-050J','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-10-06 00:00:00','2006-11-14 00:00:00',100.36,699,853,98.66,'Sun-Synch' from dual union all
select 29545,'029545','2006-050K','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-10-07 00:00:00','2006-11-14 00:00:00',101.59,819,849,98.81,'Sun-Synch' from dual union all
select 29546,'029546','2006-050L','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-05-12 00:00:00','2006-11-14 00:00:00',101.53,814,849,98.78,'Sun-Synch' from dual union all
select 29547,'029547','2006-050M','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-10-11 00:00:00','2006-11-13 00:00:00',101.57,819,847,98.81,'Sun-Synch' from dual union all
select 29548,'029548','2006-050N','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-04-03 00:00:00','2006-11-13 00:00:00',101.59,819,849,98.83,'Sun-Synch' from dual union all
select 29549,'029549','2006-050P','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-08-11 00:00:00','2006-11-13 00:00:00',100.54,718,851,98.94,'Sun-Synch' from dual union all
select 2955,'002955','1967-089D','deb R-36','',15344,'Reentered','1967-09-19 00:00:00','1967-09-19 00:00:00',87.73,110,215,49.6,'Intermediate' from dual union all
select 29550,'029550','2006-050Q','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-10-22 00:00:00','2006-11-13 00:00:00',100.61,715,860,98.67,'Sun-Synch' from dual union all
select 29551,'029551','2006-050R','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-05-25 00:00:00','2006-11-14 00:00:00',100.68,718,864,98.83,'Sun-Synch' from dual union all
select 29552,'029552','2006-050S','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-11-08 00:00:00','2006-11-14 00:00:00',100.8,737,856,98.53,'Sun-Synch' from dual union all
select 29553,'029553','2006-050T','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-11-18 00:00:00','2006-11-14 00:00:00',100.74,739,848,99.02,'Sun-Synch' from dual union all
select 29554,'029554','2006-050U','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-03-29 00:00:00','2006-11-14 00:00:00',100.9,754,849,98.82,'Sun-Synch' from dual union all
select 29555,'029555','2006-050V','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-04-17 00:00:00','2006-11-13 00:00:00',101.1,772,850,98.77,'Sun-Synch' from dual union all
select 29556,'029556','2006-050W','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-04-25 00:00:00','2006-11-13 00:00:00',101.14,774,852,98.89,'Sun-Synch' from dual union all
select 29557,'029557','2006-050X','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-04-08 00:00:00','2006-11-13 00:00:00',101.48,811,847,98.84,'Sun-Synch' from dual union all
select 29558,'029558','2006-050Y','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-11-19 00:00:00','2006-11-13 00:00:00',101.67,829,847,98.81,'Sun-Synch' from dual union all
select 29559,'029559','2006-050Z','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-05-15 00:00:00','2006-11-14 00:00:00',101.05,768,849,98.8,'Sun-Synch' from dual union all
select 2956,'002956','1967-086D','deb S1M','',15341,'Reentered','1967-09-21 00:00:00','1967-09-20 00:00:00',98.39,142,1222,82.1,'Intermediate' from dual union all
select 29560,'029560','2006-050AA','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-03-03 00:00:00','2006-11-14 00:00:00',99.93,666,845,99.11,'Sun-Synch' from dual union all
select 29561,'029561','2006-050AB','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2009-11-16 00:00:00','2006-11-14 00:00:00',100.07,676,847,98.56,'Sun-Synch' from dual union all
select 29562,'029562','2006-050AC','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-10-18 00:00:00','2006-11-14 00:00:00',100.87,750,849,98.9,'Sun-Synch' from dual union all
select 29563,'029563','2006-050AD','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-10-01 00:00:00','2006-11-13 00:00:00',101.63,821,850,98.79,'Sun-Synch' from dual union all
select 29564,'029564','2006-050AE','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-10-23 00:00:00','2006-11-13 00:00:00',101.61,821,849,98.81,'Sun-Synch' from dual union all
select 29565,'029565','2006-050AF','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2006-12-01 00:00:00','2006-11-13 00:00:00',96.41,372,802,98.57,'Sun-Synch' from dual union all
select 29566,'029566','2006-050AG','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2009-10-31 00:00:00','2006-11-14 00:00:00',99.96,664,850,98.63,'Sun-Synch' from dual union all
select 29567,'029567','2006-050AH','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-02-03 00:00:00','2006-11-13 00:00:00',100.04,673,848,98.71,'Sun-Synch' from dual union all
select 29568,'029568','2006-050AJ','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-03-11 00:00:00','2006-11-13 00:00:00',100.17,684,849,98.75,'Sun-Synch' from dual union all
select 29569,'029569','2006-050AK','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-05-17 00:00:00','2006-11-13 00:00:00',100.35,700,851,98.64,'Sun-Synch' from dual union all
select 2957,'002957','1967-086E','deb S1M','',15341,'Reentered','1968-04-06 00:00:00','1967-10-13 00:00:00',102.12,260,1458,82.15,'Intermediate' from dual union all
select 29570,'029570','2006-050AL','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-10-11 00:00:00','2006-11-13 00:00:00',100.78,733,859,98.69,'Sun-Synch' from dual union all
select 29571,'029571','2006-050AM','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-02-09 00:00:00','2006-11-13 00:00:00',100.75,741,847,98.8,'Sun-Synch' from dual union all
select 29572,'029572','2006-050AN','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-03-02 00:00:00','2006-11-13 00:00:00',101.03,766,849,98.76,'Sun-Synch' from dual union all
select 29573,'029573','2006-050AP','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-01-10 00:00:00','2006-11-14 00:00:00',100.89,752,849,98.71,'Sun-Synch' from dual union all
select 29574,'029574','2006-050AQ','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-01-08 00:00:00','2006-11-14 00:00:00',100.95,755,853,98.82,'Sun-Synch' from dual union all
select 29575,'029575','2006-050AR','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-03-13 00:00:00','2006-11-14 00:00:00',100.52,715,851,98.74,'Sun-Synch' from dual union all
select 29576,'029576','2006-050AS','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2009-02-15 00:00:00','2006-11-13 00:00:00',99.87,658,847,98.63,'Sun-Synch' from dual union all
select 29577,'029577','2006-050AT','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-06-27 00:00:00','2006-11-13 00:00:00',101.26,776,861,98.61,'Sun-Synch' from dual union all
select 29578,'029578','2006-050AU','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-03-14 00:00:00','2006-11-14 00:00:00',101.26,785,852,98.81,'Sun-Synch' from dual union all
select 29579,'029579','2006-050AV','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-05-08 00:00:00','2006-11-14 00:00:00',101.34,793,852,98.84,'Sun-Synch' from dual union all
select 2958,'002958','1967-086F','deb S1M','',15341,'Reentered','1975-03-12 00:00:00','1967-10-15 00:00:00',104.26,373,1547,82.29,'Intermediate' from dual union all
select 29580,'029580','2006-050AW','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-03-06 00:00:00','2006-11-13 00:00:00',101.04,767,849,98.76,'Sun-Synch' from dual union all
select 29581,'029581','2006-050AX','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-10-24 00:00:00','2006-11-14 00:00:00',101.11,771,852,98.74,'Sun-Synch' from dual union all
select 29582,'029582','2006-050AY','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-08-21 00:00:00','2006-11-13 00:00:00',100.97,755,855,98.75,'Sun-Synch' from dual union all
select 29583,'029583','2006-050AZ','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-01-04 00:00:00','2006-11-14 00:00:00',101.17,775,854,98.72,'Sun-Synch' from dual union all
select 29584,'029584','2006-050BA','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-03-18 00:00:00','2006-11-13 00:00:00',99.97,668,847,98.67,'Sun-Synch' from dual union all
select 29585,'029585','2006-050BB','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-09-27 00:00:00','2006-11-14 00:00:00',100.44,698,861,98.73,'Sun-Synch' from dual union all
select 29586,'029586','2006-050BC','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-04-05 00:00:00','2006-11-13 00:00:00',100.44,704,854,98.61,'Sun-Synch' from dual union all
select 29587,'029587','2006-050BD','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-02-25 00:00:00','2006-11-14 00:00:00',101.03,765,850,98.87,'Sun-Synch' from dual union all
select 29588,'029588','2006-050BE','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-04-01 00:00:00','2006-11-14 00:00:00',101.06,769,849,98.78,'Sun-Synch' from dual union all
select 29589,'029589','2006-050BF','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-05-23 00:00:00','2006-11-14 00:00:00',101.17,775,854,98.78,'Sun-Synch' from dual union all
select 2959,'002959','1967-086G','deb S1M','',15341,'Reentered','1968-04-02 00:00:00','1967-10-13 00:00:00',101.9,342,1355,81.84,'Intermediate' from dual union all
select 29590,'029590','2006-050BG','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-08-31 00:00:00','2006-11-13 00:00:00',101.31,790,852,98.81,'Sun-Synch' from dual union all
select 29591,'029591','2006-050BH','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-09-27 00:00:00','2006-11-13 00:00:00',101.3,790,851,98.77,'Sun-Synch' from dual union all
select 29592,'029592','2006-050BJ','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-04-15 00:00:00','2006-11-13 00:00:00',101.53,811,852,98.82,'Sun-Synch' from dual union all
select 29593,'029593','2006-050BK','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-09-07 00:00:00','2006-11-13 00:00:00',101.58,820,847,98.8,'Sun-Synch' from dual union all
select 29594,'029594','2006-050BL','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-07-29 00:00:00','2006-11-13 00:00:00',100.41,703,853,98.69,'Sun-Synch' from dual union all
select 29595,'029595','2006-050BM','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-11-29 00:00:00','2006-11-14 00:00:00',101.01,765,847,98.75,'Sun-Synch' from dual union all
select 29596,'029596','2006-050BN','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-05-31 00:00:00','2006-11-14 00:00:00',101.41,791,860,98.78,'Sun-Synch' from dual union all
select 29597,'029597','2006-050BP','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-10-19 00:00:00','2006-11-14 00:00:00',101.49,799,860,98.75,'Sun-Synch' from dual union all
select 29598,'029598','2006-050BQ','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-08-22 00:00:00','2006-11-14 00:00:00',101.51,814,847,98.79,'Sun-Synch' from dual union all
select 29599,'029599','2006-050BR','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2011-10-14 00:00:00','2006-11-13 00:00:00',101.55,814,851,98.82,'Sun-Synch' from dual union all
select 296,'000296','62 Tau 2','Atlas D 107D','Atlas D 107D',8203,'Reentered','1962-05-25 00:00:00','1962-05-24 00:00:00',88.63,154,260,32.5,'Intermediate' from dual union all
select 2960,'002960','1967-086H','deb S1M','',15341,'Reentered','1967-10-03 00:00:00','1967-09-25 00:00:00',94.23,181,782,82.09,'Intermediate' from dual union all
select 29600,'029600','2006-050BS','deb Delta 320','deb DCSS-4 F06',68222,'In Earth orbit','','2006-11-17 00:00:00',101.87,841,854,98.79,'Sun-Synch' from dual union all
select 29601,'029601','2006-052A','GPS IIR-16(M)','Navstar GPS SVN 58',68224,'In Earth orbit','','2007-01-17 00:00:00',717.96,20087,20277,55.07,'Medium' from dual union all
select 29602,'029602','2006-052B','Star 48B','Star 48B',68224,'Reentered','2011-06-21 00:00:00','2007-01-17 00:00:00',351.21,180,20043,39.93,'Highly Ellip' from dual union all
select 29603,'029603','2006-052C','Delta 321','Delta SSPS AJ-10-118K',68224,'Reentered','2007-05-06 00:00:00','2006-11-25 00:00:00',98.76,200,1199,32.66,'Intermediate' from dual union all
select 29604,'029604','2006-039D','deb Kosmos-2423','',68211,'Reentered','2006-11-18 00:00:00','2006-11-18 00:00:00',88.88,182,257,64.72,'Intermediate' from dual union all
select 29605,'029605','2006-039E','deb Kosmos-2423','',68211,'Reentered','2006-11-18 00:00:00','2006-11-18 00:00:00',89.03,180,273,65.06,'Intermediate' from dual union all
select 29606,'029606','2006-039F','deb Kosmos-2423','',68211,'Reentered','2006-11-20 00:00:00','2006-11-18 00:00:00',90.72,195,425,64.78,'Intermediate' from dual union all
select 29607,'029607','2006-039G','deb Kosmos-2423','',68211,'Reentered','2006-11-19 00:00:00','2006-11-18 00:00:00',88.69,177,243,64.75,'Intermediate' from dual union all
select 29608,'029608','2006-039H','deb Kosmos-2423','',68211,'Reentered','2006-11-18 00:00:00','2006-11-18 00:00:00',88.55,174,231,64.77,'Intermediate' from dual union all
select 29609,'029609','2006-039J','deb Kosmos-2423','',68211,'Reentered','2006-11-25 00:00:00','2006-11-18 00:00:00',94.14,194,760,64.51,'Intermediate' from dual union all
select 2961,'002961','1967-086J','deb S1M','',15341,'Reentered','1967-10-06 00:00:00','1967-09-23 00:00:00',97.41,153,1117,82.2,'Intermediate' from dual union all
select 29610,'029610','2006-039K','deb Kosmos-2423','',68211,'Reentered','2006-11-23 00:00:00','2006-11-18 00:00:00',94.16,185,771,64.69,'Intermediate' from dual union all
select 29611,'029611','2006-039L','deb Kosmos-2423','',68211,'Reentered','2006-11-18 00:00:00','2006-11-18 00:00:00',89.32,183,298,64.76,'Intermediate' from dual union all
select 29612,'029612','2006-039M','deb Kosmos-2423','',68211,'Reentered','2006-11-20 00:00:00','2006-11-18 00:00:00',88.71,187,235,64.79,'Intermediate' from dual union all
select 29613,'029613','2006-039N','deb Kosmos-2423','',68211,'Reentered','2006-11-18 00:00:00','2006-11-18 00:00:00',88.62,175,238,64.81,'Intermediate' from dual union all
select 29614,'029614','2006-039P','deb Kosmos-2423','',68211,'Reentered','2006-11-20 00:00:00','2006-11-18 00:00:00',88.74,185,240,64.81,'Intermediate' from dual union all
select 29615,'029615','2006-039Q','deb Kosmos-2423','',68211,'Reentered','2006-11-18 00:00:00','2006-11-18 00:00:00',89.28,186,291,64.87,'Intermediate' from dual union all
select 29616,'029616','2006-039R','deb Kosmos-2423','',68211,'Reentered','2006-11-19 00:00:00','2006-11-18 00:00:00',90.4,194,395,64.73,'Intermediate' from dual union all
select 29617,'029617','2006-039S','deb Kosmos-2423','',68211,'Reentered','2006-11-24 00:00:00','2006-11-18 00:00:00',94.13,193,760,64.83,'Intermediate' from dual union all
select 29618,'029618','2006-039T','deb Kosmos-2423','',68211,'Reentered','2006-11-25 00:00:00','2006-11-18 00:00:00',89.39,173,316,64.69,'Intermediate' from dual;
insert into SATELLITE
select 29619,'029619','2006-039U','deb Kosmos-2423','Spurious',68211,'Reentered','2006-11-26 00:00:00','2006-11-18 00:00:00',94.19,197,762,64.83,'Intermediate' from dual union all
select 2962,'002962','1967-091A','Kosmos-179','OGCh No. 14',15346,'Deorbited','1967-09-22 00:00:00','1967-09-22 00:00:00',87.95,139,207,49.57,'Intermediate' from dual union all
select 29620,'029620','2006-039V','deb Kosmos-2423','',68211,'Reentered','2006-11-18 00:00:00','2006-11-18 00:00:00',88.52,163,240,65.04,'Intermediate' from dual union all
select 29621,'029621','2006-039W','deb Kosmos-2423','',68211,'Reentered','2006-11-24 00:00:00','2006-11-18 00:00:00',93.93,194,739,65.09,'Intermediate' from dual union all
select 29622,'029622','2006-039X','deb Kosmos-2423','',68211,'Reentered','2006-11-18 00:00:00','2006-11-18 00:00:00',88.9,171,270,64.77,'Intermediate' from dual union all
select 29623,'029623','2006-039Y','deb Kosmos-2423','',68211,'Reentered','2006-11-20 00:00:00','2006-11-18 00:00:00',90.75,192,431,65.03,'Intermediate' from dual union all
select 29624,'029624','2006-039Z','deb Kosmos-2423','',68211,'Reentered','2006-11-18 00:00:00','2006-11-18 00:00:00',88.87,180,257,65.1,'Intermediate' from dual union all
select 29625,'029625','2006-039AA','deb Kosmos-2423','',68211,'Reentered','2006-11-18 00:00:00','2006-11-18 00:00:00',88.55,165,240,64.71,'Intermediate' from dual union all
select 29626,'029626','2006-039AB','deb Kosmos-2423','',68211,'Reentered','2006-11-18 00:00:00','2006-11-18 00:00:00',89.23,189,283,65.03,'Intermediate' from dual union all
select 29627,'029627','2006-039AC','deb Kosmos-2423','',68211,'Reentered','2006-11-21 00:00:00','2006-11-18 00:00:00',94.1,178,772,64.72,'Intermediate' from dual union all
select 29628,'029628','2006-039AD','deb Kosmos-2423','',68211,'Reentered','2006-11-25 00:00:00','2006-11-18 00:00:00',95.04,188,853,64.71,'Intermediate' from dual union all
select 29629,'029629','2006-039AE','deb Kosmos-2423','',68211,'Reentered','2006-11-21 00:00:00','2006-11-18 00:00:00',94.67,175,830,64.75,'Intermediate' from dual union all
select 2963,'002963','1967-091B','Kosmos-179 Adapter','OGCh Adapter',15346,'Reentered','1967-09-23 00:00:00','1967-09-22 00:00:00',87.79,117,214,49.52,'Intermediate' from dual union all
select 29630,'029630','2006-039AF','deb Kosmos-2423','',68211,'Reentered','2006-11-23 00:00:00','2006-11-18 00:00:00',95.19,193,863,64.96,'Intermediate' from dual union all
select 29631,'029631','2006-039AG','deb Kosmos-2423','',68211,'Reentered','2006-12-04 00:00:00','2006-11-18 00:00:00',94.99,189,847,64.4,'Intermediate' from dual union all
select 29632,'029632','2006-050BT','DMSP F-17 solar array band','',68222,'Reentered','2007-08-09 00:00:00','2006-11-28 00:00:00',100.28,690,854,98.88,'Sun-Synch' from dual union all
select 29633,'029633','2006-050BU','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2009-04-07 00:00:00','2006-11-26 00:00:00',100.32,704,844,98.85,'Sun-Synch' from dual union all
select 29634,'029634','2006-039AH','deb Kosmos-2423','',68211,'Reentered','2006-11-29 00:00:00','2006-11-28 00:00:00',90.81,182,446,65.24,'Intermediate' from dual union all
select 29635,'029635','2006-039AJ','deb Kosmos-2423','',68211,'Reentered','2006-11-30 00:00:00','2006-11-28 00:00:00',92.5,188,607,65.17,'Intermediate' from dual union all
select 29636,'029636','2006-046F','deb CZ-4B','',68218,'Reentered','2014-04-27 00:00:00','2006-11-28 00:00:00',96.96,526,701,97.75,'Sun-Synch' from dual union all
select 29637,'029637','2006-050BV','deb Delta 320','deb DCSS-4 F06',68222,'In Earth orbit','','2006-11-29 00:00:00',101.87,841,854,98.78,'Sun-Synch' from dual union all
select 29638,'029638','1998-067AK','EVA debris (towel)','',66503,'Reentered','2006-12-15 00:00:00','2006-11-30 00:00:00',90.89,313,324,51.63,'Intermediate' from dual union all
select 29639,'029639','1998-067AL','EVA debris (towel)','',66503,'Reentered','2006-12-16 00:00:00','2006-11-30 00:00:00',90.94,315,327,51.63,'Intermediate' from dual union all
select 2964,'002964','1967-091C','8S692','8S692',15346,'Reentered','1967-09-22 00:00:00','1967-09-22 00:00:00',87.45,141,156,49.4,'Intermediate' from dual union all
select 29640,'029640','2006-053A','Feng Yun 2D','fengyun erhao D xing',68225,'In Earth orbit','','2007-02-06 00:00:00',1436.16,35787,35789,2.48,'Stationary' from dual union all
select 29641,'029641','2006-053B','CZ-3A Stage 3','CZ-3A Stage 3',68225,'Reentered','2006-12-09 00:00:00','2006-12-09 00:00:00',638.34,226,36134,24.79,'GEO Transfer' from dual union all
select 29642,'029642','2006-053C','FG-36 AKM','Shanxi AKM',68225,'In Earth orbit','','2007-02-06 00:00:00',1449.34,35591,36500,2.54,'Drift GEO' from dual union all
select 29643,'029643','2006-054A','WildBlue 1','WildBlue 1',68226,'In Earth orbit','','2007-02-08 00:00:00',1436.09,35781,35792,.03,'Stationary' from dual union all
select 29644,'029644','2006-054B','AMC 18','AMC 18',68226,'In Earth orbit','','2007-02-07 00:00:00',1436.09,35780,35793,.02,'Stationary' from dual union all
select 29645,'029645','2006-054C','Sylda 5C','Sylda 5',68226,'Reentered','2016-07-21 00:00:00','2007-02-09 00:00:00',628.83,254,35617,2.29,'GEO Transfer' from dual union all
select 29646,'029646','2006-054D','ESC-A 534','Ariane 5 ESC-A',68226,'In Earth orbit','','2007-02-08 00:00:00',627.68,259,35553,1.78,'GEO Transfer' from dual union all
select 29647,'029647','2006-055A','Discovery (STS-116)','OV-103',68227,'Landed','2006-12-22 00:00:00','2006-12-12 00:00:00',91.37,326,357,51.63,'Intermediate' from dual union all
select 29648,'029648','2006-056A','Measat 3','Measat 3',68228,'In Earth orbit','','2007-02-13 00:00:00',1436.06,35777,35795,.02,'Stationary' from dual union all
select 29649,'029649','2006-056B','Briz-M DTB','Briz-M torus tank',68228,'In Earth orbit','','2007-02-10 00:00:00',299.58,328,16578,49.52,'Highly Ellip' from dual union all
select 2965,'002965','1967-092A','NNS O-14','NNS O-14',15347,'In Earth orbit','','1967-11-01 00:00:00',106.76,1038,1115,89.29,'Polar' from dual union all
select 29650,'029650','2006-056C','Briz-M No. 88518','Briz-M',68228,'In Earth orbit','','2007-02-10 00:00:00',773.5,7451,35618,16.96,'Highly Ellip' from dual union all
select 29651,'029651','2006-057A','USA 193','NROL-21',68229,'Exploded','2008-02-21 00:00:00','2006-12-20 00:00:00',91.72,351,367,58.5,'Intermediate' from dual union all
select 29652,'029652','2006-057B','Delta 322','Delta SSPS AJ-10-118K',68229,'Reentered','2006-12-17 00:00:00','2006-12-20 00:00:00',91.72,351,367,58.5,'Intermediate' from dual union all
select 29653,'029653','2006-058A','Tacsat 2','Tacsat 2',68230,'Reentered','2011-02-05 00:00:00','2006-12-18 00:00:00',92.92,413,423,40.01,'Intermediate' from dual union all
select 29654,'029654','2006-058B','Minotaur Stage 4','Orion 38',68230,'Reentered','2011-04-02 00:00:00','2006-12-18 00:00:00',92.88,413,418,40.02,'Intermediate' from dual union all
select 29655,'029655','2006-058C','Genesat-1','',68230,'Reentered','2010-08-04 00:00:00','2006-12-18 00:00:00',92.88,413,418,40.03,'Intermediate' from dual union all
select 29656,'029656','2006-059A','Kiku-8','ETS-8',68231,'In Earth orbit','','2007-02-21 00:00:00',1436.16,35775,35801,.03,'Stationary' from dual union all
select 29657,'029657','2006-059B','H-2A F11 Stage 2','H-2A Stage 2',68231,'In Earth orbit','','2007-02-16 00:00:00',609.13,217,34633,28.36,'GEO Transfer' from dual union all
select 29658,'029658','2006-060A','SAR-Lupe 1','SAR-Lupe 1',68232,'In Earth orbit','','2006-12-21 00:00:00',94.31,467,504,98.16,'Sun-Synch' from dual union all
select 29659,'029659','2006-060B','S3M','S3M',68232,'Reentered','2014-03-30 00:00:00','2006-12-21 00:00:00',94.31,463,508,98.16,'Sun-Synch' from dual union all
select 2966,'002966','1967-093A','Kosmos-180','Zenit-2 No. 54',15348,'Landed','1967-10-04 00:00:00','1967-09-28 00:00:00',89.97,203,343,72.88,'Intermediate' from dual union all
select 29660,'029660','2006-055B','MEPSI 2A','MEPSI 2A/2B',68227,'Reentered','2007-03-08 00:00:00','2006-12-21 00:00:00',90.95,311,331,51.64,'Intermediate' from dual union all
select 29661,'029661','2006-055C','Navy-OSCAR-60','RAFT-1',68227,'Reentered','2007-05-30 00:00:00','2006-12-21 00:00:00',90.97,313,332,51.65,'Intermediate' from dual union all
select 29662,'029662','2006-055D','NMARS','MARScom',68227,'Reentered','2007-05-06 00:00:00','2006-12-21 00:00:00',90.98,313,333,51.64,'Intermediate' from dual union all
select 29663,'029663','2006-055E','ICU Cylinder 1','',68227,'Reentered','2007-04-18 00:00:00','2006-12-22 00:00:00',90.99,312,335,51.63,'Intermediate' from dual union all
select 29664,'029664','2006-055F','Navy-OSCAR-61 (ANDE-RR)','Mock ANDE Active (Risk R',68227,'Reentered','2007-12-25 00:00:00','2006-12-22 00:00:00',91,312,335,51.64,'Intermediate' from dual union all
select 29665,'029665','2006-055G','ICU Cylinder 2','Internal Cargo Unit',68227,'Reentered','2007-02-09 00:00:00','2006-12-22 00:00:00',91.01,312,337,51.63,'Intermediate' from dual union all
select 29666,'029666','2006-055H','ICU Avionics Deck','',68227,'Reentered','2007-04-24 00:00:00','2006-12-22 00:00:00',91,312,335,51.64,'Intermediate' from dual union all
select 29667,'029667','2006-055J','Navy-OSCAR-62 (ANDE-FCL)','ANDE Fence Cal',68227,'Reentered','2008-05-25 00:00:00','2006-12-22 00:00:00',91,312,335,51.64,'Intermediate' from dual union all
select 29668,'029668','2006-061A','Meridian No. 11L','Meridian No. 11L',68233,'In Earth orbit','','2007-02-22 00:00:00',717.72,1062,39289,62.83,'Molniya' from dual union all
select 29669,'029669','2006-061B','Fregat No. 1012','Fregat 14S44 No. 1012',68233,'In Earth orbit','','2007-02-22 00:00:00',726.45,1098,39684,62.82,'Molniya' from dual union all
select 2967,'002967','1967-092B','Altair 2 X-258E6 ABL-135','X-258RHE6',15347,'In Earth orbit','','1967-12-03 00:00:00',106.74,1038,1113,89.29,'Polar' from dual union all
select 29670,'029670','2006-062A','Kosmos-2424','Uragan-M No. 715',68234,'In Earth orbit','','2007-02-24 00:00:00',675.69,19063,19195,64.84,'Medium' from dual union all
select 29671,'029671','2006-062B','Kosmos-2425','Uragan-M No. 716',68234,'In Earth orbit','','2007-02-24 00:00:00',675.71,19080,19179,64.86,'Medium' from dual union all
select 29672,'029672','2006-062C','Kosmos-2426','Uragan-M No. 717',68234,'In Earth orbit','','2007-02-24 00:00:00',675.71,19086,19173,64.84,'Medium' from dual union all
select 29673,'029673','2006-062D','Blok DM-2 No. 108L','11S861 No. 108L',68234,'In Earth orbit','','2007-02-24 00:00:00',674.76,19065,19146,64.85,'Medium' from dual union all
select 29674,'029674','2006-062E','8S812','8S812',68234,'Reentered','2006-12-26 00:00:00','2006-12-26 00:00:00',87.46,134,163,64.72,'Intermediate' from dual union all
select 29675,'029675','2006-062F','Perekhodnik','Sredniy perekhodnik',68234,'Reentered','2006-12-26 00:00:00','2006-12-26 00:00:00',87.35,140,147,64.8,'Intermediate' from dual union all
select 29676,'029676','2005-049F','SEVIRI Entry Baffle Cover','',68058,'In Earth orbit','','2005-12-29 00:00:00',1434.86,35494,36031,1.84,'Drift GEO' from dual union all
select 29677,'029677','1998-067AN','EVA camera','',66503,'Reentered','2007-04-29 00:00:00','2006-12-22 00:00:00',91,312,335,51.64,'Intermediate' from dual union all
select 29678,'029678','2006-063A','COROT','COROT',68235,'In Earth orbit','','2007-01-03 00:00:00',102.99,895,906,90.01,'Polar' from dual union all
select 29679,'029679','2006-063B','Blok-I 14S54','14S54',68235,'Reentered','2007-01-04 00:00:00','2006-12-29 00:00:00',88.75,208,218,90.01,'Polar' from dual union all
select 2968,'002968','1967-093B','11A57I','11A57I',15348,'Reentered','1967-10-05 00:00:00','1967-09-28 00:00:00',89.62,191,321,72.89,'Intermediate' from dual union all
select 29680,'029680','2006-062G','SOZ','SOZ',68234,'In Earth orbit','','2007-02-23 00:00:00',340.3,416,19119,64.91,'Highly Ellip' from dual union all
select 29681,'029681','1998-067AM','EVA debris','Tool socket',66503,'Reentered','2007-06-23 00:00:00','2007-01-03 00:00:00',91.25,321,351,51.63,'Intermediate' from dual union all
select 29682,'029682','2006-062H','SOZ','SOZ',68234,'In Earth orbit','','2007-02-24 00:00:00',340.23,412,19119,64.79,'Highly Ellip' from dual union all
select 29683,'029683','1989-089CJ','deb Delta 189','',62366,'Reentered','2006-12-14 00:00:00','2006-12-14 00:00:00',93.14,414,443,96.73,'Sun-Synch' from dual union all
select 29684,'029684','1989-089CK','deb Delta 189','',62366,'Reentered','2006-12-15 00:00:00','2006-12-15 00:00:00',92.21,333,432,96.81,'Sun-Synch' from dual union all
select 29685,'029685','1989-089CL','deb Delta 189','',62366,'Reentered','2006-12-15 00:00:00','2006-12-15 00:00:00',93.95,401,534,96.61,'Sun-Synch' from dual union all
select 29686,'029686','1989-089CM','deb Delta 189','',62366,'Reentered','2006-12-18 00:00:00','2006-12-15 00:00:00',96.73,463,742,96.83,'Sun-Synch' from dual union all
select 29687,'029687','1989-089CN','deb Delta 189','',62366,'Reentered','2006-12-19 00:00:00','2006-12-08 00:00:00',99.32,631,822,97.02,'Sun-Synch' from dual union all
select 29688,'029688','1989-089CP','deb Delta 189','',62366,'Reentered','2006-12-19 00:00:00','2006-12-09 00:00:00',99.29,630,820,96.99,'Sun-Synch' from dual union all
select 29689,'029689','1989-089CQ','deb Delta 189','1.3m2 piece',62366,'Reentered','2006-12-16 00:00:00','2006-12-15 00:00:00',96.28,477,683,96.9,'Sun-Synch' from dual union all
select 2969,'002969','1967-094A','INTELSAT II F-4','INTELSAT II F-4',15349,'In Earth orbit','','1969-01-13 00:00:00',1436.06,35761,35811,.32,'Stationary' from dual union all
select 29690,'029690','1989-089CR','deb Delta 189','',62366,'Reentered','2007-01-16 00:00:00','2006-12-06 00:00:00',99.45,642,822,97.05,'Sun-Synch' from dual union all
select 29691,'029691','1989-089CS','deb Delta 189','',62366,'Reentered','2006-12-15 00:00:00','2006-12-15 00:00:00',94.94,278,754,96.61,'Sun-Synch' from dual union all
select 29692,'029692','1989-089CT','deb Delta 189','',62366,'Reentered','2006-12-19 00:00:00','2006-12-15 00:00:00',97.32,493,767,97.02,'Sun-Synch' from dual union all
select 29693,'029693','1989-089CU','deb Delta 189','',62366,'Reentered','2006-12-21 00:00:00','2006-12-15 00:00:00',95.62,346,752,96.84,'Sun-Synch' from dual union all
select 29694,'029694','1989-089CV','deb Delta 189','',62366,'Reentered','2006-12-18 00:00:00','2006-12-15 00:00:00',97.33,468,794,96.77,'Sun-Synch' from dual union all
select 29695,'029695','1989-089CW','deb Delta 189','',62366,'Reentered','2006-12-19 00:00:00','2006-12-15 00:00:00',96.85,437,779,96.87,'Sun-Synch' from dual union all
select 29696,'029696','1989-089CX','deb Delta 189','',62366,'Reentered','2006-12-19 00:00:00','2006-12-10 00:00:00',99.27,612,835,96.98,'Sun-Synch' from dual union all
select 29697,'029697','1989-089CY','deb Delta 189','',62366,'Reentered','2006-12-15 00:00:00','2006-12-15 00:00:00',99.9,696,811,97.07,'Sun-Synch' from dual union all
select 29698,'029698','1989-089CZ','deb Delta 189','',62366,'Reentered','2006-12-20 00:00:00','2006-12-15 00:00:00',97.78,521,784,96.87,'Sun-Synch' from dual union all
select 29699,'029699','1989-089DA','deb Delta 189','',62366,'Reentered','2006-12-15 00:00:00','2006-12-15 00:00:00',92.26,102,669,96.83,'Sun-Synch' from dual union all
select 297,'000297','62 Upsilon 1','Kosmos-5','2MS No. 2',8205,'Reentered','1963-05-02 00:00:00','1962-07-02 00:00:00',102,207,1500,49.06,'Intermediate' from dual union all
select 2970,'002970','1967-094B','Delta 52','DSV-3E-3 S/N 20215',15349,'Reentered','1967-12-26 00:00:00','1967-11-14 00:00:00',91.7,237,479,28.76,'Intermediate' from dual union all
select 29700,'029700','1989-089DB','deb Delta 189','',62366,'Reentered','2006-12-22 00:00:00','2006-12-15 00:00:00',97.23,510,743,96.93,'Sun-Synch' from dual union all
select 29701,'029701','1989-089DC','deb Delta 189','',62366,'Reentered','2006-12-19 00:00:00','2006-12-15 00:00:00',96.55,435,751,97.04,'Sun-Synch' from dual union all
select 29702,'029702','1989-089DD','deb Delta 189 major piece','26m2 piece',62366,'Reentered','2006-12-19 00:00:00','2006-12-15 00:00:00',96.64,455,741,96.83,'Sun-Synch' from dual union all
select 29703,'029703','1989-089DE','deb Delta 189','',62366,'Reentered','2006-12-15 00:00:00','2006-12-15 00:00:00',95.3,396,670,96.78,'Sun-Synch' from dual union all
select 29704,'029704','1989-089DF','deb Delta 189','',62366,'Reentered','2006-12-16 00:00:00','2006-12-15 00:00:00',92.61,219,586,97.03,'Sun-Synch' from dual union all
select 29705,'029705','1989-089DG','deb Delta 189','',62366,'Reentered','2006-12-16 00:00:00','2006-12-15 00:00:00',95.42,413,665,96.85,'Sun-Synch' from dual union all
select 29706,'029706','1989-089DH','deb Delta 189','',62366,'Reentered','2006-12-24 00:00:00','2006-12-15 00:00:00',97.3,530,729,96.78,'Sun-Synch' from dual union all
select 29707,'029707','1989-089DJ','deb Delta 189','',62366,'Reentered','2006-12-14 00:00:00','2006-12-15 00:00:00',92.33,217,561,96.9,'Sun-Synch' from dual union all
select 29708,'029708','2006-050BW','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2007-08-05 00:00:00','2007-01-07 00:00:00',98.32,591,766,98.62,'Sun-Synch' from dual;
insert into SATELLITE
select 29709,'029709','2007-001A','LAPAN-TUBSAT','LAPAN-A1',68343,'In Earth orbit','','2007-01-12 00:00:00',97.3,619,640,97.94,'Sun-Synch' from dual union all
select 2971,'002971','1967-094C','FW4D S/N 41104','[UNID016] RCS 1.2',15349,'In Earth orbit','','1967-09-28 00:00:00',625.82,308,35408,26.48,'GEO Transfer' from dual union all
select 29710,'029710','2007-001B','Cartosat-2','Cartosat-2',68343,'In Earth orbit','','2007-01-12 00:00:00',97.28,620,637,97.94,'Sun-Synch' from dual union all
select 29711,'029711','2007-001C','SRE-1','Space Recovery Experimen',68343,'Landed','2007-01-22 00:00:00','2007-01-12 00:00:00',97.33,622,639,97.95,'Sun-Synch' from dual union all
select 29712,'029712','2007-001D','Peheunsat-OSCAR-63','Pehuensat-1',68343,'In Earth orbit','','2007-01-12 00:00:00',97.35,613,651,97.89,'Sun-Synch' from dual union all
select 29713,'029713','2007-001E','PS4','PS4',68343,'In Earth orbit','','2007-01-12 00:00:00',97.32,621,639,97.94,'Sun-Synch' from dual union all
select 29714,'029714','2007-002A','Progress M-59','Progress 7K-TGM No. 359',68344,'Deorbited','2007-08-01 00:00:00','2007-01-18 00:00:00',88.58,186,223,51.63,'Intermediate' from dual union all
select 29715,'029715','2007-002B','11S510','11S510',68344,'Reentered','2007-01-20 00:00:00','2007-01-18 00:00:00',88.5,185,215,51.63,'Intermediate' from dual union all
select 29716,'029716','1999-025E','deb FY-1C','',66706,'Reentered','2007-02-06 00:00:00','2007-01-18 00:00:00',93.87,167,761,99.44,'Sun-Synch' from dual union all
select 29717,'029717','1999-025F','deb FY-1C','',66706,'Reentered','2007-03-23 00:00:00','2007-01-18 00:00:00',95.12,237,812,100.39,'Sun-Synch' from dual union all
select 29718,'029718','1999-025G','deb FY-1C','',66706,'Reentered','2007-04-15 00:00:00','2007-01-18 00:00:00',95.47,245,838,99.26,'Sun-Synch' from dual union all
select 29719,'029719','1999-025H','deb FY-1C','',66706,'Reentered','2007-07-30 00:00:00','2007-01-18 00:00:00',96.06,267,873,99.02,'Sun-Synch' from dual union all
select 2972,'002972','1967-094D','deb Delta 52','',15349,'Reentered','1967-10-27 00:00:00','1967-10-08 00:00:00',92.68,241,571,28.76,'Intermediate' from dual union all
select 29720,'029720','1999-025J','deb FY-1C','',66706,'Reentered','2007-05-03 00:00:00','2007-01-18 00:00:00',96.14,314,834,98.96,'Sun-Synch' from dual union all
select 29721,'029721','1999-025K','deb FY-1C','',66706,'Reentered','2008-06-05 00:00:00','2007-01-18 00:00:00',96.31,307,857,98.99,'Sun-Synch' from dual union all
select 29722,'029722','1999-025L','deb FY-1C','',66706,'Reentered','2010-03-12 00:00:00','2007-01-18 00:00:00',96.56,319,868,98.91,'Sun-Synch' from dual union all
select 29723,'029723','1999-025M','deb FY-1C','',66706,'Reentered','2010-10-11 00:00:00','2007-01-17 00:00:00',96.83,348,865,98.97,'Sun-Synch' from dual union all
select 29724,'029724','1999-025N','deb FY-1C','',66706,'Reentered','2008-09-26 00:00:00','2007-01-18 00:00:00',97.09,362,877,99.1,'Sun-Synch' from dual union all
select 29725,'029725','1999-025P','deb FY-1C','',66706,'Reentered','2010-12-11 00:00:00','2007-01-18 00:00:00',97.19,347,902,97.98,'Sun-Synch' from dual union all
select 29726,'029726','1999-025Q','deb FY-1C','',66706,'Reentered','2012-04-02 00:00:00','2007-01-18 00:00:00',97.36,379,886,99.2,'Sun-Synch' from dual union all
select 29727,'029727','1999-025R','deb FY-1C','',66706,'Reentered','2009-02-15 00:00:00','2007-01-18 00:00:00',97.48,423,854,98.84,'Sun-Synch' from dual union all
select 29728,'029728','1999-025S','deb FY-1C','',66706,'Reentered','2011-11-11 00:00:00','2007-01-18 00:00:00',97.55,422,861,99.18,'Sun-Synch' from dual union all
select 29729,'029729','1999-025T','deb FY-1C','',66706,'Reentered','2010-08-08 00:00:00','2007-01-18 00:00:00',97.55,427,856,98.93,'Sun-Synch' from dual union all
select 2973,'002973','1967-095A','Molniya-1','Molniya-1 11F67 No. 9',15350,'Reentered','1969-03-04 00:00:00','1967-12-02 00:00:00',717.31,438,39894,64.86,'GEO Transfer' from dual union all
select 29730,'029730','1999-025U','deb FY-1C','',66706,'Reentered','2010-03-19 00:00:00','2007-01-18 00:00:00',97.58,430,856,98.88,'Sun-Synch' from dual union all
select 29731,'029731','1999-025V','deb FY-1C','',66706,'Reentered','2010-03-25 00:00:00','2007-01-17 00:00:00',97.59,431,855,98.94,'Sun-Synch' from dual union all
select 29732,'029732','1999-025W','deb FY-1C','',66706,'Reentered','2012-02-02 00:00:00','2007-01-18 00:00:00',97.59,427,859,98.9,'Sun-Synch' from dual union all
select 29733,'029733','1999-025X','deb FY-1C','',66706,'In Earth orbit','','2007-01-18 00:00:00',111.85,854,1770,99.21,'Sun-Synch' from dual union all
select 29734,'029734','1999-025Y','deb FY-1C','',66706,'In Earth orbit','','2007-01-18 00:00:00',112.78,855,1854,99.14,'Sun-Synch' from dual union all
select 29735,'029735','1999-025Z','deb FY-1C','',66706,'In Earth orbit','','2007-01-18 00:00:00',114.19,857,1981,99.38,'Sun-Synch' from dual union all
select 29736,'029736','1999-025AA','deb FY-1C','',66706,'In Earth orbit','','2007-01-18 00:00:00',114.45,858,2003,99.44,'Sun-Synch' from dual union all
select 29737,'029737','1999-025AB','deb FY-1C','',66706,'In Earth orbit','','2007-01-18 00:00:00',114.83,857,2039,99.58,'Sun-Synch' from dual union all
select 29738,'029738','1999-025AC','deb FY-1C','',66706,'In Earth orbit','','2007-01-18 00:00:00',115.77,855,2126,99.47,'Sun-Synch' from dual union all
select 29739,'029739','1999-025AD','deb FY-1C','',66706,'Reentered','2015-02-02 00:00:00','2007-01-18 00:00:00',117.55,852,2290,99.14,'Sun-Synch' from dual union all
select 2974,'002974','1967-095C','11S510','11S510',15350,'Reentered','1967-10-21 00:00:00','1967-10-11 00:00:00',90.17,190,376,64.84,'Intermediate' from dual union all
select 29740,'029740','1999-025AE','deb FY-1C','',66706,'In Earth orbit','','2007-01-17 00:00:00',120.26,856,2529,99,'Medium' from dual union all
select 29741,'029741','1999-025AF','deb FY-1C','',66706,'In Earth orbit','','2007-01-18 00:00:00',120.89,858,2584,99.18,'Medium' from dual union all
select 29742,'029742','1999-025AG','deb FY-1C','',66706,'Reentered','2007-09-24 00:00:00','2007-01-18 00:00:00',126.48,858,3078,99.69,'Medium' from dual union all
select 29743,'029743','1999-025AH','deb FY-1C','',66706,'Reentered','2010-07-09 00:00:00','2007-01-16 00:00:00',126.76,851,3111,99.81,'Medium' from dual union all
select 29744,'029744','1999-025AJ','deb FY-1C','',66706,'Reentered','2012-12-31 00:00:00','2007-01-18 00:00:00',126.9,854,3120,99.8,'Medium' from dual union all
select 29745,'029745','1999-025AK','deb FY-1C','',66706,'Reentered','2007-10-08 00:00:00','2007-01-18 00:00:00',127.41,859,3160,98.82,'Medium' from dual union all
select 29746,'029746','1999-025AL','deb FY-1C','',66706,'In Earth orbit','','2007-01-18 00:00:00',127.65,860,3180,98.88,'Medium' from dual union all
select 29747,'029747','1999-025AM','deb FY-1C','',66706,'Reentered','2010-07-31 00:00:00','2007-01-18 00:00:00',131.12,847,3496,100.16,'Medium' from dual union all
select 29748,'029748','1999-025AN','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',131.34,844,3518,100.08,'Medium' from dual union all
select 29749,'029749','1999-025AP','deb FY-1C','',66706,'Reentered','2010-01-07 00:00:00','2007-01-26 00:00:00',97.66,443,851,98.88,'Sun-Synch' from dual union all
select 2975,'002975','1967-095B','BOZ','BOZ',15350,'Reentered','1967-10-17 00:00:00','1967-10-06 00:00:00',91.1,198,459,64.78,'Intermediate' from dual union all
select 29750,'029750','1999-025AQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.43,847,901,98.8,'Sun-Synch' from dual union all
select 29751,'029751','1999-025AR','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.4,758,892,98.58,'Sun-Synch' from dual union all
select 29752,'029752','1999-025AS','deb FY-1C','',66706,'Reentered','2010-02-18 00:00:00','2007-01-26 00:00:00',97.54,430,851,98.91,'Sun-Synch' from dual union all
select 29753,'029753','1999-025AT','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.06,746,871,98.96,'Sun-Synch' from dual union all
select 29754,'029754','1999-025AU','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.42,845,902,99.2,'Sun-Synch' from dual union all
select 29755,'029755','1999-025AV','deb FY-1C','',66706,'Reentered','2012-04-10 00:00:00','2007-01-26 00:00:00',98.19,488,857,98.91,'Sun-Synch' from dual union all
select 29756,'029756','1999-025AW','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.22,771,862,99.28,'Sun-Synch' from dual union all
select 29757,'029757','1999-025AX','deb FY-1C','',66706,'Reentered','2013-04-22 00:00:00','2007-01-26 00:00:00',102.25,843,887,98.71,'Sun-Synch' from dual union all
select 29758,'029758','1999-025AY','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.5,851,903,99,'Sun-Synch' from dual union all
select 29759,'029759','1999-025AZ','deb FY-1C','',66706,'Reentered','2015-12-11 00:00:00','2007-01-26 00:00:00',98.81,545,858,98.98,'Sun-Synch' from dual union all
select 2976,'002976','1967-036C','ESSA 5 despin weight','',15291,'In Earth orbit','','1967-10-02 00:00:00',112.38,1259,1412,102.21,'Sun-Synch' from dual union all
select 29760,'029760','1999-025BA','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.48,777,974,98.93,'Sun-Synch' from dual union all
select 29761,'029761','1999-025BB','deb FY-1C','',66706,'Reentered','2013-02-17 00:00:00','2007-01-26 00:00:00',103.11,740,1072,99.6,'Sun-Synch' from dual union all
select 29762,'029762','1999-025BC','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.62,678,992,98.91,'Sun-Synch' from dual union all
select 29763,'029763','1999-025BD','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.51,781,974,99.14,'Sun-Synch' from dual union all
select 29764,'029764','1999-025BE','deb FY-1C','',66706,'Reentered','2011-12-15 00:00:00','2007-01-26 00:00:00',99.21,544,898,98.79,'Sun-Synch' from dual union all
select 29765,'029765','1999-025BF','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.78,856,925,98.88,'Sun-Synch' from dual union all
select 29766,'029766','1999-025BG','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.13,830,1078,98.63,'Sun-Synch' from dual union all
select 29767,'029767','1999-025BH','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.64,668,1287,98.2,'Sun-Synch' from dual union all
select 29768,'029768','1999-025BJ','deb FY-1C','',66706,'Reentered','2011-01-04 00:00:00','2007-01-26 00:00:00',103.18,734,1084,99.56,'Sun-Synch' from dual union all
select 29769,'029769','1999-025BK','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.33,834,999,98.9,'Sun-Synch' from dual union all
select 2977,'002977','1967-036D','ESSA 5 despin weight','',15291,'In Earth orbit','','1967-05-31 00:00:00',114.61,1391,1485,101.64,'Sun-Synch' from dual union all
select 29770,'029770','1999-025BL','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.91,832,961,99.08,'Sun-Synch' from dual union all
select 29771,'029771','1999-025BM','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.8,845,1032,99.47,'Sun-Synch' from dual union all
select 29772,'029772','1999-025BN','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.61,822,1036,98.4,'Sun-Synch' from dual union all
select 29773,'029773','1999-025BP','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.41,787,863,99.07,'Sun-Synch' from dual union all
select 29774,'029774','1999-025BQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.45,790,865,99.2,'Sun-Synch' from dual union all
select 29775,'029775','1999-025BR','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.11,668,859,99.07,'Sun-Synch' from dual union all
select 29776,'029776','1999-025BS','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.46,857,894,98.81,'Sun-Synch' from dual union all
select 29777,'029777','1999-025BT','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.57,857,998,98.86,'Sun-Synch' from dual union all
select 29778,'029778','1999-025BU','deb FY-1C','',66706,'Reentered','2011-05-19 00:00:00','2007-01-26 00:00:00',100.81,713,881,98.83,'Sun-Synch' from dual union all
select 29779,'029779','1999-025BV','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.16,813,909,98.82,'Sun-Synch' from dual union all
select 2978,'002978','1966-049B','Agena D 6502','Agena D 6502',13456,'Reentered','1981-09-29 00:00:00','1967-10-31 00:00:00',2901.03,355,121647,54.4,'Very High' from dual union all
select 29780,'029780','1999-025BW','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.94,857,939,99.36,'Sun-Synch' from dual union all
select 29781,'029781','1999-025BX','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.08,855,954,99.11,'Sun-Synch' from dual union all
select 29782,'029782','1999-025BY','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.3,852,978,98.92,'Sun-Synch' from dual union all
select 29783,'029783','1999-025BZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',99.8,637,861,99,'Sun-Synch' from dual union all
select 29784,'029784','1999-025CA','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.28,771,868,99.12,'Sun-Synch' from dual union all
select 29785,'029785','1999-025CB','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.42,696,861,99.08,'Sun-Synch' from dual union all
select 29786,'029786','1999-025CC','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.39,814,930,98.83,'Sun-Synch' from dual union all
select 29787,'029787','1999-025CD','deb FY-1C','',66706,'Reentered','2015-02-02 00:00:00','2007-01-26 00:00:00',100.4,655,899,99.08,'Sun-Synch' from dual union all
select 29788,'029788','1999-025CE','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.67,852,918,99,'Sun-Synch' from dual union all
select 29789,'029789','1999-025CF','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',107.25,858,1341,99.54,'Sun-Synch' from dual union all
select 2979,'002979','1967-086K','deb S1M','',15341,'Reentered','1968-01-13 00:00:00','1967-10-23 00:00:00',100.25,253,1288,81.99,'Intermediate' from dual union all
select 29790,'029790','1999-025CG','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.06,748,870,99.16,'Sun-Synch' from dual union all
select 29791,'029791','1999-025CH','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.23,767,866,99.09,'Sun-Synch' from dual union all
select 29792,'029792','1999-025CJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.98,857,943,99.16,'Sun-Synch' from dual union all
select 29793,'029793','1999-025CK','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.75,820,1052,98.39,'Sun-Synch' from dual union all
select 29794,'029794','1999-025CL','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.88,776,1013,98.24,'Sun-Synch' from dual union all
select 29795,'029795','1999-025CM','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.52,841,1009,98.74,'Sun-Synch' from dual union all
select 29796,'029796','1999-025CN','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.7,841,932,98.83,'Sun-Synch' from dual union all
select 29797,'029797','1999-025CP','deb FY-1C','',66706,'Reentered','2011-04-05 00:00:00','2007-01-26 00:00:00',104.49,592,1350,99.08,'Sun-Synch' from dual union all
select 29798,'029798','1999-025CQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.12,858,1049,99.05,'Sun-Synch' from dual union all
select 29799,'029799','1999-025CR','deb FY-1C','',66706,'Reentered','2011-05-15 00:00:00','2007-01-26 00:00:00',104.26,780,1140,99.11,'Sun-Synch' from dual;
insert into SATELLITE
select 298,'000298','62 Upsilon 2','S1','S1',8205,'Reentered','1962-12-15 00:00:00','1962-06-30 00:00:00',101.68,211,1466,49.05,'Intermediate' from dual union all
select 2980,'002980','1967-096A','OPS 1264','DAPP 4417',15351,'In Earth orbit','','1967-11-20 00:00:00',100.1,668,859,99.16,'Sun-Synch' from dual union all
select 29800,'029800','1999-025CS','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.58,853,1097,98.31,'Sun-Synch' from dual union all
select 29801,'029801','1999-025CT','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.56,707,864,98.97,'Sun-Synch' from dual union all
select 29802,'029802','1999-025CU','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.78,789,897,98.83,'Sun-Synch' from dual union all
select 29803,'029803','1999-025CV','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.44,810,938,98.96,'Sun-Synch' from dual union all
select 29804,'029804','1999-025CW','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.61,810,955,98.91,'Sun-Synch' from dual union all
select 29805,'029805','1999-025CX','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',105.01,854,1136,99.22,'Sun-Synch' from dual union all
select 29806,'029806','1999-025CY','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.54,857,901,98.92,'Sun-Synch' from dual union all
select 29807,'029807','1999-025CZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.48,816,936,98.92,'Sun-Synch' from dual union all
select 29808,'029808','1999-025DA','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.51,845,910,99.07,'Sun-Synch' from dual union all
select 29809,'029809','1999-025DB','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.15,672,860,98.93,'Sun-Synch' from dual union all
select 2981,'002981','1967-097A','Kosmos-181','Zenit-2 No. 55',15352,'Landed','1967-10-19 00:00:00','1967-10-14 00:00:00',89.66,195,321,65.64,'Intermediate' from dual union all
select 29810,'029810','1999-025DC','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.22,765,867,99.24,'Sun-Synch' from dual union all
select 29811,'029811','1999-025DD','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.72,821,954,98.99,'Sun-Synch' from dual union all
select 29812,'029812','1999-025DE','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.73,817,1053,98.39,'Sun-Synch' from dual union all
select 29813,'029813','1999-025DF','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.24,857,873,98.78,'Sun-Synch' from dual union all
select 29814,'029814','1999-025DG','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.16,764,864,99.17,'Sun-Synch' from dual union all
select 29815,'029815','1999-025DH','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.05,581,941,98.53,'Sun-Synch' from dual union all
select 29816,'029816','1999-025DJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.9,607,1091,99.1,'Sun-Synch' from dual union all
select 29817,'029817','1999-025DK','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.19,853,872,99.15,'Sun-Synch' from dual union all
select 29818,'029818','1999-025DL','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.77,804,976,98.61,'Sun-Synch' from dual union all
select 29819,'029819','1999-025DM','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.98,751,1049,99.3,'Sun-Synch' from dual union all
select 2982,'002982','1967-097B','11A57I','11A57I',15352,'Reentered','1967-10-19 00:00:00','1967-10-14 00:00:00',89.07,180,277,65.64,'Intermediate' from dual union all
select 29820,'029820','1999-025DN','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.12,856,957,99.06,'Sun-Synch' from dual union all
select 29821,'029821','1999-025DP','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.47,613,1233,98.92,'Sun-Synch' from dual union all
select 29822,'029822','1999-025DQ','deb FY-1C','',66706,'Reentered','2011-04-30 00:00:00','2007-01-26 00:00:00',97.64,332,960,98.7,'Sun-Synch' from dual union all
select 29823,'029823','1999-025DR','deb FY-1C','',66706,'Reentered','2011-10-03 00:00:00','2007-01-26 00:00:00',97.79,449,856,98.94,'Sun-Synch' from dual union all
select 29824,'029824','1999-025DS','deb FY-1C','',66706,'Reentered','2013-01-24 00:00:00','2007-01-26 00:00:00',98.18,486,858,98.8,'Sun-Synch' from dual union all
select 29825,'029825','1999-025DT','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',98.47,511,860,98.78,'Sun-Synch' from dual union all
select 29826,'029826','1999-025DU','deb FY-1C','',66706,'Reentered','2011-11-04 00:00:00','2007-01-26 00:00:00',98.71,502,892,97.41,'Sun-Synch' from dual union all
select 29827,'029827','1999-025DV','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.67,819,857,98.86,'Sun-Synch' from dual union all
select 29828,'029828','1999-025DW','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.03,855,949,99.26,'Sun-Synch' from dual union all
select 29829,'029829','1999-025DX','deb FY-1C','',66706,'Reentered','2010-08-04 00:00:00','2007-01-26 00:00:00',97.85,459,852,98.75,'Sun-Synch' from dual union all
select 2983,'002983','1967-086L','deb S1M','',15341,'Reentered','1967-11-04 00:00:00','1967-10-11 00:00:00',97.72,198,1102,82.19,'Intermediate' from dual union all
select 29830,'029830','1999-025DY','deb FY-1C','',66706,'Reentered','2014-03-05 00:00:00','2007-01-26 00:00:00',97.97,467,856,98.86,'Sun-Synch' from dual union all
select 29831,'029831','1999-025DZ','deb FY-1C','',66706,'Reentered','2011-02-05 00:00:00','2007-01-26 00:00:00',98.28,458,895,97.41,'Sun-Synch' from dual union all
select 29832,'029832','1999-025EA','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',99.31,575,877,98.91,'Sun-Synch' from dual union all
select 29833,'029833','1999-025EB','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',99.58,605,872,99.12,'Sun-Synch' from dual union all
select 29834,'029834','1999-025EC','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',99.96,567,946,98.06,'Sun-Synch' from dual union all
select 29835,'029835','1999-025ED','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',99.97,633,882,99.1,'Sun-Synch' from dual union all
select 29836,'029836','1999-025EE','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.09,626,900,98,'Sun-Synch' from dual union all
select 29837,'029837','1999-025EF','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.18,601,933,98.35,'Sun-Synch' from dual union all
select 29838,'029838','1999-025EG','deb FY-1C','',66706,'Reentered','2015-05-20 00:00:00','2007-01-26 00:00:00',100.08,604,920,97.27,'Sun-Synch' from dual union all
select 29839,'029839','1999-025EH','deb FY-1C','',66706,'Reentered','2014-11-21 00:00:00','2007-01-26 00:00:00',100.51,692,874,102.19,'Retrograde' from dual union all
select 2984,'002984','1967-086M','deb S1M','',15341,'Reentered','1967-10-31 00:00:00','1967-10-10 00:00:00',97.39,221,1047,82.35,'Intermediate' from dual union all
select 29840,'029840','1999-025EJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.35,654,897,99.06,'Sun-Synch' from dual union all
select 29841,'029841','1999-025EK','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.34,609,941,98.94,'Sun-Synch' from dual union all
select 29842,'029842','1999-025EL','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.7,672,911,98.29,'Sun-Synch' from dual union all
select 29843,'029843','1999-025EM','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.3,669,971,99.06,'Sun-Synch' from dual union all
select 29844,'029844','1999-025EN','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.36,684,963,99.1,'Sun-Synch' from dual union all
select 29845,'029845','1999-025EP','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.52,769,893,98.8,'Sun-Synch' from dual union all
select 29846,'029846','1999-025EQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.67,775,900,98.58,'Sun-Synch' from dual union all
select 29847,'029847','1999-025ER','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.83,829,956,99.05,'Sun-Synch' from dual union all
select 29848,'029848','1999-025ES','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.86,787,1001,98.28,'Sun-Synch' from dual union all
select 29849,'029849','1999-025ET','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.52,854,996,99.38,'Sun-Synch' from dual union all
select 2985,'002985','1967-096B','Burner II No. 5','Star 37B',15351,'In Earth orbit','','1967-11-25 00:00:00',100.11,669,859,99.1,'Sun-Synch' from dual union all
select 29850,'029850','1999-025EU','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.9,856,1030,98.09,'Sun-Synch' from dual union all
select 29851,'029851','1999-025EV','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.33,855,1071,98.99,'Sun-Synch' from dual union all
select 29852,'029852','1999-025EW','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.2,832,1082,98.99,'Sun-Synch' from dual union all
select 29853,'029853','1999-025EX','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.24,825,1093,98.41,'Sun-Synch' from dual union all
select 29854,'029854','1999-025EY','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.37,853,1077,98.93,'Sun-Synch' from dual union all
select 29855,'029855','1999-025EZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.43,795,1141,98.17,'Sun-Synch' from dual union all
select 29856,'029856','1999-025FA','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.65,590,1366,98.64,'Sun-Synch' from dual union all
select 29857,'029857','1999-025FB','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',105.27,845,1169,98.44,'Sun-Synch' from dual union all
select 29858,'029858','1999-025FC','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',105.49,853,1182,99.1,'Sun-Synch' from dual union all
select 29859,'029859','1999-025FD','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',106.31,800,1312,98.36,'Sun-Synch' from dual union all
select 2986,'002986','1964-026E','deb Transit VC','',10312,'In Earth orbit','','1967-10-10 00:00:00',103.03,860,944,90.54,'Polar' from dual union all
select 29860,'029860','1999-025FE','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',106.23,856,1248,98.63,'Sun-Synch' from dual union all
select 29861,'029861','1999-025FF','deb FY-1C','',66706,'Reentered','2009-11-11 00:00:00','2007-01-26 00:00:00',98.44,489,879,97.43,'Sun-Synch' from dual union all
select 29862,'029862','1999-025FG','deb FY-1C','',66706,'Reentered','2011-10-21 00:00:00','2007-01-26 00:00:00',98.45,490,880,98.56,'Sun-Synch' from dual union all
select 29863,'029863','1999-025FH','deb FY-1C','',66706,'Reentered','2012-11-15 00:00:00','2007-01-26 00:00:00',99.03,528,896,97.58,'Sun-Synch' from dual union all
select 29864,'029864','1999-025FJ','deb FY-1C','',66706,'Reentered','2007-05-25 00:00:00','2007-01-26 00:00:00',98.67,480,911,99.9,'Sun-Synch' from dual union all
select 29865,'029865','1999-025FK','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',99.28,590,859,99,'Sun-Synch' from dual union all
select 29866,'029866','1999-025FL','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',99.39,560,898,98.49,'Sun-Synch' from dual union all
select 29867,'029867','1999-025FM','deb FY-1C','',66706,'Reentered','2015-06-23 00:00:00','2007-01-26 00:00:00',99.62,624,857,99.52,'Sun-Synch' from dual union all
select 29868,'029868','1999-025FN','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',99.86,529,975,98.38,'Sun-Synch' from dual union all
select 29869,'029869','1999-025FP','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.77,727,864,99.03,'Sun-Synch' from dual union all
select 2987,'002987','1967-020C','X-258 yo weight','',15275,'Reentered','1970-01-30 00:00:00','1967-04-29 00:00:00',95.05,508,533,32.83,'Intermediate' from dual union all
select 29870,'029870','1999-025FQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.86,712,887,98.84,'Sun-Synch' from dual union all
select 29871,'029871','1999-025FR','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.09,756,865,99.08,'Sun-Synch' from dual union all
select 29872,'029872','1999-025FS','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.2,611,1020,98.65,'Sun-Synch' from dual union all
select 29873,'029873','1999-025FT','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.33,848,985,99.5,'Sun-Synch' from dual union all
select 29874,'029874','1999-025FU','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.45,811,1032,98.36,'Sun-Synch' from dual union all
select 29875,'029875','1999-025FV','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.12,753,870,99.02,'Sun-Synch' from dual union all
select 29876,'029876','1999-025FW','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.59,789,879,98.28,'Sun-Synch' from dual union all
select 29877,'029877','1999-025FX','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.68,802,875,98.79,'Sun-Synch' from dual union all
select 29878,'029878','1999-025FY','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.08,743,972,97,'Sun-Synch' from dual union all
select 29879,'029879','1999-025FZ','deb FY-1C','',66706,'Reentered','2011-09-24 00:00:00','2007-01-26 00:00:00',103.71,835,1033,99.89,'Sun-Synch' from dual union all
select 2988,'002988','1965-082LD','deb Transtage 4','',11832,'Reentered','1998-12-29 00:00:00','1992-02-03 00:00:00',97.34,583,680,31.61,'Intermediate' from dual union all
select 29880,'029880','1999-025GA','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.4,770,1163,98.16,'Sun-Synch' from dual union all
select 29881,'029881','1999-025GB','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',107.03,847,1331,98.72,'Sun-Synch' from dual union all
select 29882,'029882','1999-025GC','deb FY-1C','',66706,'Reentered','2010-04-04 00:00:00','2007-01-26 00:00:00',97.57,432,852,98.93,'Sun-Synch' from dual union all
select 29883,'029883','1999-025GD','deb FY-1C','',66706,'Reentered','2011-09-05 00:00:00','2007-01-26 00:00:00',100.15,654,878,98.82,'Sun-Synch' from dual union all
select 29884,'029884','1999-025GE','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.27,778,860,99.62,'Sun-Synch' from dual union all
select 29885,'029885','1999-025GF','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.05,768,944,98.25,'Sun-Synch' from dual union all
select 29886,'029886','1999-025GG','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.97,815,889,99.24,'Sun-Synch' from dual union all
select 29887,'029887','1999-025GH','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.99,593,1113,98.56,'Sun-Synch' from dual union all
select 29888,'029888','1999-025GJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.18,640,1084,99.22,'Sun-Synch' from dual union all
select 29889,'029889','1999-025GK','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.28,748,986,99.56,'Sun-Synch' from dual;
insert into SATELLITE
select 2989,'002989','1966-005F','NNS O-7 part','',13412,'In Earth orbit','','1967-05-31 00:00:00',104.44,864,1073,89.66,'Polar' from dual union all
select 29890,'029890','1999-025GL','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.39,857,887,99.54,'Sun-Synch' from dual union all
select 29891,'029891','1999-025GM','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.1,857,954,99.35,'Sun-Synch' from dual union all
select 29892,'029892','1999-025GN','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.7,708,1159,99.16,'Sun-Synch' from dual union all
select 29893,'029893','1999-025GP','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.54,843,1103,98.47,'Sun-Synch' from dual union all
select 29894,'029894','1999-025GQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.73,850,1114,99.14,'Sun-Synch' from dual union all
select 29895,'029895','1999-025GR','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.91,683,1297,99.95,'Sun-Synch' from dual union all
select 29896,'029896','1999-025GS','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',105.85,849,1220,99.27,'Sun-Synch' from dual union all
select 29897,'029897','1999-025GT','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',107.45,857,1361,98.63,'Sun-Synch' from dual union all
select 29898,'029898','1999-025GU','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',107.53,718,1506,98.22,'Sun-Synch' from dual union all
select 29899,'029899','1999-025GV','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',107.78,856,1392,98.66,'Sun-Synch' from dual union all
select 299,'000299','61 Omicron 87','deb r/b Ablestar','',7273,'In Earth orbit','','1962-05-25 00:00:00',106.21,886,1216,67.21,'Intermediate' from dual union all
select 2990,'002990','1967-011N','deb Diademe 1','',15266,'Reentered','1980-02-07 00:00:00','1967-07-29 00:00:00',103.96,569,1323,39.98,'Intermediate' from dual union all
select 29900,'029900','1999-025GW','deb FY-1C','',66706,'Reentered','2013-06-26 00:00:00','2007-01-26 00:00:00',107.8,746,1504,99.96,'Sun-Synch' from dual union all
select 29901,'029901','1999-025GX','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',108.21,854,1434,98.76,'Sun-Synch' from dual union all
select 29902,'029902','1999-025GY','deb FY-1C','',66706,'Reentered','2011-06-14 00:00:00','2007-01-26 00:00:00',97.69,442,855,98.86,'Sun-Synch' from dual union all
select 29903,'029903','1999-025GZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.46,700,860,99.04,'Sun-Synch' from dual union all
select 29904,'029904','1999-025HA','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.66,614,966,99.08,'Sun-Synch' from dual union all
select 29905,'029905','1999-025HB','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.78,734,858,99.28,'Sun-Synch' from dual union all
select 29906,'029906','1999-025HC','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.02,809,900,98.83,'Sun-Synch' from dual union all
select 29907,'029907','1999-025HD','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.42,857,984,98.95,'Sun-Synch' from dual union all
select 29908,'029908','1999-025HE','deb FY-1C','',66706,'Reentered','2010-01-23 00:00:00','2007-01-26 00:00:00',104.57,850,1099,99.21,'Sun-Synch' from dual union all
select 29909,'029909','1999-025HF','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.81,793,896,98.82,'Sun-Synch' from dual union all
select 2991,'002991','1965-082LE','deb Transtage 4','',11832,'Reentered','1979-02-20 00:00:00','1966-06-25 00:00:00',101.82,694,996,31.9,'Intermediate' from dual union all
select 29910,'029910','1999-025HG','deb FY-1C','',66706,'Reentered','2012-05-12 00:00:00','2007-01-26 00:00:00',102.19,824,901,99.6,'Sun-Synch' from dual union all
select 29911,'029911','1999-025HH','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.34,844,895,98.81,'Sun-Synch' from dual union all
select 29912,'029912','1999-025HJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.09,811,905,98.78,'Sun-Synch' from dual union all
select 29913,'029913','1999-025HK','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.63,835,932,98.99,'Sun-Synch' from dual union all
select 29914,'029914','1999-025HL','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.77,857,923,98.84,'Sun-Synch' from dual union all
select 29915,'029915','1999-025HM','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.09,666,860,99.09,'Sun-Synch' from dual union all
select 29916,'029916','1999-025HN','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',106.56,857,1278,99.22,'Sun-Synch' from dual union all
select 29917,'029917','1999-025HP','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',105.91,855,1219,98.87,'Sun-Synch' from dual union all
select 29918,'029918','1999-025HQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.34,854,885,98.67,'Sun-Synch' from dual union all
select 29919,'029919','1999-025HR','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.1,849,1056,98.47,'Sun-Synch' from dual union all
select 2992,'002992','1965-082LF','deb Transtage 4','',11832,'Reentered','1969-03-29 00:00:00','1966-07-30 00:00:00',98.3,609,746,32.06,'Intermediate' from dual union all
select 29920,'029920','1999-025HS','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.59,821,1036,98.4,'Sun-Synch' from dual union all
select 29921,'029921','1999-025HT','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.76,848,931,98.98,'Sun-Synch' from dual union all
select 29922,'029922','1999-025HU','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.15,838,977,98.8,'Sun-Synch' from dual union all
select 29923,'029923','1999-025HV','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.39,857,887,98.92,'Sun-Synch' from dual union all
select 29924,'029924','1999-025HW','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102.28,847,887,98.82,'Sun-Synch' from dual union all
select 29925,'029925','1999-025HX','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',100.35,690,861,99.07,'Sun-Synch' from dual union all
select 29926,'029926','1999-025HY','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.02,611,1003,98.91,'Sun-Synch' from dual union all
select 29927,'029927','1999-025HZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.69,811,867,99.15,'Sun-Synch' from dual union all
select 29928,'029928','1999-025JA','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.3,849,1075,98.98,'Sun-Synch' from dual union all
select 29929,'029929','1999-025JB','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',104.04,833,1066,98.56,'Sun-Synch' from dual union all
select 2993,'002993','1966-013H','deb Rubis','',13420,'Reentered','1968-03-01 00:00:00','1967-10-12 00:00:00',107,432,1743,34,'Intermediate' from dual union all
select 29930,'029930','1999-025JC','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',101.66,713,961,99.17,'Sun-Synch' from dual union all
select 29931,'029931','1999-025JD','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.74,857,920,98.83,'Sun-Synch' from dual union all
select 29932,'029932','1999-025JE','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',102,825,882,98.8,'Sun-Synch' from dual union all
select 29933,'029933','1999-025JF','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.21,853,874,98.69,'Sun-Synch' from dual union all
select 29934,'029934','1999-025JG','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.16,661,872,98.96,'Sun-Synch' from dual union all
select 29935,'029935','1999-025JH','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.97,741,962,98.36,'Sun-Synch' from dual union all
select 29936,'029936','1999-025JJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.58,701,872,98.91,'Sun-Synch' from dual union all
select 29937,'029937','1999-025JK','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.58,857,905,98.84,'Sun-Synch' from dual union all
select 29938,'029938','1999-025JL','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.46,850,901,99.01,'Sun-Synch' from dual union all
select 29939,'029939','1999-025JM','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.04,807,904,98.84,'Sun-Synch' from dual union all
select 2994,'002994','1967-092C','NNS O-14 despin weight','',15347,'In Earth orbit','','1967-12-03 00:00:00',104.51,893,1050,89.51,'Polar' from dual union all
select 29940,'029940','1999-025JN','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.19,802,923,98.89,'Sun-Synch' from dual union all
select 29941,'029941','1999-025JP','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.57,708,864,98.96,'Sun-Synch' from dual union all
select 29942,'029942','1999-025JQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.58,828,934,98.84,'Sun-Synch' from dual union all
select 29943,'029943','1999-025JR','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.2,837,889,98.83,'Sun-Synch' from dual union all
select 29944,'029944','1999-025JS','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',99.77,531,964,99.08,'Sun-Synch' from dual union all
select 29945,'029945','1999-025JT','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.57,857,904,98.83,'Sun-Synch' from dual union all
select 29946,'029946','1999-025JU','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.53,818,939,99.02,'Sun-Synch' from dual union all
select 29947,'029947','1999-025JV','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.8,857,926,98.85,'Sun-Synch' from dual union all
select 29948,'029948','1999-025JW','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.55,816,943,99.06,'Sun-Synch' from dual union all
select 29949,'029949','1999-025JX','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.4,854,890,98.83,'Sun-Synch' from dual union all
select 2995,'002995','1967-098A','Kosmos-182','Zenit-4 No. 36',15353,'Landed','1967-10-24 00:00:00','1967-10-19 00:00:00',89.86,203,332,64.96,'Intermediate' from dual union all
select 29950,'029950','1999-025JY','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.23,840,888,98.73,'Sun-Synch' from dual union all
select 29951,'029951','1999-025JZ','deb FY-1C','',66706,'Reentered','2012-09-10 00:00:00','2007-01-27 00:00:00',97.7,420,877,99.32,'Sun-Synch' from dual union all
select 29952,'029952','1999-025KA','deb FY-1C','',66706,'Reentered','2016-12-28 00:00:00','2007-01-27 00:00:00',99.22,509,934,99.44,'Sun-Synch' from dual union all
select 29953,'029953','1999-025KB','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.81,855,1022,99.45,'Sun-Synch' from dual union all
select 29954,'029954','1999-025KC','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.65,770,904,100.17,'Sun-Synch' from dual union all
select 29955,'029955','1999-025KD','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.97,844,859,99.34,'Sun-Synch' from dual union all
select 29956,'029956','1999-025KE','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.46,836,915,98.79,'Sun-Synch' from dual union all
select 29957,'029957','1999-025KF','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.59,830,933,99.09,'Sun-Synch' from dual union all
select 29958,'029958','1999-025KG','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.14,816,904,98.81,'Sun-Synch' from dual union all
select 29959,'029959','1999-025KH','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.62,851,915,98.82,'Sun-Synch' from dual union all
select 2996,'002996','1967-098B','11A57I','11A57I',15353,'Reentered','1967-10-24 00:00:00','1967-10-18 00:00:00',89.58,199,309,64.98,'Intermediate' from dual union all
select 29960,'029960','1999-025KJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.14,788,932,99.07,'Sun-Synch' from dual union all
select 29961,'029961','1999-025KK','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.21,760,872,99.09,'Sun-Synch' from dual union all
select 29962,'029962','1999-025KL','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.53,795,961,98.91,'Sun-Synch' from dual union all
select 29963,'029963','1999-025KM','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.55,790,875,99.12,'Sun-Synch' from dual union all
select 29964,'029964','1999-025KN','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.56,816,1038,98.38,'Sun-Synch' from dual union all
select 29965,'029965','1999-025KP','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.31,807,1024,99.15,'Sun-Synch' from dual union all
select 29966,'029966','1999-025KQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.55,801,864,99.5,'Sun-Synch' from dual union all
select 29967,'029967','1999-025KR','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.8,820,963,98.9,'Sun-Synch' from dual union all
select 29968,'029968','1999-025KS','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.51,810,945,99.03,'Sun-Synch' from dual union all
select 29969,'029969','1999-025KT','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.42,831,915,98.85,'Sun-Synch' from dual union all
select 2997,'002997','1963-014AU','deb West Ford','RCS 0.22',9245,'In Earth orbit','','1967-06-29 00:00:00',165.88,2448,4797,87,'Medium' from dual union all
select 29970,'029970','1999-025KU','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.56,708,862,98.97,'Sun-Synch' from dual union all
select 29971,'029971','1999-025KV','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.52,857,898,98.84,'Sun-Synch' from dual union all
select 29972,'029972','1999-025KW','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.52,705,862,98.98,'Sun-Synch' from dual union all
select 29973,'029973','1999-025KX','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.03,856,947,99.08,'Sun-Synch' from dual union all
select 29974,'029974','1999-025KY','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.99,857,944,99.2,'Sun-Synch' from dual union all
select 29975,'029975','1999-025KZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.15,823,993,98.74,'Sun-Synch' from dual union all
select 29976,'029976','1999-025LA','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.31,711,1120,99.13,'Sun-Synch' from dual union all
select 29977,'029977','1999-025LB','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.56,820,1034,98.4,'Sun-Synch' from dual union all
select 29978,'029978','1999-025LC','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.28,857,971,98.87,'Sun-Synch' from dual union all
select 29979,'029979','1999-025LD','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.57,747,919,99.04,'Sun-Synch' from dual;
insert into SATELLITE
select 2998,'002998','1963-014AV','deb West Ford','RCS 0.10',9245,'Reentered','2005-03-15 00:00:00','1966-07-29 00:00:00',165.85,2411,4832,86.96,'Medium' from dual union all
select 29980,'029980','1999-025LE','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.57,815,946,98.58,'Sun-Synch' from dual union all
select 29981,'029981','1999-025LF','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',98.74,527,870,98.94,'Sun-Synch' from dual union all
select 29982,'029982','1999-025LG','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',99.6,621,858,99.42,'Sun-Synch' from dual union all
select 29983,'029983','1999-025LH','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.66,711,869,99.21,'Sun-Synch' from dual union all
select 29984,'029984','1999-025LJ','deb FY-1C','',66706,'Reentered','2013-10-03 00:00:00','2007-01-27 00:00:00',102.59,853,910,99.23,'Sun-Synch' from dual union all
select 29985,'029985','1999-025LK','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.16,840,882,98.78,'Sun-Synch' from dual union all
select 29986,'029986','1999-025LL','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.56,857,903,99.46,'Sun-Synch' from dual union all
select 29987,'029987','1999-025LM','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.8,856,1021,98.89,'Sun-Synch' from dual union all
select 29988,'029988','1999-025LN','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.24,715,1015,98.75,'Sun-Synch' from dual union all
select 29989,'029989','1999-025LP','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.92,757,943,99,'Sun-Synch' from dual union all
select 2999,'002999','1967-098C','deb Kosmos-182','',15353,'Reentered','1967-10-24 00:00:00','1967-10-17 00:00:00',89.52,207,295,64.99,'Intermediate' from dual union all
select 29990,'029990','1999-025LQ','deb FY-1C','',66706,'Reentered','2008-03-07 00:00:00','2007-01-27 00:00:00',104.2,774,1140,99.09,'Sun-Synch' from dual union all
select 29991,'029991','1999-025LR','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.25,858,873,102.23,'Sun-Synch' from dual union all
select 29992,'029992','1999-025LS','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.13,794,925,98.9,'Sun-Synch' from dual union all
select 29993,'029993','1999-025LT','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.36,842,899,98.73,'Sun-Synch' from dual union all
select 29994,'029994','1999-025LU','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.88,789,1000,98.89,'Sun-Synch' from dual union all
select 29995,'029995','1999-025LV','deb FY-1C','',66706,'Reentered','2011-11-12 00:00:00','2007-01-27 00:00:00',103.71,841,1027,98.96,'Sun-Synch' from dual union all
select 29996,'029996','1999-025LW','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.11,831,1074,99.01,'Sun-Synch' from dual union all
select 29997,'029997','1999-025LX','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.36,857,978,99.12,'Sun-Synch' from dual union all
select 29998,'029998','1999-025LY','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.98,848,856,99.37,'Sun-Synch' from dual union all
select 29999,'029999','1999-025LZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.89,700,997,99.01,'Sun-Synch' from dual union all
select 3,'000003','57 Beta 1','2-y ISZ','PS-2',4306,'Reentered Attached','1958-04-14 00:00:00','1957-11-03 00:00:00',103.73,211,1659,65.33,'Intermediate' from dual union all
select 30,'000030','60 Gamma 1','Ablestar 002','AJ10-104',6322,'Reentered','1961-08-18 00:00:00','1960-05-16 00:00:00',95.08,319,726,51.29,'Intermediate' from dual union all
select 300,'000300','61 Omicron 88','deb r/b Ablestar','',7273,'Reentered','1979-12-15 00:00:00','1962-05-22 00:00:00',102.79,800,982,66.85,'Intermediate' from dual union all
select 3000,'003000','1967-100A','OSO 4','OSO D',15355,'Reentered','1982-06-15 00:00:00','1967-11-17 00:00:00',95.76,539,572,32.99,'Intermediate' from dual union all
select 30000,'030000','1968-081J','Transtage 5 debris','',16808,'In Earth orbit','','2007-01-27 00:00:00',1427.23,35056,36170,10.49,'Inclined Drift' from dual union all
select 30001,'030001','1999-025MA','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.92,857,937,98.85,'Sun-Synch' from dual union all
select 30002,'030002','1999-025MB','deb FY-1C','',66706,'Reentered','2012-08-02 00:00:00','2007-01-27 00:00:00',97.79,331,975,98.73,'Sun-Synch' from dual union all
select 30003,'030003','1999-025MC','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.25,850,1069,98.95,'Sun-Synch' from dual union all
select 30004,'030004','1999-025MD','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.45,842,907,99.25,'Sun-Synch' from dual union all
select 30005,'030005','1999-025ME','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',107.24,821,1377,99.27,'Sun-Synch' from dual union all
select 30006,'030006','1999-025MF','deb FY-1C','',66706,'Reentered','2013-09-22 00:00:00','2007-01-27 00:00:00',98.81,537,866,98.95,'Sun-Synch' from dual union all
select 30007,'030007','1999-025MG','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.68,825,946,99.3,'Sun-Synch' from dual union all
select 30008,'030008','1999-025MH','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.17,760,869,99.42,'Sun-Synch' from dual union all
select 30009,'030009','1999-025MJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.38,857,979,98.98,'Sun-Synch' from dual union all
select 3001,'003001','1967-099A','Kosmos-183','OGCh No. 15LU',15354,'Deorbited','1967-10-18 00:00:00','1967-10-18 00:00:00',88.94,130,315,49.63,'Intermediate' from dual union all
select 30010,'030010','1999-025MK','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.23,763,871,99.07,'Sun-Synch' from dual union all
select 30011,'030011','1999-025ML','deb FY-1C','',66706,'Reentered','2011-04-26 00:00:00','2007-01-27 00:00:00',102.85,828,959,98.91,'Sun-Synch' from dual union all
select 30012,'030012','1999-025MM','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.14,851,1058,98.96,'Sun-Synch' from dual union all
select 30013,'030013','1999-025MN','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.99,851,1043,98.9,'Sun-Synch' from dual union all
select 30014,'030014','1999-025MP','deb FY-1C','',66706,'Reentered','2011-09-20 00:00:00','2007-01-27 00:00:00',104.84,705,1269,99.53,'Sun-Synch' from dual union all
select 30015,'030015','1999-025MQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.57,802,863,99.24,'Sun-Synch' from dual union all
select 30016,'030016','1999-025MR','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.72,721,865,98.99,'Sun-Synch' from dual union all
select 30017,'030017','1999-025MS','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',99.19,546,894,98.8,'Sun-Synch' from dual union all
select 30018,'030018','1999-025MT','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',99.52,613,859,98.86,'Sun-Synch' from dual union all
select 30019,'030019','1999-025MU','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.38,697,1046,98.15,'Sun-Synch' from dual union all
select 3002,'003002','1967-099B','Kosmos-183 Adapter','OGCh Adapter',15354,'Reentered','1967-10-19 00:00:00','1967-10-18 00:00:00',88.65,130,286,49.63,'Intermediate' from dual union all
select 30020,'030020','1999-025MV','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.03,768,1036,99.07,'Sun-Synch' from dual union all
select 30021,'030021','1999-025MW','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.6,828,936,99.02,'Sun-Synch' from dual union all
select 30022,'030022','1999-025MX','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.28,771,868,99.09,'Sun-Synch' from dual union all
select 30023,'030023','1999-025MY','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.43,836,912,99.3,'Sun-Synch' from dual union all
select 30024,'030024','1999-025MZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',98.1,457,878,98.73,'Sun-Synch' from dual union all
select 30025,'030025','1999-025NA','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',98.95,469,948,98.51,'Sun-Synch' from dual union all
select 30026,'030026','1999-025NB','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',99.08,539,891,99.15,'Sun-Synch' from dual union all
select 30027,'030027','1999-025NC','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',99.54,430,1044,98.39,'Sun-Synch' from dual union all
select 30028,'030028','1999-025ND','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.01,606,911,98.59,'Sun-Synch' from dual union all
select 30029,'030029','1999-025NE','deb FY-1C','',66706,'Reentered','2012-12-17 00:00:00','2007-01-27 00:00:00',99.83,612,889,98.5,'Sun-Synch' from dual union all
select 3003,'003003','1967-099C','8S692','8S692',15354,'Reentered','1967-10-18 00:00:00','1967-10-18 00:00:00',88.11,151,211,49.6,'Intermediate' from dual union all
select 30030,'030030','1999-025NF','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',99.85,637,866,99.75,'Sun-Synch' from dual union all
select 30031,'030031','1999-025NG','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.01,658,859,99.23,'Sun-Synch' from dual union all
select 30032,'030032','1999-025NH','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',99.93,521,989,97.28,'Sun-Synch' from dual union all
select 30033,'030033','1999-025NJ','deb FY-1C','',66706,'Reentered','2014-09-14 00:00:00','2007-01-27 00:00:00',100.11,585,943,99.06,'Sun-Synch' from dual union all
select 30034,'030034','1999-025NK','deb FY-1C','',66706,'Reentered','2013-12-24 00:00:00','2007-01-27 00:00:00',100.29,557,988,97.81,'Sun-Synch' from dual union all
select 30035,'030035','1999-025NL','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.33,691,858,99.35,'Sun-Synch' from dual union all
select 30036,'030036','1999-025NM','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.58,623,950,98.47,'Sun-Synch' from dual union all
select 30037,'030037','1999-025NN','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.73,662,925,98.8,'Sun-Synch' from dual union all
select 30038,'030038','1999-025NP','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.71,587,997,98.7,'Sun-Synch' from dual union all
select 30039,'030039','1999-025NQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.1,675,947,97.02,'Sun-Synch' from dual union all
select 3004,'003004','1967-100B','FW4D','FW4D',15355,'Reentered','1980-05-02 00:00:00','1967-11-18 00:00:00',95.71,536,570,32.98,'Intermediate' from dual union all
select 30040,'030040','1999-025NR','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.14,826,893,98.28,'Sun-Synch' from dual union all
select 30041,'030041','1999-025NS','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.35,843,896,99.56,'Sun-Synch' from dual union all
select 30042,'030042','1999-025NT','deb FY-1C','',66706,'Reentered','2012-12-27 00:00:00','2007-01-27 00:00:00',98.83,492,914,99.27,'Sun-Synch' from dual union all
select 30043,'030043','1999-025NU','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.56,629,1131,98.45,'Sun-Synch' from dual union all
select 30044,'030044','1999-025NV','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.18,757,967,98.14,'Sun-Synch' from dual union all
select 30045,'030045','1999-025NW','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.58,835,1021,98.91,'Sun-Synch' from dual union all
select 30046,'030046','1999-025NX','deb FY-1C','',66706,'Reentered','2011-07-21 00:00:00','2007-01-27 00:00:00',103.83,627,1253,99.25,'Sun-Synch' from dual union all
select 30047,'030047','1999-025NY','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.85,840,1041,99.94,'Sun-Synch' from dual union all
select 30048,'030048','1999-025NZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.56,832,1116,98.64,'Sun-Synch' from dual union all
select 30049,'030049','1999-025PA','deb FY-1C','',66706,'Reentered','2014-09-02 00:00:00','2007-01-27 00:00:00',104.1,839,1065,98.98,'Sun-Synch' from dual union all
select 3005,'003005','1967-101B','BOZ','BOZ',15356,'Reentered','1967-11-07 00:00:00','1967-10-29 00:00:00',90.4,199,390,64.79,'Intermediate' from dual union all
select 30050,'030050','1999-025PB','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.69,838,1122,96.81,'Sun-Synch' from dual union all
select 30051,'030051','1999-025PC','deb FY-1C','',66706,'Reentered','2009-02-22 00:00:00','2007-01-27 00:00:00',105.02,710,1281,99.4,'Sun-Synch' from dual union all
select 30052,'030052','1999-025PD','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',105.29,784,1232,99.41,'Sun-Synch' from dual union all
select 30053,'030053','1999-025PE','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',105.33,700,1320,98.61,'Sun-Synch' from dual union all
select 30054,'030054','1999-025PF','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',105.51,805,1232,99.43,'Sun-Synch' from dual union all
select 30055,'030055','1999-025PG','deb FY-1C','',66706,'Reentered','2015-05-02 00:00:00','2007-01-27 00:00:00',105.47,765,1267,99.57,'Sun-Synch' from dual union all
select 30056,'030056','1999-025PH','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',106.33,797,1317,98.35,'Sun-Synch' from dual union all
select 30057,'030057','1999-025PJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',106.94,854,1316,98.86,'Sun-Synch' from dual union all
select 30058,'030058','1999-025PK','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',107.24,854,1344,100.06,'Sun-Synch' from dual union all
select 30059,'030059','1999-025PL','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',107.19,848,1346,98.83,'Sun-Synch' from dual union all
select 3006,'003006','1967-101C','11S510','11S510',15356,'Reentered','1967-11-03 00:00:00','1967-10-28 00:00:00',90.16,195,369,64.78,'Intermediate' from dual union all
select 30060,'030060','1999-025PM','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',108.16,856,1428,99.32,'Sun-Synch' from dual union all
select 30061,'030061','1999-025PN','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',108.54,856,1462,101.88,'Sun-Synch' from dual union all
select 30062,'030062','1999-025PP','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.72,850,925,98.96,'Sun-Synch' from dual union all
select 30063,'030063','1999-025PQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.81,821,868,99.1,'Sun-Synch' from dual union all
select 30064,'030064','1999-025PR','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.03,806,903,98.78,'Sun-Synch' from dual union all
select 30065,'030065','1999-025PS','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.94,847,1043,98.66,'Sun-Synch' from dual union all
select 30066,'030066','1999-025PT','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.67,750,926,98.84,'Sun-Synch' from dual union all
select 30067,'030067','1999-025PU','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.44,656,903,98.9,'Sun-Synch' from dual;
insert into SATELLITE
select 30068,'030068','1999-025PV','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.19,720,1004,98.97,'Sun-Synch' from dual union all
select 30069,'030069','1999-025PW','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.73,853,923,99.17,'Sun-Synch' from dual union all
select 3007,'003007','1967-101D','deb Blok-I','',15356,'Reentered','1967-10-28 00:00:00','1967-10-28 00:00:00',90.92,200,439,64.8,'Intermediate' from dual union all
select 30070,'030070','1999-025PX','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.38,647,1284,98.26,'Sun-Synch' from dual union all
select 30071,'030071','1999-025PY','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.98,803,902,98.8,'Sun-Synch' from dual union all
select 30072,'030072','1999-025PZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.13,836,978,99.08,'Sun-Synch' from dual union all
select 30073,'030073','1999-025QA','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.05,856,950,99.48,'Sun-Synch' from dual union all
select 30074,'030074','1999-025QB','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.37,834,1001,98.89,'Sun-Synch' from dual union all
select 30075,'030075','1999-025QC','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.52,857,992,98.83,'Sun-Synch' from dual union all
select 30076,'030076','1999-025QD','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.83,853,1120,99.57,'Sun-Synch' from dual union all
select 30077,'030077','1999-025QE','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.38,855,888,98.82,'Sun-Synch' from dual union all
select 30078,'030078','1999-025QF','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.92,857,937,98.98,'Sun-Synch' from dual union all
select 30079,'030079','1999-025QG','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.08,844,965,98.82,'Sun-Synch' from dual union all
select 3008,'003008','1967-101A','Molniya-1','Molniya-1 11F67 No. 12',15356,'Reentered','1969-12-31 00:00:00','1968-02-19 00:00:00',716.26,523,39757,64.83,'GEO Transfer' from dual union all
select 30080,'030080','1999-025QH','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.32,768,969,99.11,'Sun-Synch' from dual union all
select 30081,'030081','1999-025QJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.27,855,971,99.15,'Sun-Synch' from dual union all
select 30082,'030082','1999-025QK','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',99.06,568,860,98.91,'Sun-Synch' from dual union all
select 30083,'030083','1999-025QL','deb FY-1C','',66706,'Reentered','2012-09-20 00:00:00','2007-01-27 00:00:00',99.41,601,860,99.01,'Sun-Synch' from dual union all
select 30084,'030084','1999-025QM','deb FY-1C','',66706,'Reentered','2013-02-04 00:00:00','2007-01-27 00:00:00',99.48,551,917,97.29,'Sun-Synch' from dual union all
select 30085,'030085','1999-025QN','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',99.62,588,892,98.55,'Sun-Synch' from dual union all
select 30086,'030086','1999-025QP','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.91,857,936,98.87,'Sun-Synch' from dual union all
select 30087,'030087','1999-025QQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.94,620,987,98.49,'Sun-Synch' from dual union all
select 30088,'030088','1999-025QR','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.94,641,966,98.83,'Sun-Synch' from dual union all
select 30089,'030089','1999-025QS','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.12,755,868,99.08,'Sun-Synch' from dual union all
select 3009,'003009','1967-101E','deb Blok-I','',15356,'Reentered','1967-10-27 00:00:00','1967-10-23 00:00:00',90.87,200,435,64.82,'Intermediate' from dual union all
select 30090,'030090','1999-025QT','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.35,777,869,99.42,'Sun-Synch' from dual union all
select 30091,'030091','1999-025QU','deb FY-1C','',66706,'Reentered','2015-02-15 00:00:00','2007-01-27 00:00:00',100.77,611,980,98.47,'Sun-Synch' from dual union all
select 30092,'030092','1999-025QV','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.88,730,871,98.96,'Sun-Synch' from dual union all
select 30093,'030093','1999-025QW','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101,742,870,98.97,'Sun-Synch' from dual union all
select 30094,'030094','1999-025QX','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.95,744,864,99.05,'Sun-Synch' from dual union all
select 30095,'030095','1999-025QY','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.13,563,1061,99.3,'Sun-Synch' from dual union all
select 30096,'030096','1999-025QZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.04,746,870,99.25,'Sun-Synch' from dual union all
select 30097,'030097','1999-025RA','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.22,693,940,99.32,'Sun-Synch' from dual union all
select 30098,'030098','1999-025RB','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.25,694,942,99.33,'Sun-Synch' from dual union all
select 30099,'030099','1999-025RC','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.38,777,871,99.05,'Sun-Synch' from dual union all
select 301,'000301','61 Omicron 89','deb r/b Ablestar','',7273,'Reentered','1981-04-08 00:00:00','1962-05-29 00:00:00',103.01,824,978,66.87,'Intermediate' from dual union all
select 3010,'003010','1967-102A','Kosmos-184','Meteor 11F614 No. 8',15357,'Reentered','1989-04-02 00:00:00','1967-11-25 00:00:00',97.06,598,638,81.19,'Intermediate' from dual union all
select 30100,'030100','1999-025RD','deb FY-1C','',66706,'Reentered','2012-07-16 00:00:00','2007-01-27 00:00:00',101.31,416,1225,98.48,'Sun-Synch' from dual union all
select 30101,'030101','1999-025RE','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.48,791,867,99.21,'Sun-Synch' from dual union all
select 30102,'030102','1999-025RF','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.6,726,942,99.06,'Sun-Synch' from dual union all
select 30103,'030103','1999-025RG','deb FY-1C','',66706,'In Earth orbit','','2007-01-25 00:00:00',101.67,787,889,99.26,'Sun-Synch' from dual union all
select 30104,'030104','1999-025RH','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.82,783,907,98.44,'Sun-Synch' from dual union all
select 30105,'030105','1999-025RJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.81,689,1000,98.95,'Sun-Synch' from dual union all
select 30106,'030106','1999-025RK','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.98,742,963,99.02,'Sun-Synch' from dual union all
select 30107,'030107','1999-025RL','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.94,756,945,98.55,'Sun-Synch' from dual union all
select 30108,'030108','1999-025RM','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.97,702,1002,98.96,'Sun-Synch' from dual union all
select 30109,'030109','1999-025RN','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.05,725,987,99.11,'Sun-Synch' from dual union all
select 3011,'003011','1967-102B','8A92ME','8A92ME',15357,'Reentered','1991-12-19 00:00:00','1967-11-28 00:00:00',97.22,540,712,81.19,'Intermediate' from dual union all
select 30110,'030110','1999-025RP','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',106.33,854,1259,98.83,'Sun-Synch' from dual union all
select 30111,'030111','1999-025RQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.14,747,973,101.62,'Sun-Synch' from dual union all
select 30112,'030112','1999-025RR','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.15,692,1029,99.34,'Sun-Synch' from dual union all
select 30113,'030113','1999-025RS','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.24,817,913,99.11,'Sun-Synch' from dual union all
select 30114,'030114','1999-025RT','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.35,810,930,98.9,'Sun-Synch' from dual union all
select 30115,'030115','1999-025RU','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.51,820,935,99.06,'Sun-Synch' from dual union all
select 30116,'030116','1999-025RV','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.52,857,898,99.45,'Sun-Synch' from dual union all
select 30117,'030117','1999-025RW','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.56,756,1003,99.05,'Sun-Synch' from dual union all
select 30118,'030118','1999-025RX','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.59,819,944,98.94,'Sun-Synch' from dual union all
select 30119,'030119','1999-025RY','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.77,772,1007,97.97,'Sun-Synch' from dual union all
select 3012,'003012','1967-103A','OPS 4995','GAMBIT-3 Mission 4309',15358,'Reentered Attached','1967-11-05 00:00:00','1967-10-28 00:00:00',89.82,132,400,111.55,'Retrograde' from dual union all
select 30120,'030120','1999-025RZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.71,824,950,98.86,'Sun-Synch' from dual union all
select 30121,'030121','1999-025SA','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.71,850,924,102.18,'Sun-Synch' from dual union all
select 30122,'030122','1999-025SB','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.93,749,1046,95.92,'Sun-Synch' from dual union all
select 30123,'030123','1999-025SC','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.79,668,1114,96.89,'Sun-Synch' from dual union all
select 30124,'030124','1999-025SD','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.04,780,1025,98.78,'Sun-Synch' from dual union all
select 30125,'030125','1999-025SE','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.11,586,1226,98.5,'Sun-Synch' from dual union all
select 30126,'030126','1999-025SF','deb FY-1C','',66706,'Reentered','2011-04-30 00:00:00','2007-01-27 00:00:00',97.75,448,854,98.89,'Sun-Synch' from dual union all
select 30127,'030127','1999-025SG','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.15,801,1015,98.25,'Sun-Synch' from dual union all
select 30128,'030128','1999-025SH','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.15,763,1053,97.01,'Sun-Synch' from dual union all
select 30129,'030129','1999-025SJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.19,761,1057,98,'Sun-Synch' from dual union all
select 3013,'003013','1967-104A','Kosmos-185','I2-BM',15359,'Reentered','1969-01-14 00:00:00','1967-11-28 00:00:00',98.48,515,857,64.09,'Intermediate' from dual union all
select 30130,'030130','1999-025SK','deb FY-1C','',66706,'Reentered','2012-04-25 00:00:00','2007-01-27 00:00:00',103.26,842,984,98.86,'Sun-Synch' from dual union all
select 30131,'030131','1999-025SL','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.49,841,1007,99.03,'Sun-Synch' from dual union all
select 30132,'030132','1999-025SM','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.58,847,1009,99.66,'Sun-Synch' from dual union all
select 30133,'030133','1999-025SN','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.57,857,998,98.33,'Sun-Synch' from dual union all
select 30134,'030134','1999-025SP','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.41,857,888,98.87,'Sun-Synch' from dual union all
select 30135,'030135','1999-025SQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.7,719,960,99.01,'Sun-Synch' from dual union all
select 30136,'030136','1999-025SR','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.58,828,1028,98.48,'Sun-Synch' from dual union all
select 30137,'030137','1999-025SS','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.16,673,860,99.14,'Sun-Synch' from dual union all
select 30138,'030138','1999-025ST','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.41,855,891,98.82,'Sun-Synch' from dual union all
select 30139,'030139','1999-025SU','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.65,821,1041,98.39,'Sun-Synch' from dual union all
select 3014,'003014','1967-105A','Kosmos-186','Soyuz No. 6',15360,'Landed','1967-10-31 00:00:00','1967-10-29 00:00:00',88.71,171,251,51.71,'Intermediate' from dual union all
select 30140,'030140','1999-025SV','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.83,850,935,98.98,'Sun-Synch' from dual union all
select 30141,'030141','1999-025SW','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.97,832,872,99.1,'Sun-Synch' from dual union all
select 30142,'030142','1999-025SX','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.64,840,1022,99.03,'Sun-Synch' from dual union all
select 30143,'030143','1999-025SY','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.67,848,1017,98.81,'Sun-Synch' from dual union all
select 30144,'030144','1999-025SZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.86,843,1039,99.34,'Sun-Synch' from dual union all
select 30145,'030145','1999-025TA','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.05,857,1043,99.14,'Sun-Synch' from dual union all
select 30146,'030146','1999-025TB','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.35,825,1102,98.02,'Sun-Synch' from dual union all
select 30147,'030147','1999-025TC','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.2,824,1089,98.84,'Sun-Synch' from dual union all
select 30148,'030148','1999-025TD','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.26,854,1066,99.64,'Sun-Synch' from dual union all
select 30149,'030149','1999-025TE','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.29,685,860,99.08,'Sun-Synch' from dual union all
select 3015,'003015','1967-105B','11S510','11S510',15360,'Reentered','1967-10-29 00:00:00','1967-10-28 00:00:00',88.44,192,203,51.63,'Intermediate' from dual union all
select 30150,'030150','1999-025TF','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.3,847,1076,98.91,'Sun-Synch' from dual union all
select 30151,'030151','1999-025TG','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.59,848,1103,99.53,'Sun-Synch' from dual union all
select 30152,'030152','1999-025TH','deb FY-1C','',66706,'In Earth orbit','','2007-01-28 00:00:00',104.87,840,1136,99.1,'Sun-Synch' from dual union all
select 30153,'030153','1999-025TJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',105.1,838,1161,100.47,'Sun-Synch' from dual union all
select 30154,'030154','1999-025TK','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',105.07,845,1151,98.45,'Sun-Synch' from dual union all
select 30155,'030155','1999-025TL','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',106.52,855,1276,99.62,'Sun-Synch' from dual union all
select 30156,'030156','1999-025TM','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',105.37,851,1173,98.9,'Sun-Synch' from dual union all
select 30157,'030157','1999-025TN','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.05,817,895,98.77,'Sun-Synch' from dual;
insert into SATELLITE
select 30158,'030158','1999-025TP','deb FY-1C','',66706,'Reentered','2018-04-16 00:00:00','2007-01-27 00:00:00',105.73,836,1221,98.53,'Sun-Synch' from dual union all
select 30159,'030159','1999-025TQ','deb FY-1C','',66706,'Reentered','2011-10-01 00:00:00','2007-01-27 00:00:00',107.23,611,1586,99.85,'Sun-Synch' from dual union all
select 3016,'003016','1967-106A','Kosmos-187','OGCh No. 16LU',15361,'Deorbited','1967-10-28 00:00:00','1967-10-28 00:00:00',87.55,114,193,49.65,'Intermediate' from dual union all
select 30160,'030160','1999-025TR','deb FY-1C','',66706,'In Earth orbit','','2007-01-28 00:00:00',107.78,853,1395,98.86,'Sun-Synch' from dual union all
select 30161,'030161','1999-025TS','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',107.76,853,1393,100.54,'Sun-Synch' from dual union all
select 30162,'030162','1999-025TT','deb FY-1C','',66706,'In Earth orbit','','2007-01-28 00:00:00',107.75,856,1390,99.48,'Sun-Synch' from dual union all
select 30163,'030163','1999-025TU','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',108.41,855,1452,98.68,'Sun-Synch' from dual union all
select 30164,'030164','1999-025TV','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',108.56,762,1559,99.46,'Sun-Synch' from dual union all
select 30165,'030165','1999-025TW','deb FY-1C','',66706,'Reentered','2011-04-03 00:00:00','2007-01-27 00:00:00',108.84,716,1630,99.73,'Sun-Synch' from dual union all
select 30166,'030166','1999-025TX','deb FY-1C','',66706,'Reentered','2012-01-01 00:00:00','2007-01-27 00:00:00',98.58,496,886,97.43,'Sun-Synch' from dual union all
select 30167,'030167','1999-025TY','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.33,651,898,99.07,'Sun-Synch' from dual union all
select 30168,'030168','1999-025TZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.83,723,968,97.38,'Sun-Synch' from dual union all
select 30169,'030169','1999-025UA','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.9,598,1194,98.34,'Sun-Synch' from dual union all
select 3017,'003017','1967-106B','Kosmos-187 Adapter','OGCh Adapter',15361,'Reentered','1967-10-28 00:00:00','1967-10-28 00:00:00',87.52,144,160,49.63,'Intermediate' from dual union all
select 30170,'030170','1999-025UB','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.2,856,964,99.27,'Sun-Synch' from dual union all
select 30171,'030171','1999-025UC','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.32,776,1056,98.39,'Sun-Synch' from dual union all
select 30172,'030172','1999-025UD','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.48,856,991,98.84,'Sun-Synch' from dual union all
select 30173,'030173','1999-025UE','deb FY-1C','',66706,'In Earth orbit','','2007-01-28 00:00:00',103.44,839,1003,99.08,'Sun-Synch' from dual union all
select 30174,'030174','1999-025UF','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.05,811,901,98.84,'Sun-Synch' from dual union all
select 30175,'030175','1999-025UG','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.36,853,1076,99.07,'Sun-Synch' from dual union all
select 30176,'030176','1999-025UH','deb FY-1C','',66706,'Reentered','2007-07-05 00:00:00','2007-01-27 00:00:00',104.22,456,1460,99.71,'Sun-Synch' from dual union all
select 30177,'030177','1999-025UJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',106.71,708,1441,98.6,'Sun-Synch' from dual union all
select 30178,'030178','1999-025UK','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',107.09,858,1326,102.28,'Sun-Synch' from dual union all
select 30179,'030179','1999-025UL','deb FY-1C','',66706,'Reentered','2014-08-28 00:00:00','2007-01-27 00:00:00',110.34,771,1714,99.08,'Sun-Synch' from dual union all
select 3018,'003018','1967-106C','8S692','8S692',15361,'Reentered','1967-10-28 00:00:00','1967-10-28 00:00:00',88.28,139,240,49.6,'Intermediate' from dual union all
select 30180,'030180','1999-025UM','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.16,673,860,99.06,'Sun-Synch' from dual union all
select 30181,'030181','1999-025UN','deb FY-1C','',66706,'In Earth orbit','','2007-01-28 00:00:00',102.52,838,918,98.82,'Sun-Synch' from dual union all
select 30182,'030182','1999-025UP','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.27,857,969,98.87,'Sun-Synch' from dual union all
select 30183,'030183','1999-025UQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-28 00:00:00',102.54,857,901,98.82,'Sun-Synch' from dual union all
select 30184,'030184','1999-025UR','deb FY-1C','',66706,'Reentered','2012-01-23 00:00:00','2007-01-27 00:00:00',98.76,510,889,97.44,'Sun-Synch' from dual union all
select 30185,'030185','1999-025US','deb FY-1C','',66706,'Reentered','2014-01-04 00:00:00','2007-01-27 00:00:00',97.69,430,867,99.72,'Sun-Synch' from dual union all
select 30186,'030186','1999-025UT','deb FY-1C','',66706,'Reentered','2014-03-12 00:00:00','2007-01-27 00:00:00',97.9,458,859,98.92,'Sun-Synch' from dual union all
select 30187,'030187','1999-025UU','deb FY-1C','',66706,'Reentered','2011-09-16 00:00:00','2007-01-27 00:00:00',98.07,477,855,98.83,'Sun-Synch' from dual union all
select 30188,'030188','1999-025UV','deb FY-1C','',66706,'Reentered','2013-09-04 00:00:00','2007-01-27 00:00:00',98.06,390,942,98.18,'Sun-Synch' from dual union all
select 30189,'030189','1999-025UW','deb FY-1C','',66706,'Reentered','2015-04-12 00:00:00','2007-01-27 00:00:00',98.52,474,902,99.1,'Sun-Synch' from dual union all
select 3019,'003019','1967-104B','8S672','8S672',15359,'Reentered','2002-12-27 00:00:00','1967-12-05 00:00:00',98.65,517,872,64.08,'Intermediate' from dual union all
select 30190,'030190','1999-025UX','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',99.48,612,856,99.02,'Sun-Synch' from dual union all
select 30191,'030191','1999-025UY','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',99.72,628,863,98.87,'Sun-Synch' from dual union all
select 30192,'030192','1999-025UZ','deb FY-1C','',66706,'Reentered','2016-11-12 00:00:00','2007-01-28 00:00:00',100.48,701,862,99.03,'Sun-Synch' from dual union all
select 30193,'030193','1999-025VA','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.52,691,876,98.92,'Sun-Synch' from dual union all
select 30194,'030194','1999-025VB','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.96,799,904,101.28,'Sun-Synch' from dual union all
select 30195,'030195','1999-025VC','deb FY-1C','',66706,'Reentered','2015-06-18 00:00:00','2007-01-27 00:00:00',100.52,704,863,102.09,'Sun-Synch' from dual union all
select 30196,'030196','1999-025VD','deb FY-1C','',66706,'In Earth orbit','','2007-01-28 00:00:00',101.07,747,872,99.02,'Sun-Synch' from dual union all
select 30197,'030197','1999-025VE','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.28,771,868,99.09,'Sun-Synch' from dual union all
select 30198,'030198','1999-025VF','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.49,711,948,98.29,'Sun-Synch' from dual union all
select 30199,'030199','1999-025VG','deb FY-1C','',66706,'In Earth orbit','','2007-01-28 00:00:00',101.41,753,898,98.79,'Sun-Synch' from dual union all
select 302,'000302','62 Phi 1','FTV 1128','CORONA M-4',8198,'Reentered Attached','1962-06-11 00:00:00','1962-06-04 00:00:00',89.57,192,314,74.1,'Intermediate' from dual union all
select 3020,'003020','1967-107A','Kosmos-188','Soyuz No. 5',15362,'Deorbited','1967-11-02 00:00:00','1967-11-01 00:00:00',88.82,189,244,51.65,'Intermediate' from dual union all
select 30200,'030200','1999-025VH','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.5,719,941,99.11,'Sun-Synch' from dual union all
select 30201,'030201','1999-025VJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.75,818,865,99.46,'Sun-Synch' from dual union all
select 30202,'030202','1999-025VK','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.85,796,897,98.79,'Sun-Synch' from dual union all
select 30203,'030203','1999-025VL','deb FY-1C','',66706,'Reentered','2012-01-09 00:00:00','2007-01-27 00:00:00',99.08,572,858,99.06,'Sun-Synch' from dual union all
select 30204,'030204','1999-025VM','deb FY-1C','',66706,'Reentered','2015-02-21 00:00:00','2007-01-27 00:00:00',99.58,497,980,99.84,'Sun-Synch' from dual union all
select 30205,'030205','1999-025VN','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.98,855,1039,98.82,'Sun-Synch' from dual union all
select 30206,'030206','1999-025VP','deb FY-1C','',66706,'In Earth orbit','','2007-01-24 00:00:00',100.18,646,889,98.52,'Sun-Synch' from dual union all
select 30207,'030207','1999-025VQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',104.6,780,1172,98.5,'Sun-Synch' from dual union all
select 30208,'030208','1999-025VR','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.76,816,868,98.83,'Sun-Synch' from dual union all
select 30209,'030209','1999-025VS','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.4,843,902,98.99,'Sun-Synch' from dual union all
select 3021,'003021','1967-108A','Kosmos-189','Tselina-O GVM',15363,'Reentered','1978-06-08 00:00:00','1967-12-02 00:00:00',95.65,526,574,74.01,'Intermediate' from dual union all
select 30210,'030210','1999-025VT','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.97,847,952,98.73,'Sun-Synch' from dual union all
select 30211,'030211','1999-025VU','deb FY-1C','',66706,'Reentered','2010-10-18 00:00:00','2007-01-27 00:00:00',105.1,712,1287,99.41,'Sun-Synch' from dual union all
select 30212,'030212','1999-025VV','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.38,845,991,99.51,'Sun-Synch' from dual union all
select 30213,'030213','1999-025VW','deb FY-1C','',66706,'In Earth orbit','','2007-01-26 00:00:00',103.71,843,1025,98.95,'Sun-Synch' from dual union all
select 30214,'030214','1999-025VX','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.43,850,898,98.92,'Sun-Synch' from dual union all
select 30215,'030215','1999-025VY','deb FY-1C','',66706,'Reentered','2011-09-24 00:00:00','2007-01-27 00:00:00',105.04,847,1146,99.05,'Sun-Synch' from dual union all
select 30216,'030216','1999-025VZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.17,674,860,99.11,'Sun-Synch' from dual union all
select 30217,'030217','1999-025WA','deb FY-1C','',66706,'In Earth orbit','','2007-01-28 00:00:00',101.46,720,936,98.99,'Sun-Synch' from dual union all
select 30218,'030218','1999-025WB','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.1,829,982,98.91,'Sun-Synch' from dual union all
select 30219,'030219','1999-025WC','deb FY-1C','',66706,'Reentered','2012-07-31 00:00:00','2007-01-27 00:00:00',98.82,537,868,99.03,'Sun-Synch' from dual union all
select 3022,'003022','1967-107B','11S510','11S510',15362,'Reentered','1967-11-02 00:00:00','1967-10-31 00:00:00',88.69,185,235,51.64,'Intermediate' from dual union all
select 30220,'030220','1999-025WD','deb FY-1C','',66706,'Reentered','2011-12-04 00:00:00','2007-01-28 00:00:00',99.28,574,875,98.91,'Sun-Synch' from dual union all
select 30221,'030221','1999-025WE','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.15,650,882,99.06,'Sun-Synch' from dual union all
select 30222,'030222','1999-025WF','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.64,856,912,98.81,'Sun-Synch' from dual union all
select 30223,'030223','1999-025WG','deb FY-1C','',66706,'Reentered','2009-10-29 00:00:00','2007-01-28 00:00:00',103.55,785,1068,99.12,'Sun-Synch' from dual union all
select 30224,'030224','1999-025WH','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',103.91,857,1030,98.91,'Sun-Synch' from dual union all
select 30225,'030225','1999-025WJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.97,777,927,98.7,'Sun-Synch' from dual union all
select 30226,'030226','1999-025WK','deb FY-1C','',66706,'Reentered','2015-12-12 00:00:00','2007-01-27 00:00:00',99.64,625,858,99.01,'Sun-Synch' from dual union all
select 30227,'030227','1999-025WL','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.08,857,858,99.13,'Sun-Synch' from dual union all
select 30228,'030228','1999-025WM','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.45,699,861,99.04,'Sun-Synch' from dual union all
select 30229,'030229','1999-025WN','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.19,765,865,99.14,'Sun-Synch' from dual union all
select 3023,'003023','1967-108B','S3M','S3M',15363,'Reentered','1980-03-07 00:00:00','1967-12-03 00:00:00',95.75,530,580,74.02,'Intermediate' from dual union all
select 30230,'030230','1999-025WP','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',101.88,728,968,99.05,'Sun-Synch' from dual union all
select 30231,'030231','1999-025WQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',102.7,783,989,98.83,'Sun-Synch' from dual union all
select 30232,'030232','1999-025WR','deb FY-1C','',66706,'In Earth orbit','','2007-01-27 00:00:00',100.11,624,903,98.77,'Sun-Synch' from dual union all
select 30233,'030233','1999-025WS','deb FY-1C','',66706,'In Earth orbit','','2007-01-28 00:00:00',101.15,760,867,99.09,'Sun-Synch' from dual union all
select 30234,'030234','1999-025WT','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',102.03,804,906,98.81,'Sun-Synch' from dual union all
select 30235,'030235','1999-025WU','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',102.64,764,1004,98.9,'Sun-Synch' from dual union all
select 30236,'030236','1999-025WV','deb FY-1C','',66706,'Reentered','2008-09-28 00:00:00','2007-01-31 00:00:00',124.24,745,2995,100.06,'Medium' from dual union all
select 30237,'030237','1999-025WW','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',102.42,855,892,98.82,'Sun-Synch' from dual union all
select 30238,'030238','1999-025WX','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',101.02,743,871,98.96,'Sun-Synch' from dual union all
select 30239,'030239','1999-025WY','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',132.71,852,3629,99.83,'Medium' from dual union all
select 3024,'003024','1967-109A','OPS 0562','CORONA J-41',15364,'Reentered Attached','1967-12-02 00:00:00','1967-11-10 00:00:00',90.24,178,394,81.54,'Intermediate' from dual union all
select 30240,'030240','1999-025WZ','deb FY-1C','',66706,'Reentered','2010-02-04 00:00:00','2007-01-31 00:00:00',126.55,861,3082,99.68,'Medium' from dual union all
select 30241,'030241','1999-025XA','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',102.52,767,989,99.29,'Sun-Synch' from dual union all
select 30242,'030242','1999-025XB','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.25,856,875,98.79,'Sun-Synch' from dual union all
select 30243,'030243','1999-025XC','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.31,857,879,98.86,'Sun-Synch' from dual union all
select 30244,'030244','1999-025XD','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.2,855,871,99.28,'Sun-Synch' from dual union all
select 30245,'030245','1999-025XE','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',105.22,848,1162,98.91,'Sun-Synch' from dual union all
select 30246,'030246','1999-025XF','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',107.92,847,1414,98.66,'Sun-Synch' from dual union all
select 30247,'030247','1999-025XG','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',103.4,625,1214,98.26,'Sun-Synch' from dual;
insert into SATELLITE
select 30248,'030248','1999-025XH','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',103.05,851,955,98.99,'Sun-Synch' from dual union all
select 30249,'030249','1999-025XJ','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.54,836,922,99.04,'Sun-Synch' from dual union all
select 3025,'003025','1967-109B','FACADE','P-11 No. 4410',15364,'Reentered','1969-03-28 00:00:00','1967-12-04 00:00:00',94.28,448,519,81.68,'Intermediate' from dual union all
select 30250,'030250','1999-025XK','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',103.08,856,953,99.3,'Sun-Synch' from dual union all
select 30251,'030251','1999-025XL','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',101.45,786,868,99.08,'Sun-Synch' from dual union all
select 30252,'030252','1999-025XM','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',102.63,831,935,98.93,'Sun-Synch' from dual union all
select 30253,'030253','1999-025XN','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',106.1,857,1235,98.7,'Sun-Synch' from dual union all
select 30254,'030254','1999-025XP','deb FY-1C','',66706,'Reentered','2013-07-31 00:00:00','2007-01-31 00:00:00',102.08,754,961,99.06,'Sun-Synch' from dual union all
select 30255,'030255','1999-025XQ','deb FY-1C','',66706,'Reentered','2010-04-13 00:00:00','2007-02-01 00:00:00',98.37,456,906,99.02,'Sun-Synch' from dual union all
select 30256,'030256','1999-025XR','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',100.29,646,899,99.07,'Sun-Synch' from dual union all
select 30257,'030257','1999-025XS','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.31,824,912,98.91,'Sun-Synch' from dual union all
select 30258,'030258','1999-025XT','deb FY-1C','',66706,'Reentered','2011-01-15 00:00:00','2007-02-01 00:00:00',98.34,481,877,97.46,'Sun-Synch' from dual union all
select 30259,'030259','1999-025XU','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',100.26,563,979,98.82,'Sun-Synch' from dual union all
select 3026,'003026','1967-110A','Kosmos-190','Zenit-4 No. 35',15365,'Landed','1967-11-11 00:00:00','1967-11-08 00:00:00',89.67,195,322,65.66,'Intermediate' from dual union all
select 30260,'030260','1999-025XV','deb FY-1C','',66706,'Reentered','2011-07-22 00:00:00','2007-01-31 00:00:00',102.02,831,878,98.83,'Sun-Synch' from dual union all
select 30261,'030261','1999-025XW','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',98.32,497,860,98.82,'Sun-Synch' from dual union all
select 30262,'030262','1999-025XX','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',100.28,683,861,99.08,'Sun-Synch' from dual union all
select 30263,'030263','1999-025XY','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',103.89,816,1069,98.79,'Sun-Synch' from dual union all
select 30264,'030264','1999-025XZ','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',103.29,826,1003,99.02,'Sun-Synch' from dual union all
select 30265,'030265','1999-025YA','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',101.82,803,886,98.76,'Sun-Synch' from dual union all
select 30266,'030266','1999-025YB','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',103.23,831,992,98.91,'Sun-Synch' from dual union all
select 30267,'030267','1999-025YC','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.68,830,941,98.87,'Sun-Synch' from dual union all
select 30268,'030268','1999-025YD','deb FY-1C','',66706,'Reentered','2012-01-14 00:00:00','2007-02-01 00:00:00',99.07,536,893,99.01,'Sun-Synch' from dual union all
select 30269,'030269','1999-025YE','deb FY-1C','',66706,'Reentered','2012-08-01 00:00:00','2007-01-31 00:00:00',103.01,837,965,98.87,'Sun-Synch' from dual union all
select 3027,'003027','1967-110B','11A57I','11A57I',15365,'Reentered','1967-11-09 00:00:00','1967-11-05 00:00:00',89.19,180,289,65.67,'Intermediate' from dual union all
select 30270,'030270','1999-025YF','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.13,854,865,98.79,'Sun-Synch' from dual union all
select 30271,'030271','1999-025YG','deb FY-1C','',66706,'Reentered','2012-11-11 00:00:00','2007-02-01 00:00:00',103.04,849,956,98.65,'Sun-Synch' from dual union all
select 30272,'030272','1999-025YH','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',103.09,855,955,99.03,'Sun-Synch' from dual union all
select 30273,'030273','1999-025YJ','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',103.2,856,964,99.06,'Sun-Synch' from dual union all
select 30274,'030274','1999-025YK','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.01,850,858,99.37,'Sun-Synch' from dual union all
select 30275,'030275','1999-025YL','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.08,813,901,98.86,'Sun-Synch' from dual union all
select 30276,'030276','1999-025YM','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.44,851,898,98.77,'Sun-Synch' from dual union all
select 30277,'030277','1999-025YN','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',103.15,797,1019,98.91,'Sun-Synch' from dual union all
select 30278,'030278','1999-025YP','deb FY-1C','',66706,'Reentered','2017-02-21 00:00:00','2007-02-01 00:00:00',104.93,847,1136,98.98,'Sun-Synch' from dual union all
select 30279,'030279','1999-025YQ','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.21,679,1048,99.3,'Sun-Synch' from dual union all
select 3028,'003028','1967-108C','deb Kosmos-189','Tselina-O part',15363,'Reentered','1968-06-18 00:00:00','1967-12-01 00:00:00',95.45,516,565,74.06,'Intermediate' from dual union all
select 30280,'030280','1999-025YR','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',103.21,850,971,99.36,'Sun-Synch' from dual union all
select 30281,'030281','1999-025YS','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.36,848,893,99,'Sun-Synch' from dual union all
select 30282,'030282','1999-025YT','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.23,705,1024,99.19,'Sun-Synch' from dual union all
select 30283,'030283','1999-025YU','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.88,746,1044,99.1,'Sun-Synch' from dual union all
select 30284,'030284','1999-025YV','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',102.49,840,912,98.98,'Sun-Synch' from dual union all
select 30285,'030285','1999-025YW','deb FY-1C','',66706,'Reentered','2011-11-14 00:00:00','2007-01-31 00:00:00',108.83,640,1706,99.75,'Sun-Synch' from dual union all
select 30286,'030286','1999-025YX','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',102.04,808,903,98.82,'Sun-Synch' from dual union all
select 30287,'030287','1999-025YY','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',106.27,713,1395,98.65,'Sun-Synch' from dual union all
select 30288,'030288','1999-025YZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',102.54,857,901,99.45,'Sun-Synch' from dual union all
select 30289,'030289','1999-025ZA','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',103.03,857,947,99.12,'Sun-Synch' from dual union all
select 3029,'003029','1967-111A','ATS 3','ATS C',15366,'In Earth orbit','','1967-12-06 00:00:00',1445.06,35776,36148,.47,'Drift GEO' from dual union all
select 30290,'030290','1999-025ZB','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',102.68,844,927,102.64,'Sun-Synch' from dual union all
select 30291,'030291','1999-025ZC','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',102.64,731,1037,97.25,'Sun-Synch' from dual union all
select 30292,'030292','1999-025ZD','deb FY-1C','',66706,'Reentered','2015-09-24 00:00:00','2007-01-31 00:00:00',100.74,611,977,98.47,'Sun-Synch' from dual union all
select 30293,'030293','1999-025ZE','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',100.83,724,872,99.28,'Sun-Synch' from dual union all
select 30294,'030294','1999-025ZF','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',101.07,649,970,98.85,'Sun-Synch' from dual union all
select 30295,'030295','1999-025ZG','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',101.31,751,891,98.59,'Sun-Synch' from dual union all
select 30296,'030296','1999-025ZH','deb FY-1C','',66706,'Reentered','2011-02-13 00:00:00','2007-01-31 00:00:00',117.59,838,2308,99.1,'Sun-Synch' from dual union all
select 30297,'030297','1999-025ZJ','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',105.67,580,1472,98.73,'Sun-Synch' from dual union all
select 30298,'030298','1999-025ZK','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',103.24,855,969,99.03,'Sun-Synch' from dual union all
select 30299,'030299','1999-025ZL','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',103.73,843,1027,99.05,'Sun-Synch' from dual union all
select 303,'000303','62 Phi 2','CORONA experiment door','',8198,'Reentered','1962-06-02 00:00:00','1962-05-30 00:00:00',89.95,193,351,74.1,'Intermediate' from dual union all
select 3030,'003030','1967-111B','Agena D 6153','Agena D 6153',15366,'Reentered','1968-08-08 00:00:00','1967-12-05 00:00:00',612.85,151,34893,28.26,'GEO Transfer' from dual union all
select 30300,'030300','1999-025ZM','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',103.23,833,990,98.89,'Sun-Synch' from dual union all
select 30301,'030301','1999-025ZN','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',103.22,807,1014,98.62,'Sun-Synch' from dual union all
select 30302,'030302','1999-025ZP','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',104.2,848,1066,98.54,'Sun-Synch' from dual union all
select 30303,'030303','1999-025ZQ','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',103.15,831,985,98.9,'Sun-Synch' from dual union all
select 30304,'030304','1999-025ZR','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',103.18,834,984,98.9,'Sun-Synch' from dual union all
select 30305,'030305','1999-025ZS','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',106.01,842,1241,98.54,'Sun-Synch' from dual union all
select 30306,'030306','1999-025ZT','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',102.2,759,967,98.94,'Sun-Synch' from dual union all
select 30307,'030307','1999-025ZU','deb FY-1C','',66706,'Reentered','2008-12-12 00:00:00','2007-01-31 00:00:00',104.6,814,1138,99.19,'Sun-Synch' from dual union all
select 30308,'030308','1999-025ZV','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',105.1,839,1160,100.16,'Sun-Synch' from dual union all
select 30309,'030309','1999-025ZW','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',103.59,857,1000,98.9,'Sun-Synch' from dual union all
select 3031,'003031','1967-112A','Surveyor 6','Surveyor SC-6',15367,'Deep Space','1967-11-09 00:00:00','1967-11-07 00:00:00',34286.08,169,686276,29.01,'Cislunar' from dual union all
select 30310,'030310','1999-025ZX','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',103.02,704,1099,99.24,'Sun-Synch' from dual union all
select 30311,'030311','1999-025ZY','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',104.74,857,1108,98.95,'Sun-Synch' from dual union all
select 30312,'030312','1999-025ZZ','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',102.33,854,884,98.68,'Sun-Synch' from dual union all
select 30313,'030313','1999-025AAA','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',103.05,851,955,98.97,'Sun-Synch' from dual union all
select 30314,'030314','1999-025AAB','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',103.77,821,1053,98.41,'Sun-Synch' from dual union all
select 30315,'030315','1999-025AAC','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',101.85,820,873,98.77,'Sun-Synch' from dual union all
select 30316,'030316','1999-025AAD','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',101.84,796,896,98.78,'Sun-Synch' from dual union all
select 30317,'030317','1999-025AAE','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.49,857,896,98.94,'Sun-Synch' from dual union all
select 30318,'030318','1999-025AAF','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',101.84,800,892,98.98,'Sun-Synch' from dual union all
select 30319,'030319','1999-025AAG','deb FY-1C','',66706,'In Earth orbit','','2007-01-31 00:00:00',102.97,811,988,98.36,'Sun-Synch' from dual union all
select 3032,'003032','1967-113A','Apollo 4','Apollo CM-017',15368,'Landed','1967-11-09 00:00:00','1967-11-09 00:00:00',24303.53,28,568650,30.45,'Earth Escape' from dual union all
select 30320,'030320','1999-025AAH','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',103.18,843,974,98.78,'Sun-Synch' from dual union all
select 30321,'030321','1999-025AAJ','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',102.01,807,901,98.87,'Sun-Synch' from dual union all
select 30322,'030322','1999-025AAK','deb FY-1C','',66706,'In Earth orbit','','2007-02-01 00:00:00',103.59,819,1038,98.4,'Sun-Synch' from dual union all
select 30323,'030323','2007-003A','Beidou Daohang Shiyan Wei. 4','di si ke beidou daohang',68345,'In Earth orbit','','2008-01-05 00:00:00',1436.14,35265,36310,5.68,'Inclined GEO' from dual union all
select 30324,'030324','2007-003B','CZ-3A Stage 3','CZ-3A Stage 3',68345,'In Earth orbit','','2007-03-05 00:00:00',753.47,225,41876,25,'Highly Ellip' from dual union all
select 30325,'030325','1999-025AAL','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.3,855,879,98.79,'Sun-Synch' from dual union all
select 30326,'030326','1999-025AAM','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',104.08,845,1058,98.58,'Sun-Synch' from dual union all
select 30327,'030327','1999-025AAN','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',103.59,819,1038,98.4,'Sun-Synch' from dual union all
select 30328,'030328','1999-025AAP','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.08,853,862,98.8,'Sun-Synch' from dual union all
select 30329,'030329','1999-025AAQ','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',104.14,846,1062,98.92,'Sun-Synch' from dual union all
select 3033,'003033','1967-113B','Saturn S-IVB-501','Saturn S-IVB-501',15368,'Deorbited','1967-11-09 00:00:00','1967-11-09 00:00:00',88.44,192,203,32.74,'Intermediate' from dual union all
select 30330,'030330','1999-025AAR','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',105.24,846,1166,98.91,'Sun-Synch' from dual union all
select 30331,'030331','1999-025AAS','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',103.41,845,995,99,'Sun-Synch' from dual union all
select 30332,'030332','1999-025AAT','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',104.89,853,1126,99.09,'Sun-Synch' from dual union all
select 30333,'030333','1999-025AAU','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',98.74,540,857,98.95,'Sun-Synch' from dual union all
select 30334,'030334','1999-025AAV','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',104.67,853,1104,98.92,'Sun-Synch' from dual union all
select 30335,'030335','1999-025AAW','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.91,857,936,99.05,'Sun-Synch' from dual union all
select 30336,'030336','1999-025AAX','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',101.89,814,883,98.73,'Sun-Synch' from dual union all
select 30337,'030337','1999-025AAY','deb FY-1C','',66706,'Reentered','2012-02-26 00:00:00','2007-02-03 00:00:00',97.42,414,857,98.89,'Sun-Synch' from dual;
insert into SATELLITE
select 30338,'030338','1999-025AAZ','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',101.99,568,1138,98.74,'Sun-Synch' from dual union all
select 30339,'030339','1999-025ABA','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.63,857,910,98.84,'Sun-Synch' from dual union all
select 3034,'003034','1967-112B','Centaur D AC-14','Centaur D AC-14',15367,'Deep Space','1967-11-09 00:00:00','1967-11-07 00:00:00',15657.08,169,401708,28,'Cislunar' from dual union all
select 30340,'030340','1999-025ABB','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.68,777,994,98.71,'Sun-Synch' from dual union all
select 30341,'030341','1999-025ABC','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',103.11,851,960,99.27,'Sun-Synch' from dual union all
select 30342,'030342','1999-025ABD','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',104.83,854,1119,100.19,'Sun-Synch' from dual union all
select 30343,'030343','1999-025ABE','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',100.02,554,964,99.07,'Sun-Synch' from dual union all
select 30344,'030344','1999-025ABF','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.17,849,874,98.64,'Sun-Synch' from dual union all
select 30345,'030345','1999-025ABG','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.39,853,891,98.66,'Sun-Synch' from dual union all
select 30346,'030346','1999-025ABH','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.62,821,945,99.07,'Sun-Synch' from dual union all
select 30347,'030347','1999-025ABJ','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',103.54,814,1038,98.39,'Sun-Synch' from dual union all
select 30348,'030348','1999-025ABK','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.49,856,896,98.81,'Sun-Synch' from dual union all
select 30349,'030349','1999-025ABL','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',103.71,856,1011,98.54,'Sun-Synch' from dual union all
select 3035,'003035','1967-114A','ESSA 6','TOS D',15369,'In Earth orbit','','1967-12-19 00:00:00',114.83,1409,1487,102.13,'Sun-Synch' from dual union all
select 30350,'030350','1999-025ABM','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',104.75,809,1157,98.11,'Sun-Synch' from dual union all
select 30351,'030351','1999-025ABN','deb FY-1C','',66706,'Reentered','2014-04-04 00:00:00','2007-02-03 00:00:00',116.81,841,2234,98.72,'Sun-Synch' from dual union all
select 30352,'030352','1999-025ABP','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.53,857,900,98.85,'Sun-Synch' from dual union all
select 30353,'030353','1999-025ABQ','deb FY-1C','',66706,'Reentered','2012-03-23 00:00:00','2007-02-03 00:00:00',97.32,381,880,99.02,'Sun-Synch' from dual union all
select 30354,'030354','1999-025ABR','deb FY-1C','',66706,'Reentered','2010-02-20 00:00:00','2007-02-03 00:00:00',97.45,425,849,98.93,'Sun-Synch' from dual union all
select 30355,'030355','1999-025ABS','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.23,820,909,99.02,'Sun-Synch' from dual union all
select 30356,'030356','1999-025ABT','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.55,841,918,99.1,'Sun-Synch' from dual union all
select 30357,'030357','1999-025ABU','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.61,845,920,98.83,'Sun-Synch' from dual union all
select 30358,'030358','1999-025ABV','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.98,818,982,98.69,'Sun-Synch' from dual union all
select 30359,'030359','1999-025ABW','deb FY-1C','',66706,'In Earth orbit','','2007-02-02 00:00:00',105.14,857,1145,101.63,'Sun-Synch' from dual union all
select 3036,'003036','1967-114B','FW4D','FW4D S/N 41105',15369,'In Earth orbit','','1967-12-19 00:00:00',114.84,1410,1487,102.12,'Sun-Synch' from dual union all
select 30360,'030360','1999-025ABX','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.59,849,914,98.93,'Sun-Synch' from dual union all
select 30361,'030361','1999-025ABY','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',102.18,803,921,99.01,'Sun-Synch' from dual union all
select 30362,'030362','1999-025ABZ','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',103.99,857,1038,98.94,'Sun-Synch' from dual union all
select 30363,'030363','1999-025ACA','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',103.16,831,986,98.9,'Sun-Synch' from dual union all
select 30364,'030364','1999-025ACB','deb FY-1C','',66706,'In Earth orbit','','2007-02-03 00:00:00',104.86,630,1346,98.12,'Sun-Synch' from dual union all
select 30365,'030365','1999-025ACC','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',101.84,717,974,98.86,'Sun-Synch' from dual union all
select 30366,'030366','1999-025ACD','deb FY-1C','',66706,'In Earth orbit','','2007-02-07 00:00:00',101.2,764,867,99.08,'Sun-Synch' from dual union all
select 30367,'030367','1999-025ACE','deb FY-1C','',66706,'Reentered','2011-10-23 00:00:00','2007-02-08 00:00:00',102.15,839,882,98.79,'Sun-Synch' from dual union all
select 30368,'030368','1999-025ACF','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',104.93,853,1129,99,'Sun-Synch' from dual union all
select 30369,'030369','1999-025ACG','deb FY-1C','',66706,'In Earth orbit','','2007-02-07 00:00:00',102.22,789,938,98.91,'Sun-Synch' from dual union all
select 3037,'003037','61 Omicron 213','deb Ablestar','',7273,'Reentered','1968-02-01 00:00:00','1967-11-07 00:00:00',94.63,445,557,67.53,'Intermediate' from dual union all
select 30370,'030370','1999-025ACH','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',102.79,800,982,98.73,'Sun-Synch' from dual union all
select 30371,'030371','1999-025ACJ','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',101.94,844,857,98.81,'Sun-Synch' from dual union all
select 30372,'030372','1999-025ACK','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',100.61,715,860,99.11,'Sun-Synch' from dual union all
select 30373,'030373','1999-025ACL','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',100.4,695,860,99.12,'Sun-Synch' from dual union all
select 30374,'030374','1999-025ACM','deb FY-1C','',66706,'In Earth orbit','','2007-02-07 00:00:00',101.19,764,866,99.33,'Sun-Synch' from dual union all
select 30375,'030375','1999-025ACN','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',102.15,851,870,98.64,'Sun-Synch' from dual union all
select 30376,'030376','1999-025ACP','deb FY-1C','',66706,'Reentered','2009-10-21 00:00:00','2007-02-08 00:00:00',97.54,434,848,98.85,'Sun-Synch' from dual union all
select 30377,'030377','1999-025ACQ','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',103.27,822,1005,99.03,'Sun-Synch' from dual union all
select 30378,'030378','1999-025ACR','deb FY-1C','',66706,'Reentered','2014-03-30 00:00:00','2007-02-08 00:00:00',104.06,842,1059,98.91,'Sun-Synch' from dual union all
select 30379,'030379','1999-025ACS','deb FY-1C','',66706,'In Earth orbit','','2007-02-07 00:00:00',102.58,853,908,98.9,'Sun-Synch' from dual union all
select 3038,'003038','1965-020EN','deb Blok Kosmos','',11770,'In Earth orbit','','1966-01-27 00:00:00',113.44,607,2162,56.34,'Intermediate' from dual union all
select 30380,'030380','1999-025ACT','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',99.97,612,903,98.96,'Sun-Synch' from dual union all
select 30381,'030381','1999-025ACU','deb FY-1C','',66706,'Reentered','2007-10-02 00:00:00','2007-02-08 00:00:00',103.02,855,948,99.01,'Sun-Synch' from dual union all
select 30382,'030382','1999-025ACV','deb FY-1C','',66706,'In Earth orbit','','2007-02-07 00:00:00',102.12,723,994,99.07,'Sun-Synch' from dual union all
select 30383,'030383','1999-025ACW','deb FY-1C','',66706,'Reentered','2011-10-21 00:00:00','2007-02-08 00:00:00',97.21,397,854,99.33,'Sun-Synch' from dual union all
select 30384,'030384','1999-025ACX','deb FY-1C','',66706,'Reentered','2015-03-12 00:00:00','2007-02-08 00:00:00',100.28,600,944,99.02,'Sun-Synch' from dual union all
select 30385,'030385','1999-025ACY','deb FY-1C','',66706,'In Earth orbit','','2007-02-07 00:00:00',100.67,689,892,99.09,'Sun-Synch' from dual union all
select 30386,'030386','1999-025ACZ','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',107.32,850,1356,98.66,'Sun-Synch' from dual union all
select 30387,'030387','1999-025ADA','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',102.09,857,859,98.79,'Sun-Synch' from dual union all
select 30388,'030388','1999-025ADB','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',102.17,681,1042,98.8,'Sun-Synch' from dual union all
select 30389,'030389','1999-025ADC','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',102.32,854,883,98.71,'Sun-Synch' from dual union all
select 3039,'003039','1965-082LG','deb Transtage 4','',11832,'Reentered','1979-02-05 00:00:00','1967-11-17 00:00:00',99.2,685,756,32.16,'Intermediate' from dual union all
select 30390,'030390','1999-025ADD','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',102.62,815,951,98.85,'Sun-Synch' from dual union all
select 30391,'030391','1999-025ADE','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',102.86,827,961,98.97,'Sun-Synch' from dual union all
select 30392,'030392','1999-025ADF','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',102.92,746,1047,98.97,'Sun-Synch' from dual union all
select 30393,'030393','1999-025ADG','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',103.07,851,957,98.76,'Sun-Synch' from dual union all
select 30394,'030394','1999-025ADH','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',103.43,853,989,98.8,'Sun-Synch' from dual union all
select 30395,'030395','1999-025ADJ','deb FY-1C','',66706,'In Earth orbit','','2007-02-07 00:00:00',103.54,801,1050,99.1,'Sun-Synch' from dual union all
select 30396,'030396','1999-025ADK','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',103.91,844,1043,98.24,'Sun-Synch' from dual union all
select 30397,'030397','1999-025ADL','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',104,821,1075,98.4,'Sun-Synch' from dual union all
select 30398,'030398','1999-025ADM','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',104.92,857,1125,99.86,'Sun-Synch' from dual union all
select 30399,'030399','1999-025ADN','deb FY-1C','',66706,'Reentered','2012-03-17 00:00:00','2007-02-08 00:00:00',105.96,682,1397,100,'Sun-Synch' from dual union all
select 304,'000304','62 Chi 1','FTV 1127','CORONA M-5',8186,'Reentered Attached','1962-06-28 00:00:00','1962-06-10 00:00:00',90.2,196,373,74.25,'Intermediate' from dual union all
select 3040,'003040','1965-082LH','deb Transtage 4','',11832,'Reentered','1971-03-29 00:00:00','1967-11-17 00:00:00',98.75,661,737,32.17,'Intermediate' from dual union all
select 30400,'030400','1999-025ADP','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',104.73,751,1212,99.06,'Sun-Synch' from dual union all
select 30401,'030401','1999-025ADQ','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',101.08,729,891,99.06,'Sun-Synch' from dual union all
select 30402,'030402','1999-025ADR','deb FY-1C','',66706,'Reentered','2008-03-21 00:00:00','2007-02-08 00:00:00',97.24,412,842,99,'Sun-Synch' from dual union all
select 30403,'030403','1999-025ADS','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',103.19,832,986,98.9,'Sun-Synch' from dual union all
select 30404,'030404','1999-025ADT','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',108.49,857,1457,99.71,'Sun-Synch' from dual union all
select 30405,'030405','1999-025ADU','deb FY-1C','',66706,'In Earth orbit','','2007-02-07 00:00:00',102.5,827,927,99.1,'Sun-Synch' from dual union all
select 30406,'030406','1999-025ADV','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',106.99,855,1320,98.6,'Sun-Synch' from dual union all
select 30407,'030407','1999-025ADW','deb FY-1C','',66706,'Reentered','2014-02-12 00:00:00','2007-02-08 00:00:00',97.87,452,862,98.8,'Sun-Synch' from dual union all
select 30408,'030408','1999-025ADX','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',104.28,855,1067,99.04,'Sun-Synch' from dual union all
select 30409,'030409','1999-025ADY','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',103.22,780,1042,98.52,'Sun-Synch' from dual union all
select 3041,'003041','1967-108D','deb Kosmos-189','Tselina-O part',15363,'Reentered','1967-12-23 00:00:00','1967-11-30 00:00:00',95.47,523,560,74,'Intermediate' from dual union all
select 30410,'030410','1999-025ADZ','deb FY-1C','',66706,'Reentered','2013-09-15 00:00:00','2007-02-08 00:00:00',103.45,739,1105,99.71,'Sun-Synch' from dual union all
select 30411,'030411','1999-025AEA','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',104.07,833,1069,98.54,'Sun-Synch' from dual union all
select 30412,'030412','1999-025AEB','deb FY-1C','',66706,'Reentered','2010-02-13 00:00:00','2007-02-08 00:00:00',97.54,434,848,98.86,'Sun-Synch' from dual union all
select 30413,'030413','1999-025AEC','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',102.97,853,945,102.1,'Sun-Synch' from dual union all
select 30414,'030414','1999-025AED','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',102.99,831,970,99.1,'Sun-Synch' from dual union all
select 30415,'030415','1999-025AEE','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',106.1,855,1237,99.56,'Sun-Synch' from dual union all
select 30416,'030416','1999-025AEF','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',102.92,850,944,98.94,'Sun-Synch' from dual union all
select 30417,'030417','1999-025AEG','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',102.12,810,908,98.96,'Sun-Synch' from dual union all
select 30418,'030418','1999-025AEH','deb FY-1C','',66706,'In Earth orbit','','2007-02-07 00:00:00',99.38,593,864,98.89,'Sun-Synch' from dual union all
select 30419,'030419','1999-025AEJ','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',102.31,823,913,99.35,'Sun-Synch' from dual union all
select 3042,'003042','1967-108E','deb Kosmos-189','Tselina-O part',15363,'Reentered','1968-06-24 00:00:00','1967-11-29 00:00:00',95.16,512,541,74.01,'Intermediate' from dual union all
select 30420,'030420','1999-025AEK','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',98.86,550,859,98.99,'Sun-Synch' from dual union all
select 30421,'030421','1999-025AEL','deb FY-1C','',66706,'Reentered','2015-05-12 00:00:00','2007-02-08 00:00:00',100.48,656,907,99.12,'Sun-Synch' from dual union all
select 30422,'030422','1999-025AEM','deb FY-1C','',66706,'Reentered','2011-07-08 00:00:00','2007-02-08 00:00:00',100.83,717,879,98.81,'Sun-Synch' from dual union all
select 30423,'030423','1999-025AEN','deb FY-1C','',66706,'In Earth orbit','','2007-02-08 00:00:00',101.54,767,897,98.42,'Sun-Synch' from dual union all
select 30424,'030424','1999-025AEP','deb FY-1C','',66706,'Reentered','2011-11-10 00:00:00','2007-02-08 00:00:00',102.6,695,1069,99.65,'Sun-Synch' from dual union all
select 30425,'030425','1998-067AP','P3 thermal shroud','',66503,'Reentered','2007-03-07 00:00:00','2007-02-09 00:00:00',91.2,317,350,51.63,'Intermediate' from dual union all
select 30426,'030426','1998-067AQ','P3 thermal shroud','',66503,'Reentered','2007-03-02 00:00:00','2007-02-09 00:00:00',91.2,319,348,51.63,'Intermediate' from dual union all
select 30427,'030427','1998-067AR','EVA debris','',66503,'Reentered','2007-03-09 00:00:00','2007-02-13 00:00:00',91.01,310,339,51.63,'Intermediate' from dual;
insert into SATELLITE
select 30428,'030428','1998-067AS','EVA debris','',66503,'Reentered','2007-03-10 00:00:00','2007-02-13 00:00:00',91.04,313,339,51.63,'Intermediate' from dual union all
select 30429,'030429','1999-025AEQ','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',100.09,668,858,99.2,'Sun-Synch' from dual union all
select 3043,'003043','1967-115A','Kosmos-191','DS-P1-Yu No. 9',15370,'Reentered','1968-03-02 00:00:00','1967-12-22 00:00:00',91.66,261,451,71,'Intermediate' from dual union all
select 30430,'030430','1999-025AER','deb FY-1C','',66706,'Reentered','2008-08-15 00:00:00','2007-02-12 00:00:00',98.2,450,896,99.01,'Sun-Synch' from dual union all
select 30431,'030431','1999-025AES','deb FY-1C','',66706,'Reentered','2017-05-10 00:00:00','2007-02-12 00:00:00',98.72,538,857,98.68,'Sun-Synch' from dual union all
select 30432,'030432','1999-025AET','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',100.87,729,871,98.99,'Sun-Synch' from dual union all
select 30433,'030433','1999-025AEU','deb FY-1C','',66706,'Reentered','2011-03-08 00:00:00','2007-02-13 00:00:00',97.71,447,852,98.79,'Sun-Synch' from dual union all
select 30434,'030434','1999-025AEV','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',103.83,847,1032,99.39,'Sun-Synch' from dual union all
select 30435,'030435','1999-025AEW','deb FY-1C','',66706,'In Earth orbit','','2007-02-12 00:00:00',105.95,856,1222,99.04,'Sun-Synch' from dual union all
select 30436,'030436','1999-025AEX','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',103.65,820,1043,98.4,'Sun-Synch' from dual union all
select 30437,'030437','1999-025AEY','deb FY-1C','',66706,'In Earth orbit','','2007-02-12 00:00:00',100.9,735,868,99.01,'Sun-Synch' from dual union all
select 30438,'030438','1999-025AEZ','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',104.47,857,1083,98.78,'Sun-Synch' from dual union all
select 30439,'030439','1999-025AFA','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.6,852,912,98.93,'Sun-Synch' from dual union all
select 3044,'003044','1967-115B','S1M','S1M',15370,'Reentered','1968-01-14 00:00:00','1967-12-01 00:00:00',91.67,269,444,70.97,'Intermediate' from dual union all
select 30440,'030440','1999-025AFB','deb FY-1C','',66706,'Reentered','2011-11-10 00:00:00','2007-02-13 00:00:00',102.43,836,912,98.73,'Sun-Synch' from dual union all
select 30441,'030441','1999-025AFC','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',104.9,848,1132,98.97,'Sun-Synch' from dual union all
select 30442,'030442','1999-025AFD','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.85,832,955,99.13,'Sun-Synch' from dual union all
select 30443,'030443','1999-025AFE','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.63,854,913,98.65,'Sun-Synch' from dual union all
select 30444,'030444','1999-025AFF','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',103.12,686,1127,99.35,'Sun-Synch' from dual union all
select 30445,'030445','1999-025AFG','deb FY-1C','',66706,'In Earth orbit','','2007-02-12 00:00:00',100.37,683,870,98.95,'Sun-Synch' from dual union all
select 30446,'030446','1999-025AFH','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',101.83,807,884,98.74,'Sun-Synch' from dual union all
select 30447,'030447','1999-025AFJ','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.09,776,940,99.05,'Sun-Synch' from dual union all
select 30448,'030448','1999-025AFK','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.48,855,897,98.81,'Sun-Synch' from dual union all
select 30449,'030449','1999-025AFL','deb FY-1C','',66706,'Reentered','2012-07-22 00:00:00','2007-02-13 00:00:00',99.1,574,858,99.16,'Sun-Synch' from dual union all
select 3045,'003045','1965-070G','deb S3','',11820,'In Earth orbit','','1967-11-21 00:00:00',116.06,1265,1741,55.48,'Intermediate' from dual union all
select 30450,'030450','1999-025AFM','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',100.17,580,954,98.51,'Sun-Synch' from dual union all
select 30451,'030451','1999-025AFN','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',101.04,746,870,98.97,'Sun-Synch' from dual union all
select 30452,'030452','1999-025AFP','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.22,789,938,99.39,'Sun-Synch' from dual union all
select 30453,'030453','1999-025AFQ','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.66,777,992,98.86,'Sun-Synch' from dual union all
select 30454,'030454','1999-025AFR','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',104.24,776,1141,98.13,'Sun-Synch' from dual union all
select 30455,'030455','1999-025AFS','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',98.35,502,858,98.86,'Sun-Synch' from dual union all
select 30456,'030456','1999-025AFT','deb FY-1C','',66706,'Reentered','2007-03-30 00:00:00','2007-02-13 00:00:00',94.73,299,712,99.85,'Sun-Synch' from dual union all
select 30457,'030457','1999-025AFU','deb FY-1C','',66706,'Reentered','2012-05-07 00:00:00','2007-02-12 00:00:00',135,759,3920,99.85,'Medium' from dual union all
select 30458,'030458','1999-025AFV','deb FY-1C','',66706,'Reentered','2013-10-08 00:00:00','2007-02-13 00:00:00',110.46,791,1704,99.23,'Sun-Synch' from dual union all
select 30459,'030459','1999-025AFW','deb FY-1C','',66706,'Reentered','2011-05-10 00:00:00','2007-02-13 00:00:00',96.89,363,857,99.11,'Sun-Synch' from dual union all
select 3046,'003046','1965-082LJ','deb Transtage 4','',11832,'Reentered','1988-10-07 00:00:00','1967-03-30 00:00:00',104.74,737,1227,32.51,'Intermediate' from dual union all
select 30460,'030460','1999-025AFX','deb FY-1C','',66706,'Reentered','2011-04-12 00:00:00','2007-02-13 00:00:00',97.89,464,852,98.75,'Sun-Synch' from dual union all
select 30461,'030461','1999-025AFY','deb FY-1C','',66706,'Reentered','2010-10-08 00:00:00','2007-02-13 00:00:00',97.93,469,851,98.92,'Sun-Synch' from dual union all
select 30462,'030462','1999-025AFZ','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.62,777,989,99.06,'Sun-Synch' from dual union all
select 30463,'030463','1999-025AGA','deb FY-1C','',66706,'Reentered','2010-02-03 00:00:00','2007-02-13 00:00:00',97.39,421,847,98.91,'Sun-Synch' from dual union all
select 30464,'030464','1999-025AGB','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.22,783,945,98.65,'Sun-Synch' from dual union all
select 30465,'030465','1999-025AGC','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.48,855,897,98.81,'Sun-Synch' from dual union all
select 30466,'030466','1999-025AGD','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.53,773,984,99.01,'Sun-Synch' from dual union all
select 30467,'030467','1999-025AGE','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.58,826,936,98.95,'Sun-Synch' from dual union all
select 30468,'030468','1999-025AGF','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',103.01,831,971,98.92,'Sun-Synch' from dual union all
select 30469,'030469','1999-025AGG','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',104.04,818,1081,98.48,'Sun-Synch' from dual union all
select 3047,'003047','1967-116A','Kosmos-192','Tsiklon',15371,'In Earth orbit','','1968-01-15 00:00:00',99.8,743,755,74.02,'Intermediate' from dual union all
select 30470,'030470','1999-025AGH','deb FY-1C','',66706,'Reentered','2016-06-26 00:00:00','2007-02-13 00:00:00',104.25,840,1078,98.98,'Sun-Synch' from dual union all
select 30471,'030471','1999-025AGJ','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',103.36,838,996,99,'Sun-Synch' from dual union all
select 30472,'030472','1999-025AGK','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',101.91,801,898,98.8,'Sun-Synch' from dual union all
select 30473,'030473','1999-025AGL','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.28,854,880,98.71,'Sun-Synch' from dual union all
select 30474,'030474','1999-025AGM','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.6,855,909,98.75,'Sun-Synch' from dual union all
select 30475,'030475','1999-025AGN','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.7,851,922,98.96,'Sun-Synch' from dual union all
select 30476,'030476','1999-025AGP','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',102.94,852,944,98.75,'Sun-Synch' from dual union all
select 30477,'030477','1999-025AGQ','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',109.6,856,1561,99.38,'Sun-Synch' from dual union all
select 30478,'030478','1999-025AGR','deb FY-1C','',66706,'In Earth orbit','','2007-02-13 00:00:00',100.45,698,862,98.99,'Sun-Synch' from dual union all
select 30479,'030479','2007-003C','deb CZ-3','',68345,'In Earth orbit','','2007-02-13 00:00:00',750.91,269,41708,25.16,'Highly Ellip' from dual union all
select 3048,'003048','1967-116B','S3M','S3M',15371,'In Earth orbit','','1967-12-25 00:00:00',99.83,744,757,74.02,'Intermediate' from dual union all
select 30480,'030480','1999-025AGS','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',100.89,672,930,98.92,'Sun-Synch' from dual union all
select 30481,'030481','1999-025AGT','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',101.06,619,999,98.61,'Sun-Synch' from dual union all
select 30482,'030482','1999-025AGU','deb FY-1C','',66706,'Reentered','2009-10-31 00:00:00','2007-02-17 00:00:00',97.43,337,935,99.39,'Sun-Synch' from dual union all
select 30483,'030483','1999-025AGV','deb FY-1C','',66706,'In Earth orbit','','2007-02-16 00:00:00',100.23,592,946,99.03,'Sun-Synch' from dual union all
select 30484,'030484','1999-025AGW','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',100.35,692,859,99.06,'Sun-Synch' from dual union all
select 30485,'030485','1999-025AGX','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',101.78,727,959,99.04,'Sun-Synch' from dual union all
select 30486,'030486','1999-025AGY','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',98.67,531,860,98.89,'Sun-Synch' from dual union all
select 30487,'030487','1999-025AGZ','deb FY-1C','',66706,'Reentered','2014-03-01 00:00:00','2007-02-17 00:00:00',98.05,474,857,98.74,'Sun-Synch' from dual union all
select 30488,'030488','1999-025AHA','deb FY-1C','',66706,'Reentered','2014-02-06 00:00:00','2007-02-17 00:00:00',99.29,588,861,99.03,'Sun-Synch' from dual union all
select 30489,'030489','1999-025AHB','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.03,749,961,99.16,'Sun-Synch' from dual union all
select 3049,'003049','1965-082LK','deb Transtage 4','',11832,'Reentered','1968-01-06 00:00:00','1967-11-24 00:00:00',92.84,371,457,32.3,'Intermediate' from dual union all
select 30490,'030490','1999-025AHC','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',100.94,605,1002,98.66,'Sun-Synch' from dual union all
select 30491,'030491','1999-025AHD','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.23,834,895,101.54,'Sun-Synch' from dual union all
select 30492,'030492','1999-025AHE','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',103.74,795,1076,102.31,'Sun-Synch' from dual union all
select 30493,'030493','1999-025AHF','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',104.91,833,1148,98.61,'Sun-Synch' from dual union all
select 30494,'030494','1999-025AHG','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',105.29,857,1159,99.51,'Sun-Synch' from dual union all
select 30495,'030495','1999-025AHH','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',112.38,852,1820,102.28,'Sun-Synch' from dual union all
select 30496,'030496','1999-025AHJ','deb FY-1C','',66706,'In Earth orbit','','2007-02-16 00:00:00',98.77,541,859,98.87,'Sun-Synch' from dual union all
select 30497,'030497','1999-025AHK','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',101.07,705,913,99.18,'Sun-Synch' from dual union all
select 30498,'030498','1999-025AHL','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',104.04,857,1042,99.08,'Sun-Synch' from dual union all
select 30499,'030499','1999-025AHM','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',101.01,673,940,99.04,'Sun-Synch' from dual union all
select 305,'000305','62 Chi 2','OSCAR 2','OSCAR 2',8186,'Reentered','1962-06-21 00:00:00','1962-06-10 00:00:00',89.78,188,339,74.29,'Intermediate' from dual union all
select 3050,'003050','1965-082LL','deb Transtage 4','',11832,'Reentered','1979-07-03 00:00:00','1966-06-26 00:00:00',101.83,717,974,31.96,'Intermediate' from dual union all
select 30500,'030500','1999-025AHN','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',103.39,852,985,98.94,'Sun-Synch' from dual union all
select 30501,'030501','1999-025AHP','deb FY-1C','',66706,'In Earth orbit','','2007-02-16 00:00:00',102.79,851,930,98.75,'Sun-Synch' from dual union all
select 30502,'030502','1999-025AHQ','deb FY-1C','',66706,'Reentered','2012-02-08 00:00:00','2007-02-16 00:00:00',98.36,506,855,98.95,'Sun-Synch' from dual union all
select 30503,'030503','1999-025AHR','deb FY-1C','',66706,'In Earth orbit','','2007-02-16 00:00:00',102.03,848,862,99.22,'Sun-Synch' from dual union all
select 30504,'030504','1999-025AHS','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.02,764,945,98.54,'Sun-Synch' from dual union all
select 30505,'030505','1999-025AHT','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.9,835,957,98.88,'Sun-Synch' from dual union all
select 30506,'030506','1999-025AHU','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.53,822,934,98.94,'Sun-Synch' from dual union all
select 30507,'030507','1999-025AHV','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',103.14,797,1018,98.9,'Sun-Synch' from dual union all
select 30508,'030508','1999-025AHW','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',103.18,648,1170,99.28,'Sun-Synch' from dual union all
select 30509,'030509','1999-025AHX','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',103.97,855,1037,98.47,'Sun-Synch' from dual union all
select 3051,'003051','1967-114C','ESSA 6 despin weight','',15369,'In Earth orbit','','1968-01-09 00:00:00',114.14,1347,1486,101.78,'Sun-Synch' from dual union all
select 30510,'030510','1999-025AHY','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',103.4,855,984,98.85,'Sun-Synch' from dual union all
select 30511,'030511','1999-025AHZ','deb FY-1C','',66706,'Reentered','2010-02-18 00:00:00','2007-02-17 00:00:00',97.4,424,845,98.9,'Sun-Synch' from dual union all
select 30512,'030512','1999-025AJA','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',98.56,522,858,98.86,'Sun-Synch' from dual union all
select 30513,'030513','1999-025AJB','deb FY-1C','',66706,'Reentered','2018-02-24 00:00:00','2007-02-17 00:00:00',100.31,652,895,99.08,'Sun-Synch' from dual union all
select 30514,'030514','1999-025AJC','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',101.36,676,971,98.65,'Sun-Synch' from dual union all
select 30515,'030515','1999-025AJD','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',106.73,665,1486,98.49,'Sun-Synch' from dual union all
select 30516,'030516','1999-025AJE','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.4,855,890,98.79,'Sun-Synch' from dual union all
select 30517,'030517','1999-025AJF','deb FY-1C','',66706,'Reentered','2010-01-15 00:00:00','2007-02-17 00:00:00',97.42,424,847,98.93,'Sun-Synch' from dual;
insert into SATELLITE
select 30518,'030518','1999-025AJG','deb FY-1C','',66706,'Reentered','2011-02-13 00:00:00','2007-02-17 00:00:00',98.1,476,860,99.1,'Sun-Synch' from dual union all
select 30519,'030519','1999-025AJH','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',98.8,545,858,98.87,'Sun-Synch' from dual union all
select 3052,'003052','1967-117A','Kosmos-193','Zenit-2 No. 58',15372,'Landed','1967-12-03 00:00:00','1967-11-29 00:00:00',89.78,197,331,65.64,'Intermediate' from dual union all
select 30520,'030520','1999-025AJJ','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',99.07,570,859,98.8,'Sun-Synch' from dual union all
select 30521,'030521','1999-025AJK','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',100.06,634,889,99.05,'Sun-Synch' from dual union all
select 30522,'030522','1999-025AJL','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',100.65,719,859,99.44,'Sun-Synch' from dual union all
select 30523,'030523','1999-025AJM','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',101.26,631,1006,98.9,'Sun-Synch' from dual union all
select 30524,'030524','1999-025AJN','deb FY-1C','',66706,'In Earth orbit','','2007-02-16 00:00:00',106.32,855,1257,99.33,'Sun-Synch' from dual union all
select 30525,'030525','1999-025AJP','deb FY-1C','',66706,'In Earth orbit','','2007-02-16 00:00:00',106.86,841,1321,98.58,'Sun-Synch' from dual union all
select 30526,'030526','1999-025AJQ','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',107.26,857,1342,99.45,'Sun-Synch' from dual union all
select 30527,'030527','1999-025AJR','deb FY-1C','',66706,'In Earth orbit','','2007-02-16 00:00:00',108.94,848,1508,100.42,'Sun-Synch' from dual union all
select 30528,'030528','1999-025AJS','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.38,757,986,98.22,'Sun-Synch' from dual union all
select 30529,'030529','1999-025AJT','deb FY-1C','',66706,'In Earth orbit','','2007-02-16 00:00:00',105.92,846,1229,98.45,'Sun-Synch' from dual union all
select 3053,'003053','1967-117B','11A57I','11A57I',15372,'Reentered','1967-12-01 00:00:00','1967-11-27 00:00:00',89.47,185,312,65.64,'Intermediate' from dual union all
select 30530,'030530','1999-025AJU','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',100.69,714,869,98.42,'Sun-Synch' from dual union all
select 30531,'030531','1999-025AJV','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.73,848,928,102.34,'Sun-Synch' from dual union all
select 30532,'030532','1999-025AJW','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',103.09,774,1035,96.98,'Sun-Synch' from dual union all
select 30533,'030533','1999-025AJX','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.41,758,987,98.95,'Sun-Synch' from dual union all
select 30534,'030534','1999-025AJY','deb FY-1C','',66706,'Reentered','2008-10-03 00:00:00','2007-02-17 00:00:00',96.52,340,844,98.89,'Sun-Synch' from dual union all
select 30535,'030535','1999-025AJZ','deb FY-1C','',66706,'Reentered','2009-09-18 00:00:00','2007-02-17 00:00:00',97.27,414,842,98.9,'Sun-Synch' from dual union all
select 30536,'030536','1999-025AKA','deb FY-1C','',66706,'Reentered','2010-01-01 00:00:00','2007-02-16 00:00:00',97.37,420,846,98.91,'Sun-Synch' from dual union all
select 30537,'030537','1999-025AKB','deb FY-1C','',66706,'Reentered','2011-09-03 00:00:00','2007-02-17 00:00:00',97.94,467,854,99.5,'Sun-Synch' from dual union all
select 30538,'030538','1999-025AKC','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',98.96,443,975,99.22,'Sun-Synch' from dual union all
select 30539,'030539','1999-025AKD','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',100.32,663,885,98.94,'Sun-Synch' from dual union all
select 3054,'003054','1967-118A','WRESAT','WRESAT',15373,'Reentered Attached','1968-01-10 00:00:00','1967-12-06 00:00:00',97.64,173,1119,83.3,'Intermediate' from dual union all
select 30540,'030540','1999-025AKE','deb FY-1C','',66706,'In Earth orbit','','2007-02-16 00:00:00',102.63,827,940,99.08,'Sun-Synch' from dual union all
select 30541,'030541','1999-025AKF','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.39,855,888,98.91,'Sun-Synch' from dual union all
select 30542,'030542','1999-025AKG','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.5,856,898,98.81,'Sun-Synch' from dual union all
select 30543,'030543','1999-025AKH','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.79,839,943,98.99,'Sun-Synch' from dual union all
select 30544,'030544','1999-025AKJ','deb FY-1C','',66706,'Reentered','2011-05-03 00:00:00','2007-02-16 00:00:00',102.8,774,1008,99.01,'Sun-Synch' from dual union all
select 30545,'030545','1999-025AKK','deb FY-1C','',66706,'In Earth orbit','','2007-02-16 00:00:00',109.27,828,1558,99,'Sun-Synch' from dual union all
select 30546,'030546','1999-025AKL','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',99.51,611,860,98.96,'Sun-Synch' from dual union all
select 30547,'030547','1999-025AKM','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',100.61,713,862,98.92,'Sun-Synch' from dual union all
select 30548,'030548','1999-025AKN','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.78,822,959,99.74,'Sun-Synch' from dual union all
select 30549,'030549','1999-025AKP','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',104.64,718,1238,98.44,'Sun-Synch' from dual union all
select 3055,'003055','1967-119A','Kosmos-194','Zenit-4 No. 39',15374,'Landed','1967-12-11 00:00:00','1967-12-08 00:00:00',89.54,198,306,65.66,'Intermediate' from dual union all
select 30550,'030550','1999-025AKQ','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',104.65,856,1100,99.07,'Sun-Synch' from dual union all
select 30551,'030551','1999-025AKR','deb FY-1C','',66706,'Reentered','2009-12-31 00:00:00','2007-02-17 00:00:00',104.9,682,1298,99.54,'Sun-Synch' from dual union all
select 30552,'030552','1999-025AKS','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.34,851,887,98.94,'Sun-Synch' from dual union all
select 30553,'030553','1999-025AKT','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',101.51,798,863,99.19,'Sun-Synch' from dual union all
select 30554,'030554','1999-025AKU','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',101.63,805,866,98.82,'Sun-Synch' from dual union all
select 30555,'030555','1999-025AKV','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.75,846,931,98.98,'Sun-Synch' from dual union all
select 30556,'030556','1999-025AKW','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',103.11,857,955,99.31,'Sun-Synch' from dual union all
select 30557,'030557','1999-025AKX','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',103.36,857,978,99.21,'Sun-Synch' from dual union all
select 30558,'030558','1999-025AKY','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',109.67,854,1569,99.47,'Sun-Synch' from dual union all
select 30559,'030559','1999-025AKZ','deb FY-1C','',66706,'Reentered','2007-08-10 00:00:00','2007-02-17 00:00:00',126.08,655,3247,99.48,'Medium' from dual union all
select 3056,'003056','1967-119B','11A57I','11A57I',15374,'Reentered','1967-12-09 00:00:00','1967-12-05 00:00:00',89.38,192,296,65.66,'Intermediate' from dual union all
select 30560,'030560','1999-025ALA','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',101.98,764,941,98.89,'Sun-Synch' from dual union all
select 30561,'030561','1999-025ALB','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.39,826,917,98.93,'Sun-Synch' from dual union all
select 30562,'030562','1999-025ALC','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.7,831,942,99.09,'Sun-Synch' from dual union all
select 30563,'030563','1999-025ALD','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',100.62,708,868,99.13,'Sun-Synch' from dual union all
select 30564,'030564','1999-025ALE','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',103.22,740,1082,99.2,'Sun-Synch' from dual union all
select 30565,'030565','1999-025ALF','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',103.98,814,1080,98.46,'Sun-Synch' from dual union all
select 30566,'030566','1999-025ALG','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',103.37,854,982,98.85,'Sun-Synch' from dual union all
select 30567,'030567','1999-025ALH','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.55,797,962,99.04,'Sun-Synch' from dual union all
select 30568,'030568','1999-025ALJ','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.33,847,891,99.01,'Sun-Synch' from dual union all
select 30569,'030569','1999-025ALK','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',101.81,823,865,98.75,'Sun-Synch' from dual union all
select 3057,'003057','1967-120A','OV3-6','OV3-6',15375,'Reentered','1969-03-09 00:00:00','1968-01-07 00:00:00',92.94,403,434,90.66,'Polar' from dual union all
select 30570,'030570','1999-025ALL','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.35,841,899,98.8,'Sun-Synch' from dual union all
select 30571,'030571','1999-025ALM','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.37,855,887,98.8,'Sun-Synch' from dual union all
select 30572,'030572','1999-025ALN','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.46,857,894,98.81,'Sun-Synch' from dual union all
select 30573,'030573','1999-025ALP','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',103.57,846,1009,98.91,'Sun-Synch' from dual union all
select 30574,'030574','1999-025ALQ','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',106.18,855,1245,99.31,'Sun-Synch' from dual union all
select 30575,'030575','1999-025ALR','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',101.28,774,865,99.25,'Sun-Synch' from dual union all
select 30576,'030576','1999-025ALS','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',101.59,732,936,99.15,'Sun-Synch' from dual union all
select 30577,'030577','1999-025ALT','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.58,857,905,98.85,'Sun-Synch' from dual union all
select 30578,'030578','1999-025ALU','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.76,841,938,98.96,'Sun-Synch' from dual union all
select 30579,'030579','1999-025ALV','deb FY-1C','',66706,'In Earth orbit','','2007-02-17 00:00:00',102.81,855,929,98.91,'Sun-Synch' from dual union all
select 3058,'003058','1967-121A','OPS 5000','GAMBIT-3 Mission 4310',15376,'Reentered Attached','1967-12-17 00:00:00','1967-12-08 00:00:00',90.01,132,418,109.58,'Retrograde' from dual union all
select 30580,'030580','2007-004A','THEMIS A','THEMIS P5',68346,'In Earth orbit','','2009-07-02 00:00:00',1642.89,3382,76104,9.25,'Very High' from dual union all
select 30581,'030581','2007-004B','THEMIS B','THEMIS P1',68346,'Deep Space','2007-10-05 00:00:00','2009-07-01 00:00:00',5677.82,2572,195522,10.72,'Deep Space' from dual union all
select 30582,'030582','2007-004C','THEMIS C','THEMIS P2',68346,'Deep Space','2009-08-21 00:00:00','2009-07-03 00:00:00',2856.87,3173,117457,8.75,'Very High' from dual union all
select 30583,'030583','1998-067AT','EVA debris','',66503,'Reentered','2007-05-04 00:00:00','2007-02-23 00:00:00',91.2,319,348,51.64,'Intermediate' from dual union all
select 30584,'030584','1999-025ALW','deb FY-1C','',66706,'Reentered','2010-05-06 00:00:00','2007-02-24 00:00:00',97.44,426,847,98.87,'Sun-Synch' from dual union all
select 30585,'030585','2007-004F','Delta 323','Delta SSPS AJ-10-118K',68346,'Reentered','2007-07-16 00:00:00','2007-02-25 00:00:00',101.45,183,1471,22.12,'Equatorial' from dual union all
select 30586,'030586','2007-005A','JSE reda-2 gouki','IGS Radar-2',68347,'Reentered','2014-04-13 00:00:00','2007-05-20 00:00:00',94.35,485,490,97.34,'Sun-Synch' from dual union all
select 30587,'030587','2007-005B','JSE kougaku-3 g jissho eisei','IGS Optical-3 TestSatell',68347,'Reentered','2013-11-12 00:00:00','2007-05-20 00:00:00',94.34,486,488,97.29,'Sun-Synch' from dual union all
select 30588,'030588','2007-005C','H-2A F12 Stage 2','H-2A Stage 2',68347,'Reentered','2011-04-18 00:00:00','2007-05-20 00:00:00',93.33,378,497,97.22,'Sun-Synch' from dual union all
select 30589,'030589','2007-005D','4/4D-LC Lower Fairing Half','Fairing Half',68347,'Reentered','2010-10-21 00:00:00','2007-05-20 00:00:00',94.23,477,486,97.23,'Sun-Synch' from dual union all
select 3059,'003059','1967-120B','FW4S S/N 30210','FW4S',15375,'Reentered','1968-04-08 00:00:00','1968-01-06 00:00:00',92.71,392,423,90.67,'Polar' from dual union all
select 30590,'030590','2007-005E','4/4D-LC Adapter','Adapter',68347,'Reentered','2011-09-20 00:00:00','2007-05-20 00:00:00',94.25,477,488,97.24,'Sun-Synch' from dual union all
select 30591,'030591','2007-005F','4/4D-LC Lower Fairing Half','Fairing Half',68347,'Reentered','2010-09-25 00:00:00','2007-05-20 00:00:00',94.23,477,486,97.24,'Sun-Synch' from dual union all
select 30592,'030592','1999-025ALX','deb FY-1C','',66706,'Reentered','2011-11-09 00:00:00','2007-02-24 00:00:00',98.99,476,945,97.84,'Sun-Synch' from dual union all
select 30593,'030593','1999-025ALY','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',101.88,833,862,102.4,'Sun-Synch' from dual union all
select 30594,'030594','1999-025ALZ','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',101.84,701,991,99.2,'Sun-Synch' from dual union all
select 30595,'030595','1999-025AMA','deb FY-1C','',66706,'Reentered','2012-04-21 00:00:00','2007-02-24 00:00:00',98.07,478,855,98.93,'Sun-Synch' from dual union all
select 30596,'030596','1999-025AMB','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',101.01,609,1004,98.66,'Sun-Synch' from dual union all
select 30597,'030597','1999-025AMC','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',101.88,739,957,99.04,'Sun-Synch' from dual union all
select 30598,'030598','1999-025AMD','deb FY-1C','',66706,'Reentered','2011-12-07 00:00:00','2007-02-24 00:00:00',102.05,784,928,98.84,'Sun-Synch' from dual union all
select 30599,'030599','1999-025AME','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',102.5,623,1130,98.95,'Sun-Synch' from dual union all
select 306,'000306','62 Chi 3','CORONA experiment door','',8186,'Reentered','1962-06-06 00:00:00','1962-06-02 00:00:00',89.87,177,359,74.26,'Intermediate' from dual union all
select 3060,'003060','1967-120C','OV3-6 despin weight','',15375,'Reentered','1968-06-18 00:00:00','1968-01-09 00:00:00',92.81,396,429,90.69,'Polar' from dual union all
select 30600,'030600','1999-025AMF','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',106.25,675,1430,98.38,'Sun-Synch' from dual union all
select 30601,'030601','1999-025AMG','deb FY-1C','',66706,'Reentered','2009-02-05 00:00:00','2007-02-24 00:00:00',107.05,678,1503,100.04,'Sun-Synch' from dual union all
select 30602,'030602','1999-025AMH','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',102.51,843,912,99.08,'Sun-Synch' from dual union all
select 30603,'030603','1999-025AMJ','deb FY-1C','',66706,'In Earth orbit','','2007-02-23 00:00:00',105.05,845,1149,98.43,'Sun-Synch' from dual union all
select 30604,'030604','1999-025AMK','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',101.64,673,1000,99.08,'Sun-Synch' from dual union all
select 30605,'030605','1999-025AML','deb FY-1C','',66706,'Reentered','2011-02-14 00:00:00','2007-02-22 00:00:00',97.44,342,930,99.41,'Sun-Synch' from dual union all
select 30606,'030606','1999-025AMM','deb FY-1C','',66706,'In Earth orbit','','2007-02-23 00:00:00',100.47,578,984,98.67,'Sun-Synch' from dual union all
select 30607,'030607','1999-025AMN','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',102.2,820,906,98.83,'Sun-Synch' from dual;
insert into SATELLITE
select 30608,'030608','1999-025AMP','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',102.04,850,861,98.7,'Sun-Synch' from dual union all
select 30609,'030609','1999-025AMQ','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',104.04,831,1068,98.53,'Sun-Synch' from dual union all
select 3061,'003061','1967-120D','OV3-6 despin weight','',15375,'Reentered','1967-12-25 00:00:00','1967-12-18 00:00:00',91.78,362,362,90.7,'Polar' from dual union all
select 30610,'030610','1999-025AMR','deb FY-1C','',66706,'Reentered','2014-12-19 00:00:00','2007-02-23 00:00:00',97.29,405,853,98.88,'Sun-Synch' from dual union all
select 30611,'030611','1999-025AMS','deb FY-1C','',66706,'In Earth orbit','','2007-02-23 00:00:00',98.83,453,953,98.7,'Sun-Synch' from dual union all
select 30612,'030612','1999-025AMT','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',99.9,650,857,99.14,'Sun-Synch' from dual union all
select 30613,'030613','1999-025AMU','deb FY-1C','',66706,'In Earth orbit','','2007-02-23 00:00:00',101,608,1004,98.66,'Sun-Synch' from dual union all
select 30614,'030614','1999-025AMV','deb FY-1C','',66706,'Reentered','2013-09-08 00:00:00','2007-02-24 00:00:00',108.65,723,1606,100.13,'Sun-Synch' from dual union all
select 30615,'030615','1999-025AMW','deb FY-1C','',66706,'Reentered','2007-04-05 00:00:00','2007-02-24 00:00:00',94.35,286,689,99.23,'Sun-Synch' from dual union all
select 30616,'030616','1999-025AMX','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',108.62,855,1471,98.63,'Sun-Synch' from dual union all
select 30617,'030617','1999-025AMY','deb FY-1C','',66706,'Reentered','2009-10-30 00:00:00','2007-02-23 00:00:00',97.35,418,846,98.91,'Sun-Synch' from dual union all
select 30618,'030618','1999-025AMZ','deb FY-1C','',66706,'Reentered','2007-08-10 00:00:00','2007-02-24 00:00:00',96.96,397,830,99.34,'Sun-Synch' from dual union all
select 30619,'030619','1999-025ANA','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',100.63,615,962,98.47,'Sun-Synch' from dual union all
select 3062,'003062','1967-120E','deb OV3-6','',15375,'Reentered','1968-03-27 00:00:00','1968-01-04 00:00:00',92.73,374,443,90.67,'Polar' from dual union all
select 30620,'030620','1999-025ANB','deb FY-1C','',66706,'Reentered','2007-04-28 00:00:00','2007-02-24 00:00:00',95.81,353,763,98.92,'Sun-Synch' from dual union all
select 30621,'030621','1999-025ANC','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',104.38,832,1099,98.65,'Sun-Synch' from dual union all
select 30622,'030622','1999-025AND','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',104.03,854,1044,101,'Sun-Synch' from dual union all
select 30623,'030623','1999-025ANE','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',108.66,851,1479,98.84,'Sun-Synch' from dual union all
select 30624,'030624','1999-025ANF','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',102.83,856,928,98.86,'Sun-Synch' from dual union all
select 30625,'030625','1999-025ANG','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',103.49,855,993,98.83,'Sun-Synch' from dual union all
select 30626,'030626','1999-025ANH','deb FY-1C','',66706,'Reentered','2013-10-29 00:00:00','2007-02-23 00:00:00',98.8,545,858,99.02,'Sun-Synch' from dual union all
select 30627,'030627','1999-025ANJ','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',100.4,692,863,98.98,'Sun-Synch' from dual union all
select 30628,'030628','1999-025ANK','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',101.4,793,857,99.4,'Sun-Synch' from dual union all
select 30629,'030629','1999-025ANL','deb FY-1C','',66706,'In Earth orbit','','2007-02-23 00:00:00',102.08,849,866,99.31,'Sun-Synch' from dual union all
select 3063,'003063','1967-122A','OPS 1001','CORONA CR-2',15377,'Reentered Attached','1967-12-25 00:00:00','1967-12-16 00:00:00',88.52,154,249,81.63,'Intermediate' from dual union all
select 30630,'030630','1999-025ANM','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',102.25,844,887,99.23,'Sun-Synch' from dual union all
select 30631,'030631','1999-025ANN','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',102.27,818,915,99.1,'Sun-Synch' from dual union all
select 30632,'030632','1999-025ANP','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',102.27,849,884,98.84,'Sun-Synch' from dual union all
select 30633,'030633','1999-025ANQ','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',103.54,819,1033,98.4,'Sun-Synch' from dual union all
select 30634,'030634','1999-025ANR','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',104.15,833,1077,98.99,'Sun-Synch' from dual union all
select 30635,'030635','1999-025ANS','deb FY-1C','',66706,'Reentered','2009-02-18 00:00:00','2007-02-24 00:00:00',97.18,411,837,98.9,'Sun-Synch' from dual union all
select 30636,'030636','1999-025ANT','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',99.71,631,859,98.84,'Sun-Synch' from dual union all
select 30637,'030637','1999-025ANU','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',100.17,672,862,98.99,'Sun-Synch' from dual union all
select 30638,'030638','1999-025ANV','deb FY-1C','',66706,'Reentered','2011-08-24 00:00:00','2007-02-24 00:00:00',100.76,721,869,98.86,'Sun-Synch' from dual union all
select 30639,'030639','1999-025ANW','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',103.15,832,984,99,'Sun-Synch' from dual union all
select 3064,'003064','1965-082LM','deb Transtage 4','',11832,'Reentered','1979-12-25 00:00:00','1967-12-12 00:00:00',99.91,674,835,31.8,'Intermediate' from dual union all
select 30640,'030640','1999-025ANX','deb FY-1C','',66706,'In Earth orbit','','2007-02-23 00:00:00',103.28,833,994,98.9,'Sun-Synch' from dual union all
select 30641,'030641','1999-025ANY','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',99.13,574,861,99.02,'Sun-Synch' from dual union all
select 30642,'030642','1999-025ANZ','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',100.68,715,867,99.08,'Sun-Synch' from dual union all
select 30643,'030643','1999-025APA','deb FY-1C','',66706,'Reentered','2013-12-18 00:00:00','2007-02-24 00:00:00',97.53,401,879,99,'Sun-Synch' from dual union all
select 30644,'030644','1999-025APB','deb FY-1C','',66706,'Reentered','2011-10-03 00:00:00','2007-02-24 00:00:00',98.05,476,854,98.8,'Sun-Synch' from dual union all
select 30645,'030645','1999-025APC','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',100.45,699,861,99.03,'Sun-Synch' from dual union all
select 30646,'030646','1999-025APD','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',102.78,851,930,99.22,'Sun-Synch' from dual union all
select 30647,'030647','1999-025APE','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',101.82,802,887,99.09,'Sun-Synch' from dual union all
select 30648,'030648','1999-025APF','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',101.91,809,890,98.89,'Sun-Synch' from dual union all
select 30649,'030649','1999-025APG','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',102.56,829,931,99.09,'Sun-Synch' from dual union all
select 3065,'003065','1965-082LN','deb Transtage 4','',11832,'Reentered','1989-11-26 00:00:00','1989-11-26 00:00:00',88.6,203,208,31.97,'Intermediate' from dual union all
select 30650,'030650','1999-025APH','deb FY-1C','',66706,'In Earth orbit','','2007-02-24 00:00:00',103.33,848,985,98.84,'Sun-Synch' from dual union all
select 30651,'030651','2007-005G','deb H2A','',68347,'Reentered','2008-10-27 00:00:00','2007-04-20 00:00:00',94.15,465,490,97.22,'Sun-Synch' from dual union all
select 30652,'030652','1999-025APJ','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',102.38,865,878,98.76,'Sun-Synch' from dual union all
select 30653,'030653','1999-025APK','deb FY-1C','',66706,'Reentered','2009-12-25 00:00:00','2007-03-02 00:00:00',97.57,440,845,98.88,'Sun-Synch' from dual union all
select 30654,'030654','1999-025APL','deb FY-1C','',66706,'In Earth orbit','','2007-03-01 00:00:00',102.69,855,917,99.38,'Sun-Synch' from dual union all
select 30655,'030655','1999-025APM','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',102.49,854,899,98.66,'Sun-Synch' from dual union all
select 30656,'030656','1999-025APN','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',102.5,834,920,99.09,'Sun-Synch' from dual union all
select 30657,'030657','1999-025APP','deb FY-1C','',66706,'In Earth orbit','','2007-03-01 00:00:00',102.49,855,898,98.81,'Sun-Synch' from dual union all
select 30658,'030658','1999-025APQ','deb FY-1C','',66706,'In Earth orbit','','2007-02-27 00:00:00',105.22,844,1166,98.45,'Sun-Synch' from dual union all
select 30659,'030659','1999-025APR','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',99.16,550,887,99.09,'Sun-Synch' from dual union all
select 3066,'003066','1967-123A','Pioneer 8','Pioneer C',15378,'Deep Space','1968-01-16 00:00:00','1967-12-13 00:00:00',611979.1,484,4788696,32.89,'Earth Escape' from dual union all
select 30660,'030660','1999-025APS','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',99.09,553,878,98.02,'Sun-Synch' from dual union all
select 30661,'030661','1999-025APT','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',101.54,724,940,99.49,'Sun-Synch' from dual union all
select 30662,'030662','1999-025APU','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',102.86,850,937,98.78,'Sun-Synch' from dual union all
select 30663,'030663','1999-025APV','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',110.95,700,1841,99.26,'Sun-Synch' from dual union all
select 30664,'030664','1999-025APW','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',100.75,730,858,99.06,'Sun-Synch' from dual union all
select 30665,'030665','1999-025APX','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',103.49,854,994,98.85,'Sun-Synch' from dual union all
select 30666,'030666','1999-025APY','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',102.95,856,941,99.19,'Sun-Synch' from dual union all
select 30667,'030667','1999-025APZ','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',98.73,528,868,98.95,'Sun-Synch' from dual union all
select 30668,'030668','1999-025AQA','deb FY-1C','',66706,'In Earth orbit','','2007-03-01 00:00:00',103.52,853,997,98.99,'Sun-Synch' from dual union all
select 30669,'030669','1999-025AQB','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',104.02,835,1062,98.59,'Sun-Synch' from dual union all
select 3067,'003067','1967-123B','TTS 1','TTS A',15378,'Reentered','1968-04-28 00:00:00','1968-01-13 00:00:00',92.03,289,459,32.91,'Intermediate' from dual union all
select 30670,'030670','1999-025AQC','deb FY-1C','[Alleged Blits impact]',66706,'In Earth orbit','','2007-03-02 00:00:00',101.84,817,875,98.6,'Sun-Synch' from dual union all
select 30671,'030671','1999-025AQD','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',104.31,854,1070,99.15,'Sun-Synch' from dual union all
select 30672,'030672','1999-025AQE','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',100.17,675,859,99.1,'Sun-Synch' from dual union all
select 30673,'030673','1999-025AQF','deb FY-1C','',66706,'In Earth orbit','','2007-03-01 00:00:00',102.8,856,926,98.78,'Sun-Synch' from dual union all
select 30674,'030674','1999-025AQG','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',102.77,827,953,98.88,'Sun-Synch' from dual union all
select 30675,'030675','1999-025AQH','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',102.55,828,931,99.07,'Sun-Synch' from dual union all
select 30676,'030676','1999-025AQJ','deb FY-1C','',66706,'In Earth orbit','','2007-03-01 00:00:00',106.08,650,1440,98.3,'Sun-Synch' from dual union all
select 30677,'030677','1999-025AQK','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',101.79,780,907,98.27,'Sun-Synch' from dual union all
select 30678,'030678','1999-025AQL','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',103.91,855,1032,98.96,'Sun-Synch' from dual union all
select 30679,'030679','1999-025AQM','deb FY-1C','',66706,'In Earth orbit','','2007-03-01 00:00:00',104.88,855,1123,99.25,'Sun-Synch' from dual union all
select 3068,'003068','1967-123C','Delta 55','DSV-3E-3 S/N 20218',15378,'Reentered','1968-04-01 00:00:00','1968-01-13 00:00:00',91.94,291,449,32.9,'Intermediate' from dual union all
select 30680,'030680','1999-025AQN','deb FY-1C','',66706,'In Earth orbit','','2007-03-01 00:00:00',108.55,849,1471,99.57,'Sun-Synch' from dual union all
select 30681,'030681','1999-025AQP','deb FY-1C','',66706,'Reentered','2012-02-01 00:00:00','2007-03-02 00:00:00',98.21,298,1049,97.38,'Sun-Synch' from dual union all
select 30682,'030682','1999-025AQQ','deb FY-1C','',66706,'Reentered','2011-04-26 00:00:00','2007-03-02 00:00:00',97.44,409,864,98.84,'Sun-Synch' from dual union all
select 30683,'030683','1999-025AQR','deb FY-1C','',66706,'In Earth orbit','','2007-03-01 00:00:00',104.19,831,1082,99,'Sun-Synch' from dual union all
select 30684,'030684','1999-025AQS','deb FY-1C','',66706,'In Earth orbit','','2007-03-01 00:00:00',106.02,857,1228,99.6,'Sun-Synch' from dual union all
select 30685,'030685','1999-025AQT','deb FY-1C','',66706,'Reentered','2010-05-04 00:00:00','2007-03-02 00:00:00',97.45,427,847,98.86,'Sun-Synch' from dual union all
select 30686,'030686','1999-025AQU','deb FY-1C','',66706,'In Earth orbit','','2007-03-01 00:00:00',100.05,594,928,99.07,'Sun-Synch' from dual union all
select 30687,'030687','1999-025AQV','deb FY-1C','',66706,'In Earth orbit','','2007-03-01 00:00:00',100.42,697,860,99.03,'Sun-Synch' from dual union all
select 30688,'030688','1999-025AQW','deb FY-1C','',66706,'In Earth orbit','','2007-03-01 00:00:00',102.39,856,888,98.97,'Sun-Synch' from dual union all
select 30689,'030689','1999-025AQX','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',102.42,788,959,99.1,'Sun-Synch' from dual union all
select 3069,'003069','1967-123D','deb Delta 55','',15378,'Reentered','1968-02-12 00:00:00','1968-01-12 00:00:00',91.51,289,408,32.91,'Intermediate' from dual union all
select 30690,'030690','1999-025AQY','deb FY-1C','',66706,'In Earth orbit','','2007-03-01 00:00:00',103.93,854,1035,98.89,'Sun-Synch' from dual union all
select 30691,'030691','1999-025AQZ','deb FY-1C','',66706,'In Earth orbit','','2007-03-02 00:00:00',104.48,857,1084,98.79,'Sun-Synch' from dual union all
select 30692,'030692','1999-025ARA','deb FY-1C','',66706,'Reentered','2015-05-10 00:00:00','2007-03-08 00:00:00',99.94,407,1105,98.82,'Sun-Synch' from dual union all
select 30693,'030693','1999-025ARB','deb FY-1C','',66706,'In Earth orbit','','2007-03-07 00:00:00',102.24,842,888,99.06,'Sun-Synch' from dual union all
select 30694,'030694','1999-025ARC','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',99.53,570,903,99.09,'Sun-Synch' from dual union all
select 30695,'030695','1999-025ARD','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',100.07,659,865,99.01,'Sun-Synch' from dual union all
select 30696,'030696','1999-025ARE','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',101.11,725,898,98.59,'Sun-Synch' from dual union all
select 30697,'030697','1999-025ARF','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',103.25,837,988,99.06,'Sun-Synch' from dual union all
select 30698,'030698','1999-025ARG','deb FY-1C','',66706,'Reentered','2008-12-30 00:00:00','2007-03-08 00:00:00',97.65,448,845,98.99,'Sun-Synch' from dual;
insert into SATELLITE
select 30699,'030699','1999-025ARH','deb FY-1C','',66706,'In Earth orbit','','2007-03-07 00:00:00',102.06,855,858,99.38,'Sun-Synch' from dual union all
select 307,'000307','62 Psi','FTV 2402','SAMOS E-6 No. 2',8200,'Reentered Attached','1962-06-18 00:00:00','1962-06-17 00:00:00',88.47,199,199,96.3,'Sun-Synch' from dual union all
select 3070,'003070','1967-123E','deb Delta 55','',15378,'Reentered','1968-02-12 00:00:00','1968-01-15 00:00:00',91.35,280,402,32.89,'Intermediate' from dual union all
select 30700,'030700','1999-025ARJ','deb FY-1C','',66706,'In Earth orbit','','2007-03-07 00:00:00',101.42,760,891,98.59,'Sun-Synch' from dual union all
select 30701,'030701','1999-025ARK','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',101.85,821,872,98.61,'Sun-Synch' from dual union all
select 30702,'030702','1999-025ARL','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',103.32,856,976,99.33,'Sun-Synch' from dual union all
select 30703,'030703','1999-025ARM','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.77,787,993,98.66,'Sun-Synch' from dual union all
select 30704,'030704','1999-025ARN','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.95,834,962,99.08,'Sun-Synch' from dual union all
select 30705,'030705','1999-025ARP','deb FY-1C','',66706,'In Earth orbit','','2007-03-07 00:00:00',105.66,857,1194,102.22,'Sun-Synch' from dual union all
select 30706,'030706','1999-025ARQ','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102,826,880,99.2,'Sun-Synch' from dual union all
select 30707,'030707','1999-025ARR','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.66,856,912,99.47,'Sun-Synch' from dual union all
select 30708,'030708','1999-025ARS','deb FY-1C','',66706,'In Earth orbit','','2007-03-06 00:00:00',102.15,854,867,98.8,'Sun-Synch' from dual union all
select 30709,'030709','1999-025ART','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',103.81,857,1021,98.94,'Sun-Synch' from dual union all
select 3071,'003071','1967-124A','Kosmos-195','Zenit-2 No. 57',15379,'Landed','1967-12-24 00:00:00','1967-12-18 00:00:00',90.05,203,351,65.65,'Intermediate' from dual union all
select 30710,'030710','1999-025ARU','deb FY-1C','',66706,'In Earth orbit','','2007-03-07 00:00:00',105.26,801,1213,102.74,'Sun-Synch' from dual union all
select 30711,'030711','1999-025ARV','deb FY-1C','',66706,'Reentered','2011-03-04 00:00:00','2007-03-08 00:00:00',98.25,407,942,99.13,'Sun-Synch' from dual union all
select 30712,'030712','1999-025ARW','deb FY-1C','',66706,'Reentered','2015-05-05 00:00:00','2007-03-08 00:00:00',102.35,823,917,99.58,'Sun-Synch' from dual union all
select 30713,'030713','1999-025ARX','deb FY-1C','',66706,'Reentered','2011-10-30 00:00:00','2007-03-08 00:00:00',103.14,823,992,98.8,'Sun-Synch' from dual union all
select 30714,'030714','1999-025ARY','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',103.2,857,963,99.34,'Sun-Synch' from dual union all
select 30715,'030715','1999-025ARZ','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',103.37,855,981,98.99,'Sun-Synch' from dual union all
select 30716,'030716','1999-025ASA','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102,810,897,98.95,'Sun-Synch' from dual union all
select 30717,'030717','1999-025ASB','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.94,849,947,98.82,'Sun-Synch' from dual union all
select 30718,'030718','1999-025ASC','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.76,833,946,98.88,'Sun-Synch' from dual union all
select 30719,'030719','1999-025ASD','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.35,856,883,98.88,'Sun-Synch' from dual union all
select 3072,'003072','1967-124B','11A57I','11A57I',15379,'Reentered','1967-12-23 00:00:00','1967-12-18 00:00:00',89.74,193,331,65.64,'Intermediate' from dual union all
select 30720,'030720','1999-025ASE','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',101.81,817,872,99.1,'Sun-Synch' from dual union all
select 30721,'030721','1999-025ASF','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',101.84,795,897,98.79,'Sun-Synch' from dual union all
select 30722,'030722','1999-025ASG','deb FY-1C','',66706,'In Earth orbit','','2007-03-07 00:00:00',106.11,666,1427,98.39,'Sun-Synch' from dual union all
select 30723,'030723','1999-025ASH','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.36,853,887,98.75,'Sun-Synch' from dual union all
select 30724,'030724','1999-025ASJ','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',100.87,724,876,99.33,'Sun-Synch' from dual union all
select 30725,'030725','1999-025ASK','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',103.15,828,988,98.91,'Sun-Synch' from dual union all
select 30726,'030726','1999-025ASL','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',103.31,849,982,98.8,'Sun-Synch' from dual union all
select 30727,'030727','1999-025ASM','deb FY-1C','',66706,'Reentered','2017-04-06 00:00:00','2007-03-08 00:00:00',101.04,680,936,99.23,'Sun-Synch' from dual union all
select 30728,'030728','1999-025ASN','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',101.27,746,892,98.64,'Sun-Synch' from dual union all
select 30729,'030729','1999-025ASP','deb FY-1C','',66706,'Reentered','2012-02-02 00:00:00','2007-03-08 00:00:00',103.64,808,1053,99.91,'Sun-Synch' from dual union all
select 3073,'003073','1967-124C','deb Kosmos-195','',15379,'Reentered','1967-12-18 00:00:00','1967-12-16 00:00:00',89.6,201,309,65.6,'Intermediate' from dual union all
select 30730,'030730','1999-025ASQ','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.26,795,937,98.88,'Sun-Synch' from dual union all
select 30731,'030731','1999-025ASR','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.39,854,890,98.73,'Sun-Synch' from dual union all
select 30732,'030732','1999-025ASS','deb FY-1C','',66706,'In Earth orbit','','2007-03-07 00:00:00',102.96,802,996,98.99,'Sun-Synch' from dual union all
select 30733,'030733','1999-025AST','deb FY-1C','',66706,'In Earth orbit','','2007-03-07 00:00:00',100.43,698,860,99.1,'Sun-Synch' from dual union all
select 30734,'030734','1999-025ASU','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',100.14,641,890,99.05,'Sun-Synch' from dual union all
select 30735,'030735','1999-025ASV','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.7,830,942,99.09,'Sun-Synch' from dual union all
select 30736,'030736','1999-025ASW','deb FY-1C','',66706,'In Earth orbit','','2007-03-06 00:00:00',106.89,849,1317,98.8,'Sun-Synch' from dual union all
select 30737,'030737','1999-025ASX','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',101.88,830,866,99.04,'Sun-Synch' from dual union all
select 30738,'030738','1999-025ASY','deb FY-1C','',66706,'In Earth orbit','','2007-03-07 00:00:00',102.64,826,942,99.41,'Sun-Synch' from dual union all
select 30739,'030739','1999-025ASZ','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.16,783,938,99,'Sun-Synch' from dual union all
select 3074,'003074','1967-125A','Kosmos-196','DS-U1-G No. 2',15380,'Reentered','1968-07-07 00:00:00','1968-01-18 00:00:00',94.92,220,810,48.8,'Intermediate' from dual union all
select 30740,'030740','1999-025ATA','deb FY-1C','',66706,'In Earth orbit','','2007-03-07 00:00:00',101.95,811,890,98.89,'Sun-Synch' from dual union all
select 30741,'030741','1999-025ATB','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.06,851,862,98.8,'Sun-Synch' from dual union all
select 30742,'030742','1999-025ATC','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',103.36,856,979,99.2,'Sun-Synch' from dual union all
select 30743,'030743','1999-025ATD','deb FY-1C','',66706,'Reentered','2011-05-12 00:00:00','2007-03-08 00:00:00',107.58,807,1423,98.84,'Sun-Synch' from dual union all
select 30744,'030744','1999-025ATE','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',101.81,829,860,98.7,'Sun-Synch' from dual union all
select 30745,'030745','1999-025ATF','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',101.82,819,870,99.31,'Sun-Synch' from dual union all
select 30746,'030746','1999-025ATG','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',101.73,803,879,99.1,'Sun-Synch' from dual union all
select 30747,'030747','1999-025ATH','deb FY-1C','',66706,'Reentered','2011-10-28 00:00:00','2007-03-07 00:00:00',98.73,496,899,99.47,'Sun-Synch' from dual union all
select 30748,'030748','1999-025ATJ','deb FY-1C','',66706,'Reentered','2015-04-19 00:00:00','2007-03-08 00:00:00',99.59,621,857,99.52,'Sun-Synch' from dual union all
select 30749,'030749','1999-025ATK','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.97,857,942,98.9,'Sun-Synch' from dual union all
select 3075,'003075','1967-125B','S1M','S1M',15380,'Reentered','1968-02-20 00:00:00','1968-01-18 00:00:00',93.34,213,663,48.82,'Intermediate' from dual union all
select 30750,'030750','1999-025ATL','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.74,844,933,98.83,'Sun-Synch' from dual union all
select 30751,'030751','1999-025ATM','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',103.1,853,958,99.1,'Sun-Synch' from dual union all
select 30752,'030752','1999-025ATN','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',101.99,824,882,98.85,'Sun-Synch' from dual union all
select 30753,'030753','1999-025ATP','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.86,850,937,98.99,'Sun-Synch' from dual union all
select 30754,'030754','1999-025ATQ','deb FY-1C','',66706,'Reentered','2010-11-29 00:00:00','2007-03-08 00:00:00',98.01,467,860,99.22,'Sun-Synch' from dual union all
select 30755,'030755','1999-025ATR','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',100.66,658,921,98.73,'Sun-Synch' from dual union all
select 30756,'030756','1999-025ATS','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',103.02,838,965,98.88,'Sun-Synch' from dual union all
select 30757,'030757','1999-025ATT','deb FY-1C','',66706,'Reentered','2008-07-18 00:00:00','2007-03-07 00:00:00',96.88,388,831,99.02,'Sun-Synch' from dual union all
select 30758,'030758','1999-025ATU','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.72,852,923,99.24,'Sun-Synch' from dual union all
select 30759,'030759','1999-025ATV','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',101.98,841,864,99.33,'Sun-Synch' from dual union all
select 3076,'003076','1967-125C','deb Kosmos-196','',15380,'Reentered','1968-01-07 00:00:00','1967-12-26 00:00:00',92.19,220,544,48.8,'Intermediate' from dual union all
select 30760,'030760','1999-025ATW','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.7,847,926,99.02,'Sun-Synch' from dual union all
select 30761,'030761','1999-025ATX','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',101.41,752,899,99.12,'Sun-Synch' from dual union all
select 30762,'030762','1999-025ATY','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',103.43,768,1074,99.31,'Sun-Synch' from dual union all
select 30763,'030763','1999-025ATZ','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.85,856,931,98.85,'Sun-Synch' from dual union all
select 30764,'030764','1999-025AUA','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',98.53,519,857,98.84,'Sun-Synch' from dual union all
select 30765,'030765','1999-025AUB','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.68,829,942,98.88,'Sun-Synch' from dual union all
select 30766,'030766','1999-025AUC','deb FY-1C','',66706,'In Earth orbit','','2007-03-07 00:00:00',102.8,822,961,99.35,'Sun-Synch' from dual union all
select 30767,'030767','1999-025AUD','deb FY-1C','',66706,'Reentered','2007-04-02 00:00:00','2007-03-06 00:00:00',93.61,211,692,98.69,'Sun-Synch' from dual union all
select 30768,'030768','1999-025AUE','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',101.88,817,879,98.86,'Sun-Synch' from dual union all
select 30769,'030769','1999-025AUF','deb FY-1C','',66706,'Reentered','2012-01-18 00:00:00','2007-03-08 00:00:00',97.2,391,859,99.45,'Sun-Synch' from dual union all
select 3077,'003077','1967-125D','deb Kosmos-196','',15380,'Reentered','1968-01-07 00:00:00','1967-12-25 00:00:00',92.44,215,573,48.87,'Intermediate' from dual union all
select 30770,'030770','1999-025AUG','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.08,847,868,98.84,'Sun-Synch' from dual union all
select 30771,'030771','1999-025AUH','deb FY-1C','',66706,'In Earth orbit','','2007-03-08 00:00:00',102.59,853,910,98.64,'Sun-Synch' from dual union all
select 30772,'030772','2007-006A','ASTRO','Orbital Express',68348,'Reentered','2013-10-25 00:00:00','2007-03-09 00:00:00',94.49,490,497,46.03,'Intermediate' from dual union all
select 30773,'030773','2007-006B','MidStar 1','Midstar 1',68348,'In Earth orbit','','2007-03-09 00:00:00',94.53,494,498,46.03,'Intermediate' from dual union all
select 30774,'030774','2007-006C','Nextsat','Orbital Express',68348,'In Earth orbit','','2007-03-09 00:00:00',94.49,490,497,46.03,'Intermediate' from dual union all
select 30775,'030775','2007-006D','STPSat-1','STPSat-1',68348,'In Earth orbit','','2007-03-09 00:00:00',95.83,557,560,35.44,'Intermediate' from dual union all
select 30776,'030776','2007-006E','Falconsat-3','Falconsat-3',68348,'In Earth orbit','','2007-03-09 00:00:00',95.81,555,560,35.77,'Intermediate' from dual union all
select 30777,'030777','2007-006F','CFESat','CFESat',68348,'In Earth orbit','','2007-03-09 00:00:00',95.81,551,565,35.43,'Intermediate' from dual union all
select 30778,'030778','2007-006G','Centaur AV-013','Centaur AV-013',68348,'Reentered','2014-12-22 00:00:00','2007-03-09 00:00:00',95.11,502,546,35.7,'Intermediate' from dual union all
select 30779,'030779','1999-057NN','ZY-1 debris','',66738,'Reentered','2009-01-17 00:00:00','2007-03-09 00:00:00',98.4,592,772,98.39,'Sun-Synch' from dual union all
select 3078,'003078','1967-125E','deb Kosmos-196','',15380,'Reentered','1968-01-07 00:00:00','1967-12-25 00:00:00',92.48,207,586,48.77,'Intermediate' from dual union all
select 30780,'030780','1999-057NP','ZY-1 debris','',66738,'Reentered','2007-03-10 00:00:00','2007-03-09 00:00:00',93.78,408,511,98.68,'Sun-Synch' from dual union all
select 30781,'030781','1999-057NQ','ZY-1 debris','',66738,'Reentered','2007-03-14 00:00:00','2007-03-09 00:00:00',96.32,442,723,98.47,'Sun-Synch' from dual union all
select 30782,'030782','1999-057NR','ZY-1 debris','',66738,'Reentered','2010-10-22 00:00:00','2007-03-09 00:00:00',100.24,755,785,98.26,'Sun-Synch' from dual union all
select 30783,'030783','1999-057NS','ZY-1 debris','',66738,'In Earth orbit','','2007-03-09 00:00:00',100.97,774,836,98.19,'Sun-Synch' from dual union all
select 30784,'030784','1999-057NT','ZY-1 debris','',66738,'Reentered','2011-03-10 00:00:00','2007-03-09 00:00:00',100.6,751,823,98.19,'Sun-Synch' from dual union all
select 30785,'030785','1999-057NU','ZY-1 debris','',66738,'Reentered','2010-05-21 00:00:00','2007-03-09 00:00:00',100.51,764,802,98.2,'Sun-Synch' from dual union all
select 30786,'030786','1999-057NV','ZY-1 debris','',66738,'Reentered','2008-10-25 00:00:00','2007-03-09 00:00:00',98.35,589,771,98.33,'Sun-Synch' from dual union all
select 30787,'030787','1999-057NW','ZY-1 debris','',66738,'Reentered','2007-07-04 00:00:00','2007-03-09 00:00:00',97.97,555,769,98.19,'Sun-Synch' from dual union all
select 30788,'030788','1999-057NX','ZY-1 debris','',66738,'Reentered','2007-07-19 00:00:00','2007-03-09 00:00:00',98.25,582,768,98.39,'Sun-Synch' from dual;
insert into SATELLITE
select 30789,'030789','1999-057NY','ZY-1 debris','',66738,'In Earth orbit','','2007-03-09 00:00:00',100.44,773,785,98.22,'Sun-Synch' from dual union all
select 3079,'003079','1967-126A','Kosmos-197','DS-U2-V No. 3',15381,'Reentered','1968-01-30 00:00:00','1968-01-02 00:00:00',91.23,213,456,48.43,'Intermediate' from dual union all
select 30790,'030790','1999-057NZ','ZY-1 debris','',66738,'Reentered','2009-08-13 00:00:00','2007-03-09 00:00:00',100.42,774,783,98.23,'Sun-Synch' from dual union all
select 30791,'030791','1999-057PA','ZY-1 debris','',66738,'In Earth orbit','','2007-03-09 00:00:00',100.34,770,780,98.22,'Sun-Synch' from dual union all
select 30792,'030792','1999-057PB','ZY-1 debris','',66738,'In Earth orbit','','2007-03-09 00:00:00',100.4,772,782,98.21,'Sun-Synch' from dual union all
select 30793,'030793','2007-007A','Insat 4B','Insat 4B',68349,'In Earth orbit','','2007-04-11 00:00:00',1434.07,35741,35753,.08,'Drift GEO' from dual union all
select 30794,'030794','2007-007B','Skynet 5A','Skynet 5A',68349,'In Earth orbit','','2007-04-13 00:00:00',1436.04,35780,35791,.08,'Stationary' from dual union all
select 30795,'030795','2007-007C','ESC-A 535','Ariane 5 ESC-A',68349,'In Earth orbit','','2007-04-13 00:00:00',631,280,35703,4.33,'GEO Transfer' from dual union all
select 30796,'030796','2007-007D','Sylda 5C','Sylda 5',68349,'Reentered','2013-04-06 00:00:00','2007-04-11 00:00:00',630.2,273,35669,4.43,'GEO Transfer' from dual union all
select 30797,'030797','2007-004D','THEMIS D','THEMIS P3',68346,'In Earth orbit','','2009-07-02 00:00:00',1436.14,4191,67384,4.08,'Synchronous' from dual union all
select 30798,'030798','2007-004E','THEMIS E','THEMIS P4',68346,'In Earth orbit','','2009-07-02 00:00:00',1436.11,4202,67372,4.62,'Synchronous' from dual union all
select 30799,'030799','2007-004G','Star 48B','Star 48B',68346,'In Earth orbit','','2007-03-27 00:00:00',1871.03,481,87358,15.53,'Very High' from dual union all
select 308,'000308','62 Omega 1','FTV 2312','Project 102 Group 2-D',8196,'Reentered Attached','1963-10-30 00:00:00','1962-07-18 00:00:00',92.2,370,395,82.11,'Intermediate' from dual union all
select 3080,'003080','1967-126B','S1M','S1M',15381,'Reentered','1968-01-08 00:00:00','1967-12-29 00:00:00',91.19,209,457,48.42,'Intermediate' from dual union all
select 30800,'030800','1999-025AUJ','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',101.02,749,865,99.08,'Sun-Synch' from dual union all
select 30801,'030801','1999-025AUK','deb FY-1C','',66706,'Reentered','2008-08-15 00:00:00','2007-03-13 00:00:00',98.01,456,871,99.14,'Sun-Synch' from dual union all
select 30802,'030802','1999-025AUL','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',102.48,855,897,98.81,'Sun-Synch' from dual union all
select 30803,'030803','1999-025AUM','deb FY-1C','',66706,'In Earth orbit','','2007-03-14 00:00:00',103.48,837,1010,98.88,'Sun-Synch' from dual union all
select 30804,'030804','1999-025AUN','deb FY-1C','',66706,'In Earth orbit','','2007-03-14 00:00:00',100.03,598,922,98.98,'Sun-Synch' from dual union all
select 30805,'030805','1999-025AUP','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',103.2,857,963,99.32,'Sun-Synch' from dual union all
select 30806,'030806','1999-025AUQ','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',105.73,816,1242,99.07,'Sun-Synch' from dual union all
select 30807,'030807','1999-025AUR','deb FY-1C','',66706,'Reentered','2011-08-15 00:00:00','2007-03-15 00:00:00',97.24,401,852,99.16,'Sun-Synch' from dual union all
select 30808,'030808','1999-025AUS','deb FY-1C','',66706,'Reentered','2010-02-12 00:00:00','2007-03-14 00:00:00',97.36,422,842,98.95,'Sun-Synch' from dual union all
select 30809,'030809','1999-025AUT','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',102.57,828,933,99.08,'Sun-Synch' from dual union all
select 3081,'003081','1967-127A','Kosmos-198 ROCh','US-A reactor',15382,'In Earth orbit','','1968-01-27 00:00:00',103.44,893,950,65.14,'Intermediate' from dual union all
select 30810,'030810','1999-025AUU','deb FY-1C','',66706,'In Earth orbit','','2007-03-13 00:00:00',106.03,838,1248,98.69,'Sun-Synch' from dual union all
select 30811,'030811','1999-025AUV','deb FY-1C','',66706,'In Earth orbit','','2007-03-13 00:00:00',106.88,851,1313,98.79,'Sun-Synch' from dual union all
select 30812,'030812','1999-025AUW','deb FY-1C','',66706,'Reentered','2011-01-11 00:00:00','2007-03-15 00:00:00',96.98,358,871,99.37,'Sun-Synch' from dual union all
select 30813,'030813','1999-025AUX','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',100.51,703,862,98.99,'Sun-Synch' from dual union all
select 30814,'030814','1999-025AUY','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',101.88,807,889,99.17,'Sun-Synch' from dual union all
select 30815,'030815','1999-025AUZ','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',102.05,856,856,99.37,'Sun-Synch' from dual union all
select 30816,'030816','1999-025AVA','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',103.16,855,962,99,'Sun-Synch' from dual union all
select 30817,'030817','1999-025AVB','deb FY-1C','',66706,'In Earth orbit','','2007-03-14 00:00:00',103.11,857,955,99.3,'Sun-Synch' from dual union all
select 30818,'030818','1999-025AVC','deb FY-1C','',66706,'Reentered','2012-12-09 00:00:00','2007-03-15 00:00:00',98.12,378,960,98.4,'Sun-Synch' from dual union all
select 30819,'030819','1999-025AVD','deb FY-1C','',66706,'Reentered','2016-10-29 00:00:00','2007-03-15 00:00:00',100.85,723,875,97.35,'Sun-Synch' from dual union all
select 3082,'003082','1967-127B','Kosmos-198 DU','US-A DU',15382,'Reentered','1968-01-21 00:00:00','1968-01-04 00:00:00',89.35,234,251,65.1,'Intermediate' from dual union all
select 30820,'030820','1999-025AVE','deb FY-1C','',66706,'In Earth orbit','','2007-03-13 00:00:00',103.19,855,964,98.61,'Sun-Synch' from dual union all
select 30821,'030821','1999-025AVF','deb FY-1C','',66706,'In Earth orbit','','2007-03-13 00:00:00',100.47,703,859,99.16,'Sun-Synch' from dual union all
select 30822,'030822','1999-025AVG','deb FY-1C','',66706,'In Earth orbit','','2007-03-14 00:00:00',102.41,816,930,98.82,'Sun-Synch' from dual union all
select 30823,'030823','1999-025AVH','deb FY-1C','',66706,'In Earth orbit','','2007-03-14 00:00:00',103.05,797,1009,98.66,'Sun-Synch' from dual union all
select 30824,'030824','1999-025AVJ','deb FY-1C','',66706,'Reentered','2011-12-22 00:00:00','2007-03-14 00:00:00',99.59,622,856,99.99,'Sun-Synch' from dual union all
select 30825,'030825','1999-025AVK','deb FY-1C','',66706,'Reentered','2007-09-03 00:00:00','2007-03-15 00:00:00',103.49,563,1285,99.35,'Sun-Synch' from dual union all
select 30826,'030826','1999-025AVL','deb FY-1C','',66706,'In Earth orbit','','2007-03-14 00:00:00',99.2,558,883,98.9,'Sun-Synch' from dual union all
select 30827,'030827','1999-025AVM','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',103.18,853,965,98.79,'Sun-Synch' from dual union all
select 30828,'030828','1999-025AVN','deb FY-1C','',66706,'In Earth orbit','','2007-03-14 00:00:00',101.25,534,1101,98.63,'Sun-Synch' from dual union all
select 30829,'030829','1999-025AVP','deb FY-1C','',66706,'In Earth orbit','','2007-03-12 00:00:00',102.63,856,911,99.07,'Sun-Synch' from dual union all
select 3083,'003083','1967-126C','deb Blok Kosmos R-12','',15381,'Reentered','1968-01-10 00:00:00','1967-12-31 00:00:00',90.64,250,362,48.35,'Intermediate' from dual union all
select 30830,'030830','1999-025AVQ','deb FY-1C','',66706,'In Earth orbit','','2007-03-14 00:00:00',102.25,842,889,99,'Sun-Synch' from dual union all
select 30831,'030831','1999-025AVR','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',101.17,772,857,99.36,'Sun-Synch' from dual union all
select 30832,'030832','1999-025AVS','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',105.15,854,1149,100.36,'Sun-Synch' from dual union all
select 30833,'030833','1999-025AVT','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',101.54,807,856,99.88,'Sun-Synch' from dual union all
select 30834,'030834','1999-025AVU','deb FY-1C','',66706,'Reentered','2011-04-21 00:00:00','2007-03-14 00:00:00',103.31,751,1080,99.14,'Sun-Synch' from dual union all
select 30835,'030835','1999-025AVV','deb FY-1C','',66706,'In Earth orbit','','2007-03-14 00:00:00',98.52,482,894,99.1,'Sun-Synch' from dual union all
select 30836,'030836','1999-025AVW','deb FY-1C','',66706,'In Earth orbit','','2007-03-11 00:00:00',102.39,851,893,98.8,'Sun-Synch' from dual union all
select 30837,'030837','1999-025AVX','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',102.16,835,887,99.03,'Sun-Synch' from dual union all
select 30838,'030838','1999-025AVY','deb FY-1C','',66706,'In Earth orbit','','2007-03-14 00:00:00',102.15,816,905,99.14,'Sun-Synch' from dual union all
select 30839,'030839','1999-025AVZ','deb FY-1C','',66706,'Reentered','2016-09-05 00:00:00','2007-03-15 00:00:00',98.3,498,857,98.84,'Sun-Synch' from dual union all
select 3084,'003084','1967-126D','deb Blok Kosmos R-12','',15381,'Reentered','1968-01-05 00:00:00','1967-12-28 00:00:00',90.79,246,381,48.53,'Intermediate' from dual union all
select 30840,'030840','1999-025AWA','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',100.25,560,981,98.68,'Sun-Synch' from dual union all
select 30841,'030841','1999-025AWB','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',107.94,851,1412,98.75,'Sun-Synch' from dual union all
select 30842,'030842','1999-025AWC','deb FY-1C','',66706,'Reentered','2010-01-18 00:00:00','2007-03-15 00:00:00',97.73,455,846,99.57,'Sun-Synch' from dual union all
select 30843,'030843','1999-025AWD','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',99.63,625,856,98.93,'Sun-Synch' from dual union all
select 30844,'030844','1999-025AWE','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',101.09,763,857,98.81,'Sun-Synch' from dual union all
select 30845,'030845','1999-025AWF','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',104.83,840,1133,98.4,'Sun-Synch' from dual union all
select 30846,'030846','1999-025AWG','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',101.2,767,863,99.34,'Sun-Synch' from dual union all
select 30847,'030847','1999-025AWH','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',103.35,854,979,98.74,'Sun-Synch' from dual union all
select 30848,'030848','1999-025AWJ','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',101.9,805,893,99.12,'Sun-Synch' from dual union all
select 30849,'030849','1999-025AWK','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',102.97,850,949,98.83,'Sun-Synch' from dual union all
select 3085,'003085','1967-126E','deb Blok Kosmos R-12','',15381,'Reentered','1968-01-18 00:00:00','1968-01-02 00:00:00',91.16,246,417,48.34,'Intermediate' from dual union all
select 30850,'030850','1999-025AWL','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',103.76,822,1050,98.4,'Sun-Synch' from dual union all
select 30851,'030851','1999-025AWM','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',101.72,778,903,98.43,'Sun-Synch' from dual union all
select 30852,'030852','1999-025AWN','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',102.61,838,927,98.91,'Sun-Synch' from dual union all
select 30853,'030853','1999-025AWP','deb FY-1C','',66706,'Reentered','2012-07-17 00:00:00','2007-03-15 00:00:00',98.33,464,893,99.26,'Sun-Synch' from dual union all
select 30854,'030854','1999-025AWQ','deb FY-1C','',66706,'Reentered','2014-11-23 00:00:00','2007-03-14 00:00:00',106.44,781,1343,96.43,'Sun-Synch' from dual union all
select 30855,'030855','1999-025AWR','deb FY-1C','',66706,'Reentered','2011-10-03 00:00:00','2007-03-15 00:00:00',103.95,634,1257,99.35,'Sun-Synch' from dual union all
select 30856,'030856','1999-025AWS','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',103.71,810,1057,98.27,'Sun-Synch' from dual union all
select 30857,'030857','1999-025AWT','deb FY-1C','',66706,'In Earth orbit','','2007-03-14 00:00:00',103.47,851,994,98.92,'Sun-Synch' from dual union all
select 30858,'030858','1999-025AWU','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',102.08,832,883,98.65,'Sun-Synch' from dual union all
select 30859,'030859','1999-025AWV','deb FY-1C','',66706,'In Earth orbit','','2007-03-15 00:00:00',102.84,838,947,98.82,'Sun-Synch' from dual union all
select 3086,'003086','1967-126F','deb Blok Kosmos R-12','',15381,'Reentered','1968-01-26 00:00:00','1968-01-03 00:00:00',90.99,231,416,48.38,'Intermediate' from dual union all
select 30860,'030860','1999-025AWW','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.64,844,924,98.81,'Sun-Synch' from dual union all
select 30861,'030861','1999-025AWX','deb FY-1C','',66706,'Reentered','2015-01-06 00:00:00','2007-03-22 00:00:00',99.56,619,856,99.59,'Sun-Synch' from dual union all
select 30862,'030862','1999-025AWY','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.95,825,972,98.89,'Sun-Synch' from dual union all
select 30863,'030863','1999-025AWZ','deb FY-1C','',66706,'In Earth orbit','','2007-03-20 00:00:00',117.41,847,2282,99.16,'Sun-Synch' from dual union all
select 30864,'030864','1999-025AXA','deb FY-1C','',66706,'Reentered','2010-10-14 00:00:00','2007-03-22 00:00:00',97.56,423,861,98.99,'Sun-Synch' from dual union all
select 30865,'030865','1999-025AXB','deb FY-1C','',66706,'Reentered','2014-04-26 00:00:00','2007-03-21 00:00:00',97.97,467,856,98.92,'Sun-Synch' from dual union all
select 30866,'030866','1999-025AXC','deb FY-1C','',66706,'Reentered','2009-04-04 00:00:00','2007-03-22 00:00:00',96.2,325,828,99.04,'Sun-Synch' from dual union all
select 30867,'030867','1999-025AXD','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',103.06,855,952,97.86,'Sun-Synch' from dual union all
select 30868,'030868','1999-025AXE','deb FY-1C','',66706,'In Earth orbit','','2007-03-21 00:00:00',102.22,762,966,98.21,'Sun-Synch' from dual union all
select 30869,'030869','1999-025AXF','deb FY-1C','',66706,'Reentered','2012-11-04 00:00:00','2007-03-22 00:00:00',98.7,519,874,98.93,'Sun-Synch' from dual union all
select 3087,'003087','1967-126G','deb Blok Kosmos R-12','',15381,'Reentered','1968-01-03 00:00:00','1967-12-27 00:00:00',90.5,198,401,48.68,'Intermediate' from dual union all
select 30870,'030870','1999-025AXG','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',100.44,698,861,99.03,'Sun-Synch' from dual union all
select 30871,'030871','1999-025AXH','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',101.15,686,941,99.23,'Sun-Synch' from dual union all
select 30872,'030872','1999-025AXJ','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',101.5,791,868,99.26,'Sun-Synch' from dual union all
select 30873,'030873','1999-025AXK','deb FY-1C','',66706,'In Earth orbit','','2007-03-21 00:00:00',101.72,758,923,98.79,'Sun-Synch' from dual union all
select 30874,'030874','1999-025AXL','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',101.99,820,885,99.12,'Sun-Synch' from dual union all
select 30875,'030875','1999-025AXM','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.77,816,964,99.07,'Sun-Synch' from dual union all
select 30876,'030876','1999-025AXN','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',103.16,857,959,99.32,'Sun-Synch' from dual union all
select 30877,'030877','1999-025AXP','deb FY-1C','',66706,'In Earth orbit','','2007-03-21 00:00:00',103.37,845,990,98.86,'Sun-Synch' from dual union all
select 30878,'030878','1999-025AXQ','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.57,848,913,99.02,'Sun-Synch' from dual;
insert into SATELLITE
select 30879,'030879','1999-025AXR','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',104.82,611,1360,99.14,'Sun-Synch' from dual union all
select 3088,'003088','1967-127C','Kosmos-198 RLS','US-A radar',15382,'Reentered','1967-12-31 00:00:00','1967-12-29 00:00:00',89.44,240,254,65.08,'Intermediate' from dual union all
select 30880,'030880','1999-025AXS','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.53,857,900,98.84,'Sun-Synch' from dual union all
select 30881,'030881','1999-025AXT','deb FY-1C','',66706,'Reentered','2012-01-27 00:00:00','2007-03-22 00:00:00',98.76,544,855,98.94,'Sun-Synch' from dual union all
select 30882,'030882','1999-025AXU','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',100.01,552,966,98.54,'Sun-Synch' from dual union all
select 30883,'030883','1999-025AXV','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.02,769,940,98.34,'Sun-Synch' from dual union all
select 30884,'030884','1999-025AXW','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',100.76,700,890,98.99,'Sun-Synch' from dual union all
select 30885,'030885','1999-025AXX','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.35,804,936,99.1,'Sun-Synch' from dual union all
select 30886,'030886','1999-025AXY','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.77,854,926,99.31,'Sun-Synch' from dual union all
select 30887,'030887','1999-025AXZ','deb FY-1C','',66706,'In Earth orbit','','2007-03-20 00:00:00',101.31,575,1066,98.55,'Sun-Synch' from dual union all
select 30888,'030888','1999-025AYA','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',99.48,610,857,99.52,'Sun-Synch' from dual union all
select 30889,'030889','1999-025AYB','deb FY-1C','',66706,'Reentered','2014-11-03 00:00:00','2007-03-22 00:00:00',100.02,615,904,98.79,'Sun-Synch' from dual union all
select 3089,'003089','1967-126H','deb Blok Kosmos R-12','',15381,'Reentered','1967-12-30 00:00:00','1967-12-28 00:00:00',90.72,200,420,48.4,'Intermediate' from dual union all
select 30890,'030890','1999-025AYC','deb FY-1C','',66706,'Reentered','2011-12-28 00:00:00','2007-03-22 00:00:00',103.08,733,1076,99.72,'Sun-Synch' from dual union all
select 30891,'030891','1999-025AYD','deb FY-1C','',66706,'In Earth orbit','','2007-03-20 00:00:00',103.04,856,949,99.3,'Sun-Synch' from dual union all
select 30892,'030892','1999-025AYE','deb FY-1C','',66706,'In Earth orbit','','2007-03-21 00:00:00',101.96,820,883,99.72,'Sun-Synch' from dual union all
select 30893,'030893','1999-025AYF','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',104.25,707,1211,98.28,'Sun-Synch' from dual union all
select 30894,'030894','1999-025AYG','deb FY-1C','',66706,'Reentered','2014-10-25 00:00:00','2007-03-22 00:00:00',100.71,617,967,97.73,'Sun-Synch' from dual union all
select 30895,'030895','1999-025AYH','deb FY-1C','',66706,'In Earth orbit','','2007-03-19 00:00:00',101.28,644,995,98.39,'Sun-Synch' from dual union all
select 30896,'030896','1999-025AYJ','deb FY-1C','',66706,'Reentered','2015-02-11 00:00:00','2007-03-22 00:00:00',97.87,456,857,98.85,'Sun-Synch' from dual union all
select 30897,'030897','1999-025AYK','deb FY-1C','',66706,'Reentered','2011-11-25 00:00:00','2007-03-22 00:00:00',96.81,351,861,98.93,'Sun-Synch' from dual union all
select 30898,'030898','1999-025AYL','deb FY-1C','',66706,'In Earth orbit','','2007-03-19 00:00:00',100.26,649,892,99.1,'Sun-Synch' from dual union all
select 30899,'030899','1999-025AYM','deb FY-1C','',66706,'In Earth orbit','','2007-03-19 00:00:00',102.81,857,927,99.33,'Sun-Synch' from dual union all
select 309,'000309','62 A Alpha 1','Tiros 5','Tiros E',8136,'In Earth orbit','','1962-07-20 00:00:00',100.45,585,974,58.07,'Intermediate' from dual union all
select 3090,'003090','1967-126J','deb Blok Kosmos R-12','',15381,'Reentered','1967-12-30 00:00:00','1967-12-28 00:00:00',90.72,200,420,48.4,'Intermediate' from dual union all
select 30900,'030900','1999-025AYN','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.98,829,971,99.41,'Sun-Synch' from dual union all
select 30901,'030901','1999-025AYP','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',100.22,684,854,98.98,'Sun-Synch' from dual union all
select 30902,'030902','1999-025AYQ','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.59,787,976,100.28,'Sun-Synch' from dual union all
select 30903,'030903','1999-025AYR','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',103.16,837,980,99.51,'Sun-Synch' from dual union all
select 30904,'030904','1999-025AYS','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.8,824,959,98.98,'Sun-Synch' from dual union all
select 30905,'030905','1999-025AYT','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',103.11,856,956,99.1,'Sun-Synch' from dual union all
select 30906,'030906','1999-025AYU','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.62,816,950,98.63,'Sun-Synch' from dual union all
select 30907,'030907','1999-025AYV','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.8,848,935,99.95,'Sun-Synch' from dual union all
select 30908,'030908','1999-025AYW','deb FY-1C','',66706,'Reentered','2009-07-14 00:00:00','2007-03-22 00:00:00',97.19,412,837,98.93,'Sun-Synch' from dual union all
select 30909,'030909','1999-025AYX','deb FY-1C','',66706,'Reentered','2015-11-03 00:00:00','2007-03-22 00:00:00',99.59,621,857,99.52,'Sun-Synch' from dual union all
select 3091,'003091','1968-001A','Surveyor 7','Surveyor SC-7',16728,'Deep Space','1968-01-09 00:00:00','1968-01-07 00:00:00',37531.91,166,729739,30.68,'Cislunar' from dual union all
select 30910,'030910','1999-025AYY','deb FY-1C','',66706,'Reentered','2016-02-04 00:00:00','2007-03-22 00:00:00',101.06,648,970,98.58,'Sun-Synch' from dual union all
select 30911,'030911','1999-025AYZ','deb FY-1C','',66706,'Reentered','2008-03-08 00:00:00','2007-03-22 00:00:00',96.87,405,813,98.94,'Sun-Synch' from dual union all
select 30912,'030912','1999-025AZA','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',100.05,663,859,99.05,'Sun-Synch' from dual union all
select 30913,'030913','1999-025AZB','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',103.06,856,951,99.3,'Sun-Synch' from dual union all
select 30914,'030914','1999-025AZC','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',111.27,847,1724,99.94,'Sun-Synch' from dual union all
select 30915,'030915','1999-025AZD','deb FY-1C','',66706,'In Earth orbit','','2007-03-21 00:00:00',102.81,843,941,98.83,'Sun-Synch' from dual union all
select 30916,'030916','1999-025AZE','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',103.31,850,981,98.9,'Sun-Synch' from dual union all
select 30917,'030917','1999-025AZF','deb FY-1C','',66706,'Reentered','2014-10-31 00:00:00','2007-03-22 00:00:00',99.09,380,1050,98.29,'Sun-Synch' from dual union all
select 30918,'030918','1999-025AZG','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',104.09,821,1083,98.37,'Sun-Synch' from dual union all
select 30919,'030919','1999-025AZH','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',101.94,802,899,98.89,'Sun-Synch' from dual union all
select 3092,'003092','1968-001B','Centaur AC-15','Centaur 12D',16728,'Deep Space','1968-01-09 00:00:00','1968-01-07 00:00:00',20523.73,166,483702,30.68,'Cislunar' from dual union all
select 30920,'030920','1999-025AZJ','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',104.54,840,1106,98.9,'Sun-Synch' from dual union all
select 30921,'030921','1999-025AZK','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.48,855,897,98.81,'Sun-Synch' from dual union all
select 30922,'030922','1999-025AZL','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.31,857,879,98.89,'Sun-Synch' from dual union all
select 30923,'030923','1999-025AZM','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.62,850,916,98.83,'Sun-Synch' from dual union all
select 30924,'030924','1999-025AZN','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.66,832,937,98.86,'Sun-Synch' from dual union all
select 30925,'030925','1999-025AZP','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',102.37,837,905,99.04,'Sun-Synch' from dual union all
select 30926,'030926','1999-025AZQ','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',98.45,512,858,98.88,'Sun-Synch' from dual union all
select 30927,'030927','1999-025AZR','deb FY-1C','',66706,'In Earth orbit','','2007-03-21 00:00:00',103.8,844,1033,99.02,'Sun-Synch' from dual union all
select 30928,'030928','1999-025AZS','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',109.63,842,1577,98.87,'Sun-Synch' from dual union all
select 30929,'030929','1999-025AZT','deb FY-1C','',66706,'Reentered','2011-08-06 00:00:00','2007-03-22 00:00:00',97.72,344,956,98.83,'Sun-Synch' from dual union all
select 3093,'003093','1968-002A','Geos 2','Geos B',16729,'In Earth orbit','','1968-02-17 00:00:00',112.21,1084,1573,105.79,'Retrograde' from dual union all
select 30930,'030930','1999-025AZU','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',100.84,708,889,98.91,'Sun-Synch' from dual union all
select 30931,'030931','1999-025AZV','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',103.45,849,995,98.91,'Sun-Synch' from dual union all
select 30932,'030932','1999-025AZW','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',103.31,853,978,98.81,'Sun-Synch' from dual union all
select 30933,'030933','1999-025AZX','deb FY-1C','',66706,'In Earth orbit','','2007-03-21 00:00:00',104.32,855,1071,99.07,'Sun-Synch' from dual union all
select 30934,'030934','1999-025AZY','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',107.96,851,1414,98.76,'Sun-Synch' from dual union all
select 30935,'030935','1999-025AZZ','deb FY-1C','',66706,'Reentered','2017-02-11 00:00:00','2007-03-22 00:00:00',98.32,458,899,99.02,'Sun-Synch' from dual union all
select 30936,'030936','1999-025BAA','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',101.81,806,882,98.71,'Sun-Synch' from dual union all
select 30937,'030937','1999-025BAB','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',103.27,858,969,98.86,'Sun-Synch' from dual union all
select 30938,'030938','1999-025BAC','deb FY-1C','',66706,'In Earth orbit','','2007-03-22 00:00:00',101.85,810,882,98.76,'Sun-Synch' from dual union all
select 30939,'030939','1999-025BAD','deb FY-1C','',66706,'Reentered','2012-09-25 00:00:00','2007-03-22 00:00:00',102.64,733,1035,99.04,'Sun-Synch' from dual union all
select 3094,'003094','1968-002B','FW4D','FW4D S/N 2226-6',16729,'In Earth orbit','','1968-02-28 00:00:00',112.17,1084,1569,105.81,'Retrograde' from dual union all
select 30940,'030940','1999-025BAE','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',102.66,853,916,99.25,'Sun-Synch' from dual union all
select 30941,'030941','1999-025BAF','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',102.71,854,920,98.64,'Sun-Synch' from dual union all
select 30942,'030942','1999-025BAG','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',102.07,752,962,98.89,'Sun-Synch' from dual union all
select 30943,'030943','1999-025BAH','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',102.8,858,924,98.93,'Sun-Synch' from dual union all
select 30944,'030944','1999-025BAJ','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',99.61,622,858,98.92,'Sun-Synch' from dual union all
select 30945,'030945','1999-025BAK','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',102.73,857,919,99.43,'Sun-Synch' from dual union all
select 30946,'030946','1999-025BAL','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',100.45,701,859,99.13,'Sun-Synch' from dual union all
select 30947,'030947','1999-025BAM','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',100.62,712,864,98.96,'Sun-Synch' from dual union all
select 30948,'030948','1999-025BAN','deb FY-1C','',66706,'Reentered','2008-12-16 00:00:00','2007-03-29 00:00:00',96.52,351,833,99.02,'Sun-Synch' from dual union all
select 30949,'030949','1999-025BAP','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',102.13,771,948,98.92,'Sun-Synch' from dual union all
select 3095,'003095','1967-018C','deb Kosmos-144','',15273,'Reentered','1971-04-08 00:00:00','1968-01-12 00:00:00',96.73,579,625,81.2,'Intermediate' from dual union all
select 30950,'030950','1999-025BAQ','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',99.86,646,858,99.3,'Sun-Synch' from dual union all
select 30951,'030951','1999-025BAR','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',100.44,647,912,99.92,'Sun-Synch' from dual union all
select 30952,'030952','1999-025BAS','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',100.85,726,872,99.75,'Sun-Synch' from dual union all
select 30953,'030953','1999-025BAT','deb FY-1C','',66706,'Reentered','2012-04-21 00:00:00','2007-03-28 00:00:00',104.04,713,1186,101.14,'Sun-Synch' from dual union all
select 30954,'030954','1999-025BAU','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',102.87,785,1004,98.56,'Sun-Synch' from dual union all
select 30955,'030955','1999-025BAV','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',103.07,857,951,98.71,'Sun-Synch' from dual union all
select 30956,'030956','1999-025BAW','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',102.77,847,932,98.98,'Sun-Synch' from dual union all
select 30957,'030957','1999-025BAX','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',101.38,726,922,99.13,'Sun-Synch' from dual union all
select 30958,'030958','1999-025BAY','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',102.32,737,1000,97.99,'Sun-Synch' from dual union all
select 30959,'030959','1999-025BAZ','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',102.6,822,941,99.75,'Sun-Synch' from dual union all
select 3096,'003096','1968-003B','11A57I','11A57I',16730,'Reentered','1968-01-25 00:00:00','1968-01-19 00:00:00',89.76,178,348,65.63,'Intermediate' from dual union all
select 30960,'030960','1999-025BBA','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',100.86,575,1024,99.11,'Sun-Synch' from dual union all
select 30961,'030961','1999-025BBB','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',100.15,644,887,98.99,'Sun-Synch' from dual union all
select 30962,'030962','1999-025BBC','deb FY-1C','',66706,'Reentered','2017-04-23 00:00:00','2007-03-30 00:00:00',100.38,619,934,98.35,'Sun-Synch' from dual union all
select 30963,'030963','1999-025BBD','deb FY-1C','',66706,'Reentered','2015-06-29 00:00:00','2007-03-30 00:00:00',99.3,509,942,97.55,'Sun-Synch' from dual union all
select 30964,'030964','1999-025BBE','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',104.07,828,1074,98.65,'Sun-Synch' from dual union all
select 30965,'030965','1999-025BBF','deb FY-1C','',66706,'Reentered','2014-05-13 00:00:00','2007-03-30 00:00:00',100.62,616,960,97.69,'Sun-Synch' from dual union all
select 30966,'030966','1999-025BBG','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',100.66,683,897,99.16,'Sun-Synch' from dual union all
select 30967,'030967','1999-025BBH','deb FY-1C','',66706,'In Earth orbit','','2007-03-27 00:00:00',100.28,677,867,98.9,'Sun-Synch' from dual union all
select 30968,'030968','1999-025BBJ','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',100.46,696,865,99.45,'Sun-Synch' from dual;
insert into SATELLITE
select 30969,'030969','1999-025BBK','deb FY-1C','',66706,'Reentered','2011-11-28 00:00:00','2007-03-29 00:00:00',98.39,460,903,99.1,'Sun-Synch' from dual union all
select 3097,'003097','1968-004A','OPS 1965','MULTIGROUP 3',16731,'Reentered Attached','1970-07-07 00:00:00','1968-02-19 00:00:00',94.5,455,534,75.14,'Intermediate' from dual union all
select 30970,'030970','1999-025BBL','deb FY-1C','',66706,'Reentered','2016-05-21 00:00:00','2007-03-28 00:00:00',99.38,566,892,99.8,'Sun-Synch' from dual union all
select 30971,'030971','1999-025BBM','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',99.63,573,909,99.25,'Sun-Synch' from dual union all
select 30972,'030972','1999-025BBN','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',102.56,857,903,98.84,'Sun-Synch' from dual union all
select 30973,'030973','1999-025BBP','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',102.81,801,983,99.79,'Sun-Synch' from dual union all
select 30974,'030974','1999-025BBQ','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',100.95,739,869,99,'Sun-Synch' from dual union all
select 30975,'030975','1999-025BBR','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',102.6,696,1068,99.21,'Sun-Synch' from dual union all
select 30976,'030976','1999-025BBS','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',103.35,761,1073,99.04,'Sun-Synch' from dual union all
select 30977,'030977','1999-025BBT','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',100.75,693,896,99.08,'Sun-Synch' from dual union all
select 30978,'030978','1999-025BBU','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',101.81,804,885,98.26,'Sun-Synch' from dual union all
select 30979,'030979','1999-025BBV','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',100.65,540,1039,97.79,'Sun-Synch' from dual union all
select 3098,'003098','1968-005A','OPS 5028','GAMBIT-3 Mission 4311',16732,'Reentered Attached','1968-02-04 00:00:00','1968-01-25 00:00:00',90.15,133,430,111.5,'Retrograde' from dual union all
select 30980,'030980','1999-025BBW','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',100.48,701,862,98.95,'Sun-Synch' from dual union all
select 30981,'030981','1999-025BBX','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',103.7,817,1049,98.44,'Sun-Synch' from dual union all
select 30982,'030982','1999-025BBY','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',99.86,606,898,98.51,'Sun-Synch' from dual union all
select 30983,'030983','1999-025BBZ','deb FY-1C','',66706,'Reentered','2015-05-31 00:00:00','2007-03-30 00:00:00',98.88,418,993,98.71,'Sun-Synch' from dual union all
select 30984,'030984','1999-025BCA','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',100.74,673,914,98.55,'Sun-Synch' from dual union all
select 30985,'030985','1999-025BCB','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',102.92,853,940,99.29,'Sun-Synch' from dual union all
select 30986,'030986','1999-025BCC','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',101.79,802,885,98.74,'Sun-Synch' from dual union all
select 30987,'030987','1999-025BCD','deb FY-1C','',66706,'Reentered','2012-09-03 00:00:00','2007-03-30 00:00:00',100.51,497,1069,97.42,'Sun-Synch' from dual union all
select 30988,'030988','1999-025BCE','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',100.45,490,1070,98.76,'Sun-Synch' from dual union all
select 30989,'030989','1999-025BCF','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',103.26,828,998,98.93,'Sun-Synch' from dual union all
select 3099,'003099','1968-003A','Priborniy otsek','PO',16730,'Reentered','1968-02-01 00:00:00','1968-01-24 00:00:00',90.06,199,355,65.63,'Intermediate' from dual union all
select 30990,'030990','1999-025BCG','deb FY-1C','',66706,'Reentered','2011-10-10 00:00:00','2007-03-29 00:00:00',98.13,487,852,98.96,'Sun-Synch' from dual union all
select 30991,'030991','1999-025BCH','deb FY-1C','',66706,'Reentered','2009-01-05 00:00:00','2007-03-30 00:00:00',96.68,375,825,99.05,'Sun-Synch' from dual union all
select 30992,'030992','1999-025BCJ','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',102.78,858,922,99.39,'Sun-Synch' from dual union all
select 30993,'030993','1999-025BCK','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',102.46,854,896,98.81,'Sun-Synch' from dual union all
select 30994,'030994','1999-025BCL','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',102.18,849,874,98.83,'Sun-Synch' from dual union all
select 30995,'030995','1999-025BCM','deb FY-1C','',66706,'Reentered','2015-05-11 00:00:00','2007-03-29 00:00:00',99.05,523,903,98.14,'Sun-Synch' from dual union all
select 30996,'030996','1999-025BCN','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',101.44,783,871,99.11,'Sun-Synch' from dual union all
select 30997,'030997','1999-025BCP','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',101.08,743,877,99.11,'Sun-Synch' from dual union all
select 30998,'030998','1999-025BCQ','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',102.32,853,884,98.71,'Sun-Synch' from dual union all
select 30999,'030999','1999-025BCR','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',100.21,657,880,99.12,'Sun-Synch' from dual union all
select 31,'000031','60 Gamma 2','Transit 1B','Transit 1B',6322,'Reentered','1967-10-05 00:00:00','1960-05-26 00:00:00',95.75,373,737,51.28,'Intermediate' from dual union all
select 310,'000310','62 Omega 2','deb USAF satellite/Agena B','Ferret sensor cover',8196,'Reentered','1962-07-12 00:00:00','1962-07-02 00:00:00',91.37,342,342,82.11,'Intermediate' from dual union all
select 3100,'003100','1968-006A','Kosmos-200','Tselina-O',16733,'Reentered','1973-02-24 00:00:00','1968-02-20 00:00:00',95.15,516,536,74.03,'Intermediate' from dual union all
select 31000,'031000','1999-025BCS','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',102.9,758,1034,97.6,'Sun-Synch' from dual union all
select 31001,'031001','1999-025BCT','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',100.68,667,915,99.33,'Sun-Synch' from dual union all
select 31002,'031002','1999-025BCU','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',103.22,834,988,99.05,'Sun-Synch' from dual union all
select 31003,'031003','1999-025BCV','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',102.68,847,924,98.89,'Sun-Synch' from dual union all
select 31004,'031004','1999-025BCW','deb FY-1C','',66706,'Reentered','2012-02-12 00:00:00','2007-03-30 00:00:00',96.89,348,872,98.91,'Sun-Synch' from dual union all
select 31005,'031005','1999-025BCX','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',105.11,850,1150,98.6,'Sun-Synch' from dual union all
select 31006,'031006','1999-025BCY','deb FY-1C','',66706,'Reentered','2013-10-10 00:00:00','2007-03-29 00:00:00',124.46,622,3137,100.33,'Medium' from dual union all
select 31007,'031007','1999-025BCZ','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',101.54,743,921,98.13,'Sun-Synch' from dual union all
select 31008,'031008','1999-025BDA','deb FY-1C','',66706,'Reentered','2017-05-09 00:00:00','2007-03-27 00:00:00',101.06,756,862,99.1,'Sun-Synch' from dual union all
select 31009,'031009','1999-025BDB','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',103.07,858,950,99.12,'Sun-Synch' from dual union all
select 3101,'003101','1968-004B','OPS 1965 sensor cover','',16731,'Reentered','1968-03-29 00:00:00','1968-02-16 00:00:00',93.75,420,496,75.12,'Intermediate' from dual union all
select 31010,'031010','1999-025BDC','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',104.18,825,1087,101.76,'Sun-Synch' from dual union all
select 31011,'031011','1999-025BDD','deb FY-1C','',66706,'Reentered','2009-10-30 00:00:00','2007-03-29 00:00:00',113.49,635,2139,100.01,'Sun-Synch' from dual union all
select 31012,'031012','1999-025BDE','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',101.11,750,873,99.09,'Sun-Synch' from dual union all
select 31013,'031013','1999-025BDF','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',100.98,746,865,99.04,'Sun-Synch' from dual union all
select 31014,'031014','1999-025BDG','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',103.07,839,969,99.3,'Sun-Synch' from dual union all
select 31015,'031015','1999-025BDH','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',99.66,626,859,98.83,'Sun-Synch' from dual union all
select 31016,'031016','1999-025BDJ','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',101.08,722,898,98.59,'Sun-Synch' from dual union all
select 31017,'031017','1999-025BDK','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',102.57,793,967,99.19,'Sun-Synch' from dual union all
select 31018,'031018','1999-025BDL','deb FY-1C','',66706,'In Earth orbit','','2007-03-30 00:00:00',103.27,857,969,98.97,'Sun-Synch' from dual union all
select 31019,'031019','1999-025BDM','deb FY-1C','',66706,'In Earth orbit','','2007-03-29 00:00:00',122.73,854,2752,98.7,'Medium' from dual union all
select 3102,'003102','1968-006B','S3M','S3M',16733,'Reentered','1972-02-04 00:00:00','1968-02-27 00:00:00',94.95,496,537,74.01,'Intermediate' from dual union all
select 31020,'031020','1999-025BDN','deb FY-1C','',66706,'Reentered','2014-12-14 00:00:00','2007-04-04 00:00:00',98.3,492,862,98.83,'Sun-Synch' from dual union all
select 31021,'031021','1999-025BDP','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',100.32,478,1070,98.26,'Sun-Synch' from dual union all
select 31022,'031022','1999-025BDQ','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',103.87,858,1024,99.65,'Sun-Synch' from dual union all
select 31023,'031023','1999-025BDR','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',102.53,818,939,99.07,'Sun-Synch' from dual union all
select 31024,'031024','1999-025BDS','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',101.72,783,897,99.22,'Sun-Synch' from dual union all
select 31025,'031025','1999-025BDT','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',102.05,716,996,99.02,'Sun-Synch' from dual union all
select 31026,'031026','1999-025BDU','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',104.24,859,1059,98.68,'Sun-Synch' from dual union all
select 31027,'031027','1999-025BDV','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',102.67,820,949,99.05,'Sun-Synch' from dual union all
select 31028,'031028','1999-025BDW','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',103.9,819,1067,98.6,'Sun-Synch' from dual union all
select 31029,'031029','1999-025BDX','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',100.89,730,872,99.02,'Sun-Synch' from dual union all
select 3103,'003103','1968-006C','deb Kosmos-200','Tselina-O part',16733,'Reentered','1980-09-30 00:00:00','1968-02-19 00:00:00',95.31,499,569,74.01,'Intermediate' from dual union all
select 31030,'031030','1999-025BDY','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',102.26,841,890,98.76,'Sun-Synch' from dual union all
select 31031,'031031','1999-025BDZ','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',101.84,824,867,99.24,'Sun-Synch' from dual union all
select 31032,'031032','1999-025BEA','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',104.18,832,1080,99,'Sun-Synch' from dual union all
select 31033,'031033','1999-025BEB','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',101.22,699,933,98.93,'Sun-Synch' from dual union all
select 31034,'031034','1999-025BEC','deb FY-1C','',66706,'Reentered','2012-10-21 00:00:00','2007-04-04 00:00:00',97.72,431,868,99.09,'Sun-Synch' from dual union all
select 31035,'031035','1999-025BED','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',101.83,642,1049,99.05,'Sun-Synch' from dual union all
select 31036,'031036','1999-025BEE','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',99.79,640,857,99.03,'Sun-Synch' from dual union all
select 31037,'031037','1999-025BEF','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',101.19,764,866,99.1,'Sun-Synch' from dual union all
select 31038,'031038','1999-025BEG','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',100.54,710,858,99.12,'Sun-Synch' from dual union all
select 31039,'031039','1999-025BEH','deb FY-1C','',66706,'Reentered','2014-01-10 00:00:00','2007-04-03 00:00:00',99.73,636,856,99.02,'Sun-Synch' from dual union all
select 3104,'003104','1968-006D','deb Kosmos-200','Tselina-O part',16733,'Reentered','1968-04-06 00:00:00','1968-02-20 00:00:00',94.31,479,492,74.04,'Intermediate' from dual union all
select 31040,'031040','1999-025BEJ','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',101.95,803,899,98.94,'Sun-Synch' from dual union all
select 31041,'031041','1999-025BEK','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',103.87,851,1032,98.99,'Sun-Synch' from dual union all
select 31042,'031042','1999-025BEL','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',100.48,702,861,99.09,'Sun-Synch' from dual union all
select 31043,'031043','1999-025BEM','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',102.73,857,919,98.95,'Sun-Synch' from dual union all
select 31044,'031044','1999-025BEN','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',100.02,651,868,99.47,'Sun-Synch' from dual union all
select 31045,'031045','1999-025BEP','deb FY-1C','',66706,'Reentered','2013-04-30 00:00:00','2007-04-03 00:00:00',99.49,574,895,99.55,'Sun-Synch' from dual union all
select 31046,'031046','1999-025BEQ','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',103.02,805,998,98.9,'Sun-Synch' from dual union all
select 31047,'031047','1999-025BER','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',102.31,848,887,98.97,'Sun-Synch' from dual union all
select 31048,'031048','1999-025BES','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',102.69,825,947,99.06,'Sun-Synch' from dual union all
select 31049,'031049','1999-025BET','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',102.91,849,944,98.98,'Sun-Synch' from dual union all
select 3105,'003105','1963-014AW','deb West Ford','',9245,'Reentered','1973-06-07 00:00:00','1972-06-06 00:00:00',166.74,3595,3719,86.63,'Medium' from dual union all
select 31050,'031050','1999-025BEU','deb FY-1C','',66706,'Reentered','2007-09-12 00:00:00','2007-04-04 00:00:00',94.79,251,766,99.09,'Sun-Synch' from dual union all
select 31051,'031051','1999-025BEV','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',102.88,856,934,98.91,'Sun-Synch' from dual union all
select 31052,'031052','1999-025BEW','deb FY-1C','',66706,'Reentered','2014-02-14 00:00:00','2007-04-04 00:00:00',99.43,399,1063,98.53,'Sun-Synch' from dual union all
select 31053,'031053','1999-025BEX','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',106.17,781,1317,98.75,'Sun-Synch' from dual union all
select 31054,'031054','1999-025BEY','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',102.15,853,868,98.8,'Sun-Synch' from dual union all
select 31055,'031055','1999-025BEZ','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',102.46,857,894,98.82,'Sun-Synch' from dual union all
select 31056,'031056','1999-025BFA','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',102.18,842,882,99.07,'Sun-Synch' from dual union all
select 31057,'031057','1999-025BFB','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',100.14,588,943,99.31,'Sun-Synch' from dual;
insert into SATELLITE
select 31058,'031058','1999-025BFC','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',99.95,638,874,99.62,'Sun-Synch' from dual union all
select 31059,'031059','1999-025BFD','deb FY-1C','',66706,'Reentered','2012-07-05 00:00:00','2007-04-03 00:00:00',99.88,479,1026,99.02,'Sun-Synch' from dual union all
select 3106,'003106','1968-007A','LM 1 Ascent Stage','Apollo 5',16734,'Reentered','1968-01-23 00:00:00','1968-01-22 00:00:00',96.17,171,980,31.62,'Intermediate' from dual union all
select 31060,'031060','1999-025BFE','deb FY-1C','',66706,'In Earth orbit','','2007-04-01 00:00:00',100.11,651,877,99.23,'Sun-Synch' from dual union all
select 31061,'031061','1999-025BFF','deb FY-1C','',66706,'Reentered','2011-11-02 00:00:00','2007-04-02 00:00:00',101.89,794,903,98.68,'Sun-Synch' from dual union all
select 31062,'031062','1999-025BFG','deb FY-1C','',66706,'In Earth orbit','','2007-04-01 00:00:00',104.87,858,1119,99.04,'Sun-Synch' from dual union all
select 31063,'031063','1999-025BFH','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',101.35,789,857,99.89,'Sun-Synch' from dual union all
select 31064,'031064','1999-025BFJ','deb FY-1C','',66706,'In Earth orbit','','2007-03-31 00:00:00',101.69,653,1024,98.94,'Sun-Synch' from dual union all
select 31065,'031065','1999-025BFK','deb FY-1C','',66706,'In Earth orbit','','2007-04-01 00:00:00',101.89,820,877,98.97,'Sun-Synch' from dual union all
select 31066,'031066','1999-025BFL','deb FY-1C','',66706,'In Earth orbit','','2007-04-01 00:00:00',98.66,492,897,99.07,'Sun-Synch' from dual union all
select 31067,'031067','1999-025BFM','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',99.3,592,859,99.11,'Sun-Synch' from dual union all
select 31068,'031068','1999-025BFN','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',102.41,750,996,99.2,'Sun-Synch' from dual union all
select 31069,'031069','1999-025BFP','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',100.38,672,882,99.15,'Sun-Synch' from dual union all
select 3107,'003107','1968-007B','LM 1 Descent Stage','Apollo 5 DS',16734,'Reentered','1968-02-12 00:00:00','1968-01-30 00:00:00',89.57,169,338,31.63,'Intermediate' from dual union all
select 31070,'031070','1999-025BFQ','deb FY-1C','',66706,'Reentered','2013-05-09 00:00:00','2007-04-04 00:00:00',97.97,467,856,98.85,'Sun-Synch' from dual union all
select 31071,'031071','1999-025BFR','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',100.87,741,859,99.14,'Sun-Synch' from dual union all
select 31072,'031072','1999-025BFS','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',102.18,680,1044,99.37,'Sun-Synch' from dual union all
select 31073,'031073','1999-025BFT','deb FY-1C','',66706,'In Earth orbit','','2007-04-02 00:00:00',102.76,850,928,99.31,'Sun-Synch' from dual union all
select 31074,'031074','1999-025BFU','deb FY-1C','',66706,'In Earth orbit','','2007-04-02 00:00:00',103.83,857,1023,98.92,'Sun-Synch' from dual union all
select 31075,'031075','1999-025BFV','deb FY-1C','',66706,'Reentered','2009-08-23 00:00:00','2007-04-04 00:00:00',97.32,426,835,98.92,'Sun-Synch' from dual union all
select 31076,'031076','1999-025BFW','deb FY-1C','',66706,'Reentered','2008-07-01 00:00:00','2007-04-04 00:00:00',95.53,285,804,99.01,'Sun-Synch' from dual union all
select 31077,'031077','1999-025BFX','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',99.73,633,859,99.03,'Sun-Synch' from dual union all
select 31078,'031078','1999-025BFY','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',101.9,757,941,98.56,'Sun-Synch' from dual union all
select 31079,'031079','1999-025BFZ','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',102.9,854,937,99.1,'Sun-Synch' from dual union all
select 3108,'003108','1968-007C','Saturn S-IVB-204','Saturn S-IVB-204',16734,'Reentered','1968-01-23 00:00:00','1968-01-22 00:00:00',88.25,162,214,31.6,'Intermediate' from dual union all
select 31080,'031080','1999-025BGA','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',103.03,836,967,99.25,'Sun-Synch' from dual union all
select 31081,'031081','1999-025BGB','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',102.45,841,909,98.84,'Sun-Synch' from dual union all
select 31082,'031082','1999-025BGC','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',102.66,829,939,99.1,'Sun-Synch' from dual union all
select 31083,'031083','1999-025BGD','deb FY-1C','',66706,'In Earth orbit','','2007-04-01 00:00:00',101.66,797,878,98.84,'Sun-Synch' from dual union all
select 31084,'031084','1999-025BGE','deb FY-1C','',66706,'Reentered','2014-08-28 00:00:00','2007-04-01 00:00:00',105.12,793,1208,99.19,'Sun-Synch' from dual union all
select 31085,'031085','1999-025BGF','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',100.91,736,867,99.01,'Sun-Synch' from dual union all
select 31086,'031086','1999-025BGG','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',99.29,591,859,99.01,'Sun-Synch' from dual union all
select 31087,'031087','1999-025BGH','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',100.31,601,945,98.53,'Sun-Synch' from dual union all
select 31088,'031088','1999-025BGJ','deb FY-1C','',66706,'In Earth orbit','','2007-04-02 00:00:00',102.71,857,917,99.47,'Sun-Synch' from dual union all
select 31089,'031089','1999-025BGK','deb FY-1C','',66706,'In Earth orbit','','2007-04-02 00:00:00',104.27,800,1121,98.91,'Sun-Synch' from dual union all
select 3109,'003109','1968-006E','deb Kosmos-200','Tselina-O part',16733,'Reentered','1968-04-11 00:00:00','1968-02-19 00:00:00',94.22,474,487,73.98,'Intermediate' from dual union all
select 31090,'031090','1999-025BGL','deb FY-1C','',66706,'In Earth orbit','','2007-04-01 00:00:00',98.89,549,863,99.06,'Sun-Synch' from dual union all
select 31091,'031091','1999-025BGM','deb FY-1C','',66706,'In Earth orbit','','2007-03-31 00:00:00',106.75,855,1298,99.22,'Sun-Synch' from dual union all
select 31092,'031092','1999-025BGN','deb FY-1C','',66706,'In Earth orbit','','2007-04-01 00:00:00',107.73,856,1387,99.58,'Sun-Synch' from dual union all
select 31093,'031093','1999-025BGP','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',104.63,819,1136,98.62,'Sun-Synch' from dual union all
select 31094,'031094','1999-025BGQ','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',104.03,857,1041,98.92,'Sun-Synch' from dual union all
select 31095,'031095','1999-025BGR','deb FY-1C','',66706,'In Earth orbit','','2007-04-02 00:00:00',110.35,858,1628,98.57,'Sun-Synch' from dual union all
select 31096,'031096','1999-025BGS','deb FY-1C','',66706,'Reentered','2011-08-08 00:00:00','2007-04-04 00:00:00',96.71,332,870,99,'Sun-Synch' from dual union all
select 31097,'031097','1999-025BGT','deb FY-1C','',66706,'In Earth orbit','','2007-04-03 00:00:00',102.43,858,889,98.93,'Sun-Synch' from dual union all
select 31098,'031098','1999-025BGU','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',102.44,855,894,98.81,'Sun-Synch' from dual union all
select 31099,'031099','1999-025BGV','deb FY-1C','',66706,'In Earth orbit','','2007-04-04 00:00:00',101.23,768,865,99.1,'Sun-Synch' from dual union all
select 311,'000311','62 A Alpha 2','Altair','Altair X-248-A5',8136,'Reentered','1990-11-28 00:00:00','1962-07-24 00:00:00',100.4,588,967,58.08,'Intermediate' from dual union all
select 3110,'003110','1968-006F','deb Kosmos-200','Tselina-O part',16733,'Reentered','1968-06-16 00:00:00','1968-02-20 00:00:00',95.21,524,534,73.98,'Intermediate' from dual union all
select 31100,'031100','2007-008A','Soyuz TMA-10','Soyuz 11F732 No. 220',68350,'Landed','2007-10-21 00:00:00','2007-04-15 00:00:00',91.2,323,344,51.63,'Intermediate' from dual union all
select 31101,'031101','2007-008B','11S510','11S510',68350,'Reentered','2007-04-11 00:00:00','2007-04-09 00:00:00',88.36,180,207,51.65,'Intermediate' from dual union all
select 31102,'031102','2007-009A','Anik F3','Anik F3',68351,'In Earth orbit','','2007-04-17 00:00:00',1427.64,35452,35790,.09,'Drift GEO' from dual union all
select 31103,'031103','2007-009B','Briz-M DTB','Briz-M torus tank',68351,'In Earth orbit','','2007-04-17 00:00:00',315.92,318,17656,49.45,'Highly Ellip' from dual union all
select 31104,'031104','2007-009C','Briz-M No. 88521','Briz-M 88521 (88519)',68351,'In Earth orbit','','2007-04-28 00:00:00',732.22,5338,35727,10.98,'GEO Transfer' from dual union all
select 31105,'031105','2007-005H','deb H-2A','',68347,'Reentered','2007-08-04 00:00:00','2007-04-19 00:00:00',94.29,464,504,97.25,'Sun-Synch' from dual union all
select 31106,'031106','2007-005J','deb H-2A','',68347,'Reentered','2007-07-18 00:00:00','2007-04-19 00:00:00',94.12,466,486,97.22,'Sun-Synch' from dual union all
select 31107,'031107','2007-005K','deb H-2A','',68347,'Reentered','2007-09-25 00:00:00','2007-04-19 00:00:00',94.15,463,492,97.22,'Sun-Synch' from dual union all
select 31108,'031108','2007-005L','deb H-2A','',68347,'Reentered','2007-04-12 00:00:00','2007-04-20 00:00:00',94.15,465,490,97.22,'Sun-Synch' from dual union all
select 31109,'031109','2007-005M','deb H-2A','',68347,'Reentered','2007-04-13 00:00:00','2007-04-20 00:00:00',94.15,465,490,97.22,'Sun-Synch' from dual union all
select 3111,'003111','1968-006G','deb Kosmos-200','Tselina-O part',16733,'Reentered','1968-04-15 00:00:00','1968-02-19 00:00:00',94.42,480,500,73.92,'Intermediate' from dual union all
select 31110,'031110','2007-005N','deb H-2A','',68347,'Reentered','2007-04-16 00:00:00','2007-04-20 00:00:00',94.15,465,490,97.22,'Sun-Synch' from dual union all
select 31111,'031111','2007-005P','deb H-2A','',68347,'Reentered','2007-05-28 00:00:00','2007-04-26 00:00:00',93.61,412,491,97.45,'Sun-Synch' from dual union all
select 31112,'031112','2007-005Q','deb H-2A','',68347,'Reentered','2007-04-13 00:00:00','2007-04-20 00:00:00',94.15,465,490,97.22,'Sun-Synch' from dual union all
select 31113,'031113','2007-010A','Hai Yang 1B','haiyang yihao B',68352,'In Earth orbit','','2007-04-18 00:00:00',100.83,782,814,98.62,'Sun-Synch' from dual union all
select 31114,'031114','2007-010B','CZ-2C Stage 2','CZ-2C Stage 2',68352,'In Earth orbit','','2007-04-18 00:00:00',101.49,786,872,98.37,'Sun-Synch' from dual union all
select 31115,'031115','2007-011A','Beidou DW 1','yi ke beidou daohang wei',68353,'In Earth orbit','','2007-04-20 00:00:00',773.37,21519,21544,55.26,'Medium' from dual union all
select 31116,'031116','2007-011B','CZ-3A Stage 3','CZ-3A Stage 3',68353,'Reentered','2017-07-18 00:00:00','2007-04-21 00:00:00',375.05,248,21451,54.9,'Highly Ellip' from dual union all
select 31117,'031117','2007-012A','MisrSat 1','Egyptsat 1',68354,'In Earth orbit','','2007-04-24 00:00:00',97.95,657,665,98.09,'Sun-Synch' from dual union all
select 31118,'031118','2007-012B','Saudisat 3','Saudisat 3',68354,'In Earth orbit','','2007-04-24 00:00:00',98.08,655,678,98.09,'Sun-Synch' from dual union all
select 31119,'031119','2007-012C','SaudiComsat-7','SaudiComsat-7',68354,'In Earth orbit','','2007-04-24 00:00:00',98.66,651,738,98.08,'Sun-Synch' from dual union all
select 3112,'003112','1968-006H','deb Kosmos-200','Tselina-O part',16733,'Reentered','1968-06-16 00:00:00','1968-02-27 00:00:00',94.47,474,512,74.04,'Intermediate' from dual union all
select 31120,'031120','2007-012D','Dnepr plume shield','GDS',68354,'In Earth orbit','','2007-04-24 00:00:00',97.82,649,659,98.09,'Sun-Synch' from dual union all
select 31121,'031121','2007-012E','SaudiComsat-6','SaudiComsat-6',68354,'In Earth orbit','','2007-04-24 00:00:00',98.87,649,761,98.08,'Sun-Synch' from dual union all
select 31122,'031122','2007-012F','CSTB 1','CSTB 1',68354,'In Earth orbit','','2007-04-24 00:00:00',98.96,646,772,98.08,'Sun-Synch' from dual union all
select 31123,'031123','2007-012G','R-36M2 DS','R-36M2 DS',68354,'In Earth orbit','','2007-04-24 00:00:00',101.98,625,1080,98.08,'Sun-Synch' from dual union all
select 31124,'031124','2007-012H','SaudiComsat-5','SaudiComsat-5',68354,'In Earth orbit','','2007-04-24 00:00:00',98.56,652,728,98.09,'Sun-Synch' from dual union all
select 31125,'031125','2007-012J','SaudiComsat-3','SaudiComsat-3',68354,'In Earth orbit','','2007-04-24 00:00:00',98.45,653,717,98.09,'Sun-Synch' from dual union all
select 31126,'031126','2007-012K','MAST Gadget','MAST',68354,'In Earth orbit','','2007-04-24 00:00:00',99.07,647,782,98.08,'Sun-Synch' from dual union all
select 31127,'031127','2007-012L','SaudiComsat-4','SaudiComsat-4',68354,'In Earth orbit','','2007-04-24 00:00:00',98.77,650,749,98.09,'Sun-Synch' from dual union all
select 31128,'031128','2007-012M','Libertad','Libertad',68354,'In Earth orbit','','2007-04-24 00:00:00',99.17,646,792,98.08,'Sun-Synch' from dual union all
select 31129,'031129','2007-012N','CP3','CalPoly CP3',68354,'In Earth orbit','','2007-04-24 00:00:00',99.17,646,792,98.08,'Sun-Synch' from dual union all
select 3113,'003113','1968-008A','OPS 2243','CORONA J-45',16735,'Reentered Attached','1968-02-27 00:00:00','1968-01-31 00:00:00',90.55,174,429,81.5,'Intermediate' from dual union all
select 31130,'031130','2007-012P','CAPE 1','CAPE 1',68354,'In Earth orbit','','2007-04-24 00:00:00',99.17,646,792,98.08,'Sun-Synch' from dual union all
select 31131,'031131','2003-049D','CZ-4B sep motor cover','',67673,'In Earth orbit','','2007-04-24 00:00:00',98.96,648,770,98.09,'Sun-Synch' from dual union all
select 31132,'031132','2007-012Q','CP4','CalPoly CP4',68354,'In Earth orbit','','2007-04-24 00:00:00',98.96,649,769,98.08,'Sun-Synch' from dual union all
select 31133,'031133','2007-012R','Aerocube 2','Aerocube 2',68354,'In Earth orbit','','2007-04-24 00:00:00',98.96,648,770,98.09,'Sun-Synch' from dual union all
select 31134,'031134','2007-006H','Astro Separation Ring','',68348,'Reentered','2012-09-27 00:00:00','2007-04-18 00:00:00',94.48,490,496,46.03,'Intermediate' from dual union all
select 31135,'031135','2007-013A','AGILE','AGILE',68355,'In Earth orbit','','2007-05-05 00:00:00',95.39,523,552,2.47,'Equatorial' from dual union all
select 31136,'031136','2007-013B','PS4','PS4',68355,'In Earth orbit','','2007-05-05 00:00:00',95.44,531,549,2.49,'Equatorial' from dual union all
select 31137,'031137','1995-021D','deb ERS-2','',65682,'In Earth orbit','','2007-05-08 00:00:00',101.9,818,880,99.24,'Sun-Synch' from dual union all
select 31138,'031138','1979-101H','deb Satcom 3','',47585,'In Earth orbit','','2007-05-07 00:00:00',786.5,8083,35610,10.24,'Medium' from dual union all
select 31139,'031139','2003-049E','CZ-4B sep motor cover','',67673,'In Earth orbit','','2007-04-24 00:00:00',98.95,648,769,98.09,'Sun-Synch' from dual union all
select 3114,'003114','1968-008B','TIVOLI I','P-11 No. 4412',16735,'Reentered','1970-03-04 00:00:00','1968-02-26 00:00:00',94.64,465,538,81.66,'Intermediate' from dual union all
select 31140,'031140','2007-014A','NFIRE','NFIRE',68356,'Reentered','2015-11-04 00:00:00','2007-05-01 00:00:00',91.71,254,463,48.22,'Intermediate' from dual union all
select 31141,'031141','2007-014B','Minotaur Stage 4','Orion 38',68356,'Reentered','2007-09-26 00:00:00','2007-05-01 00:00:00',91.56,251,452,48.23,'Intermediate' from dual union all
select 31142,'031142','1966-040AQ','Nimbus 2 debris','',13447,'In Earth orbit','','2007-04-22 00:00:00',107.98,1096,1171,100.53,'Sun-Synch' from dual union all
select 31143,'031143','1995-059D','deb Delta','',65720,'Reentered','2007-04-27 00:00:00','2007-04-17 00:00:00',106.76,442,1712,99.36,'Sun-Synch' from dual union all
select 31144,'031144','1999-025BGW','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.89,838,953,99.19,'Sun-Synch' from dual union all
select 31145,'031145','1999-025BGX','deb FY-1C','',66706,'Reentered','2010-10-08 00:00:00','2007-04-24 00:00:00',100.5,689,875,98.85,'Sun-Synch' from dual union all
select 31146,'031146','1999-025BGY','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',101.68,663,1014,99.21,'Sun-Synch' from dual union all
select 31147,'031147','1999-025BGZ','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',100.2,595,941,99.22,'Sun-Synch' from dual;
insert into SATELLITE
select 31148,'031148','1999-025BHA','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.11,684,939,98.99,'Sun-Synch' from dual union all
select 31149,'031149','1999-025BHB','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.07,677,942,99.21,'Sun-Synch' from dual union all
select 3115,'003115','1968-003C','Kosmos-199','Zenit-2 No. 59',16730,'Reentered','1968-01-30 00:00:00','1968-01-25 00:00:00',88.36,144,242,65.67,'Intermediate' from dual union all
select 31150,'031150','1999-025BHC','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.1,847,870,98.82,'Sun-Synch' from dual union all
select 31151,'031151','1999-025BHD','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',103.38,855,982,98.86,'Sun-Synch' from dual union all
select 31152,'031152','1999-025BHE','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',100.47,684,878,98.63,'Sun-Synch' from dual union all
select 31153,'031153','1999-025BHF','deb FY-1C','',66706,'Reentered','2013-11-17 00:00:00','2007-04-24 00:00:00',99.15,579,857,99.01,'Sun-Synch' from dual union all
select 31154,'031154','1999-025BHG','deb FY-1C','',66706,'In Earth orbit','','2007-04-21 00:00:00',100.97,727,882,98.46,'Sun-Synch' from dual union all
select 31155,'031155','1999-025BHH','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',103.53,841,1010,99.04,'Sun-Synch' from dual union all
select 31156,'031156','1999-025BHJ','deb FY-1C','',66706,'Reentered','2015-10-15 00:00:00','2007-04-23 00:00:00',100.26,684,858,99.06,'Sun-Synch' from dual union all
select 31157,'031157','1999-025BHK','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',101.58,757,910,99.01,'Sun-Synch' from dual union all
select 31158,'031158','1999-025BHL','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.32,818,918,98.8,'Sun-Synch' from dual union all
select 31159,'031159','1999-025BHM','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',100.31,463,1083,97.68,'Sun-Synch' from dual union all
select 3116,'003116','1968-004C','OPS 1965 sensor cover','',16731,'Reentered','1968-03-22 00:00:00','1968-02-18 00:00:00',93.59,416,485,75.16,'Intermediate' from dual union all
select 31160,'031160','1999-025BHN','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.01,859,943,98.79,'Sun-Synch' from dual union all
select 31161,'031161','1999-025BHP','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.54,826,932,98.87,'Sun-Synch' from dual union all
select 31162,'031162','1999-025BHQ','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.02,843,960,98.89,'Sun-Synch' from dual union all
select 31163,'031163','1999-025BHR','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.32,776,867,99.07,'Sun-Synch' from dual union all
select 31164,'031164','1999-025BHS','deb FY-1C','',66706,'Reentered','2013-08-29 00:00:00','2007-04-23 00:00:00',98.1,435,901,99.37,'Sun-Synch' from dual union all
select 31165,'031165','1999-025BHT','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',100.31,686,860,98.97,'Sun-Synch' from dual union all
select 31166,'031166','1999-025BHU','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.78,824,862,98.83,'Sun-Synch' from dual union all
select 31167,'031167','1999-025BHV','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.04,752,864,99.06,'Sun-Synch' from dual union all
select 31168,'031168','1999-025BHW','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',99.76,633,861,98.99,'Sun-Synch' from dual union all
select 31169,'031169','1999-025BHX','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.36,857,978,98.84,'Sun-Synch' from dual union all
select 3117,'003117','1968-003D','deb Kosmos-199','',16730,'Reentered','1968-02-03 00:00:00','1968-01-30 00:00:00',90.7,177,441,65.58,'Intermediate' from dual union all
select 31170,'031170','1999-025BHY','deb FY-1C','',66706,'Reentered','2015-03-20 00:00:00','2007-04-24 00:00:00',98.65,510,878,98.59,'Sun-Synch' from dual union all
select 31171,'031171','1999-025BHZ','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.64,739,1123,98.03,'Sun-Synch' from dual union all
select 31172,'031172','1999-025BJA','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',100.84,686,911,98.58,'Sun-Synch' from dual union all
select 31173,'031173','1999-025BJB','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.48,858,989,98.19,'Sun-Synch' from dual union all
select 31174,'031174','1999-025BJC','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.43,857,891,99.06,'Sun-Synch' from dual union all
select 31175,'031175','1999-025BJD','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.33,797,1036,98.43,'Sun-Synch' from dual union all
select 31176,'031176','1999-025BJE','deb FY-1C','',66706,'Reentered','2014-04-02 00:00:00','2007-04-24 00:00:00',98.07,422,911,98.58,'Sun-Synch' from dual union all
select 31177,'031177','1999-025BJF','deb FY-1C','',66706,'In Earth orbit','','2007-04-22 00:00:00',108.04,856,1417,98.77,'Sun-Synch' from dual union all
select 31178,'031178','1999-025BJG','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.45,795,1049,99.56,'Sun-Synch' from dual union all
select 31179,'031179','1999-025BJH','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',104.31,788,1137,99.32,'Sun-Synch' from dual union all
select 3118,'003118','1968-009A','Kosmos-201','Zenit-4 No. 40',16736,'Landed','1968-02-14 00:00:00','1968-02-10 00:00:00',89.82,202,330,64.95,'Intermediate' from dual union all
select 31180,'031180','1999-025BJJ','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',105.54,636,1404,99.11,'Sun-Synch' from dual union all
select 31181,'031181','1999-025BJK','deb FY-1C','',66706,'In Earth orbit','','2007-04-22 00:00:00',102.96,855,943,99.07,'Sun-Synch' from dual union all
select 31182,'031182','1999-025BJL','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.2,846,880,99.02,'Sun-Synch' from dual union all
select 31183,'031183','1999-025BJM','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.21,764,867,99.11,'Sun-Synch' from dual union all
select 31184,'031184','1999-025BJN','deb FY-1C','',66706,'Reentered','2007-05-06 00:00:00','2007-04-21 00:00:00',91.8,249,477,98.03,'Sun-Synch' from dual union all
select 31185,'031185','1999-025BJP','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.8,856,926,99.01,'Sun-Synch' from dual union all
select 31186,'031186','1999-025BJQ','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.36,851,890,99.36,'Sun-Synch' from dual union all
select 31187,'031187','1999-025BJR','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',104.1,605,1300,99.25,'Sun-Synch' from dual union all
select 31188,'031188','1999-025BJS','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.72,832,943,99.1,'Sun-Synch' from dual union all
select 31189,'031189','1999-025BJT','deb FY-1C','',66706,'Reentered','2015-02-09 00:00:00','2007-04-24 00:00:00',100.37,650,903,99.1,'Sun-Synch' from dual union all
select 3119,'003119','1968-009B','11A57I','11A57I',16736,'Reentered','1968-02-13 00:00:00','1968-02-08 00:00:00',89.48,199,299,64.95,'Intermediate' from dual union all
select 31190,'031190','1999-025BJU','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.25,857,967,99.11,'Sun-Synch' from dual union all
select 31191,'031191','1999-025BJV','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',100.56,709,862,98.96,'Sun-Synch' from dual union all
select 31192,'031192','1999-025BJW','deb FY-1C','',66706,'In Earth orbit','','2007-04-21 00:00:00',101.9,646,1052,99.03,'Sun-Synch' from dual union all
select 31193,'031193','1999-025BJX','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',99.33,596,858,99.15,'Sun-Synch' from dual union all
select 31194,'031194','1999-025BJY','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.5,787,872,99.09,'Sun-Synch' from dual union all
select 31195,'031195','1999-025BJZ','deb FY-1C','',66706,'In Earth orbit','','2007-04-21 00:00:00',102.44,746,1003,98.81,'Sun-Synch' from dual union all
select 31196,'031196','1999-025BKA','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.69,792,886,98.77,'Sun-Synch' from dual union all
select 31197,'031197','1999-025BKB','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.99,847,954,99.03,'Sun-Synch' from dual union all
select 31198,'031198','1999-025BKC','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',104.76,832,1135,98.54,'Sun-Synch' from dual union all
select 31199,'031199','1999-025BKD','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.4,823,922,99.12,'Sun-Synch' from dual union all
select 312,'000312','62 A Alpha 3','Tiros 5 despin weight','Despin weight',8136,'In Earth orbit','','1962-08-15 00:00:00',101.73,602,1080,58.18,'Intermediate' from dual union all
select 3120,'003120','1968-008C','Fairing','Fairing',16735,'Reentered','1968-02-12 00:00:00','1968-02-08 00:00:00',90.01,175,375,81.5,'Intermediate' from dual union all
select 31200,'031200','1999-025BKE','deb FY-1C','',66706,'Reentered','2012-12-18 00:00:00','2007-04-24 00:00:00',99.33,599,855,98.99,'Sun-Synch' from dual union all
select 31201,'031201','1999-025BKF','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',101.58,702,964,99.09,'Sun-Synch' from dual union all
select 31202,'031202','1999-025BKG','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',99.97,656,859,96.59,'Sun-Synch' from dual union all
select 31203,'031203','1999-025BKH','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.66,832,937,98.96,'Sun-Synch' from dual union all
select 31204,'031204','1999-025BKJ','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',104.41,781,1152,98.07,'Sun-Synch' from dual union all
select 31205,'031205','1999-025BKK','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',101.31,700,942,99.35,'Sun-Synch' from dual union all
select 31206,'031206','1999-025BKL','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',100.91,733,871,99.01,'Sun-Synch' from dual union all
select 31207,'031207','1999-025BKM','deb FY-1C','',66706,'Reentered','2018-01-08 00:00:00','2007-04-24 00:00:00',101.5,755,905,98.4,'Sun-Synch' from dual union all
select 31208,'031208','1999-025BKN','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.64,814,859,99.32,'Sun-Synch' from dual union all
select 31209,'031209','1999-025BKP','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.56,854,906,98.81,'Sun-Synch' from dual union all
select 3121,'003121','1963-030F','deb Midas','',9261,'In Earth orbit','','1967-05-31 00:00:00',167.82,3665,3735,88.41,'Medium' from dual union all
select 31210,'031210','1999-025BKQ','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.37,858,883,98.94,'Sun-Synch' from dual union all
select 31211,'031211','1999-025BKR','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.57,840,920,99.03,'Sun-Synch' from dual union all
select 31212,'031212','1999-025BKS','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.39,854,984,98.86,'Sun-Synch' from dual union all
select 31213,'031213','1999-025BKT','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.69,856,916,99.47,'Sun-Synch' from dual union all
select 31214,'031214','1999-025BKU','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.51,857,898,98.83,'Sun-Synch' from dual union all
select 31215,'031215','1999-025BKV','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.82,792,898,99.14,'Sun-Synch' from dual union all
select 31216,'031216','1999-025BKW','deb FY-1C','',66706,'Reentered','2008-05-02 00:00:00','2007-04-23 00:00:00',103.79,505,1370,99.94,'Sun-Synch' from dual union all
select 31217,'031217','1999-025BKX','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.32,787,949,98.94,'Sun-Synch' from dual union all
select 31218,'031218','1999-025BKY','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.46,844,907,98.76,'Sun-Synch' from dual union all
select 31219,'031219','1999-025BKZ','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.44,856,987,98.84,'Sun-Synch' from dual union all
select 3122,'003122','1967-092D','NNS O-14 despin weight','Transit high deb',15347,'In Earth orbit','','1967-10-31 00:00:00',109.2,1041,1339,89.11,'Polar' from dual union all
select 31220,'031220','1999-025BLA','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.19,763,867,99.07,'Sun-Synch' from dual union all
select 31221,'031221','1999-025BLB','deb FY-1C','',66706,'Reentered','2011-02-10 00:00:00','2007-04-24 00:00:00',104.94,718,1266,99.38,'Sun-Synch' from dual union all
select 31222,'031222','1999-025BLC','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',99.25,498,948,98.73,'Sun-Synch' from dual union all
select 31223,'031223','1999-025BLD','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',106.99,852,1323,99.07,'Sun-Synch' from dual union all
select 31224,'031224','1999-025BLE','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',101.36,698,949,98.97,'Sun-Synch' from dual union all
select 31225,'031225','1999-025BLF','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',103.09,827,982,98.92,'Sun-Synch' from dual union all
select 31226,'031226','1999-025BLG','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.36,727,919,99,'Sun-Synch' from dual union all
select 31227,'031227','1999-025BLH','deb FY-1C','',66706,'In Earth orbit','','2007-04-21 00:00:00',102.84,777,1009,98.18,'Sun-Synch' from dual union all
select 31228,'031228','1999-025BLJ','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',100.97,739,870,99.04,'Sun-Synch' from dual union all
select 31229,'031229','1999-025BLK','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.8,839,944,98.91,'Sun-Synch' from dual union all
select 3123,'003123','1967-114D','ESSA 6 despin weight','',15369,'In Earth orbit','','1967-12-31 00:00:00',115.43,1452,1498,102.49,'Sun-Synch' from dual union all
select 31230,'031230','1999-025BLL','deb FY-1C','',66706,'Reentered','2008-02-14 00:00:00','2007-04-24 00:00:00',101.81,582,1107,98.26,'Sun-Synch' from dual union all
select 31231,'031231','1999-025BLM','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',102.7,626,1147,98.98,'Sun-Synch' from dual union all
select 31232,'031232','1999-025BLN','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.77,850,929,98.88,'Sun-Synch' from dual union all
select 31233,'031233','1999-025BLP','deb FY-1C','',66706,'Reentered','2015-07-24 00:00:00','2007-04-23 00:00:00',101.03,631,984,97.9,'Sun-Synch' from dual union all
select 31234,'031234','1999-025BLQ','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.57,825,935,99.02,'Sun-Synch' from dual union all
select 31235,'031235','1999-025BLR','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',105.14,785,1217,98.51,'Sun-Synch' from dual union all
select 31236,'031236','1999-025BLS','deb FY-1C','',66706,'In Earth orbit','','2007-04-21 00:00:00',106.73,764,1387,98.56,'Sun-Synch' from dual union all
select 31237,'031237','1999-025BLT','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.32,747,896,98.8,'Sun-Synch' from dual;
insert into SATELLITE
select 31238,'031238','1999-025BLU','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.06,748,965,99.03,'Sun-Synch' from dual union all
select 31239,'031239','1999-025BLV','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.53,840,917,99.03,'Sun-Synch' from dual union all
select 3124,'003124','61 Omicron 214','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-02 00:00:00',108.82,864,1481,67.01,'Intermediate' from dual union all
select 31240,'031240','1999-025BLW','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.77,858,922,99.04,'Sun-Synch' from dual union all
select 31241,'031241','1999-025BLX','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',100.92,745,860,99.29,'Sun-Synch' from dual union all
select 31242,'031242','1999-025BLY','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',100.61,709,865,97.43,'Sun-Synch' from dual union all
select 31243,'031243','1999-025BLZ','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.09,858,952,99.32,'Sun-Synch' from dual union all
select 31244,'031244','1999-025BMA','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',102.87,804,985,98.75,'Sun-Synch' from dual union all
select 31245,'031245','1999-025BMB','deb FY-1C','',66706,'Reentered','2012-12-05 00:00:00','2007-04-24 00:00:00',99.4,606,853,98.95,'Sun-Synch' from dual union all
select 31246,'031246','1999-025BMC','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',104.16,818,1093,98.94,'Sun-Synch' from dual union all
select 31247,'031247','1999-025BMD','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.22,855,966,99.48,'Sun-Synch' from dual union all
select 31248,'031248','1999-025BME','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.9,857,1029,98.99,'Sun-Synch' from dual union all
select 31249,'031249','1999-025BMF','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.46,801,949,98.88,'Sun-Synch' from dual union all
select 3125,'003125','61 Omicron 215','deb r/b Ablestar','',7273,'In Earth orbit','','1992-01-28 00:00:00',109.41,962,1436,67.54,'Intermediate' from dual union all
select 31250,'031250','1999-025BMG','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.61,821,1038,98.41,'Sun-Synch' from dual union all
select 31251,'031251','1999-025BMH','deb FY-1C','',66706,'Reentered','2010-06-25 00:00:00','2007-04-21 00:00:00',96.83,325,889,99.09,'Sun-Synch' from dual union all
select 31252,'031252','1999-025BMJ','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',100.88,621,980,98.56,'Sun-Synch' from dual union all
select 31253,'031253','1999-025BMK','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.01,683,1118,98.77,'Sun-Synch' from dual union all
select 31254,'031254','1999-025BML','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.08,683,937,98.49,'Sun-Synch' from dual union all
select 31255,'031255','1999-025BMM','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',102.68,673,1098,99.34,'Sun-Synch' from dual union all
select 31256,'031256','1999-025BMN','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',106.02,703,1382,98.71,'Sun-Synch' from dual union all
select 31257,'031257','1999-025BMP','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',103.91,838,1049,98.72,'Sun-Synch' from dual union all
select 31258,'031258','1999-025BMQ','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.42,855,892,98.81,'Sun-Synch' from dual union all
select 31259,'031259','1999-025BMR','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',103.41,859,981,99.04,'Sun-Synch' from dual union all
select 3126,'003126','1968-002C','Explorer 36 despin weight','',16729,'In Earth orbit','','1968-02-15 00:00:00',112.38,1087,1585,106.06,'Retrograde' from dual union all
select 31260,'031260','1999-025BMS','deb FY-1C','',66706,'In Earth orbit','','2007-04-21 00:00:00',101.49,787,871,99.09,'Sun-Synch' from dual union all
select 31261,'031261','1999-025BMT','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.31,773,869,99.13,'Sun-Synch' from dual union all
select 31262,'031262','1999-025BMU','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',103.23,857,966,98.97,'Sun-Synch' from dual union all
select 31263,'031263','1999-025BMV','deb FY-1C','',66706,'In Earth orbit','','2007-04-22 00:00:00',102.4,832,913,99.08,'Sun-Synch' from dual union all
select 31264,'031264','1999-025BMW','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.08,856,859,99.27,'Sun-Synch' from dual union all
select 31265,'031265','1999-025BMX','deb FY-1C','',66706,'In Earth orbit','','2007-04-22 00:00:00',100.9,686,917,96.57,'Sun-Synch' from dual union all
select 31266,'031266','1999-025BMY','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.66,735,939,99.29,'Sun-Synch' from dual union all
select 31267,'031267','1999-025BMZ','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',100.48,675,887,99.35,'Sun-Synch' from dual union all
select 31268,'031268','1999-025BNA','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.08,856,953,99.31,'Sun-Synch' from dual union all
select 31269,'031269','1999-025BNB','deb FY-1C','',66706,'In Earth orbit','','2007-04-21 00:00:00',103.05,611,1195,98.19,'Sun-Synch' from dual union all
select 3127,'003127','1968-002D','Explorer 36 despin weight','',16729,'In Earth orbit','','1968-02-15 00:00:00',112.17,1079,1574,105.32,'Retrograde' from dual union all
select 31270,'031270','1999-025BNC','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',100.35,648,903,99.07,'Sun-Synch' from dual union all
select 31271,'031271','1999-025BND','deb FY-1C','',66706,'Reentered','2014-10-16 00:00:00','2007-04-23 00:00:00',97.69,409,888,99.18,'Sun-Synch' from dual union all
select 31272,'031272','1999-025BNE','deb FY-1C','',66706,'In Earth orbit','','2007-04-20 00:00:00',101.89,761,936,99.19,'Sun-Synch' from dual union all
select 31273,'031273','1999-025BNF','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',101.51,766,895,98.37,'Sun-Synch' from dual union all
select 31274,'031274','1999-025BNG','deb FY-1C','',66706,'In Earth orbit','','2007-04-22 00:00:00',104.96,793,1193,99.07,'Sun-Synch' from dual union all
select 31275,'031275','1999-025BNH','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.55,841,1012,99.08,'Sun-Synch' from dual union all
select 31276,'031276','1999-025BNJ','deb FY-1C','',66706,'Reentered','2016-05-17 00:00:00','2007-04-24 00:00:00',100.67,571,1010,98.74,'Sun-Synch' from dual union all
select 31277,'031277','1999-025BNK','deb FY-1C','',66706,'Reentered','2014-05-15 00:00:00','2007-04-24 00:00:00',100.77,615,976,97.74,'Sun-Synch' from dual union all
select 31278,'031278','1999-025BNL','deb FY-1C','',66706,'Reentered','2012-04-28 00:00:00','2007-04-24 00:00:00',99.12,341,1093,96.37,'Sun-Synch' from dual union all
select 31279,'031279','1999-025BNM','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.65,776,898,98.78,'Sun-Synch' from dual union all
select 3128,'003128','1968-010A','Kosmos-202','DS-U2-V No. 4',16737,'Reentered','1968-03-24 00:00:00','1968-02-28 00:00:00',91.1,211,446,48.43,'Intermediate' from dual union all
select 31280,'031280','1999-025BNN','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',100.65,616,963,98.03,'Sun-Synch' from dual union all
select 31281,'031281','1999-025BNP','deb FY-1C','',66706,'In Earth orbit','','2007-04-21 00:00:00',100.05,517,1005,98.08,'Sun-Synch' from dual union all
select 31282,'031282','1999-025BNQ','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',100.15,666,865,99.31,'Sun-Synch' from dual union all
select 31283,'031283','1999-025BNR','deb FY-1C','',66706,'Reentered','2012-10-02 00:00:00','2007-04-24 00:00:00',101.13,419,1206,99.56,'Sun-Synch' from dual union all
select 31284,'031284','1999-025BNS','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.64,817,1044,99.82,'Sun-Synch' from dual union all
select 31285,'031285','1999-025BNT','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',101.36,669,978,98.9,'Sun-Synch' from dual union all
select 31286,'031286','1999-025BNU','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.91,850,1037,100.86,'Sun-Synch' from dual union all
select 31287,'031287','1999-025BNV','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',102.68,858,912,99.47,'Sun-Synch' from dual union all
select 31288,'031288','1999-025BNW','deb FY-1C','',66706,'In Earth orbit','','2007-04-22 00:00:00',101.86,797,897,98.79,'Sun-Synch' from dual union all
select 31289,'031289','1999-025BNX','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',98.5,414,960,97.6,'Sun-Synch' from dual union all
select 3129,'003129','1968-011A','Kosmos-203','Sfera',16738,'In Earth orbit','','1968-03-22 00:00:00',109.27,1185,1201,74.06,'Intermediate' from dual union all
select 31290,'031290','1999-025BNY','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.94,743,957,101.96,'Sun-Synch' from dual union all
select 31291,'031291','1999-025BNZ','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',101.49,761,898,98.44,'Sun-Synch' from dual union all
select 31292,'031292','1999-025BPA','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',103.01,857,945,98.97,'Sun-Synch' from dual union all
select 31293,'031293','1999-025BPB','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',100.76,701,889,98.54,'Sun-Synch' from dual union all
select 31294,'031294','1999-025BPC','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.23,786,943,99.16,'Sun-Synch' from dual union all
select 31295,'031295','1999-025BPD','deb FY-1C','',66706,'In Earth orbit','','2007-04-22 00:00:00',103.2,853,967,98.34,'Sun-Synch' from dual union all
select 31296,'031296','1999-025BPE','deb FY-1C','',66706,'In Earth orbit','','2007-04-22 00:00:00',102.5,843,910,99.12,'Sun-Synch' from dual union all
select 31297,'031297','1999-025BPF','deb FY-1C','',66706,'In Earth orbit','','2007-04-21 00:00:00',99.63,623,858,99,'Sun-Synch' from dual union all
select 31298,'031298','1999-025BPG','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',100.42,695,861,99.05,'Sun-Synch' from dual union all
select 31299,'031299','1999-025BPH','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',100.67,431,1150,96.98,'Sun-Synch' from dual union all
select 313,'000313','62 A Alpha 4','Tiros 5 despin weight','Despin weight',8136,'Reentered','1992-03-02 00:00:00','1963-02-15 00:00:00',99.09,564,867,57.99,'Intermediate' from dual union all
select 3130,'003130','1968-010B','S1M','S1M',16737,'Reentered','1968-03-04 00:00:00','1968-02-23 00:00:00',91.19,200,466,48.43,'Intermediate' from dual union all
select 31300,'031300','1999-025BPJ','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',103.09,852,958,99.63,'Sun-Synch' from dual union all
select 31301,'031301','1999-025BPK','deb FY-1C','',66706,'Reentered','2011-04-15 00:00:00','2007-04-24 00:00:00',99.02,451,973,97.61,'Sun-Synch' from dual union all
select 31302,'031302','1999-025BPL','deb FY-1C','',66706,'In Earth orbit','','2007-04-24 00:00:00',102.66,858,911,99.46,'Sun-Synch' from dual union all
select 31303,'031303','1999-025BPM','deb FY-1C','',66706,'In Earth orbit','','2007-04-23 00:00:00',102.19,855,870,98.8,'Sun-Synch' from dual union all
select 31304,'031304','2007-015A','AIM','AIM',68357,'In Earth orbit','','2007-05-02 00:00:00',96.53,585,600,97.79,'Sun-Synch' from dual union all
select 31305,'031305','2007-015B','Orion 38','Orion 38',68357,'In Earth orbit','','2007-05-03 00:00:00',96.49,582,599,97.8,'Sun-Synch' from dual union all
select 31306,'031306','2007-016A','Astra 1L','Astra 1L',68358,'In Earth orbit','','2007-05-14 00:00:00',1433.94,35704,35785,.14,'Drift GEO' from dual union all
select 31307,'031307','2007-016B','Galaxy 17','Galaxy 17',68358,'In Earth orbit','','2007-05-13 00:00:00',1430.16,35526,35815,.05,'Drift GEO' from dual union all
select 31308,'031308','2007-016C','ESC-A 536','Ariane 5 ESC-A',68358,'In Earth orbit','','2007-05-14 00:00:00',629.54,254,35654,5.9,'GEO Transfer' from dual union all
select 31309,'031309','2007-016D','Sylda 5A','Sylda 5',68358,'Reentered','2018-04-03 00:00:00','2007-05-09 00:00:00',631.86,267,35760,6.03,'GEO Transfer' from dual union all
select 3131,'003131','1968-011B','S3M','S3M',16738,'In Earth orbit','','1968-03-25 00:00:00',109.31,1182,1208,74.06,'Intermediate' from dual union all
select 31310,'031310','1999-025BPN','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',103.85,849,1032,98.93,'Sun-Synch' from dual union all
select 31311,'031311','1999-025BPP','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',102.58,859,903,98.84,'Sun-Synch' from dual union all
select 31312,'031312','1999-025BPQ','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',103.12,857,956,99.26,'Sun-Synch' from dual union all
select 31313,'031313','1999-025BPR','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',101.62,800,870,99.2,'Sun-Synch' from dual union all
select 31314,'031314','1999-025BPS','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',102.79,835,947,99.06,'Sun-Synch' from dual union all
select 31315,'031315','1999-025BPT','deb FY-1C','',66706,'Reentered','2012-04-21 00:00:00','2007-05-07 00:00:00',100.04,367,1154,98.3,'Sun-Synch' from dual union all
select 31316,'031316','1999-025BPU','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',103.02,805,998,99.05,'Sun-Synch' from dual union all
select 31317,'031317','1999-025BPV','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',102.95,765,1032,99.38,'Sun-Synch' from dual union all
select 31318,'031318','1999-025BPW','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',101.11,702,920,98.92,'Sun-Synch' from dual union all
select 31319,'031319','1999-025BPX','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',104.78,858,1111,98.74,'Sun-Synch' from dual union all
select 3132,'003132','1963-030G','deb Midas','',9261,'In Earth orbit','','1981-05-29 00:00:00',167.85,3689,3713,88.4,'Medium' from dual union all
select 31320,'031320','1999-025BPY','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',103.71,831,1037,98.32,'Sun-Synch' from dual union all
select 31321,'031321','1999-025BPZ','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',102.03,833,877,99.21,'Sun-Synch' from dual union all
select 31322,'031322','1999-025BQA','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',105.05,818,1175,98.53,'Sun-Synch' from dual union all
select 31323,'031323','1999-025BQB','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',102.71,857,917,99.47,'Sun-Synch' from dual union all
select 31324,'031324','1999-025BQC','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',101.96,814,889,99.31,'Sun-Synch' from dual union all
select 31325,'031325','1999-025BQD','deb FY-1C','',66706,'In Earth orbit','','2007-05-05 00:00:00',103.38,855,982,99.23,'Sun-Synch' from dual union all
select 31326,'031326','1999-025BQE','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',102.69,849,923,99.02,'Sun-Synch' from dual union all
select 31327,'031327','1999-025BQF','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',99.23,564,880,99.04,'Sun-Synch' from dual;
insert into SATELLITE
select 31328,'031328','1999-025BQG','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',100.9,725,878,98.83,'Sun-Synch' from dual union all
select 31329,'031329','1999-025BQH','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',100.7,720,864,98.99,'Sun-Synch' from dual union all
select 3133,'003133','1968-012A','NNS O-18','NNS O-18',16739,'In Earth orbit','','1968-04-22 00:00:00',106.95,1025,1146,89.99,'Polar' from dual union all
select 31330,'031330','1999-025BQJ','deb FY-1C','',66706,'In Earth orbit','','2007-05-06 00:00:00',102.58,858,904,98.85,'Sun-Synch' from dual union all
select 31331,'031331','1999-025BQK','deb FY-1C','',66706,'In Earth orbit','','2007-05-05 00:00:00',102.94,859,937,98.92,'Sun-Synch' from dual union all
select 31332,'031332','1999-025BQL','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',102.87,856,932,99.16,'Sun-Synch' from dual union all
select 31333,'031333','1999-025BQM','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',101.8,761,927,99.02,'Sun-Synch' from dual union all
select 31334,'031334','1999-025BQN','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',103.27,856,970,99,'Sun-Synch' from dual union all
select 31335,'031335','1999-025BQP','deb FY-1C','',66706,'In Earth orbit','','2007-05-05 00:00:00',100.5,679,886,98.89,'Sun-Synch' from dual union all
select 31336,'031336','1999-025BQQ','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',100.68,715,867,98.37,'Sun-Synch' from dual union all
select 31337,'031337','1999-025BQR','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',101.2,765,866,99.46,'Sun-Synch' from dual union all
select 31338,'031338','1999-025BQS','deb FY-1C','',66706,'In Earth orbit','','2007-05-05 00:00:00',103.79,858,1018,98.86,'Sun-Synch' from dual union all
select 31339,'031339','1999-025BQT','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',102.87,838,951,99.14,'Sun-Synch' from dual union all
select 3134,'003134','1968-013A','Zond-4','L-1 No. 6L',16740,'Deep Space','1968-03-02 00:00:00','1968-03-02 00:00:00',9992.58,200,294400,51.62,'Deep Space' from dual union all
select 31340,'031340','1999-025BQU','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',102.76,849,930,98.87,'Sun-Synch' from dual union all
select 31341,'031341','1999-025BQV','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',101.26,758,878,99.59,'Sun-Synch' from dual union all
select 31342,'031342','1999-025BQW','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',100.49,697,867,99.03,'Sun-Synch' from dual union all
select 31343,'031343','1999-025BQX','deb FY-1C','',66706,'Reentered','2013-12-13 00:00:00','2007-05-07 00:00:00',99.29,593,856,98.94,'Sun-Synch' from dual union all
select 31344,'031344','1999-025BQY','deb FY-1C','',66706,'In Earth orbit','','2007-05-06 00:00:00',101.58,790,877,99.5,'Sun-Synch' from dual union all
select 31345,'031345','1999-025BQZ','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',103.57,858,997,98.13,'Sun-Synch' from dual union all
select 31346,'031346','1999-025BRA','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',98.82,522,883,98.91,'Sun-Synch' from dual union all
select 31347,'031347','1999-025BRB','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',101.15,689,938,98.38,'Sun-Synch' from dual union all
select 31348,'031348','1999-025BRC','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',101.9,818,880,99.24,'Sun-Synch' from dual union all
select 31349,'031349','1999-025BRD','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',103.23,858,965,98.82,'Sun-Synch' from dual union all
select 3135,'003135','1968-013B','SOZ','SOZ',16740,'Reentered','1968-03-07 00:00:00','1968-03-04 00:00:00',88.63,197,217,51.53,'Intermediate' from dual union all
select 31350,'031350','1999-025BRE','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',101.16,757,871,99.01,'Sun-Synch' from dual union all
select 31351,'031351','1999-025BRF','deb FY-1C','',66706,'In Earth orbit','','2007-05-06 00:00:00',102.68,817,954,99.13,'Sun-Synch' from dual union all
select 31352,'031352','1999-025BRG','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',100.74,659,929,99.26,'Sun-Synch' from dual union all
select 31353,'031353','1999-025BRH','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',101.53,642,1021,99.17,'Sun-Synch' from dual union all
select 31354,'031354','1999-025BRJ','deb FY-1C','',66706,'In Earth orbit','','2007-05-05 00:00:00',101.12,615,1008,98.33,'Sun-Synch' from dual union all
select 31355,'031355','1999-025BRK','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',100.15,674,858,98.96,'Sun-Synch' from dual union all
select 31356,'031356','1999-025BRL','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',101.7,789,889,98.78,'Sun-Synch' from dual union all
select 31357,'031357','1999-025BRM','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',101.54,796,867,98.59,'Sun-Synch' from dual union all
select 31358,'031358','1999-025BRN','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',101.95,706,996,99.03,'Sun-Synch' from dual union all
select 31359,'031359','1999-025BRP','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',103.78,859,1015,99.08,'Sun-Synch' from dual union all
select 3136,'003136','1968-013C','SOZ','SOZ',16740,'Reentered','1968-03-07 00:00:00','1968-03-04 00:00:00',88.53,190,214,51.54,'Intermediate' from dual union all
select 31360,'031360','1999-025BRQ','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',104.29,857,1066,98,'Sun-Synch' from dual union all
select 31361,'031361','1999-025BRR','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',101.71,806,874,99.06,'Sun-Synch' from dual union all
select 31362,'031362','1999-025BRS','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',101.8,829,859,98.85,'Sun-Synch' from dual union all
select 31363,'031363','1999-025BRT','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',103.79,818,1058,98.48,'Sun-Synch' from dual union all
select 31364,'031364','1999-025BRU','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',104.02,856,1041,99.21,'Sun-Synch' from dual union all
select 31365,'031365','1999-025BRV','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',102.07,814,900,98.85,'Sun-Synch' from dual union all
select 31366,'031366','1999-025BRW','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',102.01,851,857,98.85,'Sun-Synch' from dual union all
select 31367,'031367','1999-025BRX','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',100.48,694,868,98.91,'Sun-Synch' from dual union all
select 31368,'031368','1999-025BRY','deb FY-1C','',66706,'In Earth orbit','','2007-05-05 00:00:00',102.02,814,895,99.18,'Sun-Synch' from dual union all
select 31369,'031369','1999-025BRZ','deb FY-1C','',66706,'In Earth orbit','','2007-05-05 00:00:00',101.75,800,882,98.97,'Sun-Synch' from dual union all
select 3137,'003137','1968-012B','Altair 2 X-258E5 ABL-145','X-258E5',16739,'In Earth orbit','','1968-05-05 00:00:00',106.93,1024,1145,89.98,'Polar' from dual union all
select 31370,'031370','1999-025BSA','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',102.52,852,903,98.98,'Sun-Synch' from dual union all
select 31371,'031371','1999-025BSB','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',103.2,858,961,99.33,'Sun-Synch' from dual union all
select 31372,'031372','1999-025BSC','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',102.74,857,920,99.42,'Sun-Synch' from dual union all
select 31373,'031373','1999-025BSD','deb FY-1C','',66706,'Reentered','2014-09-12 00:00:00','2007-05-05 00:00:00',100.9,621,982,97.79,'Sun-Synch' from dual union all
select 31374,'031374','1999-025BSE','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',103.55,753,1100,99.68,'Sun-Synch' from dual union all
select 31375,'031375','1999-025BSF','deb FY-1C','',66706,'Reentered','2012-01-06 00:00:00','2007-05-08 00:00:00',97.7,403,895,98.63,'Sun-Synch' from dual union all
select 31376,'031376','1999-025BSG','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',101.12,749,875,98.98,'Sun-Synch' from dual union all
select 31377,'031377','1999-025BSH','deb FY-1C','',66706,'In Earth orbit','','2007-05-05 00:00:00',100.45,697,863,98.96,'Sun-Synch' from dual union all
select 31378,'031378','1999-025BSJ','deb FY-1C','',66706,'In Earth orbit','','2007-05-06 00:00:00',104.93,858,1124,100.81,'Sun-Synch' from dual union all
select 31379,'031379','1999-025BSK','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',100.54,609,959,99.54,'Sun-Synch' from dual union all
select 3138,'003138','1968-014A','OGO 5','OGO E',16741,'Deep Space','1968-03-05 00:00:00','1968-03-04 00:00:00',3795.9,270,148185,31.13,'Deep Space' from dual union all
select 31380,'031380','1999-025BSL','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',100.7,664,920,96.55,'Sun-Synch' from dual union all
select 31381,'031381','1999-025BSM','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',101.19,645,985,98.97,'Sun-Synch' from dual union all
select 31382,'031382','1999-025BSN','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',101.17,750,879,96.93,'Sun-Synch' from dual union all
select 31383,'031383','1999-025BSP','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',103.27,825,1002,98.92,'Sun-Synch' from dual union all
select 31384,'031384','1999-025BSQ','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',106.18,644,1456,98.46,'Sun-Synch' from dual union all
select 31385,'031385','1999-025BSR','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',100.08,659,866,99.07,'Sun-Synch' from dual union all
select 31386,'031386','1999-025BSS','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',105.4,839,1187,98.47,'Sun-Synch' from dual union all
select 31387,'031387','1999-025BST','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',102.24,679,1050,99.29,'Sun-Synch' from dual union all
select 31388,'031388','1999-025BSU','deb FY-1C','',66706,'In Earth orbit','','2007-05-08 00:00:00',102.46,840,911,99.04,'Sun-Synch' from dual union all
select 31389,'031389','1999-025BSV','deb FY-1C','',66706,'In Earth orbit','','2007-05-07 00:00:00',103.08,858,951,99,'Sun-Synch' from dual union all
select 3139,'003139','1968-015A','Kosmos-204','DS-P1-I No. 3',16742,'Reentered','1969-03-02 00:00:00','1968-04-12 00:00:00',95.4,265,810,70.99,'Intermediate' from dual union all
select 31390,'031390','2006-041L','deb M-34','',68213,'Reentered','2007-11-13 00:00:00','2007-05-10 00:00:00',93.21,274,590,98.29,'Sun-Synch' from dual union all
select 31391,'031391','2006-029D','deb Dnepr','',68201,'In Earth orbit','','2007-05-07 00:00:00',95.51,521,566,64.5,'Intermediate' from dual union all
select 31392,'031392','2006-029E','deb Dnepr','',68201,'In Earth orbit','','2007-05-09 00:00:00',95.51,521,566,64.5,'Intermediate' from dual union all
select 31393,'031393','2007-017A','Progress M-60','Progress 7K-TGM No. 360',68359,'Deorbited','2007-09-25 00:00:00','2007-05-14 00:00:00',89.64,241,273,51.64,'Intermediate' from dual union all
select 31394,'031394','2007-017B','11S510','11S510',68359,'Reentered','2007-05-15 00:00:00','2007-05-13 00:00:00',88.3,169,212,51.62,'Intermediate' from dual union all
select 31395,'031395','2007-018A','Nigcomsat 1','Nigcomsat 1',68360,'In Earth orbit','','2007-05-22 00:00:00',1442.07,35806,36001,.18,'Drift GEO' from dual union all
select 31396,'031396','2007-018B','CZ-3B Stage 3','CZ-3B Stage 3',68360,'Reentered','2009-08-16 00:00:00','2007-05-16 00:00:00',754.33,219,41924,25.35,'Highly Ellip' from dual union all
select 31397,'031397','1988-005S','deb Meteor-2','',61254,'In Earth orbit','','2007-05-01 00:00:00',103.96,925,967,82.54,'Intermediate' from dual union all
select 31398,'031398','1988-005T','deb Meteor-2','',61254,'In Earth orbit','','2007-05-02 00:00:00',104.05,932,968,82.54,'Intermediate' from dual union all
select 31399,'031399','1988-005U','deb Meteor-2','',61254,'In Earth orbit','','2007-05-08 00:00:00',103.88,929,955,82.52,'Intermediate' from dual union all
select 314,'000314','62 Omega 3','deb USAF satellite/Agena B','Ferret sensor cover',8196,'Reentered','1962-07-14 00:00:00','1962-07-03 00:00:00',91.53,350,350,82.11,'Intermediate' from dual union all
select 3140,'003140','1968-016A','Kosmos-205','Zenit-2 No. 56',16743,'Landed','1968-03-13 00:00:00','1968-03-08 00:00:00',89.31,194,287,65.64,'Intermediate' from dual union all
select 31400,'031400','1988-005V','deb Meteor-2','',61254,'In Earth orbit','','2007-05-09 00:00:00',103.97,931,962,82.54,'Intermediate' from dual union all
select 31401,'031401','1988-005W','deb Meteor-2','',61254,'In Earth orbit','','2007-05-14 00:00:00',103.94,933,957,82.54,'Intermediate' from dual union all
select 31402,'031402','1988-005X','deb Meteor-2','',61254,'In Earth orbit','','2007-05-15 00:00:00',103.97,939,954,82.55,'Intermediate' from dual union all
select 31403,'031403','1996-017D','deb PS4','',65905,'In Earth orbit','','2007-05-15 00:00:00',101.23,792,842,98.36,'Sun-Synch' from dual union all
select 31404,'031404','1979-095T','deb Meteor-2','',47579,'In Earth orbit','','2007-05-12 00:00:00',102.33,851,886,81.21,'Intermediate' from dual union all
select 31405,'031405','59 Eta 3','deb Vanguard 3','',5529,'Reentered','2014-04-03 00:00:00','2007-05-11 00:00:00',124.12,523,3206,33.34,'Medium' from dual union all
select 31406,'031406','1980-073H','deb Meteor-2','',50283,'In Earth orbit','','2007-05-15 00:00:00',102.06,829,884,81.22,'Intermediate' from dual union all
select 31407,'031407','1966-041H','deb NNS O-9 insulation','',13448,'In Earth orbit','','2007-04-16 00:00:00',102.02,802,906,90.18,'Polar' from dual union all
select 31408,'031408','1978-064R','deb Seasat','',45259,'Reentered','2008-04-14 00:00:00','2007-05-16 00:00:00',99.56,699,776,108,'Retrograde' from dual union all
select 31409,'031409','2005-024J','deb SJ-7','',68033,'In Earth orbit','','2007-04-15 00:00:00',95.76,547,564,97.64,'Sun-Synch' from dual union all
select 3141,'003141','1968-017A','SR 9','Solar Explorer B',16744,'Reentered','1990-11-16 00:00:00','1968-04-14 00:00:00',98.67,509,882,59.42,'Intermediate' from dual union all
select 31410,'031410','1999-025BSW','deb FY-1C','',66706,'In Earth orbit','','2007-05-14 00:00:00',100.93,700,906,98.94,'Sun-Synch' from dual union all
select 31411,'031411','1999-025BSX','deb FY-1C','',66706,'In Earth orbit','','2007-05-14 00:00:00',101.47,746,911,98.52,'Sun-Synch' from dual union all
select 31412,'031412','1999-025BSY','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',103.59,856,1001,98.53,'Sun-Synch' from dual union all
select 31413,'031413','1999-025BSZ','deb FY-1C','',66706,'In Earth orbit','','2007-05-14 00:00:00',101.31,766,876,98.95,'Sun-Synch' from dual union all
select 31414,'031414','1999-025BTA','deb FY-1C','',66706,'In Earth orbit','','2007-05-15 00:00:00',101.09,738,883,98.5,'Sun-Synch' from dual union all
select 31415,'031415','1999-025BTB','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',102.04,799,911,99.7,'Sun-Synch' from dual union all
select 31416,'031416','1999-025BTC','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',100.6,678,896,99.11,'Sun-Synch' from dual union all
select 31417,'031417','1999-025BTD','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',101.13,761,863,99.11,'Sun-Synch' from dual;
insert into SATELLITE
select 31418,'031418','1999-025BTE','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',102.85,841,946,99.25,'Sun-Synch' from dual union all
select 31419,'031419','1999-025BTF','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',101.35,756,890,98.92,'Sun-Synch' from dual union all
select 3142,'003142','1968-016B','11A57I','11A57I',16743,'Reentered','1968-03-09 00:00:00','1968-03-07 00:00:00',88.96,185,262,65.64,'Intermediate' from dual union all
select 31420,'031420','1999-025BTG','deb FY-1C','',66706,'In Earth orbit','','2007-05-14 00:00:00',100.89,682,919,97.52,'Sun-Synch' from dual union all
select 31421,'031421','1999-025BTH','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',102.18,842,882,99.04,'Sun-Synch' from dual union all
select 31422,'031422','1999-025BTJ','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',102.52,852,903,99,'Sun-Synch' from dual union all
select 31423,'031423','1999-025BTK','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',101.39,797,852,98.88,'Sun-Synch' from dual union all
select 31424,'031424','1999-025BTL','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',102.51,806,949,98.47,'Sun-Synch' from dual union all
select 31425,'031425','1999-025BTM','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',103.03,847,957,98.77,'Sun-Synch' from dual union all
select 31426,'031426','1999-025BTN','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',105.2,715,1292,98.61,'Sun-Synch' from dual union all
select 31427,'031427','1999-025BTP','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',101.83,804,887,98.76,'Sun-Synch' from dual union all
select 31428,'031428','1999-025BTQ','deb FY-1C','',66706,'In Earth orbit','','2007-05-14 00:00:00',101.13,752,873,99.12,'Sun-Synch' from dual union all
select 31429,'031429','1999-025BTR','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',102.36,859,882,98.84,'Sun-Synch' from dual union all
select 3143,'003143','1968-015B','S1M','S1M',16742,'Reentered','1968-10-13 00:00:00','1968-04-07 00:00:00',95.12,266,783,70.98,'Intermediate' from dual union all
select 31430,'031430','1999-025BTS','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',102.22,839,889,98.76,'Sun-Synch' from dual union all
select 31431,'031431','1999-025BTT','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',103.96,828,1063,98.51,'Sun-Synch' from dual union all
select 31432,'031432','1999-025BTU','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',98.5,483,891,99.11,'Sun-Synch' from dual union all
select 31433,'031433','1999-025BTV','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',103.43,816,1026,98.39,'Sun-Synch' from dual union all
select 31434,'031434','1999-025BTW','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',101.22,765,867,99.09,'Sun-Synch' from dual union all
select 31435,'031435','1999-025BTX','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',99.6,621,858,99.05,'Sun-Synch' from dual union all
select 31436,'031436','1999-025BTY','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',102.09,741,975,98.99,'Sun-Synch' from dual union all
select 31437,'031437','1999-025BTZ','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',102.38,855,888,98.72,'Sun-Synch' from dual union all
select 31438,'031438','1999-025BUA','deb FY-1C','',66706,'Reentered','2011-11-04 00:00:00','2007-05-15 00:00:00',98.05,486,845,98.9,'Sun-Synch' from dual union all
select 31439,'031439','1999-025BUB','deb FY-1C','',66706,'In Earth orbit','','2007-05-14 00:00:00',103.83,830,1050,99.36,'Sun-Synch' from dual union all
select 3144,'003144','1968-013D','SOK','Support cone',16740,'Reentered','1968-03-05 00:00:00','1968-03-03 00:00:00',89.15,190,275,51.55,'Intermediate' from dual union all
select 31440,'031440','1999-025BUC','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',102.31,824,912,98.9,'Sun-Synch' from dual union all
select 31441,'031441','1999-025BUD','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',104.27,726,1195,99.2,'Sun-Synch' from dual union all
select 31442,'031442','1999-025BUE','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',101.03,742,873,98.97,'Sun-Synch' from dual union all
select 31443,'031443','1999-025BUF','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',101.82,634,1056,99.32,'Sun-Synch' from dual union all
select 31444,'031444','1999-025BUG','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',101.5,738,921,99.07,'Sun-Synch' from dual union all
select 31445,'031445','1999-025BUH','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',100.94,737,870,98.97,'Sun-Synch' from dual union all
select 31446,'031446','1999-025BUJ','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',102.66,817,952,98.77,'Sun-Synch' from dual union all
select 31447,'031447','1999-025BUK','deb FY-1C','',66706,'In Earth orbit','','2007-05-14 00:00:00',101.75,656,1027,98.44,'Sun-Synch' from dual union all
select 31448,'031448','1999-025BUL','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',101.76,794,889,98.86,'Sun-Synch' from dual union all
select 31449,'031449','1999-025BUM','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',100.83,685,911,98.36,'Sun-Synch' from dual union all
select 3145,'003145','1968-014B','Agena D 6503','Agena D 6503',16741,'Deep Space','1968-03-05 00:00:00','1968-09-06 00:00:00',3713.14,2101,144003,36.31,'Very High' from dual union all
select 31450,'031450','1999-025BUN','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',101,426,1185,98.62,'Sun-Synch' from dual union all
select 31451,'031451','1999-025BUP','deb FY-1C','',66706,'In Earth orbit','','2007-05-15 00:00:00',102.28,843,890,98.76,'Sun-Synch' from dual union all
select 31452,'031452','1999-025BUQ','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',102.17,831,891,99.05,'Sun-Synch' from dual union all
select 31453,'031453','1999-025BUR','deb FY-1C','',66706,'In Earth orbit','','2007-05-14 00:00:00',100.16,670,863,98.87,'Sun-Synch' from dual union all
select 31454,'031454','1999-025BUS','deb FY-1C','',66706,'Reentered','2007-05-30 00:00:00','2007-05-15 00:00:00',90.37,170,416,98.55,'Sun-Synch' from dual union all
select 31455,'031455','1999-025BUT','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',102.33,702,1035,99.2,'Sun-Synch' from dual union all
select 31456,'031456','1999-025BUU','deb FY-1C','',66706,'In Earth orbit','','2007-05-15 00:00:00',101.09,746,875,98.94,'Sun-Synch' from dual union all
select 31457,'031457','1999-025BUV','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',99.45,530,935,99.1,'Sun-Synch' from dual union all
select 31458,'031458','1999-025BUW','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',100.36,694,858,99.23,'Sun-Synch' from dual union all
select 31459,'031459','1999-025BUX','deb FY-1C','',66706,'In Earth orbit','','2007-05-15 00:00:00',102.02,751,958,98.12,'Sun-Synch' from dual union all
select 3146,'003146','1968-017B','FW4S S/N 2218-8','FW4S',16744,'Reentered','1980-12-25 00:00:00','1968-04-22 00:00:00',98.67,510,880,59.41,'Intermediate' from dual union all
select 31460,'031460','1999-025BUY','deb FY-1C','',66706,'Reentered','2014-11-07 00:00:00','2007-05-16 00:00:00',102.57,588,1172,98.36,'Sun-Synch' from dual union all
select 31461,'031461','1999-025BUZ','deb FY-1C','',66706,'Reentered','2014-04-24 00:00:00','2007-05-16 00:00:00',100.61,606,968,97.71,'Sun-Synch' from dual union all
select 31462,'031462','1999-025BVA','deb FY-1C','',66706,'In Earth orbit','','2007-05-15 00:00:00',102.28,843,890,98.83,'Sun-Synch' from dual union all
select 31463,'031463','1999-025BVB','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',101.05,632,985,99.13,'Sun-Synch' from dual union all
select 31464,'031464','1999-025BVC','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',102.31,835,901,98.73,'Sun-Synch' from dual union all
select 31465,'031465','1999-025BVD','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',103.63,828,1033,98.57,'Sun-Synch' from dual union all
select 31466,'031466','1999-025BVE','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',101.79,617,1069,99.26,'Sun-Synch' from dual union all
select 31467,'031467','1999-025BVF','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',104.62,838,1116,98.59,'Sun-Synch' from dual union all
select 31468,'031468','1999-025BVG','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',100.46,639,922,98.76,'Sun-Synch' from dual union all
select 31469,'031469','1999-025BVH','deb FY-1C','',66706,'Reentered','2011-03-02 00:00:00','2007-05-17 00:00:00',97.11,408,833,98.82,'Sun-Synch' from dual union all
select 3147,'003147','1968-011C','deb Kosmos-203','',16738,'Reentered','1982-05-27 00:00:00','1968-03-25 00:00:00',104.46,748,1191,74.04,'Intermediate' from dual union all
select 31470,'031470','1999-025BVJ','deb FY-1C','',66706,'Reentered','2012-07-06 00:00:00','2007-05-14 00:00:00',105.82,804,1262,100.11,'Sun-Synch' from dual union all
select 31471,'031471','1999-025BVK','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',103.81,791,1087,97.97,'Sun-Synch' from dual union all
select 31472,'031472','1999-025BVL','deb FY-1C','',66706,'Reentered','2014-10-30 00:00:00','2007-05-16 00:00:00',97.79,420,886,99.62,'Sun-Synch' from dual union all
select 31473,'031473','1999-025BVM','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',102.26,804,928,99,'Sun-Synch' from dual union all
select 31474,'031474','1999-025BVN','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',103.37,859,976,98.92,'Sun-Synch' from dual union all
select 31475,'031475','1999-025BVP','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',101.44,638,1016,99.08,'Sun-Synch' from dual union all
select 31476,'031476','1999-025BVQ','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',102.75,838,940,98.87,'Sun-Synch' from dual union all
select 31477,'031477','1999-025BVR','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',102.84,804,982,99.1,'Sun-Synch' from dual union all
select 31478,'031478','1999-025BVS','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',101.88,799,897,102.61,'Retrograde' from dual union all
select 31479,'031479','1999-025BVT','deb FY-1C','',66706,'Reentered','2008-01-04 00:00:00','2007-05-17 00:00:00',95.45,347,734,99.04,'Sun-Synch' from dual union all
select 3148,'003148','1968-018A','OPS 5057','GAMBIT-3 Mission 4312',16745,'Reentered Attached','1968-03-24 00:00:00','1968-03-15 00:00:00',89.82,129,403,99.87,'Retrograde' from dual union all
select 31480,'031480','1999-025BVU','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',106.58,717,1420,98.37,'Sun-Synch' from dual union all
select 31481,'031481','1999-025BVV','deb FY-1C','',66706,'In Earth orbit','','2007-05-16 00:00:00',100.83,726,869,99.28,'Sun-Synch' from dual union all
select 31482,'031482','1999-025BVW','deb FY-1C','',66706,'In Earth orbit','','2007-05-15 00:00:00',101.27,743,894,98.82,'Sun-Synch' from dual union all
select 31483,'031483','1999-025BVX','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',104.26,827,1092,102.52,'Sun-Synch' from dual union all
select 31484,'031484','1999-025BVY','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',103.99,800,1095,101.85,'Sun-Synch' from dual union all
select 31485,'031485','1999-025BVZ','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',109.38,797,1600,102.74,'Sun-Synch' from dual union all
select 31486,'031486','1999-025BWA','deb FY-1C','',66706,'In Earth orbit','','2007-05-14 00:00:00',102.36,850,891,102.28,'Sun-Synch' from dual union all
select 31487,'031487','1999-025BWB','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',104.2,855,1059,102.42,'Sun-Synch' from dual union all
select 31488,'031488','1999-025BWC','deb FY-1C','',66706,'In Earth orbit','','2007-05-17 00:00:00',102.18,787,937,102.67,'Retrograde' from dual union all
select 31489,'031489','1999-025BWD','deb FY-1C','',66706,'Reentered','2007-05-23 00:00:00','2007-05-16 00:00:00',90.83,264,367,99.66,'Retrograde' from dual union all
select 3149,'003149','1965-082LP','deb Transtage 4','',11832,'Reentered','1970-12-03 00:00:00','1967-03-11 00:00:00',105.14,750,1251,32.08,'Intermediate' from dual union all
select 31490,'031490','2007-019A','Yaogan Weixing 2','Jianbing-6 01 xing',68361,'In Earth orbit','','2007-06-01 00:00:00',97.58,631,655,97.85,'Sun-Synch' from dual union all
select 31491,'031491','1999-025BWE','deb FY-1C','',66706,'In Earth orbit','','2007-05-25 00:00:00',97.57,632,653,97.84,'Sun-Synch' from dual union all
select 31492,'031492','1999-025BWF','deb FY-1C','',66706,'In Earth orbit','','2007-05-25 00:00:00',97.58,630,656,97.83,'Sun-Synch' from dual union all
select 31493,'031493','1999-025BWG','deb FY-1C','',66706,'In Earth orbit','','2007-05-26 00:00:00',93.49,434,456,97.22,'Sun-Synch' from dual union all
select 31494,'031494','1999-025BWH','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',101.61,805,865,99.63,'Sun-Synch' from dual union all
select 31495,'031495','1999-025BWJ','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.3,843,891,99.2,'Sun-Synch' from dual union all
select 31496,'031496','1999-025BWK','deb FY-1C','',66706,'In Earth orbit','','2007-05-29 00:00:00',98.66,515,874,99.2,'Sun-Synch' from dual union all
select 31497,'031497','1999-025BWL','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',100.43,649,909,98.67,'Sun-Synch' from dual union all
select 31498,'031498','1999-025BWM','deb FY-1C','',66706,'In Earth orbit','','2007-05-29 00:00:00',105.82,846,1220,98.5,'Sun-Synch' from dual union all
select 31499,'031499','1999-025BWN','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',105.85,756,1313,97.66,'Sun-Synch' from dual union all
select 315,'000315','62 A Beta','FTV 1129','CORONA M-6',8137,'Reentered Attached','1962-07-07 00:00:00','1962-06-29 00:00:00',89.25,208,267,75.09,'Intermediate' from dual union all
select 3150,'003150','1968-019A','Kosmos-206','Meteor 11F614 No. 9',16746,'Reentered','1989-04-22 00:00:00','1968-04-20 00:00:00',97.02,599,632,81.22,'Intermediate' from dual union all
select 31500,'031500','1999-025BWP','deb FY-1C','',66706,'In Earth orbit','','2007-05-29 00:00:00',102.66,448,1320,98.99,'Sun-Synch' from dual union all
select 31501,'031501','1999-025BWQ','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.42,837,910,98.84,'Sun-Synch' from dual union all
select 31502,'031502','1999-025BWR','deb FY-1C','',66706,'In Earth orbit','','2007-05-27 00:00:00',103.43,856,986,99.01,'Sun-Synch' from dual union all
select 31503,'031503','1999-025BWS','deb FY-1C','',66706,'In Earth orbit','','2007-05-24 00:00:00',102.44,831,918,100.14,'Sun-Synch' from dual union all
select 31504,'031504','1999-025BWT','deb FY-1C','',66706,'In Earth orbit','','2007-05-29 00:00:00',102.27,703,1029,101.55,'Sun-Synch' from dual union all
select 31505,'031505','1999-025BWU','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.5,830,924,101.92,'Sun-Synch' from dual union all
select 31506,'031506','1999-025BWV','deb FY-1C','',66706,'Reentered','2007-10-02 00:00:00','2007-05-28 00:00:00',95.13,266,783,99.5,'Sun-Synch' from dual union all
select 31507,'031507','1999-025BWW','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',100.17,664,870,98.39,'Sun-Synch' from dual;
insert into SATELLITE
select 31508,'031508','1999-025BWX','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',101.76,796,887,98.76,'Sun-Synch' from dual union all
select 31509,'031509','1999-025BWY','deb FY-1C','',66706,'In Earth orbit','','2007-05-29 00:00:00',104.59,800,1150,98.94,'Sun-Synch' from dual union all
select 3151,'003151','1968-019B','8A92ME','8A92ME',16746,'Reentered','1992-11-05 00:00:00','1968-04-22 00:00:00',97.22,539,713,81.22,'Intermediate' from dual union all
select 31510,'031510','1999-025BWZ','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',101.67,808,868,98.84,'Sun-Synch' from dual union all
select 31511,'031511','1999-025BXA','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.51,773,981,99.2,'Sun-Synch' from dual union all
select 31512,'031512','1999-025BXB','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',100.76,670,920,98.97,'Sun-Synch' from dual union all
select 31513,'031513','1999-025BXC','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.64,856,912,98.96,'Sun-Synch' from dual union all
select 31514,'031514','1999-025BXD','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.99,856,945,99.37,'Sun-Synch' from dual union all
select 31515,'031515','1999-025BXE','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.93,849,946,99.33,'Sun-Synch' from dual union all
select 31516,'031516','1999-025BXF','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',101.59,717,950,99.09,'Sun-Synch' from dual union all
select 31517,'031517','1999-025BXG','deb FY-1C','',66706,'Reentered','2017-07-05 00:00:00','2007-05-28 00:00:00',101.71,809,871,99.16,'Sun-Synch' from dual union all
select 31518,'031518','1999-025BXH','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',103.87,856,1026,98.92,'Sun-Synch' from dual union all
select 31519,'031519','1999-025BXJ','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',101.7,799,880,98.85,'Sun-Synch' from dual union all
select 3152,'003152','1968-020A','OPS 4849','CORONA J-48',16747,'Reentered Attached','1968-04-10 00:00:00','1968-03-21 00:00:00',90.35,180,403,83.01,'Intermediate' from dual union all
select 31520,'031520','1999-025BXK','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.07,855,858,98.78,'Sun-Synch' from dual union all
select 31521,'031521','1999-025BXL','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',101.48,759,899,98.77,'Sun-Synch' from dual union all
select 31522,'031522','1999-025BXM','deb FY-1C','',66706,'In Earth orbit','','2007-05-29 00:00:00',103.06,857,950,99.31,'Sun-Synch' from dual union all
select 31523,'031523','1999-025BXN','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',101.86,820,873,98.6,'Sun-Synch' from dual union all
select 31524,'031524','1999-025BXP','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.85,678,1109,99.4,'Sun-Synch' from dual union all
select 31525,'031525','1999-025BXQ','deb FY-1C','',66706,'Reentered','2012-03-21 00:00:00','2007-05-28 00:00:00',99.2,512,928,97.95,'Sun-Synch' from dual union all
select 31526,'031526','1999-025BXR','deb FY-1C','',66706,'In Earth orbit','','2007-05-26 00:00:00',101.28,776,863,99.16,'Sun-Synch' from dual union all
select 31527,'031527','1999-025BXS','deb FY-1C','',66706,'In Earth orbit','','2007-05-25 00:00:00',109.1,836,1535,98.92,'Sun-Synch' from dual union all
select 31528,'031528','1999-025BXT','deb FY-1C','',66706,'Reentered','2011-10-18 00:00:00','2007-05-26 00:00:00',97.85,369,942,99.42,'Sun-Synch' from dual union all
select 31529,'031529','1999-025BXU','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.28,826,908,98.79,'Sun-Synch' from dual union all
select 3153,'003153','1968-020B','LAMPAN I','P-11 No. 4411',16747,'Reentered','1970-01-03 00:00:00','1968-04-15 00:00:00',94.53,472,520,83.1,'Intermediate' from dual union all
select 31530,'031530','1999-025BXV','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',101.42,717,935,99.41,'Sun-Synch' from dual union all
select 31531,'031531','1999-025BXW','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',101.8,619,1068,99.03,'Sun-Synch' from dual union all
select 31532,'031532','1999-025BXX','deb FY-1C','',66706,'In Earth orbit','','2007-05-27 00:00:00',103.23,780,1043,99.21,'Sun-Synch' from dual union all
select 31533,'031533','1999-025BXY','deb FY-1C','',66706,'In Earth orbit','','2007-05-26 00:00:00',101.35,770,876,99.11,'Sun-Synch' from dual union all
select 31534,'031534','1999-025BXZ','deb FY-1C','',66706,'In Earth orbit','','2007-05-27 00:00:00',102.12,839,879,99.05,'Sun-Synch' from dual union all
select 31535,'031535','1999-025BYA','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',101.79,769,918,98.06,'Sun-Synch' from dual union all
select 31536,'031536','1999-025BYB','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.99,530,1271,98.16,'Sun-Synch' from dual union all
select 31537,'031537','1999-025BYC','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',106.42,612,1510,99.07,'Sun-Synch' from dual union all
select 31538,'031538','1999-025BYD','deb FY-1C','',66706,'In Earth orbit','','2007-05-26 00:00:00',101.73,782,900,98.41,'Sun-Synch' from dual union all
select 31539,'031539','1999-025BYE','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',101.15,763,864,99.1,'Sun-Synch' from dual union all
select 3154,'003154','1968-021A','Kosmos-207','Zenit-4 No. 38',16748,'Landed','1968-03-24 00:00:00','1968-03-20 00:00:00',89.63,200,312,65.64,'Intermediate' from dual union all
select 31540,'031540','1999-025BYF','deb FY-1C','',66706,'In Earth orbit','','2007-05-26 00:00:00',100.36,587,964,99.11,'Sun-Synch' from dual union all
select 31541,'031541','1999-025BYG','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',100.12,671,857,99.04,'Sun-Synch' from dual union all
select 31542,'031542','1999-025BYH','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.61,841,924,98.96,'Sun-Synch' from dual union all
select 31543,'031543','1999-025BYJ','deb FY-1C','',66706,'Reentered','2014-04-17 00:00:00','2007-05-28 00:00:00',102.91,829,964,98.77,'Sun-Synch' from dual union all
select 31544,'031544','1999-025BYK','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.61,857,907,98.84,'Sun-Synch' from dual union all
select 31545,'031545','1999-025BYL','deb FY-1C','',66706,'In Earth orbit','','2007-05-29 00:00:00',100.7,719,865,99.06,'Sun-Synch' from dual union all
select 31546,'031546','1999-025BYM','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',103.77,792,1082,98.22,'Sun-Synch' from dual union all
select 31547,'031547','1999-025BYN','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.85,850,936,98.76,'Sun-Synch' from dual union all
select 31548,'031548','1999-025BYP','deb FY-1C','',66706,'In Earth orbit','','2007-05-29 00:00:00',101.27,709,928,99.41,'Sun-Synch' from dual union all
select 31549,'031549','1999-025BYQ','deb FY-1C','',66706,'Reentered','2017-06-24 00:00:00','2007-05-27 00:00:00',101.71,809,871,99.13,'Sun-Synch' from dual union all
select 3155,'003155','1968-021B','11A57I','11A57I',16748,'Reentered','1968-03-22 00:00:00','1968-03-18 00:00:00',89.37,192,295,65.64,'Intermediate' from dual union all
select 31550,'031550','1999-025BYR','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',101.1,644,978,98.83,'Sun-Synch' from dual union all
select 31551,'031551','1999-025BYS','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',103.69,852,1014,98.85,'Sun-Synch' from dual union all
select 31552,'031552','1999-025BYT','deb FY-1C','',66706,'In Earth orbit','','2007-05-29 00:00:00',103.08,783,1026,98.57,'Sun-Synch' from dual union all
select 31553,'031553','1999-025BYU','deb FY-1C','',66706,'In Earth orbit','','2007-05-29 00:00:00',102.72,856,919,98.76,'Sun-Synch' from dual union all
select 31554,'031554','1999-025BYV','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.54,829,929,98.91,'Sun-Synch' from dual union all
select 31555,'031555','1999-025BYW','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.1,687,1029,99.23,'Sun-Synch' from dual union all
select 31556,'031556','1999-025BYX','deb FY-1C','',66706,'Reentered','2015-02-09 00:00:00','2007-05-28 00:00:00',99.03,434,991,96.69,'Sun-Synch' from dual union all
select 31557,'031557','1999-025BYY','deb FY-1C','',66706,'In Earth orbit','','2007-05-29 00:00:00',100.56,649,922,98.77,'Sun-Synch' from dual union all
select 31558,'031558','1999-025BYZ','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',100.61,621,954,98.96,'Sun-Synch' from dual union all
select 31559,'031559','1999-025BZA','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',100.89,729,872,98.99,'Sun-Synch' from dual union all
select 3156,'003156','1968-022A','Kosmos-208','Zenit-2M',16749,'Landed','1968-04-02 00:00:00','1968-03-23 00:00:00',89.3,210,270,64.95,'Intermediate' from dual union all
select 31560,'031560','1999-025BZB','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',101.4,736,913,98.55,'Sun-Synch' from dual union all
select 31561,'031561','1999-025BZC','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',103.04,857,948,99.3,'Sun-Synch' from dual union all
select 31562,'031562','1999-025BZD','deb FY-1C','',66706,'In Earth orbit','','2007-05-27 00:00:00',101.14,758,868,99.31,'Sun-Synch' from dual union all
select 31563,'031563','1999-025BZE','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',99.7,632,857,99.01,'Sun-Synch' from dual union all
select 31564,'031564','1999-025BZF','deb FY-1C','',66706,'In Earth orbit','','2007-05-29 00:00:00',102.02,844,864,98.74,'Sun-Synch' from dual union all
select 31565,'031565','1999-025BZG','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',103.58,856,1000,100.73,'Sun-Synch' from dual union all
select 31566,'031566','1999-025BZH','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',102.6,855,909,98.74,'Sun-Synch' from dual union all
select 31567,'031567','1999-025BZJ','deb FY-1C','',66706,'In Earth orbit','','2007-05-25 00:00:00',105.7,852,1203,102.33,'Sun-Synch' from dual union all
select 31568,'031568','1999-025BZK','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',104.64,851,1105,102.09,'Sun-Synch' from dual union all
select 31569,'031569','1999-025BZL','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',105.21,843,1166,102.61,'Sun-Synch' from dual union all
select 3157,'003157','1968-022B','11A57I','11A57I',16749,'Reentered','1968-03-25 00:00:00','1968-03-23 00:00:00',88.76,185,242,64.95,'Intermediate' from dual union all
select 31570,'031570','1999-025BZM','deb FY-1C','',66706,'In Earth orbit','','2007-05-28 00:00:00',104.35,800,1127,102.6,'Sun-Synch' from dual union all
select 31571,'031571','2007-020A','Globalstar M065','Globalstar',68362,'In Earth orbit','','2007-06-06 00:00:00',103.46,914,931,52,'Intermediate' from dual union all
select 31572,'031572','2007-020B','deb Fregat','',68362,'Reentered','2013-12-17 00:00:00','2007-06-06 00:00:00',103.42,909,932,52.01,'Intermediate' from dual union all
select 31573,'031573','2007-020C','Globalstar M069','Globalstar',68362,'In Earth orbit','','2007-06-06 00:00:00',103.44,912,931,52,'Intermediate' from dual union all
select 31574,'031574','2007-020D','Globalstar M072','Globalstar',68362,'In Earth orbit','','2007-06-06 00:00:00',103.49,917,931,52,'Intermediate' from dual union all
select 31575,'031575','2007-020E','11S510','11S510',68362,'Reentered','2007-06-03 00:00:00','2007-05-31 00:00:00',88.51,186,216,51.91,'Intermediate' from dual union all
select 31576,'031576','2007-020F','Globalstar M071','Globalstar',68362,'In Earth orbit','','2007-06-05 00:00:00',103.48,916,931,52,'Intermediate' from dual union all
select 31577,'031577','2007-021A','Xinnuo 3','xinnuo sanhao',68363,'In Earth orbit','','2007-06-15 00:00:00',1436.06,35779,35793,.27,'Stationary' from dual union all
select 31578,'031578','2007-021B','CZ-3A Stage 3','CZ-3A Stage 3',68363,'Reentered','2008-05-01 00:00:00','2007-06-08 00:00:00',732.49,132,40946,25.35,'GEO Transfer' from dual union all
select 31579,'031579','1966-040AR','Nimbus 2 debris','',13447,'In Earth orbit','','2008-01-15 00:00:00',107.95,1088,1176,100.55,'Sun-Synch' from dual union all
select 3158,'003158','1968-023A','Kosmos-209','US-A reactor',16750,'In Earth orbit','','1968-04-21 00:00:00',103.13,869,944,65.34,'Intermediate' from dual union all
select 31580,'031580','1994-074J','deb Resurs-O1','',65470,'Reentered','2011-12-23 00:00:00','2007-06-05 00:00:00',97.39,626,642,97.93,'Sun-Synch' from dual union all
select 31581,'031581','1999-057PC','ZY-1 debris','',66738,'In Earth orbit','','2007-06-05 00:00:00',100.12,757,772,98.22,'Sun-Synch' from dual union all
select 31582,'031582','1999-057PD','ZY-1 debris','',66738,'Reentered','2017-11-30 00:00:00','2007-06-05 00:00:00',100.4,769,786,98.23,'Sun-Synch' from dual union all
select 31583,'031583','1999-057PE','ZY-1 debris','',66738,'In Earth orbit','','2007-06-04 00:00:00',100.44,777,782,98.19,'Sun-Synch' from dual union all
select 31584,'031584','1999-057PF','ZY-1 debris','',66738,'Reentered','2007-10-29 00:00:00','2007-06-05 00:00:00',96.57,547,642,98.32,'Sun-Synch' from dual union all
select 31585,'031585','1999-057PG','ZY-1 debris','',66738,'Reentered','2010-07-23 00:00:00','2007-06-04 00:00:00',101.51,713,948,98.47,'Sun-Synch' from dual union all
select 31586,'031586','1999-057PH','ZY-1 debris','',66738,'Reentered','2015-09-29 00:00:00','2007-06-05 00:00:00',100.31,769,778,98.21,'Sun-Synch' from dual union all
select 31587,'031587','1999-057PJ','ZY-1 debris','',66738,'Reentered','2017-08-06 00:00:00','2007-06-04 00:00:00',100.09,754,772,98.2,'Sun-Synch' from dual union all
select 31588,'031588','1999-057PK','ZY-1 debris','',66738,'Reentered','2014-04-13 00:00:00','2007-06-05 00:00:00',100.3,767,779,98.22,'Sun-Synch' from dual union all
select 31589,'031589','1999-057PL','ZY-1 debris','',66738,'In Earth orbit','','2007-06-02 00:00:00',100.7,767,816,98.22,'Sun-Synch' from dual union all
select 3159,'003159','1963-014AX','deb West Ford','',9245,'Reentered','1968-02-18 00:00:00','1968-02-18 00:00:00',93.24,279,587,83.73,'Intermediate' from dual union all
select 31590,'031590','1999-057PM','ZY-1 debris','',66738,'Reentered','2014-03-15 00:00:00','2007-06-05 00:00:00',100.28,762,782,98.23,'Sun-Synch' from dual union all
select 31591,'031591','1999-057PN','ZY-1 debris','',66738,'In Earth orbit','','2007-06-04 00:00:00',100.28,763,780,98.21,'Sun-Synch' from dual union all
select 31592,'031592','1999-057PP','ZY-1 debris','',66738,'Reentered','2016-10-14 00:00:00','2007-06-05 00:00:00',100.04,748,772,98.21,'Sun-Synch' from dual union all
select 31593,'031593','1999-057PQ','ZY-1 debris','',66738,'Reentered','2014-12-11 00:00:00','2007-06-05 00:00:00',100.12,749,780,98.22,'Sun-Synch' from dual union all
select 31594,'031594','1999-057PR','ZY-1 debris','',66738,'In Earth orbit','','2007-06-05 00:00:00',100.38,771,783,98.21,'Sun-Synch' from dual union all
select 31595,'031595','2007-022A','Kosmos-2427','Kobal''t-M No. 558',68364,'Landed','2007-08-22 00:00:00','2007-06-11 00:00:00',89.87,182,354,67.14,'Intermediate' from dual union all
select 31596,'031596','2007-022B','11S510','11S510',68364,'Reentered','2007-06-13 00:00:00','2007-06-09 00:00:00',89.13,159,304,67.13,'Intermediate' from dual union all
select 31597,'031597','2007-022C','deb Kosmos-2427','',68364,'Reentered','2007-06-09 00:00:00','2007-06-08 00:00:00',89,154,296,67.14,'Intermediate' from dual union all
select 31598,'031598','2007-023A','COSMO 1','COSMO-Skymed',68365,'In Earth orbit','','2007-06-10 00:00:00',97.16,621,625,97.86,'Sun-Synch' from dual;
insert into SATELLITE
select 31599,'031599','2007-023B','Delta 324','Delta SSPS AJ-10-118K',68365,'Reentered','2007-08-16 00:00:00','2007-06-10 00:00:00',92.68,192,620,98.39,'Sun-Synch' from dual union all
select 316,'000316','62 A Gamma','FTV 1151','CORONA M-7',8142,'Reentered Attached','1962-09-14 00:00:00','1962-07-30 00:00:00',92.31,193,583,76.08,'Intermediate' from dual union all
select 3160,'003160','1968-023D','deb Kosmos-209','',16750,'Reentered','1968-03-29 00:00:00','1968-03-23 00:00:00',89.91,233,307,65.04,'Intermediate' from dual union all
select 31600,'031600','2007-024A','Atlantis  (STS-117)','OV-104',68366,'Landed','2007-06-22 00:00:00','2007-06-11 00:00:00',91.22,329,340,51.63,'Intermediate' from dual union all
select 31601,'031601','2007-025A','''Ofeq-7','''Ofeq-7',68367,'In Earth orbit','','2007-06-13 00:00:00',93.74,340,574,141.76,'Retrograde' from dual union all
select 31602,'031602','2007-025B','AUS-51','AUS-51',68367,'Reentered','2008-07-10 00:00:00','2007-06-13 00:00:00',93.33,298,577,141.76,'Retrograde' from dual union all
select 31603,'031603','1998-067AU','STS-117 EVA-1 debris','ISS S3/S4 EVA debris',66503,'Reentered','2007-06-20 00:00:00','2007-06-12 00:00:00',91.1,326,331,51.63,'Intermediate' from dual union all
select 31604,'031604','1998-067AV','STS-117 EVA-1 debris','ISS S3/S4 EVA debris',66503,'Reentered','2007-06-20 00:00:00','2007-06-12 00:00:00',91.07,323,332,51.63,'Intermediate' from dual union all
select 31605,'031605','1998-067AW','STS-117 EVA-1 debris','ISS S3/S4 EVA debris',66503,'Reentered','2007-06-20 00:00:00','2007-06-12 00:00:00',91.06,324,330,51.64,'Intermediate' from dual union all
select 31606,'031606','1998-067AX','STS-117 EVA-1 debris','ISS S3/S4 EVA debris',66503,'Reentered','2007-06-22 00:00:00','2007-06-12 00:00:00',91.15,328,333,51.63,'Intermediate' from dual union all
select 31607,'031607','1998-067AY','STS-117 EVA-1 debris','ISS S3/S4 EVA debris',66503,'Reentered','2007-06-23 00:00:00','2007-06-12 00:00:00',91.15,327,335,51.63,'Intermediate' from dual union all
select 31608,'031608','1999-025BZN','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',99.92,560,950,98.14,'Sun-Synch' from dual union all
select 31609,'031609','1999-025BZP','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',103.79,852,1024,99,'Sun-Synch' from dual union all
select 3161,'003161','1968-023B','Kosmos-209 RLS','US-A RLS antenna',16750,'Reentered','1968-03-25 00:00:00','1968-03-24 00:00:00',89.24,228,246,65.09,'Intermediate' from dual union all
select 31610,'031610','1999-025BZQ','deb FY-1C','',66706,'Reentered','2013-11-25 00:00:00','2007-06-13 00:00:00',99.21,593,848,98.9,'Sun-Synch' from dual union all
select 31611,'031611','1999-025BZR','deb FY-1C','',66706,'In Earth orbit','','2007-06-10 00:00:00',102.15,749,972,99.01,'Sun-Synch' from dual union all
select 31612,'031612','1999-025BZS','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',98.6,521,863,100.41,'Sun-Synch' from dual union all
select 31613,'031613','1999-025BZT','deb FY-1C','',66706,'In Earth orbit','','2007-06-11 00:00:00',106.72,788,1361,98.46,'Sun-Synch' from dual union all
select 31614,'031614','1999-025BZU','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.24,803,927,99.05,'Sun-Synch' from dual union all
select 31615,'031615','1999-025BZV','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',104.26,747,1173,99.08,'Sun-Synch' from dual union all
select 31616,'031616','1999-025BZW','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.03,747,963,96.6,'Sun-Synch' from dual union all
select 31617,'031617','1999-025BZX','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.34,697,948,96.73,'Sun-Synch' from dual union all
select 31618,'031618','1999-025BZY','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.83,723,968,99.5,'Sun-Synch' from dual union all
select 31619,'031619','1999-025BZZ','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',103.41,694,1145,99.09,'Sun-Synch' from dual union all
select 3162,'003162','1968-023C','Kosmos-209 DU','US-A',16750,'Reentered','1968-04-10 00:00:00','1968-03-29 00:00:00',89.23,224,249,65.11,'Intermediate' from dual union all
select 31620,'031620','1999-025CAA','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.64,855,913,98.82,'Sun-Synch' from dual union all
select 31621,'031621','1999-025CAB','deb FY-1C','',66706,'Reentered','2016-04-06 00:00:00','2007-06-13 00:00:00',99.73,638,854,99.52,'Sun-Synch' from dual union all
select 31622,'031622','1999-025CAC','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.41,853,893,98.83,'Sun-Synch' from dual union all
select 31623,'031623','1999-025CAD','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',104.46,820,1118,97.65,'Sun-Synch' from dual union all
select 31624,'031624','1999-025CAE','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.15,766,861,99.71,'Sun-Synch' from dual union all
select 31625,'031625','1999-025CAF','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',99.45,614,851,98.92,'Sun-Synch' from dual union all
select 31626,'031626','1999-025CAG','deb FY-1C','',66706,'In Earth orbit','','2007-06-10 00:00:00',102.16,834,887,101.33,'Sun-Synch' from dual union all
select 31627,'031627','1999-025CAH','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.81,796,893,98.82,'Sun-Synch' from dual union all
select 31628,'031628','1999-025CAJ','deb FY-1C','',66706,'In Earth orbit','','2007-06-11 00:00:00',105.37,853,1171,102.29,'Sun-Synch' from dual union all
select 31629,'031629','1999-025CAK','deb FY-1C','',66706,'In Earth orbit','','2007-06-12 00:00:00',104.28,844,1078,102.19,'Sun-Synch' from dual union all
select 3163,'003163','1965-020EP','deb Blok Kosmos','',11770,'Reentered','1968-07-16 00:00:00','1968-03-22 00:00:00',95.72,330,777,56.2,'Intermediate' from dual union all
select 31630,'031630','1999-025CAL','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',100.22,487,1051,98,'Sun-Synch' from dual union all
select 31631,'031631','1999-025CAM','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',104.2,793,1121,99.38,'Sun-Synch' from dual union all
select 31632,'031632','1999-025CAN','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.6,682,987,99.09,'Sun-Synch' from dual union all
select 31633,'031633','1999-025CAP','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',103.08,856,953,99.29,'Sun-Synch' from dual union all
select 31634,'031634','1999-025CAQ','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',103.55,857,995,98.86,'Sun-Synch' from dual union all
select 31635,'031635','1999-025CAR','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.63,792,879,99.47,'Sun-Synch' from dual union all
select 31636,'031636','1999-025CAS','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',97.94,460,861,98.92,'Sun-Synch' from dual union all
select 31637,'031637','1999-025CAT','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.4,818,927,99.15,'Sun-Synch' from dual union all
select 31638,'031638','1999-025CAU','deb FY-1C','',66706,'Reentered','2011-06-02 00:00:00','2007-06-13 00:00:00',97.54,443,839,99.63,'Sun-Synch' from dual union all
select 31639,'031639','1999-025CAV','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',104.07,831,1071,98.67,'Sun-Synch' from dual union all
select 3164,'003164','61 Omicron 216','deb r/b Ablestar','',7273,'In Earth orbit','','1992-01-30 00:00:00',103.82,901,977,66.75,'Intermediate' from dual union all
select 31640,'031640','1999-025CAW','deb FY-1C','',66706,'In Earth orbit','','2007-06-10 00:00:00',99.33,493,961,99.28,'Sun-Synch' from dual union all
select 31641,'031641','1999-025CAX','deb FY-1C','',66706,'Reentered','2012-02-29 00:00:00','2007-06-13 00:00:00',99,394,1028,98.65,'Sun-Synch' from dual union all
select 31642,'031642','1999-025CAY','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',100.98,711,900,98.97,'Sun-Synch' from dual union all
select 31643,'031643','1999-025CAZ','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',103.59,849,1008,98.37,'Sun-Synch' from dual union all
select 31644,'031644','1999-025CBA','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',100.37,603,950,98.83,'Sun-Synch' from dual union all
select 31645,'031645','1999-025CBB','deb FY-1C','',66706,'Reentered','2014-11-13 00:00:00','2007-06-13 00:00:00',98.55,483,895,98.77,'Sun-Synch' from dual union all
select 31646,'031646','1999-025CBC','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.38,845,898,98.86,'Sun-Synch' from dual union all
select 31647,'031647','1999-025CBD','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',103.62,778,1082,98.07,'Sun-Synch' from dual union all
select 31648,'031648','1999-025CBE','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',104.2,825,1088,102.83,'Sun-Synch' from dual union all
select 31649,'031649','1999-025CBF','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',106.39,837,1281,98.19,'Sun-Synch' from dual union all
select 3165,'003165','1968-023E','Kosmos-209 reactor part','',16750,'Reentered','1968-05-04 00:00:00','1968-03-30 00:00:00',100.57,712,860,64.86,'Intermediate' from dual union all
select 31650,'031650','1999-025CBG','deb FY-1C','',66706,'In Earth orbit','','2007-06-12 00:00:00',104.59,666,1285,95.23,'Retrograde' from dual union all
select 31651,'031651','1999-025CBH','deb FY-1C','',66706,'Reentered','2012-04-11 00:00:00','2007-06-13 00:00:00',102.8,730,1053,95.44,'Sun-Synch' from dual union all
select 31652,'031652','1999-025CBJ','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.41,722,1024,95.97,'Sun-Synch' from dual union all
select 31653,'031653','1999-025CBK','deb FY-1C','',66706,'In Earth orbit','','2007-06-12 00:00:00',102.14,747,973,96.68,'Sun-Synch' from dual union all
select 31654,'031654','1999-025CBL','deb FY-1C','',66706,'Reentered','2016-12-01 00:00:00','2007-06-13 00:00:00',102.53,602,1155,97.61,'Sun-Synch' from dual union all
select 31655,'031655','1999-025CBM','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',107.75,850,1396,99.3,'Sun-Synch' from dual union all
select 31656,'031656','1999-025CBN','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.09,829,887,99.25,'Sun-Synch' from dual union all
select 31657,'031657','1999-025CBP','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',105.16,855,1149,102.24,'Sun-Synch' from dual union all
select 31658,'031658','1999-025CBQ','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',105.32,793,1225,98.17,'Sun-Synch' from dual union all
select 31659,'031659','1999-025CBR','deb FY-1C','',66706,'In Earth orbit','','2007-06-12 00:00:00',99.84,584,918,99.09,'Sun-Synch' from dual union all
select 3166,'003166','1963-038H','deb Transit VBN-1','',9269,'In Earth orbit','','1968-03-17 00:00:00',107.32,1070,1136,89.9,'Polar' from dual union all
select 31660,'031660','1999-025CBS','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.7,755,924,98.89,'Sun-Synch' from dual union all
select 31661,'031661','1999-025CBT','deb FY-1C','',66706,'In Earth orbit','','2007-06-11 00:00:00',102.67,828,942,98.87,'Sun-Synch' from dual union all
select 31662,'031662','1999-025CBU','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.6,846,918,98.84,'Sun-Synch' from dual union all
select 31663,'031663','1999-025CBV','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',98.7,526,867,98.92,'Sun-Synch' from dual union all
select 31664,'031664','1999-025CBW','deb FY-1C','',66706,'In Earth orbit','','2007-06-11 00:00:00',102.52,780,976,99.05,'Sun-Synch' from dual union all
select 31665,'031665','1999-025CBX','deb FY-1C','',66706,'In Earth orbit','','2007-06-10 00:00:00',104.99,837,1151,98.46,'Sun-Synch' from dual union all
select 31666,'031666','1999-025CBY','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.41,780,870,98.83,'Sun-Synch' from dual union all
select 31667,'031667','1999-025CBZ','deb FY-1C','',66706,'In Earth orbit','','2007-06-12 00:00:00',101.53,792,871,99.14,'Sun-Synch' from dual union all
select 31668,'031668','1999-025CCA','deb FY-1C','',66706,'In Earth orbit','','2007-06-12 00:00:00',101.8,733,955,96.57,'Sun-Synch' from dual union all
select 31669,'031669','1999-025CCB','deb FY-1C','',66706,'Reentered','2011-10-24 00:00:00','2007-06-12 00:00:00',98.99,489,931,97.48,'Sun-Synch' from dual union all
select 3167,'003167','1968-022C','Kapsula Kosmosa-208','Nauka 1KS No. 1',16749,'Reentered','1968-04-06 00:00:00','1968-03-30 00:00:00',89.08,195,262,64.95,'Intermediate' from dual union all
select 31670,'031670','1999-025CCC','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',106.29,844,1265,98.44,'Sun-Synch' from dual union all
select 31671,'031671','1999-025CCD','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.93,821,974,99.16,'Sun-Synch' from dual union all
select 31672,'031672','1999-025CCE','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.89,718,979,99.06,'Sun-Synch' from dual union all
select 31673,'031673','1999-025CCF','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.02,708,906,99.39,'Sun-Synch' from dual union all
select 31674,'031674','1999-025CCG','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.68,851,920,98.73,'Sun-Synch' from dual union all
select 31675,'031675','1999-025CCH','deb FY-1C','',66706,'In Earth orbit','','2007-06-12 00:00:00',103.46,807,1038,98.46,'Sun-Synch' from dual union all
select 31676,'031676','1999-025CCJ','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',108.54,839,1479,98.59,'Sun-Synch' from dual union all
select 31677,'031677','1999-025CCK','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.98,833,872,99.06,'Sun-Synch' from dual union all
select 31678,'031678','1999-025CCL','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.09,565,1151,98.52,'Sun-Synch' from dual union all
select 31679,'031679','1999-025CCM','deb FY-1C','',66706,'In Earth orbit','','2007-06-12 00:00:00',101.05,741,876,99.02,'Sun-Synch' from dual union all
select 3168,'003168','1968-024A','Kosmos-210','Zenit-2 No. 60',16751,'Landed','1968-04-11 00:00:00','1968-04-03 00:00:00',90.24,200,373,81.4,'Intermediate' from dual union all
select 31680,'031680','1999-025CCN','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',98.79,541,860,99.88,'Sun-Synch' from dual union all
select 31681,'031681','1999-025CCP','deb FY-1C','',66706,'Reentered','2014-11-25 00:00:00','2007-06-13 00:00:00',100.07,674,850,99.01,'Sun-Synch' from dual union all
select 31682,'031682','1999-025CCQ','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.82,808,882,98.68,'Sun-Synch' from dual union all
select 31683,'031683','1999-025CCR','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.29,754,886,98.49,'Sun-Synch' from dual union all
select 31684,'031684','1999-025CCS','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.9,856,936,99.14,'Sun-Synch' from dual union all
select 31685,'031685','1999-025CCT','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.25,854,877,98.77,'Sun-Synch' from dual union all
select 31686,'031686','1999-025CCU','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.1,482,1139,98.77,'Sun-Synch' from dual union all
select 31687,'031687','1999-025CCV','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.09,853,863,98.76,'Sun-Synch' from dual union all
select 31688,'031688','1999-025CCW','deb FY-1C','',66706,'In Earth orbit','','2007-06-09 00:00:00',102.85,863,924,98.8,'Sun-Synch' from dual;
insert into SATELLITE
select 31689,'031689','1999-025CCX','deb FY-1C','',66706,'Reentered','2013-12-29 00:00:00','2007-06-13 00:00:00',97.58,435,850,98.83,'Sun-Synch' from dual union all
select 3169,'003169','1968-024B','11A57I','11A57I',16751,'Reentered','1968-04-12 00:00:00','1968-04-06 00:00:00',89.7,189,331,81.35,'Intermediate' from dual union all
select 31690,'031690','1999-025CCY','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',101.66,816,859,98.85,'Sun-Synch' from dual union all
select 31691,'031691','1999-025CCZ','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',102.93,836,959,99.03,'Sun-Synch' from dual union all
select 31692,'031692','1999-025CDA','deb FY-1C','',66706,'In Earth orbit','','2007-06-09 00:00:00',104.14,854,1055,99.08,'Sun-Synch' from dual union all
select 31693,'031693','1999-025CDB','deb FY-1C','',66706,'Reentered','2014-02-25 00:00:00','2007-06-12 00:00:00',99.31,539,913,99.78,'Sun-Synch' from dual union all
select 31694,'031694','1999-025CDC','deb FY-1C','',66706,'Reentered','2015-08-31 00:00:00','2007-06-13 00:00:00',99.59,537,941,101.08,'Sun-Synch' from dual union all
select 31695,'031695','1999-025CDD','deb FY-1C','',66706,'In Earth orbit','','2007-06-13 00:00:00',100.88,574,1027,98.56,'Sun-Synch' from dual union all
select 31696,'031696','1999-025CDE','deb FY-1C','',66706,'In Earth orbit','','2007-06-11 00:00:00',104.31,842,1082,97.81,'Sun-Synch' from dual union all
select 31697,'031697','1999-025CDF','deb FY-1C','',66706,'Reentered','2011-01-04 00:00:00','2007-06-08 00:00:00',96.5,332,850,99.63,'Sun-Synch' from dual union all
select 31698,'031698','2007-026A','TerraSAR-X','TerraSAR-X',68368,'In Earth orbit','','2007-06-16 00:00:00',100.13,507,1022,97.43,'Sun-Synch' from dual union all
select 31699,'031699','2007-026B','R-36M2 DS','R-36M2 DS',68368,'In Earth orbit','','2007-06-22 00:00:00',100.13,506,1024,97.44,'Sun-Synch' from dual union all
select 317,'000317','61 Omicron 90','deb r/b Ablestar','',7273,'Reentered','1981-01-11 00:00:00','1962-06-28 00:00:00',101.66,698,977,66.43,'Intermediate' from dual union all
select 3170,'003170','1968-025A','Apollo 6','Apollo CM-020',16752,'Landed','1968-04-04 00:00:00','1968-04-04 00:00:00',389.3,33,22533,32.66,'ERROR - Unexpected value "TA"' from dual union all
select 31700,'031700','2007-026C','Dnepr plume shield','GDS',68368,'Reentered','2012-03-09 00:00:00','2007-06-22 00:00:00',94.53,481,511,97.46,'Sun-Synch' from dual union all
select 31701,'031701','2007-027A','USA 194','INTRUDER 7',68369,'In Earth orbit','','2007-06-18 00:00:00',105.43,847,1182,62.66,'Intermediate' from dual union all
select 31702,'031702','2007-027B','Centaur AV-009','Centaur AV-009',68369,'In Earth orbit','','2007-06-15 00:00:00',101.57,715,951,63.4,'Intermediate' from dual union all
select 31703,'031703','1996-029P','TIPS tether debris','',65917,'In Earth orbit','','2007-06-20 00:00:00',107.37,1050,1160,63.42,'Intermediate' from dual union all
select 31704,'031704','1996-029Q','deb USA 119','',65917,'In Earth orbit','','2007-06-20 00:00:00',107.37,1050,1160,63.42,'Intermediate' from dual union all
select 31705,'031705','1992-086Q','deb USA 89 PKM','',64798,'In Earth orbit','','2007-07-01 00:00:00',169.69,350,7200,57,'Medium' from dual union all
select 31706,'031706','1992-086R','deb USA 89 PKM','',64798,'In Earth orbit','','2007-07-01 00:00:00',169.69,350,7200,57,'Medium' from dual union all
select 31707,'031707','1983-008K','deb NOSS','',55765,'In Earth orbit','','2007-06-30 00:00:00',107.26,900,1299,63.43,'Intermediate' from dual union all
select 31708,'031708','2007-027C','USA 194 P/L 2','INTRUDER 8',68369,'In Earth orbit','','2007-06-18 00:00:00',105.43,847,1182,62.66,'Intermediate' from dual union all
select 31709,'031709','1999-025CDG','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',103.9,778,1108,98.03,'Sun-Synch' from dual union all
select 3171,'003171','1968-025B','Saturn S-IVB-502','Saturn S-IVB-502',16752,'Reentered','1968-04-26 00:00:00','1968-04-12 00:00:00',90.14,201,361,32.59,'Intermediate' from dual union all
select 31710,'031710','1999-025CDH','deb FY-1C','',66706,'In Earth orbit','','2007-06-26 00:00:00',98.75,543,855,99.03,'Sun-Synch' from dual union all
select 31711,'031711','1999-025CDJ','deb FY-1C','',66706,'In Earth orbit','','2007-06-24 00:00:00',100.38,658,896,99.09,'Sun-Synch' from dual union all
select 31712,'031712','1999-025CDK','deb FY-1C','',66706,'Reentered','2016-02-25 00:00:00','2007-06-26 00:00:00',98.42,514,853,99.07,'Sun-Synch' from dual union all
select 31713,'031713','1999-025CDL','deb FY-1C','',66706,'Reentered','2012-07-03 00:00:00','2007-06-25 00:00:00',99.17,503,935,97.9,'Sun-Synch' from dual union all
select 31714,'031714','1999-025CDM','deb FY-1C','',66706,'In Earth orbit','','2007-06-26 00:00:00',102.56,827,933,99.08,'Sun-Synch' from dual union all
select 31715,'031715','1999-025CDN','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',101.26,622,1015,99.33,'Sun-Synch' from dual union all
select 31716,'031716','1999-025CDP','deb FY-1C','',66706,'In Earth orbit','','2007-06-22 00:00:00',103.92,827,1061,98.53,'Sun-Synch' from dual union all
select 31717,'031717','1999-025CDQ','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',104.33,789,1138,96.93,'Sun-Synch' from dual union all
select 31718,'031718','1999-025CDR','deb FY-1C','',66706,'In Earth orbit','','2007-06-23 00:00:00',101.14,689,937,96.72,'Sun-Synch' from dual union all
select 31719,'031719','1999-025CDS','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',105.48,839,1195,97.55,'Sun-Synch' from dual union all
select 3172,'003172','1968-024C','deb Kosmos-210','',16751,'Reentered','1968-04-05 00:00:00','1968-04-04 00:00:00',89.09,158,301,81.4,'Intermediate' from dual union all
select 31720,'031720','1999-025CDT','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',99.79,639,858,99.13,'Sun-Synch' from dual union all
select 31721,'031721','1999-025CDU','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',105.31,760,1258,99.16,'Sun-Synch' from dual union all
select 31722,'031722','1999-025CDV','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',101.63,816,855,101.98,'Sun-Synch' from dual union all
select 31723,'031723','1999-025CDW','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',104.21,839,1076,97.83,'Sun-Synch' from dual union all
select 31724,'031724','1999-025CDX','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',102.9,716,1075,97.94,'Sun-Synch' from dual union all
select 31725,'031725','1999-025CDY','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',113.66,826,1963,99.21,'Sun-Synch' from dual union all
select 31726,'031726','1999-025CDZ','deb FY-1C','',66706,'In Earth orbit','','2007-06-26 00:00:00',103.15,705,1110,99.1,'Sun-Synch' from dual union all
select 31727,'031727','1999-025CEA','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',102.52,838,918,99.03,'Sun-Synch' from dual union all
select 31728,'031728','1999-025CEB','deb FY-1C','',66706,'Reentered','2013-10-22 00:00:00','2007-06-23 00:00:00',101.77,441,1243,98.61,'Sun-Synch' from dual union all
select 31729,'031729','1999-025CEC','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',103.57,854,1000,99.32,'Sun-Synch' from dual union all
select 3173,'003173','1968-026A','OV1-13','OV1-13',16753,'In Earth orbit','','1968-05-10 00:00:00',199.57,553,9316,99.96,'Medium' from dual union all
select 31730,'031730','1999-025CED','deb FY-1C','',66706,'Reentered','2013-03-06 00:00:00','2007-06-22 00:00:00',97.68,445,851,98.86,'Sun-Synch' from dual union all
select 31731,'031731','1999-025CEE','deb FY-1C','',66706,'In Earth orbit','','2007-06-26 00:00:00',102.28,854,880,99.36,'Sun-Synch' from dual union all
select 31732,'031732','1999-025CEF','deb FY-1C','',66706,'In Earth orbit','','2007-06-26 00:00:00',102.33,837,901,99.23,'Sun-Synch' from dual union all
select 31733,'031733','1999-025CEG','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',99.28,593,855,99.15,'Sun-Synch' from dual union all
select 31734,'031734','1999-025CEH','deb FY-1C','',66706,'In Earth orbit','','2007-06-26 00:00:00',101.3,738,903,98.8,'Sun-Synch' from dual union all
select 31735,'031735','1999-025CEJ','deb FY-1C','',66706,'In Earth orbit','','2007-06-23 00:00:00',101.06,755,863,98.92,'Sun-Synch' from dual union all
select 31736,'031736','1999-025CEK','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',102,845,862,98.85,'Sun-Synch' from dual union all
select 31737,'031737','1999-025CEL','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',99.61,593,887,98.61,'Sun-Synch' from dual union all
select 31738,'031738','1999-025CEM','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',105.33,747,1272,99.4,'Sun-Synch' from dual union all
select 31739,'031739','1999-025CEN','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',101.61,741,929,98.52,'Sun-Synch' from dual union all
select 3174,'003174','1968-026B','OV1-14','OV1-14',16753,'In Earth orbit','','1968-05-12 00:00:00',207.87,554,9938,99.99,'Medium' from dual union all
select 31740,'031740','1999-025CEP','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',101.61,811,859,99.85,'Sun-Synch' from dual union all
select 31741,'031741','1999-025CEQ','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',101.96,692,1011,99.23,'Sun-Synch' from dual union all
select 31742,'031742','1999-025CER','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',101.06,741,877,99,'Sun-Synch' from dual union all
select 31743,'031743','1999-025CES','deb FY-1C','',66706,'In Earth orbit','','2007-06-26 00:00:00',102.37,838,903,99.27,'Sun-Synch' from dual union all
select 31744,'031744','1999-025CET','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',104.7,644,1316,99.19,'Sun-Synch' from dual union all
select 31745,'031745','1999-025CEU','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',104.88,605,1373,96.76,'Sun-Synch' from dual union all
select 31746,'031746','1999-025CEV','deb FY-1C','',66706,'In Earth orbit','','2007-06-26 00:00:00',101.51,794,867,98.85,'Sun-Synch' from dual union all
select 31747,'031747','1999-025CEW','deb FY-1C','',66706,'Reentered','2010-12-28 00:00:00','2007-06-25 00:00:00',99.18,443,995,97,'Sun-Synch' from dual union all
select 31748,'031748','1999-025CEX','deb FY-1C','',66706,'Reentered','2013-12-17 00:00:00','2007-06-23 00:00:00',101.12,554,1070,97.25,'Sun-Synch' from dual union all
select 31749,'031749','1999-025CEY','deb FY-1C','',66706,'Reentered','2013-03-09 00:00:00','2007-06-26 00:00:00',98.67,448,943,99.01,'Sun-Synch' from dual union all
select 3175,'003175','1963-014AY','deb West Ford','',9245,'Reentered','1968-05-21 00:00:00','1968-04-13 00:00:00',153.49,472,5764,84.16,'Medium' from dual union all
select 31750,'031750','1999-025CEZ','deb FY-1C','',66706,'In Earth orbit','','2007-06-26 00:00:00',100.74,694,893,98.87,'Sun-Synch' from dual union all
select 31751,'031751','1999-025CFA','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',101.92,787,913,98.89,'Sun-Synch' from dual union all
select 31752,'031752','1999-025CFB','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',105.1,680,1319,98.91,'Sun-Synch' from dual union all
select 31753,'031753','1999-025CFC','deb FY-1C','',66706,'In Earth orbit','','2007-06-23 00:00:00',101.89,726,971,98.69,'Sun-Synch' from dual union all
select 31754,'031754','1999-025CFD','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',101.3,774,867,99.13,'Sun-Synch' from dual union all
select 31755,'031755','1999-025CFE','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',102.63,738,1029,99.06,'Sun-Synch' from dual union all
select 31756,'031756','1999-025CFF','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',102.14,808,912,99.43,'Sun-Synch' from dual union all
select 31757,'031757','1999-025CFG','deb FY-1C','',66706,'In Earth orbit','','2007-06-23 00:00:00',102.76,717,1061,98.92,'Sun-Synch' from dual union all
select 31758,'031758','1999-025CFH','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',105.05,563,1431,98.15,'Sun-Synch' from dual union all
select 31759,'031759','1999-025CFJ','deb FY-1C','',66706,'In Earth orbit','','2007-06-24 00:00:00',102.05,846,865,98.71,'Sun-Synch' from dual union all
select 3176,'003176','61 Omicron 217','deb r/b Ablestar','',7273,'Reentered','1990-10-05 00:00:00','1990-10-05 00:00:00',89.38,232,256,66.6,'Intermediate' from dual union all
select 31760,'031760','1999-025CFK','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',101.54,788,876,99.35,'Sun-Synch' from dual union all
select 31761,'031761','1999-025CFL','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',105.97,558,1522,98.7,'Sun-Synch' from dual union all
select 31762,'031762','1999-025CFM','deb FY-1C','',66706,'Reentered','2013-10-04 00:00:00','2007-06-24 00:00:00',97.18,379,869,99.3,'Sun-Synch' from dual union all
select 31763,'031763','1999-025CFN','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',107.67,672,1566,98.5,'Sun-Synch' from dual union all
select 31764,'031764','1999-025CFP','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',104.44,769,1167,98.14,'Sun-Synch' from dual union all
select 31765,'031765','1999-025CFQ','deb FY-1C','',66706,'In Earth orbit','','2007-06-24 00:00:00',105.57,765,1278,99.07,'Sun-Synch' from dual union all
select 31766,'031766','1999-025CFR','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',102.62,826,939,98.99,'Sun-Synch' from dual union all
select 31767,'031767','1999-025CFS','deb FY-1C','',66706,'In Earth orbit','','2007-06-24 00:00:00',102.68,854,917,98.85,'Sun-Synch' from dual union all
select 31768,'031768','1999-025CFT','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',101.26,772,865,99.14,'Sun-Synch' from dual union all
select 31769,'031769','1999-025CFU','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',103.45,831,1013,98.51,'Sun-Synch' from dual union all
select 3177,'003177','1968-026C','OV1-14P','FW4S',16753,'In Earth orbit','','1968-05-13 00:00:00',208.16,555,9959,100.02,'Medium' from dual union all
select 31770,'031770','1999-025CFV','deb FY-1C','',66706,'In Earth orbit','','2007-06-26 00:00:00',102.88,690,1100,97.73,'Sun-Synch' from dual union all
select 31771,'031771','1999-025CFW','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',102.03,844,865,102.82,'Retrograde' from dual union all
select 31772,'031772','1999-025CFX','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',103.27,788,1039,98.43,'Sun-Synch' from dual union all
select 31773,'031773','1999-025CFY','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',101.07,748,871,99.04,'Sun-Synch' from dual union all
select 31774,'031774','1999-025CFZ','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',102.84,836,950,99.11,'Sun-Synch' from dual union all
select 31775,'031775','1999-025CGA','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',102.89,855,936,98.85,'Sun-Synch' from dual union all
select 31776,'031776','1999-025CGB','deb FY-1C','',66706,'In Earth orbit','','2007-06-23 00:00:00',103.01,852,950,98.83,'Sun-Synch' from dual union all
select 31777,'031777','1999-025CGC','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',100.96,750,859,99.08,'Sun-Synch' from dual union all
select 31778,'031778','1999-025CGD','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',102.84,857,929,99.28,'Sun-Synch' from dual;
insert into SATELLITE
select 31779,'031779','1999-025CGE','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',100.65,615,964,99.38,'Sun-Synch' from dual union all
select 3178,'003178','1968-027A','Luna-14','E-6LS No. 113',16754,'Deep Space','1968-04-10 00:00:00','1968-04-07 00:00:00',21544.43,200,500000,51.8,'Cislunar' from dual union all
select 31780,'031780','1999-025CGF','deb FY-1C','',66706,'In Earth orbit','','2007-06-23 00:00:00',99.18,582,857,99.33,'Sun-Synch' from dual union all
select 31781,'031781','1999-025CGG','deb FY-1C','',66706,'In Earth orbit','','2007-06-26 00:00:00',104.93,771,1212,99.57,'Sun-Synch' from dual union all
select 31782,'031782','1999-025CGH','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',100.93,739,867,98.98,'Sun-Synch' from dual union all
select 31783,'031783','1999-025CGJ','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',102.69,802,970,99.26,'Sun-Synch' from dual union all
select 31784,'031784','1999-025CGK','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',102.43,842,906,98.95,'Sun-Synch' from dual union all
select 31785,'031785','1999-025CGL','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',100.83,727,868,96.59,'Sun-Synch' from dual union all
select 31786,'031786','1999-025CGM','deb FY-1C','',66706,'In Earth orbit','','2007-06-23 00:00:00',104.06,852,1048,97.7,'Sun-Synch' from dual union all
select 31787,'031787','1999-025CGN','deb FY-1C','',66706,'In Earth orbit','','2007-06-25 00:00:00',101.13,708,917,98.65,'Sun-Synch' from dual union all
select 31788,'031788','1999-025CGP','deb FY-1C','',66706,'In Earth orbit','','2007-06-24 00:00:00',109.1,774,1597,102.58,'Sun-Synch' from dual union all
select 31789,'031789','2007-028A','Genesis-2','Bigelow Genesis-2',68370,'In Earth orbit','','2007-06-30 00:00:00',95.81,555,560,64.51,'Intermediate' from dual union all
select 3179,'003179','1968-027B','BOZ','BOZ',16754,'Reentered','1968-04-09 00:00:00','1968-04-07 00:00:00',88.81,190,242,51.78,'Intermediate' from dual union all
select 31790,'031790','2007-028B','R-36M2 DS','R-36M2 DS',68370,'Reentered','2015-06-02 00:00:00','2007-06-30 00:00:00',108.74,545,1791,64.52,'Intermediate' from dual union all
select 31791,'031791','2007-028C','Dnepr plume shield','GDS',68370,'In Earth orbit','','2007-06-30 00:00:00',95.54,534,556,64.51,'Intermediate' from dual union all
select 31792,'031792','2007-029A','Kosmos-2428','Tselina-2',68371,'In Earth orbit','','2007-07-01 00:00:00',101.95,846,856,70.98,'Intermediate' from dual union all
select 31793,'031793','2007-029B','11S772','11S772',68371,'In Earth orbit','','2007-07-01 00:00:00',101.82,843,847,70.98,'Intermediate' from dual union all
select 31794,'031794','2007-029C','Sep Motor Cover','Sep Motor Cover',68371,'In Earth orbit','','2007-07-01 00:00:00',105.07,846,1150,71.04,'Intermediate' from dual union all
select 31795,'031795','2007-029D','Sep Motor Cover','Sep Motor Cover',68371,'In Earth orbit','','2007-07-01 00:00:00',105.31,844,1174,70.95,'Intermediate' from dual union all
select 31796,'031796','2007-029E','Sep Motor Cover','Sep Motor Cover',68371,'In Earth orbit','','2007-07-01 00:00:00',105.29,849,1167,71.04,'Intermediate' from dual union all
select 31797,'031797','2007-030A','SAR-Lupe 2','SAR-Lupe 2',68372,'In Earth orbit','','2007-07-04 00:00:00',94.36,469,507,98.18,'Sun-Synch' from dual union all
select 31798,'031798','2007-030B','S3M','S3M',68372,'Reentered','2014-09-23 00:00:00','2007-07-07 00:00:00',94.44,465,517,98.18,'Sun-Synch' from dual union all
select 31799,'031799','2007-029F','Sep Motor Cover','Sep Motor Cover',68371,'In Earth orbit','','2007-07-03 00:00:00',105.47,849,1184,70.94,'Intermediate' from dual union all
select 318,'000318','61 Omicron 91','deb r/b Ablestar','',7273,'In Earth orbit','','1962-06-27 00:00:00',101.89,689,1008,66.51,'Intermediate' from dual union all
select 3180,'003180','1968-027C','11S510','11S510',16754,'Reentered','1968-04-09 00:00:00','1968-04-07 00:00:00',88.67,180,237,51.79,'Intermediate' from dual union all
select 31800,'031800','2007-031A','Zhongxing 6B','zhongxing liu B weixing',68373,'In Earth orbit','','2007-12-21 00:00:00',1436.11,35772,35802,.05,'Stationary' from dual union all
select 31801,'031801','2007-031B','CZ-3B Stage 3','CZ-3B Stage 3',68373,'Reentered','2007-07-09 00:00:00','2007-07-05 00:00:00',909.22,169,49253,24.25,'Highly Ellip' from dual union all
select 31802,'031802','1999-025CGQ','deb FY-1C','',66706,'In Earth orbit','','2007-06-30 00:00:00',98.81,523,880,97.9,'Sun-Synch' from dual union all
select 31803,'031803','1999-025CGR','deb FY-1C','',66706,'Reentered','2015-05-03 00:00:00','2007-07-04 00:00:00',102.13,414,1305,98.17,'Sun-Synch' from dual union all
select 31804,'031804','1999-025CGS','deb FY-1C','',66706,'In Earth orbit','','2007-07-04 00:00:00',103.59,802,1055,98.33,'Sun-Synch' from dual union all
select 31805,'031805','1999-025CGT','deb FY-1C','',66706,'In Earth orbit','','2007-07-02 00:00:00',106.1,581,1511,98.39,'Sun-Synch' from dual union all
select 31806,'031806','1999-025CGU','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',106.53,768,1364,97.69,'Sun-Synch' from dual union all
select 31807,'031807','1999-025CGV','deb FY-1C','',66706,'In Earth orbit','','2007-07-04 00:00:00',100.53,684,884,99.15,'Sun-Synch' from dual union all
select 31808,'031808','1999-025CGW','deb FY-1C','',66706,'In Earth orbit','','2007-07-03 00:00:00',103.69,849,1016,99.11,'Sun-Synch' from dual union all
select 31809,'031809','1999-025CGX','deb FY-1C','',66706,'In Earth orbit','','2007-07-04 00:00:00',101.09,761,860,98.85,'Sun-Synch' from dual union all
select 3181,'003181','1968-028A','Kosmos-211','DS-P1-Yu No. 13',16755,'Reentered','1968-11-10 00:00:00','1968-05-11 00:00:00',100.81,197,1397,81.86,'Intermediate' from dual union all
select 31810,'031810','1999-025CGY','deb FY-1C','',66706,'In Earth orbit','','2007-07-02 00:00:00',100.96,612,996,98.66,'Sun-Synch' from dual union all
select 31811,'031811','1999-025CGZ','deb FY-1C','',66706,'Reentered','2017-05-05 00:00:00','2007-07-06 00:00:00',100.92,740,865,99,'Sun-Synch' from dual union all
select 31812,'031812','1999-025CHA','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',103.01,851,951,99.36,'Sun-Synch' from dual union all
select 31813,'031813','1999-025CHB','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',100.31,654,892,99.49,'Sun-Synch' from dual union all
select 31814,'031814','1999-025CHC','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',105.77,852,1209,99.43,'Sun-Synch' from dual union all
select 31815,'031815','1999-025CHD','deb FY-1C','',66706,'In Earth orbit','','2007-07-02 00:00:00',102.16,819,902,98.98,'Sun-Synch' from dual union all
select 31816,'031816','1999-025CHE','deb FY-1C','',66706,'In Earth orbit','','2007-07-05 00:00:00',104.15,854,1056,98.86,'Sun-Synch' from dual union all
select 31817,'031817','1999-025CHF','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',100.63,715,862,98.99,'Sun-Synch' from dual union all
select 31818,'031818','1999-025CHG','deb FY-1C','',66706,'Reentered','2014-05-22 00:00:00','2007-07-06 00:00:00',97.77,434,869,98.89,'Sun-Synch' from dual union all
select 31819,'031819','1999-025CHH','deb FY-1C','',66706,'In Earth orbit','','2007-06-30 00:00:00',101.22,747,886,98.8,'Sun-Synch' from dual union all
select 3182,'003182','1968-028B','S1M','S1M',16755,'Reentered','1968-08-17 00:00:00','1968-05-09 00:00:00',100.07,192,1331,81.84,'Intermediate' from dual union all
select 31820,'031820','1999-025CHJ','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',100.35,545,1006,99.04,'Sun-Synch' from dual union all
select 31821,'031821','1999-025CHK','deb FY-1C','',66706,'In Earth orbit','','2007-07-04 00:00:00',106.1,688,1404,98.2,'Sun-Synch' from dual union all
select 31822,'031822','1999-025CHL','deb FY-1C','',66706,'Reentered','2010-11-03 00:00:00','2007-07-06 00:00:00',97.19,354,895,98.56,'Sun-Synch' from dual union all
select 31823,'031823','1999-025CHM','deb FY-1C','',66706,'In Earth orbit','','2007-07-02 00:00:00',101.05,762,855,99.57,'Sun-Synch' from dual union all
select 31824,'031824','1999-025CHN','deb FY-1C','',66706,'Reentered','2011-10-04 00:00:00','2007-07-04 00:00:00',97.47,349,926,97.96,'Sun-Synch' from dual union all
select 31825,'031825','1999-025CHP','deb FY-1C','',66706,'In Earth orbit','','2007-07-04 00:00:00',101.98,719,986,98.9,'Sun-Synch' from dual union all
select 31826,'031826','1999-025CHQ','deb FY-1C','',66706,'In Earth orbit','','2007-07-04 00:00:00',102,756,951,98.96,'Sun-Synch' from dual union all
select 31827,'031827','1999-025CHR','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',104.27,853,1067,97.67,'Sun-Synch' from dual union all
select 31828,'031828','1999-025CHS','deb FY-1C','',66706,'In Earth orbit','','2007-07-02 00:00:00',103.06,708,1099,99.33,'Sun-Synch' from dual union all
select 31829,'031829','1999-025CHT','deb FY-1C','',66706,'In Earth orbit','','2007-07-02 00:00:00',104.04,820,1079,98.88,'Sun-Synch' from dual union all
select 3183,'003183','1968-029A','Kosmos-212','Soyuz No. 8',16756,'Landed','1968-04-20 00:00:00','1968-04-15 00:00:00',88.29,180,200,51.67,'Intermediate' from dual union all
select 31830,'031830','1999-025CHU','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',105.36,851,1172,99.08,'Sun-Synch' from dual union all
select 31831,'031831','1999-025CHV','deb FY-1C','',66706,'In Earth orbit','','2007-07-02 00:00:00',104.21,839,1076,97.88,'Sun-Synch' from dual union all
select 31832,'031832','1999-025CHW','deb FY-1C','',66706,'In Earth orbit','','2007-07-04 00:00:00',104.41,839,1095,97.81,'Sun-Synch' from dual union all
select 31833,'031833','1999-025CHX','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',107.4,845,1368,98.24,'Sun-Synch' from dual union all
select 31834,'031834','1999-025CHY','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',103.04,774,1031,98.34,'Sun-Synch' from dual union all
select 31835,'031835','1999-025CHZ','deb FY-1C','',66706,'In Earth orbit','','2007-07-04 00:00:00',103.79,838,1037,98.42,'Sun-Synch' from dual union all
select 31836,'031836','1999-025CJA','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',104,836,1060,98.61,'Sun-Synch' from dual union all
select 31837,'031837','1999-025CJB','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',99.82,629,870,98.31,'Sun-Synch' from dual union all
select 31838,'031838','1999-025CJC','deb FY-1C','',66706,'In Earth orbit','','2007-07-04 00:00:00',102.2,829,897,98.77,'Sun-Synch' from dual union all
select 31839,'031839','1999-025CJD','deb FY-1C','',66706,'In Earth orbit','','2007-07-04 00:00:00',106.05,658,1429,98.32,'Sun-Synch' from dual union all
select 3184,'003184','1968-029B','11S510','11S510',16756,'Reentered','1968-04-16 00:00:00','1968-04-15 00:00:00',88.49,198,202,51.68,'Intermediate' from dual union all
select 31840,'031840','1999-025CJE','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',103.49,853,995,99.2,'Sun-Synch' from dual union all
select 31841,'031841','1999-025CJF','deb FY-1C','',66706,'In Earth orbit','','2007-07-04 00:00:00',104.53,688,1257,98.47,'Sun-Synch' from dual union all
select 31842,'031842','1999-025CJG','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',98.76,435,963,97.83,'Sun-Synch' from dual union all
select 31843,'031843','1999-025CJH','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',110.83,668,1862,99.22,'Sun-Synch' from dual union all
select 31844,'031844','1999-025CJJ','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',102.46,785,966,100.31,'Sun-Synch' from dual union all
select 31845,'031845','1999-025CJK','deb FY-1C','',66706,'Reentered','2009-07-21 00:00:00','2007-07-06 00:00:00',95.99,326,807,98.44,'Sun-Synch' from dual union all
select 31846,'031846','1999-025CJL','deb FY-1C','',66706,'Reentered','2012-01-21 00:00:00','2007-07-06 00:00:00',97.67,451,844,99.71,'Sun-Synch' from dual union all
select 31847,'031847','1999-025CJM','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',103.55,842,1011,99.05,'Sun-Synch' from dual union all
select 31848,'031848','1999-025CJN','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',99.3,594,857,96.74,'Sun-Synch' from dual union all
select 31849,'031849','1999-025CJP','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',100.45,600,960,97.98,'Sun-Synch' from dual union all
select 3185,'003185','1968-025C','deb Apollo 6','',16752,'Reentered','1968-04-28 00:00:00','1968-04-16 00:00:00',90.26,234,341,32.53,'Intermediate' from dual union all
select 31850,'031850','1999-025CJQ','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',101.4,547,1102,98.29,'Sun-Synch' from dual union all
select 31851,'031851','1999-025CJR','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',105.84,851,1217,98.82,'Sun-Synch' from dual union all
select 31852,'031852','1999-025CJS','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',105.76,529,1530,98.87,'Sun-Synch' from dual union all
select 31853,'031853','1999-025CJT','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',103.03,852,952,98.94,'Sun-Synch' from dual union all
select 31854,'031854','1999-025CJU','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',103.09,842,967,99.14,'Sun-Synch' from dual union all
select 31855,'031855','1999-025CJV','deb FY-1C','',66706,'In Earth orbit','','2007-07-02 00:00:00',99.91,477,1032,96.7,'Sun-Synch' from dual union all
select 31856,'031856','1999-025CJW','deb FY-1C','',66706,'In Earth orbit','','2007-07-05 00:00:00',100.5,706,858,98.96,'Sun-Synch' from dual union all
select 31857,'031857','1999-025CJX','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',106.94,823,1347,96.21,'Sun-Synch' from dual union all
select 31858,'031858','1999-025CJY','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',106.64,781,1361,97.64,'Sun-Synch' from dual union all
select 31859,'031859','1999-025CJZ','deb FY-1C','',66706,'Reentered','2012-10-10 00:00:00','2007-07-06 00:00:00',97.44,314,959,99.11,'Sun-Synch' from dual union all
select 3186,'003186','1968-025D','deb Saturn S-IVB-502','',16752,'Reentered','1968-05-05 00:00:00','1968-04-15 00:00:00',90.8,252,376,32.57,'Intermediate' from dual union all
select 31860,'031860','1999-025CKA','deb FY-1C','',66706,'Reentered','2012-10-26 00:00:00','2007-07-06 00:00:00',99.02,466,958,98.1,'Sun-Synch' from dual union all
select 31861,'031861','1999-025CKB','deb FY-1C','',66706,'In Earth orbit','','2007-07-06 00:00:00',109.87,794,1648,102.33,'Sun-Synch' from dual union all
select 31862,'031862','2007-032A','DirecTV 10','DirecTV 10',68374,'In Earth orbit','','2007-09-19 00:00:00',1436.11,35786,35788,.06,'Stationary' from dual union all
select 31863,'031863','2007-032B','Briz-M DTB','Briz-M torus tank',68374,'In Earth orbit','','2007-07-09 00:00:00',263.92,312,14191,46.09,'Highly Ellip' from dual union all
select 31864,'031864','2007-032C','Briz-M No. 88520','Briz-M 88520',68374,'In Earth orbit','','2007-07-07 00:00:00',718.61,4841,35555,21.32,'GEO Transfer' from dual union all
select 31865,'031865','2005-031M','deb OICETS','',68040,'Reentered','2007-07-31 00:00:00','2007-07-08 00:00:00',96.34,558,609,97.86,'Sun-Synch' from dual union all
select 31866,'031866','1992-047AR','deb SOZ','',64759,'Reentered','2010-08-30 00:00:00','2007-07-16 00:00:00',153.64,435,5813,64.73,'Medium' from dual union all
select 31867,'031867','1999-057PS','ZY-1 debris','',66738,'Reentered','2016-05-06 00:00:00','2007-07-17 00:00:00',98.95,643,774,98.26,'Sun-Synch' from dual union all
select 31868,'031868','1999-057PT','ZY-1 debris','',66738,'In Earth orbit','','2007-07-16 00:00:00',100.43,773,784,98.22,'Sun-Synch' from dual;
insert into SATELLITE
select 31869,'031869','1999-057PU','ZY-1 debris','',66738,'Reentered','2015-04-01 00:00:00','2007-07-18 00:00:00',100.16,755,778,98.22,'Sun-Synch' from dual union all
select 3187,'003187','1968-025E','LTA-2R','LTA-2R',16752,'Reentered','1968-09-03 00:00:00','1968-05-09 00:00:00',90.46,236,359,32.57,'Intermediate' from dual union all
select 31870,'031870','1999-057PV','ZY-1 debris','',66738,'In Earth orbit','','2007-07-16 00:00:00',100.32,767,781,98.22,'Sun-Synch' from dual union all
select 31871,'031871','1999-057PW','ZY-1 debris','',66738,'In Earth orbit','','2007-07-18 00:00:00',100.22,760,778,98.2,'Sun-Synch' from dual union all
select 31872,'031872','1999-057PX','ZY-1 debris','',66738,'In Earth orbit','','2007-07-17 00:00:00',100.33,764,785,98.22,'Sun-Synch' from dual union all
select 31873,'031873','1999-057PY','ZY-1 debris','',66738,'In Earth orbit','','2007-07-08 00:00:00',100.36,769,783,98.21,'Sun-Synch' from dual union all
select 31874,'031874','1999-057PZ','ZY-1 debris','',66738,'In Earth orbit','','2007-07-16 00:00:00',100.18,759,776,98.21,'Sun-Synch' from dual union all
select 31875,'031875','1999-057QA','ZY-1 debris','',66738,'In Earth orbit','','2007-07-16 00:00:00',100.33,767,782,98.2,'Sun-Synch' from dual union all
select 31876,'031876','1999-057QB','ZY-1 debris','',66738,'In Earth orbit','','2007-07-18 00:00:00',100.53,771,797,98.21,'Sun-Synch' from dual union all
select 31877,'031877','1999-025CKC','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',98.98,553,867,99.09,'Sun-Synch' from dual union all
select 31878,'031878','1999-025CKD','deb FY-1C','',66706,'In Earth orbit','','2007-07-17 00:00:00',99.61,623,856,99.51,'Sun-Synch' from dual union all
select 31879,'031879','1999-025CKE','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',100.56,614,957,99.75,'Sun-Synch' from dual union all
select 3188,'003188','1968-025F','deb Saturn S-IVB-502','',16752,'Reentered','1968-04-29 00:00:00','1968-04-14 00:00:00',90.47,258,337,32.59,'Intermediate' from dual union all
select 31880,'031880','1999-025CKF','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',104.27,856,1065,100.04,'Sun-Synch' from dual union all
select 31881,'031881','1999-025CKG','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',100.45,662,898,99.1,'Sun-Synch' from dual union all
select 31882,'031882','1999-025CKH','deb FY-1C','',66706,'In Earth orbit','','2007-07-14 00:00:00',102.61,851,914,98.64,'Sun-Synch' from dual union all
select 31883,'031883','1999-025CKJ','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',99.54,614,860,99.11,'Sun-Synch' from dual union all
select 31884,'031884','1999-025CKK','deb FY-1C','',66706,'Reentered','2011-12-03 00:00:00','2007-07-18 00:00:00',97.52,435,845,99.34,'Sun-Synch' from dual union all
select 31885,'031885','1999-025CKL','deb FY-1C','',66706,'Reentered','2008-02-12 00:00:00','2007-07-18 00:00:00',95.21,347,711,99.1,'Sun-Synch' from dual union all
select 31886,'031886','1999-025CKM','deb FY-1C','',66706,'In Earth orbit','','2007-07-14 00:00:00',102.87,834,955,99.24,'Sun-Synch' from dual union all
select 31887,'031887','1999-025CKN','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',101.22,777,856,99.59,'Sun-Synch' from dual union all
select 31888,'031888','1999-025CKP','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',102.49,851,902,99.29,'Sun-Synch' from dual union all
select 31889,'031889','1999-025CKQ','deb FY-1C','',66706,'Reentered','2007-08-31 00:00:00','2007-07-18 00:00:00',93.58,338,561,97.18,'Sun-Synch' from dual union all
select 3189,'003189','1968-025G','deb Saturn S-IVB-502','',16752,'Reentered','1968-04-20 00:00:00','1968-04-15 00:00:00',90.53,204,397,32.58,'Intermediate' from dual union all
select 31890,'031890','1999-025CKR','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',102.24,811,919,98.84,'Sun-Synch' from dual union all
select 31891,'031891','1999-025CKS','deb FY-1C','',66706,'In Earth orbit','','2007-07-16 00:00:00',105.77,856,1205,99.59,'Sun-Synch' from dual union all
select 31892,'031892','1999-025CKT','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',101.04,634,982,99.32,'Sun-Synch' from dual union all
select 31893,'031893','1999-025CKU','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',99.23,573,871,99.12,'Sun-Synch' from dual union all
select 31894,'031894','1999-025CKV','deb FY-1C','',66706,'Reentered','2012-03-08 00:00:00','2007-07-17 00:00:00',96.86,361,856,98.89,'Sun-Synch' from dual union all
select 31895,'031895','1999-025CKW','deb FY-1C','',66706,'In Earth orbit','','2007-07-16 00:00:00',106.11,850,1243,99.12,'Sun-Synch' from dual union all
select 31896,'031896','1999-025CKX','deb FY-1C','',66706,'Reentered','2010-01-17 00:00:00','2007-07-18 00:00:00',97.2,441,809,98.87,'Sun-Synch' from dual union all
select 31897,'031897','1999-025CKY','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',101.55,767,898,98.82,'Sun-Synch' from dual union all
select 31898,'031898','1999-025CKZ','deb FY-1C','',66706,'In Earth orbit','','2007-07-16 00:00:00',100.46,613,948,98.83,'Sun-Synch' from dual union all
select 31899,'031899','1999-025CLA','deb FY-1C','',66706,'In Earth orbit','','2007-07-15 00:00:00',105.48,832,1202,102.1,'Sun-Synch' from dual union all
select 319,'000319','61 Omicron 92','deb r/b Ablestar','',7273,'Reentered','1981-07-23 00:00:00','1962-06-27 00:00:00',101.25,660,976,66.85,'Intermediate' from dual union all
select 3190,'003190','1968-025H','deb Saturn S-IVB-502','',16752,'Reentered','1968-04-17 00:00:00','1968-04-15 00:00:00',90.25,261,313,32.56,'Intermediate' from dual union all
select 31900,'031900','1999-025CLB','deb FY-1C','',66706,'Reentered','2014-01-30 00:00:00','2007-07-18 00:00:00',101.22,551,1082,97.81,'Sun-Synch' from dual union all
select 31901,'031901','1999-025CLC','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',103.45,823,1021,98.57,'Sun-Synch' from dual union all
select 31902,'031902','1999-025CLD','deb FY-1C','',66706,'In Earth orbit','','2007-07-16 00:00:00',101.57,693,973,98.98,'Sun-Synch' from dual union all
select 31903,'031903','1999-025CLE','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',100.04,633,888,99.3,'Sun-Synch' from dual union all
select 31904,'031904','1999-025CLF','deb FY-1C','',66706,'In Earth orbit','','2007-07-16 00:00:00',104.88,834,1144,98.95,'Sun-Synch' from dual union all
select 31905,'031905','1999-025CLG','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',101.57,664,1001,99.22,'Sun-Synch' from dual union all
select 31906,'031906','1999-025CLH','deb FY-1C','',66706,'In Earth orbit','','2007-07-17 00:00:00',103.05,854,952,98.64,'Sun-Synch' from dual union all
select 31907,'031907','1999-025CLJ','deb FY-1C','',66706,'Reentered','2012-06-22 00:00:00','2007-07-18 00:00:00',96.88,374,845,99.69,'Sun-Synch' from dual union all
select 31908,'031908','1999-025CLK','deb FY-1C','',66706,'Reentered','2016-06-09 00:00:00','2007-07-17 00:00:00',99.24,509,936,97.13,'Sun-Synch' from dual union all
select 31909,'031909','1999-025CLL','deb FY-1C','',66706,'Reentered','2012-12-20 00:00:00','2007-07-18 00:00:00',99.39,531,928,97.67,'Sun-Synch' from dual union all
select 3191,'003191','1968-025J','deb Saturn S-IVB-502','',16752,'Reentered','1968-04-17 00:00:00','1968-04-14 00:00:00',89.83,198,335,32.57,'Intermediate' from dual union all
select 31910,'031910','1999-025CLM','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',101.39,601,1048,97.71,'Sun-Synch' from dual union all
select 31911,'031911','1999-025CLN','deb FY-1C','',66706,'In Earth orbit','','2007-07-16 00:00:00',109.55,770,1642,98.75,'Sun-Synch' from dual union all
select 31912,'031912','1999-025CLP','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',109.59,770,1646,98.86,'Sun-Synch' from dual union all
select 31913,'031913','1999-025CLQ','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',106.27,847,1261,99.58,'Sun-Synch' from dual union all
select 31914,'031914','1999-025CLR','deb FY-1C','',66706,'In Earth orbit','','2007-07-15 00:00:00',99.4,586,874,99.07,'Sun-Synch' from dual union all
select 31915,'031915','1999-025CLS','deb FY-1C','',66706,'In Earth orbit','','2007-07-17 00:00:00',102.57,793,967,99.09,'Sun-Synch' from dual union all
select 31916,'031916','1999-025CLT','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',102.23,723,1006,99.42,'Sun-Synch' from dual union all
select 31917,'031917','1999-025CLU','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',101.1,650,971,98.82,'Sun-Synch' from dual union all
select 31918,'031918','1999-025CLV','deb FY-1C','',66706,'Reentered','2014-11-11 00:00:00','2007-07-16 00:00:00',101.13,757,868,99.19,'Sun-Synch' from dual union all
select 31919,'031919','1999-025CLW','deb FY-1C','',66706,'In Earth orbit','','2007-07-16 00:00:00',100.3,688,858,99.16,'Sun-Synch' from dual union all
select 3192,'003192','1968-025K','deb Saturn S-IVB-502','',16752,'Reentered','1968-04-17 00:00:00','1968-04-14 00:00:00',90.01,209,341,32.54,'Intermediate' from dual union all
select 31920,'031920','1999-025CLX','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',98.71,521,873,98.85,'Sun-Synch' from dual union all
select 31921,'031921','1999-025CLY','deb FY-1C','',66706,'In Earth orbit','','2007-07-16 00:00:00',105.58,855,1189,99.26,'Sun-Synch' from dual union all
select 31922,'031922','1999-025CLZ','deb FY-1C','',66706,'In Earth orbit','','2007-07-14 00:00:00',104.89,854,1125,98.84,'Sun-Synch' from dual union all
select 31923,'031923','1999-025CMA','deb FY-1C','',66706,'In Earth orbit','','2007-07-17 00:00:00',107.08,846,1336,98.42,'Sun-Synch' from dual union all
select 31924,'031924','1999-025CMB','deb FY-1C','',66706,'In Earth orbit','','2007-07-18 00:00:00',106.66,843,1301,98.42,'Sun-Synch' from dual union all
select 31925,'031925','1999-025CMC','deb FY-1C','',66706,'Reentered','2010-09-04 00:00:00','2007-07-17 00:00:00',97.8,408,899,98.42,'Sun-Synch' from dual union all
select 31926,'031926','1999-025CMD','deb FY-1C','',66706,'In Earth orbit','','2007-07-15 00:00:00',102.91,847,946,98.92,'Sun-Synch' from dual union all
select 31927,'031927','1998-067AZ','VSSA FSE/FRAM','ISS US EVA-9 debris',66503,'Reentered','2008-02-13 00:00:00','2007-07-23 00:00:00',91.17,324,339,51.63,'Intermediate' from dual union all
select 31928,'031928','1998-067BA','Early Ammonia Servicer','ISS EAS',66503,'Reentered','2008-11-03 00:00:00','2007-07-24 00:00:00',91.19,326,340,51.64,'Intermediate' from dual union all
select 31929,'031929','1999-025CME','deb FY-1C','',66706,'In Earth orbit','','2007-07-27 00:00:00',101.07,749,870,98.99,'Sun-Synch' from dual union all
select 3193,'003193','1968-030A','Kosmos-213','Soyuz No. 7',16757,'Landed','1968-04-20 00:00:00','1968-04-17 00:00:00',88.95,193,253,51.66,'Intermediate' from dual union all
select 31930,'031930','1999-025CMF','deb FY-1C','',66706,'In Earth orbit','','2007-07-30 00:00:00',104.05,706,1194,98.29,'Sun-Synch' from dual union all
select 31931,'031931','1999-025CMG','deb FY-1C','',66706,'In Earth orbit','','2007-07-30 00:00:00',107.79,791,1458,97.82,'Sun-Synch' from dual union all
select 31932,'031932','1999-025CMH','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',102.86,772,1016,98.47,'Sun-Synch' from dual union all
select 31933,'031933','1999-025CMJ','deb FY-1C','',66706,'Reentered','2014-12-10 00:00:00','2007-07-30 00:00:00',100.42,659,898,99.11,'Sun-Synch' from dual union all
select 31934,'031934','1999-025CMK','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',100.77,699,892,99,'Sun-Synch' from dual union all
select 31935,'031935','1999-025CML','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',102.51,818,937,99.75,'Sun-Synch' from dual union all
select 31936,'031936','1999-025CMM','deb FY-1C','',66706,'Reentered','2014-03-18 00:00:00','2007-07-31 00:00:00',100.24,685,854,98.96,'Sun-Synch' from dual union all
select 31937,'031937','1999-025CMN','deb FY-1C','',66706,'In Earth orbit','','2007-07-30 00:00:00',99.94,563,949,99.33,'Sun-Synch' from dual union all
select 31938,'031938','1999-025CMP','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',98.5,511,862,98.97,'Sun-Synch' from dual union all
select 31939,'031939','1999-025CMQ','deb FY-1C','',66706,'Reentered','2015-05-11 00:00:00','2007-07-31 00:00:00',98.98,463,957,98.84,'Sun-Synch' from dual union all
select 3194,'003194','1968-030B','11S510','11S510',16757,'Reentered','1968-04-19 00:00:00','1968-04-16 00:00:00',88.88,190,248,51.65,'Intermediate' from dual union all
select 31940,'031940','1999-025CMR','deb FY-1C','',66706,'Reentered','2014-06-25 00:00:00','2007-07-28 00:00:00',98.68,486,905,99.44,'Sun-Synch' from dual union all
select 31941,'031941','1999-025CMS','deb FY-1C','',66706,'Reentered','2012-10-26 00:00:00','2007-07-30 00:00:00',96.77,361,847,99.06,'Sun-Synch' from dual union all
select 31942,'031942','1999-025CMT','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',105.03,842,1150,98.95,'Sun-Synch' from dual union all
select 31943,'031943','1999-025CMU','deb FY-1C','',66706,'Reentered','2013-03-17 00:00:00','2007-07-31 00:00:00',99.26,590,857,99.21,'Sun-Synch' from dual union all
select 31944,'031944','1999-025CMV','deb FY-1C','',66706,'In Earth orbit','','2007-07-30 00:00:00',102.78,763,1018,99.35,'Sun-Synch' from dual union all
select 31945,'031945','1999-025CMW','deb FY-1C','',66706,'In Earth orbit','','2007-07-29 00:00:00',102.96,843,954,98.89,'Sun-Synch' from dual union all
select 31946,'031946','1999-025CMX','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',102.81,820,963,99,'Sun-Synch' from dual union all
select 31947,'031947','1999-025CMY','deb FY-1C','',66706,'In Earth orbit','','2007-07-29 00:00:00',101.63,767,905,98.77,'Sun-Synch' from dual union all
select 31948,'031948','1999-025CMZ','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',102.57,856,905,99.47,'Sun-Synch' from dual union all
select 31949,'031949','1999-025CNA','deb FY-1C','',66706,'In Earth orbit','','2007-07-28 00:00:00',101.97,755,949,99.1,'Sun-Synch' from dual union all
select 3195,'003195','1968-025L','deb Saturn S-IVB-502','',16752,'Reentered','1968-05-01 00:00:00','1968-04-21 00:00:00',90.26,237,338,32.56,'Intermediate' from dual union all
select 31950,'031950','1999-025CNB','deb FY-1C','',66706,'Reentered','2015-07-05 00:00:00','2007-07-30 00:00:00',98.73,457,939,99.18,'Sun-Synch' from dual union all
select 31951,'031951','1999-025CNC','deb FY-1C','',66706,'Reentered','2011-10-20 00:00:00','2007-07-30 00:00:00',96.91,359,863,98.99,'Sun-Synch' from dual union all
select 31952,'031952','1999-025CND','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',99.98,657,859,99.04,'Sun-Synch' from dual union all
select 31953,'031953','1999-025CNE','deb FY-1C','',66706,'In Earth orbit','','2007-07-28 00:00:00',103.93,769,1120,99.38,'Sun-Synch' from dual union all
select 31954,'031954','1999-025CNF','deb FY-1C','',66706,'In Earth orbit','','2007-07-30 00:00:00',103.57,854,1000,98.14,'Sun-Synch' from dual union all
select 31955,'031955','1999-025CNG','deb FY-1C','',66706,'In Earth orbit','','2007-07-30 00:00:00',101.21,761,871,99.36,'Sun-Synch' from dual union all
select 31956,'031956','1999-025CNH','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',102.1,821,895,99.52,'Sun-Synch' from dual union all
select 31957,'031957','1999-025CNJ','deb FY-1C','',66706,'In Earth orbit','','2007-07-30 00:00:00',101.13,767,858,99.55,'Sun-Synch' from dual union all
select 31958,'031958','1999-025CNK','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',100.42,698,859,99.01,'Sun-Synch' from dual;
insert into SATELLITE
select 31959,'031959','1999-025CNL','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',102.58,838,924,98.63,'Sun-Synch' from dual union all
select 3196,'003196','1968-025M','deb Saturn S-IVB-502','',16752,'Reentered','1968-04-22 00:00:00','1968-04-17 00:00:00',89.96,231,313,32.56,'Intermediate' from dual union all
select 31960,'031960','1999-025CNM','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',104,831,1065,99.76,'Sun-Synch' from dual union all
select 31961,'031961','1999-025CNN','deb FY-1C','',66706,'Reentered','2011-07-02 00:00:00','2007-07-31 00:00:00',98.58,470,912,97.85,'Sun-Synch' from dual union all
select 31962,'031962','1999-025CNP','deb FY-1C','',66706,'Reentered','2015-04-11 00:00:00','2007-07-31 00:00:00',100.42,697,860,99.05,'Sun-Synch' from dual union all
select 31963,'031963','1999-025CNQ','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',106.3,680,1430,101.7,'Sun-Synch' from dual union all
select 31964,'031964','1999-025CNR','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',101.76,800,884,98.91,'Sun-Synch' from dual union all
select 31965,'031965','1999-025CNS','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',102.53,840,917,99.11,'Sun-Synch' from dual union all
select 31966,'031966','1999-025CNT','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',100.2,668,867,99.08,'Sun-Synch' from dual union all
select 31967,'031967','1999-025CNU','deb FY-1C','',66706,'Reentered','2011-12-30 00:00:00','2007-07-27 00:00:00',98.6,484,900,99.41,'Sun-Synch' from dual union all
select 31968,'031968','1999-025CNV','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',101.04,742,873,98.99,'Sun-Synch' from dual union all
select 31969,'031969','1999-025CNW','deb FY-1C','',66706,'Reentered','2017-07-27 00:00:00','2007-07-31 00:00:00',101.42,693,959,98.87,'Sun-Synch' from dual union all
select 3197,'003197','1968-025N','deb Saturn S-IVB-502','',16752,'Reentered','1968-04-16 00:00:00','1968-04-15 00:00:00',89.86,219,316,32.56,'Intermediate' from dual union all
select 31970,'031970','1999-025CNX','deb FY-1C','',66706,'Reentered','2012-12-02 00:00:00','2007-07-30 00:00:00',97.91,463,855,99.51,'Sun-Synch' from dual union all
select 31971,'031971','1999-025CNY','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',102.22,854,873,102.27,'Sun-Synch' from dual union all
select 31972,'031972','1999-025CNZ','deb FY-1C','',66706,'In Earth orbit','','2007-07-30 00:00:00',102.53,820,937,98.63,'Sun-Synch' from dual union all
select 31973,'031973','1999-025CPA','deb FY-1C','',66706,'In Earth orbit','','2007-07-30 00:00:00',103.44,824,1019,98.37,'Sun-Synch' from dual union all
select 31974,'031974','1999-025CPB','deb FY-1C','',66706,'In Earth orbit','','2007-07-29 00:00:00',102.91,818,975,98.42,'Sun-Synch' from dual union all
select 31975,'031975','1999-025CPC','deb FY-1C','',66706,'Reentered','2014-10-20 00:00:00','2007-07-31 00:00:00',99.71,598,892,99.41,'Sun-Synch' from dual union all
select 31976,'031976','1999-025CPD','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',101.47,765,892,100.04,'Sun-Synch' from dual union all
select 31977,'031977','1999-025CPE','deb FY-1C','',66706,'In Earth orbit','','2007-07-30 00:00:00',105.66,789,1262,98.21,'Sun-Synch' from dual union all
select 31978,'031978','1999-025CPF','deb FY-1C','',66706,'In Earth orbit','','2007-07-29 00:00:00',106.45,847,1278,98.39,'Sun-Synch' from dual union all
select 31979,'031979','1999-025CPG','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',106.97,845,1328,99.37,'Sun-Synch' from dual union all
select 3198,'003198','1968-025P','deb Saturn S-IVB-502','',16752,'Reentered','1968-04-17 00:00:00','1968-04-16 00:00:00',89.42,222,270,32.56,'Intermediate' from dual union all
select 31980,'031980','1999-025CPH','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',108.48,822,1491,97.61,'Sun-Synch' from dual union all
select 31981,'031981','1999-025CPJ','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',108.58,850,1473,98.45,'Sun-Synch' from dual union all
select 31982,'031982','1999-025CPK','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',98.68,505,887,97.66,'Sun-Synch' from dual union all
select 31983,'031983','1999-025CPL','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',99.64,411,1072,97.66,'Sun-Synch' from dual union all
select 31984,'031984','1999-025CPM','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',101.66,788,887,98.78,'Sun-Synch' from dual union all
select 31985,'031985','1999-025CPN','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',101.69,718,960,98.13,'Sun-Synch' from dual union all
select 31986,'031986','1999-025CPP','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',102.23,769,960,99.78,'Sun-Synch' from dual union all
select 31987,'031987','1999-025CPQ','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',101.21,765,866,99.12,'Sun-Synch' from dual union all
select 31988,'031988','1999-025CPR','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',103.26,858,968,98.14,'Sun-Synch' from dual union all
select 31989,'031989','1999-025CPS','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',103.86,849,1033,98.98,'Sun-Synch' from dual union all
select 3199,'003199','1968-031A','OPS 5105','GAMBIT-3 Mission 4313',16758,'Reentered Attached','1968-04-29 00:00:00','1968-04-21 00:00:00',89.67,128,389,111.49,'Retrograde' from dual union all
select 31990,'031990','1999-025CPT','deb FY-1C','',66706,'In Earth orbit','','2007-07-30 00:00:00',106.27,857,1250,99.19,'Sun-Synch' from dual union all
select 31991,'031991','1999-025CPU','deb FY-1C','',66706,'In Earth orbit','','2007-07-30 00:00:00',105.88,852,1220,100.49,'Sun-Synch' from dual union all
select 31992,'031992','1999-025CPV','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',109.3,769,1620,99.16,'Sun-Synch' from dual union all
select 31993,'031993','1999-025CPW','deb FY-1C','',66706,'In Earth orbit','','2007-07-30 00:00:00',109.71,842,1585,99.19,'Sun-Synch' from dual union all
select 31994,'031994','1999-025CPX','deb FY-1C','',66706,'In Earth orbit','','2007-07-28 00:00:00',103.29,794,1035,98.57,'Sun-Synch' from dual union all
select 31995,'031995','1999-025CPY','deb FY-1C','',66706,'In Earth orbit','','2007-07-28 00:00:00',104.46,853,1085,98.4,'Sun-Synch' from dual union all
select 31996,'031996','1999-025CPZ','deb FY-1C','',66706,'In Earth orbit','','2007-07-30 00:00:00',107.5,851,1371,99.26,'Sun-Synch' from dual union all
select 31997,'031997','1999-025CQA','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',107.12,847,1340,102.53,'Sun-Synch' from dual union all
select 31998,'031998','1999-025CQB','deb FY-1C','',66706,'In Earth orbit','','2007-07-31 00:00:00',101.14,687,939,105.57,'Retrograde' from dual union all
select 31999,'031999','1969-009C','deb Isis 1','',18276,'In Earth orbit','','2007-07-31 00:00:00',127.52,567,3461,88.43,'Medium' from dual union all
select 32,'000032','60 Delta','Discoverer 11','CORONA C-8',6319,'Reentered Attached','1960-04-26 00:00:00','1960-04-18 00:00:00',91.69,169,546,80.01,'Intermediate' from dual union all
select 320,'000320','61 Omicron 93','deb r/b Ablestar','',7273,'In Earth orbit','','1962-06-27 00:00:00',102.05,734,978,66.91,'Intermediate' from dual union all
select 3200,'003200','1968-025Q','deb Saturn S-IVB-502','',16752,'Reentered','1968-04-17 00:00:00','1968-04-16 00:00:00',89.58,173,335,32.57,'Intermediate' from dual union all
select 32000,'032000','2005-009B','Centaur AV-004','Centaur AV-004',68018,'Reentered','2014-07-01 00:00:00','2006-10-21 00:00:00',1827.12,1290,84969,16.36,'Very High' from dual union all
select 32001,'032001','2007-033A','Progress M-61','Progress 7K-TGM No. 361',68375,'Deorbited','2008-01-22 00:00:00','2007-08-10 00:00:00',91.34,335,346,51.63,'Intermediate' from dual union all
select 32002,'032002','2007-033B','11S510','11S510',68375,'Reentered','2007-08-06 00:00:00','2007-08-04 00:00:00',88.29,173,207,51.63,'Intermediate' from dual union all
select 32003,'032003','2007-034A','Phoenix','Phoenix Lander',68376,'Deep Space','2007-08-04 00:00:00','2007-08-04 00:00:00',271.33,192,40716,28.41,'Earth Escape' from dual union all
select 32004,'032004','2007-034B','Star 48B','Star 48B',68376,'Deep Space','2007-08-04 00:00:00','2007-08-04 00:00:00',271.33,192,40716,28.41,'Earth Escape' from dual union all
select 32005,'032005','2004-021H','deb 11S772','',67865,'In Earth orbit','','2007-08-08 00:00:00',105.24,833,1179,71.1,'Intermediate' from dual union all
select 32006,'032006','2004-021J','deb 11S772','',67865,'In Earth orbit','','2007-08-07 00:00:00',104.82,840,1132,71.1,'Intermediate' from dual union all
select 32007,'032007','2007-034C','Delta 325','Delta SSPS AJ-10-118K',68376,'Reentered','2010-06-06 00:00:00','2007-08-11 00:00:00',148.81,184,5664,25.89,'Medium' from dual union all
select 32008,'032008','2007-035A','Endeavour (STS-118)','OV-105',68377,'Landed','2007-08-21 00:00:00','2007-08-11 00:00:00',91.34,335,346,51.63,'Intermediate' from dual union all
select 32009,'032009','2006-037F','deb H-2A','',68209,'Reentered','2007-08-16 00:00:00','2007-08-11 00:00:00',91.84,337,393,97.25,'Sun-Synch' from dual union all
select 3201,'003201','1968-025R','deb Saturn S-IVB-502','',16752,'Reentered','1968-04-16 00:00:00','1968-04-15 00:00:00',89.43,218,274,32.56,'Intermediate' from dual union all
select 32010,'032010','2006-037G','deb H-2A','',68209,'Reentered','2007-08-05 00:00:00','2007-08-01 00:00:00',92.27,357,414,97.24,'Sun-Synch' from dual union all
select 32011,'032011','2006-037H','deb H-2A','',68209,'Reentered','2007-08-26 00:00:00','2007-08-01 00:00:00',93.47,403,486,97.23,'Sun-Synch' from dual union all
select 32012,'032012','2006-037J','deb H-2A','',68209,'Reentered','2007-08-14 00:00:00','2007-08-12 00:00:00',91.3,326,351,97.27,'Sun-Synch' from dual union all
select 32013,'032013','2006-037K','deb H-2A','',68209,'Reentered','2007-08-15 00:00:00','2007-07-30 00:00:00',93.13,398,458,97.24,'Sun-Synch' from dual union all
select 32014,'032014','2006-037L','deb H-2A','',68209,'Reentered','2007-08-07 00:00:00','2007-07-30 00:00:00',92.91,392,442,97.25,'Sun-Synch' from dual union all
select 32015,'032015','2006-037M','deb H-2A','',68209,'Reentered','2007-08-19 00:00:00','2007-08-01 00:00:00',93.69,429,482,97.24,'Sun-Synch' from dual union all
select 32016,'032016','2006-037N','deb H-2A','',68209,'Reentered','2007-08-24 00:00:00','2007-08-13 00:00:00',92.99,389,453,97.23,'Sun-Synch' from dual union all
select 32017,'032017','2006-037P','deb H-2A','',68209,'Reentered','2007-08-19 00:00:00','2007-08-12 00:00:00',92.89,382,451,97.25,'Sun-Synch' from dual union all
select 32018,'032018','2007-036A','Spaceway 3','Spaceway 3',68378,'In Earth orbit','','2007-11-14 00:00:00',1435.35,35093,36451,.07,'Drift GEO' from dual union all
select 32019,'032019','2007-036B','BSAT-3A','BSAT 3a',68378,'In Earth orbit','','2007-10-29 00:00:00',1436.04,35772,35799,.06,'Stationary' from dual union all
select 3202,'003202','1968-025S','deb Saturn S-IVB-502','',16752,'Reentered','1968-04-17 00:00:00','1968-04-16 00:00:00',89.53,232,271,32.58,'Intermediate' from dual union all
select 32020,'032020','2007-036C','Sylda 5A','Sylda 5',68378,'Reentered','2017-01-07 00:00:00','2007-08-15 00:00:00',631.62,259,35756,1.9,'GEO Transfer' from dual union all
select 32021,'032021','2007-036D','ESC-A 537','Ariane 5 ESC-A',68378,'In Earth orbit','','2007-08-15 00:00:00',632.85,268,35810,1.94,'GEO Transfer' from dual union all
select 32022,'032022','1997-070G','SOZ debris','',66150,'Reentered','2010-11-25 00:00:00','2007-08-15 00:00:00',259.51,874,13323,46.63,'Medium' from dual union all
select 32023,'032023','1997-070H','SOZ debris','',66150,'Reentered','2010-12-15 00:00:00','2007-08-15 00:00:00',259.09,865,13303,46.67,'Medium' from dual union all
select 32024,'032024','1997-070J','SOZ debris','',66150,'In Earth orbit','','2007-08-15 00:00:00',234.98,1222,11249,46.29,'Medium' from dual union all
select 32025,'032025','2006-006C','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-15 00:00:00',275.38,435,14850,51.68,'Highly Ellip' from dual union all
select 32026,'032026','2006-006D','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-15 00:00:00',263.84,516,13980,51.22,'Medium' from dual union all
select 32027,'032027','2006-006E','deb Briz-M 88515','',68178,'Reentered','2016-06-10 00:00:00','2007-08-15 00:00:00',265.26,548,14046,51.37,'Medium' from dual union all
select 32028,'032028','2006-006F','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-14 00:00:00',283.65,472,15372,51.77,'Highly Ellip' from dual union all
select 32029,'032029','2006-006G','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-14 00:00:00',276.41,410,14946,51.59,'Highly Ellip' from dual union all
select 3203,'003203','1968-032A','Kosmos-214','Zenit-4 No. 45',16759,'Landed','1968-04-26 00:00:00','1968-04-20 00:00:00',90.2,197,372,81.39,'Intermediate' from dual union all
select 32030,'032030','2006-006H','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-15 00:00:00',273.68,352,14818,51.78,'Highly Ellip' from dual union all
select 32031,'032031','2006-006J','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-15 00:00:00',279.3,563,14989,51.46,'Highly Ellip' from dual union all
select 32032,'032032','2006-006K','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-15 00:00:00',264.48,554,13987,51.63,'Medium' from dual union all
select 32033,'032033','2006-006L','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-14 00:00:00',276.14,444,14894,51.68,'Highly Ellip' from dual union all
select 32034,'032034','2006-006M','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-15 00:00:00',275.61,445,14856,51.69,'Highly Ellip' from dual union all
select 32035,'032035','2006-006N','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-15 00:00:00',273.77,565,14612,50.93,'Highly Ellip' from dual union all
select 32036,'032036','2006-006P','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-15 00:00:00',275.44,427,14863,51.71,'Highly Ellip' from dual union all
select 32037,'032037','2006-006Q','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-14 00:00:00',267.39,349,14391,51.65,'Highly Ellip' from dual union all
select 32038,'032038','2006-006R','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-15 00:00:00',275.8,407,14907,51.71,'Highly Ellip' from dual union all
select 32039,'032039','2006-006S','deb Briz-M 88515','',68178,'In Earth orbit','2008-01-01 00:00:00','2007-08-01 00:00:00',301.34,813,16208,50.6,'Highly Ellip' from dual union all
select 3204,'003204','1968-032B','11A57I','11A57I',16759,'Reentered','1968-04-29 00:00:00','1968-04-20 00:00:00',89.91,195,345,81.37,'Intermediate' from dual union all
select 32040,'032040','2006-006T','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-14 00:00:00',273.54,378,14782,51.65,'Highly Ellip' from dual union all
select 32041,'032041','2006-006U','deb Briz-M 88515','',68178,'In Earth orbit','','2007-07-30 00:00:00',266.57,339,14345,51.83,'Highly Ellip' from dual union all
select 32042,'032042','2006-006V','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-15 00:00:00',274.96,407,14850,51.7,'Highly Ellip' from dual union all
select 32043,'032043','2006-006W','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-15 00:00:00',277.75,465,14981,51.63,'Highly Ellip' from dual union all
select 32044,'032044','2006-006X','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-15 00:00:00',301.66,905,16136,51.49,'Highly Ellip' from dual union all
select 32045,'032045','2006-006Y','deb Briz-M 88515','',68178,'Reentered','2008-01-04 00:00:00','2007-08-15 00:00:00',291.33,561,15798,52.34,'Highly Ellip' from dual union all
select 32046,'032046','2006-006Z','deb Briz-M 88515','',68178,'Reentered','2008-03-02 00:00:00','2007-08-14 00:00:00',271.09,505,14488,50.64,'Highly Ellip' from dual union all
select 32047,'032047','2006-006AA','deb Briz-M 88515','',68178,'In Earth orbit','','2007-08-15 00:00:00',264.64,571,13980,51.44,'Medium' from dual;
insert into SATELLITE
select 32048,'032048','2007-022D','Kosmos-2427 solar panel','Kobal''t-M part',68364,'Reentered','2007-08-23 00:00:00','2007-08-23 00:00:00',88.56,155,252,67.13,'Intermediate' from dual union all
select 32049,'032049','2007-022E','Kosmos-2427 solar panel','Kobal''t-M part',68364,'Reentered','2007-08-24 00:00:00','2007-08-23 00:00:00',88.77,211,216,67.13,'Intermediate' from dual union all
select 3205,'003205','1968-033A','Kosmos-215','DS-U1-A No. 1',16760,'Reentered','1968-06-30 00:00:00','1968-05-19 00:00:00',90.49,248,349,48.45,'Intermediate' from dual union all
select 32050,'032050','2007-037A','INSAT 4CR','INSAT 4CR',68379,'In Earth orbit','','2007-09-14 00:00:00',1428.12,35026,36235,.15,'Drift GEO' from dual union all
select 32051,'032051','2007-037B','GSLV CS-5','GSLV-F04 12KRB stage',68379,'Reentered','2011-02-08 00:00:00','2007-09-14 00:00:00',553.54,177,31731,15.84,'Highly Ellip' from dual union all
select 32052,'032052','2007-038A','Kosmos-2429','Parus',68380,'In Earth orbit','','2007-09-19 00:00:00',104.74,955,1010,82.98,'Intermediate' from dual union all
select 32053,'032053','2007-038B','S3M','S3M',68380,'In Earth orbit','','2007-09-18 00:00:00',104.62,955,999,82.98,'Intermediate' from dual union all
select 32054,'032054','2007-039A','Kaguya','SELENE',68381,'Deep Space','2007-09-14 00:00:00','2007-09-29 00:00:00',7109.28,281,231910,29.9,'Deep Space' from dual union all
select 32055,'032055','2007-039B','Okina','RSAT',68381,'Deep Space Attached','2007-09-14 00:00:00','2007-09-29 00:00:00',7109.28,281,231910,29.9,'Deep Space' from dual union all
select 32056,'032056','2007-039C','Ouna','VRAD',68381,'Deep Space Attached','2007-09-14 00:00:00','2007-09-29 00:00:00',7109.28,281,231910,29.9,'Deep Space' from dual union all
select 32057,'032057','2007-039D','H-2A F13 Stage 2','H-2A Stage 2',68381,'Deep Space','2007-09-14 00:00:00','2007-09-14 00:00:00',7148.24,280,232805,29.9,'Deep Space' from dual union all
select 32058,'032058','2007-040A','Foton-M No. 3','Foton 34KS No. 15',68382,'Landed','2007-09-26 00:00:00','2007-09-16 00:00:00',89.9,259,280,62.94,'Intermediate' from dual union all
select 32059,'032059','2007-040B','11S510','11S510',68382,'Reentered','2007-10-14 00:00:00','2007-09-21 00:00:00',89.69,252,267,62.94,'Intermediate' from dual union all
select 3206,'003206','1968-033B','S1M','S1M',16760,'Reentered','1968-05-19 00:00:00','1968-04-26 00:00:00',90.85,258,374,48.41,'Intermediate' from dual union all
select 32060,'032060','2007-041A','WorldView-1','WorldView-1',68383,'In Earth orbit','','2007-09-25 00:00:00',94.48,493,493,97.5,'Sun-Synch' from dual union all
select 32061,'032061','2007-041B','Delta 326','Delta SSPS AJ-10-118K',68383,'Reentered','2007-10-28 00:00:00','2007-09-25 00:00:00',91.1,184,472,112.32,'Retrograde' from dual union all
select 32062,'032062','2007-042A','Zi Yuan 1-2B','ziyuan yihao 02B weixing',68384,'In Earth orbit','','2007-09-26 00:00:00',100.31,773,774,98.55,'Sun-Synch' from dual union all
select 32063,'032063','2007-042B','CZ-4B Stage 3','CZ-4B Stage 3',68384,'In Earth orbit','','2007-09-26 00:00:00',99.37,689,768,98.41,'Sun-Synch' from dual union all
select 32064,'032064','1999-057QC','ZY-1 debris','',66738,'Reentered','2015-01-05 00:00:00','2007-09-20 00:00:00',99.9,730,778,98.25,'Sun-Synch' from dual union all
select 32065,'032065','1999-057QD','ZY-1 debris','',66738,'In Earth orbit','','2007-09-23 00:00:00',100.42,773,783,98.21,'Sun-Synch' from dual union all
select 32066,'032066','1999-057QE','ZY-1 debris','',66738,'In Earth orbit','','2007-09-21 00:00:00',100.26,764,778,98.21,'Sun-Synch' from dual union all
select 32067,'032067','1999-057QF','ZY-1 debris','',66738,'In Earth orbit','','2007-09-23 00:00:00',100.34,772,778,98.23,'Sun-Synch' from dual union all
select 32068,'032068','1999-057QG','ZY-1 debris','',66738,'In Earth orbit','','2007-09-22 00:00:00',100.37,771,781,98.22,'Sun-Synch' from dual union all
select 32069,'032069','1999-057QH','ZY-1 debris','',66738,'In Earth orbit','','2007-09-22 00:00:00',100.22,760,778,98.2,'Sun-Synch' from dual union all
select 3207,'003207','1968-034A','Kosmos-216','Zenit-2 No. 62',16761,'Landed','1968-04-28 00:00:00','1968-04-23 00:00:00',89.1,195,265,51.82,'Intermediate' from dual union all
select 32070,'032070','1999-057QJ','ZY-1 debris','',66738,'In Earth orbit','','2007-09-23 00:00:00',100.38,770,784,98.2,'Sun-Synch' from dual union all
select 32071,'032071','1999-057QK','ZY-1 debris','',66738,'In Earth orbit','','2007-09-22 00:00:00',100.22,761,777,98.2,'Sun-Synch' from dual union all
select 32072,'032072','1999-057QL','ZY-1 debris','',66738,'In Earth orbit','','2007-09-23 00:00:00',100.32,767,781,98.2,'Sun-Synch' from dual union all
select 32073,'032073','1999-057QM','ZY-1 debris','',66738,'Reentered','2014-10-15 00:00:00','2007-09-21 00:00:00',100.16,761,772,98.22,'Sun-Synch' from dual union all
select 32074,'032074','1999-057QN','ZY-1 debris','',66738,'Reentered','2016-04-21 00:00:00','2007-09-21 00:00:00',100.05,752,770,98.2,'Sun-Synch' from dual union all
select 32075,'032075','1999-057QP','ZY-1 debris','',66738,'In Earth orbit','','2007-09-23 00:00:00',100.07,751,773,98.21,'Sun-Synch' from dual union all
select 32076,'032076','1999-057QQ','ZY-1 debris','',66738,'Reentered','2015-07-30 00:00:00','2007-09-23 00:00:00',100.47,772,790,98.21,'Sun-Synch' from dual union all
select 32077,'032077','1999-057QR','ZY-1 debris','',66738,'In Earth orbit','','2007-09-23 00:00:00',100.34,772,778,98.21,'Sun-Synch' from dual union all
select 32078,'032078','1999-057QS','ZY-1 debris','',66738,'Reentered','2015-05-17 00:00:00','2007-09-22 00:00:00',100.06,751,772,98.22,'Sun-Synch' from dual union all
select 32079,'032079','1999-057QT','ZY-1 debris','',66738,'In Earth orbit','','2007-09-23 00:00:00',100.37,772,781,98.2,'Sun-Synch' from dual union all
select 3208,'003208','1968-034B','11A57I','11A57I',16761,'Reentered','1968-04-23 00:00:00','1968-04-22 00:00:00',87.98,157,191,51.81,'Intermediate' from dual union all
select 32080,'032080','1999-057QU','ZY-1 debris','',66738,'In Earth orbit','','2007-09-23 00:00:00',100.26,763,778,98.22,'Sun-Synch' from dual union all
select 32081,'032081','1999-057QV','ZY-1 debris','',66738,'In Earth orbit','','2007-09-23 00:00:00',100.66,767,813,98.2,'Sun-Synch' from dual union all
select 32082,'032082','1999-057QW','ZY-1 debris','',66738,'In Earth orbit','','2007-09-22 00:00:00',100.15,758,774,98.24,'Sun-Synch' from dual union all
select 32083,'032083','1999-057QX','ZY-1 debris','',66738,'Reentered','2015-04-09 00:00:00','2007-09-23 00:00:00',100.15,755,776,98.2,'Sun-Synch' from dual union all
select 32084,'032084','1999-057QY','ZY-1 debris','',66738,'In Earth orbit','','2007-09-19 00:00:00',100.28,770,774,98.21,'Sun-Synch' from dual union all
select 32085,'032085','1999-057QZ','ZY-1 debris','',66738,'In Earth orbit','','2007-09-20 00:00:00',100.21,763,773,98.24,'Sun-Synch' from dual union all
select 32086,'032086','1999-025CQC','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',106.71,818,1331,99.27,'Sun-Synch' from dual union all
select 32087,'032087','1999-025CQD','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.26,662,975,98.53,'Sun-Synch' from dual union all
select 32088,'032088','1999-025CQE','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',109.81,857,1579,102.47,'Sun-Synch' from dual union all
select 32089,'032089','1999-025CQF','deb FY-1C','',66706,'In Earth orbit','','2007-09-20 00:00:00',102.54,785,973,99.1,'Sun-Synch' from dual union all
select 3209,'003209','1968-035A','Molniya-1','Molniya-1 11F67 No. 10',16762,'Reentered','1974-01-29 00:00:00','1968-05-22 00:00:00',713.1,390,39734,64.85,'GEO Transfer' from dual union all
select 32090,'032090','1999-025CQG','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',102.5,818,936,99.1,'Sun-Synch' from dual union all
select 32091,'032091','1999-025CQH','deb FY-1C','',66706,'Reentered','2011-11-24 00:00:00','2007-09-14 00:00:00',98.35,520,840,98.87,'Sun-Synch' from dual union all
select 32092,'032092','1999-025CQJ','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',102.58,853,908,99.02,'Sun-Synch' from dual union all
select 32093,'032093','1999-025CQK','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',100.91,734,870,99,'Sun-Synch' from dual union all
select 32094,'032094','1999-025CQL','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',102.46,849,901,99.07,'Sun-Synch' from dual union all
select 32095,'032095','1999-025CQM','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',102.04,796,915,98.8,'Sun-Synch' from dual union all
select 32096,'032096','1999-025CQN','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',104.41,857,1076,99.43,'Sun-Synch' from dual union all
select 32097,'032097','1999-025CQP','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',99.19,568,872,99.16,'Sun-Synch' from dual union all
select 32098,'032098','1999-025CQQ','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',101.87,693,1001,98.92,'Sun-Synch' from dual union all
select 32099,'032099','1999-025CQR','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',104.82,691,1281,98.31,'Sun-Synch' from dual union all
select 321,'000321','61 Omicron 94','deb r/b Ablestar','',7273,'Reentered','1969-02-26 00:00:00','1962-06-27 00:00:00',98.39,408,955,67.03,'Intermediate' from dual union all
select 3210,'003210','1968-035C','BOZ','BOZ',16762,'Reentered','1968-05-29 00:00:00','1968-04-29 00:00:00',91.26,242,431,64.92,'Intermediate' from dual union all
select 32100,'032100','1999-025CQS','deb FY-1C','',66706,'Reentered','2012-01-25 00:00:00','2007-09-23 00:00:00',96.8,365,846,99.14,'Sun-Synch' from dual union all
select 32101,'032101','1999-025CQT','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',104.35,853,1075,99.02,'Sun-Synch' from dual union all
select 32102,'032102','1999-025CQU','deb FY-1C','',66706,'In Earth orbit','','2007-09-19 00:00:00',103.2,646,1174,99.38,'Sun-Synch' from dual union all
select 32103,'032103','1999-025CQV','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',100.83,698,898,96.73,'Sun-Synch' from dual union all
select 32104,'032104','1999-025CQW','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',105.86,762,1307,97.66,'Sun-Synch' from dual union all
select 32105,'032105','1999-025CQX','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',109.02,830,1533,103.48,'Sun-Synch' from dual union all
select 32106,'032106','1999-025CQY','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',104.15,824,1086,101.5,'Sun-Synch' from dual union all
select 32107,'032107','1999-025CQZ','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',102.92,841,953,99.02,'Sun-Synch' from dual union all
select 32108,'032108','1999-025CRA','deb FY-1C','',66706,'Reentered','2014-12-20 00:00:00','2007-09-23 00:00:00',100.35,654,897,99.09,'Sun-Synch' from dual union all
select 32109,'032109','1999-025CRB','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.26,646,991,98.33,'Sun-Synch' from dual union all
select 3211,'003211','1968-035B','11S510','11S510',16762,'Reentered','1968-05-11 00:00:00','1968-04-29 00:00:00',90.18,226,341,64.95,'Intermediate' from dual union all
select 32110,'032110','1999-025CRC','deb FY-1C','',66706,'In Earth orbit','','2007-09-21 00:00:00',102.44,846,903,99.6,'Sun-Synch' from dual union all
select 32111,'032111','1999-025CRD','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',100.95,712,896,98.66,'Sun-Synch' from dual union all
select 32112,'032112','1999-025CRE','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',100.81,588,1005,98.69,'Sun-Synch' from dual union all
select 32113,'032113','1999-025CRF','deb FY-1C','',66706,'In Earth orbit','','2007-09-21 00:00:00',104.05,758,1142,99.19,'Sun-Synch' from dual union all
select 32114,'032114','1999-025CRG','deb FY-1C','',66706,'In Earth orbit','','2007-09-14 00:00:00',100.69,721,862,99.96,'Sun-Synch' from dual union all
select 32115,'032115','1999-025CRH','deb FY-1C','',66706,'In Earth orbit','','2007-09-17 00:00:00',102.49,851,902,98.84,'Sun-Synch' from dual union all
select 32116,'032116','1999-025CRJ','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',98.15,475,866,99.57,'Sun-Synch' from dual union all
select 32117,'032117','1999-025CRK','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',102.32,846,891,99.07,'Sun-Synch' from dual union all
select 32118,'032118','1999-025CRL','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.24,766,868,102.42,'Retrograde' from dual union all
select 32119,'032119','1999-025CRM','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',103.42,851,989,99.93,'Sun-Synch' from dual union all
select 3212,'003212','1968-026D','OV1-13P','FW4S',16753,'In Earth orbit','','1968-05-10 00:00:00',199.6,556,9315,99.98,'Medium' from dual union all
select 32120,'032120','1999-025CRN','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',99.98,646,869,98.96,'Sun-Synch' from dual union all
select 32121,'032121','1999-025CRP','deb FY-1C','',66706,'Reentered','2012-12-29 00:00:00','2007-09-20 00:00:00',101.89,785,912,98.81,'Sun-Synch' from dual union all
select 32122,'032122','1999-025CRQ','deb FY-1C','',66706,'Reentered','2008-02-07 00:00:00','2007-09-23 00:00:00',94.96,385,649,98.91,'Sun-Synch' from dual union all
select 32123,'032123','1999-025CRR','deb FY-1C','',66706,'In Earth orbit','','2007-09-14 00:00:00',99.26,591,856,98.97,'Sun-Synch' from dual union all
select 32124,'032124','1999-025CRS','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',99.01,549,874,99.05,'Sun-Synch' from dual union all
select 32125,'032125','1999-025CRT','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',102.1,602,1115,99.18,'Sun-Synch' from dual union all
select 32126,'032126','1999-025CRU','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',106.85,858,1303,99.08,'Sun-Synch' from dual union all
select 32127,'032127','1999-025CRV','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',107.09,857,1327,99.23,'Sun-Synch' from dual union all
select 32128,'032128','1999-025CRW','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',104.22,635,1281,99.15,'Sun-Synch' from dual union all
select 32129,'032129','1999-025CRX','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',103.28,840,988,99.06,'Sun-Synch' from dual union all
select 3213,'003213','1968-012C','NNS O-18 despin weight','',16739,'In Earth orbit','','1968-04-19 00:00:00',104.96,952,1034,89.8,'Polar' from dual union all
select 32130,'032130','1999-025CRY','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',100.88,731,869,98.97,'Sun-Synch' from dual union all
select 32131,'032131','1999-025CRZ','deb FY-1C','',66706,'Reentered','2011-09-16 00:00:00','2007-09-23 00:00:00',105.43,649,1381,100.01,'Sun-Synch' from dual union all
select 32132,'032132','1999-025CSA','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.55,751,913,98.82,'Sun-Synch' from dual union all
select 32133,'032133','1999-025CSB','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.77,815,870,99.12,'Sun-Synch' from dual union all
select 32134,'032134','1999-025CSC','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',102.31,649,1086,98.97,'Sun-Synch' from dual union all
select 32135,'032135','1999-025CSD','deb FY-1C','',66706,'Reentered','2014-11-08 00:00:00','2007-09-23 00:00:00',100.24,686,854,99.03,'Sun-Synch' from dual union all
select 32136,'032136','1999-025CSE','deb FY-1C','',66706,'In Earth orbit','','2007-09-21 00:00:00',100.91,746,858,98.87,'Sun-Synch' from dual union all
select 32137,'032137','1999-025CSF','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',100.64,539,1038,98.38,'Sun-Synch' from dual;
insert into SATELLITE
select 32138,'032138','1999-025CSG','deb FY-1C','',66706,'In Earth orbit','','2007-09-21 00:00:00',102.14,842,877,99.09,'Sun-Synch' from dual union all
select 32139,'032139','1999-025CSH','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',101.53,646,1017,98.8,'Sun-Synch' from dual union all
select 3214,'003214','1968-012D','NNS O-18 despin weight','Transit high debris',16739,'In Earth orbit','','1968-04-19 00:00:00',108.84,1024,1323,90.21,'Polar' from dual union all
select 32140,'032140','1999-025CSJ','deb FY-1C','',66706,'Reentered','2013-02-19 00:00:00','2007-09-20 00:00:00',98.2,505,841,99.74,'Sun-Synch' from dual union all
select 32141,'032141','1999-025CSK','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',103.12,856,957,99.31,'Sun-Synch' from dual union all
select 32142,'032142','1999-025CSL','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',102.89,816,975,98.92,'Sun-Synch' from dual union all
select 32143,'032143','1999-025CSM','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.78,764,922,99.03,'Sun-Synch' from dual union all
select 32144,'032144','1999-025CSN','deb FY-1C','',66706,'Reentered','2011-08-16 00:00:00','2007-09-24 00:00:00',130.34,816,3459,100.09,'Medium' from dual union all
select 32145,'032145','1999-025CSP','deb FY-1C','',66706,'Reentered','2012-06-16 00:00:00','2007-09-24 00:00:00',99.84,534,968,97.05,'Sun-Synch' from dual union all
select 32146,'032146','1999-025CSQ','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',102.23,760,969,99.03,'Sun-Synch' from dual union all
select 32147,'032147','1999-025CSR','deb FY-1C','',66706,'Lost','','2007-09-23 00:00:00',98.2,488,858,99.07,'Sun-Synch' from dual union all
select 32148,'032148','1999-025CSS','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',99.12,537,897,99.14,'Sun-Synch' from dual union all
select 32149,'032149','1999-025CST','deb FY-1C','',66706,'Reentered','2014-12-03 00:00:00','2007-09-22 00:00:00',100.34,651,899,99.11,'Sun-Synch' from dual union all
select 3215,'003215','1968-017C','deb Explorer 37','',16744,'Reentered','1969-04-11 00:00:00','1968-04-20 00:00:00',96.83,367,847,59.2,'Intermediate' from dual union all
select 32150,'032150','1999-025CSU','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',99.23,583,861,99.21,'Sun-Synch' from dual union all
select 32151,'032151','1999-025CSV','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',100.51,710,856,99.06,'Sun-Synch' from dual union all
select 32152,'032152','1999-025CSW','deb FY-1C','',66706,'In Earth orbit','','2007-09-20 00:00:00',101.03,743,872,98.9,'Sun-Synch' from dual union all
select 32153,'032153','1999-025CSX','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',100.06,593,930,98.82,'Sun-Synch' from dual union all
select 32154,'032154','1999-025CSY','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',100.41,655,901,99.11,'Sun-Synch' from dual union all
select 32155,'032155','1999-025CSZ','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.17,756,873,99.12,'Sun-Synch' from dual union all
select 32156,'032156','1999-025CTA','deb FY-1C','',66706,'In Earth orbit','','2007-09-21 00:00:00',101.11,745,878,99.38,'Sun-Synch' from dual union all
select 32157,'032157','1999-025CTB','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',99.13,574,861,99.08,'Sun-Synch' from dual union all
select 32158,'032158','1999-025CTC','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',101.2,770,861,99.2,'Sun-Synch' from dual union all
select 32159,'032159','1999-025CTD','deb FY-1C','',66706,'Reentered','2014-01-18 00:00:00','2007-09-23 00:00:00',98.62,535,851,99.02,'Sun-Synch' from dual union all
select 3216,'003216','1968-036A','Kosmos-217','I2M',16763,'Reentered','1968-04-26 00:00:00','1968-04-24 00:00:00',88.55,144,262,62.2,'Intermediate' from dual union all
select 32160,'032160','1999-025CTE','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',100.95,734,874,98.98,'Sun-Synch' from dual union all
select 32161,'032161','1999-025CTF','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',103.02,792,1011,98.63,'Sun-Synch' from dual union all
select 32162,'032162','1999-025CTG','deb FY-1C','',66706,'In Earth orbit','','2007-09-21 00:00:00',100.89,742,860,99.51,'Sun-Synch' from dual union all
select 32163,'032163','1999-025CTH','deb FY-1C','',66706,'In Earth orbit','','2007-09-21 00:00:00',101.14,764,862,99.57,'Sun-Synch' from dual union all
select 32164,'032164','1999-025CTJ','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',99.95,656,857,99.39,'Sun-Synch' from dual union all
select 32165,'032165','1999-025CTK','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',103.98,825,1069,98.57,'Sun-Synch' from dual union all
select 32166,'032166','1999-025CTL','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',98.91,528,886,98.41,'Sun-Synch' from dual union all
select 32167,'032167','1999-025CTM','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.16,683,945,98.75,'Sun-Synch' from dual union all
select 32168,'032168','1999-025CTN','deb FY-1C','',66706,'In Earth orbit','','2007-09-21 00:00:00',105.95,801,1277,101.67,'Sun-Synch' from dual union all
select 32169,'032169','1999-025CTP','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',102.67,822,948,98.57,'Sun-Synch' from dual union all
select 3217,'003217','1968-037A','Kosmos-218','OGCh No. 17LU',16764,'Deorbited','1968-04-25 00:00:00','1968-04-25 00:00:00',87.33,123,162,49.56,'Intermediate' from dual union all
select 32170,'032170','1999-025CTQ','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',100.28,666,878,99.15,'Sun-Synch' from dual union all
select 32171,'032171','1999-025CTR','deb FY-1C','',66706,'In Earth orbit','','2007-09-16 00:00:00',109.45,784,1618,102.82,'Sun-Synch' from dual union all
select 32172,'032172','1999-025CTS','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',99.18,582,857,99.12,'Sun-Synch' from dual union all
select 32173,'032173','1999-025CTT','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',102.18,800,924,99.63,'Sun-Synch' from dual union all
select 32174,'032174','1999-025CTU','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.99,732,973,99.17,'Sun-Synch' from dual union all
select 32175,'032175','1999-025CTV','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',104.99,856,1131,98.84,'Sun-Synch' from dual union all
select 32176,'032176','1999-025CTW','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',100.92,748,857,99.45,'Sun-Synch' from dual union all
select 32177,'032177','1999-025CTX','deb FY-1C','',66706,'In Earth orbit','','2007-09-19 00:00:00',101.12,751,872,99.11,'Sun-Synch' from dual union all
select 32178,'032178','1999-025CTY','deb FY-1C','',66706,'In Earth orbit','','2007-09-16 00:00:00',102.44,794,955,100.31,'Sun-Synch' from dual union all
select 32179,'032179','1999-025CTZ','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',102.81,785,999,98.82,'Sun-Synch' from dual union all
select 3218,'003218','1968-037B','8S692','8S692',16764,'Reentered','1968-04-25 00:00:00','1968-04-25 00:00:00',87.34,125,160,49.57,'Intermediate' from dual union all
select 32180,'032180','1999-025CUA','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',104.25,859,1060,98.99,'Sun-Synch' from dual union all
select 32181,'032181','1999-025CUB','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',100.52,707,860,99.53,'Sun-Synch' from dual union all
select 32182,'032182','1999-025CUC','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',103.02,792,1011,98.23,'Sun-Synch' from dual union all
select 32183,'032183','1999-025CUD','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',102.75,855,922,99.28,'Sun-Synch' from dual union all
select 32184,'032184','1999-025CUE','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',99.68,503,983,98.38,'Sun-Synch' from dual union all
select 32185,'032185','1999-025CUF','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',104.73,526,1438,98.62,'Sun-Synch' from dual union all
select 32186,'032186','1999-025CUG','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',105.26,789,1225,99.25,'Sun-Synch' from dual union all
select 32187,'032187','1999-025CUH','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',106.86,858,1305,99.38,'Sun-Synch' from dual union all
select 32188,'032188','1999-025CUJ','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',102.68,818,952,98.97,'Sun-Synch' from dual union all
select 32189,'032189','1999-025CUK','deb FY-1C','',66706,'In Earth orbit','','2007-09-14 00:00:00',102.8,861,922,98.9,'Sun-Synch' from dual union all
select 3219,'003219','1968-037C','Kosmos-218 Adapter','OGCh Adapter',16764,'Reentered','1968-04-25 00:00:00','1968-04-25 00:00:00',87.53,133,172,49.6,'Intermediate' from dual union all
select 32190,'032190','1999-025CUL','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',100.97,743,867,99.03,'Sun-Synch' from dual union all
select 32191,'032191','1999-025CUM','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',102.57,792,969,98.96,'Sun-Synch' from dual union all
select 32192,'032192','1999-025CUN','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',104.43,830,1106,98.64,'Sun-Synch' from dual union all
select 32193,'032193','1999-025CUP','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',102.97,801,998,98.48,'Sun-Synch' from dual union all
select 32194,'032194','1999-025CUQ','deb FY-1C','',66706,'In Earth orbit','','2007-09-13 00:00:00',100.68,718,864,99.15,'Sun-Synch' from dual union all
select 32195,'032195','1999-025CUR','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',102.75,828,949,99.05,'Sun-Synch' from dual union all
select 32196,'032196','1999-025CUS','deb FY-1C','',66706,'Reentered','2012-07-09 00:00:00','2007-09-23 00:00:00',99.09,579,852,103.2,'Retrograde' from dual union all
select 32197,'032197','1999-025CUT','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.14,681,944,99.09,'Sun-Synch' from dual union all
select 32198,'032198','1999-025CUU','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',103.77,858,1016,98.34,'Sun-Synch' from dual union all
select 32199,'032199','1999-025CUV','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',101.13,675,950,99.21,'Sun-Synch' from dual union all
select 322,'000322','61 Omicron 95','deb r/b Ablestar','',7273,'Reentered','1969-06-06 00:00:00','1962-06-29 00:00:00',98.91,468,946,67.1,'Intermediate' from dual union all
select 3220,'003220','1968-038A','Kosmos-219','DS-U2-D No. 2',16765,'Reentered','1969-03-02 00:00:00','1968-05-28 00:00:00',103.63,214,1647,48.42,'Intermediate' from dual union all
select 32200,'032200','1999-025CUW','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',102.8,857,926,98.95,'Sun-Synch' from dual union all
select 32201,'032201','1999-025CUX','deb FY-1C','',66706,'In Earth orbit','','2007-09-21 00:00:00',103.24,800,1024,98.96,'Sun-Synch' from dual union all
select 32202,'032202','1999-025CUY','deb FY-1C','',66706,'In Earth orbit','','2007-09-21 00:00:00',103.76,822,1050,98.43,'Sun-Synch' from dual union all
select 32203,'032203','1999-025CUZ','deb FY-1C','',66706,'In Earth orbit','','2007-09-18 00:00:00',99.59,403,1075,97.06,'Sun-Synch' from dual union all
select 32204,'032204','1999-025CVA','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',103.52,764,1086,99.45,'Sun-Synch' from dual union all
select 32205,'032205','1999-025CVB','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',102.03,846,864,98.85,'Sun-Synch' from dual union all
select 32206,'032206','1999-025CVC','deb FY-1C','',66706,'Reentered','2014-11-13 00:00:00','2007-09-21 00:00:00',100.62,609,967,97.68,'Sun-Synch' from dual union all
select 32207,'032207','1999-025CVD','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',100.63,719,858,99.88,'Sun-Synch' from dual union all
select 32208,'032208','1999-025CVE','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.15,770,857,99,'Sun-Synch' from dual union all
select 32209,'032209','1999-025CVF','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',107.86,858,1397,99.39,'Sun-Synch' from dual union all
select 3221,'003221','1968-038B','S1M','S1M',16765,'Reentered','1969-02-24 00:00:00','1968-05-27 00:00:00',103.48,226,1621,48.44,'Intermediate' from dual union all
select 32210,'032210','1999-025CVG','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',103.16,849,968,99.02,'Sun-Synch' from dual union all
select 32211,'032211','1999-025CVH','deb FY-1C','',66706,'In Earth orbit','','2007-09-20 00:00:00',104.43,859,1077,98.62,'Sun-Synch' from dual union all
select 32212,'032212','1999-025CVJ','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',100.85,682,916,99.64,'Sun-Synch' from dual union all
select 32213,'032213','1999-025CVK','deb FY-1C','',66706,'In Earth orbit','','2007-09-21 00:00:00',103.16,774,1043,99.05,'Sun-Synch' from dual union all
select 32214,'032214','1999-025CVL','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',106.05,857,1230,99.06,'Sun-Synch' from dual union all
select 32215,'032215','1999-025CVM','deb FY-1C','',66706,'Reentered','2011-11-05 00:00:00','2007-09-13 00:00:00',99.11,476,957,97.66,'Sun-Synch' from dual union all
select 32216,'032216','1999-025CVN','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.58,787,880,99.59,'Sun-Synch' from dual union all
select 32217,'032217','1999-025CVP','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.5,787,872,99.19,'Sun-Synch' from dual union all
select 32218,'032218','1999-025CVQ','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',100.7,725,858,98.89,'Sun-Synch' from dual union all
select 32219,'032219','1999-025CVR','deb FY-1C','',66706,'In Earth orbit','','2007-09-21 00:00:00',101.44,765,889,98.81,'Sun-Synch' from dual union all
select 3222,'003222','1966-035F','Blok-L No. N103-39','Blok-L',13442,'Reentered','1973-06-18 00:00:00','1967-03-05 00:00:00',702.73,715,38895,65.39,'GEO Transfer' from dual union all
select 32220,'032220','1999-025CVS','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',103.9,767,1119,98.91,'Sun-Synch' from dual union all
select 32221,'032221','1999-025CVT','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',99.48,610,857,98.69,'Sun-Synch' from dual union all
select 32222,'032222','1999-025CVU','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',102.43,787,960,99.19,'Sun-Synch' from dual union all
select 32223,'032223','1999-025CVV','deb FY-1C','',66706,'In Earth orbit','','2007-09-20 00:00:00',102.85,850,936,99.1,'Sun-Synch' from dual union all
select 32224,'032224','1999-025CVW','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',100.9,741,862,99.1,'Sun-Synch' from dual union all
select 32225,'032225','1999-025CVX','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',102.19,809,915,98.58,'Sun-Synch' from dual union all
select 32226,'032226','1999-025CVY','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.1,758,864,99.5,'Sun-Synch' from dual union all
select 32227,'032227','1999-025CVZ','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',99.05,569,858,99.52,'Sun-Synch' from dual;
insert into SATELLITE
select 32228,'032228','1999-025CWA','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.02,719,895,98.59,'Sun-Synch' from dual union all
select 32229,'032229','1999-025CWB','deb FY-1C','',66706,'Reentered','2013-06-24 00:00:00','2007-09-23 00:00:00',98.64,524,864,98.98,'Sun-Synch' from dual union all
select 3223,'003223','1966-092D','Blok-L No. N103-40','Blok-L',13499,'Reentered','1969-06-05 00:00:00','1967-05-31 00:00:00',712.27,520,39562,64.68,'GEO Transfer' from dual union all
select 32230,'032230','1999-025CWC','deb FY-1C','',66706,'In Earth orbit','','2007-09-17 00:00:00',102.22,790,938,99,'Sun-Synch' from dual union all
select 32231,'032231','1999-025CWD','deb FY-1C','',66706,'Reentered','2012-08-09 00:00:00','2007-09-23 00:00:00',98.3,370,985,99.25,'Sun-Synch' from dual union all
select 32232,'032232','1999-025CWE','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',101.96,706,996,97.09,'Sun-Synch' from dual union all
select 32233,'032233','1999-025CWF','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',103.8,722,1155,97.68,'Sun-Synch' from dual union all
select 32234,'032234','1999-025CWG','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',104.33,850,1077,98.56,'Sun-Synch' from dual union all
select 32235,'032235','1999-025CWH','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',99.72,628,863,99.09,'Sun-Synch' from dual union all
select 32236,'032236','1999-025CWJ','deb FY-1C','',66706,'Reentered','2015-04-19 00:00:00','2007-09-23 00:00:00',99.53,619,854,99.52,'Sun-Synch' from dual union all
select 32237,'032237','1999-025CWK','deb FY-1C','',66706,'Reentered','2015-06-22 00:00:00','2007-09-23 00:00:00',98.11,448,888,100.14,'Sun-Synch' from dual union all
select 32238,'032238','1999-025CWL','deb FY-1C','',66706,'In Earth orbit','','2007-09-23 00:00:00',98.51,501,873,98.93,'Sun-Synch' from dual union all
select 32239,'032239','1999-025CWM','deb FY-1C','',66706,'Reentered','2014-06-04 00:00:00','2007-09-15 00:00:00',100.81,592,1001,97,'Sun-Synch' from dual union all
select 3224,'003224','1967-052F','Blok-L','Blok-L',15307,'Reentered','1971-12-16 00:00:00','1967-07-28 00:00:00',709.49,932,39013,64.89,'GEO Transfer' from dual union all
select 32240,'032240','1999-025CWN','deb FY-1C','',66706,'Reentered','2015-01-22 00:00:00','2007-09-14 00:00:00',100.97,623,987,97.84,'Sun-Synch' from dual union all
select 32241,'032241','1999-025CWP','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',102.9,746,1045,98.72,'Sun-Synch' from dual union all
select 32242,'032242','1999-025CWQ','deb FY-1C','',66706,'In Earth orbit','','2007-09-22 00:00:00',104.91,858,1123,97.28,'Sun-Synch' from dual union all
select 32243,'032243','1999-025CWR','deb FY-1C','',66706,'In Earth orbit','','2007-09-24 00:00:00',104.65,721,1235,98.3,'Sun-Synch' from dual union all
select 32244,'032244','1999-025CWS','deb FY-1C','',66706,'Reentered','2014-05-08 00:00:00','2007-09-22 00:00:00',99.55,480,995,97,'Sun-Synch' from dual union all
select 32245,'032245','1999-025CWT','deb FY-1C','',66706,'In Earth orbit','','2007-09-13 00:00:00',100.28,645,898,98.78,'Sun-Synch' from dual union all
select 32246,'032246','2007-040C','KB electropitaniya','KBE',68382,'Reentered','2007-12-02 00:00:00','2007-09-26 00:00:00',89.82,257,275,62.94,'Intermediate' from dual union all
select 32247,'032247','2007-040D','PTDU cover','PTDU cover',68382,'Reentered','2007-10-23 00:00:00','2007-09-26 00:00:00',91.87,265,468,62.94,'Intermediate' from dual union all
select 32248,'032248','2007-040E','PTDU cover','PTDU cover',68382,'Reentered','2007-10-21 00:00:00','2007-09-26 00:00:00',92.36,265,516,62.94,'Intermediate' from dual union all
select 32249,'032249','2007-043A','Dawn','Dawn',68385,'Deep Space','2007-09-27 00:00:00','2007-09-27 00:00:00',1096.68,208,83419,28.9,'Earth Escape' from dual union all
select 3225,'003225','1967-095D','Blok-L No. Ya716-83','Blok-L',15350,'Reentered','1969-02-10 00:00:00','1967-11-30 00:00:00',709.69,460,39495,64.17,'GEO Transfer' from dual union all
select 32250,'032250','2007-043B','Star 48B','Star 48B',68385,'Deep Space','2007-09-27 00:00:00','2007-09-27 00:00:00',1096.68,208,83419,28.9,'Earth Escape' from dual union all
select 32251,'032251','2007-043C','Delta 327','Delta SSPS AJ-10-118K',68385,'Reentered','2009-09-18 00:00:00','2007-09-29 00:00:00',161.94,165,6762,26.68,'Medium' from dual union all
select 32252,'032252','2007-044A','Optus D2','Optus D2',68386,'In Earth orbit','','2008-01-02 00:00:00',1436.09,35775,35798,.04,'Stationary' from dual union all
select 32253,'032253','2007-044B','Intelsat IS-11','Intelsat IS-11',68386,'In Earth orbit','','2007-11-05 00:00:00',1436.01,35783,35787,.05,'Stationary' from dual union all
select 32254,'032254','2007-044C','EPS 526','EPS+ 526',68386,'In Earth orbit','','2007-10-08 00:00:00',589.98,366,33481,3.92,'Highly Ellip' from dual union all
select 32255,'032255','2007-044D','Sylda 5','Sylda 5',68386,'In Earth orbit','','2007-10-08 00:00:00',637.23,595,35708,3.98,'GEO Transfer' from dual union all
select 32256,'032256','2007-045A','Soyuz TMA-11','Soyuz 11F732 No. 221',68387,'Landed','2008-04-19 00:00:00','2007-10-11 00:00:00',89.61,253,257,51.64,'Intermediate' from dual union all
select 32257,'032257','2007-045B','11S510','11S510',68387,'Reentered','2007-10-14 00:00:00','2007-10-11 00:00:00',88.53,187,216,51.61,'Intermediate' from dual union all
select 32258,'032258','2007-046A','WGS SV-1','Wideband Global Satcom 1',68388,'In Earth orbit','','2007-12-01 00:00:00',1436.01,35780,35790,.1,'Stationary' from dual union all
select 32259,'032259','2007-046B','Centaur AV-011','Centaur AV-011',68388,'Reentered','2008-03-21 00:00:00','2007-10-11 00:00:00',1328.93,477,66847,20.11,'Highly Ellip' from dual union all
select 3226,'003226','1967-101F','Blok-L No. Ya716-84','Blok-L',15356,'Reentered','1970-01-16 00:00:00','1967-11-23 00:00:00',711.73,428,39628,64.95,'GEO Transfer' from dual union all
select 32260,'032260','2007-047A','GPS IIR-17(M)','Navstar GPS SVN 55',68389,'In Earth orbit','','2007-10-27 00:00:00',717.94,20149,20214,54.85,'Medium' from dual union all
select 32261,'032261','2007-047B','Star 48B','Star 48B',68389,'In Earth orbit','','2007-10-25 00:00:00',355.66,204,20297,39.93,'Highly Ellip' from dual union all
select 32262,'032262','2007-047C','Delta 328','Delta SSPS AJ-10-118K',68389,'Reentered','2008-04-30 00:00:00','2007-10-25 00:00:00',98.79,199,1203,32.63,'Intermediate' from dual union all
select 32263,'032263','2007-048A','Globalstar M067','Globalstar',68390,'In Earth orbit','','2007-10-28 00:00:00',103.45,914,929,51.98,'Intermediate' from dual union all
select 32264,'032264','2007-048B','Globalstar M070','Globalstar',68390,'In Earth orbit','','2007-10-28 00:00:00',103.6,925,933,51.98,'Intermediate' from dual union all
select 32265,'032265','2007-048C','Globalstar M066','Globalstar',68390,'In Earth orbit','','2007-10-28 00:00:00',103.45,915,929,51.98,'Intermediate' from dual union all
select 32266,'032266','2007-048D','Globalstar M068','Globalstar',68390,'In Earth orbit','','2007-10-28 00:00:00',103.42,912,929,51.98,'Intermediate' from dual union all
select 32267,'032267','2007-048E','11S510','11S510',68390,'Reentered','2007-10-24 00:00:00','2007-10-23 00:00:00',87.84,162,174,51.89,'Intermediate' from dual union all
select 32268,'032268','2007-049A','Kosmos-2430','Oko US-KS',68391,'In Earth orbit','','2007-10-30 00:00:00',704.44,520,39175,62.87,'Molniya' from dual union all
select 32269,'032269','2007-049B','11S510','11S510',68391,'Reentered','2007-12-06 00:00:00','2007-10-30 00:00:00',91.5,192,505,62.85,'Intermediate' from dual union all
select 3227,'003227','1968-038C','deb Kosmos-219','',16765,'Reentered','1968-06-03 00:00:00','1968-04-30 00:00:00',103.22,222,1600,48.37,'Intermediate' from dual union all
select 32270,'032270','2007-049C','BOZ','BOZ',68391,'Reentered','2008-02-25 00:00:00','2007-10-30 00:00:00',92.04,228,522,62.81,'Intermediate' from dual union all
select 32271,'032271','2007-049D','Blok-2BL','Blok-2BL',68391,'In Earth orbit','','2007-10-31 00:00:00',700.74,519,38992,62.84,'Molniya' from dual union all
select 32272,'032272','2007-050A','Discovery (STS-120)','OV-103',68392,'Landed','2007-11-07 00:00:00','2007-10-30 00:00:00',91.36,340,342,51.64,'Intermediate' from dual union all
select 32273,'032273','2007-051A','Chang''e-1','chang''e yihao weixing',68393,'Deep Space','2007-11-01 00:00:00','2007-10-25 00:00:00',947.43,584,50568,31.11,'Highly Ellip' from dual union all
select 32274,'032274','2007-051B','CZ-3A Stage 3','CZ-3A Stage 3',68393,'Reentered','2008-01-28 00:00:00','2007-10-28 00:00:00',929.19,145,50184,30.78,'Highly Ellip' from dual union all
select 32275,'032275','2007-052A','Kosmos-2431','Uragan-M No. 718',68394,'In Earth orbit','','2007-10-28 00:00:00',675.63,19123,19132,64.88,'Medium' from dual union all
select 32276,'032276','2007-052B','Kosmos-2432','Uragan-M No. 719',68394,'In Earth orbit','','2007-10-30 00:00:00',675.63,19121,19134,64.89,'Medium' from dual union all
select 32277,'032277','2007-052C','Kosmos-2433','Uragan-M No. 720',68394,'In Earth orbit','','2007-10-28 00:00:00',674.8,19082,19131,64.88,'Medium' from dual union all
select 32278,'032278','2007-052D','Blok DM-2 No. 110L','11S861 No. 110L',68394,'In Earth orbit','','2007-10-30 00:00:00',674.78,19081,19131,64.88,'Medium' from dual union all
select 32279,'032279','2007-052E','8S812','8S812',68394,'Reentered','2007-10-26 00:00:00','2007-10-26 00:00:00',87.68,160,160,64.8,'Intermediate' from dual union all
select 3228,'003228','1968-039A','Fairing','Fairing',16766,'Reentered','1968-05-15 00:00:00','1968-05-15 00:00:00',88.55,153,253,83.05,'Intermediate' from dual union all
select 32280,'032280','2007-052F','SOZ','SOZ',68394,'In Earth orbit','','2007-10-31 00:00:00',340.2,399,19130,64.82,'Highly Ellip' from dual union all
select 32281,'032281','2007-052G','SOZ','SOZ',68394,'In Earth orbit','','2007-10-31 00:00:00',340.22,401,19130,64.93,'Highly Ellip' from dual union all
select 32282,'032282','2007-049E','deb BOZ','',68391,'Reentered','2007-12-07 00:00:00','2007-10-31 00:00:00',91.67,207,506,62.84,'Intermediate' from dual union all
select 32283,'032283','2007-053A','SAR-Lupe 3','SAR-Lupe 3',68395,'In Earth orbit','','2007-11-03 00:00:00',94.19,465,494,98.15,'Sun-Synch' from dual union all
select 32284,'032284','2007-053B','S3M','S3M',68395,'Reentered','2014-04-16 00:00:00','2007-11-03 00:00:00',94.26,466,500,98.15,'Sun-Synch' from dual union all
select 32285,'032285','1998-067BB','EVA debris','STS-120 EVA-4 debris',66503,'Reentered','2008-02-10 00:00:00','2007-11-06 00:00:00',91.33,338,342,51.64,'Intermediate' from dual union all
select 32286,'032286','1998-067BC','EVA debris (pliers)','STS-120 EVA-4 debris',66503,'Reentered','2008-02-02 00:00:00','2007-11-06 00:00:00',91.34,339,342,51.64,'Intermediate' from dual union all
select 32287,'032287','2007-054A','DSP F23','DSP F23',68396,'In Earth orbit','','2007-11-19 00:00:00',1439.97,35850,35875,4,'Inclined Drift' from dual union all
select 32288,'032288','2007-054B','Delta 329','DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',1439.97,35858,35866,3.4,'Inclined Drift' from dual union all
select 32289,'032289','2007-055A','Yaogan Weixing 3','Jianbing-5 02 xing',68397,'In Earth orbit','','2007-11-19 00:00:00',97.06,613,622,97.8,'Sun-Synch' from dual union all
select 3229,'003229','1968-040A','Kosmos-220','Tsiklon',16767,'In Earth orbit','','1968-06-06 00:00:00',99.07,675,754,74.04,'Intermediate' from dual union all
select 32290,'032290','2007-055B','CZ-4C Stage 3','CZ-4C Stage 3',68397,'In Earth orbit','','2007-11-19 00:00:00',95.37,457,615,97.85,'Sun-Synch' from dual union all
select 32291,'032291','1991-063C','deb UARS','',64232,'Reentered','2007-11-27 00:00:00','2007-11-15 00:00:00',91.7,266,450,56.95,'Intermediate' from dual union all
select 32292,'032292','1991-063D','deb UARS','',64232,'Reentered','2007-11-27 00:00:00','2007-11-15 00:00:00',92.53,343,454,56.98,'Intermediate' from dual union all
select 32293,'032293','2007-056A','Star One C1','Star One C1',68398,'In Earth orbit','','2007-12-13 00:00:00',1436.06,35777,35795,.03,'Stationary' from dual union all
select 32294,'032294','2007-056B','Skynet 5B','Skynet 5B',68398,'In Earth orbit','','2007-11-24 00:00:00',1432.67,35572,35867,.05,'Drift GEO' from dual union all
select 32295,'032295','2007-056C','ESC-A 538','Ariane 5 ESC-A',68398,'In Earth orbit','','2007-11-22 00:00:00',630.81,267,35706,5.95,'GEO Transfer' from dual union all
select 32296,'032296','2007-056D','Sylda 5A','Sylda 5',68398,'Reentered','2013-09-20 00:00:00','2007-11-17 00:00:00',629.16,251,35637,5.86,'GEO Transfer' from dual union all
select 32297,'032297','1991-063E','deb UARS','',64232,'Reentered','2008-11-20 00:00:00','2007-11-15 00:00:00',94.68,433,574,56.95,'Intermediate' from dual union all
select 32298,'032298','1991-063F','deb UARS','',64232,'Reentered','2008-11-13 00:00:00','2007-11-15 00:00:00',94.95,434,599,56.99,'Intermediate' from dual union all
select 32299,'032299','2007-057A','Sirius 4','Sirius 4',68399,'In Earth orbit','','2007-11-28 00:00:00',1434.27,35698,35804,.02,'Drift GEO' from dual union all
select 323,'000323','61 Omicron 96','deb r/b Ablestar','',7273,'In Earth orbit','','1962-06-27 00:00:00',103.6,862,995,66.85,'Intermediate' from dual union all
select 3230,'003230','1968-040B','S3M','S3M',16767,'In Earth orbit','','1968-06-07 00:00:00',99.12,676,758,74.05,'Intermediate' from dual union all
select 32300,'032300','2007-057B','Briz-M DTB','Briz-M torus tank',68399,'Reentered','2012-03-04 00:00:00','2007-11-20 00:00:00',632.61,391,35675,49.08,'GEO Transfer' from dual union all
select 32301,'032301','2007-057C','Briz-M No. 88523','Briz-M 88523',68399,'In Earth orbit','','2007-11-21 00:00:00',757.28,6896,35389,17.44,'Highly Ellip' from dual union all
select 32302,'032302','1998-067BD','EVA debris','ISS US EVA-12 debris',66503,'Reentered','2007-12-30 00:00:00','2007-11-25 00:00:00',91.23,334,336,51.64,'Intermediate' from dual union all
select 32303,'032303','1999-025CWU','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',103.85,830,1050,97.84,'Sun-Synch' from dual union all
select 32304,'032304','1999-025CWV','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',100.96,753,856,99.45,'Sun-Synch' from dual union all
select 32305,'032305','1999-025CWW','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',104.31,776,1149,98.02,'Sun-Synch' from dual union all
select 32306,'032306','1999-025CWX','deb FY-1C','',66706,'Reentered','2015-02-10 00:00:00','2007-11-23 00:00:00',97.92,466,852,98.85,'Sun-Synch' from dual union all
select 32307,'032307','1999-025CWY','deb FY-1C','',66706,'Reentered','2015-05-23 00:00:00','2007-11-30 00:00:00',100.85,610,988,98.24,'Sun-Synch' from dual union all
select 32308,'032308','1999-025CWZ','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',103.93,849,1040,97.78,'Sun-Synch' from dual union all
select 32309,'032309','1999-025CXA','deb FY-1C','',66706,'Reentered','2014-09-22 00:00:00','2007-11-30 00:00:00',100.14,656,874,99.98,'Sun-Synch' from dual union all
select 3231,'003231','1968-040C','deb Kosmos-220','',16767,'Reentered','1990-09-09 00:00:00','1968-06-20 00:00:00',99,668,754,74.1,'Intermediate' from dual union all
select 32310,'032310','1999-025CXB','deb FY-1C','',66706,'Reentered','2015-06-13 00:00:00','2007-11-30 00:00:00',100.57,681,891,99.12,'Sun-Synch' from dual union all
select 32311,'032311','1999-025CXC','deb FY-1C','',66706,'In Earth orbit','','2007-11-28 00:00:00',109.28,854,1533,101.75,'Sun-Synch' from dual union all
select 32312,'032312','1999-025CXD','deb FY-1C','',66706,'In Earth orbit','','2007-11-24 00:00:00',104.1,844,1060,98.74,'Sun-Synch' from dual union all
select 32313,'032313','1999-025CXE','deb FY-1C','',66706,'In Earth orbit','','2007-11-28 00:00:00',103.82,823,1056,99.32,'Sun-Synch' from dual union all
select 32314,'032314','1999-025CXF','deb FY-1C','',66706,'In Earth orbit','','2007-11-26 00:00:00',100.27,579,963,98.99,'Sun-Synch' from dual union all
select 32315,'032315','1999-025CXG','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',105.51,847,1190,100.83,'Sun-Synch' from dual union all
select 32316,'032316','1999-025CXH','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',101.19,773,857,99.57,'Sun-Synch' from dual union all
select 32317,'032317','1999-025CXJ','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',102.26,728,1004,98.02,'Sun-Synch' from dual;
insert into SATELLITE
select 32318,'032318','1999-025CXK','deb FY-1C','',66706,'In Earth orbit','','2007-11-29 00:00:00',100.92,735,870,97.37,'Sun-Synch' from dual union all
select 32319,'032319','1999-025CXL','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',99.15,575,861,99.19,'Sun-Synch' from dual union all
select 3232,'003232','1968-039B','OPS 1419','CORONA CR-3',16766,'Reentered Attached','1968-05-20 00:00:00','1968-05-16 00:00:00',88.56,152,255,83.02,'Intermediate' from dual union all
select 32320,'032320','1999-025CXM','deb FY-1C','',66706,'In Earth orbit','','2007-11-27 00:00:00',101.63,784,887,100.38,'Sun-Synch' from dual union all
select 32321,'032321','1999-025CXN','deb FY-1C','',66706,'Reentered','2014-01-02 00:00:00','2007-11-30 00:00:00',101.33,712,932,97.13,'Sun-Synch' from dual union all
select 32322,'032322','1999-025CXP','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',108.51,826,1489,98.38,'Sun-Synch' from dual union all
select 32323,'032323','1999-025CXQ','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',105.15,854,1149,98.57,'Sun-Synch' from dual union all
select 32324,'032324','1999-025CXR','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',102.71,848,926,99.63,'Sun-Synch' from dual union all
select 32325,'032325','1999-025CXS','deb FY-1C','',66706,'In Earth orbit','','2007-11-26 00:00:00',101.47,770,887,99.03,'Sun-Synch' from dual union all
select 32326,'032326','1999-025CXT','deb FY-1C','',66706,'In Earth orbit','','2007-11-28 00:00:00',105.77,727,1334,98.47,'Sun-Synch' from dual union all
select 32327,'032327','1999-025CXU','deb FY-1C','',66706,'Reentered','2009-12-27 00:00:00','2007-11-30 00:00:00',96.54,413,772,98.9,'Sun-Synch' from dual union all
select 32328,'032328','1999-025CXV','deb FY-1C','',66706,'Reentered','2013-11-08 00:00:00','2007-11-30 00:00:00',98.54,525,853,98.8,'Sun-Synch' from dual union all
select 32329,'032329','1999-025CXW','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',102.28,820,914,99.98,'Sun-Synch' from dual union all
select 3233,'003233','1968-041A','ESRO 2B','ESRO 2B',16768,'Reentered','1971-05-08 00:00:00','1968-06-22 00:00:00',98.81,329,1074,97.2,'Sun-Synch' from dual union all
select 32330,'032330','1999-025CXX','deb FY-1C','',66706,'In Earth orbit','','2007-11-24 00:00:00',103.83,791,1089,98.07,'Sun-Synch' from dual union all
select 32331,'032331','1999-025CXY','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',102.71,851,923,99.26,'Sun-Synch' from dual union all
select 32332,'032332','1999-025CXZ','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',104.31,718,1206,99.25,'Sun-Synch' from dual union all
select 32333,'032333','1999-025CYA','deb FY-1C','',66706,'In Earth orbit','','2007-11-22 00:00:00',104.24,853,1065,97.88,'Sun-Synch' from dual union all
select 32334,'032334','1999-025CYB','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',103.16,858,959,98.82,'Sun-Synch' from dual union all
select 32335,'032335','1999-025CYC','deb FY-1C','',66706,'Reentered','2012-06-18 00:00:00','2007-11-30 00:00:00',98.15,482,859,99,'Sun-Synch' from dual union all
select 32336,'032336','1999-025CYD','deb FY-1C','',66706,'In Earth orbit','','2007-11-28 00:00:00',100.44,671,888,99.13,'Sun-Synch' from dual union all
select 32337,'032337','1999-025CYE','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',107.46,854,1365,99.27,'Sun-Synch' from dual union all
select 32338,'032338','1999-025CYF','deb FY-1C','',66706,'In Earth orbit','','2007-11-28 00:00:00',101.28,760,879,99.02,'Sun-Synch' from dual union all
select 32339,'032339','1999-025CYG','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',101.78,802,883,98.76,'Sun-Synch' from dual union all
select 3234,'003234','1968-041B','FW4S S/N 2218-10','FW4S',16768,'Reentered','1970-03-30 00:00:00','1968-06-17 00:00:00',98.74,328,1069,97.18,'Sun-Synch' from dual union all
select 32340,'032340','1999-025CYH','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',101.14,758,868,99.07,'Sun-Synch' from dual union all
select 32341,'032341','1999-025CYJ','deb FY-1C','',66706,'Reentered','2012-06-20 00:00:00','2007-11-30 00:00:00',97.29,417,841,100.22,'Sun-Synch' from dual union all
select 32342,'032342','1999-025CYK','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',101.99,848,858,98.81,'Sun-Synch' from dual union all
select 32343,'032343','1999-025CYL','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',101.87,730,965,98.99,'Sun-Synch' from dual union all
select 32344,'032344','1999-025CYM','deb FY-1C','',66706,'In Earth orbit','','2007-11-26 00:00:00',98.84,538,869,100.31,'Sun-Synch' from dual union all
select 32345,'032345','1999-025CYN','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',101.2,623,1008,98.37,'Sun-Synch' from dual union all
select 32346,'032346','1999-025CYP','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',104.74,854,1111,97.85,'Sun-Synch' from dual union all
select 32347,'032347','1999-025CYQ','deb FY-1C','',66706,'Reentered','2012-03-09 00:00:00','2007-11-27 00:00:00',98.28,478,875,99.06,'Sun-Synch' from dual union all
select 32348,'032348','1999-025CYR','deb FY-1C','',66706,'Reentered','2012-08-25 00:00:00','2007-11-22 00:00:00',98.66,534,855,99.04,'Sun-Synch' from dual union all
select 32349,'032349','1999-025CYS','deb FY-1C','',66706,'Reentered','2014-05-28 00:00:00','2007-11-30 00:00:00',100.64,613,964,97.77,'Sun-Synch' from dual union all
select 3235,'003235','1963-014AZ','deb West Ford','RCS 0.09',9245,'Reentered','2015-01-15 00:00:00','1992-01-31 00:00:00',164.26,1560,5554,85.99,'Medium' from dual union all
select 32350,'032350','1999-025CYT','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',101.12,548,1076,98.63,'Sun-Synch' from dual union all
select 32351,'032351','1999-025CYU','deb FY-1C','',66706,'In Earth orbit','','2007-11-28 00:00:00',98.12,482,855,99.06,'Sun-Synch' from dual union all
select 32352,'032352','1999-025CYV','deb FY-1C','',66706,'In Earth orbit','','2007-11-28 00:00:00',100.45,682,878,98.97,'Sun-Synch' from dual union all
select 32353,'032353','1999-025CYW','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',104.46,851,1088,99,'Sun-Synch' from dual union all
select 32354,'032354','1999-025CYX','deb FY-1C','',66706,'In Earth orbit','','2007-11-28 00:00:00',103.66,757,1107,102.94,'Sun-Synch' from dual union all
select 32355,'032355','1999-025CYY','deb FY-1C','',66706,'In Earth orbit','','2007-11-29 00:00:00',102.17,737,986,100.39,'Sun-Synch' from dual union all
select 32356,'032356','1999-025CYZ','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',104.42,851,1084,102.49,'Sun-Synch' from dual union all
select 32357,'032357','1999-025CZA','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',107.99,664,1604,98.89,'Sun-Synch' from dual union all
select 32358,'032358','1999-025CZB','deb FY-1C','',66706,'Reentered','2012-08-19 00:00:00','2007-11-27 00:00:00',98.99,575,845,98.9,'Sun-Synch' from dual union all
select 32359,'032359','1999-025CZC','deb FY-1C','',66706,'Reentered','2014-04-09 00:00:00','2007-11-28 00:00:00',100.28,607,937,102.26,'Retrograde' from dual union all
select 3236,'003236','1963-014BA','deb West Ford','',9245,'Reentered','1979-05-10 00:00:00','1968-05-21 00:00:00',162.59,803,6176,84.92,'Medium' from dual union all
select 32360,'032360','1999-025CZD','deb FY-1C','',66706,'In Earth orbit','','2007-11-22 00:00:00',105.34,780,1241,98.14,'Sun-Synch' from dual union all
select 32361,'032361','1999-025CZE','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',101.96,790,913,98.56,'Sun-Synch' from dual union all
select 32362,'032362','1999-025CZF','deb FY-1C','',66706,'In Earth orbit','','2007-11-26 00:00:00',107.26,853,1347,99.73,'Sun-Synch' from dual union all
select 32363,'032363','1999-025CZG','deb FY-1C','',66706,'In Earth orbit','','2007-11-27 00:00:00',99.1,505,927,98.56,'Sun-Synch' from dual union all
select 32364,'032364','1999-025CZH','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',103.71,819,1048,98.39,'Sun-Synch' from dual union all
select 32365,'032365','1999-025CZJ','deb FY-1C','',66706,'Reentered','2011-11-21 00:00:00','2007-11-30 00:00:00',103.82,591,1288,95.13,'Retrograde' from dual union all
select 32366,'032366','1999-025CZK','deb FY-1C','',66706,'In Earth orbit','','2007-11-28 00:00:00',103.1,762,1049,96.79,'Sun-Synch' from dual union all
select 32367,'032367','1999-025CZL','deb FY-1C','',66706,'Reentered','2012-06-14 00:00:00','2007-11-24 00:00:00',98.84,483,924,97.64,'Sun-Synch' from dual union all
select 32368,'032368','1999-025CZM','deb FY-1C','',66706,'In Earth orbit','','2007-11-24 00:00:00',104.3,721,1202,97.87,'Sun-Synch' from dual union all
select 32369,'032369','1999-025CZN','deb FY-1C','',66706,'In Earth orbit','','2007-11-28 00:00:00',106.01,711,1373,98.37,'Sun-Synch' from dual union all
select 3237,'003237','1963-014BB','deb West Ford','',9245,'Reentered','1970-05-20 00:00:00','1968-06-12 00:00:00',162.88,865,6138,85.06,'Medium' from dual union all
select 32370,'032370','1999-025CZP','deb FY-1C','',66706,'In Earth orbit','','2007-11-30 00:00:00',99.7,517,972,99.14,'Sun-Synch' from dual union all
select 32371,'032371','1999-025CZQ','deb FY-1C','',66706,'In Earth orbit','','2007-11-29 00:00:00',100.94,687,920,99.31,'Sun-Synch' from dual union all
select 32372,'032372','1999-025CZR','deb FY-1C','',66706,'In Earth orbit','','2007-11-29 00:00:00',108.9,800,1552,97.33,'Sun-Synch' from dual union all
select 32373,'032373','2007-058A','Raduga-1M','Globus-1M No. 11L',68400,'In Earth orbit','','2007-12-18 00:00:00',1434.84,35572,35952,.06,'Drift GEO' from dual union all
select 32374,'032374','2007-058B','Briz-M DTB','Briz-M torus tank',68400,'In Earth orbit','','2007-12-11 00:00:00',630.32,416,35532,46.49,'GEO Transfer' from dual union all
select 32375,'032375','2007-058C','Briz-M No. 88526','Briz-M 88526',68400,'In Earth orbit','','2008-01-31 00:00:00',1378.92,33633,35687,.18,'Medium' from dual union all
select 32376,'032376','2007-059A','COSMO 2','COSMO-Skymed 2',68401,'In Earth orbit','','2007-12-11 00:00:00',97.15,621,624,97.86,'Sun-Synch' from dual union all
select 32377,'032377','2007-059B','Delta 330','Delta SSPS AJ-10-118K',68401,'Reentered','2008-02-13 00:00:00','2007-12-11 00:00:00',92.71,196,619,98.46,'Sun-Synch' from dual union all
select 32378,'032378','2007-060A','USA 198','QUASAR 15',68402,'In Earth orbit','','2007-12-12 00:00:00',301.51,259,16774,60.01,'Highly Ellip' from dual union all
select 32379,'032379','2007-060B','Centaur AV-015','Centaur AV-015',68402,'In Earth orbit','','2007-12-12 00:00:00',293.83,219,16305,60.76,'Highly Ellip' from dual union all
select 3238,'003238','1963-014BC','deb West Ford','',9245,'Reentered','1970-09-02 00:00:00','1968-06-23 00:00:00',163.11,969,6052,85.26,'Medium' from dual union all
select 32380,'032380','2007-003D','deb CZ-3','',68345,'Reentered','2007-12-14 00:00:00','2007-12-13 00:00:00',94.9,138,890,24.46,'Equatorial' from dual union all
select 32381,'032381','2007-003E','deb CZ-3','',68345,'Reentered','2007-12-23 00:00:00','2007-12-09 00:00:00',162.8,147,6849,24.51,'Medium' from dual union all
select 32382,'032382','2007-061A','Radarsat-2','Radarsat-2',68403,'In Earth orbit','','2007-12-16 00:00:00',100.75,791,797,98.62,'Sun-Synch' from dual union all
select 32383,'032383','2007-061B','11S510','11S510',68403,'Reentered','2007-12-16 00:00:00','2007-12-15 00:00:00',87.96,156,191,95.43,'Sun-Synch' from dual union all
select 32384,'032384','2007-062A','GPS IIR-18(M)','Navstar GPS SVN 57',68404,'In Earth orbit','','2008-02-15 00:00:00',717.98,20082,20283,54.99,'Medium' from dual union all
select 32385,'032385','2007-062B','Star 48B','Star 48B',68404,'Reentered','2010-08-24 00:00:00','2007-12-23 00:00:00',355.65,191,20309,40.03,'Highly Ellip' from dual union all
select 32386,'032386','2007-062C','Delta 331','Delta SSPS AJ-10-118K',68404,'Reentered','2008-04-08 00:00:00','2007-12-22 00:00:00',98.74,184,1213,33.38,'Intermediate' from dual union all
select 32387,'032387','2007-063A','Rascom-QAF 1','Rascom 1',68405,'In Earth orbit','','2008-03-09 00:00:00',1436.06,35781,35791,.05,'Stationary' from dual union all
select 32388,'032388','2007-063B','Horizons 2','Horizons 2',68405,'In Earth orbit','','2008-03-10 00:00:00',1436.11,35777,35797,.03,'Stationary' from dual union all
select 32389,'032389','2007-063C','EPS 530','EPS+ 530',68405,'In Earth orbit','','2007-12-24 00:00:00',624.87,572,35095,5.28,'GEO Transfer' from dual union all
select 3239,'003239','1963-014BD','deb West Ford','RCS 0.09',9245,'In Earth orbit','','1992-02-03 00:00:00',166.39,3611,3675,87.36,'Medium' from dual union all
select 32390,'032390','2007-063D','Sylda 5','Sylda 5',68405,'In Earth orbit','','2007-12-23 00:00:00',638.28,587,35770,5.5,'GEO Transfer' from dual union all
select 32391,'032391','2007-064A','Progress M-62','Progress 7K-TGM No.362',68406,'Deorbited','2008-02-15 00:00:00','2007-12-26 00:00:00',89.47,246,250,51.64,'Intermediate' from dual union all
select 32392,'032392','2007-064B','11S510','11S510',68406,'Reentered','2007-12-26 00:00:00','2007-12-24 00:00:00',88.37,178,210,51.64,'Intermediate' from dual union all
select 32393,'032393','2007-065A','Kosmos-2434','Uragan-M No. 721',68407,'In Earth orbit','','2007-12-27 00:00:00',675.67,19124,19133,64.66,'Medium' from dual union all
select 32394,'032394','2007-065B','Kosmos-2435','Uragan-M No. 722',68407,'In Earth orbit','','2007-12-30 00:00:00',675.65,19126,19130,64.65,'Medium' from dual union all
select 32395,'032395','2007-065C','Kosmos-2436','Uragan-M No. 723',68407,'In Earth orbit','','2007-12-31 00:00:00',678.69,19128,19281,64.67,'Medium' from dual union all
select 32396,'032396','2007-065D','Blok DM-2 No. 109L','11S861 No. 109L',68407,'In Earth orbit','','2007-12-28 00:00:00',674.8,19080,19133,64.66,'Medium' from dual union all
select 32397,'032397','2007-065E','8S812','8S812',68407,'Reentered','2007-12-27 00:00:00','2007-12-26 00:00:00',87.67,146,173,64.8,'Intermediate' from dual union all
select 32398,'032398','2007-065F','SOZ','SOZ',68407,'In Earth orbit','','2008-01-04 00:00:00',340.22,401,19129,64.69,'Highly Ellip' from dual union all
select 32399,'032399','2007-065G','SOZ','SOZ',68407,'In Earth orbit','','2008-01-04 00:00:00',340.22,401,19129,64.81,'Highly Ellip' from dual union all
select 324,'000324','61 Omicron 97','deb r/b Ablestar','',7273,'In Earth orbit','','1964-04-04 00:00:00',103.62,861,998,66.87,'Intermediate' from dual union all
select 3240,'003240','1963-014BE','deb West Ford','',9245,'Reentered','1987-10-17 00:00:00','1968-05-21 00:00:00',162.09,745,6194,84.82,'Medium' from dual union all
select 32400,'032400','2007-065H','Perekhodnik','Sredniy perekhodnik',68407,'Reentered','2007-12-25 00:00:00','2007-12-25 00:00:00',89.9,161,378,64.8,'Intermediate' from dual union all
select 32401,'032401','2007-065J','deb 8S812','',68407,'Reentered','2007-12-26 00:00:00','2007-12-26 00:00:00',90.14,167,395,64.83,'Intermediate' from dual union all
select 32402,'032402','2007-065K','deb 8S812','',68407,'Reentered','2007-12-26 00:00:00','2007-12-26 00:00:00',89.04,160,295,64.67,'Intermediate' from dual union all
select 32403,'032403','2007-065L','deb 8S812','',68407,'Reentered','2007-12-26 00:00:00','2007-12-26 00:00:00',89.6,165,345,64.95,'Intermediate' from dual union all
select 32404,'032404','2008-001A','Thuraya 3','Thuraya 3',68509,'In Earth orbit','','2008-03-09 00:00:00',1436.11,35765,35809,6.14,'Inclined GEO' from dual union all
select 32405,'032405','2008-001B','Blok DM-SL No. 15L','Blok DM-SL',68509,'In Earth orbit','','2008-01-18 00:00:00',639.47,692,35726,6.2,'GEO Transfer' from dual union all
select 32406,'032406','1999-025CZS','deb FY-1C','',66706,'In Earth orbit','','2008-01-14 00:00:00',99.62,451,1030,98.2,'Sun-Synch' from dual union all
select 32407,'032407','1999-025CZT','deb FY-1C','',66706,'In Earth orbit','','2008-01-06 00:00:00',100.36,460,1092,98.67,'Sun-Synch' from dual;
insert into SATELLITE
select 32408,'032408','1999-025CZU','deb FY-1C','',66706,'In Earth orbit','','2008-01-12 00:00:00',100.33,567,982,98.97,'Sun-Synch' from dual union all
select 32409,'032409','1999-025CZV','deb FY-1C','',66706,'In Earth orbit','','2008-01-17 00:00:00',100.82,729,866,99.59,'Sun-Synch' from dual union all
select 3241,'003241','1963-014BF','deb West Ford','',9245,'Reentered','1970-05-28 00:00:00','1968-05-21 00:00:00',163.48,1047,6004,85.49,'Medium' from dual union all
select 32410,'032410','1999-025CZW','deb FY-1C','',66706,'In Earth orbit','','2008-01-17 00:00:00',100.57,612,960,98,'Sun-Synch' from dual union all
select 32411,'032411','1999-025CZX','deb FY-1C','',66706,'In Earth orbit','','2008-01-12 00:00:00',102.55,785,974,98.35,'Sun-Synch' from dual union all
select 32412,'032412','1999-025CZY','deb FY-1C','',66706,'In Earth orbit','','2008-01-16 00:00:00',100.84,609,988,98.23,'Sun-Synch' from dual union all
select 32413,'032413','1999-025CZZ','deb FY-1C','',66706,'Reentered','2008-02-07 00:00:00','2008-01-16 00:00:00',91.74,299,420,99.19,'Sun-Synch' from dual union all
select 32414,'032414','1999-025DAA','deb FY-1C','',66706,'In Earth orbit','','2008-01-13 00:00:00',102.89,849,942,98.99,'Sun-Synch' from dual union all
select 32415,'032415','1999-025DAB','deb FY-1C','',66706,'Reentered','2009-11-25 00:00:00','2008-01-09 00:00:00',97.48,392,885,96.6,'Sun-Synch' from dual union all
select 32416,'032416','1999-025DAC','deb FY-1C','',66706,'In Earth orbit','','2008-01-12 00:00:00',106.01,852,1232,100.84,'Sun-Synch' from dual union all
select 32417,'032417','1999-025DAD','deb FY-1C','',66706,'Reentered','2015-01-19 00:00:00','2008-01-14 00:00:00',100.9,625,978,97.77,'Sun-Synch' from dual union all
select 32418,'032418','1999-025DAE','deb FY-1C','',66706,'In Earth orbit','','2008-01-12 00:00:00',105.53,788,1251,98.23,'Sun-Synch' from dual union all
select 32419,'032419','1999-025DAF','deb FY-1C','',66706,'In Earth orbit','','2008-01-13 00:00:00',102.79,660,1122,97.96,'Sun-Synch' from dual union all
select 3242,'003242','1963-014BG','deb West Ford','',9245,'Reentered','1987-10-17 00:00:00','1968-05-21 00:00:00',162.59,806,6174,84.95,'Medium' from dual union all
select 32420,'032420','1999-025DAG','deb FY-1C','',66706,'Reentered','2014-05-28 00:00:00','2008-01-10 00:00:00',100.64,606,972,97.77,'Sun-Synch' from dual union all
select 32421,'032421','1999-025DAH','deb FY-1C','',66706,'Reentered','2014-11-13 00:00:00','2008-01-14 00:00:00',100.9,621,982,97.84,'Sun-Synch' from dual union all
select 32422,'032422','1999-025DAJ','deb FY-1C','',66706,'In Earth orbit','','2008-01-13 00:00:00',103.41,689,1151,97.86,'Sun-Synch' from dual union all
select 32423,'032423','1999-025DAK','deb FY-1C','',66706,'Reentered','2013-12-25 00:00:00','2008-01-12 00:00:00',100.05,552,970,97.81,'Sun-Synch' from dual union all
select 32424,'032424','1999-025DAL','deb FY-1C','',66706,'Reentered','2016-02-27 00:00:00','2008-01-17 00:00:00',100.62,697,879,98.89,'Sun-Synch' from dual union all
select 32425,'032425','1999-025DAM','deb FY-1C','',66706,'In Earth orbit','','2008-01-14 00:00:00',102.32,680,1057,96.73,'Sun-Synch' from dual union all
select 32426,'032426','1999-025DAN','deb FY-1C','',66706,'In Earth orbit','','2008-01-06 00:00:00',106.03,818,1268,98.3,'Sun-Synch' from dual union all
select 32427,'032427','1999-025DAP','deb FY-1C','',66706,'In Earth orbit','','2008-01-16 00:00:00',101.05,731,886,96.8,'Sun-Synch' from dual union all
select 32428,'032428','1999-025DAQ','deb FY-1C','',66706,'In Earth orbit','','2008-01-14 00:00:00',100.68,721,861,99.04,'Sun-Synch' from dual union all
select 32429,'032429','1999-025DAR','deb FY-1C','',66706,'In Earth orbit','','2008-01-13 00:00:00',103.06,850,956,99.11,'Sun-Synch' from dual union all
select 3243,'003243','1963-014BH','deb West Ford','',9245,'Reentered','1987-10-17 00:00:00','1968-05-21 00:00:00',163.31,1023,6015,85.32,'Medium' from dual union all
select 32430,'032430','1999-025DAS','deb FY-1C','',66706,'In Earth orbit','','2008-01-14 00:00:00',106.79,749,1407,98.73,'Sun-Synch' from dual union all
select 32431,'032431','1999-025DAT','deb FY-1C','',66706,'In Earth orbit','','2008-01-13 00:00:00',102.58,819,943,99.61,'Sun-Synch' from dual union all
select 32432,'032432','1999-025DAU','deb FY-1C','',66706,'In Earth orbit','','2008-01-13 00:00:00',102.06,650,1063,98.25,'Sun-Synch' from dual union all
select 32433,'032433','1999-025DAV','deb FY-1C','',66706,'In Earth orbit','','2008-01-05 00:00:00',103.44,630,1212,98.24,'Sun-Synch' from dual union all
select 32434,'032434','1999-025DAW','deb FY-1C','',66706,'Reentered','2008-03-16 00:00:00','2008-01-16 00:00:00',91.62,236,472,98.43,'Sun-Synch' from dual union all
select 32435,'032435','1999-025DAX','deb FY-1C','',66706,'In Earth orbit','','2008-01-16 00:00:00',98.78,546,855,99.52,'Sun-Synch' from dual union all
select 32436,'032436','1999-025DAY','deb FY-1C','',66706,'Reentered','2011-10-11 00:00:00','2008-01-15 00:00:00',98.96,474,944,97.62,'Sun-Synch' from dual union all
select 32437,'032437','1999-025DAZ','deb FY-1C','',66706,'In Earth orbit','','2008-01-17 00:00:00',100.64,722,855,99.46,'Sun-Synch' from dual union all
select 32438,'032438','1999-025DBA','deb FY-1C','',66706,'In Earth orbit','','2008-01-17 00:00:00',101.34,789,855,99.46,'Sun-Synch' from dual union all
select 32439,'032439','1999-025DBB','deb FY-1C','',66706,'Reentered','2014-02-04 00:00:00','2008-01-14 00:00:00',100.3,511,1035,97.86,'Sun-Synch' from dual union all
select 3244,'003244','1963-014BJ','deb West Ford','RCS 0.11',9245,'In Earth orbit','','1992-02-02 00:00:00',166.43,3603,3686,87.31,'Medium' from dual union all
select 32440,'032440','1999-025DBC','deb FY-1C','',66706,'In Earth orbit','','2008-01-12 00:00:00',107.66,827,1410,99.17,'Sun-Synch' from dual union all
select 32441,'032441','1999-025DBD','deb FY-1C','',66706,'In Earth orbit','','2008-01-15 00:00:00',103.88,707,1176,97.8,'Sun-Synch' from dual union all
select 32442,'032442','1999-025DBE','deb FY-1C','',66706,'Reentered','2014-10-28 00:00:00','2008-01-17 00:00:00',101.78,640,1046,98.77,'Sun-Synch' from dual union all
select 32443,'032443','1999-025DBF','deb FY-1C','',66706,'In Earth orbit','','2008-01-15 00:00:00',102.87,816,973,102.13,'Sun-Synch' from dual union all
select 32444,'032444','1999-025DBG','deb FY-1C','',66706,'In Earth orbit','','2008-01-17 00:00:00',103.78,841,1034,99.15,'Sun-Synch' from dual union all
select 32445,'032445','1999-025DBH','deb FY-1C','',66706,'In Earth orbit','','2008-01-17 00:00:00',100.87,741,859,99.4,'Sun-Synch' from dual union all
select 32446,'032446','1999-025DBJ','deb FY-1C','',66706,'Reentered','2013-09-02 00:00:00','2008-01-15 00:00:00',98.73,428,968,98.74,'Sun-Synch' from dual union all
select 32447,'032447','1999-025DBK','deb FY-1C','',66706,'In Earth orbit','','2008-01-14 00:00:00',103.31,857,974,99.06,'Sun-Synch' from dual union all
select 32448,'032448','1999-025DBL','deb FY-1C','',66706,'In Earth orbit','','2008-01-17 00:00:00',102.1,835,882,98.96,'Sun-Synch' from dual union all
select 32449,'032449','1999-025DBM','deb FY-1C','',66706,'In Earth orbit','','2008-01-02 00:00:00',105.31,750,1268,99.35,'Sun-Synch' from dual union all
select 3245,'003245','1963-014BK','deb West Ford','',9245,'Reentered','1976-10-30 00:00:00','1968-05-21 00:00:00',162.58,813,6166,84.97,'Medium' from dual union all
select 32450,'032450','1999-025DBN','deb FY-1C','',66706,'In Earth orbit','','2008-01-14 00:00:00',103.81,855,1022,98.96,'Sun-Synch' from dual union all
select 32451,'032451','1999-025DBP','deb FY-1C','',66706,'In Earth orbit','','2008-01-11 00:00:00',105.68,854,1198,100.02,'Sun-Synch' from dual union all
select 32452,'032452','1999-025DBQ','deb FY-1C','',66706,'Reentered','2015-08-20 00:00:00','2008-01-01 00:00:00',100.94,611,996,98.33,'Sun-Synch' from dual union all
select 32453,'032453','1999-025DBR','deb FY-1C','',66706,'In Earth orbit','','2008-01-16 00:00:00',103.38,840,997,99.55,'Sun-Synch' from dual union all
select 32454,'032454','1999-025DBS','deb FY-1C','',66706,'In Earth orbit','','2008-01-03 00:00:00',103.71,677,1191,98.49,'Sun-Synch' from dual union all
select 32455,'032455','1999-025DBT','deb FY-1C','',66706,'In Earth orbit','','2008-01-17 00:00:00',105.56,794,1247,101.75,'Sun-Synch' from dual union all
select 32456,'032456','1999-025DBU','deb FY-1C','',66706,'In Earth orbit','','2008-01-13 00:00:00',108.58,855,1468,103.64,'Sun-Synch' from dual union all
select 32457,'032457','1999-025DBV','deb FY-1C','',66706,'In Earth orbit','','2008-01-12 00:00:00',109.21,852,1528,100.44,'Sun-Synch' from dual union all
select 32458,'032458','1999-025DBW','deb FY-1C','',66706,'In Earth orbit','','2008-01-13 00:00:00',101.15,761,865,99.1,'Sun-Synch' from dual union all
select 32459,'032459','1999-025DBX','deb FY-1C','',66706,'In Earth orbit','','2008-01-11 00:00:00',100.97,617,993,98.66,'Sun-Synch' from dual union all
select 3246,'003246','1963-014BL','deb West Ford','',9245,'Reentered','1987-10-17 00:00:00','1968-05-21 00:00:00',163.03,927,6087,85.24,'Medium' from dual union all
select 32460,'032460','1999-025DBY','deb FY-1C','',66706,'In Earth orbit','','2008-01-11 00:00:00',104.51,821,1122,98.04,'Sun-Synch' from dual union all
select 32461,'032461','1999-025DBZ','deb FY-1C','',66706,'In Earth orbit','','2008-01-16 00:00:00',105.8,842,1221,98.37,'Sun-Synch' from dual union all
select 32462,'032462','1999-025DCA','deb FY-1C','',66706,'Reentered','2008-03-05 00:00:00','2008-01-07 00:00:00',98.76,575,824,100.09,'Sun-Synch' from dual union all
select 32463,'032463','1999-025DCB','deb FY-1C','',66706,'Reentered','2012-05-14 00:00:00','2008-01-14 00:00:00',97.32,411,850,103.28,'Retrograde' from dual union all
select 32464,'032464','1999-025DCC','deb FY-1C','',66706,'In Earth orbit','','2008-01-16 00:00:00',100.04,580,941,97.45,'Sun-Synch' from dual union all
select 32465,'032465','1999-025DCD','deb FY-1C','',66706,'Reentered','2012-01-15 00:00:00','2008-01-14 00:00:00',97.77,465,838,98.78,'Sun-Synch' from dual union all
select 32466,'032466','1999-057RA','ZY-1 debris','',66738,'Reentered','2015-03-07 00:00:00','2008-01-15 00:00:00',100.35,774,777,98.18,'Sun-Synch' from dual union all
select 32467,'032467','1999-057RB','ZY-1 debris','',66738,'In Earth orbit','','2008-01-10 00:00:00',100.33,770,779,98.21,'Sun-Synch' from dual union all
select 32468,'032468','1999-057RC','ZY-1 debris','',66738,'In Earth orbit','','2008-01-18 00:00:00',100.34,768,782,98.2,'Sun-Synch' from dual union all
select 32469,'032469','1999-057RD','ZY-1 debris','',66738,'In Earth orbit','','2008-01-12 00:00:00',100.17,758,776,98.19,'Sun-Synch' from dual union all
select 3247,'003247','1963-014BM','deb West Ford','',9245,'Reentered','1977-04-04 00:00:00','1968-03-01 00:00:00',164.03,1268,5827,85.66,'Medium' from dual union all
select 32470,'032470','1999-057RE','ZY-1 debris','',66738,'In Earth orbit','','2008-01-09 00:00:00',100.29,768,777,98.19,'Sun-Synch' from dual union all
select 32471,'032471','1986-073E','deb NOAA 10','',59266,'Reentered','2010-01-14 00:00:00','2008-01-18 00:00:00',100.35,768,783,98.72,'Sun-Synch' from dual union all
select 32472,'032472','1986-073F','deb NOAA 10','',59266,'Reentered','2010-04-01 00:00:00','2008-01-14 00:00:00',100.42,750,807,98.76,'Sun-Synch' from dual union all
select 32473,'032473','2007-042C','CZ-4B sep motor cover','',68384,'In Earth orbit','','2008-01-17 00:00:00',99.97,670,844,98.52,'Sun-Synch' from dual union all
select 32474,'032474','2007-042D','CZ-4B sep motor cover','',68384,'In Earth orbit','','2008-01-13 00:00:00',99.07,627,802,98.63,'Sun-Synch' from dual union all
select 32475,'032475','1966-040AS','Nimbus 2 debris','',13447,'In Earth orbit','','2008-01-17 00:00:00',107.95,1086,1177,100.53,'Sun-Synch' from dual union all
select 32476,'032476','2008-002A','''Ofeq-8','TecSAR-1',68510,'In Earth orbit','','2008-01-28 00:00:00',94.44,403,579,41.02,'Intermediate' from dual union all
select 32477,'032477','2008-002B','PS4','PS4 PSLV-C10',68510,'In Earth orbit','','2008-01-28 00:00:00',95.18,470,585,41.05,'Intermediate' from dual union all
select 32478,'032478','2008-003A','Ekspress AM-33','Ekspress AM-33',68511,'In Earth orbit','','2008-02-13 00:00:00',1436.42,35741,35845,.09,'Stationary' from dual union all
select 32479,'032479','2008-003B','Briz-M No. 88530','Briz-M 88530',68511,'In Earth orbit','','2008-02-11 00:00:00',1384.57,33985,35559,.05,'Drift GEO' from dual union all
select 3248,'003248','1963-014BN','deb West Ford','RCS 0.25',9245,'Reentered','1987-10-17 00:00:00','1968-05-21 00:00:00',162.96,888,6122,85.15,'Medium' from dual union all
select 32480,'032480','2008-003C','Briz-M DTB','Briz-M torus tank',68511,'In Earth orbit','','2008-02-04 00:00:00',630.46,409,35546,46.15,'GEO Transfer' from dual union all
select 32481,'032481','2006-006AB','deb Briz-M 88515','',68178,'In Earth orbit','','2008-02-17 00:00:00',271.13,851,14145,49.77,'Medium' from dual union all
select 32482,'032482','1978-064S','deb Seasat','',45259,'Reentered','2009-04-05 00:00:00','2008-06-06 00:00:00',99.04,602,824,107.98,'Retrograde' from dual union all
select 32483,'032483','1979-009D','SVM-2B debris','SVM-2B debris',47493,'In Earth orbit','','2008-06-05 00:00:00',849.83,12321,34363,10.19,'Medium' from dual union all
select 32484,'032484','2008-004A','Progress M-63','Progress 7K-TGM No.363',68512,'Deorbited','2008-04-07 00:00:00','2008-02-13 00:00:00',91.23,331,339,51.64,'Intermediate' from dual union all
select 32485,'032485','2008-004B','11S510','11S510',68512,'Reentered','2008-02-08 00:00:00','2008-02-06 00:00:00',88.34,177,207,51.62,'Intermediate' from dual union all
select 32486,'032486','2008-005A','Atlantis (STS-122)','OV-104',68513,'Landed','2008-02-20 00:00:00','2008-02-09 00:00:00',91.23,331,339,51.64,'Intermediate' from dual union all
select 32487,'032487','2008-006A','Thor 2R','Thor 5',68514,'In Earth orbit','','2008-02-19 00:00:00',1453.83,35964,36302,.17,'Drift GEO' from dual union all
select 32488,'032488','2008-006B','Briz-M DTB','Briz-M torus tank',68514,'In Earth orbit','','2008-02-14 00:00:00',651.54,514,36521,49.03,'GEO Transfer' from dual union all
select 32489,'032489','2006-006AC','deb Briz-M 88515','',68178,'In Earth orbit','','2008-02-17 00:00:00',264.75,800,13759,50.84,'Medium' from dual union all
select 3249,'003249','1963-014BP','deb West Ford','',9245,'Reentered','1980-09-28 00:00:00','1968-05-21 00:00:00',162.72,854,6136,85.02,'Medium' from dual union all
select 32490,'032490','2006-006AD','deb Briz-M 88515','',68178,'In Earth orbit','','2008-02-18 00:00:00',225.24,547,11222,51.48,'Medium' from dual union all
select 32491,'032491','2006-006AE','deb Briz-M 88515','',68178,'In Earth orbit','2014-01-01 00:00:00','2008-02-18 00:00:00',263.11,293,14153,51.5,'Highly Ellip' from dual union all
select 32492,'032492','2006-006AF','deb Briz-M 88515','',68178,'In Earth orbit','','2008-02-18 00:00:00',273.26,539,14603,51.4,'Highly Ellip' from dual union all
select 32493,'032493','2006-006AG','deb Briz-M 88515','',68178,'In Earth orbit','','2008-02-18 00:00:00',272.11,412,14651,51.62,'Highly Ellip' from dual union all
select 32494,'032494','2006-006AH','deb Briz-M 88515','',68178,'In Earth orbit','','2008-02-18 00:00:00',275.42,450,14839,51.65,'Highly Ellip' from dual union all
select 32495,'032495','2006-006AJ','deb Briz-M 88515','',68178,'Reentered','2015-04-17 00:00:00','2008-02-18 00:00:00',252.83,279,13455,51.83,'Medium' from dual union all
select 32496,'032496','2006-006AK','deb Briz-M 88515','',68178,'In Earth orbit','','2008-02-18 00:00:00',277.4,421,15001,51.73,'Highly Ellip' from dual union all
select 32497,'032497','2006-006AL','deb Briz-M 88515','',68178,'In Earth orbit','','2008-02-17 00:00:00',274.57,421,14809,51.72,'Highly Ellip' from dual union all
select 32498,'032498','2006-006AM','deb Briz-M 88515','',68178,'In Earth orbit','','2008-02-17 00:00:00',275.95,407,14917,51.73,'Highly Ellip' from dual;
insert into SATELLITE
select 32499,'032499','2006-006AN','deb Briz-M 88515','',68178,'In Earth orbit','','2008-02-18 00:00:00',273.23,354,14785,51.67,'Highly Ellip' from dual union all
select 325,'000325','61 Omicron 98','deb r/b Ablestar','',7273,'In Earth orbit','','1962-06-26 00:00:00',103.58,877,979,66.92,'Intermediate' from dual union all
select 3250,'003250','1963-014BQ','deb West Ford','RCS 0.11',9245,'Reentered','1990-01-18 00:00:00','1968-05-21 00:00:00',162.8,863,6134,85.08,'Medium' from dual union all
select 32500,'032500','2008-007A','Kizuna','WINDS',68515,'In Earth orbit','','2008-03-10 00:00:00',1433.33,35637,35827,.08,'Drift GEO' from dual union all
select 32501,'032501','2008-007B','H-2A F14 Stage 2','H-2A Stage 2',68515,'In Earth orbit','','2008-02-24 00:00:00',612.16,258,34750,28.51,'GEO Transfer' from dual union all
select 32502,'032502','2006-057C','USA 193 debris','',68229,'Reentered','2008-03-01 00:00:00','2008-02-26 00:00:00',100.85,166,1431,57.24,'Intermediate' from dual union all
select 32503,'032503','2006-057D','USA 193 debris','',68229,'Reentered','2008-03-28 00:00:00','2008-02-26 00:00:00',105.99,213,1869,58.29,'Intermediate' from dual union all
select 32504,'032504','2006-057E','USA 193 debris','',68229,'Reentered','2008-03-01 00:00:00','2008-02-26 00:00:00',93.96,172,764,57.95,'Intermediate' from dual union all
select 32505,'032505','2006-057F','USA 193 debris','',68229,'Reentered','2008-03-27 00:00:00','2008-02-26 00:00:00',102.78,227,1554,58.42,'Intermediate' from dual union all
select 32506,'032506','2006-057G','USA 193 debris','',68229,'Reentered','2008-02-29 00:00:00','2008-02-26 00:00:00',94.45,171,812,58.82,'Intermediate' from dual union all
select 32507,'032507','2006-057H','USA 193 debris','',68229,'Reentered','2008-03-27 00:00:00','2008-02-26 00:00:00',103.38,229,1608,58.6,'Intermediate' from dual union all
select 32508,'032508','2006-057J','USA 193 debris','',68229,'Reentered','2008-03-06 00:00:00','2008-02-26 00:00:00',97.14,235,1009,58.37,'Intermediate' from dual union all
select 32509,'032509','2006-057K','USA 193 debris','',68229,'Reentered','2008-03-18 00:00:00','2008-02-26 00:00:00',100.13,240,1290,58.23,'Intermediate' from dual union all
select 3251,'003251','1963-014BR','deb West Ford','',9245,'Reentered','1987-10-17 00:00:00','1968-05-21 00:00:00',162.12,755,6186,84.78,'Medium' from dual union all
select 32510,'032510','2006-057L','USA 193 debris','',68229,'Reentered','2008-05-22 00:00:00','2008-02-26 00:00:00',95.68,242,861,58.55,'Intermediate' from dual union all
select 32511,'032511','2006-057M','USA 193 debris','',68229,'Reentered','2008-03-07 00:00:00','2008-02-26 00:00:00',94.46,226,759,58.45,'Intermediate' from dual union all
select 32512,'032512','2006-057N','USA 193 debris','',68229,'Reentered','2008-06-14 00:00:00','2008-02-26 00:00:00',96.69,239,961,58.55,'Intermediate' from dual union all
select 32513,'032513','2006-057P','USA 193 debris','',68229,'Reentered','2008-02-27 00:00:00','2008-02-26 00:00:00',89.03,195,259,57.81,'Intermediate' from dual union all
select 32514,'032514','2006-057Q','USA 193 debris','',68229,'Reentered','2008-03-02 00:00:00','2008-02-26 00:00:00',89.08,202,256,58.47,'Intermediate' from dual union all
select 32515,'032515','2006-057R','USA 193 debris','',68229,'Reentered','2008-04-03 00:00:00','2008-02-26 00:00:00',94.71,243,766,58.73,'Intermediate' from dual union all
select 32516,'032516','2006-057S','USA 193 debris','',68229,'Reentered','2008-03-01 00:00:00','2008-02-26 00:00:00',96.23,192,963,58.36,'Intermediate' from dual union all
select 32517,'032517','2006-057T','USA 193 debris','',68229,'Reentered','2008-03-15 00:00:00','2008-02-26 00:00:00',92.43,239,549,59.23,'Intermediate' from dual union all
select 32518,'032518','2006-057U','USA 193 debris','',68229,'Reentered','2008-03-01 00:00:00','2008-02-26 00:00:00',90.83,188,443,58.51,'Intermediate' from dual union all
select 32519,'032519','2006-057V','USA 193 debris','',68229,'Reentered','2008-03-17 00:00:00','2008-02-27 00:00:00',98.88,218,1193,58.46,'Intermediate' from dual union all
select 3252,'003252','1963-014BS','deb West Ford','',9245,'Reentered','1973-05-30 00:00:00','1968-05-21 00:00:00',162.46,802,6167,84.87,'Medium' from dual union all
select 32520,'032520','2006-057W','USA 193 debris','',68229,'Reentered','2008-03-01 00:00:00','2008-02-27 00:00:00',89.17,217,250,58.71,'Intermediate' from dual union all
select 32521,'032521','2006-057X','USA 193 debris','',68229,'Reentered','2008-03-17 00:00:00','2008-03-03 00:00:00',98.08,229,1104,58.63,'Intermediate' from dual union all
select 32522,'032522','2006-057Y','USA 193 debris','',68229,'Reentered','2008-03-17 00:00:00','2008-03-03 00:00:00',96.79,235,975,59.28,'Intermediate' from dual union all
select 32523,'032523','2006-057Z','USA 193 debris','',68229,'Reentered','2008-03-11 00:00:00','2008-03-03 00:00:00',91.75,233,488,56.54,'Intermediate' from dual union all
select 32524,'032524','2006-057AA','USA 193 debris','',68229,'Reentered','2008-05-25 00:00:00','2008-02-27 00:00:00',99.94,238,1274,58.81,'Intermediate' from dual union all
select 32525,'032525','2006-057AB','USA 193 debris','',68229,'Reentered','2008-03-24 00:00:00','2008-02-27 00:00:00',92.95,239,599,58.51,'Intermediate' from dual union all
select 32526,'032526','2006-057AC','USA 193 debris','',68229,'Reentered','2008-03-02 00:00:00','2008-02-27 00:00:00',91.35,219,462,59.38,'Intermediate' from dual union all
select 32527,'032527','2006-057AD','USA 193 debris','',68229,'Reentered','2008-04-15 00:00:00','2008-02-27 00:00:00',92.73,236,581,58.12,'Intermediate' from dual union all
select 32528,'032528','2006-057AE','USA 193 debris','',68229,'Reentered','2008-03-30 00:00:00','2008-02-27 00:00:00',92.68,242,570,58.02,'Intermediate' from dual union all
select 32529,'032529','2006-057AF','USA 193 debris','',68229,'Reentered','2008-03-30 00:00:00','2008-02-27 00:00:00',93.85,239,687,58.35,'Intermediate' from dual union all
select 3253,'003253','1963-014BT','deb West Ford','',9245,'Reentered','1987-10-17 00:00:00','1968-05-20 00:00:00',162.42,781,6185,84.85,'Medium' from dual union all
select 32530,'032530','2006-057AG','USA 193 debris','',68229,'Reentered','2008-03-25 00:00:00','2008-02-27 00:00:00',91.36,242,441,58.27,'Intermediate' from dual union all
select 32531,'032531','2006-057AH','USA 193 debris','',68229,'Reentered','2008-03-27 00:00:00','2008-02-27 00:00:00',91.56,234,469,58.2,'Intermediate' from dual union all
select 32532,'032532','2006-057AJ','USA 193 debris','',68229,'Reentered','2008-05-19 00:00:00','2008-02-28 00:00:00',115.03,225,2688,58.51,'Intermediate' from dual union all
select 32533,'032533','2006-057AK','USA 193 debris','',68229,'Reentered','2008-03-02 00:00:00','2008-02-28 00:00:00',90.27,225,351,58.67,'Intermediate' from dual union all
select 32534,'032534','2006-057AL','USA 193 debris','',68229,'Reentered','2008-03-17 00:00:00','2008-02-28 00:00:00',90.39,240,347,58.31,'Intermediate' from dual union all
select 32535,'032535','2006-057AM','USA 193 debris','',68229,'Reentered','2008-03-07 00:00:00','2008-02-28 00:00:00',94.43,234,748,60.39,'Intermediate' from dual union all
select 32536,'032536','2006-057AN','USA 193 debris','',68229,'Reentered','2008-03-05 00:00:00','2008-02-28 00:00:00',91.79,223,501,59.28,'Intermediate' from dual union all
select 32537,'032537','2006-057AP','USA 193 debris','',68229,'Reentered','2008-03-02 00:00:00','2008-02-28 00:00:00',91.34,231,450,58.28,'Intermediate' from dual union all
select 32538,'032538','2006-057AQ','USA 193 debris','',68229,'Reentered','2008-03-27 00:00:00','2008-02-28 00:00:00',92.98,215,625,58.65,'Intermediate' from dual union all
select 32539,'032539','2006-057AR','USA 193 debris','',68229,'Reentered','2008-03-11 00:00:00','2008-02-28 00:00:00',92.76,237,583,58.7,'Intermediate' from dual union all
select 3254,'003254','1963-014BU','deb West Ford','',9245,'Reentered','1969-11-02 00:00:00','1968-05-21 00:00:00',163.3,1008,6029,85.4,'Medium' from dual union all
select 32540,'032540','2006-057AS','USA 193 debris','',68229,'Reentered','2008-03-30 00:00:00','2008-02-28 00:00:00',93.51,239,653,58.63,'Intermediate' from dual union all
select 32541,'032541','2006-057AT','USA 193 debris','',68229,'Reentered','2008-03-04 00:00:00','2008-02-28 00:00:00',93.57,239,659,58.88,'Intermediate' from dual union all
select 32542,'032542','2006-057AU','USA 193 debris','',68229,'Reentered','2008-05-03 00:00:00','2008-02-28 00:00:00',95.63,229,870,58.91,'Intermediate' from dual union all
select 32543,'032543','2006-057AV','USA 193 debris','',68229,'Reentered','2008-06-13 00:00:00','2008-02-28 00:00:00',96.79,239,971,58.69,'Intermediate' from dual union all
select 32544,'032544','2006-057AW','USA 193 debris','',68229,'Reentered','2008-03-07 00:00:00','2008-02-28 00:00:00',90.89,232,404,58.79,'Intermediate' from dual union all
select 32545,'032545','2006-057AX','USA 193 debris','',68229,'Reentered','2008-03-18 00:00:00','2008-02-28 00:00:00',91.13,239,421,58.06,'Intermediate' from dual union all
select 32546,'032546','2006-057AY','USA 193 debris','',68229,'Reentered','2008-03-01 00:00:00','2008-02-28 00:00:00',90.66,232,382,58.95,'Intermediate' from dual union all
select 32547,'032547','2007-054C','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',151.84,900,5200,29.11,'Medium' from dual union all
select 32548,'032548','2007-054D','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',131.78,800,3600,29.11,'Medium' from dual union all
select 32549,'032549','2007-054E','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',129.48,800,3400,29.11,'Medium' from dual union all
select 3255,'003255','1963-014BV','deb West Ford','RCS 0.27',9245,'Reentered','1976-10-30 00:00:00','2000-09-21 00:00:00',166.14,3168,4098,87.28,'Medium' from dual union all
select 32550,'032550','2007-054F','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',124.36,800,2950,29.11,'Medium' from dual union all
select 32551,'032551','2007-054G','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',113.78,600,2199,29.11,'Intermediate' from dual union all
select 32552,'032552','2007-054H','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',112.68,600,2100,29.11,'Intermediate' from dual union all
select 32553,'032553','2007-054J','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',109.96,500,1950,29.11,'Intermediate' from dual union all
select 32554,'032554','2007-054K','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',109.42,500,1900,29.11,'Intermediate' from dual union all
select 32555,'032555','2007-054L','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',107.26,400,1800,29.11,'Intermediate' from dual union all
select 32556,'032556','2007-054M','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',102.45,250,1499,29.11,'Intermediate' from dual union all
select 32557,'032557','2007-054N','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',101.4,250,1400,29.11,'Intermediate' from dual union all
select 32558,'032558','2007-054P','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',101.4,250,1400,29.11,'Intermediate' from dual union all
select 32559,'032559','2007-054Q','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',99.29,250,1200,29.11,'Intermediate' from dual union all
select 3256,'003256','1963-014BW','deb West Ford','',9245,'Reentered','1977-04-12 00:00:00','1968-04-27 00:00:00',165.23,1960,5233,86.61,'Medium' from dual union all
select 32560,'032560','2007-054R','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',98.25,250,1100,29.11,'Intermediate' from dual union all
select 32561,'032561','2007-054S','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',96.68,200,1000,29.11,'Intermediate' from dual union all
select 32562,'032562','2007-054T','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',96.16,200,950,29.11,'Intermediate' from dual union all
select 32563,'032563','2007-054U','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',95.65,200,900,29.11,'Intermediate' from dual union all
select 32564,'032564','2007-054V','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',94.61,200,800,29.11,'Intermediate' from dual union all
select 32565,'032565','2007-054W','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',94.1,200,750,29.11,'Intermediate' from dual union all
select 32566,'032566','2007-054X','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',110.5,500,2000,29.11,'Intermediate' from dual union all
select 32567,'032567','2007-054Y','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',110.5,500,2000,29.11,'Intermediate' from dual union all
select 32568,'032568','2007-054Z','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',110.5,500,2000,29.11,'Intermediate' from dual union all
select 32569,'032569','2007-054AA','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',110.5,500,2000,29.11,'Intermediate' from dual union all
select 3257,'003257','1963-014BX','deb West Ford','RCS 0.21',9245,'In Earth orbit','','1992-02-02 00:00:00',166.19,3229,4041,87.33,'Medium' from dual union all
select 32570,'032570','2007-054AB','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',110.5,500,2000,29.11,'Intermediate' from dual union all
select 32571,'032571','2007-054AC','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',110.5,500,2000,29.11,'Intermediate' from dual union all
select 32572,'032572','2007-054AD','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',110.5,500,2000,29.11,'Intermediate' from dual union all
select 32573,'032573','2007-054AE','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',110.5,500,2000,29.11,'Intermediate' from dual union all
select 32574,'032574','2007-054AF','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',110.5,500,2000,29.11,'Intermediate' from dual union all
select 32575,'032575','2007-054AG','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',110.5,500,2000,29.11,'Intermediate' from dual union all
select 32576,'032576','2007-054AH','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',110.5,500,2000,29.11,'Intermediate' from dual union all
select 32577,'032577','2007-054AJ','deb Delta 329','deb DCSS-5 F02',68396,'In Earth orbit','','2007-11-11 00:00:00',110.5,500,2000,29.11,'Intermediate' from dual union all
select 32578,'032578','2006-057AZ','USA 193 debris','',68229,'Reentered','2008-04-25 00:00:00','2008-02-29 00:00:00',89.95,241,303,58.4,'Intermediate' from dual union all
select 32579,'032579','2006-057BA','USA 193 debris','',68229,'Reentered','2008-03-20 00:00:00','2008-02-29 00:00:00',89.25,222,253,58.65,'Intermediate' from dual union all
select 3258,'003258','1963-014BY','deb West Ford','',9245,'Reentered','1969-08-08 00:00:00','1967-08-06 00:00:00',164.77,1537,5619,86.32,'Medium' from dual union all
select 32580,'032580','2006-057BB','USA 193 debris','',68229,'Reentered','2008-03-15 00:00:00','2008-02-29 00:00:00',91.4,238,449,58.18,'Intermediate' from dual union all
select 32581,'032581','2006-057BC','USA 193 debris','',68229,'Reentered','2008-03-29 00:00:00','2008-02-29 00:00:00',94.6,238,761,58.45,'Intermediate' from dual union all
select 32582,'032582','2006-057BD','USA 193 debris','',68229,'Reentered','2008-04-08 00:00:00','2008-02-29 00:00:00',107.17,239,1953,58.63,'Intermediate' from dual union all
select 32583,'032583','2006-057BE','USA 193 debris','',68229,'Reentered','2008-04-06 00:00:00','2008-02-29 00:00:00',109.31,234,2156,59.12,'Intermediate' from dual union all
select 32584,'032584','2006-057BF','USA 193 debris','',68229,'Reentered','2008-03-07 00:00:00','2008-02-29 00:00:00',88.93,219,225,58.57,'Intermediate' from dual union all
select 32585,'032585','2006-057BG','USA 193 debris','',68229,'Reentered','2008-03-06 00:00:00','2008-02-29 00:00:00',90.42,229,362,58.34,'Intermediate' from dual union all
select 32586,'032586','2006-057BH','USA 193 debris','',68229,'Reentered','2008-03-12 00:00:00','2008-02-29 00:00:00',91.91,230,507,58.92,'Intermediate' from dual union all
select 32587,'032587','2006-057BJ','USA 193 debris','',68229,'Reentered','2008-03-04 00:00:00','2008-02-29 00:00:00',89.21,204,267,58.52,'Intermediate' from dual union all
select 32588,'032588','2006-057BK','USA 193 debris','',68229,'Reentered','2008-03-08 00:00:00','2008-03-03 00:00:00',98.25,186,1164,57.43,'Intermediate' from dual;
insert into SATELLITE
select 32589,'032589','2006-057BL','USA 193 debris','',68229,'Reentered','2008-03-28 00:00:00','2008-03-03 00:00:00',96.36,240,929,59.58,'Intermediate' from dual union all
select 3259,'003259','1963-014BZ','deb West Ford','',9245,'Reentered','1976-10-30 00:00:00','1968-05-20 00:00:00',161.6,667,6233,84.7,'Medium' from dual union all
select 32590,'032590','2006-057BM','USA 193 debris','',68229,'Reentered','2008-03-22 00:00:00','2008-03-03 00:00:00',89.21,213,258,58.3,'Intermediate' from dual union all
select 32591,'032591','2006-057BN','USA 193 debris','',68229,'Reentered','2008-04-12 00:00:00','2008-03-03 00:00:00',107.72,231,2012,59.58,'Intermediate' from dual union all
select 32592,'032592','2006-057BP','USA 193 debris','',68229,'Reentered','2008-03-12 00:00:00','2008-03-03 00:00:00',97.49,204,1074,58.54,'Intermediate' from dual union all
select 32593,'032593','2006-057BQ','USA 193 debris','',68229,'Reentered','2008-05-06 00:00:00','2008-03-03 00:00:00',96.93,238,986,58.49,'Intermediate' from dual union all
select 32594,'032594','2006-057BR','USA 193 debris','',68229,'Reentered','2008-03-28 00:00:00','2008-03-03 00:00:00',97.3,235,1024,58.77,'Intermediate' from dual union all
select 32595,'032595','2006-057BS','USA 193 debris','',68229,'Reentered','2008-03-24 00:00:00','2008-03-03 00:00:00',97.57,236,1049,58.65,'Intermediate' from dual union all
select 32596,'032596','2006-057BT','USA 193 debris','',68229,'Reentered','2008-03-26 00:00:00','2008-03-03 00:00:00',90.11,211,348,58.23,'Intermediate' from dual union all
select 32597,'032597','2006-057BU','USA 193 debris','',68229,'Reentered','2008-04-09 00:00:00','2008-03-03 00:00:00',108.38,242,2062,58.67,'Intermediate' from dual union all
select 32598,'032598','2006-057BV','USA 193 debris','',68229,'Reentered','2008-03-16 00:00:00','2008-03-03 00:00:00',96.43,209,967,58.23,'Intermediate' from dual union all
select 32599,'032599','2006-057BW','USA 193 debris','',68229,'Reentered','2008-03-23 00:00:00','2008-03-03 00:00:00',96.95,236,990,59.13,'Intermediate' from dual union all
select 326,'000326','61 Omicron 99','deb r/b Ablestar','',7273,'Reentered','1979-10-07 00:00:00','1962-06-28 00:00:00',100.32,582,966,66.94,'Intermediate' from dual union all
select 3260,'003260','1963-014CA','deb West Ford','',9245,'Reentered','1969-06-17 00:00:00','1968-05-17 00:00:00',160.94,636,6209,84.5,'Medium' from dual union all
select 32600,'032600','2006-057BX','USA 193 debris','',68229,'Reentered','2008-03-16 00:00:00','2008-03-03 00:00:00',95.91,236,890,59.42,'Intermediate' from dual union all
select 32601,'032601','2006-057BY','USA 193 debris','',68229,'Reentered','2008-08-30 00:00:00','2008-03-03 00:00:00',96.81,241,971,58.54,'Intermediate' from dual union all
select 32602,'032602','2006-057BZ','USA 193 debris','',68229,'Reentered','2008-05-30 00:00:00','2008-03-03 00:00:00',96.6,243,949,57.2,'Intermediate' from dual union all
select 32603,'032603','2006-057CA','USA 193 debris','',68229,'Reentered','2008-03-10 00:00:00','2008-03-03 00:00:00',94.48,200,787,58.71,'Intermediate' from dual union all
select 32604,'032604','2006-057CB','USA 193 debris','',68229,'Reentered','2008-06-08 00:00:00','2008-03-03 00:00:00',95.79,240,873,58.5,'Intermediate' from dual union all
select 32605,'032605','2006-057CC','USA 193 debris','',68229,'Reentered','2008-04-07 00:00:00','2008-03-03 00:00:00',93.44,228,657,58.61,'Intermediate' from dual union all
select 32606,'032606','2006-057CD','USA 193 debris','',68229,'Reentered','2008-04-08 00:00:00','2008-03-03 00:00:00',94.67,239,767,58.52,'Intermediate' from dual union all
select 32607,'032607','2006-057CE','USA 193 debris','',68229,'Reentered','2008-03-12 00:00:00','2008-03-03 00:00:00',89.1,208,251,58.3,'Intermediate' from dual union all
select 32608,'032608','2006-057CF','USA 193 debris','',68229,'Reentered','2008-05-04 00:00:00','2008-03-03 00:00:00',90.38,235,352,58.54,'Intermediate' from dual union all
select 32609,'032609','2006-057CG','USA 193 debris','',68229,'Reentered','2008-03-09 00:00:00','2008-03-03 00:00:00',89.43,206,287,58.58,'Intermediate' from dual union all
select 3261,'003261','1963-014CB','deb West Ford','',9245,'Reentered','1987-10-17 00:00:00','1967-08-18 00:00:00',164.4,1362,5763,86.17,'Medium' from dual union all
select 32610,'032610','2006-057CH','USA 193 debris','',68229,'Reentered','2008-03-04 00:00:00','2008-03-03 00:00:00',88.54,191,213,57.9,'Intermediate' from dual union all
select 32611,'032611','2006-057CJ','USA 193 debris','',68229,'Reentered','2008-03-12 00:00:00','2008-03-03 00:00:00',90.13,227,334,58.18,'Intermediate' from dual union all
select 32612,'032612','2006-057CK','USA 193 debris','',68229,'Reentered','2008-04-09 00:00:00','2008-03-03 00:00:00',91.06,238,416,58.4,'Intermediate' from dual union all
select 32613,'032613','2006-057CL','USA 193 debris','',68229,'Reentered','2008-03-24 00:00:00','2008-03-03 00:00:00',91.41,233,455,58.16,'Intermediate' from dual union all
select 32614,'032614','2006-057CM','USA 193 debris','',68229,'Reentered','2008-03-12 00:00:00','2008-03-03 00:00:00',89.68,230,288,58.34,'Intermediate' from dual union all
select 32615,'032615','2006-057CN','USA 193 debris','',68229,'Reentered','2008-03-13 00:00:00','2008-03-03 00:00:00',89.8,231,299,58.7,'Intermediate' from dual union all
select 32616,'032616','2006-057CP','USA 193 debris','',68229,'Reentered','2008-03-22 00:00:00','2008-03-03 00:00:00',89.27,230,247,58.65,'Intermediate' from dual union all
select 32617,'032617','2006-057CQ','USA 193 debris','',68229,'Reentered','2008-03-13 00:00:00','2008-03-03 00:00:00',89.1,183,277,58.63,'Intermediate' from dual union all
select 32618,'032618','2006-057CR','USA 193 debris','',68229,'Reentered','2008-03-08 00:00:00','2008-03-03 00:00:00',89.01,203,249,58.3,'Intermediate' from dual union all
select 32619,'032619','2006-057CS','USA 193 debris','',68229,'Reentered','2008-03-30 00:00:00','2008-03-03 00:00:00',94.33,235,737,56.67,'Intermediate' from dual union all
select 3262,'003262','1963-014CC','deb West Ford','',9245,'Reentered','1969-07-04 00:00:00','1968-05-19 00:00:00',161.3,660,6215,84.54,'Medium' from dual union all
select 32620,'032620','2006-057CT','USA 193 debris','',68229,'Reentered','2008-03-29 00:00:00','2008-03-04 00:00:00',104.41,211,1723,58.25,'Intermediate' from dual union all
select 32621,'032621','2006-057CU','USA 193 debris','',68229,'Reentered','2008-03-11 00:00:00','2008-03-04 00:00:00',97.09,223,1016,58.87,'Intermediate' from dual union all
select 32622,'032622','2006-057CV','USA 193 debris','',68229,'Reentered','2008-03-25 00:00:00','2008-03-04 00:00:00',95.96,238,892,59.54,'Intermediate' from dual union all
select 32623,'032623','2006-057CW','USA 193 debris','',68229,'Reentered','2008-03-12 00:00:00','2008-03-04 00:00:00',95.24,226,835,60.35,'Intermediate' from dual union all
select 32624,'032624','2006-057CX','USA 193 debris','',68229,'Reentered','2008-03-04 00:00:00','2008-03-04 00:00:00',87.63,146,169,58.27,'Intermediate' from dual union all
select 32625,'032625','2006-057CY','USA 193 debris','',68229,'Reentered','2008-03-17 00:00:00','2008-03-04 00:00:00',102.99,196,1605,57.65,'Intermediate' from dual union all
select 32626,'032626','2006-057CZ','USA 193 debris','',68229,'Reentered','2008-05-03 00:00:00','2008-03-04 00:00:00',89.93,237,305,58.38,'Intermediate' from dual union all
select 32627,'032627','2006-057DA','USA 193 debris','',68229,'Reentered','2008-03-10 00:00:00','2008-03-04 00:00:00',89.98,213,334,58.32,'Intermediate' from dual union all
select 32628,'032628','2006-057DB','USA 193 debris','',68229,'Reentered','2008-03-29 00:00:00','2008-03-04 00:00:00',89.74,236,287,58.75,'Intermediate' from dual union all
select 32629,'032629','2006-057DC','USA 193 debris','',68229,'Reentered','2008-03-29 00:00:00','2008-03-04 00:00:00',91.58,228,477,58.07,'Intermediate' from dual union all
select 3263,'003263','1963-014CD','deb West Ford','RCS 0.18',9245,'Reentered','1987-10-17 00:00:00','1968-05-20 00:00:00',163.01,915,6099,85.2,'Medium' from dual union all
select 32630,'032630','2006-057DD','USA 193 debris','',68229,'Reentered','2008-03-10 00:00:00','2008-03-04 00:00:00',90.75,229,393,58.4,'Intermediate' from dual union all
select 32631,'032631','2006-057DE','USA 193 debris','',68229,'Reentered','2008-03-04 00:00:00','2008-03-04 00:00:00',89.18,193,274,58.86,'Intermediate' from dual union all
select 32632,'032632','2006-057DF','USA 193 debris','',68229,'Reentered','2008-03-25 00:00:00','2008-03-04 00:00:00',94.03,208,736,58.75,'Intermediate' from dual union all
select 32633,'032633','2006-057DG','USA 193 debris','',68229,'Reentered','2008-03-06 00:00:00','2008-03-04 00:00:00',90.85,204,429,60.42,'Intermediate' from dual union all
select 32634,'032634','2006-057DH','USA 193 debris','',68229,'Reentered','2008-03-24 00:00:00','2008-03-04 00:00:00',93.2,236,626,58.27,'Intermediate' from dual union all
select 32635,'032635','2006-057DJ','USA 193 debris','',68229,'Reentered','2008-03-10 00:00:00','2008-03-04 00:00:00',88.94,215,230,58.72,'Intermediate' from dual union all
select 32636,'032636','2006-057DK','USA 193 debris','',68229,'Reentered','2008-03-06 00:00:00','2008-03-04 00:00:00',89.11,206,255,57.44,'Intermediate' from dual union all
select 32637,'032637','2006-057DL','USA 193 debris','',68229,'Reentered','2008-03-07 00:00:00','2008-03-04 00:00:00',89.82,211,321,58.59,'Intermediate' from dual union all
select 32638,'032638','2006-057DM','USA 193 debris','',68229,'Reentered','2008-03-17 00:00:00','2008-03-04 00:00:00',89.46,217,278,58.33,'Intermediate' from dual union all
select 32639,'032639','2006-057DN','USA 193 debris','',68229,'Reentered','2008-03-24 00:00:00','2008-03-04 00:00:00',91.77,238,485,57.95,'Intermediate' from dual union all
select 3264,'003264','1963-014CE','deb West Ford','',9245,'Reentered','1987-10-17 00:00:00','1968-05-20 00:00:00',163.16,963,6063,85.26,'Medium' from dual union all
select 32640,'032640','2006-057DP','USA 193 debris','',68229,'Reentered','2008-03-26 00:00:00','2008-03-04 00:00:00',94,240,701,55.81,'Intermediate' from dual union all
select 32641,'032641','2006-057DQ','USA 193 debris','',68229,'Reentered','2008-03-07 00:00:00','2008-03-04 00:00:00',94.38,212,766,58.53,'Intermediate' from dual union all
select 32642,'032642','2006-057DR','USA 193 debris','',68229,'Reentered','2008-04-11 00:00:00','2008-03-05 00:00:00',96.61,241,951,56.5,'Intermediate' from dual union all
select 32643,'032643','2006-057DS','USA 193 debris','',68229,'Reentered','2008-03-08 00:00:00','2008-03-05 00:00:00',90.08,218,338,58.26,'Intermediate' from dual union all
select 32644,'032644','2006-057DT','USA 193 debris','',68229,'Reentered','2008-03-05 00:00:00','2008-03-05 00:00:00',89.39,194,295,59.75,'Intermediate' from dual union all
select 32645,'032645','2006-057DU','USA 193 debris','',68229,'Reentered','2008-03-05 00:00:00','2008-03-04 00:00:00',88.13,162,201,58.18,'Intermediate' from dual union all
select 32646,'032646','2006-057DV','USA 193 debris','',68229,'Reentered','2008-06-22 00:00:00','2008-03-05 00:00:00',95.29,231,835,57.47,'Intermediate' from dual union all
select 32647,'032647','2006-057DW','USA 193 debris','',68229,'Reentered','2008-03-06 00:00:00','2008-03-04 00:00:00',88.4,184,207,58.34,'Intermediate' from dual union all
select 32648,'032648','2006-057DX','USA 193 debris','',68229,'Reentered','2008-03-05 00:00:00','2008-03-05 00:00:00',88.13,170,194,58.64,'Intermediate' from dual union all
select 32649,'032649','2006-057DY','USA 193 debris','',68229,'Reentered','2008-03-20 00:00:00','2008-03-05 00:00:00',90.5,231,368,58.34,'Intermediate' from dual union all
select 3265,'003265','1968-041C','ESRO 2B despin weight','',16768,'Reentered','1968-11-04 00:00:00','1968-06-16 00:00:00',97.9,329,988,97.06,'Sun-Synch' from dual union all
select 32650,'032650','2006-057DZ','USA 193 debris','',68229,'Reentered','2008-06-19 00:00:00','2008-03-05 00:00:00',90.7,237,381,58.22,'Intermediate' from dual union all
select 32651,'032651','2006-057EA','USA 193 debris','',68229,'Reentered','2008-05-08 00:00:00','2008-03-04 00:00:00',92.38,238,545,58.63,'Intermediate' from dual union all
select 32652,'032652','2006-057EB','USA 193 debris','',68229,'Reentered','2008-03-17 00:00:00','2008-03-05 00:00:00',94.94,237,795,58.95,'Intermediate' from dual union all
select 32653,'032653','2006-057EC','USA 193 debris','',68229,'Reentered','2008-03-18 00:00:00','2008-03-05 00:00:00',95.38,237,836,58.96,'Intermediate' from dual union all
select 32654,'032654','2006-057ED','USA 193 debris','',68229,'Reentered','2008-03-15 00:00:00','2008-03-05 00:00:00',94.59,233,765,59.35,'Intermediate' from dual union all
select 32655,'032655','2006-057EE','USA 193 debris','',68229,'Reentered','2008-04-09 00:00:00','2008-03-05 00:00:00',93.97,236,701,57.92,'Intermediate' from dual union all
select 32656,'032656','2006-057EF','USA 193 debris','',68229,'Reentered','2008-03-26 00:00:00','2008-03-05 00:00:00',94,236,704,56.7,'Intermediate' from dual union all
select 32657,'032657','2006-057EG','USA 193 debris','',68229,'Reentered','2008-10-09 00:00:00','2008-03-05 00:00:00',95.07,243,801,57.69,'Intermediate' from dual union all
select 32658,'032658','2006-057EH','USA 193 debris','',68229,'Reentered','2008-03-08 00:00:00','2008-03-05 00:00:00',91.54,228,473,59.39,'Intermediate' from dual union all
select 32659,'032659','2006-057EJ','USA 193 debris','',68229,'Reentered','2008-03-14 00:00:00','2008-03-05 00:00:00',93.2,236,626,57.71,'Intermediate' from dual union all
select 3266,'003266','1968-042A','OPS 7869','DAPP 5420',16769,'In Earth orbit','','1968-06-24 00:00:00',102.12,817,901,98.94,'Sun-Synch' from dual union all
select 32660,'032660','2006-057EK','USA 193 debris','',68229,'Reentered','2008-04-24 00:00:00','2008-03-05 00:00:00',94.52,242,749,57.69,'Intermediate' from dual union all
select 32661,'032661','2006-057EL','USA 193 debris','',68229,'Reentered','2008-03-10 00:00:00','2008-03-05 00:00:00',91.03,230,421,58.29,'Intermediate' from dual union all
select 32662,'032662','2006-057EM','USA 193 debris','',68229,'Reentered','2008-03-06 00:00:00','2008-03-05 00:00:00',90.25,193,380,59.7,'Intermediate' from dual union all
select 32663,'032663','2006-057EN','USA 193 debris','',68229,'Reentered','2008-03-11 00:00:00','2008-03-05 00:00:00',91.59,228,478,58.71,'Intermediate' from dual union all
select 32664,'032664','2006-057EP','USA 193 debris','',68229,'Reentered','2008-04-06 00:00:00','2008-03-06 00:00:00',99.13,237,1197,58.99,'Intermediate' from dual union all
select 32665,'032665','2006-057EQ','USA 193 debris','',68229,'Reentered','2008-04-01 00:00:00','2008-03-05 00:00:00',98.6,238,1145,58.64,'Intermediate' from dual union all
select 32666,'032666','2006-057ER','USA 193 debris','',68229,'Reentered','2008-03-12 00:00:00','2008-03-06 00:00:00',89.39,217,271,58.22,'Intermediate' from dual union all
select 32667,'032667','2006-057ES','USA 193 debris','',68229,'Reentered','2008-03-19 00:00:00','2008-03-06 00:00:00',90.77,222,403,57.73,'Intermediate' from dual union all
select 32668,'032668','2006-057ET','USA 193 debris','',68229,'Reentered','2008-04-02 00:00:00','2008-03-06 00:00:00',89.96,236,309,58.28,'Intermediate' from dual union all
select 32669,'032669','2006-057EU','USA 193 debris','',68229,'Reentered','2008-03-27 00:00:00','2008-03-06 00:00:00',90.36,234,351,58.63,'Intermediate' from dual union all
select 3267,'003267','1963-014CF','deb West Ford','[UNID017] RCS 0.04',9245,'In Earth orbit','','1992-02-02 00:00:00',127.58,576,3458,88.38,'Medium' from dual union all
select 32670,'032670','2006-057EV','USA 193 debris','',68229,'Reentered','2008-03-08 00:00:00','2008-03-06 00:00:00',90.13,216,346,57.8,'Intermediate' from dual union all
select 32671,'032671','2006-057EW','USA 193 debris','',68229,'Reentered','2008-04-09 00:00:00','2008-03-06 00:00:00',92.44,240,548,58.02,'Intermediate' from dual union all
select 32672,'032672','2006-057EX','USA 193 debris','',68229,'Reentered','2008-04-03 00:00:00','2008-03-07 00:00:00',96.02,238,897,56.46,'Intermediate' from dual union all
select 32673,'032673','2006-057EY','USA 193 debris','',68229,'Reentered','2008-03-31 00:00:00','2008-03-07 00:00:00',93.17,200,660,58.77,'Intermediate' from dual union all
select 32674,'032674','2006-057EZ','USA 193 debris','',68229,'Reentered','2008-03-07 00:00:00','2008-03-06 00:00:00',88.06,161,196,58.4,'Intermediate' from dual union all
select 32675,'032675','2006-057FA','USA 193 debris','',68229,'Reentered','2008-05-08 00:00:00','2008-03-07 00:00:00',90.46,228,367,58.51,'Intermediate' from dual union all
select 32676,'032676','2006-057FB','USA 193 debris','',68229,'Reentered','2008-03-21 00:00:00','2008-03-07 00:00:00',89.38,227,261,58.69,'Intermediate' from dual union all
select 32677,'032677','2006-057FC','USA 193 debris','',68229,'Reentered','2008-03-20 00:00:00','2008-03-07 00:00:00',89.38,222,265,58.78,'Intermediate' from dual union all
select 32678,'032678','2006-057FD','USA 193 debris','',68229,'Reentered','2008-03-14 00:00:00','2008-03-07 00:00:00',91,222,425,58.17,'Intermediate' from dual;
insert into SATELLITE
select 32679,'032679','2006-057FE','USA 193 debris','',68229,'Reentered','2008-04-17 00:00:00','2008-03-07 00:00:00',96.36,225,944,57.97,'Intermediate' from dual union all
select 3268,'003268','1963-014CG','deb West Ford','',9245,'Reentered','1969-08-01 00:00:00','1968-05-21 00:00:00',161.49,690,6201,84.63,'Medium' from dual union all
select 32680,'032680','2006-057FF','USA 193 debris','',68229,'Reentered','2008-03-17 00:00:00','2008-03-07 00:00:00',94.84,228,794,57.72,'Intermediate' from dual union all
select 32681,'032681','2006-057FG','USA 193 debris','',68229,'Reentered','2008-04-02 00:00:00','2008-03-07 00:00:00',94.35,234,740,58.44,'Intermediate' from dual union all
select 32682,'032682','2006-057FH','USA 193 debris','',68229,'Reentered','2008-03-21 00:00:00','2008-03-07 00:00:00',94.31,231,739,56.61,'Intermediate' from dual union all
select 32683,'032683','2006-057FJ','USA 193 debris','',68229,'Reentered','2008-03-28 00:00:00','2008-03-07 00:00:00',93.62,231,673,58.41,'Intermediate' from dual union all
select 32684,'032684','2006-057FK','USA 193 debris','',68229,'Reentered','2008-03-15 00:00:00','2008-03-07 00:00:00',92.51,229,566,59.93,'Intermediate' from dual union all
select 32685,'032685','2006-057FL','USA 193 debris','',68229,'Reentered','2008-05-21 00:00:00','2008-03-07 00:00:00',93.08,235,616,57.63,'Intermediate' from dual union all
select 32686,'032686','2008-008A','Jules Verne','ATV 1',68516,'Deorbited','2008-09-29 00:00:00','2008-03-16 00:00:00',90.7,307,311,51.64,'Intermediate' from dual union all
select 32687,'032687','2008-008B','EPS 528','EPS+ 528',68516,'Deorbited','2008-03-09 00:00:00','2008-03-09 00:00:00',89.62,252,260,51.64,'Intermediate' from dual union all
select 32688,'032688','2006-057FM','USA 193 debris','',68229,'Reentered','2008-03-17 00:00:00','2008-03-10 00:00:00',88.78,210,219,58.67,'Intermediate' from dual union all
select 32689,'032689','2006-057FN','USA 193 debris','',68229,'Reentered','2008-04-01 00:00:00','2008-03-10 00:00:00',89.72,232,290,58.87,'Intermediate' from dual union all
select 3269,'003269','1968-043A','Kosmos-221','DS-P1-Yu No. 14',16770,'Reentered','1969-08-31 00:00:00','1968-06-23 00:00:00',107.53,214,2011,48.42,'Intermediate' from dual union all
select 32690,'032690','2006-057FP','USA 193 debris','',68229,'Reentered','2008-03-15 00:00:00','2008-03-09 00:00:00',90.62,223,387,56.04,'Intermediate' from dual union all
select 32691,'032691','2006-057FQ','USA 193 debris','',68229,'Reentered','2008-03-31 00:00:00','2008-03-09 00:00:00',98.51,225,1149,58.61,'Intermediate' from dual union all
select 32692,'032692','2006-057FR','USA 193 debris','',68229,'Reentered','2008-03-12 00:00:00','2008-03-10 00:00:00',89.39,196,293,58.28,'Intermediate' from dual union all
select 32693,'032693','2006-057FS','USA 193 debris','',68229,'Reentered','2008-03-13 00:00:00','2008-03-10 00:00:00',88.7,198,223,58.6,'Intermediate' from dual union all
select 32694,'032694','2006-057FT','USA 193 debris','',68229,'Reentered','2008-03-14 00:00:00','2008-03-09 00:00:00',89.93,220,322,58.6,'Intermediate' from dual union all
select 32695,'032695','2006-057FU','USA 193 debris','',68229,'Reentered','2008-04-21 00:00:00','2008-03-09 00:00:00',103.06,237,1570,58.87,'Intermediate' from dual union all
select 32696,'032696','2006-057FV','USA 193 debris','',68229,'Reentered','2008-04-05 00:00:00','2008-03-10 00:00:00',104.02,234,1663,57.48,'Intermediate' from dual union all
select 32697,'032697','2006-057FW','USA 193 debris','',68229,'Reentered','2008-03-29 00:00:00','2008-03-09 00:00:00',104.51,236,1706,57.71,'Intermediate' from dual union all
select 32698,'032698','2006-057FX','USA 193 debris','',68229,'Reentered','2008-04-18 00:00:00','2008-03-10 00:00:00',112.53,212,2474,58.08,'Intermediate' from dual union all
select 32699,'032699','2008-009A','Endeavour (STS-123)','OV-105',68517,'Landed','2008-03-27 00:00:00','2008-03-18 00:00:00',91.34,340,341,51.64,'Intermediate' from dual union all
select 327,'000327','61 Omicron 100','deb r/b Ablestar','',7273,'In Earth orbit','','1962-06-27 00:00:00',104.29,876,1047,66.58,'Intermediate' from dual union all
select 3270,'003270','1968-043B','S1M','S1M',16770,'Reentered','1969-04-13 00:00:00','1968-06-23 00:00:00',106.69,216,1931,48.41,'Intermediate' from dual union all
select 32700,'032700','2006-057FY','USA 193 debris','',68229,'Reentered','2008-03-19 00:00:00','2008-03-10 00:00:00',97.92,232,1087,58.24,'Intermediate' from dual union all
select 32701,'032701','2006-057FZ','USA 193 debris','',68229,'Reentered','2008-03-11 00:00:00','2008-03-10 00:00:00',88.8,185,246,57.68,'Intermediate' from dual union all
select 32702,'032702','2006-057GA','USA 193 debris','',68229,'Reentered','2008-03-15 00:00:00','2008-03-11 00:00:00',93.23,208,658,58.32,'Intermediate' from dual union all
select 32703,'032703','2006-057GB','USA 193 debris','',68229,'Reentered','2008-03-14 00:00:00','2008-03-12 00:00:00',88.92,183,260,58.7,'Intermediate' from dual union all
select 32704,'032704','2006-057GC','USA 193 debris','',68229,'Reentered','2008-03-14 00:00:00','2008-03-11 00:00:00',92.23,220,548,60.49,'Intermediate' from dual union all
select 32705,'032705','1977-093E','Blok SO-L','Blok SO-L',42221,'Deep Space','1977-09-22 00:00:00','2008-03-13 00:00:00',5578.55,33652,161977,48.14,'Deep Space' from dual union all
select 32706,'032706','2008-010A','USA 200','RAVEN 5/HEO-2',68518,'In Earth orbit','','2008-03-20 00:00:00',684.33,1112,37580,63.56,'GEO Transfer' from dual union all
select 32707,'032707','2008-010B','Centaur AV-006','Centaur AV-006',68518,'Reentered','2015-11-03 00:00:00','2008-03-20 00:00:00',684.33,1112,37580,63.56,'GEO Transfer' from dual union all
select 32708,'032708','2008-011A','AMC 14','Americom 14',68519,'In Earth orbit','','2008-12-19 00:00:00',1435.58,35094,36459,13.11,'Inclined GEO' from dual union all
select 32709,'032709','2008-011B','Briz-M No. 88525','Briz-M 88525',68519,'In Earth orbit','','2008-03-17 00:00:00',468.7,768,26451,49.21,'Highly Ellip' from dual union all
select 3271,'003271','1968-042B','Burner II No. 6','Star 37B',16769,'In Earth orbit','','1968-09-20 00:00:00',102.14,817,903,98.87,'Sun-Synch' from dual union all
select 32710,'032710','2006-026C','deb Kosmos-2421','',68198,'Reentered','2008-04-20 00:00:00','2008-03-19 00:00:00',92.59,393,410,65.04,'Intermediate' from dual union all
select 32711,'032711','2008-012A','GPS IIR-19(M)','Navstar GPS SVN 48',68520,'In Earth orbit','','2008-05-18 00:00:00',717.98,20143,20222,55.13,'Medium' from dual union all
select 32712,'032712','2008-012B','Star 48B','Star 48B',68520,'In Earth orbit','','2008-03-17 00:00:00',355.37,190,20293,40.01,'Highly Ellip' from dual union all
select 32713,'032713','2008-012C','Delta 332','Delta SSPS AJ-10-118K',68520,'Reentered','2008-09-08 00:00:00','2008-03-17 00:00:00',98.84,187,1219,33.17,'Intermediate' from dual union all
select 32714,'032714','2006-057GD','USA 193 debris','',68229,'Reentered','2008-03-23 00:00:00','2008-03-17 00:00:00',94.89,217,810,58.68,'Intermediate' from dual union all
select 32715,'032715','2006-026D','deb Kosmos-2421','',68198,'Reentered','2008-05-04 00:00:00','2008-03-19 00:00:00',92.6,394,410,65.04,'Intermediate' from dual union all
select 32716,'032716','2006-026E','deb Kosmos-2421','',68198,'Reentered','2008-04-19 00:00:00','2008-03-19 00:00:00',92.68,394,417,65.05,'Intermediate' from dual union all
select 32717,'032717','2006-026F','deb Kosmos-2421','',68198,'Reentered','2008-04-09 00:00:00','2008-03-19 00:00:00',92.42,388,399,65.04,'Intermediate' from dual union all
select 32718,'032718','2006-026G','deb Kosmos-2421','',68198,'Reentered','2008-09-09 00:00:00','2008-03-19 00:00:00',92.4,324,460,65.12,'Intermediate' from dual union all
select 32719,'032719','2006-026H','deb Kosmos-2421','',68198,'Reentered','2008-04-22 00:00:00','2008-03-19 00:00:00',92.26,347,424,65.07,'Intermediate' from dual union all
select 3272,'003272','1968-044A','Kosmos-222','DS-P1-Yu No. 12',16771,'Reentered','1968-10-11 00:00:00','1968-06-30 00:00:00',91.78,259,465,70.92,'Intermediate' from dual union all
select 32720,'032720','2006-026J','deb Kosmos-2421','',68198,'Reentered','2008-09-21 00:00:00','2008-03-19 00:00:00',93.4,396,486,65.22,'Intermediate' from dual union all
select 32721,'032721','2006-026K','deb Kosmos-2421','',68198,'Reentered','2010-03-10 00:00:00','2008-03-19 00:00:00',93.33,400,475,65.13,'Intermediate' from dual union all
select 32722,'032722','2006-026L','deb Kosmos-2421','',68198,'Reentered','2008-05-08 00:00:00','2008-03-19 00:00:00',93.16,397,462,65.04,'Intermediate' from dual union all
select 32723,'032723','2006-026M','deb Kosmos-2421','',68198,'Reentered','2008-05-02 00:00:00','2008-03-19 00:00:00',92.92,395,441,65.04,'Intermediate' from dual union all
select 32724,'032724','2006-026N','deb Kosmos-2421','',68198,'Reentered','2008-05-01 00:00:00','2008-03-19 00:00:00',92.91,395,440,65.03,'Intermediate' from dual union all
select 32725,'032725','2006-026P','deb Kosmos-2421','',68198,'Reentered','2008-03-30 00:00:00','2008-03-18 00:00:00',91.52,325,373,65.15,'Intermediate' from dual union all
select 32726,'032726','2006-026Q','deb Kosmos-2421','',68198,'Reentered','2008-03-28 00:00:00','2008-03-19 00:00:00',93.08,361,489,65.2,'Intermediate' from dual union all
select 32727,'032727','2006-026R','deb Kosmos-2421','',68198,'Reentered','2008-03-25 00:00:00','2008-03-19 00:00:00',91.37,298,385,64.93,'Intermediate' from dual union all
select 32728,'032728','2006-026S','deb Kosmos-2421','',68198,'Reentered','2008-03-21 00:00:00','2008-03-19 00:00:00',92.14,350,409,65.01,'Intermediate' from dual union all
select 32729,'032729','2008-013A','DirecTV 11','DirecTV-11',68521,'In Earth orbit','','2008-04-28 00:00:00',1440.1,32302,39428,.1,'Near-synch' from dual union all
select 3273,'003273','1968-044B','S1M','S1M',16771,'Reentered','1968-08-13 00:00:00','1968-07-01 00:00:00',91.18,263,401,70.92,'Intermediate' from dual union all
select 32730,'032730','2008-013B','Blok DM-SL No. 26L','Blok DM-SL',68521,'In Earth orbit','','2008-03-24 00:00:00',643.49,228,36396,.05,'GEO Transfer' from dual union all
select 32731,'032731','2006-057GE','USA 193 debris','',68229,'Reentered','2008-03-25 00:00:00','2008-03-24 00:00:00',88.95,201,245,58.43,'Intermediate' from dual union all
select 32732,'032732','2006-026T','deb Kosmos-2421','',68198,'Reentered','2008-05-20 00:00:00','2008-03-24 00:00:00',92.83,393,434,65.04,'Intermediate' from dual union all
select 32733,'032733','2006-026U','deb Kosmos-2421','',68198,'Reentered','2008-04-05 00:00:00','2008-03-24 00:00:00',92.09,360,394,65.03,'Intermediate' from dual union all
select 32734,'032734','2006-026V','deb Kosmos-2421','',68198,'Reentered','2008-09-17 00:00:00','2008-03-24 00:00:00',94.6,376,623,65.85,'Intermediate' from dual union all
select 32735,'032735','2006-026W','deb Kosmos-2421','',68198,'Reentered','2008-08-14 00:00:00','2008-03-24 00:00:00',93.88,396,532,65.01,'Intermediate' from dual union all
select 32736,'032736','2006-026X','deb Kosmos-2421','',68198,'Reentered','2008-05-12 00:00:00','2008-03-24 00:00:00',92.53,390,407,65.05,'Intermediate' from dual union all
select 32737,'032737','2006-026Y','deb Kosmos-2421','',68198,'Reentered','2008-06-28 00:00:00','2008-03-24 00:00:00',93.44,394,492,65.29,'Intermediate' from dual union all
select 32738,'032738','2006-026Z','deb Kosmos-2421','',68198,'Reentered','2008-04-12 00:00:00','2008-03-24 00:00:00',92.32,379,398,65.05,'Intermediate' from dual union all
select 32739,'032739','2006-026AA','deb Kosmos-2421','',68198,'Reentered','2008-04-13 00:00:00','2008-03-24 00:00:00',92.38,380,403,65.05,'Intermediate' from dual union all
select 3274,'003274','1968-045A','Kosmos-223','Zenit-2 No. 63',16772,'Landed','1968-06-09 00:00:00','1968-06-05 00:00:00',89.81,202,329,72.86,'Intermediate' from dual union all
select 32740,'032740','2006-026AB','deb Kosmos-2421','',68198,'Reentered','2008-04-13 00:00:00','2008-03-24 00:00:00',92.29,384,390,65.05,'Intermediate' from dual union all
select 32741,'032741','2006-026AC','deb Kosmos-2421','',68198,'Reentered','2009-08-31 00:00:00','2008-03-24 00:00:00',93.09,398,454,65.27,'Intermediate' from dual union all
select 32742,'032742','2006-026AD','deb Kosmos-2421','',68198,'Reentered','2008-04-15 00:00:00','2008-03-24 00:00:00',94.31,398,573,64.99,'Intermediate' from dual union all
select 32743,'032743','2006-026AE','deb Kosmos-2421','',68198,'Reentered','2008-04-19 00:00:00','2008-03-24 00:00:00',93.03,372,474,65.08,'Intermediate' from dual union all
select 32744,'032744','2006-026AF','deb Kosmos-2421','',68198,'Reentered','2008-05-05 00:00:00','2008-03-24 00:00:00',92.96,392,447,65.05,'Intermediate' from dual union all
select 32745,'032745','2006-026AG','deb Kosmos-2421','',68198,'Reentered','2009-01-07 00:00:00','2008-03-24 00:00:00',94.01,394,548,65.29,'Intermediate' from dual union all
select 32746,'032746','2006-026AH','deb Kosmos-2421','',68198,'Reentered','2008-04-17 00:00:00','2008-03-24 00:00:00',92.47,385,407,65.05,'Intermediate' from dual union all
select 32747,'032747','2006-026AJ','deb Kosmos-2421','',68198,'Reentered','2008-04-09 00:00:00','2008-03-24 00:00:00',92.26,381,390,65.01,'Intermediate' from dual union all
select 32748,'032748','2006-026AK','deb Kosmos-2421','',68198,'Reentered','2008-04-29 00:00:00','2008-03-24 00:00:00',93.63,393,512,65.26,'Intermediate' from dual union all
select 32749,'032749','2006-057GF','USA 193 debris','',68229,'Reentered','2008-03-27 00:00:00','2008-03-23 00:00:00',89.41,205,285,58.32,'Intermediate' from dual union all
select 3275,'003275','1968-045B','11A57I','11A57I',16772,'Reentered','1968-06-09 00:00:00','1968-06-03 00:00:00',89.56,191,315,72.88,'Intermediate' from dual union all
select 32750,'032750','2008-014A','SAR-Lupe 4','SAR-Lupe 4',68522,'In Earth orbit','','2008-03-27 00:00:00',94.34,468,506,98.16,'Sun-Synch' from dual union all
select 32751,'032751','2008-014B','S3M','S3M',68522,'Reentered','2014-11-23 00:00:00','2008-03-27 00:00:00',94.45,467,517,98.16,'Sun-Synch' from dual union all
select 32752,'032752','2006-026AL','deb Kosmos-2421','',68198,'Reentered','2008-06-01 00:00:00','2008-03-27 00:00:00',92.54,389,409,65.05,'Intermediate' from dual union all
select 32753,'032753','2006-026AM','deb Kosmos-2421','',68198,'Reentered','2008-04-24 00:00:00','2008-03-27 00:00:00',92.65,388,421,65.02,'Intermediate' from dual union all
select 32754,'032754','2006-026AN','deb Kosmos-2421','',68198,'Reentered','2008-04-14 00:00:00','2008-03-27 00:00:00',92.36,379,402,65.04,'Intermediate' from dual union all
select 32755,'032755','2006-057GG','USA 193 debris','',68229,'Reentered','2008-03-31 00:00:00','2008-03-26 00:00:00',93.94,224,711,58.8,'Intermediate' from dual union all
select 32756,'032756','2008-015A','Soyuz TMA-12','Soyuz 11F732 No. 222',68523,'Landed','2008-10-24 00:00:00','2008-04-16 00:00:00',91.27,336,338,51.64,'Intermediate' from dual union all
select 32757,'032757','2008-015B','11S510','11S510',68523,'Reentered','2008-04-11 00:00:00','2008-04-09 00:00:00',88.32,179,204,51.6,'Intermediate' from dual union all
select 32758,'032758','2006-026AP','deb Kosmos-2421','',68198,'Reentered','2008-04-27 00:00:00','2008-04-11 00:00:00',92.55,356,443,65.01,'Intermediate' from dual union all
select 32759,'032759','2006-026AQ','deb Kosmos-2421','',68198,'Reentered','2008-07-09 00:00:00','2008-04-11 00:00:00',92.39,380,403,65.04,'Intermediate' from dual union all
select 3276,'003276','1968-046A','Kosmos-224','Zenit-4 No. 46',16773,'Landed','1968-06-12 00:00:00','1968-06-06 00:00:00',89.03,200,254,51.82,'Intermediate' from dual union all
select 32760,'032760','2006-026AR','deb Kosmos-2421','',68198,'Reentered','2008-05-03 00:00:00','2008-04-11 00:00:00',92.3,369,406,65.03,'Intermediate' from dual union all
select 32761,'032761','2006-026AS','deb Kosmos-2421','',68198,'Reentered','2008-04-15 00:00:00','2008-04-14 00:00:00',89.29,227,252,65.03,'Intermediate' from dual union all
select 32762,'032762','2006-026AT','deb Kosmos-2421','',68198,'Reentered','2008-04-16 00:00:00','2008-04-13 00:00:00',90.46,286,309,65.04,'Intermediate' from dual union all
select 32763,'032763','2008-016A','ICO G1','ICO G1',68524,'In Earth orbit','','2008-04-28 00:00:00',1436.06,35771,35801,5.99,'Inclined GEO' from dual union all
select 32764,'032764','2008-016B','Centaur AV-014','Centaur AV-014',68524,'Reentered','2009-02-03 00:00:00','2008-04-17 00:00:00',632.17,138,35905,22.46,'GEO Transfer' from dual union all
select 32765,'032765','2008-017A','C/NOFS','C/NOFS',68525,'Reentered','2015-11-28 00:00:00','2008-04-19 00:00:00',97.28,405,852,13,'Equatorial' from dual union all
select 32766,'032766','2008-017B','Orion 38','Orion 38',68525,'In Earth orbit','','2008-04-18 00:00:00',97.14,405,839,13,'Equatorial' from dual union all
select 32767,'032767','2008-018A','Vinasat','Vinasat',68526,'In Earth orbit','','2008-05-08 00:00:00',1436.09,35782,35791,.02,'Stationary' from dual union all
select 32768,'032768','2008-018B','Star One C2','Star One C2',68526,'In Earth orbit','','2008-05-01 00:00:00',1436.01,35784,35785,.07,'Stationary' from dual;
insert into SATELLITE
select 32769,'032769','2008-018C','ESC-A 539','Ariane 5 ESC-A',68526,'In Earth orbit','','2008-04-18 00:00:00',630.34,272,35677,2.02,'GEO Transfer' from dual union all
select 3277,'003277','1968-046B','11A57I','11A57I',16773,'Reentered','1968-06-07 00:00:00','1968-06-04 00:00:00',88.88,195,244,51.82,'Intermediate' from dual union all
select 32770,'032770','2008-018D','Sylda 5A','Sylda 5',68526,'Reentered','2014-09-26 00:00:00','2008-04-18 00:00:00',632.07,270,35768,2.05,'GEO Transfer' from dual union all
select 32771,'032771','1982-029E','deb Kosmos-1348','',54338,'In Earth orbit','','2008-04-22 00:00:00',734.96,5753,35446,69.4,'GEO Transfer' from dual union all
select 32772,'032772','1982-029F','deb Kosmos-1348','',54338,'In Earth orbit','','2008-04-23 00:00:00',743.88,5705,35930,69.33,'GEO Transfer' from dual union all
select 32773,'032773','1982-029G','deb Kosmos-1348','',54338,'In Earth orbit','','2008-04-23 00:00:00',708.96,5381,34538,70.02,'GEO Transfer' from dual union all
select 32774,'032774','1982-029H','deb Kosmos-1348','',54338,'In Earth orbit','','2008-04-23 00:00:00',735.88,5761,35483,69.61,'GEO Transfer' from dual union all
select 32775,'032775','1982-029J','deb Kosmos-1348','',54338,'In Earth orbit','','2008-04-23 00:00:00',732.2,5834,35230,69.5,'GEO Transfer' from dual union all
select 32776,'032776','1982-029K','deb Kosmos-1348','',54338,'In Earth orbit','','2008-04-21 00:00:00',725.19,5833,34887,70.11,'GEO Transfer' from dual union all
select 32777,'032777','1982-029L','deb Kosmos-1348','',54338,'In Earth orbit','','2008-04-23 00:00:00',726.96,5745,35062,69.87,'GEO Transfer' from dual union all
select 32778,'032778','1982-029M','deb Kosmos-1348','',54338,'In Earth orbit','','2007-10-13 00:00:00',734.61,5513,35669,69.42,'GEO Transfer' from dual union all
select 32779,'032779','2008-019A','Tian Lian 1A','tianlian yihao 01 xing',68527,'In Earth orbit','','2008-05-02 00:00:00',1436.19,35768,35809,.51,'Stationary' from dual union all
select 3278,'003278','1968-047A','OPS 5138','GAMBIT-3 Mission 4314',16774,'Reentered Attached','1968-06-17 00:00:00','1968-06-07 00:00:00',89.91,120,419,110.53,'Retrograde' from dual union all
select 32780,'032780','2008-019B','CZ-3C Stage 3','CZ-3C Stage 3',68527,'Reentered','2011-03-05 00:00:00','2008-05-01 00:00:00',747.85,157,41671,17.98,'GEO Transfer' from dual union all
select 32781,'032781','2008-020A','Giove-B','Galileo IOV Experiment B',68528,'In Earth orbit','','2008-04-29 00:00:00',842.64,23104,23244,55.98,'Medium' from dual union all
select 32782,'032782','2008-020B','Fregat No. 1008','Fregat 14S44 No. 1008',68528,'In Earth orbit','','2008-04-29 00:00:00',857.35,23431,23603,55.94,'Medium' from dual union all
select 32783,'032783','2008-021A','Cartosat 2A','Cartosat-2A',68529,'In Earth orbit','','2008-05-02 00:00:00',97.24,618,635,98,'Sun-Synch' from dual union all
select 32784,'032784','2008-021B','NTS','CanX-6',68529,'In Earth orbit','','2008-05-02 00:00:00',97.22,616,635,98,'Sun-Synch' from dual union all
select 32785,'032785','2008-021C','Cubesat-OSCAR-65','Cute-1.7/APD 2',68529,'In Earth orbit','','2008-05-02 00:00:00',97.19,615,634,98,'Sun-Synch' from dual union all
select 32786,'032786','2008-021D','IMS-1','Indian Mini Satellite 1',68529,'In Earth orbit','','2008-05-02 00:00:00',97.24,618,635,98,'Sun-Synch' from dual union all
select 32787,'032787','2008-021E','Compass-1','Compass-1',68529,'In Earth orbit','','2008-05-02 00:00:00',97.2,615,635,97.99,'Sun-Synch' from dual union all
select 32788,'032788','2008-021F','AAUSat-II','AAUSat-II',68529,'In Earth orbit','','2008-05-02 00:00:00',97.18,614,633,97.99,'Sun-Synch' from dual union all
select 32789,'032789','2008-021G','Delfi-OSCAR-64','Delfi-C3',68529,'In Earth orbit','','2008-05-02 00:00:00',97.18,615,633,97.99,'Sun-Synch' from dual union all
select 3279,'003279','1968-048A','Kosmos-225','DS-U1-Ya No. 2',16775,'Reentered','1968-11-02 00:00:00','1968-07-12 00:00:00',91.96,250,492,48.41,'Intermediate' from dual union all
select 32790,'032790','2008-021H','CanX-2','CanX-2',68529,'In Earth orbit','','2008-05-02 00:00:00',97.18,614,633,97.99,'Sun-Synch' from dual union all
select 32791,'032791','2008-021J','Cubesat-OSCAR-66','SEEDS 2',68529,'In Earth orbit','','2008-05-02 00:00:00',97.19,615,634,98,'Sun-Synch' from dual union all
select 32792,'032792','2008-021K','PS4','PS4 PSLV-C10',68529,'In Earth orbit','','2008-05-01 00:00:00',97.53,619,662,97.92,'Sun-Synch' from dual union all
select 32793,'032793','2006-026AU','deb Kosmos-2421','',68198,'Reentered','2008-05-08 00:00:00','2008-05-02 00:00:00',90.85,304,328,65.03,'Intermediate' from dual union all
select 32794,'032794','2008-022A','Amos 3','Amos 3',68530,'In Earth orbit','','2008-05-05 00:00:00',1479.42,35761,37500,.72,'Drift GEO' from dual union all
select 32795,'032795','2006-026CX','deb Kosmos-2421','',68198,'Reentered','2008-07-13 00:00:00','2008-05-14 00:00:00',92.59,380,423,65.1,'Intermediate' from dual union all
select 32796,'032796','2006-026AV','deb Kosmos-2421','',68198,'Reentered','2008-05-05 00:00:00','2008-05-03 00:00:00',90.08,265,291,65.04,'Intermediate' from dual union all
select 32797,'032797','2008-021L','deb PSLV','',68529,'Reentered','2012-10-21 00:00:00','2008-05-05 00:00:00',97.19,615,634,97.99,'Sun-Synch' from dual union all
select 32798,'032798','2006-026AW','deb Kosmos-2421','',68198,'Reentered','2008-06-20 00:00:00','2008-05-08 00:00:00',92.49,382,412,65.05,'Intermediate' from dual union all
select 32799,'032799','2006-026AX','deb Kosmos-2421','',68198,'Reentered','2009-10-09 00:00:00','2008-05-08 00:00:00',92.72,394,421,65.05,'Intermediate' from dual union all
select 328,'000328','61 Omicron 101','deb r/b Ablestar','',7273,'Reentered','1970-05-03 00:00:00','1962-06-29 00:00:00',100.44,599,960,66.69,'Intermediate' from dual union all
select 3280,'003280','1968-048B','S1M','S1M',16775,'Reentered','1968-08-15 00:00:00','1968-07-12 00:00:00',91.25,245,427,48.43,'Intermediate' from dual union all
select 32800,'032800','2006-026AY','deb Kosmos-2421','',68198,'Reentered','2008-08-22 00:00:00','2008-05-08 00:00:00',92.43,389,399,65,'Intermediate' from dual union all
select 32801,'032801','2006-026AZ','deb Kosmos-2421','',68198,'Reentered','2010-04-06 00:00:00','2008-05-08 00:00:00',96.2,395,759,65.15,'Intermediate' from dual union all
select 32802,'032802','2006-026BA','deb Kosmos-2421','',68198,'Reentered','2008-10-28 00:00:00','2008-05-08 00:00:00',92.83,386,441,65.08,'Intermediate' from dual union all
select 32803,'032803','2006-026BB','deb Kosmos-2421','',68198,'Reentered','2008-06-30 00:00:00','2008-05-08 00:00:00',92.69,382,431,65.11,'Intermediate' from dual union all
select 32804,'032804','2006-026BC','deb Kosmos-2421','',68198,'Reentered','2008-07-19 00:00:00','2008-05-08 00:00:00',92.56,385,415,65.05,'Intermediate' from dual union all
select 32805,'032805','2006-026BD','deb Kosmos-2421','',68198,'Reentered','2008-09-06 00:00:00','2008-05-08 00:00:00',92.61,391,413,65.07,'Intermediate' from dual union all
select 32806,'032806','2006-026BE','deb Kosmos-2421','',68198,'Reentered','2008-07-02 00:00:00','2008-05-08 00:00:00',92.65,382,426,65.1,'Intermediate' from dual union all
select 32807,'032807','2006-026BF','deb Kosmos-2421','',68198,'Reentered','2008-08-13 00:00:00','2008-05-08 00:00:00',92.45,388,402,65,'Intermediate' from dual union all
select 32808,'032808','2006-026BG','deb Kosmos-2421','',68198,'Reentered','2009-04-25 00:00:00','2008-05-08 00:00:00',94.19,397,562,65.31,'Intermediate' from dual union all
select 32809,'032809','2006-026BH','deb Kosmos-2421','',68198,'Reentered','2008-06-09 00:00:00','2008-05-08 00:00:00',91.59,322,384,63.81,'Intermediate' from dual union all
select 3281,'003281','1968-048C','deb Kosmos-225','',16775,'Reentered','1968-06-19 00:00:00','1968-06-13 00:00:00',91.38,244,441,48.4,'Intermediate' from dual union all
select 32810,'032810','2006-026BJ','deb Kosmos-2421','',68198,'Reentered','2011-02-19 00:00:00','2008-05-08 00:00:00',97.93,397,923,65.76,'Intermediate' from dual union all
select 32811,'032811','2006-026BK','deb Kosmos-2421','',68198,'Reentered','2008-07-08 00:00:00','2008-05-08 00:00:00',92.92,386,450,65.11,'Intermediate' from dual union all
select 32812,'032812','2006-026BL','deb Kosmos-2421','',68198,'Reentered','2008-08-23 00:00:00','2008-05-08 00:00:00',92.25,328,442,65.04,'Intermediate' from dual union all
select 32813,'032813','2006-026BM','deb Kosmos-2421','',68198,'Reentered','2008-09-21 00:00:00','2008-05-08 00:00:00',92.69,386,427,65.08,'Intermediate' from dual union all
select 32814,'032814','2006-026BN','deb Kosmos-2421','',68198,'Reentered','2009-01-15 00:00:00','2008-05-08 00:00:00',92.73,357,460,65.16,'Intermediate' from dual union all
select 32815,'032815','2006-026BP','deb Kosmos-2421','',68198,'Reentered','2008-06-19 00:00:00','2008-05-08 00:00:00',92.18,363,400,65.07,'Intermediate' from dual union all
select 32816,'032816','2006-026BQ','deb Kosmos-2421','',68198,'Reentered','2008-06-19 00:00:00','2008-05-08 00:00:00',92.19,365,399,65.08,'Intermediate' from dual union all
select 32817,'032817','2006-026BR','deb Kosmos-2421','',68198,'Reentered','2008-06-11 00:00:00','2008-05-08 00:00:00',92,353,392,65.07,'Intermediate' from dual union all
select 32818,'032818','2006-026BS','deb Kosmos-2421','',68198,'Reentered','2011-01-11 00:00:00','2008-05-08 00:00:00',92.71,392,423,65.04,'Intermediate' from dual union all
select 32819,'032819','2006-026BT','deb Kosmos-2421','',68198,'Reentered','2008-08-07 00:00:00','2008-05-08 00:00:00',92.71,395,420,65.01,'Intermediate' from dual union all
select 3282,'003282','1968-049A','Kosmos-226','Meteor No. 10',16776,'Reentered','1983-10-18 00:00:00','1968-08-05 00:00:00',96.83,575,639,81.22,'Intermediate' from dual union all
select 32820,'032820','2006-026BU','deb Kosmos-2421','',68198,'Reentered','2009-01-03 00:00:00','2008-05-08 00:00:00',92.83,396,431,65.07,'Intermediate' from dual union all
select 32821,'032821','2006-026BV','deb Kosmos-2421','',68198,'Reentered','2008-09-12 00:00:00','2008-05-08 00:00:00',92.51,389,407,65.06,'Intermediate' from dual union all
select 32822,'032822','2006-026BW','deb Kosmos-2421','',68198,'Reentered','2008-08-14 00:00:00','2008-05-08 00:00:00',92.4,387,398,65,'Intermediate' from dual union all
select 32823,'032823','2006-026BX','deb Kosmos-2421','',68198,'Reentered','2008-09-02 00:00:00','2008-05-08 00:00:00',92.46,391,400,65,'Intermediate' from dual union all
select 32824,'032824','2006-026BY','deb Kosmos-2421','',68198,'Reentered','2008-07-14 00:00:00','2008-05-08 00:00:00',92.35,384,396,65,'Intermediate' from dual union all
select 32825,'032825','2006-026BZ','deb Kosmos-2421','',68198,'Reentered','2008-09-04 00:00:00','2008-05-08 00:00:00',92.42,388,399,65,'Intermediate' from dual union all
select 32826,'032826','2006-026CA','deb Kosmos-2421','',68198,'Reentered','2008-05-27 00:00:00','2008-05-08 00:00:00',92.07,367,386,65.01,'Intermediate' from dual union all
select 32827,'032827','2006-026CB','deb Kosmos-2421','',68198,'Reentered','2008-06-20 00:00:00','2008-05-08 00:00:00',91.77,324,399,64.68,'Intermediate' from dual union all
select 32828,'032828','2006-026CC','deb Kosmos-2421','',68198,'Reentered','2008-11-13 00:00:00','2008-05-08 00:00:00',92.81,389,436,65.1,'Intermediate' from dual union all
select 32829,'032829','2006-026CD','deb Kosmos-2421','',68198,'Reentered','2009-02-18 00:00:00','2008-05-08 00:00:00',92.5,369,425,65.19,'Intermediate' from dual union all
select 3283,'003283','1968-049B','8A92ME','8A92ME',16776,'Reentered','1991-01-26 00:00:00','1968-07-13 00:00:00',97.06,524,711,81.22,'Intermediate' from dual union all
select 32830,'032830','2006-026CE','deb Kosmos-2421','',68198,'Reentered','2010-03-28 00:00:00','2008-05-08 00:00:00',93.45,380,507,65.18,'Intermediate' from dual union all
select 32831,'032831','2006-026CF','deb Kosmos-2421','',68198,'Reentered','2008-09-18 00:00:00','2008-05-08 00:00:00',93.78,373,546,65.25,'Intermediate' from dual union all
select 32832,'032832','2006-026CG','deb Kosmos-2421','',68198,'Reentered','2009-02-02 00:00:00','2008-05-08 00:00:00',94.17,379,578,64.92,'Intermediate' from dual union all
select 32833,'032833','2006-026CH','deb Kosmos-2421','',68198,'Reentered','2010-11-11 00:00:00','2008-05-08 00:00:00',94.52,399,591,65.37,'Intermediate' from dual union all
select 32834,'032834','2006-026CJ','deb Kosmos-2421','',68198,'Reentered','2011-04-01 00:00:00','2008-05-08 00:00:00',98.02,400,928,65.95,'Intermediate' from dual union all
select 32835,'032835','2006-026CK','deb Kosmos-2421','',68198,'Reentered','2008-06-05 00:00:00','2008-05-08 00:00:00',91.85,330,401,64.86,'Intermediate' from dual union all
select 32836,'032836','2006-026CL','deb Kosmos-2421','',68198,'Reentered','2008-05-19 00:00:00','2008-05-08 00:00:00',91.01,271,378,64.71,'Intermediate' from dual union all
select 32837,'032837','2006-026CM','deb Kosmos-2421','',68198,'Reentered','2008-06-16 00:00:00','2008-05-08 00:00:00',92.2,368,397,65.08,'Intermediate' from dual union all
select 32838,'032838','2006-026CN','deb Kosmos-2421','',68198,'Reentered','2008-08-01 00:00:00','2008-05-08 00:00:00',93.1,388,465,65.11,'Intermediate' from dual union all
select 32839,'032839','2006-026CP','deb Kosmos-2421','',68198,'Reentered','2008-07-09 00:00:00','2008-05-08 00:00:00',93.05,388,459,65.15,'Intermediate' from dual union all
select 3284,'003284','1968-050A','OPS 9341','IDCSP 20',16777,'In Earth orbit','','1968-07-24 00:00:00',1335.33,33710,33871,.09,'Medium' from dual union all
select 32840,'032840','2006-026CQ','deb Kosmos-2421','',68198,'Reentered','2008-11-23 00:00:00','2008-05-08 00:00:00',93.43,389,496,65.56,'Intermediate' from dual union all
select 32841,'032841','2006-026CR','deb Kosmos-2421','',68198,'Reentered','2008-12-07 00:00:00','2008-05-08 00:00:00',92.17,363,399,64.51,'Intermediate' from dual union all
select 32842,'032842','2006-026CS','deb Kosmos-2421','',68198,'Reentered','2009-02-05 00:00:00','2008-05-08 00:00:00',92.51,394,402,65.01,'Intermediate' from dual union all
select 32843,'032843','2006-026CT','deb Kosmos-2421','',68198,'Reentered','2009-06-21 00:00:00','2008-05-08 00:00:00',92.66,393,416,65.07,'Intermediate' from dual union all
select 32844,'032844','2006-026CU','deb Kosmos-2421','',68198,'Reentered','2008-06-03 00:00:00','2008-05-08 00:00:00',91.81,341,386,65.11,'Intermediate' from dual union all
select 32845,'032845','2006-026CV','deb Kosmos-2421','',68198,'Reentered','2008-06-19 00:00:00','2008-05-08 00:00:00',92.14,365,394,65.05,'Intermediate' from dual union all
select 32846,'032846','2006-026CW','deb Kosmos-2421','',68198,'Reentered','2009-01-27 00:00:00','2008-05-08 00:00:00',92.33,370,408,64.94,'Intermediate' from dual union all
select 32847,'032847','2008-023A','Progress M-64','Progress 7K-TGM No.364',68531,'Deorbited','2008-09-08 00:00:00','2008-05-22 00:00:00',91.33,336,344,51.64,'Intermediate' from dual union all
select 32848,'032848','2008-023B','11S510','11S510',68531,'Reentered','2008-05-18 00:00:00','2008-05-16 00:00:00',88.17,175,193,51.64,'Intermediate' from dual union all
select 32849,'032849','2006-026CY','deb Kosmos-2421','',68198,'Reentered','2008-06-27 00:00:00','2008-05-14 00:00:00',92.54,378,420,65.09,'Intermediate' from dual union all
select 3285,'003285','1968-050B','OPS 9342','IDCSP 21',16777,'In Earth orbit','','1968-07-24 00:00:00',1335.75,33717,33881,.13,'Medium' from dual union all
select 32850,'032850','2006-026CZ','deb Kosmos-2421','',68198,'Reentered','2008-07-04 00:00:00','2008-05-14 00:00:00',92.82,380,446,65.13,'Intermediate' from dual union all
select 32851,'032851','2006-026DA','deb Kosmos-2421','',68198,'Reentered','2008-06-26 00:00:00','2008-05-14 00:00:00',92.24,371,397,65.07,'Intermediate' from dual union all
select 32852,'032852','2006-026DB','deb Kosmos-2421','',68198,'Reentered','2008-06-25 00:00:00','2008-05-14 00:00:00',92.81,384,441,65.16,'Intermediate' from dual union all
select 32853,'032853','2006-026DC','deb Kosmos-2421','',68198,'Reentered','2008-06-08 00:00:00','2008-05-14 00:00:00',92.02,357,391,65.07,'Intermediate' from dual union all
select 32854,'032854','2006-026DD','deb Kosmos-2421','',68198,'Reentered','2008-06-10 00:00:00','2008-05-14 00:00:00',92.11,361,395,65.07,'Intermediate' from dual union all
select 32855,'032855','2006-026DE','deb Kosmos-2421','',68198,'Reentered','2008-06-03 00:00:00','2008-05-14 00:00:00',91.84,345,384,65.06,'Intermediate' from dual union all
select 32856,'032856','2006-026DF','deb Kosmos-2421','',68198,'Reentered','2008-11-22 00:00:00','2008-05-14 00:00:00',92.9,389,445,65.13,'Intermediate' from dual union all
select 32857,'032857','2006-026DG','deb Kosmos-2421','',68198,'Reentered','2008-06-19 00:00:00','2008-05-14 00:00:00',92.26,372,399,65.05,'Intermediate' from dual union all
select 32858,'032858','2006-026DH','deb Kosmos-2421','',68198,'Reentered','2010-06-15 00:00:00','2008-05-14 00:00:00',92.7,363,450,65.07,'Intermediate' from dual;
insert into SATELLITE
select 32859,'032859','2006-026DJ','deb Kosmos-2421','',68198,'Reentered','2008-06-29 00:00:00','2008-05-14 00:00:00',92.51,376,420,65.11,'Intermediate' from dual union all
select 3286,'003286','1968-050C','OPS 9343','IDCSP 22',16777,'In Earth orbit','','1968-07-24 00:00:00',1336.55,33710,33920,.11,'Medium' from dual union all
select 32860,'032860','2006-026DK','deb Kosmos-2421','',68198,'Reentered','2008-06-20 00:00:00','2008-05-14 00:00:00',92.31,374,402,65.04,'Intermediate' from dual union all
select 32861,'032861','2006-026DL','deb Kosmos-2421','',68198,'Reentered','2008-06-22 00:00:00','2008-05-14 00:00:00',92.33,375,403,65.05,'Intermediate' from dual union all
select 32862,'032862','2006-026DM','deb Kosmos-2421','',68198,'Reentered','2008-08-18 00:00:00','2008-05-14 00:00:00',92.47,383,409,65.05,'Intermediate' from dual union all
select 32863,'032863','2006-026DN','deb Kosmos-2421','',68198,'Reentered','2008-06-09 00:00:00','2008-05-14 00:00:00',92.26,370,401,65.07,'Intermediate' from dual union all
select 32864,'032864','2006-026DP','deb Kosmos-2421','',68198,'Reentered','2008-06-09 00:00:00','2008-05-14 00:00:00',92.22,368,399,65.08,'Intermediate' from dual union all
select 32865,'032865','2006-026DQ','deb Kosmos-2421','',68198,'Reentered','2008-06-27 00:00:00','2008-05-14 00:00:00',92.64,378,430,65.13,'Intermediate' from dual union all
select 32866,'032866','2006-026DR','deb Kosmos-2421','',68198,'Reentered','2008-09-27 00:00:00','2008-05-14 00:00:00',92.87,391,440,65,'Intermediate' from dual union all
select 32867,'032867','2006-026DS','deb Kosmos-2421','',68198,'Reentered','2008-06-27 00:00:00','2008-05-14 00:00:00',92.32,375,402,65.04,'Intermediate' from dual union all
select 32868,'032868','2006-026DT','deb Kosmos-2421','',68198,'Reentered','2008-10-02 00:00:00','2008-05-14 00:00:00',92.5,393,402,65.01,'Intermediate' from dual union all
select 32869,'032869','2006-026DU','deb Kosmos-2421','',68198,'Reentered','2008-11-03 00:00:00','2008-05-14 00:00:00',92.61,376,429,65.05,'Intermediate' from dual union all
select 3287,'003287','1968-050D','OPS 9344','IDCSP 23',16777,'In Earth orbit','','1968-07-24 00:00:00',1338,33723,33965,.08,'Medium' from dual union all
select 32870,'032870','2006-026DV','deb Kosmos-2421','',68198,'Reentered','2008-08-19 00:00:00','2008-05-14 00:00:00',92.43,390,398,65.01,'Intermediate' from dual union all
select 32871,'032871','2006-026DW','deb Kosmos-2421','',68198,'Reentered','2008-08-06 00:00:00','2008-05-14 00:00:00',91.52,302,397,64.32,'Intermediate' from dual union all
select 32872,'032872','2006-026DX','deb Kosmos-2421','',68198,'Reentered','2008-06-06 00:00:00','2008-05-14 00:00:00',91.58,326,379,64.15,'Intermediate' from dual union all
select 32873,'032873','2006-026DY','deb Kosmos-2421','',68198,'Reentered','2008-06-03 00:00:00','2008-05-14 00:00:00',91.79,337,387,65.09,'Intermediate' from dual union all
select 32874,'032874','2006-026DZ','deb Kosmos-2421','',68198,'Reentered','2008-05-30 00:00:00','2008-05-14 00:00:00',91.66,335,376,65.06,'Intermediate' from dual union all
select 32875,'032875','2006-026EA','deb Kosmos-2421','',68198,'Reentered','2009-05-27 00:00:00','2008-05-14 00:00:00',92.62,390,415,65.04,'Intermediate' from dual union all
select 32876,'032876','2006-026EB','deb Kosmos-2421','',68198,'Reentered','2008-10-02 00:00:00','2008-05-14 00:00:00',92.88,391,441,65.03,'Intermediate' from dual union all
select 32877,'032877','2006-026EC','deb Kosmos-2421','',68198,'Reentered','2008-08-05 00:00:00','2008-05-14 00:00:00',93.02,381,464,65.08,'Intermediate' from dual union all
select 32878,'032878','2006-026ED','deb Kosmos-2421','',68198,'Reentered','2008-05-26 00:00:00','2008-05-14 00:00:00',91.55,330,371,65.08,'Intermediate' from dual union all
select 32879,'032879','2006-026EE','deb Kosmos-2421','',68198,'Reentered','2008-05-20 00:00:00','2008-05-14 00:00:00',91.57,336,368,65.03,'Intermediate' from dual union all
select 3288,'003288','1968-050E','OPS 9345','IDCSP 24',16777,'In Earth orbit','','1968-07-24 00:00:00',1340.02,33721,34048,.06,'Medium' from dual union all
select 32880,'032880','2006-026EF','deb Kosmos-2421','',68198,'Reentered','2008-11-27 00:00:00','2008-05-14 00:00:00',92.83,389,437,65.11,'Intermediate' from dual union all
select 32881,'032881','2006-026EG','deb Kosmos-2421','',68198,'Reentered','2008-06-03 00:00:00','2008-05-14 00:00:00',91.83,344,385,65.09,'Intermediate' from dual union all
select 32882,'032882','2006-026EH','deb Kosmos-2421','',68198,'Reentered','2008-05-21 00:00:00','2008-05-14 00:00:00',91.5,339,358,64.79,'Intermediate' from dual union all
select 32883,'032883','2006-026EJ','deb Kosmos-2421','',68198,'Reentered','2008-06-03 00:00:00','2008-05-14 00:00:00',91.83,346,383,65.08,'Intermediate' from dual union all
select 32884,'032884','2006-026EK','deb Kosmos-2421','',68198,'Reentered','2008-10-02 00:00:00','2008-05-14 00:00:00',92.11,360,395,64.96,'Intermediate' from dual union all
select 32885,'032885','2006-026EL','deb Kosmos-2421','',68198,'Reentered','2008-07-09 00:00:00','2008-05-14 00:00:00',92.6,380,424,65.12,'Intermediate' from dual union all
select 32886,'032886','2006-026EM','deb Kosmos-2421','',68198,'Reentered','2008-12-24 00:00:00','2008-05-14 00:00:00',92.9,389,445,65.13,'Intermediate' from dual union all
select 32887,'032887','2006-026EN','deb Kosmos-2421','',68198,'Reentered','2008-06-23 00:00:00','2008-05-14 00:00:00',92.29,373,401,65.06,'Intermediate' from dual union all
select 32888,'032888','2006-026EP','deb Kosmos-2421','',68198,'Reentered','2009-02-08 00:00:00','2008-05-14 00:00:00',92.6,390,413,65.05,'Intermediate' from dual union all
select 32889,'032889','2006-026EQ','deb Kosmos-2421','',68198,'Reentered','2009-11-14 00:00:00','2008-05-14 00:00:00',92.61,390,414,65.05,'Intermediate' from dual union all
select 3289,'003289','1968-050F','OPS 9346','IDCSP 25',16777,'In Earth orbit','','1968-07-24 00:00:00',1342.39,33726,34138,.07,'Medium' from dual union all
select 32890,'032890','2006-026ER','deb Kosmos-2421','',68198,'Reentered','2008-10-13 00:00:00','2008-05-14 00:00:00',92.56,387,413,65.04,'Intermediate' from dual union all
select 32891,'032891','2006-026ES','deb Kosmos-2421','',68198,'Reentered','2008-07-07 00:00:00','2008-05-14 00:00:00',92.68,382,430,65.1,'Intermediate' from dual union all
select 32892,'032892','2006-026ET','deb Kosmos-2421','',68198,'Reentered','2008-06-25 00:00:00','2008-05-14 00:00:00',92.03,357,392,65.09,'Intermediate' from dual union all
select 32893,'032893','2006-026EU','deb Kosmos-2421','',68198,'Reentered','2008-06-21 00:00:00','2008-05-14 00:00:00',92.12,362,395,65.07,'Intermediate' from dual union all
select 32894,'032894','2006-026EV','deb Kosmos-2421','',68198,'Reentered','2008-06-21 00:00:00','2008-05-14 00:00:00',92.05,358,392,65.07,'Intermediate' from dual union all
select 32895,'032895','2006-026EW','deb Kosmos-2421','',68198,'Reentered','2008-06-19 00:00:00','2008-05-14 00:00:00',92.16,363,398,65.08,'Intermediate' from dual union all
select 32896,'032896','2006-026EX','deb Kosmos-2421','',68198,'Reentered','2008-10-02 00:00:00','2008-05-14 00:00:00',92.38,385,398,65,'Intermediate' from dual union all
select 32897,'032897','2006-026EY','deb Kosmos-2421','',68198,'Reentered','2008-06-25 00:00:00','2008-05-14 00:00:00',92.62,378,428,65.11,'Intermediate' from dual union all
select 32898,'032898','2006-026EZ','deb Kosmos-2421','',68198,'Reentered','2008-07-09 00:00:00','2008-05-14 00:00:00',92.66,380,430,65.07,'Intermediate' from dual union all
select 32899,'032899','2006-026FA','deb Kosmos-2421','',68198,'Reentered','2008-07-03 00:00:00','2008-05-14 00:00:00',92.57,376,425,65.09,'Intermediate' from dual union all
select 329,'000329','61 Omicron 102','deb r/b Ablestar','',7273,'In Earth orbit','','1962-06-28 00:00:00',106.12,958,1136,66.84,'Intermediate' from dual union all
select 3290,'003290','1968-050G','OPS 9347','IDCSP 26',16777,'In Earth orbit','','1968-07-24 00:00:00',1345.57,33733,34258,.14,'Medium' from dual union all
select 32900,'032900','2006-026FB','deb Kosmos-2421','',68198,'Reentered','2008-06-05 00:00:00','2008-05-14 00:00:00',92.3,378,396,64.98,'Intermediate' from dual union all
select 32901,'032901','2006-026FC','deb Kosmos-2421','',68198,'Reentered','2008-10-23 00:00:00','2008-05-14 00:00:00',92.44,379,410,65.04,'Intermediate' from dual union all
select 32902,'032902','2006-026FD','deb Kosmos-2421','',68198,'Reentered','2008-10-12 00:00:00','2008-05-14 00:00:00',92.86,389,441,65.12,'Intermediate' from dual union all
select 32903,'032903','2006-026FE','deb Kosmos-2421','',68198,'Reentered','2008-05-21 00:00:00','2008-05-14 00:00:00',91.37,331,353,64.93,'Intermediate' from dual union all
select 32904,'032904','2006-026FF','deb Kosmos-2421','',68198,'Reentered','2008-06-16 00:00:00','2008-05-20 00:00:00',92.16,369,392,65.05,'Intermediate' from dual union all
select 32905,'032905','2006-026FG','deb Kosmos-2421','',68198,'Reentered','2008-07-04 00:00:00','2008-05-20 00:00:00',92.28,373,400,65.04,'Intermediate' from dual union all
select 32906,'032906','2006-026FH','deb Kosmos-2421','',68198,'Reentered','2008-07-29 00:00:00','2008-05-20 00:00:00',92.54,390,408,65,'Intermediate' from dual union all
select 32907,'032907','2006-026FJ','deb Kosmos-2421','',68198,'Reentered','2008-06-11 00:00:00','2008-05-20 00:00:00',91.84,348,381,65.08,'Intermediate' from dual union all
select 32908,'032908','2006-026FK','deb Kosmos-2421','',68198,'Reentered','2008-06-05 00:00:00','2008-05-20 00:00:00',91.65,336,374,65.09,'Intermediate' from dual union all
select 32909,'032909','2006-026FL','deb Kosmos-2421','',68198,'Reentered','2008-12-30 00:00:00','2008-05-20 00:00:00',92.84,389,439,65.12,'Intermediate' from dual union all
select 3291,'003291','1968-050H','OPS 9348','IDCSP 27',16777,'In Earth orbit','','1968-07-25 00:00:00',1348.92,33725,34400,.06,'Medium' from dual union all
select 32910,'032910','2006-026FM','deb Kosmos-2421','',68198,'Reentered','2009-12-15 00:00:00','2008-05-20 00:00:00',92.59,394,409,65.03,'Intermediate' from dual union all
select 32911,'032911','2006-026FN','deb Kosmos-2421','',68198,'Reentered','2008-06-24 00:00:00','2008-05-20 00:00:00',92.01,358,388,65.08,'Intermediate' from dual union all
select 32912,'032912','2006-026FP','deb Kosmos-2421','',68198,'Reentered','2008-11-09 00:00:00','2008-05-20 00:00:00',92.82,389,436,65.1,'Intermediate' from dual union all
select 32913,'032913','2006-026FQ','deb Kosmos-2421','',68198,'Reentered','2008-08-21 00:00:00','2008-05-20 00:00:00',93.15,385,473,65.22,'Intermediate' from dual union all
select 32914,'032914','2006-026FR','deb Kosmos-2421','',68198,'Reentered','2009-02-26 00:00:00','2008-05-20 00:00:00',93.18,386,475,63.66,'Intermediate' from dual union all
select 32915,'032915','2006-026FS','deb Kosmos-2421','',68198,'Reentered','2008-06-18 00:00:00','2008-05-20 00:00:00',92.21,368,398,65.05,'Intermediate' from dual union all
select 32916,'032916','2006-026FT','deb Kosmos-2421','',68198,'Reentered','2008-06-08 00:00:00','2008-05-20 00:00:00',91.22,313,356,65.08,'Intermediate' from dual union all
select 32917,'032917','2006-026FU','deb Kosmos-2421','',68198,'Reentered','2008-05-24 00:00:00','2008-05-20 00:00:00',91.75,349,372,65.02,'Intermediate' from dual union all
select 32918,'032918','2006-026FV','deb Kosmos-2421','',68198,'Reentered','2008-06-05 00:00:00','2008-05-20 00:00:00',91.86,357,374,65,'Intermediate' from dual union all
select 32919,'032919','2006-026FW','deb Kosmos-2421','',68198,'Reentered','2008-09-18 00:00:00','2008-05-20 00:00:00',92.37,385,397,65,'Intermediate' from dual union all
select 3292,'003292','1968-050J','Transtage 16','Transtage 16',16777,'In Earth orbit','','1976-08-30 00:00:00',1363.63,33631,35081,7.87,'Medium' from dual union all
select 32920,'032920','2006-026FX','deb Kosmos-2421','',68198,'Reentered','2008-06-09 00:00:00','2008-05-20 00:00:00',91.75,344,377,65.07,'Intermediate' from dual union all
select 32921,'032921','2006-026FY','deb Kosmos-2421','',68198,'Reentered','2008-06-23 00:00:00','2008-05-20 00:00:00',92.2,370,395,65.04,'Intermediate' from dual union all
select 32922,'032922','2006-026FZ','deb Kosmos-2421','',68198,'Reentered','2008-06-28 00:00:00','2008-05-20 00:00:00',92.62,366,440,65.17,'Intermediate' from dual union all
select 32923,'032923','2006-026GA','deb Kosmos-2421','',68198,'Reentered','2008-06-25 00:00:00','2008-05-20 00:00:00',92.25,365,405,65.11,'Intermediate' from dual union all
select 32924,'032924','2006-026GB','deb Kosmos-2421','',68198,'Reentered','2008-11-08 00:00:00','2008-05-20 00:00:00',92.8,387,437,65.11,'Intermediate' from dual union all
select 32925,'032925','2006-026GC','deb Kosmos-2421','',68198,'Reentered','2008-11-03 00:00:00','2008-05-20 00:00:00',92.83,388,439,65.13,'Intermediate' from dual union all
select 32926,'032926','2006-026GD','deb Kosmos-2421','',68198,'Reentered','2009-09-23 00:00:00','2008-05-20 00:00:00',92.91,392,442,65.12,'Intermediate' from dual union all
select 32927,'032927','2006-026GE','deb Kosmos-2421','',68198,'Reentered','2008-06-19 00:00:00','2008-05-20 00:00:00',92.13,368,390,65.09,'Intermediate' from dual union all
select 32928,'032928','2006-026GF','deb Kosmos-2421','',68198,'Reentered','2008-11-01 00:00:00','2008-05-20 00:00:00',92.74,387,431,65.1,'Intermediate' from dual union all
select 32929,'032929','2006-026GG','deb Kosmos-2421','',68198,'Reentered','2008-09-11 00:00:00','2008-05-20 00:00:00',92.42,388,399,65.01,'Intermediate' from dual union all
select 3293,'003293','1968-048D','deb Kosmos-225','',16775,'Reentered','1968-06-21 00:00:00','1968-06-14 00:00:00',91.03,237,413,48.42,'Intermediate' from dual union all
select 32930,'032930','2006-026GH','deb Kosmos-2421','',68198,'Reentered','2008-06-17 00:00:00','2008-05-20 00:00:00',91.96,364,378,65.01,'Intermediate' from dual union all
select 32931,'032931','2006-026GJ','deb Kosmos-2421','',68198,'Reentered','2008-06-21 00:00:00','2008-05-20 00:00:00',92.11,364,392,65.06,'Intermediate' from dual union all
select 32932,'032932','2006-026GK','deb Kosmos-2421','',68198,'Reentered','2008-07-06 00:00:00','2008-05-20 00:00:00',92.19,369,394,65.05,'Intermediate' from dual union all
select 32933,'032933','2006-026GL','deb Kosmos-2421','',68198,'Reentered','2008-09-25 00:00:00','2008-05-20 00:00:00',92.4,386,398,65,'Intermediate' from dual union all
select 32934,'032934','2006-026GM','deb Kosmos-2421','',68198,'Reentered','2008-07-06 00:00:00','2008-05-20 00:00:00',92.44,373,416,65.1,'Intermediate' from dual union all
select 32935,'032935','2006-026GN','deb Kosmos-2421','',68198,'Reentered','2008-07-10 00:00:00','2008-05-20 00:00:00',92.48,372,421,65.08,'Intermediate' from dual union all
select 32936,'032936','2006-026GP','deb Kosmos-2421','',68198,'Reentered','2009-01-27 00:00:00','2008-05-20 00:00:00',92.82,388,437,65.08,'Intermediate' from dual union all
select 32937,'032937','2006-026GQ','deb Kosmos-2421','',68198,'Reentered','2008-12-14 00:00:00','2008-05-20 00:00:00',92.8,388,436,65.11,'Intermediate' from dual union all
select 32938,'032938','2006-026GR','deb Kosmos-2421','',68198,'Reentered','2008-07-29 00:00:00','2008-05-20 00:00:00',92.65,374,435,65.12,'Intermediate' from dual union all
select 32939,'032939','2006-026GS','deb Kosmos-2421','',68198,'Reentered','2008-06-15 00:00:00','2008-05-20 00:00:00',92,360,385,65.05,'Intermediate' from dual union all
select 3294,'003294','1968-051A','Kosmos-227','Zenit-4 No. 44',16778,'Landed','1968-06-26 00:00:00','1968-06-21 00:00:00',89.1,190,269,51.81,'Intermediate' from dual union all
select 32940,'032940','2006-026GT','deb Kosmos-2421','',68198,'Reentered','2008-07-03 00:00:00','2008-05-20 00:00:00',92.26,372,399,65.05,'Intermediate' from dual union all
select 32941,'032941','2006-026GU','deb Kosmos-2421','',68198,'Reentered','2008-06-24 00:00:00','2008-05-20 00:00:00',92.22,371,396,65.05,'Intermediate' from dual union all
select 32942,'032942','2006-026GV','deb Kosmos-2421','',68198,'Reentered','2008-09-18 00:00:00','2008-05-20 00:00:00',92.47,384,407,65.04,'Intermediate' from dual union all
select 32943,'032943','2006-026GW','deb Kosmos-2421','',68198,'Reentered','2008-07-01 00:00:00','2008-05-20 00:00:00',92.48,373,419,65.09,'Intermediate' from dual union all
select 32944,'032944','2006-026GX','deb Kosmos-2421','',68198,'Reentered','2008-09-30 00:00:00','2008-05-20 00:00:00',92.74,386,432,65.11,'Intermediate' from dual union all
select 32945,'032945','2006-026GY','deb Kosmos-2421','',68198,'Reentered','2008-06-02 00:00:00','2008-05-20 00:00:00',91.7,349,366,64.99,'Intermediate' from dual union all
select 32946,'032946','2006-026GZ','deb Kosmos-2421','',68198,'Reentered','2008-06-27 00:00:00','2008-05-20 00:00:00',92.14,367,392,65.05,'Intermediate' from dual union all
select 32947,'032947','2006-026HA','deb Kosmos-2421','',68198,'Reentered','2008-06-03 00:00:00','2008-05-20 00:00:00',91.75,347,374,65.03,'Intermediate' from dual union all
select 32948,'032948','2006-026HB','deb Kosmos-2421','',68198,'Reentered','2008-06-02 00:00:00','2008-05-20 00:00:00',91.86,353,379,65.07,'Intermediate' from dual;
insert into SATELLITE
select 32949,'032949','2006-026HC','deb Kosmos-2421','',68198,'Reentered','2008-05-29 00:00:00','2008-05-20 00:00:00',91.78,351,372,65.05,'Intermediate' from dual union all
select 3295,'003295','1968-051B','11A57I','11A57I',16778,'Reentered','1968-06-21 00:00:00','1968-06-20 00:00:00',88.35,155,231,51.8,'Intermediate' from dual union all
select 32950,'032950','2006-026HD','deb Kosmos-2421','',68198,'Reentered','2008-07-09 00:00:00','2008-05-20 00:00:00',92.55,376,423,65.11,'Intermediate' from dual union all
select 32951,'032951','2008-024A','Galaxy 18','Galaxy 18',68532,'In Earth orbit','','2008-05-29 00:00:00',1436.06,35773,35799,.06,'Stationary' from dual union all
select 32952,'032952','2008-024B','Blok DM-SL No. 27L','Blok DM-SL',68532,'In Earth orbit','','2008-05-25 00:00:00',670.93,2458,35560,.07,'GEO Transfer' from dual union all
select 32953,'032953','2008-025A','Yubileyniy','Yubileyniy',68533,'In Earth orbit','','2008-05-25 00:00:00',115.84,1480,1506,82.5,'Intermediate' from dual union all
select 32954,'032954','2008-025B','Kosmos-2437','Strela-3',68533,'In Earth orbit','','2008-05-26 00:00:00',115.87,1480,1509,82.5,'Intermediate' from dual union all
select 32955,'032955','2008-025C','Kosmos-2438','Strela-3',68533,'In Earth orbit','','2008-05-26 00:00:00',115.81,1477,1507,82.5,'Intermediate' from dual union all
select 32956,'032956','2008-025D','Kosmos-2439','Strela-3',68533,'In Earth orbit','','2008-05-25 00:00:00',115.84,1478,1509,82.51,'Intermediate' from dual union all
select 32957,'032957','2008-025E','Briz-KM No. 72509','Briz-KM 14S45',68533,'In Earth orbit','','2008-05-25 00:00:00',112.69,1195,1506,82.5,'Intermediate' from dual union all
select 32958,'032958','2008-026A','Feng Yun 3A','fengyun sanhao 01 xing',68534,'In Earth orbit','','2008-05-29 00:00:00',101.03,804,810,98.81,'Sun-Synch' from dual union all
select 32959,'032959','2008-026B','CZ-4C Stage 3','CZ-4C Stage 3',68534,'In Earth orbit','','2008-05-29 00:00:00',99.93,704,807,98.86,'Sun-Synch' from dual union all
select 3296,'003296','1968-052A','OPS 5343','CORONA J-47',16779,'Reentered Attached','1968-07-16 00:00:00','1968-06-27 00:00:00',90.24,179,394,85,'Polar' from dual union all
select 32960,'032960','2008-027A','Discovery (STS-124)','OV-103',68535,'Landed','2008-06-14 00:00:00','2008-06-03 00:00:00',91.32,336,343,51.64,'Intermediate' from dual union all
select 32961,'032961','1977-027G','deb Kosmos-903','deb Oko',42155,'Reentered','2014-06-07 00:00:00','2008-06-05 00:00:00',718,2786,37580,67.45,'GEO Transfer' from dual union all
select 32962,'032962','1977-027H','deb Kosmos-903','deb Oko',42155,'In Earth orbit','','2008-05-31 00:00:00',723.99,4295,36366,69.25,'GEO Transfer' from dual union all
select 32963,'032963','1977-027J','deb Kosmos-903','deb Oko',42155,'In Earth orbit','','2008-06-04 00:00:00',708.58,5105,34795,69.84,'GEO Transfer' from dual union all
select 32964,'032964','1977-047M','deb Kosmos-917','deb Oko',42175,'Reentered','2011-07-04 00:00:00','2008-04-18 00:00:00',727.07,1676,39136,66.61,'GEO Transfer' from dual union all
select 32965,'032965','1977-047N','deb Kosmos-917','deb Oko',42175,'In Earth orbit','','2008-06-04 00:00:00',721.47,5845,34692,67.14,'GEO Transfer' from dual union all
select 32966,'032966','1977-047P','deb Kosmos-917','deb Oko',42175,'In Earth orbit','','2008-05-14 00:00:00',765.84,6011,36689,69.17,'Highly Ellip' from dual union all
select 32967,'032967','1977-047Q','deb Kosmos-917','deb Oko',42175,'Reentered','2013-10-19 00:00:00','2008-05-07 00:00:00',675.26,2115,36121,64.57,'GEO Transfer' from dual union all
select 32968,'032968','1978-066H','deb Kosmos-1024','deb Oko',45261,'In Earth orbit','','2008-06-04 00:00:00',730.2,5385,35581,68.93,'GEO Transfer' from dual union all
select 32969,'032969','1978-083M','deb Kosmos-1030','deb Oko',45278,'In Earth orbit','','2008-05-16 00:00:00',709.1,4447,35479,66.42,'GEO Transfer' from dual union all
select 3297,'003297','1968-052B','TRIPOS III','P-11 No. 4420',16779,'Reentered','1970-01-11 00:00:00','1968-07-20 00:00:00',94.09,437,511,85.17,'Polar' from dual union all
select 32970,'032970','1982-095E','deb Kosmos-1409','deb Oko',54404,'Reentered','2009-12-26 00:00:00','2008-06-05 00:00:00',719.81,4716,35739,68.67,'GEO Transfer' from dual union all
select 32971,'032971','1982-095F','deb Kosmos-1409','deb Oko',54404,'Reentered','2010-03-16 00:00:00','2008-06-04 00:00:00',721.82,5931,34623,70.83,'GEO Transfer' from dual union all
select 32972,'032972','1978-083Q','deb Kosmos-1030','deb Oko',45278,'In Earth orbit','','2008-05-21 00:00:00',725.95,6512,34245,71.36,'GEO Transfer' from dual union all
select 32973,'032973','1978-083R','deb Kosmos-1030','deb Oko',45278,'In Earth orbit','','2008-06-01 00:00:00',671.07,3965,34060,68.4,'GEO Transfer' from dual union all
select 32974,'032974','1978-083S','deb Kosmos-1030','deb Oko',45278,'In Earth orbit','','2008-06-04 00:00:00',721.41,3489,37045,66.69,'GEO Transfer' from dual union all
select 32975,'032975','1979-058Q','deb Kosmos-1109','deb Oko',47542,'Reentered','2012-04-21 00:00:00','2008-05-10 00:00:00',729.47,1963,38967,67.05,'GEO Transfer' from dual union all
select 32976,'032976','1979-058R','deb Kosmos-1109','deb Oko',47542,'In Earth orbit','','2008-05-31 00:00:00',749.99,3596,38336,68.39,'GEO Transfer' from dual union all
select 32977,'032977','1979-058S','deb Kosmos-1109','deb Oko',47542,'In Earth orbit','','2008-06-01 00:00:00',726.6,2950,37839,71.88,'GEO Transfer' from dual union all
select 32978,'032978','1979-058T','deb Kosmos-1109','deb Oko',47542,'In Earth orbit','','2008-05-31 00:00:00',764.95,4879,37778,70.64,'Highly Ellip' from dual union all
select 32979,'032979','1979-058U','deb Kosmos-1109','deb Oko',47542,'In Earth orbit','','2008-05-26 00:00:00',726.43,2730,38051,69.72,'GEO Transfer' from dual union all
select 3298,'003298','1968-053A','Kosmos-228','Zenit-2M',16780,'Landed','1968-07-03 00:00:00','1968-06-23 00:00:00',88.99,205,245,51.63,'Intermediate' from dual union all
select 32980,'032980','1979-058V','deb Kosmos-1109','deb Oko',47542,'In Earth orbit','','2008-06-01 00:00:00',738.23,3662,37697,70.8,'GEO Transfer' from dual union all
select 32981,'032981','1979-077G','deb Kosmos-1124','',47561,'In Earth orbit','','2008-06-04 00:00:00',720.3,5936,34543,70.76,'GEO Transfer' from dual union all
select 32982,'032982','1979-077H','deb Kosmos-1124','',47561,'In Earth orbit','','2008-06-05 00:00:00',686.37,4061,34733,67.37,'GEO Transfer' from dual union all
select 32983,'032983','1979-101J','deb Satcom 3','',47585,'In Earth orbit','','2008-06-03 00:00:00',789.78,8263,35587,10.61,'Medium' from dual union all
select 32984,'032984','1979-101K','deb Satcom 3','',47585,'In Earth orbit','','2008-06-04 00:00:00',788.4,8178,35606,11.19,'Medium' from dual union all
select 32985,'032985','1979-101L','deb Satcom 3','',47585,'In Earth orbit','','2008-05-31 00:00:00',796.91,8552,35638,10.58,'Medium' from dual union all
select 32986,'032986','1979-101M','deb Satcom 3','',47585,'In Earth orbit','','2008-06-04 00:00:00',788.32,8243,35537,10.48,'Medium' from dual union all
select 32987,'032987','1979-101N','deb Satcom 3','',47585,'In Earth orbit','','2008-06-04 00:00:00',784.45,8037,35558,9.98,'Medium' from dual union all
select 32988,'032988','1979-101P','deb Satcom 3','',47585,'In Earth orbit','','2008-06-03 00:00:00',794.41,8387,35684,10.79,'Medium' from dual union all
select 32989,'032989','1979-101Q','deb Satcom 3','',47585,'In Earth orbit','','2008-06-03 00:00:00',775.37,8268,34891,10.58,'Medium' from dual union all
select 3299,'003299','1968-053C','deb Kosmos-228','',16780,'Reentered','1968-06-23 00:00:00','1968-06-23 00:00:00',88.69,200,220,51.63,'Intermediate' from dual union all
select 32990,'032990','1979-101R','Unknown deb                       ','',47585,'Lost','','2008-06-04 00:00:00',793.39,8248,35774,9.98,'Medium' from dual union all
select 32991,'032991','1979-101S','deb Satcom 3','',47585,'In Earth orbit','','2008-06-05 00:00:00',800.04,8786,35553,9.42,'Medium' from dual union all
select 32992,'032992','1979-101T','Unknown deb                       ','',47585,'In Earth orbit','','2008-06-03 00:00:00',807.79,9231,35476,11.27,'Medium' from dual union all
select 32993,'032993','1979-101U','deb Satcom 3','',47585,'In Earth orbit','','2008-06-04 00:00:00',792.28,8764,35205,10.03,'Medium' from dual union all
select 32994,'032994','1980-057M','deb Kosmos-1191','deb Oko',50267,'In Earth orbit','','2008-06-06 00:00:00',720.07,3884,36584,66.26,'GEO Transfer' from dual union all
select 32995,'032995','1980-057N','deb Kosmos-1191','deb Oko',50267,'In Earth orbit','','2008-06-05 00:00:00',728.45,3693,37187,66.66,'GEO Transfer' from dual union all
select 32996,'032996','1980-057P','deb Kosmos-1191','deb Oko',50267,'In Earth orbit','','2008-06-02 00:00:00',728,3888,36970,66.33,'GEO Transfer' from dual union all
select 32997,'032997','1980-085L','deb Kosmos-1217','',50295,'Reentered','2013-08-26 00:00:00','2008-05-31 00:00:00',710.33,2268,37719,67.07,'GEO Transfer' from dual union all
select 32998,'032998','1991-075P','deb Ariane V47','',64244,'In Earth orbit','','2008-06-05 00:00:00',502.48,675,28442,6.51,'Highly Ellip' from dual union all
select 32999,'032999','1992-021T','deb Ariane V50','',64733,'In Earth orbit','','2008-06-04 00:00:00',640.97,899,35596,3.66,'GEO Transfer' from dual union all
select 33,'000033','60 Gamma 3','Dummy subsatellite','',6322,'Reentered','1960-07-17 00:00:00','1960-05-24 00:00:00',94.08,285,663,51.29,'Intermediate' from dual union all
select 330,'000330','61 Omicron 103','deb r/b Ablestar','',7273,'In Earth orbit','','1962-06-27 00:00:00',104.7,932,1029,67.01,'Intermediate' from dual union all
select 3300,'003300','1968-053B','11A57I','11A57I',16780,'Reentered','1968-06-24 00:00:00','1968-06-23 00:00:00',88.61,194,218,51.67,'Intermediate' from dual union all
select 33000,'033000','1978-012D','IUE dust cover','',45207,'In Earth orbit','','2008-06-04 00:00:00',1434.55,27380,44133,43.39,'Near-synch' from dual union all
select 33001,'033001','2006-026HE','deb Kosmos-2421','',68198,'Reentered','2008-08-12 00:00:00','2008-06-06 00:00:00',92.32,375,402,65.06,'Intermediate' from dual union all
select 33002,'033002','2006-026HF','deb Kosmos-2421','',68198,'Reentered','2008-06-18 00:00:00','2008-06-06 00:00:00',91.35,329,352,64.97,'Intermediate' from dual union all
select 33003,'033003','2006-026HG','deb Kosmos-2421','',68198,'Reentered','2008-06-29 00:00:00','2008-06-06 00:00:00',91.97,351,392,65.09,'Intermediate' from dual union all
select 33004,'033004','2006-026HH','deb Kosmos-2421','',68198,'Reentered','2008-06-13 00:00:00','2008-06-06 00:00:00',91.05,314,339,65.1,'Intermediate' from dual union all
select 33005,'033005','2006-026HJ','deb Kosmos-2421','',68198,'Reentered','2008-10-28 00:00:00','2008-06-06 00:00:00',92.68,380,432,65.1,'Intermediate' from dual union all
select 33006,'033006','2006-026HK','deb Kosmos-2421','',68198,'Reentered','2010-02-12 00:00:00','2008-06-06 00:00:00',92.9,392,442,65.12,'Intermediate' from dual union all
select 33007,'033007','2006-026HL','deb Kosmos-2421','',68198,'Reentered','2008-07-08 00:00:00','2008-06-06 00:00:00',91.84,354,376,65.04,'Intermediate' from dual union all
select 33008,'033008','2006-026HM','deb Kosmos-2421','',68198,'Reentered','2008-06-22 00:00:00','2008-06-06 00:00:00',91.72,341,377,65.09,'Intermediate' from dual union all
select 33009,'033009','2006-026HN','deb Kosmos-2421','',68198,'Reentered','2008-08-27 00:00:00','2008-06-06 00:00:00',92.43,381,406,65.04,'Intermediate' from dual union all
select 3301,'003301','1968-053D','deb Kosmos-228','',16780,'Reentered','1968-06-23 00:00:00','1968-06-22 00:00:00',87.63,125,190,51.6,'Intermediate' from dual union all
select 33010,'033010','2006-026HP','deb Kosmos-2421','',68198,'Reentered','2008-11-29 00:00:00','2008-06-06 00:00:00',92.48,383,410,65.04,'Intermediate' from dual union all
select 33011,'033011','2006-026HQ','deb Kosmos-2421','',68198,'Reentered','2008-10-23 00:00:00','2008-06-06 00:00:00',92.63,380,426,65.06,'Intermediate' from dual union all
select 33012,'033012','2006-026HR','deb Kosmos-2421','',68198,'Reentered','2008-11-23 00:00:00','2008-06-06 00:00:00',92.74,384,434,65.09,'Intermediate' from dual union all
select 33013,'033013','2006-026HS','deb Kosmos-2421','',68198,'Reentered','2008-07-01 00:00:00','2008-06-06 00:00:00',91.88,346,387,65.08,'Intermediate' from dual union all
select 33014,'033014','2006-026HT','deb Kosmos-2421','',68198,'Reentered','2008-12-21 00:00:00','2008-06-06 00:00:00',92.75,386,433,65.12,'Intermediate' from dual union all
select 33015,'033015','2006-026HU','deb Kosmos-2421','',68198,'Reentered','2008-07-06 00:00:00','2008-06-06 00:00:00',91.77,346,377,65.08,'Intermediate' from dual union all
select 33016,'033016','2006-026HV','deb Kosmos-2421','',68198,'Reentered','2008-06-06 00:00:00','2008-06-06 00:00:00',89.64,245,269,65.07,'Intermediate' from dual union all
select 33017,'033017','2006-026HW','deb Kosmos-2421','',68198,'Reentered','2008-09-07 00:00:00','2008-06-06 00:00:00',92.21,377,389,65.01,'Intermediate' from dual union all
select 33018,'033018','2006-026HX','deb Kosmos-2421','',68198,'Reentered','2008-06-19 00:00:00','2008-06-06 00:00:00',91.26,327,345,65.08,'Intermediate' from dual union all
select 33019,'033019','2006-026HY','deb Kosmos-2421','',68198,'Reentered','2010-05-22 00:00:00','2008-06-06 00:00:00',92.42,389,398,65.04,'Intermediate' from dual union all
select 3302,'003302','1968-053E','deb Kosmos-228','',16780,'Reentered','1968-06-23 00:00:00','1968-06-22 00:00:00',87.61,154,159,51.62,'Intermediate' from dual union all
select 33020,'033020','2006-026HZ','deb Kosmos-2421','',68198,'Reentered','2008-06-28 00:00:00','2008-06-06 00:00:00',91.87,361,371,65.09,'Intermediate' from dual union all
select 33021,'033021','2006-026JA','deb Kosmos-2421','',68198,'Reentered','2008-12-04 00:00:00','2008-06-06 00:00:00',92.73,385,432,65.12,'Intermediate' from dual union all
select 33022,'033022','2006-026JB','deb Kosmos-2421','',68198,'Reentered','2008-06-26 00:00:00','2008-06-06 00:00:00',91.92,349,389,65.11,'Intermediate' from dual union all
select 33023,'033023','2006-026JC','deb Kosmos-2421','',68198,'Reentered','2008-10-25 00:00:00','2008-06-06 00:00:00',92.69,381,432,65.09,'Intermediate' from dual union all
select 33024,'033024','2006-026JD','deb Kosmos-2421','',68198,'Reentered','2008-07-09 00:00:00','2008-06-06 00:00:00',92.03,356,392,65.1,'Intermediate' from dual union all
select 33025,'033025','2006-026JE','deb Kosmos-2421','',68198,'Reentered','2008-06-13 00:00:00','2008-06-06 00:00:00',91.06,313,341,65.04,'Intermediate' from dual union all
select 33026,'033026','2006-026JF','deb Kosmos-2421','',68198,'Reentered','2008-07-09 00:00:00','2008-06-06 00:00:00',92.1,358,397,65.09,'Intermediate' from dual union all
select 33027,'033027','2006-026JG','deb Kosmos-2421','',68198,'Reentered','2008-06-24 00:00:00','2008-06-06 00:00:00',91.95,343,398,65.12,'Intermediate' from dual union all
select 33028,'033028','2006-026JH','deb Kosmos-2421','',68198,'Reentered','2008-07-09 00:00:00','2008-06-05 00:00:00',92,352,394,65.12,'Intermediate' from dual union all
select 33029,'033029','2006-026JJ','deb Kosmos-2421','',68198,'Reentered','2008-06-29 00:00:00','2008-06-06 00:00:00',91.84,356,374,65.05,'Intermediate' from dual union all
select 3303,'003303','1968-053F','deb Kosmos-228','',16780,'Reentered','1968-06-23 00:00:00','1968-06-22 00:00:00',87.6,150,162,51.59,'Intermediate' from dual union all
select 33030,'033030','2006-026JK','deb Kosmos-2421','',68198,'Reentered','2008-07-26 00:00:00','2008-06-06 00:00:00',92.13,367,391,65.04,'Intermediate' from dual union all
select 33031,'033031','2006-026JL','deb Kosmos-2421','',68198,'Reentered','2009-03-15 00:00:00','2008-06-06 00:00:00',92.81,388,437,65.11,'Intermediate' from dual union all
select 33032,'033032','2006-026JM','deb Kosmos-2421','',68198,'Reentered','2008-12-06 00:00:00','2008-06-06 00:00:00',92.73,384,433,65.09,'Intermediate' from dual union all
select 33033,'033033','2006-026JN','deb Kosmos-2421','',68198,'Reentered','2009-01-03 00:00:00','2008-06-05 00:00:00',92.75,386,433,65.12,'Intermediate' from dual union all
select 33034,'033034','2006-026JP','deb Kosmos-2421','',68198,'Reentered','2008-07-11 00:00:00','2008-06-06 00:00:00',91.9,356,380,65.03,'Intermediate' from dual union all
select 33035,'033035','2006-026JQ','deb Kosmos-2421','',68198,'Reentered','2008-10-11 00:00:00','2008-06-06 00:00:00',92.49,371,423,65.09,'Intermediate' from dual union all
select 33036,'033036','2006-026JR','deb Kosmos-2421','',68198,'Reentered','2008-11-26 00:00:00','2008-06-06 00:00:00',92.64,383,424,65.1,'Intermediate' from dual union all
select 33037,'033037','2006-026JS','deb Kosmos-2421','',68198,'Reentered','2008-07-25 00:00:00','2008-06-06 00:00:00',92.26,366,405,65.07,'Intermediate' from dual;
insert into SATELLITE
select 33038,'033038','2006-026JT','deb Kosmos-2421','',68198,'Reentered','2008-06-16 00:00:00','2008-06-06 00:00:00',91.75,351,370,65.05,'Intermediate' from dual union all
select 33039,'033039','2006-026JU','deb Kosmos-2421','',68198,'Reentered','2008-06-13 00:00:00','2008-06-06 00:00:00',90.99,312,335,65.03,'Intermediate' from dual union all
select 3304,'003304','1968-054A','Kosmos-229','Zenit-4 No. 41',16781,'Landed','1968-07-04 00:00:00','1968-06-29 00:00:00',89.77,203,323,72.88,'Intermediate' from dual union all
select 33040,'033040','2006-026JV','deb Kosmos-2421','',68198,'Reentered','2008-06-29 00:00:00','2008-06-06 00:00:00',91.66,349,363,65.04,'Intermediate' from dual union all
select 33041,'033041','2006-026JW','deb Kosmos-2421','',68198,'Reentered','2008-07-06 00:00:00','2008-06-06 00:00:00',91.83,352,376,65.04,'Intermediate' from dual union all
select 33042,'033042','2006-026JX','deb Kosmos-2421','',68198,'Reentered','2008-07-24 00:00:00','2008-06-05 00:00:00',92.09,365,389,65.05,'Intermediate' from dual union all
select 33043,'033043','2006-026JY','deb Kosmos-2421','',68198,'Reentered','2008-07-01 00:00:00','2008-06-05 00:00:00',91.69,348,366,65.05,'Intermediate' from dual union all
select 33044,'033044','2006-026JZ','deb Kosmos-2421','',68198,'Reentered','2008-06-29 00:00:00','2008-06-05 00:00:00',91.7,347,369,65.03,'Intermediate' from dual union all
select 33045,'033045','2006-026KA','deb Kosmos-2421','',68198,'Reentered','2008-09-17 00:00:00','2008-06-06 00:00:00',92.55,378,421,65.12,'Intermediate' from dual union all
select 33046,'033046','2006-026KB','deb Kosmos-2421','',68198,'Reentered','2008-09-07 00:00:00','2008-06-05 00:00:00',92.15,375,384,65.01,'Intermediate' from dual union all
select 33047,'033047','2006-026KC','deb Kosmos-2421','',68198,'Reentered','2008-10-20 00:00:00','2008-06-06 00:00:00',92.59,380,423,65.09,'Intermediate' from dual union all
select 33048,'033048','2006-026KD','deb Kosmos-2421','',68198,'Reentered','2008-10-15 00:00:00','2008-06-06 00:00:00',92.49,383,411,65.05,'Intermediate' from dual union all
select 33049,'033049','2006-026KE','deb Kosmos-2421','',68198,'Reentered','2008-06-30 00:00:00','2008-06-06 00:00:00',91.76,350,372,65.05,'Intermediate' from dual union all
select 3305,'003305','1968-054B','11A57I','11A57I',16781,'Reentered','1968-07-04 00:00:00','1968-06-28 00:00:00',89.47,192,305,72.87,'Intermediate' from dual union all
select 33050,'033050','2006-026KF','deb Kosmos-2421','',68198,'Reentered','2008-06-19 00:00:00','2008-06-04 00:00:00',91.45,336,356,65.03,'Intermediate' from dual union all
select 33051,'033051','2008-028A','Zhongxing 9','zhongxing jiuhao',68536,'In Earth orbit','','2008-06-23 00:00:00',1436.09,35775,35798,.05,'Stationary' from dual union all
select 33052,'033052','2008-028B','CZ-3B Stage 3','CZ-3B Stage 3',68536,'Reentered','2014-01-16 00:00:00','2008-06-20 00:00:00',854.71,164,46747,24.13,'Highly Ellip' from dual union all
select 33053,'033053','2008-029A','Fermi Gamma-ray Space Tel.','GLAST',68537,'In Earth orbit','','2008-06-19 00:00:00',95.68,542,561,25.58,'Intermediate' from dual union all
select 33054,'033054','2008-029B','Delta 333','Delta SSPS AJ-10-118K',68537,'Reentered','2008-06-29 00:00:00','2008-06-18 00:00:00',90.71,163,455,21.54,'Equatorial' from dual union all
select 33055,'033055','2008-030A','Skynet 5C','Skynet 5C',68538,'In Earth orbit','','2008-06-20 00:00:00',1430.04,35556,35780,.82,'Drift GEO' from dual union all
select 33056,'033056','2008-030B','Turksat 3A','Turksat 3A',68538,'In Earth orbit','','2008-06-30 00:00:00',1436.04,35782,35789,.06,'Stationary' from dual union all
select 33057,'033057','2008-030C','ESC-A 540','Ariane 5 ESC-A',68538,'In Earth orbit','','2008-06-20 00:00:00',628.98,256,35623,1.77,'GEO Transfer' from dual union all
select 33058,'033058','2008-030D','Sylda 5A','Sylda 5 No. 29A',68538,'In Earth orbit','','2008-06-20 00:00:00',647.7,266,36573,2.02,'GEO Transfer' from dual union all
select 33059,'033059','2008-022B','Blok DM-SLB No. 1TL','Blok DM-SLB No. 1TL',68530,'In Earth orbit','','2008-06-12 00:00:00',1507.29,34967,39371,.81,'Very High' from dual union all
select 3306,'003306','1968-053G','Kapsula Kosmosa-228','Nauka 1KS No. 2',16780,'Reentered','1968-07-07 00:00:00','1968-07-03 00:00:00',88.56,194,213,51.61,'Intermediate' from dual union all
select 33060,'033060','2008-031A','Orbcomm Q2','Orbcomm FM38',68539,'In Earth orbit','','2008-06-26 00:00:00',98.06,661,671,48.45,'Intermediate' from dual union all
select 33061,'033061','2008-031B','Orbcomm Q5','Orbcomm FM41',68539,'In Earth orbit','','2008-06-26 00:00:00',98.06,661,671,48.45,'Intermediate' from dual union all
select 33062,'033062','2008-031C','Orbcomm J1','Orbcomm FM29/CDS-3',68539,'In Earth orbit','','2008-06-26 00:00:00',98.06,661,671,48.45,'Intermediate' from dual union all
select 33063,'033063','2008-031D','Orbcomm Q3','Orbcomm FM39',68539,'In Earth orbit','','2008-06-26 00:00:00',98.07,661,672,48.44,'Intermediate' from dual union all
select 33064,'033064','2008-031E','Orbcomm Q1','Orbcomm FM37',68539,'In Earth orbit','','2008-06-26 00:00:00',98.06,660,672,48.45,'Intermediate' from dual union all
select 33065,'033065','2008-031F','Orbcomm Q4','Orbcomm FM40',68539,'In Earth orbit','','2008-06-30 00:00:00',98.06,661,671,48.45,'Intermediate' from dual union all
select 33066,'033066','2008-031G','S3M','S3M',68539,'In Earth orbit','','2008-06-26 00:00:00',98.1,662,674,48.45,'Intermediate' from dual union all
select 33067,'033067','1999-025DCE','deb FY-1C','',66706,'Reentered','2011-04-11 00:00:00','2008-07-29 00:00:00',97.17,455,792,98.74,'Sun-Synch' from dual union all
select 33068,'033068','2006-026KG','deb Kosmos-2421','',68198,'Reentered','2008-06-24 00:00:00','2008-06-19 00:00:00',91.05,300,353,65.14,'Intermediate' from dual union all
select 33069,'033069','2006-026KH','deb Kosmos-2421','',68198,'Reentered','2008-06-24 00:00:00','2008-06-19 00:00:00',91.26,326,347,64.93,'Intermediate' from dual union all
select 3307,'003307','1968-055A','RAE 1','RAE A',16782,'In Earth orbit','','1968-08-08 00:00:00',224.31,5840,5862,120.84,'Medium' from dual union all
select 33070,'033070','2006-026KJ','deb Kosmos-2421','',68198,'Reentered','2009-03-07 00:00:00','2008-06-19 00:00:00',94.77,362,653,65.55,'Intermediate' from dual union all
select 33071,'033071','2006-026KK','deb Kosmos-2421','',68198,'Reentered','2008-10-13 00:00:00','2008-06-19 00:00:00',93.07,386,463,65.19,'Intermediate' from dual union all
select 33072,'033072','2006-026KL','deb Kosmos-2421','',68198,'Reentered','2008-10-03 00:00:00','2008-06-19 00:00:00',92.28,376,397,65.02,'Intermediate' from dual union all
select 33073,'033073','2006-026KM','deb Kosmos-2421','',68198,'Reentered','2008-08-02 00:00:00','2008-06-19 00:00:00',92.4,366,419,65.01,'Intermediate' from dual union all
select 33074,'033074','2006-026KN','deb Kosmos-2421','',68198,'Reentered','2008-08-12 00:00:00','2008-06-19 00:00:00',92.37,375,407,65.02,'Intermediate' from dual union all
select 33075,'033075','2006-026KP','deb Kosmos-2421','',68198,'Reentered','2009-02-15 00:00:00','2008-06-19 00:00:00',92.58,388,414,65.04,'Intermediate' from dual union all
select 33076,'033076','2006-026KQ','deb Kosmos-2421','',68198,'Reentered','2010-08-28 00:00:00','2008-06-19 00:00:00',92.63,391,415,65.05,'Intermediate' from dual union all
select 33077,'033077','2006-026KR','deb Kosmos-2421','',68198,'Reentered','2008-08-07 00:00:00','2008-06-19 00:00:00',92.33,374,404,65.02,'Intermediate' from dual union all
select 33078,'033078','2006-026KS','deb Kosmos-2421','',68198,'Reentered','2009-11-06 00:00:00','2008-06-19 00:00:00',92.59,389,414,65.05,'Intermediate' from dual union all
select 33079,'033079','2006-026KT','deb Kosmos-2421','',68198,'Reentered','2010-10-28 00:00:00','2008-06-19 00:00:00',98.88,379,1032,65.25,'Intermediate' from dual union all
select 3308,'003308','1968-056A','Kosmos-230','DS-U3-S No. 2',16783,'Reentered','1968-11-02 00:00:00','1968-08-07 00:00:00',92.51,278,518,48.4,'Intermediate' from dual union all
select 33080,'033080','2006-026KU','deb Kosmos-2421','',68198,'Reentered','2008-12-11 00:00:00','2008-06-19 00:00:00',92.29,372,402,65.08,'Intermediate' from dual union all
select 33081,'033081','2006-026KV','deb Kosmos-2421','',68198,'Reentered','2008-09-10 00:00:00','2008-06-19 00:00:00',92.91,386,449,64.97,'Intermediate' from dual union all
select 33082,'033082','2006-026KW','deb Kosmos-2421','',68198,'Reentered','2009-04-12 00:00:00','2008-06-19 00:00:00',92.43,387,401,65.06,'Intermediate' from dual union all
select 33083,'033083','2006-026KX','deb Kosmos-2421','',68198,'Reentered','2008-11-12 00:00:00','2008-06-19 00:00:00',92.46,374,416,65.07,'Intermediate' from dual union all
select 33084,'033084','2006-026KY','deb Kosmos-2421','',68198,'Reentered','2008-09-03 00:00:00','2008-06-19 00:00:00',92.54,384,414,65.02,'Intermediate' from dual union all
select 33085,'033085','2006-026KZ','deb Kosmos-2421','',68198,'Reentered','2012-01-24 00:00:00','2008-06-19 00:00:00',95.8,394,721,64.93,'Intermediate' from dual union all
select 33086,'033086','2006-026LA','deb Kosmos-2421','',68198,'Reentered','2009-01-21 00:00:00','2008-06-19 00:00:00',92.61,388,417,65.08,'Intermediate' from dual union all
select 33087,'033087','2006-026LB','deb Kosmos-2421','',68198,'Reentered','2008-09-08 00:00:00','2008-06-19 00:00:00',92.01,366,381,64.99,'Intermediate' from dual union all
select 33088,'033088','2006-026LC','deb Kosmos-2421','',68198,'Reentered','2008-08-27 00:00:00','2008-06-19 00:00:00',92.41,380,406,65.02,'Intermediate' from dual union all
select 33089,'033089','2006-026LD','deb Kosmos-2421','',68198,'Reentered','2008-11-27 00:00:00','2008-06-19 00:00:00',92.58,387,415,65.05,'Intermediate' from dual union all
select 3309,'003309','1968-056B','S1M','S1M',16783,'Reentered','1968-10-16 00:00:00','1968-08-10 00:00:00',92.31,271,505,48.38,'Intermediate' from dual union all
select 33090,'033090','2006-026LE','deb Kosmos-2421','',68198,'Reentered','2008-11-29 00:00:00','2008-06-19 00:00:00',92.64,381,426,65.11,'Intermediate' from dual union all
select 33091,'033091','2006-026LF','deb Kosmos-2421','',68198,'Reentered','2008-11-02 00:00:00','2008-06-19 00:00:00',93.66,391,516,64.83,'Intermediate' from dual union all
select 33092,'033092','2006-026LG','deb Kosmos-2421','',68198,'Reentered','2008-12-19 00:00:00','2008-06-19 00:00:00',92.65,387,421,65.08,'Intermediate' from dual union all
select 33093,'033093','2006-026LH','deb Kosmos-2421','',68198,'Reentered','2008-06-27 00:00:00','2008-06-19 00:00:00',91.17,312,351,65.08,'Intermediate' from dual union all
select 33094,'033094','2006-026LJ','deb Kosmos-2421','',68198,'Reentered','2008-11-17 00:00:00','2008-06-19 00:00:00',92.72,380,436,65.06,'Intermediate' from dual union all
select 33095,'033095','2006-026LK','deb Kosmos-2421','',68198,'Reentered','2008-11-25 00:00:00','2008-06-19 00:00:00',92.65,380,429,65.1,'Intermediate' from dual union all
select 33096,'033096','2006-026LL','deb Kosmos-2421','',68198,'Reentered','2008-06-21 00:00:00','2008-06-19 00:00:00',90.3,268,311,65.05,'Intermediate' from dual union all
select 33097,'033097','2006-026LM','deb Kosmos-2421','',68198,'Reentered','2008-11-05 00:00:00','2008-06-18 00:00:00',92.26,378,393,65.01,'Intermediate' from dual union all
select 33098,'033098','2006-026LN','deb Kosmos-2421','',68198,'Reentered','2008-07-14 00:00:00','2008-06-19 00:00:00',91.69,342,373,65.06,'Intermediate' from dual union all
select 33099,'033099','2006-026LP','deb Kosmos-2421','',68198,'Reentered','2008-08-08 00:00:00','2008-06-19 00:00:00',92.44,372,416,65.1,'Intermediate' from dual union all
select 331,'000331','61 Omicron 104','deb r/b Ablestar','',7273,'Reentered','1989-03-24 00:00:00','1989-03-23 00:00:00',91,309,339,66.92,'Intermediate' from dual union all
select 3310,'003310','1968-057A','Molniya-1 (9)','Molniya-1 11F67 No. 13',16784,'Reentered','1971-05-15 00:00:00','1968-09-22 00:00:00',717.8,505,39851,65.06,'GEO Transfer' from dual union all
select 33100,'033100','2006-026LQ','deb Kosmos-2421','',68198,'Reentered','2008-06-27 00:00:00','2008-06-19 00:00:00',90.47,281,315,65.06,'Intermediate' from dual union all
select 33101,'033101','2006-026LR','deb Kosmos-2421','',68198,'Reentered','2008-09-25 00:00:00','2008-06-19 00:00:00',92.13,372,386,65,'Intermediate' from dual union all
select 33102,'033102','2006-026LS','deb Kosmos-2421','',68198,'Reentered','2012-01-28 00:00:00','2008-06-19 00:00:00',95.78,393,720,64.87,'Intermediate' from dual union all
select 33103,'033103','2006-026LT','deb Kosmos-2421','',68198,'Reentered','2008-07-10 00:00:00','2008-06-19 00:00:00',92,359,387,65.17,'Intermediate' from dual union all
select 33104,'033104','2006-026LU','deb Kosmos-2421','',68198,'Reentered','2008-06-24 00:00:00','2008-06-19 00:00:00',91.76,330,392,65.08,'Intermediate' from dual union all
select 33105,'033105','2008-032A','Jason-2','Jason-2',68540,'In Earth orbit','','2008-06-27 00:00:00',112.42,1331,1345,66.03,'Intermediate' from dual union all
select 33106,'033106','2008-032B','Delta 334','Delta SSPS AJ-10-118K',68540,'In Earth orbit','','2008-06-27 00:00:00',156.94,2160,4360,65.23,'Medium' from dual union all
select 33107,'033107','2006-026LV','deb Kosmos-2421','',68198,'Reentered','2009-02-10 00:00:00','2008-06-20 00:00:00',92.84,394,434,65,'Intermediate' from dual union all
select 33108,'033108','2008-033A','Kosmos-2440','Oko-1 No. 7127',68541,'In Earth orbit','','2008-07-04 00:00:00',1438.44,35804,35861,2.29,'Drift GEO' from dual union all
select 33109,'033109','2008-033B','Perekhodnik','Sredniy perekhodnik',68541,'Reentered','2008-06-28 00:00:00','2008-06-27 00:00:00',88.21,181,191,51.65,'Intermediate' from dual union all
select 3311,'003311','1968-057B','11S510','11S510',16784,'Reentered','1968-08-09 00:00:00','1968-07-13 00:00:00',91.02,230,420,65.05,'Intermediate' from dual union all
select 33110,'033110','2008-033C','8S812','8S812',68541,'Reentered','2008-07-01 00:00:00','2008-06-29 00:00:00',88.06,178,179,51.66,'Intermediate' from dual union all
select 33111,'033111','2008-033D','Blok DM-2 No. 111L','11S861 No. 111L',68541,'In Earth orbit','','2008-06-29 00:00:00',1436.01,35654,35916,2.34,'Stationary' from dual union all
select 33112,'033112','2008-033E','SOZ','SOZ',68541,'Reentered','2013-05-03 00:00:00','2008-07-05 00:00:00',634.75,293,35883,47.88,'GEO Transfer' from dual union all
select 33113,'033113','2008-033F','SOZ','SOZ',68541,'In Earth orbit','','2008-07-05 00:00:00',635.86,349,35884,47.93,'GEO Transfer' from dual union all
select 33114,'033114','2006-026LW','deb Kosmos-2421','',68198,'Reentered','2011-05-14 00:00:00','2008-07-01 00:00:00',94.92,375,655,65.31,'Intermediate' from dual union all
select 33115,'033115','2006-026LX','deb Kosmos-2421','',68198,'Reentered','2008-09-03 00:00:00','2008-06-30 00:00:00',92.64,381,426,64.99,'Intermediate' from dual union all
select 33116,'033116','2006-026LY','deb Kosmos-2421','',68198,'Reentered','2009-06-24 00:00:00','2008-07-01 00:00:00',92.94,386,451,64.84,'Intermediate' from dual union all
select 33117,'033117','2006-026LZ','deb Kosmos-2421','',68198,'Reentered','2008-09-27 00:00:00','2008-07-01 00:00:00',92.35,373,407,65.08,'Intermediate' from dual union all
select 33118,'033118','2006-026MA','deb Kosmos-2421','',68198,'Reentered','2008-11-25 00:00:00','2008-07-01 00:00:00',93.92,387,545,64.85,'Intermediate' from dual union all
select 33119,'033119','2006-026MB','deb Kosmos-2421','',68198,'Reentered','2010-11-13 00:00:00','2008-07-01 00:00:00',94.22,374,588,65.4,'Intermediate' from dual union all
select 3312,'003312','1968-057C','BOZ','BOZ',16784,'Reentered','1968-08-22 00:00:00','1968-07-12 00:00:00',91.49,238,458,65,'Intermediate' from dual union all
select 33120,'033120','2006-026MC','deb Kosmos-2421','',68198,'Reentered','2009-01-30 00:00:00','2008-07-01 00:00:00',92.86,375,454,65.13,'Intermediate' from dual union all
select 33121,'033121','2006-026MD','deb Kosmos-2421','',68198,'Reentered','2008-09-23 00:00:00','2008-07-01 00:00:00',92.38,378,405,65.04,'Intermediate' from dual union all
select 33122,'033122','2006-026ME','deb Kosmos-2421','',68198,'Reentered','2008-10-05 00:00:00','2008-07-01 00:00:00',92.12,363,394,65.03,'Intermediate' from dual union all
select 33123,'033123','2006-026MF','deb Kosmos-2421','',68198,'Reentered','2008-08-24 00:00:00','2008-07-01 00:00:00',92.26,371,399,65.03,'Intermediate' from dual union all
select 33124,'033124','2006-026MG','deb Kosmos-2421','',68198,'Reentered','2008-08-29 00:00:00','2008-07-01 00:00:00',92.34,373,406,65.02,'Intermediate' from dual union all
select 33125,'033125','2006-026MH','deb Kosmos-2421','',68198,'Reentered','2008-07-31 00:00:00','2008-07-01 00:00:00',91.79,265,460,65.14,'Intermediate' from dual union all
select 33126,'033126','2006-026MJ','deb Kosmos-2421','',68198,'Reentered','2008-10-18 00:00:00','2008-07-01 00:00:00',92.48,384,409,65.03,'Intermediate' from dual union all
select 33127,'033127','2006-026MK','deb Kosmos-2421','',68198,'Reentered','2008-09-11 00:00:00','2008-07-01 00:00:00',92.36,333,448,65.37,'Intermediate' from dual;
insert into SATELLITE
select 33128,'033128','2006-026ML','deb Kosmos-2421','',68198,'Reentered','2009-09-18 00:00:00','2008-07-01 00:00:00',92.88,359,473,65.24,'Intermediate' from dual union all
select 33129,'033129','2006-026MM','deb Kosmos-2421','',68198,'Reentered','2008-07-10 00:00:00','2008-07-01 00:00:00',91.1,318,339,65.08,'Intermediate' from dual union all
select 3313,'003313','1968-057D','deb Blok-I','',16784,'Reentered','1968-07-08 00:00:00','1968-07-06 00:00:00',90.72,217,403,65.03,'Intermediate' from dual union all
select 33130,'033130','2006-026MN','deb Kosmos-2421','',68198,'Reentered','2008-08-12 00:00:00','2008-07-01 00:00:00',92.07,312,440,65.29,'Intermediate' from dual union all
select 33131,'033131','2006-026MP','deb Kosmos-2421','',68198,'Reentered','2008-07-26 00:00:00','2008-06-30 00:00:00',91.5,325,372,65.08,'Intermediate' from dual union all
select 33132,'033132','2006-026MQ','deb Kosmos-2421','',68198,'Reentered','2008-11-20 00:00:00','2008-06-30 00:00:00',93.85,390,536,64.83,'Intermediate' from dual union all
select 33133,'033133','2006-026MR','deb Kosmos-2421','',68198,'Reentered','2008-07-27 00:00:00','2008-07-01 00:00:00',91.94,355,385,65.09,'Intermediate' from dual union all
select 33134,'033134','2006-026MS','deb Kosmos-2421','',68198,'Reentered','2008-11-02 00:00:00','2008-07-01 00:00:00',92.5,385,410,65.02,'Intermediate' from dual union all
select 33135,'033135','2006-026MT','deb Kosmos-2421','',68198,'Reentered','2008-09-12 00:00:00','2008-07-01 00:00:00',92.31,373,403,65.07,'Intermediate' from dual union all
select 33136,'033136','2006-026MU','deb Kosmos-2421','',68198,'Reentered','2008-10-20 00:00:00','2008-07-01 00:00:00',93.45,389,498,64.86,'Intermediate' from dual union all
select 33137,'033137','2006-026MV','deb Kosmos-2421','',68198,'Reentered','2010-09-17 00:00:00','2008-07-01 00:00:00',92.6,374,429,65.03,'Intermediate' from dual union all
select 33138,'033138','2006-026MW','deb Kosmos-2421','',68198,'Reentered','2008-10-04 00:00:00','2008-07-01 00:00:00',92.4,379,406,65.05,'Intermediate' from dual union all
select 33139,'033139','2006-026MX','deb Kosmos-2421','',68198,'Reentered','2010-01-28 00:00:00','2008-07-01 00:00:00',93.85,394,532,64.91,'Intermediate' from dual union all
select 3314,'003314','1968-057E','deb Blok-I','',16784,'Reentered','1968-07-18 00:00:00','1968-07-08 00:00:00',91.31,239,439,65.01,'Intermediate' from dual union all
select 33140,'033140','2006-026MY','deb Kosmos-2421','',68198,'Reentered','2009-04-03 00:00:00','2008-06-30 00:00:00',97.82,382,927,64.82,'Intermediate' from dual union all
select 33141,'033141','2006-026MZ','deb Kosmos-2421','',68198,'Reentered','2011-03-10 00:00:00','2008-07-01 00:00:00',92.6,395,408,65.04,'Intermediate' from dual union all
select 33142,'033142','2006-026NA','deb Kosmos-2421','',68198,'Reentered','2008-08-06 00:00:00','2008-07-01 00:00:00',91.94,351,389,65.03,'Intermediate' from dual union all
select 33143,'033143','2006-026NB','deb Kosmos-2421','',68198,'Reentered','2008-09-20 00:00:00','2008-06-30 00:00:00',92.04,361,389,65.04,'Intermediate' from dual union all
select 33144,'033144','2006-026NC','deb Kosmos-2421','',68198,'Reentered','2008-11-01 00:00:00','2008-06-30 00:00:00',92.49,384,410,65.04,'Intermediate' from dual union all
select 33145,'033145','2006-026ND','deb Kosmos-2421','',68198,'Reentered','2008-10-28 00:00:00','2008-07-01 00:00:00',92.47,383,409,65.04,'Intermediate' from dual union all
select 33146,'033146','2006-026NE','deb Kosmos-2421','',68198,'Reentered','2008-08-28 00:00:00','2008-07-01 00:00:00',92.4,377,408,65.02,'Intermediate' from dual union all
select 33147,'033147','2006-026NF','deb Kosmos-2421','',68198,'Reentered','2008-07-13 00:00:00','2008-06-30 00:00:00',91.03,310,341,65.04,'Intermediate' from dual union all
select 33148,'033148','2006-026NG','deb Kosmos-2421','',68198,'Reentered','2008-10-18 00:00:00','2008-06-30 00:00:00',92.28,374,399,65.04,'Intermediate' from dual union all
select 33149,'033149','2006-026NH','deb Kosmos-2421','',68198,'Reentered','2008-10-20 00:00:00','2008-06-30 00:00:00',92.23,373,395,65.02,'Intermediate' from dual union all
select 3315,'003315','1968-055B','Star 37D S/N 009','Star 37D S/N 9',16782,'In Earth orbit','','1968-08-23 00:00:00',156.84,638,5874,120.63,'Medium' from dual union all
select 33150,'033150','2006-026NJ','deb Kosmos-2421','',68198,'Reentered','2010-10-16 00:00:00','2008-07-01 00:00:00',97.04,396,838,64.98,'Intermediate' from dual union all
select 33151,'033151','2006-026NK','deb Kosmos-2421','',68198,'Reentered','2008-09-23 00:00:00','2008-07-01 00:00:00',91.98,342,402,64.98,'Intermediate' from dual union all
select 33152,'033152','2006-026NL','deb Kosmos-2421','',68198,'Reentered','2008-11-28 00:00:00','2008-07-01 00:00:00',92.69,367,446,65.11,'Intermediate' from dual union all
select 33153,'033153','2008-034A','Protostar 1','Protostar 1',68542,'In Earth orbit','','2008-07-14 00:00:00',1433.1,35702,35754,.06,'Drift GEO' from dual union all
select 33154,'033154','2008-034B','Badr 6','Arabsat 4AR',68542,'In Earth orbit','','2008-07-20 00:00:00',1441.68,35841,35951,.33,'Drift GEO' from dual union all
select 33155,'033155','2008-034C','ESC-A 541','Ariane 5 ESC-A',68542,'In Earth orbit','','2008-07-10 00:00:00',631.33,255,35745,1.98,'GEO Transfer' from dual union all
select 33156,'033156','2008-034D','Sylda 5A','Sylda 5',68542,'In Earth orbit','','2008-07-10 00:00:00',630.34,257,35692,1.98,'GEO Transfer' from dual union all
select 33157,'033157','2006-026NM','deb Kosmos-2421','',68198,'Reentered','2009-04-25 00:00:00','2008-07-11 00:00:00',92.62,389,417,65.01,'Intermediate' from dual union all
select 33158,'033158','2006-026NN','deb Kosmos-2421','',68198,'Reentered','2009-03-24 00:00:00','2008-07-11 00:00:00',92.67,389,422,65.04,'Intermediate' from dual union all
select 33159,'033159','2006-026NP','deb Kosmos-2421','',68198,'Reentered','2008-09-22 00:00:00','2008-07-11 00:00:00',91.98,358,386,65.04,'Intermediate' from dual union all
select 3316,'003316','1968-058A','Kosmos-231','Zenit-2 No. 64',16785,'Landed','1968-07-18 00:00:00','1968-07-13 00:00:00',89.65,205,310,64.95,'Intermediate' from dual union all
select 33160,'033160','2006-026NQ','deb Kosmos-2421','',68198,'Reentered','2008-10-26 00:00:00','2008-07-11 00:00:00',92.57,385,416,65.03,'Intermediate' from dual union all
select 33161,'033161','2006-026NR','deb Kosmos-2421','',68198,'Reentered','2008-10-13 00:00:00','2008-07-11 00:00:00',92.35,374,405,65.07,'Intermediate' from dual union all
select 33162,'033162','2006-026NS','deb Kosmos-2421','',68198,'Reentered','2008-08-18 00:00:00','2008-07-11 00:00:00',91.95,358,383,65.01,'Intermediate' from dual union all
select 33163,'033163','2006-026NT','deb Kosmos-2421','',68198,'Reentered','2008-08-12 00:00:00','2008-07-11 00:00:00',92.02,360,388,65.03,'Intermediate' from dual union all
select 33164,'033164','2006-026NU','deb Kosmos-2421','',68198,'Reentered','2008-11-07 00:00:00','2008-07-11 00:00:00',92.53,384,412,65.04,'Intermediate' from dual union all
select 33165,'033165','2006-026NV','deb Kosmos-2421','',68198,'Reentered','2008-07-15 00:00:00','2008-07-11 00:00:00',91.9,338,397,65.19,'Intermediate' from dual union all
select 33166,'033166','2006-026NW','deb Kosmos-2421','',68198,'Reentered','2008-07-15 00:00:00','2008-07-11 00:00:00',90.19,262,305,65.09,'Intermediate' from dual union all
select 33167,'033167','2006-026NX','deb Kosmos-2421','',68198,'Reentered','2008-11-01 00:00:00','2008-07-11 00:00:00',92.64,387,421,65.03,'Intermediate' from dual union all
select 33168,'033168','2006-026NY','deb Kosmos-2421','',68198,'Reentered','2011-04-03 00:00:00','2008-07-11 00:00:00',92.84,386,441,65.11,'Intermediate' from dual union all
select 33169,'033169','2006-026NZ','deb Kosmos-2421','',68198,'Reentered','2008-10-15 00:00:00','2008-07-10 00:00:00',92.59,386,416,64.99,'Intermediate' from dual union all
select 3317,'003317','1968-058B','11A57I','11A57I',16785,'Reentered','1968-07-20 00:00:00','1968-07-12 00:00:00',89.5,203,297,64.96,'Intermediate' from dual union all
select 33170,'033170','2006-026PA','deb Kosmos-2421','',68198,'Reentered','2009-02-09 00:00:00','2008-07-10 00:00:00',92.87,378,453,65.25,'Intermediate' from dual union all
select 33171,'033171','2006-026PB','deb Kosmos-2421','',68198,'Reentered','2008-09-21 00:00:00','2008-07-11 00:00:00',92.91,343,492,65.31,'Intermediate' from dual union all
select 33172,'033172','2006-026PC','deb Kosmos-2421','',68198,'Reentered','2008-09-23 00:00:00','2008-07-11 00:00:00',92.42,380,406,65.01,'Intermediate' from dual union all
select 33173,'033173','2006-026PD','deb Kosmos-2421','',68198,'Reentered','2008-09-04 00:00:00','2008-07-11 00:00:00',91.8,352,374,65,'Intermediate' from dual union all
select 33174,'033174','2006-026PE','deb Kosmos-2421','',68198,'Reentered','2008-10-03 00:00:00','2008-07-11 00:00:00',91.96,362,380,65,'Intermediate' from dual union all
select 33175,'033175','2006-026PF','deb Kosmos-2421','',68198,'Reentered','2008-10-22 00:00:00','2008-07-11 00:00:00',93.13,388,468,64.77,'Intermediate' from dual union all
select 33176,'033176','2006-026PG','deb Kosmos-2421','',68198,'Reentered','2008-07-23 00:00:00','2008-07-10 00:00:00',91.56,335,368,65.04,'Intermediate' from dual union all
select 33177,'033177','2006-026PH','deb Kosmos-2421','',68198,'Reentered','2008-08-16 00:00:00','2008-07-11 00:00:00',91.91,355,382,65.07,'Intermediate' from dual union all
select 33178,'033178','2006-026PJ','deb Kosmos-2421','',68198,'Reentered','2008-08-16 00:00:00','2008-07-11 00:00:00',92.14,362,396,65.01,'Intermediate' from dual union all
select 33179,'033179','2006-026PK','deb Kosmos-2421','',68198,'Reentered','2008-08-19 00:00:00','2008-07-10 00:00:00',92.01,355,392,65.02,'Intermediate' from dual union all
select 3318,'003318','1968-059A','OV1-15','OV1-15S',16786,'Reentered','1968-11-06 00:00:00','1968-08-12 00:00:00',101.83,147,1544,89.85,'Polar' from dual union all
select 33180,'033180','2006-026PL','deb Kosmos-2421','',68198,'Reentered','2008-10-10 00:00:00','2008-07-11 00:00:00',92.24,371,397,65.07,'Intermediate' from dual union all
select 33181,'033181','2006-026PM','deb Kosmos-2421','',68198,'Reentered','2008-10-19 00:00:00','2008-07-11 00:00:00',92.47,385,407,65.05,'Intermediate' from dual union all
select 33182,'033182','2006-026PN','deb Kosmos-2421','',68198,'Reentered','2008-10-02 00:00:00','2008-07-11 00:00:00',92.57,383,418,65.02,'Intermediate' from dual union all
select 33183,'033183','2006-026PP','deb Kosmos-2421','',68198,'Reentered','2008-08-28 00:00:00','2008-07-11 00:00:00',92.25,366,404,65.02,'Intermediate' from dual union all
select 33184,'033184','2006-026PQ','deb Kosmos-2421','',68198,'Reentered','2008-10-13 00:00:00','2008-07-11 00:00:00',93.21,388,475,64.94,'Intermediate' from dual union all
select 33185,'033185','2006-026PR','deb Kosmos-2421','',68198,'Reentered','2009-02-01 00:00:00','2008-07-11 00:00:00',92.45,376,414,65.08,'Intermediate' from dual union all
select 33186,'033186','2006-026PS','deb Kosmos-2421','',68198,'Reentered','2008-10-20 00:00:00','2008-07-11 00:00:00',92.11,370,386,65,'Intermediate' from dual union all
select 33187,'033187','2006-026PT','deb Kosmos-2421','',68198,'Reentered','2008-08-14 00:00:00','2008-07-11 00:00:00',92.02,358,390,65.03,'Intermediate' from dual union all
select 33188,'033188','2006-026PU','deb Kosmos-2421','',68198,'Reentered','2008-08-19 00:00:00','2008-07-11 00:00:00',92.4,374,411,65,'Intermediate' from dual union all
select 33189,'033189','2006-026PV','deb Kosmos-2421','',68198,'Reentered','2008-07-26 00:00:00','2008-07-11 00:00:00',91.71,344,373,65.04,'Intermediate' from dual union all
select 3319,'003319','1968-059B','LOADS','Cannonball',16786,'Reentered','1968-08-19 00:00:00','1968-07-19 00:00:00',91.25,142,530,89.8,'Polar' from dual union all
select 33190,'033190','2006-026PW','deb Kosmos-2421','',68198,'Reentered','2008-11-12 00:00:00','2008-07-11 00:00:00',92.35,370,410,65.1,'Intermediate' from dual union all
select 33191,'033191','2006-026PX','deb Kosmos-2421','',68198,'Reentered','2008-07-24 00:00:00','2008-07-11 00:00:00',91.84,347,382,65.03,'Intermediate' from dual union all
select 33192,'033192','2006-026PY','deb Kosmos-2421','',68198,'Reentered','2008-09-29 00:00:00','2008-07-11 00:00:00',92.83,384,442,64.99,'Intermediate' from dual union all
select 33193,'033193','2006-026PZ','deb Kosmos-2421','',68198,'Reentered','2008-09-20 00:00:00','2008-07-11 00:00:00',91.81,340,386,65.01,'Intermediate' from dual union all
select 33194,'033194','2006-026QA','deb Kosmos-2421','',68198,'Reentered','2009-01-04 00:00:00','2008-07-11 00:00:00',93.25,359,509,65.25,'Intermediate' from dual union all
select 33195,'033195','2006-026QB','deb Kosmos-2421','',68198,'Reentered','2008-12-04 00:00:00','2008-07-10 00:00:00',92.14,371,388,64.79,'Intermediate' from dual union all
select 33196,'033196','2006-026QC','deb Kosmos-2421','',68198,'Reentered','2009-01-03 00:00:00','2008-07-10 00:00:00',92.65,382,426,65.08,'Intermediate' from dual union all
select 33197,'033197','2006-026QD','deb Kosmos-2421','',68198,'Reentered','2008-08-25 00:00:00','2008-07-10 00:00:00',92.24,373,396,65.02,'Intermediate' from dual union all
select 33198,'033198','2006-026QE','deb Kosmos-2421','',68198,'Reentered','2008-11-16 00:00:00','2008-07-11 00:00:00',92.28,373,400,65.05,'Intermediate' from dual union all
select 33199,'033199','2006-026QF','deb Kosmos-2421','',68198,'Reentered','2008-09-12 00:00:00','2008-07-10 00:00:00',92.11,361,395,65.09,'Intermediate' from dual union all
select 332,'000332','61 Omicron 105','deb r/b Ablestar','',7273,'Reentered','1970-09-20 00:00:00','1962-06-27 00:00:00',99.98,554,961,67.12,'Intermediate' from dual union all
select 3320,'003320','1968-059C','OV1-15P','FW4S',16786,'Reentered','1968-08-16 00:00:00','1968-07-19 00:00:00',102.81,154,1629,89.87,'Polar' from dual union all
select 33200,'033200','2006-026QG','deb Kosmos-2421','',68198,'Reentered','2010-05-21 00:00:00','2008-07-11 00:00:00',92.74,381,436,65.01,'Intermediate' from dual union all
select 33201,'033201','2006-026QH','deb Kosmos-2421','',68198,'Reentered','2008-10-19 00:00:00','2008-07-11 00:00:00',92.6,385,418,65.03,'Intermediate' from dual union all
select 33202,'033202','1998-067BE','EVA debris (blanket)','ISS R EVA-20A debris',66503,'Reentered','2008-09-07 00:00:00','2008-07-12 00:00:00',91.39,337,348,51.64,'Intermediate' from dual union all
select 33203,'033203','1998-067BF','EVA debris (thruster cover)','ISS R EVA-20A debris',66503,'Reentered','2008-08-28 00:00:00','2008-07-12 00:00:00',91.36,337,346,51.64,'Intermediate' from dual union all
select 33204,'033204','1998-067BG','EVA debris (thruster cover)','ISS R EVA-20A debris',66503,'Reentered','2009-03-01 00:00:00','2008-07-13 00:00:00',91.4,337,350,51.64,'Intermediate' from dual union all
select 33205,'033205','1998-067BH','EVA debris','ISS R EVA-20A debris',66503,'Reentered','2008-09-11 00:00:00','2008-07-13 00:00:00',91.37,336,348,51.64,'Intermediate' from dual union all
select 33206,'033206','1998-067BJ','EVA debris','ISS R EVA-20A debris',66503,'Reentered','2008-08-28 00:00:00','2008-07-13 00:00:00',91.37,336,348,51.64,'Intermediate' from dual union all
select 33207,'033207','2008-035A','Echostar XI','Echostar 11',68543,'In Earth orbit','','2008-07-24 00:00:00',1433.61,35685,35791,.01,'Drift GEO' from dual union all
select 33208,'033208','2008-035B','Blok DM-SL No. 28L','Blok DM-SL',68543,'In Earth orbit','','2008-07-31 00:00:00',636.56,667,35602,0,'GEO Transfer' from dual union all
select 33209,'033209','2006-026QJ','deb Kosmos-2421','',68198,'Reentered','2008-09-28 00:00:00','2008-07-18 00:00:00',92.39,375,409,65.03,'Intermediate' from dual union all
select 3321,'003321','1968-059D','OV1-16P','FW4S',16786,'Reentered','1968-07-16 00:00:00','1968-07-13 00:00:00',90.81,146,482,89.79,'Polar' from dual union all
select 33210,'033210','2006-026QK','deb Kosmos-2421','',68198,'Reentered','2008-12-17 00:00:00','2008-07-18 00:00:00',92.6,379,424,65.1,'Intermediate' from dual union all
select 33211,'033211','2006-026QL','deb Kosmos-2421','',68198,'Reentered','2008-09-02 00:00:00','2008-07-18 00:00:00',92.07,362,391,65.05,'Intermediate' from dual union all
select 33212,'033212','2006-026QM','deb Kosmos-2421','',68198,'Reentered','2008-11-29 00:00:00','2008-07-18 00:00:00',92.63,387,420,65.03,'Intermediate' from dual union all
select 33213,'033213','2006-026QN','deb Kosmos-2421','',68198,'Reentered','2008-10-21 00:00:00','2008-07-18 00:00:00',92.49,381,413,65.04,'Intermediate' from dual union all
select 33214,'033214','2006-026QP','deb Kosmos-2421','',68198,'Reentered','2008-08-01 00:00:00','2008-07-18 00:00:00',91.44,330,361,65.05,'Intermediate' from dual union all
select 33215,'033215','2006-026QQ','deb Kosmos-2421','',68198,'Reentered','2008-08-22 00:00:00','2008-07-18 00:00:00',92.2,365,400,65.01,'Intermediate' from dual union all
select 33216,'033216','2006-026QR','deb Kosmos-2421','',68198,'Reentered','2008-10-05 00:00:00','2008-07-18 00:00:00',92.21,365,401,65.08,'Intermediate' from dual union all
select 33217,'033217','2006-026QS','deb Kosmos-2421','',68198,'Reentered','2008-11-27 00:00:00','2008-07-18 00:00:00',92.97,355,485,65.2,'Intermediate' from dual;
insert into SATELLITE
select 33218,'033218','2006-026QT','deb Kosmos-2421','',68198,'Reentered','2008-10-20 00:00:00','2008-07-18 00:00:00',92.28,378,395,65.04,'Intermediate' from dual union all
select 33219,'033219','2006-026QU','deb Kosmos-2421','',68198,'Reentered','2008-07-25 00:00:00','2008-07-18 00:00:00',91.19,316,350,65.05,'Intermediate' from dual union all
select 3322,'003322','1968-060A','Kosmos-232','Zenit-4 No. 47',16787,'Landed','1968-07-24 00:00:00','1968-07-19 00:00:00',89.79,197,331,65.45,'Intermediate' from dual union all
select 33220,'033220','2006-026QV','deb Kosmos-2421','',68198,'Reentered','2008-11-13 00:00:00','2008-07-18 00:00:00',92.59,385,417,65.01,'Intermediate' from dual union all
select 33221,'033221','2006-026QW','deb Kosmos-2421','',68198,'Reentered','2008-08-09 00:00:00','2008-07-18 00:00:00',91.71,343,374,65.03,'Intermediate' from dual union all
select 33222,'033222','2006-026QX','deb Kosmos-2421','',68198,'Reentered','2008-08-10 00:00:00','2008-07-18 00:00:00',91.48,332,363,65.02,'Intermediate' from dual union all
select 33223,'033223','2006-026QY','deb Kosmos-2421','',68198,'Reentered','2008-08-07 00:00:00','2008-07-18 00:00:00',91.53,335,364,65.07,'Intermediate' from dual union all
select 33224,'033224','2006-026QZ','deb Kosmos-2421','',68198,'Reentered','2008-10-22 00:00:00','2008-07-18 00:00:00',92.42,379,408,65.04,'Intermediate' from dual union all
select 33225,'033225','2006-026RA','deb Kosmos-2421','',68198,'Reentered','2008-09-03 00:00:00','2008-07-18 00:00:00',91.97,359,383,65.07,'Intermediate' from dual union all
select 33226,'033226','2006-026RB','deb Kosmos-2421','',68198,'Reentered','2008-11-27 00:00:00','2008-07-18 00:00:00',92.61,388,417,64.99,'Intermediate' from dual union all
select 33227,'033227','2006-026RC','deb Kosmos-2421','',68198,'Reentered','2008-11-25 00:00:00','2008-07-18 00:00:00',92.48,383,410,65.04,'Intermediate' from dual union all
select 33228,'033228','2006-026RD','deb Kosmos-2421','',68198,'Reentered','2008-08-15 00:00:00','2008-07-18 00:00:00',92.07,359,393,65,'Intermediate' from dual union all
select 33229,'033229','2006-026RE','deb Kosmos-2421','',68198,'Reentered','2008-08-25 00:00:00','2008-07-18 00:00:00',91.95,352,389,65.08,'Intermediate' from dual union all
select 3323,'003323','1968-060B','11A57I','11A57I',16787,'Reentered','1968-07-26 00:00:00','1968-07-18 00:00:00',89.58,189,319,65.39,'Intermediate' from dual union all
select 33230,'033230','2006-026RF','deb Kosmos-2421','',68198,'Reentered','2008-07-24 00:00:00','2008-07-18 00:00:00',90.92,301,339,65.07,'Intermediate' from dual union all
select 33231,'033231','2006-026RG','deb Kosmos-2421','',68198,'Reentered','2008-10-12 00:00:00','2008-07-18 00:00:00',92.12,365,392,65.04,'Intermediate' from dual union all
select 33232,'033232','2006-026RH','deb Kosmos-2421','',68198,'Reentered','2008-08-25 00:00:00','2008-07-18 00:00:00',92.07,361,392,65.03,'Intermediate' from dual union all
select 33233,'033233','2006-026RJ','deb Kosmos-2421','',68198,'Reentered','2008-09-16 00:00:00','2008-07-18 00:00:00',92.35,374,405,65.03,'Intermediate' from dual union all
select 33234,'033234','2006-026RK','deb Kosmos-2421','',68198,'Reentered','2008-08-12 00:00:00','2008-07-18 00:00:00',91.76,345,377,65,'Intermediate' from dual union all
select 33235,'033235','2006-026RL','deb Kosmos-2421','',68198,'Reentered','2008-09-16 00:00:00','2008-07-18 00:00:00',92.15,363,397,65.05,'Intermediate' from dual union all
select 33236,'033236','2006-026RM','deb Kosmos-2421','',68198,'Reentered','2009-02-10 00:00:00','2008-07-18 00:00:00',92.46,376,415,65.08,'Intermediate' from dual union all
select 33237,'033237','2006-026RN','deb Kosmos-2421','',68198,'Reentered','2008-10-13 00:00:00','2008-07-16 00:00:00',93.11,386,468,64.94,'Intermediate' from dual union all
select 33238,'033238','2006-026RP','deb Kosmos-2421','',68198,'Reentered','2008-07-31 00:00:00','2008-07-18 00:00:00',91.4,319,367,65.01,'Intermediate' from dual union all
select 33239,'033239','2006-026RQ','deb Kosmos-2421','',68198,'Reentered','2008-09-12 00:00:00','2008-07-18 00:00:00',92.1,360,395,65.09,'Intermediate' from dual union all
select 3324,'003324','1963-014CH','deb West Ford','RCS 0.08',9245,'Reentered','1990-12-14 00:00:00','1967-02-25 00:00:00',165.95,2466,4785,87,'Medium' from dual union all
select 33240,'033240','2006-026RR','deb Kosmos-2421','',68198,'Reentered','2008-08-27 00:00:00','2008-07-18 00:00:00',92.17,364,398,65.01,'Intermediate' from dual union all
select 33241,'033241','2006-026RS','deb Kosmos-2421','',68198,'Reentered','2008-09-30 00:00:00','2008-07-18 00:00:00',92.73,381,435,64.98,'Intermediate' from dual union all
select 33242,'033242','2006-026RT','deb Kosmos-2421','',68198,'Reentered','2008-08-08 00:00:00','2008-07-18 00:00:00',92.13,358,400,64.96,'Intermediate' from dual union all
select 33243,'033243','1998-067BK','EVA debris','ISS R EVA-20 debris',66503,'Reentered','2008-09-07 00:00:00','2008-07-19 00:00:00',91.36,336,347,51.64,'Intermediate' from dual union all
select 33244,'033244','2008-036A','SAR-Lupe 5','SAR-Lupe 5',68544,'In Earth orbit','','2008-07-24 00:00:00',94.31,470,500,98.15,'Sun-Synch' from dual union all
select 33245,'033245','2008-036B','S3M','S3M',68544,'Reentered','2014-05-01 00:00:00','2008-07-24 00:00:00',94.29,469,500,98.15,'Sun-Synch' from dual union all
select 33246,'033246','2006-026RU','deb Kosmos-2421','',68198,'Reentered','2008-10-10 00:00:00','2008-07-25 00:00:00',92.18,361,402,65.09,'Intermediate' from dual union all
select 33247,'033247','2006-026RV','deb Kosmos-2421','',68198,'Reentered','2008-12-19 00:00:00','2008-07-25 00:00:00',92.54,385,413,65.03,'Intermediate' from dual union all
select 33248,'033248','2006-026RW','deb Kosmos-2421','',68198,'Reentered','2008-08-14 00:00:00','2008-07-25 00:00:00',91.95,353,388,64.96,'Intermediate' from dual union all
select 33249,'033249','2006-026RX','deb Kosmos-2421','',68198,'Reentered','2008-10-12 00:00:00','2008-07-25 00:00:00',92.42,378,408,65.01,'Intermediate' from dual union all
select 3325,'003325','1965-082LQ','deb Transtage 4','',11832,'Reentered','1969-03-17 00:00:00','1967-07-30 00:00:00',96.87,532,686,32.05,'Intermediate' from dual union all
select 33250,'033250','2006-026RY','deb Kosmos-2421','',68198,'Reentered','2008-10-10 00:00:00','2008-07-25 00:00:00',92.33,376,402,65.01,'Intermediate' from dual union all
select 33251,'033251','2006-026RZ','deb Kosmos-2421','',68198,'Reentered','2008-08-12 00:00:00','2008-07-25 00:00:00',91.7,338,378,65.01,'Intermediate' from dual union all
select 33252,'033252','2006-026SA','deb Kosmos-2421','',68198,'Reentered','2008-08-12 00:00:00','2008-07-25 00:00:00',91.7,342,374,65.01,'Intermediate' from dual union all
select 33253,'033253','2006-026SB','deb Kosmos-2421','',68198,'Reentered','2008-08-26 00:00:00','2008-07-25 00:00:00',91.83,347,382,65.03,'Intermediate' from dual union all
select 33254,'033254','2006-026SC','deb Kosmos-2421','',68198,'Reentered','2008-08-13 00:00:00','2008-07-25 00:00:00',91.73,349,370,65.01,'Intermediate' from dual union all
select 33255,'033255','2006-026SD','deb Kosmos-2421','',68198,'Reentered','2008-10-05 00:00:00','2008-07-25 00:00:00',92.31,375,401,65.01,'Intermediate' from dual union all
select 33256,'033256','2006-026SE','deb Kosmos-2421','',68198,'Reentered','2008-11-12 00:00:00','2008-07-25 00:00:00',92.48,380,413,65.04,'Intermediate' from dual union all
select 33257,'033257','2006-026SF','deb Kosmos-2421','',68198,'Reentered','2008-10-10 00:00:00','2008-07-25 00:00:00',91.9,357,379,65.03,'Intermediate' from dual union all
select 33258,'033258','2006-026SG','deb Kosmos-2421','',68198,'Reentered','2008-10-05 00:00:00','2008-07-25 00:00:00',92.62,386,420,65.01,'Intermediate' from dual union all
select 33259,'033259','2006-026SH','deb Kosmos-2421','',68198,'Reentered','2008-08-27 00:00:00','2008-07-25 00:00:00',92.84,363,465,64.89,'Intermediate' from dual union all
select 3326,'003326','1968-061A','Kosmos-233','DS-P1-Yu No. 15',16788,'Reentered','1969-02-07 00:00:00','1968-08-17 00:00:00',100.91,198,1406,81.97,'Intermediate' from dual union all
select 33260,'033260','2006-026SJ','deb Kosmos-2421','',68198,'Reentered','2008-08-24 00:00:00','2008-07-25 00:00:00',91.69,339,376,65.02,'Intermediate' from dual union all
select 33261,'033261','2006-026SK','deb Kosmos-2421','',68198,'Reentered','2008-08-21 00:00:00','2008-07-25 00:00:00',91.69,341,374,65.05,'Intermediate' from dual union all
select 33262,'033262','2006-026SL','deb Kosmos-2421','',68198,'Reentered','2008-08-29 00:00:00','2008-07-25 00:00:00',91.92,349,389,65.03,'Intermediate' from dual union all
select 33263,'033263','2006-026SM','deb Kosmos-2421','',68198,'Reentered','2009-07-11 00:00:00','2008-07-25 00:00:00',100.76,308,1282,66.08,'Intermediate' from dual union all
select 33264,'033264','2006-026SN','deb Kosmos-2421','',68198,'Reentered','2008-08-24 00:00:00','2008-07-25 00:00:00',92.43,338,450,65.3,'Intermediate' from dual union all
select 33265,'033265','2006-026SP','deb Kosmos-2421','',68198,'Reentered','2008-08-27 00:00:00','2008-07-25 00:00:00',92.12,333,424,65.06,'Intermediate' from dual union all
select 33266,'033266','2006-026SQ','deb Kosmos-2421','',68198,'Reentered','2008-11-21 00:00:00','2008-07-25 00:00:00',92.38,374,409,65.03,'Intermediate' from dual union all
select 33267,'033267','2006-026SR','deb Kosmos-2421','',68198,'Reentered','2008-12-03 00:00:00','2008-07-25 00:00:00',92.56,378,422,65.1,'Intermediate' from dual union all
select 33268,'033268','2006-026SS','deb Kosmos-2421','',68198,'Reentered','2009-01-01 00:00:00','2008-07-25 00:00:00',92.64,389,419,64.98,'Intermediate' from dual union all
select 33269,'033269','2006-026ST','deb Kosmos-2421','',68198,'Reentered','2008-08-13 00:00:00','2008-07-25 00:00:00',91.29,318,358,65,'Intermediate' from dual union all
select 3327,'003327','1968-061B','S1M','S1M',16788,'Reentered','1969-01-01 00:00:00','1968-08-20 00:00:00',100.29,208,1337,81.96,'Intermediate' from dual union all
select 33270,'033270','2006-026SU','deb Kosmos-2421','',68198,'Reentered','2008-09-25 00:00:00','2008-07-25 00:00:00',92.42,376,411,65,'Intermediate' from dual union all
select 33271,'033271','2006-026SV','deb Kosmos-2421','',68198,'Reentered','2008-08-12 00:00:00','2008-07-25 00:00:00',91.43,324,366,65.08,'Intermediate' from dual union all
select 33272,'033272','2008-037A','Kosmos-2441','Persona No. 1',68545,'In Earth orbit','','2008-08-03 00:00:00',99.25,714,732,98.31,'Sun-Synch' from dual union all
select 33273,'033273','2008-037B','Blok-I 14S54','14S54',68545,'Reentered','2008-12-14 00:00:00','2008-07-28 00:00:00',93.76,199,718,98.31,'Sun-Synch' from dual union all
select 33274,'033274','2008-038A','Superbird C2','Superbird 7',68546,'In Earth orbit','','2008-08-22 00:00:00',1440.46,35762,35982,.15,'Drift GEO' from dual union all
select 33275,'033275','2008-038B','AMC 21','Americom 21',68546,'In Earth orbit','','2008-08-22 00:00:00',1433.99,35707,35784,.01,'Drift GEO' from dual union all
select 33276,'033276','2008-038C','ESC-A 542','Ariane 5 ESC-A',68546,'In Earth orbit','','2008-08-22 00:00:00',630.71,262,35706,1.99,'GEO Transfer' from dual union all
select 33277,'033277','2008-038D','Sylda 5','Sylda 5',68546,'In Earth orbit','','2008-08-22 00:00:00',631.29,262,35736,1.97,'GEO Transfer' from dual union all
select 33278,'033278','2008-039A','Inmarsat 4 F3','INMARSAT 4 F3',68547,'In Earth orbit','','2008-08-28 00:00:00',1430.88,35597,35771,3.09,'Inclined Drift' from dual union all
select 33279,'033279','2008-039B','Briz-M DTB','Briz-M torus tank',68547,'In Earth orbit','','2008-08-27 00:00:00',255.43,311,13603,46.17,'Medium' from dual union all
select 3328,'003328','1968-017D','deb Explorer 37','',16744,'Reentered','1991-09-08 00:00:00','1968-04-19 00:00:00',100.33,645,904,59.61,'Intermediate' from dual union all
select 33280,'033280','2008-039C','Briz-M No. 99502','Briz-M No. 99502',68547,'In Earth orbit','','2008-08-22 00:00:00',714.8,4256,35952,22.88,'GEO Transfer' from dual union all
select 33281,'033281','1994-074K','deb Resurs-O1','',65470,'Reentered','2012-12-05 00:00:00','2008-08-21 00:00:00',97.48,636,641,97.97,'Sun-Synch' from dual union all
select 33282,'033282','2006-026SW','deb Kosmos-2421','',68198,'Reentered','2008-10-19 00:00:00','2008-08-22 00:00:00',92.42,378,408,65.09,'Intermediate' from dual union all
select 33283,'033283','2006-026SX','deb Kosmos-2421','',68198,'Reentered','2008-09-09 00:00:00','2008-08-22 00:00:00',91.28,325,350,65.04,'Intermediate' from dual union all
select 33284,'033284','2006-026SY','deb Kosmos-2421','',68198,'Reentered','2008-08-28 00:00:00','2008-08-22 00:00:00',90.65,294,319,65.03,'Intermediate' from dual union all
select 33285,'033285','2006-026SZ','deb Kosmos-2421','',68198,'Reentered','2008-08-28 00:00:00','2008-08-22 00:00:00',90.52,290,310,65.03,'Intermediate' from dual union all
select 33286,'033286','2006-026TA','deb Kosmos-2421','',68198,'Reentered','2008-11-04 00:00:00','2008-08-22 00:00:00',92.35,376,403,65,'Intermediate' from dual union all
select 33287,'033287','2006-026TB','deb Kosmos-2421','',68198,'Reentered','2008-11-13 00:00:00','2008-08-22 00:00:00',92.3,375,399,65,'Intermediate' from dual union all
select 33288,'033288','2006-026TC','deb Kosmos-2421','',68198,'Reentered','2009-05-02 00:00:00','2008-08-22 00:00:00',92.39,372,412,65.08,'Intermediate' from dual union all
select 33289,'033289','2006-026TD','deb Kosmos-2421','',68198,'Reentered','2008-10-19 00:00:00','2008-08-22 00:00:00',92.25,372,398,64.97,'Intermediate' from dual union all
select 3329,'003329','1963-014CJ','deb West Ford','',9245,'Reentered','1987-10-17 00:00:00','1967-08-31 00:00:00',165.79,2131,5107,86.77,'Medium' from dual union all
select 33290,'033290','2006-026TE','deb Kosmos-2421','',68198,'Reentered','2008-10-21 00:00:00','2008-08-22 00:00:00',91.92,352,386,65.09,'Intermediate' from dual union all
select 33291,'033291','2006-026TF','deb Kosmos-2421','',68198,'Reentered','2008-09-26 00:00:00','2008-08-22 00:00:00',92.26,363,408,64.96,'Intermediate' from dual union all
select 33292,'033292','2006-026TG','deb Kosmos-2421','',68198,'Reentered','2008-10-29 00:00:00','2008-08-22 00:00:00',92.3,373,402,65.01,'Intermediate' from dual union all
select 33293,'033293','2006-026TH','deb Kosmos-2421','',68198,'Reentered','2008-09-10 00:00:00','2008-08-22 00:00:00',91.11,318,340,64.98,'Intermediate' from dual union all
select 33294,'033294','2006-026TJ','deb Kosmos-2421','',68198,'Reentered','2008-09-22 00:00:00','2008-08-22 00:00:00',91.7,345,370,65.04,'Intermediate' from dual union all
select 33295,'033295','2006-026TK','deb Kosmos-2421','',68198,'Reentered','2008-11-17 00:00:00','2008-08-22 00:00:00',92.24,372,397,65.04,'Intermediate' from dual union all
select 33296,'033296','2006-026TL','deb Kosmos-2421','',68198,'Reentered','2008-11-20 00:00:00','2008-08-22 00:00:00',92.23,372,396,65.04,'Intermediate' from dual union all
select 33297,'033297','2006-026TM','deb Kosmos-2421','',68198,'Reentered','2008-09-18 00:00:00','2008-08-21 00:00:00',92.02,354,394,64.93,'Intermediate' from dual union all
select 33298,'033298','2006-026TN','deb Kosmos-2421','',68198,'Reentered','2008-12-11 00:00:00','2008-08-22 00:00:00',92.56,385,415,65,'Intermediate' from dual union all
select 33299,'033299','2006-026TP','deb Kosmos-2421','',68198,'Reentered','2009-01-28 00:00:00','2008-08-22 00:00:00',92.55,386,413,64.98,'Intermediate' from dual union all
select 333,'000333','61 Omicron 106','deb r/b Ablestar','',7273,'In Earth orbit','','1962-06-29 00:00:00',102.84,806,980,66.8,'Intermediate' from dual union all
select 3330,'003330','1963-014CK','deb West Ford','',9245,'Reentered','1970-02-03 00:00:00','1967-08-31 00:00:00',165.79,2426,4811,86.98,'Medium' from dual union all
select 33300,'033300','2006-026TQ','deb Kosmos-2421','',68198,'Reentered','2008-08-30 00:00:00','2008-08-22 00:00:00',90.84,304,328,65.03,'Intermediate' from dual union all
select 33301,'033301','2006-026TR','deb Kosmos-2421','',68198,'Reentered','2008-09-26 00:00:00','2008-08-22 00:00:00',91.87,357,376,65.01,'Intermediate' from dual union all
select 33302,'033302','2006-026TS','deb Kosmos-2421','',68198,'Reentered','2008-11-12 00:00:00','2008-08-22 00:00:00',91.77,351,372,65.01,'Intermediate' from dual union all
select 33303,'033303','2006-026TT','deb Kosmos-2421','',68198,'Reentered','2008-11-29 00:00:00','2008-08-22 00:00:00',92.25,371,399,65.03,'Intermediate' from dual union all
select 33304,'033304','2006-026TU','deb Kosmos-2421','',68198,'Reentered','2008-12-17 00:00:00','2008-08-22 00:00:00',92.54,384,414,65.01,'Intermediate' from dual union all
select 33305,'033305','2006-026TV','deb Kosmos-2421','',68198,'Reentered','2008-09-22 00:00:00','2008-08-22 00:00:00',92.23,373,395,65,'Intermediate' from dual union all
select 33306,'033306','2006-026TW','deb Kosmos-2421','',68198,'Reentered','2008-12-01 00:00:00','2008-08-21 00:00:00',92.47,382,410,65.01,'Intermediate' from dual union all
select 33307,'033307','2006-026TX','deb Kosmos-2421','',68198,'Reentered','2008-11-20 00:00:00','2008-08-22 00:00:00',92.35,380,400,65,'Intermediate' from dual;
insert into SATELLITE
select 33308,'033308','2006-026TY','deb Kosmos-2421','',68198,'Reentered','2008-09-22 00:00:00','2008-08-22 00:00:00',91.22,321,347,65,'Intermediate' from dual union all
select 33309,'033309','2006-026TZ','deb Kosmos-2421','',68198,'Reentered','2008-08-26 00:00:00','2008-08-22 00:00:00',91.05,304,349,64.9,'Intermediate' from dual union all
select 3331,'003331','1965-082LR','deb Transtage 4','',11832,'Reentered','1979-12-01 00:00:00','1967-02-26 00:00:00',100.49,675,889,32.6,'Intermediate' from dual union all
select 33310,'033310','2006-026UA','deb Kosmos-2421','',68198,'Reentered','2008-11-17 00:00:00','2008-08-22 00:00:00',92.35,380,400,64.98,'Intermediate' from dual union all
select 33311,'033311','2006-026UB','deb Kosmos-2421','',68198,'Reentered','2008-08-26 00:00:00','2008-08-22 00:00:00',90.76,296,328,64.9,'Intermediate' from dual union all
select 33312,'033312','2008-040A','Mati','RapidEye 2',68548,'In Earth orbit','','2008-09-05 00:00:00',97.53,625,656,98,'Sun-Synch' from dual union all
select 33313,'033313','2008-040B','Trochia','RapidEye 5',68548,'In Earth orbit','','2008-09-05 00:00:00',97.41,624,646,98,'Sun-Synch' from dual union all
select 33314,'033314','2008-040C','Tachys','RapidEye 1',68548,'In Earth orbit','','2008-09-05 00:00:00',97.06,605,631,98,'Sun-Synch' from dual union all
select 33315,'033315','2008-040D','Choma','RapidEye 3',68548,'In Earth orbit','','2008-09-05 00:00:00',96.97,598,630,98,'Sun-Synch' from dual union all
select 33316,'033316','2008-040E','Choros','RapidEye 4',68548,'In Earth orbit','','2008-09-05 00:00:00',97.29,621,637,98,'Sun-Synch' from dual union all
select 33317,'033317','2008-040F','Dnepr payload adapter','',68548,'Reentered','2013-07-07 00:00:00','2008-09-05 00:00:00',97.53,625,656,98,'Sun-Synch' from dual union all
select 33318,'033318','2008-040G','Dnepr plume shield','GDS',68548,'In Earth orbit','','2008-09-05 00:00:00',97.13,611,632,98,'Sun-Synch' from dual union all
select 33319,'033319','2008-040H','R-36M2 DS','R-36M2 DS',68548,'In Earth orbit','','2008-09-05 00:00:00',104.74,616,1349,97.98,'Sun-Synch' from dual union all
select 3332,'003332','1968-062A','Kosmos-234','Zenit-4 No. 42',16789,'Landed','1968-08-05 00:00:00','1968-08-03 00:00:00',89.48,206,291,51.83,'Intermediate' from dual union all
select 33320,'033320','2008-041A','Huan Jing 1A','huanjing jian zai A xing',68549,'In Earth orbit','','2008-09-08 00:00:00',97.61,628,661,98.01,'Sun-Synch' from dual union all
select 33321,'033321','2008-041B','Huan Jing 1B','huanjing jian zai B xing',68549,'In Earth orbit','','2008-09-08 00:00:00',97.69,626,671,98.01,'Sun-Synch' from dual union all
select 33322,'033322','2008-041C','CZ-2C Stage 2','CZ-2C Stage 2',68549,'Reentered','2010-01-22 00:00:00','2008-09-08 00:00:00',94.02,242,701,98.01,'Sun-Synch' from dual union all
select 33323,'033323','2008-041D','SMA kick stage','',68549,'In Earth orbit','','2008-09-08 00:00:00',97.67,626,669,98.01,'Sun-Synch' from dual union all
select 33324,'033324','2008-041E','HJ-1 adapter','',68549,'Reentered','2010-08-12 00:00:00','2008-09-10 00:00:00',93.94,291,644,98.01,'Sun-Synch' from dual union all
select 33325,'033325','2008-041F','deb CZ-2C','',68549,'Reentered','2008-10-02 00:00:00','2008-09-08 00:00:00',93.73,264,650,97.96,'Sun-Synch' from dual union all
select 33326,'033326','2008-041G','deb CZ-2C','',68549,'Reentered','2009-08-27 00:00:00','2008-09-08 00:00:00',93.87,268,660,97.86,'Sun-Synch' from dual union all
select 33327,'033327','2008-041H','deb CZ-2C','',68549,'Reentered','2010-03-19 00:00:00','2008-09-08 00:00:00',94.73,349,662,97.71,'Sun-Synch' from dual union all
select 33328,'033328','2008-041J','deb CZ-2C','',68549,'Reentered','2011-10-11 00:00:00','2008-09-08 00:00:00',94.31,318,653,97.81,'Sun-Synch' from dual union all
select 33329,'033329','2008-041K','deb CZ-2C','',68549,'Reentered','2012-10-08 00:00:00','2008-09-08 00:00:00',94.47,347,639,97.71,'Sun-Synch' from dual union all
select 3333,'003333','1968-062B','11A57I','11A57I',16789,'Reentered','1968-08-07 00:00:00','1968-08-01 00:00:00',89.29,209,270,51.82,'Intermediate' from dual union all
select 33330,'033330','2008-041L','deb CZ-2C','',68549,'Reentered','2008-12-27 00:00:00','2008-09-08 00:00:00',93.12,232,623,98.03,'Sun-Synch' from dual union all
select 33331,'033331','2008-042A','GeoEye-1','GeoEye-1',68550,'In Earth orbit','','2008-09-09 00:00:00',98.32,671,685,98.13,'Sun-Synch' from dual union all
select 33332,'033332','2008-042B','Delta 335','Delta SSPS AJ-10-118K',68550,'Reentered','2008-10-21 00:00:00','2008-09-08 00:00:00',93.15,180,678,96.66,'Sun-Synch' from dual union all
select 33333,'033333','2008-041M','deb CZ-2C','',68549,'Reentered','2010-08-15 00:00:00','2008-09-08 00:00:00',94.41,318,662,97.75,'Sun-Synch' from dual union all
select 33334,'033334','2008-041N','deb CZ-2C','',68549,'Reentered','2010-06-03 00:00:00','2008-09-08 00:00:00',94.01,290,652,98.05,'Sun-Synch' from dual union all
select 33335,'033335','2008-041P','deb CZ-2C','',68549,'Reentered','2009-07-22 00:00:00','2008-09-08 00:00:00',94.34,339,634,97.63,'Sun-Synch' from dual union all
select 33336,'033336','2008-041Q','deb CZ-2C','',68549,'Reentered','2008-09-11 00:00:00','2008-09-08 00:00:00',92.6,298,505,97.7,'Sun-Synch' from dual union all
select 33337,'033337','2008-041R','deb CZ-2C','',68549,'Reentered','2009-01-28 00:00:00','2008-09-08 00:00:00',93.15,236,622,97.99,'Sun-Synch' from dual union all
select 33338,'033338','2008-041S','deb CZ-2C','',68549,'Reentered','2008-12-31 00:00:00','2008-09-08 00:00:00',94.35,321,654,97.62,'Sun-Synch' from dual union all
select 33339,'033339','2008-041T','deb CZ-2C','',68549,'Reentered','2009-10-01 00:00:00','2008-09-10 00:00:00',94.47,344,642,97.79,'Sun-Synch' from dual union all
select 3334,'003334','1968-063A','OPS 2222','CANYON 1',16790,'In Earth orbit','','1968-12-30 00:00:00',1435.4,31680,39866,9.9,'Near-synch' from dual union all
select 33340,'033340','2008-043A','Progress M-65','Progress 7K-TGM No.365',68551,'Deorbited','2008-12-07 00:00:00','2008-09-12 00:00:00',91.27,308,366,51.64,'Intermediate' from dual union all
select 33341,'033341','2008-043B','11S510','11S510',68551,'Reentered','2008-09-14 00:00:00','2008-09-13 00:00:00',88.07,169,189,51.62,'Intermediate' from dual union all
select 33342,'033342','2008-041U','deb CZ-2C','',68549,'Reentered','2008-09-20 00:00:00','2008-09-10 00:00:00',92.19,216,548,98,'Sun-Synch' from dual union all
select 33343,'033343','2008-041V','CZ-2C Stage 2 motor cover','',68549,'Reentered','2008-12-04 00:00:00','2008-09-11 00:00:00',96.01,249,886,97.82,'Sun-Synch' from dual union all
select 33344,'033344','2008-041W','deb CZ-2C','',68549,'Reentered','2008-10-19 00:00:00','2008-09-10 00:00:00',93.68,300,610,97.91,'Sun-Synch' from dual union all
select 33345,'033345','2008-041X','deb CZ-2C','',68549,'Reentered','2008-09-19 00:00:00','2008-09-11 00:00:00',93.31,316,557,97.78,'Sun-Synch' from dual union all
select 33346,'033346','2008-041Y','CZ-2C Stage 2 motor cover','',68549,'Reentered','2008-11-14 00:00:00','2008-09-11 00:00:00',95.85,238,881,97.79,'Sun-Synch' from dual union all
select 33347,'033347','2008-041Z','CZ-2C Stage 2 motor cover','',68549,'Reentered','2008-12-04 00:00:00','2008-09-11 00:00:00',95.79,249,864,98.24,'Sun-Synch' from dual union all
select 33348,'033348','2006-026UC','deb Kosmos-2421','',68198,'Reentered','2008-10-07 00:00:00','2008-09-15 00:00:00',91.62,343,365,64.97,'Intermediate' from dual union all
select 33349,'033349','2006-026UD','deb Kosmos-2421','',68198,'Reentered','2009-01-07 00:00:00','2008-09-15 00:00:00',92.44,381,408,65.01,'Intermediate' from dual union all
select 3335,'003335','1968-064A','OPS 5187','GAMBIT-3 Mission 4315',16791,'Reentered Attached','1968-08-16 00:00:00','1968-08-09 00:00:00',89.86,143,392,110,'Retrograde' from dual union all
select 33350,'033350','2006-026UE','deb Kosmos-2421','',68198,'Reentered','2008-11-04 00:00:00','2008-09-15 00:00:00',91.79,343,382,65.09,'Intermediate' from dual union all
select 33351,'033351','2006-026UF','deb Kosmos-2421','',68198,'Reentered','2010-03-10 00:00:00','2008-09-15 00:00:00',92.29,376,398,65.02,'Intermediate' from dual union all
select 33352,'033352','2006-026UG','deb Kosmos-2421','',68198,'Reentered','2009-01-03 00:00:00','2008-09-15 00:00:00',92.35,377,403,65.08,'Intermediate' from dual union all
select 33353,'033353','2006-026UH','deb Kosmos-2421','',68198,'Reentered','2008-12-25 00:00:00','2008-09-15 00:00:00',92.4,379,406,65.01,'Intermediate' from dual union all
select 33354,'033354','2006-026UJ','deb Kosmos-2421','',68198,'Reentered','2009-04-11 00:00:00','2008-09-15 00:00:00',92.64,388,420,65.01,'Intermediate' from dual union all
select 33355,'033355','2006-026UK','deb Kosmos-2421','',68198,'Reentered','2008-11-07 00:00:00','2008-09-14 00:00:00',92,357,389,65.1,'Intermediate' from dual union all
select 33356,'033356','2006-026UL','deb Kosmos-2421','',68198,'Reentered','2008-12-02 00:00:00','2008-09-15 00:00:00',92.18,369,394,65.04,'Intermediate' from dual union all
select 33357,'033357','2006-026UM','deb Kosmos-2421','',68198,'Reentered','2008-09-23 00:00:00','2008-09-15 00:00:00',90.62,289,321,65.08,'Intermediate' from dual union all
select 33358,'033358','2006-026UN','deb Kosmos-2421','',68198,'Reentered','2008-10-08 00:00:00','2008-09-11 00:00:00',91.51,339,359,65.05,'Intermediate' from dual union all
select 33359,'033359','2006-026UP','deb Kosmos-2421','',68198,'Reentered','2009-01-23 00:00:00','2008-09-15 00:00:00',92.18,361,402,65.09,'Intermediate' from dual union all
select 3336,'003336','1968-065A','OPS 5955','CORONA CR-4',16792,'Reentered Attached','1968-08-27 00:00:00','1968-08-14 00:00:00',88.52,151,252,82.16,'Intermediate' from dual union all
select 33360,'033360','2006-026UQ','deb Kosmos-2421','',68198,'Reentered','2008-09-29 00:00:00','2008-09-15 00:00:00',91,308,340,65.09,'Intermediate' from dual union all
select 33361,'033361','2006-026UR','deb Kosmos-2421','',68198,'Reentered','2008-09-27 00:00:00','2008-09-15 00:00:00',91.14,317,344,64.98,'Intermediate' from dual union all
select 33362,'033362','2006-026US','deb Kosmos-2421','',68198,'Reentered','2008-10-02 00:00:00','2008-09-15 00:00:00',91.18,321,344,65.02,'Intermediate' from dual union all
select 33363,'033363','2006-026UT','deb Kosmos-2421','',68198,'Reentered','2008-12-18 00:00:00','2008-09-15 00:00:00',92.36,377,404,65,'Intermediate' from dual union all
select 33364,'033364','2006-026UU','deb Kosmos-2421','',68198,'Reentered','2008-09-30 00:00:00','2008-09-15 00:00:00',91.2,317,350,65.04,'Intermediate' from dual union all
select 33365,'033365','2006-026UV','deb Kosmos-2421','',68198,'Reentered','2008-11-21 00:00:00','2008-09-15 00:00:00',92.05,362,389,65.04,'Intermediate' from dual union all
select 33366,'033366','2006-026UW','deb Kosmos-2421','',68198,'Reentered','2008-12-12 00:00:00','2008-09-15 00:00:00',92.36,377,404,65,'Intermediate' from dual union all
select 33367,'033367','2006-026UX','deb Kosmos-2421','',68198,'Reentered','2008-10-15 00:00:00','2008-09-15 00:00:00',91.44,330,361,65.05,'Intermediate' from dual union all
select 33368,'033368','2006-026UY','deb Kosmos-2421','',68198,'Reentered','2008-10-29 00:00:00','2008-09-15 00:00:00',91.89,358,377,64.99,'Intermediate' from dual union all
select 33369,'033369','2006-026UZ','deb Kosmos-2421','',68198,'Reentered','2008-10-22 00:00:00','2008-09-15 00:00:00',91.75,343,378,65.07,'Intermediate' from dual union all
select 3337,'003337','1968-066A','Explorer 39','AD-C',16793,'Reentered','1981-06-22 00:00:00','1968-09-08 00:00:00',118.17,684,2514,80.69,'Intermediate' from dual union all
select 33370,'033370','2006-026VA','deb Kosmos-2421','',68198,'Reentered','2009-01-07 00:00:00','2008-09-15 00:00:00',92.18,362,401,65.08,'Intermediate' from dual union all
select 33371,'033371','2006-026VB','deb Kosmos-2421','',68198,'Reentered','2008-10-23 00:00:00','2008-09-15 00:00:00',91.68,338,376,65.09,'Intermediate' from dual union all
select 33372,'033372','2008-041AA','CZ-2C Stage 2 motor cover','',68549,'Reentered','2008-11-17 00:00:00','2008-09-19 00:00:00',95.51,236,851,98.19,'Sun-Synch' from dual union all
select 33373,'033373','2008-044A','Nimiq 4','Nimiq 4',68552,'In Earth orbit','','2008-09-26 00:00:00',1434.07,35705,35788,.03,'Drift GEO' from dual union all
select 33374,'033374','2008-044B','Briz-M DTB','Briz-M torus tank',68552,'In Earth orbit','','2008-09-27 00:00:00',300.95,315,16681,49.52,'Highly Ellip' from dual union all
select 33375,'033375','2008-044C','Briz-M No. 88528','Briz-M 88528',68552,'In Earth orbit','','2008-09-26 00:00:00',707.02,4285,35538,12.69,'GEO Transfer' from dual union all
select 33376,'033376','2008-045A','Galaxy 19','Galaxy 19',68553,'In Earth orbit','','2008-10-01 00:00:00',1433.07,35643,35812,.08,'Drift GEO' from dual union all
select 33377,'033377','2008-045B','Blok DM-SL No. 29L','Blok DM-SL',68553,'In Earth orbit','','2008-10-01 00:00:00',676.45,2724,35572,.06,'GEO Transfer' from dual union all
select 33378,'033378','2008-046A','Kosmos-2442','Uragan-M No. 724',68554,'In Earth orbit','','2008-10-02 00:00:00',675.18,19087,19145,64.8,'Medium' from dual union all
select 33379,'033379','2008-046B','Kosmos-2443','Uragan-M No. 725',68554,'In Earth orbit','','2008-10-02 00:00:00',678.97,19140,19283,64.79,'Medium' from dual union all
select 3338,'003338','1968-066B','Injun 5','Injun E',16793,'In Earth orbit','','1968-09-11 00:00:00',118.31,679,2531,80.66,'Intermediate' from dual union all
select 33380,'033380','2008-046C','Kosmos-2444','Uragan-M No. 726',68554,'In Earth orbit','','2008-10-03 00:00:00',678.95,19139,19283,64.8,'Medium' from dual union all
select 33381,'033381','2008-046D','Perekhodnik','Sredniy perekhodnik',68554,'Reentered','2008-09-26 00:00:00','2008-09-25 00:00:00',87.43,146,149,64.81,'Intermediate' from dual union all
select 33382,'033382','2008-046E','8S812','8S812',68554,'Reentered','2008-09-26 00:00:00','2008-09-25 00:00:00',87.77,155,174,64.81,'Intermediate' from dual union all
select 33383,'033383','2008-046F','Blok DM-2 No. 112L','11S861 No. 112L',68554,'In Earth orbit','','2008-10-02 00:00:00',671.88,18924,19142,64.79,'Medium' from dual union all
select 33384,'033384','2008-046G','SOZ','SOZ',68554,'In Earth orbit','','2008-10-02 00:00:00',340.37,398,19143,64.84,'Highly Ellip' from dual union all
select 33385,'033385','2008-046H','SOZ','SOZ',68554,'In Earth orbit','','2008-10-02 00:00:00',340.39,398,19144,64.7,'Highly Ellip' from dual union all
select 33386,'033386','2008-047A','Shenzhou 7','shenzhou qihao fanhui ca',68555,'Landed','2008-09-28 00:00:00','2008-09-27 00:00:00',91.18,329,335,42.4,'Intermediate' from dual union all
select 33387,'033387','2008-047B','CZ-2F Stage 2','CZ-2F Stage 2',68555,'Reentered','2008-10-17 00:00:00','2008-09-27 00:00:00',89.67,198,319,42.41,'Intermediate' from dual union all
select 33388,'033388','2008-047C','CZ-2F sep motor cover','',68555,'Reentered','2008-09-29 00:00:00','2008-09-27 00:00:00',90.14,184,379,42.57,'Intermediate' from dual union all
select 33389,'033389','2008-047D','CZ-2F sep motor cover','',68555,'Reentered','2008-09-30 00:00:00','2008-09-27 00:00:00',90.46,183,412,42.53,'Intermediate' from dual union all
select 3339,'003339','1968-067A','Kosmos-235','Zenit-2 No. 61',16794,'Landed','1968-08-17 00:00:00','1968-08-13 00:00:00',89.36,203,283,51.8,'Intermediate' from dual union all
select 33390,'033390','2008-047E','CZ-2F sep motor cover','',68555,'Reentered','2008-09-30 00:00:00','2008-09-27 00:00:00',91.2,192,475,42.29,'Intermediate' from dual union all
select 33391,'033391','2008-047F','CZ-2F sep motor cover','',68555,'Reentered','2008-10-01 00:00:00','2008-09-27 00:00:00',91.27,192,482,42.24,'Intermediate' from dual union all
select 33392,'033392','2008-047G','BX-1','shenzhou qihao banxing',68555,'Reentered','2009-10-29 00:00:00','2008-09-27 00:00:00',91.2,328,338,42.4,'Intermediate' from dual union all
select 33393,'033393','2008-048A','Ratsat','Mass simulator',68556,'In Earth orbit','','2008-09-30 00:00:00',97.35,621,643,9.35,'Equatorial' from dual union all
select 33394,'033394','2008-047H','Shenzhou 7 Guidao Cang','shenzhou qihao guidao ca',68555,'Reentered','2010-01-04 00:00:00','2008-09-29 00:00:00',91.18,328,337,42.4,'Intermediate' from dual union all
select 33395,'033395','1988-033K','deb NNS O-23 insulation','',61282,'In Earth orbit','','2008-10-21 00:00:00',108.49,1014,1299,90.25,'Polar' from dual union all
select 33396,'033396','2008-049A','Thaichote','THEOS',68557,'In Earth orbit','','2008-10-01 00:00:00',98.39,672,692,98.8,'Sun-Synch' from dual union all
select 33397,'033397','2008-049B','R-36M2 DS','R-36M2 DS',68557,'In Earth orbit','','2007-06-30 00:00:00',108.74,545,1791,64.52,'Intermediate' from dual union all
select 33398,'033398','2008-049C','Dnepr plume shield','GDS',68557,'In Earth orbit','','2007-06-30 00:00:00',95.54,534,556,64.51,'Intermediate' from dual;
insert into SATELLITE
select 33399,'033399','2008-050A','Soyuz TMA-13','Soyuz 11F732A17 No. 223',68558,'Landed','2009-04-08 00:00:00','2008-10-15 00:00:00',91.57,350,354,51.64,'Intermediate' from dual union all
select 334,'000334','61 Omicron 107','deb r/b Ablestar','',7273,'In Earth orbit','','1991-04-05 00:00:00',103.62,873,987,66.92,'Intermediate' from dual union all
select 3340,'003340','1968-067B','11A57I','11A57I',16794,'Reentered','1968-08-14 00:00:00','1968-08-11 00:00:00',89.06,189,267,51.79,'Intermediate' from dual union all
select 33400,'033400','2008-050B','11S510','11S510',68558,'Reentered','2008-10-16 00:00:00','2008-10-15 00:00:00',88.14,172,193,51.63,'Intermediate' from dual union all
select 33401,'033401','2008-051A','IBEX','IBEX',68559,'Deep Space','2008-10-20 00:00:00','2009-09-25 00:00:00',11084.78,12260,304347,21.38,'Deep Space' from dual union all
select 33402,'033402','2008-051B','Orion 38','Orion 38',68559,'Reentered','2008-11-16 00:00:00','2008-10-20 00:00:00',90.71,210,409,11,'Equatorial' from dual union all
select 33403,'033403','2008-051C','IBEX adapter cone','',68559,'Reentered','2008-10-28 00:00:00','2008-10-21 00:00:00',90.61,208,401,11,'Equatorial' from dual union all
select 33404,'033404','2008-051D','IBEX kick motor','Star 27H',68559,'Deep Space','2008-10-20 00:00:00','2008-10-19 00:00:00',7921.04,219,250281,11.03,'Deep Space' from dual union all
select 33405,'033405','2008-052A','Chandrayaan-1','Chandrayaan-1',68560,'Deep Space','2008-10-26 00:00:00','2008-10-29 00:00:00',4340.93,372,163167,17.97,'Deep Space' from dual union all
select 33406,'033406','2008-052B','PSLV-C11 PS4','PS4',68560,'In Earth orbit','','2008-10-28 00:00:00',402.98,257,23132,17.81,'Highly Ellip' from dual union all
select 33407,'033407','2008-052C','deb PSLV','',68560,'Lost','','2008-11-17 00:00:00',402.38,257,23096,17.83,'Highly Ellip' from dual union all
select 33408,'033408','2008-053A','Shi Jian 6/3B','shijian liuhao san zu B',68561,'In Earth orbit','','2008-10-27 00:00:00',96.54,582,604,97.74,'Sun-Synch' from dual union all
select 33409,'033409','2008-053B','Shi Jian 6/3A','shijian liuhao san zu A',68561,'In Earth orbit','','2008-10-27 00:00:00',96.52,580,604,97.74,'Sun-Synch' from dual union all
select 3341,'003341','1968-066C','FW4S S/N 2223-4','FW4S',16793,'In Earth orbit','','1968-09-24 00:00:00',118.33,680,2532,80.66,'Intermediate' from dual union all
select 33410,'033410','2008-053C','CZ-4B Stage  3','CZ-4B',68561,'In Earth orbit','','2008-10-27 00:00:00',96.76,504,703,97.6,'Sun-Synch' from dual union all
select 33411,'033411','2008-053D','Adapter','',68561,'Reentered','2017-11-11 00:00:00','2008-10-27 00:00:00',96.5,579,603,97.74,'Sun-Synch' from dual union all
select 33412,'033412','2008-054A','COSMO 3','COSMO-Skymed 3',68562,'In Earth orbit','','2008-10-27 00:00:00',97.15,621,624,97.86,'Sun-Synch' from dual union all
select 33413,'033413','2008-054B','Delta 336','Delta SSPS AJ-10-118K',68562,'Reentered','2008-12-21 00:00:00','2008-10-27 00:00:00',92.62,188,618,98.36,'Sun-Synch' from dual union all
select 33414,'033414','2008-055A','Simon Bolivar','Venesat',68563,'In Earth orbit','','2008-11-12 00:00:00',1436.06,35784,35788,.36,'Stationary' from dual union all
select 33415,'033415','2008-055B','CZ-3B Stage 3','CZ-3B Stage 3',68563,'In Earth orbit','','2008-11-09 00:00:00',751.12,219,41768,24.36,'Highly Ellip' from dual union all
select 33416,'033416','2006-026VC','deb Kosmos-2421','',68198,'Reentered','2008-11-18 00:00:00','2008-11-03 00:00:00',91.12,325,334,65,'Intermediate' from dual union all
select 33417,'033417','2006-026VD','deb Kosmos-2421','',68198,'Reentered','2008-12-04 00:00:00','2008-11-01 00:00:00',91.5,342,355,65.13,'Intermediate' from dual union all
select 33418,'033418','2006-026VE','deb Kosmos-2421','',68198,'Reentered','2009-01-12 00:00:00','2008-11-03 00:00:00',92.02,370,378,65.03,'Intermediate' from dual union all
select 33419,'033419','2006-026VF','deb Kosmos-2421','',68198,'Reentered','2009-01-23 00:00:00','2008-11-03 00:00:00',92.02,368,380,64.99,'Intermediate' from dual union all
select 3342,'003342','1968-066D','Injun despin weight','',16793,'Reentered','2002-01-31 00:00:00','1968-09-08 00:00:00',118.28,673,2535,80.7,'Intermediate' from dual union all
select 33420,'033420','2006-026VG','deb Kosmos-2421','',68198,'Reentered','2008-11-15 00:00:00','2008-11-03 00:00:00',90.97,314,330,65.02,'Intermediate' from dual union all
select 33421,'033421','2006-026VH','deb Kosmos-2421','',68198,'Reentered','2008-12-14 00:00:00','2008-11-03 00:00:00',91.65,347,363,65.01,'Intermediate' from dual union all
select 33422,'033422','2006-026VJ','deb Kosmos-2421','',68198,'Reentered','2008-11-12 00:00:00','2008-11-03 00:00:00',90.69,301,315,65.02,'Intermediate' from dual union all
select 33423,'033423','2006-026VK','deb Kosmos-2421','',68198,'Reentered','2008-11-23 00:00:00','2008-11-03 00:00:00',90.84,305,327,65.03,'Intermediate' from dual union all
select 33424,'033424','2006-026VL','deb Kosmos-2421','',68198,'Reentered','2008-11-18 00:00:00','2008-11-03 00:00:00',91.24,326,345,64.99,'Intermediate' from dual union all
select 33425,'033425','2006-026VM','deb Kosmos-2421','',68198,'Reentered','2008-11-09 00:00:00','2008-11-03 00:00:00',90.2,278,291,65.05,'Intermediate' from dual union all
select 33426,'033426','2006-026VN','deb Kosmos-2421','',68198,'Reentered','2008-12-19 00:00:00','2008-11-03 00:00:00',91.49,330,365,65.08,'Intermediate' from dual union all
select 33427,'033427','2006-026VP','deb Kosmos-2421','',68198,'Reentered','2008-12-23 00:00:00','2008-11-03 00:00:00',91.75,355,366,64.98,'Intermediate' from dual union all
select 33428,'033428','2006-026VQ','deb Kosmos-2421','',68198,'Reentered','2009-01-08 00:00:00','2008-11-03 00:00:00',92.03,368,381,64.98,'Intermediate' from dual union all
select 33429,'033429','2006-026VR','deb Kosmos-2421','',68198,'Reentered','2009-08-25 00:00:00','2008-11-03 00:00:00',92.26,371,399,65.07,'Intermediate' from dual union all
select 3343,'003343','1968-066E','Injun despin weight','',16793,'Reentered','2000-02-09 00:00:00','1968-09-07 00:00:00',118.42,680,2540,80.67,'Intermediate' from dual union all
select 33430,'033430','2006-026VS','deb Kosmos-2421','',68198,'Reentered','2008-12-01 00:00:00','2008-10-31 00:00:00',91.57,345,359,65.01,'Intermediate' from dual union all
select 33431,'033431','2006-026VT','deb Kosmos-2421','',68198,'Reentered','2008-11-12 00:00:00','2008-11-03 00:00:00',90.63,298,312,65.03,'Intermediate' from dual union all
select 33432,'033432','2006-026VU','deb Kosmos-2421','',68198,'Reentered','2008-11-13 00:00:00','2008-11-03 00:00:00',90.73,299,322,65.09,'Intermediate' from dual union all
select 33433,'033433','2008-056A','Shiyan Weixing 3','shiyan weixing sanhao',68564,'In Earth orbit','','2008-11-12 00:00:00',100.76,786,804,98.48,'Sun-Synch' from dual union all
select 33434,'033434','2008-056B','Chuangxin-1 02','chuangxin yihao er xing',68564,'In Earth orbit','','2008-11-12 00:00:00',100.76,786,804,98.48,'Sun-Synch' from dual union all
select 33435,'033435','2008-056C','CZ-2D Stage 2','CZ-2D Stage 2',68564,'Reentered','2013-01-29 00:00:00','2008-11-12 00:00:00',96.09,309,833,98.07,'Sun-Synch' from dual union all
select 33436,'033436','2008-057A','Astra 1M','Astra 1M',68565,'In Earth orbit','','2008-12-03 00:00:00',1436.09,35776,35797,.16,'Stationary' from dual union all
select 33437,'033437','2008-057B','Briz-M DTB','Briz-M torus tank',68565,'In Earth orbit','','2008-11-13 00:00:00',263.56,310,14167,49.63,'Highly Ellip' from dual union all
select 33438,'033438','2008-057C','Briz-M No. 88531','Briz-M 88531',68565,'In Earth orbit','','2008-11-13 00:00:00',719.95,4773,35689,21.42,'GEO Transfer' from dual union all
select 33439,'033439','2008-058A','Kosmos-2445','Kobal''t-M No. 559',68566,'Landed','2009-02-23 00:00:00','2008-11-21 00:00:00',89.86,193,342,67.16,'Intermediate' from dual union all
select 3344,'003344','1968-068A','ATS 4','ATS D',16795,'Reentered Attached','1968-10-17 00:00:00','1968-09-09 00:00:00',93.28,212,659,29.05,'Intermediate' from dual union all
select 33440,'033440','2008-058B','11S510','11S510',68566,'Reentered','2008-11-23 00:00:00','2008-11-16 00:00:00',89.04,166,289,67.18,'Intermediate' from dual union all
select 33441,'033441','2008-059A','Endeavour (STS-126)','OV-105',68567,'Landed','2008-11-30 00:00:00','2008-11-28 00:00:00',91.6,349,358,51.64,'Intermediate' from dual union all
select 33442,'033442','1998-067BL','Crew Lock bag','STS-126 EVA-1 debris',66503,'Reentered','2009-08-03 00:00:00','2008-11-21 00:00:00',91.54,348,353,51.64,'Intermediate' from dual union all
select 33443,'033443','2008-060A','Progress M-01M','Progress 7K-TGM No.401',68568,'Deorbited','2009-02-08 00:00:00','2008-11-28 00:00:00',89.75,254,271,51.64,'Intermediate' from dual union all
select 33444,'033444','2008-060B','11S510','11S510',68568,'Reentered','2008-11-29 00:00:00','2008-11-27 00:00:00',88.27,175,202,51.62,'Intermediate' from dual union all
select 33445,'033445','2008-059B','Picosat Solar Cell Testbed','PSSC',68567,'Reentered','2010-02-17 00:00:00','2008-11-30 00:00:00',91.42,342,347,51.64,'Intermediate' from dual union all
select 33446,'033446','2008-061A','Yaogan Weixing 4','Jianbing-6 02 xing',68569,'In Earth orbit','','2008-12-02 00:00:00',97.56,633,651,97.92,'Sun-Synch' from dual union all
select 33447,'033447','2008-062A','Kosmos-2446','Oko US-KS No. 6070',68570,'Reentered','2017-05-06 00:00:00','2008-12-03 00:00:00',704.79,522,39190,62.86,'Molniya' from dual union all
select 33448,'033448','2008-062B','BOZ','BOZ',68570,'Reentered','2009-01-27 00:00:00','2008-12-03 00:00:00',91.84,195,535,62.84,'Intermediate' from dual union all
select 33449,'033449','2008-062C','11S510','11S510',68570,'Reentered','2009-04-06 00:00:00','2008-12-03 00:00:00',91.82,229,499,62.86,'Intermediate' from dual union all
select 3345,'003345','1968-069A','ESSA 7','TOS E',16796,'In Earth orbit','','1968-09-18 00:00:00',114.93,1431,1474,101.73,'Sun-Synch' from dual union all
select 33450,'033450','2008-062D','Blok-2BL','Blok-2BL No. 76713-626',68570,'In Earth orbit','','2008-12-03 00:00:00',701.85,522,39044,62.83,'Molniya' from dual union all
select 33451,'033451','2008-062E','deb 11S510','',68570,'Reentered','2009-02-07 00:00:00','2008-12-03 00:00:00',92.02,212,536,62.84,'Intermediate' from dual union all
select 33452,'033452','2008-053E','deb SJ-6/3','',68561,'Reentered','2011-06-05 00:00:00','2008-12-04 00:00:00',96.39,570,601,97.75,'Sun-Synch' from dual union all
select 33453,'033453','2008-063A','Ciel 2','Ciel 2',68571,'In Earth orbit','','2009-06-30 00:00:00',1436.06,35774,35798,.01,'Stationary' from dual union all
select 33454,'033454','2008-063B','Briz-M DTB','Briz-M torus tank',68571,'In Earth orbit','','2008-12-14 00:00:00',301.7,331,16714,49.49,'Highly Ellip' from dual union all
select 33455,'033455','2008-063C','Briz-M No. 99503','Briz-M 99503',68571,'In Earth orbit','','2008-12-13 00:00:00',727.94,5319,35536,19.43,'GEO Transfer' from dual union all
select 33456,'033456','2008-064A','Yaogan Weixing 5','Jianbing-10 01 xing',68572,'Reentered','2014-09-02 00:00:00','2008-12-17 00:00:00',94.32,480,492,97.4,'Sun-Synch' from dual union all
select 33457,'033457','2008-064B','CZ-4C Stage 3','CZ-4C Stage 3',68572,'Reentered','2011-12-06 00:00:00','2008-12-17 00:00:00',93.03,359,486,97.28,'Sun-Synch' from dual union all
select 33458,'033458','2006-053D','FY-2D imager cover','',68225,'In Earth orbit','','2008-12-14 00:00:00',1436.01,35581,35989,.75,'Stationary' from dual union all
select 33459,'033459','2008-065A','Hot Bird 9','Hot Bird 9',68573,'In Earth orbit','','2009-07-02 00:00:00',1436.04,35763,35808,.06,'Stationary' from dual union all
select 3346,'003346','1968-069B','Delta 58','DSV-3E-3 S/N 20227',16796,'In Earth orbit','','1968-09-16 00:00:00',114.82,1424,1471,101.72,'Sun-Synch' from dual union all
select 33460,'033460','2008-065B','Eutelsat W2M','Eutelsat W2M',68573,'In Earth orbit','','2009-07-01 00:00:00',1436.06,35769,35802,.03,'Stationary' from dual union all
select 33461,'033461','2008-065C','ESC-A 543','Ariane 5 ESC-A',68573,'In Earth orbit','','2008-12-23 00:00:00',631.33,266,35734,3.99,'GEO Transfer' from dual union all
select 33462,'033462','2008-065D','Sylda 5A','Sylda 5',68573,'In Earth orbit','','2008-12-22 00:00:00',632.63,265,35802,3.99,'GEO Transfer' from dual union all
select 33463,'033463','2008-066A','Feng Yun 2E','fengyun erhao 06 xing',68574,'In Earth orbit','','2009-07-03 00:00:00',1435.55,35773,35779,2.2,'Stationary' from dual union all
select 33464,'033464','2008-066B','CZ-3A Stage 3','CZ-3A Stage 3',68574,'In Earth orbit','','2008-12-23 00:00:00',1406.48,34967,35443,2.65,'Drift GEO' from dual union all
select 33465,'033465','2008-066C','FG-36 AKM','Shanxi AKM',68574,'In Earth orbit','','2008-12-24 00:00:00',1410.49,35119,35449,2.7,'Drift GEO' from dual union all
select 33466,'033466','2008-067A','Kosmos-2447','Uragan-M No. 727',68575,'In Earth orbit','','2008-12-25 00:00:00',676.41,19131,19163,64.82,'Medium' from dual union all
select 33467,'033467','2008-067B','Kosmos-2449','Uragan-M No. 729',68575,'In Earth orbit','','2013-01-10 00:00:00',675.71,19122,19137,64.09,'Medium' from dual union all
select 33468,'033468','2008-067C','Kosmos-2448','Uragan-M No. 728',68575,'In Earth orbit','','2013-01-10 00:00:00',675.71,19117,19142,64.1,'Medium' from dual union all
select 33469,'033469','2008-067D','8S812','8S812',68575,'Reentered','2008-12-26 00:00:00','2008-12-25 00:00:00',87.78,155,174,64.82,'Intermediate' from dual union all
select 3347,'003347','1968-070A','Kosmos-236','Strela-2 No. 4',16797,'Reentered','1990-03-04 00:00:00','1968-10-03 00:00:00',96.86,591,626,56.07,'Intermediate' from dual union all
select 33470,'033470','2008-067E','Perekhodnik','Sredniy perekhodnik',68575,'Reentered','2008-12-25 00:00:00','2008-12-25 00:00:00',87.52,147,157,64.81,'Intermediate' from dual union all
select 33471,'033471','2008-067F','Blok DM-2 No. 114L','11S861 No. 114L',68575,'In Earth orbit','','2009-01-03 00:00:00',675.24,19101,19134,64.8,'Medium' from dual union all
select 33472,'033472','2008-067G','SOZ','SOZ',68575,'In Earth orbit','','2008-12-26 00:00:00',348.55,465,19591,64.96,'Highly Ellip' from dual union all
select 33473,'033473','2008-067H','SOZ','SOZ',68575,'In Earth orbit','','2009-01-02 00:00:00',340.31,399,19138,64.73,'Highly Ellip' from dual union all
select 33474,'033474','2006-026VV','deb Kosmos-2421','',68198,'Reentered','2010-03-30 00:00:00','2009-01-06 00:00:00',92.39,385,399,65.04,'Intermediate' from dual union all
select 33475,'033475','2006-026VW','deb Kosmos-2421','',68198,'Reentered','2009-07-19 00:00:00','2009-01-07 00:00:00',92.25,384,385,65,'Intermediate' from dual union all
select 33476,'033476','2006-026VX','deb Kosmos-2421','',68198,'Reentered','2009-02-27 00:00:00','2009-01-07 00:00:00',91.34,336,345,65.03,'Intermediate' from dual union all
select 33477,'033477','2006-026VY','deb Kosmos-2421','',68198,'Reentered','2010-03-16 00:00:00','2009-01-06 00:00:00',92.51,388,408,65.02,'Intermediate' from dual union all
select 33478,'033478','2006-026VZ','deb Kosmos-2421','',68198,'Reentered','2010-04-02 00:00:00','2009-01-06 00:00:00',92.43,377,411,65.04,'Intermediate' from dual union all
select 33479,'033479','2006-026WA','deb Kosmos-2421','',68198,'Reentered','2009-10-14 00:00:00','2009-01-07 00:00:00',94.33,382,591,65.42,'Intermediate' from dual union all
select 3348,'003348','1968-071A','Kosmos-237','Zenit-4 No. 43',16798,'Landed','1968-09-04 00:00:00','1968-08-29 00:00:00',89.68,197,321,65.42,'Intermediate' from dual union all
select 33480,'033480','2006-026WB','deb Kosmos-2421','',68198,'Reentered','2009-03-14 00:00:00','2009-01-06 00:00:00',91.51,340,358,65.04,'Intermediate' from dual union all
select 33481,'033481','2006-026WC','deb Kosmos-2421','',68198,'Reentered','2009-01-23 00:00:00','2009-01-07 00:00:00',90.31,283,297,65.08,'Intermediate' from dual union all
select 33482,'033482','2006-026WD','deb Kosmos-2421','',68198,'Reentered','2009-01-19 00:00:00','2009-01-07 00:00:00',90.45,289,305,65.03,'Intermediate' from dual union all
select 33483,'033483','2006-026WE','deb Kosmos-2421','',68198,'Reentered','2009-01-12 00:00:00','2009-01-07 00:00:00',90.11,272,288,64.99,'Intermediate' from dual union all
select 33484,'033484','2008-053F','CZ-4B sep motor cover','',68561,'Reentered','2013-03-23 00:00:00','2009-01-14 00:00:00',96.11,477,668,97.66,'Sun-Synch' from dual union all
select 33485,'033485','1983-044FX','deb Kosmos-1461','',55801,'Reentered','2013-01-03 00:00:00','2009-01-13 00:00:00',97.11,536,705,65,'Intermediate' from dual union all
select 33486,'033486','1983-044FY','deb Kosmos-1461','',55801,'Reentered','2011-10-21 00:00:00','2009-01-13 00:00:00',97.09,534,705,65.01,'Intermediate' from dual union all
select 33487,'033487','1983-044FZ','deb Kosmos-1461','',55801,'Reentered','2013-01-13 00:00:00','2009-01-14 00:00:00',97.16,536,710,65.01,'Intermediate' from dual union all
select 33488,'033488','1983-044GA','deb Kosmos-1461','',55801,'Reentered','2011-09-30 00:00:00','2009-01-13 00:00:00',97.12,536,706,65.02,'Intermediate' from dual;
insert into SATELLITE
select 33489,'033489','1983-044GB','deb Kosmos-1461','',55801,'Reentered','2011-05-29 00:00:00','2009-01-14 00:00:00',97.15,535,710,65.01,'Intermediate' from dual union all
select 3349,'003349','1968-070B','S3','S3',16797,'Reentered','1982-10-08 00:00:00','1968-10-01 00:00:00',96.71,575,628,56.07,'Intermediate' from dual union all
select 33490,'033490','2009-001A','USA 202','ORION 6',68682,'In Earth orbit','','2009-02-01 00:00:00',1436.01,35730,35840,0,'Stationary' from dual union all
select 33491,'033491','2009-001B','Delta 337','DCSS-5 F03',68682,'In Earth orbit','','2009-01-25 00:00:00',1499.12,35939,38084,2.98,'Drift GEO' from dual union all
select 33492,'033492','2009-002A','Ibuki','GOSAT',68683,'In Earth orbit','','2009-01-25 00:00:00',98.14,668,672,98.03,'Sun-Synch' from dual union all
select 33493,'033493','2009-002B','Hitomi','PRISM',68683,'In Earth orbit','','2009-01-25 00:00:00',97.24,589,665,98.05,'Sun-Synch' from dual union all
select 33494,'033494','2009-002C','Raijin','SpriteSat',68683,'In Earth orbit','','2009-01-25 00:00:00',97.98,657,668,98.02,'Sun-Synch' from dual union all
select 33495,'033495','2009-002D','Kagayaki','SORUNSAT-1',68683,'In Earth orbit','','2009-01-25 00:00:00',98.11,668,669,98.03,'Sun-Synch' from dual union all
select 33496,'033496','2009-002E','Maido-1','SOHLA 1',68683,'In Earth orbit','','2009-01-25 00:00:00',97.93,653,667,98.03,'Sun-Synch' from dual union all
select 33497,'033497','2009-002F','SDS-1','Small Demonstration Sat',68683,'In Earth orbit','','2009-01-25 00:00:00',98.04,662,668,98.03,'Sun-Synch' from dual union all
select 33498,'033498','2009-002G','Kukai','STARS-1 Mother',68683,'In Earth orbit','','2009-01-25 00:00:00',97.86,647,666,98.03,'Sun-Synch' from dual union all
select 33499,'033499','2009-002H','Kiseki','KKS-1',68683,'In Earth orbit','','2009-01-25 00:00:00',97.89,650,666,98.03,'Sun-Synch' from dual union all
select 335,'000335','61 Omicron 108','deb r/b Ablestar','',7273,'In Earth orbit','','1991-04-08 00:00:00',102.74,855,922,66.83,'Intermediate' from dual union all
select 3350,'003350','1968-071B','11A57I','11A57I',16798,'Reentered','1968-09-04 00:00:00','1968-08-29 00:00:00',89.42,183,309,65.42,'Intermediate' from dual union all
select 33500,'033500','2009-002J','H-2A F15 Stage 2','H-2A Stage 2',68683,'In Earth orbit','','2009-01-25 00:00:00',97.24,613,641,98.05,'Sun-Synch' from dual union all
select 33501,'033501','2009-002K','deb H-2A','',68683,'Reentered','2010-10-25 00:00:00','2009-01-25 00:00:00',97.17,605,641,98.04,'Sun-Synch' from dual union all
select 33502,'033502','1999-025DCF','deb FY-1C','',66706,'In Earth orbit','','2009-01-26 00:00:00',103.63,841,1020,98.93,'Sun-Synch' from dual union all
select 33503,'033503','1981-053MT','deb Kosmos-1275','',52576,'In Earth orbit','','2009-01-27 00:00:00',100.23,752,787,82.96,'Intermediate' from dual union all
select 33504,'033504','2009-003A','Koronas-Foton','Koronas-Foton',68684,'In Earth orbit','','2009-02-01 00:00:00',95.65,539,561,82.48,'Intermediate' from dual union all
select 33505,'033505','2009-003B','S5M','S5M',68684,'In Earth orbit','','2009-02-01 00:00:00',95.66,539,562,82.48,'Intermediate' from dual union all
select 33506,'033506','2009-004A','Omid','',68685,'Reentered','2009-04-25 00:00:00','2009-02-04 00:00:00',90.74,245,377,55.51,'Intermediate' from dual union all
select 33507,'033507','2009-004B','Safir Stage 2','',68685,'Reentered','2009-05-31 00:00:00','2009-02-04 00:00:00',91.36,245,438,55.55,'Intermediate' from dual union all
select 33508,'033508','1967-048G','deb NNS O-13 insulation','',15303,'In Earth orbit','','2009-02-04 00:00:00',106.56,1048,1087,89.64,'Polar' from dual union all
select 33509,'033509','1968-081K','Transtage 5 debris','',16808,'In Earth orbit','','2009-02-04 00:00:00',1411.23,35021,35576,9.64,'Inclined Drift' from dual union all
select 3351,'003351','1968-072A','Kosmos-238','Soyuz No. 9',16799,'Landed','1968-09-01 00:00:00','1968-08-30 00:00:00',89.09,206,253,51.67,'Intermediate' from dual union all
select 33510,'033510','1968-081L','Transtage 5 debris','',16808,'In Earth orbit','','2009-01-30 00:00:00',1414.43,35040,35683,9.52,'Inclined Drift' from dual union all
select 33511,'033511','1968-081M','Transtage 5 debris','',16808,'In Earth orbit','','2009-01-22 00:00:00',1428.94,34990,36303,9.9,'Inclined Drift' from dual union all
select 33512,'033512','1968-081N','Transtage 5 debris','',16808,'In Earth orbit','','2009-01-25 00:00:00',1426.59,34508,36693,9.66,'Inclined Drift' from dual union all
select 33513,'033513','1968-081P','Transtage 5 debris','',16808,'In Earth orbit','','2009-02-03 00:00:00',1420.13,35093,35854,9.54,'Inclined Drift' from dual union all
select 33514,'033514','1971-024E','deb Isis 2','',24842,'In Earth orbit','','2009-01-15 00:00:00',111.86,1255,1370,88.18,'Polar' from dual union all
select 33515,'033515','1971-024F','deb Isis 2','',24842,'In Earth orbit','','2009-01-23 00:00:00',112.26,1300,1361,88.15,'Polar' from dual union all
select 33516,'033516','1971-024G','deb Isis 2','',24842,'In Earth orbit','','2009-01-17 00:00:00',112.06,1279,1363,88.16,'Polar' from dual union all
select 33517,'033517','1971-024H','deb Isis 2','',24842,'In Earth orbit','','2009-01-21 00:00:00',112.51,1317,1367,88.17,'Polar' from dual union all
select 33518,'033518','1971-024J','deb Isis 2','',24842,'In Earth orbit','','2009-01-19 00:00:00',112.18,1302,1351,88.19,'Polar' from dual union all
select 33519,'033519','1977-092L','Ekran 2 debris','',42220,'In Earth orbit','','2009-01-31 00:00:00',1437.9,35256,36388,14.27,'Inclined Drift' from dual union all
select 3352,'003352','1968-072B','11S510','11S510',16799,'Reentered','1968-08-29 00:00:00','1968-08-28 00:00:00',88.39,162,227,51.81,'Intermediate' from dual union all
select 33520,'033520','1981-031M','deb Kosmos-1261','',52554,'In Earth orbit','','2009-02-04 00:00:00',716.97,2957,37358,65.18,'GEO Transfer' from dual union all
select 33521,'033521','1981-049D','Star 27 S/N 0041','Star 27 TE-M-616-5',52572,'In Earth orbit','','2009-02-04 00:00:00',1750.28,33801,49662,22.07,'Very High' from dual union all
select 33522,'033522','1981-071X','deb Kosmos-1285','',52594,'In Earth orbit','','2009-02-03 00:00:00',727.04,2949,37862,65.15,'GEO Transfer' from dual union all
select 33523,'033523','1981-071Y','deb Kosmos-1285','',52594,'In Earth orbit','','2009-01-26 00:00:00',727.15,3012,37804,65.3,'GEO Transfer' from dual union all
select 33524,'033524','1981-071Z','deb Kosmos-1285','',52594,'In Earth orbit','','2009-02-02 00:00:00',726.45,2978,37804,65.37,'GEO Transfer' from dual union all
select 33525,'033525','1981-071AA','deb Kosmos-1285','',52594,'In Earth orbit','','2009-02-03 00:00:00',728.57,3548,37338,66.98,'GEO Transfer' from dual union all
select 33526,'033526','1981-071AB','deb Kosmos-1285','',52594,'In Earth orbit','','2009-01-30 00:00:00',727.21,3684,37135,67.07,'GEO Transfer' from dual union all
select 33527,'033527','1981-071AC','deb Kosmos-1285','',52594,'In Earth orbit','','2009-02-02 00:00:00',728.14,2870,37995,65.21,'GEO Transfer' from dual union all
select 33528,'033528','1981-071AD','deb Kosmos-1285','',52594,'In Earth orbit','','2009-01-27 00:00:00',725.42,3965,36766,66.61,'GEO Transfer' from dual union all
select 33529,'033529','1982-016E','deb Kosmos-1341','',54325,'In Earth orbit','','2008-06-12 00:00:00',701.68,3767,35791,66.62,'GEO Transfer' from dual union all
select 3353,'003353','1968-073A','Kosmos-239','Zenit-4 No. 48',16800,'Landed','1968-09-13 00:00:00','1968-09-07 00:00:00',89.22,200,271,51.8,'Intermediate' from dual union all
select 33530,'033530','1982-016F','deb Kosmos-1341','',54325,'In Earth orbit','','2008-05-24 00:00:00',722.83,3766,36838,65.95,'GEO Transfer' from dual union all
select 33531,'033531','1983-070K','deb Kosmos-1481','',55827,'In Earth orbit','','2008-03-21 00:00:00',707.85,3280,36584,69.27,'GEO Transfer' from dual union all
select 33532,'033532','1987-079BM','deb SOZ','',60169,'In Earth orbit','','2009-01-31 00:00:00',321.12,387,17924,64.56,'Highly Ellip' from dual union all
select 33533,'033533','1988-040G','deb Ariane V23','',61289,'In Earth orbit','','2009-02-01 00:00:00',629.93,553,35375,7.9,'GEO Transfer' from dual union all
select 33534,'033534','1988-040H','deb Ariane V23','',61289,'In Earth orbit','','2009-02-03 00:00:00',629.82,496,35426,7.83,'GEO Transfer' from dual union all
select 33535,'033535','1988-040J','deb Ariane V23','',61289,'In Earth orbit','','2009-01-28 00:00:00',625.36,537,35155,8.03,'GEO Transfer' from dual union all
select 33536,'033536','1988-081L','deb Ariane V25','deb H10',61330,'In Earth orbit','','2009-02-04 00:00:00',668.28,1305,36579,6.56,'GEO Transfer' from dual union all
select 33537,'033537','1988-081M','deb Ariane V25','deb H10',61330,'In Earth orbit','','2009-02-03 00:00:00',664.08,1207,36465,7.25,'GEO Transfer' from dual union all
select 33538,'033538','1988-081N','deb Ariane V25','deb H10',61330,'In Earth orbit','','2009-02-01 00:00:00',645.95,534,36216,6.9,'GEO Transfer' from dual union all
select 33539,'033539','1988-081P','deb Ariane V25','deb H10',61330,'In Earth orbit','','2009-01-31 00:00:00',658.66,825,36572,6.94,'GEO Transfer' from dual union all
select 3354,'003354','1968-073B','11A57I','11A57I',16800,'Reentered','1968-09-09 00:00:00','1968-09-07 00:00:00',88.85,180,256,51.8,'Intermediate' from dual union all
select 33540,'033540','1988-081Q','deb Ariane V25','deb H10',61330,'In Earth orbit','','2007-11-11 00:00:00',667.03,1433,36388,6.43,'GEO Transfer' from dual union all
select 33541,'033541','1988-109Q','deb Ariane V27','',61358,'In Earth orbit','','2009-02-02 00:00:00',643.41,791,35829,7.19,'GEO Transfer' from dual union all
select 33542,'033542','1988-109R','deb Ariane V27','',61358,'In Earth orbit','','2009-02-04 00:00:00',642.69,705,35878,7.61,'GEO Transfer' from dual union all
select 33543,'033543','1988-109S','deb Ariane V27','',61358,'In Earth orbit','','2009-02-04 00:00:00',639.9,536,35904,7.27,'GEO Transfer' from dual union all
select 33544,'033544','1988-109T','deb Ariane V27','',61358,'In Earth orbit','','2009-02-01 00:00:00',618.5,368,34969,6.39,'GEO Transfer' from dual union all
select 33545,'033545','1988-109U','deb Ariane V27','',61358,'In Earth orbit','','2009-02-02 00:00:00',614.34,413,34708,6.39,'GEO Transfer' from dual union all
select 33546,'033546','1988-109V','deb Ariane V27','',61358,'In Earth orbit','','2009-02-01 00:00:00',334.98,550,18649,64.97,'Highly Ellip' from dual union all
select 33547,'033547','1988-109W','deb Ariane V27','',61358,'In Earth orbit','','2009-02-01 00:00:00',614.84,563,34584,6.93,'GEO Transfer' from dual union all
select 33548,'033548','1989-006AQ','deb Ariane V28','',62283,'In Earth orbit','','2009-01-31 00:00:00',634.23,502,35647,8.45,'GEO Transfer' from dual union all
select 33549,'033549','1989-006AR','deb Ariane V28','',62283,'In Earth orbit','','2009-02-04 00:00:00',640.68,805,35675,7.22,'GEO Transfer' from dual union all
select 3355,'003355','1965-082LS','deb Transtage 4','',11832,'Reentered','1992-11-03 00:00:00','1992-02-03 00:00:00',95.28,511,553,31.67,'Intermediate' from dual union all
select 33550,'033550','1989-006AS','deb Ariane V28','',62283,'In Earth orbit','','2008-07-05 00:00:00',646.44,986,35789,8.1,'GEO Transfer' from dual union all
select 33551,'033551','1989-006AT','deb Ariane V28','',62283,'In Earth orbit','','2009-01-29 00:00:00',651.95,1409,35647,9.06,'GEO Transfer' from dual union all
select 33552,'033552','1989-006AU','deb Ariane V28','',62283,'Reentered','2012-09-10 00:00:00','2009-02-04 00:00:00',348.8,477,19595,7.99,'Highly Ellip' from dual union all
select 33553,'033553','1989-006AV','deb Ariane V28','',62283,'In Earth orbit','','2008-12-28 00:00:00',635.38,539,35669,8.71,'GEO Transfer' from dual union all
select 33554,'033554','1989-006AW','deb Ariane V28','',62283,'In Earth orbit','','2009-02-04 00:00:00',620.74,456,34997,8.65,'GEO Transfer' from dual union all
select 33555,'033555','1989-006AX','deb Ariane V28','',62283,'In Earth orbit','','2007-11-28 00:00:00',643.39,771,35848,8.11,'GEO Transfer' from dual union all
select 33556,'033556','1989-006AY','deb Ariane V28','',62283,'In Earth orbit','','2009-01-31 00:00:00',630.81,558,35415,9.89,'GEO Transfer' from dual union all
select 33557,'033557','1989-006AZ','deb Ariane V28','',62283,'In Earth orbit','','2009-02-04 00:00:00',646.31,1176,35591,8.87,'GEO Transfer' from dual union all
select 33558,'033558','1989-006BA','deb Ariane V28','',62283,'In Earth orbit','','2009-02-03 00:00:00',648.19,1166,35698,9.55,'GEO Transfer' from dual union all
select 33559,'033559','1989-006BB','deb Ariane V28','',62283,'In Earth orbit','','2008-01-09 00:00:00',642.24,785,35775,7.98,'GEO Transfer' from dual union all
select 3356,'003356','1965-082LT','deb Transtage 4','',11832,'Reentered','1979-03-07 00:00:00','1968-09-17 00:00:00',100.44,709,850,32.33,'Intermediate' from dual union all
select 33560,'033560','1989-006BC','deb Ariane V28','',62283,'In Earth orbit','','2009-02-03 00:00:00',640.05,823,35625,10.26,'GEO Transfer' from dual union all
select 33561,'033561','1989-006BD','deb Ariane V28','',62283,'In Earth orbit','','2009-02-03 00:00:00',648.19,1128,35735,8.21,'GEO Transfer' from dual union all
select 33562,'033562','1989-006BE','deb Ariane V28','',62283,'In Earth orbit','','2009-02-04 00:00:00',644.8,1007,35684,7.94,'GEO Transfer' from dual union all
select 33563,'033563','1989-006BF','deb Ariane V28','',62283,'In Earth orbit','','2009-02-01 00:00:00',643.69,852,35782,7.85,'GEO Transfer' from dual union all
select 33564,'033564','1989-006BG','deb Ariane V28','',62283,'In Earth orbit','','2007-06-04 00:00:00',636.15,603,35645,8.39,'GEO Transfer' from dual union all
select 33565,'033565','1989-006BH','deb Ariane V28','',62283,'In Earth orbit','','2007-01-07 00:00:00',638.39,690,35673,9.2,'GEO Transfer' from dual union all
select 33566,'033566','1989-006BJ','deb Ariane V28','',62283,'In Earth orbit','','2009-02-04 00:00:00',634.32,584,35570,8.52,'GEO Transfer' from dual union all
select 33567,'033567','1989-006BK','deb Ariane V28','',62283,'In Earth orbit','','2009-02-01 00:00:00',636.56,667,35602,8.34,'GEO Transfer' from dual union all
select 33568,'033568','1989-006BL','deb Ariane V28','',62283,'In Earth orbit','','2009-02-02 00:00:00',630.77,438,35533,8.45,'GEO Transfer' from dual union all
select 33569,'033569','1989-006BM','deb Ariane V28','',62283,'In Earth orbit','','2008-01-12 00:00:00',635.51,696,35519,8.52,'GEO Transfer' from dual union all
select 3357,'003357','1965-082LU','deb Transtage 4','',11832,'Reentered','1984-04-09 00:00:00','1968-09-20 00:00:00',97.87,591,723,32.39,'Intermediate' from dual union all
select 33570,'033570','1989-006BN','deb Ariane V28','',62283,'In Earth orbit','','2009-02-02 00:00:00',635.84,601,35631,8.6,'GEO Transfer' from dual union all
select 33571,'033571','1989-006BP','deb Ariane V28','',62283,'In Earth orbit','','2009-01-25 00:00:00',648.68,1166,35723,11.21,'GEO Transfer' from dual union all
select 33572,'033572','1989-006BQ','deb Ariane V28','',62283,'In Earth orbit','','2007-05-22 00:00:00',644.8,1045,35646,7.95,'GEO Transfer' from dual union all
select 33573,'033573','1989-006BR','deb Ariane V28','',62283,'In Earth orbit','','2008-05-11 00:00:00',644.19,887,35773,11.41,'GEO Transfer' from dual union all
select 33574,'033574','1989-006BS','deb Ariane V28','',62283,'In Earth orbit','','2009-02-04 00:00:00',633.66,557,35563,8.61,'GEO Transfer' from dual union all
select 33575,'033575','1989-006BT','deb Ariane V28','',62283,'In Earth orbit','','2007-02-11 00:00:00',642.55,825,35751,9.43,'GEO Transfer' from dual union all
select 33576,'033576','1989-006BU','deb Ariane V28','',62283,'In Earth orbit','','2007-05-06 00:00:00',644.78,961,35729,8.65,'GEO Transfer' from dual union all
select 33577,'033577','1989-006BV','deb Ariane V28','',62283,'In Earth orbit','','2008-09-18 00:00:00',630.63,598,35366,8.43,'GEO Transfer' from dual union all
select 33578,'033578','1989-006BW','deb Ariane V28','',62283,'In Earth orbit','','2009-02-04 00:00:00',630.26,474,35471,8.42,'GEO Transfer' from dual;
insert into SATELLITE
select 33579,'033579','1989-006BX','deb Ariane V28','',62283,'In Earth orbit','','2009-02-04 00:00:00',641.11,934,35568,10.54,'GEO Transfer' from dual union all
select 3358,'003358','1965-082LV','deb Transtage 4','',11832,'Reentered','1982-03-27 00:00:00','1968-09-24 00:00:00',101.2,730,900,31.72,'Intermediate' from dual union all
select 33580,'033580','1989-006BY','deb Ariane V28','',62283,'In Earth orbit','','2009-01-30 00:00:00',634.07,548,35593,9.22,'GEO Transfer' from dual union all
select 33581,'033581','1989-006BZ','deb Ariane V28','',62283,'In Earth orbit','','2008-06-06 00:00:00',632.13,586,35455,8.05,'GEO Transfer' from dual union all
select 33582,'033582','1989-006CA','deb Ariane V28','',62283,'In Earth orbit','','2009-02-03 00:00:00',707.71,4248,35609,8.16,'GEO Transfer' from dual union all
select 33583,'033583','1989-006CB','deb Ariane V28','',62283,'Lost','','2008-06-04 00:00:00',617.13,545,34721,8.41,'GEO Transfer' from dual union all
select 33584,'033584','1989-027E','deb Ariane V30','',62304,'In Earth orbit','','2009-02-04 00:00:00',400.21,413,22810,6.13,'Highly Ellip' from dual union all
select 33585,'033585','1989-039ED','deb SOZ','',62316,'Lost','','2009-02-03 00:00:00',329.62,395,18461,64.76,'Highly Ellip' from dual union all
select 33586,'033586','1989-039EE','deb SOZ','',62316,'In Earth orbit','','2009-02-01 00:00:00',334.98,550,18649,64.97,'Highly Ellip' from dual union all
select 33587,'033587','1989-039EF','deb SOZ','',62316,'Reentered','2011-07-01 00:00:00','2009-02-03 00:00:00',325.34,285,18297,65.22,'Highly Ellip' from dual union all
select 33588,'033588','1991-003R','deb Ariane V41','',64172,'In Earth orbit','','2009-02-04 00:00:00',541.65,569,30697,7.29,'Highly Ellip' from dual union all
select 33589,'033589','1991-003S','deb Ariane V41','',64172,'Lost','','2009-02-02 00:00:00',546.72,787,30753,6.9,'Highly Ellip' from dual union all
select 3359,'003359','1965-082LW','deb Transtage 4','',11832,'Reentered','1990-09-26 00:00:00','1990-09-26 00:00:00',89.04,221,234,32.39,'Intermediate' from dual union all
select 33590,'033590','1991-003T','deb Ariane V41','',64172,'In Earth orbit','','2009-02-01 00:00:00',540.95,645,30583,7.25,'Highly Ellip' from dual union all
select 33591,'033591','2009-005A','NOAA 19','NOAA N''',68686,'In Earth orbit','','2009-02-07 00:00:00',102.04,846,865,98.73,'Sun-Synch' from dual union all
select 33592,'033592','2009-005B','Delta 338','Delta SSPS AJ-10-118K',68686,'Reentered','2009-04-21 00:00:00','2009-02-07 00:00:00',94.56,177,818,102.02,'Retrograde' from dual union all
select 33593,'033593','2009-006A','Progress M-66','Progress 7K-TGM No.366',68687,'Deorbited','2009-05-18 00:00:00','2009-02-11 00:00:00',89.74,243,281,51.64,'Intermediate' from dual union all
select 33594,'033594','2009-006B','11S510','11S510',68687,'Reentered','2009-02-13 00:00:00','2009-02-11 00:00:00',88.34,178,206,51.62,'Intermediate' from dual union all
select 33595,'033595','2009-007A','Ekspress AM-44','Ekspress AM-44',68688,'In Earth orbit','','2009-02-11 00:00:00',1477.28,35657,37521,.22,'Drift GEO' from dual union all
select 33596,'033596','2009-007B','Ekspress MD-1','Ekspress MD-1',68688,'In Earth orbit','','2009-02-11 00:00:00',1422.09,35296,35728,.23,'Drift GEO' from dual union all
select 33597,'033597','2009-007C','Briz-M DTB','Briz-M torus tank',68688,'In Earth orbit','','2009-02-11 00:00:00',633.02,405,35682,44.68,'GEO Transfer' from dual union all
select 33598,'033598','2009-007D','Briz-M No. 99501','Briz-M 14S43 No. 99501',68688,'In Earth orbit','','2009-02-13 00:00:00',1474.99,35607,37482,.2,'Drift GEO' from dual union all
select 33599,'033599','1999-025DCG','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',100.68,512,1069,96.62,'Sun-Synch' from dual union all
select 336,'000336','61 Omicron 109','deb r/b Ablestar','',7273,'In Earth orbit','','1991-04-10 00:00:00',105.45,953,1078,67.02,'Intermediate' from dual union all
select 3360,'003360','1965-082LX','deb Transtage 4','',11832,'Reentered','1972-06-11 00:00:00','1968-09-03 00:00:00',99.08,677,753,31.59,'Intermediate' from dual union all
select 33600,'033600','1999-025DCH','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',101.62,510,1161,96.31,'Sun-Synch' from dual union all
select 33601,'033601','1999-025DCJ','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',103.45,786,1058,98.36,'Sun-Synch' from dual union all
select 33602,'033602','1999-025DCK','deb FY-1C','',66706,'Reentered','2011-12-17 00:00:00','2009-02-08 00:00:00',99.23,493,950,97.31,'Sun-Synch' from dual union all
select 33603,'033603','1999-025DCL','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',98.31,489,867,98.84,'Sun-Synch' from dual union all
select 33604,'033604','1999-025DCM','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',98.28,516,837,99.03,'Sun-Synch' from dual union all
select 33605,'033605','1999-025DCN','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',102.8,851,932,99.24,'Sun-Synch' from dual union all
select 33606,'033606','1999-025DCP','deb FY-1C','',66706,'Reentered','2016-01-14 00:00:00','2009-02-03 00:00:00',105.79,777,1285,95.73,'Sun-Synch' from dual union all
select 33607,'033607','1999-025DCQ','deb FY-1C','',66706,'Reentered','2013-11-10 00:00:00','2009-02-07 00:00:00',99.09,446,984,97.48,'Sun-Synch' from dual union all
select 33608,'033608','1999-025DCR','deb FY-1C','',66706,'Reentered','2012-07-23 00:00:00','2009-01-30 00:00:00',99,506,916,98.36,'Sun-Synch' from dual union all
select 33609,'033609','1999-025DCS','deb FY-1C','',66706,'Reentered','2012-08-11 00:00:00','2009-02-07 00:00:00',100.48,429,1134,98.4,'Sun-Synch' from dual union all
select 3361,'003361','1965-082LY','deb Transtage 4','',11832,'Reentered','2001-10-31 00:00:00','1992-02-02 00:00:00',99.05,644,783,32.28,'Intermediate' from dual union all
select 33610,'033610','1999-025DCT','deb FY-1C','',66706,'Reentered','2014-01-09 00:00:00','2009-02-07 00:00:00',98.83,567,839,99.14,'Sun-Synch' from dual union all
select 33611,'033611','1999-025DCU','deb FY-1C','',66706,'Reentered','2011-10-28 00:00:00','2009-02-06 00:00:00',98.28,500,853,97.22,'Sun-Synch' from dual union all
select 33612,'033612','1999-025DCV','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',103.78,841,1034,98.65,'Sun-Synch' from dual union all
select 33613,'033613','1999-025DCW','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',102.53,851,906,99.04,'Sun-Synch' from dual union all
select 33614,'033614','1999-025DCX','deb FY-1C','',66706,'In Earth orbit','','2009-02-02 00:00:00',101.16,687,941,96.57,'Sun-Synch' from dual union all
select 33615,'033615','1999-025DCY','deb FY-1C','',66706,'Reentered','2012-08-13 00:00:00','2009-02-07 00:00:00',99.4,515,945,97.11,'Sun-Synch' from dual union all
select 33616,'033616','1999-025DCZ','deb FY-1C','',66706,'Reentered','2014-02-21 00:00:00','2009-02-08 00:00:00',99.95,443,1070,98.28,'Sun-Synch' from dual union all
select 33617,'033617','1999-025DDA','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',98.1,436,900,98,'Sun-Synch' from dual union all
select 33618,'033618','1999-025DDB','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',100.43,607,950,98.29,'Sun-Synch' from dual union all
select 33619,'033619','1999-025DDC','deb FY-1C','',66706,'In Earth orbit','','2009-02-02 00:00:00',102.99,854,947,99.14,'Sun-Synch' from dual union all
select 3362,'003362','1965-082LZ','deb Transtage 4','',11832,'Reentered','1981-12-03 00:00:00','1968-08-27 00:00:00',101.47,721,936,31.96,'Intermediate' from dual union all
select 33620,'033620','1999-025DDD','deb FY-1C','',66706,'Reentered','2014-10-03 00:00:00','2009-02-02 00:00:00',100.63,619,958,97.76,'Sun-Synch' from dual union all
select 33621,'033621','1999-025DDE','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',102.39,634,1110,98.23,'Sun-Synch' from dual union all
select 33622,'033622','1999-025DDF','deb FY-1C','',66706,'In Earth orbit','','2009-02-01 00:00:00',102.66,852,917,98.99,'Sun-Synch' from dual union all
select 33623,'033623','1999-025DDG','deb FY-1C','',66706,'In Earth orbit','','2009-02-09 00:00:00',103.44,853,990,98.2,'Sun-Synch' from dual union all
select 33624,'033624','1999-025DDH','deb FY-1C','',66706,'In Earth orbit','','2009-02-05 00:00:00',104.22,516,1399,98.32,'Sun-Synch' from dual union all
select 33625,'033625','1999-025DDJ','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',101.22,562,1071,98.47,'Sun-Synch' from dual union all
select 33626,'033626','1999-025DDK','deb FY-1C','',66706,'Reentered','2014-10-17 00:00:00','2009-02-05 00:00:00',100.05,581,941,98.36,'Sun-Synch' from dual union all
select 33627,'033627','1999-025DDL','deb FY-1C','',66706,'Reentered','2012-04-10 00:00:00','2009-02-07 00:00:00',99.81,470,1029,97.31,'Sun-Synch' from dual union all
select 33628,'033628','1999-025DDM','deb FY-1C','',66706,'In Earth orbit','','2009-02-06 00:00:00',99.51,555,916,98.47,'Sun-Synch' from dual union all
select 33629,'033629','1999-025DDN','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',101.11,597,1026,98.53,'Sun-Synch' from dual union all
select 3363,'003363','1965-082MA','deb Transtage 4','',11832,'Reentered','1982-04-12 00:00:00','1968-09-05 00:00:00',100.85,704,893,31.86,'Intermediate' from dual union all
select 33630,'033630','1999-025DDP','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',102.08,751,964,98.52,'Sun-Synch' from dual union all
select 33631,'033631','1999-025DDQ','deb FY-1C','',66706,'Reentered','2012-11-14 00:00:00','2009-02-08 00:00:00',98.57,538,843,99.02,'Sun-Synch' from dual union all
select 33632,'033632','1999-025DDR','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',101.04,757,859,99.09,'Sun-Synch' from dual union all
select 33633,'033633','1999-025DDS','deb FY-1C','',66706,'In Earth orbit','','2009-02-05 00:00:00',103.39,836,1002,99.08,'Sun-Synch' from dual union all
select 33634,'033634','1999-025DDT','deb FY-1C','',66706,'Reentered','2013-03-28 00:00:00','2009-02-11 00:00:00',97.41,392,878,98.81,'Sun-Synch' from dual union all
select 33635,'033635','1999-025DDU','deb FY-1C','',66706,'Reentered','2014-10-13 00:00:00','2009-02-07 00:00:00',100.8,739,854,99.14,'Sun-Synch' from dual union all
select 33636,'033636','1999-025DDV','deb FY-1C','',66706,'In Earth orbit','','2009-02-11 00:00:00',101.53,775,888,98.82,'Sun-Synch' from dual union all
select 33637,'033637','1999-025DDW','deb FY-1C','',66706,'In Earth orbit','','2009-02-05 00:00:00',101.84,767,925,98.92,'Sun-Synch' from dual union all
select 33638,'033638','1999-025DDX','deb FY-1C','',66706,'In Earth orbit','','2009-02-09 00:00:00',106.07,828,1261,98.48,'Sun-Synch' from dual union all
select 33639,'033639','1999-025DDY','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',102.15,807,913,98.98,'Sun-Synch' from dual union all
select 3364,'003364','1965-082MB','deb Transtage 4','',11832,'Reentered','1980-11-30 00:00:00','1967-03-19 00:00:00',104.22,692,1224,32.62,'Intermediate' from dual union all
select 33640,'033640','1999-025DDZ','deb FY-1C','',66706,'In Earth orbit','','2009-01-26 00:00:00',100.5,704,861,98.98,'Sun-Synch' from dual union all
select 33641,'033641','1999-025DEA','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',100.25,688,852,99.01,'Sun-Synch' from dual union all
select 33642,'033642','1999-025DEB','deb FY-1C','',66706,'In Earth orbit','','2009-02-08 00:00:00',101.64,549,1124,99.36,'Sun-Synch' from dual union all
select 33643,'033643','1999-025DEC','deb FY-1C','',66706,'In Earth orbit','','2009-02-11 00:00:00',98.87,556,854,99.1,'Sun-Synch' from dual union all
select 33644,'033644','1999-025DED','deb FY-1C','',66706,'Lost','','2009-02-09 00:00:00',98.55,521,858,99.24,'Sun-Synch' from dual union all
select 33645,'033645','1999-025DEE','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',103.81,579,1298,99.67,'Sun-Synch' from dual union all
select 33646,'033646','1999-025DEF','deb FY-1C','',66706,'In Earth orbit','','2009-02-09 00:00:00',103.97,849,1044,98.6,'Sun-Synch' from dual union all
select 33647,'033647','1999-025DEG','deb FY-1C','',66706,'In Earth orbit','','2009-02-09 00:00:00',105.3,791,1226,98.23,'Sun-Synch' from dual union all
select 33648,'033648','1999-025DEH','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',102.14,810,909,98.94,'Sun-Synch' from dual union all
select 33649,'033649','1999-025DEJ','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',100.07,674,850,98.98,'Sun-Synch' from dual union all
select 3365,'003365','1965-082MC','deb Transtage 4','',11832,'Reentered','1980-03-09 00:00:00','1979-12-20 00:00:00',96.13,559,587,31.97,'Intermediate' from dual union all
select 33650,'033650','1999-025DEK','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',104.64,854,1101,99.09,'Sun-Synch' from dual union all
select 33651,'033651','1999-025DEL','deb FY-1C','',66706,'In Earth orbit','','2009-02-06 00:00:00',101.61,746,924,98.64,'Sun-Synch' from dual union all
select 33652,'033652','1999-025DEM','deb FY-1C','',66706,'Reentered','2009-11-25 00:00:00','2009-02-11 00:00:00',94.7,389,620,98.88,'Sun-Synch' from dual union all
select 33653,'033653','1999-025DEN','deb FY-1C','',66706,'Reentered','2010-10-21 00:00:00','2009-02-05 00:00:00',95.23,354,705,99.79,'Sun-Synch' from dual union all
select 33654,'033654','1999-025DEP','deb FY-1C','',66706,'Reentered','2012-10-20 00:00:00','2009-02-11 00:00:00',98.74,536,861,100.06,'Sun-Synch' from dual union all
select 33655,'033655','1999-025DEQ','deb FY-1C','',66706,'In Earth orbit','','2009-02-08 00:00:00',100.09,667,859,98.9,'Sun-Synch' from dual union all
select 33656,'033656','1999-025DER','deb FY-1C','',66706,'In Earth orbit','','2009-02-06 00:00:00',101.7,768,911,98.93,'Sun-Synch' from dual union all
select 33657,'033657','1999-025DES','deb FY-1C','',66706,'Reentered','2015-01-21 00:00:00','2009-02-07 00:00:00',98.63,536,850,99.1,'Sun-Synch' from dual union all
select 33658,'033658','1999-025DET','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',107.25,855,1344,99.56,'Sun-Synch' from dual union all
select 33659,'033659','1999-025DEU','deb FY-1C','',66706,'Reentered','2012-03-04 00:00:00','2009-02-11 00:00:00',96.58,377,812,99.21,'Sun-Synch' from dual union all
select 3366,'003366','1965-082MD','deb Transtage 4','',11832,'Reentered','1981-04-13 00:00:00','1968-09-06 00:00:00',101.12,718,905,31.78,'Intermediate' from dual union all
select 33660,'033660','1999-025DEV','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',103.79,854,1022,98.63,'Sun-Synch' from dual union all
select 33661,'033661','1999-025DEW','deb FY-1C','',66706,'In Earth orbit','','2009-02-09 00:00:00',105.81,843,1222,98.42,'Sun-Synch' from dual union all
select 33662,'033662','1999-025DEX','deb FY-1C','',66706,'Reentered','2009-06-27 00:00:00','2009-02-10 00:00:00',92.67,291,520,99.19,'Sun-Synch' from dual union all
select 33663,'033663','1999-025DEY','deb FY-1C','',66706,'In Earth orbit','','2009-02-03 00:00:00',103.64,786,1076,99.03,'Sun-Synch' from dual union all
select 33664,'033664','1999-025DEZ','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',102.94,774,1022,98.72,'Sun-Synch' from dual union all
select 33665,'033665','1999-025DFA','deb FY-1C','',66706,'In Earth orbit','','2009-02-09 00:00:00',104.38,561,1370,98.88,'Sun-Synch' from dual union all
select 33666,'033666','1999-025DFB','deb FY-1C','',66706,'In Earth orbit','','2009-02-09 00:00:00',100.58,698,875,98.96,'Sun-Synch' from dual union all
select 33667,'033667','1999-025DFC','deb FY-1C','',66706,'Reentered','2012-07-21 00:00:00','2009-02-11 00:00:00',98.4,524,841,99.01,'Sun-Synch' from dual union all
select 33668,'033668','1999-025DFD','deb FY-1C','',66706,'In Earth orbit','','2009-01-24 00:00:00',99.23,547,897,99.04,'Sun-Synch' from dual;
insert into SATELLITE
select 33669,'033669','1999-025DFE','deb FY-1C','',66706,'Reentered','2014-04-12 00:00:00','2009-02-07 00:00:00',99.2,425,1016,98.07,'Sun-Synch' from dual union all
select 3367,'003367','1965-082ME','deb Transtage 4','',11832,'Reentered','1980-01-21 00:00:00','1967-02-10 00:00:00',101.41,733,918,32.81,'Intermediate' from dual union all
select 33670,'033670','1999-025DFF','deb FY-1C','',66706,'Reentered','2014-05-09 00:00:00','2009-02-11 00:00:00',98.78,532,869,99.09,'Sun-Synch' from dual union all
select 33671,'033671','1999-025DFG','deb FY-1C','',66706,'In Earth orbit','','2009-02-11 00:00:00',99.49,627,842,99.55,'Sun-Synch' from dual union all
select 33672,'033672','1999-025DFH','deb FY-1C','',66706,'Reentered','2012-07-10 00:00:00','2009-02-07 00:00:00',99.41,486,974,99.64,'Sun-Synch' from dual union all
select 33673,'033673','1999-025DFJ','deb FY-1C','',66706,'In Earth orbit','','2009-01-27 00:00:00',103.72,851,1018,99,'Sun-Synch' from dual union all
select 33674,'033674','1999-025DFK','deb FY-1C','',66706,'In Earth orbit','','2009-01-27 00:00:00',105.76,853,1207,98.63,'Sun-Synch' from dual union all
select 33675,'033675','1999-025DFL','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',101.12,768,856,99.21,'Sun-Synch' from dual union all
select 33676,'033676','1999-025DFM','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',100.66,634,946,99.09,'Sun-Synch' from dual union all
select 33677,'033677','1999-025DFN','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',100.41,700,856,98.94,'Sun-Synch' from dual union all
select 33678,'033678','1999-025DFP','deb FY-1C','',66706,'In Earth orbit','','2009-02-11 00:00:00',100.82,612,983,99.12,'Sun-Synch' from dual union all
select 33679,'033679','1999-025DFQ','deb FY-1C','',66706,'In Earth orbit','','2009-01-25 00:00:00',101.2,520,1111,99.4,'Sun-Synch' from dual union all
select 3368,'003368','1965-082MF','deb Transtage 4','',11832,'Reentered','1970-03-01 00:00:00','1968-09-05 00:00:00',98.5,675,699,32.1,'Intermediate' from dual union all
select 33680,'033680','1999-025DFR','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',100.61,689,886,99.45,'Sun-Synch' from dual union all
select 33681,'033681','1999-025DFS','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',103.37,837,998,99,'Sun-Synch' from dual union all
select 33682,'033682','1999-025DFT','deb FY-1C','',66706,'In Earth orbit','','2009-01-25 00:00:00',102.64,854,914,99.46,'Sun-Synch' from dual union all
select 33683,'033683','1999-025DFU','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',101.41,760,891,98.37,'Sun-Synch' from dual union all
select 33684,'033684','1999-025DFV','deb FY-1C','',66706,'In Earth orbit','','2009-02-09 00:00:00',102,480,1227,99.22,'Sun-Synch' from dual union all
select 33685,'033685','1999-025DFW','deb FY-1C','',66706,'Reentered','2012-12-22 00:00:00','2009-02-08 00:00:00',97.13,424,819,98.88,'Sun-Synch' from dual union all
select 33686,'033686','1999-025DFX','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',100.46,703,857,98.95,'Sun-Synch' from dual union all
select 33687,'033687','1999-025DFY','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',99.67,508,978,99.03,'Sun-Synch' from dual union all
select 33688,'033688','1999-025DFZ','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',102.43,809,939,99.42,'Sun-Synch' from dual union all
select 33689,'033689','1999-025DGA','deb FY-1C','',66706,'In Earth orbit','','2009-02-08 00:00:00',101.32,776,867,99.55,'Sun-Synch' from dual union all
select 3369,'003369','1965-082MG','deb Transtage 4','',11832,'Reentered','1990-02-26 00:00:00','1968-09-06 00:00:00',103.07,726,1081,31.38,'Intermediate' from dual union all
select 33690,'033690','1999-025DGB','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',101.69,800,877,98.94,'Sun-Synch' from dual union all
select 33691,'033691','1999-025DGC','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',103.39,851,987,98.88,'Sun-Synch' from dual union all
select 33692,'033692','1999-025DGD','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',101.63,806,866,99.16,'Sun-Synch' from dual union all
select 33693,'033693','1999-025DGE','deb FY-1C','',66706,'In Earth orbit','','2009-02-08 00:00:00',105.92,854,1221,99.35,'Sun-Synch' from dual union all
select 33694,'033694','1999-025DGF','deb FY-1C','',66706,'Reentered','2013-11-21 00:00:00','2009-02-10 00:00:00',96.85,389,827,99.64,'Sun-Synch' from dual union all
select 33695,'033695','1999-025DGG','deb FY-1C','',66706,'Reentered','2011-12-04 00:00:00','2009-02-10 00:00:00',97.81,494,814,98.96,'Sun-Synch' from dual union all
select 33696,'033696','1999-025DGH','deb FY-1C','',66706,'Reentered','2012-01-01 00:00:00','2009-02-11 00:00:00',96.81,362,850,99.12,'Sun-Synch' from dual union all
select 33697,'033697','1999-025DGJ','deb FY-1C','',66706,'In Earth orbit','','2009-02-09 00:00:00',103.3,853,977,99.37,'Sun-Synch' from dual union all
select 33698,'033698','1999-025DGK','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',99.64,627,855,99.59,'Sun-Synch' from dual union all
select 33699,'033699','1999-025DGL','deb FY-1C','',66706,'In Earth orbit','','2009-01-31 00:00:00',103.39,827,1011,98.97,'Sun-Synch' from dual union all
select 337,'000337','61 Omicron 110','deb r/b Ablestar','',7273,'In Earth orbit','','1991-04-13 00:00:00',105.26,930,1084,67.02,'Intermediate' from dual union all
select 3370,'003370','1965-082MH','deb Transtage 4','',11832,'Reentered','1980-03-30 00:00:00','1968-09-06 00:00:00',97.28,547,710,32.27,'Intermediate' from dual union all
select 33700,'033700','1999-025DGM','deb FY-1C','',66706,'In Earth orbit','','2009-02-03 00:00:00',99.86,617,887,99.15,'Sun-Synch' from dual union all
select 33701,'033701','1999-025DGN','deb FY-1C','',66706,'In Earth orbit','','2009-01-27 00:00:00',103.97,827,1066,98.66,'Sun-Synch' from dual union all
select 33702,'033702','1999-025DGP','deb FY-1C','',66706,'Reentered','2011-11-29 00:00:00','2009-02-10 00:00:00',97.82,377,932,96.54,'Sun-Synch' from dual union all
select 33703,'033703','1999-025DGQ','deb FY-1C','',66706,'Reentered','2014-10-26 00:00:00','2009-02-09 00:00:00',108.55,792,1528,99.3,'Sun-Synch' from dual union all
select 33704,'033704','1999-025DGR','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',98.38,524,839,99.01,'Sun-Synch' from dual union all
select 33705,'033705','1999-025DGS','deb FY-1C','',66706,'Reentered','2012-08-18 00:00:00','2009-02-11 00:00:00',98.41,527,838,99.02,'Sun-Synch' from dual union all
select 33706,'033706','1999-025DGT','deb FY-1C','',66706,'In Earth orbit','','2009-02-09 00:00:00',101.12,686,937,99.03,'Sun-Synch' from dual union all
select 33707,'033707','1999-025DGU','deb FY-1C','',66706,'In Earth orbit','','2009-02-05 00:00:00',99.8,644,854,99.09,'Sun-Synch' from dual union all
select 33708,'033708','1999-025DGV','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',100.13,676,854,99.47,'Sun-Synch' from dual union all
select 33709,'033709','1999-025DGW','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',100.31,652,895,99.09,'Sun-Synch' from dual union all
select 3371,'003371','1965-082MJ','deb Transtage 4','',11832,'Reentered','1982-08-10 00:00:00','1968-09-06 00:00:00',100.13,657,872,31.4,'Intermediate' from dual union all
select 33710,'033710','1999-025DGX','deb FY-1C','',66706,'In Earth orbit','','2009-02-01 00:00:00',103.02,855,948,99.33,'Sun-Synch' from dual union all
select 33711,'033711','1999-025DGY','deb FY-1C','',66706,'In Earth orbit','','2009-01-27 00:00:00',103.4,847,992,98.97,'Sun-Synch' from dual union all
select 33712,'033712','1999-025DGZ','deb FY-1C','',66706,'Reentered','2015-03-11 00:00:00','2009-02-11 00:00:00',99.54,628,846,99.54,'Sun-Synch' from dual union all
select 33713,'033713','1999-025DHA','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',99.7,574,915,99.09,'Sun-Synch' from dual union all
select 33714,'033714','1999-025DHB','deb FY-1C','',66706,'In Earth orbit','','2009-02-11 00:00:00',101.51,779,882,99.92,'Sun-Synch' from dual union all
select 33715,'033715','1999-025DHC','deb FY-1C','',66706,'Reentered','2014-11-13 00:00:00','2009-02-11 00:00:00',100.12,684,845,99.02,'Sun-Synch' from dual union all
select 33716,'033716','1999-025DHD','deb FY-1C','',66706,'In Earth orbit','','2009-02-11 00:00:00',100.73,724,863,99.04,'Sun-Synch' from dual union all
select 33717,'033717','1999-025DHE','deb FY-1C','',66706,'Reentered','2015-01-27 00:00:00','2009-02-09 00:00:00',100.62,714,862,98.91,'Sun-Synch' from dual union all
select 33718,'033718','1999-025DHF','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',103.69,839,1027,99.75,'Sun-Synch' from dual union all
select 33719,'033719','1999-025DHG','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',102.19,832,893,98.82,'Sun-Synch' from dual union all
select 3372,'003372','1965-082MK','deb Transtage 4','',11832,'Reentered','1978-11-01 00:00:00','1968-09-06 00:00:00',98.1,627,709,32.03,'Intermediate' from dual union all
select 33720,'033720','1999-025DHH','deb FY-1C','',66706,'In Earth orbit','','2009-02-07 00:00:00',100.82,729,866,98.85,'Sun-Synch' from dual union all
select 33721,'033721','1999-025DHJ','deb FY-1C','',66706,'In Earth orbit','','2009-02-11 00:00:00',99.5,615,855,98.87,'Sun-Synch' from dual union all
select 33722,'033722','1999-025DHK','deb FY-1C','',66706,'In Earth orbit','','2009-01-27 00:00:00',102.2,732,994,99.01,'Sun-Synch' from dual union all
select 33723,'033723','1999-025DHL','deb FY-1C','',66706,'In Earth orbit','','2009-01-30 00:00:00',103.72,841,1028,99.09,'Sun-Synch' from dual union all
select 33724,'033724','1999-025DHM','deb FY-1C','',66706,'In Earth orbit','','2009-02-08 00:00:00',101.96,829,874,98.89,'Sun-Synch' from dual union all
select 33725,'033725','1999-025DHN','deb FY-1C','',66706,'Reentered','2012-01-18 00:00:00','2009-02-09 00:00:00',97.66,476,818,99.13,'Sun-Synch' from dual union all
select 33726,'033726','1999-025DHP','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',102.33,718,1020,98.59,'Sun-Synch' from dual union all
select 33727,'033727','1999-025DHQ','deb FY-1C','',66706,'In Earth orbit','','2009-02-09 00:00:00',101.2,778,853,99.75,'Sun-Synch' from dual union all
select 33728,'033728','1999-025DHR','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',98.65,524,865,98.99,'Sun-Synch' from dual union all
select 33729,'033729','1999-025DHS','deb FY-1C','',66706,'Reentered','2014-11-01 00:00:00','2009-02-07 00:00:00',100.13,686,843,99.05,'Sun-Synch' from dual union all
select 3373,'003373','1965-082ML','deb Transtage 4','',11832,'In Earth orbit','','1992-02-02 00:00:00',103.36,699,1136,31.01,'Intermediate' from dual union all
select 33730,'033730','1999-025DHT','deb FY-1C','',66706,'In Earth orbit','','2009-02-01 00:00:00',101.1,758,864,99.13,'Sun-Synch' from dual union all
select 33731,'033731','1999-025DHU','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',104.35,855,1073,99.15,'Sun-Synch' from dual union all
select 33732,'033732','1999-025DHV','deb FY-1C','',66706,'Reentered','2013-03-19 00:00:00','2009-02-07 00:00:00',98.11,501,836,98.72,'Sun-Synch' from dual union all
select 33733,'033733','1999-025DHW','deb FY-1C','',66706,'In Earth orbit','','2009-02-11 00:00:00',103.59,836,1021,99.06,'Sun-Synch' from dual union all
select 33734,'033734','1999-025DHX','deb FY-1C','',66706,'In Earth orbit','','2009-02-10 00:00:00',102.43,768,979,99.11,'Sun-Synch' from dual union all
select 33735,'033735','1999-025DHY','deb FY-1C','',66706,'In Earth orbit','','2009-02-11 00:00:00',102.48,848,903,98.9,'Sun-Synch' from dual union all
select 33736,'033736','1999-025DHZ','deb FY-1C','',66706,'In Earth orbit','','2009-02-09 00:00:00',101.31,783,859,99.7,'Sun-Synch' from dual union all
select 33737,'033737','1999-025DJA','deb FY-1C','',66706,'In Earth orbit','','2009-02-11 00:00:00',103.59,823,1033,98.44,'Sun-Synch' from dual union all
select 33738,'033738','1999-025DJB','deb FY-1C','',66706,'In Earth orbit','','2009-01-29 00:00:00',103.56,778,1076,99.01,'Sun-Synch' from dual union all
select 33739,'033739','1999-025DJC','deb FY-1C','',66706,'In Earth orbit','','2009-01-25 00:00:00',100.84,734,863,99.03,'Sun-Synch' from dual union all
select 3374,'003374','1965-082MM','deb Transtage 4','',11832,'Reentered','1979-04-10 00:00:00','1968-09-05 00:00:00',99.09,654,777,32.48,'Intermediate' from dual union all
select 33740,'033740','1999-025DJD','deb FY-1C','',66706,'In Earth orbit','','2009-02-03 00:00:00',102.12,858,859,98.87,'Sun-Synch' from dual union all
select 33741,'033741','1999-025DJE','deb FY-1C','',66706,'In Earth orbit','','2009-02-11 00:00:00',104,830,1066,98.58,'Sun-Synch' from dual union all
select 33742,'033742','1999-025DJF','deb FY-1C','',66706,'Reentered','2012-05-11 00:00:00','2009-02-08 00:00:00',97.23,428,825,98.95,'Sun-Synch' from dual union all
select 33743,'033743','1999-025DJG','deb FY-1C','',66706,'Reentered','2013-03-27 00:00:00','2009-02-10 00:00:00',98.43,527,841,99,'Sun-Synch' from dual union all
select 33744,'033744','1999-025DJH','deb FY-1C','',66706,'Reentered','2014-10-22 00:00:00','2009-02-10 00:00:00',100.13,644,886,99.05,'Sun-Synch' from dual union all
select 33745,'033745','1999-025DJJ','deb FY-1C','',66706,'In Earth orbit','','2009-02-11 00:00:00',102.83,844,941,98.79,'Sun-Synch' from dual union all
select 33746,'033746','1999-025DJK','deb FY-1C','',66706,'Reentered','2016-03-06 00:00:00','2009-01-27 00:00:00',100.81,741,852,99.07,'Sun-Synch' from dual union all
select 33747,'033747','1999-025DJL','deb FY-1C','',66706,'In Earth orbit','','2009-02-05 00:00:00',103.49,837,1011,99.12,'Sun-Synch' from dual union all
select 33748,'033748','1999-025DJM','deb FY-1C','',66706,'In Earth orbit','','2009-02-11 00:00:00',101.91,811,887,98.84,'Sun-Synch' from dual union all
select 33749,'033749','2009-008A','NSS 9','NSS 9',68689,'In Earth orbit','','2009-02-20 00:00:00',1433.76,35703,35779,.05,'Drift GEO' from dual union all
select 3375,'003375','1968-074A','OPS 5247','GAMBIT-3 Mission 4316',16801,'Reentered Attached','1968-09-25 00:00:00','1968-09-17 00:00:00',89.81,129,402,106.02,'Retrograde' from dual union all
select 33750,'033750','2009-008B','Atlantic Bird 4A','Hot Bird 10',68689,'In Earth orbit','','2009-02-20 00:00:00',1430.45,35540,35812,.07,'Drift GEO' from dual union all
select 33751,'033751','2009-008C','Spirale A','Spirale A',68689,'In Earth orbit','','2009-02-20 00:00:00',634.21,422,35726,2,'GEO Transfer' from dual union all
select 33752,'033752','2009-008D','Spirale B','Spirale B',68689,'In Earth orbit','','2009-02-20 00:00:00',634.23,433,35716,2,'GEO Transfer' from dual union all
select 33753,'033753','2009-008E','Sylda 5A','Sylda 5 No. 29A',68689,'Reentered','2018-02-03 00:00:00','2009-02-21 00:00:00',634.07,267,35874,2.25,'GEO Transfer' from dual union all
select 33754,'033754','2009-008F','ESC-A 545','Ariane 5 ESC-A',68689,'In Earth orbit','','2009-02-22 00:00:00',632.59,271,35794,1.99,'GEO Transfer' from dual union all
select 33755,'033755','2006-026WF','deb Kosmos-2421','',68198,'Reentered','2010-02-06 00:00:00','2009-02-13 00:00:00',92.32,376,401,65.01,'Intermediate' from dual union all
select 33756,'033756','1965-101F','deb FR-1','',11851,'Reentered','2011-11-10 00:00:00','2009-02-09 00:00:00',97.69,645,652,75.85,'Intermediate' from dual union all
select 33757,'033757','1993-036E','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-17 00:00:00',100.57,776,796,74.04,'Intermediate' from dual union all
select 33758,'033758','1993-036F','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-17 00:00:00',100.58,770,802,74.04,'Intermediate' from dual;
insert into SATELLITE
select 33759,'033759','1993-036G','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-17 00:00:00',100.49,767,797,74.03,'Intermediate' from dual union all
select 3376,'003376','1965-082MN','deb Transtage 4','',11832,'Reentered','1980-05-30 00:00:00','1968-09-24 00:00:00',98.82,669,736,32.57,'Intermediate' from dual union all
select 33760,'033760','1993-036H','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-17 00:00:00',100.47,766,796,74.03,'Intermediate' from dual union all
select 33761,'033761','1993-036J','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-17 00:00:00',100.51,764,802,74.04,'Intermediate' from dual union all
select 33762,'033762','1993-036K','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-17 00:00:00',100.51,772,794,74.05,'Intermediate' from dual union all
select 33763,'033763','1993-036L','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-17 00:00:00',100.7,776,808,73.96,'Intermediate' from dual union all
select 33764,'033764','1993-036M','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-17 00:00:00',100.54,767,801,74.04,'Intermediate' from dual union all
select 33765,'033765','1993-036N','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-17 00:00:00',99.85,603,899,74.03,'Intermediate' from dual union all
select 33766,'033766','1993-036P','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-17 00:00:00',101.07,757,861,74.19,'Intermediate' from dual union all
select 33767,'033767','1993-036Q','deb Kosmos-2251','',65138,'Reentered','2011-02-09 00:00:00','2009-02-17 00:00:00',96.39,391,781,73.94,'Intermediate' from dual union all
select 33768,'033768','1993-036R','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-17 00:00:00',106.35,776,1339,74.07,'Intermediate' from dual union all
select 33769,'033769','1993-036S','deb Kosmos-2251','',65138,'Reentered','2010-11-17 00:00:00','2009-02-17 00:00:00',96.33,391,774,74.02,'Intermediate' from dual union all
select 3377,'003377','1965-082MP','deb Transtage 4','',11832,'Reentered','1980-10-16 00:00:00','1968-09-15 00:00:00',101.42,720,931,32.11,'Intermediate' from dual union all
select 33770,'033770','1993-036T','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-17 00:00:00',100.31,732,814,74.05,'Intermediate' from dual union all
select 33771,'033771','1997-051J','deb Iridium 33','',66131,'Reentered','2014-03-19 00:00:00','2009-02-17 00:00:00',100.36,775,777,86.41,'Polar' from dual union all
select 33772,'033772','1997-051K','deb Iridium 33','',66131,'In Earth orbit','','2009-02-17 00:00:00',101.3,775,866,86.43,'Polar' from dual union all
select 33773,'033773','1997-051L','deb Iridium 33','',66131,'In Earth orbit','','2009-02-17 00:00:00',100.35,766,785,86.4,'Polar' from dual union all
select 33774,'033774','1997-051M','deb Iridium 33','',66131,'Reentered','2013-08-01 00:00:00','2009-02-17 00:00:00',100.87,776,824,86.42,'Polar' from dual union all
select 33775,'033775','1997-051N','deb Iridium 33','',66131,'In Earth orbit','','2009-02-17 00:00:00',100.63,776,800,86.37,'Polar' from dual union all
select 33776,'033776','1997-051P','deb Iridium 33','',66131,'In Earth orbit','','2009-02-17 00:00:00',100.62,776,800,86.41,'Polar' from dual union all
select 33777,'033777','1997-051Q','deb Iridium 33','',66131,'In Earth orbit','','2009-02-17 00:00:00',100.37,771,781,86.39,'Polar' from dual union all
select 33778,'033778','1997-051R','deb Iridium 33','',66131,'Reentered','2011-11-14 00:00:00','2009-02-17 00:00:00',100.89,780,821,86.32,'Polar' from dual union all
select 33779,'033779','1993-036U','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',100.7,768,815,74.05,'Intermediate' from dual union all
select 3378,'003378','1965-082MQ','deb Transtage 4','',11832,'Reentered','1979-01-17 00:00:00','1968-09-14 00:00:00',99.41,703,758,32.21,'Intermediate' from dual union all
select 33780,'033780','1993-036V','deb Kosmos-2251','',65138,'Reentered','2011-04-26 00:00:00','2009-02-19 00:00:00',96.67,411,788,73.95,'Intermediate' from dual union all
select 33781,'033781','1993-036W','deb Kosmos-2251','',65138,'Reentered','2014-03-28 00:00:00','2009-02-19 00:00:00',97.77,521,783,74.21,'Intermediate' from dual union all
select 33782,'033782','1993-036X','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',100.61,771,803,74.04,'Intermediate' from dual union all
select 33783,'033783','1993-036Y','deb Kosmos-2251','',65138,'Reentered','2011-05-30 00:00:00','2009-02-19 00:00:00',96.93,433,791,74.13,'Intermediate' from dual union all
select 33784,'033784','1993-036Z','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.28,671,778,74.22,'Intermediate' from dual union all
select 33785,'033785','1993-036AA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',100.73,767,820,73.9,'Intermediate' from dual union all
select 33786,'033786','1993-036AB','deb Kosmos-2251','',65138,'Reentered','2015-02-27 00:00:00','2009-02-19 00:00:00',98.49,597,776,73.96,'Intermediate' from dual union all
select 33787,'033787','1993-036AC','deb Kosmos-2251','',65138,'Reentered','2009-05-25 00:00:00','2009-02-19 00:00:00',94.89,258,769,74.18,'Intermediate' from dual union all
select 33788,'033788','1993-036AD','deb Kosmos-2251','',65138,'Reentered','2009-05-24 00:00:00','2009-02-19 00:00:00',94.97,271,764,73.93,'Intermediate' from dual union all
select 33789,'033789','1993-036AE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',104.12,768,1139,74.07,'Intermediate' from dual union all
select 3379,'003379','1965-082MR','deb Transtage 4','',11832,'Reentered','1979-07-30 00:00:00','1968-09-15 00:00:00',98.96,681,737,32.19,'Intermediate' from dual union all
select 33790,'033790','1993-036AF','deb Kosmos-2251','',65138,'Reentered','2015-09-24 00:00:00','2009-02-19 00:00:00',98.24,572,777,73.93,'Intermediate' from dual union all
select 33791,'033791','1993-036AG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',100.8,771,822,74.18,'Intermediate' from dual union all
select 33792,'033792','1993-036AH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',107.51,776,1446,73.9,'Intermediate' from dual union all
select 33793,'033793','1993-036AJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',101.71,775,904,73.99,'Intermediate' from dual union all
select 33794,'033794','1993-036AK','deb Kosmos-2251','',65138,'Reentered','2009-06-30 00:00:00','2009-02-19 00:00:00',95.17,284,770,73.79,'Intermediate' from dual union all
select 33795,'033795','1993-036AL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.09,647,784,74.05,'Intermediate' from dual union all
select 33796,'033796','1993-036AM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',101.6,774,895,74.06,'Intermediate' from dual union all
select 33797,'033797','1993-036AN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.74,689,804,74.05,'Intermediate' from dual union all
select 33798,'033798','1993-036AP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.96,687,827,74.03,'Intermediate' from dual union all
select 33799,'033799','1993-036AQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',100.37,740,813,74.05,'Intermediate' from dual union all
select 338,'000338','62 A Delta 1','Kosmos-6','DS-P1 No. 1',8139,'Reentered','1962-09-08 00:00:00','1962-07-30 00:00:00',90.17,262,303,48.94,'Intermediate' from dual union all
select 3380,'003380','1965-082MS','deb Transtage 4','',11832,'Reentered','1990-09-25 00:00:00','1990-09-24 00:00:00',88.56,202,205,31.34,'Intermediate' from dual union all
select 33800,'033800','1993-036AR','deb Kosmos-2251','',65138,'Reentered','2010-05-23 00:00:00','2009-02-19 00:00:00',95.91,355,771,74.08,'Intermediate' from dual union all
select 33801,'033801','1993-036AS','deb Kosmos-2251','',65138,'Reentered','2011-11-16 00:00:00','2009-02-19 00:00:00',96.34,387,780,74.03,'Intermediate' from dual union all
select 33802,'033802','1993-036AT','deb Kosmos-2251','',65138,'Reentered','2014-03-12 00:00:00','2009-02-19 00:00:00',97.72,516,784,73.91,'Intermediate' from dual union all
select 33803,'033803','1993-036AU','deb Kosmos-2251','',65138,'Reentered','2012-01-03 00:00:00','2009-02-19 00:00:00',97.86,514,798,74.11,'Intermediate' from dual union all
select 33804,'033804','1993-036AV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.21,656,785,74.03,'Intermediate' from dual union all
select 33805,'033805','1993-036AW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.37,678,779,74.04,'Intermediate' from dual union all
select 33806,'033806','1993-036AX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.68,681,806,74.1,'Intermediate' from dual union all
select 33807,'033807','1993-036AY','deb Kosmos-2251','',65138,'Reentered','2011-11-14 00:00:00','2009-02-19 00:00:00',96.31,373,791,74.04,'Intermediate' from dual union all
select 33808,'033808','1993-036AZ','deb Kosmos-2251','',65138,'Reentered','2010-03-03 00:00:00','2009-02-19 00:00:00',95.95,345,784,74.06,'Intermediate' from dual union all
select 33809,'033809','1993-036BA','deb Kosmos-2251','',65138,'Reentered','2012-11-08 00:00:00','2009-02-19 00:00:00',97.52,496,784,74.2,'Intermediate' from dual union all
select 3381,'003381','1965-082MT','deb Transtage 4','',11832,'Reentered','1998-12-28 00:00:00','1992-02-03 00:00:00',97.09,578,661,32,'Intermediate' from dual union all
select 33810,'033810','1993-036BB','deb Kosmos-2251','',65138,'Reentered','2014-06-27 00:00:00','2009-02-19 00:00:00',97.99,550,776,74,'Intermediate' from dual union all
select 33811,'033811','1993-036BC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.1,656,776,73.86,'Intermediate' from dual union all
select 33812,'033812','1993-036BD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.76,524,971,74.04,'Intermediate' from dual union all
select 33813,'033813','1993-036BE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',98.77,612,788,74.06,'Intermediate' from dual union all
select 33814,'033814','1993-036BF','deb Kosmos-2251','',65138,'Reentered','2011-12-16 00:00:00','2009-02-19 00:00:00',96.6,415,777,74.16,'Intermediate' from dual union all
select 33815,'033815','1993-036BG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',101.29,773,867,74,'Intermediate' from dual union all
select 33816,'033816','1993-036BH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',104.68,773,1186,73.9,'Intermediate' from dual union all
select 33817,'033817','1993-036BJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.18,663,776,74.08,'Intermediate' from dual union all
select 33818,'033818','1993-036BK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.29,662,788,74.11,'Intermediate' from dual union all
select 33819,'033819','1993-036BL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',101.44,754,900,74.18,'Intermediate' from dual union all
select 3382,'003382','1965-082MU','deb Transtage 4','',11832,'Reentered','1978-09-06 00:00:00','1968-09-15 00:00:00',100.87,688,912,33.03,'Intermediate' from dual union all
select 33820,'033820','1993-036BM','deb Kosmos-2251','',65138,'Reentered','2012-03-19 00:00:00','2009-02-19 00:00:00',96.7,423,778,74.1,'Intermediate' from dual union all
select 33821,'033821','1993-036BN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',98.15,556,785,74.06,'Intermediate' from dual union all
select 33822,'033822','1993-036BP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',103.32,775,1057,74.08,'Intermediate' from dual union all
select 33823,'033823','1993-036BQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',103.6,763,1095,73.81,'Intermediate' from dual union all
select 33824,'033824','1993-036BR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.6,650,829,74.03,'Intermediate' from dual union all
select 33825,'033825','1993-036BS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',100.2,670,866,74,'Intermediate' from dual union all
select 33826,'033826','1993-036BT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',101.52,734,927,74.07,'Intermediate' from dual union all
select 33827,'033827','1993-036BU','deb Kosmos-2251','',65138,'Reentered','2013-12-05 00:00:00','2009-02-19 00:00:00',98.59,554,829,73.99,'Intermediate' from dual union all
select 33828,'033828','1993-036BV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',100.45,755,804,74.05,'Intermediate' from dual union all
select 33829,'033829','1993-036BW','deb Kosmos-2251','',65138,'Reentered','2009-09-27 00:00:00','2009-02-19 00:00:00',99.24,634,811,73.98,'Intermediate' from dual union all
select 3383,'003383','1965-082MV','deb Transtage 4','',11832,'Reentered','1991-10-29 00:00:00','1991-10-28 00:00:00',89.4,242,248,31.93,'Intermediate' from dual union all
select 33830,'033830','1993-036BX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.41,682,778,74.05,'Intermediate' from dual union all
select 33831,'033831','1993-036BY','deb Kosmos-2251','',65138,'Reentered','2014-12-26 00:00:00','2009-02-19 00:00:00',100.66,767,813,74.04,'Intermediate' from dual union all
select 33832,'033832','1993-036BZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',100.67,760,821,74.07,'Intermediate' from dual union all
select 33833,'033833','1993-036CA','deb Kosmos-2251','',65138,'Reentered','2009-07-27 00:00:00','2009-02-19 00:00:00',95.39,297,778,73.89,'Intermediate' from dual union all
select 33834,'033834','1993-036CB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.08,653,776,74.09,'Intermediate' from dual union all
select 33835,'033835','1993-036CC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',100.13,703,826,74.07,'Intermediate' from dual union all
select 33836,'033836','1993-036CD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.71,714,776,74.01,'Intermediate' from dual union all
select 33837,'033837','1993-036CE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',99.77,720,775,74.02,'Intermediate' from dual union all
select 33838,'033838','1993-036CF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',100.12,741,788,74.04,'Intermediate' from dual union all
select 33839,'033839','1993-036CG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',98.98,639,781,74.05,'Intermediate' from dual union all
select 3384,'003384','1965-082MW','deb Transtage 4','',11832,'Reentered','1981-06-18 00:00:00','1968-09-12 00:00:00',101.43,733,920,32.21,'Intermediate' from dual union all
select 33840,'033840','1993-036CH','deb Kosmos-2251','',65138,'Reentered','2013-08-25 00:00:00','2009-02-19 00:00:00',97.8,529,777,74,'Intermediate' from dual union all
select 33841,'033841','1993-036CJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',100.49,766,798,74.04,'Intermediate' from dual union all
select 33842,'033842','1993-036CK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',100.34,737,812,74.04,'Intermediate' from dual union all
select 33843,'033843','1993-036CL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',100.66,592,988,74,'Intermediate' from dual union all
select 33844,'033844','1993-036CM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',101.03,768,846,74.07,'Intermediate' from dual union all
select 33845,'033845','1993-036CN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',100.98,743,867,74.06,'Intermediate' from dual union all
select 33846,'033846','1993-036CP','deb Kosmos-2251','',65138,'Reentered','2012-11-14 00:00:00','2009-02-19 00:00:00',99.52,685,786,74.04,'Intermediate' from dual union all
select 33847,'033847','1993-036CQ','deb Kosmos-2251','',65138,'Reentered','2013-03-18 00:00:00','2009-02-19 00:00:00',99.51,651,819,74.01,'Intermediate' from dual union all
select 33848,'033848','1993-036CR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-19 00:00:00',101.14,771,855,74.04,'Intermediate' from dual;
insert into SATELLITE
select 33849,'033849','1997-051S','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',102.13,772,947,86.13,'Polar' from dual union all
select 3385,'003385','1965-082MX','deb Transtage 4','',11832,'Reentered','1969-04-12 00:00:00','1968-09-12 00:00:00',96.06,530,610,32.43,'Intermediate' from dual union all
select 33850,'033850','1997-051T','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',100.57,775,797,86.35,'Polar' from dual union all
select 33851,'033851','1997-051U','deb Iridium 33','',66131,'Reentered','2014-04-16 00:00:00','2009-02-19 00:00:00',99.42,685,777,86.41,'Polar' from dual union all
select 33852,'033852','1997-051V','deb Iridium 33','',66131,'Reentered','2011-11-16 00:00:00','2009-02-19 00:00:00',99.98,713,803,86.2,'Polar' from dual union all
select 33853,'033853','1997-051W','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',104,771,1125,86.01,'Polar' from dual union all
select 33854,'033854','1997-051X','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',100.01,741,777,86.24,'Polar' from dual union all
select 33855,'033855','1997-051Y','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',103.89,776,1108,86.4,'Polar' from dual union all
select 33856,'033856','1997-051Z','deb Iridium 33','',66131,'Reentered','2015-04-19 00:00:00','2009-02-19 00:00:00',99.92,726,784,86.34,'Polar' from dual union all
select 33857,'033857','1997-051AA','deb Iridium 33','',66131,'Reentered','2013-01-04 00:00:00','2009-02-19 00:00:00',100.83,760,836,86.47,'Polar' from dual union all
select 33858,'033858','1997-051AB','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',100.25,719,821,86.16,'Polar' from dual union all
select 33859,'033859','1997-051AC','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',101.01,776,837,86.35,'Polar' from dual union all
select 3386,'003386','1965-082MY','deb Transtage 4','',11832,'In Earth orbit','','1992-02-03 00:00:00',98.42,676,691,32.37,'Intermediate' from dual union all
select 33860,'033860','1997-051AD','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',100.31,761,785,86.4,'Polar' from dual union all
select 33861,'033861','1997-051AE','deb Iridium 33','',66131,'Reentered','2011-09-19 00:00:00','2009-02-19 00:00:00',99.04,645,780,86.44,'Polar' from dual union all
select 33862,'033862','1997-051AF','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',100.24,725,815,86.42,'Polar' from dual union all
select 33863,'033863','1997-051AG','deb Iridium 33','',66131,'Reentered','2010-09-22 00:00:00','2009-02-19 00:00:00',100.61,764,811,86.47,'Polar' from dual union all
select 33864,'033864','1997-051AH','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',100.47,776,786,86.28,'Polar' from dual union all
select 33865,'033865','1997-051AJ','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',100.63,739,838,86.36,'Polar' from dual union all
select 33866,'033866','1997-051AK','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',102.38,776,966,86.29,'Polar' from dual union all
select 33867,'033867','1997-051AL','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',99.9,730,778,86.42,'Polar' from dual union all
select 33868,'033868','1997-051AM','deb Iridium 33','',66131,'Reentered','2016-11-13 00:00:00','2009-02-19 00:00:00',100.03,744,776,86.35,'Polar' from dual union all
select 33869,'033869','1997-051AN','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',101.9,776,922,86.38,'Polar' from dual union all
select 3387,'003387','1965-082MZ','deb Transtage 4','',11832,'Reentered','1990-03-11 00:00:00','1968-09-12 00:00:00',102.09,680,1036,30.62,'Intermediate' from dual union all
select 33870,'033870','1997-051AP','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',100.75,775,814,86.38,'Polar' from dual union all
select 33871,'033871','1997-051AQ','deb Iridium 33','',66131,'Reentered','2010-06-13 00:00:00','2009-02-19 00:00:00',102.28,773,961,86.39,'Polar' from dual union all
select 33872,'033872','1997-051AR','deb Iridium 33','',66131,'Reentered','2017-08-16 00:00:00','2009-02-19 00:00:00',100,740,777,86.34,'Polar' from dual union all
select 33873,'033873','1997-051AS','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',100.3,757,788,86.38,'Polar' from dual union all
select 33874,'033874','1997-051AT','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',101.85,776,917,86.29,'Polar' from dual union all
select 33875,'033875','1997-051AU','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',99.61,707,773,86.42,'Polar' from dual union all
select 33876,'033876','1997-051AV','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',100.56,749,822,86.22,'Polar' from dual union all
select 33877,'033877','1997-051AW','deb Iridium 33','',66131,'Reentered','2013-10-18 00:00:00','2009-02-19 00:00:00',101.58,774,893,86.17,'Polar' from dual union all
select 33878,'033878','1997-051AX','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',102.17,776,947,86.34,'Polar' from dual union all
select 33879,'033879','1997-051AY','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',100.33,765,783,86.37,'Polar' from dual union all
select 3388,'003388','1968-075A','Kosmos-240','Zenit-2 No. 66',16802,'Landed','1968-09-21 00:00:00','1968-09-16 00:00:00',89.28,195,283,51.82,'Intermediate' from dual union all
select 33880,'033880','1997-051AZ','deb Iridium 33','',66131,'Reentered','2011-12-07 00:00:00','2009-02-19 00:00:00',100.69,775,807,86.44,'Polar' from dual union all
select 33881,'033881','1997-051BA','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',102.01,771,936,86.39,'Polar' from dual union all
select 33882,'033882','1997-051BB','deb Iridium 33','',66131,'Reentered','2016-05-06 00:00:00','2009-02-19 00:00:00',99.73,690,802,86.31,'Polar' from dual union all
select 33883,'033883','1997-051BC','deb Iridium 33','',66131,'Reentered','2015-10-30 00:00:00','2009-02-19 00:00:00',100.16,708,825,86.18,'Polar' from dual union all
select 33884,'033884','1997-051BD','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',100.33,772,777,86.38,'Polar' from dual union all
select 33885,'033885','1997-051BE','deb Iridium 33','',66131,'Reentered','2013-12-12 00:00:00','2009-02-19 00:00:00',100.46,776,785,86.39,'Polar' from dual union all
select 33886,'033886','1997-051BF','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',100.58,774,798,86.38,'Polar' from dual union all
select 33887,'033887','1997-051BG','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',100.45,773,786,86.3,'Polar' from dual union all
select 33888,'033888','1997-051BH','deb Iridium 33','',66131,'In Earth orbit','','2009-02-19 00:00:00',101.39,777,871,86.32,'Polar' from dual union all
select 33889,'033889','1993-036CS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',99.24,641,804,74.13,'Intermediate' from dual union all
select 3389,'003389','1968-075B','11A57I','11A57I',16802,'Reentered','1968-09-17 00:00:00','1968-09-16 00:00:00',88.63,167,247,51.82,'Intermediate' from dual union all
select 33890,'033890','1993-036CT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',99.88,711,795,74.03,'Intermediate' from dual union all
select 33891,'033891','1993-036CU','deb Kosmos-2251','',65138,'Reentered','2012-03-12 00:00:00','2009-02-20 00:00:00',98.11,486,851,74.08,'Intermediate' from dual union all
select 33892,'033892','1993-036CV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',102.02,758,951,73.8,'Intermediate' from dual union all
select 33893,'033893','1993-036CW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',101.32,627,1016,74.06,'Intermediate' from dual union all
select 33894,'033894','1993-036CX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',109.96,763,1687,73.74,'Intermediate' from dual union all
select 33895,'033895','1993-036CY','deb Kosmos-2251','',65138,'Reentered','2011-07-21 00:00:00','2009-02-20 00:00:00',96.16,375,774,73.78,'Intermediate' from dual union all
select 33896,'033896','1993-036CZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',102.33,757,981,73.77,'Intermediate' from dual union all
select 33897,'033897','1993-036DA','deb Kosmos-2251','',65138,'Reentered','2013-04-08 00:00:00','2009-02-20 00:00:00',97.3,466,793,73.83,'Intermediate' from dual union all
select 33898,'033898','1993-036DB','deb Kosmos-2251','',65138,'Reentered','2009-05-22 00:00:00','2009-02-20 00:00:00',94.74,248,764,73.8,'Intermediate' from dual union all
select 33899,'033899','1993-036DC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.06,700,823,74.06,'Intermediate' from dual union all
select 339,'000339','62 A Delta 2','S1','S1',8139,'Reentered','1962-08-08 00:00:00','1962-07-09 00:00:00',90.35,262,322,48.95,'Intermediate' from dual union all
select 3390,'003390','1968-066F','deb Explorer 40','',16793,'Reentered','2011-11-24 00:00:00','1968-09-08 00:00:00',118.36,681,2534,80.61,'Intermediate' from dual union all
select 33900,'033900','1993-036DD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',99.68,700,787,74.03,'Intermediate' from dual union all
select 33901,'033901','1993-036DE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',99.91,649,860,73.8,'Intermediate' from dual union all
select 33902,'033902','1993-036DF','deb Kosmos-2251','',65138,'Reentered','2014-07-25 00:00:00','2009-02-20 00:00:00',99.3,651,800,74.04,'Intermediate' from dual union all
select 33903,'033903','1993-036DG','deb Kosmos-2251','',65138,'Reentered','2016-08-16 00:00:00','2009-02-20 00:00:00',101.27,771,867,74.06,'Intermediate' from dual union all
select 33904,'033904','1993-036DH','deb Kosmos-2251','',65138,'Reentered','2014-04-23 00:00:00','2009-02-20 00:00:00',98.37,580,782,74.02,'Intermediate' from dual union all
select 33905,'033905','1993-036DJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',101.22,769,864,73.89,'Intermediate' from dual union all
select 33906,'033906','1993-036DK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',101.25,775,861,74.04,'Intermediate' from dual union all
select 33907,'033907','1993-036DL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',102.96,772,1026,74.06,'Intermediate' from dual union all
select 33908,'033908','1993-036DM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',104.15,748,1161,73.85,'Intermediate' from dual union all
select 33909,'033909','1993-036DN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',101.81,771,917,74.09,'Intermediate' from dual union all
select 3391,'003391','1968-066G','deb Explorer 40','',16793,'Reentered','2002-09-17 00:00:00','1968-09-08 00:00:00',118.51,685,2543,80.69,'Intermediate' from dual union all
select 33910,'033910','1993-036DP','deb Kosmos-2251','',65138,'Reentered','2015-06-02 00:00:00','2009-02-20 00:00:00',99.7,635,854,74.07,'Intermediate' from dual union all
select 33911,'033911','1993-036DQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.52,762,805,74.11,'Intermediate' from dual union all
select 33912,'033912','1993-036DR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.44,768,791,74.04,'Intermediate' from dual union all
select 33913,'033913','1993-036DS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.32,732,816,74.04,'Intermediate' from dual union all
select 33914,'033914','1993-036DT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',105.33,776,1243,74.09,'Intermediate' from dual union all
select 33915,'033915','1993-036DU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',101.07,721,898,74.05,'Intermediate' from dual union all
select 33916,'033916','1993-036DV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',105.54,773,1267,74.05,'Intermediate' from dual union all
select 33917,'033917','1993-036DW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',105.96,774,1305,74.1,'Intermediate' from dual union all
select 33918,'033918','1993-036DX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',99.59,672,806,74.08,'Intermediate' from dual union all
select 33919,'033919','1993-036DY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.12,733,796,74.06,'Intermediate' from dual union all
select 3392,'003392','1968-066H','deb Explorer 40','',16793,'In Earth orbit','','1968-09-08 00:00:00',118.42,678,2542,80.71,'Intermediate' from dual union all
select 33920,'033920','1993-036DZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',101.04,764,852,74.08,'Intermediate' from dual union all
select 33921,'033921','1993-036EA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.25,759,782,74,'Intermediate' from dual union all
select 33922,'033922','1993-036EB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.11,744,784,74.05,'Intermediate' from dual union all
select 33923,'033923','1993-036EC','deb Kosmos-2251','',65138,'Reentered','2013-11-17 00:00:00','2009-02-20 00:00:00',100.55,698,872,74.08,'Intermediate' from dual union all
select 33924,'033924','1993-036ED','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',107.04,776,1403,73.92,'Intermediate' from dual union all
select 33925,'033925','1993-036EE','deb Kosmos-2251','',65138,'Reentered','2011-11-16 00:00:00','2009-02-20 00:00:00',99.29,642,808,74.06,'Intermediate' from dual union all
select 33926,'033926','1993-036EF','deb Kosmos-2251','',65138,'Reentered','2014-03-20 00:00:00','2009-02-20 00:00:00',100.45,755,804,74.04,'Intermediate' from dual union all
select 33927,'033927','1993-036EG','deb Kosmos-2251','',65138,'Reentered','2011-10-07 00:00:00','2009-02-20 00:00:00',97.16,444,802,73.95,'Intermediate' from dual union all
select 33928,'033928','1993-036EH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',99.98,709,806,73.93,'Intermediate' from dual union all
select 33929,'033929','1993-036EJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.41,775,781,74.04,'Intermediate' from dual union all
select 3393,'003393','1968-066J','deb Explorer 40','',16793,'Reentered','2014-07-10 00:00:00','1968-09-08 00:00:00',118.22,682,2520,80.62,'Intermediate' from dual union all
select 33930,'033930','1993-036EK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.84,776,820,74.04,'Intermediate' from dual union all
select 33931,'033931','1993-036EL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.55,567,1003,74.04,'Intermediate' from dual union all
select 33932,'033932','1993-036EM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.76,764,826,74.04,'Intermediate' from dual union all
select 33933,'033933','1993-036EN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',101,748,864,74.09,'Intermediate' from dual union all
select 33934,'033934','1993-036EP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.14,748,783,74.05,'Intermediate' from dual union all
select 33935,'033935','1993-036EQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',99.76,704,791,74.02,'Intermediate' from dual union all
select 33936,'033936','1993-036ER','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',99.92,726,784,74.03,'Intermediate' from dual union all
select 33937,'033937','1993-036ES','deb Kosmos-2251','',65138,'Reentered','2013-07-01 00:00:00','2009-02-20 00:00:00',97.43,487,785,73.76,'Intermediate' from dual union all
select 33938,'033938','1993-036ET','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.35,761,790,74.05,'Intermediate' from dual;
insert into SATELLITE
select 33939,'033939','1993-036EU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.64,771,807,74.04,'Intermediate' from dual union all
select 3394,'003394','1968-076A','Zond-5','L-1 No. 9L',16803,'Deep Space','1968-09-17 00:00:00','1968-09-14 00:00:00',14722.02,200,385000,51.5,'Cislunar' from dual union all
select 33940,'033940','1993-036EV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.36,762,789,74.05,'Intermediate' from dual union all
select 33941,'033941','1993-036EW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',98.74,614,783,74.02,'Intermediate' from dual union all
select 33942,'033942','1993-036EX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.49,766,798,74.04,'Intermediate' from dual union all
select 33943,'033943','1993-036EY','deb Kosmos-2251','',65138,'Reentered','2011-05-16 00:00:00','2009-02-20 00:00:00',100.46,573,988,74,'Intermediate' from dual union all
select 33944,'033944','1993-036EZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.67,701,880,74.05,'Intermediate' from dual union all
select 33945,'033945','1993-036FA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.58,770,802,74.03,'Intermediate' from dual union all
select 33946,'033946','1993-036FB','deb Kosmos-2251','',65138,'Reentered','2012-03-02 00:00:00','2009-02-20 00:00:00',97.48,495,781,73.92,'Intermediate' from dual union all
select 33947,'033947','1993-036FC','deb Kosmos-2251','',65138,'Reentered','2015-03-24 00:00:00','2009-02-20 00:00:00',98.61,607,778,73.97,'Intermediate' from dual union all
select 33948,'033948','1993-036FD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-20 00:00:00',100.58,769,803,74.04,'Intermediate' from dual union all
select 33949,'033949','1997-051BJ','deb Iridium 33','',66131,'Reentered','2013-04-09 00:00:00','2009-02-20 00:00:00',100.82,777,817,86.4,'Polar' from dual union all
select 3395,'003395','1968-076B','SOZ','SOZ',16803,'Reentered','1968-09-16 00:00:00','1968-09-14 00:00:00',88.87,203,235,51.52,'Intermediate' from dual union all
select 33950,'033950','1997-051BK','deb Iridium 33','',66131,'In Earth orbit','','2009-02-20 00:00:00',100.73,776,811,86.39,'Polar' from dual union all
select 33951,'033951','1997-051BL','deb Iridium 33','',66131,'Reentered','2017-01-30 00:00:00','2009-02-20 00:00:00',100.9,775,828,86.46,'Polar' from dual union all
select 33952,'033952','1997-051BM','deb Iridium 33','',66131,'In Earth orbit','','2009-02-20 00:00:00',100.45,762,798,86.36,'Polar' from dual union all
select 33953,'033953','1997-051BN','deb Iridium 33','',66131,'In Earth orbit','','2009-02-20 00:00:00',100.67,770,811,86.4,'Polar' from dual union all
select 33954,'033954','1997-051BP','deb Iridium 33','',66131,'In Earth orbit','','2009-02-20 00:00:00',100.95,776,832,86.39,'Polar' from dual union all
select 33955,'033955','1997-051BQ','deb Iridium 33','',66131,'In Earth orbit','','2009-02-20 00:00:00',101.4,776,873,86.38,'Polar' from dual union all
select 33956,'033956','1997-051BR','deb Iridium 33','',66131,'In Earth orbit','','2009-02-20 00:00:00',101.1,771,851,86.44,'Polar' from dual union all
select 33957,'033957','1997-051BS','deb Iridium 33','',66131,'Reentered','2014-02-14 00:00:00','2009-02-20 00:00:00',101.96,775,927,86.38,'Polar' from dual union all
select 33958,'033958','1997-051BT','deb Iridium 33','',66131,'Reentered','2013-07-11 00:00:00','2009-02-20 00:00:00',100.38,743,811,86.35,'Polar' from dual union all
select 33959,'033959','1997-051BU','deb Iridium 33','',66131,'In Earth orbit','','2009-02-20 00:00:00',101.9,776,922,86.36,'Polar' from dual union all
select 3396,'003396','1968-076C','SOZ','SOZ',16803,'Reentered','1968-09-18 00:00:00','1968-09-16 00:00:00',88.6,191,220,51.52,'Intermediate' from dual union all
select 33960,'033960','1997-051BV','deb Iridium 33','',66131,'In Earth orbit','','2009-02-20 00:00:00',100.36,773,779,86.28,'Polar' from dual union all
select 33961,'033961','1997-051BW','deb Iridium 33','',66131,'In Earth orbit','','2009-02-20 00:00:00',100.71,757,828,86.28,'Polar' from dual union all
select 33962,'033962','1997-051BX','deb Iridium 33','',66131,'In Earth orbit','','2009-02-20 00:00:00',100.58,771,801,86.36,'Polar' from dual union all
select 33963,'033963','1997-051BY','deb Iridium 33','',66131,'Reentered','2012-04-23 00:00:00','2009-02-20 00:00:00',105.58,778,1266,85.61,'Polar' from dual union all
select 33964,'033964','1997-051BZ','deb Iridium 33','',66131,'Reentered','2018-05-22 00:00:00','2009-02-20 00:00:00',100.28,747,797,86.35,'Polar' from dual union all
select 33965,'033965','1997-051CA','deb Iridium 33','',66131,'In Earth orbit','','2009-02-20 00:00:00',100.48,772,791,86.38,'Polar' from dual union all
select 33966,'033966','1997-051CB','deb Iridium 33','',66131,'In Earth orbit','','2009-02-20 00:00:00',100.3,769,776,86.4,'Polar' from dual union all
select 33967,'033967','1997-051CC','deb Iridium 33','',66131,'In Earth orbit','','2009-02-20 00:00:00',100.53,760,808,86.4,'Polar' from dual union all
select 33968,'033968','1997-051CD','deb Iridium 33','',66131,'Reentered','2011-10-08 00:00:00','2009-02-20 00:00:00',100.51,769,797,86.42,'Polar' from dual union all
select 33969,'033969','2008-058C','Kosmos-2445 solar panel','Kobal''t-M part',68566,'Reentered','2009-02-24 00:00:00','2009-02-23 00:00:00',89.15,188,277,67.15,'Intermediate' from dual union all
select 3397,'003397','1968-076D','SOK','Support cone',16803,'Reentered','1968-09-16 00:00:00','1968-09-14 00:00:00',88.94,205,240,51.6,'Intermediate' from dual union all
select 33970,'033970','2008-058D','Kosmos-2445 solar panel','Kobal''t-M part',68566,'Reentered','2009-02-24 00:00:00','2009-02-23 00:00:00',88.64,177,238,67.14,'Intermediate' from dual union all
select 33971,'033971','1993-036FE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.47,766,796,74.03,'Intermediate' from dual union all
select 33972,'033972','1993-036FF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',102.27,738,995,74,'Intermediate' from dual union all
select 33973,'033973','1993-036FG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',99.87,717,788,74.01,'Intermediate' from dual union all
select 33974,'033974','1993-036FH','deb Kosmos-2251','',65138,'Reentered','2015-05-11 00:00:00','2009-02-23 00:00:00',98.83,627,779,73.98,'Intermediate' from dual union all
select 33975,'033975','1993-036FJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',103.28,767,1061,73.81,'Intermediate' from dual union all
select 33976,'033976','1993-036FK','deb Kosmos-2251','',65138,'Reentered','2015-02-10 00:00:00','2009-02-23 00:00:00',98.1,528,808,73.82,'Intermediate' from dual union all
select 33977,'033977','1993-036FL','deb Kosmos-2251','',65138,'Reentered','2010-11-01 00:00:00','2009-02-23 00:00:00',95.98,355,777,73.77,'Intermediate' from dual union all
select 33978,'033978','1993-036FM','deb Kosmos-2251','',65138,'Reentered','2010-05-12 00:00:00','2009-02-23 00:00:00',95.17,283,770,73.88,'Intermediate' from dual union all
select 33979,'033979','1993-036FN','deb Kosmos-2251','',65138,'Reentered','2009-05-03 00:00:00','2009-02-23 00:00:00',94.95,264,769,73.79,'Intermediate' from dual union all
select 3398,'003398','1968-077A','Kosmos-241','Zenit-4 No. 49',16804,'Landed','1968-09-24 00:00:00','1968-09-19 00:00:00',89.66,196,320,65.43,'Intermediate' from dual union all
select 33980,'033980','1993-036FP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',99.97,729,786,74.03,'Intermediate' from dual union all
select 33981,'033981','1993-036FQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',98.67,615,776,73.85,'Intermediate' from dual union all
select 33982,'033982','1993-036FR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',99.53,668,804,74.08,'Intermediate' from dual union all
select 33983,'033983','1993-036FS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.12,745,784,74.06,'Intermediate' from dual union all
select 33984,'033984','1993-036FT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',99.83,725,775,74.02,'Intermediate' from dual union all
select 33985,'033985','1993-036FU','deb Kosmos-2251','',65138,'Reentered','2011-04-22 00:00:00','2009-02-23 00:00:00',97.31,477,783,73.96,'Intermediate' from dual union all
select 33986,'033986','1993-036FV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.8,759,834,74.04,'Intermediate' from dual union all
select 33987,'033987','1993-036FW','deb Kosmos-2251','',65138,'Reentered','2013-04-03 00:00:00','2009-02-23 00:00:00',97.05,448,787,73.92,'Intermediate' from dual union all
select 33988,'033988','1993-036FX','deb Kosmos-2251','',65138,'Reentered','2012-02-27 00:00:00','2009-02-23 00:00:00',96.86,438,778,73.97,'Intermediate' from dual union all
select 33989,'033989','1993-036FY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',99.55,699,776,74.03,'Intermediate' from dual union all
select 3399,'003399','1968-077B','11A57I','11A57I',16804,'Reentered','1968-09-22 00:00:00','1968-09-19 00:00:00',89.29,186,293,65.41,'Intermediate' from dual union all
select 33990,'033990','1993-036FZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',107.1,776,1409,73.93,'Intermediate' from dual union all
select 33991,'033991','1993-036GA','deb Kosmos-2251','',65138,'Reentered','2014-12-11 00:00:00','2009-02-23 00:00:00',99.5,676,793,74.02,'Intermediate' from dual union all
select 33992,'033992','1993-036GB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',99.75,690,804,74.03,'Intermediate' from dual union all
select 33993,'033993','1993-036GC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',99.94,725,787,74.03,'Intermediate' from dual union all
select 33994,'033994','1993-036GD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.08,745,779,74.05,'Intermediate' from dual union all
select 33995,'033995','1993-036GE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',101.46,774,882,74.07,'Intermediate' from dual union all
select 33996,'033996','1993-036GF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',101.62,753,918,74.12,'Intermediate' from dual union all
select 33997,'033997','1993-036GG','deb Kosmos-2251','',65138,'Reentered','2013-04-01 00:00:00','2009-02-23 00:00:00',99.4,674,785,74.04,'Intermediate' from dual union all
select 33998,'033998','1993-036GH','deb Kosmos-2251','',65138,'Reentered','2009-06-04 00:00:00','2009-02-23 00:00:00',97.39,537,731,74.16,'Intermediate' from dual union all
select 33999,'033999','1993-036GJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',99.85,688,815,73.95,'Intermediate' from dual union all
select 34,'000034','60 Epsilon 1','Priborniy otsek','PO',6320,'Reentered','1962-09-05 00:00:00','1960-06-17 00:00:00',94.15,291,664,64.89,'Intermediate' from dual union all
select 340,'000340','62 A Epsilon 1','Telstar 1','Telstar 1',8140,'In Earth orbit','','1962-08-21 00:00:00',157.8,956,5634,44.82,'Medium' from dual union all
select 3400,'003400','1965-082NA','deb Transtage 4','',11832,'Reentered','1979-03-10 00:00:00','1967-12-30 00:00:00',100.25,726,815,31.95,'Intermediate' from dual union all
select 34000,'034000','1993-036GK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.2,743,793,74.05,'Intermediate' from dual union all
select 34001,'034001','1993-036GL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.48,766,797,74.04,'Intermediate' from dual union all
select 34002,'034002','1993-036GM','deb Kosmos-2251','',65138,'Reentered','2014-01-20 00:00:00','2009-02-23 00:00:00',98.44,577,792,74.04,'Intermediate' from dual union all
select 34003,'034003','1993-036GN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',105.22,771,1239,74.09,'Intermediate' from dual union all
select 34004,'034004','1993-036GP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',101.04,776,840,74.05,'Intermediate' from dual union all
select 34005,'034005','1993-036GQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.18,752,782,74.05,'Intermediate' from dual union all
select 34006,'034006','1993-036GR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.75,771,818,74.08,'Intermediate' from dual union all
select 34007,'034007','1993-036GS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',99.76,700,794,74.03,'Intermediate' from dual union all
select 34008,'034008','1993-036GT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.66,773,807,74.04,'Intermediate' from dual union all
select 34009,'034009','1993-036GU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.68,771,811,74.04,'Intermediate' from dual union all
select 3401,'003401','1965-082NB','deb Transtage 4','',11832,'Reentered','1972-03-01 00:00:00','1968-09-28 00:00:00',98.73,660,736,32.51,'Intermediate' from dual union all
select 34010,'034010','1993-036GV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.05,736,786,74.03,'Intermediate' from dual union all
select 34011,'034011','1993-036GW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.97,771,839,74.03,'Intermediate' from dual union all
select 34012,'034012','1993-036GX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',101.61,772,898,74.05,'Intermediate' from dual union all
select 34013,'034013','1993-036GY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.18,725,809,74.03,'Intermediate' from dual union all
select 34014,'034014','1993-036GZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',99.62,697,783,74.03,'Intermediate' from dual union all
select 34015,'034015','1993-036HA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.55,764,806,74.04,'Intermediate' from dual union all
select 34016,'034016','1993-036HB','deb Kosmos-2251','',65138,'Reentered','2013-09-15 00:00:00','2009-02-23 00:00:00',98.08,535,799,73.99,'Intermediate' from dual union all
select 34017,'034017','1993-036HC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',101.14,771,855,74.04,'Intermediate' from dual union all
select 34018,'034018','1993-036HD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',104.62,775,1179,73.88,'Intermediate' from dual union all
select 34019,'034019','1993-036HE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',101.45,775,879,73.96,'Intermediate' from dual union all
select 3402,'003402','1965-082NC','deb Transtage 4','',11832,'Reentered','1978-07-05 00:00:00','1968-09-23 00:00:00',99.24,688,756,32.43,'Intermediate' from dual union all
select 34020,'034020','1993-036HF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',99.74,706,787,74.05,'Intermediate' from dual union all
select 34021,'034021','1993-036HG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.06,744,778,74.05,'Intermediate' from dual union all
select 34022,'034022','1993-036HH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.43,762,796,74.06,'Intermediate' from dual union all
select 34023,'034023','1993-036HJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.48,767,796,74.03,'Intermediate' from dual union all
select 34024,'034024','1993-036HK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.6,771,803,74.05,'Intermediate' from dual union all
select 34025,'034025','1993-036HL','deb Kosmos-2251','',65138,'Reentered','2010-12-03 00:00:00','2009-02-23 00:00:00',95.49,311,774,74.09,'Intermediate' from dual union all
select 34026,'034026','1993-036HM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',106.2,775,1325,73.77,'Intermediate' from dual union all
select 34027,'034027','1993-036HN','deb Kosmos-2251','',65138,'Reentered','2011-12-23 00:00:00','2009-02-23 00:00:00',97.36,483,781,74.09,'Intermediate' from dual;
insert into SATELLITE
select 34028,'034028','1993-036HP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',109.34,775,1618,73.89,'Intermediate' from dual union all
select 34029,'034029','1993-036HQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',98.57,593,788,73.96,'Intermediate' from dual union all
select 3403,'003403','1965-082ND','deb Transtage 4','',11832,'Reentered','1980-11-06 00:00:00','1968-09-27 00:00:00',99.11,685,748,32.76,'Intermediate' from dual union all
select 34030,'034030','1993-036HR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',97.14,465,779,73.99,'Intermediate' from dual union all
select 34031,'034031','1993-036HS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.43,753,805,74.03,'Intermediate' from dual union all
select 34032,'034032','1993-036HT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.89,753,849,74.04,'Intermediate' from dual union all
select 34033,'034033','1993-036HU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',101.13,756,868,74.02,'Intermediate' from dual union all
select 34034,'034034','1993-036HV','deb Kosmos-2251','',65138,'Reentered','2010-04-25 00:00:00','2009-02-23 00:00:00',95.83,344,774,73.98,'Intermediate' from dual union all
select 34035,'034035','1993-036HW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',101.16,766,862,74.08,'Intermediate' from dual union all
select 34036,'034036','1993-036HX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.86,759,840,74.05,'Intermediate' from dual union all
select 34037,'034037','1993-036HY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',101.1,766,856,74.06,'Intermediate' from dual union all
select 34038,'034038','1993-036HZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.68,759,822,74.08,'Intermediate' from dual union all
select 34039,'034039','1993-036JA','deb Kosmos-2251','',65138,'Reentered','2012-06-07 00:00:00','2009-02-23 00:00:00',100.24,730,809,74.04,'Intermediate' from dual union all
select 3404,'003404','1965-082NE','deb Transtage 4','',11832,'Reentered','1970-03-03 00:00:00','1968-09-26 00:00:00',97.16,581,665,31.84,'Intermediate' from dual union all
select 34040,'034040','1993-036JB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',106.14,773,1323,73.96,'Intermediate' from dual union all
select 34041,'034041','1993-036JC','deb Kosmos-2251','',65138,'Reentered','2015-03-17 00:00:00','2009-02-23 00:00:00',97.58,510,776,73.94,'Intermediate' from dual union all
select 34042,'034042','1993-036JD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',98.14,562,778,73.89,'Intermediate' from dual union all
select 34043,'034043','1993-036JE','deb Kosmos-2251','',65138,'Reentered','2012-03-31 00:00:00','2009-02-23 00:00:00',100.44,758,801,74.04,'Intermediate' from dual union all
select 34044,'034044','1993-036JF','deb Kosmos-2251','',65138,'Reentered','2011-05-31 00:00:00','2009-02-23 00:00:00',99.19,650,790,74.04,'Intermediate' from dual union all
select 34045,'034045','1993-036JG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.54,765,804,74.04,'Intermediate' from dual union all
select 34046,'034046','1993-036JH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',101.27,738,900,74.05,'Intermediate' from dual union all
select 34047,'034047','1993-036JJ','deb Kosmos-2251','',65138,'Reentered','2011-05-16 00:00:00','2009-02-23 00:00:00',96.7,416,786,74.21,'Intermediate' from dual union all
select 34048,'034048','1993-036JK','deb Kosmos-2251','',65138,'Reentered','2013-03-03 00:00:00','2009-02-23 00:00:00',97.44,494,778,73.89,'Intermediate' from dual union all
select 34049,'034049','1993-036JL','deb Kosmos-2251','',65138,'Reentered','2012-10-15 00:00:00','2009-02-23 00:00:00',97.68,515,781,74.01,'Intermediate' from dual union all
select 3405,'003405','1965-082NF','deb Transtage 4','',11832,'Reentered','1969-05-21 00:00:00','1968-09-22 00:00:00',95.97,524,607,31.86,'Intermediate' from dual union all
select 34050,'034050','1993-036JM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',103.78,767,1108,74.09,'Intermediate' from dual union all
select 34051,'034051','1993-036JN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',99.18,662,776,74,'Intermediate' from dual union all
select 34052,'034052','1993-036JP','deb Kosmos-2251','',65138,'Reentered','2010-01-12 00:00:00','2009-02-23 00:00:00',99.45,299,1166,74.2,'Intermediate' from dual union all
select 34053,'034053','1993-036JQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',107.13,770,1418,73.85,'Intermediate' from dual union all
select 34054,'034054','1993-036JR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',99.98,727,789,74.05,'Intermediate' from dual union all
select 34055,'034055','1993-036JS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.14,677,854,74.12,'Intermediate' from dual union all
select 34056,'034056','1993-036JT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',102.89,775,1016,74.08,'Intermediate' from dual union all
select 34057,'034057','1993-036JU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.33,745,804,74.04,'Intermediate' from dual union all
select 34058,'034058','1993-036JV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',106.53,776,1356,73.93,'Intermediate' from dual union all
select 34059,'034059','1993-036JW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.44,762,797,74.05,'Intermediate' from dual union all
select 3406,'003406','1965-082NG','deb Transtage 4','',11832,'Reentered','1969-05-08 00:00:00','1968-09-24 00:00:00',96.51,519,663,30.52,'Intermediate' from dual union all
select 34060,'034060','1993-036JX','deb Kosmos-2251','',65138,'Reentered','2014-10-20 00:00:00','2009-02-23 00:00:00',99.32,650,803,74.04,'Intermediate' from dual union all
select 34061,'034061','1993-036JY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.49,766,798,74.04,'Intermediate' from dual union all
select 34062,'034062','1993-036JZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.72,673,912,74.05,'Intermediate' from dual union all
select 34063,'034063','1993-036KA','deb Kosmos-2251','',65138,'Reentered','2014-03-01 00:00:00','2009-02-23 00:00:00',99.95,728,785,74.02,'Intermediate' from dual union all
select 34064,'034064','1993-036KB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.63,770,807,74.04,'Intermediate' from dual union all
select 34065,'034065','1993-036KC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.4,765,790,74.04,'Intermediate' from dual union all
select 34066,'034066','1993-036KD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',102.15,595,1125,73.98,'Intermediate' from dual union all
select 34067,'034067','1993-036KE','deb Kosmos-2251','',65138,'Reentered','2009-07-01 00:00:00','2009-02-23 00:00:00',94.89,266,761,74,'Intermediate' from dual union all
select 34068,'034068','1993-036KF','deb Kosmos-2251','',65138,'Reentered','2011-11-03 00:00:00','2009-02-23 00:00:00',97.87,514,799,74.02,'Intermediate' from dual union all
select 34069,'034069','1993-036KG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',99.59,697,780,74.03,'Intermediate' from dual union all
select 3407,'003407','1965-082NH','deb Transtage 4','',11832,'Reentered','1978-04-17 00:00:00','1968-10-01 00:00:00',101.78,724,962,32.02,'Intermediate' from dual union all
select 34070,'034070','1993-036KH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-23 00:00:00',100.65,756,823,74.02,'Intermediate' from dual union all
select 34071,'034071','1997-051CE','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',100.43,771,787,86.35,'Polar' from dual union all
select 34072,'034072','1997-051CF','deb Iridium 33','',66131,'Reentered','2010-10-11 00:00:00','2009-02-23 00:00:00',100.03,730,790,86.32,'Polar' from dual union all
select 34073,'034073','1997-051CG','deb Iridium 33','',66131,'Reentered','2015-09-02 00:00:00','2009-02-23 00:00:00',100.61,776,799,86.41,'Polar' from dual union all
select 34074,'034074','1997-051CH','deb Iridium 33','',66131,'Reentered','2014-05-29 00:00:00','2009-02-23 00:00:00',100.97,776,833,86.31,'Polar' from dual union all
select 34075,'034075','1997-051CJ','deb Iridium 33','',66131,'Reentered','2016-09-16 00:00:00','2009-02-23 00:00:00',101.03,762,853,86.48,'Polar' from dual union all
select 34076,'034076','1997-051CK','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',99.82,722,778,86.34,'Polar' from dual union all
select 34077,'034077','1997-051CL','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',100.78,776,816,86.4,'Polar' from dual union all
select 34078,'034078','1997-051CM','deb Iridium 33','',66131,'Reentered','2013-01-07 00:00:00','2009-02-23 00:00:00',102.09,779,937,86.39,'Polar' from dual union all
select 34079,'034079','1997-051CN','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',100.26,765,777,86.4,'Polar' from dual union all
select 3408,'003408','1968-078A','OPS 0165','CORONA J-49',16805,'Reentered Attached','1968-10-08 00:00:00','1968-09-25 00:00:00',90.17,177,389,83.01,'Intermediate' from dual union all
select 34080,'034080','1997-051CP','deb Iridium 33','',66131,'Reentered','2010-09-29 00:00:00','2009-02-23 00:00:00',99.9,720,788,86.36,'Polar' from dual union all
select 34081,'034081','1997-051CQ','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',99.97,705,810,86.33,'Polar' from dual union all
select 34082,'034082','1997-051CR','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',103.08,719,1090,86.43,'Polar' from dual union all
select 34083,'034083','1997-051CS','deb Iridium 33','',66131,'Reentered','2011-04-03 00:00:00','2009-02-23 00:00:00',98.41,587,779,86.55,'Polar' from dual union all
select 34084,'034084','1997-051CT','deb Iridium 33','',66131,'Reentered','2011-11-18 00:00:00','2009-02-23 00:00:00',97.72,524,776,86.28,'Polar' from dual union all
select 34085,'034085','1997-051CU','deb Iridium 33','',66131,'Reentered','2010-01-12 00:00:00','2009-02-23 00:00:00',98.89,595,817,86.21,'Polar' from dual union all
select 34086,'034086','1997-051CV','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',101.11,777,845,86.39,'Polar' from dual union all
select 34087,'034087','1997-051CW','deb Iridium 33','',66131,'Reentered','2012-10-20 00:00:00','2009-02-23 00:00:00',101.4,776,873,86.3,'Polar' from dual union all
select 34088,'034088','1997-051CX','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',100.55,774,795,86.41,'Polar' from dual union all
select 34089,'034089','1997-051CY','deb Iridium 33','',66131,'Reentered','2011-01-12 00:00:00','2009-02-23 00:00:00',101.42,774,878,86.3,'Polar' from dual union all
select 3409,'003409','1968-078B','VAMPAN I','P-11 No. 4413',16805,'Reentered','1969-09-28 00:00:00','1968-10-29 00:00:00',94.55,491,503,83.21,'Intermediate' from dual union all
select 34090,'034090','1997-051CZ','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',99.93,734,777,86.43,'Polar' from dual union all
select 34091,'034091','1997-051DA','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',101.69,773,905,86.37,'Polar' from dual union all
select 34092,'034092','1997-051DB','deb Iridium 33','',66131,'Reentered','2009-05-16 00:00:00','2009-02-23 00:00:00',99.19,633,807,86.34,'Polar' from dual union all
select 34093,'034093','1997-051DC','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',99.6,703,776,86.31,'Polar' from dual union all
select 34094,'034094','1997-051DD','deb Iridium 33','',66131,'Reentered','2011-12-11 00:00:00','2009-02-23 00:00:00',99.87,721,784,86.43,'Polar' from dual union all
select 34095,'034095','1997-051DE','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',101.13,772,853,86.39,'Polar' from dual union all
select 34096,'034096','1997-051DF','deb Iridium 33','',66131,'Reentered','2009-09-28 00:00:00','2009-02-23 00:00:00',98.63,590,797,86.35,'Polar' from dual union all
select 34097,'034097','1997-051DG','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',102.03,775,935,86.2,'Polar' from dual union all
select 34098,'034098','1997-051DH','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',100.26,745,797,86.15,'Polar' from dual union all
select 34099,'034099','1997-051DJ','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',100.71,776,809,86.38,'Polar' from dual union all
select 341,'000341','62 A Epsilon 2','Altair','Altair X-248-A5 S/N 3023',8140,'In Earth orbit','','1962-08-09 00:00:00',157.64,937,5640,44.78,'Medium' from dual union all
select 3410,'003410','1965-082NJ','deb Transtage 4','',11832,'Reentered','1973-10-05 00:00:00','1968-09-12 00:00:00',104.4,717,1216,31.81,'Intermediate' from dual union all
select 34100,'034100','1997-051DK','deb Iridium 33','',66131,'Reentered','2015-08-13 00:00:00','2009-02-23 00:00:00',101.06,769,849,86.23,'Polar' from dual union all
select 34101,'034101','1997-051DL','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',102.6,765,999,86.41,'Polar' from dual union all
select 34102,'034102','1997-051DM','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',102.79,776,1006,86.4,'Polar' from dual union all
select 34103,'034103','1997-051DN','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',100.83,776,820,86.36,'Polar' from dual union all
select 34104,'034104','1997-051DP','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',100.2,753,783,86.35,'Polar' from dual union all
select 34105,'034105','1997-051DQ','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',100.67,775,806,86.37,'Polar' from dual union all
select 34106,'034106','1997-051DR','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',100.29,761,783,86.37,'Polar' from dual union all
select 34107,'034107','1997-051DS','deb Iridium 33','',66131,'In Earth orbit','','2009-02-23 00:00:00',100.32,763,784,86.39,'Polar' from dual union all
select 34108,'034108','1997-051DT','deb Iridium 33','',66131,'Reentered','2010-12-28 00:00:00','2009-02-23 00:00:00',102.46,780,971,86.37,'Polar' from dual union all
select 34109,'034109','1997-051DU','deb Iridium 33','',66131,'Reentered','2012-07-07 00:00:00','2009-02-23 00:00:00',100.65,780,799,86.35,'Polar' from dual union all
select 3411,'003411','1965-082NK','deb Transtage 4','',11832,'Reentered','1980-10-06 00:00:00','1968-09-16 00:00:00',102.14,723,997,31.74,'Intermediate' from dual union all
select 34110,'034110','1997-051DV','deb Iridium 33','',66131,'Reentered','2011-12-31 00:00:00','2009-02-23 00:00:00',100.4,765,790,86.41,'Polar' from dual union all
select 34111,'034111','2009-009A','Telstar 11N','Telstar 11N',68690,'In Earth orbit','','2009-06-29 00:00:00',1436.14,35777,35797,.01,'Stationary' from dual union all
select 34112,'034112','2009-009B','Blok DM-SLB No. 2L','Blok DM-SLB No. 2L',68690,'In Earth orbit','','2009-03-02 00:00:00',653.15,1338,35779,34.78,'GEO Transfer' from dual union all
select 34113,'034113','1993-036KJ','deb Kosmos-2251','',65138,'Reentered','2010-11-16 00:00:00','2009-02-26 00:00:00',102.73,750,1026,74.04,'Intermediate' from dual union all
select 34114,'034114','1993-036KK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',101.55,773,892,74.07,'Intermediate' from dual union all
select 34115,'034115','1993-036KL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',99.58,701,776,74.03,'Intermediate' from dual union all
select 34116,'034116','1993-036KM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',100.41,766,789,74.05,'Intermediate' from dual union all
select 34117,'034117','1993-036KN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',100.94,754,853,74.05,'Intermediate' from dual;
insert into SATELLITE
select 34118,'034118','1993-036KP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',100.03,652,868,73.98,'Intermediate' from dual union all
select 34119,'034119','1993-036KQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',100.89,770,831,74.06,'Intermediate' from dual union all
select 3412,'003412','1965-082NL','deb Transtage 4','',11832,'Reentered','1970-05-14 00:00:00','1968-09-16 00:00:00',101.89,795,901,30.67,'Intermediate' from dual union all
select 34120,'034120','1993-036KR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',103.28,775,1053,74,'Intermediate' from dual union all
select 34121,'034121','1993-036KS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',104.62,776,1177,74.01,'Intermediate' from dual union all
select 34122,'034122','1993-036KT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',100.38,684,870,74.05,'Intermediate' from dual union all
select 34123,'034123','1993-036KU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',100.57,770,802,74.04,'Intermediate' from dual union all
select 34124,'034124','1993-036KV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',100.52,763,803,74.04,'Intermediate' from dual union all
select 34125,'034125','1993-036KW','deb Kosmos-2251','',65138,'Reentered','2009-03-27 00:00:00','2009-02-26 00:00:00',93.51,224,669,73.93,'Intermediate' from dual union all
select 34126,'034126','1993-036KX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',99.16,661,775,74.02,'Intermediate' from dual union all
select 34127,'034127','1993-036KY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',102.56,774,986,74.18,'Intermediate' from dual union all
select 34128,'034128','1993-036KZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',100.05,698,824,74.03,'Intermediate' from dual union all
select 34129,'034129','1993-036LA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',99.9,711,797,74.05,'Intermediate' from dual union all
select 3413,'003413','1965-082NM','deb Transtage 4','',11832,'Reentered','1969-03-08 00:00:00','1968-09-17 00:00:00',94.87,473,552,32.25,'Intermediate' from dual union all
select 34130,'034130','1993-036LB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',99.89,727,780,74.05,'Intermediate' from dual union all
select 34131,'034131','1993-036LC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',100.43,770,788,74.13,'Intermediate' from dual union all
select 34132,'034132','1993-036LD','deb Kosmos-2251','',65138,'Reentered','2012-05-13 00:00:00','2009-02-26 00:00:00',96.99,449,781,73.98,'Intermediate' from dual union all
select 34133,'034133','1993-036LE','deb Kosmos-2251','',65138,'Reentered','2009-03-28 00:00:00','2009-02-26 00:00:00',94.13,233,720,74.14,'Intermediate' from dual union all
select 34134,'034134','1993-036LF','deb Kosmos-2251','',65138,'Reentered','2016-07-04 00:00:00','2009-02-26 00:00:00',98.91,638,776,73.96,'Intermediate' from dual union all
select 34135,'034135','1993-036LG','deb Kosmos-2251','',65138,'Reentered','2013-06-30 00:00:00','2009-02-26 00:00:00',97.2,474,776,74.08,'Intermediate' from dual union all
select 34136,'034136','1993-036LH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',98.66,606,783,74.03,'Intermediate' from dual union all
select 34137,'034137','1993-036LJ','deb Kosmos-2251','',65138,'Reentered','2011-02-16 00:00:00','2009-02-26 00:00:00',95.94,353,774,74.08,'Intermediate' from dual union all
select 34138,'034138','1993-036LK','deb Kosmos-2251','',65138,'Reentered','2013-08-06 00:00:00','2009-02-26 00:00:00',97.57,509,776,73.99,'Intermediate' from dual union all
select 34139,'034139','1993-036LL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',100.96,754,855,74.04,'Intermediate' from dual union all
select 3414,'003414','1968-079A','Kosmos-242','DS-P1-I No. 4',16806,'Reentered','1968-11-13 00:00:00','1968-09-29 00:00:00',91.12,268,391,70.96,'Intermediate' from dual union all
select 34140,'034140','1993-036LM','deb Kosmos-2251','',65138,'Reentered','2009-04-03 00:00:00','2009-02-26 00:00:00',93.92,231,702,73.98,'Intermediate' from dual union all
select 34141,'034141','1993-036LN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',101.51,773,888,74.02,'Intermediate' from dual union all
select 34142,'034142','1993-036LP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-02-26 00:00:00',99.41,682,778,74,'Intermediate' from dual union all
select 34143,'034143','1997-051DW','deb Iridium 33','',66131,'In Earth orbit','','2009-02-26 00:00:00',100.58,771,801,86.4,'Polar' from dual union all
select 34144,'034144','1997-051DX','deb Iridium 33','',66131,'Reentered','2014-01-11 00:00:00','2009-02-26 00:00:00',99.19,650,790,86.34,'Polar' from dual union all
select 34145,'034145','1997-051DY','deb Iridium 33','',66131,'In Earth orbit','','2009-02-26 00:00:00',101.35,772,874,86.22,'Polar' from dual union all
select 34146,'034146','1997-051DZ','deb Iridium 33','',66131,'In Earth orbit','','2009-02-26 00:00:00',100.77,770,821,86.29,'Polar' from dual union all
select 34147,'034147','1997-051EA','deb Iridium 33','',66131,'In Earth orbit','','2009-02-26 00:00:00',101.63,767,905,86.48,'Polar' from dual union all
select 34148,'034148','1997-051EB','deb Iridium 33','',66131,'In Earth orbit','','2009-02-26 00:00:00',100.64,778,799,86.41,'Polar' from dual union all
select 34149,'034149','1997-051EC','deb Iridium 33','',66131,'Reentered','2012-12-24 00:00:00','2009-02-26 00:00:00',101.16,789,838,86.31,'Polar' from dual union all
select 3415,'003415','1968-079B','S1M','S1M',16806,'Reentered','1968-10-22 00:00:00','1968-09-28 00:00:00',90.89,268,369,70.97,'Intermediate' from dual union all
select 34150,'034150','1997-051ED','deb Iridium 33','',66131,'In Earth orbit','','2009-02-26 00:00:00',101.51,765,896,86.51,'Polar' from dual union all
select 34151,'034151','1997-051EE','deb Iridium 33','',66131,'Reentered','2010-07-21 00:00:00','2009-02-26 00:00:00',101.98,745,960,86.43,'Polar' from dual union all
select 34152,'034152','1997-051EF','deb Iridium 33','',66131,'Reentered','2013-12-19 00:00:00','2009-02-26 00:00:00',99.23,652,792,86.3,'Polar' from dual union all
select 34153,'034153','1997-051EG','deb Iridium 33','',66131,'Reentered','2014-10-04 00:00:00','2009-02-26 00:00:00',99.45,668,796,86.31,'Polar' from dual union all
select 34154,'034154','1997-051EH','deb Iridium 33','',66131,'Reentered','2009-04-28 00:00:00','2009-02-26 00:00:00',101.35,749,897,86.59,'Polar' from dual union all
select 34155,'034155','1997-051EJ','deb Iridium 33','',66131,'In Earth orbit','','2009-02-26 00:00:00',101.01,774,839,86.49,'Polar' from dual union all
select 34156,'034156','1997-051EK','deb Iridium 33','',66131,'Reentered','2016-05-02 00:00:00','2009-02-26 00:00:00',100.93,778,827,86.32,'Polar' from dual union all
select 34157,'034157','1997-051EL','deb Iridium 33','',66131,'Reentered','2016-01-19 00:00:00','2009-02-26 00:00:00',100.57,778,793,86.39,'Polar' from dual union all
select 34158,'034158','1997-051EM','deb Iridium 33','',66131,'Reentered','2013-09-10 00:00:00','2009-02-26 00:00:00',101.26,781,855,86.4,'Polar' from dual union all
select 34159,'034159','1997-051EN','deb Iridium 33','',66131,'In Earth orbit','','2009-02-26 00:00:00',100.75,769,820,86.35,'Polar' from dual union all
select 3416,'003416','1968-069C','ESSA 7 despin weight','',16796,'In Earth orbit','','1968-11-14 00:00:00',113.7,1303,1490,101.77,'Sun-Synch' from dual union all
select 34160,'034160','1997-051EP','deb Iridium 33','',66131,'In Earth orbit','','2009-02-26 00:00:00',101.22,776,857,86.42,'Polar' from dual union all
select 34161,'034161','1997-051EQ','deb Iridium 33','',66131,'Reentered','2009-05-09 00:00:00','2009-02-26 00:00:00',99.12,630,804,86.25,'Polar' from dual union all
select 34162,'034162','1997-051ER','deb Iridium 33','',66131,'Reentered','2013-01-12 00:00:00','2009-02-26 00:00:00',100.53,768,800,86.43,'Polar' from dual union all
select 34163,'034163','1965-108CW','deb Transtage 8','',11858,'In Earth orbit','','2009-02-13 00:00:00',577.34,1229,31950,26.45,'Highly Ellip' from dual union all
select 34164,'034164','1965-108CX','deb Transtage 8','',11858,'In Earth orbit','','2009-02-24 00:00:00',588.19,471,33282,25.12,'Highly Ellip' from dual union all
select 34165,'034165','1965-108CY','deb Transtage 8','',11858,'In Earth orbit','','2009-02-25 00:00:00',458.02,522,26087,26.8,'Highly Ellip' from dual union all
select 34166,'034166','1965-108CZ','deb Transtage 8','[UNID149]',11858,'Lost','','2009-01-18 00:00:00',626.09,376,35354,25.83,'GEO Transfer' from dual union all
select 34167,'034167','1965-108DA','deb Transtage 8','',11858,'In Earth orbit','','2009-02-26 00:00:00',452.62,546,25753,26.22,'Highly Ellip' from dual union all
select 34168,'034168','1965-108DB','deb Transtage 8','',11858,'In Earth orbit','','2009-02-23 00:00:00',546.86,1024,30524,27.58,'Highly Ellip' from dual union all
select 34169,'034169','1965-108DC','deb Transtage 8','',11858,'In Earth orbit','','2009-02-25 00:00:00',546.18,309,31202,27.98,'Highly Ellip' from dual union all
select 3417,'003417','1968-069D','ESSA 7 despin weight','',16796,'In Earth orbit','','1968-11-14 00:00:00',116.18,1458,1560,101.73,'Sun-Synch' from dual union all
select 34170,'034170','1965-108DD','deb Transtage 8','',11858,'In Earth orbit','','2009-02-21 00:00:00',444.25,542,25274,26.22,'Highly Ellip' from dual union all
select 34171,'034171','1965-108DE','deb Transtage 8','',11858,'Lost','','2009-02-15 00:00:00',492.41,472,28084,26.27,'Highly Ellip' from dual union all
select 34172,'034172','1965-108DF','deb Transtage 8','',11858,'In Earth orbit','','2009-02-21 00:00:00',542.2,782,30514,26.8,'Highly Ellip' from dual union all
select 34173,'034173','1965-108DG','deb Transtage 8','',11858,'In Earth orbit','','2009-02-25 00:00:00',536.9,852,30156,27.39,'Highly Ellip' from dual union all
select 34174,'034174','1987-011E','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2009-02-25 00:00:00',100.58,777,795,65.01,'Intermediate' from dual union all
select 34175,'034175','1987-011F','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2009-02-26 00:00:00',100.6,778,796,65.01,'Intermediate' from dual union all
select 34176,'034176','1987-011G','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2009-02-26 00:00:00',100.61,777,798,65.01,'Intermediate' from dual union all
select 34177,'034177','1987-011H','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2009-02-26 00:00:00',100.6,777,797,65.01,'Intermediate' from dual union all
select 34178,'034178','1992-021U','deb Ariane V50','',64733,'In Earth orbit','','2009-02-21 00:00:00',641.91,982,35561,3.94,'GEO Transfer' from dual union all
select 34179,'034179','1992-021V','deb Ariane V50','',64733,'In Earth orbit','','2009-02-24 00:00:00',642.36,1008,35558,4.13,'GEO Transfer' from dual union all
select 3418,'003418','1968-080A','Kosmos-243','Zenit-2M',16807,'Landed','1968-10-04 00:00:00','1968-09-25 00:00:00',89.49,204,294,71.32,'Intermediate' from dual union all
select 34180,'034180','1992-021W','deb Ariane V50','',64733,'In Earth orbit','','2009-02-25 00:00:00',647.29,1245,35573,4.25,'GEO Transfer' from dual union all
select 34181,'034181','1992-041K','deb Ariane V47','',64753,'In Earth orbit','','2009-02-26 00:00:00',592.41,432,33543,5.99,'Highly Ellip' from dual union all
select 34182,'034182','1992-047AS','deb SOZ','',64759,'Reentered','2010-09-18 00:00:00','2009-02-26 00:00:00',140.7,502,4664,64.72,'Medium' from dual union all
select 34183,'034183','1992-047AT','deb SOZ','',64759,'Reentered','2015-08-01 00:00:00','2009-02-26 00:00:00',138.05,426,4514,65.06,'Medium' from dual union all
select 34184,'034184','1992-072Q','deb Ariane V54','',64784,'In Earth orbit','','2009-02-26 00:00:00',487.25,257,28010,6.94,'Highly Ellip' from dual union all
select 34185,'034185','1992-072R','deb Ariane V54','',64784,'In Earth orbit','','2009-02-25 00:00:00',613.45,468,34607,7.93,'GEO Transfer' from dual union all
select 34186,'034186','1992-072S','deb Ariane V54','',64784,'In Earth orbit','','2009-02-23 00:00:00',546.42,429,31095,7.24,'Highly Ellip' from dual union all
select 34187,'034187','1992-072T','deb Ariane V54','',64784,'Lost','','2009-02-22 00:00:00',441.23,441,25200,7.27,'Highly Ellip' from dual union all
select 34188,'034188','1992-072U','deb Ariane V54','',64784,'Reentered','2016-01-19 00:00:00','2009-02-25 00:00:00',389.15,397,22160,12.82,'Highly Ellip' from dual union all
select 34189,'034189','1992-072V','deb Ariane V54','',64784,'In Earth orbit','','2009-02-26 00:00:00',496.01,796,27961,6.71,'Highly Ellip' from dual union all
select 3419,'003419','1968-080B','11A57I','11A57I',16807,'Reentered','1968-09-29 00:00:00','1968-09-25 00:00:00',89.04,193,262,71.33,'Intermediate' from dual union all
select 34190,'034190','1992-072W','deb Ariane V54','',64784,'In Earth orbit','','2009-02-26 00:00:00',485.01,639,27502,7.11,'Highly Ellip' from dual union all
select 34191,'034191','1992-072X','deb Ariane V54','',64784,'In Earth orbit','','2009-02-26 00:00:00',493.13,824,27772,7.74,'Highly Ellip' from dual union all
select 34192,'034192','1992-072Y','deb Ariane V54','',64784,'In Earth orbit','','2009-02-26 00:00:00',468.23,439,26753,7.71,'Highly Ellip' from dual union all
select 34193,'034193','1992-072Z','deb Ariane V54','',64784,'In Earth orbit','','2009-02-17 00:00:00',464.89,412,26590,8.5,'Highly Ellip' from dual union all
select 34194,'034194','1992-072AA','deb Ariane V54','',64784,'In Earth orbit','','2009-02-24 00:00:00',417.25,465,23772,10.29,'Highly Ellip' from dual union all
select 34195,'034195','1992-072AB','deb Ariane V54','',64784,'In Earth orbit','','2009-02-26 00:00:00',488.95,671,27691,5.49,'Highly Ellip' from dual union all
select 34196,'034196','1992-072AC','deb Ariane V54','',64784,'In Earth orbit','','2009-02-25 00:00:00',495.53,952,27778,7.58,'Highly Ellip' from dual union all
select 34197,'034197','1992-072AD','deb Ariane V54','',64784,'In Earth orbit','','2009-02-26 00:00:00',382.33,398,21745,13.44,'Highly Ellip' from dual union all
select 34198,'034198','1992-072AE','deb Ariane V54','',64784,'In Earth orbit','','2009-02-25 00:00:00',481.03,599,27318,8.46,'Highly Ellip' from dual union all
select 34199,'034199','1992-072AF','deb Ariane V54','',64784,'In Earth orbit','','2009-02-25 00:00:00',496.35,1010,27766,6.74,'Highly Ellip' from dual union all
select 342,'000342','62 A Zeta 1','FTV 2403','SAMOS E-6 No. 3',8159,'Reentered Attached','1962-07-25 00:00:00','1962-07-20 00:00:00',88.7,180,241,96.13,'Sun-Synch' from dual union all
select 3420,'003420','1965-082NN','deb Transtage 4','',11832,'Reentered','1981-04-13 00:00:00','1968-09-30 00:00:00',101.4,722,928,32.6,'Intermediate' from dual union all
select 34200,'034200','1997-070K','SOZ debris','',66150,'Reentered','2010-11-04 00:00:00','2009-02-05 00:00:00',256.54,695,13297,47.64,'Medium' from dual union all
select 34201,'034201','1997-070L','SOZ debris','',66150,'Reentered','2013-09-01 00:00:00','2009-02-08 00:00:00',247.14,949,12385,48.15,'Medium' from dual union all
select 34202,'034202','1997-070M','SOZ debris','',66150,'In Earth orbit','','2009-02-12 00:00:00',225.37,763,11016,48.01,'Medium' from dual union all
select 34203,'034203','1999-035L','FUSE deb','',66716,'Reentered','2011-01-27 00:00:00','2009-02-26 00:00:00',98.34,671,687,24.98,'Equatorial' from dual union all
select 34204,'034204','1999-057RF','ZY-1 debris','',66738,'In Earth orbit','','2009-02-24 00:00:00',100.26,765,777,98.18,'Sun-Synch' from dual union all
select 34205,'034205','1999-057RG','ZY-1 debris','',66738,'In Earth orbit','','2009-02-22 00:00:00',99.46,726,740,98.46,'Sun-Synch' from dual union all
select 34206,'034206','1999-057RH','ZY-1 debris','',66738,'Reentered','2012-08-20 00:00:00','2009-02-26 00:00:00',99.5,725,745,98.25,'Sun-Synch' from dual union all
select 34207,'034207','1999-057RJ','ZY-1 debris','',66738,'In Earth orbit','','2009-02-26 00:00:00',100.27,750,793,98.24,'Sun-Synch' from dual;
insert into SATELLITE
select 34208,'034208','1999-057RK','ZY-1 debris','',66738,'In Earth orbit','','2009-02-26 00:00:00',100.24,763,776,98.17,'Sun-Synch' from dual union all
select 34209,'034209','1999-057RL','ZY-1 debris','',66738,'In Earth orbit','','2009-02-22 00:00:00',100.33,768,781,98.2,'Sun-Synch' from dual union all
select 3421,'003421','1965-082NP','deb Transtage 4','',11832,'Reentered','1969-01-13 00:00:00','1968-09-24 00:00:00',101.42,676,976,31.55,'Intermediate' from dual union all
select 34210,'034210','1999-057RM','ZY-1 debris','',66738,'Reentered','2015-03-12 00:00:00','2009-02-24 00:00:00',100.26,764,778,98.19,'Sun-Synch' from dual union all
select 34211,'034211','1999-057RN','CZ-4 debris','',66738,'In Earth orbit','','2009-02-21 00:00:00',99.55,669,806,98.08,'Sun-Synch' from dual union all
select 34212,'034212','1999-057RP','ZY-1 debris','',66738,'Reentered','2011-11-12 00:00:00','2009-02-17 00:00:00',99.52,733,738,98.34,'Sun-Synch' from dual union all
select 34213,'034213','1999-057RQ','ZY-1 debris','',66738,'In Earth orbit','','2009-02-24 00:00:00',100.05,750,771,98.17,'Sun-Synch' from dual union all
select 34214,'034214','1999-057RR','ZY-1 debris','',66738,'In Earth orbit','','2009-02-26 00:00:00',99.95,742,771,98.17,'Sun-Synch' from dual union all
select 34215,'034215','1999-057RS','ZY-1 debris','',66738,'In Earth orbit','','2009-02-22 00:00:00',100.36,770,782,98.18,'Sun-Synch' from dual union all
select 34216,'034216','1999-057RT','ZY-1 debris','',66738,'In Earth orbit','','2009-02-26 00:00:00',98.83,692,713,98.6,'Sun-Synch' from dual union all
select 34217,'034217','1999-057RU','ZY-1 debris','',66738,'In Earth orbit','','2009-02-24 00:00:00',100.66,765,815,98.17,'Sun-Synch' from dual union all
select 34218,'034218','1999-057RV','ZY-1 debris','',66738,'Reentered','2011-09-19 00:00:00','2009-02-20 00:00:00',99.5,712,758,98.29,'Sun-Synch' from dual union all
select 34219,'034219','2001-049PP','deb PSLV-C3 PS4','',67202,'Reentered','2014-11-10 00:00:00','2009-02-14 00:00:00',96.77,584,623,97.81,'Sun-Synch' from dual union all
select 3422,'003422','1965-082NQ','deb Transtage 4','',11832,'Reentered','1969-09-24 00:00:00','1968-09-24 00:00:00',96.92,575,648,32.28,'Intermediate' from dual union all
select 34220,'034220','2001-049PQ','deb PSLV-C3 PS4','',67202,'Lost','','2009-02-25 00:00:00',95.99,550,583,98.2,'Sun-Synch' from dual union all
select 34221,'034221','2001-049PR','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-02-26 00:00:00',96.36,519,649,98.53,'Sun-Synch' from dual union all
select 34222,'034222','2001-049PS','deb PSLV-C3 PS4','',67202,'Reentered','2014-12-12 00:00:00','2009-02-23 00:00:00',95.7,492,613,97.37,'Sun-Synch' from dual union all
select 34223,'034223','2002-024H','CZ-4B sep motor cover','',67400,'In Earth orbit','','2009-02-27 00:00:00',102.9,793,999,98.78,'Sun-Synch' from dual union all
select 34224,'034224','2006-006AP','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-27 00:00:00',276.61,580,14790,52.18,'Highly Ellip' from dual union all
select 34225,'034225','2006-006AQ','deb Briz-M 88515','',68178,'Reentered','2016-06-30 00:00:00','2009-02-27 00:00:00',205.58,644,9677,53.21,'Medium' from dual union all
select 34226,'034226','2006-006AR','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-23 00:00:00',273.52,468,14691,51.71,'Highly Ellip' from dual union all
select 34227,'034227','2006-006AS','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-21 00:00:00',270.44,604,14345,51.98,'Medium' from dual union all
select 34228,'034228','2006-006AT','deb Briz-M 88515','',68178,'Lost','','2009-02-20 00:00:00',257.36,359,13689,51.81,'Medium' from dual union all
select 34229,'034229','2006-006AU','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-27 00:00:00',268.8,405,14433,51.51,'Highly Ellip' from dual union all
select 3423,'003423','1965-082NR','deb Transtage 4','',11832,'Reentered','1980-07-13 00:00:00','1968-09-24 00:00:00',98.65,653,735,32.13,'Intermediate' from dual union all
select 34230,'034230','2006-006AV','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-26 00:00:00',274.76,482,14762,51.81,'Highly Ellip' from dual union all
select 34231,'034231','2006-006AW','deb Briz-M 88515','',68178,'In Earth orbit','','2009-01-27 00:00:00',286.98,633,15435,52.2,'Highly Ellip' from dual union all
select 34232,'034232','2006-006AX','deb Briz-M 88515','',68178,'Lost','','2009-02-09 00:00:00',268.57,413,14408,51.73,'Highly Ellip' from dual union all
select 34233,'034233','2006-006AY','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-26 00:00:00',279.57,501,15069,52.23,'Highly Ellip' from dual union all
select 34234,'034234','2006-006AZ','deb Briz-M 88515','',68178,'Lost','','2009-02-05 00:00:00',272.61,431,14666,51.68,'Highly Ellip' from dual union all
select 34235,'034235','2006-006BA','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-27 00:00:00',274.52,492,14735,51.75,'Highly Ellip' from dual union all
select 34236,'034236','2006-006BB','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-11 00:00:00',270.84,417,14560,51.73,'Highly Ellip' from dual union all
select 34237,'034237','2006-006BC','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-27 00:00:00',275.85,498,14819,51.75,'Highly Ellip' from dual union all
select 34238,'034238','2006-006BD','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-20 00:00:00',282.48,502,15263,51.54,'Highly Ellip' from dual union all
select 34239,'034239','2006-006BE','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-24 00:00:00',158.3,679,5952,50.39,'Medium' from dual union all
select 3424,'003424','1965-082NS','deb Transtage 4','',11832,'Reentered','1979-04-22 00:00:00','1968-09-24 00:00:00',99.91,724,785,31.54,'Intermediate' from dual union all
select 34240,'034240','2006-006BF','deb Briz-M 88515','',68178,'Reentered','2009-09-18 00:00:00','2009-02-01 00:00:00',236.23,1041,11519,52.05,'Medium' from dual union all
select 34241,'034241','2006-006BG','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-27 00:00:00',270.53,600,14356,51.85,'Medium' from dual union all
select 34242,'034242','2006-006BH','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-13 00:00:00',289.86,467,15793,52.28,'Highly Ellip' from dual union all
select 34243,'034243','2006-006BJ','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-26 00:00:00',276.29,497,14850,51.67,'Highly Ellip' from dual union all
select 34244,'034244','2006-006BK','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-24 00:00:00',268.63,524,14301,52.08,'Medium' from dual union all
select 34245,'034245','2006-006BL','deb Briz-M 88515','',68178,'In Earth orbit','2013-04-01 00:00:00','2009-01-06 00:00:00',271.39,425,14589,51.94,'Highly Ellip' from dual union all
select 34246,'034246','2006-006BM','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-26 00:00:00',274.61,450,14784,51.66,'Highly Ellip' from dual union all
select 34247,'034247','2006-006BN','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-26 00:00:00',275.04,533,14730,52.2,'Highly Ellip' from dual union all
select 34248,'034248','2006-006BP','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-22 00:00:00',269.24,405,14462,51.75,'Highly Ellip' from dual union all
select 34249,'034249','2006-006BQ','deb Briz-M 88515','',68178,'Reentered','2014-08-20 00:00:00','2009-02-27 00:00:00',256.87,400,13614,50.9,'Medium' from dual union all
select 3425,'003425','1965-082NT','deb Transtage 4','',11832,'Reentered','1981-10-17 00:00:00','1968-09-24 00:00:00',101.5,709,950,32.81,'Intermediate' from dual union all
select 34250,'034250','2006-006BR','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-27 00:00:00',277.15,523,14883,51.75,'Highly Ellip' from dual union all
select 34251,'034251','2006-006BS','deb Briz-M 88515','',68178,'Reentered','2010-02-10 00:00:00','2009-02-27 00:00:00',177.91,255,7946,50.98,'Medium' from dual union all
select 34252,'034252','2006-006BT','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-27 00:00:00',276.19,506,14835,51.69,'Highly Ellip' from dual union all
select 34253,'034253','2006-006BU','deb Briz-M 88515','',68178,'Reentered','2013-10-16 00:00:00','2009-02-27 00:00:00',221.37,283,11205,51.88,'Medium' from dual union all
select 34254,'034254','2006-006BV','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-27 00:00:00',271.73,446,14592,51.67,'Highly Ellip' from dual union all
select 34255,'034255','2006-006BW','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-25 00:00:00',280.01,501,15098,52.22,'Highly Ellip' from dual union all
select 34256,'034256','2006-006BX','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-08 00:00:00',285.16,570,15376,52.08,'Highly Ellip' from dual union all
select 34257,'034257','2006-006BY','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-20 00:00:00',280.28,598,15020,52.19,'Highly Ellip' from dual union all
select 34258,'034258','2006-006BZ','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-23 00:00:00',281.13,554,15120,51.98,'Highly Ellip' from dual union all
select 34259,'034259','2006-006CA','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-27 00:00:00',283.61,1048,14793,50.73,'Medium' from dual union all
select 3426,'003426','1965-082NU','deb Transtage 4','',11832,'Reentered','1981-02-18 00:00:00','1968-09-24 00:00:00',99.53,718,754,31.88,'Intermediate' from dual union all
select 34260,'034260','2006-006CB','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-25 00:00:00',273.52,659,14500,52.06,'Medium' from dual union all
select 34261,'034261','2006-006CC','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-21 00:00:00',264.94,487,14085,52.21,'Medium' from dual union all
select 34262,'034262','2006-006CD','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-27 00:00:00',279.17,613,14929,51.8,'Highly Ellip' from dual union all
select 34263,'034263','2006-006CE','deb Briz-M 88515','',68178,'In Earth orbit','','2009-02-21 00:00:00',272.67,455,14646,51.74,'Highly Ellip' from dual union all
select 34264,'034264','2009-010A','Raduga-1','Globus-1 No. 18L',68691,'In Earth orbit','','2009-03-01 00:00:00',1443.25,35907,35946,1.49,'Drift GEO' from dual union all
select 34265,'034265','2009-010B','Blok DM-2 No 107L','11S861 No. 107L',68691,'In Earth orbit','','2009-03-01 00:00:00',1440.18,35824,35909,1.51,'Drift GEO' from dual union all
select 34266,'034266','2009-010C','Perekhodnik','Sredniy perekhodnik',68691,'Reentered','2009-03-01 00:00:00','2009-03-03 00:00:00',87.6,153,159,51.65,'Intermediate' from dual union all
select 34267,'034267','2009-010D','8S812','8S812',68691,'Reentered','2009-03-03 00:00:00','2009-02-28 00:00:00',88.15,181,185,51.64,'Intermediate' from dual union all
select 34268,'034268','2009-010E','SOZ','SOZ',68691,'Reentered','2009-09-14 00:00:00','2009-03-09 00:00:00',635.98,310,35929,47.52,'GEO Transfer' from dual union all
select 34269,'034269','2009-010F','SOZ','SOZ',68691,'Reentered','2009-08-11 00:00:00','2009-03-09 00:00:00',634.87,247,35935,47.52,'GEO Transfer' from dual union all
select 3427,'003427','1965-082NV','deb Transtage 4','',11832,'Reentered','1980-05-28 00:00:00','1968-09-30 00:00:00',109.92,684,1762,30.5,'Intermediate' from dual union all
select 34270,'034270','1993-036LQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',99.6,690,789,74.02,'Intermediate' from dual union all
select 34271,'034271','1993-036LR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.88,770,830,74.04,'Intermediate' from dual union all
select 34272,'034272','1993-036LS','deb Kosmos-2251','',65138,'Reentered','2009-04-19 00:00:00','2009-03-03 00:00:00',94.28,236,732,73.8,'Intermediate' from dual union all
select 34273,'034273','1993-036LT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.13,748,782,74.05,'Intermediate' from dual union all
select 34274,'034274','1993-036LU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.22,743,795,74.02,'Intermediate' from dual union all
select 34275,'034275','1993-036LV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.58,769,803,74.04,'Intermediate' from dual union all
select 34276,'034276','1993-036LW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.06,738,784,74.03,'Intermediate' from dual union all
select 34277,'034277','1993-036LX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',99.64,686,796,73.72,'Intermediate' from dual union all
select 34278,'034278','1993-036LY','deb Kosmos-2251','',65138,'Reentered','2012-10-29 00:00:00','2009-03-03 00:00:00',97.53,502,779,74.2,'Intermediate' from dual union all
select 34279,'034279','1993-036LZ','deb Kosmos-2251','',65138,'Reentered','2012-10-27 00:00:00','2009-03-03 00:00:00',99.11,620,812,74.02,'Intermediate' from dual union all
select 3428,'003428','1968-081A','OV2-5','OV2-5',16808,'In Earth orbit','','1968-11-26 00:00:00',1417.96,35064,35797,2.86,'Drift GEO' from dual union all
select 34280,'034280','1993-036MA','deb Kosmos-2251','',65138,'Reentered','2011-10-10 00:00:00','2009-03-03 00:00:00',95.96,356,773,74.15,'Intermediate' from dual union all
select 34281,'034281','1993-036MB','deb Kosmos-2251','',65138,'Reentered','2011-10-06 00:00:00','2009-03-03 00:00:00',96.82,434,779,74.07,'Intermediate' from dual union all
select 34282,'034282','1993-036MC','deb Kosmos-2251','',65138,'Reentered','2009-03-31 00:00:00','2009-03-03 00:00:00',93.08,203,648,73.92,'Intermediate' from dual union all
select 34283,'034283','1993-036MD','deb Kosmos-2251','',65138,'Reentered','2010-02-01 00:00:00','2009-03-03 00:00:00',95.56,310,782,74.2,'Intermediate' from dual union all
select 34284,'034284','1993-036ME','deb Kosmos-2251','',65138,'Reentered','2012-06-07 00:00:00','2009-03-03 00:00:00',95.87,345,776,74.02,'Intermediate' from dual union all
select 34285,'034285','1993-036MF','deb Kosmos-2251','',65138,'Reentered','2009-12-11 00:00:00','2009-03-03 00:00:00',95.94,344,784,73.96,'Intermediate' from dual union all
select 34286,'034286','1993-036MG','deb Kosmos-2251','',65138,'Reentered','2011-01-13 00:00:00','2009-03-03 00:00:00',96.08,346,796,73.93,'Intermediate' from dual union all
select 34287,'034287','1993-036MH','deb Kosmos-2251','',65138,'Reentered','2009-12-30 00:00:00','2009-03-03 00:00:00',96.79,441,769,74.08,'Intermediate' from dual union all
select 34288,'034288','1993-036MJ','deb Kosmos-2251','',65138,'Reentered','2011-04-15 00:00:00','2009-03-03 00:00:00',96.89,423,797,73.96,'Intermediate' from dual union all
select 34289,'034289','1993-036MK','deb Kosmos-2251','',65138,'Reentered','2010-02-03 00:00:00','2009-03-03 00:00:00',97.43,474,798,73.93,'Intermediate' from dual union all
select 3429,'003429','1968-081B','ERS 28','OV5-2',16808,'Reentered','1971-02-15 00:00:00','1968-12-27 00:00:00',602.21,174,34315,25.99,'GEO Transfer' from dual union all
select 34290,'034290','1993-036ML','deb Kosmos-2251','',65138,'Reentered','2010-01-15 00:00:00','2009-03-03 00:00:00',97.4,498,771,74.17,'Intermediate' from dual union all
select 34291,'034291','1993-036MM','deb Kosmos-2251','',65138,'Reentered','2014-12-09 00:00:00','2009-03-03 00:00:00',97.7,509,788,74.19,'Intermediate' from dual union all
select 34292,'034292','1993-036MN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',99.01,638,785,74.07,'Intermediate' from dual union all
select 34293,'034293','1993-036MP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',99.07,608,821,73.97,'Intermediate' from dual union all
select 34294,'034294','1993-036MQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',99.81,619,880,73.98,'Intermediate' from dual union all
select 34295,'034295','1993-036MR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',99.73,666,826,74.03,'Intermediate' from dual union all
select 34296,'034296','1993-036MS','deb Kosmos-2251','',65138,'Reentered','2016-09-12 00:00:00','2009-03-03 00:00:00',99.79,540,957,74.07,'Intermediate' from dual union all
select 34297,'034297','1993-036MT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.44,767,792,74.05,'Intermediate' from dual union all
select 34298,'034298','1993-036MU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.55,745,824,74.07,'Intermediate' from dual;
insert into SATELLITE
select 34299,'034299','1993-036MV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.8,587,1006,74.04,'Intermediate' from dual union all
select 343,'000343','62 A Zeta 2','FTV 2403 RV','E-6 RV',8159,'Reentered','1962-07-27 00:00:00','1962-07-20 00:00:00',88.67,184,233,96.08,'Sun-Synch' from dual union all
select 3430,'003430','1968-081C','ERS 21','OV5-4',16808,'In Earth orbit','','1995-07-19 00:00:00',1436.44,35765,35822,13.58,'Inclined GEO' from dual union all
select 34300,'034300','1993-036MW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.87,665,935,74.03,'Intermediate' from dual union all
select 34301,'034301','1993-036MX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.91,771,833,74.03,'Intermediate' from dual union all
select 34302,'034302','1993-036MY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',101.15,745,882,74.19,'Intermediate' from dual union all
select 34303,'034303','1993-036MZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',101.17,664,965,74.02,'Intermediate' from dual union all
select 34304,'034304','1993-036NA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',101.77,719,966,74.02,'Intermediate' from dual union all
select 34305,'034305','1993-036NB','deb Kosmos-2251','',65138,'Reentered','2011-04-18 00:00:00','2009-03-03 00:00:00',99.12,639,795,74.05,'Intermediate' from dual union all
select 34306,'034306','1993-036NC','deb Kosmos-2251','',65138,'Reentered','2009-04-26 00:00:00','2009-03-03 00:00:00',97.31,556,703,74.16,'Intermediate' from dual union all
select 34307,'034307','1993-036ND','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',106.95,776,1395,74.06,'Intermediate' from dual union all
select 34308,'034308','1993-036NE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',99.73,687,805,74,'Intermediate' from dual union all
select 34309,'034309','1993-036NF','deb Kosmos-2251','',65138,'Reentered','2013-04-06 00:00:00','2009-03-03 00:00:00',96.67,424,774,74.11,'Intermediate' from dual union all
select 3431,'003431','1968-081D','LES 6','LES 6',16808,'In Earth orbit','','1977-03-04 00:00:00',1436.6,35654,35939,4.43,'Inclined Drift' from dual union all
select 34310,'034310','1993-036NG','deb Kosmos-2251','',65138,'Reentered','2013-04-28 00:00:00','2009-03-03 00:00:00',97.41,482,788,73.89,'Intermediate' from dual union all
select 34311,'034311','1993-036NH','deb Kosmos-2251','',65138,'Reentered','2011-03-25 00:00:00','2009-03-03 00:00:00',95.55,310,781,73.96,'Intermediate' from dual union all
select 34312,'034312','1993-036NJ','deb Kosmos-2251','',65138,'Reentered','2011-10-16 00:00:00','2009-03-03 00:00:00',97.15,466,779,74.06,'Intermediate' from dual union all
select 34313,'034313','1993-036NK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.26,722,820,74.01,'Intermediate' from dual union all
select 34314,'034314','1993-036NL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',102.19,775,950,74.03,'Intermediate' from dual union all
select 34315,'034315','1993-036NM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',99.05,629,798,74.13,'Intermediate' from dual union all
select 34316,'034316','1993-036NN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100,741,776,74.09,'Intermediate' from dual union all
select 34317,'034317','1993-036NP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',105.91,775,1299,73.93,'Intermediate' from dual union all
select 34318,'034318','1993-036NQ','deb Kosmos-2251','',65138,'Reentered','2014-11-17 00:00:00','2009-03-03 00:00:00',99.06,627,801,74.09,'Intermediate' from dual union all
select 34319,'034319','1993-036NR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.63,761,815,74.05,'Intermediate' from dual union all
select 3432,'003432','1968-081E','Transtage 5','Transtage 5',16808,'In Earth orbit','','1972-06-18 00:00:00',1419.26,35106,35807,1.36,'Drift GEO' from dual union all
select 34320,'034320','1993-036NS','deb Kosmos-2251','',65138,'Reentered','2012-07-26 00:00:00','2009-03-03 00:00:00',98.85,578,830,74.04,'Intermediate' from dual union all
select 34321,'034321','1993-036NT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.24,756,784,74.04,'Intermediate' from dual union all
select 34322,'034322','1993-036NU','deb Kosmos-2251','',65138,'Reentered','2014-09-12 00:00:00','2009-03-03 00:00:00',97.28,481,776,74.01,'Intermediate' from dual union all
select 34323,'034323','1993-036NV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.82,704,891,73.98,'Intermediate' from dual union all
select 34324,'034324','1993-036NW','deb Kosmos-2251','',65138,'Reentered','2015-01-14 00:00:00','2009-03-03 00:00:00',99.37,675,782,74.1,'Intermediate' from dual union all
select 34325,'034325','1993-036NX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',103.83,772,1108,73.88,'Intermediate' from dual union all
select 34326,'034326','1993-036NY','deb Kosmos-2251','',65138,'Reentered','2011-02-08 00:00:00','2009-03-03 00:00:00',96.69,395,805,74.22,'Intermediate' from dual union all
select 34327,'034327','1993-036NZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',99.44,686,778,74.01,'Intermediate' from dual union all
select 34328,'034328','1993-036PA','deb Kosmos-2251','',65138,'Reentered','2011-10-28 00:00:00','2009-03-03 00:00:00',96.52,408,776,74.05,'Intermediate' from dual union all
select 34329,'034329','1993-036PB','deb Kosmos-2251','',65138,'Reentered','2012-03-30 00:00:00','2009-03-03 00:00:00',96.86,432,785,73.95,'Intermediate' from dual union all
select 3433,'003433','1965-082NW','deb Transtage 4','',11832,'Reentered','1978-10-19 00:00:00','1967-07-16 00:00:00',100.67,634,947,31.86,'Intermediate' from dual union all
select 34330,'034330','1993-036PC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.22,754,784,74.05,'Intermediate' from dual union all
select 34331,'034331','1993-036PD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.85,753,844,74.04,'Intermediate' from dual union all
select 34332,'034332','1993-036PE','deb Kosmos-2251','',65138,'Reentered','2014-07-03 00:00:00','2009-03-03 00:00:00',99.36,669,787,74.02,'Intermediate' from dual union all
select 34333,'034333','1993-036PF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',99.42,657,804,74.06,'Intermediate' from dual union all
select 34334,'034334','1993-036PG','deb Kosmos-2251','',65138,'Reentered','2010-03-05 00:00:00','2009-03-03 00:00:00',95.57,320,773,74.07,'Intermediate' from dual union all
select 34335,'034335','1993-036PH','deb Kosmos-2251','',65138,'Reentered','2011-01-31 00:00:00','2009-03-03 00:00:00',95.94,354,774,73.94,'Intermediate' from dual union all
select 34336,'034336','1993-036PJ','deb Kosmos-2251','',65138,'Reentered','2011-04-01 00:00:00','2009-03-03 00:00:00',97.05,405,830,74.12,'Intermediate' from dual union all
select 34337,'034337','1993-036PK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',98.27,548,804,74,'Intermediate' from dual union all
select 34338,'034338','1993-036PL','deb Kosmos-2251','',65138,'Reentered','2010-06-25 00:00:00','2009-03-03 00:00:00',98.33,555,803,74,'Intermediate' from dual union all
select 34339,'034339','1993-036PM','deb Kosmos-2251','',65138,'Reentered','2016-04-26 00:00:00','2009-03-03 00:00:00',99.64,609,874,74.06,'Intermediate' from dual union all
select 3434,'003434','1965-082NX','deb Transtage 4','',11832,'Reentered','1982-04-13 00:00:00','1968-09-25 00:00:00',99.97,702,813,32.75,'Intermediate' from dual union all
select 34340,'034340','1993-036PN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',105.24,775,1236,74.08,'Intermediate' from dual union all
select 34341,'034341','1993-036PP','deb Kosmos-2251','',65138,'Reentered','2015-11-26 00:00:00','2009-03-03 00:00:00',100.42,754,802,74.05,'Intermediate' from dual union all
select 34342,'034342','1993-036PQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',99.36,628,828,74.19,'Intermediate' from dual union all
select 34343,'034343','1993-036PR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',100.35,751,800,74.04,'Intermediate' from dual union all
select 34344,'034344','1993-036PS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',103.29,775,1053,73.99,'Intermediate' from dual union all
select 34345,'034345','1993-036PT','deb Kosmos-2251','',65138,'Reentered','2014-11-15 00:00:00','2009-03-03 00:00:00',98.93,526,888,74.06,'Intermediate' from dual union all
select 34346,'034346','1993-036PU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-03 00:00:00',103.93,770,1118,74.01,'Intermediate' from dual union all
select 34347,'034347','1993-036PV','deb Kosmos-2251','',65138,'Reentered','2010-05-31 00:00:00','2009-03-03 00:00:00',96.01,358,777,73.97,'Intermediate' from dual union all
select 34348,'034348','1993-036PW','deb Kosmos-2251','',65138,'Reentered','2009-04-18 00:00:00','2009-03-03 00:00:00',94.77,292,722,74.02,'Intermediate' from dual union all
select 34349,'034349','1993-036PX','deb Kosmos-2251','',65138,'Reentered','2009-03-12 00:00:00','2009-03-03 00:00:00',92.3,199,575,73.98,'Intermediate' from dual union all
select 3435,'003435','1965-082NY','deb Transtage 4','',11832,'Reentered','1979-12-20 00:00:00','1968-09-25 00:00:00',99.34,666,789,32.55,'Intermediate' from dual union all
select 34350,'034350','1997-051ES','deb Iridium 33','',66131,'In Earth orbit','','2009-03-03 00:00:00',100.27,753,789,86.33,'Polar' from dual union all
select 34351,'034351','1997-051ET','deb Iridium 33','',66131,'In Earth orbit','','2009-03-03 00:00:00',100.33,734,815,86.43,'Polar' from dual union all
select 34352,'034352','1997-051EU','deb Iridium 33','',66131,'Reentered','2012-09-21 00:00:00','2009-03-03 00:00:00',100.71,777,808,86.39,'Polar' from dual union all
select 34353,'034353','1997-051EV','deb Iridium 33','',66131,'Reentered','2013-04-30 00:00:00','2009-03-03 00:00:00',100.9,777,826,86.31,'Polar' from dual union all
select 34354,'034354','1997-051EW','deb Iridium 33','',66131,'In Earth orbit','','2009-03-03 00:00:00',100.84,750,846,86.23,'Polar' from dual union all
select 34355,'034355','1997-051EX','deb Iridium 33','',66131,'Reentered','2010-11-18 00:00:00','2009-03-03 00:00:00',100.34,765,785,86.33,'Polar' from dual union all
select 34356,'034356','1997-051EY','deb Iridium 33','',66131,'Reentered','2015-08-02 00:00:00','2009-03-03 00:00:00',100.8,777,815,86.39,'Polar' from dual union all
select 34357,'034357','1997-051EZ','deb Iridium 33','',66131,'Reentered','2014-02-13 00:00:00','2009-03-03 00:00:00',101.76,777,907,86.42,'Polar' from dual union all
select 34358,'034358','1997-051FA','deb Iridium 33','',66131,'In Earth orbit','','2009-03-03 00:00:00',100.65,776,803,86.34,'Polar' from dual union all
select 34359,'034359','1997-051FB','deb Iridium 33','',66131,'In Earth orbit','','2009-03-03 00:00:00',100.4,742,813,86.37,'Polar' from dual union all
select 3436,'003436','1965-082NZ','deb Transtage 4','',11832,'Reentered','1969-03-08 00:00:00','1969-02-06 00:00:00',94.74,506,506,31.12,'Intermediate' from dual union all
select 34360,'034360','1997-051FC','deb Iridium 33','',66131,'Reentered','2015-11-09 00:00:00','2009-03-03 00:00:00',100.4,767,788,86.36,'Polar' from dual union all
select 34361,'034361','1997-051FD','deb Iridium 33','',66131,'In Earth orbit','','2009-03-03 00:00:00',101.16,777,851,86.39,'Polar' from dual union all
select 34362,'034362','1997-051FE','deb Iridium 33','',66131,'Reentered','2014-03-10 00:00:00','2009-03-03 00:00:00',100.72,776,810,86.39,'Polar' from dual union all
select 34363,'034363','1997-051FF','deb Iridium 33','',66131,'In Earth orbit','','2009-03-03 00:00:00',101.36,777,870,86.37,'Polar' from dual union all
select 34364,'034364','1997-051FG','deb Iridium 33','',66131,'Reentered','2015-04-20 00:00:00','2009-03-03 00:00:00',100.72,778,808,86.38,'Polar' from dual union all
select 34365,'034365','1997-051FH','deb Iridium 33','',66131,'Reentered','2013-02-26 00:00:00','2009-03-03 00:00:00',100.81,777,817,86.3,'Polar' from dual union all
select 34366,'034366','1997-051FJ','deb Iridium 33','',66131,'In Earth orbit','','2009-03-03 00:00:00',100.81,769,825,86.38,'Polar' from dual union all
select 34367,'034367','1997-051FK','deb Iridium 33','',66131,'In Earth orbit','','2009-03-03 00:00:00',101.03,773,842,86.43,'Polar' from dual union all
select 34368,'034368','1997-051FL','deb Iridium 33','',66131,'In Earth orbit','','2009-03-03 00:00:00',101.94,777,924,86.4,'Polar' from dual union all
select 34369,'034369','1997-051FM','deb Iridium 33','',66131,'Reentered','2012-01-20 00:00:00','2009-03-03 00:00:00',100.87,778,822,86.35,'Polar' from dual union all
select 3437,'003437','1965-082PA','deb Transtage 4','',11832,'Reentered','1981-11-28 00:00:00','1968-09-25 00:00:00',101.87,700,994,31.98,'Intermediate' from dual union all
select 34370,'034370','1997-051FN','deb Iridium 33','',66131,'Reentered','2012-04-28 00:00:00','2009-03-03 00:00:00',100.68,789,793,86.35,'Polar' from dual union all
select 34371,'034371','1997-051FP','deb Iridium 33','',66131,'Reentered','2010-04-02 00:00:00','2009-03-03 00:00:00',99.31,656,796,86.34,'Polar' from dual union all
select 34372,'034372','1997-051FQ','deb Iridium 33','',66131,'Reentered','2012-12-19 00:00:00','2009-03-03 00:00:00',99.49,691,778,86.49,'Polar' from dual union all
select 34373,'034373','1997-051FR','deb Iridium 33','',66131,'Reentered','2015-06-29 00:00:00','2009-03-03 00:00:00',100.67,778,802,86.4,'Polar' from dual union all
select 34374,'034374','1997-051FS','deb Iridium 33','',66131,'Reentered','2016-03-18 00:00:00','2009-03-03 00:00:00',100.97,776,833,86.49,'Polar' from dual union all
select 34375,'034375','1997-051FT','deb Iridium 33','',66131,'In Earth orbit','','2009-03-03 00:00:00',102.09,777,939,86.43,'Polar' from dual union all
select 34376,'034376','1997-051FU','deb Iridium 33','',66131,'In Earth orbit','','2009-03-03 00:00:00',101.07,775,844,86.32,'Polar' from dual union all
select 34377,'034377','1997-051FV','deb Iridium 33','',66131,'Reentered','2010-10-22 00:00:00','2009-03-03 00:00:00',99.33,660,794,86.38,'Polar' from dual union all
select 34378,'034378','1997-051FW','deb Iridium 33','',66131,'In Earth orbit','','2009-03-03 00:00:00',101.88,778,918,86.37,'Polar' from dual union all
select 34379,'034379','1997-051FX','deb Iridium 33','',66131,'Reentered','2012-05-03 00:00:00','2009-03-03 00:00:00',100.18,724,810,86.24,'Polar' from dual union all
select 3438,'003438','1965-082PB','deb Transtage 4','',11832,'Reentered','1971-10-13 00:00:00','1968-09-25 00:00:00',97.18,566,682,32.63,'Intermediate' from dual union all
select 34380,'034380','2009-011A','Kepler','Kepler',68692,'Deep Space','2009-03-07 00:00:00','2009-03-07 00:00:00',156563.19,182,1937434,28.53,'Earth Escape' from dual union all
select 34381,'034381','2009-011B','Star 48B','Star 48B',68692,'Deep Space','2009-03-07 00:00:00','2009-03-07 00:00:00',160085.81,184,1966195,28.53,'Earth Escape' from dual union all
select 34382,'034382','2009-011C','Delta 339','Delta SSPS AJ-10-118K',68692,'In Earth orbit','','2009-03-07 00:00:00',196.09,2245,7360,28.55,'Medium' from dual union all
select 34383,'034383','1993-036PY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.05,744,777,74.05,'Intermediate' from dual union all
select 34384,'034384','1993-036PZ','deb Kosmos-2251','',65138,'Reentered','2009-10-03 00:00:00','2009-03-09 00:00:00',99.8,626,872,74.01,'Intermediate' from dual union all
select 34385,'034385','1993-036QA','deb Kosmos-2251','',65138,'Reentered','2014-07-21 00:00:00','2009-03-09 00:00:00',99.32,655,798,74.03,'Intermediate' from dual union all
select 34386,'034386','1993-036QB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.5,768,797,74.04,'Intermediate' from dual union all
select 34387,'034387','1993-036QC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.32,743,805,74.05,'Intermediate' from dual union all
select 34388,'034388','1993-036QD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.53,771,797,74.05,'Intermediate' from dual;
insert into SATELLITE
select 34389,'034389','1993-036QE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.53,760,808,74.03,'Intermediate' from dual union all
select 3439,'003439','1965-082PC','deb Transtage 4','',11832,'Reentered','1980-01-31 00:00:00','1968-09-25 00:00:00',100.07,721,802,32.63,'Intermediate' from dual union all
select 34390,'034390','1993-036QF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.02,693,826,74.1,'Intermediate' from dual union all
select 34391,'034391','1993-036QG','deb Kosmos-2251','',65138,'Reentered','2015-02-04 00:00:00','2009-03-09 00:00:00',98.06,504,827,73.97,'Intermediate' from dual union all
select 34392,'034392','1993-036QH','deb Kosmos-2251','',65138,'Reentered','2014-12-26 00:00:00','2009-03-09 00:00:00',98.31,571,785,73.96,'Intermediate' from dual union all
select 34393,'034393','1993-036QJ','deb Kosmos-2251','',65138,'Reentered','2017-03-09 00:00:00','2009-03-09 00:00:00',99.15,660,775,73.99,'Intermediate' from dual union all
select 34394,'034394','1993-036QK','deb Kosmos-2251','',65138,'Reentered','2014-10-15 00:00:00','2009-03-09 00:00:00',100.53,765,803,74.04,'Intermediate' from dual union all
select 34395,'034395','1993-036QL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.55,771,799,74.05,'Intermediate' from dual union all
select 34396,'034396','1993-036QM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.54,769,800,74.05,'Intermediate' from dual union all
select 34397,'034397','1993-036QN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',98.63,611,776,73.86,'Intermediate' from dual union all
select 34398,'034398','1993-036QP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.05,601,921,73.98,'Intermediate' from dual union all
select 34399,'034399','1993-036QQ','deb Kosmos-2251','',65138,'Reentered','2012-04-16 00:00:00','2009-03-09 00:00:00',105.11,743,1257,74.06,'Intermediate' from dual union all
select 344,'000344','62 A Eta','FTV 1130','CORONA M-8',8141,'Reentered Attached','1962-08-14 00:00:00','1962-07-28 00:00:00',90.15,200,364,70.31,'Intermediate' from dual union all
select 3440,'003440','1965-082PD','deb Transtage 4','',11832,'Reentered','1981-04-13 00:00:00','1968-09-25 00:00:00',102.26,713,1019,32.25,'Intermediate' from dual union all
select 34400,'034400','1993-036QR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',99.24,660,785,74.01,'Intermediate' from dual union all
select 34401,'034401','1993-036QS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.11,702,826,74.06,'Intermediate' from dual union all
select 34402,'034402','1993-036QT','deb Kosmos-2251','',65138,'Reentered','2010-06-28 00:00:00','2009-03-09 00:00:00',95.56,323,769,74.15,'Intermediate' from dual union all
select 34403,'034403','1993-036QU','deb Kosmos-2251','',65138,'Reentered','2013-09-20 00:00:00','2009-03-09 00:00:00',99.61,670,810,74.03,'Intermediate' from dual union all
select 34404,'034404','1993-036QV','deb Kosmos-2251','',65138,'Reentered','2010-03-11 00:00:00','2009-03-09 00:00:00',96.07,367,774,74.03,'Intermediate' from dual union all
select 34405,'034405','1993-036QW','deb Kosmos-2251','',65138,'Reentered','2009-12-07 00:00:00','2009-03-09 00:00:00',96.26,312,847,74.04,'Intermediate' from dual union all
select 34406,'034406','1993-036QX','deb Kosmos-2251','',65138,'Reentered','2011-03-15 00:00:00','2009-03-09 00:00:00',96.78,425,783,73.92,'Intermediate' from dual union all
select 34407,'034407','1993-036QY','deb Kosmos-2251','',65138,'Reentered','2011-12-10 00:00:00','2009-03-09 00:00:00',97.23,451,802,74.01,'Intermediate' from dual union all
select 34408,'034408','1993-036QZ','deb Kosmos-2251','',65138,'Reentered','2011-12-04 00:00:00','2009-03-09 00:00:00',98.42,589,778,74.05,'Intermediate' from dual union all
select 34409,'034409','1993-036RA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',99.24,658,786,74.07,'Intermediate' from dual union all
select 3441,'003441','1965-082PE','deb Transtage 4','',11832,'Reentered','1988-12-04 00:00:00','1968-10-05 00:00:00',103.64,723,1138,31.98,'Intermediate' from dual union all
select 34410,'034410','1993-036RB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',99.29,666,784,73.78,'Intermediate' from dual union all
select 34411,'034411','1993-036RC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',99.29,625,825,74.05,'Intermediate' from dual union all
select 34412,'034412','1993-036RD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',101.45,635,1019,73.96,'Intermediate' from dual union all
select 34413,'034413','1993-036RE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',102.28,775,959,74.03,'Intermediate' from dual union all
select 34414,'034414','1993-036RF','deb Kosmos-2251','',65138,'Reentered','2009-03-27 00:00:00','2009-03-09 00:00:00',93.43,265,620,74.05,'Intermediate' from dual union all
select 34415,'034415','1993-036RG','deb Kosmos-2251','',65138,'Reentered','2009-11-29 00:00:00','2009-03-09 00:00:00',95.82,356,761,74.08,'Intermediate' from dual union all
select 34416,'034416','1993-036RH','deb Kosmos-2251','',65138,'Reentered','2011-10-04 00:00:00','2009-03-09 00:00:00',96.38,384,787,73.96,'Intermediate' from dual union all
select 34417,'034417','1993-036RJ','deb Kosmos-2251','',65138,'Reentered','2010-10-01 00:00:00','2009-03-08 00:00:00',96.47,407,773,74.05,'Intermediate' from dual union all
select 34418,'034418','1993-036RK','deb Kosmos-2251','',65138,'Reentered','2011-11-27 00:00:00','2009-03-09 00:00:00',97.41,493,777,74,'Intermediate' from dual union all
select 34419,'034419','1993-036RL','deb Kosmos-2251','',65138,'Reentered','2012-11-18 00:00:00','2009-03-09 00:00:00',98.21,550,797,74.08,'Intermediate' from dual union all
select 3442,'003442','1965-082PF','deb Transtage 4','',11832,'Reentered','1969-06-12 00:00:00','1969-02-06 00:00:00',100.1,625,901,31.54,'Intermediate' from dual union all
select 34420,'034420','1993-036RM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',98.59,605,778,74.04,'Intermediate' from dual union all
select 34421,'034421','1993-036RN','deb Kosmos-2251','',65138,'Reentered','2016-09-28 00:00:00','2009-03-09 00:00:00',98.82,627,778,73.99,'Intermediate' from dual union all
select 34422,'034422','1993-036RP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',102.05,575,1137,73.98,'Intermediate' from dual union all
select 34423,'034423','1993-036RQ','deb Kosmos-2251','',65138,'Reentered','2015-03-19 00:00:00','2009-03-09 00:00:00',98.94,640,776,74.06,'Intermediate' from dual union all
select 34424,'034424','1993-036RR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',99.4,678,782,74.03,'Intermediate' from dual union all
select 34425,'034425','1993-036RS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.06,734,789,74.04,'Intermediate' from dual union all
select 34426,'034426','1993-036RT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.01,735,783,74.06,'Intermediate' from dual union all
select 34427,'034427','1993-036RU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.28,719,825,74.04,'Intermediate' from dual union all
select 34428,'034428','1993-036RV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.42,743,814,74.04,'Intermediate' from dual union all
select 34429,'034429','1993-036RW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',99.73,704,788,74.02,'Intermediate' from dual union all
select 3443,'003443','1965-082PG','deb Transtage 4','',11832,'Reentered','1980-04-15 00:00:00','1967-12-30 00:00:00',105.54,528,1512,30.11,'Intermediate' from dual union all
select 34430,'034430','1993-036RX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.66,740,840,74.06,'Intermediate' from dual union all
select 34431,'034431','1993-036RY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',101.53,776,886,74.08,'Intermediate' from dual union all
select 34432,'034432','1993-036RZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',103.37,771,1064,73.83,'Intermediate' from dual union all
select 34433,'034433','1993-036SA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',103.3,775,1055,74.03,'Intermediate' from dual union all
select 34434,'034434','1993-036SB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',106.09,773,1318,74.07,'Intermediate' from dual union all
select 34435,'034435','1993-036SC','deb Kosmos-2251','',65138,'Reentered','2016-05-06 00:00:00','2009-03-09 00:00:00',98.98,642,778,74.02,'Intermediate' from dual union all
select 34436,'034436','1993-036SD','deb Kosmos-2251','',65138,'Reentered','2015-04-21 00:00:00','2009-03-09 00:00:00',99.52,608,863,74.01,'Intermediate' from dual union all
select 34437,'034437','1993-036SE','deb Kosmos-2251','',65138,'Reentered','2012-04-23 00:00:00','2009-03-09 00:00:00',97.5,504,775,73.95,'Intermediate' from dual union all
select 34438,'034438','1993-036SF','deb Kosmos-2251','',65138,'Reentered','2013-08-09 00:00:00','2009-03-09 00:00:00',98.14,549,791,74.01,'Intermediate' from dual union all
select 34439,'034439','1993-036SG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',99.94,629,883,74.07,'Intermediate' from dual union all
select 3444,'003444','1965-082PH','deb Transtage 4','',11832,'Reentered','1981-05-01 00:00:00','1968-10-14 00:00:00',101.27,709,928,31.43,'Intermediate' from dual union all
select 34440,'034440','1993-036SH','deb Kosmos-2251','',65138,'Reentered','2012-06-01 00:00:00','2009-03-09 00:00:00',96.97,434,794,73.9,'Intermediate' from dual union all
select 34441,'034441','1993-036SJ','deb Kosmos-2251','',65138,'Reentered','2012-06-14 00:00:00','2009-03-09 00:00:00',98.04,539,791,73.94,'Intermediate' from dual union all
select 34442,'034442','1993-036SK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',99.27,657,790,74.02,'Intermediate' from dual union all
select 34443,'034443','1993-036SL','deb Kosmos-2251','',65138,'Reentered','2011-04-15 00:00:00','2009-03-09 00:00:00',96.07,357,784,73.93,'Intermediate' from dual union all
select 34444,'034444','1993-036SM','deb Kosmos-2251','',65138,'Reentered','2009-05-10 00:00:00','2009-03-09 00:00:00',94.53,249,743,74.05,'Intermediate' from dual union all
select 34445,'034445','1993-036SN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',98.94,591,825,74.03,'Intermediate' from dual union all
select 34446,'034446','1993-036SP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',99.63,702,780,74.05,'Intermediate' from dual union all
select 34447,'034447','1993-036SQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',99.83,715,786,74.03,'Intermediate' from dual union all
select 34448,'034448','1993-036SR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.29,769,776,74.07,'Intermediate' from dual union all
select 34449,'034449','1993-036SS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.51,722,844,74.06,'Intermediate' from dual union all
select 3445,'003445','1965-082PJ','deb Transtage 4','',11832,'Reentered','1980-07-25 00:00:00','1968-09-30 00:00:00',99.42,676,786,32.39,'Intermediate' from dual union all
select 34450,'034450','1993-036ST','deb Kosmos-2251','',65138,'Reentered','2009-06-29 00:00:00','2009-03-09 00:00:00',98.88,611,800,74.02,'Intermediate' from dual union all
select 34451,'034451','1993-036SU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.56,724,847,74.07,'Intermediate' from dual union all
select 34452,'034452','1993-036SV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',98.17,567,775,74.05,'Intermediate' from dual union all
select 34453,'034453','1993-036SW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.56,738,833,74.18,'Intermediate' from dual union all
select 34454,'034454','1993-036SX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.44,752,807,74.06,'Intermediate' from dual union all
select 34455,'034455','1993-036SY','deb Kosmos-2251','',65138,'Reentered','2013-01-16 00:00:00','2009-03-09 00:00:00',96.59,416,775,73.92,'Intermediate' from dual union all
select 34456,'034456','1993-036SZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',101.1,750,872,74.05,'Intermediate' from dual union all
select 34457,'034457','1993-036TA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',99.62,643,838,74.01,'Intermediate' from dual union all
select 34458,'034458','1993-036TB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.57,768,803,74.04,'Intermediate' from dual union all
select 34459,'034459','1993-036TC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',103.23,765,1058,74.08,'Intermediate' from dual union all
select 3446,'003446','1965-082PK','deb Transtage 4','',11832,'Reentered','1981-02-13 00:00:00','1968-09-29 00:00:00',99.77,711,784,32.6,'Intermediate' from dual union all
select 34460,'034460','1993-036TD','deb Kosmos-2251','',65138,'Reentered','2009-03-22 00:00:00','2009-03-09 00:00:00',96.79,493,717,74.11,'Intermediate' from dual union all
select 34461,'034461','1993-036TE','deb Kosmos-2251','',65138,'Reentered','2013-10-30 00:00:00','2009-03-08 00:00:00',97.74,492,810,74.06,'Intermediate' from dual union all
select 34462,'034462','1993-036TF','deb Kosmos-2251','',65138,'Reentered','2015-01-29 00:00:00','2009-03-09 00:00:00',98.72,604,791,74.03,'Intermediate' from dual union all
select 34463,'034463','1993-036TG','deb Kosmos-2251','',65138,'Reentered','2009-05-03 00:00:00','2009-03-09 00:00:00',98.5,516,858,73.95,'Intermediate' from dual union all
select 34464,'034464','1993-036TH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',99.3,675,776,74.03,'Intermediate' from dual union all
select 34465,'034465','1993-036TJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.53,693,874,74.08,'Intermediate' from dual union all
select 34466,'034466','1993-036TK','deb Kosmos-2251','',65138,'Reentered','2011-12-04 00:00:00','2009-03-09 00:00:00',96.83,440,774,73.95,'Intermediate' from dual union all
select 34467,'034467','1993-036TL','deb Kosmos-2251','',65138,'Reentered','2014-05-15 00:00:00','2009-03-09 00:00:00',99.22,650,793,74.03,'Intermediate' from dual union all
select 34468,'034468','1993-036TM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.01,698,820,74.02,'Intermediate' from dual union all
select 34469,'034469','1993-036TN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.23,753,786,74.03,'Intermediate' from dual union all
select 3447,'003447','1965-082PL','deb Transtage 4','',11832,'Reentered','1979-07-01 00:00:00','1968-09-29 00:00:00',98.39,614,749,31.71,'Intermediate' from dual union all
select 34470,'034470','1993-036TP','deb Kosmos-2251','',65138,'Reentered','2009-05-12 00:00:00','2009-03-09 00:00:00',105.37,489,1535,73.99,'Intermediate' from dual union all
select 34471,'034471','1993-036TQ','deb Kosmos-2251','',65138,'Reentered','2012-09-06 00:00:00','2009-03-09 00:00:00',97.75,527,776,73.95,'Intermediate' from dual union all
select 34472,'034472','1993-036TR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',105.92,773,1302,74.08,'Intermediate' from dual union all
select 34473,'034473','1993-036TS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.7,771,813,74.04,'Intermediate' from dual union all
select 34474,'034474','1993-036TT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',103.94,765,1125,74.09,'Intermediate' from dual union all
select 34475,'034475','1993-036TU','deb Kosmos-2251','',65138,'Reentered','2009-05-14 00:00:00','2009-03-09 00:00:00',95.08,302,742,74.05,'Intermediate' from dual union all
select 34476,'034476','1993-036TV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',106.55,773,1361,74.02,'Intermediate' from dual union all
select 34477,'034477','1993-036TW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.17,694,840,73.99,'Intermediate' from dual union all
select 34478,'034478','1993-036TX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.33,689,860,74.07,'Intermediate' from dual;
insert into SATELLITE
select 34479,'034479','1993-036TY','deb Kosmos-2251','',65138,'Reentered','2009-03-21 00:00:00','2009-03-09 00:00:00',91.85,197,534,73.89,'Intermediate' from dual union all
select 3448,'003448','1965-082PM','deb Transtage 4','',11832,'Reentered','1983-01-01 00:00:00','1968-09-30 00:00:00',101.75,710,973,32.01,'Intermediate' from dual union all
select 34480,'034480','1993-036TZ','deb Kosmos-2251','',65138,'Reentered','2011-10-29 00:00:00','2009-03-09 00:00:00',97.38,466,801,73.96,'Intermediate' from dual union all
select 34481,'034481','1993-036UA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-08 00:00:00',101.31,763,879,73.99,'Intermediate' from dual union all
select 34482,'034482','1993-036UB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-09 00:00:00',100.91,737,867,74.05,'Intermediate' from dual union all
select 34483,'034483','1997-051FY','deb Iridium 33','',66131,'Reentered','2011-10-25 00:00:00','2009-03-09 00:00:00',99.66,659,826,86.19,'Polar' from dual union all
select 34484,'034484','1997-051FZ','deb Iridium 33','',66131,'Reentered','2011-08-27 00:00:00','2009-03-09 00:00:00',97.45,498,776,86.29,'Polar' from dual union all
select 34485,'034485','1997-051GA','deb Iridium 33','',66131,'Reentered','2012-10-17 00:00:00','2009-03-09 00:00:00',98.48,581,791,86.29,'Polar' from dual union all
select 34486,'034486','1997-051GB','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',100.11,745,783,86.42,'Polar' from dual union all
select 34487,'034487','1997-051GC','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',100.16,741,791,86.35,'Polar' from dual union all
select 34488,'034488','1997-051GD','deb Iridium 33','',66131,'In Earth orbit','','2009-03-08 00:00:00',99.98,734,782,86.35,'Polar' from dual union all
select 34489,'034489','1997-051GE','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',102.05,778,933,86.43,'Polar' from dual union all
select 3449,'003449','1968-082A','Kosmos-244','OGCh No. V22500-10',16809,'Deorbited','1968-10-02 00:00:00','1968-10-02 00:00:00',87.4,134,157,49.57,'Intermediate' from dual union all
select 34490,'034490','1997-051GF','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',100.97,775,834,86.34,'Polar' from dual union all
select 34491,'034491','1997-051GG','deb Iridium 33','',66131,'Reentered','2015-05-22 00:00:00','2009-03-09 00:00:00',101.92,778,922,86.41,'Polar' from dual union all
select 34492,'034492','1997-051GH','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',105.15,768,1235,86.52,'Polar' from dual union all
select 34493,'034493','1997-051GJ','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',100.86,739,860,86.28,'Polar' from dual union all
select 34494,'034494','1997-051GK','deb Iridium 33','',66131,'Reentered','2013-01-18 00:00:00','2009-03-09 00:00:00',101.42,744,908,86.19,'Polar' from dual union all
select 34495,'034495','1997-051GL','deb Iridium 33','',66131,'Reentered','2012-12-11 00:00:00','2009-03-09 00:00:00',98.51,597,778,86.49,'Polar' from dual union all
select 34496,'034496','1997-051GM','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',102.81,785,999,86.44,'Polar' from dual union all
select 34497,'034497','1997-051GN','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',101.76,772,912,86.28,'Polar' from dual union all
select 34498,'034498','1997-051GP','deb Iridium 33','',66131,'Reentered','2011-11-25 00:00:00','2009-03-09 00:00:00',104.33,773,1154,86.4,'Polar' from dual union all
select 34499,'034499','1997-051GQ','deb Iridium 33','',66131,'Reentered','2015-01-02 00:00:00','2009-03-09 00:00:00',99.41,646,814,86.4,'Polar' from dual union all
select 345,'000345','62 A Theta','FTV 1131','CORONA M-9',8156,'Reentered Attached','1962-08-24 00:00:00','1962-08-10 00:00:00',90.13,187,375,71.09,'Intermediate' from dual union all
select 3450,'003450','1968-082B','8S692','8S692',16809,'Reentered','1968-10-02 00:00:00','1968-10-02 00:00:00',87.4,133,159,49.57,'Intermediate' from dual union all
select 34500,'034500','1997-051GR','deb Iridium 33','',66131,'Reentered','2012-05-12 00:00:00','2009-03-09 00:00:00',101.15,781,846,86.42,'Polar' from dual union all
select 34501,'034501','1997-051GS','deb Iridium 33','',66131,'Reentered','2010-02-04 00:00:00','2009-03-09 00:00:00',99.07,615,814,86.26,'Polar' from dual union all
select 34502,'034502','1997-051GT','deb Iridium 33','',66131,'Reentered','2012-03-22 00:00:00','2009-03-09 00:00:00',99.94,732,780,86.34,'Polar' from dual union all
select 34503,'034503','1997-051GU','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',99.91,731,778,86.32,'Polar' from dual union all
select 34504,'034504','1997-051GV','deb Iridium 33','',66131,'Reentered','2011-12-24 00:00:00','2009-03-09 00:00:00',100.11,718,809,86.39,'Polar' from dual union all
select 34505,'034505','1997-051GW','deb Iridium 33','',66131,'Reentered','2015-02-02 00:00:00','2009-03-09 00:00:00',100.26,754,788,86.43,'Polar' from dual union all
select 34506,'034506','1997-051GX','deb Iridium 33','',66131,'Reentered','2013-10-28 00:00:00','2009-03-09 00:00:00',100.35,770,781,86.44,'Polar' from dual union all
select 34507,'034507','1997-051GY','deb Iridium 33','',66131,'Reentered','2014-01-21 00:00:00','2009-03-09 00:00:00',100.47,776,786,86.45,'Polar' from dual union all
select 34508,'034508','1997-051GZ','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',101.7,778,901,86.38,'Polar' from dual union all
select 34509,'034509','1997-051HA','deb Iridium 33','',66131,'Reentered','2011-06-20 00:00:00','2009-03-09 00:00:00',101.92,778,922,86.43,'Polar' from dual union all
select 3451,'003451','1968-082C','Kosmos-244 Adapter','OGCh Adapter',16809,'Reentered','1968-10-02 00:00:00','1968-10-02 00:00:00',87.9,149,193,49.58,'Intermediate' from dual union all
select 34510,'034510','1997-051HB','deb Iridium 33','',66131,'Reentered','2014-10-31 00:00:00','2009-03-09 00:00:00',100.95,780,827,86.32,'Polar' from dual union all
select 34511,'034511','1997-051HC','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',100.93,777,829,86.39,'Polar' from dual union all
select 34512,'034512','1997-051HD','deb Iridium 33','',66131,'Reentered','2013-04-14 00:00:00','2009-03-09 00:00:00',101.05,778,839,86.33,'Polar' from dual union all
select 34513,'034513','1997-051HE','deb Iridium 33','',66131,'Reentered','2009-04-20 00:00:00','2009-03-08 00:00:00',99.28,660,789,86.74,'Polar' from dual union all
select 34514,'034514','1997-051HF','deb Iridium 33','',66131,'Reentered','2009-07-17 00:00:00','2009-03-09 00:00:00',102.58,795,967,86.34,'Polar' from dual union all
select 34515,'034515','1997-051HG','deb Iridium 33','',66131,'Reentered','2015-12-26 00:00:00','2009-03-09 00:00:00',100.01,713,805,86.31,'Polar' from dual union all
select 34516,'034516','1997-051HH','deb Iridium 33','',66131,'Reentered','2012-08-26 00:00:00','2009-03-09 00:00:00',100.49,740,824,86.39,'Polar' from dual union all
select 34517,'034517','1997-051HJ','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',100.55,778,791,86.38,'Polar' from dual union all
select 34518,'034518','1997-051HK','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',100.77,778,813,86.41,'Polar' from dual union all
select 34519,'034519','1997-051HL','deb Iridium 33','',66131,'Reentered','2011-11-25 00:00:00','2009-03-09 00:00:00',100.58,772,800,86.4,'Polar' from dual union all
select 3452,'003452','1968-080C','Kapsula Kosmosa-243','Nauka 2KS No. 1',16807,'Reentered','1968-10-12 00:00:00','1968-10-03 00:00:00',89.23,199,274,71.32,'Intermediate' from dual union all
select 34520,'034520','1997-051HM','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',100.95,778,830,86.39,'Polar' from dual union all
select 34521,'034521','1997-051HN','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',100.38,773,780,86.39,'Polar' from dual union all
select 34522,'034522','1997-051HP','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',101.69,773,905,86.38,'Polar' from dual union all
select 34523,'034523','1997-051HQ','deb Iridium 33','',66131,'Reentered','2014-03-08 00:00:00','2009-03-09 00:00:00',99.32,675,778,86.42,'Polar' from dual union all
select 34524,'034524','1997-051HR','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',100.87,766,834,86.36,'Polar' from dual union all
select 34525,'034525','1997-051HS','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',100.46,777,784,86.37,'Polar' from dual union all
select 34526,'034526','1997-051HT','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',100.97,778,832,86.36,'Polar' from dual union all
select 34527,'034527','1997-051HU','deb Iridium 33','',66131,'Reentered','2013-05-04 00:00:00','2009-03-09 00:00:00',100.68,776,806,86.39,'Polar' from dual union all
select 34528,'034528','1997-051HV','deb Iridium 33','',66131,'Reentered','2013-01-20 00:00:00','2009-03-09 00:00:00',100.93,778,827,86.47,'Polar' from dual union all
select 34529,'034529','1997-051HW','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',100.28,762,782,86.36,'Polar' from dual union all
select 3453,'003453','1965-082PN','deb Transtage 4','',11832,'Reentered','1969-04-29 00:00:00','1968-10-04 00:00:00',99.18,657,781,32.42,'Intermediate' from dual union all
select 34530,'034530','1997-051HX','deb Iridium 33','',66131,'Reentered','2014-10-27 00:00:00','2009-03-09 00:00:00',102.2,776,949,86.43,'Polar' from dual union all
select 34531,'034531','1997-051HY','deb Iridium 33','',66131,'Reentered','2013-04-28 00:00:00','2009-03-09 00:00:00',100.84,778,818,86.4,'Polar' from dual union all
select 34532,'034532','1997-051HZ','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',101.7,779,900,86.37,'Polar' from dual union all
select 34533,'034533','1997-051JA','deb Iridium 33','',66131,'Reentered','2012-11-16 00:00:00','2009-03-09 00:00:00',98.22,566,782,86.19,'Polar' from dual union all
select 34534,'034534','1997-051JB','deb Iridium 33','',66131,'Reentered','2014-08-29 00:00:00','2009-03-09 00:00:00',99.58,681,796,86.3,'Polar' from dual union all
select 34535,'034535','1997-051JC','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',100.25,756,785,86.35,'Polar' from dual union all
select 34536,'034536','1997-051JD','deb Iridium 33','',66131,'Reentered','2011-03-26 00:00:00','2009-03-09 00:00:00',100.26,759,783,86.36,'Polar' from dual union all
select 34537,'034537','1997-051JE','deb Iridium 33','',66131,'Reentered','2014-04-28 00:00:00','2009-03-09 00:00:00',99.37,659,798,86.33,'Polar' from dual union all
select 34538,'034538','1997-051JF','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',101.08,777,843,86.28,'Polar' from dual union all
select 34539,'034539','1997-051JG','deb Iridium 33','',66131,'Reentered','2013-11-30 00:00:00','2009-03-09 00:00:00',101.77,776,909,86.38,'Polar' from dual union all
select 3454,'003454','1965-082PP','deb Transtage 4','',11832,'In Earth orbit','','1992-02-02 00:00:00',98.63,666,721,32.27,'Intermediate' from dual union all
select 34540,'034540','1997-051JH','deb Iridium 33','',66131,'In Earth orbit','','2009-03-09 00:00:00',99.91,731,778,86.33,'Polar' from dual union all
select 34541,'034541','2009-012A','Discovery (STS-119)','OV-103',68693,'Landed','2009-03-28 00:00:00','2009-03-25 00:00:00',91.6,347,360,51.64,'Intermediate' from dual union all
select 34542,'034542','1993-036UC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',99.83,723,778,74.03,'Intermediate' from dual union all
select 34543,'034543','1993-036UD','deb Kosmos-2251','',65138,'Reentered','2014-06-11 00:00:00','2009-03-16 00:00:00',99.3,650,801,74.03,'Intermediate' from dual union all
select 34544,'034544','1993-036UE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',106.73,775,1376,73.92,'Intermediate' from dual union all
select 34545,'034545','1993-036UF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.38,758,796,74.05,'Intermediate' from dual union all
select 34546,'034546','1993-036UG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.58,767,805,74.04,'Intermediate' from dual union all
select 34547,'034547','1993-036UH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.41,746,809,74.05,'Intermediate' from dual union all
select 34548,'034548','1993-036UJ','deb Kosmos-2251','',65138,'Reentered','2014-04-20 00:00:00','2009-03-16 00:00:00',97.77,529,775,73.9,'Intermediate' from dual union all
select 34549,'034549','1993-036UK','deb Kosmos-2251','',65138,'Reentered','2012-11-10 00:00:00','2009-03-15 00:00:00',97.42,492,779,73.96,'Intermediate' from dual union all
select 3455,'003455','1965-082PQ','deb Transtage 4','',11832,'Reentered','1980-12-24 00:00:00','1968-10-08 00:00:00',101.32,687,956,32.2,'Intermediate' from dual union all
select 34550,'034550','1993-036UL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',102.55,705,1054,73.99,'Intermediate' from dual union all
select 34551,'034551','1993-036UM','deb Kosmos-2251','',65138,'Reentered','2011-09-20 00:00:00','2009-03-16 00:00:00',96.89,447,773,73.87,'Intermediate' from dual union all
select 34552,'034552','1993-036UN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.83,766,830,74.04,'Intermediate' from dual union all
select 34553,'034553','1993-036UP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-15 00:00:00',100.62,766,810,74.04,'Intermediate' from dual union all
select 34554,'034554','1993-036UQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.62,770,806,74.04,'Intermediate' from dual union all
select 34555,'034555','1993-036UR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.62,765,811,74.04,'Intermediate' from dual union all
select 34556,'034556','1993-036US','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.64,769,809,74.04,'Intermediate' from dual union all
select 34557,'034557','1993-036UT','deb Kosmos-2251','',65138,'Reentered','2014-11-19 00:00:00','2009-03-16 00:00:00',100.94,769,837,74.02,'Intermediate' from dual union all
select 34558,'034558','1993-036UU','deb Kosmos-2251','',65138,'Reentered','2014-10-10 00:00:00','2009-03-16 00:00:00',99.4,661,799,74.04,'Intermediate' from dual union all
select 34559,'034559','1993-036UV','deb Kosmos-2251','',65138,'Reentered','2013-06-17 00:00:00','2009-03-16 00:00:00',100.48,756,807,74.05,'Intermediate' from dual union all
select 3456,'003456','1965-082PR','deb Transtage 4','',11832,'Reentered','1982-02-18 00:00:00','1968-10-01 00:00:00',100.45,705,854,31.92,'Intermediate' from dual union all
select 34560,'034560','1993-036UW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.64,764,814,74.03,'Intermediate' from dual union all
select 34561,'034561','1993-036UX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.29,730,815,74.04,'Intermediate' from dual union all
select 34562,'034562','1993-036UY','deb Kosmos-2251','',65138,'Reentered','2015-02-16 00:00:00','2009-03-16 00:00:00',98.75,622,776,74,'Intermediate' from dual union all
select 34563,'034563','1993-036UZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.35,764,786,74.04,'Intermediate' from dual union all
select 34564,'034564','1993-036VA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.38,762,792,74.04,'Intermediate' from dual union all
select 34565,'034565','1993-036VB','deb Kosmos-2251','',65138,'Reentered','2014-11-09 00:00:00','2009-03-16 00:00:00',100.75,760,829,74.05,'Intermediate' from dual union all
select 34566,'034566','1993-036VC','deb Kosmos-2251','',65138,'Reentered','2009-04-17 00:00:00','2009-03-16 00:00:00',95.23,237,823,74.11,'Intermediate' from dual union all
select 34567,'034567','1993-036VD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',101.4,769,881,74.04,'Intermediate' from dual union all
select 34568,'034568','1993-036VE','deb Kosmos-2251','',65138,'Reentered','2010-12-27 00:00:00','2009-03-16 00:00:00',101.3,616,1025,74.05,'Intermediate' from dual;
insert into SATELLITE
select 34569,'034569','1993-036VF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',101.91,717,982,74,'Intermediate' from dual union all
select 3457,'003457','1968-083A','Kosmos-245','DS-P1-Yu No. 16',16810,'Reentered','1969-01-15 00:00:00','1968-11-02 00:00:00',91.56,260,442,70.99,'Intermediate' from dual union all
select 34570,'034570','1993-036VG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-15 00:00:00',103.47,774,1072,73.97,'Intermediate' from dual union all
select 34571,'034571','1993-036VH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',102.19,773,952,74.08,'Intermediate' from dual union all
select 34572,'034572','1993-036VJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',101.99,775,931,74.02,'Intermediate' from dual union all
select 34573,'034573','1993-036VK','deb Kosmos-2251','',65138,'Reentered','2016-03-07 00:00:00','2009-03-15 00:00:00',98.88,635,776,74.02,'Intermediate' from dual union all
select 34574,'034574','1993-036VL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.01,678,840,74.08,'Intermediate' from dual union all
select 34575,'034575','1993-036VM','deb Kosmos-2251','',65138,'Reentered','2014-11-08 00:00:00','2009-03-16 00:00:00',100.25,762,779,74.06,'Intermediate' from dual union all
select 34576,'034576','1993-036VN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-15 00:00:00',100.26,763,778,74.05,'Intermediate' from dual union all
select 34577,'034577','1993-036VP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',99.71,714,776,74.04,'Intermediate' from dual union all
select 34578,'034578','1993-036VQ','deb Kosmos-2251','',65138,'Reentered','2013-03-19 00:00:00','2009-03-16 00:00:00',97.41,457,813,74.07,'Intermediate' from dual union all
select 34579,'034579','1993-036VR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.38,754,800,74.06,'Intermediate' from dual union all
select 3458,'003458','1968-083B','S1M','S1M',16810,'Reentered','1968-11-16 00:00:00','1968-10-11 00:00:00',91.56,276,427,71,'Intermediate' from dual union all
select 34580,'034580','1993-036VS','deb Kosmos-2251','',65138,'Reentered','2011-12-15 00:00:00','2009-03-16 00:00:00',97.99,525,801,74.02,'Intermediate' from dual union all
select 34581,'034581','1993-036VT','deb Kosmos-2251','',65138,'Reentered','2014-10-10 00:00:00','2009-03-16 00:00:00',99.51,680,791,74.06,'Intermediate' from dual union all
select 34582,'034582','1993-036VU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.72,750,835,74.03,'Intermediate' from dual union all
select 34583,'034583','1993-036VV','deb Kosmos-2251','',65138,'Reentered','2014-03-20 00:00:00','2009-03-16 00:00:00',99.32,688,765,74.11,'Intermediate' from dual union all
select 34584,'034584','1993-036VW','deb Kosmos-2251','',65138,'Reentered','2015-03-04 00:00:00','2009-03-16 00:00:00',99.07,647,782,74.06,'Intermediate' from dual union all
select 34585,'034585','1993-036VX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.05,744,777,74.04,'Intermediate' from dual union all
select 34586,'034586','1993-036VY','deb Kosmos-2251','',65138,'Reentered','2011-08-04 00:00:00','2009-03-16 00:00:00',95.79,342,772,74.04,'Intermediate' from dual union all
select 34587,'034587','1993-036VZ','deb Kosmos-2251','',65138,'Reentered','2011-06-29 00:00:00','2009-03-16 00:00:00',97.44,494,778,74.05,'Intermediate' from dual union all
select 34588,'034588','1993-036WA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',100.56,770,800,74.05,'Intermediate' from dual union all
select 34589,'034589','1993-036WB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-15 00:00:00',104.39,772,1160,73.93,'Intermediate' from dual union all
select 3459,'003459','1968-084A','Aurorae','ESRO 1A',16811,'Reentered','1970-06-26 00:00:00','1968-11-03 00:00:00',102.43,258,1490,93.76,'Polar' from dual union all
select 34590,'034590','1993-036WC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-16 00:00:00',99.28,671,778,74.04,'Intermediate' from dual union all
select 34591,'034591','1993-036WD','deb Kosmos-2251','',65138,'Reentered','2010-07-29 00:00:00','2009-03-16 00:00:00',96.09,368,775,73.94,'Intermediate' from dual union all
select 34592,'034592','1997-051JJ','deb Iridium 33','',66131,'Reentered','2011-06-10 00:00:00','2009-03-16 00:00:00',99.1,641,791,86.26,'Polar' from dual union all
select 34593,'034593','1997-051JK','deb Iridium 33','',66131,'In Earth orbit','','2009-03-16 00:00:00',101.1,778,844,86.4,'Polar' from dual union all
select 34594,'034594','1997-051JL','deb Iridium 33','',66131,'Reentered','2011-09-07 00:00:00','2009-03-16 00:00:00',100.29,744,801,86.45,'Polar' from dual union all
select 34595,'034595','1997-051JM','deb Iridium 33','',66131,'Reentered','2012-06-14 00:00:00','2009-03-16 00:00:00',100.93,753,853,86.38,'Polar' from dual union all
select 34596,'034596','1997-051JN','deb Iridium 33','',66131,'Reentered','2012-09-04 00:00:00','2009-03-16 00:00:00',98.97,641,778,86.42,'Polar' from dual union all
select 34597,'034597','1997-051JP','deb Iridium 33','',66131,'Reentered','2014-08-06 00:00:00','2009-03-16 00:00:00',100.36,772,780,86.46,'Polar' from dual union all
select 34598,'034598','1997-051JQ','deb Iridium 33','',66131,'Reentered','2011-10-30 00:00:00','2009-03-16 00:00:00',101.87,771,924,86.52,'Polar' from dual union all
select 34599,'034599','1997-051JR','deb Iridium 33','',66131,'Reentered','2013-11-04 00:00:00','2009-03-16 00:00:00',99.19,662,777,86.43,'Polar' from dual union all
select 346,'000346','62 A Iota 1','Kosmos-7','Zenit-2 No. 4',8143,'Landed','1962-08-01 00:00:00','1962-07-30 00:00:00',90.03,200,351,64.95,'Intermediate' from dual union all
select 3460,'003460','1968-084B','FW4S S/N 2223-1','FW4S',16811,'Reentered','1969-08-13 00:00:00','1968-11-02 00:00:00',101.97,254,1450,93.75,'Polar' from dual union all
select 34600,'034600','1997-051JS','deb Iridium 33','',66131,'Reentered','2012-09-03 00:00:00','2009-03-16 00:00:00',100.34,771,779,86.4,'Polar' from dual union all
select 34601,'034601','1997-051JT','deb Iridium 33','',66131,'Reentered','2011-02-18 00:00:00','2009-03-16 00:00:00',102.36,771,970,86.47,'Polar' from dual union all
select 34602,'034602','2009-013A','GOCE','GOCE',68694,'Reentered','2013-11-11 00:00:00','2009-03-25 00:00:00',90.11,272,288,96.7,'Sun-Synch' from dual union all
select 34603,'034603','2009-013B','Briz-KM No. 72511','Briz-KM 14S45',68694,'Reentered','2009-05-01 00:00:00','2009-03-24 00:00:00',89.54,225,278,96.64,'Sun-Synch' from dual union all
select 34604,'034604','2009-002L','deb H-2A','',68683,'Reentered','2014-04-25 00:00:00','2009-03-17 00:00:00',98.1,665,671,98.02,'Sun-Synch' from dual union all
select 34605,'034605','1998-067BM','S6 ECU Thermal Cover','',66503,'Reentered','2009-04-05 00:00:00','2009-03-20 00:00:00',91.59,345,361,51.64,'Intermediate' from dual union all
select 34606,'034606','1998-067BN','S6 ECU Thermal Cover','',66503,'Reentered','2009-03-31 00:00:00','2009-03-20 00:00:00',91.59,346,359,51.63,'Intermediate' from dual union all
select 34607,'034607','1998-067BP','S6 SSU Thermal Cover','',66503,'Reentered','2009-04-04 00:00:00','2009-03-20 00:00:00',91.58,345,360,51.64,'Intermediate' from dual union all
select 34608,'034608','1998-067BQ','S6 SSU Thermal Cover','',66503,'Reentered','2009-04-01 00:00:00','2009-03-20 00:00:00',91.57,344,360,51.64,'Intermediate' from dual union all
select 34609,'034609','1993-036WE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-23 00:00:00',102.23,741,988,73.76,'Intermediate' from dual union all
select 3461,'003461','1965-082PS','deb Transtage 4','',11832,'Reentered','1984-07-19 00:00:00','1968-10-02 00:00:00',99.45,727,737,32.51,'Intermediate' from dual union all
select 34610,'034610','1993-036WF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-23 00:00:00',103.77,775,1099,74.04,'Intermediate' from dual union all
select 34611,'034611','1993-036WG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-23 00:00:00',100.3,737,809,74.07,'Intermediate' from dual union all
select 34612,'034612','1993-036WH','deb Kosmos-2251','',65138,'Reentered','2010-03-18 00:00:00','2009-03-23 00:00:00',96.52,419,765,73.88,'Intermediate' from dual union all
select 34613,'034613','1993-036WJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-23 00:00:00',102.54,774,983,73.96,'Intermediate' from dual union all
select 34614,'034614','1993-036WK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-22 00:00:00',105.1,774,1224,73.9,'Intermediate' from dual union all
select 34615,'034615','1993-036WL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-22 00:00:00',106.94,776,1394,74.01,'Intermediate' from dual union all
select 34616,'034616','1993-036WM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-22 00:00:00',100.93,771,835,74.05,'Intermediate' from dual union all
select 34617,'034617','1993-036WN','deb Kosmos-2251','',65138,'Reentered','2012-04-24 00:00:00','2009-03-22 00:00:00',98.32,577,780,73.96,'Intermediate' from dual union all
select 34618,'034618','1993-036WP','deb Kosmos-2251','',65138,'Reentered','2015-03-10 00:00:00','2009-03-23 00:00:00',98.73,620,775,73.98,'Intermediate' from dual union all
select 34619,'034619','1993-036WQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-23 00:00:00',98.95,631,786,74.05,'Intermediate' from dual union all
select 3462,'003462','1965-082PT','deb Transtage 4','',11832,'In Earth orbit','','1992-02-03 00:00:00',97.49,614,664,32.02,'Intermediate' from dual union all
select 34620,'034620','1993-036WR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-23 00:00:00',102.37,769,973,74.06,'Intermediate' from dual union all
select 34621,'034621','1993-036WS','deb Kosmos-2251','',65138,'Reentered','2011-10-24 00:00:00','2009-03-23 00:00:00',97.32,447,814,74.02,'Intermediate' from dual union all
select 34622,'034622','1993-036WT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-22 00:00:00',102.27,774,959,74.03,'Intermediate' from dual union all
select 34623,'034623','1993-036WU','deb Kosmos-2251','',65138,'Reentered','2009-07-05 00:00:00','2009-03-23 00:00:00',98.44,574,795,73.99,'Intermediate' from dual union all
select 34624,'034624','1993-036WV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-23 00:00:00',100.81,760,834,74.03,'Intermediate' from dual union all
select 34625,'034625','1993-036WW','deb Kosmos-2251','',65138,'Reentered','2014-03-29 00:00:00','2009-03-23 00:00:00',98.45,582,787,74.01,'Intermediate' from dual union all
select 34626,'034626','1993-036WX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-23 00:00:00',99.73,713,779,74.06,'Intermediate' from dual union all
select 34627,'034627','1993-036WY','deb Kosmos-2251','',65138,'Reentered','2010-01-10 00:00:00','2009-03-23 00:00:00',99.81,536,962,74.04,'Intermediate' from dual union all
select 34628,'034628','1993-036WZ','deb Kosmos-2251','',65138,'Reentered','2014-03-19 00:00:00','2009-03-23 00:00:00',98.81,569,834,74.02,'Intermediate' from dual union all
select 34629,'034629','1993-036XA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-23 00:00:00',98.93,623,792,73.8,'Intermediate' from dual union all
select 3463,'003463','1965-082PU','deb Transtage 4','',11832,'Reentered','1982-05-21 00:00:00','1968-09-24 00:00:00',103.16,740,1076,31.55,'Intermediate' from dual union all
select 34630,'034630','1993-036XB','deb Kosmos-2251','',65138,'Reentered','2009-04-28 00:00:00','2009-03-23 00:00:00',97.73,464,837,74.1,'Intermediate' from dual union all
select 34631,'034631','1993-036XC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-22 00:00:00',99.55,695,779,74.03,'Intermediate' from dual union all
select 34632,'034632','1993-036XD','deb Kosmos-2251','',65138,'Reentered','2013-12-01 00:00:00','2009-03-23 00:00:00',100.28,526,1018,74.06,'Intermediate' from dual union all
select 34633,'034633','1993-036XE','deb Kosmos-2251','',65138,'Reentered','2012-02-01 00:00:00','2009-03-23 00:00:00',97.28,479,778,73.9,'Intermediate' from dual union all
select 34634,'034634','1993-036XF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-22 00:00:00',99.67,704,782,74.06,'Intermediate' from dual union all
select 34635,'034635','1993-036XG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-23 00:00:00',100.14,727,804,74.06,'Intermediate' from dual union all
select 34636,'034636','1993-036XH','deb Kosmos-2251','',65138,'Reentered','2011-10-29 00:00:00','2009-03-22 00:00:00',97.62,497,793,74.01,'Intermediate' from dual union all
select 34637,'034637','1993-036XJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-23 00:00:00',100.13,712,817,74.06,'Intermediate' from dual union all
select 34638,'034638','1993-036XK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-23 00:00:00',100.38,764,790,74.05,'Intermediate' from dual union all
select 34639,'034639','1993-036XL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-23 00:00:00',100.66,744,835,74.04,'Intermediate' from dual union all
select 3464,'003464','1965-082PV','deb Transtage 4','',11832,'Reentered','1981-10-21 00:00:00','1968-10-02 00:00:00',107.61,571,1661,30.54,'Intermediate' from dual union all
select 34640,'034640','1993-036XM','deb Kosmos-2251','',65138,'Reentered','2009-04-11 00:00:00','2009-03-23 00:00:00',96.8,389,822,74.14,'Intermediate' from dual union all
select 34641,'034641','1993-036XN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-22 00:00:00',100.56,769,802,73.85,'Intermediate' from dual union all
select 34642,'034642','1993-036XP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-23 00:00:00',104.04,774,1125,74.01,'Intermediate' from dual union all
select 34643,'034643','1997-051JU','deb Iridium 33','',66131,'In Earth orbit','','2009-03-23 00:00:00',100.8,766,827,86.39,'Polar' from dual union all
select 34644,'034644','1997-051JV','deb Iridium 33','',66131,'Reentered','2009-05-01 00:00:00','2009-03-23 00:00:00',99.59,654,824,86.51,'Polar' from dual union all
select 34645,'034645','1997-051JW','deb Iridium 33','',66131,'Reentered','2015-03-19 00:00:00','2009-03-23 00:00:00',100.1,749,778,86.35,'Polar' from dual union all
select 34646,'034646','1997-051JX','deb Iridium 33','',66131,'Reentered','2009-06-04 00:00:00','2009-03-23 00:00:00',101.28,788,851,86.44,'Polar' from dual union all
select 34647,'034647','1997-051JY','deb Iridium 33','',66131,'Reentered','2009-04-21 00:00:00','2009-03-23 00:00:00',104.6,758,1194,86.59,'Polar' from dual union all
select 34648,'034648','1997-051JZ','deb Iridium 33','',66131,'In Earth orbit','','2009-03-23 00:00:00',104.94,776,1208,86.42,'Polar' from dual union all
select 34649,'034649','1997-051KA','deb Iridium 33','',66131,'Reentered','2011-05-10 00:00:00','2009-03-23 00:00:00',98.97,622,797,86.38,'Polar' from dual union all
select 3465,'003465','1965-082PW','deb Transtage 4','',11832,'Reentered','1972-09-19 00:00:00','1968-09-30 00:00:00',98.71,661,733,31.71,'Intermediate' from dual union all
select 34650,'034650','1997-051KB','deb Iridium 33','',66131,'Reentered','2013-05-30 00:00:00','2009-03-22 00:00:00',99.69,689,799,86.25,'Polar' from dual union all
select 34651,'034651','1997-051KC','deb Iridium 33','',66131,'In Earth orbit','','2009-03-23 00:00:00',103.77,778,1095,86.3,'Polar' from dual union all
select 34652,'034652','1997-051KD','deb Iridium 33','',66131,'In Earth orbit','','2009-03-23 00:00:00',100.1,731,796,86.33,'Polar' from dual union all
select 34653,'034653','1997-051KE','deb Iridium 33','',66131,'Reentered','2014-07-20 00:00:00','2009-03-23 00:00:00',101.98,778,927,86.37,'Polar' from dual union all
select 34654,'034654','1997-051KF','deb Iridium 33','',66131,'Reentered','2014-06-26 00:00:00','2009-03-23 00:00:00',101.12,783,841,86.45,'Polar' from dual union all
select 34655,'034655','1997-051KG','deb Iridium 33','',66131,'Reentered','2012-01-07 00:00:00','2009-03-23 00:00:00',102,783,924,86.45,'Polar' from dual union all
select 34656,'034656','1997-051KH','deb Iridium 33','',66131,'Reentered','2012-02-03 00:00:00','2009-03-23 00:00:00',100.07,704,820,86.38,'Polar' from dual union all
select 34657,'034657','1997-051KJ','deb Iridium 33','',66131,'Reentered','2017-12-24 00:00:00','2009-03-23 00:00:00',100.43,749,809,86.33,'Polar' from dual union all
select 34658,'034658','1997-051KK','deb Iridium 33','',66131,'Reentered','2014-01-30 00:00:00','2009-03-23 00:00:00',99.71,709,781,86.36,'Polar' from dual;
insert into SATELLITE
select 34659,'034659','1997-051KL','deb Iridium 33','',66131,'Reentered','2012-05-19 00:00:00','2009-03-22 00:00:00',101.36,773,874,86.35,'Polar' from dual union all
select 3466,'003466','1965-082PX','deb Transtage 4','',11832,'Reentered','1991-11-18 00:00:00','1991-11-10 00:00:00',92.44,380,409,31.82,'Intermediate' from dual union all
select 34660,'034660','1991-025H','deb SOZ','SOZ',64194,'Reentered','2009-03-28 00:00:00','2009-03-23 00:00:00',167.81,235,7165,66.26,'Medium' from dual union all
select 34661,'034661','2009-014A','GPS IIR-20(M)','Navstar GPS SVN 49',68695,'In Earth orbit','','2013-01-10 00:00:00',717.92,20035,20327,55.94,'Medium' from dual union all
select 34662,'034662','2009-014B','Star 48B','Star 48B',68695,'Reentered','2015-09-28 00:00:00','2009-03-26 00:00:00',354.51,189,20240,40.09,'Highly Ellip' from dual union all
select 34663,'034663','2009-014C','Delta 340','Delta SSPS AJ-10-118K',68695,'Reentered','2009-06-29 00:00:00','2009-03-26 00:00:00',98.55,175,1204,34.37,'Intermediate' from dual union all
select 34664,'034664','1993-036XQ','deb Kosmos-2251','',65138,'Reentered','2014-11-04 00:00:00','2009-03-30 00:00:00',99.34,660,795,74.04,'Intermediate' from dual union all
select 34665,'034665','1991-025J','deb SOZ','SOZ',64194,'Reentered','2009-05-14 00:00:00','2009-03-24 00:00:00',292.49,357,16078,65.58,'Highly Ellip' from dual union all
select 34666,'034666','1991-025K','deb SOZ','SOZ',64194,'Reentered','2009-03-28 00:00:00','2009-03-24 00:00:00',131.81,241,4162,64.9,'Medium' from dual union all
select 34667,'034667','1991-025L','deb SOZ','SOZ',64194,'In Earth orbit','','2009-03-24 00:00:00',336.37,737,18550,65.47,'Highly Ellip' from dual union all
select 34668,'034668','1991-025M','deb SOZ','SOZ',64194,'Reentered','2009-05-30 00:00:00','2009-03-24 00:00:00',204.37,447,9782,64.86,'Medium' from dual union all
select 34669,'034669','2009-015A','Soyuz TMA-14','Soyuz 11F732A17 No. 224',68696,'Landed','2009-10-11 00:00:00','2009-03-27 00:00:00',89.46,224,272,51.64,'Intermediate' from dual union all
select 3467,'003467','1965-082PY','deb Transtage 4','',11832,'Reentered','2001-07-02 00:00:00','1992-02-03 00:00:00',97.16,606,640,32.16,'Intermediate' from dual union all
select 34670,'034670','2009-015B','11S510','11S510',68696,'Reentered','2009-03-30 00:00:00','2009-03-28 00:00:00',88.2,174,197,51.63,'Intermediate' from dual union all
select 34671,'034671','1993-036XR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-30 00:00:00',100.5,764,801,74.04,'Intermediate' from dual union all
select 34672,'034672','1993-036XS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-30 00:00:00',100.62,770,806,74.04,'Intermediate' from dual union all
select 34673,'034673','1993-036XT','deb Kosmos-2251','',65138,'Reentered','2009-08-18 00:00:00','2009-03-30 00:00:00',99.47,506,960,74.05,'Intermediate' from dual union all
select 34674,'034674','1993-036XU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-29 00:00:00',99.9,645,863,73.99,'Intermediate' from dual union all
select 34675,'034675','1993-036XV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-30 00:00:00',100.53,749,819,74.05,'Intermediate' from dual union all
select 34676,'034676','1993-036XW','deb Kosmos-2251','',65138,'Reentered','2015-04-01 00:00:00','2009-03-30 00:00:00',98.66,612,778,73.98,'Intermediate' from dual union all
select 34677,'034677','1993-036XX','deb Kosmos-2251','',65138,'Reentered','2010-12-24 00:00:00','2009-03-30 00:00:00',97.98,563,762,74.03,'Intermediate' from dual union all
select 34678,'034678','1993-036XY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-30 00:00:00',100.72,775,811,74.03,'Intermediate' from dual union all
select 34679,'034679','1993-036XZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-30 00:00:00',99.79,699,798,74.02,'Intermediate' from dual union all
select 3468,'003468','1965-082PZ','deb Transtage 4','',11832,'Reentered','1980-10-17 00:00:00','1968-09-30 00:00:00',99.84,719,782,32.18,'Intermediate' from dual union all
select 34680,'034680','1993-036YA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-30 00:00:00',100.3,763,782,74.05,'Intermediate' from dual union all
select 34681,'034681','1993-036YB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-30 00:00:00',99.68,707,780,74.08,'Intermediate' from dual union all
select 34682,'034682','1993-036YC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-30 00:00:00',100.05,742,780,74.04,'Intermediate' from dual union all
select 34683,'034683','1993-036YD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-30 00:00:00',100.24,746,794,74.03,'Intermediate' from dual union all
select 34684,'034684','1993-036YE','deb Kosmos-2251','',65138,'Reentered','2014-03-20 00:00:00','2009-03-30 00:00:00',99.03,627,798,73.99,'Intermediate' from dual union all
select 34685,'034685','1993-036YF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-30 00:00:00',100.37,760,793,74.04,'Intermediate' from dual union all
select 34686,'034686','1993-036YG','deb Kosmos-2251','',65138,'Reentered','2009-08-07 00:00:00','2009-03-30 00:00:00',100.86,501,1097,74.07,'Intermediate' from dual union all
select 34687,'034687','1993-036YH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-30 00:00:00',98.34,563,796,73.94,'Intermediate' from dual union all
select 34688,'034688','1993-036YJ','deb Kosmos-2251','',65138,'Reentered','2012-11-18 00:00:00','2009-03-30 00:00:00',99.18,657,781,74.05,'Intermediate' from dual union all
select 34689,'034689','1993-036YK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-03-30 00:00:00',105.43,771,1259,74.11,'Intermediate' from dual union all
select 3469,'003469','1968-085A','Molniya-1','Molniya-1 11F67 No. 14',16812,'Reentered','1976-07-16 00:00:00','1968-11-11 00:00:00',718.13,444,39928,65.06,'GEO Transfer' from dual union all
select 34690,'034690','1997-051KM','deb Iridium 33','',66131,'In Earth orbit','','2009-03-30 00:00:00',100.53,776,792,86.39,'Polar' from dual union all
select 34691,'034691','1997-051KN','deb Iridium 33','',66131,'Reentered','2009-04-14 00:00:00','2009-03-30 00:00:00',97.83,556,754,86.58,'Polar' from dual union all
select 34692,'034692','1997-051KP','deb Iridium 33','',66131,'Reentered','2011-10-01 00:00:00','2009-03-30 00:00:00',101.82,775,915,86.27,'Polar' from dual union all
select 34693,'034693','1997-051KQ','deb Iridium 33','',66131,'In Earth orbit','','2009-03-30 00:00:00',100.05,736,786,86.34,'Polar' from dual union all
select 34694,'034694','1997-051KR','deb Iridium 33','',66131,'Reentered','2014-12-30 00:00:00','2009-03-30 00:00:00',100.57,775,797,86.45,'Polar' from dual union all
select 34695,'034695','1997-051KS','deb Iridium 33','',66131,'Reentered','2009-04-18 00:00:00','2009-03-30 00:00:00',96.65,446,751,86.32,'Polar' from dual union all
select 34696,'034696','1997-051KT','deb Iridium 33','',66131,'In Earth orbit','','2009-03-30 00:00:00',103.08,779,1030,86.21,'Polar' from dual union all
select 34697,'034697','1997-051KU','deb Iridium 33','',66131,'Reentered','2015-01-11 00:00:00','2009-03-30 00:00:00',101.46,780,876,86.46,'Polar' from dual union all
select 34698,'034698','1997-051KV','deb Iridium 33','',66131,'Reentered','2016-07-18 00:00:00','2009-03-30 00:00:00',100.61,776,799,86.37,'Polar' from dual union all
select 34699,'034699','1997-051KW','deb Iridium 33','',66131,'Reentered','2010-03-20 00:00:00','2009-03-30 00:00:00',101.25,645,991,86.42,'Polar' from dual union all
select 347,'000347','62 A Iota 2','8A92E','8A92E',8143,'Reentered','1962-08-21 00:00:00','1962-08-05 00:00:00',89.75,206,319,65.01,'Intermediate' from dual union all
select 3470,'003470','1968-085B','11S510','11S510',16812,'Reentered','1968-10-30 00:00:00','1968-10-13 00:00:00',90.82,231,399,64.97,'Intermediate' from dual union all
select 34700,'034700','1997-051KX','deb Iridium 33','',66131,'Reentered','2014-11-19 00:00:00','2009-03-30 00:00:00',100.03,705,815,86.41,'Polar' from dual union all
select 34701,'034701','1997-051KY','deb Iridium 33','',66131,'Reentered','2014-03-26 00:00:00','2009-03-30 00:00:00',100.62,777,798,86.39,'Polar' from dual union all
select 34702,'034702','1997-051KZ','deb Iridium 33','',66131,'In Earth orbit','','2009-03-30 00:00:00',99.72,705,785,86.31,'Polar' from dual union all
select 34703,'034703','1997-051LA','deb Iridium 33','',66131,'Reentered','2014-04-06 00:00:00','2009-03-30 00:00:00',100.43,772,786,86.39,'Polar' from dual union all
select 34704,'034704','1997-051LB','deb Iridium 33','',66131,'Reentered','2014-05-14 00:00:00','2009-03-30 00:00:00',99.89,729,778,86.42,'Polar' from dual union all
select 34705,'034705','1997-051LC','deb Iridium 33','',66131,'In Earth orbit','','2009-03-30 00:00:00',101.24,779,856,86.5,'Polar' from dual union all
select 34706,'034706','1997-051LD','deb Iridium 33','',66131,'In Earth orbit','','2009-03-30 00:00:00',100.01,730,788,86.33,'Polar' from dual union all
select 34707,'034707','1997-051LE','deb Iridium 33','',66131,'Reentered','2011-04-29 00:00:00','2009-03-30 00:00:00',102.96,688,1110,86.56,'Polar' from dual union all
select 34708,'034708','1997-051LF','deb Iridium 33','',66131,'Reentered','2009-04-17 00:00:00','2009-03-30 00:00:00',98.65,524,865,86.78,'Polar' from dual union all
select 34709,'034709','1997-051LG','deb Iridium 33','',66131,'Reentered','2017-12-12 00:00:00','2009-03-30 00:00:00',100.4,776,779,86.35,'Polar' from dual union all
select 3471,'003471','1968-085C','BOZ','BOZ',16812,'Reentered','1968-11-04 00:00:00','1968-10-13 00:00:00',91.31,218,460,64.96,'Intermediate' from dual union all
select 34710,'034710','2009-016A','Eutelsat W2A','Eutelsat W2A',68697,'In Earth orbit','','2009-07-01 00:00:00',1436.09,35751,35822,.03,'Stationary' from dual union all
select 34711,'034711','2009-016B','Briz-M No. 99504','Briz-M',68697,'In Earth orbit','','2009-04-07 00:00:00',653.43,3761,33370,20.95,'GEO Transfer' from dual union all
select 34712,'034712','2009-016C','Briz-M DTB','Briz-M torus tank',68697,'In Earth orbit','','2009-04-06 00:00:00',275.16,298,14972,46.09,'Highly Ellip' from dual union all
select 34713,'034713','2009-017A','WGS SV-2','Wideband Global Satcom 2',68698,'In Earth orbit','','2009-04-08 00:00:00',1436.01,35780,35790,.1,'Stationary' from dual union all
select 34714,'034714','2009-017B','Centaur AV-016','Centaur AV-016',68698,'In Earth orbit','','2009-04-04 00:00:00',1326.31,408,66811,20.9,'Highly Ellip' from dual union all
select 34715,'034715','2008-062F','deb Blok-I','',68570,'Reentered','2010-01-19 00:00:00','2009-04-06 00:00:00',91.77,250,473,62.84,'Intermediate' from dual union all
select 34716,'034716','2008-062G','deb Blok-I','',68570,'Reentered','2009-12-09 00:00:00','2009-03-31 00:00:00',91.74,248,472,62.85,'Intermediate' from dual union all
select 34717,'034717','2008-062H','deb Blok-I','',68570,'Reentered','2009-09-26 00:00:00','2009-04-06 00:00:00',91.36,237,446,62.84,'Intermediate' from dual union all
select 34718,'034718','1993-036YL','deb Kosmos-2251','',65138,'Reentered','2015-03-01 00:00:00','2009-04-08 00:00:00',97.48,497,779,73.99,'Intermediate' from dual union all
select 34719,'034719','1993-036YM','deb Kosmos-2251','',65138,'Reentered','2015-02-28 00:00:00','2009-04-08 00:00:00',99.17,659,779,74.06,'Intermediate' from dual union all
select 3472,'003472','1968-086A','OPS 0964','STRAWMAN 1',16813,'Reentered Attached','1971-03-26 00:00:00','1968-11-04 00:00:00',94.48,481,505,75,'Intermediate' from dual union all
select 34720,'034720','1993-036YN','deb Kosmos-2251','',65138,'Reentered','2011-02-21 00:00:00','2009-04-08 00:00:00',97.97,557,767,74.01,'Intermediate' from dual union all
select 34721,'034721','1993-036YP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',104.21,774,1140,73.92,'Intermediate' from dual union all
select 34722,'034722','1993-036YQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-07 00:00:00',102.7,750,1023,73.89,'Intermediate' from dual union all
select 34723,'034723','1993-036YR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',99.98,565,951,74.01,'Intermediate' from dual union all
select 34724,'034724','1993-036YS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-07 00:00:00',104.99,770,1218,74.04,'Intermediate' from dual union all
select 34725,'034725','1993-036YT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-07 00:00:00',101.26,768,868,74.21,'Intermediate' from dual union all
select 34726,'034726','1993-036YU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',99.61,701,779,74.03,'Intermediate' from dual union all
select 34727,'034727','1993-036YV','deb Kosmos-2251','',65138,'Reentered','2015-11-07 00:00:00','2009-04-08 00:00:00',98.75,619,779,73.94,'Intermediate' from dual union all
select 34728,'034728','1993-036YW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',99.45,688,777,74.02,'Intermediate' from dual union all
select 34729,'034729','1993-036YX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-07 00:00:00',100.09,738,788,74.03,'Intermediate' from dual union all
select 3473,'003473','1968-087A','Kosmos-246','Zenit-4 No. 51',16814,'Landed','1968-10-12 00:00:00','1968-10-09 00:00:00',89.18,145,323,65.37,'Intermediate' from dual union all
select 34730,'034730','1993-036YY','deb Kosmos-2251','',65138,'Reentered','2016-06-26 00:00:00','2009-04-08 00:00:00',99.72,578,912,73.99,'Intermediate' from dual union all
select 34731,'034731','1993-036YZ','deb Kosmos-2251','',65138,'Reentered','2014-04-30 00:00:00','2009-04-08 00:00:00',99.13,646,788,74.03,'Intermediate' from dual union all
select 34732,'034732','1993-036ZA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',99.74,717,776,74.04,'Intermediate' from dual union all
select 34733,'034733','1993-036ZB','deb Kosmos-2251','',65138,'Reentered','2015-08-21 00:00:00','2009-04-08 00:00:00',100.38,762,792,74.02,'Intermediate' from dual union all
select 34734,'034734','1993-036ZC','deb Kosmos-2251','',65138,'Reentered','2009-06-23 00:00:00','2009-04-08 00:00:00',99.74,564,928,74.06,'Intermediate' from dual union all
select 34735,'034735','1993-036ZD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',100.06,699,824,74.04,'Intermediate' from dual union all
select 34736,'034736','1993-036ZE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',100.58,768,804,74.03,'Intermediate' from dual union all
select 34737,'034737','1993-036ZF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',100.84,770,826,74.05,'Intermediate' from dual union all
select 34738,'034738','1993-036ZG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',100.11,727,801,74.02,'Intermediate' from dual union all
select 34739,'034739','1993-036ZH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-07 00:00:00',99.41,660,801,74.03,'Intermediate' from dual union all
select 3474,'003474','1968-087B','11A57I','11A57I',16814,'Reentered','1968-10-08 00:00:00','1968-10-07 00:00:00',89.04,143,311,65.39,'Intermediate' from dual union all
select 34740,'034740','1993-036ZJ','deb Kosmos-2251','',65138,'Reentered','2011-12-23 00:00:00','2009-04-08 00:00:00',98.21,576,771,73.98,'Intermediate' from dual union all
select 34741,'034741','1993-036ZK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',99.79,566,930,74,'Intermediate' from dual union all
select 34742,'034742','1993-036ZL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',98.81,629,774,74.04,'Intermediate' from dual union all
select 34743,'034743','1993-036ZM','deb Kosmos-2251','',65138,'Reentered','2017-04-28 00:00:00','2009-04-08 00:00:00',99.68,703,784,74.02,'Intermediate' from dual union all
select 34744,'034744','1993-036ZN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',99.75,701,793,74.02,'Intermediate' from dual union all
select 34745,'034745','1993-036ZP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',99.81,723,776,74.04,'Intermediate' from dual union all
select 34746,'034746','1993-036ZQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',100.17,740,794,74.04,'Intermediate' from dual union all
select 34747,'034747','1993-036ZR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',98.91,638,776,74,'Intermediate' from dual union all
select 34748,'034748','1993-036ZS','deb Kosmos-2251','',65138,'Reentered','2013-05-18 00:00:00','2009-04-07 00:00:00',98.67,598,793,74.06,'Intermediate' from dual;
insert into SATELLITE
select 34749,'034749','1993-036ZT','deb Kosmos-2251','',65138,'Reentered','2009-06-16 00:00:00','2009-04-07 00:00:00',98.5,496,878,73.95,'Intermediate' from dual union all
select 3475,'003475','1965-082QA','deb Transtage 4','',11832,'Reentered','1973-05-03 00:00:00','1968-10-02 00:00:00',100.92,687,918,31.86,'Intermediate' from dual union all
select 34750,'034750','1993-036ZU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-07 00:00:00',100.48,736,827,74.12,'Intermediate' from dual union all
select 34751,'034751','1993-036ZV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',101.08,774,846,74.02,'Intermediate' from dual union all
select 34752,'034752','1993-036ZW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-07 00:00:00',98.8,625,778,74.02,'Intermediate' from dual union all
select 34753,'034753','1993-036ZX','deb Kosmos-2251','',65138,'Reentered','2015-03-09 00:00:00','2009-04-07 00:00:00',99.48,680,788,74.02,'Intermediate' from dual union all
select 34754,'034754','1993-036ZY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',100.33,752,797,74.03,'Intermediate' from dual union all
select 34755,'034755','1993-036ZZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-07 00:00:00',107.15,774,1416,73.93,'Intermediate' from dual union all
select 34756,'034756','1993-036AAA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-07 00:00:00',106.16,773,1324,74.01,'Intermediate' from dual union all
select 34757,'034757','1993-036AAB','deb Kosmos-2251','',65138,'Reentered','2014-02-18 00:00:00','2009-04-08 00:00:00',99.34,682,772,74.24,'Intermediate' from dual union all
select 34758,'034758','1993-036AAC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-07 00:00:00',106.79,771,1385,74.11,'Intermediate' from dual union all
select 34759,'034759','1993-036AAD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',100.72,766,820,74.04,'Intermediate' from dual union all
select 3476,'003476','1965-082QB','deb Transtage 4','',11832,'Reentered','1972-04-19 00:00:00','1967-07-31 00:00:00',106.23,741,1363,32,'Intermediate' from dual union all
select 34760,'034760','1993-036AAE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-08 00:00:00',102.05,770,942,74.04,'Intermediate' from dual union all
select 34761,'034761','1993-036AAF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-07 00:00:00',101.03,775,840,74.03,'Intermediate' from dual union all
select 34762,'034762','1993-036AAG','deb Kosmos-2251','',65138,'Reentered','2009-12-03 00:00:00','2009-04-08 00:00:00',99.73,634,857,74.07,'Intermediate' from dual union all
select 34763,'034763','1993-036AAH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-07 00:00:00',101.75,772,911,74.18,'Intermediate' from dual union all
select 34764,'034764','1997-051LH','deb Iridium 33','',66131,'In Earth orbit','','2009-04-08 00:00:00',100.66,778,802,86.38,'Polar' from dual union all
select 34765,'034765','1997-051LJ','deb Iridium 33','',66131,'In Earth orbit','','2009-04-08 00:00:00',100.65,777,802,86.34,'Polar' from dual union all
select 34766,'034766','1997-051LK','deb Iridium 33','',66131,'Reentered','2009-05-19 00:00:00','2009-04-08 00:00:00',103.66,699,1165,86.83,'Polar' from dual union all
select 34767,'034767','1997-051LL','deb Iridium 33','',66131,'Reentered','2014-02-01 00:00:00','2009-04-08 00:00:00',100.3,766,779,86.49,'Polar' from dual union all
select 34768,'034768','1997-051LM','deb Iridium 33','',66131,'Reentered','2011-12-12 00:00:00','2009-04-08 00:00:00',105.39,718,1308,86.21,'Polar' from dual union all
select 34769,'034769','1997-051LN','deb Iridium 33','',66131,'Reentered','2012-04-11 00:00:00','2009-04-08 00:00:00',104.51,830,1113,86.42,'Polar' from dual union all
select 3477,'003477','1965-082QC','deb Transtage 4','',11832,'Reentered','1971-04-07 00:00:00','1968-10-02 00:00:00',97.88,613,702,32.1,'Intermediate' from dual union all
select 34770,'034770','1997-051LP','deb Iridium 33','',66131,'Reentered','2009-04-15 00:00:00','2009-04-07 00:00:00',98.66,499,891,86.43,'Polar' from dual union all
select 34771,'034771','1997-051LQ','deb Iridium 33','',66131,'Reentered','2013-03-16 00:00:00','2009-04-07 00:00:00',100.86,778,821,86.41,'Polar' from dual union all
select 34772,'034772','1997-051LR','deb Iridium 33','',66131,'Reentered','2014-10-01 00:00:00','2009-04-08 00:00:00',99.94,733,779,86.41,'Polar' from dual union all
select 34773,'034773','1997-051LS','deb Iridium 33','',66131,'In Earth orbit','','2009-04-08 00:00:00',100.65,777,802,86.42,'Polar' from dual union all
select 34774,'034774','1997-051LT','deb Iridium 33','',66131,'In Earth orbit','','2009-04-08 00:00:00',100.43,776,782,86.44,'Polar' from dual union all
select 34775,'034775','1997-051LU','deb Iridium 33','',66131,'In Earth orbit','','2009-04-08 00:00:00',100.9,779,824,86.38,'Polar' from dual union all
select 34776,'034776','1997-051LV','deb Iridium 33','',66131,'Reentered','2014-12-01 00:00:00','2009-04-07 00:00:00',99.98,736,780,86.39,'Polar' from dual union all
select 34777,'034777','1997-051LW','deb Iridium 33','',66131,'Reentered','2010-08-17 00:00:00','2009-04-08 00:00:00',100.27,738,804,86.43,'Polar' from dual union all
select 34778,'034778','1997-051LX','deb Iridium 33','',66131,'Reentered','2009-04-22 00:00:00','2009-04-07 00:00:00',98.05,544,787,86.49,'Polar' from dual union all
select 34779,'034779','2009-018A','Beidou DW 2','er ke beidou daohang wei',68699,'In Earth orbit','','2009-04-29 00:00:00',1436.06,35774,35798,.95,'Stationary' from dual union all
select 3478,'003478','1965-082QD','deb Transtage 4','',11832,'Reentered','1980-03-04 00:00:00','1968-10-02 00:00:00',100.63,700,877,31.95,'Intermediate' from dual union all
select 34780,'034780','2009-018B','CZ-3C Stage 3','CZ-3C Stage 3',68699,'In Earth orbit','','2009-04-22 00:00:00',632.19,270,35774,20.61,'GEO Transfer' from dual union all
select 34781,'034781','1993-036AAJ','deb Kosmos-2251','',65138,'Reentered','2017-02-03 00:00:00','2009-04-16 00:00:00',99.73,688,803,74.07,'Intermediate' from dual union all
select 34782,'034782','1993-036AAK','deb Kosmos-2251','',65138,'Reentered','2012-05-03 00:00:00','2009-04-16 00:00:00',97.39,487,781,74.12,'Intermediate' from dual union all
select 34783,'034783','1993-036AAL','deb Kosmos-2251','',65138,'Reentered','2012-05-29 00:00:00','2009-04-16 00:00:00',97.5,504,775,73.91,'Intermediate' from dual union all
select 34784,'034784','1993-036AAM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-15 00:00:00',99.61,694,786,74.01,'Intermediate' from dual union all
select 34785,'034785','1993-036AAN','deb Kosmos-2251','',65138,'Reentered','2014-04-02 00:00:00','2009-04-16 00:00:00',97.65,500,793,74.08,'Intermediate' from dual union all
select 34786,'034786','1993-036AAP','deb Kosmos-2251','',65138,'Reentered','2012-08-11 00:00:00','2009-04-15 00:00:00',98.94,633,783,74.02,'Intermediate' from dual union all
select 34787,'034787','1993-036AAQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-16 00:00:00',100.04,742,779,74.04,'Intermediate' from dual union all
select 34788,'034788','1993-036AAR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-16 00:00:00',97.92,526,792,74,'Intermediate' from dual union all
select 34789,'034789','1993-036AAS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-15 00:00:00',102.35,771,968,74.2,'Intermediate' from dual union all
select 3479,'003479','1968-084C','ESRO 1A despin weight','',16811,'Reentered','1969-04-12 00:00:00','1968-11-03 00:00:00',101.01,250,1363,94.18,'Polar' from dual union all
select 34790,'034790','1993-036AAT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-16 00:00:00',101.89,766,931,74.05,'Intermediate' from dual union all
select 34791,'034791','1993-036AAU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-15 00:00:00',99.05,653,774,74.03,'Intermediate' from dual union all
select 34792,'034792','1993-036AAV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-16 00:00:00',100.48,767,796,74.03,'Intermediate' from dual union all
select 34793,'034793','1993-036AAW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-16 00:00:00',100.5,743,822,74,'Intermediate' from dual union all
select 34794,'034794','1993-036AAX','deb Kosmos-2251','',65138,'Reentered','2015-09-04 00:00:00','2009-04-16 00:00:00',98.42,584,783,73.92,'Intermediate' from dual union all
select 34795,'034795','1993-036AAY','deb Kosmos-2251','',65138,'Reentered','2015-04-18 00:00:00','2009-04-16 00:00:00',99.69,565,923,73.97,'Intermediate' from dual union all
select 34796,'034796','1993-036AAZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-15 00:00:00',100.52,694,873,74.04,'Intermediate' from dual union all
select 34797,'034797','1993-036ABA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-16 00:00:00',101.92,766,934,73.99,'Intermediate' from dual union all
select 34798,'034798','1991-025N','deb SOZ','SOZ',64194,'Reentered','2009-04-28 00:00:00','2009-04-17 00:00:00',168.5,225,7230,65.45,'Medium' from dual union all
select 34799,'034799','1991-025P','deb SOZ','SOZ',64194,'Reentered','2009-05-28 00:00:00','2009-04-17 00:00:00',337.83,1091,18289,65.76,'Highly Ellip' from dual union all
select 348,'000348','62 A Iota 3','Kosmos-7 sensor cover','',8143,'Reentered','1962-07-31 00:00:00','1962-07-29 00:00:00',90.01,200,350,65,'Intermediate' from dual union all
select 3480,'003480','1965-082QE','deb Transtage 4','',11832,'Reentered','1969-03-14 00:00:00','1968-10-02 00:00:00',98.76,665,733,32.58,'Intermediate' from dual union all
select 34800,'034800','1991-025Q','deb SOZ','SOZ',64194,'Reentered','2009-12-26 00:00:00','2009-04-16 00:00:00',333.93,660,18472,66.02,'Highly Ellip' from dual union all
select 34801,'034801','1991-025R','deb SOZ','SOZ',64194,'In Earth orbit','','2009-04-17 00:00:00',337.61,822,18543,65.51,'Highly Ellip' from dual union all
select 34802,'034802','1991-025S','deb SOZ','SOZ',64194,'Reentered','2009-05-13 00:00:00','2009-04-17 00:00:00',220.99,250,11210,65.27,'Medium' from dual union all
select 34803,'034803','1991-025T','deb SOZ','SOZ',64194,'Reentered','2009-04-21 00:00:00','2009-04-17 00:00:00',301.99,263,16802,66.99,'Highly Ellip' from dual union all
select 34804,'034804','1991-025U','deb SOZ','SOZ',64194,'Reentered','2009-05-28 00:00:00','2009-04-17 00:00:00',282.42,306,15455,65.71,'Highly Ellip' from dual union all
select 34805,'034805','1991-025V','deb SOZ','SOZ',64194,'Reentered','2009-05-02 00:00:00','2009-04-17 00:00:00',332.83,450,18612,66.73,'Highly Ellip' from dual union all
select 34806,'034806','1991-025W','deb SOZ','SOZ',64194,'Reentered','2009-04-18 00:00:00','2009-04-17 00:00:00',244.66,213,12946,69.96,'Medium' from dual union all
select 34807,'034807','2009-019A','RISAT-2','TecSAR-2',68700,'In Earth orbit','','2009-04-22 00:00:00',94.12,400,552,41.21,'Intermediate' from dual union all
select 34808,'034808','2009-019B','Anusat','Anna Univ. Satellite',68700,'Reentered','2012-04-18 00:00:00','2009-04-22 00:00:00',94.14,402,552,41.21,'Intermediate' from dual union all
select 34809,'034809','2009-019C','PS4','PS4 PSLV-C12',68700,'Reentered','2015-05-27 00:00:00','2009-04-22 00:00:00',94.74,453,559,41.1,'Intermediate' from dual union all
select 3481,'003481','1965-082QF','deb Transtage 4','',11832,'Reentered','1980-02-10 00:00:00','1968-10-02 00:00:00',101.6,697,971,31.73,'Intermediate' from dual union all
select 34810,'034810','2009-020A','SICRAL 1B','SICRAL 1B',68701,'In Earth orbit','','2009-05-01 00:00:00',1433.51,35692,35780,.05,'Drift GEO' from dual union all
select 34811,'034811','2009-020B','Blok DM-SL No. 30L','Blok DM-SL',68701,'In Earth orbit','','2009-04-28 00:00:00',793.41,8514,35509,.02,'Medium' from dual union all
select 34812,'034812','1993-036ABB','deb Kosmos-2251','',65138,'Reentered','2015-12-15 00:00:00','2009-04-20 00:00:00',99.67,702,784,74.02,'Intermediate' from dual union all
select 34813,'034813','1993-036ABC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-19 00:00:00',108.7,771,1563,73.94,'Intermediate' from dual union all
select 34814,'034814','1993-036ABD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-20 00:00:00',101.29,773,867,73.9,'Intermediate' from dual union all
select 34815,'034815','1993-036ABE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-20 00:00:00',99.61,668,812,74.05,'Intermediate' from dual union all
select 34816,'034816','1993-036ABF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-20 00:00:00',106.58,775,1362,73.88,'Intermediate' from dual union all
select 34817,'034817','1993-036ABG','deb Kosmos-2251','',65138,'Reentered','2014-05-22 00:00:00','2009-04-20 00:00:00',99.3,650,801,74.03,'Intermediate' from dual union all
select 34818,'034818','1993-036ABH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-20 00:00:00',100.58,768,804,74.03,'Intermediate' from dual union all
select 34819,'034819','1993-036ABJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-19 00:00:00',100.52,763,803,74.03,'Intermediate' from dual union all
select 3482,'003482','1965-082QG','deb Transtage 4','',11832,'Reentered','1975-03-05 00:00:00','1968-10-02 00:00:00',99.2,686,755,31.88,'Intermediate' from dual union all
select 34820,'034820','1993-036ABK','deb Kosmos-2251','',65138,'Reentered','2016-01-13 00:00:00','2009-04-20 00:00:00',99.16,542,895,74,'Intermediate' from dual union all
select 34821,'034821','1993-036ABL','deb Kosmos-2251','',65138,'Reentered','2014-12-02 00:00:00','2009-04-20 00:00:00',99.43,671,792,74.04,'Intermediate' from dual union all
select 34822,'034822','1993-036ABM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-20 00:00:00',99.94,725,787,74.02,'Intermediate' from dual union all
select 34823,'034823','1993-036ABN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-20 00:00:00',100.07,623,901,74.04,'Intermediate' from dual union all
select 34824,'034824','1997-051LY','deb Iridium 33','',66131,'Reentered','2009-07-01 00:00:00','2009-04-20 00:00:00',101.14,715,911,86.69,'Polar' from dual union all
select 34825,'034825','1997-051LZ','deb Iridium 33','',66131,'In Earth orbit','','2009-04-20 00:00:00',100.68,778,804,86.38,'Polar' from dual union all
select 34826,'034826','1997-051MA','deb Iridium 33','',66131,'Reentered','2011-05-14 00:00:00','2009-04-19 00:00:00',98.56,603,776,86.2,'Polar' from dual union all
select 34827,'034827','1997-051MB','deb Iridium 33','',66131,'In Earth orbit','','2009-04-20 00:00:00',100.69,778,805,86.39,'Polar' from dual union all
select 34828,'034828','1997-051MC','deb Iridium 33','',66131,'Reentered','2014-02-27 00:00:00','2009-04-19 00:00:00',99.93,726,785,86.14,'Polar' from dual union all
select 34829,'034829','1997-051MD','deb Iridium 33','',66131,'Reentered','2012-10-23 00:00:00','2009-04-20 00:00:00',98.63,532,855,86.49,'Polar' from dual union all
select 3483,'003483','1965-082QH','deb Transtage 4','',11832,'Reentered','1978-05-29 00:00:00','1968-10-02 00:00:00',99.33,699,755,32.63,'Intermediate' from dual union all
select 34830,'034830','1997-051ME','deb Iridium 33','',66131,'Reentered','2010-06-08 00:00:00','2009-04-20 00:00:00',97.78,504,801,86.01,'Polar' from dual union all
select 34831,'034831','1997-051MF','deb Iridium 33','',66131,'Reentered','2013-06-27 00:00:00','2009-04-20 00:00:00',101.65,746,928,86.35,'Polar' from dual union all
select 34832,'034832','1997-051MG','deb Iridium 33','',66131,'Reentered','2014-08-22 00:00:00','2009-04-20 00:00:00',100.05,740,782,86.42,'Polar' from dual union all
select 34833,'034833','1997-051MH','deb Iridium 33','',66131,'In Earth orbit','','2009-04-20 00:00:00',100.46,764,797,86.39,'Polar' from dual union all
select 34834,'034834','1997-051MJ','deb Iridium 33','',66131,'Reentered','2011-12-14 00:00:00','2009-04-20 00:00:00',99.54,697,776,86.32,'Polar' from dual union all
select 34835,'034835','1997-051MK','deb Iridium 33','',66131,'Reentered','2013-10-21 00:00:00','2009-04-20 00:00:00',100.94,774,832,86.44,'Polar' from dual union all
select 34836,'034836','2009-020C','deb Blok DM-SL','',68701,'In Earth orbit','','2009-04-20 00:00:00',794.81,8704,35386,.33,'Medium' from dual union all
select 34837,'034837','2009-020D','deb Blok DM-SL','',68701,'In Earth orbit','','2009-05-07 00:00:00',793.43,8518,35506,.06,'Medium' from dual union all
select 34838,'034838','2009-020E','deb Blok DM-SL','',68701,'In Earth orbit','','2009-04-20 00:00:00',793.93,8633,35415,.06,'Medium' from dual;
insert into SATELLITE
select 34839,'034839','2009-021A','Yaogan Weixing 6','Jianbing-7 01 xing',68702,'In Earth orbit','','2009-04-24 00:00:00',94.73,492,519,97.63,'Sun-Synch' from dual union all
select 3484,'003484','1968-088A','Kosmos-247','Zenit-2 No. 65',16815,'Landed','1968-10-19 00:00:00','1968-10-13 00:00:00',89.92,199,342,65.42,'Intermediate' from dual union all
select 34840,'034840','2009-021B','CZ-2C Stage 2','CZ-2C Stage 2',68702,'Reentered','2014-12-14 00:00:00','2009-04-24 00:00:00',94.76,431,583,97.6,'Sun-Synch' from dual union all
select 34841,'034841','2009-021C','Sep motor cover','',68702,'Reentered','2012-04-01 00:00:00','2009-04-24 00:00:00',96.02,490,645,97.84,'Sun-Synch' from dual union all
select 34842,'034842','2009-021D','Sep motor cover','',68702,'Reentered','2009-12-08 00:00:00','2009-04-24 00:00:00',93.38,342,538,97.4,'Sun-Synch' from dual union all
select 34843,'034843','2009-021E','Sep motor cover','',68702,'Reentered','2009-11-21 00:00:00','2009-04-24 00:00:00',93.29,336,536,97.83,'Sun-Synch' from dual union all
select 34844,'034844','2009-021F','Sep motor cover','',68702,'Reentered','2012-08-26 00:00:00','2009-04-24 00:00:00',96.22,494,660,97.42,'Sun-Synch' from dual union all
select 34845,'034845','1993-036ABP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-27 00:00:00',100.08,746,779,74.02,'Intermediate' from dual union all
select 34846,'034846','1993-036ABQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-28 00:00:00',100.7,767,816,74.04,'Intermediate' from dual union all
select 34847,'034847','1993-036ABR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-27 00:00:00',97.98,550,775,74.05,'Intermediate' from dual union all
select 34848,'034848','1993-036ABS','deb Kosmos-2251','',65138,'Reentered','2011-12-03 00:00:00','2009-04-28 00:00:00',97.54,512,770,73.93,'Intermediate' from dual union all
select 34849,'034849','1993-036ABT','deb Kosmos-2251','',65138,'Reentered','2013-12-09 00:00:00','2009-04-28 00:00:00',99.82,705,795,74.02,'Intermediate' from dual union all
select 3485,'003485','1968-088B','11A57I','11A57I',16815,'Reentered','1968-10-18 00:00:00','1968-10-13 00:00:00',89.49,189,310,65.42,'Intermediate' from dual union all
select 34850,'034850','1993-036ABU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-28 00:00:00',100.29,711,834,74.02,'Intermediate' from dual union all
select 34851,'034851','1993-036ABV','deb Kosmos-2251','',65138,'Reentered','2012-07-26 00:00:00','2009-04-27 00:00:00',98.45,520,850,73.88,'Intermediate' from dual union all
select 34852,'034852','1993-036ABW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-27 00:00:00',100.4,766,789,74.03,'Intermediate' from dual union all
select 34853,'034853','1993-036ABX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-27 00:00:00',99.69,684,804,74.08,'Intermediate' from dual union all
select 34854,'034854','1993-036ABY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-27 00:00:00',99.07,646,782,74.03,'Intermediate' from dual union all
select 34855,'034855','1993-036ABZ','deb Kosmos-2251','',65138,'Reentered','2016-04-07 00:00:00','2009-04-28 00:00:00',98.85,634,774,73.97,'Intermediate' from dual union all
select 34856,'034856','1993-036ACA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-28 00:00:00',99.25,655,791,74.02,'Intermediate' from dual union all
select 34857,'034857','1993-036ACB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-28 00:00:00',100.24,703,837,74.02,'Intermediate' from dual union all
select 34858,'034858','1993-036ACC','deb Kosmos-2251','',65138,'Reentered','2017-03-05 00:00:00','2009-04-28 00:00:00',100.23,685,854,74.07,'Intermediate' from dual union all
select 34859,'034859','1993-036ACD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-27 00:00:00',100.28,767,777,74.05,'Intermediate' from dual union all
select 3486,'003486','1968-089A','Apollo 7','Apollo CM-101',16816,'Landed','1968-10-22 00:00:00','1968-10-13 00:00:00',89.79,227,301,31.65,'Intermediate' from dual union all
select 34860,'034860','1993-036ACE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-28 00:00:00',100.38,747,806,74.03,'Intermediate' from dual union all
select 34861,'034861','1993-036ACF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-04-28 00:00:00',100.55,759,811,74.04,'Intermediate' from dual union all
select 34862,'034862','1993-036ACG','deb Kosmos-2251','',65138,'Reentered','2014-03-21 00:00:00','2009-04-28 00:00:00',99.03,634,791,74.02,'Intermediate' from dual union all
select 34863,'034863','1993-036ACH','deb Kosmos-2251','',65138,'Reentered','2015-08-30 00:00:00','2009-04-28 00:00:00',99.22,621,822,74.03,'Intermediate' from dual union all
select 34864,'034864','1993-036ACJ','deb Kosmos-2251','',65138,'Reentered','2010-09-19 00:00:00','2009-04-28 00:00:00',96.13,401,745,74.06,'Intermediate' from dual union all
select 34865,'034865','1993-036ACK','deb Kosmos-2251','',65138,'Reentered','2012-06-18 00:00:00','2009-04-27 00:00:00',97.9,541,775,74,'Intermediate' from dual union all
select 34866,'034866','1997-051ML','deb Iridium 33','',66131,'Reentered','2015-01-16 00:00:00','2009-04-27 00:00:00',100.13,750,780,86.35,'Polar' from dual union all
select 34867,'034867','1997-051MM','deb Iridium 33','',66131,'Reentered','2013-12-18 00:00:00','2009-04-27 00:00:00',100.1,708,818,86.28,'Polar' from dual union all
select 34868,'034868','1997-051MN','deb Iridium 33','',66131,'In Earth orbit','','2009-04-27 00:00:00',101.55,699,966,86.55,'Polar' from dual union all
select 34869,'034869','1997-051MP','deb Iridium 33','',66131,'In Earth orbit','','2009-04-27 00:00:00',101.99,778,928,86.46,'Polar' from dual union all
select 3487,'003487','1968-089B','Saturn S-IVB-205','Saturn S-IVB-205',16816,'Reentered','1968-10-18 00:00:00','1968-10-13 00:00:00',89.54,217,286,31.63,'Intermediate' from dual union all
select 34870,'034870','1997-051MQ','deb Iridium 33','',66131,'In Earth orbit','','2009-04-27 00:00:00',100.07,737,786,86.32,'Polar' from dual union all
select 34871,'034871','2009-022A','Kosmos-2450','Kobal''t-M No. 560',68703,'Landed','2009-07-27 00:00:00','2009-04-30 00:00:00',89.52,168,334,67.14,'Intermediate' from dual union all
select 34872,'034872','2009-022B','11S510','11S510',68703,'Reentered','2009-05-04 00:00:00','2009-04-30 00:00:00',89.26,159,317,67.14,'Intermediate' from dual union all
select 34873,'034873','1993-036ACL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-03 00:00:00',101.14,768,858,74.02,'Intermediate' from dual union all
select 34874,'034874','1993-036ACM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-04 00:00:00',101.34,774,870,74.04,'Intermediate' from dual union all
select 34875,'034875','1993-036ACN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-03 00:00:00',100.15,753,779,74.05,'Intermediate' from dual union all
select 34876,'034876','1993-036ACP','deb Kosmos-2251','',65138,'Reentered','2011-10-07 00:00:00','2009-05-04 00:00:00',97.23,478,775,74.03,'Intermediate' from dual union all
select 34877,'034877','1993-036ACQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-04 00:00:00',97.9,541,775,73.95,'Intermediate' from dual union all
select 34878,'034878','1993-036ACR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-03 00:00:00',107.6,774,1458,73.96,'Intermediate' from dual union all
select 34879,'034879','1993-036ACS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-04 00:00:00',109.56,772,1641,73.92,'Intermediate' from dual union all
select 3488,'003488','1965-082QJ','deb Transtage 4','',11832,'In Earth orbit','','1992-02-02 00:00:00',98.95,684,733,32.24,'Intermediate' from dual union all
select 34880,'034880','1993-036ACT','deb Kosmos-2251','',65138,'Reentered','2013-05-03 00:00:00','2009-05-04 00:00:00',97.99,552,774,73.86,'Intermediate' from dual union all
select 34881,'034881','1993-036ACU','deb Kosmos-2251','',65138,'Reentered','2015-01-10 00:00:00','2009-05-03 00:00:00',97.66,509,785,74.08,'Intermediate' from dual union all
select 34882,'034882','1993-036ACV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-04 00:00:00',98.98,645,775,74.05,'Intermediate' from dual union all
select 34883,'034883','1993-036ACW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-03 00:00:00',103.05,772,1034,74.18,'Intermediate' from dual union all
select 34884,'034884','1993-036ACX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-04 00:00:00',100.43,762,796,74.03,'Intermediate' from dual union all
select 34885,'034885','1993-036ACY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-04 00:00:00',100.43,764,794,74.04,'Intermediate' from dual union all
select 34886,'034886','1993-036ACZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-04 00:00:00',98.56,594,786,73.98,'Intermediate' from dual union all
select 34887,'034887','1993-036ADA','deb Kosmos-2251','',65138,'Reentered','2015-01-04 00:00:00','2009-05-04 00:00:00',98.57,607,774,73.95,'Intermediate' from dual union all
select 34888,'034888','1993-036ADB','deb Kosmos-2251','',65138,'Reentered','2014-09-25 00:00:00','2009-05-04 00:00:00',99.3,653,798,74.03,'Intermediate' from dual union all
select 34889,'034889','1997-051MR','deb Iridium 33','',66131,'In Earth orbit','','2009-05-04 00:00:00',100.63,776,800,86.38,'Polar' from dual union all
select 3489,'003489','1965-082QK','deb Transtage 4','',11832,'Reentered','1970-06-03 00:00:00','1968-10-17 00:00:00',98.41,672,694,32.29,'Intermediate' from dual union all
select 34890,'034890','1997-051MS','deb Iridium 33','',66131,'In Earth orbit','','2009-05-04 00:00:00',101.95,770,932,86.32,'Polar' from dual union all
select 34891,'034891','1997-051MT','deb Iridium 33','',66131,'Reentered','2013-09-22 00:00:00','2009-05-03 00:00:00',98.87,612,798,86.16,'Polar' from dual union all
select 34892,'034892','1997-051MU','deb Iridium 33','',66131,'Reentered','2013-02-22 00:00:00','2009-05-03 00:00:00',98.83,628,778,86.28,'Polar' from dual union all
select 34893,'034893','1997-051MV','deb Iridium 33','',66131,'In Earth orbit','','2009-05-03 00:00:00',101.48,776,882,86.17,'Polar' from dual union all
select 34894,'034894','1997-051MW','deb Iridium 33','',66131,'Reentered','2013-07-06 00:00:00','2009-05-03 00:00:00',101.25,760,876,86.36,'Polar' from dual union all
select 34895,'034895','1997-051MX','deb Iridium 33','',66131,'Reentered','2017-05-01 00:00:00','2009-05-03 00:00:00',100.48,683,880,86.44,'Polar' from dual union all
select 34896,'034896','1997-051MY','deb Iridium 33','',66131,'In Earth orbit','','2009-05-04 00:00:00',99.92,726,784,86.34,'Polar' from dual union all
select 34897,'034897','1997-051MZ','deb Iridium 33','',66131,'In Earth orbit','','2009-05-04 00:00:00',100.74,769,818,86.43,'Polar' from dual union all
select 34898,'034898','1997-051NA','deb Iridium 33','',66131,'In Earth orbit','','2009-05-03 00:00:00',99.74,715,778,86.34,'Polar' from dual union all
select 34899,'034899','1997-051NB','deb Iridium 33','',66131,'In Earth orbit','','2009-05-04 00:00:00',101.29,777,863,86.4,'Polar' from dual union all
select 349,'000349','62 A Iota 4','Kosmos-7 sensor cover','',8143,'Reentered','1962-07-30 00:00:00','1962-07-29 00:00:00',90.01,200,350,65,'Intermediate' from dual union all
select 3490,'003490','1965-082QL','deb Transtage 4','',11832,'Reentered','1978-01-13 00:00:00','1967-02-25 00:00:00',100.5,719,846,32.04,'Intermediate' from dual union all
select 34900,'034900','1997-051NC','deb Iridium 33','',66131,'Reentered','2013-06-03 00:00:00','2009-05-04 00:00:00',100.85,775,823,86.41,'Polar' from dual union all
select 34901,'034901','1997-051ND','deb Iridium 33','',66131,'Reentered','2013-07-24 00:00:00','2009-05-04 00:00:00',100.95,753,855,86.41,'Polar' from dual union all
select 34902,'034902','1997-051NE','deb Iridium 33','',66131,'Reentered','2012-09-21 00:00:00','2009-05-05 00:00:00',101.03,774,841,86.38,'Polar' from dual union all
select 34903,'034903','2009-023A','STSS-ATRR','STSS Adv Tech Risk Reduc',68704,'In Earth orbit','','2009-05-07 00:00:00',102.4,867,877,98.95,'Sun-Synch' from dual union all
select 34904,'034904','2009-023B','Delta 341','Delta SSPS AJ-10-118K',68704,'Reentered','2009-07-30 00:00:00','2009-05-05 00:00:00',95.45,181,900,112.78,'Retrograde' from dual union all
select 34905,'034905','2009-024A','Progress M-02M','Progress 7K-TGM No.402',68705,'Deorbited','2009-07-13 00:00:00','2009-05-09 00:00:00',90.79,291,335,51.63,'Intermediate' from dual union all
select 34906,'034906','2009-024B','11S510','11S510',68705,'Reentered','2009-05-10 00:00:00','2009-05-08 00:00:00',88.42,181,212,51.62,'Intermediate' from dual union all
select 34907,'034907','1993-036ADC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-10 00:00:00',100.66,771,809,74.03,'Intermediate' from dual union all
select 34908,'034908','1993-036ADD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-10 00:00:00',101.02,772,841,74.06,'Intermediate' from dual union all
select 34909,'034909','1993-036ADE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-10 00:00:00',98.78,617,784,74.02,'Intermediate' from dual union all
select 3491,'003491','1965-082QM','deb Transtage 4','',11832,'Reentered','1980-06-08 00:00:00','1968-10-24 00:00:00',101.33,714,930,31.47,'Intermediate' from dual union all
select 34910,'034910','1993-036ADF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-10 00:00:00',108.14,773,1509,73.87,'Intermediate' from dual union all
select 34911,'034911','1993-036ADG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-11 00:00:00',99.02,648,776,74.04,'Intermediate' from dual union all
select 34912,'034912','1993-036ADH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-10 00:00:00',99.08,652,778,74.17,'Intermediate' from dual union all
select 34913,'034913','1993-036ADJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-11 00:00:00',100.44,764,795,74.03,'Intermediate' from dual union all
select 34914,'034914','1993-036ADK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-11 00:00:00',99.84,594,908,74.01,'Intermediate' from dual union all
select 34915,'034915','1993-036ADL','deb Kosmos-2251','',65138,'Reentered','2012-03-23 00:00:00','2009-05-11 00:00:00',97.42,492,779,73.77,'Intermediate' from dual union all
select 34916,'034916','1993-036ADM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-11 00:00:00',99.79,642,855,74.09,'Intermediate' from dual union all
select 34917,'034917','1993-036ADN','deb Kosmos-2251','',65138,'Reentered','2014-09-16 00:00:00','2009-05-11 00:00:00',99.32,657,796,74.03,'Intermediate' from dual union all
select 34918,'034918','1993-036ADP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-11 00:00:00',101.15,773,853,74.08,'Intermediate' from dual union all
select 34919,'034919','1993-036ADQ','deb Kosmos-2251','',65138,'Reentered','2012-03-03 00:00:00','2009-05-11 00:00:00',97.43,497,775,73.98,'Intermediate' from dual union all
select 3492,'003492','1965-082QN','deb Transtage 4','',11832,'Reentered','2000-11-22 00:00:00','1992-02-03 00:00:00',99.82,523,976,30.48,'Intermediate' from dual union all
select 34920,'034920','1993-036ADR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-11 00:00:00',100.58,764,808,74.04,'Intermediate' from dual union all
select 34921,'034921','1993-036ADS','deb Kosmos-2251','',65138,'Reentered','2015-11-08 00:00:00','2009-05-11 00:00:00',100.41,741,815,74.04,'Intermediate' from dual union all
select 34922,'034922','1993-036ADT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-11 00:00:00',98.54,580,798,74.16,'Intermediate' from dual union all
select 34923,'034923','1993-036ADU','deb Kosmos-2251','',65138,'Reentered','2013-10-15 00:00:00','2009-05-11 00:00:00',97.86,542,771,73.99,'Intermediate' from dual union all
select 34924,'034924','1993-036ADV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-11 00:00:00',100.12,707,822,74.05,'Intermediate' from dual union all
select 34925,'034925','1993-036ADW','deb Kosmos-2251','',65138,'Reentered','2012-01-27 00:00:00','2009-05-11 00:00:00',97.67,524,771,73.98,'Intermediate' from dual union all
select 34926,'034926','1997-051NF','deb Iridium 33','',66131,'In Earth orbit','','2009-05-10 00:00:00',103.3,777,1053,86.18,'Polar' from dual union all
select 34927,'034927','1997-051NG','deb Iridium 33','',66131,'Reentered','2013-10-18 00:00:00','2009-05-10 00:00:00',99.86,729,774,86.35,'Polar' from dual union all
select 34928,'034928','1997-051NH','deb Iridium 33','',66131,'In Earth orbit','','2009-05-11 00:00:00',99.77,718,778,86.41,'Polar' from dual;
insert into SATELLITE
select 34929,'034929','1997-051NJ','deb Iridium 33','',66131,'Reentered','2013-06-07 00:00:00','2009-05-11 00:00:00',100.3,764,782,86.45,'Polar' from dual union all
select 3493,'003493','1965-082QP','deb Transtage 4','',11832,'Reentered','1969-01-21 00:00:00','1968-10-26 00:00:00',95.96,539,591,32.81,'Intermediate' from dual union all
select 34930,'034930','1997-051NK','deb Iridium 33','',66131,'Reentered','2012-05-26 00:00:00','2009-05-11 00:00:00',100.63,759,818,86.39,'Polar' from dual union all
select 34931,'034931','1997-051NL','deb Iridium 33','',66131,'Reentered','2016-09-17 00:00:00','2009-05-11 00:00:00',100.91,776,828,86.36,'Polar' from dual union all
select 34932,'034932','1997-051NM','deb Iridium 33','',66131,'Reentered','2009-05-18 00:00:00','2009-05-10 00:00:00',98.98,447,973,86.02,'Polar' from dual union all
select 34933,'034933','2009-025A','Atlantis (STS-125)','OV-104',68706,'Landed','2009-05-24 00:00:00','2009-05-19 00:00:00',95.94,562,566,28.47,'Intermediate' from dual union all
select 34934,'034934','1991-025X','deb SOZ','SOZ',64194,'Reentered','2009-05-19 00:00:00','2009-05-13 00:00:00',175.26,229,7762,65.94,'Medium' from dual union all
select 34935,'034935','1991-025Y','deb SOZ','SOZ',64194,'Reentered','2009-05-17 00:00:00','2009-05-13 00:00:00',178.32,219,8014,65.94,'Medium' from dual union all
select 34936,'034936','1991-025Z','deb SOZ','SOZ',64194,'Reentered','2009-11-20 00:00:00','2009-05-13 00:00:00',286.63,333,15711,65.99,'Highly Ellip' from dual union all
select 34937,'034937','2009-026A','Herschel','Herschel',68707,'Deep Space','2009-05-15 00:00:00','2009-05-15 00:00:00',73462.72,273,1149048,5.93,'Cislunar' from dual union all
select 34938,'034938','2009-026B','Planck','Planck',68707,'Deep Space','2009-05-15 00:00:00','2009-05-22 00:00:00',72199.13,369,1135587,3.81,'Cislunar' from dual union all
select 34939,'034939','2009-026C','ESC-A 546','Ariane 5 ESC-A',68707,'Deep Space','2009-05-15 00:00:00','2009-05-14 00:00:00',77729.44,269,1193622,6,'Cislunar' from dual union all
select 3494,'003494','1965-082QQ','deb Transtage 4','',11832,'In Earth orbit','','1992-02-03 00:00:00',99.3,703,748,31.89,'Intermediate' from dual union all
select 34940,'034940','2009-026D','Sylda 5F','Sylda 5F',68707,'Deep Space','2009-05-15 00:00:00','2009-05-22 00:00:00',81639.32,366,1233658,2.87,'Cislunar' from dual union all
select 34941,'034941','2009-027A','Protostar 2','Indostar II/Protostar II',68708,'In Earth orbit','','2009-06-29 00:00:00',1436.06,35784,35788,.04,'Stationary' from dual union all
select 34942,'034942','2009-027B','Briz-M No. 99505','Briz-M',68708,'In Earth orbit','','2009-05-18 00:00:00',920.7,14171,35773,8.43,'Medium' from dual union all
select 34943,'034943','2009-027C','Briz-M DTB','Briz-M torus tank',68708,'Reentered','2016-01-15 00:00:00','2009-05-18 00:00:00',493.56,364,28256,49.18,'Highly Ellip' from dual union all
select 34944,'034944','1993-036ADX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',102.67,704,1066,73.76,'Intermediate' from dual union all
select 34945,'034945','1993-036ADY','deb Kosmos-2251','',65138,'Reentered','2015-05-01 00:00:00','2009-05-18 00:00:00',98.53,560,817,73.94,'Intermediate' from dual union all
select 34946,'034946','1993-036ADZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',99.67,695,791,74.03,'Intermediate' from dual union all
select 34947,'034947','1993-036AEA','deb Kosmos-2251','',65138,'Reentered','2012-08-09 00:00:00','2009-05-18 00:00:00',98.28,533,820,73.98,'Intermediate' from dual union all
select 34948,'034948','1993-036AEB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-17 00:00:00',100.78,770,822,74.03,'Intermediate' from dual union all
select 34949,'034949','1993-036AEC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-17 00:00:00',98.83,629,777,74.02,'Intermediate' from dual union all
select 3495,'003495','1965-082QR','deb Transtage 4','',11832,'Reentered','1969-09-21 00:00:00','1968-10-16 00:00:00',96.68,558,642,32.58,'Intermediate' from dual union all
select 34950,'034950','1993-036AED','deb Kosmos-2251','',65138,'Reentered','2013-05-26 00:00:00','2009-05-18 00:00:00',98.08,484,850,73.69,'Intermediate' from dual union all
select 34951,'034951','1993-036AEE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',99.31,665,786,74.02,'Intermediate' from dual union all
select 34952,'034952','1993-036AEF','deb Kosmos-2251','',65138,'Reentered','2012-07-26 00:00:00','2009-05-18 00:00:00',96.92,403,820,73.86,'Intermediate' from dual union all
select 34953,'034953','1993-036AEG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',99.82,686,813,74.1,'Intermediate' from dual union all
select 34954,'034954','1993-036AEH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',100.67,762,819,74,'Intermediate' from dual union all
select 34955,'034955','1993-036AEJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',98.27,575,777,74.05,'Intermediate' from dual union all
select 34956,'034956','1993-036AEK','deb Kosmos-2251','',65138,'Reentered','2012-02-27 00:00:00','2009-05-17 00:00:00',97.42,501,770,74.11,'Intermediate' from dual union all
select 34957,'034957','1993-036AEL','deb Kosmos-2251','',65138,'Reentered','2012-09-20 00:00:00','2009-05-18 00:00:00',99.46,673,792,74.03,'Intermediate' from dual union all
select 34958,'034958','1993-036AEM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',97.89,522,793,74.02,'Intermediate' from dual union all
select 34959,'034959','1993-036AEN','deb Kosmos-2251','',65138,'Reentered','2012-05-02 00:00:00','2009-05-18 00:00:00',96.46,405,773,73.95,'Intermediate' from dual union all
select 3496,'003496','1965-082QS','deb Transtage 4','',11832,'Reentered','1979-04-05 00:00:00','1968-11-01 00:00:00',101.82,702,988,33.02,'Intermediate' from dual union all
select 34960,'034960','1993-036AEP','deb Kosmos-2251','',65138,'Reentered','2014-08-27 00:00:00','2009-05-18 00:00:00',99.91,732,777,74.03,'Intermediate' from dual union all
select 34961,'034961','1993-036AEQ','deb Kosmos-2251','',65138,'Reentered','2014-05-18 00:00:00','2009-05-18 00:00:00',98.36,589,771,73.92,'Intermediate' from dual union all
select 34962,'034962','1993-036AER','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',100.32,759,789,74.03,'Intermediate' from dual union all
select 34963,'034963','1993-036AES','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-17 00:00:00',101.05,765,852,74.18,'Intermediate' from dual union all
select 34964,'034964','1993-036AET','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',104.28,774,1147,73.99,'Intermediate' from dual union all
select 34965,'034965','1993-036AEU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',99.95,736,776,74.05,'Intermediate' from dual union all
select 34966,'034966','1993-036AEV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-17 00:00:00',99.42,678,784,73.99,'Intermediate' from dual union all
select 34967,'034967','1993-036AEW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-17 00:00:00',100.73,758,828,74,'Intermediate' from dual union all
select 34968,'034968','1993-036AEX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',99.05,647,780,74.04,'Intermediate' from dual union all
select 34969,'034969','1993-036AEY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-17 00:00:00',101.02,747,867,74.01,'Intermediate' from dual union all
select 3497,'003497','1965-082QT','deb Transtage 4','',11832,'Reentered','1980-03-19 00:00:00','1968-11-14 00:00:00',102.75,713,1065,32.83,'Intermediate' from dual union all
select 34970,'034970','1993-036AEZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-17 00:00:00',99.49,680,788,74.03,'Intermediate' from dual union all
select 34971,'034971','1993-036AFA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-17 00:00:00',99.36,673,783,73.98,'Intermediate' from dual union all
select 34972,'034972','1993-036AFB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',100.31,727,820,74,'Intermediate' from dual union all
select 34973,'034973','1993-036AFC','deb Kosmos-2251','',65138,'Reentered','2011-05-26 00:00:00','2009-05-17 00:00:00',96.6,432,760,74.05,'Intermediate' from dual union all
select 34974,'034974','1993-036AFD','deb Kosmos-2251','',65138,'Reentered','2011-11-27 00:00:00','2009-05-18 00:00:00',97.72,525,774,74.01,'Intermediate' from dual union all
select 34975,'034975','1993-036AFE','deb Kosmos-2251','',65138,'Reentered','2012-09-05 00:00:00','2009-05-17 00:00:00',98.34,540,819,73.95,'Intermediate' from dual union all
select 34976,'034976','1993-036AFF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-17 00:00:00',99.69,687,801,74.07,'Intermediate' from dual union all
select 34977,'034977','1993-036AFG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',100.75,742,847,74.03,'Intermediate' from dual union all
select 34978,'034978','1993-036AFH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',99.49,688,781,74.03,'Intermediate' from dual union all
select 34979,'034979','1993-036AFJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',99.6,678,801,74.05,'Intermediate' from dual union all
select 3498,'003498','1965-082QU','deb Transtage 4','',11832,'Reentered','1991-10-27 00:00:00','1991-10-27 00:00:00',89.89,264,274,32.01,'Intermediate' from dual union all
select 34980,'034980','1993-036AFK','deb Kosmos-2251','',65138,'Reentered','2009-07-06 00:00:00','2009-05-18 00:00:00',99.51,591,880,74.04,'Intermediate' from dual union all
select 34981,'034981','1993-036AFL','deb Kosmos-2251','',65138,'Reentered','2014-01-26 00:00:00','2009-05-18 00:00:00',97.08,460,777,73.93,'Intermediate' from dual union all
select 34982,'034982','1997-051NN','deb Iridium 33','',66131,'Reentered','2015-05-15 00:00:00','2009-05-18 00:00:00',99.82,719,781,86.42,'Polar' from dual union all
select 34983,'034983','1997-051NP','deb Iridium 33','',66131,'In Earth orbit','','2009-05-18 00:00:00',101.34,772,873,86.38,'Polar' from dual union all
select 34984,'034984','1997-051NQ','deb Iridium 33','',66131,'Reentered','2012-01-24 00:00:00','2009-05-17 00:00:00',98.01,524,803,86.35,'Polar' from dual union all
select 34985,'034985','1997-051NR','deb Iridium 33','',66131,'In Earth orbit','','2009-05-18 00:00:00',100.73,775,812,86.37,'Polar' from dual union all
select 34986,'034986','1997-051NS','deb Iridium 33','',66131,'In Earth orbit','','2009-05-17 00:00:00',101.03,777,838,86.39,'Polar' from dual union all
select 34987,'034987','1997-051NT','deb Iridium 33','',66131,'In Earth orbit','','2009-05-18 00:00:00',100.16,758,775,86.36,'Polar' from dual union all
select 34988,'034988','1983-113H','deb DMSP F-7','',55870,'Reentered','2010-12-22 00:00:00','2009-05-18 00:00:00',100.87,783,817,98.9,'Sun-Synch' from dual union all
select 34989,'034989','1993-036AFM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-19 00:00:00',99.96,730,784,74.03,'Intermediate' from dual union all
select 3499,'003499','1965-082QV','deb Transtage 4','',11832,'Reentered','1980-05-22 00:00:00','1967-06-19 00:00:00',98.81,659,744,31.94,'Intermediate' from dual union all
select 34990,'034990','1993-036AFN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-19 00:00:00',100.47,767,795,74.05,'Intermediate' from dual union all
select 34991,'034991','1993-036AFP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',100.14,750,780,74.05,'Intermediate' from dual union all
select 34992,'034992','1993-036AFQ','deb Kosmos-2251','',65138,'Reentered','2015-12-11 00:00:00','2009-05-19 00:00:00',97.61,515,773,73.86,'Intermediate' from dual union all
select 34993,'034993','1993-036AFR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-19 00:00:00',101.13,762,863,74.03,'Intermediate' from dual union all
select 34994,'034994','1993-036AFS','deb Kosmos-2251','',65138,'Reentered','2014-04-12 00:00:00','2009-05-19 00:00:00',96.6,411,780,73.91,'Intermediate' from dual union all
select 34995,'034995','1993-036AFT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-19 00:00:00',100.53,734,834,73.85,'Intermediate' from dual union all
select 34996,'034996','1993-036AFU','deb Kosmos-2251','',65138,'Reentered','2014-08-15 00:00:00','2009-05-18 00:00:00',98.74,532,865,74.01,'Intermediate' from dual union all
select 34997,'034997','1993-036AFV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',101.48,729,929,73.89,'Intermediate' from dual union all
select 34998,'034998','1993-036AFW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-18 00:00:00',100.31,755,792,74.02,'Intermediate' from dual union all
select 34999,'034999','1993-036AFX','deb Kosmos-2251','',65138,'Reentered','2017-04-06 00:00:00','2009-05-19 00:00:00',100.05,734,788,74.03,'Intermediate' from dual union all
select 35,'000035','60 Epsilon 2','8K72E','8K72E',6320,'Reentered','1960-07-17 00:00:00','1960-06-24 00:00:00',90.27,288,288,64.89,'Intermediate' from dual union all
select 350,'000350','61 Omicron 111','deb r/b Ablestar','',7273,'In Earth orbit','','1991-04-11 00:00:00',101.78,747,939,66.91,'Intermediate' from dual union all
select 3500,'003500','1965-082QW','deb Transtage 4','',11832,'Reentered','1970-07-14 00:00:00','1968-10-30 00:00:00',100.26,725,817,32,'Intermediate' from dual union all
select 35000,'035000','1993-036AFY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-19 00:00:00',99.83,681,819,74.05,'Intermediate' from dual union all
select 35001,'035001','2009-028A','Tacsat 3','Tacsat 3',68709,'Reentered','2012-04-30 00:00:00','2009-05-22 00:00:00',93.57,432,467,40.46,'Intermediate' from dual union all
select 35002,'035002','2009-028B','Pharmasat','Pharmasat',68709,'Reentered','2012-08-14 00:00:00','2009-05-22 00:00:00',93.52,428,466,40.47,'Intermediate' from dual union all
select 35003,'035003','2009-028C','CP6','CalPoly CP6',68709,'Reentered','2011-10-06 00:00:00','2009-05-22 00:00:00',93.5,426,466,40.46,'Intermediate' from dual union all
select 35004,'035004','2009-028D','HawkSat-1','HawkSat-1',68709,'Reentered','2011-09-04 00:00:00','2009-05-22 00:00:00',93.5,426,466,40.46,'Intermediate' from dual union all
select 35005,'035005','2009-028E','Aerocube-3','Aerocube 3/Drag device',68709,'Reentered','2011-01-06 00:00:00','2009-05-22 00:00:00',93.51,427,466,40.48,'Intermediate' from dual union all
select 35006,'035006','2009-028F','Minotaur Stage 4','Orion 38',68709,'Reentered','2012-11-05 00:00:00','2009-05-26 00:00:00',93.51,427,466,40.47,'Intermediate' from dual union all
select 35007,'035007','1983-003C','deb Kosmos-1437','',55760,'Reentered','2010-08-11 00:00:00','2009-06-09 00:00:00',94.84,502,520,81.17,'Intermediate' from dual union all
select 35008,'035008','2009-029A','Meridian No. 12L','Meridian No. 12',68710,'In Earth orbit','','2009-06-21 00:00:00',645.99,303,36449,62.85,'GEO Transfer' from dual union all
select 35009,'035009','2009-029B','Fregat No. 1018','Fregat 14S44 No. 1018',68710,'In Earth orbit','','2009-06-21 00:00:00',641.89,196,36346,62.86,'GEO Transfer' from dual union all
select 3501,'003501','1965-082QX','deb Transtage 4','',11832,'Reentered','1971-09-23 00:00:00','1968-10-20 00:00:00',98.89,656,755,31.56,'Intermediate' from dual union all
select 35010,'035010','2009-030A','Soyuz TMA-15','Soyuz 11F732A17 No. 225',68711,'Landed','2009-12-01 00:00:00','2009-05-28 00:00:00',90.16,151,413,51.69,'Intermediate' from dual union all
select 35011,'035011','2009-030B','11S510','11S510',68711,'Reentered','2009-05-31 00:00:00','2009-05-29 00:00:00',88.31,185,197,51.63,'Intermediate' from dual union all
select 35012,'035012','1993-036AFZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-26 00:00:00',99.89,693,814,74.02,'Intermediate' from dual union all
select 35013,'035013','1993-036AGA','deb Kosmos-2251','',65138,'Reentered','2012-10-29 00:00:00','2009-05-26 00:00:00',98.13,518,820,73.96,'Intermediate' from dual union all
select 35014,'035014','1993-036AGB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-26 00:00:00',100.28,760,784,74.01,'Intermediate' from dual union all
select 35015,'035015','1993-036AGC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-27 00:00:00',100.87,728,872,73.95,'Intermediate' from dual union all
select 35016,'035016','1993-036AGD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-27 00:00:00',100.93,711,895,73.93,'Intermediate' from dual union all
select 35017,'035017','1993-036AGE','deb Kosmos-2251','',65138,'Reentered','2013-02-28 00:00:00','2009-05-27 00:00:00',97.7,527,770,73.94,'Intermediate' from dual;
insert into SATELLITE
select 35018,'035018','1993-036AGF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-26 00:00:00',106.32,773,1340,73.99,'Intermediate' from dual union all
select 35019,'035019','1993-036AGG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-27 00:00:00',100.05,730,792,74.03,'Intermediate' from dual union all
select 3502,'003502','1965-082QY','deb Transtage 4','',11832,'Reentered','1979-03-27 00:00:00','1968-10-18 00:00:00',99.69,711,776,32.18,'Intermediate' from dual union all
select 35020,'035020','1993-036AGH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-26 00:00:00',98.85,633,774,74.05,'Intermediate' from dual union all
select 35021,'035021','1993-036AGJ','deb Kosmos-2251','',65138,'Reentered','2015-07-19 00:00:00','2009-05-27 00:00:00',98.08,561,773,73.96,'Intermediate' from dual union all
select 35022,'035022','1993-036AGK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-27 00:00:00',99.53,695,778,74.03,'Intermediate' from dual union all
select 35023,'035023','1993-036AGL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-27 00:00:00',99.49,679,790,74.05,'Intermediate' from dual union all
select 35024,'035024','1993-036AGM','deb Kosmos-2251','',65138,'Reentered','2013-12-03 00:00:00','2009-05-27 00:00:00',98.85,633,774,74.09,'Intermediate' from dual union all
select 35025,'035025','1993-036AGN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-27 00:00:00',98.86,627,782,73.94,'Intermediate' from dual union all
select 35026,'035026','1993-036AGP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-26 00:00:00',99.21,660,782,74.05,'Intermediate' from dual union all
select 35027,'035027','1993-036AGQ','deb Kosmos-2251','',65138,'Reentered','2011-05-18 00:00:00','2009-05-27 00:00:00',97,458,773,74.2,'Intermediate' from dual union all
select 35028,'035028','1993-036AGR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-27 00:00:00',99.08,653,776,74.04,'Intermediate' from dual union all
select 35029,'035029','1993-036AGS','deb Kosmos-2251','',65138,'Reentered','2012-01-15 00:00:00','2009-05-26 00:00:00',96.08,376,766,74.05,'Intermediate' from dual union all
select 3503,'003503','1968-090A','Kosmos-248','I2M',16817,'Reentered','1980-02-26 00:00:00','1968-11-30 00:00:00',94.74,469,543,62.26,'Intermediate' from dual union all
select 35030,'035030','1993-036AGT','deb Kosmos-2251','',65138,'Reentered','2013-04-30 00:00:00','2009-05-26 00:00:00',98.56,589,790,74.06,'Intermediate' from dual union all
select 35031,'035031','1993-036AGU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-27 00:00:00',100.41,763,793,74.04,'Intermediate' from dual union all
select 35032,'035032','1993-036AGV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-27 00:00:00',100.32,706,842,73.86,'Intermediate' from dual union all
select 35033,'035033','1993-036AGW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-26 00:00:00',100.37,763,790,74.03,'Intermediate' from dual union all
select 35034,'035034','1993-036AGX','deb Kosmos-2251','',65138,'In Earth orbit','2016-01-01 00:00:00','2009-05-27 00:00:00',98.67,614,777,74.05,'Intermediate' from dual union all
select 35035,'035035','1993-036AGY','deb Kosmos-2251','',65138,'Reentered','2014-04-18 00:00:00','2009-05-26 00:00:00',98.37,581,781,74.06,'Intermediate' from dual union all
select 35036,'035036','1993-036AGZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-27 00:00:00',102.14,764,956,74.1,'Intermediate' from dual union all
select 35037,'035037','1993-036AHA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-26 00:00:00',99.69,704,783,74.01,'Intermediate' from dual union all
select 35038,'035038','1993-036AHB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-27 00:00:00',102.72,770,1005,74.08,'Intermediate' from dual union all
select 35039,'035039','1993-036AHC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-26 00:00:00',100.5,745,820,74.04,'Intermediate' from dual union all
select 3504,'003504','1968-091A','Kosmos-249','I2P',16818,'Exploded','1968-10-20 00:00:00','1968-10-20 00:00:00',86.89,104,136,62.27,'Intermediate' from dual union all
select 35040,'035040','1993-036AHD','deb Kosmos-2251','',65138,'Reentered','2014-09-17 00:00:00','2009-05-27 00:00:00',98.47,593,777,73.98,'Intermediate' from dual union all
select 35041,'035041','1993-036AHE','deb Kosmos-2251','',65138,'Reentered','2013-11-07 00:00:00','2009-05-27 00:00:00',98.47,565,806,73.95,'Intermediate' from dual union all
select 35042,'035042','1993-036AHF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-26 00:00:00',99.44,692,771,74.02,'Intermediate' from dual union all
select 35043,'035043','1993-036AHG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-27 00:00:00',102.38,689,1054,73.98,'Intermediate' from dual union all
select 35044,'035044','1993-036AHH','deb Kosmos-2251','',65138,'Reentered','2014-04-18 00:00:00','2009-05-27 00:00:00',98.15,563,778,74.04,'Intermediate' from dual union all
select 35045,'035045','1993-036AHJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-05-27 00:00:00',103.41,774,1066,73.72,'Intermediate' from dual union all
select 35046,'035046','1993-036AHK','deb Kosmos-2251','',65138,'Reentered','2013-09-30 00:00:00','2009-05-27 00:00:00',97.27,483,773,74,'Intermediate' from dual union all
select 35047,'035047','1993-036AHL','deb Kosmos-2251','',65138,'Reentered','2016-11-08 00:00:00','2009-05-27 00:00:00',99.64,688,795,74.02,'Intermediate' from dual union all
select 35048,'035048','1997-051NU','deb Iridium 33','',66131,'Reentered','2015-10-21 00:00:00','2009-05-27 00:00:00',100.07,740,784,86.36,'Polar' from dual union all
select 35049,'035049','1997-051NV','deb Iridium 33','',66131,'In Earth orbit','','2009-05-27 00:00:00',100.26,759,783,86.38,'Polar' from dual union all
select 3505,'003505','1968-091B','8S672','8S672',16818,'Reentered','1968-10-21 00:00:00','1968-10-20 00:00:00',87.65,135,182,62.25,'Intermediate' from dual union all
select 35050,'035050','1997-051NW','deb Iridium 33','',66131,'In Earth orbit','','2009-05-27 00:00:00',100.49,770,794,86.34,'Polar' from dual union all
select 35051,'035051','1997-051NX','deb Iridium 33','',66131,'In Earth orbit','','2009-05-27 00:00:00',100.53,775,793,86.33,'Polar' from dual union all
select 35052,'035052','1997-051NY','deb Iridium 33','',66131,'In Earth orbit','','2009-05-26 00:00:00',100.44,769,789,86.33,'Polar' from dual union all
select 35053,'035053','1997-051NZ','deb Iridium 33','',66131,'Reentered','2013-06-24 00:00:00','2009-05-27 00:00:00',99.09,533,897,86.58,'Polar' from dual union all
select 35054,'035054','1997-051PA','deb Iridium 33','',66131,'In Earth orbit','','2009-05-27 00:00:00',100.82,773,822,86.36,'Polar' from dual union all
select 35055,'035055','1997-051PB','deb Iridium 33','',66131,'Reentered','2013-04-30 00:00:00','2009-05-27 00:00:00',100.47,743,819,86.26,'Polar' from dual union all
select 35056,'035056','1991-025AA','deb SOZ','SOZ',64194,'In Earth orbit','','2009-05-28 00:00:00',337.45,903,18453,73.44,'Highly Ellip' from dual union all
select 35057,'035057','1991-025AB','deb SOZ','SOZ',64194,'Reentered','2009-07-05 00:00:00','2009-05-29 00:00:00',328.31,446,18327,73.62,'Highly Ellip' from dual union all
select 35058,'035058','1993-036AHM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-01 00:00:00',98.96,607,811,74.02,'Intermediate' from dual union all
select 35059,'035059','1993-036AHN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-02 00:00:00',100.28,740,803,74,'Intermediate' from dual union all
select 3506,'003506','1968-091C','deb Kosmos-249','',16818,'In Earth orbit','','1969-01-16 00:00:00',112.71,543,2160,62.35,'Intermediate' from dual union all
select 35060,'035060','1993-036AHP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-01 00:00:00',100.44,764,795,74.04,'Intermediate' from dual union all
select 35061,'035061','1993-036AHQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-02 00:00:00',99.53,696,777,74.05,'Intermediate' from dual union all
select 35062,'035062','1993-036AHR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-01 00:00:00',102.91,771,1022,74.11,'Intermediate' from dual union all
select 35063,'035063','1993-036AHS','deb Kosmos-2251','',65138,'Reentered','2015-01-31 00:00:00','2009-06-02 00:00:00',98.71,541,853,74.03,'Intermediate' from dual union all
select 35064,'035064','1993-036AHT','deb Kosmos-2251','',65138,'Reentered','2012-12-16 00:00:00','2009-06-01 00:00:00',96.88,445,774,74.26,'Intermediate' from dual union all
select 35065,'035065','1993-036AHU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-02 00:00:00',100.34,758,792,74.02,'Intermediate' from dual union all
select 35066,'035066','1993-036AHV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-01 00:00:00',99.82,711,789,74.02,'Intermediate' from dual union all
select 35067,'035067','1993-036AHW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-02 00:00:00',99.83,683,818,73.76,'Intermediate' from dual union all
select 35068,'035068','1993-036AHX','deb Kosmos-2251','',65138,'Reentered','2014-09-15 00:00:00','2009-06-02 00:00:00',99.28,653,795,74.04,'Intermediate' from dual union all
select 35069,'035069','1993-036AHY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-01 00:00:00',99.68,645,841,74.03,'Intermediate' from dual union all
select 3507,'003507','1968-091D','deb Kosmos-249','',16818,'Reentered','1989-08-09 00:00:00','1968-12-20 00:00:00',111.54,451,2144,62.3,'Intermediate' from dual union all
select 35070,'035070','1993-036AHZ','deb Kosmos-2251','',65138,'Reentered','2013-03-14 00:00:00','2009-06-01 00:00:00',98.12,560,778,73.92,'Intermediate' from dual union all
select 35071,'035071','1993-036AJA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-02 00:00:00',98.32,583,774,73.95,'Intermediate' from dual union all
select 35072,'035072','1993-036AJB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-01 00:00:00',101.1,755,867,74.1,'Intermediate' from dual union all
select 35073,'035073','1993-036AJC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-02 00:00:00',100.21,732,804,74.03,'Intermediate' from dual union all
select 35074,'035074','1993-036AJD','deb Kosmos-2251','',65138,'Reentered','2014-03-07 00:00:00','2009-06-01 00:00:00',98.76,621,778,73.99,'Intermediate' from dual union all
select 35075,'035075','1993-036AJE','deb Kosmos-2251','',65138,'Reentered','2015-04-03 00:00:00','2009-06-01 00:00:00',99.4,662,798,74.03,'Intermediate' from dual union all
select 35076,'035076','1993-036AJF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-01 00:00:00',99.18,656,782,73.97,'Intermediate' from dual union all
select 35077,'035077','1997-051PC','deb Iridium 33','',66131,'In Earth orbit','','2009-06-01 00:00:00',100.8,770,822,86.3,'Polar' from dual union all
select 35078,'035078','1997-051PD','deb Iridium 33','',66131,'In Earth orbit','','2009-06-02 00:00:00',100.87,773,827,86.41,'Polar' from dual union all
select 35079,'035079','1997-051PE','deb Iridium 33','',66131,'In Earth orbit','','2009-06-02 00:00:00',103.11,767,1045,86.42,'Polar' from dual union all
select 3508,'003508','1968-091E','deb Kosmos-249','',16818,'Reentered','1998-12-21 00:00:00','1968-11-25 00:00:00',112.16,504,2148,62.31,'Intermediate' from dual union all
select 35080,'035080','1997-051PF','deb Iridium 33','',66131,'In Earth orbit','','2009-06-02 00:00:00',103.45,775,1069,86.25,'Polar' from dual union all
select 35081,'035081','1997-051PG','deb Iridium 33','',66131,'Reentered','2016-01-01 00:00:00','2009-06-02 00:00:00',100.47,761,801,86.49,'Polar' from dual union all
select 35082,'035082','1991-025AC','deb SOZ','SOZ',64194,'Lost','','2009-05-28 00:00:00',304.98,833,16428,84.23,'Highly Ellip' from dual union all
select 35083,'035083','1991-025AD','deb SOZ','SOZ',64194,'In Earth orbit','','2009-06-01 00:00:00',272.48,959,14129,83.23,'Medium' from dual union all
select 35084,'035084','1991-025AE','deb SOZ','SOZ',64194,'In Earth orbit','','2009-05-27 00:00:00',325.92,668,17952,72.07,'Highly Ellip' from dual union all
select 35085,'035085','1991-025AF','deb SOZ','SOZ',64194,'In Earth orbit','','2009-05-30 00:00:00',332.02,617,18393,72.27,'Highly Ellip' from dual union all
select 35086,'035086','1999-025DJN','deb FY-1C','',66706,'In Earth orbit','','2009-06-03 00:00:00',100.35,697,853,98.95,'Sun-Synch' from dual union all
select 35087,'035087','1999-025DJP','deb FY-1C','',66706,'In Earth orbit','','2009-06-01 00:00:00',99.67,559,927,98.31,'Sun-Synch' from dual union all
select 35088,'035088','1999-025DJQ','deb FY-1C','',66706,'In Earth orbit','','2009-06-03 00:00:00',106.24,746,1358,94.75,'Polar' from dual union all
select 35089,'035089','1999-025DJR','deb FY-1C','',66706,'In Earth orbit','','2009-06-05 00:00:00',102.08,834,880,99.07,'Sun-Synch' from dual union all
select 3509,'003509','1968-091F','deb Kosmos-249','',16818,'In Earth orbit','','1968-11-30 00:00:00',112.04,486,2155,62.37,'Intermediate' from dual union all
select 35090,'035090','1999-025DJS','deb FY-1C','',66706,'In Earth orbit','','2009-06-01 00:00:00',103.09,852,958,98.86,'Sun-Synch' from dual union all
select 35091,'035091','1999-025DJT','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',101.62,771,900,98.85,'Sun-Synch' from dual union all
select 35092,'035092','1999-025DJU','deb FY-1C','',66706,'In Earth orbit','','2009-05-27 00:00:00',100.81,729,865,98.98,'Sun-Synch' from dual union all
select 35093,'035093','1999-025DJV','deb FY-1C','',66706,'In Earth orbit','','2009-06-06 00:00:00',101.09,656,965,98.59,'Sun-Synch' from dual union all
select 35094,'035094','1999-025DJW','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',102.32,656,1081,97.79,'Sun-Synch' from dual union all
select 35095,'035095','1999-025DJX','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',100.82,598,997,98.7,'Sun-Synch' from dual union all
select 35096,'035096','1999-025DJY','deb FY-1C','',66706,'In Earth orbit','','2009-06-03 00:00:00',100.36,603,949,97.71,'Sun-Synch' from dual union all
select 35097,'035097','1999-025DJZ','deb FY-1C','',66706,'In Earth orbit','','2009-06-03 00:00:00',99.48,542,926,98.75,'Sun-Synch' from dual union all
select 35098,'035098','1999-025DKA','deb FY-1C','',66706,'Reentered','2013-08-31 00:00:00','2009-06-03 00:00:00',100.66,557,1023,98.54,'Sun-Synch' from dual union all
select 35099,'035099','1999-025DKB','deb FY-1C','',66706,'In Earth orbit','','2009-06-05 00:00:00',99.31,519,933,98.24,'Sun-Synch' from dual union all
select 351,'000351','61 Omicron 112','deb r/b Ablestar','',7273,'In Earth orbit','','1962-08-01 00:00:00',103.38,859,978,66.77,'Intermediate' from dual union all
select 3510,'003510','1968-092A','OPS 4078','DAPP 6422',16819,'In Earth orbit','','1969-01-18 00:00:00',101.38,801,847,99,'Sun-Synch' from dual union all
select 35100,'035100','1999-025DKC','deb FY-1C','',66706,'Reentered','2014-11-23 00:00:00','2009-06-08 00:00:00',100.73,615,972,97.86,'Sun-Synch' from dual union all
select 35101,'035101','1999-025DKD','deb FY-1C','',66706,'Reentered','2011-10-18 00:00:00','2009-05-29 00:00:00',98.26,473,878,96.18,'Sun-Synch' from dual union all
select 35102,'035102','1999-025DKE','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',101.44,779,875,98.79,'Sun-Synch' from dual union all
select 35103,'035103','1999-025DKF','deb FY-1C','',66706,'Reentered','2012-06-23 00:00:00','2009-06-07 00:00:00',98.6,484,900,97.66,'Sun-Synch' from dual union all
select 35104,'035104','1999-025DKG','deb FY-1C','',66706,'In Earth orbit','','2009-06-05 00:00:00',101.9,825,873,98.9,'Sun-Synch' from dual union all
select 35105,'035105','1999-025DKH','deb FY-1C','',66706,'In Earth orbit','','2009-06-02 00:00:00',100.29,682,863,98.96,'Sun-Synch' from dual union all
select 35106,'035106','1999-025DKJ','deb FY-1C','',66706,'Reentered','2014-01-09 00:00:00','2009-06-03 00:00:00',99.62,557,924,98.31,'Sun-Synch' from dual union all
select 35107,'035107','1999-025DKK','deb FY-1C','',66706,'Reentered','2013-04-29 00:00:00','2009-06-04 00:00:00',100.33,560,989,96.53,'Sun-Synch' from dual;
insert into SATELLITE
select 35108,'035108','1999-025DKL','deb FY-1C','',66706,'In Earth orbit','','2009-05-27 00:00:00',102.67,782,988,99.09,'Sun-Synch' from dual union all
select 35109,'035109','1999-025DKM','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',101.66,720,955,98.92,'Sun-Synch' from dual union all
select 3511,'003511','1968-093A','Soyuz-2','Soyuz No. 11',16820,'Landed','1968-10-28 00:00:00','1968-10-27 00:00:00',88.22,177,196,51.67,'Intermediate' from dual union all
select 35110,'035110','1999-025DKN','deb FY-1C','',66706,'In Earth orbit','','2009-06-03 00:00:00',101.2,701,930,96.66,'Sun-Synch' from dual union all
select 35111,'035111','1999-025DKP','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',102.49,859,893,99.53,'Sun-Synch' from dual union all
select 35112,'035112','1999-025DKQ','deb FY-1C','',66706,'Reentered','2014-12-19 00:00:00','2009-06-03 00:00:00',100.37,590,962,98.18,'Sun-Synch' from dual union all
select 35113,'035113','1999-025DKR','deb FY-1C','',66706,'Reentered','2014-12-24 00:00:00','2009-06-02 00:00:00',100.34,601,949,97.75,'Sun-Synch' from dual union all
select 35114,'035114','1999-025DKS','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',98.75,464,933,97.62,'Sun-Synch' from dual union all
select 35115,'035115','1999-025DKT','deb FY-1C','',66706,'Reentered','2012-03-09 00:00:00','2009-06-06 00:00:00',99.86,500,1003,97.32,'Sun-Synch' from dual union all
select 35116,'035116','1999-025DKU','deb FY-1C','',66706,'In Earth orbit','','2009-06-06 00:00:00',102.2,811,915,98.9,'Sun-Synch' from dual union all
select 35117,'035117','1999-025DKV','deb FY-1C','',66706,'In Earth orbit','','2009-06-03 00:00:00',101.67,793,883,98.82,'Sun-Synch' from dual union all
select 35118,'035118','1999-025DKW','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',99.63,569,913,97.96,'Sun-Synch' from dual union all
select 35119,'035119','1999-025DKX','deb FY-1C','',66706,'In Earth orbit','','2009-06-04 00:00:00',100.64,724,854,99.96,'Sun-Synch' from dual union all
select 3512,'003512','1968-093B','11S510','11S510',16820,'Reentered','1968-10-27 00:00:00','1968-10-26 00:00:00',88.09,180,180,51.72,'Intermediate' from dual union all
select 35120,'035120','1999-025DKY','deb FY-1C','',66706,'In Earth orbit','','2009-06-04 00:00:00',103.88,796,1087,98,'Sun-Synch' from dual union all
select 35121,'035121','1999-025DKZ','deb FY-1C','',66706,'In Earth orbit','','2009-05-26 00:00:00',102.71,760,1013,98.39,'Sun-Synch' from dual union all
select 35122,'035122','1999-025DLA','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',102.17,843,880,98.89,'Sun-Synch' from dual union all
select 35123,'035123','1999-025DLB','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',101.88,806,890,99.16,'Sun-Synch' from dual union all
select 35124,'035124','1999-025DLC','deb FY-1C','',66706,'Reentered','2010-08-26 00:00:00','2009-06-05 00:00:00',95.38,342,732,98.22,'Sun-Synch' from dual union all
select 35125,'035125','1999-025DLD','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',101.9,799,899,98.34,'Sun-Synch' from dual union all
select 35126,'035126','1999-025DLE','deb FY-1C','',66706,'Reentered','2015-04-14 00:00:00','2009-06-03 00:00:00',100.66,594,986,97.3,'Sun-Synch' from dual union all
select 35127,'035127','1999-025DLF','deb FY-1C','',66706,'Reentered','2012-04-27 00:00:00','2009-06-08 00:00:00',99.72,470,1020,97.3,'Sun-Synch' from dual union all
select 35128,'035128','1999-025DLG','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',103.62,814,1046,98.37,'Sun-Synch' from dual union all
select 35129,'035129','1999-025DLH','deb FY-1C','',66706,'In Earth orbit','','2009-05-28 00:00:00',103.43,803,1039,98.32,'Sun-Synch' from dual union all
select 3513,'003513','1968-085D','Blok-L No. Ya716-86','Blok-L',16812,'Reentered','1975-05-07 00:00:00','1968-11-11 00:00:00',707.69,444,39412,64.94,'GEO Transfer' from dual union all
select 35130,'035130','1999-025DLJ','deb FY-1C','',66706,'In Earth orbit','','2009-06-08 00:00:00',102.92,853,940,99.08,'Sun-Synch' from dual union all
select 35131,'035131','1999-025DLK','deb FY-1C','',66706,'Reentered','2014-03-02 00:00:00','2009-06-08 00:00:00',99.1,450,982,99.66,'Sun-Synch' from dual union all
select 35132,'035132','1999-025DLL','deb FY-1C','',66706,'Reentered','2013-03-11 00:00:00','2009-06-08 00:00:00',99.29,533,917,98.36,'Sun-Synch' from dual union all
select 35133,'035133','1999-025DLM','deb FY-1C','',66706,'Reentered','2012-01-20 00:00:00','2009-06-05 00:00:00',99.17,454,984,97.41,'Sun-Synch' from dual union all
select 35134,'035134','1999-025DLN','deb FY-1C','',66706,'In Earth orbit','','2009-06-03 00:00:00',100.98,746,865,98.97,'Sun-Synch' from dual union all
select 35135,'035135','1999-025DLP','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',101.8,780,908,98.87,'Sun-Synch' from dual union all
select 35136,'035136','1999-025DLQ','deb FY-1C','',66706,'In Earth orbit','','2009-06-03 00:00:00',101.6,770,899,98.82,'Sun-Synch' from dual union all
select 35137,'035137','1999-025DLR','deb FY-1C','',66706,'In Earth orbit','','2009-06-08 00:00:00',103.23,681,1142,99.16,'Sun-Synch' from dual union all
select 35138,'035138','1999-025DLS','deb FY-1C','',66706,'Reentered','2014-04-21 00:00:00','2009-05-31 00:00:00',100.23,691,848,98.91,'Sun-Synch' from dual union all
select 35139,'035139','1999-025DLT','deb FY-1C','',66706,'In Earth orbit','','2009-06-08 00:00:00',101.78,790,896,98.87,'Sun-Synch' from dual union all
select 3514,'003514','1968-035D','Blok-L No. Ya716-82','Blok-L',16762,'Reentered','1974-07-09 00:00:00','1968-10-20 00:00:00',709.12,631,39296,65.05,'GEO Transfer' from dual union all
select 35140,'035140','1999-025DLU','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',101.6,614,1054,99.05,'Sun-Synch' from dual union all
select 35141,'035141','1999-025DLV','deb FY-1C','',66706,'In Earth orbit','','2009-05-30 00:00:00',103.18,841,977,99.16,'Sun-Synch' from dual union all
select 35142,'035142','1999-025DLW','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',103.73,854,1016,99.34,'Sun-Synch' from dual union all
select 35143,'035143','1999-025DLX','deb FY-1C','',66706,'Reentered','2013-12-03 00:00:00','2009-06-05 00:00:00',101.51,801,860,98.87,'Sun-Synch' from dual union all
select 35144,'035144','1999-025DLY','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',101.71,801,878,98.79,'Sun-Synch' from dual union all
select 35145,'035145','1999-025DLZ','deb FY-1C','',66706,'In Earth orbit','','2009-06-01 00:00:00',98.57,517,864,99.66,'Sun-Synch' from dual union all
select 35146,'035146','1999-025DMA','deb FY-1C','',66706,'In Earth orbit','','2009-06-02 00:00:00',101.13,573,1051,98.11,'Sun-Synch' from dual union all
select 35147,'035147','1999-025DMB','deb FY-1C','',66706,'In Earth orbit','','2009-06-03 00:00:00',101.07,764,855,99.51,'Sun-Synch' from dual union all
select 35148,'035148','1999-025DMC','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',100.21,683,854,98.97,'Sun-Synch' from dual union all
select 35149,'035149','1999-025DMD','deb FY-1C','',66706,'Reentered','2012-12-19 00:00:00','2009-06-05 00:00:00',98.93,495,920,97.73,'Sun-Synch' from dual union all
select 3515,'003515','1968-057F','Blok-L No. Ya716-85','Blok-L',16784,'Reentered','1970-09-02 00:00:00','1968-10-20 00:00:00',711.02,530,39491,64.97,'GEO Transfer' from dual union all
select 35150,'035150','1999-025DME','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',99.68,634,852,99.25,'Sun-Synch' from dual union all
select 35151,'035151','1999-025DMF','deb FY-1C','',66706,'Reentered','2015-04-12 00:00:00','2009-06-04 00:00:00',100.4,694,861,99.35,'Sun-Synch' from dual union all
select 35152,'035152','1999-025DMG','deb FY-1C','',66706,'In Earth orbit','','2009-06-05 00:00:00',105.18,850,1156,99.87,'Sun-Synch' from dual union all
select 35153,'035153','1999-025DMH','deb FY-1C','',66706,'In Earth orbit','','2009-06-08 00:00:00',99.08,554,876,98.92,'Sun-Synch' from dual union all
select 35154,'035154','1999-025DMJ','deb FY-1C','',66706,'In Earth orbit','','2009-06-06 00:00:00',102,712,994,99.46,'Sun-Synch' from dual union all
select 35155,'035155','1999-025DMK','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',102.17,846,876,99.14,'Sun-Synch' from dual union all
select 35156,'035156','1999-025DML','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',102.42,846,901,99.11,'Sun-Synch' from dual union all
select 35157,'035157','1999-025DMM','deb FY-1C','',66706,'Reentered','2013-03-09 00:00:00','2009-06-01 00:00:00',99.84,549,952,97.17,'Sun-Synch' from dual union all
select 35158,'035158','1999-025DMN','deb FY-1C','',66706,'Reentered','2011-12-03 00:00:00','2009-05-22 00:00:00',97.68,479,817,98.99,'Sun-Synch' from dual union all
select 35159,'035159','1999-025DMP','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',102.1,847,870,98.72,'Sun-Synch' from dual union all
select 3516,'003516','1968-094A','Soyuz-3','Soyuz No. 10',16821,'Landed','1968-10-30 00:00:00','1968-10-28 00:00:00',88.39,180,210,51.67,'Intermediate' from dual union all
select 35160,'035160','1999-025DMQ','deb FY-1C','',66706,'In Earth orbit','','2009-05-31 00:00:00',100.89,659,942,99.21,'Sun-Synch' from dual union all
select 35161,'035161','1999-025DMR','deb FY-1C','',66706,'In Earth orbit','','2009-06-08 00:00:00',99.5,616,854,99.55,'Sun-Synch' from dual union all
select 35162,'035162','1999-025DMS','deb FY-1C','',66706,'In Earth orbit','','2009-06-08 00:00:00',101.72,727,954,96.74,'Sun-Synch' from dual union all
select 35163,'035163','1999-025DMT','deb FY-1C','',66706,'In Earth orbit','','2009-06-04 00:00:00',99.9,652,856,99.11,'Sun-Synch' from dual union all
select 35164,'035164','1999-025DMU','deb FY-1C','',66706,'In Earth orbit','','2009-06-04 00:00:00',105.03,827,1165,98.65,'Sun-Synch' from dual union all
select 35165,'035165','1999-025DMV','deb FY-1C','',66706,'In Earth orbit','','2009-05-16 00:00:00',102.53,854,903,98.89,'Sun-Synch' from dual union all
select 35166,'035166','1999-025DMW','deb FY-1C','',66706,'Reentered','2012-02-29 00:00:00','2009-06-06 00:00:00',97.97,492,832,98.94,'Sun-Synch' from dual union all
select 35167,'035167','1999-025DMX','deb FY-1C','',66706,'In Earth orbit','','2009-06-06 00:00:00',101.52,638,1024,98.94,'Sun-Synch' from dual union all
select 35168,'035168','1999-025DMY','deb FY-1C','',66706,'In Earth orbit','','2009-05-31 00:00:00',102.1,764,953,98.3,'Sun-Synch' from dual union all
select 35169,'035169','1999-025DMZ','deb FY-1C','',66706,'In Earth orbit','','2009-05-31 00:00:00',100.16,618,915,99.15,'Sun-Synch' from dual union all
select 3517,'003517','1968-094B','11S510','11S510',16821,'Reentered','1968-10-28 00:00:00','1968-10-27 00:00:00',88.23,187,187,51.61,'Intermediate' from dual union all
select 35170,'035170','1999-025DNA','deb FY-1C','',66706,'In Earth orbit','','2009-06-05 00:00:00',103.36,831,1004,99.88,'Sun-Synch' from dual union all
select 35171,'035171','1999-025DNB','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',104.58,852,1098,99.05,'Sun-Synch' from dual union all
select 35172,'035172','1999-025DNC','deb FY-1C','',66706,'Reentered','2012-01-01 00:00:00','2009-06-07 00:00:00',97.81,451,857,98.68,'Sun-Synch' from dual union all
select 35173,'035173','1999-025DND','deb FY-1C','',66706,'In Earth orbit','','2009-06-01 00:00:00',108.91,771,1581,99.77,'Sun-Synch' from dual union all
select 35174,'035174','1999-025DNE','deb FY-1C','',66706,'In Earth orbit','','2009-05-22 00:00:00',102.32,539,1198,99.17,'Sun-Synch' from dual union all
select 35175,'035175','1999-025DNF','deb FY-1C','',66706,'In Earth orbit','','2009-06-03 00:00:00',101.72,795,886,98.85,'Sun-Synch' from dual union all
select 35176,'035176','1999-025DNG','deb FY-1C','',66706,'In Earth orbit','','2009-06-04 00:00:00',102,811,896,99.11,'Sun-Synch' from dual union all
select 35177,'035177','1999-025DNH','deb FY-1C','',66706,'In Earth orbit','','2009-05-30 00:00:00',102.19,829,895,99.15,'Sun-Synch' from dual union all
select 35178,'035178','1999-025DNJ','deb FY-1C','',66706,'Reentered','2013-08-02 00:00:00','2009-06-06 00:00:00',98.58,553,829,98.88,'Sun-Synch' from dual union all
select 35179,'035179','1999-025DNK','deb FY-1C','',66706,'Reentered','2012-08-20 00:00:00','2009-06-09 00:00:00',98.6,379,1005,98.52,'Sun-Synch' from dual union all
select 3518,'003518','1968-091G','deb Kosmos-249','',16818,'Reentered','1979-11-13 00:00:00','1979-11-01 00:00:00',94.58,388,609,62.46,'Intermediate' from dual union all
select 35180,'035180','1999-025DNL','deb FY-1C','',66706,'In Earth orbit','','2009-06-03 00:00:00',108.55,645,1675,99.73,'Sun-Synch' from dual union all
select 35181,'035181','1999-025DNM','deb FY-1C','',66706,'Reentered','2009-08-23 00:00:00','2009-05-25 00:00:00',93.58,397,503,98.94,'Sun-Synch' from dual union all
select 35182,'035182','1999-025DNN','deb FY-1C','',66706,'Reentered','2012-05-15 00:00:00','2009-06-08 00:00:00',96.95,389,837,99.33,'Sun-Synch' from dual union all
select 35183,'035183','1999-025DNP','deb FY-1C','',66706,'In Earth orbit','','2009-06-05 00:00:00',99.91,657,852,98.92,'Sun-Synch' from dual union all
select 35184,'035184','1999-025DNQ','deb FY-1C','',66706,'Reentered','2011-11-21 00:00:00','2009-06-02 00:00:00',98.34,506,852,99.32,'Sun-Synch' from dual union all
select 35185,'035185','1999-025DNR','deb FY-1C','',66706,'In Earth orbit','','2009-06-06 00:00:00',99.48,600,867,99.54,'Sun-Synch' from dual union all
select 35186,'035186','1999-025DNS','deb FY-1C','',66706,'In Earth orbit','','2009-06-05 00:00:00',101.28,772,867,99.58,'Sun-Synch' from dual union all
select 35187,'035187','1999-025DNT','deb FY-1C','',66706,'In Earth orbit','','2009-06-03 00:00:00',101.28,785,854,99.7,'Sun-Synch' from dual union all
select 35188,'035188','1999-025DNU','deb FY-1C','',66706,'Reentered','2014-09-18 00:00:00','2009-06-04 00:00:00',99.72,615,876,99,'Sun-Synch' from dual union all
select 35189,'035189','1999-025DNV','deb FY-1C','',66706,'In Earth orbit','','2009-05-25 00:00:00',103.12,855,958,99.16,'Sun-Synch' from dual union all
select 3519,'003519','1968-091H','deb Kosmos-249','',16818,'Reentered','1982-03-06 00:00:00','1982-03-06 00:00:00',88.48,175,224,62.26,'Intermediate' from dual union all
select 35190,'035190','1999-025DNW','deb FY-1C','',66706,'In Earth orbit','','2009-06-04 00:00:00',101.35,718,928,98.54,'Sun-Synch' from dual union all
select 35191,'035191','1999-025DNX','deb FY-1C','',66706,'In Earth orbit','','2009-06-03 00:00:00',103.65,854,1008,99.4,'Sun-Synch' from dual union all
select 35192,'035192','1999-025DNY','deb FY-1C','',66706,'In Earth orbit','','2009-06-06 00:00:00',101.96,614,1089,99.22,'Sun-Synch' from dual union all
select 35193,'035193','1999-025DNZ','deb FY-1C','',66706,'Reentered','2011-10-19 00:00:00','2009-06-07 00:00:00',97.29,474,784,99.18,'Sun-Synch' from dual union all
select 35194,'035194','1999-025DPA','deb FY-1C','',66706,'In Earth orbit','','2009-05-30 00:00:00',100.41,657,899,99.02,'Sun-Synch' from dual union all
select 35195,'035195','1999-025DPB','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',100.5,679,886,99.05,'Sun-Synch' from dual union all
select 35196,'035196','1999-025DPC','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',102.37,691,1050,99.15,'Sun-Synch' from dual union all
select 35197,'035197','1999-025DPD','deb FY-1C','',66706,'Reentered','2011-05-16 00:00:00','2009-06-09 00:00:00',96.88,352,866,100.48,'Sun-Synch' from dual union all
select 35198,'035198','1999-025DPE','deb FY-1C','',66706,'Reentered','2012-05-29 00:00:00','2009-06-09 00:00:00',98.3,524,831,99.02,'Sun-Synch' from dual;
insert into SATELLITE
select 35199,'035199','1999-025DPF','deb FY-1C','',66706,'In Earth orbit','','2009-06-04 00:00:00',104.08,853,1050,98.48,'Sun-Synch' from dual union all
select 352,'000352','61 Omicron 113','deb r/b Ablestar','',7273,'Reentered','1966-12-10 00:00:00','1962-08-01 00:00:00',97.99,426,900,66.56,'Intermediate' from dual union all
select 3520,'003520','1968-091J','deb Kosmos-249','',16818,'Reentered','1981-05-29 00:00:00','1981-04-15 00:00:00',98.66,464,926,62.28,'Intermediate' from dual union all
select 35200,'035200','1999-025DPG','deb FY-1C','',66706,'In Earth orbit','','2009-06-06 00:00:00',107.5,841,1381,98.92,'Sun-Synch' from dual union all
select 35201,'035201','1999-025DPH','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',102.16,856,866,98.9,'Sun-Synch' from dual union all
select 35202,'035202','1999-025DPJ','deb FY-1C','',66706,'In Earth orbit','','2009-06-03 00:00:00',108.37,859,1444,100.12,'Sun-Synch' from dual union all
select 35203,'035203','1999-025DPK','deb FY-1C','',66706,'In Earth orbit','','2009-06-05 00:00:00',110.13,833,1633,102.03,'Sun-Synch' from dual union all
select 35204,'035204','1999-025DPL','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',102.14,813,907,99.12,'Sun-Synch' from dual union all
select 35205,'035205','1999-025DPM','deb FY-1C','',66706,'In Earth orbit','','2009-06-08 00:00:00',103.04,843,961,98.91,'Sun-Synch' from dual union all
select 35206,'035206','1999-025DPN','deb FY-1C','',66706,'Reentered','2013-12-09 00:00:00','2009-06-07 00:00:00',98.58,530,852,98.93,'Sun-Synch' from dual union all
select 35207,'035207','1999-025DPP','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',101.52,766,896,98.61,'Sun-Synch' from dual union all
select 35208,'035208','1999-025DPQ','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',103.8,856,1021,99.35,'Sun-Synch' from dual union all
select 35209,'035209','1999-025DPR','deb FY-1C','',66706,'Reentered','2014-03-29 00:00:00','2009-06-03 00:00:00',98.35,403,957,98.26,'Sun-Synch' from dual union all
select 3521,'003521','1968-091K','deb Kosmos-249','',16818,'Reentered','1974-04-06 00:00:00','1974-04-03 00:00:00',89.41,225,266,62.03,'Intermediate' from dual union all
select 35210,'035210','1999-025DPS','deb FY-1C','',66706,'In Earth orbit','','2009-06-06 00:00:00',103.13,849,965,98.84,'Sun-Synch' from dual union all
select 35211,'035211','1999-025DPT','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',103.57,819,1036,98.45,'Sun-Synch' from dual union all
select 35212,'035212','1999-025DPU','deb FY-1C','',66706,'Reentered','2017-04-14 00:00:00','2009-05-23 00:00:00',101.36,623,1024,99.4,'Sun-Synch' from dual union all
select 35213,'035213','1999-025DPV','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',99.68,547,940,98.65,'Sun-Synch' from dual union all
select 35214,'035214','1999-025DPW','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',100.45,662,898,98.54,'Sun-Synch' from dual union all
select 35215,'035215','1999-025DPX','deb FY-1C','',66706,'In Earth orbit','','2009-05-16 00:00:00',102.7,573,1200,98.96,'Sun-Synch' from dual union all
select 35216,'035216','1999-025DPY','deb FY-1C','',66706,'Reentered','2013-10-02 00:00:00','2009-06-09 00:00:00',97.73,465,836,98.97,'Sun-Synch' from dual union all
select 35217,'035217','1999-025DPZ','deb FY-1C','',66706,'In Earth orbit','','2009-06-06 00:00:00',107.44,852,1364,101.89,'Sun-Synch' from dual union all
select 35218,'035218','1999-025DQA','deb FY-1C','',66706,'Reentered','2010-11-13 00:00:00','2009-06-07 00:00:00',104.69,589,1371,99.92,'Sun-Synch' from dual union all
select 35219,'035219','1999-025DQB','deb FY-1C','',66706,'In Earth orbit','','2009-06-05 00:00:00',100.85,680,917,99.14,'Sun-Synch' from dual union all
select 3522,'003522','1968-092B','Burner II No. 8','Star 37B',16819,'In Earth orbit','','1968-12-01 00:00:00',101.38,800,848,98.99,'Sun-Synch' from dual union all
select 35220,'035220','1999-025DQC','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',102.86,849,938,98.85,'Sun-Synch' from dual union all
select 35221,'035221','1999-025DQD','deb FY-1C','',66706,'In Earth orbit','','2009-06-06 00:00:00',102.02,799,910,98.46,'Sun-Synch' from dual union all
select 35222,'035222','1999-025DQE','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',98.52,516,860,98.94,'Sun-Synch' from dual union all
select 35223,'035223','1999-025DQF','deb FY-1C','',66706,'In Earth orbit','','2009-06-08 00:00:00',98.89,555,857,98.95,'Sun-Synch' from dual union all
select 35224,'035224','1999-025DQG','deb FY-1C','',66706,'Reentered','2017-01-03 00:00:00','2009-06-07 00:00:00',99.2,594,847,98.96,'Sun-Synch' from dual union all
select 35225,'035225','1999-025DQH','deb FY-1C','',66706,'In Earth orbit','','2009-06-01 00:00:00',102.81,842,942,103.31,'Retrograde' from dual union all
select 35226,'035226','1999-025DQJ','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',100.31,696,851,99.01,'Sun-Synch' from dual union all
select 35227,'035227','1999-025DQK','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',100.21,683,854,98.97,'Sun-Synch' from dual union all
select 35228,'035228','1999-025DQL','deb FY-1C','',66706,'Reentered','2015-07-20 00:00:00','2009-05-27 00:00:00',100.41,664,891,98.57,'Sun-Synch' from dual union all
select 35229,'035229','1999-025DQM','deb FY-1C','',66706,'In Earth orbit','','2009-06-05 00:00:00',103.03,819,985,98.49,'Sun-Synch' from dual union all
select 3523,'003523','1968-091L','deb Kosmos-249','',16818,'Reentered','2001-10-22 00:00:00','1992-02-03 00:00:00',102.5,502,1251,62.3,'Intermediate' from dual union all
select 35230,'035230','1999-025DQN','deb FY-1C','',66706,'In Earth orbit','','2009-06-08 00:00:00',101.97,795,909,98.44,'Sun-Synch' from dual union all
select 35231,'035231','1999-025DQP','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',101.82,819,870,98.7,'Sun-Synch' from dual union all
select 35232,'035232','1999-025DQQ','deb FY-1C','',66706,'Reentered','2011-10-12 00:00:00','2009-05-28 00:00:00',100.03,467,1052,97.73,'Sun-Synch' from dual union all
select 35233,'035233','1999-025DQR','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',102.12,816,902,98.87,'Sun-Synch' from dual union all
select 35234,'035234','1999-025DQS','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',100.22,656,882,98.65,'Sun-Synch' from dual union all
select 35235,'035235','1999-025DQT','deb FY-1C','',66706,'In Earth orbit','','2009-06-04 00:00:00',99.85,649,854,99.46,'Sun-Synch' from dual union all
select 35236,'035236','1999-025DQU','deb FY-1C','',66706,'In Earth orbit','','2009-06-02 00:00:00',100.6,711,862,98.96,'Sun-Synch' from dual union all
select 35237,'035237','1999-025DQV','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',100.88,719,882,99.6,'Sun-Synch' from dual union all
select 35238,'035238','1999-025DQW','deb FY-1C','',66706,'Reentered','2014-06-03 00:00:00','2009-06-06 00:00:00',98.52,532,844,99,'Sun-Synch' from dual union all
select 35239,'035239','1999-025DQX','deb FY-1C','',66706,'In Earth orbit','','2009-06-06 00:00:00',105.91,856,1218,99.35,'Sun-Synch' from dual union all
select 3524,'003524','1968-091M','deb Kosmos-249','',16818,'Reentered','2001-06-15 00:00:00','1992-02-03 00:00:00',101.87,464,1231,62.31,'Intermediate' from dual union all
select 35240,'035240','1999-025DQY','deb FY-1C','',66706,'In Earth orbit','','2009-06-07 00:00:00',102.88,856,934,99.41,'Sun-Synch' from dual union all
select 35241,'035241','1999-025DQZ','deb FY-1C','',66706,'In Earth orbit','','2009-06-08 00:00:00',104.06,830,1071,98.65,'Sun-Synch' from dual union all
select 35242,'035242','1999-025DRA','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',107.33,855,1351,99.88,'Sun-Synch' from dual union all
select 35243,'035243','1999-025DRB','deb FY-1C','',66706,'In Earth orbit','','2009-06-05 00:00:00',100.47,670,892,98.97,'Sun-Synch' from dual union all
select 35244,'035244','1999-025DRC','deb FY-1C','',66706,'In Earth orbit','','2009-05-30 00:00:00',102.94,845,951,99.02,'Sun-Synch' from dual union all
select 35245,'035245','1999-025DRD','deb FY-1C','',66706,'In Earth orbit','','2009-06-09 00:00:00',103.43,860,982,99.08,'Sun-Synch' from dual union all
select 35246,'035246','1999-025DRE','deb FY-1C','',66706,'Reentered','2011-05-10 00:00:00','2009-06-05 00:00:00',99.93,738,773,98.13,'Sun-Synch' from dual union all
select 35247,'035247','1999-025DRF','deb FY-1C','',66706,'In Earth orbit','','2009-06-08 00:00:00',101.58,780,887,98.67,'Sun-Synch' from dual union all
select 35248,'035248','2007-055C','CZ-4C sep motor cover','',68397,'In Earth orbit','','2009-06-10 00:00:00',99.51,599,872,97.9,'Sun-Synch' from dual union all
select 35249,'035249','2007-055D','CZ-4C sep motor cover','',68397,'In Earth orbit','','2009-06-08 00:00:00',101.09,613,1008,97.84,'Sun-Synch' from dual union all
select 3525,'003525','1968-091N','deb Kosmos-249','',16818,'Reentered','1981-10-19 00:00:00','1981-10-18 00:00:00',90.95,281,362,62.38,'Intermediate' from dual union all
select 35250,'035250','2007-055E','CZ-4C sep motor cover','',68397,'Reentered','2010-09-30 00:00:00','2009-06-09 00:00:00',96.37,564,606,97.81,'Sun-Synch' from dual union all
select 35251,'035251','2007-055F','CZ-4C sep motor cover','',68397,'In Earth orbit','','2009-06-08 00:00:00',100.41,614,942,97.85,'Sun-Synch' from dual union all
select 35252,'035252','2007-055G','deb Yaogan Weixing 3','',68397,'Reentered','2016-02-07 00:00:00','2009-06-09 00:00:00',97.07,610,627,97.8,'Sun-Synch' from dual union all
select 35253,'035253','1979-095U','deb Meteor-2','',47579,'In Earth orbit','','2009-06-09 00:00:00',101.28,806,833,81.21,'Intermediate' from dual union all
select 35254,'035254','1979-095V','deb Meteor-2','',47579,'In Earth orbit','','2009-06-10 00:00:00',101.09,809,811,81.22,'Intermediate' from dual union all
select 35255,'035255','1979-095W','deb Meteor-2','',47579,'In Earth orbit','','2009-05-27 00:00:00',102.3,856,878,81.21,'Intermediate' from dual union all
select 35256,'035256','1979-095X','deb Meteor-2','',47579,'In Earth orbit','','2009-05-24 00:00:00',102.31,857,879,81.21,'Intermediate' from dual union all
select 35257,'035257','1979-095Y','deb Meteor-2','',47579,'In Earth orbit','','2009-05-17 00:00:00',102.16,846,876,81.21,'Intermediate' from dual union all
select 35258,'035258','1991-050M','deb ERS-1','',64219,'In Earth orbit','','2009-06-10 00:00:00',100.25,745,796,98.16,'Sun-Synch' from dual union all
select 35259,'035259','1991-050N','deb ERS-1','',64219,'In Earth orbit','','2009-06-10 00:00:00',100.02,743,775,98.17,'Sun-Synch' from dual union all
select 3526,'003526','1968-095A','Kosmos-250','Tselina-O',16822,'Reentered','1978-02-15 00:00:00','1968-12-01 00:00:00',95.22,520,539,74.01,'Intermediate' from dual union all
select 35260,'035260','1991-050P','deb ERS-1','',64219,'In Earth orbit','','2009-06-10 00:00:00',100.15,744,788,98.18,'Sun-Synch' from dual union all
select 35261,'035261','1991-050Q','deb ERS-1','',64219,'In Earth orbit','','2009-06-10 00:00:00',100.25,744,797,98.16,'Sun-Synch' from dual union all
select 35262,'035262','1991-050R','deb ERS-1','',64219,'In Earth orbit','','2009-06-10 00:00:00',99.81,733,765,98.21,'Sun-Synch' from dual union all
select 35263,'035263','1994-074L','deb Resurs-O1','',65470,'Reentered','2011-11-14 00:00:00','2009-06-03 00:00:00',97.08,613,624,97.89,'Sun-Synch' from dual union all
select 35264,'035264','1994-074M','deb Resurs-O1','',65470,'Reentered','2013-07-10 00:00:00','2009-06-08 00:00:00',97.28,626,631,97.95,'Sun-Synch' from dual union all
select 35265,'035265','1994-074N','deb Resurs-O1','',65470,'Reentered','2012-04-26 00:00:00','2009-06-10 00:00:00',97.3,628,630,97.96,'Sun-Synch' from dual union all
select 35266,'035266','1994-074P','deb Resurs-O1','',65470,'Reentered','2012-05-29 00:00:00','2009-06-09 00:00:00',97.23,624,629,97.96,'Sun-Synch' from dual union all
select 35267,'035267','1994-074Q','deb Resurs-O1','',65470,'Reentered','2011-10-12 00:00:00','2009-06-09 00:00:00',96.79,604,606,97.9,'Sun-Synch' from dual union all
select 35268,'035268','2004-035L','deb CZ-4B','',67879,'In Earth orbit','','2009-06-04 00:00:00',96.59,594,597,97.75,'Sun-Synch' from dual union all
select 35269,'035269','2004-035M','CZ-4B sep motor cover','',67879,'In Earth orbit','','2009-06-10 00:00:00',99.28,591,857,97.91,'Sun-Synch' from dual union all
select 3527,'003527','1968-095B','S3M','S3M',16822,'Reentered','1978-11-08 00:00:00','1968-12-10 00:00:00',95.11,515,533,74.01,'Intermediate' from dual union all
select 35270,'035270','2004-035N','CZ-4B sep motor cover','',67879,'In Earth orbit','','2009-06-08 00:00:00',99.9,597,910,97.8,'Sun-Synch' from dual union all
select 35271,'035271','2005-024K','deb SJ-7','',68033,'In Earth orbit','','2009-06-09 00:00:00',95.74,547,562,97.7,'Sun-Synch' from dual union all
select 35272,'035272','2005-024L','deb SJ-7','',68033,'In Earth orbit','','2009-06-02 00:00:00',95.72,541,565,97.7,'Sun-Synch' from dual union all
select 35273,'035273','2005-024M','deb SJ-7','',68033,'In Earth orbit','','2009-06-07 00:00:00',95.69,539,564,97.69,'Sun-Synch' from dual union all
select 35274,'035274','2008-056D','CZ-2D sep motor cover','',68564,'In Earth orbit','','2009-05-28 00:00:00',102.6,754,1010,98.74,'Sun-Synch' from dual union all
select 35275,'035275','2008-056E','SW-3 upper debris','',68564,'In Earth orbit','','2009-06-10 00:00:00',101.38,717,931,98.49,'Sun-Synch' from dual union all
select 35276,'035276','1993-059K','deb 11S772','',65161,'Reentered','2017-04-26 00:00:00','2009-06-07 00:00:00',101.63,818,854,71,'Intermediate' from dual union all
select 35277,'035277','1993-059L','deb 11S772','',65161,'Reentered','2014-08-19 00:00:00','2009-06-08 00:00:00',101.63,822,850,70.99,'Intermediate' from dual union all
select 35278,'035278','1993-036AJG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-10 00:00:00',101.03,740,875,74.06,'Intermediate' from dual union all
select 35279,'035279','1993-036AJH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-10 00:00:00',99.65,702,782,74.02,'Intermediate' from dual union all
select 3528,'003528','1968-096A','KDU','KDU',16823,'Reentered','1968-11-18 00:00:00','1968-11-07 00:00:00',89.65,208,307,64.91,'Intermediate' from dual union all
select 35280,'035280','1993-036AJJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-10 00:00:00',100.78,739,853,74.04,'Intermediate' from dual union all
select 35281,'035281','1993-036AJK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-10 00:00:00',100.24,753,786,74.01,'Intermediate' from dual union all
select 35282,'035282','1993-036AJL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-10 00:00:00',100.65,693,886,74.09,'Intermediate' from dual union all
select 35283,'035283','1993-036AJM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-09 00:00:00',100.44,766,793,74.05,'Intermediate' from dual union all
select 35284,'035284','1993-036AJN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-10 00:00:00',100.38,764,790,74.05,'Intermediate' from dual union all
select 35285,'035285','1993-036AJP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-09 00:00:00',100.31,757,789,74.03,'Intermediate' from dual union all
select 35286,'035286','1993-036AJQ','deb Kosmos-2251','',65138,'Reentered','2015-02-02 00:00:00','2009-06-10 00:00:00',98.04,556,774,73.97,'Intermediate' from dual union all
select 35287,'035287','1993-036AJR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-10 00:00:00',100.47,767,795,74.03,'Intermediate' from dual union all
select 35288,'035288','1993-036AJS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-10 00:00:00',100.15,726,806,73.96,'Intermediate' from dual;
insert into SATELLITE
select 35289,'035289','1993-036AJT','deb Kosmos-2251','',65138,'Reentered','2012-10-11 00:00:00','2009-06-09 00:00:00',98.34,571,787,73.94,'Intermediate' from dual union all
select 3529,'003529','1968-096B','11A57I','11A57I',16823,'Reentered','1968-11-02 00:00:00','1968-10-31 00:00:00',88.91,209,232,64.9,'Intermediate' from dual union all
select 35290,'035290','1993-036AJU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-10 00:00:00',100.38,764,790,74.02,'Intermediate' from dual union all
select 35291,'035291','1993-036AJV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-10 00:00:00',98.09,561,774,74.05,'Intermediate' from dual union all
select 35292,'035292','1993-036AJW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-10 00:00:00',99.41,684,777,74.02,'Intermediate' from dual union all
select 35293,'035293','1997-051PH','deb Iridium 33','',66131,'In Earth orbit','','2009-06-10 00:00:00',99.56,695,781,86.34,'Polar' from dual union all
select 35294,'035294','1997-051PJ','deb Iridium 33','',66131,'In Earth orbit','','2009-06-10 00:00:00',100.53,774,793,86.34,'Polar' from dual union all
select 35295,'035295','1997-051PK','deb Iridium 33','',66131,'Reentered','2014-03-09 00:00:00','2009-06-10 00:00:00',99.91,736,773,86.36,'Polar' from dual union all
select 35296,'035296','1997-051PL','deb Iridium 33','',66131,'In Earth orbit','','2009-06-09 00:00:00',100.52,772,795,86.32,'Polar' from dual union all
select 35297,'035297','1997-051PM','deb Iridium 33','',66131,'In Earth orbit','','2009-06-10 00:00:00',100.67,775,806,86.39,'Polar' from dual union all
select 35298,'035298','1997-051PN','deb Iridium 33','',66131,'Reentered','2013-04-07 00:00:00','2009-06-10 00:00:00',99.43,655,808,85.96,'Polar' from dual union all
select 35299,'035299','1997-051PP','deb Iridium 33','',66131,'In Earth orbit','','2009-06-10 00:00:00',101.46,775,880,86.3,'Polar' from dual union all
select 353,'000353','61 Omicron 114','deb r/b Ablestar','',7273,'In Earth orbit','','1991-04-13 00:00:00',101.22,770,863,66.54,'Intermediate' from dual union all
select 3530,'003530','1968-097A','Kosmos-252','I2P',16824,'Exploded','1968-11-01 00:00:00','1968-11-08 00:00:00',112.43,530,2147,62.32,'Intermediate' from dual union all
select 35300,'035300','1997-051PQ','deb Iridium 33','',66131,'Reentered','2012-03-12 00:00:00','2009-06-10 00:00:00',99.83,709,792,86.34,'Polar' from dual union all
select 35301,'035301','1997-051PR','deb Iridium 33','',66131,'Reentered','2011-03-28 00:00:00','2009-06-10 00:00:00',97.98,578,746,86.5,'Polar' from dual union all
select 35302,'035302','1997-051PS','deb Iridium 33','',66131,'In Earth orbit','','2009-06-10 00:00:00',101.16,773,855,86.28,'Polar' from dual union all
select 35303,'035303','1997-051PT','deb Iridium 33','',66131,'Reentered','2013-10-27 00:00:00','2009-06-10 00:00:00',101.58,755,912,86.4,'Polar' from dual union all
select 35304,'035304','1997-051PU','deb Iridium 33','',66131,'In Earth orbit','','2009-06-10 00:00:00',102.42,769,977,86.36,'Polar' from dual union all
select 35305,'035305','1997-051PV','deb Iridium 33','',66131,'Reentered','2011-04-02 00:00:00','2009-06-10 00:00:00',98.21,601,746,86.49,'Polar' from dual union all
select 35306,'035306','1997-051PW','deb Iridium 33','',66131,'Reentered','2014-10-09 00:00:00','2009-06-10 00:00:00',100.21,763,773,86.35,'Polar' from dual union all
select 35307,'035307','1991-025AG','deb SOZ','SOZ',64194,'In Earth orbit','','2009-06-17 00:00:00',335.75,695,18553,66.77,'Highly Ellip' from dual union all
select 35308,'035308','1991-025AH','deb SOZ','SOZ',64194,'Reentered','2010-03-24 00:00:00','2009-06-15 00:00:00',325.86,388,18228,66.78,'Highly Ellip' from dual union all
select 35309,'035309','1991-025AJ','deb SOZ','SOZ',64194,'Reentered','2010-11-14 00:00:00','2009-06-15 00:00:00',329.32,429,18409,66.67,'Highly Ellip' from dual union all
select 3531,'003531','1968-098A','OPS 1315','CORONA CR-5',16825,'Reentered Attached','1968-11-23 00:00:00','1968-11-10 00:00:00',88.69,156,264,82.15,'Intermediate' from dual union all
select 35310,'035310','1991-025AK','deb SOZ','SOZ',64194,'In Earth orbit','','2009-06-17 00:00:00',333.33,594,18499,65.95,'Highly Ellip' from dual union all
select 35311,'035311','1991-025AL','deb SOZ','SOZ',64194,'Reentered','2010-03-19 00:00:00','2009-06-16 00:00:00',327.88,403,18343,67.33,'Highly Ellip' from dual union all
select 35312,'035312','1991-025AM','deb SOZ','SOZ',64194,'Reentered','2009-06-19 00:00:00','2009-06-17 00:00:00',158.53,301,6349,66.18,'Medium' from dual union all
select 35313,'035313','1991-025AN','deb SOZ','SOZ',64194,'Reentered','2010-03-30 00:00:00','2009-06-16 00:00:00',335.83,387,18866,67.07,'Highly Ellip' from dual union all
select 35314,'035314','1991-025AP','deb SOZ','SOZ',64194,'In Earth orbit','','2009-06-15 00:00:00',293.77,1126,15394,64.83,'Medium' from dual union all
select 35315,'035315','2009-031A','Lunar Reconnaissance Orbiter','LRO',68712,'Deep Space','2009-06-19 00:00:00','2009-06-18 00:00:00',13019.44,194,353700,28.2,'Cislunar' from dual union all
select 35316,'035316','2009-031B','LCROSS Shepherding S/C','S-S/C',68712,'Deep Space Attached','2009-06-19 00:00:00','2009-06-18 00:00:00',12747.63,133,348640,28,'Cislunar' from dual union all
select 35317,'035317','2009-031C','LCROSS EDUS','Centaur AV-020',68712,'Deep Space','2009-06-19 00:00:00','2009-06-18 00:00:00',12747.63,133,348640,28,'Cislunar' from dual union all
select 35318,'035318','1999-057RW','CZ-4 debris','',66738,'Reentered','2010-06-17 00:00:00','2009-06-18 00:00:00',96.81,580,632,98.61,'Sun-Synch' from dual union all
select 35319,'035319','1999-057RX','CZ-4 debris','',66738,'In Earth orbit','','2009-05-31 00:00:00',102.49,725,1028,98.67,'Sun-Synch' from dual union all
select 3532,'003532','1968-099A','OPS 5296','GAMBIT-3 Mission 4317',16826,'Reentered Attached','1968-11-20 00:00:00','1968-11-08 00:00:00',89.58,126,382,106.04,'Retrograde' from dual union all
select 35320,'035320','1999-057RY','CZ-4 debris','',66738,'In Earth orbit','','2009-06-09 00:00:00',98.74,695,702,98.01,'Sun-Synch' from dual union all
select 35321,'035321','1999-057RZ','CZ-4 debris','',66738,'Reentered','2011-08-16 00:00:00','2009-06-09 00:00:00',96.24,501,656,98.12,'Sun-Synch' from dual union all
select 35322,'035322','1999-057SA','CZ-4 debris','',66738,'In Earth orbit','','2009-06-09 00:00:00',102.18,711,1013,98.59,'Sun-Synch' from dual union all
select 35323,'035323','1999-057SB','CZ-4 debris','',66738,'In Earth orbit','','2009-05-29 00:00:00',100.16,700,833,98.27,'Sun-Synch' from dual union all
select 35324,'035324','1999-057SC','CZ-4 debris','',66738,'In Earth orbit','','2009-06-11 00:00:00',100.47,711,851,98.26,'Sun-Synch' from dual union all
select 35325,'035325','1999-057SD','CZ-4 debris','',66738,'In Earth orbit','','2009-06-14 00:00:00',98.53,673,703,98.52,'Sun-Synch' from dual union all
select 35326,'035326','1999-057SE','CZ-4 debris','',66738,'In Earth orbit','','2009-06-14 00:00:00',102.92,728,1066,98.47,'Sun-Synch' from dual union all
select 35327,'035327','1999-057SF','CZ-4 debris','',66738,'In Earth orbit','','2009-06-08 00:00:00',100.07,688,836,98.25,'Sun-Synch' from dual union all
select 35328,'035328','1999-057SG','CZ-4 debris','',66738,'Reentered','2012-09-09 00:00:00','2009-06-15 00:00:00',96.67,599,600,98.75,'Sun-Synch' from dual union all
select 35329,'035329','1999-057SH','CZ-4 debris','',66738,'In Earth orbit','','2009-06-17 00:00:00',100.94,705,902,98.39,'Sun-Synch' from dual union all
select 3533,'003533','1968-100A','Pioneer 9','Pioneer D',16827,'Deep Space','1968-11-10 00:00:00','1968-11-08 00:00:00',3409.11,455,163279,34.67,'Earth Escape' from dual union all
select 35330,'035330','1999-057SJ','CZ-4 debris','',66738,'In Earth orbit','','2009-06-17 00:00:00',100.18,705,830,98.17,'Sun-Synch' from dual union all
select 35331,'035331','1999-057SK','CZ-4 debris','',66738,'In Earth orbit','','2009-06-14 00:00:00',100.49,681,883,98.37,'Sun-Synch' from dual union all
select 35332,'035332','1999-057SL','CZ-4 debris','',66738,'In Earth orbit','','2009-06-14 00:00:00',101.01,716,897,98.4,'Sun-Synch' from dual union all
select 35333,'035333','1999-057SM','CZ-4 debris','',66738,'Reentered','2011-12-13 00:00:00','2009-06-19 00:00:00',96.16,536,614,98.71,'Sun-Synch' from dual union all
select 35334,'035334','1999-057SN','CZ-4 debris','',66738,'In Earth orbit','','2009-06-18 00:00:00',99.91,694,815,98.14,'Sun-Synch' from dual union all
select 35335,'035335','2001-049PT','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-06-05 00:00:00',98.09,631,704,97.73,'Sun-Synch' from dual union all
select 35336,'035336','2001-049PU','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-05-25 00:00:00',98.89,642,770,97.85,'Sun-Synch' from dual union all
select 35337,'035337','2001-049PV','deb PSLV-C3 PS4','',67202,'Reentered','2013-05-19 00:00:00','2009-06-19 00:00:00',95.49,523,561,97.51,'Sun-Synch' from dual union all
select 35338,'035338','2001-049PW','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-06-15 00:00:00',100.68,647,934,97.85,'Sun-Synch' from dual union all
select 35339,'035339','2001-049PX','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-06-18 00:00:00',100.89,655,946,98.28,'Sun-Synch' from dual union all
select 3534,'003534','1968-100B','TETR 2','TETR B',16827,'Reentered','1979-09-19 00:00:00','1968-12-10 00:00:00',97.86,379,934,32.87,'Intermediate' from dual union all
select 35340,'035340','2001-049PY','deb PSLV-C3 PS4','',67202,'Reentered','2013-02-16 00:00:00','2009-06-15 00:00:00',97.68,645,651,98.55,'Sun-Synch' from dual union all
select 35341,'035341','2001-049PZ','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-06-06 00:00:00',98.31,645,710,97.83,'Sun-Synch' from dual union all
select 35342,'035342','2001-049QA','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-06-12 00:00:00',103.99,666,1229,98.27,'Sun-Synch' from dual union all
select 35343,'035343','2001-049QB','deb PSLV-C3 PS4','',67202,'Reentered','2013-06-12 00:00:00','2009-06-05 00:00:00',97.42,608,662,98.59,'Sun-Synch' from dual union all
select 35344,'035344','2001-049QC','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-06-18 00:00:00',99.07,668,761,98.25,'Sun-Synch' from dual union all
select 35345,'035345','2001-049QD','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-06-12 00:00:00',99.43,674,789,98.19,'Sun-Synch' from dual union all
select 35346,'035346','2001-049QE','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-06-12 00:00:00',100.25,735,805,98.94,'Sun-Synch' from dual union all
select 35347,'035347','2001-049QF','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-06-19 00:00:00',98.72,651,744,98.31,'Sun-Synch' from dual union all
select 35348,'035348','2001-049QG','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-06-14 00:00:00',99.17,668,770,98.13,'Sun-Synch' from dual union all
select 35349,'035349','2001-049QH','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-06-17 00:00:00',97.03,590,643,97.7,'Sun-Synch' from dual union all
select 3535,'003535','1968-101A','Zond-6','L-1 No. 12L',16828,'Deep Space','1968-11-13 00:00:00','1968-11-11 00:00:00',15562.18,200,400000,51.5,'Cislunar' from dual union all
select 35350,'035350','2001-049QJ','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-06-01 00:00:00',99.88,697,809,98.11,'Sun-Synch' from dual union all
select 35351,'035351','2001-049QK','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-06-19 00:00:00',100.46,777,784,98.74,'Sun-Synch' from dual union all
select 35352,'035352','1993-014BB','deb Start','',65116,'Reentered','2012-11-02 00:00:00','2009-06-12 00:00:00',99.98,634,882,75.74,'Intermediate' from dual union all
select 35353,'035353','1993-014BC','deb Start','',65116,'Reentered','2015-03-20 00:00:00','2009-06-19 00:00:00',100.68,680,901,75.76,'Intermediate' from dual union all
select 35354,'035354','1993-014BD','deb Start','',65116,'In Earth orbit','','2009-06-15 00:00:00',100.44,668,891,75.75,'Intermediate' from dual union all
select 35355,'035355','1993-014BE','deb Start','',65116,'Reentered','2012-05-29 00:00:00','2009-06-15 00:00:00',99.34,652,803,75.76,'Intermediate' from dual union all
select 35356,'035356','1993-014BF','deb Start','',65116,'In Earth orbit','','2009-06-17 00:00:00',100.64,673,905,75.76,'Intermediate' from dual union all
select 35357,'035357','1993-014BG','deb Start','',65116,'Reentered','2013-02-17 00:00:00','2009-06-16 00:00:00',99.69,661,826,75.75,'Intermediate' from dual union all
select 35358,'035358','1993-014BH','deb Start','',65116,'Reentered','2013-12-19 00:00:00','2009-06-12 00:00:00',100.16,643,890,75.75,'Intermediate' from dual union all
select 35359,'035359','1993-014BJ','deb Start','',65116,'In Earth orbit','','2009-06-15 00:00:00',100.7,686,898,75.76,'Intermediate' from dual union all
select 3536,'003536','1968-101B','SOZ','SOZ',16828,'Reentered','1968-11-12 00:00:00','1968-11-11 00:00:00',87.97,174,174,51.49,'Intermediate' from dual union all
select 35360,'035360','1993-014BK','deb Start','',65116,'In Earth orbit','','2009-06-15 00:00:00',100.47,667,895,75.75,'Intermediate' from dual union all
select 35361,'035361','1993-014BL','deb Start','',65116,'Reentered','2016-10-16 00:00:00','2009-06-19 00:00:00',100.06,661,861,75.75,'Intermediate' from dual union all
select 35362,'035362','2009-032A','Measat 3A','Measat 3A',68713,'In Earth orbit','','2009-07-22 00:00:00',1436.11,35785,35789,.02,'Stationary' from dual union all
select 35363,'035363','2009-032B','Blok DM-SLB No. 3L','Blok DM-SLB No. 3L',68713,'In Earth orbit','','2009-07-26 00:00:00',853.57,11242,35616,10.5,'Medium' from dual union all
select 35364,'035364','1988-005Y','deb Meteor-2','',61254,'In Earth orbit','','2009-05-28 00:00:00',103.76,929,944,82.56,'Intermediate' from dual union all
select 35365,'035365','1988-005Z','deb Meteor-2','',61254,'In Earth orbit','','2009-06-06 00:00:00',103.9,934,952,82.53,'Intermediate' from dual union all
select 35366,'035366','1988-005AA','deb Meteor-2','',61254,'In Earth orbit','','2009-06-18 00:00:00',103.9,933,952,82.54,'Intermediate' from dual union all
select 35367,'035367','1988-005AB','deb Meteor-2','',61254,'In Earth orbit','','2009-06-20 00:00:00',103.95,933,958,82.52,'Intermediate' from dual union all
select 35368,'035368','1988-005AC','deb Meteor-2','',61254,'In Earth orbit','','2009-06-15 00:00:00',103.7,918,949,82.57,'Intermediate' from dual union all
select 35369,'035369','1988-005AD','deb Meteor-2','',61254,'In Earth orbit','','2009-06-05 00:00:00',103.98,932,962,82.53,'Intermediate' from dual union all
select 3537,'003537','1968-101C','SOZ','SOZ',16828,'Reentered','1968-11-13 00:00:00','1968-11-13 00:00:00',87.79,140,191,51.47,'Intermediate' from dual union all
select 35370,'035370','1988-005AE','deb Meteor-2','',61254,'In Earth orbit','','2009-06-21 00:00:00',103.9,926,960,82.52,'Intermediate' from dual union all
select 35371,'035371','1988-005AF','deb Meteor-2','',61254,'In Earth orbit','','2009-06-16 00:00:00',103.87,929,953,82.52,'Intermediate' from dual union all
select 35372,'035372','1988-005AG','deb Meteor-2','',61254,'In Earth orbit','','2009-06-15 00:00:00',104.04,937,962,82.54,'Intermediate' from dual union all
select 35373,'035373','1988-005AH','deb Meteor-2','',61254,'In Earth orbit','','2009-06-12 00:00:00',103.87,931,951,82.53,'Intermediate' from dual union all
select 35374,'035374','2006-046G','CZ-4B sep motor cover','',68218,'In Earth orbit','','2009-06-22 00:00:00',95.71,492,614,97.59,'Sun-Synch' from dual union all
select 35375,'035375','2006-046H','CZ-4B sep motor cover','',68218,'In Earth orbit','','2009-06-21 00:00:00',96.47,563,616,97.59,'Sun-Synch' from dual union all
select 35376,'035376','2006-046J','CZ-4B sep motor cover','',68218,'In Earth orbit','','2009-06-21 00:00:00',100.11,593,934,97.78,'Sun-Synch' from dual union all
select 35377,'035377','2006-046K','SJ-6 low period debris','',68218,'Reentered','2013-01-18 00:00:00','2009-06-22 00:00:00',95.95,484,644,97.65,'Sun-Synch' from dual union all
select 35378,'035378','2006-046L','CZ-4B sep motor cover','',68218,'In Earth orbit','','2009-06-22 00:00:00',99.61,589,890,97.84,'Sun-Synch' from dual;
insert into SATELLITE
select 35379,'035379','2003-049F','CZ-4B sep motor cover','',67673,'In Earth orbit','','2009-06-22 00:00:00',103.82,808,1071,98.81,'Sun-Synch' from dual union all
select 3538,'003538','1968-101D','SOK','Support cone',16828,'Reentered','1968-11-13 00:00:00','1968-11-11 00:00:00',87.9,165,176,51.47,'Intermediate' from dual union all
select 35380,'035380','2003-049G','deb CZ-4B','',67673,'In Earth orbit','','2009-06-22 00:00:00',98.85,650,758,98.2,'Sun-Synch' from dual union all
select 35381,'035381','2003-049H','CZ-4B sep motor cover','',67673,'In Earth orbit','','2009-06-22 00:00:00',99.11,631,802,98.66,'Sun-Synch' from dual union all
select 35382,'035382','2003-049J','deb CZ-4B','',67673,'In Earth orbit','','2009-06-22 00:00:00',101.84,727,964,98.42,'Sun-Synch' from dual union all
select 35383,'035383','2003-049K','deb CZ-4B','',67673,'In Earth orbit','','2009-06-09 00:00:00',98.99,673,748,98.49,'Sun-Synch' from dual union all
select 35384,'035384','1999-057SP','ZY-1 debris','',66738,'In Earth orbit','','2009-06-21 00:00:00',100.06,751,772,98.19,'Sun-Synch' from dual union all
select 35385,'035385','1999-057SQ','ZY-1 debris','',66738,'Reentered','2013-05-31 00:00:00','2009-06-05 00:00:00',99.59,725,752,98.29,'Sun-Synch' from dual union all
select 35386,'035386','1999-057SR','ZY-1 debris','',66738,'Reentered','2015-06-18 00:00:00','2009-06-17 00:00:00',100.29,761,783,98.18,'Sun-Synch' from dual union all
select 35387,'035387','1999-057SS','ZY-1 debris','',66738,'In Earth orbit','','2009-06-21 00:00:00',100.32,769,779,98.16,'Sun-Synch' from dual union all
select 35388,'035388','2008-064C','CZ-4C sep motor cover','',68572,'Reentered','2015-03-30 00:00:00','2009-06-22 00:00:00',98.19,490,855,97.24,'Sun-Synch' from dual union all
select 35389,'035389','2008-064D','CZ-4C sep motor cover','',68572,'Reentered','2015-02-09 00:00:00','2009-06-22 00:00:00',98.22,490,857,97.46,'Sun-Synch' from dual union all
select 3539,'003539','1968-101E','L-1 part','',16828,'Reentered','1968-11-13 00:00:00','1968-11-13 00:00:00',88.08,170,188,51.54,'Intermediate' from dual union all
select 35390,'035390','2008-064E','deb CZ-4C','',68572,'Reentered','2011-11-11 00:00:00','2009-06-22 00:00:00',92.97,358,482,97.29,'Sun-Synch' from dual union all
select 35391,'035391','1995-021E','deb ERS-2','',65682,'In Earth orbit','','2009-06-21 00:00:00',100.6,783,791,98.44,'Sun-Synch' from dual union all
select 35392,'035392','1995-021F','deb ERS-2','',65682,'In Earth orbit','','2009-06-22 00:00:00',100.46,777,784,98.55,'Sun-Synch' from dual union all
select 35393,'035393','1995-021G','deb ERS-2','',65682,'In Earth orbit','','2009-06-21 00:00:00',100.29,769,776,98.4,'Sun-Synch' from dual union all
select 35394,'035394','1983-004D','deb IRAS','',55761,'Reentered','2011-09-09 00:00:00','2009-06-21 00:00:00',99.63,690,792,99.67,'Sun-Synch' from dual union all
select 35395,'035395','1983-004E','deb IRAS','',55761,'Reentered','2010-10-13 00:00:00','2009-06-22 00:00:00',99.59,696,781,99.61,'Sun-Synch' from dual union all
select 35396,'035396','1983-004F','deb IRAS','',55761,'Reentered','2010-04-09 00:00:00','2009-06-22 00:00:00',99.79,632,865,100.21,'Sun-Synch' from dual union all
select 35397,'035397','2007-042E','CZ-4B sep motor cover','',68384,'In Earth orbit','','2009-06-19 00:00:00',99.42,637,825,98.58,'Sun-Synch' from dual union all
select 35398,'035398','2007-042F','CZ-4B sep motor cover','',68384,'In Earth orbit','','2009-06-22 00:00:00',98.48,581,791,98.46,'Sun-Synch' from dual union all
select 35399,'035399','1991-068AV','deb S5M','',64237,'In Earth orbit','','2009-05-29 00:00:00',116.7,1305,1760,82.36,'Intermediate' from dual union all
select 354,'000354','61 Omicron 115','deb r/b Ablestar','',7273,'In Earth orbit','','1991-04-13 00:00:00',107.14,958,1231,67.07,'Intermediate' from dual union all
select 3540,'003540','1968-096C','Blenda','BZA',16823,'Reentered','1968-11-16 00:00:00','1968-11-13 00:00:00',90.17,212,354,64.95,'Intermediate' from dual union all
select 35400,'035400','1991-068AW','deb S5M','',64237,'In Earth orbit','','2009-06-22 00:00:00',112,1171,1467,82.92,'Intermediate' from dual union all
select 35401,'035401','1993-041E','deb RADCAL','',65143,'In Earth orbit','','2009-06-21 00:00:00',99.6,697,782,89.55,'Polar' from dual union all
select 35402,'035402','1993-041F','deb RADCAL','',65143,'Reentered','2012-02-22 00:00:00','2009-06-08 00:00:00',96.56,573,615,89.52,'Polar' from dual union all
select 35403,'035403','1993-041G','deb RADCAL','',65143,'In Earth orbit','','2009-06-20 00:00:00',100.94,741,866,89.53,'Polar' from dual union all
select 35404,'035404','2002-024J','CZ-4B debris','',67400,'In Earth orbit','','2009-06-17 00:00:00',101.47,789,868,98.48,'Sun-Synch' from dual union all
select 35405,'035405','2002-024K','CZ-4B sep motor cover','',67400,'In Earth orbit','','2009-06-19 00:00:00',102.1,762,955,98.61,'Sun-Synch' from dual union all
select 35406,'035406','1982-116F','deb Meteor-2','',54425,'In Earth orbit','','2009-06-22 00:00:00',101.64,801,871,81.25,'Intermediate' from dual union all
select 35407,'035407','1982-116G','deb Meteor-2','',54425,'In Earth orbit','','2009-06-17 00:00:00',101.51,799,862,81.25,'Intermediate' from dual union all
select 35408,'035408','1986-019FL','deb Ariane V16','',59212,'In Earth orbit','','2009-06-22 00:00:00',101.1,787,835,98.98,'Sun-Synch' from dual union all
select 35409,'035409','1986-019VP','deb Ariane V16','',59212,'Reentered','2012-12-24 00:00:00','2009-06-21 00:00:00',99.88,732,774,98.44,'Sun-Synch' from dual union all
select 3541,'003541','1968-096D','KDU part','KDU part',16823,'Reentered','1968-11-13 00:00:00','1968-11-13 00:00:00',89.57,249,258,64.9,'Intermediate' from dual union all
select 35410,'035410','1970-025QR','deb Agena D','',21502,'Reentered','2014-12-02 00:00:00','2009-06-22 00:00:00',98.7,627,766,99.88,'Sun-Synch' from dual union all
select 35411,'035411','1982-118F','deb DMSP Block 5D-2 F-6','',54427,'Reentered','2011-02-14 00:00:00','2009-06-22 00:00:00',99.29,700,750,98.65,'Sun-Synch' from dual union all
select 35412,'035412','1987-079BN','deb SOZ','',60169,'Reentered','2009-12-29 00:00:00','2009-06-22 00:00:00',105.29,426,1589,65.05,'Intermediate' from dual union all
select 35413,'035413','2006-029F','deb Dnepr','',68201,'In Earth orbit','','2009-06-21 00:00:00',95.48,528,556,64.51,'Intermediate' from dual union all
select 35414,'035414','2001-055F','Jason impact debris','',67208,'In Earth orbit','','2009-06-22 00:00:00',111.02,1230,1318,66.03,'Intermediate' from dual union all
select 35415,'035415','2000-047D','Titan 4B deb','',66949,'In Earth orbit','','2009-06-21 00:00:00',96.71,552,650,67.99,'Intermediate' from dual union all
select 35416,'035416','1968-012F','deb NNS O-18 insulation','',16739,'In Earth orbit','','2009-06-22 00:00:00',106.6,1017,1122,89.98,'Polar' from dual union all
select 35417,'035417','2004-012L','CZ-2C Stage 2 part','',67856,'In Earth orbit','','2009-06-22 00:00:00',99.19,602,838,97.81,'Sun-Synch' from dual union all
select 35418,'035418','2006-002Z','deb ALOS','',68174,'In Earth orbit','','2009-06-22 00:00:00',98.62,692,694,98.1,'Sun-Synch' from dual union all
select 35419,'035419','1991-082CM','deb DMSP F-11','',64251,'In Earth orbit','','2009-06-22 00:00:00',102.01,782,925,98.63,'Sun-Synch' from dual union all
select 3542,'003542','1968-102A','Kosmos-253','Zenit-2 No. 67',16829,'Landed','1968-11-18 00:00:00','1968-11-17 00:00:00',89.77,197,330,65.42,'Intermediate' from dual union all
select 35420,'035420','2001-056M','deb Meteor-3M ','',67209,'In Earth orbit','','2009-06-22 00:00:00',104.86,969,1006,99.36,'Sun-Synch' from dual union all
select 35421,'035421','2006-050BX','deb Delta 320','deb DCSS-4 F06',68222,'Reentered','2010-03-02 00:00:00','2009-06-22 00:00:00',98.05,616,715,99.21,'Sun-Synch' from dual union all
select 35422,'035422','1975-124D','deb Meteor','',36571,'In Earth orbit','','2009-06-18 00:00:00',101.95,842,860,81.27,'Intermediate' from dual union all
select 35423,'035423','1977-002F','deb Meteor-2','',42130,'In Earth orbit','','2009-06-10 00:00:00',116.84,956,2122,103.91,'Sun-Synch' from dual union all
select 35424,'035424','1981-043M','deb Meteor-2','',52566,'In Earth orbit','','2009-06-21 00:00:00',102.04,824,887,81.27,'Intermediate' from dual union all
select 35425,'035425','2006-057GH','USA 193 debris','',68229,'Reentered','2009-10-28 00:00:00','2009-06-22 00:00:00',96.35,234,934,56.1,'Intermediate' from dual union all
select 35426,'035426','1980-073J','deb Meteor-2','',50283,'In Earth orbit','','2009-06-15 00:00:00',101.67,814,862,81.22,'Intermediate' from dual union all
select 35427,'035427','1972-022C','deb Meteor','',27954,'In Earth orbit','','2009-06-06 00:00:00',101.83,822,869,81.22,'Intermediate' from dual union all
select 35428,'035428','2004-045D','Star 48 yo weight','',67889,'In Earth orbit','','2009-06-22 00:00:00',290.99,192,16144,39.03,'Highly Ellip' from dual union all
select 35429,'035429','1993-036AJX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',98.7,523,870,73.97,'Intermediate' from dual union all
select 3543,'003543','1968-102B','11A57I','11A57I',16829,'Reentered','1968-11-20 00:00:00','1968-11-15 00:00:00',89.52,194,308,65.4,'Intermediate' from dual union all
select 35430,'035430','1993-036AJY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',99.18,666,773,74.05,'Intermediate' from dual union all
select 35431,'035431','1993-036AJZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-25 00:00:00',100.11,740,788,74.03,'Intermediate' from dual union all
select 35432,'035432','1993-036AKA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',100.43,765,793,74.05,'Intermediate' from dual union all
select 35433,'035433','1993-036AKB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',100.74,772,815,74.03,'Intermediate' from dual union all
select 35434,'035434','1993-036AKC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',101.8,769,919,74.05,'Intermediate' from dual union all
select 35435,'035435','1993-036AKD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',100.56,763,807,74.03,'Intermediate' from dual union all
select 35436,'035436','1993-036AKE','deb Kosmos-2251','',65138,'Reentered','2012-08-24 00:00:00','2009-06-26 00:00:00',98.51,608,766,73.93,'Intermediate' from dual union all
select 35437,'035437','1993-036AKF','deb Kosmos-2251','',65138,'Reentered','2013-11-29 00:00:00','2009-06-25 00:00:00',98.13,568,771,73.94,'Intermediate' from dual union all
select 35438,'035438','1993-036AKG','deb Kosmos-2251','',65138,'Reentered','2010-03-24 00:00:00','2009-06-26 00:00:00',94.77,299,716,73.96,'Intermediate' from dual union all
select 35439,'035439','1993-036AKH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',104.77,771,1197,74.03,'Intermediate' from dual union all
select 3544,'003544','1968-103A','Proton-4','N-6 No. 1',16830,'Reentered','1969-07-24 00:00:00','1969-01-01 00:00:00',91.49,249,447,51.55,'Intermediate' from dual union all
select 35440,'035440','1993-036AKJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',99.19,655,785,74.02,'Intermediate' from dual union all
select 35441,'035441','1993-036AKK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',100.89,754,848,74.14,'Intermediate' from dual union all
select 35442,'035442','1993-036AKL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',100.55,768,802,74.05,'Intermediate' from dual union all
select 35443,'035443','1993-036AKM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',102.02,650,1059,74.05,'Intermediate' from dual union all
select 35444,'035444','1993-036AKN','deb Kosmos-2251','',65138,'Reentered','2017-12-27 00:00:00','2009-06-26 00:00:00',97.29,484,774,73.93,'Intermediate' from dual union all
select 35445,'035445','1993-036AKP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',101.24,646,988,74.04,'Intermediate' from dual union all
select 35446,'035446','1993-036AKQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',99.24,665,780,74.04,'Intermediate' from dual union all
select 35447,'035447','1993-036AKR','deb Kosmos-2251','',65138,'Reentered','2012-01-19 00:00:00','2009-06-25 00:00:00',96.67,427,772,74.06,'Intermediate' from dual union all
select 35448,'035448','1993-036AKS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-25 00:00:00',100.47,769,793,73.91,'Intermediate' from dual union all
select 35449,'035449','1993-036AKT','deb Kosmos-2251','',65138,'Reentered','2016-01-01 00:00:00','2009-06-26 00:00:00',99.36,686,770,73.96,'Intermediate' from dual union all
select 3545,'003545','1968-103B','8S812','8S812',16830,'Reentered','1969-01-25 00:00:00','1968-12-16 00:00:00',90.93,232,409,51.54,'Intermediate' from dual union all
select 35450,'035450','1993-036AKU','deb Kosmos-2251','',65138,'Reentered','2013-09-27 00:00:00','2009-06-26 00:00:00',99.2,535,906,73.96,'Intermediate' from dual union all
select 35451,'035451','1993-036AKV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',98.32,582,775,73.95,'Intermediate' from dual union all
select 35452,'035452','1993-036AKW','deb Kosmos-2251','',65138,'Reentered','2012-09-02 00:00:00','2009-06-26 00:00:00',97.71,523,776,73.96,'Intermediate' from dual union all
select 35453,'035453','1993-036AKX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',101.22,753,880,74.01,'Intermediate' from dual union all
select 35454,'035454','1993-036AKY','deb Kosmos-2251','',65138,'Reentered','2014-05-23 00:00:00','2009-06-26 00:00:00',98.11,564,773,73.86,'Intermediate' from dual union all
select 35455,'035455','1993-036AKZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',100.97,702,908,74.03,'Intermediate' from dual union all
select 35456,'035456','1993-036ALA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',100.72,709,877,73.96,'Intermediate' from dual union all
select 35457,'035457','1993-036ALB','deb Kosmos-2251','',65138,'Reentered','2014-02-11 00:00:00','2009-06-26 00:00:00',98.65,614,775,73.93,'Intermediate' from dual union all
select 35458,'035458','1993-036ALC','deb Kosmos-2251','',65138,'Reentered','2013-11-27 00:00:00','2009-06-26 00:00:00',97.69,524,773,73.95,'Intermediate' from dual union all
select 35459,'035459','1993-036ALD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',100.61,774,801,73.87,'Intermediate' from dual union all
select 3546,'003546','1968-100C','deb Delta 60','',16827,'Reentered','1971-10-06 00:00:00','1968-12-14 00:00:00',97.87,373,941,32.85,'Intermediate' from dual union all
select 35460,'035460','1993-036ALE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',99.15,631,805,74,'Intermediate' from dual union all
select 35461,'035461','1993-036ALF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-25 00:00:00',100.94,738,869,73.84,'Intermediate' from dual union all
select 35462,'035462','1993-036ALG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',102.16,770,952,74.06,'Intermediate' from dual union all
select 35463,'035463','1993-036ALH','deb Kosmos-2251','',65138,'Reentered','2016-01-23 00:00:00','2009-06-26 00:00:00',98.25,573,777,74.04,'Intermediate' from dual union all
select 35464,'035464','1993-036ALJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-25 00:00:00',99.52,689,783,74.02,'Intermediate' from dual union all
select 35465,'035465','1993-036ALK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-25 00:00:00',100.42,725,832,74,'Intermediate' from dual union all
select 35466,'035466','1993-036ALL','deb Kosmos-2251','',65138,'Reentered','2013-06-12 00:00:00','2009-06-25 00:00:00',98.76,607,792,74.03,'Intermediate' from dual union all
select 35467,'035467','1993-036ALM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',97.88,541,774,74.05,'Intermediate' from dual union all
select 35468,'035468','1993-036ALN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',99.17,655,783,74.05,'Intermediate' from dual;
insert into SATELLITE
select 35469,'035469','1993-036ALP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',99.56,680,796,74.01,'Intermediate' from dual union all
select 3547,'003547','1968-100D','Delta 60','DSV-3E-3 S/N 20220',16827,'Reentered','1978-04-13 00:00:00','1968-12-12 00:00:00',97.84,373,938,32.87,'Intermediate' from dual union all
select 35470,'035470','1993-036ALQ','deb Kosmos-2251','',65138,'Reentered','2014-10-16 00:00:00','2009-06-26 00:00:00',98.53,604,773,73.95,'Intermediate' from dual union all
select 35471,'035471','1993-036ALR','deb Kosmos-2251','',65138,'Reentered','2014-01-26 00:00:00','2009-06-26 00:00:00',98.9,616,797,74.08,'Intermediate' from dual union all
select 35472,'035472','1993-036ALS','deb Kosmos-2251','',65138,'Reentered','2013-05-19 00:00:00','2009-06-26 00:00:00',99.2,665,775,74.05,'Intermediate' from dual union all
select 35473,'035473','1993-036ALT','deb Kosmos-2251','',65138,'Reentered','2015-02-04 00:00:00','2009-06-26 00:00:00',99.53,676,797,74.04,'Intermediate' from dual union all
select 35474,'035474','1993-036ALU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',100.23,767,772,74,'Intermediate' from dual union all
select 35475,'035475','1993-036ALV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',100.73,760,826,74.04,'Intermediate' from dual union all
select 35476,'035476','1993-036ALW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-06-26 00:00:00',100.41,753,803,73.99,'Intermediate' from dual union all
select 35477,'035477','1997-051PX','deb Iridium 33','',66131,'In Earth orbit','','2009-06-26 00:00:00',102.7,769,1004,86.41,'Polar' from dual union all
select 35478,'035478','1997-051PY','deb Iridium 33','',66131,'In Earth orbit','','2009-06-26 00:00:00',101.8,767,920,86.47,'Polar' from dual union all
select 35479,'035479','1997-051PZ','deb Iridium 33','',66131,'In Earth orbit','','2009-06-25 00:00:00',102.57,773,988,86.38,'Polar' from dual union all
select 3548,'003548','1968-100E','TETR 2 cover','Canister cover',16827,'Reentered','1972-01-06 00:00:00','1968-12-17 00:00:00',97.83,374,936,32.87,'Intermediate' from dual union all
select 35480,'035480','1997-051QA','deb Iridium 33','',66131,'In Earth orbit','','2009-06-26 00:00:00',100.41,766,789,86.36,'Polar' from dual union all
select 35481,'035481','1997-051QB','deb Iridium 33','',66131,'Reentered','2013-03-22 00:00:00','2009-06-26 00:00:00',98.88,646,765,86.45,'Polar' from dual union all
select 35482,'035482','1997-051QC','deb Iridium 33','',66131,'Reentered','2012-09-27 00:00:00','2009-06-25 00:00:00',98.75,502,896,86.61,'Polar' from dual union all
select 35483,'035483','1997-051QD','deb Iridium 33','',66131,'In Earth orbit','','2009-06-26 00:00:00',101.71,773,907,86.44,'Polar' from dual union all
select 35484,'035484','1997-051QE','deb Iridium 33','',66131,'In Earth orbit','','2009-06-26 00:00:00',100.16,719,814,86.38,'Polar' from dual union all
select 35485,'035485','1997-051QF','deb Iridium 33','',66131,'Reentered','2012-08-09 00:00:00','2009-06-26 00:00:00',99.07,643,786,86.22,'Polar' from dual union all
select 35486,'035486','1997-051QG','deb Iridium 33','',66131,'Reentered','2014-11-23 00:00:00','2009-06-26 00:00:00',101.14,769,857,86.4,'Polar' from dual union all
select 35487,'035487','1997-051QH','deb Iridium 33','',66131,'In Earth orbit','','2009-06-26 00:00:00',101.79,770,917,86.36,'Polar' from dual union all
select 35488,'035488','1997-051QJ','deb Iridium 33','',66131,'In Earth orbit','','2009-06-26 00:00:00',99.66,708,777,86.34,'Polar' from dual union all
select 35489,'035489','1997-051QK','deb Iridium 33','',66131,'Reentered','2015-02-26 00:00:00','2009-06-26 00:00:00',100.34,762,788,86.41,'Polar' from dual union all
select 3549,'003549','1968-097B','deb Kosmos-252','',16824,'In Earth orbit','','1969-01-21 00:00:00',112.34,532,2137,62.32,'Intermediate' from dual union all
select 35490,'035490','1997-051QL','deb Iridium 33','',66131,'Reentered','2010-06-19 00:00:00','2009-06-26 00:00:00',99.07,699,729,86.29,'Polar' from dual union all
select 35491,'035491','2009-033A','GOES 14','GOES O',68714,'In Earth orbit','','2009-08-16 00:00:00',1436.09,35783,35790,.41,'Stationary' from dual union all
select 35492,'035492','2009-033B','Delta 342','DCSS-4 F07',68714,'In Earth orbit','','2009-07-28 00:00:00',745.42,6591,35119,13.24,'GEO Transfer' from dual union all
select 35493,'035493','2009-034A','Sirius FM-5','Sirius CD Radio 5',68715,'In Earth orbit','','2009-07-07 00:00:00',1415.85,35005,35774,.26,'Drift GEO' from dual union all
select 35494,'035494','2009-034B','Briz-M No. 99506','Briz-M',68715,'In Earth orbit','','2009-07-08 00:00:00',704.56,4018,35683,22.98,'GEO Transfer' from dual union all
select 35495,'035495','2009-034C','Briz-M DTB','Briz-M torus tank',68715,'In Earth orbit','','2009-07-08 00:00:00',269.11,325,14534,49.62,'Highly Ellip' from dual union all
select 35496,'035496','2009-035A','Terrestar 1','Terrestar 1',68716,'In Earth orbit','','2009-07-08 00:00:00',1436.19,35707,35870,5.94,'Inclined GEO' from dual union all
select 35497,'035497','2009-035B','ESC-A 547','Ariane 5 ESC-A',68716,'Reentered','2016-10-31 00:00:00','2009-07-09 00:00:00',630.73,218,35751,6.08,'GEO Transfer' from dual union all
select 35498,'035498','2009-036A','Kosmos-2451','Strela-3 No. 58',68717,'In Earth orbit','','2009-07-13 00:00:00',116.03,1498,1507,82.49,'Intermediate' from dual union all
select 35499,'035499','2009-036B','Kosmos-2452','Rodnik No. 12',68717,'In Earth orbit','','2009-07-13 00:00:00',116.01,1497,1506,82.5,'Intermediate' from dual union all
select 355,'000355','61 Omicron 116','deb r/b Ablestar','',7273,'In Earth orbit','','1962-07-29 00:00:00',109.51,979,1430,67.23,'Intermediate' from dual union all
select 3550,'003550','1968-097C','deb Kosmos-252','',16824,'Reentered','1988-05-08 00:00:00','1988-05-05 00:00:00',90.57,284,320,62.24,'Intermediate' from dual union all
select 35500,'035500','2009-036C','Kosmos-2453','Strela-3 No. 59',68717,'In Earth orbit','','2009-07-13 00:00:00',115.99,1496,1504,82.49,'Intermediate' from dual union all
select 35501,'035501','2009-036D','Briz-KM No. 72510','Briz-KM 14S45',68717,'In Earth orbit','','2009-07-13 00:00:00',112.79,1205,1505,82.49,'Intermediate' from dual union all
select 35502,'035502','1965-108DH','deb Transtage 8','',11858,'In Earth orbit','','2009-07-04 00:00:00',494.38,460,28206,27.03,'Highly Ellip' from dual union all
select 35503,'035503','1965-108DJ','deb Transtage 8','',11858,'In Earth orbit','','2009-07-09 00:00:00',554.57,768,31195,26.12,'Highly Ellip' from dual union all
select 35504,'035504','1977-047R','deb Kosmos-917','deb Oko',42175,'In Earth orbit','','2009-07-10 00:00:00',717.6,2713,37633,69.89,'GEO Transfer' from dual union all
select 35505,'035505','1977-047S','deb Kosmos-917','deb Oko',42175,'In Earth orbit','','2009-07-08 00:00:00',714.13,5524,34651,66.75,'GEO Transfer' from dual union all
select 35506,'035506','1979-095Z','deb Meteor-2','',47579,'In Earth orbit','','2009-07-09 00:00:00',102.01,840,868,81.2,'Intermediate' from dual union all
select 35507,'035507','1981-031N','deb Kosmos-1261','',52554,'In Earth orbit','','2009-07-10 00:00:00',713.59,4329,35819,66.44,'GEO Transfer' from dual union all
select 35508,'035508','1981-108H','deb Kosmos-1317','',52631,'In Earth orbit','','2009-07-08 00:00:00',717.4,5737,34598,71.12,'GEO Transfer' from dual union all
select 35509,'035509','1981-108J','deb Kosmos-1317','',52631,'In Earth orbit','','2009-06-30 00:00:00',721.19,5610,34913,70.06,'GEO Transfer' from dual union all
select 3551,'003551','1968-097D','deb Kosmos-252','',16824,'In Earth orbit','','1992-02-03 00:00:00',108.2,517,1769,62.3,'Intermediate' from dual union all
select 35510,'035510','1981-108K','deb Kosmos-1317','',52631,'In Earth orbit','','2009-04-12 00:00:00',719.52,5791,34650,70.81,'GEO Transfer' from dual union all
select 35511,'035511','1981-108L','deb Kosmos-1317','',52631,'In Earth orbit','','2009-07-10 00:00:00',719.04,5853,34564,70.98,'GEO Transfer' from dual union all
select 35512,'035512','1981-108M','deb Kosmos-1317','',52631,'In Earth orbit','','2009-07-10 00:00:00',713.47,5399,34743,70.72,'GEO Transfer' from dual union all
select 35513,'035513','1981-108N','deb Kosmos-1317','',52631,'In Earth orbit','','2009-07-09 00:00:00',712.74,5334,34772,70.49,'GEO Transfer' from dual union all
select 35514,'035514','1981-108P','deb Kosmos-1317','',52631,'In Earth orbit','','2009-07-02 00:00:00',710.68,5026,34978,70.05,'GEO Transfer' from dual union all
select 35515,'035515','1982-029N','deb Kosmos-1348','',54338,'In Earth orbit','','2009-07-07 00:00:00',700.96,4969,34553,69.56,'GEO Transfer' from dual union all
select 35516,'035516','1982-116H','deb Meteor-2','',54425,'In Earth orbit','','2009-07-10 00:00:00',101.68,790,886,81.25,'Intermediate' from dual union all
select 35517,'035517','1986-019VQ','deb Ariane V16','',59212,'In Earth orbit','','2009-06-26 00:00:00',105.53,730,1309,98.82,'Sun-Synch' from dual union all
select 35518,'035518','1987-011J','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2009-07-07 00:00:00',100.6,778,796,65.01,'Intermediate' from dual union all
select 35519,'035519','1987-068CN','deb S5M ','[UNID]',60158,'In Earth orbit','','2009-07-05 00:00:00',101.48,812,845,82.85,'Intermediate' from dual union all
select 3552,'003552','1968-097E','deb Kosmos-252','',16824,'Reentered','1989-07-06 00:00:00','1989-07-06 00:00:00',88.93,207,237,62.27,'Intermediate' from dual union all
select 35520,'035520','1987-068CP','deb S5M','',60158,'In Earth orbit','','2009-07-10 00:00:00',103.94,939,951,82.55,'Intermediate' from dual union all
select 35521,'035521','1987-068CQ','deb S5M','',60158,'In Earth orbit','','2009-06-27 00:00:00',103.88,936,948,82.56,'Intermediate' from dual union all
select 35522,'035522','1987-068CR','deb S5M','',60158,'In Earth orbit','','2009-07-09 00:00:00',103.95,932,958,82.55,'Intermediate' from dual union all
select 35523,'035523','1987-068CS','deb S5M','',60158,'In Earth orbit','','2009-07-06 00:00:00',103.4,909,930,82.53,'Intermediate' from dual union all
select 35524,'035524','1987-068CT','deb S5M','',60158,'In Earth orbit','','2009-07-09 00:00:00',103.36,892,943,82.54,'Intermediate' from dual union all
select 35525,'035525','1987-068CU','deb S5M','',60158,'In Earth orbit','','2009-07-10 00:00:00',104.19,942,970,82.54,'Intermediate' from dual union all
select 35526,'035526','1988-074P','deb NNS O-25 insulation','',61323,'In Earth orbit','','2009-06-23 00:00:00',106.58,1018,1119,89.9,'Polar' from dual union all
select 35527,'035527','1988-074Q','deb NNS O-25 insulation','',61323,'In Earth orbit','','2009-03-22 00:00:00',104.56,945,1003,89.84,'Polar' from dual union all
select 35528,'035528','1988-074R','deb NNS O-25 insulation','',61323,'In Earth orbit','','2009-06-27 00:00:00',106.61,996,1144,89.88,'Polar' from dual union all
select 35529,'035529','1988-074S','deb NNS O-25 insulation','',61323,'In Earth orbit','','2009-07-08 00:00:00',106.41,986,1135,89.88,'Polar' from dual union all
select 3553,'003553','1968-097F','deb Kosmos-252','',16824,'Reentered','2004-07-23 00:00:00','1992-02-02 00:00:00',104.12,504,1402,62.28,'Intermediate' from dual union all
select 35530,'035530','1991-068AX','deb S5M','',64237,'In Earth orbit','','2009-07-13 00:00:00',115.76,1465,1515,82.71,'Intermediate' from dual union all
select 35531,'035531','1991-082CN','deb DMSP F-11','',64251,'In Earth orbit','','2009-07-05 00:00:00',100.85,772,826,98.55,'Sun-Synch' from dual union all
select 35532,'035532','1991-082CP','deb DMSP F-11','',64251,'In Earth orbit','','2009-07-13 00:00:00',103.61,818,1041,98.76,'Sun-Synch' from dual union all
select 35533,'035533','1975-052LG','deb Delta 111','',36499,'In Earth orbit','','2009-07-11 00:00:00',102.21,846,881,99.76,'Sun-Synch' from dual union all
select 35534,'035534','1991-082CQ','deb DMSP F-11','',64251,'In Earth orbit','','2009-07-12 00:00:00',102.23,816,913,98.7,'Sun-Synch' from dual union all
select 35535,'035535','1975-052LH','deb Delta 111','',36499,'In Earth orbit','','2009-07-08 00:00:00',104.43,899,1036,99,'Sun-Synch' from dual union all
select 35536,'035536','2001-049QL','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-07-04 00:00:00',105.2,678,1330,98.6,'Sun-Synch' from dual union all
select 35537,'035537','2001-049QM','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2009-07-02 00:00:00',100.15,702,830,98.14,'Sun-Synch' from dual union all
select 35538,'035538','2006-006CF','deb Briz-M 88515','',68178,'In Earth orbit','','2009-07-11 00:00:00',287.17,674,15407,51.96,'Highly Ellip' from dual union all
select 35539,'035539','2006-006CG','deb Briz-M 88515','',68178,'Reentered','2010-07-01 00:00:00','2009-07-10 00:00:00',263.27,483,13974,51.72,'Medium' from dual union all
select 3554,'003554','1968-097G','deb Kosmos-252','',16824,'In Earth orbit','','1992-02-03 00:00:00',107.53,524,1701,62.28,'Intermediate' from dual union all
select 35540,'035540','2006-006CH','deb Briz-M 88515','',68178,'In Earth orbit','','2009-07-12 00:00:00',280.04,594,15007,52.24,'Highly Ellip' from dual union all
select 35541,'035541','2006-006CJ','deb Briz-M 88515','',68178,'In Earth orbit','','2009-07-13 00:00:00',281.1,517,15156,51.61,'Highly Ellip' from dual union all
select 35542,'035542','2006-006CK','deb Briz-M 88515','',68178,'In Earth orbit','','2009-07-12 00:00:00',281.36,767,14923,51.66,'Medium' from dual union all
select 35543,'035543','2006-006CL','deb Briz-M 88515','',68178,'In Earth orbit','','2009-07-07 00:00:00',288.99,671,15532,52.8,'Highly Ellip' from dual union all
select 35544,'035544','2006-006CM','deb Briz-M 88515','',68178,'In Earth orbit','','2009-07-12 00:00:00',259.62,369,13836,51.49,'Medium' from dual union all
select 35545,'035545','2006-006CN','deb Briz-M 88515','',68178,'In Earth orbit','','2009-07-09 00:00:00',245.16,355,12839,51.89,'Medium' from dual union all
select 35546,'035546','2006-006CP','deb Briz-M 88515','',68178,'In Earth orbit','','2009-07-13 00:00:00',316.35,937,17066,51.65,'Highly Ellip' from dual union all
select 35547,'035547','2007-003F','deb CZ-3','[UNID150]',68345,'In Earth orbit','','2009-07-12 00:00:00',709.36,249,39690,24.05,'GEO Transfer' from dual union all
select 35548,'035548','2007-003G','deb CZ-3','',68345,'In Earth orbit','','2009-07-07 00:00:00',791.82,693,43254,25.46,'Highly Ellip' from dual union all
select 35549,'035549','2007-003H','deb CZ-3','',68345,'In Earth orbit','','2009-07-13 00:00:00',736.1,304,40951,23.95,'GEO Transfer' from dual union all
select 3555,'003555','1968-097H','deb Kosmos-252','',16824,'In Earth orbit','','1992-02-02 00:00:00',110.72,537,1983,62.32,'Intermediate' from dual union all
select 35550,'035550','2007-003J','deb CZ-3','',68345,'In Earth orbit','','2009-07-09 00:00:00',770.15,425,42483,23.53,'Highly Ellip' from dual union all
select 35551,'035551','2007-003K','deb CZ-3','',68345,'In Earth orbit','','2009-07-12 00:00:00',671.82,570,37493,24.81,'GEO Transfer' from dual union all
select 35552,'035552','2007-003L','deb CZ-3','',68345,'In Earth orbit','','2009-07-13 00:00:00',763.02,472,42092,25.44,'Highly Ellip' from dual union all
select 35553,'035553','2007-003M','deb CZ-3','',68345,'Reentered','2016-01-27 00:00:00','2009-05-29 00:00:00',720.56,283,40209,23.35,'GEO Transfer' from dual union all
select 35554,'035554','2007-003N','deb CZ-3','',68345,'In Earth orbit','','2009-07-12 00:00:00',784.43,663,42931,23.43,'Highly Ellip' from dual union all
select 35555,'035555','2007-003P','deb CZ-3','',68345,'In Earth orbit','','2009-07-12 00:00:00',731.04,394,40613,26.12,'GEO Transfer' from dual union all
select 35556,'035556','2007-003Q','deb CZ-3','',68345,'In Earth orbit','','2009-07-12 00:00:00',756.21,479,41755,25.31,'Highly Ellip' from dual union all
select 35557,'035557','2007-003R','deb CZ-3','',68345,'In Earth orbit','','2009-07-10 00:00:00',737.39,309,41009,22.76,'GEO Transfer' from dual union all
select 35558,'035558','2007-003S','deb CZ-3','',68345,'In Earth orbit','','2009-07-08 00:00:00',762.03,616,41900,22.66,'Highly Ellip' from dual;
insert into SATELLITE
select 35559,'035559','2007-003T','deb CZ-3','',68345,'In Earth orbit','','2009-07-10 00:00:00',719.5,583,39857,23.99,'GEO Transfer' from dual union all
select 3556,'003556','1968-097J','deb Kosmos-252','',16824,'Reentered','1982-11-20 00:00:00','1982-11-19 00:00:00',90.06,254,301,62.27,'Intermediate' from dual union all
select 35560,'035560','2007-003U','deb CZ-3','',68345,'In Earth orbit','','2009-07-12 00:00:00',755.05,385,41793,22.89,'Highly Ellip' from dual union all
select 35561,'035561','2007-003V','deb CZ-3','',68345,'Lost','','2009-07-11 00:00:00',726.7,247,40547,23.99,'GEO Transfer' from dual union all
select 35562,'035562','2007-003W','deb CZ-3','',68345,'In Earth orbit','','2009-07-08 00:00:00',765.2,714,41954,22.48,'Highly Ellip' from dual union all
select 35563,'035563','2007-003X','deb CZ-3','',68345,'In Earth orbit','','2009-07-08 00:00:00',765.24,629,42042,22.01,'Highly Ellip' from dual union all
select 35564,'035564','2007-003Y','deb CZ-3','',68345,'In Earth orbit','','2009-07-11 00:00:00',771.65,569,42411,25.6,'Highly Ellip' from dual union all
select 35565,'035565','2007-003Z','deb CZ-3','',68345,'In Earth orbit','','2009-07-09 00:00:00',768.82,535,42309,25.98,'Highly Ellip' from dual union all
select 35566,'035566','2007-003AA','deb CZ-3','',68345,'In Earth orbit','','2009-07-02 00:00:00',774.18,529,42573,25.79,'Highly Ellip' from dual union all
select 35567,'035567','2007-042G','deb CZ-4B','',68384,'In Earth orbit','','2009-07-13 00:00:00',102.03,844,865,98.77,'Sun-Synch' from dual union all
select 35568,'035568','2007-047D','Star 48 yo weight','',68389,'Reentered','2016-12-01 00:00:00','2009-07-12 00:00:00',343.84,182,19577,39.98,'Highly Ellip' from dual union all
select 35569,'035569','2008-041AB','deb CZ-2C Stage 2','',68549,'Reentered','2009-07-08 00:00:00','2009-05-21 00:00:00',91.15,250,411,97.71,'Sun-Synch' from dual union all
select 3557,'003557','1968-097K','deb Kosmos-252','',16824,'In Earth orbit','','1992-02-03 00:00:00',110.09,534,1928,62.32,'Intermediate' from dual union all
select 35570,'035570','2008-042C','deb GeoEye 1','',68550,'Reentered','2011-10-23 00:00:00','2009-07-10 00:00:00',98.12,661,677,98.16,'Sun-Synch' from dual union all
select 35571,'035571','2008-053G','CZ-4B sep motor cover','',68561,'In Earth orbit','','2009-07-13 00:00:00',99.74,596,897,97.69,'Sun-Synch' from dual union all
select 35572,'035572','2008-053H','CZ-4B sep motor cover','',68561,'Reentered','2014-11-19 00:00:00','2009-07-13 00:00:00',96.93,520,704,97.72,'Sun-Synch' from dual union all
select 35573,'035573','2008-053J','CZ-4B sep motor cover','',68561,'In Earth orbit','','2009-07-13 00:00:00',99.64,592,891,97.72,'Sun-Synch' from dual union all
select 35574,'035574','2008-056F','SW-3 lower debris','',68564,'In Earth orbit','','2009-07-11 00:00:00',100.2,666,870,98.43,'Sun-Synch' from dual union all
select 35575,'035575','2008-061B','CZ-2B sep motor cover','',68569,'In Earth orbit','','2009-07-12 00:00:00',101.58,651,1015,97.77,'Sun-Synch' from dual union all
select 35576,'035576','2008-064F','deb Yaogan','',68572,'Reentered','2010-05-08 00:00:00','2009-05-19 00:00:00',93.9,452,479,97.29,'Sun-Synch' from dual union all
select 35577,'035577','2009-009C','deb Blok DM-SLB','',68690,'In Earth orbit','','2009-07-06 00:00:00',654.15,1152,36016,35.3,'GEO Transfer' from dual union all
select 35578,'035578','2009-037A','RazakSat','MACSat',68718,'In Earth orbit','','2009-07-16 00:00:00',98.29,666,688,8.99,'Equatorial' from dual union all
select 35579,'035579','2009-037B','Falcon 1 F005 Stage 2','Falcon 1 Stage 2',68718,'In Earth orbit','','2009-07-16 00:00:00',97.74,616,686,9.04,'Equatorial' from dual union all
select 3558,'003558','1968-097L','deb Kosmos-252','',16824,'In Earth orbit','','1992-02-02 00:00:00',104.32,540,1386,62.21,'Intermediate' from dual union all
select 35580,'035580','1993-036ALX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',105.21,766,1242,73.91,'Intermediate' from dual union all
select 35581,'035581','1993-036ALY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-13 00:00:00',98.83,619,787,74,'Intermediate' from dual union all
select 35582,'035582','1993-036ALZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',100.43,764,794,74.04,'Intermediate' from dual union all
select 35583,'035583','1993-036AMA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',106.83,772,1387,73.92,'Intermediate' from dual union all
select 35584,'035584','1993-036AMB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',103.09,767,1042,74.1,'Intermediate' from dual union all
select 35585,'035585','1993-036AMC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',105.77,773,1288,73.98,'Intermediate' from dual union all
select 35586,'035586','1993-036AMD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',106.89,771,1395,74.02,'Intermediate' from dual union all
select 35587,'035587','1993-036AME','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',98.56,605,775,74.01,'Intermediate' from dual union all
select 35588,'035588','1993-036AMF','deb Kosmos-2251','',65138,'Reentered','2013-03-29 00:00:00','2009-07-12 00:00:00',97.13,467,776,74.06,'Intermediate' from dual union all
select 35589,'035589','1993-036AMG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-12 00:00:00',98.79,530,872,73.99,'Intermediate' from dual union all
select 3559,'003559','1968-097M','deb Kosmos-252','',16824,'In Earth orbit','','1992-02-03 00:00:00',107.02,514,1663,62.3,'Intermediate' from dual union all
select 35590,'035590','1993-036AMH','deb Kosmos-2251','',65138,'Reentered','2013-02-15 00:00:00','2009-07-14 00:00:00',97.85,542,770,73.9,'Intermediate' from dual union all
select 35591,'035591','1993-036AMJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-13 00:00:00',100.48,740,823,74.02,'Intermediate' from dual union all
select 35592,'035592','1993-036AMK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',98.02,549,778,74.02,'Intermediate' from dual union all
select 35593,'035593','1993-036AML','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',102.08,762,953,74.1,'Intermediate' from dual union all
select 35594,'035594','1993-036AMM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',99.7,695,793,73.82,'Intermediate' from dual union all
select 35595,'035595','1993-036AMN','deb Kosmos-2251','',65138,'Reentered','2011-07-18 00:00:00','2009-07-14 00:00:00',95.8,363,751,73.9,'Intermediate' from dual union all
select 35596,'035596','1993-036AMP','deb Kosmos-2251','',65138,'Reentered','2017-07-06 00:00:00','2009-07-14 00:00:00',100.1,694,832,73.93,'Intermediate' from dual union all
select 35597,'035597','1993-036AMQ','deb Kosmos-2251','',65138,'Reentered','2013-06-08 00:00:00','2009-07-14 00:00:00',97.15,465,779,74.06,'Intermediate' from dual union all
select 35598,'035598','1993-036AMR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',97.95,546,775,73.92,'Intermediate' from dual union all
select 35599,'035599','1993-036AMS','deb Kosmos-2251','',65138,'Reentered','2011-12-15 00:00:00','2009-07-14 00:00:00',97.37,495,771,74.01,'Intermediate' from dual union all
select 356,'000356','61 Omicron 117','deb r/b Ablestar','',7273,'Reentered','1981-02-02 00:00:00','1962-08-01 00:00:00',103.62,861,998,67.04,'Intermediate' from dual union all
select 3560,'003560','1968-097N','deb Kosmos-252','',16824,'Reentered','2001-02-09 00:00:00','1992-02-03 00:00:00',101.89,496,1201,62.27,'Intermediate' from dual union all
select 35600,'035600','1993-036AMT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',100.02,694,825,73.99,'Intermediate' from dual union all
select 35601,'035601','1993-036AMU','deb Kosmos-2251','',65138,'Reentered','2010-10-07 00:00:00','2009-07-12 00:00:00',96.19,405,748,73.96,'Intermediate' from dual union all
select 35602,'035602','1993-036AMV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-13 00:00:00',97.17,471,776,74.03,'Intermediate' from dual union all
select 35603,'035603','1993-036AMW','deb Kosmos-2251','',65138,'Reentered','2016-11-20 00:00:00','2009-07-14 00:00:00',99.21,634,807,74.02,'Intermediate' from dual union all
select 35604,'035604','1993-036AMX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',101.43,770,883,74.03,'Intermediate' from dual union all
select 35605,'035605','1993-036AMY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',100.76,699,891,73.91,'Intermediate' from dual union all
select 35606,'035606','1993-036AMZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',107.9,772,1488,73.81,'Intermediate' from dual union all
select 35607,'035607','1993-036ANA','deb Kosmos-2251','',65138,'Reentered','2011-04-11 00:00:00','2009-07-13 00:00:00',96.26,412,747,74.17,'Intermediate' from dual union all
select 35608,'035608','1993-036ANB','deb Kosmos-2251','',65138,'Reentered','2014-09-16 00:00:00','2009-07-14 00:00:00',99.44,696,768,74.23,'Intermediate' from dual union all
select 35609,'035609','1993-036ANC','deb Kosmos-2251','',65138,'Reentered','2018-02-17 00:00:00','2009-07-14 00:00:00',99.96,741,773,74.04,'Intermediate' from dual union all
select 3561,'003561','1968-097P','deb Kosmos-252','',16824,'In Earth orbit','','1992-02-03 00:00:00',107.42,513,1702,62.29,'Intermediate' from dual union all
select 35610,'035610','1993-036AND','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-12 00:00:00',100.84,760,836,74.03,'Intermediate' from dual union all
select 35611,'035611','1993-036ANE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',100.83,760,836,74.03,'Intermediate' from dual union all
select 35612,'035612','1993-036ANF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',99.12,673,761,74.03,'Intermediate' from dual union all
select 35613,'035613','1993-036ANG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',99.17,664,774,73.99,'Intermediate' from dual union all
select 35614,'035614','1993-036ANH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-14 00:00:00',99.93,673,838,73.96,'Intermediate' from dual union all
select 35615,'035615','1997-051QM','deb Iridium 33','',66131,'In Earth orbit','','2009-07-13 00:00:00',100.18,756,779,86.38,'Polar' from dual union all
select 35616,'035616','1997-051QN','deb Iridium 33','',66131,'In Earth orbit','','2009-07-12 00:00:00',100.11,746,782,86.37,'Polar' from dual union all
select 35617,'035617','1997-051QP','deb Iridium 33','',66131,'In Earth orbit','','2009-07-14 00:00:00',101.44,766,888,86.53,'Polar' from dual union all
select 35618,'035618','1997-051QQ','deb Iridium 33','',66131,'In Earth orbit','','2009-07-14 00:00:00',100.04,734,787,86.34,'Polar' from dual union all
select 35619,'035619','1997-051QR','deb Iridium 33','',66131,'Reentered','2015-04-17 00:00:00','2009-07-13 00:00:00',100.15,755,776,86.34,'Polar' from dual union all
select 3562,'003562','1968-104A','Kosmos-254','Zenit-4 No. 50',16831,'Landed','1968-11-29 00:00:00','1968-11-26 00:00:00',89.72,195,327,65.4,'Intermediate' from dual union all
select 35620,'035620','1997-051QS','deb Iridium 33','',66131,'In Earth orbit','','2009-07-14 00:00:00',104.14,744,1165,86.4,'Polar' from dual union all
select 35621,'035621','1997-051QT','deb Iridium 33','',66131,'Reentered','2015-04-03 00:00:00','2009-07-14 00:00:00',100.5,772,792,86.3,'Polar' from dual union all
select 35622,'035622','1997-051QU','deb Iridium 33','',66131,'In Earth orbit','','2009-07-14 00:00:00',100.7,769,815,86.23,'Polar' from dual union all
select 35623,'035623','1997-051QV','deb Iridium 33','',66131,'Reentered','2016-06-23 00:00:00','2009-07-14 00:00:00',102.3,763,972,86.49,'Polar' from dual union all
select 35624,'035624','1997-051QW','deb Iridium 33','',66131,'In Earth orbit','','2009-07-12 00:00:00',101.69,766,912,86.38,'Polar' from dual union all
select 35625,'035625','1997-051QX','deb Iridium 33','',66131,'In Earth orbit','','2009-07-14 00:00:00',102.54,773,984,86.2,'Polar' from dual union all
select 35626,'035626','1997-051QY','deb Iridium 33','',66131,'Reentered','2014-02-24 00:00:00','2009-07-13 00:00:00',99.66,683,802,86.31,'Polar' from dual union all
select 35627,'035627','1997-051QZ','deb Iridium 33','',66131,'In Earth orbit','','2009-07-14 00:00:00',101.26,775,862,86.37,'Polar' from dual union all
select 35628,'035628','1997-051RA','deb Iridium 33','',66131,'In Earth orbit','','2009-07-13 00:00:00',100.38,760,794,86.37,'Polar' from dual union all
select 35629,'035629','1997-051RB','deb Iridium 33','',66131,'In Earth orbit','','2009-07-14 00:00:00',102.26,771,961,86.39,'Polar' from dual union all
select 3563,'003563','1968-104B','11A57I','11A57I',16831,'Reentered','1968-11-27 00:00:00','1968-11-24 00:00:00',89.32,176,306,65.39,'Intermediate' from dual union all
select 35630,'035630','1997-051RC','deb Iridium 33','',66131,'Reentered','2013-02-08 00:00:00','2009-07-12 00:00:00',100.69,760,822,86.39,'Polar' from dual union all
select 35631,'035631','1997-051RD','deb Iridium 33','',66131,'In Earth orbit','','2009-07-14 00:00:00',100.09,748,777,86.37,'Polar' from dual union all
select 35632,'035632','1997-051RE','deb Iridium 33','',66131,'In Earth orbit','','2009-07-14 00:00:00',102.46,774,977,86.42,'Polar' from dual union all
select 35633,'035633','2009-038A','Endeavour (STS-127)','OV-105',68719,'Landed','2009-07-31 00:00:00','2009-07-18 00:00:00',91.48,344,351,51.64,'Intermediate' from dual union all
select 35634,'035634','1998-067BR','JEM ABM Thermal Cover','',66503,'Reentered','2009-07-26 00:00:00','2009-07-18 00:00:00',91.48,341,354,51.64,'Intermediate' from dual union all
select 35635,'035635','2009-039A','Kosmos-2454','Parus No. 206',68720,'In Earth orbit','','2009-07-22 00:00:00',103.62,917,943,82.96,'Intermediate' from dual union all
select 35636,'035636','2009-039B','Sterkh No. 1','Sterkh No.11L',68720,'In Earth orbit','','2009-07-22 00:00:00',103.63,918,943,82.95,'Intermediate' from dual union all
select 35637,'035637','2009-039C','S3M','S3M',68720,'In Earth orbit','','2009-07-22 00:00:00',103.52,917,933,82.95,'Intermediate' from dual union all
select 35638,'035638','1998-067BS','JEF Thermal Cover','',66503,'Reentered','2009-07-30 00:00:00','2009-07-22 00:00:00',91.41,342,346,51.64,'Intermediate' from dual union all
select 35639,'035639','1998-067BT','JEF Thermal Cover','',66503,'Reentered','2009-08-02 00:00:00','2009-07-22 00:00:00',91.39,340,346,51.64,'Intermediate' from dual union all
select 3564,'003564','1968-097Q','deb Kosmos-252','',16824,'Reentered','1999-01-28 00:00:00','1992-02-03 00:00:00',99.53,501,972,62.24,'Intermediate' from dual union all
select 35640,'035640','1998-067BU','JEF Thermal Cover','',66503,'Reentered','2009-07-31 00:00:00','2009-07-22 00:00:00',91.36,341,342,51.64,'Intermediate' from dual union all
select 35641,'035641','2009-040A','Progress M-67','Progress 7K-TGM No.367',68721,'Deorbited','2009-09-27 00:00:00','2009-07-25 00:00:00',90.15,254,310,51.64,'Intermediate' from dual union all
select 35642,'035642','2009-040B','11S510','11S510',68721,'Reentered','2009-07-28 00:00:00','2009-07-25 00:00:00',88.54,184,221,51.63,'Intermediate' from dual union all
select 35643,'035643','1993-036ANJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',99.88,713,793,74.02,'Intermediate' from dual union all
select 35644,'035644','1993-036ANK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-27 00:00:00',99.37,684,773,74.03,'Intermediate' from dual union all
select 35645,'035645','1993-036ANL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-27 00:00:00',101.85,772,920,73.98,'Intermediate' from dual union all
select 35646,'035646','1993-036ANM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-27 00:00:00',107.97,771,1495,73.87,'Intermediate' from dual union all
select 35647,'035647','1993-036ANN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',100.84,754,842,74.03,'Intermediate' from dual union all
select 35648,'035648','1993-036ANP','deb Kosmos-2251','',65138,'Reentered','2014-05-19 00:00:00','2009-07-28 00:00:00',97.53,499,782,74.09,'Intermediate' from dual;
insert into SATELLITE
select 35649,'035649','1993-036ANQ','deb Kosmos-2251','',65138,'Reentered','2014-05-04 00:00:00','2009-07-27 00:00:00',97.14,449,795,73.97,'Intermediate' from dual union all
select 3565,'003565','1968-097R','deb Kosmos-252','',16824,'In Earth orbit','','1992-02-03 00:00:00',107.15,515,1674,62.31,'Intermediate' from dual union all
select 35650,'035650','1993-036ANR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-27 00:00:00',100.12,737,792,74.06,'Intermediate' from dual union all
select 35651,'035651','1993-036ANS','deb Kosmos-2251','',65138,'Reentered','2013-09-04 00:00:00','2009-07-28 00:00:00',98.34,472,887,74.01,'Intermediate' from dual union all
select 35652,'035652','1993-036ANT','deb Kosmos-2251','',65138,'Reentered','2015-06-24 00:00:00','2009-07-27 00:00:00',100.12,737,792,74.02,'Intermediate' from dual union all
select 35653,'035653','1993-036ANU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',100.01,736,782,74.02,'Intermediate' from dual union all
select 35654,'035654','1993-036ANV','deb Kosmos-2251','',65138,'Reentered','2015-01-10 00:00:00','2009-07-27 00:00:00',97.72,516,784,74.08,'Intermediate' from dual union all
select 35655,'035655','1993-036ANW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-27 00:00:00',99.95,739,774,74.01,'Intermediate' from dual union all
select 35656,'035656','1993-036ANX','deb Kosmos-2251','',65138,'Reentered','2011-04-30 00:00:00','2009-07-28 00:00:00',96.51,427,756,73.95,'Intermediate' from dual union all
select 35657,'035657','1993-036ANY','deb Kosmos-2251','',65138,'Reentered','2018-02-02 00:00:00','2009-07-28 00:00:00',100,736,781,74.02,'Intermediate' from dual union all
select 35658,'035658','1993-036ANZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',100.53,763,804,74.02,'Intermediate' from dual union all
select 35659,'035659','1993-036APA','deb Kosmos-2251','',65138,'Reentered','2012-09-12 00:00:00','2009-07-27 00:00:00',97.81,531,777,74.06,'Intermediate' from dual union all
select 3566,'003566','1968-097S','deb Kosmos-252','',16824,'Reentered','2000-06-18 00:00:00','1992-02-03 00:00:00',101.26,490,1147,62.29,'Intermediate' from dual union all
select 35660,'035660','1993-036APB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-27 00:00:00',100.07,747,777,74.04,'Intermediate' from dual union all
select 35661,'035661','1993-036APC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',98.99,603,817,74.02,'Intermediate' from dual union all
select 35662,'035662','1993-036APD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',99.61,695,784,74.05,'Intermediate' from dual union all
select 35663,'035663','1993-036APE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',100.47,769,793,74.03,'Intermediate' from dual union all
select 35664,'035664','1993-036APF','deb Kosmos-2251','',65138,'Reentered','2017-07-28 00:00:00','2009-07-27 00:00:00',98.96,633,784,73.87,'Intermediate' from dual union all
select 35665,'035665','1993-036APG','deb Kosmos-2251','',65138,'Reentered','2011-08-08 00:00:00','2009-07-28 00:00:00',96.4,413,760,74.1,'Intermediate' from dual union all
select 35666,'035666','1993-036APH','deb Kosmos-2251','',65138,'Reentered','2014-03-06 00:00:00','2009-07-28 00:00:00',98.13,563,776,73.9,'Intermediate' from dual union all
select 35667,'035667','1993-036APJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',100.31,749,798,74.06,'Intermediate' from dual union all
select 35668,'035668','1993-036APK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',99.03,643,782,74.05,'Intermediate' from dual union all
select 35669,'035669','1993-036APL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',106.74,767,1385,73.87,'Intermediate' from dual union all
select 3567,'003567','1968-097T','deb Kosmos-252','',16824,'Reentered','1982-03-02 00:00:00','1982-03-01 00:00:00',91.49,302,394,62.26,'Intermediate' from dual union all
select 35670,'035670','1993-036APM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',99.33,672,782,73.91,'Intermediate' from dual union all
select 35671,'035671','1993-036APN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',99.81,718,781,74.02,'Intermediate' from dual union all
select 35672,'035672','1993-036APP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',99.83,704,797,74.06,'Intermediate' from dual union all
select 35673,'035673','1993-036APQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',99.54,702,772,73.98,'Intermediate' from dual union all
select 35674,'035674','1993-036APR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',101.09,703,918,74.07,'Intermediate' from dual union all
select 35675,'035675','1993-036APS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-28 00:00:00',101.3,770,871,74.02,'Intermediate' from dual union all
select 35676,'035676','1993-036APT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-07-27 00:00:00',100.09,729,797,74.02,'Intermediate' from dual union all
select 35677,'035677','1997-051RF','deb Iridium 33','',66131,'Reentered','2011-10-24 00:00:00','2009-07-27 00:00:00',98.39,588,775,86.11,'Polar' from dual union all
select 35678,'035678','1997-051RG','deb Iridium 33','',66131,'In Earth orbit','','2009-07-27 00:00:00',100.46,769,791,86.37,'Polar' from dual union all
select 35679,'035679','1997-051RH','deb Iridium 33','',66131,'In Earth orbit','','2009-07-28 00:00:00',101.23,771,863,86.35,'Polar' from dual union all
select 3568,'003568','1968-097U','deb Kosmos-252','',16824,'Reentered','1979-04-19 00:00:00','1979-03-27 00:00:00',95.58,452,642,62.32,'Intermediate' from dual union all
select 35680,'035680','1997-051RJ','deb Iridium 33','',66131,'In Earth orbit','','2009-07-27 00:00:00',103.7,778,1089,86.12,'Polar' from dual union all
select 35681,'035681','2009-041A','Deimos-1','Deimos-1 DMC',68722,'In Earth orbit','','2009-07-31 00:00:00',97.84,634,677,98.13,'Sun-Synch' from dual union all
select 35682,'035682','2009-041B','DubaiSat-1','DubaiSat-1',68722,'In Earth orbit','','2009-07-31 00:00:00',98.21,666,681,98.13,'Sun-Synch' from dual union all
select 35683,'035683','2009-041C','UK-DMC-2','UK-DMC-2',68722,'In Earth orbit','','2009-07-31 00:00:00',97.72,624,676,98.13,'Sun-Synch' from dual union all
select 35684,'035684','2009-041D','AprizeSat-4','',68722,'In Earth orbit','','2009-07-31 00:00:00',97.54,606,675,98.13,'Sun-Synch' from dual union all
select 35685,'035685','2009-041E','Nanosat 1B','Nanosat',68722,'In Earth orbit','','2009-07-31 00:00:00',97.34,586,677,98.13,'Sun-Synch' from dual union all
select 35686,'035686','2009-041F','AprizeSat-3','',68722,'In Earth orbit','','2009-07-31 00:00:00',97.12,565,676,98.13,'Sun-Synch' from dual union all
select 35687,'035687','2009-041G','Gazodinamicheskiy Ekran','GDS (Gas dynamic shield)',68722,'Reentered','2012-11-20 00:00:00','2009-07-31 00:00:00',96.87,541,677,98.12,'Sun-Synch' from dual union all
select 35688,'035688','2009-041H','R-36M2 DS','R-36M2 DS',68722,'In Earth orbit','','2009-07-31 00:00:00',104.79,668,1302,98.13,'Sun-Synch' from dual union all
select 35689,'035689','2009-041J','Dnepr Platform A/Fairing','',68722,'In Earth orbit','','2009-07-31 00:00:00',97.92,642,676,98.13,'Sun-Synch' from dual union all
select 3569,'003569','1968-097V','deb Kosmos-252','',16824,'In Earth orbit','','1992-02-03 00:00:00',107.94,518,1745,62.32,'Intermediate' from dual union all
select 35690,'035690','2009-038B','DRAGONSAT AggieSat 2','AggieSat 2',68719,'Reentered','2010-03-17 00:00:00','2009-07-30 00:00:00',91.1,325,332,51.64,'Intermediate' from dual union all
select 35691,'035691','2009-038C','ICU Cylinder 2','Internal Cargo Unit',68719,'Reentered','2010-01-10 00:00:00','2009-07-31 00:00:00',91.12,328,331,51.64,'Intermediate' from dual union all
select 35692,'035692','2009-038D','ICU Cylinder 1','',68719,'Reentered','2009-11-21 00:00:00','2009-07-31 00:00:00',91.12,327,332,51.64,'Intermediate' from dual union all
select 35693,'035693','2009-038E','ANDE-PS','ANDE Passive Sphere',68719,'Reentered','2010-03-29 00:00:00','2009-07-31 00:00:00',91.13,328,332,51.64,'Intermediate' from dual union all
select 35694,'035694','2009-038F','ANDE-AS','ANDE Active Sphere',68719,'Reentered','2010-08-18 00:00:00','2009-07-31 00:00:00',91.14,328,333,51.64,'Intermediate' from dual union all
select 35695,'035695','2009-038G','ICU Avionics Deck','',68719,'Reentered','2010-02-19 00:00:00','2009-07-31 00:00:00',91.14,328,333,51.64,'Intermediate' from dual union all
select 35696,'035696','2009-042A','Asiasat 5','Asiasat 5',68723,'In Earth orbit','','2009-08-19 00:00:00',1435.37,35676,35869,.08,'Drift GEO' from dual union all
select 35697,'035697','2009-042B','Briz-M No. 99507','Briz-M',68723,'In Earth orbit','','2009-09-01 00:00:00',899.98,14668,34332,6.16,'Medium' from dual union all
select 35698,'035698','2009-042C','Briz-M DTB','Briz-M torus tank',68723,'Reentered','2010-06-22 00:00:00','2009-08-19 00:00:00',634.03,392,35747,49.18,'GEO Transfer' from dual union all
select 35699,'035699','1993-036APU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-12 00:00:00',98.5,596,778,74.02,'Intermediate' from dual union all
select 357,'000357','61 Omicron 118','deb r/b Ablestar','',7273,'In Earth orbit','','1962-07-30 00:00:00',104.78,975,994,66.85,'Intermediate' from dual union all
select 3570,'003570','1968-097W','deb Kosmos-252','',16824,'Reentered','1982-04-10 00:00:00','1982-04-04 00:00:00',92.57,355,446,62.35,'Intermediate' from dual union all
select 35700,'035700','1993-036APV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',101.23,762,872,74.02,'Intermediate' from dual union all
select 35701,'035701','1993-036APW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-12 00:00:00',105.52,772,1266,74.02,'Intermediate' from dual union all
select 35702,'035702','1993-036APX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',100.17,740,794,74.05,'Intermediate' from dual union all
select 35703,'035703','1993-036APY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',98.77,615,784,74.05,'Intermediate' from dual union all
select 35704,'035704','1993-036APZ','deb Kosmos-2251','',65138,'Reentered','2014-04-21 00:00:00','2009-08-13 00:00:00',98.09,563,772,73.94,'Intermediate' from dual union all
select 35705,'035705','1993-036AQA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',101.43,772,881,74.31,'Intermediate' from dual union all
select 35706,'035706','1993-036AQB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',106.31,773,1339,73.89,'Intermediate' from dual union all
select 35707,'035707','1993-036AQC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-12 00:00:00',100.71,756,829,74.03,'Intermediate' from dual union all
select 35708,'035708','1993-036AQD','deb Kosmos-2251','',65138,'Reentered','2012-08-15 00:00:00','2009-08-13 00:00:00',97.4,492,777,73.57,'Intermediate' from dual union all
select 35709,'035709','1993-036AQE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-12 00:00:00',101.34,767,878,74.01,'Intermediate' from dual union all
select 3571,'003571','1968-090B','deb Kosmos-248','',16817,'Reentered','1968-12-28 00:00:00','1968-11-27 00:00:00',93.79,437,483,62.22,'Intermediate' from dual union all
select 35710,'035710','1993-036AQF','deb Kosmos-2251','',65138,'Reentered','2015-05-01 00:00:00','2009-08-13 00:00:00',100.09,742,784,74.04,'Intermediate' from dual union all
select 35711,'035711','1993-036AQG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',99.01,641,781,74.05,'Intermediate' from dual union all
select 35712,'035712','1993-036AQH','deb Kosmos-2251','',65138,'Reentered','2012-10-11 00:00:00','2009-08-12 00:00:00',97.13,465,777,73.88,'Intermediate' from dual union all
select 35713,'035713','1993-036AQJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',99.2,660,781,74.04,'Intermediate' from dual union all
select 35714,'035714','1993-036AQK','deb Kosmos-2251','',65138,'Reentered','2011-11-16 00:00:00','2009-08-13 00:00:00',97.5,408,871,74,'Intermediate' from dual union all
select 35715,'035715','1993-036AQL','deb Kosmos-2251','',65138,'Reentered','2015-01-12 00:00:00','2009-08-12 00:00:00',98.33,578,780,74,'Intermediate' from dual union all
select 35716,'035716','1993-036AQM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',98.96,640,777,73.99,'Intermediate' from dual union all
select 35717,'035717','1993-036AQN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',101.75,770,912,74.17,'Intermediate' from dual union all
select 35718,'035718','1993-036AQP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',100.49,765,799,74.04,'Intermediate' from dual union all
select 35719,'035719','1993-036AQQ','deb Kosmos-2251','',65138,'Reentered','2013-11-26 00:00:00','2009-08-13 00:00:00',99.38,583,875,74.13,'Intermediate' from dual union all
select 3572,'003572','1968-090C','deb Kosmos-248','',16817,'Reentered','1969-02-20 00:00:00','1968-11-29 00:00:00',94.19,444,515,62.24,'Intermediate' from dual union all
select 35720,'035720','1993-036AQR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',101.31,723,918,74,'Intermediate' from dual union all
select 35721,'035721','1993-036AQS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',102.23,722,1006,73.96,'Intermediate' from dual union all
select 35722,'035722','1993-036AQT','deb Kosmos-2251','',65138,'Reentered','2014-10-15 00:00:00','2009-08-13 00:00:00',97.98,536,789,74.13,'Intermediate' from dual union all
select 35723,'035723','1993-036AQU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-12 00:00:00',100.9,760,843,74.02,'Intermediate' from dual union all
select 35724,'035724','1993-036AQV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',99.32,677,776,74.03,'Intermediate' from dual union all
select 35725,'035725','1993-036AQW','deb Kosmos-2251','',65138,'Reentered','2011-12-11 00:00:00','2009-08-13 00:00:00',97.9,533,784,73.9,'Intermediate' from dual union all
select 35726,'035726','1993-036AQX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',99.58,700,777,74.06,'Intermediate' from dual union all
select 35727,'035727','1993-036AQY','deb Kosmos-2251','',65138,'Reentered','2015-11-22 00:00:00','2009-08-13 00:00:00',98.08,557,777,73.97,'Intermediate' from dual union all
select 35728,'035728','1993-036AQZ','deb Kosmos-2251','',65138,'Reentered','2012-10-11 00:00:00','2009-08-13 00:00:00',97.03,457,776,74.01,'Intermediate' from dual union all
select 35729,'035729','1993-036ARA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-13 00:00:00',99.7,715,773,74.04,'Intermediate' from dual union all
select 3573,'003573','1968-090D','deb Kosmos-248','',16817,'Reentered','1969-02-06 00:00:00','1968-11-28 00:00:00',94.3,451,519,62.28,'Intermediate' from dual union all
select 35730,'035730','1993-036ARB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-12 00:00:00',99.22,671,772,74,'Intermediate' from dual union all
select 35731,'035731','1993-036ARC','deb Kosmos-2251','',65138,'Reentered','2011-12-26 00:00:00','2009-08-12 00:00:00',97.49,508,770,74.01,'Intermediate' from dual union all
select 35732,'035732','1997-051RK','deb Iridium 33','',66131,'In Earth orbit','','2009-08-13 00:00:00',101.42,775,877,86.25,'Polar' from dual union all
select 35733,'035733','1997-051RL','deb Iridium 33','',66131,'Reentered','2015-04-07 00:00:00','2009-08-13 00:00:00',100.43,762,796,86.4,'Polar' from dual union all
select 35734,'035734','1997-051RM','deb Iridium 33','',66131,'Reentered','2014-12-06 00:00:00','2009-08-12 00:00:00',99.24,669,775,86.42,'Polar' from dual union all
select 35735,'035735','1997-051RN','deb Iridium 33','',66131,'In Earth orbit','','2009-08-13 00:00:00',99.87,727,778,86.4,'Polar' from dual union all
select 35736,'035736','1997-051RP','deb Iridium 33','',66131,'Reentered','2012-07-18 00:00:00','2009-08-12 00:00:00',98.13,506,833,86.45,'Polar' from dual union all
select 35737,'035737','1997-051RQ','deb Iridium 33','',66131,'In Earth orbit','','2009-08-12 00:00:00',101.8,775,913,86.44,'Polar' from dual union all
select 35738,'035738','1997-051RR','deb Iridium 33','',66131,'Reentered','2011-11-28 00:00:00','2009-08-12 00:00:00',97.78,475,830,86.44,'Polar' from dual;
insert into SATELLITE
select 35739,'035739','1997-051RS','deb Iridium 33','',66131,'In Earth orbit','','2009-08-13 00:00:00',101.87,772,923,86.41,'Polar' from dual union all
select 3574,'003574','1968-105A','Kosmos-255','Zenit-2 No. 68',16832,'Landed','1968-12-07 00:00:00','1968-12-03 00:00:00',89.57,195,312,65.42,'Intermediate' from dual union all
select 35740,'035740','1997-051RT','deb Iridium 33','',66131,'Reentered','2011-12-10 00:00:00','2009-08-12 00:00:00',99.9,738,770,86.4,'Polar' from dual union all
select 35741,'035741','1997-051RU','deb Iridium 33','',66131,'Reentered','2015-09-09 00:00:00','2009-08-12 00:00:00',100.9,778,825,86.38,'Polar' from dual union all
select 35742,'035742','1997-051RV','deb Iridium 33','',66131,'In Earth orbit','','2009-08-12 00:00:00',101.34,778,867,86.45,'Polar' from dual union all
select 35743,'035743','1997-051RW','deb Iridium 33','',66131,'In Earth orbit','','2009-08-12 00:00:00',101.26,778,858,86.39,'Polar' from dual union all
select 35744,'035744','1997-051RX','deb Iridium 33','',66131,'In Earth orbit','','2009-08-13 00:00:00',99.96,731,783,86.33,'Polar' from dual union all
select 35745,'035745','1997-051RY','deb Iridium 33','',66131,'In Earth orbit','','2009-08-13 00:00:00',102.54,773,984,86.39,'Polar' from dual union all
select 35746,'035746','1997-051RZ','deb Iridium 33','',66131,'Reentered','2011-03-13 00:00:00','2009-08-12 00:00:00',102.44,645,1104,86.36,'Polar' from dual union all
select 35747,'035747','1997-051SA','deb Iridium 33','',66131,'In Earth orbit','','2009-08-13 00:00:00',103.44,781,1062,86.14,'Polar' from dual union all
select 35748,'035748','1997-051SB','deb Iridium 33','',66131,'Reentered','2017-12-31 00:00:00','2009-08-13 00:00:00',100.51,775,791,86.41,'Polar' from dual union all
select 35749,'035749','1997-051SC','deb Iridium 33','',66131,'In Earth orbit','','2009-08-13 00:00:00',100.54,775,794,86.38,'Polar' from dual union all
select 3575,'003575','1968-105B','11A57I','11A57I',16832,'Reentered','1968-12-05 00:00:00','1968-12-01 00:00:00',89.3,184,295,65.4,'Intermediate' from dual union all
select 35750,'035750','1997-051SD','deb Iridium 33','',66131,'Reentered','2016-02-17 00:00:00','2009-08-13 00:00:00',100.38,765,788,86.12,'Polar' from dual union all
select 35751,'035751','1997-051SE','deb Iridium 33','',66131,'Reentered','2012-10-02 00:00:00','2009-08-13 00:00:00',100.47,737,825,86.24,'Polar' from dual union all
select 35752,'035752','2009-043A','GPS IIR-21(M)','Navstar GPS SVN 50',68724,'In Earth orbit','','2013-01-09 00:00:00',718,20101,20265,54.4,'Medium' from dual union all
select 35753,'035753','2009-043B','Delta 343','Delta SSPS AJ-10-118K',68724,'Reentered','2009-11-26 00:00:00','2009-08-24 00:00:00',98.4,176,1189,34.27,'Intermediate' from dual union all
select 35754,'035754','2009-043C','Star 48B','Star 48B',68724,'In Earth orbit','','2009-08-19 00:00:00',356.31,199,20342,40,'Highly Ellip' from dual union all
select 35755,'035755','2009-044A','JCSAT RA','JCSAT 12',68725,'In Earth orbit','','2009-10-30 00:00:00',1436.09,35764,35809,.1,'Stationary' from dual union all
select 35756,'035756','2009-044B','Optus D3','Optus D3',68725,'In Earth orbit','','2009-09-17 00:00:00',1436.09,35773,35799,0,'Stationary' from dual union all
select 35757,'035757','2009-044C','ESC-A 548','Ariane 5 ESC-A',68725,'In Earth orbit','','2009-08-24 00:00:00',630.19,261,35680,1.49,'GEO Transfer' from dual union all
select 35758,'035758','2009-044D','Sylda 5A','Sylda 5 No. 29-A',68725,'In Earth orbit','','2009-08-24 00:00:00',631.72,270,35750,1.96,'GEO Transfer' from dual union all
select 35759,'035759','1993-036ARD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-26 00:00:00',99.04,647,779,74.04,'Intermediate' from dual union all
select 3576,'003576','1968-106A','Kosmos-256','Sfera',16833,'In Earth orbit','','1968-12-31 00:00:00',109.4,1173,1225,74.05,'Intermediate' from dual union all
select 35760,'035760','1993-036ARE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-26 00:00:00',99.23,669,775,74,'Intermediate' from dual union all
select 35761,'035761','1993-036ARF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-27 00:00:00',100,732,784,74.03,'Intermediate' from dual union all
select 35762,'035762','1993-036ARG','deb Kosmos-2251','',65138,'Reentered','2012-11-05 00:00:00','2009-08-26 00:00:00',98.15,572,769,74.01,'Intermediate' from dual union all
select 35763,'035763','1993-036ARH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-26 00:00:00',100.85,719,879,73.87,'Intermediate' from dual union all
select 35764,'035764','1993-036ARJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-26 00:00:00',99.92,723,787,74.04,'Intermediate' from dual union all
select 35765,'035765','1993-036ARK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-27 00:00:00',101.03,659,956,74.12,'Intermediate' from dual union all
select 35766,'035766','1993-036ARL','deb Kosmos-2251','',65138,'Reentered','2013-05-10 00:00:00','2009-08-27 00:00:00',98.42,585,781,73.95,'Intermediate' from dual union all
select 35767,'035767','1993-036ARM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-26 00:00:00',101.3,765,876,74,'Intermediate' from dual union all
select 35768,'035768','1993-036ARN','deb Kosmos-2251','',65138,'Reentered','2014-10-12 00:00:00','2009-08-26 00:00:00',100.38,756,797,74.03,'Intermediate' from dual union all
select 35769,'035769','1993-036ARP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-26 00:00:00',100.78,726,865,74.08,'Intermediate' from dual union all
select 3577,'003577','1968-106B','S3M','S3M',16833,'In Earth orbit','','1968-12-30 00:00:00',109.25,1165,1220,74.06,'Intermediate' from dual union all
select 35770,'035770','1993-036ARQ','deb Kosmos-2251','',65138,'Reentered','2014-10-04 00:00:00','2009-08-27 00:00:00',100,701,816,74.22,'Intermediate' from dual union all
select 35771,'035771','1993-036ARR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-27 00:00:00',105.9,758,1314,74.18,'Intermediate' from dual union all
select 35772,'035772','1993-036ARS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-27 00:00:00',100.63,698,879,73.99,'Intermediate' from dual union all
select 35773,'035773','1993-036ART','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-26 00:00:00',99.71,700,789,74.01,'Intermediate' from dual union all
select 35774,'035774','1993-036ARU','deb Kosmos-2251','',65138,'Reentered','2011-06-06 00:00:00','2009-08-26 00:00:00',96.44,426,750,73.93,'Intermediate' from dual union all
select 35775,'035775','1993-036ARV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-27 00:00:00',99.65,711,773,74.04,'Intermediate' from dual union all
select 35776,'035776','1993-036ARW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-27 00:00:00',102.97,566,1233,74.02,'Intermediate' from dual union all
select 35777,'035777','1993-036ARX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-27 00:00:00',107.71,774,1468,73.77,'Intermediate' from dual union all
select 35778,'035778','1993-036ARY','deb Kosmos-2251','',65138,'Reentered','2014-12-07 00:00:00','2009-08-27 00:00:00',99.93,742,769,74.04,'Intermediate' from dual union all
select 35779,'035779','1993-036ARZ','deb Kosmos-2251','',65138,'Reentered','2014-04-02 00:00:00','2009-08-27 00:00:00',98.2,576,770,74.01,'Intermediate' from dual union all
select 3578,'003578','1968-107A','Kosmos-257','DS-P1-Yu No. 17',16834,'Reentered','1969-03-05 00:00:00','1969-01-05 00:00:00',91.1,261,396,70.96,'Intermediate' from dual union all
select 35780,'035780','1993-036ASA','deb Kosmos-2251','',65138,'Reentered','2015-03-16 00:00:00','2009-08-26 00:00:00',97.07,464,773,73.89,'Intermediate' from dual union all
select 35781,'035781','1993-036ASB','deb Kosmos-2251','',65138,'Reentered','2015-06-14 00:00:00','2009-08-27 00:00:00',99.86,710,794,74.04,'Intermediate' from dual union all
select 35782,'035782','1993-036ASC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-27 00:00:00',99.52,698,773,73.99,'Intermediate' from dual union all
select 35783,'035783','1993-036ASD','deb Kosmos-2251','',65138,'Reentered','2010-11-27 00:00:00','2009-08-26 00:00:00',96.43,420,756,73.95,'Intermediate' from dual union all
select 35784,'035784','1993-036ASE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-27 00:00:00',99.85,718,785,74.04,'Intermediate' from dual union all
select 35785,'035785','1993-036ASF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-26 00:00:00',101.24,751,884,73.96,'Intermediate' from dual union all
select 35786,'035786','1993-036ASG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-27 00:00:00',100.08,722,803,74.03,'Intermediate' from dual union all
select 35787,'035787','1993-036ASH','deb Kosmos-2251','',65138,'Reentered','2014-11-08 00:00:00','2009-08-27 00:00:00',100.51,767,799,74.04,'Intermediate' from dual union all
select 35788,'035788','1993-036ASJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-27 00:00:00',100.22,747,791,74.02,'Intermediate' from dual union all
select 35789,'035789','1993-036ASK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-27 00:00:00',100.46,745,816,74.03,'Intermediate' from dual union all
select 3579,'003579','1968-107B','S1M','S1M',16834,'Reentered','1969-01-16 00:00:00','1968-12-10 00:00:00',91.23,269,401,70.94,'Intermediate' from dual union all
select 35790,'035790','1993-036ASL','deb Kosmos-2251','',65138,'Reentered','2014-01-16 00:00:00','2009-08-26 00:00:00',98.45,473,896,74,'Intermediate' from dual union all
select 35791,'035791','1993-036ASM','deb Kosmos-2251','',65138,'Reentered','2014-08-26 00:00:00','2009-08-27 00:00:00',99.37,607,849,73.99,'Intermediate' from dual union all
select 35792,'035792','1993-036ASN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-27 00:00:00',101.39,767,882,73.99,'Intermediate' from dual union all
select 35793,'035793','1993-036ASP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-27 00:00:00',100.26,760,782,74.05,'Intermediate' from dual union all
select 35794,'035794','1993-036ASQ','deb Kosmos-2251','',65138,'Reentered','2015-09-30 00:00:00','2009-08-27 00:00:00',99.5,545,924,74.08,'Intermediate' from dual union all
select 35795,'035795','1993-036ASR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-26 00:00:00',99.68,714,773,74.02,'Intermediate' from dual union all
select 35796,'035796','1993-036ASS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-08-26 00:00:00',100.89,760,842,74.01,'Intermediate' from dual union all
select 35797,'035797','1997-051SF','deb Iridium 33','',66131,'In Earth orbit','','2009-08-26 00:00:00',100.08,744,781,86.35,'Polar' from dual union all
select 35798,'035798','1997-051SG','deb Iridium 33','',66131,'Reentered','2011-07-04 00:00:00','2009-08-27 00:00:00',100.78,745,847,86.46,'Polar' from dual union all
select 35799,'035799','1997-051SH','deb Iridium 33','',66131,'In Earth orbit','','2009-08-26 00:00:00',101.12,738,886,85.96,'Polar' from dual union all
select 358,'000358','61 Omicron 119','deb r/b Ablestar','',7273,'Reentered','1966-08-31 00:00:00','1962-07-29 00:00:00',97.93,411,909,66.38,'Intermediate' from dual union all
select 3580,'003580','1968-090E','deb Kosmos-248','',16817,'Reentered','1968-12-13 00:00:00','1968-12-01 00:00:00',92.31,382,394,62.16,'Intermediate' from dual union all
select 35800,'035800','1997-051SJ','deb Iridium 33','',66131,'In Earth orbit','','2009-08-26 00:00:00',99.76,700,794,86.29,'Polar' from dual union all
select 35801,'035801','1997-051SK','deb Iridium 33','',66131,'Reentered','2012-09-23 00:00:00','2009-08-26 00:00:00',98.79,604,798,86.32,'Polar' from dual union all
select 35802,'035802','1997-051SL','deb Iridium 33','',66131,'In Earth orbit','','2009-08-27 00:00:00',101.99,777,929,86.47,'Polar' from dual union all
select 35803,'035803','1997-051SM','deb Iridium 33','',66131,'Reentered','2015-04-23 00:00:00','2009-08-27 00:00:00',100.78,775,817,86.26,'Polar' from dual union all
select 35804,'035804','1997-051SN','deb Iridium 33','',66131,'Reentered','2013-12-18 00:00:00','2009-08-26 00:00:00',99.84,726,776,86.36,'Polar' from dual union all
select 35805,'035805','1997-051SP','deb Iridium 33','',66131,'In Earth orbit','','2009-08-26 00:00:00',102.51,779,976,86.34,'Polar' from dual union all
select 35806,'035806','1997-051SQ','deb Iridium 33','',66131,'In Earth orbit','','2009-08-27 00:00:00',100.67,777,804,86.35,'Polar' from dual union all
select 35807,'035807','1997-051SR','deb Iridium 33','',66131,'Reentered','2014-08-30 00:00:00','2009-08-27 00:00:00',101.17,778,851,86.43,'Polar' from dual union all
select 35808,'035808','1997-051SS','deb Iridium 33','',66131,'In Earth orbit','','2009-08-26 00:00:00',100.65,773,806,86.4,'Polar' from dual union all
select 35809,'035809','1997-051ST','deb Iridium 33','',66131,'In Earth orbit','','2009-08-26 00:00:00',100.24,752,787,86.34,'Polar' from dual union all
select 3581,'003581','1968-091P','deb Kosmos-249','',16818,'Reentered','1982-09-07 00:00:00','1968-12-21 00:00:00',114.66,688,2192,62.24,'Intermediate' from dual union all
select 35810,'035810','1991-025AQ','deb SOZ','SOZ',64194,'Reentered','2009-07-28 00:00:00','2009-07-20 00:00:00',331.89,1121,17881,66.17,'Highly Ellip' from dual union all
select 35811,'035811','2009-045A','Discovery (STS-128)','OV-103',68726,'Landed','2009-09-12 00:00:00','2009-08-29 00:00:00',89.33,219,264,51.66,'Intermediate' from dual union all
select 35812,'035812','2009-046A','Palapa D','Palapa D',68727,'In Earth orbit','','2009-11-07 00:00:00',1436.14,35785,35790,.07,'Stationary' from dual union all
select 35813,'035813','2009-046B','CZ-3B Stage 3','CZ-3B Stage 3',68727,'Reentered','2009-09-02 00:00:00','2009-09-01 00:00:00',370.2,158,21243,22.32,'Highly Ellip' from dual union all
select 35814,'035814','1998-067BV','STS-128 EVA debris','',66503,'Reentered','2009-10-29 00:00:00','2009-09-08 00:00:00',91.45,337,355,51.64,'Intermediate' from dual union all
select 35815,'035815','2009-047A','NEMESIS 1','P360/PAN',68728,'In Earth orbit','','2009-09-22 00:00:00',1436.09,35782,35791,.06,'Stationary' from dual union all
select 35816,'035816','2009-047B','Centaur AV-018','Centaur AV-018',68728,'In Earth orbit','','2009-09-08 00:00:00',898.76,13544,35400,23.06,'Medium' from dual union all
select 35817,'035817','2009-048A','HTV 1','HTV 1 Gijutso jissho-ki',68729,'Deorbited','2009-11-01 00:00:00','2009-09-12 00:00:00',89.6,210,299,51.66,'Intermediate' from dual union all
select 35818,'035818','2009-048B','H-2B F1 Stage 2','H-2B Stage 2',68729,'Reentered','2009-09-14 00:00:00','2009-09-12 00:00:00',88.44,153,242,51.59,'Intermediate' from dual union all
select 35819,'035819','2009-048C','HTV adapter panel','',68729,'Reentered','2009-09-13 00:00:00','2009-09-11 00:00:00',89.03,172,282,51.63,'Intermediate' from dual union all
select 3582,'003582','1968-091Q','deb Kosmos-249','',16818,'Reentered','1980-03-28 00:00:00','1980-03-22 00:00:00',90.78,267,359,62.36,'Intermediate' from dual union all
select 35820,'035820','2009-048D','HTV adapter panel','',68729,'Reentered','2009-09-15 00:00:00','2009-09-11 00:00:00',89.26,170,306,51.64,'Intermediate' from dual union all
select 35821,'035821','1993-036AST','deb Kosmos-2251','',65138,'Reentered','2016-03-19 00:00:00','2009-09-13 00:00:00',98.9,628,785,73.98,'Intermediate' from dual union all
select 35822,'035822','1993-036ASU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-14 00:00:00',100.62,757,819,74.04,'Intermediate' from dual union all
select 35823,'035823','1993-036ASV','deb Kosmos-2251','',65138,'Reentered','2011-09-24 00:00:00','2009-09-13 00:00:00',96.78,426,783,73.95,'Intermediate' from dual union all
select 35824,'035824','1993-036ASW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-13 00:00:00',104.78,770,1199,73.85,'Intermediate' from dual union all
select 35825,'035825','1993-036ASX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-13 00:00:00',106.11,755,1338,74.11,'Intermediate' from dual union all
select 35826,'035826','1993-036ASY','deb Kosmos-2251','',65138,'Reentered','2010-09-08 00:00:00','2009-09-13 00:00:00',94.7,312,697,73.98,'Intermediate' from dual union all
select 35827,'035827','1993-036ASZ','deb Kosmos-2251','',65138,'Reentered','2014-05-23 00:00:00','2009-09-13 00:00:00',99.06,594,834,74.13,'Intermediate' from dual union all
select 35828,'035828','1993-036ATA','deb Kosmos-2251','',65138,'Reentered','2014-03-08 00:00:00','2009-09-13 00:00:00',97.92,538,781,73.84,'Intermediate' from dual;
insert into SATELLITE
select 35829,'035829','1993-036ATB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-14 00:00:00',100.45,766,794,74.05,'Intermediate' from dual union all
select 3583,'003583','1968-091R','deb Kosmos-249','',16818,'In Earth orbit','','1992-02-02 00:00:00',113.02,808,1923,62.11,'Intermediate' from dual union all
select 35830,'035830','1993-036ATC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-13 00:00:00',102.66,767,1001,73.9,'Intermediate' from dual union all
select 35831,'035831','1993-036ATD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-13 00:00:00',101.07,769,849,74.02,'Intermediate' from dual union all
select 35832,'035832','1993-036ATE','deb Kosmos-2251','',65138,'Reentered','2013-07-31 00:00:00','2009-09-13 00:00:00',98.34,564,795,73.95,'Intermediate' from dual union all
select 35833,'035833','1993-036ATF','deb Kosmos-2251','',65138,'Reentered','2014-12-28 00:00:00','2009-09-13 00:00:00',100.32,760,787,74.04,'Intermediate' from dual union all
select 35834,'035834','1993-036ATG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-14 00:00:00',102.09,775,941,74.04,'Intermediate' from dual union all
select 35835,'035835','1993-036ATH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-13 00:00:00',105.16,757,1247,74.07,'Intermediate' from dual union all
select 35836,'035836','1993-036ATJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-13 00:00:00',100.74,754,833,74.04,'Intermediate' from dual union all
select 35837,'035837','1993-036ATK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-13 00:00:00',107.98,741,1526,73.72,'Intermediate' from dual union all
select 35838,'035838','1993-036ATL','deb Kosmos-2251','',65138,'Reentered','2012-09-28 00:00:00','2009-09-14 00:00:00',96.94,458,766,73.93,'Intermediate' from dual union all
select 35839,'035839','1993-036ATM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-13 00:00:00',99.7,714,775,74.24,'Intermediate' from dual union all
select 3584,'003584','1968-091S','deb Kosmos-249','',16818,'In Earth orbit','','1992-02-03 00:00:00',106.81,526,1632,62.34,'Intermediate' from dual union all
select 35840,'035840','1993-036ATN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-13 00:00:00',102.33,749,989,73.93,'Intermediate' from dual union all
select 35841,'035841','1993-036ATP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-14 00:00:00',99.47,579,887,74.06,'Intermediate' from dual union all
select 35842,'035842','1993-036ATQ','deb Kosmos-2251','',65138,'Reentered','2012-06-11 00:00:00','2009-09-13 00:00:00',98.02,513,814,73.99,'Intermediate' from dual union all
select 35843,'035843','1993-036ATR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-13 00:00:00',102.05,737,975,74,'Intermediate' from dual union all
select 35844,'035844','1997-051SU','deb Iridium 33','',66131,'In Earth orbit','','2009-09-14 00:00:00',100.22,742,796,86.34,'Polar' from dual union all
select 35845,'035845','1997-051SV','deb Iridium 33','',66131,'Reentered','2010-01-18 00:00:00','2009-09-13 00:00:00',97.49,557,721,86.2,'Polar' from dual union all
select 35846,'035846','1997-051SW','deb Iridium 33','',66131,'In Earth orbit','','2009-09-13 00:00:00',100.7,775,809,86.38,'Polar' from dual union all
select 35847,'035847','1997-051SX','deb Iridium 33','',66131,'Reentered','2014-01-28 00:00:00','2009-09-14 00:00:00',100.47,762,800,86.38,'Polar' from dual union all
select 35848,'035848','1997-051SY','deb Iridium 33','',66131,'In Earth orbit','','2009-09-13 00:00:00',100.16,755,778,86.41,'Polar' from dual union all
select 35849,'035849','1997-051SZ','deb Iridium 33','',66131,'Reentered','2013-08-04 00:00:00','2009-09-14 00:00:00',100.49,756,808,86.32,'Polar' from dual union all
select 3585,'003585','1968-091T','deb Kosmos-249','',16818,'Reentered','1990-02-09 00:00:00','1990-02-09 00:00:00',88.67,194,224,62.19,'Intermediate' from dual union all
select 35850,'035850','1997-051TA','deb Iridium 33','',66131,'In Earth orbit','','2009-09-13 00:00:00',100.14,751,780,86.36,'Polar' from dual union all
select 35851,'035851','1997-051TB','deb Iridium 33','',66131,'In Earth orbit','','2009-09-14 00:00:00',101.66,774,901,86.43,'Polar' from dual union all
select 35852,'035852','1997-051TC','deb Iridium 33','',66131,'Reentered','2014-03-29 00:00:00','2009-09-13 00:00:00',100.11,757,771,86.42,'Polar' from dual union all
select 35853,'035853','1997-051TD','deb Iridium 33','',66131,'In Earth orbit','','2009-09-14 00:00:00',101.81,759,930,86.32,'Polar' from dual union all
select 35854,'035854','1997-051TE','deb Iridium 33','',66131,'In Earth orbit','','2009-09-14 00:00:00',100.3,771,775,86.31,'Polar' from dual union all
select 35855,'035855','1997-051TF','deb Iridium 33','',66131,'In Earth orbit','','2009-09-13 00:00:00',101.47,757,900,86.44,'Polar' from dual union all
select 35856,'035856','1997-051TG','deb Iridium 33','',66131,'In Earth orbit','','2009-09-13 00:00:00',100.62,776,800,86.39,'Polar' from dual union all
select 35857,'035857','1997-051TH','deb Iridium 33','',66131,'In Earth orbit','','2009-09-13 00:00:00',100.06,749,774,86.44,'Polar' from dual union all
select 35858,'035858','1997-051TJ','deb Iridium 33','',66131,'In Earth orbit','','2009-09-13 00:00:00',101.45,778,877,86.38,'Polar' from dual union all
select 35859,'035859','1997-051TK','deb Iridium 33','',66131,'Reentered','2014-09-26 00:00:00','2009-09-14 00:00:00',100.29,767,778,86.38,'Polar' from dual union all
select 3586,'003586','1968-091U','deb Kosmos-249','',16818,'Reentered','1970-03-31 00:00:00','1970-03-12 00:00:00',92.27,264,507,62.12,'Intermediate' from dual union all
select 35860,'035860','1997-051TL','deb Iridium 33','',66131,'In Earth orbit','','2009-09-14 00:00:00',100.44,763,796,86.47,'Polar' from dual union all
select 35861,'035861','1997-051TM','deb Iridium 33','',66131,'Reentered','2011-12-31 00:00:00','2009-09-13 00:00:00',98.47,527,844,86.32,'Polar' from dual union all
select 35862,'035862','1997-051TN','deb Iridium 33','',66131,'In Earth orbit','','2009-09-13 00:00:00',100.69,775,807,86.39,'Polar' from dual union all
select 35863,'035863','1997-051TP','deb Iridium 33','',66131,'In Earth orbit','','2009-09-13 00:00:00',101.84,777,915,86.36,'Polar' from dual union all
select 35864,'035864','2009-046C','deb CZ-3C','',68727,'Reentered','2009-10-02 00:00:00','2009-09-13 00:00:00',111,126,2420,22.28,'Equatorial' from dual union all
select 35865,'035865','2009-049A','Meteor-M No. 1','Meteor-M No. 1',68730,'In Earth orbit','','2009-09-19 00:00:00',101.26,816,821,98.8,'Sun-Synch' from dual union all
select 35866,'035866','2009-049B','Sterkh No. 12L','Sterkh No. 12L',68730,'In Earth orbit','','2009-09-19 00:00:00',101.23,814,819,98.8,'Sun-Synch' from dual union all
select 35867,'035867','2009-049C','Fregat No. 1014','Fregat 14S44 No. 1014',68730,'In Earth orbit','','2009-09-19 00:00:00',94.55,491,503,97.37,'Sun-Synch' from dual union all
select 35868,'035868','2009-049D','Universitetskiy/Tat''yana-2','MGU Tat''yana-2',68730,'In Earth orbit','','2009-09-19 00:00:00',101.25,814,821,98.8,'Sun-Synch' from dual union all
select 35869,'035869','2009-049E','UGATUSAT','',68730,'In Earth orbit','','2009-09-19 00:00:00',101.25,814,821,98.8,'Sun-Synch' from dual union all
select 3587,'003587','1968-091V','deb Kosmos-249','',16818,'Reentered','2002-09-21 00:00:00','1992-02-02 00:00:00',103.79,577,1298,62.21,'Intermediate' from dual union all
select 35870,'035870','2009-049F','Sumbandila-OSCAR-67','SumbandilaSat',68730,'In Earth orbit','','2009-09-19 00:00:00',94.55,490,503,97.37,'Sun-Synch' from dual union all
select 35871,'035871','2009-049G','Blits','Blits',68730,'In Earth orbit','','2009-09-22 00:00:00',101.29,816,824,98.8,'Sun-Synch' from dual union all
select 35872,'035872','2009-049H','Blok-I 14S54','14S54',68730,'Reentered','2009-09-20 00:00:00','2009-09-18 00:00:00',88.1,175,186,98.82,'Sun-Synch' from dual union all
select 35873,'035873','2009-050A','Nimiq 5','Nimiq 5',68731,'In Earth orbit','','2009-11-27 00:00:00',1436.09,35780,35793,.02,'Stationary' from dual union all
select 35874,'035874','2009-050B','Briz-M No. 99508','Briz-M 14S43 No. 99508',68731,'In Earth orbit','','2011-06-02 00:00:00',745.77,7847,33880,11.99,'Medium' from dual union all
select 35875,'035875','2009-050C','Briz-M DTB','Briz-M torus tank',68731,'In Earth orbit','','2009-09-18 00:00:00',373.94,500,21131,49.32,'Highly Ellip' from dual union all
select 35876,'035876','1967-116C','deb Kosmos-192','',15371,'Reentered','2009-10-24 00:00:00','2009-09-21 00:00:00',98.22,578,770,74.04,'Intermediate' from dual union all
select 35877,'035877','1967-116D','deb Kosmos-192','',15371,'Reentered','2009-09-22 00:00:00','2009-09-19 00:00:00',93.13,335,521,74.39,'Intermediate' from dual union all
select 35878,'035878','1967-116E','deb Kosmos-192','',15371,'Reentered','2009-10-18 00:00:00','2009-09-21 00:00:00',98.99,664,757,74.2,'Intermediate' from dual union all
select 35879,'035879','1993-036ATS','deb Kosmos-2251','',65138,'Reentered','2011-01-13 00:00:00','2009-09-22 00:00:00',95.9,391,734,74.06,'Intermediate' from dual union all
select 3588,'003588','1968-091W','deb Kosmos-249','',16818,'Reentered','1969-05-05 00:00:00','1969-05-02 00:00:00',91.36,216,467,62.19,'Intermediate' from dual union all
select 35880,'035880','1993-036ATT','deb Kosmos-2251','',65138,'Reentered','2013-04-16 00:00:00','2009-09-22 00:00:00',96.91,448,774,74.05,'Intermediate' from dual union all
select 35881,'035881','1993-036ATU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-21 00:00:00',102.35,804,936,74.2,'Intermediate' from dual union all
select 35882,'035882','1993-036ATV','deb Kosmos-2251','',65138,'Reentered','2011-05-22 00:00:00','2009-09-22 00:00:00',96.12,401,744,74.06,'Intermediate' from dual union all
select 35883,'035883','1993-036ATW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-22 00:00:00',99.82,683,817,74.03,'Intermediate' from dual union all
select 35884,'035884','1993-036ATX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-21 00:00:00',102.71,769,1005,74.01,'Intermediate' from dual union all
select 35885,'035885','1993-036ATY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-21 00:00:00',100.62,769,806,74.03,'Intermediate' from dual union all
select 35886,'035886','1993-036ATZ','deb Kosmos-2251','',65138,'Reentered','2015-02-24 00:00:00','2009-09-22 00:00:00',98.74,623,774,74.02,'Intermediate' from dual union all
select 35887,'035887','1993-036AUA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-21 00:00:00',102.17,763,960,73.97,'Intermediate' from dual union all
select 35888,'035888','1993-036AUB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-21 00:00:00',99.61,697,783,74.04,'Intermediate' from dual union all
select 35889,'035889','1993-036AUC','deb Kosmos-2251','',65138,'Reentered','2015-07-12 00:00:00','2009-09-22 00:00:00',98.6,598,786,74.07,'Intermediate' from dual union all
select 3589,'003589','1967-014G','deb Diademe 2','',15269,'Reentered','1979-11-20 00:00:00','1967-06-19 00:00:00',110.12,593,1872,39.45,'Intermediate' from dual union all
select 35890,'035890','1993-036AUD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-22 00:00:00',101.17,748,881,73.95,'Intermediate' from dual union all
select 35891,'035891','1993-036AUE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-22 00:00:00',98.12,558,780,74.01,'Intermediate' from dual union all
select 35892,'035892','1993-036AUF','deb Kosmos-2251','',65138,'Reentered','2010-12-08 00:00:00','2009-09-22 00:00:00',95.87,404,717,74.09,'Intermediate' from dual union all
select 35893,'035893','1993-036AUG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-22 00:00:00',100.86,733,865,73.67,'Intermediate' from dual union all
select 35894,'035894','1993-036AUH','deb Kosmos-2251','',65138,'Reentered','2014-04-13 00:00:00','2009-09-21 00:00:00',97.39,504,764,74,'Intermediate' from dual union all
select 35895,'035895','1993-036AUJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-21 00:00:00',104,771,1125,74.06,'Intermediate' from dual union all
select 35896,'035896','1993-036AUK','deb Kosmos-2251','',65138,'Reentered','2015-03-27 00:00:00','2009-09-22 00:00:00',99.44,696,768,73.97,'Intermediate' from dual union all
select 35897,'035897','1993-036AUL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-22 00:00:00',97.64,518,774,73.97,'Intermediate' from dual union all
select 35898,'035898','1993-036AUM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-22 00:00:00',98.52,588,788,74.01,'Intermediate' from dual union all
select 35899,'035899','1993-036AUN','deb Kosmos-2251','',65138,'Reentered','2013-08-29 00:00:00','2009-09-21 00:00:00',98.72,630,765,74.03,'Intermediate' from dual union all
select 359,'000359','61 Omicron 120','deb r/b Ablestar','',7273,'Reentered','1965-08-31 00:00:00','1962-07-29 00:00:00',96.8,338,872,67.02,'Intermediate' from dual union all
select 3590,'003590','1966-024E','NNS O-8 insulation','',13431,'Reentered','1978-03-23 00:00:00','1968-07-01 00:00:00',105.24,889,1123,89.74,'Polar' from dual union all
select 35900,'035900','1993-036AUP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-22 00:00:00',100.75,766,822,74.04,'Intermediate' from dual union all
select 35901,'035901','1993-036AUQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-21 00:00:00',100.43,764,794,74.04,'Intermediate' from dual union all
select 35902,'035902','1993-036AUR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-21 00:00:00',102.2,761,964,74,'Intermediate' from dual union all
select 35903,'035903','1993-036AUS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-22 00:00:00',98.58,599,783,74.07,'Intermediate' from dual union all
select 35904,'035904','1993-036AUT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-22 00:00:00',99.88,729,777,74.03,'Intermediate' from dual union all
select 35905,'035905','1993-036AUU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-22 00:00:00',98.12,560,778,74.04,'Intermediate' from dual union all
select 35906,'035906','1993-036AUV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-09-21 00:00:00',99.45,607,857,74.03,'Intermediate' from dual union all
select 35907,'035907','1997-051TQ','deb Iridium 33','',66131,'In Earth orbit','','2009-09-21 00:00:00',102.44,770,979,86.36,'Polar' from dual union all
select 35908,'035908','1997-051TR','deb Iridium 33','',66131,'In Earth orbit','','2009-09-21 00:00:00',101.76,770,913,86.24,'Polar' from dual union all
select 35909,'035909','1997-051TS','deb Iridium 33','',66131,'Reentered','2014-06-29 00:00:00','2009-09-22 00:00:00',102.42,771,976,86.4,'Polar' from dual union all
select 3591,'003591','1966-041E','NNS O-9 part','',13448,'In Earth orbit','','1969-01-13 00:00:00',103.36,853,982,89.87,'Polar' from dual union all
select 35910,'035910','1997-051TT','deb Iridium 33','',66131,'Reentered','2016-02-20 00:00:00','2009-09-22 00:00:00',102.67,767,1003,86.41,'Polar' from dual union all
select 35911,'035911','1997-051TU','deb Iridium 33','',66131,'In Earth orbit','','2009-09-21 00:00:00',102.09,774,942,86.24,'Polar' from dual union all
select 35912,'035912','1997-051TV','deb Iridium 33','',66131,'Reentered','2012-06-20 00:00:00','2009-09-22 00:00:00',100.43,721,837,86.45,'Polar' from dual union all
select 35913,'035913','1997-051TW','deb Iridium 33','',66131,'Reentered','2015-04-30 00:00:00','2009-09-22 00:00:00',100.15,597,934,86.58,'Polar' from dual union all
select 35914,'035914','1997-051TX','deb Iridium 33','',66131,'Reentered','2015-06-15 00:00:00','2009-09-22 00:00:00',100.83,721,875,86.41,'Polar' from dual union all
select 35915,'035915','1997-051TY','deb Iridium 33','',66131,'In Earth orbit','','2009-09-22 00:00:00',100.2,749,787,86.36,'Polar' from dual union all
select 35916,'035916','1997-051TZ','deb Iridium 33','',66131,'Reentered','2015-05-04 00:00:00','2009-09-22 00:00:00',100.52,775,792,86.39,'Polar' from dual union all
select 35917,'035917','1997-051UA','deb Iridium 33','',66131,'In Earth orbit','','2009-09-22 00:00:00',100.69,743,840,86.37,'Polar' from dual union all
select 35918,'035918','1997-051UB','deb Iridium 33','',66131,'In Earth orbit','','2009-09-22 00:00:00',100.84,772,824,86.41,'Polar' from dual;
insert into SATELLITE
select 35919,'035919','1997-051UC','deb Iridium 33','',66131,'Reentered','2014-01-14 00:00:00','2009-09-22 00:00:00',100.05,748,774,86.29,'Polar' from dual union all
select 3592,'003592','1965-048F','deb NNS O-4','',11798,'In Earth orbit','','1969-01-26 00:00:00',106.56,1022,1113,89.92,'Polar' from dual union all
select 35920,'035920','1997-051UD','deb Iridium 33','',66131,'Reentered','2015-08-15 00:00:00','2009-09-22 00:00:00',100.6,773,801,86.38,'Polar' from dual union all
select 35921,'035921','1997-051UE','deb Iridium 33','',66131,'In Earth orbit','','2009-09-22 00:00:00',102.99,768,1032,86.44,'Polar' from dual union all
select 35922,'035922','1997-051UF','deb Iridium 33','',66131,'Reentered','2012-12-26 00:00:00','2013-03-08 00:00:00',99.97,732,783,86.39,'Polar' from dual union all
select 35923,'035923','1997-051UG','deb Iridium 33','',66131,'Reentered','2012-05-29 00:00:00','2009-09-22 00:00:00',100.23,745,794,86.43,'Polar' from dual union all
select 35924,'035924','1997-051UH','deb Iridium 33','',66131,'Reentered','2013-11-14 00:00:00','2009-09-21 00:00:00',100.62,764,811,86.42,'Polar' from dual union all
select 35925,'035925','1997-051UJ','deb Iridium 33','',66131,'In Earth orbit','','2009-09-22 00:00:00',100.7,770,814,86.34,'Polar' from dual union all
select 35926,'035926','1997-051UK','deb Iridium 33','',66131,'In Earth orbit','','2009-09-22 00:00:00',100.25,760,781,86.36,'Polar' from dual union all
select 35927,'035927','1997-051UL','deb Iridium 33','',66131,'Reentered','2011-11-29 00:00:00','2009-09-21 00:00:00',99.68,717,770,86.47,'Polar' from dual union all
select 35928,'035928','1997-051UM','deb Iridium 33','',66131,'Reentered','2011-11-28 00:00:00','2009-09-22 00:00:00',100.65,749,830,86.5,'Polar' from dual union all
select 35929,'035929','1997-051UN','deb Iridium 33','',66131,'In Earth orbit','','2009-09-21 00:00:00',99.89,727,780,86.36,'Polar' from dual union all
select 3593,'003593','1968-097X','deb Kosmos-252','',16824,'Reentered','1981-10-09 00:00:00','1981-10-07 00:00:00',91.58,321,384,62.2,'Intermediate' from dual union all
select 35930,'035930','1997-051UP','deb Iridium 33','',66131,'Reentered','2010-12-18 00:00:00','2009-09-22 00:00:00',100.35,722,828,86.38,'Polar' from dual union all
select 35931,'035931','2009-051A','Oceansat-2','IRS Oceansat-2',68732,'In Earth orbit','','2009-09-23 00:00:00',99.18,715,724,98.34,'Sun-Synch' from dual union all
select 35932,'035932','2009-051B','SwissCube','SwissCube',68732,'In Earth orbit','','2009-09-23 00:00:00',99.15,713,722,98.34,'Sun-Synch' from dual union all
select 35933,'035933','2009-051C','BEESAT','BEESAT',68732,'In Earth orbit','','2009-09-23 00:00:00',99.16,714,723,98.33,'Sun-Synch' from dual union all
select 35934,'035934','2009-051D','UWE-2','UWE-2',68732,'In Earth orbit','','2009-09-23 00:00:00',99.11,713,720,98.33,'Sun-Synch' from dual union all
select 35935,'035935','2009-051E','ITUpSAT-1','ITU piko SATellite 1',68732,'In Earth orbit','','2009-09-23 00:00:00',99.11,712,721,98.31,'Sun-Synch' from dual union all
select 35936,'035936','2009-051F','PS4 C14','PSLV-C14-PS4-EB',68732,'In Earth orbit','','2009-09-24 00:00:00',99.92,716,794,98.29,'Sun-Synch' from dual union all
select 35937,'035937','2009-052A','STSS 1','STSS-Demo 1',68733,'In Earth orbit','','2009-09-25 00:00:00',112.66,1346,1352,58,'Intermediate' from dual union all
select 35938,'035938','2009-052B','STSS 2','STSS-Demo 2',68733,'In Earth orbit','','2009-09-25 00:00:00',112.56,1339,1349,58,'Intermediate' from dual union all
select 35939,'035939','2009-052C','Delta 344','Delta SSPS AJ-10-118K',68733,'Reentered','2010-02-20 00:00:00','2009-09-25 00:00:00',112.56,1339,1349,58,'Intermediate' from dual union all
select 3594,'003594','1968-108A','OPS 6518','GAMBIT-3 Mission 4318',16835,'Reentered Attached','1968-12-12 00:00:00','1968-12-06 00:00:00',92.36,127,654,106.18,'Retrograde' from dual union all
select 35940,'035940','2009-053A','Soyuz TMA-16','Soyuz 11F732A17 No. 226',68734,'Landed','2010-03-17 00:00:00','2009-10-07 00:00:00',91.44,340,350,51.64,'Intermediate' from dual union all
select 35941,'035941','2009-053B','11S510','11S510',68734,'Reentered','2009-10-04 00:00:00','2009-10-02 00:00:00',88.34,181,204,51.6,'Intermediate' from dual union all
select 35942,'035942','2009-054A','Amazonas-2','Amazonas 2',68735,'In Earth orbit','','2009-10-09 00:00:00',1428.12,35423,35837,.07,'Drift GEO' from dual union all
select 35943,'035943','2009-054B','COMSATBw-1','COMSATBw-1',68735,'In Earth orbit','','2009-10-27 00:00:00',1436.04,35775,35796,.01,'Stationary' from dual union all
select 35944,'035944','2009-054C','ESC-A 549','Ariane 5 ESC-A',68735,'In Earth orbit','','2009-10-09 00:00:00',630.26,267,35678,3.22,'GEO Transfer' from dual union all
select 35945,'035945','2009-054D','Sylda 5A','Sylda 5 No. 36A',68735,'In Earth orbit','','2009-10-11 00:00:00',631.33,271,35729,2.95,'GEO Transfer' from dual union all
select 35946,'035946','2009-055A','WorldView-2','WorldView-2',68736,'In Earth orbit','','2009-10-15 00:00:00',100.17,765,769,98.58,'Sun-Synch' from dual union all
select 35947,'035947','2009-055B','Delta 345','Delta SSPS AJ-10-118K',68736,'Reentered','2010-03-11 00:00:00','2009-10-15 00:00:00',94.2,209,751,92.02,'Polar' from dual union all
select 35948,'035948','2009-056A','Progress M-03M','Progress 7K-TGM No.403',68737,'Deorbited','2010-04-27 00:00:00','2009-05-09 00:00:00',90.79,291,335,51.63,'Intermediate' from dual union all
select 35949,'035949','2009-056B','11S510','11S510',68737,'Reentered','2009-10-17 00:00:00','2009-05-08 00:00:00',88.42,181,212,51.62,'Intermediate' from dual union all
select 3595,'003595','1968-109A','HEOS 1','HEOS A1',16836,'Deep Space','1968-12-06 00:00:00','1970-03-19 00:00:00',6704.09,12243,210550,52.62,'Deep Space' from dual union all
select 35950,'035950','2009-002M','deb H-2A','',68683,'Reentered','2010-05-03 00:00:00','2009-10-16 00:00:00',97.19,584,665,98.06,'Sun-Synch' from dual union all
select 35951,'035951','2009-057A','DMSP F-18','DMSP Block 5D-3 S-18',68738,'In Earth orbit','','2009-10-21 00:00:00',101.92,843,857,98.93,'Sun-Synch' from dual union all
select 35952,'035952','2009-057B','Centaur AV-017','Centaur AV-017',68738,'Deep Space','2009-10-19 00:00:00','2009-10-18 00:00:00',181.96,767,34797,97.17,'Earth Escape' from dual union all
select 35953,'035953','2009-057C','DMSP F-18 solar array band','',68738,'In Earth orbit','','2009-10-20 00:00:00',101.91,847,852,98.93,'Sun-Synch' from dual union all
select 35954,'035954','2009-057D','DMSP F-18 solar array band','',68738,'In Earth orbit','','2009-10-20 00:00:00',101.91,847,852,98.93,'Sun-Synch' from dual union all
select 35955,'035955','1993-061K','deb Ariane V59','',65163,'In Earth orbit','','2009-10-19 00:00:00',100.64,784,794,98.59,'Sun-Synch' from dual union all
select 35956,'035956','1993-036AUW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',105.71,765,1290,74.1,'Intermediate' from dual union all
select 35957,'035957','1993-036AUX','deb Kosmos-2251','',65138,'Reentered','2015-10-31 00:00:00','2009-10-19 00:00:00',99.08,650,780,73.94,'Intermediate' from dual union all
select 35958,'035958','1993-036AUY','deb Kosmos-2251','',65138,'Reentered','2018-02-11 00:00:00','2009-10-19 00:00:00',99.15,654,781,73.83,'Intermediate' from dual union all
select 35959,'035959','1993-036AUZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',100.58,770,802,74.03,'Intermediate' from dual union all
select 3596,'003596','1968-109B','Delta 61','DSV-3E-3 61',16836,'Reentered','1968-12-27 00:00:00','1968-12-12 00:00:00',93.17,163,696,28.3,'Intermediate' from dual union all
select 35960,'035960','1993-036AVA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-18 00:00:00',98.91,616,797,74.02,'Intermediate' from dual union all
select 35961,'035961','1993-036AVB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-18 00:00:00',103.73,768,1102,73.92,'Intermediate' from dual union all
select 35962,'035962','1993-036AVC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',101.17,769,860,74.01,'Intermediate' from dual union all
select 35963,'035963','1993-036AVD','deb Kosmos-2251','',65138,'Reentered','2012-09-25 00:00:00','2009-10-19 00:00:00',99.05,665,762,74.05,'Intermediate' from dual union all
select 35964,'035964','1993-036AVE','deb Kosmos-2251','',65138,'Reentered','2014-01-29 00:00:00','2009-10-18 00:00:00',98.13,564,775,73.94,'Intermediate' from dual union all
select 35965,'035965','1993-036AVF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',99.12,665,769,74.05,'Intermediate' from dual union all
select 35966,'035966','1993-036AVG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',102.69,745,1027,74.06,'Intermediate' from dual union all
select 35967,'035967','1993-036AVH','deb Kosmos-2251','',65138,'Reentered','2013-03-10 00:00:00','2009-10-19 00:00:00',98.83,641,764,74,'Intermediate' from dual union all
select 35968,'035968','1993-036AVJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-18 00:00:00',98.85,607,800,73.72,'Intermediate' from dual union all
select 35969,'035969','1993-036AVK','deb Kosmos-2251','',65138,'Reentered','2011-02-04 00:00:00','2009-10-19 00:00:00',96.43,425,751,73.95,'Intermediate' from dual union all
select 3597,'003597','1968-110A','OAO 2','OAO A2',16837,'In Earth orbit','','1969-01-06 00:00:00',100.3,768,778,35,'Intermediate' from dual union all
select 35970,'035970','1993-036AVL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',99.73,711,781,74.05,'Intermediate' from dual union all
select 35971,'035971','1993-036AVM','deb Kosmos-2251','',65138,'Reentered','2011-09-20 00:00:00','2009-10-18 00:00:00',97.17,485,761,73.95,'Intermediate' from dual union all
select 35972,'035972','1993-036AVN','deb Kosmos-2251','',65138,'Reentered','2012-03-21 00:00:00','2009-10-18 00:00:00',97.67,524,771,73.93,'Intermediate' from dual union all
select 35973,'035973','1993-036AVP','deb Kosmos-2251','',65138,'Reentered','2013-04-01 00:00:00','2009-10-19 00:00:00',96.43,410,766,73.92,'Intermediate' from dual union all
select 35974,'035974','1993-036AVQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',100.21,756,781,74.03,'Intermediate' from dual union all
select 35975,'035975','1993-036AVR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-18 00:00:00',102.54,773,984,74.02,'Intermediate' from dual union all
select 35976,'035976','1993-036AVS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-18 00:00:00',100.89,752,849,74.05,'Intermediate' from dual union all
select 35977,'035977','1993-036AVT','deb Kosmos-2251','',65138,'Reentered','2012-10-03 00:00:00','2009-10-19 00:00:00',98.38,603,760,73.87,'Intermediate' from dual union all
select 35978,'035978','1993-036AVU','deb Kosmos-2251','',65138,'Reentered','2015-02-03 00:00:00','2009-10-18 00:00:00',98.73,616,780,73.97,'Intermediate' from dual union all
select 35979,'035979','1993-036AVV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',99.41,666,795,74.02,'Intermediate' from dual union all
select 3598,'003598','1968-110B','Centaur AC-16','Centaur 13D',16837,'In Earth orbit','','1969-01-14 00:00:00',100.21,722,814,35,'Intermediate' from dual union all
select 35980,'035980','1993-036AVW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',100.69,728,855,73.92,'Intermediate' from dual union all
select 35981,'035981','1993-036AVX','deb Kosmos-2251','',65138,'Reentered','2014-06-13 00:00:00','2009-10-19 00:00:00',97.94,540,781,74.07,'Intermediate' from dual union all
select 35982,'035982','1993-036AVY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',99.6,679,800,74.06,'Intermediate' from dual union all
select 35983,'035983','1993-036AVZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',99.87,737,767,74.05,'Intermediate' from dual union all
select 35984,'035984','1993-036AWA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',100.21,554,982,74.01,'Intermediate' from dual union all
select 35985,'035985','1993-036AWB','deb Kosmos-2251','',65138,'Reentered','2012-12-02 00:00:00','2009-10-18 00:00:00',98.26,564,787,74.06,'Intermediate' from dual union all
select 35986,'035986','1993-036AWC','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',100.14,746,785,74.03,'Intermediate' from dual union all
select 35987,'035987','1993-036AWD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-18 00:00:00',101.08,764,855,74.01,'Intermediate' from dual union all
select 35988,'035988','1993-036AWE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',100.95,769,839,74.02,'Intermediate' from dual union all
select 35989,'035989','1993-036AWF','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',100.51,766,800,74.04,'Intermediate' from dual union all
select 3599,'003599','1968-091X','deb Kosmos-249 [*]','',16818,'Reentered','2009-08-04 00:00:00','1993-08-07 00:00:00',104.06,483,1418,62.13,'Intermediate' from dual union all
select 35990,'035990','1993-036AWG','deb Kosmos-2251','',65138,'Reentered','2012-02-16 00:00:00','2009-10-19 00:00:00',97.58,522,764,74,'Intermediate' from dual union all
select 35991,'035991','1993-036AWH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',100.62,610,966,73.93,'Intermediate' from dual union all
select 35992,'035992','1993-036AWJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',98.88,631,780,73.95,'Intermediate' from dual union all
select 35993,'035993','1993-036AWK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',99.73,719,772,74.02,'Intermediate' from dual union all
select 35994,'035994','1993-036AWL','deb Kosmos-2251','',65138,'Reentered','2013-11-25 00:00:00','2009-10-19 00:00:00',99.12,618,816,74.05,'Intermediate' from dual union all
select 35995,'035995','1993-036AWM','deb Kosmos-2251','',65138,'Reentered','2011-08-25 00:00:00','2009-10-19 00:00:00',96.85,461,755,74.03,'Intermediate' from dual union all
select 35996,'035996','1993-036AWN','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',97.89,537,778,74.02,'Intermediate' from dual union all
select 35997,'035997','1993-036AWP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',100.37,763,790,74.05,'Intermediate' from dual union all
select 35998,'035998','1993-036AWQ','deb Kosmos-2251','',65138,'Reentered','2011-05-08 00:00:00','2009-10-19 00:00:00',96.86,446,771,73.95,'Intermediate' from dual union all
select 35999,'035999','1993-036AWR','deb Kosmos-2251','',65138,'Reentered','2011-11-29 00:00:00','2009-10-19 00:00:00',97.19,489,759,74,'Intermediate' from dual union all
select 36,'000036','60 Epsilon 3','Korabl''-Sputnik','Vostok-1P',6320,'Reentered','1965-10-15 00:00:00','1960-06-20 00:00:00',94.11,277,674,64.89,'Intermediate' from dual union all
select 360,'000360','62 A Kappa 1','FTV 1152','CORONA M-10',8144,'Reentered Attached','1962-08-26 00:00:00','1962-08-09 00:00:00',90.42,193,398,82.25,'Intermediate' from dual union all
select 3600,'003600','1968-097Y','deb Kosmos-249','',16824,'Reentered','1981-11-03 00:00:00','1981-10-10 00:00:00',95.41,421,656,62.24,'Intermediate' from dual union all
select 36000,'036000','1993-036AWS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',100.61,719,856,74.03,'Intermediate' from dual union all
select 36001,'036001','1993-036AWT','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',100.52,768,799,74.02,'Intermediate' from dual union all
select 36002,'036002','1993-036AWU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-18 00:00:00',99,641,781,74.02,'Intermediate' from dual union all
select 36003,'036003','1993-036AWV','deb Kosmos-2251','',65138,'Reentered','2012-02-03 00:00:00','2009-10-18 00:00:00',96.69,440,761,73.98,'Intermediate' from dual union all
select 36004,'036004','1993-036AWW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-18 00:00:00',97.42,498,773,73.91,'Intermediate' from dual union all
select 36005,'036005','1993-036AWX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-19 00:00:00',99.82,704,796,74.02,'Intermediate' from dual union all
select 36006,'036006','1993-036AWY','deb Kosmos-2251','',65138,'Reentered','2013-01-02 00:00:00','2009-10-19 00:00:00',98.29,588,766,74.11,'Intermediate' from dual union all
select 36007,'036007','1993-036AWZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-18 00:00:00',98.61,606,779,74.04,'Intermediate' from dual;
insert into SATELLITE
select 36008,'036008','1993-036AXA','deb Kosmos-2251','',65138,'In Earth orbit','','2009-10-18 00:00:00',104.27,767,1154,74.03,'Intermediate' from dual union all
select 36009,'036009','1997-051UQ','deb Iridium 33','',66131,'Reentered','2013-01-14 00:00:00','2009-10-18 00:00:00',100.34,749,801,86.36,'Polar' from dual union all
select 3601,'003601','1968-097Z','deb Kosmos-249','',16824,'Reentered','1980-06-30 00:00:00','1980-06-27 00:00:00',91.65,299,412,62.32,'Intermediate' from dual union all
select 36010,'036010','1997-051UR','deb Iridium 33','',66131,'Reentered','2015-03-08 00:00:00','2009-10-18 00:00:00',100.52,728,839,86.3,'Polar' from dual union all
select 36011,'036011','1997-051US','deb Iridium 33','',66131,'In Earth orbit','','2009-10-19 00:00:00',100.68,774,808,86.35,'Polar' from dual union all
select 36012,'036012','1997-051UT','deb Iridium 33','',66131,'In Earth orbit','','2009-10-18 00:00:00',104.99,772,1215,86.47,'Polar' from dual union all
select 36013,'036013','1997-051UU','deb Iridium 33','',66131,'Reentered','2014-12-11 00:00:00','2009-10-18 00:00:00',100.47,774,788,86.39,'Polar' from dual union all
select 36014,'036014','1997-051UV','deb Iridium 33','',66131,'Reentered','2014-06-18 00:00:00','2009-10-18 00:00:00',99.71,717,773,86.37,'Polar' from dual union all
select 36015,'036015','1997-051UW','deb Iridium 33','',66131,'In Earth orbit','','2009-10-19 00:00:00',99.98,740,775,86.35,'Polar' from dual union all
select 36016,'036016','1997-051UX','deb Iridium 33','',66131,'In Earth orbit','','2009-10-18 00:00:00',100.14,748,783,86.41,'Polar' from dual union all
select 36017,'036017','1997-051UY','deb Iridium 33','',66131,'In Earth orbit','','2009-10-18 00:00:00',100.56,776,795,86.39,'Polar' from dual union all
select 36018,'036018','1997-051UZ','deb Iridium 33','',66131,'In Earth orbit','','2009-10-19 00:00:00',100.76,774,816,86.4,'Polar' from dual union all
select 36019,'036019','1997-051VA','deb Iridium 33','',66131,'In Earth orbit','','2009-10-18 00:00:00',100.25,764,777,86.4,'Polar' from dual union all
select 3602,'003602','1968-111A','Kosmos-258','Zenit-2 No. 69',16838,'Landed','1968-12-18 00:00:00','1968-12-12 00:00:00',89.54,204,300,64.96,'Intermediate' from dual union all
select 36020,'036020','1997-051VB','deb Iridium 33','',66131,'Reentered','2012-11-15 00:00:00','2009-10-18 00:00:00',101.04,740,875,86.33,'Polar' from dual union all
select 36021,'036021','1997-051VC','deb Iridium 33','',66131,'In Earth orbit','','2009-10-18 00:00:00',102.57,780,981,86.43,'Polar' from dual union all
select 36022,'036022','1997-051VD','deb Iridium 33','',66131,'Reentered','2012-12-28 00:00:00','2009-10-18 00:00:00',101.06,775,843,86.36,'Polar' from dual union all
select 36023,'036023','1997-051VE','deb Iridium 33','',66131,'In Earth orbit','','2009-10-19 00:00:00',100.92,776,829,86.35,'Polar' from dual union all
select 36024,'036024','1997-051VF','deb Iridium 33','',66131,'Reentered','2015-02-05 00:00:00','2009-10-18 00:00:00',100.35,771,780,86.49,'Polar' from dual union all
select 36025,'036025','1997-051VG','deb Iridium 33','',66131,'In Earth orbit','','2009-10-18 00:00:00',101.8,761,927,86.59,'Polar' from dual union all
select 36026,'036026','1997-051VH','deb Iridium 33','',66131,'Reentered','2015-11-09 00:00:00','2009-10-19 00:00:00',99.92,730,779,86.43,'Polar' from dual union all
select 36027,'036027','1997-051VJ','deb Iridium 33','',66131,'Reentered','2015-09-16 00:00:00','2009-10-18 00:00:00',100.48,777,786,86.42,'Polar' from dual union all
select 36028,'036028','1997-051VK','deb Iridium 33','',66131,'In Earth orbit','','2009-10-18 00:00:00',99.91,724,785,86.34,'Polar' from dual union all
select 36029,'036029','1997-051VL','deb Iridium 33','',66131,'Reentered','2014-10-22 00:00:00','2009-10-19 00:00:00',99.88,732,774,86.44,'Polar' from dual union all
select 3603,'003603','1968-111B','11A57I','11A57I',16838,'Reentered','1968-12-17 00:00:00','1968-12-12 00:00:00',89.3,198,282,64.97,'Intermediate' from dual union all
select 36030,'036030','1996-051M','deb Zenit 11S772','',65939,'In Earth orbit','','2009-10-19 00:00:00',101.84,831,861,70.87,'Intermediate' from dual union all
select 36031,'036031','1996-051N','deb Zenit 11S772','',65939,'In Earth orbit','','2009-10-22 00:00:00',101.94,839,862,70.89,'Intermediate' from dual union all
select 36032,'036032','2009-058A','NSS 12','NSS 12',68739,'In Earth orbit','','2009-11-06 00:00:00',1435.37,35743,35802,.04,'Drift GEO' from dual union all
select 36033,'036033','2009-058B','Thor 6','Thor 6',68739,'In Earth orbit','','2009-11-05 00:00:00',1433.69,35677,35802,.03,'Drift GEO' from dual union all
select 36034,'036034','2009-058C','ESC-A 550','Ariane 5 ESC-A',68739,'In Earth orbit','','2009-11-08 00:00:00',629.74,242,35676,5.75,'GEO Transfer' from dual union all
select 36035,'036035','2009-058D','Sylda 5A','Sylda 5 No. 37A',68739,'Reentered','2010-09-09 00:00:00','2009-11-06 00:00:00',631.21,249,35745,5.98,'GEO Transfer' from dual union all
select 36036,'036036','2009-059A','SMOS','SMOS',68740,'In Earth orbit','','2009-11-09 00:00:00',100.01,758,760,98.45,'Sun-Synch' from dual union all
select 36037,'036037','2009-059B','PROBA-2','Proba-2',68740,'In Earth orbit','','2009-11-09 00:00:00',99.17,710,727,98.31,'Sun-Synch' from dual union all
select 36038,'036038','2009-059C','PS adapter','PS adapter',68740,'In Earth orbit','','2009-11-09 00:00:00',99.09,709,722,98.31,'Sun-Synch' from dual union all
select 36039,'036039','2009-059D','Briz-KM No. 72513','Briz-KM 14S45',68740,'In Earth orbit','','2009-11-09 00:00:00',97.07,510,727,98.45,'Sun-Synch' from dual union all
select 3604,'003604','1968-112A','OPS 4740','CORONA J-50',16839,'Reentered Attached','1968-12-28 00:00:00','1968-12-19 00:00:00',88.64,178,237,81.03,'Intermediate' from dual union all
select 36040,'036040','1993-036AXB','deb Kosmos-2251','',65138,'Reentered','2011-11-01 00:00:00','2009-11-04 00:00:00',96.82,456,757,74.24,'Intermediate' from dual union all
select 36041,'036041','1993-036AXC','deb Kosmos-2251','',65138,'Reentered','2011-06-19 00:00:00','2009-11-04 00:00:00',97.16,497,748,74.05,'Intermediate' from dual union all
select 36042,'036042','1993-036AXD','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-03 00:00:00',98.42,595,771,73.88,'Intermediate' from dual union all
select 36043,'036043','1993-036AXE','deb Kosmos-2251','',65138,'Reentered','2010-12-03 00:00:00','2009-11-04 00:00:00',96.52,359,825,74.01,'Intermediate' from dual union all
select 36044,'036044','1993-036AXF','deb Kosmos-2251','',65138,'Reentered','2014-07-24 00:00:00','2009-11-04 00:00:00',100.11,757,771,74.03,'Intermediate' from dual union all
select 36045,'036045','1993-036AXG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-03 00:00:00',101.02,724,890,73.78,'Intermediate' from dual union all
select 36046,'036046','1993-036AXH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-04 00:00:00',99.44,679,785,74.02,'Intermediate' from dual union all
select 36047,'036047','1993-036AXJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-03 00:00:00',100.08,746,779,74.05,'Intermediate' from dual union all
select 36048,'036048','1993-036AXK','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-04 00:00:00',100.54,764,805,74.05,'Intermediate' from dual union all
select 36049,'036049','1993-036AXL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-03 00:00:00',100.32,761,787,74.04,'Intermediate' from dual union all
select 3605,'003605','1968-112B','OPS 7684','P801-1',16839,'In Earth orbit','','1969-01-15 00:00:00',114.43,1385,1474,80.36,'Intermediate' from dual union all
select 36050,'036050','1993-036AXM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-03 00:00:00',98.08,558,776,74.04,'Intermediate' from dual union all
select 36051,'036051','1993-036AXN','deb Kosmos-2251','',65138,'Reentered','2015-09-28 00:00:00','2009-11-04 00:00:00',100.77,761,830,74.05,'Intermediate' from dual union all
select 36052,'036052','1993-036AXP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-04 00:00:00',100.4,763,792,74.03,'Intermediate' from dual union all
select 36053,'036053','1993-036AXQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-03 00:00:00',100.85,766,832,73.88,'Intermediate' from dual union all
select 36054,'036054','1993-036AXR','deb Kosmos-2251','',65138,'Reentered','2011-06-02 00:00:00','2009-11-04 00:00:00',96.77,460,747,73.94,'Intermediate' from dual union all
select 36055,'036055','1993-036AXS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-04 00:00:00',104.76,758,1209,74.05,'Intermediate' from dual union all
select 36056,'036056','1993-036AXT','deb Kosmos-2251','',65138,'Reentered','2011-06-18 00:00:00','2009-11-03 00:00:00',95.76,369,742,73.93,'Intermediate' from dual union all
select 36057,'036057','1993-036AXU','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-04 00:00:00',99.22,663,780,73.98,'Intermediate' from dual union all
select 36058,'036058','1993-036AXV','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-04 00:00:00',99.85,709,794,74.02,'Intermediate' from dual union all
select 36059,'036059','1993-036AXW','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-04 00:00:00',101.3,766,875,74.01,'Intermediate' from dual union all
select 3606,'003606','1968-091Y','deb Kosmos-249','',16818,'In Earth orbit','','1993-08-06 00:00:00',112.08,672,1973,62.25,'Intermediate' from dual union all
select 36060,'036060','1993-036AXX','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-04 00:00:00',100.42,767,790,74.03,'Intermediate' from dual union all
select 36061,'036061','1993-036AXY','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-03 00:00:00',100.88,770,830,74.02,'Intermediate' from dual union all
select 36062,'036062','1993-036AXZ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-03 00:00:00',98.13,563,776,73.95,'Intermediate' from dual union all
select 36063,'036063','1993-036AYA','deb Kosmos-2251','',65138,'Reentered','2011-12-19 00:00:00','2009-11-04 00:00:00',97.28,497,760,74,'Intermediate' from dual union all
select 36064,'036064','1993-036AYB','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-04 00:00:00',100.93,742,864,73.85,'Intermediate' from dual union all
select 36065,'036065','1993-036AYC','deb Kosmos-2251','',65138,'Lost','','2009-11-04 00:00:00',96.83,453,761,74.02,'Intermediate' from dual union all
select 36066,'036066','1993-036AYD','deb Kosmos-2251','',65138,'Reentered','2015-03-12 00:00:00','2009-11-03 00:00:00',98.67,615,776,73.99,'Intermediate' from dual union all
select 36067,'036067','1993-036AYE','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-04 00:00:00',100.53,766,802,74.04,'Intermediate' from dual union all
select 36068,'036068','1993-036AYF','deb Kosmos-2251','',65138,'Reentered','2015-01-18 00:00:00','2009-11-04 00:00:00',101.28,637,1002,73.99,'Intermediate' from dual union all
select 36069,'036069','1993-036AYG','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-04 00:00:00',98.49,597,776,74.05,'Intermediate' from dual union all
select 3607,'003607','1968-091Z','deb Kosmos-249 [*]','',16818,'Reentered','1991-03-20 00:00:00','1991-03-20 00:00:00',88.6,192,219,62.58,'Intermediate' from dual union all
select 36070,'036070','1993-036AYH','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-04 00:00:00',100.38,763,791,74.04,'Intermediate' from dual union all
select 36071,'036071','1993-036AYJ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-03 00:00:00',100.65,761,818,74.04,'Intermediate' from dual union all
select 36072,'036072','1993-036AYK','deb Kosmos-2251','',65138,'Reentered','2011-08-21 00:00:00','2009-11-04 00:00:00',96.79,429,780,74.04,'Intermediate' from dual union all
select 36073,'036073','1993-036AYL','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-04 00:00:00',100.67,771,810,74.03,'Intermediate' from dual union all
select 36074,'036074','1993-036AYM','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-03 00:00:00',101.2,772,859,73.99,'Intermediate' from dual union all
select 36075,'036075','1993-036AYN','deb Kosmos-2251','',65138,'Reentered','2014-09-15 00:00:00','2009-11-04 00:00:00',99.47,606,861,74,'Intermediate' from dual union all
select 36076,'036076','1993-036AYP','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-03 00:00:00',100.41,723,833,73.85,'Intermediate' from dual union all
select 36077,'036077','1993-036AYQ','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-04 00:00:00',98.74,612,785,73.99,'Intermediate' from dual union all
select 36078,'036078','1993-036AYR','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-03 00:00:00',100.31,758,789,74.05,'Intermediate' from dual union all
select 36079,'036079','1993-036AYS','deb Kosmos-2251','',65138,'In Earth orbit','','2009-11-03 00:00:00',99,642,780,74.06,'Intermediate' from dual union all
select 3608,'003608','1968-091AA','deb Kosmos-249','',16818,'Reentered','2012-08-13 00:00:00','1992-02-02 00:00:00',103.96,464,1428,62.51,'Intermediate' from dual union all
select 36080,'036080','1997-051VM','deb Iridium 33','',66131,'In Earth orbit','','2009-11-03 00:00:00',100.53,769,799,86.33,'Polar' from dual union all
select 36081,'036081','1997-051VN','deb Iridium 33','',66131,'In Earth orbit','','2009-11-04 00:00:00',101.61,777,893,86.26,'Polar' from dual union all
select 36082,'036082','1997-051VP','deb Iridium 33','',66131,'Reentered','2013-02-25 00:00:00','2009-11-03 00:00:00',100.88,759,841,86.42,'Polar' from dual union all
select 36083,'036083','1997-051VQ','deb Iridium 33','',66131,'In Earth orbit','','2009-11-04 00:00:00',100.67,767,814,86.35,'Polar' from dual union all
select 36084,'036084','1997-051VR','deb Iridium 33','',66131,'Reentered','2014-03-14 00:00:00','2009-11-03 00:00:00',99.76,719,775,86.3,'Polar' from dual union all
select 36085,'036085','1997-051VS','deb Iridium 33','',66131,'Reentered','2010-02-17 00:00:00','2009-11-04 00:00:00',103.8,666,1210,86.46,'Polar' from dual union all
select 36086,'036086','2009-060A','Poisk','ISS MIM-2',68741,'In Earth orbit','','2009-11-18 00:00:00',91.33,336,344,51.65,'Intermediate' from dual union all
select 36087,'036087','2009-060B','11S510','11S510',68741,'Reentered','2009-11-14 00:00:00','2009-11-12 00:00:00',88.17,169,199,51.61,'Intermediate' from dual union all
select 36088,'036088','2009-061A','Shi Jian 11-01','shijian shi yihao 01 xin',68742,'In Earth orbit','','2009-11-19 00:00:00',98.68,689,702,98.28,'Sun-Synch' from dual union all
select 36089,'036089','2009-061B','CZ-2C Stage 2','CZ-2C Stage 2',68742,'In Earth orbit','','2009-11-19 00:00:00',99.41,672,789,98.34,'Sun-Synch' from dual union all
select 3609,'003609','1968-091AB','deb Kosmos-249','',16818,'Reentered','1982-12-26 00:00:00','1982-11-18 00:00:00',110.98,602,1942,62.46,'Intermediate' from dual union all
select 36090,'036090','2009-061C','CZ-2C Stage 2 motor cover','',68742,'In Earth orbit','','2009-11-19 00:00:00',99.41,672,789,98.34,'Sun-Synch' from dual union all
select 36091,'036091','2009-061D','CZ-2C Stage 2 motor cover','',68742,'In Earth orbit','','2009-11-20 00:00:00',99.43,664,799,98.02,'Sun-Synch' from dual union all
select 36092,'036092','2009-061E','CZ-2C Stage 2 motor cover','',68742,'In Earth orbit','','2009-11-19 00:00:00',101.22,699,933,98.52,'Sun-Synch' from dual union all
select 36093,'036093','2009-061F','CZ-2C Stage 2 motor cover','',68742,'In Earth orbit','','2009-11-20 00:00:00',101.2,698,933,98.1,'Sun-Synch' from dual union all
select 36094,'036094','2009-062A','Atlantis (STS-129)','OV-104',68743,'Landed','2009-11-27 00:00:00','2009-11-24 00:00:00',91.31,335,343,51.64,'Intermediate' from dual union all
select 36095,'036095','2009-063A','Kosmos-2455','Lotos-S No.  801',68744,'In Earth orbit','','2009-11-22 00:00:00',95.72,204,903,67.16,'Intermediate' from dual union all
select 36096,'036096','2009-063B','11S510','11S510',68744,'Reentered','2010-04-07 00:00:00','2009-11-22 00:00:00',95.57,201,892,67.17,'Intermediate' from dual union all
select 36097,'036097','2009-064A','Intelsat IS-14','IS-14',68745,'In Earth orbit','','2009-11-30 00:00:00',1444.09,35092,36794,.05,'Drift GEO' from dual union all
select 36098,'036098','2009-064B','Centaur AV-024','Centaur AV-024',68745,'In Earth orbit','','2009-12-03 00:00:00',816.62,6225,38900,22.94,'Highly Ellip' from dual;
insert into SATELLITE
select 36099,'036099','2009-057E','DMSP F-18 OLS Cover','',68738,'In Earth orbit','','2009-11-23 00:00:00',101.94,843,857,98.93,'Sun-Synch' from dual union all
select 361,'000361','62 A Lambda','FTV 2404','SAMOS E-6 No. 4',8145,'Reentered Attached','1962-08-06 00:00:00','1962-08-05 00:00:00',88.53,202,202,96.3,'Sun-Synch' from dual union all
select 3610,'003610','1968-097AA','deb Kosmos-252','',16824,'Reentered','1991-08-24 00:00:00','1991-07-30 00:00:00',93.27,388,481,62.2,'Intermediate' from dual union all
select 36100,'036100','2009-057F','DMSP F-18 OLS Cooler Cover','',68738,'In Earth orbit','','2009-11-23 00:00:00',101.92,842,858,98.93,'Sun-Synch' from dual union all
select 36101,'036101','2009-065A','Eutelsat W7','Eutelsat W7',68746,'In Earth orbit','','2009-12-16 00:00:00',1436.06,35785,35787,.07,'Stationary' from dual union all
select 36102,'036102','2009-065B','Briz-M No. 99509','Briz-M',68746,'In Earth orbit','','2009-12-04 00:00:00',656.28,3960,33316,21.19,'GEO Transfer' from dual union all
select 36103,'036103','2009-065C','Briz-M DTB','Briz-M torus tank',68746,'In Earth orbit','','2009-12-01 00:00:00',289.94,323,15943,49.5,'Highly Ellip' from dual union all
select 36104,'036104','2009-066A','JSE kougaku-3 gouki','IGS Optical-3',68747,'In Earth orbit','','2009-12-01 00:00:00',94.3,480,490,97.3,'Sun-Synch' from dual union all
select 36105,'036105','2009-066B','H-2A F16 Stage 2','H-2A Stage 2',68747,'In Earth orbit','','2009-12-01 00:00:00',93.38,380,500,97.3,'Sun-Synch' from dual union all
select 36106,'036106','2009-067A','Intelsat IS-15','IS-15',68748,'In Earth orbit','','2009-12-09 00:00:00',1435.09,35737,35797,.05,'Drift GEO' from dual union all
select 36107,'036107','2009-067B','Blok DM-SLB No. 4L','Blok DM-SLB No. 4L',68748,'In Earth orbit','','2009-12-08 00:00:00',829.52,9964,35769,11.94,'Medium' from dual union all
select 36108,'036108','2009-068A','WGS SV-3','Wideband Global Satcom 3',68749,'In Earth orbit','','2010-01-20 00:00:00',1436.27,35780,35800,.1,'Stationary' from dual union all
select 36109,'036109','2009-068B','Delta 346','DCSS-5 F04',68749,'Lost','','2009-12-06 00:00:00',1331.32,439,66981,24,'Highly Ellip' from dual union all
select 3611,'003611','1966-025E','deb OV1-5','',13432,'In Earth orbit','','1967-11-26 00:00:00',105.56,973,1069,144.6,'Retrograde' from dual union all
select 36110,'036110','2009-069A','Yaogan Weixing 7','Jianbing-6 03 xing',68750,'In Earth orbit','','2009-12-16 00:00:00',97.54,623,659,97.84,'Sun-Synch' from dual union all
select 36111,'036111','2009-070A','Kosmos-2456','Uragan-M No. 730',68751,'In Earth orbit','','2009-12-21 00:00:00',675.73,19128,19132,64.81,'Medium' from dual union all
select 36112,'036112','2009-070B','Kosmos-2457','Uragan-M No. 733',68751,'In Earth orbit','','2009-12-16 00:00:00',675.73,19125,19135,64.79,'Medium' from dual union all
select 36113,'036113','2009-070C','Kosmos-2458','Uragan-M No. 734',68751,'In Earth orbit','','2009-12-16 00:00:00',675.79,19120,19143,64.81,'Medium' from dual union all
select 36114,'036114','2009-070D','Perekhodnik','Sredniy perekhodnik',68751,'Reentered','2009-12-15 00:00:00','2009-12-15 00:00:00',87.76,145,183,64.78,'Intermediate' from dual union all
select 36115,'036115','2009-070E','SOZ','SOZ',68751,'In Earth orbit','','2009-12-18 00:00:00',340.28,396,19139,64.85,'Highly Ellip' from dual union all
select 36116,'036116','2009-070F','SOZ','SOZ',68751,'In Earth orbit','','2009-12-18 00:00:00',340.26,400,19134,64.74,'Highly Ellip' from dual union all
select 36117,'036117','2009-070G','8S812','8S812',68751,'Reentered','2009-12-15 00:00:00','2009-12-14 00:00:00',87.64,147,169,64.81,'Intermediate' from dual union all
select 36118,'036118','2009-070H','Blok DM-2 No. 115L','11S861 No. 115L',68751,'In Earth orbit','','2009-12-16 00:00:00',674.92,19085,19134,64.8,'Medium' from dual union all
select 36119,'036119','2009-071A','WISE','Wide Field IR Explorer',68752,'In Earth orbit','','2009-12-16 00:00:00',95.2,526,530,97.5,'Sun-Synch' from dual union all
select 3612,'003612','1968-113A','Kosmos-259','DS-U2-I No. 3',16840,'Reentered','1969-05-05 00:00:00','1969-01-14 00:00:00',99,212,1210,48.42,'Intermediate' from dual union all
select 36120,'036120','2009-071B','Delta 347','Delta SSPS AJ-10-118K',68752,'In Earth orbit','','2009-12-16 00:00:00',245.21,2113,11085,97.52,'Medium' from dual union all
select 36121,'036121','2009-072A','Yaogan Weixing 8','Jianbing-9 01 xing',68753,'In Earth orbit','','2009-12-17 00:00:00',109.37,1192,1204,100.5,'Sun-Synch' from dual union all
select 36122,'036122','2009-072B','Hope-OSCAR-68 (Xi Wang 1)','xiwang yihao',68753,'In Earth orbit','','2009-12-17 00:00:00',109.38,1193,1204,100.5,'Sun-Synch' from dual union all
select 36123,'036123','2009-072C','CZ-4C Stage 3','CZ-4C Stage 3',68753,'In Earth orbit','','2009-12-17 00:00:00',106.97,969,1204,100.43,'Sun-Synch' from dual union all
select 36124,'036124','2009-073A','Helios IIB','Helios 2B',68754,'In Earth orbit','','2009-12-19 00:00:00',98.3,670,685,98.1,'Sun-Synch' from dual union all
select 36125,'036125','2009-073B','EPS 532','EPS',68754,'In Earth orbit','','2009-12-19 00:00:00',97.52,600,680,98.1,'Sun-Synch' from dual union all
select 36126,'036126','2009-070J','deb Proton','',68751,'Reentered','2009-12-17 00:00:00','2009-12-17 00:00:00',88.99,151,299,64.83,'Intermediate' from dual union all
select 36127,'036127','2009-070K','deb Proton','',68751,'Reentered','2009-12-19 00:00:00','2009-12-18 00:00:00',88.78,151,278,64.67,'Intermediate' from dual union all
select 36128,'036128','2009-070L','deb Proton','',68751,'Reentered','2009-12-18 00:00:00','2009-12-18 00:00:00',88.96,156,291,64.95,'Intermediate' from dual union all
select 36129,'036129','2009-074A','Soyuz TMA-17','Soyuz 11F732A17 No. 227',68755,'Landed','2010-06-02 00:00:00','2009-12-23 00:00:00',91.3,333,344,51.64,'Intermediate' from dual union all
select 3613,'003613','1968-113B','S1M','S1M',16840,'Reentered','1969-05-10 00:00:00','1969-01-15 00:00:00',98.76,214,1185,48.43,'Intermediate' from dual union all
select 36130,'036130','2009-074B','11S510','11S510',68755,'Reentered','2009-12-24 00:00:00','2009-12-20 00:00:00',88.78,193,235,51.64,'Intermediate' from dual union all
select 36131,'036131','2009-075A','DirecTV 12','DirecTV-12',68756,'In Earth orbit','','2010-01-21 00:00:00',1436.7,33113,38484,.2,'Near-synch' from dual union all
select 36132,'036132','1987-011K','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-01-22 00:00:00',100.6,775,799,65.01,'Intermediate' from dual union all
select 36133,'036133','2009-075C','Briz-M DTB','Briz-M torus tank',68756,'In Earth orbit','','2010-01-05 00:00:00',281.44,312,15384,46.03,'Highly Ellip' from dual union all
select 36134,'036134','2009-071C','WISE Cryostat Cover','',68752,'Reentered','2014-08-26 00:00:00','2009-12-29 00:00:00',95.21,525,533,97.5,'Sun-Synch' from dual union all
select 36135,'036135','1999-025DRG','deb FY-1C','',66706,'Reentered','2018-02-22 00:00:00','2010-01-11 00:00:00',100.87,736,864,98.96,'Sun-Synch' from dual union all
select 36136,'036136','1999-025DRH','deb FY-1C','',66706,'In Earth orbit','','2010-01-08 00:00:00',100.87,736,864,98.96,'Sun-Synch' from dual union all
select 36137,'036137','1999-025DRJ','deb FY-1C','',66706,'In Earth orbit','','2010-01-08 00:00:00',100.88,737,863,98.96,'Sun-Synch' from dual union all
select 36138,'036138','1999-025DRK','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',102.07,814,900,99.18,'Sun-Synch' from dual union all
select 36139,'036139','1999-025DRL','deb FY-1C','',66706,'In Earth orbit','','2009-12-25 00:00:00',103.92,820,1068,97.93,'Sun-Synch' from dual union all
select 3614,'003614','1968-113C','deb Kosmos-259','',16840,'Reentered','1969-05-13 00:00:00','1968-12-15 00:00:00',99.68,213,1274,48.4,'Intermediate' from dual union all
select 36140,'036140','1999-025DRM','deb FY-1C','',66706,'In Earth orbit','','2010-01-07 00:00:00',100.2,575,960,98.53,'Sun-Synch' from dual union all
select 36141,'036141','1999-025DRN','deb FY-1C','',66706,'In Earth orbit','','2009-12-30 00:00:00',103.83,813,1067,98.48,'Sun-Synch' from dual union all
select 36142,'036142','1999-025DRP','deb FY-1C','',66706,'In Earth orbit','','2010-01-08 00:00:00',105.51,838,1199,98.39,'Sun-Synch' from dual union all
select 36143,'036143','1999-025DRQ','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',101.05,534,1083,98.38,'Sun-Synch' from dual union all
select 36144,'036144','1999-025DRR','deb FY-1C','',66706,'In Earth orbit','','2009-12-30 00:00:00',103.47,819,1027,98.44,'Sun-Synch' from dual union all
select 36145,'036145','1999-025DRS','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',100.6,723,851,98.99,'Sun-Synch' from dual union all
select 36146,'036146','1999-025DRT','deb FY-1C','',66706,'In Earth orbit','','2010-01-07 00:00:00',100.34,594,956,97.87,'Sun-Synch' from dual union all
select 36147,'036147','1999-025DRU','deb FY-1C','',66706,'In Earth orbit','','2009-12-23 00:00:00',107.99,770,1498,98.5,'Sun-Synch' from dual union all
select 36148,'036148','1999-025DRV','deb FY-1C','',66706,'In Earth orbit','','2010-01-01 00:00:00',106.7,793,1355,98.41,'Sun-Synch' from dual union all
select 36149,'036149','1999-025DRW','deb FY-1C','',66706,'In Earth orbit','','2010-01-09 00:00:00',102.27,842,890,99.61,'Sun-Synch' from dual union all
select 3615,'003615','1968-114A','ESSA 8','TOS F',16841,'In Earth orbit','','1969-02-01 00:00:00',114.65,1414,1465,101.89,'Sun-Synch' from dual union all
select 36150,'036150','1999-025DRX','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',105.07,791,1205,98.21,'Sun-Synch' from dual union all
select 36151,'036151','1999-025DRY','deb FY-1C','',66706,'Reentered','2014-12-01 00:00:00','2010-01-09 00:00:00',99.06,514,914,97.5,'Sun-Synch' from dual union all
select 36152,'036152','1999-025DRZ','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',102.64,838,929,99.09,'Sun-Synch' from dual union all
select 36153,'036153','1999-025DSA','deb FY-1C','',66706,'In Earth orbit','','2010-01-06 00:00:00',101.69,803,875,98.92,'Sun-Synch' from dual union all
select 36154,'036154','1999-025DSB','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',105.63,761,1287,97.72,'Sun-Synch' from dual union all
select 36155,'036155','1999-025DSC','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',102.46,851,900,98.89,'Sun-Synch' from dual union all
select 36156,'036156','1999-025DSD','deb FY-1C','',66706,'In Earth orbit','','2009-12-23 00:00:00',101.77,800,885,98.91,'Sun-Synch' from dual union all
select 36157,'036157','1999-025DSE','deb FY-1C','',66706,'Reentered','2015-04-29 00:00:00','2010-01-03 00:00:00',100.75,628,961,97.92,'Sun-Synch' from dual union all
select 36158,'036158','1999-025DSF','deb FY-1C','',66706,'Reentered','2016-12-25 00:00:00','2010-01-11 00:00:00',100.66,583,997,98.68,'Sun-Synch' from dual union all
select 36159,'036159','1999-025DSG','deb FY-1C','',66706,'In Earth orbit','','2010-01-07 00:00:00',100.45,484,1076,99.5,'Sun-Synch' from dual union all
select 3616,'003616','1968-114B','Delta 62','DSV-3E-3 62',16841,'In Earth orbit','','1969-01-31 00:00:00',114.55,1411,1458,101.89,'Sun-Synch' from dual union all
select 36160,'036160','1999-025DSH','deb FY-1C','',66706,'In Earth orbit','','2009-12-30 00:00:00',102.2,761,964,98.25,'Sun-Synch' from dual union all
select 36161,'036161','1999-025DSJ','deb FY-1C','',66706,'In Earth orbit','','2010-01-03 00:00:00',100.47,607,955,97.74,'Sun-Synch' from dual union all
select 36162,'036162','1999-025DSK','deb FY-1C','',66706,'In Earth orbit','','2010-01-08 00:00:00',105.6,754,1290,97.64,'Sun-Synch' from dual union all
select 36163,'036163','1999-025DSL','deb FY-1C','',66706,'In Earth orbit','','2009-12-30 00:00:00',102.78,826,955,99.15,'Sun-Synch' from dual union all
select 36164,'036164','1999-025DSM','deb FY-1C','',66706,'Reentered','2011-11-06 00:00:00','2010-01-09 00:00:00',98.57,480,901,97.27,'Sun-Synch' from dual union all
select 36165,'036165','1999-025DSN','deb FY-1C','',66706,'In Earth orbit','','2010-01-05 00:00:00',103.63,778,1083,98.03,'Sun-Synch' from dual union all
select 36166,'036166','1999-025DSP','deb FY-1C','',66706,'In Earth orbit','','2010-01-09 00:00:00',106.62,787,1354,97.58,'Sun-Synch' from dual union all
select 36167,'036167','1999-025DSQ','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',100.82,730,865,98.95,'Sun-Synch' from dual union all
select 36168,'036168','1999-025DSR','deb FY-1C','',66706,'Reentered','2015-02-03 00:00:00','2009-12-25 00:00:00',100.65,615,964,97.9,'Sun-Synch' from dual union all
select 36169,'036169','1999-025DSS','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',100.67,626,955,98.04,'Sun-Synch' from dual union all
select 3617,'003617','1968-112C','OPS 7684 sensor cover','',16839,'In Earth orbit','','1969-02-21 00:00:00',114.07,1378,1449,80.17,'Intermediate' from dual union all
select 36170,'036170','1999-025DST','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',104.13,848,1059,97.99,'Sun-Synch' from dual union all
select 36171,'036171','1999-025DSU','deb FY-1C','',66706,'Reentered','2012-11-12 00:00:00','2010-01-11 00:00:00',98.11,520,817,98.66,'Sun-Synch' from dual union all
select 36172,'036172','1999-025DSV','deb FY-1C','',66706,'In Earth orbit','','2010-01-03 00:00:00',104.85,815,1160,98.16,'Sun-Synch' from dual union all
select 36173,'036173','1999-025DSW','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',101.48,713,945,98.27,'Sun-Synch' from dual union all
select 36174,'036174','1999-025DSX','deb FY-1C','',66706,'Reentered','2015-07-01 00:00:00','2010-01-11 00:00:00',100.8,608,985,98.36,'Sun-Synch' from dual union all
select 36175,'036175','1999-025DSY','deb FY-1C','',66706,'In Earth orbit','','2010-01-07 00:00:00',100.45,520,1040,97.35,'Sun-Synch' from dual union all
select 36176,'036176','1999-025DSZ','deb FY-1C','',66706,'Reentered','2012-05-29 00:00:00','2010-01-08 00:00:00',98.83,483,923,98.48,'Sun-Synch' from dual union all
select 36177,'036177','1999-025DTA','deb FY-1C','',66706,'In Earth orbit','','2010-01-03 00:00:00',102.46,842,908,99.16,'Sun-Synch' from dual union all
select 36178,'036178','1999-025DTB','deb FY-1C','',66706,'Reentered','2011-05-26 00:00:00','2010-01-08 00:00:00',94.97,352,683,97.69,'Sun-Synch' from dual union all
select 36179,'036179','1999-025DTC','deb FY-1C','',66706,'Reentered','2012-04-19 00:00:00','2010-01-08 00:00:00',100.33,436,1113,98.4,'Sun-Synch' from dual union all
select 3618,'003618','1968-112D','OPS 7684 sensor cover','',16839,'In Earth orbit','','1969-02-21 00:00:00',114.77,1377,1513,80.53,'Intermediate' from dual union all
select 36180,'036180','1999-025DTD','deb FY-1C','',66706,'In Earth orbit','','2009-12-31 00:00:00',102.18,688,1036,98.49,'Sun-Synch' from dual union all
select 36181,'036181','1999-025DTE','deb FY-1C','',66706,'In Earth orbit','','2010-01-08 00:00:00',100.84,736,861,98.98,'Sun-Synch' from dual union all
select 36182,'036182','1999-025DTF','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',102.03,835,875,99.2,'Sun-Synch' from dual union all
select 36183,'036183','1999-025DTG','deb FY-1C','',66706,'Reentered','2012-03-30 00:00:00','2010-01-10 00:00:00',96.64,374,822,98.59,'Sun-Synch' from dual union all
select 36184,'036184','1999-025DTH','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',103.58,816,1039,98.64,'Sun-Synch' from dual union all
select 36185,'036185','1999-025DTJ','deb FY-1C','',66706,'Reentered','2014-07-17 00:00:00','2009-12-29 00:00:00',100.13,685,845,99.01,'Sun-Synch' from dual union all
select 36186,'036186','1999-025DTK','deb FY-1C','',66706,'In Earth orbit','','2010-01-01 00:00:00',102.59,830,933,99.07,'Sun-Synch' from dual union all
select 36187,'036187','1999-025DTL','deb FY-1C','',66706,'Reentered','2011-09-23 00:00:00','2010-01-05 00:00:00',97.74,459,843,97.76,'Sun-Synch' from dual union all
select 36188,'036188','1999-025DTM','deb FY-1C','',66706,'In Earth orbit','','2010-01-09 00:00:00',101.62,770,901,98.86,'Sun-Synch' from dual;
insert into SATELLITE
select 36189,'036189','1999-025DTN','deb FY-1C','',66706,'In Earth orbit','','2009-12-25 00:00:00',100.35,700,851,98.96,'Sun-Synch' from dual union all
select 3619,'003619','1968-115A','Kosmos-260','Molniya-1Yu 11F67 No. 15',16842,'Reentered','1973-07-09 00:00:00','1969-01-18 00:00:00',712.23,522,39559,64.99,'GEO Transfer' from dual union all
select 36190,'036190','1999-025DTP','deb FY-1C','',66706,'In Earth orbit','','2010-01-08 00:00:00',101.95,812,890,98.96,'Sun-Synch' from dual union all
select 36191,'036191','1999-025DTQ','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',100.71,729,855,98.96,'Sun-Synch' from dual union all
select 36192,'036192','1999-025DTR','deb FY-1C','',66706,'In Earth orbit','','2009-12-29 00:00:00',101.01,652,961,98.84,'Sun-Synch' from dual union all
select 36193,'036193','1999-025DTS','deb FY-1C','',66706,'Reentered','2012-06-20 00:00:00','2010-01-01 00:00:00',98.31,504,852,98.42,'Sun-Synch' from dual union all
select 36194,'036194','1999-025DTT','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',100.98,756,855,99.06,'Sun-Synch' from dual union all
select 36195,'036195','1999-025DTU','deb FY-1C','',66706,'Reentered','2014-04-17 00:00:00','2010-01-09 00:00:00',99.16,596,841,99,'Sun-Synch' from dual union all
select 36196,'036196','1999-025DTV','deb FY-1C','',66706,'In Earth orbit','','2010-01-01 00:00:00',102.7,852,920,99.19,'Sun-Synch' from dual union all
select 36197,'036197','1999-025DTW','deb FY-1C','',66706,'In Earth orbit','','2010-01-01 00:00:00',102.54,828,930,99.14,'Sun-Synch' from dual union all
select 36198,'036198','1999-025DTX','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',103.66,850,1013,98.69,'Sun-Synch' from dual union all
select 36199,'036199','1999-025DTY','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',101.13,741,884,98.97,'Sun-Synch' from dual union all
select 362,'000362','62 A Kappa 2','deb USAF satellite/Agena D','CORONA experiment cover',8144,'Reentered','1962-08-08 00:00:00','1962-08-06 00:00:00',91.67,200,513,82.2,'Intermediate' from dual union all
select 3620,'003620','1968-115B','BOZ','BOZ',16842,'Reentered','1969-02-06 00:00:00','1968-12-30 00:00:00',91.53,236,464,64.97,'Intermediate' from dual union all
select 36200,'036200','1999-025DTZ','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',102.89,833,958,99.42,'Sun-Synch' from dual union all
select 36201,'036201','1999-025DUA','deb FY-1C','',66706,'Reentered','2018-05-02 00:00:00','2010-01-03 00:00:00',97.73,456,844,99.37,'Sun-Synch' from dual union all
select 36202,'036202','1999-025DUB','deb FY-1C','',66706,'Reentered','2014-11-26 00:00:00','2010-01-07 00:00:00',100.25,649,892,99.11,'Sun-Synch' from dual union all
select 36203,'036203','1999-025DUC','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',99.89,648,859,99.09,'Sun-Synch' from dual union all
select 36204,'036204','1999-025DUD','deb FY-1C','',66706,'In Earth orbit','','2010-01-01 00:00:00',101.86,827,867,99.34,'Sun-Synch' from dual union all
select 36205,'036205','1999-025DUE','deb FY-1C','',66706,'In Earth orbit','','2010-01-04 00:00:00',102.08,786,929,99.45,'Sun-Synch' from dual union all
select 36206,'036206','1999-025DUF','deb FY-1C','',66706,'In Earth orbit','','2010-01-01 00:00:00',101.97,827,877,99.11,'Sun-Synch' from dual union all
select 36207,'036207','1999-025DUG','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',104.46,837,1102,98.62,'Sun-Synch' from dual union all
select 36208,'036208','1999-025DUH','deb FY-1C','',66706,'Reentered','2012-02-26 00:00:00','2010-01-09 00:00:00',97.73,493,808,98.89,'Sun-Synch' from dual union all
select 36209,'036209','1999-025DUJ','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',103.28,850,978,98.91,'Sun-Synch' from dual union all
select 3621,'003621','1968-115C','11S510','11S510',16842,'Reentered','1969-01-18 00:00:00','1968-12-23 00:00:00',91.29,229,447,64.99,'Intermediate' from dual union all
select 36210,'036210','1999-025DUK','deb FY-1C','',66706,'Reentered','2013-03-05 00:00:00','2010-01-07 00:00:00',97.12,414,828,99.83,'Sun-Synch' from dual union all
select 36211,'036211','1999-025DUL','deb FY-1C','',66706,'In Earth orbit','','2010-01-07 00:00:00',101.87,626,1068,99.02,'Sun-Synch' from dual union all
select 36212,'036212','1999-025DUM','deb FY-1C','',66706,'Reentered','2014-02-10 00:00:00','2010-01-01 00:00:00',98.44,453,916,99.07,'Sun-Synch' from dual union all
select 36213,'036213','1999-025DUN','deb FY-1C','',66706,'Reentered','2013-02-14 00:00:00','2010-01-01 00:00:00',98.57,545,836,99.07,'Sun-Synch' from dual union all
select 36214,'036214','1999-025DUP','deb FY-1C','',66706,'In Earth orbit','','2009-12-30 00:00:00',102.51,829,925,99.13,'Sun-Synch' from dual union all
select 36215,'036215','1999-025DUQ','deb FY-1C','',66706,'Reentered','2011-11-04 00:00:00','2010-01-09 00:00:00',96.94,443,782,98.77,'Sun-Synch' from dual union all
select 36216,'036216','1999-025DUR','deb FY-1C','',66706,'In Earth orbit','','2010-01-01 00:00:00',101.87,812,883,98.84,'Sun-Synch' from dual union all
select 36217,'036217','1999-025DUS','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',100.29,653,891,99.11,'Sun-Synch' from dual union all
select 36218,'036218','1999-025DUT','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',100.15,678,854,99.05,'Sun-Synch' from dual union all
select 36219,'036219','1999-025DUU','deb FY-1C','',66706,'In Earth orbit','','2010-01-08 00:00:00',100.29,653,891,99.06,'Sun-Synch' from dual union all
select 3622,'003622','1968-115D','Blok-L No. Ya716-87','Blok-L',16842,'Reentered','1973-09-21 00:00:00','1969-01-20 00:00:00',708.66,513,39390,64.94,'GEO Transfer' from dual union all
select 36220,'036220','1999-025DUV','deb FY-1C','',66706,'Reentered','2013-02-15 00:00:00','2010-01-01 00:00:00',97.6,463,825,98.7,'Sun-Synch' from dual union all
select 36221,'036221','1999-025DUW','deb FY-1C','',66706,'Reentered','2012-02-26 00:00:00','2010-01-08 00:00:00',97.81,500,807,98.89,'Sun-Synch' from dual union all
select 36222,'036222','1999-025DUX','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',101.69,791,887,98.92,'Sun-Synch' from dual union all
select 36223,'036223','1999-025DUY','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',103.46,810,1035,98.37,'Sun-Synch' from dual union all
select 36224,'036224','1999-025DUZ','deb FY-1C','',66706,'In Earth orbit','','2010-01-07 00:00:00',102.06,772,940,98.77,'Sun-Synch' from dual union all
select 36225,'036225','1999-025DVA','deb FY-1C','',66706,'In Earth orbit','','2010-01-09 00:00:00',99.96,656,858,98.88,'Sun-Synch' from dual union all
select 36226,'036226','1999-025DVB','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',103.39,833,1005,98.67,'Sun-Synch' from dual union all
select 36227,'036227','1999-025DVC','deb FY-1C','',66706,'Reentered','2012-07-18 00:00:00','2010-01-11 00:00:00',96.99,429,800,99.57,'Sun-Synch' from dual union all
select 36228,'036228','1999-025DVD','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',101.21,692,940,99.29,'Sun-Synch' from dual union all
select 36229,'036229','1999-025DVE','deb FY-1C','',66706,'In Earth orbit','','2010-01-01 00:00:00',103.89,841,1044,100.93,'Sun-Synch' from dual union all
select 3623,'003623','1968-116A','INTELSAT III F-2','INTELSAT III F-2',16843,'In Earth orbit','','1969-12-01 00:00:00',1436.06,35771,35801,.32,'Stationary' from dual union all
select 36230,'036230','1999-025DVF','deb FY-1C','',66706,'In Earth orbit','','2010-01-01 00:00:00',105.82,854,1212,99.79,'Sun-Synch' from dual union all
select 36231,'036231','1999-025DVG','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',102.1,814,903,99.11,'Sun-Synch' from dual union all
select 36232,'036232','1999-025DVH','deb FY-1C','',66706,'In Earth orbit','','2010-01-01 00:00:00',102.25,777,954,99.11,'Sun-Synch' from dual union all
select 36233,'036233','1999-025DVJ','deb FY-1C','',66706,'Reentered','2011-10-28 00:00:00','2010-01-07 00:00:00',97.53,461,820,99.25,'Sun-Synch' from dual union all
select 36234,'036234','1999-025DVK','deb FY-1C','',66706,'In Earth orbit','','2009-12-29 00:00:00',107.22,857,1338,99,'Sun-Synch' from dual union all
select 36235,'036235','1999-025DVL','deb FY-1C','',66706,'Reentered','2011-04-08 00:00:00','2010-01-09 00:00:00',96.77,422,786,98.44,'Sun-Synch' from dual union all
select 36236,'036236','1999-025DVM','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',104.42,854,1081,98.95,'Sun-Synch' from dual union all
select 36237,'036237','1999-025DVN','deb FY-1C','',66706,'In Earth orbit','','2010-01-07 00:00:00',100.73,721,866,98.98,'Sun-Synch' from dual union all
select 36238,'036238','1999-025DVP','deb FY-1C','',66706,'In Earth orbit','','2010-01-05 00:00:00',102.02,840,868,99.12,'Sun-Synch' from dual union all
select 36239,'036239','1999-025DVQ','deb FY-1C','',66706,'Reentered','2014-10-08 00:00:00','2010-01-08 00:00:00',100.13,646,884,99.07,'Sun-Synch' from dual union all
select 3624,'003624','1968-117A','Kosmos-261','DS-U2-GK No. 1',16844,'Reentered','1969-02-12 00:00:00','1968-12-29 00:00:00',92.68,201,611,71.04,'Intermediate' from dual union all
select 36240,'036240','1999-025DVR','deb FY-1C','',66706,'In Earth orbit','','2010-01-08 00:00:00',101.9,584,1114,99.4,'Sun-Synch' from dual union all
select 36241,'036241','1999-025DVS','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',99.32,474,979,98.49,'Sun-Synch' from dual union all
select 36242,'036242','1999-025DVT','deb FY-1C','',66706,'In Earth orbit','','2009-12-29 00:00:00',103.2,855,965,98.16,'Sun-Synch' from dual union all
select 36243,'036243','1999-025DVU','deb FY-1C','',66706,'In Earth orbit','','2010-01-05 00:00:00',98.81,546,858,99.2,'Sun-Synch' from dual union all
select 36244,'036244','1999-025DVV','deb FY-1C','',66706,'In Earth orbit','','2010-01-04 00:00:00',102.63,838,929,99.2,'Sun-Synch' from dual union all
select 36245,'036245','1999-025DVW','deb FY-1C','',66706,'Reentered','2014-07-23 00:00:00','2010-01-09 00:00:00',99.65,639,845,98.91,'Sun-Synch' from dual union all
select 36246,'036246','1999-025DVX','deb FY-1C','',66706,'In Earth orbit','','2010-01-06 00:00:00',102.39,773,971,98.99,'Sun-Synch' from dual union all
select 36247,'036247','1999-025DVY','deb FY-1C','',66706,'In Earth orbit','','2010-01-08 00:00:00',101.14,661,965,98.95,'Sun-Synch' from dual union all
select 36248,'036248','1999-025DVZ','deb FY-1C','',66706,'In Earth orbit','','2010-01-05 00:00:00',104.88,817,1160,98.15,'Sun-Synch' from dual union all
select 36249,'036249','1999-025DWA','deb FY-1C','',66706,'Reentered','2010-05-24 00:00:00','2010-01-02 00:00:00',93.81,330,592,104.73,'Retrograde' from dual union all
select 3625,'003625','1968-117B','S1M','S1M',16844,'Reentered','1969-01-07 00:00:00','1968-12-26 00:00:00',92.1,188,567,71.01,'Intermediate' from dual union all
select 36250,'036250','1999-025DWB','deb FY-1C','',66706,'In Earth orbit','','2010-01-06 00:00:00',100.68,731,851,99.09,'Sun-Synch' from dual union all
select 36251,'036251','1999-025DWC','deb FY-1C','',66706,'Reentered','2014-02-04 00:00:00','2010-01-10 00:00:00',97.17,410,837,99.11,'Sun-Synch' from dual union all
select 36252,'036252','1999-025DWD','deb FY-1C','',66706,'Reentered','2018-02-04 00:00:00','2010-01-09 00:00:00',99.23,494,950,98.53,'Sun-Synch' from dual union all
select 36253,'036253','1999-025DWE','deb FY-1C','',66706,'Reentered','2014-03-26 00:00:00','2010-01-10 00:00:00',98.28,513,840,98.77,'Sun-Synch' from dual union all
select 36254,'036254','1999-025DWF','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',100.18,657,878,99.56,'Sun-Synch' from dual union all
select 36255,'036255','1999-025DWG','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',101.25,761,875,99.43,'Sun-Synch' from dual union all
select 36256,'036256','1999-025DWH','deb FY-1C','',66706,'Reentered','2013-01-23 00:00:00','2010-01-07 00:00:00',99.36,613,842,98.98,'Sun-Synch' from dual union all
select 36257,'036257','1999-025DWJ','deb FY-1C','',66706,'In Earth orbit','','2010-01-06 00:00:00',101.85,824,869,99.15,'Sun-Synch' from dual union all
select 36258,'036258','1999-025DWK','deb FY-1C','',66706,'In Earth orbit','','2010-01-07 00:00:00',98.94,548,868,99.04,'Sun-Synch' from dual union all
select 36259,'036259','1999-025DWL','deb FY-1C','',66706,'In Earth orbit','','2010-01-08 00:00:00',101.89,844,853,98.86,'Sun-Synch' from dual union all
select 3626,'003626','1968-118A','Apollo 8','Apollo CM-103',16845,'Deep Space','1968-12-23 00:00:00','1968-12-21 00:00:00',24399.38,214,544356,30.71,'Cislunar' from dual union all
select 36260,'036260','1999-025DWM','deb FY-1C','',66706,'In Earth orbit','','2010-01-06 00:00:00',101.91,838,861,98.91,'Sun-Synch' from dual union all
select 36261,'036261','1999-025DWN','deb FY-1C','',66706,'In Earth orbit','','2010-01-02 00:00:00',107.34,829,1379,101.37,'Sun-Synch' from dual union all
select 36262,'036262','1999-025DWP','deb FY-1C','',66706,'In Earth orbit','','2010-01-05 00:00:00',101.21,770,862,99.06,'Sun-Synch' from dual union all
select 36263,'036263','1999-025DWQ','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',101.89,807,889,99.11,'Sun-Synch' from dual union all
select 36264,'036264','1999-025DWR','deb FY-1C','',66706,'Reentered','2014-12-08 00:00:00','2010-01-10 00:00:00',100.54,605,964,97.86,'Sun-Synch' from dual union all
select 36265,'036265','1999-025DWS','deb FY-1C','',66706,'In Earth orbit','','2010-01-08 00:00:00',98.65,547,842,99.48,'Sun-Synch' from dual union all
select 36266,'036266','1999-025DWT','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',103.16,849,968,98.85,'Sun-Synch' from dual union all
select 36267,'036267','1999-025DWU','deb FY-1C','',66706,'In Earth orbit','','2010-01-09 00:00:00',103.32,797,1034,98.55,'Sun-Synch' from dual union all
select 36268,'036268','1999-025DWV','deb FY-1C','',66706,'Reentered','2013-07-08 00:00:00','2010-01-11 00:00:00',99.1,609,823,98.81,'Sun-Synch' from dual union all
select 36269,'036269','1999-025DWW','deb FY-1C','',66706,'In Earth orbit','','2010-01-09 00:00:00',101.87,650,1044,99.22,'Sun-Synch' from dual union all
select 3627,'003627','1968-118B','Saturn S-IVB-503N','Saturn S-IVB-503N',16845,'Deep Space','1968-12-23 00:00:00','1968-12-21 00:00:00',23992.91,140,538222,30.71,'Cislunar' from dual union all
select 36270,'036270','1999-025DWX','deb FY-1C','',66706,'In Earth orbit','','2010-01-09 00:00:00',102.74,828,949,99.03,'Sun-Synch' from dual union all
select 36271,'036271','1999-025DWY','deb FY-1C','',66706,'In Earth orbit','','2010-01-05 00:00:00',103.43,821,1020,99.05,'Sun-Synch' from dual union all
select 36272,'036272','1999-025DWZ','deb FY-1C','',66706,'In Earth orbit','','2010-01-01 00:00:00',102.09,726,990,99.23,'Sun-Synch' from dual union all
select 36273,'036273','1999-025DXA','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',102,780,927,99.57,'Sun-Synch' from dual union all
select 36274,'036274','1999-025DXB','deb FY-1C','',66706,'In Earth orbit','','2010-01-10 00:00:00',103.63,851,1010,99.1,'Sun-Synch' from dual union all
select 36275,'036275','1999-025DXC','deb FY-1C','',66706,'Reentered','2016-02-01 00:00:00','2009-12-27 00:00:00',100.29,613,932,97.8,'Sun-Synch' from dual union all
select 36276,'036276','1999-025DXD','deb FY-1C','',66706,'Reentered','2011-12-03 00:00:00','2010-01-08 00:00:00',97.43,478,794,98.9,'Sun-Synch' from dual union all
select 36277,'036277','1999-025DXE','deb FY-1C','',66706,'Reentered','2012-06-19 00:00:00','2009-12-29 00:00:00',96.93,403,820,99.18,'Sun-Synch' from dual union all
select 36278,'036278','1999-025DXF','deb FY-1C','',66706,'In Earth orbit','','2010-01-05 00:00:00',102.06,808,905,99.27,'Sun-Synch' from dual;
insert into SATELLITE
select 36279,'036279','1999-025DXG','deb FY-1C','',66706,'In Earth orbit','','2010-01-06 00:00:00',101.91,803,896,99.12,'Sun-Synch' from dual union all
select 3628,'003628','1968-117C','deb Kosmos-261','',16844,'Reentered','1969-01-06 00:00:00','1968-12-27 00:00:00',93.73,298,616,71.06,'Intermediate' from dual union all
select 36280,'036280','1999-025DXH','deb FY-1C','',66706,'In Earth orbit','','2010-01-03 00:00:00',102.98,841,959,99.21,'Sun-Synch' from dual union all
select 36281,'036281','1999-025DXJ','deb FY-1C','',66706,'In Earth orbit','','2010-01-09 00:00:00',107.48,849,1372,99.25,'Sun-Synch' from dual union all
select 36282,'036282','1999-025DXK','deb FY-1C','',66706,'Reentered','2013-06-19 00:00:00','2010-01-10 00:00:00',98.7,407,985,98.5,'Sun-Synch' from dual union all
select 36283,'036283','1999-025DXL','deb FY-1C','',66706,'In Earth orbit','','2010-01-11 00:00:00',99.39,605,854,99.43,'Sun-Synch' from dual union all
select 36284,'036284','1999-025DXM','deb FY-1C','',66706,'Reentered','2015-12-05 00:00:00','2010-01-06 00:00:00',100.53,711,856,98.89,'Sun-Synch' from dual union all
select 36285,'036285','1998-067BW','Poisk thermal covers','',66503,'Reentered','2010-01-27 00:00:00','2010-01-14 00:00:00',91.24,329,342,51.64,'Intermediate' from dual union all
select 36286,'036286','1998-067BX','Poisk thermal covers','',66503,'Reentered','2010-01-27 00:00:00','2010-01-14 00:00:00',91.23,330,340,51.64,'Intermediate' from dual union all
select 36287,'036287','2010-001A','Beidou DW 3','di san ke beidou daohang',68861,'In Earth orbit','','2010-01-23 00:00:00',1436.06,35600,35972,1.8,'Stationary' from dual union all
select 36288,'036288','2010-001B','CZ-3C Stage 3','CZ-3C Stage 3',68861,'In Earth orbit','','2010-01-20 00:00:00',626.75,294,35469,20.41,'GEO Transfer' from dual union all
select 36289,'036289','2008-053K','deb SJ-6/3','',68561,'In Earth orbit','','2010-01-22 00:00:00',96.64,588,608,97.67,'Sun-Synch' from dual union all
select 3629,'003629','1968-119A','Kosmos-262','DS-U2-GF No. 1',16846,'Reentered','1969-07-18 00:00:00','1969-01-29 00:00:00',94.64,255,747,48.44,'Intermediate' from dual union all
select 36290,'036290','2008-053L','deb SJ-6/3','',68561,'In Earth orbit','','2010-01-21 00:00:00',96.44,573,604,97.68,'Sun-Synch' from dual union all
select 36291,'036291','2008-056G','CZ-2D sep motor cover','',68564,'In Earth orbit','','2010-01-21 00:00:00',102.83,758,1027,98.43,'Sun-Synch' from dual union all
select 36292,'036292','2008-056H','CZ-2D sep motor cover','',68564,'In Earth orbit','','2010-01-06 00:00:00',103.01,771,1031,98.54,'Sun-Synch' from dual union all
select 36293,'036293','2001-049QN','deb PSLV-C3 PS4','',67202,'In Earth orbit','','2010-01-19 00:00:00',98.87,600,810,97.87,'Sun-Synch' from dual union all
select 36294,'036294','2000-033E','deb Nadezhda','',66935,'In Earth orbit','','2010-01-20 00:00:00',98.31,668,687,98.37,'Sun-Synch' from dual union all
select 36295,'036295','1978-098D','deb Delta 145','',45293,'Reentered','2010-02-26 00:00:00','2010-01-21 00:00:00',103.33,612,1221,98.74,'Sun-Synch' from dual union all
select 36296,'036296','1974-015G','Burner 2A collision debris','',33618,'In Earth orbit','','2010-01-14 00:00:00',100.24,693,847,99.02,'Sun-Synch' from dual union all
select 36297,'036297','1993-059M','deb 11S772','',65161,'Reentered','2015-01-20 00:00:00','2010-01-22 00:00:00',101.57,813,853,70.99,'Intermediate' from dual union all
select 36298,'036298','2003-048K','deb DMSP 5D-3 F-16','',67672,'Reentered','2012-10-17 00:00:00','2010-01-13 00:00:00',99.83,739,762,98.96,'Sun-Synch' from dual union all
select 36299,'036299','1976-005C','deb Kosmos-789','',38957,'In Earth orbit','','2010-01-22 00:00:00',104.88,965,1013,82.97,'Intermediate' from dual union all
select 363,'000363','62 A Mu 1','Vostok-3','Vostok-3A No. 5',8146,'Landed','1962-08-15 00:00:00','1962-08-14 00:00:00',88.07,158,200,64.96,'Intermediate' from dual union all
select 3630,'003630','1968-119B','S1M','S1M',16846,'Reentered','1969-04-30 00:00:00','1969-01-29 00:00:00',94.23,255,707,48.43,'Intermediate' from dual union all
select 36300,'036300','1976-067BZ','deb Kosmos-839','',39019,'In Earth orbit','','2010-01-22 00:00:00',116.7,987,2077,65.65,'Intermediate' from dual union all
select 36301,'036301','1969-009D','deb Isis 1','',18276,'In Earth orbit','','2010-01-22 00:00:00',127.3,577,3432,88.4,'Medium' from dual union all
select 36302,'036302','1975-087C','deb Meteor','',36534,'In Earth orbit','','2010-01-22 00:00:00',102.22,787,940,81.24,'Intermediate' from dual union all
select 36303,'036303','1999-057SU','CZ-4 debris','',66738,'Reentered','2014-09-17 00:00:00','2010-01-22 00:00:00',98.52,648,728,98.26,'Sun-Synch' from dual union all
select 36304,'036304','1999-057SV','CZ-4 debris','',66738,'In Earth orbit','','2010-01-22 00:00:00',99.65,688,795,98.82,'Sun-Synch' from dual union all
select 36305,'036305','1972-022D','deb Meteor','',27954,'In Earth orbit','','2010-01-05 00:00:00',101.78,831,855,81.23,'Intermediate' from dual union all
select 36306,'036306','1972-022E','deb Meteor','',27954,'In Earth orbit','','2010-01-16 00:00:00',101.78,815,870,81.23,'Intermediate' from dual union all
select 36307,'036307','1980-073K','deb Meteor-2','',50283,'In Earth orbit','','2010-01-21 00:00:00',102,831,876,81.23,'Intermediate' from dual union all
select 36308,'036308','1980-073L','deb Meteor-2','',50283,'In Earth orbit','','2010-01-15 00:00:00',101.86,818,875,81.22,'Intermediate' from dual union all
select 36309,'036309','1980-073M','deb Meteor-2','',50283,'In Earth orbit','','2010-01-16 00:00:00',102.01,830,878,81.23,'Intermediate' from dual union all
select 3631,'003631','1968-117D','deb Blok Kosmos R-12','',16844,'Reentered','1969-02-01 00:00:00','1968-12-28 00:00:00',93.28,234,637,71.11,'Intermediate' from dual union all
select 36310,'036310','2009-002N','deb H-2A','',68683,'Reentered','2011-04-04 00:00:00','2010-01-21 00:00:00',96.98,578,651,98.04,'Sun-Synch' from dual union all
select 36311,'036311','2009-002P','deb H-2A','',68683,'Reentered','2011-06-01 00:00:00','2010-01-22 00:00:00',97.13,582,661,98.06,'Sun-Synch' from dual union all
select 36312,'036312','2009-002Q','deb H-2A','',68683,'Reentered','2010-03-18 00:00:00','2010-01-22 00:00:00',96.1,509,634,98.04,'Sun-Synch' from dual union all
select 36313,'036313','2009-002R','deb H-2A','',68683,'Reentered','2010-03-03 00:00:00','2010-01-21 00:00:00',95.83,510,608,97.98,'Sun-Synch' from dual union all
select 36314,'036314','2009-002S','deb H-2A','',68683,'Reentered','2010-05-12 00:00:00','2010-01-22 00:00:00',96.77,555,653,98.04,'Sun-Synch' from dual union all
select 36315,'036315','2009-002T','deb H-2A','',68683,'Reentered','2010-03-01 00:00:00','2010-01-22 00:00:00',96.17,512,639,98,'Sun-Synch' from dual union all
select 36316,'036316','2009-002U','deb H-2A','',68683,'Reentered','2010-04-17 00:00:00','2010-01-21 00:00:00',96.56,533,655,98.03,'Sun-Synch' from dual union all
select 36317,'036317','1999-057SW','ZY-1 debris','',66738,'In Earth orbit','','2010-01-18 00:00:00',100.17,764,770,98.19,'Sun-Synch' from dual union all
select 36318,'036318','1982-025C','deb Meteor-2','',54334,'In Earth orbit','','2010-01-22 00:00:00',102.98,888,911,82.54,'Intermediate' from dual union all
select 36319,'036319','1982-025D','deb Meteor-2','',54334,'In Earth orbit','','2010-01-10 00:00:00',102.63,876,891,82.54,'Intermediate' from dual union all
select 3632,'003632','1968-117E','deb Blok Kosmos R-12','',16844,'Reentered','1969-01-22 00:00:00','1969-01-04 00:00:00',91.65,275,436,71.06,'Intermediate' from dual union all
select 36320,'036320','1982-025E','deb Meteor-2','',54334,'In Earth orbit','','2010-01-19 00:00:00',102.7,874,899,82.53,'Intermediate' from dual union all
select 36321,'036321','1982-025F','deb Meteor-2','',54334,'In Earth orbit','','2010-01-19 00:00:00',102.86,888,900,82.53,'Intermediate' from dual union all
select 36322,'036322','1982-025G','deb Meteor-2','',54334,'In Earth orbit','','2010-01-18 00:00:00',102.7,879,894,82.54,'Intermediate' from dual union all
select 36323,'036323','1982-025H','deb Meteor-2','',54334,'In Earth orbit','','2010-01-05 00:00:00',102.94,893,903,82.55,'Intermediate' from dual union all
select 36324,'036324','1982-025J','deb Meteor-2','',54334,'In Earth orbit','','2010-01-17 00:00:00',103.13,885,928,82.53,'Intermediate' from dual union all
select 36325,'036325','1982-025K','deb Meteor-2','',54334,'In Earth orbit','','2010-01-18 00:00:00',102.94,894,902,82.54,'Intermediate' from dual union all
select 36326,'036326','1982-025L','deb Meteor-2','',54334,'In Earth orbit','','2010-01-19 00:00:00',103.2,885,935,82.53,'Intermediate' from dual union all
select 36327,'036327','1982-025M','deb Meteor-2','',54334,'In Earth orbit','','2010-01-18 00:00:00',103.19,901,918,82.53,'Intermediate' from dual union all
select 36328,'036328','1982-025N','deb Meteor-2','',54334,'In Earth orbit','','2010-01-07 00:00:00',103.05,895,911,82.53,'Intermediate' from dual union all
select 36329,'036329','1982-025P','deb Meteor-2','',54334,'In Earth orbit','','2010-01-13 00:00:00',102.61,870,894,82.53,'Intermediate' from dual union all
select 3633,'003633','1968-117F','deb Blok Kosmos R-12','',16844,'Reentered','1969-02-17 00:00:00','1968-12-28 00:00:00',94.37,375,601,71.01,'Intermediate' from dual union all
select 36330,'036330','1982-025Q','deb Meteor-2','',54334,'In Earth orbit','','2010-01-16 00:00:00',102.74,885,891,82.54,'Intermediate' from dual union all
select 36331,'036331','1982-025R','deb Meteor-2','',54334,'In Earth orbit','','2010-01-20 00:00:00',102.77,886,894,82.53,'Intermediate' from dual union all
select 36332,'036332','1982-025S','deb Meteor-2','',54334,'In Earth orbit','','2010-01-13 00:00:00',103.52,909,941,82.54,'Intermediate' from dual union all
select 36333,'036333','1982-025T','deb Meteor-2','',54334,'In Earth orbit','','2010-01-15 00:00:00',102.78,883,898,82.53,'Intermediate' from dual union all
select 36334,'036334','1982-025U','deb Meteor-2','',54334,'In Earth orbit','','2010-01-19 00:00:00',102.87,884,905,82.52,'Intermediate' from dual union all
select 36335,'036335','1982-025V','deb Meteor-2','',54334,'In Earth orbit','','2010-01-20 00:00:00',102.8,869,913,82.53,'Intermediate' from dual union all
select 36336,'036336','1982-025W','deb Meteor-2','',54334,'In Earth orbit','','2010-01-19 00:00:00',103.05,900,906,82.54,'Intermediate' from dual union all
select 36337,'036337','1982-025X','deb Meteor-2','',54334,'In Earth orbit','','2010-01-16 00:00:00',103.07,892,915,82.54,'Intermediate' from dual union all
select 36338,'036338','1982-025Y','deb Meteor-2','',54334,'In Earth orbit','','2010-01-21 00:00:00',102.73,874,902,82.54,'Intermediate' from dual union all
select 36339,'036339','1982-025Z','deb Meteor-2','',54334,'In Earth orbit','','2010-01-19 00:00:00',101.77,842,843,82.52,'Intermediate' from dual union all
select 3634,'003634','1968-117G','deb Blok Kosmos R-12','',16844,'Reentered','1969-08-18 00:00:00','1969-01-19 00:00:00',96.53,574,611,70.92,'Intermediate' from dual union all
select 36340,'036340','1982-025AA','deb Meteor-2','',54334,'In Earth orbit','','2010-01-01 00:00:00',102.24,839,890,82.54,'Intermediate' from dual union all
select 36341,'036341','1982-025AB','deb Meteor-2','',54334,'In Earth orbit','','2010-01-22 00:00:00',102.89,877,914,82.53,'Intermediate' from dual union all
select 36342,'036342','1982-025AC','deb Meteor-2','',54334,'In Earth orbit','','2010-01-19 00:00:00',103.07,880,928,82.54,'Intermediate' from dual union all
select 36343,'036343','1982-025AD','deb Meteor-2','',54334,'In Earth orbit','','2010-01-22 00:00:00',102.91,881,912,82.52,'Intermediate' from dual union all
select 36344,'036344','1982-025AE','deb Meteor-2','',54334,'In Earth orbit','','2010-01-19 00:00:00',102.08,834,880,82.53,'Intermediate' from dual union all
select 36345,'036345','1982-025AF','deb Meteor-2','',54334,'In Earth orbit','','2010-01-19 00:00:00',102.84,888,898,82.54,'Intermediate' from dual union all
select 36346,'036346','1982-025AG','deb Meteor-2','',54334,'In Earth orbit','','2010-01-19 00:00:00',102.75,881,897,82.55,'Intermediate' from dual union all
select 36347,'036347','1982-025AH','deb Meteor-2','',54334,'In Earth orbit','','2010-01-04 00:00:00',103.71,923,945,82.54,'Intermediate' from dual union all
select 36348,'036348','1982-025AJ','deb Meteor-2','',54334,'In Earth orbit','','2010-01-16 00:00:00',102.7,882,891,82.5,'Intermediate' from dual union all
select 36349,'036349','1982-025AK','deb Meteor-2','',54334,'In Earth orbit','','2010-01-14 00:00:00',102.64,872,896,82.53,'Intermediate' from dual union all
select 3635,'003635','1968-117H','deb Blok Kosmos R-12','',16844,'Reentered','1969-03-15 00:00:00','1969-01-20 00:00:00',93.26,300,569,71.07,'Intermediate' from dual union all
select 36350,'036350','1982-025AL','deb Meteor-2','',54334,'In Earth orbit','','2010-01-19 00:00:00',102.69,874,897,82.54,'Intermediate' from dual union all
select 36351,'036351','1982-025AM','deb Meteor-2','',54334,'In Earth orbit','','2010-01-18 00:00:00',102.98,894,906,82.55,'Intermediate' from dual union all
select 36352,'036352','1982-025AN','deb Meteor-2','',54334,'In Earth orbit','','2010-01-07 00:00:00',102.18,845,879,82.53,'Intermediate' from dual union all
select 36353,'036353','1982-025AP','deb Meteor-2','',54334,'In Earth orbit','','2010-01-03 00:00:00',103.08,901,908,82.53,'Intermediate' from dual union all
select 36354,'036354','1982-025AQ','deb Meteor-2','',54334,'In Earth orbit','','2010-01-19 00:00:00',102.87,881,908,82.53,'Intermediate' from dual union all
select 36355,'036355','1982-025AR','deb Meteor-2','',54334,'In Earth orbit','','2010-01-22 00:00:00',102.76,876,903,82.53,'Intermediate' from dual union all
select 36356,'036356','1982-025AS','deb Meteor-2','',54334,'In Earth orbit','','2010-01-19 00:00:00',102.76,874,905,82.53,'Intermediate' from dual union all
select 36357,'036357','1982-025AT','deb Meteor-2','',54334,'In Earth orbit','','2010-01-09 00:00:00',103.1,896,914,82.54,'Intermediate' from dual union all
select 36358,'036358','2010-002A','Raduga-1M','Globus-1M No. 12L',68862,'In Earth orbit','','2010-02-01 00:00:00',1425.68,35535,35630,.03,'Drift GEO' from dual union all
select 36359,'036359','2010-002B','Briz-M No. 88527','Briz-M',68862,'In Earth orbit','','2010-01-31 00:00:00',1384.19,34076,35453,.25,'Drift GEO' from dual union all
select 3636,'003636','1968-117J','deb Blok Kosmos R-12','',16844,'Reentered','1969-02-01 00:00:00','1968-12-28 00:00:00',93.18,267,594,71.04,'Intermediate' from dual union all
select 36360,'036360','2010-002C','Briz-M DTB','Briz-M torus tank',68862,'In Earth orbit','','2010-02-01 00:00:00',630.71,410,35558,46.48,'GEO Transfer' from dual union all
select 36361,'036361','2010-003A','Progress M-04M','Progress 7K-TGM No.404',68863,'Deorbited','2010-07-01 00:00:00','2010-02-11 00:00:00',91.36,334,349,51.64,'Intermediate' from dual union all
select 36362,'036362','2010-003B','11S510','11S510',68863,'Reentered','2010-02-05 00:00:00','2010-02-04 00:00:00',88.19,175,195,51.63,'Intermediate' from dual union all
select 36363,'036363','2006-015G','deb Yaogan','',68187,'Reentered','2012-08-04 00:00:00','2010-02-04 00:00:00',96.94,596,629,97.86,'Sun-Synch' from dual union all
select 36364,'036364','2006-015H','deb Yaogan','',68187,'Reentered','2017-07-10 00:00:00','2010-02-04 00:00:00',96.9,589,632,97.89,'Sun-Synch' from dual union all
select 36365,'036365','2006-015J','deb Yaogan','',68187,'In Earth orbit','','2010-02-04 00:00:00',97.68,627,669,97.82,'Sun-Synch' from dual union all
select 36366,'036366','1993-036AYT','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-05 00:00:00',102.26,774,957,74.06,'Intermediate' from dual union all
select 36367,'036367','1993-036AYU','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-05 00:00:00',104.89,774,1205,74.1,'Intermediate' from dual union all
select 36368,'036368','1993-036AYV','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-04 00:00:00',103.14,772,1042,74.18,'Intermediate' from dual;
insert into SATELLITE
select 36369,'036369','1993-036AYW','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-04 00:00:00',100.27,749,794,74.04,'Intermediate' from dual union all
select 3637,'003637','1968-117K','deb Blok Kosmos R-12','',16844,'Reentered','1969-01-22 00:00:00','1968-12-28 00:00:00',92.96,261,577,71.04,'Intermediate' from dual union all
select 36370,'036370','1993-036AYX','deb Kosmos-2251','',65138,'Reentered','2014-01-22 00:00:00','2010-02-04 00:00:00',98.78,604,797,74.08,'Intermediate' from dual union all
select 36371,'036371','1993-036AYY','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-05 00:00:00',100.71,775,810,74.03,'Intermediate' from dual union all
select 36372,'036372','1993-036AYZ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-04 00:00:00',101.16,774,854,74.02,'Intermediate' from dual union all
select 36373,'036373','1993-036AZA','deb Kosmos-2251','',65138,'Reentered','2014-03-17 00:00:00','2010-02-04 00:00:00',98.85,604,804,74.04,'Intermediate' from dual union all
select 36374,'036374','1993-036AZB','deb Kosmos-2251','',65138,'Reentered','2012-08-24 00:00:00','2010-02-05 00:00:00',98.4,594,771,73.96,'Intermediate' from dual union all
select 36375,'036375','1993-036AZC','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-04 00:00:00',100.08,747,778,74.05,'Intermediate' from dual union all
select 36376,'036376','1993-036AZD','deb Kosmos-2251','',65138,'Reentered','2015-12-02 00:00:00','2010-02-05 00:00:00',99.61,540,940,74.01,'Intermediate' from dual union all
select 36377,'036377','1993-036AZE','deb Kosmos-2251','',65138,'Reentered','2012-10-15 00:00:00','2010-02-05 00:00:00',96.6,429,763,74.16,'Intermediate' from dual union all
select 36378,'036378','1993-036AZF','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-05 00:00:00',98.51,595,779,74.06,'Intermediate' from dual union all
select 36379,'036379','1993-036AZG','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-05 00:00:00',100.7,756,827,74.01,'Intermediate' from dual union all
select 3638,'003638','1968-117L','deb Blok Kosmos R-12','',16844,'Reentered','1969-02-13 00:00:00','1969-01-09 00:00:00',94.11,377,574,71.03,'Intermediate' from dual union all
select 36380,'036380','1993-036AZH','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-04 00:00:00',99.45,690,774,74.02,'Intermediate' from dual union all
select 36381,'036381','1993-036AZJ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-04 00:00:00',101.26,773,864,74.03,'Intermediate' from dual union all
select 36382,'036382','1993-036AZK','deb Kosmos-2251','',65138,'Reentered','2011-02-25 00:00:00','2010-02-05 00:00:00',95.61,383,714,73.81,'Intermediate' from dual union all
select 36383,'036383','1993-036AZL','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-05 00:00:00',100.94,666,941,73.83,'Intermediate' from dual union all
select 36384,'036384','1993-036AZM','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-04 00:00:00',100.48,768,794,74.04,'Intermediate' from dual union all
select 36385,'036385','1993-036AZN','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-05 00:00:00',101,619,993,73.99,'Intermediate' from dual union all
select 36386,'036386','1993-036AZP','deb Kosmos-2251','',65138,'Reentered','2014-02-08 00:00:00','2010-02-04 00:00:00',97.17,473,774,73.99,'Intermediate' from dual union all
select 36387,'036387','1993-036AZQ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-05 00:00:00',101.35,772,874,74.01,'Intermediate' from dual union all
select 36388,'036388','1993-036AZR','deb Kosmos-2251','',65138,'In Earth orbit','','2010-02-04 00:00:00',98.88,586,825,73.99,'Intermediate' from dual union all
select 36389,'036389','1997-051VT','deb Iridium 33','',66131,'Reentered','2010-10-01 00:00:00','2010-02-05 00:00:00',98.68,658,733,86.33,'Polar' from dual union all
select 3639,'003639','1968-117M','deb Blok Kosmos R-12','',16844,'Reentered','1969-05-02 00:00:00','1969-01-25 00:00:00',92.98,264,577,71.05,'Intermediate' from dual union all
select 36390,'036390','1997-051VU','deb Iridium 33','',66131,'In Earth orbit','','2010-02-04 00:00:00',99.94,715,796,86.32,'Polar' from dual union all
select 36391,'036391','1997-051VV','deb Iridium 33','',66131,'Reentered','2013-08-25 00:00:00','2010-02-05 00:00:00',98.87,644,766,86.43,'Polar' from dual union all
select 36392,'036392','1997-051VW','deb Iridium 33','',66131,'Reentered','2011-10-03 00:00:00','2010-02-05 00:00:00',97.9,551,766,86.09,'Polar' from dual union all
select 36393,'036393','1997-051VX','deb Iridium 33','',66131,'Reentered','2012-06-14 00:00:00','2010-02-05 00:00:00',100.72,748,837,86.38,'Polar' from dual union all
select 36394,'036394','2010-004A','Endeavour (STS-130)','OV-105',68864,'Landed','2010-02-22 00:00:00','2010-02-11 00:00:00',91.36,334,349,51.64,'Intermediate' from dual union all
select 36395,'036395','2010-005A','Solar Dynamics Observatory','SDO',68865,'In Earth orbit','','2010-03-18 00:00:00',1436.04,35781,35790,28.1,'Synchronous' from dual union all
select 36396,'036396','2010-005B','Centaur AV-021','Centaur AV-021',68865,'In Earth orbit','','2010-02-19 00:00:00',629.08,2397,33487,28.32,'GEO Transfer' from dual union all
select 36397,'036397','2010-006A','Intelsat IS-16','Intelsat IS-16',68866,'In Earth orbit','','2010-02-19 00:00:00',1451.98,35909,36285,.13,'Drift GEO' from dual union all
select 36398,'036398','2010-006B','Briz-M No. 99511','Briz-M',68866,'In Earth orbit','','2010-02-12 00:00:00',1538.19,37515,38009,.1,'Very High' from dual union all
select 36399,'036399','2010-006C','Briz-M DTB','Briz-M torus tank',68866,'In Earth orbit','','2010-02-19 00:00:00',673.94,509,37661,46.13,'GEO Transfer' from dual union all
select 364,'000364','62 A Mu 2','8K72KE','8K72KE',8146,'Reentered','1962-08-14 00:00:00','1962-08-13 00:00:00',87.5,151,151,64.82,'Intermediate' from dual union all
select 3640,'003640','1968-117N','deb Blok Kosmos R-12','',16844,'Reentered','1969-02-10 00:00:00','1969-01-07 00:00:00',93.01,280,564,71.04,'Intermediate' from dual union all
select 36400,'036400','2010-007A','Kosmos-2459','Uragan-M No. 731',68867,'In Earth orbit','','2010-03-18 00:00:00',675.69,19044,19214,64.77,'Medium' from dual union all
select 36401,'036401','2010-007B','Kosmos-2461','Uragan-M No. 735',68867,'In Earth orbit','','2010-03-09 00:00:00',675.79,19131,19132,64.76,'Medium' from dual union all
select 36402,'036402','2010-007C','Kosmos-2460','Uragan-M No. 732',68867,'In Earth orbit','','2010-03-09 00:00:00',678.81,19128,19287,64.77,'Medium' from dual union all
select 36403,'036403','2010-007D','Blok DM-2 No. 116L','11S861 No. 116L',68867,'In Earth orbit','','2010-03-09 00:00:00',675.06,19090,19136,64.76,'Medium' from dual union all
select 36404,'036404','2010-007E','8S812','8S812',68867,'Reentered','2010-03-02 00:00:00','2010-03-02 00:00:00',88.04,151,203,64.79,'Intermediate' from dual union all
select 36405,'036405','2010-007F','Perekhodnik','Sredniy perekhodnik',68867,'Reentered','2010-03-02 00:00:00','2010-03-02 00:00:00',87.85,166,170,64.79,'Intermediate' from dual union all
select 36406,'036406','2010-007G','SOZ','SOZ',68867,'In Earth orbit','','2010-03-11 00:00:00',340.3,399,19137,64.68,'Highly Ellip' from dual union all
select 36407,'036407','2010-007H','SOZ','SOZ',68867,'In Earth orbit','','2010-03-11 00:00:00',340.3,400,19136,64.84,'Highly Ellip' from dual union all
select 36408,'036408','2010-007J','deb 8S812','',68867,'Reentered','2010-03-07 00:00:00','2010-03-03 00:00:00',90.96,170,474,64.79,'Intermediate' from dual union all
select 36409,'036409','2010-007K','deb 8S812','',68867,'Reentered','2010-03-06 00:00:00','2010-03-04 00:00:00',90.23,169,403,64.65,'Intermediate' from dual union all
select 3641,'003641','1968-117P','deb Blok Kosmos R-12','',16844,'Reentered','1969-05-07 00:00:00','1969-01-20 00:00:00',93.41,295,588,71.05,'Intermediate' from dual union all
select 36410,'036410','2010-007L','deb 8S812','',68867,'Reentered','2010-03-05 00:00:00','2010-03-04 00:00:00',90.15,168,396,64.93,'Intermediate' from dual union all
select 36411,'036411','2010-008A','GOES 15','GOES P',68868,'In Earth orbit','','2010-03-18 00:00:00',1435.93,35778,35789,.46,'Stationary' from dual union all
select 36412,'036412','2010-008B','Delta 348','DCSS-4 F08',68868,'In Earth orbit','','2010-03-07 00:00:00',746.47,6597,35164,13.13,'GEO Transfer' from dual union all
select 36413,'036413','2010-009A','Yaogan Weixing 9','Jianbing-8 01 xing',68869,'In Earth orbit','','2010-03-07 00:00:00',107.04,1082,1097,63.41,'Intermediate' from dual union all
select 36414,'036414','2010-009B','Yaogan Weixing 9 fu xing yi','Jianbing-8 01 subsat 1',68869,'In Earth orbit','','2010-03-07 00:00:00',107.08,1084,1099,63.41,'Intermediate' from dual union all
select 36415,'036415','2010-009C','Yaogan Weixing 9 fu xing er','Jianbing-8 01 subsat 2',68869,'In Earth orbit','','2010-03-07 00:00:00',107.09,1085,1099,63.41,'Intermediate' from dual union all
select 36416,'036416','2010-009D','CZ-4C Stage 3','CZ-4C Stage 3',68869,'In Earth orbit','','2010-03-07 00:00:00',105.15,907,1096,63.46,'Intermediate' from dual union all
select 36417,'036417','2010-009E','deb YW-9','',68869,'In Earth orbit','','2010-03-07 00:00:00',106.88,1067,1098,63.41,'Intermediate' from dual union all
select 36418,'036418','2010-009F','deb YW-9','',68869,'In Earth orbit','','2010-03-07 00:00:00',107.01,1078,1099,63.41,'Intermediate' from dual union all
select 36419,'036419','1993-036AZS','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-16 00:00:00',101.81,754,934,73.86,'Intermediate' from dual union all
select 3642,'003642','1969-001A','Venera-5','4V-1 No. 330',18268,'Deep Space','1969-01-07 00:00:00','',null,null,null,null,'' from dual union all
select 36420,'036420','1993-036AZT','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-15 00:00:00',100.45,767,793,74.04,'Intermediate' from dual union all
select 36421,'036421','1993-036AZU','deb Kosmos-2251','',65138,'Reentered','2011-11-17 00:00:00','2010-03-16 00:00:00',96.16,419,731,73.98,'Intermediate' from dual union all
select 36422,'036422','1993-036AZV','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-16 00:00:00',99.22,596,847,74.02,'Intermediate' from dual union all
select 36423,'036423','1993-036AZW','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-16 00:00:00',103.82,663,1216,73.84,'Intermediate' from dual union all
select 36424,'036424','1993-036AZX','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-17 00:00:00',102.21,772,954,74.19,'Intermediate' from dual union all
select 36425,'036425','1993-036AZY','deb Kosmos-2251','',65138,'Reentered','2011-12-01 00:00:00','2010-03-15 00:00:00',96.96,470,756,73.97,'Intermediate' from dual union all
select 36426,'036426','1993-036AZZ','deb Kosmos-2251','',65138,'Reentered','2011-10-07 00:00:00','2010-03-16 00:00:00',96.69,413,788,74.06,'Intermediate' from dual union all
select 36427,'036427','1993-036BAA','deb Kosmos-2251','',65138,'Reentered','2014-12-16 00:00:00','2010-03-16 00:00:00',99.3,529,922,74.01,'Intermediate' from dual union all
select 36428,'036428','1993-036BAB','deb Kosmos-2251','',65138,'Reentered','2010-04-25 00:00:00','2010-03-17 00:00:00',92.55,311,488,73.95,'Intermediate' from dual union all
select 36429,'036429','1993-036BAC','deb Kosmos-2251','',65138,'Reentered','2010-10-27 00:00:00','2010-03-15 00:00:00',94.23,340,623,73.7,'Intermediate' from dual union all
select 3643,'003643','1969-001B','11S510','11S510',18268,'Reentered','1969-01-06 00:00:00','1969-01-05 00:00:00',88.29,184,195,51.79,'Intermediate' from dual union all
select 36430,'036430','1993-036BAD','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-16 00:00:00',101.39,750,899,74.01,'Intermediate' from dual union all
select 36431,'036431','1993-036BAE','deb Kosmos-2251','',65138,'Reentered','2012-05-20 00:00:00','2010-03-16 00:00:00',98.36,523,838,74.12,'Intermediate' from dual union all
select 36432,'036432','1993-036BAF','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-15 00:00:00',99.09,622,809,73.74,'Intermediate' from dual union all
select 36433,'036433','1993-036BAG','deb Kosmos-2251','',65138,'Reentered','2013-03-20 00:00:00','2010-03-17 00:00:00',97.71,524,775,74.05,'Intermediate' from dual union all
select 36434,'036434','1993-036BAH','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-15 00:00:00',100.95,773,835,74.02,'Intermediate' from dual union all
select 36435,'036435','1993-036BAJ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-15 00:00:00',101.58,752,915,73.83,'Intermediate' from dual union all
select 36436,'036436','1993-036BAK','deb Kosmos-2251','',65138,'Reentered','2011-12-02 00:00:00','2010-03-17 00:00:00',97.77,552,752,74.1,'Intermediate' from dual union all
select 36437,'036437','1993-036BAL','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-17 00:00:00',100.33,750,799,73.96,'Intermediate' from dual union all
select 36438,'036438','1993-036BAM','deb Kosmos-2251','',65138,'Reentered','2011-12-16 00:00:00','2010-03-17 00:00:00',97.13,468,775,73.92,'Intermediate' from dual union all
select 36439,'036439','1993-036BAN','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-15 00:00:00',107.67,774,1464,73.97,'Intermediate' from dual union all
select 3644,'003644','1968-117Q','deb Blok Kosmos R-12','',16844,'Reentered','1969-03-06 00:00:00','1969-01-19 00:00:00',93.44,324,562,71.05,'Intermediate' from dual union all
select 36440,'036440','1993-036BAP','deb Kosmos-2251','',65138,'Reentered','2011-12-25 00:00:00','2010-03-16 00:00:00',97.45,522,752,73.95,'Intermediate' from dual union all
select 36441,'036441','1993-036BAQ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-15 00:00:00',97.95,518,804,73.85,'Intermediate' from dual union all
select 36442,'036442','1993-036BAR','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-16 00:00:00',100.69,762,821,74,'Intermediate' from dual union all
select 36443,'036443','1993-036BAS','deb Kosmos-2251','',65138,'Reentered','2014-06-27 00:00:00','2010-03-15 00:00:00',98.71,600,794,74.13,'Intermediate' from dual union all
select 36444,'036444','1993-036BAT','deb Kosmos-2251','',65138,'Reentered','2011-03-30 00:00:00','2010-03-16 00:00:00',94.53,326,666,73.9,'Intermediate' from dual union all
select 36445,'036445','1993-036BAU','deb Kosmos-2251','',65138,'Reentered','2013-08-20 00:00:00','2010-03-15 00:00:00',98.59,586,797,74.12,'Intermediate' from dual union all
select 36446,'036446','1993-036BAV','deb Kosmos-2251','',65138,'Reentered','2014-12-31 00:00:00','2010-03-15 00:00:00',98.45,556,814,74.02,'Intermediate' from dual union all
select 36447,'036447','1993-036BAW','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-17 00:00:00',98.34,572,787,73.95,'Intermediate' from dual union all
select 36448,'036448','1993-036BAX','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-16 00:00:00',101.12,772,852,74.09,'Intermediate' from dual union all
select 36449,'036449','1993-036BAY','deb Kosmos-2251','',65138,'Reentered','2013-09-08 00:00:00','2010-03-16 00:00:00',97.45,486,788,74.11,'Intermediate' from dual union all
select 3645,'003645','1968-117R','deb Blok Kosmos R-12','',16844,'Reentered','1969-03-08 00:00:00','1969-01-24 00:00:00',93.46,350,538,70.93,'Intermediate' from dual union all
select 36450,'036450','1993-036BAZ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-17 00:00:00',100.66,770,810,74.02,'Intermediate' from dual union all
select 36451,'036451','1993-036BBA','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-15 00:00:00',102.75,725,1052,73.94,'Intermediate' from dual union all
select 36452,'036452','1993-036BBB','deb Kosmos-2251','',65138,'Reentered','2014-10-27 00:00:00','2010-03-17 00:00:00',100.52,753,814,73.99,'Intermediate' from dual union all
select 36453,'036453','1993-036BBC','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-16 00:00:00',100.65,722,857,73.93,'Intermediate' from dual union all
select 36454,'036454','1993-036BBD','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-15 00:00:00',100.01,741,777,74.02,'Intermediate' from dual union all
select 36455,'036455','1993-036BBE','deb Kosmos-2251','',65138,'Reentered','2014-09-16 00:00:00','2010-03-15 00:00:00',99.94,743,769,74.05,'Intermediate' from dual union all
select 36456,'036456','1993-036BBF','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-16 00:00:00',101.34,773,872,74.02,'Intermediate' from dual union all
select 36457,'036457','1993-036BBG','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-17 00:00:00',99.77,716,780,74.05,'Intermediate' from dual union all
select 36458,'036458','1993-036BBH','deb Kosmos-2251','',65138,'Reentered','2011-05-17 00:00:00','2010-03-17 00:00:00',96.3,455,708,73.82,'Intermediate' from dual;
insert into SATELLITE
select 36459,'036459','1993-036BBJ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-15 00:00:00',102.72,776,998,74.03,'Intermediate' from dual union all
select 3646,'003646','1969-001C','BOZ','BOZ',18268,'Reentered','1969-01-07 00:00:00','1969-01-05 00:00:00',88.59,185,225,51.8,'Intermediate' from dual union all
select 36460,'036460','1993-036BBK','deb Kosmos-2251','',65138,'Reentered','2014-05-23 00:00:00','2010-03-15 00:00:00',99.3,602,849,74,'Intermediate' from dual union all
select 36461,'036461','1993-036BBL','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-16 00:00:00',98.22,576,772,73.95,'Intermediate' from dual union all
select 36462,'036462','1993-036BBM','deb Kosmos-2251','',65138,'Reentered','2018-02-13 00:00:00','2010-03-16 00:00:00',98.68,608,784,73.84,'Intermediate' from dual union all
select 36463,'036463','1993-036BBN','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-15 00:00:00',99.87,736,769,74.05,'Intermediate' from dual union all
select 36464,'036464','1993-036BBP','deb Kosmos-2251','',65138,'Reentered','2012-03-12 00:00:00','2010-03-17 00:00:00',97.57,516,769,74.01,'Intermediate' from dual union all
select 36465,'036465','1993-036BBQ','deb Kosmos-2251','',65138,'Reentered','2011-10-26 00:00:00','2010-03-16 00:00:00',96.43,450,726,73.91,'Intermediate' from dual union all
select 36466,'036466','1993-036BBR','deb Kosmos-2251','',65138,'Reentered','2011-12-10 00:00:00','2010-03-15 00:00:00',95.75,375,735,74.06,'Intermediate' from dual union all
select 36467,'036467','1993-036BBS','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-17 00:00:00',102.22,750,977,73.85,'Intermediate' from dual union all
select 36468,'036468','1993-036BBT','deb Kosmos-2251','',65138,'Reentered','2014-10-16 00:00:00','2010-03-16 00:00:00',99.47,550,917,74.05,'Intermediate' from dual union all
select 36469,'036469','1993-036BBU','deb Kosmos-2251','',65138,'Reentered','2013-07-24 00:00:00','2010-03-15 00:00:00',97.22,483,769,73.93,'Intermediate' from dual union all
select 3647,'003647','1968-117S','deb Blok Kosmos R-12','',16844,'Reentered','1969-03-19 00:00:00','1969-01-20 00:00:00',94.16,381,575,70.96,'Intermediate' from dual union all
select 36470,'036470','1993-036BBV','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-15 00:00:00',100.34,716,834,73.88,'Intermediate' from dual union all
select 36471,'036471','1993-036BBW','deb Kosmos-2251','',65138,'Reentered','2018-03-13 00:00:00','2010-03-15 00:00:00',97.71,527,772,73.91,'Intermediate' from dual union all
select 36472,'036472','1993-036BBX','deb Kosmos-2251','',65138,'Reentered','2012-03-03 00:00:00','2010-03-16 00:00:00',96.65,444,753,74.05,'Intermediate' from dual union all
select 36473,'036473','1993-036BBY','deb Kosmos-2251','',65138,'Reentered','2013-01-25 00:00:00','2010-03-15 00:00:00',96.9,467,754,74.02,'Intermediate' from dual union all
select 36474,'036474','1993-036BBZ','deb Kosmos-2251','',65138,'Reentered','2016-01-25 00:00:00','2010-03-15 00:00:00',98.13,559,779,73.95,'Intermediate' from dual union all
select 36475,'036475','1993-036BCA','deb Kosmos-2251','',65138,'Reentered','2012-01-25 00:00:00','2010-03-16 00:00:00',96.88,461,758,74.06,'Intermediate' from dual union all
select 36476,'036476','1993-036BCB','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-17 00:00:00',100.45,763,797,74.04,'Intermediate' from dual union all
select 36477,'036477','1993-036BCC','deb Kosmos-2251','',65138,'Reentered','2014-01-04 00:00:00','2010-03-15 00:00:00',97.88,548,766,74.01,'Intermediate' from dual union all
select 36478,'036478','1993-036BCD','deb Kosmos-2251','',65138,'Reentered','2013-07-16 00:00:00','2010-03-15 00:00:00',97.93,550,770,74.19,'Intermediate' from dual union all
select 36479,'036479','1993-036BCE','deb Kosmos-2251','',65138,'In Earth orbit','','2010-03-15 00:00:00',98.68,612,780,74.05,'Intermediate' from dual union all
select 3648,'003648','1969-002A','Venera-6','4V-1 No. 331',18269,'Deep Space','1969-01-12 00:00:00','',null,null,null,null,'' from dual union all
select 36480,'036480','1993-036BCF','deb Kosmos-2251','',65138,'Reentered','2015-12-01 00:00:00','2010-03-16 00:00:00',100.3,770,776,74.03,'Intermediate' from dual union all
select 36481,'036481','1993-036BCG','deb Kosmos-2251','',65138,'Reentered','2010-06-09 00:00:00','2010-03-16 00:00:00',92.83,304,523,73.86,'Intermediate' from dual union all
select 36482,'036482','1997-051VY','deb Iridium 33','',66131,'In Earth orbit','','2010-03-15 00:00:00',100.08,748,777,86.37,'Polar' from dual union all
select 36483,'036483','1997-051VZ','deb Iridium 33','',66131,'In Earth orbit','','2010-03-16 00:00:00',100.53,778,789,86.39,'Polar' from dual union all
select 36484,'036484','1997-051WA','deb Iridium 33','',66131,'Reentered','2014-08-27 00:00:00','2010-03-15 00:00:00',100.05,727,795,86.03,'Polar' from dual union all
select 36485,'036485','1997-051WB','deb Iridium 33','',66131,'In Earth orbit','','2010-03-16 00:00:00',102.67,762,1008,86.45,'Polar' from dual union all
select 36486,'036486','1997-051WC','deb Iridium 33','',66131,'In Earth orbit','','2010-03-15 00:00:00',100.22,760,778,86.37,'Polar' from dual union all
select 36487,'036487','1997-051WD','deb Iridium 33','',66131,'In Earth orbit','','2010-03-15 00:00:00',103.63,773,1088,86.38,'Polar' from dual union all
select 36488,'036488','1997-051WE','deb Iridium 33','',66131,'In Earth orbit','','2010-03-16 00:00:00',100.83,772,823,86.46,'Polar' from dual union all
select 36489,'036489','1997-051WF','deb Iridium 33','',66131,'In Earth orbit','','2010-03-16 00:00:00',100.75,773,815,86.44,'Polar' from dual union all
select 3649,'003649','1969-002B','11S510','11S510',18269,'Reentered','1969-01-11 00:00:00','1969-01-10 00:00:00',88.26,184,193,51.75,'Intermediate' from dual union all
select 36490,'036490','1997-051WG','deb Iridium 33','',66131,'In Earth orbit','','2010-03-16 00:00:00',100.34,773,777,86.39,'Polar' from dual union all
select 36491,'036491','1997-051WH','deb Iridium 33','',66131,'Reentered','2017-02-12 00:00:00','2010-03-17 00:00:00',101.4,769,881,86.35,'Polar' from dual union all
select 36492,'036492','1997-051WJ','deb Iridium 33','',66131,'In Earth orbit','','2010-03-16 00:00:00',100.55,772,798,86.44,'Polar' from dual union all
select 36493,'036493','1997-051WK','deb Iridium 33','',66131,'In Earth orbit','','2010-03-15 00:00:00',100.38,764,790,86.37,'Polar' from dual union all
select 36494,'036494','1997-051WL','deb Iridium 33','',66131,'Reentered','2013-04-08 00:00:00','2010-03-17 00:00:00',99.69,717,770,86.02,'Polar' from dual union all
select 36495,'036495','1997-051WM','deb Iridium 33','',66131,'In Earth orbit','','2010-03-17 00:00:00',100.92,774,831,86.42,'Polar' from dual union all
select 36496,'036496','1997-051WN','deb Iridium 33','',66131,'Reentered','2014-12-22 00:00:00','2010-03-16 00:00:00',99.87,729,776,86.36,'Polar' from dual union all
select 36497,'036497','1997-051WP','deb Iridium 33','',66131,'In Earth orbit','','2010-03-15 00:00:00',100.97,746,864,86.16,'Polar' from dual union all
select 36498,'036498','1997-051WQ','deb Iridium 33','',66131,'Reentered','2013-09-13 00:00:00','2010-03-17 00:00:00',99.07,653,776,86.1,'Polar' from dual union all
select 36499,'036499','2010-010A','Echostar XIV','Echostar 14',68870,'In Earth orbit','','2010-03-27 00:00:00',1435.4,35760,35786,.07,'Drift GEO' from dual union all
select 365,'000365','62 A Nu 1','Vostok-4','Vostok-3A No. 6',8147,'Landed','1962-08-15 00:00:00','1962-08-12 00:00:00',88.38,168,221,64.95,'Intermediate' from dual union all
select 3650,'003650','1969-002C','BOZ','BOZ',18269,'Reentered','1969-01-13 00:00:00','1969-01-12 00:00:00',88.38,191,198,51.67,'Intermediate' from dual union all
select 36500,'036500','2010-010B','Briz-M No. 99514','Briz-M',68870,'In Earth orbit','','2010-04-21 00:00:00',641.75,2546,33988,26.56,'GEO Transfer' from dual union all
select 36501,'036501','2010-010C','Briz-M DTB','Briz-M torus tank',68870,'In Earth orbit','','2010-04-20 00:00:00',256.34,328,13649,49.66,'Medium' from dual union all
select 36502,'036502','2006-015K','deb Yaogan','',68187,'Reentered','2015-03-13 00:00:00','2010-03-29 00:00:00',97.25,624,631,97.89,'Sun-Synch' from dual union all
select 36503,'036503','2006-015L','deb Yaogan','',68187,'Reentered','2010-03-24 00:00:00','2010-03-23 00:00:00',91.87,341,392,97.87,'Sun-Synch' from dual union all
select 36504,'036504','2006-015M','deb Yaogan','',68187,'Reentered','2010-04-07 00:00:00','2010-03-26 00:00:00',94.91,496,533,97.83,'Sun-Synch' from dual union all
select 36505,'036505','2010-011A','Soyuz TMA-18','Soyuz 11F732A17 No. 228',68871,'Landed','2010-09-25 00:00:00','2010-04-04 00:00:00',91.45,342,350,51.65,'Intermediate' from dual union all
select 36506,'036506','2010-011B','11S510','11S510',68871,'Reentered','2010-04-05 00:00:00','2010-04-03 00:00:00',88.4,183,207,51.64,'Intermediate' from dual union all
select 36507,'036507','2010-012A','Discovery (STS-131)','OV-103',68872,'Landed','2010-04-20 00:00:00','2010-04-07 00:00:00',91.45,342,350,51.65,'Intermediate' from dual union all
select 36508,'036508','2010-013A','CRYOSAT 2','Cryosat 2',68873,'In Earth orbit','','2010-04-10 00:00:00',99.15,710,726,92.02,'Polar' from dual union all
select 36509,'036509','2010-013B','R-36M2 DS','R-36M2 DS',68873,'In Earth orbit','','2010-04-10 00:00:00',103.36,717,1117,91.92,'Polar' from dual union all
select 3651,'003651','1969-003A','Kosmos-263','Zenit-2 No. 70',18270,'Landed','1969-01-20 00:00:00','1969-01-16 00:00:00',89.66,197,319,65.43,'Intermediate' from dual union all
select 36510,'036510','2010-013C','Dnepr plume shield','GDS',68873,'In Earth orbit','','2010-04-10 00:00:00',98.8,683,720,92.03,'Polar' from dual union all
select 36511,'036511','2010-014A','Kosmos-2462','Kobal''t-M No. 561',68874,'Landed','2010-07-21 00:00:00','2010-04-16 00:00:00',89.46,169,327,67.17,'Intermediate' from dual union all
select 36512,'036512','2010-014B','11S510','11S510',68874,'Reentered','2010-04-21 00:00:00','2010-04-16 00:00:00',89.33,169,314,67.17,'Intermediate' from dual union all
select 36513,'036513','2010-014C','deb Kosmos-2462','',68874,'Reentered','2010-04-18 00:00:00','2010-04-16 00:00:00',89.56,169,337,67.14,'Intermediate' from dual union all
select 36514,'036514','2010-015A','X-37B OTV-1','X-37B No. 1',68875,'Landed','2010-12-03 00:00:00','2010-05-20 00:00:00',92.72,407,409,40,'Intermediate' from dual union all
select 36515,'036515','2010-015B','Centaur AV-012','Centaur AV-012',68875,'Deep Space','2010-04-23 00:00:00','2010-04-24 00:00:00',1500.53,409,99999,40,'Earth Escape' from dual union all
select 36516,'036516','2010-016A','SES-1','SES-1',68876,'In Earth orbit','','2010-05-03 00:00:00',1432.79,35664,35780,.07,'Drift GEO' from dual union all
select 36517,'036517','2010-016B','Briz-M No. 99512','Briz-M',68876,'In Earth orbit','','2010-05-19 00:00:00',1195.5,28198,33672,.46,'Medium' from dual union all
select 36518,'036518','2010-016C','Briz-M DTB','Briz-M torus tank',68876,'In Earth orbit','','2010-05-02 00:00:00',597.21,518,33709,46.19,'Highly Ellip' from dual union all
select 36519,'036519','2010-017A','Kosmos-2463','Parus No. 207',68877,'In Earth orbit','','2010-04-29 00:00:00',105,969,1020,82.96,'Intermediate' from dual union all
select 3652,'003652','1969-003B','11A57I','11A57I',18270,'Reentered','1969-01-18 00:00:00','1969-01-14 00:00:00',89.38,172,316,65.34,'Intermediate' from dual union all
select 36520,'036520','2010-017B','S3M','S3M',68877,'In Earth orbit','','2010-04-28 00:00:00',104.88,968,1010,82.96,'Intermediate' from dual union all
select 36521,'036521','2010-018A','Progress M-05M','Progress 7K-TGM No.405',68878,'Deorbited','2010-11-15 00:00:00','2010-04-30 00:00:00',89.58,245,263,51.64,'Intermediate' from dual union all
select 36522,'036522','2010-018B','11S510','11S510',68878,'Reentered','2010-05-01 00:00:00','2010-04-30 00:00:00',88.32,177,206,51.63,'Intermediate' from dual union all
select 36523,'036523','1993-036BCH','deb Kosmos-2251','',65138,'Reentered','2013-05-30 00:00:00','2010-04-28 00:00:00',97.74,525,777,73.93,'Intermediate' from dual union all
select 36524,'036524','1993-036BCJ','deb Kosmos-2251','',65138,'Reentered','2014-12-05 00:00:00','2010-04-27 00:00:00',98.02,524,803,74.04,'Intermediate' from dual union all
select 36525,'036525','1993-036BCK','deb Kosmos-2251','',65138,'Reentered','2012-03-07 00:00:00','2010-04-28 00:00:00',97.91,442,876,74.01,'Intermediate' from dual union all
select 36526,'036526','1993-036BCL','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-28 00:00:00',98.4,592,772,73.98,'Intermediate' from dual union all
select 36527,'036527','1993-036BCM','deb Kosmos-2251','',65138,'Reentered','2014-09-30 00:00:00','2010-04-28 00:00:00',100.07,753,770,74.03,'Intermediate' from dual union all
select 36528,'036528','1993-036BCN','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-27 00:00:00',101.62,764,907,74.02,'Intermediate' from dual union all
select 36529,'036529','1993-036BCP','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-27 00:00:00',103.42,770,1071,74.06,'Intermediate' from dual union all
select 3653,'003653','1969-003C','deb Kosmos-263','',18270,'Reentered','1969-01-19 00:00:00','1969-01-13 00:00:00',89.4,181,309,65.29,'Intermediate' from dual union all
select 36530,'036530','1993-036BCQ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-27 00:00:00',99.8,718,779,74.02,'Intermediate' from dual union all
select 36531,'036531','1993-036BCR','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-28 00:00:00',104.63,771,1184,74.11,'Intermediate' from dual union all
select 36532,'036532','1993-036BCS','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-28 00:00:00',99.01,646,777,74,'Intermediate' from dual union all
select 36533,'036533','1993-036BCT','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-27 00:00:00',100.07,721,802,74.01,'Intermediate' from dual union all
select 36534,'036534','1993-036BCU','deb Kosmos-2251','',65138,'Reentered','2012-04-11 00:00:00','2010-04-28 00:00:00',97.62,525,765,73.93,'Intermediate' from dual union all
select 36535,'036535','1993-036BCV','deb Kosmos-2251','',65138,'Reentered','2016-06-01 00:00:00','2010-04-28 00:00:00',99.87,682,823,73.88,'Intermediate' from dual union all
select 36536,'036536','1993-036BCW','deb Kosmos-2251','',65138,'Reentered','2014-10-21 00:00:00','2010-04-27 00:00:00',99.12,634,800,74.07,'Intermediate' from dual union all
select 36537,'036537','1993-036BCX','deb Kosmos-2251','',65138,'Reentered','2011-12-28 00:00:00','2010-04-27 00:00:00',97.7,536,762,73.93,'Intermediate' from dual union all
select 36538,'036538','1993-036BCY','deb Kosmos-2251','',65138,'Reentered','2014-12-31 00:00:00','2010-04-28 00:00:00',99.84,709,793,74.19,'Intermediate' from dual union all
select 36539,'036539','1993-036BCZ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-28 00:00:00',99.94,729,783,74.03,'Intermediate' from dual union all
select 3654,'003654','1969-004A','Soyuz-4','Soyuz No. 12',18271,'Landed','1969-01-17 00:00:00','1969-01-15 00:00:00',88.79,206,224,51.7,'Intermediate' from dual union all
select 36540,'036540','1993-036BDA','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-28 00:00:00',100.54,768,801,74.03,'Intermediate' from dual union all
select 36541,'036541','1993-036BDB','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-28 00:00:00',101.14,773,853,74.02,'Intermediate' from dual union all
select 36542,'036542','1993-036BDC','deb Kosmos-2251','',65138,'Reentered','2016-12-05 00:00:00','2010-04-27 00:00:00',98.6,555,829,73.81,'Intermediate' from dual union all
select 36543,'036543','1993-036BDD','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-27 00:00:00',100.76,723,867,73.85,'Intermediate' from dual union all
select 36544,'036544','1993-036BDE','deb Kosmos-2251','',65138,'Reentered','2015-04-29 00:00:00','2010-04-28 00:00:00',98.49,515,858,73.96,'Intermediate' from dual union all
select 36545,'036545','1993-036BDF','deb Kosmos-2251','',65138,'Reentered','2011-07-28 00:00:00','2010-04-27 00:00:00',96.35,448,720,73.87,'Intermediate' from dual union all
select 36546,'036546','1993-036BDG','deb Kosmos-2251','',65138,'Reentered','2012-04-11 00:00:00','2010-04-28 00:00:00',97.63,521,769,73.88,'Intermediate' from dual union all
select 36547,'036547','1993-036BDH','deb Kosmos-2251','',65138,'Reentered','2014-02-20 00:00:00','2010-04-28 00:00:00',98.14,543,796,74.09,'Intermediate' from dual union all
select 36548,'036548','1993-036BDJ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-28 00:00:00',100.07,732,792,74.03,'Intermediate' from dual;
insert into SATELLITE
select 36549,'036549','1993-036BDK','deb Kosmos-2251','',65138,'Reentered','2014-11-02 00:00:00','2010-04-27 00:00:00',99.13,639,796,74.03,'Intermediate' from dual union all
select 3655,'003655','1969-004B','11S510','11S510',18271,'Reentered','1969-01-15 00:00:00','1969-01-14 00:00:00',87.79,148,183,51.7,'Intermediate' from dual union all
select 36550,'036550','1993-036BDL','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-28 00:00:00',101.7,773,905,74.05,'Intermediate' from dual union all
select 36551,'036551','1993-036BDM','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-28 00:00:00',99.28,638,811,73.9,'Intermediate' from dual union all
select 36552,'036552','1993-036BDN','deb Kosmos-2251','',65138,'Reentered','2014-11-27 00:00:00','2010-04-28 00:00:00',99.24,666,779,74,'Intermediate' from dual union all
select 36553,'036553','1993-036BDP','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-28 00:00:00',99.12,661,773,74.05,'Intermediate' from dual union all
select 36554,'036554','1993-036BDQ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-27 00:00:00',99.5,674,796,74.02,'Intermediate' from dual union all
select 36555,'036555','1993-036BDR','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-28 00:00:00',98.3,586,768,74,'Intermediate' from dual union all
select 36556,'036556','1993-036BDS','deb Kosmos-2251','',65138,'Reentered','2015-04-10 00:00:00','2010-04-28 00:00:00',100.02,732,787,74.03,'Intermediate' from dual union all
select 36557,'036557','1993-036BDT','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-28 00:00:00',100.78,762,829,73.99,'Intermediate' from dual union all
select 36558,'036558','1993-036BDU','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-27 00:00:00',100.3,760,785,74.05,'Intermediate' from dual union all
select 36559,'036559','1993-036BDV','deb Kosmos-2251','',65138,'In Earth orbit','','2010-04-28 00:00:00',105.96,775,1304,74.13,'Intermediate' from dual union all
select 3656,'003656','1969-005A','Soyuz-5','Soyuz No. 13',18272,'Landed','1969-01-18 00:00:00','1969-01-17 00:00:00',88.81,205,227,51.69,'Intermediate' from dual union all
select 36560,'036560','1993-036BDW','deb Kosmos-2251','',65138,'Reentered','2010-06-12 00:00:00','2010-04-28 00:00:00',92.65,325,484,73.91,'Intermediate' from dual union all
select 36561,'036561','1993-036BDX','deb Kosmos-2251','',65138,'Reentered','2016-03-04 00:00:00','2010-04-28 00:00:00',99.75,674,820,74.11,'Intermediate' from dual union all
select 36562,'036562','1997-051WR','deb Iridium 33','',66131,'Reentered','2014-02-28 00:00:00','2010-04-27 00:00:00',99.03,648,777,86.09,'Polar' from dual union all
select 36563,'036563','1997-051WS','deb Iridium 33','',66131,'In Earth orbit','','2010-04-27 00:00:00',100.37,752,801,86.43,'Polar' from dual union all
select 36564,'036564','1997-051WT','deb Iridium 33','',66131,'Reentered','2013-05-07 00:00:00','2010-04-28 00:00:00',99.73,724,768,86.01,'Polar' from dual union all
select 36565,'036565','1997-051WU','deb Iridium 33','',66131,'Reentered','2011-07-15 00:00:00','2010-04-28 00:00:00',98.15,610,731,86.21,'Polar' from dual union all
select 36566,'036566','1997-051WV','deb Iridium 33','',66131,'In Earth orbit','','2010-04-28 00:00:00',103.69,773,1093,86.44,'Polar' from dual union all
select 36567,'036567','1997-051WW','deb Iridium 33','',66131,'Reentered','2010-07-03 00:00:00','2010-04-27 00:00:00',96.84,566,648,86.51,'Polar' from dual union all
select 36568,'036568','1997-051WX','deb Iridium 33','',66131,'Reentered','2017-05-01 00:00:00','2010-04-28 00:00:00',100.05,748,774,86.31,'Polar' from dual union all
select 36569,'036569','1997-051WY','deb Iridium 33','',66131,'Reentered','2011-03-18 00:00:00','2010-04-28 00:00:00',99.98,653,863,86.38,'Polar' from dual union all
select 3657,'003657','1969-005B','11S510','11S510',18272,'Reentered','1969-01-17 00:00:00','1969-01-16 00:00:00',88.32,181,202,51.67,'Intermediate' from dual union all
select 36570,'036570','1997-051WZ','deb Iridium 33','',66131,'Reentered','2014-04-10 00:00:00','2010-04-27 00:00:00',99.58,681,796,86.29,'Polar' from dual union all
select 36571,'036571','2006-002AA','deb ALOS','',68174,'In Earth orbit','','2010-04-29 00:00:00',98.65,693,696,98.15,'Sun-Synch' from dual union all
select 36572,'036572','2010-019A','Atlantis (STS-132)','OV-104',68879,'Landed','2010-05-26 00:00:00','2010-05-15 00:00:00',88.85,200,236,51.64,'Intermediate' from dual union all
select 36573,'036573','2010-020A','Hayato','K-SAT',68880,'Reentered','2010-06-28 00:00:00','2010-05-23 00:00:00',90.4,283,306,29.97,'Intermediate' from dual union all
select 36574,'036574','2010-020B','Waseda-Sat 2','Waseda-Sat 2',68880,'Reentered','2010-07-12 00:00:00','2010-05-23 00:00:00',90.44,290,302,29.98,'Intermediate' from dual union all
select 36575,'036575','2010-020C','Negai*','Negai-star (Negai*)',68880,'Reentered','2010-06-26 00:00:00','2010-05-23 00:00:00',90.42,286,305,29.97,'Intermediate' from dual union all
select 36576,'036576','2010-020D','Akatsuki','PLANET-C',68880,'Deep Space','2010-05-21 00:00:00','2010-05-20 00:00:00',526.45,298,56249,31.47,'Earth Escape' from dual union all
select 36577,'036577','2010-020E','Ikaros','Ikaros',68880,'Deep Space','2010-05-21 00:00:00','2010-05-20 00:00:00',526.45,298,56249,31.47,'Earth Escape' from dual union all
select 36578,'036578','2010-020F','Shi''nen','Unitec-1',68880,'Deep Space','2010-05-21 00:00:00','2010-05-20 00:00:00',526.45,298,56249,31.47,'Earth Escape' from dual union all
select 36579,'036579','2010-020G','H-2A F17 Stage 2','H-2A Stage 2',68880,'Deep Space','2010-05-21 00:00:00','2010-05-20 00:00:00',526.45,298,56249,31.47,'Earth Escape' from dual union all
select 3658,'003658','1969-005C','Soyuz-5 TV camera','',18272,'Reentered','1969-01-18 00:00:00','1969-01-16 00:00:00',88.94,211,234,51.7,'Intermediate' from dual union all
select 36580,'036580','2010-020H','PAF900M','Adapter',68880,'Deep Space','2010-05-21 00:00:00','2010-05-20 00:00:00',526.45,298,56249,31.47,'Earth Escape' from dual union all
select 36581,'036581','2010-021A','Astra 3B','Astra 3B',68881,'In Earth orbit','','2010-07-23 00:00:00',1436.04,35765,35806,.08,'Stationary' from dual union all
select 36582,'036582','2010-021B','COMSATBw-2','COMSATBw-2',68881,'In Earth orbit','','2010-05-31 00:00:00',1436.09,35780,35793,.08,'Stationary' from dual union all
select 36583,'036583','2010-021C','ESC-A 551','Ariane 5 ESC-A',68881,'In Earth orbit','','2010-06-01 00:00:00',631.41,262,35742,3.03,'GEO Transfer' from dual union all
select 36584,'036584','2010-021D','Sylda 5A','Sylda 5 No. 38A',68881,'In Earth orbit','','2010-05-29 00:00:00',630.92,259,35720,3.16,'GEO Transfer' from dual union all
select 36585,'036585','2010-022A','GPS IIF-1','GPS SVN 62',68882,'In Earth orbit','','2010-06-04 00:00:00',729.35,20443,20481,54.98,'Medium' from dual union all
select 36586,'036586','2010-022B','Delta 349','DCSS-4 F09',68882,'In Earth orbit','','2010-06-05 00:00:00',754.12,20457,21676,54.72,'Medium' from dual union all
select 36587,'036587','2009-075B','Briz-M No. 99510','Briz-M',68756,'In Earth orbit','','2010-05-26 00:00:00',722.1,4958,35610,20.89,'GEO Transfer' from dual union all
select 36588,'036588','2010-023A','SERVIS-2','SERVIS-2',68883,'In Earth orbit','','2010-06-09 00:00:00',109.37,1185,1211,100.44,'Sun-Synch' from dual union all
select 36589,'036589','2010-023B','Briz-KM No. 72516','Briz-KM 14S45',68883,'In Earth orbit','','2010-06-09 00:00:00',106,876,1206,100.56,'Sun-Synch' from dual union all
select 3659,'003659','1968-095C','deb Kosmos-250','Tselina-O part',16822,'Reentered','1969-04-08 00:00:00','1969-01-17 00:00:00',94.1,462,488,73.94,'Intermediate' from dual union all
select 36590,'036590','2010-024A','Beidou DW 4','di si ke beidou daohang',68884,'In Earth orbit','','2010-06-09 00:00:00',1436.04,35777,35794,1.83,'Stationary' from dual union all
select 36591,'036591','2010-024B','CZ-3C Stage 3','CZ-3C Stage 3',68884,'In Earth orbit','','2010-06-11 00:00:00',626.99,209,35567,20.53,'GEO Transfer' from dual union all
select 36592,'036592','2010-025A','Badr 5','Arabsat 5B',68885,'In Earth orbit','','2010-06-30 00:00:00',1436.11,35777,35797,.07,'Stationary' from dual union all
select 36593,'036593','2010-025B','Briz-M No. 99513','Briz-M',68885,'In Earth orbit','','2010-07-01 00:00:00',697.06,5006,34322,18.2,'GEO Transfer' from dual union all
select 36594,'036594','2010-025C','Briz-M DTB','Briz-M torus tank',68885,'In Earth orbit','','2010-06-11 00:00:00',301.84,329,16726,49.49,'Highly Ellip' from dual union all
select 36595,'036595','2010-026A','Dragon Qual Unit','Dragon Spacecraft',68886,'Reentered Attached','2010-06-27 00:00:00','2010-06-11 00:00:00',89.44,234,260,34.5,'Intermediate' from dual union all
select 36596,'036596','2010-027A','Shi Jian 12','shijian shi erhao weixin',68887,'In Earth orbit','','2010-06-22 00:00:00',96.41,575,599,97.68,'Sun-Synch' from dual union all
select 36597,'036597','2010-027B','CZ-2D Stage 2','CZ-2D Stage 2',68887,'In Earth orbit','','2010-06-22 00:00:00',96.99,566,664,97.56,'Sun-Synch' from dual union all
select 36598,'036598','2010-028A','Picard','Picard',68888,'In Earth orbit','','2010-06-22 00:00:00',99.33,726,728,98.29,'Sun-Synch' from dual union all
select 36599,'036599','2010-028B','PRISMA-Mango','Prisma MAIN',68888,'In Earth orbit','','2010-06-22 00:00:00',99.92,725,785,98.28,'Sun-Synch' from dual union all
select 366,'000366','62 A Nu 2','8K72KE','8K72KE',8147,'Reentered','1962-08-14 00:00:00','1962-08-13 00:00:00',88.34,169,216,64.8,'Intermediate' from dual union all
select 3660,'003660','1968-095D','deb Kosmos-250','Tselina-O part',16822,'Reentered','1969-05-05 00:00:00','1969-01-22 00:00:00',94.58,495,502,73.95,'Intermediate' from dual union all
select 36600,'036600','2010-028C','R-36M2 DS','R-36M2 DS',68888,'In Earth orbit','','2010-06-22 00:00:00',104.92,714,1268,98.24,'Sun-Synch' from dual union all
select 36601,'036601','2010-028D','Gazodinamicheskiy Ekran','GDS (Gas dynamic shield)',68888,'In Earth orbit','','2010-06-22 00:00:00',99.56,726,749,98.28,'Sun-Synch' from dual union all
select 36602,'036602','2010-028E','Dnepr Platform A/Fairing','',68888,'In Earth orbit','','2010-06-22 00:00:00',98.99,694,726,98.29,'Sun-Synch' from dual union all
select 36603,'036603','2010-029A','Soyuz TMA-19','Soyuz 11F732A17 No. 229',68889,'Landed','2010-11-26 00:00:00','2010-06-23 00:00:00',91.59,347,359,51.65,'Intermediate' from dual union all
select 36604,'036604','2010-029B','11S510','11S510',68889,'Reentered','2010-06-19 00:00:00','2010-06-18 00:00:00',88.31,177,204,51.61,'Intermediate' from dual union all
select 36605,'036605','2010-030A','Tandem-X','TDX',68890,'In Earth orbit','','2010-06-28 00:00:00',94.68,497,510,97.45,'Sun-Synch' from dual union all
select 36606,'036606','2010-030B','R-36M2 DS','R-36M2 DS',68890,'In Earth orbit','','2010-06-28 00:00:00',101.32,498,1145,97.39,'Sun-Synch' from dual union all
select 36607,'036607','2010-030C','Dnepr plume shield','GDS',68890,'Reentered','2012-07-12 00:00:00','2010-06-28 00:00:00',94.41,474,505,97.45,'Sun-Synch' from dual union all
select 36608,'036608','2010-031A','''Ofeq-9','''Ofeq-9',68891,'In Earth orbit','','2010-06-30 00:00:00',93.88,343,586,141.78,'Retrograde' from dual union all
select 36609,'036609','2010-031B','AUS-51','AUS-51',68891,'Reentered','2011-06-22 00:00:00','2010-06-30 00:00:00',93.45,300,587,141.79,'Retrograde' from dual union all
select 3661,'003661','1968-095E','deb Kosmos-250','Tselina-O part',16822,'Reentered','1969-04-17 00:00:00','1969-01-21 00:00:00',94.3,474,496,74.04,'Intermediate' from dual union all
select 36610,'036610','1993-036BDY','deb Kosmos-2251','',65138,'Reentered','2012-10-28 00:00:00','2010-06-23 00:00:00',97.74,544,758,74,'Intermediate' from dual union all
select 36611,'036611','1993-036BDZ','deb Kosmos-2251','',65138,'Reentered','2012-03-26 00:00:00','2010-06-24 00:00:00',96.77,452,756,74.09,'Intermediate' from dual union all
select 36612,'036612','1993-036BEA','deb Kosmos-2251','',65138,'Reentered','2011-10-04 00:00:00','2010-06-23 00:00:00',97.73,561,740,74.05,'Intermediate' from dual union all
select 36613,'036613','1993-036BEB','deb Kosmos-2251','',65138,'Reentered','2015-06-14 00:00:00','2010-06-24 00:00:00',98.7,617,776,74.04,'Intermediate' from dual union all
select 36614,'036614','1993-036BEC','deb Kosmos-2251','',65138,'In Earth orbit','','2010-06-24 00:00:00',100.56,767,804,74.03,'Intermediate' from dual union all
select 36615,'036615','1993-036BED','deb Kosmos-2251','',65138,'In Earth orbit','','2010-06-23 00:00:00',102.94,767,1029,74.06,'Intermediate' from dual union all
select 36616,'036616','1993-036BEE','deb Kosmos-2251','',65138,'Reentered','2014-12-12 00:00:00','2010-06-24 00:00:00',99.38,676,782,74.03,'Intermediate' from dual union all
select 36617,'036617','1993-036BEF','deb Kosmos-2251','',65138,'In Earth orbit','','2010-06-23 00:00:00',101.11,653,970,74,'Intermediate' from dual union all
select 36618,'036618','1993-036BEG','deb Kosmos-2251','',65138,'Reentered','2013-10-31 00:00:00','2010-06-24 00:00:00',96.41,416,757,73.96,'Intermediate' from dual union all
select 36619,'036619','1993-036BEH','deb Kosmos-2251','',65138,'Reentered','2011-05-04 00:00:00','2010-06-24 00:00:00',96.11,453,692,73.92,'Intermediate' from dual union all
select 3662,'003662','1968-095F','deb Kosmos-250','Tselina-O part',16822,'Reentered','1969-03-26 00:00:00','1969-01-22 00:00:00',93.8,449,472,74.02,'Intermediate' from dual union all
select 36620,'036620','1993-036BEJ','deb Kosmos-2251','',65138,'Reentered','2014-05-09 00:00:00','2010-06-23 00:00:00',98.29,553,801,74.01,'Intermediate' from dual union all
select 36621,'036621','1993-036BEK','deb Kosmos-2251','',65138,'Reentered','2011-08-12 00:00:00','2010-06-24 00:00:00',96.52,453,731,73.94,'Intermediate' from dual union all
select 36622,'036622','1993-036BEL','deb Kosmos-2251','',65138,'In Earth orbit','','2010-06-24 00:00:00',99.04,642,784,74.02,'Intermediate' from dual union all
select 36623,'036623','1993-036BEM','deb Kosmos-2251','',65138,'In Earth orbit','','2010-06-24 00:00:00',99.47,529,938,74.1,'Intermediate' from dual union all
select 36624,'036624','1993-036BEN','deb Kosmos-2251','',65138,'Reentered','2014-03-10 00:00:00','2010-06-23 00:00:00',99.3,673,778,74.2,'Intermediate' from dual union all
select 36625,'036625','1993-036BEP','deb Kosmos-2251','',65138,'In Earth orbit','','2010-06-23 00:00:00',99.21,660,782,73.88,'Intermediate' from dual union all
select 36626,'036626','1993-036BEQ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-06-24 00:00:00',97.39,486,782,74.05,'Intermediate' from dual union all
select 36627,'036627','1993-036BER','deb Kosmos-2251','',65138,'In Earth orbit','','2010-06-23 00:00:00',99.48,700,768,74.26,'Intermediate' from dual union all
select 36628,'036628','1993-036BES','deb Kosmos-2251','',65138,'Reentered','2011-11-07 00:00:00','2010-06-24 00:00:00',96.78,471,737,73.93,'Intermediate' from dual union all
select 36629,'036629','1993-036BET','deb Kosmos-2251','',65138,'Reentered','2015-02-11 00:00:00','2010-06-23 00:00:00',99.76,618,877,73.99,'Intermediate' from dual union all
select 3663,'003663','1969-006A','OSO 5','OSO F',18273,'Reentered','1984-04-02 00:00:00','1969-03-09 00:00:00',95.67,539,563,32.98,'Intermediate' from dual union all
select 36630,'036630','1993-036BEU','deb Kosmos-2251','',65138,'In Earth orbit','','2010-06-24 00:00:00',101.02,768,845,73.99,'Intermediate' from dual union all
select 36631,'036631','1993-036BEV','deb Kosmos-2251','',65138,'Reentered','2011-09-01 00:00:00','2010-06-24 00:00:00',96.36,441,728,73.95,'Intermediate' from dual union all
select 36632,'036632','1993-036BEW','deb Kosmos-2251','',65138,'In Earth orbit','','2010-06-24 00:00:00',100.87,769,830,74.01,'Intermediate' from dual union all
select 36633,'036633','1993-036BEX','deb Kosmos-2251','',65138,'Reentered','2010-11-20 00:00:00','2010-06-23 00:00:00',93.78,325,593,73.99,'Intermediate' from dual union all
select 36634,'036634','1993-036BEY','deb Kosmos-2251','',65138,'Reentered','2012-03-04 00:00:00','2010-06-24 00:00:00',97.44,510,763,73.88,'Intermediate' from dual union all
select 36635,'036635','1993-036BEZ','deb Kosmos-2251','',65138,'Reentered','2012-07-26 00:00:00','2010-06-24 00:00:00',97.7,537,761,74.01,'Intermediate' from dual union all
select 36636,'036636','1993-036BFA','deb Kosmos-2251','',65138,'Reentered','2014-06-21 00:00:00','2010-06-24 00:00:00',100.03,747,773,74.03,'Intermediate' from dual union all
select 36637,'036637','1993-036BFB','deb Kosmos-2251','',65138,'Reentered','2012-02-01 00:00:00','2010-06-24 00:00:00',96.84,462,753,74.09,'Intermediate' from dual union all
select 36638,'036638','1993-036BFC','deb Kosmos-2251','',65138,'Reentered','2011-04-26 00:00:00','2010-06-24 00:00:00',95.02,371,669,73.88,'Intermediate' from dual;
insert into SATELLITE
select 36639,'036639','1993-036BFD','deb Kosmos-2251','',65138,'Reentered','2014-10-26 00:00:00','2010-06-23 00:00:00',99.69,714,774,74.23,'Intermediate' from dual union all
select 3664,'003664','1969-006B','FW4D','FW4D',18273,'Reentered','1979-12-08 00:00:00','1969-03-28 00:00:00',95.6,537,559,32.98,'Intermediate' from dual union all
select 36640,'036640','1997-051XA','deb Iridium 33','',66131,'Reentered','2014-09-30 00:00:00','2010-06-24 00:00:00',100.21,762,775,86.36,'Polar' from dual union all
select 36641,'036641','1997-051XB','deb Iridium 33','',66131,'Reentered','2014-10-19 00:00:00','2010-06-23 00:00:00',100.63,757,820,86.32,'Polar' from dual union all
select 36642,'036642','1997-051XC','deb Iridium 33','',66131,'In Earth orbit','','2010-06-23 00:00:00',100.73,769,818,86.38,'Polar' from dual union all
select 36643,'036643','1997-051XD','deb Iridium 33','',66131,'In Earth orbit','','2010-06-24 00:00:00',99.52,717,755,86.33,'Polar' from dual union all
select 36644,'036644','1999-025DXN','deb FY-1C','',66706,'In Earth orbit','','2010-06-20 00:00:00',106.7,846,1302,98.4,'Sun-Synch' from dual union all
select 36645,'036645','1999-025DXP','deb FY-1C','',66706,'In Earth orbit','','2010-06-19 00:00:00',107.27,852,1349,98.5,'Sun-Synch' from dual union all
select 36646,'036646','1999-025DXQ','deb FY-1C','',66706,'Reentered','2011-10-21 00:00:00','2010-06-19 00:00:00',96.84,362,853,96.71,'Sun-Synch' from dual union all
select 36647,'036647','1999-025DXR','deb FY-1C','',66706,'In Earth orbit','','2010-06-14 00:00:00',105.45,797,1234,96.16,'Sun-Synch' from dual union all
select 36648,'036648','1999-025DXS','deb FY-1C','',66706,'In Earth orbit','','2010-06-11 00:00:00',100.51,621,945,97.98,'Sun-Synch' from dual union all
select 36649,'036649','1999-025DXT','deb FY-1C','',66706,'In Earth orbit','','2010-06-23 00:00:00',100.48,717,846,98.93,'Sun-Synch' from dual union all
select 3665,'003665','1969-007A','OPS 7585','GAMBIT-3 Mission 4319',18274,'Reentered Attached','1969-02-03 00:00:00','1969-01-24 00:00:00',96.87,132,1086,106.15,'Retrograde' from dual union all
select 36650,'036650','1999-025DXU','deb FY-1C','',66706,'Reentered','2014-03-21 00:00:00','2010-06-16 00:00:00',100.41,556,1000,98.65,'Sun-Synch' from dual union all
select 36651,'036651','1999-025DXV','deb FY-1C','',66706,'In Earth orbit','','2010-06-23 00:00:00',100.72,609,976,98.36,'Sun-Synch' from dual union all
select 36652,'036652','1999-025DXW','deb FY-1C','',66706,'In Earth orbit','','2010-06-24 00:00:00',102.91,848,945,99.03,'Sun-Synch' from dual union all
select 36653,'036653','1999-025DXX','deb FY-1C','',66706,'Reentered','2014-12-11 00:00:00','2010-06-12 00:00:00',99.91,606,903,98.34,'Sun-Synch' from dual union all
select 36654,'036654','1999-025DXY','deb FY-1C','',66706,'In Earth orbit','','2010-06-18 00:00:00',101.17,581,1048,99.5,'Sun-Synch' from dual union all
select 36655,'036655','1999-025DXZ','deb FY-1C','',66706,'In Earth orbit','','2010-06-16 00:00:00',100.44,709,850,98.98,'Sun-Synch' from dual union all
select 36656,'036656','1999-025DYA','deb FY-1C','',66706,'In Earth orbit','','2010-06-19 00:00:00',106.99,821,1353,97.41,'Sun-Synch' from dual union all
select 36657,'036657','1999-025DYB','deb FY-1C','',66706,'In Earth orbit','','2010-06-19 00:00:00',99.81,644,855,99.41,'Sun-Synch' from dual union all
select 36658,'036658','1999-025DYC','deb FY-1C','',66706,'In Earth orbit','','2010-06-13 00:00:00',101.86,776,918,98.89,'Sun-Synch' from dual union all
select 36659,'036659','1999-025DYD','deb FY-1C','',66706,'In Earth orbit','','2010-06-18 00:00:00',104.29,855,1067,99.79,'Sun-Synch' from dual union all
select 3666,'003666','1969-007B','OPS 7585 sensor cover','',18274,'Reentered','1969-01-26 00:00:00','1969-01-23 00:00:00',92.8,130,694,106.11,'Retrograde' from dual union all
select 36660,'036660','1999-025DYE','deb FY-1C','',66706,'In Earth orbit','','2010-06-08 00:00:00',102.42,838,909,99.15,'Sun-Synch' from dual union all
select 36661,'036661','1999-025DYF','deb FY-1C','',66706,'In Earth orbit','','2010-06-17 00:00:00',102.38,852,890,98.91,'Sun-Synch' from dual union all
select 36662,'036662','1999-025DYG','deb FY-1C','',66706,'In Earth orbit','','2010-06-20 00:00:00',100.5,713,852,96.58,'Sun-Synch' from dual union all
select 36663,'036663','1999-025DYH','deb FY-1C','',66706,'In Earth orbit','','2010-06-15 00:00:00',102.19,853,872,98.76,'Sun-Synch' from dual union all
select 36664,'036664','1999-025DYJ','deb FY-1C','',66706,'In Earth orbit','','2010-06-24 00:00:00',101.68,710,967,98.78,'Sun-Synch' from dual union all
select 36665,'036665','1999-025DYK','deb FY-1C','',66706,'In Earth orbit','','2010-06-25 00:00:00',103.1,782,1028,98.48,'Sun-Synch' from dual union all
select 36666,'036666','1999-025DYL','deb FY-1C','',66706,'Reentered','2014-04-16 00:00:00','2010-06-16 00:00:00',99.05,416,1011,98.44,'Sun-Synch' from dual union all
select 36667,'036667','1999-025DYM','deb FY-1C','',66706,'In Earth orbit','','2010-06-24 00:00:00',103.67,828,1036,97.75,'Sun-Synch' from dual union all
select 36668,'036668','1999-025DYN','deb FY-1C','',66706,'In Earth orbit','','2010-06-21 00:00:00',102.4,832,913,99.14,'Sun-Synch' from dual union all
select 36669,'036669','1999-025DYP','deb FY-1C','',66706,'Reentered','2016-03-23 00:00:00','2010-06-21 00:00:00',100.14,691,840,99.03,'Sun-Synch' from dual union all
select 3667,'003667','1969-008A','Kosmos-264','Zenit-4M',18275,'Landed','1969-02-05 00:00:00','1969-01-25 00:00:00',89.53,206,297,69.92,'Intermediate' from dual union all
select 36670,'036670','1999-025DYQ','deb FY-1C','',66706,'In Earth orbit','','2010-06-22 00:00:00',101.71,806,874,98.83,'Sun-Synch' from dual union all
select 36671,'036671','1999-025DYR','deb FY-1C','',66706,'Reentered','2014-12-30 00:00:00','2010-06-14 00:00:00',100.28,576,968,97.34,'Sun-Synch' from dual union all
select 36672,'036672','1999-025DYS','deb FY-1C','',66706,'In Earth orbit','','2010-06-18 00:00:00',100.27,697,846,99,'Sun-Synch' from dual union all
select 36673,'036673','1999-025DYT','deb FY-1C','',66706,'In Earth orbit','','2010-06-21 00:00:00',99.94,642,870,99.05,'Sun-Synch' from dual union all
select 36674,'036674','1999-025DYU','deb FY-1C','',66706,'Reentered','2016-02-29 00:00:00','2010-06-23 00:00:00',100.86,752,847,99.08,'Sun-Synch' from dual union all
select 36675,'036675','1999-025DYV','deb FY-1C','',66706,'In Earth orbit','','2010-06-16 00:00:00',102.14,827,893,99.26,'Sun-Synch' from dual union all
select 36676,'036676','1999-025DYW','deb FY-1C','',66706,'In Earth orbit','','2010-06-21 00:00:00',100.44,702,856,98.88,'Sun-Synch' from dual union all
select 36677,'036677','1999-025DYX','deb FY-1C','',66706,'In Earth orbit','','2010-06-16 00:00:00',102.75,841,937,99.18,'Sun-Synch' from dual union all
select 36678,'036678','1999-025DYY','deb FY-1C','',66706,'In Earth orbit','','2010-06-22 00:00:00',100.64,670,908,98.81,'Sun-Synch' from dual union all
select 36679,'036679','1999-025DYZ','deb FY-1C','',66706,'In Earth orbit','','2010-06-21 00:00:00',102.98,836,964,99.16,'Sun-Synch' from dual union all
select 3668,'003668','1969-008B','11A57I','11A57I',18275,'Reentered','1969-01-30 00:00:00','1969-01-25 00:00:00',89.36,206,280,69.94,'Intermediate' from dual union all
select 36680,'036680','1999-025DZA','deb FY-1C','',66706,'In Earth orbit','','2010-06-16 00:00:00',102.12,708,1010,99.1,'Sun-Synch' from dual union all
select 36681,'036681','1999-025DZB','deb FY-1C','',66706,'In Earth orbit','','2010-06-21 00:00:00',102.72,853,921,99.2,'Sun-Synch' from dual union all
select 36682,'036682','1999-025DZC','deb FY-1C','',66706,'In Earth orbit','','2010-06-22 00:00:00',105.98,848,1232,98.44,'Sun-Synch' from dual union all
select 36683,'036683','1999-025DZD','deb FY-1C','',66706,'Reentered','2012-10-04 00:00:00','2010-06-22 00:00:00',98.21,529,817,99.04,'Sun-Synch' from dual union all
select 36684,'036684','1999-025DZE','deb FY-1C','',66706,'In Earth orbit','','2010-06-19 00:00:00',100.98,752,859,99.48,'Sun-Synch' from dual union all
select 36685,'036685','1999-025DZF','deb FY-1C','',66706,'In Earth orbit','','2010-06-18 00:00:00',99.4,604,856,98.82,'Sun-Synch' from dual union all
select 36686,'036686','1999-025DZG','deb FY-1C','',66706,'In Earth orbit','','2010-06-18 00:00:00',99.85,623,880,98.96,'Sun-Synch' from dual union all
select 36687,'036687','1999-025DZH','deb FY-1C','',66706,'In Earth orbit','','2010-06-21 00:00:00',102.72,815,959,98.69,'Sun-Synch' from dual union all
select 36688,'036688','1999-025DZJ','deb FY-1C','',66706,'In Earth orbit','','2010-06-20 00:00:00',101.15,757,869,99.08,'Sun-Synch' from dual union all
select 36689,'036689','1999-025DZK','deb FY-1C','',66706,'In Earth orbit','','2010-06-21 00:00:00',106.35,784,1331,98.46,'Sun-Synch' from dual union all
select 3669,'003669','1969-009A','Isis 1','Isis A',18276,'In Earth orbit','','1969-03-16 00:00:00',128.33,575,3525,88.41,'Medium' from dual union all
select 36690,'036690','1999-025DZL','deb FY-1C','',66706,'Reentered','2011-08-08 00:00:00','2010-06-21 00:00:00',96.67,467,732,98.55,'Sun-Synch' from dual union all
select 36691,'036691','1999-025DZM','deb FY-1C','',66706,'In Earth orbit','','2010-06-16 00:00:00',104.11,831,1074,98.72,'Sun-Synch' from dual union all
select 36692,'036692','1999-025DZN','deb FY-1C','',66706,'In Earth orbit','','2010-06-19 00:00:00',103.66,844,1020,99.99,'Sun-Synch' from dual union all
select 36693,'036693','1999-025DZP','deb FY-1C','',66706,'In Earth orbit','','2010-06-22 00:00:00',101.34,608,1037,97.61,'Sun-Synch' from dual union all
select 36694,'036694','1999-025DZQ','deb FY-1C','',66706,'In Earth orbit','','2010-06-18 00:00:00',102.23,777,952,98.71,'Sun-Synch' from dual union all
select 36695,'036695','1999-025DZR','deb FY-1C','',66706,'In Earth orbit','','2010-06-20 00:00:00',100.36,572,980,98.75,'Sun-Synch' from dual union all
select 36696,'036696','1999-025DZS','deb FY-1C','',66706,'In Earth orbit','','2010-06-21 00:00:00',102.44,852,897,98.73,'Sun-Synch' from dual union all
select 36697,'036697','1999-025DZT','deb FY-1C','',66706,'In Earth orbit','','2010-06-22 00:00:00',98.67,530,861,98.75,'Sun-Synch' from dual union all
select 36698,'036698','1999-025DZU','deb FY-1C','',66706,'In Earth orbit','','2010-06-25 00:00:00',100.78,740,852,99.05,'Sun-Synch' from dual union all
select 36699,'036699','1999-025DZV','deb FY-1C','',66706,'In Earth orbit','','2010-06-21 00:00:00',101.45,756,899,98.87,'Sun-Synch' from dual union all
select 367,'000367','62 A Xi 1','Kosmos-8','DS-K-8 No. 1',8158,'Reentered','1963-08-17 00:00:00','1962-09-21 00:00:00',92.84,259,569,48.97,'Intermediate' from dual union all
select 3670,'003670','1969-009B','FW4D','FW4D S/N 2226-10',18276,'In Earth orbit','','1969-03-02 00:00:00',128.21,575,3513,88.41,'Medium' from dual union all
select 36700,'036700','1999-025DZW','deb FY-1C','',66706,'In Earth orbit','','2010-06-25 00:00:00',104.6,854,1098,99.85,'Sun-Synch' from dual union all
select 36701,'036701','1999-025DZX','deb FY-1C','',66706,'In Earth orbit','','2010-06-22 00:00:00',106.51,848,1282,102.76,'Sun-Synch' from dual union all
select 36702,'036702','1999-025DZY','deb FY-1C','',66706,'In Earth orbit','','2010-06-11 00:00:00',103.28,815,1013,98.38,'Sun-Synch' from dual union all
select 36703,'036703','1999-025DZZ','deb FY-1C','',66706,'In Earth orbit','','2010-06-10 00:00:00',103.63,824,1037,97.83,'Sun-Synch' from dual union all
select 36704,'036704','1999-025EAA','deb FY-1C','',66706,'Reentered','2014-11-01 00:00:00','2010-06-23 00:00:00',99.49,632,836,99.31,'Sun-Synch' from dual union all
select 36705,'036705','1999-025EAB','deb FY-1C','',66706,'Reentered','2014-11-12 00:00:00','2010-06-23 00:00:00',99.64,638,845,98.98,'Sun-Synch' from dual union all
select 36706,'036706','1999-025EAC','deb FY-1C','',66706,'In Earth orbit','','2010-06-18 00:00:00',102.99,801,999,98.5,'Sun-Synch' from dual union all
select 36707,'036707','1999-025EAD','deb FY-1C','',66706,'In Earth orbit','','2010-06-15 00:00:00',102.48,835,916,98.97,'Sun-Synch' from dual union all
select 36708,'036708','1999-025EAE','deb FY-1C','',66706,'Reentered','2011-12-10 00:00:00','2010-06-23 00:00:00',98.47,435,936,98.14,'Sun-Synch' from dual union all
select 36709,'036709','1999-025EAF','deb FY-1C','',66706,'Reentered','2013-11-01 00:00:00','2010-07-08 00:00:00',99.03,585,840,98.86,'Sun-Synch' from dual union all
select 3671,'003671','1969-008C','KDU','KDU',18275,'Reentered','1969-02-13 00:00:00','1969-02-04 00:00:00',89.58,187,321,69.91,'Intermediate' from dual union all
select 36710,'036710','1999-025EAG','deb FY-1C','',66706,'In Earth orbit','','2010-06-19 00:00:00',101.24,748,887,98.85,'Sun-Synch' from dual union all
select 36711,'036711','1999-025EAH','deb FY-1C','',66706,'In Earth orbit','','2010-06-11 00:00:00',102.92,767,1027,100.81,'Sun-Synch' from dual union all
select 36712,'036712','1999-025EAJ','deb FY-1C','',66706,'In Earth orbit','','2010-06-15 00:00:00',100.77,734,857,98.97,'Sun-Synch' from dual union all
select 36713,'036713','1999-025EAK','deb FY-1C','',66706,'In Earth orbit','','2010-06-25 00:00:00',99.71,533,956,98.62,'Sun-Synch' from dual union all
select 36714,'036714','1999-025EAL','deb FY-1C','',66706,'In Earth orbit','','2010-06-08 00:00:00',102.27,839,894,99.08,'Sun-Synch' from dual union all
select 36715,'036715','1999-025EAM','deb FY-1C','',66706,'In Earth orbit','','2010-06-20 00:00:00',99.87,605,900,98.59,'Sun-Synch' from dual union all
select 36716,'036716','1999-025EAN','deb FY-1C','',66706,'In Earth orbit','','2010-06-10 00:00:00',103.92,831,1057,99.02,'Sun-Synch' from dual union all
select 36717,'036717','1999-025EAP','deb FY-1C','',66706,'In Earth orbit','','2010-06-19 00:00:00',100.29,647,898,99.09,'Sun-Synch' from dual union all
select 36718,'036718','1999-025EAQ','deb FY-1C','',66706,'Reentered','2014-04-09 00:00:00','2010-06-22 00:00:00',99.59,651,827,99.13,'Sun-Synch' from dual union all
select 36719,'036719','1999-025EAR','deb FY-1C','',66706,'In Earth orbit','','2010-06-09 00:00:00',102.06,772,940,98.58,'Sun-Synch' from dual union all
select 3672,'003672','1969-010A','OPS 3890','CORONA CR-6',18277,'Reentered Attached','1969-02-24 00:00:00','1969-02-13 00:00:00',88.72,137,285,81.54,'Intermediate' from dual union all
select 36720,'036720','1999-025EAS','deb FY-1C','',66706,'Reentered','2012-08-01 00:00:00','2010-06-20 00:00:00',98.22,537,810,99,'Sun-Synch' from dual union all
select 36721,'036721','1999-025EAT','deb FY-1C','',66706,'Reentered','2014-10-06 00:00:00','2010-06-19 00:00:00',99.73,660,832,99.08,'Sun-Synch' from dual union all
select 36722,'036722','1999-025EAU','deb FY-1C','',66706,'Reentered','2013-11-21 00:00:00','2010-06-19 00:00:00',99.3,603,848,99.02,'Sun-Synch' from dual union all
select 36723,'036723','1999-025EAV','deb FY-1C','',66706,'Reentered','2015-01-18 00:00:00','2010-06-22 00:00:00',100.4,708,847,98.88,'Sun-Synch' from dual union all
select 36724,'036724','1999-025EAW','deb FY-1C','',66706,'In Earth orbit','','2010-06-18 00:00:00',103.06,800,1006,99.5,'Sun-Synch' from dual union all
select 36725,'036725','1999-025EAX','deb FY-1C','',66706,'Reentered','2015-03-23 00:00:00','2010-06-21 00:00:00',99.46,628,838,99.52,'Sun-Synch' from dual union all
select 36726,'036726','1999-025EAY','deb FY-1C','',66706,'In Earth orbit','','2010-06-20 00:00:00',99.79,598,899,98.98,'Sun-Synch' from dual union all
select 36727,'036727','1999-025EAZ','deb FY-1C','',66706,'In Earth orbit','','2010-06-21 00:00:00',100.13,676,854,99.04,'Sun-Synch' from dual union all
select 36728,'036728','1999-025EBA','deb FY-1C','',66706,'Reentered','2013-05-20 00:00:00','2010-06-21 00:00:00',98.84,583,824,99.03,'Sun-Synch' from dual;
insert into SATELLITE
select 36729,'036729','1999-025EBB','deb FY-1C','',66706,'In Earth orbit','','2010-06-24 00:00:00',102.37,836,906,99.14,'Sun-Synch' from dual union all
select 3673,'003673','1969-010B','OPS 2644','P801-2',18277,'In Earth orbit','','1969-03-29 00:00:00',114.13,1398,1434,80.4,'Intermediate' from dual union all
select 36730,'036730','1999-025EBC','deb FY-1C','',66706,'In Earth orbit','','2010-06-23 00:00:00',101.32,787,856,99.64,'Sun-Synch' from dual union all
select 36731,'036731','1999-025EBD','deb FY-1C','',66706,'In Earth orbit','','2010-06-16 00:00:00',101.39,782,867,99.14,'Sun-Synch' from dual union all
select 36732,'036732','1999-025EBE','deb FY-1C','',66706,'In Earth orbit','','2010-06-21 00:00:00',98.73,545,851,98.95,'Sun-Synch' from dual union all
select 36733,'036733','1999-025EBF','deb FY-1C','',66706,'In Earth orbit','','2010-06-21 00:00:00',104.11,827,1079,99.06,'Sun-Synch' from dual union all
select 36734,'036734','1999-025EBG','deb FY-1C','',66706,'In Earth orbit','','2010-06-23 00:00:00',102.77,853,926,98.84,'Sun-Synch' from dual union all
select 36735,'036735','1999-025EBH','deb FY-1C','',66706,'In Earth orbit','','2010-06-23 00:00:00',101.28,735,904,99.13,'Sun-Synch' from dual union all
select 36736,'036736','1999-025EBJ','deb FY-1C','',66706,'In Earth orbit','','2010-06-06 00:00:00',102.66,829,939,99.06,'Sun-Synch' from dual union all
select 36737,'036737','1999-025EBK','deb FY-1C','',66706,'Reentered','2014-11-03 00:00:00','2010-06-20 00:00:00',100.08,649,876,99.07,'Sun-Synch' from dual union all
select 36738,'036738','1999-025EBL','deb FY-1C','',66706,'In Earth orbit','','2010-06-08 00:00:00',101.87,805,890,100.79,'Sun-Synch' from dual union all
select 36739,'036739','1999-025EBM','deb FY-1C','',66706,'In Earth orbit','','2010-06-24 00:00:00',99.83,657,844,99.03,'Sun-Synch' from dual union all
select 3674,'003674','1969-011A','INTELSAT III F-3','INTELSAT III F-3',18278,'In Earth orbit','','1969-03-17 00:00:00',1436.44,35779,35808,1.34,'Stationary' from dual union all
select 36740,'036740','1999-025EBN','deb FY-1C','',66706,'Reentered','2016-02-17 00:00:00','2010-06-20 00:00:00',102.95,830,967,98.7,'Sun-Synch' from dual union all
select 36741,'036741','1999-025EBP','deb FY-1C','',66706,'In Earth orbit','','2010-06-20 00:00:00',103.22,815,1007,98.47,'Sun-Synch' from dual union all
select 36742,'036742','1999-025EBQ','deb FY-1C','',66706,'In Earth orbit','','2010-06-07 00:00:00',103.06,832,974,98.89,'Sun-Synch' from dual union all
select 36743,'036743','1999-025EBR','deb FY-1C','',66706,'In Earth orbit','','2010-06-14 00:00:00',102.08,813,901,98.96,'Sun-Synch' from dual union all
select 36744,'036744','2010-032A','Chollian','COMS-1',68892,'In Earth orbit','','2010-07-09 00:00:00',1436.09,35783,35790,.01,'Stationary' from dual union all
select 36745,'036745','2010-032B','Arabsat 5A','Arabsat 5A',68892,'In Earth orbit','','2010-07-15 00:00:00',1427.44,35601,35633,.03,'Drift GEO' from dual union all
select 36746,'036746','2010-032C','ESC-A 552','Ariane 5 ESC-A',68892,'In Earth orbit','','2010-07-04 00:00:00',629.23,246,35646,1.53,'GEO Transfer' from dual union all
select 36747,'036747','2010-032D','Sylda 5A','Sylda 5',68892,'In Earth orbit','','2010-06-30 00:00:00',632.11,261,35779,1.97,'GEO Transfer' from dual union all
select 36748,'036748','2010-033A','Progress M-06M','Progress 7K-TGM No.406',68893,'Deorbited','2010-09-06 00:00:00','2010-07-02 00:00:00',89.49,235,264,51.65,'Intermediate' from dual union all
select 36749,'036749','2010-033B','11S510','11S510',68893,'Reentered','2010-07-03 00:00:00','2010-07-02 00:00:00',88.29,175,204,51.62,'Intermediate' from dual union all
select 3675,'003675','1969-012A','Kosmos-265','DS-P1-Yu No. 21',18279,'Reentered','1969-05-01 00:00:00','1969-03-11 00:00:00',91.27,261,413,70.99,'Intermediate' from dual union all
select 36750,'036750','2009-042D','deb Briz-M','',68723,'Reentered','2010-07-14 00:00:00','2010-07-01 00:00:00',97.46,413,862,47.91,'Intermediate' from dual union all
select 36751,'036751','2009-042E','deb Briz-M','',68723,'Reentered','2010-07-12 00:00:00','2010-07-02 00:00:00',102.26,410,1322,47.28,'Intermediate' from dual union all
select 36752,'036752','2009-042F','deb Briz-M','',68723,'Reentered','2010-07-03 00:00:00','2010-07-01 00:00:00',94.4,163,816,48.97,'Intermediate' from dual union all
select 36753,'036753','2009-042G','deb Briz-M','',68723,'Reentered','2010-09-27 00:00:00','2010-07-02 00:00:00',105.54,204,1836,48.43,'Intermediate' from dual union all
select 36754,'036754','2009-042H','deb Briz-M','',68723,'Reentered','2010-07-03 00:00:00','2010-07-01 00:00:00',96.27,170,990,47.35,'Intermediate' from dual union all
select 36755,'036755','2009-042J','deb Briz-M','',68723,'Reentered','2010-07-13 00:00:00','2010-07-02 00:00:00',101.57,187,1479,48.41,'Intermediate' from dual union all
select 36756,'036756','2009-042K','deb Briz-M','',68723,'Reentered','2010-07-22 00:00:00','2010-07-01 00:00:00',105.56,476,1566,47.36,'Intermediate' from dual union all
select 36757,'036757','2009-042L','deb Briz-M','',68723,'Reentered','2010-07-16 00:00:00','2010-07-02 00:00:00',99.25,343,1103,49.55,'Intermediate' from dual union all
select 36758,'036758','2009-042M','deb Briz-M','',68723,'Reentered','2010-09-01 00:00:00','2010-07-01 00:00:00',102.03,400,1310,48.31,'Intermediate' from dual union all
select 36759,'036759','2009-042N','deb Briz-M','',68723,'Reentered','2010-07-16 00:00:00','2010-07-02 00:00:00',99.29,167,1282,48.49,'Intermediate' from dual union all
select 3676,'003676','1969-012B','S1M','S1M',18279,'Reentered','1969-03-17 00:00:00','1969-02-14 00:00:00',91.45,270,422,71.01,'Intermediate' from dual union all
select 36760,'036760','2009-042P','deb Briz-M','',68723,'Reentered','2010-07-18 00:00:00','2010-07-02 00:00:00',100.14,169,1362,48.66,'Intermediate' from dual union all
select 36761,'036761','2009-042Q','deb Briz-M','',68723,'Reentered','2010-09-06 00:00:00','2010-07-01 00:00:00',114.15,377,2457,47.77,'Intermediate' from dual union all
select 36762,'036762','2009-042R','deb Briz-M','',68723,'Reentered','2010-08-12 00:00:00','2010-07-08 00:00:00',101.21,355,1277,47.94,'Intermediate' from dual union all
select 36763,'036763','2009-042S','deb Briz-M','',68723,'Reentered','2010-08-02 00:00:00','2010-07-08 00:00:00',107.84,297,1957,47.94,'Intermediate' from dual union all
select 36764,'036764','2009-042T','deb Briz-M','',68723,'Reentered','2010-09-08 00:00:00','2010-07-08 00:00:00',104.85,401,1574,47.89,'Intermediate' from dual union all
select 36765,'036765','2009-042U','deb Briz-M','',68723,'Reentered','2010-08-12 00:00:00','2010-07-08 00:00:00',102.26,358,1373,48.03,'Intermediate' from dual union all
select 36766,'036766','2009-042V','deb Briz-M','',68723,'Reentered','2010-09-17 00:00:00','2010-07-08 00:00:00',103.69,372,1494,47.74,'Intermediate' from dual union all
select 36767,'036767','2009-042W','deb Briz-M','',68723,'Reentered','2010-10-10 00:00:00','2010-07-08 00:00:00',109.97,345,2106,47.88,'Intermediate' from dual union all
select 36768,'036768','2009-042X','deb Briz-M','',68723,'Reentered','2010-08-04 00:00:00','2010-07-08 00:00:00',117.72,413,2744,47.03,'Intermediate' from dual union all
select 36769,'036769','2009-042Y','deb Briz-M','',68723,'Reentered','2010-07-24 00:00:00','2010-07-08 00:00:00',110.34,390,2095,48.47,'Intermediate' from dual union all
select 3677,'003677','1968-091AC','deb Kosmos-249','',16818,'Reentered','1980-05-29 00:00:00','1980-05-25 00:00:00',92.6,376,428,62.17,'Intermediate' from dual union all
select 36770,'036770','2009-042Z','deb Briz-M','',68723,'Reentered','2010-08-07 00:00:00','2010-07-08 00:00:00',106.73,318,1833,47.35,'Intermediate' from dual union all
select 36771,'036771','2009-042AA','deb Briz-M','',68723,'Reentered','2010-07-11 00:00:00','2010-07-07 00:00:00',107.84,335,1919,46.99,'Intermediate' from dual union all
select 36772,'036772','2009-042AB','deb Briz-M','',68723,'Reentered','2010-08-30 00:00:00','2010-07-08 00:00:00',102.52,537,1219,47.92,'Intermediate' from dual union all
select 36773,'036773','2009-042AC','deb Briz-M','',68723,'Reentered','2010-07-22 00:00:00','2010-07-08 00:00:00',99.07,466,962,48.2,'Intermediate' from dual union all
select 36774,'036774','2009-042AD','deb Briz-M','',68723,'Reentered','2010-07-29 00:00:00','2010-07-08 00:00:00',102.1,428,1289,46.89,'Intermediate' from dual union all
select 36775,'036775','2009-042AE','deb Briz-M','',68723,'Reentered','2010-08-06 00:00:00','2010-07-08 00:00:00',109.82,471,1966,47.14,'Intermediate' from dual union all
select 36776,'036776','2009-042AF','deb Briz-M','',68723,'Reentered','2010-08-18 00:00:00','2010-07-08 00:00:00',106.15,597,1499,47.57,'Intermediate' from dual union all
select 36777,'036777','2009-042AG','deb Briz-M','',68723,'Reentered','2010-07-24 00:00:00','2010-07-08 00:00:00',102.69,366,1405,48.53,'Intermediate' from dual union all
select 36778,'036778','2009-042AH','deb Briz-M','',68723,'Reentered','2010-09-06 00:00:00','2010-07-08 00:00:00',113.73,456,2340,47.67,'Intermediate' from dual union all
select 36779,'036779','2009-042AJ','deb Briz-M','',68723,'Reentered','2010-09-30 00:00:00','2010-07-08 00:00:00',103.99,261,1633,47.08,'Intermediate' from dual union all
select 3678,'003678','1968-091AD','deb Kosmos-249','',16818,'Reentered','1981-06-07 00:00:00','1981-06-04 00:00:00',90.16,262,303,62.24,'Intermediate' from dual union all
select 36780,'036780','2009-042AK','deb Briz-M','',68723,'Reentered','2010-09-21 00:00:00','2010-07-07 00:00:00',107.74,406,1839,49.31,'Intermediate' from dual union all
select 36781,'036781','2009-042AL','deb Briz-M','',68723,'Reentered','2010-08-08 00:00:00','2010-07-08 00:00:00',105.47,313,1720,47.82,'Intermediate' from dual union all
select 36782,'036782','2009-042AM','deb Briz-M','',68723,'Reentered','2010-08-25 00:00:00','2010-07-08 00:00:00',113.25,555,2197,46.55,'Intermediate' from dual union all
select 36783,'036783','2009-042AN','deb Briz-M','',68723,'Reentered','2010-07-24 00:00:00','2010-07-08 00:00:00',110.45,378,2117,46.87,'Intermediate' from dual union all
select 36784,'036784','2009-042AP','deb Briz-M','',68723,'Reentered','2010-07-24 00:00:00','2010-07-08 00:00:00',108.56,411,1910,46.97,'Intermediate' from dual union all
select 36785,'036785','2009-042AQ','deb Briz-M','',68723,'Reentered','2010-07-09 00:00:00','2010-07-08 00:00:00',93.2,181,682,46.99,'Intermediate' from dual union all
select 36786,'036786','2009-042AR','deb Briz-M','',68723,'Reentered','2010-08-31 00:00:00','2010-07-08 00:00:00',107.51,334,1889,47.67,'Intermediate' from dual union all
select 36787,'036787','2009-042AS','deb Briz-M','',68723,'Reentered','2010-07-24 00:00:00','2010-07-08 00:00:00',99.26,437,1009,47.81,'Intermediate' from dual union all
select 36788,'036788','2009-042AT','deb Briz-M','',68723,'Reentered','2010-07-24 00:00:00','2010-07-08 00:00:00',99.21,214,1227,47.44,'Intermediate' from dual union all
select 36789,'036789','2009-042AU','deb Briz-M','',68723,'Reentered','2010-09-03 00:00:00','2010-07-08 00:00:00',106.54,330,1803,47.84,'Intermediate' from dual union all
select 3679,'003679','1968-091AE','deb Kosmos-249','[UNID018] RCS 0.0',16818,'In Earth orbit','','1969-02-08 00:00:00',114.38,529,2326,62.76,'Intermediate' from dual union all
select 36790,'036790','2009-042AV','deb Briz-M','',68723,'Reentered','2010-08-12 00:00:00','2010-07-08 00:00:00',101.68,371,1306,48.4,'Intermediate' from dual union all
select 36791,'036791','2009-042AW','deb Briz-M','',68723,'Reentered','2010-07-21 00:00:00','2010-07-08 00:00:00',102.07,389,1325,48.8,'Intermediate' from dual union all
select 36792,'036792','2010-034A','Echostar XV','',68894,'In Earth orbit','','2010-07-16 00:00:00',1436.67,35790,35806,.04,'Drift GEO' from dual union all
select 36793,'036793','2010-034B','Briz-M No. 99515','Briz-M',68894,'In Earth orbit','','2010-07-14 00:00:00',704.68,5349,34358,18.45,'GEO Transfer' from dual union all
select 36794,'036794','2010-034C','Briz-M DTB','Briz-M torus tank',68894,'In Earth orbit','','2010-07-12 00:00:00',316.85,332,17703,49.51,'Highly Ellip' from dual union all
select 36795,'036795','2010-035A','Cartosat 2B','Cartosat-2B',68895,'In Earth orbit','','2010-07-14 00:00:00',97.33,622,639,98.15,'Sun-Synch' from dual union all
select 36796,'036796','2010-035B','Studsat','Studsat',68895,'In Earth orbit','','2010-07-14 00:00:00',97.24,619,635,98.15,'Sun-Synch' from dual union all
select 36797,'036797','2010-035C','AISSAT-1','AISSAT-1',68895,'In Earth orbit','','2010-07-14 00:00:00',97.31,621,639,98.15,'Sun-Synch' from dual union all
select 36798,'036798','2010-035D','ALSAT-2A','ALSAT-2A',68895,'In Earth orbit','','2010-07-14 00:00:00',97.3,621,638,98.14,'Sun-Synch' from dual union all
select 36799,'036799','2010-035E','Tlsat-1','Tlsat-1',68895,'In Earth orbit','','2010-07-14 00:00:00',97.28,619,637,98.14,'Sun-Synch' from dual union all
select 368,'000368','62 A Xi 2','S1','S1',8158,'Reentered','1962-12-19 00:00:00','1962-09-18 00:00:00',92.6,263,541,48.99,'Intermediate' from dual union all
select 3680,'003680','1968-091AF','deb Kosmos-249','[UNID019] RCS 0.0',16818,'Reentered','1981-06-09 00:00:00','1981-06-08 00:00:00',90.71,290,328,62.27,'Intermediate' from dual union all
select 36800,'036800','2010-035F','PS4','PS4 PSLV-C15',68895,'In Earth orbit','','2010-07-14 00:00:00',96.84,589,626,98.09,'Sun-Synch' from dual union all
select 36801,'036801','2010-035G','DLA-U','PSLV-C15-DLA-U',68895,'In Earth orbit','','2010-07-14 00:00:00',97.3,620,639,98.15,'Sun-Synch' from dual union all
select 36802,'036802','2009-042AX','deb Briz-M','',68723,'Reentered','2010-08-25 00:00:00','2010-07-14 00:00:00',103.76,569,1304,47.32,'Intermediate' from dual union all
select 36803,'036803','2009-042AY','deb Briz-M','',68723,'Reentered','2010-08-06 00:00:00','2010-07-13 00:00:00',99.34,199,1256,47.53,'Intermediate' from dual union all
select 36804,'036804','2009-042AZ','deb Briz-M','',68723,'Reentered','2010-07-29 00:00:00','2010-07-14 00:00:00',101.72,414,1266,48,'Intermediate' from dual union all
select 36805,'036805','2009-042BA','deb Briz-M','',68723,'Reentered','2010-09-12 00:00:00','2010-07-14 00:00:00',116.34,555,2478,47.62,'Intermediate' from dual union all
select 36806,'036806','2009-042BB','deb Briz-M','',68723,'Reentered','2010-08-01 00:00:00','2010-07-12 00:00:00',99.22,534,909,49.77,'Intermediate' from dual union all
select 36807,'036807','2009-042BC','deb Briz-M','',68723,'Reentered','2010-08-20 00:00:00','2010-07-14 00:00:00',100.65,216,1363,48.05,'Intermediate' from dual union all
select 36808,'036808','2009-042BD','deb Briz-M','',68723,'Reentered','2010-07-15 00:00:00','2010-07-14 00:00:00',103.13,337,1477,47.07,'Intermediate' from dual union all
select 36809,'036809','2009-042BE','deb Briz-M','',68723,'Reentered','2010-07-31 00:00:00','2010-07-14 00:00:00',113.31,615,2142,46.71,'Intermediate' from dual union all
select 3681,'003681','1968-091AG','deb Kosmos-249','[UNID020] RCS 0.1',16818,'Reentered','1992-06-15 00:00:00','1992-02-03 00:00:00',95.35,450,621,62.12,'Intermediate' from dual union all
select 36810,'036810','2009-042BF','deb Briz-M','',68723,'Reentered','2010-09-26 00:00:00','2010-07-13 00:00:00',121.22,514,2957,46.55,'Medium' from dual union all
select 36811,'036811','2009-042BG','deb Briz-M','',68723,'Reentered','2010-08-14 00:00:00','2010-07-14 00:00:00',111.46,494,2094,46.82,'Intermediate' from dual union all
select 36812,'036812','2009-042BH','deb Briz-M','',68723,'Reentered','2010-07-18 00:00:00','2010-07-14 00:00:00',99.24,411,1034,47.78,'Intermediate' from dual union all
select 36813,'036813','2009-042BJ','deb Briz-M','',68723,'Reentered','2010-07-20 00:00:00','2010-07-14 00:00:00',96.28,427,734,47.68,'Intermediate' from dual union all
select 36814,'036814','2009-042BK','deb Briz-M','',68723,'Reentered','2010-09-01 00:00:00','2010-07-14 00:00:00',113.56,481,2299,47.65,'Intermediate' from dual union all
select 36815,'036815','2009-042BL','deb Briz-M','',68723,'Reentered','2010-07-29 00:00:00','2010-07-14 00:00:00',99.15,308,1128,48.19,'Intermediate' from dual union all
select 36816,'036816','2009-042BM','deb Briz-M','',68723,'Reentered','2010-08-17 00:00:00','2010-07-14 00:00:00',115.4,574,2374,47.75,'Intermediate' from dual union all
select 36817,'036817','2009-042BN','deb Briz-M','',68723,'Reentered','2010-07-27 00:00:00','2010-07-14 00:00:00',100.87,396,1204,48.87,'Intermediate' from dual union all
select 36818,'036818','2009-042BP','deb Briz-M','',68723,'Reentered','2010-08-05 00:00:00','2010-07-14 00:00:00',103.04,322,1483,47.54,'Intermediate' from dual;
insert into SATELLITE
select 36819,'036819','2009-042BQ','deb Briz-M','',68723,'Reentered','2010-08-11 00:00:00','2010-07-14 00:00:00',100.56,203,1368,47.48,'Intermediate' from dual union all
select 3682,'003682','1968-091AH','deb Kosmos-249','',16818,'Reentered','1977-12-14 00:00:00','1977-12-08 00:00:00',92.27,359,413,62.5,'Intermediate' from dual union all
select 36820,'036820','2009-042BR','deb Briz-M','',68723,'Reentered','2010-07-30 00:00:00','2010-07-14 00:00:00',120.41,900,2499,46.78,'Medium' from dual union all
select 36821,'036821','2010-014D','deb Kosmos-2462','',68874,'Reentered','2010-07-20 00:00:00','2010-07-20 00:00:00',89.02,181,272,67.15,'Intermediate' from dual union all
select 36822,'036822','2010-014E','deb Kosmos-2462','',68874,'Reentered','2010-07-20 00:00:00','2010-07-20 00:00:00',89.02,181,272,67.15,'Intermediate' from dual union all
select 36823,'036823','2010-014F','deb Kosmos-2462','',68874,'Reentered','2010-07-20 00:00:00','2010-07-20 00:00:00',89.02,181,272,67.15,'Intermediate' from dual union all
select 36824,'036824','1998-067BY','Klest-154 camera','',66503,'Reentered','2010-11-18 00:00:00','2010-07-29 00:00:00',91.59,349,357,51.65,'Intermediate' from dual union all
select 36825,'036825','1998-067BZ','Cable bundle handle','',66503,'Reentered','2010-09-05 00:00:00','2010-07-29 00:00:00',91.56,346,357,51.65,'Intermediate' from dual union all
select 36826,'036826','1998-067CA','EVA debris','',66503,'Reentered','2010-11-20 00:00:00','2010-07-29 00:00:00',91.62,349,359,51.65,'Intermediate' from dual union all
select 36827,'036827','2010-028F','PRISMA-Tango','Prisma TARGET',68888,'In Earth orbit','','2010-08-12 00:00:00',99.92,723,787,98.28,'Sun-Synch' from dual union all
select 36828,'036828','2010-036A','Beidou DW 5','diwu ke beidou daohang w',68896,'In Earth orbit','','2010-08-10 00:00:00',1435.78,35667,35894,55.09,'Synchronous' from dual union all
select 36829,'036829','2010-036B','CZ-3A Stage 3','CZ-3A Stage 3',68896,'Reentered','2010-11-26 00:00:00','2010-08-08 00:00:00',630.28,173,35773,54.62,'GEO Transfer' from dual union all
select 3683,'003683','1968-091AJ','deb Kosmos-249','[UNID021] RCS 0.0',16818,'Reentered','2001-08-14 00:00:00','1969-02-14 00:00:00',111.59,531,2069,62.31,'Intermediate' from dual union all
select 36830,'036830','2010-037A','Nilesat 201','Nilesat 201',68897,'In Earth orbit','','2010-08-18 00:00:00',1436.09,35785,35788,.02,'Stationary' from dual union all
select 36831,'036831','2010-037B','RASCOM-QAF 1R','RASCOM-QAF 1R',68897,'In Earth orbit','','2010-08-18 00:00:00',1436.06,35709,35863,.01,'Stationary' from dual union all
select 36832,'036832','2010-037C','ESC-A 554','Ariane 5 ESC-A',68897,'In Earth orbit','','2010-08-12 00:00:00',628.79,249,35620,2.29,'GEO Transfer' from dual union all
select 36833,'036833','2010-037D','Sylda 5 No. 42-K','Sylda 5K',68897,'In Earth orbit','','2010-08-16 00:00:00',630.44,261,35692,2.03,'GEO Transfer' from dual union all
select 36834,'036834','2010-038A','Yaogan Weixing 10','Jianbing-5 03 xing',68898,'In Earth orbit','','2010-08-12 00:00:00',96.97,607,621,97.82,'Sun-Synch' from dual union all
select 36835,'036835','2010-038B','CZ-4C Stage 3','CZ-4C Stage 3',68898,'Reentered','2016-03-09 00:00:00','2010-08-12 00:00:00',94.91,408,621,97.78,'Sun-Synch' from dual union all
select 36836,'036836','2009-042BS','deb Briz-M','',68723,'Reentered','2010-09-10 00:00:00','2010-08-10 00:00:00',120.73,608,2819,45.99,'Medium' from dual union all
select 36837,'036837','2009-042BT','deb Briz-M','',68723,'Reentered','2010-08-24 00:00:00','2010-08-10 00:00:00',100.15,224,1308,47.43,'Intermediate' from dual union all
select 36838,'036838','2009-042BU','deb Briz-M','',68723,'Reentered','2010-08-30 00:00:00','2010-08-10 00:00:00',101.8,549,1139,47.36,'Intermediate' from dual union all
select 36839,'036839','2009-042BV','deb Briz-M','',68723,'Reentered','2010-08-22 00:00:00','2010-08-10 00:00:00',99.17,350,1088,47.31,'Intermediate' from dual union all
select 3684,'003684','1968-097AB','deb Kosmos-252','',16824,'Reentered','1982-03-28 00:00:00','1982-03-27 00:00:00',90.53,269,332,62.84,'Intermediate' from dual union all
select 36840,'036840','2009-042BW','deb Briz-M','',68723,'Reentered','2010-09-25 00:00:00','2010-08-10 00:00:00',104.87,575,1402,46.73,'Intermediate' from dual union all
select 36841,'036841','2009-042BX','deb Briz-M','',68723,'Reentered','2010-08-15 00:00:00','2010-08-10 00:00:00',107.99,412,1856,47.58,'Intermediate' from dual union all
select 36842,'036842','2009-042BY','deb Briz-M','',68723,'Reentered','2010-08-15 00:00:00','2010-08-10 00:00:00',96.83,332,882,47.73,'Intermediate' from dual union all
select 36843,'036843','2009-042BZ','deb Briz-M','',68723,'Reentered','2010-09-04 00:00:00','2010-08-10 00:00:00',104.98,310,1677,48.84,'Intermediate' from dual union all
select 36844,'036844','2009-042CA','deb Briz-M','',68723,'Reentered','2010-09-07 00:00:00','2010-08-10 00:00:00',100.02,377,1142,47.9,'Intermediate' from dual union all
select 36845,'036845','2009-042CB','deb Briz-M','',68723,'Reentered','2010-09-15 00:00:00','2010-08-10 00:00:00',111.54,340,2255,48.86,'Intermediate' from dual union all
select 36846,'036846','2009-042CC','deb Briz-M','',68723,'Reentered','2010-09-08 00:00:00','2010-08-11 00:00:00',127.32,784,3227,47.2,'Medium' from dual union all
select 36847,'036847','2009-042CD','deb Briz-M','',68723,'Reentered','2010-08-14 00:00:00','2010-08-11 00:00:00',103.35,380,1454,48.16,'Intermediate' from dual union all
select 36848,'036848','1994-074R','deb Resurs-O1','',65470,'Reentered','2012-06-22 00:00:00','2010-08-13 00:00:00',97.35,630,634,98.04,'Sun-Synch' from dual union all
select 36849,'036849','1994-074S','deb Resurs-O1','',65470,'Reentered','2012-10-24 00:00:00','2010-08-13 00:00:00',97.44,634,639,98.06,'Sun-Synch' from dual union all
select 3685,'003685','1968-097AC','deb Kosmos-252','[UNID022] RCS 0.0',16824,'Reentered','1991-10-02 00:00:00','1991-10-01 00:00:00',89.01,219,233,62.23,'Intermediate' from dual union all
select 36850,'036850','1994-074T','deb Resurs-O1','',65470,'Reentered','2012-10-23 00:00:00','2010-08-09 00:00:00',97.29,627,631,98.02,'Sun-Synch' from dual union all
select 36851,'036851','1994-074U','deb Resurs-O1','',65470,'Reentered','2012-05-14 00:00:00','2010-08-12 00:00:00',97.23,626,626,98.02,'Sun-Synch' from dual union all
select 36852,'036852','1994-074V','deb Resurs-O1','',65470,'Reentered','2013-02-01 00:00:00','2010-08-13 00:00:00',97.31,628,632,98.01,'Sun-Synch' from dual union all
select 36853,'036853','1994-074W','deb Resurs-O1','',65470,'Reentered','2011-12-02 00:00:00','2010-08-09 00:00:00',96.73,599,606,98.01,'Sun-Synch' from dual union all
select 36854,'036854','1987-053F','deb DMSP F-8','',60143,'Reentered','2011-11-20 00:00:00','2010-08-13 00:00:00',100.97,782,828,98.75,'Sun-Synch' from dual union all
select 36855,'036855','1987-053G','deb DMSP F-8','',60143,'Reentered','2011-10-26 00:00:00','2010-08-10 00:00:00',100.88,761,840,98.78,'Sun-Synch' from dual union all
select 36856,'036856','1987-053H','deb DMSP F-8','',60143,'Reentered','2011-07-14 00:00:00','2010-08-13 00:00:00',99.17,685,752,99.1,'Sun-Synch' from dual union all
select 36857,'036857','1982-025AU','deb Meteor-2','',54334,'In Earth orbit','','2010-08-05 00:00:00',102.81,859,925,82.53,'Intermediate' from dual union all
select 36858,'036858','1982-025AV','deb Meteor-2','',54334,'In Earth orbit','','2010-07-25 00:00:00',102.96,881,917,82.52,'Intermediate' from dual union all
select 36859,'036859','1982-025AW','deb Meteor-2','',54334,'In Earth orbit','','2010-08-05 00:00:00',102.91,886,907,82.52,'Intermediate' from dual union all
select 3686,'003686','1968-097AD','deb Kosmos-252','[UNID023] RCS 0.0',16824,'In Earth orbit','','1992-02-02 00:00:00',112.55,757,1931,62.3,'Intermediate' from dual union all
select 36860,'036860','1982-025AX','deb Meteor-2','',54334,'In Earth orbit','','2010-08-09 00:00:00',103.09,896,914,82.53,'Intermediate' from dual union all
select 36861,'036861','1982-025AY','deb Meteor-2','',54334,'In Earth orbit','','2010-07-19 00:00:00',102.84,885,901,82.52,'Intermediate' from dual union all
select 36862,'036862','1982-025AZ','deb Meteor-2','',54334,'In Earth orbit','','2010-08-13 00:00:00',102.98,887,913,82.55,'Intermediate' from dual union all
select 36863,'036863','1982-025BA','deb Meteor-2','',54334,'In Earth orbit','','2010-08-11 00:00:00',102.94,894,902,82.53,'Intermediate' from dual union all
select 36864,'036864','1982-025BB','deb Meteor-2','',54334,'In Earth orbit','','2010-08-11 00:00:00',103.09,892,918,82.54,'Intermediate' from dual union all
select 36865,'036865','1982-025BC','deb Meteor-2','',54334,'In Earth orbit','','2010-08-06 00:00:00',102.63,868,899,82.55,'Intermediate' from dual union all
select 36866,'036866','1982-025BD','deb Meteor-2','',54334,'In Earth orbit','','2010-07-26 00:00:00',103.58,913,943,82.53,'Intermediate' from dual union all
select 36867,'036867','1982-025BE','deb Meteor-2','',54334,'In Earth orbit','','2010-07-28 00:00:00',102.74,875,902,82.54,'Intermediate' from dual union all
select 36868,'036868','2010-039A','AEHF SV-1','AEHF SV-1',68899,'In Earth orbit','','2012-05-20 00:00:00',1436.09,35772,35801,3.97,'Inclined GEO' from dual union all
select 36869,'036869','2010-039B','Centaur AV-019','Centaur AV-019',68899,'In Earth orbit','','2010-08-14 00:00:00',930.76,221,50178,22.24,'Highly Ellip' from dual union all
select 3687,'003687','1968-097AE','deb Kosmos-252','',16824,'Reentered','2001-02-14 00:00:00','1992-02-02 00:00:00',103.08,489,1320,62.45,'Intermediate' from dual union all
select 36870,'036870','2009-066C','deb H-2A F16','',68747,'Reentered','2010-09-11 00:00:00','2010-08-17 00:00:00',94.54,475,518,97.73,'Sun-Synch' from dual union all
select 36871,'036871','2009-066D','deb H-2A F16','',68747,'Reentered','2010-08-24 00:00:00','2010-08-16 00:00:00',93.37,425,454,97.66,'Sun-Synch' from dual union all
select 36872,'036872','2009-066E','deb H-2A F16','',68747,'Reentered','2010-08-21 00:00:00','2010-08-17 00:00:00',92.68,385,427,97.65,'Sun-Synch' from dual union all
select 36873,'036873','2009-066F','deb H-2A F16','',68747,'Reentered','2010-08-23 00:00:00','2010-08-17 00:00:00',93.2,411,451,97.66,'Sun-Synch' from dual union all
select 36874,'036874','2009-066G','deb H-2A F16','',68747,'Reentered','2010-11-02 00:00:00','2010-08-17 00:00:00',95.46,519,563,97.64,'Sun-Synch' from dual union all
select 36875,'036875','1981-043N','deb Meteor-2','',52566,'In Earth orbit','','2010-08-13 00:00:00',100.92,777,828,81.26,'Intermediate' from dual union all
select 36876,'036876','1981-043P','deb Meteor-2','',52566,'Reentered','2016-07-13 00:00:00','2010-08-08 00:00:00',100.11,743,784,81.24,'Intermediate' from dual union all
select 36877,'036877','1981-043Q','deb Meteor-2','',52566,'Reentered','2016-05-16 00:00:00','2010-08-17 00:00:00',99.91,738,771,81.25,'Intermediate' from dual union all
select 36878,'036878','1981-043R','deb Meteor-2','',52566,'In Earth orbit','','2010-08-13 00:00:00',102.07,825,889,81.27,'Intermediate' from dual union all
select 36879,'036879','1981-043S','deb Meteor-2','',52566,'In Earth orbit','','2010-08-13 00:00:00',102.07,823,891,81.26,'Intermediate' from dual union all
select 3688,'003688','1968-097AF','deb Kosmos-252','',16824,'Reentered','1978-09-15 00:00:00','1978-05-05 00:00:00',101.34,486,1159,62.34,'Intermediate' from dual union all
select 36880,'036880','1993-041H','deb RADCAL','',65143,'In Earth orbit','','2010-08-10 00:00:00',100.63,723,854,89.53,'Polar' from dual union all
select 36881,'036881','2005-011C','deb XSS-11','',68020,'In Earth orbit','','2010-08-13 00:00:00',101.96,844,858,98.95,'Sun-Synch' from dual union all
select 36882,'036882','1989-001V','deb SOZ','',62278,'In Earth orbit','','2010-08-16 00:00:00',159.44,658,6066,64.48,'Medium' from dual union all
select 36883,'036883','1989-001W','deb SOZ','',62278,'Reentered','2013-04-26 00:00:00','2010-08-17 00:00:00',159.83,636,6119,64.35,'Medium' from dual union all
select 36884,'036884','1989-039EG','deb SOZ','',62316,'In Earth orbit','','2010-08-04 00:00:00',336.79,767,18547,64.98,'Highly Ellip' from dual union all
select 36885,'036885','1979-095AA','deb Meteor-2','',47579,'Reentered','2014-11-02 00:00:00','2010-08-11 00:00:00',99.59,730,748,81.2,'Intermediate' from dual union all
select 36886,'036886','1979-095AB','deb Meteor-2','',47579,'In Earth orbit','','2010-08-12 00:00:00',99.5,724,746,81.17,'Intermediate' from dual union all
select 36887,'036887','1979-095AC','deb Meteor-2','',47579,'Reentered','2015-11-07 00:00:00','2010-08-08 00:00:00',98.98,682,738,81.22,'Intermediate' from dual union all
select 36888,'036888','1979-095AD','deb Meteor-2','',47579,'Reentered','2015-05-03 00:00:00','2010-08-14 00:00:00',99.33,719,734,81.19,'Intermediate' from dual union all
select 36889,'036889','1979-095AE','deb Meteor-2','',47579,'Reentered','2013-07-14 00:00:00','2010-08-14 00:00:00',98.2,667,679,81.19,'Intermediate' from dual union all
select 3689,'003689','1968-097AG','deb Kosmos-252','',16824,'Reentered','1974-07-13 00:00:00','1979-07-16 00:00:00',102.3,480,1255,62.57,'Intermediate' from dual union all
select 36890,'036890','1979-095AF','deb Meteor-2','',47579,'Reentered','2015-10-05 00:00:00','2010-08-12 00:00:00',99.55,731,743,81.18,'Intermediate' from dual union all
select 36891,'036891','1979-095AG','deb Meteor-2','',47579,'In Earth orbit','','2010-08-15 00:00:00',99.82,738,762,81.19,'Intermediate' from dual union all
select 36892,'036892','1979-095AH','deb Meteor-2','',47579,'Reentered','2014-09-16 00:00:00','2010-08-13 00:00:00',98.75,691,707,81.18,'Intermediate' from dual union all
select 36893,'036893','1979-095AJ','deb Meteor-2','',47579,'Reentered','2014-10-22 00:00:00','2010-08-13 00:00:00',99.48,721,746,81.18,'Intermediate' from dual union all
select 36894,'036894','1979-095AK','deb Meteor-2','',47579,'In Earth orbit','','2010-08-16 00:00:00',100.57,781,790,81.21,'Intermediate' from dual union all
select 36895,'036895','1979-095AL','deb Meteor-2','',47579,'Reentered','2012-05-20 00:00:00','2010-07-31 00:00:00',97.57,626,659,81.16,'Intermediate' from dual union all
select 36896,'036896','1979-095AM','deb Meteor-2','',47579,'Reentered','2015-01-09 00:00:00','2010-08-04 00:00:00',99.13,703,732,81.2,'Intermediate' from dual union all
select 36897,'036897','1979-095AN','deb Meteor-2','',47579,'Reentered','2016-04-09 00:00:00','2010-08-17 00:00:00',99.81,743,756,81.19,'Intermediate' from dual union all
select 36898,'036898','1979-095AP','deb Meteor-2','',47579,'In Earth orbit','','2010-08-13 00:00:00',99.65,728,756,81.19,'Intermediate' from dual union all
select 36899,'036899','1979-095AQ','deb Meteor-2','',47579,'In Earth orbit','','2010-08-12 00:00:00',99.96,755,759,81.19,'Intermediate' from dual union all
select 369,'000369','62 A Omicron 1','FTV 3502','P35-2',8149,'In Earth orbit','','1962-10-16 00:00:00',99.47,606,861,98.63,'Sun-Synch' from dual union all
select 3690,'003690','1968-097AH','deb Kosmos-252 [*]','',16824,'In Earth orbit','','1992-02-01 00:00:00',112.61,721,1972,62.36,'Intermediate' from dual union all
select 36900,'036900','1979-095AR','deb Meteor-2','',47579,'Reentered','2013-05-23 00:00:00','2010-08-10 00:00:00',98.06,657,675,81.17,'Intermediate' from dual union all
select 36901,'036901','1979-095AS','deb Meteor-2','',47579,'In Earth orbit','','2010-08-12 00:00:00',100.6,782,791,81.2,'Intermediate' from dual union all
select 36902,'036902','1979-095AT','deb Meteor-2','',47579,'In Earth orbit','','2010-08-15 00:00:00',99.85,742,761,81.19,'Intermediate' from dual union all
select 36903,'036903','1979-095AU','deb Meteor-2','',47579,'In Earth orbit','','2010-08-14 00:00:00',99.96,754,760,81.2,'Intermediate' from dual union all
select 36904,'036904','1979-095AV','deb Meteor-2','',47579,'Reentered','2015-05-03 00:00:00','2010-08-11 00:00:00',99.27,694,754,81.25,'Intermediate' from dual union all
select 36905,'036905','1979-095AW','deb Meteor-2','',47579,'Reentered','2015-05-26 00:00:00','2010-08-17 00:00:00',99.2,709,732,81.2,'Intermediate' from dual union all
select 36906,'036906','1979-095AX','deb Meteor-2','',47579,'In Earth orbit','','2010-08-15 00:00:00',100.81,790,804,81.2,'Intermediate' from dual union all
select 36907,'036907','1979-095AY','deb Meteor-2','',47579,'Reentered','2016-07-05 00:00:00','2010-08-10 00:00:00',99.45,708,757,81.21,'Intermediate' from dual union all
select 36908,'036908','1979-095AZ','deb Meteor-2','',47579,'Reentered','2017-03-14 00:00:00','2010-08-14 00:00:00',99.75,732,762,81.2,'Intermediate' from dual;
insert into SATELLITE
select 36909,'036909','1979-095BA','deb Meteor-2','',47579,'Reentered','2015-01-07 00:00:00','2010-07-30 00:00:00',98.68,688,704,81.19,'Intermediate' from dual union all
select 3691,'003691','1969-013A','OPS 0757','Tacsat',18280,'In Earth orbit','','1969-03-25 00:00:00',1436.04,35768,35803,.88,'Stationary' from dual union all
select 36910,'036910','1979-095BB','deb Meteor-2','',47579,'Reentered','2015-09-19 00:00:00','2010-08-13 00:00:00',99.09,701,730,81.19,'Intermediate' from dual union all
select 36911,'036911','1979-095BC','deb Meteor-2','',47579,'In Earth orbit','','2010-08-14 00:00:00',102.3,853,881,81.2,'Intermediate' from dual union all
select 36912,'036912','1979-095BD','deb Meteor-2','',47579,'Reentered','2015-08-08 00:00:00','2010-08-16 00:00:00',99.48,721,746,81.2,'Intermediate' from dual union all
select 36913,'036913','1979-095BE','deb Meteor-2','',47579,'In Earth orbit','','2010-08-14 00:00:00',100.68,784,797,81.21,'Intermediate' from dual union all
select 36914,'036914','1979-095BF','deb Meteor-2','',47579,'In Earth orbit','','2010-08-12 00:00:00',99.55,723,752,81.2,'Intermediate' from dual union all
select 36915,'036915','1979-095BG','deb Meteor-2','',47579,'In Earth orbit','','2010-08-13 00:00:00',99.7,733,756,81.19,'Intermediate' from dual union all
select 36916,'036916','1979-095BH','deb Meteor-2','',47579,'In Earth orbit','','2010-08-12 00:00:00',100.73,789,797,81.2,'Intermediate' from dual union all
select 36917,'036917','1979-095BJ','deb Meteor-2','',47579,'Reentered','2014-03-27 00:00:00','2010-08-14 00:00:00',98.47,677,694,81.19,'Intermediate' from dual union all
select 36918,'036918','1979-095BK','deb Meteor-2','',47579,'In Earth orbit','','2010-08-16 00:00:00',100.81,787,807,81.21,'Intermediate' from dual union all
select 36919,'036919','1979-095BL','deb Meteor-2','',47579,'Reentered','2015-04-12 00:00:00','2010-08-17 00:00:00',99.31,716,736,81.19,'Intermediate' from dual union all
select 3692,'003692','1969-013B','Transtage 17','Transtage 17',18280,'In Earth orbit','','1980-06-05 00:00:00',1434.07,35661,35833,7.9,'Inclined Drift' from dual union all
select 36920,'036920','1979-095BM','deb Meteor-2','',47579,'Reentered','2016-02-20 00:00:00','2010-08-17 00:00:00',99.8,718,779,81.19,'Intermediate' from dual union all
select 36921,'036921','1979-095BN','deb Meteor-2','',47579,'In Earth orbit','','2010-08-12 00:00:00',99.94,747,765,81.18,'Intermediate' from dual union all
select 36922,'036922','1979-095BP','deb Meteor-2','',47579,'In Earth orbit','','2010-08-06 00:00:00',99.94,748,764,81.2,'Intermediate' from dual union all
select 36923,'036923','1979-095BQ','deb Meteor-2','',47579,'In Earth orbit','','2010-07-25 00:00:00',102.26,857,875,81.21,'Intermediate' from dual union all
select 36924,'036924','1971-120M','deb Meteor','',24938,'Reentered','2014-11-09 00:00:00','2010-08-17 00:00:00',99.38,720,738,81.25,'Intermediate' from dual union all
select 36925,'036925','1971-120N','deb Meteor','',24938,'Reentered','2014-07-09 00:00:00','2010-08-04 00:00:00',99.08,709,721,81.23,'Intermediate' from dual union all
select 36926,'036926','1971-120P','deb Meteor','',24938,'Reentered','2014-06-21 00:00:00','2010-08-05 00:00:00',99.08,702,727,81.24,'Intermediate' from dual union all
select 36927,'036927','1971-120Q','deb Meteor','',24938,'Reentered','2015-01-30 00:00:00','2010-08-17 00:00:00',99.53,726,747,81.25,'Intermediate' from dual union all
select 36928,'036928','1971-120R','deb Meteor','',24938,'Reentered','2014-12-06 00:00:00','2010-08-12 00:00:00',99.47,721,746,81.24,'Intermediate' from dual union all
select 36929,'036929','1971-120S','deb Meteor','',24938,'Reentered','2015-03-03 00:00:00','2010-08-14 00:00:00',99.36,716,739,81.25,'Intermediate' from dual union all
select 3693,'003693','1968-091AK','deb Kosmos-249','',16818,'Reentered','1976-12-05 00:00:00','1981-03-06 00:00:00',98.43,482,886,62.34,'Intermediate' from dual union all
select 36930,'036930','1971-120T','deb Meteor','',24938,'Reentered','2015-04-05 00:00:00','2010-08-13 00:00:00',99.61,722,758,81.25,'Intermediate' from dual union all
select 36931,'036931','1971-120U','deb Meteor','',24938,'Reentered','2014-03-15 00:00:00','2010-08-12 00:00:00',98.97,701,718,81.24,'Intermediate' from dual union all
select 36932,'036932','1987-011L','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-12 00:00:00',100.58,770,802,65,'Intermediate' from dual union all
select 36933,'036933','1987-011M','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-16 00:00:00',100.57,771,801,65.01,'Intermediate' from dual union all
select 36934,'036934','1987-011N','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-16 00:00:00',100.57,770,802,65.01,'Intermediate' from dual union all
select 36935,'036935','1987-011P','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-16 00:00:00',100.57,770,802,65.01,'Intermediate' from dual union all
select 36936,'036936','1987-011Q','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-16 00:00:00',100.57,771,801,65,'Intermediate' from dual union all
select 36937,'036937','1987-011R','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-18 00:00:00',100.58,771,801,65.01,'Intermediate' from dual union all
select 36938,'036938','1987-011S','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-14 00:00:00',100.57,770,802,65.01,'Intermediate' from dual union all
select 36939,'036939','1987-011T','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-16 00:00:00',100.57,771,801,65.01,'Intermediate' from dual union all
select 3694,'003694','1968-097AJ','deb Kosmos-252','',16824,'Reentered','1980-10-28 00:00:00','1981-09-04 00:00:00',99.73,471,1020,62.14,'Intermediate' from dual union all
select 36940,'036940','1987-011U','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-14 00:00:00',100.63,771,806,65.01,'Intermediate' from dual union all
select 36941,'036941','1987-011V','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-07 00:00:00',100.57,770,802,65.01,'Intermediate' from dual union all
select 36942,'036942','1987-011W','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-17 00:00:00',100.54,772,796,65,'Intermediate' from dual union all
select 36943,'036943','1987-011X','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-16 00:00:00',100.63,772,805,65.01,'Intermediate' from dual union all
select 36944,'036944','1987-011Y','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-16 00:00:00',100.64,776,802,65.01,'Intermediate' from dual union all
select 36945,'036945','1987-011Z','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-14 00:00:00',100.56,771,800,65,'Intermediate' from dual union all
select 36946,'036946','1987-011AA','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-14 00:00:00',100.61,773,802,65.02,'Intermediate' from dual union all
select 36947,'036947','1987-011AB','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-18 00:00:00',100.56,769,802,65.01,'Intermediate' from dual union all
select 36948,'036948','1987-011AC','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-16 00:00:00',100.63,775,801,65,'Intermediate' from dual union all
select 36949,'036949','1987-011AD','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-14 00:00:00',100.6,771,803,65.01,'Intermediate' from dual union all
select 3695,'003695','1968-091AL','deb Kosmos-249','',16818,'In Earth orbit','','1992-02-03 00:00:00',113.1,758,1980,62.27,'Intermediate' from dual union all
select 36950,'036950','1987-011AE','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-14 00:00:00',100.64,773,804,65.01,'Intermediate' from dual union all
select 36951,'036951','1987-011AF','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-14 00:00:00',100.6,771,803,65,'Intermediate' from dual union all
select 36952,'036952','1987-011AG','deb Kosmos-1818 coolant','',60101,'Reentered','2012-12-19 00:00:00','2010-08-14 00:00:00',100.62,771,805,65,'Intermediate' from dual union all
select 36953,'036953','1987-011AH','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-14 00:00:00',100.62,771,805,65,'Intermediate' from dual union all
select 36954,'036954','1987-011AJ','deb Kosmos-1818 coolant','',60101,'In Earth orbit','','2010-08-12 00:00:00',100.63,774,803,65.01,'Intermediate' from dual union all
select 36955,'036955','2009-042CE','deb Briz-M','',68723,'Reentered','2010-09-14 00:00:00','2010-08-19 00:00:00',113.58,431,2351,47.03,'Intermediate' from dual union all
select 36956,'036956','2009-042CF','deb Briz-M','',68723,'Reentered','2010-11-06 00:00:00','2010-08-19 00:00:00',109.79,424,2010,47.8,'Intermediate' from dual union all
select 36957,'036957','2009-042CG','deb Briz-M','',68723,'Reentered','2010-09-28 00:00:00','2010-08-19 00:00:00',101.67,442,1233,48.29,'Intermediate' from dual union all
select 36958,'036958','2006-006CQ','deb Briz-M 88515','',68178,'In Earth orbit','','2010-07-25 00:00:00',274.91,361,14892,51.78,'Highly Ellip' from dual union all
select 36959,'036959','2006-006CR','deb Briz-M 88515','',68178,'In Earth orbit','','2010-08-12 00:00:00',264.36,333,14199,51.94,'Highly Ellip' from dual union all
select 3696,'003696','1968-097AK','deb Kosmos-252                    ','[UNID024]',16824,'Reentered','1970-03-10 00:00:00','1970-02-04 00:00:00',110.76,455,2069,62.34,'Intermediate' from dual union all
select 36960,'036960','2006-006CS','deb Briz-M 88515','',68178,'In Earth orbit','','2010-08-13 00:00:00',281.44,454,15242,52.07,'Highly Ellip' from dual union all
select 36961,'036961','2006-006CT','deb Briz-M 88515','',68178,'In Earth orbit','','2010-08-14 00:00:00',269.49,339,14545,51.9,'Highly Ellip' from dual union all
select 36962,'036962','2006-006CU','deb Briz-M 88515','',68178,'Lost','','2010-08-17 00:00:00',271.75,393,14646,51.78,'Highly Ellip' from dual union all
select 36963,'036963','2006-006CV','deb Briz-M 88515','',68178,'In Earth orbit','','2010-08-16 00:00:00',272.88,374,14742,51.68,'Highly Ellip' from dual union all
select 36964,'036964','2006-006CW','deb Briz-M 88515','',68178,'In Earth orbit','','2010-08-19 00:00:00',277.16,390,15016,52.07,'Highly Ellip' from dual union all
select 36965,'036965','2007-003AB','deb CZ-3','',68345,'In Earth orbit','','2010-08-19 00:00:00',825.65,637,44914,26.54,'Highly Ellip' from dual union all
select 36966,'036966','2007-003AC','deb CZ-3','',68345,'In Earth orbit','','2010-08-16 00:00:00',759.9,772,41641,25.11,'Highly Ellip' from dual union all
select 36967,'036967','2007-003AD','deb CZ-3','',68345,'In Earth orbit','','2010-08-13 00:00:00',779.86,944,42431,27.47,'Highly Ellip' from dual union all
select 36968,'036968','2007-003AE','deb CZ-3','',68345,'In Earth orbit','','2010-08-03 00:00:00',799.39,413,43895,26.62,'Highly Ellip' from dual union all
select 36969,'036969','2007-003AF','deb CZ-3','',68345,'Lost','','2010-08-16 00:00:00',799.14,442,43854,26.62,'Highly Ellip' from dual union all
select 3697,'003697','1968-091AM','deb Kosmos-249                    ','[UNID025] RCS 0.1',16818,'Reentered','1978-01-03 00:00:00','1969-02-10 00:00:00',110.87,523,2011,62.3,'Intermediate' from dual union all
select 36970,'036970','2007-003AG','deb CZ-3','',68345,'In Earth orbit','','2010-08-16 00:00:00',815.96,803,44291,26.1,'Highly Ellip' from dual union all
select 36971,'036971','2007-003AH','deb CZ-3','',68345,'In Earth orbit','','2010-08-17 00:00:00',820.52,558,44751,26.41,'Highly Ellip' from dual union all
select 36972,'036972','1981-043T','deb Meteor-2','',52566,'Reentered','2015-04-24 00:00:00','2010-08-16 00:00:00',99.49,715,754,81.25,'Intermediate' from dual union all
select 36973,'036973','1981-043U','deb Meteor-2','',52566,'Reentered','2014-11-08 00:00:00','2010-08-17 00:00:00',98.68,683,709,81.25,'Intermediate' from dual union all
select 36974,'036974','1981-043V','deb Meteor-2','',52566,'Reentered','2014-03-31 00:00:00','2010-08-13 00:00:00',99.1,693,739,81.23,'Intermediate' from dual union all
select 36975,'036975','2009-042CH','deb Briz-M','',68723,'Reentered','2010-10-11 00:00:00','2010-08-23 00:00:00',116.71,403,2663,47.42,'Intermediate' from dual union all
select 36976,'036976','2009-042CJ','deb Briz-M','',68723,'Reentered','2010-10-04 00:00:00','2010-08-23 00:00:00',102.96,245,1553,47.57,'Intermediate' from dual union all
select 36977,'036977','2009-042CK','deb Briz-M','',68723,'Reentered','2010-08-26 00:00:00','2010-08-22 00:00:00',110.66,383,2131,47.51,'Intermediate' from dual union all
select 36978,'036978','2009-042CL','deb Briz-M','',68723,'Reentered','2010-10-13 00:00:00','2010-08-23 00:00:00',116.92,582,2503,46.98,'Intermediate' from dual union all
select 36979,'036979','1965-108DK','deb Transtage 8','',11858,'In Earth orbit','','2010-08-15 00:00:00',563.33,1130,31303,26.59,'Highly Ellip' from dual union all
select 3698,'003698','1968-117T','deb Blok Kosmos R-12','',16844,'Reentered','1969-03-20 00:00:00','1969-02-10 00:00:00',92.33,295,483,71.03,'Intermediate' from dual union all
select 36980,'036980','1965-108DL','deb Transtage 8','',11858,'In Earth orbit','','2010-08-20 00:00:00',532.67,604,30174,26.61,'Highly Ellip' from dual union all
select 36981,'036981','1965-108DM','deb Transtage 8','',11858,'Lost','','2010-08-12 00:00:00',498.18,309,28569,29.75,'Highly Ellip' from dual union all
select 36982,'036982','1965-108DN','deb Transtage 8','',11858,'In Earth orbit','','2010-08-23 00:00:00',435.4,396,24906,26.45,'Highly Ellip' from dual union all
select 36983,'036983','1965-108DP','deb Transtage 8','',11858,'In Earth orbit','','2010-08-22 00:00:00',262.07,291,14083,27.03,'Highly Ellip' from dual union all
select 36984,'036984','1965-108DQ','deb Transtage 8','',11858,'In Earth orbit','','2010-08-23 00:00:00',221.99,485,11047,26.5,'Medium' from dual union all
select 36985,'036985','2010-040A','Tianhui 1-01','tianhui yi hao weixing',68900,'In Earth orbit','','2010-08-24 00:00:00',94.52,487,504,97.35,'Sun-Synch' from dual union all
select 36986,'036986','1999-025EBS','deb FY-1C','',66706,'In Earth orbit','','2010-08-21 00:00:00',103.55,836,1017,99.08,'Sun-Synch' from dual union all
select 36987,'036987','1999-025EBT','deb FY-1C','',66706,'In Earth orbit','','2010-08-20 00:00:00',107.32,857,1349,100.78,'Sun-Synch' from dual union all
select 36988,'036988','1999-025EBU','deb FY-1C','',66706,'In Earth orbit','','2010-08-22 00:00:00',100.85,611,986,98.39,'Sun-Synch' from dual union all
select 36989,'036989','1999-025EBV','deb FY-1C','',66706,'In Earth orbit','','2010-08-13 00:00:00',105.01,788,1202,98.2,'Sun-Synch' from dual union all
select 3699,'003699','1968-091AN','deb Kosmos-249 [*]                ','',16818,'Reentered','2014-08-03 00:00:00','1992-02-02 00:00:00',105.12,525,1476,62.33,'Intermediate' from dual union all
select 36990,'036990','1999-025EBW','deb FY-1C','',66706,'In Earth orbit','','2010-08-21 00:00:00',106.47,837,1290,96.58,'Sun-Synch' from dual union all
select 36991,'036991','1999-025EBX','deb FY-1C','',66706,'In Earth orbit','','2010-08-20 00:00:00',105.96,819,1260,98.13,'Sun-Synch' from dual union all
select 36992,'036992','1999-025EBY','deb FY-1C','',66706,'Reentered','2013-12-03 00:00:00','2010-08-15 00:00:00',99.24,536,908,98.4,'Sun-Synch' from dual union all
select 36993,'036993','1999-025EBZ','deb FY-1C','',66706,'Reentered','2014-05-12 00:00:00','2010-08-16 00:00:00',99.98,575,941,97.83,'Sun-Synch' from dual union all
select 36994,'036994','1999-025ECA','deb FY-1C','',66706,'In Earth orbit','','2010-08-24 00:00:00',103.64,846,1016,98.72,'Sun-Synch' from dual union all
select 36995,'036995','1999-025ECB','deb FY-1C','',66706,'In Earth orbit','','2010-07-31 00:00:00',102.57,850,911,98.38,'Sun-Synch' from dual union all
select 36996,'036996','1999-025ECC','deb FY-1C','',66706,'In Earth orbit','','2010-08-04 00:00:00',103.23,700,1123,98.31,'Sun-Synch' from dual union all
select 36997,'036997','1999-025ECD','deb FY-1C','',66706,'In Earth orbit','','2010-08-16 00:00:00',100.86,653,946,98.5,'Sun-Synch' from dual union all
select 36998,'036998','1999-025ECE','deb FY-1C','',66706,'Reentered','2012-02-12 00:00:00','2010-08-25 00:00:00',98.62,448,938,98.68,'Sun-Synch' from dual union all
select 36999,'036999','1999-025ECF','deb FY-1C','',66706,'In Earth orbit','','2010-08-19 00:00:00',105.34,842,1179,99.18,'Sun-Synch' from dual;
insert into SATELLITE
select 37,'000037','60 Epsilon 4','Korabl''-Sputnik part','Vostok-1P part',6320,'Reentered','1961-06-30 00:00:00','1960-07-02 00:00:00',94.09,282,667,64.89,'Intermediate' from dual union all
select 370,'000370','62 A Omicron 2','P35 despin weight','',8149,'Reentered','1980-03-02 00:00:00','1962-09-24 00:00:00',98.21,592,755,98.62,'Sun-Synch' from dual union all
select 3700,'003700','1968-117U','deb Blok Kosmos R-12','',16844,'Reentered','1969-03-14 00:00:00','1969-02-11 00:00:00',93.6,394,508,70.99,'Intermediate' from dual union all
select 37000,'037000','1999-025ECG','deb FY-1C','',66706,'Reentered','2012-06-14 00:00:00','2010-07-29 00:00:00',99.76,556,938,98.66,'Sun-Synch' from dual union all
select 37001,'037001','1999-025ECH','deb FY-1C','',66706,'In Earth orbit','','2010-08-21 00:00:00',102.64,856,912,99.09,'Sun-Synch' from dual union all
select 37002,'037002','1999-025ECJ','deb FY-1C','',66706,'Reentered','2014-05-25 00:00:00','2010-08-05 00:00:00',99.6,485,994,97.37,'Sun-Synch' from dual union all
select 37003,'037003','1999-025ECK','deb FY-1C','',66706,'In Earth orbit','','2010-08-22 00:00:00',103.64,857,1004,99.44,'Sun-Synch' from dual union all
select 37004,'037004','1999-025ECL','deb FY-1C','',66706,'In Earth orbit','','2010-08-25 00:00:00',100.41,607,948,97.73,'Sun-Synch' from dual union all
select 37005,'037005','1999-025ECM','deb FY-1C','',66706,'In Earth orbit','','2010-08-14 00:00:00',100.28,692,852,98.94,'Sun-Synch' from dual union all
select 37006,'037006','1999-025ECN','deb FY-1C','',66706,'In Earth orbit','','2010-08-19 00:00:00',101.08,569,1051,99.28,'Sun-Synch' from dual union all
select 37007,'037007','1999-025ECP','deb FY-1C','',66706,'In Earth orbit','','2010-08-09 00:00:00',102.12,773,945,99.15,'Sun-Synch' from dual union all
select 37008,'037008','1999-025ECQ','deb FY-1C','',66706,'In Earth orbit','','2010-08-21 00:00:00',103.06,854,952,99.33,'Sun-Synch' from dual union all
select 37009,'037009','1999-025ECR','deb FY-1C','',66706,'In Earth orbit','','2010-08-23 00:00:00',102.98,795,1005,102.38,'Sun-Synch' from dual union all
select 3701,'003701','1968-097AL','deb Kosmos-252','',16824,'Reentered','2014-03-05 00:00:00','1992-02-03 00:00:00',104.83,535,1437,62.76,'Intermediate' from dual union all
select 37010,'037010','1999-025ECS','deb FY-1C','',66706,'Reentered','2011-05-25 00:00:00','2010-08-25 00:00:00',97.94,443,878,97.16,'Sun-Synch' from dual union all
select 37011,'037011','1999-025ECT','deb FY-1C','',66706,'In Earth orbit','','2010-08-26 00:00:00',100.9,738,865,98.95,'Sun-Synch' from dual union all
select 37012,'037012','1999-025ECU','deb FY-1C','',66706,'In Earth orbit','','2010-08-23 00:00:00',104.51,799,1144,98.6,'Sun-Synch' from dual union all
select 37013,'037013','1999-025ECV','deb FY-1C','',66706,'In Earth orbit','','2010-08-26 00:00:00',101.09,639,982,98.38,'Sun-Synch' from dual union all
select 37014,'037014','1999-025ECW','deb FY-1C','',66706,'In Earth orbit','','2010-08-24 00:00:00',102.56,848,912,98.93,'Sun-Synch' from dual union all
select 37015,'037015','1999-025ECX','deb FY-1C','',66706,'Reentered','2015-09-27 00:00:00','2010-08-12 00:00:00',100.24,690,850,96.38,'Sun-Synch' from dual union all
select 37016,'037016','1999-025ECY','deb FY-1C','',66706,'Reentered','2015-06-25 00:00:00','2010-08-22 00:00:00',102.04,712,999,95.87,'Sun-Synch' from dual union all
select 37017,'037017','1999-025ECZ','deb FY-1C','',66706,'In Earth orbit','','2010-08-18 00:00:00',102.31,581,1155,98.65,'Sun-Synch' from dual union all
select 37018,'037018','1999-025EDA','deb FY-1C','',66706,'In Earth orbit','','2010-08-09 00:00:00',105.1,690,1308,98.53,'Sun-Synch' from dual union all
select 37019,'037019','1999-025EDB','deb FY-1C','',66706,'In Earth orbit','','2010-08-20 00:00:00',101.71,684,996,99.05,'Sun-Synch' from dual union all
select 3702,'003702','1968-091AP','deb Kosmos-249','',16818,'Reentered','1976-12-05 00:00:00','1969-02-10 00:00:00',111.57,472,2126,62.3,'Intermediate' from dual union all
select 37020,'037020','1999-025EDC','deb FY-1C','',66706,'In Earth orbit','','2010-08-22 00:00:00',100.78,544,1047,97.84,'Sun-Synch' from dual union all
select 37021,'037021','1999-025EDD','deb FY-1C','',66706,'In Earth orbit','','2010-08-18 00:00:00',101.61,686,983,98.98,'Sun-Synch' from dual union all
select 37022,'037022','1999-025EDE','deb FY-1C','',66706,'In Earth orbit','','2010-08-24 00:00:00',103.75,813,1058,98.47,'Sun-Synch' from dual union all
select 37023,'037023','1999-025EDF','deb FY-1C','',66706,'Reentered','2011-12-17 00:00:00','2010-08-25 00:00:00',98.12,450,888,96.69,'Sun-Synch' from dual union all
select 37024,'037024','1999-025EDG','deb FY-1C','',66706,'In Earth orbit','','2010-08-24 00:00:00',100.98,764,846,99.35,'Sun-Synch' from dual union all
select 37025,'037025','1999-025EDH','deb FY-1C','',66706,'In Earth orbit','','2010-08-22 00:00:00',102.37,822,920,98.26,'Sun-Synch' from dual union all
select 37026,'037026','1999-025EDJ','deb FY-1C','',66706,'In Earth orbit','','2010-08-19 00:00:00',101.66,792,882,98.85,'Sun-Synch' from dual union all
select 37027,'037027','1999-025EDK','deb FY-1C','',66706,'In Earth orbit','','2010-08-17 00:00:00',101.41,733,918,98.88,'Sun-Synch' from dual union all
select 37028,'037028','1999-025EDL','deb FY-1C','',66706,'In Earth orbit','','2010-08-25 00:00:00',102.83,858,927,99.33,'Sun-Synch' from dual union all
select 37029,'037029','1999-025EDM','deb FY-1C','',66706,'In Earth orbit','','2010-08-01 00:00:00',101.22,785,848,99.71,'Sun-Synch' from dual union all
select 3703,'003703','1968-117V','deb Blok Kosmos R-12','',16844,'Reentered','1969-02-17 00:00:00','1969-02-11 00:00:00',90.3,251,327,71.05,'Intermediate' from dual union all
select 37030,'037030','1999-025EDN','deb FY-1C','',66706,'In Earth orbit','','2010-08-24 00:00:00',102.51,843,912,99.73,'Sun-Synch' from dual union all
select 37031,'037031','1999-025EDP','deb FY-1C','',66706,'In Earth orbit','','2010-08-21 00:00:00',100.96,763,846,99.69,'Sun-Synch' from dual union all
select 37032,'037032','1999-025EDQ','deb FY-1C','',66706,'In Earth orbit','','2010-08-10 00:00:00',100.63,626,951,99.39,'Sun-Synch' from dual union all
select 37033,'037033','1999-025EDR','deb FY-1C','',66706,'In Earth orbit','','2010-08-09 00:00:00',100.17,677,857,99.69,'Sun-Synch' from dual union all
select 37034,'037034','1999-025EDS','deb FY-1C','',66706,'In Earth orbit','','2010-07-30 00:00:00',100.37,700,853,98.95,'Sun-Synch' from dual union all
select 37035,'037035','1999-025EDT','deb FY-1C','',66706,'In Earth orbit','','2010-08-17 00:00:00',102.98,846,954,100.65,'Sun-Synch' from dual union all
select 37036,'037036','1999-025EDU','deb FY-1C','',66706,'In Earth orbit','','2010-08-24 00:00:00',102.9,806,986,99.29,'Sun-Synch' from dual union all
select 37037,'037037','1999-025EDV','deb FY-1C','',66706,'Reentered','2011-10-14 00:00:00','2010-08-24 00:00:00',96.99,462,768,99.4,'Sun-Synch' from dual union all
select 37038,'037038','1999-025EDW','deb FY-1C','',66706,'In Earth orbit','','2010-08-25 00:00:00',102.34,812,927,99.09,'Sun-Synch' from dual union all
select 37039,'037039','1999-025EDX','deb FY-1C','',66706,'In Earth orbit','','2010-08-23 00:00:00',99.17,605,833,98.95,'Sun-Synch' from dual union all
select 3704,'003704','1968-097AM','deb Kosmos-252 [*]                ','',16824,'Reentered','1974-09-09 00:00:00','1979-05-20 00:00:00',97.57,384,901,62.04,'Intermediate' from dual union all
select 37040,'037040','1999-025EDY','deb FY-1C','',66706,'In Earth orbit','','2010-08-24 00:00:00',102.33,797,940,99.55,'Sun-Synch' from dual union all
select 37041,'037041','1999-025EDZ','deb FY-1C','',66706,'In Earth orbit','','2010-07-29 00:00:00',104.03,815,1082,98.99,'Sun-Synch' from dual union all
select 37042,'037042','1999-025EEA','deb FY-1C','',66706,'In Earth orbit','','2010-08-04 00:00:00',101.17,725,904,98.68,'Sun-Synch' from dual union all
select 37043,'037043','1999-025EEB','deb FY-1C','',66706,'In Earth orbit','','2010-08-24 00:00:00',102.27,759,973,99.31,'Sun-Synch' from dual union all
select 37044,'037044','1999-025EEC','deb FY-1C','',66706,'In Earth orbit','','2010-08-25 00:00:00',107.72,639,1604,98.96,'Sun-Synch' from dual union all
select 37045,'037045','1999-025EED','deb FY-1C','',66706,'In Earth orbit','','2010-08-18 00:00:00',103.71,819,1048,98.37,'Sun-Synch' from dual union all
select 37046,'037046','1999-025EEE','deb FY-1C','',66706,'In Earth orbit','','2010-08-22 00:00:00',101.89,807,889,98.94,'Sun-Synch' from dual union all
select 37047,'037047','1999-025EEF','deb FY-1C','',66706,'In Earth orbit','','2010-08-21 00:00:00',106.16,819,1278,99.11,'Sun-Synch' from dual union all
select 37048,'037048','1999-025EEG','deb FY-1C','',66706,'In Earth orbit','','2010-08-17 00:00:00',105.83,837,1230,98.95,'Sun-Synch' from dual union all
select 37049,'037049','1999-025EEH','deb FY-1C','',66706,'In Earth orbit','','2010-08-24 00:00:00',102.22,823,905,99.27,'Sun-Synch' from dual union all
select 3705,'003705','1968-117W','deb Blok Kosmos R-12','',16844,'Reentered','1969-02-15 00:00:00','1969-02-03 00:00:00',90.59,250,357,70.98,'Intermediate' from dual union all
select 37050,'037050','1999-025EEJ','deb FY-1C','',66706,'Reentered','2015-03-31 00:00:00','2010-08-25 00:00:00',100.74,652,935,98.88,'Sun-Synch' from dual union all
select 37051,'037051','1999-025EEK','deb FY-1C','',66706,'In Earth orbit','','2010-08-22 00:00:00',100.73,732,854,98.95,'Sun-Synch' from dual union all
select 37052,'037052','1999-025EEL','deb FY-1C','',66706,'In Earth orbit','','2010-08-13 00:00:00',109.54,770,1641,98.74,'Sun-Synch' from dual union all
select 37053,'037053','1999-025EEM','deb FY-1C','',66706,'In Earth orbit','','2010-08-26 00:00:00',101.17,766,863,99.16,'Sun-Synch' from dual union all
select 37054,'037054','1999-025EEN','deb FY-1C','',66706,'In Earth orbit','','2010-08-20 00:00:00',101.79,768,919,99.04,'Sun-Synch' from dual union all
select 37055,'037055','1999-025EEP','deb FY-1C','',66706,'In Earth orbit','','2010-08-25 00:00:00',100.88,703,898,98.8,'Sun-Synch' from dual union all
select 37056,'037056','1999-025EEQ','deb FY-1C','',66706,'In Earth orbit','','2010-08-26 00:00:00',100.46,546,1014,98.93,'Sun-Synch' from dual union all
select 37057,'037057','1999-025EER','deb FY-1C','',66706,'In Earth orbit','','2010-08-13 00:00:00',103.52,708,1141,99.21,'Sun-Synch' from dual union all
select 37058,'037058','1999-025EES','deb FY-1C','',66706,'In Earth orbit','','2010-08-26 00:00:00',103.35,788,1045,96.73,'Sun-Synch' from dual union all
select 37059,'037059','1999-025EET','deb FY-1C','',66706,'In Earth orbit','','2010-08-21 00:00:00',100.54,582,987,98.69,'Sun-Synch' from dual union all
select 3706,'003706','1968-091AQ','deb Kosmos-249                    ','[UNID026]',16818,'Reentered','1970-07-05 00:00:00','1970-06-13 00:00:00',117.76,536,2625,62.39,'Intermediate' from dual union all
select 37060,'037060','1999-025EEU','deb FY-1C','',66706,'In Earth orbit','','2010-08-13 00:00:00',103.23,850,973,99.01,'Sun-Synch' from dual union all
select 37061,'037061','1999-025EEV','deb FY-1C','',66706,'In Earth orbit','','2010-08-20 00:00:00',112.78,602,2107,98.03,'Sun-Synch' from dual union all
select 37062,'037062','1999-025EEW','deb FY-1C','',66706,'In Earth orbit','','2010-08-09 00:00:00',100.18,609,926,98.55,'Sun-Synch' from dual union all
select 37063,'037063','1999-025EEX','deb FY-1C','',66706,'Reentered','2013-03-20 00:00:00','2010-08-24 00:00:00',97.32,426,835,99.18,'Sun-Synch' from dual union all
select 37064,'037064','1999-025EEY','deb FY-1C','',66706,'In Earth orbit','','2010-08-26 00:00:00',104.41,855,1078,98.96,'Sun-Synch' from dual union all
select 37065,'037065','1999-025EEZ','deb FY-1C','',66706,'In Earth orbit','','2010-07-31 00:00:00',107.31,859,1346,103.27,'Sun-Synch' from dual union all
select 37066,'037066','1999-025EFA','deb FY-1C','',66706,'Reentered','2015-12-04 00:00:00','2010-08-22 00:00:00',99.39,615,844,98.88,'Sun-Synch' from dual union all
select 37067,'037067','1999-025EFB','deb FY-1C','',66706,'In Earth orbit','','2010-08-20 00:00:00',105.75,833,1226,99.28,'Sun-Synch' from dual union all
select 37068,'037068','1999-025EFC','deb FY-1C','',66706,'In Earth orbit','','2010-08-26 00:00:00',100.65,577,1002,98.6,'Sun-Synch' from dual union all
select 37069,'037069','1999-025EFD','deb FY-1C','',66706,'In Earth orbit','','2010-08-24 00:00:00',109.1,829,1542,95.85,'Retrograde' from dual union all
select 3707,'003707','1965-020EQ','deb Blok Kosmos','',11770,'Reentered','1979-10-13 00:00:00','1979-09-30 00:00:00',97.32,471,790,56.37,'Intermediate' from dual union all
select 37070,'037070','1999-025EFE','deb FY-1C','',66706,'In Earth orbit','','2010-08-21 00:00:00',100.37,637,916,98.89,'Sun-Synch' from dual union all
select 37071,'037071','1999-025EFF','deb FY-1C','',66706,'Reentered','2012-07-01 00:00:00','2010-08-22 00:00:00',97.9,513,804,99.01,'Sun-Synch' from dual union all
select 37072,'037072','1999-025EFG','deb FY-1C','',66706,'Reentered','2012-02-19 00:00:00','2010-08-04 00:00:00',97.06,466,770,98.95,'Sun-Synch' from dual union all
select 37073,'037073','1999-025EFH','deb FY-1C','',66706,'Reentered','2013-03-17 00:00:00','2010-08-21 00:00:00',97.43,401,870,98.54,'Sun-Synch' from dual union all
select 37074,'037074','1999-025EFJ','deb FY-1C','',66706,'Lost','','2010-08-25 00:00:00',97.38,475,791,99.24,'Sun-Synch' from dual union all
select 37075,'037075','1999-025EFK','deb FY-1C','',66706,'Reentered','2011-08-12 00:00:00','2010-08-26 00:00:00',94.15,315,640,99.02,'Sun-Synch' from dual union all
select 37076,'037076','1999-025EFL','deb FY-1C','',66706,'In Earth orbit','','2010-08-24 00:00:00',103.18,820,998,98.35,'Sun-Synch' from dual union all
select 37077,'037077','1999-025EFM','deb FY-1C','',66706,'In Earth orbit','','2010-08-24 00:00:00',100.54,706,863,99.04,'Sun-Synch' from dual union all
select 37078,'037078','1999-025EFN','deb FY-1C','',66706,'Reentered','2012-03-16 00:00:00','2010-08-23 00:00:00',96.91,445,777,99.41,'Sun-Synch' from dual union all
select 37079,'037079','1999-025EFP','deb FY-1C','',66706,'In Earth orbit','','2010-08-22 00:00:00',102.21,842,885,98.94,'Sun-Synch' from dual union all
select 3708,'003708','1965-020ER','deb Blok Kosmos','',11770,'Reentered','2002-08-10 00:00:00','1992-02-02 00:00:00',103.08,602,1207,56.33,'Intermediate' from dual union all
select 37080,'037080','1999-025EFQ','deb FY-1C','',66706,'In Earth orbit','','2010-08-20 00:00:00',103.87,796,1087,98.17,'Sun-Synch' from dual union all
select 37081,'037081','1999-025EFR','deb FY-1C','',66706,'In Earth orbit','','2010-08-03 00:00:00',106.62,750,1391,98.79,'Sun-Synch' from dual union all
select 37082,'037082','2009-042CM','deb Briz-M','',68723,'Reentered','2010-09-15 00:00:00','2010-08-27 00:00:00',97.61,202,1086,48.34,'Intermediate' from dual union all
select 37083,'037083','2009-042CN','deb Briz-M','',68723,'Reentered','2010-10-01 00:00:00','2010-08-27 00:00:00',101.42,401,1251,48.24,'Intermediate' from dual union all
select 37084,'037084','1993-036BFE','deb Kosmos-2251','',65138,'In Earth orbit','','2010-08-31 00:00:00',97.94,545,776,74.04,'Intermediate' from dual union all
select 37085,'037085','1993-036BFF','deb Kosmos-2251','',65138,'In Earth orbit','','2010-08-31 00:00:00',98.62,611,775,73.99,'Intermediate' from dual union all
select 37086,'037086','1993-036BFG','deb Kosmos-2251','',65138,'In Earth orbit','','2010-08-31 00:00:00',99.89,699,808,74.01,'Intermediate' from dual union all
select 37087,'037087','1993-036BFH','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',101.88,630,1066,74.03,'Intermediate' from dual union all
select 37088,'037088','1993-036BFJ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',99.84,724,778,74.02,'Intermediate' from dual;
insert into SATELLITE
select 37089,'037089','1993-036BFK','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',102.15,688,1033,74.05,'Intermediate' from dual union all
select 3709,'003709','1968-117X','deb Blok Kosmos R-12','',16844,'Reentered','1969-02-21 00:00:00','1969-02-05 00:00:00',92.51,348,448,70.93,'Intermediate' from dual union all
select 37090,'037090','1993-036BFL','deb Kosmos-2251','',65138,'Reentered','2011-04-27 00:00:00','2010-09-01 00:00:00',95.06,383,660,74.08,'Intermediate' from dual union all
select 37091,'037091','1993-036BFM','deb Kosmos-2251','',65138,'In Earth orbit','','2010-08-31 00:00:00',98.13,557,782,73.92,'Intermediate' from dual union all
select 37092,'037092','1993-036BFN','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',100.48,759,804,74.04,'Intermediate' from dual union all
select 37093,'037093','1993-036BFP','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',99.84,597,904,74.03,'Intermediate' from dual union all
select 37094,'037094','1993-036BFQ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',101.39,748,901,74.11,'Intermediate' from dual union all
select 37095,'037095','1993-036BFR','deb Kosmos-2251','',65138,'In Earth orbit','','2010-08-31 00:00:00',103.86,770,1112,73.85,'Intermediate' from dual union all
select 37096,'037096','1993-036BFS','deb Kosmos-2251','',65138,'Reentered','2013-02-26 00:00:00','2010-09-01 00:00:00',98.06,570,762,73.91,'Intermediate' from dual union all
select 37097,'037097','1993-036BFT','deb Kosmos-2251','',65138,'In Earth orbit','','2010-08-31 00:00:00',99.76,722,773,74.04,'Intermediate' from dual union all
select 37098,'037098','1993-036BFU','deb Kosmos-2251','',65138,'Reentered','2014-03-02 00:00:00','2010-08-31 00:00:00',97.87,543,771,73.95,'Intermediate' from dual union all
select 37099,'037099','1993-036BFV','deb Kosmos-2251','',65138,'Reentered','2012-07-18 00:00:00','2010-09-01 00:00:00',96.85,472,744,73.99,'Intermediate' from dual union all
select 371,'000371','62 A Pi 1','[AMS Venera]','2MV-1 No. 1',8151,'Reentered Attached','1962-08-28 00:00:00','1962-08-25 00:00:00',88.71,174,248,64.88,'Intermediate' from dual union all
select 3710,'003710','1968-117Y','deb Blok Kosmos R-12','',16844,'Reentered','1969-03-04 00:00:00','1969-02-06 00:00:00',92.5,308,487,71.1,'Intermediate' from dual union all
select 37100,'037100','1993-036BFW','deb Kosmos-2251','',65138,'Reentered','2014-05-07 00:00:00','2010-08-31 00:00:00',99.92,697,813,74.04,'Intermediate' from dual union all
select 37101,'037101','1993-036BFX','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',100.24,718,821,74.04,'Intermediate' from dual union all
select 37102,'037102','1993-036BFY','deb Kosmos-2251','',65138,'Reentered','2016-06-23 00:00:00','2010-09-01 00:00:00',98.6,611,773,74.05,'Intermediate' from dual union all
select 37103,'037103','1993-036BFZ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-08-31 00:00:00',99.22,641,801,73.79,'Intermediate' from dual union all
select 37104,'037104','1993-036BGA','deb Kosmos-2251','',65138,'Reentered','2012-01-26 00:00:00','2010-08-31 00:00:00',97.2,507,743,73.99,'Intermediate' from dual union all
select 37105,'037105','1993-036BGB','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',105.23,676,1334,73.91,'Intermediate' from dual union all
select 37106,'037106','1993-036BGC','deb Kosmos-2251','',65138,'Reentered','2012-09-09 00:00:00','2010-08-31 00:00:00',97.63,540,750,74,'Intermediate' from dual union all
select 37107,'037107','1993-036BGD','deb Kosmos-2251','',65138,'Reentered','2011-12-12 00:00:00','2010-09-01 00:00:00',96.35,448,720,73.94,'Intermediate' from dual union all
select 37108,'037108','1993-036BGE','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',101.46,736,920,74.05,'Intermediate' from dual union all
select 37109,'037109','1993-036BGF','deb Kosmos-2251','',65138,'Reentered','2016-02-04 00:00:00','2010-08-31 00:00:00',99.65,689,795,73.99,'Intermediate' from dual union all
select 3711,'003711','1968-097AN','deb Kosmos-252','',16824,'Reentered','1969-04-22 00:00:00','1969-04-20 00:00:00',91.14,233,427,62.13,'Intermediate' from dual union all
select 37110,'037110','1993-036BGG','deb Kosmos-2251','',65138,'In Earth orbit','','2010-08-31 00:00:00',99.26,661,786,73.81,'Intermediate' from dual union all
select 37111,'037111','1993-036BGH','deb Kosmos-2251','',65138,'In Earth orbit','','2010-08-31 00:00:00',99,609,813,74.02,'Intermediate' from dual union all
select 37112,'037112','1993-036BGJ','deb Kosmos-2251','',65138,'Reentered','2014-03-13 00:00:00','2010-09-01 00:00:00',99.09,664,767,73.87,'Intermediate' from dual union all
select 37113,'037113','1993-036BGK','deb Kosmos-2251','',65138,'Reentered','2013-09-16 00:00:00','2010-08-31 00:00:00',98.19,582,762,73.98,'Intermediate' from dual union all
select 37114,'037114','1993-036BGL','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',100.73,742,845,73.83,'Intermediate' from dual union all
select 37115,'037115','1993-036BGM','deb Kosmos-2251','',65138,'Reentered','2018-03-01 00:00:00','2010-09-01 00:00:00',99.59,698,780,73.6,'Intermediate' from dual union all
select 37116,'037116','1993-036BGN','deb Kosmos-2251','',65138,'Reentered','2013-07-29 00:00:00','2010-09-01 00:00:00',98.3,559,796,73.95,'Intermediate' from dual union all
select 37117,'037117','1993-036BGP','deb Kosmos-2251','',65138,'Reentered','2011-08-10 00:00:00','2010-08-31 00:00:00',94.73,355,656,73.98,'Intermediate' from dual union all
select 37118,'037118','1993-036BGQ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-08-31 00:00:00',100.41,766,789,74.03,'Intermediate' from dual union all
select 37119,'037119','1993-036BGR','deb Kosmos-2251','',65138,'Reentered','2015-06-26 00:00:00','2010-09-01 00:00:00',100.1,610,917,73.97,'Intermediate' from dual union all
select 3712,'003712','1968-097AP','deb Kosmos-252','',16824,'Reentered','1973-02-26 00:00:00','1973-02-24 00:00:00',90.91,307,332,62.73,'Intermediate' from dual union all
select 37120,'037120','1993-036BGS','deb Kosmos-2251','',65138,'In Earth orbit','','2010-08-31 00:00:00',101.23,769,865,74.03,'Intermediate' from dual union all
select 37121,'037121','1993-036BGT','deb Kosmos-2251','',65138,'Reentered','2012-04-11 00:00:00','2010-08-31 00:00:00',98.47,620,751,73.88,'Intermediate' from dual union all
select 37122,'037122','1993-036BGU','deb Kosmos-2251','',65138,'Reentered','2015-04-03 00:00:00','2010-09-01 00:00:00',98.39,516,848,73.84,'Intermediate' from dual union all
select 37123,'037123','1993-036BGV','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',100.13,754,776,74.01,'Intermediate' from dual union all
select 37124,'037124','1993-036BGW','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',101.36,734,913,74.03,'Intermediate' from dual union all
select 37125,'037125','1993-036BGX','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',99.72,720,771,74.03,'Intermediate' from dual union all
select 37126,'037126','1993-036BGY','deb Kosmos-2251','',65138,'In Earth orbit','','2010-08-31 00:00:00',98.48,584,788,73.96,'Intermediate' from dual union all
select 37127,'037127','1993-036BGZ','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',101.04,769,847,74.02,'Intermediate' from dual union all
select 37128,'037128','1993-036BHA','deb Kosmos-2251','',65138,'Reentered','2011-06-02 00:00:00','2010-08-31 00:00:00',94.97,371,664,74.05,'Intermediate' from dual union all
select 37129,'037129','1993-036BHB','deb Kosmos-2251','',65138,'Reentered','2014-04-22 00:00:00','2010-08-31 00:00:00',99.3,674,777,74.01,'Intermediate' from dual union all
select 3713,'003713','1968-097AQ','deb Kosmos-252 [*]','',16824,'In Earth orbit','','1992-02-02 00:00:00',112.73,737,1967,62.23,'Intermediate' from dual union all
select 37130,'037130','1993-036BHC','deb Kosmos-2251','',65138,'Reentered','2014-04-19 00:00:00','2010-09-01 00:00:00',98.85,621,786,74.06,'Intermediate' from dual union all
select 37131,'037131','1993-036BHD','deb Kosmos-2251','',65138,'In Earth orbit','','2010-08-29 00:00:00',100.45,760,799,74.03,'Intermediate' from dual union all
select 37132,'037132','1993-036BHE','deb Kosmos-2251','',65138,'In Earth orbit','','2010-08-30 00:00:00',99.52,689,783,74.01,'Intermediate' from dual union all
select 37133,'037133','1993-036BHF','deb Kosmos-2251','',65138,'In Earth orbit','','2010-09-01 00:00:00',105.38,768,1256,74.02,'Intermediate' from dual union all
select 37134,'037134','1997-051XE','deb Iridium 33','',66131,'Reentered','2013-10-14 00:00:00','2010-08-31 00:00:00',99.94,737,775,86.36,'Polar' from dual union all
select 37135,'037135','1997-051XF','deb Iridium 33','',66131,'Reentered','2012-08-05 00:00:00','2010-08-31 00:00:00',99.64,725,758,86.3,'Polar' from dual union all
select 37136,'037136','1997-051XG','deb Iridium 33','',66131,'In Earth orbit','','2010-08-31 00:00:00',101.71,773,907,86.17,'Polar' from dual union all
select 37137,'037137','2010-041A','Kosmos-2464','Uragan-M No. 736',68901,'In Earth orbit','','2010-09-04 00:00:00',675.59,19120,19133,64.82,'Medium' from dual union all
select 37138,'037138','2010-041B','Kosmos-2465','Uragan-M No. 737',68901,'In Earth orbit','','2010-09-04 00:00:00',675.57,19118,19134,64.83,'Medium' from dual union all
select 37139,'037139','2010-041C','Kosmos-2466','Uragan-M No. 738',68901,'In Earth orbit','','2010-09-04 00:00:00',675.59,19116,19137,64.83,'Medium' from dual union all
select 3714,'003714','1968-097AR','deb Kosmos-252','',16824,'Reentered','1980-12-20 00:00:00','1981-01-14 00:00:00',91.74,313,407,62.23,'Intermediate' from dual union all
select 37140,'037140','2010-041D','Blok DM-2 No. 118L','11S861 No. 118L',68901,'In Earth orbit','','2010-09-04 00:00:00',674.6,19068,19135,64.82,'Medium' from dual union all
select 37141,'037141','2010-041E','8S812','8S812',68901,'Reentered','2010-09-02 00:00:00','2010-09-02 00:00:00',87.79,155,176,64.83,'Intermediate' from dual union all
select 37142,'037142','2010-041F','Perekhodnik','Sredniy perekhodnik',68901,'Reentered','2010-09-03 00:00:00','2010-09-02 00:00:00',87.74,152,173,64.81,'Intermediate' from dual union all
select 37143,'037143','2010-041G','SOZ','SOZ',68901,'In Earth orbit','','2010-09-08 00:00:00',340.34,402,19137,64.86,'Highly Ellip' from dual union all
select 37144,'037144','2010-041H','SOZ','SOZ',68901,'In Earth orbit','','2010-09-03 00:00:00',340.34,405,19134,64.77,'Highly Ellip' from dual union all
select 37145,'037145','2010-041J','deb 8S812','',68901,'Reentered','2010-09-05 00:00:00','2010-09-05 00:00:00',88.39,143,246,64.79,'Intermediate' from dual union all
select 37146,'037146','2010-041K','deb 8S812','',68901,'Reentered','2010-09-05 00:00:00','2010-09-04 00:00:00',89.96,170,375,64.66,'Intermediate' from dual union all
select 37147,'037147','2010-041L','deb 8S812','',68901,'Reentered','2010-09-06 00:00:00','2010-09-04 00:00:00',89.92,165,376,64.81,'Intermediate' from dual union all
select 37148,'037148','1974-089FH','deb Delta 104','',33692,'In Earth orbit','','2010-09-02 00:00:00',91.79,156,569,64.97,'Intermediate' from dual union all
select 37149,'037149','2010-041M','deb 8S812','',68901,'Reentered','2010-09-06 00:00:00','2010-09-06 00:00:00',88.43,158,236,64.96,'Intermediate' from dual union all
select 3715,'003715','1968-097AS','deb Kosmos-252 [*]','',16824,'Reentered','1976-12-14 00:00:00','1976-11-21 00:00:00',90.24,266,307,61.88,'Intermediate' from dual union all
select 37150,'037150','2010-042A','Zhongxing 6A','zhongxing liu A weixing',68902,'In Earth orbit','','2010-09-28 00:00:00',1436.06,35783,35789,.39,'Stationary' from dual union all
select 37151,'037151','2010-042B','CZ-3B Stage 3','CZ-3B Stage 3',68902,'Reentered','2015-12-04 00:00:00','2010-09-05 00:00:00',753.2,233,41854,25.04,'Highly Ellip' from dual union all
select 37152,'037152','2010-043A','Gonets-M No. 12','Gonets-M No. 12',68903,'In Earth orbit','','2010-09-09 00:00:00',115.99,1495,1506,82.45,'Intermediate' from dual union all
select 37153,'037153','2010-043B','Kosmos-2467','Strela-3 No. 56',68903,'In Earth orbit','','2010-09-09 00:00:00',116.06,1498,1508,82.45,'Intermediate' from dual union all
select 37154,'037154','2010-043C','Kosmos-2468','Rodnik No. 13',68903,'In Earth orbit','','2010-09-09 00:00:00',116.02,1497,1507,82.46,'Intermediate' from dual union all
select 37155,'037155','2010-043D','Briz-KM No. 72514','Briz-KM 14S45',68903,'In Earth orbit','','2010-09-09 00:00:00',112.62,1192,1502,82.46,'Intermediate' from dual union all
select 37156,'037156','2010-044A','Progress M-07M','Progress 7K-TGM No.407',68904,'Deorbited','2011-02-20 00:00:00','2010-09-10 00:00:00',89.74,249,275,51.64,'Intermediate' from dual union all
select 37157,'037157','2010-044B','11S510','11S510',68904,'Reentered','2010-09-12 00:00:00','2010-09-10 00:00:00',88.47,186,212,51.62,'Intermediate' from dual union all
select 37158,'037158','2010-045A','Michibiki','QZSS',68905,'In Earth orbit','','2010-10-12 00:00:00',1436.14,32613,38962,40.95,'Synchronous' from dual union all
select 37159,'037159','2010-045B','H-2A F18 Stage 2','H-2A Stage 2',68905,'In Earth orbit','','2010-10-11 00:00:00',624.24,278,35356,31.77,'GEO Transfer' from dual union all
select 3716,'003716','1968-097AT','deb Kosmos-252','',16824,'Reentered','1981-09-12 00:00:00','1969-03-01 00:00:00',115.4,530,2418,62.3,'Intermediate' from dual union all
select 37160,'037160','2009-042CP','deb Briz-M','',68723,'Reentered','2010-09-29 00:00:00','2010-09-14 00:00:00',106.83,402,1758,46.54,'Intermediate' from dual union all
select 37161,'037161','2009-042CQ','deb Briz-M','',68723,'Reentered','2010-10-08 00:00:00','2010-09-14 00:00:00',104.74,364,1601,47.92,'Intermediate' from dual union all
select 37162,'037162','2010-046A','USA 215','TOPAZ 1',68906,'In Earth orbit','','2010-10-27 00:00:00',107.33,1102,1105,122.99,'Retrograde' from dual union all
select 37163,'037163','2009-042CR','deb Briz-M','',68723,'Reentered','2010-12-13 00:00:00','2010-09-21 00:00:00',113.38,667,2097,48.9,'Intermediate' from dual union all
select 37164,'037164','2009-042CS','deb Briz-M','',68723,'Reentered','2010-10-14 00:00:00','2010-09-21 00:00:00',103.65,339,1523,48.41,'Intermediate' from dual union all
select 37165,'037165','2010-047A','Yaogan Weixing 11','Jianbing-6 04 xing',68907,'In Earth orbit','','2010-10-21 00:00:00',97.52,623,657,98,'Sun-Synch' from dual union all
select 37166,'037166','2010-047B','Zheda Pixing 1A','ZDPS 1A-1',68907,'In Earth orbit','','2010-10-21 00:00:00',97.49,621,657,98.01,'Sun-Synch' from dual union all
select 37167,'037167','2010-047C','Zheda Pixing 1A','ZDPS 1A-2',68907,'In Earth orbit','','2010-10-21 00:00:00',97.5,622,657,97.99,'Sun-Synch' from dual union all
select 37168,'037168','2010-048A','SBSS 1','SBSS Block 10',68908,'In Earth orbit','','2010-11-10 00:00:00',97.36,631,634,98.02,'Sun-Synch' from dual union all
select 37169,'037169','2010-048B','Minotaur IV GCA','GCA/Orion 38',68908,'In Earth orbit','','2010-11-10 00:00:00',95.33,530,540,98,'Sun-Synch' from dual union all
select 3717,'003717','1968-097AU','deb Kosmos-252','',16824,'Reentered','2001-12-25 00:00:00','1992-02-02 00:00:00',101.87,487,1208,62.28,'Intermediate' from dual union all
select 37170,'037170','2010-049A','Kosmos-2469','Oko US-KS',68909,'In Earth orbit','','2010-10-31 00:00:00',717.64,607,39741,62.81,'Molniya' from dual union all
select 37171,'037171','2010-049B','Blok-2BL','Blok-2BL',68909,'In Earth orbit','','2010-11-01 00:00:00',704.68,658,39049,62.85,'Molniya' from dual union all
select 37172,'037172','2010-049C','11S510','11S510',68909,'Reentered','2010-12-14 00:00:00','2010-10-30 00:00:00',91.18,220,445,62.79,'Intermediate' from dual union all
select 37173,'037173','2010-049D','BOZ','BOZ',68909,'Reentered','2010-11-01 00:00:00','2010-10-07 00:00:00',91.62,189,519,62.82,'Intermediate' from dual union all
select 37174,'037174','2010-050A','Chang''e-2','chang''e erhao weixing',68910,'Deep Space','2010-10-02 00:00:00','2010-10-01 00:00:00',14450.95,300,380000,32.4,'Cislunar' from dual union all
select 37175,'037175','2010-050B','CZ-3C Y7 Stage 3','CZ-3C Stage 3',68910,'Deep Space','2010-10-02 00:00:00','2010-11-15 00:00:00',33684.12,244195,434042,78.38,'Cislunar' from dual union all
select 37176,'037176','2010-049E','deb 11S510','',68909,'Reentered','2010-11-21 00:00:00','2010-10-07 00:00:00',92.12,223,534,62.83,'Intermediate' from dual union all
select 37177,'037177','2009-042CT','deb Briz-M','',68723,'Reentered','2010-10-27 00:00:00','2010-10-05 00:00:00',105.82,353,1712,48.08,'Intermediate' from dual union all
select 37178,'037178','2009-042CU','deb Briz-M','',68723,'Reentered','2010-11-23 00:00:00','2010-10-05 00:00:00',127.13,868,3126,47.69,'Medium' from dual;
insert into SATELLITE
select 37179,'037179','2010-051A','Shi Jian 6/4B','shijian liuhao 4 zu B xi',68911,'In Earth orbit','','2010-11-05 00:00:00',96.6,586,606,97.75,'Sun-Synch' from dual union all
select 3718,'003718','1965-082QZ','deb Transtage 4','',11832,'Reentered','1988-09-21 00:00:00','1988-09-12 00:00:00',90.42,157,433,31.91,'Intermediate' from dual union all
select 37180,'037180','2010-051B','Shi Jian 6/4A','shijian liuhao 4 zu A xi',68911,'In Earth orbit','','2010-11-05 00:00:00',96.6,588,604,97.75,'Sun-Synch' from dual union all
select 37181,'037181','2010-051C','CZ-4B Stage  3','CZ-4B',68911,'In Earth orbit','','2010-11-05 00:00:00',96.68,511,689,97.61,'Sun-Synch' from dual union all
select 37182,'037182','2010-051D','Adapter','',68911,'In Earth orbit','','2010-11-05 00:00:00',96.57,585,603,97.75,'Sun-Synch' from dual union all
select 37183,'037183','2010-052A','Soyuz TMA-01M','Soyuz 11F732A47 No. 701',68912,'Landed','2011-03-16 00:00:00','2010-10-09 00:00:00',89.8,245,284,51.65,'Intermediate' from dual union all
select 37184,'037184','2010-052B','11S510','11S510',68912,'Reentered','2010-10-11 00:00:00','2010-10-10 00:00:00',88.26,175,202,51.61,'Intermediate' from dual union all
select 37185,'037185','2010-053A','Sirius XM-5','',68913,'In Earth orbit','','2010-11-14 00:00:00',1436.09,35780,35793,0,'Stationary' from dual union all
select 37186,'037186','2010-053B','Briz-M No. 99517','Briz-M',68913,'In Earth orbit','','2010-11-14 00:00:00',663.45,3588,34052,22.57,'GEO Transfer' from dual union all
select 37187,'037187','2010-053C','Briz-M DTB','Briz-M torus tank',68913,'In Earth orbit','','2010-11-14 00:00:00',279.35,323,15231,49.55,'Highly Ellip' from dual union all
select 37188,'037188','2010-054A','Globalstar M079','Globalstar-2 FM8',68914,'In Earth orbit','','2010-11-18 00:00:00',104.94,989,995,52.01,'Intermediate' from dual union all
select 37189,'037189','2010-054B','Globalstar M074','Globalstar-2 FM3',68914,'In Earth orbit','','2010-11-19 00:00:00',105.18,995,1011,52,'Intermediate' from dual union all
select 3719,'003719','1965-082RA','deb Transtage 4','',11832,'Reentered','1971-03-26 00:00:00','1969-02-13 00:00:00',97.58,602,684,32.16,'Intermediate' from dual union all
select 37190,'037190','2010-054C','Globalstar M076','Globalstar-2 FM5',68914,'In Earth orbit','','2010-11-18 00:00:00',105.29,1004,1012,52.01,'Intermediate' from dual union all
select 37191,'037191','2010-054D','Globalstar M077','Globalstar-2 FM6',68914,'In Earth orbit','','2010-11-18 00:00:00',103.42,916,924,52,'Intermediate' from dual union all
select 37192,'037192','2010-054E','Globalstar M075','Globalstar-2 FM4',68914,'In Earth orbit','','2010-11-18 00:00:00',114.06,1412,1413,51.99,'Intermediate' from dual union all
select 37193,'037193','2010-054F','Globalstar M073','Globalstar-2 FM2',68914,'In Earth orbit','','2010-11-18 00:00:00',103.43,917,925,52,'Intermediate' from dual union all
select 37194,'037194','2010-054G','Fregat No. 1023','Fregat 14S44 No. 1023',68914,'Deorbited','2010-10-19 00:00:00','2010-10-19 00:00:00',103.37,912,924,52.01,'Intermediate' from dual union all
select 37195,'037195','1991-063G','deb UARS','',64232,'Reentered','2010-11-04 00:00:00','2010-10-25 00:00:00',91.39,325,361,56.88,'Intermediate' from dual union all
select 37196,'037196','2010-055A','Progress M-08M','Progress 7K-TGM No.408',68915,'Deorbited','2011-01-24 00:00:00','2010-10-29 00:00:00',89.75,250,274,51.65,'Intermediate' from dual union all
select 37197,'037197','2010-055B','11S510','11S510',68915,'Reentered','2010-10-29 00:00:00','2010-10-28 00:00:00',88.37,181,207,51.63,'Intermediate' from dual union all
select 37198,'037198','2008-011C','Briz-M DTB debris','',68519,'In Earth orbit','','2010-10-28 00:00:00',463.96,368,26581,48.94,'Highly Ellip' from dual union all
select 37199,'037199','2008-011D','deb Briz-M No. 88525','',68519,'In Earth orbit','','2010-10-27 00:00:00',469.88,737,26549,48.89,'Highly Ellip' from dual union all
select 372,'000372','62 A Pi 2','8K78I','8K78I',8151,'Reentered','1962-09-02 00:00:00','1962-08-27 00:00:00',89.06,179,277,64.89,'Intermediate' from dual union all
select 3720,'003720','1965-082RB','deb Transtage 4','',11832,'Reentered','1979-11-07 00:00:00','1969-02-13 00:00:00',99.05,700,727,32.43,'Intermediate' from dual union all
select 37200,'037200','2008-011E','deb Briz-M No. 88525','Large debris (2 sq m)',68519,'In Earth orbit','','2010-10-28 00:00:00',476.57,1070,26595,48.69,'Highly Ellip' from dual union all
select 37201,'037201','2008-011F','deb Briz-M No. 88525','',68519,'In Earth orbit','','2010-10-27 00:00:00',470.55,769,26555,48.88,'Highly Ellip' from dual union all
select 37202,'037202','2008-011G','deb Briz-M No. 88525','',68519,'In Earth orbit','','2010-10-28 00:00:00',469.02,721,26516,48.83,'Highly Ellip' from dual union all
select 37203,'037203','2008-011H','deb Briz-M No. 88525','',68519,'In Earth orbit','','2010-10-28 00:00:00',466.08,565,26505,48.53,'Highly Ellip' from dual union all
select 37204,'037204','2008-011J','deb Briz-M No. 88525','',68519,'In Earth orbit','','2010-10-27 00:00:00',469.79,731,26550,48.89,'Highly Ellip' from dual union all
select 37205,'037205','2008-011K','deb Briz-M No. 88525','',68519,'In Earth orbit','','2010-10-27 00:00:00',470.66,771,26559,48.9,'Highly Ellip' from dual union all
select 37206,'037206','2010-056A','Eutelsat W3B','Eutelsat W3B',68916,'In Earth orbit','','2010-11-30 00:00:00',632.32,264,35787,1.94,'GEO Transfer' from dual union all
select 37207,'037207','2010-056B','BSAT-3B','BSAT 3B',68916,'In Earth orbit','','2010-11-28 00:00:00',1436.04,35765,35806,.05,'Stationary' from dual union all
select 37208,'037208','2010-056C','ESC-A 555','Ariane 5 ESC-A',68916,'In Earth orbit','','2010-11-28 00:00:00',629.7,260,35656,2.28,'GEO Transfer' from dual union all
select 37209,'037209','2010-056D','Sylda 5A','Sylda 5 No. 40-A',68916,'Reentered','2015-11-02 00:00:00','2010-11-28 00:00:00',628.5,257,35597,1.98,'GEO Transfer' from dual union all
select 3721,'003721','1965-082RC','deb Transtage 4','',11832,'Reentered','1972-07-27 00:00:00','1969-02-13 00:00:00',98.44,658,711,32.3,'Intermediate' from dual union all
select 37210,'037210','2010-057A','Beidou DW 6','di liu ke beidou daohang',68917,'In Earth orbit','','2010-12-01 00:00:00',1436.09,35779,35794,1.78,'Stationary' from dual union all
select 37211,'037211','2010-057B','CZ-3C Stage 3','CZ-3C Stage 3',68917,'Reentered','2011-09-28 00:00:00','2010-12-01 00:00:00',608.45,152,34663,20.29,'GEO Transfer' from dual union all
select 37212,'037212','2010-058A','Meridian No. 13L','Meridian No. 13L',68918,'In Earth orbit','','2010-12-02 00:00:00',717.62,959,39388,62.81,'Molniya' from dual union all
select 37213,'037213','2010-058B','Fregat No. 1022','Fregat 14S44 No. 1022',68918,'In Earth orbit','','2010-12-02 00:00:00',718.92,798,39613,62.83,'Molniya' from dual union all
select 37214,'037214','2010-059A','Feng Yun 3B','fengyun sanhao 01 B xing',68919,'In Earth orbit','','2010-12-05 00:00:00',101.42,825,827,98.7,'Sun-Synch' from dual union all
select 37215,'037215','2010-059B','CZ-4C Stage 3','CZ-4C Stage 3',68919,'In Earth orbit','','2010-12-05 00:00:00',99.63,677,805,98.76,'Sun-Synch' from dual union all
select 37216,'037216','2010-060A','COSMO 4','COSMO-Skymed 4',68920,'In Earth orbit','','2010-12-06 00:00:00',97.14,621,622,97.86,'Sun-Synch' from dual union all
select 37217,'037217','2010-060B','Delta 350','Delta SSPS AJ-10-118K',68920,'Reentered','2010-12-16 00:00:00','2010-11-13 00:00:00',92.33,184,594,98.31,'Sun-Synch' from dual union all
select 37218,'037218','2010-061A','SkyTerra 1','MSV 1',68921,'In Earth orbit','','2011-01-19 00:00:00',1436.06,35772,35800,6,'Inclined GEO' from dual union all
select 37219,'037219','2010-061B','Briz-M No. 99516','Briz-M',68921,'In Earth orbit','','2010-11-22 00:00:00',1240.67,27933,35805,7.47,'Medium' from dual union all
select 3722,'003722','1965-082RD','deb Transtage 4','',11832,'Reentered','1970-05-15 00:00:00','1969-02-13 00:00:00',99.68,735,752,32.38,'Intermediate' from dual union all
select 37220,'037220','2010-061C','Briz-M DTB','Briz-M torus tank',68921,'In Earth orbit','','2010-11-22 00:00:00',306.1,320,17015,49.5,'Highly Ellip' from dual union all
select 37221,'037221','1998-067CB','EVA debris','',66503,'Reentered','2010-12-16 00:00:00','2010-11-16 00:00:00',91.51,344,354,51.64,'Intermediate' from dual union all
select 37222,'037222','2010-062A','STPSat-2','STP-SIV',68922,'In Earth orbit','','2010-11-20 00:00:00',97.55,628,654,71.9,'Intermediate' from dual union all
select 37223,'037223','2010-062B','RAX','Radio Aurora Explorer',68922,'In Earth orbit','','2010-12-02 00:00:00',97.46,622,653,71.97,'Intermediate' from dual union all
select 37224,'037224','2010-062C','O/OREOS','O/OREOS',68922,'In Earth orbit','','2010-12-03 00:00:00',97.49,624,654,71.97,'Intermediate' from dual union all
select 37225,'037225','2010-062D','Fastsat-HSV','Fastsat-HSV',68922,'In Earth orbit','','2010-12-03 00:00:00',97.52,626,653,71.97,'Intermediate' from dual union all
select 37226,'037226','2010-062E','Falconsat-5','Falconsat-5',68922,'In Earth orbit','','2010-11-20 00:00:00',97.6,633,654,71.9,'Intermediate' from dual union all
select 37227,'037227','2010-062F','Fastrac-OSCAR-69','Fastrac 1',68922,'In Earth orbit','','2010-12-01 00:00:00',97.53,627,654,71.97,'Intermediate' from dual union all
select 37228,'037228','2010-062G','Minotaur IV GCA','GCA/Orion 38',68922,'In Earth orbit','','2010-11-20 00:00:00',97.6,633,654,71.9,'Intermediate' from dual union all
select 37229,'037229','2010-062H','HAPS Stage','',68922,'In Earth orbit','','2010-11-20 00:00:00',109.44,1201,1201,71.97,'Intermediate' from dual union all
select 3723,'003723','1965-082RE','deb Transtage 4','',11832,'Reentered','1979-07-14 00:00:00','1969-02-18 00:00:00',101.7,724,955,33.34,'Intermediate' from dual union all
select 37230,'037230','2010-062J','STP-S26 Ballast A','',68922,'In Earth orbit','','2010-11-20 00:00:00',109.46,1200,1204,72,'Intermediate' from dual union all
select 37231,'037231','2010-062K','STP-S26 Ballast B','',68922,'In Earth orbit','','2010-11-20 00:00:00',109.46,1200,1204,72,'Intermediate' from dual union all
select 37232,'037232','2010-063A','USA 223','ORION 7',68923,'In Earth orbit','','2010-11-22 00:00:00',1434.17,35555,35943,6,'Inclined Drift' from dual union all
select 37233,'037233','2010-063B','Delta 351','DCSS-5 F05',68923,'In Earth orbit','','2010-12-12 00:00:00',1384.12,33852,35674,7.05,'Inclined Drift' from dual union all
select 37234,'037234','2010-064A','Zhongxing 20A','Shentong yi hao 02 xing',68924,'In Earth orbit','','2010-12-02 00:00:00',1436.04,35772,35799,.55,'Stationary' from dual union all
select 37235,'037235','2010-064B','CZ-3A Stage 3','CZ-3A Stage 3',68924,'In Earth orbit','','2010-12-06 00:00:00',752.31,205,41840,24.82,'Highly Ellip' from dual union all
select 37236,'037236','1977-087D','deb Kosmos-951','',42215,'In Earth orbit','','2010-12-03 00:00:00',104.79,960,1009,82.97,'Intermediate' from dual union all
select 37237,'037237','2010-065A','Hylas','Hylas',68925,'In Earth orbit','','2010-12-16 00:00:00',1436.14,35774,35801,.05,'Stationary' from dual union all
select 37238,'037238','2010-065B','Intelsat IS-17','Intelsat IS-17',68925,'In Earth orbit','','2010-12-03 00:00:00',1429.5,35512,35803,.08,'Drift GEO' from dual union all
select 37239,'037239','2010-065C','ESC-A 556','Ariane 5 ESC-A',68925,'Reentered','2014-08-16 00:00:00','2010-12-04 00:00:00',630.65,215,35750,1.78,'GEO Transfer' from dual union all
select 3724,'003724','1965-082RF','deb Transtage 4','',11832,'Reentered','1969-12-06 00:00:00','1969-02-13 00:00:00',106.64,733,1409,32.46,'Intermediate' from dual union all
select 37240,'037240','2010-065D','Sylda 5A','Sylda 5 No. A',68925,'Reentered','2011-06-21 00:00:00','2010-12-04 00:00:00',631.29,226,35772,2.03,'GEO Transfer' from dual union all
select 37241,'037241','1988-089D','deb NOAA 11','',61338,'In Earth orbit','','2010-12-03 00:00:00',101.53,812,851,98.78,'Sun-Synch' from dual union all
select 37242,'037242','1988-089E','deb NOAA 11','',61338,'In Earth orbit','','2010-12-03 00:00:00',101.91,842,857,98.76,'Sun-Synch' from dual union all
select 37243,'037243','2006-015N','deb Yaogan','',68187,'Reentered','2010-12-20 00:00:00','2010-12-02 00:00:00',94.36,471,505,97.41,'Sun-Synch' from dual union all
select 37244,'037244','2010-066A','Dragon C1','Dragon C101',68926,'Landed','2010-12-08 00:00:00','2010-12-08 00:00:00',90.38,279,308,34.54,'Intermediate' from dual union all
select 37245,'037245','2010-066B','QbX-2','NRO Colony I',68926,'Reentered','2011-01-16 00:00:00','2010-12-15 00:00:00',90.33,277,305,34.54,'Intermediate' from dual union all
select 37246,'037246','2010-066C','SMDC-ONE','SMDC-ONE',68926,'Reentered','2011-01-12 00:00:00','2010-12-15 00:00:00',90.29,276,302,34.53,'Intermediate' from dual union all
select 37247,'037247','2010-066D','Perseus 003','LANL Cubesat 3',68926,'Reentered','2010-12-31 00:00:00','2010-12-15 00:00:00',90.21,270,300,34.54,'Intermediate' from dual union all
select 37248,'037248','2010-066E','Perseus 001','LANL Cubesat 1',68926,'Reentered','2010-12-31 00:00:00','2010-12-15 00:00:00',90.2,270,299,34.54,'Intermediate' from dual union all
select 37249,'037249','2010-066F','QbX-1','NRO Colony I',68926,'Reentered','2011-01-06 00:00:00','2010-12-15 00:00:00',90.29,276,302,34.53,'Intermediate' from dual union all
select 3725,'003725','1965-082RG','deb Transtage 4','',11832,'Reentered','1969-11-11 00:00:00','1969-02-13 00:00:00',102.08,697,1017,32.67,'Intermediate' from dual union all
select 37250,'037250','2010-066G','Perseus 002','LANL Cubesat 2',68926,'Reentered','2010-12-30 00:00:00','2010-12-15 00:00:00',90.15,269,295,34.53,'Intermediate' from dual union all
select 37251,'037251','2010-066H','Perseus 000','LANL Cubesat 0',68926,'Reentered','2010-12-30 00:00:00','2010-12-15 00:00:00',90.15,269,295,34.53,'Intermediate' from dual union all
select 37252,'037252','2010-066J','Caerus/Mayflower','Caerus',68926,'Reentered','2010-12-22 00:00:00','2010-12-10 00:00:00',90.44,279,314,34.54,'Intermediate' from dual union all
select 37253,'037253','2010-066K','Dragon Trunk','Dragon SM Qual',68926,'In Earth orbit','','2010-12-15 00:00:00',219.62,278,11081,34.52,'Medium' from dual union all
select 37254,'037254','2010-067A','Soyuz TMA-20','Soyuz 11F732A17 No. 230',68927,'Landed','2011-05-24 00:00:00','2010-12-17 00:00:00',89.41,228,263,51.65,'Intermediate' from dual union all
select 37255,'037255','2010-067B','11S510','11S510',68927,'Reentered','2010-12-19 00:00:00','2010-12-18 00:00:00',88.17,173,195,51.62,'Intermediate' from dual union all
select 37256,'037256','2010-068A','Beidou DW 7','diqi ke beidou daohang w',68928,'In Earth orbit','','2010-12-25 00:00:00',1436.06,35716,35856,55.23,'Synchronous' from dual union all
select 37257,'037257','2010-068B','CZ-3A Stage 3','CZ-3A Stage 3',68928,'Reentered','2011-01-31 00:00:00','2010-12-26 00:00:00',591.88,97,33850,55.31,'Highly Ellip' from dual union all
select 37258,'037258','2010-069A','KA-Sat','KA-Sat',68929,'In Earth orbit','','2011-01-20 00:00:00',1436.04,35780,35791,.12,'Stationary' from dual union all
select 37259,'037259','2010-069B','Briz-M No. 99518','Briz-M',68929,'In Earth orbit','','2010-12-29 00:00:00',649.62,3138,33799,24.3,'GEO Transfer' from dual union all
select 3726,'003726','1965-082RH','deb Transtage 4','',11832,'Reentered','1969-04-06 00:00:00','1969-02-14 00:00:00',94.6,473,525,32.92,'Intermediate' from dual union all
select 37260,'037260','2010-069C','Briz-M DTB','Briz-M torus tank',68929,'In Earth orbit','','2010-12-29 00:00:00',268.8,328,14509,49.59,'Highly Ellip' from dual union all
select 37261,'037261','2007-005R','deb H2A Adapter','',68347,'Reentered','2011-01-02 00:00:00','2010-12-28 00:00:00',92.39,375,409,97.35,'Sun-Synch' from dual union all
select 37262,'037262','2007-005S','deb H2A Adapter','',68347,'Reentered','2011-01-21 00:00:00','2010-12-28 00:00:00',93,412,430,97.34,'Sun-Synch' from dual union all
select 37263,'037263','2007-005T','deb H2A Adapter','',68347,'Reentered','2011-01-14 00:00:00','2010-12-28 00:00:00',93.15,423,435,97.38,'Sun-Synch' from dual union all
select 37264,'037264','2010-070A','Hispasat 1E','Hispasat 1E',68930,'In Earth orbit','','2011-01-10 00:00:00',1435.98,35783,35786,.06,'Stationary' from dual union all
select 37265,'037265','2010-070B','Olleh 1','Koreasat 6',68930,'In Earth orbit','','2011-01-10 00:00:00',1436.11,35780,35794,.05,'Stationary' from dual union all
select 37266,'037266','2010-070C','ESC-A 557','Ariane 5 ESC-A',68930,'In Earth orbit','','2011-01-01 00:00:00',631.12,256,35733,3,'GEO Transfer' from dual union all
select 37267,'037267','2010-070D','Sylda 5A','Sylda 5 No.',68930,'In Earth orbit','','2011-01-01 00:00:00',631.14,256,35734,3,'GEO Transfer' from dual union all
select 37268,'037268','1993-036BHG','deb Kosmos-2251','',65138,'Reentered','2011-08-04 00:00:00','2011-01-04 00:00:00',94.75,380,633,73.97,'Intermediate' from dual;
insert into SATELLITE
select 37269,'037269','1993-036BHH','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',102.42,770,977,74.15,'Intermediate' from dual union all
select 3727,'003727','1965-082RJ','deb Transtage 4','',11832,'Reentered','1969-06-22 00:00:00','1969-02-14 00:00:00',98.01,642,685,32.29,'Intermediate' from dual union all
select 37270,'037270','1993-036BHJ','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',107.79,765,1484,73.94,'Intermediate' from dual union all
select 37271,'037271','1993-036BHK','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',99.82,732,768,74.06,'Intermediate' from dual union all
select 37272,'037272','1993-036BHL','deb Kosmos-2251','',65138,'Reentered','2012-03-02 00:00:00','2011-01-04 00:00:00',96.07,432,709,73.87,'Intermediate' from dual union all
select 37273,'037273','1993-036BHM','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',100.85,766,832,74.02,'Intermediate' from dual union all
select 37274,'037274','1993-036BHN','deb Kosmos-2251','',65138,'Reentered','2014-10-18 00:00:00','2011-01-04 00:00:00',99.94,696,816,74.04,'Intermediate' from dual union all
select 37275,'037275','1993-036BHP','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',102.94,752,1044,73.95,'Intermediate' from dual union all
select 37276,'037276','1993-036BHQ','deb Kosmos-2251','',65138,'Reentered','2014-11-16 00:00:00','2011-01-03 00:00:00',97.36,498,767,73.86,'Intermediate' from dual union all
select 37277,'037277','1993-036BHR','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',100.67,759,822,74.06,'Intermediate' from dual union all
select 37278,'037278','1993-036BHS','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',103.29,650,1179,73.79,'Intermediate' from dual union all
select 37279,'037279','1993-036BHT','deb Kosmos-2251','',65138,'Reentered','2014-10-28 00:00:00','2011-01-03 00:00:00',96.89,458,762,73.86,'Intermediate' from dual union all
select 3728,'003728','1965-082RK','deb Transtage 4','',11832,'Reentered','1970-02-01 00:00:00','1969-02-14 00:00:00',97.6,623,665,32.27,'Intermediate' from dual union all
select 37280,'037280','1993-036BHU','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',102.89,768,1023,74.02,'Intermediate' from dual union all
select 37281,'037281','1993-036BHV','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',100.36,752,800,74.03,'Intermediate' from dual union all
select 37282,'037282','1993-036BHW','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',100.06,690,833,73.9,'Intermediate' from dual union all
select 37283,'037283','1993-036BHX','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',98.84,636,771,74,'Intermediate' from dual union all
select 37284,'037284','1993-036BHY','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',103.27,760,1067,74.03,'Intermediate' from dual union all
select 37285,'037285','1993-036BHZ','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',100.49,761,802,74.03,'Intermediate' from dual union all
select 37286,'037286','1993-036BJA','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',98.64,594,793,74.03,'Intermediate' from dual union all
select 37287,'037287','1993-036BJB','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',97.08,472,766,74.06,'Intermediate' from dual union all
select 37288,'037288','1993-036BJC','deb Kosmos-2251','',65138,'Reentered','2012-04-12 00:00:00','2011-01-03 00:00:00',97.28,513,743,74.06,'Intermediate' from dual union all
select 37289,'037289','1993-036BJD','deb Kosmos-2251','',65138,'Reentered','2015-07-09 00:00:00','2011-01-04 00:00:00',100.07,757,767,74.03,'Intermediate' from dual union all
select 3729,'003729','1965-082RL','deb Transtage 4','',11832,'Reentered','1969-06-26 00:00:00','1969-02-14 00:00:00',99.68,622,865,33.4,'Intermediate' from dual union all
select 37290,'037290','1993-036BJE','deb Kosmos-2251','',65138,'Reentered','2013-12-04 00:00:00','2011-01-04 00:00:00',100.47,450,1111,74.04,'Intermediate' from dual union all
select 37291,'037291','1993-036BJF','deb Kosmos-2251','',65138,'Reentered','2017-05-22 00:00:00','2011-01-04 00:00:00',99.5,617,853,74.02,'Intermediate' from dual union all
select 37292,'037292','1993-036BJG','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',104.04,759,1140,74.13,'Intermediate' from dual union all
select 37293,'037293','1993-036BJH','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',101.15,746,881,73.93,'Intermediate' from dual union all
select 37294,'037294','1993-036BJJ','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',100.04,722,798,74.08,'Intermediate' from dual union all
select 37295,'037295','1993-036BJK','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',99.47,690,777,74.01,'Intermediate' from dual union all
select 37296,'037296','1993-036BJL','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',101.09,743,878,73.84,'Intermediate' from dual union all
select 37297,'037297','1993-036BJM','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',99.86,733,771,74.03,'Intermediate' from dual union all
select 37298,'037298','1993-036BJN','deb Kosmos-2251','',65138,'Reentered','2011-10-26 00:00:00','2011-01-04 00:00:00',98.16,606,736,73.94,'Intermediate' from dual union all
select 37299,'037299','1993-036BJP','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',100.16,699,834,73.87,'Intermediate' from dual union all
select 373,'000373','62 A Pi 3','BOZ','BOZ',8151,'Reentered','1962-08-31 00:00:00','1962-08-27 00:00:00',90.66,137,477,65.56,'Intermediate' from dual union all
select 3730,'003730','1965-082RM','deb Transtage 4','',11832,'Reentered','1977-12-21 00:00:00','1967-02-13 00:00:00',107.83,738,1514,32.49,'Intermediate' from dual union all
select 37300,'037300','1993-036BJQ','deb Kosmos-2251','',65138,'Reentered','2013-08-14 00:00:00','2011-01-04 00:00:00',98.31,587,769,74.04,'Intermediate' from dual union all
select 37301,'037301','1993-036BJR','deb Kosmos-2251','',65138,'Reentered','2015-02-15 00:00:00','2011-01-03 00:00:00',99.8,692,806,74.01,'Intermediate' from dual union all
select 37302,'037302','1993-036BJS','deb Kosmos-2251','',65138,'Reentered','2015-07-13 00:00:00','2011-01-04 00:00:00',100.03,737,783,74.02,'Intermediate' from dual union all
select 37303,'037303','1993-036BJT','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',99.66,702,783,74.02,'Intermediate' from dual union all
select 37304,'037304','1993-036BJU','deb Kosmos-2251','',65138,'Reentered','2014-07-17 00:00:00','2011-01-04 00:00:00',98.36,588,773,73.96,'Intermediate' from dual union all
select 37305,'037305','1993-036BJV','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',101.14,772,854,73.86,'Intermediate' from dual union all
select 37306,'037306','1993-036BJW','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',100.13,677,852,74.07,'Intermediate' from dual union all
select 37307,'037307','1993-036BJX','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',99.38,689,769,74.04,'Intermediate' from dual union all
select 37308,'037308','1993-036BJY','deb Kosmos-2251','',65138,'Reentered','2013-07-12 00:00:00','2011-01-04 00:00:00',98.59,603,780,73.97,'Intermediate' from dual union all
select 37309,'037309','1993-036BJZ','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',98.89,598,814,74.05,'Intermediate' from dual union all
select 3731,'003731','1965-082RN','deb Transtage 4','',11832,'Reentered','1971-08-16 00:00:00','1969-02-14 00:00:00',98.16,646,696,31.79,'Intermediate' from dual union all
select 37310,'037310','1993-036BKA','deb Kosmos-2251','',65138,'Reentered','2015-10-22 00:00:00','2011-01-03 00:00:00',99.62,660,821,74.08,'Intermediate' from dual union all
select 37311,'037311','1993-036BKB','deb Kosmos-2251','',65138,'Reentered','2012-07-29 00:00:00','2011-01-04 00:00:00',97.54,537,744,74,'Intermediate' from dual union all
select 37312,'037312','1993-036BKC','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',100.35,760,790,74.03,'Intermediate' from dual union all
select 37313,'037313','1993-036BKD','deb Kosmos-2251','',65138,'Reentered','2015-03-22 00:00:00','2011-01-03 00:00:00',100.33,758,791,74.03,'Intermediate' from dual union all
select 37314,'037314','1993-036BKE','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',98.85,608,799,73.94,'Intermediate' from dual union all
select 37315,'037315','1993-036BKF','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',100.36,760,791,74.03,'Intermediate' from dual union all
select 37316,'037316','1993-036BKG','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',100.3,708,838,74.04,'Intermediate' from dual union all
select 37317,'037317','1993-036BKH','deb Kosmos-2251','',65138,'Reentered','2014-01-16 00:00:00','2011-01-03 00:00:00',98.55,615,764,74,'Intermediate' from dual union all
select 37318,'037318','1993-036BKJ','deb Kosmos-2251','',65138,'Reentered','2013-01-22 00:00:00','2011-01-03 00:00:00',97.38,524,743,74.04,'Intermediate' from dual union all
select 37319,'037319','1993-036BKK','deb Kosmos-2251','',65138,'Reentered','2011-12-09 00:00:00','2011-01-03 00:00:00',96.1,437,706,74.08,'Intermediate' from dual union all
select 3732,'003732','1965-082RP','deb Transtage 4','',11832,'Reentered','1978-06-07 00:00:00','1969-02-13 00:00:00',106.66,734,1410,32.62,'Intermediate' from dual union all
select 37320,'037320','1993-036BKL','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',99.3,671,780,74.06,'Intermediate' from dual union all
select 37321,'037321','1993-036BKM','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',99.65,719,765,74.04,'Intermediate' from dual union all
select 37322,'037322','1993-036BKN','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',99.06,644,784,74.07,'Intermediate' from dual union all
select 37323,'037323','1993-036BKP','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',97.88,544,771,73.92,'Intermediate' from dual union all
select 37324,'037324','1993-036BKQ','deb Kosmos-2251','',65138,'Reentered','2011-10-07 00:00:00','2011-01-04 00:00:00',96.14,476,672,73.95,'Intermediate' from dual union all
select 37325,'037325','1993-036BKR','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',99.94,712,800,74.13,'Intermediate' from dual union all
select 37326,'037326','1993-036BKS','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',99.6,687,792,74.02,'Intermediate' from dual union all
select 37327,'037327','1993-036BKT','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',100.26,750,792,74.03,'Intermediate' from dual union all
select 37328,'037328','1993-036BKU','deb Kosmos-2251','',65138,'Reentered','2015-04-18 00:00:00','2011-01-04 00:00:00',99.3,661,790,74.02,'Intermediate' from dual union all
select 37329,'037329','1993-036BKV','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',99.47,688,778,74.03,'Intermediate' from dual union all
select 3733,'003733','1965-082RQ','deb Transtage 4','',11832,'Reentered','1979-09-05 00:00:00','1969-02-14 00:00:00',101.34,687,958,33.34,'Intermediate' from dual union all
select 37330,'037330','1993-036BKW','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',99.37,679,777,74.02,'Intermediate' from dual union all
select 37331,'037331','1993-036BKX','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',101.1,763,858,74.07,'Intermediate' from dual union all
select 37332,'037332','1993-036BKY','deb Kosmos-2251','',65138,'Reentered','2013-11-19 00:00:00','2011-01-03 00:00:00',99.36,703,752,74.02,'Intermediate' from dual union all
select 37333,'037333','1993-036BKZ','deb Kosmos-2251','',65138,'Reentered','2014-05-18 00:00:00','2011-01-03 00:00:00',97.31,502,758,73.92,'Intermediate' from dual union all
select 37334,'037334','1993-036BLA','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',98.57,602,779,74.03,'Intermediate' from dual union all
select 37335,'037335','1993-036BLB','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',101.06,765,852,74.02,'Intermediate' from dual union all
select 37336,'037336','1993-036BLC','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',100.61,765,810,74.04,'Intermediate' from dual union all
select 37337,'037337','1993-036BLD','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',99.76,711,784,74.02,'Intermediate' from dual union all
select 37338,'037338','1993-036BLE','deb Kosmos-2251','',65138,'Reentered','2016-09-17 00:00:00','2011-01-03 00:00:00',98.95,646,771,74,'Intermediate' from dual union all
select 37339,'037339','1993-036BLF','deb Kosmos-2251','',65138,'Reentered','2014-12-12 00:00:00','2011-01-03 00:00:00',99.17,664,774,73.99,'Intermediate' from dual union all
select 3734,'003734','1965-082RR','deb Transtage 4','',11832,'Reentered','1969-12-02 00:00:00','1969-02-14 00:00:00',104.16,661,1249,32.84,'Intermediate' from dual union all
select 37340,'037340','1993-036BLG','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',100.5,768,797,74.02,'Intermediate' from dual union all
select 37341,'037341','1993-036BLH','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',99.13,667,768,74,'Intermediate' from dual union all
select 37342,'037342','1993-036BLJ','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-03 00:00:00',99.76,706,789,73.75,'Intermediate' from dual union all
select 37343,'037343','1993-036BLK','deb Kosmos-2251','',65138,'In Earth orbit','','2011-01-04 00:00:00',101.15,713,914,74.02,'Intermediate' from dual union all
select 37344,'037344','2011-001A','Elektro-L No. 1','Elektro-L',69053,'In Earth orbit','','2011-01-28 00:00:00',1423.51,35285,35795,.44,'Drift GEO' from dual union all
select 37345,'037345','2011-001B','Fregat-SB No. 2001','Fregat-SB    No. 2001',69053,'In Earth orbit','','2011-02-07 00:00:00',1400.75,34485,35699,.47,'Drift GEO' from dual union all
select 37346,'037346','2011-001C','11S772 2SB80 No. 1-2007/1','11S772',69053,'Reentered','2011-03-19 00:00:00','2011-01-20 00:00:00',92.59,177,625,51.34,'Intermediate' from dual union all
select 37347,'037347','2011-001D','Fregat SBB','',69053,'In Earth orbit','','2011-01-27 00:00:00',135.27,299,4403,50.38,'Medium' from dual union all
select 37348,'037348','2011-002A','USA 224','Enhanced CRYSTAL 2109',69054,'In Earth orbit','','2011-01-24 00:00:00',97.46,252,1023,97.9,'Sun-Synch' from dual union all
select 37349,'037349','1998-067CC','EVA debris (cable reel)','',66503,'Reentered','2011-02-26 00:00:00','2011-01-22 00:00:00',91.54,350,350,51.65,'Intermediate' from dual union all
select 3735,'003735','1965-082RS','deb Transtage 4','',11832,'Reentered','1969-09-23 00:00:00','1966-12-27 00:00:00',107.41,834,1380,32.46,'Intermediate' from dual union all
select 37350,'037350','1998-067CD','EVA debris (antenna cover)','',66503,'Reentered','2011-03-04 00:00:00','2011-01-22 00:00:00',91.56,351,352,51.65,'Intermediate' from dual union all
select 37351,'037351','2011-003A','Kounotori 2 gouki','HTV 2',69055,'Deorbited','2011-03-30 00:00:00','2011-01-24 00:00:00',90.84,308,324,51.65,'Intermediate' from dual union all
select 37352,'037352','1998-067CE','EVA debris','',66503,'Reentered','2011-03-25 00:00:00','2011-01-20 00:00:00',91.59,346,359,51.64,'Intermediate' from dual union all
select 37353,'037353','1998-067CF','EVA debris','',66503,'Reentered','2011-03-09 00:00:00','2011-01-25 00:00:00',91.52,347,352,51.65,'Intermediate' from dual union all
select 37354,'037354','2011-001E','Sep Motor Cover','Sep Motor Cover',69053,'Reentered','2011-03-05 00:00:00','2011-01-27 00:00:00',95.05,189,852,51.36,'Intermediate' from dual union all
select 37355,'037355','2011-001F','Sep Motor Cover','Sep Motor Cover',69053,'Reentered','2011-02-15 00:00:00','2011-01-27 00:00:00',94.5,173,816,51.36,'Intermediate' from dual union all
select 37356,'037356','2011-001G','Sep Motor Cover','Sep Motor Cover',69053,'Reentered','2011-02-15 00:00:00','2011-01-27 00:00:00',94.44,172,811,51.32,'Intermediate' from dual union all
select 37357,'037357','2011-001H','Sep Motor Cover','Sep Motor Cover',69053,'Reentered','2011-03-06 00:00:00','2011-01-27 00:00:00',94.99,190,846,51.33,'Intermediate' from dual union all
select 37358,'037358','1985-021G','deb GEOSAT 1','',58244,'Reentered','2014-02-08 00:00:00','2011-01-26 00:00:00',97.5,594,684,108.23,'Retrograde' from dual;
insert into SATELLITE
select 37359,'037359','2011-004A','Progress M-09M','Progress 7K-TGM No.409',69056,'Deorbited','2011-04-26 00:00:00','2011-01-29 00:00:00',90.12,260,300,51.65,'Intermediate' from dual union all
select 3736,'003736','1965-082RT','deb Transtage 4','',11832,'Reentered','1978-04-05 00:00:00','1969-02-15 00:00:00',100.6,729,845,74.02,'Intermediate' from dual union all
select 37360,'037360','2011-004B','11S510','11S510',69056,'Reentered','2011-01-31 00:00:00','2011-01-29 00:00:00',88.35,176,210,51.63,'Intermediate' from dual union all
select 37361,'037361','2010-062L','Nanosail-D2','NanoSail-D2',68922,'Reentered','2011-09-17 00:00:00','2011-02-22 00:00:00',97.34,615,648,71.97,'Intermediate' from dual union all
select 37362,'037362','2011-005A','Geo-IK-2 No. 11','Musson-2 No. 11',69057,'Reentered','2013-07-15 00:00:00','2011-02-08 00:00:00',98.48,320,1051,99.46,'Sun-Synch' from dual union all
select 37363,'037363','2011-005B','Briz-KM No. 72517','Briz-KM 14S45',69057,'In Earth orbit','','2011-02-08 00:00:00',98.22,356,992,99.46,'Sun-Synch' from dual union all
select 37364,'037364','2011-006A','USA 225','RPP Aquila',69058,'In Earth orbit','','2011-02-06 00:00:00',109.76,1202,1229,90,'Polar' from dual union all
select 37365,'037365','2011-006B','Minotaur Stage 4','Orion 38',69058,'In Earth orbit','','2011-02-06 00:00:00',109.76,1202,1229,90,'Polar' from dual union all
select 37366,'037366','2011-006C','deb USA 225','',69058,'In Earth orbit','','2011-02-06 00:00:00',109.76,1202,1229,90,'Polar' from dual union all
select 37367,'037367','2011-006D','deb USA 225','',69058,'In Earth orbit','','2011-02-06 00:00:00',109.76,1202,1229,90,'Polar' from dual union all
select 37368,'037368','2011-007A','Johannes Kepler','ATV 2',69059,'Deorbited','2011-06-21 00:00:00','2011-02-19 00:00:00',89.88,266,271,51.64,'Intermediate' from dual union all
select 37369,'037369','2011-007B','EPS 544','EPS+ 544',69059,'Deorbited','2011-02-17 00:00:00','2011-02-16 00:00:00',89.53,249,254,51.63,'Intermediate' from dual union all
select 3737,'003737','1967-045D','deb Kosmos-158','',15300,'Reentered','1989-11-16 00:00:00','1969-02-15 00:00:00',100.6,729,845,74.02,'Intermediate' from dual union all
select 37370,'037370','1998-067CG','Ferrozond foot restraint','EVA debris (Yakor)',66503,'Reentered','2011-04-11 00:00:00','2011-02-17 00:00:00',91.52,346,353,51.65,'Intermediate' from dual union all
select 37371,'037371','2011-008A','Discovery (STS-133)','OV-103',69060,'Landed','2011-03-09 00:00:00','2011-02-25 00:00:00',88.89,208,232,51.64,'Intermediate' from dual union all
select 37372,'037372','2011-009A','Glonass-K1 No. 701K','Uragan-K1 No. 11L',69061,'In Earth orbit','','2011-02-28 00:00:00',689.89,19251,19719,64.77,'Medium' from dual union all
select 37373,'037373','2011-009B','Fregat No. 1026','Fregat 14S44 No. 1026',69061,'In Earth orbit','','2011-03-02 00:00:00',676.13,19128,19152,64.77,'Medium' from dual union all
select 37374,'037374','1998-067CH','EVA debris','',66503,'Reentered','2011-03-02 00:00:00','2011-03-02 00:00:00',90.59,299,308,51.65,'Intermediate' from dual union all
select 37375,'037375','2011-010A','X-37B OTV-2','X-37B No. 2',69062,'Landed','2012-06-16 00:00:00','2011-03-13 00:00:00',91.08,315,341,42.86,'Intermediate' from dual union all
select 37376,'037376','2008-019C','deb CZ-3C','',68527,'Reentered','2011-03-09 00:00:00','2011-03-06 00:00:00',97.14,173,1071,17.59,'Equatorial' from dual union all
select 37377,'037377','2011-011A','USA 227','QUASAR 16',69063,'In Earth orbit','','2011-03-20 00:00:00',1436.27,35780,35800,0,'Stationary' from dual union all
select 37378,'037378','2011-011B','Delta 353','DCSS-4 F10',69063,'In Earth orbit','','2011-03-15 00:00:00',747.27,6600,35200,13,'GEO Transfer' from dual union all
select 37379,'037379','2001-055G','Jason impact debris','',67208,'In Earth orbit','','2011-03-18 00:00:00',109.58,1097,1317,66.1,'Intermediate' from dual union all
select 3738,'003738','61 Delta 5','deb Explorer IX','',7263,'Reentered','1982-03-29 00:00:00','1969-07-15 00:00:00',117.45,647,2486,38.7,'Intermediate' from dual union all
select 37380,'037380','2010-062M','Fastrac-OSCAR-70','Fastrac 2',68922,'In Earth orbit','','2011-03-26 00:00:00',97.52,629,651,71.97,'Intermediate' from dual union all
select 37381,'037381','2008-006C','Briz-M No. 88524','Briz-M 88524',68514,'In Earth orbit','','2011-04-03 00:00:00',1513.81,36159,38430,3.76,'Very High' from dual union all
select 37382,'037382','2011-012A','Soyuz TMA-21','Soyuz 11F732A17 No. 231',69064,'Landed','2011-09-16 00:00:00','2011-04-06 00:00:00',89.8,248,282,51.64,'Intermediate' from dual union all
select 37383,'037383','2011-012B','11S510','11S510',69064,'Reentered','2011-04-07 00:00:00','2011-04-06 00:00:00',88.37,180,208,51.62,'Intermediate' from dual union all
select 37384,'037384','2011-013A','Beidou DW 8','diba ke beidou daohang w',69065,'In Earth orbit','','2011-04-17 00:00:00',1435.86,35694,35870,55.29,'Synchronous' from dual union all
select 37385,'037385','2011-013B','CZ-3A Stage 3','CZ-3A Stage 3',69065,'In Earth orbit','','2011-04-16 00:00:00',607.69,154,34621,55.26,'GEO Transfer' from dual union all
select 37386,'037386','2011-014A','USA 229','INTRUDER 9',69066,'In Earth orbit','','2011-04-15 00:00:00',107.51,1016,1207,63.4,'Intermediate' from dual union all
select 37387,'037387','2011-015A','Resourcesat-2','Resourcesat-2',69067,'In Earth orbit','','2011-04-23 00:00:00',101.33,821,823,98.78,'Sun-Synch' from dual union all
select 37388,'037388','2011-015B','Youthsat','IMS-1A',69067,'In Earth orbit','','2011-04-22 00:00:00',101.12,803,821,98.77,'Sun-Synch' from dual union all
select 37389,'037389','2011-015C','X-Sat','',69067,'In Earth orbit','','2011-04-22 00:00:00',101.14,803,823,98.77,'Sun-Synch' from dual union all
select 3739,'003739','61 Nu 2','Explorer XI meteoroid shield','',7271,'Reentered','1989-03-03 00:00:00','1969-02-15 00:00:00',106.68,484,1662,28.8,'Intermediate' from dual union all
select 37390,'037390','2011-015D','PSLV-C16 PS4','PS4',69067,'In Earth orbit','','2011-04-22 00:00:00',100.91,789,815,98.66,'Sun-Synch' from dual union all
select 37391,'037391','2011-014B','USA 229 P/L 2','INTRUDER 10',69066,'In Earth orbit','','2011-04-15 00:00:00',107.5,1017,1205,63.45,'Intermediate' from dual union all
select 37392,'037392','2011-016A','Intelsat New Dawn','Intelsat New Dawn',69068,'In Earth orbit','','2011-12-04 00:00:00',1436.06,35777,35795,.06,'Stationary' from dual union all
select 37393,'037393','2011-016B','Yahsat 1A','Yahsat 1',69068,'In Earth orbit','','2011-05-05 00:00:00',1482.1,35771,37594,.04,'Drift GEO' from dual union all
select 37394,'037394','2011-016C','ESC-A 558','Ariane 5 ESC-A',69068,'In Earth orbit','','2011-04-24 00:00:00',630.55,245,35715,5.7,'GEO Transfer' from dual union all
select 37395,'037395','2011-016D','Sylda 5A','Sylda 5 No. 44A',69068,'Reentered','2018-02-27 00:00:00','2011-04-25 00:00:00',631.95,253,35779,6,'GEO Transfer' from dual union all
select 37396,'037396','2011-017A','Progress M-10M','Progress 7K-TGM No.410',69069,'Deorbited','2011-10-29 00:00:00','2011-04-28 00:00:00',89.42,231,261,51.64,'Intermediate' from dual union all
select 37397,'037397','2011-017B','11S510','11S510',69069,'Reentered','2011-04-29 00:00:00','2011-04-28 00:00:00',88.23,177,197,51.6,'Intermediate' from dual union all
select 37398,'037398','2011-018A','Meridian No. 14L','Meridian No. 14L',69070,'In Earth orbit','','2011-05-07 00:00:00',724.87,971,39733,62.8,'Molniya' from dual union all
select 37399,'037399','2011-018B','Fregat No. 1028','Fregat 14S44 No. 1028',69070,'In Earth orbit','','2011-05-06 00:00:00',718.67,835,39564,62.81,'Molniya' from dual union all
select 374,'000374','62 A Rho 1','Mariner 2','Mariner R-2',8153,'Deep Space','1962-08-29 00:00:00','',null,null,null,null,'' from dual union all
select 3740,'003740','61 Omicron 218','deb r/b Ablestar','',7273,'Reentered','1974-12-25 00:00:00','2000-10-20 00:00:00',102.45,812,938,66.34,'Intermediate' from dual union all
select 37400,'037400','1999-025EFS','deb FY-1C','',66706,'In Earth orbit','','2011-04-30 00:00:00',106.01,731,1353,98.89,'Sun-Synch' from dual union all
select 37401,'037401','1999-025EFT','deb FY-1C','',66706,'In Earth orbit','','2011-04-12 00:00:00',101.07,674,945,98.73,'Sun-Synch' from dual union all
select 37402,'037402','1999-025EFU','deb FY-1C','',66706,'In Earth orbit','','2011-04-22 00:00:00',101.09,780,841,99.37,'Sun-Synch' from dual union all
select 37403,'037403','1999-025EFV','deb FY-1C','',66706,'In Earth orbit','','2011-05-04 00:00:00',100.72,727,859,98.95,'Sun-Synch' from dual union all
select 37404,'037404','1999-025EFW','deb FY-1C','',66706,'In Earth orbit','','2011-05-05 00:00:00',101.43,790,863,98.89,'Sun-Synch' from dual union all
select 37405,'037405','1999-025EFX','deb FY-1C','',66706,'In Earth orbit','','2011-05-01 00:00:00',100.56,696,874,98.63,'Sun-Synch' from dual union all
select 37406,'037406','1999-025EFY','deb FY-1C','',66706,'In Earth orbit','','2011-05-04 00:00:00',103.47,798,1048,98.45,'Sun-Synch' from dual union all
select 37407,'037407','1999-025EFZ','deb FY-1C','',66706,'Reentered','2014-02-27 00:00:00','2011-05-01 00:00:00',99.39,580,879,97.53,'Sun-Synch' from dual union all
select 37408,'037408','1999-025EGA','deb FY-1C','',66706,'In Earth orbit','','2011-05-06 00:00:00',102.1,781,936,98.65,'Sun-Synch' from dual union all
select 37409,'037409','1999-025EGB','deb FY-1C','',66706,'In Earth orbit','','2011-05-02 00:00:00',103.32,808,1024,97.79,'Sun-Synch' from dual union all
select 3741,'003741','1963-047L','Centaur AC-2 part','Insulation blanket part',9278,'In Earth orbit','','1969-02-15 00:00:00',108.56,657,1663,29.93,'Intermediate' from dual union all
select 37410,'037410','1999-025EGC','deb FY-1C','',66706,'In Earth orbit','','2011-05-02 00:00:00',104.22,849,1067,98.12,'Sun-Synch' from dual union all
select 37411,'037411','1999-025EGD','deb FY-1C','',66706,'In Earth orbit','','2011-05-05 00:00:00',104.08,732,1171,97.81,'Sun-Synch' from dual union all
select 37412,'037412','1999-025EGE','deb FY-1C','',66706,'In Earth orbit','','2011-05-05 00:00:00',101.96,834,869,99.04,'Sun-Synch' from dual union all
select 37413,'037413','1999-025EGF','deb FY-1C','',66706,'In Earth orbit','','2011-05-03 00:00:00',102.21,793,933,98.64,'Sun-Synch' from dual union all
select 37414,'037414','1999-025EGG','deb FY-1C','',66706,'In Earth orbit','','2011-05-02 00:00:00',103.95,770,1121,98.05,'Sun-Synch' from dual union all
select 37415,'037415','1999-025EGH','deb FY-1C','',66706,'In Earth orbit','','2011-05-04 00:00:00',106.71,856,1293,98.91,'Sun-Synch' from dual union all
select 37416,'037416','1999-025EGJ','deb FY-1C','',66706,'In Earth orbit','','2011-04-29 00:00:00',102.02,810,898,99.24,'Sun-Synch' from dual union all
select 37417,'037417','1999-025EGK','deb FY-1C','',66706,'In Earth orbit','','2011-04-26 00:00:00',102.6,821,943,99.14,'Sun-Synch' from dual union all
select 37418,'037418','1999-025EGL','deb FY-1C','',66706,'In Earth orbit','','2011-05-01 00:00:00',121.34,853,2629,98.61,'Medium' from dual union all
select 37419,'037419','1999-025EGM','deb FY-1C','',66706,'In Earth orbit','','2011-04-25 00:00:00',106.57,662,1474,98.59,'Sun-Synch' from dual union all
select 3742,'003742','1967-011P','deb Diademe 1','',15266,'Reentered','1980-08-27 00:00:00','1967-07-26 00:00:00',104.2,570,1344,39.97,'Intermediate' from dual union all
select 37420,'037420','1999-025EGN','deb FY-1C','',66706,'In Earth orbit','','2011-04-28 00:00:00',103.2,857,963,99.28,'Sun-Synch' from dual union all
select 37421,'037421','1999-025EGP','deb FY-1C','',66706,'In Earth orbit','','2011-05-05 00:00:00',107.66,806,1431,98.85,'Sun-Synch' from dual union all
select 37422,'037422','1999-025EGQ','deb FY-1C','',66706,'In Earth orbit','','2011-05-01 00:00:00',102.36,840,901,99.06,'Sun-Synch' from dual union all
select 37423,'037423','1999-025EGR','deb FY-1C','',66706,'In Earth orbit','','2011-04-23 00:00:00',102.96,854,944,99.27,'Sun-Synch' from dual union all
select 37424,'037424','1999-025EGS','deb FY-1C','',66706,'In Earth orbit','','2011-04-29 00:00:00',102.64,701,1067,97.2,'Sun-Synch' from dual union all
select 37425,'037425','1999-025EGT','deb FY-1C','',66706,'In Earth orbit','','2011-04-28 00:00:00',104.63,845,1110,97.16,'Sun-Synch' from dual union all
select 37426,'037426','1999-025EGU','deb FY-1C','',66706,'In Earth orbit','','2011-04-18 00:00:00',100.95,728,880,96.64,'Sun-Synch' from dual union all
select 37427,'037427','1999-025EGV','deb FY-1C','',66706,'In Earth orbit','','2011-05-06 00:00:00',98.63,474,913,98.58,'Sun-Synch' from dual union all
select 37428,'037428','1999-025EGW','deb FY-1C','',66706,'In Earth orbit','','2011-04-23 00:00:00',102.61,821,943,97.76,'Sun-Synch' from dual union all
select 37429,'037429','1999-025EGX','deb FY-1C','',66706,'In Earth orbit','','2011-05-03 00:00:00',102.57,801,960,99.19,'Sun-Synch' from dual union all
select 3743,'003743','1965-020ES','deb Blok Kosmos','',11770,'In Earth orbit','','1992-02-02 00:00:00',118.09,1381,1810,56.7,'Intermediate' from dual union all
select 37430,'037430','1999-025EGY','deb FY-1C','',66706,'In Earth orbit','','2011-05-03 00:00:00',102.36,847,893,99.02,'Sun-Synch' from dual union all
select 37431,'037431','1999-025EGZ','deb FY-1C','',66706,'Reentered','2016-05-13 00:00:00','2011-05-02 00:00:00',100.11,691,837,99.76,'Sun-Synch' from dual union all
select 37432,'037432','1999-025EHA','deb FY-1C','',66706,'In Earth orbit','','2011-05-04 00:00:00',100.57,613,959,98.71,'Sun-Synch' from dual union all
select 37433,'037433','1999-025EHB','deb FY-1C','',66706,'In Earth orbit','','2011-04-30 00:00:00',105.05,771,1222,98.13,'Sun-Synch' from dual union all
select 37434,'037434','1999-025EHC','deb FY-1C','',66706,'In Earth orbit','','2011-05-02 00:00:00',100.82,731,864,99.13,'Sun-Synch' from dual union all
select 37435,'037435','1999-025EHD','deb FY-1C','',66706,'In Earth orbit','','2011-04-20 00:00:00',102.32,831,906,99.07,'Sun-Synch' from dual union all
select 37436,'037436','1999-025EHE','deb FY-1C','',66706,'In Earth orbit','','2011-04-26 00:00:00',102.91,856,937,99.01,'Sun-Synch' from dual union all
select 37437,'037437','1999-025EHF','deb FY-1C','',66706,'In Earth orbit','','2011-05-05 00:00:00',102.02,764,945,99.03,'Sun-Synch' from dual union all
select 37438,'037438','1999-025EHG','deb FY-1C','',66706,'In Earth orbit','','2011-04-28 00:00:00',102.39,848,895,98.99,'Sun-Synch' from dual union all
select 37439,'037439','1999-025EHH','deb FY-1C','',66706,'In Earth orbit','','2011-05-05 00:00:00',103.11,849,963,99.07,'Sun-Synch' from dual union all
select 3744,'003744','61 Omicron 219','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-02 00:00:00',105.84,898,1170,66.98,'Intermediate' from dual union all
select 37440,'037440','1999-025EHJ','deb FY-1C','',66706,'In Earth orbit','','2011-05-04 00:00:00',102.02,796,913,99.06,'Sun-Synch' from dual union all
select 37441,'037441','1999-025EHK','deb FY-1C','',66706,'In Earth orbit','','2011-05-01 00:00:00',110.23,830,1645,99.18,'Sun-Synch' from dual union all
select 37442,'037442','1999-025EHL','deb FY-1C','',66706,'In Earth orbit','','2011-04-29 00:00:00',100.9,758,845,99.14,'Sun-Synch' from dual union all
select 37443,'037443','1999-025EHM','deb FY-1C','',66706,'In Earth orbit','','2011-04-22 00:00:00',103.65,810,1053,98.44,'Sun-Synch' from dual union all
select 37444,'037444','1999-025EHN','deb FY-1C','',66706,'In Earth orbit','','2011-04-27 00:00:00',101.06,578,1039,99.06,'Sun-Synch' from dual union all
select 37445,'037445','1999-025EHP','deb FY-1C','',66706,'In Earth orbit','','2011-05-06 00:00:00',101.68,699,978,99.24,'Sun-Synch' from dual union all
select 37446,'037446','1999-025EHQ','deb FY-1C','',66706,'Reentered','2014-03-25 00:00:00','2011-04-29 00:00:00',98.87,587,823,98.97,'Sun-Synch' from dual union all
select 37447,'037447','1999-025EHR','deb FY-1C','',66706,'In Earth orbit','','2011-04-22 00:00:00',102.4,838,907,98.93,'Sun-Synch' from dual union all
select 37448,'037448','1999-025EHS','deb FY-1C','',66706,'In Earth orbit','','2011-04-19 00:00:00',105.2,735,1273,99.55,'Sun-Synch' from dual;
insert into SATELLITE
select 37449,'037449','1999-025EHT','deb FY-1C','',66706,'In Earth orbit','','2011-05-02 00:00:00',98.73,542,854,98.83,'Sun-Synch' from dual union all
select 3745,'003745','1965-020ET','deb Blok Kosmos                   ','[UNID]',11770,'In Earth orbit','','1992-01-11 00:00:00',115.27,1344,1591,56.02,'Intermediate' from dual union all
select 37450,'037450','1999-025EHU','deb FY-1C','',66706,'In Earth orbit','','2011-04-30 00:00:00',105.7,756,1299,97.69,'Sun-Synch' from dual union all
select 37451,'037451','1999-025EHV','deb FY-1C','',66706,'Reentered','2015-01-16 00:00:00','2011-04-30 00:00:00',99.73,660,832,98.86,'Sun-Synch' from dual union all
select 37452,'037452','1999-025EHW','deb FY-1C','',66706,'In Earth orbit','','2011-04-24 00:00:00',100.95,755,853,99.03,'Sun-Synch' from dual union all
select 37453,'037453','1999-025EHX','deb FY-1C','',66706,'In Earth orbit','','2011-05-05 00:00:00',107.36,771,1438,97.65,'Sun-Synch' from dual union all
select 37454,'037454','1999-025EHY','deb FY-1C','',66706,'In Earth orbit','','2011-05-04 00:00:00',100.15,688,844,98.98,'Sun-Synch' from dual union all
select 37455,'037455','1999-025EHZ','deb FY-1C','',66706,'In Earth orbit','','2011-05-01 00:00:00',99.9,547,961,99.45,'Sun-Synch' from dual union all
select 37456,'037456','1999-025EJA','deb FY-1C','',66706,'In Earth orbit','','2011-05-01 00:00:00',99.53,624,848,98.87,'Sun-Synch' from dual union all
select 37457,'037457','1999-025EJB','deb FY-1C','',66706,'In Earth orbit','','2011-04-28 00:00:00',103.2,850,969,99,'Sun-Synch' from dual union all
select 37458,'037458','1999-025EJC','deb FY-1C','',66706,'Reentered','2014-05-02 00:00:00','2011-05-02 00:00:00',97.63,478,812,98.71,'Sun-Synch' from dual union all
select 37459,'037459','1999-025EJD','deb FY-1C','',66706,'Reentered','2015-06-21 00:00:00','2011-04-26 00:00:00',99.73,660,832,98.88,'Sun-Synch' from dual union all
select 3746,'003746','1965-082RU','deb Transtage 4','',11832,'Reentered','1980-11-29 00:00:00','1969-02-16 00:00:00',107.57,559,1670,30.59,'Intermediate' from dual union all
select 37460,'037460','1999-025EJE','deb FY-1C','',66706,'In Earth orbit','','2011-05-04 00:00:00',101.88,781,915,98.56,'Sun-Synch' from dual union all
select 37461,'037461','1999-025EJF','deb FY-1C','',66706,'In Earth orbit','','2011-05-04 00:00:00',113.67,848,1942,99.87,'Sun-Synch' from dual union all
select 37462,'037462','1999-025EJG','deb FY-1C','',66706,'In Earth orbit','','2011-04-29 00:00:00',99.36,514,942,98.83,'Sun-Synch' from dual union all
select 37463,'037463','1999-025EJH','deb FY-1C','',66706,'In Earth orbit','','2011-05-05 00:00:00',100.37,666,887,98.83,'Sun-Synch' from dual union all
select 37464,'037464','1999-025EJJ','deb FY-1C','',66706,'In Earth orbit','','2011-05-05 00:00:00',108.64,765,1563,102.72,'Sun-Synch' from dual union all
select 37465,'037465','1999-025EJK','deb FY-1C','',66706,'In Earth orbit','','2011-04-27 00:00:00',100.93,755,850,99.34,'Sun-Synch' from dual union all
select 37466,'037466','1999-025EJL','deb FY-1C','',66706,'In Earth orbit','','2011-04-29 00:00:00',101.19,687,943,99.12,'Sun-Synch' from dual union all
select 37467,'037467','1999-025EJM','deb FY-1C','',66706,'In Earth orbit','','2011-04-23 00:00:00',101.12,767,857,99.46,'Sun-Synch' from dual union all
select 37468,'037468','1999-025EJN','deb FY-1C','',66706,'In Earth orbit','','2011-04-30 00:00:00',103.46,845,1000,98.11,'Sun-Synch' from dual union all
select 37469,'037469','1999-025EJP','deb FY-1C','',66706,'In Earth orbit','','2011-04-26 00:00:00',100.03,675,844,98.94,'Sun-Synch' from dual union all
select 3747,'003747','1964-074B','Explorer 23 antenna','',10360,'Reentered','1973-06-13 00:00:00','1969-02-17 00:00:00',97.74,494,808,51.69,'Intermediate' from dual union all
select 37470,'037470','1999-025EJQ','deb FY-1C','',66706,'In Earth orbit','','2011-05-05 00:00:00',101.73,818,864,98.92,'Sun-Synch' from dual union all
select 37471,'037471','1999-025EJR','deb FY-1C','',66706,'In Earth orbit','','2011-04-20 00:00:00',99.56,547,929,98.71,'Sun-Synch' from dual union all
select 37472,'037472','1999-025EJS','deb FY-1C','',66706,'Reentered','2015-06-10 00:00:00','2011-05-05 00:00:00',100.1,687,840,98.98,'Sun-Synch' from dual union all
select 37473,'037473','1999-025EJT','deb FY-1C','',66706,'In Earth orbit','','2011-05-01 00:00:00',102.03,509,1201,99.22,'Sun-Synch' from dual union all
select 37474,'037474','1999-025EJU','deb FY-1C','',66706,'In Earth orbit','','2011-05-01 00:00:00',100.67,614,967,99.11,'Sun-Synch' from dual union all
select 37475,'037475','1999-025EJV','deb FY-1C','',66706,'In Earth orbit','','2011-05-02 00:00:00',102.06,762,951,99.07,'Sun-Synch' from dual union all
select 37476,'037476','1999-025EJW','deb FY-1C','',66706,'In Earth orbit','','2011-05-02 00:00:00',121.94,857,2678,98.99,'Medium' from dual union all
select 37477,'037477','1999-025EJX','deb FY-1C','',66706,'In Earth orbit','','2011-04-24 00:00:00',102.21,838,889,99.06,'Sun-Synch' from dual union all
select 37478,'037478','1999-025EJY','deb FY-1C','',66706,'In Earth orbit','','2011-05-01 00:00:00',111.66,857,1748,99.62,'Sun-Synch' from dual union all
select 37479,'037479','1999-025EJZ','deb FY-1C','',66706,'In Earth orbit','','2011-04-23 00:00:00',102.39,846,898,99.14,'Sun-Synch' from dual union all
select 3748,'003748','61 Omicron 220','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-02 00:00:00',112.96,946,1779,67.74,'Intermediate' from dual union all
select 37480,'037480','1999-025EKA','deb FY-1C','',66706,'In Earth orbit','','2011-05-02 00:00:00',98.29,514,839,99.1,'Sun-Synch' from dual union all
select 37481,'037481','2011-019A','SBIRS GEO-1','SBIRS GEO SV1',69071,'In Earth orbit','','2011-08-05 00:00:00',1436.11,35752,35822,6.39,'Inclined GEO' from dual union all
select 37482,'037482','2011-019B','Centaur AV-022','Centaur AV-022',69071,'Reentered','2011-08-17 00:00:00','2011-05-09 00:00:00',628.42,107,35743,21.2,'GEO Transfer' from dual union all
select 37483,'037483','1993-036BLL','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-10 00:00:00',100.16,694,839,73.98,'Intermediate' from dual union all
select 37484,'037484','1993-036BLM','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-04 00:00:00',100.98,769,842,73.83,'Intermediate' from dual union all
select 37485,'037485','1993-036BLN','deb Kosmos-2251','',65138,'Reentered','2012-08-28 00:00:00','2011-05-07 00:00:00',94.69,328,680,73.96,'Intermediate' from dual union all
select 37486,'037486','1993-036BLP','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-08 00:00:00',104.25,734,1185,73.72,'Intermediate' from dual union all
select 37487,'037487','1993-036BLQ','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-11 00:00:00',101.19,770,860,73.95,'Intermediate' from dual union all
select 37488,'037488','1993-036BLR','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-10 00:00:00',99.65,645,839,74,'Intermediate' from dual union all
select 37489,'037489','1993-036BLS','deb Kosmos-2251','',65138,'Reentered','2014-05-01 00:00:00','2011-04-26 00:00:00',96.62,446,747,73.88,'Intermediate' from dual union all
select 3749,'003749','1965-020EU','deb Blok Kosmos','',11770,'In Earth orbit','','1992-02-02 00:00:00',107.41,613,1601,56.13,'Intermediate' from dual union all
select 37490,'037490','1993-036BLT','deb Kosmos-2251','',65138,'Reentered','2013-03-13 00:00:00','2011-05-11 00:00:00',96.95,476,750,74.02,'Intermediate' from dual union all
select 37491,'037491','1993-036BLU','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-09 00:00:00',105.5,769,1267,73.91,'Intermediate' from dual union all
select 37492,'037492','1993-036BLV','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-09 00:00:00',99.21,673,769,74.01,'Intermediate' from dual union all
select 37493,'037493','1993-036BLW','deb Kosmos-2251','',65138,'Reentered','2016-02-04 00:00:00','2011-05-08 00:00:00',100.37,755,798,74.03,'Intermediate' from dual union all
select 37494,'037494','1993-036BLX','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-02 00:00:00',100.3,681,865,74.01,'Intermediate' from dual union all
select 37495,'037495','1993-036BLY','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-01 00:00:00',98.84,629,777,74.01,'Intermediate' from dual union all
select 37496,'037496','1993-036BLZ','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-11 00:00:00',100.29,645,900,73.98,'Intermediate' from dual union all
select 37497,'037497','1993-036BMA','deb Kosmos-2251','',65138,'Reentered','2011-12-17 00:00:00','2011-04-23 00:00:00',96.56,450,738,74.04,'Intermediate' from dual union all
select 37498,'037498','1993-036BMB','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-10 00:00:00',100.45,764,796,74.05,'Intermediate' from dual union all
select 37499,'037499','1993-036BMC','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-07 00:00:00',100.63,767,810,74.04,'Intermediate' from dual union all
select 375,'000375','62 A Rho 2','Agena B 6902','Agena B 6902',8153,'Deep Space','1962-08-29 00:00:00','1962-08-27 00:00:00',88.49,200,200,32,'Intermediate' from dual union all
select 3750,'003750','1963-053J','deb Explorer XIX','',9284,'In Earth orbit','','1964-01-27 00:00:00',115.81,589,2396,78.58,'Intermediate' from dual union all
select 37500,'037500','1993-036BMD','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-11 00:00:00',96.64,423,773,73.97,'Intermediate' from dual union all
select 37501,'037501','1993-036BME','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-09 00:00:00',98.82,636,769,74.01,'Intermediate' from dual union all
select 37502,'037502','1993-036BMF','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-09 00:00:00',101.08,761,859,74.22,'Intermediate' from dual union all
select 37503,'037503','1993-036BMG','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-11 00:00:00',99.85,732,771,74.02,'Intermediate' from dual union all
select 37504,'037504','1993-036BMH','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-03 00:00:00',99.55,672,803,74.09,'Intermediate' from dual union all
select 37505,'037505','1993-036BMJ','deb Kosmos-2251','',65138,'Reentered','2014-05-28 00:00:00','2011-05-09 00:00:00',96.83,447,767,74,'Intermediate' from dual union all
select 37506,'037506','1993-036BMK','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-06 00:00:00',100.58,757,815,74.07,'Intermediate' from dual union all
select 37507,'037507','1993-036BML','deb Kosmos-2251','',65138,'Reentered','2014-02-15 00:00:00','2011-05-07 00:00:00',97.1,506,734,74.01,'Intermediate' from dual union all
select 37508,'037508','1993-036BMM','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-06 00:00:00',99.76,651,843,73.76,'Intermediate' from dual union all
select 37509,'037509','1993-036BMN','deb Kosmos-2251','',65138,'Reentered','2014-10-03 00:00:00','2011-05-11 00:00:00',96.85,461,755,73.88,'Intermediate' from dual union all
select 3751,'003751','1965-082RV','deb Transtage 4','',11832,'Reentered','1983-01-24 00:00:00','1969-02-16 00:00:00',99.38,671,787,32.78,'Intermediate' from dual union all
select 37510,'037510','1993-036BMP','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-11 00:00:00',99.52,695,777,74.03,'Intermediate' from dual union all
select 37511,'037511','1993-036BMQ','deb Kosmos-2251','',65138,'Reentered','2015-07-24 00:00:00','2011-05-08 00:00:00',99.09,666,765,74.05,'Intermediate' from dual union all
select 37512,'037512','1993-036BMR','deb Kosmos-2251','',65138,'Reentered','2011-10-24 00:00:00','2011-05-09 00:00:00',95.45,451,630,73.98,'Intermediate' from dual union all
select 37513,'037513','1993-036BMS','deb Kosmos-2251','',65138,'Reentered','2013-05-22 00:00:00','2011-04-22 00:00:00',98.01,564,763,73.87,'Intermediate' from dual union all
select 37514,'037514','1993-036BMT','deb Kosmos-2251','',65138,'Reentered','2013-10-08 00:00:00','2011-05-11 00:00:00',97.13,484,759,73.9,'Intermediate' from dual union all
select 37515,'037515','1993-036BMU','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-09 00:00:00',101.08,728,892,74.17,'Intermediate' from dual union all
select 37516,'037516','1993-036BMV','deb Kosmos-2251','',65138,'Reentered','2014-12-21 00:00:00','2011-05-07 00:00:00',98.18,577,767,73.85,'Intermediate' from dual union all
select 37517,'037517','1993-036BMW','deb Kosmos-2251','',65138,'Reentered','2011-12-03 00:00:00','2011-05-09 00:00:00',96.1,460,684,74.03,'Intermediate' from dual union all
select 37518,'037518','1993-036BMX','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-11 00:00:00',100.1,736,791,74.01,'Intermediate' from dual union all
select 37519,'037519','1993-036BMY','deb Kosmos-2251','',65138,'Reentered','2014-10-27 00:00:00','2011-05-10 00:00:00',99.41,601,860,73.98,'Intermediate' from dual union all
select 3752,'003752','61 Omicron 221','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-02 00:00:00',107.6,950,1281,67.64,'Intermediate' from dual union all
select 37520,'037520','1993-036BMZ','deb Kosmos-2251','',65138,'Reentered','2016-09-28 00:00:00','2011-05-11 00:00:00',98.83,644,762,74.04,'Intermediate' from dual union all
select 37521,'037521','1993-036BNA','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-10 00:00:00',98.84,630,777,74.05,'Intermediate' from dual union all
select 37522,'037522','1993-036BNB','deb Kosmos-2251','',65138,'Reentered','2012-08-02 00:00:00','2011-04-23 00:00:00',98.47,635,736,73.88,'Intermediate' from dual union all
select 37523,'037523','1993-036BNC','deb Kosmos-2251','',65138,'Reentered','2015-02-22 00:00:00','2011-05-08 00:00:00',99.03,618,806,73.97,'Intermediate' from dual union all
select 37524,'037524','1993-036BND','deb Kosmos-2251','',65138,'Reentered','2014-07-14 00:00:00','2011-05-01 00:00:00',98.73,640,756,74.06,'Intermediate' from dual union all
select 37525,'037525','1993-036BNE','deb Kosmos-2251','',65138,'Reentered','2012-01-04 00:00:00','2011-05-08 00:00:00',96.35,481,686,74,'Intermediate' from dual union all
select 37526,'037526','1993-036BNF','deb Kosmos-2251','',65138,'Reentered','2013-03-17 00:00:00','2011-05-08 00:00:00',97.04,502,732,73.94,'Intermediate' from dual union all
select 37527,'037527','1993-036BNG','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-11 00:00:00',99.73,712,780,74.02,'Intermediate' from dual union all
select 37528,'037528','1993-036BNH','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-11 00:00:00',100.01,742,776,74.04,'Intermediate' from dual union all
select 37529,'037529','1993-036BNJ','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-09 00:00:00',100.22,723,814,74.03,'Intermediate' from dual union all
select 3753,'003753','1968-091AR','deb Kosmos-249 [*]','',16818,'In Earth orbit','','1992-02-02 00:00:00',104.33,507,1419,62.46,'Intermediate' from dual union all
select 37530,'037530','1993-036BNK','deb Kosmos-2251','',65138,'In Earth orbit','','2011-04-30 00:00:00',100.7,771,813,74.04,'Intermediate' from dual union all
select 37531,'037531','1993-036BNL','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-07 00:00:00',101.06,762,856,74.03,'Intermediate' from dual union all
select 37532,'037532','1993-036BNM','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-08 00:00:00',100.24,758,782,74.05,'Intermediate' from dual union all
select 37533,'037533','1993-036BNN','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-08 00:00:00',103.27,770,1056,74.06,'Intermediate' from dual union all
select 37534,'037534','1993-036BNP','deb Kosmos-2251','',65138,'Reentered','2011-10-27 00:00:00','2011-05-02 00:00:00',93.68,348,562,73.97,'Intermediate' from dual union all
select 37535,'037535','1993-036BNQ','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-10 00:00:00',100.01,733,784,74.03,'Intermediate' from dual union all
select 37536,'037536','1993-036BNR','deb Kosmos-2251','',65138,'Reentered','2012-09-05 00:00:00','2011-05-06 00:00:00',96.33,449,716,74.05,'Intermediate' from dual union all
select 37537,'037537','1993-036BNS','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-07 00:00:00',100.5,743,822,74.01,'Intermediate' from dual union all
select 37538,'037538','1993-036BNT','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-08 00:00:00',99.8,712,786,74.02,'Intermediate' from dual;
insert into SATELLITE
select 37539,'037539','1993-036BNU','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-07 00:00:00',100.42,765,792,74.06,'Intermediate' from dual union all
select 3754,'003754','1968-091AS','deb Kosmos-249','',16818,'Reentered','1975-05-21 00:00:00','1973-08-12 00:00:00',108.57,529,1793,62.32,'Intermediate' from dual union all
select 37540,'037540','1993-036BNV','deb Kosmos-2251','',65138,'Reentered','2012-10-03 00:00:00','2011-05-09 00:00:00',95.28,379,686,73.94,'Intermediate' from dual union all
select 37541,'037541','1993-036BNW','deb Kosmos-2251','',65138,'Reentered','2012-10-10 00:00:00','2011-05-08 00:00:00',97.88,579,735,73.95,'Intermediate' from dual union all
select 37542,'037542','1993-036BNX','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-10 00:00:00',100.98,765,846,74,'Intermediate' from dual union all
select 37543,'037543','1993-036BNY','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-06 00:00:00',98.03,556,773,73.98,'Intermediate' from dual union all
select 37544,'037544','1993-036BNZ','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-09 00:00:00',99.66,703,782,73.99,'Intermediate' from dual union all
select 37545,'037545','1993-036BPA','deb Kosmos-2251','',65138,'Reentered','2014-04-25 00:00:00','2011-04-23 00:00:00',97.99,572,754,73.94,'Intermediate' from dual union all
select 37546,'037546','1993-036BPB','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-09 00:00:00',100.45,767,793,74.04,'Intermediate' from dual union all
select 37547,'037547','1993-036BPC','deb Kosmos-2251','',65138,'In Earth orbit','','2011-05-08 00:00:00',99.71,725,764,74.04,'Intermediate' from dual union all
select 37548,'037548','1997-051XH','deb Iridium 33','',66131,'In Earth orbit','','2011-05-11 00:00:00',100.86,774,825,86.36,'Polar' from dual union all
select 37549,'037549','1997-051XJ','deb Iridium 33','',66131,'In Earth orbit','','2011-05-10 00:00:00',101.3,760,881,86.41,'Polar' from dual union all
select 3755,'003755','1968-091AT','deb Kosmos-249','',16818,'In Earth orbit','','1992-02-02 00:00:00',113.96,766,2051,62.26,'Intermediate' from dual union all
select 37550,'037550','1997-051XK','deb Iridium 33','',66131,'In Earth orbit','','2011-05-11 00:00:00',100.32,773,774,86.39,'Polar' from dual union all
select 37551,'037551','1997-051XL','deb Iridium 33','',66131,'Reentered','2013-10-23 00:00:00','2011-05-09 00:00:00',98.6,639,745,86.43,'Polar' from dual union all
select 37552,'037552','1997-051XM','deb Iridium 33','',66131,'Reentered','2013-11-12 00:00:00','2011-05-09 00:00:00',98.57,630,751,86.43,'Polar' from dual union all
select 37553,'037553','1997-051XN','deb Iridium 33','',66131,'Reentered','2014-05-20 00:00:00','2011-04-27 00:00:00',100.2,763,773,86.38,'Polar' from dual union all
select 37554,'037554','1997-051XP','deb Iridium 33','',66131,'Reentered','2016-07-31 00:00:00','2011-05-04 00:00:00',100.24,752,787,86.39,'Polar' from dual union all
select 37555,'037555','1997-051XQ','deb Iridium 33','',66131,'In Earth orbit','','2011-05-10 00:00:00',101.24,749,885,86.47,'Polar' from dual union all
select 37556,'037556','1997-051XR','deb Iridium 33','',66131,'Reentered','2012-12-06 00:00:00','2011-05-01 00:00:00',99.24,659,786,86.42,'Polar' from dual union all
select 37557,'037557','1997-051XS','deb Iridium 33','',66131,'In Earth orbit','','2011-05-08 00:00:00',102.89,766,1025,86.41,'Polar' from dual union all
select 37558,'037558','1997-051XT','deb Iridium 33','',66131,'In Earth orbit','','2011-05-01 00:00:00',101.05,718,898,86.41,'Polar' from dual union all
select 37559,'037559','1997-051XU','deb Iridium 33','',66131,'Reentered','2014-12-25 00:00:00','2011-05-10 00:00:00',99.79,729,768,86.31,'Polar' from dual union all
select 3756,'003756','1963-014CL','deb West Ford','',9245,'Reentered','1987-10-17 00:00:00','1969-02-20 00:00:00',164.08,1330,5770,85.74,'Medium' from dual union all
select 37560,'037560','1997-051XV','deb Iridium 33','',66131,'In Earth orbit','','2011-05-11 00:00:00',102.3,765,970,86.41,'Polar' from dual union all
select 37561,'037561','1997-051XW','deb Iridium 33','',66131,'Reentered','2012-12-10 00:00:00','2011-05-08 00:00:00',98.83,652,754,86.38,'Polar' from dual union all
select 37562,'037562','1997-051XX','deb Iridium 33','',66131,'In Earth orbit','','2011-05-11 00:00:00',100.61,761,814,86.39,'Polar' from dual union all
select 37563,'037563','1997-051XY','deb Iridium 33','',66131,'Reentered','2012-11-29 00:00:00','2011-05-10 00:00:00',98.14,587,753,86.25,'Polar' from dual union all
select 37564,'037564','1997-051XZ','deb Iridium 33','',66131,'Reentered','2016-07-10 00:00:00','2011-05-07 00:00:00',100.35,765,786,86.39,'Polar' from dual union all
select 37565,'037565','1997-051YA','deb Iridium 33','',66131,'In Earth orbit','','2011-04-20 00:00:00',100.2,757,779,86.36,'Polar' from dual union all
select 37566,'037566','1997-051YB','deb Iridium 33','',66131,'In Earth orbit','','2011-05-10 00:00:00',101.12,747,877,86.46,'Polar' from dual union all
select 37567,'037567','2010-038C','CZ-4C ullage motor cover','',68898,'In Earth orbit','','2011-05-12 00:00:00',99.5,604,866,97.84,'Sun-Synch' from dual union all
select 37568,'037568','2010-038D','CZ-4C ullage motor cover','',68898,'In Earth orbit','','2011-05-12 00:00:00',100.11,612,915,97.86,'Sun-Synch' from dual union all
select 37569,'037569','2010-038E','CZ-4C ullage motor cover','',68898,'In Earth orbit','','2011-05-09 00:00:00',99.72,607,884,97.8,'Sun-Synch' from dual union all
select 3757,'003757','1968-091AU','deb Kosmos-249','',16818,'In Earth orbit','','1992-02-03 00:00:00',107.85,513,1742,62,'Intermediate' from dual union all
select 37570,'037570','2010-038F','CZ-4C ullage motor cover','',68898,'In Earth orbit','','2011-05-09 00:00:00',100.05,611,910,97.81,'Sun-Synch' from dual union all
select 37571,'037571','2008-026C','CZ-4C ullage motor cover','',68534,'In Earth orbit','','2011-05-11 00:00:00',104.88,804,1174,98.79,'Sun-Synch' from dual union all
select 37572,'037572','2008-026D','CZ-4C ullage motor cover','',68534,'In Earth orbit','','2011-05-11 00:00:00',104.82,804,1168,98.72,'Sun-Synch' from dual union all
select 37573,'037573','2008-026E','CZ-4C ullage motor cover','',68534,'In Earth orbit','','2011-05-11 00:00:00',104.49,803,1139,98.71,'Sun-Synch' from dual union all
select 37574,'037574','2008-026F','CZ-4C ullage motor cover','',68534,'In Earth orbit','','2011-05-09 00:00:00',104.96,804,1182,98.77,'Sun-Synch' from dual union all
select 37575,'037575','1998-017C','deb SPOT-4','',66453,'Reentered','2012-11-14 00:00:00','2011-05-13 00:00:00',100.71,781,804,98.88,'Sun-Synch' from dual union all
select 37576,'037576','1998-017D','deb SPOT-4','',66453,'Reentered','2012-06-18 00:00:00','2011-05-13 00:00:00',100.15,747,785,98.91,'Sun-Synch' from dual union all
select 37577,'037577','2011-020A','Endeavour (STS-134)','OV-105',69072,'Landed','2011-06-01 00:00:00','2011-05-17 00:00:00',91.17,321,343,51.64,'Intermediate' from dual union all
select 37578,'037578','1999-025EKB','deb FY-1C','',66706,'In Earth orbit','','2011-05-06 00:00:00',101.4,760,890,98.87,'Sun-Synch' from dual union all
select 37579,'037579','1999-025EKC','deb FY-1C','',66706,'In Earth orbit','','2011-05-07 00:00:00',102.44,844,905,99.3,'Sun-Synch' from dual union all
select 3758,'003758','1968-091AV','deb Kosmos-249 [*]','',16818,'In Earth orbit','','1992-01-26 00:00:00',113.95,765,2051,62.26,'Intermediate' from dual union all
select 37580,'037580','1999-025EKD','deb FY-1C','',66706,'In Earth orbit','','2011-05-17 00:00:00',102.64,788,980,98.99,'Sun-Synch' from dual union all
select 37581,'037581','1999-025EKE','deb FY-1C','',66706,'In Earth orbit','','2011-05-17 00:00:00',101.52,747,915,98.59,'Sun-Synch' from dual union all
select 37582,'037582','1999-057SX','ZY-1 debris','',66738,'In Earth orbit','','2011-05-09 00:00:00',100.09,754,772,98.21,'Sun-Synch' from dual union all
select 37583,'037583','1999-057SY','ZY-1 debris','',66738,'Reentered','2013-02-17 00:00:00','2011-05-05 00:00:00',99.29,707,743,98.25,'Sun-Synch' from dual union all
select 37584,'037584','1974-015H','Burner 2A collision debris','',33618,'In Earth orbit','','2011-05-15 00:00:00',100.16,684,848,99.07,'Sun-Synch' from dual union all
select 37585,'037585','1991-082CR','deb DMSP F-11','',64251,'In Earth orbit','','2011-05-11 00:00:00',101.52,823,839,98.71,'Sun-Synch' from dual union all
select 37586,'037586','2004-025L','deb Dnepr','',67869,'In Earth orbit','','2011-05-18 00:00:00',103.71,678,1190,98.17,'Sun-Synch' from dual union all
select 37587,'037587','1999-025EKF','deb FY-1C','',66706,'In Earth orbit','','2011-05-12 00:00:00',100.18,590,945,97.84,'Sun-Synch' from dual union all
select 37588,'037588','1999-025EKG','deb FY-1C','',66706,'In Earth orbit','','2011-05-17 00:00:00',103.28,841,986,98.11,'Sun-Synch' from dual union all
select 37589,'037589','1999-025EKH','deb FY-1C','',66706,'In Earth orbit','','2011-05-11 00:00:00',104.51,848,1095,99.12,'Sun-Synch' from dual union all
select 3759,'003759','1969-014A','Mariner 6','Mariner 69-3',18281,'Deep Space','1969-03-06 00:00:00','',null,null,null,null,'' from dual union all
select 37590,'037590','1999-025EKJ','deb FY-1C','',66706,'In Earth orbit','','2011-05-05 00:00:00',101.27,685,953,99,'Sun-Synch' from dual union all
select 37591,'037591','1999-025EKK','deb FY-1C','',66706,'In Earth orbit','','2011-05-16 00:00:00',103.47,805,1041,98.44,'Sun-Synch' from dual union all
select 37592,'037592','1999-025EKL','deb FY-1C','',66706,'In Earth orbit','','2011-05-19 00:00:00',100.27,697,846,98.92,'Sun-Synch' from dual union all
select 37593,'037593','1999-025EKM','deb FY-1C','',66706,'In Earth orbit','','2011-05-03 00:00:00',101.8,701,987,99.08,'Sun-Synch' from dual union all
select 37594,'037594','1999-025EKN','deb FY-1C','',66706,'In Earth orbit','','2011-05-19 00:00:00',102.32,802,934,98.96,'Sun-Synch' from dual union all
select 37595,'037595','1999-025EKP','deb FY-1C','',66706,'In Earth orbit','','2011-05-09 00:00:00',102.63,831,935,99.14,'Sun-Synch' from dual union all
select 37596,'037596','1999-025EKQ','deb FY-1C','',66706,'In Earth orbit','','2011-05-19 00:00:00',102.4,848,896,98.97,'Sun-Synch' from dual union all
select 37597,'037597','1999-025EKR','deb FY-1C','',66706,'In Earth orbit','','2011-05-19 00:00:00',103.09,635,1175,98.31,'Sun-Synch' from dual union all
select 37598,'037598','1999-025EKS','deb FY-1C','',66706,'In Earth orbit','','2011-05-16 00:00:00',102.17,849,874,98.91,'Sun-Synch' from dual union all
select 37599,'037599','1999-025EKT','deb FY-1C','',66706,'In Earth orbit','','2011-05-16 00:00:00',99.63,603,879,99.01,'Sun-Synch' from dual union all
select 376,'000376','62 A Pi 6','deb Sputnik','',8151,'Reentered','1962-09-06 00:00:00','1962-08-27 00:00:00',90.6,167,440,65.83,'Intermediate' from dual union all
select 3760,'003760','1969-014B','Centaur AC-20','Centaur 17D',18281,'Deep Space','1969-03-06 00:00:00','',null,null,null,null,'' from dual union all
select 37600,'037600','1999-025EKU','deb FY-1C','',66706,'In Earth orbit','','2011-05-18 00:00:00',107.15,733,1456,98.9,'Sun-Synch' from dual union all
select 37601,'037601','1999-025EKV','deb FY-1C','',66706,'Reentered','2012-07-21 00:00:00','2011-05-01 00:00:00',97.63,482,809,97.69,'Sun-Synch' from dual union all
select 37602,'037602','2011-021A','Estrela do Sul 2','Telstar 14R',69073,'In Earth orbit','','2011-06-16 00:00:00',1436.04,35780,35791,.02,'Stationary' from dual union all
select 37603,'037603','2011-021B','Briz-M No. 99520','Briz-M',69073,'In Earth orbit','','2011-05-23 00:00:00',804.94,8755,35817,13.68,'Medium' from dual union all
select 37604,'037604','2011-021C','Briz-M DTB','Briz-M torus tank',69073,'In Earth orbit','','2011-05-23 00:00:00',373.19,346,21239,49.37,'Highly Ellip' from dual union all
select 37605,'037605','2011-022A','GSAT-8','INSAT-4G',69074,'In Earth orbit','','2011-06-17 00:00:00',1436.09,35780,35793,.12,'Stationary' from dual union all
select 37606,'037606','2011-022B','ST-2','Singapore-Taiwan 2',69074,'In Earth orbit','','2011-05-27 00:00:00',1438.49,35762,35905,.02,'Drift GEO' from dual union all
select 37607,'037607','2011-022C','ESC-A 559','Ariane 5 ESC-A',69074,'In Earth orbit','','2011-05-23 00:00:00',629.02,230,35651,2.21,'GEO Transfer' from dual union all
select 37608,'037608','2011-022D','Sylda 5A','Sylda 5 No. 45A',69074,'Reentered','2013-01-21 00:00:00','2011-05-23 00:00:00',631.45,244,35762,2.48,'GEO Transfer' from dual union all
select 37609,'037609','1990-081CV','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-22 00:00:00',102.41,822,924,99.02,'Sun-Synch' from dual union all
select 3761,'003761','1969-015A','Kosmos-266','Zenit-2 No. 71',18282,'Landed','1969-03-05 00:00:00','1969-02-27 00:00:00',89.87,201,335,72.9,'Intermediate' from dual union all
select 37610,'037610','1990-081CW','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-23 00:00:00',104.74,867,1098,99.12,'Sun-Synch' from dual union all
select 37611,'037611','1990-081CX','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-09 00:00:00',101.13,793,832,98.98,'Sun-Synch' from dual union all
select 37612,'037612','1990-081CY','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-21 00:00:00',100.47,753,809,98.56,'Sun-Synch' from dual union all
select 37613,'037613','1990-081CZ','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-17 00:00:00',104.25,805,1114,98.92,'Sun-Synch' from dual union all
select 37614,'037614','1990-081DA','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-24 00:00:00',102.57,825,935,99.03,'Sun-Synch' from dual union all
select 37615,'037615','1990-081DB','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-24 00:00:00',101.25,806,829,98.77,'Sun-Synch' from dual union all
select 37616,'037616','1990-081DC','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-21 00:00:00',103.59,853,1004,99.37,'Sun-Synch' from dual union all
select 37617,'037617','1990-081DD','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-16 00:00:00',102.59,874,889,99.07,'Sun-Synch' from dual union all
select 37618,'037618','1990-081DE','deb CZ-4 Stage 3','',63323,'Reentered','2014-04-18 00:00:00','2011-05-24 00:00:00',98.44,669,700,98.82,'Sun-Synch' from dual union all
select 37619,'037619','1990-081DF','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-19 00:00:00',102.49,867,886,99.05,'Sun-Synch' from dual union all
select 3762,'003762','1969-015B','11A57I','11A57I',18282,'Reentered','1969-03-04 00:00:00','1969-02-27 00:00:00',89.57,196,311,72.89,'Intermediate' from dual union all
select 37620,'037620','1990-081DG','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-19 00:00:00',101.38,822,826,98.76,'Sun-Synch' from dual union all
select 37621,'037621','1990-081DH','deb CZ-4 Stage 3','',63323,'Reentered','2014-09-28 00:00:00','2011-05-24 00:00:00',98.81,686,718,98.86,'Sun-Synch' from dual union all
select 37622,'037622','1990-081DJ','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-23 00:00:00',100.62,758,818,98.61,'Sun-Synch' from dual union all
select 37623,'037623','1990-081DK','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-11 00:00:00',101.8,826,862,98.92,'Sun-Synch' from dual union all
select 37624,'037624','1990-081DL','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-24 00:00:00',102.37,861,881,99.03,'Sun-Synch' from dual union all
select 37625,'037625','1990-081DM','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-25 00:00:00',100.84,797,800,98.63,'Sun-Synch' from dual union all
select 37626,'037626','1990-081DN','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2011-05-21 00:00:00',105.2,880,1128,98.97,'Sun-Synch' from dual union all
select 37627,'037627','1975-052LJ','deb Delta 111','',36499,'Reentered','2011-05-29 00:00:00','2011-05-27 00:00:00',98.45,404,966,100.69,'Sun-Synch' from dual union all
select 37628,'037628','1975-052LK','deb Delta 111','',36499,'Reentered','2011-07-16 00:00:00','2011-05-27 00:00:00',102.81,693,1091,100.24,'Sun-Synch' from dual;
insert into SATELLITE
select 37629,'037629','1975-052LL','deb Delta 111','',36499,'Reentered','2011-06-18 00:00:00','2011-05-27 00:00:00',102.62,611,1155,100.28,'Sun-Synch' from dual union all
select 3763,'003763','1969-015C','deb Kosmos-266','',18282,'Reentered','1969-02-26 00:00:00','1969-02-25 00:00:00',89.31,198,283,72.8,'Intermediate' from dual union all
select 37630,'037630','1975-052LM','deb Delta 111','',36499,'Reentered','2011-06-09 00:00:00','2011-05-27 00:00:00',101.81,575,1114,100.38,'Sun-Synch' from dual union all
select 37631,'037631','1986-073G','deb NOAA 10','',59266,'Reentered','2011-11-04 00:00:00','2011-05-02 00:00:00',116.51,1500,1548,101.83,'Sun-Synch' from dual union all
select 37632,'037632','1982-102C','deb Kosmos-1417','',54411,'In Earth orbit','','2011-05-20 00:00:00',104.56,952,996,82.96,'Intermediate' from dual union all
select 37633,'037633','2011-023A','Soyuz TMA-02M','Soyuz 11F732A47 No. 702',69075,'Landed','2011-11-22 00:00:00','2011-06-08 00:00:00',89.86,247,288,51.65,'Intermediate' from dual union all
select 37634,'037634','2011-023B','11S510','11S510',69075,'Reentered','2011-06-11 00:00:00','2011-06-10 00:00:00',88.37,179,209,51.65,'Intermediate' from dual union all
select 37635,'037635','1978-026HW','deb Delta 139','',45221,'In Earth orbit','','2011-06-04 00:00:00',101.82,801,889,99.15,'Sun-Synch' from dual union all
select 37636,'037636','1978-026HX','deb Delta 139','',45221,'In Earth orbit','','2011-05-28 00:00:00',102.8,858,924,98.93,'Sun-Synch' from dual union all
select 37637,'037637','1978-026HY','deb Delta 139','',45221,'In Earth orbit','','2011-06-05 00:00:00',103.44,866,977,99.05,'Sun-Synch' from dual union all
select 37638,'037638','1978-026HZ','deb Delta 139','',45221,'In Earth orbit','','2011-05-19 00:00:00',102.49,830,923,99.14,'Sun-Synch' from dual union all
select 37639,'037639','1978-026JA','deb Delta 139','',45221,'In Earth orbit','','2011-06-07 00:00:00',104.2,870,1044,98.77,'Sun-Synch' from dual union all
select 3764,'003764','1969-016A','ESSA 9','TOS G',18283,'In Earth orbit','','1969-04-25 00:00:00',115.22,1425,1506,101.79,'Sun-Synch' from dual union all
select 37640,'037640','1978-026JB','deb Delta 139','',45221,'In Earth orbit','','2011-05-16 00:00:00',101.99,819,887,99,'Sun-Synch' from dual union all
select 37641,'037641','1978-026JC','deb Delta 139','',45221,'In Earth orbit','','2011-06-02 00:00:00',101.25,761,875,98.46,'Sun-Synch' from dual union all
select 37642,'037642','1978-026JD','deb Delta 139','',45221,'In Earth orbit','','2011-06-08 00:00:00',97.95,616,706,98.12,'Sun-Synch' from dual union all
select 37643,'037643','1978-026JE','deb Delta 139','',45221,'In Earth orbit','','2011-05-16 00:00:00',101.14,779,847,98.47,'Sun-Synch' from dual union all
select 37644,'037644','1978-026JF','deb Delta 139','',45221,'In Earth orbit','','2011-06-02 00:00:00',101.92,809,891,99.03,'Sun-Synch' from dual union all
select 37645,'037645','1978-026JG','deb Delta 139','',45221,'In Earth orbit','','2011-05-16 00:00:00',99.2,683,758,98.62,'Sun-Synch' from dual union all
select 37646,'037646','1978-026JH','deb Delta 139','',45221,'In Earth orbit','','2011-06-05 00:00:00',102.83,817,968,98.95,'Sun-Synch' from dual union all
select 37647,'037647','1978-026JJ','deb Delta 139','',45221,'In Earth orbit','','2011-05-28 00:00:00',105.55,839,1202,98.81,'Sun-Synch' from dual union all
select 37648,'037648','1978-026JK','deb Delta 139','',45221,'In Earth orbit','','2011-05-24 00:00:00',100.88,726,875,98.5,'Sun-Synch' from dual union all
select 37649,'037649','1978-026JL','deb Delta 139','',45221,'In Earth orbit','','2011-05-28 00:00:00',103.77,896,978,99.08,'Sun-Synch' from dual union all
select 3765,'003765','1969-017A','Kosmos-267','Zenit-4 No. 54',18284,'Landed','1969-03-06 00:00:00','1969-03-02 00:00:00',89.76,204,322,64.99,'Intermediate' from dual union all
select 37650,'037650','1978-026JM','deb Delta 139','',45221,'Reentered','2016-09-06 00:00:00','2011-06-06 00:00:00',98.68,652,740,99.05,'Sun-Synch' from dual union all
select 37651,'037651','1978-026JN','deb Delta 139','',45221,'In Earth orbit','','2011-06-04 00:00:00',101.53,801,861,99.22,'Sun-Synch' from dual union all
select 37652,'037652','1978-026JP','deb Delta 139','',45221,'In Earth orbit','','2011-06-05 00:00:00',101.88,827,869,98.68,'Sun-Synch' from dual union all
select 37653,'037653','1978-026JQ','deb Delta 139','',45221,'In Earth orbit','','2011-05-21 00:00:00',102.6,857,907,98.7,'Sun-Synch' from dual union all
select 37654,'037654','1978-026JR','deb Delta 139','',45221,'In Earth orbit','','2011-06-03 00:00:00',100.6,705,869,98.51,'Sun-Synch' from dual union all
select 37655,'037655','1978-026JS','deb Delta 139','',45221,'In Earth orbit','','2011-05-23 00:00:00',102.45,872,878,98.65,'Sun-Synch' from dual union all
select 37656,'037656','1978-026JT','deb Delta 139','',45221,'In Earth orbit','','2011-05-19 00:00:00',102.22,849,878,98.61,'Sun-Synch' from dual union all
select 37657,'037657','1978-026JU','deb Delta 139','',45221,'In Earth orbit','','2011-05-23 00:00:00',101.15,794,833,98.44,'Sun-Synch' from dual union all
select 37658,'037658','1978-026JV','deb Delta 139','',45221,'In Earth orbit','','2011-06-07 00:00:00',103.19,860,958,99.15,'Sun-Synch' from dual union all
select 37659,'037659','1978-026JW','deb Delta 139','',45221,'In Earth orbit','','2011-05-17 00:00:00',104.91,875,1105,98.75,'Sun-Synch' from dual union all
select 3766,'003766','1969-017B','11A57I','11A57I',18284,'Reentered','1969-03-05 00:00:00','1969-02-28 00:00:00',89.43,194,298,64.99,'Intermediate' from dual union all
select 37660,'037660','1978-026JX','deb Delta 139','',45221,'In Earth orbit','','2011-05-31 00:00:00',101.12,797,827,98.42,'Sun-Synch' from dual union all
select 37661,'037661','1978-026JY','deb Delta 139','',45221,'In Earth orbit','','2011-06-04 00:00:00',108.2,891,1396,98.92,'Sun-Synch' from dual union all
select 37662,'037662','1978-026JZ','deb Delta 139','',45221,'In Earth orbit','','2011-05-17 00:00:00',102,827,880,98.61,'Sun-Synch' from dual union all
select 37663,'037663','1978-026KA','deb Delta 139','',45221,'In Earth orbit','','2011-05-17 00:00:00',101.67,809,866,98.56,'Sun-Synch' from dual union all
select 37664,'037664','1978-026KB','deb Delta 139','',45221,'In Earth orbit','','2011-05-13 00:00:00',103.76,880,993,98.77,'Sun-Synch' from dual union all
select 37665,'037665','1978-026KC','deb Delta 139','',45221,'In Earth orbit','','2011-05-28 00:00:00',101.42,798,854,99.21,'Sun-Synch' from dual union all
select 37666,'037666','1978-026KD','deb Delta 139','',45221,'In Earth orbit','','2011-05-30 00:00:00',101.81,816,872,98.56,'Sun-Synch' from dual union all
select 37667,'037667','1978-026KE','deb Delta 139','',45221,'In Earth orbit','','2011-06-04 00:00:00',99.55,701,774,98.16,'Sun-Synch' from dual union all
select 37668,'037668','1978-026KF','deb Delta 139','',45221,'In Earth orbit','','2011-06-05 00:00:00',101.97,822,881,98.61,'Sun-Synch' from dual union all
select 37669,'037669','1978-026KG','deb Delta 139','',45221,'In Earth orbit','','2011-06-08 00:00:00',102.56,872,888,98.7,'Sun-Synch' from dual union all
select 3767,'003767','1969-016B','FW4D','FW4D S/N 2322-1',18283,'In Earth orbit','','1969-03-28 00:00:00',115.15,1422,1503,101.8,'Sun-Synch' from dual union all
select 37670,'037670','1978-026KH','deb Delta 139','',45221,'In Earth orbit','','2011-06-06 00:00:00',103.85,889,992,98.92,'Sun-Synch' from dual union all
select 37671,'037671','1978-026KJ','deb Delta 139','',45221,'In Earth orbit','','2011-05-21 00:00:00',101.35,801,845,98.48,'Sun-Synch' from dual union all
select 37672,'037672','1978-026KK','deb Delta 139','',45221,'In Earth orbit','','2011-06-05 00:00:00',102.14,842,877,98.62,'Sun-Synch' from dual union all
select 37673,'037673','2011-024A','SAC-D/Aquarius','SAC-D',69076,'In Earth orbit','','2011-06-17 00:00:00',97.81,653,655,98.01,'Sun-Synch' from dual union all
select 37674,'037674','2011-024B','Delta 354','Delta SSPS AJ-10-118K',69076,'Reentered','2011-07-30 00:00:00','2011-06-17 00:00:00',92.72,182,634,103.56,'Retrograde' from dual union all
select 37675,'037675','2011-025A','Rasad','',69077,'Reentered','2011-07-06 00:00:00','2011-06-22 00:00:00',89.54,233,271,55.67,'Intermediate' from dual union all
select 37676,'037676','2011-025B','Safir Stage 2','',69077,'Reentered','2011-07-01 00:00:00','2011-06-22 00:00:00',89.28,218,260,55.69,'Intermediate' from dual union all
select 37677,'037677','2011-026A','Zhongxing 10','zhongxing shi hao weixin',69078,'In Earth orbit','','2011-06-30 00:00:00',1436.01,35774,35796,.17,'Stationary' from dual union all
select 37678,'037678','2011-026B','CZ-3B Stage 3','CZ-3B Stage 3',69078,'Reentered','2012-03-04 00:00:00','2011-06-24 00:00:00',755.73,169,42041,25.98,'Highly Ellip' from dual union all
select 37679,'037679','2011-027A','Progress M-11M','Progress 7K-TGM No.411',69079,'Deorbited','2011-09-01 00:00:00','2011-06-22 00:00:00',90.58,283,323,51.64,'Intermediate' from dual union all
select 3768,'003768','1968-117Z','deb Blok Kosmos R-12','',16844,'Reentered','1969-03-04 00:00:00','1969-02-26 00:00:00',90.38,246,341,70.97,'Intermediate' from dual union all
select 37680,'037680','2011-027B','11S510','11S510',69079,'Reentered','2011-06-23 00:00:00','2011-06-22 00:00:00',88.34,180,205,51.62,'Intermediate' from dual union all
select 37681,'037681','1999-025EKW','deb FY-1C','',66706,'Reentered','2015-02-07 00:00:00','2011-06-23 00:00:00',99.94,644,868,99.1,'Sun-Synch' from dual union all
select 37682,'037682','1999-025EKX','deb FY-1C','',66706,'In Earth orbit','','2011-06-23 00:00:00',105.99,853,1228,98.59,'Sun-Synch' from dual union all
select 37683,'037683','1999-025EKY','deb FY-1C','',66706,'In Earth orbit','','2011-06-13 00:00:00',109.5,711,1697,97.98,'Sun-Synch' from dual union all
select 37684,'037684','1999-025EKZ','deb FY-1C','',66706,'In Earth orbit','','2011-06-22 00:00:00',105.06,807,1187,98.18,'Sun-Synch' from dual union all
select 37685,'037685','1999-025ELA','deb FY-1C','',66706,'In Earth orbit','','2011-06-20 00:00:00',104.07,775,1127,98.03,'Sun-Synch' from dual union all
select 37686,'037686','1999-025ELB','deb FY-1C','',66706,'In Earth orbit','','2011-06-16 00:00:00',108.11,833,1446,99.02,'Sun-Synch' from dual union all
select 37687,'037687','1999-025ELC','deb FY-1C','',66706,'In Earth orbit','','2011-06-13 00:00:00',106.99,741,1433,98.21,'Sun-Synch' from dual union all
select 37688,'037688','1999-025ELD','deb FY-1C','',66706,'In Earth orbit','','2011-06-18 00:00:00',102.78,852,929,99.04,'Sun-Synch' from dual union all
select 37689,'037689','1999-025ELE','deb FY-1C','',66706,'In Earth orbit','','2011-06-20 00:00:00',102.35,848,892,98.78,'Sun-Synch' from dual union all
select 3769,'003769','1969-018A','Gumdrop','Apollo CM-104',18285,'Landed','1969-03-13 00:00:00','1969-03-05 00:00:00',91.55,204,497,33.83,'Intermediate' from dual union all
select 37690,'037690','1999-025ELF','deb FY-1C','',66706,'In Earth orbit','','2011-06-24 00:00:00',102.67,850,920,99.11,'Sun-Synch' from dual union all
select 37691,'037691','1999-025ELG','deb FY-1C','',66706,'In Earth orbit','','2011-06-10 00:00:00',104.29,549,1374,98.37,'Sun-Synch' from dual union all
select 37692,'037692','1999-025ELH','deb FY-1C','',66706,'In Earth orbit','','2011-06-24 00:00:00',101.86,842,852,98.96,'Sun-Synch' from dual union all
select 37693,'037693','1999-025ELJ','deb FY-1C','',66706,'In Earth orbit','','2011-05-30 00:00:00',102.75,829,949,99.2,'Sun-Synch' from dual union all
select 37694,'037694','1999-025ELK','deb FY-1C','',66706,'In Earth orbit','','2011-06-21 00:00:00',103.6,787,1071,98.19,'Sun-Synch' from dual union all
select 37695,'037695','1999-025ELL','deb FY-1C','',66706,'In Earth orbit','','2011-06-12 00:00:00',106.54,686,1446,98.24,'Sun-Synch' from dual union all
select 37696,'037696','1999-025ELM','deb FY-1C','',66706,'In Earth orbit','','2011-06-17 00:00:00',101.07,778,841,99.67,'Sun-Synch' from dual union all
select 37697,'037697','1999-025ELN','deb FY-1C','',66706,'In Earth orbit','','2011-06-19 00:00:00',100.63,723,854,96.6,'Sun-Synch' from dual union all
select 37698,'037698','1999-025ELP','deb FY-1C','',66706,'In Earth orbit','','2011-06-16 00:00:00',106.5,782,1346,98,'Sun-Synch' from dual union all
select 37699,'037699','1999-025ELQ','deb FY-1C','',66706,'In Earth orbit','','2011-06-14 00:00:00',103.89,845,1039,98.01,'Sun-Synch' from dual union all
select 377,'000377','62 A Sigma','FTV 1153','CORONA M-11',8154,'Reentered Attached','1962-09-10 00:00:00','1962-09-01 00:00:00',90.14,169,394,65.21,'Intermediate' from dual union all
select 3770,'003770','1969-018B','Saturn S-IVB-504N','Saturn S-IVB-504N',18285,'Deep Space','1969-03-07 00:00:00','1969-03-03 00:00:00',55733.66,2279,981685,33.83,'Earth Escape' from dual union all
select 37700,'037700','1999-025ELR','deb FY-1C','',66706,'In Earth orbit','','2011-06-05 00:00:00',102.07,833,881,99.04,'Sun-Synch' from dual union all
select 37701,'037701','1999-025ELS','deb FY-1C','',66706,'In Earth orbit','','2011-06-19 00:00:00',105.63,763,1285,97.64,'Sun-Synch' from dual union all
select 37702,'037702','1999-025ELT','deb FY-1C','',66706,'In Earth orbit','','2011-06-17 00:00:00',101.61,795,874,98.88,'Sun-Synch' from dual union all
select 37703,'037703','1999-025ELU','deb FY-1C','',66706,'In Earth orbit','','2011-06-06 00:00:00',101.82,800,890,99.14,'Sun-Synch' from dual union all
select 37704,'037704','1999-025ELV','deb FY-1C','',66706,'In Earth orbit','','2011-06-20 00:00:00',105.87,841,1230,98.35,'Sun-Synch' from dual union all
select 37705,'037705','1999-025ELW','deb FY-1C','',66706,'In Earth orbit','','2011-06-10 00:00:00',99.29,608,842,99.53,'Sun-Synch' from dual union all
select 37706,'037706','1999-025ELX','deb FY-1C','',66706,'In Earth orbit','','2011-06-11 00:00:00',102.61,839,925,99.08,'Sun-Synch' from dual union all
select 37707,'037707','1999-025ELY','deb FY-1C','',66706,'In Earth orbit','','2011-06-14 00:00:00',100.11,632,896,100.2,'Sun-Synch' from dual union all
select 37708,'037708','1999-025ELZ','deb FY-1C','',66706,'In Earth orbit','','2011-06-17 00:00:00',101.77,554,1131,98.32,'Sun-Synch' from dual union all
select 37709,'037709','1999-025EMA','deb FY-1C','',66706,'Reentered','2013-05-18 00:00:00','2011-06-24 00:00:00',97.83,520,790,99.08,'Sun-Synch' from dual union all
select 3771,'003771','1969-018C','Spider','LM 3 AS',18285,'Reentered','1981-10-23 00:00:00','1969-04-02 00:00:00',164.39,228,6896,28.91,'Medium' from dual union all
select 37710,'037710','1999-025EMB','deb FY-1C','',66706,'In Earth orbit','','2011-06-11 00:00:00',106.44,855,1269,99.22,'Sun-Synch' from dual union all
select 37711,'037711','1999-025EMC','deb FY-1C','',66706,'Reentered','2011-12-10 00:00:00','2011-06-17 00:00:00',99.47,523,943,94.52,'Polar' from dual union all
select 37712,'037712','1999-025EMD','deb FY-1C','',66706,'In Earth orbit','','2011-06-20 00:00:00',100.91,736,867,98.89,'Sun-Synch' from dual union all
select 37713,'037713','1999-025EME','deb FY-1C','',66706,'In Earth orbit','','2011-06-21 00:00:00',104.41,849,1085,98.1,'Sun-Synch' from dual union all
select 37714,'037714','1999-025EMF','deb FY-1C','',66706,'In Earth orbit','','2011-06-17 00:00:00',102.5,828,926,99.14,'Sun-Synch' from dual union all
select 37715,'037715','1999-025EMG','deb FY-1C','',66706,'In Earth orbit','','2011-06-20 00:00:00',103.61,841,1018,99.3,'Sun-Synch' from dual union all
select 37716,'037716','1999-025EMH','deb FY-1C','',66706,'In Earth orbit','','2011-06-23 00:00:00',102.09,831,884,98.96,'Sun-Synch' from dual union all
select 37717,'037717','1999-025EMJ','deb FY-1C','',66706,'In Earth orbit','','2011-06-24 00:00:00',100.42,672,885,98.81,'Sun-Synch' from dual union all
select 37718,'037718','1999-025EMK','deb FY-1C','',66706,'Reentered','2015-09-19 00:00:00','2011-06-18 00:00:00',98.03,489,840,99.17,'Sun-Synch' from dual;
insert into SATELLITE
select 37719,'037719','1999-025EML','deb FY-1C','',66706,'In Earth orbit','','2011-06-23 00:00:00',100.96,758,851,99.51,'Sun-Synch' from dual union all
select 3772,'003772','1969-019A','OPS 4248','GAMBIT-3 Mission 4320',18286,'Reentered Attached','1969-03-18 00:00:00','1969-03-06 00:00:00',90.31,132,447,92.01,'Polar' from dual union all
select 37720,'037720','1999-025EMM','deb FY-1C','',66706,'Reentered','2011-08-16 00:00:00','2011-06-14 00:00:00',94.24,401,563,98.55,'Sun-Synch' from dual union all
select 37721,'037721','1999-025EMN','deb FY-1C','',66706,'Reentered','2014-10-03 00:00:00','2011-06-23 00:00:00',99.68,665,821,99.05,'Sun-Synch' from dual union all
select 37722,'037722','1999-025EMP','deb FY-1C','',66706,'In Earth orbit','','2011-06-23 00:00:00',99.28,578,871,98.46,'Sun-Synch' from dual union all
select 37723,'037723','1999-025EMQ','deb FY-1C','',66706,'In Earth orbit','','2011-06-22 00:00:00',102.05,733,979,99.14,'Sun-Synch' from dual union all
select 37724,'037724','1999-025EMR','deb FY-1C','',66706,'In Earth orbit','','2011-06-17 00:00:00',99.94,617,895,99.2,'Sun-Synch' from dual union all
select 37725,'037725','1999-025EMS','deb FY-1C','',66706,'In Earth orbit','','2011-06-14 00:00:00',99.06,570,857,98.83,'Sun-Synch' from dual union all
select 37726,'037726','2011-028A','Kosmos-2472','Kobal''t-M No. 562',69080,'Landed','2011-10-24 00:00:00','2011-06-29 00:00:00',90.06,217,338,81.39,'Intermediate' from dual union all
select 37727,'037727','2011-028B','11S510','11S510',69080,'Reentered','2011-07-01 00:00:00','2011-06-29 00:00:00',88.3,172,209,81.39,'Intermediate' from dual union all
select 37728,'037728','2011-029A','USA 231','ORS-1',69081,'Reentered','2018-03-12 00:00:00','2011-07-03 00:00:00',92.61,396,409,40,'Intermediate' from dual union all
select 37729,'037729','2011-029B','Minotaur Stage 4','Orion 38',69081,'Reentered','2012-05-20 00:00:00','2011-07-05 00:00:00',92.61,399,406,40,'Intermediate' from dual union all
select 3773,'003773','1969-020A','Kosmos-268','DS-P1-Yu No. 18',18287,'Reentered','1970-05-09 00:00:00','1969-04-04 00:00:00',108.07,211,2063,48.42,'Intermediate' from dual union all
select 37730,'037730','2011-030A','Shi Jian 11-03','shijian shi yihao 03 xin',69082,'In Earth orbit','','2011-07-08 00:00:00',98.71,690,703,98.23,'Sun-Synch' from dual union all
select 37731,'037731','2011-030B','CZ-2C Stage 2','CZ-2C Stage 2',69082,'In Earth orbit','','2011-07-08 00:00:00',97.84,611,700,98.02,'Sun-Synch' from dual union all
select 37732,'037732','2011-030C','CZ-2C Stage 2 motor cover','',69082,'In Earth orbit','','2011-07-08 00:00:00',101.41,696,955,98.02,'Sun-Synch' from dual union all
select 37733,'037733','2011-030D','CZ-2C Stage 2 motor cover','',69082,'In Earth orbit','','2011-07-08 00:00:00',101.19,696,933,98.49,'Sun-Synch' from dual union all
select 37734,'037734','2011-030E','CZ-2C Stage 2 motor cover','',69082,'In Earth orbit','','2011-07-08 00:00:00',99.54,663,811,98.02,'Sun-Synch' from dual union all
select 37735,'037735','2011-030F','CZ-2C Stage 2 motor cover','',69082,'In Earth orbit','','2011-07-08 00:00:00',99.47,659,808,98.43,'Sun-Synch' from dual union all
select 37736,'037736','2011-031A','Atlantis (STS-135)','OV-104',69083,'Landed','2011-07-21 00:00:00','2011-07-09 00:00:00',89.9,214,325,51.64,'Intermediate' from dual union all
select 37737,'037737','2011-032A','Tian Lian 1B','tianlian yihao 02 xing',69084,'In Earth orbit','','2011-07-19 00:00:00',1436.11,35775,35799,1.07,'Stationary' from dual union all
select 37738,'037738','2011-032B','CZ-3C Stage 3','CZ-3C Stage 3',69084,'Reentered','2011-10-26 00:00:00','2011-07-21 00:00:00',758.52,149,42197,17.8,'Highly Ellip' from dual union all
select 37739,'037739','2011-033A','Globalstar M083','Globalstar-2 FM7',69085,'In Earth orbit','','2011-07-20 00:00:00',103.53,919,932,52,'Intermediate' from dual union all
select 3774,'003774','1969-020B','S1M','S1M',18287,'Reentered','1970-02-11 00:00:00','1969-04-08 00:00:00',107.48,223,1998,48.4,'Intermediate' from dual union all
select 37740,'037740','2011-033B','Globalstar M088','Globalstar-2 FM8',69085,'In Earth orbit','','2011-07-20 00:00:00',103.52,919,930,52,'Intermediate' from dual union all
select 37741,'037741','2011-033C','Globalstar M091','Globalstar-2 FM9',69085,'In Earth orbit','','2011-07-20 00:00:00',103.52,918,931,52,'Intermediate' from dual union all
select 37742,'037742','2011-033D','Globalstar M085','Globalstar-2 FM10',69085,'In Earth orbit','','2011-07-20 00:00:00',103.48,916,931,52,'Intermediate' from dual union all
select 37743,'037743','2011-033E','Globalstar M081','Globalstar-2 FM11',69085,'In Earth orbit','','2011-07-20 00:00:00',103.49,917,931,52,'Intermediate' from dual union all
select 37744,'037744','2011-033F','Globalstar M089','Globalstar-2 FM12',69085,'In Earth orbit','','2011-07-20 00:00:00',103.48,916,931,52,'Intermediate' from dual union all
select 37745,'037745','2011-033G','Fregat No. 1024','Fregat 14S44 No. 1024',69085,'Deorbited','2011-07-13 00:00:00','2011-07-13 00:00:00',103.48,916,931,52,'Intermediate' from dual union all
select 37746,'037746','2011-034A','GSAT-12','GSAT-12',69086,'In Earth orbit','','2011-07-28 00:00:00',1431.72,35685,35717,.17,'Drift GEO' from dual union all
select 37747,'037747','2011-034B','PSLV-C17 PS4','PS4',69086,'In Earth orbit','','2011-07-20 00:00:00',373.81,276,21347,17.83,'Highly Ellip' from dual union all
select 37748,'037748','2011-035A','SES-3','SES-3',69087,'In Earth orbit','','2011-07-25 00:00:00',1431.67,35619,35781,.08,'Drift GEO' from dual union all
select 37749,'037749','2011-035B','Kazsat-2','Kazsat-2',69087,'In Earth orbit','','2011-08-02 00:00:00',1435.12,35604,35931,.13,'Drift GEO' from dual union all
select 3775,'003775','1969-021A','Kosmos-269','Tselina-OM',18288,'Reentered','1978-10-21 00:00:00','1969-04-12 00:00:00',95.24,520,541,74.05,'Intermediate' from dual union all
select 37750,'037750','2011-035C','Briz-M DTB','Briz-M torus tank',69087,'In Earth orbit','','2011-07-23 00:00:00',440.09,338,25237,45.71,'Highly Ellip' from dual union all
select 37751,'037751','2011-035D','Briz-M No. 99519','Briz-M',69087,'In Earth orbit','','2011-07-24 00:00:00',1277.46,29661,35582,.35,'Medium' from dual union all
select 37752,'037752','2011-031B','PSSC-2','Picosat Solar Cell Testb',69083,'Reentered','2011-12-08 00:00:00','2011-07-20 00:00:00',92,365,381,51.64,'Intermediate' from dual union all
select 37753,'037753','2011-036A','GPS IIF-2','GPS SVN 63',69088,'In Earth orbit','','2011-07-24 00:00:00',729.16,20452,20463,55.03,'Medium' from dual union all
select 37754,'037754','2011-036B','Delta 355','DCSS-4 F11',69088,'In Earth orbit','','2011-07-24 00:00:00',757.09,20462,21815,54.78,'Medium' from dual union all
select 37755,'037755','2011-037A','Spektr-R','RadioAstron',69089,'Deep Space','2011-07-18 00:00:00','2011-07-25 00:00:00',12076.77,1248,334727,51.83,'Cislunar' from dual union all
select 37756,'037756','2011-037B','Fregat SBB','',69089,'In Earth orbit','','2011-07-25 00:00:00',128.69,429,3702,51.46,'Medium' from dual union all
select 37757,'037757','2011-037C','Sep Motor Cover','Sep Motor Cover',69089,'Reentered','2011-08-01 00:00:00','2011-07-20 00:00:00',90.37,176,410,51.41,'Intermediate' from dual union all
select 37758,'037758','2011-037D','11S772 2SB80.3','11S772 Zenit-3SLBFSBM',69089,'Reentered','2011-08-08 00:00:00','2011-07-25 00:00:00',90.04,173,379,51.41,'Intermediate' from dual union all
select 37759,'037759','2011-037E','Sep Motor Cover','Sep Motor Cover',69089,'Reentered','2011-08-03 00:00:00','2011-07-20 00:00:00',92.96,175,664,51.43,'Intermediate' from dual union all
select 3776,'003776','1969-021B','S3M','S3M',18288,'Reentered','1978-07-20 00:00:00','1969-04-04 00:00:00',95.41,536,541,74.16,'Intermediate' from dual union all
select 37760,'037760','2011-037F','Sep Motor Cover','Sep Motor Cover',69089,'Reentered','2011-08-03 00:00:00','2011-07-25 00:00:00',91.89,168,567,51.38,'Intermediate' from dual union all
select 37761,'037761','2011-037G','Sep Motor Cover','Sep Motor Cover',69089,'Reentered','2011-08-03 00:00:00','2011-07-25 00:00:00',92.1,172,583,51.38,'Intermediate' from dual union all
select 37762,'037762','1978-012E','Star 37E yo weight','',45207,'In Earth orbit','','2011-07-26 00:00:00',698.91,3688,35732,24.65,'GEO Transfer' from dual union all
select 37763,'037763','2011-038A','Beidou DW 9','di jiu ke beidou daohang',69090,'In Earth orbit','','2011-08-31 00:00:00',1436.04,35699,35872,55.19,'Synchronous' from dual union all
select 37764,'037764','2011-038B','CZ-3A Stage 3','CZ-3A Stage 3',69090,'Reentered','2012-03-03 00:00:00','2011-07-28 00:00:00',630.38,246,35705,54.88,'GEO Transfer' from dual union all
select 37765,'037765','2011-039A','Shi Jian 11-02','shijian shi yihao 02 xin',69091,'In Earth orbit','','2011-07-31 00:00:00',98.35,645,715,98.13,'Sun-Synch' from dual union all
select 37766,'037766','2011-039B','CZ-2C Stage 2','CZ-2C Stage 2',69091,'In Earth orbit','','2011-07-31 00:00:00',98.7,688,705,98.11,'Sun-Synch' from dual union all
select 37767,'037767','2011-039C','CZ-2C Stage 2 motor cover','',69091,'In Earth orbit','','2011-07-31 00:00:00',101.23,695,939,97.91,'Sun-Synch' from dual union all
select 37768,'037768','2011-039D','CZ-2C Stage 2 motor cover','',69091,'In Earth orbit','','2011-07-31 00:00:00',101.19,696,933,98.32,'Sun-Synch' from dual union all
select 37769,'037769','2011-039E','CZ-2C Stage 2 motor cover','',69091,'In Earth orbit','','2011-07-31 00:00:00',99.47,657,810,98.31,'Sun-Synch' from dual union all
select 3777,'003777','1969-022A','Kosmos-270','Zenit-4 No. 53',18289,'Landed','1969-03-14 00:00:00','1969-03-09 00:00:00',89.78,199,329,65.41,'Intermediate' from dual union all
select 37770,'037770','2011-039F','CZ-2C Stage 2 motor cover','',69091,'In Earth orbit','','2011-08-01 00:00:00',99.61,659,821,97.87,'Sun-Synch' from dual union all
select 37771,'037771','1998-067CJ','BTLS-N thernal cover','',66503,'Reentered','2011-09-21 00:00:00','2011-08-04 00:00:00',92.26,377,394,51.64,'Intermediate' from dual union all
select 37772,'037772','1998-067CK','RadioSkaf-V','ARISSat-1',66503,'Reentered','2012-01-04 00:00:00','2011-08-04 00:00:00',92.26,377,394,51.64,'Intermediate' from dual union all
select 37773,'037773','2011-040A','Juno','Juno',69092,'Deep Space','2011-08-05 00:00:00','2011-08-05 00:00:00',240.64,262,38650,28.91,'Earth Escape' from dual union all
select 37774,'037774','2011-040B','Centaur AV-029','Centaur AV-029',69092,'Deep Space','2011-08-05 00:00:00','2011-08-05 00:00:00',246.5,251,39053,28.91,'Earth Escape' from dual union all
select 37775,'037775','2011-041A','Astra 1N','Astra 1N',69093,'In Earth orbit','','2011-09-02 00:00:00',1436.09,35783,35790,.09,'Stationary' from dual union all
select 37776,'037776','2011-041B','BSAT 3c','JCSAT-110R',69093,'In Earth orbit','','2011-09-01 00:00:00',1436.14,35786,35789,.01,'Stationary' from dual union all
select 37777,'037777','2011-041C','ESC-A 560','Ariane 5 ESC-A',69093,'In Earth orbit','','2011-08-11 00:00:00',630.69,272,35695,1.99,'GEO Transfer' from dual union all
select 37778,'037778','2011-041D','Sylda 5A','Sylda 5 No. 46A',69093,'In Earth orbit','','2011-08-08 00:00:00',629.14,255,35632,1.64,'GEO Transfer' from dual union all
select 37779,'037779','2011-042A','Paksat 1R','Paksat 1R',69094,'In Earth orbit','','2011-09-01 00:00:00',1438.39,35818,35845,.15,'Drift GEO' from dual union all
select 3778,'003778','1969-022B','11A57I','11A57I',18289,'Reentered','1969-03-12 00:00:00','1969-03-08 00:00:00',89.52,190,312,65.39,'Intermediate' from dual union all
select 37780,'037780','2011-042B','CZ-3B Stage 3','CZ-3B Stage 3',69094,'Reentered','2011-12-21 00:00:00','2011-08-14 00:00:00',751.26,197,41797,25.02,'Highly Ellip' from dual union all
select 37781,'037781','2011-043A','Hai Yang 2','Haiyang er hao',69095,'In Earth orbit','','2011-08-18 00:00:00',103.21,905,916,99.36,'Sun-Synch' from dual union all
select 37782,'037782','2011-043B','CZ-4B Stage 3','CZ-4B Stage 3',69095,'In Earth orbit','','2011-08-18 00:00:00',100.25,630,911,99.47,'Sun-Synch' from dual union all
select 37783,'037783','2011-019C','deb Centaur','',69071,'Reentered','2011-08-17 00:00:00','2011-08-17 00:00:00',94.3,95,875,20.93,'Equatorial' from dual union all
select 37784,'037784','1970-103H','deb SOZ','Kosmos-382 part',21580,'In Earth orbit','','2011-08-19 00:00:00',171,2456,5199,55.84,'Medium' from dual union all
select 37785,'037785','1983-044GC','deb Kosmos-1461','',55801,'Reentered','2012-05-01 00:00:00','2011-08-24 00:00:00',97.08,533,705,65.01,'Intermediate' from dual union all
select 37786,'037786','1963-014FK','deb ERS 5','',9245,'Reentered','2011-11-09 00:00:00','2011-10-20 00:00:00',105.99,346,1736,84.37,'Intermediate' from dual union all
select 37787,'037787','2011-035E','SES-3 payload adapter','',69087,'In Earth orbit','','2011-07-26 00:00:00',698.91,3688,35732,24.65,'GEO Transfer' from dual union all
select 37788,'037788','2011-044A','Edusat','Edusat-ASI',69096,'In Earth orbit','','2011-08-19 00:00:00',98.11,640,697,98.26,'Sun-Synch' from dual union all
select 37789,'037789','2011-044B','NigeriaSat-2','DMC NigeriaSat-2',69096,'In Earth orbit','','2011-08-19 00:00:00',98.99,693,728,98.26,'Sun-Synch' from dual union all
select 3779,'003779','1969-021C','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1969-04-24 00:00:00','1969-03-12 00:00:00',94.02,419,524,73.06,'Intermediate' from dual union all
select 37790,'037790','2011-044C','NigeriaSat-X','DMC NigeriaSat-X',69096,'In Earth orbit','','2011-08-19 00:00:00',98.28,656,697,98.26,'Sun-Synch' from dual union all
select 37791,'037791','2011-044D','Rasat','Rasat',69096,'In Earth orbit','','2011-08-19 00:00:00',98.39,667,697,98.26,'Sun-Synch' from dual union all
select 37792,'037792','2011-044E','AprizeSat-5','AprizeSat-5',69096,'In Earth orbit','','2011-08-19 00:00:00',97.79,610,696,98.27,'Sun-Synch' from dual union all
select 37793,'037793','2011-044F','AprizeSat-6','AprizeSat-6',69096,'In Earth orbit','','2011-08-19 00:00:00',97.96,627,695,98.26,'Sun-Synch' from dual union all
select 37794,'037794','2011-044G','Sich-2','MS-2-8 No. 1',69096,'In Earth orbit','','2011-08-19 00:00:00',98.63,685,702,98.26,'Sun-Synch' from dual union all
select 37795,'037795','2011-044H','R-36M2 DS','R-36M2 DS',69096,'In Earth orbit','','2011-08-19 00:00:00',104.96,691,1295,98.22,'Sun-Synch' from dual union all
select 37796,'037796','2011-044J','Dnepr Platform A/Fairing','',69096,'In Earth orbit','','2011-08-18 00:00:00',98.78,691,710,98.26,'Sun-Synch' from dual union all
select 37797,'037797','2011-044K','Gazodinamicheskiy Ekran','GDS (Gas dynamic shield)',69096,'Reentered','2016-03-03 00:00:00','2011-08-18 00:00:00',98.1,640,696,98.26,'Sun-Synch' from dual union all
select 37798,'037798','2011-045A','Ekspress AM-4','Ekspress AM-4',69097,'Deorbited','2012-03-25 00:00:00','2011-08-25 00:00:00',362.65,696,20239,51.16,'Highly Ellip' from dual union all
select 37799,'037799','2011-045B','Briz-M No. 99522','Briz-M',69097,'In Earth orbit','','2011-08-26 00:00:00',368.83,1014,20303,51.28,'Highly Ellip' from dual union all
select 378,'000378','62 A Omicron 3','P35 despin weight','',8149,'Reentered','1992-08-12 00:00:00','1962-09-23 00:00:00',100.75,619,970,98.61,'Sun-Synch' from dual union all
select 3780,'003780','1969-018D','LM 3 DS','LM 3 DS',18285,'Reentered','1969-03-22 00:00:00','1969-03-12 00:00:00',89.27,233,244,33.62,'Intermediate' from dual union all
select 37800,'037800','2011-045C','Briz-M DTB','Briz-M torus tank',69097,'Reentered','2012-09-12 00:00:00','2011-08-24 00:00:00',218.94,146,11163,49.47,'Medium' from dual union all
select 37801,'037801','2011-046A','Ebb','GRAIL A',69098,'Deep Space','2011-09-11 00:00:00','2011-09-18 00:00:00',81032.59,0,1227839,29.56,'Cislunar' from dual union all
select 37802,'037802','2011-046B','Flow','GRAIL B',69098,'Deep Space','2011-09-11 00:00:00','2011-09-19 00:00:00',82294.84,80,1240609,29.47,'Cislunar' from dual union all
select 37803,'037803','2011-046C','Delta 356','Delta SSPS AJ-10-118K',69098,'Deep Space','2011-09-11 00:00:00','2011-09-10 00:00:00',3932.67,1260,179080,29.32,'Earth Escape' from dual union all
select 37804,'037804','2011-047A','Zhongxing 1A','Fenghuo 2',69099,'In Earth orbit','','2011-09-28 00:00:00',1436.06,35781,35791,.57,'Stationary' from dual union all
select 37805,'037805','2011-047B','CZ-3B Stage 3','CZ-3B Stage 3',69099,'Reentered','2012-02-07 00:00:00','2011-09-28 00:00:00',630.53,178,35781,26.54,'GEO Transfer' from dual union all
select 37806,'037806','2011-048A','Kosmos-2473','Garpun No. 11',69100,'In Earth orbit','','2011-10-20 00:00:00',1436.06,35778,35793,.1,'Stationary' from dual union all
select 37807,'037807','2011-048B','Briz-M No. 88529','Briz-M',69100,'In Earth orbit','','2011-10-21 00:00:00',1381.09,33767,35639,.12,'Drift GEO' from dual union all
select 37808,'037808','2011-048C','Briz-M DTB','Briz-M torus tank',69100,'In Earth orbit','','2011-09-28 00:00:00',631.1,401,35587,48.61,'GEO Transfer' from dual;
insert into SATELLITE
select 37809,'037809','2011-049A','SES-2','SES-2',69101,'In Earth orbit','','2011-10-18 00:00:00',1436.09,35783,35790,.05,'Stationary' from dual union all
select 3781,'003781','1969-021D','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1969-05-07 00:00:00','1969-04-04 00:00:00',93.91,464,468,73.77,'Intermediate' from dual union all
select 37810,'037810','2011-049B','Arabsat 5C','Arabsat 5C',69101,'In Earth orbit','','2011-09-29 00:00:00',1423.69,35414,35673,.18,'Drift GEO' from dual union all
select 37811,'037811','2011-049C','ESC-A 561','Ariane 5 ESC-A',69101,'In Earth orbit','','2011-09-29 00:00:00',629.72,260,35657,2.41,'GEO Transfer' from dual union all
select 37812,'037812','2011-049D','Sylda 5A','Sylda 5 No. 47-A',69101,'Reentered','2015-09-19 00:00:00','2011-09-29 00:00:00',629.76,256,35663,2.42,'GEO Transfer' from dual union all
select 37813,'037813','2011-050A','JSE kougaku 4 gouki','IGS Optical-4',69102,'In Earth orbit','','2011-09-23 00:00:00',96.46,588,591,97.69,'Sun-Synch' from dual union all
select 37814,'037814','2011-050B','H-2A F19 Stage 2','H-2A Stage 2',69102,'In Earth orbit','','2011-09-23 00:00:00',96.22,564,590,97.69,'Sun-Synch' from dual union all
select 37815,'037815','2011-050C','deb IGS O-4','',69102,'Reentered','2013-01-12 00:00:00','2011-10-03 00:00:00',96.17,563,588,97.63,'Sun-Synch' from dual union all
select 37816,'037816','2011-051A','Atlantic Bird 7','Atlantic Bird 7',69103,'In Earth orbit','','2011-10-02 00:00:00',1431.87,35618,35790,.06,'Drift GEO' from dual union all
select 37817,'037817','2011-051B','Blok DM-SL No. 31L','Blok DM-SL',69103,'In Earth orbit','','2011-10-06 00:00:00',653.48,1599,35535,.03,'GEO Transfer' from dual union all
select 37818,'037818','2011-052A','TacSat-4','TacSat-4',69104,'In Earth orbit','','2011-10-05 00:00:00',238.86,748,12000,63.63,'Medium' from dual union all
select 37819,'037819','2011-052B','Minotaur Stage 4','Star 48V',69104,'Reentered','2013-09-13 00:00:00','2011-10-05 00:00:00',228.12,185,11792,63.43,'Medium' from dual union all
select 3782,'003782','1969-021E','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1970-04-02 00:00:00','1969-04-16 00:00:00',97.23,536,716,72.96,'Intermediate' from dual union all
select 37820,'037820','2011-053A','Tiangong 1','Tiangong 1',69105,'Reentered','2018-04-02 00:00:00','2011-10-06 00:00:00',91.35,334,348,42.78,'Intermediate' from dual union all
select 37821,'037821','2011-053B','CZ-2F Stage 2','CZ-2F Stage 2',69105,'Reentered','2011-10-10 00:00:00','2011-10-01 00:00:00',89.56,193,313,42.79,'Intermediate' from dual union all
select 37822,'037822','2011-053C','CZ-2F sep motor cover','',69105,'Reentered','2011-10-01 00:00:00','2011-09-30 00:00:00',90.04,182,371,42.85,'Intermediate' from dual union all
select 37823,'037823','2011-053D','CZ-2F sep motor cover','',69105,'Reentered','2011-10-01 00:00:00','2011-09-30 00:00:00',90.85,166,467,42.54,'Intermediate' from dual union all
select 37824,'037824','2011-053E','CZ-2F sep motor cover','',69105,'Reentered','2011-10-02 00:00:00','2011-10-01 00:00:00',90.88,186,450,42.66,'Intermediate' from dual union all
select 37825,'037825','2011-053F','CZ-2F sep motor cover','',69105,'Reentered','2011-10-01 00:00:00','2011-09-30 00:00:00',89.88,179,358,42.91,'Intermediate' from dual union all
select 37826,'037826','2011-054A','QuetzSat-1','QuetzSat-1',69106,'In Earth orbit','','2011-10-14 00:00:00',1436.04,35774,35797,.06,'Stationary' from dual union all
select 37827,'037827','2011-054B','Briz-M No. 99524','Briz-M',69106,'In Earth orbit','','2011-11-21 00:00:00',693.43,5149,33998,18.23,'GEO Transfer' from dual union all
select 37828,'037828','2011-054C','Briz-M DTB','Briz-M torus tank',69106,'In Earth orbit','','2011-10-08 00:00:00',316.86,328,17708,49.49,'Highly Ellip' from dual union all
select 37829,'037829','2011-055A','Glonass-M No. 42','Uragan-M No. 742',69107,'In Earth orbit','','2011-10-09 00:00:00',671.84,18913,19151,64.8,'Medium' from dual union all
select 3783,'003783','1968-097AV','deb Kosmos-252 [*]','',16824,'Reentered','2002-07-18 00:00:00','1989-11-19 00:00:00',105.67,508,1544,62.4,'Intermediate' from dual union all
select 37830,'037830','2011-055B','Fregat No. 1045','Fregat 14S44 No. 1045',69107,'In Earth orbit','','2011-10-13 00:00:00',687.15,19150,19683,64.77,'Medium' from dual union all
select 37831,'037831','1991-032D','deb NOAA 12','',64201,'In Earth orbit','','2011-10-03 00:00:00',100.7,785,799,98.73,'Sun-Synch' from dual union all
select 37832,'037832','1991-032E','deb NOAA 12','',64201,'In Earth orbit','','2011-10-03 00:00:00',100.71,787,797,98.74,'Sun-Synch' from dual union all
select 37833,'037833','2011-050D','deb IGS O-4','',69102,'Reentered','2012-11-01 00:00:00','2011-10-04 00:00:00',96.22,563,592,97.65,'Sun-Synch' from dual union all
select 37834,'037834','2011-056A','Intelsat IS-18','Intelsat IS-18',69108,'In Earth orbit','','2011-10-26 00:00:00',1436.04,35780,35791,.04,'Stationary' from dual union all
select 37835,'037835','2011-056B','Blok DM-SLB No. 5L','Blok DM-SLB',69108,'In Earth orbit','','2011-10-13 00:00:00',731.69,5270,35769,19.47,'GEO Transfer' from dual union all
select 37836,'037836','2011-057A','Eutelsat W3C','Eutelsat W3C',69109,'In Earth orbit','','2011-10-15 00:00:00',1433.89,35275,36212,.05,'Drift GEO' from dual union all
select 37837,'037837','2011-057B','CZ-3B Stage 3','CZ-3B Stage 3',69109,'Reentered','2011-10-30 00:00:00','2011-10-16 00:00:00',344.94,613,19215,26.18,'Highly Ellip' from dual union all
select 37838,'037838','2011-058A','Megha-Tropiques','Megha-Tropiques',69110,'In Earth orbit','','2011-10-14 00:00:00',101.34,779,866,19.79,'Equatorial' from dual union all
select 37839,'037839','2011-058B','Jugnu','Jugnu',69110,'In Earth orbit','','2011-10-13 00:00:00',102.1,850,867,19.98,'Equatorial' from dual union all
select 3784,'003784','1968-097AW','deb Kosmos-252','',16824,'In Earth orbit','','1992-02-02 00:00:00',111.22,670,1896,62.22,'Intermediate' from dual union all
select 37840,'037840','2011-058C','VesselSat1','AIS VesselSat 1',69110,'In Earth orbit','','2011-10-13 00:00:00',102.07,847,867,19.97,'Equatorial' from dual union all
select 37841,'037841','2011-058D','SRMSAT','SRMSAT',69110,'In Earth orbit','','2011-10-13 00:00:00',102.08,848,867,19.98,'Equatorial' from dual union all
select 37842,'037842','2011-058E','PS4','PS4 PSLV-C18',69110,'In Earth orbit','','2011-10-13 00:00:00',101.97,838,866,19.96,'Equatorial' from dual union all
select 37843,'037843','2011-059A','ViaSat-1','ViaSat-1',69111,'In Earth orbit','','2011-11-15 00:00:00',1436.04,35777,35794,0,'Stationary' from dual union all
select 37844,'037844','2011-059B','Briz-M No. 99521','Briz-M',69111,'In Earth orbit','','2011-11-10 00:00:00',633.43,1835,34273,30.88,'GEO Transfer' from dual union all
select 37845,'037845','2011-059C','Briz-M DTB','Briz-M torus tank',69111,'In Earth orbit','','2011-10-27 00:00:00',234.2,326,12090,49.75,'Medium' from dual union all
select 37846,'037846','2011-060A','GalileoSat-1','Galileo IOV PFM',69112,'In Earth orbit','','2011-10-28 00:00:00',846.79,23238,23304,54.69,'Medium' from dual union all
select 37847,'037847','2011-060B','GalileoSat-2','Galileo IOV FM2',69112,'In Earth orbit','','2011-10-25 00:00:00',846.81,23238,23305,54.69,'Medium' from dual union all
select 37848,'037848','2011-060C','Fregat-MT No. 1030','Fregat-MT    No. 1030',69112,'In Earth orbit','','2011-10-29 00:00:00',862.04,23603,23649,54.7,'Medium' from dual union all
select 37849,'037849','2011-061A','Suomi NPP','NPOESS Prep Project',69113,'In Earth orbit','','2011-11-04 00:00:00',101.29,817,823,98.71,'Sun-Synch' from dual union all
select 3785,'003785','1968-097AX','deb Kosmos-252','',16824,'In Earth orbit','','1992-02-03 00:00:00',114.1,772,2057,62.27,'Intermediate' from dual union all
select 37850,'037850','1993-036BPE','deb Kosmos-2251','',65138,'In Earth orbit','','2011-10-28 00:00:00',101.24,816,819,98.71,'Sun-Synch' from dual union all
select 37851,'037851','2011-061B','DICE-1','DICE-1',69113,'In Earth orbit','','2011-11-04 00:00:00',97.43,457,815,101.7,'Retrograde' from dual union all
select 37852,'037852','2011-061C','DICE-2','DICE-2',69113,'In Earth orbit','','2011-11-07 00:00:00',97.42,456,815,101.7,'Retrograde' from dual union all
select 37853,'037853','2011-061D','RAX-2','Radio Aurora Explorer',69113,'In Earth orbit','','2011-11-04 00:00:00',97.44,458,815,101.7,'Retrograde' from dual union all
select 37854,'037854','2011-061E','AubieSat-OSCAR-71','AubieSat-1',69113,'In Earth orbit','','2011-10-31 00:00:00',97.44,457,815,101.71,'Retrograde' from dual union all
select 37855,'037855','2011-061F','Hiscock Rad. Belt Explorer','Explorer 1 Prime Unit 2',69113,'In Earth orbit','','2011-11-04 00:00:00',97.44,458,815,101.7,'Retrograde' from dual union all
select 37856,'037856','2011-061G','Delta 357','Delta SSPS AJ-10-118K',69113,'Reentered','2011-11-29 00:00:00','2011-11-04 00:00:00',92.86,177,653,107.31,'Retrograde' from dual union all
select 37857,'037857','2011-062A','Progress M-13M','Progress 7K-TGM No.413',69114,'Deorbited','2012-01-25 00:00:00','2011-11-01 00:00:00',89.79,254,275,51.64,'Intermediate' from dual union all
select 37858,'037858','2011-062B','11S510','11S510',69114,'Reentered','2011-11-01 00:00:00','2011-10-31 00:00:00',87.96,163,183,51.63,'Intermediate' from dual union all
select 37859,'037859','2011-063A','Shenzhou 8','shenzhou ba fanhui cang',69115,'Landed','2011-11-17 00:00:00','2011-11-01 00:00:00',90.26,262,313,42.78,'Intermediate' from dual union all
select 3786,'003786','1968-097AY','deb Kosmos-252','',16824,'Reentered','1979-12-21 00:00:00','1979-12-20 00:00:00',106.64,495,1647,62.32,'Intermediate' from dual union all
select 37860,'037860','2011-063B','CZ-2F Stage 2','CZ-2F Stage 2',69115,'Reentered','2011-11-08 00:00:00','2011-11-03 00:00:00',89.33,192,291,42.79,'Intermediate' from dual union all
select 37861,'037861','2011-063C','CZ-2F sep motor cover','',69115,'Reentered','2011-11-06 00:00:00','2011-11-01 00:00:00',91.38,191,494,42.61,'Intermediate' from dual union all
select 37862,'037862','1965-063AE','deb FW4S','',11813,'In Earth orbit','','2011-11-15 00:00:00',91.8,216,510,51.45,'Intermediate' from dual union all
select 37863,'037863','1982-001E','deb Kosmos-1331','Strela-2M debris',54310,'In Earth orbit','','2011-11-15 00:00:00',92.12,242,514,51.41,'Intermediate' from dual union all
select 37864,'037864','1970-025QS','deb Agena D','',21502,'Reentered','2014-01-08 00:00:00','2011-11-15 00:00:00',92.21,242,524,51.45,'Intermediate' from dual union all
select 37865,'037865','1982-102D','deb Kosmos-1417','',54411,'In Earth orbit','','2011-11-15 00:00:00',92.02,220,528,51.41,'Intermediate' from dual union all
select 37866,'037866','1993-036BPD','deb Kosmos-2251','',65138,'Reentered','2014-09-10 00:00:00','2011-11-16 00:00:00',98.16,580,761,73.97,'Intermediate' from dual union all
select 37867,'037867','2011-064A','Glonass-M No. 43','Uragan-M No. 743',69116,'In Earth orbit','','2011-11-11 00:00:00',672.26,18912,19173,64.77,'Medium' from dual union all
select 37868,'037868','2011-064B','Glonass-M No. 44','Uragan-M No. 744',69116,'In Earth orbit','','2011-11-11 00:00:00',680.11,19161,19319,64.79,'Medium' from dual union all
select 37869,'037869','2011-064C','Glonass-M No. 45','Uragan-M No. 745',69116,'In Earth orbit','','2011-11-12 00:00:00',674.46,19018,19178,64.79,'Medium' from dual union all
select 3787,'003787','1968-097AZ','deb Kosmos-252','',16824,'Reentered','1982-11-12 00:00:00','1971-01-06 00:00:00',104.74,545,1420,62.29,'Intermediate' from dual union all
select 37870,'037870','2011-064D','Briz-M No. 99523','Briz-M',69116,'In Earth orbit','','2011-11-30 00:00:00',638.38,17254,19108,64.86,'Medium' from dual union all
select 37871,'037871','2011-064E','Briz-M DTB','Briz-M torus tank',69116,'In Earth orbit','','2011-11-12 00:00:00',338.46,313,19107,52.53,'Highly Ellip' from dual union all
select 37872,'037872','2011-065A','Fobos-Grunt','Pereletniy modul'' FG',69117,'Reentered Attached','2012-01-15 00:00:00','2011-11-10 00:00:00',89.95,206,338,51.43,'Intermediate' from dual union all
select 37873,'037873','2011-065B','11S772 2SB41.1','11S772 Zenit-2SB',69117,'Reentered','2011-11-22 00:00:00','2011-11-10 00:00:00',89.7,198,322,51.43,'Intermediate' from dual union all
select 37874,'037874','2011-066A','Tianxun-1','tianxun yihao',69118,'Reentered','2016-02-07 00:00:00','2011-11-11 00:00:00',94.28,479,489,97.42,'Sun-Synch' from dual union all
select 37875,'037875','2011-066B','Yaogan Weixing 12','Jianbing-10 02 xing',69118,'In Earth orbit','','2011-11-11 00:00:00',94.34,483,491,97.42,'Sun-Synch' from dual union all
select 37876,'037876','2011-066C','CZ-4B Stage 3','CZ-4B Stage 3',69118,'Reentered','2011-12-18 00:00:00','2011-11-11 00:00:00',91.46,213,480,97.51,'Sun-Synch' from dual union all
select 37877,'037877','2011-067A','Soyuz TMA-22','Soyuz 11F732A17 No. 232',69119,'Landed','2012-04-27 00:00:00','2011-11-15 00:00:00',90.61,281,328,51.64,'Intermediate' from dual union all
select 37878,'037878','2011-067B','11S510','11S510',69119,'Reentered','2011-11-16 00:00:00','2011-11-15 00:00:00',88.41,179,213,51.62,'Intermediate' from dual union all
select 37879,'037879','2011-065C','Sep Motor Cover','',69117,'Reentered','2011-12-01 00:00:00','2011-11-15 00:00:00',91.8,216,510,51.45,'Intermediate' from dual union all
select 3788,'003788','1968-097BA','deb Kosmos-252','',16824,'In Earth orbit','','1992-01-31 00:00:00',121.24,1280,2193,61.89,'Medium' from dual union all
select 37880,'037880','2011-065D','Sep Motor Cover','',69117,'Reentered','2011-12-11 00:00:00','2011-11-15 00:00:00',92.12,242,514,51.41,'Intermediate' from dual union all
select 37881,'037881','2011-065E','Sep Motor Cover','',69117,'Reentered','2011-12-12 00:00:00','2011-11-15 00:00:00',92.21,242,524,51.45,'Intermediate' from dual union all
select 37882,'037882','2011-065F','Sep Motor Cover','',69117,'Reentered','2011-11-28 00:00:00','2011-11-15 00:00:00',92.02,220,528,51.41,'Intermediate' from dual union all
select 37883,'037883','2011-063D','Shenzhou 8 Guidao Cang','shenzhou ba hao guidao c',69115,'Reentered','2012-04-02 00:00:00','2011-11-18 00:00:00',91.2,332,335,42.8,'Intermediate' from dual union all
select 37884,'037884','1987-079BP','deb SOZ','',60169,'Reentered','2009-12-29 00:00:00','2011-11-18 00:00:00',304.59,466,16770,65.22,'Highly Ellip' from dual union all
select 37885,'037885','1990-105AF','deb Star 37S','',63347,'In Earth orbit','','2011-11-16 00:00:00',99.96,705,809,98.29,'Sun-Synch' from dual union all
select 37886,'037886','1971-114E','deb Kosmos-468','',24932,'In Earth orbit','','2011-11-17 00:00:00',100.15,749,783,74.02,'Intermediate' from dual union all
select 37887,'037887','1971-114F','deb Kosmos-468','',24932,'In Earth orbit','','2011-11-18 00:00:00',100.05,751,771,74.01,'Intermediate' from dual union all
select 37888,'037888','1966-040AT','Nimbus 2 debris','',13447,'In Earth orbit','','2011-11-19 00:00:00',107.87,1084,1173,100.59,'Sun-Synch' from dual union all
select 37889,'037889','1966-040AU','Nimbus 2 debris','',13447,'In Earth orbit','','2011-11-19 00:00:00',107.87,1085,1172,100.56,'Sun-Synch' from dual union all
select 3789,'003789','1968-091AW','deb Kosmos-249 [*]','',16818,'Reentered','1975-07-02 00:00:00','1977-08-20 00:00:00',92.59,383,420,62.22,'Intermediate' from dual union all
select 37890,'037890','1966-040AV','Nimbus 2 debris','',13447,'In Earth orbit','','2011-11-19 00:00:00',107.96,1090,1175,100.5,'Sun-Synch' from dual union all
select 37891,'037891','1966-040AW','Nimbus 2 debris','',13447,'In Earth orbit','','2011-11-19 00:00:00',107.95,1080,1184,100.52,'Sun-Synch' from dual union all
select 37892,'037892','1966-040AX','Nimbus 2 debris','',13447,'In Earth orbit','','2011-11-13 00:00:00',107.96,1092,1173,100.51,'Sun-Synch' from dual union all
select 37893,'037893','1966-040AY','Nimbus 2 debris','',13447,'In Earth orbit','','2011-11-17 00:00:00',107.82,1077,1175,100.52,'Sun-Synch' from dual union all
select 37894,'037894','1999-025EMT','deb FY-1C','',66706,'In Earth orbit','','2011-11-18 00:00:00',101.32,709,934,99.02,'Sun-Synch' from dual union all
select 37895,'037895','1999-025EMU','deb FY-1C','',66706,'In Earth orbit','','2011-11-12 00:00:00',107.48,852,1369,99.19,'Sun-Synch' from dual union all
select 37896,'037896','1999-025EMV','deb FY-1C','',66706,'In Earth orbit','','2011-10-29 00:00:00',99.9,468,1040,98.71,'Sun-Synch' from dual union all
select 37897,'037897','1999-025EMW','deb FY-1C','',66706,'In Earth orbit','','2011-10-30 00:00:00',101.73,826,856,98.93,'Sun-Synch' from dual union all
select 37898,'037898','1999-025EMX','deb FY-1C','',66706,'In Earth orbit','','2011-11-10 00:00:00',103.54,829,1023,98.33,'Sun-Synch' from dual union all
select 37899,'037899','1999-025EMY','deb FY-1C','',66706,'In Earth orbit','','2011-11-17 00:00:00',105.72,849,1208,98.81,'Sun-Synch' from dual;
insert into SATELLITE
select 379,'000379','62 A Pi 4','deb Sputnik','',8151,'Reentered','1962-09-05 00:00:00','1962-08-30 00:00:00',88.06,130,227,65.59,'Intermediate' from dual union all
select 3790,'003790','1968-091AX','deb Kosmos-249 [*]','',16818,'In Earth orbit','','1992-02-02 00:00:00',109.73,585,1844,63.02,'Intermediate' from dual union all
select 37900,'037900','1999-025EMZ','deb FY-1C','',66706,'In Earth orbit','','2011-11-15 00:00:00',104.39,767,1164,102.34,'Sun-Synch' from dual union all
select 37901,'037901','1999-025ENA','deb FY-1C','',66706,'In Earth orbit','','2011-11-12 00:00:00',100.72,634,952,97.86,'Sun-Synch' from dual union all
select 37902,'037902','1999-025ENB','deb FY-1C','',66706,'In Earth orbit','','2011-11-10 00:00:00',107.65,850,1385,98.71,'Sun-Synch' from dual union all
select 37903,'037903','1999-025ENC','deb FY-1C','',66706,'In Earth orbit','','2011-11-01 00:00:00',105.37,719,1304,98.2,'Sun-Synch' from dual union all
select 37904,'037904','1999-025END','deb FY-1C','',66706,'In Earth orbit','','2011-11-08 00:00:00',101.31,756,886,98.37,'Sun-Synch' from dual union all
select 37905,'037905','1999-025ENE','deb FY-1C','',66706,'In Earth orbit','','2011-11-19 00:00:00',104.21,852,1063,99.38,'Sun-Synch' from dual union all
select 37906,'037906','1999-025ENF','deb FY-1C','',66706,'In Earth orbit','','2011-11-17 00:00:00',104.69,854,1106,99.3,'Sun-Synch' from dual union all
select 37907,'037907','1999-025ENG','deb FY-1C','',66706,'In Earth orbit','','2011-11-17 00:00:00',103.21,847,973,99.23,'Sun-Synch' from dual union all
select 37908,'037908','1999-025ENH','deb FY-1C','',66706,'In Earth orbit','','2011-11-18 00:00:00',101.03,744,870,99.06,'Sun-Synch' from dual union all
select 37909,'037909','1999-025ENJ','deb FY-1C','',66706,'In Earth orbit','','2011-11-17 00:00:00',105.15,841,1162,98.91,'Sun-Synch' from dual union all
select 3791,'003791','1968-091AY','deb Kosmos-249','',16818,'Reentered','1990-01-29 00:00:00','1990-01-29 00:00:00',88.63,200,214,62.26,'Intermediate' from dual union all
select 37910,'037910','1999-025ENK','deb FY-1C','',66706,'In Earth orbit','','2011-11-19 00:00:00',101.48,793,865,99.03,'Sun-Synch' from dual union all
select 37911,'037911','1999-025ENL','deb FY-1C','',66706,'In Earth orbit','','2011-11-10 00:00:00',104.39,845,1086,99.39,'Sun-Synch' from dual union all
select 37912,'037912','1999-025ENM','deb FY-1C','',66706,'In Earth orbit','','2011-11-18 00:00:00',103.85,847,1033,99.06,'Sun-Synch' from dual union all
select 37913,'037913','1999-025ENN','deb FY-1C','',66706,'In Earth orbit','','2011-10-23 00:00:00',101.36,664,983,98.95,'Sun-Synch' from dual union all
select 37914,'037914','1999-025ENP','deb FY-1C','',66706,'In Earth orbit','','2011-11-18 00:00:00',102.78,836,945,98.87,'Sun-Synch' from dual union all
select 37915,'037915','1999-025ENQ','deb FY-1C','',66706,'In Earth orbit','','2011-11-17 00:00:00',104.61,807,1146,98.31,'Sun-Synch' from dual union all
select 37916,'037916','1999-025ENR','deb FY-1C','',66706,'Reentered','2014-09-10 00:00:00','2011-10-26 00:00:00',99.48,641,827,98.98,'Sun-Synch' from dual union all
select 37917,'037917','1999-025ENS','deb FY-1C','',66706,'In Earth orbit','','2011-11-15 00:00:00',102.69,849,923,99.25,'Sun-Synch' from dual union all
select 37918,'037918','1999-025ENT','deb FY-1C','',66706,'In Earth orbit','','2011-11-16 00:00:00',101.22,768,865,99.03,'Sun-Synch' from dual union all
select 37919,'037919','1999-025ENU','deb FY-1C','',66706,'In Earth orbit','','2011-11-15 00:00:00',101.46,790,866,99.08,'Sun-Synch' from dual union all
select 3792,'003792','1968-091AZ','deb Kosmos-249','',16818,'Reentered','1981-04-13 00:00:00','1969-05-06 00:00:00',109.87,498,1944,62.37,'Intermediate' from dual union all
select 37920,'037920','1999-025ENV','deb FY-1C','',66706,'In Earth orbit','','2011-11-07 00:00:00',102.74,835,942,99.1,'Sun-Synch' from dual union all
select 37921,'037921','1999-025ENW','deb FY-1C','',66706,'In Earth orbit','','2011-11-18 00:00:00',103.38,856,980,99.16,'Sun-Synch' from dual union all
select 37922,'037922','1999-025ENX','deb FY-1C','',66706,'In Earth orbit','','2011-11-12 00:00:00',102.94,845,951,99.08,'Sun-Synch' from dual union all
select 37923,'037923','1999-025ENY','deb FY-1C','',66706,'Reentered','2014-11-21 00:00:00','2011-11-19 00:00:00',99.66,633,852,99.06,'Sun-Synch' from dual union all
select 37924,'037924','1999-025ENZ','deb FY-1C','',66706,'In Earth orbit','','2011-11-15 00:00:00',102.18,819,905,98.99,'Sun-Synch' from dual union all
select 37925,'037925','1999-025EPA','deb FY-1C','',66706,'In Earth orbit','','2011-11-11 00:00:00',100.21,697,840,99.09,'Sun-Synch' from dual union all
select 37926,'037926','1999-025EPB','deb FY-1C','',66706,'In Earth orbit','','2011-11-16 00:00:00',102.51,828,927,99.12,'Sun-Synch' from dual union all
select 37927,'037927','1999-025EPC','deb FY-1C','',66706,'In Earth orbit','','2011-11-18 00:00:00',101.81,835,854,98.93,'Sun-Synch' from dual union all
select 37928,'037928','1999-025EPD','deb FY-1C','',66706,'In Earth orbit','','2011-11-18 00:00:00',101.06,755,863,98.94,'Sun-Synch' from dual union all
select 37929,'037929','1999-025EPE','deb FY-1C','',66706,'In Earth orbit','','2011-11-14 00:00:00',99.74,579,914,98.94,'Sun-Synch' from dual union all
select 3793,'003793','1968-091BA','deb Kosmos-249','',16818,'In Earth orbit','','1992-02-02 00:00:00',112.45,737,1942,62.24,'Intermediate' from dual union all
select 37930,'037930','2011-068A','Chuangxin-1 03','chuangxin yihao 03 xing',69120,'In Earth orbit','','2011-11-27 00:00:00',100.74,784,803,98.45,'Sun-Synch' from dual union all
select 37931,'037931','2011-068B','Shiyan Weixing 4','shiyan weixing sihao',69120,'In Earth orbit','','2011-11-27 00:00:00',100.72,784,802,98.45,'Sun-Synch' from dual union all
select 37932,'037932','2011-068C','CZ-2D Stage 2','CZ-2D Stage 2',69120,'In Earth orbit','','2011-11-27 00:00:00',101.3,794,847,98.33,'Sun-Synch' from dual union all
select 37933,'037933','2011-069A','Asiasat 7','Asiasat 7',69121,'In Earth orbit','','2011-12-03 00:00:00',1430.37,35582,35767,.05,'Drift GEO' from dual union all
select 37934,'037934','2011-069B','Briz-M No. 99527','Briz-M',69121,'In Earth orbit','','2011-12-02 00:00:00',839.15,11872,34312,.25,'Medium' from dual union all
select 37935,'037935','2011-069C','Briz-M DTB','Briz-M torus tank',69121,'In Earth orbit','','2011-12-03 00:00:00',629.54,383,35525,49.15,'GEO Transfer' from dual union all
select 37936,'037936','2011-070A','Curiosity','Mars Science Lab Rover',69122,'Deep Space','2011-11-27 00:00:00','2011-11-26 00:00:00',1188.91,194,87301,34.9,'Earth Escape' from dual union all
select 37937,'037937','2011-070B','Centaur AV-028','Centaur AV-028',69122,'Deep Space','2011-11-27 00:00:00','2011-11-26 00:00:00',1222.65,22,88529,35.19,'Earth Escape' from dual union all
select 37938,'037938','2011-071A','Glonass-M No. 46','Uragan-M No. 746',69123,'In Earth orbit','','2011-12-05 00:00:00',677.86,19157,19210,64.82,'Medium' from dual union all
select 37939,'037939','2011-071B','Fregat No. 1046','Fregat 14S44 No. 1046',69123,'In Earth orbit','','2011-12-08 00:00:00',688.89,19243,19677,64.79,'Medium' from dual union all
select 3794,'003794','1968-091BB','deb Kosmos-249','',16818,'In Earth orbit','','1992-02-03 00:00:00',112.17,740,1913,62.25,'Intermediate' from dual union all
select 37940,'037940','2011-065G','deb Fobos-Grunt','',69117,'Reentered','2011-11-29 00:00:00','2011-11-29 00:00:00',89.59,239,270,51.31,'Intermediate' from dual union all
select 37941,'037941','2011-072A','Yaogan Weixing 13','Jianbing-7 02 xing',69124,'In Earth orbit','','2011-12-06 00:00:00',94.78,505,511,97.11,'Sun-Synch' from dual union all
select 37942,'037942','2011-072B','CZ-2C Stage 2','CZ-2C Stage 2',69124,'Reentered','2014-03-19 00:00:00','2011-12-06 00:00:00',93.79,407,513,97.02,'Sun-Synch' from dual union all
select 37943,'037943','2011-072C','Sep motor cover','',69124,'Reentered','2013-06-26 00:00:00','2011-12-06 00:00:00',96.18,488,664,97.26,'Sun-Synch' from dual union all
select 37944,'037944','2011-072D','Sep motor cover','',69124,'Reentered','2013-05-05 00:00:00','2011-12-06 00:00:00',95.98,490,642,96.85,'Sun-Synch' from dual union all
select 37945,'037945','2011-072E','Sep motor cover','',69124,'Reentered','2012-01-08 00:00:00','2011-12-06 00:00:00',92.97,345,495,97.27,'Sun-Synch' from dual union all
select 37946,'037946','2011-072F','Sep motor cover','',69124,'Reentered','2012-01-06 00:00:00','2011-12-06 00:00:00',92.76,332,488,96.87,'Sun-Synch' from dual union all
select 37947,'037947','2011-065H','deb Fobos-Grunt','',69117,'Reentered','2011-12-02 00:00:00','2011-11-30 00:00:00',89,192,259,51.42,'Intermediate' from dual union all
select 37948,'037948','2011-073A','Beidou DW 10','di shi ke beidou daohang',69125,'In Earth orbit','','2012-01-01 00:00:00',1436.04,35707,35863,55.16,'Synchronous' from dual union all
select 37949,'037949','2011-073B','CZ-3A Stage 3','CZ-3A Stage 3',69125,'Reentered','2012-07-07 00:00:00','2012-01-01 00:00:00',626.19,175,35560,55.35,'GEO Transfer' from dual union all
select 3795,'003795','1968-091BC','deb Kosmos-249 [*]','',16818,'Reentered','1980-05-21 00:00:00','1969-03-11 00:00:00',108.38,499,1805,62.45,'Intermediate' from dual union all
select 37950,'037950','2011-074A','Amos 5','Amos 5',69126,'In Earth orbit','','2012-01-10 00:00:00',1436.04,35763,35808,.07,'Stationary' from dual union all
select 37951,'037951','2011-074B','Luch-5A','Luch-5A',69126,'In Earth orbit','','2012-01-10 00:00:00',1435.98,35778,35791,4.83,'Inclined GEO' from dual union all
select 37952,'037952','2011-074C','Briz-M No. 99525','Briz-M 14S43 No. 99525',69126,'In Earth orbit','','2012-01-25 00:00:00',1606.64,35661,42463,1.12,'Very High' from dual union all
select 37953,'037953','2011-074D','Briz-M DTB','Briz-M torus tank',69126,'Reentered','2014-09-09 00:00:00','2012-01-10 00:00:00',636.6,388,35883,47.66,'GEO Transfer' from dual union all
select 37954,'037954','2011-075A','JSE reda-3 gouki','IGS Radar-3',69127,'In Earth orbit','','2011-12-12 00:00:00',94.47,473,513,97.42,'Sun-Synch' from dual union all
select 37955,'037955','2011-075B','H-2A F20 Stage 2','H-2A Stage 2',69127,'Reentered','2015-08-07 00:00:00','2011-12-12 00:00:00',94.47,473,513,97.42,'Sun-Synch' from dual union all
select 37956,'037956','1993-036BPF','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-14 00:00:00',99.49,688,781,74.05,'Intermediate' from dual union all
select 37957,'037957','1993-036BPG','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-15 00:00:00',100.21,750,786,74.03,'Intermediate' from dual union all
select 37958,'037958','1993-036BPH','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-10 00:00:00',99.88,737,769,74.04,'Intermediate' from dual union all
select 37959,'037959','1993-036BPJ','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-15 00:00:00',99.92,699,811,74.02,'Intermediate' from dual union all
select 3796,'003796','1968-091BD','deb Kosmos-249 [*]','',16818,'Reentered','1977-01-23 00:00:00','1969-04-01 00:00:00',110.05,536,1922,62.4,'Intermediate' from dual union all
select 37960,'037960','1993-036BPK','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-12 00:00:00',103.13,700,1113,74.06,'Intermediate' from dual union all
select 37961,'037961','1993-036BPL','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-13 00:00:00',100.2,725,811,74.05,'Intermediate' from dual union all
select 37962,'037962','1993-036BPM','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-14 00:00:00',99.55,633,842,74.08,'Intermediate' from dual union all
select 37963,'037963','1993-036BPN','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-12 00:00:00',102.67,766,1003,74.06,'Intermediate' from dual union all
select 37964,'037964','1993-036BPP','deb Kosmos-2251','',65138,'Reentered','2013-10-07 00:00:00','2011-12-07 00:00:00',96.4,480,693,74.05,'Intermediate' from dual union all
select 37965,'037965','1993-036BPQ','deb Kosmos-2251','',65138,'Reentered','2014-12-28 00:00:00','2011-12-14 00:00:00',98.65,641,748,74.05,'Intermediate' from dual union all
select 37966,'037966','1993-036BPR','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-14 00:00:00',99.75,725,769,74.01,'Intermediate' from dual union all
select 37967,'037967','1993-036BPS','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-14 00:00:00',101.81,690,999,74.03,'Intermediate' from dual union all
select 37968,'037968','1993-036BPT','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-08 00:00:00',98.41,573,792,74.01,'Intermediate' from dual union all
select 37969,'037969','1993-036BPU','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-13 00:00:00',102.84,744,1042,73.96,'Intermediate' from dual union all
select 3797,'003797','1968-091BE','deb Kosmos-249','',16818,'Reentered','1988-12-10 00:00:00','1988-12-09 00:00:00',90.44,287,306,62.36,'Intermediate' from dual union all
select 37970,'037970','1993-036BPV','deb Kosmos-2251','',65138,'Reentered','2015-06-21 00:00:00','2011-12-07 00:00:00',97.47,531,745,74.01,'Intermediate' from dual union all
select 37971,'037971','1993-036BPW','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-07 00:00:00',101.49,751,908,74,'Intermediate' from dual union all
select 37972,'037972','1993-036BPX','deb Kosmos-2251','',65138,'Reentered','2013-12-30 00:00:00','2011-12-15 00:00:00',97.06,501,734,74.07,'Intermediate' from dual union all
select 37973,'037973','1993-036BPY','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-10 00:00:00',100.72,759,826,74.04,'Intermediate' from dual union all
select 37974,'037974','1993-036BPZ','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-14 00:00:00',97.32,506,755,74.05,'Intermediate' from dual union all
select 37975,'037975','1993-036BQA','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-11 00:00:00',101.51,679,982,74.03,'Intermediate' from dual union all
select 37976,'037976','1993-036BQB','deb Kosmos-2251','',65138,'Reentered','2016-02-05 00:00:00','2011-12-12 00:00:00',99.25,679,767,74.05,'Intermediate' from dual union all
select 37977,'037977','1993-036BQC','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-11 00:00:00',100.75,761,828,73.99,'Intermediate' from dual union all
select 37978,'037978','1993-036BQD','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-14 00:00:00',99.21,642,800,74.07,'Intermediate' from dual union all
select 37979,'037979','1993-036BQE','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-15 00:00:00',99.59,523,954,74.05,'Intermediate' from dual union all
select 3798,'003798','1969-021F','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1970-05-02 00:00:00','1969-05-07 00:00:00',96.71,527,675,73.6,'Intermediate' from dual union all
select 37980,'037980','1993-036BQF','deb Kosmos-2251','',65138,'Reentered','2014-12-13 00:00:00','2011-12-11 00:00:00',100.11,712,815,74.03,'Intermediate' from dual union all
select 37981,'037981','1993-036BQG','deb Kosmos-2251','',65138,'In Earth orbit','','2011-11-29 00:00:00',100.74,755,833,74,'Intermediate' from dual union all
select 37982,'037982','1993-036BQH','deb Kosmos-2251','',65138,'Reentered','2014-04-01 00:00:00','2011-12-10 00:00:00',98.3,592,763,74.13,'Intermediate' from dual union all
select 37983,'037983','1993-036BQJ','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-15 00:00:00',99.54,710,763,74.03,'Intermediate' from dual union all
select 37984,'037984','1993-036BQK','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-11 00:00:00',99.76,677,818,73.9,'Intermediate' from dual union all
select 37985,'037985','1993-036BQL','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-15 00:00:00',100.38,761,793,74.03,'Intermediate' from dual union all
select 37986,'037986','1993-036BQM','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-14 00:00:00',100.23,754,785,74.04,'Intermediate' from dual union all
select 37987,'037987','1993-036BQN','deb Kosmos-2251','',65138,'Reentered','2014-04-06 00:00:00','2011-12-14 00:00:00',97.92,584,735,73.91,'Intermediate' from dual union all
select 37988,'037988','1993-036BQP','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-15 00:00:00',99.81,677,822,74.07,'Intermediate' from dual union all
select 37989,'037989','1993-036BQQ','deb Kosmos-2251','',65138,'Reentered','2012-05-08 00:00:00','2011-11-27 00:00:00',95.45,462,619,73.99,'Intermediate' from dual;
insert into SATELLITE
select 3799,'003799','1969-021G','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1969-11-26 00:00:00','1969-04-09 00:00:00',96.77,471,737,72.8,'Intermediate' from dual union all
select 37990,'037990','1993-036BQR','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-13 00:00:00',100.88,745,855,74.03,'Intermediate' from dual union all
select 37991,'037991','1993-036BQS','deb Kosmos-2251','',65138,'Reentered','2012-01-24 00:00:00','2011-12-09 00:00:00',94.62,455,546,73.9,'Intermediate' from dual union all
select 37992,'037992','1993-036BQT','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-14 00:00:00',100.41,760,796,74.04,'Intermediate' from dual union all
select 37993,'037993','1993-036BQU','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-13 00:00:00',100.46,752,809,74.04,'Intermediate' from dual union all
select 37994,'037994','1993-036BQV','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-15 00:00:00',100.51,764,802,74.04,'Intermediate' from dual union all
select 37995,'037995','1993-036BQW','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-12 00:00:00',99.17,675,763,73.98,'Intermediate' from dual union all
select 37996,'037996','1993-036BQX','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-09 00:00:00',105.35,758,1264,74.08,'Intermediate' from dual union all
select 37997,'037997','1993-036BQY','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-13 00:00:00',101.07,765,854,74.03,'Intermediate' from dual union all
select 37998,'037998','1993-036BQZ','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-12 00:00:00',100.45,763,797,74.04,'Intermediate' from dual union all
select 37999,'037999','1993-036BRA','deb Kosmos-2251','',65138,'Reentered','2018-03-11 00:00:00','2011-12-15 00:00:00',100.27,740,803,74.05,'Intermediate' from dual union all
select 38,'000038','60 Epsilon 5','Korabl''-Sputnik part','Vostok-1P part',6320,'Reentered','1960-10-01 00:00:00','1960-06-21 00:00:00',93.28,287,584,64.89,'Intermediate' from dual union all
select 380,'000380','62 A Pi 5','deb Sputnik','',8151,'Reentered','1962-08-30 00:00:00','1962-08-30 00:00:00',90.05,130,424,65.59,'Intermediate' from dual union all
select 3800,'003800','1969-021H','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1969-09-08 00:00:00','1969-04-07 00:00:00',96.37,454,716,72.68,'Intermediate' from dual union all
select 38000,'038000','1993-036BRB','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-10 00:00:00',100.69,749,834,74.04,'Intermediate' from dual union all
select 38001,'038001','1993-036BRC','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-15 00:00:00',99.01,643,779,74.01,'Intermediate' from dual union all
select 38002,'038002','1993-036BRD','deb Kosmos-2251','',65138,'Reentered','2014-10-05 00:00:00','2011-12-09 00:00:00',98.97,664,755,74.01,'Intermediate' from dual union all
select 38003,'038003','1993-036BRE','deb Kosmos-2251','',65138,'Reentered','2014-03-27 00:00:00','2011-12-11 00:00:00',98.97,685,733,74.01,'Intermediate' from dual union all
select 38004,'038004','1993-036BRF','deb Kosmos-2251','',65138,'In Earth orbit','','2011-12-15 00:00:00',100.35,753,798,74.04,'Intermediate' from dual union all
select 38005,'038005','1993-036BRG','deb Kosmos-2251','',65138,'Reentered','2013-11-11 00:00:00','2011-12-08 00:00:00',97.3,545,714,73.91,'Intermediate' from dual union all
select 38006,'038006','1993-036BRH','deb Kosmos-2251','',65138,'In Earth orbit','','2011-11-24 00:00:00',101.9,763,934,73.86,'Intermediate' from dual union all
select 38007,'038007','2011-076A','Elisa W11','Elisa 1',69128,'In Earth orbit','','2011-12-17 00:00:00',98.42,675,692,98.22,'Sun-Synch' from dual union all
select 38008,'038008','2011-076B','Elisa E24','Elisa 4',69128,'In Earth orbit','','2011-12-17 00:00:00',98.43,676,692,98.22,'Sun-Synch' from dual union all
select 38009,'038009','2011-076C','Elisa W23','Elisa 3',69128,'In Earth orbit','','2011-12-17 00:00:00',98.49,681,692,98.21,'Sun-Synch' from dual union all
select 3801,'003801','1969-021J','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1970-04-04 00:00:00','1969-04-08 00:00:00',96.89,499,721,73.19,'Intermediate' from dual union all
select 38010,'038010','2011-076D','Elisa E12','Elisa 2',69128,'In Earth orbit','','2011-12-17 00:00:00',98.41,674,691,98.22,'Sun-Synch' from dual union all
select 38011,'038011','2011-076E','Fasat-Charlie','SSOT',69128,'In Earth orbit','','2011-12-17 00:00:00',96.77,596,611,97.99,'Sun-Synch' from dual union all
select 38012,'038012','2011-076F','Pleiades 1a','Pleiades HR-1',69128,'In Earth orbit','','2011-12-17 00:00:00',98.43,676,692,98.22,'Sun-Synch' from dual union all
select 38013,'038013','2011-076G','ASAP-S','Elisa Dispenser',69128,'In Earth orbit','','2011-12-17 00:00:00',96.85,605,611,97.99,'Sun-Synch' from dual union all
select 38014,'038014','2011-077A','Nigcomsat 1R','Nigcomsat 1R',69129,'In Earth orbit','','2011-12-28 00:00:00',1436.09,35786,35787,.27,'Stationary' from dual union all
select 38015,'038015','2011-077B','CZ-3B Stage 3','CZ-3B Stage 3',69129,'In Earth orbit','','2011-12-28 00:00:00',750.03,226,41707,24.38,'Highly Ellip' from dual union all
select 38016,'038016','1997-051YC','deb Iridium 33','',66131,'In Earth orbit','','2011-12-14 00:00:00',101.07,759,860,86.41,'Polar' from dual union all
select 38017,'038017','1997-051YD','deb Iridium 33','',66131,'In Earth orbit','','2011-12-16 00:00:00',103.62,752,1108,86.35,'Polar' from dual union all
select 38018,'038018','1997-051YE','deb Iridium 33','',66131,'Lost','','2011-12-09 00:00:00',100.1,754,773,86.29,'Polar' from dual union all
select 38019,'038019','1997-051YF','deb Iridium 33','',66131,'In Earth orbit','','2011-12-09 00:00:00',101.96,755,947,86.42,'Polar' from dual union all
select 3802,'003802','1969-021K','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1970-05-02 00:00:00','1969-04-09 00:00:00',97.07,525,712,73.7,'Intermediate' from dual union all
select 38020,'038020','1997-051YG','deb Iridium 33','',66131,'In Earth orbit','','2011-12-17 00:00:00',103.3,746,1084,86.33,'Polar' from dual union all
select 38021,'038021','1997-051YH','deb Iridium 33','',66131,'Reentered','2014-04-23 00:00:00','2011-12-15 00:00:00',99.97,740,775,86.29,'Polar' from dual union all
select 38022,'038022','1997-051YJ','deb Iridium 33','',66131,'In Earth orbit','','2011-12-11 00:00:00',101.23,773,861,86.29,'Polar' from dual union all
select 38023,'038023','1997-051YK','deb Iridium 33','',66131,'In Earth orbit','','2011-12-19 00:00:00',102.36,756,985,86.44,'Polar' from dual union all
select 38024,'038024','1997-051YL','deb Iridium 33','',66131,'In Earth orbit','','2011-12-19 00:00:00',102.33,746,991,86.48,'Polar' from dual union all
select 38025,'038025','1997-051YM','deb Iridium 33','',66131,'In Earth orbit','','2011-12-19 00:00:00',103.22,742,1080,86.5,'Polar' from dual union all
select 38026,'038026','1997-051YN','deb Iridium 33','',66131,'Reentered','2015-05-27 00:00:00','2011-12-19 00:00:00',100.43,756,802,86.45,'Polar' from dual union all
select 38027,'038027','1997-051YP','deb Iridium 33','',66131,'In Earth orbit','','2011-12-19 00:00:00',100.13,759,771,86.3,'Polar' from dual union all
select 38028,'038028','1997-051YQ','deb Iridium 33','',66131,'In Earth orbit','','2011-12-13 00:00:00',100.23,746,793,86.33,'Polar' from dual union all
select 38029,'038029','1997-051YR','deb Iridium 33','',66131,'Reentered','2014-04-30 00:00:00','2011-12-16 00:00:00',99.71,736,754,86.36,'Polar' from dual union all
select 3803,'003803','1969-021L','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1969-06-11 00:00:00','1969-04-08 00:00:00',94.89,421,605,72.58,'Intermediate' from dual union all
select 38030,'038030','1997-051YS','deb Iridium 33','',66131,'In Earth orbit','','2011-12-20 00:00:00',101.95,758,944,86.4,'Polar' from dual union all
select 38031,'038031','1997-051YT','deb Iridium 33','',66131,'Reentered','2014-05-27 00:00:00','2011-12-10 00:00:00',99.02,681,743,86.47,'Polar' from dual union all
select 38032,'038032','1997-051YU','deb Iridium 33','',66131,'Reentered','2014-04-11 00:00:00','2011-12-17 00:00:00',99.28,669,780,86.33,'Polar' from dual union all
select 38033,'038033','1997-051YV','deb Iridium 33','',66131,'In Earth orbit','','2011-12-16 00:00:00',99.58,708,769,86.29,'Polar' from dual union all
select 38034,'038034','1997-051YW','deb Iridium 33','',66131,'Reentered','2016-12-14 00:00:00','2011-12-19 00:00:00',99.58,718,759,86.37,'Polar' from dual union all
select 38035,'038035','1997-051YX','deb Iridium 33','',66131,'Reentered','2015-01-06 00:00:00','2011-12-20 00:00:00',100.1,745,782,86.37,'Polar' from dual union all
select 38036,'038036','2011-078A','Soyuz TMA-03M','Soyuz 11F732A47 No. 703',69130,'Landed','2012-07-01 00:00:00','2011-12-22 00:00:00',90.9,279,359,51.64,'Intermediate' from dual union all
select 38037,'038037','2011-078B','11S510','11S510',69130,'Reentered','2011-12-24 00:00:00','2011-12-22 00:00:00',88.52,181,222,51.63,'Intermediate' from dual union all
select 38038,'038038','2011-079A','Zi Yuan 1-2C','ziyuan yihao 02C weixing',69131,'In Earth orbit','','2011-12-29 00:00:00',100.31,773,774,98.56,'Sun-Synch' from dual union all
select 38039,'038039','2011-079B','CZ-4B Stage 3','CZ-4B Stage 3',69131,'In Earth orbit','','2011-12-29 00:00:00',97.12,474,768,98.44,'Sun-Synch' from dual union all
select 3804,'003804','1969-021M','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1969-03-19 00:00:00','1969-03-14 00:00:00',91.39,281,405,74.28,'Intermediate' from dual union all
select 38040,'038040','2011-080A','Globalstar M084','Globalstar-2 FM13',69132,'In Earth orbit','','2012-01-04 00:00:00',103.46,919,926,52.01,'Intermediate' from dual union all
select 38041,'038041','2011-080B','Globalstar M080','Globalstar-2 FM14',69132,'In Earth orbit','','2012-01-04 00:00:00',103.46,919,926,52.01,'Intermediate' from dual union all
select 38042,'038042','2011-080C','Globalstar M082','Globalstar-2 FM20',69132,'In Earth orbit','','2012-01-04 00:00:00',103.43,916,925,52.01,'Intermediate' from dual union all
select 38043,'038043','2011-080D','Globalstar M092','Globalstar-2 FM16',69132,'In Earth orbit','','2012-01-04 00:00:00',103.43,916,925,52.01,'Intermediate' from dual union all
select 38044,'038044','2011-080E','Globalstar M090','Globalstar-2 FM17',69132,'In Earth orbit','','2012-01-04 00:00:00',103.42,915,926,52.01,'Intermediate' from dual union all
select 38045,'038045','2011-080F','Globalstar M086','Globalstar-2 FM18',69132,'In Earth orbit','','2012-01-04 00:00:00',103.45,918,926,52.01,'Intermediate' from dual union all
select 38046,'038046','2012-001A','Zi Yuan 3','ziyuan san hao',69256,'In Earth orbit','','2012-01-16 00:00:00',94.65,497,507,97.49,'Sun-Synch' from dual union all
select 38047,'038047','2012-001B','VesselSat2','AIS VesselSat 2',69256,'Reentered','2016-10-27 00:00:00','2012-01-16 00:00:00',94.5,489,500,97.49,'Sun-Synch' from dual union all
select 38048,'038048','2012-001C','CZ-4B Stage 3','CZ-4B Stage 3',69256,'Reentered','2012-03-07 00:00:00','2012-01-16 00:00:00',91.49,218,478,97.58,'Sun-Synch' from dual union all
select 38049,'038049','2012-002A','Feng Yun 2F','fengyun erhao 07 xing',69257,'In Earth orbit','','2012-01-19 00:00:00',1435.93,35757,35810,2.4,'Stationary' from dual union all
select 3805,'003805','1969-021N','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1969-09-04 00:00:00','1969-04-08 00:00:00',94.6,490,509,74.04,'Intermediate' from dual union all
select 38050,'038050','2012-002B','CZ-3A Stage 3','CZ-3A Stage 3',69257,'In Earth orbit','','2012-01-20 00:00:00',634.25,368,35782,24.36,'GEO Transfer' from dual union all
select 38051,'038051','2011-062C','Chibis-M','RS-39',69114,'Reentered','2014-10-15 00:00:00','2012-01-25 00:00:00',94.58,489,507,51.66,'Intermediate' from dual union all
select 38052,'038052','1993-036BRJ','deb Kosmos-2251','',65138,'Reentered','2016-05-06 00:00:00','2012-01-11 00:00:00',100.29,692,852,73.98,'Intermediate' from dual union all
select 38053,'038053','1993-036BRK','deb Kosmos-2251','',65138,'Reentered','2016-01-31 00:00:00','2012-01-12 00:00:00',99.53,719,754,74.05,'Intermediate' from dual union all
select 38054,'038054','1993-036BRL','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-13 00:00:00',107.26,788,1411,73.96,'Intermediate' from dual union all
select 38055,'038055','1993-036BRM','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-12 00:00:00',99.27,651,797,74.01,'Intermediate' from dual union all
select 38056,'038056','1993-036BRN','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-12 00:00:00',105.82,763,1303,73.92,'Intermediate' from dual union all
select 38057,'038057','1993-036BRP','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-13 00:00:00',103.03,760,1044,74,'Intermediate' from dual union all
select 38058,'038058','1993-036BRQ','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-12 00:00:00',107.26,752,1448,74.02,'Intermediate' from dual union all
select 38059,'038059','1993-036BRR','deb Kosmos-2251','',65138,'Reentered','2014-12-16 00:00:00','2012-01-11 00:00:00',98.06,597,735,73.99,'Intermediate' from dual union all
select 3806,'003806','1969-021P','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1969-06-29 00:00:00','1969-04-04 00:00:00',95.06,418,625,73.21,'Intermediate' from dual union all
select 38060,'038060','1993-036BRS','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-12 00:00:00',99.34,674,781,74.04,'Intermediate' from dual union all
select 38061,'038061','1993-036BRT','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-13 00:00:00',100.33,733,816,73.99,'Intermediate' from dual union all
select 38062,'038062','1993-036BRU','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-12 00:00:00',99.81,722,777,74.03,'Intermediate' from dual union all
select 38063,'038063','1993-036BRV','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-12 00:00:00',98.88,612,798,74.02,'Intermediate' from dual union all
select 38064,'038064','1993-036BRW','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-11 00:00:00',100.16,718,815,74.02,'Intermediate' from dual union all
select 38065,'038065','1993-036BRX','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-13 00:00:00',100.46,761,800,74.04,'Intermediate' from dual union all
select 38066,'038066','1993-036BRY','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-13 00:00:00',100.38,753,801,74.04,'Intermediate' from dual union all
select 38067,'038067','1993-036BRZ','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-12 00:00:00',99.28,666,782,74.02,'Intermediate' from dual union all
select 38068,'038068','1993-036BSA','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-12 00:00:00',100.64,770,807,74.02,'Intermediate' from dual union all
select 38069,'038069','1993-036BSB','deb Kosmos-2251','',65138,'In Earth orbit','','2012-01-13 00:00:00',101.99,766,940,74.19,'Intermediate' from dual union all
select 3807,'003807','1969-023A','Kosmos-271','Zenit-4 No. 52',18290,'Landed','1969-03-23 00:00:00','1969-03-18 00:00:00',89.62,194,318,65.4,'Intermediate' from dual union all
select 38070,'038070','2012-003A','WGS SV-4','Wideband Global Satcom 4',69258,'In Earth orbit','','2013-01-01 00:00:00',1436.11,35786,35788,.1,'Stationary' from dual union all
select 38071,'038071','2012-003B','Delta 358','DCSS-5 F07',69258,'In Earth orbit','','2012-01-29 00:00:00',1251.76,471,63721,23.88,'Highly Ellip' from dual union all
select 38072,'038072','2012-002C','FG-36 AKM','Shanxi AKM',69257,'In Earth orbit','','2012-03-06 00:00:00',1448.75,35813,36255,2.34,'Drift GEO' from dual union all
select 38073,'038073','2012-004A','Progress M-14M','Progress 7K-TGM No.414',69259,'Deorbited','2012-04-28 00:00:00','2012-01-27 00:00:00',89.76,234,292,51.64,'Intermediate' from dual union all
select 38074,'038074','2012-004B','11S510','11S510',69259,'Reentered','2012-01-28 00:00:00','2012-01-27 00:00:00',88.25,169,207,51.61,'Intermediate' from dual union all
select 38075,'038075','2012-005A','Navid-e Elm-o Sanat','Navid',69260,'Reentered','2012-04-02 00:00:00','2012-02-05 00:00:00',91,275,373,56.02,'Intermediate' from dual union all
select 38076,'038076','2012-005B','Safir Stage 2','',69260,'Reentered','2012-03-19 00:00:00','2012-02-05 00:00:00',90.91,270,369,56.02,'Intermediate' from dual union all
select 38077,'038077','2012-006A','LARES','LARES',69261,'In Earth orbit','','2012-02-15 00:00:00',114.73,1435,1452,69.48,'Intermediate' from dual union all
select 38078,'038078','2012-006B','ALMASat','ALMASat',69261,'In Earth orbit','','2012-02-15 00:00:00',102.52,316,1439,69.49,'Intermediate' from dual;
insert into SATELLITE
select 38079,'038079','2012-006C','e-st@r','E-STAR',69261,'Reentered','2015-01-16 00:00:00','2012-02-15 00:00:00',102.45,309,1441,69.49,'Intermediate' from dual union all
select 3808,'003808','1969-023B','11A57I','11A57I',18290,'Reentered','1969-03-22 00:00:00','1969-03-17 00:00:00',89.3,182,298,65.39,'Intermediate' from dual union all
select 38080,'038080','2012-006D','Goliat','Goliat',69261,'Reentered','2014-12-31 00:00:00','2012-02-15 00:00:00',102.45,312,1437,69.49,'Intermediate' from dual union all
select 38081,'038081','2012-006E','MagyarSat-OSCAR-72','MaSat-1',69261,'Reentered','2015-01-09 00:00:00','2012-02-15 00:00:00',102.46,311,1439,69.49,'Intermediate' from dual union all
select 38082,'038082','2012-006F','XaTcobeo','XaTcobeo',69261,'Reentered','2014-08-31 00:00:00','2012-02-15 00:00:00',102.44,308,1441,69.48,'Intermediate' from dual union all
select 38083,'038083','2012-006G','PW-Sat 1','PW-Sat 1',69261,'Reentered','2014-10-28 00:00:00','2012-02-15 00:00:00',102.46,308,1443,69.48,'Intermediate' from dual union all
select 38084,'038084','2012-006H','ROBUSTA','ROBUSTA',69261,'Reentered','2015-01-27 00:00:00','2012-02-15 00:00:00',102.45,311,1439,69.49,'Intermediate' from dual union all
select 38085,'038085','2012-006J','UNICubeSat GG','UNICubeSat',69261,'Reentered','2015-02-16 00:00:00','2012-02-17 00:00:00',102.45,311,1439,69.48,'Intermediate' from dual union all
select 38086,'038086','2012-006K','AVUM/LARES A&H/SS','LARES A&H/SS',69261,'Reentered Attached','2016-11-02 00:00:00','2012-02-13 00:00:00',114.73,1435,1452,69.49,'Intermediate' from dual union all
select 38087,'038087','2012-007A','SES-4','SES-4',69262,'In Earth orbit','','2012-02-29 00:00:00',1436.06,35784,35788,.1,'Stationary' from dual union all
select 38088,'038088','2012-007B','Briz-M No. 99526','Briz-M',69262,'In Earth orbit','','2012-02-23 00:00:00',648.44,3051,33826,25.1,'GEO Transfer' from dual union all
select 38089,'038089','2012-007C','Briz-M DTB','Briz-M torus tank',69262,'In Earth orbit','','2012-02-22 00:00:00',267.69,325,14437,49.6,'Highly Ellip' from dual union all
select 3809,'003809','1965-065N','deb NNS O-5','',11815,'Reentered','1982-11-12 00:00:00','1969-03-16 00:00:00',108.03,1086,1186,89.99,'Polar' from dual union all
select 38090,'038090','1998-067CL','MRM2 Strela base MLI cover','',66503,'Reentered','2012-02-23 00:00:00','2012-02-21 00:00:00',90.96,314,330,51.64,'Intermediate' from dual union all
select 38091,'038091','2012-008A','Beidou DW 11','di shiyi ke beidou d.w.',69263,'In Earth orbit','','2012-03-05 00:00:00',1436.14,35776,35799,1.87,'Stationary' from dual union all
select 38092,'038092','2012-008B','CZ-3C Stage 3','CZ-3C Stage 3',69263,'Reentered','2012-11-25 00:00:00','2012-03-03 00:00:00',630.61,150,35813,20.71,'GEO Transfer' from dual union all
select 38093,'038093','2012-009A','MUOS 1','MUOS SV 1',69264,'In Earth orbit','','2012-03-01 00:00:00',1436.27,35780,35800,2,'Stationary' from dual union all
select 38094,'038094','2012-009B','Centaur AV-030','Centaur AV-030',69264,'In Earth orbit','','2012-03-03 00:00:00',679.85,3259,35208,18.54,'GEO Transfer' from dual union all
select 38095,'038095','2012-008C','deb CZ-3C VEB','',69263,'In Earth orbit','','2012-03-03 00:00:00',637.01,238,36054,21.17,'GEO Transfer' from dual union all
select 38096,'038096','2012-010A','Edoardo Amaldi','ATV 3',69265,'Deorbited','2012-10-03 00:00:00','2012-03-25 00:00:00',90.08,270,286,51.64,'Intermediate' from dual union all
select 38097,'038097','2012-010B','EPS 553','EPS+ 553',69265,'Deorbited','2012-03-23 00:00:00','2012-03-23 00:00:00',89.67,250,267,51.63,'Intermediate' from dual union all
select 38098,'038098','2012-011A','Intelsat IS-22','Intelsat IS-22',69266,'In Earth orbit','','2012-04-26 00:00:00',1436.04,35783,35787,.06,'Stationary' from dual union all
select 38099,'038099','2012-011B','Briz-M No. 99537','Briz-M 14S43 No. 99537',69266,'In Earth orbit','','2012-04-09 00:00:00',1287.55,2365,63287,29.68,'Highly Ellip' from dual union all
select 381,'000381','62 A Tau 1','[AMS Venera]','2MV-1 No. 2',8155,'Reentered Attached','1962-09-06 00:00:00','1962-09-02 00:00:00',88.8,185,246,64.8,'Intermediate' from dual union all
select 3810,'003810','1965-065P','deb NNS O-5','',11815,'In Earth orbit','','1966-11-13 00:00:00',108.07,1085,1190,90.03,'Polar' from dual union all
select 38100,'038100','2012-011C','Briz-M DTB','Briz-M torus tank',69266,'In Earth orbit','','2012-04-01 00:00:00',267.85,367,14405,50.76,'Highly Ellip' from dual union all
select 38101,'038101','2012-012A','Kosmos-2479','Oko-1 No. 7128',69267,'In Earth orbit','','2012-04-01 00:00:00',1442.04,35898,35907,2.27,'Drift GEO' from dual union all
select 38102,'038102','2012-012B','8S812','8S812',69267,'Reentered','2012-04-01 00:00:00','2012-03-31 00:00:00',88.05,175,181,51.66,'Intermediate' from dual union all
select 38103,'038103','2012-012C','Perekhodnik','Sredniy perekhodnik',69267,'Reentered','2012-03-30 00:00:00','2012-03-30 00:00:00',88.15,179,186,51.65,'Intermediate' from dual union all
select 38104,'038104','2012-012D','Blok DM-2 No. 117L','11S861 No. 117L',69267,'In Earth orbit','','2012-04-01 00:00:00',1437.75,35749,35889,2.31,'Drift GEO' from dual union all
select 38105,'038105','2012-012E','SOZ','SOZ',69267,'In Earth orbit','','2012-06-25 00:00:00',635.34,259,35947,47.73,'GEO Transfer' from dual union all
select 38106,'038106','2012-012F','SOZ','SOZ',69267,'Reentered','2012-12-14 00:00:00','2012-06-27 00:00:00',632.09,203,35836,47.69,'GEO Transfer' from dual union all
select 38107,'038107','2012-013A','Apstar 7','Apstar7',69268,'In Earth orbit','','2012-05-21 00:00:00',1436.06,35783,35789,.04,'Stationary' from dual union all
select 38108,'038108','2012-013B','CZ-3B Stage 3','CZ-3B Stage 3',69268,'Reentered','2013-06-21 00:00:00','2012-04-02 00:00:00',923.01,155,49894,27.46,'Highly Ellip' from dual union all
select 38109,'038109','2012-014A','USA 234','TOPAZ 2',69269,'In Earth orbit','','2012-04-09 00:00:00',106.98,1080,1094,122.99,'Retrograde' from dual union all
select 3811,'003811','1968-114C','ESSA 8 despin weight','',16841,'In Earth orbit','','1969-03-16 00:00:00',112.89,1251,1467,101.94,'Sun-Synch' from dual union all
select 38110,'038110','1999-025EPF','deb FY-1C','',66706,'In Earth orbit','','2012-03-31 00:00:00',103.9,787,1099,98.15,'Sun-Synch' from dual union all
select 38111,'038111','1999-025EPG','deb FY-1C','',66706,'Reentered','2015-03-04 00:00:00','2012-04-10 00:00:00',97.46,483,792,99.23,'Sun-Synch' from dual union all
select 38112,'038112','1999-025EPH','deb FY-1C','',66706,'Reentered','2015-05-01 00:00:00','2012-04-09 00:00:00',95.61,534,563,98.25,'Sun-Synch' from dual union all
select 38113,'038113','1999-025EPJ','deb FY-1C','',66706,'In Earth orbit','','2012-04-07 00:00:00',103.27,801,1026,98.45,'Sun-Synch' from dual union all
select 38114,'038114','1999-025EPK','deb FY-1C','',66706,'In Earth orbit','','2012-03-13 00:00:00',102.92,773,1020,97.93,'Sun-Synch' from dual union all
select 38115,'038115','1999-025EPL','deb FY-1C','',66706,'In Earth orbit','','2012-03-22 00:00:00',105.52,749,1289,97.7,'Sun-Synch' from dual union all
select 38116,'038116','1999-025EPM','deb FY-1C','',66706,'In Earth orbit','','2012-04-04 00:00:00',102.14,802,918,98.96,'Sun-Synch' from dual union all
select 38117,'038117','1999-025EPN','deb FY-1C','',66706,'Reentered','2013-03-04 00:00:00','2012-04-09 00:00:00',97.52,496,784,97.32,'Sun-Synch' from dual union all
select 38118,'038118','1999-025EPP','deb FY-1C','',66706,'Reentered','2014-07-13 00:00:00','2012-04-08 00:00:00',99.71,693,797,99.16,'Sun-Synch' from dual union all
select 38119,'038119','1999-025EPQ','deb FY-1C','',66706,'In Earth orbit','','2012-03-30 00:00:00',106.24,749,1355,98.64,'Sun-Synch' from dual union all
select 3812,'003812','1968-114D','ESSA 8 despin weight','',16841,'In Earth orbit','','1969-03-16 00:00:00',116.39,1463,1574,101.81,'Sun-Synch' from dual union all
select 38120,'038120','1999-025EPR','deb FY-1C','',66706,'In Earth orbit','','2012-04-07 00:00:00',101.03,760,854,98.41,'Sun-Synch' from dual union all
select 38121,'038121','1999-025EPS','deb FY-1C','',66706,'In Earth orbit','','2012-04-03 00:00:00',102.39,694,1050,99.15,'Sun-Synch' from dual union all
select 38122,'038122','1999-025EPT','deb FY-1C','',66706,'In Earth orbit','','2012-04-01 00:00:00',100.8,605,988,98.75,'Sun-Synch' from dual union all
select 38123,'038123','1999-025EPU','deb FY-1C','',66706,'In Earth orbit','','2012-04-07 00:00:00',109.24,847,1537,100.55,'Sun-Synch' from dual union all
select 38124,'038124','1999-025EPV','deb FY-1C','',66706,'In Earth orbit','','2012-03-27 00:00:00',107.51,840,1383,97.81,'Sun-Synch' from dual union all
select 38125,'038125','1999-025EPW','deb FY-1C','',66706,'In Earth orbit','','2012-03-23 00:00:00',109.02,849,1513,100.5,'Sun-Synch' from dual union all
select 38126,'038126','1999-025EPX','deb FY-1C','',66706,'In Earth orbit','','2012-04-07 00:00:00',107.53,851,1374,98.72,'Sun-Synch' from dual union all
select 38127,'038127','1999-025EPY','deb FY-1C','',66706,'In Earth orbit','','2012-03-24 00:00:00',101,756,856,98.99,'Sun-Synch' from dual union all
select 38128,'038128','1999-025EPZ','deb FY-1C','',66706,'Reentered','2013-10-29 00:00:00','2012-04-06 00:00:00',99.33,634,819,97.5,'Sun-Synch' from dual union all
select 38129,'038129','1999-025EQA','deb FY-1C','',66706,'In Earth orbit','','2012-03-27 00:00:00',104.15,850,1060,97.99,'Sun-Synch' from dual union all
select 3813,'003813','1968-097BB','deb Kosmos-252','',16824,'Reentered','1981-11-03 00:00:00','1981-04-05 00:00:00',105.32,504,1515,62.11,'Intermediate' from dual union all
select 38130,'038130','1999-025EQB','deb FY-1C','',66706,'In Earth orbit','','2012-04-09 00:00:00',105.6,847,1198,98.6,'Sun-Synch' from dual union all
select 38131,'038131','1999-025EQC','deb FY-1C','',66706,'In Earth orbit','','2012-03-28 00:00:00',103.69,828,1038,97.91,'Sun-Synch' from dual union all
select 38132,'038132','1999-025EQD','deb FY-1C','',66706,'In Earth orbit','','2012-04-02 00:00:00',103.36,749,1086,99.21,'Sun-Synch' from dual union all
select 38133,'038133','1999-025EQE','deb FY-1C','',66706,'In Earth orbit','','2012-04-10 00:00:00',105.64,810,1239,98.3,'Sun-Synch' from dual union all
select 38134,'038134','1999-025EQF','deb FY-1C','',66706,'In Earth orbit','','2012-04-06 00:00:00',103.15,805,1010,98.48,'Sun-Synch' from dual union all
select 38135,'038135','1999-025EQG','deb FY-1C','',66706,'In Earth orbit','','2012-03-24 00:00:00',101.75,661,1021,98.83,'Sun-Synch' from dual union all
select 38136,'038136','1999-025EQH','deb FY-1C','',66706,'In Earth orbit','','2012-04-05 00:00:00',104.45,847,1091,98.58,'Sun-Synch' from dual union all
select 38137,'038137','1999-025EQJ','deb FY-1C','',66706,'In Earth orbit','','2012-04-08 00:00:00',102.67,852,918,99.03,'Sun-Synch' from dual union all
select 38138,'038138','1999-025EQK','deb FY-1C','',66706,'In Earth orbit','','2012-04-01 00:00:00',102.28,823,911,99.09,'Sun-Synch' from dual union all
select 38139,'038139','1999-025EQL','deb FY-1C','',66706,'In Earth orbit','','2012-04-09 00:00:00',101.06,783,835,99.13,'Sun-Synch' from dual union all
select 3814,'003814','1968-097BC','deb Kosmos-252','',16824,'In Earth orbit','','1969-03-16 00:00:00',115.87,762,2228,62.23,'Intermediate' from dual union all
select 38140,'038140','1999-025EQM','deb FY-1C','',66706,'In Earth orbit','','2012-03-23 00:00:00',101.58,783,884,98.9,'Sun-Synch' from dual union all
select 38141,'038141','1999-025EQN','deb FY-1C','',66706,'Reentered','2012-07-23 00:00:00','2012-03-15 00:00:00',94.77,423,591,98.37,'Sun-Synch' from dual union all
select 38142,'038142','1999-025EQP','deb FY-1C','',66706,'In Earth orbit','','2012-04-06 00:00:00',100.62,698,878,98.74,'Sun-Synch' from dual union all
select 38143,'038143','1999-025EQQ','deb FY-1C','',66706,'In Earth orbit','','2012-03-27 00:00:00',103.01,853,948,99.05,'Sun-Synch' from dual union all
select 38144,'038144','1999-025EQR','deb FY-1C','',66706,'In Earth orbit','','2012-04-08 00:00:00',106.73,843,1307,98.98,'Sun-Synch' from dual union all
select 38145,'038145','1999-025EQS','deb FY-1C','',66706,'In Earth orbit','','2012-04-06 00:00:00',106.76,742,1412,97.64,'Sun-Synch' from dual union all
select 38146,'038146','1999-025EQT','deb FY-1C','',66706,'In Earth orbit','','2012-04-01 00:00:00',100.77,585,1006,99.55,'Sun-Synch' from dual union all
select 38147,'038147','1999-025EQU','deb FY-1C','',66706,'In Earth orbit','','2012-03-31 00:00:00',101.78,802,883,98.94,'Sun-Synch' from dual union all
select 38148,'038148','1999-025EQV','deb FY-1C','',66706,'In Earth orbit','','2012-03-25 00:00:00',104.91,849,1131,99.05,'Sun-Synch' from dual union all
select 38149,'038149','1999-025EQW','deb FY-1C','',66706,'In Earth orbit','','2012-04-08 00:00:00',101.26,760,877,98.89,'Sun-Synch' from dual union all
select 3815,'003815','1968-097BD','deb Kosmos-252','',16824,'Reentered','1969-10-04 00:00:00','1969-09-28 00:00:00',92.72,321,495,62.11,'Intermediate' from dual union all
select 38150,'038150','1999-025EQX','deb FY-1C','',66706,'In Earth orbit','','2012-03-24 00:00:00',100.51,694,872,98.73,'Sun-Synch' from dual union all
select 38151,'038151','1999-025EQY','deb FY-1C','',66706,'Reentered','2014-03-06 00:00:00','2012-03-24 00:00:00',98.61,589,796,99.17,'Sun-Synch' from dual union all
select 38152,'038152','1999-025EQZ','deb FY-1C','',66706,'Reentered','2015-02-03 00:00:00','2012-04-04 00:00:00',99.34,628,827,99.08,'Sun-Synch' from dual union all
select 38153,'038153','1999-025ERA','deb FY-1C','',66706,'Reentered','2015-01-14 00:00:00','2012-03-30 00:00:00',99.41,628,833,99.06,'Sun-Synch' from dual union all
select 38154,'038154','1999-025ERB','deb FY-1C','',66706,'In Earth orbit','','2012-03-22 00:00:00',99.81,608,891,98.51,'Sun-Synch' from dual union all
select 38155,'038155','1999-025ERC','deb FY-1C','',66706,'In Earth orbit','','2012-04-08 00:00:00',103.4,819,1020,98.41,'Sun-Synch' from dual union all
select 38156,'038156','1999-025ERD','deb FY-1C','',66706,'In Earth orbit','','2012-04-09 00:00:00',99.79,639,858,99.05,'Sun-Synch' from dual union all
select 38157,'038157','1999-025ERE','deb FY-1C','',66706,'In Earth orbit','','2012-03-27 00:00:00',102.38,639,1104,99.69,'Sun-Synch' from dual union all
select 38158,'038158','1999-025ERF','deb FY-1C','',66706,'In Earth orbit','','2012-03-26 00:00:00',105.54,850,1190,99.77,'Sun-Synch' from dual union all
select 38159,'038159','1999-025ERG','deb FY-1C','',66706,'In Earth orbit','','2012-04-07 00:00:00',102.96,853,945,99.33,'Sun-Synch' from dual union all
select 3816,'003816','1968-097BE','deb Kosmos-252','',16824,'Reentered','1982-06-25 00:00:00','1969-03-16 00:00:00',118.42,533,2687,62.26,'Intermediate' from dual union all
select 38160,'038160','1999-025ERH','deb FY-1C','',66706,'In Earth orbit','','2012-04-05 00:00:00',102.46,847,903,99.1,'Sun-Synch' from dual union all
select 38161,'038161','1999-025ERJ','deb FY-1C','',66706,'Reentered','2013-04-27 00:00:00','2012-04-01 00:00:00',98.02,579,749,99.5,'Sun-Synch' from dual union all
select 38162,'038162','1999-025ERK','deb FY-1C','',66706,'In Earth orbit','','2012-04-09 00:00:00',103.42,722,1119,99.83,'Sun-Synch' from dual union all
select 38163,'038163','1999-025ERL','deb FY-1C','',66706,'In Earth orbit','','2012-04-09 00:00:00',103.07,838,970,99,'Sun-Synch' from dual union all
select 38164,'038164','1999-025ERM','deb FY-1C','',66706,'In Earth orbit','','2012-03-22 00:00:00',99.46,646,820,99.05,'Sun-Synch' from dual union all
select 38165,'038165','1999-025ERN','deb FY-1C','',66706,'In Earth orbit','','2012-03-30 00:00:00',101.63,812,860,99.08,'Sun-Synch' from dual union all
select 38166,'038166','1999-025ERP','deb FY-1C','',66706,'In Earth orbit','','2012-03-30 00:00:00',103.05,841,965,99.08,'Sun-Synch' from dual union all
select 38167,'038167','1999-025ERQ','deb FY-1C','',66706,'Reentered','2016-08-23 00:00:00','2012-04-04 00:00:00',99.92,649,861,98.82,'Sun-Synch' from dual union all
select 38168,'038168','1999-025ERR','deb FY-1C','',66706,'In Earth orbit','','2012-04-03 00:00:00',102.32,825,912,99.08,'Sun-Synch' from dual;
insert into SATELLITE
select 38169,'038169','1999-025ERS','deb FY-1C','',66706,'In Earth orbit','','2012-03-30 00:00:00',101.24,773,862,99,'Sun-Synch' from dual union all
select 3817,'003817','1968-097BF','deb Kosmos-252 [*]','',16824,'Reentered','1982-12-07 00:00:00','1982-12-06 00:00:00',91.12,313,346,62.76,'Intermediate' from dual union all
select 38170,'038170','1999-025ERT','deb FY-1C','',66706,'In Earth orbit','','2012-04-06 00:00:00',102.12,747,971,99.05,'Sun-Synch' from dual union all
select 38171,'038171','1999-025ERU','deb FY-1C','',66706,'In Earth orbit','','2012-03-26 00:00:00',100.34,506,1044,99.52,'Sun-Synch' from dual union all
select 38172,'038172','1999-025ERV','deb FY-1C','',66706,'In Earth orbit','','2012-03-23 00:00:00',106.7,855,1292,100.43,'Sun-Synch' from dual union all
select 38173,'038173','1999-025ERW','deb FY-1C','',66706,'In Earth orbit','','2012-04-06 00:00:00',101.7,685,994,99.05,'Sun-Synch' from dual union all
select 38174,'038174','2010-047D','Sep motor cover','',68907,'In Earth orbit','','2012-04-11 00:00:00',101.1,651,970,97.89,'Sun-Synch' from dual union all
select 38175,'038175','2010-047E','Sep motor cover','',68907,'In Earth orbit','','2012-04-11 00:00:00',101,649,963,97.99,'Sun-Synch' from dual union all
select 38176,'038176','2011-068D','CZ-2D sep motor cover','',69120,'In Earth orbit','','2012-04-10 00:00:00',100.42,686,871,98.65,'Sun-Synch' from dual union all
select 38177,'038177','2011-068E','CZ-2D sep motor cover','',69120,'In Earth orbit','','2012-04-10 00:00:00',100.55,680,890,98.42,'Sun-Synch' from dual union all
select 38178,'038178','2011-068F','SW-4 lower debris','',69120,'In Earth orbit','','2012-04-09 00:00:00',100.93,703,902,98.38,'Sun-Synch' from dual union all
select 38179,'038179','2011-068G','CZ-2D sep motor cover','',69120,'In Earth orbit','','2012-04-08 00:00:00',101.79,741,946,98.65,'Sun-Synch' from dual union all
select 3818,'003818','1969-024A','Kosmos-272','Sfera',18291,'In Earth orbit','','1969-08-02 00:00:00',109.3,1179,1209,73.99,'Intermediate' from dual union all
select 38180,'038180','2011-068H','CZ-2D sep motor cover','',69120,'In Earth orbit','','2012-04-04 00:00:00',101.73,730,952,98.57,'Sun-Synch' from dual union all
select 38181,'038181','1966-040AZ','Nimbus 2 debris','',13447,'In Earth orbit','','2012-04-09 00:00:00',107.36,1059,1150,100.64,'Sun-Synch' from dual union all
select 38182,'038182','1966-040BA','Nimbus 2 debris','',13447,'In Earth orbit','','2012-04-06 00:00:00',107.78,1085,1163,100.58,'Sun-Synch' from dual union all
select 38183,'038183','1966-040BB','Nimbus 2 debris','',13447,'In Earth orbit','','2012-04-09 00:00:00',107.94,1087,1176,100.48,'Sun-Synch' from dual union all
select 38184,'038184','2010-051E','CZ-4B sep motor cover','',68911,'In Earth orbit','','2012-04-10 00:00:00',99.39,596,863,97.75,'Sun-Synch' from dual union all
select 38185,'038185','2010-051F','CZ-4B sep motor cover','',68911,'Reentered','2014-02-01 00:00:00','2012-04-11 00:00:00',95.58,482,612,97.66,'Sun-Synch' from dual union all
select 38186,'038186','2010-051G','CZ-4B sep motor cover','',68911,'Reentered','2015-02-17 00:00:00','2012-04-10 00:00:00',96.44,514,663,97.71,'Sun-Synch' from dual union all
select 38187,'038187','2011-069D','deb Briz-M','',69121,'In Earth orbit','','2012-04-14 00:00:00',837.32,11813,34285,.56,'Medium' from dual union all
select 38188,'038188','1993-036BSC','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-18 00:00:00',100.75,754,835,74.02,'Intermediate' from dual union all
select 38189,'038189','1993-036BSD','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-18 00:00:00',100.03,742,777,74.04,'Intermediate' from dual union all
select 3819,'003819','1969-024B','S3M','S3M',18291,'In Earth orbit','','1969-07-03 00:00:00',109.18,1180,1198,74.01,'Intermediate' from dual union all
select 38190,'038190','1993-036BSE','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-17 00:00:00',100.2,751,785,74.05,'Intermediate' from dual union all
select 38191,'038191','1993-036BSF','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-16 00:00:00',100.86,756,843,74.02,'Intermediate' from dual union all
select 38192,'038192','1993-036BSG','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-16 00:00:00',100,735,782,74.03,'Intermediate' from dual union all
select 38193,'038193','1993-036BSH','deb Kosmos-2251','',65138,'Reentered','2013-03-27 00:00:00','2012-03-20 00:00:00',94.64,377,626,73.93,'Intermediate' from dual union all
select 38194,'038194','1993-036BSJ','deb Kosmos-2251','',65138,'Reentered','2012-12-14 00:00:00','2012-04-17 00:00:00',96.05,513,626,73.87,'Intermediate' from dual union all
select 38195,'038195','1993-036BSK','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-14 00:00:00',100.41,726,829,73.84,'Intermediate' from dual union all
select 38196,'038196','1993-036BSL','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-17 00:00:00',101.68,643,1034,73.9,'Intermediate' from dual union all
select 38197,'038197','1993-036BSM','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-11 00:00:00',101.85,645,1048,73.97,'Intermediate' from dual union all
select 38198,'038198','1993-036BSN','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-01 00:00:00',99.59,697,780,74.05,'Intermediate' from dual union all
select 38199,'038199','1993-036BSP','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-18 00:00:00',105.94,753,1323,73.96,'Intermediate' from dual union all
select 382,'000382','62 A Tau 2','8K78I','8K78I',8155,'Reentered','1962-09-03 00:00:00','1962-09-02 00:00:00',87.76,164,164,64.72,'Intermediate' from dual union all
select 3820,'003820','1969-021Q','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1969-04-07 00:00:00','1969-03-17 00:00:00',92.45,305,484,74.13,'Intermediate' from dual union all
select 38200,'038200','1993-036BSQ','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-08 00:00:00',98.94,572,844,74.09,'Intermediate' from dual union all
select 38201,'038201','1993-036BSR','deb Kosmos-2251','',65138,'Reentered','2014-01-14 00:00:00','2012-04-08 00:00:00',98.42,631,735,74,'Intermediate' from dual union all
select 38202,'038202','1993-036BSS','deb Kosmos-2251','',65138,'Reentered','2014-02-12 00:00:00','2012-04-09 00:00:00',97.39,539,729,74,'Intermediate' from dual union all
select 38203,'038203','1993-036BST','deb Kosmos-2251','',65138,'Reentered','2014-07-07 00:00:00','2012-04-15 00:00:00',97.88,591,724,73.95,'Intermediate' from dual union all
select 38204,'038204','1993-036BSU','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-15 00:00:00',101.35,740,906,73.99,'Intermediate' from dual union all
select 38205,'038205','1993-036BSV','deb Kosmos-2251','',65138,'Reentered','2017-01-27 00:00:00','2012-04-07 00:00:00',98.81,546,858,73.98,'Intermediate' from dual union all
select 38206,'038206','1993-036BSW','deb Kosmos-2251','',65138,'Reentered','2017-07-05 00:00:00','2012-04-17 00:00:00',100.8,617,976,74.17,'Intermediate' from dual union all
select 38207,'038207','1993-036BSX','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-18 00:00:00',100.68,758,824,74.01,'Intermediate' from dual union all
select 38208,'038208','1993-036BSY','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-14 00:00:00',100.34,760,790,74.01,'Intermediate' from dual union all
select 38209,'038209','1993-036BSZ','deb Kosmos-2251','',65138,'Reentered','2015-01-13 00:00:00','2012-04-15 00:00:00',98.2,582,764,74.14,'Intermediate' from dual union all
select 3821,'003821','1969-021R','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1969-05-17 00:00:00','1969-04-06 00:00:00',94.57,333,663,71.97,'Intermediate' from dual union all
select 38210,'038210','1993-036BTA','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-16 00:00:00',99.65,717,766,74.02,'Intermediate' from dual union all
select 38211,'038211','1993-036BTB','deb Kosmos-2251','',65138,'Reentered','2015-07-07 00:00:00','2012-04-18 00:00:00',100.25,750,790,74.04,'Intermediate' from dual union all
select 38212,'038212','1993-036BTC','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-17 00:00:00',98.57,601,780,74.06,'Intermediate' from dual union all
select 38213,'038213','1993-036BTD','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-12 00:00:00',100.25,751,789,74.04,'Intermediate' from dual union all
select 38214,'038214','1993-036BTE','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-15 00:00:00',99.45,717,748,74.03,'Intermediate' from dual union all
select 38215,'038215','1993-036BTF','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-18 00:00:00',100.41,760,796,74.04,'Intermediate' from dual union all
select 38216,'038216','1993-036BTG','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-16 00:00:00',100.16,743,789,74.04,'Intermediate' from dual union all
select 38217,'038217','1993-036BTH','deb Kosmos-2251','',65138,'Reentered','2013-05-02 00:00:00','2012-04-17 00:00:00',97.68,615,681,74.02,'Intermediate' from dual union all
select 38218,'038218','1993-036BTJ','deb Kosmos-2251','',65138,'Reentered','2017-11-05 00:00:00','2012-04-11 00:00:00',100.03,740,780,74.03,'Intermediate' from dual union all
select 38219,'038219','1993-036BTK','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-13 00:00:00',99.05,638,788,73.66,'Intermediate' from dual union all
select 3822,'003822','1969-021S','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1969-04-09 00:00:00','1969-03-17 00:00:00',93.34,396,480,74.41,'Intermediate' from dual union all
select 38220,'038220','1993-036BTL','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-15 00:00:00',100.34,758,792,74.03,'Intermediate' from dual union all
select 38221,'038221','1993-036BTM','deb Kosmos-2251','',65138,'In Earth orbit','','2012-04-16 00:00:00',99.8,660,837,74.03,'Intermediate' from dual union all
select 38222,'038222','2012-015A','Progress M-15M','Progress 7K-TGM No.415',69270,'Deorbited','2012-08-20 00:00:00','2012-04-21 00:00:00',90.05,252,302,51.65,'Intermediate' from dual union all
select 38223,'038223','2012-015B','11S510','11S510',69270,'Reentered','2012-04-22 00:00:00','2012-04-21 00:00:00',88.32,175,208,51.62,'Intermediate' from dual union all
select 38224,'038224','1997-051YY','deb Iridium 33','',66131,'In Earth orbit','','2012-04-19 00:00:00',102.17,758,965,86.41,'Polar' from dual union all
select 38225,'038225','1997-051YZ','deb Iridium 33','',66131,'In Earth orbit','','2012-04-19 00:00:00',103.7,771,1095,86.37,'Polar' from dual union all
select 38226,'038226','1997-051ZA','deb Iridium 33','',66131,'In Earth orbit','','2012-04-04 00:00:00',100.5,775,789,86.39,'Polar' from dual union all
select 38227,'038227','1997-051ZB','deb Iridium 33','',66131,'In Earth orbit','','2012-04-17 00:00:00',100.14,758,773,86.33,'Polar' from dual union all
select 38228,'038228','1997-051ZC','deb Iridium 33','',66131,'In Earth orbit','','2012-04-10 00:00:00',101.33,771,872,86.3,'Polar' from dual union all
select 38229,'038229','1997-051ZD','deb Iridium 33','',66131,'In Earth orbit','','2012-04-16 00:00:00',104.26,774,1146,86.38,'Polar' from dual union all
select 3823,'003823','1969-025A','OV1-17','OV1-17S',18292,'Reentered','1970-03-05 00:00:00','1969-05-05 00:00:00',92.85,386,443,99.17,'Sun-Synch' from dual union all
select 38230,'038230','1997-051ZE','deb Iridium 33','',66131,'Reentered','2014-02-19 00:00:00','2012-04-18 00:00:00',99.32,721,732,86.4,'Polar' from dual union all
select 38231,'038231','1997-051ZF','deb Iridium 33','',66131,'In Earth orbit','','2012-04-05 00:00:00',101.03,743,872,86.27,'Polar' from dual union all
select 38232,'038232','1997-051ZG','deb Iridium 33','',66131,'In Earth orbit','','2012-04-15 00:00:00',102.03,768,942,86.48,'Polar' from dual union all
select 38233,'038233','1997-051ZH','deb Iridium 33','',66131,'In Earth orbit','','2012-04-08 00:00:00',103.11,768,1044,86.24,'Polar' from dual union all
select 38234,'038234','1997-051ZJ','deb Iridium 33','',66131,'In Earth orbit','','2012-04-10 00:00:00',100.9,750,852,86.47,'Polar' from dual union all
select 38235,'038235','1997-051ZK','deb Iridium 33','',66131,'In Earth orbit','','2012-04-11 00:00:00',100.33,762,787,86.41,'Polar' from dual union all
select 38236,'038236','1997-051ZL','deb Iridium 33','',66131,'In Earth orbit','','2012-04-16 00:00:00',101.4,763,887,86.25,'Polar' from dual union all
select 38237,'038237','1997-051ZM','deb Iridium 33','',66131,'In Earth orbit','','2012-04-17 00:00:00',100.35,757,794,86.39,'Polar' from dual union all
select 38238,'038238','1997-051ZN','deb Iridium 33','',66131,'Reentered','2013-04-10 00:00:00','2012-04-18 00:00:00',97.84,620,691,86.3,'Polar' from dual union all
select 38239,'038239','1997-051ZP','deb Iridium 33','',66131,'Reentered','2015-03-14 00:00:00','2012-04-20 00:00:00',98.86,684,724,86.4,'Polar' from dual union all
select 3824,'003824','1969-025B','OV1-18','OV1-18S',18292,'Reentered','1972-08-28 00:00:00','1969-04-17 00:00:00',95.02,461,578,98.87,'Sun-Synch' from dual union all
select 38240,'038240','1997-051ZQ','deb Iridium 33','',66131,'Reentered','2014-10-16 00:00:00','2012-04-20 00:00:00',99.37,696,761,86.28,'Polar' from dual union all
select 38241,'038241','1997-051ZR','deb Iridium 33','',66131,'In Earth orbit','','2012-04-20 00:00:00',100.23,766,773,86.33,'Polar' from dual union all
select 38242,'038242','1997-051ZS','deb Iridium 33','',66131,'Reentered','2014-04-02 00:00:00','2012-04-13 00:00:00',100.17,703,831,86.35,'Polar' from dual union all
select 38243,'038243','1997-051ZT','deb Iridium 33','',66131,'In Earth orbit','','2012-04-13 00:00:00',100.33,755,794,86.38,'Polar' from dual union all
select 38244,'038244','1997-051ZU','deb Iridium 33','',66131,'In Earth orbit','','2012-04-17 00:00:00',99.91,749,760,86.45,'Polar' from dual union all
select 38245,'038245','2012-016A','Yahsat 1B','Yahsat 1B',69271,'In Earth orbit','','2012-05-03 00:00:00',1436.5,35785,35804,.01,'Drift GEO' from dual union all
select 38246,'038246','2012-016B','Briz-M DTB','Briz-M torus tank',69271,'In Earth orbit','','2012-05-01 00:00:00',271.5,329,14692,49.57,'Highly Ellip' from dual union all
select 38247,'038247','2012-016C','Briz-M No. 99529','Briz-M 14S43 No. 99529',69271,'In Earth orbit','','2012-05-01 00:00:00',654.57,3286,33903,24.19,'GEO Transfer' from dual union all
select 38248,'038248','2012-017A','RISAT-1','RISAT-1',69272,'In Earth orbit','','2012-04-28 00:00:00',95,508,530,97.6,'Sun-Synch' from dual union all
select 38249,'038249','2012-017B','PSLV-C19 PS4','PS4',69272,'Reentered','2018-04-03 00:00:00','2012-04-28 00:00:00',94.58,467,529,97.53,'Sun-Synch' from dual union all
select 3825,'003825','1969-025C','OV1-19','OV1-19S',18292,'In Earth orbit','','1969-05-11 00:00:00',153.57,467,5776,104.77,'Medium' from dual union all
select 38250,'038250','2012-018A','Beidou DW 12','di shier ke beidou-2 w.',69273,'In Earth orbit','','2013-01-11 00:00:00',773.21,21455,21600,55.26,'Medium' from dual union all
select 38251,'038251','2012-018B','Beidou DW 13','di shisan ke beidou-2 w.',69273,'In Earth orbit','','2013-01-10 00:00:00',773.21,21446,21609,55.19,'Medium' from dual union all
select 38252,'038252','2012-018C','CZ-3B payload adapter','Dual Sat Serial ESS',69273,'In Earth orbit','','2012-05-09 00:00:00',369.91,146,21237,55.06,'Highly Ellip' from dual union all
select 38253,'038253','2012-018D','CZ-3B Stage 3','CZ-3B Stage 3',69273,'Reentered','2017-08-18 00:00:00','2012-05-09 00:00:00',375.15,204,21501,55.05,'Highly Ellip' from dual union all
select 38254,'038254','2012-019A','AEHF SV-2','AEHF SV-2',69274,'In Earth orbit','','2013-01-01 00:00:00',1436.27,35780,35800,0,'Stationary' from dual union all
select 38255,'038255','2012-019B','Centaur AV-031','Centaur AV-031',69274,'Reentered','2012-07-31 00:00:00','2012-05-07 00:00:00',930.83,97,50306,20.38,'Highly Ellip' from dual union all
select 38256,'038256','2012-020A','Tianhui 1-02','tianhui yi hao 02 xing',69275,'In Earth orbit','','2012-05-06 00:00:00',94.57,490,506,97.37,'Sun-Synch' from dual union all
select 38257,'038257','2012-021A','Yaogan Weixing 14','Jiabing-11 01 xing',69276,'In Earth orbit','','2012-05-12 00:00:00',94.01,469,472,97.24,'Sun-Synch' from dual union all
select 38258,'038258','2012-021B','Tiantuo 1','tiantuo yihao weixing',69276,'Reentered','2014-11-03 00:00:00','2012-05-12 00:00:00',93.98,468,471,97.24,'Sun-Synch' from dual;
insert into SATELLITE
select 38259,'038259','2012-021C','CZ-4B Stage 3','CZ-4B Stage 3',69276,'Reentered','2013-02-27 00:00:00','2012-05-12 00:00:00',92.54,327,471,97.17,'Sun-Synch' from dual union all
select 3826,'003826','1969-025D','OV1-17A','ORBIS CAL 2',18292,'Reentered Attached','1969-03-24 00:00:00','1969-03-20 00:00:00',89.87,176,360,99.05,'Sun-Synch' from dual union all
select 38260,'038260','2012-021D','CZ-4B sep motor cover','',69276,'Reentered','2014-02-28 00:00:00','2012-05-15 00:00:00',96.42,452,723,97.29,'Sun-Synch' from dual union all
select 38261,'038261','2012-021E','CZ-4B sep motor cover','',69276,'Reentered','2013-06-14 00:00:00','2012-05-14 00:00:00',95.39,440,635,97.38,'Sun-Synch' from dual union all
select 38262,'038262','2007-019B','CZ-2D sep motor cover','',68361,'In Earth orbit','','2012-05-12 00:00:00',100.69,651,931,97.81,'Sun-Synch' from dual union all
select 38263,'038263','2007-019C','CZ-2D sep motor cover','',68361,'In Earth orbit','','2012-05-12 00:00:00',101.36,652,994,97.91,'Sun-Synch' from dual union all
select 38264,'038264','2008-061C','CZ-2D sep motor cover','',68569,'In Earth orbit','','2012-05-07 00:00:00',101.87,646,1048,97.92,'Sun-Synch' from dual union all
select 38265,'038265','2009-069B','CZ-2D sep motor cover','',68750,'In Earth orbit','','2012-05-08 00:00:00',101.62,652,1018,97.8,'Sun-Synch' from dual union all
select 38266,'038266','2009-069C','CZ-2D sep motor cover','',68750,'In Earth orbit','','2012-05-12 00:00:00',101.95,651,1051,97.94,'Sun-Synch' from dual union all
select 38267,'038267','2010-040B','CZ-2D sep motor cover','',68900,'Reentered','2017-09-30 00:00:00','2012-05-10 00:00:00',97.73,496,805,97.35,'Sun-Synch' from dual union all
select 38268,'038268','2010-040C','CZ-2D sep motor cover','',68900,'Reentered','2015-12-05 00:00:00','2012-05-11 00:00:00',97.22,495,757,97.28,'Sun-Synch' from dual union all
select 38269,'038269','2012-020B','CZ-2D sep motor cover','',69275,'In Earth orbit','','2012-05-09 00:00:00',98.02,505,823,97.42,'Sun-Synch' from dual union all
select 3827,'003827','1969-025E','OV1-19P','FW4S',18292,'In Earth orbit','','1969-05-13 00:00:00',153.35,467,5758,104.75,'Medium' from dual union all
select 38270,'038270','2012-020C','CZ-2D sep motor cover','',69275,'In Earth orbit','','2012-05-12 00:00:00',98.3,504,851,97.31,'Sun-Synch' from dual union all
select 38271,'038271','1979-095BR','deb Meteor-2','',47579,'In Earth orbit','','2012-05-08 00:00:00',102.25,861,869,81.21,'Intermediate' from dual union all
select 38272,'038272','1979-095BS','deb Meteor-2','',47579,'In Earth orbit','','2012-05-11 00:00:00',102.24,854,876,81.2,'Intermediate' from dual union all
select 38273,'038273','1979-095BT','deb Meteor-2','',47579,'In Earth orbit','','2012-05-11 00:00:00',102.19,856,869,81.21,'Intermediate' from dual union all
select 38274,'038274','1979-095BU','deb Meteor-2','',47579,'In Earth orbit','','2012-05-08 00:00:00',102.15,843,877,81.2,'Intermediate' from dual union all
select 38275,'038275','1979-095BV','deb Meteor-2','',47579,'In Earth orbit','','2012-05-08 00:00:00',102.14,855,865,81.21,'Intermediate' from dual union all
select 38276,'038276','1979-095BW','deb Meteor-2','',47579,'In Earth orbit','','2012-05-10 00:00:00',102.3,862,872,81.21,'Intermediate' from dual union all
select 38277,'038277','1979-095BX','deb Meteor-2','',47579,'In Earth orbit','','2012-04-20 00:00:00',102.18,858,866,81.2,'Intermediate' from dual union all
select 38278,'038278','1979-095BY','deb Meteor-2','',47579,'In Earth orbit','','2012-05-11 00:00:00',102.28,866,868,81.22,'Intermediate' from dual union all
select 38279,'038279','1979-095BZ','deb Meteor-2','',47579,'In Earth orbit','','2012-05-07 00:00:00',102,846,861,81.19,'Intermediate' from dual union all
select 3828,'003828','1969-025F','OV1-18P','FW4S',18292,'Reentered','1979-03-16 00:00:00','1969-04-17 00:00:00',95.05,462,580,98.87,'Sun-Synch' from dual union all
select 38280,'038280','1979-095CA','deb Meteor-2','',47579,'In Earth orbit','','2012-05-09 00:00:00',102.2,860,866,81.21,'Intermediate' from dual union all
select 38281,'038281','1988-005AJ','deb Meteor-2','',61254,'In Earth orbit','','2012-05-12 00:00:00',103.77,923,951,82.54,'Intermediate' from dual union all
select 38282,'038282','1988-005AK','deb Meteor-2','',61254,'In Earth orbit','','2012-05-11 00:00:00',103.74,926,944,82.53,'Intermediate' from dual union all
select 38283,'038283','1988-005AL','deb Meteor-2','',61254,'In Earth orbit','','2012-04-27 00:00:00',103.75,921,951,82.55,'Intermediate' from dual union all
select 38284,'038284','1988-005AM','deb Meteor-2','',61254,'In Earth orbit','','2012-04-19 00:00:00',103.81,930,948,82.54,'Intermediate' from dual union all
select 38285,'038285','1988-005AN','deb Meteor-2','',61254,'In Earth orbit','','2012-05-06 00:00:00',103.85,936,945,82.55,'Intermediate' from dual union all
select 38286,'038286','1988-005AP','deb Meteor-2','',61254,'In Earth orbit','','2012-05-11 00:00:00',103.8,910,967,82.53,'Intermediate' from dual union all
select 38287,'038287','1988-005AQ','deb Meteor-2','',61254,'In Earth orbit','','2012-04-30 00:00:00',103.82,923,956,82.54,'Intermediate' from dual union all
select 38288,'038288','1981-043W','deb Meteor-2','',52566,'In Earth orbit','','2012-05-09 00:00:00',101.97,819,885,81.27,'Intermediate' from dual union all
select 38289,'038289','1982-025BF','deb Meteor-2','',54334,'In Earth orbit','','2012-05-01 00:00:00',102.71,874,900,82.55,'Intermediate' from dual union all
select 3829,'003829','1969-026A','OPS 3722','CORONA J-43',18293,'Reentered Attached','1969-03-24 00:00:00','1969-03-21 00:00:00',88.65,177,239,83.04,'Intermediate' from dual union all
select 38290,'038290','1982-116J','deb Meteor-2','',54425,'In Earth orbit','','2012-05-09 00:00:00',101.35,774,872,81.25,'Intermediate' from dual union all
select 38291,'038291','2012-022A','Soyuz TMA-04M','Soyuz 11F732A47 No. 705',69277,'Landed','2012-09-17 00:00:00','2012-05-15 00:00:00',89.78,244,283,51.64,'Intermediate' from dual union all
select 38292,'038292','2012-022B','11S510','11S510',69277,'Reentered','2012-05-17 00:00:00','2012-05-15 00:00:00',88.54,191,213,51.65,'Intermediate' from dual union all
select 38293,'038293','2010-059C','deb CZ-4C','',68919,'In Earth orbit','','2012-05-14 00:00:00',100.82,790,805,98.77,'Sun-Synch' from dual union all
select 38294,'038294','2010-059D','CZ-4C sep motor cover','',68919,'In Earth orbit','','2012-05-14 00:00:00',105.08,800,1197,98.72,'Sun-Synch' from dual union all
select 38295,'038295','2010-059E','CZ-4C sep motor cover','',68919,'In Earth orbit','','2012-05-14 00:00:00',105.25,803,1209,98.73,'Sun-Synch' from dual union all
select 38296,'038296','2010-059F','CZ-4C sep motor cover','',68919,'In Earth orbit','','2012-05-15 00:00:00',104.79,802,1168,98.79,'Sun-Synch' from dual union all
select 38297,'038297','2010-059G','CZ-4C sep motor cover','',68919,'In Earth orbit','','2012-05-14 00:00:00',104.76,803,1164,98.76,'Sun-Synch' from dual union all
select 38298,'038298','2010-027C','CZ-2D sep motor cover','',68887,'In Earth orbit','','2012-05-14 00:00:00',99.42,567,894,97.68,'Sun-Synch' from dual union all
select 38299,'038299','2010-027D','CZ-2D sep motor cover','',68887,'In Earth orbit','','2012-05-15 00:00:00',99.53,569,904,97.81,'Sun-Synch' from dual union all
select 383,'000383','62 A Tau 3','BOZ','BOZ',8155,'Reentered','1962-10-01 00:00:00','1962-09-10 00:00:00',89.69,192,327,64.68,'Intermediate' from dual union all
select 3830,'003830','1969-026B','TIVOLI II','P-11 No. 4418',18293,'Reentered','1971-12-06 00:00:00','1969-04-20 00:00:00',94.68,501,506,83.09,'Intermediate' from dual union all
select 38300,'038300','2010-027E','CZ-2D sep motor cover','',68887,'In Earth orbit','','2012-05-14 00:00:00',99.26,569,878,97.63,'Sun-Synch' from dual union all
select 38301,'038301','2010-027F','CZ-2D sep motor cover','',68887,'In Earth orbit','','2012-05-14 00:00:00',99.27,565,883,97.76,'Sun-Synch' from dual union all
select 38302,'038302','2010-027G','CZ-2D sep motor cover','',68887,'In Earth orbit','','2012-05-15 00:00:00',99.54,567,907,97.61,'Sun-Synch' from dual union all
select 38303,'038303','2010-009G','CZ-4C sep motor cover','',68869,'In Earth orbit','','2012-05-13 00:00:00',106.42,978,1143,63.41,'Intermediate' from dual union all
select 38304,'038304','2010-009H','CZ-4C sep motor cover','',68869,'In Earth orbit','','2012-05-01 00:00:00',110.47,1133,1364,63.32,'Intermediate' from dual union all
select 38305,'038305','2010-009J','CZ-4C sep motor cover','',68869,'In Earth orbit','','2012-05-08 00:00:00',110.78,1134,1391,63.49,'Intermediate' from dual union all
select 38306,'038306','2011-066D','CZ-4B sep motor cover','',69118,'Reentered','2012-07-11 00:00:00','2012-05-09 00:00:00',92.77,370,451,97.44,'Sun-Synch' from dual union all
select 38307,'038307','2011-066E','CZ-4B sep motor cover','',69118,'In Earth orbit','','2012-05-15 00:00:00',98.08,487,847,97.37,'Sun-Synch' from dual union all
select 38308,'038308','2011-066F','CZ-4B sep motor cover','',69118,'Reentered','2016-12-11 00:00:00','2012-05-15 00:00:00',97.79,487,819,97.33,'Sun-Synch' from dual union all
select 38309,'038309','2011-079C','CZ-4B sep motor cover','',69131,'In Earth orbit','','2012-05-14 00:00:00',99.88,660,845,98.46,'Sun-Synch' from dual union all
select 3831,'003831','1969-027A','Kosmos-273','Zenit-2 No. 77',18294,'Landed','1969-03-30 00:00:00','1969-03-26 00:00:00',89.75,197,328,65.44,'Intermediate' from dual union all
select 38310,'038310','2011-079D','CZ-4B sep motor cover','',69131,'In Earth orbit','','2012-05-15 00:00:00',100.04,660,861,98.69,'Sun-Synch' from dual union all
select 38311,'038311','2011-079E','CZ-4B sep motor cover','',69131,'In Earth orbit','','2012-05-13 00:00:00',100.34,682,867,98.52,'Sun-Synch' from dual union all
select 38312,'038312','2011-079F','CZ-4B sep motor cover','',69131,'In Earth orbit','','2012-05-14 00:00:00',100.6,694,880,98.51,'Sun-Synch' from dual union all
select 38313,'038313','2011-043C','CZ-4B sep motor cover','',69095,'In Earth orbit','','2012-05-15 00:00:00',106.66,907,1237,99.37,'Sun-Synch' from dual union all
select 38314,'038314','2011-043D','CZ-4B sep motor cover','',69095,'In Earth orbit','','2012-05-12 00:00:00',107.42,907,1308,99.39,'Sun-Synch' from dual union all
select 38315,'038315','2011-043E','CZ-4B sep motor cover','',69095,'In Earth orbit','','2012-05-13 00:00:00',106.64,907,1235,99.42,'Sun-Synch' from dual union all
select 38316,'038316','2011-043F','CZ-4B sep motor cover','',69095,'In Earth orbit','','2012-05-14 00:00:00',106.82,907,1252,99.39,'Sun-Synch' from dual union all
select 38317,'038317','2012-001D','CZ-4B sep motor cover','',69256,'Reentered','2015-05-27 00:00:00','2012-05-13 00:00:00',96.86,488,729,97.46,'Sun-Synch' from dual union all
select 38318,'038318','2012-001E','CZ-4B sep motor cover','',69256,'In Earth orbit','','2012-05-12 00:00:00',98.2,496,850,97.51,'Sun-Synch' from dual union all
select 38319,'038319','2012-001F','CZ-4B sep motor cover','',69256,'Reentered','2012-06-01 00:00:00','2012-05-14 00:00:00',91.52,317,382,97.55,'Sun-Synch' from dual union all
select 3832,'003832','1969-027B','11A57I','11A57I',18294,'Reentered','1969-03-28 00:00:00','1969-03-24 00:00:00',89.51,197,303,65.43,'Intermediate' from dual union all
select 38320,'038320','2012-001G','CZ-4B sep motor cover','',69256,'Reentered','2012-11-13 00:00:00','2012-05-15 00:00:00',94.03,413,531,97.71,'Sun-Synch' from dual union all
select 38321,'038321','2008-064G','CZ-4B sep motor cover','',68572,'Reentered','2014-10-27 00:00:00','2012-05-08 00:00:00',97.03,473,760,97.34,'Sun-Synch' from dual union all
select 38322,'038322','2008-064H','CZ-4B sep motor cover','',68572,'Reentered','2015-03-10 00:00:00','2012-05-15 00:00:00',97.48,478,799,97.37,'Sun-Synch' from dual union all
select 38323,'038323','2008-056J','CZ-2D sep motor cover','',68564,'In Earth orbit','','2012-05-14 00:00:00',103.52,776,1074,98.54,'Sun-Synch' from dual union all
select 38324,'038324','2008-056K','deb SW-3','',68564,'In Earth orbit','','2012-05-08 00:00:00',100.37,652,901,98.61,'Sun-Synch' from dual union all
select 38325,'038325','2009-072D','deb CZ-4C','',68753,'In Earth orbit','','2012-05-12 00:00:00',106.97,969,1204,100.36,'Sun-Synch' from dual union all
select 38326,'038326','2010-051H','CZ-4B sep motor cover','',68911,'In Earth orbit','','2012-05-13 00:00:00',99.84,599,903,97.71,'Sun-Synch' from dual union all
select 38327,'038327','2011-068J','SW-4 upper debris','',69120,'In Earth orbit','','2012-05-14 00:00:00',103.63,776,1085,98.56,'Sun-Synch' from dual union all
select 38328,'038328','2001-049QP','deb PSLV-C3 PS4','',67202,'Reentered','2016-04-24 00:00:00','2012-05-15 00:00:00',96.26,533,626,97.47,'Sun-Synch' from dual union all
select 38329,'038329','1991-082CS','deb DMSP F-11','',64251,'In Earth orbit','','2012-05-05 00:00:00',101.38,820,827,98.75,'Sun-Synch' from dual union all
select 3833,'003833','1969-028A','Kosmos-274','Zenit-4 No. 58',18295,'Landed','1969-04-01 00:00:00','1969-03-26 00:00:00',89.53,204,299,64.98,'Intermediate' from dual union all
select 38330,'038330','1988-005AR','deb Meteor-2','',61254,'In Earth orbit','','2012-05-02 00:00:00',103.65,915,948,82.52,'Intermediate' from dual union all
select 38331,'038331','2012-023A','JCSAT 4B','JCSAT 13',69278,'In Earth orbit','','2012-05-30 00:00:00',1435.88,35763,35802,.05,'Stationary' from dual union all
select 38332,'038332','2012-023B','Vinasat-2','Vinasat-2',69278,'In Earth orbit','','2012-05-23 00:00:00',1433.64,35716,35761,.08,'Drift GEO' from dual union all
select 38333,'038333','2012-023C','ESC-A 562','Ariane 5 ESC-A',69278,'In Earth orbit','','2012-05-23 00:00:00',629.95,264,35665,1.71,'GEO Transfer' from dual union all
select 38334,'038334','2012-023D','Sylda 5A','Sylda 49-A',69278,'In Earth orbit','','2012-05-27 00:00:00',629.64,266,35646,2.03,'GEO Transfer' from dual union all
select 38335,'038335','2012-024A','Kosmos-2480','Kobal''t-M No. 563',69279,'Landed','2012-09-24 00:00:00','2012-05-17 00:00:00',88.9,186,255,81.37,'Intermediate' from dual union all
select 38336,'038336','2012-024B','11S510','11S510',69279,'Reentered','2012-05-20 00:00:00','2012-05-17 00:00:00',88.77,184,244,81.38,'Intermediate' from dual union all
select 38337,'038337','2012-025A','Shizuku','GCOM-W1',69280,'In Earth orbit','','2012-05-17 00:00:00',98.75,663,735,98.17,'Sun-Synch' from dual union all
select 38338,'038338','2012-025B','Arirang-3','KOMPSAT-3',69280,'In Earth orbit','','2012-05-25 00:00:00',98.17,665,677,98.18,'Sun-Synch' from dual union all
select 38339,'038339','2012-025C','SDS-4','SPAISE/SDS-4',69280,'In Earth orbit','','2012-05-24 00:00:00',98.09,662,673,98.17,'Sun-Synch' from dual union all
select 3834,'003834','1969-028B','11A57I','11A57I',18295,'Reentered','1969-03-29 00:00:00','1969-03-26 00:00:00',89.17,206,261,64.98,'Intermediate' from dual union all
select 38340,'038340','2012-025D','Horyu-2','Horyu-2',69280,'In Earth orbit','','2012-05-29 00:00:00',97.95,651,671,98.17,'Sun-Synch' from dual union all
select 38341,'038341','2012-025E','H-2A F21 Stage 2','H-2A Stage 2',69280,'In Earth orbit','','2012-05-24 00:00:00',97.34,589,674,98.43,'Sun-Synch' from dual union all
select 38342,'038342','2012-026A','Nimiq 6','Nimiq 6',69281,'In Earth orbit','','2012-05-25 00:00:00',1433.87,35738,35748,.03,'Drift GEO' from dual union all
select 38343,'038343','2012-026B','Briz-M No. 99530','Briz-M 14S43 No. 99530',69281,'In Earth orbit','','2012-05-26 00:00:00',813.97,10622,34378,10.65,'Medium' from dual union all
select 38344,'038344','2012-026C','Briz-M DTB','Briz-M torus tank',69281,'In Earth orbit','','2012-06-07 00:00:00',434.73,354,24909,49.33,'Highly Ellip' from dual union all
select 38345,'038345','2012-025F','4/4D-LC Adapter','Adapter',69280,'In Earth orbit','','2012-05-24 00:00:00',98.1,662,674,98.17,'Sun-Synch' from dual union all
select 38346,'038346','2012-025G','4/4D-LC Lower Fairing Half','Fairing Half',69280,'In Earth orbit','','2012-05-24 00:00:00',98.09,661,674,98.18,'Sun-Synch' from dual union all
select 38347,'038347','2012-025H','4/4D-LC Lower Fairing Half','Fairing Half',69280,'In Earth orbit','','2012-05-24 00:00:00',98.06,658,673,98.18,'Sun-Synch' from dual union all
select 38348,'038348','2012-027A','Dragon C2','Dragon C102',69282,'Landed','2012-05-31 00:00:00','2012-05-24 00:00:00',92.41,387,399,51.61,'Intermediate' from dual;
insert into SATELLITE
select 38349,'038349','2012-027B','Falcon 9-003 Stage 2','F9-003 St2',69282,'Reentered','2012-06-27 00:00:00','2012-05-29 00:00:00',90.33,264,318,51.66,'Intermediate' from dual union all
select 3835,'003835','1969-029A','Meteor','Meteor 11F614M No. 12',18296,'Reentered','2012-03-26 00:00:00','1969-05-03 00:00:00',97.86,633,680,81.2,'Intermediate' from dual union all
select 38350,'038350','2012-027C','Dragon C2 solar array cover','',69282,'Reentered','2012-06-07 00:00:00','2012-05-30 00:00:00',90.37,280,306,51.66,'Intermediate' from dual union all
select 38351,'038351','2012-027D','Dragon C2 solar array cover','',69282,'Reentered','2012-06-09 00:00:00','2012-05-29 00:00:00',90.52,287,313,51.68,'Intermediate' from dual union all
select 38352,'038352','2012-028A','Zhongxing 2A','Shentong-2',69283,'In Earth orbit','','2012-06-08 00:00:00',1436.06,35780,35792,.19,'Stationary' from dual union all
select 38353,'038353','2012-028B','CZ-3B Stage 3','CZ-3B Stage 3',69283,'In Earth orbit','','2012-06-05 00:00:00',630.36,189,35761,26.68,'GEO Transfer' from dual union all
select 38354,'038354','2012-029A','Yaogan Weixing 15','Jianbing-9 02 xing',69284,'In Earth orbit','','2012-06-05 00:00:00',109.49,1201,1206,100.12,'Sun-Synch' from dual union all
select 38355,'038355','2012-029B','CZ-4C Stage 3','CZ-4C Stage 3',69284,'In Earth orbit','','2012-06-05 00:00:00',106.97,965,1207,100.05,'Sun-Synch' from dual union all
select 38356,'038356','2012-030A','Intelsat IS-19','Intelsat IS-19',69285,'In Earth orbit','','2012-06-11 00:00:00',1435.17,35667,35870,.17,'Drift GEO' from dual union all
select 38357,'038357','2012-030B','Blok DM-SL No. 32L','Blok DM-SL',69285,'In Earth orbit','','2012-06-11 00:00:00',639.04,798,35598,.11,'GEO Transfer' from dual union all
select 38358,'038358','2012-031A','NuSTAR','NuSTAR',69286,'In Earth orbit','','2012-06-17 00:00:00',97.17,614,633,6.02,'Equatorial' from dual union all
select 38359,'038359','2012-031B','Orion 38','Orion 38',69286,'In Earth orbit','','2012-06-15 00:00:00',97.24,621,633,6.03,'Equatorial' from dual union all
select 3836,'003836','1969-029B','8A92ME','8A92ME',18296,'Reentered','1989-06-10 00:00:00','1969-05-16 00:00:00',98.04,464,866,81.21,'Intermediate' from dual union all
select 38360,'038360','1965-027BM','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',110.88,1239,1296,90.26,'Polar' from dual union all
select 38361,'038361','1965-027BN','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',110.9,1240,1296,90.28,'Polar' from dual union all
select 38362,'038362','1965-027BP','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',111.15,1249,1311,90.28,'Polar' from dual union all
select 38363,'038363','1965-027BQ','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-12 00:00:00',110.85,1241,1291,90.27,'Polar' from dual union all
select 38364,'038364','1965-027BR','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-13 00:00:00',111.19,1259,1304,90.27,'Polar' from dual union all
select 38365,'038365','1965-027BS','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-09 00:00:00',110.99,1256,1289,90.24,'Polar' from dual union all
select 38366,'038366','1965-027BT','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',110.91,1250,1288,90.27,'Polar' from dual union all
select 38367,'038367','1965-027BU','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',111.21,1262,1302,90.27,'Polar' from dual union all
select 38368,'038368','1965-027BV','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',111.21,1267,1298,90.28,'Polar' from dual union all
select 38369,'038369','1965-027BW','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-08 00:00:00',111.08,1254,1299,90.23,'Polar' from dual union all
select 3837,'003837','1969-030A','Mariner 7','Mariner 69-2',18297,'Deep Space','1969-04-01 00:00:00','1969-03-27 00:00:00',602.83,506,60540,31.01,'Earth Escape' from dual union all
select 38370,'038370','1965-027BX','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-13 00:00:00',110.88,1245,1289,90.27,'Polar' from dual union all
select 38371,'038371','1965-027BY','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',110.51,1233,1268,90.26,'Polar' from dual union all
select 38372,'038372','1965-027BZ','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',111.09,1247,1307,90.32,'Polar' from dual union all
select 38373,'038373','1965-027CA','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-04 00:00:00',110.85,1240,1292,90.31,'Polar' from dual union all
select 38374,'038374','1965-027CB','deb SNAPSHOT','',11777,'In Earth orbit','','2012-05-29 00:00:00',110.84,1241,1289,90.27,'Polar' from dual union all
select 38375,'038375','1965-027CC','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-08 00:00:00',111.32,1261,1314,90.27,'Polar' from dual union all
select 38376,'038376','1965-027CD','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',111.23,1261,1306,90.27,'Polar' from dual union all
select 38377,'038377','1965-027CE','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-13 00:00:00',111.19,1254,1309,90.29,'Polar' from dual union all
select 38378,'038378','1965-027CF','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-12 00:00:00',110.68,1228,1287,90.2,'Polar' from dual union all
select 38379,'038379','1965-027CG','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',110.89,1248,1287,90.26,'Polar' from dual union all
select 3838,'003838','1965-082RW','deb Transtage 4','',11832,'Reentered','1980-11-30 00:00:00','1971-11-18 00:00:00',103.03,717,1087,31.45,'Intermediate' from dual union all
select 38380,'038380','1965-027CH','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',111.03,1252,1297,90.15,'Polar' from dual union all
select 38381,'038381','1965-027CJ','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-03 00:00:00',110.93,1251,1288,90.3,'Polar' from dual union all
select 38382,'038382','1965-027CK','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',111.21,1261,1303,90.27,'Polar' from dual union all
select 38383,'038383','1965-027CL','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',111.17,1259,1302,90.29,'Polar' from dual union all
select 38384,'038384','1965-027CM','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-07 00:00:00',111.01,1254,1293,90.27,'Polar' from dual union all
select 38385,'038385','1965-027CN','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-09 00:00:00',111.15,1247,1313,90.25,'Polar' from dual union all
select 38386,'038386','1965-027CP','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',110.83,1235,1295,90.3,'Polar' from dual union all
select 38387,'038387','1965-027CQ','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-05 00:00:00',111.18,1258,1304,90.25,'Polar' from dual union all
select 38388,'038388','1965-027CR','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-03 00:00:00',110.95,1242,1299,90.28,'Polar' from dual union all
select 38389,'038389','1965-027CS','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-06 00:00:00',110.9,1248,1289,90.32,'Polar' from dual union all
select 3839,'003839','1965-082RX','deb Transtage 4','',11832,'Reentered','1977-02-23 00:00:00','1969-03-30 00:00:00',102.63,759,1008,32.67,'Intermediate' from dual union all
select 38390,'038390','1965-027CT','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',111.11,1257,1299,90.25,'Polar' from dual union all
select 38391,'038391','1965-027CU','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-07 00:00:00',111.09,1247,1307,90.28,'Polar' from dual union all
select 38392,'038392','1965-027CV','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-12 00:00:00',111.24,1264,1304,90.26,'Polar' from dual union all
select 38393,'038393','1965-027CW','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-08 00:00:00',111.05,1253,1296,90.29,'Polar' from dual union all
select 38394,'038394','1965-027CX','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-09 00:00:00',111,1248,1298,90.3,'Polar' from dual union all
select 38395,'038395','1965-027CY','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-13 00:00:00',111.05,1248,1302,90.28,'Polar' from dual union all
select 38396,'038396','1965-027CZ','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-13 00:00:00',111.11,1254,1301,90.27,'Polar' from dual union all
select 38397,'038397','1965-027DA','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-07 00:00:00',110.91,1236,1302,90.29,'Polar' from dual union all
select 38398,'038398','1965-027DB','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',111.17,1257,1304,90.28,'Polar' from dual union all
select 38399,'038399','1965-027DC','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-13 00:00:00',111.18,1261,1301,90.29,'Polar' from dual union all
select 384,'000384','62 A Tau 4','deb Sputnik','',8155,'Reentered','1962-09-21 00:00:00','1962-09-10 00:00:00',89.58,183,325,64.72,'Intermediate' from dual union all
select 3840,'003840','1969-010C','OPS 2644 sensor cover','',18277,'In Earth orbit','','1969-03-27 00:00:00',114.51,1408,1459,80.61,'Intermediate' from dual union all
select 38400,'038400','1965-027DD','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',111.06,1251,1299,90.29,'Polar' from dual union all
select 38401,'038401','1965-027DE','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',111.29,1263,1309,90.26,'Polar' from dual union all
select 38402,'038402','1965-027DF','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-12 00:00:00',111.43,1274,1311,90.26,'Polar' from dual union all
select 38403,'038403','1965-027DG','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',111.32,1266,1308,90.28,'Polar' from dual union all
select 38404,'038404','1965-027DH','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-07 00:00:00',110.96,1254,1287,90.26,'Polar' from dual union all
select 38405,'038405','1965-027DJ','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-05 00:00:00',111,1244,1302,90.3,'Polar' from dual union all
select 38406,'038406','1965-027DK','deb SNAPSHOT','',11777,'In Earth orbit','','2012-05-27 00:00:00',110.9,1247,1289,90.28,'Polar' from dual union all
select 38407,'038407','1965-027DL','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-12 00:00:00',111.22,1261,1305,90.29,'Polar' from dual union all
select 38408,'038408','1965-027DM','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',110.75,1230,1292,90.27,'Polar' from dual union all
select 38409,'038409','1965-027DN','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-13 00:00:00',111,1251,1294,90.29,'Polar' from dual union all
select 3841,'003841','1969-010D','OPS 2644 sensor cover','',18277,'In Earth orbit','','1969-03-27 00:00:00',113.77,1376,1423,80.18,'Intermediate' from dual union all
select 38410,'038410','1965-027DP','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-13 00:00:00',111.14,1253,1306,90.27,'Polar' from dual union all
select 38411,'038411','1965-027DQ','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-12 00:00:00',111.15,1249,1311,90.28,'Polar' from dual union all
select 38412,'038412','1965-027DR','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-12 00:00:00',111.36,1275,1304,90.26,'Polar' from dual union all
select 38413,'038413','1965-027DS','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-06 00:00:00',110.95,1246,1295,90.27,'Polar' from dual union all
select 38414,'038414','1965-027DT','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-07 00:00:00',111.11,1251,1304,90.23,'Polar' from dual union all
select 38415,'038415','1965-027DU','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',110.75,1242,1281,90.27,'Polar' from dual union all
select 38416,'038416','1965-027DV','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',111.23,1261,1306,90.27,'Polar' from dual union all
select 38417,'038417','1965-027DW','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',111.22,1262,1304,90.28,'Polar' from dual union all
select 38418,'038418','1965-027DX','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-08 00:00:00',110.85,1241,1291,90.27,'Polar' from dual union all
select 38419,'038419','1965-027DY','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',111.29,1264,1308,90.26,'Polar' from dual union all
select 3842,'003842','61 Omicron 222','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-02 00:00:00',105.51,919,1117,66.75,'Intermediate' from dual union all
select 38420,'038420','1965-027DZ','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-13 00:00:00',110.94,1243,1297,90.27,'Polar' from dual union all
select 38421,'038421','1965-027EA','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-13 00:00:00',110.33,1218,1266,90.28,'Polar' from dual union all
select 38422,'038422','1965-027EB','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-12 00:00:00',111.12,1253,1304,90.3,'Polar' from dual union all
select 38423,'038423','1965-027EC','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',111.29,1265,1307,90.27,'Polar' from dual union all
select 38424,'038424','1965-027ED','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',110.71,1241,1278,90.26,'Polar' from dual union all
select 38425,'038425','1965-027EE','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-07 00:00:00',111.35,1265,1312,90.27,'Polar' from dual union all
select 38426,'038426','1965-027EF','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-07 00:00:00',111.2,1258,1306,90.27,'Polar' from dual union all
select 38427,'038427','1965-027EG','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-08 00:00:00',111.27,1264,1307,90.25,'Polar' from dual union all
select 38428,'038428','1965-027EH','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-12 00:00:00',111.13,1249,1309,90.28,'Polar' from dual union all
select 38429,'038429','1965-027EJ','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',111.09,1247,1307,90.25,'Polar' from dual union all
select 3843,'003843','1968-097BG','deb Kosmos-252','',16824,'In Earth orbit','','1992-02-03 00:00:00',113.31,775,1982,62.28,'Intermediate' from dual union all
select 38430,'038430','1965-027EK','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-12 00:00:00',111.08,1241,1312,90.29,'Polar' from dual union all
select 38431,'038431','1965-027EL','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-13 00:00:00',110.88,1248,1287,90.24,'Polar' from dual union all
select 38432,'038432','1965-027EM','deb SNAPSHOT','',11777,'In Earth orbit','','2012-05-28 00:00:00',111.12,1259,1297,90.28,'Polar' from dual union all
select 38433,'038433','1965-027EN','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-12 00:00:00',110.25,1216,1260,90.27,'Polar' from dual union all
select 38434,'038434','1965-027EP','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-12 00:00:00',111.33,1269,1307,90.26,'Polar' from dual union all
select 38435,'038435','1965-027EQ','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-08 00:00:00',111.07,1250,1302,90.27,'Polar' from dual union all
select 38436,'038436','1965-027ER','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',110.93,1244,1295,90.24,'Polar' from dual union all
select 38437,'038437','1965-027ES','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',111.27,1261,1309,90.26,'Polar' from dual union all
select 38438,'038438','1965-027ET','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-13 00:00:00',111.44,1271,1315,90.26,'Polar' from dual;
insert into SATELLITE
select 38439,'038439','1965-027EU','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-07 00:00:00',111,1250,1296,90.32,'Polar' from dual union all
select 3844,'003844','1969-021T','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1983-07-19 00:00:00','1969-04-06 00:00:00',95.68,538,565,74.17,'Intermediate' from dual union all
select 38440,'038440','1965-027EV','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',111.02,1238,1310,90.29,'Polar' from dual union all
select 38441,'038441','1965-027EW','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',111.26,1260,1310,90.27,'Polar' from dual union all
select 38442,'038442','1965-027EX','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-09 00:00:00',111.38,1270,1311,90.26,'Polar' from dual union all
select 38443,'038443','1965-027EY','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-12 00:00:00',110.89,1241,1295,90.26,'Polar' from dual union all
select 38444,'038444','1965-027EZ','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-04 00:00:00',111.26,1267,1303,90.27,'Polar' from dual union all
select 38445,'038445','1965-027FA','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-08 00:00:00',111.3,1265,1308,90.25,'Polar' from dual union all
select 38446,'038446','1965-027FB','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',110.73,1247,1273,90.29,'Polar' from dual union all
select 38447,'038447','1965-027FC','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-08 00:00:00',111.06,1250,1301,90.25,'Polar' from dual union all
select 38448,'038448','1965-027FD','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-08 00:00:00',110.9,1240,1296,90.23,'Polar' from dual union all
select 38449,'038449','1965-027FE','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',111.27,1265,1306,90.27,'Polar' from dual union all
select 3845,'003845','1969-030B','Centaur AC-19','Centaur 16D',18297,'Deep Space','1969-04-01 00:00:00','1969-03-27 00:00:00',602.83,506,60540,31.01,'Earth Escape' from dual union all
select 38450,'038450','1965-027FF','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',110.9,1246,1290,90.26,'Polar' from dual union all
select 38451,'038451','1965-027FG','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-01 00:00:00',111.2,1262,1302,90.29,'Polar' from dual union all
select 38452,'038452','1965-027FH','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',111.37,1279,1301,90.25,'Polar' from dual union all
select 38453,'038453','1965-027FJ','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',111.36,1273,1305,90.27,'Polar' from dual union all
select 38454,'038454','1965-027FK','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-14 00:00:00',111.31,1264,1310,90.27,'Polar' from dual union all
select 38455,'038455','1965-027FL','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-14 00:00:00',111.08,1258,1295,90.26,'Polar' from dual union all
select 38456,'038456','1965-027FM','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-10 00:00:00',111.14,1260,1298,90.25,'Polar' from dual union all
select 38457,'038457','1965-027FN','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-14 00:00:00',111.05,1250,1300,90.25,'Polar' from dual union all
select 38458,'038458','1965-027FP','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-09 00:00:00',111.46,1274,1314,90.26,'Polar' from dual union all
select 38459,'038459','1965-027FQ','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-11 00:00:00',111.46,1272,1316,90.27,'Polar' from dual union all
select 3846,'003846','1969-031A','Kosmos-275','DS-P1-I No. 5',18298,'Reentered','1970-02-07 00:00:00','1969-05-06 00:00:00',94.59,269,729,70.97,'Intermediate' from dual union all
select 38460,'038460','1965-027FR','deb SNAPSHOT','',11777,'In Earth orbit','','2012-06-09 00:00:00',111.35,1266,1312,90.26,'Polar' from dual union all
select 38461,'038461','2012-032A','Shenzhou 9','shenzhou jiu fanhui cang',69287,'Landed','2012-06-29 00:00:00','2012-06-18 00:00:00',91.19,328,337,42.78,'Intermediate' from dual union all
select 38462,'038462','2012-032B','CZ-2F Stage 2','CZ-2F Stage 2',69287,'Reentered','2012-06-28 00:00:00','2012-06-18 00:00:00',89.34,192,292,42.78,'Intermediate' from dual union all
select 38463,'038463','2012-032C','CZ-2F sep motor cover','',69287,'Reentered','2012-06-18 00:00:00','2012-06-16 00:00:00',91.52,201,498,42.95,'Intermediate' from dual union all
select 38464,'038464','2012-032D','CZ-2F sep motor cover','',69287,'Reentered','2012-06-18 00:00:00','2012-06-17 00:00:00',91.44,204,487,42.99,'Intermediate' from dual union all
select 38465,'038465','2012-032E','CZ-2F sep motor cover','',69287,'Reentered','2012-08-15 00:00:00','2012-06-21 00:00:00',90.93,263,377,42.8,'Intermediate' from dual union all
select 38466,'038466','2012-033A','USA 236','QUASAR 17',69288,'In Earth orbit','','2012-07-16 00:00:00',1436.04,35777,35794,4.91,'Inclined GEO' from dual union all
select 38467,'038467','2012-032F','CZ-2F sep motor cover','',69287,'Reentered','2012-06-18 00:00:00','2012-06-18 00:00:00',90.17,181,384,42.91,'Intermediate' from dual union all
select 38468,'038468','1997-051ZV','deb Iridium 33','',66131,'In Earth orbit','','2012-06-16 00:00:00',101.7,768,911,86.37,'Polar' from dual union all
select 38469,'038469','1997-051ZW','deb Iridium 33','',66131,'In Earth orbit','','2012-06-14 00:00:00',103.1,741,1070,86.3,'Polar' from dual union all
select 3847,'003847','1969-031B','S1M','S1M',18298,'Reentered','1969-09-16 00:00:00','1969-05-06 00:00:00',93.99,267,673,70.98,'Intermediate' from dual union all
select 38470,'038470','1997-051ZX','deb Iridium 33','',66131,'In Earth orbit','','2012-06-17 00:00:00',101.88,755,941,86.5,'Polar' from dual union all
select 38471,'038471','1997-051ZY','deb Iridium 33','',66131,'In Earth orbit','','2012-06-17 00:00:00',100.06,748,775,86.32,'Polar' from dual union all
select 38472,'038472','1997-051ZZ','deb Iridium 33','',66131,'Reentered','2017-10-01 00:00:00','2012-06-21 00:00:00',100.21,734,803,86.42,'Polar' from dual union all
select 38473,'038473','1997-051AAA','deb Iridium 33','',66131,'Reentered','2014-02-24 00:00:00','2012-06-20 00:00:00',98.89,697,715,86.32,'Polar' from dual union all
select 38474,'038474','1997-051AAB','deb Iridium 33','',66131,'In Earth orbit','','2012-06-17 00:00:00',100.23,746,793,86.35,'Polar' from dual union all
select 38475,'038475','1997-051AAC','deb Iridium 33','',66131,'Reentered','2012-08-12 00:00:00','2012-06-20 00:00:00',96.03,545,592,86.39,'Polar' from dual union all
select 38476,'038476','1997-051AAD','deb Iridium 33','',66131,'Reentered','2013-01-11 00:00:00','2012-06-15 00:00:00',96.09,525,617,86.39,'Polar' from dual union all
select 38477,'038477','1997-051AAE','deb Iridium 33','',66131,'In Earth orbit','','2012-06-13 00:00:00',108.37,770,1533,86.43,'Polar' from dual union all
select 38478,'038478','1993-036BTN','deb Kosmos-2251','',65138,'In Earth orbit','','2012-06-19 00:00:00',98.74,644,753,74.03,'Intermediate' from dual union all
select 38479,'038479','1993-036BTP','deb Kosmos-2251','',65138,'In Earth orbit','','2012-06-21 00:00:00',101.96,760,943,74.02,'Intermediate' from dual union all
select 3848,'003848','1968-055C','Star 17','Star 17',16782,'In Earth orbit','','1968-08-03 00:00:00',224.1,5824,5863,120.84,'Medium' from dual union all
select 38480,'038480','1993-036BTQ','deb Kosmos-2251','',65138,'In Earth orbit','','2012-06-19 00:00:00',98.13,584,755,73.87,'Intermediate' from dual union all
select 38481,'038481','1993-036BTR','deb Kosmos-2251','',65138,'In Earth orbit','','2012-06-11 00:00:00',104.26,770,1150,73.94,'Intermediate' from dual union all
select 38482,'038482','1993-036BTS','deb Kosmos-2251','',65138,'In Earth orbit','','2012-06-17 00:00:00',100.34,728,822,73.95,'Intermediate' from dual union all
select 38483,'038483','1993-036BTT','deb Kosmos-2251','',65138,'In Earth orbit','','2012-06-17 00:00:00',101.86,749,945,74.26,'Intermediate' from dual union all
select 38484,'038484','1993-036BTU','deb Kosmos-2251','',65138,'In Earth orbit','','2012-06-21 00:00:00',99.63,723,759,74.06,'Intermediate' from dual union all
select 38485,'038485','1993-036BTV','deb Kosmos-2251','',65138,'In Earth orbit','','2012-06-16 00:00:00',99.41,693,768,74.02,'Intermediate' from dual union all
select 38486,'038486','1993-036BTW','deb Kosmos-2251','',65138,'In Earth orbit','','2012-06-20 00:00:00',101.04,768,848,73.96,'Intermediate' from dual union all
select 38487,'038487','1993-036BTX','deb Kosmos-2251','',65138,'In Earth orbit','','2012-06-17 00:00:00',98.88,645,766,74,'Intermediate' from dual union all
select 38488,'038488','1999-025ERX','deb FY-1C','',66706,'Reentered','2014-05-18 00:00:00','2012-06-14 00:00:00',99.04,581,845,98.57,'Sun-Synch' from dual union all
select 38489,'038489','1999-025ERY','deb FY-1C','',66706,'In Earth orbit','','2012-06-18 00:00:00',105,811,1178,98.22,'Sun-Synch' from dual union all
select 3849,'003849','1969-029C','deb Blok-E','',18296,'Reentered','1980-11-05 00:00:00','1980-10-29 00:00:00',88.69,205,215,81.31,'Intermediate' from dual union all
select 38490,'038490','1999-025ERZ','deb FY-1C','',66706,'In Earth orbit','','2012-06-03 00:00:00',102.64,845,923,99.22,'Sun-Synch' from dual union all
select 38491,'038491','1999-025ESA','deb FY-1C','',66706,'In Earth orbit','','2012-06-17 00:00:00',103.53,750,1101,99.29,'Sun-Synch' from dual union all
select 38492,'038492','1999-025ESB','deb FY-1C','',66706,'In Earth orbit','','2012-06-20 00:00:00',105.82,807,1258,98.62,'Sun-Synch' from dual union all
select 38493,'038493','1999-025ESC','deb FY-1C','',66706,'In Earth orbit','','2012-06-19 00:00:00',106,714,1369,99.1,'Sun-Synch' from dual union all
select 38494,'038494','1999-025ESD','deb FY-1C','',66706,'In Earth orbit','','2012-06-11 00:00:00',106.08,651,1438,98.46,'Sun-Synch' from dual union all
select 38495,'038495','1999-025ESE','deb FY-1C','',66706,'In Earth orbit','','2012-06-18 00:00:00',106.44,851,1272,99.23,'Sun-Synch' from dual union all
select 38496,'038496','1999-025ESF','deb FY-1C','',66706,'In Earth orbit','','2012-06-21 00:00:00',102.81,825,959,99.16,'Sun-Synch' from dual union all
select 38497,'038497','1999-025ESG','deb FY-1C','',66706,'In Earth orbit','','2012-06-20 00:00:00',101.17,744,885,98.67,'Sun-Synch' from dual union all
select 38498,'038498','1999-025ESH','deb FY-1C','',66706,'In Earth orbit','','2012-06-19 00:00:00',111.63,858,1746,99.73,'Sun-Synch' from dual union all
select 38499,'038499','1999-025ESJ','deb FY-1C','',66706,'In Earth orbit','','2012-06-14 00:00:00',105.71,843,1213,98.43,'Sun-Synch' from dual union all
select 385,'000385','62 A Upsilon','FTV 1132','ARGON 10',8157,'Reentered Attached','1964-10-26 00:00:00','1962-10-01 00:00:00',94.18,288,670,82.82,'Intermediate' from dual union all
select 3850,'003850','1969-029D','deb Blok-E','',18296,'Reentered','1969-09-08 00:00:00','1969-09-05 00:00:00',90.48,296,301,81.18,'Intermediate' from dual union all
select 38500,'038500','1999-025ESK','deb FY-1C','',66706,'In Earth orbit','','2012-06-15 00:00:00',105.65,846,1204,98.65,'Sun-Synch' from dual union all
select 38501,'038501','1999-025ESL','deb FY-1C','',66706,'Reentered','2017-07-14 00:00:00','2012-06-21 00:00:00',99.97,695,820,97.38,'Sun-Synch' from dual union all
select 38502,'038502','1999-025ESM','deb FY-1C','',66706,'In Earth orbit','','2012-06-20 00:00:00',102.55,844,914,99.22,'Sun-Synch' from dual union all
select 38503,'038503','1999-025ESN','deb FY-1C','',66706,'In Earth orbit','','2012-06-14 00:00:00',99.76,644,851,99.5,'Sun-Synch' from dual union all
select 38504,'038504','1999-025ESP','deb FY-1C','',66706,'In Earth orbit','','2012-06-18 00:00:00',103.88,765,1119,98.04,'Sun-Synch' from dual union all
select 38505,'038505','1999-025ESQ','deb FY-1C','',66706,'In Earth orbit','','2012-06-20 00:00:00',103.74,795,1076,98.36,'Sun-Synch' from dual union all
select 38506,'038506','1999-025ESR','deb FY-1C','',66706,'In Earth orbit','','2012-06-09 00:00:00',103.86,787,1095,99.03,'Sun-Synch' from dual union all
select 38507,'038507','1999-025ESS','deb FY-1C','',66706,'In Earth orbit','','2012-06-17 00:00:00',101.73,810,871,98.95,'Sun-Synch' from dual union all
select 38508,'038508','1999-025EST','deb FY-1C','',66706,'In Earth orbit','','2012-06-21 00:00:00',100.89,622,979,98.77,'Sun-Synch' from dual union all
select 38509,'038509','1999-025ESU','deb FY-1C','',66706,'Reentered','2014-06-25 00:00:00','2012-06-05 00:00:00',98.22,548,799,98.42,'Sun-Synch' from dual union all
select 3851,'003851','1969-029E','deb Blok-E','',18296,'Reentered','1986-11-12 00:00:00','1986-11-11 00:00:00',87.73,158,167,81.16,'Intermediate' from dual union all
select 38510,'038510','1999-025ESV','deb FY-1C','',66706,'In Earth orbit','','2012-06-16 00:00:00',102.45,796,954,99.12,'Sun-Synch' from dual union all
select 38511,'038511','1999-025ESW','deb FY-1C','',66706,'In Earth orbit','','2012-06-11 00:00:00',102.76,756,1023,96.65,'Sun-Synch' from dual union all
select 38512,'038512','1999-025ESX','deb FY-1C','',66706,'In Earth orbit','','2012-06-13 00:00:00',105.61,840,1206,98.4,'Sun-Synch' from dual union all
select 38513,'038513','1999-025ESY','deb FY-1C','',66706,'In Earth orbit','','2012-06-19 00:00:00',102.2,504,1222,97.63,'Sun-Synch' from dual union all
select 38514,'038514','1999-025ESZ','deb FY-1C','',66706,'In Earth orbit','','2012-06-21 00:00:00',106.41,707,1413,98.08,'Sun-Synch' from dual union all
select 38515,'038515','1999-025ETA','deb FY-1C','',66706,'In Earth orbit','','2012-06-14 00:00:00',102.4,843,902,99.08,'Sun-Synch' from dual union all
select 38516,'038516','1999-025ETB','deb FY-1C','',66706,'In Earth orbit','','2012-06-10 00:00:00',100.61,705,870,98.7,'Sun-Synch' from dual union all
select 38517,'038517','1999-025ETC','deb FY-1C','',66706,'In Earth orbit','','2012-06-20 00:00:00',100.61,684,891,98.73,'Sun-Synch' from dual union all
select 38518,'038518','1999-025ETD','deb FY-1C','',66706,'In Earth orbit','','2012-06-21 00:00:00',101.13,777,848,99.18,'Sun-Synch' from dual union all
select 38519,'038519','1999-025ETE','deb FY-1C','',66706,'In Earth orbit','','2012-06-21 00:00:00',105,847,1142,99.33,'Sun-Synch' from dual union all
select 3852,'003852','1969-029F','deb Blok-E','',18296,'Reentered','1970-07-13 00:00:00','1970-07-10 00:00:00',87.96,173,174,81.07,'Intermediate' from dual union all
select 38520,'038520','2012-032G','deb CZ-2F','',69287,'Reentered','2012-06-18 00:00:00','2012-06-18 00:00:00',88.72,156,267,42.65,'Intermediate' from dual union all
select 38521,'038521','1963-038Q','deb NNS','',9269,'In Earth orbit','','2012-06-24 00:00:00',107.08,1062,1121,90.12,'Polar' from dual union all
select 38522,'038522','1963-038R','deb NNS','',9269,'In Earth orbit','','2012-06-27 00:00:00',105.43,935,1095,90.11,'Polar' from dual union all
select 38523,'038523','1966-005N','deb NNS O-7 insulation','',13412,'Reentered','2016-10-15 00:00:00','2012-06-22 00:00:00',98.32,639,718,89.99,'Polar' from dual union all
select 38524,'038524','1964-083N','deb VE-5','',10369,'In Earth orbit','','2012-06-27 00:00:00',103.52,830,1020,89.87,'Polar' from dual union all
select 38525,'038525','1964-083P','deb VE-5','',10369,'In Earth orbit','','2012-06-27 00:00:00',103.55,787,1066,89.88,'Polar' from dual union all
select 38526,'038526','1964-083Q','deb VE-5','',10369,'In Earth orbit','','2012-06-27 00:00:00',104.05,869,1031,89.93,'Polar' from dual union all
select 38527,'038527','1964-083R','deb VE-5','',10369,'In Earth orbit','','2012-06-27 00:00:00',103.99,861,1033,89.92,'Polar' from dual union all
select 38528,'038528','2012-034A','USA 237','ORION 8',69289,'In Earth orbit','','2012-07-15 00:00:00',1441.25,35675,36100,3.52,'Inclined Drift' from dual;
insert into SATELLITE
select 38529,'038529','2012-034B','Delta 360','DCSS-5 F09',69289,'In Earth orbit','','2012-06-30 00:00:00',1434.22,35700,35800,7,'Inclined Drift' from dual union all
select 3853,'003853','1969-029G','deb Blok-E','',18296,'Reentered','1970-10-07 00:00:00','1970-09-23 00:00:00',91.76,338,384,81.35,'Intermediate' from dual union all
select 38530,'038530','1967-048H','deb NNS O-13 insulation','',15303,'In Earth orbit','','2012-06-29 00:00:00',106.29,1039,1071,89.7,'Polar' from dual union all
select 38531,'038531','1971-046C','deb Kosmos-422','',24864,'In Earth orbit','','2012-06-29 00:00:00',104.82,975,997,74.02,'Intermediate' from dual union all
select 38532,'038532','1975-064E','deb Meteor-2','',36511,'In Earth orbit','','2012-06-23 00:00:00',101.88,831,865,81.27,'Intermediate' from dual union all
select 38533,'038533','1977-024D','deb Meteor','',42152,'In Earth orbit','','2012-06-23 00:00:00',102.03,814,896,81.25,'Intermediate' from dual union all
select 38534,'038534','1979-095CB','deb Meteor-2','',47579,'In Earth orbit','','2012-06-20 00:00:00',102.13,852,867,81.19,'Intermediate' from dual union all
select 38535,'038535','1979-095CC','deb Meteor-2','',47579,'In Earth orbit','','2012-06-29 00:00:00',102.01,849,859,81.2,'Intermediate' from dual union all
select 38536,'038536','1983-109D','deb Meteor-2','',55866,'In Earth orbit','','2012-06-29 00:00:00',100.38,730,823,81.14,'Intermediate' from dual union all
select 38537,'038537','1983-109E','deb Meteor-2','',55866,'In Earth orbit','','2012-06-29 00:00:00',100.53,730,837,81.16,'Intermediate' from dual union all
select 38538,'038538','1983-109F','deb Meteor-2','',55866,'In Earth orbit','','2012-06-24 00:00:00',100.49,726,838,81.16,'Intermediate' from dual union all
select 38539,'038539','1988-005AS','deb Meteor-2','',61254,'In Earth orbit','','2012-06-18 00:00:00',103.78,916,958,82.54,'Intermediate' from dual union all
select 3854,'003854','1969-032A','Kosmos-276','Zenit-4 No. 55',18299,'Landed','1969-04-11 00:00:00','1969-04-06 00:00:00',90.18,198,369,81.35,'Intermediate' from dual union all
select 38540,'038540','1988-005AT','deb Meteor-2','',61254,'In Earth orbit','','2012-06-24 00:00:00',102.45,846,904,82.53,'Intermediate' from dual union all
select 38541,'038541','1988-005AU','deb Meteor-2','',61254,'In Earth orbit','','2012-06-27 00:00:00',103.69,923,942,82.54,'Intermediate' from dual union all
select 38542,'038542','1988-005AV','deb Meteor-2','',61254,'In Earth orbit','','2012-06-22 00:00:00',103.79,920,956,82.53,'Intermediate' from dual union all
select 38543,'038543','1988-005AW','deb Meteor-2','',61254,'In Earth orbit','','2012-06-28 00:00:00',102.42,857,889,82.54,'Intermediate' from dual union all
select 38544,'038544','1988-005AX','deb Meteor-2','',61254,'In Earth orbit','','2012-06-23 00:00:00',103.75,925,947,82.53,'Intermediate' from dual union all
select 38545,'038545','2006-016E','deb Cloudsat','',68188,'Reentered','2012-09-08 00:00:00','2012-06-27 00:00:00',95.47,534,549,98.41,'Sun-Synch' from dual union all
select 38546,'038546','1991-032F','deb NOAA 12','',64201,'In Earth orbit','','2012-06-26 00:00:00',100.65,782,797,98.72,'Sun-Synch' from dual union all
select 38547,'038547','1991-032G','deb NOAA 12','',64201,'In Earth orbit','','2012-06-27 00:00:00',100.83,795,801,98.73,'Sun-Synch' from dual union all
select 38548,'038548','1992-047AU','deb SOZ','',64759,'Reentered','2013-05-17 00:00:00','2012-06-29 00:00:00',294.66,895,15684,64.47,'Highly Ellip' from dual union all
select 38549,'038549','1968-014C','deb OGO 5','',16741,'In Earth orbit','','2012-06-25 00:00:00',624.1,175,35452,49.58,'GEO Transfer' from dual union all
select 3855,'003855','1969-033A','Kosmos-277','DS-P1-Yu No. 20',18300,'Reentered','1969-07-06 00:00:00','1969-05-05 00:00:00',91.21,256,412,70.95,'Intermediate' from dual union all
select 38550,'038550','2012-032H','Shenzhou 9 Guidao Cang','shenzhou jiu hao guidao',69287,'Reentered','2012-12-02 00:00:00','2012-06-29 00:00:00',91.4,333,354,42.78,'Intermediate' from dual union all
select 38551,'038551','2012-035A','Echostar 17','Jupiter 1',69290,'In Earth orbit','','2012-07-13 00:00:00',1426.72,35429,35777,.01,'Drift GEO' from dual union all
select 38552,'038552','2012-035B','Meteosat 10','MSG 3',69290,'In Earth orbit','','2012-07-18 00:00:00',1429.25,35649,35656,1.73,'Drift GEO' from dual union all
select 38553,'038553','2012-035C','ESC-A 563','Ariane 5 ESC-A',69290,'In Earth orbit','','2012-07-13 00:00:00',630.3,253,35694,6.38,'GEO Transfer' from dual union all
select 38554,'038554','2012-035D','Sylda 5B','Sylda No. 51-B',69290,'In Earth orbit','','2012-07-13 00:00:00',630.94,257,35722,5.99,'GEO Transfer' from dual union all
select 38555,'038555','2002-032B','deb NOAA 17','',67408,'Reentered','2013-09-04 00:00:00','2012-07-02 00:00:00',96.78,592,616,98.77,'Sun-Synch' from dual union all
select 38556,'038556','1989-001X','deb SOZ','',62278,'Reentered','2013-05-16 00:00:00','2012-07-07 00:00:00',121.26,616,2859,64.68,'Medium' from dual union all
select 38557,'038557','2006-006CX','deb Briz-M 88515','',68178,'In Earth orbit','','2012-07-07 00:00:00',264.37,490,14043,51.87,'Medium' from dual union all
select 38558,'038558','2006-006CY','deb Briz-M 88515','',68178,'In Earth orbit','','2012-06-27 00:00:00',214.31,319,10650,51.24,'Medium' from dual union all
select 38559,'038559','2006-006CZ','deb Briz-M 88515','',68178,'In Earth orbit','','2012-07-06 00:00:00',278.95,557,14971,52.09,'Highly Ellip' from dual union all
select 3856,'003856','1969-032B','11A57I','11A57I',18299,'Reentered','1969-04-14 00:00:00','1969-04-07 00:00:00',89.87,201,335,81.35,'Intermediate' from dual union all
select 38560,'038560','2006-006DA','deb Briz-M 88515','',68178,'Lost','','2012-06-10 00:00:00',274.08,455,14742,51.82,'Highly Ellip' from dual union all
select 38561,'038561','2006-006DB','deb Briz-M 88515','',68178,'Reentered','2012-09-28 00:00:00','2012-07-07 00:00:00',122.88,345,3273,51.39,'Medium' from dual union all
select 38562,'038562','2006-006DC','deb Briz-M 88515','',68178,'In Earth orbit','','2012-07-07 00:00:00',272.3,478,14599,51.82,'Highly Ellip' from dual union all
select 38563,'038563','2006-006DD','deb Briz-M 88515','',68178,'In Earth orbit','','2012-06-13 00:00:00',277.07,481,14918,51.67,'Highly Ellip' from dual union all
select 38564,'038564','2006-006DE','deb Briz-M 88515','',68178,'In Earth orbit','','2012-07-02 00:00:00',274.13,480,14721,51.64,'Highly Ellip' from dual union all
select 38565,'038565','2008-011L','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',464.94,639,26366,49.22,'Highly Ellip' from dual union all
select 38566,'038566','2008-011M','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',457.86,431,26169,49.07,'Highly Ellip' from dual union all
select 38567,'038567','2008-011N','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-05 00:00:00',488.02,1318,26992,49.37,'Highly Ellip' from dual union all
select 38568,'038568','2008-011P','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',465.47,455,26580,49.57,'Highly Ellip' from dual union all
select 38569,'038569','2008-011Q','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-06-27 00:00:00',477.33,1012,26696,49.05,'Highly Ellip' from dual union all
select 3857,'003857','1969-033B','S1M','S1M',18300,'Reentered','1969-05-18 00:00:00','1969-04-13 00:00:00',91.49,261,435,70.95,'Intermediate' from dual union all
select 38570,'038570','2008-011R','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',478.6,740,27040,49.12,'Highly Ellip' from dual union all
select 38571,'038571','2008-011S','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-04 00:00:00',482.22,1187,26797,49.62,'Highly Ellip' from dual union all
select 38572,'038572','2008-011T','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-06-18 00:00:00',474.96,1027,26547,48.83,'Highly Ellip' from dual union all
select 38573,'038573','2008-011U','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',476.71,1164,26509,49.09,'Highly Ellip' from dual union all
select 38574,'038574','2008-011V','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-06-26 00:00:00',477.15,1261,26437,48.9,'Highly Ellip' from dual union all
select 38575,'038575','2008-011W','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',448.43,621,25437,49.29,'Highly Ellip' from dual union all
select 38576,'038576','2008-011X','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-03 00:00:00',462.04,526,26313,49.29,'Highly Ellip' from dual union all
select 38577,'038577','2008-011Y','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',480.71,680,27219,49.22,'Highly Ellip' from dual union all
select 38578,'038578','2008-011Z','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',479.81,625,27223,47.93,'Highly Ellip' from dual union all
select 38579,'038579','2008-011AA','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',481.78,751,27208,49.12,'Highly Ellip' from dual union all
select 3858,'003858','1969-029H','deb Blok-E','',18296,'Reentered','1970-05-18 00:00:00','1970-05-11 00:00:00',91.52,318,381,80.91,'Intermediate' from dual union all
select 38580,'038580','2008-011AB','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-06-25 00:00:00',490.2,727,27705,48.43,'Highly Ellip' from dual union all
select 38581,'038581','2008-011AC','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',439.38,514,25020,49.44,'Highly Ellip' from dual union all
select 38582,'038582','2008-011AD','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',479.77,532,27314,48.85,'Highly Ellip' from dual union all
select 38583,'038583','2008-011AE','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',464.54,553,26429,49.36,'Highly Ellip' from dual union all
select 38584,'038584','2008-011AF','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-06-28 00:00:00',470.2,709,26595,48.82,'Highly Ellip' from dual union all
select 38585,'038585','2008-011AG','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-05 00:00:00',454.86,465,25963,49.34,'Highly Ellip' from dual union all
select 38586,'038586','2008-011AH','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',492.79,1212,27365,49.58,'Highly Ellip' from dual union all
select 38587,'038587','2008-011AJ','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-01 00:00:00',467.37,764,26379,49.12,'Highly Ellip' from dual union all
select 38588,'038588','2008-011AK','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',475.97,806,26825,50.06,'Highly Ellip' from dual union all
select 38589,'038589','2008-011AL','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',487.24,937,27329,48.69,'Highly Ellip' from dual union all
select 3859,'003859','1969-029J','deb Blok-E','',18296,'Reentered','1979-11-28 00:00:00','1979-11-17 00:00:00',90.53,287,314,81.14,'Intermediate' from dual union all
select 38590,'038590','2008-011AM','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',499.02,1083,27842,49.01,'Highly Ellip' from dual union all
select 38591,'038591','2008-011AN','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-05 00:00:00',498.86,514,28402,49.11,'Highly Ellip' from dual union all
select 38592,'038592','2008-011AP','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-06-29 00:00:00',461.91,668,26164,48.81,'Highly Ellip' from dual union all
select 38593,'038593','2008-011AQ','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',475.6,596,27014,49.28,'Highly Ellip' from dual union all
select 38594,'038594','2008-011AR','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-04 00:00:00',484.69,762,27361,49.21,'Highly Ellip' from dual union all
select 38595,'038595','2008-011AS','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-05 00:00:00',464.73,587,26406,50.05,'Highly Ellip' from dual union all
select 38596,'038596','2008-011AT','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-02 00:00:00',469.9,881,26406,49.15,'Highly Ellip' from dual union all
select 38597,'038597','2008-011AU','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',466.45,463,26628,49.35,'Highly Ellip' from dual union all
select 38598,'038598','2008-011AV','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',466.63,686,26415,49.16,'Highly Ellip' from dual union all
select 38599,'038599','2008-011AW','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-06-22 00:00:00',507.02,727,28642,48.78,'Highly Ellip' from dual union all
select 386,'000386','62 A Pi 7','deb Sputnik','',8151,'Reentered','1962-09-08 00:00:00','1962-09-02 00:00:00',90.29,152,426,65.31,'Intermediate' from dual union all
select 3860,'003860','1969-029K','deb Blok-E','',18296,'Reentered','1971-12-26 00:00:00','1971-12-15 00:00:00',90.38,275,312,81.17,'Intermediate' from dual union all
select 38600,'038600','2008-011AX','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-01 00:00:00',463.19,559,26346,49.14,'Highly Ellip' from dual union all
select 38601,'038601','2008-011AY','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',449.46,345,25772,48.39,'Highly Ellip' from dual union all
select 38602,'038602','2008-011AZ','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',467.23,490,26645,48.97,'Highly Ellip' from dual union all
select 38603,'038603','2008-011BA','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',463.14,592,26310,49.84,'Highly Ellip' from dual union all
select 38604,'038604','2008-011BB','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',466.14,681,26392,49.26,'Highly Ellip' from dual union all
select 38605,'038605','2008-011BC','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',468.81,832,26393,49.17,'Highly Ellip' from dual union all
select 38606,'038606','2008-011BD','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',469.71,558,26718,48.68,'Highly Ellip' from dual union all
select 38607,'038607','2008-011BE','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',470.73,924,26410,49.71,'Highly Ellip' from dual union all
select 38608,'038608','2008-011BF','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-01 00:00:00',440.5,365,25234,48.64,'Highly Ellip' from dual union all
select 38609,'038609','2008-011BG','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',463.79,474,26465,48.76,'Highly Ellip' from dual union all
select 3861,'003861','1969-029L','deb Blok-E','',18296,'Reentered','1974-06-04 00:00:00','1974-06-01 00:00:00',89.01,219,233,81.31,'Intermediate' from dual union all
select 38610,'038610','2008-011BH','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',470.6,940,26387,49.17,'Highly Ellip' from dual union all
select 38611,'038611','2008-011BJ','deb Briz-M No. 88525','',68519,'Reentered','2013-05-17 00:00:00','2012-06-28 00:00:00',253.65,252,13538,49.21,'Highly Ellip' from dual union all
select 38612,'038612','2008-011BK','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',479.05,738,27067,47.9,'Highly Ellip' from dual union all
select 38613,'038613','2008-011BL','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',464.75,446,26548,48.94,'Highly Ellip' from dual union all
select 38614,'038614','2008-011BM','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',475.38,1210,26388,49.26,'Highly Ellip' from dual union all
select 38615,'038615','2008-011BN','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',463.66,581,26351,49.25,'Highly Ellip' from dual union all
select 38616,'038616','2008-011BP','deb Briz-M No. 88525','',68519,'Lost','','2012-06-25 00:00:00',278.92,306,15220,50.48,'Highly Ellip' from dual union all
select 38617,'038617','2008-011BQ','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-06-28 00:00:00',449.94,593,25552,49.37,'Highly Ellip' from dual union all
select 38618,'038618','2008-011BR','deb Briz-M No. 88525','',68519,'Lost','','2012-06-25 00:00:00',467.84,505,26665,49,'Highly Ellip' from dual;
insert into SATELLITE
select 38619,'038619','2008-011BS','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',470.87,959,26383,49.16,'Highly Ellip' from dual union all
select 3862,'003862','1969-029M','deb Blok-E','',18296,'Reentered','1970-12-13 00:00:00','1970-12-07 00:00:00',90.28,269,308,81.3,'Intermediate' from dual union all
select 38620,'038620','2008-011BT','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',468.42,812,26391,49.16,'Highly Ellip' from dual union all
select 38621,'038621','2008-011BU','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',476.27,841,26807,49.07,'Highly Ellip' from dual union all
select 38622,'038622','2008-011BV','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',466.42,622,26467,49.28,'Highly Ellip' from dual union all
select 38623,'038623','2008-011BW','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-05 00:00:00',471.34,925,26444,49.04,'Highly Ellip' from dual union all
select 38624,'038624','2008-011BX','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',463.77,586,26352,48.85,'Highly Ellip' from dual union all
select 38625,'038625','2008-011BY','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-03 00:00:00',460.55,513,26241,49.15,'Highly Ellip' from dual union all
select 38626,'038626','2008-011BZ','deb Briz-M No. 88525','',68519,'Lost','','2012-07-04 00:00:00',458.77,424,26228,48.99,'Highly Ellip' from dual union all
select 38627,'038627','2008-011CA','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-06-21 00:00:00',480.78,633,27270,49.7,'Highly Ellip' from dual union all
select 38628,'038628','2008-011CB','deb Briz-M No. 88525','',68519,'Lost','','2012-07-07 00:00:00',468.9,529,26701,48.97,'Highly Ellip' from dual union all
select 38629,'038629','2008-011CC','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',440.98,405,25222,50.22,'Highly Ellip' from dual union all
select 3863,'003863','1969-029N','deb Blok-E','',18296,'Reentered','1972-06-18 00:00:00','1972-06-16 00:00:00',89.57,248,259,81.05,'Intermediate' from dual union all
select 38630,'038630','2008-011CD','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',479.06,518,27288,49.31,'Highly Ellip' from dual union all
select 38631,'038631','2008-011CE','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-05 00:00:00',465.12,692,26323,49.41,'Highly Ellip' from dual union all
select 38632,'038632','2008-011CF','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-03 00:00:00',463.8,547,26393,49.25,'Highly Ellip' from dual union all
select 38633,'038633','2008-011CG','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-05 00:00:00',507.11,704,28670,49.63,'Highly Ellip' from dual union all
select 38634,'038634','2008-011CH','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-06-15 00:00:00',485.31,920,27238,48.99,'Highly Ellip' from dual union all
select 38635,'038635','2008-011CJ','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-02 00:00:00',469.11,770,26472,48.87,'Highly Ellip' from dual union all
select 38636,'038636','2008-011CK','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-02 00:00:00',468.05,583,26599,49.36,'Highly Ellip' from dual union all
select 38637,'038637','2008-011CL','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-06-12 00:00:00',469.44,860,26401,49.19,'Highly Ellip' from dual union all
select 38638,'038638','2008-011CM','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-06-29 00:00:00',466.7,499,26606,48.88,'Highly Ellip' from dual union all
select 38639,'038639','2008-011CN','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-05 00:00:00',475.19,797,26790,49.66,'Highly Ellip' from dual union all
select 3864,'003864','1969-029P','deb Blok-E','',18296,'Reentered','1970-02-23 00:00:00','1970-02-11 00:00:00',92.82,380,446,81.31,'Intermediate' from dual union all
select 38640,'038640','2008-011CP','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',477.03,826,26865,48.89,'Highly Ellip' from dual union all
select 38641,'038641','2008-011CQ','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',476.18,1212,26431,49.09,'Highly Ellip' from dual union all
select 38642,'038642','2008-011CR','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-06-20 00:00:00',467.42,775,26371,48.81,'Highly Ellip' from dual union all
select 38643,'038643','2008-011CS','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',470.16,572,26730,49.69,'Highly Ellip' from dual union all
select 38644,'038644','2008-011CT','deb Briz-M No. 88525','',68519,'Lost','','2012-07-07 00:00:00',445.02,593,25268,49.27,'Highly Ellip' from dual union all
select 38645,'038645','2008-011CU','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-04 00:00:00',469.95,850,26440,49.09,'Highly Ellip' from dual union all
select 38646,'038646','2008-011CV','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',470.67,923,26408,49.17,'Highly Ellip' from dual union all
select 38647,'038647','2008-011CW','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',471.68,1001,26387,49.14,'Highly Ellip' from dual union all
select 38648,'038648','2008-011CX','deb Briz-M No. 88525','Large debris (9 sq m)',68519,'In Earth orbit','','2012-07-07 00:00:00',475.14,674,26910,49.17,'Highly Ellip' from dual union all
select 38649,'038649','2008-011CY','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-06 00:00:00',462.89,540,26348,49.01,'Highly Ellip' from dual union all
select 3865,'003865','1969-029Q','deb Blok-E','',18296,'Reentered','1973-04-25 00:00:00','1973-04-17 00:00:00',89.18,220,248,81.27,'Intermediate' from dual union all
select 38650,'038650','2008-011CZ','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-06-29 00:00:00',469.71,885,26391,49.14,'Highly Ellip' from dual union all
select 38651,'038651','2008-011DA','deb Briz-M No. 88525','',68519,'In Earth orbit','','2012-07-07 00:00:00',470.57,916,26409,49.16,'Highly Ellip' from dual union all
select 38652,'038652','2012-036A','SES-5','Astra 4B',69291,'In Earth orbit','','2012-07-16 00:00:00',1433.71,35705,35775,.13,'Drift GEO' from dual union all
select 38653,'038653','2012-036B','Briz-M No. 99531','Briz-M 14S43 No. 99531',69291,'In Earth orbit','','2012-07-20 00:00:00',663.1,3512,34110,22.83,'GEO Transfer' from dual union all
select 38654,'038654','2012-036C','Briz-M DTB','Briz-M torus tank',69291,'In Earth orbit','','2012-07-17 00:00:00',277.54,328,15105,49.63,'Highly Ellip' from dual union all
select 38655,'038655','1988-005AY','deb Meteor-2','',61254,'In Earth orbit','','2012-07-02 00:00:00',103.36,899,936,82.55,'Intermediate' from dual union all
select 38656,'038656','1988-005AZ','deb Meteor-2','',61254,'In Earth orbit','','2012-06-25 00:00:00',103.62,918,941,82.52,'Intermediate' from dual union all
select 38657,'038657','2009-072E','CZ-4C sep motor cover','',68753,'In Earth orbit','','2012-07-06 00:00:00',108.92,1076,1278,100.46,'Sun-Synch' from dual union all
select 38658,'038658','2009-072F','CZ-4C sep motor cover','',68753,'In Earth orbit','','2012-07-07 00:00:00',110.36,1141,1346,100.26,'Sun-Synch' from dual union all
select 38659,'038659','2009-072G','CZ-4C sep motor cover','',68753,'In Earth orbit','','2012-07-10 00:00:00',110.35,1140,1346,100.46,'Sun-Synch' from dual union all
select 3866,'003866','1969-029R','deb Blok-E','',18296,'Reentered','1969-06-01 00:00:00','1969-05-29 00:00:00',91.1,304,353,81.06,'Intermediate' from dual union all
select 38660,'038660','2009-072H','CZ-4C sep motor cover','',68753,'In Earth orbit','','2012-07-06 00:00:00',111.43,1173,1412,100.41,'Sun-Synch' from dual union all
select 38661,'038661','2012-029C','CZ-4C sep motor cover','',69284,'In Earth orbit','','2012-07-10 00:00:00',109.18,1092,1286,100.11,'Sun-Synch' from dual union all
select 38662,'038662','2012-029D','CZ-4C sep motor cover','',69284,'In Earth orbit','','2012-07-10 00:00:00',109.19,1090,1289,100.15,'Sun-Synch' from dual union all
select 38663,'038663','2012-029E','CZ-4C sep motor cover','',69284,'In Earth orbit','','2012-07-11 00:00:00',110.08,1126,1334,100.17,'Sun-Synch' from dual union all
select 38664,'038664','2012-029F','CZ-4C sep motor cover','',69284,'In Earth orbit','','2012-07-06 00:00:00',109.77,1112,1321,100.1,'Sun-Synch' from dual union all
select 38665,'038665','1971-035E','deb Kosmos-407','Strela-2M debris',24853,'In Earth orbit','','2012-07-07 00:00:00',100.25,748,793,74.05,'Intermediate' from dual union all
select 38666,'038666','1971-035F','deb Kosmos-407','Strela-2M debris',24853,'In Earth orbit','','2012-07-03 00:00:00',100.28,751,793,74.05,'Intermediate' from dual union all
select 38667,'038667','1974-071E','deb Kosmos-676','Strela-2M debris',33674,'In Earth orbit','','2012-06-28 00:00:00',100.33,759,789,74.05,'Intermediate' from dual union all
select 38668,'038668','1979-058W','deb Kosmos-1109','deb Oko',47542,'Reentered','2012-07-09 00:00:00','2012-07-06 00:00:00',129.33,135,4052,63.14,'Medium' from dual union all
select 38669,'038669','1979-058X','deb Kosmos-1109','deb Oko',47542,'Reentered','2013-01-06 00:00:00','2012-07-09 00:00:00',724.68,300,40395,63.41,'Molniya' from dual union all
select 3867,'003867','1969-029S','deb Blok-E','',18296,'Reentered','1971-02-13 00:00:00','1971-02-10 00:00:00',89.17,214,253,81.3,'Intermediate' from dual union all
select 38670,'038670','2007-003AJ','deb CZ-3','',68345,'In Earth orbit','','2012-07-01 00:00:00',792.78,891,43102,26.07,'Highly Ellip' from dual union all
select 38671,'038671','2012-037A','Soyuz TMA-05M','Soyuz 11F732A47 No. 706',69292,'Landed','2012-11-19 00:00:00','2012-07-16 00:00:00',90.92,292,348,51.64,'Intermediate' from dual union all
select 38672,'038672','2012-037B','11S510','11S510',69292,'Reentered','2012-07-17 00:00:00','2012-07-16 00:00:00',88.45,184,212,51.63,'Intermediate' from dual union all
select 38673,'038673','1986-010H','deb CZ-3 Stage 3','',59203,'In Earth orbit','2017-05-01 00:00:00','2012-06-20 00:00:00',526.15,404,30018,30.4,'Highly Ellip' from dual union all
select 38674,'038674','2006-042D','Star 48B yo weight','',68214,'In Earth orbit','','2012-07-15 00:00:00',294.11,183,16360,39.91,'Highly Ellip' from dual union all
select 38675,'038675','2006-052D','Star 48 yo weight','',68224,'Reentered','2014-10-26 00:00:00','2012-07-16 00:00:00',175.42,160,7845,39.97,'Medium' from dual union all
select 38676,'038676','2007-062D','Star 48 yo weight','',68404,'Reentered','2013-09-11 00:00:00','2012-07-09 00:00:00',170.33,140,7461,39.89,'Medium' from dual union all
select 38677,'038677','2008-012D','Star 48 yo weight','',68520,'Reentered','2015-04-01 00:00:00','2012-06-19 00:00:00',306.39,206,17148,40.14,'Highly Ellip' from dual union all
select 38678,'038678','2009-014D','Star 48 yo weight','',68695,'In Earth orbit','','2012-07-04 00:00:00',289.19,194,16022,40.22,'Highly Ellip' from dual union all
select 38679,'038679','2009-067C','deb Blok DM-SLB 4L','',68748,'In Earth orbit','','2012-07-17 00:00:00',830.8,9531,36262,13.01,'Medium' from dual union all
select 3868,'003868','1969-029T','deb Blok-E','',18296,'Reentered','1970-02-04 00:00:00','1970-02-02 00:00:00',90.36,275,310,81.15,'Intermediate' from dual union all
select 38680,'038680','1996-021E','deb Blok DM3','',65909,'In Earth orbit','','2012-06-21 00:00:00',878.62,12915,35104,6.17,'Medium' from dual union all
select 38681,'038681','1996-021F','deb Blok DM3','',65909,'In Earth orbit','','2012-07-16 00:00:00',876.02,12802,35097,6.26,'Medium' from dual union all
select 38682,'038682','1979-009E','SVM-2B debris','SVM-2B debris',47493,'In Earth orbit','','2012-07-16 00:00:00',854.34,12463,34431,8.75,'Medium' from dual union all
select 38683,'038683','1979-009F','SVM-2B debris','SVM-2B debris',47493,'In Earth orbit','','2012-07-18 00:00:00',849.6,12346,34327,7.67,'Medium' from dual union all
select 38684,'038684','1980-018E','deb Ayame 2','',50228,'In Earth orbit','','2012-07-16 00:00:00',812.96,9612,35340,8.71,'Medium' from dual union all
select 38685,'038685','1980-018F','deb Ayame 2','',50228,'In Earth orbit','','2012-07-17 00:00:00',817.34,9846,35313,8.6,'Medium' from dual union all
select 38686,'038686','1979-101V','deb Satcom 3','',47585,'In Earth orbit','','2012-06-27 00:00:00',790.18,8202,35667,8.33,'Medium' from dual union all
select 38687,'038687','1979-101W','deb Satcom 3','',47585,'In Earth orbit','','2012-07-18 00:00:00',799.54,8808,35507,9.92,'Medium' from dual union all
select 38688,'038688','1979-101X','deb Satcom 3','',47585,'In Earth orbit','','2012-07-17 00:00:00',796.72,7884,36297,8.03,'Medium' from dual union all
select 38689,'038689','1979-101Y','deb Satcom 3','',47585,'In Earth orbit','','2012-07-15 00:00:00',795.53,8499,35625,9.09,'Medium' from dual union all
select 3869,'003869','1969-029U','deb Blok-E','',18296,'Reentered','1971-06-01 00:00:00','1969-04-25 00:00:00',98.49,480,893,81.38,'Intermediate' from dual union all
select 38690,'038690','1968-081Q','Transtage 5 debris','',16808,'In Earth orbit','','2012-07-17 00:00:00',1449.65,34952,37151,9,'Inclined Drift' from dual union all
select 38691,'038691','1968-081R','Transtage 5 debris','',16808,'In Earth orbit','','2012-06-30 00:00:00',1429.86,34883,36446,7.98,'Inclined Drift' from dual union all
select 38692,'038692','1968-081S','Transtage 5 debris','',16808,'In Earth orbit','','2012-07-11 00:00:00',1452.13,34778,37422,9,'Inclined Drift' from dual union all
select 38693,'038693','1968-081T','Transtage 5 debris','',16808,'In Earth orbit','','2012-07-17 00:00:00',1400.42,34655,35515,7.21,'Inclined Drift' from dual union all
select 38694,'038694','1968-081U','Transtage 5 debris','',16808,'In Earth orbit','','2012-07-12 00:00:00',1391.21,34500,35307,8.28,'Inclined Drift' from dual union all
select 38695,'038695','1968-081V','Transtage 5 debris','',16808,'In Earth orbit','','2012-07-16 00:00:00',1449.88,34211,37901,8.64,'Inclined Drift' from dual union all
select 38696,'038696','1968-081W','Transtage 5 debris','',16808,'In Earth orbit','','2012-07-10 00:00:00',1447.37,35152,36862,8.6,'Inclined Drift' from dual union all
select 38697,'038697','1968-081X','Unk GEO debris                    ','',16808,'In Earth orbit','','2012-07-18 00:00:00',1419.03,32511,38393,6.69,'Near-synch' from dual union all
select 38698,'038698','1968-081Y','Transtage 5 debris','Large debris 5.0m2',16808,'In Earth orbit','','2012-07-17 00:00:00',1379.45,34101,35240,6.77,'Medium' from dual union all
select 38699,'038699','1968-081Z','Transtage 5 debris','',16808,'In Earth orbit','','2012-07-17 00:00:00',1427.72,35187,36058,8.41,'Inclined Drift' from dual union all
select 387,'000387','62 A Pi 8','deb Sputnik','',8151,'Reentered','1962-09-05 00:00:00','1962-09-01 00:00:00',90.02,211,340,65.33,'Intermediate' from dual union all
select 3870,'003870','1969-029V','deb Blok-E','',18296,'Reentered','1970-08-26 00:00:00','1970-08-17 00:00:00',90.36,281,304,81.12,'Intermediate' from dual union all
select 38700,'038700','1968-081AA','Transtage 5 debris','',16808,'In Earth orbit','','2012-07-19 00:00:00',1401.69,34648,35573,7.9,'Inclined Drift' from dual union all
select 38701,'038701','1968-081AB','Transtage 5 debris','',16808,'In Earth orbit','','2012-07-18 00:00:00',1423.41,34799,36277,7.95,'Inclined Drift' from dual union all
select 38702,'038702','1968-081AC','Transtage 5 debris','',16808,'In Earth orbit','','2012-07-15 00:00:00',1408.97,34435,36073,9.01,'Inclined Drift' from dual union all
select 38703,'038703','1968-081AD','Transtage 5 debris','',16808,'In Earth orbit','','2012-07-18 00:00:00',1365.89,33345,35457,7.43,'Medium' from dual union all
select 38704,'038704','1968-081AE','Transtage 5 debris','',16808,'In Earth orbit','','2012-07-16 00:00:00',1408.84,34544,35958,6.27,'Inclined Drift' from dual union all
select 38705,'038705','1968-081AF','Transtage 5 debris','',16808,'In Earth orbit','','2012-07-09 00:00:00',1411.23,34397,36199,7.11,'Inclined Drift' from dual union all
select 38706,'038706','2012-038A','Kounotori 3 gouki','HTV 3',69293,'Deorbited','2012-09-14 00:00:00','2012-07-23 00:00:00',89.83,233,300,51.66,'Intermediate' from dual union all
select 38707,'038707','2012-039A','Kanopus-V','Kanopus-V',69294,'In Earth orbit','','2012-07-29 00:00:00',94.7,501,508,97.49,'Sun-Synch' from dual union all
select 38708,'038708','2012-039B','BelKA-2','BelKA-2',69294,'In Earth orbit','','2012-07-29 00:00:00',94.76,505,508,97.49,'Sun-Synch' from dual;
insert into SATELLITE
select 38709,'038709','2012-039C','exactView-1','ADS-1B',69294,'In Earth orbit','','2012-07-29 00:00:00',101.14,805,821,98.95,'Sun-Synch' from dual union all
select 3871,'003871','1969-029W','deb Blok-E','',18296,'Reentered','1982-06-03 00:00:00','1982-06-03 00:00:00',94.59,412,586,81.34,'Intermediate' from dual union all
select 38710,'038710','2012-039D','TET-1','TET-1',69294,'In Earth orbit','','2012-07-29 00:00:00',94.74,503,508,97.49,'Sun-Synch' from dual union all
select 38711,'038711','2012-039E','MKA-PN1','MKA-FKI',69294,'In Earth orbit','','2012-07-29 00:00:00',101.13,804,820,98.94,'Sun-Synch' from dual union all
select 38712,'038712','2012-039F','Fregat No. 1019','Fregat 14S44 No. 1019',69294,'Deorbited','2012-07-22 00:00:00','2012-07-22 00:00:00',101.38,814,834,98.8,'Sun-Synch' from dual union all
select 38713,'038713','2012-039G','11S510','11S510',69294,'Reentered','2012-07-24 00:00:00','2012-07-23 00:00:00',87.78,159,171,95.45,'Sun-Synch' from dual union all
select 38714,'038714','1999-025ETF','deb FY-1C','',66706,'In Earth orbit','','2012-07-24 00:00:00',102.03,633,1077,97.94,'Sun-Synch' from dual union all
select 38715,'038715','1999-025ETG','deb FY-1C','',66706,'In Earth orbit','','2012-07-24 00:00:00',103.22,783,1039,98.22,'Sun-Synch' from dual union all
select 38716,'038716','1999-025ETH','deb FY-1C','',66706,'In Earth orbit','','2012-07-18 00:00:00',106.24,843,1261,98.46,'Sun-Synch' from dual union all
select 38717,'038717','1999-025ETJ','deb FY-1C','',66706,'In Earth orbit','','2012-07-20 00:00:00',102.91,843,950,98.93,'Sun-Synch' from dual union all
select 38718,'038718','1999-025ETK','deb FY-1C','',66706,'In Earth orbit','','2012-07-20 00:00:00',104.95,828,1157,98.62,'Sun-Synch' from dual union all
select 38719,'038719','1999-025ETL','deb FY-1C','',66706,'Reentered','2013-06-14 00:00:00','2012-07-17 00:00:00',96.38,506,665,98.88,'Sun-Synch' from dual union all
select 3872,'003872','1968-091BF','deb Kosmos-249','',16818,'Reentered','1970-02-15 00:00:00','1969-04-13 00:00:00',107.1,340,1845,62.28,'Intermediate' from dual union all
select 38720,'038720','1999-025ETM','deb FY-1C','',66706,'In Earth orbit','','2012-07-21 00:00:00',102.67,830,940,99.12,'Sun-Synch' from dual union all
select 38721,'038721','1999-025ETN','deb FY-1C','',66706,'In Earth orbit','','2012-07-24 00:00:00',102.28,806,928,99.09,'Sun-Synch' from dual union all
select 38722,'038722','1999-025ETP','deb FY-1C','',66706,'In Earth orbit','','2012-07-23 00:00:00',102.41,666,1080,99.34,'Sun-Synch' from dual union all
select 38723,'038723','1999-025ETQ','deb FY-1C','',66706,'In Earth orbit','','2012-07-11 00:00:00',101.52,791,871,99.45,'Sun-Synch' from dual union all
select 38724,'038724','1999-025ETR','deb FY-1C','',66706,'In Earth orbit','','2012-07-24 00:00:00',104.44,837,1100,98.78,'Sun-Synch' from dual union all
select 38725,'038725','1999-025ETS','deb FY-1C','',66706,'In Earth orbit','','2012-07-22 00:00:00',102.15,823,898,99.06,'Sun-Synch' from dual union all
select 38726,'038726','1999-025ETT','deb FY-1C','',66706,'In Earth orbit','','2012-07-22 00:00:00',101.98,782,923,99.17,'Sun-Synch' from dual union all
select 38727,'038727','1999-025ETU','deb FY-1C','',66706,'In Earth orbit','','2012-07-21 00:00:00',103.73,843,1027,98.67,'Sun-Synch' from dual union all
select 38728,'038728','1999-025ETV','deb FY-1C','',66706,'In Earth orbit','','2012-07-18 00:00:00',100,673,844,99.29,'Sun-Synch' from dual union all
select 38729,'038729','1999-025ETW','deb FY-1C','',66706,'In Earth orbit','','2012-07-18 00:00:00',103.27,838,989,99.04,'Sun-Synch' from dual union all
select 3873,'003873','1968-091BG','deb Kosmos-249 [*]','',16818,'Reentered','1989-11-03 00:00:00','1989-11-03 00:00:00',88.83,210,223,62.3,'Intermediate' from dual union all
select 38730,'038730','2012-040A','Tian Lian 1-03','tianlian yihao 03 xing',69295,'In Earth orbit','','2012-08-25 00:00:00',1436.09,35773,35799,2.03,'Stationary' from dual union all
select 38731,'038731','2012-040B','CZ-3C Stage 3','CZ-3C Stage 3',69295,'Reentered','2012-11-22 00:00:00','2012-08-01 00:00:00',768.2,180,42634,18.14,'Highly Ellip' from dual union all
select 38732,'038732','1994-089D','deb NOAA-14','',65485,'In Earth orbit','','2012-07-26 00:00:00',101.78,833,853,98.81,'Sun-Synch' from dual union all
select 38733,'038733','2012-041A','Kosmos-2481','Strela-3 No. 57',69296,'In Earth orbit','','2012-07-30 00:00:00',115.89,1482,1509,82.48,'Intermediate' from dual union all
select 38734,'038734','2012-041B','Gonets-M No. 13','Gonets-M No. 13',69296,'In Earth orbit','','2012-07-30 00:00:00',115.85,1480,1508,82.48,'Intermediate' from dual union all
select 38735,'038735','2012-041C','Mikhail Reshetnyov','Yubileyniy-2',69296,'In Earth orbit','','2012-07-30 00:00:00',115.87,1482,1508,82.48,'Intermediate' from dual union all
select 38736,'038736','2012-041D','Gonets-M No. 15','Gonets-M No. 15',69296,'In Earth orbit','','2012-07-30 00:00:00',115.87,1481,1509,82.48,'Intermediate' from dual union all
select 38737,'038737','2012-041E','Briz-KM No. 72515','Briz-KM 14S45',69296,'In Earth orbit','','2012-07-30 00:00:00',112.33,1164,1503,82.48,'Intermediate' from dual union all
select 38738,'038738','2012-042A','Progress M-16M','Progress 7K-TGM No.416',69297,'Deorbited','2013-02-09 00:00:00','2012-08-01 00:00:00',90.17,263,303,51.64,'Intermediate' from dual union all
select 38739,'038739','2012-042B','11S510','11S510',69297,'Reentered','2012-08-04 00:00:00','2012-08-03 00:00:00',88.14,172,193,51.63,'Intermediate' from dual union all
select 3874,'003874','1968-091BH','deb Kosmos-249','',16818,'In Earth orbit','','1993-08-05 00:00:00',112.27,726,1936,62.27,'Intermediate' from dual union all
select 38740,'038740','2012-043A','Intelsat IS-20','Intelsat IS-20',69298,'In Earth orbit','','2012-09-02 00:00:00',1434.58,35745,35769,.03,'Drift GEO' from dual union all
select 38741,'038741','2012-043B','Hylas 2','Hylas 2',69298,'In Earth orbit','','2012-09-04 00:00:00',1436.04,35774,35797,.05,'Stationary' from dual union all
select 38742,'038742','2012-043C','ESC-A 564','Ariane 5 ESC-A',69298,'In Earth orbit','','2012-09-02 00:00:00',629.93,264,35664,5.78,'GEO Transfer' from dual union all
select 38743,'038743','2012-043D','Sylda 5','Sylda 5 No. 50B',69298,'In Earth orbit','','2012-09-02 00:00:00',628.96,266,35612,6.06,'GEO Transfer' from dual union all
select 38744,'038744','2012-044A','Telkom-3','Telkom-3',69299,'In Earth orbit','','2012-08-13 00:00:00',142.03,266,5013,49.9,'Medium' from dual union all
select 38745,'038745','2012-044B','Ekspress MD-2','Ekspress MD-2',69299,'In Earth orbit','','2012-08-13 00:00:00',142,266,5010,49.9,'Medium' from dual union all
select 38746,'038746','2012-044C','Briz-M No. 99532','Briz-M 14S43 No. 99532',69299,'In Earth orbit','','2012-08-14 00:00:00',142.04,265,5015,49.91,'Medium' from dual union all
select 38747,'038747','2012-044D','Adapter','Adapter',69299,'In Earth orbit','','2012-08-15 00:00:00',141.93,266,5003,49.9,'Medium' from dual union all
select 38748,'038748','2010-042C','deb CZ-3B','',68902,'In Earth orbit','','2012-08-12 00:00:00',729.51,350,40582,25.94,'GEO Transfer' from dual union all
select 38749,'038749','2012-045A','Intelsat IS-21','Intelsat IS-21',69300,'In Earth orbit','','2012-08-26 00:00:00',1421.5,35214,35786,.07,'Drift GEO' from dual union all
select 3875,'003875','1968-091BJ','deb Kosmos-249','',16818,'Reentered','1981-05-27 00:00:00','1969-02-17 00:00:00',112.71,557,2146,62.34,'Intermediate' from dual union all
select 38750,'038750','2012-045B','Blok DM-SL No. 33L','Blok DM-SL',69300,'In Earth orbit','','2012-08-26 00:00:00',628.38,275,35573,.08,'GEO Transfer' from dual union all
select 38751,'038751','1998-067CM','Sfera-53','Vektor-T',66503,'Reentered','2012-11-24 00:00:00','2012-08-03 00:00:00',92.6,401,403,51.64,'Intermediate' from dual union all
select 38752,'038752','2012-046A','Radiation Belt Storm Probe A','RBSP A',69301,'In Earth orbit','','2012-09-06 00:00:00',539,595,30527,10.03,'Highly Ellip' from dual union all
select 38753,'038753','2012-046B','Radiation Belt Storm Probe B','RBSP B',69301,'In Earth orbit','','2012-09-10 00:00:00',541.67,598,30669,9.99,'Highly Ellip' from dual union all
select 38754,'038754','2012-046C','Centaur AV-032','Centaur AV-032',69301,'Deorbited','2012-08-30 00:00:00','2012-08-30 00:00:00',628.26,219,36062,10.26,'Suborbital' from dual union all
select 38755,'038755','2012-047A','SPOT 6','SPOT 6',69302,'In Earth orbit','','2012-09-16 00:00:00',98.31,677,679,98.24,'Sun-Synch' from dual union all
select 38756,'038756','2012-047B','PROITERES','PROITERES',69302,'In Earth orbit','','2012-09-16 00:00:00',97.68,640,656,98.29,'Sun-Synch' from dual union all
select 38757,'038757','2012-047C','PS4','PS4 PSLV-C18',69302,'In Earth orbit','','2012-09-16 00:00:00',97.59,640,647,98.18,'Sun-Synch' from dual union all
select 38758,'038758','2012-048A','USA 238','INTRUDER 11',69303,'In Earth orbit','','2012-09-27 00:00:00',107.4,1011,1202,63.43,'Intermediate' from dual union all
select 38759,'038759','2012-048B','SMDC-ONE 2.2 (Baker)','ORSES (Colony 2B-S)',69303,'In Earth orbit','','2012-09-14 00:00:00',97.2,470,779,64.67,'Intermediate' from dual union all
select 3876,'003876','1968-091BK','deb Kosmos-249','',16818,'Reentered','1975-06-15 00:00:00','1978-04-27 00:00:00',101.42,454,1198,62.31,'Intermediate' from dual union all
select 38760,'038760','2012-048C','Aeneas','Aeneas',69303,'In Earth orbit','','2012-09-14 00:00:00',97.2,470,779,64.67,'Intermediate' from dual union all
select 38761,'038761','2012-048D','CSSWE','CSSWE',69303,'In Earth orbit','','2012-09-14 00:00:00',97.19,472,776,64.67,'Intermediate' from dual union all
select 38762,'038762','2012-048E','CXBN','Unbridled Spirit',69303,'In Earth orbit','','2012-09-14 00:00:00',97.17,469,778,64.67,'Intermediate' from dual union all
select 38763,'038763','2012-048F','CP5','CalPoly CP5',69303,'In Earth orbit','','2012-09-14 00:00:00',97.17,468,779,64.67,'Intermediate' from dual union all
select 38764,'038764','2012-048G','CINEMA 1','CINEMA 1',69303,'In Earth orbit','','2012-09-14 00:00:00',97.2,470,779,64.67,'Intermediate' from dual union all
select 38765,'038765','2012-048H','Re','STARE-A',69303,'In Earth orbit','','2012-09-14 00:00:00',97.2,470,779,64.67,'Intermediate' from dual union all
select 38766,'038766','2012-048J','SMDC-ONE 2.1 (Able)','(Colony 2B-S)',69303,'In Earth orbit','','2012-09-14 00:00:00',97.23,473,780,64.67,'Intermediate' from dual union all
select 38767,'038767','2012-048K','Aerocube 4.5A','AC4-A',69303,'In Earth orbit','','2012-09-14 00:00:00',97.28,478,778,64.68,'Intermediate' from dual union all
select 38768,'038768','2012-048L','Aerocube 4.5B','AC4-B',69303,'In Earth orbit','','2012-09-14 00:00:00',97.33,483,779,64.68,'Intermediate' from dual union all
select 38769,'038769','2012-048M','Aerocube 4','AC4-C',69303,'In Earth orbit','','2012-09-14 00:00:00',97.39,488,779,64.69,'Intermediate' from dual union all
select 3877,'003877','1969-029X','deb Blok-E','',18296,'Reentered','1979-11-23 00:00:00','1979-11-19 00:00:00',89.66,239,277,81,'Intermediate' from dual union all
select 38770,'038770','2012-048N','OUTSat','NPSCuL-1 OUTSat',69303,'In Earth orbit','','2012-09-23 00:00:00',97.05,461,774,64.66,'Intermediate' from dual union all
select 38771,'038771','2012-049A','METOP B','METOP B',69304,'In Earth orbit','','2012-09-24 00:00:00',101.04,806,810,98.74,'Sun-Synch' from dual union all
select 38772,'038772','2012-049B','Fregat No. 1037','Fregat 14S44 No. 1037',69304,'Deorbited','2012-09-17 00:00:00','2012-09-17 00:00:00',100.91,796,807,98.73,'Sun-Synch' from dual union all
select 38773,'038773','2012-048P','USA 238 P/L 2','INTRUDER 12',69303,'In Earth orbit','','2012-09-27 00:00:00',107.39,1013,1199,63.46,'Intermediate' from dual union all
select 38774,'038774','2012-050A','Beidou DW 14','14 ke beidou daohang wei',69305,'In Earth orbit','','2013-02-04 00:00:00',773.18,21464,21590,54.95,'Medium' from dual union all
select 38775,'038775','2012-050B','Beidou DW 15','15 ke beidou daohang wei',69305,'In Earth orbit','','2013-02-03 00:00:00',773.21,21487,21568,55.05,'Medium' from dual union all
select 38776,'038776','2012-050C','CZ-3B Stage 3','CZ-3B Stage 3',69305,'Reentered','2013-06-22 00:00:00','2012-09-21 00:00:00',376.56,197,21594,55,'Highly Ellip' from dual union all
select 38777,'038777','2012-050D','CZ-3B payload adapter','Dual Sat Serial ESS',69305,'Reentered','2013-07-05 00:00:00','2012-09-24 00:00:00',376.33,193,21584,54.98,'Highly Ellip' from dual union all
select 38778,'038778','2012-051A','Astra 2F','Astra 2F',69306,'In Earth orbit','','2012-10-26 00:00:00',1436.09,35782,35791,.06,'Stationary' from dual union all
select 38779,'038779','2012-051B','GSAT-10','GSAT-10',69306,'In Earth orbit','','2012-11-06 00:00:00',1436.06,35774,35798,.09,'Stationary' from dual union all
select 3878,'003878','1969-029Y','deb Blok-E','',18296,'Reentered','1969-06-23 00:00:00','1969-06-20 00:00:00',91.46,304,389,81.08,'Intermediate' from dual union all
select 38780,'038780','2012-051C','ESC-A 565','Ariane 5 ESC-A',69306,'In Earth orbit','','2012-10-01 00:00:00',630.05,261,35673,5.98,'GEO Transfer' from dual union all
select 38781,'038781','2012-051D','Sylda 5','Sylda 5 No. 48A',69306,'In Earth orbit','','2012-10-01 00:00:00',630.03,262,35671,5.96,'GEO Transfer' from dual union all
select 38782,'038782','2012-052A','Miranda','VRSS-1',69307,'In Earth orbit','','2012-10-01 00:00:00',97.46,621,653,98.03,'Sun-Synch' from dual union all
select 38783,'038783','1999-025ETX','deb FY-1C','',66706,'In Earth orbit','','2012-09-30 00:00:00',101.27,570,1068,98.86,'Sun-Synch' from dual union all
select 38784,'038784','1999-025ETY','deb FY-1C','',66706,'In Earth orbit','','2012-10-01 00:00:00',99.97,683,832,99.44,'Sun-Synch' from dual union all
select 38785,'038785','1999-025ETZ','deb FY-1C','',66706,'In Earth orbit','','2012-09-30 00:00:00',103.97,766,1127,98.02,'Sun-Synch' from dual union all
select 38786,'038786','1999-025EUA','deb FY-1C','',66706,'In Earth orbit','','2012-09-18 00:00:00',106.67,838,1307,99.6,'Sun-Synch' from dual union all
select 38787,'038787','1999-025EUB','deb FY-1C','',66706,'In Earth orbit','','2012-10-02 00:00:00',105.55,837,1204,98.38,'Sun-Synch' from dual union all
select 38788,'038788','1999-025EUC','deb FY-1C','',66706,'In Earth orbit','','2012-09-14 00:00:00',100.82,608,987,97.93,'Sun-Synch' from dual union all
select 38789,'038789','1999-025EUD','deb FY-1C','',66706,'In Earth orbit','','2012-10-03 00:00:00',100.94,614,993,98.87,'Sun-Synch' from dual union all
select 3879,'003879','1969-029Z','deb Blok-E','',18296,'Reentered','1987-10-14 00:00:00','1987-10-13 00:00:00',87.77,159,170,81.31,'Intermediate' from dual union all
select 38790,'038790','1999-025EUE','deb FY-1C','',66706,'In Earth orbit','','2012-09-25 00:00:00',102.44,809,940,99.11,'Sun-Synch' from dual union all
select 38791,'038791','1999-025EUF','deb FY-1C','',66706,'In Earth orbit','','2012-09-16 00:00:00',101.87,830,865,99.24,'Sun-Synch' from dual union all
select 38792,'038792','1999-025EUG','deb FY-1C','',66706,'In Earth orbit','','2012-10-03 00:00:00',98.99,597,824,99.61,'Sun-Synch' from dual union all
select 38793,'038793','1999-025EUH','deb FY-1C','',66706,'In Earth orbit','','2012-09-26 00:00:00',100.85,700,897,98.76,'Sun-Synch' from dual union all
select 38794,'038794','1999-025EUJ','deb FY-1C','',66706,'In Earth orbit','','2012-10-03 00:00:00',105.26,840,1174,99.08,'Sun-Synch' from dual union all
select 38795,'038795','1999-025EUK','deb FY-1C','',66706,'In Earth orbit','','2012-10-01 00:00:00',109.43,788,1613,98.72,'Sun-Synch' from dual union all
select 38796,'038796','1999-025EUL','deb FY-1C','',66706,'In Earth orbit','','2012-10-01 00:00:00',100.24,707,832,98.91,'Sun-Synch' from dual union all
select 38797,'038797','1999-025EUM','deb FY-1C','',66706,'In Earth orbit','','2012-10-01 00:00:00',105.97,855,1225,99.52,'Sun-Synch' from dual union all
select 38798,'038798','1999-025EUN','deb FY-1C','',66706,'In Earth orbit','','2012-10-03 00:00:00',108.75,849,1489,98.81,'Sun-Synch' from dual union all
select 38799,'038799','1999-025EUP','deb FY-1C','',66706,'In Earth orbit','','2012-09-21 00:00:00',101.12,648,975,98.85,'Sun-Synch' from dual;
insert into SATELLITE
select 388,'000388','62 A Omicron 4','MG-18 S/N 7','MG-18 S/N 7/Sep B',8149,'Reentered','1999-05-03 00:00:00','1962-09-24 00:00:00',99.46,612,854,98.62,'Sun-Synch' from dual union all
select 3880,'003880','1969-029AA','deb Blok-E','',18296,'Reentered','1969-08-21 00:00:00','1969-04-01 00:00:00',96.42,417,758,81.1,'Intermediate' from dual union all
select 38800,'038800','1999-025EUQ','deb FY-1C','',66706,'Reentered','2014-06-05 00:00:00','2012-10-02 00:00:00',98.87,566,844,97.84,'Sun-Synch' from dual union all
select 38801,'038801','1999-025EUR','deb FY-1C','',66706,'In Earth orbit','','2012-10-02 00:00:00',102.24,573,1156,98.44,'Sun-Synch' from dual union all
select 38802,'038802','1999-025EUS','deb FY-1C','',66706,'In Earth orbit','','2012-09-23 00:00:00',101.84,819,873,98.99,'Sun-Synch' from dual union all
select 38803,'038803','1999-025EUT','deb FY-1C','',66706,'In Earth orbit','','2012-09-18 00:00:00',103.08,843,965,98.96,'Sun-Synch' from dual union all
select 38804,'038804','1999-025EUU','deb FY-1C','',66706,'In Earth orbit','','2012-09-26 00:00:00',106.8,761,1396,97.53,'Sun-Synch' from dual union all
select 38805,'038805','1999-025EUV','deb FY-1C','',66706,'Reentered','2015-05-02 00:00:00','2012-10-02 00:00:00',99.33,628,825,99.1,'Sun-Synch' from dual union all
select 38806,'038806','1999-025EUW','deb FY-1C','',66706,'In Earth orbit','','2012-10-02 00:00:00',103.47,816,1029,98.59,'Sun-Synch' from dual union all
select 38807,'038807','1999-025EUX','deb FY-1C','',66706,'In Earth orbit','','2012-10-02 00:00:00',105.66,844,1206,98.48,'Sun-Synch' from dual union all
select 38808,'038808','1999-025EUY','deb FY-1C','',66706,'In Earth orbit','','2012-09-30 00:00:00',101.46,687,969,98.9,'Sun-Synch' from dual union all
select 38809,'038809','1999-025EUZ','deb FY-1C','',66706,'In Earth orbit','','2012-09-26 00:00:00',102.22,683,1044,99.04,'Sun-Synch' from dual union all
select 3881,'003881','1969-029AB','deb Blok-E','',18296,'Reentered','1972-01-06 00:00:00','1972-01-05 00:00:00',88.56,201,205,81.07,'Intermediate' from dual union all
select 38810,'038810','1999-025EVA','deb FY-1C','',66706,'In Earth orbit','','2012-09-30 00:00:00',103.43,817,1024,99.38,'Sun-Synch' from dual union all
select 38811,'038811','1999-025EVB','deb FY-1C','',66706,'In Earth orbit','','2012-09-26 00:00:00',102.54,836,922,99.03,'Sun-Synch' from dual union all
select 38812,'038812','1999-025EVC','deb FY-1C','',66706,'In Earth orbit','','2012-10-02 00:00:00',106.27,838,1270,98.46,'Sun-Synch' from dual union all
select 38813,'038813','1999-025EVD','deb FY-1C','',66706,'In Earth orbit','','2012-10-01 00:00:00',101.96,847,855,99.23,'Sun-Synch' from dual union all
select 38814,'038814','1999-025EVE','deb FY-1C','',66706,'In Earth orbit','','2012-10-03 00:00:00',102.76,782,996,98.44,'Sun-Synch' from dual union all
select 38815,'038815','1999-025EVF','deb FY-1C','',66706,'In Earth orbit','','2012-09-22 00:00:00',100.47,742,819,99.7,'Sun-Synch' from dual union all
select 38816,'038816','1999-025EVG','deb FY-1C','',66706,'In Earth orbit','','2012-10-01 00:00:00',101.59,737,931,98.83,'Sun-Synch' from dual union all
select 38817,'038817','1999-025EVH','deb FY-1C','',66706,'In Earth orbit','','2012-09-21 00:00:00',106.95,852,1319,98.62,'Sun-Synch' from dual union all
select 38818,'038818','1999-025EVJ','deb FY-1C','',66706,'In Earth orbit','','2012-09-27 00:00:00',100.67,719,862,98.78,'Sun-Synch' from dual union all
select 38819,'038819','1999-025EVK','deb FY-1C','',66706,'In Earth orbit','','2012-09-26 00:00:00',101.92,787,913,98.98,'Sun-Synch' from dual union all
select 3882,'003882','1969-029AC','deb Blok-E','',18296,'Reentered','1977-11-19 00:00:00','1977-10-12 00:00:00',91.49,334,362,81.04,'Intermediate' from dual union all
select 38820,'038820','1999-025EVL','deb FY-1C','',66706,'In Earth orbit','','2012-09-21 00:00:00',105.21,779,1230,98.25,'Sun-Synch' from dual union all
select 38821,'038821','1999-025EVM','deb FY-1C','',66706,'In Earth orbit','','2012-09-22 00:00:00',103.91,847,1040,98.5,'Sun-Synch' from dual union all
select 38822,'038822','1999-025EVN','deb FY-1C','',66706,'In Earth orbit','','2012-10-03 00:00:00',103.89,843,1042,98.16,'Sun-Synch' from dual union all
select 38823,'038823','1999-025EVP','deb FY-1C','',66706,'In Earth orbit','','2012-09-25 00:00:00',103.5,836,1013,99.2,'Sun-Synch' from dual union all
select 38824,'038824','1999-025EVQ','deb FY-1C','',66706,'In Earth orbit','','2012-09-29 00:00:00',103.82,845,1034,99.4,'Sun-Synch' from dual union all
select 38825,'038825','1999-025EVR','deb FY-1C','',66706,'In Earth orbit','','2012-09-23 00:00:00',104.92,817,1165,98.99,'Sun-Synch' from dual union all
select 38826,'038826','1999-025EVS','deb FY-1C','',66706,'In Earth orbit','','2012-09-22 00:00:00',101.8,799,888,99.9,'Sun-Synch' from dual union all
select 38827,'038827','1999-025EVT','deb FY-1C','',66706,'In Earth orbit','','2012-10-01 00:00:00',101.66,648,1027,98.94,'Sun-Synch' from dual union all
select 38828,'038828','1999-025EVU','deb FY-1C','',66706,'In Earth orbit','','2012-09-24 00:00:00',102.57,558,1202,99.51,'Sun-Synch' from dual union all
select 38829,'038829','1999-025EVV','deb FY-1C','',66706,'In Earth orbit','','2012-09-22 00:00:00',102.68,854,917,99.14,'Sun-Synch' from dual union all
select 3883,'003883','1969-034A','Kosmos-278','Zenit-2 No. 78',18301,'Landed','1969-04-17 00:00:00','1969-04-16 00:00:00',89.5,194,305,65.42,'Intermediate' from dual union all
select 38830,'038830','1999-025EVW','deb FY-1C','',66706,'Reentered','2014-05-23 00:00:00','2012-09-25 00:00:00',95.55,386,705,99.02,'Sun-Synch' from dual union all
select 38831,'038831','1999-025EVX','deb FY-1C','',66706,'In Earth orbit','','2012-09-23 00:00:00',102.71,846,928,99.03,'Sun-Synch' from dual union all
select 38832,'038832','1999-025EVY','deb FY-1C','',66706,'In Earth orbit','','2012-09-29 00:00:00',103.4,845,994,99.06,'Sun-Synch' from dual union all
select 38833,'038833','2012-053A','GPS IIF-3','GPS SVN 65',69308,'In Earth orbit','','2012-10-12 00:00:00',728.98,20426,20480,54.98,'Medium' from dual union all
select 38834,'038834','2012-053B','Delta 361','DCSS-4 F12',69308,'In Earth orbit','','2012-10-12 00:00:00',760.09,20447,21975,54.66,'Medium' from dual union all
select 38835,'038835','1983-044GD','deb Kosmos-1461','',55801,'Reentered','2013-06-09 00:00:00','2012-10-03 00:00:00',96.78,526,683,65.01,'Intermediate' from dual union all
select 38836,'038836','1980-073N','deb Meteor-2','',50283,'In Earth orbit','','2012-10-04 00:00:00',102.07,831,883,81.23,'Intermediate' from dual union all
select 38837,'038837','1980-073P','deb Meteor-2','',50283,'In Earth orbit','','2012-09-29 00:00:00',101.75,824,859,81.24,'Intermediate' from dual union all
select 38838,'038838','1980-073Q','deb Meteor-2','',50283,'In Earth orbit','','2012-10-04 00:00:00',100.17,753,780,81.22,'Intermediate' from dual union all
select 38839,'038839','1983-022L','NOAA 8 battery debris','',55779,'In Earth orbit','','2012-10-02 00:00:00',100.42,771,786,98.47,'Sun-Synch' from dual union all
select 3884,'003884','1969-034B','11A57I','11A57I',18301,'Reentered','1969-04-16 00:00:00','1969-04-11 00:00:00',89.21,181,290,65.4,'Intermediate' from dual union all
select 38840,'038840','1983-022M','NOAA 8 battery debris','',55779,'In Earth orbit','','2012-10-04 00:00:00',100.69,781,801,98.47,'Sun-Synch' from dual union all
select 38841,'038841','1983-022N','NOAA 8 battery debris','',55779,'In Earth orbit','','2012-10-04 00:00:00',100.55,773,797,98.47,'Sun-Synch' from dual union all
select 38842,'038842','2005-017F','deb PS4','',68026,'In Earth orbit','','2012-10-05 00:00:00',97.75,599,704,97.73,'Sun-Synch' from dual union all
select 38843,'038843','1974-029D','deb Kosmos-651 reactor','',33632,'In Earth orbit','','2012-09-16 00:00:00',103.2,871,949,64.97,'Intermediate' from dual union all
select 38844,'038844','2006-060C','deb S3M','',68232,'Reentered','2014-04-06 00:00:00','2012-09-26 00:00:00',93.31,419,454,98.11,'Sun-Synch' from dual union all
select 38845,'038845','1988-019G','deb US-A coolant','',61268,'In Earth orbit','','2012-10-03 00:00:00',104.36,941,988,65.04,'Intermediate' from dual union all
select 38846,'038846','2012-054A','Dragon CRS-1','Dragon C103',69309,'Landed','2012-10-28 00:00:00','2012-10-09 00:00:00',92.66,393,416,51.64,'Intermediate' from dual union all
select 38847,'038847','2012-054B','Orbcomm OG2-1','Orbcomm OG2-1',69309,'Reentered','2012-10-10 00:00:00','2012-10-09 00:00:00',89.2,165,305,51.64,'Intermediate' from dual union all
select 38848,'038848','2012-054C','Falcon 9-004 Stage 2','F9-004 St2',69309,'Reentered','2012-10-11 00:00:00','2012-10-09 00:00:00',88.76,163,264,51.65,'Intermediate' from dual union all
select 38849,'038849','2012-054D','Dragon solar array cover','',69309,'Reentered','2012-10-10 00:00:00','2012-10-09 00:00:00',89.09,186,272,51.66,'Intermediate' from dual union all
select 3885,'003885','1969-035A','Molniya-1','Molniya-1 11F67 No. 16',18302,'Reentered','1974-04-17 00:00:00','1969-05-15 00:00:00',718.17,484,39890,65.09,'GEO Transfer' from dual union all
select 38850,'038850','2012-054E','deb Dragon CRS-1','',69309,'Reentered','2012-10-11 00:00:00','2012-10-10 00:00:00',88.89,182,257,51.63,'Intermediate' from dual union all
select 38851,'038851','2012-054F','Dragon solar array cover','',69309,'Reentered','2012-10-09 00:00:00','2012-10-08 00:00:00',89.54,195,309,51.65,'Intermediate' from dual union all
select 38852,'038852','1998-067CN','Raiko','RAIKO',66503,'Reentered','2013-08-06 00:00:00','2012-10-09 00:00:00',92.8,399,425,51.65,'Intermediate' from dual union all
select 38853,'038853','1998-067CP','Niwaka eisei','FITSAT-1',66503,'Reentered','2013-07-04 00:00:00','2012-10-09 00:00:00',92.8,402,422,51.65,'Intermediate' from dual union all
select 38854,'038854','1998-067CQ','TechEdSat-1','TES-1',66503,'Reentered','2013-05-05 00:00:00','2012-10-09 00:00:00',92.8,402,422,51.65,'Intermediate' from dual union all
select 38855,'038855','1998-067CR','F-1','F-1',66503,'Reentered','2013-05-09 00:00:00','2012-10-08 00:00:00',92.8,402,422,51.65,'Intermediate' from dual union all
select 38856,'038856','1998-067CS','WE WISH','WE WISH',66503,'Reentered','2013-03-11 00:00:00','2012-10-09 00:00:00',92.79,399,424,51.65,'Intermediate' from dual union all
select 38857,'038857','2012-055A','GalileoSat-3','Galileo IOV FM3',69310,'In Earth orbit','','2012-10-23 00:00:00',844.75,23215,23232,55.34,'Medium' from dual union all
select 38858,'038858','2012-055B','GalileoSat-4','Galileo IOV FM4',69310,'In Earth orbit','','2012-10-23 00:00:00',842.7,23136,23215,55.34,'Medium' from dual union all
select 38859,'038859','2012-055C','Fregat-MT No. 1031','Fregat-MT    No. 1031',69310,'In Earth orbit','','2012-10-20 00:00:00',863.79,23640,23693,55.36,'Medium' from dual union all
select 3886,'003886','1969-035C','Blok-L No. Ya716-88','Blok-L',18302,'Reentered','1974-07-01 00:00:00','1969-05-11 00:00:00',710.35,472,39516,65.02,'GEO Transfer' from dual union all
select 38860,'038860','2012-056A','Shi Jian 9A','shi jian liuhao A weixin',69311,'In Earth orbit','','2012-10-21 00:00:00',97.4,622,647,98,'Sun-Synch' from dual union all
select 38861,'038861','2012-056B','Shi Jian 9B','shi jian liuhao B weixin',69311,'In Earth orbit','','2012-10-21 00:00:00',97.44,623,649,97.99,'Sun-Synch' from dual union all
select 38862,'038862','2012-056C','CZ-2C Stage 2','CZ-2C Stage 2',69311,'Reentered','2013-06-05 00:00:00','2012-10-21 00:00:00',93.83,259,664,97.97,'Sun-Synch' from dual union all
select 38863,'038863','2012-056D','Adapter','',69311,'In Earth orbit','','2012-10-21 00:00:00',97.41,623,647,97.99,'Sun-Synch' from dual union all
select 38864,'038864','2012-056E','SMA kick stage','',69311,'Reentered','2014-11-15 00:00:00','2012-10-21 00:00:00',94.5,375,614,98.15,'Sun-Synch' from dual union all
select 38865,'038865','2012-056F','CZ-2C sep motor cover','',69311,'Reentered','2012-11-20 00:00:00','2012-10-21 00:00:00',95.44,253,827,98.17,'Sun-Synch' from dual union all
select 38866,'038866','2012-056G','CZ-2C sep motor cover','',69311,'Reentered','2012-11-20 00:00:00','2012-10-21 00:00:00',95.31,256,812,98.18,'Sun-Synch' from dual union all
select 38867,'038867','2012-057A','Intelsat IS-23','Intelsat IS-23',69312,'In Earth orbit','','2012-10-21 00:00:00',1469.14,35947,36915,.12,'Drift GEO' from dual union all
select 38868,'038868','2012-057B','Briz-M No. 99534','Briz-M 14S43 No. 99534',69312,'In Earth orbit','','2013-01-06 00:00:00',1629.2,37201,41772,.08,'Very High' from dual union all
select 38869,'038869','2012-057C','Briz-M DTB','Briz-M torus tank',69312,'In Earth orbit','','2012-10-21 00:00:00',663.04,496,37123,46.2,'GEO Transfer' from dual union all
select 3887,'003887','1969-035B','11S510','11S510',18302,'Reentered','1969-05-01 00:00:00','1969-04-18 00:00:00',90.49,227,371,65.04,'Intermediate' from dual union all
select 38870,'038870','2012-056H','CZ-2C sep motor cover','',69311,'Reentered','2012-11-22 00:00:00','2012-10-21 00:00:00',95.4,254,822,97.77,'Sun-Synch' from dual union all
select 38871,'038871','2012-058A','Soyuz TMA-06M','Soyuz 11F732A47 No. 707',69313,'Landed','2013-03-16 00:00:00','2012-10-24 00:00:00',90.17,249,317,51.65,'Intermediate' from dual union all
select 38872,'038872','2012-058B','11S510','11S510',69313,'Reentered','2012-10-25 00:00:00','2012-10-24 00:00:00',88.2,176,195,51.63,'Intermediate' from dual union all
select 38873,'038873','2012-044E','deb Briz-M No. 99532','',69299,'Reentered','2014-01-15 00:00:00','2012-10-24 00:00:00',136.4,264,4535,50.17,'Medium' from dual union all
select 38874,'038874','2012-044F','deb Briz-M No. 99532','',69299,'Reentered','2013-05-25 00:00:00','2012-10-24 00:00:00',133.26,261,4268,50.02,'Medium' from dual union all
select 38875,'038875','2012-044G','deb Briz-M No. 99532','',69299,'Reentered','2014-03-21 00:00:00','2012-10-24 00:00:00',136.06,257,4513,49.82,'Medium' from dual union all
select 38876,'038876','2012-044H','deb Briz-M No. 99532','',69299,'Reentered','2013-09-29 00:00:00','2012-10-24 00:00:00',138.62,255,4734,49.92,'Medium' from dual union all
select 38877,'038877','2012-044J','deb Briz-M No. 99532','',69299,'Reentered','2013-04-21 00:00:00','2012-10-24 00:00:00',130.23,261,4005,50.28,'Medium' from dual union all
select 38878,'038878','2012-044K','deb Briz-M No. 99532','',69299,'Reentered','2013-03-07 00:00:00','2012-10-24 00:00:00',135.75,253,4489,49.79,'Medium' from dual union all
select 38879,'038879','2012-044L','deb Briz-M No. 99532','',69299,'Reentered','2013-10-10 00:00:00','2012-10-24 00:00:00',146.63,262,5404,50.37,'Medium' from dual union all
select 3888,'003888','1969-035D','BOZ','BOZ',18302,'Reentered','1969-05-05 00:00:00','1969-04-20 00:00:00',90.74,214,408,65.01,'Intermediate' from dual union all
select 38880,'038880','2012-044M','deb Briz-M No. 99532','',69299,'In Earth orbit','','2012-10-24 00:00:00',141.78,265,4993,49.86,'Medium' from dual union all
select 38881,'038881','2012-044N','deb Briz-M No. 99532','',69299,'In Earth orbit','','2012-10-24 00:00:00',145.15,242,5300,50.51,'Medium' from dual union all
select 38882,'038882','2012-044P','deb Briz-M No. 99532','',69299,'Reentered','2012-10-29 00:00:00','2012-10-23 00:00:00',121.12,258,3204,50.65,'Medium' from dual union all
select 38883,'038883','2012-044Q','deb Briz-M No. 99532','',69299,'Reentered','2014-01-20 00:00:00','2012-10-24 00:00:00',138.55,260,4723,49.85,'Medium' from dual union all
select 38884,'038884','2012-044R','deb Briz-M No. 99532','',69299,'In Earth orbit','','2012-10-24 00:00:00',140.73,265,4904,50,'Medium' from dual union all
select 38885,'038885','2012-044S','deb Briz-M No. 99532','',69299,'Reentered','2014-04-20 00:00:00','2012-10-24 00:00:00',139.58,263,4807,50.28,'Medium' from dual union all
select 38886,'038886','2012-044T','deb Briz-M No. 99532','',69299,'Reentered','2017-02-19 00:00:00','2012-10-24 00:00:00',141.84,262,5001,50.09,'Medium' from dual union all
select 38887,'038887','2012-044U','deb Briz-M No. 99532','',69299,'Reentered','2012-10-28 00:00:00','2012-10-23 00:00:00',116.13,246,2768,50.15,'Intermediate' from dual union all
select 38888,'038888','2012-044V','deb Briz-M No. 99532','',69299,'In Earth orbit','','2012-10-24 00:00:00',139.8,262,4828,50.07,'Medium' from dual union all
select 38889,'038889','2012-044W','deb Briz-M No. 99532','',69299,'Reentered','2013-07-21 00:00:00','2012-10-24 00:00:00',139.05,266,4760,49.48,'Medium' from dual;
insert into SATELLITE
select 3889,'003889','1969-036A','OPS 3148','CANYON 2',18303,'In Earth orbit','','1969-12-31 00:00:00',1445.04,32672,39251,10.2,'Near-synch' from dual union all
select 38890,'038890','2012-044X','deb Briz-M No. 99532','',69299,'Reentered','2016-06-29 00:00:00','2012-10-24 00:00:00',142.3,261,5041,50.15,'Medium' from dual union all
select 38891,'038891','2012-044Y','deb Briz-M No. 99532','',69299,'Reentered','2012-10-23 00:00:00','2012-10-23 00:00:00',103.06,104,1703,50.2,'Intermediate' from dual union all
select 38892,'038892','2012-044Z','deb Briz-M No. 99532','',69299,'Reentered','2013-10-31 00:00:00','2012-10-24 00:00:00',141.46,252,4979,49.99,'Medium' from dual union all
select 38893,'038893','2012-044AA','deb Briz-M No. 99532','',69299,'Reentered','2014-01-15 00:00:00','2012-10-24 00:00:00',135.37,260,4451,49.99,'Medium' from dual union all
select 38894,'038894','2012-044AB','deb Briz-M No. 99532','',69299,'Reentered','2015-08-03 00:00:00','2012-10-24 00:00:00',142.21,260,5034,49.82,'Medium' from dual union all
select 38895,'038895','2012-044AC','deb Briz-M No. 99532','',69299,'Reentered','2015-10-18 00:00:00','2012-10-24 00:00:00',142.55,266,5057,49.74,'Medium' from dual union all
select 38896,'038896','2012-044AD','deb Briz-M No. 99532','',69299,'Reentered','2013-12-23 00:00:00','2012-10-24 00:00:00',142.99,265,5095,49.88,'Medium' from dual union all
select 38897,'038897','2012-044AE','deb Briz-M No. 99532','',69299,'Reentered','2012-10-26 00:00:00','2012-10-23 00:00:00',111.6,256,2345,49.99,'Intermediate' from dual union all
select 38898,'038898','2012-044AF','deb Briz-M No. 99532','Large debris',69299,'Reentered','2015-07-04 00:00:00','2012-10-24 00:00:00',138.53,266,4715,50,'Medium' from dual union all
select 38899,'038899','2012-044AG','deb Briz-M No. 99532','',69299,'Reentered','2014-01-08 00:00:00','2012-10-24 00:00:00',134.06,261,4337,50.09,'Medium' from dual union all
select 389,'000389','62 A Phi 1','[AMS Venera]','2MV-2 No. 1',8150,'Reentered Attached','1962-09-14 00:00:00','1962-09-12 00:00:00',88.07,163,195,64.8,'Intermediate' from dual union all
select 3890,'003890','1969-037A','Nimbus 3','Nimbus B2',18304,'In Earth orbit','','1969-06-02 00:00:00',107.33,1073,1134,99.92,'Sun-Synch' from dual union all
select 38900,'038900','2012-044AH','deb Briz-M No. 99532','',69299,'Reentered','2014-08-19 00:00:00','2012-10-24 00:00:00',141.84,259,5004,50.34,'Medium' from dual union all
select 38901,'038901','2012-044AJ','deb Briz-M No. 99532','',69299,'Reentered','2014-10-16 00:00:00','2012-10-24 00:00:00',139.82,264,4827,50.11,'Medium' from dual union all
select 38902,'038902','2012-044AK','deb Briz-M No. 99532','Medium debris',69299,'Reentered','2015-07-03 00:00:00','2012-10-24 00:00:00',140.52,263,4888,49.76,'Medium' from dual union all
select 38903,'038903','2012-044AL','deb Briz-M No. 99532','',69299,'Reentered','2013-11-25 00:00:00','2012-10-24 00:00:00',138.9,264,4749,50.28,'Medium' from dual union all
select 38904,'038904','2012-044AM','deb Briz-M No. 99532','',69299,'Reentered','2013-03-06 00:00:00','2012-10-24 00:00:00',132.79,262,4226,49.84,'Medium' from dual union all
select 38905,'038905','2012-044AN','deb Briz-M No. 99532','Medium debris',69299,'Reentered','2016-03-13 00:00:00','2012-10-24 00:00:00',138.53,262,4718,50.07,'Medium' from dual union all
select 38906,'038906','2012-044AP','deb Briz-M No. 99532','',69299,'Reentered','2017-02-27 00:00:00','2012-10-24 00:00:00',138.2,266,4687,50.02,'Medium' from dual union all
select 38907,'038907','2012-044AQ','deb Briz-M No. 99532','',69299,'Reentered','2013-10-10 00:00:00','2012-10-24 00:00:00',139.96,264,4839,49.89,'Medium' from dual union all
select 38908,'038908','2012-044AR','deb Briz-M No. 99532','Medium debris',69299,'Reentered','2014-12-11 00:00:00','2012-10-24 00:00:00',140.02,253,4855,49.78,'Medium' from dual union all
select 38909,'038909','2012-044AS','deb Briz-M No. 99532','',69299,'Reentered','2014-09-08 00:00:00','2012-10-24 00:00:00',139.41,260,4795,50.23,'Medium' from dual union all
select 3891,'003891','1969-037B','EGRS XIII','Secor Type II S/N 14',18304,'In Earth orbit','','1969-08-02 00:00:00',107.29,1072,1130,99.91,'Sun-Synch' from dual union all
select 38910,'038910','2012-044AT','deb Briz-M No. 99532','',69299,'Reentered','2014-08-03 00:00:00','2012-10-24 00:00:00',141.46,256,4974,50.3,'Medium' from dual union all
select 38911,'038911','2012-044AU','deb Briz-M No. 99532','',69299,'Reentered','2014-12-31 00:00:00','2012-10-24 00:00:00',141.3,259,4957,49.78,'Medium' from dual union all
select 38912,'038912','2012-044AV','deb Briz-M No. 99532','',69299,'Reentered','2013-08-02 00:00:00','2012-10-24 00:00:00',140.76,264,4907,49.92,'Medium' from dual union all
select 38913,'038913','2012-044AW','deb Briz-M No. 99532','',69299,'Reentered','2013-08-24 00:00:00','2012-10-24 00:00:00',141.89,251,5015,50.18,'Medium' from dual union all
select 38914,'038914','2012-044AX','deb Briz-M No. 99532','',69299,'Reentered','2015-07-05 00:00:00','2012-10-24 00:00:00',147.11,258,5448,50.16,'Medium' from dual union all
select 38915,'038915','2012-044AY','deb Briz-M No. 99532','',69299,'Reentered','2013-06-27 00:00:00','2012-10-24 00:00:00',139.46,252,4809,50.36,'Medium' from dual union all
select 38916,'038916','2012-044AZ','deb Briz-M No. 99532','',69299,'Reentered','2014-10-26 00:00:00','2012-10-24 00:00:00',138.77,262,4740,50.08,'Medium' from dual union all
select 38917,'038917','2012-044BA','deb Briz-M No. 99532','',69299,'Reentered','2014-10-09 00:00:00','2012-10-24 00:00:00',142.74,259,5080,50.29,'Medium' from dual union all
select 38918,'038918','2012-044BB','deb Briz-M No. 99532','',69299,'Reentered','2015-06-14 00:00:00','2012-10-24 00:00:00',139.04,267,4758,49.93,'Medium' from dual union all
select 38919,'038919','2012-044BC','deb Briz-M No. 99532','',69299,'Reentered','2014-06-06 00:00:00','2012-10-24 00:00:00',139.36,255,4797,50.14,'Medium' from dual union all
select 3892,'003892','1969-037C','Agena D 6222','Agena D',18304,'In Earth orbit','','1969-06-28 00:00:00',107.42,1077,1137,99.92,'Sun-Synch' from dual union all
select 38920,'038920','2012-044BD','deb Briz-M No. 99532','',69299,'Reentered','2013-09-23 00:00:00','2012-10-24 00:00:00',138.51,263,4717,50.07,'Medium' from dual union all
select 38921,'038921','2012-044BE','deb Briz-M No. 99532','',69299,'In Earth orbit','','2012-10-24 00:00:00',145.67,259,5327,50.28,'Medium' from dual union all
select 38922,'038922','2012-044BF','deb Briz-M No. 99532','',69299,'In Earth orbit','','2012-10-24 00:00:00',139.78,261,4826,50.08,'Medium' from dual union all
select 38923,'038923','2012-044BG','deb Briz-M No. 99532','',69299,'Reentered','2014-07-07 00:00:00','2012-10-24 00:00:00',142.5,259,5060,50.16,'Medium' from dual union all
select 38924,'038924','2012-044BH','deb Briz-M No. 99532','',69299,'Reentered','2013-10-30 00:00:00','2012-10-24 00:00:00',137.68,261,4647,50.09,'Medium' from dual union all
select 38925,'038925','2012-044BJ','deb Briz-M No. 99532','',69299,'Reentered','2014-08-06 00:00:00','2012-10-24 00:00:00',138.02,262,4676,50.09,'Medium' from dual union all
select 38926,'038926','2012-044BK','deb Briz-M No. 99532','',69299,'In Earth orbit','','2012-10-24 00:00:00',139.36,260,4792,50.1,'Medium' from dual union all
select 38927,'038927','2012-044BL','deb Briz-M No. 99532','',69299,'Reentered','2014-11-17 00:00:00','2012-10-24 00:00:00',140.4,265,4876,50.07,'Medium' from dual union all
select 38928,'038928','2012-044BM','deb Briz-M No. 99532','',69299,'Reentered','2013-07-24 00:00:00','2012-10-24 00:00:00',137.41,265,4621,49.99,'Medium' from dual union all
select 38929,'038929','2012-044BN','deb Briz-M No. 99532','',69299,'Reentered','2013-10-18 00:00:00','2012-10-24 00:00:00',138.95,255,4762,49.74,'Medium' from dual union all
select 3893,'003893','1969-038A','Kosmos-279','Zenit-4 No. 61',18305,'Landed','1969-04-23 00:00:00','1969-04-18 00:00:00',89.08,191,267,51.74,'Intermediate' from dual union all
select 38930,'038930','2012-044BP','deb Briz-M No. 99532','',69299,'Reentered','2013-04-26 00:00:00','2012-10-24 00:00:00',140.13,255,4863,49.72,'Medium' from dual union all
select 38931,'038931','2012-044BQ','deb Briz-M No. 99532','',69299,'Reentered','2014-04-05 00:00:00','2012-10-24 00:00:00',139.72,263,4820,49.98,'Medium' from dual union all
select 38932,'038932','2012-044BR','deb Briz-M No. 99532','',69299,'Reentered','2013-11-29 00:00:00','2012-10-24 00:00:00',142.34,266,5039,49.21,'Medium' from dual union all
select 38933,'038933','2012-044BS','deb Briz-M No. 99532','',69299,'Reentered','2013-06-26 00:00:00','2012-10-24 00:00:00',132.79,258,4230,50.05,'Medium' from dual union all
select 38934,'038934','2012-044BT','deb Briz-M No. 99532','',69299,'Reentered','2012-11-17 00:00:00','2012-10-24 00:00:00',129.41,263,3931,49.1,'Medium' from dual union all
select 38935,'038935','2012-044BU','deb Briz-M No. 99532','',69299,'Reentered','2013-06-11 00:00:00','2012-10-24 00:00:00',134.06,266,4332,50.23,'Medium' from dual union all
select 38936,'038936','2012-044BV','deb Briz-M No. 99532','',69299,'Reentered','2014-03-25 00:00:00','2012-10-24 00:00:00',142.01,254,5023,50.27,'Medium' from dual union all
select 38937,'038937','2012-044BW','deb Briz-M No. 99532','',69299,'Reentered','2014-01-31 00:00:00','2012-10-24 00:00:00',143.5,268,5134,49.54,'Medium' from dual union all
select 38938,'038938','2012-044BX','deb Briz-M No. 99532','',69299,'Reentered','2013-12-05 00:00:00','2012-10-24 00:00:00',143.59,255,5156,50.14,'Medium' from dual union all
select 38939,'038939','2012-044BY','deb Briz-M No. 99532','',69299,'Reentered','2013-11-25 00:00:00','2012-10-24 00:00:00',147.03,270,5430,50.37,'Medium' from dual union all
select 3894,'003894','1969-038B','11A57I','11A57I',18305,'Reentered','1969-04-18 00:00:00','1969-04-17 00:00:00',88.47,164,233,51.74,'Intermediate' from dual union all
select 38940,'038940','2012-044BZ','deb Briz-M No. 99532','',69299,'Reentered','2013-06-23 00:00:00','2012-10-24 00:00:00',134.52,264,4373,50.15,'Medium' from dual union all
select 38941,'038941','2012-044CA','deb Briz-M No. 99532','',69299,'Reentered','2016-03-28 00:00:00','2012-10-24 00:00:00',144.51,268,5219,49.63,'Medium' from dual union all
select 38942,'038942','2012-044CB','deb Briz-M No. 99532','',69299,'Reentered','2014-07-16 00:00:00','2012-10-24 00:00:00',135.12,259,4430,49.87,'Medium' from dual union all
select 38943,'038943','2012-044CC','deb Briz-M No. 99532','',69299,'Reentered','2014-02-23 00:00:00','2012-10-24 00:00:00',138.17,265,4686,50.14,'Medium' from dual union all
select 38944,'038944','2012-044CD','deb Briz-M No. 99532','',69299,'Reentered','2014-06-17 00:00:00','2012-10-24 00:00:00',137.91,252,4676,49.77,'Medium' from dual union all
select 38945,'038945','2012-044CE','deb Briz-M No. 99532','',69299,'Reentered','2013-12-11 00:00:00','2012-10-24 00:00:00',134.79,261,4400,50.04,'Medium' from dual union all
select 38946,'038946','2012-044CF','deb Briz-M No. 99532','',69299,'Reentered','2013-07-18 00:00:00','2012-10-24 00:00:00',137.15,254,4608,50.04,'Medium' from dual union all
select 38947,'038947','2012-044CG','deb Briz-M No. 99532','',69299,'Reentered','2013-06-29 00:00:00','2012-10-24 00:00:00',133.53,266,4286,49.99,'Medium' from dual union all
select 38948,'038948','2012-044CH','deb Briz-M No. 99532','',69299,'In Earth orbit','','2012-10-24 00:00:00',139.77,262,4825,50.11,'Medium' from dual union all
select 38949,'038949','2012-044CJ','deb Briz-M No. 99532','',69299,'Reentered','2012-11-01 00:00:00','2012-10-24 00:00:00',121.34,272,3210,49.62,'Medium' from dual union all
select 3895,'003895','1969-039A','OPS 5310','GAMBIT-3 Mission 4321',18306,'Reentered Attached','1969-04-30 00:00:00','1969-04-24 00:00:00',89.77,131,396,108.71,'Retrograde' from dual union all
select 38950,'038950','2012-044CK','deb Briz-M No. 99532','',69299,'Reentered','2014-06-27 00:00:00','2012-10-24 00:00:00',145.76,266,5327,50.25,'Medium' from dual union all
select 38951,'038951','2012-044CL','deb Briz-M No. 99532','',69299,'Reentered','2013-02-25 00:00:00','2012-10-24 00:00:00',118.16,161,3036,50.19,'Intermediate' from dual union all
select 38952,'038952','2012-044CM','deb Briz-M No. 99532','',69299,'Reentered','2013-08-08 00:00:00','2012-10-24 00:00:00',138.49,253,4725,50.22,'Medium' from dual union all
select 38953,'038953','2012-059A','Beidou DW 16','16 ke beidou daohang wei',69314,'In Earth orbit','','2012-11-01 00:00:00',1436.09,35774,35799,1.84,'Stationary' from dual union all
select 38954,'038954','2012-059B','CZ-3C Stage 3','CZ-3C Stage 3',69314,'In Earth orbit','','2012-10-28 00:00:00',610.18,205,34700,20.2,'GEO Transfer' from dual union all
select 38955,'038955','2012-044CN','deb Briz-M No. 99532','',69299,'Reentered','2014-11-04 00:00:00','2012-10-30 00:00:00',141.1,263,4936,49.94,'Medium' from dual union all
select 38956,'038956','2012-044CP','deb Briz-M No. 99532','',69299,'Reentered','2013-06-29 00:00:00','2012-10-30 00:00:00',138.4,264,4705,50.1,'Medium' from dual union all
select 38957,'038957','2012-044CQ','deb Briz-M No. 99532','',69299,'In Earth orbit','','2012-10-30 00:00:00',144.26,259,5208,50.05,'Medium' from dual union all
select 38958,'038958','2012-044CR','deb Briz-M No. 99532','',69299,'Reentered','2013-01-15 00:00:00','2012-10-30 00:00:00',127.18,264,3735,49.98,'Medium' from dual union all
select 38959,'038959','2012-044CS','deb Briz-M No. 99532','',69299,'Reentered','2013-10-10 00:00:00','2012-10-30 00:00:00',146.9,262,5426,50.09,'Medium' from dual union all
select 3896,'003896','1968-097BH','deb Kosmos-252 [*]                ','',16824,'Reentered','2013-01-04 00:00:00','1992-11-08 00:00:00',104.31,526,1399,62.58,'Intermediate' from dual union all
select 38960,'038960','2012-044CT','deb Briz-M No. 99532','',69299,'In Earth orbit','','2012-10-30 00:00:00',141.42,265,4962,50.16,'Medium' from dual union all
select 38961,'038961','2012-044CU','deb Briz-M No. 99532','',69299,'In Earth orbit','','2012-10-30 00:00:00',139.36,261,4791,50.13,'Medium' from dual union all
select 38962,'038962','2012-044CV','deb Briz-M No. 99532','',69299,'Reentered','2014-12-21 00:00:00','2012-10-30 00:00:00',143.36,257,5134,50.07,'Medium' from dual union all
select 38963,'038963','2012-044CW','deb Briz-M No. 99532','',69299,'Reentered','2014-09-15 00:00:00','2012-10-30 00:00:00',140.95,264,4923,49.91,'Medium' from dual union all
select 38964,'038964','2012-044CX','deb Briz-M No. 99532','',69299,'Reentered','2013-09-17 00:00:00','2012-10-30 00:00:00',137.01,260,4591,50.12,'Medium' from dual union all
select 38965,'038965','2012-044CY','deb Briz-M No. 99532','',69299,'Reentered','2014-08-20 00:00:00','2012-10-30 00:00:00',138.1,261,4684,50.11,'Medium' from dual union all
select 38966,'038966','2012-044CZ','deb Briz-M No. 99532','',69299,'Reentered','2013-10-10 00:00:00','2012-10-30 00:00:00',140.9,270,4912,49.79,'Medium' from dual union all
select 38967,'038967','2012-044DA','deb Briz-M No. 99532','',69299,'Reentered','2014-10-11 00:00:00','2012-10-30 00:00:00',138.2,265,4688,50.09,'Medium' from dual union all
select 38968,'038968','2012-044DB','deb Briz-M No. 99532','',69299,'Reentered','2013-04-30 00:00:00','2012-10-29 00:00:00',139.53,251,4816,49.74,'Medium' from dual union all
select 38969,'038969','2012-044DC','deb Briz-M No. 99532','',69299,'Reentered','2013-10-10 00:00:00','2012-10-30 00:00:00',145.4,271,5292,50.34,'Medium' from dual union all
select 3897,'003897','1968-097BJ','deb Kosmos-252','',16824,'Reentered','1982-01-02 00:00:00','1969-02-13 00:00:00',113.62,517,2269,62.27,'Intermediate' from dual union all
select 38970,'038970','2012-044DD','deb Briz-M No. 99532','',69299,'Reentered','2014-01-20 00:00:00','2012-10-30 00:00:00',139.04,260,4764,50.43,'Medium' from dual union all
select 38971,'038971','2012-044DE','deb Briz-M No. 99532','',69299,'In Earth orbit','','2012-10-29 00:00:00',140.56,264,4890,50.12,'Medium' from dual union all
select 38972,'038972','2012-044DF','deb Briz-M No. 99532','',69299,'Reentered','2015-04-09 00:00:00','2012-10-30 00:00:00',138.67,206,4787,50.2,'Medium' from dual union all
select 38973,'038973','2012-044DG','deb Briz-M No. 99532','',69299,'Reentered','2013-09-25 00:00:00','2012-10-30 00:00:00',136.28,263,4526,50.21,'Medium' from dual union all
select 38974,'038974','2012-044DH','deb Briz-M No. 99532','',69299,'Reentered','2012-11-16 00:00:00','2012-10-30 00:00:00',119.7,261,3074,50.09,'Intermediate' from dual union all
select 38975,'038975','2012-060A','Progress M-17M','Progress 7K-TGM No.417',69315,'Deorbited','2013-04-21 00:00:00','2012-10-31 00:00:00',92.81,401,423,51.65,'Intermediate' from dual union all
select 38976,'038976','2012-060B','11S510','11S510',69315,'Reentered','2012-11-02 00:00:00','2012-11-01 00:00:00',88.03,171,183,51.63,'Intermediate' from dual union all
select 38977,'038977','2012-061A','Luch-5B','Luch-5B',69316,'In Earth orbit','','2012-12-20 00:00:00',1436.11,35772,35802,.24,'Stationary' from dual union all
select 38978,'038978','2012-061B','Yamal-300K','Yamal-300 No. KA-3',69316,'In Earth orbit','','2012-11-26 00:00:00',1431.82,35586,35820,.18,'Drift GEO' from dual union all
select 38979,'038979','2012-061C','Briz-M DTB','Briz-M torus tank',69316,'In Earth orbit','','2012-11-26 00:00:00',633.68,390,35731,48.7,'GEO Transfer' from dual;
insert into SATELLITE
select 3898,'003898','1968-097BK','deb Kosmos-252','',16824,'In Earth orbit','','1992-02-03 00:00:00',113.42,754,2013,62.3,'Intermediate' from dual union all
select 38980,'038980','2012-061D','Briz-M No. 99533','Briz-M 14S43 No. 99533',69316,'In Earth orbit','','2012-12-03 00:00:00',1280.12,29576,35775,.5,'Medium' from dual union all
select 38981,'038981','2012-044DJ','deb Briz-M No. 99532','',69299,'Reentered','2013-02-28 00:00:00','2012-11-09 00:00:00',129.12,261,3908,50.56,'Medium' from dual union all
select 38982,'038982','2012-044DK','deb Briz-M No. 99532','',69299,'Reentered','2013-10-25 00:00:00','2012-11-09 00:00:00',134.55,261,4379,50.33,'Medium' from dual union all
select 38983,'038983','2012-044DL','deb Briz-M No. 99532','',69299,'Reentered','2013-07-29 00:00:00','2012-11-09 00:00:00',132.16,255,4178,50.36,'Medium' from dual union all
select 38984,'038984','2012-044DM','deb Briz-M No. 99532','',69299,'Reentered','2013-09-20 00:00:00','2012-11-09 00:00:00',135.81,256,4492,50.52,'Medium' from dual union all
select 38985,'038985','2012-044DN','deb Briz-M No. 99532','',69299,'Reentered','2013-07-31 00:00:00','2012-11-09 00:00:00',130.45,257,4028,49.91,'Medium' from dual union all
select 38986,'038986','2012-044DP','deb Briz-M No. 99532','',69299,'Reentered','2013-10-10 00:00:00','2012-11-09 00:00:00',133.41,264,4277,50.07,'Medium' from dual union all
select 38987,'038987','2012-044DQ','deb Briz-M No. 99532','',69299,'Reentered','2013-01-18 00:00:00','2012-11-09 00:00:00',129.45,265,3933,49.77,'Medium' from dual union all
select 38988,'038988','2012-044DR','deb Briz-M No. 99532','',69299,'Reentered','2014-06-10 00:00:00','2012-11-09 00:00:00',140.37,262,4875,49.53,'Medium' from dual union all
select 38989,'038989','2012-044DS','deb Briz-M No. 99532','',69299,'Reentered','2013-10-10 00:00:00','2012-11-09 00:00:00',137.99,265,4670,50.02,'Medium' from dual union all
select 3899,'003899','1968-097BL','deb Kosmos-252','',16824,'In Earth orbit','','1992-01-26 00:00:00',105.49,545,1490,62.16,'Intermediate' from dual union all
select 38990,'038990','2012-044DT','deb Briz-M No. 99532','',69299,'Reentered','2013-07-12 00:00:00','2012-11-09 00:00:00',133.11,255,4261,50.12,'Medium' from dual union all
select 38991,'038991','2012-062A','Star One C3','Star One C3',69317,'In Earth orbit','','2012-11-30 00:00:00',1436.04,35779,35792,.04,'Stationary' from dual union all
select 38992,'038992','2012-062B','EUTELSAT 21B','EUTELSAT 21B',69317,'In Earth orbit','','2012-11-18 00:00:00',1432.05,35647,35768,.03,'Drift GEO' from dual union all
select 38993,'038993','2012-062C','ESC-A 566','Ariane 5 ESC-A',69317,'In Earth orbit','','2012-11-18 00:00:00',630.79,259,35713,2.27,'GEO Transfer' from dual union all
select 38994,'038994','2012-062D','Sylda 5','Sylda 5 No. 48A',69317,'Reentered','2016-11-08 00:00:00','2012-11-18 00:00:00',630.75,259,35711,1.97,'GEO Transfer' from dual union all
select 38995,'038995','2012-063A','Meridian No. 16L','Meridian No. 16L',69318,'In Earth orbit','','2012-11-22 00:00:00',725.13,983,39734,62.82,'Molniya' from dual union all
select 38996,'038996','2012-063B','Fregat No. 1034','Fregat 14S44 No. 1034',69318,'In Earth orbit','','2012-11-23 00:00:00',719.63,863,39583,62.74,'Molniya' from dual union all
select 38997,'038997','2012-064A','Huan Jing 1C','huan jing yihao C weixin',69319,'In Earth orbit','','2012-11-26 00:00:00',94.51,488,502,97.35,'Sun-Synch' from dual union all
select 38998,'038998','2012-064B','Fengniao 1A','Fengniao shiyan zaihe',69319,'In Earth orbit','','2012-11-26 00:00:00',94.47,487,499,97.35,'Sun-Synch' from dual union all
select 38999,'038999','2012-064C','Xinyan-1','Xinjishu yanzhen weixing',69319,'In Earth orbit','','2012-11-26 00:00:00',94.49,488,500,97.35,'Sun-Synch' from dual union all
select 39,'000039','60 Epsilon 6','Korabl''-Sputnik part','Vostok-1P part',6320,'Reentered','1960-09-24 00:00:00','1960-07-02 00:00:00',93.18,282,579,64.89,'Intermediate' from dual union all
select 390,'000390','62 A Phi 2','8K78I','8K78I',8150,'Reentered','1962-09-12 00:00:00','1962-09-12 00:00:00',88.07,163,195,64.8,'Intermediate' from dual union all
select 3900,'003900','1968-097BM','deb Kosmos-252','',16824,'In Earth orbit','','1992-02-02 00:00:00',105.72,517,1540,62.27,'Intermediate' from dual union all
select 39000,'039000','2012-064D','CZ-2C Stage 2','CZ-2C Stage 2',69319,'Reentered','2016-06-27 00:00:00','2012-11-26 00:00:00',94.19,472,487,97.35,'Sun-Synch' from dual union all
select 39001,'039001','2012-064E','Sep motor cover','',69319,'Reentered','2014-12-05 00:00:00','2012-12-27 00:00:00',97.05,492,743,97.59,'Sun-Synch' from dual union all
select 39002,'039002','2012-025J','deb H-2A F14','',69280,'Reentered','2013-01-20 00:00:00','2012-11-19 00:00:00',97.7,635,663,98.18,'Sun-Synch' from dual union all
select 39003,'039003','2012-025K','deb H-2A F14','',69280,'Reentered','2013-05-20 00:00:00','2012-11-19 00:00:00',98.26,641,710,98.16,'Sun-Synch' from dual union all
select 39004,'039004','2012-025L','deb H-2A F14','',69280,'Reentered','2013-07-30 00:00:00','2012-11-19 00:00:00',98.32,639,718,98.17,'Sun-Synch' from dual union all
select 39005,'039005','2012-025M','deb H-2A F14','',69280,'Reentered','2013-08-15 00:00:00','2012-11-19 00:00:00',98.68,636,756,98.16,'Sun-Synch' from dual union all
select 39006,'039006','2012-025N','deb H-2A F14','',69280,'Reentered','2012-12-21 00:00:00','2012-11-19 00:00:00',96.89,594,626,98.18,'Sun-Synch' from dual union all
select 39007,'039007','2012-025P','deb H-2A F14','',69280,'Reentered','2013-01-18 00:00:00','2012-11-19 00:00:00',97.71,636,663,98.19,'Sun-Synch' from dual union all
select 39008,'039008','2012-065A','Echostar XVI','Echostar 16',69320,'In Earth orbit','','2012-11-28 00:00:00',1435.65,35755,35801,.06,'Stationary' from dual union all
select 39009,'039009','2012-065B','Briz-M DTB','Briz-M torus tank',69320,'In Earth orbit','','2012-11-28 00:00:00',645.54,1349,35380,30.62,'GEO Transfer' from dual union all
select 3901,'003901','1968-097BN','deb Kosmos-252                    ','[UNID027] RCS 0.0',16824,'In Earth orbit','','1992-02-01 00:00:00',112.51,663,2021,61.29,'Intermediate' from dual union all
select 39010,'039010','2012-065C','Briz-M No. 99528','Briz-M 14S43 No. 99528',69320,'In Earth orbit','','2012-11-28 00:00:00',238.36,320,12392,49.73,'Medium' from dual union all
select 39011,'039011','2012-066A','Yaogan Weixing 16','Jianbing-8 02 xing',69321,'In Earth orbit','','2012-12-02 00:00:00',106.93,1080,1089,63.38,'Intermediate' from dual union all
select 39012,'039012','2012-066B','Yaogan Weixing 16 fu xing yi','Jianbing-8 02 subsat 1',69321,'In Earth orbit','','2012-12-02 00:00:00',106.91,1078,1090,63.38,'Intermediate' from dual union all
select 39013,'039013','2012-066C','Yaogan Weixing 16 fu xing er','Jianbing-8 02 subsat 2',69321,'In Earth orbit','','2012-12-02 00:00:00',106.32,1032,1081,63.38,'Intermediate' from dual union all
select 39014,'039014','2012-066D','CZ-4C Stage 3','CZ-4C Stage 3',69321,'In Earth orbit','','2012-12-02 00:00:00',105.09,891,1107,63.44,'Intermediate' from dual union all
select 39015,'039015','2012-066E','Subsatellite cover','',69321,'In Earth orbit','','2012-12-02 00:00:00',107.14,1084,1104,63.39,'Intermediate' from dual union all
select 39016,'039016','2012-066F','Subsatellite cover','',69321,'In Earth orbit','','2012-12-03 00:00:00',107.11,1081,1105,63.39,'Intermediate' from dual union all
select 39017,'039017','2012-067A','Zhongxing 12','zhongxing shier hao weix',69322,'In Earth orbit','','2012-12-10 00:00:00',1435.88,35758,35807,.04,'Stationary' from dual union all
select 39018,'039018','2012-067B','CZ-3B Stage 3','CZ-3B Stage 3',69322,'Reentered','2013-04-22 00:00:00','2012-12-09 00:00:00',918.58,278,49570,26.69,'Highly Ellip' from dual union all
select 39019,'039019','2012-068A','Pleiades 1B','Pleiades HR-2',69323,'In Earth orbit','','2012-12-02 00:00:00',98.29,673,681,98.2,'Sun-Synch' from dual union all
select 3902,'003902','1968-097BP','deb Kosmos-252 [*]                ','',16824,'Reentered','1981-12-02 00:00:00','1969-04-16 00:00:00',112.61,545,2148,62.32,'Intermediate' from dual union all
select 39020,'039020','2012-069A','EUTELSAT 70B','EUTELSAT 70B',69324,'In Earth orbit','','2012-12-11 00:00:00',1431.06,35629,35746,.03,'Drift GEO' from dual union all
select 39021,'039021','2012-069B','Blok DM-SL No. 34L','Blok DM-SL',69324,'In Earth orbit','','2012-12-12 00:00:00',648.93,1373,35529,.02,'GEO Transfer' from dual union all
select 39022,'039022','2012-070A','Yamal-402','Yamal 402',69325,'In Earth orbit','','2012-12-16 00:00:00',1430.34,35627,35721,.06,'Drift GEO' from dual union all
select 39023,'039023','2012-070B','Briz-M No. 99535','Briz-M 14S43 No. 99535',69325,'In Earth orbit','','2012-12-21 00:00:00',633.51,499,35613,47.92,'GEO Transfer' from dual union all
select 39024,'039024','2012-070C','Briz-M DTB','Briz-M torus tank',69325,'In Earth orbit','','2012-12-15 00:00:00',685.67,3061,35698,26.02,'GEO Transfer' from dual union all
select 39025,'039025','2012-071A','X-37B OTV-3','X-37B No. 1',69326,'Landed','2014-10-17 00:00:00','2012-12-20 00:00:00',91.52,342,357,43.49,'Intermediate' from dual union all
select 39026,'039026','2012-072A','Kwangmyongsong-3-ho 2-hogi','KMS-3 Unit 2',69327,'In Earth orbit','','2012-12-14 00:00:00',95.43,498,581,97.41,'Sun-Synch' from dual union all
select 39027,'039027','2012-072B','Unha-3 Stage 3','',69327,'In Earth orbit','','2012-12-14 00:00:00',95.49,497,587,97.41,'Sun-Synch' from dual union all
select 39028,'039028','2012-072C','deb Kwangmyongsong-3 F2','',69327,'In Earth orbit','','2012-12-14 00:00:00',95.32,498,571,97.45,'Sun-Synch' from dual union all
select 39029,'039029','2012-072D','deb Kwangmyongsong-3 F2','',69327,'In Earth orbit','','2012-12-14 00:00:00',95.47,498,585,97.37,'Sun-Synch' from dual union all
select 3903,'003903','1968-097BQ','deb Kosmos-252','',16824,'Reentered','1982-11-12 00:00:00','1969-05-07 00:00:00',113.06,649,2086,62.32,'Intermediate' from dual union all
select 39030,'039030','2012-073A','Gokturk-2','GK-2',69328,'In Earth orbit','','2012-12-19 00:00:00',98.33,669,689,98.16,'Sun-Synch' from dual union all
select 39031,'039031','2012-073B','CZ-2D Stage 2','CZ-2D Stage 2',69328,'Reentered','2012-12-31 00:00:00','2012-12-19 00:00:00',92.68,145,667,96.57,'Sun-Synch' from dual union all
select 39032,'039032','2012-074A','Soyuz TMA-07M','Soyuz 11F732A47 No. 704A',69329,'Landed','2013-05-14 00:00:00','2012-12-20 00:00:00',90.53,252,349,51.65,'Intermediate' from dual union all
select 39033,'039033','2012-074B','11S510','11S510',69329,'Reentered','2012-12-22 00:00:00','2012-12-19 00:00:00',88.68,191,228,51.64,'Intermediate' from dual union all
select 39034,'039034','2012-075A','Skynet 5D','Skynet 5D',69330,'In Earth orbit','','2012-12-28 00:00:00',1432.21,35515,35906,.23,'Drift GEO' from dual union all
select 39035,'039035','2012-075B','Mexsat Bicentenario','Mexsat-3',69330,'In Earth orbit','','2012-12-27 00:00:00',1430.34,35557,35791,.1,'Drift GEO' from dual union all
select 39036,'039036','2012-075C','ESC-A 567','Ariane 5 ESC-A',69330,'In Earth orbit','','2012-12-27 00:00:00',631.1,260,35728,2.31,'GEO Transfer' from dual union all
select 39037,'039037','2012-075D','Sylda 5','Sylda 5 No. 53A',69330,'In Earth orbit','','2012-12-27 00:00:00',631.06,260,35726,1.98,'GEO Transfer' from dual union all
select 39038,'039038','2012-064F','Sep motor cover','',69319,'Reentered','2014-11-11 00:00:00','2012-12-27 00:00:00',97.03,493,740,97.13,'Sun-Synch' from dual union all
select 39039,'039039','2012-064G','Sep motor cover','',69319,'Reentered','2014-01-09 00:00:00','2012-12-27 00:00:00',95.49,462,622,97.53,'Sun-Synch' from dual union all
select 3904,'003904','1968-097BR','deb Kosmos-252 [*]                ','',16824,'Reentered','1979-10-24 00:00:00','1979-10-14 00:00:00',94.43,392,590,62.45,'Intermediate' from dual union all
select 39040,'039040','2012-064H','Sep motor cover','',69319,'Reentered','2014-01-04 00:00:00','2012-12-24 00:00:00',95.62,465,633,97.14,'Sun-Synch' from dual union all
select 39041,'039041','1988-006G','deb DMSP F-9','',61255,'Reentered','2014-06-13 00:00:00','2012-12-27 00:00:00',100.95,799,809,98.85,'Sun-Synch' from dual union all
select 39042,'039042','1988-006H','deb DMSP F-9','',61255,'Reentered','2013-01-06 00:00:00','2012-12-27 00:00:00',101.35,517,1129,98.89,'Sun-Synch' from dual union all
select 39043,'039043','1988-006J','deb DMSP F-9','',61255,'Reentered','2017-03-20 00:00:00','2012-12-27 00:00:00',101.66,809,866,98.86,'Sun-Synch' from dual union all
select 39044,'039044','1988-006K','deb DMSP F-9','',61255,'Reentered','2013-07-18 00:00:00','2012-12-27 00:00:00',100.81,775,819,98.88,'Sun-Synch' from dual union all
select 39045,'039045','1988-006L','deb DMSP F-9','',61255,'Reentered','2013-03-31 00:00:00','2012-12-26 00:00:00',102.23,797,932,98.66,'Sun-Synch' from dual union all
select 39046,'039046','1988-006M','deb DMSP F-9','',61255,'In Earth orbit','','2012-12-27 00:00:00',102.12,795,923,98.9,'Sun-Synch' from dual union all
select 39047,'039047','1994-074X','deb Resurs-O1','',65470,'Reentered','2013-01-13 00:00:00','2012-12-25 00:00:00',92.55,394,405,98.11,'Sun-Synch' from dual union all
select 39048,'039048','1994-074Y','deb Resurs-O1','',65470,'Reentered','2013-12-22 00:00:00','2012-12-27 00:00:00',96.86,607,609,98.15,'Sun-Synch' from dual union all
select 39049,'039049','1994-074Z','deb Resurs-O1','',65470,'Reentered','2015-02-12 00:00:00','2012-12-27 00:00:00',97.38,632,635,98.15,'Sun-Synch' from dual union all
select 3905,'003905','1969-029AD','deb Blok-E','',18296,'Reentered','1969-09-12 00:00:00','1969-09-10 00:00:00',90.14,281,282,80.98,'Intermediate' from dual union all
select 39050,'039050','1994-074AA','deb Resurs-O1','',65470,'Reentered','2014-03-16 00:00:00','2012-12-27 00:00:00',97.35,630,634,98.16,'Sun-Synch' from dual union all
select 39051,'039051','1994-074AB','deb Resurs-O1','',65470,'Reentered','2014-03-27 00:00:00','2012-12-26 00:00:00',97.39,633,635,98.15,'Sun-Synch' from dual union all
select 39052,'039052','1999-057SZ','CZ-4 debris','',66738,'In Earth orbit','','2012-12-27 00:00:00',96.93,563,661,98.76,'Sun-Synch' from dual union all
select 39053,'039053','1983-044GE','deb Kosmos-1461','',55801,'Reentered','2013-03-30 00:00:00','2012-12-25 00:00:00',95.3,479,588,65,'Intermediate' from dual union all
select 39054,'039054','1992-047AV','deb SOZ','',64759,'In Earth orbit','','2012-12-13 00:00:00',167.9,1289,6118,65.61,'Medium' from dual union all
select 39055,'039055','1990-081DP','deb CZ-4 Stage 3','',63323,'In Earth orbit','','2012-12-21 00:00:00',101.8,837,851,98.8,'Sun-Synch' from dual union all
select 39056,'039056','1981-053MU','deb Kosmos-1275','',52576,'In Earth orbit','','2012-12-18 00:00:00',105.87,938,1133,82.95,'Intermediate' from dual union all
select 39057,'039057','2013-001A','Kosmos-2482','Rodnik-S',69464,'In Earth orbit','','2013-01-16 00:00:00',115.74,1476,1502,82.49,'Intermediate' from dual union all
select 39058,'039058','2013-001B','Kosmos-2483','Rodnik-S',69464,'In Earth orbit','','2013-01-16 00:00:00',115.75,1478,1501,82.5,'Intermediate' from dual union all
select 39059,'039059','2013-001C','Kosmos-2484','Rodnik-S',69464,'In Earth orbit','','2013-01-16 00:00:00',115.79,1480,1503,82.5,'Intermediate' from dual union all
select 3906,'003906','1969-040A','Kosmos-280','Zenit-4M',18307,'Landed','1969-05-06 00:00:00','1969-04-27 00:00:00',89.01,206,246,51.61,'Intermediate' from dual union all
select 39060,'039060','2013-001D','Briz-KM No. 72518','Briz-KM 14S45',69464,'In Earth orbit','','2013-01-16 00:00:00',115.87,1486,1504,82.49,'Intermediate' from dual union all
select 39061,'039061','2013-002A','JSE reda-4 gouki','IGS Radar-4',69465,'In Earth orbit','','2013-01-27 00:00:00',94.84,508,514,97.51,'Sun-Synch' from dual union all
select 39062,'039062','2013-002B','JSE jissho eisei','IGS Optical 5 Demo',69465,'In Earth orbit','','2013-01-27 00:00:00',94.96,511,523,97.5,'Sun-Synch' from dual union all
select 39063,'039063','2013-002C','H-2A F22 Stage 2','H-2A Stage 2',69465,'Reentered','2015-02-04 00:00:00','2013-01-27 00:00:00',94.1,429,521,97.45,'Sun-Synch' from dual union all
select 39064,'039064','2013-002D','4/4D-LC Adapter','Adapter',69465,'Reentered','2014-08-16 00:00:00','2013-01-27 00:00:00',94.88,512,514,97.51,'Sun-Synch' from dual union all
select 39065,'039065','2013-002E','4/4D-LC Lower Fairing Half','Fairing Half',69465,'Reentered','2014-01-12 00:00:00','2013-01-27 00:00:00',94.86,511,513,97.51,'Sun-Synch' from dual union all
select 39066,'039066','2013-002F','4/4D-LC Lower Fairing Half','Fairing Half',69465,'Reentered','2014-01-22 00:00:00','2013-01-27 00:00:00',94.86,511,513,97.5,'Sun-Synch' from dual union all
select 39067,'039067','2013-002G','deb H-2A','',69465,'Reentered','2013-02-09 00:00:00','2013-01-27 00:00:00',94.08,431,517,97.45,'Sun-Synch' from dual union all
select 39068,'039068','2013-003A','Naro Scientific Satellite','STSat-2C',69466,'In Earth orbit','','2013-02-07 00:00:00',103.03,296,1508,80.28,'Intermediate' from dual;
insert into SATELLITE
select 39069,'039069','2013-003B','Naro-1 Stage 2','KSLV-1 Stage 2',69466,'In Earth orbit','','2013-02-06 00:00:00',103.06,297,1510,80.28,'Intermediate' from dual union all
select 3907,'003907','1969-040B','11A57I','11A57I',18307,'Reentered','1969-04-26 00:00:00','1969-04-25 00:00:00',88.33,158,225,51.52,'Intermediate' from dual union all
select 39070,'039070','2013-004A','TDRS 11','TDRS K',69467,'In Earth orbit','','2013-03-15 00:00:00',1435.96,35737,35831,6.99,'Inclined GEO' from dual union all
select 39071,'039071','2013-004B','Centaur AV-036','Centaur AV-036',69467,'In Earth orbit','','2013-02-03 00:00:00',686.43,4146,34651,26.05,'GEO Transfer' from dual union all
select 39072,'039072','2013-005A','Globalstar M078','Globalstar-2 FM15',69468,'In Earth orbit','','2013-02-09 00:00:00',103.45,918,926,52.01,'Intermediate' from dual union all
select 39073,'039073','2013-005B','Globalstar M093','Globalstar-2 FM19',69468,'In Earth orbit','','2013-02-09 00:00:00',103.43,916,925,52.01,'Intermediate' from dual union all
select 39074,'039074','2013-005C','Globalstar M094','Globalstar-2 FM21',69468,'In Earth orbit','','2013-02-09 00:00:00',103.42,915,926,52.01,'Intermediate' from dual union all
select 39075,'039075','2013-005D','Globalstar M095','Globalstar-2 FM22',69468,'In Earth orbit','','2013-02-09 00:00:00',103.42,915,926,52.01,'Intermediate' from dual union all
select 39076,'039076','2013-005E','Globalstar M096','Globalstar-2 FM23',69468,'In Earth orbit','','2013-02-09 00:00:00',103.46,919,926,52.01,'Intermediate' from dual union all
select 39077,'039077','2013-005F','Globalstar M097','Globalstar-2 FM25',69468,'In Earth orbit','','2013-02-08 00:00:00',103.46,919,926,52.01,'Intermediate' from dual union all
select 39078,'039078','2013-006A','Amazonas 3','Amazonas 3',69469,'In Earth orbit','','2013-02-15 00:00:00',1434.5,35697,35814,.09,'Drift GEO' from dual union all
select 39079,'039079','2013-006B','Azerspace','Africasat-1a',69469,'In Earth orbit','','2013-02-17 00:00:00',1434.15,35733,35764,.05,'Drift GEO' from dual union all
select 3908,'003908','1968-091BL','deb Kosmos-249 [*]','',16818,'In Earth orbit','','1969-04-24 00:00:00',112.51,534,2150,62.27,'Intermediate' from dual union all
select 39080,'039080','2013-006C','ESC-A 568','Ariane 5 ESC-A',69469,'In Earth orbit','','2013-02-15 00:00:00',633.08,258,35832,6.36,'GEO Transfer' from dual union all
select 39081,'039081','2013-006D','Sylda 5','Sylda 5 No. 54A',69469,'In Earth orbit','','2013-02-18 00:00:00',631.35,267,35734,6,'GEO Transfer' from dual union all
select 39082,'039082','2013-007A','Progress M-18M','Progress 7K-TGM No.418',69470,'Deorbited','2013-07-26 00:00:00','2013-02-11 00:00:00',92.74,401,416,51.65,'Intermediate' from dual union all
select 39083,'039083','2013-007B','11S510','11S510',69470,'Reentered','2013-02-13 00:00:00','2013-02-12 00:00:00',88.23,172,202,51.64,'Intermediate' from dual union all
select 39084,'039084','2013-008A','Landsat 8','Landsat Data Cont. Missi',69471,'In Earth orbit','','2013-02-14 00:00:00',98.27,668,683,98.24,'Sun-Synch' from dual union all
select 39085,'039085','2013-008B','Centaur AV-035','Centaur AV-035',69471,'Deep Space','2013-02-12 00:00:00','2013-02-11 00:00:00',1540.84,688,101827,98.28,'Earth Escape' from dual union all
select 39086,'039086','2013-009A','SARAL','SARAL',69472,'In Earth orbit','','2013-02-27 00:00:00',100.45,772,788,98.63,'Sun-Synch' from dual union all
select 39087,'039087','2013-009B','AAUSAT3','AAUSAT3',69472,'In Earth orbit','','2013-02-27 00:00:00',100.41,770,786,98.63,'Sun-Synch' from dual union all
select 39088,'039088','2013-009C','Sapphire','Sapphire',69472,'In Earth orbit','','2013-02-27 00:00:00',100.42,771,786,98.63,'Sun-Synch' from dual union all
select 39089,'039089','2013-009D','NEOSSat','NEOSSat',69472,'In Earth orbit','','2013-02-26 00:00:00',100.42,771,786,98.63,'Sun-Synch' from dual union all
select 3909,'003909','1968-091BM','deb Kosmos-249','',16818,'Reentered','1975-03-16 00:00:00','1973-08-16 00:00:00',111.21,503,2062,62.35,'Intermediate' from dual union all
select 39090,'039090','2013-009E','STRaND-1','STRaND-1',69472,'In Earth orbit','','2013-02-27 00:00:00',100.38,771,783,98.63,'Sun-Synch' from dual union all
select 39091,'039091','2013-009F','TUGSAT-1','BRITE-A',69472,'In Earth orbit','','2013-02-26 00:00:00',100.37,770,783,98.63,'Sun-Synch' from dual union all
select 39092,'039092','2013-009G','UniBRITE','BRITE-U',69472,'In Earth orbit','','2013-02-27 00:00:00',100.38,771,783,98.63,'Sun-Synch' from dual union all
select 39093,'039093','2013-009H','PS4','PS4 PSLV-C20',69472,'In Earth orbit','','2013-02-27 00:00:00',99.58,706,771,98.65,'Sun-Synch' from dual union all
select 39094,'039094','2013-009J','DLA-U','PSLV-C20-DLA-U',69472,'In Earth orbit','','2013-02-27 00:00:00',100.38,771,783,98.63,'Sun-Synch' from dual union all
select 39095,'039095','1973-086HH','deb Delta 98','',30688,'In Earth orbit','','2013-02-23 00:00:00',117.01,1501,1592,101.99,'Sun-Synch' from dual union all
select 39096,'039096','1973-086HJ','deb Delta 98','',30688,'In Earth orbit','','2013-02-11 00:00:00',116.48,1466,1579,101.3,'Sun-Synch' from dual union all
select 39097,'039097','1973-086HK','deb Delta 98','',30688,'In Earth orbit','','2013-02-16 00:00:00',116.7,1508,1556,101.95,'Sun-Synch' from dual union all
select 39098,'039098','1977-065GH','deb Delta 132','',42193,'In Earth orbit','','2013-02-16 00:00:00',104.67,503,1455,29.04,'Intermediate' from dual union all
select 39099,'039099','1977-065GJ','deb Delta 132','',42193,'In Earth orbit','','2013-02-20 00:00:00',103.93,612,1277,28.35,'Intermediate' from dual union all
select 391,'000391','62 A Phi 3','BOZ','BOZ',8150,'Reentered','1962-09-17 00:00:00','1962-09-14 00:00:00',88.63,189,225,64.75,'Intermediate' from dual union all
select 3910,'003910','1968-091BN','deb Kosmos-249','',16818,'Reentered','1982-05-18 00:00:00','1982-02-10 00:00:00',103.71,602,1266,62.26,'Intermediate' from dual union all
select 39100,'039100','1977-065GK','deb Delta 132','',42193,'In Earth orbit','','2013-02-21 00:00:00',102.03,465,1245,29.31,'Intermediate' from dual union all
select 39101,'039101','1977-065GL','deb Delta 132','',42193,'In Earth orbit','','2013-02-24 00:00:00',100.42,459,1098,29.32,'Intermediate' from dual union all
select 39102,'039102','1977-065GM','deb Delta 132','',42193,'In Earth orbit','','2013-02-21 00:00:00',110.61,542,1968,28.98,'Intermediate' from dual union all
select 39103,'039103','1986-019VR','deb Ariane V16','',59212,'In Earth orbit','','2013-02-25 00:00:00',102.44,524,1225,62.26,'Intermediate' from dual union all
select 39104,'039104','1986-019VS','deb Ariane V16','',59212,'In Earth orbit','','2013-02-19 00:00:00',99.85,717,786,98.49,'Sun-Synch' from dual union all
select 39105,'039105','1986-019VT','deb Ariane V16','',59212,'In Earth orbit','','2013-02-19 00:00:00',102.28,779,955,99.09,'Sun-Synch' from dual union all
select 39106,'039106','1986-019VU','deb Ariane V16','',59212,'In Earth orbit','','2013-02-23 00:00:00',101.57,785,881,98.96,'Sun-Synch' from dual union all
select 39107,'039107','1986-019VV','deb Ariane V16','',59212,'In Earth orbit','','2013-02-23 00:00:00',102.58,764,998,99.18,'Sun-Synch' from dual union all
select 39108,'039108','1986-019VW','deb Ariane V16','',59212,'Reentered','2014-09-08 00:00:00','2013-02-11 00:00:00',98.6,654,730,99.05,'Sun-Synch' from dual union all
select 39109,'039109','1993-036BTY','deb Kosmos-2251','',65138,'In Earth orbit','','2013-02-25 00:00:00',99.08,658,772,74.05,'Intermediate' from dual union all
select 3911,'003911','1968-091BP','deb Kosmos-249','',16818,'In Earth orbit','','1992-02-03 00:00:00',110.78,566,1960,62.96,'Intermediate' from dual union all
select 39110,'039110','1993-036BTZ','deb Kosmos-2251','',65138,'In Earth orbit','','2013-02-09 00:00:00',98.66,610,780,73.96,'Intermediate' from dual union all
select 39111,'039111','1993-036BUA','deb Kosmos-2251','',65138,'In Earth orbit','','2013-02-23 00:00:00',100.22,751,786,74.04,'Intermediate' from dual union all
select 39112,'039112','1999-025EVZ','deb FY-1C','',66706,'In Earth orbit','','2013-02-25 00:00:00',101.2,777,854,99.12,'Sun-Synch' from dual union all
select 39113,'039113','1999-025EWA','deb FY-1C','',66706,'In Earth orbit','','2013-02-19 00:00:00',102.56,845,915,99.17,'Sun-Synch' from dual union all
select 39114,'039114','1999-025EWB','deb FY-1C','',66706,'In Earth orbit','','2013-02-21 00:00:00',102.3,805,930,99.1,'Sun-Synch' from dual union all
select 39115,'039115','2013-010A','Dragon CRS-2','Dragon C104',69473,'Landed','2013-03-26 00:00:00','2013-03-01 00:00:00',89.74,203,321,51.68,'Intermediate' from dual union all
select 39116,'039116','2013-010B','Falcon 9-005 Stage 2','F9-005 St2',69473,'Reentered','2013-03-11 00:00:00','2013-03-01 00:00:00',89.73,205,318,51.66,'Intermediate' from dual union all
select 39117,'039117','2013-010C','Dragon solar array cover','',69473,'Reentered','2013-03-04 00:00:00','2013-03-01 00:00:00',89.81,197,334,51.67,'Intermediate' from dual union all
select 39118,'039118','2013-010D','Dragon solar array cover','',69473,'Reentered','2013-03-04 00:00:00','2013-03-01 00:00:00',89.61,202,309,51.68,'Intermediate' from dual union all
select 39119,'039119','2009-049J','deb Blits','[Collision]',68730,'In Earth orbit','','2013-03-03 00:00:00',101.29,817,823,98.61,'Sun-Synch' from dual union all
select 3912,'003912','1968-091BQ','deb Kosmos-249','',16818,'In Earth orbit','','1992-02-02 00:00:00',114.46,784,2078,62.27,'Intermediate' from dual union all
select 39120,'039120','2013-011A','SBIRS GEO-2','SBIRS GEO SV2',69474,'In Earth orbit','','2013-04-01 00:00:00',1436.27,35780,35800,1,'Stationary' from dual union all
select 39121,'039121','2013-011B','Centaur AV-037','Centaur AV-037',69474,'Reentered','2014-11-22 00:00:00','2013-07-25 00:00:00',627.12,279,35504,21.91,'GEO Transfer' from dual union all
select 39122,'039122','2013-012A','Satmex 8','',69475,'In Earth orbit','','2013-05-27 00:00:00',1436.06,35778,35793,0,'Stationary' from dual union all
select 39123,'039123','2013-012B','Briz-M No. 99536','Briz-M 14S43 No. 99536',69475,'In Earth orbit','','2013-05-26 00:00:00',700.98,5428,34095,18.19,'GEO Transfer' from dual union all
select 39124,'039124','2013-012C','Briz-M DTB','Briz-M torus tank',69475,'In Earth orbit','','2013-05-27 00:00:00',321.27,339,17982,49.49,'Highly Ellip' from dual union all
select 39125,'039125','2013-013A','Soyuz TMA-08M','Soyuz 11F732A47 No. 708',69476,'Landed','2013-09-11 00:00:00','2013-03-29 00:00:00',92.75,401,418,51.65,'Intermediate' from dual union all
select 39126,'039126','2013-013B','11S510','11S510',69476,'Reentered','2013-03-31 00:00:00','2013-03-30 00:00:00',88.26,179,198,51.65,'Intermediate' from dual union all
select 39127,'039127','2013-014A','Anik G-1','Anik G-1',69477,'In Earth orbit','','2013-05-16 00:00:00',1436.11,35777,35797,.01,'Stationary' from dual union all
select 39128,'039128','2013-014B','Briz-M No. 99538','Briz-M 14S43 No. 99538',69477,'In Earth orbit','','2013-05-16 00:00:00',756.19,8003,34230,13.92,'Medium' from dual union all
select 39129,'039129','2013-014C','Briz-M DTB','Briz-M torus tank',69477,'In Earth orbit','','2013-05-15 00:00:00',378.72,353,21570,49.41,'Highly Ellip' from dual union all
select 3913,'003913','1969-029AE','deb Blok-E','',18296,'Reentered','1970-03-07 00:00:00','1970-02-25 00:00:00',92.84,373,455,81.31,'Intermediate' from dual union all
select 39130,'039130','2013-015A','Bion-M No. 1','Bion-M No. 1',69478,'Landed','2013-05-19 00:00:00','2013-04-26 00:00:00',96.04,557,581,64.88,'Intermediate' from dual union all
select 39131,'039131','2013-015B','OSSI 1','OSSI 1',69478,'Reentered','2013-07-08 00:00:00','2013-04-21 00:00:00',92.63,258,549,64.94,'Intermediate' from dual union all
select 39132,'039132','2013-015C','Dove-2','Dove 0405',69478,'In Earth orbit','','2013-04-26 00:00:00',96.06,559,580,64.88,'Intermediate' from dual union all
select 39133,'039133','2013-015D','AIST 2','AIST 2',69478,'In Earth orbit','','2013-04-26 00:00:00',96.05,558,581,64.88,'Intermediate' from dual union all
select 39134,'039134','2013-015E','SOMP','SOMP',69478,'In Earth orbit','','2013-04-27 00:00:00',96,554,579,64.88,'Intermediate' from dual union all
select 39135,'039135','2013-015F','BEESAT 3','BEESAT 3',69478,'In Earth orbit','','2013-04-27 00:00:00',96,554,579,64.88,'Intermediate' from dual union all
select 39136,'039136','2013-015G','BEESAT 2','BEESAT 2',69478,'In Earth orbit','','2013-04-26 00:00:00',96,554,579,64.88,'Intermediate' from dual union all
select 39137,'039137','2013-015H','Blok-I 14S54/RD-0110','374BL02A No. B15000-014',69478,'Reentered','2013-08-03 00:00:00','2013-04-26 00:00:00',92.45,251,539,64.9,'Intermediate' from dual union all
select 39138,'039138','1998-067CT','Obstanovka cable reel','Obstanovka cable reel',66503,'Reentered','2013-06-26 00:00:00','2013-02-12 00:00:00',92.74,401,416,51.65,'Intermediate' from dual union all
select 39139,'039139','1998-067CU','Obstanovka ShKD-1 container','Probe container',66503,'Reentered','2013-05-26 00:00:00','2013-04-19 00:00:00',92.74,401,416,51.65,'Intermediate' from dual union all
select 3914,'003914','1969-041A','OPS 1101','CORONA J-44',18308,'Reentered Attached','1969-05-23 00:00:00','1969-05-10 00:00:00',89.5,175,324,65,'Intermediate' from dual union all
select 39140,'039140','1998-067CV','Obstanovka ShKD-2 container','Probe container',66503,'Reentered','2013-06-28 00:00:00','2013-04-19 00:00:00',92.74,401,416,51.65,'Intermediate' from dual union all
select 39141,'039141','1998-067CW','Vinoslivost panel 1','ISS Vinoslivost P1',66503,'Reentered','2013-07-13 00:00:00','2012-01-28 00:00:00',92.36,375,406,51.64,'Intermediate' from dual union all
select 39142,'039142','2013-016A','Bell','TES-2/PhoneSat 1.0 V1B',69479,'Reentered','2013-04-27 00:00:00','2013-04-24 00:00:00',88.95,218,228,51.64,'Intermediate' from dual union all
select 39143,'039143','2013-016B','Dove-1','Dove 0601',69479,'Reentered','2013-04-27 00:00:00','2013-04-24 00:00:00',88.91,210,232,51.62,'Intermediate' from dual union all
select 39144,'039144','2013-016C','Alexander','PhoneSat 2.0',69479,'Reentered','2013-04-27 00:00:00','2013-04-24 00:00:00',89.02,218,235,51.62,'Intermediate' from dual union all
select 39145,'039145','2013-016D','Cygnus Mass Simulator','Cygnus Mass Simulator',69479,'Reentered','2013-05-10 00:00:00','2013-04-29 00:00:00',89.1,223,236,51.63,'Intermediate' from dual union all
select 39146,'039146','2013-016E','Graham','PhoneSat 1.0 V1A',69479,'Reentered','2013-04-26 00:00:00','2013-04-24 00:00:00',88.89,213,227,51.63,'Intermediate' from dual union all
select 39147,'039147','2013-016F','Antares A-1 Stage 2','Castor 30',69479,'Reentered','2013-05-01 00:00:00','2013-04-24 00:00:00',89.2,226,244,51.6,'Intermediate' from dual union all
select 39148,'039148','2013-017A','Progress M-19M','Progress 7K-TGM No.419',69480,'Deorbited','2013-06-19 00:00:00','2013-04-25 00:00:00',91.04,298,354,51.64,'Intermediate' from dual union all
select 39149,'039149','2013-017B','11S510','11S510',69480,'Reentered','2013-04-26 00:00:00','2013-04-25 00:00:00',87.98,173,176,51.62,'Intermediate' from dual union all
select 3915,'003915','1969-041B','LAMPAN II','P-11 No. 4417',18308,'Reentered','1970-02-16 00:00:00','1969-06-02 00:00:00',93.19,408,454,65.71,'Intermediate' from dual union all
select 39150,'039150','2013-018A','Gao Fen 1 01 xing','gaofen 1',69481,'In Earth orbit','','2013-04-28 00:00:00',97.69,635,662,98.36,'Sun-Synch' from dual union all
select 39151,'039151','2013-018B','NEE-01 Pegaso','Pegaso',69481,'In Earth orbit','','2013-04-28 00:00:00',97.55,629,654,98.07,'Sun-Synch' from dual union all
select 39152,'039152','2013-018C','Turksat-3USat','Turksat-3USat',69481,'In Earth orbit','','2013-04-28 00:00:00',97.54,629,652,98.06,'Sun-Synch' from dual union all
select 39153,'039153','2013-018D','CubeBug-1','Capitan Beto',69481,'In Earth orbit','','2013-04-28 00:00:00',97.53,629,651,98.07,'Sun-Synch' from dual union all
select 39154,'039154','2013-018E','CZ-2D Stage 2','CZ-2D Stage 2',69481,'In Earth orbit','','2013-04-28 00:00:00',97.53,629,651,98.07,'Sun-Synch' from dual union all
select 39155,'039155','2013-019A','Glonass-M No. 47','Uragan-M No. 747',69482,'In Earth orbit','','2013-05-26 00:00:00',675.71,19086,19173,64.77,'Medium' from dual union all
select 39156,'039156','2013-019B','Fregat No. 1047','Fregat 14S44 No. 1047',69482,'In Earth orbit','','2013-05-27 00:00:00',691.31,19324,19717,64.71,'Medium' from dual union all
select 39157,'039157','2013-020A','Zhongxing 11','zhongxing shiyi hao',69483,'In Earth orbit','','2013-06-01 00:00:00',1436.04,35776,35795,.61,'Stationary' from dual union all
select 39158,'039158','2013-020B','CZ-3B Stage 3','CZ-3B Stage 3',69483,'In Earth orbit','','2013-06-01 00:00:00',776.87,242,42989,26.75,'Highly Ellip' from dual;
insert into SATELLITE
select 39159,'039159','2013-021A','PROBA-V','Proba-V',69484,'In Earth orbit','','2013-05-09 00:00:00',101.2,812,819,98.73,'Sun-Synch' from dual union all
select 3916,'003916','1968-084D','ESRO 1A despin weight','',16811,'Reentered','1969-05-09 00:00:00','1969-05-02 00:00:00',90.83,216,414,93.16,'Polar' from dual union all
select 39160,'039160','2013-021B','VNRedSat-1A','VNRedSat-1A',69484,'In Earth orbit','','2013-05-09 00:00:00',98.16,670,672,98.13,'Sun-Synch' from dual union all
select 39161,'039161','2013-021C','ESTCube-1','ESTCube-1',69484,'In Earth orbit','','2013-05-09 00:00:00',98.03,658,671,98.13,'Sun-Synch' from dual union all
select 39162,'039162','2013-021D','VESPA Upper Part','VUP',69484,'In Earth orbit','','2013-05-10 00:00:00',99.66,670,815,98.73,'Sun-Synch' from dual union all
select 39163,'039163','2013-022A','EUTELSAT 3D','EUTELSAT 3D',69485,'In Earth orbit','','2013-06-13 00:00:00',1436.04,35785,35785,.02,'Stationary' from dual union all
select 39164,'039164','2013-022B','Briz-M No. 99539','Briz-M 14S43 No. 99539',69485,'In Earth orbit','','2013-06-15 00:00:00',712.15,5840,34237,17.98,'GEO Transfer' from dual union all
select 39165,'039165','2013-022C','Briz-M DTB','Briz-M torus tank',69485,'In Earth orbit','','2013-06-13 00:00:00',325,344,18217,49.5,'Highly Ellip' from dual union all
select 39166,'039166','2013-023A','GPS IIF-4','GPS SVN 66',69486,'In Earth orbit','','2013-06-15 00:00:00',717.94,20177,20186,55.01,'Medium' from dual union all
select 39167,'039167','2013-023B','Centaur AV-039','Centaur AV-039',69486,'In Earth orbit','','2013-06-15 00:00:00',750.81,20500,21472,55.37,'Medium' from dual union all
select 39168,'039168','2013-024A','WGS SV-5','Wideband Global Satcom 5',69487,'In Earth orbit','','2013-05-26 00:00:00',1329.3,439,66900,24,'Highly Ellip' from dual union all
select 39169,'039169','2013-024B','Delta 362','DCSS-5 F10',69487,'In Earth orbit','','2013-05-26 00:00:00',1329.3,439,66900,24,'Highly Ellip' from dual union all
select 3917,'003917','1969-029AF','deb Blok-E','',18296,'Reentered','1969-07-01 00:00:00','1969-06-19 00:00:00',91.89,347,388,81.15,'Intermediate' from dual union all
select 39170,'039170','2013-025A','Soyuz TMA-09M','Soyuz 11F732A47 No. 709',69488,'Landed','2013-11-11 00:00:00','2013-05-29 00:00:00',92.85,408,421,51.65,'Intermediate' from dual union all
select 39171,'039171','2013-025B','11S510','11S510',69488,'Reentered','2013-05-31 00:00:00','2013-05-30 00:00:00',88.43,184,210,51.65,'Intermediate' from dual union all
select 39172,'039172','2013-026A','SES-6','SES-6',69489,'In Earth orbit','','2013-08-04 00:00:00',1436.06,35781,35791,.05,'Stationary' from dual union all
select 39173,'039173','2013-026B','Briz-M No. 99541','Briz-M 14S43 No. 99541',69489,'In Earth orbit','','2013-07-01 00:00:00',1302.98,2941,63336,27.85,'Highly Ellip' from dual union all
select 39174,'039174','2013-026C','Briz-M DTB','Briz-M torus tank',69489,'In Earth orbit','','2013-06-11 00:00:00',275.14,360,14909,50.77,'Highly Ellip' from dual union all
select 39175,'039175','2013-027A','Albert Einstein','ATV 4',69490,'Deorbited','2013-11-02 00:00:00','2013-06-07 00:00:00',89.65,251,264,51.62,'Intermediate' from dual union all
select 39176,'039176','2013-027B','EPS 592','EPS+ 592',69490,'Deorbited','2013-06-06 00:00:00','2013-06-05 00:00:00',89.54,250,254,51.62,'Intermediate' from dual union all
select 39177,'039177','2013-028A','Kosmos-2486','Persona No. 2',69491,'In Earth orbit','','2013-06-09 00:00:00',99.04,697,729,98.3,'Sun-Synch' from dual union all
select 39178,'039178','2013-028B','Blok-I 14S54','14S54',69491,'Reentered','2013-07-28 00:00:00','2013-06-09 00:00:00',93.15,185,672,98.31,'Sun-Synch' from dual union all
select 39179,'039179','2013-029A','Shenzhou 10','shenzhou shi hao',69492,'Landed','2013-06-26 00:00:00','2013-06-12 00:00:00',90.28,262,315,42.78,'Intermediate' from dual union all
select 3918,'003918','1969-029AG','deb Blok-E','',18296,'Reentered','1969-05-10 00:00:00','1969-03-01 00:00:00',92.12,308,449,81.2,'Intermediate' from dual union all
select 39180,'039180','2013-029B','CZ-2F Stage 2','CZ-2F Stage 2',69492,'Reentered','2013-06-21 00:00:00','2013-06-13 00:00:00',89.37,193,294,42.78,'Intermediate' from dual union all
select 39181,'039181','2013-029C','CZ-2F sep motor cover','',69492,'Reentered','2013-06-22 00:00:00','2013-06-13 00:00:00',89.25,202,273,42.77,'Intermediate' from dual union all
select 39182,'039182','2013-029D','CZ-2F sep motor cover','',69492,'Reentered','2013-06-13 00:00:00','2013-06-12 00:00:00',90.68,185,430,42.92,'Intermediate' from dual union all
select 39183,'039183','2013-029E','CZ-2F sep motor cover','',69492,'Reentered','2013-06-13 00:00:00','2013-06-12 00:00:00',90.89,188,449,42.97,'Intermediate' from dual union all
select 39184,'039184','2013-029F','CZ-2F sep motor cover','',69492,'Reentered','2013-06-13 00:00:00','2013-06-12 00:00:00',91.27,191,483,42.65,'Intermediate' from dual union all
select 39185,'039185','2013-029G','CZ-2F sep motor cover','',69492,'Reentered','2013-06-14 00:00:00','2013-06-12 00:00:00',91.19,191,475,42.6,'Intermediate' from dual union all
select 39186,'039186','2013-030A','Resurs-P No. 1','Resurs-P No. 1',69493,'In Earth orbit','','2013-07-02 00:00:00',93.89,458,472,97.28,'Sun-Synch' from dual union all
select 39187,'039187','2013-030B','Blok-I 14S54','14S54',69493,'Reentered','2013-09-25 00:00:00','2013-07-02 00:00:00',91.53,249,451,97.3,'Sun-Synch' from dual union all
select 39188,'039188','2013-031A','O3b FM5','O3b 5',69494,'In Earth orbit','','2013-07-04 00:00:00',287.92,8064,8067,.08,'Medium' from dual union all
select 39189,'039189','2013-031B','O3b FM4','O3b 4',69494,'In Earth orbit','','2013-07-04 00:00:00',286.63,7981,8064,.08,'Medium' from dual union all
select 3919,'003919','1969-040C','KDU','KDU',18307,'Reentered','1969-05-08 00:00:00','1969-05-05 00:00:00',88.92,194,248,51.59,'Intermediate' from dual union all
select 39190,'039190','2013-031C','O3b FM2','O3b 2',69494,'In Earth orbit','','2013-07-04 00:00:00',288.01,8063,8074,.08,'Medium' from dual union all
select 39191,'039191','2013-031D','O3b PFM','O3b 1/PFM',69494,'In Earth orbit','','2013-07-03 00:00:00',286.89,7991,8070,.03,'Medium' from dual union all
select 39192,'039192','2013-031E','Fregat-MT No. 1041','Fregat-MT    No. 1041',69494,'In Earth orbit','','2013-07-04 00:00:00',276.38,7662,7692,.09,'Medium' from dual union all
select 39193,'039193','2013-029H','Shenzhou 10 Guidao Cang','shenzhou shi hao guidao',69492,'Reentered','2013-12-01 00:00:00','2013-06-26 00:00:00',91.22,332,337,42.79,'Intermediate' from dual union all
select 39194,'039194','2013-032A','Kosmos-2487','Kondor No. 202',69495,'In Earth orbit','','2013-07-04 00:00:00',94.58,497,500,74.73,'Intermediate' from dual union all
select 39195,'039195','2013-032B','Strela APB','Strela APB No. 1832',69495,'In Earth orbit','','2013-07-05 00:00:00',94.56,495,500,74.73,'Intermediate' from dual union all
select 39196,'039196','1998-067CX','EVA debris','',66503,'Reentered','2013-09-03 00:00:00','2013-06-27 00:00:00',92.83,408,418,51.65,'Intermediate' from dual union all
select 39197,'039197','2013-033A','IRIS','IRIS',69496,'In Earth orbit','','2013-07-05 00:00:00',97.55,621,662,97.9,'Sun-Synch' from dual union all
select 39198,'039198','2013-033B','Orion 38','Orion 38',69496,'In Earth orbit','','2013-07-05 00:00:00',97.47,621,655,97.91,'Sun-Synch' from dual union all
select 39199,'039199','2013-034A','IRNSS-R1A','IRNSS-1A',69497,'In Earth orbit','','2013-07-08 00:00:00',1430.45,35482,35870,27.02,'Near-synch' from dual union all
select 392,'000392','62 A Phi 4','deb Sputnik','',8150,'Reentered','1962-09-13 00:00:00','1962-09-13 00:00:00',88.13,162,201,64.83,'Intermediate' from dual union all
select 3920,'003920','1969-040D','Blenda','BZA',18307,'Reentered','1969-05-08 00:00:00','1969-05-06 00:00:00',88.37,189,198,51.58,'Intermediate' from dual union all
select 39200,'039200','2013-034B','PSLV-C22 PS4','PS4',69497,'In Earth orbit','','2013-08-02 00:00:00',366.63,280,20901,17.81,'Highly Ellip' from dual union all
select 39201,'039201','1998-017E','deb SPOT-4','',66453,'Reentered','2014-09-17 00:00:00','2013-07-07 00:00:00',99.1,707,725,98.51,'Sun-Synch' from dual union all
select 39202,'039202','2013-035A','Shi Jian 11-05','shijian shi yihao 05 xin',69498,'In Earth orbit','','2013-07-17 00:00:00',99.04,677,749,98.14,'Sun-Synch' from dual union all
select 39203,'039203','2013-035B','CZ-2C Stage 2','CZ-2C Stage 2',69498,'In Earth orbit','','2013-07-17 00:00:00',98.68,688,704,98.1,'Sun-Synch' from dual union all
select 39204,'039204','2013-035C','CZ-2C Stage 2 motor cover','',69498,'In Earth orbit','','2013-07-17 00:00:00',99.41,653,808,98.29,'Sun-Synch' from dual union all
select 39205,'039205','2013-035D','CZ-2C Stage 2 motor cover','',69498,'In Earth orbit','','2013-07-17 00:00:00',99.61,658,822,97.89,'Sun-Synch' from dual union all
select 39206,'039206','2013-036A','MUOS 2','MUOS SV 2',69499,'In Earth orbit','','2013-07-19 00:00:00',702.15,3800,35781,19.1,'GEO Transfer' from dual union all
select 39207,'039207','2013-036B','Centaur AV-040','Centaur AV-040',69499,'In Earth orbit','','2013-07-19 00:00:00',702.25,3800,35786,19.1,'GEO Transfer' from dual union all
select 39208,'039208','2013-037A','Shiyan Weixing 7','shiyan weixing qihao',69500,'In Earth orbit','','2013-07-20 00:00:00',98.11,664,673,98.06,'Sun-Synch' from dual union all
select 39209,'039209','2013-037B','Chuangxin-3','chuangxin sanhao',69500,'In Earth orbit','','2013-07-20 00:00:00',98.13,666,672,98.06,'Sun-Synch' from dual union all
select 3921,'003921','1969-029AH','deb Blok-E','',18296,'Reentered','1971-07-19 00:00:00','1971-07-14 00:00:00',89.99,269,278,81.31,'Intermediate' from dual union all
select 39210,'039210','2013-037C','Shi Jian 15','shijian shiwu hao',69500,'In Earth orbit','','2013-07-20 00:00:00',98.07,660,673,98.06,'Sun-Synch' from dual union all
select 39211,'039211','2013-037D','CZ-4C Stage 3','CZ-4C Stage 3',69500,'In Earth orbit','','2013-07-20 00:00:00',96.15,478,670,98.1,'Sun-Synch' from dual union all
select 39212,'039212','2013-037E','SJ-15 adapter','',69500,'In Earth orbit','','2013-07-20 00:00:00',98.1,663,672,98.06,'Sun-Synch' from dual union all
select 39213,'039213','2013-037F','SJ-15 adapter','',69500,'In Earth orbit','','2013-07-20 00:00:00',98.02,655,673,98.05,'Sun-Synch' from dual union all
select 39214,'039214','2013-037G','deb SJ-15','',69500,'In Earth orbit','','2013-07-20 00:00:00',98.17,663,680,98.03,'Sun-Synch' from dual union all
select 39215,'039215','2013-038A','Inmarsat 4A F4','Alphasat',69501,'In Earth orbit','','2013-08-03 00:00:00',1434.45,35703,35806,.02,'Drift GEO' from dual union all
select 39216,'039216','2013-038B','Insat 3D','Insat 3D',69501,'In Earth orbit','','2013-08-02 00:00:00',1428.25,35470,35796,.26,'Drift GEO' from dual union all
select 39217,'039217','2013-038C','ESC-A 569','Ariane 5 ESC-A',69501,'In Earth orbit','','2013-08-02 00:00:00',632.54,232,35830,3.48,'GEO Transfer' from dual union all
select 39218,'039218','2013-038D','Sylda 5','Sylda 5',69501,'In Earth orbit','','2013-08-02 00:00:00',634.03,228,35911,3.07,'GEO Transfer' from dual union all
select 39219,'039219','2013-039A','Progress M-20M','Progress 7K-TGM No.420',69502,'Deorbited','2014-02-11 00:00:00','2013-07-30 00:00:00',92.88,413,418,51.65,'Intermediate' from dual union all
select 3922,'003922','1963-047M','Centaur AC-2 part','Insulation blanket part',9278,'Reentered','1991-04-10 00:00:00','1969-05-15 00:00:00',107.3,607,1597,29.93,'Intermediate' from dual union all
select 39220,'039220','2013-039B','11S510','11S510',69502,'Reentered','2013-07-30 00:00:00','2013-07-29 00:00:00',88.18,174,195,51.63,'Intermediate' from dual union all
select 39221,'039221','2013-040A','Kounotori 4 gouki','HTV 4',69503,'Deorbited','2013-09-07 00:00:00','2013-08-05 00:00:00',90.34,281,302,51.64,'Intermediate' from dual union all
select 39222,'039222','2013-041A','WGS SV-6','Wideband Global Satcom 6',69504,'In Earth orbit','','2013-09-01 00:00:00',1436.78,35750,35850,.01,'Drift GEO' from dual union all
select 39223,'039223','2013-041B','Delta 363','DCSS-5 F11',69504,'Reentered','2017-07-11 00:00:00','2013-08-08 00:00:00',1329.3,439,66900,24,'Highly Ellip' from dual union all
select 39224,'039224','2013-035E','CZ-2C Stage 2 motor cover','',69498,'In Earth orbit','','2013-09-05 00:00:00',101.41,691,960,97.91,'Sun-Synch' from dual union all
select 39225,'039225','2013-035F','CZ-2C Stage 2 motor cover','',69498,'In Earth orbit','','2013-09-05 00:00:00',101.47,690,967,98.33,'Sun-Synch' from dual union all
select 39226,'039226','2013-030C','deb Resurs-P No. 1','deb Resurs-P No. 1',69493,'Reentered','2016-04-01 00:00:00','2013-09-01 00:00:00',93.88,455,473,97.28,'Sun-Synch' from dual union all
select 39227,'039227','2013-042A','Arirang-5','KOMPSAT-5',69505,'In Earth orbit','','2013-08-24 00:00:00',95.51,535,552,97.6,'Sun-Synch' from dual union all
select 39228,'039228','2013-042B','R-36M2 DS','R-36M2 DS',69505,'In Earth orbit','','2013-08-25 00:00:00',102.45,540,1210,97.5,'Sun-Synch' from dual union all
select 39229,'039229','2013-042C','Gazodinamicheskiy Ekran','GDS (Gas dynamic shield)',69505,'Reentered','2015-10-09 00:00:00','2013-08-25 00:00:00',95.23,512,548,97.6,'Sun-Synch' from dual union all
select 3923,'003923','1968-091BR','deb Kosmos-249','',16818,'In Earth orbit','','1992-02-03 00:00:00',110.22,659,1815,62.23,'Intermediate' from dual union all
select 39230,'039230','1998-067CY','WAL-6 antenna cover','ISS WAL-6 cover',66503,'Reentered','2013-10-25 00:00:00','2013-08-26 00:00:00',92.78,409,413,51.65,'Intermediate' from dual union all
select 39231,'039231','2013-037H','deb SJ-15','',69500,'In Earth orbit','','2013-08-25 00:00:00',102.21,672,1055,98.08,'Sun-Synch' from dual union all
select 39232,'039232','2013-043A','USA 245','Enhanced CRYSTAL 2110',69506,'In Earth orbit','','2013-08-29 00:00:00',97.23,254,999,97.88,'Sun-Synch' from dual union all
select 39233,'039233','2013-044A','Es''hail','EUTELSAT 25B',69507,'In Earth orbit','','2013-09-01 00:00:00',631.7,245,35774,3.45,'GEO Transfer' from dual union all
select 39234,'039234','2013-044B','GSAT-7','INAV 1',69507,'In Earth orbit','','2013-09-03 00:00:00',1428.35,35471,35799,.22,'Drift GEO' from dual union all
select 39235,'039235','2013-044C','ESC-A 570','Ariane 5 ESC-A',69507,'In Earth orbit','','2013-09-01 00:00:00',632.56,245,35818,3.69,'GEO Transfer' from dual union all
select 39236,'039236','2013-044D','Sylda 5','Sylda 5 No. 55D',69507,'In Earth orbit','','2013-09-01 00:00:00',965.48,16198,35763,.73,'Medium' from dual union all
select 39237,'039237','2013-045A','Amos 4','Amos 4',69508,'In Earth orbit','','2013-09-03 00:00:00',683.54,2860,35792,26.87,'GEO Transfer' from dual union all
select 39238,'039238','2013-045B','Blok DM-SLB No. 5L','Blok DM-SLB',69508,'In Earth orbit','','2013-09-03 00:00:00',683.54,2860,35792,26.87,'GEO Transfer' from dual union all
select 39239,'039239','2013-046A','Yaogan Weixing 17','Jianbing-8 03 xing',69509,'In Earth orbit','','2013-09-03 00:00:00',107.11,1076,1109,63.41,'Intermediate' from dual union all
select 3924,'003924','1969-021U','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1969-12-30 00:00:00','1969-05-11 00:00:00',95.95,481,647,72.77,'Intermediate' from dual union all
select 39240,'039240','2013-046B','Yaogan Weixing 17 fu xing y1','Jianbing-8 03 subsat 1',69509,'In Earth orbit','','2013-09-03 00:00:00',107.12,1076,1111,63.41,'Intermediate' from dual union all
select 39241,'039241','2013-046C','Yaogan Weixing 17 fu xing er','Jianbing-8 03 subsat 2',69509,'In Earth orbit','','2013-09-03 00:00:00',107.14,1079,1110,63.41,'Intermediate' from dual union all
select 39242,'039242','2013-046D','CZ-4C Stage 3','CZ-4C Stage 3',69509,'In Earth orbit','','2013-09-03 00:00:00',105.12,890,1110,63.47,'Intermediate' from dual union all
select 39243,'039243','2013-046E','deb CZ-4C','',69509,'In Earth orbit','','2013-09-03 00:00:00',107.13,1078,1110,63.41,'Intermediate' from dual union all
select 39244,'039244','2013-046F','deb CZ-4C','',69509,'In Earth orbit','','2013-09-03 00:00:00',106.79,1059,1096,63.41,'Intermediate' from dual union all
select 39245,'039245','1998-067CZ','EVA debris','',66503,'Reentered','2013-10-23 00:00:00','2013-09-05 00:00:00',92.62,400,406,51.65,'Intermediate' from dual union all
select 39246,'039246','2013-047A','LADEE','LADEE',69510,'Deep Space','2013-09-07 00:00:00','2013-09-07 00:00:00',9044.36,200,274635,37.57,'Deep Space' from dual union all
select 39247,'039247','1967-053L','deb NRL PL153','deb POPPY 5',15308,'Reentered','2014-12-22 00:00:00','2013-10-15 00:00:00',99.24,713,732,69.96,'Intermediate' from dual union all
select 39248,'039248','2013-047B','Minotaur Stage 4','Star 48V',69510,'Reentered','2013-11-27 00:00:00','2013-09-07 00:00:00',95,206,832,37.6,'Intermediate' from dual;
insert into SATELLITE
select 39249,'039249','2013-048A','Gonets-M No. 14','Gonets-M No. 14',69511,'In Earth orbit','','2013-09-14 00:00:00',116.03,1495,1510,82.49,'Intermediate' from dual union all
select 3925,'003925','1965-082RY','deb Transtage 4','',11832,'Reentered','1980-09-30 00:00:00','1980-09-18 00:00:00',94.3,407,563,28.45,'Intermediate' from dual union all
select 39250,'039250','2013-048B','Gonets-M No. 16','Gonets-M No. 16',69511,'In Earth orbit','','2013-09-14 00:00:00',115.98,1493,1506,82.49,'Intermediate' from dual union all
select 39251,'039251','2013-048C','Gonets-M No. 17','Gonets-M No. 17',69511,'In Earth orbit','','2013-09-14 00:00:00',116.01,1494,1509,82.49,'Intermediate' from dual union all
select 39252,'039252','2013-048D','Briz-KM No. 72519','Briz-KM 14S45',69511,'In Earth orbit','','2013-09-14 00:00:00',112.5,1179,1504,82.49,'Intermediate' from dual union all
select 39253,'039253','2013-049A','Hisaki','SPRINT-A',69512,'In Earth orbit','','2013-09-16 00:00:00',102.78,808,973,29.75,'Intermediate' from dual union all
select 39254,'039254','2013-049B','Epsilon PBS','Post-Boost Stage',69512,'In Earth orbit','','2013-09-17 00:00:00',104.71,847,1114,29.7,'Intermediate' from dual union all
select 39255,'039255','2013-049C','Epsilon Stage 3','KM-V2b',69512,'In Earth orbit','','2013-09-16 00:00:00',106.26,952,1154,29.72,'Intermediate' from dual union all
select 39256,'039256','2013-050A','AEHF SV-3','AEHF SV-3',69513,'In Earth orbit','','2013-09-19 00:00:00',927.86,217,50052,20.59,'Highly Ellip' from dual union all
select 39257,'039257','2013-050B','Centaur AV-041','Centaur AV-041',69513,'Reentered','2015-02-08 00:00:00','2013-09-19 00:00:00',888.44,150,48321,20.26,'Highly Ellip' from dual union all
select 39258,'039258','2013-051A','SS G. David Low','Cygnus PCM-0',69514,'Deorbited','2013-10-23 00:00:00','2013-09-20 00:00:00',92.58,387,415,51.63,'Intermediate' from dual union all
select 39259,'039259','2013-051B','Antares Stage 2','Castor 30',69514,'Reentered','2013-10-09 00:00:00','2013-09-20 00:00:00',89.78,258,270,51.61,'Intermediate' from dual union all
select 3926,'003926','1965-082RZ','deb Transtage 4','',11832,'Reentered','1979-02-26 00:00:00','1969-05-07 00:00:00',106.59,726,1412,32.24,'Intermediate' from dual union all
select 39260,'039260','2013-052A','Feng Yun 3C','di san ke fengyun sanhao',69515,'In Earth orbit','','2013-09-24 00:00:00',101.07,804,815,98.8,'Sun-Synch' from dual union all
select 39261,'039261','2013-052B','CZ-4C Stage 3','CZ-4C Stage 3',69515,'In Earth orbit','','2013-09-24 00:00:00',100.49,758,805,98.83,'Sun-Synch' from dual union all
select 39262,'039262','2013-053A','Kuaizhou-1','kuai zhou yi hao',69516,'Reentered Attached','2015-09-01 00:00:00','2013-09-25 00:00:00',90.18,274,293,96.65,'Sun-Synch' from dual union all
select 39263,'039263','2013-054A','Soyuz TMA-10M','Soyuz 11F732A47 No. 710',69517,'Landed','2014-03-11 00:00:00','2013-09-25 00:00:00',88.68,195,224,51.67,'Intermediate' from dual union all
select 39264,'039264','2013-054B','11S510','11S510',69517,'Reentered','2013-09-28 00:00:00','2013-09-25 00:00:00',88.57,191,217,51.68,'Intermediate' from dual union all
select 39265,'039265','2013-055A','CASSIOPE','CASSIOPE',69518,'In Earth orbit','','2013-10-07 00:00:00',103.1,325,1486,81,'Intermediate' from dual union all
select 39266,'039266','2013-055B','CUSat','CUSat Bottom',69518,'In Earth orbit','','2013-10-07 00:00:00',103.07,325,1483,80.99,'Intermediate' from dual union all
select 39267,'039267','2013-055C','DANDE','',69518,'In Earth orbit','','2013-10-07 00:00:00',103.09,325,1485,80.99,'Intermediate' from dual union all
select 39268,'039268','2013-055D','POPACS 1','POPACS 1',69518,'In Earth orbit','','2013-10-07 00:00:00',102.98,324,1476,81,'Intermediate' from dual union all
select 39269,'039269','2013-055E','POPACS 2','POPACS 2',69518,'In Earth orbit','','2013-10-07 00:00:00',102.98,324,1476,81,'Intermediate' from dual union all
select 3927,'003927','61 Delta 6','deb Explorer IX','',7263,'In Earth orbit','','1969-05-18 00:00:00',117.61,633,2514,38.87,'Intermediate' from dual union all
select 39270,'039270','2013-055F','POPACS 3','POPACS 3',69518,'In Earth orbit','','2013-10-10 00:00:00',103.04,324,1480,81,'Intermediate' from dual union all
select 39271,'039271','2013-055G','Falcon 9-006 Stage 2','F9-006 St2',69518,'In Earth orbit','','2013-10-07 00:00:00',103.06,319,1488,81.01,'Intermediate' from dual union all
select 39272,'039272','2013-055H','deb Falcon 9 insulation','',69518,'Reentered','2013-10-16 00:00:00','2013-10-06 00:00:00',97.53,262,1019,81.04,'Intermediate' from dual union all
select 39273,'039273','2013-055J','deb Falcon 9 insulation','',69518,'Reentered','2013-10-06 00:00:00','2013-10-01 00:00:00',100.86,297,1301,80.95,'Intermediate' from dual union all
select 39274,'039274','2013-055K','deb Falcon 9 insulation','',69518,'Reentered','2013-11-05 00:00:00','2013-10-06 00:00:00',102.56,422,1338,80.94,'Intermediate' from dual union all
select 39275,'039275','2013-055L','deb Falcon 9 insulation','',69518,'Reentered','2013-10-08 00:00:00','2013-10-05 00:00:00',95.16,405,648,80.96,'Intermediate' from dual union all
select 39276,'039276','2013-055M','deb Falcon 9 insulation','',69518,'Reentered','2013-10-22 00:00:00','2013-10-07 00:00:00',101.91,406,1293,80.95,'Intermediate' from dual union all
select 39277,'039277','2013-055N','deb Falcon 9 insulation','',69518,'Reentered','2013-11-14 00:00:00','2013-10-06 00:00:00',104.54,456,1489,80.99,'Intermediate' from dual union all
select 39278,'039278','2013-055P','deb Falcon 9 insulation','',69518,'Reentered','2013-11-14 00:00:00','2013-10-06 00:00:00',104.25,453,1466,80.96,'Intermediate' from dual union all
select 39279,'039279','2013-055Q','deb Falcon 9 insulation','',69518,'Reentered','2013-10-30 00:00:00','2013-10-07 00:00:00',102.54,425,1333,80.96,'Intermediate' from dual union all
select 3928,'003928','1965-082SA','deb Transtage 4','',11832,'Reentered','1983-04-17 00:00:00','1969-05-30 00:00:00',99.08,652,778,32.19,'Intermediate' from dual union all
select 39280,'039280','2013-055R','POPACS Top Spacer','POPACS Top Spacer',69518,'Reentered','2015-02-15 00:00:00','2013-10-11 00:00:00',102.96,325,1473,81.01,'Intermediate' from dual union all
select 39281,'039281','2013-055S','deb Falcon 9 insulation','',69518,'Reentered','2013-10-11 00:00:00','2013-10-07 00:00:00',98.95,362,1055,80.87,'Intermediate' from dual union all
select 39282,'039282','2013-055T','deb Falcon 9 insulation','',69518,'Reentered','2013-10-09 00:00:00','2013-10-02 00:00:00',102.26,350,1382,80.91,'Intermediate' from dual union all
select 39283,'039283','2013-055U','deb Falcon 9 insulation','',69518,'Reentered','2013-10-11 00:00:00','2013-10-07 00:00:00',98.01,499,828,81.17,'Intermediate' from dual union all
select 39284,'039284','2013-055V','deb Falcon 9 insulation','',69518,'Reentered','2013-10-14 00:00:00','2013-10-01 00:00:00',103.7,392,1475,80.99,'Intermediate' from dual union all
select 39285,'039285','2013-056A','Astra 2E','Astra 2E',69519,'In Earth orbit','','2013-10-07 00:00:00',1431.72,35672,35730,.14,'Drift GEO' from dual union all
select 39286,'039286','2013-056B','Briz-M No. 99540','Briz-M 14S43 No. 99540',69519,'In Earth orbit','','2013-10-07 00:00:00',660.02,3532,33933,23.51,'GEO Transfer' from dual union all
select 39287,'039287','2013-056C','Briz-M DTB','Briz-M torus tank',69519,'In Earth orbit','','2013-10-07 00:00:00',278.3,343,15141,49.59,'Highly Ellip' from dual union all
select 39288,'039288','2013-055W','deb Falcon 9 insulation','',69518,'Reentered','2013-11-01 00:00:00','2013-10-11 00:00:00',102.99,403,1398,80.94,'Intermediate' from dual union all
select 39289,'039289','2013-055X','deb Falcon 9 insulation','',69518,'Reentered','2013-10-13 00:00:00','2013-10-12 00:00:00',92.92,302,534,80.99,'Intermediate' from dual union all
select 3929,'003929','1965-082SB','deb Transtage 4','',11832,'Reentered','1970-12-27 00:00:00','1970-12-13 00:00:00',92.79,402,421,32.55,'Intermediate' from dual union all
select 39290,'039290','2013-055Y','POPACS Middle Spacer 1','POPACS Middle Spacer',69518,'Reentered','2015-06-30 00:00:00','2013-10-16 00:00:00',102.93,324,1470,81,'Intermediate' from dual union all
select 39291,'039291','2013-055Z','deb Falcon 9 insulation','',69518,'Reentered','2014-05-18 00:00:00','2013-10-14 00:00:00',102.97,324,1474,80.99,'Intermediate' from dual union all
select 39292,'039292','2013-055AA','POPACS Middle Spacer 2','POPACS Middle Spacer',69518,'Reentered','2015-03-30 00:00:00','2013-10-15 00:00:00',102.91,323,1470,81,'Intermediate' from dual union all
select 39293,'039293','2013-055AB','POPACS Bottom Spacer','POPACS Bottom Spacer',69518,'Reentered','2015-06-24 00:00:00','2013-10-15 00:00:00',102.93,324,1470,81,'Intermediate' from dual union all
select 39294,'039294','1966-040BC','Nimbus 2 debris','',13447,'In Earth orbit','','2013-10-15 00:00:00',107.92,1092,1169,100.44,'Sun-Synch' from dual union all
select 39295,'039295','1966-040BD','Nimbus 2 debris','',13447,'In Earth orbit','','2013-10-14 00:00:00',107.86,1089,1166,100.49,'Sun-Synch' from dual union all
select 39296,'039296','1968-081AG','Transtage 5 debris','',16808,'In Earth orbit','','2013-10-21 00:00:00',1431.36,34966,36422,8.4,'Inclined Drift' from dual union all
select 39297,'039297','1968-081AH','Transtage 5 debris','',16808,'In Earth orbit','','2013-10-22 00:00:00',1437.08,34726,36886,8.53,'Inclined Drift' from dual union all
select 39298,'039298','1968-081AJ','Transtage 5 debris','',16808,'In Earth orbit','','2013-10-16 00:00:00',1438.16,35086,36568,8.5,'Inclined Drift' from dual union all
select 39299,'039299','1968-081AK','Transtage 5 debris','',16808,'In Earth orbit','','2013-04-12 00:00:00',1434.4,35250,36257,8.72,'Inclined Drift' from dual union all
select 393,'000393','62 A Phi 5','deb Sputnik','',8150,'Reentered','1962-09-13 00:00:00','1962-09-13 00:00:00',87.91,162,181,64.83,'Intermediate' from dual union all
select 3930,'003930','1965-082SC','deb Transtage 4','',11832,'Reentered','1982-11-19 00:00:00','1969-05-07 00:00:00',111.18,849,1712,32.9,'Intermediate' from dual union all
select 39300,'039300','1992-093JR','deb 11S772','',64805,'In Earth orbit','','2013-10-14 00:00:00',103.22,833,989,71.12,'Intermediate' from dual union all
select 39301,'039301','1992-093JS','deb 11S772','',64805,'In Earth orbit','','2013-10-10 00:00:00',102.1,831,886,70.87,'Intermediate' from dual union all
select 39302,'039302','1992-093JT','deb 11S772','',64805,'In Earth orbit','','2013-09-24 00:00:00',102.59,765,997,70.97,'Intermediate' from dual union all
select 39303,'039303','1992-093JU','deb 11S772','',64805,'In Earth orbit','','2013-10-12 00:00:00',102.08,809,906,70.92,'Intermediate' from dual union all
select 39304,'039304','1992-093JV','deb 11S772','',64805,'In Earth orbit','','2013-10-13 00:00:00',102.89,828,963,71.21,'Intermediate' from dual union all
select 39305,'039305','1992-093JW','deb 11S772','',64805,'In Earth orbit','','2013-10-15 00:00:00',102.85,826,961,70.96,'Intermediate' from dual union all
select 39306,'039306','1994-029AER','deb HAPS','',65425,'In Earth orbit','','2013-10-09 00:00:00',102.17,572,1151,81.79,'Intermediate' from dual union all
select 39307,'039307','1994-029AES','deb HAPS','',65425,'In Earth orbit','','2013-10-14 00:00:00',99.53,537,936,81.96,'Intermediate' from dual union all
select 39308,'039308','1994-029AET','deb HAPS','',65425,'In Earth orbit','','2013-10-15 00:00:00',103.29,557,1272,81.97,'Intermediate' from dual union all
select 39309,'039309','1994-029AEU','deb HAPS','',65425,'In Earth orbit','','2013-10-14 00:00:00',100.69,585,998,81.75,'Intermediate' from dual union all
select 3931,'003931','1965-020EV','deb Blok Kosmos','[UNID]',11770,'In Earth orbit','','1992-01-18 00:00:00',116.6,1358,1697,56.07,'Intermediate' from dual union all
select 39310,'039310','1994-029AEV','deb HAPS','',65425,'In Earth orbit','','2013-10-14 00:00:00',100.94,577,1030,81.83,'Intermediate' from dual union all
select 39311,'039311','1994-029AEW','deb HAPS','',65425,'In Earth orbit','','2013-10-14 00:00:00',100.29,575,970,81.44,'Intermediate' from dual union all
select 39312,'039312','1994-029AEX','deb HAPS','',65425,'In Earth orbit','','2013-10-14 00:00:00',99.94,554,957,81.99,'Intermediate' from dual union all
select 39313,'039313','1994-029AEY','deb HAPS','',65425,'In Earth orbit','','2013-10-14 00:00:00',98.26,554,797,81.8,'Intermediate' from dual union all
select 39314,'039314','1994-029AEZ','deb HAPS','',65425,'In Earth orbit','','2013-10-14 00:00:00',99.58,572,905,81.66,'Intermediate' from dual union all
select 39315,'039315','1994-029AFA','deb HAPS','',65425,'In Earth orbit','','2013-10-14 00:00:00',103.77,594,1280,81.47,'Intermediate' from dual union all
select 39316,'039316','1994-029AFB','deb HAPS','',65425,'In Earth orbit','','2013-10-15 00:00:00',96.95,542,684,82.02,'Intermediate' from dual union all
select 39317,'039317','1994-029AFC','deb HAPS','',65425,'In Earth orbit','','2013-10-14 00:00:00',96.71,534,669,82,'Intermediate' from dual union all
select 39318,'039318','1994-029AFD','deb HAPS','',65425,'In Earth orbit','','2013-10-15 00:00:00',98.91,558,856,81.95,'Intermediate' from dual union all
select 39319,'039319','1994-029AFE','deb HAPS','',65425,'In Earth orbit','','2013-10-15 00:00:00',98.81,554,850,81.95,'Intermediate' from dual union all
select 3932,'003932','61 Omicron 223','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-02 00:00:00',109.17,959,1418,65.8,'Intermediate' from dual union all
select 39320,'039320','1994-029AFF','deb HAPS','',65425,'In Earth orbit','','2013-10-15 00:00:00',101.48,593,1065,81.76,'Intermediate' from dual union all
select 39321,'039321','1994-029AFG','deb HAPS','',65425,'In Earth orbit','','2013-10-15 00:00:00',103.95,605,1286,82.29,'Intermediate' from dual union all
select 39322,'039322','1994-029AFH','deb HAPS','',65425,'In Earth orbit','','2013-10-14 00:00:00',100.66,573,1007,81.94,'Intermediate' from dual union all
select 39323,'039323','1994-029AFJ','deb HAPS','',65425,'In Earth orbit','','2013-10-15 00:00:00',99.79,572,925,81.99,'Intermediate' from dual union all
select 39324,'039324','2008-011DB','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-10-22 00:00:00',479.72,638,27205,47.37,'Highly Ellip' from dual union all
select 39325,'039325','2008-011DC','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-10-12 00:00:00',476.53,1067,26596,49.41,'Highly Ellip' from dual union all
select 39326,'039326','2008-011DD','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-10-22 00:00:00',498.31,2351,26534,48.6,'Highly Ellip' from dual union all
select 39327,'039327','2008-011DE','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-10-14 00:00:00',488.27,741,27583,49.36,'Highly Ellip' from dual union all
select 39328,'039328','2008-011DF','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-09-11 00:00:00',486.6,908,27322,48.66,'Highly Ellip' from dual union all
select 39329,'039329','2008-011DG','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-10-16 00:00:00',469.58,714,26555,48.78,'Highly Ellip' from dual union all
select 3933,'003933','1969-029AJ','deb Blok-E','',18296,'Reentered','1970-08-06 00:00:00','1970-08-03 00:00:00',90.23,261,310,81.28,'Intermediate' from dual union all
select 39330,'039330','2008-011DH','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-10-06 00:00:00',478.6,760,27020,49.58,'Highly Ellip' from dual union all
select 39331,'039331','2008-011DJ','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-10-06 00:00:00',463.35,397,26517,48.94,'Highly Ellip' from dual union all
select 39332,'039332','2010-042D','deb CZ-3B','',68902,'In Earth orbit','','2013-10-21 00:00:00',751.51,414,41592,23.7,'Highly Ellip' from dual union all
select 39333,'039333','2010-042E','deb CZ-3B','',68902,'In Earth orbit','','2013-10-20 00:00:00',756.85,580,41685,24.03,'Highly Ellip' from dual union all
select 39334,'039334','2010-042F','deb CZ-3B','',68902,'In Earth orbit','','2013-10-22 00:00:00',772.69,1043,41987,26.59,'Highly Ellip' from dual union all
select 39335,'039335','2010-042G','deb CZ-3B','',68902,'In Earth orbit','','2013-10-17 00:00:00',730.71,688,40303,27.89,'GEO Transfer' from dual union all
select 39336,'039336','2010-042H','deb CZ-3B','',68902,'In Earth orbit','','2013-10-19 00:00:00',751.2,429,41562,24.35,'Highly Ellip' from dual union all
select 39337,'039337','2010-042J','deb CZ-3B','',68902,'In Earth orbit','','2013-10-21 00:00:00',787.61,622,43124,25.34,'Highly Ellip' from dual union all
select 39338,'039338','2010-057C','deb CZ-3C','',68917,'In Earth orbit','','2013-10-21 00:00:00',640.54,524,35949,19.93,'GEO Transfer' from dual;
insert into SATELLITE
select 39339,'039339','2010-057D','deb CZ-3C','',68917,'In Earth orbit','','2013-10-21 00:00:00',671.66,816,37239,21.11,'GEO Transfer' from dual union all
select 3934,'003934','1968-091BS','deb Kosmos-249','',16818,'Reentered','1977-08-24 00:00:00','1973-08-11 00:00:00',115.87,783,2207,62.31,'Intermediate' from dual union all
select 39340,'039340','2010-057E','deb CZ-3C','',68917,'In Earth orbit','','2013-10-21 00:00:00',639.88,509,35930,19.57,'GEO Transfer' from dual union all
select 39341,'039341','2010-057F','deb CZ-3C','',68917,'In Earth orbit','','2013-10-21 00:00:00',614.22,381,34734,18.85,'GEO Transfer' from dual union all
select 39342,'039342','2010-057G','deb CZ-3C','',68917,'In Earth orbit','','2013-10-18 00:00:00',652.97,626,36481,20.3,'GEO Transfer' from dual union all
select 39343,'039343','2010-057H','deb CZ-3C','',68917,'In Earth orbit','','2013-10-14 00:00:00',640.91,652,35840,22.13,'GEO Transfer' from dual union all
select 39344,'039344','2010-057J','deb CZ-3C','',68917,'In Earth orbit','','2013-10-20 00:00:00',608.67,349,34477,20.31,'GEO Transfer' from dual union all
select 39345,'039345','2010-057K','deb CZ-3C','',68917,'In Earth orbit','','2013-10-20 00:00:00',608.88,314,34523,20.07,'GEO Transfer' from dual union all
select 39346,'039346','2010-057L','deb CZ-3C','',68917,'In Earth orbit','','2013-10-21 00:00:00',661.1,698,36823,20.71,'GEO Transfer' from dual union all
select 39347,'039347','2010-057M','deb CZ-3C','',68917,'In Earth orbit','','2013-10-19 00:00:00',603.24,343,34200,20,'GEO Transfer' from dual union all
select 39348,'039348','2010-057N','deb CZ-3C','',68917,'In Earth orbit','','2013-10-21 00:00:00',640.5,432,36039,20.3,'GEO Transfer' from dual union all
select 39349,'039349','2010-057P','deb CZ-3C','',68917,'In Earth orbit','','2013-10-19 00:00:00',631.1,564,35424,20.44,'GEO Transfer' from dual union all
select 3935,'003935','1967-014H','deb Diademe 2','',15269,'Reentered','1981-08-02 00:00:00','1967-07-01 00:00:00',110.12,593,1872,39.45,'Intermediate' from dual union all
select 39350,'039350','2010-057Q','deb CZ-3C','',68917,'In Earth orbit','','2013-10-20 00:00:00',647.79,544,36300,20.12,'GEO Transfer' from dual union all
select 39351,'039351','2010-057R','deb CZ-3C','',68917,'In Earth orbit','','2013-10-19 00:00:00',636.33,761,35496,20.84,'GEO Transfer' from dual union all
select 39352,'039352','2010-057S','deb CZ-3C','',68917,'In Earth orbit','','2013-10-18 00:00:00',660.98,675,36840,20.93,'GEO Transfer' from dual union all
select 39353,'039353','2010-057T','deb CZ-3C','',68917,'In Earth orbit','','2013-10-21 00:00:00',643.39,659,35960,20.17,'GEO Transfer' from dual union all
select 39354,'039354','2010-057U','deb CZ-3C','',68917,'In Earth orbit','','2013-10-22 00:00:00',649.99,638,36318,20.43,'GEO Transfer' from dual union all
select 39355,'039355','2010-057V','deb CZ-3C','',68917,'In Earth orbit','','2013-10-15 00:00:00',633.33,556,35547,20.3,'GEO Transfer' from dual union all
select 39356,'039356','2010-057W','deb CZ-3C','',68917,'In Earth orbit','','2013-10-22 00:00:00',639.1,471,35928,20.45,'GEO Transfer' from dual union all
select 39357,'039357','2013-037J','Shiyan 7B','Tansuo 4B',69500,'In Earth orbit','','2013-10-19 00:00:00',98.08,660,673,98.05,'Sun-Synch' from dual union all
select 39358,'039358','2013-057A','Shi Jian 16','shijian shiliu hao',69520,'In Earth orbit','','2013-10-27 00:00:00',96.85,600,616,74.98,'Intermediate' from dual union all
select 39359,'039359','2013-057B','CZ-4B Stage 3','CZ-4B Stage 3',69520,'In Earth orbit','','2013-10-27 00:00:00',95.42,475,602,75.02,'Intermediate' from dual union all
select 3936,'003936','1965-082SD','deb Transtage 4','',11832,'Reentered','1970-04-22 00:00:00','1970-04-14 00:00:00',96.15,558,591,30.94,'Intermediate' from dual union all
select 39360,'039360','2013-058A','Sirius FM-6','Sirius CD Radio 6',69521,'In Earth orbit','','2013-11-02 00:00:00',1434.15,35706,35791,.09,'Drift GEO' from dual union all
select 39361,'039361','2013-058B','Briz-M No. 99542','Briz-M 14S43 No. 99542',69521,'In Earth orbit','','2013-11-03 00:00:00',662.39,3390,34196,23.62,'GEO Transfer' from dual union all
select 39362,'039362','2013-058C','Briz-M DTB','Briz-M torus tank',69521,'In Earth orbit','','2013-11-02 00:00:00',277.93,325,15133,49.6,'Highly Ellip' from dual union all
select 39363,'039363','2013-059A','Yaogan Weixing 18','Jianbing-7 03 xing',69522,'In Earth orbit','','2013-10-31 00:00:00',94.87,511,513,97.55,'Sun-Synch' from dual union all
select 39364,'039364','2013-059B','CZ-2C Stage 2','CZ-2C Stage 2',69522,'In Earth orbit','','2013-10-31 00:00:00',95.16,482,571,97.55,'Sun-Synch' from dual union all
select 39365,'039365','2013-059C','Sep motor cover','',69522,'Reentered','2014-11-01 00:00:00','2013-10-31 00:00:00',95.89,478,646,97.73,'Sun-Synch' from dual union all
select 39366,'039366','2013-059D','Sep motor cover','',69522,'Reentered','2014-10-16 00:00:00','2013-10-31 00:00:00',96,479,654,97.36,'Sun-Synch' from dual union all
select 39367,'039367','2013-059E','Sep motor cover','',69522,'Reentered','2013-12-07 00:00:00','2013-10-31 00:00:00',93.26,347,522,97.31,'Sun-Synch' from dual union all
select 39368,'039368','2013-059F','Sep motor cover','',69522,'Reentered','2013-12-04 00:00:00','2013-10-31 00:00:00',93.08,334,517,97.72,'Sun-Synch' from dual union all
select 39369,'039369','2013-055AC','DANDE LAB','Lightband Adapter Bracke',69518,'In Earth orbit','','2013-11-07 00:00:00',103.03,324,1480,80.99,'Intermediate' from dual union all
select 3937,'003937','1969-021V','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1969-08-13 00:00:00','1969-05-09 00:00:00',93.97,458,480,74.07,'Intermediate' from dual union all
select 39370,'039370','2013-060A','Mars Orbiter Mission','Mars Orbiter Spacecraft',69523,'Deep Space','2013-11-16 00:00:00','2013-11-15 00:00:00',5574.13,844,194676,19.42,'Deep Space' from dual union all
select 39371,'039371','2013-060B','PSLV-C25 PS4','PS4',69523,'In Earth orbit','','2013-11-15 00:00:00',415.62,259,23882,19.36,'Highly Ellip' from dual union all
select 39372,'039372','2011-066G','deb CZ-4B','',69118,'Reentered','2015-10-16 00:00:00','2014-06-09 00:00:00',93.16,426,433,97.33,'Sun-Synch' from dual union all
select 39373,'039373','2013-061A','Soyuz TMA-11M','Soyuz 11F732A47 No. 711',69524,'Landed','2014-05-14 00:00:00','2013-11-07 00:00:00',92.9,416,418,51.65,'Intermediate' from dual union all
select 39374,'039374','2013-061B','11S510','11S510',69524,'Reentered','2013-11-09 00:00:00','2013-11-08 00:00:00',88.17,178,190,51.66,'Intermediate' from dual union all
select 39375,'039375','2013-062A','Raduga-1M','Globus-1M No. 13L',69525,'In Earth orbit','','2013-11-20 00:00:00',1426.09,35433,35748,.17,'Drift GEO' from dual union all
select 39376,'039376','2013-062B','Briz-M No. 88532','Briz-M',69525,'In Earth orbit','','2013-11-26 00:00:00',1381.87,33714,35723,.08,'Drift GEO' from dual union all
select 39377,'039377','2013-062C','Briz-M DTB','Briz-M torus tank',69525,'Reentered','2017-04-21 00:00:00','2013-11-25 00:00:00',630.53,414,35545,48.61,'GEO Transfer' from dual union all
select 39378,'039378','2013-063A','MAVEN','MAVEN',69526,'Deep Space','2013-11-19 00:00:00','2013-11-18 00:00:00',976.3,193,78149,27.68,'Earth Escape' from dual union all
select 39379,'039379','2013-063B','Centaur AV-038','Centaur AV-038',69526,'Deep Space','2013-11-19 00:00:00','2013-11-18 00:00:00',976.3,193,78149,27.68,'Earth Escape' from dual union all
select 3938,'003938','1968-097BS','deb Kosmos-252 [*]','',16824,'In Earth orbit','','1992-02-03 00:00:00',117.21,569,2542,63.02,'Intermediate' from dual union all
select 39380,'039380','2013-064A','STPSat-3','STP-SIV 2',69527,'In Earth orbit','','2013-11-22 00:00:00',94.65,498,506,40.52,'Intermediate' from dual union all
select 39381,'039381','2013-064B','PhoneSat 2.4','PhoneSat 2.4',69527,'Reentered','2017-01-31 00:00:00','2013-12-12 00:00:00',94.62,499,502,40.51,'Intermediate' from dual union all
select 39382,'039382','2013-064C','Louisiana-OSCAR-75','CAPE 2',69527,'Reentered','2014-10-23 00:00:00','2013-12-17 00:00:00',94.6,498,500,40.51,'Intermediate' from dual union all
select 39383,'039383','2013-064D','DragonSat','DragonSat-1',69527,'Reentered','2016-08-31 00:00:00','2013-12-12 00:00:00',94.61,498,502,40.53,'Intermediate' from dual union all
select 39384,'039384','2013-064E','KySat-2','KySat-2',69527,'Reentered','2015-02-12 00:00:00','2013-12-17 00:00:00',94.61,496,504,40.53,'Intermediate' from dual union all
select 39385,'039385','2013-064F','TJ3Sat','TJ-Cubesat',69527,'Reentered','2015-09-16 00:00:00','2013-12-12 00:00:00',94.61,496,504,40.53,'Intermediate' from dual union all
select 39386,'039386','2013-064G','ORSES','ORS Enabler Satellite',69527,'Reentered','2016-01-03 00:00:00','2013-11-23 00:00:00',94.65,499,504,40.51,'Intermediate' from dual union all
select 39387,'039387','2013-064H','ORS Tech 1','VECTOR JCTD 1',69527,'Reentered','2015-03-23 00:00:00','2013-11-22 00:00:00',94.66,500,505,40.51,'Intermediate' from dual union all
select 39388,'039388','2013-064J','SENSE SV1','SENSE-A',69527,'Reentered','2015-03-21 00:00:00','2013-12-06 00:00:00',94.62,497,504,40.53,'Intermediate' from dual union all
select 39389,'039389','2013-064K','NPS-SCAT','NPS-SCAT',69527,'Reentered','2015-10-28 00:00:00','2013-12-24 00:00:00',94.59,495,503,40.51,'Intermediate' from dual union all
select 3939,'003939','1969-042A','Kosmos-281','Zenit-2 No. 72',18309,'Landed','1969-05-21 00:00:00','1969-05-15 00:00:00',89.39,188,300,65.42,'Intermediate' from dual union all
select 39390,'039390','2013-064L','Prometheus 1-4','Prometheus 4',69527,'Reentered','2015-12-13 00:00:00','2013-12-12 00:00:00',94.61,497,503,40.51,'Intermediate' from dual union all
select 39391,'039391','2013-064M','Prometheus 1-2','Prometheus 2',69527,'Reentered','2015-12-12 00:00:00','2013-12-12 00:00:00',94.61,497,503,40.51,'Intermediate' from dual union all
select 39392,'039392','2013-064N','SENSE SV2','SENSE-B',69527,'In Earth orbit','','2013-12-14 00:00:00',94.62,499,502,40.51,'Intermediate' from dual union all
select 39393,'039393','2013-064P','Prometheus 1-5','Prometheus 5',69527,'Reentered','2015-11-29 00:00:00','2013-12-12 00:00:00',94.61,498,502,40.52,'Intermediate' from dual union all
select 39394,'039394','2013-064Q','Prometheus 1-6','Prometheus 6',69527,'Reentered','2015-12-10 00:00:00','2013-12-12 00:00:00',94.61,498,502,40.53,'Intermediate' from dual union all
select 39395,'039395','2013-064R','COPPER','Copper-Cube',69527,'Reentered','2016-02-04 00:00:00','2013-12-12 00:00:00',94.61,498,502,40.53,'Intermediate' from dual union all
select 39396,'039396','2013-064S','ORS Tech 2','VECTOR JCTD 2',69527,'Reentered','2015-04-03 00:00:00','2013-12-12 00:00:00',94.6,498,500,40.53,'Intermediate' from dual union all
select 39397,'039397','2013-064T','Horus','STARE-B',69527,'Reentered','2018-04-26 00:00:00','2013-12-06 00:00:00',94.63,499,503,40.52,'Intermediate' from dual union all
select 39398,'039398','2013-064U','Black Knight 1','',69527,'Reentered','2015-07-16 00:00:00','2013-12-24 00:00:00',94.59,495,503,40.51,'Intermediate' from dual union all
select 39399,'039399','2013-064V','Prometheus 1-7','Prometheus 7',69527,'Reentered','2015-12-05 00:00:00','2013-12-06 00:00:00',94.61,498,502,40.53,'Intermediate' from dual union all
select 394,'000394','62 A Phi 6','deb Sputnik','',8150,'Reentered','1962-09-16 00:00:00','1962-09-14 00:00:00',88.16,162,204,64.83,'Intermediate' from dual union all
select 3940,'003940','1969-042B','11A57I','11A57I',18309,'Reentered','1969-05-17 00:00:00','1969-05-15 00:00:00',88.99,177,273,65.43,'Intermediate' from dual union all
select 39400,'039400','2013-064W','Trailblazer','SPA-1 Trailblazer',69527,'Reentered','2016-05-22 00:00:00','2013-12-24 00:00:00',94.58,494,502,40.51,'Intermediate' from dual union all
select 39401,'039401','2013-064X','Prometheus 1-8','Prometheus 8',69527,'Reentered','2015-12-01 00:00:00','2013-12-06 00:00:00',94.61,499,501,40.51,'Intermediate' from dual union all
select 39402,'039402','2013-064Y','SwampSat','',69527,'Reentered','2015-12-13 00:00:00','2013-12-11 00:00:00',94.62,497,504,40.53,'Intermediate' from dual union all
select 39403,'039403','2013-064Z','Ho''oponopono 2','Ho''oponopono 2',69527,'In Earth orbit','','2013-12-12 00:00:00',94.62,497,504,40.51,'Intermediate' from dual union all
select 39404,'039404','2013-064AA','Firefly','NSF Firefly',69527,'Reentered','2017-11-01 00:00:00','2013-12-12 00:00:00',94.62,499,502,40.51,'Intermediate' from dual union all
select 39405,'039405','2013-064AB','ChargerSat 1','ChargerSat 1.2',69527,'Reentered','2015-03-20 00:00:00','2013-12-12 00:00:00',94.63,500,502,40.51,'Intermediate' from dual union all
select 39406,'039406','2013-064AC','Prometheus 1-1','Prometheus 1',69527,'Reentered','2015-12-12 00:00:00','2013-12-06 00:00:00',94.62,497,504,40.53,'Intermediate' from dual union all
select 39407,'039407','2013-064AD','Vermont Lunar Cubesat','Lunar O/L',69527,'Reentered','2015-11-21 00:00:00','2013-12-24 00:00:00',94.58,494,502,40.53,'Intermediate' from dual union all
select 39408,'039408','2013-064AE','Prometheus 1-3','Prometheus 3',69527,'Reentered','2015-12-08 00:00:00','2013-12-12 00:00:00',94.62,499,502,40.5,'Intermediate' from dual union all
select 39409,'039409','2013-064AF','ORS-3','ORS-3',69527,'Reentered Attached','2015-12-25 00:00:00','2013-11-22 00:00:00',94.67,499,506,40.51,'Intermediate' from dual union all
select 3941,'003941','1969-043A','Charlie Brown','Apollo CM-106',18310,'Deep Space','1969-05-21 00:00:00','1969-05-18 00:00:00',28664.42,226,607532,31.8,'Cislunar' from dual union all
select 39410,'039410','2013-065A','Yaogan Weixing 19','Jianbing-9 03 xing',69528,'In Earth orbit','','2013-11-22 00:00:00',109.5,1201,1207,100.48,'Sun-Synch' from dual union all
select 39411,'039411','2013-065B','CZ-4C Stage 3','CZ-4C Stage 3',69528,'In Earth orbit','','2013-11-23 00:00:00',107.17,991,1201,100.41,'Sun-Synch' from dual union all
select 39412,'039412','1998-067DA','Ardusat-1','Ardusat-1',66503,'Reentered','2014-04-15 00:00:00','2013-11-22 00:00:00',92.8,410,414,51.65,'Intermediate' from dual union all
select 39413,'039413','1998-067DB','PicoDragon','PicoDragon',66503,'Reentered','2014-02-28 00:00:00','2013-11-22 00:00:00',92.8,410,414,51.65,'Intermediate' from dual union all
select 39414,'039414','1998-067DC','Ardusat-X','Ardusat-X',66503,'Reentered','2014-04-15 00:00:00','2013-11-22 00:00:00',92.8,409,415,51.65,'Intermediate' from dual union all
select 39415,'039415','1998-067DD','TechEdSat-3p','TES-3',66503,'Reentered','2014-01-16 00:00:00','2013-11-20 00:00:00',92.81,410,415,51.65,'Intermediate' from dual union all
select 39416,'039416','2013-066A','AprizeSat-7','AprizeSat-7',69529,'In Earth orbit','','2013-11-23 00:00:00',97.16,593,653,97.8,'Sun-Synch' from dual union all
select 39417,'039417','2013-066B','TshepisoSat','ZACUBE-1',69529,'In Earth orbit','','2013-12-03 00:00:00',97.47,593,683,97.8,'Sun-Synch' from dual union all
select 39418,'039418','2013-066C','SkySat 1','SkySat 1',69529,'In Earth orbit','','2013-11-23 00:00:00',96.31,567,597,97.81,'Sun-Synch' from dual union all
select 39419,'039419','2013-066D','DubaiSat-2','DubaiSat-2',69529,'In Earth orbit','','2013-11-23 00:00:00',96.55,585,602,97.81,'Sun-Synch' from dual union all
select 3942,'003942','1969-041C','Fairing','Fairing',18308,'Reentered','1969-05-19 00:00:00','1969-05-18 00:00:00',89.06,163,293,64.96,'Intermediate' from dual union all
select 39420,'039420','2013-066E','OPTOS','OPTOS',69529,'In Earth orbit','','2013-11-23 00:00:00',98.66,593,797,97.79,'Sun-Synch' from dual union all
select 39421,'039421','2013-066F','Unisat-5','Unisat-5',69529,'In Earth orbit','','2013-11-23 00:00:00',97.02,594,638,97.8,'Sun-Synch' from dual union all
select 39422,'039422','2013-066G','STSat-3','Gwahaggisul-wiseong 3ho',69529,'In Earth orbit','','2013-11-23 00:00:00',96.87,592,626,97.81,'Sun-Synch' from dual union all
select 39423,'039423','2013-066H','WNISAT-1','WNISAT-1',69529,'In Earth orbit','','2013-11-24 00:00:00',99.21,593,848,97.78,'Sun-Synch' from dual union all
select 39424,'039424','2013-066J','CINEMA 2','KHUSAT-1',69529,'In Earth orbit','','2013-11-23 00:00:00',97.97,593,730,97.79,'Sun-Synch' from dual union all
select 39425,'039425','2013-066K','AprizeSat-8','',69529,'In Earth orbit','','2013-11-24 00:00:00',97.32,593,667,97.8,'Sun-Synch' from dual union all
select 39426,'039426','2013-066L','CINEMA 3','KHUSAT-2',69529,'In Earth orbit','','2013-11-24 00:00:00',98.14,593,747,97.79,'Sun-Synch' from dual union all
select 39427,'039427','2013-066M','Triton 1','Triton 1',69529,'In Earth orbit','','2013-11-23 00:00:00',98.31,593,763,97.79,'Sun-Synch' from dual union all
select 39428,'039428','2013-066N','Delfi-n3Xt','Delfi-n3Xt',69529,'In Earth orbit','','2013-11-23 00:00:00',98.49,593,780,97.79,'Sun-Synch' from dual;
insert into SATELLITE
select 39429,'039429','2013-066P','Dove-3','Dove 0711',69529,'In Earth orbit','','2013-11-24 00:00:00',98.85,593,815,97.78,'Sun-Synch' from dual union all
select 3943,'003943','1969-043B','Saturn S-IVB-505N','Saturn S-IVB-505N',18310,'Deep Space','1969-05-20 00:00:00','1969-05-18 00:00:00',33506.15,645,675156,31.76,'Cislunar' from dual union all
select 39430,'039430','2013-066Q','GATOSS','GOMX-1',69529,'In Earth orbit','','2013-11-23 00:00:00',98.88,593,818,97.78,'Sun-Synch' from dual union all
select 39431,'039431','2013-066R','Lem','BRITE-P1',69529,'In Earth orbit','','2013-11-23 00:00:00',99.6,593,886,97.78,'Sun-Synch' from dual union all
select 39432,'039432','2013-066S','ICUBE-1','ICUBE-1',69529,'In Earth orbit','','2013-11-29 00:00:00',97.05,596,638,97.8,'Sun-Synch' from dual union all
select 39433,'039433','2013-066T','HUMSAT-D','HUMSAT-D',69529,'In Earth orbit','','2013-12-03 00:00:00',97.06,596,639,97.8,'Sun-Synch' from dual union all
select 39434,'039434','2013-066U','Pocket-PUCP','Pocket-PUCP',69529,'In Earth orbit','','2013-12-03 00:00:00',96.97,589,639,97.8,'Sun-Synch' from dual union all
select 39435,'039435','2013-066V','WREN','WREN',69529,'In Earth orbit','','2013-11-29 00:00:00',97.02,593,639,97.8,'Sun-Synch' from dual union all
select 39436,'039436','2013-066W','Morehead-OSCAR-76','$50SAT',69529,'Reentered','2018-05-19 00:00:00','2013-11-29 00:00:00',96.98,590,639,97.8,'Sun-Synch' from dual union all
select 39437,'039437','2013-066X','T-LogoQube','Eagle-1',69529,'In Earth orbit','','2013-11-29 00:00:00',96.97,589,639,97.8,'Sun-Synch' from dual union all
select 39438,'039438','2013-066Y','VELOX-P2','VELOX-P2',69529,'In Earth orbit','','2013-11-29 00:00:00',97.64,593,699,97.8,'Sun-Synch' from dual union all
select 39439,'039439','2013-066Z','First-MOVE','First-MOVE',69529,'In Earth orbit','','2013-12-03 00:00:00',97.64,593,699,97.8,'Sun-Synch' from dual union all
select 3944,'003944','1969-044A','Kosmos-282','Zenit-4 No. 57',18311,'Landed','1969-05-28 00:00:00','1969-05-21 00:00:00',89.71,201,320,65.41,'Intermediate' from dual union all
select 39440,'039440','2013-066AA','LUSAT-OSCAR-74','Manolito',69529,'In Earth orbit','','2013-11-29 00:00:00',97.81,594,713,97.79,'Sun-Synch' from dual union all
select 39441,'039441','2013-066AB','NEE-02 KRYSAOR','Krysaor',69529,'In Earth orbit','','2013-11-29 00:00:00',97.8,593,714,97.8,'Sun-Synch' from dual union all
select 39442,'039442','2013-066AC','PUCP-Sat-1','PUCP-Sat-1',69529,'In Earth orbit','','2013-11-29 00:00:00',97.06,597,639,97.81,'Sun-Synch' from dual union all
select 39443,'039443','2013-066AD','QubeScout-S1','5 cm cube, 0.4 kg',69529,'In Earth orbit','','2013-11-29 00:00:00',96.98,590,639,97.8,'Sun-Synch' from dual union all
select 39444,'039444','2013-066AE','AMSAT-OSCAR-73','FUNcube-1',69529,'In Earth orbit','','2013-12-03 00:00:00',97.46,593,682,97.8,'Sun-Synch' from dual union all
select 39445,'039445','2013-066AF','HiNCube-1','HiNCube-1',69529,'In Earth orbit','','2013-11-23 00:00:00',97.47,594,682,97.8,'Sun-Synch' from dual union all
select 39446,'039446','2013-066AG','UWE-3','UWE-3',69529,'In Earth orbit','','2013-11-29 00:00:00',97.63,593,697,97.8,'Sun-Synch' from dual union all
select 39447,'039447','1978-083T','deb Kosmos-1030','deb Oko',45278,'In Earth orbit','','2015-04-08 00:00:00',718.67,5932,34467,68.48,'GEO Transfer' from dual union all
select 39448,'039448','2013-066AJ','R-36M2 DS','R-36M2 DS',69529,'In Earth orbit','','2013-11-23 00:00:00',107.26,591,1609,97.7,'Sun-Synch' from dual union all
select 39449,'039449','2013-066AK','Gazodinamicheskiy Ekran','GDS (Gas dynamic shield)',69529,'Reentered','2015-07-12 00:00:00','2013-11-24 00:00:00',96.11,548,597,97.81,'Sun-Synch' from dual union all
select 3945,'003945','1969-044B','11A57I','11A57I',18311,'Reentered','1969-05-28 00:00:00','1969-05-22 00:00:00',89.43,205,288,65.41,'Intermediate' from dual union all
select 39450,'039450','2013-066AL','Dnepr Platform A/Fairing','',69529,'In Earth orbit','','2013-11-24 00:00:00',96.68,590,610,97.81,'Sun-Synch' from dual union all
select 39451,'039451','2013-067A','Swarm B','Swarm 2',69530,'In Earth orbit','','2013-11-25 00:00:00',94.52,490,501,87.55,'Polar' from dual union all
select 39452,'039452','2013-067B','Swarm A','Swarm 1',69530,'In Earth orbit','','2013-11-25 00:00:00',94.52,490,501,87.55,'Polar' from dual union all
select 39453,'039453','2013-067C','Swarm C','Swarm 3',69530,'In Earth orbit','','2013-11-24 00:00:00',94.52,490,501,87.55,'Polar' from dual union all
select 39454,'039454','2013-067D','Briz-KM No. 72524','Briz-KM 14S45',69530,'In Earth orbit','','2013-11-25 00:00:00',93.9,460,471,87.56,'Polar' from dual union all
select 39455,'039455','2013-068A','Shiyan Weixing 5','shiyan weixing wuhao',69531,'In Earth orbit','','2013-11-27 00:00:00',99.74,738,754,97.99,'Sun-Synch' from dual union all
select 39456,'039456','2013-069A','Progress M-21M','Progress 7K-TGM No.421',69532,'Deorbited','2014-06-09 00:00:00','2013-11-28 00:00:00',92.49,383,411,51.66,'Intermediate' from dual union all
select 39457,'039457','2013-069B','11S510','11S510',69532,'Reentered','2013-11-28 00:00:00','2013-11-26 00:00:00',88.37,177,211,51.65,'Intermediate' from dual union all
select 39458,'039458','2013-070A','Chang''e-3','Chang''e-3',69533,'Deep Space','2013-12-02 00:00:00','2013-12-02 00:00:00',14946.16,210,389019,28.5,'Cislunar' from dual union all
select 39459,'039459','2013-070B','CZ-3B Y23 Stage 3','CZ-3B Stage 3',69533,'Deep Space','2013-12-02 00:00:00','2013-12-02 00:00:00',14946.16,210,389019,28.5,'Cislunar' from dual union all
select 3946,'003946','1968-097BT','deb Kosmos-252','',16824,'Reentered','1969-05-27 00:00:00','1969-05-21 00:00:00',93.14,287,570,62.15,'Intermediate' from dual union all
select 39460,'039460','2013-071A','SES-8','SES-8',69534,'In Earth orbit','','2013-12-04 00:00:00',1649.14,395,79325,20.51,'Very High' from dual union all
select 39461,'039461','2013-071B','Falcon 9-007 Stage 2','F9-007 St2',69534,'Reentered','2014-04-30 00:00:00','2013-12-04 00:00:00',1649.14,395,79325,20.51,'Very High' from dual union all
select 39462,'039462','2013-072A','USA 247','TOPAZ 3',69535,'In Earth orbit','','2013-12-17 00:00:00',106.82,1070,1088,123.01,'Retrograde' from dual union all
select 39463,'039463','2013-072B','FIREBIRD U1','FIREBIRD A',69535,'In Earth orbit','','2013-12-20 00:00:00',98.27,462,889,120.5,'Retrograde' from dual union all
select 39464,'039464','2013-072C','FIREBIRD U2','FIREBIRD B',69535,'In Earth orbit','','2013-12-28 00:00:00',98.27,462,889,120.5,'Retrograde' from dual union all
select 39465,'039465','2013-072D','Aerocube 5a','Aerocube 5a',69535,'In Earth orbit','','2013-12-20 00:00:00',98.27,462,889,120.5,'Retrograde' from dual union all
select 39466,'039466','2013-072E','Aerocube 5b','Aerocube 5b',69535,'In Earth orbit','','2013-12-20 00:00:00',98.27,462,889,120.5,'Retrograde' from dual union all
select 39467,'039467','2013-072F','ALICE','ALICE',69535,'In Earth orbit','','2013-12-20 00:00:00',98.27,462,889,120.5,'Retrograde' from dual union all
select 39468,'039468','2013-072G','SNAP-3','SMDC Nanosatellite Progr',69535,'In Earth orbit','','2013-12-20 00:00:00',98.27,462,889,120.5,'Retrograde' from dual union all
select 39469,'039469','2013-072H','MCubed-2','M-Cubed/COVE',69535,'In Earth orbit','','2013-12-28 00:00:00',98.27,464,888,120.49,'Retrograde' from dual union all
select 3947,'003947','1969-045A','INTELSAT III F-4','INTELSAT III F-4',18312,'In Earth orbit','','1969-12-01 00:00:00',1435.96,35761,35807,.56,'Stationary' from dual union all
select 39470,'039470','2013-072J','CUNYSAT-1','CUNYSAT-1',69535,'In Earth orbit','','2013-12-20 00:00:00',98.27,462,889,120.5,'Retrograde' from dual union all
select 39471,'039471','2013-072K','CP8/IPEX','CalPoly CP8',69535,'In Earth orbit','','2013-12-28 00:00:00',98.27,462,889,120.5,'Retrograde' from dual union all
select 39472,'039472','2013-072L','SMDC-ONE 3.2','SMDC-ONE David',69535,'In Earth orbit','','2013-12-20 00:00:00',98.27,462,889,120.5,'Retrograde' from dual union all
select 39473,'039473','2013-072M','Tacsat-6','Tacsat-6',69535,'In Earth orbit','','2013-12-20 00:00:00',98.27,462,889,120.5,'Retrograde' from dual union all
select 39474,'039474','2013-072N','SMDC-ONE 3.1','SMDC-ONE Charlie',69535,'In Earth orbit','','2013-12-20 00:00:00',98.27,462,889,120.5,'Retrograde' from dual union all
select 39475,'039475','2013-072P','GEMSat','NPSCuL-2 GEMSat',69535,'In Earth orbit','','2013-12-17 00:00:00',98.53,487,890,120.45,'Retrograde' from dual union all
select 39476,'039476','2013-073A','Inmarsat 5F1','Inmarsat 5F1',69536,'In Earth orbit','','2013-12-19 00:00:00',1616.62,31117,47383,.79,'Very High' from dual union all
select 39477,'039477','2013-073B','Briz-M DTB','Briz-M torus tank',69536,'In Earth orbit','','2013-12-16 00:00:00',274.27,361,14849,50.75,'Highly Ellip' from dual union all
select 39478,'039478','1995-021H','deb ERS-2','',65682,'Reentered','2014-05-08 00:00:00','2014-01-13 00:00:00',95.36,534,537,98.56,'Sun-Synch' from dual union all
select 39479,'039479','2013-074A','Gaia','Gaia',69537,'Deep Space','2013-12-19 00:00:00','2013-12-19 00:00:00',56526.13,344,962689,14.98,'Cislunar' from dual union all
select 3948,'003948','1969-043C','LM 4 DS','LM 4 DS',18310,'Deep Space Attached','1969-05-21 00:00:00','1969-05-18 00:00:00',28664.42,226,607532,31.8,'Cislunar' from dual union all
select 39480,'039480','2013-074B','Fregat-MT No. 1040','Fregat-MT    No. 1040',69537,'Deep Space','2013-12-19 00:00:00','2013-12-19 00:00:00',186588.26,374,2150152,14.99,'Cislunar' from dual union all
select 39481,'039481','2013-075A','Tupac Katari','Tupac Katari (Bolivarsat',69538,'In Earth orbit','','2013-12-31 00:00:00',1436.04,35773,35798,.27,'Stationary' from dual union all
select 39482,'039482','2013-075B','CZ-3B Stage 3','CZ-3B Stage 3',69538,'In Earth orbit','','2013-12-29 00:00:00',794.39,195,43874,24.75,'Highly Ellip' from dual union all
select 39483,'039483','2013-076A','Kosmos-2488','Rodnik-S',69539,'In Earth orbit','','2013-12-27 00:00:00',115.84,1479,1508,82.48,'Intermediate' from dual union all
select 39484,'039484','2013-076B','Kosmos-2489','Rodnik-S',69539,'In Earth orbit','','2013-12-27 00:00:00',112.22,1156,1502,82.48,'Intermediate' from dual union all
select 39485,'039485','2013-076C','Kosmos-2490','Rodnik-S',69539,'In Earth orbit','','2013-12-28 00:00:00',115.88,1481,1510,82.49,'Intermediate' from dual union all
select 39486,'039486','2013-076D','Briz-KM No. 72520','Briz-KM 14S45',69539,'In Earth orbit','','2013-12-27 00:00:00',115.86,1480,1509,82.49,'Intermediate' from dual union all
select 39487,'039487','2013-077A','Ekspress-AM5','Ekspress AM-5',69540,'In Earth orbit','','2014-01-25 00:00:00',1429.48,34224,37090,.14,'Drift GEO' from dual union all
select 39488,'039488','2013-077B','Briz-M No. 99543','Briz-M 14S43 No. 99543',69540,'In Earth orbit','','2014-01-27 00:00:00',1607.88,36817,41354,.66,'Very High' from dual union all
select 39489,'039489','2013-077C','Briz-M DTB','Briz-M torus tank',69540,'Reentered','2014-10-18 00:00:00','2014-01-10 00:00:00',672.52,391,37707,49.42,'GEO Transfer' from dual union all
select 3949,'003949','1969-043D','Snoopy','LM 4 AS',18310,'Deep Space Attached','1969-05-21 00:00:00','1969-05-18 00:00:00',28664.42,226,607532,31.8,'Cislunar' from dual union all
select 39490,'039490','2013-078A','Kosmos-2493','SKRL-756 No. 1',69541,'In Earth orbit','','2013-12-29 00:00:00',96.89,595,625,82.42,'Intermediate' from dual union all
select 39491,'039491','2013-078B','Kosmos-2494','SKRL-756 No. 2',69541,'In Earth orbit','','2013-12-29 00:00:00',96.93,598,626,82.42,'Intermediate' from dual union all
select 39492,'039492','2013-078C','Aist 1','Aist No. 1',69541,'In Earth orbit','','2013-12-29 00:00:00',93.09,258,594,82.43,'Intermediate' from dual union all
select 39493,'039493','2013-078D','Blok-I 14S54','14S54',69541,'Reentered','2014-05-08 00:00:00','2013-12-29 00:00:00',96.93,599,625,82.42,'Intermediate' from dual union all
select 39494,'039494','2013-073C','Briz-M No. 99546','Briz-M 14S43 No. 99546',69536,'In Earth orbit','','2014-02-28 00:00:00',1310.72,2827,63763,27.26,'Highly Ellip' from dual union all
select 39495,'039495','1998-067DE','MRC Cable Reel','ISS RS MRC Cable Reel',66503,'Reentered','2014-02-27 00:00:00','2013-11-28 00:00:00',92.86,412,418,51.65,'Intermediate' from dual union all
select 39496,'039496','1998-067DF','Vsplesk','ISS RS Vsplesk',66503,'Reentered','2014-07-07 00:00:00','2008-07-15 00:00:00',91.4,337,350,51.65,'Intermediate' from dual union all
select 39497,'039497','2013-076E','Kosmos-2491','Reshetnev Tech 14F153 No',69539,'In Earth orbit','','2014-01-01 00:00:00',115.87,1482,1508,82.49,'Intermediate' from dual union all
select 39498,'039498','2014-001A','GSAT-14','GSAT-14',69705,'In Earth orbit','','2014-01-13 00:00:00',1426.85,35470,35741,.26,'Drift GEO' from dual union all
select 39499,'039499','2014-001B','GSLV CUS-05','GSLV-D5 CUS',69705,'Reentered','2014-06-08 00:00:00','2014-01-13 00:00:00',626.52,177,35575,19.45,'GEO Transfer' from dual union all
select 395,'000395','62 A Phi 7','deb Sputnik','',8150,'Reentered','1962-09-15 00:00:00','1962-09-12 00:00:00',88.67,189,228,64.75,'Intermediate' from dual union all
select 3950,'003950','1969-046A','ERS 29','ERS 29',18313,'Lost','','1969-07-09 00:00:00',3120.18,16995,111711,33.16,'Very High' from dual union all
select 39500,'039500','2014-002A','Thaicom 6','Thaicom 6',69706,'In Earth orbit','','2014-01-15 00:00:00',1638.03,35904,43399,.35,'Very High' from dual union all
select 39501,'039501','2014-002B','Falcon 9-008 Stage 2','F9-008 St2',69706,'Reentered','2014-05-28 00:00:00','2014-01-14 00:00:00',1929.98,369,89572,22.33,'Very High' from dual union all
select 39502,'039502','2014-003A','SS C. Gordon Fullerton','Cygnus PCM-1',69707,'Deorbited','2014-02-19 00:00:00','2014-01-11 00:00:00',92.77,406,415,51.63,'Intermediate' from dual union all
select 39503,'039503','2014-003B','Antares Stage 2','Castor 30B',69707,'Reentered','2014-01-18 00:00:00','2014-01-11 00:00:00',89.15,215,249,51.62,'Intermediate' from dual union all
select 39504,'039504','2014-004A','TDRS 12','TDRS L',69708,'In Earth orbit','','2014-02-03 00:00:00',1436.01,35781,35789,7.02,'Inclined GEO' from dual union all
select 39505,'039505','2014-004B','Centaur AV-043','Centaur AV-043',69708,'In Earth orbit','','2014-02-05 00:00:00',697.24,4684,34653,25.65,'GEO Transfer' from dual union all
select 39506,'039506','2014-005A','Progress M-22M','Progress 7K-TGM No.422',69709,'Deorbited','2014-04-18 00:00:00','2014-02-06 00:00:00',92.88,413,418,51.65,'Intermediate' from dual union all
select 39507,'039507','2014-005B','11S510','11S510',69709,'Reentered','2014-02-07 00:00:00','2014-02-06 00:00:00',88.37,180,208,51.63,'Intermediate' from dual union all
select 39508,'039508','2014-006A','ABS-2','ABS-2',69710,'In Earth orbit','','2014-02-19 00:00:00',1436.01,35778,35792,.08,'Stationary' from dual union all
select 39509,'039509','2014-006B','ATHENA-FIDUS','ATHENA-FIDUS',69710,'In Earth orbit','','2014-02-26 00:00:00',1436.04,35784,35786,.05,'Stationary' from dual union all
select 3951,'003951','1969-046B','ERS 26','ERS 26',18313,'Lost','','1969-07-08 00:00:00',3115.32,16976,111583,33.15,'Very High' from dual union all
select 39510,'039510','2014-006C','ESC-A 572','Ariane 5 ESC-A',69710,'In Earth orbit','','2014-02-14 00:00:00',632.01,254,35781,6.27,'GEO Transfer' from dual union all
select 39511,'039511','2014-006D','Sylda 5B','Sylda 5 No. 58B',69710,'In Earth orbit','','2014-02-16 00:00:00',628.57,252,35606,6,'GEO Transfer' from dual union all
select 39512,'039512','1998-067DG','Flock 1-03','Dove 074B',66503,'Reentered','2014-05-15 00:00:00','2014-02-12 00:00:00',92.83,411,416,51.65,'Intermediate' from dual union all
select 39513,'039513','1998-067DH','Flock 1-04','Dove 073C',66503,'Reentered','2014-06-09 00:00:00','2014-02-11 00:00:00',92.67,382,429,51.69,'Intermediate' from dual union all
select 39514,'039514','1998-067DJ','Flock 1-01','Dove 073D',66503,'Reentered','2014-05-25 00:00:00','2014-02-12 00:00:00',92.83,410,417,51.64,'Intermediate' from dual union all
select 39515,'039515','1998-067DK','Flock 1-02','Dove 0740',66503,'Reentered','2014-06-18 00:00:00','2014-02-12 00:00:00',92.83,409,418,51.65,'Intermediate' from dual union all
select 39516,'039516','1994-074AC','deb Resurs-O1','',65470,'Reentered','2014-04-16 00:00:00','2014-02-09 00:00:00',95.22,525,534,98.13,'Sun-Synch' from dual union all
select 39517,'039517','1994-074AD','deb Resurs-O1','',65470,'Reentered','2014-05-20 00:00:00','2014-02-10 00:00:00',95.6,547,549,98.17,'Sun-Synch' from dual union all
select 39518,'039518','1998-067DL','Flock 1-05','Dove 072C',66503,'Reentered','2014-06-02 00:00:00','2014-02-13 00:00:00',92.86,409,421,51.64,'Intermediate' from dual;
insert into SATELLITE
select 39519,'039519','1998-067DM','Flock 1-06','Dove 0735',66503,'Reentered','2014-06-15 00:00:00','2014-02-13 00:00:00',92.8,408,416,51.65,'Intermediate' from dual union all
select 3952,'003952','1969-046C','OV5-9','OV5-9',18313,'Lost','','1970-07-20 00:00:00',3116.67,16958,111642,33.71,'Very High' from dual union all
select 39520,'039520','1998-067DN','Flock 1-11','Dove 074D',66503,'Reentered','2014-06-06 00:00:00','2014-02-14 00:00:00',92.83,409,418,51.65,'Intermediate' from dual union all
select 39521,'039521','1998-067DP','Flock 1-12','Dove 0730',66503,'Reentered','2014-05-25 00:00:00','2014-02-14 00:00:00',92.83,409,418,51.65,'Intermediate' from dual union all
select 39522,'039522','2014-007A','Turksat 4A','Turksat 4A',69711,'In Earth orbit','','2014-02-18 00:00:00',1376.63,33448,35781,.47,'Medium' from dual union all
select 39523,'039523','2014-007B','Briz-M No. 99544','Briz-M 14S43 No. 99544',69711,'In Earth orbit','','2014-02-18 00:00:00',763.85,8412,34192,13.07,'Medium' from dual union all
select 39524,'039524','2014-007C','Briz-M DTB','Briz-M torus tank',69711,'In Earth orbit','','2014-02-18 00:00:00',388.46,350,22165,49.37,'Highly Ellip' from dual union all
select 39525,'039525','1998-067DQ','Flock 1-13','Dove 0737',66503,'Reentered','2014-07-06 00:00:00','2014-02-15 00:00:00',92.83,409,418,51.65,'Intermediate' from dual union all
select 39526,'039526','1998-067DR','Flock 1-07','Dove 073B',66503,'Reentered','2014-06-14 00:00:00','2014-02-15 00:00:00',92.84,410,417,51.65,'Intermediate' from dual union all
select 39527,'039527','1998-067DS','Flock 1-15','Dove 0721',66503,'Reentered','2014-06-13 00:00:00','2014-02-15 00:00:00',92.82,409,417,51.65,'Intermediate' from dual union all
select 39528,'039528','1998-067DT','Flock 1-16','Dove 0727',66503,'Reentered','2014-10-26 00:00:00','2014-02-15 00:00:00',92.83,409,418,51.65,'Intermediate' from dual union all
select 39529,'039529','1998-067DU','Flock 1-09','Dove 0736',66503,'Reentered','2014-07-02 00:00:00','2014-02-16 00:00:00',92.83,412,415,51.65,'Intermediate' from dual union all
select 3953,'003953','1968-097BU','deb Kosmos-252 [*]','',16824,'Reentered','1981-04-13 00:00:00','1969-06-08 00:00:00',111.71,517,2094,62.48,'Intermediate' from dual union all
select 39530,'039530','1998-067DV','Flock 1-10','Dove 0742',66503,'Reentered','2014-06-20 00:00:00','2014-02-16 00:00:00',92.83,411,416,51.65,'Intermediate' from dual union all
select 39531,'039531','1998-067DW','Flock 1-14','Dove 072E',66503,'Reentered','2014-05-21 00:00:00','2014-02-16 00:00:00',92.82,409,417,51.65,'Intermediate' from dual union all
select 39532,'039532','1998-067DX','Flock 1-08','Dove 0741',66503,'Reentered','2014-06-20 00:00:00','2014-02-16 00:00:00',92.83,409,418,51.65,'Intermediate' from dual union all
select 39533,'039533','2014-008A','GPS IIF-5','GPS SVN 64',69712,'In Earth orbit','','2014-02-23 00:00:00',729.35,20456,20468,54.99,'Medium' from dual union all
select 39534,'039534','2014-008B','Delta 4 Second Stage','DCSS-4',69712,'In Earth orbit','','2014-02-22 00:00:00',754.66,20463,21696,54.73,'Medium' from dual union all
select 39535,'039535','1999-025EWC','deb FY-1C','',66706,'In Earth orbit','','2014-02-21 00:00:00',102.92,849,945,99.18,'Sun-Synch' from dual union all
select 39536,'039536','1999-025EWD','deb FY-1C','',66706,'In Earth orbit','','2014-02-23 00:00:00',105.8,838,1226,99.62,'Sun-Synch' from dual union all
select 39537,'039537','1999-025EWE','deb FY-1C','',66706,'In Earth orbit','','2014-02-22 00:00:00',105.01,847,1143,99.22,'Sun-Synch' from dual union all
select 39538,'039538','1999-025EWF','deb FY-1C','',66706,'Reentered','2014-06-22 00:00:00','2014-02-13 00:00:00',96.27,531,629,99.89,'Sun-Synch' from dual union all
select 39539,'039539','1999-025EWG','deb FY-1C','',66706,'In Earth orbit','','2014-02-23 00:00:00',98.33,574,784,98.99,'Sun-Synch' from dual union all
select 3954,'003954','1969-046D','Vela 5A','Vela 5A',18313,'Deep Space','1984-07-01 00:00:00','1969-07-22 00:00:00',6706.31,110891,111955,33.13,'Very High' from dual union all
select 39540,'039540','1999-025EWH','deb FY-1C','',66706,'In Earth orbit','','2014-02-21 00:00:00',109.08,843,1526,106.18,'Retrograde' from dual union all
select 39541,'039541','1999-025EWJ','deb FY-1C','',66706,'In Earth orbit','','2014-02-24 00:00:00',101.92,792,908,99,'Sun-Synch' from dual union all
select 39542,'039542','1999-025EWK','deb FY-1C','',66706,'In Earth orbit','','2014-02-16 00:00:00',104.88,768,1210,98.22,'Sun-Synch' from dual union all
select 39543,'039543','1999-025EWL','deb FY-1C','',66706,'In Earth orbit','','2014-02-24 00:00:00',105.63,759,1289,99.08,'Sun-Synch' from dual union all
select 39544,'039544','1999-025EWM','deb FY-1C','',66706,'In Earth orbit','','2014-02-24 00:00:00',102.77,845,934,98.9,'Sun-Synch' from dual union all
select 39545,'039545','1993-036BUB','deb Kosmos-2251','',65138,'In Earth orbit','','2014-02-22 00:00:00',103.2,753,1067,73.92,'Intermediate' from dual union all
select 39546,'039546','1993-036BUC','deb Kosmos-2251','',65138,'In Earth orbit','','2014-02-24 00:00:00',98.99,651,769,74.01,'Intermediate' from dual union all
select 39547,'039547','1993-036BUD','deb Kosmos-2251','',65138,'In Earth orbit','','2014-02-24 00:00:00',99.96,745,769,74.03,'Intermediate' from dual union all
select 39548,'039548','1993-036BUE','deb Kosmos-2251','',65138,'In Earth orbit','','2014-02-22 00:00:00',99.66,729,756,74.02,'Intermediate' from dual union all
select 39549,'039549','1993-036BUF','deb Kosmos-2251','',65138,'In Earth orbit','','2014-02-19 00:00:00',99.74,728,765,74.03,'Intermediate' from dual union all
select 3955,'003955','1969-046E','Vela 5B','Vela 5B',18313,'Deep Space','1983-09-30 00:00:00','1969-07-22 00:00:00',6702.42,110741,112014,33.15,'Very High' from dual union all
select 39550,'039550','1993-036BUG','deb Kosmos-2251','',65138,'In Earth orbit','','2014-02-09 00:00:00',100.51,688,878,74.12,'Intermediate' from dual union all
select 39551,'039551','1993-036BUH','deb Kosmos-2251','',65138,'Reentered','2017-01-06 00:00:00','2014-02-07 00:00:00',100.28,721,823,73.99,'Intermediate' from dual union all
select 39552,'039552','1993-036BUJ','deb Kosmos-2251','',65138,'In Earth orbit','','2014-02-22 00:00:00',99.1,684,748,74.04,'Intermediate' from dual union all
select 39553,'039553','1993-036BUK','deb Kosmos-2251','',65138,'In Earth orbit','','2014-02-20 00:00:00',103.72,759,1109,74.07,'Intermediate' from dual union all
select 39554,'039554','1993-036BUL','deb Kosmos-2251','',65138,'In Earth orbit','','2014-02-25 00:00:00',99.5,719,751,74.03,'Intermediate' from dual union all
select 39555,'039555','1998-067DY','Flock 1-17','Dove 074C',66503,'Reentered','2014-05-19 00:00:00','2014-02-26 00:00:00',92.81,409,416,51.64,'Intermediate' from dual union all
select 39556,'039556','1998-067DZ','Flock 1-18','Dove 0734',66503,'Reentered','2014-10-29 00:00:00','2014-02-26 00:00:00',92.8,408,416,51.65,'Intermediate' from dual union all
select 39557,'039557','1998-067EA','Flock 1-21','Dove 072B',66503,'Reentered','2014-05-25 00:00:00','2014-02-26 00:00:00',92.81,409,416,51.65,'Intermediate' from dual union all
select 39558,'039558','1998-067EB','Flock 1-22','Dove 0728',66503,'Reentered','2014-06-05 00:00:00','2014-02-26 00:00:00',92.81,408,417,51.65,'Intermediate' from dual union all
select 39559,'039559','1998-067EC','Flock 1-19','Dove 073A',66503,'Reentered','2014-06-03 00:00:00','2014-02-27 00:00:00',92.81,408,417,51.65,'Intermediate' from dual union all
select 3956,'003956','1969-046F','Transtage 15','Transtage 15',18313,'In Earth orbit','','2004-07-21 00:00:00',3107.32,7052,121265,35.07,'Very High' from dual union all
select 39560,'039560','1998-067ED','Flock 1-20','Dove 0743',66503,'Reentered','2014-10-09 00:00:00','2014-02-27 00:00:00',92.81,409,416,51.65,'Intermediate' from dual union all
select 39561,'039561','1998-067EE','Flock 1-25','Dove 0722',66503,'Reentered','2014-07-10 00:00:00','2014-02-27 00:00:00',92.82,408,418,51.66,'Intermediate' from dual union all
select 39562,'039562','1998-067EF','Flock 1-23','Dove 0725',66503,'Reentered','2014-06-16 00:00:00','2014-02-27 00:00:00',92.81,399,426,51.64,'Intermediate' from dual union all
select 39563,'039563','1998-067EG','Flock 1-26','Dove 0744',66503,'Reentered','2014-06-24 00:00:00','2014-02-27 00:00:00',92.8,409,415,51.65,'Intermediate' from dual union all
select 39564,'039564','1998-067EH','Flock 1-24','Dove 0723',66503,'Reentered','2014-06-10 00:00:00','2014-02-27 00:00:00',92.8,409,415,51.65,'Intermediate' from dual union all
select 39565,'039565','1998-067EJ','Ardusat-2','Ardusat-2',66503,'Reentered','2014-05-03 00:00:00','2014-03-01 00:00:00',92.79,405,418,51.68,'Intermediate' from dual union all
select 39566,'039566','1998-067EK','UAP-SAT  ','UAP-SAT',66503,'Reentered','2014-06-03 00:00:00','2014-03-01 00:00:00',92.8,409,415,51.64,'Intermediate' from dual union all
select 39567,'039567','1998-067EL','SkyCube-1','SkyCube-1',66503,'Reentered','2014-11-08 00:00:00','2014-03-01 00:00:00',92.78,406,416,51.65,'Intermediate' from dual union all
select 39568,'039568','1998-067EM','Litsat-1','Litsat-1',66503,'Reentered','2014-05-22 00:00:00','2014-03-01 00:00:00',92.78,408,414,51.64,'Intermediate' from dual union all
select 39569,'039569','1998-067EN','LituanicaSAT-1','LituanicaSAT-1',66503,'Reentered','2014-07-28 00:00:00','2014-03-01 00:00:00',92.79,408,414,51.65,'Intermediate' from dual union all
select 3957,'003957','1969-047A','Kosmos-283','DS-P1-Yu No. 19',18314,'Reentered','1969-12-10 00:00:00','1969-06-27 00:00:00',100.45,196,1364,81.96,'Intermediate' from dual union all
select 39570,'039570','1998-067EP','Flock 1-27','Dove 0729',66503,'Reentered','2014-05-23 00:00:00','2014-03-01 00:00:00',92.77,403,418,51.66,'Intermediate' from dual union all
select 39571,'039571','1998-067EQ','Flock 1-28','Dove 0720',66503,'Reentered','2014-07-01 00:00:00','2014-03-01 00:00:00',92.77,403,418,51.66,'Intermediate' from dual union all
select 39572,'039572','2014-009A','Ginrei','ShindaiSat',69713,'Reentered','2014-11-24 00:00:00','2014-02-28 00:00:00',92.47,387,404,65.02,'Intermediate' from dual union all
select 39573,'039573','2014-009B','Yui','ITF-1',69713,'Reentered','2014-06-29 00:00:00','2014-02-28 00:00:00',92.28,382,391,65.01,'Intermediate' from dual union all
select 39574,'039574','2014-009C','GPM Core','GPM',69713,'In Earth orbit','','2014-02-28 00:00:00',92.38,383,399,65.01,'Intermediate' from dual union all
select 39575,'039575','2014-009D','Kosu mozu','OPUSat',69713,'Reentered','2014-07-24 00:00:00','2014-02-28 00:00:00',92.27,383,389,65.01,'Intermediate' from dual union all
select 39576,'039576','2014-009E','Teikyosat-3','Microbial Observation Sa',69713,'Reentered','2014-10-25 00:00:00','2014-02-28 00:00:00',92.32,384,393,65.02,'Intermediate' from dual union all
select 39577,'039577','2014-009F','Cubesat-OSCAR-77','Invader',69713,'Reentered','2014-09-02 00:00:00','2014-02-28 00:00:00',92.35,384,396,65.01,'Intermediate' from dual union all
select 39578,'039578','2014-009G','Hayato 2','K-SAT-2',69713,'Reentered','2014-05-18 00:00:00','2014-02-28 00:00:00',92.26,382,388,65.01,'Intermediate' from dual union all
select 39579,'039579','2014-009H','Gennai','STARS-2 Mother',69713,'Reentered','2014-04-26 00:00:00','2014-02-28 00:00:00',92.02,367,381,65.01,'Intermediate' from dual union all
select 3958,'003958','1969-047B','S1M','S1M',18314,'Reentered','1969-10-01 00:00:00','1969-06-29 00:00:00',99.3,196,1254,81.95,'Intermediate' from dual union all
select 39580,'039580','2014-009J','H-2A F23 Stage 2','H-2A Stage 2',69713,'Reentered','2014-06-16 00:00:00','2014-02-28 00:00:00',91.88,359,374,65.03,'Intermediate' from dual union all
select 39581,'039581','2014-009K','deb H-2A','',69713,'Reentered','2014-03-10 00:00:00','2014-03-03 00:00:00',91.58,344,360,65,'Intermediate' from dual union all
select 39582,'039582','1993-036BUM','deb Kosmos-2251','',65138,'In Earth orbit','','2014-02-28 00:00:00',99.29,662,788,74.03,'Intermediate' from dual union all
select 39583,'039583','1993-036BUN','deb Kosmos-2251','',65138,'In Earth orbit','','2014-03-01 00:00:00',100.94,647,959,73.94,'Intermediate' from dual union all
select 39584,'039584','1993-036BUP','deb Kosmos-2251','',65138,'In Earth orbit','','2014-03-04 00:00:00',98.98,667,753,74.12,'Intermediate' from dual union all
select 39585,'039585','1993-036BUQ','deb Kosmos-2251','',65138,'In Earth orbit','','2014-03-03 00:00:00',99.19,664,775,74.04,'Intermediate' from dual union all
select 39586,'039586','1993-036BUR','deb Kosmos-2251','',65138,'In Earth orbit','','2014-03-02 00:00:00',101.75,763,920,74.02,'Intermediate' from dual union all
select 39587,'039587','1993-036BUS','deb Kosmos-2251','',65138,'In Earth orbit','','2014-02-24 00:00:00',97,512,718,74.02,'Intermediate' from dual union all
select 39588,'039588','1993-036BUT','deb Kosmos-2251','',65138,'In Earth orbit','','2014-03-02 00:00:00',100.51,720,846,74.14,'Intermediate' from dual union all
select 39589,'039589','1993-036BUU','deb Kosmos-2251','',65138,'In Earth orbit','','2014-03-04 00:00:00',98.54,649,729,74.02,'Intermediate' from dual union all
select 3959,'003959','1968-091BT','deb Kosmos-249','',16818,'Reentered','1979-10-29 00:00:00','1979-10-24 00:00:00',98.07,457,875,62.45,'Intermediate' from dual union all
select 39590,'039590','1993-036BUV','deb Kosmos-2251','',65138,'In Earth orbit','','2014-02-28 00:00:00',99.82,735,764,74.02,'Intermediate' from dual union all
select 39591,'039591','1993-036BUW','deb Kosmos-2251','',65138,'In Earth orbit','','2014-03-04 00:00:00',100.4,754,801,74.02,'Intermediate' from dual union all
select 39592,'039592','1970-025QT','deb Agena D','',21502,'In Earth orbit','','2014-03-12 00:00:00',106.55,1064,1070,100.02,'Sun-Synch' from dual union all
select 39593,'039593','1997-051AAF','deb Iridium 33','',66131,'Reentered','2016-04-15 00:00:00','2014-03-07 00:00:00',98.84,698,709,86.35,'Polar' from dual union all
select 39594,'039594','1997-051AAG','deb Iridium 33','',66131,'In Earth orbit','','2014-03-12 00:00:00',100.11,753,775,86.41,'Polar' from dual union all
select 39595,'039595','1997-051AAH','deb Iridium 33','',66131,'In Earth orbit','','2014-03-12 00:00:00',100.21,722,814,86.38,'Polar' from dual union all
select 39596,'039596','1999-057TA','ZY-1 debris','',66738,'In Earth orbit','','2014-03-10 00:00:00',99.84,747,755,98.29,'Sun-Synch' from dual union all
select 39597,'039597','2001-049QQ','deb PSLV-C3 PS4','',67202,'Reentered','2016-05-19 00:00:00','2014-03-12 00:00:00',98.34,585,774,97.58,'Sun-Synch' from dual union all
select 39598,'039598','2007-003AK','deb CZ-3','',68345,'In Earth orbit','','2014-03-12 00:00:00',789.53,848,42990,26.32,'Highly Ellip' from dual union all
select 39599,'039599','2008-011DK','deb Briz-M No. 88525','',68519,'In Earth orbit','','2014-03-10 00:00:00',457.88,722,25879,49.28,'Highly Ellip' from dual union all
select 396,'000396','62 A Chi','FTV 1133','CORONA M-12',8138,'Reentered Attached','1962-11-19 00:00:00','1962-10-17 00:00:00',91.74,192,528,81.86,'Intermediate' from dual union all
select 3960,'003960','1968-091BU','deb Kosmos-249','',16818,'Reentered','1978-11-26 00:00:00','1971-04-24 00:00:00',108.79,502,1840,62.31,'Intermediate' from dual union all
select 39600,'039600','2010-057X','deb CZ-3C','',68917,'In Earth orbit','','2014-03-10 00:00:00',642.22,401,36158,19.55,'GEO Transfer' from dual union all
select 39601,'039601','2012-044DU','deb Briz-M No. 99532','',69299,'In Earth orbit','','2014-03-12 00:00:00',128.61,266,3858,49.96,'Medium' from dual union all
select 39602,'039602','1979-095CD','deb Meteor-2','',47579,'In Earth orbit','','2014-03-11 00:00:00',102.18,844,880,81.22,'Intermediate' from dual union all
select 39603,'039603','1979-095CE','deb Meteor-2','',47579,'In Earth orbit','','2014-03-05 00:00:00',102.24,852,878,81.21,'Intermediate' from dual union all
select 39604,'039604','1979-095CF','deb Meteor-2','',47579,'In Earth orbit','','2014-03-09 00:00:00',99,701,720,81.23,'Intermediate' from dual union all
select 39605,'039605','1979-095CG','deb Meteor-2','',47579,'In Earth orbit','','2014-03-05 00:00:00',102.02,846,863,81.2,'Intermediate' from dual union all
select 39606,'039606','1979-095CH','deb Meteor-2','',47579,'In Earth orbit','','2014-03-03 00:00:00',101.94,834,867,81.2,'Intermediate' from dual union all
select 39607,'039607','1979-095CJ','deb Meteor-2','',47579,'In Earth orbit','','2014-03-04 00:00:00',101.81,831,858,81.21,'Intermediate' from dual union all
select 39608,'039608','1979-095CK','deb Meteor-2','',47579,'In Earth orbit','','2014-03-06 00:00:00',102.03,843,867,81.2,'Intermediate' from dual;
insert into SATELLITE
select 39609,'039609','1965-027FS','deb SNAPSHOT','',11777,'In Earth orbit','','2014-03-07 00:00:00',111.07,1258,1293,90.29,'Polar' from dual union all
select 3961,'003961','1968-091BV','deb Kosmos-249','',16818,'Reentered','1982-11-12 00:00:00','1973-08-18 00:00:00',114.01,738,2083,62.53,'Intermediate' from dual union all
select 39610,'039610','1991-050S','deb ERS-1','',64219,'In Earth orbit','','2014-03-10 00:00:00',100.14,743,787,98.3,'Sun-Synch' from dual union all
select 39611,'039611','1994-074AE','deb Resurs-O1','',65470,'Reentered','2014-04-06 00:00:00','2014-03-04 00:00:00',94.48,489,497,98.14,'Sun-Synch' from dual union all
select 39612,'039612','2014-010A','Ekspress-AT1','Ekspress AT-1',69714,'In Earth orbit','','2014-03-23 00:00:00',1431.31,35523,35863,.16,'Drift GEO' from dual union all
select 39613,'039613','2014-010B','Ekspress-AT2','Ekspress AT-2',69714,'In Earth orbit','','2014-03-23 00:00:00',1426.14,35404,35779,.15,'Drift GEO' from dual union all
select 39614,'039614','2014-010C','Briz-M No. 99545','Briz-M 14S43 No. 99545',69714,'In Earth orbit','','2014-03-24 00:00:00',1589.8,35638,41850,.17,'Very High' from dual union all
select 39615,'039615','2014-010D','Briz-M DTB','Briz-M torus tank',69714,'Reentered','2017-09-15 00:00:00','2014-03-24 00:00:00',633.14,392,35700,48.62,'GEO Transfer' from dual union all
select 39616,'039616','2014-011A','Amazonas 4A','Amazonas 4A',69715,'In Earth orbit','','2014-04-04 00:00:00',1435.83,35777,35786,.13,'Stationary' from dual union all
select 39617,'039617','2014-011B','Astra 5B','Astra 5B',69715,'In Earth orbit','','2014-03-31 00:00:00',1431.9,35698,35711,.04,'Drift GEO' from dual union all
select 39618,'039618','2014-011C','ESC-A 571','Ariane 5 ESC-A',69715,'In Earth orbit','','2014-03-25 00:00:00',629.35,261,35637,3.33,'GEO Transfer' from dual union all
select 39619,'039619','2014-011D','Sylda 5A','Sylda 5 No. 57A',69715,'In Earth orbit','','2014-03-30 00:00:00',628.87,264,35609,3.02,'GEO Transfer' from dual union all
select 3962,'003962','1968-091BW','deb Kosmos-249 [*]','',16818,'Reentered','2000-05-10 00:00:00','1992-02-03 00:00:00',100.28,489,1055,62.87,'Intermediate' from dual union all
select 39620,'039620','2014-012A','Kosmos-2492','Uragan-M No. 754',69716,'In Earth orbit','','2014-04-02 00:00:00',676.71,19108,19201,64.81,'Medium' from dual union all
select 39621,'039621','2014-012B','Fregat-M No. 112-01','Fregat 14S44 No. 112-01',69716,'In Earth orbit','','2014-03-27 00:00:00',689.23,19269,19668,64.77,'Medium' from dual union all
select 39622,'039622','2014-013A','Soyuz TMA-12M','Soyuz 11F732A47 No. 712',69717,'Landed','2014-09-11 00:00:00','2014-03-26 00:00:00',92.97,417,422,51.63,'Intermediate' from dual union all
select 39623,'039623','2014-013B','11S510','11S510',69717,'Reentered','2014-03-28 00:00:00','2014-03-27 00:00:00',88.35,178,208,51.64,'Intermediate' from dual union all
select 39624,'039624','2014-014A','Shi Jian 11-06','shijian shi yihao 06 xin',69718,'In Earth orbit','','2014-04-02 00:00:00',98.67,687,704,98.26,'Sun-Synch' from dual union all
select 39625,'039625','2014-014B','CZ-2C Stage 2','CZ-2C Stage 2',69718,'In Earth orbit','','2014-04-03 00:00:00',96.36,481,688,98.33,'Sun-Synch' from dual union all
select 39626,'039626','2014-014C','CZ-2C Stage 2 motor cover','',69718,'In Earth orbit','','2014-04-02 00:00:00',99.45,652,813,98.04,'Sun-Synch' from dual union all
select 39627,'039627','2014-014D','CZ-2C Stage 2 motor cover','',69718,'In Earth orbit','','2014-04-03 00:00:00',101.4,690,960,98.08,'Sun-Synch' from dual union all
select 39628,'039628','2014-014E','CZ-2C Stage 2 motor cover','',69718,'In Earth orbit','','2014-04-02 00:00:00',99.45,652,813,98.49,'Sun-Synch' from dual union all
select 39629,'039629','2014-014F','CZ-2C Stage 2 motor cover','',69718,'In Earth orbit','','2014-04-02 00:00:00',101.32,690,953,98.49,'Sun-Synch' from dual union all
select 3963,'003963','1965-020EW','deb Blok Kosmos','',11770,'Reentered','1991-07-09 00:00:00','1991-07-10 00:00:00',90.07,259,297,55.83,'Intermediate' from dual union all
select 39630,'039630','2014-015A','DMSP 5D-3 F-19','DMSP 5D-3 S-19',69719,'In Earth orbit','','2014-04-08 00:00:00',101.84,839,853,98.85,'Sun-Synch' from dual union all
select 39631,'039631','2014-015B','Centaur AV-044','Centaur AV-044',69719,'Deep Space','2014-04-03 00:00:00','2014-04-08 00:00:00',152.27,891,32540,98.8,'Earth Escape' from dual union all
select 39632,'039632','2014-015C','DMSP F-19 solar array band','',69719,'In Earth orbit','','2014-04-03 00:00:00',101.83,844,847,98.86,'Sun-Synch' from dual union all
select 39633,'039633','2014-015D','DMSP F-19 solar array band','',69719,'In Earth orbit','','2014-04-03 00:00:00',101.84,844,848,98.86,'Sun-Synch' from dual union all
select 39634,'039634','2014-016A','Sentinel-1A','Sentinel-1A',69720,'In Earth orbit','','2014-04-06 00:00:00',98.51,686,688,98.17,'Sun-Synch' from dual union all
select 39635,'039635','2014-017A','IRNSS-R1B','IRNSS-1B',69721,'In Earth orbit','','2014-04-15 00:00:00',1432.77,35570,35873,30.98,'Near-synch' from dual union all
select 39636,'039636','2014-017B','PSLV-C24 PS4','PS4',69721,'In Earth orbit','','2014-04-12 00:00:00',366.94,281,20919,19.1,'Highly Ellip' from dual union all
select 39637,'039637','1987-060E','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-08 00:00:00',100.6,775,799,65.01,'Intermediate' from dual union all
select 39638,'039638','1987-060F','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-08 00:00:00',100.63,775,801,65.01,'Intermediate' from dual union all
select 39639,'039639','1987-060G','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-07 00:00:00',100.63,776,800,65.01,'Intermediate' from dual union all
select 3964,'003964','1965-020EX','deb Blok Kosmos','',11770,'Reentered','1980-10-29 00:00:00','1980-10-25 00:00:00',91.48,308,387,55.88,'Intermediate' from dual union all
select 39640,'039640','1987-060H','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-06 00:00:00',100.58,775,797,65.01,'Intermediate' from dual union all
select 39641,'039641','1987-060J','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-08 00:00:00',100.63,776,800,65.01,'Intermediate' from dual union all
select 39642,'039642','1987-060K','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-07 00:00:00',100.64,777,801,65.01,'Intermediate' from dual union all
select 39643,'039643','1987-060L','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-08 00:00:00',100.6,775,799,65.01,'Intermediate' from dual union all
select 39644,'039644','1987-060M','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-08 00:00:00',100.58,775,797,65.01,'Intermediate' from dual union all
select 39645,'039645','1987-060N','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-08 00:00:00',100.62,775,801,65.01,'Intermediate' from dual union all
select 39646,'039646','1987-060P','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-08 00:00:00',100.62,776,800,65.01,'Intermediate' from dual union all
select 39647,'039647','1987-060Q','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-08 00:00:00',100.64,777,801,65.01,'Intermediate' from dual union all
select 39648,'039648','2014-018A','Progress M-23M','Progress 7K-TGM No.427',69722,'Deorbited','2014-07-31 00:00:00','2014-04-09 00:00:00',92.43,374,414,51.64,'Intermediate' from dual union all
select 39649,'039649','2014-018B','11S510','11S510',69722,'Reentered','2014-04-11 00:00:00','2014-04-10 00:00:00',88.12,171,191,51.63,'Intermediate' from dual union all
select 3965,'003965','1965-020EY','deb Blok Kosmos','',11770,'In Earth orbit','','1992-02-02 00:00:00',117.75,1361,1799,56.3,'Intermediate' from dual union all
select 39650,'039650','2014-019A','''Ofeq-10','TecSAR-3',69723,'In Earth orbit','','2014-04-16 00:00:00',94.54,384,609,140.95,'Retrograde' from dual union all
select 39651,'039651','2014-019B','AUS-51','AUS-51',69723,'Reentered','2014-05-30 00:00:00','2014-04-10 00:00:00',93.24,255,612,140.95,'Retrograde' from dual union all
select 39652,'039652','2014-020A','USA 250','SHARP 1',69724,'In Earth orbit','','2014-04-10 00:00:00',1428.23,35014,36251,7.7,'Inclined Drift' from dual union all
select 39653,'039653','2014-020B','Centaur AV-045','Centaur AV-045',69724,'In Earth orbit','','2014-10-20 00:00:00',858.68,7957,39139,11.44,'Highly Ellip' from dual union all
select 39654,'039654','1987-060R','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.63,776,800,65.01,'Intermediate' from dual union all
select 39655,'039655','1987-060S','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.62,775,801,65.01,'Intermediate' from dual union all
select 39656,'039656','1987-060T','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.62,775,801,65.01,'Intermediate' from dual union all
select 39657,'039657','1987-060U','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.61,775,800,65.01,'Intermediate' from dual union all
select 39658,'039658','1987-060V','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.6,774,800,65.01,'Intermediate' from dual union all
select 39659,'039659','1987-060W','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.6,775,799,65.01,'Intermediate' from dual union all
select 3966,'003966','1969-029AK','deb Blok-E','',18296,'Reentered','1969-09-10 00:00:00','1969-08-24 00:00:00',93.78,392,527,81.02,'Intermediate' from dual union all
select 39660,'039660','1987-060X','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.63,777,799,65.01,'Intermediate' from dual union all
select 39661,'039661','1987-060Y','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.61,775,800,65.01,'Intermediate' from dual union all
select 39662,'039662','1987-060Z','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.61,775,800,65.01,'Intermediate' from dual union all
select 39663,'039663','1987-060AA','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.62,775,801,65.01,'Intermediate' from dual union all
select 39664,'039664','1987-060AB','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.62,775,801,65.01,'Intermediate' from dual union all
select 39665,'039665','1987-060AC','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.62,776,800,65.01,'Intermediate' from dual union all
select 39666,'039666','1987-060AD','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.6,774,800,65.01,'Intermediate' from dual union all
select 39667,'039667','1987-060AE','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.6,775,799,65.01,'Intermediate' from dual union all
select 39668,'039668','1987-060AF','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.61,774,801,65.01,'Intermediate' from dual union all
select 39669,'039669','1987-060AG','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.61,774,801,65.02,'Intermediate' from dual union all
select 3967,'003967','1969-029AL','deb Blok-E','',18296,'Reentered','1969-06-26 00:00:00','1969-06-05 00:00:00',93.78,404,514,81.1,'Intermediate' from dual union all
select 39670,'039670','1987-060AH','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.61,774,801,65.01,'Intermediate' from dual union all
select 39671,'039671','1987-060AJ','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.61,774,801,65.01,'Intermediate' from dual union all
select 39672,'039672','1987-060AK','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.62,775,801,65.01,'Intermediate' from dual union all
select 39673,'039673','1987-060AL','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-12 00:00:00',100.62,775,801,65.01,'Intermediate' from dual union all
select 39674,'039674','1987-060AM','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.62,776,800,65.01,'Intermediate' from dual union all
select 39675,'039675','1987-060AN','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.61,775,800,65.01,'Intermediate' from dual union all
select 39676,'039676','1987-060AP','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.62,775,801,65.01,'Intermediate' from dual union all
select 39677,'039677','1987-060AQ','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2014-04-15 00:00:00',100.62,775,801,65.01,'Intermediate' from dual union all
select 39678,'039678','2014-021A','EgyptSat-2','559GK No. 1',69725,'In Earth orbit','','2014-04-18 00:00:00',96.04,435,703,51.62,'Intermediate' from dual union all
select 39679,'039679','2014-021B','11S510','11S510',69725,'In Earth orbit','','2014-04-19 00:00:00',95.96,434,696,51.62,'Intermediate' from dual union all
select 3968,'003968','61 Omicron 224','deb r/b Ablestar','',7273,'Reentered','1969-06-03 00:00:00','1969-06-01 00:00:00',89.63,200,312,65.62,'Intermediate' from dual union all
select 39680,'039680','2014-022A','Dragon CRS-3','Dragon C105',69726,'Landed','2014-05-18 00:00:00','2014-04-19 00:00:00',90.92,310,330,51.65,'Intermediate' from dual union all
select 39681,'039681','2014-022B','SporeSat','SporeSat',69726,'Reentered','2014-06-04 00:00:00','2014-04-21 00:00:00',90.71,294,325,51.65,'Intermediate' from dual union all
select 39682,'039682','2014-022C','TSAT','TestSat-Lite',69726,'Reentered','2014-05-28 00:00:00','2014-04-20 00:00:00',90.79,295,332,51.65,'Intermediate' from dual union all
select 39683,'039683','2014-022D','All-Star','ALL-STAR/THEIA',69726,'Reentered','2014-05-26 00:00:00','2014-04-21 00:00:00',90.8,299,329,51.65,'Intermediate' from dual union all
select 39684,'039684','2014-022E','PhoneSat 2.5','PhoneSat 2.5',69726,'Reentered','2014-05-15 00:00:00','2014-04-22 00:00:00',90.71,294,325,51.65,'Intermediate' from dual union all
select 39685,'039685','2014-022F','KickSat','KickSat',69726,'Reentered','2014-05-14 00:00:00','2014-04-21 00:00:00',90.74,296,326,51.64,'Intermediate' from dual union all
select 39686,'039686','2014-022G','Dragon solar array cover','',69726,'Reentered','2014-04-29 00:00:00','2014-04-22 00:00:00',90.49,291,306,51.66,'Intermediate' from dual union all
select 39687,'039687','2014-022H','Dragon solar array cover','',69726,'Reentered','2014-04-28 00:00:00','2014-04-22 00:00:00',90.39,284,304,51.64,'Intermediate' from dual union all
select 39688,'039688','1976-004E','CTS JBSA','Jettisonable Body S. Arr',38956,'In Earth orbit','','2014-04-17 00:00:00',1436.16,35660,35916,12.54,'Inclined GEO' from dual union all
select 39689,'039689','1976-004F','CTS JBSA','Jettisonable Body S. Arr',38956,'In Earth orbit','','2013-09-26 00:00:00',1436.62,35237,36357,12.77,'Inclined Drift' from dual union all
select 3969,'003969','61 Omicron 225','deb r/b Ablestar','',7273,'Reentered','1969-06-27 00:00:00','1969-06-08 00:00:00',91.95,357,384,64.96,'Intermediate' from dual union all
select 39690,'039690','1977-055C','deb Kosmos-921','',42183,'In Earth orbit','','2014-04-17 00:00:00',96.25,556,602,75.83,'Intermediate' from dual union all
select 39691,'039691','1991-009DE','deb S3M','',64178,'In Earth orbit','','2014-04-15 00:00:00',117.95,1471,1707,74.05,'Intermediate' from dual union all
select 39692,'039692','1991-009DF','deb S3M','',64178,'In Earth orbit','','2014-04-17 00:00:00',116.38,1440,1596,74.09,'Intermediate' from dual union all
select 39693,'039693','1991-009DG','deb S3M','',64178,'In Earth orbit','','2014-04-15 00:00:00',117.94,1484,1693,74.04,'Intermediate' from dual union all
select 39694,'039694','1991-009DH','deb S3M','',64178,'In Earth orbit','','2014-04-18 00:00:00',118.19,1497,1702,74.08,'Intermediate' from dual union all
select 39695,'039695','1991-009DJ','deb S3M','',64178,'In Earth orbit','','2014-04-01 00:00:00',118.19,1490,1710,74.06,'Intermediate' from dual union all
select 39696,'039696','1991-009DK','deb S3M','',64178,'In Earth orbit','','2014-04-18 00:00:00',117.92,1443,1732,74.02,'Intermediate' from dual union all
select 39697,'039697','1991-009DL','deb S3M','',64178,'In Earth orbit','','2014-04-16 00:00:00',117.98,1471,1709,74.07,'Intermediate' from dual union all
select 39698,'039698','1991-009DM','deb S3M','',64178,'In Earth orbit','','2014-04-18 00:00:00',117.97,1441,1739,74.05,'Intermediate' from dual union all
select 39699,'039699','1991-009DN','deb S3M','',64178,'In Earth orbit','','2014-04-17 00:00:00',118.35,1495,1719,74.09,'Intermediate' from dual;
insert into SATELLITE
select 397,'000397','62 A Psi 1','Tiros 6','Tiros F2',8152,'In Earth orbit','','1962-10-21 00:00:00',98.72,682,713,58.28,'Intermediate' from dual union all
select 3970,'003970','1966-013J','deb Rubis','',13420,'Reentered','1981-05-28 00:00:00','1967-07-26 00:00:00',118.11,505,2687,34.08,'Intermediate' from dual union all
select 39700,'039700','1991-009DP','deb S3M','',64178,'In Earth orbit','','2014-04-17 00:00:00',117.26,1461,1655,73.97,'Intermediate' from dual union all
select 39701,'039701','1991-009DQ','deb S3M','',64178,'In Earth orbit','','2014-04-18 00:00:00',117.58,1464,1681,73.99,'Intermediate' from dual union all
select 39702,'039702','1991-009DR','deb S3M','',64178,'In Earth orbit','','2014-04-18 00:00:00',118.53,1465,1765,74.23,'Intermediate' from dual union all
select 39703,'039703','1991-009DS','deb S3M','',64178,'In Earth orbit','','2014-04-17 00:00:00',117.39,1416,1712,73.83,'Intermediate' from dual union all
select 39704,'039704','1992-060F','deb Ariane V53','',64772,'In Earth orbit','','2014-04-10 00:00:00',683.38,1002,37642,8.02,'GEO Transfer' from dual union all
select 39705,'039705','1992-060G','deb Ariane V53','',64772,'In Earth orbit','','2014-04-10 00:00:00',679.41,922,37523,7.63,'GEO Transfer' from dual union all
select 39706,'039706','1992-060H','deb Ariane V53','',64772,'In Earth orbit','','2014-04-17 00:00:00',667.09,1093,36731,7.2,'GEO Transfer' from dual union all
select 39707,'039707','1992-093JX','deb 11S772','',64805,'In Earth orbit','','2014-04-18 00:00:00',101.96,773,930,71.1,'Intermediate' from dual union all
select 39708,'039708','1992-093JY','deb 11S772','',64805,'In Earth orbit','','2014-04-18 00:00:00',98.1,603,732,71.29,'Intermediate' from dual union all
select 39709,'039709','1992-093JZ','deb 11S772','',64805,'In Earth orbit','','2014-04-18 00:00:00',103.55,818,1035,71.15,'Intermediate' from dual union all
select 3971,'003971','1969-048A','Kosmos-284','Zenit-4 No. 62',18315,'Landed','1969-06-06 00:00:00','1969-05-31 00:00:00',89.49,205,294,51.75,'Intermediate' from dual union all
select 39710,'039710','1992-093KA','deb 11S772','',64805,'In Earth orbit','','2014-04-06 00:00:00',102.41,808,938,71.21,'Intermediate' from dual union all
select 39711,'039711','1992-093KB','deb 11S772','',64805,'In Earth orbit','','2014-04-18 00:00:00',102.89,824,967,70.84,'Intermediate' from dual union all
select 39712,'039712','1992-093KC','deb 11S772','',64805,'In Earth orbit','','2014-04-17 00:00:00',102.55,813,946,71.11,'Intermediate' from dual union all
select 39713,'039713','1992-093KD','deb 11S772','',64805,'In Earth orbit','','2014-04-14 00:00:00',102.43,806,941,71.11,'Intermediate' from dual union all
select 39714,'039714','1992-093KE','deb 11S772','',64805,'In Earth orbit','','2014-04-16 00:00:00',101.32,750,893,71.09,'Intermediate' from dual union all
select 39715,'039715','1992-093KF','deb 11S772','',64805,'In Earth orbit','','2014-04-15 00:00:00',101.02,804,810,71.02,'Intermediate' from dual union all
select 39716,'039716','1992-093KG','deb 11S772','',64805,'In Earth orbit','','2014-03-22 00:00:00',102.67,812,958,71.15,'Intermediate' from dual union all
select 39717,'039717','1992-093KH','deb 11S772','',64805,'In Earth orbit','','2014-04-14 00:00:00',102.96,822,975,70.69,'Intermediate' from dual union all
select 39718,'039718','1992-093KJ','deb 11S772','',64805,'In Earth orbit','','2014-04-18 00:00:00',100.58,759,813,71.2,'Intermediate' from dual union all
select 39719,'039719','1992-093KK','deb 11S772','',64805,'In Earth orbit','','2014-04-18 00:00:00',103.19,838,981,71.3,'Intermediate' from dual union all
select 3972,'003972','1969-048B','11A57I','11A57I',18315,'Reentered','1969-06-04 00:00:00','1969-05-31 00:00:00',89.17,193,273,51.76,'Intermediate' from dual union all
select 39720,'039720','1992-093KL','deb 11S772','',64805,'In Earth orbit','','2014-04-18 00:00:00',99.89,726,781,70.79,'Intermediate' from dual union all
select 39721,'039721','1992-093KM','deb 11S772','',64805,'In Earth orbit','','2014-04-17 00:00:00',103.33,823,1010,71.19,'Intermediate' from dual union all
select 39722,'039722','1992-093KN','deb 11S772','',64805,'In Earth orbit','','2014-04-18 00:00:00',102.97,819,980,70.89,'Intermediate' from dual union all
select 39723,'039723','1992-093KP','deb 11S772','',64805,'In Earth orbit','','2014-02-22 00:00:00',101.78,798,888,71.1,'Intermediate' from dual union all
select 39724,'039724','1992-093KQ','deb 11S772','',64805,'In Earth orbit','','2014-04-18 00:00:00',102.88,830,959,70.85,'Intermediate' from dual union all
select 39725,'039725','1992-093KR','deb 11S772','',64805,'In Earth orbit','','2014-04-16 00:00:00',102.38,802,941,71.18,'Intermediate' from dual union all
select 39726,'039726','1992-093KS','deb 11S772','',64805,'In Earth orbit','','2014-04-18 00:00:00',103.57,839,1016,71.16,'Intermediate' from dual union all
select 39727,'039727','2014-023A','Luch-5V','Luch-5V',69727,'In Earth orbit','','2014-05-06 00:00:00',1425.5,35432,35726,4.82,'Inclined Drift' from dual union all
select 39728,'039728','2014-023B','Kazsat-3','Kazsat-3',69727,'In Earth orbit','','2014-05-06 00:00:00',1429.63,35442,35878,.21,'Drift GEO' from dual union all
select 39729,'039729','2014-023C','Briz-M No. 99548','Briz-M 14S43 No. 99548',69727,'In Earth orbit','','2014-05-05 00:00:00',1555.83,35598,40600,1.13,'Very High' from dual union all
select 3973,'003973','1969-048C','deb Kosmos-284','',18315,'Reentered','1969-05-30 00:00:00','1969-05-29 00:00:00',88.92,193,250,51.76,'Intermediate' from dual union all
select 39730,'039730','2014-023D','Briz-M DTB','Briz-M torus tank',69727,'In Earth orbit','','2014-05-06 00:00:00',633.22,367,35730,47.85,'GEO Transfer' from dual union all
select 39731,'039731','2014-024A','KazEOSat-1','DZZ-HR',69728,'In Earth orbit','','2014-04-30 00:00:00',99.59,737,741,98.56,'Sun-Synch' from dual union all
select 39732,'039732','2014-025A','Kosmos-2495','Kobal''t-M No. 564',69729,'Landed','2014-09-03 00:00:00','2014-05-06 00:00:00',89.1,176,284,81.41,'Intermediate' from dual union all
select 39733,'039733','2014-025B','Blok-I 14S54/RD-0110','374BL02A',69729,'Reentered','2014-05-10 00:00:00','2014-05-08 00:00:00',88.48,162,237,81.41,'Intermediate' from dual union all
select 39734,'039734','1999-008E','deb Delta 267','',66689,'Reentered','2014-05-06 00:00:00','2014-05-05 00:00:00',94.6,415,583,96.25,'Sun-Synch' from dual union all
select 39735,'039735','1999-008F','deb Delta 267','',66689,'Reentered','2014-05-03 00:00:00','2014-05-03 00:00:00',94.41,424,556,96.34,'Sun-Synch' from dual union all
select 39736,'039736','1999-008G','deb Delta 267','',66689,'Reentered','2014-05-11 00:00:00','2014-05-10 00:00:00',96.55,487,700,96.29,'Sun-Synch' from dual union all
select 39737,'039737','1999-008H','deb Delta 267','',66689,'Reentered','2014-05-05 00:00:00','2014-05-04 00:00:00',97.53,507,773,96.36,'Sun-Synch' from dual union all
select 39738,'039738','1999-008J','deb Delta 267','',66689,'Reentered','2014-05-13 00:00:00','2014-05-12 00:00:00',94.85,430,592,96.3,'Sun-Synch' from dual union all
select 39739,'039739','1999-008K','deb Delta 267','',66689,'Reentered','2014-05-13 00:00:00','2014-05-09 00:00:00',97.57,532,753,96.32,'Sun-Synch' from dual union all
select 3974,'003974','1968-069E','ESSA 9 despin weight','',16796,'In Earth orbit','','1969-05-30 00:00:00',114.93,1425,1480,102.03,'Sun-Synch' from dual union all
select 39740,'039740','1999-008L','deb Delta 267','',66689,'Reentered','2014-05-16 00:00:00','2014-05-12 00:00:00',98.1,555,781,96.38,'Sun-Synch' from dual union all
select 39741,'039741','2014-026A','GPS IIF-6','GPS SVN 67',69730,'In Earth orbit','','2014-05-20 00:00:00',729.53,20458,20475,55.04,'Medium' from dual union all
select 39742,'039742','2014-026B','Delta 4 Second Stage','DCSS-4',69730,'In Earth orbit','','2014-05-20 00:00:00',728.86,20400,20500,55,'Medium' from dual union all
select 39743,'039743','2007-029G','deb Kosmos-2428','deb Tselina-2',68371,'Reentered','2014-05-17 00:00:00','2014-05-17 00:00:00',96.92,475,748,70.73,'Intermediate' from dual union all
select 39744,'039744','2007-029H','deb Kosmos-2428','deb Tselina-2',68371,'Reentered','2014-06-06 00:00:00','2014-05-18 00:00:00',101.07,713,906,70.88,'Intermediate' from dual union all
select 39745,'039745','2007-029J','deb Kosmos-2428','deb Tselina-2',68371,'Reentered','2014-06-07 00:00:00','2014-05-18 00:00:00',100.54,710,858,70.8,'Intermediate' from dual union all
select 39746,'039746','2007-029K','deb Kosmos-2428','deb Tselina-2',68371,'Reentered','2014-06-04 00:00:00','2014-05-18 00:00:00',101.03,718,897,70.88,'Intermediate' from dual union all
select 39747,'039747','2007-029L','deb Kosmos-2428','deb Tselina-2',68371,'Reentered','2014-05-19 00:00:00','2014-05-18 00:00:00',97.72,508,792,70.66,'Intermediate' from dual union all
select 39748,'039748','2007-029M','deb Kosmos-2428','deb Tselina-2',68371,'Reentered','2014-09-23 00:00:00','2014-05-18 00:00:00',102.21,843,884,70.97,'Intermediate' from dual union all
select 39749,'039749','2007-029N','deb Kosmos-2428','deb Tselina-2',68371,'Reentered','2014-05-17 00:00:00','2014-05-17 00:00:00',94.53,422,569,70.73,'Intermediate' from dual union all
select 3975,'003975','1968-069F','ESSA 9 despin weight','',16796,'In Earth orbit','','1969-03-31 00:00:00',114.92,1417,1487,101.58,'Sun-Synch' from dual union all
select 39750,'039750','2007-029P','deb Kosmos-2428','deb Tselina-2',68371,'Reentered','2014-05-23 00:00:00','2014-05-18 00:00:00',100.03,629,890,70.82,'Intermediate' from dual union all
select 39751,'039751','2014-027A','USA 252','QUASAR 18',69731,'In Earth orbit','','2015-03-15 00:00:00',1436.09,35769,35804,4.33,'Inclined GEO' from dual union all
select 39752,'039752','1976-105Q','deb Kosmos-862 baffle','',39057,'Reentered','2014-06-06 00:00:00','2014-06-07 00:00:00',117.78,109,3054,61.89,'Intermediate' from dual union all
select 39753,'039753','1976-105R','deb Kosmos-862 baffle','',39057,'Reentered','2014-05-22 00:00:00','2014-06-07 00:00:00',108.06,119,2155,61.74,'Intermediate' from dual union all
select 39754,'039754','2014-026C','deb Delta 4','',69730,'Reentered','2014-05-27 00:00:00','2014-05-22 00:00:00',158.62,210,6447,52.76,'Medium' from dual union all
select 39755,'039755','2014-026D','deb Delta 4','',69730,'Reentered','2014-06-02 00:00:00','2014-05-22 00:00:00',182.36,205,8344,52.69,'Medium' from dual union all
select 39756,'039756','2014-026E','deb Delta 4','',69730,'Reentered','2014-05-30 00:00:00','2014-05-22 00:00:00',134.28,204,4413,52.66,'Medium' from dual union all
select 39757,'039757','2014-026F','deb Delta 4','',69730,'Reentered','2014-05-21 00:00:00','2014-05-20 00:00:00',95.4,185,891,52.58,'Intermediate' from dual union all
select 39758,'039758','2014-026G','deb Delta 4','',69730,'Reentered','2014-05-24 00:00:00','2014-05-22 00:00:00',130.79,207,4107,52.62,'Medium' from dual union all
select 39759,'039759','2014-026H','deb Delta 4','',69730,'Reentered','2014-05-23 00:00:00','2014-05-22 00:00:00',104.84,197,1777,52.57,'Intermediate' from dual union all
select 3976,'003976','1969-021W','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1970-05-16 00:00:00','1969-05-29 00:00:00',98.39,518,846,72.32,'Intermediate' from dual union all
select 39760,'039760','2014-026J','deb Delta 4','',69730,'Reentered','2014-05-22 00:00:00','2014-05-21 00:00:00',97.79,196,1110,52.58,'Intermediate' from dual union all
select 39761,'039761','2014-028A','Kosmos-2496','Rodnik-S',69732,'In Earth orbit','','2014-05-30 00:00:00',115.86,1479,1510,82.45,'Intermediate' from dual union all
select 39762,'039762','2014-028B','Kosmos-2497','Rodnik-S',69732,'In Earth orbit','','2014-05-30 00:00:00',115.84,1478,1509,82.45,'Intermediate' from dual union all
select 39763,'039763','2014-028C','Kosmos-2498','Rodnik-S',69732,'In Earth orbit','','2014-05-30 00:00:00',115.81,1477,1507,82.45,'Intermediate' from dual union all
select 39764,'039764','2014-028D','Briz-KM No. 72521','Briz-KM 14S45',69732,'In Earth orbit','','2014-05-25 00:00:00',112.18,1151,1502,82.44,'Intermediate' from dual union all
select 39765,'039765','2014-028E','Kosmos-2499','Reshetnev Tech 14F1543 N',69732,'In Earth orbit','','2014-05-30 00:00:00',115.85,1479,1508,82.45,'Intermediate' from dual union all
select 39766,'039766','2014-029A','Daichi-2','ALOS-2',69733,'In Earth orbit','','2014-05-26 00:00:00',97.36,631,634,97.88,'Sun-Synch' from dual union all
select 39767,'039767','2014-029B','UNIFORM-1','UNIFORM-1',69733,'In Earth orbit','','2014-05-26 00:00:00',97.15,617,628,97.87,'Sun-Synch' from dual union all
select 39768,'039768','2014-029C','SOCRATES','SOCRATES',69733,'In Earth orbit','','2014-05-26 00:00:00',97.25,625,630,97.88,'Sun-Synch' from dual union all
select 39769,'039769','2014-029D','Raijin-2','RISING-2',69733,'In Earth orbit','','2014-05-26 00:00:00',97.2,621,629,97.87,'Sun-Synch' from dual union all
select 3977,'003977','1969-021X','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1969-08-18 00:00:00','1969-05-29 00:00:00',93.54,447,449,74.01,'Intermediate' from dual union all
select 39770,'039770','2014-029E','SPROUT','SPROUT',69733,'In Earth orbit','','2014-05-26 00:00:00',97.11,613,628,97.87,'Sun-Synch' from dual union all
select 39771,'039771','2014-029F','H-2A F24 Stage 2','H-2A Stage 2',69733,'In Earth orbit','','2014-05-26 00:00:00',96.63,586,609,98.11,'Sun-Synch' from dual union all
select 39772,'039772','2014-029G','deb H-2A','',69733,'Reentered','2014-07-09 00:00:00','2014-05-27 00:00:00',96.61,590,603,98.1,'Sun-Synch' from dual union all
select 39773,'039773','2014-030A','EUTELSAT 3B','EUTELSAT 3B',69734,'In Earth orbit','','2014-05-29 00:00:00',957.32,15980,35616,.08,'Medium' from dual union all
select 39774,'039774','2014-030B','Blok DM-SL No. 36L','Blok DM-SL',69734,'In Earth orbit','','2014-05-30 00:00:00',629.97,386,35544,.02,'GEO Transfer' from dual union all
select 39775,'039775','2014-031A','Soyuz TMA-13M','Soyuz 11F732A47 No. 713',69735,'Landed','2014-11-10 00:00:00','2014-05-28 00:00:00',88.73,197,227,51.63,'Intermediate' from dual union all
select 39776,'039776','2014-031B','11S510','11S510',69735,'Reentered','2014-05-31 00:00:00','2014-05-30 00:00:00',88.3,182,199,51.62,'Intermediate' from dual union all
select 39777,'039777','1997-051AAJ','deb Iridium 33','',66131,'In Earth orbit','','2014-06-01 00:00:00',101.39,757,891,86,'Polar' from dual union all
select 39778,'039778','1997-051AAK','deb Iridium 33','',66131,'In Earth orbit','','2014-05-31 00:00:00',100.25,764,777,86.39,'Polar' from dual union all
select 39779,'039779','1997-051AAL','deb Iridium 33','',66131,'In Earth orbit','','2014-05-24 00:00:00',100.89,742,860,86.3,'Polar' from dual union all
select 3978,'003978','1965-082SE','deb Transtage 4','',11832,'In Earth orbit','','1992-02-03 00:00:00',98.93,676,739,32.57,'Intermediate' from dual union all
select 39780,'039780','1997-051AAM','deb Iridium 33','',66131,'Reentered','2014-11-30 00:00:00','2014-05-31 00:00:00',97.81,627,680,86.15,'Polar' from dual union all
select 39781,'039781','1997-051AAN','deb Iridium 33','',66131,'In Earth orbit','','2014-06-02 00:00:00',104.44,740,1197,86.39,'Polar' from dual union all
select 39782,'039782','1997-051AAP','deb Iridium 33','',66131,'In Earth orbit','','2014-06-01 00:00:00',99.21,711,731,86.43,'Polar' from dual union all
select 39783,'039783','1997-051AAQ','deb Iridium 33','',66131,'In Earth orbit','','2014-06-01 00:00:00',100.77,740,851,86.19,'Polar' from dual union all
select 39784,'039784','1997-051AAR','deb Iridium 33','',66131,'In Earth orbit','','2014-05-27 00:00:00',100.58,712,861,86.38,'Polar' from dual union all
select 39785,'039785','1997-051AAS','deb Iridium 33','',66131,'In Earth orbit','','2014-05-25 00:00:00',99.5,698,772,86.4,'Polar' from dual union all
select 39786,'039786','1997-051AAT','deb Iridium 33','',66131,'In Earth orbit','','2014-06-02 00:00:00',99.47,723,744,86.39,'Polar' from dual union all
select 39787,'039787','1997-051AAU','deb Iridium 33','',66131,'In Earth orbit','','2014-05-31 00:00:00',100.02,725,793,86.21,'Polar' from dual union all
select 39788,'039788','1997-051AAV','deb Iridium 33','',66131,'In Earth orbit','','2014-05-24 00:00:00',101.78,692,993,86.35,'Polar' from dual union all
select 39789,'039789','1997-051AAW','deb Iridium 33','',66131,'Reentered','2016-12-30 00:00:00','2014-05-29 00:00:00',98.71,665,729,86.53,'Polar' from dual;
insert into SATELLITE
select 3979,'003979','1968-097BV','deb Kosmos-252','',16824,'Reentered','1971-07-02 00:00:00','1969-06-01 00:00:00',115.5,780,2177,62.32,'Intermediate' from dual union all
select 39790,'039790','1997-051AAX','deb Iridium 33','',66131,'In Earth orbit','','2014-05-31 00:00:00',99.63,722,760,86.26,'Polar' from dual union all
select 39791,'039791','1997-051AAY','deb Iridium 33','',66131,'In Earth orbit','','2014-05-29 00:00:00',102.73,720,1056,86.39,'Polar' from dual union all
select 39792,'039792','1997-051AAZ','deb Iridium 33','',66131,'In Earth orbit','','2014-05-31 00:00:00',103.15,766,1050,86.44,'Polar' from dual union all
select 39793,'039793','1997-051ABA','deb Iridium 33','',66131,'In Earth orbit','','2014-06-01 00:00:00',98.8,699,704,86.38,'Polar' from dual union all
select 39794,'039794','1997-051ABB','deb Iridium 33','',66131,'In Earth orbit','','2014-06-02 00:00:00',98.93,682,733,86.36,'Polar' from dual union all
select 39795,'039795','1997-051ABC','deb Iridium 33','',66131,'Reentered','2015-07-22 00:00:00','2014-06-02 00:00:00',98.37,668,694,86.37,'Polar' from dual union all
select 39796,'039796','1997-051ABD','deb Iridium 33','',66131,'In Earth orbit','','2014-05-29 00:00:00',100.14,701,830,86.37,'Polar' from dual union all
select 39797,'039797','2008-046J','deb SOZ','',68554,'Reentered','2018-03-17 00:00:00','2014-06-03 00:00:00',329.87,452,18421,65.44,'Highly Ellip' from dual union all
select 39798,'039798','2008-046K','deb SOZ','',68554,'In Earth orbit','','2014-06-04 00:00:00',340.47,938,18609,65.01,'Highly Ellip' from dual union all
select 39799,'039799','2008-046L','deb SOZ','',68554,'In Earth orbit','','2014-06-04 00:00:00',341.74,894,18733,65.29,'Highly Ellip' from dual union all
select 398,'000398','62 A Psi 2','Altair','Altair X-248-A5',8152,'Reentered','1983-01-10 00:00:00','1962-10-19 00:00:00',98.67,684,707,58.29,'Intermediate' from dual union all
select 3980,'003980','1965-082SF','deb Transtage 4','',11832,'Reentered','1970-04-24 00:00:00','1970-04-11 00:00:00',93.06,371,478,30.08,'Intermediate' from dual union all
select 39800,'039800','2008-046M','deb SOZ','',68554,'In Earth orbit','','2014-06-04 00:00:00',341.21,769,18825,65.31,'Highly Ellip' from dual union all
select 39801,'039801','2008-046N','deb SOZ','',68554,'In Earth orbit','','2014-06-04 00:00:00',326.82,618,18060,65.22,'Highly Ellip' from dual union all
select 39802,'039802','2008-046P','deb SOZ','',68554,'In Earth orbit','','2014-06-03 00:00:00',332.44,588,18449,65.34,'Highly Ellip' from dual union all
select 39803,'039803','2008-046Q','deb SOZ','',68554,'In Earth orbit','','2014-06-04 00:00:00',340.82,977,18592,65.04,'Highly Ellip' from dual union all
select 39804,'039804','2008-046R','deb SOZ','',68554,'In Earth orbit','','2014-06-04 00:00:00',334.07,784,18357,66.13,'Highly Ellip' from dual union all
select 39805,'039805','1987-020ED','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-03 00:00:00',115.87,1478,1512,73.59,'Intermediate' from dual union all
select 39806,'039806','1987-020EE','deb Kosmos-1823','',60110,'In Earth orbit','','2014-05-22 00:00:00',116.29,1456,1571,73.64,'Intermediate' from dual union all
select 39807,'039807','1987-020EF','deb Kosmos-1823','',60110,'In Earth orbit','','2014-05-15 00:00:00',104.51,955,988,74.02,'Intermediate' from dual union all
select 39808,'039808','1987-020EG','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-05 00:00:00',115.85,1454,1533,73.62,'Intermediate' from dual union all
select 39809,'039809','1987-020EH','deb Kosmos-1823','',60110,'In Earth orbit','','2012-05-31 00:00:00',114.44,1327,1533,73.72,'Intermediate' from dual union all
select 3981,'003981','1965-082SG','deb Transtage 4','',11832,'Reentered','1970-02-19 00:00:00','1969-05-29 00:00:00',96.43,569,607,32.03,'Intermediate' from dual union all
select 39810,'039810','1987-020EJ','deb Kosmos-1823','',60110,'In Earth orbit','','2014-05-20 00:00:00',116.01,1475,1528,73.7,'Intermediate' from dual union all
select 39811,'039811','1987-020EK','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-01 00:00:00',115.66,1459,1511,73.63,'Intermediate' from dual union all
select 39812,'039812','1987-020EL','deb Kosmos-1823','',60110,'In Earth orbit','','2014-05-19 00:00:00',116.17,1476,1541,73.62,'Intermediate' from dual union all
select 39813,'039813','1987-020EM','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-06 00:00:00',115.5,1453,1504,73.64,'Intermediate' from dual union all
select 39814,'039814','1987-020EN','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-05 00:00:00',115.87,1473,1517,73.61,'Intermediate' from dual union all
select 39815,'039815','1987-020EP','deb Kosmos-1823','',60110,'In Earth orbit','','2014-05-10 00:00:00',115.96,1477,1521,73.59,'Intermediate' from dual union all
select 39816,'039816','1987-020EQ','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-06 00:00:00',97.21,598,653,74.04,'Intermediate' from dual union all
select 39817,'039817','1987-020ER','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-06 00:00:00',115.13,1444,1479,73.72,'Intermediate' from dual union all
select 39818,'039818','1987-020ES','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-06 00:00:00',115.35,1452,1491,73.62,'Intermediate' from dual union all
select 39819,'039819','1987-020ET','deb Kosmos-1823','',60110,'In Earth orbit','','2014-05-29 00:00:00',115.85,1474,1514,73.68,'Intermediate' from dual union all
select 3982,'003982','1965-082SH','deb Transtage 4','',11832,'Reentered','1969-10-25 00:00:00','1969-09-09 00:00:00',100.23,611,927,32.71,'Intermediate' from dual union all
select 39820,'039820','1987-020EU','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-03 00:00:00',115.67,1468,1503,73.63,'Intermediate' from dual union all
select 39821,'039821','1987-020EV','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-03 00:00:00',113.59,1316,1467,73.24,'Intermediate' from dual union all
select 39822,'039822','1987-020EW','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-06 00:00:00',117.2,1465,1645,74.02,'Intermediate' from dual union all
select 39823,'039823','1987-020EX','deb Kosmos-1823','',60110,'In Earth orbit','','2014-05-17 00:00:00',115.79,1457,1526,73.71,'Intermediate' from dual union all
select 39824,'039824','1987-020EY','deb Kosmos-1823','',60110,'In Earth orbit','','2013-03-30 00:00:00',115.69,1461,1513,73.64,'Intermediate' from dual union all
select 39825,'039825','1987-020EZ','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-06 00:00:00',115.87,1475,1515,73.62,'Intermediate' from dual union all
select 39826,'039826','1987-020FA','deb Kosmos-1823','',60110,'In Earth orbit','','2013-03-09 00:00:00',117.89,1453,1720,73.51,'Intermediate' from dual union all
select 39827,'039827','1987-020FB','deb Kosmos-1823','',60110,'In Earth orbit','','2014-05-27 00:00:00',115.96,1477,1521,73.59,'Intermediate' from dual union all
select 39828,'039828','1987-020FC','deb Kosmos-1823','',60110,'In Earth orbit','','2014-05-28 00:00:00',119.35,1501,1802,74.3,'Intermediate' from dual union all
select 39829,'039829','1987-020FD','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-04 00:00:00',115.93,1474,1522,73.64,'Intermediate' from dual union all
select 3983,'003983','1969-049A','Kosmos-285','DS-P1-Yu No. 24',18316,'Reentered','1969-10-07 00:00:00','1969-07-04 00:00:00',91.63,257,452,71.02,'Intermediate' from dual union all
select 39830,'039830','1987-020FE','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-06 00:00:00',115.58,1461,1503,73.6,'Intermediate' from dual union all
select 39831,'039831','1987-020FF','deb Kosmos-1823','',60110,'In Earth orbit','','2012-05-31 00:00:00',115.89,1476,1515,73.63,'Intermediate' from dual union all
select 39832,'039832','1987-020FG','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-06 00:00:00',116.09,1476,1534,73.61,'Intermediate' from dual union all
select 39833,'039833','1987-020FH','deb Kosmos-1823','',60110,'In Earth orbit','','2014-06-04 00:00:00',114.56,1416,1455,73.78,'Intermediate' from dual union all
select 39834,'039834','1987-020FJ','deb Kosmos-1823','',60110,'In Earth orbit','','2014-03-15 00:00:00',116.31,1477,1552,73.6,'Intermediate' from dual union all
select 39835,'039835','1991-009DT','deb S3M','',64178,'In Earth orbit','','2014-05-31 00:00:00',117.83,1430,1737,74.05,'Intermediate' from dual union all
select 39836,'039836','1991-009DU','deb S3M','',64178,'In Earth orbit','','2014-05-24 00:00:00',117.13,1444,1660,73.91,'Intermediate' from dual union all
select 39837,'039837','1991-009DV','deb S3M','',64178,'In Earth orbit','','2014-06-05 00:00:00',111.22,1257,1309,73.82,'Intermediate' from dual union all
select 39838,'039838','1991-009DW','deb S3M','',64178,'In Earth orbit','','2014-04-21 00:00:00',118.99,1453,1819,74.12,'Intermediate' from dual union all
select 39839,'039839','1991-009DX','deb S3M','',64178,'In Earth orbit','','2014-06-03 00:00:00',115.24,1395,1538,73.83,'Intermediate' from dual union all
select 3984,'003984','1969-050A','OPS 1077','GAMBIT-3 Mission 4322',18317,'Reentered Attached','1969-06-14 00:00:00','1969-06-05 00:00:00',89.97,131,415,109.96,'Retrograde' from dual union all
select 39840,'039840','1991-009DY','deb S3M','',64178,'In Earth orbit','','2014-05-19 00:00:00',107.76,1014,1233,73.85,'Intermediate' from dual union all
select 39841,'039841','1992-093KT','deb 11S772','',64805,'In Earth orbit','','2014-06-03 00:00:00',97.97,617,707,70.77,'Intermediate' from dual union all
select 39842,'039842','1992-093KU','deb 11S772','',64805,'In Earth orbit','','2014-05-25 00:00:00',99.27,679,769,70.97,'Intermediate' from dual union all
select 39843,'039843','1992-093KV','deb 11S772','',64805,'In Earth orbit','','2014-06-06 00:00:00',99.53,697,776,70.92,'Intermediate' from dual union all
select 39844,'039844','1992-093KW','deb 11S772','',64805,'In Earth orbit','','2014-06-02 00:00:00',100.94,768,839,70.66,'Intermediate' from dual union all
select 39845,'039845','1992-093KX','deb 11S772','',64805,'In Earth orbit','','2014-06-05 00:00:00',102.73,788,987,70.98,'Intermediate' from dual union all
select 39846,'039846','1992-093KY','deb 11S772','',64805,'In Earth orbit','','2014-04-18 00:00:00',101.53,758,905,71.28,'Intermediate' from dual union all
select 39847,'039847','1992-093KZ','deb 11S772','',64805,'In Earth orbit','','2014-05-28 00:00:00',100.64,743,835,71.04,'Intermediate' from dual union all
select 39848,'039848','1992-093LA','deb 11S772','',64805,'In Earth orbit','','2014-04-22 00:00:00',100.8,751,842,71.23,'Intermediate' from dual union all
select 39849,'039849','1992-093LB','deb 11S772','',64805,'In Earth orbit','','2014-06-01 00:00:00',102.72,821,954,70.87,'Intermediate' from dual union all
select 3985,'003985','1969-049B','S1M','S1M',18316,'Reentered','1969-08-05 00:00:00','1969-07-03 00:00:00',90.91,252,386,71.03,'Intermediate' from dual union all
select 39850,'039850','1992-093LC','deb 11S772','',64805,'In Earth orbit','','2014-02-13 00:00:00',102.76,817,962,71.33,'Intermediate' from dual union all
select 39851,'039851','1992-093LD','deb 11S772','',64805,'In Earth orbit','','2013-06-02 00:00:00',102.27,747,986,71.12,'Intermediate' from dual union all
select 39852,'039852','1992-093LE','deb 11S772','',64805,'In Earth orbit','','2014-06-05 00:00:00',100.23,765,773,71.01,'Intermediate' from dual union all
select 39853,'039853','1992-093LF','deb 11S772','',64805,'In Earth orbit','','2014-06-05 00:00:00',97.81,599,709,71.29,'Intermediate' from dual union all
select 39854,'039854','1992-093LG','deb 11S772','',64805,'In Earth orbit','','2014-06-06 00:00:00',102.2,803,923,71,'Intermediate' from dual union all
select 39855,'039855','1992-093LH','deb 11S772','',64805,'In Earth orbit','','2014-06-06 00:00:00',101.15,724,903,70.53,'Intermediate' from dual union all
select 39856,'039856','1992-093LJ','deb 11S772','',64805,'In Earth orbit','','2014-02-19 00:00:00',99.73,715,777,70.82,'Intermediate' from dual union all
select 39857,'039857','1992-093LK','deb 11S772','',64805,'In Earth orbit','','2014-06-06 00:00:00',101.79,797,890,70.92,'Intermediate' from dual union all
select 39858,'039858','1992-093LL','deb 11S772','',64805,'In Earth orbit','','2014-06-05 00:00:00',99.97,714,801,70.95,'Intermediate' from dual union all
select 39859,'039859','1992-093LM','deb 11S772','',64805,'Reentered','2014-10-19 00:00:00','2014-02-02 00:00:00',96.51,588,595,70.98,'Intermediate' from dual union all
select 3986,'003986','1969-051A','OGO 6','OGO F',18318,'Reentered','1979-10-12 00:00:00','1969-07-09 00:00:00',99.62,396,1085,82,'Intermediate' from dual union all
select 39860,'039860','1992-093LN','deb 11S772','',64805,'In Earth orbit','','2014-06-05 00:00:00',101.19,793,837,70.79,'Intermediate' from dual union all
select 39861,'039861','1992-093LP','deb 11S772','',64805,'Reentered','2015-12-16 00:00:00','2014-06-03 00:00:00',95.22,496,563,71.25,'Intermediate' from dual union all
select 39862,'039862','1992-093LQ','deb 11S772','',64805,'In Earth orbit','','2013-02-25 00:00:00',99.43,663,800,70.84,'Intermediate' from dual union all
select 39863,'039863','1992-093LR','deb 11S772','',64805,'In Earth orbit','','2014-06-06 00:00:00',102.89,771,1020,70.99,'Intermediate' from dual union all
select 39864,'039864','1992-093LS','deb 11S772','',64805,'In Earth orbit','','2014-06-06 00:00:00',102.44,790,959,71.15,'Intermediate' from dual union all
select 39865,'039865','1992-093LT','deb 11S772','',64805,'In Earth orbit','','2014-06-04 00:00:00',100.8,794,799,71.02,'Intermediate' from dual union all
select 39866,'039866','1992-093LU','deb 11S772','',64805,'In Earth orbit','','2014-06-05 00:00:00',101.5,749,911,71.29,'Intermediate' from dual union all
select 39867,'039867','1992-093LV','deb 11S772','',64805,'In Earth orbit','','2014-06-04 00:00:00',103.29,779,1049,70.97,'Intermediate' from dual union all
select 39868,'039868','1993-016AS','deb 11S772','',65118,'In Earth orbit','','2014-04-18 00:00:00',102.51,761,994,70.98,'Intermediate' from dual union all
select 39869,'039869','1993-016AT','deb 11S772','',65118,'In Earth orbit','','2014-06-01 00:00:00',101.92,753,946,71,'Intermediate' from dual union all
select 3987,'003987','1969-051B','Agena D 6803','Agena D',18318,'Reentered','1980-06-12 00:00:00','1969-07-09 00:00:00',99.55,395,1080,82.01,'Intermediate' from dual union all
select 39870,'039870','1993-016AU','deb 11S772','',65118,'In Earth orbit','','2014-04-25 00:00:00',102.09,752,964,70.98,'Intermediate' from dual union all
select 39871,'039871','1993-016AV','deb 11S772','',65118,'In Earth orbit','','2014-06-06 00:00:00',97.35,573,690,71.03,'Intermediate' from dual union all
select 39872,'039872','1993-016AW','deb 11S772','',65118,'In Earth orbit','','2014-06-05 00:00:00',103.46,814,1030,70.99,'Intermediate' from dual union all
select 39873,'039873','1993-016AX','deb 11S772','',65118,'In Earth orbit','','2014-06-05 00:00:00',101.8,767,920,70.42,'Intermediate' from dual union all
select 39874,'039874','1993-016AY','deb 11S772','',65118,'In Earth orbit','','2014-06-03 00:00:00',101.65,745,929,70.95,'Intermediate' from dual union all
select 39875,'039875','1993-016AZ','deb 11S772','',65118,'In Earth orbit','','2013-03-18 00:00:00',102.41,766,980,70.96,'Intermediate' from dual union all
select 39876,'039876','1993-016BA','deb 11S772','',65118,'In Earth orbit','','2014-06-05 00:00:00',101.9,751,946,70.98,'Intermediate' from dual union all
select 39877,'039877','1993-016BB','deb 11S772','',65118,'In Earth orbit','','2014-05-18 00:00:00',102.45,760,990,70.97,'Intermediate' from dual union all
select 39878,'039878','1993-016BC','deb 11S772','',65118,'In Earth orbit','','2014-05-26 00:00:00',102.46,758,993,70.97,'Intermediate' from dual union all
select 39879,'039879','1993-016BD','deb 11S772','',65118,'In Earth orbit','','2014-06-05 00:00:00',102.06,754,959,70.98,'Intermediate' from dual;
insert into SATELLITE
select 3988,'003988','1969-052A','Kosmos-286','Zenit-4 No. 60',18319,'Landed','1969-06-23 00:00:00','1969-06-17 00:00:00',89.75,199,326,65.42,'Intermediate' from dual union all
select 39880,'039880','1993-016BE','deb 11S772','',65118,'In Earth orbit','','2011-10-16 00:00:00',102.71,774,999,70.99,'Intermediate' from dual union all
select 39881,'039881','1993-016BF','deb 11S772','',65118,'In Earth orbit','','2014-06-05 00:00:00',103.2,809,1011,71,'Intermediate' from dual union all
select 39882,'039882','1993-016BG','deb 11S772','',65118,'In Earth orbit','','2014-06-04 00:00:00',101.9,754,944,70.97,'Intermediate' from dual union all
select 39883,'039883','1993-016BH','deb 11S772','',65118,'In Earth orbit','','2014-05-25 00:00:00',102.26,753,979,70.98,'Intermediate' from dual union all
select 39884,'039884','1993-016BJ','deb 11S772','',65118,'In Earth orbit','','2014-03-30 00:00:00',102.19,767,958,70.97,'Intermediate' from dual union all
select 39885,'039885','1993-016BK','deb 11S772','',65118,'In Earth orbit','','2014-06-03 00:00:00',101.12,738,886,70.96,'Intermediate' from dual union all
select 39886,'039886','1993-016BL','deb 11S772','',65118,'In Earth orbit','','2014-06-03 00:00:00',99,661,760,71.02,'Intermediate' from dual union all
select 39887,'039887','1993-016BM','deb 11S772','',65118,'In Earth orbit','','2014-06-04 00:00:00',102.4,755,990,70.97,'Intermediate' from dual union all
select 39888,'039888','1993-016BN','deb 11S772','',65118,'In Earth orbit','','2014-06-05 00:00:00',102.16,777,945,70.99,'Intermediate' from dual union all
select 39889,'039889','1993-016BP','deb 11S772','',65118,'In Earth orbit','','2014-06-05 00:00:00',100.27,705,838,70.94,'Intermediate' from dual union all
select 3989,'003989','1969-052B','11A57I','11A57I',18319,'Reentered','1969-06-23 00:00:00','1969-06-17 00:00:00',89.41,187,304,65.4,'Intermediate' from dual union all
select 39890,'039890','1993-016BQ','deb 11S772','',65118,'In Earth orbit','','2014-05-04 00:00:00',102,756,951,70.99,'Intermediate' from dual union all
select 39891,'039891','1993-016BR','deb 11S772','',65118,'In Earth orbit','','2014-06-06 00:00:00',100.67,782,799,71,'Intermediate' from dual union all
select 39892,'039892','1993-016BS','deb 11S772','',65118,'In Earth orbit','','2014-06-03 00:00:00',101.68,741,936,70.99,'Intermediate' from dual union all
select 39893,'039893','1993-016BT','deb 11S772','',65118,'In Earth orbit','','2014-03-21 00:00:00',102.09,759,957,70.98,'Intermediate' from dual union all
select 39894,'039894','1993-016BU','deb 11S772','',65118,'In Earth orbit','','2011-12-01 00:00:00',102.42,765,982,70.97,'Intermediate' from dual union all
select 39895,'039895','1993-016BV','deb 11S772','',65118,'In Earth orbit','','2014-04-03 00:00:00',103.36,827,1008,70.98,'Intermediate' from dual union all
select 39896,'039896','1993-016BW','deb 11S772','',65118,'In Earth orbit','','2014-06-06 00:00:00',101.73,743,939,70.97,'Intermediate' from dual union all
select 39897,'039897','1993-016BX','deb 11S772','',65118,'In Earth orbit','','2014-04-22 00:00:00',102.03,748,961,70.96,'Intermediate' from dual union all
select 39898,'039898','1993-016BY','deb 11S772','',65118,'In Earth orbit','','2014-05-17 00:00:00',99.79,693,804,70.95,'Intermediate' from dual union all
select 39899,'039899','1993-016BZ','deb 11S772','',65118,'In Earth orbit','','2014-06-05 00:00:00',99.12,714,719,70.99,'Intermediate' from dual union all
select 399,'000399','62 A Psi 3','Tiros VI despin weight','',8152,'In Earth orbit','','1962-11-21 00:00:00',99.37,692,765,58.42,'Intermediate' from dual union all
select 3990,'003990','1969-053A','IMP 5','IMP G',18320,'Deep Space','1969-06-22 00:00:00','1969-07-25 00:00:00',4841.99,639,176219,87.13,'Deep Space' from dual union all
select 39900,'039900','1993-016CA','deb 11S772','',65118,'In Earth orbit','','2014-06-05 00:00:00',101.05,735,882,70.95,'Intermediate' from dual union all
select 39901,'039901','1993-016CB','deb 11S772','',65118,'In Earth orbit','','2014-06-01 00:00:00',102.69,760,1012,70.97,'Intermediate' from dual union all
select 39902,'039902','1993-016CC','deb 11S772','',65118,'In Earth orbit','','2014-06-06 00:00:00',104.85,844,1131,71,'Intermediate' from dual union all
select 39903,'039903','1993-016CD','deb 11S772','',65118,'In Earth orbit','','2014-05-27 00:00:00',103.91,844,1043,70.99,'Intermediate' from dual union all
select 39904,'039904','1993-016CE','deb 11S772','',65118,'In Earth orbit','','2011-08-24 00:00:00',102.52,770,986,70.99,'Intermediate' from dual union all
select 39905,'039905','1993-016CF','deb 11S772','',65118,'In Earth orbit','','2014-06-05 00:00:00',102.12,761,957,70.96,'Intermediate' from dual union all
select 39906,'039906','1993-016CG','deb 11S772','',65118,'In Earth orbit','','2014-06-05 00:00:00',102.14,759,961,70.98,'Intermediate' from dual union all
select 39907,'039907','1993-016CH','deb 11S772','',65118,'In Earth orbit','','2014-05-22 00:00:00',103.81,842,1036,70.99,'Intermediate' from dual union all
select 39908,'039908','1993-016CJ','deb 11S772','',65118,'In Earth orbit','','2014-06-05 00:00:00',101.28,812,827,71,'Intermediate' from dual union all
select 39909,'039909','1993-016CK','deb 11S772','',65118,'In Earth orbit','','2011-03-01 00:00:00',100.76,790,800,71.05,'Intermediate' from dual union all
select 3991,'003991','1969-054A','Kosmos-287','Zenit-2 No. 76',18321,'Landed','1969-07-02 00:00:00','1969-06-27 00:00:00',88.93,187,257,51.76,'Intermediate' from dual union all
select 39910,'039910','1993-016CL','deb 11S772','',65118,'In Earth orbit','','2014-06-05 00:00:00',100.08,744,781,71,'Intermediate' from dual union all
select 39911,'039911','1993-016CM','deb 11S772','',65118,'In Earth orbit','','2014-06-03 00:00:00',104.96,844,1142,71,'Intermediate' from dual union all
select 39912,'039912','1994-029AFK','deb HAPS','',65425,'In Earth orbit','','2014-06-01 00:00:00',97.19,538,711,81.34,'Intermediate' from dual union all
select 39913,'039913','1994-029AFL','deb HAPS','',65425,'In Earth orbit','','2014-06-03 00:00:00',105.82,586,1480,81.58,'Intermediate' from dual union all
select 39914,'039914','1994-029AFM','deb HAPS','',65425,'In Earth orbit','','2014-06-05 00:00:00',102.95,562,1235,81.33,'Intermediate' from dual union all
select 39915,'039915','1994-029AFN','deb HAPS','',65425,'In Earth orbit','','2014-06-05 00:00:00',95.58,443,650,82.01,'Intermediate' from dual union all
select 39916,'039916','1994-029AFP','deb HAPS','',65425,'In Earth orbit','','2014-06-06 00:00:00',99.17,558,880,81.58,'Intermediate' from dual union all
select 39917,'039917','1994-029AFQ','deb HAPS','',65425,'In Earth orbit','','2014-01-10 00:00:00',107.38,579,1632,82.13,'Intermediate' from dual union all
select 39918,'039918','1994-029AFR','deb HAPS','',65425,'Reentered','2015-10-20 00:00:00','2014-06-05 00:00:00',94.9,481,546,82.05,'Intermediate' from dual union all
select 39919,'039919','1994-029AFS','deb HAPS','',65425,'In Earth orbit','','2014-06-02 00:00:00',100.85,579,1018,81.85,'Intermediate' from dual union all
select 3992,'003992','1969-054B','11A57I','11A57I',18321,'Reentered','1969-06-27 00:00:00','1969-06-25 00:00:00',88.42,165,228,51.75,'Intermediate' from dual union all
select 39920,'039920','1994-029AFT','deb HAPS','',65425,'Reentered','2014-04-29 00:00:00','2014-03-08 00:00:00',98.98,656,764,82.49,'Intermediate' from dual union all
select 39921,'039921','1994-029AFU','deb HAPS','',65425,'In Earth orbit','','2014-06-05 00:00:00',103.11,594,1218,81.54,'Intermediate' from dual union all
select 39922,'039922','1994-029AFV','deb HAPS','',65425,'In Earth orbit','','2014-06-05 00:00:00',98.73,564,832,81.6,'Intermediate' from dual union all
select 39923,'039923','1994-029AFW','deb HAPS','',65425,'In Earth orbit','','2014-05-31 00:00:00',106.66,599,1545,82.15,'Intermediate' from dual union all
select 39924,'039924','1994-029AFX','deb HAPS','',65425,'In Earth orbit','','2014-06-05 00:00:00',98.12,566,771,81.87,'Intermediate' from dual union all
select 39925,'039925','1994-029AFY','deb HAPS','',65425,'In Earth orbit','','2014-06-05 00:00:00',97.71,534,765,81.52,'Intermediate' from dual union all
select 39926,'039926','1994-029AFZ','deb HAPS','',65425,'In Earth orbit','','2014-06-06 00:00:00',101.73,604,1078,81.84,'Intermediate' from dual union all
select 39927,'039927','1994-029AGA','deb HAPS','',65425,'In Earth orbit','','2014-06-05 00:00:00',97.1,562,678,81.83,'Intermediate' from dual union all
select 39928,'039928','1994-029AGB','deb HAPS','',65425,'In Earth orbit','','2014-06-06 00:00:00',100.83,561,1035,81.04,'Intermediate' from dual union all
select 39929,'039929','1994-029AGC','deb HAPS','',65425,'In Earth orbit','','2014-06-04 00:00:00',100.77,569,1022,81.92,'Intermediate' from dual union all
select 3993,'003993','1969-053B','FW4D','FW4D S/N 2322-2',18320,'Deep Space','1969-06-22 00:00:00','1976-07-02 00:00:00',4764.94,1174,173666,83.99,'Deep Space' from dual union all
select 39930,'039930','1994-029AGD','deb HAPS','',65425,'In Earth orbit','','2014-06-03 00:00:00',97.08,530,708,81.57,'Intermediate' from dual union all
select 39931,'039931','1994-029AGE','deb HAPS','',65425,'Reentered','2014-07-17 00:00:00','2014-06-05 00:00:00',92.59,381,422,82.01,'Intermediate' from dual union all
select 39932,'039932','1994-029AGF','deb HAPS','',65425,'In Earth orbit','','2014-06-05 00:00:00',101.6,588,1081,81.81,'Intermediate' from dual union all
select 39933,'039933','2006-006DF','deb Briz-M 88515','',68178,'In Earth orbit','','2013-01-12 00:00:00',283.8,564,15290,52.03,'Highly Ellip' from dual union all
select 39934,'039934','2006-006DG','deb Briz-M 88515','',68178,'Lost','','2013-06-23 00:00:00',206.17,352,10013,51.99,'Medium' from dual union all
select 39935,'039935','1981-053MV','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-05 00:00:00',103.5,897,951,82.83,'Intermediate' from dual union all
select 39936,'039936','1981-053MW','deb Kosmos-1275','',52576,'In Earth orbit','','2014-05-19 00:00:00',104.14,901,1008,83.15,'Intermediate' from dual union all
select 39937,'039937','1981-053MX','deb Kosmos-1275','',52576,'In Earth orbit','','2014-02-26 00:00:00',104.26,909,1010,82.88,'Intermediate' from dual union all
select 39938,'039938','1981-053MY','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-06 00:00:00',105.73,920,1138,83.07,'Intermediate' from dual union all
select 39939,'039939','1981-053MZ','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-03 00:00:00',104.74,931,1034,82.82,'Intermediate' from dual union all
select 3994,'003994','1969-055A','Kosmos-288','Zenit-4 No. 59',18322,'Landed','1969-07-05 00:00:00','1969-06-29 00:00:00',89.19,199,270,51.75,'Intermediate' from dual union all
select 39940,'039940','1981-053NA','deb Kosmos-1275','',52576,'In Earth orbit','','2014-01-28 00:00:00',105.42,952,1077,82.98,'Intermediate' from dual union all
select 39941,'039941','1981-053NB','deb Kosmos-1275','',52576,'In Earth orbit','','2014-05-28 00:00:00',104.63,898,1056,83.03,'Intermediate' from dual union all
select 39942,'039942','1981-053NC','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-03 00:00:00',103.72,901,968,83.18,'Intermediate' from dual union all
select 39943,'039943','1981-053ND','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-06 00:00:00',106.2,934,1167,83.07,'Intermediate' from dual union all
select 39944,'039944','1981-053NE','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-06 00:00:00',99.85,734,769,82.84,'Intermediate' from dual union all
select 39945,'039945','1981-053NF','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-05 00:00:00',104.46,939,1000,82.96,'Intermediate' from dual union all
select 39946,'039946','1981-053NG','deb Kosmos-1275','',52576,'In Earth orbit','','2014-04-24 00:00:00',104.37,938,992,82.96,'Intermediate' from dual union all
select 39947,'039947','1981-053NH','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-05 00:00:00',105.68,930,1123,83.01,'Intermediate' from dual union all
select 39948,'039948','1981-053NJ','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-06 00:00:00',106.58,920,1217,83.06,'Intermediate' from dual union all
select 39949,'039949','1981-053NK','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-03 00:00:00',103.93,910,978,82.82,'Intermediate' from dual union all
select 3995,'003995','1969-055B','11A57I','11A57I',18322,'Reentered','1969-07-02 00:00:00','1969-06-29 00:00:00',88.94,185,260,51.71,'Intermediate' from dual union all
select 39950,'039950','1981-053NL','deb Kosmos-1275','',52576,'In Earth orbit','','2014-05-26 00:00:00',104.91,904,1077,83.17,'Intermediate' from dual union all
select 39951,'039951','1981-053NM','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-03 00:00:00',103.07,856,952,82.83,'Intermediate' from dual union all
select 39952,'039952','1981-053NN','deb Kosmos-1275','',52576,'In Earth orbit','','2014-05-16 00:00:00',104.26,894,1026,83.06,'Intermediate' from dual union all
select 39953,'039953','1981-053NP','deb Kosmos-1275','',52576,'In Earth orbit','','2014-01-09 00:00:00',106.58,948,1188,83.01,'Intermediate' from dual union all
select 39954,'039954','1981-053NQ','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-05 00:00:00',102.14,821,899,82.91,'Intermediate' from dual union all
select 39955,'039955','1981-053NR','deb Kosmos-1275','',52576,'In Earth orbit','','2014-05-20 00:00:00',105.05,914,1080,83.42,'Intermediate' from dual union all
select 39956,'039956','1981-053NS','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-05 00:00:00',105.24,910,1102,83.03,'Intermediate' from dual union all
select 39957,'039957','1981-053NT','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-05 00:00:00',104.77,956,1012,82.92,'Intermediate' from dual union all
select 39958,'039958','1981-053NU','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-02 00:00:00',105.16,936,1067,83.03,'Intermediate' from dual union all
select 39959,'039959','1981-053NV','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-06 00:00:00',104.72,952,1011,82.96,'Intermediate' from dual union all
select 3996,'003996','1965-082SJ','deb Transtage 4','',11832,'Reentered','1969-12-26 00:00:00','1969-06-28 00:00:00',94.4,460,519,33.03,'Intermediate' from dual union all
select 39960,'039960','1981-053NW','deb Kosmos-1275','',52576,'In Earth orbit','','2014-05-30 00:00:00',107.22,920,1276,83.15,'Intermediate' from dual union all
select 39961,'039961','1981-053NX','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-05 00:00:00',103.85,900,981,82.77,'Intermediate' from dual union all
select 39962,'039962','1981-053NY','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-05 00:00:00',104.62,923,1031,82.82,'Intermediate' from dual union all
select 39963,'039963','1981-053NZ','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-03 00:00:00',105.8,933,1131,83.01,'Intermediate' from dual union all
select 39964,'039964','1981-053PA','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-05 00:00:00',100.13,725,805,83.09,'Intermediate' from dual union all
select 39965,'039965','1981-053PB','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-06 00:00:00',104.36,930,998,82.97,'Intermediate' from dual union all
select 39966,'039966','1981-053PC','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-06 00:00:00',103.42,886,955,82.71,'Intermediate' from dual union all
select 39967,'039967','1981-053PD','deb Kosmos-1275','',52576,'In Earth orbit','','2014-06-06 00:00:00',103.6,905,952,82.93,'Intermediate' from dual union all
select 39968,'039968','1981-053PE','deb Kosmos-1275','',52576,'In Earth orbit','','2013-09-13 00:00:00',104.88,931,1047,83.16,'Intermediate' from dual union all
select 39969,'039969','1987-068CV','deb S5M','',60158,'In Earth orbit','','2014-06-03 00:00:00',101.94,834,867,82.51,'Intermediate' from dual;
insert into SATELLITE
select 3997,'003997','1968-097BW','deb Kosmos-252','',16824,'Reentered','1982-11-22 00:00:00','1969-07-01 00:00:00',111.67,536,2071,62.4,'Intermediate' from dual union all
select 39970,'039970','1987-068CW','deb S5M','',60158,'In Earth orbit','','2014-06-04 00:00:00',102.56,872,888,82.56,'Intermediate' from dual union all
select 39971,'039971','1987-068CX','deb S5M','',60158,'In Earth orbit','','2014-05-24 00:00:00',104.23,911,1006,82.47,'Intermediate' from dual union all
select 39972,'039972','1987-068CY','deb S5M','',60158,'In Earth orbit','','2014-06-05 00:00:00',103.81,926,951,82.5,'Intermediate' from dual union all
select 39973,'039973','1987-068CZ','deb S5M','',60158,'In Earth orbit','','2014-06-06 00:00:00',100.82,776,819,83.04,'Intermediate' from dual union all
select 39974,'039974','1987-068DA','deb S5M','',60158,'In Earth orbit','','2014-06-06 00:00:00',102.73,881,895,82.54,'Intermediate' from dual union all
select 39975,'039975','1987-068DB','deb S5M','',60158,'In Earth orbit','','2014-06-03 00:00:00',102.95,825,972,82.6,'Intermediate' from dual union all
select 39976,'039976','1987-068DC','deb S5M','',60158,'In Earth orbit','','2014-06-03 00:00:00',103.2,901,919,82.55,'Intermediate' from dual union all
select 39977,'039977','1987-068DD','deb S5M','',60158,'In Earth orbit','','2014-06-03 00:00:00',100.48,756,807,82.49,'Intermediate' from dual union all
select 39978,'039978','1987-068DE','deb S5M','',60158,'In Earth orbit','','2014-06-06 00:00:00',103.62,890,970,82.41,'Intermediate' from dual union all
select 39979,'039979','1987-068DF','deb S5M','',60158,'In Earth orbit','','2014-06-05 00:00:00',101.61,811,859,82.54,'Intermediate' from dual union all
select 3998,'003998','1965-082SK','deb Transtage 4','',11832,'Reentered','1971-02-06 00:00:00','1969-06-28 00:00:00',97.64,625,667,32.29,'Intermediate' from dual union all
select 39980,'039980','1987-068DG','deb S5M','',60158,'In Earth orbit','','2014-06-05 00:00:00',99.16,709,728,82.51,'Intermediate' from dual union all
select 39981,'039981','1987-068DH','deb S5M','',60158,'In Earth orbit','','2014-05-31 00:00:00',103.59,921,936,82.56,'Intermediate' from dual union all
select 39982,'039982','1987-068DJ','deb S5M','',60158,'In Earth orbit','','2014-06-06 00:00:00',103.73,921,949,82.55,'Intermediate' from dual union all
select 39983,'039983','1987-068DK','deb S5M','',60158,'In Earth orbit','','2014-06-06 00:00:00',102.16,793,929,82.17,'Intermediate' from dual union all
select 39984,'039984','1987-068DL','deb S5M','',60158,'In Earth orbit','','2014-06-06 00:00:00',102,847,860,82.84,'Intermediate' from dual union all
select 39985,'039985','1987-068DM','deb S5M','',60158,'In Earth orbit','','2014-06-03 00:00:00',102.89,883,908,82.53,'Intermediate' from dual union all
select 39986,'039986','1987-068DN','deb S5M','',60158,'In Earth orbit','','2014-04-21 00:00:00',104.09,921,983,82.5,'Intermediate' from dual union all
select 39987,'039987','2008-011DL','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-03-18 00:00:00',468.56,800,26411,49.23,'Highly Ellip' from dual union all
select 39988,'039988','2008-011DM','deb Briz-M No. 88525','',68519,'In Earth orbit','','2014-06-04 00:00:00',502.76,2772,26361,49.35,'Highly Ellip' from dual union all
select 39989,'039989','2008-011DN','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-04-10 00:00:00',463.96,916,26033,48.48,'Highly Ellip' from dual union all
select 3999,'003999','1965-108E','deb Transtage 8','',11858,'Reentered','1969-08-21 00:00:00','1969-06-28 00:00:00',115.17,207,2720,26.48,'Intermediate' from dual union all
select 39990,'039990','2008-011DP','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-04-03 00:00:00',439.55,481,25063,48.91,'Highly Ellip' from dual union all
select 39991,'039991','2008-011DQ','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-01-29 00:00:00',474.09,574,26951,48.28,'Highly Ellip' from dual union all
select 39992,'039992','2008-011DR','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-04-10 00:00:00',471.68,993,26395,49.03,'Highly Ellip' from dual union all
select 39993,'039993','2008-011DS','deb Briz-M No. 88525','',68519,'In Earth orbit','','2014-05-31 00:00:00',453.26,743,25593,51.09,'Highly Ellip' from dual union all
select 39994,'039994','2008-011DT','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-04-27 00:00:00',467.91,716,26458,49.1,'Highly Ellip' from dual union all
select 39995,'039995','2008-011DU','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-06-11 00:00:00',467.42,689,26457,49.25,'Highly Ellip' from dual union all
select 39996,'039996','2008-011DV','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-10-25 00:00:00',490.43,2174,26271,47.31,'Highly Ellip' from dual union all
select 39997,'039997','2008-011DW','deb Briz-M No. 88525','',68519,'In Earth orbit','','2013-10-23 00:00:00',529.3,705,29889,49.87,'Highly Ellip' from dual union all
select 39998,'039998','2002-040E','SEVIRI Cooler Cover','',67416,'In Earth orbit','','2014-06-05 00:00:00',1412.12,34927,35705,7.87,'Inclined Drift' from dual union all
select 39999,'039999','2002-040F','SEVIRI Entry Baffle Cover','',67416,'In Earth orbit','','2014-06-05 00:00:00',1412.12,34927,35705,7.87,'Inclined Drift' from dual union all
select 4,'000004','58 Alpha','Explorer 1','Explorer 1',4792,'Reentered Attached','1970-03-31 00:00:00','1958-02-01 00:00:00',114.89,359,2542,33.18,'Intermediate' from dual union all
select 40,'000040','60 Epsilon 7','Korabl''-Sputnik part','Vostok-1P part',6320,'Reentered','1960-09-24 00:00:00','1960-07-02 00:00:00',92.75,282,537,64.89,'Intermediate' from dual union all
select 400,'000400','62 A Psi 4','Tiros VI despin weight','',8152,'Reentered','1989-04-05 00:00:00','1962-12-03 00:00:00',98.03,637,692,58.19,'Intermediate' from dual union all
select 4000,'004000','1969-056A','Biosatellite 3','Biosatellite D Adapter',18323,'Reentered','1970-01-20 00:00:00','1969-08-02 00:00:00',91.94,352,388,33.55,'Intermediate' from dual union all
select 40000,'040000','2004-042D','FY-2C imager cover','',67886,'In Earth orbit','','2014-06-04 00:00:00',1436.88,35499,36105,7.47,'Inclined Drift' from dual union all
select 40001,'040001','2014-032A','Kosmos-2500','Uragan-M No. 755',69736,'In Earth orbit','','2014-06-22 00:00:00',678.66,19145,19262,64.78,'Medium' from dual union all
select 40002,'040002','2014-032B','Fregat-M No. 112-02','Fregat 14S44 No. 112-02',69736,'In Earth orbit','','2014-06-23 00:00:00',688.43,19237,19660,64.79,'Medium' from dual union all
select 40003,'040003','2014-026K','deb Delta 4','',69730,'Reentered','2014-06-11 00:00:00','2014-06-08 00:00:00',222.08,168,11371,52.54,'Medium' from dual union all
select 40004,'040004','2014-026L','deb Delta 4','',69730,'Reentered','2014-06-13 00:00:00','2014-06-10 00:00:00',128.49,173,3941,52.71,'Medium' from dual union all
select 40005,'040005','2014-026M','deb Delta 4','',69730,'Reentered','2014-06-22 00:00:00','2014-06-18 00:00:00',173.51,166,7687,52.7,'Medium' from dual union all
select 40006,'040006','2014-026N','deb Delta 4','',69730,'Reentered','2014-06-13 00:00:00','2014-06-08 00:00:00',199.62,171,9701,52.7,'Medium' from dual union all
select 40007,'040007','2014-026P','deb Delta 4','',69730,'Reentered','2014-06-18 00:00:00','2014-06-17 00:00:00',157.08,183,6347,52.67,'Medium' from dual union all
select 40008,'040008','2014-026Q','deb Delta 4','',69730,'Reentered','2014-06-12 00:00:00','2014-06-11 00:00:00',104.21,160,1755,52.56,'Intermediate' from dual union all
select 40009,'040009','2014-026R','deb Delta 4','',69730,'Reentered','2014-06-09 00:00:00','2014-06-09 00:00:00',117.3,170,2949,52.7,'Intermediate' from dual union all
select 4001,'004001','1969-056B','Delta 70','DSV-3E-3 S/N 20234',18323,'Reentered','1969-12-01 00:00:00','1969-08-05 00:00:00',91.7,344,372,33.55,'Intermediate' from dual union all
select 40010,'040010','2014-033A','KazEOSat-2','Kazakh MRES',69737,'In Earth orbit','','2014-06-26 00:00:00',97.17,611,636,97.99,'Sun-Synch' from dual union all
select 40011,'040011','2014-033B','Hodoyoshi-4','Hodoyoshi-4',69737,'In Earth orbit','','2014-06-27 00:00:00',97.33,612,650,97.99,'Sun-Synch' from dual union all
select 40012,'040012','2014-033C','Unisat-6','Unisat-6',69737,'In Earth orbit','','2014-06-26 00:00:00',97.85,613,699,97.98,'Sun-Synch' from dual union all
select 40013,'040013','2014-033D','Deimos-2','Deimos-2',69737,'In Earth orbit','','2014-06-27 00:00:00',96.84,597,617,97.99,'Sun-Synch' from dual union all
select 40014,'040014','2014-033E','Tita','BugSat-1',69737,'In Earth orbit','','2014-06-27 00:00:00',96.54,570,616,98,'Sun-Synch' from dual union all
select 40015,'040015','2014-033F','Hodoyoshi-3','Hodoyoshi-3',69737,'In Earth orbit','','2014-06-27 00:00:00',97.49,613,664,97.98,'Sun-Synch' from dual union all
select 40016,'040016','2014-033G','Saudisat 4','Saudisat 4',69737,'In Earth orbit','','2014-06-26 00:00:00',97.66,613,681,97.98,'Sun-Synch' from dual union all
select 40017,'040017','2014-033H','TabletSat-Aurora','TabletSat-Aurora',69737,'In Earth orbit','','2014-06-27 00:00:00',96.68,583,617,97.99,'Sun-Synch' from dual union all
select 40018,'040018','2014-033J','AprizeSat-9','',69737,'In Earth orbit','','2014-06-26 00:00:00',98.01,612,715,97.98,'Sun-Synch' from dual union all
select 40019,'040019','2014-033K','AprizeSat-10','',69737,'In Earth orbit','','2014-06-27 00:00:00',98.2,613,733,97.97,'Sun-Synch' from dual union all
select 4002,'004002','1968-091BX','deb Kosmos-249','',16818,'Reentered','1970-08-03 00:00:00','1969-06-28 00:00:00',103.04,527,1278,62.31,'Intermediate' from dual union all
select 40020,'040020','2014-033L','BRITE-Toronto','BRITE-C1',69737,'In Earth orbit','','2014-06-27 00:00:00',98.26,613,738,97.97,'Sun-Synch' from dual union all
select 40021,'040021','2014-033M','Duchifat-1','Duchifat-1',69737,'In Earth orbit','','2014-06-27 00:00:00',96.93,603,621,97.98,'Sun-Synch' from dual union all
select 40022,'040022','2014-033N','PACE','PACE',69737,'In Earth orbit','','2014-06-27 00:00:00',96.94,604,621,97.99,'Sun-Synch' from dual union all
select 40023,'040023','2014-033P','Flock 1c-10','Dove 090C',69737,'In Earth orbit','','2014-06-26 00:00:00',96.93,603,621,97.99,'Sun-Synch' from dual union all
select 40024,'040024','2014-033Q','NanoSatC-Br 1','NanoSatC-Br 1 (NCBR-1)',69737,'In Earth orbit','','2014-06-27 00:00:00',96.93,604,620,97.98,'Sun-Synch' from dual union all
select 40025,'040025','2014-033R','QB50P1','QB50P1',69737,'In Earth orbit','','2014-06-27 00:00:00',96.94,604,621,97.98,'Sun-Synch' from dual union all
select 40026,'040026','2014-033S','Flock 1c-7','Dove 0909',69737,'In Earth orbit','','2014-06-27 00:00:00',96.94,605,619,97.99,'Sun-Synch' from dual union all
select 40027,'040027','2014-033T','Flock 1c-1','Dove 0903',69737,'In Earth orbit','','2014-06-27 00:00:00',96.94,604,621,97.99,'Sun-Synch' from dual union all
select 40028,'040028','2014-033U','POPSAT-HIP1','POPSAT-HIP1',69737,'In Earth orbit','','2014-06-27 00:00:00',96.95,604,622,97.99,'Sun-Synch' from dual union all
select 40029,'040029','2014-033V','Flock 1c-2','Dove 0904',69737,'In Earth orbit','','2014-06-27 00:00:00',96.96,604,623,97.99,'Sun-Synch' from dual union all
select 4003,'004003','1965-082SL','deb Transtage 4','',11832,'Reentered','1981-02-18 00:00:00','1981-02-15 00:00:00',91.34,322,359,30.54,'Intermediate' from dual union all
select 40030,'040030','2014-033W','DTUSat-2','DTUSat-2',69737,'In Earth orbit','','2014-06-27 00:00:00',96.97,606,622,97.98,'Sun-Synch' from dual union all
select 40031,'040031','2014-033X','Flock 1c-4','Dove 0906',69737,'In Earth orbit','','2014-06-27 00:00:00',96.96,605,621,97.99,'Sun-Synch' from dual union all
select 40032,'040032','2014-033Y','QB50P2','QB50P2',69737,'In Earth orbit','','2014-06-27 00:00:00',96.96,606,621,97.98,'Sun-Synch' from dual union all
select 40033,'040033','2014-033Z','Flock 1c-11','Dove 090D',69737,'In Earth orbit','','2014-06-26 00:00:00',96.97,606,622,97.99,'Sun-Synch' from dual union all
select 40034,'040034','2014-033AA','ANTELSAT','ANTELSAT',69737,'In Earth orbit','','2014-06-27 00:00:00',96.97,606,622,97.99,'Sun-Synch' from dual union all
select 40035,'040035','2014-033AB','Flock 1c-9','Dove 090B',69737,'In Earth orbit','','2014-06-26 00:00:00',96.97,606,622,97.99,'Sun-Synch' from dual union all
select 40036,'040036','2014-033AC','Flock 1c-6','Dove 0908',69737,'In Earth orbit','','2014-06-26 00:00:00',96.98,607,622,97.99,'Sun-Synch' from dual union all
select 40037,'040037','2014-033AD','Perseus-M 2','Perseus-M2',69737,'In Earth orbit','','2014-06-27 00:00:00',96.97,605,623,97.99,'Sun-Synch' from dual union all
select 40038,'040038','2014-033AE','Flock 1c-5','Dove 0907',69737,'In Earth orbit','','2014-06-26 00:00:00',97,606,625,97.99,'Sun-Synch' from dual union all
select 40039,'040039','2014-033AF','Perseus-M 1','Perseus-M1',69737,'In Earth orbit','','2014-06-26 00:00:00',96.99,606,623,97.99,'Sun-Synch' from dual union all
select 4004,'004004','1965-082SM','deb Transtage 4','',11832,'Reentered','1970-01-05 00:00:00','1969-06-28 00:00:00',104.74,633,1332,32.37,'Intermediate' from dual union all
select 40040,'040040','2014-033AG','Flock 1c-8','Dove 090A',69737,'In Earth orbit','','2014-06-27 00:00:00',97,607,624,97.99,'Sun-Synch' from dual union all
select 40041,'040041','2014-033AH','Flock 1c-3','Dove 0905',69737,'In Earth orbit','','2014-06-26 00:00:00',97,607,624,97.99,'Sun-Synch' from dual union all
select 40042,'040042','2014-033AJ','PolyITAN 1','PolyITAN 1',69737,'In Earth orbit','','2014-06-27 00:00:00',97.02,606,626,97.99,'Sun-Synch' from dual union all
select 40043,'040043','2014-033AK','Tigris','Tigrisat',69737,'In Earth orbit','','2014-06-27 00:00:00',97.8,610,697,97.98,'Sun-Synch' from dual union all
select 40044,'040044','2014-033AL','Lemur-1','Lemur-1',69737,'In Earth orbit','','2014-06-27 00:00:00',97.81,611,696,97.98,'Sun-Synch' from dual union all
select 40045,'040045','2014-033AM','Aerocube 6A','CubeRAD-a',69737,'In Earth orbit','','2014-06-26 00:00:00',97.86,613,700,97.98,'Sun-Synch' from dual union all
select 40046,'040046','2014-033AN','Aerocube 6B','CubeRAD-b',69737,'In Earth orbit','','2014-06-26 00:00:00',97.86,613,700,97.98,'Sun-Synch' from dual union all
select 40047,'040047','2014-033AP','R-36M2 DS','R-36M2 DS',69737,'In Earth orbit','','2014-06-27 00:00:00',105.77,608,1453,97.9,'Sun-Synch' from dual union all
select 40048,'040048','2014-033AQ','Gazodinamicheskiy Ekran','GDS (Gas dynamic shield)',69737,'In Earth orbit','','2014-06-27 00:00:00',96.35,552,616,98,'Sun-Synch' from dual union all
select 40049,'040049','2014-033AR','Dnepr Platform A/Fairing','',69737,'In Earth orbit','','2014-06-27 00:00:00',97.87,614,699,97.98,'Sun-Synch' from dual union all
select 4005,'004005','61 Omicron 226','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-04 00:00:00',105.19,934,1073,66.75,'Intermediate' from dual union all
select 40050,'040050','1993-014BM','deb Start','',65116,'In Earth orbit','','2015-03-08 00:00:00',99.98,678,837,75.77,'Intermediate' from dual union all
select 40051,'040051','1998-067ER','MPAC/SEED Carrier Frame','ISS RS Frame',66503,'Reentered','2014-10-31 00:00:00','2014-06-20 00:00:00',92.82,409,417,51.65,'Intermediate' from dual union all
select 40052,'040052','2007-029Q','deb 11S772 sep motor','deb Tselina-2',68371,'In Earth orbit','','2014-06-25 00:00:00',105.91,932,1142,71.1,'Intermediate' from dual union all
select 40053,'040053','2014-034A','SPOT 7','SPOT 7',69738,'In Earth orbit','','2014-07-02 00:00:00',97.74,642,659,98.26,'Sun-Synch' from dual union all
select 40054,'040054','2014-034B','AISat-1','AISat-1',69738,'In Earth orbit','','2014-07-02 00:00:00',97.74,642,659,98.25,'Sun-Synch' from dual union all
select 40055,'040055','2014-034C','CanX-4','CanX-4',69738,'In Earth orbit','','2014-07-02 00:00:00',97.72,641,659,98.26,'Sun-Synch' from dual union all
select 40056,'040056','2014-034D','CanX-5','CanX-5',69738,'In Earth orbit','','2014-07-02 00:00:00',97.7,642,656,98.25,'Sun-Synch' from dual union all
select 40057,'040057','2014-034E','VELOX-I','VELOX-I N-Sat',69738,'In Earth orbit','','2014-07-02 00:00:00',97.68,641,655,98.25,'Sun-Synch' from dual;
insert into SATELLITE
select 40058,'040058','2014-034F','PS4','PS4 PSLV-C23',69738,'In Earth orbit','','2014-07-02 00:00:00',97.46,623,652,98.19,'Sun-Synch' from dual union all
select 40059,'040059','2014-035A','OCO-2','Orbiting Carbon Observat',69739,'In Earth orbit','','2014-07-04 00:00:00',98.53,687,690,98.19,'Sun-Synch' from dual union all
select 4006,'004006','61 Omicron 227','deb r/b Ablestar','',7273,'Reentered','2000-06-10 00:00:00','1992-02-05 00:00:00',99.58,706,771,68.06,'Intermediate' from dual union all
select 40060,'040060','2014-035B','Delta 367','Delta SSPS AJ-10-118K',69739,'In Earth orbit','','2014-07-08 00:00:00',235.57,2376,10138,98.18,'Medium' from dual union all
select 40061,'040061','2014-036A','Gonets-M No. 18','Gonets-M No. 18',69740,'In Earth orbit','','2014-07-05 00:00:00',115.86,1479,1510,82.52,'Intermediate' from dual union all
select 40062,'040062','2014-036B','Gonets-M No. 19','Gonets-M No. 19',69740,'In Earth orbit','','2014-07-05 00:00:00',115.88,1480,1511,82.51,'Intermediate' from dual union all
select 40063,'040063','2014-036C','Gonets-M No. 20','Gonets-M No. 20',69740,'In Earth orbit','','2014-07-05 00:00:00',115.82,1477,1509,82.51,'Intermediate' from dual union all
select 40064,'040064','2014-036D','Briz-KM No. 72522','Briz-KM 14S45',69740,'In Earth orbit','','2014-07-05 00:00:00',112.34,1162,1507,82.52,'Intermediate' from dual union all
select 40065,'040065','2009-021G','deb Yaogan 6','',68702,'Reentered','2014-11-19 00:00:00','2014-07-07 00:00:00',94.78,491,525,97.26,'Sun-Synch' from dual union all
select 40066,'040066','2009-021H','deb Yaogan 6','',68702,'Reentered','2014-10-13 00:00:00','2014-07-07 00:00:00',94.77,491,524,97.28,'Sun-Synch' from dual union all
select 40067,'040067','2009-021J','deb Yaogan 6','',68702,'Reentered','2014-10-13 00:00:00','2014-07-07 00:00:00',94.79,492,525,97.29,'Sun-Synch' from dual union all
select 40068,'040068','2009-021K','deb Yaogan 6','',68702,'Reentered','2015-01-13 00:00:00','2014-07-08 00:00:00',94.81,489,530,97.3,'Sun-Synch' from dual union all
select 40069,'040069','2014-037A','Meteor-M No. 2','Meteor-M No. 2',69741,'In Earth orbit','','2014-07-09 00:00:00',101.35,818,827,98.81,'Sun-Synch' from dual union all
select 4007,'004007','1966-025F','deb OV1-5                         ','[UNID028]',13432,'Reentered','1977-04-12 00:00:00','1969-06-27 00:00:00',104.4,949,984,144.27,'Retrograde' from dual union all
select 40070,'040070','2014-037B','Vernov','MKA-FKI (PN2)',69741,'In Earth orbit','','2014-07-09 00:00:00',99.21,623,819,98.38,'Sun-Synch' from dual union all
select 40071,'040071','2014-037C','DX-1','DX-1',69741,'In Earth orbit','','2014-07-09 00:00:00',97.29,626,632,98.4,'Sun-Synch' from dual union all
select 40072,'040072','2014-037D','SkySat 2','SkySat 2',69741,'In Earth orbit','','2014-07-09 00:00:00',97.3,626,633,98.4,'Sun-Synch' from dual union all
select 40073,'040073','2014-037E','M3MSat Mass Mock-up','M3MSat dummy',69741,'In Earth orbit','','2014-07-09 00:00:00',97.25,625,630,98.39,'Sun-Synch' from dual union all
select 40074,'040074','2014-037F','UKube-1','UKube-1',69741,'In Earth orbit','','2014-07-09 00:00:00',97.25,625,630,98.4,'Sun-Synch' from dual union all
select 40075,'040075','2014-037G','AISSAT-2','AISSAT-2',69741,'In Earth orbit','','2014-07-09 00:00:00',97.25,625,630,98.4,'Sun-Synch' from dual union all
select 40076,'040076','2014-037H','TDS-1','TechDemoSat-1',69741,'In Earth orbit','','2014-07-09 00:00:00',97.31,625,635,98.41,'Sun-Synch' from dual union all
select 40077,'040077','2014-037J','Blok-I 14S54','14S54',69741,'Reentered','2014-07-10 00:00:00','2014-07-09 00:00:00',88,169,181,98.79,'Sun-Synch' from dual union all
select 40078,'040078','2014-037K','Upper frame adapter','',69741,'In Earth orbit','','2014-07-09 00:00:00',99.2,622,819,98.38,'Sun-Synch' from dual union all
select 40079,'040079','2014-038A','O3b FM3','O3b 3',69742,'In Earth orbit','','2014-07-27 00:00:00',287.92,8062,8069,.05,'Medium' from dual union all
select 4008,'004008','1968-091BY','deb Kosmos-249 [*]','',16818,'In Earth orbit','','1992-02-05 00:00:00',106.84,514,1647,62.86,'Intermediate' from dual union all
select 40080,'040080','2014-038B','O3b FM7','O3b 7',69742,'In Earth orbit','','2014-08-04 00:00:00',288.01,8066,8071,.09,'Medium' from dual union all
select 40081,'040081','2014-038C','O3b FM6','O3b 6',69742,'In Earth orbit','','2014-08-04 00:00:00',287.99,8065,8070,.09,'Medium' from dual union all
select 40082,'040082','2014-038D','O3b FM8','O3b 8',69742,'In Earth orbit','','2014-07-26 00:00:00',287.92,8062,8069,.02,'Medium' from dual union all
select 40083,'040083','2014-038E','Fregat-MT No. 1032','Fregat-MT    No. 1032',69742,'In Earth orbit','','2014-07-18 00:00:00',276.31,7657,7692,.08,'Medium' from dual union all
select 40084,'040084','2014-039A','SS Janice Voss','Cygnus PCM-2',69743,'Deorbited','2014-08-17 00:00:00','2014-07-15 00:00:00',92.54,386,411,51.64,'Intermediate' from dual union all
select 40085,'040085','2014-039B','Antares Stage 2','Castor 30B',69743,'Reentered','2014-07-19 00:00:00','2014-07-15 00:00:00',88.77,185,242,51.61,'Intermediate' from dual union all
select 40086,'040086','2014-040A','Orbcomm FM109','Orbcomm OG2 SV9',69744,'In Earth orbit','','2014-07-22 00:00:00',98.32,619,738,47.01,'Intermediate' from dual union all
select 40087,'040087','2014-040B','Orbcomm FM107','Orbcomm OG2 SV7',69744,'In Earth orbit','','2014-07-22 00:00:00',98.32,619,738,47.01,'Intermediate' from dual union all
select 40088,'040088','2014-040C','Orbcomm FM106','Orbcomm OG2 SV6',69744,'In Earth orbit','','2014-07-22 00:00:00',98.33,619,738,47.01,'Intermediate' from dual union all
select 40089,'040089','2014-040D','Orbcomm FM111','Orbcomm OG2 SV11',69744,'In Earth orbit','','2014-07-22 00:00:00',98.33,619,738,47.01,'Intermediate' from dual union all
select 4009,'004009','1967-011Q','deb Diademe 2                     ','[UNID029] RCS 0.0',15266,'Reentered','1982-11-12 00:00:00','1969-06-30 00:00:00',109.42,563,1837,39.46,'Intermediate' from dual union all
select 40090,'040090','2014-040E','Orbcomm FM104','Orbcomm OG2 SV4',69744,'In Earth orbit','','2014-07-22 00:00:00',98.34,619,740,47.01,'Intermediate' from dual union all
select 40091,'040091','2014-040F','Orbcomm FM103','Orbcomm OG2 SV3',69744,'In Earth orbit','','2014-07-22 00:00:00',98.34,619,740,47.01,'Intermediate' from dual union all
select 40092,'040092','2011-043G','deb HY-2A','',69095,'Reentered','2014-11-12 00:00:00','2014-07-18 00:00:00',103.61,878,980,99.35,'Sun-Synch' from dual union all
select 40093,'040093','2011-043H','deb HY-2A','',69095,'Reentered','2014-11-07 00:00:00','2014-07-18 00:00:00',103.72,890,979,99.4,'Sun-Synch' from dual union all
select 40094,'040094','2011-043J','deb HY-2A','',69095,'Reentered','2014-12-15 00:00:00','2014-07-18 00:00:00',104.28,916,1006,99.35,'Sun-Synch' from dual union all
select 40095,'040095','2014-041A','Foton-M No. 4','Foton-M 34KSM No. 1',69745,'Landed','2014-09-01 00:00:00','2014-07-26 00:00:00',92.56,251,548,64.88,'Intermediate' from dual union all
select 40096,'040096','2014-041B','Blok-I 14S54/RD-0110','374BL02A',69745,'Reentered','2014-10-31 00:00:00','2014-07-26 00:00:00',92.5,250,545,64.89,'Intermediate' from dual union all
select 40097,'040097','2014-042A','Progress M-24M','Progress 7K-TGM No.423',69746,'Deorbited','2014-11-19 00:00:00','2014-07-24 00:00:00',92.86,411,419,51.64,'Intermediate' from dual union all
select 40098,'040098','2014-042B','11S510','11S510',69746,'Reentered','2014-07-25 00:00:00','2014-07-24 00:00:00',87.93,166,177,51.64,'Intermediate' from dual union all
select 40099,'040099','2014-043A','USA 253','Hornet 1',69747,'In Earth orbit','','2014-08-01 00:00:00',1439.33,35800,35900,.5,'Drift GEO' from dual union all
select 401,'000401','61 Omicron 121','deb r/b Ablestar','',7273,'Reentered','1989-01-25 00:00:00','1989-01-24 00:00:00',91.25,327,345,67,'Intermediate' from dual union all
select 4010,'004010','1968-097BX','deb Kosmos-252','',16824,'Reentered','1969-09-10 00:00:00','1969-06-28 00:00:00',99.83,316,1185,62.38,'Intermediate' from dual union all
select 40100,'040100','2014-043B','USA 254','Hornet 2',69747,'In Earth orbit','','2014-08-01 00:00:00',1439.33,35800,35900,.5,'Drift GEO' from dual union all
select 40101,'040101','2014-043C','USA 255','ANGELS',69747,'In Earth orbit','','2014-08-01 00:00:00',1454.7,36100,36200,.5,'Drift GEO' from dual union all
select 40102,'040102','2014-043D','Delta 4 Second Stage','DCSS-4',69747,'In Earth orbit','','2014-08-01 00:00:00',1454.7,36100,36200,.5,'Drift GEO' from dual union all
select 40103,'040103','2014-044A','Georges Lemaitre','ATV 5',69748,'Deorbited','2015-02-15 00:00:00','2014-08-01 00:00:00',91.17,329,335,51.64,'Intermediate' from dual union all
select 40104,'040104','1998-067ES','EVA debris','',66503,'Reentered','2014-08-24 00:00:00','2014-07-30 00:00:00',91.95,362,379,51.64,'Intermediate' from dual union all
select 40105,'040105','2014-045A','GPS IIF-7','GPS SVN 68',69749,'In Earth orbit','','2014-08-12 00:00:00',729.43,20450,20478,55.01,'Medium' from dual union all
select 40106,'040106','2014-045B','Centaur AV-048','Centaur AV-048',69749,'In Earth orbit','','2014-08-09 00:00:00',754.58,20441,21714,55.33,'Medium' from dual union all
select 40107,'040107','2014-046A','Asiasat 8','Asiasat 8',69750,'In Earth orbit','','2014-08-12 00:00:00',1433.1,35724,35732,.08,'Drift GEO' from dual union all
select 40108,'040108','2014-046B','Falcon 9-011 Stage 2','F9-011 St2',69750,'In Earth orbit','','2014-08-12 00:00:00',628.57,187,35671,24.32,'GEO Transfer' from dual union all
select 40109,'040109','2014-047A','Yaogan Weixing 20','Jianbing-8 04 xing',69751,'In Earth orbit','','2014-08-16 00:00:00',106.39,1054,1065,63.4,'Intermediate' from dual union all
select 4011,'004011','1968-097BY','deb Kosmos-252','',16824,'Reentered','1982-11-12 00:00:00','1969-06-28 00:00:00',115.8,766,2218,62.22,'Intermediate' from dual union all
select 40110,'040110','2014-047B','Yaogan Weixing 20 fu xing yi','Jianbing-8 04 subsat 1',69751,'In Earth orbit','','2014-08-16 00:00:00',107.04,1087,1093,63.41,'Intermediate' from dual union all
select 40111,'040111','2014-047C','Yaogan Weixing 20 fu xing er','Jianbing-8 04 subsat 2',69751,'In Earth orbit','','2014-08-16 00:00:00',107.15,1086,1104,63.4,'Intermediate' from dual union all
select 40112,'040112','2014-047D','CZ-4C Stage 3','CZ-4C Stage 3',69751,'In Earth orbit','','2014-08-15 00:00:00',105.15,897,1106,63.46,'Intermediate' from dual union all
select 40113,'040113','2014-047E','deb YW-20','',69751,'In Earth orbit','','2014-08-11 00:00:00',107.12,1084,1103,63.4,'Intermediate' from dual union all
select 40114,'040114','2014-047F','deb YW-20','',69751,'In Earth orbit','','2014-08-12 00:00:00',107.16,1088,1103,63.41,'Intermediate' from dual union all
select 40115,'040115','2014-048A','WorldView-3','WorldView-3',69752,'In Earth orbit','','2014-08-15 00:00:00',96.95,612,613,97.99,'Sun-Synch' from dual union all
select 40116,'040116','2014-048B','Centaur AV-047','Centaur AV-047',69752,'Deep Space','2014-08-14 00:00:00','2014-08-15 00:00:00',210.66,655,36867,97.99,'Earth Escape' from dual union all
select 40117,'040117','1998-067ET','Chasqui 1','Chasqui 1',66503,'Reentered','2015-01-15 00:00:00','2014-08-18 00:00:00',92.87,412,419,51.64,'Intermediate' from dual union all
select 40118,'040118','2014-049A','Gao Fen 2','gao fen 2',69753,'In Earth orbit','','2014-08-21 00:00:00',97.09,608,630,98.02,'Sun-Synch' from dual union all
select 40119,'040119','2014-049B','Heweliusz','BRITE-P2',69753,'In Earth orbit','','2014-08-21 00:00:00',97.12,609,633,98.02,'Sun-Synch' from dual union all
select 4012,'004012','1968-097BZ','deb Kosmos-252 [*]','',16824,'Reentered','1979-03-09 00:00:00','1979-02-16 00:00:00',96.02,421,715,62.43,'Intermediate' from dual union all
select 40120,'040120','2014-049C','CZ-4B Stage 3','CZ-4B Stage 3',69753,'Reentered','2017-05-27 00:00:00','2014-08-22 00:00:00',94.24,353,611,98.12,'Sun-Synch' from dual union all
select 40121,'040121','2008-046S','deb SOZ','',68554,'In Earth orbit','','2014-08-21 00:00:00',333.52,576,18530,65.16,'Highly Ellip' from dual union all
select 40122,'040122','1998-067EU','Flock 1b-24','Dove 0801',66503,'Reentered','2015-03-15 00:00:00','2014-08-20 00:00:00',92.83,410,417,51.65,'Intermediate' from dual union all
select 40123,'040123','1998-067EV','Flock 1b-23','Dove 080B',66503,'Reentered','2015-03-03 00:00:00','2014-08-20 00:00:00',92.83,410,417,51.65,'Intermediate' from dual union all
select 40124,'040124','1998-067EW','Flock 1b-26','Dove 0814',66503,'Reentered','2014-12-14 00:00:00','2014-08-20 00:00:00',92.84,413,415,51.65,'Intermediate' from dual union all
select 40125,'040125','1998-067EX','Flock 1b-25','Dove 0818',66503,'Reentered','2015-02-01 00:00:00','2014-08-20 00:00:00',92.84,413,415,51.64,'Intermediate' from dual union all
select 40126,'040126','1998-067EY','Flock 1b-15','Dove 0812',66503,'Reentered','2015-01-07 00:00:00','2014-08-20 00:00:00',92.85,411,418,51.64,'Intermediate' from dual union all
select 40127,'040127','1998-067EZ','Flock 1b-16','Dove 081D',66503,'Reentered','2014-12-13 00:00:00','2014-08-20 00:00:00',92.85,411,418,51.64,'Intermediate' from dual union all
select 40128,'040128','2014-050A','GalileoSat-5','Galileo FOC FM01',69754,'In Earth orbit','','2014-08-31 00:00:00',703.29,13721,25917,49.69,'Medium' from dual union all
select 40129,'040129','2014-050B','GalileoSat-6','Galileo FOC FM02',69754,'In Earth orbit','','2014-08-31 00:00:00',702.67,13702,25905,49.69,'Medium' from dual union all
select 4013,'004013','1968-097CA','deb Kosmos-252 [*]','',16824,'Reentered','1980-10-16 00:00:00','1980-11-13 00:00:00',95.7,435,670,62.45,'Intermediate' from dual union all
select 40130,'040130','2014-050C','Fregat-MT No. 1039','Fregat-MT    No. 1039',69754,'In Earth orbit','','2014-08-25 00:00:00',702.33,13713,25877,49.71,'Medium' from dual union all
select 40131,'040131','1998-067FA','Flock 1b-1','Dove 0813',66503,'Reentered','2015-01-01 00:00:00','2014-08-23 00:00:00',92.83,409,418,51.64,'Intermediate' from dual union all
select 40132,'040132','1998-067FB','Flock 1b-2','Dove 080C',66503,'Reentered','2015-03-14 00:00:00','2014-08-22 00:00:00',92.84,409,419,51.64,'Intermediate' from dual union all
select 40133,'040133','1998-067FC','Flock 1b-8','Dove 0811',66503,'Reentered','2015-03-05 00:00:00','2014-08-24 00:00:00',92.84,408,420,51.64,'Intermediate' from dual union all
select 40134,'040134','1998-067FD','Flock 1b-7','Dove 080F',66503,'Reentered','2015-01-20 00:00:00','2014-08-24 00:00:00',92.83,408,418,51.65,'Intermediate' from dual union all
select 40135,'040135','2009-073C','deb Helios 2B','',68754,'In Earth orbit','','2014-09-05 00:00:00',98.35,678,682,98.47,'Sun-Synch' from dual union all
select 40136,'040136','2014-051A','Lingqiao Tongxin Shiyan W.','lingqiao tongxin shiyan',69755,'In Earth orbit','','2014-09-06 00:00:00',100.71,778,807,98.46,'Sun-Synch' from dual union all
select 40137,'040137','2014-051B','Chuangxin-1 04','chuangxin yihao 04 xing',69755,'In Earth orbit','','2014-09-06 00:00:00',100.73,778,809,98.46,'Sun-Synch' from dual union all
select 40138,'040138','2014-051C','CZ-2D Stage 2','CZ-2D Stage 2',69755,'Reentered','2015-06-13 00:00:00','2014-09-09 00:00:00',95.48,254,830,98,'Sun-Synch' from dual union all
select 40139,'040139','1998-067FE','Flock 1b-18','Dove 080E',66503,'Reentered','2015-01-16 00:00:00','2014-09-05 00:00:00',92.83,409,418,51.65,'Intermediate' from dual union all
select 4014,'004014','61 Omicron 228','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-02 00:00:00',117.76,914,2247,67.59,'Intermediate' from dual union all
select 40140,'040140','1998-067FF','Flock 1b-17','Dove 081E',66503,'Reentered','2015-03-04 00:00:00','2014-09-05 00:00:00',92.83,409,418,51.65,'Intermediate' from dual union all
select 40141,'040141','2014-052A','Asiasat 6','Asiasat 6',69756,'In Earth orbit','','2014-09-11 00:00:00',1071.24,20843,35762,3.07,'Medium' from dual union all
select 40142,'040142','2014-052B','Falcon 9-013 Stage 2','F9-013 St2',69756,'Reentered','2014-12-28 00:00:00','2014-09-10 00:00:00',628.98,155,35724,25.38,'GEO Transfer' from dual union all
select 40143,'040143','2014-053A','Yaogan Weixing 21','Jianbing-10 03 xing',69757,'In Earth orbit','','2014-09-10 00:00:00',94.33,479,494,97.42,'Sun-Synch' from dual union all
select 40144,'040144','2014-053B','Tiantuo 2','tiantuo erhao weixing',69757,'In Earth orbit','','2014-09-10 00:00:00',94.29,476,493,97.41,'Sun-Synch' from dual union all
select 40145,'040145','2014-053C','CZ-4B Stage 3','CZ-4B Stage 3',69757,'Reentered','2014-10-23 00:00:00','2014-09-10 00:00:00',91.53,218,482,97.51,'Sun-Synch' from dual union all
select 40146,'040146','2014-054A','Optus 10','Optus 10',69758,'In Earth orbit','','2014-09-14 00:00:00',1031.69,19159,35728,1.05,'Medium' from dual union all
select 40147,'040147','2014-054B','Measat 3B','Measat 3B',69758,'In Earth orbit','','2014-09-14 00:00:00',758.31,6574,35762,2.87,'Highly Ellip' from dual;
insert into SATELLITE
select 40148,'040148','2014-054C','ESC-A 573','Ariane 5 ESC-A',69758,'In Earth orbit','','2014-09-14 00:00:00',633.2,251,35845,5.53,'GEO Transfer' from dual union all
select 40149,'040149','2014-054D','Sylda 5A','Sylda 5 No. 59A',69758,'In Earth orbit','','2014-09-15 00:00:00',630.65,251,35714,5.96,'GEO Transfer' from dual union all
select 4015,'004015','61 Omicron 229','deb r/b Ablestar','',7273,'Reentered','2002-03-01 00:00:00','1992-02-04 00:00:00',102.41,817,929,66.44,'Intermediate' from dual union all
select 40150,'040150','2014-041C','deb Foton-M4','',69745,'Reentered','2014-11-15 00:00:00','2014-09-16 00:00:00',95.58,301,792,64.73,'Intermediate' from dual union all
select 40151,'040151','2014-041D','deb Foton-M4','',69745,'Reentered','2015-01-26 00:00:00','2014-09-16 00:00:00',96.51,529,654,65.01,'Intermediate' from dual union all
select 40152,'040152','2007-003AL','deb CZ-3','',68345,'In Earth orbit','','2014-09-14 00:00:00',754.95,1062,41111,28.79,'Highly Ellip' from dual union all
select 40153,'040153','2007-003AM','deb CZ-3','',68345,'In Earth orbit','','2009-09-24 00:00:00',865.88,708,46722,25.49,'Highly Ellip' from dual union all
select 40154,'040154','2007-003AN','deb CZ-3','',68345,'In Earth orbit','','2009-07-25 00:00:00',861.33,726,46493,25.66,'Highly Ellip' from dual union all
select 40155,'040155','2007-003AP','deb CZ-3','',68345,'In Earth orbit','','2013-03-01 00:00:00',721.35,808,39723,24.41,'GEO Transfer' from dual union all
select 40156,'040156','2007-003AQ','deb CZ-3','',68345,'In Earth orbit','','2014-09-10 00:00:00',782.33,613,42879,26.98,'Highly Ellip' from dual union all
select 40157,'040157','2007-003AR','deb CZ-3','',68345,'In Earth orbit','','2013-02-20 00:00:00',785.92,796,42869,23.65,'Highly Ellip' from dual union all
select 40158,'040158','2010-042K','deb CZ-3B','',68902,'In Earth orbit','','2014-09-15 00:00:00',763.25,954,41621,25.51,'Highly Ellip' from dual union all
select 40159,'040159','2010-042L','deb CZ-3B','',68902,'In Earth orbit','','2014-09-10 00:00:00',678.38,427,37966,26.68,'GEO Transfer' from dual union all
select 4016,'004016','1965-020EZ','deb Blok Kosmos','',11770,'Reentered','1981-04-13 00:00:00','1969-07-07 00:00:00',106.64,481,1661,56.08,'Intermediate' from dual union all
select 40160,'040160','2010-042M','deb CZ-3B','',68902,'In Earth orbit','','2014-09-12 00:00:00',758.37,1088,41251,26.28,'Highly Ellip' from dual union all
select 40161,'040161','2010-042N','deb CZ-3B','',68902,'In Earth orbit','','2014-09-14 00:00:00',723.02,676,39937,27.51,'GEO Transfer' from dual union all
select 40162,'040162','2010-042P','deb CZ-3B','',68902,'In Earth orbit','','2014-09-14 00:00:00',749.86,1139,40787,26.13,'GEO Transfer' from dual union all
select 40163,'040163','2010-042Q','deb CZ-3B','',68902,'In Earth orbit','','2013-04-21 00:00:00',748.18,790,41054,28.11,'GEO Transfer' from dual union all
select 40164,'040164','2010-042R','deb CZ-3B','',68902,'In Earth orbit','','2014-09-15 00:00:00',758.48,1008,41336,26.26,'Highly Ellip' from dual union all
select 40165,'040165','2010-042S','deb CZ-3B','',68902,'In Earth orbit','','2013-02-21 00:00:00',746.9,797,40985,28.45,'GEO Transfer' from dual union all
select 40166,'040166','2010-042T','deb CZ-3B','',68902,'In Earth orbit','','2014-09-15 00:00:00',732.43,1421,39654,25.73,'GEO Transfer' from dual union all
select 40167,'040167','2010-042U','deb CZ-3B','',68902,'In Earth orbit','','2014-09-15 00:00:00',743.7,1267,40359,26.22,'GEO Transfer' from dual union all
select 40168,'040168','2010-042V','deb CZ-3B','',68902,'In Earth orbit','','2014-06-29 00:00:00',715.98,763,39503,24,'GEO Transfer' from dual union all
select 40169,'040169','2010-042W','deb CZ-3B','',68902,'In Earth orbit','','2014-09-12 00:00:00',742.59,1141,40431,25.85,'GEO Transfer' from dual union all
select 4017,'004017','1965-082SN','deb Transtage 4','',11832,'Reentered','1979-08-13 00:00:00','1979-03-03 00:00:00',98.52,605,771,31.7,'Intermediate' from dual union all
select 40170,'040170','2010-057Y','deb CZ-3C','',68917,'In Earth orbit','','2014-09-15 00:00:00',644.39,659,36011,20.27,'GEO Transfer' from dual union all
select 40171,'040171','2010-057Z','deb CZ-3C','',68917,'In Earth orbit','','2014-05-19 00:00:00',431.92,341,24758,22.04,'Highly Ellip' from dual union all
select 40172,'040172','2010-057AA','deb CZ-3C','',68917,'In Earth orbit','','2014-08-27 00:00:00',183.6,200,8444,20,'Medium' from dual union all
select 40173,'040173','2010-057AB','deb CZ-3C','',68917,'In Earth orbit','','2014-01-06 00:00:00',630.36,541,35409,22.99,'GEO Transfer' from dual union all
select 40174,'040174','2010-057AC','deb CZ-3C','',68917,'In Earth orbit','','2013-12-16 00:00:00',617.28,539,34735,19.72,'GEO Transfer' from dual union all
select 40175,'040175','2010-057AD','deb CZ-3C','',68917,'In Earth orbit','','2014-09-13 00:00:00',606.17,426,34270,19.31,'GEO Transfer' from dual union all
select 40176,'040176','2010-057AE','deb CZ-3C','',68917,'In Earth orbit','','2013-03-30 00:00:00',692.67,842,38267,20.84,'GEO Transfer' from dual union all
select 40177,'040177','2010-057AF','deb CZ-3C','',68917,'In Earth orbit','','2014-09-12 00:00:00',657.97,578,36784,17.34,'GEO Transfer' from dual union all
select 40178,'040178','2010-057AG','deb CZ-3C','',68917,'In Earth orbit','','2014-09-07 00:00:00',687.37,742,38102,18.62,'GEO Transfer' from dual union all
select 40179,'040179','2011-077C','deb CZ-3B','',69129,'In Earth orbit','','2014-08-28 00:00:00',751.22,679,41312,25.72,'Highly Ellip' from dual union all
select 4018,'004018','1968-097CB','deb Kosmos-252','',16824,'Reentered','1982-11-10 00:00:00','1969-07-06 00:00:00',111.82,528,2093,62.24,'Intermediate' from dual union all
select 40180,'040180','2011-077D','deb CZ-3B','',69129,'In Earth orbit','','2014-05-04 00:00:00',757.24,461,41822,23.91,'Highly Ellip' from dual union all
select 40181,'040181','2011-077E','deb CZ-3B','',69129,'In Earth orbit','','2014-04-25 00:00:00',781.74,369,43096,22.02,'Highly Ellip' from dual union all
select 40182,'040182','2011-077F','deb CZ-3B','',69129,'In Earth orbit','','2014-09-07 00:00:00',783.95,480,43090,22.86,'Highly Ellip' from dual union all
select 40183,'040183','2011-077G','deb CZ-3B','',69129,'In Earth orbit','','2014-08-30 00:00:00',795.17,483,43624,22.2,'Highly Ellip' from dual union all
select 40184,'040184','2011-077H','deb CZ-3B','',69129,'In Earth orbit','','2014-09-14 00:00:00',781.62,373,43086,22.37,'Highly Ellip' from dual union all
select 40185,'040185','2011-077J','deb CZ-3B','',69129,'In Earth orbit','','2014-09-09 00:00:00',754.6,511,41645,25.47,'Highly Ellip' from dual union all
select 40186,'040186','2011-077K','deb CZ-3B','',69129,'In Earth orbit','','2014-08-31 00:00:00',763.17,1105,41466,25.9,'Highly Ellip' from dual union all
select 40187,'040187','2011-077L','deb CZ-3B','',69129,'In Earth orbit','','2014-09-15 00:00:00',754.43,808,41340,26.06,'Highly Ellip' from dual union all
select 40188,'040188','2011-077M','deb CZ-3B','',69129,'In Earth orbit','','2014-09-14 00:00:00',753.92,789,41334,26.66,'Highly Ellip' from dual union all
select 40189,'040189','2011-077N','deb CZ-3B','',69129,'In Earth orbit','','2014-09-15 00:00:00',756.68,898,41359,26.39,'Highly Ellip' from dual union all
select 4019,'004019','1964-074C','deb Explorer XXIII','',10360,'Reentered','1969-07-15 00:00:00','1969-07-06 00:00:00',92.38,177,606,51.94,'Intermediate' from dual union all
select 40190,'040190','2011-077P','deb CZ-3B','',69129,'Lost','','2014-02-22 00:00:00',658.9,422,36987,26.74,'GEO Transfer' from dual union all
select 40191,'040191','2011-077Q','deb CZ-3B','',69129,'In Earth orbit','','2014-08-08 00:00:00',744.56,562,41106,26.47,'GEO Transfer' from dual union all
select 40192,'040192','2011-077R','deb CZ-3B','',69129,'In Earth orbit','','2014-06-16 00:00:00',755.18,762,41422,25.67,'Highly Ellip' from dual union all
select 40193,'040193','2011-077S','deb CZ-3B','',69129,'In Earth orbit','','2014-05-24 00:00:00',771.71,880,42103,26.14,'Highly Ellip' from dual union all
select 40194,'040194','2012-008D','deb CZ-3C','',69263,'In Earth orbit','','2014-09-12 00:00:00',675.81,2794,35470,20.02,'GEO Transfer' from dual union all
select 40195,'040195','2012-008E','deb CZ-3C','',69263,'In Earth orbit','','2014-09-15 00:00:00',626.38,422,35323,20.52,'GEO Transfer' from dual union all
select 40196,'040196','2012-008F','deb CZ-3C','',69263,'In Earth orbit','','2014-09-13 00:00:00',607.57,564,34205,22.03,'GEO Transfer' from dual union all
select 40197,'040197','2012-008G','deb CZ-3C','',69263,'In Earth orbit','','2014-09-14 00:00:00',655.16,1147,36072,20.3,'GEO Transfer' from dual union all
select 40198,'040198','2012-008H','deb CZ-3C','',69263,'In Earth orbit','','2014-09-15 00:00:00',666.73,517,37289,19.47,'GEO Transfer' from dual union all
select 40199,'040199','2012-008J','deb CZ-3C','',69263,'In Earth orbit','','2014-09-10 00:00:00',689.45,794,38154,18.19,'GEO Transfer' from dual union all
select 402,'000402','61 Omicron 122','deb r/b Ablestar','',7273,'In Earth orbit','','1962-09-20 00:00:00',104.56,920,1028,66.71,'Intermediate' from dual union all
select 4020,'004020','1965-082SP','deb Transtage 4','',11832,'Reentered','1979-06-16 00:00:00','1979-03-28 00:00:00',96.71,472,730,30.4,'Intermediate' from dual union all
select 40200,'040200','2012-008K','deb CZ-3C','',69263,'In Earth orbit','','2014-09-10 00:00:00',656.98,1130,36182,20.62,'GEO Transfer' from dual union all
select 40201,'040201','2012-008L','deb CZ-3C','',69263,'In Earth orbit','','2014-09-11 00:00:00',641.34,803,35711,21.39,'GEO Transfer' from dual union all
select 40202,'040202','2012-008M','deb CZ-3C','',69263,'In Earth orbit','','2014-09-14 00:00:00',619.02,477,34886,20.29,'GEO Transfer' from dual union all
select 40203,'040203','2012-008N','deb CZ-3C','',69263,'In Earth orbit','','2014-09-14 00:00:00',617.48,474,34810,20.9,'GEO Transfer' from dual union all
select 40204,'040204','2012-008P','deb CZ-3C','',69263,'In Earth orbit','','2014-09-07 00:00:00',634.58,734,35433,20.49,'GEO Transfer' from dual union all
select 40205,'040205','2012-008Q','deb CZ-3C','',69263,'In Earth orbit','','2014-09-08 00:00:00',656.1,1092,36175,20.24,'GEO Transfer' from dual union all
select 40206,'040206','2012-008R','deb CZ-3C','',69263,'In Earth orbit','','2014-09-11 00:00:00',661.1,566,36955,18.14,'GEO Transfer' from dual union all
select 40207,'040207','2012-008S','deb CZ-3C','',69263,'In Earth orbit','','2014-09-14 00:00:00',649.66,516,36423,19.22,'GEO Transfer' from dual union all
select 40208,'040208','2014-055A','USA 257','NEMESIS 2',69759,'In Earth orbit','','2014-09-17 00:00:00',880.16,11800,36290,25,'Medium' from dual union all
select 40209,'040209','2014-055B','Centaur AV-049','Centaur AV-049',69759,'In Earth orbit','','2014-09-17 00:00:00',500.97,174,28859,27.91,'Highly Ellip' from dual union all
select 4021,'004021','1965-082SU','deb Transtage 4','',11832,'Reentered','1981-02-06 00:00:00','1981-02-04 00:00:00',90.6,292,316,32.74,'Intermediate' from dual union all
select 40210,'040210','2014-056A','Dragon CRS-4','Dragon C106',69760,'Landed','2014-10-25 00:00:00','2014-09-21 00:00:00',90.05,203,351,51.68,'Intermediate' from dual union all
select 40211,'040211','2014-056B','Dragon solar array cover','',69760,'Reentered','2014-09-23 00:00:00','2014-09-21 00:00:00',90.1,202,357,51.66,'Intermediate' from dual union all
select 40212,'040212','2014-056C','Dragon solar array cover','',69760,'Reentered','2014-09-24 00:00:00','2014-09-21 00:00:00',90.1,202,357,51.67,'Intermediate' from dual union all
select 40213,'040213','1979-095CL','deb Meteor-2','',47579,'In Earth orbit','','2014-09-21 00:00:00',101.79,837,850,81.21,'Intermediate' from dual union all
select 40214,'040214','1979-095CM','deb Meteor-2','',47579,'In Earth orbit','','2014-09-24 00:00:00',102.1,850,867,81.21,'Intermediate' from dual union all
select 40215,'040215','1979-095CN','deb Meteor-2','',47579,'In Earth orbit','','2014-09-24 00:00:00',102.14,859,861,81.2,'Intermediate' from dual union all
select 40216,'040216','1986-088E','deb Polar BEAR','',59281,'In Earth orbit','','2014-09-23 00:00:00',104.62,949,1005,89.53,'Polar' from dual union all
select 40217,'040217','1988-056C','deb Okean-O1','',61305,'In Earth orbit','','2014-09-24 00:00:00',95.75,545,565,82.51,'Intermediate' from dual union all
select 40218,'040218','1993-036BUX','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-24 00:00:00',100.34,761,789,74.02,'Intermediate' from dual union all
select 40219,'040219','1993-036BUY','deb Kosmos-2251','',65138,'Reentered','2016-05-30 00:00:00','2014-09-19 00:00:00',98.11,652,685,74.02,'Intermediate' from dual union all
select 4022,'004022','1969-056C','deb Bios 3 capsule','',18323,'Reentered','1969-07-13 00:00:00','1969-07-08 00:00:00',93.38,336,543,33.79,'Intermediate' from dual union all
select 40220,'040220','1993-036BUZ','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-24 00:00:00',99.55,722,753,74.04,'Intermediate' from dual union all
select 40221,'040221','1993-036BVA','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-24 00:00:00',99.28,559,890,73.91,'Intermediate' from dual union all
select 40222,'040222','1993-036BVB','deb Kosmos-2251','',65138,'Reentered','2015-12-15 00:00:00','2014-09-21 00:00:00',97.63,624,667,74.02,'Intermediate' from dual union all
select 40223,'040223','1993-036BVC','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-21 00:00:00',100.34,737,812,73.97,'Intermediate' from dual union all
select 40224,'040224','1993-036BVD','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-21 00:00:00',99.32,621,831,74.04,'Intermediate' from dual union all
select 40225,'040225','1993-036BVE','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-22 00:00:00',99.39,714,745,74.05,'Intermediate' from dual union all
select 40226,'040226','1993-036BVF','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-15 00:00:00',98.41,616,750,74.09,'Intermediate' from dual union all
select 40227,'040227','1993-036BVG','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-18 00:00:00',98.43,670,698,74.04,'Intermediate' from dual union all
select 40228,'040228','1993-036BVH','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-20 00:00:00',100.31,754,793,74.06,'Intermediate' from dual union all
select 40229,'040229','1993-036BVJ','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-11 00:00:00',98.06,605,726,74.01,'Intermediate' from dual union all
select 4023,'004023','1965-082SQ','deb Transtage 4','',11832,'Reentered','1970-02-02 00:00:00','1969-07-07 00:00:00',100.92,656,949,31.51,'Intermediate' from dual union all
select 40230,'040230','1993-036BVK','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-24 00:00:00',100.28,753,790,74.05,'Intermediate' from dual union all
select 40231,'040231','1993-036BVL','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-17 00:00:00',97.79,590,715,74.02,'Intermediate' from dual union all
select 40232,'040232','1993-036BVM','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-23 00:00:00',98.83,672,734,74.13,'Intermediate' from dual union all
select 40233,'040233','1993-036BVN','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-23 00:00:00',100.13,718,811,73.95,'Intermediate' from dual union all
select 40234,'040234','1993-036BVP','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-24 00:00:00',99.29,705,745,74.03,'Intermediate' from dual union all
select 40235,'040235','1993-036BVQ','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-21 00:00:00',99.92,740,769,74.03,'Intermediate' from dual union all
select 40236,'040236','1993-036BVR','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-23 00:00:00',109.23,766,1617,73.53,'Intermediate' from dual union all
select 40237,'040237','1993-036BVS','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-24 00:00:00',99.9,747,761,74.03,'Intermediate' from dual;
insert into SATELLITE
select 40238,'040238','1993-036BVT','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-23 00:00:00',109.45,754,1649,74,'Intermediate' from dual union all
select 40239,'040239','1993-036BVU','deb Kosmos-2251','',65138,'In Earth orbit','','2014-09-24 00:00:00',99.12,646,788,74.03,'Intermediate' from dual union all
select 4024,'004024','1968-097CC','deb Kosmos-252','',16824,'Reentered','1991-09-06 00:00:00','1991-09-04 00:00:00',90.9,285,352,62.59,'Intermediate' from dual union all
select 40240,'040240','1994-029AGG','deb HAPS','',65425,'Reentered','2015-02-28 00:00:00','2014-09-24 00:00:00',93.9,430,501,81.57,'Intermediate' from dual union all
select 40241,'040241','1994-029AGH','deb HAPS','',65425,'Reentered','2015-12-02 00:00:00','2014-09-24 00:00:00',96.19,468,685,82.1,'Intermediate' from dual union all
select 40242,'040242','1994-029AGJ','deb HAPS','',65425,'In Earth orbit','','2014-09-24 00:00:00',103.1,572,1239,81.88,'Intermediate' from dual union all
select 40243,'040243','1994-029AGK','deb HAPS','',65425,'In Earth orbit','','2014-09-23 00:00:00',102.83,566,1218,81.96,'Intermediate' from dual union all
select 40244,'040244','1994-029AGL','deb HAPS','',65425,'In Earth orbit','','2014-09-23 00:00:00',100.07,572,951,81.92,'Intermediate' from dual union all
select 40245,'040245','2008-046T','deb SOZ','',68554,'In Earth orbit','','2014-09-23 00:00:00',331.72,687,18304,65.63,'Highly Ellip' from dual union all
select 40246,'040246','2014-057A','Soyuz TMA-14M','Soyuz 11F732A47 No. 714',69761,'Landed','2015-03-12 00:00:00','2014-09-25 00:00:00',89.47,169,328,52.06,'Intermediate' from dual union all
select 40247,'040247','2014-057B','11S510','11S510',69761,'Reentered','2014-09-28 00:00:00','2014-09-26 00:00:00',88.63,191,223,51.63,'Intermediate' from dual union all
select 40248,'040248','1997-082H','deb Iridium 47','',66162,'In Earth orbit','','2014-09-18 00:00:00',100.52,718,849,85.88,'Polar' from dual union all
select 40249,'040249','1997-082J','deb Iridium 47','',66162,'In Earth orbit','','2014-09-26 00:00:00',100.4,775,780,86.4,'Polar' from dual union all
select 4025,'004025','1965-082SR','deb Transtage 4','',11832,'Reentered','1969-10-16 00:00:00','1969-07-08 00:00:00',104.41,690,1243,33.26,'Intermediate' from dual union all
select 40250,'040250','1997-082K','deb Iridium 47','',66162,'Reentered','2015-01-29 00:00:00','2014-09-26 00:00:00',98.95,691,725,85.89,'Polar' from dual union all
select 40251,'040251','1997-082L','deb Iridium 47','',66162,'In Earth orbit','','2014-09-26 00:00:00',103.21,773,1048,86.35,'Polar' from dual union all
select 40252,'040252','1997-082M','deb Iridium 47','',66162,'In Earth orbit','','2014-09-26 00:00:00',99.95,744,769,86.4,'Polar' from dual union all
select 40253,'040253','1997-082N','deb Iridium 47','',66162,'Reentered','2015-05-22 00:00:00','2014-09-26 00:00:00',99.52,729,743,86.14,'Polar' from dual union all
select 40254,'040254','1997-082P','deb Iridium 47','',66162,'In Earth orbit','','2014-09-26 00:00:00',103.65,772,1091,86.04,'Polar' from dual union all
select 40255,'040255','1997-082Q','deb Iridium 47','',66162,'In Earth orbit','','2014-09-26 00:00:00',103.8,773,1104,86.28,'Polar' from dual union all
select 40256,'040256','1997-082R','deb Iridium 47','',66162,'In Earth orbit','','2014-09-26 00:00:00',98.93,656,759,86.29,'Polar' from dual union all
select 40257,'040257','1997-082S','deb Iridium 47','',66162,'Reentered','2015-03-29 00:00:00','2014-09-26 00:00:00',99.39,701,758,86.31,'Polar' from dual union all
select 40258,'040258','2014-058A','Luch','Olimp-K',69762,'In Earth orbit','','2014-10-28 00:00:00',1436.09,35785,35788,.05,'Stationary' from dual union all
select 40259,'040259','2014-058B','Briz-M No. 99549','Briz-M 14S43 No. 99549',69762,'In Earth orbit','','2014-10-05 00:00:00',1571.1,35688,41091,.11,'Very High' from dual union all
select 4026,'004026','61 Delta 7','deb Explorer IX','',7263,'In Earth orbit','','1969-02-15 00:00:00',117.51,639,2499,38.84,'Intermediate' from dual union all
select 40260,'040260','2014-058C','Briz-M DTB','Briz-M torus tank',69762,'In Earth orbit','','2014-10-28 00:00:00',634.23,483,35666,46.26,'GEO Transfer' from dual union all
select 40261,'040261','2014-059A','Shi Jian 11-07','shijian shi yihao 07 xin',69763,'In Earth orbit','','2014-10-28 00:00:00',98.68,685,707,98.12,'Sun-Synch' from dual union all
select 40262,'040262','2014-059B','CZ-2C Stage 2','CZ-2C Stage 2',69763,'In Earth orbit','','2014-10-29 00:00:00',97.94,627,694,98.16,'Sun-Synch' from dual union all
select 40263,'040263','2014-059C','CZ-2C Stage 2 motor cover','',69763,'In Earth orbit','','2014-10-29 00:00:00',99.52,653,819,98.33,'Sun-Synch' from dual union all
select 40264,'040264','2014-059D','CZ-2C Stage 2 motor cover','',69763,'In Earth orbit','','2014-10-05 00:00:00',99.6,656,823,97.89,'Sun-Synch' from dual union all
select 40265,'040265','2014-059E','CZ-2C Stage 2 motor cover','',69763,'In Earth orbit','','2014-10-29 00:00:00',101.54,687,977,97.94,'Sun-Synch' from dual union all
select 40266,'040266','2014-059F','CZ-2C Stage 2 motor cover','',69763,'In Earth orbit','','2014-10-29 00:00:00',101.48,687,971,98.35,'Sun-Synch' from dual union all
select 40267,'040267','2014-060A','Himawari-8','Himawari 8',69764,'In Earth orbit','','2014-10-15 00:00:00',1435.98,35756,35813,.08,'Stationary' from dual union all
select 40268,'040268','2014-060B','H-2A F25 Stage 2','H-2A Stage 2',69764,'In Earth orbit','','2014-10-16 00:00:00',620.87,277,35182,22.41,'GEO Transfer' from dual union all
select 40269,'040269','2014-061A','IRNSS-R1C','IRNSS-1C',69765,'In Earth orbit','','2014-10-23 00:00:00',1432.08,35638,35778,4.98,'Inclined Drift' from dual union all
select 4027,'004027','61 Omicron 230','deb r/b Ablestar','',7273,'Reentered','1970-12-19 00:00:00','1967-07-31 00:00:00',112.38,260,2412,64.61,'Intermediate' from dual union all
select 40270,'040270','2014-061B','PSLV-C26 PS4','PS4',69765,'In Earth orbit','','2014-10-23 00:00:00',366.56,267,20910,17.78,'Highly Ellip' from dual union all
select 40271,'040271','2014-062A','Intelsat IS-30','Intelsat ISDLA-1',69766,'In Earth orbit','','2014-10-24 00:00:00',1428.56,35445,35833,.07,'Drift GEO' from dual union all
select 40272,'040272','2014-062B','ARSAT-1','ARSAT-1',69766,'In Earth orbit','','2014-10-28 00:00:00',1436.96,35782,35825,.01,'Drift GEO' from dual union all
select 40273,'040273','2014-062C','ESC-A 574','Ariane 5 ESC-A',69766,'In Earth orbit','','2014-10-26 00:00:00',630.5,261,35696,6.01,'GEO Transfer' from dual union all
select 40274,'040274','2014-062D','Sylda 5C','Sylda 5 No. 61C',69766,'In Earth orbit','','2014-10-30 00:00:00',629.08,263,35621,5.94,'GEO Transfer' from dual union all
select 40275,'040275','2014-063A','Yaogan Weixing 22','Jianbing-9 04 xing',69767,'In Earth orbit','','2014-10-27 00:00:00',109.46,1197,1207,100.32,'Sun-Synch' from dual union all
select 40276,'040276','2014-063B','CZ-4C Stage 3','CZ-4C Stage 3',69767,'In Earth orbit','','2014-10-27 00:00:00',107.22,991,1205,100.27,'Sun-Synch' from dual union all
select 40277,'040277','2014-064A','Ekspress-AM6','Ekspress AM-6',69768,'In Earth orbit','','2014-10-29 00:00:00',1373.26,31305,37790,.7,'Medium' from dual union all
select 40278,'040278','2014-064B','Briz-M No. 99550','Briz-M 14S43 No. 99550',69768,'In Earth orbit','','2014-10-29 00:00:00',1512.2,34898,39629,.98,'Very High' from dual union all
select 40279,'040279','2014-064C','Briz-M DTB','Briz-M torus tank',69768,'Reentered','2015-06-17 00:00:00','2014-10-29 00:00:00',672.58,361,37740,49.51,'GEO Transfer' from dual union all
select 4028,'004028','61 Omicron 231','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-04 00:00:00',100.35,678,872,66.76,'Intermediate' from dual union all
select 40280,'040280','1998-067FG','RK-21-8','ISS RS SVCh-Radiometria',66503,'Reentered','2015-03-11 00:00:00','2014-10-22 00:00:00',92.83,411,416,51.64,'Intermediate' from dual union all
select 40281,'040281','1998-067FH','MIM-2 2ASF1-1 antenna','',66503,'Reentered','2015-02-07 00:00:00','2014-10-22 00:00:00',92.8,409,415,51.64,'Intermediate' from dual union all
select 40282,'040282','1998-067FJ','EVA debris','',66503,'Reentered','2014-11-28 00:00:00','2014-10-22 00:00:00',92.83,412,415,51.64,'Intermediate' from dual union all
select 40283,'040283','2014-065A','Chang''e-5 FTD','Chang''e 5 feixing shiyan',69769,'Deep Space','2014-10-24 00:00:00','2014-10-25 00:00:00',13584.48,1600,366027,30.66,'Cislunar' from dual union all
select 40284,'040284','2014-065B','CZ-3C/G2  Stage 3','CZ-3C Stage 3',69769,'Deep Space','2014-10-24 00:00:00','2014-10-25 00:00:00',13584.48,1600,366027,30.66,'Cislunar' from dual union all
select 40285,'040285','1998-067FK','MIM-2 2ASF1-2 antenna','',66503,'Reentered','2015-02-07 00:00:00','2014-10-24 00:00:00',92.79,409,414,51.65,'Intermediate' from dual union all
select 40286,'040286','2014-066A','Shi Jian 11-08','shijian shi yihao 08 xin',69770,'In Earth orbit','','2014-11-03 00:00:00',98.66,687,703,98.21,'Sun-Synch' from dual union all
select 40287,'040287','2014-066B','CZ-2C Stage 2','CZ-2C Stage 2',69770,'In Earth orbit','','2014-11-03 00:00:00',98.37,674,687,97.95,'Sun-Synch' from dual union all
select 40288,'040288','2014-066C','CZ-2C Stage 2 motor cover','',69770,'In Earth orbit','','2014-11-04 00:00:00',101.51,690,971,97.97,'Sun-Synch' from dual union all
select 40289,'040289','2014-066D','CZ-2C Stage 2 motor cover','',69770,'In Earth orbit','','2014-11-03 00:00:00',99.54,655,819,98.43,'Sun-Synch' from dual union all
select 4029,'004029','61 Omicron 232','deb r/b Ablestar','',7273,'Reentered','2000-05-10 00:00:00','1992-02-05 00:00:00',99.61,724,755,66.99,'Intermediate' from dual union all
select 40290,'040290','2014-066E','CZ-2C Stage 2 motor cover','',69770,'In Earth orbit','','2014-11-03 00:00:00',99.39,655,804,97.96,'Sun-Synch' from dual union all
select 40291,'040291','2014-066F','CZ-2C Stage 2 motor cover','',69770,'In Earth orbit','','2014-11-04 00:00:00',101.3,690,951,98.44,'Sun-Synch' from dual union all
select 40292,'040292','2014-067A','Progress M-25M','Progress 7K-TGM No.424',69771,'Deorbited','2015-04-26 00:00:00','2014-10-29 00:00:00',90.07,258,298,51.65,'Intermediate' from dual union all
select 40293,'040293','2014-067B','Blok-I 14S54/RD-0110','374BL02A',69771,'Reentered','2014-10-30 00:00:00','2014-10-30 00:00:00',88.13,172,192,51.65,'Intermediate' from dual union all
select 40294,'040294','2014-068A','GPS IIF-8','GPS SVN 69',69772,'In Earth orbit','','2014-11-07 00:00:00',729.53,20459,20474,54.97,'Medium' from dual union all
select 40295,'040295','2014-068B','Centaur AV-050','Centaur AV-050',69772,'In Earth orbit','','2014-10-29 00:00:00',730.98,20445,20559,55.18,'Medium' from dual union all
select 40296,'040296','2014-069A','Meridian No. 17L','Meridian No. 17L',69773,'In Earth orbit','','2014-11-06 00:00:00',725.13,965,39752,62.81,'Molniya' from dual union all
select 40297,'040297','2014-069B','Fregat No. 1026','Fregat 14S44 No. 1026',69773,'In Earth orbit','','2014-11-08 00:00:00',718.27,808,39570,62.78,'Molniya' from dual union all
select 40298,'040298','2014-070A','ASNARO-1','ASNARO 1',69774,'In Earth orbit','','2014-11-13 00:00:00',94.75,506,507,97.48,'Sun-Synch' from dual union all
select 40299,'040299','2014-070B','Hodoyoshi-1','Hodoyoshi-1',69774,'In Earth orbit','','2014-11-13 00:00:00',94.91,505,524,97.47,'Sun-Synch' from dual union all
select 403,'000403','61 Omicron 123','deb r/b Ablestar','',7273,'In Earth orbit','','1962-09-20 00:00:00',104.56,955,992,66.86,'Intermediate' from dual union all
select 4030,'004030','1968-097CD','deb Kosmos-252 [*]','',16824,'Reentered','1982-02-02 00:00:00','1969-07-12 00:00:00',113.95,592,2224,62.33,'Intermediate' from dual union all
select 40300,'040300','2014-070C','Kinshachi 1','ChubuSat-1',69774,'In Earth orbit','','2014-11-13 00:00:00',95.05,504,538,97.47,'Sun-Synch' from dual union all
select 40301,'040301','2014-070D','Tsukushi','QSAT-EOS',69774,'In Earth orbit','','2014-11-13 00:00:00',95.21,504,554,97.47,'Sun-Synch' from dual union all
select 40302,'040302','2014-070E','Tsubame','Tsubame',69774,'In Earth orbit','','2014-11-13 00:00:00',95.36,504,568,97.47,'Sun-Synch' from dual union all
select 40303,'040303','2014-070F','R-36M2 DS','R-36M2 DS',69774,'In Earth orbit','','2014-11-13 00:00:00',94.45,479,504,97.48,'Sun-Synch' from dual union all
select 40304,'040304','2014-070G','Gazodinamicheskiy Ekran','GDS (Gas dynamic shield)',69774,'Reentered','2017-12-29 00:00:00','2014-11-13 00:00:00',109.71,484,1943,97.17,'Sun-Synch' from dual union all
select 40305,'040305','2014-071A','Yaogan Weixing 23','Jianbing-7 04 xing',69775,'In Earth orbit','','2014-11-16 00:00:00',94.88,512,514,97.34,'Sun-Synch' from dual union all
select 40306,'040306','2014-071B','CZ-2C Stage 2','CZ-2C Stage 2',69775,'In Earth orbit','','2014-11-16 00:00:00',94.33,470,503,97.14,'Sun-Synch' from dual union all
select 40307,'040307','2014-071C','Sep motor cover','',69775,'Reentered','2014-12-13 00:00:00','2014-11-17 00:00:00',93.13,343,513,97.1,'Sun-Synch' from dual union all
select 40308,'040308','2014-071D','Sep motor cover','',69775,'Reentered','2014-12-21 00:00:00','2014-11-17 00:00:00',93.14,342,515,97.52,'Sun-Synch' from dual union all
select 40309,'040309','2014-071E','Sep motor cover','',69775,'Reentered','2016-01-11 00:00:00','2014-11-17 00:00:00',96.09,482,661,97.13,'Sun-Synch' from dual union all
select 4031,'004031','1968-097CE','deb Kosmos-252','',16824,'Reentered','1982-07-30 00:00:00','1982-07-27 00:00:00',90.56,289,314,62.27,'Intermediate' from dual union all
select 40310,'040310','2014-072A','Yaogan Weixing 24','Jianbing-6 05 xing',69776,'In Earth orbit','','2014-11-22 00:00:00',97.53,629,651,97.91,'Sun-Synch' from dual union all
select 40311,'040311','2014-073A','Kuaizhou-2','kuai zhou er hao',69777,'Reentered Attached','2016-10-09 00:00:00','2014-11-23 00:00:00',90.4,288,301,96.56,'Sun-Synch' from dual union all
select 40312,'040312','2014-074A','Soyuz TMA-15M','Soyuz 11F732A47 No. 715',69778,'Landed','2015-06-11 00:00:00','2014-11-25 00:00:00',92.8,407,417,51.65,'Intermediate' from dual union all
select 40313,'040313','2014-074B','11S510','11S510',69778,'Reentered','2014-11-26 00:00:00','2014-11-25 00:00:00',88.34,188,197,51.63,'Intermediate' from dual union all
select 40314,'040314','1998-067FL','SpinSat','SpinSat',66503,'Reentered','2017-03-11 00:00:00','2014-11-28 00:00:00',92.79,406,417,51.65,'Intermediate' from dual union all
select 40315,'040315','2014-075A','Kosmos-2501','Uragan-K1 No. 12L (702K)',69779,'In Earth orbit','','2014-12-08 00:00:00',677.6,19155,19199,64.82,'Medium' from dual union all
select 40316,'040316','2014-075B','Fregat No. 1044','Fregat 14S44 No. 1044',69779,'In Earth orbit','','2014-12-07 00:00:00',690.19,19327,19658,64.82,'Medium' from dual union all
select 40317,'040317','1984-072C','deb Meteor-2','',57126,'In Earth orbit','','2014-11-30 00:00:00',103.93,932,957,82.53,'Intermediate' from dual union all
select 40318,'040318','1984-072D','deb Meteor-2','',57126,'In Earth orbit','','2014-11-30 00:00:00',103.99,937,958,82.53,'Intermediate' from dual union all
select 40319,'040319','2014-076A','Hayabusa 2','Hayabusa-2',69780,'Deep Space','2014-12-03 00:00:00','2014-12-03 00:00:00',432.69,245,50902,29.89,'Earth Escape' from dual union all
select 4032,'004032','1965-082SS','deb Transtage 4','',11832,'Reentered','1969-12-30 00:00:00','1969-07-08 00:00:00',98.48,612,760,31.72,'Intermediate' from dual union all
select 40320,'040320','2014-076B','Fuji-OSCAR-82','Shin''en-2',69780,'Deep Space','2014-12-03 00:00:00','2014-12-03 00:00:00',432.69,245,50902,29.89,'Earth Escape' from dual union all
select 40321,'040321','2014-076C','DESPATCH','ARTSAT-2',69780,'Deep Space','2014-12-03 00:00:00','2014-12-03 00:00:00',432.69,245,50902,29.89,'Earth Escape' from dual union all
select 40322,'040322','2014-076D','PROCYON','PROCYON',69780,'Deep Space','2014-12-03 00:00:00','2014-12-03 00:00:00',432.69,245,50902,29.89,'Earth Escape' from dual union all
select 40323,'040323','2014-076E','H-2A F25 Stage 2','H-2A Stage 2',69780,'Deep Space','2014-12-03 00:00:00','2014-12-03 00:00:00',432.69,245,50902,29.89,'Earth Escape' from dual union all
select 40324,'040324','2002-005G','deb Iridium 91','',67381,'Reentered','2015-11-01 00:00:00','2014-12-03 00:00:00',101.11,723,900,86.45,'Polar' from dual union all
select 40325,'040325','2002-005H','deb Iridium 91','',67381,'Reentered','2015-03-22 00:00:00','2014-12-03 00:00:00',99.52,697,775,86.42,'Polar' from dual union all
select 40326,'040326','2002-005J','deb Iridium 91','',67381,'In Earth orbit','','2014-12-03 00:00:00',99.71,696,794,86.45,'Polar' from dual union all
select 40327,'040327','2002-005K','deb Iridium 91','',67381,'In Earth orbit','','2014-12-03 00:00:00',99.95,734,779,86.4,'Polar' from dual;
insert into SATELLITE
select 40328,'040328','2014-000A','Unknown Oko debris','',null,'In Earth orbit','','2014-12-01 00:00:00',710.76,6395,33613,70.46,'GEO Transfer' from dual union all
select 40329,'040329','2014-077A','Orion EFT-1','EFT-1 Crew Module',69781,'Landed','2014-12-05 00:00:00','2014-12-05 00:00:00',147.78,32,5794,28.76,'Suborbital' from dual union all
select 4033,'004033','1965-082ST','deb Transtage 4','',11832,'Reentered','1970-09-03 00:00:00','1969-07-08 00:00:00',103.79,710,1166,32.34,'Intermediate' from dual union all
select 40330,'040330','2014-077B','Delta 369','DCSS-5 F13',69781,'Deorbited','2014-12-05 00:00:00','2014-12-05 00:00:00',95.19,191,865,28.81,'Intermediate' from dual union all
select 40331,'040331','2014-077C','deb Delta 369  (spurious)','',69781,'Reentered','2014-12-05 00:00:00','2014-12-05 00:00:00',95.19,191,865,28.81,'Intermediate' from dual union all
select 40332,'040332','2014-078A','GSAT 16','GSAT 16',69782,'In Earth orbit','','2014-12-15 00:00:00',1430.57,35588,35769,.18,'Drift GEO' from dual union all
select 40333,'040333','2014-078B','DirectTV-14','DirecTV-14',69782,'In Earth orbit','','2014-12-13 00:00:00',1435.3,35755,35787,.08,'Drift GEO' from dual union all
select 40334,'040334','2014-078C','ESC-A 575','Ariane 5 ESC-A',69782,'In Earth orbit','','2014-12-14 00:00:00',629.97,244,35686,6.08,'GEO Transfer' from dual union all
select 40335,'040335','2014-078D','Sylda 5C','Sylda 5C No. 62C',69782,'In Earth orbit','','2014-12-08 00:00:00',631.29,247,35751,6.03,'GEO Transfer' from dual union all
select 40336,'040336','2014-079A','CBERS 4','ziyuan yihao 04 weixing',69783,'In Earth orbit','','2014-12-14 00:00:00',100.3,773,773,98.55,'Sun-Synch' from dual union all
select 40337,'040337','2014-079B','CZ-4B Stage 3','CZ-4B Stage 3',69783,'In Earth orbit','','2014-12-14 00:00:00',96.85,479,736,98.45,'Sun-Synch' from dual union all
select 40338,'040338','2014-080A','Yaogan Weixing 25','Jianbing-8 05 xing',69784,'In Earth orbit','','2014-12-12 00:00:00',107.1,1088,1097,63.41,'Intermediate' from dual union all
select 40339,'040339','2014-080B','Yaogan Weixing 25 fu xing yi','Jianbing-8 05 subsat 1',69784,'In Earth orbit','','2014-12-12 00:00:00',107.12,1090,1096,63.41,'Intermediate' from dual union all
select 4034,'004034','1969-057A','Kosmos-289','Zenit-4 No. 56',18324,'Landed','1969-07-15 00:00:00','1969-07-14 00:00:00',89.66,193,323,65.4,'Intermediate' from dual union all
select 40340,'040340','2014-080C','Yaogan Weixing 25 fu xing er','Jianbing-8 05 subsat 2',69784,'In Earth orbit','','2014-12-12 00:00:00',107.3,1093,1111,63.4,'Intermediate' from dual union all
select 40341,'040341','2014-080D','CZ-4C Stage 3','CZ-4C Stage 3',69784,'In Earth orbit','','2014-12-12 00:00:00',105.18,906,1100,63.47,'Intermediate' from dual union all
select 40342,'040342','2014-080E','deb YW-25','',69784,'In Earth orbit','','2014-12-14 00:00:00',107.14,1092,1096,63.41,'Intermediate' from dual union all
select 40343,'040343','2014-080F','deb YW-25','',69784,'In Earth orbit','','2014-12-14 00:00:00',107.12,1090,1096,63.41,'Intermediate' from dual union all
select 40344,'040344','2014-081A','USA 259','RAVEN 6/HEO-3',69785,'In Earth orbit','','2014-12-20 00:00:00',707.77,2117,37743,62.84,'Molniya' from dual union all
select 40345,'040345','2014-082A','Yamal-401','Yamal 401',69786,'In Earth orbit','','2015-01-04 00:00:00',1436.09,35786,35787,.03,'Stationary' from dual union all
select 40346,'040346','2014-082B','Briz-M No. 99551','Briz-M 14S43 No. 99551',69786,'In Earth orbit','','2015-01-05 00:00:00',1293.37,30481,35408,.48,'Medium' from dual union all
select 40347,'040347','2014-082C','Briz-M DTB','Briz-M torus tank',69786,'Reentered','2018-04-29 00:00:00','2014-12-23 00:00:00',630.09,402,35534,47.64,'GEO Transfer' from dual union all
select 40348,'040348','2014-083A','O3b FM10','O3b 10',69787,'In Earth orbit','','2015-01-07 00:00:00',287.92,8063,8067,.04,'Medium' from dual union all
select 40349,'040349','2014-083B','O3b FM11','O3b 11',69787,'In Earth orbit','','2015-01-07 00:00:00',287.9,8062,8068,.04,'Medium' from dual union all
select 4035,'004035','1969-057B','11A57I','11A57I',18324,'Reentered','1969-07-16 00:00:00','1969-07-12 00:00:00',89.13,175,287,65.37,'Intermediate' from dual union all
select 40350,'040350','2014-083C','O3b FM12','O3b 12',69787,'In Earth orbit','','2015-01-07 00:00:00',287.9,8062,8068,.04,'Medium' from dual union all
select 40351,'040351','2014-083D','O3b FM9','O3b 9',69787,'In Earth orbit','','2015-01-07 00:00:00',287.9,8062,8068,.04,'Medium' from dual union all
select 40352,'040352','2014-083E','Fregat-MT No. 133-01','Fregat-MT    No. 133-01',69787,'In Earth orbit','','2014-12-29 00:00:00',276.56,7670,7696,.07,'Medium' from dual union all
select 40353,'040353','2014-084A','Kondor-E No. 2','Kondor-E No. 2',69788,'In Earth orbit','','2014-12-21 00:00:00',94.6,498,500,74.75,'Intermediate' from dual union all
select 40354,'040354','2014-084B','Strela APB','Strela APB',69788,'In Earth orbit','','2014-12-21 00:00:00',93.96,437,500,74.74,'Intermediate' from dual union all
select 40355,'040355','2014-085A','MGM','Angara test payload',69789,'In Earth orbit','','2015-01-12 00:00:00',1530.87,36155,39089,.3,'Very High' from dual union all
select 40356,'040356','2014-085B','Briz-M DTB','Briz-M torus tank',69789,'In Earth orbit','','2014-12-30 00:00:00',636.02,437,35804,60.61,'GEO Transfer' from dual union all
select 40357,'040357','1991-009DZ','deb S3M','',64178,'In Earth orbit','','2015-03-07 00:00:00',117.21,1413,1698,73.95,'Intermediate' from dual union all
select 40358,'040358','2014-086A','Kosmos-2502','Lotos-S No.  802',69790,'In Earth orbit','','2014-12-27 00:00:00',103.07,899,909,67.15,'Intermediate' from dual union all
select 40359,'040359','2014-086B','Blok-I 14S54','14S54',69790,'Reentered','2015-08-01 00:00:00','2014-12-27 00:00:00',95.7,238,867,67.14,'Intermediate' from dual union all
select 4036,'004036','1969-058A','Luna-15 KT','E-8-5 No. 401',18325,'Deep Space','1969-07-17 00:00:00','1969-07-14 00:00:00',15562.18,200,400000,51.5,'Cislunar' from dual union all
select 40360,'040360','2014-087A','Resurs-P No. 2','Resurs-P No. 2',69791,'In Earth orbit','','2014-12-28 00:00:00',90.5,183,416,97.27,'Sun-Synch' from dual union all
select 40361,'040361','2014-087B','Blok-I 14S54','14S54',69791,'Reentered','2015-01-09 00:00:00','2014-12-28 00:00:00',90.85,189,444,97.28,'Sun-Synch' from dual union all
select 40362,'040362','2014-088A','Yaogan Weixing 26','Jianbing-13 01 xing',69792,'In Earth orbit','','2014-12-29 00:00:00',94.32,481,491,97.44,'Sun-Synch' from dual union all
select 40363,'040363','2014-088B','CZ-4B Stage 3','CZ-4B Stage 3',69792,'Reentered','2015-02-24 00:00:00','2014-12-29 00:00:00',91.65,230,481,97.34,'Sun-Synch' from dual union all
select 40364,'040364','2014-089A','Astra 2G','Astra 2G',69793,'In Earth orbit','','2015-01-27 00:00:00',1436.06,35782,35790,.07,'Stationary' from dual union all
select 40365,'040365','2014-089B','Briz-M No. 99552','Briz-M 14S43 No. 99552',69793,'In Earth orbit','','2015-01-28 00:00:00',655.94,3446,33813,23.61,'GEO Transfer' from dual union all
select 40366,'040366','2014-089C','Briz-M DTB','Briz-M torus tank',69793,'In Earth orbit','','2015-01-29 00:00:00',278.08,334,15135,49.57,'Highly Ellip' from dual union all
select 40367,'040367','2014-090A','Feng Yun 2G','fengyun erhao 08 xing',69794,'In Earth orbit','','2015-01-08 00:00:00',1435.98,35770,35799,2.27,'Stationary' from dual union all
select 40368,'040368','2014-090B','CZ-3A Stage 3','CZ-3A Stage 3',69794,'In Earth orbit','','2015-01-07 00:00:00',661.1,326,37195,24.68,'GEO Transfer' from dual union all
select 40369,'040369','2014-090C','FG-36 AKM','Shanxi AKM',69794,'In Earth orbit','','2015-01-09 00:00:00',1446.04,35794,36168,2.33,'Drift GEO' from dual union all
select 4037,'004037','1969-058B','SOZ','SOZ',18325,'Reentered','1969-07-16 00:00:00','1969-07-14 00:00:00',88.38,184,205,51.53,'Intermediate' from dual union all
select 40370,'040370','2015-001A','Dragon CRS-5','Dragon C107',69939,'Landed','2015-02-11 00:00:00','2015-01-11 00:00:00',91.5,345,351,51.65,'Intermediate' from dual union all
select 40371,'040371','2015-001B','Dragon solar array cover','',69939,'Reentered','2015-01-13 00:00:00','2015-01-11 00:00:00',89.38,191,297,51.66,'Intermediate' from dual union all
select 40372,'040372','2015-001C','Dragon solar array cover','',69939,'Reentered','2015-01-12 00:00:00','2015-01-11 00:00:00',89.8,194,336,51.65,'Intermediate' from dual union all
select 40373,'040373','2015-001D','Falcon 9-014 Stage 2','F9-014 St2',69939,'Reentered','2015-01-17 00:00:00','2015-01-12 00:00:00',89.6,190,320,51.65,'Intermediate' from dual union all
select 40374,'040374','2015-002A','MUOS 3','MUOS SV 4',69940,'In Earth orbit','','2015-01-21 00:00:00',702.91,3830,35789,19.06,'GEO Transfer' from dual union all
select 40375,'040375','2015-002B','Centaur AV-052','Centaur AV-052',69940,'In Earth orbit','','2015-01-28 00:00:00',689.27,3743,35196,18.98,'GEO Transfer' from dual union all
select 40376,'040376','2015-003A','SMAP','SMAP',69941,'In Earth orbit','','2015-02-01 00:00:00',98.2,660,685,98.13,'Sun-Synch' from dual union all
select 40377,'040377','2015-003B','FIREBIRD U3','FIREBIRD U3',69941,'In Earth orbit','','2015-02-01 00:00:00',95.72,439,668,99.13,'Sun-Synch' from dual union all
select 40378,'040378','2015-003C','FIREBIRD U4','FIREBIRD U4',69941,'In Earth orbit','','2015-02-01 00:00:00',95.72,436,670,99.13,'Sun-Synch' from dual union all
select 40379,'040379','2015-003D','GRIFEX','GRIFEX',69941,'In Earth orbit','','2015-02-01 00:00:00',95.72,437,670,99.13,'Sun-Synch' from dual union all
select 4038,'004038','1969-058C','SOZ','SOZ',18325,'Reentered','1969-07-16 00:00:00','1969-07-14 00:00:00',88.49,184,216,51.51,'Intermediate' from dual union all
select 40380,'040380','2015-003E','ExoCube','CalPoly CP10',69941,'In Earth orbit','','2015-02-01 00:00:00',95.73,439,669,99.13,'Sun-Synch' from dual union all
select 40381,'040381','2015-004A','JSE reda yobi-ki','IGS Radar Spare',69942,'In Earth orbit','','2015-02-02 00:00:00',94.62,490,511,97.52,'Sun-Synch' from dual union all
select 40382,'040382','2015-004B','H-2A F27 Stage 2','H-2A Stage 2',69942,'In Earth orbit','','2015-02-02 00:00:00',94.41,480,500,97.49,'Sun-Synch' from dual union all
select 40383,'040383','2015-004C','deb IGS Radar-Spare','Large Deb',69942,'Reentered','2015-02-05 00:00:00','2015-02-02 00:00:00',94.57,497,499,97.53,'Sun-Synch' from dual union all
select 40384,'040384','2015-005A','Inmarsat 5F2','Inmarsat 5F2',69943,'In Earth orbit','','2015-02-02 00:00:00',1379.27,4366,64967,26.79,'Highly Ellip' from dual union all
select 40385,'040385','2015-005B','Briz-M No. 99554','Briz-M 14S43 No. 99554',69943,'In Earth orbit','','2015-02-02 00:00:00',1297.29,2816,63232,28.07,'Highly Ellip' from dual union all
select 40386,'040386','2015-005C','Briz-M DTB','Briz-M torus tank',69943,'In Earth orbit','','2015-02-10 00:00:00',274.26,366,14844,50.77,'Highly Ellip' from dual union all
select 40387,'040387','2015-006A','Fajr','Fajr',69944,'Reentered','2015-02-26 00:00:00','2015-02-02 00:00:00',91.46,224,469,55.54,'Intermediate' from dual union all
select 40388,'040388','2015-006B','Safir Stage 2','',69944,'Reentered','2015-03-07 00:00:00','2015-02-02 00:00:00',91.36,224,459,55.54,'Intermediate' from dual union all
select 40389,'040389','1998-067FM','AESP-14','AESP-14',66503,'Reentered','2015-05-11 00:00:00','2015-02-06 00:00:00',92.58,397,405,51.64,'Intermediate' from dual union all
select 4039,'004039','1969-059A','Columbia','Apollo CM-107',18326,'Deep Space','1969-07-19 00:00:00','1969-07-16 00:00:00',25831.89,222,565954,31.4,'Cislunar' from dual union all
select 40390,'040390','2015-007A','DSCOVR','Deep Space Climate Obsy.',69945,'Deep Space','2015-02-12 00:00:00','2015-02-12 00:00:00',95491.65,186,1371156,37.03,'Cislunar' from dual union all
select 40391,'040391','2015-007B','Falcon 9-015 Stage 2','F9-015 St2',69945,'Deep Space','2015-02-12 00:00:00','2015-02-12 00:00:00',124597.43,180,1639773,37.04,'Cislunar' from dual union all
select 40392,'040392','2015-008A','Progress M-26M','Progress 7K-TGM No.425',69946,'Deorbited','2015-08-14 00:00:00','2015-02-17 00:00:00',88.72,186,236,51.65,'Intermediate' from dual union all
select 40393,'040393','2015-008B','11S510','11S510',69946,'Reentered','2015-02-19 00:00:00','2015-02-18 00:00:00',88.32,177,206,51.61,'Intermediate' from dual union all
select 40394,'040394','1995-015F','deb DMSP F-13','',65676,'Reentered','2015-06-10 00:00:00','2015-02-25 00:00:00',95.98,343,789,98.79,'Sun-Synch' from dual union all
select 40395,'040395','1995-015G','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.33,801,843,98.75,'Sun-Synch' from dual union all
select 40396,'040396','1995-015H','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.51,819,842,98.74,'Sun-Synch' from dual union all
select 40397,'040397','1995-015J','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.64,830,842,98.75,'Sun-Synch' from dual union all
select 40398,'040398','1995-015K','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.7,834,845,98.76,'Sun-Synch' from dual union all
select 40399,'040399','1995-015L','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.67,832,843,98.76,'Sun-Synch' from dual union all
select 404,'000404','61 Omicron 124','deb r/b Ablestar','',7273,'In Earth orbit','','1962-09-19 00:00:00',105.99,973,1109,66.88,'Intermediate' from dual union all
select 4040,'004040','1969-059B','Saturn S-IVB-506','Saturn S-IVB-506',18326,'Deep Space','1969-07-19 00:00:00','1969-07-16 00:00:00',25831.89,222,565954,31.4,'Cislunar' from dual union all
select 40400,'040400','1995-015M','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',100.85,687,911,98.65,'Sun-Synch' from dual union all
select 40401,'040401','1995-015N','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',102.77,842,937,98.75,'Sun-Synch' from dual union all
select 40402,'040402','1995-015P','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.47,710,947,98.73,'Sun-Synch' from dual union all
select 40403,'040403','1995-015Q','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.58,825,842,98.76,'Sun-Synch' from dual union all
select 40404,'040404','1995-015R','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.71,835,845,98.76,'Sun-Synch' from dual union all
select 40405,'040405','1995-015S','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',102.19,761,964,98.81,'Sun-Synch' from dual union all
select 40406,'040406','1995-015T','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.66,834,841,98.75,'Sun-Synch' from dual union all
select 40407,'040407','1995-015U','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.71,835,845,98.75,'Sun-Synch' from dual union all
select 40408,'040408','1995-015V','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.62,828,842,98.75,'Sun-Synch' from dual union all
select 40409,'040409','1995-015W','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.7,836,843,98.75,'Sun-Synch' from dual union all
select 4041,'004041','1969-059C','Eagle','LM 5 AS',18326,'Deep Space Attached','1969-07-19 00:00:00','1969-07-16 00:00:00',25831.89,222,565954,31.4,'Cislunar' from dual union all
select 40410,'040410','1995-015X','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.91,835,864,98.77,'Sun-Synch' from dual union all
select 40411,'040411','1995-015Y','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',102.14,843,877,98.77,'Sun-Synch' from dual union all
select 40412,'040412','1995-015Z','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.48,815,843,98.75,'Sun-Synch' from dual union all
select 40413,'040413','1995-015AA','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.48,816,841,98.77,'Sun-Synch' from dual union all
select 40414,'040414','1995-015AB','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.5,817,843,98.77,'Sun-Synch' from dual union all
select 40415,'040415','1995-015AC','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.71,836,843,98.75,'Sun-Synch' from dual union all
select 40416,'040416','1995-015AD','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.71,835,845,98.75,'Sun-Synch' from dual union all
select 40417,'040417','1995-015AE','deb DMSP F-13','',65676,'Reentered','2015-12-01 00:00:00','2015-02-25 00:00:00',97.6,470,818,98.88,'Sun-Synch' from dual;
insert into SATELLITE
select 40418,'040418','1995-015AF','deb DMSP F-13','',65676,'In Earth orbit','','2015-02-25 00:00:00',101.54,821,843,98.75,'Sun-Synch' from dual union all
select 40419,'040419','1995-015AG','deb DMSP F-13','',65676,'Reentered','2015-04-05 00:00:00','2015-02-25 00:00:00',97,459,772,98.9,'Sun-Synch' from dual union all
select 4042,'004042','1969-060A','Kosmos-290','Zenit-2 No. 75',18327,'Landed','1969-07-30 00:00:00','1969-07-24 00:00:00',89.73,193,330,65.41,'Intermediate' from dual union all
select 40420,'040420','2015-009A','Kosmos-2503','Bars-M No. 1L',69947,'In Earth orbit','','2015-03-01 00:00:00',93.24,327,540,97.63,'Sun-Synch' from dual union all
select 40421,'040421','2015-009B','Blok-I 14S54/RD-0110','374BL02A',69947,'Reentered','2016-03-02 00:00:00','2015-03-01 00:00:00',93.19,326,535,97.64,'Sun-Synch' from dual union all
select 40422,'040422','1998-067FN','Flock 1b-27','Dove 0823',66503,'Reentered','2015-10-14 00:00:00','2015-02-27 00:00:00',92.56,396,404,51.65,'Intermediate' from dual union all
select 40423,'040423','1998-067FP','Flock 1b-28','Dove 0818',66503,'Reentered','2015-08-07 00:00:00','2015-02-27 00:00:00',92.56,396,404,51.65,'Intermediate' from dual union all
select 40424,'040424','2015-010A','ABS-3A','ABS-3A',69948,'In Earth orbit','','2015-03-04 00:00:00',1239.19,358,63318,24.81,'Highly Ellip' from dual union all
select 40425,'040425','2015-010B','EUTELSAT 115 West B','EUTELSAT 115 West B',69948,'In Earth orbit','','2015-03-04 00:00:00',1240.69,360,63379,24.81,'Highly Ellip' from dual union all
select 40426,'040426','2015-010C','Falcon 9-016 Stage 2','F9-016 St2',69948,'In Earth orbit','','2015-03-04 00:00:00',1235.54,365,63161,24.97,'Highly Ellip' from dual union all
select 40427,'040427','1998-067FQ','Flock 1b-21','Dove 0817',66503,'Reentered','2015-10-12 00:00:00','2015-03-02 00:00:00',92.55,397,402,51.64,'Intermediate' from dual union all
select 40428,'040428','1998-067FR','Flock 1b-22','Dove 081F',66503,'Reentered','2015-10-02 00:00:00','2015-03-02 00:00:00',92.55,396,403,51.64,'Intermediate' from dual union all
select 40429,'040429','1998-067FS','Flock 1b-10','Dove 081C',66503,'Reentered','2015-08-23 00:00:00','2015-03-02 00:00:00',92.55,393,406,51.62,'Intermediate' from dual union all
select 4043,'004043','1969-061A','Molniya-1','Molniya-1 11F67 No. 18',18328,'Reentered','1971-06-18 00:00:00','1969-08-24 00:00:00',722.39,515,40067,64.97,'GEO Transfer' from dual union all
select 40430,'040430','1998-067FT','Flock 1b-9','Dove 0819',66503,'Reentered','2015-06-28 00:00:00','2015-03-02 00:00:00',92.67,398,413,51.64,'Intermediate' from dual union all
select 40431,'040431','1995-015AH','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-03 00:00:00',100.21,694,843,98.75,'Sun-Synch' from dual union all
select 40432,'040432','1995-015AJ','deb DMSP F-13','',65676,'Reentered','2015-10-03 00:00:00','2015-03-03 00:00:00',101.67,777,899,98.75,'Sun-Synch' from dual union all
select 40433,'040433','1995-015AK','deb DMSP F-13','',65676,'Reentered','2015-12-03 00:00:00','2015-03-03 00:00:00',100.72,753,833,98.74,'Sun-Synch' from dual union all
select 40434,'040434','1995-015AL','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-02 00:00:00',102.66,745,1023,98.69,'Sun-Synch' from dual union all
select 40435,'040435','1995-015AM','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-03 00:00:00',102.9,822,970,98.66,'Sun-Synch' from dual union all
select 40436,'040436','1995-015AN','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-03 00:00:00',105.27,833,1182,98.77,'Sun-Synch' from dual union all
select 40437,'040437','1995-015AP','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-03 00:00:00',101.46,815,840,98.75,'Sun-Synch' from dual union all
select 40438,'040438','1995-015AQ','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-03 00:00:00',102.19,837,888,98.69,'Sun-Synch' from dual union all
select 40439,'040439','1995-015AR','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-03 00:00:00',101.58,827,839,98.75,'Sun-Synch' from dual union all
select 4044,'004044','1969-060B','11A57I','11A57I',18327,'Reentered','1969-07-29 00:00:00','1969-07-22 00:00:00',89.71,196,325,65.4,'Intermediate' from dual union all
select 40440,'040440','1995-015AS','deb DMSP F-13','',65676,'Reentered','2015-10-06 00:00:00','2015-03-03 00:00:00',101.35,783,863,98.76,'Sun-Synch' from dual union all
select 40441,'040441','1995-015AT','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-03 00:00:00',101.96,835,868,98.77,'Sun-Synch' from dual union all
select 40442,'040442','1995-015AU','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-03 00:00:00',101.82,836,854,98.75,'Sun-Synch' from dual union all
select 40443,'040443','1995-015AV','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-03 00:00:00',101.29,791,848,98.77,'Sun-Synch' from dual union all
select 40444,'040444','1995-015AW','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-03 00:00:00',101.22,791,841,98.76,'Sun-Synch' from dual union all
select 40445,'040445','1995-015AX','deb DMSP F-13','',65676,'Reentered','2015-11-02 00:00:00','2015-03-03 00:00:00',101.26,781,855,98.76,'Sun-Synch' from dual union all
select 40446,'040446','1995-015AY','deb DMSP F-13','',65676,'Reentered','2015-07-08 00:00:00','2015-03-03 00:00:00',100.88,734,866,98.78,'Sun-Synch' from dual union all
select 40447,'040447','1995-015AZ','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-03 00:00:00',101.5,818,842,98.76,'Sun-Synch' from dual union all
select 40448,'040448','1995-015BA','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-03 00:00:00',102.52,843,913,98.74,'Sun-Synch' from dual union all
select 40449,'040449','1995-015BB','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-03 00:00:00',102.18,838,886,98.68,'Sun-Synch' from dual union all
select 4045,'004045','1969-061B','11S510','11S510',18328,'Reentered','1969-08-28 00:00:00','1969-07-31 00:00:00',91.4,224,462,64.94,'Intermediate' from dual union all
select 40450,'040450','1995-015BC','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-03 00:00:00',101.47,814,842,98.75,'Sun-Synch' from dual union all
select 40451,'040451','1998-067FU','Flock 1d''-1','Dove 0A16',66503,'Reentered','2015-10-13 00:00:00','2015-03-03 00:00:00',92.56,394,406,51.64,'Intermediate' from dual union all
select 40452,'040452','1998-067FV','Flock 1d''-2','Dove 0C02',66503,'Reentered','2015-12-27 00:00:00','2015-03-03 00:00:00',92.56,394,406,51.64,'Intermediate' from dual union all
select 40453,'040453','1998-067FW','Flock 1b-5','Dove 0822',66503,'Reentered','2015-08-17 00:00:00','2015-03-03 00:00:00',92.58,396,406,51.64,'Intermediate' from dual union all
select 40454,'040454','1998-067FX','Flock 1b-6','Dove 0901',66503,'Reentered','2015-10-16 00:00:00','2015-03-03 00:00:00',92.58,396,406,51.64,'Intermediate' from dual union all
select 40455,'040455','1998-067FY','TechEdSat-4','TES-4',66503,'Reentered','2015-04-03 00:00:00','2015-03-05 00:00:00',92.5,393,402,51.64,'Intermediate' from dual union all
select 40456,'040456','1998-067FZ','GEARRS','GEARRS',66503,'Reentered','2015-11-07 00:00:00','2015-03-05 00:00:00',92.55,395,404,51.65,'Intermediate' from dual union all
select 40457,'040457','1998-067GA','MicroMAS','MicroMAS',66503,'Reentered','2015-08-01 00:00:00','2015-03-05 00:00:00',92.55,393,406,51.64,'Intermediate' from dual union all
select 40458,'040458','1998-067GB','LambdaSat','LambdaSat',66503,'Reentered','2015-05-16 00:00:00','2015-03-05 00:00:00',92.54,393,404,51.64,'Intermediate' from dual union all
select 40459,'040459','1998-067GC','Flock 1b-11','Dove 0815',66503,'Reentered','2015-09-06 00:00:00','2015-03-05 00:00:00',92.56,396,404,51.65,'Intermediate' from dual union all
select 4046,'004046','1969-061C','BOZ','BOZ',18328,'Reentered','1969-08-23 00:00:00','1969-07-29 00:00:00',91.55,211,491,64.93,'Intermediate' from dual union all
select 40460,'040460','1998-067GD','Flock 1b-12','Dove 0824',66503,'Reentered','2015-09-29 00:00:00','2015-03-05 00:00:00',92.57,396,405,51.64,'Intermediate' from dual union all
select 40461,'040461','1995-015BD','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-05 00:00:00',92.5,393,402,51.64,'Intermediate' from dual union all
select 40462,'040462','1995-015BE','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-10 00:00:00',102.67,842,928,98.81,'Sun-Synch' from dual union all
select 40463,'040463','1995-015BF','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-10 00:00:00',100.88,760,841,98.83,'Sun-Synch' from dual union all
select 40464,'040464','1995-015BG','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-08 00:00:00',103.29,842,987,98.75,'Sun-Synch' from dual union all
select 40465,'040465','1995-015BH','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-12 00:00:00',102.54,843,915,98.81,'Sun-Synch' from dual union all
select 40466,'040466','1995-015BJ','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-10 00:00:00',100.8,748,845,98.7,'Sun-Synch' from dual union all
select 40467,'040467','1995-015BK','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-10 00:00:00',99.41,632,829,98.76,'Sun-Synch' from dual union all
select 40468,'040468','1995-015BL','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-09 00:00:00',101.59,824,844,98.78,'Sun-Synch' from dual union all
select 40469,'040469','1995-015BM','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-10 00:00:00',101.2,790,841,98.76,'Sun-Synch' from dual union all
select 4047,'004047','1969-062A','OPS 1127','DAPP 7421',18329,'In Earth orbit','','1969-09-07 00:00:00',101.29,786,854,98.82,'Sun-Synch' from dual union all
select 40470,'040470','1995-015BN','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-08 00:00:00',102.62,841,925,98.76,'Sun-Synch' from dual union all
select 40471,'040471','1995-015BP','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-10 00:00:00',102.19,841,884,98.76,'Sun-Synch' from dual union all
select 40472,'040472','1995-015BQ','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-09 00:00:00',99.17,600,838,98.75,'Sun-Synch' from dual union all
select 40473,'040473','1995-015BR','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-10 00:00:00',103.79,838,1037,98.73,'Sun-Synch' from dual union all
select 40474,'040474','1995-015BS','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-10 00:00:00',100.5,727,838,98.86,'Sun-Synch' from dual union all
select 40475,'040475','1995-015BT','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-08 00:00:00',102.64,842,926,98.78,'Sun-Synch' from dual union all
select 40476,'040476','1995-015BU','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-10 00:00:00',100.27,701,841,98.66,'Sun-Synch' from dual union all
select 40477,'040477','1995-015BV','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-10 00:00:00',101.41,802,849,98.76,'Sun-Synch' from dual union all
select 40478,'040478','1995-015BW','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-10 00:00:00',103.09,840,970,98.73,'Sun-Synch' from dual union all
select 40479,'040479','1995-015BX','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-10 00:00:00',101.73,839,842,98.77,'Sun-Synch' from dual union all
select 4048,'004048','1969-062B','Burner II No. 7','Star 37B',18329,'In Earth orbit','','1969-08-26 00:00:00',101.3,787,854,98.76,'Sun-Synch' from dual union all
select 40480,'040480','1995-015BY','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-10 00:00:00',103.03,842,962,98.75,'Sun-Synch' from dual union all
select 40481,'040481','1995-015BZ','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-10 00:00:00',102.73,841,935,98.75,'Sun-Synch' from dual union all
select 40482,'040482','2015-011A','MMS 1','Magnetospheric Multiscal',69949,'Deep Space','2017-03-30 00:00:00','2015-03-16 00:00:00',1409.5,565,69964,28.81,'Near-synch' from dual union all
select 40483,'040483','2015-011B','MMS 2','Magnetospheric Multiscal',69949,'Deep Space','2017-04-10 00:00:00','2015-03-16 00:00:00',1414.17,553,70160,28.88,'Near-synch' from dual union all
select 40484,'040484','2015-011C','MMS 3','Magnetospheric Multiscal',69949,'Deep Space','2017-04-07 00:00:00','2015-03-16 00:00:00',1414.56,556,70172,28.86,'Near-synch' from dual union all
select 40485,'040485','2015-011D','MMS 4','Magnetospheric Multiscal',69949,'Deep Space','2017-04-05 00:00:00','2015-03-16 00:00:00',1415.04,548,70199,28.83,'Near-synch' from dual union all
select 40486,'040486','2015-011E','Centaur AV-053','Centaur AV-053',69949,'Reentered','2017-08-26 00:00:00','2015-03-15 00:00:00',1312.58,332,66334,28.56,'Highly Ellip' from dual union all
select 40487,'040487','1995-015CA','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-13 00:00:00',102.74,841,936,98.78,'Sun-Synch' from dual union all
select 40488,'040488','1995-015CB','deb DMSP F-13','',65676,'Reentered','2015-04-10 00:00:00','2015-03-13 00:00:00',98.14,601,739,98.65,'Sun-Synch' from dual union all
select 40489,'040489','1995-015CC','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-12 00:00:00',102.74,839,938,98.74,'Sun-Synch' from dual union all
select 4049,'004049','1969-061D','Blok-L No. Ya716-89','Blok-L',18328,'Reentered','1971-06-10 00:00:00','1969-08-26 00:00:00',707.93,499,39369,64.92,'GEO Transfer' from dual union all
select 40490,'040490','1995-015CD','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-12 00:00:00',104.45,838,1100,98.73,'Sun-Synch' from dual union all
select 40491,'040491','1995-015CE','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-12 00:00:00',101.24,790,845,98.71,'Sun-Synch' from dual union all
select 40492,'040492','1995-015CF','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-13 00:00:00',101.46,801,855,98.77,'Sun-Synch' from dual union all
select 40493,'040493','1995-015CG','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-13 00:00:00',100.6,736,838,98.7,'Sun-Synch' from dual union all
select 40494,'040494','1995-015CH','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-13 00:00:00',100,683,833,98.75,'Sun-Synch' from dual union all
select 40495,'040495','1995-015CJ','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-13 00:00:00',100.62,734,842,98.74,'Sun-Synch' from dual union all
select 40496,'040496','1995-015CK','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-13 00:00:00',101.39,789,860,98.68,'Sun-Synch' from dual union all
select 40497,'040497','1995-015CL','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-12 00:00:00',101.67,833,843,98.76,'Sun-Synch' from dual union all
select 40498,'040498','1995-015CM','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-12 00:00:00',102.14,832,888,98.73,'Sun-Synch' from dual union all
select 40499,'040499','1995-015CN','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-12 00:00:00',101.68,830,846,98.74,'Sun-Synch' from dual union all
select 405,'000405','61 Omicron 125','deb r/b Ablestar','',7273,'In Earth orbit','','1964-04-28 00:00:00',106.01,970,1114,66.9,'Intermediate' from dual union all
select 4050,'004050','1969-063A','OPS 3654','CORONA CR-7',18330,'Reentered Attached','1969-08-23 00:00:00','1969-07-31 00:00:00',88.41,177,214,74.96,'Intermediate' from dual union all
select 40500,'040500','1995-015CP','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-13 00:00:00',102.15,837,884,98.86,'Sun-Synch' from dual union all
select 40501,'040501','1995-015CQ','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-13 00:00:00',99.33,619,834,98.7,'Sun-Synch' from dual union all
select 40502,'040502','1995-015CR','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-13 00:00:00',100.41,716,840,98.76,'Sun-Synch' from dual union all
select 40503,'040503','1995-015CS','deb DMSP F-13','',65676,'Reentered','2018-03-01 00:00:00','2015-03-13 00:00:00',100.02,684,835,98.76,'Sun-Synch' from dual union all
select 40504,'040504','1995-015CT','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-12 00:00:00',102.66,841,928,98.77,'Sun-Synch' from dual union all
select 40505,'040505','2015-012A','Ekspress AM-7','Ekspress AM-7',69950,'In Earth orbit','','2015-04-07 00:00:00',1436.11,35780,35794,.02,'Stationary' from dual union all
select 40506,'040506','2015-012B','Briz-M No. 99553','Briz-M',69950,'In Earth orbit','','2015-03-29 00:00:00',682.1,4415,34165,20.02,'GEO Transfer' from dual union all
select 40507,'040507','2015-012C','Briz-M DTB','Briz-M torus tank',69950,'In Earth orbit','','2015-03-30 00:00:00',301.99,334,16731,49.48,'Highly Ellip' from dual;
insert into SATELLITE
select 40508,'040508','1995-015CU','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-22 00:00:00',102.64,842,926,98.79,'Sun-Synch' from dual union all
select 40509,'040509','1995-015CV','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-23 00:00:00',101.43,810,843,98.78,'Sun-Synch' from dual union all
select 4051,'004051','1969-064A','INTELSAT III F-5','INTELSAT III F-5',18331,'Reentered','1988-10-14 00:00:00','1969-08-27 00:00:00',146.58,271,5391,30.36,'Medium' from dual union all
select 40510,'040510','1995-015CW','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-23 00:00:00',99.74,661,832,98.75,'Sun-Synch' from dual union all
select 40511,'040511','1995-015CX','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-23 00:00:00',102.06,838,875,98.82,'Sun-Synch' from dual union all
select 40512,'040512','1995-015CY','deb DMSP F-13','',65676,'Reentered','2016-03-30 00:00:00','2015-03-22 00:00:00',101.26,807,830,98.75,'Sun-Synch' from dual union all
select 40513,'040513','1995-015CZ','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-23 00:00:00',101.7,834,845,98.76,'Sun-Synch' from dual union all
select 40514,'040514','1995-015DA','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-20 00:00:00',100.9,772,830,98.76,'Sun-Synch' from dual union all
select 40515,'040515','1995-015DB','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-23 00:00:00',102.12,839,879,98.76,'Sun-Synch' from dual union all
select 40516,'040516','1995-015DC','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-23 00:00:00',101.88,825,871,98.72,'Sun-Synch' from dual union all
select 40517,'040517','1995-015DD','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-23 00:00:00',101.38,804,844,98.75,'Sun-Synch' from dual union all
select 40518,'040518','1995-015DE','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-21 00:00:00',100.08,681,844,98.77,'Sun-Synch' from dual union all
select 40519,'040519','1995-015DF','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-23 00:00:00',99.64,643,840,98.75,'Sun-Synch' from dual union all
select 4052,'004052','1969-064B','deb Star 37D','Star 37D major piece',18331,'Reentered','1970-07-16 00:00:00','1969-08-26 00:00:00',144.95,268,5257,30.33,'Medium' from dual union all
select 40520,'040520','1995-015DG','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-23 00:00:00',101.21,789,843,98.79,'Sun-Synch' from dual union all
select 40521,'040521','1995-015DH','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-21 00:00:00',101.13,783,842,98.79,'Sun-Synch' from dual union all
select 40522,'040522','1995-015DJ','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-23 00:00:00',101.67,836,839,98.76,'Sun-Synch' from dual union all
select 40523,'040523','1995-015DK','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-23 00:00:00',101.43,811,842,98.8,'Sun-Synch' from dual union all
select 40524,'040524','1995-015DL','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-23 00:00:00',100.65,732,846,98.64,'Sun-Synch' from dual union all
select 40525,'040525','1995-015DM','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-23 00:00:00',101.79,831,855,98.82,'Sun-Synch' from dual union all
select 40526,'040526','1995-015DN','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-21 00:00:00',101.49,803,856,98.76,'Sun-Synch' from dual union all
select 40527,'040527','1995-015DP','deb DMSP F-13','',65676,'In Earth orbit','','2015-03-20 00:00:00',102.52,838,918,98.73,'Sun-Synch' from dual union all
select 40528,'040528','2000-047E','Titan 4B deb','',66949,'In Earth orbit','','2015-03-23 00:00:00',96.51,550,633,68,'Intermediate' from dual union all
select 40529,'040529','2000-047F','Titan 4B deb','',66949,'In Earth orbit','','2015-03-23 00:00:00',96.5,550,632,68,'Intermediate' from dual union all
select 4053,'004053','1969-064C','Star 37D S/N 008','Star 37D',18331,'Reentered','2016-10-21 00:00:00','1969-09-04 00:00:00',146.69,268,5403,30.35,'Medium' from dual union all
select 40530,'040530','2013-057C','CZ-4B sep motor cover','',69520,'In Earth orbit','','2015-03-23 00:00:00',100.1,598,929,75.06,'Intermediate' from dual union all
select 40531,'040531','2013-057D','CZ-4B sep motor cover','',69520,'In Earth orbit','','2015-03-22 00:00:00',100.22,599,939,74.98,'Intermediate' from dual union all
select 40532,'040532','2013-057E','CZ-4B sep motor cover','',69520,'In Earth orbit','','2015-03-22 00:00:00',100.28,600,944,74.88,'Intermediate' from dual union all
select 40533,'040533','2013-057F','CZ-4B sep motor cover','',69520,'In Earth orbit','','2015-03-23 00:00:00',99.94,596,916,74.93,'Intermediate' from dual union all
select 40534,'040534','2015-013A','GPS IIF-9','GPS SVN 71',69951,'In Earth orbit','','2015-04-02 00:00:00',729.14,20445,20469,55.01,'Medium' from dual union all
select 40535,'040535','2015-013B','Delta 4 Second Stage','DCSS-4',69951,'In Earth orbit','','2015-04-02 00:00:00',746.18,20789,20958,54.69,'Medium' from dual union all
select 40536,'040536','2015-014A','Arirang-3A','KOMPSAT-3A',69952,'In Earth orbit','','2015-04-02 00:00:00',95.25,522,540,97.51,'Sun-Synch' from dual union all
select 40537,'040537','2015-014B','Gazodinamicheskiy Ekran','GDS (Gas dynamic shield)',69952,'In Earth orbit','','2015-04-02 00:00:00',94.94,499,533,97.52,'Sun-Synch' from dual union all
select 40538,'040538','2015-015A','JSE kougaku 5 gouki','IGS Optical-5',69953,'In Earth orbit','','2015-04-05 00:00:00',94.88,511,515,97.5,'Sun-Synch' from dual union all
select 40539,'040539','2015-015B','H-2A F28 Stage 2','H-2A Stage 2',69953,'In Earth orbit','','2015-04-05 00:00:00',94.17,468,489,97.27,'Sun-Synch' from dual union all
select 4054,'004054','1969-065A','OPS 8285','STRAWMAN 2',18332,'Reentered Attached','1973-01-04 00:00:00','1969-09-01 00:00:00',94.58,459,538,75.02,'Intermediate' from dual union all
select 40540,'040540','2015-015C','H-2A F28 adapter','',69953,'Reentered','2015-04-05 00:00:00','2015-04-05 00:00:00',94.87,510,515,97.5,'Sun-Synch' from dual union all
select 40541,'040541','2015-014C','R-36M2 DS','R-36M2 DS',69952,'In Earth orbit','','2015-04-02 00:00:00',104.73,524,1440,97.38,'Sun-Synch' from dual union all
select 40542,'040542','2015-016A','Soyuz TMA-16M','Soyuz 11F732A47 No. 716',69954,'Landed','2015-09-12 00:00:00','2015-03-27 00:00:00',88.6,198,213,51.61,'Intermediate' from dual union all
select 40543,'040543','2015-016B','11S510','11S510',69954,'Reentered','2015-03-29 00:00:00','2015-03-28 00:00:00',88.03,171,183,51.61,'Intermediate' from dual union all
select 40544,'040544','2015-017A','GalileoSat-7','Galileo FOC FM03',69955,'In Earth orbit','','2015-04-08 00:00:00',858.55,23516,23574,55.07,'Medium' from dual union all
select 40545,'040545','2015-017B','GalileoSat-8','Galileo FOC FM04',69955,'In Earth orbit','','2015-04-08 00:00:00',850.93,23353,23382,55.13,'Medium' from dual union all
select 40546,'040546','2015-017C','Fregat-MT No. 133-02','Fregat-MT    No. 133-02',69955,'In Earth orbit','','2015-04-01 00:00:00',858.79,23503,23598,55.04,'Medium' from dual union all
select 40547,'040547','2015-018A','IRNSS-R1D','IRNSS-1D',69956,'In Earth orbit','','2015-04-07 00:00:00',1435.07,35699,35834,30.48,'Near-synch' from dual union all
select 40548,'040548','2015-018B','PSLV-C27 PS4','PS4',69956,'In Earth orbit','','2015-04-06 00:00:00',366.51,285,20889,19.12,'Highly Ellip' from dual union all
select 40549,'040549','2015-019A','Beidou DW 17','17 ke beidou daohang wei',69957,'In Earth orbit','','2015-04-08 00:00:00',1461.81,35796,36781,54.98,'Near-synch' from dual union all
select 4055,'004055','1969-064D','deb Star 37D','',18331,'Reentered','1970-02-16 00:00:00','1969-08-26 00:00:00',129.5,270,3932,30.3,'Medium' from dual union all
select 40550,'040550','2015-019B','CZ-3C Stage 3','CZ-3C Stage 3',69957,'Reentered','2016-03-23 00:00:00','2015-04-07 00:00:00',437.54,195,25232,54.96,'Highly Ellip' from dual union all
select 40551,'040551','1965-108DR','deb Transtage 8','',11858,'In Earth orbit','','2015-11-24 00:00:00',558.48,593,31580,26.55,'Highly Ellip' from dual union all
select 40552,'040552','2015-020A','Gonets-M No. 21','Gonets-M No. 21',69958,'In Earth orbit','','2015-04-08 00:00:00',116.03,1497,1508,82.49,'Intermediate' from dual union all
select 40553,'040553','2015-020B','Gonets-M No. 22','Gonets-M No. 22',69958,'In Earth orbit','','2015-04-08 00:00:00',116.01,1496,1506,82.49,'Intermediate' from dual union all
select 40554,'040554','2015-020C','Gonets-M No. 23','Gonets-M No. 23',69958,'In Earth orbit','','2015-04-08 00:00:00',115.98,1494,1506,82.49,'Intermediate' from dual union all
select 40555,'040555','2015-020D','Kosmos-2504','[Reshetnev Tech 3]',69958,'In Earth orbit','','2015-04-08 00:00:00',112.41,1171,1504,82.49,'Intermediate' from dual union all
select 40556,'040556','2015-020E','Briz-KM No. 72526','Briz-KM 14S45',69958,'In Earth orbit','','2015-04-07 00:00:00',112.43,1172,1504,82.49,'Intermediate' from dual union all
select 40557,'040557','1995-015DQ','deb DMSP F-13','',65676,'Reentered','2015-07-06 00:00:00','2015-04-02 00:00:00',99.6,727,752,98.74,'Sun-Synch' from dual union all
select 40558,'040558','1995-015DR','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',102.6,840,923,98.74,'Sun-Synch' from dual union all
select 40559,'040559','1995-015DS','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-01 00:00:00',102.64,838,929,98.63,'Sun-Synch' from dual union all
select 4056,'004056','1969-064E','deb Star 37D','',18331,'Reentered','1970-07-04 00:00:00','1969-08-25 00:00:00',130.75,268,4043,30.25,'Medium' from dual union all
select 40560,'040560','1995-015DT','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',101.46,816,840,98.77,'Sun-Synch' from dual union all
select 40561,'040561','1995-015DU','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',100.58,735,838,98.86,'Sun-Synch' from dual union all
select 40562,'040562','1995-015DV','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',101.91,839,860,98.75,'Sun-Synch' from dual union all
select 40563,'040563','1995-015DW','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',101.57,814,852,98.74,'Sun-Synch' from dual union all
select 40564,'040564','1995-015DX','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-02 00:00:00',101.35,800,846,98.79,'Sun-Synch' from dual union all
select 40565,'040565','1995-015DY','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-02 00:00:00',104.73,842,1122,98.73,'Sun-Synch' from dual union all
select 40566,'040566','1995-015DZ','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',100.48,719,844,98.77,'Sun-Synch' from dual union all
select 40567,'040567','1995-015EA','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',100.28,711,833,98.78,'Sun-Synch' from dual union all
select 40568,'040568','1995-015EB','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',101.99,824,882,98.71,'Sun-Synch' from dual union all
select 40569,'040569','1995-015EC','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',101.48,808,849,98.75,'Sun-Synch' from dual union all
select 4057,'004057','1969-064F','deb Star 37D','',18331,'Reentered','1969-10-22 00:00:00','1969-09-11 00:00:00',117.56,269,2874,29.43,'Intermediate' from dual union all
select 40570,'040570','1995-015ED','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-01 00:00:00',101.52,818,844,98.75,'Sun-Synch' from dual union all
select 40571,'040571','1995-015EE','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',100.52,728,839,98.76,'Sun-Synch' from dual union all
select 40572,'040572','1995-015EF','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',101.39,795,853,98.72,'Sun-Synch' from dual union all
select 40573,'040573','1995-015EG','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-02 00:00:00',100.69,736,847,98.65,'Sun-Synch' from dual union all
select 40574,'040574','1995-015EH','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',102.76,840,939,98.79,'Sun-Synch' from dual union all
select 40575,'040575','1995-015EJ','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',101.4,810,840,98.75,'Sun-Synch' from dual union all
select 40576,'040576','1995-015EK','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',101.24,790,845,98.78,'Sun-Synch' from dual union all
select 40577,'040577','1995-015EL','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',101.58,821,846,98.74,'Sun-Synch' from dual union all
select 40578,'040578','1995-015EM','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',100.42,713,844,98.9,'Sun-Synch' from dual union all
select 40579,'040579','1995-015EN','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',100.74,744,844,98.66,'Sun-Synch' from dual union all
select 4058,'004058','1969-066A','Kosmos-291','IS-M GVM',18333,'Reentered','1969-09-08 00:00:00','1969-08-16 00:00:00',90.89,142,495,62.24,'Intermediate' from dual union all
select 40580,'040580','1995-015EP','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',102.04,840,871,98.74,'Sun-Synch' from dual union all
select 40581,'040581','1995-015EQ','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',101.94,830,871,98.86,'Sun-Synch' from dual union all
select 40582,'040582','1995-015ER','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-02 00:00:00',103.27,836,991,98.74,'Sun-Synch' from dual union all
select 40583,'040583','1995-015ES','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',101.06,778,840,98.82,'Sun-Synch' from dual union all
select 40584,'040584','1995-015ET','deb DMSP F-13','',65676,'Reentered','2015-06-19 00:00:00','2015-04-03 00:00:00',100.5,699,866,98.4,'Sun-Synch' from dual union all
select 40585,'040585','1995-015EU','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-03 00:00:00',101.84,823,869,98.41,'Sun-Synch' from dual union all
select 40586,'040586','1995-015EV','deb DMSP F-13','',65676,'Reentered','2017-04-16 00:00:00','2015-04-03 00:00:00',98.44,564,804,98.75,'Sun-Synch' from dual union all
select 40587,'040587','1995-015EW','deb DMSP F-13','',65676,'Reentered','2015-12-11 00:00:00','2015-04-03 00:00:00',101.07,765,854,98.68,'Sun-Synch' from dual union all
select 40588,'040588','2015-021A','Dragon CRS-6','Dragon C108',69959,'Landed','2015-05-21 00:00:00','2015-04-16 00:00:00',92.32,384,393,51.64,'Intermediate' from dual union all
select 40589,'040589','2015-021B','Falcon 9-017 Stage 2','F9-017 St2',69959,'Reentered','2015-04-22 00:00:00','2015-04-16 00:00:00',89.88,199,338,51.65,'Intermediate' from dual union all
select 4059,'004059','1969-064G','deb Star 37D','',18331,'Reentered','1969-12-30 00:00:00','1969-08-26 00:00:00',136.32,227,4565,31.49,'Medium' from dual union all
select 40590,'040590','2015-021C','Dragon solar array cover','',69959,'Reentered','2015-04-17 00:00:00','2015-04-15 00:00:00',89.56,195,310,51.64,'Intermediate' from dual union all
select 40591,'040591','2015-021D','Dragon solar array cover','',69959,'Reentered','2015-04-17 00:00:00','2015-04-16 00:00:00',89.43,192,301,51.66,'Intermediate' from dual union all
select 40592,'040592','1964-051F','deb Explorer XX','',10337,'In Earth orbit','','2015-04-22 00:00:00',102.72,821,954,79.9,'Intermediate' from dual union all
select 40593,'040593','2014-088C','deb YW-26','',69792,'Reentered','2016-12-02 00:00:00','2015-04-21 00:00:00',93.92,463,470,97.42,'Sun-Synch' from dual union all
select 40594,'040594','2014-088D','YW-26 sensor cover','',69792,'In Earth orbit','','2015-04-22 00:00:00',94.2,476,484,97.42,'Sun-Synch' from dual union all
select 40595,'040595','2014-088E','deb YW-26','',69792,'Reentered','2015-04-17 00:00:00','2015-04-15 00:00:00',90.16,259,306,97.52,'Sun-Synch' from dual union all
select 40596,'040596','2014-088F','deb YW-26','',69792,'Reentered','2015-05-29 00:00:00','2015-04-22 00:00:00',92.48,351,442,97.36,'Sun-Synch' from dual union all
select 40597,'040597','2015-013C','deb Delta 4','',69951,'Reentered','2015-04-23 00:00:00','2015-04-20 00:00:00',165.69,180,7050,52.23,'Medium' from dual union all
select 40598,'040598','2015-013D','deb Delta 4','',69951,'Reentered','2015-04-18 00:00:00','2015-04-04 00:00:00',210.29,201,10471,52.15,'Medium' from dual;
insert into SATELLITE
select 40599,'040599','2015-013E','deb Delta 4','',69951,'Reentered','2015-04-16 00:00:00','2015-04-15 00:00:00',102.81,152,1631,52.2,'Intermediate' from dual union all
select 406,'000406','61 Omicron 126','deb r/b Ablestar','',7273,'In Earth orbit','','1962-09-20 00:00:00',106.87,975,1189,66.8,'Intermediate' from dual union all
select 4060,'004060','1969-064H','deb Star 37D','',18331,'Reentered','1969-12-19 00:00:00','1969-09-04 00:00:00',132.59,274,4197,30.58,'Medium' from dual union all
select 40600,'040600','1995-015EX','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-22 00:00:00',102.44,836,913,98.88,'Sun-Synch' from dual union all
select 40601,'040601','1995-015EY','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-21 00:00:00',101.63,824,847,98.79,'Sun-Synch' from dual union all
select 40602,'040602','1995-015EZ','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-22 00:00:00',101.01,777,836,98.8,'Sun-Synch' from dual union all
select 40603,'040603','1995-015FA','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-20 00:00:00',99.28,631,818,98.66,'Sun-Synch' from dual union all
select 40604,'040604','1995-015FB','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-21 00:00:00',99.43,630,833,98.74,'Sun-Synch' from dual union all
select 40605,'040605','1995-015FC','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-19 00:00:00',101.38,798,850,98.78,'Sun-Synch' from dual union all
select 40606,'040606','1995-015FD','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-22 00:00:00',101.6,831,838,98.76,'Sun-Synch' from dual union all
select 40607,'040607','1995-015FE','deb DMSP F-13','',65676,'Reentered','2018-02-16 00:00:00','2015-04-19 00:00:00',100.05,684,838,98.71,'Sun-Synch' from dual union all
select 40608,'040608','1995-015FF','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-22 00:00:00',101.27,796,842,98.75,'Sun-Synch' from dual union all
select 40609,'040609','1995-015FG','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-21 00:00:00',101.67,836,839,98.76,'Sun-Synch' from dual union all
select 4061,'004061','1969-066B','11S692','11S692',18333,'Reentered','1969-08-11 00:00:00','1969-08-08 00:00:00',90.74,142,480,62.24,'Intermediate' from dual union all
select 40610,'040610','1995-015FH','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-22 00:00:00',101.67,815,861,98.75,'Sun-Synch' from dual union all
select 40611,'040611','1995-015FJ','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-20 00:00:00',101.88,822,874,98.74,'Sun-Synch' from dual union all
select 40612,'040612','1995-015FK','deb DMSP F-13','',65676,'In Earth orbit','','2015-04-22 00:00:00',101.66,833,842,98.76,'Sun-Synch' from dual union all
select 40613,'040613','2015-022A','Thor 7','Thor 7',69960,'In Earth orbit','','2015-05-04 00:00:00',1433.41,35727,35741,.03,'Drift GEO' from dual union all
select 40614,'040614','2015-022B','Sicral 2','Sicral 2',69960,'In Earth orbit','','2015-10-09 00:00:00',1436.09,35782,35791,.05,'Stationary' from dual union all
select 40615,'040615','2015-022C','ESC-A 576','Ariane 5 ESC-A',69960,'In Earth orbit','','2015-05-01 00:00:00',632.52,243,35818,5.92,'GEO Transfer' from dual union all
select 40616,'040616','2015-022D','Sylda 5C','Sylda 5C No. 62C',69960,'In Earth orbit','','2015-08-17 00:00:00',619.37,224,35158,5.94,'GEO Transfer' from dual union all
select 40617,'040617','2015-023A','TurkmenAlem52E/MonacoSAT','Turkmensat 1',69961,'In Earth orbit','','2015-05-05 00:00:00',1432.69,35655,35784,.03,'Drift GEO' from dual union all
select 40618,'040618','2015-023B','Falcon 9-018 Stage 2','F9-018 St2',69961,'In Earth orbit','','2015-05-20 00:00:00',618.07,206,35109,25.5,'GEO Transfer' from dual union all
select 40619,'040619','2015-024A','Progress M-27M','Progress 7K-TGM No.426',69962,'Reentered','2015-05-08 00:00:00','2015-04-30 00:00:00',88.84,183,252,51.64,'Intermediate' from dual union all
select 4062,'004062','1969-067A','Zond-7','L-1 No. 11',18334,'Deep Space','1969-08-10 00:00:00','1969-08-07 00:00:00',15562.18,200,400000,51.52,'Cislunar' from dual union all
select 40620,'040620','2015-024B','Blok-I 14S54/RD-0110','374BL02A',69962,'Reentered','2015-04-29 00:00:00','2015-04-28 00:00:00',88.01,170,181,51.65,'Intermediate' from dual union all
select 40621,'040621','2015-024C','deb Progress M-27M','',69962,'Reentered','2015-04-28 00:00:00','2015-04-28 00:00:00',89.54,164,340,51.63,'Intermediate' from dual union all
select 40622,'040622','2015-024D','deb Progress M-27M','',69962,'Reentered','2015-04-28 00:00:00','2015-04-28 00:00:00',88.82,152,281,51.51,'Intermediate' from dual union all
select 40623,'040623','2015-024E','deb Progress M-27M','',69962,'Reentered','2015-04-28 00:00:00','2015-04-28 00:00:00',89.02,177,276,51.61,'Intermediate' from dual union all
select 40624,'040624','2015-024F','deb Progress M-27M','',69962,'Reentered','2015-04-28 00:00:00','2015-04-28 00:00:00',88.15,165,201,51.65,'Intermediate' from dual union all
select 40625,'040625','2015-024G','deb Progress M-27M','',69962,'Reentered','2015-04-29 00:00:00','2015-04-28 00:00:00',88.6,174,236,51.64,'Intermediate' from dual union all
select 40626,'040626','2015-024H','deb Progress M-27M','',69962,'Reentered','2015-04-29 00:00:00','2015-04-28 00:00:00',89.89,173,365,51.67,'Intermediate' from dual union all
select 40627,'040627','2015-024J','deb Progress M-27M','',69962,'Reentered','2015-04-28 00:00:00','2015-04-28 00:00:00',88.17,154,213,51.63,'Intermediate' from dual union all
select 40628,'040628','2015-024K','deb Progress M-27M','',69962,'Reentered','2015-04-28 00:00:00','2015-04-28 00:00:00',87.71,152,171,51.65,'Intermediate' from dual union all
select 40629,'040629','2015-024L','deb Progress M-27M','',69962,'Reentered','2015-04-28 00:00:00','2015-04-28 00:00:00',87.62,133,181,51.66,'Intermediate' from dual union all
select 4063,'004063','1969-067B','SOZ','SOZ',18334,'Reentered','1969-08-10 00:00:00','1969-08-09 00:00:00',88.09,176,184,51.52,'Intermediate' from dual union all
select 40630,'040630','2015-024M','deb Progress M-27M','',69962,'Reentered','2015-04-29 00:00:00','2015-04-28 00:00:00',88.08,168,191,51.67,'Intermediate' from dual union all
select 40631,'040631','2015-024N','deb Progress M-27M','',69962,'Reentered','2015-04-29 00:00:00','2015-04-28 00:00:00',88.53,153,251,51.65,'Intermediate' from dual union all
select 40632,'040632','2015-024P','deb Progress M-27M','',69962,'Reentered','2015-04-29 00:00:00','2015-04-29 00:00:00',88.33,170,213,51.64,'Intermediate' from dual union all
select 40633,'040633','2015-024Q','deb Progress M-27M','',69962,'Reentered','2015-05-02 00:00:00','2015-04-30 00:00:00',88.51,172,230,51.64,'Intermediate' from dual union all
select 40634,'040634','2015-024R','deb Progress M-27M','',69962,'Reentered','2015-04-29 00:00:00','2015-04-28 00:00:00',88.09,175,185,51.65,'Intermediate' from dual union all
select 40635,'040635','2015-024S','deb Progress M-27M','',69962,'Reentered','2015-05-03 00:00:00','2015-05-01 00:00:00',88.34,173,211,51.64,'Intermediate' from dual union all
select 40636,'040636','2015-024T','deb Progress M-27M','',69962,'Reentered','2015-04-30 00:00:00','2015-04-30 00:00:00',88.42,168,225,51.63,'Intermediate' from dual union all
select 40637,'040637','2015-024U','deb Progress M-27M','',69962,'Reentered','2015-05-07 00:00:00','2015-04-30 00:00:00',89.04,183,272,51.64,'Intermediate' from dual union all
select 40638,'040638','2015-024V','deb Progress M-27M','',69962,'Reentered','2015-04-29 00:00:00','2015-04-29 00:00:00',88.29,167,212,51.65,'Intermediate' from dual union all
select 40639,'040639','2015-024W','deb Progress M-27M','',69962,'Reentered','2015-04-29 00:00:00','2015-04-29 00:00:00',87.59,143,168,51.63,'Intermediate' from dual union all
select 4064,'004064','1969-067C','SOZ','SOZ',18334,'Reentered','1969-08-12 00:00:00','1969-08-09 00:00:00',88.75,198,228,51.52,'Intermediate' from dual union all
select 40640,'040640','2015-024X','deb Progress M-27M','',69962,'Reentered','2015-05-02 00:00:00','2015-05-01 00:00:00',88.04,161,194,51.63,'Intermediate' from dual union all
select 40641,'040641','1966-040BE','Nimbus 2 debris','',13447,'In Earth orbit','','2015-05-10 00:00:00',107.8,1085,1165,100.41,'Sun-Synch' from dual union all
select 40642,'040642','1966-040BF','Nimbus 2 debris','',13447,'In Earth orbit','','2015-05-10 00:00:00',107.62,1066,1167,100.47,'Sun-Synch' from dual union all
select 40643,'040643','1966-040BG','Nimbus 2 debris','',13447,'In Earth orbit','','2015-05-08 00:00:00',107.68,1072,1167,100.42,'Sun-Synch' from dual union all
select 40644,'040644','1966-040BH','Nimbus 2 debris','',13447,'In Earth orbit','','2015-05-08 00:00:00',107.82,1079,1172,100.39,'Sun-Synch' from dual union all
select 40645,'040645','1966-040BJ','Nimbus 2 debris','',13447,'In Earth orbit','','2015-05-09 00:00:00',107.84,1081,1173,100.39,'Sun-Synch' from dual union all
select 40646,'040646','1966-040BK','Nimbus 2 debris','',13447,'In Earth orbit','','2015-05-10 00:00:00',107.76,1081,1166,100.37,'Sun-Synch' from dual union all
select 40647,'040647','1966-040BL','Nimbus 2 debris','',13447,'In Earth orbit','','2015-05-11 00:00:00',107.87,1096,1161,100.36,'Sun-Synch' from dual union all
select 40648,'040648','1966-040BM','Nimbus 2 debris','',13447,'In Earth orbit','','2015-05-11 00:00:00',107.89,1089,1170,100.36,'Sun-Synch' from dual union all
select 40649,'040649','2015-013F','deb Delta 4','',69951,'In Earth orbit','','2015-05-11 00:00:00',374.22,986,20662,52.42,'Highly Ellip' from dual union all
select 4065,'004065','1969-068A','OSO 6','OSO G',18335,'Reentered','1981-03-07 00:00:00','1969-09-14 00:00:00',95.09,490,556,32.96,'Intermediate' from dual union all
select 40650,'040650','1994-074AF','deb Resurs-O1','',65470,'In Earth orbit','','2015-05-11 00:00:00',97.34,630,633,98.15,'Sun-Synch' from dual union all
select 40651,'040651','2015-025A','X-37B OTV-4','X-37B No. 2',69963,'Landed','2017-05-07 00:00:00','2015-05-27 00:00:00',90.83,309,322,38.02,'Intermediate' from dual union all
select 40652,'040652','2015-025B','USS Langley','Unix Space Server Langle',69963,'In Earth orbit','','2015-06-03 00:00:00',95.19,356,700,55.01,'Intermediate' from dual union all
select 40653,'040653','2015-025C','Opticube 01','Optical CubeSat 1',69963,'In Earth orbit','','2015-06-03 00:00:00',95.19,356,700,55.01,'Intermediate' from dual union all
select 40654,'040654','2015-025D','ParkinsonSat','ParkinsonSat-1',69963,'In Earth orbit','','2015-06-03 00:00:00',95.19,356,700,55.01,'Intermediate' from dual union all
select 40655,'040655','2015-025E','BRICSat-P','BRICSat-P',69963,'In Earth orbit','','2015-06-03 00:00:00',95.19,356,700,55.01,'Intermediate' from dual union all
select 40656,'040656','2015-025F','Opticube 02','Optical CubeSat 2',69963,'In Earth orbit','','2015-06-03 00:00:00',95.19,356,700,55.01,'Intermediate' from dual union all
select 40657,'040657','2015-025G','GEARRS-2','GEARRS-2',69963,'In Earth orbit','','2015-06-03 00:00:00',95.19,356,700,55.01,'Intermediate' from dual union all
select 40658,'040658','2015-025H','Opticube 03','Optical CubeSat 3',69963,'In Earth orbit','','2015-06-03 00:00:00',95.19,356,700,55.01,'Intermediate' from dual union all
select 40659,'040659','2015-025J','Aerocube 8A','Aerocube-8a',69963,'In Earth orbit','','2015-06-03 00:00:00',95.19,356,700,55.01,'Intermediate' from dual union all
select 4066,'004066','1969-068B','PAC','PAC',18335,'Reentered Attached','1977-04-28 00:00:00','1969-09-14 00:00:00',95.01,486,553,32.96,'Intermediate' from dual union all
select 40660,'040660','2015-025K','Aerocube 8B','Aerocube-8b',69963,'In Earth orbit','','2015-06-03 00:00:00',95.19,356,700,55.01,'Intermediate' from dual union all
select 40661,'040661','2015-025L','LightSail-A','LightSail-A',69963,'Reentered','2015-06-14 00:00:00','2015-06-03 00:00:00',95.19,356,700,55.01,'Intermediate' from dual union all
select 40662,'040662','2015-025M','ULTRASat','NPSCuL-3 ULTRASat',69963,'Reentered Attached','2015-05-20 00:00:00','2015-05-20 00:00:00',95.19,356,700,55.01,'Intermediate' from dual union all
select 40663,'040663','2015-026A','DirecTV-15','DirecTV-15',69964,'In Earth orbit','','2015-06-04 00:00:00',1434.07,35685,35809,.12,'Drift GEO' from dual union all
select 40664,'040664','2015-026B','SKY Mexico-1','SKY Mexico-1',69964,'In Earth orbit','','2015-06-04 00:00:00',1257.25,28687,35731,.21,'Medium' from dual union all
select 40665,'040665','2015-026C','ESC-A 577','Ariane 5 ESC-A',69964,'In Earth orbit','','2015-06-04 00:00:00',628.61,245,35615,4.01,'GEO Transfer' from dual union all
select 40666,'040666','2015-026D','Sylda 5B','Sylda 5B No. 58B',69964,'In Earth orbit','','2015-06-07 00:00:00',631.39,258,35745,4.42,'GEO Transfer' from dual union all
select 40667,'040667','2015-027A','Kosmos-2505','Kobal''t-M No. 565',69965,'Landed','2015-09-17 00:00:00','2015-06-05 00:00:00',89.12,177,285,81.42,'Intermediate' from dual union all
select 40668,'040668','2015-027B','Blok-I 14S54/RD-0110','374BL02A',69965,'Reentered','2015-06-09 00:00:00','2015-06-05 00:00:00',89.07,175,281,81.4,'Intermediate' from dual union all
select 40669,'040669','1994-074AG','deb Resurs-O1','',65470,'Reentered','2015-09-20 00:00:00','2015-06-10 00:00:00',94.38,484,494,98.13,'Sun-Synch' from dual union all
select 4067,'004067','1969-067D','SOK','Support cone',18334,'Reentered','1969-08-13 00:00:00','1969-08-10 00:00:00',88.52,84,319,51.56,'Intermediate' from dual union all
select 40670,'040670','1994-074AH','deb Resurs-O1','',65470,'Reentered','2015-07-24 00:00:00','2015-06-03 00:00:00',93.97,463,475,98.14,'Sun-Synch' from dual union all
select 40671,'040671','1994-074AJ','deb Resurs-O1','',65470,'Reentered','2015-09-24 00:00:00','2015-06-09 00:00:00',94.29,478,491,98.15,'Sun-Synch' from dual union all
select 40672,'040672','1995-015FL','deb DMSP F-13','',65676,'In Earth orbit','','2015-06-09 00:00:00',101.61,827,843,98.76,'Sun-Synch' from dual union all
select 40673,'040673','1995-015FM','deb DMSP F-13','',65676,'In Earth orbit','','2015-06-09 00:00:00',101.44,809,845,98.74,'Sun-Synch' from dual union all
select 40674,'040674','1995-015FN','deb DMSP F-13','',65676,'In Earth orbit','','2015-06-09 00:00:00',101.27,798,840,98.76,'Sun-Synch' from dual union all
select 40675,'040675','1995-015FP','deb DMSP F-13','',65676,'In Earth orbit','','2015-06-10 00:00:00',101.66,832,843,98.76,'Sun-Synch' from dual union all
select 40676,'040676','1995-015FQ','deb DMSP F-13','',65676,'In Earth orbit','','2015-06-10 00:00:00',101.64,830,842,98.76,'Sun-Synch' from dual union all
select 40677,'040677','1995-015FR','deb DMSP F-13','',65676,'In Earth orbit','','2015-06-10 00:00:00',101.08,772,848,98.71,'Sun-Synch' from dual union all
select 40678,'040678','1995-015FS','deb DMSP F-13','',65676,'In Earth orbit','','2015-06-09 00:00:00',101.3,798,843,98.73,'Sun-Synch' from dual union all
select 40679,'040679','1995-015FT','deb DMSP F-13','',65676,'In Earth orbit','','2015-06-08 00:00:00',101.26,793,844,98.78,'Sun-Synch' from dual union all
select 4068,'004068','1969-069A','ATS 5','ATS E',18336,'In Earth orbit','','1969-10-02 00:00:00',1435.98,35778,35791,2.55,'Stationary' from dual union all
select 40680,'040680','1995-015FU','deb DMSP F-13','',65676,'In Earth orbit','','2015-06-09 00:00:00',101.34,804,841,98.78,'Sun-Synch' from dual union all
select 40681,'040681','1995-015FV','deb DMSP F-13','',65676,'In Earth orbit','','2015-06-09 00:00:00',101.69,834,844,98.75,'Sun-Synch' from dual union all
select 40682,'040682','1999-025EWN','deb FY-1C','',66706,'In Earth orbit','','2015-06-07 00:00:00',99.66,695,790,99.04,'Sun-Synch' from dual union all
select 40683,'040683','1999-025EWP','deb FY-1C','',66706,'In Earth orbit','','2015-05-29 00:00:00',96.33,473,693,96.98,'Sun-Synch' from dual union all
select 40684,'040684','1999-025EWQ','deb FY-1C','',66706,'In Earth orbit','','2015-06-02 00:00:00',102.22,840,887,99.01,'Sun-Synch' from dual union all
select 40685,'040685','1999-025EWR','deb FY-1C','',66706,'In Earth orbit','','2015-06-09 00:00:00',104.45,836,1102,99.5,'Sun-Synch' from dual union all
select 40686,'040686','1999-025EWS','deb FY-1C','',66706,'In Earth orbit','','2015-05-29 00:00:00',99.5,728,742,99.24,'Sun-Synch' from dual union all
select 40687,'040687','1999-025EWT','deb FY-1C','',66706,'In Earth orbit','','2015-05-19 00:00:00',108.51,827,1488,99.15,'Sun-Synch' from dual union all
select 40688,'040688','1999-025EWU','deb FY-1C','',66706,'In Earth orbit','','2015-05-25 00:00:00',104.26,853,1067,99.15,'Sun-Synch' from dual;
insert into SATELLITE
select 40689,'040689','1999-025EWV','deb FY-1C','',66706,'In Earth orbit','','2015-06-08 00:00:00',106.17,803,1296,99.09,'Sun-Synch' from dual union all
select 4069,'004069','1969-069B','Centaur AC-18','Centaur 15D',18336,'In Earth orbit','','1969-09-25 00:00:00',703.17,2238,37394,17.43,'GEO Transfer' from dual union all
select 40690,'040690','1999-025EWW','deb FY-1C','',66706,'In Earth orbit','','2015-06-09 00:00:00',103.42,849,992,99.16,'Sun-Synch' from dual union all
select 40691,'040691','1999-025EWX','deb FY-1C','',66706,'In Earth orbit','','2015-06-09 00:00:00',103.69,838,1028,99.39,'Sun-Synch' from dual union all
select 40692,'040692','1999-025EWY','deb FY-1C','',66706,'In Earth orbit','','2015-06-09 00:00:00',102.75,810,967,99.24,'Sun-Synch' from dual union all
select 40693,'040693','1999-025EWZ','deb FY-1C','',66706,'In Earth orbit','','2015-05-20 00:00:00',106.43,835,1288,98.91,'Sun-Synch' from dual union all
select 40694,'040694','1999-025EXA','deb FY-1C','',66706,'In Earth orbit','','2015-06-11 00:00:00',101.72,750,931,98.36,'Sun-Synch' from dual union all
select 40695,'040695','1999-025EXB','deb FY-1C','',66706,'In Earth orbit','','2015-06-09 00:00:00',99.79,528,969,99.48,'Sun-Synch' from dual union all
select 40696,'040696','1999-025EXC','deb FY-1C','',66706,'In Earth orbit','','2015-06-08 00:00:00',103.19,849,970,99.14,'Sun-Synch' from dual union all
select 40697,'040697','2015-028A','Sentinel-2A','Sentinel-2',69966,'In Earth orbit','','2015-07-01 00:00:00',100.62,787,788,98.58,'Sun-Synch' from dual union all
select 40698,'040698','2015-028B','Vega AVUM VV05','AVUM',69966,'In Earth orbit','','2015-06-30 00:00:00',96.51,399,784,98.64,'Sun-Synch' from dual union all
select 40699,'040699','2015-029A','Kosmos-2506','Persona-1 No. 3',69967,'In Earth orbit','','2015-06-30 00:00:00',99.09,706,725,98.3,'Sun-Synch' from dual union all
select 407,'000407','61 Omicron 127','deb r/b Ablestar','',7273,'In Earth orbit','','1962-09-20 00:00:00',105.52,884,1154,66.36,'Intermediate' from dual union all
select 4070,'004070','1969-070A','Kosmos-292','Tsiklon',18337,'In Earth orbit','','1969-09-23 00:00:00',99.9,744,764,74.06,'Intermediate' from dual union all
select 40700,'040700','2015-029B','Blok-I 14S54','14S54',69967,'Reentered','2015-08-17 00:00:00','2015-07-02 00:00:00',92.72,185,631,98.31,'Sun-Synch' from dual union all
select 40701,'040701','2015-030A','Gao Fen 8','Gaofen ba hao weixing',69968,'In Earth orbit','','2015-07-03 00:00:00',94.36,484,492,97.3,'Sun-Synch' from dual union all
select 40702,'040702','2015-030B','CZ-4B Stage 3','CZ-4B Stage 3',69968,'Reentered','2015-09-08 00:00:00','2015-07-03 00:00:00',91.43,226,464,97.22,'Sun-Synch' from dual union all
select 40703,'040703','1984-072E','deb Meteor-2','',57126,'In Earth orbit','','2015-07-01 00:00:00',103.83,928,952,82.53,'Intermediate' from dual union all
select 40704,'040704','1984-072F','deb Meteor-2','',57126,'In Earth orbit','','2015-07-01 00:00:00',103.83,925,954,82.53,'Intermediate' from dual union all
select 40705,'040705','1984-072G','deb Meteor-2','',57126,'In Earth orbit','','2015-06-30 00:00:00',103.81,924,954,82.53,'Intermediate' from dual union all
select 40706,'040706','1984-072H','deb Meteor-2','',57126,'In Earth orbit','','2015-07-01 00:00:00',103.86,928,954,82.53,'Intermediate' from dual union all
select 40707,'040707','1984-072J','deb Meteor-2','',57126,'In Earth orbit','','2015-07-01 00:00:00',103.79,923,953,82.53,'Intermediate' from dual union all
select 40708,'040708','1984-072K','deb Meteor-2','',57126,'In Earth orbit','','2015-06-29 00:00:00',103.82,925,954,82.52,'Intermediate' from dual union all
select 40709,'040709','1984-072L','deb Meteor-2','',57126,'In Earth orbit','','2015-07-01 00:00:00',103.79,926,950,82.52,'Intermediate' from dual union all
select 4071,'004071','1969-070B','S3M','S3M',18337,'In Earth orbit','','1969-12-20 00:00:00',99.77,736,759,74.06,'Intermediate' from dual union all
select 40710,'040710','1984-072M','deb Meteor-2','',57126,'In Earth orbit','','2015-06-27 00:00:00',103.81,928,949,82.53,'Intermediate' from dual union all
select 40711,'040711','1993-055D','deb Meteor-2','',65157,'In Earth orbit','','2015-06-30 00:00:00',103.39,909,929,82.55,'Intermediate' from dual union all
select 40712,'040712','1993-055E','deb Meteor-2','',65157,'In Earth orbit','','2015-05-07 00:00:00',103.61,913,946,82.55,'Intermediate' from dual union all
select 40713,'040713','2015-031A','Progress M-28M','Progress 7K-TGM No.428',69969,'Deorbited','2015-12-19 00:00:00','2015-07-04 00:00:00',90.02,261,290,51.63,'Intermediate' from dual union all
select 40714,'040714','2015-031B','11S510','11S510',69969,'Reentered','2015-07-05 00:00:00','2015-07-04 00:00:00',88.2,174,197,51.61,'Intermediate' from dual union all
select 40715,'040715','2015-032A','DMC3-1','DMC3-1',69970,'In Earth orbit','','2015-07-12 00:00:00',97.71,636,663,98.05,'Sun-Synch' from dual union all
select 40716,'040716','2015-032B','DMC3-2','DMC3-2',69970,'In Earth orbit','','2015-07-12 00:00:00',97.71,636,663,98.05,'Sun-Synch' from dual union all
select 40717,'040717','2015-032C','DMC3-3','DMC3-3',69970,'In Earth orbit','','2015-07-12 00:00:00',97.69,636,661,98.05,'Sun-Synch' from dual union all
select 40718,'040718','2015-032D','Carbonite 1','CBNT-1',69970,'In Earth orbit','','2015-07-13 00:00:00',97.66,636,658,98.05,'Sun-Synch' from dual union all
select 40719,'040719','2015-032E','DeOrbitSail','DeOrbitSail',69970,'In Earth orbit','','2015-07-13 00:00:00',97.64,635,657,98.05,'Sun-Synch' from dual union all
select 4072,'004072','1969-071A','Kosmos-293','Zenit-2M',18338,'Landed','1969-08-28 00:00:00','1969-08-20 00:00:00',89.07,205,252,51.78,'Intermediate' from dual union all
select 40720,'040720','2015-032F','PSLV-C28 PS4','PS4',69970,'In Earth orbit','','2015-07-13 00:00:00',97.32,626,635,98.11,'Sun-Synch' from dual union all
select 40721,'040721','1995-015FW','deb DMSP F-13','',65676,'In Earth orbit','','2015-07-13 00:00:00',101.17,789,840,98.74,'Sun-Synch' from dual union all
select 40722,'040722','1998-067GE','Flock 1e-2','Dove 0B10',66503,'Reentered','2016-03-05 00:00:00','2015-07-14 00:00:00',92.55,396,403,51.65,'Intermediate' from dual union all
select 40723,'040723','1998-067GF','Flock 1e-1','Dove 0B1A',66503,'Reentered','2016-05-18 00:00:00','2015-07-14 00:00:00',92.55,396,403,51.65,'Intermediate' from dual union all
select 40724,'040724','1998-067GG','Flock 1e-4','Dove 0C06',66503,'Reentered','2016-06-23 00:00:00','2015-07-14 00:00:00',92.55,399,399,51.65,'Intermediate' from dual union all
select 40725,'040725','1998-067GH','Flock 1e-3','Dove 0C03',66503,'Reentered','2016-05-29 00:00:00','2015-07-14 00:00:00',92.55,398,401,51.64,'Intermediate' from dual union all
select 40726,'040726','1998-067GJ','Flock 1e-7','Dove 0B0E',66503,'Reentered','2016-07-16 00:00:00','2015-07-17 00:00:00',92.54,396,402,51.64,'Intermediate' from dual union all
select 40727,'040727','1998-067GK','Flock 1e-8','Dove 0B0D',66503,'Reentered','2016-06-22 00:00:00','2015-07-17 00:00:00',92.54,396,402,51.65,'Intermediate' from dual union all
select 40728,'040728','1998-067GL','Flock 1e-5','Dove 0B07',66503,'Reentered','2016-06-20 00:00:00','2015-07-17 00:00:00',92.54,395,403,51.65,'Intermediate' from dual union all
select 40729,'040729','1998-067GM','Flock 1e-6','Dove 0B0B',66503,'Reentered','2016-07-21 00:00:00','2015-07-17 00:00:00',92.55,396,403,51.65,'Intermediate' from dual union all
select 4073,'004073','1969-071B','11A57I','11A57I',18338,'Reentered','1969-08-20 00:00:00','1969-08-18 00:00:00',88.73,187,237,51.73,'Intermediate' from dual union all
select 40730,'040730','2015-033A','GPS IIF-10','GPS SVN 72',69971,'In Earth orbit','','2015-07-16 00:00:00',728.96,20446,20459,54.99,'Medium' from dual union all
select 40731,'040731','2015-033B','Centaur AV-055','Centaur AV-055',69971,'In Earth orbit','','2015-07-16 00:00:00',754.39,20440,21706,55.34,'Medium' from dual union all
select 40732,'040732','2015-034A','Meteosat 11','MSG 4',69972,'In Earth orbit','','2015-07-17 00:00:00',708.29,4150,35735,2.52,'GEO Transfer' from dual union all
select 40733,'040733','2015-034B','Star One C4','Star One C4',69972,'In Earth orbit','','2015-07-16 00:00:00',632.38,258,35796,3.96,'GEO Transfer' from dual union all
select 40734,'040734','2015-034C','ESC-A 578','Ariane 5 ESC-A',69972,'In Earth orbit','','2015-07-16 00:00:00',632.56,258,35805,4.02,'GEO Transfer' from dual union all
select 40735,'040735','2015-034D','Sylda 5A','Sylda 5A No. 65A',69972,'In Earth orbit','','2015-07-16 00:00:00',631.16,259,35732,3.95,'GEO Transfer' from dual union all
select 40736,'040736','1998-067GN','Flock 1e-9','Dove 0B1F',66503,'Reentered','2016-02-08 00:00:00','2015-07-17 00:00:00',92.56,398,402,51.64,'Intermediate' from dual union all
select 40737,'040737','1998-067GP','Flock 1e-10','Dove 0B09',66503,'Reentered','2016-08-09 00:00:00','2015-07-17 00:00:00',92.55,398,401,51.65,'Intermediate' from dual union all
select 40738,'040738','1998-067GQ','Flock 1e-11','Dove 0B1E',66503,'Reentered','2016-08-09 00:00:00','2015-07-17 00:00:00',92.54,396,402,51.65,'Intermediate' from dual union all
select 40739,'040739','1998-067GR','Flock 1e-12','Dove 0B0A',66503,'Reentered','2016-08-09 00:00:00','2015-07-17 00:00:00',92.54,396,402,51.65,'Intermediate' from dual union all
select 4074,'004074','1969-072A','Kosmos-294','Zenit-4 No. 63',18339,'Landed','1969-08-27 00:00:00','1969-08-22 00:00:00',89.74,199,325,65.41,'Intermediate' from dual union all
select 40740,'040740','1998-067GS','Flock 1e-13','Dove 0C07',66503,'Reentered','2016-08-24 00:00:00','2015-07-17 00:00:00',92.55,397,402,51.65,'Intermediate' from dual union all
select 40741,'040741','1998-067GT','Flock 1e-14','Dove 0B0F',66503,'Reentered','2016-07-30 00:00:00','2015-07-17 00:00:00',92.53,395,401,51.64,'Intermediate' from dual union all
select 40742,'040742','1998-067GU','Arkyd-3R','Arkyd-3',66503,'Reentered','2015-12-23 00:00:00','2015-07-17 00:00:00',92.54,396,402,51.65,'Intermediate' from dual union all
select 40743,'040743','1998-067GV','Centennial 1','Centennial 1',66503,'Reentered','2016-01-06 00:00:00','2015-07-17 00:00:00',92.54,395,403,51.65,'Intermediate' from dual union all
select 40744,'040744','2015-035A','Soyuz TMA-17M','Soyuz 11F732A47 No. 717',69973,'Landed','2015-12-11 00:00:00','2015-07-23 00:00:00',92.58,400,402,51.65,'Intermediate' from dual union all
select 40745,'040745','2015-035B','11S510','11S510',69973,'Reentered','2015-07-25 00:00:00','2015-07-24 00:00:00',88.23,178,196,51.63,'Intermediate' from dual union all
select 40746,'040746','2015-036A','WGS SV-7','Wideband Global Satcom 7',69974,'In Earth orbit','','2015-08-01 00:00:00',2071.9,28177,66739,.68,'Very High' from dual union all
select 40747,'040747','2015-036B','Delta 372','DCSS-5 F14',69974,'In Earth orbit','','2015-08-06 00:00:00',1308.22,577,65913,24.32,'Highly Ellip' from dual union all
select 40748,'040748','2015-037A','Beidou DW 19','19 ke beidou daohang wei',69975,'In Earth orbit','','2015-08-13 00:00:00',773.25,21514,21543,54.99,'Medium' from dual union all
select 40749,'040749','2015-037B','Beidou DW 18','18 ke beidou daohang wei',69975,'In Earth orbit','','2015-08-13 00:00:00',773.21,21514,21541,54.99,'Medium' from dual union all
select 4075,'004075','1969-072B','11A57I','11A57I',18339,'Reentered','1969-08-27 00:00:00','1969-08-23 00:00:00',89.18,182,285,65.41,'Intermediate' from dual union all
select 40750,'040750','2015-037C','CZ-3B Y26 Stage 3','CZ-3B Stage 3',69975,'In Earth orbit','','2015-08-14 00:00:00',323.77,193,18289,55.04,'Highly Ellip' from dual union all
select 40751,'040751','2015-037D','Yuanzheng-1 Y2','YZ-1',69975,'In Earth orbit','','2015-08-13 00:00:00',919.55,22100,27792,54.91,'Medium' from dual union all
select 40752,'040752','1976-077FV','deb Delta 126','',39029,'In Earth orbit','','2015-08-07 00:00:00',126.58,1501,2445,102.18,'Medium' from dual union all
select 40753,'040753','1976-077FW','deb Delta 126','',39029,'In Earth orbit','','2015-08-06 00:00:00',117.06,1499,1599,101.94,'Sun-Synch' from dual union all
select 40754,'040754','1976-077FX','deb Delta 126','',39029,'In Earth orbit','','2015-08-07 00:00:00',117.54,1496,1645,101.99,'Sun-Synch' from dual union all
select 40755,'040755','1976-077FY','deb Delta 126','',39029,'In Earth orbit','','2015-08-07 00:00:00',115.71,1472,1503,101.62,'Sun-Synch' from dual union all
select 40756,'040756','1976-077FZ','deb Delta 126','',39029,'In Earth orbit','','2015-08-04 00:00:00',117.45,1505,1628,102.03,'Sun-Synch' from dual union all
select 40757,'040757','1976-077GA','deb Delta 126','',39029,'In Earth orbit','','2015-08-05 00:00:00',114.8,1442,1451,101.38,'Sun-Synch' from dual union all
select 40758,'040758','1976-077GB','deb Delta 126','',39029,'In Earth orbit','','2015-08-08 00:00:00',117.52,1499,1640,102.01,'Sun-Synch' from dual union all
select 40759,'040759','1976-077GC','deb Delta 126','',39029,'In Earth orbit','','2015-08-09 00:00:00',113.72,1169,1626,102.23,'Sun-Synch' from dual union all
select 4076,'004076','1969-073A','Kosmos-295','DS-P1-Yu No. 29',18340,'Reentered','1969-12-01 00:00:00','1969-09-22 00:00:00',91.48,262,433,70.99,'Intermediate' from dual union all
select 40760,'040760','1976-077GD','deb Delta 126','',39029,'In Earth orbit','','2015-08-08 00:00:00',124.56,1496,2272,103.4,'Medium' from dual union all
select 40761,'040761','1976-077GE','deb Delta 126','',39029,'In Earth orbit','','2015-08-07 00:00:00',118.95,1492,1776,102.27,'Sun-Synch' from dual union all
select 40762,'040762','1976-077GF','deb Delta 126','',39029,'In Earth orbit','','2015-08-10 00:00:00',121.7,1489,2025,102.35,'Medium' from dual union all
select 40763,'040763','1976-077GG','deb Delta 126','',39029,'In Earth orbit','','2015-07-31 00:00:00',114.58,1415,1458,101.49,'Sun-Synch' from dual union all
select 40764,'040764','1976-077GH','deb Delta 126','',39029,'In Earth orbit','','2015-08-02 00:00:00',116.07,1503,1505,101.7,'Sun-Synch' from dual union all
select 40765,'040765','1976-077GJ','deb Delta 126','',39029,'In Earth orbit','','2015-08-09 00:00:00',115.71,1466,1510,101.68,'Sun-Synch' from dual union all
select 40766,'040766','1976-077GK','deb Delta 126','',39029,'In Earth orbit','','2015-08-07 00:00:00',117.73,1497,1661,101.88,'Sun-Synch' from dual union all
select 40767,'040767','1976-077GL','deb Delta 126','',39029,'In Earth orbit','','2015-07-04 00:00:00',117.87,1495,1676,102.11,'Sun-Synch' from dual union all
select 40768,'040768','1976-077GM','deb Delta 126','',39029,'In Earth orbit','','2015-08-09 00:00:00',114.16,1378,1457,101.43,'Sun-Synch' from dual union all
select 40769,'040769','1976-077GN','deb Delta 126','',39029,'In Earth orbit','','2015-07-31 00:00:00',115.93,1491,1504,101.69,'Sun-Synch' from dual union all
select 4077,'004077','1969-073B','S1M','S1M',18340,'Reentered','1969-10-11 00:00:00','1969-08-30 00:00:00',91.65,269,442,71.01,'Intermediate' from dual union all
select 40770,'040770','1976-077GP','deb Delta 126','',39029,'In Earth orbit','','2015-08-03 00:00:00',114.81,1407,1487,101.42,'Sun-Synch' from dual union all
select 40771,'040771','1976-077GQ','deb Delta 126','',39029,'In Earth orbit','','2015-08-07 00:00:00',116.17,1496,1521,101.72,'Sun-Synch' from dual union all
select 40772,'040772','1976-077GR','deb Delta 126','',39029,'In Earth orbit','','2015-08-04 00:00:00',119.5,1507,1810,102.28,'Sun-Synch' from dual union all
select 40773,'040773','1976-077GS','deb Delta 126','',39029,'In Earth orbit','','2015-08-07 00:00:00',117.34,1497,1626,102,'Sun-Synch' from dual union all
select 40774,'040774','1979-104AB','deb Ariane L01','',47588,'In Earth orbit','','2015-08-04 00:00:00',372.48,502,21039,18.7,'Highly Ellip' from dual union all
select 40775,'040775','1979-104AC','deb Ariane L01','',47588,'In Earth orbit','','2015-08-10 00:00:00',673.43,617,37527,16.31,'GEO Transfer' from dual union all
select 40776,'040776','1979-104AD','deb Ariane L01','',47588,'In Earth orbit','','2015-07-25 00:00:00',647.72,656,36184,17.68,'GEO Transfer' from dual union all
select 40777,'040777','1979-104AE','deb Ariane L01','',47588,'Reentered','2016-02-08 00:00:00','2015-08-10 00:00:00',309.18,219,17316,20.1,'Highly Ellip' from dual union all
select 40778,'040778','1979-104AF','deb Ariane L01','',47588,'In Earth orbit','','2015-08-10 00:00:00',260.81,794,13493,17.89,'Medium' from dual;
insert into SATELLITE
select 40779,'040779','1979-104AG','deb Ariane L01','',47588,'In Earth orbit','','2015-08-10 00:00:00',694.03,649,38528,19.21,'GEO Transfer' from dual union all
select 4078,'004078','1969-074A','OPS 7807','GAMBIT-3 Mission 4323',18341,'Reentered Attached','1969-09-07 00:00:00','1969-08-30 00:00:00',89.5,132,368,108,'Retrograde' from dual union all
select 40780,'040780','1979-104AH','deb Ariane L01','',47588,'In Earth orbit','','2015-07-23 00:00:00',370.18,451,20949,18.7,'Highly Ellip' from dual union all
select 40781,'040781','1979-104AJ','deb Ariane L01','',47588,'In Earth orbit','','2015-08-10 00:00:00',459.62,488,26213,19.18,'Highly Ellip' from dual union all
select 40782,'040782','1979-104AK','deb Ariane L01','',47588,'In Earth orbit','','2015-08-08 00:00:00',658.25,756,36620,16.69,'GEO Transfer' from dual union all
select 40783,'040783','1989-006CC','deb Ariane V28','',62283,'In Earth orbit','','2015-07-30 00:00:00',642.02,919,35629,7.48,'GEO Transfer' from dual union all
select 40784,'040784','1989-006CD','deb Ariane V28','',62283,'In Earth orbit','','2015-08-04 00:00:00',401.56,354,22950,8.32,'Highly Ellip' from dual union all
select 40785,'040785','1989-006CE','deb Ariane V28','',62283,'In Earth orbit','','2015-08-04 00:00:00',617.57,481,34808,7.57,'GEO Transfer' from dual union all
select 40786,'040786','1989-006CF','deb Ariane V28','',62283,'In Earth orbit','','2015-08-07 00:00:00',636.33,675,35582,7.99,'GEO Transfer' from dual union all
select 40787,'040787','1989-006CG','deb Ariane V28','',62283,'In Earth orbit','','2015-08-07 00:00:00',639.41,710,35705,8.07,'GEO Transfer' from dual union all
select 40788,'040788','1989-006CH','deb Ariane V28','',62283,'In Earth orbit','','2015-08-10 00:00:00',488,424,27885,8.64,'Highly Ellip' from dual union all
select 40789,'040789','1994-085AA','deb Briz','',65481,'In Earth orbit','','2015-08-10 00:00:00',128.6,1272,2851,64.85,'Medium' from dual union all
select 4079,'004079','1964-076M','deb Explorer XXIV','',10362,'Reentered','1971-10-21 00:00:00','1969-08-21 00:00:00',100.91,510,1094,81.39,'Intermediate' from dual union all
select 40790,'040790','1994-085AB','deb Briz','',65481,'Reentered','2016-05-13 00:00:00','2015-08-09 00:00:00',127.1,841,3151,64.91,'Medium' from dual union all
select 40791,'040791','1994-085AC','deb Briz','',65481,'In Earth orbit','','2015-08-08 00:00:00',128.63,969,3157,64.89,'Medium' from dual union all
select 40792,'040792','1993-036BVV','deb Kosmos-2251','',65138,'In Earth orbit','','2015-08-10 00:00:00',99.53,717,756,74.06,'Intermediate' from dual union all
select 40793,'040793','1993-036BVW','deb Kosmos-2251','',65138,'In Earth orbit','','2015-08-06 00:00:00',103.01,733,1069,73.98,'Intermediate' from dual union all
select 40794,'040794','1993-036BVX','deb Kosmos-2251','',65138,'In Earth orbit','','2015-08-10 00:00:00',100.13,737,793,74,'Intermediate' from dual union all
select 40795,'040795','1993-036BVY','deb Kosmos-2251','',65138,'In Earth orbit','','2015-07-26 00:00:00',98.97,607,812,73.97,'Intermediate' from dual union all
select 40796,'040796','1993-036BVZ','deb Kosmos-2251','',65138,'In Earth orbit','','2015-07-12 00:00:00',98.64,636,752,74.03,'Intermediate' from dual union all
select 40797,'040797','1993-036BWA','deb Kosmos-2251','',65138,'In Earth orbit','','2015-07-30 00:00:00',97.48,548,729,73.94,'Intermediate' from dual union all
select 40798,'040798','1993-036BWB','deb Kosmos-2251','',65138,'In Earth orbit','','2015-08-08 00:00:00',99.4,725,735,74.04,'Intermediate' from dual union all
select 40799,'040799','1993-036BWC','deb Kosmos-2251','',65138,'In Earth orbit','','2015-07-25 00:00:00',97.44,522,751,73.94,'Intermediate' from dual union all
select 408,'000408','61 Omicron 128','deb r/b Ablestar','',7273,'In Earth orbit','','1962-09-20 00:00:00',105.86,975,1095,66.8,'Intermediate' from dual union all
select 4080,'004080','1969-075A','Kosmos-296','Zenit-4 No. 68',18342,'Landed','1969-09-06 00:00:00','1969-08-31 00:00:00',89.57,206,301,64.95,'Intermediate' from dual union all
select 40800,'040800','1993-036BWD','deb Kosmos-2251','',65138,'In Earth orbit','','2015-07-24 00:00:00',99.87,608,897,73.79,'Intermediate' from dual union all
select 40801,'040801','1993-036BWE','deb Kosmos-2251','',65138,'In Earth orbit','','2015-07-03 00:00:00',96.52,552,632,74.02,'Intermediate' from dual union all
select 40802,'040802','1993-036BWF','deb Kosmos-2251','',65138,'In Earth orbit','','2015-08-07 00:00:00',98.6,632,752,74.05,'Intermediate' from dual union all
select 40803,'040803','1993-036BWG','deb Kosmos-2251','',65138,'In Earth orbit','','2015-08-09 00:00:00',99.59,713,765,74.02,'Intermediate' from dual union all
select 40804,'040804','1993-036BWH','deb Kosmos-2251','',65138,'In Earth orbit','','2015-07-27 00:00:00',97.48,625,651,74.04,'Intermediate' from dual union all
select 40805,'040805','1993-036BWJ','deb Kosmos-2251','',65138,'In Earth orbit','','2015-08-09 00:00:00',97.82,633,676,74.05,'Intermediate' from dual union all
select 40806,'040806','1993-036BWK','deb Kosmos-2251','',65138,'In Earth orbit','','2015-08-09 00:00:00',98.45,677,693,74.02,'Intermediate' from dual union all
select 40807,'040807','1993-036BWL','deb Kosmos-2251','',65138,'In Earth orbit','','2015-08-09 00:00:00',101.23,742,892,74.03,'Intermediate' from dual union all
select 40808,'040808','1993-036BWM','deb Kosmos-2251','',65138,'In Earth orbit','','2015-08-08 00:00:00',99.93,742,769,74.05,'Intermediate' from dual union all
select 40809,'040809','1993-036BWN','deb Kosmos-2251','',65138,'In Earth orbit','','2015-07-25 00:00:00',98.5,666,708,74.02,'Intermediate' from dual union all
select 4081,'004081','1969-075B','11A57I','11A57I',18342,'Reentered','1969-09-05 00:00:00','1969-08-31 00:00:00',89.26,205,271,64.96,'Intermediate' from dual union all
select 40810,'040810','1993-036BWP','deb Kosmos-2251','',65138,'In Earth orbit','','2015-08-07 00:00:00',99.65,725,759,74.04,'Intermediate' from dual union all
select 40811,'040811','1993-036BWQ','deb Kosmos-2251','',65138,'In Earth orbit','','2015-08-09 00:00:00',98.54,653,725,74.03,'Intermediate' from dual union all
select 40812,'040812','1998-067GW','ATV-PCE WAL-6','ISS RS WAL-6',66503,'Reentered','2016-01-19 00:00:00','2015-08-11 00:00:00',92.53,395,401,51.64,'Intermediate' from dual union all
select 40813,'040813','2007-003AS','deb CZ-3','',68345,'In Earth orbit','','2015-08-04 00:00:00',852.34,579,46222,27.39,'Highly Ellip' from dual union all
select 40814,'040814','2010-042X','deb CZ-3B','',68902,'In Earth orbit','','2015-08-09 00:00:00',756.39,939,41304,26.46,'Highly Ellip' from dual union all
select 40815,'040815','2010-042Y','deb CZ-3B','',68902,'In Earth orbit','','2015-08-06 00:00:00',754.04,861,41268,25.99,'Highly Ellip' from dual union all
select 40816,'040816','2010-057AH','deb CZ-3C','',68917,'In Earth orbit','','2015-08-10 00:00:00',628.54,691,35165,20.56,'GEO Transfer' from dual union all
select 40817,'040817','2010-057AJ','deb CZ-3C','',68917,'In Earth orbit','','2015-08-04 00:00:00',655.04,720,36493,19.74,'GEO Transfer' from dual union all
select 40818,'040818','2010-057AK','deb CZ-3C','',68917,'In Earth orbit','','2015-08-06 00:00:00',638.04,710,35635,20.66,'GEO Transfer' from dual union all
select 40819,'040819','2010-057AL','deb CZ-3C','',68917,'In Earth orbit','','2015-07-28 00:00:00',628.5,570,35284,20.89,'GEO Transfer' from dual union all
select 4082,'004082','1969-076A','Kosmos-297','Zenit-4 No. 64',18343,'Landed','1969-09-10 00:00:00','1969-09-05 00:00:00',89.6,203,307,72.89,'Intermediate' from dual union all
select 40820,'040820','2010-057AM','deb CZ-3C','',68917,'In Earth orbit','','2015-08-10 00:00:00',649.23,536,36381,20.41,'GEO Transfer' from dual union all
select 40821,'040821','2010-057AN','deb CZ-3C','',68917,'In Earth orbit','','2015-08-11 00:00:00',645.5,641,36085,19.88,'GEO Transfer' from dual union all
select 40822,'040822','2010-057AP','deb CZ-3C','',68917,'In Earth orbit','','2015-08-07 00:00:00',648.05,568,36289,21.07,'GEO Transfer' from dual union all
select 40823,'040823','2010-057AQ','deb CZ-3C','',68917,'In Earth orbit','','2015-08-10 00:00:00',644.72,594,36093,21.01,'GEO Transfer' from dual union all
select 40824,'040824','2010-057AR','deb CZ-3C','',68917,'In Earth orbit','','2015-08-07 00:00:00',648.87,599,36300,20.02,'GEO Transfer' from dual union all
select 40825,'040825','2010-057AS','deb CZ-3C','',68917,'In Earth orbit','','2015-08-04 00:00:00',647.44,569,36257,21.14,'GEO Transfer' from dual union all
select 40826,'040826','2010-057AT','deb CZ-3C','',68917,'In Earth orbit','','2015-08-10 00:00:00',640.77,642,35843,21.18,'GEO Transfer' from dual union all
select 40827,'040827','2010-057AU','deb CZ-3C','',68917,'In Earth orbit','','2015-08-05 00:00:00',636.13,669,35578,20.85,'GEO Transfer' from dual union all
select 40828,'040828','2010-057AV','deb CZ-3C','',68917,'In Earth orbit','','2015-07-22 00:00:00',641.4,743,35774,20.89,'GEO Transfer' from dual union all
select 40829,'040829','2010-057AW','deb CZ-3C','',68917,'In Earth orbit','','2015-08-11 00:00:00',617.07,509,34754,21.22,'GEO Transfer' from dual union all
select 4083,'004083','1969-076B','11A57I','11A57I',18343,'Reentered','1969-09-11 00:00:00','1969-09-04 00:00:00',89.31,197,284,72.9,'Intermediate' from dual union all
select 40830,'040830','2010-057AX','deb CZ-3C','',68917,'In Earth orbit','','2015-07-30 00:00:00',646.27,533,36233,20.39,'GEO Transfer' from dual union all
select 40831,'040831','2010-057AY','deb CZ-3C','',68917,'In Earth orbit','','2015-08-05 00:00:00',652.11,669,36395,20.51,'GEO Transfer' from dual union all
select 40832,'040832','2010-057AZ','deb CZ-3C','',68917,'In Earth orbit','','2015-08-10 00:00:00',638.24,565,35790,20.6,'GEO Transfer' from dual union all
select 40833,'040833','2010-057BA','deb CZ-3C','',68917,'In Earth orbit','','2015-08-04 00:00:00',660,621,36843,20.62,'GEO Transfer' from dual union all
select 40834,'040834','2010-057BB','deb CZ-3C','',68917,'In Earth orbit','','2015-08-10 00:00:00',655.67,661,36584,20.42,'GEO Transfer' from dual union all
select 40835,'040835','2010-057BC','deb CZ-3C','',68917,'In Earth orbit','','2015-08-04 00:00:00',653.94,641,36516,20.35,'GEO Transfer' from dual union all
select 40836,'040836','2010-057BD','deb CZ-3C','',68917,'In Earth orbit','','2015-07-31 00:00:00',660.57,674,36820,20.12,'GEO Transfer' from dual union all
select 40837,'040837','2010-057BE','deb CZ-3C','',68917,'In Earth orbit','','2015-08-10 00:00:00',634.46,534,35627,20.71,'GEO Transfer' from dual union all
select 40838,'040838','2011-077T','deb CZ-3B','',69129,'In Earth orbit','','2015-08-11 00:00:00',744.36,525,41133,26.55,'GEO Transfer' from dual union all
select 40839,'040839','2011-077U','deb CZ-3B','',69129,'In Earth orbit','','2015-08-09 00:00:00',746.12,449,41295,25.97,'GEO Transfer' from dual union all
select 4084,'004084','1969-070C','deb Kosmos-292','',18337,'In Earth orbit','','1969-10-03 00:00:00',100.04,744,777,74.06,'Intermediate' from dual union all
select 40840,'040840','2011-077V','deb CZ-3B','',69129,'In Earth orbit','','2015-08-11 00:00:00',746.49,870,40892,27.84,'GEO Transfer' from dual union all
select 40841,'040841','2011-077W','deb CZ-3B','',69129,'In Earth orbit','','2015-08-05 00:00:00',743.7,896,40730,27.39,'GEO Transfer' from dual union all
select 40842,'040842','2011-077X','deb CZ-3B','',69129,'In Earth orbit','','2015-08-06 00:00:00',815.75,684,44400,22.78,'Highly Ellip' from dual union all
select 40843,'040843','2011-077Y','deb CZ-3B','',69129,'In Earth orbit','','2015-08-10 00:00:00',754.41,387,41760,22.56,'Highly Ellip' from dual union all
select 40844,'040844','2011-077Z','deb CZ-3B','',69129,'In Earth orbit','','2015-08-09 00:00:00',790.96,575,43331,22.83,'Highly Ellip' from dual union all
select 40845,'040845','2011-077AA','deb CZ-3B','',69129,'In Earth orbit','','2015-08-03 00:00:00',758.89,354,42010,22.48,'Highly Ellip' from dual union all
select 40846,'040846','2011-077AB','deb CZ-3B','',69129,'Lost','','2015-08-10 00:00:00',770.03,505,42396,23.7,'Highly Ellip' from dual union all
select 40847,'040847','2011-077AC','deb CZ-3B','',69129,'In Earth orbit','','2015-08-07 00:00:00',707.75,414,39445,25.86,'GEO Transfer' from dual union all
select 40848,'040848','2011-077AD','deb CZ-3B','',69129,'In Earth orbit','','2015-08-11 00:00:00',755.4,835,41360,26.75,'Highly Ellip' from dual union all
select 40849,'040849','2011-077AE','deb CZ-3B','',69129,'In Earth orbit','','2015-08-11 00:00:00',755.38,797,41397,26.16,'Highly Ellip' from dual union all
select 4085,'004085','1969-064J','deb Star 37D','',18331,'Reentered','1969-09-18 00:00:00','1969-09-12 00:00:00',103.67,274,1590,30.25,'Intermediate' from dual union all
select 40850,'040850','2011-077AF','deb CZ-3B','',69129,'In Earth orbit','','2015-08-09 00:00:00',750.6,553,41409,25.56,'Highly Ellip' from dual union all
select 40851,'040851','2011-077AG','deb CZ-3B','',69129,'In Earth orbit','','2015-08-09 00:00:00',757.18,896,41385,26.31,'Highly Ellip' from dual union all
select 40852,'040852','2011-077AH','deb CZ-3B','',69129,'In Earth orbit','','2015-08-09 00:00:00',755.32,937,41254,27.13,'Highly Ellip' from dual union all
select 40853,'040853','2012-008T','deb CZ-3C','',69263,'In Earth orbit','','2015-08-11 00:00:00',631.95,815,35217,21.08,'GEO Transfer' from dual union all
select 40854,'040854','2012-008U','deb CZ-3C','',69263,'In Earth orbit','','2015-07-25 00:00:00',617.65,600,34693,21.05,'GEO Transfer' from dual union all
select 40855,'040855','2012-008V','deb CZ-3C','',69263,'In Earth orbit','','2015-08-10 00:00:00',621.06,696,34774,22.49,'GEO Transfer' from dual union all
select 40856,'040856','2012-008W','deb CZ-3C','',69263,'In Earth orbit','','2015-08-08 00:00:00',612.3,680,34335,21.68,'GEO Transfer' from dual union all
select 40857,'040857','2012-008X','deb CZ-3C','',69263,'In Earth orbit','','2015-08-11 00:00:00',650.32,729,36244,20.14,'GEO Transfer' from dual union all
select 40858,'040858','2012-008Y','deb CZ-3C','',69263,'In Earth orbit','','2015-08-07 00:00:00',680.88,586,37933,18.61,'GEO Transfer' from dual union all
select 40859,'040859','2012-008Z','deb CZ-3C','',69263,'In Earth orbit','','2015-08-10 00:00:00',655.41,512,36720,22.2,'GEO Transfer' from dual union all
select 4086,'004086','1969-064K','deb Star 37D','',18331,'Reentered','1970-02-18 00:00:00','1969-09-15 00:00:00',135.6,279,4451,30.02,'Medium' from dual union all
select 40860,'040860','2012-008AA','deb CZ-3C','',69263,'In Earth orbit','','2015-08-10 00:00:00',670.75,453,37556,19.36,'GEO Transfer' from dual union all
select 40861,'040861','2012-008AB','deb CZ-3C','',69263,'In Earth orbit','','2015-08-04 00:00:00',629.25,689,35204,20.93,'GEO Transfer' from dual union all
select 40862,'040862','2012-008AC','deb CZ-3C','',69263,'In Earth orbit','','2015-08-09 00:00:00',694.65,634,38574,19.06,'GEO Transfer' from dual union all
select 40863,'040863','2012-008AD','deb CZ-3C','',69263,'In Earth orbit','','2015-08-10 00:00:00',651.99,986,36072,21.26,'GEO Transfer' from dual union all
select 40864,'040864','2012-008AE','deb CZ-3C','',69263,'In Earth orbit','','2015-08-08 00:00:00',639.19,760,35644,20.14,'GEO Transfer' from dual union all
select 40865,'040865','2012-008AF','deb CZ-3C','',69263,'In Earth orbit','','2015-08-10 00:00:00',698.17,774,38609,19.23,'GEO Transfer' from dual union all
select 40866,'040866','2012-008AG','deb CZ-3C','',69263,'In Earth orbit','','2015-08-03 00:00:00',638.77,912,35470,20.6,'GEO Transfer' from dual union all
select 40867,'040867','2012-008AH','deb CZ-3C','',69263,'In Earth orbit','','2015-08-09 00:00:00',643.82,530,36111,21.86,'GEO Transfer' from dual union all
select 40868,'040868','2012-008AJ','deb CZ-3C','',69263,'In Earth orbit','','2015-08-08 00:00:00',631.04,1385,34600,21.91,'GEO Transfer' from dual;
insert into SATELLITE
select 40869,'040869','2012-008AK','deb CZ-3C','',69263,'In Earth orbit','','2015-08-10 00:00:00',693.65,734,38424,18.36,'GEO Transfer' from dual union all
select 4087,'004087','1969-064L','deb Star 37D','',18331,'Reentered','1970-01-09 00:00:00','1969-09-16 00:00:00',134.14,281,4324,30.18,'Medium' from dual union all
select 40870,'040870','2012-008AL','deb CZ-3C','',69263,'In Earth orbit','','2015-07-14 00:00:00',703.7,678,38980,17.86,'GEO Transfer' from dual union all
select 40871,'040871','2012-035E','SEVIRI Cooler Cover','MSG 3 Cooler Cover',69290,'In Earth orbit','','2015-08-05 00:00:00',1418.45,35323,35558,.89,'Drift GEO' from dual union all
select 40872,'040872','2012-035F','SEVIRI Entry Baffle Cover','MSG 3 Baffle Cover',69290,'In Earth orbit','','2015-08-07 00:00:00',1401.46,34729,35483,.86,'Drift GEO' from dual union all
select 40873,'040873','2015-038A','Kounotori 5 gouki','HTV 5',69976,'Deorbited','2015-09-29 00:00:00','2015-08-21 00:00:00',89.97,244,302,51.68,'Intermediate' from dual union all
select 40874,'040874','2015-039A','Intelsat IS-34','Intelsat IS-34',69977,'In Earth orbit','','2015-09-20 00:00:00',1437.31,35806,35815,.02,'Drift GEO' from dual union all
select 40875,'040875','2015-039B','EUTELSAT 8 WEST B','EUTELSAT 8 WEST B',69977,'In Earth orbit','','2015-09-21 00:00:00',1439.05,35840,35849,.05,'Drift GEO' from dual union all
select 40876,'040876','2015-039C','ESC-A 579','Ariane 5 ESC-A',69977,'In Earth orbit','','2015-09-21 00:00:00',630.61,257,35706,5.39,'GEO Transfer' from dual union all
select 40877,'040877','2015-039D','Sylda 5A','Sylda 5A No. 66A',69977,'In Earth orbit','','2015-09-21 00:00:00',629.84,260,35663,4.77,'GEO Transfer' from dual union all
select 40878,'040878','2015-040A','Yaogan Weixing 27','Jianbing-9 05 xing',69978,'In Earth orbit','','2015-09-03 00:00:00',109.42,1194,1205,100.5,'Sun-Synch' from dual union all
select 40879,'040879','2015-040B','CZ-4C Stage 3','CZ-4C Stage 3',69978,'In Earth orbit','','2015-09-03 00:00:00',107.99,1064,1204,100.45,'Sun-Synch' from dual union all
select 4088,'004088','1969-064M','deb Star 37D','',18331,'Reentered','1969-11-21 00:00:00','1969-09-15 00:00:00',121.32,283,3197,30.49,'Medium' from dual union all
select 40880,'040880','2015-041A','GSAT-6','GSAT-6',69979,'In Earth orbit','','2015-09-28 00:00:00',1436.04,35750,35821,1.13,'Stationary' from dual union all
select 40881,'040881','2015-041B','GSLV CUS-06','GSLV-D6 CUS',69979,'Reentered','2018-03-02 00:00:00','2015-09-27 00:00:00',624.41,163,35480,20.17,'GEO Transfer' from dual union all
select 40882,'040882','2015-042A','Inmarsat 5F3','Inmarsat 5F3',69980,'In Earth orbit','','2015-09-06 00:00:00',2117.59,31691,64802,1.26,'Very High' from dual union all
select 40883,'040883','2015-042B','Briz-M No. 99556','Briz-M 14S43 No. 99556',69980,'In Earth orbit','','2015-08-31 00:00:00',1306.44,2809,63609,27.92,'Highly Ellip' from dual union all
select 40884,'040884','2015-042C','Briz-M DTB','Briz-M torus tank',69980,'In Earth orbit','','2015-09-29 00:00:00',274.29,353,14859,50.7,'Highly Ellip' from dual union all
select 40885,'040885','2015-043A','Soyuz TMA-18M','Soyuz 11F732A47 No. 718',69981,'Landed','2016-03-02 00:00:00','2015-09-03 00:00:00',89.68,238,280,51.64,'Intermediate' from dual union all
select 40886,'040886','2015-043B','11S510','11S510',69981,'Reentered','2015-09-05 00:00:00','2015-09-03 00:00:00',88.32,181,202,51.65,'Intermediate' from dual union all
select 40887,'040887','2015-044A','MUOS 4','MUOS SV 5',69982,'In Earth orbit','','2015-09-16 00:00:00',1436.09,35782,35791,5.04,'Inclined GEO' from dual union all
select 40888,'040888','2015-044B','Centaur AV-056','Centaur AV-056',69982,'In Earth orbit','','2015-09-09 00:00:00',689.75,3737,35226,19.04,'GEO Transfer' from dual union all
select 40889,'040889','2015-045A','GalileoSat-9','Galileo FOC FM05',69983,'In Earth orbit','','2015-09-20 00:00:00',856.75,23440,23566,57.38,'Medium' from dual union all
select 4089,'004089','1969-064N','deb Star 37D','',18331,'Reentered','1969-11-27 00:00:00','1969-09-15 00:00:00',125.24,286,3542,29.83,'Medium' from dual union all
select 40890,'040890','2015-045B','GalileoSat-10','Galileo FOC FM06',69983,'In Earth orbit','','2015-09-18 00:00:00',846.55,23245,23286,57.42,'Medium' from dual union all
select 40891,'040891','2015-045C','Fregat-MT No. 133-03','Fregat-MT    No. 133-03',69983,'In Earth orbit','','2015-09-19 00:00:00',858,23493,23571,57.38,'Medium' from dual union all
select 40892,'040892','2015-046A','Tongxin Jishu Shiyan 1','Qianshao 3',69984,'In Earth orbit','','2015-09-23 00:00:00',1436.04,35776,35795,.18,'Stationary' from dual union all
select 40893,'040893','2015-046B','CZ-3B Stage 3','CZ-3B Stage 3',69984,'Reentered','2016-01-30 00:00:00','2015-09-20 00:00:00',612.22,177,34834,27.04,'GEO Transfer' from dual union all
select 40894,'040894','2015-047A','Gao Fen 9','gao fen 9',69985,'In Earth orbit','','2015-09-21 00:00:00',97.54,618,664,98.01,'Sun-Synch' from dual union all
select 40895,'040895','2015-048A','Ekspress AM-8','Ekspress AM-8',69986,'In Earth orbit','','2015-09-21 00:00:00',1439.43,35814,35890,.04,'Drift GEO' from dual union all
select 40896,'040896','2015-048B','Blok DM-03 No. 5L','11S861-03 No. 5L',69986,'In Earth orbit','','2015-10-06 00:00:00',1459.57,35819,36671,.06,'Drift GEO' from dual union all
select 40897,'040897','1998-067GX','SERPENS','SERPENS',66503,'Reentered','2016-03-27 00:00:00','2015-09-18 00:00:00',92.6,397,407,51.64,'Intermediate' from dual union all
select 40898,'040898','1998-067GY','S-CUBE','S-CUBE',66503,'Reentered','2016-11-23 00:00:00','2015-09-18 00:00:00',92.61,397,408,51.64,'Intermediate' from dual union all
select 40899,'040899','2015-049A','Luliang-1                         ','Luliang-1',69987,'In Earth orbit','','2015-09-22 00:00:00',93.73,392,522,97.42,'Sun-Synch' from dual union all
select 409,'000409','61 Omicron 129','deb r/b Ablestar','',7273,'In Earth orbit','','1962-09-23 00:00:00',103.42,853,988,66.71,'Intermediate' from dual union all
select 4090,'004090','1969-064P','deb Star 37D','',18331,'Reentered','1969-10-09 00:00:00','1969-09-15 00:00:00',119.14,283,3002,29.32,'Intermediate' from dual union all
select 40900,'040900','2015-049B','Zhineng hao shouji weixing','NUDT-Phone-Sat',69987,'In Earth orbit','','2015-09-22 00:00:00',95.15,516,536,97.45,'Sun-Synch' from dual union all
select 40901,'040901','2015-049C','Zheda Pixing 2A','ZDPS 2A',69987,'In Earth orbit','','2015-09-22 00:00:00',95.15,517,534,97.46,'Sun-Synch' from dual union all
select 40902,'040902','2015-049D','Zheda Pixing 2B','ZDPS 2B',69987,'In Earth orbit','','2015-09-22 00:00:00',95.16,517,535,97.45,'Sun-Synch' from dual union all
select 40903,'040903','2015-049E','XW-2A','CAS-3A',69987,'In Earth orbit','','2015-09-22 00:00:00',95.17,518,535,97.46,'Sun-Synch' from dual union all
select 40904,'040904','2015-049F','Kaituo-1','Xinjishu Shiyan Weixing',69987,'In Earth orbit','','2015-09-22 00:00:00',95.18,518,536,97.46,'Sun-Synch' from dual union all
select 40905,'040905','2015-049G','Naxing-2                          ','Naxing 2 hao',69987,'In Earth orbit','','2015-09-22 00:00:00',95.22,519,540,97.45,'Sun-Synch' from dual union all
select 40906,'040906','2015-049H','XW-2C','CAS-3C',69987,'In Earth orbit','','2015-09-22 00:00:00',95.21,519,538,97.46,'Sun-Synch' from dual union all
select 40907,'040907','2015-049J','XW-2D','CAS-3D',69987,'In Earth orbit','','2015-09-22 00:00:00',95.22,520,539,97.46,'Sun-Synch' from dual union all
select 40908,'040908','2015-049K','Zidingxiang-2','LilacSat-2 (CAS-3H)',69987,'In Earth orbit','','2015-09-22 00:00:00',95.24,520,541,97.47,'Sun-Synch' from dual union all
select 40909,'040909','2015-049L','XW-2E','CAS-3E',69987,'In Earth orbit','','2015-09-22 00:00:00',95.2,519,538,97.46,'Sun-Synch' from dual union all
select 4091,'004091','1969-064Q','deb Star 37D','',18331,'Reentered','1969-11-08 00:00:00','1969-09-15 00:00:00',118.48,278,2948,29.77,'Intermediate' from dual union all
select 40910,'040910','2015-049M','XW-2F','CAS-3F',69987,'In Earth orbit','','2015-09-22 00:00:00',95.23,520,540,97.46,'Sun-Synch' from dual union all
select 40911,'040911','2015-049N','XW-2B','CAS-3B',69987,'In Earth orbit','','2015-09-22 00:00:00',95.22,520,539,97.46,'Sun-Synch' from dual union all
select 40912,'040912','2015-049P','DCBB','DCBB (CAS-3G/KT-1B)',69987,'In Earth orbit','','2015-09-22 00:00:00',95.13,514,536,97.46,'Sun-Synch' from dual union all
select 40913,'040913','2015-049Q','CZ-6 Stage 3','CZ-6  Stage 3',69987,'In Earth orbit','','2015-09-22 00:00:00',95.14,515,536,97.46,'Sun-Synch' from dual union all
select 40914,'040914','1982-118G','deb DMSP F-6','',54427,'In Earth orbit','','2016-06-18 00:00:00',100.44,768,791,98.55,'Sun-Synch' from dual union all
select 40915,'040915','1982-118H','deb DMSP F-6','',54427,'In Earth orbit','','2016-06-20 00:00:00',100.36,761,790,98.53,'Sun-Synch' from dual union all
select 40916,'040916','1982-118J','deb DMSP F-6','',54427,'In Earth orbit','','2016-06-20 00:00:00',100.36,761,790,98.52,'Sun-Synch' from dual union all
select 40917,'040917','1982-118K','deb DMSP F-6','',54427,'In Earth orbit','','2016-06-20 00:00:00',99.69,718,770,98.57,'Sun-Synch' from dual union all
select 40918,'040918','1982-118L','deb DMSP F-6','',54427,'In Earth orbit','','2016-06-20 00:00:00',98.51,588,787,98.49,'Sun-Synch' from dual union all
select 40919,'040919','1982-118M','deb DMSP F-6','',54427,'In Earth orbit','','2016-06-18 00:00:00',100.36,762,789,98.53,'Sun-Synch' from dual union all
select 4092,'004092','1969-077A','Kosmos-298','OGCh No. 21',18344,'Deorbited','1969-09-15 00:00:00','1969-09-15 00:00:00',87.37,127,162,49.6,'Intermediate' from dual union all
select 40920,'040920','2015-050A','Kosmos-2507','Rodnik-S',69988,'In Earth orbit','','2015-09-25 00:00:00',116.01,1497,1506,82.5,'Intermediate' from dual union all
select 40921,'040921','2015-050B','Kosmos-2508','Rodnik-S',69988,'In Earth orbit','','2015-09-25 00:00:00',115.99,1496,1504,82.49,'Intermediate' from dual union all
select 40922,'040922','2015-050C','Kosmos-2509','Rodnik-S',69988,'In Earth orbit','','2015-09-25 00:00:00',116.04,1498,1508,82.49,'Intermediate' from dual union all
select 40923,'040923','2015-050D','Briz-KM No. 72512','Briz-KM 14S45',69988,'In Earth orbit','','2015-09-25 00:00:00',112.45,1180,1499,82.49,'Intermediate' from dual union all
select 40924,'040924','1985-102G','deb Kosmos-1700','',58325,'In Earth orbit','','2015-10-25 00:00:00',1435.65,35708,35848,14.67,'Inclined GEO' from dual union all
select 40925,'040925','2015-051A','Pujiang-1','PJ-1',69989,'In Earth orbit','','2015-09-27 00:00:00',94.14,468,486,97.31,'Sun-Synch' from dual union all
select 40926,'040926','2015-051B','NJFA-1','TW-1C',69989,'In Earth orbit','','2015-09-27 00:00:00',94.12,466,486,97.31,'Sun-Synch' from dual union all
select 40927,'040927','2015-051C','NJUST-2','TW-1B',69989,'In Earth orbit','','2015-09-27 00:00:00',94.11,466,485,97.32,'Sun-Synch' from dual union all
select 40928,'040928','2015-051D','Shangkeda-2','TW-1A',69989,'In Earth orbit','','2015-09-27 00:00:00',94.1,466,484,97.31,'Sun-Synch' from dual union all
select 40929,'040929','2015-051E','CZ-11-4','CZ-11-4',69989,'Reentered','2015-10-02 00:00:00','2015-09-27 00:00:00',90.93,144,497,97.34,'Sun-Synch' from dual union all
select 4093,'004093','1969-077B','8S692','8S692',18344,'Reentered','1969-09-16 00:00:00','1969-09-15 00:00:00',87.24,123,153,49.57,'Intermediate' from dual union all
select 40930,'040930','2015-052A','ASTROSAT','ASTROSAT',69990,'In Earth orbit','','2015-09-29 00:00:00',97.54,632,649,5.99,'Equatorial' from dual union all
select 40931,'040931','2015-052B','LAPAN-A2/ORARI','LAPAN-A2',69990,'In Earth orbit','','2015-09-29 00:00:00',97.53,631,650,6,'Equatorial' from dual union all
select 40932,'040932','2015-052C','Lemur-2-Joel','Lemur-2-01',69990,'In Earth orbit','','2015-09-29 00:00:00',97.49,629,649,6,'Equatorial' from dual union all
select 40933,'040933','2015-052D','Lemur-2-Chris','Lemur-2-04',69990,'In Earth orbit','','2015-09-29 00:00:00',97.57,635,649,6,'Equatorial' from dual union all
select 40934,'040934','2015-052E','Lemur-2-Jeroen','Lemur-2-03',69990,'In Earth orbit','','2015-09-29 00:00:00',97.57,635,649,6.01,'Equatorial' from dual union all
select 40935,'040935','2015-052F','Lemur-2-Peter','Lemur-2-02',69990,'In Earth orbit','','2015-09-29 00:00:00',97.56,634,650,6.01,'Equatorial' from dual union all
select 40936,'040936','2015-052G','exactView-9','EV-9',69990,'In Earth orbit','','2015-09-29 00:00:00',97.5,630,649,6,'Equatorial' from dual union all
select 40937,'040937','2015-052H','PSLV-C30 PS4','PS4',69990,'In Earth orbit','','2015-07-13 00:00:00',97.32,626,635,98.11,'Sun-Synch' from dual union all
select 40938,'040938','2015-053A','Beidou DW 20','20 ke beidou daohang wei',69991,'In Earth orbit','','2015-10-08 00:00:00',1435.73,35606,35953,55.02,'Synchronous' from dual union all
select 40939,'040939','2015-053B','CZ-3B Stage 3','CZ-3B Stage 3',69991,'Reentered','2016-04-08 00:00:00','2015-10-01 00:00:00',656.65,241,37054,55.01,'GEO Transfer' from dual union all
select 4094,'004094','1969-077C','Kosmos-298 Adapter','OGCh Adapter',18344,'Reentered','1969-09-16 00:00:00','1969-09-15 00:00:00',87.6,131,180,49.59,'Intermediate' from dual union all
select 40940,'040940','2015-054A','Sky Muster','NBN Co. 1',69992,'In Earth orbit','','2015-10-08 00:00:00',1434.76,35744,35777,.07,'Drift GEO' from dual union all
select 40941,'040941','2015-054B','ARSAT-2','ARSAT-2',69992,'In Earth orbit','','2015-10-13 00:00:00',1436.55,34792,36799,.06,'Drift GEO' from dual union all
select 40942,'040942','2015-054C','ESC-A 580','Ariane 5 ESC-A',69992,'In Earth orbit','','2015-10-10 00:00:00',630.05,233,35701,6.29,'GEO Transfer' from dual union all
select 40943,'040943','2015-054D','Sylda 5A','Sylda 5C No. 64C',69992,'In Earth orbit','','2015-10-10 00:00:00',629.78,247,35673,5.99,'GEO Transfer' from dual union all
select 40944,'040944','2015-055A','Progress M-29M','Progress 7K-TGM No.429',69993,'Deorbited','2016-04-08 00:00:00','2015-10-01 00:00:00',90.97,308,337,51.64,'Intermediate' from dual union all
select 40945,'040945','2015-055B','11S510','11S510',69993,'Reentered','2015-10-03 00:00:00','2015-10-01 00:00:00',88.56,186,221,51.64,'Intermediate' from dual union all
select 40946,'040946','2015-056A','Morelos 3','Mexsat-2',69994,'In Earth orbit','','2015-10-18 00:00:00',1436.06,35767,35805,7.12,'Inclined GEO' from dual union all
select 40947,'040947','2015-056B','Centaur AV-059','Centaur AV-059',69994,'In Earth orbit','','2015-10-11 00:00:00',702.83,4316,35299,26.65,'GEO Transfer' from dual union all
select 40948,'040948','1998-067GZ','AAUSAT-5','AAUSAT5',66503,'Reentered','2016-03-15 00:00:00','2015-10-05 00:00:00',92.59,394,409,51.64,'Intermediate' from dual union all
select 40949,'040949','1998-067HA','GOMX-3','GOMX-3',66503,'Reentered','2016-10-18 00:00:00','2015-10-05 00:00:00',92.6,395,408,51.64,'Intermediate' from dual union all
select 4095,'004095','1969-077D','deb R-36','',18344,'Reentered','1969-09-16 00:00:00','1969-09-15 00:00:00',87.24,123,153,49.57,'Intermediate' from dual union all
select 40950,'040950','1998-067HB','Flock 2b-1','Dove 0C70',66503,'Reentered','2016-09-21 00:00:00','2015-10-06 00:00:00',92.6,400,403,51.65,'Intermediate' from dual union all
select 40951,'040951','1998-067HC','Flock 2b-2','Dove 0C18',66503,'Reentered','2016-10-02 00:00:00','2015-10-06 00:00:00',92.6,398,406,51.64,'Intermediate' from dual union all
select 40952,'040952','1998-067HD','Flock 2b-3','Dove 0C53',66503,'Reentered','2016-10-14 00:00:00','2015-10-06 00:00:00',92.6,399,405,51.64,'Intermediate' from dual union all
select 40953,'040953','1998-067HE','Flock 2b-4','Dove 0C72',66503,'Reentered','2016-10-17 00:00:00','2015-10-06 00:00:00',92.6,401,403,51.63,'Intermediate' from dual union all
select 40954,'040954','1998-067HF','Flock 2b-5','Dove 0C73',66503,'Reentered','2016-09-25 00:00:00','2015-10-06 00:00:00',92.61,396,409,51.65,'Intermediate' from dual union all
select 40955,'040955','1998-067HG','Flock 2b-6','Dove 0C69',66503,'Reentered','2016-08-22 00:00:00','2015-10-06 00:00:00',92.61,396,409,51.65,'Intermediate' from dual union all
select 40956,'040956','1998-067HH','Flock 2b-7','Dove 0C66',66503,'Reentered','2016-09-21 00:00:00','2015-10-06 00:00:00',92.6,396,408,51.64,'Intermediate' from dual union all
select 40957,'040957','1998-067HJ','Flock 2b-8','Dove 0C55',66503,'Reentered','2016-05-22 00:00:00','2015-10-06 00:00:00',92.61,397,408,51.64,'Intermediate' from dual union all
select 40958,'040958','2015-057A','Jilin-1 Jishu Yanzheng','LCY Weixing 1',69995,'In Earth orbit','','2015-10-09 00:00:00',97.73,638,662,98.04,'Sun-Synch' from dual;
insert into SATELLITE
select 40959,'040959','2015-057B','Jilin-1 Shipin 1','LSC Weixing 1',69995,'In Earth orbit','','2015-10-09 00:00:00',97.75,639,664,98.04,'Sun-Synch' from dual union all
select 4096,'004096','1969-077E','deb R-36','',18344,'Reentered','1969-09-16 00:00:00','1969-09-15 00:00:00',87.24,123,153,49.57,'Intermediate' from dual union all
select 40960,'040960','2015-057C','Jilin-1 Shipin 2','LSC Weixing 2',69995,'In Earth orbit','','2015-10-09 00:00:00',97.74,639,663,98.04,'Sun-Synch' from dual union all
select 40961,'040961','2015-057D','Jilin-1 Optical A','Jilin-1A',69995,'In Earth orbit','','2015-10-09 00:00:00',97.77,640,663,98.04,'Sun-Synch' from dual union all
select 40962,'040962','1998-067HK','Flock 2b-10','Dove 0C64',66503,'Reentered','2016-10-12 00:00:00','2015-10-07 00:00:00',92.6,395,408,51.64,'Intermediate' from dual union all
select 40963,'040963','1998-067HL','Flock 2b-9','Dove 0C68',66503,'Reentered','2016-10-16 00:00:00','2015-10-07 00:00:00',92.6,396,408,51.64,'Intermediate' from dual union all
select 40964,'040964','2015-058A','USA 264','[INTRUDER 13]',69996,'In Earth orbit','','2015-10-20 00:00:00',107.4,1014,1199,63.43,'Intermediate' from dual union all
select 40965,'040965','2015-058B','Aerocube 5c','AC5-C',69996,'In Earth orbit','','2015-10-13 00:00:00',97.74,500,802,64.78,'Intermediate' from dual union all
select 40966,'040966','2015-058C','Aerocube 7','OCSD A',69996,'In Earth orbit','','2015-10-13 00:00:00',97.74,500,802,64.78,'Intermediate' from dual union all
select 40967,'040967','2015-058D','AMSAT AO-85','Fox-1A',69996,'In Earth orbit','','2015-10-13 00:00:00',97.7,496,802,64.78,'Intermediate' from dual union all
select 40968,'040968','2015-058E','BisonSat','Nwist Qwiqway',69996,'In Earth orbit','','2015-10-13 00:00:00',97.7,497,801,64.78,'Intermediate' from dual union all
select 40969,'040969','2015-058F','ARC-1','ARC-1',69996,'In Earth orbit','','2015-10-13 00:00:00',97.71,497,801,64.78,'Intermediate' from dual union all
select 4097,'004097','1969-078A','Kosmos-299','Zenit-4 No. 65',18345,'Landed','1969-09-22 00:00:00','1969-09-19 00:00:00',89.38,206,282,64.98,'Intermediate' from dual union all
select 40970,'040970','2015-058G','SNAP-3 ALICE','SMDC Nanosatellite Progr',69996,'In Earth orbit','','2015-10-13 00:00:00',97.68,495,801,64.78,'Intermediate' from dual union all
select 40971,'040971','2015-058H','LMRST','LMRST',69996,'In Earth orbit','','2015-10-13 00:00:00',97.65,493,800,64.78,'Intermediate' from dual union all
select 40972,'040972','2015-058J','SNAP-3 EDDIE','SMDC Nanosatellite Progr',69996,'In Earth orbit','','2015-10-13 00:00:00',97.69,496,801,64.78,'Intermediate' from dual union all
select 40973,'040973','2015-058K','PropCube Merryweather','PropCube-1',69996,'In Earth orbit','','2015-10-13 00:00:00',97.66,494,800,64.78,'Intermediate' from dual union all
select 40974,'040974','2015-058L','SINOD-D-1','SINOD-D-1',69996,'In Earth orbit','','2015-10-13 00:00:00',97.67,494,800,64.78,'Intermediate' from dual union all
select 40975,'040975','2015-058M','SNAP-3 JIMI','SMDC Nanosatellite Progr',69996,'In Earth orbit','','2015-10-13 00:00:00',97.67,494,800,64.78,'Intermediate' from dual union all
select 40976,'040976','2015-058N','PropCube Flora','PropCube-3',69996,'In Earth orbit','','2015-10-13 00:00:00',97.65,493,800,64.78,'Intermediate' from dual union all
select 40977,'040977','2015-058P','SINOD-D-3','SINOD-D-3',69996,'In Earth orbit','','2015-10-13 00:00:00',97.66,494,800,64.78,'Intermediate' from dual union all
select 40978,'040978','2015-058Q','GRACE','NPSCuL-4 GRACE',69996,'In Earth orbit','','2015-10-20 00:00:00',97.36,481,784,64.77,'Intermediate' from dual union all
select 40979,'040979','1998-067HM','Flock 2b-13','Dove 0C74',66503,'Reentered','2016-10-11 00:00:00','2015-10-08 00:00:00',92.6,397,407,51.64,'Intermediate' from dual union all
select 4098,'004098','1969-078B','11A57I','11A57I',18345,'Reentered','1969-09-24 00:00:00','1969-09-21 00:00:00',88.85,198,238,64.98,'Intermediate' from dual union all
select 40980,'040980','1998-067HN','Flock 2b-14','Dove 0C47',66503,'Reentered','2016-09-12 00:00:00','2015-10-08 00:00:00',92.6,398,406,51.64,'Intermediate' from dual union all
select 40981,'040981','2015-058R','USA 264 P/L 2','[INTRUDER 14]',69996,'In Earth orbit','','2015-10-20 00:00:00',107.42,1014,1201,63.44,'Intermediate' from dual union all
select 40982,'040982','2015-059A','Apstar 9','Apstar 9',69997,'In Earth orbit','','2015-10-29 00:00:00',1436.01,35782,35788,.12,'Stationary' from dual union all
select 40983,'040983','2015-059B','CZ-3B Stage 3','CZ-3B Stage 3',69997,'Reentered','2016-01-15 00:00:00','2015-10-19 00:00:00',715.67,170,40081,27.13,'GEO Transfer' from dual union all
select 40984,'040984','2015-060A','Turksat 4B','Turksat 4B',69998,'In Earth orbit','','2015-10-30 00:00:00',1436.57,35787,35805,.03,'Drift GEO' from dual union all
select 40985,'040985','2015-060B','Briz-M No. 99557','Briz-M 14S43 No. 99557',69998,'In Earth orbit','','2015-10-26 00:00:00',751.41,7925,34076,13.51,'Medium' from dual union all
select 40986,'040986','2015-060C','Briz-M DTB','Briz-M torus tank',69998,'In Earth orbit','','2015-10-19 00:00:00',378.93,350,21586,49.36,'Highly Ellip' from dual union all
select 40987,'040987','2008-066D','FY-2E imager cover','',68574,'In Earth orbit','','2015-10-25 00:00:00',1435.5,35429,36121,2.83,'Stationary' from dual union all
select 40988,'040988','2015-061A','Tianhui 1-03','tianhui yi hao 03 xing',69999,'In Earth orbit','','2015-10-28 00:00:00',94.5,489,500,97.36,'Sun-Synch' from dual union all
select 40989,'040989','2015-034E','SEVIRI Entry Baffle Cover','MSG 4 Baffle Cover',69972,'In Earth orbit','','2015-10-29 00:00:00',1418.37,35301,35577,2.98,'Drift GEO' from dual union all
select 4099,'004099','1969-064R','deb Star 37D','',18331,'Reentered','1970-01-02 00:00:00','1969-09-19 00:00:00',129.59,266,3943,29.43,'Medium' from dual union all
select 40990,'040990','2015-034F','SEVIRI Cooler Cover','MSG 4 Cooler Cover',69972,'In Earth orbit','','2015-10-28 00:00:00',1402.75,34807,35456,2.98,'Drift GEO' from dual union all
select 40991,'040991','2015-044C','deb AV-056','',69982,'In Earth orbit','','2015-10-30 00:00:00',690.37,3856,35138,19.18,'GEO Transfer' from dual union all
select 40992,'040992','1997-051ABE','deb Iridium 33','',66131,'In Earth orbit','','2015-10-23 00:00:00',99.04,693,733,86.31,'Polar' from dual union all
select 40993,'040993','1997-051ABF','deb Iridium 33','',66131,'In Earth orbit','','2015-10-25 00:00:00',98.93,692,723,86.38,'Polar' from dual union all
select 40994,'040994','1997-051ABG','deb Iridium 33','',66131,'In Earth orbit','','2015-10-26 00:00:00',98.54,681,697,86.4,'Polar' from dual union all
select 40995,'040995','1997-051ABH','deb Iridium 33','',66131,'In Earth orbit','','2015-10-30 00:00:00',99.85,723,779,86.39,'Polar' from dual union all
select 40996,'040996','1997-051ABJ','deb Iridium 33','',66131,'In Earth orbit','','2015-10-30 00:00:00',99.67,736,749,86.38,'Polar' from dual union all
select 40997,'040997','1997-051ABK','deb Iridium 33','',66131,'In Earth orbit','','2015-10-30 00:00:00',97.84,648,663,86.37,'Polar' from dual union all
select 40998,'040998','1997-051ABL','deb Iridium 33','',66131,'In Earth orbit','','2015-10-28 00:00:00',100.24,764,775,86.39,'Polar' from dual union all
select 40999,'040999','1999-025EXD','deb FY-1C','',66706,'In Earth orbit','','2015-10-17 00:00:00',102.88,845,945,99.18,'Sun-Synch' from dual union all
select 41,'000041','60 Epsilon 8','Korabl''-Sputnik part','Vostok-1P part',6320,'Reentered','1960-10-01 00:00:00','1960-06-21 00:00:00',94.22,271,691,64.89,'Intermediate' from dual union all
select 410,'000410','61 Omicron 130','deb r/b Ablestar','',7273,'In Earth orbit','','1962-09-23 00:00:00',104.92,972,1010,66.7,'Intermediate' from dual union all
select 4100,'004100','1969-064S','deb Star 37D','',18331,'Reentered','1970-02-19 00:00:00','1969-09-19 00:00:00',134.29,282,4336,29.98,'Medium' from dual union all
select 41000,'041000','1999-025EXE','deb FY-1C','',66706,'In Earth orbit','','2015-10-18 00:00:00',106.17,757,1342,97.54,'Sun-Synch' from dual union all
select 41001,'041001','1999-025EXF','deb FY-1C','',66706,'In Earth orbit','','2015-10-27 00:00:00',101.72,814,867,98.94,'Sun-Synch' from dual union all
select 41002,'041002','1999-025EXG','deb FY-1C','',66706,'In Earth orbit','','2015-10-30 00:00:00',103.85,831,1050,98.17,'Sun-Synch' from dual union all
select 41003,'041003','1999-025EXH','deb FY-1C','',66706,'In Earth orbit','','2015-10-26 00:00:00',101.06,704,914,98.96,'Sun-Synch' from dual union all
select 41004,'041004','1999-025EXJ','deb FY-1C','',66706,'In Earth orbit','','2015-10-23 00:00:00',106.03,843,1243,99.16,'Sun-Synch' from dual union all
select 41005,'041005','1999-025EXK','deb FY-1C','',66706,'In Earth orbit','','2015-10-30 00:00:00',107.7,651,1589,98.95,'Sun-Synch' from dual union all
select 41006,'041006','1999-025EXL','deb FY-1C','',66706,'In Earth orbit','','2015-10-30 00:00:00',105.11,833,1167,102.38,'Sun-Synch' from dual union all
select 41007,'041007','1999-025EXM','deb FY-1C','',66706,'In Earth orbit','','2015-10-29 00:00:00',104.7,839,1122,99.97,'Sun-Synch' from dual union all
select 41008,'041008','1999-025EXN','deb FY-1C','',66706,'In Earth orbit','','2015-09-29 00:00:00',106.23,665,1439,99.51,'Sun-Synch' from dual union all
select 41009,'041009','1999-025EXP','deb FY-1C','',66706,'In Earth orbit','','2015-10-28 00:00:00',101.63,822,850,99.18,'Sun-Synch' from dual union all
select 4101,'004101','1969-064T','deb Star 37D','',18331,'Reentered','1969-09-23 00:00:00','1969-09-20 00:00:00',95.32,235,834,29.82,'Intermediate' from dual union all
select 41010,'041010','1999-025EXQ','deb FY-1C','',66706,'In Earth orbit','','2015-10-30 00:00:00',102.26,836,896,99.2,'Sun-Synch' from dual union all
select 41011,'041011','1999-025EXR','deb FY-1C','',66706,'In Earth orbit','','2015-10-30 00:00:00',100.98,540,1071,99.18,'Sun-Synch' from dual union all
select 41012,'041012','1999-025EXS','deb FY-1C','',66706,'In Earth orbit','','2015-10-08 00:00:00',99.52,706,765,99.09,'Sun-Synch' from dual union all
select 41013,'041013','1999-025EXT','deb FY-1C','',66706,'In Earth orbit','','2015-10-29 00:00:00',103.75,618,1254,99.1,'Sun-Synch' from dual union all
select 41014,'041014','1999-025EXU','deb FY-1C','',66706,'In Earth orbit','','2015-10-29 00:00:00',99.17,665,773,98.89,'Sun-Synch' from dual union all
select 41015,'041015','1999-025EXV','deb FY-1C','',66706,'In Earth orbit','','2015-10-29 00:00:00',97.71,486,813,99.27,'Sun-Synch' from dual union all
select 41016,'041016','1999-025EXW','deb FY-1C','',66706,'In Earth orbit','','2015-10-28 00:00:00',103.25,783,1041,98.23,'Sun-Synch' from dual union all
select 41017,'041017','1999-025EXX','deb FY-1C','',66706,'In Earth orbit','','2015-09-29 00:00:00',99.33,712,742,99.13,'Sun-Synch' from dual union all
select 41018,'041018','1999-025EXY','deb FY-1C','',66706,'In Earth orbit','','2015-10-29 00:00:00',107.09,848,1336,99.84,'Sun-Synch' from dual union all
select 41019,'041019','2015-062A','GPS IIF-11','GPS SVN 73',70000,'In Earth orbit','','2015-11-09 00:00:00',729.55,20443,20491,54.98,'Medium' from dual union all
select 4102,'004102','1969-079A','OPS 3531','CORONA J-46',18346,'Reentered Attached','1969-10-13 00:00:00','1969-09-29 00:00:00',88.8,176,254,85.02,'Polar' from dual union all
select 41020,'041020','2015-062B','Centaur AV-060','Centaur AV-060',70000,'In Earth orbit','','2015-11-07 00:00:00',756.35,20508,21733,55.31,'Medium' from dual union all
select 41021,'041021','2015-063A','Zhongxing 2C','Shentong-2',70001,'In Earth orbit','','2015-11-12 00:00:00',1436.04,35778,35793,.56,'Stationary' from dual union all
select 41022,'041022','2015-063B','CZ-3B Stage 3','CZ-3B Stage 3',70001,'In Earth orbit','','2015-11-11 00:00:00',657.34,221,37109,27.03,'GEO Transfer' from dual union all
select 41023,'041023','2015-061B','CZ-2D sep motor cover','',69999,'In Earth orbit','','2015-11-06 00:00:00',97.78,499,806,97.33,'Sun-Synch' from dual union all
select 41024,'041024','2015-061C','CZ-2D sep motor cover','',69999,'In Earth orbit','','2015-11-05 00:00:00',98.16,499,843,97.37,'Sun-Synch' from dual union all
select 41025,'041025','2015-061D','deb Tianhui-1 03','',69999,'Lost','','2015-11-03 00:00:00',94.48,487,499,97.36,'Sun-Synch' from dual union all
select 41026,'041026','2015-064A','Yaogan Weixing 28','Jianbing-11 02 xing',70002,'In Earth orbit','','2015-11-10 00:00:00',94,459,481,97.24,'Sun-Synch' from dual union all
select 41027,'041027','2015-064B','CZ-4B Stage 3','CZ-4B Stage 3',70002,'Reentered','2015-12-26 00:00:00','2015-11-10 00:00:00',91.25,216,456,97.36,'Sun-Synch' from dual union all
select 41028,'041028','2015-065A','GSAT-15','GSAT-15',70003,'In Earth orbit','','2015-11-17 00:00:00',1431.8,35642,35762,.1,'Drift GEO' from dual union all
select 41029,'041029','2015-065B','Badr 7','Arabsat 6B',70003,'In Earth orbit','','2015-11-17 00:00:00',1430.57,35630,35727,.09,'Drift GEO' from dual union all
select 4103,'004103','1969-079B','SAVANT II','P-11 No. 4419',18346,'Reentered','1971-05-16 00:00:00','1969-10-30 00:00:00',94.35,484,491,85.17,'Polar' from dual union all
select 41030,'041030','2015-065C','ESC-A 581','Ariane 5 ESC-A',70003,'In Earth orbit','','2015-11-18 00:00:00',632.21,261,35784,4.1,'GEO Transfer' from dual union all
select 41031,'041031','2015-065D','Sylda 5A','Sylda 5A No. 67A',70003,'In Earth orbit','','2015-11-18 00:00:00',629.93,247,35681,3.97,'GEO Transfer' from dual union all
select 41032,'041032','2015-066A','Kosmos-2510','EKS No. 1L',70004,'In Earth orbit','','2015-11-24 00:00:00',714.2,1621,38557,63.81,'Molniya' from dual union all
select 41033,'041033','2015-066B','Fregat No. 1033','Fregat 14S44 No. 1033',70004,'In Earth orbit','','2015-11-24 00:00:00',714.44,1631,38559,63.82,'Molniya' from dual union all
select 41034,'041034','2015-067A','LaoSat 1','LaoSat 1',70005,'In Earth orbit','','2015-11-30 00:00:00',1436.11,35781,35793,.11,'Stationary' from dual union all
select 41035,'041035','2015-067B','CZ-3B Stage 3','CZ-3B Stage 3',70005,'Reentered','2016-07-03 00:00:00','2015-11-28 00:00:00',710.82,156,39855,18.31,'GEO Transfer' from dual union all
select 41036,'041036','2015-068A','Telstar 12 Vantage','Telstar 12V',70006,'In Earth orbit','','2015-12-02 00:00:00',1430.6,35637,35721,.1,'Drift GEO' from dual union all
select 41037,'041037','2015-068B','H-2A F29 Stage 2','H-2A Stage 2',70006,'In Earth orbit','','2015-12-03 00:00:00',677.4,2710,35634,19.14,'GEO Transfer' from dual union all
select 41038,'041038','2015-069A','Yaogan Weixing 29','Jianbing-5 04 xing',70007,'In Earth orbit','','2015-11-28 00:00:00',97.24,626,627,97.84,'Sun-Synch' from dual union all
select 41039,'041039','2015-069B','CZ-4C Stage 3','CZ-4C Stage 3',70007,'In Earth orbit','','2015-11-28 00:00:00',95.69,483,621,97.79,'Sun-Synch' from dual union all
select 4104,'004104','1969-080A','Kosmos-300','E-8-5 No. 403',18347,'Reentered','1969-09-27 00:00:00','1969-09-25 00:00:00',88.03,175,179,51.45,'Intermediate' from dual union all
select 41040,'041040','2015-064C','CZ-4C Sep motor cover','',70002,'In Earth orbit','','2015-11-29 00:00:00',93.94,455,480,97.25,'Sun-Synch' from dual union all
select 41041,'041041','2015-064D','CZ-4C Sep motor cover','',70002,'In Earth orbit','','2015-11-29 00:00:00',97.39,469,798,97.27,'Sun-Synch' from dual union all
select 41042,'041042','2015-064E','CZ-4C Sep motor cover','',70002,'In Earth orbit','','2015-11-28 00:00:00',97.27,467,789,97.19,'Sun-Synch' from dual union all
select 41043,'041043','2015-070A','LISA Pathfinder','LISA',70008,'Deep Space','2015-12-12 00:00:00','2015-12-04 00:00:00',101.82,215,1475,5.97,'Equatorial' from dual union all
select 41044,'041044','2015-070B','Vega AVUM VV06','AVUM',70008,'Reentered','2017-03-16 00:00:00','2015-12-04 00:00:00',102.24,209,1521,5.97,'Equatorial' from dual union all
select 41045,'041045','2000-055C','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.81,832,857,98.9,'Sun-Synch' from dual union all
select 41046,'041046','2000-055D','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.88,837,859,98.94,'Sun-Synch' from dual union all
select 41047,'041047','2000-055E','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.9,844,854,98.93,'Sun-Synch' from dual;
insert into SATELLITE
select 41048,'041048','2000-055F','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.91,844,855,98.92,'Sun-Synch' from dual union all
select 41049,'041049','2000-055G','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',102.04,851,860,98.9,'Sun-Synch' from dual union all
select 4105,'004105','1969-080B','Blok D No. 403L','11S824',18347,'Reentered','1969-09-27 00:00:00','1969-09-25 00:00:00',87.95,173,173,51.55,'Intermediate' from dual union all
select 41050,'041050','2000-055H','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.68,820,856,98.94,'Sun-Synch' from dual union all
select 41051,'041051','2000-055J','deb NOAA 16','',66957,'In Earth orbit','','2015-12-02 00:00:00',101.86,833,861,98.92,'Sun-Synch' from dual union all
select 41052,'041052','2000-055K','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',102.56,755,1005,99.01,'Sun-Synch' from dual union all
select 41053,'041053','2000-055L','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.46,800,856,98.97,'Sun-Synch' from dual union all
select 41054,'041054','2000-055M','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101,754,858,98.99,'Sun-Synch' from dual union all
select 41055,'041055','2000-055N','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.76,827,857,98.91,'Sun-Synch' from dual union all
select 41056,'041056','2000-055P','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',102.23,853,876,98.91,'Sun-Synch' from dual union all
select 41057,'041057','2000-055Q','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.65,816,858,98.94,'Sun-Synch' from dual union all
select 41058,'041058','2000-055R','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',102.36,851,890,98.92,'Sun-Synch' from dual union all
select 41059,'041059','2000-055S','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.76,825,859,98.94,'Sun-Synch' from dual union all
select 4106,'004106','1969-081A','Kosmos-301','Zenit-2 No. 79',18348,'Landed','1969-10-02 00:00:00','1969-09-26 00:00:00',89.28,193,285,65.41,'Intermediate' from dual union all
select 41060,'041060','2000-055T','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',100.8,730,862,98.96,'Sun-Synch' from dual union all
select 41061,'041061','2000-055U','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.76,825,859,98.93,'Sun-Synch' from dual union all
select 41062,'041062','2000-055V','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',102.5,855,899,98.93,'Sun-Synch' from dual union all
select 41063,'041063','2000-055W','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',102.18,843,880,98.95,'Sun-Synch' from dual union all
select 41064,'041064','2000-055X','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.82,828,862,98.91,'Sun-Synch' from dual union all
select 41065,'041065','2000-055Y','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',102.17,852,871,98.93,'Sun-Synch' from dual union all
select 41066,'041066','2000-055Z','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.78,830,856,98.94,'Sun-Synch' from dual union all
select 41067,'041067','2000-055AA','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.98,848,856,98.93,'Sun-Synch' from dual union all
select 41068,'041068','2000-055AB','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',100.7,672,911,99.14,'Sun-Synch' from dual union all
select 41069,'041069','2000-055AC','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',100.6,695,879,98.72,'Sun-Synch' from dual union all
select 4107,'004107','1969-081B','11A57I','11A57I',18348,'Reentered','1969-09-29 00:00:00','1969-09-26 00:00:00',88.97,195,253,65.39,'Intermediate' from dual union all
select 41070,'041070','2000-055AD','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',97.9,462,854,98.8,'Sun-Synch' from dual union all
select 41071,'041071','2000-055AE','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.98,842,863,98.96,'Sun-Synch' from dual union all
select 41072,'041072','2000-055AF','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',106.45,848,1277,98.84,'Sun-Synch' from dual union all
select 41073,'041073','2000-055AG','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.65,820,854,98.91,'Sun-Synch' from dual union all
select 41074,'041074','2000-055AH','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',100.46,706,854,98.77,'Sun-Synch' from dual union all
select 41075,'041075','2000-055AJ','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',99.09,547,884,99.08,'Sun-Synch' from dual union all
select 41076,'041076','2000-055AK','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',99.02,558,866,99.19,'Sun-Synch' from dual union all
select 41077,'041077','2000-055AL','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',103.53,840,1011,99.22,'Sun-Synch' from dual union all
select 41078,'041078','2000-055AM','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',106.18,831,1269,98.92,'Sun-Synch' from dual union all
select 41079,'041079','2000-055AN','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',104.25,852,1067,99.07,'Sun-Synch' from dual union all
select 4108,'004108','1969-064U','deb Star 37D','',18331,'Reentered','1970-01-30 00:00:00','1969-09-24 00:00:00',135.67,282,4454,29.54,'Medium' from dual union all
select 41080,'041080','2000-055AP','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.91,844,855,98.92,'Sun-Synch' from dual union all
select 41081,'041081','2000-055AQ','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.91,741,958,99.15,'Sun-Synch' from dual union all
select 41082,'041082','2000-055AR','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',105.26,843,1171,98.77,'Sun-Synch' from dual union all
select 41083,'041083','2000-055AS','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',104.89,836,1143,99,'Sun-Synch' from dual union all
select 41084,'041084','2000-055AT','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.73,826,856,98.93,'Sun-Synch' from dual union all
select 41085,'041085','2000-055AU','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.71,823,857,98.93,'Sun-Synch' from dual union all
select 41086,'041086','2000-055AV','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.23,779,855,98.84,'Sun-Synch' from dual union all
select 41087,'041087','2000-055AW','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',102.26,855,877,98.94,'Sun-Synch' from dual union all
select 41088,'041088','2000-055AX','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.25,777,859,98.92,'Sun-Synch' from dual union all
select 41089,'041089','2000-055AY','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',102.02,850,859,98.9,'Sun-Synch' from dual union all
select 4109,'004109','1969-064V','deb Star 37D','',18331,'Reentered','1970-09-04 00:00:00','1969-09-24 00:00:00',137.94,257,4674,30.8,'Medium' from dual union all
select 41090,'041090','2000-055AZ','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.53,805,857,98.93,'Sun-Synch' from dual union all
select 41091,'041091','2000-055BA','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',102.6,837,927,99.03,'Sun-Synch' from dual union all
select 41092,'041092','2000-055BB','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',100.69,728,855,98.88,'Sun-Synch' from dual union all
select 41093,'041093','2000-055BC','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',100.58,717,856,98.89,'Sun-Synch' from dual union all
select 41094,'041094','2000-055BD','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',100.97,752,857,99.01,'Sun-Synch' from dual union all
select 41095,'041095','2000-055BE','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',101.17,774,855,99,'Sun-Synch' from dual union all
select 41096,'041096','2000-055BF','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',99.81,643,856,98.87,'Sun-Synch' from dual union all
select 41097,'041097','2000-055BG','deb NOAA 16','',66957,'In Earth orbit','','2015-12-04 00:00:00',103.63,855,1006,98.93,'Sun-Synch' from dual union all
select 41098,'041098','2015-071A','Kosmos-2511','Kanopus-ST No. 1',70009,'Reentered','2015-12-08 00:00:00','2015-12-05 00:00:00',98.55,685,694,98.18,'Sun-Synch' from dual union all
select 41099,'041099','2015-071B','Kosmos-2512','KYuA No. 1',70009,'In Earth orbit','','2015-12-05 00:00:00',98.53,684,693,98.18,'Sun-Synch' from dual union all
select 411,'000411','61 Omicron 131','deb r/b Ablestar','',7273,'Reentered','1978-06-20 00:00:00','1962-09-27 00:00:00',101.87,740,955,66.92,'Intermediate' from dual union all
select 4110,'004110','1969-064W','deb Star 37D','',18331,'Reentered','1969-10-24 00:00:00','1969-09-24 00:00:00',119.38,261,3046,31.54,'Intermediate' from dual union all
select 41100,'041100','2015-071C','Blok-I 14S54','14S54',70009,'Reentered','2016-03-16 00:00:00','2015-12-05 00:00:00',93.47,208,681,98.19,'Sun-Synch' from dual union all
select 41101,'041101','2015-072A','SS Deke Slayton II','Cygnus PCM-4',70010,'Deorbited','2016-02-20 00:00:00','2015-12-08 00:00:00',92.78,404,418,51.65,'Intermediate' from dual union all
select 41102,'041102','2015-071D','Volga No. 002','Volga',70009,'Reentered','2015-12-08 00:00:00','2015-12-07 00:00:00',88.86,89,348,98.17,'Sun-Synch' from dual union all
select 41103,'041103','2015-073A','Zhongxing 1C','Fenghuo-2',70011,'In Earth orbit','','2015-12-20 00:00:00',1436.09,35777,35796,.11,'Stationary' from dual union all
select 41104,'041104','2015-073B','CZ-3B Stage 3','CZ-3B Stage 3',70011,'Reentered','2016-05-14 00:00:00','2015-12-11 00:00:00',602.54,140,34366,27.08,'GEO Transfer' from dual union all
select 41105,'041105','2015-074A','Elektro-L No. 2','Elektro-L',70012,'In Earth orbit','','2015-12-20 00:00:00',1434.04,35680,35813,.46,'Drift GEO' from dual union all
select 41106,'041106','2015-074B','Fregat-SB No. 222-02','Fregat-SB    No. 222-02',70012,'In Earth orbit','','2015-12-21 00:00:00',1399.46,34314,35819,.49,'Drift GEO' from dual union all
select 41107,'041107','2015-074C','11S772 2SB80.4','11S772',70012,'Reentered','2016-01-01 00:00:00','2015-12-13 00:00:00',91.37,164,519,51.38,'Intermediate' from dual union all
select 41108,'041108','2015-074D','Fregat SBB','',70012,'In Earth orbit','','2015-12-13 00:00:00',133.46,271,4275,50.38,'Medium' from dual union all
select 41109,'041109','2015-074E','Sep Motor Cover','Sep Motor Cover',70012,'Reentered','2015-12-25 00:00:00','2015-12-13 00:00:00',93.95,164,772,51.37,'Intermediate' from dual union all
select 4111,'004111','1969-082A','WESTON','P-11 No. 4407',18349,'Reentered','1970-10-30 00:00:00','1969-10-30 00:00:00',93.78,442,477,69.65,'Intermediate' from dual union all
select 41110,'041110','2015-074F','Sep Motor Cover','Sep Motor Cover',70012,'Reentered','2015-12-29 00:00:00','2015-12-13 00:00:00',94.2,178,782,51.37,'Intermediate' from dual union all
select 41111,'041111','2015-074G','Sep Motor Cover','Sep Motor Cover',70012,'Reentered','2016-01-01 00:00:00','2015-12-13 00:00:00',94.31,177,794,51.4,'Intermediate' from dual union all
select 41112,'041112','2015-074H','Sep Motor Cover','Sep Motor Cover',70012,'Reentered','2015-12-24 00:00:00','2015-12-13 00:00:00',94.14,163,791,51.41,'Intermediate' from dual union all
select 41113,'041113','2015-074J','deb Fregat SBB','',70012,'Reentered','2017-11-13 00:00:00','2015-12-13 00:00:00',132.33,268,4180,50.49,'Medium' from dual union all
select 41114,'041114','2015-074K','deb Fregat SBB','',70012,'Reentered','2016-01-12 00:00:00','2015-12-15 00:00:00',117.52,268,2871,50.34,'Intermediate' from dual union all
select 41115,'041115','2015-074L','deb Fregat SBB','',70012,'Reentered','2016-01-14 00:00:00','2015-12-14 00:00:00',128.63,265,3861,50.32,'Medium' from dual union all
select 41116,'041116','1980-085M','deb Kosmos-1217','',50295,'Reentered','2016-04-20 00:00:00','2015-12-09 00:00:00',614.01,197,34907,63.5,'GEO Transfer' from dual union all
select 41117,'041117','1980-085N','deb Kosmos-1217','',50295,'In Earth orbit','','2015-12-15 00:00:00',723.06,1105,39510,65.65,'GEO Transfer' from dual union all
select 41118,'041118','1991-015Q','deb Ariane V42','',64184,'In Earth orbit','','2015-12-15 00:00:00',507.96,661,28760,3.88,'Highly Ellip' from dual union all
select 41119,'041119','2011-077AJ','deb CZ-3B','',69129,'In Earth orbit','','2015-12-08 00:00:00',848.29,1705,44907,21.88,'Highly Ellip' from dual union all
select 4112,'004112','1969-082CZ','deb Agena D','',18349,'Reentered','2014-08-19 00:00:00','1992-02-03 00:00:00',101.38,779,869,69.8,'Intermediate' from dual union all
select 41120,'041120','2011-077AK','deb CZ-3B','',69129,'In Earth orbit','','2015-12-15 00:00:00',762.3,1054,41475,26.54,'Highly Ellip' from dual union all
select 41121,'041121','2015-075A','Kosmos-2513','Garpun No. 12L',70013,'In Earth orbit','','2015-12-16 00:00:00',631.02,402,35582,48.58,'GEO Transfer' from dual union all
select 41122,'041122','2015-075B','Briz-M No. 88533','Briz-M',70013,'In Earth orbit','','2015-12-21 00:00:00',1375.9,33552,35648,.13,'Medium' from dual union all
select 41123,'041123','2015-075C','Briz-M DTB','Briz-M torus tank',70013,'In Earth orbit','','2015-12-21 00:00:00',631.02,407,35577,48.59,'GEO Transfer' from dual union all
select 41124,'041124','2015-076A','Soyuz TMA-19M','Soyuz 11F732A47 No. 719',70014,'Landed','2016-06-18 00:00:00','2015-12-15 00:00:00',91.35,321,361,51.65,'Intermediate' from dual union all
select 41125,'041125','2015-076B','11S510','11S510',70014,'Reentered','2015-12-17 00:00:00','2015-12-16 00:00:00',88.23,178,196,51.62,'Intermediate' from dual union all
select 41126,'041126','2000-055BH','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',102.98,843,957,98.92,'Sun-Synch' from dual union all
select 41127,'041127','2000-055BJ','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',100.67,726,855,98.84,'Sun-Synch' from dual union all
select 41128,'041128','2000-055BK','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',102.08,852,863,99.05,'Sun-Synch' from dual union all
select 41129,'041129','2000-055BL','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',100.65,723,856,98.88,'Sun-Synch' from dual union all
select 4113,'004113','1969-082DA','deb Agena D','',18349,'Reentered','1969-10-21 00:00:00','1969-10-07 00:00:00',93.3,425,448,69.62,'Intermediate' from dual union all
select 41130,'041130','2000-055BM','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',101.64,819,854,98.94,'Sun-Synch' from dual union all
select 41131,'041131','2000-055BN','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',103.08,853,956,98.97,'Sun-Synch' from dual union all
select 41132,'041132','2000-055BP','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',101.3,784,857,98.87,'Sun-Synch' from dual union all
select 41133,'041133','2000-055BQ','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',101.94,847,854,98.92,'Sun-Synch' from dual union all
select 41134,'041134','2000-055BR','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',102.34,847,892,98.94,'Sun-Synch' from dual union all
select 41135,'041135','2000-055BS','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',101.12,768,856,98.87,'Sun-Synch' from dual union all
select 41136,'041136','2000-055BT','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',101.36,793,854,99.1,'Sun-Synch' from dual union all
select 41137,'041137','2000-055BU','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',102.2,842,884,98.9,'Sun-Synch' from dual;
insert into SATELLITE
select 41138,'041138','2000-055BV','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',102,849,858,98.92,'Sun-Synch' from dual union all
select 41139,'041139','2000-055BW','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',103.16,851,965,98.98,'Sun-Synch' from dual union all
select 4114,'004114','1969-083A','Boreas','ESRO 1B',18350,'Reentered','1969-11-23 00:00:00','1969-10-08 00:00:00',91.2,289,378,85.13,'Polar' from dual union all
select 41140,'041140','2000-055BX','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',101.15,766,861,98.84,'Sun-Synch' from dual union all
select 41141,'041141','2000-055BY','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',101.05,758,858,98.98,'Sun-Synch' from dual union all
select 41142,'041142','2000-055BZ','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',101.2,768,862,98.87,'Sun-Synch' from dual union all
select 41143,'041143','2000-055CA','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',103.59,853,1004,98.89,'Sun-Synch' from dual union all
select 41144,'041144','2000-055CB','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',101.96,828,875,98.84,'Sun-Synch' from dual union all
select 41145,'041145','2000-055CC','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',100.13,672,858,99.01,'Sun-Synch' from dual union all
select 41146,'041146','2000-055CD','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',102.14,838,881,98.91,'Sun-Synch' from dual union all
select 41147,'041147','2000-055CE','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',103.42,851,989,98.95,'Sun-Synch' from dual union all
select 41148,'041148','2000-055CF','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',100.48,712,850,98.88,'Sun-Synch' from dual union all
select 41149,'041149','2000-055CG','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',100.97,750,860,98.93,'Sun-Synch' from dual union all
select 4115,'004115','1969-082DC','deb Agena D','',18349,'Reentered','1969-10-21 00:00:00','1969-10-07 00:00:00',93.42,430,454,69.63,'Intermediate' from dual union all
select 41150,'041150','2000-055CH','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',101.19,769,861,98.83,'Sun-Synch' from dual union all
select 41151,'041151','2000-055CJ','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',103.2,850,969,98.91,'Sun-Synch' from dual union all
select 41152,'041152','2000-055CK','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',101.78,833,853,98.93,'Sun-Synch' from dual union all
select 41153,'041153','2000-055CL','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',101.79,829,858,98.92,'Sun-Synch' from dual union all
select 41154,'041154','2000-055CM','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',102.99,850,951,98.98,'Sun-Synch' from dual union all
select 41155,'041155','2000-055CN','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',101.85,831,861,98.95,'Sun-Synch' from dual union all
select 41156,'041156','2000-055CP','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',101.11,768,855,98.94,'Sun-Synch' from dual union all
select 41157,'041157','2000-055CQ','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',102.52,847,909,98.97,'Sun-Synch' from dual union all
select 41158,'041158','2000-055CR','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',101.8,833,855,98.97,'Sun-Synch' from dual union all
select 41159,'041159','2000-055CS','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',101.69,821,857,98.86,'Sun-Synch' from dual union all
select 4116,'004116','1969-083C','ESRO 1B despin weight','',18350,'Reentered','1969-10-21 00:00:00','1969-10-08 00:00:00',90.85,281,351,85.6,'Polar' from dual union all
select 41160,'041160','2000-055CT','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',102.32,851,885,98.91,'Sun-Synch' from dual union all
select 41161,'041161','2000-055CU','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',102.34,837,901,98.94,'Sun-Synch' from dual union all
select 41162,'041162','2000-055CV','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',102.28,841,892,98.91,'Sun-Synch' from dual union all
select 41163,'041163','2000-055CW','deb NOAA 16','',66957,'In Earth orbit','','2015-12-15 00:00:00',100.92,749,856,99.03,'Sun-Synch' from dual union all
select 41164,'041164','2000-055CX','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',103.07,852,956,99.03,'Sun-Synch' from dual union all
select 41165,'041165','2000-055CY','deb NOAA 16','',66957,'In Earth orbit','','2015-12-14 00:00:00',102.31,847,889,98.81,'Sun-Synch' from dual union all
select 41166,'041166','2015-077A','VELOX-C1','VELOX-C1',70015,'In Earth orbit','','2015-12-18 00:00:00',95.45,531,549,14.99,'Equatorial' from dual union all
select 41167,'041167','2015-077B','Kent Ridge 1','Kent Ridge 1',70015,'In Earth orbit','','2015-12-18 00:00:00',95.42,529,549,14.99,'Equatorial' from dual union all
select 41168,'041168','2015-077C','Athenoxat-1','Athenoxat-1',70015,'In Earth orbit','','2015-12-18 00:00:00',95.48,534,550,14.98,'Equatorial' from dual union all
select 41169,'041169','2015-077D','TeLEOS-1','TeLEOS-1',70015,'In Earth orbit','','2015-12-18 00:00:00',95.47,533,550,14.98,'Equatorial' from dual union all
select 4117,'004117','1969-083B','FW4S S/N 2223-6','FW4S/LaRC perf. expt.',18350,'Reentered','1969-11-03 00:00:00','1969-10-08 00:00:00',91.15,294,368,85.14,'Polar' from dual union all
select 41170,'041170','2015-077E','Galassia','Galassia',70015,'In Earth orbit','','2015-12-18 00:00:00',95.5,536,550,14.99,'Equatorial' from dual union all
select 41171,'041171','2015-077F','VELOX-II','VELOX-II',70015,'In Earth orbit','','2015-12-19 00:00:00',95.47,533,550,14.98,'Equatorial' from dual union all
select 41172,'041172','2015-077G','PS4','PS4 PSLV-C29',70015,'In Earth orbit','','2015-12-18 00:00:00',94.32,399,573,14.69,'Equatorial' from dual union all
select 41173,'041173','2015-078A','Wukong','DAMPE',70016,'In Earth orbit','','2015-12-20 00:00:00',94.52,487,504,97.3,'Sun-Synch' from dual union all
select 41174,'041174','2015-079A','GalileoSat-11','Galileo FOC FM08',70017,'In Earth orbit','','2015-12-30 00:00:00',849.02,23226,23420,54.88,'Medium' from dual union all
select 41175,'041175','2015-079B','GalileoSat-12','Galileo FOC FM09',70017,'In Earth orbit','','2015-12-30 00:00:00',852.54,23347,23463,54.79,'Medium' from dual union all
select 41176,'041176','2015-079C','Fregat-MT No. 133-04','Fregat-MT    No. 133-04',70017,'In Earth orbit','','2015-12-25 00:00:00',859.37,23535,23593,54.97,'Medium' from dual union all
select 41177,'041177','2015-080A','Progress MS-01','Progress 7K-TGM No. 431',70018,'Deorbited','2016-07-03 00:00:00','2015-12-21 00:00:00',90,270,279,51.64,'Intermediate' from dual union all
select 41178,'041178','2015-080B','Blok-I 14S54/RD-0110','374BL02A',70018,'Reentered','2015-12-23 00:00:00','2015-12-21 00:00:00',88.3,179,202,51.66,'Intermediate' from dual union all
select 41179,'041179','2015-081A','Orbcomm FM114','Orbcomm OG2 SV14',70019,'In Earth orbit','','2015-12-22 00:00:00',97.41,613,657,47,'Intermediate' from dual union all
select 4118,'004118','1969-083D','ESRO 1B despin weight','',18350,'Reentered','1969-10-25 00:00:00','1969-10-10 00:00:00',90.87,293,342,84.65,'Intermediate' from dual union all
select 41180,'041180','2015-081B','Orbcomm FM119','Orbcomm OG2 SV19',70019,'In Earth orbit','','2015-12-22 00:00:00',97.41,613,657,47,'Intermediate' from dual union all
select 41181,'041181','2015-081C','Orbcomm FM105','Orbcomm OG2 SV5',70019,'In Earth orbit','','2015-12-22 00:00:00',97.43,614,657,47,'Intermediate' from dual union all
select 41182,'041182','2015-081D','Orbcomm FM110','Orbcomm OG2 SV10',70019,'In Earth orbit','','2015-12-22 00:00:00',97.42,614,656,47,'Intermediate' from dual union all
select 41183,'041183','2015-081E','Orbcomm FM118','Orbcomm OG2 SV18',70019,'In Earth orbit','','2015-12-22 00:00:00',97.42,614,656,47,'Intermediate' from dual union all
select 41184,'041184','2015-081F','Orbcomm FM112','Orbcomm OG2 SV12',70019,'In Earth orbit','','2015-12-22 00:00:00',97.42,614,656,47,'Intermediate' from dual union all
select 41185,'041185','2015-081G','Orbcomm FM113','Orbcomm OG2 SV13',70019,'In Earth orbit','','2015-12-22 00:00:00',97.39,613,655,47,'Intermediate' from dual union all
select 41186,'041186','2015-081H','Orbcomm FM115','Orbcomm OG2 SV15',70019,'In Earth orbit','','2015-12-22 00:00:00',97.37,612,654,47,'Intermediate' from dual union all
select 41187,'041187','2015-081J','Orbcomm FM108','Orbcomm OG2 SV8',70019,'In Earth orbit','','2015-12-22 00:00:00',97.4,613,656,47,'Intermediate' from dual union all
select 41188,'041188','2015-081K','Orbcomm FM117','Orbcomm OG2 SV17',70019,'In Earth orbit','','2015-12-22 00:00:00',97.41,614,655,47,'Intermediate' from dual union all
select 41189,'041189','2015-081L','Orbcomm FM116','Orbcomm OG2 SV16',70019,'In Earth orbit','','2015-12-22 00:00:00',97.38,613,654,47,'Intermediate' from dual union all
select 4119,'004119','1969-084A','Meteor','Meteor 11F614M No. 15',18351,'Reentered','2002-08-20 00:00:00','1969-11-13 00:00:00',97.63,618,673,81.24,'Intermediate' from dual union all
select 41190,'041190','1987-038C','deb Kosmos-1842','',60128,'Reentered','2016-01-13 00:00:00','2015-12-22 00:00:00',92.79,407,416,82.5,'Intermediate' from dual union all
select 41191,'041191','2015-082A','Ekspress AMU-1','Ekspress AMU-1',70020,'In Earth orbit','','2016-01-03 00:00:00',712.15,4325,35751,22.41,'GEO Transfer' from dual union all
select 41192,'041192','2015-082B','Briz-M No. 99559','Briz-M',70020,'In Earth orbit','','2016-01-03 00:00:00',664.91,3689,34025,22.94,'GEO Transfer' from dual union all
select 41193,'041193','2015-082C','Briz-M DTB','Briz-M torus tank',70020,'In Earth orbit','','2016-01-03 00:00:00',281.94,333,15396,49.54,'Highly Ellip' from dual union all
select 41194,'041194','2015-083A','Gao Fen 4','Gaofen si hao weixing',70021,'In Earth orbit','','2016-01-07 00:00:00',1436.06,35784,35788,.58,'Stationary' from dual union all
select 41195,'041195','2015-083B','CZ-3B Stage 3','CZ-3B Stage 3',70021,'In Earth orbit','','2016-01-10 00:00:00',600.99,163,34262,22.96,'GEO Transfer' from dual union all
select 41196,'041196','2000-055CZ','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',102.9,853,939,98.94,'Sun-Synch' from dual union all
select 41197,'041197','2000-055DA','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',102.67,849,921,98.77,'Sun-Synch' from dual union all
select 41198,'041198','2000-055DB','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',102.59,853,910,98.94,'Sun-Synch' from dual union all
select 41199,'041199','2000-055DC','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',103.01,850,952,98.96,'Sun-Synch' from dual union all
select 412,'000412','61 Omicron 132','deb r/b Ablestar','',7273,'Reentered','1980-06-30 00:00:00','1962-09-26 00:00:00',103.24,813,1011,66.44,'Intermediate' from dual union all
select 4120,'004120','1969-084B','8A92ME','8A92ME',18351,'Reentered','1999-06-15 00:00:00','1969-11-13 00:00:00',97.78,545,759,81.25,'Intermediate' from dual union all
select 41200,'041200','2000-055DD','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',101.6,817,852,98.89,'Sun-Synch' from dual union all
select 41201,'041201','2000-055DE','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',102.8,854,929,98.98,'Sun-Synch' from dual union all
select 41202,'041202','2000-055DF','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',103.31,853,978,98.93,'Sun-Synch' from dual union all
select 41203,'041203','2000-055DG','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',102.4,844,901,98.94,'Sun-Synch' from dual union all
select 41204,'041204','2000-055DH','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',102.33,851,886,98.97,'Sun-Synch' from dual union all
select 41205,'041205','2000-055DJ','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',101.83,829,862,98.97,'Sun-Synch' from dual union all
select 41206,'041206','2000-055DK','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',102.09,848,867,98.86,'Sun-Synch' from dual union all
select 41207,'041207','2000-055DL','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',100.91,751,853,98.9,'Sun-Synch' from dual union all
select 41208,'041208','2000-055DM','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',102.61,853,912,98.89,'Sun-Synch' from dual union all
select 41209,'041209','2000-055DN','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',100.94,752,854,98.89,'Sun-Synch' from dual union all
select 4121,'004121','1969-025G','deb OV1-18','',18292,'Reentered','1969-11-06 00:00:00','1969-10-08 00:00:00',94.59,452,545,98.85,'Sun-Synch' from dual union all
select 41210,'041210','2000-055DP','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',102.31,847,889,98.95,'Sun-Synch' from dual union all
select 41211,'041211','2000-055DQ','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',102.33,846,891,98.94,'Sun-Synch' from dual union all
select 41212,'041212','2000-055DR','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',101.87,831,864,98.97,'Sun-Synch' from dual union all
select 41213,'041213','2000-055DS','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',101.88,823,873,99,'Sun-Synch' from dual union all
select 41214,'041214','2000-055DT','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',101.53,807,856,98.88,'Sun-Synch' from dual union all
select 41215,'041215','2000-055DU','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',101.53,810,853,98.88,'Sun-Synch' from dual union all
select 41216,'041216','2000-055DV','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',103.75,853,1019,98.92,'Sun-Synch' from dual union all
select 41217,'041217','2000-055DW','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',102.61,846,919,98.91,'Sun-Synch' from dual union all
select 41218,'041218','2000-055DX','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',101.98,808,897,98.92,'Sun-Synch' from dual union all
select 41219,'041219','2000-055DY','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',102.02,849,860,98.87,'Sun-Synch' from dual union all
select 4122,'004122','1969-085A','Soyuz-6','Soyuz No. 14',18352,'Landed','1969-10-16 00:00:00','1969-10-13 00:00:00',88.78,200,229,51.68,'Intermediate' from dual union all
select 41220,'041220','2000-055DZ','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',102.27,853,880,98.92,'Sun-Synch' from dual union all
select 41221,'041221','2000-055EA','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',101.11,767,855,98.93,'Sun-Synch' from dual union all
select 41222,'041222','2000-055EB','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',102.99,853,948,98.87,'Sun-Synch' from dual union all
select 41223,'041223','2000-055EC','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',102.57,853,907,98.9,'Sun-Synch' from dual union all
select 41224,'041224','2000-055ED','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',102.48,846,906,98.95,'Sun-Synch' from dual union all
select 41225,'041225','2000-055EE','deb NOAA 16','',66957,'In Earth orbit','','2015-12-27 00:00:00',101.7,808,871,98.94,'Sun-Synch' from dual union all
select 41226,'041226','2000-055EF','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',102.48,850,902,98.95,'Sun-Synch' from dual union all
select 41227,'041227','2000-055EG','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',101.52,805,857,99.05,'Sun-Synch' from dual;
insert into SATELLITE
select 41228,'041228','2000-055EH','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',102.14,842,877,98.93,'Sun-Synch' from dual union all
select 41229,'041229','2000-055EJ','deb NOAA 16','',66957,'In Earth orbit','','2015-12-28 00:00:00',102.9,832,960,98.9,'Sun-Synch' from dual union all
select 4123,'004123','1969-085B','11S510','11S510',18352,'Reentered','1969-10-12 00:00:00','1969-10-11 00:00:00',88.2,156,215,51.67,'Intermediate' from dual union all
select 41230,'041230','2000-055EK','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',101.38,794,854,98.97,'Sun-Synch' from dual union all
select 41231,'041231','2000-055EL','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',101.04,762,854,98.93,'Sun-Synch' from dual union all
select 41232,'041232','2000-055EM','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',102.8,849,933,98.96,'Sun-Synch' from dual union all
select 41233,'041233','2000-055EN','deb NOAA 16','',66957,'In Earth orbit','','2015-12-29 00:00:00',102.44,840,909,98.91,'Sun-Synch' from dual union all
select 41234,'041234','2000-055EP','deb NOAA 16','',66957,'In Earth orbit','','2015-12-28 00:00:00',100.48,707,856,98.87,'Sun-Synch' from dual union all
select 41235,'041235','2000-055EQ','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',102.01,847,860,98.92,'Sun-Synch' from dual union all
select 41236,'041236','2000-055ER','deb NOAA 16','',66957,'In Earth orbit','','2015-12-30 00:00:00',101.49,797,861,99.05,'Sun-Synch' from dual union all
select 41237,'041237','2000-055ES','deb NOAA 16','',66957,'In Earth orbit','','2015-12-27 00:00:00',101.26,780,857,98.98,'Sun-Synch' from dual union all
select 41238,'041238','2016-001A','BelinterSat-1','BelinterSat-1',70179,'In Earth orbit','','2016-01-28 00:00:00',1436.09,35779,35794,.06,'Stationary' from dual union all
select 41239,'041239','2016-001B','CZ-3B Stage 3','CZ-3B Stage 3',70179,'Reentered','2016-07-04 00:00:00','2016-01-22 00:00:00',712.03,153,39918,26.36,'GEO Transfer' from dual union all
select 4124,'004124','1969-086A','Soyuz-7','Soyuz No. 15',18353,'Landed','1969-10-17 00:00:00','1969-10-14 00:00:00',88.82,210,223,51.66,'Intermediate' from dual union all
select 41240,'041240','2016-002A','Jason-3','Jason-3',70180,'In Earth orbit','','2016-01-20 00:00:00',111.84,1301,1322,66.05,'Intermediate' from dual union all
select 41241,'041241','2016-003A','IRNSS-R1E','IRNSS-1E',70181,'In Earth orbit','','2016-01-27 00:00:00',1427.16,35324,35899,28.04,'Near-synch' from dual union all
select 41242,'041242','2016-003B','PSLV-C31 PS4','PS4',70181,'In Earth orbit','','2016-01-27 00:00:00',366.92,287,20912,19.08,'Highly Ellip' from dual union all
select 41243,'041243','2000-055ET','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.05,852,860,98.92,'Sun-Synch' from dual union all
select 41244,'041244','2000-055EU','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',101.92,839,861,99.02,'Sun-Synch' from dual union all
select 41245,'041245','2000-055EV','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.24,852,878,98.92,'Sun-Synch' from dual union all
select 41246,'041246','2000-055EW','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',101.04,758,858,98.96,'Sun-Synch' from dual union all
select 41247,'041247','2000-055EX','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',101.4,774,876,98.91,'Sun-Synch' from dual union all
select 41248,'041248','2000-055EY','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',103.48,852,995,98.91,'Sun-Synch' from dual union all
select 41249,'041249','2000-055EZ','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',101.26,784,853,98.94,'Sun-Synch' from dual union all
select 4125,'004125','1969-086B','11S510','11S510',18353,'Reentered','1969-10-14 00:00:00','1969-10-12 00:00:00',88.6,201,210,51.66,'Intermediate' from dual union all
select 41250,'041250','2000-055FA','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.41,832,914,99.06,'Sun-Synch' from dual union all
select 41251,'041251','2000-055FB','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.49,830,923,99.02,'Sun-Synch' from dual union all
select 41252,'041252','2000-055FC','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.94,853,943,98.97,'Sun-Synch' from dual union all
select 41253,'041253','2000-055FD','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',101.1,764,857,99,'Sun-Synch' from dual union all
select 41254,'041254','2000-055FE','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',101.58,808,859,99.08,'Sun-Synch' from dual union all
select 41255,'041255','2000-055FF','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',101.27,785,853,98.83,'Sun-Synch' from dual union all
select 41256,'041256','2000-055FG','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',101.67,821,855,99.09,'Sun-Synch' from dual union all
select 41257,'041257','2000-055FH','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.05,835,877,98.98,'Sun-Synch' from dual union all
select 41258,'041258','2000-055FJ','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',101.16,770,858,98.98,'Sun-Synch' from dual union all
select 41259,'041259','2000-055FK','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',100.69,715,868,98.85,'Sun-Synch' from dual union all
select 4126,'004126','1969-087A','Soyuz-8','Soyuz No. 16',18354,'Landed','1969-10-18 00:00:00','1969-10-15 00:00:00',88.77,201,227,51.65,'Intermediate' from dual union all
select 41260,'041260','2000-055FL','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.32,849,888,98.99,'Sun-Synch' from dual union all
select 41261,'041261','2000-055FM','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.61,850,915,98.89,'Sun-Synch' from dual union all
select 41262,'041262','2000-055FN','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',103.06,853,954,98.87,'Sun-Synch' from dual union all
select 41263,'041263','2000-055FP','deb NOAA 16','',66957,'In Earth orbit','','2016-01-24 00:00:00',103.38,853,984,98.93,'Sun-Synch' from dual union all
select 41264,'041264','2000-055FQ','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.04,852,858,98.88,'Sun-Synch' from dual union all
select 41265,'041265','2000-055FR','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.55,852,907,98.82,'Sun-Synch' from dual union all
select 41266,'041266','2000-055FS','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.04,852,858,98.91,'Sun-Synch' from dual union all
select 41267,'041267','2000-055FT','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',101.19,772,858,98.83,'Sun-Synch' from dual union all
select 41268,'041268','2000-055FU','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.5,851,903,99.02,'Sun-Synch' from dual union all
select 41269,'041269','2000-055FV','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.49,851,902,99,'Sun-Synch' from dual union all
select 4127,'004127','1969-087B','11S510','11S510',18354,'Reentered','1969-10-15 00:00:00','1969-10-14 00:00:00',88.44,190,205,51.66,'Intermediate' from dual union all
select 41270,'041270','2000-055FW','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.5,850,904,99.04,'Sun-Synch' from dual union all
select 41271,'041271','2000-055FX','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',100.91,752,852,98.82,'Sun-Synch' from dual union all
select 41272,'041272','2000-055FY','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.39,848,895,98.94,'Sun-Synch' from dual union all
select 41273,'041273','2000-055FZ','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',100.97,758,852,98.91,'Sun-Synch' from dual union all
select 41274,'041274','2000-055GA','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',101.16,767,861,98.84,'Sun-Synch' from dual union all
select 41275,'041275','2000-055GB','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.19,843,882,98.88,'Sun-Synch' from dual union all
select 41276,'041276','2000-055GC','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.19,850,875,98.9,'Sun-Synch' from dual union all
select 41277,'041277','2000-055GD','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',101.44,793,861,98.95,'Sun-Synch' from dual union all
select 41278,'041278','2000-055GE','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',101.36,794,852,98.87,'Sun-Synch' from dual union all
select 41279,'041279','2000-055GF','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.56,852,908,99,'Sun-Synch' from dual union all
select 4128,'004128','1969-088A','Interkosmos-1','DS-U3-IK No. 1',18355,'Reentered','1970-01-02 00:00:00','1969-11-14 00:00:00',92.33,250,528,48.39,'Intermediate' from dual union all
select 41280,'041280','2000-055GG','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.56,854,906,98.83,'Sun-Synch' from dual union all
select 41281,'041281','2000-055GH','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.62,853,912,98.93,'Sun-Synch' from dual union all
select 41282,'041282','2000-055GJ','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',103.12,853,959,98.9,'Sun-Synch' from dual union all
select 41283,'041283','2000-055GK','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',103.12,855,958,98.89,'Sun-Synch' from dual union all
select 41284,'041284','2000-055GL','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',103.47,851,994,98.88,'Sun-Synch' from dual union all
select 41285,'041285','2000-055GM','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.1,815,902,98.92,'Sun-Synch' from dual union all
select 41286,'041286','2000-055GN','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',101.47,806,851,98.92,'Sun-Synch' from dual union all
select 41287,'041287','2000-055GP','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.67,853,917,98.92,'Sun-Synch' from dual union all
select 41288,'041288','2000-055GQ','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.57,849,912,98.97,'Sun-Synch' from dual union all
select 41289,'041289','2000-055GR','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.59,851,911,98.91,'Sun-Synch' from dual union all
select 4129,'004129','1969-088B','S1M','S1M',18355,'Reentered','1969-12-16 00:00:00','1969-11-14 00:00:00',92.04,239,511,48.36,'Intermediate' from dual union all
select 41290,'041290','2000-055GS','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',101.64,817,855,99.01,'Sun-Synch' from dual union all
select 41291,'041291','2000-055GT','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',101.2,778,853,98.97,'Sun-Synch' from dual union all
select 41292,'041292','2000-055GU','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',101.26,782,855,98.95,'Sun-Synch' from dual union all
select 41293,'041293','2000-055GV','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.55,851,908,98.94,'Sun-Synch' from dual union all
select 41294,'041294','2000-055GW','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',101.72,823,858,98.87,'Sun-Synch' from dual union all
select 41295,'041295','2000-055GX','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.38,849,894,98.9,'Sun-Synch' from dual union all
select 41296,'041296','2000-055GY','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.41,850,896,98.97,'Sun-Synch' from dual union all
select 41297,'041297','2000-055GZ','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',103.09,850,959,99.01,'Sun-Synch' from dual union all
select 41298,'041298','2000-055HA','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.18,819,905,98.93,'Sun-Synch' from dual union all
select 41299,'041299','2000-055HB','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.35,832,908,98.94,'Sun-Synch' from dual union all
select 413,'000413','61 Omicron 133','deb r/b Ablestar','',7273,'Reentered','1989-12-02 00:00:00','1989-12-01 00:00:00',91.04,317,334,66.44,'Intermediate' from dual union all
select 4130,'004130','1969-089A','Kosmos-302','Zenit-4 No. 67',18356,'Landed','1969-10-25 00:00:00','1969-10-20 00:00:00',89.64,196,318,65.41,'Intermediate' from dual union all
select 41300,'041300','2000-055HC','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.68,850,920,98.95,'Sun-Synch' from dual union all
select 41301,'041301','2000-055HD','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.4,851,894,98.86,'Sun-Synch' from dual union all
select 41302,'041302','2000-055HE','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',102.34,849,889,98.93,'Sun-Synch' from dual union all
select 41303,'041303','2000-055HF','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',100.48,706,857,99,'Sun-Synch' from dual union all
select 41304,'041304','2000-055HG','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.4,834,911,99.08,'Sun-Synch' from dual union all
select 41305,'041305','2000-055HH','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',102.73,854,921,98.83,'Sun-Synch' from dual union all
select 41306,'041306','2000-055HJ','deb NOAA 16','',66957,'In Earth orbit','','2016-01-26 00:00:00',100.85,742,856,98.91,'Sun-Synch' from dual union all
select 41307,'041307','2000-055HK','deb NOAA 16','',66957,'In Earth orbit','','2016-01-25 00:00:00',101.92,834,866,98.96,'Sun-Synch' from dual union all
select 41308,'041308','2016-004A','Intelsat IS-29e','Intelsat IS-29e',70182,'In Earth orbit','','2016-01-30 00:00:00',627.72,267,35547,.52,'GEO Transfer' from dual union all
select 41309,'041309','2016-004B','ESC-A 583','Ariane 5 ESC-A',70182,'In Earth orbit','','2016-02-06 00:00:00',626.81,274,35492,.68,'GEO Transfer' from dual union all
select 4131,'004131','1969-089B','11A57I','11A57I',18356,'Reentered','1969-10-24 00:00:00','1969-10-19 00:00:00',89.23,176,297,65.39,'Intermediate' from dual union all
select 41310,'041310','2016-005A','EUTELSAT 9B','EUTELSAT 9B/EDRS-A',70183,'In Earth orbit','','2016-02-08 00:00:00',1430.5,35638,35716,.06,'Drift GEO' from dual union all
select 41311,'041311','2016-005B','Briz-M No. 99558','Briz-M 14S43 No. 99558',70183,'In Earth orbit','','2016-01-30 00:00:00',663.63,3755,33894,12.53,'GEO Transfer' from dual union all
select 41312,'041312','2016-005C','Briz-M DTB','Briz-M torus tank',70183,'In Earth orbit','','2016-02-01 00:00:00',346.24,347,19564,49.4,'Highly Ellip' from dual union all
select 41313,'041313','1998-067HP','AggieSat-4','AggieSat-4/LONESTAR 2',66503,'Reentered','2018-03-12 00:00:00','2016-01-30 00:00:00',92.63,399,408,51.64,'Intermediate' from dual union all
select 41314,'041314','1998-067HQ','Bevo-2','Bevo 2/LONESTAR 2',66503,'Reentered','2017-06-13 00:00:00','2016-01-30 00:00:00',92.63,399,408,51.64,'Intermediate' from dual union all
select 41315,'041315','2016-006A','Beidou DW 21','21 ke beidou daohang wei',70184,'In Earth orbit','','2016-02-03 00:00:00',782.93,21531,21991,55.01,'Medium' from dual union all
select 41316,'041316','2016-006B','Yuanzheng-1 Y3','YZ-1',70184,'In Earth orbit','','2016-02-03 00:00:00',899.15,21829,27133,55.18,'Medium' from dual union all
select 41317,'041317','2012-026D','deb Briz-M No. 99530','',69281,'In Earth orbit','','2016-02-01 00:00:00',803.22,10334,34156,12.1,'Medium' from dual;
insert into SATELLITE
select 41318,'041318','2012-026E','deb Briz-M No. 99530','',69281,'In Earth orbit','','2016-02-01 00:00:00',815.73,10488,34595,12.16,'Medium' from dual union all
select 41319,'041319','2012-026F','deb Briz-M No. 99530','',69281,'In Earth orbit','','2016-02-01 00:00:00',814.93,10501,34544,12.17,'Medium' from dual union all
select 4132,'004132','1969-082K','SOICAL CONE','S69-4 Cone',18349,'In Earth orbit','','1969-11-01 00:00:00',103.47,908,938,70,'Intermediate' from dual union all
select 41320,'041320','2012-026G','deb Briz-M No. 99530','',69281,'In Earth orbit','','2016-02-01 00:00:00',817.97,10478,34710,12.13,'Medium' from dual union all
select 41321,'041321','2012-026H','deb Briz-M No. 99530','',69281,'In Earth orbit','','2016-01-31 00:00:00',813.34,10445,34525,12.17,'Medium' from dual union all
select 41322,'041322','2012-026J','deb Briz-M No. 99530','',69281,'In Earth orbit','','2016-01-30 00:00:00',812.52,10468,34463,12.06,'Medium' from dual union all
select 41323,'041323','2012-026K','deb Briz-M No. 99530','',69281,'In Earth orbit','','2016-02-01 00:00:00',809.98,10373,34438,12.04,'Medium' from dual union all
select 41324,'041324','2012-026L','deb Briz-M No. 99530','',69281,'In Earth orbit','','2016-01-31 00:00:00',801.01,10301,34084,12.15,'Medium' from dual union all
select 41325,'041325','2016-006C','CZ-3C Stage 3','CZ-3C Stage 3',70184,'In Earth orbit','','2016-02-03 00:00:00',326.17,219,18416,55.01,'Highly Ellip' from dual union all
select 41326,'041326','1998-067HR','EVA debris (VKD-42)','',66503,'Reentered','2016-03-10 00:00:00','2016-02-03 00:00:00',92.61,398,407,51.65,'Intermediate' from dual union all
select 41327,'041327','1998-067HS','Fleshka "SMS na MKS"','',66503,'Reentered','2016-03-25 00:00:00','2016-02-03 00:00:00',92.6,396,408,51.64,'Intermediate' from dual union all
select 41328,'041328','2016-007A','GPS IIF-12','GPS SVN 70',70185,'In Earth orbit','','2016-02-09 00:00:00',728.78,20440,20456,55.04,'Medium' from dual union all
select 41329,'041329','2016-007B','Centaur AV-057','Centaur AV-057',70185,'In Earth orbit','','2016-02-10 00:00:00',750.73,20493,21475,55.33,'Medium' from dual union all
select 4133,'004133','1969-082CP','deb Agena D','',18349,'In Earth orbit','','1992-02-03 00:00:00',100.23,719,820,69.93,'Intermediate' from dual union all
select 41330,'041330','2016-008A','Kosmos-2514','Uragan-M No. 751',70186,'In Earth orbit','','2016-02-14 00:00:00',678.26,19156,19231,64.8,'Medium' from dual union all
select 41331,'041331','2016-008B','Fregat-M No. 112-03','Fregat 14S44 No. 112-03',70186,'In Earth orbit','','2016-02-16 00:00:00',687.73,19182,19680,64.76,'Medium' from dual union all
select 41332,'041332','2016-009A','Kwangmyongsong-4-ho','KMS-4',70187,'In Earth orbit','','2016-02-09 00:00:00',94.26,465,500,97.53,'Sun-Synch' from dual union all
select 41333,'041333','2016-009B','Kwangmyongsong Stage 3','Unha-3 Stage 3',70187,'In Earth orbit','','2016-02-09 00:00:00',93.92,432,501,97.54,'Sun-Synch' from dual union all
select 41334,'041334','2016-010A','USA 267','TOPAZ 4',70188,'In Earth orbit','','2016-02-12 00:00:00',106.85,1077,1084,122.97,'Retrograde' from dual union all
select 41335,'041335','2016-011A','Sentinel-3A','Sentinel-3A',70189,'In Earth orbit','','2016-02-18 00:00:00',100.95,802,806,98.62,'Sun-Synch' from dual union all
select 41336,'041336','2016-011B','Briz-KM No. 72525','Briz-KM 14S45',70189,'In Earth orbit','','2016-02-18 00:00:00',96.2,410,744,98.69,'Sun-Synch' from dual union all
select 41337,'041337','2016-012A','Hitomi','ASTRO-H',70190,'In Earth orbit','','2016-02-29 00:00:00',96.1,564,579,31.01,'Intermediate' from dual union all
select 41338,'041338','2016-012B','Kinshachi 2','ChubuSat-2',70190,'In Earth orbit','','2016-02-29 00:00:00',96.03,559,578,31,'Intermediate' from dual union all
select 41339,'041339','2016-012C','Kinshachi 3','ChubuSat-3',70190,'In Earth orbit','','2016-02-29 00:00:00',96.03,559,578,31.01,'Intermediate' from dual union all
select 4134,'004134','1969-082CW','deb Agena D','',18349,'Reentered','2000-03-14 00:00:00','1992-02-05 00:00:00',97.72,613,687,69.94,'Intermediate' from dual union all
select 41340,'041340','2016-012D','Horyu-4','Horyu-4',70190,'In Earth orbit','','2016-02-29 00:00:00',96,557,577,31.01,'Intermediate' from dual union all
select 41341,'041341','2016-012E','H-2A F30 Stage 2','H-2A Stage 2',70190,'In Earth orbit','','2016-02-29 00:00:00',95.76,543,568,30.63,'Intermediate' from dual union all
select 41342,'041342','2016-012F','deb H-2A','',70190,'Reentered','2017-01-29 00:00:00','2016-02-28 00:00:00',96.03,561,576,31.01,'Intermediate' from dual union all
select 41343,'041343','2000-055HL','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',101.83,825,865,98.81,'Sun-Synch' from dual union all
select 41344,'041344','2000-055HM','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',101.54,806,858,99.04,'Sun-Synch' from dual union all
select 41345,'041345','2000-055HN','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',100.74,734,854,98.86,'Sun-Synch' from dual union all
select 41346,'041346','2000-055HP','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',104.26,838,1082,98.91,'Sun-Synch' from dual union all
select 41347,'041347','2000-055HQ','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',102.12,842,876,98.88,'Sun-Synch' from dual union all
select 41348,'041348','2000-055HR','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',102.03,824,886,98.85,'Sun-Synch' from dual union all
select 41349,'041349','2000-055HS','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',101.36,791,856,98.94,'Sun-Synch' from dual union all
select 4135,'004135','1969-082DP','deb Agena D','',18349,'In Earth orbit','','1992-02-03 00:00:00',101.69,799,879,69.98,'Intermediate' from dual union all
select 41350,'041350','2000-055HT','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',101.7,817,862,99.01,'Sun-Synch' from dual union all
select 41351,'041351','2000-055HU','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',102.19,853,872,98.85,'Sun-Synch' from dual union all
select 41352,'041352','2000-055HV','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',101.32,783,860,98.91,'Sun-Synch' from dual union all
select 41353,'041353','2000-055HW','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',102.21,844,883,98.93,'Sun-Synch' from dual union all
select 41354,'041354','2000-055HX','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',102.21,847,879,98.92,'Sun-Synch' from dual union all
select 41355,'041355','2000-055HY','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',101.98,844,861,98.88,'Sun-Synch' from dual union all
select 41356,'041356','2000-055HZ','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',102.46,843,908,98.92,'Sun-Synch' from dual union all
select 41357,'041357','2000-055JA','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',103.33,854,979,98.96,'Sun-Synch' from dual union all
select 41358,'041358','2000-055JB','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',101.98,839,866,98.88,'Sun-Synch' from dual union all
select 41359,'041359','2000-055JC','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',101.42,797,855,98.91,'Sun-Synch' from dual union all
select 4136,'004136','1969-090A','Kosmos-303','DS-P1-Yu No. 28',18357,'Reentered','1970-01-23 00:00:00','1969-11-17 00:00:00',91.39,259,427,71,'Intermediate' from dual union all
select 41360,'041360','2000-055JD','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',102,833,874,98.97,'Sun-Synch' from dual union all
select 41361,'041361','2000-055JE','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',102.54,851,907,98.91,'Sun-Synch' from dual union all
select 41362,'041362','2000-055JF','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',101.26,780,857,98.92,'Sun-Synch' from dual union all
select 41363,'041363','2000-055JG','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',101.58,812,855,98.92,'Sun-Synch' from dual union all
select 41364,'041364','2000-055JH','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',102.06,843,870,98.85,'Sun-Synch' from dual union all
select 41365,'041365','2000-055JJ','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',101.49,795,864,98.91,'Sun-Synch' from dual union all
select 41366,'041366','2000-055JK','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',102.69,853,919,98.99,'Sun-Synch' from dual union all
select 41367,'041367','2000-055JL','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',101.49,789,870,98.97,'Sun-Synch' from dual union all
select 41368,'041368','2000-055JM','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',101.86,822,872,99.01,'Sun-Synch' from dual union all
select 41369,'041369','2000-055JN','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',102,832,875,98.85,'Sun-Synch' from dual union all
select 4137,'004137','1969-090B','S1M','S1M',18357,'Reentered','1969-12-07 00:00:00','1969-10-25 00:00:00',91.59,267,438,71,'Intermediate' from dual union all
select 41370,'041370','2000-055JP','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',100.65,724,855,98.9,'Sun-Synch' from dual union all
select 41371,'041371','2000-055JQ','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',101.88,829,867,98.92,'Sun-Synch' from dual union all
select 41372,'041372','2000-055JR','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',100.81,737,857,98.98,'Sun-Synch' from dual union all
select 41373,'041373','2000-055JS','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',101.59,812,856,98.92,'Sun-Synch' from dual union all
select 41374,'041374','2000-055JT','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',102.45,850,900,98.95,'Sun-Synch' from dual union all
select 41375,'041375','2000-055JU','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',101.79,822,865,98.96,'Sun-Synch' from dual union all
select 41376,'041376','2000-055JV','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',101.85,831,861,98.91,'Sun-Synch' from dual union all
select 41377,'041377','2000-055JW','deb NOAA 16','',66957,'In Earth orbit','','2016-03-02 00:00:00',101.99,835,870,98.96,'Sun-Synch' from dual union all
select 41378,'041378','2000-055JX','deb NOAA 16','',66957,'In Earth orbit','','2016-03-01 00:00:00',102.46,843,908,98.88,'Sun-Synch' from dual union all
select 41379,'041379','2000-055JY','deb NOAA 16','',66957,'In Earth orbit','','2016-02-29 00:00:00',101.53,806,857,98.88,'Sun-Synch' from dual union all
select 4138,'004138','1969-091A','Kosmos-304','Tsiklon',18358,'In Earth orbit','','1969-12-14 00:00:00',99.82,740,760,74.03,'Intermediate' from dual union all
select 41380,'041380','2016-013A','SES-9','SES-9',70191,'In Earth orbit','','2016-03-14 00:00:00',1405.59,28570,41804,.44,'Near-synch' from dual union all
select 41381,'041381','2016-013B','Falcon 9-022 Stage 2','F9-022 St2',70191,'In Earth orbit','','2016-03-12 00:00:00',729.49,325,40606,27.93,'GEO Transfer' from dual union all
select 41382,'041382','2016-014A','EUTELSAT 65 West A','EUTELSAT 65 West A',70192,'In Earth orbit','','2016-03-16 00:00:00',1433.94,35729,35760,.03,'Drift GEO' from dual union all
select 41383,'041383','2016-014B','ESC-A 582','Ariane 5 ESC-A',70192,'In Earth orbit','','2016-03-17 00:00:00',628.63,219,35642,.47,'GEO Transfer' from dual union all
select 41384,'041384','2016-015A','IRNSS-R1F','IRNSS-1F',70193,'In Earth orbit','','2016-03-20 00:00:00',1433.25,35628,35834,5.09,'Inclined Drift' from dual union all
select 41385,'041385','2016-015B','PSLV-C32 PS4','PS4',70193,'In Earth orbit','','2016-03-17 00:00:00',368.01,274,20992,17.74,'Highly Ellip' from dual union all
select 41386,'041386','2016-016A','Resurs-P No. 3','Resurs-P No. 3',70194,'In Earth orbit','','2016-03-16 00:00:00',91.85,281,449,97.28,'Sun-Synch' from dual union all
select 41387,'041387','2016-016B','Blok-I 14S54','14S54',70194,'Reentered','2016-09-24 00:00:00','2016-03-16 00:00:00',91.75,280,441,97.32,'Sun-Synch' from dual union all
select 41388,'041388','2016-017A','ExoMars Trace Gas Orbiter','ExoMars TGO',70195,'Deep Space','2016-03-15 00:00:00','2016-03-14 00:00:00',816.21,749,71368,51.51,'Earth Escape' from dual union all
select 41389,'041389','2016-017B','Briz-M No. 99560','Briz-M 14S43 No. 99560',70195,'Deep Space','2016-03-15 00:00:00','2016-03-14 00:00:00',834.87,944,72441,51.24,'Earth Escape' from dual union all
select 4139,'004139','1969-091B','S3M','S3M',18358,'In Earth orbit','','1969-11-20 00:00:00',99.68,730,757,74.05,'Intermediate' from dual union all
select 41390,'041390','2016-017C','Briz-M DTB','Briz-M torus tank',70195,'In Earth orbit','','2016-03-21 00:00:00',376.38,695,21085,51.59,'Highly Ellip' from dual union all
select 41391,'041391','2016-018A','Soyuz TMA-20M','Soyuz 11F732A47 No. 720',70196,'Landed','2016-09-07 00:00:00','2016-03-19 00:00:00',89.66,202,314,51.67,'Intermediate' from dual union all
select 41392,'041392','2016-018B','11S510','11S510',70196,'Reentered','2016-03-21 00:00:00','2016-03-20 00:00:00',88.21,173,199,51.61,'Intermediate' from dual union all
select 41393,'041393','2016-019A','SS Rick Husband','Cygnus PCM-6',70197,'Deorbited','2016-06-22 00:00:00','2016-03-24 00:00:00',89.33,234,249,51.65,'Intermediate' from dual union all
select 41394,'041394','2016-020A','Kosmos-2515','Bars-M No. 2L',70198,'In Earth orbit','','2015-03-01 00:00:00',93.24,327,540,97.63,'Sun-Synch' from dual union all
select 41395,'041395','2016-020B','Blok-I 14S54/RD-0110','374BL02A',70198,'In Earth orbit','','2015-03-01 00:00:00',93.19,326,535,97.64,'Sun-Synch' from dual union all
select 41396,'041396','2000-055JZ','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',100.08,668,857,98.86,'Sun-Synch' from dual union all
select 41397,'041397','2000-055KA','deb NOAA 16','',66957,'In Earth orbit','','2016-03-24 00:00:00',101.42,791,861,98.91,'Sun-Synch' from dual union all
select 41398,'041398','2000-055KB','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.75,824,859,98.91,'Sun-Synch' from dual union all
select 41399,'041399','2000-055KC','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',102.62,850,916,98.94,'Sun-Synch' from dual union all
select 414,'000414','61 Omicron 134','deb r/b Ablestar','',7273,'In Earth orbit','','1962-09-20 00:00:00',104.88,957,1021,66.46,'Intermediate' from dual union all
select 4140,'004140','1969-082AL','deb Agena D','',18349,'Reentered','2014-04-13 00:00:00','1992-01-31 00:00:00',99.82,707,793,70.11,'Intermediate' from dual union all
select 41400,'041400','2000-055KD','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.81,827,861,98.89,'Sun-Synch' from dual union all
select 41401,'041401','2000-055KE','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',102.34,854,885,98.87,'Sun-Synch' from dual union all
select 41402,'041402','2000-055KF','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',102.53,854,903,98.91,'Sun-Synch' from dual union all
select 41403,'041403','2000-055KG','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.57,809,857,98.92,'Sun-Synch' from dual union all
select 41404,'041404','2000-055KH','deb NOAA 16','',66957,'In Earth orbit','','2016-03-23 00:00:00',101.47,796,861,98.99,'Sun-Synch' from dual union all
select 41405,'041405','2000-055KJ','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',102.13,846,873,98.92,'Sun-Synch' from dual union all
select 41406,'041406','2000-055KK','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.3,786,855,98.92,'Sun-Synch' from dual union all
select 41407,'041407','2000-055KL','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.46,796,859,98.96,'Sun-Synch' from dual;
insert into SATELLITE
select 41408,'041408','2000-055KM','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.83,833,858,98.9,'Sun-Synch' from dual union all
select 41409,'041409','2000-055KN','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.76,812,872,98.83,'Sun-Synch' from dual union all
select 4141,'004141','1969-082AJ','deb Agena D','',18349,'In Earth orbit','','1992-02-02 00:00:00',100.28,712,831,69.9,'Intermediate' from dual union all
select 41410,'041410','2000-055KP','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',102.42,857,889,98.92,'Sun-Synch' from dual union all
select 41411,'041411','2000-055KQ','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.7,823,855,98.91,'Sun-Synch' from dual union all
select 41412,'041412','2000-055KR','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',102.36,851,890,98.8,'Sun-Synch' from dual union all
select 41413,'041413','2000-055KS','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',102.16,845,877,98.89,'Sun-Synch' from dual union all
select 41414,'041414','2000-055KT','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.26,764,873,98.97,'Sun-Synch' from dual union all
select 41415,'041415','2000-055KU','deb NOAA 16','',66957,'In Earth orbit','','2016-03-24 00:00:00',101.88,841,855,98.9,'Sun-Synch' from dual union all
select 41416,'041416','2000-055KV','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.76,814,870,98.89,'Sun-Synch' from dual union all
select 41417,'041417','2000-055KW','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.99,835,870,98.92,'Sun-Synch' from dual union all
select 41418,'041418','2000-055KX','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.38,793,855,98.98,'Sun-Synch' from dual union all
select 41419,'041419','2000-055KY','deb NOAA 16','',66957,'In Earth orbit','','2016-03-23 00:00:00',102.21,851,876,98.91,'Sun-Synch' from dual union all
select 4142,'004142','1969-082L','deb Agena D','',18349,'Reentered','1980-11-03 00:00:00','1969-11-02 00:00:00',101.97,788,916,70.02,'Intermediate' from dual union all
select 41420,'041420','2000-055KZ','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.15,771,856,98.94,'Sun-Synch' from dual union all
select 41421,'041421','2000-055LA','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.35,781,864,98.93,'Sun-Synch' from dual union all
select 41422,'041422','2000-055LB','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',102.15,849,871,98.93,'Sun-Synch' from dual union all
select 41423,'041423','2000-055LC','deb NOAA 16','',66957,'In Earth orbit','','2016-03-23 00:00:00',101.77,829,855,98.87,'Sun-Synch' from dual union all
select 41424,'041424','2000-055LD','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.77,819,866,98.96,'Sun-Synch' from dual union all
select 41425,'041425','2000-055LE','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.13,770,855,98.95,'Sun-Synch' from dual union all
select 41426,'041426','2000-055LF','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',102.17,847,876,98.96,'Sun-Synch' from dual union all
select 41427,'041427','2000-055LG','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',102.54,852,906,98.97,'Sun-Synch' from dual union all
select 41428,'041428','2000-055LH','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.83,830,860,98.92,'Sun-Synch' from dual union all
select 41429,'041429','2000-055LJ','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.99,837,869,98.92,'Sun-Synch' from dual union all
select 4143,'004143','1969-082M','deb Agena D','',18349,'In Earth orbit','','1969-11-02 00:00:00',102.1,798,918,70.07,'Intermediate' from dual union all
select 41430,'041430','2000-055LK','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.41,787,863,98.95,'Sun-Synch' from dual union all
select 41431,'041431','2000-055LL','deb NOAA 16','',66957,'In Earth orbit','','2016-03-24 00:00:00',101.04,763,852,98.97,'Sun-Synch' from dual union all
select 41432,'041432','2000-055LM','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',100.45,696,864,98.86,'Sun-Synch' from dual union all
select 41433,'041433','2000-055LN','deb NOAA 16','',66957,'In Earth orbit','','2016-03-25 00:00:00',101.55,806,859,99.01,'Sun-Synch' from dual union all
select 41434,'041434','2016-021A','Beidou DW 22','ershier ke beidou daohan',70199,'In Earth orbit','','2016-04-04 00:00:00',1435.86,35675,35889,54.97,'Synchronous' from dual union all
select 41435,'041435','2016-021B','CZ-3A Stage 3','CZ-3A Stage 3',70199,'In Earth orbit','','2016-04-02 00:00:00',624.85,224,35442,55.01,'GEO Transfer' from dual union all
select 41436,'041436','2016-022A','Progress MS-02','Progress 7K-TGM No. 432',70200,'Deorbited','2016-10-14 00:00:00','2016-04-01 00:00:00',89.83,264,269,51.64,'Intermediate' from dual union all
select 41437,'041437','2016-022B','Blok-I 14S54/RD-0110','374BL02A',70200,'Reentered','2016-04-03 00:00:00','2016-04-01 00:00:00',88.27,177,201,51.65,'Intermediate' from dual union all
select 41438,'041438','2016-012G','deb ASTRO-H solar panel','High drag debris',70190,'Reentered','2016-04-24 00:00:00','2016-04-01 00:00:00',95.82,552,565,31,'Intermediate' from dual union all
select 41439,'041439','2016-012H','deb ASTRO-H','',70190,'In Earth orbit','','2016-04-01 00:00:00',96.05,560,579,31.01,'Intermediate' from dual union all
select 4144,'004144','1969-082N','deb Agena D','',18349,'In Earth orbit','','1970-08-14 00:00:00',102.06,797,916,70.08,'Intermediate' from dual union all
select 41440,'041440','2016-012J','deb ASTRO-H','',70190,'In Earth orbit','','2016-04-01 00:00:00',95.97,554,577,31.02,'Intermediate' from dual union all
select 41441,'041441','2016-012K','deb ASTRO-H','',70190,'In Earth orbit','','2016-04-01 00:00:00',95.58,528,565,30.99,'Intermediate' from dual union all
select 41442,'041442','2016-012L','deb ASTRO-H EOB','Large debris',70190,'In Earth orbit','','2016-03-26 00:00:00',96.06,560,579,31.02,'Intermediate' from dual union all
select 41443,'041443','2016-012M','deb ASTRO-H solar panel','High drag debris',70190,'Reentered','2016-04-20 00:00:00','2016-04-01 00:00:00',95.63,544,555,31.01,'Intermediate' from dual union all
select 41444,'041444','2016-012N','deb ASTRO-H','',70190,'In Earth orbit','','2016-04-01 00:00:00',96.11,564,581,31.01,'Intermediate' from dual union all
select 41445,'041445','2016-012P','deb ASTRO-H','',70190,'In Earth orbit','','2016-04-01 00:00:00',96.09,563,580,31.01,'Intermediate' from dual union all
select 41446,'041446','2016-012Q','deb ASTRO-H','',70190,'Reentered','2017-11-04 00:00:00','2016-04-01 00:00:00',96.1,566,578,31.01,'Intermediate' from dual union all
select 41447,'041447','2016-012R','deb ASTRO-H','',70190,'In Earth orbit','','2016-04-01 00:00:00',96.12,566,580,31.01,'Intermediate' from dual union all
select 41448,'041448','2016-023A','Shi Jian 10 RV','SJ-10',70201,'Landed','2016-04-18 00:00:00','2016-04-07 00:00:00',89.5,233,267,42.89,'Intermediate' from dual union all
select 41449,'041449','2016-023B','CZ-2D Stage 2','CZ-2D Stage 2',70201,'Reentered','2016-04-18 00:00:00','2016-04-07 00:00:00',89.44,232,262,42.89,'Intermediate' from dual union all
select 4145,'004145','1969-082P','deb Agena D','',18349,'In Earth orbit','','1969-11-03 00:00:00',102.39,825,919,70.06,'Intermediate' from dual union all
select 41450,'041450','2016-023C','CZ-2D Stage 2 motor cover','',70201,'Reentered','2016-04-07 00:00:00','2016-04-06 00:00:00',89.19,222,247,42.87,'Intermediate' from dual union all
select 41451,'041451','2016-023D','CZ-2D Stage 2 motor cover','',70201,'Reentered','2016-04-06 00:00:00','2016-04-06 00:00:00',88.86,209,228,42.9,'Intermediate' from dual union all
select 41452,'041452','2016-024A','Dragon CRS-8','Dragon C110',70202,'Landed','2016-05-11 00:00:00','2016-04-08 00:00:00',90.13,209,353,51.66,'Intermediate' from dual union all
select 41453,'041453','2016-024B','Dragon solar array cover','',70202,'Reentered','2016-04-13 00:00:00','2016-04-08 00:00:00',90.22,212,358,51.67,'Intermediate' from dual union all
select 41454,'041454','2016-024C','Dragon solar array cover','',70202,'Reentered','2016-04-13 00:00:00','2016-04-08 00:00:00',90.15,211,353,51.64,'Intermediate' from dual union all
select 41455,'041455','2016-023E','Shi Jian 10','SJ-10',70201,'Reentered','2016-07-01 00:00:00','2016-04-07 00:00:00',89.5,233,267,42.89,'Intermediate' from dual union all
select 41456,'041456','2016-025A','Sentinel-1B','Sentinel-1B',70203,'In Earth orbit','','2016-04-28 00:00:00',98.71,694,700,98.19,'Sun-Synch' from dual union all
select 41457,'041457','2016-025B','Microscope','Microscope',70203,'In Earth orbit','','2016-04-28 00:00:00',99.03,712,713,98.24,'Sun-Synch' from dual union all
select 41458,'041458','2016-025C','OUFTI 1','OUFTI 1',70203,'In Earth orbit','','2016-04-27 00:00:00',95.94,442,686,98.22,'Sun-Synch' from dual union all
select 41459,'041459','2016-025D','E-st@r-II','E-STAR-II',70203,'In Earth orbit','','2016-04-27 00:00:00',95.94,442,686,98.22,'Sun-Synch' from dual union all
select 4146,'004146','1969-082Q','deb Agena D','',18349,'Reentered','1988-12-18 00:00:00','1969-11-09 00:00:00',101.98,761,944,70.33,'Intermediate' from dual union all
select 41460,'041460','2016-025E','AAUSAT-4','AAUSAT4',70203,'In Earth orbit','','2016-04-27 00:00:00',95.94,442,686,98.22,'Sun-Synch' from dual union all
select 41461,'041461','2016-025F','ASAP-S','ASAP-S payload adapter',70203,'In Earth orbit','','2016-04-28 00:00:00',95.98,441,691,98.22,'Sun-Synch' from dual union all
select 41462,'041462','2016-023F','CZ-2D Stage 2 motor cover','',70201,'Reentered','2016-04-30 00:00:00','2016-04-26 00:00:00',92.2,202,563,42.91,'Intermediate' from dual union all
select 41463,'041463','1998-067HT','Diwata-1','Diwata-1',66503,'In Earth orbit','','2016-04-27 00:00:00',92.61,401,404,51.64,'Intermediate' from dual union all
select 41464,'041464','2016-026A','Mikhailo Lomonosov-300','MVL-300',70204,'In Earth orbit','','2016-04-30 00:00:00',94.17,471,486,97.27,'Sun-Synch' from dual union all
select 41465,'041465','2016-026B','AIST 2D','AIST 2D',70204,'In Earth orbit','','2016-04-30 00:00:00',94.16,471,485,97.28,'Sun-Synch' from dual union all
select 41466,'041466','2016-026C','Samsat-218/D','Kontakt-Nanosputnik',70204,'In Earth orbit','','2016-04-30 00:00:00',94.15,471,483,97.28,'Sun-Synch' from dual union all
select 41467,'041467','2016-026D','Blok-I 14S54/RD-0110','Blok-I 14S54/RD-0110',70204,'Reentered','2016-04-30 00:00:00','2016-04-29 00:00:00',87.99,168,182,97.3,'Sun-Synch' from dual union all
select 41468,'041468','2016-026E','Volga 14S46 No. 6','Volga R15000-006',70204,'Deorbited','2016-04-28 00:00:00','2016-04-28 00:00:00',94.15,470,484,97.27,'Sun-Synch' from dual union all
select 41469,'041469','2016-027A','IRNSS-R1G','IRNSS-1G',70205,'In Earth orbit','','2016-05-06 00:00:00',1428.3,35435,35833,5.08,'Inclined Drift' from dual union all
select 4147,'004147','1969-082R','deb Agena D','',18349,'Reentered','1981-10-16 00:00:00','1969-10-31 00:00:00',102.51,836,919,70.09,'Intermediate' from dual union all
select 41470,'041470','2016-027B','PSLV-C33 PS4','PS4',70205,'In Earth orbit','','2016-05-06 00:00:00',367.1,269,20941,17.73,'Highly Ellip' from dual union all
select 41471,'041471','2016-028A','JCSAT 2B','JCSAT 14',70206,'In Earth orbit','','2016-05-13 00:00:00',1437.34,35758,35864,.03,'Drift GEO' from dual union all
select 41472,'041472','2016-028B','Falcon 9-024 Stage 2','F9-024 St2',70206,'Reentered','2017-09-16 00:00:00','2016-05-13 00:00:00',631.37,177,35825,23.68,'GEO Transfer' from dual union all
select 41473,'041473','2016-029A','Yaogan Weixing 30','Jianbing-6 06 xing',70207,'In Earth orbit','','2016-05-17 00:00:00',97.56,628,656,98.07,'Sun-Synch' from dual union all
select 41474,'041474','1998-067HU','MinXSS','MinXSS',66503,'Reentered','2017-05-06 00:00:00','2016-05-17 00:00:00',92.57,397,404,51.64,'Intermediate' from dual union all
select 41475,'041475','1998-067HV','CADRE','CADRE',66503,'Reentered','2017-01-03 00:00:00','2016-05-17 00:00:00',92.58,400,402,51.64,'Intermediate' from dual union all
select 41476,'041476','1998-067HW','STMSat 1','STMSat 1',66503,'Reentered','2017-04-21 00:00:00','2016-05-18 00:00:00',92.57,398,403,51.64,'Intermediate' from dual union all
select 41477,'041477','1998-067HX','Nodes 1','Nodes 1',66503,'Reentered','2017-09-19 00:00:00','2016-05-18 00:00:00',92.57,399,402,51.64,'Intermediate' from dual union all
select 41478,'041478','1998-067HY','Nodes 2','Nodes 2',66503,'Reentered','2017-09-23 00:00:00','2016-05-18 00:00:00',92.57,399,402,51.64,'Intermediate' from dual union all
select 41479,'041479','1998-067HZ','Flock 2e''-1','Dove 0D05',66503,'Reentered','2018-04-15 00:00:00','2016-05-18 00:00:00',92.57,399,402,51.64,'Intermediate' from dual union all
select 4148,'004148','1969-082S','deb Agena D','',18349,'Reentered','1982-01-25 00:00:00','1969-10-30 00:00:00',102.45,831,919,69.91,'Intermediate' from dual union all
select 41480,'041480','1998-067JA','Flock 2e''-3','Dove 0D06',66503,'Reentered','2018-04-18 00:00:00','2016-05-18 00:00:00',92.58,400,402,51.64,'Intermediate' from dual union all
select 41481,'041481','1998-067JB','Flock 2e''-2','Dove 0C1B',66503,'Reentered','2017-12-01 00:00:00','2016-05-18 00:00:00',92.57,399,402,51.64,'Intermediate' from dual union all
select 41482,'041482','1998-067JC','Flock 2e''-4','Dove 0C22',66503,'Reentered','2017-11-08 00:00:00','2016-05-18 00:00:00',92.58,400,402,51.64,'Intermediate' from dual union all
select 41483,'041483','1998-067JD','Flock 2e-1','Dove 0C37',66503,'In Earth orbit','','2016-05-18 00:00:00',92.58,400,402,51.64,'Intermediate' from dual union all
select 41484,'041484','1998-067JE','Flock 2e-2','Dove 0C78',66503,'Reentered','2018-01-23 00:00:00','2016-05-18 00:00:00',92.59,400,403,51.64,'Intermediate' from dual union all
select 41485,'041485','1998-067JF','Lemur-2-Theresacondor','Lemur-2-05',66503,'Reentered','2017-03-30 00:00:00','2016-05-19 00:00:00',92.57,399,402,51.64,'Intermediate' from dual union all
select 41486,'041486','1998-067JG','Flock 2e-3','Dove 0C60',66503,'Reentered','2018-04-05 00:00:00','2016-05-18 00:00:00',92.58,399,403,51.64,'Intermediate' from dual union all
select 41487,'041487','1998-067JH','Flock 2e-4','Dove 0C41',66503,'Reentered','2018-03-11 00:00:00','2016-05-19 00:00:00',92.58,398,403,51.64,'Intermediate' from dual union all
select 41488,'041488','1998-067JJ','Lemur-2-Nick-Allain','Lemur-2-07',66503,'Reentered','2017-04-05 00:00:00','2016-05-23 00:00:00',92.57,399,402,51.64,'Intermediate' from dual union all
select 41489,'041489','1998-067JK','Lemur-2-Kane','Lemur-2-06',66503,'Reentered','2017-04-07 00:00:00','2016-05-23 00:00:00',92.56,399,400,51.64,'Intermediate' from dual union all
select 4149,'004149','1969-082T','deb Agena D','',18349,'In Earth orbit','','1969-10-31 00:00:00',102.93,863,932,70.32,'Intermediate' from dual union all
select 41490,'041490','1998-067JL','Lemur-2-Jeff','Lemur-2-08',66503,'Reentered','2017-03-24 00:00:00','2016-05-23 00:00:00',92.56,399,400,51.64,'Intermediate' from dual union all
select 41491,'041491','2000-055LP','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',103.45,839,1004,98.92,'Sun-Synch' from dual union all
select 41492,'041492','2000-055LQ','deb NOAA 16','',66957,'In Earth orbit','','2016-05-21 00:00:00',104.08,854,1049,98.84,'Sun-Synch' from dual union all
select 41493,'041493','2000-055LR','deb NOAA 16','',66957,'In Earth orbit','','2016-05-22 00:00:00',101.49,805,854,98.9,'Sun-Synch' from dual union all
select 41494,'041494','2000-055LS','deb NOAA 16','',66957,'In Earth orbit','','2016-05-22 00:00:00',101.41,801,850,98.92,'Sun-Synch' from dual union all
select 41495,'041495','2000-055LT','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',102.64,850,918,98.88,'Sun-Synch' from dual union all
select 41496,'041496','2000-055LU','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.71,826,854,98.89,'Sun-Synch' from dual union all
select 41497,'041497','2000-055LV','deb NOAA 16','',66957,'In Earth orbit','','2016-05-13 00:00:00',102.44,846,903,98.92,'Sun-Synch' from dual union all
select 41498,'041498','2000-055LW','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.76,828,856,98.95,'Sun-Synch' from dual;
insert into SATELLITE
select 41499,'041499','2000-055LX','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.86,814,880,98.83,'Sun-Synch' from dual union all
select 415,'000415','61 Omicron 135','deb r/b Ablestar','',7273,'Reentered','1989-05-27 00:00:00','1989-05-26 00:00:00',89.82,246,286,67.62,'Intermediate' from dual union all
select 4150,'004150','1969-092A','SOZ','SOZ',18359,'Reentered','1969-10-24 00:00:00','1969-10-23 00:00:00',88.29,170,210,51.51,'Intermediate' from dual union all
select 41500,'041500','2000-055LY','deb NOAA 16','',66957,'In Earth orbit','','2016-05-22 00:00:00',101.4,791,859,98.98,'Sun-Synch' from dual union all
select 41501,'041501','2000-055LZ','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.06,769,849,98.74,'Sun-Synch' from dual union all
select 41502,'041502','2000-055MA','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',102.18,851,873,98.89,'Sun-Synch' from dual union all
select 41503,'041503','2000-055MB','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.45,802,853,98.84,'Sun-Synch' from dual union all
select 41504,'041504','2000-055MC','deb NOAA 16','',66957,'In Earth orbit','','2016-05-22 00:00:00',102.38,843,900,98.93,'Sun-Synch' from dual union all
select 41505,'041505','2000-055MD','deb NOAA 16','',66957,'In Earth orbit','','2016-05-22 00:00:00',101.78,810,876,98.91,'Sun-Synch' from dual union all
select 41506,'041506','2000-055ME','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.09,771,849,98.81,'Sun-Synch' from dual union all
select 41507,'041507','2000-055MF','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.64,824,849,98.92,'Sun-Synch' from dual union all
select 41508,'041508','2000-055MG','deb NOAA 16','',66957,'In Earth orbit','','2016-05-21 00:00:00',101.78,819,867,98.93,'Sun-Synch' from dual union all
select 41509,'041509','2000-055MH','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.95,834,868,98.93,'Sun-Synch' from dual union all
select 4151,'004151','1969-092B','SOZ','SOZ',18359,'Reentered','1969-10-24 00:00:00','1969-10-23 00:00:00',88.23,171,203,51.5,'Intermediate' from dual union all
select 41510,'041510','2000-055MJ','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',102.03,832,878,98.94,'Sun-Synch' from dual union all
select 41511,'041511','2000-055MK','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',102.28,844,889,98.93,'Sun-Synch' from dual union all
select 41512,'041512','2000-055ML','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.31,791,851,98.92,'Sun-Synch' from dual union all
select 41513,'041513','2000-055MM','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',102.91,850,943,98.88,'Sun-Synch' from dual union all
select 41514,'041514','2000-055MN','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.46,799,857,98.91,'Sun-Synch' from dual union all
select 41515,'041515','2000-055MP','deb NOAA 16','',66957,'In Earth orbit','','2016-05-22 00:00:00',101.73,830,852,99.02,'Sun-Synch' from dual union all
select 41516,'041516','2000-055MQ','deb NOAA 16','',66957,'In Earth orbit','','2016-05-22 00:00:00',101.79,834,853,98.87,'Sun-Synch' from dual union all
select 41517,'041517','2000-055MR','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.07,770,848,98.83,'Sun-Synch' from dual union all
select 41518,'041518','2000-055MS','deb NOAA 16','',66957,'In Earth orbit','','2016-05-22 00:00:00',101.67,816,860,98.87,'Sun-Synch' from dual union all
select 41519,'041519','2000-055MT','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.92,838,862,98.87,'Sun-Synch' from dual union all
select 4152,'004152','1969-082U','deb Agena D','',18349,'Reentered','1984-02-03 00:00:00','1969-10-31 00:00:00',102.17,805,918,69.99,'Intermediate' from dual union all
select 41520,'041520','2000-055MU','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',102.26,820,912,98.94,'Sun-Synch' from dual union all
select 41521,'041521','2000-055MV','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.72,792,889,98.8,'Sun-Synch' from dual union all
select 41522,'041522','2000-055MW','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.63,818,854,98.89,'Sun-Synch' from dual union all
select 41523,'041523','2000-055MX','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',102.1,848,869,98.94,'Sun-Synch' from dual union all
select 41524,'041524','2000-055MY','deb NOAA 16','',66957,'In Earth orbit','','2016-05-16 00:00:00',101.53,809,853,98.89,'Sun-Synch' from dual union all
select 41525,'041525','2000-055MZ','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.99,844,862,98.89,'Sun-Synch' from dual union all
select 41526,'041526','2000-055NA','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.61,813,857,98.97,'Sun-Synch' from dual union all
select 41527,'041527','2000-055NB','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.21,780,852,99,'Sun-Synch' from dual union all
select 41528,'041528','2000-055NC','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.88,827,869,98.8,'Sun-Synch' from dual union all
select 41529,'041529','2000-055ND','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',102.66,851,918,98.93,'Sun-Synch' from dual union all
select 4153,'004153','1969-082V','deb Agena D','',18349,'Reentered','1971-04-14 00:00:00','1969-11-02 00:00:00',100.88,691,910,69.92,'Intermediate' from dual union all
select 41530,'041530','2000-055NE','deb NOAA 16','',66957,'In Earth orbit','','2016-05-22 00:00:00',100.76,726,864,98.78,'Sun-Synch' from dual union all
select 41531,'041531','2000-055NF','deb NOAA 16','',66957,'In Earth orbit','','2016-05-21 00:00:00',102.52,851,905,98.91,'Sun-Synch' from dual union all
select 41532,'041532','2000-055NG','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.81,837,851,98.9,'Sun-Synch' from dual union all
select 41533,'041533','2000-055NH','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',100.53,714,854,98.82,'Sun-Synch' from dual union all
select 41534,'041534','2000-055NJ','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',102.39,850,894,98.92,'Sun-Synch' from dual union all
select 41535,'041535','2000-055NK','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.61,819,851,98.94,'Sun-Synch' from dual union all
select 41536,'041536','2000-055NL','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.76,831,853,98.89,'Sun-Synch' from dual union all
select 41537,'041537','2000-055NM','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.91,844,855,98.92,'Sun-Synch' from dual union all
select 41538,'041538','2000-055NN','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',102.88,852,937,98.85,'Sun-Synch' from dual union all
select 41539,'041539','2000-055NP','deb NOAA 16','',66957,'In Earth orbit','','2016-05-23 00:00:00',101.58,816,851,98.88,'Sun-Synch' from dual union all
select 4154,'004154','1969-082W','deb Agena D','',18349,'In Earth orbit','','1969-11-06 00:00:00',102.66,847,922,70.07,'Intermediate' from dual union all
select 41540,'041540','2000-055NQ','deb NOAA 16','',66957,'In Earth orbit','','2016-05-21 00:00:00',102.46,845,906,98.93,'Sun-Synch' from dual union all
select 41541,'041541','2014-087C','deb Resurs-P No. 2','deb Resurs-P No. 2',69791,'In Earth orbit','','2016-05-23 00:00:00',93.29,431,441,97.28,'Sun-Synch' from dual union all
select 41542,'041542','2014-087D','deb Resurs-P No. 2','deb Resurs-P No. 2',69791,'Lost','','2016-04-11 00:00:00',93.33,434,441,97.28,'Sun-Synch' from dual union all
select 41543,'041543','2015-075D','deb Briz-M','',70013,'In Earth orbit','','2016-05-15 00:00:00',1367.37,33394,35467,.38,'Medium' from dual union all
select 41544,'041544','2015-075E','deb Briz-M','',70013,'In Earth orbit','','2016-05-20 00:00:00',1362.57,33120,35550,.38,'Medium' from dual union all
select 41545,'041545','2015-075F','deb Briz-M','',70013,'In Earth orbit','','2016-05-05 00:00:00',1363.13,33291,35401,.37,'Medium' from dual union all
select 41546,'041546','2015-075G','deb Briz-M','',70013,'In Earth orbit','','2016-05-16 00:00:00',1376.15,33594,35616,.42,'Medium' from dual union all
select 41547,'041547','2015-075H','deb Briz-M','',70013,'In Earth orbit','','2016-05-21 00:00:00',1395.41,33843,36130,.44,'Drift GEO' from dual union all
select 41548,'041548','2015-075J','deb Briz-M','',70013,'In Earth orbit','','2016-05-22 00:00:00',1409.04,33622,36889,.86,'Drift GEO' from dual union all
select 41549,'041549','2016-030A','GalileoSat-14','Galileo FOC FM11',70208,'In Earth orbit','','2016-06-27 00:00:00',846.83,23268,23276,57.39,'Medium' from dual union all
select 4155,'004155','1969-082X','deb Agena D','',18349,'In Earth orbit','','1969-11-03 00:00:00',102.66,819,950,70.06,'Intermediate' from dual union all
select 41550,'041550','2016-030B','GalileoSat-13','Galileo FOC FM10',70208,'In Earth orbit','','2016-06-29 00:00:00',844.67,23215,23228,57.39,'Medium' from dual union all
select 41551,'041551','2016-030C','Fregat-MT No. 133-05','Fregat-MT    No. 133-05',70208,'In Earth orbit','','2016-06-24 00:00:00',859.18,23513,23606,57.4,'Medium' from dual union all
select 41552,'041552','2016-031A','Thaicom 8','Thaicom 8',70209,'In Earth orbit','','2016-05-30 00:00:00',2320.53,13074,90292,6.95,'Very High' from dual union all
select 41553,'041553','2016-031B','Falcon 9-025 Stage 2','F9-025 St2',70209,'Lost','','2016-05-27 00:00:00',1968.44,374,90927,21.12,'Very High' from dual union all
select 41554,'041554','2016-032A','Kosmos-2516','Uragan-M No. 753',70210,'In Earth orbit','','2016-06-28 00:00:00',675.71,19102,19157,64.8,'Medium' from dual union all
select 41555,'041555','2016-032B','Fregat-M No. 112-04','Fregat 14S44 No. 112-04',70210,'In Earth orbit','','2016-06-29 00:00:00',691.99,19214,19861,64.75,'Medium' from dual union all
select 41556,'041556','2016-033A','Zi Yuan 3 02 xing','ziyuan san hao 02 xing',70211,'In Earth orbit','','2016-06-01 00:00:00',94.45,484,500,97.5,'Sun-Synch' from dual union all
select 41557,'041557','2016-033B','Fresco','Nusat-1',70211,'In Earth orbit','','2016-06-01 00:00:00',94.4,480,498,97.5,'Sun-Synch' from dual union all
select 41558,'041558','2016-033C','Batata','Nusat-2',70211,'In Earth orbit','','2016-06-01 00:00:00',94.41,481,499,97.5,'Sun-Synch' from dual union all
select 41559,'041559','2016-033D','CZ-4B Stage 3','CZ-4B Stage 3',70211,'Reentered','2017-01-08 00:00:00','2016-06-01 00:00:00',92.13,264,494,97.41,'Sun-Synch' from dual union all
select 4156,'004156','1969-082Y','deb Agena D','',18349,'Reentered','1995-11-13 00:00:00','1969-11-11 00:00:00',104.29,852,1071,69.48,'Intermediate' from dual union all
select 41560,'041560','2008-067J','deb SOZ','',68575,'In Earth orbit','','2016-07-01 00:00:00',341.29,719,18880,65.44,'Highly Ellip' from dual union all
select 41561,'041561','2008-067K','deb SOZ','',68575,'In Earth orbit','','2016-07-01 00:00:00',328.23,383,18385,65.16,'Highly Ellip' from dual union all
select 41562,'041562','2008-067L','deb SOZ','',68575,'In Earth orbit','','2016-06-30 00:00:00',342.72,785,18903,65.45,'Highly Ellip' from dual union all
select 41563,'041563','1998-067JM','Flock 2e-6','Dove 0C75',66503,'Reentered','2018-02-12 00:00:00','2016-05-30 00:00:00',92.58,400,402,51.64,'Intermediate' from dual union all
select 41564,'041564','1998-067JN','Flock 2e-5','Dove 0C43',66503,'In Earth orbit','','2016-05-30 00:00:00',92.58,399,403,51.64,'Intermediate' from dual union all
select 41565,'041565','1998-067JP','Flock 2e-7','Dove 0C24',66503,'Reentered','2018-04-07 00:00:00','2016-05-31 00:00:00',92.57,398,403,51.64,'Intermediate' from dual union all
select 41566,'041566','1998-067JQ','Flock 2e-8','Dove 0C2B',66503,'Reentered','2017-07-25 00:00:00','2016-05-31 00:00:00',92.57,397,404,51.64,'Intermediate' from dual union all
select 41567,'041567','1998-067JR','Flock 2e''-5','Dove 0C59',66503,'In Earth orbit','','2016-05-31 00:00:00',92.57,399,402,51.65,'Intermediate' from dual union all
select 41568,'041568','1998-067JS','Flock 2e''-6','Dove 0C46',66503,'Reentered','2018-04-05 00:00:00','2016-06-01 00:00:00',92.58,398,403,51.64,'Intermediate' from dual union all
select 41569,'041569','1998-067JT','Flock 2e''-8','Dove 0C76',66503,'Reentered','2018-04-05 00:00:00','2016-06-01 00:00:00',92.55,396,403,51.64,'Intermediate' from dual union all
select 4157,'004157','1969-082Z','deb Agena D','',18349,'In Earth orbit','','1969-11-01 00:00:00',101.81,771,917,70.05,'Intermediate' from dual union all
select 41570,'041570','1998-067JU','Flock 2e''-7','Dove 0C42',66503,'Reentered','2017-10-03 00:00:00','2016-06-01 00:00:00',92.57,399,402,51.64,'Intermediate' from dual union all
select 41571,'041571','1998-067JV','Flock 2e-9','Dove 0C14',66503,'Reentered','2017-12-25 00:00:00','2016-06-01 00:00:00',92.57,398,403,51.64,'Intermediate' from dual union all
select 41572,'041572','1998-067JW','Flock 2e-10','Dove 0C82',66503,'Reentered','2018-02-28 00:00:00','2016-06-01 00:00:00',92.56,398,402,51.64,'Intermediate' from dual union all
select 41573,'041573','1998-067JX','Flock 2e-12','Dove 0C79',66503,'Reentered','2018-03-03 00:00:00','2016-06-03 00:00:00',92.57,395,406,51.64,'Intermediate' from dual union all
select 41574,'041574','1998-067JY','Flock 2e-11','Dove 0C13',66503,'Reentered','2018-03-04 00:00:00','2016-06-03 00:00:00',92.56,397,403,51.65,'Intermediate' from dual union all
select 41575,'041575','1998-067JZ','Flock 2e''-9','Dove 0C19',66503,'Reentered','2018-03-12 00:00:00','2016-06-03 00:00:00',92.57,395,406,51.64,'Intermediate' from dual union all
select 41576,'041576','1998-067KA','Flock 2e''-10','Dove 0C65',66503,'Reentered','2018-02-17 00:00:00','2016-06-03 00:00:00',92.57,398,403,51.64,'Intermediate' from dual union all
select 41577,'041577','1998-067KB','Flock 2e''-11','Dove 0C27',66503,'Reentered','2018-03-11 00:00:00','2016-06-03 00:00:00',92.56,397,403,51.64,'Intermediate' from dual union all
select 41578,'041578','1998-067KC','Flock 2e''-12','Dove 0C81',66503,'Reentered','2018-02-28 00:00:00','2016-06-03 00:00:00',92.58,398,403,51.64,'Intermediate' from dual union all
select 41579,'041579','2016-034A','Kosmos-2517','Musson-2 No. 12L',70212,'In Earth orbit','','2016-06-07 00:00:00',104.02,936,961,99.28,'Sun-Synch' from dual union all
select 4158,'004158','1969-082AA','deb Agena D','',18349,'In Earth orbit','','1992-01-29 00:00:00',102.3,836,898,70,'Intermediate' from dual union all
select 41580,'041580','2016-034B','Briz-KM No. 72527','Briz-KM 14S45',70212,'In Earth orbit','','2016-06-06 00:00:00',101.04,660,955,99.23,'Sun-Synch' from dual union all
select 41581,'041581','2016-035A','Intelsat IS-31','Intelsat ISDLA-2',70213,'In Earth orbit','','2016-07-09 00:00:00',1436.04,35776,35795,.12,'Stationary' from dual union all
select 41582,'041582','2016-035B','Briz-M No. 99570','Briz-M 14S43 No. 99570',70213,'In Earth orbit','','2016-06-17 00:00:00',1278.18,1797,63474,30.98,'Highly Ellip' from dual union all
select 41583,'041583','2016-035C','Briz-M DTB','Briz-M torus tank',70213,'In Earth orbit','','2016-07-09 00:00:00',262.21,356,14028,50.82,'Highly Ellip' from dual union all
select 41584,'041584','2016-036A','USA 268','ORION 9',70214,'In Earth orbit','','2016-07-21 00:00:00',1436.09,35594,35979,7.49,'Inclined GEO' from dual union all
select 41585,'041585','2016-036B','Delta 374','DCSS-5 F15',70214,'In Earth orbit','','2016-07-27 00:00:00',1383.59,33864,35640,7.52,'Inclined Drift' from dual union all
select 41586,'041586','2016-037A','Beidou DW 23','23 ke beidou daohang wei',70215,'In Earth orbit','','2016-07-12 00:00:00',1436.09,35776,35797,1.8,'Stationary' from dual union all
select 41587,'041587','2016-037B','CZ-3C Stage 3','CZ-3C Stage 3',70215,'In Earth orbit','','2016-07-12 00:00:00',607.15,221,34526,19.36,'GEO Transfer' from dual union all
select 41588,'041588','2016-038A','ABS-2A','ABS-2A',70216,'In Earth orbit','','2016-06-17 00:00:00',1222.57,397,62595,24.64,'Highly Ellip' from dual;
insert into SATELLITE
select 41589,'041589','2016-038B','EUTELSAT 117 West B','EUTELSAT 117 West B',70216,'In Earth orbit','','2016-06-17 00:00:00',1226.37,394,62754,24.64,'Highly Ellip' from dual union all
select 4159,'004159','1969-082AB','Agena D 2706','Agena D',18349,'In Earth orbit','','1969-11-07 00:00:00',105.2,917,1091,69.96,'Intermediate' from dual union all
select 41590,'041590','2016-038C','Falcon 9-026 Stage 2','F9-026 St2',70216,'In Earth orbit','','2016-06-18 00:00:00',1224.07,390,62663,24.64,'Highly Ellip' from dual union all
select 41591,'041591','2016-039A','BRISat','BRISat',70217,'In Earth orbit','','2016-07-19 00:00:00',1436.16,35784,35792,.02,'Stationary' from dual union all
select 41592,'041592','2016-039B','Echostar XVIII','Echostar 18',70217,'In Earth orbit','','2016-07-19 00:00:00',1436.09,35776,35797,.05,'Stationary' from dual union all
select 41593,'041593','2016-039C','ESC-A 584','Ariane 5 ESC-A',70217,'In Earth orbit','','2016-07-19 00:00:00',630.15,267,35672,6.33,'GEO Transfer' from dual union all
select 41594,'041594','2016-039D','Sylda 5A','Sylda 5A',70217,'In Earth orbit','','2016-07-20 00:00:00',89.73,148,375,51.58,'Intermediate' from dual union all
select 41595,'041595','2016-019B','Lemur-2-DrMuzz','Lemur-2-10',70197,'Reentered','2017-06-25 00:00:00','2016-06-21 00:00:00',92.37,387,395,51.64,'Intermediate' from dual union all
select 41596,'041596','2016-019C','Lemur-2-Bridgeman','Lemur-2-11',70197,'Reentered','2017-03-08 00:00:00','2016-06-21 00:00:00',92.37,388,394,51.64,'Intermediate' from dual union all
select 41597,'041597','2016-019D','Lemur-2-Cubecheese','Lemur-2-09',70197,'Reentered','2017-03-06 00:00:00','2016-06-21 00:00:00',92.38,389,394,51.64,'Intermediate' from dual union all
select 41598,'041598','2016-019E','Lemur-2-Nate','Lemur-2-12',70197,'Reentered','2017-02-27 00:00:00','2016-06-21 00:00:00',92.38,389,394,51.64,'Intermediate' from dual union all
select 41599,'041599','2016-040A','Cartosat-2 Series Satellite','Cartosat-2C',70218,'In Earth orbit','','2016-06-24 00:00:00',94.8,500,518,97.51,'Sun-Synch' from dual union all
select 416,'000416','61 Omicron 136','deb r/b Ablestar','',7273,'In Earth orbit','','1962-09-20 00:00:00',103.2,845,975,66.53,'Intermediate' from dual union all
select 4160,'004160','1969-082AC','deb Agena D','',18349,'In Earth orbit','','1992-02-02 00:00:00',99.73,696,795,70.06,'Intermediate' from dual union all
select 41600,'041600','2016-040B','Sathyabamasat','Sathyabamasat',70218,'In Earth orbit','','2016-06-24 00:00:00',94.79,500,516,97.51,'Sun-Synch' from dual union all
select 41601,'041601','2016-040C','SkySat 3','SkySat C1',70218,'In Earth orbit','','2016-06-24 00:00:00',94.76,499,515,97.51,'Sun-Synch' from dual union all
select 41602,'041602','2016-040D','Claire','GHGSat-D',70218,'In Earth orbit','','2016-06-24 00:00:00',94.73,500,511,97.5,'Sun-Synch' from dual union all
select 41603,'041603','2016-040E','LAPAN-A3','LAPAN-A3',70218,'In Earth orbit','','2016-06-24 00:00:00',94.79,500,516,97.51,'Sun-Synch' from dual union all
select 41604,'041604','2016-040F','BIROS','BIROS',70218,'In Earth orbit','','2016-06-24 00:00:00',94.78,500,516,97.51,'Sun-Synch' from dual union all
select 41605,'041605','2016-040G','M3MSat','exactView-7',70218,'In Earth orbit','','2016-06-24 00:00:00',94.77,500,515,97.51,'Sun-Synch' from dual union all
select 41606,'041606','2016-040H','Flock 2P-6','Dove 0E19',70218,'In Earth orbit','','2016-06-29 00:00:00',94.76,500,514,97.5,'Sun-Synch' from dual union all
select 41607,'041607','2016-040J','Swayam','Swayam',70218,'In Earth orbit','','2016-06-24 00:00:00',94.76,500,514,97.5,'Sun-Synch' from dual union all
select 41608,'041608','2016-040K','Flock 2P-11','Dove 0E30',70218,'In Earth orbit','','2016-06-29 00:00:00',94.76,500,514,97.51,'Sun-Synch' from dual union all
select 41609,'041609','2016-040L','Flock 2P-2','Dove 0E0E',70218,'In Earth orbit','','2016-06-29 00:00:00',94.76,500,514,97.51,'Sun-Synch' from dual union all
select 4161,'004161','1969-082AD','deb Agena D','',18349,'In Earth orbit','','1992-02-04 00:00:00',103.03,856,947,70.01,'Intermediate' from dual union all
select 41610,'041610','2016-040M','Flock 2P-9','Dove 0E26',70218,'In Earth orbit','','2016-06-30 00:00:00',94.75,499,513,97.51,'Sun-Synch' from dual union all
select 41611,'041611','2016-040N','Flock 2P-4','Dove 0E14',70218,'In Earth orbit','','2016-06-29 00:00:00',94.75,499,513,97.51,'Sun-Synch' from dual union all
select 41612,'041612','2016-040P','Flock 2P-10','Dove 0E2F',70218,'In Earth orbit','','2016-06-29 00:00:00',94.74,499,513,97.51,'Sun-Synch' from dual union all
select 41613,'041613','2016-040Q','Flock 2P-8','Dove 0E20',70218,'In Earth orbit','','2016-06-29 00:00:00',94.74,499,513,97.51,'Sun-Synch' from dual union all
select 41614,'041614','2016-040R','Flock 2P-12','Dove 0E3A',70218,'In Earth orbit','','2016-06-29 00:00:00',94.74,499,513,97.51,'Sun-Synch' from dual union all
select 41615,'041615','2016-040S','Flock 2P-7','Dove 0E1F',70218,'In Earth orbit','','2016-06-29 00:00:00',94.74,499,513,97.51,'Sun-Synch' from dual union all
select 41616,'041616','2016-040T','Flock 2P-5','Dove 0E16',70218,'In Earth orbit','','2016-06-29 00:00:00',94.74,499,513,97.51,'Sun-Synch' from dual union all
select 41617,'041617','2016-040U','Flock 2P-1','Dove 0E0D',70218,'In Earth orbit','','2016-06-24 00:00:00',94.79,499,518,97.51,'Sun-Synch' from dual union all
select 41618,'041618','2016-040V','Flock 2P-3','Dove 0E0F',70218,'In Earth orbit','','2016-06-29 00:00:00',94.74,499,513,97.51,'Sun-Synch' from dual union all
select 41619,'041619','2016-040W','BEESAT 4','BEESAT 4',70218,'In Earth orbit','','2016-10-11 00:00:00',94.76,499,515,97.51,'Sun-Synch' from dual union all
select 4162,'004162','1969-082AE','deb Agena D','',18349,'Reentered','1979-04-14 00:00:00','1979-01-27 00:00:00',95.22,522,537,70.19,'Intermediate' from dual union all
select 41620,'041620','2016-040X','PS4','PS4 PSLV-C34',70218,'In Earth orbit','','2016-06-23 00:00:00',93.86,447,480,97.53,'Sun-Synch' from dual union all
select 41621,'041621','2016-040Y','DLA-U','PSLV-C34-DLA-U',70218,'In Earth orbit','','2016-06-29 00:00:00',94.78,499,517,97.51,'Sun-Synch' from dual union all
select 41622,'041622','2016-041A','MUOS 5','MUOS SV 3',70219,'In Earth orbit','','2016-07-29 00:00:00',945.32,15268,35789,9.8,'Medium' from dual union all
select 41623,'041623','2016-041B','Centaur AV-063','Centaur AV-063',70219,'In Earth orbit','','2016-07-13 00:00:00',687.19,3664,35171,18.98,'GEO Transfer' from dual union all
select 41624,'041624','2016-042A','Yuanzheng-1A Y1','YZ-1A',70220,'Deorbited','2016-06-27 00:00:00','2016-06-25 00:00:00',91.21,287,381,40.81,'Intermediate' from dual union all
select 41625,'041625','2016-042B','Aoxiang zhixing','Aoxiang zhixing',70220,'Reentered','2016-09-29 00:00:00','2016-06-25 00:00:00',90.3,200,379,40.8,'Intermediate' from dual union all
select 41626,'041626','2016-042C','ADRV adapter','',70220,'Reentered','2016-09-11 00:00:00','2016-06-25 00:00:00',90.28,200,377,40.8,'Intermediate' from dual union all
select 41627,'041627','2016-042D','Pei zhong zhijia','Reentry shroud',70220,'Reentered','2016-07-02 00:00:00','2016-06-25 00:00:00',90.32,203,378,40.8,'Intermediate' from dual union all
select 41628,'041628','2016-042E','CZ-7 Stage 2','CZ-7  Stage 2',70220,'Reentered','2016-07-28 00:00:00','2016-06-25 00:00:00',91.17,288,375,40.81,'Intermediate' from dual union all
select 41629,'041629','2016-042F','Aolong 1','Ao Long yi hao',70220,'Reentered','2016-08-27 00:00:00','2016-06-25 00:00:00',90.32,202,379,40.81,'Intermediate' from dual union all
select 4163,'004163','1969-082AF','deb Agena D','',18349,'Reentered','1981-01-09 00:00:00','1969-11-08 00:00:00',101.67,758,918,70.08,'Intermediate' from dual union all
select 41630,'041630','2016-042G','CZ-7 sep motor cover','',70220,'Reentered','2016-07-04 00:00:00','2016-06-25 00:00:00',92.14,198,561,40.65,'Intermediate' from dual union all
select 41631,'041631','2016-042H','CZ-7 sep motor cover','',70220,'Reentered','2016-07-04 00:00:00','2016-06-25 00:00:00',92.31,198,578,40.47,'Intermediate' from dual union all
select 41632,'041632','2016-042J','CZ-7 sep motor cover','',70220,'Reentered','2016-07-04 00:00:00','2016-06-25 00:00:00',92.4,199,586,41.17,'Intermediate' from dual union all
select 41633,'041633','2016-042K','CZ-7 sep motor cover','',70220,'Reentered','2016-07-03 00:00:00','2016-06-25 00:00:00',92.42,196,591,40.98,'Intermediate' from dual union all
select 41634,'041634','2016-043A','Shi Jian 16 02 xing','shijian shiliu hao 02 xi',70221,'In Earth orbit','','2016-07-01 00:00:00',96.81,596,616,75,'Intermediate' from dual union all
select 41635,'041635','2016-043B','CZ-4B Stage 3','CZ-4B Stage 3',70221,'In Earth orbit','','2016-07-01 00:00:00',93.92,337,596,74.98,'Intermediate' from dual union all
select 41636,'041636','2016-042L','Tiange feixingqi 1','Tiange feixingqi 1',70220,'Reentered','2016-08-27 00:00:00','2016-06-25 00:00:00',90.28,200,377,40.8,'Intermediate' from dual union all
select 41637,'041637','2016-042M','Tiange feixingqi 2','Tiange feixingqi 2',70220,'Reentered','2016-08-24 00:00:00','2016-06-25 00:00:00',90.32,203,378,40.8,'Intermediate' from dual union all
select 41638,'041638','2016-042N','deb YZ-1A','',70220,'Reentered','2016-06-29 00:00:00','2016-06-25 00:00:00',88.49,136,264,40.82,'Intermediate' from dual union all
select 41639,'041639','2016-044A','Soyuz MS-01','Soyuz 11F732A48 No. 731',70222,'Landed','2016-10-30 00:00:00','2016-07-08 00:00:00',90.24,280,293,51.64,'Intermediate' from dual union all
select 4164,'004164','1969-082AG','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',102.7,850,923,70.12,'Intermediate' from dual union all
select 41640,'041640','2016-044B','11S510','11S510',70222,'Reentered','2016-07-10 00:00:00','2016-07-08 00:00:00',88.43,185,209,51.64,'Intermediate' from dual union all
select 41641,'041641','2008-067M','deb SOZ','',68575,'In Earth orbit','','2016-07-08 00:00:00',344.7,741,19072,65.21,'Highly Ellip' from dual union all
select 41642,'041642','2008-067N','deb SOZ','',68575,'In Earth orbit','','2016-07-08 00:00:00',337.08,635,18697,65.48,'Highly Ellip' from dual union all
select 41643,'041643','2008-067P','deb SOZ','',68575,'In Earth orbit','','2016-07-08 00:00:00',348.5,835,19218,64.9,'Highly Ellip' from dual union all
select 41644,'041644','2008-067Q','deb SOZ','',68575,'In Earth orbit','','2016-07-08 00:00:00',339.1,557,18902,64.86,'Highly Ellip' from dual union all
select 41645,'041645','2008-067R','deb SOZ','',68575,'In Earth orbit','','2016-07-08 00:00:00',347.04,790,19171,65.11,'Highly Ellip' from dual union all
select 41646,'041646','2008-067S','deb SOZ','',68575,'In Earth orbit','','2016-07-08 00:00:00',347.23,793,19180,65.1,'Highly Ellip' from dual union all
select 41647,'041647','2008-067T','deb SOZ','',68575,'In Earth orbit','','2016-07-08 00:00:00',347.59,713,19283,65.09,'Highly Ellip' from dual union all
select 41648,'041648','2008-067U','deb SOZ','',68575,'In Earth orbit','','2016-07-08 00:00:00',347.74,773,19232,65.07,'Highly Ellip' from dual union all
select 41649,'041649','2008-067V','deb SOZ','',68575,'In Earth orbit','','2016-07-08 00:00:00',347.67,807,19194,65.08,'Highly Ellip' from dual union all
select 4165,'004165','1969-082AH','deb Agena D','',18349,'Reentered','1989-12-02 00:00:00','1989-11-30 00:00:00',91,314,333,70.05,'Intermediate' from dual union all
select 41650,'041650','2008-067W','deb SOZ','',68575,'In Earth orbit','','2016-07-08 00:00:00',347.67,783,19217,65.11,'Highly Ellip' from dual union all
select 41651,'041651','2008-067X','deb SOZ','',68575,'In Earth orbit','','2016-07-08 00:00:00',347.62,765,19233,65.08,'Highly Ellip' from dual union all
select 41652,'041652','2000-055NR','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',102.41,838,908,98.94,'Sun-Synch' from dual union all
select 41653,'041653','2000-055NS','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',101.82,839,850,98.93,'Sun-Synch' from dual union all
select 41654,'041654','2000-055NT','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',102.02,819,890,98.85,'Sun-Synch' from dual union all
select 41655,'041655','2000-055NU','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',101.69,822,856,98.88,'Sun-Synch' from dual union all
select 41656,'041656','2000-055NV','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',102.08,851,864,98.9,'Sun-Synch' from dual union all
select 41657,'041657','2000-055NW','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',101.9,836,862,98.84,'Sun-Synch' from dual union all
select 41658,'041658','2000-055NX','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',101.69,813,865,98.91,'Sun-Synch' from dual union all
select 41659,'041659','2000-055NY','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',101.85,831,861,98.93,'Sun-Synch' from dual union all
select 4166,'004166','1969-082J','SOICAL CYLINDER','S69-4 Cylinder',18349,'In Earth orbit','','1970-08-20 00:00:00',103.39,903,935,70.02,'Intermediate' from dual union all
select 41660,'041660','2000-055NZ','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',101.64,820,853,98.87,'Sun-Synch' from dual union all
select 41661,'041661','2000-055PA','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',101.66,816,859,98.99,'Sun-Synch' from dual union all
select 41662,'041662','2000-055PB','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',101.84,832,860,98.91,'Sun-Synch' from dual union all
select 41663,'041663','2000-055PC','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',101.67,823,853,98.89,'Sun-Synch' from dual union all
select 41664,'041664','2000-055PD','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',101.67,822,854,98.92,'Sun-Synch' from dual union all
select 41665,'041665','2000-055PE','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',102.52,848,908,98.91,'Sun-Synch' from dual union all
select 41666,'041666','2000-055PF','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',101.52,808,853,98.9,'Sun-Synch' from dual union all
select 41667,'041667','2000-055PG','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',101.55,809,856,98.9,'Sun-Synch' from dual union all
select 41668,'041668','2000-055PH','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',101.39,787,862,98.93,'Sun-Synch' from dual union all
select 41669,'041669','2000-055PJ','deb NOAA 16','',66957,'In Earth orbit','','2016-07-08 00:00:00',101.66,817,857,98.83,'Sun-Synch' from dual union all
select 4167,'004167','1969-082AK','deb Agena D','',18349,'Reentered','1989-09-04 00:00:00','1989-09-03 00:00:00',89.91,251,289,69.4,'Intermediate' from dual union all
select 41670,'041670','2016-045A','Progress MS-03','Progress 7K-TGM No. 433',70223,'Deorbited','2017-01-31 00:00:00','2016-07-18 00:00:00',90.15,280,284,51.64,'Intermediate' from dual union all
select 41671,'041671','2016-045B','11S510','11S510',70223,'Reentered','2016-07-19 00:00:00','2016-07-18 00:00:00',88.07,167,191,51.66,'Intermediate' from dual union all
select 41672,'041672','2016-046A','Dragon CRS-9','Dragon C111',70224,'Landed','2016-08-26 00:00:00','2016-07-19 00:00:00',92.37,390,392,51.66,'Intermediate' from dual union all
select 41673,'041673','2016-046B','Dragon solar array cover','',70224,'Reentered','2016-07-22 00:00:00','2016-07-20 00:00:00',89.37,192,295,51.65,'Intermediate' from dual union all
select 41674,'041674','2016-046C','Dragon solar array cover','',70224,'Reentered','2016-07-22 00:00:00','2016-07-20 00:00:00',89.45,193,302,51.67,'Intermediate' from dual union all
select 41675,'041675','1994-076J','deb SOZ','',65472,'In Earth orbit','','2016-07-27 00:00:00',337.08,704,18628,64.85,'Highly Ellip' from dual union all
select 41676,'041676','2007-052H','deb SOZ','',68394,'In Earth orbit','','2016-07-27 00:00:00',343.67,782,18967,66.45,'Highly Ellip' from dual union all
select 41677,'041677','2007-052J','deb SOZ','',68394,'In Earth orbit','','2016-07-27 00:00:00',337.23,481,18861,64.99,'Highly Ellip' from dual union all
select 41678,'041678','2007-052K','deb SOZ','',68394,'In Earth orbit','','2016-06-30 00:00:00',321.71,845,17504,65.31,'Highly Ellip' from dual;
insert into SATELLITE
select 41679,'041679','2007-052L','deb SOZ','',68394,'In Earth orbit','','2016-07-27 00:00:00',338.12,652,18746,64.67,'Highly Ellip' from dual union all
select 4168,'004168','1969-082H','Tempsat 2','Tempsat 2',18349,'In Earth orbit','','1969-10-31 00:00:00',103.44,907,936,70.02,'Intermediate' from dual union all
select 41680,'041680','2007-052M','deb SOZ','',68394,'In Earth orbit','','2016-07-26 00:00:00',271.95,479,14573,65.37,'Highly Ellip' from dual union all
select 41681,'041681','2007-052N','deb SOZ','',68394,'In Earth orbit','','2016-07-27 00:00:00',330.53,410,18504,65.11,'Highly Ellip' from dual union all
select 41682,'041682','2007-052P','deb SOZ','',68394,'In Earth orbit','','2016-07-27 00:00:00',343.53,1092,18648,65.83,'Highly Ellip' from dual union all
select 41683,'041683','2007-052Q','deb SOZ','',68394,'In Earth orbit','','2016-07-23 00:00:00',333.95,663,18469,64.34,'Highly Ellip' from dual union all
select 41684,'041684','2007-052R','deb SOZ','',68394,'In Earth orbit','','2016-07-23 00:00:00',316.77,749,17281,65.53,'Highly Ellip' from dual union all
select 41685,'041685','2007-052S','deb SOZ','',68394,'In Earth orbit','','2016-07-26 00:00:00',340.22,788,18743,65.69,'Highly Ellip' from dual union all
select 41686,'041686','2007-052T','deb SOZ','',68394,'In Earth orbit','','2016-07-25 00:00:00',337.96,670,18718,64.66,'Highly Ellip' from dual union all
select 41687,'041687','2007-052U','deb SOZ','',68394,'In Earth orbit','','2016-07-13 00:00:00',320.3,828,17430,64.83,'Highly Ellip' from dual union all
select 41688,'041688','2007-052V','deb SOZ','',68394,'In Earth orbit','','2016-07-26 00:00:00',337.68,668,18702,64.67,'Highly Ellip' from dual union all
select 41689,'041689','2007-052W','deb SOZ','',68394,'In Earth orbit','','2016-07-27 00:00:00',331.23,631,18329,65.44,'Highly Ellip' from dual union all
select 4169,'004169','1969-082AM','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',100.68,749,833,70.2,'Intermediate' from dual union all
select 41690,'041690','2007-052X','deb SOZ','',68394,'In Earth orbit','','2016-07-18 00:00:00',336.29,571,18711,64.42,'Highly Ellip' from dual union all
select 41691,'041691','2007-052Y','deb SOZ','',68394,'In Earth orbit','','2016-07-21 00:00:00',334.01,575,18561,64.77,'Highly Ellip' from dual union all
select 41692,'041692','2007-052Z','deb SOZ','',68394,'In Earth orbit','','2016-07-23 00:00:00',339.38,473,19005,65.37,'Highly Ellip' from dual union all
select 41693,'041693','2007-052AA','deb SOZ','',68394,'In Earth orbit','','2016-07-26 00:00:00',336.3,478,18805,65.66,'Highly Ellip' from dual union all
select 41694,'041694','2007-052AB','deb SOZ','',68394,'In Earth orbit','','2016-07-22 00:00:00',341.88,538,19098,65.6,'Highly Ellip' from dual union all
select 41695,'041695','2010-007M','deb SOZ','',68867,'In Earth orbit','','2016-07-27 00:00:00',323.65,485,17989,64.89,'Highly Ellip' from dual union all
select 41696,'041696','2010-007N','deb SOZ','',68867,'In Earth orbit','','2016-07-13 00:00:00',354.16,919,19488,64.61,'Highly Ellip' from dual union all
select 41697,'041697','2010-007P','deb SOZ','',68867,'In Earth orbit','','2016-07-26 00:00:00',348.09,1069,18958,65.23,'Highly Ellip' from dual union all
select 41698,'041698','2010-007Q','deb SOZ','',68867,'In Earth orbit','','2016-07-27 00:00:00',331.36,802,18166,65.2,'Highly Ellip' from dual union all
select 41699,'041699','2010-007R','deb SOZ','',68867,'In Earth orbit','','2016-07-27 00:00:00',333.8,832,18292,65.17,'Highly Ellip' from dual union all
select 417,'000417','61 Omicron 137','deb r/b Ablestar','',7273,'In Earth orbit','','1962-09-14 00:00:00',103.22,845,977,66.72,'Intermediate' from dual union all
select 4170,'004170','1969-082AN','deb Agena D','',18349,'Reentered','1990-01-23 00:00:00','1990-01-20 00:00:00',90.74,306,316,70.07,'Intermediate' from dual union all
select 41700,'041700','2010-007S','deb SOZ','',68867,'In Earth orbit','','2016-07-27 00:00:00',350.67,1022,19167,65.21,'Highly Ellip' from dual union all
select 41701,'041701','2010-007T','deb SOZ','',68867,'In Earth orbit','','2016-07-27 00:00:00',348.23,1077,18959,65.54,'Highly Ellip' from dual union all
select 41702,'041702','2010-007U','deb SOZ','',68867,'In Earth orbit','','2016-07-25 00:00:00',342.08,694,18955,64.99,'Highly Ellip' from dual union all
select 41703,'041703','2010-007V','deb SOZ','',68867,'In Earth orbit','','2016-07-26 00:00:00',334.79,784,18403,65.19,'Highly Ellip' from dual union all
select 41704,'041704','2010-007W','deb SOZ','',68867,'In Earth orbit','','2016-07-27 00:00:00',330.87,961,17976,65.49,'Highly Ellip' from dual union all
select 41705,'041705','2000-055PK','deb NOAA 16','',66957,'In Earth orbit','','2016-07-26 00:00:00',101.79,831,855,98.91,'Sun-Synch' from dual union all
select 41706,'041706','2000-055PL','deb NOAA 16','',66957,'In Earth orbit','','2016-07-25 00:00:00',102.54,852,906,98.93,'Sun-Synch' from dual union all
select 41707,'041707','2000-055PM','deb NOAA 16','',66957,'In Earth orbit','','2016-07-27 00:00:00',101.52,805,857,98.88,'Sun-Synch' from dual union all
select 41708,'041708','2000-055PN','deb NOAA 16','',66957,'In Earth orbit','','2016-07-27 00:00:00',101.91,836,862,98.92,'Sun-Synch' from dual union all
select 41709,'041709','2000-055PP','deb NOAA 16','',66957,'In Earth orbit','','2016-07-27 00:00:00',101.71,813,867,98.98,'Sun-Synch' from dual union all
select 4171,'004171','1969-082AP','deb Agena D','',18349,'In Earth orbit','','1992-02-04 00:00:00',101.59,798,870,70.08,'Intermediate' from dual union all
select 41710,'041710','2000-055PQ','deb NOAA 16','',66957,'In Earth orbit','','2016-07-27 00:00:00',101.35,791,854,98.9,'Sun-Synch' from dual union all
select 41711,'041711','2000-055PR','deb NOAA 16','',66957,'In Earth orbit','','2016-07-27 00:00:00',101.75,827,855,98.89,'Sun-Synch' from dual union all
select 41712,'041712','2000-055PS','deb NOAA 16','',66957,'In Earth orbit','','2016-07-27 00:00:00',102,816,891,98.9,'Sun-Synch' from dual union all
select 41713,'041713','2000-055PT','deb NOAA 16','',66957,'In Earth orbit','','2016-07-27 00:00:00',103.52,841,1009,98.9,'Sun-Synch' from dual union all
select 41714,'041714','2000-055PU','deb NOAA 16','',66957,'In Earth orbit','','2016-07-25 00:00:00',103.75,847,1024,98.93,'Sun-Synch' from dual union all
select 41715,'041715','2000-055PV','deb NOAA 16','',66957,'In Earth orbit','','2016-07-27 00:00:00',102.01,844,864,98.79,'Sun-Synch' from dual union all
select 41716,'041716','2000-055PW','deb NOAA 16','',66957,'In Earth orbit','','2016-07-24 00:00:00',101.92,845,855,98.89,'Sun-Synch' from dual union all
select 41717,'041717','2000-055PX','deb NOAA 16','',66957,'In Earth orbit','','2016-07-27 00:00:00',101.64,818,855,98.89,'Sun-Synch' from dual union all
select 41718,'041718','2015-020F','deb Briz-KM','',69958,'In Earth orbit','','2016-07-26 00:00:00',111.51,1121,1472,82.49,'Intermediate' from dual union all
select 41719,'041719','2015-020G','deb Briz-KM','',69958,'In Earth orbit','','2016-07-27 00:00:00',112.92,1189,1533,82.49,'Intermediate' from dual union all
select 4172,'004172','1969-082AQ','deb Agena D','',18349,'Reentered','1991-10-24 00:00:00','1991-10-24 00:00:00',89.7,243,277,69.98,'Intermediate' from dual union all
select 41720,'041720','2015-020H','deb Briz-KM','',69958,'In Earth orbit','','2016-07-24 00:00:00',111.54,1121,1474,82.49,'Intermediate' from dual union all
select 41721,'041721','2015-020J','deb Briz-KM','',69958,'In Earth orbit','','2016-07-23 00:00:00',112.97,1173,1553,82.49,'Intermediate' from dual union all
select 41722,'041722','2015-020K','deb Briz-KM','',69958,'In Earth orbit','','2016-07-26 00:00:00',113.34,1241,1519,82.49,'Intermediate' from dual union all
select 41723,'041723','2015-020L','deb Briz-KM','',69958,'In Earth orbit','','2016-07-23 00:00:00',111.48,1124,1466,82.49,'Intermediate' from dual union all
select 41724,'041724','2016-047A','USA 269','QUASAR 19',70225,'In Earth orbit','','2016-10-07 00:00:00',1436.09,35771,35802,4.94,'Inclined GEO' from dual union all
select 41725,'041725','2016-048A','Tiantong-1 01 xing','Tiantong-1-01',70226,'In Earth orbit','','2016-09-05 00:00:00',1436.06,35777,35795,4.97,'Inclined GEO' from dual union all
select 41726,'041726','2016-048B','CZ-3B Stage 3','CZ-3B Stage 3',70226,'Reentered','2017-07-21 00:00:00','2016-09-05 00:00:00',608.92,179,34660,28.46,'GEO Transfer' from dual union all
select 41727,'041727','2016-049A','Gao Fen 3','Gaofen san hao weixing',70227,'In Earth orbit','','2016-08-12 00:00:00',99.83,750,751,98.42,'Sun-Synch' from dual union all
select 41728,'041728','2016-049B','CZ-4C Stage 3','CZ-4C Stage 3',70227,'In Earth orbit','','2016-08-12 00:00:00',98.1,592,744,98.39,'Sun-Synch' from dual union all
select 41729,'041729','2016-050A','JCSAT 16','JCSAT 16',70228,'In Earth orbit','','2016-09-13 00:00:00',1436.09,35772,35801,.03,'Stationary' from dual union all
select 4173,'004173','1969-082AR','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',100.67,751,830,70.09,'Intermediate' from dual union all
select 41730,'041730','2016-050B','Falcon 9-028 Stage 2','F9-028 St2',70228,'Reentered','2016-09-26 00:00:00','2016-08-22 00:00:00',549.7,92,31609,20.94,'Highly Ellip' from dual union all
select 41731,'041731','2016-051A','Mozi hao','QUESS',70229,'In Earth orbit','','2016-08-15 00:00:00',94.53,488,503,97.36,'Sun-Synch' from dual union all
select 41732,'041732','2016-051B','3Cat-2','3Cat-2',70229,'In Earth orbit','','2016-08-15 00:00:00',94.49,485,503,97.38,'Sun-Synch' from dual union all
select 41733,'041733','2016-051C','Lixing 1','Qibo Daqi K Weixing',70229,'Reentered','2016-08-19 00:00:00','2016-08-15 00:00:00',94.47,483,503,97.35,'Sun-Synch' from dual union all
select 41734,'041734','1965-065U','deb Dodecapole 2','',11815,'In Earth orbit','','2016-08-15 00:00:00',92.12,217,540,97.4,'Sun-Synch' from dual union all
select 41735,'041735','2009-055C','deb Worldview-2','',68736,'In Earth orbit','','2016-08-17 00:00:00',100.3,767,779,98.54,'Sun-Synch' from dual union all
select 41736,'041736','2009-055D','deb Worldview-2','',68736,'In Earth orbit','','2016-08-17 00:00:00',100.23,766,773,98.54,'Sun-Synch' from dual union all
select 41737,'041737','2009-055E','deb Worldview-2','',68736,'In Earth orbit','','2016-08-17 00:00:00',100.15,760,771,98.55,'Sun-Synch' from dual union all
select 41738,'041738','2009-055F','deb Worldview-2','',68736,'In Earth orbit','','2016-08-17 00:00:00',99.38,679,778,98.56,'Sun-Synch' from dual union all
select 41739,'041739','2009-055G','deb Worldview-2','',68736,'In Earth orbit','','2016-08-16 00:00:00',99.93,726,785,98.56,'Sun-Synch' from dual union all
select 4174,'004174','1969-082AS','deb Agena D','',18349,'In Earth orbit','','1992-02-04 00:00:00',101.47,788,869,70,'Intermediate' from dual union all
select 41740,'041740','2009-055H','deb Worldview-2','',68736,'In Earth orbit','','2016-08-17 00:00:00',100.18,765,769,98.55,'Sun-Synch' from dual union all
select 41741,'041741','2009-055J','deb Worldview-2','',68736,'In Earth orbit','','2016-08-17 00:00:00',100.25,769,772,98.55,'Sun-Synch' from dual union all
select 41742,'041742','2009-055K','deb Worldview-2','',68736,'In Earth orbit','','2016-08-16 00:00:00',100.07,756,768,98.55,'Sun-Synch' from dual union all
select 41743,'041743','2009-055L','deb Worldview-2','',68736,'In Earth orbit','','2016-08-17 00:00:00',100.23,766,773,98.55,'Sun-Synch' from dual union all
select 41744,'041744','2016-052A','USA 270','Hornet 3',70230,'In Earth orbit','','2016-08-26 00:00:00',1436.04,35752,35819,0,'Stationary' from dual union all
select 41745,'041745','2016-052B','USA 271','Hornet 4',70230,'In Earth orbit','','2016-08-26 00:00:00',1438.46,35791,35874,.02,'Drift GEO' from dual union all
select 41746,'041746','2016-052C','Delta 4 Second Stage','DCSS-4',70230,'In Earth orbit','','2016-10-04 00:00:00',1454.75,36134,36168,.91,'Drift GEO' from dual union all
select 41747,'041747','2016-053A','Intelsat IS-36','Intelsat IS-36',70231,'In Earth orbit','','2016-09-02 00:00:00',1436.55,35785,35806,.13,'Drift GEO' from dual union all
select 41748,'041748','2016-053B','Intelsat IS-33e','Intelsat IS-33e',70231,'In Earth orbit','','2016-09-02 00:00:00',828.97,9959,35748,2.05,'Medium' from dual union all
select 41749,'041749','2016-053C','ESC-A 586','Ariane 5 ESC-A',70231,'In Earth orbit','','2016-09-01 00:00:00',629.64,261,35652,6.07,'GEO Transfer' from dual union all
select 4175,'004175','1969-082AT','deb Agena D','',18349,'Reentered','1992-12-14 00:00:00','1992-02-05 00:00:00',96.27,566,594,69.98,'Intermediate' from dual union all
select 41750,'041750','2016-053D','Sylda 5A','Sylda 5A',70231,'In Earth orbit','','2016-09-01 00:00:00',629.8,263,35658,5.96,'GEO Transfer' from dual union all
select 41751,'041751','1998-067KD','EVA-36 debris','Antenna cover',66503,'Reentered','2016-11-23 00:00:00','2016-08-26 00:00:00',92.54,398,400,51.65,'Intermediate' from dual union all
select 41752,'041752','2016-054A','Insat 3DR','Insat 3DR',70232,'In Earth orbit','','2016-09-15 00:00:00',1432.23,35401,36021,.13,'Drift GEO' from dual union all
select 41753,'041753','2016-054B','GSLV CUS-07','GSLV-F05 CUS',70232,'In Earth orbit','','2016-09-16 00:00:00',627.2,174,35613,20.65,'GEO Transfer' from dual union all
select 41754,'041754','1998-067KE','EVA-37 debris','',66503,'Reentered','2016-12-01 00:00:00','2016-09-08 00:00:00',92.6,400,403,51.64,'Intermediate' from dual union all
select 41755,'041755','1998-067KF','EVA-37 debris','',66503,'Reentered','2016-12-29 00:00:00','2016-09-08 00:00:00',92.55,397,402,51.65,'Intermediate' from dual union all
select 41756,'041756','1998-067KG','EVA-37 debris','',66503,'Reentered','2016-12-27 00:00:00','2016-09-08 00:00:00',92.56,397,403,51.65,'Intermediate' from dual union all
select 41757,'041757','2016-055A','OSIRIS-REx','OSIRIS-REx',70233,'Deep Space','2016-09-09 00:00:00','2016-09-08 00:00:00',263.28,202,40174,29.54,'Earth Escape' from dual union all
select 41758,'041758','2016-055B','Centaur AV-067','Centaur AV-067',70233,'Deep Space','2016-09-09 00:00:00','2016-09-08 00:00:00',263.28,202,40174,29.54,'Earth Escape' from dual union all
select 41759,'041759','2016-056A','''Ofeq-11','''Ofeq-11',70234,'In Earth orbit','','2016-09-18 00:00:00',93.94,341,594,142.53,'Retrograde' from dual union all
select 4176,'004176','1969-082AU','deb Agena D','',18349,'Reentered','1980-05-11 00:00:00','1980-05-08 00:00:00',92.36,366,414,70.1,'Intermediate' from dual union all
select 41760,'041760','2016-056B','AUS-51','AUS-51',70234,'In Earth orbit','','2016-09-18 00:00:00',93.94,341,594,142.53,'Retrograde' from dual union all
select 41761,'041761','1998-067KH','Flock 2e''-13','Dove 0C45',66503,'Reentered','2018-06-03 00:00:00','2016-09-15 00:00:00',92.62,397,408,51.65,'Intermediate' from dual union all
select 41762,'041762','1998-067KJ','Flock 2e''-14','Dove 0C56',66503,'In Earth orbit','','2016-09-15 00:00:00',92.62,397,408,51.64,'Intermediate' from dual union all
select 41763,'041763','1998-067KK','Flock 2e''-16','Dove 0C0B',66503,'Reentered','2018-06-01 00:00:00','2016-09-15 00:00:00',92.63,398,409,51.64,'Intermediate' from dual union all
select 41764,'041764','1998-067KL','Flock 2e''-15','Dove 0C54',66503,'In Earth orbit','','2016-09-15 00:00:00',92.63,399,408,51.65,'Intermediate' from dual union all
select 41765,'041765','2016-057A','Tiangong 2','Tiangong 2',70235,'In Earth orbit','','2016-09-18 00:00:00',92.01,369,377,42.79,'Intermediate' from dual union all
select 41766,'041766','2016-057B','CZ-2F Stage 2','CZ-2F Stage 2',70235,'Reentered','2016-09-29 00:00:00','2016-09-17 00:00:00',89.63,193,319,42.78,'Intermediate' from dual union all
select 41767,'041767','2016-057C','CZ-2F sep motor cover','',70235,'Reentered','2016-09-18 00:00:00','2016-09-17 00:00:00',90.35,199,384,42.89,'Intermediate' from dual union all
select 41768,'041768','2006-062J','deb SOZ','',68234,'In Earth orbit','','2016-10-12 00:00:00',343.46,461,19275,64.35,'Highly Ellip' from dual;
insert into SATELLITE
select 41769,'041769','1998-067KM','Flock 2e''-18','Dove 0C44',66503,'In Earth orbit','','2016-09-16 00:00:00',92.62,398,408,51.64,'Intermediate' from dual union all
select 4177,'004177','1969-082AV','deb Agena D','',18349,'In Earth orbit','','1992-02-02 00:00:00',104.51,882,1060,70.06,'Intermediate' from dual union all
select 41770,'041770','2016-058A','PeruSat-1','Peru Recon',70236,'In Earth orbit','','2016-09-17 00:00:00',98.29,676,678,98.21,'Sun-Synch' from dual union all
select 41771,'041771','2016-058B','SkySat 4','SkySat C4',70236,'In Earth orbit','','2016-09-17 00:00:00',94.64,497,506,97.43,'Sun-Synch' from dual union all
select 41772,'041772','2016-058C','SkySat 5','SkySat C5',70236,'In Earth orbit','','2016-09-17 00:00:00',94.65,501,502,97.42,'Sun-Synch' from dual union all
select 41773,'041773','2016-058D','SkySat 6','SkySat C2',70236,'In Earth orbit','','2016-09-17 00:00:00',94.64,501,502,97.42,'Sun-Synch' from dual union all
select 41774,'041774','2016-058E','SkySat 7','SkySat C3',70236,'In Earth orbit','','2016-09-17 00:00:00',94.64,501,502,97.42,'Sun-Synch' from dual union all
select 41775,'041775','2016-058F','VESPA Upper Part','VUP',70236,'In Earth orbit','','2016-09-17 00:00:00',94.68,498,509,97.45,'Sun-Synch' from dual union all
select 41776,'041776','1998-067KN','Flock 2e''-17','Dove 0C12',66503,'In Earth orbit','','2016-09-15 00:00:00',92.62,398,408,51.64,'Intermediate' from dual union all
select 41777,'041777','1998-067KP','Flock 2e''-19','Dove 0C62',66503,'Reentered','2018-05-12 00:00:00','2016-09-15 00:00:00',92.63,398,409,51.64,'Intermediate' from dual union all
select 41778,'041778','2016-057D','CZ-2F sep motor cover','',70235,'Reentered','2016-09-19 00:00:00','2016-09-18 00:00:00',90.49,186,411,42.61,'Intermediate' from dual union all
select 41779,'041779','2016-057E','CZ-2F sep motor cover','',70235,'Reentered','2016-09-19 00:00:00','2016-09-18 00:00:00',90.78,186,439,42.63,'Intermediate' from dual union all
select 4178,'004178','1969-082AW','deb Agena D','',18349,'In Earth orbit','','1992-02-02 00:00:00',101.68,761,916,69.84,'Intermediate' from dual union all
select 41780,'041780','2016-057F','CZ-2F sep motor cover','',70235,'Reentered','2016-09-18 00:00:00','2016-09-17 00:00:00',90.47,185,410,42.98,'Intermediate' from dual union all
select 41781,'041781','2016-057G','deb Tiangong-2','',70235,'Reentered','2016-09-16 00:00:00','2016-09-16 00:00:00',88.17,132,236,42.88,'Intermediate' from dual union all
select 41782,'041782','1998-067KQ','Flock 2e''-20','Dove 0C38',66503,'In Earth orbit','','2016-09-17 00:00:00',92.62,398,408,51.64,'Intermediate' from dual union all
select 41783,'041783','2016-059A','Pratham','Pratham',70237,'In Earth orbit','','2016-10-03 00:00:00',98.57,661,720,98.28,'Sun-Synch' from dual union all
select 41784,'041784','2016-059B','PISAT','PISAT',70237,'In Earth orbit','','2016-10-03 00:00:00',98.4,661,704,98.21,'Sun-Synch' from dual union all
select 41785,'041785','2016-059C','ALSAT-1B','DMC Alsat-1B',70237,'In Earth orbit','','2016-10-05 00:00:00',98.4,661,704,98.2,'Sun-Synch' from dual union all
select 41786,'041786','2016-059D','ALSAT-2B','ALSAT-2B',70237,'In Earth orbit','','2016-10-03 00:00:00',97.79,646,660,98.2,'Sun-Synch' from dual union all
select 41787,'041787','2016-059E','Pathfinder 1','BlackSky Pathfinder 1',70237,'In Earth orbit','','2016-10-03 00:00:00',98.38,661,701,98.21,'Sun-Synch' from dual union all
select 41788,'041788','2016-059F','CanX-7','CanX-7',70237,'In Earth orbit','','2016-10-03 00:00:00',98.37,661,701,98.21,'Sun-Synch' from dual union all
select 41789,'041789','2016-059G','ALSAT-1N','ALSAT-Nano',70237,'In Earth orbit','','2016-10-03 00:00:00',98.35,661,698,98.21,'Sun-Synch' from dual union all
select 4179,'004179','1969-082AX','deb Agena D','',18349,'Reentered','1982-10-13 00:00:00','1982-10-13 00:00:00',89.39,237,252,69.88,'Intermediate' from dual union all
select 41790,'041790','2016-059H','ScatSat','ScatSat',70237,'In Earth orbit','','2016-10-03 00:00:00',99.28,717,731,98.11,'Sun-Synch' from dual union all
select 41791,'041791','2016-059J','PSLV-C35 PS4','PS4',70237,'In Earth orbit','','2016-10-03 00:00:00',98.43,661,707,98.21,'Sun-Synch' from dual union all
select 41792,'041792','2016-059K','DLA','PSLV-C35-DLA',70237,'In Earth orbit','','2016-10-03 00:00:00',98.35,661,698,98.21,'Sun-Synch' from dual union all
select 41793,'041793','2016-060A','GSAT-18','GSAT-18',70238,'In Earth orbit','','2016-11-04 00:00:00',1436.06,35762,35810,.07,'Stationary' from dual union all
select 41794,'041794','2016-060B','Sky Muster 2','NBN Co. 2',70238,'In Earth orbit','','2016-11-05 00:00:00',1436.09,35785,35788,.01,'Stationary' from dual union all
select 41795,'041795','2016-060C','ESC-A 585','Ariane 5 ESC-A',70238,'In Earth orbit','','2016-11-05 00:00:00',630.13,256,35682,5.77,'GEO Transfer' from dual union all
select 41796,'041796','2016-060D','Sylda 5A','Sylda 5A',70238,'In Earth orbit','','2016-11-06 00:00:00',628.83,247,35624,6.04,'GEO Transfer' from dual union all
select 41797,'041797','2012-017C','deb RISAT-1','',69272,'Reentered','2016-10-12 00:00:00','2016-10-06 00:00:00',94.76,491,522,97.56,'Sun-Synch' from dual union all
select 41798,'041798','2014-016B','deb Sentinel-1A solar array','',69720,'In Earth orbit','','2016-10-10 00:00:00',97.57,594,690,98.2,'Sun-Synch' from dual union all
select 41799,'041799','2014-016C','deb Sentinel-1A solar array','',69720,'Reentered','2017-04-07 00:00:00','2016-10-10 00:00:00',98.48,661,711,98.19,'Sun-Synch' from dual union all
select 418,'000418','61 Omicron 138','deb r/b Ablestar','',7273,'Reentered','1991-11-05 00:00:00','1962-11-04 00:00:00',102.02,676,1033,67.75,'Intermediate' from dual union all
select 4180,'004180','1969-082AY','deb Agena D','',18349,'In Earth orbit','','1992-02-02 00:00:00',101.15,750,877,69.98,'Intermediate' from dual union all
select 41800,'041800','2014-016D','deb Sentinel-1A solar array','',69720,'Reentered','2017-12-12 00:00:00','2016-10-11 00:00:00',98.71,673,720,98.18,'Sun-Synch' from dual union all
select 41801,'041801','2014-016E','deb Sentinel-1A solar array','',69720,'Reentered','2017-03-26 00:00:00','2016-10-11 00:00:00',98.36,655,705,98.2,'Sun-Synch' from dual union all
select 41802,'041802','2014-016F','deb Sentinel-1A solar array','',69720,'Reentered','2017-03-31 00:00:00','2016-10-11 00:00:00',98.6,652,732,98.18,'Sun-Synch' from dual union all
select 41803,'041803','2014-016G','deb Sentinel-1A solar array','',69720,'Reentered','2017-09-19 00:00:00','2016-10-11 00:00:00',98.12,654,683,98.18,'Sun-Synch' from dual union all
select 41804,'041804','2014-016H','deb Sentinel-1A solar array','',69720,'In Earth orbit','','2016-10-11 00:00:00',97.7,605,692,98.2,'Sun-Synch' from dual union all
select 41805,'041805','2006-062K','deb SOZ','',68234,'In Earth orbit','','2016-10-10 00:00:00',339.68,388,19109,65.25,'Highly Ellip' from dual union all
select 41806,'041806','2006-062L','deb SOZ','',68234,'In Earth orbit','','2016-10-11 00:00:00',348.95,607,19474,64.26,'Highly Ellip' from dual union all
select 41807,'041807','2006-062M','deb SOZ','',68234,'In Earth orbit','','2016-10-11 00:00:00',337.71,395,18976,64.67,'Highly Ellip' from dual union all
select 41808,'041808','2006-062N','deb SOZ','',68234,'In Earth orbit','','2016-10-09 00:00:00',330.2,343,18551,64.64,'Highly Ellip' from dual union all
select 41809,'041809','2006-062P','deb SOZ','',68234,'Reentered','2017-04-26 00:00:00','2016-10-10 00:00:00',335.42,372,18855,64.15,'Highly Ellip' from dual union all
select 4181,'004181','1969-082AZ','deb Agena D','',18349,'Reentered','1989-01-10 00:00:00','1989-01-08 00:00:00',89.89,259,279,69.7,'Intermediate' from dual union all
select 41810,'041810','2006-062Q','deb SOZ','',68234,'In Earth orbit','','2016-10-10 00:00:00',335.44,333,18895,64.73,'Highly Ellip' from dual union all
select 41811,'041811','2006-062R','deb SOZ','',68234,'In Earth orbit','','2016-10-11 00:00:00',335.53,347,18887,64.65,'Highly Ellip' from dual union all
select 41812,'041812','2016-061A','Shenzhou 11','shenzhou shiyi hao',70239,'Landed','2016-11-18 00:00:00','2016-10-17 00:00:00',91.53,334,366,42.78,'Intermediate' from dual union all
select 41813,'041813','2016-061B','CZ-2F Stage 2','CZ-2F Stage 2',70239,'Reentered','2016-11-03 00:00:00','2016-10-17 00:00:00',90.06,197,358,42.79,'Intermediate' from dual union all
select 41814,'041814','2016-061C','CZ-2F sep motor cover','',70239,'Reentered','2016-10-19 00:00:00','2016-10-18 00:00:00',90.98,189,456,42.6,'Intermediate' from dual union all
select 41815,'041815','2016-061D','CZ-2F sep motor cover','',70239,'Reentered','2016-10-19 00:00:00','2016-10-18 00:00:00',90.93,189,452,42.65,'Intermediate' from dual union all
select 41816,'041816','2016-061E','CZ-2F sep motor cover','',70239,'Reentered','2016-10-21 00:00:00','2016-10-18 00:00:00',91.71,192,525,42.94,'Intermediate' from dual union all
select 41817,'041817','2016-061F','CZ-2F sep motor cover','',70239,'Reentered','2016-10-20 00:00:00','2016-10-18 00:00:00',91.6,189,518,42.89,'Intermediate' from dual union all
select 41818,'041818','2016-062A','SS Alan Poindexter','Cygnus PCM-5',70240,'Deorbited','2016-11-27 00:00:00','2016-10-18 00:00:00',90.11,209,351,51.61,'Intermediate' from dual union all
select 41819,'041819','2016-062B','Antares Stage 2','Castor 30B',70240,'Reentered','2016-11-06 00:00:00','2016-10-18 00:00:00',90.11,213,347,51.59,'Intermediate' from dual union all
select 4182,'004182','1969-093A','Kosmos-306','Zenit-2M',18360,'Landed','1969-11-05 00:00:00','1969-10-25 00:00:00',89.64,215,299,64.97,'Intermediate' from dual union all
select 41820,'041820','2016-063A','Soyuz MS-02','Soyuz 11F732A48 No. 732',70241,'Landed','2017-04-10 00:00:00','2016-10-20 00:00:00',90.45,287,306,51.64,'Intermediate' from dual union all
select 41821,'041821','2016-063B','11S510','11S510',70241,'Reentered','2016-10-23 00:00:00','2016-10-21 00:00:00',88.33,177,207,51.63,'Intermediate' from dual union all
select 41822,'041822','1999-025EXZ','deb FY-1C','',66706,'In Earth orbit','','2016-10-21 00:00:00',102.08,812,903,98.98,'Sun-Synch' from dual union all
select 41823,'041823','1999-025EYA','deb FY-1C','',66706,'In Earth orbit','','2016-10-21 00:00:00',102.1,812,905,98.98,'Sun-Synch' from dual union all
select 41824,'041824','1999-025EYB','deb FY-1C','',66706,'In Earth orbit','','2016-10-21 00:00:00',102.12,813,905,98.98,'Sun-Synch' from dual union all
select 41825,'041825','1999-025EYC','deb FY-1C','',66706,'In Earth orbit','','2016-10-20 00:00:00',102.08,810,904,98.98,'Sun-Synch' from dual union all
select 41826,'041826','1999-025EYD','deb FY-1C','',66706,'In Earth orbit','','2016-10-21 00:00:00',102.08,810,904,98.98,'Sun-Synch' from dual union all
select 41827,'041827','1999-025EYE','deb FY-1C','',66706,'In Earth orbit','','2016-10-21 00:00:00',102.07,810,904,98.98,'Sun-Synch' from dual union all
select 41828,'041828','1999-025EYF','deb FY-1C','',66706,'In Earth orbit','','2016-10-19 00:00:00',102.08,811,904,98.98,'Sun-Synch' from dual union all
select 41829,'041829','1999-025EYG','deb FY-1C','',66706,'In Earth orbit','','2016-10-21 00:00:00',102.07,810,904,98.98,'Sun-Synch' from dual union all
select 4183,'004183','1969-093B','11A57I','11A57I',18360,'Reentered','1969-10-29 00:00:00','1969-10-26 00:00:00',89.23,194,279,64.96,'Intermediate' from dual union all
select 41830,'041830','1999-025EYH','deb FY-1C','',66706,'In Earth orbit','','2016-10-20 00:00:00',102.08,811,904,98.98,'Sun-Synch' from dual union all
select 41831,'041831','1999-025EYJ','deb FY-1C','',66706,'In Earth orbit','','2016-10-21 00:00:00',102.08,811,904,98.98,'Sun-Synch' from dual union all
select 41832,'041832','1999-025EYK','deb FY-1C','',66706,'In Earth orbit','','2016-10-21 00:00:00',102.09,812,904,98.98,'Sun-Synch' from dual union all
select 41833,'041833','1999-025EYL','deb FY-1C','',66706,'In Earth orbit','','2016-10-21 00:00:00',102.09,812,904,98.98,'Sun-Synch' from dual union all
select 41834,'041834','2016-057H','BX-2','TG-2 bansui weixing',70235,'In Earth orbit','','2016-10-24 00:00:00',92.13,374,384,42.79,'Intermediate' from dual union all
select 41835,'041835','1994-057F','deb DMSP F-12','',65453,'In Earth orbit','','2016-10-24 00:00:00',101.61,822,848,99.12,'Sun-Synch' from dual union all
select 41836,'041836','2016-064A','Himawari-9','Himawari 9',70242,'In Earth orbit','','2017-01-01 00:00:00',1436.04,35782,35789,.01,'Stationary' from dual union all
select 41837,'041837','2016-064B','H-2A F31 Stage 2','H-2A Stage 2',70242,'In Earth orbit','','2017-01-02 00:00:00',619.64,257,35139,22.35,'GEO Transfer' from dual union all
select 41838,'041838','2016-065A','Shi Jian 17','SJ-17',70243,'In Earth orbit','','2017-01-02 00:00:00',1436.06,35757,35815,.89,'Stationary' from dual union all
select 41839,'041839','2016-065B','CZ-5 Stage 2','CZ-5 Stage 2',70243,'In Earth orbit','','2017-01-02 00:00:00',494.95,158,28540,19.26,'Highly Ellip' from dual union all
select 4184,'004184','1969-094A','Kosmos-307','DS-P1-Yu No. 22',18361,'Reentered','1970-12-30 00:00:00','1969-12-11 00:00:00',107.65,213,2023,48.43,'Intermediate' from dual union all
select 41840,'041840','2016-065C','Yuanzheng-2 Y1','YZ-2',70243,'In Earth orbit','','2017-01-02 00:00:00',1540.51,35643,39970,.51,'Very High' from dual union all
select 41841,'041841','2016-066A','Maichong Xing Shiyan Weixing','XPNAV-1',70244,'In Earth orbit','','2016-11-17 00:00:00',94.66,493,512,97.41,'Sun-Synch' from dual union all
select 41842,'041842','2016-066B','Xiaoxiang-1','Tianyi 1',70244,'In Earth orbit','','2016-11-17 00:00:00',94.63,491,511,97.4,'Sun-Synch' from dual union all
select 41843,'041843','2016-066C','Pina-2 01','Pegasus-1',70244,'In Earth orbit','','2016-11-14 00:00:00',94.64,491,512,97.4,'Sun-Synch' from dual union all
select 41844,'041844','2016-066D','Pina-2 02','Pina-2 02',70244,'In Earth orbit','','2016-11-14 00:00:00',94.64,491,512,97.4,'Sun-Synch' from dual union all
select 41845,'041845','2016-066E','Lishui 1-01','Lishui 1A',70244,'In Earth orbit','','2016-11-14 00:00:00',100.16,504,1028,98.78,'Sun-Synch' from dual union all
select 41846,'041846','2016-066F','deb Lishui','',70244,'In Earth orbit','','2016-11-14 00:00:00',100.23,505,1034,98.79,'Sun-Synch' from dual union all
select 41847,'041847','2016-066G','KS-1Q','Kechuang Weixing',70244,'In Earth orbit','','2016-11-14 00:00:00',99.82,503,996,98.78,'Sun-Synch' from dual union all
select 41848,'041848','2016-067A','WorldView-4','WorldView-4',70245,'In Earth orbit','','2016-11-18 00:00:00',96.92,610,613,97.97,'Sun-Synch' from dual union all
select 41849,'041849','2016-067B','RAVAN','RAVAN',70245,'In Earth orbit','','2016-11-19 00:00:00',96.26,573,585,97.97,'Sun-Synch' from dual union all
select 4185,'004185','1969-094B','S1M','S1M',18361,'Reentered','1970-07-20 00:00:00','1969-11-23 00:00:00',107.33,212,1995,48.41,'Intermediate' from dual union all
select 41850,'041850','2016-067C','CELTEE','CELTEE',70245,'In Earth orbit','','2016-11-18 00:00:00',96.26,573,585,97.97,'Sun-Synch' from dual union all
select 41851,'041851','2016-067D','Opticube 04','Optical CubeSat 4',70245,'In Earth orbit','','2016-11-19 00:00:00',96.25,573,585,97.97,'Sun-Synch' from dual union all
select 41852,'041852','2016-067E','Aerocube 8D','AC8D (IMPACT)',70245,'In Earth orbit','','2016-11-19 00:00:00',96.25,573,585,97.97,'Sun-Synch' from dual union all
select 41853,'041853','2016-067F','Aerocube 8C','AC8C (IMPACT)',70245,'In Earth orbit','','2016-11-19 00:00:00',96.22,572,583,97.97,'Sun-Synch' from dual union all
select 41854,'041854','2016-067G','Prometheus 2.1','Prometheus 2-1',70245,'In Earth orbit','','2016-11-19 00:00:00',96.2,572,581,97.97,'Sun-Synch' from dual union all
select 41855,'041855','2016-067H','Prometheus 2.3','Prometheus 2-3',70245,'In Earth orbit','','2016-11-19 00:00:00',96.22,572,583,97.98,'Sun-Synch' from dual union all
select 41856,'041856','2016-067J','Centaur AV-062','Centaur AV-062',70245,'Deep Space','2016-11-12 00:00:00','2016-11-11 00:00:00',269.88,570,40994,97.97,'Earth Escape' from dual union all
select 41857,'041857','2016-068A','Yunhai 1-01','Yunhai-1 01 xing',70246,'In Earth orbit','','2016-11-19 00:00:00',100.5,775,789,98.51,'Sun-Synch' from dual union all
select 41858,'041858','2016-068B','CZ-2D Stage 2','CZ-2D Stage 2',70246,'In Earth orbit','','2016-11-19 00:00:00',100.29,748,796,98.36,'Sun-Synch' from dual;
insert into SATELLITE
select 41859,'041859','2016-069A','GalileoSat-15','Galileo FOC FM07',70247,'In Earth orbit','','2016-12-18 00:00:00',846.19,23237,23277,54.6,'Medium' from dual union all
select 4186,'004186','1969-095A','OPS 8455','GAMBIT-3 Mission 4324',18362,'Reentered Attached','1969-11-08 00:00:00','1969-10-31 00:00:00',91.56,124,579,108,'Retrograde' from dual union all
select 41860,'041860','2016-069B','GalileoSat-16','Galileo FOC FM12',70247,'In Earth orbit','','2016-12-18 00:00:00',836.89,23031,23048,54.58,'Medium' from dual union all
select 41861,'041861','2016-069C','GalileoSat-17','Galileo FOC FM13',70247,'In Earth orbit','','2016-12-18 00:00:00',834.14,22965,22985,54.58,'Medium' from dual union all
select 41862,'041862','2016-069D','GalileoSat-18','Galileo FOC FM14',70247,'In Earth orbit','','2016-12-17 00:00:00',842.12,23150,23174,54.58,'Medium' from dual union all
select 41863,'041863','2016-069E','EPS 594','EPS+ 594 (VA233)',70247,'In Earth orbit','','2017-01-01 00:00:00',830.9,22896,22902,54.52,'Medium' from dual union all
select 41864,'041864','2016-070A','Soyuz MS-03','Soyuz 11F732A48 No. 733',70248,'Landed','2017-06-02 00:00:00','2016-11-19 00:00:00',90.89,317,320,51.64,'Intermediate' from dual union all
select 41865,'041865','2016-070B','11S510','11S510',70248,'Reentered','2016-11-21 00:00:00','2016-11-19 00:00:00',88.38,181,208,51.61,'Intermediate' from dual union all
select 41866,'041866','2016-071A','GOES 16','GOES R',70249,'In Earth orbit','','2016-12-20 00:00:00',1436.06,35781,35791,.01,'Stationary' from dual union all
select 41867,'041867','2016-071B','Centaur AV-069','Centaur AV-069',70249,'In Earth orbit','','2016-12-20 00:00:00',760.46,7524,34916,10.77,'Medium' from dual union all
select 41868,'041868','2016-061G','Shenzhou 11 Guidao Cang','shenzhou shiyi hao guida',70239,'In Earth orbit','','2016-11-18 00:00:00',92.16,375,386,42.78,'Intermediate' from dual union all
select 41869,'041869','2016-072A','Tian Lian 1-04','tianlian yihao 04 xing',70250,'In Earth orbit','','2016-12-23 00:00:00',1436.06,35781,35791,3.02,'Inclined GEO' from dual union all
select 4187,'004187','1969-082BA','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',99.86,725,778,69.95,'Intermediate' from dual union all
select 41870,'041870','2016-072B','CZ-3C Stage 3','CZ-3C Stage 3',70250,'In Earth orbit','','2016-12-23 00:00:00',710.41,155,39836,17.33,'GEO Transfer' from dual union all
select 41871,'041871','2016-062C','Lemur-2-Xiaoqing','Lemur-2-15',70240,'In Earth orbit','','2016-11-28 00:00:00',94.66,498,507,51.64,'Intermediate' from dual union all
select 41872,'041872','2016-062D','Lemur-2-Sokolsky','Lemur-2-14',70240,'In Earth orbit','','2016-11-28 00:00:00',94.66,498,507,51.64,'Intermediate' from dual union all
select 41873,'041873','2016-062E','Lemur-2-Anubhavthakur','Lemur-2-16',70240,'In Earth orbit','','2016-11-28 00:00:00',94.66,498,507,51.64,'Intermediate' from dual union all
select 41874,'041874','2016-062F','Lemur-2-Wingo','Lemur-2-17',70240,'In Earth orbit','','2016-11-28 00:00:00',94.66,498,507,51.64,'Intermediate' from dual union all
select 41875,'041875','2016-073A','Gokturk-1','Gokturk-1',70251,'In Earth orbit','','2016-12-06 00:00:00',98.42,683,683,98.12,'Sun-Synch' from dual union all
select 41876,'041876','2016-073B','Vega AVUM VV08','AVUM',70251,'Deorbited','2016-12-05 00:00:00','2016-12-05 00:00:00',98.33,672,686,98.15,'Sun-Synch' from dual union all
select 41877,'041877','2016-074A','Resourcesat-2A','Resourcesat-2A',70252,'In Earth orbit','','2016-12-09 00:00:00',101.34,814,831,98.73,'Sun-Synch' from dual union all
select 41878,'041878','2016-074B','PSLV-C36 PS4','PS4',70252,'In Earth orbit','','2016-12-08 00:00:00',101.16,812,816,98.81,'Sun-Synch' from dual union all
select 41879,'041879','2016-075A','WGS SV-8','Wideband Global Satcom 8',70253,'In Earth orbit','','2017-01-09 00:00:00',1435.12,28594,42941,.17,'Near-synch' from dual union all
select 4188,'004188','1969-082BB','deb Agena D','',18349,'Reentered','1980-12-24 00:00:00','1980-12-19 00:00:00',92.28,372,401,69.98,'Intermediate' from dual union all
select 41880,'041880','2016-075B','Delta 376','DCSS-5 F16',70253,'Deorbited','2016-12-08 00:00:00','2016-12-08 00:00:00',648.05,211,37068,27,'Suborbital' from dual union all
select 41881,'041881','2016-076A','Kounotori 6 gouki','HTV 6',70254,'Deorbited','2017-02-05 00:00:00','2016-12-10 00:00:00',90.29,276,302,51.65,'Intermediate' from dual union all
select 41882,'041882','2016-077A','Feng Yun 4A','fengyun sihao 01 xing',70255,'In Earth orbit','','2017-01-11 00:00:00',1436.19,35777,35800,.33,'Stationary' from dual union all
select 41883,'041883','2016-077B','CZ-3B Stage 3','CZ-3B Stage 3',70255,'In Earth orbit','','2017-01-09 00:00:00',591.35,142,33777,28.38,'Highly Ellip' from dual union all
select 41884,'041884','2016-078A','CYGNSS FM05','CYGNSS E',70256,'In Earth orbit','','2016-12-23 00:00:00',95.15,515,537,34.96,'Intermediate' from dual union all
select 41885,'041885','2016-078B','CYGNSS FM04','CYGNSS D',70256,'In Earth orbit','','2016-12-23 00:00:00',95.13,515,534,34.95,'Intermediate' from dual union all
select 41886,'041886','2016-078C','CYGNSS FM02','CYGNSS B',70256,'In Earth orbit','','2016-12-23 00:00:00',95.13,514,536,34.97,'Intermediate' from dual union all
select 41887,'041887','2016-078D','CYGNSS FM01','CYGNSS A',70256,'In Earth orbit','','2016-12-23 00:00:00',95.16,515,537,34.96,'Intermediate' from dual union all
select 41888,'041888','2016-078E','CYGNSS FM08','CYGNSS H',70256,'In Earth orbit','','2016-12-23 00:00:00',95.15,515,537,34.95,'Intermediate' from dual union all
select 41889,'041889','2016-078F','CYGNSS FM06','CYGNSS F',70256,'In Earth orbit','','2016-12-23 00:00:00',95.12,515,534,34.96,'Intermediate' from dual union all
select 4189,'004189','1969-082BC','deb Agena D','',18349,'Reentered','1980-12-15 00:00:00','1969-10-30 00:00:00',101.55,744,921,69.91,'Intermediate' from dual union all
select 41890,'041890','2016-078G','CYGNSS FM07','CYGNSS G',70256,'In Earth orbit','','2016-12-23 00:00:00',95.11,514,534,34.96,'Intermediate' from dual union all
select 41891,'041891','2016-078H','CYGNSS FM03','CYGNSS C',70256,'In Earth orbit','','2016-12-23 00:00:00',95.1,514,533,34.96,'Intermediate' from dual union all
select 41892,'041892','2016-078J','Orion 38','Orion 38',70256,'In Earth orbit','','2016-12-23 00:00:00',95.07,513,531,34.96,'Intermediate' from dual union all
select 41893,'041893','2016-079A','Echostar 19','Jupiter 2',70257,'In Earth orbit','','2016-12-26 00:00:00',2242.47,35692,65054,1.36,'Very High' from dual union all
select 41894,'041894','2016-079B','Centaur AV-071','Centaur AV-071',70257,'Reentered','2017-03-05 00:00:00','2016-12-27 00:00:00',1222.93,175,62831,25.75,'Highly Ellip' from dual union all
select 41895,'041895','1998-067KR','Hagoromo','STARS-C Oyaki',66503,'Reentered','2018-03-02 00:00:00','2016-12-19 00:00:00',92.62,400,406,51.64,'Intermediate' from dual union all
select 41896,'041896','2016-080A','Arase','ERG',70258,'In Earth orbit','','2017-01-24 00:00:00',568.05,429,32256,31.66,'Highly Ellip' from dual union all
select 41897,'041897','2016-080B','Epsilon Stage 3','KM-V2c',70258,'In Earth orbit','','2017-01-20 00:00:00',555.44,237,31773,31.41,'Highly Ellip' from dual union all
select 41898,'041898','2016-081A','Tan weixing','TanSat',70259,'In Earth orbit','','2016-12-29 00:00:00',98.86,690,718,98.15,'Sun-Synch' from dual union all
select 41899,'041899','2016-081B','Chao fenbianlu duoguangpu','Chuangxin-6 02 xing',70259,'In Earth orbit','','2016-12-29 00:00:00',98.87,690,720,98.15,'Sun-Synch' from dual union all
select 419,'000419','61 Omicron 139','deb r/b Ablestar','',7273,'Reentered','2014-12-09 00:00:00','1962-09-17 00:00:00',106.47,977,1149,66.91,'Intermediate' from dual union all
select 4190,'004190','1969-082BD','deb Agena D','',18349,'In Earth orbit','','1992-02-04 00:00:00',100.28,728,816,70.05,'Intermediate' from dual union all
select 41900,'041900','2016-081C','SPARK-01','Spark-01',70259,'In Earth orbit','','2016-12-29 00:00:00',98.94,690,725,98.15,'Sun-Synch' from dual union all
select 41901,'041901','2016-081D','SPARK-02','Spark-02',70259,'In Earth orbit','','2016-12-29 00:00:00',98.96,690,727,98.15,'Sun-Synch' from dual union all
select 41902,'041902','2016-081E','CZ-2D Stage 2','CZ-2D Stage 2',70259,'In Earth orbit','','2016-12-28 00:00:00',94.63,280,722,97.86,'Sun-Synch' from dual union all
select 41903,'041903','2016-082A','JCSAT 15','JCSAT 15',70260,'In Earth orbit','','2017-01-23 00:00:00',1441.63,35885,35905,.07,'Drift GEO' from dual union all
select 41904,'041904','2016-082B','Star One D1','Star One D1',70260,'In Earth orbit','','2017-01-23 00:00:00',1436.04,35780,35791,.06,'Stationary' from dual union all
select 41905,'041905','2016-082C','ESC-A 587','Ariane 5 ESC-A',70260,'In Earth orbit','','2017-01-21 00:00:00',630.69,248,35719,6.39,'GEO Transfer' from dual union all
select 41906,'041906','2016-082D','Sylda 5A','Sylda 5A',70260,'In Earth orbit','','2017-01-21 00:00:00',629.27,242,35652,5.94,'GEO Transfer' from dual union all
select 41907,'041907','2016-083A','GaoJing yi hao 01 xing','SuperView-1',70261,'In Earth orbit','','2016-12-30 00:00:00',93.35,352,525,97.6,'Sun-Synch' from dual union all
select 41908,'041908','2016-083B','GaoJing yi hao 02 xing','SuperView-2',70261,'In Earth orbit','','2016-12-30 00:00:00',93.49,343,548,97.59,'Sun-Synch' from dual union all
select 41909,'041909','2016-083C','Bayi kepu weixing 01 xing','BY70-1',70261,'Reentered','2017-02-18 00:00:00','2016-12-30 00:00:00',91.87,215,518,97.59,'Sun-Synch' from dual union all
select 4191,'004191','1969-082BE','deb Agena D','',18349,'Reentered','2005-11-19 00:00:00','1992-02-05 00:00:00',100.22,747,791,70.5,'Intermediate' from dual union all
select 41910,'041910','2016-083D','CZ-2D Stage 2','CZ-2D Stage 2',70261,'Reentered','2017-01-23 00:00:00','2016-12-30 00:00:00',91.75,212,508,97.59,'Sun-Synch' from dual union all
select 41911,'041911','2017-001A','Tongxin Jishu Shiyan 2','Huoyan-1',70391,'In Earth orbit','','2017-02-04 00:00:00',1436.14,35780,35795,.96,'Stationary' from dual union all
select 41912,'041912','2017-001B','CZ-3B Stage 3','CZ-3B Stage 3',70391,'In Earth orbit','','2017-02-05 00:00:00',589.94,159,33686,27.39,'Highly Ellip' from dual union all
select 41913,'041913','2017-002A','Xingyun Shiyan-1','Xingyun Shiyan-1',70392,'In Earth orbit','','2017-01-16 00:00:00',95.33,529,541,97.54,'Sun-Synch' from dual union all
select 41914,'041914','2017-002B','Jilin-1 Shipin 3','LSC Weixing 3',70392,'In Earth orbit','','2017-01-16 00:00:00',95.4,531,544,97.54,'Sun-Synch' from dual union all
select 41915,'041915','2017-002C','Kaidun-1','Caton-1',70392,'In Earth orbit','','2017-01-16 00:00:00',95.4,530,546,97.54,'Sun-Synch' from dual union all
select 41916,'041916','2017-002D','KZ-1A Stage 4','KZ-1A Stage 4',70392,'Reentered','2017-07-17 00:00:00','2017-01-16 00:00:00',92.31,250,526,97.54,'Sun-Synch' from dual union all
select 41917,'041917','2017-003A','Iridium Next SV106','Iridium Next SV106',70393,'In Earth orbit','','2017-01-24 00:00:00',98.82,701,704,86.53,'Polar' from dual union all
select 41918,'041918','2017-003B','Iridium Next SV103','Iridium Next SV103',70393,'In Earth orbit','','2017-01-22 00:00:00',97.03,610,623,86.67,'Polar' from dual union all
select 41919,'041919','2017-003C','Iridium Next SV109','Iridium Next SV109',70393,'In Earth orbit','','2017-01-22 00:00:00',97.02,610,622,86.67,'Polar' from dual union all
select 4192,'004192','1969-082BF','deb Agena D','',18349,'Reentered','1981-10-15 00:00:00','1981-10-12 00:00:00',91.07,315,340,70.12,'Intermediate' from dual union all
select 41920,'041920','2017-003D','Iridium Next SV102','Iridium Next SV102',70393,'In Earth orbit','','2017-01-22 00:00:00',97,609,621,86.67,'Polar' from dual union all
select 41921,'041921','2017-003E','Iridium Next SV105','Iridium Next SV105',70393,'In Earth orbit','','2017-01-22 00:00:00',97,608,622,86.67,'Polar' from dual union all
select 41922,'041922','2017-003F','Iridium Next SV104','Iridium Next SV104',70393,'In Earth orbit','','2017-01-22 00:00:00',96.99,608,622,86.66,'Polar' from dual union all
select 41923,'041923','2017-003G','Iridium Next SV114','Iridium Next SV114',70393,'In Earth orbit','','2017-01-22 00:00:00',96.99,608,622,86.66,'Polar' from dual union all
select 41924,'041924','2017-003H','Iridium Next SV108','Iridium Next SV108',70393,'In Earth orbit','','2017-01-22 00:00:00',96.99,608,622,86.66,'Polar' from dual union all
select 41925,'041925','2017-003J','Iridium Next SV112','Iridium Next SV112',70393,'In Earth orbit','','2017-01-22 00:00:00',96.98,607,622,86.66,'Polar' from dual union all
select 41926,'041926','2017-003K','Iridium Next SV111','Iridium Next SV111',70393,'In Earth orbit','','2017-01-22 00:00:00',96.98,607,622,86.67,'Polar' from dual union all
select 41927,'041927','1964-083S','deb VE-5','',10369,'In Earth orbit','','2017-01-14 00:00:00',96.98,607,622,86.68,'Polar' from dual union all
select 41928,'041928','2016-066H','CZ-11 sep motor cover','',70244,'In Earth orbit','','2017-02-10 00:00:00',101.58,528,1139,98.55,'Sun-Synch' from dual union all
select 41929,'041929','2015-019C','Yuanzheng-1 Y1','YZ-1',69957,'Deep Space','2015-04-01 00:00:00','2017-02-12 00:00:00',5157.3,27017,157986,41.17,'Deep Space' from dual union all
select 4193,'004193','1969-082BG','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',103.87,795,1088,69.95,'Intermediate' from dual union all
select 41930,'041930','1998-067KS','FREEDOM','FREEDOM',66503,'Reentered','2017-02-05 00:00:00','2017-01-17 00:00:00',92.56,395,405,51.64,'Intermediate' from dual union all
select 41931,'041931','1998-067KT','TANCREDO-1','TANCREDO-1',66503,'Reentered','2017-10-18 00:00:00','2017-01-17 00:00:00',92.6,396,408,51.64,'Intermediate' from dual union all
select 41932,'041932','1998-067KU','Tsukuba-OSCAR-89','ITF-2',66503,'In Earth orbit','','2017-01-17 00:00:00',92.61,397,408,51.64,'Intermediate' from dual union all
select 41933,'041933','1998-067KV','Waseda-SAT 3','Waseda-SAT 3',66503,'In Earth orbit','','2017-01-17 00:00:00',92.6,396,408,51.64,'Intermediate' from dual union all
select 41934,'041934','1998-067KW','EGG','EGG',66503,'Reentered','2017-05-15 00:00:00','2017-01-17 00:00:00',92.61,396,409,51.64,'Intermediate' from dual union all
select 41935,'041935','1998-067KX','AOBA-VeloxIII','AOBA-VeloxIII',66503,'In Earth orbit','','2017-01-17 00:00:00',92.61,398,407,51.64,'Intermediate' from dual union all
select 41936,'041936','1998-067KY','TuPOD','TuPOD',66503,'Reentered','2017-09-08 00:00:00','2017-01-20 00:00:00',92.6,396,408,51.64,'Intermediate' from dual union all
select 41937,'041937','2017-004A','SBIRS GEO-3','SBIRS GEO SV4',70394,'In Earth orbit','','2017-02-21 00:00:00',1436.11,35779,35795,.01,'Stationary' from dual union all
select 41938,'041938','2017-004B','Centaur AV-066','Centaur AV-066',70394,'In Earth orbit','','2017-01-21 00:00:00',606.82,133,34597,23.29,'GEO Transfer' from dual union all
select 41939,'041939','1998-067KZ','OSNSAT','OSNSAT',66503,'Reentered','2018-01-11 00:00:00','2017-01-21 00:00:00',92.58,394,408,51.64,'Intermediate' from dual union all
select 4194,'004194','1969-082BH','deb Agena D','',18349,'Reentered','1982-10-12 00:00:00','1982-10-12 00:00:00',89.24,224,249,69.29,'Intermediate' from dual union all
select 41940,'041940','2017-005A','Kirameki 2','DSN 2',70395,'In Earth orbit','','2017-01-24 00:00:00',615.66,190,35000,19,'GEO Transfer' from dual union all
select 41941,'041941','2017-005B','H-2A F32 Stage 2','H-2A Stage 2',70395,'In Earth orbit','','2017-01-24 00:00:00',615.66,190,35000,19,'GEO Transfer' from dual union all
select 41942,'041942','2017-006A','Hispasat 36W-1','Hispasat AG1',70396,'In Earth orbit','','2017-02-28 00:00:00',1436.11,35781,35793,.06,'Stationary' from dual union all
select 41943,'041943','2017-006B','Fregat-MT No. 133-07','Fregat-MT    No. 133-07',70396,'In Earth orbit','','2017-02-05 00:00:00',626.02,180,35546,5.44,'GEO Transfer' from dual union all
select 41944,'041944','2017-007A','Telkom-3S','Telkom 3S',70397,'In Earth orbit','','2017-02-21 00:00:00',1429.25,35527,35778,.03,'Drift GEO' from dual union all
select 41945,'041945','2017-007B','SkyBrasil-1','Intelsat IS-32e',70397,'In Earth orbit','','2017-02-22 00:00:00',1430.93,35619,35751,.09,'Drift GEO' from dual union all
select 41946,'041946','2017-007C','ESC-A 588','Ariane 5 ESC-A',70397,'In Earth orbit','','2017-02-22 00:00:00',632.17,262,35781,3.87,'GEO Transfer' from dual union all
select 41947,'041947','2017-007D','Sylda 5A','Sylda 5A',70397,'In Earth orbit','','2017-02-22 00:00:00',630.71,259,35709,4.01,'GEO Transfer' from dual union all
select 41948,'041948','2017-008A','Cartosat-2 Series Satellite','Cartosat-2D',70398,'In Earth orbit','','2017-02-15 00:00:00',94.6,491,508,97.51,'Sun-Synch' from dual;
insert into SATELLITE
select 41949,'041949','2017-008B','INS-1A','INS-1A',70398,'In Earth orbit','','2017-02-17 00:00:00',94.65,496,508,97.51,'Sun-Synch' from dual union all
select 4195,'004195','1969-082BJ','deb Agena D','',18349,'Reentered','2012-01-05 00:00:00','1992-02-05 00:00:00',100,729,788,70.05,'Intermediate' from dual union all
select 41950,'041950','2017-008C','Flock 3p-20','Dove 1029',70398,'In Earth orbit','','2017-02-24 00:00:00',94.57,495,501,97.51,'Sun-Synch' from dual union all
select 41951,'041951','2017-008D','Flock 3p-8','Dove 100C',70398,'In Earth orbit','','2017-02-17 00:00:00',94.63,496,506,97.51,'Sun-Synch' from dual union all
select 41952,'041952','2017-008E','Flock 3p-51','Dove 1006',70398,'In Earth orbit','','2017-02-17 00:00:00',94.59,494,504,97.51,'Sun-Synch' from dual union all
select 41953,'041953','2017-008F','Flock 3p-37','Dove 1037',70398,'In Earth orbit','','2017-02-19 00:00:00',94.55,497,497,97.51,'Sun-Synch' from dual union all
select 41954,'041954','2017-008G','INS-1B','INS-1B',70398,'In Earth orbit','','2017-02-17 00:00:00',94.65,496,508,97.51,'Sun-Synch' from dual union all
select 41955,'041955','2017-008H','Flock 3p-19','Dove 1024',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41956,'041956','2017-008J','Flock 3p-24','Dove 1026',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41957,'041957','2017-008K','Flock 3p-18','Dove 102B',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41958,'041958','2017-008L','Flock 3p-22','Dove 1028',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41959,'041959','2017-008M','Flock 3p-21','Dove 102A',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 4196,'004196','1969-082BK','deb Agena D','',18349,'Reentered','1973-05-07 00:00:00','1969-11-05 00:00:00',101.01,688,924,69.71,'Intermediate' from dual union all
select 41960,'041960','2017-008N','Flock 3p-28','Dove 102C',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41961,'041961','2017-008P','Flock 3p-26','Dove 102D',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41962,'041962','2017-008Q','Flock 3p-17','Dove 1023',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41963,'041963','2017-008R','Flock 3p-27','Dove 1030',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41964,'041964','2017-008S','Flock 3p-25','Dove 102F',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41965,'041965','2017-008T','Flock 3p-4','Dove 1003',70398,'In Earth orbit','','2017-02-21 00:00:00',94.63,496,506,97.51,'Sun-Synch' from dual union all
select 41966,'041966','2017-008U','Flock 3p-2','Dove 1001',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41967,'041967','2017-008V','Flock 3p-1','Dove 1000',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41968,'041968','2017-008W','Flock 3p-3','Dove 1002',70398,'In Earth orbit','','2017-02-21 00:00:00',94.63,496,506,97.51,'Sun-Synch' from dual union all
select 41969,'041969','2017-008X','Flock 3p-6','Dove 0F35',70398,'In Earth orbit','','2017-02-24 00:00:00',94.63,496,506,97.51,'Sun-Synch' from dual union all
select 4197,'004197','1969-082BL','deb Agena D','',18349,'Reentered','1980-04-02 00:00:00','1980-04-01 00:00:00',91.39,317,369,70.36,'Intermediate' from dual union all
select 41970,'041970','2017-008Y','Flock 3p-7','Dove 100B',70398,'In Earth orbit','','2017-02-24 00:00:00',94.63,496,506,97.51,'Sun-Synch' from dual union all
select 41971,'041971','2017-008Z','Flock 3p-5','Dove 0F18',70398,'In Earth orbit','','2017-02-24 00:00:00',94.63,496,506,97.51,'Sun-Synch' from dual union all
select 41972,'041972','2017-008AA','Flock 3p-12','Dove 1010',70398,'In Earth orbit','','2017-02-24 00:00:00',94.63,496,506,97.51,'Sun-Synch' from dual union all
select 41973,'041973','2017-008AB','Flock 3p-9','Dove 1021',70398,'In Earth orbit','','2017-02-24 00:00:00',94.63,496,506,97.51,'Sun-Synch' from dual union all
select 41974,'041974','2017-008AC','Flock 3p-10','Dove 1022',70398,'In Earth orbit','','2017-02-24 00:00:00',94.63,496,506,97.51,'Sun-Synch' from dual union all
select 41975,'041975','2017-008AD','Flock 3p-11','Dove 101F',70398,'In Earth orbit','','2017-02-24 00:00:00',94.63,496,506,97.51,'Sun-Synch' from dual union all
select 41976,'041976','2017-008AE','Flock 3p-60','Dove 101B',70398,'In Earth orbit','','2017-02-24 00:00:00',94.63,496,506,97.51,'Sun-Synch' from dual union all
select 41977,'041977','2017-008AF','Flock 3p-58','Dove 1009',70398,'In Earth orbit','','2017-02-24 00:00:00',94.62,495,506,97.51,'Sun-Synch' from dual union all
select 41978,'041978','2017-008AG','Flock 3p-57','Dove 1014',70398,'In Earth orbit','','2017-02-24 00:00:00',94.63,496,506,97.51,'Sun-Synch' from dual union all
select 41979,'041979','2017-008AH','Flock 3p-75','Dove 0F12',70398,'In Earth orbit','','2017-02-24 00:00:00',94.62,495,506,97.51,'Sun-Synch' from dual union all
select 4198,'004198','1969-082BM','deb Agena D','',18349,'Reentered','1981-04-08 00:00:00','1981-04-06 00:00:00',90.11,268,292,69.99,'Intermediate' from dual union all
select 41980,'041980','2017-008AJ','Flock 3p-70','Dove 0F15',70398,'In Earth orbit','','2017-02-24 00:00:00',94.62,495,506,97.52,'Sun-Synch' from dual union all
select 41981,'041981','2017-008AK','Flock 3p-73','Dove 0F1B',70398,'In Earth orbit','','2017-02-24 00:00:00',94.62,496,505,97.51,'Sun-Synch' from dual union all
select 41982,'041982','2017-008AL','Flock 3p-88','Dove 0F38',70398,'In Earth orbit','','2017-02-24 00:00:00',94.62,495,506,97.51,'Sun-Synch' from dual union all
select 41983,'041983','2017-008AM','Flock 3p-85','Dove 0F1D',70398,'In Earth orbit','','2017-02-24 00:00:00',94.61,495,505,97.51,'Sun-Synch' from dual union all
select 41984,'041984','2017-008AN','Flock 3p-79','Dove 0F52',70398,'In Earth orbit','','2017-02-24 00:00:00',94.62,496,505,97.51,'Sun-Synch' from dual union all
select 41985,'041985','2017-008AP','Flock 3p-86','Dove 0F34',70398,'In Earth orbit','','2017-02-24 00:00:00',94.61,495,505,97.51,'Sun-Synch' from dual union all
select 41986,'041986','2017-008AQ','Flock 3p-36','Dove 103A',70398,'In Earth orbit','','2017-02-24 00:00:00',94.62,496,505,97.51,'Sun-Synch' from dual union all
select 41987,'041987','2017-008AR','Flock 3p-30','Dove 102E',70398,'In Earth orbit','','2017-02-24 00:00:00',94.62,496,505,97.51,'Sun-Synch' from dual union all
select 41988,'041988','2017-008AS','Flock 3p-34','Dove 103B',70398,'In Earth orbit','','2017-02-24 00:00:00',94.62,496,505,97.51,'Sun-Synch' from dual union all
select 41989,'041989','2017-008AT','Flock 3p-35','Dove 103D',70398,'In Earth orbit','','2017-02-24 00:00:00',94.62,496,505,97.51,'Sun-Synch' from dual union all
select 4199,'004199','1969-082BN','deb Agena D','',18349,'Reentered','1981-08-31 00:00:00','1981-08-24 00:00:00',91.95,365,376,70.01,'Intermediate' from dual union all
select 41990,'041990','2017-008AU','Flock 3p-33','Dove 103C',70398,'In Earth orbit','','2017-02-24 00:00:00',94.61,495,505,97.51,'Sun-Synch' from dual union all
select 41991,'041991','2017-008AV','Lemur-2-Satchmo','Lemur-2-24',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41992,'041992','2017-008AW','Lemur-2-Mia-Grace','Lemur-2-27',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41993,'041993','2017-008AX','Lemur-2-Smita-Sharad','Lemur-2-26',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41994,'041994','2017-008AY','Lemur-2-Spire-Minions','Lemur-2-23',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41995,'041995','2017-008AZ','Lemur-2-RDeaton','Lemur-2-25',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41996,'041996','2017-008BA','Lemur-2-NoguesCorreig','Lemur-2-28',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41997,'041997','2017-008BB','Lemur-2-Jobanputra','Lemur-2-22',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41998,'041998','2017-008BC','Lemur-2-Tachikoma','Lemur-2-29',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 41999,'041999','2017-008BD','BGUSat','BGUSat',70398,'In Earth orbit','','2017-02-17 00:00:00',94.63,496,506,97.51,'Sun-Synch' from dual union all
select 42,'000042','60 Epsilon 9','Korabl''-Sputnik part','Vostok-1P part',6320,'Reentered','1960-10-01 00:00:00','1960-07-02 00:00:00',93.22,282,582,64.89,'Intermediate' from dual union all
select 420,'000420','61 Omicron 140','deb r/b Ablestar','',7273,'In Earth orbit','','1962-11-11 00:00:00',102.84,802,984,66.84,'Intermediate' from dual union all
select 4200,'004200','1969-082BP','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',100.34,734,816,70.11,'Intermediate' from dual union all
select 42000,'042000','2017-008BE','Dido-2','Dido-2',70398,'In Earth orbit','','2017-02-21 00:00:00',94.65,496,508,97.51,'Sun-Synch' from dual union all
select 42001,'042001','2017-008BF','Flock 3p-49','Dove 1038',70398,'In Earth orbit','','2017-02-24 00:00:00',94.61,495,505,97.51,'Sun-Synch' from dual union all
select 42002,'042002','2017-008BG','Flock 3p-67','Dove 1017',70398,'In Earth orbit','','2017-02-24 00:00:00',94.61,495,505,97.51,'Sun-Synch' from dual union all
select 42003,'042003','2017-008BH','Flock 3p-68','Dove 1004',70398,'In Earth orbit','','2017-02-24 00:00:00',94.61,495,505,97.51,'Sun-Synch' from dual union all
select 42004,'042004','2017-008BJ','Flock 3p-41','Dove 1031',70398,'In Earth orbit','','2017-02-24 00:00:00',94.6,495,503,97.51,'Sun-Synch' from dual union all
select 42005,'042005','2017-008BK','Flock 3p-45','Dove 101A',70398,'In Earth orbit','','2017-02-24 00:00:00',94.6,495,503,97.51,'Sun-Synch' from dual union all
select 42006,'042006','2017-008BL','Flock 3p-48','Dove 1042',70398,'In Earth orbit','','2017-02-24 00:00:00',94.6,495,503,97.51,'Sun-Synch' from dual union all
select 42007,'042007','2017-008BM','Flock 3p-43','Dove 1041',70398,'In Earth orbit','','2017-02-24 00:00:00',94.6,495,503,97.51,'Sun-Synch' from dual union all
select 42008,'042008','2017-008BN','Flock 3p-42','Dove 101E',70398,'In Earth orbit','','2017-02-24 00:00:00',94.59,494,504,97.51,'Sun-Synch' from dual union all
select 42009,'042009','2017-008BP','Flock 3p-61','Dove 1012',70398,'In Earth orbit','','2017-02-24 00:00:00',94.59,494,504,97.51,'Sun-Synch' from dual union all
select 4201,'004201','1969-082BQ','deb Agena D','',18349,'Reentered','2002-04-18 00:00:00','1992-02-02 00:00:00',99.25,674,772,70.01,'Intermediate' from dual union all
select 42010,'042010','2017-008BQ','Flock 3p-40','Dove 1039',70398,'In Earth orbit','','2017-02-24 00:00:00',94.58,494,502,97.51,'Sun-Synch' from dual union all
select 42011,'042011','2017-008BR','Flock 3p-16','Dove 100E',70398,'In Earth orbit','','2017-02-24 00:00:00',94.58,494,502,97.51,'Sun-Synch' from dual union all
select 42012,'042012','2017-008BS','Flock 3p-14','Dove 1018',70398,'In Earth orbit','','2017-02-24 00:00:00',94.59,495,503,97.51,'Sun-Synch' from dual union all
select 42013,'042013','2017-008BT','Flock 3p-53','Dove 1007',70398,'In Earth orbit','','2017-02-24 00:00:00',94.59,495,503,97.51,'Sun-Synch' from dual union all
select 42014,'042014','2017-008BU','Flock 3p-54','Dove 1005',70398,'In Earth orbit','','2017-02-24 00:00:00',94.58,494,502,97.51,'Sun-Synch' from dual union all
select 42015,'042015','2017-008BV','PEASSS','PEASSS',70398,'In Earth orbit','','2017-02-17 00:00:00',94.59,494,504,97.51,'Sun-Synch' from dual union all
select 42016,'042016','2017-008BW','Al-Farabi 1','Al-Farabi 1',70398,'In Earth orbit','','2017-02-19 00:00:00',94.55,497,497,97.51,'Sun-Synch' from dual union all
select 42017,'042017','2017-008BX','Emirates OSCAR 88','Nayif-1',70398,'In Earth orbit','','2017-02-21 00:00:00',94.64,496,507,97.51,'Sun-Synch' from dual union all
select 42018,'042018','2017-008BY','Flock 3p-23','Dove 1025',70398,'In Earth orbit','','2017-03-01 00:00:00',94.63,495,506,97.51,'Sun-Synch' from dual union all
select 42019,'042019','2017-008BZ','Flock 3p-76','Dove 0F17',70398,'In Earth orbit','','2017-03-01 00:00:00',94.62,495,506,97.51,'Sun-Synch' from dual union all
select 4202,'004202','1969-082BR','deb Agena D','',18349,'Reentered','1980-06-08 00:00:00','1969-10-31 00:00:00',101.83,751,940,70.23,'Intermediate' from dual union all
select 42020,'042020','2017-008CA','Flock 3p-69','Dove 0F43',70398,'In Earth orbit','','2017-03-01 00:00:00',94.62,495,506,97.51,'Sun-Synch' from dual union all
select 42021,'042021','2017-008CB','Flock 3p-84','Dove 0F42',70398,'In Earth orbit','','2017-03-01 00:00:00',94.62,495,506,97.51,'Sun-Synch' from dual union all
select 42022,'042022','2017-008CC','Flock 3p-59','Dove 1045',70398,'In Earth orbit','','2017-03-01 00:00:00',94.62,495,506,97.51,'Sun-Synch' from dual union all
select 42023,'042023','2017-008CD','Flock 3p-32','Dove 1035',70398,'In Earth orbit','','2017-03-01 00:00:00',94.62,495,506,97.51,'Sun-Synch' from dual union all
select 42024,'042024','2017-008CE','Flock 3p-71','Dove 0F10',70398,'In Earth orbit','','2017-03-09 00:00:00',94.62,495,506,97.51,'Sun-Synch' from dual union all
select 42025,'042025','2017-008CF','Flock 3p-77','Dove 0F28',70398,'In Earth orbit','','2017-03-01 00:00:00',94.62,495,506,97.51,'Sun-Synch' from dual union all
select 42026,'042026','2017-008CG','Flock 3p-80','Dove 0F4E',70398,'In Earth orbit','','2017-03-01 00:00:00',94.62,495,506,97.51,'Sun-Synch' from dual union all
select 42027,'042027','2017-008CH','Flock 3p-66','Dove 1013',70398,'In Earth orbit','','2017-03-01 00:00:00',94.6,495,503,97.51,'Sun-Synch' from dual union all
select 42028,'042028','2017-008CJ','Flock 3p-65','Dove 1040',70398,'In Earth orbit','','2017-03-01 00:00:00',94.6,494,505,97.51,'Sun-Synch' from dual union all
select 42029,'042029','2017-008CK','Flock 3p-50','Dove 1034',70398,'In Earth orbit','','2017-03-01 00:00:00',94.6,494,505,97.51,'Sun-Synch' from dual union all
select 4203,'004203','1969-082BS','deb Agena D','',18349,'Reentered','1982-01-08 00:00:00','1969-10-31 00:00:00',102.13,800,919,70.22,'Intermediate' from dual union all
select 42030,'042030','2017-008CL','Flock 3p-52','Dove 1044',70398,'In Earth orbit','','2017-03-01 00:00:00',94.6,494,505,97.51,'Sun-Synch' from dual union all
select 42031,'042031','2017-008CM','Flock 3p-46','Dove 1011',70398,'In Earth orbit','','2017-03-01 00:00:00',94.59,494,504,97.51,'Sun-Synch' from dual union all
select 42032,'042032','2017-008CN','Flock 3p-47','Dove 1027',70398,'In Earth orbit','','2017-03-01 00:00:00',94.59,494,504,97.51,'Sun-Synch' from dual union all
select 42033,'042033','2017-008CP','Flock 3p-44','Dove 1019',70398,'In Earth orbit','','2017-03-01 00:00:00',94.59,494,504,97.51,'Sun-Synch' from dual union all
select 42034,'042034','2017-008CQ','Flock 3p-64','Dove 100A',70398,'In Earth orbit','','2017-03-01 00:00:00',94.59,494,504,97.51,'Sun-Synch' from dual union all
select 42035,'042035','2017-008CR','Flock 3p-63','Dove 1015',70398,'In Earth orbit','','2017-03-01 00:00:00',94.58,494,502,97.51,'Sun-Synch' from dual union all
select 42036,'042036','2017-008CS','Flock 3p-62','Dove 1046',70398,'In Earth orbit','','2017-03-01 00:00:00',94.59,494,504,97.51,'Sun-Synch' from dual union all
select 42037,'042037','2017-008CT','Flock 3p-38','Dove 1036',70398,'In Earth orbit','','2017-03-01 00:00:00',94.59,494,504,97.51,'Sun-Synch' from dual;
insert into SATELLITE
select 42038,'042038','2017-008CU','Flock 3p-39','Dove 103E',70398,'In Earth orbit','','2017-03-01 00:00:00',94.58,494,502,97.51,'Sun-Synch' from dual union all
select 42039,'042039','2017-008CV','Flock 3p-15','Dove 101D',70398,'In Earth orbit','','2017-02-24 00:00:00',94.58,495,502,97.51,'Sun-Synch' from dual union all
select 4204,'004204','1969-082BT','deb Agena D','',18349,'Reentered','1979-09-08 00:00:00','1979-08-19 00:00:00',93.06,408,441,69.87,'Intermediate' from dual union all
select 42040,'042040','2017-008CW','Flock 3p-13','Dove 1016',70398,'In Earth orbit','','2017-02-24 00:00:00',94.58,495,502,97.51,'Sun-Synch' from dual union all
select 42041,'042041','2017-008CX','Flock 3p-55','Dove 1008',70398,'In Earth orbit','','2017-03-01 00:00:00',94.58,494,502,97.51,'Sun-Synch' from dual union all
select 42042,'042042','2017-008CY','Flock 3p-56','Dove 100F',70398,'In Earth orbit','','2017-03-01 00:00:00',94.58,494,502,97.51,'Sun-Synch' from dual union all
select 42043,'042043','2017-008CZ','Flock 3p-81','Dove 0F25',70398,'In Earth orbit','','2017-03-09 00:00:00',94.61,494,506,97.51,'Sun-Synch' from dual union all
select 42044,'042044','2017-008DA','Flock 3p-87','Dove 0F31',70398,'In Earth orbit','','2017-03-09 00:00:00',94.61,494,506,97.51,'Sun-Synch' from dual union all
select 42045,'042045','2017-008DB','Flock 3p-29','Dove 1033',70398,'In Earth orbit','','2017-02-21 00:00:00',94.65,496,508,97.51,'Sun-Synch' from dual union all
select 42046,'042046','2017-008DC','Flock 3p-82','Dove 0F41',70398,'In Earth orbit','','2017-03-09 00:00:00',94.61,494,506,97.51,'Sun-Synch' from dual union all
select 42047,'042047','2017-008DD','Flock 3p-78','Dove 0F51',70398,'In Earth orbit','','2017-03-09 00:00:00',94.62,494,507,97.51,'Sun-Synch' from dual union all
select 42048,'042048','2017-008DE','Flock 3p-74','Dove 0F22',70398,'In Earth orbit','','2017-03-09 00:00:00',94.61,494,506,97.51,'Sun-Synch' from dual union all
select 42049,'042049','2017-008DF','Flock 3p-31','Dove 1032',70398,'In Earth orbit','','2017-03-09 00:00:00',94.61,494,506,97.51,'Sun-Synch' from dual union all
select 4205,'004205','1969-082BU','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',101.5,778,882,70.06,'Intermediate' from dual union all
select 42050,'042050','2017-008DG','Flock 3p-83','Dove 0F3F',70398,'In Earth orbit','','2017-03-09 00:00:00',94.62,494,507,97.51,'Sun-Synch' from dual union all
select 42051,'042051','2017-008DH','Flock 3p-72','Dove 0F11',70398,'In Earth orbit','','2017-03-01 00:00:00',94.62,495,506,97.51,'Sun-Synch' from dual union all
select 42052,'042052','2017-008DJ','PS4','PS4 PSLV-C37',70398,'In Earth orbit','','2017-02-15 00:00:00',94.28,474,493,97.52,'Sun-Synch' from dual union all
select 42053,'042053','2017-009A','Dragon CRS-10','Dragon C112',70399,'Landed','2017-03-19 00:00:00','2017-02-21 00:00:00',91.75,357,363,51.63,'Intermediate' from dual union all
select 42054,'042054','2017-009B','Dragon solar array cover','',70399,'Reentered','2017-02-23 00:00:00','2017-02-21 00:00:00',89.49,197,302,51.65,'Intermediate' from dual union all
select 42055,'042055','2017-009C','Dragon solar array cover','',70399,'Reentered','2017-02-23 00:00:00','2017-02-21 00:00:00',89.49,194,305,51.62,'Intermediate' from dual union all
select 42056,'042056','2017-010A','Progress MS-05','Progress 7K-TGM No. 435',70400,'Deorbited','2017-07-20 00:00:00','2017-02-23 00:00:00',88.1,173,188,51.6,'Intermediate' from dual union all
select 42057,'042057','2017-010B','11S510','11S510',70400,'Reentered','2017-02-24 00:00:00','2017-02-23 00:00:00',90.61,295,314,51.64,'Intermediate' from dual union all
select 42058,'042058','2017-011A','USA 274','[INTRUDER 15]',70401,'In Earth orbit','','2017-03-07 00:00:00',107.4,1009,1204,63.45,'Intermediate' from dual union all
select 42059,'042059','1998-067LA','Lemur-2-Redfern-Goes','Lemur-2-21',66503,'In Earth orbit','','2017-03-08 00:00:00',92.61,397,408,51.64,'Intermediate' from dual union all
select 4206,'004206','1969-082BV','deb Agena D','',18349,'In Earth orbit','','1992-02-02 00:00:00',100.48,727,836,69.95,'Intermediate' from dual union all
select 42060,'042060','1982-092C','deb Kosmos-1408','',54401,'Reentered','2017-02-23 00:00:00','2017-02-17 00:00:00',91.07,320,335,82.55,'Intermediate' from dual union all
select 42061,'042061','2017-012A','Tiankun-1','Tiankun-1',70402,'In Earth orbit','','2017-03-05 00:00:00',92.45,384,406,96.93,'Sun-Synch' from dual union all
select 42062,'042062','2017-012B','KT-2 Stage 3','KT-2 Stage 3',70402,'Reentered','2017-04-08 00:00:00','2017-03-05 00:00:00',90.61,216,392,96.67,'Sun-Synch' from dual union all
select 42063,'042063','2017-013A','Sentinel-2B','Sentinel-2B',70403,'In Earth orbit','','2017-03-07 00:00:00',100.41,777,779,98.56,'Sun-Synch' from dual union all
select 42064,'042064','2017-013B','Vega AVUM VV09','AVUM',70403,'Deorbited','2017-03-07 00:00:00','2017-03-07 00:00:00',100.48,770,792,98.54,'Sun-Synch' from dual union all
select 42065,'042065','2017-011B','USA 274 P/L 2','[INTRUDER 16]',70401,'In Earth orbit','','2017-03-07 00:00:00',107.39,1009,1203,63.45,'Intermediate' from dual union all
select 42066,'042066','1998-067LB','TechEdSat-5','TES-5',66503,'Reentered','2017-07-29 00:00:00','2017-03-08 00:00:00',92.61,397,408,51.64,'Intermediate' from dual union all
select 42067,'042067','1998-067LC','Lemur-2-Trutna','Lemur-2-18',66503,'Reentered','2018-04-15 00:00:00','2017-03-11 00:00:00',92.6,396,408,51.64,'Intermediate' from dual union all
select 42068,'042068','1998-067LD','Lemur-2-Austintacious','Lemur-2-20',66503,'In Earth orbit','','2017-03-08 00:00:00',92.6,396,408,51.63,'Intermediate' from dual union all
select 42069,'042069','1998-067LE','Lemur-2-TrutnaHD','Lemur-2-19',66503,'In Earth orbit','','2017-03-08 00:00:00',92.59,397,406,51.64,'Intermediate' from dual union all
select 4207,'004207','1969-082BW','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',100.77,760,831,69.98,'Intermediate' from dual union all
select 42070,'042070','2017-014A','Echostar 23','Echostar 23',70404,'In Earth orbit','','2017-03-18 00:00:00',706.72,3924,35883,13.69,'GEO Transfer' from dual union all
select 42071,'042071','2017-014B','Falcon 9-031 Stage 2','F9-031 St2',70404,'In Earth orbit','','2017-03-19 00:00:00',631.54,176,35834,22.44,'GEO Transfer' from dual union all
select 42072,'042072','2017-015A','JSE reda-5 gouki','IGS Radar-5',70405,'In Earth orbit','','2017-03-30 00:00:00',94.45,485,499,97.23,'Sun-Synch' from dual union all
select 42073,'042073','2017-015B','H-2A F33 Stage 2','H-2A Stage 2',70405,'In Earth orbit','','2017-03-30 00:00:00',94.45,485,499,97.23,'Sun-Synch' from dual union all
select 42074,'042074','2017-015C','deb IGS Radar-5','',70405,'Reentered','2017-05-13 00:00:00','2017-03-30 00:00:00',94.45,485,499,97.23,'Sun-Synch' from dual union all
select 42075,'042075','2017-016A','WGS SV-9','Wideband Global Satcom 9',70406,'In Earth orbit','','2017-03-19 00:00:00',809.69,433,44364,27,'Highly Ellip' from dual union all
select 42076,'042076','2017-016B','Delta 377','DCSS-5 F17',70406,'Deorbited','2017-03-19 00:00:00','2017-03-19 00:00:00',709.36,140,40080,27,'Suborbital' from dual union all
select 42077,'042077','2017-012C','deb TK-1','',70402,'Reentered','2017-05-21 00:00:00','2017-03-21 00:00:00',92.59,387,415,96.95,'Sun-Synch' from dual union all
select 42078,'042078','1968-114E','deb Delta 62','',16841,'In Earth orbit','','2016-11-12 00:00:00',115.21,1444,1486,101.57,'Sun-Synch' from dual union all
select 42079,'042079','1968-114F','deb Delta 62','',16841,'In Earth orbit','','2017-03-21 00:00:00',117.46,1316,1818,102.11,'Sun-Synch' from dual union all
select 4208,'004208','1969-082BX','deb Agena D','',18349,'In Earth orbit','','1972-06-09 00:00:00',102.84,893,893,70,'Intermediate' from dual union all
select 42080,'042080','1968-114G','deb Delta 62','',16841,'In Earth orbit','','2017-03-26 00:00:00',116.19,1456,1563,101.58,'Sun-Synch' from dual union all
select 42081,'042081','1968-114H','deb Delta 62','',16841,'In Earth orbit','','2017-03-08 00:00:00',116.92,1450,1634,101.75,'Sun-Synch' from dual union all
select 42082,'042082','1968-114J','deb Delta 62','',16841,'In Earth orbit','','2017-03-21 00:00:00',115.9,1427,1566,101.57,'Sun-Synch' from dual union all
select 42083,'042083','1968-114K','deb Delta 62','',16841,'In Earth orbit','','2017-03-28 00:00:00',110.7,1050,1468,101.87,'Sun-Synch' from dual union all
select 42084,'042084','1968-114L','deb Delta 62','',16841,'In Earth orbit','','2017-03-22 00:00:00',115.03,1189,1725,101.97,'Sun-Synch' from dual union all
select 42085,'042085','1968-114M','deb Delta 62','',16841,'In Earth orbit','','2017-03-27 00:00:00',112.19,1240,1414,101.05,'Sun-Synch' from dual union all
select 42086,'042086','1968-114N','deb Delta 62','',16841,'In Earth orbit','','2017-03-20 00:00:00',120.84,1447,1990,102.08,'Medium' from dual union all
select 42087,'042087','1968-114P','deb Delta 62','',16841,'In Earth orbit','','2017-03-27 00:00:00',116.71,1376,1690,101.43,'Sun-Synch' from dual union all
select 42088,'042088','1968-114Q','deb Delta 62','',16841,'In Earth orbit','','2017-03-24 00:00:00',115.31,1454,1485,101.44,'Sun-Synch' from dual union all
select 42089,'042089','1968-114R','deb Delta 62','',16841,'In Earth orbit','','2017-03-19 00:00:00',116.98,1420,1671,101.81,'Sun-Synch' from dual union all
select 4209,'004209','1969-082BY','deb Agena D','',18349,'In Earth orbit','','1992-02-04 00:00:00',102.63,855,911,70.03,'Intermediate' from dual union all
select 42090,'042090','1968-114S','deb Delta 62','',16841,'In Earth orbit','','2017-02-27 00:00:00',116.09,1407,1603,101.43,'Sun-Synch' from dual union all
select 42091,'042091','1968-114T','deb Delta 62','',16841,'In Earth orbit','','2017-03-18 00:00:00',115.46,1448,1505,101.68,'Sun-Synch' from dual union all
select 42092,'042092','1968-114U','deb Delta 62','',16841,'In Earth orbit','','2017-03-14 00:00:00',114.41,1408,1450,101.64,'Sun-Synch' from dual union all
select 42093,'042093','1968-114V','deb Delta 62','',16841,'In Earth orbit','','2017-03-28 00:00:00',111.95,1216,1416,102.51,'Sun-Synch' from dual union all
select 42094,'042094','1968-114W','deb Delta 62','',16841,'In Earth orbit','','2016-12-24 00:00:00',116.5,1368,1679,101.54,'Sun-Synch' from dual union all
select 42095,'042095','1981-053PF','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-21 00:00:00',105.03,895,1097,82.76,'Intermediate' from dual union all
select 42096,'042096','1981-053PG','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-20 00:00:00',102,812,895,82.73,'Intermediate' from dual union all
select 42097,'042097','1981-053PH','deb Kosmos-1275','',52576,'In Earth orbit','','2016-09-02 00:00:00',102.31,822,913,82.81,'Intermediate' from dual union all
select 42098,'042098','1981-053PJ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-28 00:00:00',105.68,921,1132,83.16,'Intermediate' from dual union all
select 42099,'042099','1981-053PK','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-26 00:00:00',101.97,828,876,82.86,'Intermediate' from dual union all
select 421,'000421','61 Omicron 141','deb r/b Ablestar','',7273,'Reentered','1984-10-02 00:00:00','1962-09-23 00:00:00',103.89,915,970,66.73,'Intermediate' from dual union all
select 4210,'004210','1969-082BZ','deb Agena D','',18349,'Reentered','2002-02-13 00:00:00','1992-02-05 00:00:00',99.75,708,785,69.91,'Intermediate' from dual union all
select 42100,'042100','1981-053PL','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-23 00:00:00',101.43,809,844,82.87,'Intermediate' from dual union all
select 42101,'042101','1981-053PM','deb Kosmos-1275','',52576,'In Earth orbit','','2016-03-21 00:00:00',100.57,780,792,82.78,'Intermediate' from dual union all
select 42102,'042102','1981-053PN','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-28 00:00:00',104.23,900,1017,83.01,'Intermediate' from dual union all
select 42103,'042103','1981-053PP','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-21 00:00:00',102.59,860,903,83.18,'Intermediate' from dual union all
select 42104,'042104','1981-053PQ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-26 00:00:00',104.36,933,995,82.87,'Intermediate' from dual union all
select 42105,'042105','1981-053PR','deb Kosmos-1275','',52576,'In Earth orbit','','2017-01-03 00:00:00',106.08,905,1185,82.98,'Intermediate' from dual union all
select 42106,'042106','1981-053PS','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',99.41,707,754,83.08,'Intermediate' from dual union all
select 42107,'042107','1981-053PT','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-23 00:00:00',102.91,856,937,82.81,'Intermediate' from dual union all
select 42108,'042108','1981-053PU','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-11 00:00:00',101.7,822,857,83.18,'Intermediate' from dual union all
select 42109,'042109','1981-053PV','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-25 00:00:00',104.49,949,993,82.95,'Intermediate' from dual union all
select 4211,'004211','1969-082CA','deb Agena D','',18349,'In Earth orbit','','1992-01-30 00:00:00',102.39,845,899,70.07,'Intermediate' from dual union all
select 42110,'042110','1981-053PW','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',101.39,803,846,82.92,'Intermediate' from dual union all
select 42111,'042111','1981-053PX','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',101.9,815,883,82.81,'Intermediate' from dual union all
select 42112,'042112','1981-053PY','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-25 00:00:00',101.15,802,825,82.85,'Intermediate' from dual union all
select 42113,'042113','1981-053PZ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',105.25,862,1151,82.75,'Intermediate' from dual union all
select 42114,'042114','1981-053QA','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-26 00:00:00',101.76,810,874,82.83,'Intermediate' from dual union all
select 42115,'042115','1981-053QB','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',102.97,854,944,82.66,'Intermediate' from dual union all
select 42116,'042116','1981-053QC','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-22 00:00:00',101.25,806,829,83.14,'Intermediate' from dual union all
select 42117,'042117','1981-053QD','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',103.43,908,933,82.98,'Intermediate' from dual union all
select 42118,'042118','1981-053QE','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-26 00:00:00',101.61,793,877,82.97,'Intermediate' from dual union all
select 42119,'042119','1981-053QF','deb Kosmos-1275','',52576,'In Earth orbit','','2017-02-19 00:00:00',104.12,913,994,83.18,'Intermediate' from dual union all
select 4212,'004212','1969-082CB','deb Agena D','',18349,'Reentered','1991-07-06 00:00:00','1991-07-06 00:00:00',89.1,217,243,70.29,'Intermediate' from dual union all
select 42120,'042120','1981-053QG','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',106.38,917,1201,82.93,'Intermediate' from dual union all
select 42121,'042121','1981-053QH','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-16 00:00:00',102.69,853,919,82.93,'Intermediate' from dual union all
select 42122,'042122','1981-053QJ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-21 00:00:00',104.42,897,1038,82.94,'Intermediate' from dual union all
select 42123,'042123','1981-053QK','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-26 00:00:00',103.91,893,994,82.94,'Intermediate' from dual union all
select 42124,'042124','1981-053QL','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-25 00:00:00',105.6,927,1118,83.01,'Intermediate' from dual union all
select 42125,'042125','1981-053QM','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',102.9,860,932,82.94,'Intermediate' from dual union all
select 42126,'042126','1981-053QN','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',106.38,938,1180,83.16,'Intermediate' from dual union all
select 42127,'042127','1981-053QP','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-05 00:00:00',102.37,842,900,82.83,'Intermediate' from dual;
insert into SATELLITE
select 42128,'042128','1981-053QQ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-28 00:00:00',103.41,901,939,82.93,'Intermediate' from dual union all
select 42129,'042129','1981-053QR','deb Kosmos-1275','',52576,'In Earth orbit','','2017-01-22 00:00:00',103.93,899,990,82.84,'Intermediate' from dual union all
select 4213,'004213','1969-082CC','deb Agena D','',18349,'Reentered','1981-04-13 00:00:00','1981-03-05 00:00:00',95.89,540,584,69.58,'Intermediate' from dual union all
select 42130,'042130','1981-053QS','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',102.35,849,891,82.87,'Intermediate' from dual union all
select 42131,'042131','1981-053QT','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-25 00:00:00',101.28,803,836,82.89,'Intermediate' from dual union all
select 42132,'042132','1981-053QU','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',105.55,915,1126,83.15,'Intermediate' from dual union all
select 42133,'042133','1981-053QV','deb Kosmos-1275','',52576,'In Earth orbit','','2016-03-28 00:00:00',106.29,934,1176,83.08,'Intermediate' from dual union all
select 42134,'042134','1981-053QW','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-28 00:00:00',106.58,932,1205,83,'Intermediate' from dual union all
select 42135,'042135','1981-053QX','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-28 00:00:00',105.19,918,1089,83.13,'Intermediate' from dual union all
select 42136,'042136','1981-053QY','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',103.76,884,989,83.03,'Intermediate' from dual union all
select 42137,'042137','1981-053QZ','deb Kosmos-1275','',52576,'In Earth orbit','','2016-12-31 00:00:00',106,926,1157,82.89,'Intermediate' from dual union all
select 42138,'042138','1981-053RA','deb Kosmos-1275','',52576,'In Earth orbit','','2017-02-25 00:00:00',104.06,871,1030,83.14,'Intermediate' from dual union all
select 42139,'042139','1981-053RB','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-25 00:00:00',103.66,895,968,83,'Intermediate' from dual union all
select 4214,'004214','1969-082CD','deb Agena D','',18349,'In Earth orbit','','1992-02-04 00:00:00',100.85,748,850,70.05,'Intermediate' from dual union all
select 42140,'042140','1981-053RC','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',106.91,931,1237,83.13,'Intermediate' from dual union all
select 42141,'042141','1981-053RD','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-18 00:00:00',101.92,828,871,82.78,'Intermediate' from dual union all
select 42142,'042142','1981-053RE','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',100.34,731,819,83.02,'Intermediate' from dual union all
select 42143,'042143','1981-053RF','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-15 00:00:00',104.79,905,1064,83.02,'Intermediate' from dual union all
select 42144,'042144','1981-053RG','deb Kosmos-1275','',52576,'In Earth orbit','','2017-02-18 00:00:00',104.05,905,995,82.8,'Intermediate' from dual union all
select 42145,'042145','1981-053RH','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',100.1,743,784,83.37,'Intermediate' from dual union all
select 42146,'042146','1981-053RJ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-16 00:00:00',104.56,899,1049,83.01,'Intermediate' from dual union all
select 42147,'042147','1981-053RK','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',101.9,834,864,83.08,'Intermediate' from dual union all
select 42148,'042148','1981-053RL','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-28 00:00:00',104.36,905,1024,83.1,'Intermediate' from dual union all
select 42149,'042149','1981-053RM','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-26 00:00:00',104.08,890,1013,82.8,'Intermediate' from dual union all
select 4215,'004215','1969-082CE','deb Agena D','',18349,'Reentered','2001-03-08 00:00:00','1992-02-04 00:00:00',98.36,636,725,70.25,'Intermediate' from dual union all
select 42150,'042150','1981-053RN','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',107.01,923,1253,83.06,'Intermediate' from dual union all
select 42151,'042151','1981-053RP','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-25 00:00:00',104.51,907,1036,82.94,'Intermediate' from dual union all
select 42152,'042152','1981-053RQ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-10 00:00:00',103.41,872,968,83.12,'Intermediate' from dual union all
select 42153,'042153','1981-053RR','deb Kosmos-1275','',52576,'In Earth orbit','','2016-02-16 00:00:00',102.7,848,925,82.86,'Intermediate' from dual union all
select 42154,'042154','1981-053RS','deb Kosmos-1275','',52576,'In Earth orbit','','2017-02-02 00:00:00',105.49,913,1122,82.81,'Intermediate' from dual union all
select 42155,'042155','1981-053RT','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',102.74,856,920,82.92,'Intermediate' from dual union all
select 42156,'042156','1981-053RU','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',102.91,856,937,82.75,'Intermediate' from dual union all
select 42157,'042157','1981-053RV','deb Kosmos-1275','',52576,'In Earth orbit','','2016-07-29 00:00:00',100.68,778,804,83.08,'Intermediate' from dual union all
select 42158,'042158','1981-053RW','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',104.77,901,1067,83.01,'Intermediate' from dual union all
select 42159,'042159','1981-053RX','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-23 00:00:00',104.48,917,1024,82.81,'Intermediate' from dual union all
select 4216,'004216','1969-082CF','deb Agena D','',18349,'In Earth orbit','','1992-01-30 00:00:00',101.67,801,875,70.1,'Intermediate' from dual union all
select 42160,'042160','1981-053RY','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-25 00:00:00',101.14,808,818,82.92,'Intermediate' from dual union all
select 42161,'042161','1981-053RZ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-14 00:00:00',103.58,883,973,82.93,'Intermediate' from dual union all
select 42162,'042162','1981-053SA','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-10 00:00:00',104.13,893,1015,82.88,'Intermediate' from dual union all
select 42163,'042163','1981-053SB','deb Kosmos-1275','',52576,'In Earth orbit','','2016-05-10 00:00:00',105.15,914,1089,83.01,'Intermediate' from dual union all
select 42164,'042164','1981-053SC','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-20 00:00:00',101.15,790,837,82.93,'Intermediate' from dual union all
select 42165,'042165','1981-053SD','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-25 00:00:00',102.23,834,895,82.73,'Intermediate' from dual union all
select 42166,'042166','1981-053SE','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',104.05,839,1061,82.9,'Intermediate' from dual union all
select 42167,'042167','1981-053SF','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-25 00:00:00',104.31,919,1005,82.84,'Intermediate' from dual union all
select 42168,'042168','1981-053SG','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',99.98,746,770,82.8,'Intermediate' from dual union all
select 42169,'042169','1981-053SH','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-24 00:00:00',100.51,766,800,82.88,'Intermediate' from dual union all
select 4217,'004217','1969-082CG','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',101.29,789,851,70.03,'Intermediate' from dual union all
select 42170,'042170','1981-053SJ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-09 00:00:00',107.03,945,1234,82.79,'Intermediate' from dual union all
select 42171,'042171','1981-053SK','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-26 00:00:00',102.67,856,914,83.09,'Intermediate' from dual union all
select 42172,'042172','1981-053SL','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-16 00:00:00',102.81,850,934,82.96,'Intermediate' from dual union all
select 42173,'042173','1981-053SM','deb Kosmos-1275','',52576,'In Earth orbit','','2017-02-15 00:00:00',106.89,923,1243,83.23,'Intermediate' from dual union all
select 42174,'042174','1981-053SN','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-22 00:00:00',100.89,770,831,82.78,'Intermediate' from dual union all
select 42175,'042175','1981-053SP','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',106.73,973,1178,83.03,'Intermediate' from dual union all
select 42176,'042176','1981-053SQ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-28 00:00:00',105.2,921,1087,82.9,'Intermediate' from dual union all
select 42177,'042177','1981-053SR','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',100.16,739,793,82.71,'Intermediate' from dual union all
select 42178,'042178','1981-053SS','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-25 00:00:00',101.84,839,853,82.95,'Intermediate' from dual union all
select 42179,'042179','1981-053ST','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',104.99,919,1069,83.13,'Intermediate' from dual union all
select 4218,'004218','1969-082CH','deb Agena D','',18349,'In Earth orbit','','1992-02-04 00:00:00',101.05,771,846,70.11,'Intermediate' from dual union all
select 42180,'042180','1981-053SU','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-21 00:00:00',101.1,782,839,82.89,'Intermediate' from dual union all
select 42181,'042181','1981-053SV','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',103.78,886,989,82.91,'Intermediate' from dual union all
select 42182,'042182','1981-053SW','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-22 00:00:00',105.91,932,1142,82.95,'Intermediate' from dual union all
select 42183,'042183','1981-053SX','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-17 00:00:00',104.18,919,993,82.94,'Intermediate' from dual union all
select 42184,'042184','1981-053SY','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',105.79,961,1102,83.02,'Intermediate' from dual union all
select 42185,'042185','1981-053SZ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-28 00:00:00',104.74,891,1074,83.05,'Intermediate' from dual union all
select 42186,'042186','1981-053TA','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',103.18,865,953,83.13,'Intermediate' from dual union all
select 42187,'042187','1981-053TB','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-26 00:00:00',103.38,871,965,82.97,'Intermediate' from dual union all
select 42188,'042188','1981-053TC','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',102.64,845,923,82.86,'Intermediate' from dual union all
select 42189,'042189','1981-053TD','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-26 00:00:00',105.33,914,1106,83.02,'Intermediate' from dual union all
select 4219,'004219','1969-096A','Kosmos-308','DS-P1-I No. 7',18363,'Reentered','1970-01-04 00:00:00','1969-12-05 00:00:00',90.49,251,346,71.01,'Intermediate' from dual union all
select 42190,'042190','1981-053TE','deb Kosmos-1275','',52576,'In Earth orbit','','2017-02-24 00:00:00',101.72,813,868,82.93,'Intermediate' from dual union all
select 42191,'042191','1981-053TF','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',102.9,866,926,82.83,'Intermediate' from dual union all
select 42192,'042192','1981-053TG','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',105.31,910,1108,83.11,'Intermediate' from dual union all
select 42193,'042193','1981-053TH','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-25 00:00:00',105.57,923,1119,82.95,'Intermediate' from dual union all
select 42194,'042194','1981-053TJ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-07 00:00:00',105.47,877,1156,82.84,'Intermediate' from dual union all
select 42195,'042195','1981-053TK','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-24 00:00:00',104.86,902,1074,83.04,'Intermediate' from dual union all
select 42196,'042196','1981-053TL','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',105.16,912,1091,82.79,'Intermediate' from dual union all
select 42197,'042197','1981-053TM','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',105.37,924,1100,82.86,'Intermediate' from dual union all
select 42198,'042198','1981-053TN','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-25 00:00:00',103.95,914,976,82.81,'Intermediate' from dual union all
select 42199,'042199','1981-053TP','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',103.65,901,962,82.95,'Intermediate' from dual union all
select 422,'000422','62 A Omega 1','Kosmos-9','Zenit-2 No. 7',8148,'Landed','1962-10-01 00:00:00','1962-09-29 00:00:00',90.88,297,339,64.99,'Intermediate' from dual union all
select 4220,'004220','1969-096B','S1M','S1M',18363,'Reentered','1969-12-06 00:00:00','1969-11-11 00:00:00',90.88,265,371,71.03,'Intermediate' from dual union all
select 42200,'042200','1981-053TQ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-26 00:00:00',103.1,851,960,82.97,'Intermediate' from dual union all
select 42201,'042201','1981-053TR','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',101.88,812,883,82.7,'Intermediate' from dual union all
select 42202,'042202','1981-053TS','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',104.03,910,987,82.84,'Intermediate' from dual union all
select 42203,'042203','1981-053TT','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-26 00:00:00',105.19,912,1094,83,'Intermediate' from dual union all
select 42204,'042204','1981-053TU','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-28 00:00:00',105.5,919,1117,83.07,'Intermediate' from dual union all
select 42205,'042205','1981-053TV','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',105.17,918,1087,83.04,'Intermediate' from dual union all
select 42206,'042206','1981-053TW','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-22 00:00:00',103.98,916,977,82.92,'Intermediate' from dual union all
select 42207,'042207','1981-053TX','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',106.47,902,1224,83.05,'Intermediate' from dual union all
select 42208,'042208','1981-053TY','deb Kosmos-1275','',52576,'In Earth orbit','','2017-01-19 00:00:00',102.34,804,935,82.93,'Intermediate' from dual union all
select 42209,'042209','1981-053TZ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',103.99,904,991,82.88,'Intermediate' from dual union all
select 4221,'004221','1969-097A','Azur','GRS-A',18364,'In Earth orbit','','1969-12-10 00:00:00',121.88,388,3142,102.97,'Medium' from dual union all
select 42210,'042210','1981-053UA','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',105.34,930,1091,83.01,'Intermediate' from dual union all
select 42211,'042211','1981-053UB','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-28 00:00:00',102.32,837,900,82.85,'Intermediate' from dual union all
select 42212,'042212','1981-053UC','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-28 00:00:00',105.94,925,1151,83.06,'Intermediate' from dual union all
select 42213,'042213','1981-053UD','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',102.42,832,915,82.83,'Intermediate' from dual union all
select 42214,'042214','1981-053UE','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-19 00:00:00',104.67,914,1044,83.02,'Intermediate' from dual union all
select 42215,'042215','1981-053UF','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-28 00:00:00',104.82,903,1068,82.9,'Intermediate' from dual union all
select 42216,'042216','1981-053UG','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',102.22,850,878,82.88,'Intermediate' from dual union all
select 42217,'042217','1981-053UH','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',105.49,923,1112,83.03,'Intermediate' from dual;
insert into SATELLITE
select 42218,'042218','1981-053UJ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',103.07,859,949,82.88,'Intermediate' from dual union all
select 42219,'042219','1981-053UK','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-28 00:00:00',102.55,841,918,82.95,'Intermediate' from dual union all
select 4222,'004222','1969-097B','FW4S S/N 2223-5','FW4S',18364,'Reentered','1999-11-15 00:00:00','1969-12-11 00:00:00',121.86,386,3142,102.95,'Medium' from dual union all
select 42220,'042220','1981-053UL','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',101.09,786,834,82.92,'Intermediate' from dual union all
select 42221,'042221','1981-053UM','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',104.71,921,1041,82.89,'Intermediate' from dual union all
select 42222,'042222','1981-053UN','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-23 00:00:00',102.98,850,950,82.99,'Intermediate' from dual union all
select 42223,'042223','1981-053UP','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',101.7,803,876,82.87,'Intermediate' from dual union all
select 42224,'042224','1981-053UQ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-23 00:00:00',102.23,822,907,82.74,'Intermediate' from dual union all
select 42225,'042225','1981-053UR','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-29 00:00:00',102.4,849,896,83.14,'Intermediate' from dual union all
select 42226,'042226','1981-053US','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',105.49,927,1108,82.96,'Intermediate' from dual union all
select 42227,'042227','1981-053UT','deb Kosmos-1275','',52576,'In Earth orbit','','2017-01-05 00:00:00',104.41,904,1029,82.91,'Intermediate' from dual union all
select 42228,'042228','1981-053UU','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-10 00:00:00',104.33,933,993,82.93,'Intermediate' from dual union all
select 42229,'042229','1981-053UV','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-17 00:00:00',107.13,925,1263,83.03,'Intermediate' from dual union all
select 4223,'004223','1969-098A','Kosmos-309','Zenit-2 No. 80',18365,'Landed','1969-11-20 00:00:00','1969-11-14 00:00:00',89.97,183,363,65.41,'Intermediate' from dual union all
select 42230,'042230','1981-053UW','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',102.67,842,928,82.75,'Intermediate' from dual union all
select 42231,'042231','1981-053UX','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-27 00:00:00',102.02,820,889,82.77,'Intermediate' from dual union all
select 42232,'042232','1981-053UY','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-28 00:00:00',106.44,925,1199,83.02,'Intermediate' from dual union all
select 42233,'042233','1981-053UZ','deb Kosmos-1275','',52576,'In Earth orbit','','2017-03-23 00:00:00',105.57,922,1121,83.04,'Intermediate' from dual union all
select 42234,'042234','1993-016CN','deb 11S772','',65118,'In Earth orbit','','2017-03-26 00:00:00',101.49,735,924,70.98,'Intermediate' from dual union all
select 42235,'042235','1993-016CP','deb 11S772','',65118,'In Earth orbit','','2017-03-29 00:00:00',101.91,747,951,71,'Intermediate' from dual union all
select 42236,'042236','1993-016CQ','deb 11S772','',65118,'In Earth orbit','','2017-03-18 00:00:00',102.04,752,958,70.99,'Intermediate' from dual union all
select 42237,'042237','1993-016CR','deb 11S772','',65118,'In Earth orbit','','2017-03-29 00:00:00',101.68,741,936,70.96,'Intermediate' from dual union all
select 42238,'042238','1993-016CS','deb 11S772','',65118,'In Earth orbit','','2017-03-22 00:00:00',101.39,733,916,70.98,'Intermediate' from dual union all
select 42239,'042239','1993-016CT','deb 11S772','',65118,'In Earth orbit','','2017-03-29 00:00:00',102.24,769,961,71.14,'Intermediate' from dual union all
select 4224,'004224','1969-098B','11A57I','11A57I',18365,'Reentered','1969-11-22 00:00:00','1969-11-14 00:00:00',89.78,192,335,65.44,'Intermediate' from dual union all
select 42240,'042240','1993-016CU','deb 11S772','',65118,'In Earth orbit','','2016-10-02 00:00:00',101.99,752,954,70.98,'Intermediate' from dual union all
select 42241,'042241','1993-016CV','deb 11S772','',65118,'In Earth orbit','','2016-12-07 00:00:00',101.5,744,916,70.96,'Intermediate' from dual union all
select 42242,'042242','1993-016CW','deb 11S772','',65118,'In Earth orbit','','2017-03-29 00:00:00',102.12,755,962,70.98,'Intermediate' from dual union all
select 42243,'042243','1993-016CX','deb 11S772','',65118,'In Earth orbit','','2017-03-27 00:00:00',101.92,749,950,70.97,'Intermediate' from dual union all
select 42244,'042244','1993-016CY','deb 11S772','',65118,'In Earth orbit','','2017-03-17 00:00:00',101.44,736,918,70.97,'Intermediate' from dual union all
select 42245,'042245','1993-016CZ','deb 11S772','',65118,'In Earth orbit','','2016-01-12 00:00:00',101.46,734,922,70.98,'Intermediate' from dual union all
select 42246,'042246','1993-016DA','deb 11S772','',65118,'In Earth orbit','','2017-03-28 00:00:00',101.64,744,929,70.96,'Intermediate' from dual union all
select 42247,'042247','1993-016DB','deb 11S772','',65118,'In Earth orbit','','2017-03-27 00:00:00',101.59,740,928,70.99,'Intermediate' from dual union all
select 42248,'042248','1993-016DC','deb 11S772','',65118,'In Earth orbit','','2017-03-29 00:00:00',101.33,730,914,70.97,'Intermediate' from dual union all
select 42249,'042249','1993-016DD','deb 11S772','',65118,'In Earth orbit','','2017-03-03 00:00:00',101,720,892,70.97,'Intermediate' from dual union all
select 4225,'004225','1969-099A','Yankee Clipper','Apollo CM-108',18366,'Deep Space','1969-11-15 00:00:00','1969-11-14 00:00:00',16096.61,215,409386,30.36,'Cislunar' from dual union all
select 42250,'042250','1993-016DE','deb 11S772','',65118,'In Earth orbit','','2017-03-29 00:00:00',101.54,823,841,71,'Intermediate' from dual union all
select 42251,'042251','1993-016DF','deb 11S772','',65118,'In Earth orbit','','2017-03-29 00:00:00',100.33,711,838,70.98,'Intermediate' from dual union all
select 42252,'042252','1993-016DG','deb 11S772','',65118,'In Earth orbit','','2017-03-22 00:00:00',102.56,761,999,70.98,'Intermediate' from dual union all
select 42253,'042253','1993-016DH','deb 11S772','',65118,'In Earth orbit','','2017-03-25 00:00:00',101.23,731,903,70.96,'Intermediate' from dual union all
select 42254,'042254','1993-016DJ','deb 11S772','',65118,'In Earth orbit','','2017-03-26 00:00:00',100.55,715,855,70.99,'Intermediate' from dual union all
select 42255,'042255','1993-016DK','deb 11S772','',65118,'In Earth orbit','','2017-03-29 00:00:00',101.53,822,840,71,'Intermediate' from dual union all
select 42256,'042256','1993-016DL','deb 11S772','',65118,'In Earth orbit','','2015-02-22 00:00:00',101.7,746,933,70.96,'Intermediate' from dual union all
select 42257,'042257','1995-015FX','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',100.3,724,822,98.82,'Sun-Synch' from dual union all
select 42258,'042258','1995-015FY','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',103.99,835,1060,98.75,'Sun-Synch' from dual union all
select 42259,'042259','1995-015FZ','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-28 00:00:00',101.39,813,835,98.77,'Sun-Synch' from dual union all
select 4226,'004226','1969-099B','Saturn S-IVB-507','Saturn S-IVB-507',18366,'Deep Space','1969-11-15 00:00:00','1969-11-14 00:00:00',16096.61,215,409386,30.36,'Cislunar' from dual union all
select 42260,'042260','1995-015GA','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',99.32,643,809,98.73,'Sun-Synch' from dual union all
select 42261,'042261','1995-015GB','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',100.35,712,839,98.73,'Sun-Synch' from dual union all
select 42262,'042262','1995-015GC','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',100.81,760,834,98.7,'Sun-Synch' from dual union all
select 42263,'042263','1995-015GD','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',103.63,839,1021,98.79,'Sun-Synch' from dual union all
select 42264,'042264','1995-015GE','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',98.2,574,772,98.73,'Sun-Synch' from dual union all
select 42265,'042265','1995-015GF','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-27 00:00:00',101.52,826,836,98.78,'Sun-Synch' from dual union all
select 42266,'042266','1995-015GG','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',101.26,793,844,98.79,'Sun-Synch' from dual union all
select 42267,'042267','1995-015GH','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-27 00:00:00',103.27,834,993,98.77,'Sun-Synch' from dual union all
select 42268,'042268','1995-015GJ','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',103.76,831,1042,98.77,'Sun-Synch' from dual union all
select 42269,'042269','1995-015GK','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',98.83,608,797,98.7,'Sun-Synch' from dual union all
select 4227,'004227','1969-082CJ','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',99.95,702,811,69.9,'Intermediate' from dual union all
select 42270,'042270','1995-015GL','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-27 00:00:00',101.85,817,876,98.75,'Sun-Synch' from dual union all
select 42271,'042271','1995-015GM','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',101.42,821,831,98.77,'Sun-Synch' from dual union all
select 42272,'042272','1995-015GN','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',101.89,827,870,98.74,'Sun-Synch' from dual union all
select 42273,'042273','1995-015GP','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',101.13,801,824,98.76,'Sun-Synch' from dual union all
select 42274,'042274','1995-015GQ','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',101.06,794,824,98.76,'Sun-Synch' from dual union all
select 42275,'042275','1995-015GR','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',99.15,631,805,98.64,'Sun-Synch' from dual union all
select 42276,'042276','1995-015GS','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',103.47,838,1007,98.82,'Sun-Synch' from dual union all
select 42277,'042277','1995-015GT','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',100.17,729,804,98.74,'Sun-Synch' from dual union all
select 42278,'042278','1995-015GU','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',102.21,832,895,98.77,'Sun-Synch' from dual union all
select 42279,'042279','1995-015GV','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',101.57,826,839,98.76,'Sun-Synch' from dual union all
select 4228,'004228','1969-082CK','deb Agena D','',18349,'Reentered','1981-03-25 00:00:00','1981-03-24 00:00:00',90.78,291,335,70.03,'Intermediate' from dual union all
select 42280,'042280','1995-015GW','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',101.39,816,833,98.82,'Sun-Synch' from dual union all
select 42281,'042281','1995-015GX','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',100.91,770,834,98.74,'Sun-Synch' from dual union all
select 42282,'042282','1995-015GY','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',102.15,831,890,98.76,'Sun-Synch' from dual union all
select 42283,'042283','1995-015GZ','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',100.2,709,826,98.75,'Sun-Synch' from dual union all
select 42284,'042284','1995-015HA','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',100.08,720,805,98.82,'Sun-Synch' from dual union all
select 42285,'042285','1995-015HB','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',102.24,837,893,98.85,'Sun-Synch' from dual union all
select 42286,'042286','1995-015HC','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-22 00:00:00',102.6,833,930,98.78,'Sun-Synch' from dual union all
select 42287,'042287','1995-015HD','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',100.97,797,813,98.76,'Sun-Synch' from dual union all
select 42288,'042288','1995-015HE','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',100.21,712,825,98.83,'Sun-Synch' from dual union all
select 42289,'042289','1995-015HF','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-27 00:00:00',102.42,830,917,98.81,'Sun-Synch' from dual union all
select 4229,'004229','1969-082CL','deb Agena D','',18349,'Reentered','1979-12-10 00:00:00','1979-10-16 00:00:00',95.63,504,594,70.11,'Intermediate' from dual union all
select 42290,'042290','1995-015HG','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',98.56,620,760,98.78,'Sun-Synch' from dual union all
select 42291,'042291','1995-015HH','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-27 00:00:00',101.55,813,852,98.74,'Sun-Synch' from dual union all
select 42292,'042292','1995-015HJ','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',101.47,806,851,98.76,'Sun-Synch' from dual union all
select 42293,'042293','1995-015HK','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',101.17,801,828,98.81,'Sun-Synch' from dual union all
select 42294,'042294','1995-015HL','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',102.25,829,901,98.79,'Sun-Synch' from dual union all
select 42295,'042295','1995-015HM','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',101.88,821,875,98.75,'Sun-Synch' from dual union all
select 42296,'042296','1995-015HN','deb DMSP F-13','',65676,'In Earth orbit','','2016-06-04 00:00:00',101.57,821,844,98.81,'Sun-Synch' from dual union all
select 42297,'042297','1995-015HP','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-27 00:00:00',102.31,833,902,98.77,'Sun-Synch' from dual union all
select 42298,'042298','1995-015HQ','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',106.28,837,1272,98.72,'Sun-Synch' from dual union all
select 42299,'042299','1995-015HR','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',101.47,808,848,98.77,'Sun-Synch' from dual union all
select 423,'000423','62 A Omega 2','8A92E','8A92E',8148,'Reentered','1962-12-22 00:00:00','1962-10-28 00:00:00',90.6,281,327,64.96,'Intermediate' from dual union all
select 4230,'004230','1969-082CM','deb Agena D','',18349,'Reentered','1981-10-07 00:00:00','1981-10-05 00:00:00',92,353,392,69.43,'Intermediate' from dual union all
select 42300,'042300','1995-015HS','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',101.27,792,845,98.78,'Sun-Synch' from dual union all
select 42301,'042301','1995-015HT','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',99.94,701,810,98.72,'Sun-Synch' from dual union all
select 42302,'042302','1995-015HU','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',100.77,766,824,98.72,'Sun-Synch' from dual union all
select 42303,'042303','1995-015HV','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',102.05,828,884,98.69,'Sun-Synch' from dual union all
select 42304,'042304','1995-015HW','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',101.01,792,821,98.76,'Sun-Synch' from dual union all
select 42305,'042305','1995-015HX','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',101.47,822,834,98.77,'Sun-Synch' from dual union all
select 42306,'042306','1995-015HY','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-28 00:00:00',101.19,790,840,98.77,'Sun-Synch' from dual union all
select 42307,'042307','1995-015HZ','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',97.19,553,696,98.73,'Sun-Synch' from dual;
insert into SATELLITE
select 42308,'042308','1995-015JA','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',102.04,832,879,98.77,'Sun-Synch' from dual union all
select 42309,'042309','1995-015JB','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',100.96,774,835,98.7,'Sun-Synch' from dual union all
select 4231,'004231','1969-082CN','deb Agena D','',18349,'In Earth orbit','','1992-02-02 00:00:00',101.17,781,848,70.29,'Intermediate' from dual union all
select 42310,'042310','1995-015JC','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',100.8,770,822,98.74,'Sun-Synch' from dual union all
select 42311,'042311','1995-015JD','deb DMSP F-13','',65676,'Reentered','2017-05-01 00:00:00','2017-02-06 00:00:00',94.77,472,543,98.75,'Sun-Synch' from dual union all
select 42312,'042312','1995-015JE','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',97.92,582,736,98.75,'Sun-Synch' from dual union all
select 42313,'042313','1995-015JF','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',102.34,827,911,98.76,'Sun-Synch' from dual union all
select 42314,'042314','1995-015JG','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',97.35,546,718,98.74,'Sun-Synch' from dual union all
select 42315,'042315','1995-015JH','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',101.09,785,836,98.79,'Sun-Synch' from dual union all
select 42316,'042316','1995-015JJ','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',100.38,736,818,98.73,'Sun-Synch' from dual union all
select 42317,'042317','1995-015JK','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',100.97,782,828,98.81,'Sun-Synch' from dual union all
select 42318,'042318','1995-015JL','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-27 00:00:00',105.43,834,1196,98.73,'Sun-Synch' from dual union all
select 42319,'042319','1995-015JM','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-27 00:00:00',102.14,834,885,98.84,'Sun-Synch' from dual union all
select 4232,'004232','1969-100A','Kosmos-310','Zenit-4 No. 69',18367,'Landed','1969-11-23 00:00:00','1969-11-19 00:00:00',89.83,202,331,64.99,'Intermediate' from dual union all
select 42320,'042320','1995-015JN','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',101.38,814,834,98.77,'Sun-Synch' from dual union all
select 42321,'042321','1995-015JP','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',100.71,764,821,98.74,'Sun-Synch' from dual union all
select 42322,'042322','1995-015JQ','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',101.65,801,873,98.73,'Sun-Synch' from dual union all
select 42323,'042323','1995-015JR','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',101.75,840,843,98.78,'Sun-Synch' from dual union all
select 42324,'042324','1995-015JS','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-27 00:00:00',101.29,812,828,98.8,'Sun-Synch' from dual union all
select 42325,'042325','1995-015JT','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-27 00:00:00',101.72,830,851,98.83,'Sun-Synch' from dual union all
select 42326,'042326','1995-015JU','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',101.49,819,839,98.84,'Sun-Synch' from dual union all
select 42327,'042327','1995-015JV','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',101.5,824,836,98.78,'Sun-Synch' from dual union all
select 42328,'042328','1995-015JW','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',102.04,824,887,98.75,'Sun-Synch' from dual union all
select 42329,'042329','1995-015JX','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-29 00:00:00',100.24,718,821,98.67,'Sun-Synch' from dual union all
select 4233,'004233','1969-100B','11A57I','11A57I',18367,'Reentered','1969-11-23 00:00:00','1969-11-17 00:00:00',89.57,201,306,65.02,'Intermediate' from dual union all
select 42330,'042330','1995-015JY','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-27 00:00:00',101.16,804,823,98.76,'Sun-Synch' from dual union all
select 42331,'042331','1995-015JZ','deb DMSP F-13','',65676,'In Earth orbit','','2017-03-30 00:00:00',101.33,815,828,98.77,'Sun-Synch' from dual union all
select 42332,'042332','2000-055PY','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.69,810,868,99.06,'Sun-Synch' from dual union all
select 42333,'042333','2000-055PZ','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.24,851,879,99.01,'Sun-Synch' from dual union all
select 42334,'042334','2000-055QA','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.71,827,852,98.9,'Sun-Synch' from dual union all
select 42335,'042335','2000-055QB','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',101.66,813,862,98.78,'Sun-Synch' from dual union all
select 42336,'042336','2000-055QC','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',101.81,841,847,98.97,'Sun-Synch' from dual union all
select 42337,'042337','2000-055QD','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.07,757,861,98.79,'Sun-Synch' from dual union all
select 42338,'042338','2000-055QE','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.11,772,850,98.76,'Sun-Synch' from dual union all
select 42339,'042339','2000-055QF','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.7,821,857,99.03,'Sun-Synch' from dual union all
select 4234,'004234','1969-098C','Nauka part','Nauka part',18365,'Reentered','1969-11-26 00:00:00','1969-11-17 00:00:00',89.56,189,317,65.38,'Intermediate' from dual union all
select 42340,'042340','2000-055QG','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.48,839,913,99.05,'Sun-Synch' from dual union all
select 42341,'042341','2000-055QH','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.25,776,860,98.78,'Sun-Synch' from dual union all
select 42342,'042342','2000-055QJ','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',100.52,760,807,99,'Sun-Synch' from dual union all
select 42343,'042343','2000-055QK','deb NOAA 16','',66957,'In Earth orbit','','2017-03-25 00:00:00',101.95,836,865,98.98,'Sun-Synch' from dual union all
select 42344,'042344','2000-055QL','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',101.29,784,856,98.8,'Sun-Synch' from dual union all
select 42345,'042345','2000-055QM','deb NOAA 16','',66957,'In Earth orbit','','2017-03-28 00:00:00',101.29,780,859,98.76,'Sun-Synch' from dual union all
select 42346,'042346','2000-055QN','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',100.91,755,848,99.06,'Sun-Synch' from dual union all
select 42347,'042347','2000-055QP','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.34,782,863,98.9,'Sun-Synch' from dual union all
select 42348,'042348','2000-055QQ','deb NOAA 16','',66957,'In Earth orbit','','2017-03-27 00:00:00',102.94,842,954,98.83,'Sun-Synch' from dual union all
select 42349,'042349','2000-055QR','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.13,853,866,98.89,'Sun-Synch' from dual union all
select 4235,'004235','1969-098D','Nauka part','Nauka part',18365,'Reentered','1969-11-25 00:00:00','1969-11-17 00:00:00',89.68,186,332,65.36,'Intermediate' from dual union all
select 42350,'042350','2000-055QS','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',102.42,853,894,98.85,'Sun-Synch' from dual union all
select 42351,'042351','2000-055QT','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.08,770,850,98.76,'Sun-Synch' from dual union all
select 42352,'042352','2000-055QU','deb NOAA 16','',66957,'In Earth orbit','','2017-03-28 00:00:00',101.87,844,851,98.92,'Sun-Synch' from dual union all
select 42353,'042353','2000-055QV','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101,762,850,98.81,'Sun-Synch' from dual union all
select 42354,'042354','2000-055QW','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',102.28,858,876,98.97,'Sun-Synch' from dual union all
select 42355,'042355','2000-055QX','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.6,853,911,98.91,'Sun-Synch' from dual union all
select 42356,'042356','2000-055QY','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.65,808,866,98.91,'Sun-Synch' from dual union all
select 42357,'042357','2000-055QZ','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.89,837,860,98.87,'Sun-Synch' from dual union all
select 42358,'042358','2000-055RA','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.12,818,900,98.92,'Sun-Synch' from dual union all
select 42359,'042359','2000-055RB','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.6,818,850,98.93,'Sun-Synch' from dual union all
select 4236,'004236','1969-098E','Kapsula Kosmosa-309','Nauka 3KS',18365,'Reentered','1969-11-30 00:00:00','1969-11-19 00:00:00',89.72,188,334,65.41,'Intermediate' from dual union all
select 42360,'042360','2000-055RC','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.44,763,891,98.89,'Sun-Synch' from dual union all
select 42361,'042361','2000-055RD','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',102.43,809,939,98.87,'Sun-Synch' from dual union all
select 42362,'042362','2000-055RE','deb NOAA 16','',66957,'In Earth orbit','','2017-03-27 00:00:00',103.96,850,1041,98.9,'Sun-Synch' from dual union all
select 42363,'042363','2000-055RF','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',103.12,848,965,98.91,'Sun-Synch' from dual union all
select 42364,'042364','2000-055RG','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',101.99,807,899,98.9,'Sun-Synch' from dual union all
select 42365,'042365','2000-055RH','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.33,780,863,98.91,'Sun-Synch' from dual union all
select 42366,'042366','2000-055RJ','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',100.68,735,847,98.86,'Sun-Synch' from dual union all
select 42367,'042367','2000-055RK','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.33,851,886,98.94,'Sun-Synch' from dual union all
select 42368,'042368','2000-055RL','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.08,849,866,98.91,'Sun-Synch' from dual union all
select 42369,'042369','2000-055RM','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.84,839,853,98.89,'Sun-Synch' from dual union all
select 4237,'004237','1969-082E','OPS 7613 P/L 4','POPPY 6 SV2',18349,'In Earth orbit','','1969-11-22 00:00:00',103.45,905,938,70.01,'Intermediate' from dual union all
select 42370,'042370','2000-055RN','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',103.18,826,992,98.88,'Sun-Synch' from dual union all
select 42371,'042371','2000-055RP','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.24,779,856,99.05,'Sun-Synch' from dual union all
select 42372,'042372','2000-055RQ','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.89,839,858,98.93,'Sun-Synch' from dual union all
select 42373,'042373','2000-055RR','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',100.51,716,849,98.98,'Sun-Synch' from dual union all
select 42374,'042374','2000-055RS','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.05,845,867,98.9,'Sun-Synch' from dual union all
select 42375,'042375','2000-055RT','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.97,847,857,98.88,'Sun-Synch' from dual union all
select 42376,'042376','2000-055RU','deb NOAA 16','',66957,'In Earth orbit','','2017-03-28 00:00:00',103.96,853,1038,98.84,'Sun-Synch' from dual union all
select 42377,'042377','2000-055RV','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',101.52,806,856,99.07,'Sun-Synch' from dual union all
select 42378,'042378','2000-055RW','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.53,811,852,98.88,'Sun-Synch' from dual union all
select 42379,'042379','2000-055RX','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.5,789,871,98.92,'Sun-Synch' from dual union all
select 4238,'004238','1969-082CQ','deb Agena D','',18349,'Reentered','1978-12-30 00:00:00','1978-11-25 00:00:00',94.31,483,487,70.12,'Intermediate' from dual union all
select 42380,'042380','2000-055RY','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.21,780,852,98.86,'Sun-Synch' from dual union all
select 42381,'042381','2000-055RZ','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.08,772,848,99.01,'Sun-Synch' from dual union all
select 42382,'042382','2000-055SA','deb NOAA 16','',66957,'In Earth orbit','','2017-03-28 00:00:00',102.48,841,911,98.97,'Sun-Synch' from dual union all
select 42383,'042383','2000-055SB','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.07,816,897,98.89,'Sun-Synch' from dual union all
select 42384,'042384','2000-055SC','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.9,836,862,98.86,'Sun-Synch' from dual union all
select 42385,'042385','2000-055SD','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',101.79,836,850,98.83,'Sun-Synch' from dual union all
select 42386,'042386','2000-055SE','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.7,821,857,98.86,'Sun-Synch' from dual union all
select 42387,'042387','2000-055SF','deb NOAA 16','',66957,'In Earth orbit','','2017-03-28 00:00:00',101.77,828,857,98.9,'Sun-Synch' from dual union all
select 42388,'042388','2000-055SG','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.91,836,862,98.78,'Sun-Synch' from dual union all
select 42389,'042389','2000-055SH','deb NOAA 16','',66957,'In Earth orbit','','2017-03-28 00:00:00',101.12,761,863,99.1,'Sun-Synch' from dual union all
select 4239,'004239','1969-082CR','deb Agena D','',18349,'Reentered','1979-11-19 00:00:00','1969-12-18 00:00:00',101.17,709,920,70.28,'Intermediate' from dual union all
select 42390,'042390','2000-055SJ','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.72,827,854,98.85,'Sun-Synch' from dual union all
select 42391,'042391','2000-055SK','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.51,811,850,99.02,'Sun-Synch' from dual union all
select 42392,'042392','2000-055SL','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.57,815,851,98.96,'Sun-Synch' from dual union all
select 42393,'042393','2000-055SM','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.73,848,928,98.99,'Sun-Synch' from dual union all
select 42394,'042394','2000-055SN','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.27,779,859,98.91,'Sun-Synch' from dual union all
select 42395,'042395','2000-055SP','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.62,812,859,98.93,'Sun-Synch' from dual union all
select 42396,'042396','2000-055SQ','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.3,789,852,98.82,'Sun-Synch' from dual union all
select 42397,'042397','2000-055SR','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',101.95,814,888,98.84,'Sun-Synch' from dual union all
select 42398,'042398','2000-055SS','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.53,853,904,98.85,'Sun-Synch' from dual;
insert into SATELLITE
select 42399,'042399','2000-055ST','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.03,846,864,98.89,'Sun-Synch' from dual union all
select 424,'000424','62 B Alpha 1','Alouette 1','Alouette 1',8161,'In Earth orbit','','1962-11-10 00:00:00',105.39,999,1027,80.47,'Intermediate' from dual union all
select 4240,'004240','1969-082CS','deb Agena D','',18349,'Reentered','1980-05-24 00:00:00','1969-11-23 00:00:00',101.29,724,916,69.92,'Intermediate' from dual union all
select 42400,'042400','2000-055SU','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.09,841,875,98.85,'Sun-Synch' from dual union all
select 42401,'042401','2000-055SV','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.1,759,863,98.9,'Sun-Synch' from dual union all
select 42402,'042402','2000-055SW','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.34,792,852,98.94,'Sun-Synch' from dual union all
select 42403,'042403','2000-055SX','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',98.99,593,828,98.88,'Sun-Synch' from dual union all
select 42404,'042404','2000-055SY','deb NOAA 16','',66957,'In Earth orbit','','2017-03-28 00:00:00',98.64,545,843,98.83,'Sun-Synch' from dual union all
select 42405,'042405','2000-055SZ','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.34,794,851,98.99,'Sun-Synch' from dual union all
select 42406,'042406','2000-055TA','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',101.83,833,858,98.88,'Sun-Synch' from dual union all
select 42407,'042407','2000-055TB','deb NOAA 16','',66957,'In Earth orbit','','2017-03-28 00:00:00',102.07,843,871,99,'Sun-Synch' from dual union all
select 42408,'042408','2000-055TC','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.3,854,880,98.91,'Sun-Synch' from dual union all
select 42409,'042409','2000-055TD','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.24,780,854,98.81,'Sun-Synch' from dual union all
select 4241,'004241','1969-082CT','deb Agena D','',18349,'Reentered','1980-10-09 00:00:00','1980-09-21 00:00:00',94.19,473,486,70.1,'Intermediate' from dual union all
select 42410,'042410','2000-055TE','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.35,789,857,98.84,'Sun-Synch' from dual union all
select 42411,'042411','2000-055TF','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.25,785,851,98.87,'Sun-Synch' from dual union all
select 42412,'042412','2000-055TG','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.73,826,856,98.81,'Sun-Synch' from dual union all
select 42413,'042413','2000-055TH','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.48,806,852,98.89,'Sun-Synch' from dual union all
select 42414,'042414','2000-055TJ','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',101.33,820,824,99.04,'Sun-Synch' from dual union all
select 42415,'042415','2000-055TK','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.65,812,862,98.95,'Sun-Synch' from dual union all
select 42416,'042416','2000-055TL','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.89,832,865,98.91,'Sun-Synch' from dual union all
select 42417,'042417','2000-055TM','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.17,839,884,98.89,'Sun-Synch' from dual union all
select 42418,'042418','2000-055TN','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.9,838,860,98.87,'Sun-Synch' from dual union all
select 42419,'042419','2000-055TP','deb NOAA 16','',66957,'In Earth orbit','','2017-03-28 00:00:00',102.05,835,877,98.89,'Sun-Synch' from dual union all
select 4242,'004242','1969-097C','Azur despin weight 1','',18364,'Reentered','1981-10-07 00:00:00','1969-12-09 00:00:00',121.77,386,3134,102.8,'Medium' from dual union all
select 42420,'042420','2000-055TQ','deb NOAA 16','',66957,'In Earth orbit','','2017-03-27 00:00:00',101.64,813,860,99.03,'Sun-Synch' from dual union all
select 42421,'042421','2000-055TR','deb NOAA 16','',66957,'In Earth orbit','','2017-03-28 00:00:00',103.39,847,991,98.88,'Sun-Synch' from dual union all
select 42422,'042422','2000-055TS','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.87,836,859,98.9,'Sun-Synch' from dual union all
select 42423,'042423','2000-055TT','deb NOAA 16','',66957,'In Earth orbit','','2017-03-26 00:00:00',102.16,843,879,98.95,'Sun-Synch' from dual union all
select 42424,'042424','2000-055TU','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.64,814,859,99.01,'Sun-Synch' from dual union all
select 42425,'042425','2000-055TV','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',102.44,854,894,98.9,'Sun-Synch' from dual union all
select 42426,'042426','2000-055TW','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',102.13,847,872,98.8,'Sun-Synch' from dual union all
select 42427,'042427','2000-055TX','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',100.68,732,849,98.86,'Sun-Synch' from dual union all
select 42428,'042428','2000-055TY','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.55,812,852,98.91,'Sun-Synch' from dual union all
select 42429,'042429','2000-055TZ','deb NOAA 16','',66957,'In Earth orbit','','2017-03-27 00:00:00',102.44,850,898,98.89,'Sun-Synch' from dual union all
select 4243,'004243','1969-097D','Azur despin weight 2','',18364,'Reentered','1981-08-24 00:00:00','1969-12-13 00:00:00',121.62,387,3120,103.14,'Medium' from dual union all
select 42430,'042430','2000-055UA','deb NOAA 16','',66957,'In Earth orbit','','2017-03-30 00:00:00',101.91,848,851,98.9,'Sun-Synch' from dual union all
select 42431,'042431','2000-055UB','deb NOAA 16','',66957,'In Earth orbit','','2017-03-29 00:00:00',101.72,806,874,98.99,'Sun-Synch' from dual union all
select 42432,'042432','2017-017A','SES-10','SES-10',70407,'In Earth orbit','','2017-04-04 00:00:00',1318.56,31185,35722,.79,'Medium' from dual union all
select 42433,'042433','2017-017B','Falcon 9-033 Stage 2','F9-033 St2',70407,'In Earth orbit','','2017-04-04 00:00:00',586.01,234,33404,26.18,'Highly Ellip' from dual union all
select 42434,'042434','1998-067LF','Node 3 ACBM Shield 1','',66503,'Reentered','2017-10-23 00:00:00','2016-04-04 00:00:00',92.63,402,404,51.64,'Intermediate' from dual union all
select 42435,'042435','1966-040BN','Nimbus 2 debris','',13447,'In Earth orbit','','2017-03-31 00:00:00',107.92,1090,1171,100.32,'Sun-Synch' from dual union all
select 42436,'042436','1966-040BP','Nimbus 2 debris','',13447,'In Earth orbit','','2017-04-02 00:00:00',107.76,1086,1161,100.36,'Sun-Synch' from dual union all
select 42437,'042437','1966-040BQ','Nimbus 2 debris','',13447,'In Earth orbit','','2017-03-31 00:00:00',107.81,1085,1166,100.34,'Sun-Synch' from dual union all
select 42438,'042438','1966-040BR','Nimbus 2 debris','',13447,'In Earth orbit','','2016-10-22 00:00:00',107.85,1091,1164,100.35,'Sun-Synch' from dual union all
select 42439,'042439','1966-040BS','Nimbus 2 debris','',13447,'In Earth orbit','','2017-03-30 00:00:00',107.65,1089,1147,100.39,'Sun-Synch' from dual union all
select 4244,'004244','1969-082CU','deb Agena D','',18349,'Reentered','1979-06-11 00:00:00','1979-04-17 00:00:00',95.91,530,596,69.89,'Intermediate' from dual union all
select 42440,'042440','1966-040BT','Nimbus 2 debris','',13447,'In Earth orbit','','2017-04-02 00:00:00',107.9,1086,1173,100.33,'Sun-Synch' from dual union all
select 42441,'042441','1966-040BU','Nimbus 2 debris','',13447,'In Earth orbit','','2017-04-03 00:00:00',107.96,1095,1170,100.32,'Sun-Synch' from dual union all
select 42442,'042442','1966-040BV','Nimbus 2 debris','',13447,'In Earth orbit','','2017-03-31 00:00:00',107.95,1090,1174,100.32,'Sun-Synch' from dual union all
select 42443,'042443','1978-064T','deb Seasat','',45259,'In Earth orbit','','2017-04-03 00:00:00',99.17,689,749,107.97,'Retrograde' from dual union all
select 42444,'042444','1978-064U','deb Seasat','',45259,'Reentered','2017-06-26 00:00:00','2017-03-30 00:00:00',98.9,637,776,107.93,'Retrograde' from dual union all
select 42445,'042445','1983-044GF','deb Kosmos-1461','',55801,'In Earth orbit','','2017-04-03 00:00:00',95.93,517,610,65,'Intermediate' from dual union all
select 42446,'042446','1983-044GG','deb Kosmos-1461','',55801,'In Earth orbit','','2017-04-03 00:00:00',96.19,522,631,65,'Intermediate' from dual union all
select 42447,'042447','1983-044GH','deb Kosmos-1461','',55801,'In Earth orbit','','2017-04-02 00:00:00',96.26,524,635,64.99,'Intermediate' from dual union all
select 42448,'042448','1983-044GJ','deb Kosmos-1461','',55801,'In Earth orbit','','2017-04-03 00:00:00',95.68,510,593,65,'Intermediate' from dual union all
select 42449,'042449','1983-044GK','deb Kosmos-1461','',55801,'Reentered','2017-11-15 00:00:00','2017-04-03 00:00:00',94.79,477,540,65,'Intermediate' from dual union all
select 4245,'004245','1969-082CV','deb Agena D','',18349,'Reentered','1986-07-25 00:00:00','1986-06-21 00:00:00',91.98,366,378,70.33,'Intermediate' from dual union all
select 42450,'042450','1983-044GL','deb Kosmos-1461','',55801,'Reentered','2017-01-27 00:00:00','2017-04-03 00:00:00',94.56,467,528,64.98,'Intermediate' from dual union all
select 42451,'042451','1983-044GM','deb Kosmos-1461','',55801,'In Earth orbit','','2017-04-02 00:00:00',95.69,505,599,64.99,'Intermediate' from dual union all
select 42452,'042452','1984-072N','deb Meteor-2','',57126,'In Earth orbit','','2017-03-28 00:00:00',103.87,927,955,82.53,'Intermediate' from dual union all
select 42453,'042453','1984-072P','deb Meteor-2','',57126,'In Earth orbit','','2017-04-02 00:00:00',103.85,922,959,82.53,'Intermediate' from dual union all
select 42454,'042454','1984-072Q','deb Meteor-2','',57126,'In Earth orbit','','2017-04-03 00:00:00',103.72,916,953,82.52,'Intermediate' from dual union all
select 42455,'042455','1984-072R','deb Meteor-2','',57126,'In Earth orbit','','2017-03-31 00:00:00',103.86,929,953,82.53,'Intermediate' from dual union all
select 42456,'042456','1984-072S','deb Meteor-2','',57126,'In Earth orbit','','2017-04-03 00:00:00',103.9,916,970,82.54,'Intermediate' from dual union all
select 42457,'042457','1984-072T','deb Meteor-2','',57126,'In Earth orbit','','2017-02-27 00:00:00',103.92,933,954,82.53,'Intermediate' from dual union all
select 42458,'042458','2010-042Z','deb CZ-3B','',68902,'In Earth orbit','','2017-04-02 00:00:00',794.35,774,43294,30.77,'Highly Ellip' from dual union all
select 42459,'042459','2010-042AA','deb CZ-3B','',68902,'In Earth orbit','','2017-04-03 00:00:00',769.9,1197,41699,27.6,'Highly Ellip' from dual union all
select 4246,'004246','1969-099C','Intrepid','LM 6 AS',18366,'Deep Space Attached','1969-11-15 00:00:00','1969-11-14 00:00:00',16096.61,215,409386,30.36,'Cislunar' from dual union all
select 42460,'042460','2010-042AB','deb CZ-3B','',68902,'In Earth orbit','','2017-04-02 00:00:00',768.56,558,42273,30.58,'Highly Ellip' from dual union all
select 42461,'042461','2010-042AC','deb CZ-3B','',68902,'In Earth orbit','','2017-03-27 00:00:00',808.34,1320,43413,23.39,'Highly Ellip' from dual union all
select 42462,'042462','2010-042AD','deb CZ-3B','',68902,'In Earth orbit','','2017-03-20 00:00:00',713.51,781,39363,23.73,'GEO Transfer' from dual union all
select 42463,'042463','2010-042AE','deb CZ-3B','',68902,'In Earth orbit','','2017-02-04 00:00:00',755.88,537,41681,24.68,'Highly Ellip' from dual union all
select 42464,'042464','2010-042AF','deb CZ-3B','',68902,'In Earth orbit','','2017-04-01 00:00:00',789.72,1103,42744,25.15,'Highly Ellip' from dual union all
select 42465,'042465','2010-042AG','deb CZ-3B','',68902,'In Earth orbit','','2017-04-02 00:00:00',769.76,700,42189,24,'Highly Ellip' from dual union all
select 42466,'042466','2010-057BF','deb CZ-3C','',68917,'In Earth orbit','','2017-03-30 00:00:00',658.32,670,36710,20.27,'GEO Transfer' from dual union all
select 42467,'042467','2010-057BG','deb CZ-3C','',68917,'In Earth orbit','','2017-04-02 00:00:00',261.59,418,13923,20.2,'Medium' from dual union all
select 42468,'042468','2010-057BH','deb CZ-3C','',68917,'In Earth orbit','','2017-03-31 00:00:00',637.44,523,35791,20.04,'GEO Transfer' from dual union all
select 42469,'042469','2010-057BJ','deb CZ-3C','',68917,'In Earth orbit','','2017-04-03 00:00:00',632.9,385,35696,20.18,'GEO Transfer' from dual union all
select 4247,'004247','1969-082F','OPS 7613 P/L 5','POPPY 6 SV3',18349,'In Earth orbit','','1969-11-19 00:00:00',103.45,906,938,70.01,'Intermediate' from dual union all
select 42470,'042470','2010-057BK','deb CZ-3C','',68917,'In Earth orbit','','2017-04-02 00:00:00',551.37,453,31338,21.41,'Highly Ellip' from dual union all
select 42471,'042471','2011-077AL','deb CZ-3B','',69129,'In Earth orbit','','2017-03-30 00:00:00',761.8,722,41783,25.5,'Highly Ellip' from dual union all
select 42472,'042472','2011-077AM','deb CZ-3B','',69129,'In Earth orbit','','2017-04-02 00:00:00',803.01,1679,42801,25.57,'Highly Ellip' from dual union all
select 42473,'042473','2011-077AN','deb CZ-3B','',69129,'In Earth orbit','','2017-04-02 00:00:00',851.1,1948,44795,23.91,'Highly Ellip' from dual union all
select 42474,'042474','2011-077AP','deb CZ-3B','',69129,'In Earth orbit','','2017-04-02 00:00:00',862.24,2018,45243,23.38,'Highly Ellip' from dual union all
select 42475,'042475','2011-077AQ','deb CZ-3B','',69129,'In Earth orbit','','2017-04-03 00:00:00',753.96,765,41360,25.6,'Highly Ellip' from dual union all
select 42476,'042476','2011-077AR','deb CZ-3B','',69129,'In Earth orbit','','2017-04-02 00:00:00',766.48,772,41959,25.92,'Highly Ellip' from dual union all
select 42477,'042477','2012-008AM','deb CZ-3C','',69263,'In Earth orbit','','2017-04-04 00:00:00',593.88,405,33647,20.13,'Highly Ellip' from dual union all
select 42478,'042478','2012-008AN','deb CZ-3C','',69263,'In Earth orbit','','2017-03-30 00:00:00',642.61,1245,35334,20.93,'GEO Transfer' from dual union all
select 42479,'042479','2012-008AP','deb CZ-3C','',69263,'In Earth orbit','','2017-04-03 00:00:00',621.08,416,35055,21.17,'GEO Transfer' from dual union all
select 4248,'004248','1969-082CX','deb Agena D','',18349,'Reentered','1978-11-21 00:00:00','1978-10-29 00:00:00',95.13,525,525,69.94,'Intermediate' from dual union all
select 42480,'042480','2012-008AQ','deb CZ-3C','',69263,'In Earth orbit','','2017-04-03 00:00:00',644.29,1038,35626,20.95,'GEO Transfer' from dual union all
select 42481,'042481','2012-008AR','deb CZ-3C','',69263,'In Earth orbit','','2017-04-03 00:00:00',641.28,1147,35364,22.05,'GEO Transfer' from dual union all
select 42482,'042482','1989-006CJ','deb Ariane V28','',62283,'In Earth orbit','','2017-04-01 00:00:00',619.66,526,34871,10.2,'GEO Transfer' from dual union all
select 42483,'042483','1989-006CK','deb Ariane V28','',62283,'In Earth orbit','','2017-04-01 00:00:00',646.33,1051,35718,8.42,'GEO Transfer' from dual union all
select 42484,'042484','1989-006CL','deb Ariane V28','',62283,'In Earth orbit','','2017-03-29 00:00:00',623.15,533,35045,10.33,'GEO Transfer' from dual union all
select 42485,'042485','1989-006CM','deb Ariane V28','',62283,'In Earth orbit','','2017-04-02 00:00:00',661.06,1839,35680,8.57,'GEO Transfer' from dual union all
select 42486,'042486','1989-006CN','deb Ariane V28','',62283,'In Earth orbit','','2017-04-01 00:00:00',655.96,1428,35832,9.88,'GEO Transfer' from dual union all
select 42487,'042487','1989-006CP','deb Ariane V28','',62283,'In Earth orbit','','2017-04-02 00:00:00',649.28,1287,35633,8.59,'GEO Transfer' from dual union all
select 42488,'042488','1989-006CQ','deb Ariane V28','',62283,'In Earth orbit','','2017-04-04 00:00:00',647.48,895,35933,8.03,'GEO Transfer' from dual;
insert into SATELLITE
select 42489,'042489','1989-006CR','deb Ariane V28','',62283,'In Earth orbit','','2017-04-02 00:00:00',654.45,1612,35571,9.48,'GEO Transfer' from dual union all
select 4249,'004249','1969-082CY','deb Agena D','',18349,'In Earth orbit','','1992-02-04 00:00:00',102.1,805,911,70.05,'Intermediate' from dual union all
select 42490,'042490','1970-025QU','deb Agena D','',21502,'In Earth orbit','','2017-04-08 00:00:00',106.82,992,1167,99.92,'Sun-Synch' from dual union all
select 42491,'042491','1970-025QV','deb Agena D','',21502,'In Earth orbit','','2017-03-17 00:00:00',105.47,1006,1026,99.97,'Sun-Synch' from dual union all
select 42492,'042492','1970-025QW','deb Agena D','',21502,'In Earth orbit','','2017-04-09 00:00:00',105.77,1026,1035,99.6,'Sun-Synch' from dual union all
select 42493,'042493','1970-025QX','deb Agena D','',21502,'In Earth orbit','','2017-04-09 00:00:00',99.6,723,756,99.48,'Sun-Synch' from dual union all
select 42494,'042494','1970-025QY','deb Agena D','',21502,'In Earth orbit','','2017-04-07 00:00:00',106.01,968,1116,100.03,'Sun-Synch' from dual union all
select 42495,'042495','1970-025QZ','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',106.26,1033,1074,99.99,'Sun-Synch' from dual union all
select 42496,'042496','1970-025RA','deb Agena D','',21502,'In Earth orbit','','2017-04-09 00:00:00',106.88,1042,1123,99.86,'Sun-Synch' from dual union all
select 42497,'042497','1970-025RB','deb Agena D','',21502,'In Earth orbit','','2017-04-03 00:00:00',103.75,910,962,99.99,'Sun-Synch' from dual union all
select 42498,'042498','1970-025RC','deb Agena D','',21502,'In Earth orbit','','2016-06-29 00:00:00',108.37,1019,1284,100.3,'Sun-Synch' from dual union all
select 42499,'042499','1970-025RD','deb Agena D','',21502,'In Earth orbit','','2017-04-03 00:00:00',104.73,976,987,100.17,'Sun-Synch' from dual union all
select 425,'000425','62 A Omega 3','Kosmos-9 sensor cover','',8148,'Reentered','1962-10-06 00:00:00','1962-09-29 00:00:00',90.85,299,334,65.03,'Intermediate' from dual union all
select 4250,'004250','1969-101A','Skynet IA','Skynet IA',18368,'In Earth orbit','','1970-01-19 00:00:00',1436.44,35791,35796,2.31,'Stationary' from dual union all
select 42500,'042500','1970-025RE','deb Agena D','',21502,'In Earth orbit','','2017-04-09 00:00:00',106.4,1052,1068,99.76,'Sun-Synch' from dual union all
select 42501,'042501','1970-025RF','deb Agena D','',21502,'In Earth orbit','','2017-04-05 00:00:00',112.65,1027,1670,98.8,'Sun-Synch' from dual union all
select 42502,'042502','1970-025RG','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',106.3,1045,1066,99.79,'Sun-Synch' from dual union all
select 42503,'042503','1970-025RH','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',104.25,949,970,99.92,'Sun-Synch' from dual union all
select 42504,'042504','1970-025RJ','deb Agena D','',21502,'In Earth orbit','','2017-04-09 00:00:00',102.24,801,929,99.15,'Sun-Synch' from dual union all
select 42505,'042505','1970-025RK','deb Agena D','',21502,'In Earth orbit','','2017-04-08 00:00:00',106.09,1012,1079,100.19,'Sun-Synch' from dual union all
select 42506,'042506','1970-025RL','deb Agena D','',21502,'In Earth orbit','','2017-04-09 00:00:00',104.43,930,1006,99.6,'Sun-Synch' from dual union all
select 42507,'042507','1970-025RM','deb Agena D','',21502,'In Earth orbit','','2017-04-09 00:00:00',100.69,753,830,99.19,'Sun-Synch' from dual union all
select 42508,'042508','1970-025RN','deb Agena D','',21502,'In Earth orbit','','2017-04-06 00:00:00',103.64,841,1021,99.87,'Sun-Synch' from dual union all
select 42509,'042509','1970-025RP','deb Agena D','',21502,'In Earth orbit','','2017-04-03 00:00:00',106.64,971,1171,100.1,'Sun-Synch' from dual union all
select 4251,'004251','1969-101B','Star 37D S/N 017','Star 37D',18368,'In Earth orbit','','1969-11-22 00:00:00',647.83,217,36629,27.2,'GEO Transfer' from dual union all
select 42510,'042510','1970-025RQ','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',104.64,975,980,100.01,'Sun-Synch' from dual union all
select 42511,'042511','1970-025RR','deb Agena D','',21502,'In Earth orbit','','2017-04-09 00:00:00',113.36,1021,1741,100.16,'Sun-Synch' from dual union all
select 42512,'042512','1970-025RS','deb Agena D','',21502,'In Earth orbit','','2017-04-08 00:00:00',107.17,1091,1101,100.04,'Sun-Synch' from dual union all
select 42513,'042513','1970-025RT','deb Agena D','',21502,'In Earth orbit','','2017-04-02 00:00:00',104.05,940,960,100,'Sun-Synch' from dual union all
select 42514,'042514','1970-025RU','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',108.96,1037,1321,99.38,'Sun-Synch' from dual union all
select 42515,'042515','1970-025RV','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',106.59,1039,1098,99.4,'Sun-Synch' from dual union all
select 42516,'042516','1970-025RW','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',106.35,1017,1098,99.73,'Sun-Synch' from dual union all
select 42517,'042517','1970-025RX','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',107.41,1038,1175,100.19,'Sun-Synch' from dual union all
select 42518,'042518','1970-025RY','deb Agena D','',21502,'In Earth orbit','','2017-04-07 00:00:00',105.11,966,1034,100.45,'Sun-Synch' from dual union all
select 42519,'042519','1970-025RZ','deb Agena D','',21502,'In Earth orbit','','2016-02-28 00:00:00',104.39,951,981,99.49,'Sun-Synch' from dual union all
select 4252,'004252','1969-102A','Kosmos-311','DS-P1-Yu No. 27',18369,'Reentered','1970-03-10 00:00:00','1969-12-24 00:00:00',91.54,263,438,71.05,'Intermediate' from dual union all
select 42520,'042520','1970-025SA','deb Agena D','',21502,'In Earth orbit','','2017-04-02 00:00:00',104.53,898,1046,100.32,'Sun-Synch' from dual union all
select 42521,'042521','1970-025SB','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',104.62,967,987,99.99,'Sun-Synch' from dual union all
select 42522,'042522','1970-025SC','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',105.75,1006,1053,99.6,'Sun-Synch' from dual union all
select 42523,'042523','1970-025SD','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',104.85,968,1006,99.97,'Sun-Synch' from dual union all
select 42524,'042524','1970-025SE','deb Agena D','',21502,'In Earth orbit','','2017-04-08 00:00:00',105.35,986,1036,99.77,'Sun-Synch' from dual union all
select 42525,'042525','1970-025SF','deb Agena D','',21502,'In Earth orbit','','2017-03-20 00:00:00',97.73,543,757,99.91,'Sun-Synch' from dual union all
select 42526,'042526','1970-025SG','deb Agena D','',21502,'In Earth orbit','','2017-04-09 00:00:00',103.64,864,998,99.82,'Sun-Synch' from dual union all
select 42527,'042527','1970-025SH','deb Agena D','',21502,'In Earth orbit','','2017-04-09 00:00:00',101.73,815,867,99.15,'Sun-Synch' from dual union all
select 42528,'042528','1970-025SJ','deb Agena D','',21502,'In Earth orbit','','2017-04-07 00:00:00',101.06,791,826,99.74,'Sun-Synch' from dual union all
select 42529,'042529','1970-025SK','deb Agena D','',21502,'In Earth orbit','','2017-03-16 00:00:00',104.85,827,1147,98.67,'Sun-Synch' from dual union all
select 4253,'004253','1969-102B','S1M','S1M',18369,'Reentered','1970-01-19 00:00:00','1969-12-04 00:00:00',91.57,269,435,71.05,'Intermediate' from dual union all
select 42530,'042530','1970-025SL','deb Agena D','',21502,'In Earth orbit','','2017-04-07 00:00:00',106.22,854,1248,99.21,'Sun-Synch' from dual union all
select 42531,'042531','1970-025SM','deb Agena D','',21502,'In Earth orbit','','2017-04-06 00:00:00',106.75,1037,1116,99.47,'Sun-Synch' from dual union all
select 42532,'042532','1970-025SN','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',103.4,918,921,99.97,'Sun-Synch' from dual union all
select 42533,'042533','1970-025SP','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',104.35,854,1074,99.67,'Sun-Synch' from dual union all
select 42534,'042534','1970-025SQ','deb Agena D','',21502,'In Earth orbit','','2017-04-08 00:00:00',111.21,1135,1430,101.82,'Sun-Synch' from dual union all
select 42535,'042535','1970-025SR','deb Agena D','',21502,'In Earth orbit','','2017-04-08 00:00:00',105.05,937,1057,99.87,'Sun-Synch' from dual union all
select 42536,'042536','1970-025SS','deb Agena D','',21502,'In Earth orbit','','2017-04-08 00:00:00',103.43,864,978,99.5,'Sun-Synch' from dual union all
select 42537,'042537','1970-025ST','deb Agena D','',21502,'In Earth orbit','','2016-03-19 00:00:00',104.49,966,976,100.07,'Sun-Synch' from dual union all
select 42538,'042538','1970-025SU','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',106.35,1056,1059,99.87,'Sun-Synch' from dual union all
select 42539,'042539','1970-025SV','deb Agena D','',21502,'In Earth orbit','','2017-03-21 00:00:00',102.56,826,934,99.21,'Sun-Synch' from dual union all
select 4254,'004254','1969-103A','Kosmos-312','Sfera',18370,'In Earth orbit','','1969-12-26 00:00:00',108.54,1142,1177,74.03,'Intermediate' from dual union all
select 42540,'042540','1970-025SW','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',95.02,513,527,99.07,'Sun-Synch' from dual union all
select 42541,'042541','1970-025SX','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',105.06,971,1024,100.12,'Sun-Synch' from dual union all
select 42542,'042542','1970-025SY','deb Agena D','',21502,'In Earth orbit','','2017-03-28 00:00:00',102.64,803,965,99.18,'Sun-Synch' from dual union all
select 42543,'042543','1970-025SZ','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',108.66,1028,1302,99.89,'Sun-Synch' from dual union all
select 42544,'042544','1970-025TA','deb Agena D','',21502,'In Earth orbit','','2017-04-07 00:00:00',99.82,691,809,98.72,'Sun-Synch' from dual union all
select 42545,'042545','1970-025TB','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',101.69,836,842,99.95,'Sun-Synch' from dual union all
select 42546,'042546','1970-025TC','deb Agena D','',21502,'In Earth orbit','','2017-04-09 00:00:00',105.85,992,1077,100.07,'Sun-Synch' from dual union all
select 42547,'042547','1970-025TD','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',104.51,960,983,100.13,'Sun-Synch' from dual union all
select 42548,'042548','1970-025TE','deb Agena D','',21502,'In Earth orbit','','2017-04-07 00:00:00',102.21,814,913,99.59,'Sun-Synch' from dual union all
select 42549,'042549','1970-025TF','deb Agena D','',21502,'In Earth orbit','','2017-03-27 00:00:00',106.07,895,1194,99.45,'Sun-Synch' from dual union all
select 4255,'004255','1969-103B','S3M','S3M',18370,'In Earth orbit','','1970-01-06 00:00:00',108.39,1139,1166,74.03,'Intermediate' from dual union all
select 42550,'042550','1970-025TG','deb Agena D','',21502,'In Earth orbit','','2017-03-11 00:00:00',102.21,846,881,99.3,'Sun-Synch' from dual union all
select 42551,'042551','1970-025TH','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',103.41,909,931,99.95,'Sun-Synch' from dual union all
select 42552,'042552','1970-025TJ','deb Agena D','',21502,'In Earth orbit','','2017-04-05 00:00:00',107.73,986,1258,99.37,'Sun-Synch' from dual union all
select 42553,'042553','1970-025TK','deb Agena D','',21502,'In Earth orbit','','2017-04-10 00:00:00',109.87,1124,1318,100.34,'Sun-Synch' from dual union all
select 42554,'042554','1970-025TL','deb Agena D','',21502,'In Earth orbit','','2017-03-31 00:00:00',104.65,948,1009,100,'Sun-Synch' from dual union all
select 42555,'042555','1972-058JN','Unknown deb                       ','',27990,'In Earth orbit','','2017-04-06 00:00:00',98.13,662,677,98.04,'Sun-Synch' from dual union all
select 42556,'042556','1972-058JP','deb Delta 89','',27990,'In Earth orbit','','2017-04-09 00:00:00',102.26,639,1093,98.38,'Sun-Synch' from dual union all
select 42557,'042557','1972-058JQ','Unknown deb                       ','',27990,'In Earth orbit','','2017-04-08 00:00:00',100.78,787,805,98.96,'Sun-Synch' from dual union all
select 42558,'042558','1972-058JR','Unknown deb                       ','',27990,'In Earth orbit','','2017-04-10 00:00:00',98.58,662,719,98,'Sun-Synch' from dual union all
select 42559,'042559','1972-058JS','Unknown deb                       ','',27990,'In Earth orbit','','2017-04-10 00:00:00',96.42,585,590,98.25,'Sun-Synch' from dual union all
select 4256,'004256','1969-082B','OPS 7613 P/L 1','NRL PL176',18349,'In Earth orbit','','1969-12-12 00:00:00',103.44,905,938,70.01,'Intermediate' from dual union all
select 42560,'042560','1972-058JT','Unknown deb                       ','',27990,'In Earth orbit','','2017-04-10 00:00:00',99.55,666,809,97.89,'Sun-Synch' from dual union all
select 42561,'042561','1972-058JU','Unknown deb                       ','',27990,'In Earth orbit','','2017-04-10 00:00:00',110.36,869,1618,98.9,'Sun-Synch' from dual union all
select 42562,'042562','1972-058JV','deb Delta 89','',27990,'In Earth orbit','','2017-04-08 00:00:00',99.39,611,848,98.83,'Sun-Synch' from dual union all
select 42563,'042563','1972-058JW','Unknown deb                       ','',27990,'In Earth orbit','','2017-04-09 00:00:00',104.89,802,1177,98.76,'Sun-Synch' from dual union all
select 42564,'042564','1972-058JX','Unknown deb                       ','',27990,'In Earth orbit','','2017-03-25 00:00:00',103.07,818,989,99.06,'Sun-Synch' from dual union all
select 42565,'042565','1972-058JY','Unknown deb                       ','',27990,'In Earth orbit','','2017-04-01 00:00:00',99.15,692,744,98.02,'Sun-Synch' from dual union all
select 42566,'042566','1972-058JZ','deb Delta 89','',27990,'In Earth orbit','','2017-04-04 00:00:00',106.03,670,1416,98.15,'Sun-Synch' from dual union all
select 42567,'042567','1972-058KA','deb Delta 89','',27990,'In Earth orbit','','2017-04-09 00:00:00',98.55,602,777,98.59,'Sun-Synch' from dual union all
select 42568,'042568','1972-058KB','Unknown deb                       ','',27990,'In Earth orbit','','2017-04-09 00:00:00',101.62,814,857,98.76,'Sun-Synch' from dual union all
select 42569,'042569','1972-058KC','Unknown deb                       ','',27990,'In Earth orbit','','2017-04-10 00:00:00',107.72,848,1394,98.89,'Sun-Synch' from dual union all
select 4257,'004257','1969-082C','Timation 2','Timation 2',18349,'In Earth orbit','','1969-12-04 00:00:00',103.45,905,938,70.01,'Intermediate' from dual union all
select 42570,'042570','1972-058KD','deb Delta 89','',27990,'In Earth orbit','','2017-04-09 00:00:00',101.72,708,973,99.05,'Sun-Synch' from dual union all
select 42571,'042571','1972-058KE','deb Delta 89                      ','',27990,'In Earth orbit','','2017-04-09 00:00:00',96.43,573,603,99.05,'Sun-Synch' from dual union all
select 42572,'042572','1973-086HL','Unknown deb                       ','',30688,'In Earth orbit','','2017-03-23 00:00:00',114.95,996,1911,101.58,'Sun-Synch' from dual union all
select 42573,'042573','1973-086HM','deb Delta 98','',30688,'In Earth orbit','','2017-04-03 00:00:00',115.6,1449,1517,101.92,'Sun-Synch' from dual union all
select 42574,'042574','1973-086HN','deb Delta 98','',30688,'In Earth orbit','','2017-04-10 00:00:00',115.53,1454,1505,101.69,'Sun-Synch' from dual union all
select 42575,'042575','1973-086HP','deb Delta 98','',30688,'In Earth orbit','','2017-01-16 00:00:00',113.26,1256,1496,103.23,'Sun-Synch' from dual union all
select 42576,'042576','1973-086HQ','deb Delta 98','',30688,'In Earth orbit','','2017-04-09 00:00:00',112.86,1123,1593,102.19,'Sun-Synch' from dual union all
select 42577,'042577','1973-086HR','deb Delta 98','',30688,'In Earth orbit','','2017-04-09 00:00:00',113.42,1326,1441,101.1,'Sun-Synch' from dual union all
select 42578,'042578','1973-086HS','deb Delta 98','',30688,'In Earth orbit','','2017-04-07 00:00:00',127.74,1313,2735,100.74,'Medium' from dual;
insert into SATELLITE
select 42579,'042579','1973-086HT','deb Delta 98','',30688,'In Earth orbit','','2017-03-27 00:00:00',117.41,1502,1627,102.28,'Sun-Synch' from dual union all
select 4258,'004258','1969-082DB','deb Agena D','',18349,'Reentered','1989-09-16 00:00:00','1989-09-16 00:00:00',89.69,256,263,70.05,'Intermediate' from dual union all
select 42580,'042580','1973-086HU','deb Delta 98','',30688,'In Earth orbit','','2017-04-10 00:00:00',105.04,938,1055,101.48,'Sun-Synch' from dual union all
select 42581,'042581','1973-086HV','deb Delta 98','',30688,'In Earth orbit','','2017-03-03 00:00:00',113.61,1336,1449,102.22,'Sun-Synch' from dual union all
select 42582,'042582','1973-086HW','deb Delta 98','',30688,'In Earth orbit','','2017-04-07 00:00:00',115.67,1473,1499,101.5,'Sun-Synch' from dual union all
select 42583,'042583','1973-086HX','deb Delta 98','',30688,'In Earth orbit','','2017-04-07 00:00:00',116.74,1455,1614,102.02,'Sun-Synch' from dual union all
select 42584,'042584','1973-086HY','deb Delta 98','',30688,'In Earth orbit','','2017-04-08 00:00:00',116.41,1504,1535,102.05,'Sun-Synch' from dual union all
select 42585,'042585','1973-086HZ','deb Delta 98','',30688,'In Earth orbit','','2017-04-10 00:00:00',111.9,1125,1503,102.04,'Sun-Synch' from dual union all
select 42586,'042586','1973-086JA','deb Delta 98','',30688,'In Earth orbit','','2017-04-10 00:00:00',116.56,1499,1554,102.13,'Sun-Synch' from dual union all
select 42587,'042587','1973-086JB','deb Delta 98','',30688,'In Earth orbit','','2017-04-06 00:00:00',117.79,1507,1657,102.18,'Sun-Synch' from dual union all
select 42588,'042588','1973-086JC','deb Delta 98','',30688,'In Earth orbit','','2017-04-01 00:00:00',116.52,1471,1578,102.02,'Sun-Synch' from dual union all
select 42589,'042589','1973-086JD','deb Delta 98','',30688,'In Earth orbit','','2017-04-02 00:00:00',110.26,997,1480,102.54,'Sun-Synch' from dual union all
select 4259,'004259','1969-082D','OPS 7613 P/L 3','POPPY 6 SV1',18349,'In Earth orbit','','1969-12-11 00:00:00',103.45,906,938,70.02,'Intermediate' from dual union all
select 42590,'042590','1973-086JE','deb Delta 98','',30688,'In Earth orbit','','2017-04-03 00:00:00',113.49,1342,1431,101.22,'Sun-Synch' from dual union all
select 42591,'042591','1974-089BV','deb Delta 104','',33692,'In Earth orbit','','2017-04-10 00:00:00',116.71,1442,1623,101.71,'Sun-Synch' from dual union all
select 42592,'042592','1974-089CW','deb Delta 104','',33692,'In Earth orbit','','2017-03-31 00:00:00',115.18,1438,1490,101.42,'Sun-Synch' from dual union all
select 42593,'042593','1974-089FJ','deb Delta 104','',33692,'In Earth orbit','','2017-04-09 00:00:00',121.74,1458,2060,101.19,'Medium' from dual union all
select 42594,'042594','1974-089FK','deb Delta 104','',33692,'In Earth orbit','','2017-04-02 00:00:00',117.83,1450,1717,100.87,'Sun-Synch' from dual union all
select 42595,'042595','1974-089FL','deb Delta 104','',33692,'In Earth orbit','','2017-03-31 00:00:00',115.54,1455,1505,101.69,'Sun-Synch' from dual union all
select 42596,'042596','1974-089FM','deb Delta 104','',33692,'In Earth orbit','','2017-04-10 00:00:00',117.47,1452,1683,101.49,'Sun-Synch' from dual union all
select 42597,'042597','1974-089FN','deb Delta 104','',33692,'In Earth orbit','','2017-04-10 00:00:00',113.55,1234,1545,101.19,'Sun-Synch' from dual union all
select 42598,'042598','1974-089FP','deb Delta 104','',33692,'In Earth orbit','','2017-04-11 00:00:00',116.06,1356,1651,102.8,'Sun-Synch' from dual union all
select 42599,'042599','1974-089FQ','deb Delta 104','',33692,'In Earth orbit','','2017-04-10 00:00:00',114.15,1382,1452,101.57,'Sun-Synch' from dual union all
select 426,'000426','62 B Alpha 2','TAVE','TAVE',8161,'In Earth orbit','','1962-11-10 00:00:00',105.34,998,1023,80.48,'Intermediate' from dual union all
select 4260,'004260','1969-082DD','deb Agena D','',18349,'Reentered','1990-09-29 00:00:00','1969-12-18 00:00:00',103.43,875,967,70.44,'Intermediate' from dual union all
select 42600,'042600','1974-089FR','deb Delta 104','',33692,'In Earth orbit','','2016-08-22 00:00:00',115.05,1417,1499,101.57,'Sun-Synch' from dual union all
select 42601,'042601','1974-089FS','deb Delta 104','',33692,'In Earth orbit','','2017-03-18 00:00:00',110.4,1002,1488,101.43,'Sun-Synch' from dual union all
select 42602,'042602','1974-089FT','deb Delta 104','',33692,'In Earth orbit','','2016-10-25 00:00:00',118.99,1372,1900,101.44,'Sun-Synch' from dual union all
select 42603,'042603','1974-089FU','deb Delta 104','',33692,'In Earth orbit','','2017-04-11 00:00:00',118.39,1438,1779,100.88,'Sun-Synch' from dual union all
select 42604,'042604','1974-089FV','deb Delta 104','',33692,'In Earth orbit','','2017-02-07 00:00:00',124.47,1402,2358,103,'Medium' from dual union all
select 42605,'042605','1974-089FW','deb Delta 104','',33692,'In Earth orbit','','2017-02-26 00:00:00',117.16,1453,1654,101.43,'Sun-Synch' from dual union all
select 42606,'042606','1974-089FX','deb Delta 104','',33692,'In Earth orbit','','2017-04-09 00:00:00',113.24,1230,1520,102.21,'Sun-Synch' from dual union all
select 42607,'042607','1974-089FY','deb Delta 104','',33692,'In Earth orbit','','2017-04-08 00:00:00',101.8,814,873,101.88,'Sun-Synch' from dual union all
select 42608,'042608','1974-089FZ','deb Delta 104','',33692,'In Earth orbit','','2017-04-11 00:00:00',117.25,1498,1616,102.07,'Sun-Synch' from dual union all
select 42609,'042609','1974-089GA','deb Delta 104','',33692,'In Earth orbit','','2017-04-10 00:00:00',113.7,1362,1431,101.19,'Sun-Synch' from dual union all
select 4261,'004261','1969-097E','Azur despin weight 3','',18364,'Reentered','1980-07-11 00:00:00','1969-12-11 00:00:00',121.96,392,3145,103.69,'Medium' from dual union all
select 42610,'042610','1974-089GB','deb Delta 104','',33692,'In Earth orbit','','2017-04-10 00:00:00',116.12,1443,1570,101.34,'Sun-Synch' from dual union all
select 42611,'042611','1974-089GC','deb Delta 104','',33692,'In Earth orbit','','2017-04-10 00:00:00',117.36,1458,1667,101.09,'Sun-Synch' from dual union all
select 42612,'042612','1974-089GD','deb Delta 104','',33692,'In Earth orbit','','2017-04-08 00:00:00',113.44,1316,1453,101.22,'Sun-Synch' from dual union all
select 42613,'042613','1974-089GE','deb Delta 104','',33692,'In Earth orbit','','2017-04-05 00:00:00',115.77,1437,1543,101.83,'Sun-Synch' from dual union all
select 42614,'042614','1974-089GF','deb Delta 104','',33692,'In Earth orbit','','2017-04-11 00:00:00',114.58,1408,1465,101.77,'Sun-Synch' from dual union all
select 42615,'042615','1974-089GG','deb Delta 104','',33692,'In Earth orbit','','2017-04-10 00:00:00',114.3,1361,1487,101.33,'Sun-Synch' from dual union all
select 42616,'042616','1974-089GH','deb Delta 104','',33692,'In Earth orbit','','2017-02-28 00:00:00',124.26,1448,2293,101.36,'Medium' from dual union all
select 42617,'042617','1974-089GJ','deb Delta 104','',33692,'In Earth orbit','','2017-01-13 00:00:00',118.11,1449,1742,101.54,'Sun-Synch' from dual union all
select 42618,'042618','1974-089GK','deb Delta 104','',33692,'In Earth orbit','','2017-03-08 00:00:00',116.72,1497,1570,102.05,'Sun-Synch' from dual union all
select 42619,'042619','1974-089GL','deb Delta 104','',33692,'In Earth orbit','','2017-04-11 00:00:00',107.72,982,1261,102.08,'Sun-Synch' from dual union all
select 4262,'004262','1969-104A','Kosmos-313','Zenit-2M',18371,'Landed','1969-12-15 00:00:00','1969-12-05 00:00:00',88.99,198,252,65.4,'Intermediate' from dual union all
select 42620,'042620','1974-089GM','deb Delta 104','',33692,'In Earth orbit','','2017-03-31 00:00:00',117.74,1461,1698,101.16,'Sun-Synch' from dual union all
select 42621,'042621','1974-089GN','deb Delta 104','',33692,'In Earth orbit','','2017-04-10 00:00:00',121.51,1443,2054,101.02,'Medium' from dual union all
select 42622,'042622','1974-089GP','deb Delta 104','',33692,'In Earth orbit','','2017-04-10 00:00:00',117.62,1452,1696,101.39,'Sun-Synch' from dual union all
select 42623,'042623','1974-089GQ','deb Delta 104','',33692,'In Earth orbit','','2017-04-07 00:00:00',115.77,1434,1547,101.86,'Sun-Synch' from dual union all
select 42624,'042624','1974-089GR','deb Delta 104','',33692,'In Earth orbit','','2016-08-25 00:00:00',113.03,1317,1415,101.15,'Sun-Synch' from dual union all
select 42625,'042625','1974-089GS','deb Delta 104','',33692,'In Earth orbit','','2017-04-07 00:00:00',115.81,1487,1498,101.66,'Sun-Synch' from dual union all
select 42626,'042626','1974-089GT','deb Delta 104','',33692,'In Earth orbit','','2017-03-28 00:00:00',114.39,1331,1524,101.28,'Sun-Synch' from dual union all
select 42627,'042627','1974-089GU','deb Delta 104','',33692,'In Earth orbit','','2017-03-31 00:00:00',114.56,1388,1483,101.37,'Sun-Synch' from dual union all
select 42628,'042628','1974-089GV','deb Delta 104','',33692,'In Earth orbit','','2017-04-11 00:00:00',119.32,1443,1858,101.17,'Sun-Synch' from dual union all
select 42629,'042629','1975-052LN','deb Delta 111','',36499,'In Earth orbit','','2017-04-09 00:00:00',108.22,1090,1199,99.86,'Sun-Synch' from dual union all
select 4263,'004263','1969-104B','11A57I','11A57I',18371,'Reentered','1969-12-06 00:00:00','1969-12-05 00:00:00',88.62,175,238,65.4,'Intermediate' from dual union all
select 42630,'042630','1975-052LP','deb Delta 111','',36499,'In Earth orbit','','2017-04-11 00:00:00',99.67,635,850,99.91,'Sun-Synch' from dual union all
select 42631,'042631','1975-052LQ','deb Delta 111','',36499,'In Earth orbit','','2017-04-11 00:00:00',106.21,1017,1084,100.04,'Sun-Synch' from dual union all
select 42632,'042632','1975-052LR','deb Delta 111','',36499,'In Earth orbit','','2017-04-11 00:00:00',113.44,1075,1694,99.56,'Sun-Synch' from dual union all
select 42633,'042633','1975-052LS','deb Delta 111','',36499,'In Earth orbit','','2017-04-11 00:00:00',108.78,1075,1266,99.66,'Sun-Synch' from dual union all
select 42634,'042634','1975-052LT','deb Delta 111','',36499,'In Earth orbit','','2017-04-11 00:00:00',107.97,1086,1180,99.94,'Sun-Synch' from dual union all
select 42635,'042635','1975-052LU','deb Delta 111','',36499,'In Earth orbit','','2017-04-11 00:00:00',106.21,1017,1084,100.04,'Sun-Synch' from dual union all
select 42636,'042636','1975-052LV','deb Delta 111','',36499,'In Earth orbit','','2017-04-10 00:00:00',109.04,1081,1284,99.68,'Sun-Synch' from dual union all
select 42637,'042637','1975-052LW','deb Delta 111','',36499,'In Earth orbit','','2017-04-10 00:00:00',109.27,1087,1298,99.86,'Sun-Synch' from dual union all
select 42638,'042638','1975-052LX','deb Delta 111','',36499,'In Earth orbit','','2017-04-07 00:00:00',106.16,1012,1086,99.75,'Sun-Synch' from dual union all
select 42639,'042639','1975-052LY','deb Delta 111','',36499,'In Earth orbit','','2017-04-08 00:00:00',108.15,1088,1195,99.96,'Sun-Synch' from dual union all
select 4264,'004264','1969-105A','OPS 6617','CORONA CR-9',18372,'Reentered Attached','1970-01-10 00:00:00','1969-12-11 00:00:00',88.47,170,228,81.5,'Intermediate' from dual union all
select 42640,'042640','1975-052LZ','deb Delta 111','',36499,'In Earth orbit','','2017-04-10 00:00:00',107.27,934,1266,99.9,'Sun-Synch' from dual union all
select 42641,'042641','1975-052MA','deb Delta 111','',36499,'In Earth orbit','','2017-04-11 00:00:00',107.88,1083,1175,99.62,'Sun-Synch' from dual union all
select 42642,'042642','1975-052MB','deb Delta 111','',36499,'In Earth orbit','','2017-03-29 00:00:00',108.2,1087,1200,99.76,'Sun-Synch' from dual union all
select 42643,'042643','1975-052MC','deb Delta 111','',36499,'In Earth orbit','','2017-04-05 00:00:00',111.78,1083,1534,99.66,'Sun-Synch' from dual union all
select 42644,'042644','1975-052MD','deb Delta 111','',36499,'In Earth orbit','','2017-04-07 00:00:00',109.35,1076,1318,99.57,'Sun-Synch' from dual union all
select 42645,'042645','1975-052ME','deb Delta 111','',36499,'In Earth orbit','','2017-03-29 00:00:00',111.61,1095,1506,99.74,'Sun-Synch' from dual union all
select 42646,'042646','1975-052MF','deb Delta 111','',36499,'In Earth orbit','','2017-04-11 00:00:00',107.83,1085,1168,99.82,'Sun-Synch' from dual union all
select 42647,'042647','1975-052MG','deb Delta 111','',36499,'In Earth orbit','','2017-04-09 00:00:00',108.08,1088,1188,99.92,'Sun-Synch' from dual union all
select 42648,'042648','1975-052MH','deb Delta 111','',36499,'In Earth orbit','','2017-04-01 00:00:00',108.36,970,1332,99.77,'Sun-Synch' from dual union all
select 42649,'042649','1975-052MJ','deb Delta 111','',36499,'In Earth orbit','','2017-04-04 00:00:00',107.57,1087,1142,100.15,'Sun-Synch' from dual union all
select 4265,'004265','1969-097F','Azur despin weight 4','',18364,'Reentered','1981-04-13 00:00:00','1969-12-10 00:00:00',121.4,378,3108,102.26,'Medium' from dual union all
select 42650,'042650','1975-052MK','deb Delta 111','',36499,'In Earth orbit','','2017-04-06 00:00:00',107.48,930,1291,99.65,'Sun-Synch' from dual union all
select 42651,'042651','1975-052ML','deb Delta 111','',36499,'In Earth orbit','','2017-04-09 00:00:00',107.46,991,1228,99.24,'Sun-Synch' from dual union all
select 42652,'042652','1975-052MM','deb Delta 111','',36499,'In Earth orbit','','2017-04-09 00:00:00',108.08,1083,1193,99.91,'Sun-Synch' from dual union all
select 42653,'042653','1975-052MN','deb Delta 111','',36499,'In Earth orbit','','2017-04-10 00:00:00',108.2,1091,1196,99.94,'Sun-Synch' from dual union all
select 42654,'042654','1975-052MP','deb Delta 111','',36499,'In Earth orbit','','2017-04-04 00:00:00',111.49,1086,1504,99.74,'Sun-Synch' from dual union all
select 42655,'042655','1975-052MQ','deb Delta 111','',36499,'In Earth orbit','','2017-03-06 00:00:00',111.02,844,1704,99.65,'Sun-Synch' from dual union all
select 42656,'042656','1975-052MR','deb Delta 111','',36499,'In Earth orbit','','2017-04-10 00:00:00',107.98,1081,1186,99.93,'Sun-Synch' from dual union all
select 42657,'042657','1975-052MS','deb Delta 111','',36499,'In Earth orbit','','2017-04-11 00:00:00',106.33,1022,1092,99.98,'Sun-Synch' from dual union all
select 42658,'042658','1975-052MT','deb Delta 111','',36499,'In Earth orbit','','2017-03-27 00:00:00',114.21,1078,1761,99.83,'Sun-Synch' from dual union all
select 42659,'042659','1975-052MU','deb Delta 111','',36499,'In Earth orbit','','2017-04-09 00:00:00',97.71,648,651,98.35,'Sun-Synch' from dual union all
select 4266,'004266','1969-106A','Kosmos-314','DS-P1-Yu No. 30',18373,'Reentered','1970-03-22 00:00:00','1970-01-14 00:00:00',91.43,263,427,71,'Intermediate' from dual union all
select 42660,'042660','1975-052MV','deb Delta 111','',36499,'In Earth orbit','','2017-04-10 00:00:00',103.31,703,1128,99.49,'Sun-Synch' from dual union all
select 42661,'042661','1975-052MW','deb Delta 111','',36499,'In Earth orbit','','2017-04-06 00:00:00',100.18,765,769,98.47,'Sun-Synch' from dual union all
select 42662,'042662','2017-018A','Shi Jian 13','Zhongxing 16',70408,'In Earth orbit','','2017-04-25 00:00:00',1436.09,35768,35804,.07,'Stationary' from dual union all
select 42663,'042663','2017-018B','CZ-3B Stage 3','CZ-3B Stage 3',70408,'In Earth orbit','','2017-04-19 00:00:00',725.36,228,40500,20.94,'GEO Transfer' from dual union all
select 42664,'042664','61 A Delta 10','West Ford package fragment','',7251,'In Earth orbit','','2017-04-05 00:00:00',165.69,3286,3944,95.84,'Medium' from dual union all
select 42665,'042665','61 A Delta 11','West Ford package fragment','',7251,'In Earth orbit','','2017-03-13 00:00:00',165.38,2725,4480,95.88,'Medium' from dual union all
select 42666,'042666','1963-049P','deb Transit VBN-2','',9280,'In Earth orbit','','2017-04-13 00:00:00',104.78,941,1028,89.93,'Polar' from dual union all
select 42667,'042667','1966-041J','deb NNS O-9 insulation','',13448,'In Earth orbit','','2017-04-14 00:00:00',100.91,769,835,90.13,'Polar' from dual union all
select 42668,'042668','1968-012G','deb NNS O-18 insulation','',16739,'In Earth orbit','','2017-04-14 00:00:00',104.86,918,1058,89.96,'Polar' from dual;
insert into SATELLITE
select 42669,'042669','1991-050T','deb ERS-1','',64219,'In Earth orbit','','2017-04-14 00:00:00',100.14,743,787,98.45,'Sun-Synch' from dual union all
select 4267,'004267','1969-106B','S1M','S1M',18373,'Reentered','1970-01-27 00:00:00','1969-12-18 00:00:00',91.47,270,424,71,'Intermediate' from dual union all
select 42670,'042670','2015-069C','CZ-4C Sep motor cover','',70007,'In Earth orbit','','2017-04-11 00:00:00',95.61,481,616,97.83,'Sun-Synch' from dual union all
select 42671,'042671','2015-069D','CZ-4C Sep motor cover','',70007,'In Earth orbit','','2017-04-14 00:00:00',99.02,595,829,97.89,'Sun-Synch' from dual union all
select 42672,'042672','2015-069E','CZ-4C Sep motor cover','',70007,'In Earth orbit','','2017-04-13 00:00:00',98.78,584,817,97.95,'Sun-Synch' from dual union all
select 42673,'042673','2015-069F','deb Yaogan 29','',70007,'In Earth orbit','','2017-04-13 00:00:00',96.97,611,617,97.89,'Sun-Synch' from dual union all
select 42674,'042674','2016-012S','deb ASTRO-H','',70190,'In Earth orbit','','2017-04-14 00:00:00',95.75,548,562,31,'Intermediate' from dual union all
select 42675,'042675','2016-012T','deb ASTRO-H','',70190,'In Earth orbit','','2017-04-14 00:00:00',95.81,553,563,31.01,'Intermediate' from dual union all
select 42676,'042676','2016-029B','CZ-2C Sep motor cover','',70207,'In Earth orbit','','2017-04-14 00:00:00',101.26,633,1004,98.07,'Sun-Synch' from dual union all
select 42677,'042677','2016-029C','CZ-2C Sep motor cover','',70207,'In Earth orbit','','2017-04-12 00:00:00',101.33,636,1007,97.93,'Sun-Synch' from dual union all
select 42678,'042678','2016-049C','deb Gaofen-3','',70227,'In Earth orbit','','2017-04-12 00:00:00',99.63,734,747,98.42,'Sun-Synch' from dual union all
select 42679,'042679','2016-049D','deb Gaofen-3','',70227,'In Earth orbit','','2017-04-12 00:00:00',99.63,734,747,98.42,'Sun-Synch' from dual union all
select 4268,'004268','1969-082DE','deb Agena D','',18349,'Reentered','1990-01-19 00:00:00','1990-01-18 00:00:00',90.95,311,331,69.81,'Intermediate' from dual union all
select 42680,'042680','2016-049E','deb Gaofen-3','',70227,'In Earth orbit','','2017-04-12 00:00:00',99.63,734,747,98.42,'Sun-Synch' from dual union all
select 42681,'042681','2017-019A','SS John Glenn','Cygnus PCM-7',70409,'Deorbited','2017-06-11 00:00:00','2017-04-21 00:00:00',92.55,395,404,51.64,'Intermediate' from dual union all
select 42682,'042682','2017-020A','Soyuz MS-04','Soyuz 11F732A48 No. 735',70410,'Landed','2017-09-03 00:00:00','2017-04-20 00:00:00',92.64,399,409,51.64,'Intermediate' from dual union all
select 42683,'042683','2017-020B','11S510','11S510',70410,'Reentered','2017-04-23 00:00:00','2017-04-21 00:00:00',88.37,180,208,51.62,'Intermediate' from dual union all
select 42684,'042684','2017-021A','Tianzhou-1','Tianzhou-1',70411,'Deorbited','2017-09-22 00:00:00','2017-04-21 00:00:00',91.98,367,377,42.78,'Intermediate' from dual union all
select 42685,'042685','2017-021B','CZ-7 Stage 2','CZ-7  Stage 2',70411,'Reentered','2017-05-18 00:00:00','2017-04-27 00:00:00',89.79,197,331,42.79,'Intermediate' from dual union all
select 42686,'042686','2017-021C','CZ-7 sep motor cover','',70411,'Reentered','2017-04-27 00:00:00','2017-04-22 00:00:00',91.24,192,479,42.46,'Intermediate' from dual union all
select 42687,'042687','2017-021D','CZ-7 sep motor cover','',70411,'Reentered','2017-04-26 00:00:00','2017-04-22 00:00:00',90.99,187,460,42.63,'Intermediate' from dual union all
select 42688,'042688','2017-021E','CZ-7 sep motor cover','',70411,'Reentered','2017-04-28 00:00:00','2017-04-22 00:00:00',91.1,193,464,43.11,'Intermediate' from dual union all
select 42689,'042689','2017-022A','USA 276','NROL-76',70412,'In Earth orbit','','2017-05-01 00:00:00',90.52,300,300,51,'Intermediate' from dual union all
select 4269,'004269','1969-082DF','deb Agena D','',18349,'Reentered','1979-03-29 00:00:00','1979-03-17 00:00:00',96.9,595,626,70,'Intermediate' from dual union all
select 42690,'042690','2017-022B','Falcon 9-035 Stage 2','F9-035 St2',70412,'Deorbited','2017-05-01 00:00:00','2017-05-01 00:00:00',90.52,300,300,51,'Intermediate' from dual union all
select 42691,'042691','2017-023A','Koreasat 7','Koreasat 7',70413,'In Earth orbit','','2017-05-12 00:00:00',1433.64,35686,35791,.05,'Drift GEO' from dual union all
select 42692,'042692','2017-023B','SGDC','SGDC',70413,'In Earth orbit','','2017-05-11 00:00:00',1429.48,35557,35756,.05,'Drift GEO' from dual union all
select 42693,'042693','2017-023C','ESC-A 589','Ariane 5 ESC-A',70413,'In Earth orbit','','2017-05-12 00:00:00',632.69,266,35804,4.06,'GEO Transfer' from dual union all
select 42694,'042694','2017-023D','Sylda 5A','Sylda 5A',70413,'In Earth orbit','','2017-05-14 00:00:00',630.63,277,35687,4.02,'GEO Transfer' from dual union all
select 42695,'042695','2017-024A','South Asia Satellite','GSAT 9',70414,'In Earth orbit','','2017-05-12 00:00:00',1433.81,35633,35851,.06,'Drift GEO' from dual union all
select 42696,'042696','2017-024B','GSLV CUS-08','GSLV-F09 CUS',70414,'Reentered','2017-10-10 00:00:00','2017-05-12 00:00:00',627.1,123,35659,20.61,'GEO Transfer' from dual union all
select 42697,'042697','1998-067LG','deb MBSU','',66503,'Reentered','2017-10-28 00:00:00','2017-05-08 00:00:00',92.64,401,407,51.64,'Intermediate' from dual union all
select 42698,'042698','2017-025A','Inmarsat 5F4','Inmarsat 5F4',70415,'In Earth orbit','','2017-05-18 00:00:00',1408.82,384,70118,24.52,'Near-synch' from dual union all
select 42699,'042699','2017-025B','Falcon 9-034 Stage 2','F9-034 St2',70415,'In Earth orbit','','2017-05-17 00:00:00',1401.49,378,69835,24.47,'Near-synch' from dual union all
select 427,'000427','62 B Beta','FTV 1154','CORONA M-13',8162,'Reentered Attached','1962-10-08 00:00:00','1962-10-01 00:00:00',90.22,200,371,65.37,'Intermediate' from dual union all
select 4270,'004270','1969-082DG','deb Agena D','',18349,'Reentered','1980-06-16 00:00:00','1969-12-25 00:00:00',103.45,907,937,70.03,'Intermediate' from dual union all
select 42700,'042700','1998-067LH','SOMP 2','QB50-DE02',66503,'In Earth orbit','','2017-05-17 00:00:00',92.6,398,406,51.64,'Intermediate' from dual union all
select 42701,'042701','1998-067LJ','HAVELSAT','QB50-TR02',66503,'In Earth orbit','','2017-05-17 00:00:00',92.61,397,408,51.63,'Intermediate' from dual union all
select 42702,'042702','1998-067LK','Columbia Sat','QB50-US04/QBUS 4',66503,'Reentered','2018-03-07 00:00:00','2017-05-17 00:00:00',92.61,401,404,51.64,'Intermediate' from dual union all
select 42703,'042703','1998-067LL','SGSat','KySat-3',66503,'Reentered','2018-05-06 00:00:00','2017-05-17 00:00:00',92.61,401,404,51.64,'Intermediate' from dual union all
select 42704,'042704','1998-067LM','CXBN-2','CXBN-2',66503,'In Earth orbit','','2017-05-17 00:00:00',92.62,400,406,51.64,'Intermediate' from dual union all
select 42705,'042705','1998-067LN','IceCube','IceCube',66503,'In Earth orbit','','2017-05-17 00:00:00',92.61,401,404,51.64,'Intermediate' from dual union all
select 42706,'042706','1998-067LP','Phoenix','QB50-TW01',66503,'In Earth orbit','','2017-05-17 00:00:00',92.61,402,403,51.63,'Intermediate' from dual union all
select 42707,'042707','1998-067LQ','X-Cubesat','QB50-FR01',66503,'In Earth orbit','','2017-05-17 00:00:00',92.6,401,403,51.64,'Intermediate' from dual union all
select 42708,'042708','1998-067LR','qbee50-LTU-OC','QB50-SE01',66503,'In Earth orbit','','2017-05-17 00:00:00',92.6,401,403,51.64,'Intermediate' from dual union all
select 42709,'042709','2017-026A','SES-15','SES-15',70416,'In Earth orbit','','2017-05-20 00:00:00',583.99,2215,31316,5.94,'Highly Ellip' from dual union all
select 4271,'004271','1969-082DH','deb Agena D','',18349,'Reentered','1980-02-23 00:00:00','1969-12-19 00:00:00',101.61,760,910,69.96,'Intermediate' from dual union all
select 42710,'042710','2017-026B','Fregat No. 133-09','Fregat-M     No. 133-09',70416,'In Earth orbit','','2017-05-20 00:00:00',586.7,2350,31324,6.15,'Highly Ellip' from dual union all
select 42711,'042711','1998-067LS','Altair Pathfinder','Altair 1',66503,'In Earth orbit','','2017-05-19 00:00:00',92.61,401,404,51.64,'Intermediate' from dual union all
select 42712,'042712','1998-067LT','SHARC','SHARC',66503,'In Earth orbit','','2017-05-19 00:00:00',92.62,400,406,51.64,'Intermediate' from dual union all
select 42713,'042713','1998-067LU','ZA-AeroSat','QB50-ZA01',66503,'In Earth orbit','','2017-05-19 00:00:00',92.61,402,403,51.65,'Intermediate' from dual union all
select 42714,'042714','1998-067LV','LINK','QB50-KR01',66503,'In Earth orbit','','2017-05-19 00:00:00',92.61,401,404,51.63,'Intermediate' from dual union all
select 42715,'042715','1998-067LW','CSUNSat 1','CSUNSat 1',66503,'In Earth orbit','','2017-05-19 00:00:00',92.62,402,404,51.65,'Intermediate' from dual union all
select 42716,'042716','1998-067LX','UPSat','QB50-GR02',66503,'In Earth orbit','','2017-05-19 00:00:00',92.62,399,407,51.64,'Intermediate' from dual union all
select 42717,'042717','1998-067LY','SpaceCube','QB50-FR05',66503,'In Earth orbit','','2017-05-23 00:00:00',92.6,400,403,51.64,'Intermediate' from dual union all
select 42718,'042718','1998-067LZ','Hoopoe','QB50-IL01',66503,'In Earth orbit','','2017-05-23 00:00:00',92.6,400,403,51.64,'Intermediate' from dual union all
select 42719,'042719','2017-027A','Kosmos-2518','EKS No. 2L',70417,'In Earth orbit','','2017-05-27 00:00:00',713.85,1650,38511,63.81,'Molniya' from dual union all
select 4272,'004272','1969-082DJ','deb Agena D','',18349,'Reentered','1970-09-09 00:00:00','1970-09-06 00:00:00',92.11,348,408,69.69,'Intermediate' from dual union all
select 42720,'042720','2017-027B','Fregat No. 111-301','Fregat 14S44 No. 111-301',70417,'In Earth orbit','','2017-05-28 00:00:00',713.14,1664,38462,63.89,'Molniya' from dual union all
select 42721,'042721','1998-067MA','Challenger','QB50-US01/QBUS 1',66503,'In Earth orbit','','2017-05-26 00:00:00',92.62,398,408,51.64,'Intermediate' from dual union all
select 42722,'042722','1998-067MB','NJUST-1','QB50-CN03/BE03',66503,'In Earth orbit','','2017-05-25 00:00:00',92.62,398,408,51.65,'Intermediate' from dual union all
select 42723,'042723','1998-067MC','UNSW-EC0','QB50-AU02',66503,'In Earth orbit','','2017-05-25 00:00:00',92.61,397,408,51.64,'Intermediate' from dual union all
select 42724,'042724','1998-067MD','DUTHSat','QB50-GR01',66503,'In Earth orbit','','2017-05-26 00:00:00',92.62,399,407,51.64,'Intermediate' from dual union all
select 42725,'042725','1998-067ME','Zidingxiang-1','QB50-CN02/BE02',66503,'In Earth orbit','','2017-05-26 00:00:00',92.61,402,403,51.64,'Intermediate' from dual union all
select 42726,'042726','1998-067MF','nSIGHT 1','QB50-AZ02',66503,'In Earth orbit','','2017-05-26 00:00:00',92.61,399,406,51.64,'Intermediate' from dual union all
select 42727,'042727','1998-067MG','SNUSAT 1b','QB50-KR03',66503,'In Earth orbit','','2017-05-26 00:00:00',92.61,397,408,51.64,'Intermediate' from dual union all
select 42728,'042728','1998-067MH','QBITO','QB50-ES01',66503,'In Earth orbit','','2017-05-27 00:00:00',92.62,400,406,51.64,'Intermediate' from dual union all
select 42729,'042729','1998-067MJ','Aalto-2','QB50-FI01',66503,'In Earth orbit','','2017-05-27 00:00:00',92.61,399,406,51.64,'Intermediate' from dual union all
select 4273,'004273','1969-107A','Kosmos-315','Tselina-OM',18374,'Reentered','1979-03-25 00:00:00','1970-01-21 00:00:00',95.17,516,538,74.04,'Intermediate' from dual union all
select 42730,'042730','1998-067MK','SUSat','QB50-AU01',66503,'In Earth orbit','','2017-05-27 00:00:00',92.62,397,408,51.63,'Intermediate' from dual union all
select 42731,'042731','1998-067ML','i-INSPIRE 2','QB50-AU03',66503,'In Earth orbit','','2017-05-27 00:00:00',92.61,398,407,51.64,'Intermediate' from dual union all
select 42732,'042732','1998-067MM','PolyITAN-2-SAU','QB50-UA01',66503,'In Earth orbit','','2017-05-27 00:00:00',92.61,397,408,51.63,'Intermediate' from dual union all
select 42733,'042733','1998-067MN','SNUSAT 1','QB50-KR02',66503,'In Earth orbit','','2017-05-27 00:00:00',92.62,398,408,51.64,'Intermediate' from dual union all
select 42734,'042734','1998-067MP','Ex-Alta 1','QB50-CA03',66503,'In Earth orbit','','2017-05-27 00:00:00',92.62,397,408,51.64,'Intermediate' from dual union all
select 42735,'042735','1998-067MQ','Aoxiang yi hao','QB50-CN04/BE04',66503,'In Earth orbit','','2017-05-27 00:00:00',92.62,397,408,51.64,'Intermediate' from dual union all
select 42736,'042736','1998-067MR','BeEagleSat','QB50-TR01',66503,'In Earth orbit','','2017-05-27 00:00:00',92.62,395,411,51.63,'Intermediate' from dual union all
select 42737,'042737','1998-067MS','Atlantis','QB50-US02/QBUS 2',66503,'Reentered','2018-03-28 00:00:00','2017-05-27 00:00:00',92.61,396,409,51.63,'Intermediate' from dual union all
select 42738,'042738','2017-028A','Michibiki-2','QZSS 2',70418,'In Earth orbit','','2017-06-05 00:00:00',1444.52,35912,35991,44.76,'Near-synch' from dual union all
select 42739,'042739','2017-028B','H-2A F34 Stage 2','H-2A Stage 2',70418,'In Earth orbit','','2017-06-04 00:00:00',626.4,286,35460,31.82,'GEO Transfer' from dual union all
select 4274,'004274','1969-107B','S3M','S3M',18374,'Reentered','1979-04-02 00:00:00','1970-01-20 00:00:00',95.05,505,537,74.05,'Intermediate' from dual union all
select 42740,'042740','2017-029A','ViaSat-2','ViaSat-2',70419,'In Earth orbit','','2017-06-09 00:00:00',1028.03,303,54424,5.96,'Highly Ellip' from dual union all
select 42741,'042741','2017-029B','EUTELSAT 172B','EUTELSAT 172B',70419,'In Earth orbit','','2017-06-09 00:00:00',639.78,524,35910,5.89,'GEO Transfer' from dual union all
select 42742,'042742','2017-029C','ESC-A 590','Ariane 5 ESC-A',70419,'In Earth orbit','','2017-06-09 00:00:00',631.78,269,35754,6.05,'GEO Transfer' from dual union all
select 42743,'042743','2017-029D','Sylda 5A','Sylda 5A',70419,'In Earth orbit','','2017-06-11 00:00:00',629.86,265,35659,6.04,'GEO Transfer' from dual union all
select 42744,'042744','2017-030A','Dragon CRS-11','Dragon C106.2',70420,'Landed','2017-07-03 00:00:00','2017-06-05 00:00:00',92.65,401,408,51.64,'Intermediate' from dual union all
select 42745,'042745','2017-030B','Dragon solar array cover','',70420,'Reentered','2017-06-09 00:00:00','2017-06-04 00:00:00',90.13,205,357,51.64,'Intermediate' from dual union all
select 42746,'042746','2017-030C','Dragon solar array cover','',70420,'Reentered','2017-06-09 00:00:00','2017-06-05 00:00:00',89.84,203,331,51.63,'Intermediate' from dual union all
select 42747,'042747','2017-031A','GSAT-19E','GSAT-19E',70421,'In Earth orbit','','2017-06-14 00:00:00',605.77,162,34513,21.73,'GEO Transfer' from dual union all
select 42748,'042748','2017-031B','GSLV-3 Stage 2','GSLV C25',70421,'Reentered','2018-02-08 00:00:00','2017-06-14 00:00:00',605.71,160,34512,21.73,'GEO Transfer' from dual union all
select 42749,'042749','2017-032A','Echostar 21','Terrestar 2',70422,'In Earth orbit','','2017-06-15 00:00:00',1158.69,24526,35804,9.89,'Medium' from dual union all
select 4275,'004275','1969-107C','deb Kosmos-315','Tselina-O part',18374,'Reentered','1970-04-29 00:00:00','1970-01-19 00:00:00',94.65,477,527,74.08,'Intermediate' from dual union all
select 42750,'042750','2017-032B','Briz-M No. 99571','Briz-M 14S43 No. 99571',70422,'In Earth orbit','','2017-06-15 00:00:00',636.1,1792,34453,31.18,'GEO Transfer' from dual union all
select 42751,'042751','2017-032C','Briz-M DTB','Briz-M torus tank',70422,'In Earth orbit','','2017-06-15 00:00:00',232.75,323,11988,49.72,'Medium' from dual union all
select 42752,'042752','2017-019B','Lemur-2-JennyBarna','Lemur-2-30',70409,'In Earth orbit','','2017-06-10 00:00:00',94.26,476,490,51.64,'Intermediate' from dual union all
select 42753,'042753','2017-019C','Lemur-2-Angela','Lemur-2-31',70409,'In Earth orbit','','2017-06-10 00:00:00',94.26,476,490,51.64,'Intermediate' from dual union all
select 42754,'042754','2017-019D','Lemur-2-Spirovision','Lemur-2-32',70409,'In Earth orbit','','2017-06-10 00:00:00',94.26,477,489,51.64,'Intermediate' from dual union all
select 42755,'042755','2017-019E','Lemur-2-RobMoore','Lemur-2-33',70409,'In Earth orbit','','2017-06-10 00:00:00',94.27,476,490,51.64,'Intermediate' from dual union all
select 42756,'042756','2017-033A','Progress MS-06','Progress 7K-TGM No. 436',70423,'Deorbited','2017-12-28 00:00:00','2017-06-16 00:00:00',92.64,401,407,51.64,'Intermediate' from dual union all
select 42757,'042757','2017-033B','Blok-I 14S54/RD-0110','374BL02A',70423,'Reentered','2017-06-17 00:00:00','2017-06-16 00:00:00',87.83,162,173,51.65,'Intermediate' from dual union all
select 42758,'042758','2017-034A','Huiyan','HXMT',70424,'In Earth orbit','','2017-06-17 00:00:00',95.45,536,545,43.02,'Intermediate' from dual;
insert into SATELLITE
select 42759,'042759','2017-034B','OVS-1B','Zhuhai-1-02',70424,'In Earth orbit','','2017-06-17 00:00:00',95.42,533,545,43.02,'Intermediate' from dual union all
select 4276,'004276','1969-082DK','deb Agena D','',18349,'In Earth orbit','','1992-02-04 00:00:00',100.54,754,814,70.08,'Intermediate' from dual union all
select 42760,'042760','2017-034C','Milanesat','Nusat-3',70424,'In Earth orbit','','2017-06-19 00:00:00',95.44,535,545,43.02,'Intermediate' from dual union all
select 42761,'042761','2017-034D','OVS-1A','Zhuhai-1-01',70424,'In Earth orbit','','2017-06-18 00:00:00',95.42,533,545,43.02,'Intermediate' from dual union all
select 42762,'042762','2017-034E','CZ-4B Stage 3','CZ-4B Stage 3',70424,'In Earth orbit','','2017-06-17 00:00:00',92.65,268,541,43.12,'Intermediate' from dual union all
select 42763,'042763','2017-035A','Zhongxing 9A','zhongxing 9A',70425,'In Earth orbit','','2017-06-19 00:00:00',294.2,193,16356,25.68,'Highly Ellip' from dual union all
select 42764,'042764','2017-035B','CZ-3B Stage 3','CZ-3B Stage 3',70425,'In Earth orbit','','2017-06-20 00:00:00',294.07,191,16349,25.69,'Highly Ellip' from dual union all
select 42765,'042765','2017-036A','UCLSat','QB50-GB03',70426,'In Earth orbit','','2017-06-24 00:00:00',94.7,496,513,97.45,'Sun-Synch' from dual union all
select 42766,'042766','2017-036B','NIUSAT-Keralshree','NIUSAT',70426,'In Earth orbit','','2017-06-24 00:00:00',94.76,496,517,97.45,'Sun-Synch' from dual union all
select 42767,'042767','2017-036C','Cartosat-2 Series Satellite','Cartosat-2E',70426,'In Earth orbit','','2017-06-23 00:00:00',91.34,334,347,96.92,'Sun-Synch' from dual union all
select 42768,'042768','2017-036D','LituanicaSAT 2','QB50-LT01',70426,'In Earth orbit','','2017-06-24 00:00:00',94.69,496,512,97.45,'Sun-Synch' from dual union all
select 42769,'042769','2017-036E','CE-SAT-I','CE-SAT-I',70426,'In Earth orbit','','2017-06-24 00:00:00',94.76,497,517,97.45,'Sun-Synch' from dual union all
select 4277,'004277','1969-082DL','deb Agena D','',18349,'In Earth orbit','','1992-01-31 00:00:00',102.49,846,907,70.02,'Intermediate' from dual union all
select 42770,'042770','2017-036F','InflateSail','QB50-GB06',70426,'Reentered','2017-09-03 00:00:00','2017-06-24 00:00:00',94.69,496,512,97.45,'Sun-Synch' from dual union all
select 42771,'042771','2017-036G','Lemur-2-ShainaJohl','Lemur-2-34',70426,'In Earth orbit','','2017-06-24 00:00:00',94.68,496,511,97.45,'Sun-Synch' from dual union all
select 42772,'042772','2017-036H','Lemur-2-XueniTerence','Lemur-2-35',70426,'In Earth orbit','','2017-06-24 00:00:00',94.68,496,511,97.45,'Sun-Synch' from dual union all
select 42773,'042773','2017-036J','Lemur-2-LucyBryce','Lemur-2-36',70426,'In Earth orbit','','2017-06-24 00:00:00',94.68,496,511,97.45,'Sun-Synch' from dual union all
select 42774,'042774','2017-036K','Lemur-2-KungFoo','Lemur-2-37',70426,'In Earth orbit','','2017-06-24 00:00:00',94.67,496,510,97.45,'Sun-Synch' from dual union all
select 42775,'042775','2017-036L','Aalto-1','Aalto-1',70426,'In Earth orbit','','2017-06-24 00:00:00',94.76,497,517,97.45,'Sun-Synch' from dual union all
select 42776,'042776','2017-036M','URSA MAIOR','QB50-IT02',70426,'In Earth orbit','','2017-06-24 00:00:00',94.69,496,512,97.45,'Sun-Synch' from dual union all
select 42777,'042777','2017-036N','COMPASS-2','QB50-DE04',70426,'In Earth orbit','','2017-06-24 00:00:00',94.71,496,513,97.45,'Sun-Synch' from dual union all
select 42778,'042778','2017-036P','Max Valier','Max Valier',70426,'In Earth orbit','','2017-06-24 00:00:00',94.75,496,516,97.45,'Sun-Synch' from dual union all
select 42779,'042779','2017-036Q','Lemur-2-Lynsey-Symo','Lemur-2-41',70426,'In Earth orbit','','2017-06-24 00:00:00',94.65,496,508,97.45,'Sun-Synch' from dual union all
select 4278,'004278','1969-082DM','deb Agena D','',18349,'Reentered','1988-08-13 00:00:00','1988-08-13 00:00:00',88.01,159,193,69.98,'Intermediate' from dual union all
select 42780,'042780','2017-036R','Lemur-2-Lisasaurus','Lemur-2-40',70426,'In Earth orbit','','2017-06-24 00:00:00',94.66,496,509,97.45,'Sun-Synch' from dual union all
select 42781,'042781','2017-036S','Lemur-2-Sam-Amelia','Lemur-2-39',70426,'In Earth orbit','','2017-06-24 00:00:00',94.67,496,510,97.45,'Sun-Synch' from dual union all
select 42782,'042782','2017-036T','Lemur-2-McPeake','Lemur-2-38',70426,'In Earth orbit','','2017-06-24 00:00:00',94.67,496,510,97.45,'Sun-Synch' from dual union all
select 42783,'042783','2017-036U','Diamond Red','3-Diamonds 3',70426,'In Earth orbit','','2017-06-24 00:00:00',94.68,496,511,97.45,'Sun-Synch' from dual union all
select 42784,'042784','2017-036V','Pegasus','QB50-AT03',70426,'In Earth orbit','','2017-06-24 00:00:00',94.75,497,516,97.45,'Sun-Synch' from dual union all
select 42785,'042785','2017-036W','Diamond Green','3-Diamonds 2',70426,'In Earth orbit','','2017-06-24 00:00:00',94.68,496,511,97.45,'Sun-Synch' from dual union all
select 42786,'042786','2017-036X','Diamond Blue','3-Diamonds 1',70426,'In Earth orbit','','2017-06-24 00:00:00',94.68,496,511,97.45,'Sun-Synch' from dual union all
select 42787,'042787','2017-036Y','NUDTSat','QB50-CN06/BE06',70426,'In Earth orbit','','2017-06-24 00:00:00',94.74,497,515,97.45,'Sun-Synch' from dual union all
select 42788,'042788','2017-036Z','SUCHAI','SUCHAI',70426,'In Earth orbit','','2017-06-24 00:00:00',94.69,496,512,97.45,'Sun-Synch' from dual union all
select 42789,'042789','2017-036AA','skCUBE','skCUBE',70426,'In Earth orbit','','2017-06-24 00:00:00',94.69,496,512,97.45,'Sun-Synch' from dual union all
select 4279,'004279','1969-107D','deb Kosmos-315','Tselina-O part',18374,'Reentered','1971-10-10 00:00:00','1971-01-02 00:00:00',94.95,499,534,74.01,'Intermediate' from dual union all
select 42790,'042790','2017-036AB','VZLUSat','QB50-CZ03',70426,'In Earth orbit','','2017-06-24 00:00:00',94.74,497,515,97.45,'Sun-Synch' from dual union all
select 42791,'042791','2017-036AC','Venta-1','Venta-1',70426,'In Earth orbit','','2017-06-24 00:00:00',94.65,496,508,97.45,'Sun-Synch' from dual union all
select 42792,'042792','2017-036AD','ROBUSTA 1B','ROBUSTA 1B',70426,'In Earth orbit','','2017-06-24 00:00:00',94.69,496,512,97.45,'Sun-Synch' from dual union all
select 42793,'042793','2017-036AE','CICERO-6','CICERO-6',70426,'In Earth orbit','','2017-06-24 00:00:00',94.74,496,515,97.45,'Sun-Synch' from dual union all
select 42794,'042794','2017-036AF','D-SAT','D-SAT',70426,'In Earth orbit','','2017-06-24 00:00:00',94.75,497,516,97.45,'Sun-Synch' from dual union all
select 42795,'042795','2017-036AG','PACSCISAT','TYVAK-53B',70426,'In Earth orbit','','2017-06-24 00:00:00',94.75,497,516,97.45,'Sun-Synch' from dual union all
select 42796,'042796','2017-036AH','PSLV-C38 PS4','PSLV-C38 PS4',70426,'In Earth orbit','','2017-07-01 00:00:00',91.59,336,370,94.61,'Polar' from dual union all
select 42797,'042797','2017-036AJ','deb PSLV','',70426,'In Earth orbit','','2017-07-15 00:00:00',94.68,494,512,97.45,'Sun-Synch' from dual union all
select 42798,'042798','2017-037A','Kosmos-2519','Napryazhenie No. 1 ',70427,'In Earth orbit','','2017-06-24 00:00:00',97.95,653,668,98.05,'Sun-Synch' from dual union all
select 42799,'042799','2017-037B','Volga 14S46 No. 3','Volga R15000-003',70427,'Deorbited','2017-06-24 00:00:00','2017-06-24 00:00:00',97.97,654,670,98.05,'Sun-Synch' from dual union all
select 428,'000428','62 A Omega 4','deb Kosmos-9','',8148,'Reentered','1962-10-08 00:00:00','1962-10-01 00:00:00',90.73,297,324,64.99,'Intermediate' from dual union all
select 4280,'004280','1969-109A','Kosmos-317','Zenit-4MK',18376,'Landed','1970-01-05 00:00:00','1969-12-25 00:00:00',89.2,191,279,65.43,'Intermediate' from dual union all
select 42800,'042800','2017-037C','Blok-I 14S54/RD-0110','Blok-I 14S54/RD-0110',70427,'In Earth orbit','','2017-06-24 00:00:00',93.92,284,648,98.06,'Sun-Synch' from dual union all
select 42801,'042801','2017-038A','BulgariaSat-1','BulgariaSat-1',70428,'In Earth orbit','','2017-06-24 00:00:00',1285.16,196,65360,23.95,'Highly Ellip' from dual union all
select 42802,'042802','2017-038B','Falcon 9-036 Stage 2','F9-036 St2',70428,'Reentered','2017-10-31 00:00:00','2017-06-24 00:00:00',1282.43,192,65253,23.95,'Highly Ellip' from dual union all
select 42803,'042803','2017-039A','Iridium Next SV113','Iridium Next SV113',70429,'In Earth orbit','','2017-06-27 00:00:00',97,607,624,86.67,'Polar' from dual union all
select 42804,'042804','2017-039B','Iridium Next SV123','Iridium Next SV123',70429,'In Earth orbit','','2017-06-27 00:00:00',97,607,624,86.67,'Polar' from dual union all
select 42805,'042805','2017-039C','Iridium Next SV120','Iridium Next SV120',70429,'In Earth orbit','','2017-06-27 00:00:00',97.02,607,624,86.67,'Polar' from dual union all
select 42806,'042806','2017-039D','Iridium Next SV115','Iridium Next SV115',70429,'In Earth orbit','','2017-06-27 00:00:00',96.99,606,623,86.67,'Polar' from dual union all
select 42807,'042807','2017-039E','Iridium Next SV118','Iridium Next SV118',70429,'In Earth orbit','','2017-06-28 00:00:00',97.03,608,625,86.67,'Polar' from dual union all
select 42808,'042808','2017-039F','Iridium Next SV117','Iridium Next SV117',70429,'In Earth orbit','','2017-06-27 00:00:00',96.99,606,623,86.67,'Polar' from dual union all
select 42809,'042809','2017-039G','Iridium Next SV126','Iridium Next SV126',70429,'In Earth orbit','','2017-06-27 00:00:00',96.99,606,623,86.67,'Polar' from dual union all
select 4281,'004281','1969-109B','11A57I','11A57I',18376,'Reentered','1969-12-28 00:00:00','1969-12-26 00:00:00',88.72,185,238,65.41,'Intermediate' from dual union all
select 42810,'042810','2017-039H','Iridium Next SV124','Iridium Next SV124',70429,'In Earth orbit','','2017-06-27 00:00:00',97.03,608,625,86.67,'Polar' from dual union all
select 42811,'042811','2017-039J','Iridium Next SV128','Iridium Next SV128',70429,'In Earth orbit','','2017-06-27 00:00:00',97.04,609,625,86.67,'Polar' from dual union all
select 42812,'042812','2017-039K','Iridium Next SV121','Iridium Next SV121',70429,'In Earth orbit','','2017-06-27 00:00:00',97.03,608,625,86.67,'Polar' from dual union all
select 42813,'042813','1998-067MT','ROSA','ROSA',66503,'Reentered','2018-02-04 00:00:00','2017-06-27 00:00:00',92.64,401,407,51.64,'Intermediate' from dual union all
select 42814,'042814','2017-040A','Hellas Sat 3-Inmarsat S EAN','Hellas Sat 3',70430,'In Earth orbit','','2017-07-07 00:00:00',1433.53,35685,35788,.05,'Drift GEO' from dual union all
select 42815,'042815','2017-040B','GSAT-17','GSAT-17',70430,'In Earth orbit','','2017-07-07 00:00:00',1430.14,35497,35843,.11,'Drift GEO' from dual union all
select 42816,'042816','2017-040C','ESC-A 591','Ariane 5 ESC-A',70430,'In Earth orbit','','2017-07-06 00:00:00',633.04,253,35835,2.92,'GEO Transfer' from dual union all
select 42817,'042817','2017-040D','Sylda 5A','Sylda 5A',70430,'In Earth orbit','','2017-07-10 00:00:00',630.59,249,35713,2.99,'GEO Transfer' from dual union all
select 42818,'042818','2017-041A','Intelsat IS-35e','Intelsat IS-35e',70431,'In Earth orbit','','2017-07-18 00:00:00',1464.02,35274,37389,.13,'Drift GEO' from dual union all
select 42819,'042819','2017-041B','Falcon 9-039 Stage 2','F9-039 St2',70431,'In Earth orbit','','2017-07-13 00:00:00',771.21,288,42671,25.89,'Highly Ellip' from dual union all
select 4282,'004282','1969-108A','Kosmos-316','IM IS (I2P)',18375,'Reentered','1970-08-28 00:00:00','1970-01-24 00:00:00',101.78,148,1537,49.51,'Intermediate' from dual union all
select 42820,'042820','1998-067MU','TOKI','BIRD-J',66503,'In Earth orbit','','2017-07-07 00:00:00',92.6,402,402,51.65,'Intermediate' from dual union all
select 42821,'042821','1998-067MV','Ghanasat-1','BIRD-G',66503,'In Earth orbit','','2017-07-07 00:00:00',92.61,401,404,51.64,'Intermediate' from dual union all
select 42822,'042822','1998-067MW','Mazaalai','BIRD-M',66503,'In Earth orbit','','2017-07-07 00:00:00',92.68,401,410,51.62,'Intermediate' from dual union all
select 42823,'042823','1998-067MX','BRAC ONNESHA','BIRD-B',66503,'In Earth orbit','','2017-07-07 00:00:00',92.61,401,404,51.67,'Intermediate' from dual union all
select 42824,'042824','1998-067MY','Nigeria EduSat-1','BIRD-N',66503,'In Earth orbit','','2017-07-07 00:00:00',92.62,396,410,51.65,'Intermediate' from dual union all
select 42825,'042825','2017-042A','Kanopus-V-IK No. 2','Kanopus-V-IK No. 2',70432,'In Earth orbit','','2017-07-16 00:00:00',94.76,505,508,97.44,'Sun-Synch' from dual union all
select 42826,'042826','2017-042B','NORSAT-1','NORSAT-1',70432,'In Earth orbit','','2017-07-16 00:00:00',96.59,586,605,97.62,'Sun-Synch' from dual union all
select 42827,'042827','2017-042C','Iskra-MAI-85                      ','Iskra-MAI-85',70432,'In Earth orbit','','2017-07-16 00:00:00',96.56,586,602,97.61,'Sun-Synch' from dual union all
select 42828,'042828','2017-042D','NORSAT-2','NORSAT-2',70432,'In Earth orbit','','2017-07-16 00:00:00',96.59,586,605,97.61,'Sun-Synch' from dual union all
select 42829,'042829','2017-042E','Technosat','Technosat',70432,'In Earth orbit','','2017-07-16 00:00:00',96.59,586,605,97.61,'Sun-Synch' from dual union all
select 4283,'004283','1969-108B','Kosmos-316 fairing','',18375,'Reentered','1970-01-01 00:00:00','1970-12-23 00:00:00',101.29,183,1457,49.3,'Intermediate' from dual union all
select 42830,'042830','2017-042F','Mayak                             ','Mayak',70432,'In Earth orbit','','2017-07-16 00:00:00',96.56,586,602,97.61,'Sun-Synch' from dual union all
select 42831,'042831','2017-042G','FLP','Flying Laptop',70432,'In Earth orbit','','2017-07-16 00:00:00',96.6,586,606,97.61,'Sun-Synch' from dual union all
select 42832,'042832','2017-042H','Ekvador-UTE-YuZGU                 ','UTE-UESOR',70432,'In Earth orbit','','2017-07-16 00:00:00',96.55,585,602,97.61,'Sun-Synch' from dual union all
select 42833,'042833','2017-042J','deb Fregat                       ','',70432,'In Earth orbit','','2017-07-16 00:00:00',96.52,587,597,97.61,'Sun-Synch' from dual union all
select 42834,'042834','2017-042K','MKA-N 1                           ','MKA-N No. 1',70432,'In Earth orbit','','2017-07-16 00:00:00',96.58,586,604,97.61,'Sun-Synch' from dual union all
select 42835,'042835','2017-042L','WNISAT-1R','WNISAT-1R',70432,'In Earth orbit','','2017-07-16 00:00:00',96.6,586,606,97.61,'Sun-Synch' from dual union all
select 42836,'042836','2017-042M','MKA-N 2                           ','MKA-N No. 2',70432,'In Earth orbit','','2017-07-16 00:00:00',96.57,586,602,97.61,'Sun-Synch' from dual union all
select 42837,'042837','2017-042N','Lemur-2-Greenberg','Lemur-2-42',70432,'In Earth orbit','','2017-07-16 00:00:00',96.55,585,602,97.61,'Sun-Synch' from dual union all
select 42838,'042838','2017-042P','Lemur-2-AndiS','Lemur-2-47',70432,'In Earth orbit','','2017-07-16 00:00:00',96.52,586,598,97.61,'Sun-Synch' from dual union all
select 42839,'042839','2017-042Q','Lemur-2-Monson','Lemur-2-44',70432,'In Earth orbit','','2017-07-16 00:00:00',96.54,585,601,97.61,'Sun-Synch' from dual union all
select 4284,'004284','1969-108C','11S692','11S692',18375,'Reentered','1970-01-28 00:00:00','1969-12-30 00:00:00',101.16,149,1479,49.49,'Intermediate' from dual union all
select 42840,'042840','2017-042R','Lemur-2-Furiaus','Lemur-2-45',70432,'In Earth orbit','','2017-07-16 00:00:00',96.53,584,601,97.61,'Sun-Synch' from dual union all
select 42841,'042841','2017-042S','Lemur-2-PeterG','Lemur-2-48',70432,'In Earth orbit','','2017-07-16 00:00:00',96.5,586,596,97.61,'Sun-Synch' from dual union all
select 42842,'042842','2017-042T','Lemur-2-Dembitz','Lemur-2-49',70432,'In Earth orbit','','2017-07-16 00:00:00',96.5,586,596,97.61,'Sun-Synch' from dual union all
select 42843,'042843','2017-042U','CICERO-1                          ','CICERO-1',70432,'In Earth orbit','','2017-07-16 00:00:00',96.57,586,602,97.61,'Sun-Synch' from dual union all
select 42844,'042844','2017-042V','NanoACE','NanoACE',70432,'In Earth orbit','','2017-07-16 00:00:00',96.57,586,602,97.61,'Sun-Synch' from dual union all
select 42845,'042845','2017-042W','Lemur-2-Zachary','Lemur-2-46',70432,'In Earth orbit','','2017-07-16 00:00:00',96.53,587,597,97.61,'Sun-Synch' from dual union all
select 42846,'042846','2017-042X','Landmapper-BC2','Corvus-BC 2',70432,'In Earth orbit','','2017-07-16 00:00:00',96.58,586,604,97.61,'Sun-Synch' from dual union all
select 42847,'042847','2017-042Y','Landmapper-BC1','Corvus-BC 1',70432,'In Earth orbit','','2017-07-16 00:00:00',96.58,586,604,97.61,'Sun-Synch' from dual union all
select 42848,'042848','2017-042Z','CICERO-2                          ','CICERO-2',70432,'In Earth orbit','','2017-07-16 00:00:00',96.57,586,602,97.61,'Sun-Synch' from dual;
insert into SATELLITE
select 42849,'042849','2017-042AA','CICERO-3                          ','CICERO-3',70432,'In Earth orbit','','2017-07-16 00:00:00',96.57,586,602,97.61,'Sun-Synch' from dual union all
select 4285,'004285','1969-110A','Interkosmos-2','DS-U1-IK No. 1',18377,'Reentered','1970-06-07 00:00:00','1970-01-24 00:00:00',97.49,200,1078,48.42,'Intermediate' from dual union all
select 42850,'042850','2017-042AB','Flock 2k-3','Dove 0F21',70432,'In Earth orbit','','2017-07-16 00:00:00',93.96,461,476,97,'Sun-Synch' from dual union all
select 42851,'042851','2017-042AC','Flock 2k-4','Dove 0F24',70432,'In Earth orbit','','2017-07-16 00:00:00',93.95,461,475,97,'Sun-Synch' from dual union all
select 42852,'042852','2017-042AD','Flock 2k-1','Dove 0F1A',70432,'In Earth orbit','','2017-07-16 00:00:00',93.96,461,476,97,'Sun-Synch' from dual union all
select 42853,'042853','2017-042AE','Flock 2k-2','Dove 0F1E',70432,'In Earth orbit','','2017-07-16 00:00:00',93.96,461,476,97,'Sun-Synch' from dual union all
select 42854,'042854','2017-042AF','Flock 2k-47','Dove 1043',70432,'In Earth orbit','','2017-07-16 00:00:00',93.9,457,474,97,'Sun-Synch' from dual union all
select 42855,'042855','2017-042AG','Flock 2k-48','Dove 101C',70432,'In Earth orbit','','2017-07-16 00:00:00',93.9,457,474,97,'Sun-Synch' from dual union all
select 42856,'042856','2017-042AH','Flock 2k-45','Dove 100D',70432,'In Earth orbit','','2017-07-16 00:00:00',93.9,457,474,97,'Sun-Synch' from dual union all
select 42857,'042857','2017-042AJ','Flock 2k-24','Dove 0F4F',70432,'In Earth orbit','','2017-07-16 00:00:00',93.93,459,474,97,'Sun-Synch' from dual union all
select 42858,'042858','2017-042AK','Flock 2k-46','Dove 103F',70432,'In Earth orbit','','2017-07-16 00:00:00',93.9,457,474,97,'Sun-Synch' from dual union all
select 42859,'042859','2017-042AL','Flock 2k-23','Dove 0F4B',70432,'In Earth orbit','','2017-07-16 00:00:00',93.94,460,475,97,'Sun-Synch' from dual union all
select 4286,'004286','1969-110B','S1M','S1M',18377,'Reentered','1970-03-21 00:00:00','1970-01-24 00:00:00',96.37,202,968,48.42,'Intermediate' from dual union all
select 42860,'042860','2017-042AM','Flock 2k-21','Dove 0F49',70432,'In Earth orbit','','2017-07-16 00:00:00',93.94,460,475,97,'Sun-Synch' from dual union all
select 42861,'042861','2017-042AN','Flock 2k-22','Dove 0F4A',70432,'In Earth orbit','','2017-07-16 00:00:00',93.94,460,475,97,'Sun-Synch' from dual union all
select 42862,'042862','2017-042AP','Flock 2k-7','Dove 0F2B',70432,'In Earth orbit','','2017-07-16 00:00:00',93.95,461,475,97,'Sun-Synch' from dual union all
select 42863,'042863','2017-042AQ','Flock 2k-8','Dove 0F2D',70432,'In Earth orbit','','2017-07-16 00:00:00',93.95,461,475,97,'Sun-Synch' from dual union all
select 42864,'042864','2017-042AR','Flock 2k-5','Dove 0F29',70432,'In Earth orbit','','2017-07-16 00:00:00',93.95,461,475,97,'Sun-Synch' from dual union all
select 42865,'042865','2017-042AS','Flock 2k-40','Dove 1053',70432,'In Earth orbit','','2017-07-16 00:00:00',93.91,458,474,97,'Sun-Synch' from dual union all
select 42866,'042866','2017-042AT','Flock 2k-39','Dove 1052',70432,'In Earth orbit','','2017-07-16 00:00:00',93.92,458,475,97,'Sun-Synch' from dual union all
select 42867,'042867','2017-042AU','Flock 2k-37','Dove 1050',70432,'In Earth orbit','','2017-07-16 00:00:00',93.92,458,475,97,'Sun-Synch' from dual union all
select 42868,'042868','2017-042AV','Flock 2k-38','Dove 1051',70432,'In Earth orbit','','2017-07-16 00:00:00',93.92,458,475,97,'Sun-Synch' from dual union all
select 42869,'042869','2017-042AW','Flock 2k-31','Dove 104A',70432,'In Earth orbit','','2017-07-16 00:00:00',93.93,459,474,97,'Sun-Synch' from dual union all
select 4287,'004287','1969-107E','deb Kosmos-315','Tselina-O part',18374,'Reentered','1970-06-13 00:00:00','1970-01-22 00:00:00',94.6,476,522,74.1,'Intermediate' from dual union all
select 42870,'042870','2017-042AX','Flock 2k-32','Dove 104B',70432,'In Earth orbit','','2017-07-16 00:00:00',93.92,458,475,97,'Sun-Synch' from dual union all
select 42871,'042871','2017-042AY','Flock 2k-29','Dove 1048',70432,'In Earth orbit','','2017-07-16 00:00:00',93.93,459,474,97,'Sun-Synch' from dual union all
select 42872,'042872','2017-042AZ','Flock 2k-30','Dove 1049',70432,'In Earth orbit','','2017-07-16 00:00:00',93.93,459,474,97,'Sun-Synch' from dual union all
select 42873,'042873','2017-042BA','Flock 2k-44','Dove 1020',70432,'In Earth orbit','','2017-07-16 00:00:00',93.9,457,474,97,'Sun-Synch' from dual union all
select 42874,'042874','2017-042BB','Flock 2k-43','Dove 1056',70432,'In Earth orbit','','2017-07-16 00:00:00',93.91,458,474,97,'Sun-Synch' from dual union all
select 42875,'042875','2017-042BC','Flock 2k-41','Dove 1054',70432,'In Earth orbit','','2017-07-16 00:00:00',93.91,458,474,97,'Sun-Synch' from dual union all
select 42876,'042876','2017-042BD','Flock 2k-36','Dove 104F',70432,'In Earth orbit','','2017-07-16 00:00:00',93.92,458,475,97,'Sun-Synch' from dual union all
select 42877,'042877','2017-042BE','Flock 2k-35','Dove 104E',70432,'In Earth orbit','','2017-07-16 00:00:00',93.92,458,475,97,'Sun-Synch' from dual union all
select 42878,'042878','2017-042BF','Flock 2k-34','Dove 104D',70432,'In Earth orbit','','2017-07-16 00:00:00',93.92,458,475,97,'Sun-Synch' from dual union all
select 42879,'042879','2017-042BG','Flock 2k-33','Dove 104C',70432,'In Earth orbit','','2017-07-16 00:00:00',93.92,458,475,97,'Sun-Synch' from dual union all
select 4288,'004288','1969-082DN','deb Agena D','',18349,'Reentered','1970-09-12 00:00:00','1970-09-08 00:00:00',93.13,397,458,69.64,'Intermediate' from dual union all
select 42880,'042880','2017-042BH','Flock 2k-28','Dove 1047',70432,'In Earth orbit','','2017-07-16 00:00:00',93.93,459,474,97,'Sun-Synch' from dual union all
select 42881,'042881','2017-042BJ','Lemur-2-ArtFisher','Lemur-2-43',70432,'In Earth orbit','','2017-07-16 00:00:00',96.54,585,601,97.61,'Sun-Synch' from dual union all
select 42882,'042882','2017-042BK','Flock 2k-27','Dove 0F54',70432,'In Earth orbit','','2017-07-16 00:00:00',93.93,459,474,97,'Sun-Synch' from dual union all
select 42883,'042883','2017-042BL','Flock 2k-26','Dove 0F53',70432,'In Earth orbit','','2017-07-16 00:00:00',93.93,459,474,97,'Sun-Synch' from dual union all
select 42884,'042884','2017-042BM','Flock 2k-25','Dove 0F4D',70432,'In Earth orbit','','2017-07-16 00:00:00',93.93,459,474,97,'Sun-Synch' from dual union all
select 42885,'042885','2017-042BN','Flock 2k-20','Dove 0F47',70432,'In Earth orbit','','2017-07-16 00:00:00',93.94,460,475,97,'Sun-Synch' from dual union all
select 42886,'042886','2017-042BP','Flock 2k-19','Dove 0F46',70432,'In Earth orbit','','2017-07-16 00:00:00',93.94,460,475,97,'Sun-Synch' from dual union all
select 42887,'042887','2017-042BQ','Flock 2k-18','Dove 0F44',70432,'In Earth orbit','','2017-07-16 00:00:00',93.95,461,475,97,'Sun-Synch' from dual union all
select 42888,'042888','2017-042BR','Flock 2k-17','Dove 0F40',70432,'In Earth orbit','','2017-07-16 00:00:00',93.94,460,475,97,'Sun-Synch' from dual union all
select 42889,'042889','2017-042BS','Flock 2k-16','Dove 0F3D',70432,'In Earth orbit','','2017-07-16 00:00:00',93.95,461,475,97,'Sun-Synch' from dual union all
select 4289,'004289','1969-029AM','deb Blok-E','',18296,'Reentered','1970-12-26 00:00:00','1969-08-01 00:00:00',96.82,436,777,81.25,'Intermediate' from dual union all
select 42890,'042890','2017-042BT','Flock 2k-15','Dove 0F3C',70432,'In Earth orbit','','2017-07-16 00:00:00',93.95,461,475,97,'Sun-Synch' from dual union all
select 42891,'042891','2017-042BU','Flock 2k-13','Dove 0F37',70432,'In Earth orbit','','2017-07-16 00:00:00',93.95,461,475,97,'Sun-Synch' from dual union all
select 42892,'042892','2017-042BV','Flock 2k-14','Dove 0F3B',70432,'In Earth orbit','','2017-07-16 00:00:00',93.95,461,475,97,'Sun-Synch' from dual union all
select 42893,'042893','2017-042BW','Flock 2k-12','Dove 0F36',70432,'In Earth orbit','','2017-07-16 00:00:00',93.95,461,475,97,'Sun-Synch' from dual union all
select 42894,'042894','2017-042BX','Flock 2k-11','Dove 0F33',70432,'In Earth orbit','','2017-07-16 00:00:00',93.95,461,475,97,'Sun-Synch' from dual union all
select 42895,'042895','2017-042BY','Flock 2k-10','Dove 0F32',70432,'In Earth orbit','','2017-07-16 00:00:00',93.95,461,475,97,'Sun-Synch' from dual union all
select 42896,'042896','2017-042BZ','Flock 2k-9','Dove 0F2E',70432,'In Earth orbit','','2017-07-16 00:00:00',93.95,461,475,97,'Sun-Synch' from dual union all
select 42897,'042897','2017-042CA','Flock 2k-6','Dove 0F2A',70432,'In Earth orbit','','2017-07-16 00:00:00',93.95,461,475,97,'Sun-Synch' from dual union all
select 42898,'042898','2017-043A','Soyuz MS-05','Soyuz 11F732A48 No. 736',70433,'Landed','2017-12-14 00:00:00','2017-07-28 00:00:00',92.64,400,408,51.64,'Intermediate' from dual union all
select 42899,'042899','2017-043B','11S510','11S510',70433,'Reentered','2017-08-01 00:00:00','2017-07-30 00:00:00',88.21,179,193,51.64,'Intermediate' from dual union all
select 429,'000429','62 A Omega 5','deb Kosmos-9','',8148,'Reentered','1962-10-04 00:00:00','1962-10-01 00:00:00',90.66,297,317,64.99,'Intermediate' from dual union all
select 4290,'004290','1969-109C','Blenda','BZA',18376,'Reentered','1970-01-09 00:00:00','1970-01-03 00:00:00',89.07,189,267,65.4,'Intermediate' from dual union all
select 42900,'042900','2017-044A','Optsat 3000','Optsat 3000',70434,'In Earth orbit','','2017-08-05 00:00:00',93.64,452,453,97.2,'Sun-Synch' from dual union all
select 42901,'042901','2017-044B','Venmus','Venus (Ven{\mu}s)',70434,'In Earth orbit','','2017-08-04 00:00:00',99.22,719,724,98.35,'Sun-Synch' from dual union all
select 42902,'042902','2017-044C','VESPA Upper Part','VUP',70434,'In Earth orbit','','2017-08-04 00:00:00',93.47,439,449,97.22,'Sun-Synch' from dual union all
select 42903,'042903','2017-021F','Silu-1','Silu-1',70411,'In Earth orbit','','2017-08-02 00:00:00',92.36,386,395,42.79,'Intermediate' from dual union all
select 42904,'042904','2017-045A','Dragon CRS-12','Dragon C113',70435,'Landed','2017-09-17 00:00:00','2017-08-16 00:00:00',92.64,400,408,51.64,'Intermediate' from dual union all
select 42905,'042905','2017-045B','Dragon solar array cover','',70435,'Reentered','2017-08-17 00:00:00','2017-08-16 00:00:00',89.68,199,319,51.65,'Intermediate' from dual union all
select 42906,'042906','2017-045C','Dragon solar array cover','',70435,'Reentered','2017-08-17 00:00:00','2017-08-16 00:00:00',89.63,199,314,51.63,'Intermediate' from dual union all
select 42907,'042907','2017-046A','Kosmos-2520','Blagovest No. 11L',70436,'In Earth orbit','','2017-09-16 00:00:00',1438.11,35788,35864,.03,'Drift GEO' from dual union all
select 42908,'042908','2017-046B','Briz-M DTB','Briz-M torus tank',70436,'In Earth orbit','','2017-09-16 00:00:00',633.24,384,35714,48.59,'GEO Transfer' from dual union all
select 42909,'042909','2017-046C','Briz-M No. 99xxx','Briz-M',70436,'In Earth orbit','','2017-09-17 00:00:00',1594.06,35750,41899,.15,'Very High' from dual union all
select 4291,'004291','1969-109D','KDU','KDU',18376,'Reentered','1970-01-08 00:00:00','1970-01-03 00:00:00',89.17,190,277,65.41,'Intermediate' from dual union all
select 42910,'042910','1998-067MZ','Tomsk-TPU-120','Tomsk-TPU-120',66503,'In Earth orbit','','2017-08-18 00:00:00',92.61,397,408,51.63,'Intermediate' from dual union all
select 42911,'042911','1998-067NA','Tanyusha YuZGU No. 1','Radioskaf RS-6S',66503,'In Earth orbit','','2017-08-17 00:00:00',92.6,397,407,51.64,'Intermediate' from dual union all
select 42912,'042912','1998-067NB','Tanyusha YuZGU No. 2','Radioskaf RS-7S',66503,'In Earth orbit','','2017-08-18 00:00:00',92.63,398,409,51.64,'Intermediate' from dual union all
select 42913,'042913','1998-067NC','TS530-Zerkalo','Sfera-53',66503,'In Earth orbit','','2017-08-18 00:00:00',92.61,398,407,51.64,'Intermediate' from dual union all
select 42914,'042914','1998-067ND','TNS-0-2 Nanosputnik','TEKh-42-2',66503,'In Earth orbit','','2017-08-18 00:00:00',92.61,398,407,51.64,'Intermediate' from dual union all
select 42915,'042915','2017-047A','TDRS 13','TDRS M',70437,'In Earth orbit','','2017-09-19 00:00:00',1436.06,35768,35804,6.99,'Inclined GEO' from dual union all
select 42916,'042916','2017-047B','Centaur AV-074','Centaur AV-074',70437,'In Earth orbit','','2017-09-18 00:00:00',696.88,4499,34820,26.4,'GEO Transfer' from dual union all
select 42917,'042917','2017-048A','Michibiki-3','QZSS 3',70438,'In Earth orbit','','2017-09-19 00:00:00',1436.14,35767,35808,.08,'Stationary' from dual union all
select 42918,'042918','2017-048B','H-2A F35 Stage 2','H-2A Stage 2',70438,'In Earth orbit','','2017-09-19 00:00:00',624.58,312,35340,20.07,'GEO Transfer' from dual union all
select 42919,'042919','2017-037D','Kosmos-2521','Sputnik-Inspektor',70427,'In Earth orbit','','2017-08-25 00:00:00',97.99,650,676,97.92,'Sun-Synch' from dual union all
select 4292,'004292','1970-001A','Kosmos-318','Zenit-2M',21478,'Landed','1970-01-21 00:00:00','1970-01-14 00:00:00',89.2,200,269,64.97,'Intermediate' from dual union all
select 42920,'042920','2017-049A','FORMOSAT 5','Formosat 5',70439,'In Earth orbit','','2017-08-24 00:00:00',99.18,710,729,98.3,'Sun-Synch' from dual union all
select 42921,'042921','2017-050A','ORS-5','ORS-5',70440,'In Earth orbit','','2017-08-26 00:00:00',96.71,599,604,.02,'Equatorial' from dual union all
select 42922,'042922','2017-050B','DHFR','DARPA High Freq. Receive',70440,'In Earth orbit','','2017-08-26 00:00:00',93.73,309,605,24.52,'Equatorial' from dual union all
select 42923,'042923','2017-050C','Prometheus 2.2','Prometheus 2.2',70440,'In Earth orbit','','2017-08-26 00:00:00',94.51,388,601,24.52,'Equatorial' from dual union all
select 42924,'042924','2017-050D','Prometheus 2.4','Prometheus 2.4',70440,'Reentered','2017-09-16 00:00:00','2017-08-26 00:00:00',94.48,384,603,24.53,'Equatorial' from dual union all
select 42925,'042925','2017-050E','Minotaur IV ISA','Orion 38',70440,'In Earth orbit','','2017-08-26 00:00:00',96.71,599,604,.02,'Equatorial' from dual union all
select 42926,'042926','2017-050F','deb Minotaur IV','',70440,'Reentered','2017-09-01 00:00:00','2017-08-31 00:00:00',94.48,384,603,24.53,'Equatorial' from dual union all
select 42927,'042927','2017-050G','Minotaur IV GCA','GCA/Orion 38',70440,'In Earth orbit','','2017-08-26 00:00:00',94.61,400,600,24.6,'Equatorial' from dual union all
select 42928,'042928','2017-051A','IRNSS-R1H','IRNSS-1H',70441,'In Earth orbit','','2017-09-30 00:00:00',156.01,157,6287,19.18,'Medium' from dual union all
select 42929,'042929','2017-051B','deb IRNSS-R1H','',70441,'Reentered','2017-12-05 00:00:00','2017-09-02 00:00:00',158.75,176,6492,19.16,'Medium' from dual union all
select 4293,'004293','1970-001B','11A57I','11A57I',21478,'Reentered','1970-01-15 00:00:00','1970-01-11 00:00:00',88.95,194,252,64.98,'Intermediate' from dual union all
select 42930,'042930','2017-051C','deb IRNSS-R1H','',70441,'Reentered','2017-12-05 00:00:00','2017-09-03 00:00:00',157.31,168,6382,19.16,'Medium' from dual union all
select 42931,'042931','2017-044D','Optsat-3000 optics cover','',70434,'Reentered','2017-12-29 00:00:00','2017-09-06 00:00:00',93.57,438,461,97.22,'Sun-Synch' from dual union all
select 42932,'042973','2010-041N','deb SOZ','',68901,'In Earth orbit','','2017-10-18 00:00:00',340.14,613,18913,65.07,'Highly Ellip' from dual union all
select 42933,'042974','2010-041P','deb SOZ','',68901,'In Earth orbit','','2017-10-18 00:00:00',335.94,805,18454,65.2,'Highly Ellip' from dual union all
select 42934,'042975','2010-041Q','deb SOZ','',68901,'In Earth orbit','','2017-10-18 00:00:00',338.54,557,18868,65.2,'Highly Ellip' from dual union all
select 42935,'042976','2010-041R','deb SOZ','',68901,'In Earth orbit','','2017-10-16 00:00:00',340.22,584,18947,65.24,'Highly Ellip' from dual union all
select 42936,'042977','2010-041S','deb SOZ','',68901,'In Earth orbit','','2017-10-18 00:00:00',328.46,477,18305,64.86,'Highly Ellip' from dual union all
select 42937,'042978','2010-041T','deb SOZ','',68901,'In Earth orbit','','2017-10-17 00:00:00',319.52,754,17453,65.67,'Highly Ellip' from dual union all
select 42938,'042979','2010-041U','deb SOZ','',68901,'In Earth orbit','','2017-10-18 00:00:00',338.38,756,18659,65.19,'Highly Ellip' from dual;
insert into SATELLITE
select 42939,'042980','2010-041V','deb SOZ','',68901,'In Earth orbit','','2017-10-18 00:00:00',343.78,796,18960,65.37,'Highly Ellip' from dual union all
select 4294,'004294','1969-107F','deb Kosmos-315','Tselina-O part',18374,'Reentered','1970-05-14 00:00:00','1970-01-20 00:00:00',94.81,490,529,74.02,'Intermediate' from dual union all
select 42940,'042981','2010-041W','deb SOZ','',68901,'In Earth orbit','','2017-10-13 00:00:00',320.84,766,17527,65.79,'Highly Ellip' from dual union all
select 42941,'042982','1998-067NE','KestrelEye IIM','KestrelEye II-M',66503,'In Earth orbit','','2017-10-24 00:00:00',92.63,400,407,51.64,'Intermediate' from dual union all
select 42942,'042983','1998-067NF','SIMPL','SIMPL',66503,'In Earth orbit','','2017-10-27 00:00:00',92.63,400,407,51.64,'Intermediate' from dual union all
select 42943,'042986','2017-037E','Kosmos-2523','Subsatellite',70427,'In Earth orbit','','2017-10-31 00:00:00',96.86,554,663,97.88,'Sun-Synch' from dual union all
select 42944,'042999','1974-011C','deb Meteor','',33614,'In Earth orbit','','2017-10-31 00:00:00',101.76,813,870,81.22,'Intermediate' from dual union all
select 42945,'043000','1970-093D','DSP 1 Aperture Cover','',21570,'In Earth orbit','','2017-10-31 00:00:00',1194.71,25511,36326,9.26,'Medium' from dual union all
select 42946,'043020','1998-067NH','ASTERIA','ASTERIA',66503,'In Earth orbit','','2017-11-20 00:00:00',92.61,400,405,51.64,'Intermediate' from dual union all
select 42947,'043021','1998-067NJ','Dellingr','Dellingr',66503,'In Earth orbit','','2017-11-20 00:00:00',92.61,401,404,51.64,'Intermediate' from dual union all
select 42948,'043027','1998-067NL','OSIRIS-3U','OSIRIS-3U',66503,'In Earth orbit','','2017-11-21 00:00:00',92.59,394,409,51.65,'Intermediate' from dual union all
select 42949,'043085','1983-044GN','deb Kosmos-1461','',55801,'In Earth orbit','','2017-12-23 00:00:00',96.36,506,663,65,'Intermediate' from dual union all
select 4295,'004295','1969-082G','OPS 7613 P/L 6','POPPY 6 SV4',18349,'In Earth orbit','','1970-01-13 00:00:00',103.44,904,939,70.02,'Intermediate' from dual union all
select 42950,'043086','1983-044GP','deb Kosmos-1461','',55801,'In Earth orbit','','2017-12-24 00:00:00',96.37,505,665,65,'Intermediate' from dual union all
select 42951,'043106','2017-008DK','deb PSLV','',70398,'In Earth orbit','','2018-01-09 00:00:00',94.22,472,489,97.47,'Sun-Synch' from dual union all
select 42952,'043117','1988-053E','deb Kosmos-1954','Strela-2M debris',61302,'In Earth orbit','','2018-05-31 00:00:00',100.21,754,782,74.05,'Intermediate' from dual union all
select 42953,'043151','1985-119C','deb Meteor','',58342,'In Earth orbit','','2018-01-10 00:00:00',103.91,932,955,82.54,'Intermediate' from dual union all
select 42954,'043203','1998-067NM','ShA-317A ONA Receiver','ONA HF Receiver Box',66503,'In Earth orbit','','2018-02-03 00:00:00',92.61,401,404,51.65,'Intermediate' from dual union all
select 42955,'043206','1998-067NN','ISS deb','',66503,'In Earth orbit','','2018-02-07 00:00:00',92.63,401,406,51.64,'Intermediate' from dual union all
select 42956,'043213','1999-025EYM','deb FY-1C','',66706,'In Earth orbit','','2018-02-16 00:00:00',102.05,809,903,98.94,'Sun-Synch' from dual union all
select 42957,'043214','1999-025EYN','deb FY-1C','',66706,'In Earth orbit','','2018-02-16 00:00:00',102.05,808,903,98.94,'Sun-Synch' from dual union all
select 42958,'043282','1969-013C','deb Transtage 17','',18280,'In Earth orbit','','2018-04-09 00:00:00',1575.26,37234,39703,5.95,'Very High' from dual union all
select 42959,'043283','1969-013D','deb Transtage 17','',18280,'In Earth orbit','','2018-04-09 00:00:00',1546.66,36521,39327,6.49,'Very High' from dual union all
select 4296,'004296','1970-002A','OPS 6531','GAMBIT-3 Mission 4325',21479,'Reentered Attached','1970-02-01 00:00:00','1970-01-21 00:00:00',89.51,126,375,109.94,'Retrograde' from dual union all
select 42960,'043284','1969-013E','deb Transtage 17','',18280,'In Earth orbit','','2018-04-06 00:00:00',1481.02,36060,37263,6.18,'Inclined Drift' from dual union all
select 42961,'043285','1969-013F','deb Transtage 17','',18280,'In Earth orbit','','2018-04-09 00:00:00',1542.29,37107,38574,6.7,'Very High' from dual union all
select 42962,'043288','1963-014FL','deb West Ford','',9245,'In Earth orbit','','2018-04-10 00:00:00',165.94,2483,4767,87.06,'Medium' from dual union all
select 42963,'043289','1964-083T','deb VE-5','',10369,'In Earth orbit','','2018-04-13 00:00:00',100.12,667,861,89.87,'Polar' from dual union all
select 42964,'043290','1964-083U','deb VE-5','',10369,'In Earth orbit','','2018-04-13 00:00:00',98.7,639,754,89.88,'Polar' from dual union all
select 42965,'043291','1971-120V','deb Meteor','',24938,'In Earth orbit','','2018-04-13 00:00:00',98.37,676,685,81.24,'Intermediate' from dual union all
select 42966,'043292','1971-120W','deb Meteor','',24938,'In Earth orbit','','2018-04-13 00:00:00',98.34,670,689,81.23,'Intermediate' from dual union all
select 42967,'043293','1971-120X','deb Meteor','',24938,'In Earth orbit','','2018-04-13 00:00:00',98.43,679,689,81.24,'Intermediate' from dual union all
select 42968,'043294','1976-108C','deb Kosmos-864','Parus fragment',39060,'In Earth orbit','','2018-04-13 00:00:00',104.68,958,1001,82.94,'Intermediate' from dual union all
select 42969,'043295','1979-057E','deb NOAA 6','',47541,'In Earth orbit','','2018-04-13 00:00:00',98.5,670,704,98.47,'Sun-Synch' from dual union all
select 4297,'004297','1970-003A','INTELSAT III F-6','INTELSAT III F-6',21480,'In Earth orbit','','1970-03-09 00:00:00',1436.39,35784,35801,.86,'Stationary' from dual union all
select 42970,'043296','1979-057F','deb NOAA 6','',47541,'Reentered','2018-04-10 00:00:00','2018-04-06 00:00:00',92.73,392,425,98.38,'Sun-Synch' from dual union all
select 42971,'043297','1980-073R','deb Meteor-2','',50283,'In Earth orbit','','2018-04-13 00:00:00',101.5,799,860,81.22,'Intermediate' from dual union all
select 42972,'043298','1980-073S','deb Meteor-2','',50283,'In Earth orbit','','2018-04-08 00:00:00',101.46,806,850,81.21,'Intermediate' from dual union all
select 42973,'043299','1980-073T','deb Meteor-2','',50283,'In Earth orbit','','2018-04-09 00:00:00',101.38,803,844,81.22,'Intermediate' from dual union all
select 42974,'043300','1981-043X','deb Meteor-2','',52566,'In Earth orbit','','2018-04-13 00:00:00',101.91,823,876,81.27,'Intermediate' from dual union all
select 42975,'043301','1981-043Y','deb Meteor-2','',52566,'In Earth orbit','','2018-04-11 00:00:00',101.98,819,886,81.26,'Intermediate' from dual union all
select 42976,'043302','1981-043Z','deb Meteor-2','',52566,'In Earth orbit','','2018-04-10 00:00:00',101.96,822,881,81.26,'Intermediate' from dual union all
select 42977,'043303','1983-044GQ','deb Kosmos-1461','',55801,'In Earth orbit','','2018-04-13 00:00:00',95.71,480,626,65,'Intermediate' from dual union all
select 42978,'043304','1983-044GR','deb Kosmos-1461','',55801,'In Earth orbit','','2018-04-13 00:00:00',95.82,483,634,65,'Intermediate' from dual union all
select 42979,'043305','1983-044GS','deb Kosmos-1461','',55801,'In Earth orbit','','2018-04-13 00:00:00',96.19,492,661,65.01,'Intermediate' from dual union all
select 4298,'004298','1970-003B','Star 37D S/N 016','Star 37D',21480,'In Earth orbit','','1970-02-26 00:00:00',631.84,313,35713,28.11,'GEO Transfer' from dual union all
select 42980,'043306','1983-044GT','deb Kosmos-1461','',55801,'In Earth orbit','','2018-04-13 00:00:00',95.47,476,606,65.01,'Intermediate' from dual union all
select 42981,'043307','1983-044GU','deb Kosmos-1461','',55801,'In Earth orbit','','2018-04-13 00:00:00',95.41,476,600,64.99,'Intermediate' from dual union all
select 42982,'043308','1983-044GV','deb Kosmos-1461','',55801,'In Earth orbit','','2018-04-13 00:00:00',95.48,476,608,65,'Intermediate' from dual union all
select 42983,'043309','1984-072U','deb Meteor-2','',57126,'In Earth orbit','','2018-04-13 00:00:00',103.74,919,952,82.53,'Intermediate' from dual union all
select 42984,'043310','1984-072V','deb Meteor-2','',57126,'In Earth orbit','','2018-04-11 00:00:00',103.89,931,954,82.54,'Intermediate' from dual union all
select 42985,'043311','1984-072W','deb Meteor-2','',57126,'In Earth orbit','','2018-04-09 00:00:00',103.85,913,968,82.53,'Intermediate' from dual union all
select 42986,'043312','1984-106AE','deb SOZ','',57160,'In Earth orbit','','2018-04-12 00:00:00',101.77,830,855,71.03,'Intermediate' from dual union all
select 42987,'043313','1985-047C','deb Kosmos-1660','Musson debris',58270,'In Earth orbit','','2018-04-13 00:00:00',116.02,1481,1522,73.63,'Intermediate' from dual union all
select 42988,'043314','1985-094Y','deb Kosmos-1695','',58317,'In Earth orbit','','2018-04-12 00:00:00',112.77,1339,1369,82.56,'Intermediate' from dual union all
select 42989,'043315','1985-094Z','deb Kosmos-1695','',58317,'In Earth orbit','','2018-04-11 00:00:00',113.54,1363,1415,82.56,'Intermediate' from dual union all
select 4299,'004299','1970-004A','Kosmos-319','DS-P1-Yu No. 25',21481,'Reentered','1970-07-01 00:00:00','1970-02-14 00:00:00',100.52,196,1371,81.96,'Intermediate' from dual union all
select 42990,'043316','1985-094AA','deb Kosmos-1695','',58317,'In Earth orbit','','2018-04-09 00:00:00',113.44,1370,1399,82.61,'Intermediate' from dual union all
select 42991,'043317','1985-094AB','deb Kosmos-1695','',58317,'In Earth orbit','','2018-04-12 00:00:00',112.94,1344,1379,82.56,'Intermediate' from dual union all
select 42992,'043318','1985-094AC','deb Kosmos-1695','',58317,'In Earth orbit','','2018-04-09 00:00:00',112.55,1335,1352,82.58,'Intermediate' from dual union all
select 42993,'043319','1986-073H','deb NOAA 10','',59266,'In Earth orbit','','2018-04-13 00:00:00',96.88,541,678,98.33,'Sun-Synch' from dual union all
select 42994,'043320','1986-094C','deb Kosmos-1803','Musson debris',59287,'In Earth orbit','','2018-04-02 00:00:00',115.92,1493,1502,82.61,'Intermediate' from dual union all
select 42995,'043321','1986-094D','deb Kosmos-1803','Musson debris',59287,'In Earth orbit','','2018-03-30 00:00:00',115.92,1493,1502,82.6,'Intermediate' from dual union all
select 42996,'043322','1987-020FK','deb Kosmos-1823','',60110,'In Earth orbit','','2018-04-12 00:00:00',115.97,1476,1523,73.6,'Intermediate' from dual union all
select 42997,'043323','1987-027G','deb Kosmos-1833','',60117,'In Earth orbit','','2018-04-13 00:00:00',101.03,793,822,71,'Intermediate' from dual union all
select 42998,'043324','1987-060AR','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2018-04-13 00:00:00',100.58,774,798,65.01,'Intermediate' from dual union all
select 42999,'043325','1987-060AS','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2018-04-13 00:00:00',100.53,771,797,65.01,'Intermediate' from dual union all
select 43,'000043','60 Zeta 1','Midas 2','Midas ETS-I F2',6335,'Reentered Attached','1974-02-07 00:00:00','1960-06-26 00:00:00',94.57,482,514,33,'Intermediate' from dual union all
select 430,'000430','62 A Omega 6','deb Kosmos-9','',8148,'Reentered','1962-10-08 00:00:00','1962-10-02 00:00:00',90.56,297,307,64.99,'Intermediate' from dual union all
select 4300,'004300','1970-004B','S1M','S1M',21481,'Reentered','1970-05-01 00:00:00','1970-02-17 00:00:00',99.26,195,1252,81.94,'Intermediate' from dual union all
select 43000,'043326','1987-060AT','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2018-04-11 00:00:00',100.57,774,798,65.01,'Intermediate' from dual union all
select 43001,'043327','1987-060AU','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2018-04-13 00:00:00',100.53,771,797,65.01,'Intermediate' from dual union all
select 43002,'043328','1987-060AV','deb Kosmos-1867 coolant','',60150,'In Earth orbit','','2018-04-13 00:00:00',100.56,773,798,65.01,'Intermediate' from dual union all
select 43003,'043329','1988-006N','deb DMSP F-9','',61255,'In Earth orbit','','2018-04-13 00:00:00',100.2,732,804,99.1,'Sun-Synch' from dual union all
select 43004,'043330','1988-033L','deb NNS O-23 insulation','',61282,'In Earth orbit','','2018-01-15 00:00:00',108.31,1004,1294,90.36,'Polar' from dual union all
select 43005,'043331','1988-033M','deb NNS O-23 insulation','',61282,'In Earth orbit','','2018-03-14 00:00:00',108.03,1005,1267,90.36,'Polar' from dual union all
select 43006,'043332','1988-033N','deb NNS O-23 insulation','',61282,'In Earth orbit','','2018-03-18 00:00:00',108.1,1002,1275,90.36,'Polar' from dual union all
select 43007,'043333','1988-051G','Mage 1','Mage 1',61300,'In Earth orbit','','2018-04-13 00:00:00',1446.52,35756,36224,15.14,'Inclined Drift' from dual union all
select 43008,'043334','1989-001Y','deb SOZ','',62278,'In Earth orbit','','2018-04-13 00:00:00',146.04,896,4720,65.03,'Medium' from dual union all
select 43009,'043335','1989-028C','deb Kosmos-2016','Parus fragment',62305,'In Earth orbit','','2018-04-13 00:00:00',104.39,937,995,82.94,'Intermediate' from dual union all
select 4301,'004301','1970-005A','Kosmos-320','DS-MO No. 3',21482,'Reentered','1970-02-10 00:00:00','1970-01-25 00:00:00',89.89,240,298,48.41,'Intermediate' from dual union all
select 43010,'043336','1989-050C','deb Nadezhda','Nadezhda fragment',62327,'In Earth orbit','','2018-04-02 00:00:00',104.51,939,1004,82.95,'Intermediate' from dual union all
select 43011,'043337','1989-084E','deb IUS-19 SRM-1','slag',62361,'In Earth orbit','','2018-03-15 00:00:00',95.78,183,930,34.09,'Intermediate' from dual union all
select 43012,'043343','1990-010C','deb Kosmos-2058','Tselina-R fragment',63252,'In Earth orbit','','2018-04-10 00:00:00',95.71,545,561,82.48,'Intermediate' from dual union all
select 43013,'043344','1994-057G','deb DMSP F-12','',65453,'In Earth orbit','','2018-04-16 00:00:00',101.65,826,848,99.07,'Sun-Synch' from dual union all
select 43014,'043345','1994-057H','deb DMSP F-12','',65453,'In Earth orbit','','2018-04-16 00:00:00',99.96,726,788,99.05,'Sun-Synch' from dual union all
select 43015,'043346','1994-057J','deb DMSP F-12','',65453,'In Earth orbit','','2018-04-16 00:00:00',99.68,714,773,99.05,'Sun-Synch' from dual union all
select 43016,'043347','1994-074AK','deb Resurs-O1','',65470,'In Earth orbit','','2018-04-16 00:00:00',96.95,610,615,98.09,'Sun-Synch' from dual union all
select 43017,'043348','1994-074AL','deb Resurs-O1','',65470,'In Earth orbit','','2018-04-16 00:00:00',97.15,621,624,98.07,'Sun-Synch' from dual union all
select 43018,'043349','1994-074AM','deb Resurs-O1','',65470,'In Earth orbit','','2018-04-16 00:00:00',96.94,610,615,98.1,'Sun-Synch' from dual union all
select 43019,'043350','1994-074AN','deb Resurs-O1','',65470,'In Earth orbit','','2018-04-15 00:00:00',96.84,604,611,98.07,'Sun-Synch' from dual union all
select 4302,'004302','1970-005B','S1M','S1M',21482,'Reentered','1970-01-28 00:00:00','1970-01-18 00:00:00',90.25,255,319,48.41,'Intermediate' from dual union all
select 43020,'043351','1995-021J','deb ERS-2','',65682,'In Earth orbit','','2018-04-16 00:00:00',99.62,735,746,98.75,'Sun-Synch' from dual union all
select 43021,'043352','1995-058M','Zenit part','',65719,'In Earth orbit','','2018-04-16 00:00:00',100.46,774,787,71.01,'Intermediate' from dual union all
select 43022,'043353','1995-058N','Zenit part','',65719,'In Earth orbit','','2018-04-16 00:00:00',101.13,796,829,71.01,'Intermediate' from dual union all
select 43023,'043354','1996-051P','deb Zenit 11S772','',65939,'In Earth orbit','','2018-04-15 00:00:00',101.9,780,918,71.2,'Intermediate' from dual union all
select 43024,'043355','1997-029D','FY-2A imager cover','ISON UU061',66109,'In Earth orbit','','2017-11-15 00:00:00',1436.78,35194,36406,14.23,'Inclined Drift' from dual union all
select 43025,'043356','1998-008G','deb Globalstar','',66444,'In Earth orbit','','2018-04-16 00:00:00',113.01,1330,1400,51.91,'Intermediate' from dual union all
select 43026,'043357','1998-008H','deb Globalstar','',66444,'In Earth orbit','','2018-04-16 00:00:00',114.59,1404,1470,51.96,'Intermediate' from dual union all
select 43027,'043358','1999-025EYP','deb FY-1C','',66706,'In Earth orbit','','2018-04-16 00:00:00',107.93,846,1415,98.68,'Sun-Synch' from dual;
insert into SATELLITE
select 43028,'043359','1999-025EYQ','deb FY-1C','',66706,'In Earth orbit','','2018-04-14 00:00:00',102.45,808,942,99.19,'Sun-Synch' from dual union all
select 43029,'043360','1999-040H','deb IUS-27 SRM-1 motor slag','',66721,'In Earth orbit','','2018-04-16 00:00:00',202.06,749,9308,28,'Medium' from dual union all
select 4303,'004303','1970-005C','deb Kosmos-320','',21482,'Reentered','1970-01-24 00:00:00','1970-01-18 00:00:00',91.11,288,370,48.37,'Intermediate' from dual union all
select 43030,'043361','1999-040J','deb IUS-27 SRM-1 motor slag','',66721,'In Earth orbit','','2018-01-29 00:00:00',198.92,434,9385,28.26,'Medium' from dual union all
select 43031,'043362','1999-040K','deb IUS-27 SRM-1 motor slag','',66721,'In Earth orbit','','2018-02-20 00:00:00',203.63,468,9707,28.31,'Medium' from dual union all
select 43032,'043363','1999-040L','deb IUS-27 SRM-1 motor slag','',66721,'In Earth orbit','','2018-04-16 00:00:00',179.3,437,7872,28.37,'Medium' from dual union all
select 43033,'043364','1999-040M','deb IUS-27 SRM-1 motor slag','',66721,'In Earth orbit','','2018-04-16 00:00:00',204.41,649,9584,28.18,'Medium' from dual union all
select 43034,'043365','1999-040N','deb IUS-27 SRM-1 motor slag','',66721,'In Earth orbit','','2018-04-13 00:00:00',201.17,361,9629,28.48,'Medium' from dual union all
select 43035,'043366','1999-040P','deb IUS-27 SRM-1 motor slag','',66721,'In Earth orbit','','2018-04-16 00:00:00',204.89,592,9677,28.2,'Medium' from dual union all
select 43036,'043367','1999-040Q','deb IUS-27 SRM-1 motor slag','',66721,'In Earth orbit','','2018-04-16 00:00:00',204.38,539,9692,28.25,'Medium' from dual union all
select 43037,'043368','2000-055UC','deb NOAA 16','',66957,'In Earth orbit','','2018-04-16 00:00:00',100.9,773,830,99.14,'Sun-Synch' from dual union all
select 43038,'043369','2001-056N','deb 11S772','',67209,'In Earth orbit','','2018-04-12 00:00:00',101.95,780,921,99.27,'Sun-Synch' from dual union all
select 43039,'043370','2001-056P','deb 11S772','',67209,'In Earth orbit','','2018-04-15 00:00:00',106.25,952,1154,99.37,'Sun-Synch' from dual union all
select 4304,'004304','1970-005D','deb Kosmos-320','',21482,'Reentered','1970-02-11 00:00:00','1970-01-24 00:00:00',92.35,292,488,48.41,'Intermediate' from dual union all
select 43040,'043371','2002-003F','deb H2A','',67379,'In Earth orbit','','2018-04-16 00:00:00',323.85,393,18093,28.97,'Highly Ellip' from dual union all
select 43041,'043372','2002-037BW','deb SOZ','',67413,'In Earth orbit','','2018-04-16 00:00:00',118.82,1340,1916,63.4,'Intermediate' from dual union all
select 43042,'043373','2003-061C','deb Tan Ce 1','',67685,'In Earth orbit','','2018-03-11 00:00:00',408.63,236,23490,18.8,'Highly Ellip' from dual union all
select 43043,'043374','2007-029R','deb 11S772 sep motor','deb Tselina-2',68371,'In Earth orbit','','2018-04-13 00:00:00',104.73,819,1145,71.07,'Intermediate' from dual union all
select 43044,'043375','2007-029S','deb 11S772 sep motor','deb Tselina-2',68371,'In Earth orbit','','2018-04-10 00:00:00',102.83,802,983,70.91,'Intermediate' from dual union all
select 43045,'043376','2007-052AC','deb SOZ','',68394,'In Earth orbit','','2018-02-21 00:00:00',351.74,601,19655,65.29,'Highly Ellip' from dual union all
select 43046,'043377','2007-052AD','deb SOZ','',68394,'In Earth orbit','','2018-04-14 00:00:00',338.18,441,18961,64.42,'Highly Ellip' from dual union all
select 43047,'043378','2007-052AE','deb SOZ','',68394,'In Earth orbit','','2018-04-16 00:00:00',336.95,463,18861,64.17,'Highly Ellip' from dual union all
select 43048,'043379','2007-052AF','deb SOZ','',68394,'In Earth orbit','','2018-04-15 00:00:00',334.95,421,18776,64.01,'Highly Ellip' from dual union all
select 43049,'043380','2007-052AG','deb SOZ','',68394,'In Earth orbit','','2018-02-16 00:00:00',337.77,468,18908,64.36,'Highly Ellip' from dual union all
select 4305,'004305','1970-005E','deb Kosmos-320','',21482,'Reentered','1970-03-03 00:00:00','1970-01-26 00:00:00',91.91,299,437,48.4,'Intermediate' from dual union all
select 43050,'043381','2008-020C','deb Fregat No. 1008','',68528,'In Earth orbit','','2018-04-16 00:00:00',858.15,21659,25412,56.77,'Medium' from dual union all
select 43051,'043382','2008-030E','deb ESC-A L540','',68538,'In Earth orbit','','2018-04-16 00:00:00',587.32,306,33401,1.78,'Highly Ellip' from dual union all
select 43052,'043383','2008-030F','deb ESC-A L540','',68538,'In Earth orbit','','2018-03-24 00:00:00',626.07,639,35090,1.56,'GEO Transfer' from dual union all
select 43053,'043384','2008-030G','deb ESC-A L540','',68538,'In Earth orbit','','2018-04-15 00:00:00',628.36,395,35452,1.99,'GEO Transfer' from dual union all
select 43054,'043385','2009-009D','deb Blok DM-SLB','',68690,'In Earth orbit','','2018-03-29 00:00:00',650.87,1163,35838,34.25,'GEO Transfer' from dual union all
select 43055,'043386','2009-055M','deb Worldview-2','',68736,'In Earth orbit','','2018-04-16 00:00:00',99.43,709,754,98.45,'Sun-Synch' from dual union all
select 43056,'043387','2009-055N','deb Worldview-2','',68736,'In Earth orbit','','2018-04-15 00:00:00',98.99,700,721,98.71,'Sun-Synch' from dual union all
select 43057,'043388','2009-055P','deb Worldview-2','',68736,'In Earth orbit','','2018-04-12 00:00:00',97.74,622,680,98.78,'Sun-Synch' from dual union all
select 43058,'043389','2009-055Q','deb Worldview-2','',68736,'In Earth orbit','','2018-04-13 00:00:00',96.29,537,625,98.73,'Sun-Synch' from dual union all
select 43059,'043390','2009-055R','deb Worldview-2','',68736,'In Earth orbit','','2018-04-16 00:00:00',99.21,708,734,98.57,'Sun-Synch' from dual union all
select 4306,'004306','1970-005F','deb Kosmos-320','',21482,'Reentered','1970-01-24 00:00:00','1970-01-18 00:00:00',91.52,291,408,48.41,'Intermediate' from dual union all
select 43060,'043391','2009-055S','deb Worldview-2','',68736,'In Earth orbit','','2018-04-16 00:00:00',99.42,710,751,98.69,'Sun-Synch' from dual union all
select 43061,'043392','2009-055T','deb Worldview-2','',68736,'In Earth orbit','','2018-04-16 00:00:00',99.89,739,768,98.59,'Sun-Synch' from dual union all
select 43062,'043393','2010-023C','deb SERVIS-2','',68883,'In Earth orbit','','2018-04-16 00:00:00',108.53,1144,1174,100.4,'Sun-Synch' from dual union all
select 43063,'043394','2010-023D','deb SERVIS-2','',68883,'In Earth orbit','','2018-01-11 00:00:00',108.42,1124,1184,100.43,'Sun-Synch' from dual union all
select 43064,'043395','2011-001J','deb Fregat-SB No. 2001','',69053,'In Earth orbit','','2018-02-18 00:00:00',1399.87,34463,35686,4.61,'Inclined Drift' from dual union all
select 43065,'043396','2012-002D','FY-2F imager cover','',69257,'In Earth orbit','','2017-11-30 00:00:00',1436.7,35211,36386,2.05,'Drift GEO' from dual union all
select 43066,'043397','2012-025Q','deb H-2A F14','',69280,'In Earth orbit','','2018-04-16 00:00:00',95.33,529,541,98.34,'Sun-Synch' from dual union all
select 43067,'043398','2012-025R','deb H-2A F14','',69280,'In Earth orbit','','2018-04-16 00:00:00',94.96,512,522,98.33,'Sun-Synch' from dual union all
select 43068,'043399','2012-025S','deb H-2A F14','',69280,'In Earth orbit','','2018-04-16 00:00:00',96.97,605,623,98.31,'Sun-Synch' from dual union all
select 43069,'043400','2012-026M','deb Briz-M No. 99530','',69281,'In Earth orbit','','2018-02-18 00:00:00',815.05,10669,34382,12.84,'Medium' from dual union all
select 4307,'004307','1970-005G','deb Kosmos-320','',21482,'Reentered','1970-01-29 00:00:00','1970-01-18 00:00:00',91.34,282,398,48.36,'Intermediate' from dual union all
select 43070,'043401','2012-026N','deb Briz-M No. 99530','',69281,'In Earth orbit','','2018-04-16 00:00:00',819.12,10625,34618,12.39,'Medium' from dual union all
select 43071,'043402','2012-052B','deb CZ-2D sep motor cover','',69307,'In Earth orbit','','2018-04-16 00:00:00',101.04,633,983,98.07,'Sun-Synch' from dual union all
select 43072,'043403','2012-052C','deb CZ-2D sep motor cover','',69307,'In Earth orbit','','2018-04-16 00:00:00',100.78,635,957,98.02,'Sun-Synch' from dual union all
select 43073,'043404','2012-066G','deb CZ-4C sep motor cover','',69321,'In Earth orbit','','2018-04-15 00:00:00',111.38,1203,1378,63.36,'Intermediate' from dual union all
select 43074,'043405','2012-073C','deb Gokturk-2','',69328,'In Earth orbit','','2018-04-15 00:00:00',95.06,473,570,98.22,'Sun-Synch' from dual union all
select 43075,'043406','2012-073D','deb Gokturk-2','',69328,'In Earth orbit','','2018-04-16 00:00:00',97.16,549,696,97.96,'Sun-Synch' from dual union all
select 43076,'043407','2013-014D','deb Briz-M No. 99538','',69477,'In Earth orbit','','2018-04-17 00:00:00',751.88,7712,34312,14.63,'Medium' from dual union all
select 43077,'043408','2013-052C','CZ-4C sep motor cover','',69515,'In Earth orbit','','2018-04-17 00:00:00',104.37,801,1129,98.75,'Sun-Synch' from dual union all
select 43078,'043409','2013-052D','CZ-4C sep motor cover','',69515,'In Earth orbit','','2018-04-16 00:00:00',104.67,801,1157,98.75,'Sun-Synch' from dual union all
select 43079,'043410','2013-052E','CZ-4C sep motor cover','',69515,'In Earth orbit','','2018-04-17 00:00:00',104.77,801,1167,98.82,'Sun-Synch' from dual union all
select 4308,'004308','1970-006A','Kosmos-321','DS-U2-MG No. 1',21483,'Reentered','1970-03-23 00:00:00','1970-02-19 00:00:00',91.29,259,417,70.97,'Intermediate' from dual union all
select 43080,'043411','2013-052F','CZ-4C sep motor cover','',69515,'In Earth orbit','','2018-04-17 00:00:00',100.94,799,808,98.67,'Sun-Synch' from dual union all
select 43081,'043412','2013-065C','CZ-4C sep motor cover','',69528,'In Earth orbit','','2018-04-16 00:00:00',109.31,1094,1295,100.48,'Sun-Synch' from dual union all
select 43082,'043413','2013-068B','CZ-2C sep motor cover','',69531,'In Earth orbit','','2018-04-17 00:00:00',100.08,669,856,97.97,'Sun-Synch' from dual union all
select 43083,'043414','2014-007D','deb Briz-M No. 99544','',69711,'In Earth orbit','','2018-04-16 00:00:00',760.25,8224,34206,14.37,'Medium' from dual union all
select 43084,'043415','2014-015E','DMSP F-19 OLS Cover','',69719,'In Earth orbit','','2018-04-17 00:00:00',101.57,827,839,98.77,'Sun-Synch' from dual union all
select 43085,'043416','2014-015F','DMSP F-19 OLS Cooler Cover','',69719,'In Earth orbit','','2018-04-17 00:00:00',101.61,828,842,98.78,'Sun-Synch' from dual union all
select 43086,'043417','2014-016J','deb Sentinel-1A solar array','',69720,'In Earth orbit','','2018-04-17 00:00:00',95.81,549,567,98.18,'Sun-Synch' from dual union all
select 43087,'043418','2014-080G','CZ-4C sep motor cover','',69784,'In Earth orbit','','2018-04-13 00:00:00',107.09,1005,1178,63.41,'Intermediate' from dual union all
select 43088,'043419','2014-080H','CZ-4C sep motor cover','',69784,'In Earth orbit','','2018-04-17 00:00:00',107.01,997,1180,63.41,'Intermediate' from dual union all
select 43089,'043420','2014-080J','CZ-4C sep motor cover','',69784,'In Earth orbit','','2018-04-17 00:00:00',107,996,1179,63.41,'Intermediate' from dual union all
select 4309,'004309','1970-006B','S1M','S1M',21483,'Reentered','1970-03-07 00:00:00','1970-01-27 00:00:00',91.7,267,449,70.96,'Intermediate' from dual union all
select 43090,'043421','2014-080K','CZ-4C sep motor cover','',69784,'In Earth orbit','','2018-04-17 00:00:00',107.13,1009,1179,63.41,'Intermediate' from dual union all
select 43091,'043422','2015-030C','CZ-4B sep motor cover','',69968,'In Earth orbit','','2018-04-15 00:00:00',93.75,454,462,97.41,'Sun-Synch' from dual union all
select 43092,'043423','2015-030D','CZ-4B sep motor cover','',69968,'In Earth orbit','','2018-04-17 00:00:00',93.72,451,462,97.41,'Sun-Synch' from dual union all
select 43093,'043424','2015-040C','CZ-4C sep motor cover','',69978,'In Earth orbit','','2018-04-15 00:00:00',109.92,1108,1338,100.35,'Sun-Synch' from dual union all
select 43094,'043425','2015-040D','CZ-4C sep motor cover','',69978,'In Earth orbit','','2018-04-16 00:00:00',108.93,1070,1284,100.42,'Sun-Synch' from dual union all
select 43095,'043426','2015-078B','CZ-2C sep motor cover','',70016,'In Earth orbit','','2018-04-17 00:00:00',97.47,494,781,97.39,'Sun-Synch' from dual union all
select 43096,'043427','2016-033E','CZ-4B sep motor cover','',70211,'In Earth orbit','','2018-04-17 00:00:00',97.52,489,791,97.49,'Sun-Synch' from dual union all
select 43097,'043428','2016-043C','CZ-4B sep motor cover','',70221,'In Earth orbit','','2018-04-15 00:00:00',101,601,1011,74.97,'Intermediate' from dual union all
select 43098,'043429','2016-043D','CZ-4B sep motor cover','',70221,'In Earth orbit','','2018-04-17 00:00:00',100.33,602,947,74.97,'Intermediate' from dual union all
select 43099,'043430','2017-034F','CZ-4B sep motor cover','',70424,'In Earth orbit','','2018-04-15 00:00:00',99.19,547,893,43.01,'Intermediate' from dual union all
select 431,'000431','62 A Omega 7','deb Kosmos-9','',8148,'Reentered','1962-10-03 00:00:00','1962-10-01 00:00:00',90.56,297,307,64.99,'Intermediate' from dual union all
select 4310,'004310','1969-082DQ','deb Agena D','',18349,'Reentered','1982-04-07 00:00:00','1970-01-23 00:00:00',102.6,798,966,69.76,'Intermediate' from dual union all
select 43100,'043431','2017-034G','CZ-4B sep motor cover','',70424,'In Earth orbit','','2018-04-17 00:00:00',98.94,548,868,43.1,'Intermediate' from dual union all
select 43101,'043447','1969-013G','deb Transtage 17','',18280,'In Earth orbit','','2018-04-25 00:00:00',1482.39,36110,37266,6.22,'Inclined Drift' from dual union all
select 43102,'043448','1969-013H','deb Transtage 17','',18280,'In Earth orbit','','2018-04-25 00:00:00',1459.37,35057,37425,6.43,'Inclined Drift' from dual union all
select 43103,'043449','1969-013J','deb Transtage 17','',18280,'In Earth orbit','','2018-04-26 00:00:00',1490.81,36342,37360,6.09,'Inclined Drift' from dual union all
select 43104,'043452','1969-013K','deb Transtage 17','',18280,'In Earth orbit','','2018-05-03 00:00:00',1396.6,32122,37898,6.44,'Near-synch' from dual union all
select 43105,'043453','1969-013L','deb Transtage 17','',18280,'In Earth orbit','','2018-05-04 00:00:00',1341.19,29963,37853,6.64,'Medium' from dual union all
select 43106,'043454','1969-013M','deb Transtage 17','',18280,'In Earth orbit','','2018-05-02 00:00:00',1400.17,32893,37268,6.46,'Near-synch' from dual union all
select 43107,'043455','1969-013N','deb Transtage 17','',18280,'In Earth orbit','','2018-05-02 00:00:00',1527.46,36723,38390,7.18,'Very High' from dual union all
select 43108,'043456','1969-013P','deb Transtage 17','',18280,'In Earth orbit','','2018-04-30 00:00:00',1397,31387,38649,5.69,'Near-synch' from dual union all
select 43109,'043499','1987-101D','deb Kosmos-1900 coolant','',60191,'In Earth orbit','','2018-06-13 00:00:00',98.9,688,725,66.08,'Intermediate' from dual union all
select 4311,'004311','1969-082DR','deb Agena D','',18349,'In Earth orbit','','1969-11-15 00:00:00',102.06,767,946,70.31,'Intermediate' from dual union all
select 43110,'043500','1987-101E','deb Kosmos-1900 coolant','',60191,'In Earth orbit','','2018-06-14 00:00:00',98.87,687,723,66.09,'Intermediate' from dual union all
select 43111,'043501','1987-101F','deb Kosmos-1900 coolant','',60191,'In Earth orbit','','2018-05-19 00:00:00',98.94,688,728,66.08,'Intermediate' from dual union all
select 43112,'043502','1990-055E','deb Kosmos-2084','',63297,'In Earth orbit','','2018-06-15 00:00:00',92.24,353,416,62.77,'Intermediate' from dual union all
select 43113,'043503','2008-061D','CZ-2D sep motor cover','',68569,'In Earth orbit','','2018-06-15 00:00:00',96.07,516,625,97.71,'Sun-Synch' from dual union all
select 4312,'004312','1969-082DS','deb Agena D','',18349,'Reentered','1983-05-26 00:00:00','1969-11-30 00:00:00',103.62,784,1076,70.14,'Intermediate' from dual union all
select 4313,'004313','1970-006C','deb Kosmos-321','',21483,'Reentered','1970-01-25 00:00:00','1970-01-22 00:00:00',91.31,266,411,70.99,'Intermediate' from dual union all
select 4314,'004314','1970-006D','deb Kosmos-321','',21483,'Reentered','1970-01-29 00:00:00','1970-01-22 00:00:00',91.48,192,503,70.8,'Intermediate' from dual union all
select 4315,'004315','1970-007A','Kosmos-322','Zenit-4 No. 70',21484,'Landed','1970-01-29 00:00:00','1970-01-24 00:00:00',89.58,194,314,65.42,'Intermediate' from dual;
insert into SATELLITE
select 4316,'004316','1970-007B','11A57I','11A57I',21484,'Reentered','1970-01-25 00:00:00','1970-01-23 00:00:00',89.17,167,300,65.42,'Intermediate' from dual union all
select 4317,'004317','1969-082DT','deb Agena D','',18349,'Reentered','1982-11-12 00:00:00','1970-01-22 00:00:00',102.52,846,909,70.16,'Intermediate' from dual union all
select 4318,'004318','1969-082DU','deb Agena D','',18349,'Reentered','2012-12-01 00:00:00','1970-02-07 00:00:00',102.56,832,928,70.03,'Intermediate' from dual union all
select 4319,'004319','1969-082DV','deb Agena D','',18349,'Reentered','1981-11-06 00:00:00','1969-11-27 00:00:00',102.12,802,916,70.06,'Intermediate' from dual union all
select 432,'000432','62 B Gamma 1','Explorer 14','EPE B',8167,'In Earth orbit','1965-01-01 00:00:00','1964-10-20 00:00:00',2157.87,915,96959,33.6,'Very High' from dual union all
select 4320,'004320','1970-008A','ITOS 1','Tiros M',21485,'In Earth orbit','','1970-04-04 00:00:00',115.04,1433,1482,102.01,'Sun-Synch' from dual union all
select 4321,'004321','1970-008B','Australis-OSCAR 5','Australis',21485,'In Earth orbit','','1970-02-25 00:00:00',115.01,1434,1478,101.97,'Sun-Synch' from dual union all
select 4322,'004322','1970-008C','Delta 76','DSV-3E-3 76',21485,'In Earth orbit','','1970-06-17 00:00:00',115.04,1434,1481,101.96,'Sun-Synch' from dual union all
select 4323,'004323','1969-082DW','deb Agena D','',18349,'Reentered','1971-02-03 00:00:00','1970-02-01 00:00:00',96.45,508,670,69.62,'Intermediate' from dual union all
select 4324,'004324','1969-082DX','deb Agena D','',18349,'Reentered','1970-05-10 00:00:00','1970-02-01 00:00:00',119.62,357,2971,69.59,'Intermediate' from dual union all
select 4325,'004325','1969-082DY','deb Agena D','',18349,'Reentered','2000-10-21 00:00:00','1992-02-05 00:00:00',100.52,699,868,70.34,'Intermediate' from dual union all
select 4326,'004326','1969-082DZ','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',101.5,776,884,70.05,'Intermediate' from dual union all
select 4327,'004327','1970-009A','SERT 2','SERT 2',21486,'In Earth orbit','','1970-04-10 00:00:00',105.67,1020,1031,99.12,'Sun-Synch' from dual union all
select 4328,'004328','1970-010A','Kosmos-323','Zenit-4 No. 71',21487,'Landed','1970-02-18 00:00:00','1970-02-17 00:00:00',89.5,197,302,65.43,'Intermediate' from dual union all
select 4329,'004329','1970-010B','11A57I','11A57I',21487,'Reentered','1970-02-15 00:00:00','1970-02-12 00:00:00',89.24,187,287,65.42,'Intermediate' from dual union all
select 433,'000433','62 B Delta 1','Sigma Seven','Mercury SC16',8164,'Landed','1962-10-03 00:00:00','1962-10-03 00:00:00',88.91,156,286,32.55,'Intermediate' from dual union all
select 4330,'004330','1970-011A','Ohsumi','L-4S-5',21488,'Reentered Attached','2003-08-01 00:00:00','1970-04-03 00:00:00',144.08,337,5115,31.11,'Medium' from dual union all
select 4331,'004331','1970-012A','DAPP 1524','DAPP Block 5A F-1',21489,'In Earth orbit','','1970-04-08 00:00:00',101.31,773,869,98.66,'Sun-Synch' from dual union all
select 4332,'004332','1970-012B','Burner II No. 9','Star 37B',21489,'In Earth orbit','','1970-05-02 00:00:00',101.31,771,871,98.72,'Sun-Synch' from dual union all
select 4333,'004333','1970-013B','BOZ','BOZ',21490,'Reentered','1970-03-09 00:00:00','1970-02-26 00:00:00',90.79,198,429,65.43,'Intermediate' from dual union all
select 4334,'004334','1970-013C','11S510','11S510',21490,'Reentered','1970-03-11 00:00:00','1970-02-26 00:00:00',90.76,214,410,65.44,'Intermediate' from dual union all
select 4335,'004335','1969-082EA','deb Agena D','',18349,'Reentered','1973-05-07 00:00:00','1969-11-02 00:00:00',101.43,740,913,69.81,'Intermediate' from dual union all
select 4336,'004336','1970-013A','Molniya-1','Molniya-1 11F67 No. 17',21490,'Reentered','1975-09-29 00:00:00','1970-04-06 00:00:00',717.62,474,39873,65.53,'GEO Transfer' from dual union all
select 4337,'004337','1970-013D','Blok-L','Blok-L',21490,'Reentered','1976-01-09 00:00:00','1970-03-31 00:00:00',699.11,457,38973,65.55,'GEO Transfer' from dual union all
select 4338,'004338','1970-014A','Kosmos-324','DS-P1-Yu No. 32',21491,'Reentered','1970-05-23 00:00:00','1970-04-09 00:00:00',90.97,258,387,71.04,'Intermediate' from dual union all
select 4339,'004339','1970-014B','S1M','S1M',21491,'Reentered','1970-04-11 00:00:00','1970-03-08 00:00:00',91.43,272,417,71.04,'Intermediate' from dual union all
select 434,'000434','62 B Delta 2','Atlas D 113D','Atlas D 113D',8164,'Reentered','1962-10-04 00:00:00','1962-10-03 00:00:00',88.83,157,277,32.55,'Intermediate' from dual union all
select 4340,'004340','1970-015A','Kosmos-325','Zenit-2 No. 73',21492,'Landed','1970-03-12 00:00:00','1970-03-06 00:00:00',89.75,199,326,65.4,'Intermediate' from dual union all
select 4341,'004341','1970-015B','11A57I','11A57I',21492,'Reentered','1970-03-10 00:00:00','1970-03-06 00:00:00',89.47,193,304,65.4,'Intermediate' from dual union all
select 4342,'004342','1970-016A','OPS 0440','CORONA CR-10',21493,'Reentered Attached','1970-03-26 00:00:00','1970-03-11 00:00:00',88.81,180,252,88.03,'Polar' from dual union all
select 4343,'004343','1970-016B','TIVOLI III','P-11 No. 4422',21493,'Reentered','1971-11-10 00:00:00','1970-04-14 00:00:00',93.97,435,503,88.14,'Polar' from dual union all
select 4344,'004344','1970-017A','Wika','DIAL Wissenschaftliche K',21494,'Reentered','1978-10-05 00:00:00','1970-04-11 00:00:00',104.26,313,1607,5.4,'Equatorial' from dual union all
select 4345,'004345','1970-017B','Mika','DIAL Mini Kapsel',21494,'Reentered Attached','1974-09-09 00:00:00','1970-04-24 00:00:00',104.56,310,1638,5.41,'Equatorial' from dual union all
select 4346,'004346','1970-018A','Kosmos-326','Zenit-2 No. 74',21495,'Landed','1970-03-21 00:00:00','1970-03-13 00:00:00',88.85,206,230,81.4,'Intermediate' from dual union all
select 4347,'004347','1970-018B','11A57I','11A57I',21495,'Reentered','1970-03-23 00:00:00','1970-03-13 00:00:00',90.02,211,340,81.3,'Intermediate' from dual union all
select 4348,'004348','1969-082EB','deb Agena D','',18349,'Reentered','1990-11-28 00:00:00','1990-11-28 00:00:00',89.94,257,285,70.09,'Intermediate' from dual union all
select 4349,'004349','1970-019A','Meteor','Meteor 11F614M No. 14',21496,'Reentered','1983-11-18 00:00:00','1970-04-16 00:00:00',96.33,533,632,81.19,'Intermediate' from dual union all
select 435,'000435','62 A Omega 8','deb Kosmos-9','',8148,'Reentered','1962-10-06 00:00:00','1962-10-03 00:00:00',89.62,256,256,64.99,'Intermediate' from dual union all
select 4350,'004350','1970-019B','8A92ME','8A92ME',21496,'Reentered','1982-11-26 00:00:00','1970-05-03 00:00:00',96.49,464,717,81.2,'Intermediate' from dual union all
select 4351,'004351','1970-020A','Kosmos-327','DS-P1-I No. 8',21497,'Reentered','1971-01-19 00:00:00','1970-04-20 00:00:00',95.12,263,786,70.96,'Intermediate' from dual union all
select 4352,'004352','1970-020B','S1M','S1M',21497,'Reentered','1970-09-27 00:00:00','1970-04-20 00:00:00',94.74,264,748,70.95,'Intermediate' from dual union all
select 4353,'004353','1970-021A','NATO IIA','NATO IIA',21498,'In Earth orbit','','1977-01-25 00:00:00',1436.11,35780,35794,2.95,'Stationary' from dual union all
select 4354,'004354','1970-021B','Star 37D S/N 015','Star 37D',21498,'In Earth orbit','','1970-04-21 00:00:00',654.72,296,36901,25.79,'GEO Transfer' from dual union all
select 4355,'004355','1970-022A','Kosmos-328','Zenit-4MK',21499,'Landed','1970-04-09 00:00:00','1970-04-01 00:00:00',89.53,199,303,72.89,'Intermediate' from dual union all
select 4356,'004356','1970-022B','11A57I','11A57I',21499,'Reentered','1970-04-01 00:00:00','1970-03-31 00:00:00',88.34,173,211,72.85,'Intermediate' from dual union all
select 4357,'004357','1970-023A','Kosmos-329','Zenit-2M',21500,'Landed','1970-04-15 00:00:00','1970-04-06 00:00:00',88.64,195,220,81.33,'Intermediate' from dual union all
select 4358,'004358','1970-023B','11A57I','11A57I',21500,'Reentered','1970-04-05 00:00:00','1970-04-04 00:00:00',88.45,184,212,81.32,'Intermediate' from dual union all
select 4359,'004359','1969-082EC','deb Agena D','',18349,'Reentered','1980-10-01 00:00:00','1970-04-07 00:00:00',95.16,515,537,74.05,'Intermediate' from dual union all
select 436,'000436','62 B Epsilon','FTV 1134','ARGON 9',8165,'Reentered Attached','1962-11-16 00:00:00','1962-10-17 00:00:00',90.69,216,401,81.97,'Intermediate' from dual union all
select 4360,'004360','1970-024A','Kosmos-330','Tselina-OM',21501,'Reentered','1979-06-12 00:00:00','1970-09-12 00:00:00',95.05,512,530,74.04,'Intermediate' from dual union all
select 4361,'004361','1970-024B','S3M','S3M',21501,'Reentered','1979-11-10 00:00:00','1970-06-22 00:00:00',95.02,501,539,74.05,'Intermediate' from dual union all
select 4362,'004362','1970-025A','Nimbus 4','Nimbus D',21502,'In Earth orbit','','1970-05-18 00:00:00',107.16,1091,1099,99.88,'Sun-Synch' from dual union all
select 4363,'004363','1970-025B','TOPO I','Secor Type II S/N 18',21502,'In Earth orbit','','1970-05-18 00:00:00',106.99,1085,1090,99.89,'Sun-Synch' from dual union all
select 4364,'004364','1970-026A','Kosmos-331','Zenit-4 No. 66',21503,'Landed','1970-04-16 00:00:00','1970-04-10 00:00:00',89.74,205,319,65,'Intermediate' from dual union all
select 4365,'004365','1970-026B','11A57I','11A57I',21503,'Reentered','1970-04-16 00:00:00','1970-04-10 00:00:00',89.57,204,303,65.03,'Intermediate' from dual union all
select 4366,'004366','1970-027A','Vela 6A','Vela 6A',21504,'Deep Space','1990-01-01 00:00:00','1970-05-25 00:00:00',6711.65,111307,111664,32.26,'Very High' from dual union all
select 4367,'004367','1970-025C','Agena D 6231','Agena D',21502,'Exploded','1970-11-04 00:00:00','1970-06-02 00:00:00',106.71,1062,1086,99.88,'Sun-Synch' from dual union all
select 4368,'004368','1970-027B','Vela 6B','Vela 6B',21504,'Deep Space','1990-01-01 00:00:00','1970-05-25 00:00:00',6704.6,110828,111977,32.21,'Very High' from dual union all
select 4369,'004369','1970-028A','Kosmos-332','Tsiklon',21505,'In Earth orbit','','1970-05-26 00:00:00',99.94,752,759,74.04,'Intermediate' from dual union all
select 437,'000437','62 B Zeta 1','Kosmos-10','Zenit-2 No. 5',8184,'Landed','1962-10-21 00:00:00','1962-10-17 00:00:00',90.15,196,368,65,'Intermediate' from dual union all
select 4370,'004370','1970-028B','S3M','S3M',21505,'In Earth orbit','','1970-05-12 00:00:00',99.85,740,763,74.04,'Intermediate' from dual union all
select 4371,'004371','1970-029A','Odyssey','Apollo CM-109',21506,'Deep Space','1970-04-12 00:00:00','1970-04-11 00:00:00',24717.38,219,549183,31.74,'Cislunar' from dual union all
select 4372,'004372','1970-029B','Saturn S-IVB-508','Saturn S-IVB-508',21506,'Deep Space','1970-04-12 00:00:00','1970-04-12 00:00:00',22767.09,47,519381,31.83,'Cislunar' from dual union all
select 4373,'004373','1970-030A','Kosmos-333','Zenit-4M  No. 44',21507,'Landed','1970-04-28 00:00:00','1970-04-19 00:00:00',88.94,213,232,81.35,'Intermediate' from dual union all
select 4374,'004374','1970-030B','11A57I','11A57I',21507,'Reentered','1970-04-18 00:00:00','1970-04-18 00:00:00',87.52,152,152,81.36,'Intermediate' from dual union all
select 4375,'004375','1970-031A','OPS 2863','GAMBIT-3 Mission 4326',21508,'Reentered Attached','1970-05-06 00:00:00','1970-04-22 00:00:00',89.48,128,370,110.97,'Retrograde' from dual union all
select 4376,'004376','1970-032A','INTELSAT III F-7','INTELSAT III F-7',21509,'In Earth orbit','','1970-06-01 00:00:00',1436.21,35771,35807,.16,'Stationary' from dual union all
select 4377,'004377','1970-032B','Star 37D S/N 014','Star 37D',21509,'In Earth orbit','','1971-11-18 00:00:00',629.25,240,35653,28.15,'GEO Transfer' from dual union all
select 4378,'004378','1970-033A','Kosmos-334','DS-P1-Yu No. 31',21510,'Reentered','1970-08-09 00:00:00','1970-05-26 00:00:00',91.42,259,430,70.92,'Intermediate' from dual union all
select 4379,'004379','1970-033B','S1M','S1M',21510,'Reentered','1970-05-16 00:00:00','1970-04-30 00:00:00',91.27,241,433,70.95,'Intermediate' from dual union all
select 438,'000438','62 B Zeta 2','8A92E','8A92E',8184,'Reentered','1962-11-05 00:00:00','1962-10-25 00:00:00',89.73,195,328,64.9,'Intermediate' from dual union all
select 4380,'004380','1970-035A','Kosmos-335','DS-U1-R No. 1',21512,'Reentered','1970-06-22 00:00:00','1970-05-01 00:00:00',90.9,247,391,48.41,'Intermediate' from dual union all
select 4381,'004381','1970-035B','S1M','S1M',21512,'Reentered','1970-05-17 00:00:00','1970-05-01 00:00:00',90.65,244,369,48.47,'Intermediate' from dual union all
select 4382,'004382','1970-034A','DFH-1','Dong Fang Hong',21511,'In Earth orbit','','1970-05-25 00:00:00',114.04,437,2387,68.45,'Intermediate' from dual union all
select 4383,'004383','1970-036A','Kosmos-336','Strela-1M',21513,'In Earth orbit','','1980-01-12 00:00:00',115.43,1464,1486,74.03,'Intermediate' from dual union all
select 4384,'004384','1970-036B','Kosmos-337','Strela-1M',21513,'In Earth orbit','','1970-09-15 00:00:00',116.22,1470,1552,74.04,'Intermediate' from dual union all
select 4385,'004385','1970-036C','Kosmos-338','Strela-1M',21513,'In Earth orbit','','1970-06-08 00:00:00',115.82,1468,1518,74.04,'Intermediate' from dual union all
select 4386,'004386','1970-036D','Kosmos-339','Strela-1M',21513,'In Earth orbit','','1970-06-11 00:00:00',115.05,1445,1470,74.04,'Intermediate' from dual union all
select 4387,'004387','1970-036E','Kosmos-340','Strela-1M',21513,'In Earth orbit','','1970-06-03 00:00:00',114.63,1408,1470,74.03,'Intermediate' from dual union all
select 4388,'004388','1970-036F','Kosmos-341','Strela-1M',21513,'In Earth orbit','','1970-06-08 00:00:00',113.91,1343,1469,74.04,'Intermediate' from dual union all
select 4389,'004389','1970-036G','Kosmos-342','Strela-1M',21513,'In Earth orbit','','1970-06-19 00:00:00',113.56,1311,1468,74.03,'Intermediate' from dual union all
select 439,'000439','62 B Eta 1','Ranger 5','Ranger 5',8166,'Deep Space','1962-10-21 00:00:00','1962-10-19 00:00:00',24359.33,207,543753,28.32,'Cislunar' from dual union all
select 4390,'004390','1970-036H','Kosmos-343','Strela-1M',21513,'In Earth orbit','','1970-06-08 00:00:00',114.27,1375,1470,74.04,'Intermediate' from dual union all
select 4391,'004391','1970-036J','S3M','S3M',21513,'In Earth orbit','','1970-06-27 00:00:00',116.61,1468,1589,74.03,'Intermediate' from dual union all
select 4392,'004392','1970-034B','FG-02/Balloon','FG-02',21511,'Reentered','2000-12-29 00:00:00','1970-06-01 00:00:00',113.95,440,2376,68.44,'Intermediate' from dual union all
select 4393,'004393','1970-037A','Meteor','Meteor 11F614M No. 13',21514,'Reentered','2004-03-16 00:00:00','1970-06-06 00:00:00',98.05,625,705,81.23,'Intermediate' from dual union all
select 4394,'004394','1970-037B','8A92ME','8A92ME',21514,'In Earth orbit','','1970-06-09 00:00:00',98.26,569,782,81.24,'Intermediate' from dual union all
select 4395,'004395','1970-030C','KDU','KDU',21507,'Reentered','1970-05-02 00:00:00','1970-04-28 00:00:00',88.58,193,216,81.34,'Intermediate' from dual union all
select 4396,'004396','1970-030D','PTDU cover','',21507,'Reentered','1970-05-09 00:00:00','1970-04-28 00:00:00',90.11,190,370,81.36,'Intermediate' from dual union all
select 4397,'004397','1970-033C','deb Kosmos-334','',21510,'Reentered','1970-07-08 00:00:00','1970-06-03 00:00:00',91.13,278,382,70.9,'Intermediate' from dual union all
select 4398,'004398','1970-033D','deb Kosmos-334','',21510,'Reentered','1970-05-25 00:00:00','1970-05-02 00:00:00',91.21,239,429,70.93,'Intermediate' from dual union all
select 4399,'004399','1970-033E','deb Kosmos-334','',21510,'Reentered','1970-05-16 00:00:00','1970-05-04 00:00:00',91.05,245,408,70.91,'Intermediate' from dual union all
select 44,'000044','60 Zeta 2','Midas 2 Fairing','Midas 2 Fairing',6335,'Reentered','1960-12-05 00:00:00','1960-09-16 00:00:00',93.66,454,454,33,'Intermediate' from dual union all
select 440,'000440','62 B Eta 2','Agena B 6005','Agena B 6005',8166,'Deep Space','1962-10-21 00:00:00','1962-10-18 00:00:00',25318.91,217,558269,28.32,'Cislunar' from dual union all
select 4400,'004400','1970-034C','DFH-1 part','',21511,'Reentered','1983-03-23 00:00:00','1970-06-03 00:00:00',113.47,432,2340,68.52,'Intermediate' from dual union all
select 4401,'004401','1970-038A','Kosmos-344','Zenit-2 No. 81',21515,'Landed','1970-05-20 00:00:00','1970-05-17 00:00:00',89.69,201,318,72.9,'Intermediate' from dual union all
select 4402,'004402','1970-038B','11A57I','11A57I',21515,'Reentered','1970-05-20 00:00:00','1970-05-14 00:00:00',89.38,203,285,72.9,'Intermediate' from dual union all
select 4403,'004403','1970-039A','Kosmos-345','Zenit-4 No. 77',21516,'Landed','1970-05-28 00:00:00','1970-05-23 00:00:00',89.04,191,264,51.75,'Intermediate' from dual union all
select 4404,'004404','1970-039B','11A57I','11A57I',21516,'Reentered','1970-05-23 00:00:00','1970-05-22 00:00:00',88.09,159,201,51.68,'Intermediate' from dual union all
select 4405,'004405','1970-040A','OPS 4720','CORONA CR-11',21517,'Reentered Attached','1970-06-17 00:00:00','1970-05-31 00:00:00',88.52,171,232,83,'Intermediate' from dual;
insert into SATELLITE
select 4406,'004406','1970-040B','TRIPOS IV','P-11 No. 4421',21517,'Reentered','1974-03-08 00:00:00','1970-06-29 00:00:00',94.45,488,495,83.12,'Intermediate' from dual union all
select 4407,'004407','1970-041A','Soyuz-9','Soyuz No. 17',21518,'Landed','1970-06-19 00:00:00','1970-06-08 00:00:00',89.36,236,249,51.67,'Intermediate' from dual union all
select 4408,'004408','1970-041B','11S510','11S510',21518,'Reentered','1970-06-03 00:00:00','1970-06-02 00:00:00',88.51,199,203,51.67,'Intermediate' from dual union all
select 4409,'004409','1970-042A','Kosmos-346','Zenit-4 No. 72',21519,'Landed','1970-06-17 00:00:00','1970-06-14 00:00:00',89.2,196,274,51.73,'Intermediate' from dual union all
select 441,'000441','62 B Theta 1','Kosmos-11','DS-A1 No. 1',8181,'Reentered','1964-05-18 00:00:00','1962-11-27 00:00:00',95.78,249,863,48.98,'Intermediate' from dual union all
select 4410,'004410','1970-042B','11A57I','11A57I',21519,'Reentered','1970-06-13 00:00:00','1970-06-11 00:00:00',88.88,186,253,51.75,'Intermediate' from dual union all
select 4411,'004411','1970-043A','Kosmos-347','DS-P1-Yu No. 35',21520,'Reentered','1971-11-07 00:00:00','1970-07-13 00:00:00',107.1,214,1971,48.43,'Intermediate' from dual union all
select 4412,'004412','1970-043B','S1M','S1M',21520,'Reentered','1971-02-14 00:00:00','1970-07-17 00:00:00',106.26,216,1891,48.42,'Intermediate' from dual union all
select 4413,'004413','1970-044A','Kosmos-348','DS-U2-GK No. 2',21521,'Reentered','1970-07-25 00:00:00','1970-06-23 00:00:00',92.43,199,589,71,'Intermediate' from dual union all
select 4414,'004414','1970-044B','S1M','S1M',21521,'Reentered','1970-07-09 00:00:00','1970-06-20 00:00:00',92.2,197,568,70.98,'Intermediate' from dual union all
select 4415,'004415','1970-044C','deb Kosmos-348','',21521,'Reentered','1970-06-17 00:00:00','1970-06-15 00:00:00',90.32,184,397,70.98,'Intermediate' from dual union all
select 4416,'004416','1970-045A','Kosmos-349','Zenit-4 No. 73',21522,'Landed','1970-06-25 00:00:00','1970-06-20 00:00:00',89.74,196,328,65.4,'Intermediate' from dual union all
select 4417,'004417','1970-045B','11A57I','11A57I',21522,'Reentered','1970-06-22 00:00:00','1970-06-19 00:00:00',89.33,183,300,65.39,'Intermediate' from dual union all
select 4418,'004418','1970-046A','OPS 5346','RHYOLITE 1',21523,'In Earth orbit','','1970-07-01 00:00:00',1435.5,35700,35850,0,'Stationary' from dual union all
select 4419,'004419','1970-047A','Meteor','Meteor-M No. 17',21524,'In Earth orbit','','1970-07-23 00:00:00',102.08,829,886,81.22,'Intermediate' from dual union all
select 442,'000442','62 B Theta 2','S1','S1',8181,'Reentered','1963-06-06 00:00:00','1962-11-23 00:00:00',95.35,246,825,48.98,'Intermediate' from dual union all
select 4420,'004420','1970-047B','8A92ME','8A92ME',21524,'In Earth orbit','','1970-07-27 00:00:00',102.26,807,924,81.23,'Intermediate' from dual union all
select 4421,'004421','1968-097CF','deb Kosmos-252 [*]','',16824,'In Earth orbit','','1992-02-04 00:00:00',109.86,527,1914,62.78,'Intermediate' from dual union all
select 4422,'004422','1970-048A','OPS 6820','GAMBIT-3 Mission 4327',21525,'Reentered Attached','1970-07-06 00:00:00','1970-06-28 00:00:00',89.78,128,400,108.88,'Retrograde' from dual union all
select 4423,'004423','1970-049C','11S510','11S510',21526,'Reentered','1970-07-17 00:00:00','1970-07-03 00:00:00',90.79,207,420,65.39,'Intermediate' from dual union all
select 4424,'004424','1970-049B','BOZ','BOZ',21526,'Reentered','1970-07-25 00:00:00','1970-07-03 00:00:00',91.04,219,433,65.39,'Intermediate' from dual union all
select 4425,'004425','1970-050A','Kosmos-350','Zenit-2M',21527,'Landed','1970-07-08 00:00:00','1970-06-28 00:00:00',89.04,202,253,51.75,'Intermediate' from dual union all
select 4426,'004426','1970-050B','11A57I','11A57I',21527,'Reentered','1970-06-29 00:00:00','1970-06-27 00:00:00',88.53,170,234,51.74,'Intermediate' from dual union all
select 4427,'004427','1970-051A','Kosmos-351','DS-P1-Yu No. 38',21528,'Reentered','1970-10-13 00:00:00','1970-07-28 00:00:00',91.43,261,429,70.98,'Intermediate' from dual union all
select 4428,'004428','1970-051B','S1M','S1M',21528,'Reentered','1970-08-18 00:00:00','1970-07-05 00:00:00',91.47,264,429,70.98,'Intermediate' from dual union all
select 4429,'004429','1965-108F','deb Transtage 8','',11858,'Reentered','1975-11-03 00:00:00','1966-01-31 00:00:00',588.19,193,33560,26.39,'Highly Ellip' from dual union all
select 443,'000443','62 B Iota 1','[AMS Mars]','2MV-4 No. 1',8168,'Reentered','1962-10-29 00:00:00','1962-10-24 00:00:00',89.12,202,260,65.11,'Intermediate' from dual union all
select 4430,'004430','1970-049A','Molniya-1','Molniya-1 11F67 No. 21',21526,'Reentered','1976-02-16 00:00:00','1970-07-27 00:00:00',717.58,473,39872,65.32,'GEO Transfer' from dual union all
select 4431,'004431','1970-049D','Blok-L','Blok-L',21526,'Reentered','1976-03-02 00:00:00','1970-07-27 00:00:00',700.3,454,39035,65.37,'GEO Transfer' from dual union all
select 4432,'004432','1969-082ED','deb Agena D','',18349,'In Earth orbit','','1992-02-04 00:00:00',101.5,784,876,69.99,'Intermediate' from dual union all
select 4433,'004433','1969-082EE','deb Agena D','',18349,'Reentered','1979-04-19 00:00:00','1979-01-01 00:00:00',97.55,634,648,70.23,'Intermediate' from dual union all
select 4434,'004434','1969-082EF','deb Agena D','',18349,'Reentered','1980-01-05 00:00:00','1970-08-09 00:00:00',101.78,781,905,70.72,'Intermediate' from dual union all
select 4435,'004435','1969-082EG','deb Agena D','',18349,'In Earth orbit','','1992-02-04 00:00:00',100.84,776,820,70.04,'Intermediate' from dual union all
select 4436,'004436','1969-082EH','deb Agena D','',18349,'Reentered','1990-11-20 00:00:00','1990-11-20 00:00:00',89.22,227,245,70.01,'Intermediate' from dual union all
select 4437,'004437','1969-082EJ','deb Agena D','',18349,'Reentered','1981-06-22 00:00:00','1981-05-02 00:00:00',93.73,435,479,70.17,'Intermediate' from dual union all
select 4438,'004438','1969-082EK','deb Agena D','',18349,'In Earth orbit','','1992-02-04 00:00:00',100.36,719,832,70.11,'Intermediate' from dual union all
select 4439,'004439','1969-082EL','deb Agena D','',18349,'Reentered','1972-05-09 00:00:00','1972-05-05 00:00:00',91.92,364,373,69.87,'Intermediate' from dual union all
select 444,'000444','62 B Kappa','Starad','CORONA R&D FTV 1401',8169,'Reentered Attached','1967-10-05 00:00:00','1962-11-26 00:00:00',146.95,199,5494,71.41,'Medium' from dual union all
select 4440,'004440','1969-082EM','deb Agena D','',18349,'Reentered','1988-10-13 00:00:00','1988-10-13 00:00:00',90.33,267,314,69.98,'Intermediate' from dual union all
select 4441,'004441','1969-082EN','deb Agena D','',18349,'Reentered','1992-02-02 00:00:00','1992-02-01 00:00:00',90.05,267,287,70.11,'Intermediate' from dual union all
select 4442,'004442','1969-082EP','deb Agena D','',18349,'Reentered','2014-03-04 00:00:00','1992-02-05 00:00:00',101.97,812,892,70,'Intermediate' from dual union all
select 4443,'004443','1969-082EQ','deb Agena D','',18349,'Reentered','1970-07-28 00:00:00','1970-07-27 00:00:00',91.15,311,351,70.39,'Intermediate' from dual union all
select 4444,'004444','1969-082ER','deb Agena D','',18349,'Reentered','2001-12-08 00:00:00','1992-02-05 00:00:00',99.08,688,742,69.93,'Intermediate' from dual union all
select 4445,'004445','1969-082ES','deb Agena D','',18349,'Reentered','1997-04-24 00:00:00','1992-02-06 00:00:00',98.33,637,721,70.22,'Intermediate' from dual union all
select 4446,'004446','1970-052A','Kosmos-352','Zenit-4 No. 74',21529,'Landed','1970-07-15 00:00:00','1970-07-12 00:00:00',89.41,200,291,51.76,'Intermediate' from dual union all
select 4447,'004447','1970-052B','11A57I','11A57I',21529,'Reentered','1970-07-12 00:00:00','1970-07-09 00:00:00',89.12,177,285,51.75,'Intermediate' from dual union all
select 4448,'004448','1969-082ET','deb Agena D','',18349,'Reentered','1980-09-16 00:00:00','1978-06-22 00:00:00',99.02,710,714,70,'Intermediate' from dual union all
select 4449,'004449','1969-082EU','deb Agena D','',18349,'In Earth orbit','','1992-02-06 00:00:00',101.43,768,885,69.98,'Intermediate' from dual union all
select 445,'000445','62 B Lambda 1','Explorer 15','EPE C',8170,'Reentered','1978-01-15 00:00:00','1962-12-09 00:00:00',315.24,312,17619,17.97,'Highly Ellip' from dual union all
select 4450,'004450','1969-082EV','deb Agena D','',18349,'Reentered','1981-12-18 00:00:00','1981-12-16 00:00:00',91.46,337,356,69.94,'Intermediate' from dual union all
select 4451,'004451','1969-082EW','deb Agena D','',18349,'In Earth orbit','','1992-02-03 00:00:00',100.96,772,837,70.05,'Intermediate' from dual union all
select 4452,'004452','1969-082EX','deb Agena D','',18349,'Reentered','1985-04-25 00:00:00','1970-07-06 00:00:00',102.81,870,913,70.15,'Intermediate' from dual union all
select 4453,'004453','1969-082EY','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',102.4,845,900,70.13,'Intermediate' from dual union all
select 4454,'004454','1969-082EZ','deb Agena D','',18349,'Reentered','1981-10-25 00:00:00','1981-10-24 00:00:00',91.46,330,362,70.18,'Intermediate' from dual union all
select 4455,'004455','1970-053A','Kosmos-353','Zenit-2M',21530,'Landed','1970-07-21 00:00:00','1970-07-11 00:00:00',89.38,204,284,65.42,'Intermediate' from dual union all
select 4456,'004456','1970-053B','11A57I','11A57I',21530,'Reentered','1970-07-16 00:00:00','1970-07-11 00:00:00',88.86,183,253,65.41,'Intermediate' from dual union all
select 4457,'004457','1969-082FA','deb Agena D','',18349,'Reentered','1970-10-19 00:00:00','1970-10-14 00:00:00',92.85,407,422,69.74,'Intermediate' from dual union all
select 4458,'004458','1969-082FB','deb Agena D','',18349,'Reentered','2014-09-07 00:00:00','1992-02-06 00:00:00',99.5,699,771,69.76,'Intermediate' from dual union all
select 4459,'004459','1969-082FC','deb Agena D','',18349,'Reentered','1980-02-19 00:00:00','1981-04-12 00:00:00',94.49,474,514,70.03,'Intermediate' from dual union all
select 446,'000446','62 B Mu 1','ANNA 1B','ANNA 1B',8171,'In Earth orbit','','1962-12-02 00:00:00',107.9,1081,1179,50.15,'Intermediate' from dual union all
select 4460,'004460','1969-082FD','deb Agena D','',18349,'Reentered','1984-03-08 00:00:00','1984-03-06 00:00:00',91.12,321,338,69.85,'Intermediate' from dual union all
select 4461,'004461','1969-082FE','deb Agena D','',18349,'Reentered','1974-06-13 00:00:00','1970-07-19 00:00:00',100.35,689,862,69.75,'Intermediate' from dual union all
select 4462,'004462','1969-082FF','deb Agena D','',18349,'Reentered','1982-03-25 00:00:00','1970-07-18 00:00:00',101.97,793,911,69.99,'Intermediate' from dual union all
select 4463,'004463','1969-082FG','deb Agena D','',18349,'Reentered','1978-07-08 00:00:00','1978-06-19 00:00:00',93.91,462,470,69.79,'Intermediate' from dual union all
select 4464,'004464','1969-082FH','deb Agena D','',18349,'Reentered','1981-04-02 00:00:00','1970-09-11 00:00:00',102.07,700,1014,69.96,'Intermediate' from dual union all
select 4465,'004465','1969-082FJ','deb Agena D','',18349,'Reentered','1989-09-18 00:00:00','1989-09-18 00:00:00',90,272,277,70,'Intermediate' from dual union all
select 4466,'004466','1969-082FK','deb Agena D','',18349,'Reentered','1990-07-13 00:00:00','1970-07-18 00:00:00',103.25,829,996,70.53,'Intermediate' from dual union all
select 4467,'004467','1969-082FL','deb Agena D','',18349,'Reentered','1990-03-10 00:00:00','1990-03-08 00:00:00',91.06,316,338,70.25,'Intermediate' from dual union all
select 4468,'004468','1969-082FM','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',104.29,893,1030,70.07,'Intermediate' from dual union all
select 4469,'004469','1969-082FN','deb Agena D','',18349,'Reentered','1980-11-30 00:00:00','1980-11-24 00:00:00',92.84,402,426,70.06,'Intermediate' from dual union all
select 447,'000447','62 B Mu 2','Ablestar 012','AJ10-104 012',8171,'In Earth orbit','','1962-12-08 00:00:00',107.6,1068,1164,50.13,'Intermediate' from dual union all
select 4470,'004470','1969-082FP','deb Agena D','',18349,'Reentered','1981-09-15 00:00:00','1981-03-04 00:00:00',98.2,656,690,70.05,'Intermediate' from dual union all
select 4471,'004471','1969-082FQ','deb Agena D','',18349,'Reentered','1983-10-17 00:00:00','1983-10-15 00:00:00',90.43,288,303,69.93,'Intermediate' from dual union all
select 4472,'004472','1969-082FR','deb Agena D','',18349,'Reentered','1980-07-21 00:00:00','1970-07-18 00:00:00',101.21,724,908,69.99,'Intermediate' from dual union all
select 4473,'004473','1969-082FS','deb Agena D','',18349,'Reentered','1981-02-18 00:00:00','1981-02-16 00:00:00',90.31,281,298,69.6,'Intermediate' from dual union all
select 4474,'004474','1969-082FT','deb Agena D','',18349,'Reentered','1979-11-14 00:00:00','1979-10-16 00:00:00',95.46,499,582,70.11,'Intermediate' from dual union all
select 4475,'004475','1969-082FU','deb Agena D','',18349,'Reentered','2002-05-12 00:00:00','1992-02-06 00:00:00',101.62,779,892,69.96,'Intermediate' from dual union all
select 4476,'004476','1965-108G','LES 4','',11858,'Reentered','2012-11-11 00:00:00','1968-01-11 00:00:00',503.27,261,28900,26.39,'Highly Ellip' from dual union all
select 4477,'004477','1970-054A','OPS 4324','CORONA CR-12',21531,'Reentered Attached','1970-08-19 00:00:00','1970-07-30 00:00:00',89.98,156,391,59.99,'Intermediate' from dual union all
select 4478,'004478','1970-055A','INTELSAT III F-8','INTELSAT III F-8',21532,'In Earth orbit','','1977-01-28 00:00:00',1408.11,33867,36607,7.42,'Inclined Drift' from dual union all
select 4479,'004479','1970-056C','Kosmos-354 Adapter','OGCh Adapter',21533,'Reentered','1970-07-29 00:00:00','1970-07-28 00:00:00',87.6,134,177,49.62,'Intermediate' from dual union all
select 448,'000448','62 B Nu 1','8K78I','8K78I',8172,'Reentered','1962-11-02 00:00:00','1962-11-02 00:00:00',88.44,157,238,65.09,'Intermediate' from dual union all
select 4480,'004480','1970-056B','8S692','8S692',21533,'Reentered','1970-07-29 00:00:00','1970-07-29 00:00:00',87.18,114,156,49.57,'Intermediate' from dual union all
select 4481,'004481','1970-056A','Kosmos-354','OGCh 8F673 No. 22500-11',21533,'Deorbited','1970-07-28 00:00:00','1970-07-28 00:00:00',87.5,137,164,49.6,'Intermediate' from dual union all
select 4482,'004482','1970-057A','Interkosmos-3','DS-U2-IK No. 1',21534,'Reentered','1970-12-06 00:00:00','1970-09-07 00:00:00',98.3,201,1153,48.41,'Intermediate' from dual union all
select 4483,'004483','1970-057B','S1M','S1M',21534,'Reentered','1970-11-17 00:00:00','1970-09-20 00:00:00',97.27,198,1058,48.38,'Intermediate' from dual union all
select 4484,'004484','1970-058A','Kosmos-355','Zenit-4 No. 76',21535,'Landed','1970-08-15 00:00:00','1970-08-09 00:00:00',89.68,198,320,65.4,'Intermediate' from dual union all
select 4485,'004485','1970-058B','11A57I','11A57I',21535,'Reentered','1970-08-14 00:00:00','1970-08-09 00:00:00',89.42,190,301,65.4,'Intermediate' from dual union all
select 4486,'004486','1970-055B','Star 37D S/N 019','Star 37D',21532,'In Earth orbit','','1970-08-27 00:00:00',642.26,273,36288,27.94,'GEO Transfer' from dual union all
select 4487,'004487','1970-059A','Kosmos-356','DS-U2-MG No. 2',21536,'Reentered','1970-10-02 00:00:00','1970-08-18 00:00:00',92.29,226,548,81.94,'Intermediate' from dual union all
select 4488,'004488','1970-059B','S1M','S1M',21536,'Reentered','1970-10-01 00:00:00','1970-08-17 00:00:00',92.2,236,529,81.95,'Intermediate' from dual union all
select 4489,'004489','1970-060A','Venera-7','4V-1 No. 630',21537,'Deep Space','1970-08-18 00:00:00','',null,null,null,null,'' from dual union all
select 449,'000449','62 B Nu 2','BOZ','BOZ',8172,'Reentered','1962-11-03 00:00:00','1962-11-02 00:00:00',88.07,157,201,65.09,'Intermediate' from dual union all
select 4490,'004490','1970-060B','11S510','11S510',21537,'Reentered','1970-08-18 00:00:00','1970-08-17 00:00:00',88.33,182,201,51.76,'Intermediate' from dual union all
select 4491,'004491','1970-060C','BOZ','BOZ',21537,'Reentered','1970-08-18 00:00:00','1970-08-17 00:00:00',88.56,175,231,51.66,'Intermediate' from dual union all
select 4492,'004492','1970-061A','OPS 7874','GAMBIT-3 Mission 4328',21538,'Reentered Attached','1970-09-04 00:00:00','1970-08-25 00:00:00',90.44,151,442,110.98,'Retrograde' from dual union all
select 4493,'004493','1970-062A','Skynet IB','Skynet IB',21539,'Lost','','1970-08-22 00:00:00',778.3,7300,36000,12,'Highly Ellip' from dual union all
select 4494,'004494','62 Kappa 5','West Ford Drag dipole','West Ford Drag Experimen',8192,'Reentered','1977-05-22 00:00:00','1962-06-07 00:00:00',152.59,2784,3378,86.65,'Medium' from dual union all
select 4495,'004495','1970-063A','Kosmos-357','DS-P1-Yu No. 40',21540,'Reentered','1970-11-24 00:00:00','1970-09-20 00:00:00',91.49,263,433,70.99,'Intermediate' from dual union all
select 4496,'004496','1970-063B','S1M','S1M',21540,'Reentered','1970-10-15 00:00:00','1970-08-28 00:00:00',91.58,272,433,70.99,'Intermediate' from dual;
insert into SATELLITE
select 4497,'004497','1970-064A','Kosmos-358','Lira',21541,'Reentered','1990-06-26 00:00:00','1970-09-22 00:00:00',95.14,514,536,74.03,'Intermediate' from dual union all
select 4498,'004498','1970-064B','S3M','S3M',21541,'Reentered','1979-07-07 00:00:00','1970-09-19 00:00:00',94.98,494,542,74.05,'Intermediate' from dual union all
select 4499,'004499','1968-069G','deb ESSA 7','',16796,'In Earth orbit','','1970-08-03 00:00:00',115.12,1439,1483,101.94,'Sun-Synch' from dual union all
select 45,'000045','60 Eta 1','Transit 2A','Transit 2A',6321,'In Earth orbit','','1960-08-10 00:00:00',101.63,626,1046,66.77,'Intermediate' from dual union all
select 450,'000450','62 B Nu 3','AMS Mars','2MV-4 No. 2',8172,'Deep Space','1962-11-02 00:00:00','',null,null,null,null,'' from dual union all
select 4500,'004500','1968-091BZ','deb Kosmos-249','',16818,'Reentered','1970-12-12 00:00:00','1970-10-27 00:00:00',107.53,510,1714,62.46,'Intermediate' from dual union all
select 4501,'004501','1970-065A','Kosmos-359','4V-1 No. 631',21542,'Reentered','1970-11-06 00:00:00','1970-09-26 00:00:00',93.46,203,685,51.12,'Intermediate' from dual union all
select 4502,'004502','1970-065B','11S510','11S510',21542,'Reentered','1970-09-11 00:00:00','1970-08-24 00:00:00',89.38,204,284,51.81,'Intermediate' from dual union all
select 4503,'004503','1970-066A','OPS 8329','STRAWMAN 3',21543,'Reentered Attached','1975-03-26 00:00:00','1970-10-06 00:00:00',94.43,482,500,74.98,'Intermediate' from dual union all
select 4504,'004504','1970-065C','Blok-NVL No. 61','Blok-L',21542,'Reentered','1970-09-11 00:00:00','1970-09-03 00:00:00',93.28,141,730,51.23,'Intermediate' from dual union all
select 4505,'004505','1970-065D','Kosmos-359 SA','Spuskaemiy apparat',21542,'Reentered','1971-10-06 00:00:00','1970-09-28 00:00:00',95.05,206,835,51.18,'Intermediate' from dual union all
select 4506,'004506','1970-065E','Blok-L part','',21542,'Reentered','1970-09-09 00:00:00','1970-08-31 00:00:00',93.23,213,652,51.19,'Intermediate' from dual union all
select 4507,'004507','1970-067A','NNS O-19','NNS O-19',21544,'In Earth orbit','','1970-09-28 00:00:00',106.98,952,1222,90.03,'Polar' from dual union all
select 4508,'004508','1970-068A','Kosmos-360','Zenit-4M',21545,'Landed','1970-09-08 00:00:00','1970-08-31 00:00:00',89.63,203,310,65,'Intermediate' from dual union all
select 4509,'004509','1970-068B','11A57I','11A57I',21545,'Reentered','1970-09-02 00:00:00','1970-09-01 00:00:00',88.36,175,212,65,'Intermediate' from dual union all
select 451,'000451','62 B Xi 1','[AMS Mars]','2MV-3 No. 1',8183,'Reentered Attached','1962-11-05 00:00:00','1962-11-05 00:00:00',87.88,170,170,64.85,'Intermediate' from dual union all
select 4510,'004510','1970-069A','OPS 7329','CANYON 3',21546,'In Earth orbit','','1970-09-01 00:00:00',1441.94,31947,39855,10.3,'Near-synch' from dual union all
select 4511,'004511','1970-046B','Agena D 1551','Agena D',21523,'In Earth orbit','','1973-08-20 00:00:00',472.52,240,27196,28.3,'Highly Ellip' from dual union all
select 4512,'004512','1970-070A','DAPP 2525','DAPP Block 5A F-2',21547,'In Earth orbit','','1970-10-03 00:00:00',101.23,763,871,98.71,'Sun-Synch' from dual union all
select 4513,'004513','1970-070B','Burner II No. 13','Star 37B',21547,'In Earth orbit','','1970-10-09 00:00:00',101.22,762,871,98.75,'Sun-Synch' from dual union all
select 4514,'004514','1970-064C','deb Kosmos-358','DS-P1-M part',21541,'Reentered','1970-10-29 00:00:00','1970-09-26 00:00:00',94.32,480,492,74.03,'Intermediate' from dual union all
select 4515,'004515','1970-067B','Altair 2 X-258E5 ABL-149','X-258E5',21544,'In Earth orbit','','1970-10-06 00:00:00',106.97,951,1222,90.02,'Polar' from dual union all
select 4516,'004516','1970-068C','Blenda','BZA',21545,'Reentered','1970-09-09 00:00:00','1970-09-07 00:00:00',88.59,191,219,64.91,'Intermediate' from dual union all
select 4517,'004517','1970-068D','KDU','KDU',21545,'Reentered','1970-09-11 00:00:00','1970-09-07 00:00:00',88.82,195,238,65,'Intermediate' from dual union all
select 4518,'004518','1969-082FV','deb Agena D','',18349,'Reentered','1993-06-05 00:00:00','1992-02-06 00:00:00',95.71,521,585,69.8,'Intermediate' from dual union all
select 4519,'004519','1969-082FW','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',99.75,705,789,70.09,'Intermediate' from dual union all
select 452,'000452','62 B Xi 2','8K78I','8K78I',8183,'Reentered','1962-11-08 00:00:00','1962-11-05 00:00:00',88.84,185,250,64.85,'Intermediate' from dual union all
select 4520,'004520','1969-082FX','deb Agena D','',18349,'Reentered','1977-12-20 00:00:00','1977-11-11 00:00:00',95.1,508,539,70.3,'Intermediate' from dual union all
select 4521,'004521','1969-082FY','deb Agena D','',18349,'Reentered','1980-10-18 00:00:00','1980-10-07 00:00:00',93.05,403,444,70.04,'Intermediate' from dual union all
select 4522,'004522','1969-082FZ','deb Agena D','',18349,'Reentered','1978-05-08 00:00:00','1978-04-02 00:00:00',95.96,564,566,70,'Intermediate' from dual union all
select 4523,'004523','1969-082GA','deb Agena D','',18349,'Reentered','1993-08-26 00:00:00','1992-02-06 00:00:00',96.86,596,621,70.05,'Intermediate' from dual union all
select 4524,'004524','1970-071A','Kosmos-361','Zenit-4M No. 46',21548,'Landed','1970-09-21 00:00:00','1970-09-11 00:00:00',89.49,203,296,72.87,'Intermediate' from dual union all
select 4525,'004525','1970-071B','11A57I','11A57I',21548,'Reentered','1970-09-14 00:00:00','1970-09-10 00:00:00',89.18,191,276,72.86,'Intermediate' from dual union all
select 4526,'004526','1970-068E','PTDU cover','',21545,'Reentered','1970-09-11 00:00:00','1970-09-09 00:00:00',89.44,207,287,65,'Intermediate' from dual union all
select 4527,'004527','1970-072A','Luna-16 KT','E-8-5 No. 406',21549,'Deep Space','1970-09-13 00:00:00','1970-09-12 00:00:00',15562.18,200,400000,51.5,'Cislunar' from dual union all
select 4528,'004528','1970-072B','SOZ','SOZ',21549,'Reentered','1970-09-15 00:00:00','1970-09-13 00:00:00',88.54,185,220,51.54,'Intermediate' from dual union all
select 4529,'004529','1970-072C','SOZ','SOZ',21549,'Reentered','1970-09-15 00:00:00','1970-09-13 00:00:00',88.51,179,223,51.49,'Intermediate' from dual union all
select 453,'000453','62 B Omicron','FTV 1136','CORONA M-14',8174,'Reentered Attached','1962-12-03 00:00:00','1962-11-15 00:00:00',90.36,194,391,75.02,'Intermediate' from dual union all
select 4530,'004530','1969-082GB','deb Agena D','',18349,'Reentered','2014-04-18 00:00:00','1992-02-05 00:00:00',99.55,690,785,70.24,'Intermediate' from dual union all
select 4531,'004531','1969-082GC','deb Agena D','',18349,'Reentered','1990-07-31 00:00:00','1990-07-30 00:00:00',90.3,277,302,70.39,'Intermediate' from dual union all
select 4532,'004532','1969-082GD','deb Agena D','',18349,'Reentered','1979-03-04 00:00:00','1979-01-30 00:00:00',94.46,470,515,69.74,'Intermediate' from dual union all
select 4533,'004533','1969-082GE','deb Agena D','',18349,'Reentered','1981-05-01 00:00:00','1970-09-15 00:00:00',101.73,802,879,70.1,'Intermediate' from dual union all
select 4534,'004534','1969-082GF','deb Agena D','',18349,'Reentered','1983-04-10 00:00:00','1970-09-15 00:00:00',102.76,839,940,70.45,'Intermediate' from dual union all
select 4535,'004535','1969-082GG','deb Agena D','',18349,'Reentered','1978-06-13 00:00:00','1978-04-10 00:00:00',96.59,578,613,70.2,'Intermediate' from dual union all
select 4536,'004536','1970-073A','Kosmos-362','DS-P1-I No. 9',21550,'Reentered','1971-10-13 00:00:00','1970-10-17 00:00:00',95.27,265,799,70.97,'Intermediate' from dual union all
select 4537,'004537','1970-073B','S1M','S1M',21550,'Reentered','1971-03-28 00:00:00','1970-10-29 00:00:00',94.67,261,745,70.98,'Intermediate' from dual union all
select 4538,'004538','1970-074A','Kosmos-363','Zenit-2M',21551,'Landed','1970-09-29 00:00:00','1970-09-20 00:00:00',89.45,203,291,65,'Intermediate' from dual union all
select 4539,'004539','1970-074B','11A57I','11A57I',21551,'Reentered','1970-09-22 00:00:00','1970-09-19 00:00:00',89,196,254,65,'Intermediate' from dual union all
select 454,'000454','62 B Xi 3','BOZ','BOZ',8183,'Reentered','1963-01-19 00:00:00','1962-12-09 00:00:00',91.14,204,456,64.72,'Intermediate' from dual union all
select 4540,'004540','1969-082GH','deb Agena D','',18349,'Reentered','1972-02-03 00:00:00','1972-01-27 00:00:00',93.77,441,477,69.75,'Intermediate' from dual union all
select 4541,'004541','1969-082GJ','deb Agena D','',18349,'Reentered','1979-12-20 00:00:00','1979-12-19 00:00:00',92.43,365,423,70.14,'Intermediate' from dual union all
select 4542,'004542','1969-082GK','deb Agena D','',18349,'Reentered','1970-12-28 00:00:00','1970-12-27 00:00:00',90.11,280,280,69.8,'Intermediate' from dual union all
select 4543,'004543','1969-082GL','deb Agena D','',18349,'Reentered','1990-08-03 00:00:00','1990-08-01 00:00:00',90.65,290,323,70.34,'Intermediate' from dual union all
select 4544,'004544','1969-082GM','deb Agena D','',18349,'Reentered','1980-07-22 00:00:00','1980-07-18 00:00:00',91.2,316,351,69.6,'Intermediate' from dual union all
select 4545,'004545','1970-071C','KDU','KDU',21548,'Reentered','1970-10-05 00:00:00','1970-09-21 00:00:00',89.82,198,334,72.87,'Intermediate' from dual union all
select 4546,'004546','1970-071D','Blenda','BZA',21548,'Reentered','1970-09-26 00:00:00','1970-09-21 00:00:00',89.8,200,330,72.87,'Intermediate' from dual union all
select 4547,'004547','1969-082GN','deb Agena D','',18349,'In Earth orbit','','1992-02-05 00:00:00',101.78,815,870,69.98,'Intermediate' from dual union all
select 4548,'004548','1969-082GP','deb Agena D','',18349,'Reentered','1982-02-14 00:00:00','1982-02-11 00:00:00',91.63,344,364,70.38,'Intermediate' from dual union all
select 4549,'004549','1969-082GQ','deb Agena D','',18349,'Reentered','1979-12-12 00:00:00','1981-05-28 00:00:00',95.16,494,559,69.96,'Intermediate' from dual union all
select 455,'000455','62 B Pi','FTV 2405','SAMOS E-6 No. 5',8176,'Reentered Attached','1962-11-12 00:00:00','1962-11-11 00:00:00',89,216,235,96.11,'Sun-Synch' from dual union all
select 4550,'004550','1969-082GR','deb Agena D','',18349,'Reentered','1978-01-05 00:00:00','1970-09-19 00:00:00',99.25,595,851,69.42,'Intermediate' from dual union all
select 4551,'004551','1969-082GS','deb Agena D','',18349,'Reentered','1981-10-26 00:00:00','1981-03-11 00:00:00',99.63,723,759,70.11,'Intermediate' from dual union all
select 4552,'004552','1969-082GT','deb Agena D','',18349,'Reentered','2014-12-28 00:00:00','1992-02-02 00:00:00',99.98,724,791,70.04,'Intermediate' from dual union all
select 4553,'004553','1970-075A','Kosmos-364','Zenit-4MK',21552,'Landed','1970-10-02 00:00:00','1970-09-24 00:00:00',89.48,199,299,65.4,'Intermediate' from dual union all
select 4554,'004554','1970-075B','11A57I','11A57I',21552,'Reentered','1970-09-28 00:00:00','1970-09-24 00:00:00',89.23,188,285,65.39,'Intermediate' from dual union all
select 4555,'004555','1966-041F','NNS O-9 part','',13448,'In Earth orbit','','1970-09-23 00:00:00',103.32,853,979,89.85,'Polar' from dual union all
select 4556,'004556','1970-076A','Kosmos-365','OGCh 8F673K No. Kh22511-',21553,'Deorbited','1970-09-25 00:00:00','1970-09-25 00:00:00',87.64,143,173,49.5,'Intermediate' from dual union all
select 4557,'004557','1970-076B','Kosmos-365 Adapter','OGCh Adapter',21553,'Reentered','1970-09-26 00:00:00','1970-09-25 00:00:00',87.55,133,174,49.66,'Intermediate' from dual union all
select 4558,'004558','1970-076C','8S692','8S692',21553,'Reentered','1970-09-25 00:00:00','1970-09-25 00:00:00',87.26,117,161,49.66,'Intermediate' from dual union all
select 4559,'004559','1970-077B','BOZ','BOZ',21554,'Reentered','1970-10-23 00:00:00','1970-10-06 00:00:00',91.1,216,441,65.38,'Intermediate' from dual union all
select 456,'000456','62 B Iota 2','8K78I','8K78I',8168,'Reentered','1962-12-22 00:00:00','1962-11-19 00:00:00',91.6,182,524,64.82,'Intermediate' from dual union all
select 4560,'004560','1970-077C','11S510','11S510',21554,'Reentered','1970-10-16 00:00:00','1970-10-06 00:00:00',89.39,201,287,64.96,'Intermediate' from dual union all
select 4561,'004561','1970-078A','Kosmos-366','Zenit-2M',21555,'Landed','1970-10-13 00:00:00','1970-10-02 00:00:00',89.49,204,294,66,'Intermediate' from dual union all
select 4562,'004562','1970-078B','11A57I','11A57I',21555,'Reentered','1970-10-06 00:00:00','1970-10-03 00:00:00',89.11,176,284,64.91,'Intermediate' from dual union all
select 4563,'004563','1970-075C','KDU','KDU',21552,'Reentered','1970-10-09 00:00:00','1970-10-01 00:00:00',89.48,195,303,65.43,'Intermediate' from dual union all
select 4564,'004564','1970-079A','Kosmos-367','US-A reactor',21556,'In Earth orbit','','1970-11-19 00:00:00',104.51,918,1025,65.28,'Intermediate' from dual union all
select 4565,'004565','1970-079B','deb Kosmos-367','US-A',21556,'Reentered','1970-10-06 00:00:00','1970-10-05 00:00:00',88.94,220,225,65.21,'Intermediate' from dual union all
select 4566,'004566','1970-079C','Kosmos-367 DU','US-A DU',21556,'Reentered','1970-10-31 00:00:00','1970-10-14 00:00:00',89.28,235,243,65.08,'Intermediate' from dual union all
select 4567,'004567','1970-079D','deb Kosmos-367','',21556,'Reentered','1970-10-08 00:00:00','1970-10-05 00:00:00',89.27,238,238,65.09,'Intermediate' from dual union all
select 4568,'004568','1970-079E','Kosmos-367 RLS','US-A RLS antenna',21556,'Reentered','1970-10-12 00:00:00','1970-10-07 00:00:00',88.62,181,232,64.77,'Intermediate' from dual union all
select 4569,'004569','1970-077A','Molniya-1','Molniya-1 11F67 No. 19',21554,'Reentered','1976-03-20 00:00:00','1970-11-10 00:00:00',717.62,667,39680,65.47,'GEO Transfer' from dual union all
select 457,'000457','62 B Iota 3','Blok-L No. T103-15','Blok-L',8168,'Reentered','1963-02-26 00:00:00','1962-11-26 00:00:00',93.26,182,686,64.92,'Intermediate' from dual union all
select 4570,'004570','1970-077D','Blok-L','Blok-L',21554,'Reentered','1972-05-31 00:00:00','1970-11-02 00:00:00',704.64,486,39219,65.49,'GEO Transfer' from dual union all
select 4571,'004571','1970-080A','Kosmos-368','Zenit-2M (Bion)',21557,'Landed','1970-10-14 00:00:00','1970-10-12 00:00:00',90.5,205,393,64.99,'Intermediate' from dual union all
select 4572,'004572','1970-080B','11A57I','11A57I',21557,'Reentered','1970-10-20 00:00:00','1970-10-11 00:00:00',90.17,202,364,65,'Intermediate' from dual union all
select 4573,'004573','1970-081A','Kosmos-369','DS-P1-Yu No. 42',21558,'Reentered','1971-01-22 00:00:00','1970-11-09 00:00:00',91.64,256,454,70.94,'Intermediate' from dual union all
select 4574,'004574','1970-081B','S1M','S1M',21558,'Reentered','1970-11-30 00:00:00','1970-10-17 00:00:00',91.87,268,464,70.93,'Intermediate' from dual union all
select 4575,'004575','1970-080C','Kosmos-368 part','Bion part',21557,'Reentered','1970-11-01 00:00:00','1970-10-09 00:00:00',90.22,212,358,64.97,'Intermediate' from dual union all
select 4576,'004576','1970-082A','Kosmos-370','Zenit-4M',21559,'Landed','1970-10-22 00:00:00','1970-10-11 00:00:00',89.37,201,285,64.91,'Intermediate' from dual union all
select 4577,'004577','1970-082B','11A57I','11A57I',21559,'Reentered','1970-10-13 00:00:00','1970-10-11 00:00:00',88.75,186,239,64.93,'Intermediate' from dual union all
select 4578,'004578','1970-083A','Kosmos-371','Tsiklon',21560,'In Earth orbit','','1970-11-19 00:00:00',99.86,750,753,74,'Intermediate' from dual union all
select 4579,'004579','1970-083B','S3M','S3M',21560,'In Earth orbit','','1970-11-19 00:00:00',99.75,739,754,74,'Intermediate' from dual union all
select 458,'000458','62 B Iota 4','BOZ','BOZ',8168,'Reentered','1962-11-29 00:00:00','1962-11-19 00:00:00',91.16,198,465,64.92,'Intermediate' from dual union all
select 4580,'004580','1970-084A','Interkosmos-4','DS-U3-IK No. 2',21561,'Reentered','1971-01-17 00:00:00','1970-11-14 00:00:00',92.71,254,561,48.42,'Intermediate' from dual union all
select 4581,'004581','1970-084B','S1M','S1M',21561,'Reentered','1970-12-17 00:00:00','1970-11-19 00:00:00',91.66,250,462,48.42,'Intermediate' from dual union all
select 4582,'004582','1970-080D','KBE part','KBE part',21557,'Reentered','1970-10-19 00:00:00','1970-10-14 00:00:00',90.08,189,367,65,'Intermediate' from dual union all
select 4583,'004583','1970-085A','Meteor','Meteor-M No. 16',21562,'Reentered','1999-01-08 00:00:00','1970-11-25 00:00:00',97.41,625,644,81.23,'Intermediate' from dual union all
select 4584,'004584','1970-085B','8A92ME','8A92ME',21562,'Reentered','2001-01-01 00:00:00','1970-11-26 00:00:00',97.54,548,734,81.24,'Intermediate' from dual union all
select 4585,'004585','1970-080E','Kapsula Kosmosa-368','Nauka 1KS No. 3',21557,'Reentered','1970-11-04 00:00:00','1970-10-15 00:00:00',90.39,202,386,64.99,'Intermediate' from dual union all
select 4586,'004586','1963-049K','deb Transit VBN-2','[UNID030] RCS 0.0',9280,'Reentered','1979-06-01 00:00:00','1970-10-22 00:00:00',105.76,847,1212,89.75,'Polar' from dual;
insert into SATELLITE
select 4587,'004587','1963-038J','deb Transit VBN-1','[was UNID031] RCS 0.0',9269,'Reentered','1981-10-04 00:00:00','1970-11-08 00:00:00',107.24,1066,1132,89.9,'Polar' from dual union all
select 4588,'004588','1970-086A','Kosmos-372','Strela-2M',21563,'In Earth orbit','','1970-11-23 00:00:00',100.74,784,803,74.06,'Intermediate' from dual union all
select 4589,'004589','1970-086B','S3M','S3M',21563,'In Earth orbit','','1970-12-05 00:00:00',100.65,772,806,74.06,'Intermediate' from dual union all
select 459,'000459','62 B Iota 5','deb Sputnik','',8168,'Reentered','1962-12-24 00:00:00','1962-11-27 00:00:00',90.7,177,441,64.87,'Intermediate' from dual union all
select 4590,'004590','1970-087A','Kosmos-373','IS-M',21564,'Reentered','1980-03-08 00:00:00','1970-12-14 00:00:00',94.8,452,566,62.91,'Intermediate' from dual union all
select 4591,'004591','1970-088A','Zond-8','L-1 No. 14',21565,'Deep Space','1970-10-21 00:00:00','1970-10-20 00:00:00',15562.18,200,400000,51.6,'Cislunar' from dual union all
select 4592,'004592','1970-088B','SOZ','SOZ',21565,'Reentered','1970-10-26 00:00:00','1970-10-23 00:00:00',88.36,191,196,51.51,'Intermediate' from dual union all
select 4593,'004593','1970-082C','KDU','KDU',21559,'Reentered','1970-10-27 00:00:00','1970-10-22 00:00:00',88.74,182,243,64.9,'Intermediate' from dual union all
select 4594,'004594','1970-089A','Kosmos-374','I2P',21566,'Exploded','1970-10-23 00:00:00','1970-10-23 00:00:00',100.67,529,1052,62.96,'Intermediate' from dual union all
select 4595,'004595','1970-087B','11S692','11S692',21564,'Reentered','1971-01-18 00:00:00','1970-10-28 00:00:00',92.56,126,674,62.25,'Intermediate' from dual union all
select 4596,'004596','1970-090A','OPS 7568','GAMBIT-3 Mission 4329',21567,'Reentered Attached','1970-11-11 00:00:00','1970-11-01 00:00:00',89.59,122,387,111.02,'Retrograde' from dual union all
select 4597,'004597','1970-089B','Kosmos-374 main debris','',21566,'In Earth orbit','','1970-12-10 00:00:00',111.82,516,2105,62.93,'Intermediate' from dual union all
select 4598,'004598','1970-091A','Kosmos-375','I2P',21568,'Exploded','1970-10-30 00:00:00','1970-10-30 00:00:00',100.43,565,993,62.86,'Intermediate' from dual union all
select 4599,'004599','1970-092A','Kosmos-376','Zenit-4M No. 47',21569,'Landed','1970-11-12 00:00:00','1970-11-03 00:00:00',89.36,204,282,65.4,'Intermediate' from dual union all
select 46,'000046','60 Eta 2','GREB','GRAB SV2',6321,'In Earth orbit','','1960-08-17 00:00:00',101.63,614,1058,66.77,'Intermediate' from dual union all
select 460,'000460','62 B Iota 6','deb Sputnik','',8168,'Reentered','1962-12-08 00:00:00','1962-11-19 00:00:00',92.74,185,633,64.77,'Intermediate' from dual union all
select 4600,'004600','1970-092B','11A57I','11A57I',21569,'Reentered','1970-11-05 00:00:00','1970-11-01 00:00:00',89.02,204,249,65.4,'Intermediate' from dual union all
select 4601,'004601','1970-025D','deb Agena D','',21502,'Reentered','2000-04-20 00:00:00','1992-02-06 00:00:00',100.26,674,867,100.02,'Sun-Synch' from dual union all
select 4602,'004602','1970-025E','deb Agena D','',21502,'Reentered','1982-10-14 00:00:00','1970-11-10 00:00:00',102.17,632,1091,99.49,'Sun-Synch' from dual union all
select 4603,'004603','1970-025F','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.61,1043,1097,99.94,'Sun-Synch' from dual union all
select 4604,'004604','1970-025G','deb Agena D','',21502,'Reentered','1994-02-04 00:00:00','1992-02-06 00:00:00',96.93,557,666,99.88,'Sun-Synch' from dual union all
select 4605,'004605','1970-025H','deb Agena D','',21502,'Reentered','1986-08-11 00:00:00','1986-08-09 00:00:00',89.7,249,271,99.35,'Retrograde' from dual union all
select 4606,'004606','1970-025J','deb Agena D','',21502,'Reentered','1979-08-14 00:00:00','1970-11-13 00:00:00',100.83,510,1086,99.6,'Sun-Synch' from dual union all
select 4607,'004607','1970-025K','deb Agena D','',21502,'In Earth orbit','','1992-01-29 00:00:00',117.26,1059,2057,100.21,'Sun-Synch' from dual union all
select 4608,'004608','1970-025L','deb Agena D','',21502,'Reentered','1992-01-26 00:00:00','1992-01-26 00:00:00',87.97,164,184,99.78,'Retrograde' from dual union all
select 4609,'004609','1970-025M','deb Agena D','',21502,'Reentered','1991-05-15 00:00:00','1991-05-14 00:00:00',89.94,261,282,98.86,'Sun-Synch' from dual union all
select 461,'000461','61 Omicron 142','deb r/b Ablestar','',7273,'In Earth orbit','','1962-11-15 00:00:00',103.04,827,978,67.59,'Intermediate' from dual union all
select 4610,'004610','1970-025N','deb Agena D','',21502,'Reentered','1999-01-01 00:00:00','1992-02-06 00:00:00',99.07,670,759,99.6,'Sun-Synch' from dual union all
select 4611,'004611','1970-025P','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',103.46,811,1034,99.85,'Sun-Synch' from dual union all
select 4612,'004612','1970-025Q','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',103.89,868,1017,99.77,'Sun-Synch' from dual union all
select 4613,'004613','1970-025R','deb Agena D','',21502,'In Earth orbit','','1992-02-02 00:00:00',103.41,820,1019,99.08,'Sun-Synch' from dual union all
select 4614,'004614','1970-025S','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',104.57,907,1042,100.13,'Sun-Synch' from dual union all
select 4615,'004615','1970-025T','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.39,966,1060,100.21,'Sun-Synch' from dual union all
select 4616,'004616','1970-089C','deb Kosmos-374','',21566,'In Earth orbit','','1992-02-05 00:00:00',114.23,622,2219,62.82,'Intermediate' from dual union all
select 4617,'004617','1970-089D','deb Kosmos-374','',21566,'In Earth orbit','','1992-02-06 00:00:00',106.84,514,1647,62.93,'Intermediate' from dual union all
select 4618,'004618','1970-089E','deb Kosmos-374','',21566,'Reentered','2000-12-23 00:00:00','1992-02-06 00:00:00',102.05,477,1235,62.89,'Intermediate' from dual union all
select 4619,'004619','1970-089F','deb Kosmos-374','',21566,'Reentered','2002-10-09 00:00:00','1992-02-05 00:00:00',105.39,634,1392,63.3,'Intermediate' from dual union all
select 462,'000462','61 Omicron 143','deb r/b Ablestar','',7273,'Reentered','2001-04-01 00:00:00','1963-01-02 00:00:00',103.1,827,984,67.62,'Intermediate' from dual union all
select 4620,'004620','1970-089G','deb Kosmos-374 [*]','',21566,'Reentered','2001-03-11 00:00:00','1992-02-06 00:00:00',101.91,529,1170,62.78,'Intermediate' from dual union all
select 4621,'004621','1970-089H','deb Kosmos-374','',21566,'In Earth orbit','','1992-02-05 00:00:00',109.49,530,1877,62.93,'Intermediate' from dual union all
select 4622,'004622','1970-091B','deb Kosmos-375','',21568,'Reentered','2002-09-03 00:00:00','1970-11-30 00:00:00',111.46,521,2067,62.79,'Intermediate' from dual union all
select 4623,'004623','1970-091C','deb Kosmos-375','',21568,'Reentered','2001-03-11 00:00:00','1992-02-05 00:00:00',101.59,537,1131,62.81,'Intermediate' from dual union all
select 4624,'004624','1970-091D','deb Kosmos-375','',21568,'Reentered','2000-06-08 00:00:00','1992-02-05 00:00:00',100.13,530,999,62.81,'Intermediate' from dual union all
select 4625,'004625','1970-091E','deb Kosmos-375','',21568,'Reentered','2000-11-24 00:00:00','1992-02-05 00:00:00',100.69,537,1046,62.81,'Intermediate' from dual union all
select 4626,'004626','1970-091F','deb Kosmos-375','',21568,'Reentered','1980-02-20 00:00:00','1980-02-18 00:00:00',91.54,305,396,62.73,'Intermediate' from dual union all
select 4627,'004627','1970-091G','deb Kosmos-375','',21568,'Reentered','1988-11-25 00:00:00','1988-11-25 00:00:00',88.55,196,210,62.69,'Intermediate' from dual union all
select 4628,'004628','1970-091H','deb Kosmos-375','',21568,'In Earth orbit','','1992-02-05 00:00:00',106.33,559,1555,62.81,'Intermediate' from dual union all
select 4629,'004629','1970-091J','deb Kosmos-375','',21568,'In Earth orbit','','1992-02-05 00:00:00',111.26,527,2043,62.83,'Intermediate' from dual union all
select 463,'000463','61 Omicron 144','deb r/b Ablestar','',7273,'In Earth orbit','','1962-11-15 00:00:00',103.1,836,974,66.81,'Intermediate' from dual union all
select 4630,'004630','1970-093A','DSP F1','DSP 1',21570,'In Earth orbit','','1971-04-24 00:00:00',1198.03,26045,35930,7.61,'Medium' from dual union all
select 4631,'004631','1970-093C','Titan 23C-1 Stage 2','LR91AJ11 23C-1',21570,'Reentered','1970-11-07 00:00:00','1970-11-07 00:00:00',89.31,149,332,28.61,'Intermediate' from dual union all
select 4632,'004632','1970-093B','Transtage 19','Transtage 19',21570,'In Earth orbit','','1976-11-30 00:00:00',1197.67,25557,36403,8.08,'Medium' from dual union all
select 4633,'004633','1970-091K','deb Kosmos-375','',21568,'Reentered','1984-02-21 00:00:00','1984-02-21 00:00:00',89.57,236,271,62.57,'Intermediate' from dual union all
select 4634,'004634','1970-091L','deb Kosmos-375','',21568,'In Earth orbit','','1992-02-06 00:00:00',106.84,558,1603,62.81,'Intermediate' from dual union all
select 4635,'004635','1970-091M','deb Kosmos-375','',21568,'In Earth orbit','','1992-02-04 00:00:00',110.74,564,1958,62.82,'Intermediate' from dual union all
select 4636,'004636','1970-025U','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.72,998,1059,100.02,'Sun-Synch' from dual union all
select 4637,'004637','1970-025V','deb Agena D','',21502,'In Earth orbit','','1992-02-04 00:00:00',105.96,1020,1059,99.78,'Sun-Synch' from dual union all
select 4638,'004638','1970-025W','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',104.78,955,1014,100,'Sun-Synch' from dual union all
select 4639,'004639','1970-025X','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',108.25,1059,1232,99.75,'Sun-Synch' from dual union all
select 464,'000464','61 Omicron 145','deb r/b Ablestar','',7273,'Reentered','1989-12-18 00:00:00','1962-11-15 00:00:00',102.62,762,1004,67.78,'Intermediate' from dual union all
select 4640,'004640','1970-025Y','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',108.18,1047,1239,99.75,'Sun-Synch' from dual union all
select 4641,'004641','1970-025Z','deb Agena D','',21502,'Reentered','1996-09-21 00:00:00','1992-02-06 00:00:00',99.88,751,755,99.75,'Sun-Synch' from dual union all
select 4642,'004642','1970-025AA','deb Agena D','',21502,'Reentered','2013-06-12 00:00:00','1992-02-05 00:00:00',103.8,913,964,99.94,'Sun-Synch' from dual union all
select 4643,'004643','1970-025AB','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.56,1059,1076,99.57,'Sun-Synch' from dual union all
select 4644,'004644','1970-025AC','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.4,1050,1069,99.45,'Sun-Synch' from dual union all
select 4645,'004645','1970-025AD','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',112.12,1067,1582,99.86,'Sun-Synch' from dual union all
select 4646,'004646','1970-025AE','deb Agena D','',21502,'Reentered','1991-10-25 00:00:00','1991-10-24 00:00:00',90.24,282,291,99.86,'Retrograde' from dual union all
select 4647,'004647','1970-025AF','deb Agena D','',21502,'Reentered','2001-01-01 00:00:00','1992-02-05 00:00:00',98.64,600,787,99.91,'Sun-Synch' from dual union all
select 4648,'004648','1970-025AG','deb Agena D','',21502,'Reentered','2015-05-19 00:00:00','1992-02-05 00:00:00',100.18,650,885,99.82,'Sun-Synch' from dual union all
select 4649,'004649','1970-025AH','deb Agena D','',21502,'In Earth orbit','','1992-02-02 00:00:00',100.91,676,927,99.9,'Sun-Synch' from dual union all
select 465,'000465','61 Omicron 146','deb r/b Ablestar','',7273,'In Earth orbit','','1962-11-15 00:00:00',103.77,894,980,66.32,'Intermediate' from dual union all
select 4650,'004650','1970-025AJ','deb Agena D','',21502,'In Earth orbit','','1992-02-03 00:00:00',103.44,819,1024,99.91,'Sun-Synch' from dual union all
select 4651,'004651','1970-025AK','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',104.31,915,1009,100.11,'Sun-Synch' from dual union all
select 4652,'004652','1970-025AL','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',104.95,966,1019,99.64,'Sun-Synch' from dual union all
select 4653,'004653','1970-025AM','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',103.58,894,962,100.06,'Sun-Synch' from dual union all
select 4654,'004654','1970-025AN','deb Agena D','',21502,'In Earth orbit','','1992-02-03 00:00:00',104.3,932,992,100.02,'Sun-Synch' from dual union all
select 4655,'004655','1970-025AP','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.57,979,1064,100.08,'Sun-Synch' from dual union all
select 4656,'004656','1970-025AQ','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.76,1010,1049,99.71,'Sun-Synch' from dual union all
select 4657,'004657','1970-025AR','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.73,1016,1042,100.14,'Sun-Synch' from dual union all
select 4658,'004658','1970-025AS','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.05,984,1010,99.98,'Sun-Synch' from dual union all
select 4659,'004659','1970-025AT','deb Agena D','',21502,'In Earth orbit','','1992-02-02 00:00:00',105.36,999,1024,100.15,'Sun-Synch' from dual union all
select 466,'000466','61 Omicron 147','deb r/b Ablestar','',7273,'Reentered','1988-11-26 00:00:00','1962-11-15 00:00:00',101.24,658,977,66.82,'Intermediate' from dual union all
select 4660,'004660','1970-025AU','deb Agena D','',21502,'Reentered','2013-05-02 00:00:00','1992-02-06 00:00:00',103.82,917,962,100.07,'Sun-Synch' from dual union all
select 4661,'004661','1970-091N','deb Kosmos-375','',21568,'Reentered','1995-08-30 00:00:00','1992-02-06 00:00:00',98.06,513,818,62.83,'Intermediate' from dual union all
select 4662,'004662','1970-091P','deb Kosmos-375','',21568,'Reentered','1982-04-07 00:00:00','1982-04-07 00:00:00',89.99,264,284,62.66,'Intermediate' from dual union all
select 4663,'004663','1970-091Q','deb Kosmos-375','',21568,'Reentered','1988-08-24 00:00:00','1988-08-21 00:00:00',89.66,239,277,62.72,'Intermediate' from dual union all
select 4664,'004664','1970-091R','deb Kosmos-375','',21568,'Reentered','2012-01-09 00:00:00','1992-02-05 00:00:00',104.78,533,1435,62.76,'Intermediate' from dual union all
select 4665,'004665','1970-091S','deb Kosmos-375','',21568,'In Earth orbit','','1992-02-04 00:00:00',110.61,498,2012,62.57,'Intermediate' from dual union all
select 4666,'004666','1970-089J','deb Kosmos-374','',21566,'Reentered','1990-09-25 00:00:00','1990-09-24 00:00:00',90.25,258,316,62.82,'Intermediate' from dual union all
select 4667,'004667','1970-089K','deb Kosmos-374','',21566,'In Earth orbit','','1992-02-04 00:00:00',109.21,563,1818,62.96,'Intermediate' from dual union all
select 4668,'004668','1970-089L','deb Kosmos-374','',21566,'In Earth orbit','','1992-02-06 00:00:00',112.12,575,2074,62.96,'Intermediate' from dual union all
select 4669,'004669','1970-089M','deb Kosmos-374','',21566,'Reentered','1980-08-08 00:00:00','1980-08-06 00:00:00',90.88,279,356,62.59,'Intermediate' from dual union all
select 467,'000467','61 Omicron 148','deb r/b Ablestar','',7273,'In Earth orbit','','1962-11-14 00:00:00',110.51,985,1516,66.55,'Intermediate' from dual union all
select 4670,'004670','1970-089N','deb Kosmos-374','',21566,'Reentered','1978-04-03 00:00:00','1971-01-02 00:00:00',110.94,431,2109,63.46,'Intermediate' from dual union all
select 4671,'004671','1970-089P','deb Kosmos-374','',21566,'Reentered','1979-01-27 00:00:00','1970-12-02 00:00:00',111.2,474,2090,62.6,'Intermediate' from dual union all
select 4672,'004672','1970-025AV','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.68,1042,1104,99.75,'Sun-Synch' from dual union all
select 4673,'004673','1970-025AW','deb Agena D','',21502,'Reentered','2015-06-06 00:00:00','1992-02-05 00:00:00',103.95,824,1067,99.07,'Sun-Synch' from dual union all
select 4674,'004674','1970-025AX','deb Agena D','',21502,'Reentered','1982-08-13 00:00:00','1982-08-11 00:00:00',92.59,384,419,99.3,'Sun-Synch' from dual union all
select 4675,'004675','1970-025AY','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.62,1070,1071,100.06,'Sun-Synch' from dual union all
select 4676,'004676','1970-025AZ','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.45,1062,1063,99.99,'Sun-Synch' from dual;
insert into SATELLITE
select 4677,'004677','1970-025BA','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.32,1049,1064,99.95,'Sun-Synch' from dual union all
select 4678,'004678','1970-025BB','deb Agena D','',21502,'Reentered','1980-02-20 00:00:00','1970-11-08 00:00:00',101.68,604,1072,100.1,'Sun-Synch' from dual union all
select 4679,'004679','1970-025BC','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.91,1023,1050,100.2,'Sun-Synch' from dual union all
select 468,'000468','61 Omicron 149','deb r/b Ablestar','',7273,'Reentered','1981-12-03 00:00:00','1962-11-15 00:00:00',104.19,880,1033,66.62,'Intermediate' from dual union all
select 4680,'004680','1970-025BD','deb Agena D','',21502,'Reentered','1988-12-16 00:00:00','1988-12-12 00:00:00',93.64,446,460,99.69,'Sun-Synch' from dual union all
select 4681,'004681','1970-025BE','deb Agena D','',21502,'In Earth orbit','','1992-02-02 00:00:00',110.48,1057,1441,99.8,'Sun-Synch' from dual union all
select 4682,'004682','1970-025BF','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',109.12,1065,1308,99.8,'Sun-Synch' from dual union all
select 4683,'004683','1970-025BG','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',108.34,1054,1246,99.9,'Sun-Synch' from dual union all
select 4684,'004684','1970-025BH','deb Agena D','',21502,'Reentered','1990-08-08 00:00:00','1990-08-07 00:00:00',91.06,309,345,99.69,'Retrograde' from dual union all
select 4685,'004685','1970-025BJ','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.69,1031,1116,99.94,'Sun-Synch' from dual union all
select 4686,'004686','1970-025BK','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.65,1042,1100,100.35,'Sun-Synch' from dual union all
select 4687,'004687','1970-025BL','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.84,1058,1103,99.89,'Sun-Synch' from dual union all
select 4688,'004688','1970-025BM','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.73,1059,1092,100.17,'Sun-Synch' from dual union all
select 4689,'004689','1970-025BN','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.65,1018,1125,99.74,'Sun-Synch' from dual union all
select 469,'000469','61 Omicron 150','deb r/b Ablestar','',7273,'In Earth orbit','','1962-11-15 00:00:00',103.11,839,973,66.85,'Intermediate' from dual union all
select 4690,'004690','1970-094A','OFO','OFO',21571,'Reentered','1971-05-09 00:00:00','1970-12-10 00:00:00',92.48,287,506,37.43,'Intermediate' from dual union all
select 4691,'004691','1970-095A','Luna-17','E-8 No. 203',21572,'Deep Space','1970-11-11 00:00:00','1970-11-10 00:00:00',15562.18,200,400000,51.52,'Cislunar' from dual union all
select 4692,'004692','1970-094B','RM','RM',21571,'Reentered','1971-02-07 00:00:00','1970-12-11 00:00:00',92.19,289,475,37.43,'Intermediate' from dual union all
select 4693,'004693','1970-094C','RM yo weight','',21571,'Reentered','1970-12-18 00:00:00','1970-11-21 00:00:00',91.86,269,463,37.38,'Intermediate' from dual union all
select 4694,'004694','1970-095B','SOZ','SOZ',21572,'Reentered','1970-11-13 00:00:00','1970-11-12 00:00:00',88.34,180,205,51.52,'Intermediate' from dual union all
select 4695,'004695','1970-096A','Kosmos-377','Zenit-2M',21573,'Landed','1970-11-23 00:00:00','1970-11-14 00:00:00',89.32,203,279,65,'Intermediate' from dual union all
select 4696,'004696','1970-096B','11A57I','11A57I',21573,'Reentered','1970-11-16 00:00:00','1970-11-13 00:00:00',89.07,190,267,64.96,'Intermediate' from dual union all
select 4697,'004697','1970-095C','SOZ','SOZ',21572,'Reentered','1970-11-13 00:00:00','1970-11-12 00:00:00',88.35,179,207,51.53,'Intermediate' from dual union all
select 4698,'004698','1970-089Q','deb Kosmos-374','',21566,'In Earth orbit','','1992-02-05 00:00:00',109.4,566,1832,63.1,'Intermediate' from dual union all
select 4699,'004699','1970-089R','deb Kosmos-374','',21566,'In Earth orbit','','1992-02-05 00:00:00',111.66,550,2056,62.94,'Intermediate' from dual union all
select 47,'000047','60 Eta 3','Ablestar 003','AJ10-104',6321,'In Earth orbit','','1960-08-18 00:00:00',101.33,616,1028,66.77,'Intermediate' from dual union all
select 470,'000470','61 Omicron 151','deb r/b Ablestar','',7273,'In Earth orbit','','1962-11-15 00:00:00',104.29,901,1022,66.25,'Intermediate' from dual union all
select 4700,'004700','1970-092C','KDU','KDU',21569,'Reentered','1970-11-20 00:00:00','1970-11-14 00:00:00',88.81,197,234,65.41,'Intermediate' from dual union all
select 4701,'004701','1970-025BP','deb Agena D','',21502,'Reentered','1972-07-22 00:00:00','1970-11-14 00:00:00',98.91,368,1046,100.05,'Sun-Synch' from dual union all
select 4702,'004702','1970-025BQ','deb Agena D','',21502,'Reentered','1972-07-07 00:00:00','1970-11-14 00:00:00',99.89,450,1057,99.98,'Sun-Synch' from dual union all
select 4703,'004703','1970-025BR','deb Agena D','',21502,'Reentered','1978-12-01 00:00:00','1978-11-19 00:00:00',91.88,326,408,99.95,'Retrograde' from dual union all
select 4704,'004704','1970-025BS','deb Agena D','',21502,'Reentered','1992-01-27 00:00:00','1992-01-23 00:00:00',91.1,318,339,99.89,'Retrograde' from dual union all
select 4705,'004705','1970-025BT','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.67,1037,1107,99.89,'Sun-Synch' from dual union all
select 4706,'004706','1970-025BU','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',107.11,1067,1119,99.67,'Sun-Synch' from dual union all
select 4707,'004707','1970-025BV','deb Agena D','',21502,'Reentered','1983-10-14 00:00:00','1983-10-12 00:00:00',92.23,374,394,99.47,'Sun-Synch' from dual union all
select 4708,'004708','1970-025BW','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',105.94,975,1102,100.11,'Sun-Synch' from dual union all
select 4709,'004709','1970-025BX','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',102.1,753,964,100.08,'Sun-Synch' from dual union all
select 471,'000471','61 Omicron 152','deb r/b Ablestar','',7273,'Reentered','2007-08-02 00:00:00','1993-08-05 00:00:00',100.27,737,806,66.87,'Intermediate' from dual union all
select 4710,'004710','1970-025BY','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.5,1061,1068,100.2,'Sun-Synch' from dual union all
select 4711,'004711','1970-089S','deb Kosmos-374','',21566,'In Earth orbit','','1992-02-03 00:00:00',109.41,507,1892,63.11,'Intermediate' from dual union all
select 4712,'004712','1970-091T','deb Kosmos-375','',21568,'In Earth orbit','','1992-02-05 00:00:00',106.91,565,1603,62.82,'Intermediate' from dual union all
select 4713,'004713','1970-097A','Kosmos-378','DS-U2-IP No. 1',21574,'Reentered','1972-08-17 00:00:00','1970-12-18 00:00:00',104.37,233,1697,74.01,'Intermediate' from dual union all
select 4714,'004714','1970-097B','S3M','S3M',21574,'Reentered','1972-09-30 00:00:00','1970-12-20 00:00:00',104.28,226,1696,73.99,'Intermediate' from dual union all
select 4715,'004715','1970-025BZ','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',105.04,966,1027,99.18,'Sun-Synch' from dual union all
select 4716,'004716','1970-025CA','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',104.8,976,995,99.99,'Sun-Synch' from dual union all
select 4717,'004717','1970-025CB','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.5,1062,1067,100.06,'Sun-Synch' from dual union all
select 4718,'004718','1970-025CC','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',103.53,836,1015,100.09,'Sun-Synch' from dual union all
select 4719,'004719','1970-025CD','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',103.96,846,1046,99.94,'Sun-Synch' from dual union all
select 472,'000472','61 Omicron 153','deb r/b Ablestar','',7273,'Reentered','2003-04-26 00:00:00','1993-08-08 00:00:00',99.37,681,776,66.75,'Intermediate' from dual union all
select 4720,'004720','1970-025CE','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',103.7,855,1012,99.99,'Sun-Synch' from dual union all
select 4721,'004721','1970-098A','OPS 4992','CORONA QR-2R',21575,'Reentered Attached','1970-12-11 00:00:00','1970-11-26 00:00:00',88.4,175,216,83,'Intermediate' from dual union all
select 4722,'004722','1970-098B','TOPHAT I','P-11 No. 4423',21575,'Reentered','1977-09-14 00:00:00','1971-01-10 00:00:00',94.48,478,508,83.19,'Intermediate' from dual union all
select 4723,'004723','1970-091U','deb Kosmos-375','',21568,'In Earth orbit','','1992-02-06 00:00:00',104.85,555,1420,62.8,'Intermediate' from dual union all
select 4724,'004724','1970-091V','deb Kosmos-375','',21568,'In Earth orbit','','1992-02-06 00:00:00',106.38,564,1554,62.81,'Intermediate' from dual union all
select 4725,'004725','1970-091W','deb Kosmos-375','',21568,'Reentered','1988-08-27 00:00:00','1988-08-25 00:00:00',90.17,260,306,62.22,'Intermediate' from dual union all
select 4726,'004726','1970-091X','deb Kosmos-375','',21568,'Reentered','2001-07-07 00:00:00','1992-02-05 00:00:00',101.88,534,1162,62.78,'Intermediate' from dual union all
select 4727,'004727','1970-091Y','deb Kosmos-375','',21568,'In Earth orbit','','1992-02-05 00:00:00',107.17,526,1666,62.72,'Intermediate' from dual union all
select 4728,'004728','1970-091Z','deb Kosmos-375','',21568,'In Earth orbit','','1992-02-06 00:00:00',109.74,519,1911,62.9,'Intermediate' from dual union all
select 4729,'004729','1970-091AA','deb Kosmos-375','',21568,'Reentered','1980-10-22 00:00:00','1980-10-19 00:00:00',90.91,289,350,62.67,'Intermediate' from dual union all
select 473,'000473','61 Omicron 154','deb r/b Ablestar','',7273,'Reentered','1972-03-02 00:00:00','1962-11-15 00:00:00',101.8,727,961,67.13,'Intermediate' from dual union all
select 4730,'004730','1970-091AB','deb Kosmos-375','',21568,'Reentered','1978-11-26 00:00:00','1978-10-12 00:00:00',95.62,406,692,62.7,'Intermediate' from dual union all
select 4731,'004731','1970-091AC','deb Kosmos-375','',21568,'Reentered','1981-02-24 00:00:00','1981-02-24 00:00:00',89.99,268,280,62.69,'Intermediate' from dual union all
select 4732,'004732','1970-025CF','deb Agena D','',21502,'Reentered','1972-02-06 00:00:00','1970-11-22 00:00:00',98.51,357,1018,100.07,'Sun-Synch' from dual union all
select 4733,'004733','1970-025CG','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',101.23,689,945,99.92,'Sun-Synch' from dual union all
select 4734,'004734','1970-025CH','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',104.69,913,1047,100.12,'Sun-Synch' from dual union all
select 4735,'004735','1970-025CJ','deb Agena D','',21502,'In Earth orbit','','1992-02-04 00:00:00',102.73,820,956,99.67,'Sun-Synch' from dual union all
select 4736,'004736','1970-025CK','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',103.82,863,1016,100.07,'Sun-Synch' from dual union all
select 4737,'004737','1970-025CL','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',104.11,925,981,99.2,'Sun-Synch' from dual union all
select 4738,'004738','1970-025CM','deb Agena D','',21502,'In Earth orbit','','1992-02-03 00:00:00',104.29,919,1004,100.15,'Sun-Synch' from dual union all
select 4739,'004739','1970-025CN','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.2,1000,1008,99.91,'Sun-Synch' from dual union all
select 474,'000474','61 Omicron 155','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-05 00:00:00',101.03,772,842,66.26,'Intermediate' from dual union all
select 4740,'004740','1970-025CP','deb Agena D','',21502,'Reentered','1972-08-10 00:00:00','1972-07-29 00:00:00',91.62,299,409,99.99,'Retrograde' from dual union all
select 4741,'004741','1970-025CQ','deb Agena D','',21502,'In Earth orbit','','1992-02-04 00:00:00',120.76,1075,2355,100.29,'Medium' from dual union all
select 4742,'004742','1970-025CR','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.66,1057,1087,99.84,'Sun-Synch' from dual union all
select 4743,'004743','1970-025CS','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.52,1044,1087,99.87,'Sun-Synch' from dual union all
select 4744,'004744','1970-025CT','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.43,1049,1074,100.16,'Sun-Synch' from dual union all
select 4745,'004745','1970-025CU','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',104.9,965,1015,100.07,'Sun-Synch' from dual union all
select 4746,'004746','1970-025CV','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',103.7,922,945,100.01,'Sun-Synch' from dual union all
select 4747,'004747','1970-094D','OFO despin weight','',21571,'Reentered','1971-01-25 00:00:00','1970-12-13 00:00:00',91.93,279,460,37.47,'Intermediate' from dual union all
select 4748,'004748','1970-025CW','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.18,1045,1055,100.07,'Sun-Synch' from dual union all
select 4749,'004749','1970-025CX','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.55,1009,1031,99.37,'Sun-Synch' from dual union all
select 475,'000475','61 Omicron 156','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-08 00:00:00',103.89,808,1077,66.24,'Intermediate' from dual union all
select 4750,'004750','1970-025CY','deb Agena D','',21502,'Reentered','1990-08-23 00:00:00','1990-08-23 00:00:00',90.53,287,314,99.83,'Retrograde' from dual union all
select 4751,'004751','1970-025CZ','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',105.47,1011,1021,100.16,'Sun-Synch' from dual union all
select 4752,'004752','1970-025DA','deb Agena D','',21502,'In Earth orbit','','1992-02-04 00:00:00',104.56,961,987,99.91,'Sun-Synch' from dual union all
select 4753,'004753','1970-025DB','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.51,1014,1023,99.95,'Sun-Synch' from dual union all
select 4754,'004754','1970-025DC','deb Agena D','',21502,'In Earth orbit','','1992-02-03 00:00:00',104.76,956,1011,99.6,'Sun-Synch' from dual union all
select 4755,'004755','1970-025DD','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.34,1005,1016,100.17,'Sun-Synch' from dual union all
select 4756,'004756','1970-025DE','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.94,1064,1105,100.28,'Sun-Synch' from dual union all
select 4757,'004757','1970-025DF','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.29,1044,1066,99.42,'Sun-Synch' from dual union all
select 4758,'004758','1970-025DG','deb Agena D','',21502,'Reentered','1982-03-18 00:00:00','1982-03-16 00:00:00',90.1,267,292,99.72,'Retrograde' from dual union all
select 4759,'004759','1970-025DH','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',105.76,1029,1030,99.95,'Sun-Synch' from dual union all
select 476,'000476','61 Omicron 157','deb r/b Ablestar','',7273,'Reentered','2014-04-02 00:00:00','1993-08-08 00:00:00',101.45,803,851,67.1,'Intermediate' from dual union all
select 4760,'004760','1970-099A','Kosmos-379','T2K No. 1',21576,'Reentered','1983-09-21 00:00:00','1970-12-26 00:00:00',258.37,170,13948,51.68,'Highly Ellip' from dual union all
select 4761,'004761','1970-099B','11S510','11S510',21576,'Reentered','1970-11-26 00:00:00','1970-11-26 00:00:00',87.8,162,170,51.61,'Intermediate' from dual union all
select 4762,'004762','1970-100A','Kosmos-380','DS-P1-Yu No. 26',21577,'Reentered','1971-06-17 00:00:00','1970-12-26 00:00:00',100.57,197,1374,81.95,'Intermediate' from dual union all
select 4763,'004763','1970-100B','S1M','S1M',21577,'Reentered','1971-04-09 00:00:00','1970-12-26 00:00:00',99.67,193,1293,81.94,'Intermediate' from dual union all
select 4764,'004764','1970-025DJ','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',105.11,961,1039,99.64,'Sun-Synch' from dual union all
select 4765,'004765','1970-025DK','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',102.21,781,946,99.58,'Sun-Synch' from dual union all
select 4766,'004766','1970-025DL','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.37,1051,1066,99.96,'Sun-Synch' from dual;
insert into SATELLITE
select 4767,'004767','1970-025DM','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',105.43,974,1056,100.08,'Sun-Synch' from dual union all
select 4768,'004768','1970-025DN','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.55,996,1045,100.13,'Sun-Synch' from dual union all
select 4769,'004769','1970-025DP','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',102.45,801,949,99.61,'Sun-Synch' from dual union all
select 477,'000477','61 Omicron 158','deb r/b Ablestar','',7273,'Reentered','1982-01-18 00:00:00','1962-12-02 00:00:00',102.23,755,974,66.63,'Intermediate' from dual union all
select 4770,'004770','1970-025DQ','deb Agena D','',21502,'Reentered','1989-05-16 00:00:00','1989-05-15 00:00:00',91.03,265,386,99.62,'Retrograde' from dual union all
select 4771,'004771','1970-025DR','deb Agena D','',21502,'Reentered','1983-02-22 00:00:00','1983-02-21 00:00:00',91.22,317,352,99.7,'Retrograde' from dual union all
select 4772,'004772','1970-025DS','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',109.41,1057,1342,99.82,'Sun-Synch' from dual union all
select 4773,'004773','1970-025DT','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.74,1050,1102,99.92,'Sun-Synch' from dual union all
select 4774,'004774','1970-025DU','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.1,1011,1081,99.77,'Sun-Synch' from dual union all
select 4775,'004775','1970-025DV','deb Agena D','',21502,'Reentered','1981-07-31 00:00:00','1981-04-16 00:00:00',96.67,520,679,100.09,'Sun-Synch' from dual union all
select 4776,'004776','1970-025DW','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.12,985,1015,100.11,'Sun-Synch' from dual union all
select 4777,'004777','1970-025DX','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.46,1009,1023,99.95,'Sun-Synch' from dual union all
select 4778,'004778','1970-025DY','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.59,1042,1096,99.56,'Sun-Synch' from dual union all
select 4779,'004779','1970-101A','Molniya-1','Molniya-1 11F67 No. 23',21578,'Reentered','1975-11-25 00:00:00','1971-01-08 00:00:00',717.76,490,39864,65.4,'GEO Transfer' from dual union all
select 478,'000478','61 Omicron 159','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-01 00:00:00',100.91,749,855,66.08,'Intermediate' from dual union all
select 4780,'004780','1970-099C','T2K LPU','T2K lower section',21576,'Reentered','1971-02-13 00:00:00','1971-01-08 00:00:00',94.56,179,816,51.58,'Intermediate' from dual union all
select 4781,'004781','1970-101B','11S510','11S510',21578,'Reentered','1970-12-11 00:00:00','1970-11-29 00:00:00',90.52,211,388,65.43,'Intermediate' from dual union all
select 4782,'004782','1970-101C','BOZ','BOZ',21578,'Reentered','1970-12-17 00:00:00','1970-12-04 00:00:00',90.56,207,397,65.4,'Intermediate' from dual union all
select 4783,'004783','1970-102A','Kosmos-381','Ionosfernaya Stantsiya N',21579,'In Earth orbit','','1971-01-05 00:00:00',104.88,966,1011,74.04,'Intermediate' from dual union all
select 4784,'004784','1970-102B','S3M','S3M',21579,'In Earth orbit','','1971-01-07 00:00:00',104.77,966,1002,74.04,'Intermediate' from dual union all
select 4785,'004785','1970-101D','Blok-L','Blok-L',21578,'Reentered','1976-01-02 00:00:00','1970-12-27 00:00:00',702.95,420,39201,65.41,'GEO Transfer' from dual union all
select 4786,'004786','1970-103A','Kosmos-382','L-1e No. 2K',21580,'In Earth orbit','','1971-01-01 00:00:00',171.02,2579,5077,55.89,'Medium' from dual union all
select 4787,'004787','1970-104A','Kosmos-383','Zenit-4MK',21581,'Landed','1970-12-16 00:00:00','1970-12-07 00:00:00',89.02,185,268,65.42,'Intermediate' from dual union all
select 4788,'004788','1970-104B','11A57I','11A57I',21581,'Reentered','1970-12-09 00:00:00','1970-12-05 00:00:00',88.81,185,247,65.4,'Intermediate' from dual union all
select 4789,'004789','1970-103B','SOZ','SOZ',21580,'In Earth orbit','','1971-01-05 00:00:00',158.8,1589,5083,51.6,'Medium' from dual union all
select 479,'000479','61 Omicron 160','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-08 00:00:00',101.91,804,894,66.72,'Intermediate' from dual union all
select 4790,'004790','1970-103C','SOZ','SOZ',21580,'In Earth orbit','','1971-01-03 00:00:00',159.11,1615,5082,51.6,'Medium' from dual union all
select 4791,'004791','1970-105A','Kosmos-384','Zenit-2M',21582,'Landed','1970-12-22 00:00:00','1970-12-12 00:00:00',89.4,203,287,72.87,'Intermediate' from dual union all
select 4792,'004792','1970-105B','11A57I','11A57I',21582,'Reentered','1970-12-15 00:00:00','1970-12-12 00:00:00',89.09,193,266,72.89,'Intermediate' from dual union all
select 4793,'004793','1970-106A','NOAA 1','ITOS A',21583,'In Earth orbit','','1971-01-18 00:00:00',114.87,1425,1474,101.95,'Sun-Synch' from dual union all
select 4794,'004794','1970-106B','CEPE','CEP',21583,'In Earth orbit','','1971-02-03 00:00:00',114.84,1423,1474,101.92,'Sun-Synch' from dual union all
select 4795,'004795','1970-105C','Sensor cover','Sensor cover',21582,'Reentered','1970-12-18 00:00:00','1970-12-10 00:00:00',89.26,198,278,72.88,'Intermediate' from dual union all
select 4796,'004796','1970-105D','Sensor cover','Sensor cover',21582,'Reentered','1970-12-18 00:00:00','1970-12-18 00:00:00',89.02,175,278,72.98,'Intermediate' from dual union all
select 4797,'004797','1970-107A','Uhuru','SAS A',21584,'Reentered','1979-04-05 00:00:00','1971-01-15 00:00:00',95.72,534,572,3.03,'Equatorial' from dual union all
select 4798,'004798','1970-107B','FW4S S/N 2223-10','FW4S',21584,'Reentered','1978-12-15 00:00:00','1971-01-15 00:00:00',95.72,534,572,3.03,'Equatorial' from dual union all
select 4799,'004799','1970-108A','Kosmos-385','Tsiklon',21585,'In Earth orbit','','1971-01-13 00:00:00',104.71,978,984,74.01,'Intermediate' from dual union all
select 48,'000048','60 Theta','Discoverer 13','CORONA Diagnostic 2',6333,'Reentered Attached','1960-11-14 00:00:00','1960-09-09 00:00:00',93.16,245,614,82.85,'Intermediate' from dual union all
select 480,'000480','61 Omicron 161','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-07 00:00:00',101.69,766,912,66.24,'Intermediate' from dual union all
select 4800,'004800','1970-108B','S3M','S3M',21585,'In Earth orbit','','1971-01-15 00:00:00',104.59,970,981,74.02,'Intermediate' from dual union all
select 4801,'004801','1970-109A','Peole','Peole',21586,'Reentered','1980-06-16 00:00:00','1971-01-13 00:00:00',97.21,509,742,15.01,'Equatorial' from dual union all
select 4802,'004802','1970-109B','P.068','P068',21586,'Reentered','2001-12-23 00:00:00','1971-02-25 00:00:00',98.65,637,752,15.01,'Equatorial' from dual union all
select 4803,'004803','1970-109C','deb Peole','[spurious]',21586,'Reentered','1980-05-14 00:00:00','1970-12-12 00:00:00',94.91,454,575,15,'Equatorial' from dual union all
select 4804,'004804','1970-110A','Kosmos-386','Zenit-4M',21587,'Landed','1970-12-28 00:00:00','1970-12-23 00:00:00',89.99,212,336,64.99,'Intermediate' from dual union all
select 4805,'004805','1970-110B','11A57I','11A57I',21587,'Reentered','1970-12-19 00:00:00','1970-12-17 00:00:00',88.43,184,210,65.03,'Intermediate' from dual union all
select 4806,'004806','1970-111A','Kosmos-387','Tselina-OM',21588,'Reentered','1980-01-19 00:00:00','1971-01-16 00:00:00',95.24,524,537,74.01,'Intermediate' from dual union all
select 4807,'004807','1970-111B','S3M','S3M',21588,'Reentered','1980-03-25 00:00:00','1971-01-16 00:00:00',95.07,508,536,74.03,'Intermediate' from dual union all
select 4808,'004808','1970-110C','Sensor cover','Zenit-4M sensor cover',21587,'Reentered','1970-12-17 00:00:00','1970-12-15 00:00:00',88.02,165,188,65,'Intermediate' from dual union all
select 4809,'004809','1970-111C','deb Kosmos-387','Tselina-O part',21588,'Reentered','1971-12-12 00:00:00','1971-01-17 00:00:00',94.6,482,517,74.04,'Intermediate' from dual union all
select 481,'000481','62 B Rho','FTV 1135','CORONA M-15',8178,'Reentered Attached','1962-12-13 00:00:00','1962-12-02 00:00:00',89.73,206,317,65.13,'Intermediate' from dual union all
select 4810,'004810','1970-105E','Kapsula Kosmosa-384','Nauka 2KS No. 2',21582,'Reentered','1970-12-27 00:00:00','1970-12-17 00:00:00',89.26,201,274,72.86,'Intermediate' from dual union all
select 4811,'004811','1970-112A','Kosmos-388','DS-P1-Yu No. 43',21589,'Reentered','1971-05-10 00:00:00','1971-01-17 00:00:00',91.96,263,479,70.95,'Intermediate' from dual union all
select 4812,'004812','1970-112B','S1M','S1M',21589,'Reentered','1971-02-17 00:00:00','1971-01-17 00:00:00',91.27,255,419,70.95,'Intermediate' from dual union all
select 4813,'004813','1970-113A','Kosmos-389','Tselina-D No. Yu2250-01',21590,'Reentered','2003-11-24 00:00:00','1971-01-21 00:00:00',97.98,638,687,81.19,'Intermediate' from dual union all
select 4814,'004814','1970-113B','8A92ME','8A92ME',21590,'In Earth orbit','','1971-01-19 00:00:00',98.01,598,729,81.2,'Intermediate' from dual union all
select 4815,'004815','1970-111D','deb Kosmos-387','Tselina-O part',21588,'Reentered','1971-09-03 00:00:00','1971-01-16 00:00:00',94.69,483,525,74.06,'Intermediate' from dual union all
select 4816,'004816','1970-111E','deb Kosmos-387','Tselina-O part',21588,'Reentered','1971-08-24 00:00:00','1971-01-19 00:00:00',94.73,486,525,73.96,'Intermediate' from dual union all
select 4817,'004817','1970-099D','T2K part','T2K part',21576,'Reentered','1971-01-02 00:00:00','1970-12-21 00:00:00',94.04,179,766,51.62,'Intermediate' from dual union all
select 4818,'004818','1970-025DZ','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',110.3,1046,1434,99.83,'Sun-Synch' from dual union all
select 4819,'004819','1970-025EA','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',104.36,910,1019,100.05,'Sun-Synch' from dual union all
select 482,'000482','62 B Iota 7','deb Sputnik','',8168,'Reentered','1962-12-10 00:00:00','1962-11-26 00:00:00',92.23,182,586,64.75,'Intermediate' from dual union all
select 4820,'004820','1970-025EB','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',104.79,960,1009,99.69,'Sun-Synch' from dual union all
select 4821,'004821','1970-025EC','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.41,1050,1071,99.81,'Sun-Synch' from dual union all
select 4822,'004822','1970-025ED','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',105.09,941,1057,99.59,'Sun-Synch' from dual union all
select 4823,'004823','1970-089T','deb Kosmos-374','',21566,'In Earth orbit','','1970-12-01 00:00:00',115.57,653,2310,63.02,'Intermediate' from dual union all
select 4824,'004824','1970-089U','deb Kosmos-374','',21566,'Reentered','1979-10-21 00:00:00','1970-12-01 00:00:00',110.41,437,2055,62.32,'Intermediate' from dual union all
select 4825,'004825','1970-089V','deb Kosmos-374','',21566,'Reentered','2004-05-06 00:00:00','1970-12-22 00:00:00',112.16,518,2134,62.87,'Intermediate' from dual union all
select 4826,'004826','1970-091AD','deb Kosmos-375  [*]','',21568,'Reentered','1981-08-05 00:00:00','1971-01-01 00:00:00',112.41,525,2150,62.92,'Intermediate' from dual union all
select 4827,'004827','1970-091AE','deb Kosmos-375  [*]','',21568,'Reentered','1990-03-02 00:00:00','1990-03-01 00:00:00',90.71,286,333,62.71,'Intermediate' from dual union all
select 4828,'004828','1970-091AF','deb Kosmos-375','',21568,'Reentered','1980-11-09 00:00:00','1973-08-15 00:00:00',111.97,542,2093,62.81,'Intermediate' from dual union all
select 4829,'004829','1970-114A','Molniya-1','Molniya-1 11F67 No. 22',21591,'Reentered','1975-12-22 00:00:00','1971-01-25 00:00:00',717.74,472,39881,65.03,'GEO Transfer' from dual union all
select 483,'000483','62 B Iota 8','deb Sputnik','',8168,'Reentered','1963-01-02 00:00:00','1962-11-26 00:00:00',92.87,182,649,64.77,'Intermediate' from dual union all
select 4830,'004830','1970-114B','11S510','11S510',21591,'Reentered','1971-01-23 00:00:00','1971-01-02 00:00:00',90.74,225,397,65.02,'Intermediate' from dual union all
select 4831,'004831','1970-114C','BOZ','BOZ',21591,'Reentered','1971-02-10 00:00:00','1971-01-01 00:00:00',91.5,237,460,64.97,'Intermediate' from dual union all
select 4832,'004832','1970-114D','deb Blok-I','',21591,'Reentered','1971-01-02 00:00:00','1971-12-30 00:00:00',91.63,195,514,65,'Intermediate' from dual union all
select 4833,'004833','1970-114E','deb Blok-I','',21591,'Reentered','1971-01-02 00:00:00','1970-12-27 00:00:00',90.79,224,403,65.01,'Intermediate' from dual union all
select 4834,'004834','1970-113C','deb Kosmos-389','',21590,'Reentered','1972-09-19 00:00:00','1971-01-17 00:00:00',97.89,639,677,81.18,'Intermediate' from dual union all
select 4835,'004835','1970-110D','KDU','KDU',21587,'Reentered','1971-01-12 00:00:00','1970-12-27 00:00:00',89.92,217,323,64.99,'Intermediate' from dual union all
select 4836,'004836','1970-110E','Blenda','BZA',21587,'Reentered','1971-01-03 00:00:00','1970-12-27 00:00:00',89.74,210,314,65,'Intermediate' from dual union all
select 4837,'004837','1970-114F','Blok-L','Blok-L',21591,'Reentered','1973-03-03 00:00:00','1971-01-31 00:00:00',709.77,382,39577,65.01,'GEO Transfer' from dual union all
select 4838,'004838','1970-110F','PTDU cover','',21587,'Reentered','1971-01-02 00:00:00','1970-12-29 00:00:00',91.63,227,482,65.01,'Intermediate' from dual union all
select 4839,'004839','1970-109D','PEOLE despin weight','',21586,'Reentered','1979-04-20 00:00:00','1971-01-13 00:00:00',97.12,502,740,14.99,'Equatorial' from dual union all
select 484,'000484','62 B Iota 9','deb Sputnik','',8168,'Reentered','1962-12-04 00:00:00','1962-11-26 00:00:00',90.68,176,440,64.87,'Intermediate' from dual union all
select 4840,'004840','1970-102C','deb Kosmos-381','',21579,'Reentered','1992-04-23 00:00:00','1971-01-03 00:00:00',104.82,964,1008,74.03,'Intermediate' from dual union all
select 4841,'004841','1968-055D','deb Star 37D','',16782,'In Earth orbit','','1968-08-03 00:00:00',156.53,645,5840,120.83,'Medium' from dual union all
select 4842,'004842','1970-025EE','deb Agena D','',21502,'In Earth orbit','','1992-02-04 00:00:00',105,962,1027,99.91,'Sun-Synch' from dual union all
select 4843,'004843','1967-034E','NNS O-12 part','[UNID032] RCD 0.0',15289,'In Earth orbit','','1970-08-31 00:00:00',106.46,1048,1078,90.21,'Polar' from dual union all
select 4844,'004844','1970-094E','OFO despin weight','',21571,'Reentered','1971-01-21 00:00:00','1971-01-03 00:00:00',90.82,256,374,37.38,'Intermediate' from dual union all
select 4845,'004845','1971-001A','Kosmos-390','Zenit-4M',24819,'Landed','1971-01-25 00:00:00','1971-01-14 00:00:00',89.26,203,272,65.01,'Intermediate' from dual union all
select 4846,'004846','1971-001B','11A57I','11A57I',24819,'Reentered','1971-01-17 00:00:00','1971-01-14 00:00:00',88.98,197,252,65.02,'Intermediate' from dual union all
select 4847,'004847','1971-002A','Kosmos-391','DS-P1-I No. 11',24820,'Reentered','1972-02-21 00:00:00','1971-02-14 00:00:00',95.04,262,779,70.91,'Intermediate' from dual union all
select 4848,'004848','1971-002B','S1M','S1M',24820,'Reentered','1971-08-21 00:00:00','1971-02-13 00:00:00',94.69,260,748,70.92,'Intermediate' from dual union all
select 4849,'004849','1971-003A','Meteor','Meteor-M No. 19',24821,'Reentered','2005-06-14 00:00:00','1971-02-21 00:00:00',97.53,628,653,81.21,'Intermediate' from dual union all
select 485,'000485','62 B Iota 10','deb Sputnik','',8168,'Reentered','1962-12-10 00:00:00','1962-11-27 00:00:00',91.3,177,500,64.87,'Intermediate' from dual union all
select 4850,'004850','1971-003B','8A92ME','8A92ME',24821,'Reentered','2002-10-18 00:00:00','1971-02-21 00:00:00',97.69,562,735,81.22,'Intermediate' from dual union all
select 4851,'004851','1970-025EF','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',103.87,926,956,99.87,'Sun-Synch' from dual union all
select 4852,'004852','1970-025EG','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',102.76,848,931,99.77,'Sun-Synch' from dual union all
select 4853,'004853','1970-025EH','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',102.56,834,926,99.74,'Sun-Synch' from dual union all
select 4854,'004854','1970-025EJ','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.56,1054,1080,99.64,'Sun-Synch' from dual union all
select 4855,'004855','1970-025EK','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.67,1012,1040,99.9,'Sun-Synch' from dual union all
select 4856,'004856','1970-025EL','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',102.9,772,1019,99.71,'Sun-Synch' from dual;
insert into SATELLITE
select 4857,'004857','1970-025EM','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',103.57,878,977,99.79,'Sun-Synch' from dual union all
select 4858,'004858','1970-025EN','deb Agena D','',21502,'Reentered','1979-05-23 00:00:00','1979-05-13 00:00:00',93.1,400,453,100,'Sun-Synch' from dual union all
select 4859,'004859','1970-025EP','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.74,1035,1117,99.96,'Sun-Synch' from dual union all
select 486,'000486','62 B Iota 11','deb Sputnik','',8168,'Reentered','1962-11-30 00:00:00','1962-11-28 00:00:00',89.36,177,309,64.87,'Intermediate' from dual union all
select 4860,'004860','1970-025EQ','deb Agena D','',21502,'Reentered','1990-05-06 00:00:00','1990-05-05 00:00:00',90.85,298,334,98.71,'Sun-Synch' from dual union all
select 4861,'004861','1970-025ER','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',102.66,840,929,99.75,'Sun-Synch' from dual union all
select 4862,'004862','1970-025ES','deb Agena D','',21502,'Reentered','2001-10-26 00:00:00','1992-02-05 00:00:00',100.14,694,837,99.63,'Sun-Synch' from dual union all
select 4863,'004863','1970-025ET','deb Agena D','',21502,'Reentered','2011-12-01 00:00:00','1992-02-05 00:00:00',102.43,838,910,99.74,'Sun-Synch' from dual union all
select 4864,'004864','1970-025EU','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.98,1051,1122,99.54,'Sun-Synch' from dual union all
select 4865,'004865','1970-025EV','deb Agena D','',21502,'Reentered','1981-10-26 00:00:00','1981-10-05 00:00:00',97.63,638,653,99.83,'Sun-Synch' from dual union all
select 4866,'004866','1970-025EW','deb Agena D','',21502,'Reentered','2003-01-27 00:00:00','1992-02-06 00:00:00',102.02,811,898,99.71,'Sun-Synch' from dual union all
select 4867,'004867','1970-025EX','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',103.46,815,1030,99.6,'Sun-Synch' from dual union all
select 4868,'004868','1970-025EY','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',104.29,912,1011,99.74,'Sun-Synch' from dual union all
select 4869,'004869','1970-025EZ','deb Agena D','',21502,'Reentered','1999-12-21 00:00:00','1992-02-06 00:00:00',98.14,580,759,99.96,'Sun-Synch' from dual union all
select 487,'000487','62 B Iota 12','deb Sputnik','',8168,'Reentered','1962-12-01 00:00:00','1962-11-27 00:00:00',90.57,177,427,64.87,'Intermediate' from dual union all
select 4870,'004870','1970-025FA','deb Agena D','',21502,'Reentered','1979-11-09 00:00:00','1979-10-20 00:00:00',97.48,533,744,100.03,'Sun-Synch' from dual union all
select 4871,'004871','1970-025FB','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',102.69,840,931,99.73,'Sun-Synch' from dual union all
select 4872,'004872','1971-004A','Kosmos-392','Zenit-2M',24822,'Landed','1971-02-02 00:00:00','1971-01-23 00:00:00',89.3,203,277,65,'Intermediate' from dual union all
select 4873,'004873','1971-004B','11A57I','11A57I',24822,'Reentered','1971-01-26 00:00:00','1971-01-23 00:00:00',88.97,189,258,65,'Intermediate' from dual union all
select 4874,'004874','1971-005A','OPS 7776','GAMBIT-3 Mission 4330',24823,'Reentered Attached','1971-02-09 00:00:00','1971-01-28 00:00:00',89.98,130,417,110.83,'Retrograde' from dual union all
select 4875,'004875','1971-005B','OPS 7776 sensor cover','',24823,'Reentered','1971-01-22 00:00:00','1971-01-21 00:00:00',89.77,161,366,110.73,'Retrograde' from dual union all
select 4876,'004876','1969-082GU','deb Agena D','',18349,'Reentered','1981-04-13 00:00:00','1981-03-06 00:00:00',97.11,612,628,70.05,'Intermediate' from dual union all
select 4877,'004877','1970-025FC','deb Agena D','',21502,'Reentered','2013-02-05 00:00:00','1992-02-05 00:00:00',102.19,808,916,99.68,'Sun-Synch' from dual union all
select 4878,'004878','1971-001C','KDU','KDU',24819,'Reentered','1971-02-01 00:00:00','1971-01-23 00:00:00',89.03,202,252,65,'Intermediate' from dual union all
select 4879,'004879','1971-001D','Blenda','BZA',24819,'Reentered','1971-01-27 00:00:00','1971-01-23 00:00:00',88.97,198,250,65,'Intermediate' from dual union all
select 488,'000488','62 B Xi 4','deb Sputnik','',8183,'Reentered','1962-12-25 00:00:00','1962-11-29 00:00:00',91.55,200,502,64.75,'Intermediate' from dual union all
select 4880,'004880','1969-082GV','deb Agena D','',18349,'Reentered','2006-04-06 00:00:00','1992-02-06 00:00:00',99.56,697,779,70.06,'Intermediate' from dual union all
select 4881,'004881','1971-006A','INTELSAT IV F2','INTELSAT IV F2',24824,'In Earth orbit','','1971-04-22 00:00:00',1436.32,35785,35797,.47,'Stationary' from dual union all
select 4882,'004882','1971-006B','Centaur AC-25','Centaur 20D',24824,'In Earth orbit','','1971-03-01 00:00:00',654.37,599,36580,28.1,'GEO Transfer' from dual union all
select 4883,'004883','1971-001E','PTDU cover','',24819,'Reentered','1971-01-27 00:00:00','1971-01-26 00:00:00',90.01,215,335,65.04,'Intermediate' from dual union all
select 4884,'004884','1971-007A','Kosmos-393','DS-P1-Yu No. 34',24825,'Reentered','1971-06-16 00:00:00','1971-03-01 00:00:00',91.68,263,451,71.01,'Intermediate' from dual union all
select 4885,'004885','1971-007B','S1M','S1M',24825,'Reentered','1971-03-31 00:00:00','1971-02-26 00:00:00',91.16,258,405,71.02,'Intermediate' from dual union all
select 4886,'004886','1971-001F','PTDU cover','',24819,'Reentered','1971-01-27 00:00:00','1971-01-25 00:00:00',90.12,221,340,64.9,'Intermediate' from dual union all
select 4887,'004887','1969-082GW','deb Agena D','',18349,'Reentered','1978-07-16 00:00:00','1978-04-11 00:00:00',96.08,522,620,70.03,'Intermediate' from dual union all
select 4888,'004888','1969-082GX','deb Agena D','',18349,'Reentered','1983-06-21 00:00:00','1983-06-14 00:00:00',92.37,375,407,69.58,'Intermediate' from dual union all
select 4889,'004889','1969-082GY','deb Agena D','',18349,'Reentered','1978-08-01 00:00:00','1978-05-02 00:00:00',97.55,605,677,70.16,'Intermediate' from dual union all
select 489,'000489','62 B Xi 5','deb Sputnik','',8183,'Reentered','1962-12-27 00:00:00','1962-11-29 00:00:00',91.86,262,470,64.77,'Intermediate' from dual union all
select 4890,'004890','1969-082GZ','deb Agena D','',18349,'Reentered','2001-08-29 00:00:00','1992-02-06 00:00:00',99.98,737,779,70.28,'Intermediate' from dual union all
select 4891,'004891','1969-082HA','deb Agena D','',18349,'Reentered','1972-01-01 00:00:00','1980-09-17 00:00:00',93.15,412,446,69.99,'Intermediate' from dual union all
select 4892,'004892','1969-082HB','deb Agena D','',18349,'Reentered','1972-08-24 00:00:00','1971-01-27 00:00:00',97.53,538,743,69.61,'Intermediate' from dual union all
select 4893,'004893','1969-082HC','deb Agena D','',18349,'Reentered','1973-05-07 00:00:00','1971-01-27 00:00:00',97.91,538,780,69.4,'Intermediate' from dual union all
select 4894,'004894','1969-082HD','deb Agena D','',18349,'Reentered','1998-10-02 00:00:00','1992-02-06 00:00:00',98.7,679,714,70.63,'Intermediate' from dual union all
select 4895,'004895','1969-082HE','deb Agena D','',18349,'In Earth orbit','','1992-02-04 00:00:00',101.31,783,859,70.18,'Intermediate' from dual union all
select 4896,'004896','1970-091AG','deb Kosmos-375  [*]','',21568,'Reentered','1977-09-16 00:00:00','1977-07-31 00:00:00',97.29,286,971,63.36,'Intermediate' from dual union all
select 4897,'004897','1969-082HF','deb Agena D','',18349,'Reentered','1981-03-11 00:00:00','1981-01-31 00:00:00',95.36,526,546,70.2,'Intermediate' from dual union all
select 4898,'004898','1970-025FD','deb Agena D','',21502,'Reentered','1982-06-10 00:00:00','1971-01-31 00:00:00',107.11,1091,1095,99.9,'Sun-Synch' from dual union all
select 4899,'004899','1970-025FE','deb Agena D','',21502,'Reentered','2002-04-22 00:00:00','1992-02-06 00:00:00',101.61,788,881,99.66,'Sun-Synch' from dual union all
select 49,'000049','60 Iota 1','Echo 1','A-11',6323,'Reentered','1968-05-24 00:00:00','1960-08-12 00:00:00',118.21,1517,1683,47.24,'Intermediate' from dual union all
select 490,'000490','62 B Sigma','FTV 1155','CORONA M-16',8179,'Reentered Attached','1962-12-08 00:00:00','1962-12-06 00:00:00',88.34,123,262,65.19,'Intermediate' from dual union all
select 4900,'004900','1971-008A','Kitty Hawk','Apollo CM-110',24826,'Deep Space','1971-02-01 00:00:00','1971-02-01 00:00:00',18772.57,220,454980,31,'Cislunar' from dual union all
select 4901,'004901','1967-001E','deb unknown                       ','[UNID033]',15256,'Reentered','1971-02-09 00:00:00','1971-02-01 00:00:00',96.93,178,1046,26.59,'Intermediate' from dual union all
select 4902,'004902','1971-009A','NATO IIB','NATOSAT-II',24827,'In Earth orbit','','1977-01-25 00:00:00',1436.06,35777,35795,2.37,'Stationary' from dual union all
select 4903,'004903','1971-009B','Star 37D S/N 022','Star 37D',24827,'In Earth orbit','','1999-08-21 00:00:00',438.45,272,25208,25.23,'Highly Ellip' from dual union all
select 4904,'004904','1971-008B','Saturn S-IVB-509','Saturn S-IVB-509',24826,'Deep Space','1971-02-01 00:00:00','1971-02-01 00:00:00',16744.7,105,420757,30.83,'Cislunar' from dual union all
select 4905,'004905','1971-008C','Antares','LM 8 AS',24826,'Deep Space Attached','1971-02-01 00:00:00','1971-02-01 00:00:00',18772.57,220,454980,31,'Cislunar' from dual union all
select 4906,'004906','1969-082HG','deb Agena D','',18349,'Reentered','1981-09-17 00:00:00','1981-09-15 00:00:00',91.25,322,350,70.01,'Intermediate' from dual union all
select 4907,'004907','1965-082SV','deb Transtage 4','',11832,'In Earth orbit','','1992-02-06 00:00:00',99.33,686,768,32.49,'Intermediate' from dual union all
select 4908,'004908','1970-025FF','deb Agena D','',21502,'Reentered','1991-10-27 00:00:00','1991-10-26 00:00:00',91.35,326,356,99.63,'Sun-Synch' from dual union all
select 4909,'004909','1971-005C','Fairing','SRV Fairing',24823,'Reentered','1971-02-09 00:00:00','1971-02-09 00:00:00',89.88,141,395,110.7,'Retrograde' from dual union all
select 491,'000491','62 B Iota 13','deb Sputnik','',8168,'Reentered','1962-12-25 00:00:00','1962-12-04 00:00:00',91.33,159,521,64.87,'Intermediate' from dual union all
select 4910,'004910','1970-025FG','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.88,919,1153,99.35,'Sun-Synch' from dual union all
select 4911,'004911','1970-025FH','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',103.1,831,980,99.93,'Sun-Synch' from dual union all
select 4912,'004912','1970-025FJ','deb Agena D','',21502,'In Earth orbit','','1992-02-04 00:00:00',103.82,895,984,99.7,'Sun-Synch' from dual union all
select 4913,'004913','1970-025FK','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',103.48,815,1032,99.84,'Sun-Synch' from dual union all
select 4914,'004914','1970-025FL','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.11,969,1124,99.44,'Sun-Synch' from dual union all
select 4915,'004915','1970-025FM','deb Agena D','',21502,'Reentered','1971-08-08 00:00:00','1971-04-03 00:00:00',105.33,945,1075,99.64,'Sun-Synch' from dual union all
select 4916,'004916','1970-025FN','deb Agena D','',21502,'Reentered','2012-03-08 00:00:00','1992-02-06 00:00:00',102.13,805,914,99.66,'Sun-Synch' from dual union all
select 4917,'004917','1970-025FP','deb Agena D','',21502,'Reentered','2018-03-08 00:00:00','1992-02-05 00:00:00',102.58,829,932,99.73,'Sun-Synch' from dual union all
select 4918,'004918','1970-025FQ','deb Agena D','',21502,'In Earth orbit','','1992-01-30 00:00:00',108.9,1048,1304,99.46,'Sun-Synch' from dual union all
select 4919,'004919','1970-025FR','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',103.89,898,987,100.07,'Sun-Synch' from dual union all
select 492,'000492','62 B Iota 14','deb Sputnik','',8168,'Reentered','1962-12-15 00:00:00','1962-12-02 00:00:00',91.08,177,479,64.79,'Intermediate' from dual union all
select 4920,'004920','1970-025FS','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',107.34,1032,1176,99.4,'Sun-Synch' from dual union all
select 4921,'004921','1970-025FT','deb Agena D','',21502,'In Earth orbit','','1992-02-02 00:00:00',103.23,854,969,99.76,'Sun-Synch' from dual union all
select 4922,'004922','1971-010A','Kosmos-394','DS-P1-M No. 2',24828,'In Earth orbit','','1971-03-17 00:00:00',96.51,569,613,65.84,'Intermediate' from dual union all
select 4923,'004923','1971-010B','S3M','S3M',24828,'Reentered','1984-03-21 00:00:00','1971-03-17 00:00:00',96.4,561,612,65.84,'Intermediate' from dual union all
select 4924,'004924','62 B Lambda 2','Altair X-248-A5DM','[UNID005 71-000A] RCS 0.',8170,'Reentered','1983-12-22 00:00:00','1971-02-11 00:00:00',277.44,343,15082,18.05,'Highly Ellip' from dual union all
select 4925,'004925','1964-086B','Explorer XXVI dup','[UNID006 71-000B] RCS 1.',10372,'Spurious','1983-12-19 00:00:00','1971-09-30 00:00:00',396.05,250,22723,20.09,'Highly Ellip' from dual union all
select 4926,'004926','1964-086C','Altair 2 X-258C2 KW-0139','[UNID007 71-000C] RCS 2.',10372,'Reentered','1982-05-07 00:00:00','1971-02-11 00:00:00',393.41,298,22516,20.1,'Highly Ellip' from dual union all
select 4927,'004927','1971-010C','deb Kosmos-394','DS-P1-M part [RCS 0.15]',24828,'Reentered','1981-04-18 00:00:00','1971-03-17 00:00:00',96.49,569,612,65.84,'Intermediate' from dual union all
select 4928,'004928','1970-025FU','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',113.81,1051,1752,99.69,'Sun-Synch' from dual union all
select 4929,'004929','1970-025FV','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',114.44,1057,1803,99.71,'Sun-Synch' from dual union all
select 493,'000493','62 B Iota 15','deb Sputnik','',8168,'Reentered','1962-12-21 00:00:00','1962-12-02 00:00:00',91.11,184,473,65,'Intermediate' from dual union all
select 4930,'004930','1970-025FW','deb Agena D','',21502,'Reentered','2000-10-19 00:00:00','1992-02-05 00:00:00',101.95,782,920,99.21,'Sun-Synch' from dual union all
select 4931,'004931','1970-025FX','deb Agena D','',21502,'Reentered','1978-08-04 00:00:00','1978-07-02 00:00:00',94.42,444,537,100.07,'Sun-Synch' from dual union all
select 4932,'004932','1970-025FY','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.67,995,1057,99.81,'Sun-Synch' from dual union all
select 4933,'004933','1970-025FZ','deb Agena D','',21502,'Reentered','1979-06-08 00:00:00','1979-03-23 00:00:00',95.95,477,652,100.06,'Sun-Synch' from dual union all
select 4934,'004934','1970-025GA','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',105.69,1022,1032,100.05,'Sun-Synch' from dual union all
select 4935,'004935','1970-025GB','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',104.93,936,1046,99.85,'Sun-Synch' from dual union all
select 4936,'004936','1970-025GC','deb Agena D','',21502,'Reentered','1980-07-22 00:00:00','1971-02-13 00:00:00',102.25,671,1060,99.16,'Sun-Synch' from dual union all
select 4937,'004937','1969-082HH','deb Agena D','',18349,'Reentered','1991-05-12 00:00:00','1991-05-12 00:00:00',89.2,219,251,70.27,'Intermediate' from dual union all
select 4938,'004938','1969-082HJ','deb Agena D','',18349,'In Earth orbit','','1993-08-07 00:00:00',100.51,754,812,70.14,'Intermediate' from dual union all
select 4939,'004939','1969-082HK','deb Agena D','',18349,'Reentered','1980-10-27 00:00:00','1980-10-20 00:00:00',92.76,384,436,69.97,'Intermediate' from dual union all
select 494,'000494','62 B Iota 16','deb Sputnik','',8168,'Reentered','1962-12-17 00:00:00','1962-12-01 00:00:00',91.44,175,516,64.99,'Intermediate' from dual union all
select 4940,'004940','1970-091AH','deb Kosmos-375','',21568,'Reentered','2001-08-13 00:00:00','1992-02-06 00:00:00',101.87,540,1155,62.79,'Intermediate' from dual union all
select 4941,'004941','1970-091AJ','deb Kosmos-375','',21568,'Reentered','2015-11-17 00:00:00','1992-02-05 00:00:00',104.58,554,1396,62.8,'Intermediate' from dual union all
select 4942,'004942','1970-091AK','deb Kosmos-375','',21568,'In Earth orbit','','1992-02-05 00:00:00',111.94,600,2032,63.06,'Intermediate' from dual union all
select 4943,'004943','1970-025GD','deb Agena D','',21502,'In Earth orbit','','1992-02-01 00:00:00',110.19,1058,1413,99.68,'Sun-Synch' from dual union all
select 4944,'004944','1970-025GE','deb Agena D','',21502,'Reentered','2001-12-31 00:00:00','1992-02-05 00:00:00',100.21,690,847,100.15,'Sun-Synch' from dual union all
select 4945,'004945','1970-025GF','deb Agena D','',21502,'In Earth orbit','','1992-02-02 00:00:00',105.98,953,1128,99.89,'Sun-Synch' from dual union all
select 4946,'004946','1970-025GG','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.46,1005,1027,99.65,'Sun-Synch' from dual;
insert into SATELLITE
select 4947,'004947','1970-025GH','deb Agena D','',21502,'Reentered','1981-07-18 00:00:00','1981-07-15 00:00:00',91.13,314,346,99.78,'Retrograde' from dual union all
select 4948,'004948','1970-025GJ','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',108.06,954,1320,99.5,'Sun-Synch' from dual union all
select 4949,'004949','1970-025GK','deb Agena D','',21502,'Reentered','1972-03-05 00:00:00','1972-02-26 00:00:00',91.93,309,430,100.27,'Retrograde' from dual union all
select 495,'000495','62 B Iota 17','deb Sputnik','',8168,'Reentered','1962-12-26 00:00:00','1962-12-02 00:00:00',91.63,175,534,64.83,'Intermediate' from dual union all
select 4950,'004950','1970-025GL','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',110.12,1061,1404,99.79,'Sun-Synch' from dual union all
select 4951,'004951','1970-025GM','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.15,1035,1061,100.19,'Sun-Synch' from dual union all
select 4952,'004952','1971-011A','Tansei-1','MS-T1',24829,'In Earth orbit','','1971-03-21 00:00:00',106.08,985,1105,29.67,'Intermediate' from dual union all
select 4953,'004953','1971-012A','DAPP 3526','DAPP Block 5A F-3',24830,'In Earth orbit','','1971-03-19 00:00:00',100.85,767,831,98.82,'Sun-Synch' from dual union all
select 4954,'004954','1971-012B','Burner II No. 11','Star 37B',24830,'In Earth orbit','','1971-08-11 00:00:00',100.88,770,830,98.78,'Sun-Synch' from dual union all
select 4955,'004955','1971-013A','Kosmos-395','Tselina-OM',24831,'Reentered','1980-04-06 00:00:00','1971-03-20 00:00:00',95.33,527,543,74.05,'Intermediate' from dual union all
select 4956,'004956','1971-013B','S3M','S3M',24831,'Reentered','1980-07-19 00:00:00','1971-03-19 00:00:00',95.24,520,541,74.05,'Intermediate' from dual union all
select 4957,'004957','1971-012C','S70-3 Drag Sphere','NRL PL170A',24830,'Reentered','1989-10-17 00:00:00','1971-03-24 00:00:00',100.82,764,831,98.83,'Sun-Synch' from dual union all
select 4958,'004958','1971-012D','S70-3 Drag Sphere','NRL PL170B',24830,'Reentered','1989-09-20 00:00:00','1971-05-08 00:00:00',100.77,760,831,98.83,'Sun-Synch' from dual union all
select 4959,'004959','1971-014A','Kosmos-396','Zenit-4M No. 49',24832,'Landed','1971-03-03 00:00:00','1971-02-20 00:00:00',89.38,204,284,65.41,'Intermediate' from dual union all
select 496,'000496','62 B Iota 18','deb Sputnik','',8168,'Reentered','1962-12-23 00:00:00','1962-12-04 00:00:00',91.41,177,511,64.87,'Intermediate' from dual union all
select 4960,'004960','1971-014B','11A57I','11A57I',24832,'Reentered','1971-02-25 00:00:00','1971-02-20 00:00:00',89.16,204,262,65.41,'Intermediate' from dual union all
select 4961,'004961','1971-014C','Sensor cover','Zenit-4M sensor cover',24832,'Reentered','1971-02-20 00:00:00','1971-02-18 00:00:00',88.81,136,296,65.29,'Intermediate' from dual union all
select 4962,'004962','1971-014D','Sensor cover','Zenit-4M sensor cover',24832,'Reentered','1971-02-21 00:00:00','1971-02-20 00:00:00',89.39,220,268,65.41,'Intermediate' from dual union all
select 4963,'004963','1971-012E','S70-3 Drag Sphere','NRL PL170C',24830,'Reentered','1990-01-07 00:00:00','1971-03-20 00:00:00',100.88,771,830,98.83,'Sun-Synch' from dual union all
select 4964,'004964','1971-015A','Kosmos-397','I2P',24833,'In Earth orbit','','1971-03-27 00:00:00',113.49,574,2200,65.72,'Intermediate' from dual union all
select 4965,'004965','1971-015B','11S692','11S692',24833,'Reentered','1971-03-04 00:00:00','1971-02-28 00:00:00',91.1,140,517,65.13,'Intermediate' from dual union all
select 4966,'004966','1971-016A','Kosmos-398','T2K No. 2',24834,'Reentered','1995-12-10 00:00:00','1971-03-28 00:00:00',215.71,203,10869,51.6,'Medium' from dual union all
select 4967,'004967','1971-016B','11S510','11S510',24834,'Reentered','1971-03-01 00:00:00','1971-02-28 00:00:00',88.26,177,200,51.6,'Intermediate' from dual union all
select 4968,'004968','1970-025GN','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.08,1030,1060,100.17,'Sun-Synch' from dual union all
select 4969,'004969','1970-025GP','deb Agena D','',21502,'Reentered','1997-07-08 00:00:00','1992-02-06 00:00:00',98.77,665,735,99.64,'Sun-Synch' from dual union all
select 497,'000497','62 B Iota 19','deb Sputnik','',8168,'Reentered','1962-12-26 00:00:00','1962-12-02 00:00:00',91.76,174,547,65,'Intermediate' from dual union all
select 4970,'004970','1970-025GQ','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.46,1034,1092,99.82,'Sun-Synch' from dual union all
select 4971,'004971','1970-025GR','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',112.51,1071,1613,99.91,'Sun-Synch' from dual union all
select 4972,'004972','1970-025GS','deb Agena D','',21502,'Reentered','1971-05-29 00:00:00','1971-03-01 00:00:00',105.55,967,1074,100,'Sun-Synch' from dual union all
select 4973,'004973','1970-025GT','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',104.18,886,1026,100.05,'Sun-Synch' from dual union all
select 4974,'004974','1970-025GU','deb Agena D','',21502,'Reentered','1971-07-02 00:00:00','1971-03-01 00:00:00',106.41,1008,1113,99.9,'Sun-Synch' from dual union all
select 4975,'004975','1970-025GV','deb Agena D','',21502,'Reentered','1978-06-02 00:00:00','1978-03-06 00:00:00',94.24,409,554,100.17,'Sun-Synch' from dual union all
select 4976,'004976','1970-025GW','deb Agena D','',21502,'Reentered','2002-02-25 00:00:00','1992-02-06 00:00:00',101.25,765,870,99.67,'Sun-Synch' from dual union all
select 4977,'004977','1970-025GX','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.61,1054,1086,99.98,'Sun-Synch' from dual union all
select 4978,'004978','1970-025GY','deb Agena D','',21502,'Reentered','1981-04-13 00:00:00','1971-02-27 00:00:00',105.25,933,1080,100.29,'Sun-Synch' from dual union all
select 4979,'004979','1970-025GZ','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.18,1041,1059,100.01,'Sun-Synch' from dual union all
select 498,'000498','62 B Iota 20','deb Sputnik','',8168,'Reentered','1962-12-10 00:00:00','1962-12-02 00:00:00',90.27,182,394,64.96,'Intermediate' from dual union all
select 4980,'004980','1970-025HA','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.27,1047,1061,99.55,'Sun-Synch' from dual union all
select 4981,'004981','1970-025HB','deb Agena D','',21502,'Reentered','1999-12-23 00:00:00','1992-02-05 00:00:00',100.64,772,806,99.89,'Sun-Synch' from dual union all
select 4982,'004982','1970-025HC','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',107.67,1058,1180,99.6,'Sun-Synch' from dual union all
select 4983,'004983','1970-025HD','deb Agena D','',21502,'Reentered','2001-10-10 00:00:00','1992-02-06 00:00:00',99.75,669,825,99.89,'Sun-Synch' from dual union all
select 4984,'004984','1970-025HE','deb Agena D','',21502,'Reentered','1972-04-11 00:00:00','1971-02-24 00:00:00',97.82,382,927,99.95,'Sun-Synch' from dual union all
select 4985,'004985','1970-025HF','deb Agena D','',21502,'Reentered','2000-06-09 00:00:00','1992-02-06 00:00:00',101.33,810,834,99.84,'Sun-Synch' from dual union all
select 4986,'004986','1970-025HG','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',107.39,1065,1147,99.87,'Sun-Synch' from dual union all
select 4987,'004987','1970-025HH','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.88,1056,1109,99.42,'Sun-Synch' from dual union all
select 4988,'004988','1970-025HJ','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.16,1038,1059,99.86,'Sun-Synch' from dual union all
select 4989,'004989','1970-025HK','deb Agena D','',21502,'In Earth orbit','','1992-02-04 00:00:00',107.26,985,1215,99.73,'Sun-Synch' from dual union all
select 499,'000499','62 B Iota 21','deb Sputnik','',8168,'Reentered','1962-12-09 00:00:00','1962-12-02 00:00:00',90.1,180,379,64.96,'Intermediate' from dual union all
select 4990,'004990','1970-025HL','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',104.96,936,1050,100.04,'Sun-Synch' from dual union all
select 4991,'004991','1970-025HM','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.43,1046,1076,100.1,'Sun-Synch' from dual union all
select 4992,'004992','1970-025HN','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.12,1037,1057,99.93,'Sun-Synch' from dual union all
select 4993,'004993','1970-025HP','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',104.13,876,1032,100.22,'Sun-Synch' from dual union all
select 4994,'004994','1970-025HQ','deb Agena D','',21502,'Reentered','2004-11-12 00:00:00','1992-02-06 00:00:00',102.19,818,906,99.74,'Sun-Synch' from dual union all
select 4995,'004995','1970-025HR','deb Agena D','',21502,'Reentered','1989-10-02 00:00:00','1989-10-01 00:00:00',91.91,338,399,99.63,'Sun-Synch' from dual union all
select 4996,'004996','1970-025HS','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.18,926,1080,100.37,'Sun-Synch' from dual union all
select 4997,'004997','1970-025HT','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',104.3,912,1012,99.93,'Sun-Synch' from dual union all
select 4998,'004998','1970-025HU','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.5,1057,1072,99.94,'Sun-Synch' from dual union all
select 4999,'004999','1970-025HV','deb Agena D','',21502,'Reentered','1979-10-28 00:00:00','1979-10-25 00:00:00',98.11,622,715,100.14,'Sun-Synch' from dual union all
select 5,'000005','58 Beta 2','Vanguard I','Vanguard Test Satellite',4793,'In Earth orbit','','1959-05-23 00:00:00',133.99,657,3935,34.25,'Medium' from dual union all
select 50,'000050','60 Iota 2','Altair','Altair X-248-A5 S/N 3012',6323,'In Earth orbit','','1960-10-09 00:00:00',118.05,1528,1659,47.28,'Intermediate' from dual union all
select 500,'000500','62 B Iota 22','deb Sputnik','',8168,'Reentered','1962-12-28 00:00:00','1962-12-06 00:00:00',92.12,184,573,64.86,'Intermediate' from dual union all
select 5000,'005000','1970-025HW','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',114.17,1015,1821,99.76,'Sun-Synch' from dual union all
select 5001,'005001','1971-013C','deb Kosmos-395','Tselina-O part',24831,'Reentered','1972-02-15 00:00:00','1971-03-19 00:00:00',95.01,508,531,74.1,'Intermediate' from dual union all
select 5002,'005002','1971-013D','deb Kosmos-395','Tselina-O part',24831,'Reentered','1972-01-17 00:00:00','1971-03-20 00:00:00',94.9,502,526,74.07,'Intermediate' from dual union all
select 5003,'005003','1971-017A','Kosmos-399','Zenit-4M',24835,'Landed','1971-03-17 00:00:00','1971-03-12 00:00:00',89.64,197,317,65,'Intermediate' from dual union all
select 5004,'005004','1971-017B','11A57I','11A57I',24835,'Reentered','1971-03-08 00:00:00','1971-03-05 00:00:00',88.91,190,252,65.01,'Intermediate' from dual union all
select 5005,'005005','1971-014E','KDU','KDU',24832,'Reentered','1971-03-08 00:00:00','1971-03-03 00:00:00',88.97,178,270,65.42,'Intermediate' from dual union all
select 5006,'005006','1971-014F','Blenda','BZA',24832,'Reentered','1971-03-04 00:00:00','1971-03-03 00:00:00',88.59,172,238,65.36,'Intermediate' from dual union all
select 5007,'005007','1971-018A','Shi Jian','shijian',24836,'Reentered','1979-06-17 00:00:00','1971-04-30 00:00:00',105.79,264,1799,69.9,'Intermediate' from dual union all
select 5008,'005008','1971-016C','T2K lower section','T2K lower section',24834,'Reentered','1971-05-12 00:00:00','1971-03-28 00:00:00',96.54,182,1003,51.59,'Intermediate' from dual union all
select 5009,'005009','1971-015C','deb Kosmos-397','',24833,'Reentered','1993-04-11 00:00:00','1971-03-31 00:00:00',113.5,567,2208,65.72,'Intermediate' from dual union all
select 501,'000501','62 B Iota 23','deb Sputnik','',8168,'Reentered','1962-12-26 00:00:00','1962-12-03 00:00:00',93.64,169,737,64.55,'Intermediate' from dual union all
select 5010,'005010','1971-015D','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-05 00:00:00',110.34,568,1917,65.85,'Intermediate' from dual union all
select 5011,'005011','1971-015E','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-06 00:00:00',107.73,560,1684,65.69,'Intermediate' from dual union all
select 5012,'005012','1971-015F','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-06 00:00:00',108.84,556,1791,65.77,'Intermediate' from dual union all
select 5013,'005013','1971-015G','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-06 00:00:00',111.83,566,2056,65.79,'Intermediate' from dual union all
select 5014,'005014','1971-015H','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-06 00:00:00',109.2,548,1832,65.62,'Intermediate' from dual union all
select 5015,'005015','1971-015J','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-06 00:00:00',109.53,560,1850,65.73,'Intermediate' from dual union all
select 5016,'005016','1971-015K','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-05 00:00:00',110.09,562,1900,65.77,'Intermediate' from dual union all
select 5017,'005017','1971-015L','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-06 00:00:00',110.52,557,1945,65.72,'Intermediate' from dual union all
select 5018,'005018','1971-015M','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-06 00:00:00',112.77,563,2145,65.71,'Intermediate' from dual union all
select 5019,'005019','1971-015N','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-05 00:00:00',108.08,556,1720,65.7,'Intermediate' from dual union all
select 502,'000502','62 B Tau 1','NRL PL120','POPPY 1 SV1',8180,'Reentered','1967-02-09 00:00:00','1963-01-19 00:00:00',115.76,233,2747,70.34,'Intermediate' from dual union all
select 5020,'005020','1970-089W','deb Kosmos-374','',21566,'In Earth orbit','','1992-02-06 00:00:00',109.5,534,1874,62.94,'Intermediate' from dual union all
select 5021,'005021','1970-089X','deb Kosmos-374','',21566,'Reentered','2000-04-21 00:00:00','1992-02-06 00:00:00',101.69,479,1199,62.91,'Intermediate' from dual union all
select 5022,'005022','1970-089Y','deb Kosmos-374  [*]','',21566,'Reentered','1972-06-24 00:00:00','1974-03-30 00:00:00',99.07,511,918,62.61,'Intermediate' from dual union all
select 5023,'005023','1970-025HX','deb Agena D','',21502,'Reentered','1979-09-29 00:00:00','1979-07-11 00:00:00',94.73,470,541,100.03,'Sun-Synch' from dual union all
select 5024,'005024','1970-025HY','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',103.1,822,989,99.84,'Sun-Synch' from dual union all
select 5025,'005025','1970-025HZ','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',107.04,1056,1124,99.96,'Sun-Synch' from dual union all
select 5026,'005026','1970-025JA','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.23,1018,1086,100.21,'Sun-Synch' from dual union all
select 5027,'005027','1970-025JB','deb Agena D','',21502,'Reentered','1991-01-17 00:00:00','1991-01-17 00:00:00',89.27,219,257,99.99,'Retrograde' from dual union all
select 5028,'005028','1970-089Z','deb Kosmos-374  [*]','',21566,'In Earth orbit','','1992-02-06 00:00:00',111.74,749,1865,63.31,'Intermediate' from dual union all
select 5029,'005029','1970-089AA','deb Kosmos-374  [*]','',21566,'Reentered','1981-06-03 00:00:00','1981-03-28 00:00:00',100.12,455,1073,62.79,'Intermediate' from dual union all
select 503,'000503','62 B Upsilon 1','Relay 1','Relay A',8182,'In Earth orbit','','1963-02-07 00:00:00',185.06,1323,7436,47.5,'Medium' from dual union all
select 5030,'005030','1968-097CG','deb Kosmos-252','',16824,'Reentered','1978-02-26 00:00:00','1971-03-09 00:00:00',105.72,511,1546,62.47,'Intermediate' from dual union all
select 5031,'005031','1970-089AB','deb Kosmos-374','',21566,'Reentered','1990-03-30 00:00:00','1990-03-29 00:00:00',90.25,280,293,62.93,'Intermediate' from dual union all
select 5032,'005032','1970-089AC','deb Kosmos-374  [*]','',21566,'Reentered','1983-08-14 00:00:00','1983-08-12 00:00:00',90.69,289,328,62.73,'Intermediate' from dual union all
select 5033,'005033','1970-089AD','deb Kosmos-374  [*]','',21566,'Reentered','1990-03-01 00:00:00','1971-03-09 00:00:00',111.02,518,2030,62.82,'Intermediate' from dual union all
select 5034,'005034','1970-089AE','deb Kosmos-374','',21566,'Reentered','2001-04-14 00:00:00','1992-02-06 00:00:00',103.62,593,1266,62.75,'Intermediate' from dual union all
select 5035,'005035','1963-014CM','deb West Ford','',9245,'Reentered','1984-09-24 00:00:00','1971-02-26 00:00:00',165.26,1971,5224,86.4,'Medium' from dual;
insert into SATELLITE
select 5036,'005036','1970-067C','NNS O-19 despin weight','',21544,'In Earth orbit','','1970-09-28 00:00:00',104.59,952,999,89.95,'Polar' from dual union all
select 5037,'005037','1970-025JC','deb Agena D','',21502,'Reentered','1980-07-18 00:00:00','1971-03-09 00:00:00',101.46,582,1074,100.22,'Sun-Synch' from dual union all
select 5038,'005038','1970-025JD','deb Agena D','',21502,'Reentered','1971-07-18 00:00:00','1971-03-09 00:00:00',95.84,340,779,100.06,'Sun-Synch' from dual union all
select 5039,'005039','1971-016D','T2K part','T2K part',24834,'Reentered','1971-03-28 00:00:00','1971-03-09 00:00:00',97.16,186,1060,51.58,'Intermediate' from dual union all
select 504,'000504','62 B Tau 2','Injun 3','Injun 3',8180,'Reentered','1968-08-25 00:00:00','1963-01-16 00:00:00',116.03,231,2774,70.34,'Intermediate' from dual union all
select 5040,'005040','1971-016E','T2K part','T2K part',24834,'Reentered','1971-03-23 00:00:00','1971-03-07 00:00:00',97.15,184,1060,51.61,'Intermediate' from dual union all
select 5041,'005041','1971-018B','FG-02','FG-02',24836,'Reentered','1976-02-16 00:00:00','1971-04-19 00:00:00',105.69,268,1785,69.9,'Intermediate' from dual union all
select 5042,'005042','1971-015P','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-05 00:00:00',110.39,553,1937,65.66,'Intermediate' from dual union all
select 5043,'005043','1971-019A','IMP 6','IMP I',24837,'Deep Space','1971-03-14 00:00:00','1971-04-19 00:00:00',5975.14,1568,203824,30.2,'Deep Space' from dual union all
select 5044,'005044','1971-019B','Delta 83','DSV-3E-3 83',24837,'Reentered','1971-05-22 00:00:00','1971-04-12 00:00:00',91.62,233,475,28.74,'Intermediate' from dual union all
select 5045,'005045','1971-019C','Star 37D S/N 023','Star 37D',24837,'Deep Space','1971-03-14 00:00:00','1971-03-13 00:00:00',5956.08,352,204576,28.8,'Deep Space' from dual union all
select 5046,'005046','1971-019D','Delta 83 clamp band half','',24837,'Reentered','1971-04-19 00:00:00','1971-03-20 00:00:00',92.22,238,529,28.76,'Intermediate' from dual union all
select 5047,'005047','1971-017C','Blenda','BZA',24835,'Reentered','1971-03-25 00:00:00','1971-03-16 00:00:00',90.52,195,404,64.98,'Intermediate' from dual union all
select 5048,'005048','1971-017D','KDU','KDU',24835,'Reentered','1971-03-31 00:00:00','1971-03-16 00:00:00',90.6,193,415,65.01,'Intermediate' from dual union all
select 5049,'005049','1971-017E','PTDU cover','',24835,'Reentered','1971-03-23 00:00:00','1971-03-17 00:00:00',92.41,273,512,65,'Intermediate' from dual union all
select 505,'000505','62 B Phi','FTV 1156','CORONA M-17',8175,'Reentered Attached','1963-01-08 00:00:00','1962-12-23 00:00:00',90.08,195,361,70.98,'Intermediate' from dual union all
select 5050,'005050','1971-020A','Kosmos-400','DS-P1-M No. 3',24838,'In Earth orbit','','1971-04-18 00:00:00',104.96,981,1005,65.83,'Intermediate' from dual union all
select 5051,'005051','1971-020B','S3M','S3M',24838,'In Earth orbit','','1971-04-23 00:00:00',104.86,985,991,65.83,'Intermediate' from dual union all
select 5052,'005052','1971-020C','deb Kosmos-400','DS-P1-M part [RCS 0.54]',24838,'In Earth orbit','','1971-04-18 00:00:00',105,983,1006,65.82,'Intermediate' from dual union all
select 5053,'005053','1971-021A','OPS 4788','JUMPSEAT 1',24839,'In Earth orbit','','1971-03-21 00:00:00',700.58,182,39320,63.28,'Molniya' from dual union all
select 5054,'005054','1971-021B','Agena D','Agena D',24839,'In Earth orbit','','1971-05-05 00:00:00',700.58,182,39320,63.28,'Molniya' from dual union all
select 5055,'005055','1970-037C','deb Meteor','',21514,'Reentered','1979-02-26 00:00:00','1971-03-20 00:00:00',97.99,622,703,81.23,'Intermediate' from dual union all
select 5056,'005056','1965-082SW','deb Transtage 4','',11832,'Reentered','1971-05-03 00:00:00','1971-03-21 00:00:00',99.87,598,907,31.4,'Intermediate' from dual union all
select 5057,'005057','1965-082SX','deb Transtage 4','',11832,'Reentered','1978-05-26 00:00:00','1978-05-01 00:00:00',92.92,412,424,32.95,'Intermediate' from dual union all
select 5058,'005058','1965-082SY','deb Transtage 4','',11832,'Reentered','1981-03-07 00:00:00','1971-03-23 00:00:00',101.84,712,980,32.43,'Intermediate' from dual union all
select 5059,'005059','1971-022A','OPS 5300','CORONA CR-14',24840,'Reentered Attached','1971-04-12 00:00:00','1971-03-31 00:00:00',88.44,169,226,81.52,'Intermediate' from dual union all
select 506,'000506','62 B Chi','Explorer 16','NASA S-55B',8163,'In Earth orbit','','1963-01-22 00:00:00',104.36,759,1170,51.99,'Intermediate' from dual union all
select 5060,'005060','1970-025JE','deb Agena D','',21502,'In Earth orbit','','1992-02-02 00:00:00',102.73,761,1015,99.99,'Sun-Synch' from dual union all
select 5061,'005061','1970-025JF','deb Agena D','',21502,'In Earth orbit','','1992-02-07 00:00:00',102.56,836,923,99.73,'Sun-Synch' from dual union all
select 5062,'005062','1970-025JG','deb Agena D','',21502,'Reentered','2002-05-10 00:00:00','1992-02-05 00:00:00',101.75,799,884,99.69,'Sun-Synch' from dual union all
select 5063,'005063','1970-025JH','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',104.44,954,982,99.89,'Sun-Synch' from dual union all
select 5064,'005064','1970-025JJ','deb Agena D','',21502,'Reentered','2005-08-20 00:00:00','1992-02-06 00:00:00',102.12,813,905,99.7,'Sun-Synch' from dual union all
select 5065,'005065','1970-025JK','deb Agena D','',21502,'Reentered','1984-01-30 00:00:00','1984-01-29 00:00:00',91.04,314,338,99.4,'Sun-Synch' from dual union all
select 5066,'005066','1970-025JL','deb Agena D','',21502,'In Earth orbit','','1992-02-02 00:00:00',104.1,894,1011,99.84,'Sun-Synch' from dual union all
select 5067,'005067','1970-025JM','deb Agena D','',21502,'In Earth orbit','','1992-02-07 00:00:00',106.06,1038,1050,99.95,'Sun-Synch' from dual union all
select 5068,'005068','1970-089AF','deb Kosmos-374  [*]','',21566,'Reentered','1976-12-05 00:00:00','1971-03-24 00:00:00',111.09,518,2036,62.84,'Intermediate' from dual union all
select 5069,'005069','1970-089AG','deb Kosmos-374','',21566,'Reentered','1972-02-22 00:00:00','1971-03-23 00:00:00',108.15,393,1890,63.87,'Intermediate' from dual union all
select 507,'000507','62 B Tau 3','NRL PL121','POPPY 1 SV2',8180,'Reentered','1963-07-01 00:00:00','1963-01-14 00:00:00',112.94,232,2492,70.31,'Intermediate' from dual union all
select 5070,'005070','1970-089AH','deb Kosmos-374  [*]','',21566,'Reentered','2000-09-28 00:00:00','1992-02-07 00:00:00',101.54,560,1104,63.06,'Intermediate' from dual union all
select 5071,'005071','1970-089AJ','deb Kosmos-374','',21566,'Reentered','1978-10-02 00:00:00','1978-06-20 00:00:00',96.47,416,764,62.41,'Intermediate' from dual union all
select 5072,'005072','1970-089AK','deb Kosmos-374','',21566,'Reentered','1981-04-13 00:00:00','1971-03-21 00:00:00',109.94,517,1931,62.85,'Intermediate' from dual union all
select 5073,'005073','1970-089AL','deb Kosmos-374','',21566,'Reentered','1971-07-26 00:00:00','1971-03-24 00:00:00',106.44,348,1776,63.49,'Intermediate' from dual union all
select 5074,'005074','1970-089AM','deb Kosmos-374  [*]','',21566,'Reentered','1981-04-13 00:00:00','1971-03-24 00:00:00',116.32,548,2483,62.4,'Intermediate' from dual union all
select 5075,'005075','1970-089AN','deb Kosmos-374  [*]','',21566,'Reentered','1995-10-05 00:00:00','1992-02-06 00:00:00',97.73,503,798,63.03,'Intermediate' from dual union all
select 5076,'005076','1970-089AP','deb Kosmos-374','',21566,'In Earth orbit','','1992-02-06 00:00:00',112.71,566,2137,62.8,'Intermediate' from dual union all
select 5077,'005077','1970-089AQ','deb Kosmos-374','',21566,'Reentered','2000-11-06 00:00:00','1992-02-06 00:00:00',101.86,568,1126,62.84,'Intermediate' from dual union all
select 5078,'005078','1970-089AR','deb Kosmos-374','',21566,'Reentered','1972-04-22 00:00:00','1971-06-22 00:00:00',104.29,342,1581,63.51,'Intermediate' from dual union all
select 5079,'005079','1970-089AS','deb Kosmos-374','',21566,'Reentered','1979-03-26 00:00:00','1971-03-23 00:00:00',120.87,534,2906,62.85,'Medium' from dual union all
select 508,'000508','62 B Tau 4','SURCAL 2','SURCAL 2',8180,'Reentered','1966-01-18 00:00:00','1963-01-18 00:00:00',115.53,225,2734,70.34,'Intermediate' from dual union all
select 5080,'005080','1970-089AT','deb Kosmos-374  [*]','',21566,'Reentered','1981-06-29 00:00:00','1992-11-17 00:00:00',89.11,215,246,62.66,'Intermediate' from dual union all
select 5081,'005081','1970-089AU','deb Kosmos-374','',21566,'Reentered','1981-10-29 00:00:00','1981-06-22 00:00:00',97.43,467,805,62.91,'Intermediate' from dual union all
select 5082,'005082','1970-091AL','deb Kosmos-375','',21568,'Reentered','1988-08-29 00:00:00','1980-09-02 00:00:00',108.38,502,1801,65.82,'Intermediate' from dual union all
select 5083,'005083','1970-091AM','deb Kosmos-375','',21568,'Reentered','2005-01-23 00:00:00','1990-11-21 00:00:00',101.67,376,1300,62.84,'Intermediate' from dual union all
select 5084,'005084','1970-091AN','deb Kosmos-375','',21568,'Reentered','1982-11-22 00:00:00','1988-08-28 00:00:00',88.45,188,208,62.73,'Intermediate' from dual union all
select 5085,'005085','1965-082SZ','deb Transtage 4','',11832,'Reentered','1980-09-12 00:00:00','1971-03-25 00:00:00',104.51,706,1237,32.34,'Intermediate' from dual union all
select 5086,'005086','1971-023A','Kosmos-401','Zenit-4M',24841,'Landed','1971-04-09 00:00:00','1971-03-29 00:00:00',89.23,191,282,72.84,'Intermediate' from dual union all
select 5087,'005087','1971-023B','11A57I','11A57I',24841,'Reentered','1971-04-02 00:00:00','1971-03-29 00:00:00',89.21,194,277,72.84,'Intermediate' from dual union all
select 5088,'005088','1971-015Q','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-06 00:00:00',110.89,558,1978,65.7,'Intermediate' from dual union all
select 5089,'005089','1970-091AP','deb Kosmos-375','',21568,'Reentered','1976-12-05 00:00:00','1971-03-25 00:00:00',111.86,486,2139,63.69,'Intermediate' from dual union all
select 509,'000509','62 B Psi 1','Transit VA-1','Transit VA-1',8177,'Reentered','1986-09-25 00:00:00','1963-01-19 00:00:00',99.01,700,723,90.63,'Polar' from dual union all
select 5090,'005090','1971-015R','deb Kosmos-397','',24833,'Reentered','1989-10-10 00:00:00','1971-04-02 00:00:00',113.03,574,2158,65.72,'Intermediate' from dual union all
select 5091,'005091','1971-015S','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-06 00:00:00',108.23,561,1728,65.74,'Intermediate' from dual union all
select 5092,'005092','1971-015T','deb Kosmos-397','',24833,'Reentered','1988-05-15 00:00:00','1988-05-14 00:00:00',89.68,249,269,65.62,'Intermediate' from dual union all
select 5093,'005093','1970-091AQ','deb Kosmos-375','',21568,'Reentered','1999-10-26 00:00:00','1992-02-06 00:00:00',101.06,480,1138,62.73,'Intermediate' from dual union all
select 5094,'005094','1971-015U','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-07 00:00:00',109.28,552,1835,65.72,'Intermediate' from dual union all
select 5095,'005095','1971-015V','deb Kosmos-397','',24833,'Reentered','1992-04-18 00:00:00','1992-02-07 00:00:00',96.24,457,700,65.53,'Intermediate' from dual union all
select 5096,'005096','1970-089AV','deb Kosmos-374  [*]','',21566,'Reentered','1979-05-01 00:00:00','1979-01-20 00:00:00',95.81,346,770,64.76,'Intermediate' from dual union all
select 5097,'005097','1971-015W','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-06 00:00:00',110.47,562,1935,65.68,'Intermediate' from dual union all
select 5098,'005098','1971-015X','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-04 00:00:00',110,564,1890,65.85,'Intermediate' from dual union all
select 5099,'005099','1971-015Y','deb Kosmos-397','',24833,'Reentered','1990-04-27 00:00:00','1990-04-26 00:00:00',90.42,266,325,65.61,'Intermediate' from dual union all
select 51,'000051','60 Iota 3','A-11 Canister half','A-11',6323,'In Earth orbit','','1960-09-18 00:00:00',118.23,1514,1689,47.2,'Intermediate' from dual union all
select 510,'000510','62 B Alpha 3','deb Alouette 1','',8161,'In Earth orbit','','1962-12-15 00:00:00',105.27,986,1029,80.46,'Intermediate' from dual union all
select 5100,'005100','1971-015Z','deb Kosmos-397','',24833,'Reentered','1982-02-10 00:00:00','1982-02-09 00:00:00',91.28,322,352,65.55,'Intermediate' from dual union all
select 5101,'005101','1971-015AA','deb Kosmos-397','',24833,'Reentered','1989-08-24 00:00:00','1989-08-24 00:00:00',89.25,219,256,65.62,'Intermediate' from dual union all
select 5102,'005102','1971-015AB','deb Kosmos-397','',24833,'Reentered','1991-11-09 00:00:00','1991-11-09 00:00:00',88.81,205,227,65.6,'Intermediate' from dual union all
select 5103,'005103','1971-015AC','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-05 00:00:00',105.33,543,1477,65.78,'Intermediate' from dual union all
select 5104,'005104','1971-024A','Isis 2','Isis B',24842,'In Earth orbit','','1971-05-02 00:00:00',113.6,1356,1428,88.15,'Polar' from dual union all
select 5105,'005105','1971-025A','Kosmos-402','US-A reactor',24843,'In Earth orbit','','1971-05-03 00:00:00',104.92,948,1034,64.99,'Intermediate' from dual union all
select 5106,'005106','1971-024B','FW4D','FW4D',24842,'In Earth orbit','','1971-05-06 00:00:00',113.55,1355,1424,88.17,'Polar' from dual union all
select 5107,'005107','1971-025B','Kosmos-402 DU','US-A DU',24843,'Reentered','1971-05-06 00:00:00','1971-04-11 00:00:00',89.39,236,252,64.97,'Intermediate' from dual union all
select 5108,'005108','1971-026A','Kosmos-403','Zenit-2M',24844,'Landed','1971-04-14 00:00:00','1971-04-04 00:00:00',88.88,212,227,81.34,'Intermediate' from dual union all
select 5109,'005109','1971-025C','deb Kosmos-402','',24843,'Reentered','1971-04-06 00:00:00','1971-04-02 00:00:00',89.45,238,257,64.97,'Intermediate' from dual union all
select 511,'000511','62 B Alpha 4','deb Alouette 1','',8161,'In Earth orbit','','1962-12-13 00:00:00',105.4,983,1044,80.44,'Intermediate' from dual union all
select 5110,'005110','1971-025D','Kosmos-402 RLS','',24843,'Reentered','1971-04-05 00:00:00','1971-04-03 00:00:00',88.94,214,231,64.98,'Intermediate' from dual union all
select 5111,'005111','1971-026B','11A57I','11A57I',24844,'Reentered','1971-04-05 00:00:00','1971-04-04 00:00:00',88.39,182,208,81.33,'Intermediate' from dual union all
select 5112,'005112','1971-019E','Delta 83 clamp band half','',24837,'Reentered','1971-04-19 00:00:00','1971-04-01 00:00:00',91.51,234,464,28.75,'Intermediate' from dual union all
select 5113,'005113','1971-027A','Kosmos-404','I2P',24845,'Deorbited','1971-04-04 00:00:00','1971-04-04 00:00:00',103.09,801,1009,65.74,'Intermediate' from dual union all
select 5114,'005114','1971-027B','11S692','11S692',24845,'Reentered','1971-04-12 00:00:00','1971-04-06 00:00:00',92.01,144,602,65.07,'Intermediate' from dual union all
select 5115,'005115','1971-027C','deb Kosmos-404','',24845,'Reentered','1971-04-06 00:00:00','1971-04-05 00:00:00',90.44,135,458,65.26,'Intermediate' from dual union all
select 5116,'005116','1971-027D','deb Kosmos-404','',24845,'Reentered','1971-04-07 00:00:00','1971-04-06 00:00:00',91.41,131,557,65.03,'Intermediate' from dual union all
select 5117,'005117','1971-028A','Kosmos-405','Tselina-D',24846,'In Earth orbit','','1971-05-10 00:00:00',98.27,672,680,81.23,'Intermediate' from dual union all
select 5118,'005118','1971-028B','8A92ME','8A92ME',24846,'In Earth orbit','','1971-05-07 00:00:00',98.35,613,747,81.26,'Intermediate' from dual union all
select 5119,'005119','1971-023C','KDU','KDU',24841,'Reentered','1971-04-16 00:00:00','1971-04-09 00:00:00',90.07,183,372,72.8,'Intermediate' from dual union all
select 512,'000512','62 B Iota 24','deb Sputnik','',8168,'Reentered','1962-12-31 00:00:00','1962-12-18 00:00:00',90.65,172,441,64.92,'Intermediate' from dual union all
select 5120,'005120','1971-023D','Blenda','BZA',24841,'Reentered','1971-04-12 00:00:00','1971-04-09 00:00:00',89.79,182,346,72.84,'Intermediate' from dual union all
select 5121,'005121','1971-023E','PTDU cover','',24841,'Reentered','1971-04-12 00:00:00','1971-04-10 00:00:00',90.85,196,437,72.88,'Intermediate' from dual union all
select 5122,'005122','1967-001F','deb unknown                       ','[UNID034]',15256,'Reentered','1972-04-30 00:00:00','1971-04-15 00:00:00',125.17,286,3536,26.48,'Medium' from dual union all
select 5123,'005123','1967-001G','deb unknown                       ','[UNID035]',15256,'Reentered','1971-10-06 00:00:00','1971-04-13 00:00:00',116.59,229,2826,26.33,'Intermediate' from dual union all
select 5124,'005124','1971-029A','Kosmos-406','Zenit-4M',24847,'Landed','1971-04-24 00:00:00','1971-04-17 00:00:00',89.02,214,239,81.31,'Intermediate' from dual union all
select 5125,'005125','1971-029B','11A57I','11A57I',24847,'Reentered','1971-04-17 00:00:00','1971-04-16 00:00:00',88.49,193,207,81.32,'Intermediate' from dual;
insert into SATELLITE
select 5126,'005126','1971-011B','M-40','M-40',24829,'In Earth orbit','','1971-03-31 00:00:00',104.79,976,994,29.68,'Intermediate' from dual union all
select 5127,'005127','1971-029C','Sensor cover','Zenit-4M sensor cover',24847,'Reentered','1971-04-14 00:00:00','1971-04-14 00:00:00',88.56,168,239,81.43,'Intermediate' from dual union all
select 5128,'005128','1971-030A','Tournesol','D-2A',24848,'Reentered','1980-01-28 00:00:00','1971-05-15 00:00:00',96.19,457,696,46.37,'Intermediate' from dual union all
select 5129,'005129','1970-025JN','deb Agena D','',21502,'In Earth orbit','','1989-11-18 00:00:00',108.75,987,1351,99.31,'Sun-Synch' from dual union all
select 513,'000513','62 B Tau 5','Calsphere 1','Calsphere 1',8180,'Reentered','1967-02-05 00:00:00','1963-01-19 00:00:00',115.75,231,2748,70.34,'Intermediate' from dual union all
select 5130,'005130','1971-015AD','deb Kosmos-397','',24833,'Reentered','1980-10-29 00:00:00','1980-10-27 00:00:00',91.74,312,408,65.49,'Intermediate' from dual union all
select 5131,'005131','1971-015AE','deb Kosmos-397','',24833,'Reentered','2013-11-27 00:00:00','1992-02-07 00:00:00',105.2,543,1465,65.7,'Intermediate' from dual union all
select 5132,'005132','1970-089AW','deb Kosmos-374  [*]','',21566,'Reentered','1982-11-10 00:00:00','1971-04-14 00:00:00',111.49,518,2073,62.84,'Intermediate' from dual union all
select 5133,'005133','1970-025JP','deb Agena D','',21502,'Reentered','2012-11-15 00:00:00','1992-02-07 00:00:00',102.38,830,912,99.72,'Sun-Synch' from dual union all
select 5134,'005134','1969-061E','deb Molniya-1','',18328,'Reentered','1971-05-04 00:00:00','1971-04-15 00:00:00',224.46,135,11577,65.09,'Medium' from dual union all
select 5135,'005135','1971-030B','P.068','P068',24848,'Reentered','1979-07-09 00:00:00','1971-05-24 00:00:00',96.12,449,697,46.37,'Intermediate' from dual union all
select 5136,'005136','1970-089AX','deb Kosmos-374  [*]','',21566,'Reentered','1981-11-09 00:00:00','1981-04-11 00:00:00',101.55,483,1181,62.83,'Intermediate' from dual union all
select 5137,'005137','1971-015AF','deb Kosmos-397','',24833,'Reentered','2003-05-26 00:00:00','1992-02-07 00:00:00',102.36,545,1196,65.72,'Intermediate' from dual union all
select 5138,'005138','1970-089AY','deb Kosmos-374  [*]','',21566,'Reentered','1972-03-24 00:00:00','1972-03-07 00:00:00',94.77,380,634,62.01,'Intermediate' from dual union all
select 5139,'005139','1971-030C','deb Tournesol','',24848,'Reentered','1972-07-15 00:00:00','1971-05-15 00:00:00',95.99,455,677,46.35,'Intermediate' from dual union all
select 514,'000514','62 B Psi 2','Transit VA-1 despin weight','',8177,'Reentered','1969-04-22 00:00:00','1963-02-14 00:00:00',97.8,575,731,90.74,'Polar' from dual union all
select 5140,'005140','1971-030D','deb Tournesol','',24848,'Reentered','1977-07-15 00:00:00','1971-05-15 00:00:00',96.23,456,700,46.39,'Intermediate' from dual union all
select 5141,'005141','1971-030E','deb Tournesol','',24848,'Reentered','1976-04-15 00:00:00','1971-05-15 00:00:00',96.18,457,695,45.64,'Intermediate' from dual union all
select 5142,'005142','1971-031A','Meteor','Meteor-M No. 21 (71)',24849,'Reentered','1991-01-10 00:00:00','1971-05-21 00:00:00',97.09,607,632,81.24,'Intermediate' from dual union all
select 5143,'005143','1971-031B','8A92ME','8A92ME',24849,'Reentered','2001-04-19 00:00:00','1971-05-21 00:00:00',97.29,549,709,81.25,'Intermediate' from dual union all
select 5144,'005144','1971-015AG','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-06 00:00:00',108.56,562,1759,65.74,'Intermediate' from dual union all
select 5145,'005145','1971-015AH','deb Kosmos-397','',24833,'Reentered','1998-07-29 00:00:00','1992-02-07 00:00:00',100.03,512,1008,65.53,'Intermediate' from dual union all
select 5146,'005146','1970-025JQ','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.91,1003,1071,99.92,'Sun-Synch' from dual union all
select 5147,'005147','1969-082HL','deb Agena D','',18349,'Reentered','1995-08-09 00:00:00','1992-02-06 00:00:00',97.98,617,707,69.85,'Intermediate' from dual union all
select 5148,'005148','1970-025JR','deb Agena D','',21502,'Reentered','2003-03-28 00:00:00','1992-02-07 00:00:00',101.92,806,894,99.72,'Sun-Synch' from dual union all
select 5149,'005149','1970-025JS','deb Agena D','',21502,'In Earth orbit','','1992-02-07 00:00:00',103.5,866,983,99.59,'Sun-Synch' from dual union all
select 515,'000515','62 B Upsilon 2','Altair SV-22','Altair X-248-A5DM',8182,'In Earth orbit','','1963-02-03 00:00:00',184.86,1346,7397,47.45,'Medium' from dual union all
select 5150,'005150','1970-025JT','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.34,927,1093,99.61,'Sun-Synch' from dual union all
select 5151,'005151','1970-025JU','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.17,1049,1050,100.02,'Sun-Synch' from dual union all
select 5152,'005152','1970-025JV','deb Agena D','',21502,'Reentered','1991-03-13 00:00:00','1991-03-12 00:00:00',90.67,290,324,99.56,'Retrograde' from dual union all
select 5153,'005153','1970-025JW','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.71,1048,1101,99.92,'Sun-Synch' from dual union all
select 5154,'005154','1970-025JX','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',103.97,942,951,99.87,'Sun-Synch' from dual union all
select 5155,'005155','1971-015AJ','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-07 00:00:00',106.87,556,1608,65.38,'Intermediate' from dual union all
select 5156,'005156','1971-015AK','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-06 00:00:00',109.48,550,1856,65.7,'Intermediate' from dual union all
select 5157,'005157','1971-015AL','deb Kosmos-397','',24833,'Reentered','2012-07-31 00:00:00','1992-02-06 00:00:00',106.77,539,1616,65.3,'Intermediate' from dual union all
select 5158,'005158','1969-082HM','deb Agena D','',18349,'Reentered','1979-08-07 00:00:00','1979-06-19 00:00:00',95.89,546,578,70.08,'Intermediate' from dual union all
select 5159,'005159','1969-082HN','deb Agena D','',18349,'Reentered','1972-12-30 00:00:00','1972-09-20 00:00:00',95.83,543,575,70.02,'Intermediate' from dual union all
select 516,'000516','61 Omicron 162','deb r/b Ablestar','',7273,'In Earth orbit','','1962-12-20 00:00:00',104.35,908,1020,66.03,'Intermediate' from dual union all
select 5160,'005160','1971-032A','Salyut','DOS-1 17K No. 121',24850,'Deorbited','1971-10-11 00:00:00','1971-05-22 00:00:00',89.07,223,233,51.57,'Intermediate' from dual union all
select 5161,'005161','1971-032B','8S812','8S812',24850,'Reentered','1971-04-20 00:00:00','1971-04-19 00:00:00',88.43,180,214,51.4,'Intermediate' from dual union all
select 5162,'005162','1971-032C','deb Salyut','',24850,'Reentered','1971-04-20 00:00:00','1971-04-20 00:00:00',88.53,202,202,51.51,'Intermediate' from dual union all
select 5163,'005163','1971-032D','deb Salyut','',24850,'Reentered','1971-04-21 00:00:00','1971-04-20 00:00:00',88.07,179,179,51.55,'Intermediate' from dual union all
select 5164,'005164','1971-030F','deb Tournesol','',24848,'Reentered','1977-02-03 00:00:00','1971-05-19 00:00:00',96.13,457,689,46.34,'Intermediate' from dual union all
select 5165,'005165','1970-025JY','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',102.57,840,920,99.74,'Sun-Synch' from dual union all
select 5166,'005166','1971-032E','deb Salyut','',24850,'Reentered','1971-04-24 00:00:00','1971-04-21 00:00:00',88.99,190,259,51.62,'Intermediate' from dual union all
select 5167,'005167','1971-032F','deb Salyut','',24850,'Reentered','1971-04-21 00:00:00','1971-04-20 00:00:00',87.76,160,168,51.55,'Intermediate' from dual union all
select 5168,'005168','1971-032G','deb Salyut','',24850,'Reentered','1971-04-21 00:00:00','1971-04-21 00:00:00',88.34,161,224,51.3,'Intermediate' from dual union all
select 5169,'005169','1971-029D','Blenda','BZA',24847,'Reentered','1971-04-24 00:00:00','1971-04-21 00:00:00',88.79,203,227,81.34,'Intermediate' from dual union all
select 517,'000517','62 B Omega 1','Kosmos-12','Zenit-2 No. 6',8173,'Landed','1962-12-30 00:00:00','1962-12-24 00:00:00',90.37,199,387,64.92,'Intermediate' from dual union all
select 5170,'005170','1971-029E','KDU','KDU',24847,'Reentered','1971-04-28 00:00:00','1971-04-23 00:00:00',88.7,204,217,81.34,'Intermediate' from dual union all
select 5171,'005171','1971-033A','OPS 7899','GAMBIT-3 Mission 4331',24851,'Reentered Attached','1971-05-13 00:00:00','1971-05-01 00:00:00',89.63,131,382,110.92,'Retrograde' from dual union all
select 5172,'005172','1971-034A','Soyuz-10','Soyuz 7K-T No. 31',24852,'Landed','1971-04-24 00:00:00','1971-04-23 00:00:00',88.85,204,232,51.57,'Intermediate' from dual union all
select 5173,'005173','1971-034B','11S510','11S510',24852,'Reentered','1971-04-25 00:00:00','1971-04-24 00:00:00',88.48,194,205,51.59,'Intermediate' from dual union all
select 5174,'005174','1971-035A','Kosmos-407','Strela-2M',24853,'In Earth orbit','','1971-05-24 00:00:00',100.93,789,816,74.05,'Intermediate' from dual union all
select 5175,'005175','1971-035B','S3M','S3M',24853,'In Earth orbit','','1971-05-27 00:00:00',100.85,777,820,74.05,'Intermediate' from dual union all
select 5176,'005176','1971-036A','San Marco 3','San Marco C',24854,'Reentered','1971-11-29 00:00:00','1971-05-30 00:00:00',93.6,224,677,3.23,'Equatorial' from dual union all
select 5177,'005177','1971-037A','Kosmos-408','DS-P1-Yu No. 37',24855,'Reentered','1971-12-29 00:00:00','1971-05-26 00:00:00',100.66,197,1383,81.82,'Intermediate' from dual union all
select 5178,'005178','1971-037B','S1M','S1M',24855,'Reentered','1971-09-29 00:00:00','1971-05-27 00:00:00',99.82,201,1299,81.81,'Intermediate' from dual union all
select 5179,'005179','1971-029F','PTDU cover','',24847,'Reentered','1971-04-26 00:00:00','1971-04-24 00:00:00',90.41,207,383,81.32,'Intermediate' from dual union all
select 518,'000518','62 B Omega 2','8A92E','8A92E',8173,'Reentered','1963-01-22 00:00:00','1963-01-02 00:00:00',90.15,198,366,64.93,'Intermediate' from dual union all
select 5180,'005180','1971-038A','Kosmos-409','Sfera',24856,'In Earth orbit','','1971-05-29 00:00:00',109.3,1176,1212,74,'Intermediate' from dual union all
select 5181,'005181','1971-038B','S3M','S3M',24856,'In Earth orbit','','1971-05-31 00:00:00',109.19,1170,1208,73.99,'Intermediate' from dual union all
select 5182,'005182','1963-014CN','deb West Ford','RCS 0.08',9245,'Reentered','1987-10-17 00:00:00','1967-07-14 00:00:00',165.43,2028,5180,86.79,'Medium' from dual union all
select 5183,'005183','1970-025JZ','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.42,987,1042,100.11,'Sun-Synch' from dual union all
select 5184,'005184','1970-025KA','deb Agena D','',21502,'Reentered','2014-04-14 00:00:00','1992-02-07 00:00:00',101.77,755,930,99.38,'Sun-Synch' from dual union all
select 5185,'005185','1970-025KB','deb Agena D','',21502,'In Earth orbit','','1992-02-07 00:00:00',105.82,963,1103,99.64,'Sun-Synch' from dual union all
select 5186,'005186','1970-025KC','deb Agena D','',21502,'Reentered','2014-06-13 00:00:00','1992-02-04 00:00:00',102.41,821,925,100.01,'Sun-Synch' from dual union all
select 5187,'005187','1971-015AM','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-07 00:00:00',109.33,554,1838,65.67,'Intermediate' from dual union all
select 5188,'005188','1970-089AZ','deb Kosmos-374','',21566,'In Earth orbit','','1990-11-22 00:00:00',105.04,633,1359,62.86,'Intermediate' from dual union all
select 5189,'005189','1970-089BA','deb Kosmos-374  [*]','',21566,'Reentered','1982-11-10 00:00:00','1971-05-02 00:00:00',111.58,512,2087,62.64,'Intermediate' from dual union all
select 519,'000519','62 B Psi 3','Altair X-248-A5 BSB-240','X-248',8177,'Reentered','1989-10-21 00:00:00','1963-05-19 00:00:00',99,696,726,90.62,'Polar' from dual union all
select 5190,'005190','1970-025KD','deb Agena D','',21502,'Reentered','1979-11-10 00:00:00','1979-08-25 00:00:00',96.37,497,673,100.03,'Sun-Synch' from dual union all
select 5191,'005191','1971-015AN','deb Kosmos-397','',24833,'Reentered','1993-01-28 00:00:00','1992-02-06 00:00:00',98.52,485,891,65.79,'Intermediate' from dual union all
select 5192,'005192','1971-015AP','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-07 00:00:00',106.2,555,1546,65.6,'Intermediate' from dual union all
select 5193,'005193','1971-015AQ','deb Kosmos-397','',24833,'Reentered','1980-03-08 00:00:00','1971-05-19 00:00:00',111.13,547,2011,65.46,'Intermediate' from dual union all
select 5194,'005194','1971-015AR','deb Kosmos-397','',24833,'Reentered','1980-05-25 00:00:00','1971-05-04 00:00:00',113.74,578,2219,65.71,'Intermediate' from dual union all
select 5195,'005195','1969-082HP','deb Agena D','',18349,'Reentered','1981-06-15 00:00:00','1971-05-02 00:00:00',102,699,1007,69.84,'Intermediate' from dual union all
select 5196,'005196','1971-015AS','deb Kosmos-397','',24833,'Reentered','1990-10-22 00:00:00','1971-05-04 00:00:00',112.89,578,2141,65.85,'Intermediate' from dual union all
select 5197,'005197','1971-015AT','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-07 00:00:00',110.72,573,1947,65.58,'Intermediate' from dual union all
select 5198,'005198','1970-089BB','deb Kosmos-374','',21566,'Reentered','2001-11-07 00:00:00','1992-02-07 00:00:00',101.89,478,1219,62.93,'Intermediate' from dual union all
select 5199,'005199','1970-089BC','deb Kosmos-374  [*]','',21566,'Reentered','1976-12-05 00:00:00','1971-05-03 00:00:00',111.21,516,2049,62.8,'Intermediate' from dual union all
select 52,'000052','60 Iota 4','Mylar piece','',6323,'Lost','','1976-02-28 00:00:00',118.24,1490,1714,47.15,'Intermediate' from dual union all
select 520,'000520','62 B Tau 6','WILD BILL 2','AFTRACK 27/Mission 7202',8180,'Reentered Attached','1968-03-24 00:00:00','1963-01-22 00:00:00',115.98,238,2762,70.34,'Intermediate' from dual union all
select 5200,'005200','1971-015AU','deb Kosmos-397','',24833,'Reentered','1991-02-13 00:00:00','1991-02-12 00:00:00',90.46,274,321,66.15,'Intermediate' from dual union all
select 5201,'005201','1971-015AV','deb Kosmos-397','',24833,'Reentered','1977-08-09 00:00:00','1971-05-02 00:00:00',112.81,573,2139,65.74,'Intermediate' from dual union all
select 5202,'005202','1970-089BD','deb Kosmos-374','',21566,'Reentered','1977-08-23 00:00:00','1977-07-19 00:00:00',92.25,328,442,62.67,'Intermediate' from dual union all
select 5203,'005203','1970-025KE','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',110.2,1059,1413,99.85,'Sun-Synch' from dual union all
select 5204,'005204','1971-039A','DSP F2','DSP 3',24857,'In Earth orbit','','1979-10-31 00:00:00',1436.11,35776,35797,6.56,'Inclined GEO' from dual union all
select 5205,'005205','1971-039B','Transtage 20','Transtage 20',24857,'In Earth orbit','','1971-06-01 00:00:00',1431.67,35600,35800,2,'Drift GEO' from dual union all
select 5206,'005206','1971-039C','Titan 23C-2 Stage 2','LR91AJ11 23C-2',24857,'Reentered','1971-05-05 00:00:00','1971-05-05 00:00:00',89.5,150,350,28.6,'Intermediate' from dual union all
select 5207,'005207','1971-040A','Kosmos-410','Zenit-2M',24858,'Landed','1971-05-18 00:00:00','1971-05-08 00:00:00',89.29,202,277,64.97,'Intermediate' from dual union all
select 5208,'005208','1971-040B','11A57I','11A57I',24858,'Reentered','1971-05-11 00:00:00','1971-05-08 00:00:00',88.95,198,248,64.95,'Intermediate' from dual union all
select 5209,'005209','1971-030G','deb Tournesol','',24848,'Reentered','1972-07-13 00:00:00','1971-05-15 00:00:00',96.03,455,682,46.36,'Intermediate' from dual union all
select 521,'000521','1963-001A','8K78I','8K78I',9232,'Reentered','1963-01-05 00:00:00','1963-01-04 00:00:00',87.52,152,152,64.63,'Intermediate' from dual union all
select 5210,'005210','1971-041A','Kosmos-411','Strela-1M',24859,'In Earth orbit','','1971-06-13 00:00:00',113.85,1317,1490,74.01,'Intermediate' from dual union all
select 5211,'005211','1971-041B','Kosmos-412','Strela-1M',24859,'In Earth orbit','','1971-06-06 00:00:00',116.14,1480,1535,74.02,'Intermediate' from dual union all
select 5212,'005212','1971-041C','Kosmos-413','Strela-1M',24859,'In Earth orbit','','1971-06-06 00:00:00',115.78,1476,1506,74.02,'Intermediate' from dual union all
select 5213,'005213','1971-041D','Kosmos-414','Strela-1M',24859,'In Earth orbit','','1971-06-07 00:00:00',115.11,1426,1495,74.02,'Intermediate' from dual union all
select 5214,'005214','1971-041E','Kosmos-415','Strela-1M',24859,'In Earth orbit','','1971-06-08 00:00:00',115.45,1451,1501,74.02,'Intermediate' from dual union all
select 5215,'005215','1971-041F','Kosmos-416','Strela-1M',24859,'In Earth orbit','','1971-06-06 00:00:00',114.48,1372,1491,74.02,'Intermediate' from dual;
insert into SATELLITE
select 5216,'005216','1971-041G','Kosmos-417','Strela-1M',24859,'In Earth orbit','','1971-06-11 00:00:00',114.17,1343,1493,74.02,'Intermediate' from dual union all
select 5217,'005217','1971-041H','Kosmos-418','Strela-1M',24859,'In Earth orbit','','1971-06-07 00:00:00',114.8,1401,1491,74.04,'Intermediate' from dual union all
select 5218,'005218','1971-041J','S3M','S3M',24859,'In Earth orbit','','1971-06-06 00:00:00',116.81,1485,1590,74.03,'Intermediate' from dual union all
select 5219,'005219','1970-025KF','deb Agena D','',21502,'Reentered','1988-10-28 00:00:00','1988-10-28 00:00:00',91.65,329,382,99.92,'Retrograde' from dual union all
select 522,'000522','1963-001B','[Luna-4]','E-6 No. 2',9232,'Reentered','1963-01-11 00:00:00','1963-01-07 00:00:00',88.21,178,194,64.75,'Intermediate' from dual union all
select 5220,'005220','1970-025KG','deb Agena D','',21502,'In Earth orbit','','1992-02-02 00:00:00',112.5,1048,1634,99.86,'Sun-Synch' from dual union all
select 5221,'005221','1971-042A','Kosmos-419','3MS No. 170',24860,'Reentered Attached','1971-05-12 00:00:00','1971-05-11 00:00:00',87.65,126,191,51.46,'Intermediate' from dual union all
select 5222,'005222','1971-015AW','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-06 00:00:00',109.61,569,1849,65.74,'Intermediate' from dual union all
select 5223,'005223','1968-097CH','deb Kosmos-252  [*]','',16824,'Reentered','1983-05-02 00:00:00','1971-05-15 00:00:00',110.35,530,1956,62.36,'Intermediate' from dual union all
select 5224,'005224','1970-089BE','deb Kosmos-374','',21566,'Reentered','1978-12-22 00:00:00','1978-10-01 00:00:00',101.19,334,1296,63.41,'Intermediate' from dual union all
select 5225,'005225','1970-102D','deb Kosmos-381','',21579,'In Earth orbit','','1971-05-14 00:00:00',104.65,965,992,74.04,'Intermediate' from dual union all
select 5226,'005226','1970-025KH','deb Agena D','',21502,'Reentered','2003-09-23 00:00:00','1992-02-07 00:00:00',102.06,814,899,99.72,'Sun-Synch' from dual union all
select 5227,'005227','1971-013E','deb Kosmos-395','Tselina-O part',24831,'Reentered','1971-09-06 00:00:00','1971-05-17 00:00:00',95.05,512,530,74.06,'Intermediate' from dual union all
select 5228,'005228','1971-040C','Kapsula Kosmosa-410','Nauka 8KS No. 1',24858,'Reentered','1971-05-25 00:00:00','1971-05-18 00:00:00',89.01,195,257,64.94,'Intermediate' from dual union all
select 5229,'005229','1971-040D','Nauka part','',24858,'Reentered','1971-05-19 00:00:00','1971-05-17 00:00:00',88.87,196,242,64.97,'Intermediate' from dual union all
select 523,'000523','62 B Psi 4','Transit VA-1 despin weight','',8177,'Reentered','1980-11-30 00:00:00','1963-01-29 00:00:00',100.16,698,835,90.49,'Polar' from dual union all
select 5230,'005230','1971-043A','Kosmos-420','Zenit-4M',24861,'Landed','1971-05-29 00:00:00','1971-05-21 00:00:00',89.02,200,252,51.76,'Intermediate' from dual union all
select 5231,'005231','1971-043B','11A57I','11A57I',24861,'Reentered','1971-05-21 00:00:00','1971-05-20 00:00:00',88.12,169,193,51.76,'Intermediate' from dual union all
select 5232,'005232','1971-044A','Kosmos-421','DS-P1-Yu No. 48',24862,'Reentered','1971-11-08 00:00:00','1971-06-21 00:00:00',91.65,266,445,70.98,'Intermediate' from dual union all
select 5233,'005233','1971-044B','S1M','S1M',24862,'Reentered','1971-08-23 00:00:00','1971-06-18 00:00:00',91.28,264,411,70.99,'Intermediate' from dual union all
select 5234,'005234','1971-045A','Mars-2','4M No. 171',24863,'Deep Space','1971-05-20 00:00:00','',null,null,null,null,'' from dual union all
select 5235,'005235','1971-045B','SOZ','SOZ',24863,'Reentered','1971-05-21 00:00:00','1971-05-20 00:00:00',87.56,136,171,51.52,'Intermediate' from dual union all
select 5236,'005236','1971-045C','SOZ','SOZ',24863,'Reentered','1971-05-21 00:00:00','1971-05-20 00:00:00',87.58,137,173,51.52,'Intermediate' from dual union all
select 5237,'005237','61 Delta 8','deb Explorer IX','',7263,'Reentered','1974-07-20 00:00:00','1971-07-31 00:00:00',115.65,581,2389,38.8,'Intermediate' from dual union all
select 5238,'005238','1971-046A','Kosmos-422','Tsiklon',24864,'In Earth orbit','','1971-06-23 00:00:00',105.06,986,1009,74.02,'Intermediate' from dual union all
select 5239,'005239','1971-046B','S3M','S3M',24864,'In Earth orbit','','1971-06-23 00:00:00',104.98,981,1006,74.03,'Intermediate' from dual union all
select 524,'000524','1963-001C','BOZ','BOZ',9232,'Reentered','1963-01-11 00:00:00','1963-01-07 00:00:00',88.53,178,226,64.75,'Intermediate' from dual union all
select 5240,'005240','1970-089BF','deb Kosmos-397','',21566,'In Earth orbit','','1992-02-05 00:00:00',113.95,641,2175,62.86,'Intermediate' from dual union all
select 5241,'005241','1971-015AX','deb Kosmos-374','',24833,'Reentered','1983-07-12 00:00:00','1971-06-01 00:00:00',112.02,578,2061,65.9,'Intermediate' from dual union all
select 5242,'005242','1970-089BG','deb Kosmos-374','',21566,'Reentered','1992-11-17 00:00:00','1992-02-07 00:00:00',97.41,448,822,62.7,'Intermediate' from dual union all
select 5243,'005243','1971-015AY','deb Kosmos-397','',24833,'Reentered','1988-12-15 00:00:00','1988-12-14 00:00:00',89.25,216,259,65.53,'Intermediate' from dual union all
select 5244,'005244','1971-015AZ','deb Kosmos-397','',24833,'Reentered','1996-03-07 00:00:00','1992-02-06 00:00:00',98.15,475,866,65.94,'Intermediate' from dual union all
select 5245,'005245','1967-001H','deb unknown                       ','[UNID036]',15256,'Reentered','1971-06-17 00:00:00','1971-05-22 00:00:00',207.76,142,10342,26.07,'Medium' from dual union all
select 5246,'005246','1971-047A','Kosmos-423','DS-P1-Yu No. 47',24865,'Reentered','1971-11-26 00:00:00','1971-06-27 00:00:00',91.84,268,462,71.03,'Intermediate' from dual union all
select 5247,'005247','1971-047B','S1M','S1M',24865,'Reentered','1971-08-29 00:00:00','1971-06-26 00:00:00',91.33,265,415,71.03,'Intermediate' from dual union all
select 5248,'005248','1971-048A','Kosmos-424','Zenit-4M',24866,'Landed','1971-06-10 00:00:00','1971-05-30 00:00:00',89.34,198,286,65.37,'Intermediate' from dual union all
select 5249,'005249','1971-048B','11A57I','11A57I',24866,'Reentered','1971-06-03 00:00:00','1971-05-30 00:00:00',89.14,195,269,65.41,'Intermediate' from dual union all
select 525,'000525','1963-002A','OPS 0048','CORONA M-18',9233,'Reentered Attached','1963-01-24 00:00:00','1963-01-17 00:00:00',89.4,178,312,82.24,'Intermediate' from dual union all
select 5250,'005250','1971-043C','KDU','KDU',24861,'Reentered','1971-06-03 00:00:00','1971-05-28 00:00:00',88.85,197,239,51.78,'Intermediate' from dual union all
select 5251,'005251','1971-043D','Blenda','BZA',24861,'Reentered','1971-05-30 00:00:00','1971-05-28 00:00:00',88.74,193,232,51.74,'Intermediate' from dual union all
select 5252,'005252','1971-049A','Mars-3','4M No. 172',24867,'Deep Space','1971-05-29 00:00:00','',null,null,null,null,'' from dual union all
select 5253,'005253','1971-050A','Kosmos-425','Tselina-OM',24868,'Reentered','1980-01-15 00:00:00','1971-06-28 00:00:00',95.18,507,548,74.03,'Intermediate' from dual union all
select 5254,'005254','1971-050B','S3M','S3M',24868,'Reentered','1979-11-28 00:00:00','1971-06-28 00:00:00',95.13,492,558,74.03,'Intermediate' from dual union all
select 5255,'005255','1971-049B','SOZ','SOZ',24867,'Reentered','1971-05-31 00:00:00','1971-05-30 00:00:00',87.61,122,191,51.77,'Intermediate' from dual union all
select 5256,'005256','1971-049C','SOZ','SOZ',24867,'Reentered','1971-05-31 00:00:00','1971-05-29 00:00:00',88.23,172,202,51.49,'Intermediate' from dual union all
select 5257,'005257','1971-050C','deb Kosmos-425','Tselina-O part',24868,'Reentered','1972-03-22 00:00:00','1971-07-04 00:00:00',94.77,470,545,74.06,'Intermediate' from dual union all
select 5258,'005258','1971-050D','deb Kosmos-425','Tselina-O part',24868,'Reentered','1972-04-06 00:00:00','1971-06-28 00:00:00',94.91,484,545,73.96,'Intermediate' from dual union all
select 5259,'005259','1971-050E','deb Kosmos-425','Tselina-O part',24868,'Reentered','1972-06-09 00:00:00','1971-07-09 00:00:00',94.89,483,544,74.09,'Intermediate' from dual union all
select 526,'000526','1963-002B','deb USAF satellite/Agena D','',9233,'Reentered','1963-01-16 00:00:00','1963-01-09 00:00:00',90.07,192,364,82.21,'Intermediate' from dual union all
select 5260,'005260','1971-043E','PTDU cover','',24861,'Reentered','1971-05-31 00:00:00','1971-05-30 00:00:00',89.96,198,347,51.78,'Intermediate' from dual union all
select 5261,'005261','1971-051A','Mariner 9','Mariner 71J',24869,'Deep Space','1971-05-31 00:00:00','',null,null,null,null,'' from dual union all
select 5262,'005262','1971-015BA','deb Kosmos-397','',24833,'Reentered','1974-10-04 00:00:00','1971-06-01 00:00:00',112.99,573,2155,65.7,'Intermediate' from dual union all
select 5263,'005263','1971-049D','deb Blok Proton','',24867,'Reentered','1971-06-01 00:00:00','1971-05-30 00:00:00',87.79,139,192,51.29,'Intermediate' from dual union all
select 5264,'005264','1970-109E','PEOLE despin weight','',21586,'Reentered','1980-01-06 00:00:00','1971-06-02 00:00:00',97.03,500,733,15.03,'Equatorial' from dual union all
select 5265,'005265','1965-065Q','deb Ablestar','',11815,'In Earth orbit','','1970-10-31 00:00:00',107.86,1096,1159,89.99,'Polar' from dual union all
select 5266,'005266','1970-025KJ','deb Agena D','',21502,'Reentered','2011-10-21 00:00:00','1992-02-05 00:00:00',102.25,819,912,99.7,'Sun-Synch' from dual union all
select 5267,'005267','1971-051B','Centaur AC-23','Centaur 21D',24869,'Deep Space','1971-05-31 00:00:00','',null,null,null,null,'' from dual union all
select 5268,'005268','1971-030H','deb Tournesol','',24848,'Reentered','1975-03-22 00:00:00','1971-06-02 00:00:00',95.91,456,670,47.08,'Intermediate' from dual union all
select 5269,'005269','1967-001J','deb unknown                       ','[UNID037] RCS 0.0',15256,'Reentered','1978-10-29 00:00:00','1971-06-05 00:00:00',171.7,345,7364,26.33,'Medium' from dual union all
select 527,'000527','1963-003A','OPS 0180','Project 698BK Group 1-D',9234,'Reentered Attached','1969-01-09 00:00:00','1963-02-17 00:00:00',94.53,459,533,81.89,'Intermediate' from dual union all
select 5270,'005270','1969-082HQ','deb Agena D','',18349,'Reentered','1974-01-20 00:00:00','1971-03-01 00:00:00',95.61,532,564,70.12,'Intermediate' from dual union all
select 5271,'005271','1969-082HR','deb Agena D','',18349,'Reentered','1973-04-07 00:00:00','1973-04-01 00:00:00',91.69,332,383,69.76,'Intermediate' from dual union all
select 5272,'005272','1971-015BB','deb Kosmos-397','',24833,'In Earth orbit','','1971-06-01 00:00:00',113.16,565,2179,65.55,'Intermediate' from dual union all
select 5273,'005273','1971-015BC','deb Kosmos-397','',24833,'Reentered','1991-04-24 00:00:00','1991-04-24 00:00:00',89.8,257,273,65.61,'Intermediate' from dual union all
select 5274,'005274','1971-015BD','deb Kosmos-397','',24833,'Reentered','1980-08-19 00:00:00','1978-11-18 00:00:00',111.78,472,2145,65.83,'Intermediate' from dual union all
select 5275,'005275','1969-082HS','deb Agena D','',18349,'Reentered','1996-10-21 00:00:00','1992-02-06 00:00:00',97.31,612,648,70.06,'Intermediate' from dual union all
select 5276,'005276','1970-089BH','deb Kosmos-374','',21566,'Reentered','1979-06-16 00:00:00','1971-06-07 00:00:00',111.05,514,2036,62.77,'Intermediate' from dual union all
select 5277,'005277','1971-015BE','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-07 00:00:00',104.07,872,1030,65.59,'Intermediate' from dual union all
select 5278,'005278','1970-089BJ','deb Kosmos-374','[UNID038] RCS 0.1',21566,'Reentered','1983-03-06 00:00:00','1970-11-22 00:00:00',109.12,538,1835,62.53,'Intermediate' from dual union all
select 5279,'005279','1968-097CQ','deb Kosmos-252','',16824,'Reentered','1984-09-24 00:00:00','1971-06-01 00:00:00',109.58,556,1859,62.15,'Intermediate' from dual union all
select 528,'000528','1963-003B','deb USAF satellite/Agena D','',9234,'Reentered','1963-11-08 00:00:00','1963-03-01 00:00:00',94.23,449,514,81.89,'Intermediate' from dual union all
select 5280,'005280','1971-015BF','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-06 00:00:00',110.3,559,1922,65.67,'Intermediate' from dual union all
select 5281,'005281','1971-052A','Kosmos-426','DS-U2-K No. 1',24870,'Reentered','2002-05-11 00:00:00','1971-07-08 00:00:00',109.22,388,1994,74.03,'Intermediate' from dual union all
select 5282,'005282','1971-052B','S3M','S3M',24870,'Reentered','2003-06-30 00:00:00','1971-07-04 00:00:00',109.07,386,1982,74.02,'Intermediate' from dual union all
select 5283,'005283','1971-053A','Soyuz-11','Soyuz 7K-T No. 32',24871,'Landed','1971-06-29 00:00:00','1971-06-13 00:00:00',89.58,250,258,51.58,'Intermediate' from dual union all
select 5284,'005284','1971-053B','11S510','11S510',24871,'Reentered','1971-06-07 00:00:00','1971-06-06 00:00:00',88.35,176,210,51.63,'Intermediate' from dual union all
select 5285,'005285','1971-054A','P70-1','P70-1',24872,'Reentered Attached','1982-01-31 00:00:00','1971-07-10 00:00:00',95.86,542,579,90.22,'Polar' from dual union all
select 5286,'005286','1971-048C','KDU','KDU',24866,'Reentered','1971-06-15 00:00:00','1971-06-09 00:00:00',89.14,174,290,65.42,'Intermediate' from dual union all
select 5287,'005287','1971-048D','Blenda','BZA',24866,'Reentered','1971-06-11 00:00:00','1971-06-09 00:00:00',88.96,171,276,65.43,'Intermediate' from dual union all
select 5288,'005288','1969-061F','deb Molniya-1','',18328,'Reentered','1971-06-15 00:00:00','1971-06-13 00:00:00',122.16,74,3480,65.25,'ERROR - Unexpected value "TA"' from dual union all
select 5289,'005289','1971-055A','Kosmos-427','Zenit-4MK',24873,'Landed','1971-06-23 00:00:00','1971-06-13 00:00:00',89.58,184,324,72.85,'Intermediate' from dual union all
select 529,'000529','1963-003C','deb USAF satellite/Agena D','',9234,'Reentered','1963-12-31 00:00:00','1963-02-20 00:00:00',94.47,448,538,81.9,'Intermediate' from dual union all
select 5290,'005290','1971-055B','11A57I','11A57I',24873,'Reentered','1971-06-20 00:00:00','1971-06-14 00:00:00',89.33,196,287,72.85,'Intermediate' from dual union all
select 5291,'005291','1971-055C','Sensor cover','Zenit-4MK sensor cover',24873,'Reentered','1971-06-15 00:00:00','1971-06-12 00:00:00',89.54,194,310,72.64,'Intermediate' from dual union all
select 5292,'005292','1971-048E','PTDU cover','KDU part',24866,'Reentered','1971-06-11 00:00:00','1971-06-10 00:00:00',89.47,177,319,65.4,'Intermediate' from dual union all
select 5293,'005293','1970-089BL','deb Kosmos-374','',21566,'Reentered','1976-12-05 00:00:00','1971-06-12 00:00:00',104.99,431,1557,62.54,'Intermediate' from dual union all
select 5294,'005294','1970-089BM','deb Kosmos-374  [*]','',21566,'Reentered','1972-02-16 00:00:00','1972-02-12 00:00:00',92.21,306,460,63.81,'Intermediate' from dual union all
select 5295,'005295','1970-089BN','deb Kosmos-374  [*]','',21566,'Reentered','1991-12-16 00:00:00','1991-12-16 00:00:00',89.23,232,241,63.01,'Intermediate' from dual union all
select 5296,'005296','1970-089BP','deb Kosmos-374  [*]','',21566,'Reentered','1991-05-20 00:00:00','1991-05-16 00:00:00',91.43,318,372,62.67,'Intermediate' from dual union all
select 5297,'005297','1971-056A','OPS 7809','HEXAGON SV-1',24874,'Deorbited','1971-08-06 00:00:00','1971-06-22 00:00:00',89.13,178,285,96.39,'Sun-Synch' from dual union all
select 5298,'005298','1971-056B','Titan 23D-1 Stage 2','LR91AJ11 23D-1',24874,'Reentered','1971-06-20 00:00:00','1971-06-17 00:00:00',88.86,177,260,96.39,'Sun-Synch' from dual union all
select 5299,'005299','1971-050F','deb Kosmos-425','Tselina-O part',24868,'Reentered','1972-05-14 00:00:00','1971-07-01 00:00:00',95.02,496,543,74.03,'Intermediate' from dual union all
select 53,'000053','60 Iota 5','A-11 Canister half','A-11',6323,'In Earth orbit','','1960-09-18 00:00:00',118.38,1512,1705,47.2,'Intermediate' from dual union all
select 530,'000530','61 Omicron 163','deb r/b Ablestar','',7273,'In Earth orbit','','1963-01-22 00:00:00',103.6,865,993,66.88,'Intermediate' from dual union all
select 5300,'005300','1971-035C','deb Kosmos-407','Strela-2M debris',24853,'In Earth orbit','','1971-06-14 00:00:00',101.34,809,836,74.05,'Intermediate' from dual union all
select 5301,'005301','1971-035D','deb Kosmos-407','Strela-2M debris',24853,'In Earth orbit','','1971-06-14 00:00:00',101.42,809,842,74.06,'Intermediate' from dual union all
select 5302,'005302','1969-061G','deb Molniya-1','',18328,'Reentered','1971-06-18 00:00:00','1971-06-18 00:00:00',98.88,55,1356,64.92,'ERROR - Unexpected value "TA"' from dual union all
select 5303,'005303','1971-055D','Blenda','Sunshade',24873,'Reentered','1971-06-25 00:00:00','1971-06-23 00:00:00',89.15,163,301,72.86,'Intermediate' from dual union all
select 5304,'005304','1971-055E','KDU','KDU',24873,'Reentered','1971-06-28 00:00:00','1971-06-23 00:00:00',89.32,171,311,72.86,'Intermediate' from dual union all
select 5305,'005305','1971-057A','Kosmos-428','Zenit-2M',24875,'Landed','1971-07-06 00:00:00','1971-06-26 00:00:00',89.1,206,253,51.76,'Intermediate' from dual;
insert into SATELLITE
select 5306,'005306','1971-057B','11A57I','11A57I',24875,'Reentered','1971-06-28 00:00:00','1971-06-26 00:00:00',88.72,185,238,51.76,'Intermediate' from dual union all
select 5307,'005307','1971-057C','Kosmos-428 part','Zenit-2M sensor cover',24875,'Reentered','1971-06-27 00:00:00','1971-06-24 00:00:00',89.1,204,256,51.8,'Intermediate' from dual union all
select 5308,'005308','1971-057D','Kosmos-428 part','Zenit-2M sensor cover',24875,'Reentered','1971-06-27 00:00:00','1971-06-24 00:00:00',88.6,190,221,51.79,'Intermediate' from dual union all
select 5309,'005309','1965-108H','deb Transtage 8','[UNID008 71-000D]',11858,'Reentered','1989-06-21 00:00:00','1972-06-21 00:00:00',462.67,361,26514,26.67,'Highly Ellip' from dual union all
select 531,'000531','61 Omicron 164','deb r/b Ablestar','',7273,'Reentered','1971-04-15 00:00:00','1963-01-27 00:00:00',100.31,506,1041,66.12,'Intermediate' from dual union all
select 5310,'005310','1971-000E','DSP 3 aperture cover','[UNID009 71-000E]',24818,'In Earth orbit','','1971-06-30 00:00:00',1435.7,35794,35763,1.2,'Stationary' from dual union all
select 5311,'005311','1971-028C','deb Kosmos-405','',24846,'Reentered','1974-05-12 00:00:00','1971-06-30 00:00:00',98.11,665,672,81.24,'Intermediate' from dual union all
select 5312,'005312','1965-082TA','deb Transtage 4','',11832,'Reentered','1972-04-02 00:00:00','1972-03-23 00:00:00',95.25,519,543,31.74,'Intermediate' from dual union all
select 5313,'005313','1971-057E','Nauka part','Nauka',24875,'Reentered','1971-07-07 00:00:00','1971-07-05 00:00:00',88.72,195,228,51.74,'Intermediate' from dual union all
select 5314,'005314','1971-057F','Nauka part','Nauka',24875,'Reentered','1971-07-05 00:00:00','1971-07-05 00:00:00',88.11,150,212,51.73,'Intermediate' from dual union all
select 5315,'005315','1971-057G','Kapsula Kosmosa-428','Nauka 1KS No. 4',24875,'Reentered','1971-07-13 00:00:00','1971-07-05 00:00:00',88.87,199,238,51.75,'Intermediate' from dual union all
select 5316,'005316','1970-103D','Kosmos-382 part','7K-L1e adapter',21580,'Reentered','1990-11-07 00:00:00','1971-01-03 00:00:00',172.33,2896,4864,55.71,'Medium' from dual union all
select 5317,'005317','1971-058A','Solrad 10','Solar Explorer C',24876,'Reentered','1979-12-15 00:00:00','1971-08-07 00:00:00',95.29,435,631,51.05,'Intermediate' from dual union all
select 5318,'005318','1971-058B','FW4S S/N 2223-7','FW4S',24876,'Reentered','1976-01-28 00:00:00','1971-08-13 00:00:00',95.28,434,631,51.05,'Intermediate' from dual union all
select 5319,'005319','1971-058C','Solrad 10 despin weight','',24876,'Reentered','1972-09-22 00:00:00','1971-08-09 00:00:00',94.98,385,651,51.25,'Intermediate' from dual union all
select 532,'000532','1963-004B','Altair','Altair X-248-A5DM SV-330',9235,'Reentered','1981-11-02 00:00:00','1963-03-16 00:00:00',607.78,274,34506,33.12,'GEO Transfer' from dual union all
select 5320,'005320','1971-058D','Solrad 10 despin weight','',24876,'Reentered','1975-11-07 00:00:00','1971-08-17 00:00:00',95.52,477,610,50.86,'Intermediate' from dual union all
select 5321,'005321','1970-089BQ','deb Kosmos-374  [*]','',21566,'Reentered','1976-12-05 00:00:00','1980-11-08 00:00:00',91.29,295,381,62.71,'Intermediate' from dual union all
select 5322,'005322','1968-097CJ','deb Kosmos-252','[UNID039] RCS 0.0',16824,'In Earth orbit','','1992-02-05 00:00:00',111.74,664,1950,61.3,'Intermediate' from dual union all
select 5323,'005323','1971-015BG','deb Kosmos-397','',24833,'Reentered','1981-04-13 00:00:00','1971-07-14 00:00:00',111.82,567,2054,65.73,'Intermediate' from dual union all
select 5324,'005324','1971-015BH','deb Kosmos-397','',24833,'Reentered','1980-11-19 00:00:00','1971-07-14 00:00:00',112.22,573,2085,65.75,'Intermediate' from dual union all
select 5325,'005325','1971-015BJ','deb Kosmos-397','',24833,'Reentered','1991-02-25 00:00:00','1991-02-24 00:00:00',91.51,318,380,66.14,'Intermediate' from dual union all
select 5326,'005326','1970-103E','deb SOZ','Kosmos-382 part',21580,'Reentered','1977-01-03 00:00:00','1971-07-14 00:00:00',158.7,1606,5058,51.41,'Medium' from dual union all
select 5327,'005327','1971-059A','Meteor','Meteor-M No. 20',24877,'Reentered','1991-08-27 00:00:00','1971-08-21 00:00:00',97.23,613,640,81.2,'Intermediate' from dual union all
select 5328,'005328','1971-059B','8A92ME','8A92ME',24877,'Reentered','2001-10-13 00:00:00','1971-08-30 00:00:00',97.45,554,720,81.22,'Intermediate' from dual union all
select 5329,'005329','1971-060A','OPS 8373','STRAWMAN 4',24878,'Reentered Attached','1978-08-31 00:00:00','1971-08-18 00:00:00',94.52,486,505,75,'Intermediate' from dual union all
select 533,'000533','1963-005A','OPS 0240','P35-3',9236,'Reentered','1979-12-26 00:00:00','1963-03-23 00:00:00',97.67,504,791,100.48,'Sun-Synch' from dual union all
select 5330,'005330','1971-059C','deb Meteor','',24877,'Reentered','1974-08-25 00:00:00','1971-08-16 00:00:00',97.17,609,638,81.2,'Intermediate' from dual union all
select 5331,'005331','1971-061A','Kosmos-429','Zenit-4M',24879,'Landed','1971-08-02 00:00:00','1971-07-22 00:00:00',89.03,202,252,51.77,'Intermediate' from dual union all
select 5332,'005332','1971-061B','11A57I','11A57I',24879,'Reentered','1971-07-23 00:00:00','1971-07-22 00:00:00',88.38,171,218,51.78,'Intermediate' from dual union all
select 5333,'005333','1967-001K','deb unknown                       ','[UNID040]',15256,'Reentered','1971-10-14 00:00:00','1971-07-22 00:00:00',174.23,143,7768,26.04,'Medium' from dual union all
select 5334,'005334','1968-097CK','deb Kosmos-252  [*]','',16824,'Reentered','1972-02-23 00:00:00','1971-01-01 00:00:00',109.1,411,1960,63.21,'Intermediate' from dual union all
select 5335,'005335','1970-089BR','deb Kosmos-374','',21566,'Reentered','1976-11-20 00:00:00','1971-07-21 00:00:00',111.11,516,2040,62.79,'Intermediate' from dual union all
select 5336,'005336','1971-015BK','deb Kosmos-397','',24833,'Reentered','1992-02-12 00:00:00','1992-02-06 00:00:00',92.4,343,442,65.77,'Intermediate' from dual union all
select 5337,'005337','1970-089BS','deb Kosmos-374','',21566,'In Earth orbit','','1992-02-06 00:00:00',113.17,700,2045,62.96,'Intermediate' from dual union all
select 5338,'005338','1970-089BT','deb Kosmos-374','',21566,'Reentered','1992-06-16 00:00:00','1992-02-05 00:00:00',97.57,493,792,62.23,'Intermediate' from dual union all
select 5339,'005339','1969-082HT','deb Agena D','',18349,'Reentered','1982-06-10 00:00:00','1982-03-03 00:00:00',98.15,621,720,69.62,'Intermediate' from dual union all
select 534,'000534','1963-005B','MG-18 S/N 6','MG-18',9236,'Reentered','1984-05-29 00:00:00','1963-04-01 00:00:00',97.65,507,786,100.49,'Sun-Synch' from dual union all
select 5340,'005340','1971-062A','Kosmos-430','Zenit-4M',24880,'Landed','1971-08-05 00:00:00','1971-07-27 00:00:00',89,186,265,65.4,'Intermediate' from dual union all
select 5341,'005341','1971-062B','11A57I','11A57I',24880,'Reentered','1971-07-29 00:00:00','1971-07-25 00:00:00',89.01,188,264,65.41,'Intermediate' from dual union all
select 5342,'005342','1971-062C','Sensor cover','Zenit-4M sensor cover',24880,'Reentered','1971-07-26 00:00:00','1971-07-25 00:00:00',88.92,199,244,65.37,'Intermediate' from dual union all
select 5343,'005343','1971-062D','Sensor cover','Zenit-4M sensor cover',24880,'Reentered','1971-07-25 00:00:00','1971-07-23 00:00:00',89.52,189,313,65.42,'Intermediate' from dual union all
select 5344,'005344','1965-082TB','deb Transtage 4','',11832,'Reentered','1973-03-25 00:00:00','1973-03-19 00:00:00',93.25,394,474,31.45,'Intermediate' from dual union all
select 5345,'005345','1965-082TC','deb Transtage 4','',11832,'Reentered','1980-06-25 00:00:00','1980-06-23 00:00:00',91.78,344,380,32.2,'Intermediate' from dual union all
select 5346,'005346','1970-025KK','deb Agena D','',21502,'Reentered','1971-10-14 00:00:00','1971-10-07 00:00:00',91.35,289,393,100.23,'Retrograde' from dual union all
select 5347,'005347','1970-025KL','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',104.91,953,1028,100.01,'Sun-Synch' from dual union all
select 5348,'005348','1970-025KM','deb Agena D','',21502,'Reentered','1992-03-04 00:00:00','1992-02-06 00:00:00',95.29,511,555,100.03,'Sun-Synch' from dual union all
select 5349,'005349','1970-025KN','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.69,960,1094,99.83,'Sun-Synch' from dual union all
select 535,'000535','1963-005C','FTV 0240 despin weight','',9236,'Reentered','1969-06-29 00:00:00','1963-03-27 00:00:00',96.97,475,753,100.48,'Sun-Synch' from dual union all
select 5350,'005350','1970-025KP','deb Agena D','',21502,'Reentered','2012-02-26 00:00:00','1992-02-05 00:00:00',102.28,819,915,99.72,'Sun-Synch' from dual union all
select 5351,'005351','1971-063A','Endeavour','Apollo CM-112',24881,'Deep Space','1971-07-27 00:00:00','1971-07-26 00:00:00',23611.9,203,532310,29.69,'Cislunar' from dual union all
select 5352,'005352','1971-063B','Saturn S-IVB-510','Saturn S-IVB-510',24881,'Deep Space','1971-07-27 00:00:00','1971-07-26 00:00:00',21777.38,127,503764,28.42,'Cislunar' from dual union all
select 5353,'005353','1971-064B','11S510','11S510',24882,'Reentered','1971-08-29 00:00:00','1971-08-04 00:00:00',90.94,211,431,65.41,'Intermediate' from dual union all
select 5354,'005354','1971-064C','BOZ','BOZ',24882,'Reentered','1971-08-24 00:00:00','1971-08-04 00:00:00',90.84,200,432,65.42,'Intermediate' from dual union all
select 5355,'005355','1971-064E','deb Blok-I','',24882,'Reentered','1971-08-04 00:00:00','1971-07-30 00:00:00',90.93,208,433,65.4,'Intermediate' from dual union all
select 5356,'005356','1971-064F','deb Blok-I','',24882,'Reentered','1971-08-02 00:00:00','1971-07-30 00:00:00',90.64,182,430,65.4,'Intermediate' from dual union all
select 5357,'005357','1970-086C','deb Kosmos-372','Strela-2M debris',21563,'Reentered','2004-04-02 00:00:00','1971-07-20 00:00:00',100.75,786,803,74.06,'Intermediate' from dual union all
select 5358,'005358','1970-086D','deb Kosmos-372','Strela-2M debris',21563,'In Earth orbit','','1971-07-18 00:00:00',101.05,805,812,74.06,'Intermediate' from dual union all
select 5359,'005359','1970-025KQ','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.86,1032,1038,99.97,'Sun-Synch' from dual union all
select 536,'000536','1963-005D','FTV 0240 despin weight','',9236,'Reentered','1978-07-18 00:00:00','1963-03-21 00:00:00',98.34,527,832,100.51,'Sun-Synch' from dual union all
select 5360,'005360','1971-024C','deb Isis 2','',24842,'In Earth orbit','','1971-07-21 00:00:00',113.61,1361,1424,88.28,'Polar' from dual union all
select 5361,'005361','1966-025G','deb OV1-5','',13432,'In Earth orbit','','1971-07-27 00:00:00',105.32,969,1049,144.66,'Retrograde' from dual union all
select 5362,'005362','1969-082HU','deb Agena D','',18349,'Reentered','1980-09-30 00:00:00','1971-07-27 00:00:00',102.15,812,909,70.09,'Intermediate' from dual union all
select 5363,'005363','1965-065R','deb NNS O-5','',11815,'Reentered','1980-11-09 00:00:00','1971-06-30 00:00:00',106.74,1020,1131,89.99,'Polar' from dual union all
select 5364,'005364','1971-065A','Kosmos-431','Zenit-2M',24883,'Landed','1971-08-11 00:00:00','1971-08-01 00:00:00',88.98,194,255,51.76,'Intermediate' from dual union all
select 5365,'005365','1971-065B','11A57I','11A57I',24883,'Reentered','1971-08-04 00:00:00','1971-08-01 00:00:00',88.75,194,232,51.79,'Intermediate' from dual union all
select 5366,'005366','1971-063C','Falcon','LM 10 AS',24881,'Deep Space Attached','1971-07-27 00:00:00','1971-07-26 00:00:00',23611.9,203,532310,29.69,'Cislunar' from dual union all
select 5367,'005367','1971-064A','Molniya-1','Molniya-1 11F67 No. 20',24882,'Reentered','1977-07-19 00:00:00','1971-08-29 00:00:00',717.46,478,39861,65.47,'GEO Transfer' from dual union all
select 5368,'005368','1971-064D','Blok-L','Blok-L',24882,'Reentered','1977-08-14 00:00:00','1971-08-30 00:00:00',700.6,490,39014,65.45,'GEO Transfer' from dual union all
select 5369,'005369','1971-065C','Sensor cover','Sensor cover',24883,'Reentered','1971-08-05 00:00:00','1971-08-02 00:00:00',88.51,186,216,51.69,'Intermediate' from dual union all
select 537,'000537','61 Omicron 165','deb r/b Ablestar','',7273,'In Earth orbit','','1963-02-27 00:00:00',111.08,975,1578,67.28,'Intermediate' from dual union all
select 5370,'005370','1971-061C','Blenda','BZA',24879,'Reentered','1971-08-03 00:00:00','1971-08-01 00:00:00',88.67,183,235,51.78,'Intermediate' from dual union all
select 5371,'005371','1971-061D','KDU','KDU',24879,'Reentered','1971-08-04 00:00:00','1971-08-01 00:00:00',88.84,181,254,51.78,'Intermediate' from dual union all
select 5372,'005372','1965-082TD','deb Transtage 4','',11832,'Reentered','1982-11-22 00:00:00','1971-05-14 00:00:00',101.87,706,988,32.07,'Intermediate' from dual union all
select 5373,'005373','1971-061E','PTDU cover','',24879,'Reentered','1971-08-04 00:00:00','1971-08-02 00:00:00',89.38,183,305,51.85,'Intermediate' from dual union all
select 5374,'005374','1971-062E','Blenda','BZA',24880,'Reentered','1971-08-06 00:00:00','1971-08-04 00:00:00',88.53,177,227,65.42,'Intermediate' from dual union all
select 5375,'005375','1965-082TE','deb Transtage 4','',11832,'Reentered','1978-04-19 00:00:00','1978-11-16 00:00:00',105.21,726,1283,31.42,'Intermediate' from dual union all
select 5376,'005376','1965-082TF','deb Transtage 4','',11832,'Reentered','1991-12-07 00:00:00','1991-12-05 00:00:00',89.9,260,279,32.17,'Intermediate' from dual union all
select 5377,'005377','1971-063D','Apollo 15 Subsatellite','Apollo 15 Subsatellite',24881,'Deep Space Attached','1971-07-27 00:00:00','1971-08-04 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 5378,'005378','1971-062F','KDU','KDU',24880,'Reentered','1971-08-09 00:00:00','1971-08-04 00:00:00',88.76,168,259,65.42,'Intermediate' from dual union all
select 5379,'005379','1971-066A','Kosmos-432','Zenit-4M',24884,'Landed','1971-08-18 00:00:00','1971-08-07 00:00:00',88.99,189,261,51.77,'Intermediate' from dual union all
select 538,'000538','61 Omicron 166','deb r/b Ablestar','',7273,'In Earth orbit','','1963-02-26 00:00:00',103.55,862,991,67.85,'Intermediate' from dual union all
select 5380,'005380','1971-066B','11A57I','11A57I',24884,'Reentered','1971-08-09 00:00:00','1971-08-08 00:00:00',88.35,166,220,51.76,'Intermediate' from dual union all
select 5381,'005381','1971-066C','Sensor cover','Zenit-4M sensor cover',24884,'Reentered','1971-08-08 00:00:00','1971-08-06 00:00:00',88.87,178,260,51.73,'Intermediate' from dual union all
select 5382,'005382','1971-067C','LOADS 2','OAR-901',24885,'Reentered','1972-01-31 00:00:00','1971-09-06 00:00:00',103.93,127,1761,92.01,'Polar' from dual union all
select 5383,'005383','1971-067D','RTDS','OAR-907',24885,'Reentered','1971-09-19 00:00:00','1971-08-14 00:00:00',94.01,131,811,87.59,'Polar' from dual union all
select 5384,'005384','1971-067N','AVL-802-1 Canister','AVL-802',24885,'In Earth orbit','','1971-09-09 00:00:00',101.85,778,914,87.64,'Polar' from dual union all
select 5385,'005385','1971-062G','PTDU cover','KDU part',24880,'Reentered','1971-08-07 00:00:00','1971-08-06 00:00:00',89.5,185,315,65.4,'Intermediate' from dual union all
select 5386,'005386','1970-025KR','deb Agena D','',21502,'Reentered','2011-12-05 00:00:00','1992-02-06 00:00:00',103.66,885,979,99.67,'Sun-Synch' from dual union all
select 5387,'005387','1970-025KS','deb Agena D','',21502,'In Earth orbit','','1992-02-07 00:00:00',120.7,1363,2062,100.91,'Medium' from dual union all
select 5388,'005388','1970-025KT','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',108.48,1046,1267,99.91,'Sun-Synch' from dual union all
select 5389,'005389','1970-025KU','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',109.97,1068,1383,99.88,'Sun-Synch' from dual union all
select 539,'000539','61 Omicron 167','deb r/b Ablestar','',7273,'Reentered','1991-02-24 00:00:00','1963-03-14 00:00:00',105.18,966,1039,67,'Intermediate' from dual union all
select 5390,'005390','1970-025KV','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.25,963,1049,100.09,'Sun-Synch' from dual union all
select 5391,'005391','1970-025KW','deb Agena D','',21502,'Reentered','1982-02-16 00:00:00','1971-08-07 00:00:00',102.08,641,1074,99.95,'Sun-Synch' from dual union all
select 5392,'005392','1970-025KX','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',103.99,901,993,100.06,'Sun-Synch' from dual union all
select 5393,'005393','1970-025KY','deb Agena D','',21502,'Reentered','2001-04-24 00:00:00','1992-02-07 00:00:00',100.16,703,830,100.09,'Sun-Synch' from dual union all
select 5394,'005394','1971-067A','OV1-20','OV1-20P/FW4S',24885,'Reentered','1971-08-29 00:00:00','1971-08-14 00:00:00',102.34,129,1610,91.98,'Polar' from dual union all
select 5395,'005395','1971-067K','OV1-21 AKM/LCS Can Half','Star 13A',24885,'In Earth orbit','','1971-09-11 00:00:00',101.99,789,916,87.64,'Polar' from dual union all
select 5396,'005396','1971-067F','AVL-802 Mylar Sphere','AVL-802',24885,'Reentered','1972-06-11 00:00:00','1971-09-06 00:00:00',101.64,761,912,87.57,'Polar' from dual;
insert into SATELLITE
select 5397,'005397','1971-067B','OV1-21','OV1-21P/FW4S',24885,'In Earth orbit','','1971-09-06 00:00:00',101.95,787,915,87.64,'Polar' from dual union all
select 5398,'005398','1971-067E','LCS 4','LCS 4',24885,'In Earth orbit','','1971-09-06 00:00:00',101.64,761,912,87.57,'Polar' from dual union all
select 5399,'005399','1971-067L','AVL-802-2 Canister','AVL-802',24885,'Reentered','1999-02-19 00:00:00','1971-09-12 00:00:00',101.92,783,917,87.65,'Polar' from dual union all
select 54,'000054','60 Kappa','Discoverer 14','CORONA C-9',6324,'Reentered Attached','1960-09-16 00:00:00','1960-08-28 00:00:00',93.24,178,689,79.65,'Intermediate' from dual union all
select 540,'000540','61 Omicron 168','deb r/b Ablestar','',7273,'Reentered','1991-08-20 00:00:00','1991-08-19 00:00:00',90.84,296,336,66.79,'Intermediate' from dual union all
select 5400,'005400','1971-067M','AVL-802-1 Canister part','',24885,'Reentered','1998-09-15 00:00:00','1971-09-06 00:00:00',101.89,782,915,87.62,'Polar' from dual union all
select 5401,'005401','1971-067G','AVL-802 Grid Sphere 7-2','AVL-802',24885,'Reentered','1979-03-18 00:00:00','1971-09-08 00:00:00',101.83,778,913,87.63,'Polar' from dual union all
select 5402,'005402','1971-068A','Kosmos-433','OGCh 8F673K No. 5102740-',24886,'Deorbited','1971-08-09 00:00:00','1971-08-09 00:00:00',88.6,112,299,49.41,'Intermediate' from dual union all
select 5403,'005403','1971-068B','Kosmos-433 Adapter','OGCh Adapter',24886,'Reentered','1971-08-10 00:00:00','1971-08-09 00:00:00',88.37,151,236,49.57,'Intermediate' from dual union all
select 5404,'005404','1971-068C','8S692','8S692',24886,'Reentered','1971-08-10 00:00:00','1971-08-09 00:00:00',87.9,145,197,49.56,'Intermediate' from dual union all
select 5405,'005405','1971-067J','deb OV1-21','',24885,'Reentered','1998-07-20 00:00:00','1971-09-12 00:00:00',101.78,772,913,87.64,'Polar' from dual union all
select 5406,'005406','1971-067H','AVL-802 Grid Sphere 7-1','AVL-802',24885,'Reentered','1979-11-02 00:00:00','1971-09-06 00:00:00',101.85,778,914,87.63,'Polar' from dual union all
select 5407,'005407','1971-069A','Kosmos-434','T2K No. 3',24887,'Reentered','1981-08-23 00:00:00','1971-09-11 00:00:00',227.94,187,11777,51.58,'Medium' from dual union all
select 5408,'005408','1971-069B','11S510','11S510',24887,'Reentered','1971-08-18 00:00:00','1971-08-14 00:00:00',88.87,195,243,51.59,'Intermediate' from dual union all
select 5409,'005409','1971-070A','OPS 8607','GAMBIT-3 Mission 4332',24888,'Reentered Attached','1971-09-03 00:00:00','1971-08-19 00:00:00',89.86,130,405,110.98,'Retrograde' from dual union all
select 541,'000541','61 Omicron 169','deb r/b Ablestar','',7273,'In Earth orbit','','1963-02-27 00:00:00',104.87,921,1056,67.24,'Intermediate' from dual union all
select 5410,'005410','1971-067P','AVL-802 Rigid Sphere','',24885,'Reentered','1981-09-01 00:00:00','1971-09-09 00:00:00',101.92,784,916,87.63,'Polar' from dual union all
select 5411,'005411','1970-089BU','Unknown deb                       ','',21566,'Reentered','1982-06-02 00:00:00','1971-08-14 00:00:00',109.06,510,1857,62.95,'Intermediate' from dual union all
select 5412,'005412','1970-089BV','Unknown deb                       ','[UNID041]',21566,'Reentered','1980-04-30 00:00:00','1971-09-01 00:00:00',101.61,491,1179,62,'Intermediate' from dual union all
select 5413,'005413','1970-089BW','Unknown deb                       ','[UNID042] RCS 0.0',21566,'Reentered','1980-07-24 00:00:00','1971-08-13 00:00:00',108.33,555,1744,62.26,'Intermediate' from dual union all
select 5414,'005414','1971-015BL','Unknown deb                       ','[UNID043] RCS 0.0',24833,'In Earth orbit','','1992-02-05 00:00:00',102.95,840,956,66.76,'Intermediate' from dual union all
select 5415,'005415','1971-015BM','deb Kosmos-397','',24833,'Reentered','1978-07-09 00:00:00','1978-06-17 00:00:00',91.08,328,328,64.88,'Intermediate' from dual union all
select 5416,'005416','1971-015BN','Unknown deb                       ','',24833,'In Earth orbit','','1992-02-04 00:00:00',105.8,973,1091,65.8,'Intermediate' from dual union all
select 5417,'005417','1968-097CL','deb Kosmos-252','',16824,'Reentered','2005-01-31 00:00:00','1992-02-06 00:00:00',102.96,554,1244,62.84,'Intermediate' from dual union all
select 5418,'005418','1968-097CM','deb Kosmos-252','',16824,'Reentered','1982-11-19 00:00:00','1971-08-23 00:00:00',113.77,551,2248,62.46,'Intermediate' from dual union all
select 5419,'005419','1971-011C','deb Tansei 1','',24829,'Reentered','1980-07-26 00:00:00','1971-08-11 00:00:00',102.78,548,1233,30.42,'Intermediate' from dual union all
select 542,'000542','61 Omicron 170','deb r/b Ablestar','',7273,'Reentered','1991-07-06 00:00:00','1991-06-12 00:00:00',92.9,403,431,68.12,'Intermediate' from dual union all
select 5420,'005420','1966-013K','deb Rubis','',13420,'Reentered','1982-11-22 00:00:00','1971-08-30 00:00:00',115.14,497,2427,34.09,'Intermediate' from dual union all
select 5421,'005421','1970-025KZ','deb Agena D','',21502,'Reentered','1979-08-29 00:00:00','1979-08-12 00:00:00',93.55,424,473,99.1,'Sun-Synch' from dual union all
select 5422,'005422','1970-025LA','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',103.83,828,1051,100.1,'Sun-Synch' from dual union all
select 5423,'005423','1970-025LB','deb Agena D','',21502,'Reentered','1971-12-05 00:00:00','1971-11-30 00:00:00',91.21,313,355,100.33,'Retrograde' from dual union all
select 5424,'005424','1970-025LC','deb Agena D','',21502,'Reentered','1982-03-19 00:00:00','1982-03-15 00:00:00',91.76,350,372,99.85,'Retrograde' from dual union all
select 5425,'005425','1969-021Y','deb Blok Kosmos','Tselina-O part',18288,'Reentered','1972-12-20 00:00:00','1971-08-23 00:00:00',94.66,500,505,74.15,'Intermediate' from dual union all
select 5426,'005426','1971-069C','Eole despin weight','[UNID146] RCS 0.04',24887,'In Earth orbit','','1971-10-02 00:00:00',101.13,700,924,49.56,'Intermediate' from dual union all
select 5427,'005427','1971-069D','T2K lower section','T2K lower section',24887,'Reentered','1971-09-19 00:00:00','1971-08-20 00:00:00',98.63,186,1201,51.58,'Intermediate' from dual union all
select 5428,'005428','1971-069E','T2K part','T2K part',24887,'Reentered','1971-10-28 00:00:00','1971-09-11 00:00:00',97.1,184,1055,51.61,'Intermediate' from dual union all
select 5429,'005429','1969-082HV','deb Agena D','',18349,'Reentered','1979-06-16 00:00:00','1979-01-08 00:00:00',98.12,646,692,70.07,'Intermediate' from dual union all
select 543,'000543','61 Omicron 171','deb r/b Ablestar','',7273,'In Earth orbit','','1963-02-26 00:00:00',105.87,978,1093,67.03,'Intermediate' from dual union all
select 5430,'005430','1969-082HW','deb Agena D','',18349,'In Earth orbit','','1992-02-06 00:00:00',102.41,827,919,70.09,'Intermediate' from dual union all
select 5431,'005431','1970-089BX','deb Kosmos-374','',21566,'Reentered','1972-01-26 00:00:00','1971-08-14 00:00:00',97.8,423,884,62.9,'Intermediate' from dual union all
select 5432,'005432','1968-097CY','deb Kosmos-252','',16824,'Reentered','1999-06-03 00:00:00','1992-02-07 00:00:00',98.93,508,907,62.29,'Intermediate' from dual union all
select 5433,'005433','1970-089BZ','deb Kosmos-374','',21566,'Reentered','1982-01-01 00:00:00','1971-08-14 00:00:00',115.58,511,2453,62.92,'Intermediate' from dual union all
select 5434,'005434','1971-015BP','deb Kosmos-397','',24833,'Reentered','1991-03-19 00:00:00','1991-03-19 00:00:00',90.34,270,313,66.11,'Intermediate' from dual union all
select 5435,'005435','1971-071A','Eole','Eole',24889,'In Earth orbit','','1971-09-17 00:00:00',100.67,676,905,50.15,'Intermediate' from dual union all
select 5436,'005436','1971-066D','KDU','KDU',24884,'Reentered','1971-08-21 00:00:00','1971-08-17 00:00:00',88.66,187,229,51.73,'Intermediate' from dual union all
select 5437,'005437','1971-066E','Blenda','BZA',24884,'Reentered','1971-08-18 00:00:00','1971-08-17 00:00:00',88.8,184,247,51.79,'Intermediate' from dual union all
select 5438,'005438','1971-071B','FW4S S/N 2223-15','FW4S',24889,'In Earth orbit','','1971-09-20 00:00:00',100.66,675,905,50.16,'Intermediate' from dual union all
select 5439,'005439','1967-052G','deb Molniya-1','',15307,'Reentered','1971-09-01 00:00:00','1971-08-15 00:00:00',166.16,125,7143,65.09,'Medium' from dual union all
select 544,'000544','61 Omicron 172','deb r/b Ablestar','',7273,'In Earth orbit','','1963-02-26 00:00:00',107.2,940,1254,66.88,'Intermediate' from dual union all
select 5440,'005440','1971-071C','Eole despin weight','',24889,'Reentered','2000-09-16 00:00:00','1971-09-23 00:00:00',100.25,648,893,50.74,'Intermediate' from dual union all
select 5441,'005441','1971-072A','Kosmos-435','DS-P1-Yu No. 41',24890,'Reentered','1972-01-28 00:00:00','1971-09-28 00:00:00',91.75,265,456,70.96,'Intermediate' from dual union all
select 5442,'005442','1971-072B','S1M','S1M',24890,'Reentered','1971-11-20 00:00:00','1971-09-26 00:00:00',91.44,264,427,70.95,'Intermediate' from dual union all
select 5443,'005443','1967-114E','FW4D yo weight','',15369,'In Earth orbit','','1968-02-10 00:00:00',114.62,1390,1487,101.86,'Sun-Synch' from dual union all
select 5444,'005444','1964-083K','deb VE-5','',10369,'Reentered','1979-01-13 00:00:00','1971-08-31 00:00:00',100.41,815,741,89.8,'Polar' from dual union all
select 5445,'005445','1970-025LD','deb Agena D','',21502,'Reentered','1988-10-28 00:00:00','1988-10-22 00:00:00',90.62,295,315,100.02,'Retrograde' from dual union all
select 5446,'005446','1970-025LE','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.71,1001,1148,100.41,'Sun-Synch' from dual union all
select 5447,'005447','1970-067D','NNS O-19 despin weight','Transit high deb',21544,'In Earth orbit','','1971-08-26 00:00:00',109.64,955,1465,90.1,'Polar' from dual union all
select 5448,'005448','1971-073A','Luna-18 KT','E-8-5 No. 407',24891,'Deep Space','1971-09-03 00:00:00','1971-09-02 00:00:00',15562.18,200,400000,51.5,'Cislunar' from dual union all
select 5449,'005449','1971-073B','Blok D No. 0601L','11S824',24891,'Deep Space','1971-09-03 00:00:00','1971-09-02 00:00:00',15562.18,200,400000,51.5,'Cislunar' from dual union all
select 545,'000545','61 Omicron 173','deb r/b Ablestar','',7273,'In Earth orbit','','1963-02-26 00:00:00',106.43,919,1204,67.26,'Intermediate' from dual union all
select 5450,'005450','1971-073C','SOZ','SOZ',24891,'Reentered','1971-09-07 00:00:00','1971-09-04 00:00:00',88.54,184,221,51.49,'Intermediate' from dual union all
select 5451,'005451','1971-073D','SOZ','SOZ',24891,'Reentered','1971-09-07 00:00:00','1971-09-04 00:00:00',88.53,188,216,51.58,'Intermediate' from dual union all
select 5452,'005452','1971-070B','Fairing','GAMBIT SRV-2 Fairing',24888,'Reentered','1971-09-04 00:00:00','1971-09-04 00:00:00',89.47,132,365,110.96,'Retrograde' from dual union all
select 5453,'005453','1971-073E','deb Blok D','',24891,'Reentered','1971-09-08 00:00:00','1971-09-04 00:00:00',88.58,186,223,51.55,'Intermediate' from dual union all
select 5454,'005454','1970-089CA','deb Kosmos-374  [*]','',21566,'Reentered','1980-04-21 00:00:00','1971-09-04 00:00:00',107.41,556,1658,62.39,'Intermediate' from dual union all
select 5455,'005455','1970-089CB','deb Kosmos-374','',21566,'Reentered','1975-09-03 00:00:00','1971-09-01 00:00:00',106.44,438,1686,63.1,'Intermediate' from dual union all
select 5456,'005456','1970-089CC','deb Kosmos-374  [*]','',21566,'Reentered','1977-09-03 00:00:00','1977-05-03 00:00:00',97.61,425,864,62.68,'Intermediate' from dual union all
select 5457,'005457','1965-108J','deb Transtage 8','',11858,'In Earth orbit','','2004-06-03 00:00:00',530.89,886,29795,27.57,'Highly Ellip' from dual union all
select 5458,'005458','1970-089CE','deb Kosmos-374  [*]','',21566,'Reentered','1976-04-12 00:00:00','1979-07-14 00:00:00',109.86,543,1898,62.33,'Intermediate' from dual union all
select 5459,'005459','1969-082HX','deb Agena D','',18349,'Reentered','1980-09-24 00:00:00','1980-09-17 00:00:00',92.6,388,416,70.62,'Intermediate' from dual union all
select 546,'000546','61 Omicron 174','deb r/b Ablestar','',7273,'Reentered','2014-11-05 00:00:00','1963-02-27 00:00:00',107.11,965,1221,67.25,'Intermediate' from dual union all
select 5460,'005460','1968-097CN','deb Kosmos-252','',16824,'Reentered','1983-10-29 00:00:00','1971-09-04 00:00:00',115.04,500,2415,62.84,'Intermediate' from dual union all
select 5461,'005461','1971-074A','Kosmos-436','Tselina-OM',24892,'Reentered','1980-01-04 00:00:00','1971-10-08 00:00:00',95.13,509,541,74.04,'Intermediate' from dual union all
select 5462,'005462','1971-074B','S3M','S3M',24892,'Reentered','1980-03-13 00:00:00','1971-10-07 00:00:00',94.94,499,533,74.05,'Intermediate' from dual union all
select 5463,'005463','1971-074C','deb Kosmos-436','Tselina-O part',24892,'Reentered','1972-05-12 00:00:00','1971-10-07 00:00:00',94.85,486,537,74.03,'Intermediate' from dual union all
select 5464,'005464','1971-074D','deb Kosmos-436','Tselina-O part',24892,'Reentered','1972-01-22 00:00:00','1971-10-13 00:00:00',94.8,494,523,74.04,'Intermediate' from dual union all
select 5465,'005465','1971-074E','deb Kosmos-436','Tselina-O part',24892,'Reentered','1972-05-09 00:00:00','1971-10-07 00:00:00',94.83,485,536,73.99,'Intermediate' from dual union all
select 5466,'005466','1971-075A','Kosmos-437','Tselina-OM',24893,'Reentered','1980-03-29 00:00:00','1971-10-10 00:00:00',95.25,519,543,74.05,'Intermediate' from dual union all
select 5467,'005467','1971-075B','S3M','S3M',24893,'Reentered','1980-01-06 00:00:00','1971-10-11 00:00:00',95.13,506,543,74.05,'Intermediate' from dual union all
select 5468,'005468','1971-076A','OPS 5454','CORONA CR-15',24894,'Reentered Attached','1971-10-05 00:00:00','1971-09-17 00:00:00',88.2,146,225,74.95,'Intermediate' from dual union all
select 5469,'005469','1971-076B','ARROYO','P-11 No. 4427',24894,'Reentered','1976-02-03 00:00:00','1971-10-10 00:00:00',94.52,487,504,75.07,'Intermediate' from dual union all
select 547,'000547','61 Omicron 175','deb r/b Ablestar','',7273,'Reentered','1989-11-20 00:00:00','1963-02-27 00:00:00',104.9,965,1015,66.89,'Intermediate' from dual union all
select 5470,'005470','1971-074F','deb Kosmos-436','Tselina-O part',24892,'Reentered','1972-05-11 00:00:00','1971-10-08 00:00:00',94.84,487,534,74.11,'Intermediate' from dual union all
select 5471,'005471','1965-082TG','deb Transtage 4','',11832,'Reentered','1979-08-31 00:00:00','1979-05-29 00:00:00',96.77,538,670,32.39,'Intermediate' from dual union all
select 5472,'005472','1965-108K','deb Transtage 8','',11858,'Reentered','1972-11-25 00:00:00','1971-08-29 00:00:00',281.51,196,15504,26.44,'Highly Ellip' from dual union all
select 5473,'005473','1969-064X','deb Star 37D','',18331,'Reentered','1975-01-05 00:00:00','1971-08-30 00:00:00',141.34,271,4949,30.38,'Medium' from dual union all
select 5474,'005474','1965-108L','deb Transtage 8','',11858,'Reentered','1973-02-03 00:00:00','1971-09-01 00:00:00',186.78,233,8658,26.53,'Medium' from dual union all
select 5475,'005475','1971-077A','Kosmos-438','Zenit-4MK',24895,'Landed','1971-09-27 00:00:00','1971-09-16 00:00:00',89.5,204,296,65.39,'Intermediate' from dual union all
select 5476,'005476','1971-077B','11A57I','11A57I',24895,'Reentered','1971-09-22 00:00:00','1971-09-16 00:00:00',89.17,194,272,65.39,'Intermediate' from dual union all
select 5477,'005477','1971-077C','Sensor cover','Zenit-4MK sensor cover',24895,'Reentered','1971-09-16 00:00:00','1971-09-15 00:00:00',89,181,270,65.39,'Intermediate' from dual union all
select 5478,'005478','1971-078A','Kosmos-439','Zenit-2M',24896,'Landed','1971-10-02 00:00:00','1971-09-23 00:00:00',89.39,206,283,65.44,'Intermediate' from dual union all
select 5479,'005479','1971-078B','11A57I','11A57I',24896,'Reentered','1971-09-27 00:00:00','1971-09-23 00:00:00',89.13,197,266,65.41,'Intermediate' from dual union all
select 548,'000548','61 Omicron 176','deb r/b Ablestar','',7273,'In Earth orbit','','1963-02-26 00:00:00',101.34,669,976,66.72,'Intermediate' from dual union all
select 5480,'005480','1971-079A','Kosmos-440','DS-P1-I No. 10',24897,'Reentered','1972-10-29 00:00:00','1971-10-24 00:00:00',95.01,269,770,71.01,'Intermediate' from dual union all
select 5481,'005481','1971-079B','S1M','S1M',24897,'Reentered','1972-05-10 00:00:00','1971-10-24 00:00:00',94.84,269,752,70.99,'Intermediate' from dual union all
select 5482,'005482','1971-077D','Blenda','Sunshade',24895,'Reentered','1971-09-28 00:00:00','1971-09-25 00:00:00',89.22,171,301,65.38,'Intermediate' from dual union all
select 5483,'005483','1971-077E','KDU part','KDU part',24895,'Reentered','1971-09-27 00:00:00','1971-09-26 00:00:00',88.88,168,271,65.62,'Intermediate' from dual union all
select 5484,'005484','1971-077F','KDU','KDU',24895,'Reentered','1971-09-30 00:00:00','1971-09-27 00:00:00',88.75,162,264,65.39,'Intermediate' from dual union all
select 5485,'005485','1971-080A','Shinsei','MS-F2',24898,'In Earth orbit','','1971-10-28 00:00:00',113.08,869,1867,32.07,'Intermediate' from dual union all
select 5486,'005486','1971-081A','Kosmos-441','Zenit-4M',24899,'Landed','1971-10-10 00:00:00','1971-09-30 00:00:00',88.9,181,260,65.02,'Intermediate' from dual;
insert into SATELLITE
select 5487,'005487','1971-081B','11A57I','11A57I',24899,'Reentered','1971-10-03 00:00:00','1971-09-30 00:00:00',88.83,193,241,65.05,'Intermediate' from dual union all
select 5488,'005488','1971-082A','Luna-19','E-8LS No. 202',24900,'Deep Space','1971-09-29 00:00:00','1971-09-28 00:00:00',15562.18,200,400000,51.5,'Cislunar' from dual union all
select 5489,'005489','1971-082B','SOZ','SOZ',24900,'Reentered','1971-10-01 00:00:00','1971-09-30 00:00:00',88.24,178,196,51.53,'Intermediate' from dual union all
select 549,'000549','61 Omicron 177','deb r/b Ablestar','',7273,'In Earth orbit','','1963-02-27 00:00:00',108.23,975,1314,66.68,'Intermediate' from dual union all
select 5490,'005490','1971-082C','Blok D No. 400L','11S824',24900,'Deep Space','1971-09-29 00:00:00','1971-09-28 00:00:00',15562.18,200,400000,51.5,'Cislunar' from dual union all
select 5491,'005491','1971-083A','OSO 7','OSO H',24901,'Reentered','1974-07-09 00:00:00','1971-10-31 00:00:00',93.51,324,569,33.13,'Intermediate' from dual union all
select 5492,'005492','1971-083B','TETR 3','TETR D',24901,'Reentered','1978-09-19 00:00:00','1971-10-30 00:00:00',94.31,400,571,33.08,'Intermediate' from dual union all
select 5493,'005493','1971-084A','Kosmos-442','Zenit-4M',24902,'Landed','1971-10-12 00:00:00','1971-10-01 00:00:00',89.51,199,302,72.9,'Intermediate' from dual union all
select 5494,'005494','1971-082D','SOZ','SOZ',24900,'Reentered','1971-10-01 00:00:00','1971-09-30 00:00:00',88.26,176,201,51.59,'Intermediate' from dual union all
select 5495,'005495','1971-081C','Sensor cover','Zenit-4M sensor cover',24899,'Reentered','1971-10-06 00:00:00','1971-09-30 00:00:00',88.91,198,244,65.04,'Intermediate' from dual union all
select 5496,'005496','1971-081D','Sensor cover','Zenit-4M sensor cover',24899,'Reentered','1971-09-28 00:00:00','1971-09-28 00:00:00',88.4,175,216,65.05,'Intermediate' from dual union all
select 5497,'005497','1971-084B','11A57I','11A57I',24902,'Reentered','1971-10-05 00:00:00','1971-10-01 00:00:00',89.24,197,277,72.88,'Intermediate' from dual union all
select 5498,'005498','1971-080B','M-40','M-40',24898,'In Earth orbit','','1971-11-05 00:00:00',111.86,868,1757,32.04,'Intermediate' from dual union all
select 5499,'005499','1971-083C','Delta 85','DSV-3E-3 85',24901,'Reentered','1974-01-24 00:00:00','1971-11-01 00:00:00',93.99,373,566,33.08,'Intermediate' from dual union all
select 55,'000055','60 Lambda 1','Korabl''-Sputnik-2','Vostok-1 No. 2',6325,'Landed','1960-08-20 00:00:00','1960-08-19 00:00:00',90.66,307,307,64.9,'Intermediate' from dual union all
select 550,'000550','61 Omicron 178','deb r/b Ablestar','',7273,'Reentered','1991-02-03 00:00:00','1963-02-26 00:00:00',105.22,959,1051,67.05,'Intermediate' from dual union all
select 5500,'005500','1963-014CP','deb West Ford','',9245,'Reentered','1977-01-23 00:00:00','1971-10-01 00:00:00',162.04,654,6281,84.49,'Medium' from dual union all
select 5501,'005501','1970-025LF','deb Agena D','',21502,'Reentered','1978-12-26 00:00:00','1978-12-17 00:00:00',93.53,421,474,99.6,'Sun-Synch' from dual union all
select 5502,'005502','1970-025LG','deb Agena D','',21502,'In Earth orbit','','1992-02-07 00:00:00',104.44,935,1002,99.55,'Sun-Synch' from dual union all
select 5503,'005503','1963-014CQ','deb West Ford','',9245,'Reentered','1977-01-03 00:00:00','1971-10-03 00:00:00',163.72,1190,5881,85.15,'Medium' from dual union all
select 5504,'005504','1970-025LH','deb Agena D','',21502,'Reentered','1991-06-23 00:00:00','1991-09-23 00:00:00',100.32,738,810,99.06,'Sun-Synch' from dual union all
select 5505,'005505','1970-025LJ','deb Agena D','',21502,'Reentered','1991-03-20 00:00:00','1991-03-20 00:00:00',89.96,257,288,99.95,'Retrograde' from dual union all
select 5506,'005506','1970-025LK','deb Agena D','',21502,'Reentered','1991-03-28 00:00:00','1991-03-27 00:00:00',90.8,293,335,99.94,'Retrograde' from dual union all
select 5507,'005507','1970-025LL','deb Agena D','',21502,'In Earth orbit','','1992-02-04 00:00:00',104.16,920,991,99.08,'Sun-Synch' from dual union all
select 5508,'005508','1970-025LM','deb Agena D','',21502,'Reentered','1991-11-23 00:00:00','1991-11-22 00:00:00',89.77,252,275,99.91,'Retrograde' from dual union all
select 5509,'005509','1970-025LN','deb Agena D','',21502,'Reentered','1972-04-11 00:00:00','1971-10-03 00:00:00',97.15,415,830,100.04,'Sun-Synch' from dual union all
select 551,'000551','61 Omicron 179','deb r/b Ablestar','',7273,'Reentered','1990-02-16 00:00:00','1967-11-02 00:00:00',101.01,647,966,66.8,'Intermediate' from dual union all
select 5510,'005510','1970-025LP','deb Agena D','',21502,'Reentered','1991-03-18 00:00:00','1991-03-18 00:00:00',90.74,293,329,99.88,'Retrograde' from dual union all
select 5511,'005511','1963-014CR','deb West Ford','',9245,'Reentered','1975-01-23 00:00:00','1971-10-04 00:00:00',161.74,601,6310,84.42,'Medium' from dual union all
select 5512,'005512','1963-014CS','deb West Ford','',9245,'Reentered','1976-12-05 00:00:00','1971-10-04 00:00:00',162.52,777,6197,84.7,'Medium' from dual union all
select 5513,'005513','1963-014CT','deb West Ford','',9245,'Reentered','1975-02-19 00:00:00','1971-10-06 00:00:00',162.44,733,6235,84.66,'Medium' from dual union all
select 5514,'005514','1968-097CP','Unknown deb                       ','',16824,'Reentered','1981-01-05 00:00:00','1971-09-01 00:00:00',108.01,550,1720,62.26,'Intermediate' from dual union all
select 5515,'005515','1965-108M','deb Transtage 8','',11858,'In Earth orbit','','2004-06-15 00:00:00',542.2,1110,30186,28.12,'Highly Ellip' from dual union all
select 5516,'005516','1968-097CR','Unknown deb                       ','',16824,'Reentered','1979-10-10 00:00:00','1979-04-04 00:00:00',109.83,532,1906,62.31,'Intermediate' from dual union all
select 5517,'005517','1968-097CS','deb Kosmos-252','',16824,'Reentered','1982-10-08 00:00:00','1989-02-15 00:00:00',90.41,258,332,61.94,'Intermediate' from dual union all
select 5518,'005518','1968-097CT','Unknown deb                       ','',16824,'Reentered','1976-11-20 00:00:00','1971-10-04 00:00:00',111.23,562,2005,62.31,'Intermediate' from dual union all
select 5519,'005519','1969-082HY','deb Agena D','',18349,'In Earth orbit','','1992-02-06 00:00:00',101.1,768,854,70.06,'Intermediate' from dual union all
select 552,'000552','61 Omicron 180','deb r/b Ablestar','',7273,'In Earth orbit','','1963-02-27 00:00:00',101.51,619,1042,66.71,'Intermediate' from dual union all
select 5520,'005520','1969-082HZ','deb Agena D','',18349,'Reentered','2001-10-11 00:00:00','1992-02-06 00:00:00',99.15,693,743,70.09,'Intermediate' from dual union all
select 5521,'005521','1969-082JA','deb Agena D','',18349,'Reentered','1979-07-27 00:00:00','1970-09-15 00:00:00',100.15,653,879,69.55,'Intermediate' from dual union all
select 5522,'005522','1969-082JB','deb Agena D','',18349,'Reentered','1971-12-06 00:00:00','1971-12-02 00:00:00',90.59,282,324,69.5,'Intermediate' from dual union all
select 5523,'005523','1970-089CF','Unknown deb                       ','',21566,'Reentered','1981-02-24 00:00:00','1971-09-26 00:00:00',108.87,514,1835,63.04,'Intermediate' from dual union all
select 5524,'005524','1970-089CG','Unknown deb                       ','',21566,'Reentered','1979-04-07 00:00:00','1971-08-31 00:00:00',108.76,514,1825,62.93,'Intermediate' from dual union all
select 5525,'005525','1970-089CH','Unknown deb                       ','',21566,'Reentered','1990-12-14 00:00:00','1990-12-14 00:00:00',89.32,219,263,62.23,'Intermediate' from dual union all
select 5526,'005526','1971-015BQ','deb Kosmos-397','',24833,'Reentered','1989-01-23 00:00:00','1989-01-22 00:00:00',89.59,228,281,65.74,'Intermediate' from dual union all
select 5527,'005527','1971-015BR','Unknown deb                       ','',24833,'In Earth orbit','','1992-02-06 00:00:00',104.62,893,1060,66.92,'Intermediate' from dual union all
select 5528,'005528','1971-015BS','deb Kosmos-397','',24833,'Reentered','1978-02-21 00:00:00','1981-04-17 00:00:00',110.31,556,1926,65.76,'Intermediate' from dual union all
select 5529,'005529','1971-015BT','Unknown deb                       ','',24833,'In Earth orbit','','1992-02-05 00:00:00',101.96,805,898,66.88,'Intermediate' from dual union all
select 553,'000553','1963-004A','Syncom 1','Syncom 1',9235,'In Earth orbit','','1963-10-14 00:00:00',1426.47,34181,37015,33.51,'Near-synch' from dual union all
select 5530,'005530','1971-015BU','deb Kosmos-397','',24833,'Reentered','1989-08-11 00:00:00','1989-08-10 00:00:00',90.69,289,328,66.38,'Intermediate' from dual union all
select 5531,'005531','1971-015BV','Unknown deb                       ','',24833,'In Earth orbit','','1992-02-06 00:00:00',102.18,825,899,66.67,'Intermediate' from dual union all
select 5532,'005532','1971-015BW','deb Kosmos-397','',24833,'Reentered','1981-10-16 00:00:00','1981-10-06 00:00:00',95.27,429,635,65.8,'Intermediate' from dual union all
select 5533,'005533','1971-015BX','deb Kosmos-397','',24833,'Reentered','1999-10-30 00:00:00','1992-02-06 00:00:00',101.16,519,1109,65.65,'Intermediate' from dual union all
select 5534,'005534','1971-015BY','deb Kosmos-397','',24833,'Reentered','1990-12-28 00:00:00','1990-12-28 00:00:00',89.15,219,246,66.12,'Intermediate' from dual union all
select 5535,'005535','1970-025LQ','deb Agena D','',21502,'Reentered','1972-03-24 00:00:00','1971-10-06 00:00:00',96.65,375,822,100.01,'Sun-Synch' from dual union all
select 5536,'005536','1971-085A','Kosmos-443','Zenit-2M',24903,'Landed','1971-10-19 00:00:00','1971-10-09 00:00:00',89.51,201,300,65.41,'Intermediate' from dual union all
select 5537,'005537','1971-085B','11A57I','11A57I',24903,'Reentered','1971-10-13 00:00:00','1971-10-09 00:00:00',89.28,196,282,65.41,'Intermediate' from dual union all
select 5538,'005538','1971-085C','Sensor cover','',24903,'Reentered','1971-10-14 00:00:00','1971-10-08 00:00:00',89.07,186,270,65.34,'Intermediate' from dual union all
select 5539,'005539','1971-085D','Sensor cover','',24903,'Reentered','1971-10-10 00:00:00','1971-10-08 00:00:00',89.13,191,272,65.46,'Intermediate' from dual union all
select 554,'000554','1963-006A','Kosmos-13','Zenit-2 No. 9',9237,'Landed','1963-03-29 00:00:00','1963-03-23 00:00:00',89.69,201,318,64.95,'Intermediate' from dual union all
select 5540,'005540','1964-083L','deb VE-5','',10369,'Reentered','1979-02-20 00:00:00','1971-10-08 00:00:00',103.73,889,981,89.85,'Polar' from dual union all
select 5541,'005541','1971-081E','KDU','KDU',24899,'Reentered','1971-10-13 00:00:00','1971-10-09 00:00:00',89.09,172,287,65.04,'Intermediate' from dual union all
select 5542,'005542','1971-081F','Blenda','BZA',24899,'Reentered','1971-10-11 00:00:00','1971-10-09 00:00:00',88.87,179,259,65,'Intermediate' from dual union all
select 5543,'005543','1971-081G','PTDU cover','KDU part',24899,'Reentered','1971-10-12 00:00:00','1971-10-11 00:00:00',90.62,205,405,65.09,'Intermediate' from dual union all
select 5544,'005544','1971-084C','Blenda','BZA',24902,'Reentered','1971-10-14 00:00:00','1971-10-11 00:00:00',89.15,177,288,72.88,'Intermediate' from dual union all
select 5545,'005545','1971-084D','KDU','KDU',24902,'Reentered','1971-10-17 00:00:00','1971-10-11 00:00:00',89.36,178,308,72.84,'Intermediate' from dual union all
select 5546,'005546','1971-084E','PTDU cover','KDU part',24902,'Reentered','1971-10-19 00:00:00','1971-10-14 00:00:00',89.02,181,272,72.83,'Intermediate' from dual union all
select 5547,'005547','1971-086A','Kosmos-444','Strela-1M',24904,'In Earth orbit','','1971-11-18 00:00:00',114.12,1322,1508,74.03,'Intermediate' from dual union all
select 5548,'005548','1971-086B','Kosmos-445','Strela-1M',24904,'In Earth orbit','','1971-11-16 00:00:00',114.47,1351,1512,74.03,'Intermediate' from dual union all
select 5549,'005549','1971-086C','Kosmos-446','Strela-1M',24904,'In Earth orbit','','1971-11-13 00:00:00',114.81,1382,1511,74.03,'Intermediate' from dual union all
select 555,'000555','1963-006B','8A92E','8A92E',9237,'Reentered','1963-04-09 00:00:00','1963-04-01 00:00:00',89.32,193,289,64.94,'Intermediate' from dual union all
select 5550,'005550','1971-086D','Kosmos-447','Strela-1M',24904,'In Earth orbit','','1971-11-13 00:00:00',115.16,1412,1514,74.03,'Intermediate' from dual union all
select 5551,'005551','1971-086E','Kosmos-448','Strela-1M',24904,'In Earth orbit','','1971-11-12 00:00:00',115.52,1441,1517,74.03,'Intermediate' from dual union all
select 5552,'005552','1971-086F','Kosmos-449','Strela-1M',24904,'In Earth orbit','','1971-11-17 00:00:00',116.25,1483,1542,74.02,'Intermediate' from dual union all
select 5553,'005553','1971-086G','Kosmos-450','Strela-1M',24904,'In Earth orbit','','1971-11-18 00:00:00',115.89,1462,1530,74.03,'Intermediate' from dual union all
select 5554,'005554','1971-086H','Kosmos-451','Strela-1M',24904,'In Earth orbit','','1971-11-12 00:00:00',116.67,1489,1574,74.02,'Intermediate' from dual union all
select 5555,'005555','1971-086J','S3M','S3M',24904,'In Earth orbit','','1971-11-17 00:00:00',117.38,1500,1627,74.02,'Intermediate' from dual union all
select 5556,'005556','1971-087B','Burner IIA No. 10 Stage 2','Star 26B S/N 001',24905,'In Earth orbit','','1971-11-16 00:00:00',101.7,791,887,98.94,'Sun-Synch' from dual union all
select 5557,'005557','1971-087A','DMSP 4527','DMSP Block 5B F-1 (SV-2)',24905,'In Earth orbit','','1971-11-13 00:00:00',101.62,793,878,98.96,'Sun-Synch' from dual union all
select 5558,'005558','1971-088A','Kosmos-452','Zenit-4M',24906,'Landed','1971-10-27 00:00:00','1971-10-17 00:00:00',89.06,175,280,64.97,'Intermediate' from dual union all
select 5559,'005559','1971-088B','11A57I','11A57I',24906,'Reentered','1971-10-18 00:00:00','1971-10-16 00:00:00',88.56,186,221,65,'Intermediate' from dual union all
select 556,'000556','61 Omicron 181','deb r/b Ablestar','',7273,'Reentered','1990-05-28 00:00:00','1990-05-27 00:00:00',89.94,255,288,67.09,'Intermediate' from dual union all
select 5560,'005560','1971-089A','P71-2','ASTEX P71-2',24907,'In Earth orbit','','1971-11-25 00:00:00',100.57,772,800,92.73,'Polar' from dual union all
select 5561,'005561','1971-085E','Nauka part','',24903,'Reentered','1971-10-21 00:00:00','1971-10-18 00:00:00',89.15,199,265,65.41,'Intermediate' from dual union all
select 5562,'005562','1971-085F','Kapsula Kosmosa-443','Nauka 8KS No. 2',24903,'Reentered','1971-10-30 00:00:00','1971-10-18 00:00:00',89.36,198,288,65.42,'Intermediate' from dual union all
select 5563,'005563','1971-090A','Kosmos-453','DS-P1-Yu No. 44',24908,'Reentered','1972-03-19 00:00:00','1971-11-18 00:00:00',91.91,266,471,70.99,'Intermediate' from dual union all
select 5564,'005564','1971-090B','S1M','S1M',24908,'Reentered','1972-01-01 00:00:00','1971-11-18 00:00:00',91.46,261,431,70.98,'Intermediate' from dual union all
select 5565,'005565','1971-091A','Delta 86','DSV-3E-3 86',24909,'Reentered','1972-07-21 00:00:00','1971-11-22 00:00:00',102.19,301,1423,102.58,'Retrograde' from dual union all
select 5566,'005566','1971-091B','deb Delta 86','',24909,'Reentered','1971-11-18 00:00:00','1971-11-09 00:00:00',96.03,174,963,102.91,'Retrograde' from dual union all
select 5567,'005567','1965-082TH','deb Transtage 4','',11832,'Reentered','1972-11-21 00:00:00','1972-10-23 00:00:00',93.12,420,435,32.36,'Intermediate' from dual union all
select 5568,'005568','1965-082TJ','deb Transtage 4','',11832,'Reentered','1981-01-24 00:00:00','1971-10-16 00:00:00',107.69,786,1454,32.85,'Intermediate' from dual union all
select 5569,'005569','1965-082TK','deb Transtage 4','',11832,'Reentered','1980-01-22 00:00:00','1980-09-15 00:00:00',97.25,590,665,31.75,'Intermediate' from dual union all
select 557,'000557','61 Omicron 182','deb r/b Ablestar','',7273,'Reentered','1967-02-14 00:00:00','1963-03-26 00:00:00',97.96,459,864,67.55,'Intermediate' from dual union all
select 5570,'005570','1971-091C','deb Delta 86','',24909,'Reentered','1972-02-09 00:00:00','1971-11-21 00:00:00',101.01,268,1345,102.59,'Retrograde' from dual union all
select 5571,'005571','1965-082TL','deb Transtage 4','',11832,'Reentered','1981-04-06 00:00:00','1971-10-17 00:00:00',101.06,714,904,32.04,'Intermediate' from dual union all
select 5572,'005572','1965-082TM','deb Transtage 4','',11832,'Reentered','1975-08-24 00:00:00','1975-06-27 00:00:00',99.25,628,818,32.37,'Intermediate' from dual union all
select 5573,'005573','1970-019C','deb Blok E','[UNID044]',21496,'Reentered','1974-11-03 00:00:00','1971-10-25 00:00:00',96.69,456,744,81.08,'Intermediate' from dual union all
select 5574,'005574','1968-091CA','deb Kosmos-249  [*]','',16818,'Reentered','2012-10-31 00:00:00','1992-02-07 00:00:00',104.2,519,1395,62.29,'Intermediate' from dual union all
select 5575,'005575','1971-092A','OPS 7616','GAMBIT-3 Mission 4333',24910,'Reentered Attached','1971-11-17 00:00:00','1971-10-30 00:00:00',89.73,130,393,110.94,'Retrograde' from dual union all
select 5576,'005576','1971-091D','deb Delta 86','',24909,'Reentered','1983-07-23 00:00:00','1971-11-25 00:00:00',105.88,629,1443,103.42,'Sun-Synch' from dual;
insert into SATELLITE
select 5577,'005577','1971-088C','Blenda','BZA',24906,'Reentered','1971-10-28 00:00:00','1971-10-26 00:00:00',88.97,171,277,65.09,'Intermediate' from dual union all
select 5578,'005578','1971-088D','KDU','KDU',24906,'Reentered','1971-10-31 00:00:00','1971-10-26 00:00:00',89.14,175,288,64.95,'Intermediate' from dual union all
select 5579,'005579','1971-088E','PTDU cover','KDU part',24906,'Reentered','1971-10-30 00:00:00','1971-10-27 00:00:00',90.78,252,373,64.94,'Intermediate' from dual union all
select 558,'000558','61 Omicron 183','deb r/b Ablestar','',7273,'In Earth orbit','','1963-05-20 00:00:00',112.59,964,1728,65.71,'Intermediate' from dual union all
select 5580,'005580','1971-093A','Prospero','X-3',24911,'In Earth orbit','','1971-11-28 00:00:00',106.45,545,1580,82.06,'Intermediate' from dual union all
select 5581,'005581','1971-093B','Waxwing R-3','Waxwing',24911,'In Earth orbit','','1971-11-28 00:00:00',106.55,545,1589,82.06,'Intermediate' from dual union all
select 5582,'005582','1971-093C','Prospero aerial','',24911,'Reentered','1979-12-16 00:00:00','1971-11-28 00:00:00',106.54,547,1586,82.15,'Intermediate' from dual union all
select 5583,'005583','1971-074G','deb Kosmos-436','Tselina-O part',24892,'Reentered','1972-07-30 00:00:00','1971-10-30 00:00:00',94.89,492,534,73.99,'Intermediate' from dual union all
select 5584,'005584','1971-074H','deb Kosmos-436','Tselina-O part',24892,'Reentered','1972-04-19 00:00:00','1971-10-30 00:00:00',94.69,477,531,74.05,'Intermediate' from dual union all
select 5585,'005585','1971-094A','Kosmos-454','Zenit-4M',24912,'Landed','1971-11-16 00:00:00','1971-11-04 00:00:00',90.02,199,352,65.44,'Intermediate' from dual union all
select 5586,'005586','1971-094B','11A57I','11A57I',24912,'Reentered','1971-11-06 00:00:00','1971-11-04 00:00:00',88.72,183,240,65.41,'Intermediate' from dual union all
select 5587,'005587','1971-095A','DSCS II F-1','DSCS II A-1',24913,'In Earth orbit','','1971-12-10 00:00:00',1436.14,35783,35792,2.56,'Stationary' from dual union all
select 5588,'005588','1971-095B','DSCS II F-2','DSCS II A-2',24913,'In Earth orbit','','1971-12-12 00:00:00',1435.78,35773,35788,2.58,'Stationary' from dual union all
select 5589,'005589','1971-095C','Transtage 21','Transtage 21',24913,'In Earth orbit','','1977-02-20 00:00:00',1481.28,36023,37310,1.99,'Drift GEO' from dual union all
select 559,'000559','61 Omicron 184','deb r/b Ablestar','',7273,'In Earth orbit','','1963-03-27 00:00:00',104.15,851,1058,66.6,'Intermediate' from dual union all
select 5590,'005590','1971-088F','PTDU cover','KDU part',24906,'Reentered','1971-11-16 00:00:00','1971-11-09 00:00:00',90.96,200,443,65.19,'Intermediate' from dual union all
select 5591,'005591','1971-015BZ','deb Kosmos-397','',24833,'Reentered','2001-12-13 00:00:00','1992-02-06 00:00:00',101.63,520,1152,65.56,'Intermediate' from dual union all
select 5592,'005592','1968-097CU','deb Kosmos-252','',16824,'Reentered','1979-07-22 00:00:00','1979-03-03 00:00:00',100.13,487,1042,62.06,'Intermediate' from dual union all
select 5593,'005593','1969-082JC','deb Agena D','',18349,'Reentered','1973-05-02 00:00:00','1971-11-11 00:00:00',99.13,656,778,69.95,'Intermediate' from dual union all
select 5594,'005594','1968-097CV','deb Kosmos-252','',16824,'Reentered','1978-11-25 00:00:00','1980-05-06 00:00:00',106.62,525,1616,62.13,'Intermediate' from dual union all
select 5595,'005595','1968-097CW','Unknown deb                       ','',16824,'In Earth orbit','','1992-02-06 00:00:00',104.84,533,1440,62.77,'Intermediate' from dual union all
select 5596,'005596','1971-090C','deb Kosmos-453','',24908,'Reentered','1971-12-24 00:00:00','1971-11-19 00:00:00',91.65,318,393,71.1,'Intermediate' from dual union all
select 5597,'005597','1971-015CA','Unknown deb                       ','',24833,'In Earth orbit','','1992-02-07 00:00:00',101.94,821,880,66.6,'Intermediate' from dual union all
select 5598,'005598','1971-096A','SSS 1','SSS A',24914,'Reentered','1987-05-03 00:00:00','1971-12-24 00:00:00',465.1,234,26780,3.56,'Highly Ellip' from dual union all
select 5599,'005599','1966-025H','deb OV1-5','',13432,'In Earth orbit','','1971-10-26 00:00:00',105.29,922,1093,144.61,'Retrograde' from dual union all
select 56,'000056','60 Lambda 2','8K72E','8K72E',6325,'Reentered','1960-09-23 00:00:00','1960-08-26 00:00:00',90.49,299,299,64.9,'Intermediate' from dual union all
select 560,'000560','61 Omicron 185','deb r/b Ablestar','',7273,'In Earth orbit','','1963-03-27 00:00:00',110.76,904,1620,67.17,'Intermediate' from dual union all
select 5600,'005600','1970-025LR','deb Agena D','',21502,'Reentered','1990-04-16 00:00:00','1990-04-15 00:00:00',90.91,297,341,99.98,'Retrograde' from dual union all
select 5601,'005601','1970-025LS','deb Agena D','',21502,'Reentered','1988-11-12 00:00:00','1988-11-09 00:00:00',93.33,420,454,99.49,'Sun-Synch' from dual union all
select 5602,'005602','1971-091E','deb Delta 86','',24909,'Reentered','1974-07-01 00:00:00','1971-11-20 00:00:00',103.95,424,1467,104.48,'Retrograde' from dual union all
select 5603,'005603','1971-094C','Blenda','BZA',24912,'Reentered','1971-11-22 00:00:00','1971-11-15 00:00:00',89.77,200,327,65.46,'Intermediate' from dual union all
select 5604,'005604','1971-094D','KDU','KDU',24912,'Reentered','1971-11-25 00:00:00','1971-11-15 00:00:00',89.9,201,338,65.44,'Intermediate' from dual union all
select 5605,'005605','1971-094E','KDU part','KDU part',24912,'Reentered','1971-11-22 00:00:00','1971-11-15 00:00:00',89.14,192,272,65.51,'Intermediate' from dual union all
select 5606,'005606','1971-092B','Fairing','GAMBIT SRV-2 Fairing',24910,'Reentered','1971-11-17 00:00:00','1971-11-17 00:00:00',88.72,93,330,110.92,'Retrograde' from dual union all
select 5607,'005607','1971-094F','PTDU cover','KDU part',24912,'Reentered','1971-11-21 00:00:00','1971-11-17 00:00:00',91.57,208,496,65.45,'Intermediate' from dual union all
select 5608,'005608','1971-097A','Kosmos-455','DS-P1-Yu No. 54',24915,'Reentered','1972-04-09 00:00:00','1971-12-17 00:00:00',91.88,266,468,70.98,'Intermediate' from dual union all
select 5609,'005609','1971-097B','S1M','S1M',24915,'Reentered','1972-02-08 00:00:00','1971-12-17 00:00:00',91.39,267,419,71,'Intermediate' from dual union all
select 561,'000561','61 Omicron 186','deb r/b Ablestar','',7273,'In Earth orbit','','1963-03-27 00:00:00',117.14,973,2132,67.77,'Intermediate' from dual union all
select 5610,'005610','1971-083D','deb OSO 7','',24901,'Reentered','1971-12-20 00:00:00','1971-11-18 00:00:00',92.27,305,467,33.13,'Intermediate' from dual union all
select 5611,'005611','1971-098A','Kosmos-456','Zenit-4M',24916,'Landed','1971-12-02 00:00:00','1971-11-21 00:00:00',89.27,175,302,72.86,'Intermediate' from dual union all
select 5612,'005612','1971-098B','11A57I','11A57I',24916,'Reentered','1971-11-25 00:00:00','1971-11-21 00:00:00',89.24,195,279,72.87,'Intermediate' from dual union all
select 5613,'005613','1971-098C','Sensor cover','Zenit-4M sensor cover',24916,'Reentered','1971-11-20 00:00:00','1971-11-19 00:00:00',88.93,202,242,72.85,'Intermediate' from dual union all
select 5614,'005614','1971-099A','Kosmos-457','Sfera',24917,'In Earth orbit','','1971-12-24 00:00:00',109.45,1183,1220,74.05,'Intermediate' from dual union all
select 5615,'005615','1971-099B','S3M','S3M',24917,'In Earth orbit','','1971-12-23 00:00:00',109.32,1181,1210,74.05,'Intermediate' from dual union all
select 5616,'005616','1971-098D','Sensor cover','Zenit-4M sensor cover',24916,'Reentered','1971-11-20 00:00:00','1971-11-19 00:00:00',89.55,197,308,72.83,'Intermediate' from dual union all
select 5617,'005617','1971-100B','11S510','11S510',24918,'Reentered','1971-12-19 00:00:00','1971-12-02 00:00:00',90.61,212,397,65.41,'Intermediate' from dual union all
select 5618,'005618','1971-100C','BOZ','BOZ',24918,'Reentered','1971-12-30 00:00:00','1971-12-01 00:00:00',91.14,223,438,65.39,'Intermediate' from dual union all
select 5619,'005619','1971-100D','deb Blok-I','',24918,'Reentered','1971-11-29 00:00:00','1971-11-27 00:00:00',89.79,197,331,65.35,'Intermediate' from dual union all
select 562,'000562','1963-007A','OPS 0720','CORONA M-19',9238,'Reentered Attached','1963-04-26 00:00:00','1963-04-08 00:00:00',90.3,198,381,75.25,'Intermediate' from dual union all
select 5620,'005620','1971-100A','Molniya-2','Molniya-2',24918,'Reentered','1976-05-10 00:00:00','1972-01-01 00:00:00',705.97,453,39318,65.44,'GEO Transfer' from dual union all
select 5621,'005621','1970-025LT','deb Agena D','',21502,'Reentered','1972-03-21 00:00:00','1971-11-27 00:00:00',99.74,589,904,99.68,'Sun-Synch' from dual union all
select 5622,'005622','1970-025LU','deb Agena D','',21502,'Reentered','1991-08-06 00:00:00','1991-08-05 00:00:00',90.01,261,289,99.98,'Retrograde' from dual union all
select 5623,'005623','1971-101A','Kosmos-458','DS-P1-Yu No. 53',24919,'Reentered','1972-04-20 00:00:00','1971-12-29 00:00:00',91.93,266,473,70.98,'Intermediate' from dual union all
select 5624,'005624','1971-101B','S1M','S1M',24919,'Reentered','1972-02-14 00:00:00','1971-12-30 00:00:00',91.31,262,416,70.96,'Intermediate' from dual union all
select 5625,'005625','1971-102A','Kosmos-459','DS-P1-M No. 5',24920,'Reentered','1971-12-27 00:00:00','1971-12-08 00:00:00',89.16,217,249,65.81,'Intermediate' from dual union all
select 5626,'005626','1971-102B','S3M','S3M',24920,'Reentered','1971-12-11 00:00:00','1971-12-01 00:00:00',89.27,219,257,65.82,'Intermediate' from dual union all
select 5627,'005627','1971-102C','deb Kosmos-459','DS-P1-M part',24920,'Reentered','1971-12-04 00:00:00','1971-12-01 00:00:00',89.16,223,242,65.82,'Intermediate' from dual union all
select 5628,'005628','1971-103A','Kosmos-460','Tselina-OM',24921,'Reentered','1980-03-05 00:00:00','1971-12-30 00:00:00',95.17,516,538,74.02,'Intermediate' from dual union all
select 5629,'005629','1971-103B','S3M','S3M',24921,'Reentered','1980-05-04 00:00:00','1972-01-01 00:00:00',95.09,506,540,74.04,'Intermediate' from dual union all
select 563,'000563','1963-008A','8K78I','8K78I',9239,'Reentered','1963-04-03 00:00:00','1963-04-02 00:00:00',87.98,167,181,64.77,'Intermediate' from dual union all
select 5630,'005630','1971-103C','deb Kosmos-460','Tselina-O part',24921,'Reentered','1972-11-21 00:00:00','1971-12-30 00:00:00',94.85,495,528,74.07,'Intermediate' from dual union all
select 5631,'005631','1968-097CX','deb Kosmos-252  [*]','',16824,'Reentered','1981-10-15 00:00:00','1990-02-15 00:00:00',90.19,271,297,62.24,'Intermediate' from dual union all
select 5632,'005632','1965-108N','deb Transtage 8','',11858,'In Earth orbit','','2004-10-04 00:00:00',411.29,372,23512,26.21,'Highly Ellip' from dual union all
select 5633,'005633','1969-082JD','deb Agena D','',18349,'Reentered','1989-12-09 00:00:00','1989-12-07 00:00:00',90.77,307,318,69.88,'Intermediate' from dual union all
select 5634,'005634','1969-082JE','deb Agena D','',18349,'Reentered','1981-04-13 00:00:00','1971-11-29 00:00:00',102.61,769,995,70.27,'Intermediate' from dual union all
select 5635,'005635','1969-082JF','deb Agena D','',18349,'In Earth orbit','','1992-02-06 00:00:00',102.33,861,877,69.97,'Intermediate' from dual union all
select 5636,'005636','1969-082JG','deb Agena D','',18349,'Reentered','1978-05-13 00:00:00','1979-04-16 00:00:00',98.6,613,771,69.78,'Intermediate' from dual union all
select 5637,'005637','1969-082JH','deb Agena D','',18349,'Reentered','1991-05-18 00:00:00','1991-05-07 00:00:00',93.09,393,459,69.22,'Intermediate' from dual union all
select 5638,'005638','1970-089CJ','deb Kosmos-374  [*]','',21566,'Reentered','2003-06-17 00:00:00','1992-02-06 00:00:00',103.83,515,1365,62.5,'Intermediate' from dual union all
select 5639,'005639','1971-098E','KDU','KDU',24916,'Reentered','1971-12-10 00:00:00','1971-12-02 00:00:00',89.71,186,335,72.84,'Intermediate' from dual union all
select 564,'000564','1963-009A','Explorer 17','AE-A',9240,'Reentered','1966-11-24 00:00:00','1963-05-05 00:00:00',96.27,246,914,57.61,'Intermediate' from dual union all
select 5640,'005640','1971-098F','Blenda','BZA',24916,'Reentered','1971-12-06 00:00:00','1971-12-01 00:00:00',89.57,186,321,72.81,'Intermediate' from dual union all
select 5641,'005641','1971-104A','Interkosmos-5','DS-U2-IK No. 2',24922,'Reentered','1972-04-07 00:00:00','1972-01-01 00:00:00',97.12,198,1044,48.43,'Intermediate' from dual union all
select 5642,'005642','1971-104B','S1M','S1M',24922,'Reentered','1972-03-02 00:00:00','1972-01-01 00:00:00',96.2,195,959,48.47,'Intermediate' from dual union all
select 5643,'005643','1971-105A','Kosmos-461','DS-U2-MT No. 1',24923,'Reentered','1979-02-21 00:00:00','1972-01-02 00:00:00',94.55,486,508,69.24,'Intermediate' from dual union all
select 5644,'005644','1971-105B','S3M','S3M',24923,'Reentered','1979-04-29 00:00:00','1972-01-03 00:00:00',94.41,472,508,69.24,'Intermediate' from dual union all
select 5645,'005645','1971-104C','deb Interkosmos-5','',24922,'Reentered','1971-12-13 00:00:00','1971-12-07 00:00:00',95.71,179,926,48.94,'Intermediate' from dual union all
select 5646,'005646','1971-106A','Kosmos-462','I2P',24924,'Reentered','1975-04-04 00:00:00','1972-01-05 00:00:00',105.15,225,1778,65.74,'Intermediate' from dual union all
select 5647,'005647','1971-105C','deb Kosmos-461','',24923,'Reentered','1972-04-03 00:00:00','1972-01-09 00:00:00',93.85,443,483,69.22,'Intermediate' from dual union all
select 5648,'005648','1971-105D','deb Kosmos-461','',24923,'Reentered','1972-04-03 00:00:00','1972-01-03 00:00:00',93.87,442,485,69.23,'Intermediate' from dual union all
select 5649,'005649','1971-106B','deb R-36','',24924,'Reentered','1972-02-02 00:00:00','1972-01-12 00:00:00',102.7,257,1516,65.58,'Intermediate' from dual union all
select 565,'000565','1963-009B','Altair','Altair X-248-A5DM SU-33',9240,'Reentered','1963-11-24 00:00:00','1963-05-05 00:00:00',95.79,239,875,57.59,'Intermediate' from dual union all
select 5650,'005650','1971-106C','11S692','11S692',24924,'Reentered','1972-01-05 00:00:00','1971-12-10 00:00:00',100.68,142,1439,62.31,'Intermediate' from dual union all
select 5651,'005651','1971-106D','deb Kosmos-462','',24924,'Reentered','1972-02-09 00:00:00','1972-01-14 00:00:00',98.48,214,1157,65.71,'Intermediate' from dual union all
select 5652,'005652','1971-106E','deb Kosmos-462','',24924,'Reentered','1971-12-24 00:00:00','1971-12-19 00:00:00',98.13,209,1130,65.92,'Intermediate' from dual union all
select 5653,'005653','1971-098G','KDU part','KDU part',24916,'Reentered','1971-12-05 00:00:00','1971-12-04 00:00:00',88.29,168,212,72.85,'Intermediate' from dual union all
select 5654,'005654','1971-106F','deb Kosmos-462','',24924,'Reentered','1972-02-10 00:00:00','1972-01-17 00:00:00',98.06,211,1121,65.85,'Intermediate' from dual union all
select 5655,'005655','1971-106G','deb Kosmos-462','',24924,'Reentered','1972-03-31 00:00:00','1972-01-06 00:00:00',102.72,220,1554,66.04,'Intermediate' from dual union all
select 5656,'005656','1971-106H','deb Kosmos-462','',24924,'Reentered','1971-12-24 00:00:00','1971-12-14 00:00:00',100.07,217,1307,65.76,'Intermediate' from dual union all
select 5657,'005657','1971-098H','PTDU cover','KDU part',24916,'Reentered','1971-12-06 00:00:00','1971-12-04 00:00:00',89.86,198,337,73.07,'Intermediate' from dual union all
select 5658,'005658','1971-106J','deb Kosmos-462','',24924,'Reentered','1971-12-19 00:00:00','1971-12-10 00:00:00',100.1,236,1290,65.78,'Intermediate' from dual union all
select 5659,'005659','1971-106K','deb Kosmos-462','',24924,'Reentered','1972-02-18 00:00:00','1972-01-09 00:00:00',100.75,215,1374,65.75,'Intermediate' from dual union all
select 566,'000566','1963-008B','Luna-4','E-6 No. 4',9239,'Deep Space','1963-04-06 00:00:00','1963-04-02 00:00:00',34858.07,199,694000,65,'Cislunar' from dual union all
select 5660,'005660','1971-106L','deb Kosmos-462','',24924,'Reentered','1972-03-20 00:00:00','1972-01-03 00:00:00',106.76,216,1938,65.33,'Intermediate' from dual union all
select 5661,'005661','1971-107A','Kosmos-463','Zenit-4M',24925,'Landed','1971-12-11 00:00:00','1971-12-08 00:00:00',88.94,182,263,64.96,'Intermediate' from dual union all
select 5662,'005662','1971-107B','11A57I','11A57I',24925,'Reentered','1971-12-11 00:00:00','1971-12-08 00:00:00',88.83,189,245,65.03,'Intermediate' from dual union all
select 5663,'005663','1971-107C','Sensor cover','Zenit-4M sensor cover',24925,'Reentered','1971-12-09 00:00:00','1971-12-08 00:00:00',88.97,194,254,64.98,'Intermediate' from dual union all
select 5664,'005664','1971-106M','Kosmos-462 DU','Large debris',24924,'Reentered','1971-12-11 00:00:00','1971-12-06 00:00:00',97.66,141,1153,62.27,'Intermediate' from dual union all
select 5665,'005665','1971-106N','deb Kosmos-462','',24924,'Reentered','1971-12-11 00:00:00','1971-12-07 00:00:00',96.39,141,1031,62.31,'Intermediate' from dual union all
select 5666,'005666','1971-106P','deb Kosmos-462','',24924,'Reentered','1972-03-09 00:00:00','1972-01-03 00:00:00',103.12,222,1591,65.79,'Intermediate' from dual;
insert into SATELLITE
select 5667,'005667','1971-049E','SA Marsa-3','SA',24867,'Deep Space','1972-05-29 00:00:00','1971-12-02 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 5668,'005668','1971-106Q','deb Kosmos-462','',24924,'Reentered','1972-01-26 00:00:00','1971-12-10 00:00:00',105.16,226,1777,65.78,'Intermediate' from dual union all
select 5669,'005669','1971-107D','KDU','KDU',24925,'Reentered','1971-12-15 00:00:00','1971-12-10 00:00:00',89.16,179,287,64.97,'Intermediate' from dual union all
select 567,'000567','1963-010A','Kosmos-14','Omega-1 No. 1',9241,'Reentered','1963-08-29 00:00:00','1963-05-16 00:00:00',91.6,262,445,48.95,'Intermediate' from dual union all
select 5670,'005670','1971-108A','Kosmos-464','Zenit-4M',24926,'Landed','1971-12-16 00:00:00','1971-12-12 00:00:00',90.31,203,376,72.86,'Intermediate' from dual union all
select 5671,'005671','1971-108B','11A57I','11A57I',24926,'Reentered','1971-12-24 00:00:00','1971-12-12 00:00:00',90.11,202,358,72.88,'Intermediate' from dual union all
select 5672,'005672','1971-106R','deb Kosmos-462','',24924,'Reentered','1971-12-13 00:00:00','1971-12-09 00:00:00',94.17,139,818,62.21,'Intermediate' from dual union all
select 5673,'005673','1971-106S','deb Kosmos-462','',24924,'Reentered','1972-03-18 00:00:00','1972-01-08 00:00:00',103.77,240,1634,65.41,'Intermediate' from dual union all
select 5674,'005674','1971-108C','Sensor cover','Zenit-4M sensor cover',24926,'Reentered','1971-12-27 00:00:00','1971-12-17 00:00:00',89.27,185,291,72.85,'Intermediate' from dual union all
select 5675,'005675','1971-109A','Ariel 4','UK 4',24927,'Reentered','1978-12-12 00:00:00','1972-01-10 00:00:00',95.26,473,589,82.99,'Intermediate' from dual union all
select 5676,'005676','1971-109B','FW4S S/N 2223-12','FW4S',24927,'Reentered','1978-08-09 00:00:00','1972-01-10 00:00:00',95.24,473,588,83,'Intermediate' from dual union all
select 5677,'005677','1971-109C','Ariel 4 despin weight','',24927,'Reentered','1973-01-18 00:00:00','1972-01-10 00:00:00',94.91,499,530,82.85,'Intermediate' from dual union all
select 5678,'005678','1971-110A','OPS 7898 P/L 1','POPPY 7 SV1',24928,'In Earth orbit','','1977-01-01 00:00:00',104.89,985,994,69.99,'Intermediate' from dual union all
select 5679,'005679','1971-110B','Agena D 2707','Agena D',24928,'In Earth orbit','','1990-04-17 00:00:00',104.04,929,970,70,'Intermediate' from dual union all
select 568,'000568','1963-010B','S1','S1',9241,'Reentered','1963-07-06 00:00:00','1963-05-20 00:00:00',91.27,248,426,48.94,'Intermediate' from dual union all
select 5680,'005680','1971-110C','OPS 7898 P/L 2','POPPY 7 SV2',24928,'In Earth orbit','','1996-11-09 00:00:00',104.84,987,987,70,'Intermediate' from dual union all
select 5681,'005681','1971-110D','OPS 7898 P/L 3','POPPY 7 SV3',24928,'In Earth orbit','','1997-01-21 00:00:00',104.8,978,992,70,'Intermediate' from dual union all
select 5682,'005682','1971-110E','OPS 7898 P/L 4','POPPY 7 SV4',24928,'In Earth orbit','','1997-01-21 00:00:00',104.83,979,994,70,'Intermediate' from dual union all
select 5683,'005683','1971-111A','Kosmos-465','Tsiklon',24929,'In Earth orbit','','1972-01-18 00:00:00',104.89,969,1010,74.03,'Intermediate' from dual union all
select 5684,'005684','1971-108D','Blenda','BZA',24926,'Reentered','1971-12-17 00:00:00','1971-12-15 00:00:00',88.92,173,269,72.86,'Intermediate' from dual union all
select 5685,'005685','1971-111B','S3M','S3M',24929,'In Earth orbit','','1972-01-22 00:00:00',104.77,967,1001,74.03,'Intermediate' from dual union all
select 5686,'005686','1971-108E','KDU','KDU',24926,'Reentered','1971-12-19 00:00:00','1971-12-15 00:00:00',89.02,164,289,72.8,'Intermediate' from dual union all
select 5687,'005687','1971-112A','Kosmos-466','Zenit-4M',24930,'Landed','1971-12-27 00:00:00','1971-12-19 00:00:00',89.04,178,277,64.99,'Intermediate' from dual union all
select 5688,'005688','1971-112B','11A57I','11A57I',24930,'Reentered','1971-12-21 00:00:00','1971-12-19 00:00:00',88.62,182,231,65.01,'Intermediate' from dual union all
select 5689,'005689','1971-100E','Blok-L','Blok-L',24918,'Reentered','1974-01-05 00:00:00','1972-01-17 00:00:00',703.6,463,39190,65.42,'GEO Transfer' from dual union all
select 569,'000569','1963-011A','Kosmos-15','Zenit-2 No. 8',9242,'Landed','1963-04-27 00:00:00','1963-04-24 00:00:00',89.67,160,356,64.97,'Intermediate' from dual union all
select 5690,'005690','1970-025LV','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.76,1040,1114,99.77,'Sun-Synch' from dual union all
select 5691,'005691','1970-025LW','deb Agena D','',21502,'In Earth orbit','','1992-02-07 00:00:00',104.7,953,1008,99.23,'Sun-Synch' from dual union all
select 5692,'005692','1970-025LX','deb Agena D','',21502,'In Earth orbit','','1992-02-07 00:00:00',106.81,990,1168,99.49,'Sun-Synch' from dual union all
select 5693,'005693','1970-025LY','deb Agena D','',21502,'Reentered','1991-03-29 00:00:00','1991-03-28 00:00:00',89.82,256,276,99.95,'Retrograde' from dual union all
select 5694,'005694','1970-025LZ','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',104.08,909,993,99.85,'Sun-Synch' from dual union all
select 5695,'005695','1970-025MA','deb Agena D','',21502,'Reentered','1984-12-13 00:00:00','1984-12-09 00:00:00',90.85,306,327,99.92,'Retrograde' from dual union all
select 5696,'005696','1970-025MB','deb Agena D','',21502,'In Earth orbit','','1992-02-07 00:00:00',106.84,1037,1124,99.41,'Sun-Synch' from dual union all
select 5697,'005697','1970-025MC','deb Agena D','',21502,'Reentered','1991-04-01 00:00:00','1991-04-01 00:00:00',90.22,257,314,99.95,'Retrograde' from dual union all
select 5698,'005698','1970-025MD','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.4,1053,1067,99.45,'Sun-Synch' from dual union all
select 5699,'005699','1970-025ME','deb Agena D','',21502,'Reentered','1989-12-15 00:00:00','1989-12-10 00:00:00',92.27,367,405,99.97,'Retrograde' from dual union all
select 57,'000057','60 Mu','Discoverer 15','CORONA C-10',6326,'Reentered Attached','1960-10-18 00:00:00','1960-09-23 00:00:00',93.16,200,658,80.9,'Intermediate' from dual union all
select 570,'000570','1963-011B','8A92E','8A92E',9242,'Reentered','1963-05-01 00:00:00','1963-04-25 00:00:00',89.48,169,328,64.95,'Intermediate' from dual union all
select 5700,'005700','1970-025MF','deb Agena D','',21502,'Reentered','1979-06-16 00:00:00','1971-12-13 00:00:00',101.29,574,1066,99.69,'Sun-Synch' from dual union all
select 5701,'005701','1970-025MG','deb Agena D','',21502,'Reentered','1990-01-27 00:00:00','1990-01-26 00:00:00',90.86,293,341,99.98,'Retrograde' from dual union all
select 5702,'005702','1963-014CU','deb West Ford','RCS 0.20',9245,'Reentered','1987-10-17 00:00:00','2000-08-04 00:00:00',166.11,2824,4440,87.2,'Medium' from dual union all
select 5703,'005703','62 Kappa 6','West Ford Drag dipole','',8192,'Reentered','1976-11-07 00:00:00','1962-06-07 00:00:00',152.59,2784,3378,86.65,'Medium' from dual union all
select 5704,'005704','1971-113A','Kosmos-467','DS-P1-Yu No. 45',24931,'Reentered','1972-04-18 00:00:00','1972-01-16 00:00:00',91.67,262,451,70.99,'Intermediate' from dual union all
select 5705,'005705','1971-114A','Kosmos-468','Strela-2M',24932,'In Earth orbit','','1972-01-17 00:00:00',100.77,784,807,74.02,'Intermediate' from dual union all
select 5706,'005706','1971-113B','S1M','S1M',24931,'Reentered','1972-02-17 00:00:00','1972-01-16 00:00:00',91.07,252,403,70.99,'Intermediate' from dual union all
select 5707,'005707','1971-114B','S3M','S3M',24932,'In Earth orbit','','1972-01-17 00:00:00',100.71,775,810,74.03,'Intermediate' from dual union all
select 5708,'005708','1971-115C','11S510','11S510',24933,'Reentered','1972-01-26 00:00:00','1971-12-26 00:00:00',91.33,220,460,65.4,'Intermediate' from dual union all
select 5709,'005709','1971-116A','INTELSAT IV F3','INTELSAT IV F3',24934,'In Earth orbit','','1977-01-13 00:00:00',1436.16,35779,35797,.2,'Stationary' from dual union all
select 571,'000571','1963-012A','Kosmos-16','Zenit-2 No. 10',9243,'Landed','1963-05-08 00:00:00','1963-04-30 00:00:00',90.3,195,384,65.04,'Intermediate' from dual union all
select 5710,'005710','1971-115D','BOZ','BOZ',24933,'Reentered','1972-01-27 00:00:00','1971-12-26 00:00:00',91.42,219,470,65.39,'Intermediate' from dual union all
select 5711,'005711','1965-082TN','deb Transtage 4','',11832,'Reentered','1980-06-21 00:00:00','1979-10-14 00:00:00',96.38,566,605,32.06,'Intermediate' from dual union all
select 5712,'005712','1971-115A','Molniya-1','Molniya-1 11F67 No. 25',24933,'Reentered','1977-04-13 00:00:00','1972-01-18 00:00:00',717.78,432,39923,65.47,'GEO Transfer' from dual union all
select 5713,'005713','1971-115B','Blok-L','Blok-L',24933,'Reentered','1977-09-22 00:00:00','1972-01-19 00:00:00',699.21,440,38994,65.45,'GEO Transfer' from dual union all
select 5714,'005714','1968-091CB','deb Kosmos-249','',16818,'Reentered','2000-04-06 00:00:00','1992-02-06 00:00:00',99.77,394,1102,62.34,'Intermediate' from dual union all
select 5715,'005715','61 Omicron 233','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-06 00:00:00',106.76,940,1214,66.59,'Intermediate' from dual union all
select 5716,'005716','61 Omicron 234','deb r/b Ablestar','',7273,'Reentered','1982-04-09 00:00:00','2000-10-20 00:00:00',100.22,690,848,66.7,'Intermediate' from dual union all
select 5717,'005717','1968-091CC','deb Kosmos-249','',16818,'Reentered','1982-05-14 00:00:00','1971-12-22 00:00:00',110.4,559,1932,62.54,'Intermediate' from dual union all
select 5718,'005718','1971-074J','deb Kosmos-436','Tselina-O part',24892,'Reentered','1972-06-07 00:00:00','1971-12-23 00:00:00',94.65,484,520,74.02,'Intermediate' from dual union all
select 5719,'005719','1965-082TP','deb Transtage 4','',11832,'Reentered','1975-07-30 00:00:00','1971-12-31 00:00:00',99.68,705,781,32,'Intermediate' from dual union all
select 572,'000572','1963-012B','8A92E','8A92E',9243,'Reentered','1963-05-20 00:00:00','1963-05-07 00:00:00',89.92,191,350,64.98,'Intermediate' from dual union all
select 5720,'005720','1971-015CB','Unknown deb                       ','',24833,'In Earth orbit','','1992-02-06 00:00:00',102.63,809,958,66.73,'Intermediate' from dual union all
select 5721,'005721','1971-117A','Kosmos-469','US-A reactor',24935,'In Earth orbit','','1972-01-29 00:00:00',104.7,944,1017,64.48,'Intermediate' from dual union all
select 5722,'005722','1971-052C','deb Kosmos-426','',24870,'Reentered','1973-03-24 00:00:00','1971-08-30 00:00:00',108.5,385,1930,74.04,'Intermediate' from dual union all
select 5723,'005723','1971-052D','deb Kosmos-426','',24870,'Reentered','1975-01-01 00:00:00','1971-12-25 00:00:00',107.8,379,1871,74.04,'Intermediate' from dual union all
select 5724,'005724','1971-028D','deb Kosmos-405','',24846,'Reentered','2002-04-14 00:00:00','1971-12-26 00:00:00',98.26,672,679,81.23,'Intermediate' from dual union all
select 5725,'005725','1971-112C','KDU','KDU',24930,'Reentered','1972-01-03 00:00:00','1971-12-27 00:00:00',89.76,172,354,65.02,'Intermediate' from dual union all
select 5726,'005726','1971-112D','Blenda','BZA',24930,'Reentered','1971-12-31 00:00:00','1971-12-27 00:00:00',89.54,150,354,64,'Intermediate' from dual union all
select 5727,'005727','1971-118A','Kosmos-470','Zenit-4MT',24936,'Landed','1972-01-06 00:00:00','1971-12-29 00:00:00',88.99,192,258,65.41,'Intermediate' from dual union all
select 5728,'005728','1971-118B','11S510','11S510',24936,'Reentered','1971-12-30 00:00:00','1971-12-29 00:00:00',88.51,170,232,65.37,'Intermediate' from dual union all
select 5729,'005729','1971-119A','Aureole 1','DS-U2-GKA No. 1',24937,'In Earth orbit','','1972-01-30 00:00:00',114.58,399,2474,73.99,'Intermediate' from dual union all
select 573,'000573','1963-013A','Telstar 2','Telstar 2',9244,'In Earth orbit','','1963-06-13 00:00:00',225.33,972,10803,42.72,'Medium' from dual union all
select 5730,'005730','1971-119B','S3M','S3M',24937,'In Earth orbit','','1972-01-28 00:00:00',114.43,394,2465,74.01,'Intermediate' from dual union all
select 5731,'005731','1971-120A','Meteor','Meteor-MV No. 18',24938,'In Earth orbit','','1972-01-31 00:00:00',102.64,840,928,81.25,'Intermediate' from dual union all
select 5732,'005732','1971-120B','8A92ME','8A92ME',24938,'In Earth orbit','','1972-01-28 00:00:00',102.59,873,890,81.24,'Intermediate' from dual union all
select 5733,'005733','1965-020FA','deb Blok Kosmos','',11770,'Reentered','1972-03-11 00:00:00','1972-01-02 00:00:00',94.25,369,596,55.9,'Intermediate' from dual union all
select 5734,'005734','1965-082TQ','deb Transtage 4','',11832,'Reentered','1980-07-03 00:00:00','1972-02-25 00:00:00',99.81,664,835,32.19,'Intermediate' from dual union all
select 5735,'005735','1967-001L','deb unknown                       ','[UNID045]',15256,'Reentered','1973-03-09 00:00:00','1972-01-01 00:00:00',404.04,353,23099,26.93,'Highly Ellip' from dual union all
select 5736,'005736','1970-025MH','deb Agena D','',21502,'In Earth orbit','','1992-02-07 00:00:00',110.32,1059,1424,101.39,'Sun-Synch' from dual union all
select 5737,'005737','1971-117B','Kosmos-469 DU','US-A DU',24935,'Reentered','1972-02-09 00:00:00','1972-01-04 00:00:00',89.6,244,266,64.96,'Intermediate' from dual union all
select 5738,'005738','1971-117C','Kosmos-469 RLS','US-A RLS',24935,'Reentered','1972-01-07 00:00:00','1972-01-04 00:00:00',89.46,242,254,64.96,'Intermediate' from dual union all
select 5739,'005739','1971-045D','SA Marsa-2','SA',24863,'Deep Space Attached','1971-05-22 00:00:00','',null,null,null,null,'' from dual union all
select 574,'000574','1963-014A','Midas 7','Midas ETS-III F2',9245,'In Earth orbit','','1963-06-11 00:00:00',166.35,3569,3714,87.38,'Medium' from dual union all
select 5740,'005740','1971-118C','KB electropitaniya','KBE',24936,'Reentered','1972-01-06 00:00:00','1972-01-06 00:00:00',88.41,182,210,65.43,'Intermediate' from dual union all
select 5741,'005741','1971-118D','Blenda','',24936,'Reentered','1972-01-06 00:00:00','1972-01-06 00:00:00',88.8,108,323,65.39,'Intermediate' from dual union all
select 5742,'005742','1971-118E','KBE part','',24936,'Reentered','1972-01-07 00:00:00','1972-01-06 00:00:00',87.5,89,213,65.33,'Intermediate' from dual union all
select 5743,'005743','1968-017E','deb Explorer 37','',16744,'Reentered','1977-10-25 00:00:00','1972-01-06 00:00:00',98.87,423,987,59.31,'Intermediate' from dual union all
select 5744,'005744','1965-082TR','deb Transtage 4','',11832,'Reentered','1978-11-21 00:00:00','1978-02-25 00:00:00',93,395,448,31.73,'Intermediate' from dual union all
select 5745,'005745','61 Omicron 235','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-05 00:00:00',104.8,884,1086,67.07,'Intermediate' from dual union all
select 5746,'005746','61 Omicron 236','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-06 00:00:00',103.92,907,981,66.31,'Intermediate' from dual union all
select 5747,'005747','61 Omicron 237','deb r/b Ablestar','',7273,'Reentered','1977-01-03 00:00:00','1972-01-07 00:00:00',103.76,882,991,66.85,'Intermediate' from dual union all
select 5748,'005748','61 Omicron 238','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-06 00:00:00',106.44,950,1174,67.57,'Intermediate' from dual union all
select 5749,'005749','1968-097CZ','deb Kosmos-252','',16824,'Reentered','1976-10-12 00:00:00','1972-01-08 00:00:00',108.27,551,1743,62.35,'Intermediate' from dual union all
select 575,'000575','1963-013B','Altair','Altair X-248-A5DM',9244,'In Earth orbit','','1963-10-22 00:00:00',225.09,965,10794,42.74,'Medium' from dual union all
select 5750,'005750','1969-082JJ','deb Agena D','',18349,'Reentered','1977-01-23 00:00:00','1972-01-08 00:00:00',101.43,742,911,69.88,'Intermediate' from dual union all
select 5751,'005751','1969-082JK','deb Agena D','',18349,'Reentered','1980-01-02 00:00:00','1972-01-08 00:00:00',100.09,662,864,69.42,'Intermediate' from dual union all
select 5752,'005752','1969-082JL','deb Agena D','',18349,'Reentered','1976-10-12 00:00:00','1972-01-08 00:00:00',100.76,710,879,69.95,'Intermediate' from dual union all
select 5753,'005753','1969-082JM','deb Agena D','',18349,'Reentered','1979-10-06 00:00:00','1979-04-09 00:00:00',98.77,640,760,69.7,'Intermediate' from dual union all
select 5754,'005754','1969-082JN','deb Agena D','',18349,'Reentered','1983-07-19 00:00:00','1983-07-24 00:00:00',91.62,299,409,70.32,'Intermediate' from dual union all
select 5755,'005755','1969-082JP','deb Agena D','',18349,'In Earth orbit','','1992-02-04 00:00:00',102.05,850,862,69.97,'Intermediate' from dual union all
select 5756,'005756','1969-082JQ','deb Agena D','',18349,'Reentered','1974-10-26 00:00:00','1972-01-02 00:00:00',102.15,725,996,69.93,'Intermediate' from dual;
insert into SATELLITE
select 5757,'005757','1969-082JR','deb Agena D','',18349,'Reentered','1991-12-20 00:00:00','1991-12-20 00:00:00',88.5,154,246,70.63,'Intermediate' from dual union all
select 5758,'005758','1969-082JS','deb Agena D','',18349,'In Earth orbit','','1992-01-28 00:00:00',104.37,932,998,69.81,'Intermediate' from dual union all
select 5759,'005759','1970-089CK','Unknown deb                       ','',21566,'Reentered','2002-01-20 00:00:00','1992-02-05 00:00:00',102.76,540,1239,62.78,'Intermediate' from dual union all
select 576,'000576','1963-015A','Faith Seven','Mercury SC20',9246,'Landed','1963-05-16 00:00:00','1963-05-15 00:00:00',88.77,163,265,32.54,'Intermediate' from dual union all
select 5760,'005760','1971-015CC','deb Kosmos-397','',24833,'Reentered','1992-02-12 00:00:00','1992-02-06 00:00:00',92.85,366,462,65.64,'Intermediate' from dual union all
select 5761,'005761','1971-015CD','deb Kosmos-397','',24833,'Reentered','1999-11-15 00:00:00','1992-02-06 00:00:00',100.7,429,1155,65.83,'Intermediate' from dual union all
select 5762,'005762','1971-015CE','deb Kosmos-397','',24833,'Reentered','1981-10-05 00:00:00','1981-04-17 00:00:00',101.76,505,1179,65.83,'Intermediate' from dual union all
select 5763,'005763','1971-015CF','Unknown deb                       ','',24833,'In Earth orbit','','1992-02-05 00:00:00',109.92,882,1564,66.97,'Intermediate' from dual union all
select 5764,'005764','1972-001A','Kosmos-471','Zenit-4M',27933,'Landed','1972-01-25 00:00:00','1972-01-14 00:00:00',89.3,175,304,65.05,'Intermediate' from dual union all
select 5765,'005765','1972-001B','11A57I','11A57I',27933,'Reentered','1972-01-20 00:00:00','1972-01-14 00:00:00',89.37,196,291,65,'Intermediate' from dual union all
select 5766,'005766','1967-001M','Unknown deb                       ','[UNID046]',15256,'Reentered','1972-01-20 00:00:00','1972-01-12 00:00:00',102.58,176,1586,26.37,'Intermediate' from dual union all
select 5767,'005767','1972-001C','Sensor cover','Zenit-4M sensor cover',27933,'Reentered','1972-01-15 00:00:00','1972-01-15 00:00:00',89.28,181,297,65.01,'Intermediate' from dual union all
select 5768,'005768','1965-082TS','deb Transtage 4','',11832,'Reentered','1972-03-01 00:00:00','1972-02-13 00:00:00',95.82,519,598,31.37,'Intermediate' from dual union all
select 5769,'005769','1972-002A','OPS 1737','HEXAGON SV-2',27934,'Deorbited','1972-02-29 00:00:00','1972-01-27 00:00:00',89.18,147,321,97.02,'Sun-Synch' from dual union all
select 577,'000577','1963-015B','Atlas D 130D','Atlas D 130D',9246,'Reentered','1963-05-16 00:00:00','1963-05-15 00:00:00',88.34,163,222,32.54,'Intermediate' from dual union all
select 5770,'005770','1972-002B','Titan 23D-2 Stage 2','LR91AJ11 23D-2',27934,'Reentered','1972-01-23 00:00:00','1972-01-22 00:00:00',88.64,148,267,97.01,'Sun-Synch' from dual union all
select 5771,'005771','1972-002C','KH-9 sensor cover','',27934,'Reentered','1972-01-23 00:00:00','1972-01-22 00:00:00',88.02,143,210,97,'Sun-Synch' from dual union all
select 5772,'005772','1972-002D','MABELI','P-11 No. 4424',27934,'Reentered','1979-04-17 00:00:00','1972-02-20 00:00:00',94.78,469,547,96.58,'Sun-Synch' from dual union all
select 5773,'005773','1972-001D','KDU','KDU',27933,'Reentered','1972-01-24 00:00:00','1972-01-22 00:00:00',88.84,154,281,65.02,'Intermediate' from dual union all
select 5774,'005774','1972-001E','Blenda','BZA',27933,'Reentered','1972-01-24 00:00:00','1972-01-22 00:00:00',88.77,172,256,64.98,'Intermediate' from dual union all
select 5775,'005775','1972-003A','INTELSAT IV F4','INTELSAT IV F4',27935,'In Earth orbit','','1977-01-22 00:00:00',1436.16,35671,35905,.31,'Stationary' from dual union all
select 5776,'005776','1971-074K','deb Kosmos-436','Tselina-O part',24892,'Reentered','1972-05-06 00:00:00','1972-01-22 00:00:00',94.19,453,506,74.09,'Intermediate' from dual union all
select 5777,'005777','1971-074L','deb Kosmos-436','Tselina-O part',24892,'Reentered','1972-06-11 00:00:00','1972-01-22 00:00:00',94.6,478,521,74.05,'Intermediate' from dual union all
select 5778,'005778','1971-114C','deb Kosmos-468','Strela-2M debris',24932,'In Earth orbit','','1972-01-20 00:00:00',101.32,807,836,74.04,'Intermediate' from dual union all
select 5779,'005779','1971-074M','deb Kosmos-436','Tselina-O part',24892,'Reentered','1972-04-11 00:00:00','1972-01-22 00:00:00',93.79,452,468,74.06,'Intermediate' from dual union all
select 578,'000578','1963-016A','OPS 0924','LANYARD 2',9247,'Reentered Attached','1963-05-27 00:00:00','1963-05-20 00:00:00',91.01,158,490,74.64,'Intermediate' from dual union all
select 5780,'005780','1971-074N','deb Kosmos-436','Tselina-O part',24892,'Reentered','1972-07-05 00:00:00','1972-01-20 00:00:00',94.81,498,521,74.03,'Intermediate' from dual union all
select 5781,'005781','1971-074P','deb Kosmos-436','Tselina-O part',24892,'Reentered','1972-05-27 00:00:00','1972-01-22 00:00:00',94.37,468,509,74.13,'Intermediate' from dual union all
select 5782,'005782','1971-052E','deb Kosmos-426','DS-U2 part',24870,'Reentered','1973-06-22 00:00:00','1971-08-24 00:00:00',108.39,380,1925,74.05,'Intermediate' from dual union all
select 5783,'005783','1971-074Q','deb Kosmos-436','Tselina-O part',24892,'Reentered','1972-07-12 00:00:00','1972-01-22 00:00:00',94.7,488,521,73.98,'Intermediate' from dual union all
select 5784,'005784','61 Omicron 239','deb r/b Ablestar','',7273,'Reentered','2013-03-27 00:00:00','1992-02-06 00:00:00',100.5,738,827,66.88,'Intermediate' from dual union all
select 5785,'005785','61 Omicron 240','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-06 00:00:00',104.84,870,1104,65.64,'Intermediate' from dual union all
select 5786,'005786','61 Omicron 241','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-01 00:00:00',103.89,839,1046,67.12,'Intermediate' from dual union all
select 5787,'005787','61 Omicron 242','deb r/b Ablestar','',7273,'Reentered','1974-08-28 00:00:00','1972-01-22 00:00:00',100.24,618,921,67.14,'Intermediate' from dual union all
select 5788,'005788','1968-097DA','deb Kosmos-252','',16824,'Reentered','1981-09-05 00:00:00','1981-04-16 00:00:00',101.27,488,1149,62.79,'Intermediate' from dual union all
select 5789,'005789','1968-097DB','deb Kosmos-252 [*]','',16824,'Reentered','1989-09-21 00:00:00','1989-09-21 00:00:00',87.97,166,182,62.53,'Intermediate' from dual union all
select 579,'000579','1963-014B','TRS 5','ERS 5',9245,'Reentered','2011-11-11 00:00:00','1963-06-25 00:00:00',166.38,3604,3680,87.35,'Medium' from dual union all
select 5790,'005790','1968-097DC','deb Kosmos-252','',16824,'Reentered','1981-08-18 00:00:00','1972-01-22 00:00:00',110.53,549,1954,62.25,'Intermediate' from dual union all
select 5791,'005791','1969-082JT','deb Agena D','',18349,'Reentered','1977-01-03 00:00:00','1980-12-23 00:00:00',97.54,592,690,69.76,'Intermediate' from dual union all
select 5792,'005792','1969-082JU','deb Agena D','',18349,'Reentered','1972-04-05 00:00:00','1972-03-27 00:00:00',91.28,327,348,69.25,'Intermediate' from dual union all
select 5793,'005793','1969-082JV','deb Agena D','',18349,'Reentered','1972-04-08 00:00:00','1972-03-16 00:00:00',95.04,488,552,69.68,'Intermediate' from dual union all
select 5794,'005794','1969-082JW','deb Agena D','',18349,'In Earth orbit','','1992-02-06 00:00:00',100.92,767,838,70.02,'Intermediate' from dual union all
select 5795,'005795','1970-089CL','deb Kosmos-374','',21566,'Reentered','2009-01-17 00:00:00','1992-02-06 00:00:00',104.75,492,1474,62.89,'Intermediate' from dual union all
select 5796,'005796','1970-089CM','deb Kosmos-374','',21566,'Reentered','1972-08-06 00:00:00','1972-01-29 00:00:00',100.69,462,1121,62.82,'Intermediate' from dual union all
select 5797,'005797','1970-089CN','deb Kosmos-374 [*]','',21566,'Reentered','1974-05-05 00:00:00','1972-01-21 00:00:00',109.09,510,1860,62.75,'Intermediate' from dual union all
select 5798,'005798','1971-106T','deb Kosmos-462','',24924,'Reentered','1972-03-02 00:00:00','1972-02-25 00:00:00',93.21,201,663,65.59,'Intermediate' from dual union all
select 5799,'005799','1971-106U','deb Kosmos-462','',24924,'Reentered','1972-03-18 00:00:00','1972-03-09 00:00:00',92.25,188,582,65.7,'Intermediate' from dual union all
select 58,'000058','60 Nu 1','Courier 1B','Courier 1B',6327,'In Earth orbit','','1960-11-07 00:00:00',107.03,943,1235,28.3,'Intermediate' from dual union all
select 580,'000580','1963-017A','Kosmos-17','DS-A1 No. 2',9248,'Reentered','1965-06-02 00:00:00','1963-06-24 00:00:00',94.78,265,751,48.97,'Intermediate' from dual union all
select 5800,'005800','1971-106V','deb Kosmos-462','',24924,'Reentered','1972-03-08 00:00:00','1972-01-22 00:00:00',100.26,218,1324,65.74,'Intermediate' from dual union all
select 5801,'005801','1971-106W','deb Kosmos-462','',24924,'Reentered','1973-02-06 00:00:00','1972-01-22 00:00:00',104.52,223,1720,65.76,'Intermediate' from dual union all
select 5802,'005802','1971-106X','deb Kosmos-462','',24924,'Reentered','1972-10-29 00:00:00','1972-01-22 00:00:00',104.16,223,1688,65.73,'Intermediate' from dual union all
select 5803,'005803','1971-106Y','deb Kosmos-462','',24924,'Reentered','1972-02-26 00:00:00','1972-02-25 00:00:00',90.1,183,376,65.7,'Intermediate' from dual union all
select 5804,'005804','1972-004A','Kosmos-472','DS-P1-Yu No. 52',27936,'Reentered','1972-08-18 00:00:00','1972-02-24 00:00:00',101,195,1417,81.99,'Intermediate' from dual union all
select 5805,'005805','1972-004B','S1M','S1M',27936,'Reentered','1972-05-06 00:00:00','1972-02-24 00:00:00',99.91,192,1316,81.99,'Intermediate' from dual union all
select 5806,'005806','1972-001F','PDTU cover','KDU part',27933,'Reentered','1972-01-26 00:00:00','1972-01-25 00:00:00',90.73,169,452,64.99,'Intermediate' from dual union all
select 5807,'005807','59 Alpha 3','Vanguard II strap','',5525,'Reentered','1982-11-22 00:00:00','1977-01-01 00:00:00',127,549,3434,32.89,'Medium' from dual union all
select 5808,'005808','61 Omicron 243','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-06 00:00:00',99.81,666,833,66.62,'Intermediate' from dual union all
select 5809,'005809','61 Omicron 244','deb r/b Ablestar','',7273,'In Earth orbit','','1992-01-28 00:00:00',103.71,907,961,65.11,'Intermediate' from dual union all
select 581,'000581','1963-017B','deb Kosmos-17','',9248,'Reentered','1963-08-26 00:00:00','1963-05-29 00:00:00',94.08,249,699,48.99,'Intermediate' from dual union all
select 5810,'005810','1968-097DD','deb Kosmos-252','',16824,'Reentered','2002-10-05 00:00:00','1992-02-05 00:00:00',103.06,525,1282,62.42,'Intermediate' from dual union all
select 5811,'005811','1965-082TT','deb Transtage 4','',11832,'Reentered','1979-07-18 00:00:00','1979-12-22 00:00:00',98.07,657,675,32.43,'Intermediate' from dual union all
select 5812,'005812','1972-001G','PTDU cover','KDU',27933,'Reentered','1972-01-26 00:00:00','1972-01-26 00:00:00',88.53,172,232,65.07,'Intermediate' from dual union all
select 5813,'005813','1965-020FB','deb Blok Kosmos','',11770,'Reentered','1981-02-13 00:00:00','1972-01-28 00:00:00',105.71,550,1506,56.11,'Intermediate' from dual union all
select 5814,'005814','1972-005A','HEOS 2','HEOS A2',27937,'Deep Space','1972-02-01 00:00:00','1972-03-28 00:00:00',7466.46,994,239334,90,'Deep Space' from dual union all
select 5815,'005815','1972-005B','Delta 87','Delta DSV-3E-4',27937,'Reentered','1978-09-27 00:00:00','1972-03-02 00:00:00',99.07,312,1117,89.79,'Polar' from dual union all
select 5816,'005816','1972-003B','Centaur AC-28','Centaur 20D',27935,'In Earth orbit','','1972-02-23 00:00:00',654.35,560,36618,28.34,'GEO Transfer' from dual union all
select 5817,'005817','1972-005C','FW4D','FW4D',27937,'Deep Space','1972-02-01 00:00:00','1972-02-01 00:00:00',7061.71,652,230445,90.2,'Deep Space' from dual union all
select 5818,'005818','1972-005D','Delta 87 clamp band half','',27937,'Reentered','1975-06-08 00:00:00','1972-03-02 00:00:00',99.1,315,1117,89.78,'Polar' from dual union all
select 5819,'005819','1972-005E','Delta 87 clamp band half','',27937,'Reentered','1975-09-02 00:00:00','1972-03-01 00:00:00',99.09,319,1112,89.79,'Polar' from dual union all
select 582,'000582','1963-017C','deb Kosmos-17','DS-A1 high debris',9248,'Reentered','1965-05-12 00:00:00','1963-06-29 00:00:00',96.23,351,805,49.22,'Intermediate' from dual union all
select 5820,'005820','1964-086D','Altair','(duplicate)',10372,'Spurious','1972-02-01 00:00:00','1972-02-01 00:00:00',396.05,250,22723,19.99,'Highly Ellip' from dual union all
select 5821,'005821','1972-006A','Kosmos-473','Zenit-2M',27938,'Landed','1972-02-15 00:00:00','1972-02-05 00:00:00',89.64,205,309,65.01,'Intermediate' from dual union all
select 5822,'005822','1972-006B','11A57I','11A57I',27938,'Reentered','1972-02-11 00:00:00','1972-02-05 00:00:00',89.42,201,291,65.05,'Intermediate' from dual union all
select 5823,'005823','1969-082JX','deb Agena D','',18349,'Reentered','1972-04-09 00:00:00','1972-03-31 00:00:00',91.11,310,348,69.29,'Intermediate' from dual union all
select 5824,'005824','1970-025MJ','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',109.56,704,1708,97.88,'Sun-Synch' from dual union all
select 5825,'005825','1970-025MK','deb Agena D','',21502,'Reentered','1972-12-03 00:00:00','1972-10-05 00:00:00',96.24,510,647,99.62,'Sun-Synch' from dual union all
select 5826,'005826','61 Omicron 245','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-07 00:00:00',104.14,899,1009,66.69,'Intermediate' from dual union all
select 5827,'005827','1968-097DE','deb Kosmos-252','',16824,'Reentered','1982-11-19 00:00:00','1970-07-31 00:00:00',110.24,551,1925,62.31,'Intermediate' from dual union all
select 5828,'005828','1968-097DF','deb Kosmos-252','',16824,'Reentered','1982-11-19 00:00:00','1972-02-06 00:00:00',111.58,621,1978,62.24,'Intermediate' from dual union all
select 5829,'005829','1968-097DG','deb Kosmos-252','',16824,'Reentered','1978-03-31 00:00:00','1978-03-26 00:00:00',90.62,267,343,62.8,'Intermediate' from dual union all
select 583,'000583','1963-017D','deb Kosmos-17','',9248,'Reentered','1963-08-30 00:00:00','1963-05-28 00:00:00',94.31,257,713,49.07,'Intermediate' from dual union all
select 5830,'005830','1968-097DH','deb Kosmos-252','',16824,'Reentered','1979-02-21 00:00:00','1972-02-06 00:00:00',108.02,519,1752,62.36,'Intermediate' from dual union all
select 5831,'005831','1971-106Z','deb Kosmos-462','',24924,'Reentered','1972-03-16 00:00:00','1972-03-12 00:00:00',90.72,181,439,65.65,'Intermediate' from dual union all
select 5832,'005832','1971-106AA','deb Kosmos-462','',24924,'Reentered','1972-03-15 00:00:00','1972-03-11 00:00:00',90.42,181,410,65.42,'Intermediate' from dual union all
select 5833,'005833','1971-106AB','deb Kosmos-462','',24924,'Reentered','1972-02-25 00:00:00','1972-02-15 00:00:00',94.2,204,756,65.71,'Intermediate' from dual union all
select 5834,'005834','1971-106AC','deb Kosmos-462','',24924,'Reentered','1972-03-19 00:00:00','1972-02-06 00:00:00',99.73,218,1274,65.74,'Intermediate' from dual union all
select 5835,'005835','1972-007A','Luna-20 VA','VA',27939,'Deep Space Attached','1972-02-15 00:00:00','1972-02-25 00:00:00',15550.88,0,400000,51.5,'Cislunar' from dual union all
select 5836,'005836','1972-007B','Blok D No. 0801L','11S824',27939,'Deep Space','1972-02-15 00:00:00','1972-02-14 00:00:00',15562.18,200,400000,51.5,'Cislunar' from dual union all
select 5837,'005837','1972-007C','SOZ','SOZ',27939,'Reentered','1972-02-17 00:00:00','1972-02-16 00:00:00',88.26,167,210,51.52,'Intermediate' from dual union all
select 5838,'005838','1972-007D','SOZ','SOZ',27939,'Reentered','1972-02-17 00:00:00','1972-02-16 00:00:00',88.25,48,328,51.13,'ERROR - Unexpected value "TA"' from dual union all
select 5839,'005839','1972-008A','Kosmos-474','Zenit-4M',27940,'Landed','1972-02-29 00:00:00','1972-02-18 00:00:00',89.5,176,324,64.97,'Intermediate' from dual union all
select 584,'000584','1963-017E','deb Kosmos-17','',9248,'Reentered','1963-07-25 00:00:00','1963-06-03 00:00:00',94.37,251,726,48.98,'Intermediate' from dual union all
select 5840,'005840','1972-008B','11A57I','11A57I',27940,'Reentered','1972-02-23 00:00:00','1972-02-19 00:00:00',89.32,193,289,64.99,'Intermediate' from dual union all
select 5841,'005841','1972-007E','Luna-20 KT','E-8-5 No. 408',27939,'Deep Space','1972-02-15 00:00:00','1972-02-14 00:00:00',15562.18,200,400000,51.5,'Cislunar' from dual union all
select 5842,'005842','1970-025ML','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',102.62,820,946,100.06,'Sun-Synch' from dual union all
select 5843,'005843','1970-025MM','deb Agena D','',21502,'Reentered','1989-02-23 00:00:00','1989-02-22 00:00:00',93.07,407,443,100.8,'Retrograde' from dual union all
select 5844,'005844','1965-082TU','deb Transtage 4','',11832,'Reentered','1974-05-13 00:00:00','1974-04-08 00:00:00',99.81,668,831,31.38,'Intermediate' from dual union all
select 5845,'005845','1965-082TV','deb Transtage 4','',11832,'Reentered','1982-09-17 00:00:00','1972-02-28 00:00:00',99.76,663,832,32,'Intermediate' from dual union all
select 5846,'005846','1972-009A','Kosmos-475','Tsiklon',27941,'In Earth orbit','','1972-03-26 00:00:00',104.75,965,1001,74.05,'Intermediate' from dual;
insert into SATELLITE
select 5847,'005847','1972-009B','S3M','S3M',27941,'In Earth orbit','','1972-03-28 00:00:00',104.65,967,990,74.05,'Intermediate' from dual union all
select 5848,'005848','1972-008C','KDU','KDU',27940,'Reentered','1972-02-29 00:00:00','1972-02-27 00:00:00',88.8,170,261,64.93,'Intermediate' from dual union all
select 5849,'005849','1972-008D','Blenda','BZA',27940,'Reentered','1972-02-28 00:00:00','1972-02-27 00:00:00',88.07,154,204,64.98,'Intermediate' from dual union all
select 585,'000585','1963-017F','deb Kosmos-17','',9248,'Reentered','1963-09-30 00:00:00','1963-09-20 00:00:00',95.6,322,774,49.32,'Intermediate' from dual union all
select 5850,'005850','1972-008E','PTDU cover','',27940,'Reentered','1972-03-01 00:00:00','1972-02-29 00:00:00',90.44,194,399,64.98,'Intermediate' from dual union all
select 5851,'005851','1972-010A','DSP F3','DSP 4',27942,'In Earth orbit','','1979-10-29 00:00:00',1436.09,35692,35880,5.72,'Inclined GEO' from dual union all
select 5852,'005852','1972-011A','Kosmos-476','Tselina-D',27943,'Reentered','1991-10-25 00:00:00','1972-04-03 00:00:00',97.16,613,633,81.21,'Intermediate' from dual union all
select 5853,'005853','1972-011B','8A92ME','8A92ME',27943,'Reentered','1996-01-18 00:00:00','1972-03-31 00:00:00',97.28,564,693,81.23,'Intermediate' from dual union all
select 5854,'005854','1972-010B','Transtage 22','Transtage 22',27942,'In Earth orbit','','1972-03-30 00:00:00',1431.67,35600,35800,2,'Drift GEO' from dual union all
select 5855,'005855','1972-010C','Titan 23C-4 Stage 2','LR91AJ11 23C-4',27942,'Reentered','1972-03-03 00:00:00','1972-03-02 00:00:00',89.03,152,302,28.63,'Intermediate' from dual union all
select 5856,'005856','1965-082TW','deb Transtage 4','',11832,'Reentered','1972-10-23 00:00:00','1972-08-04 00:00:00',94.22,470,492,31.46,'Intermediate' from dual union all
select 5857,'005857','1965-082TX','deb Transtage 4','',11832,'Reentered','1979-08-03 00:00:00','1979-05-03 00:00:00',97.35,598,666,32.32,'Intermediate' from dual union all
select 5858,'005858','1971-114D','deb Kosmos-407','Strela-2M debris',24932,'In Earth orbit','','1972-03-05 00:00:00',101.25,807,828,74.04,'Intermediate' from dual union all
select 5859,'005859','1969-082JY','deb Agena D','',18349,'Reentered','1982-05-10 00:00:00','1972-03-01 00:00:00',103.02,792,1011,70.07,'Intermediate' from dual union all
select 586,'000586','1963-018A','Kosmos-18','Zenit-2 No. 11',9249,'Landed','1963-06-02 00:00:00','1963-05-30 00:00:00',89.31,203,278,64.91,'Intermediate' from dual union all
select 5860,'005860','1972-012A','Pioneer 10','Pioneer F',27944,'Deep Space','1972-03-03 00:00:00','1972-03-03 00:00:00',53.75,186,22379,31.48,'Earth Escape' from dual union all
select 5861,'005861','1972-012B','Star 37E S/N 40016','Star 37E S/N 40016',27944,'Deep Space','1972-03-03 00:00:00','1972-03-03 00:00:00',53.75,186,22379,31.48,'Earth Escape' from dual union all
select 5862,'005862','1972-013A','Kosmos-477','Zenit-2M',27945,'Landed','1972-03-16 00:00:00','1972-03-06 00:00:00',89.51,201,300,72.86,'Intermediate' from dual union all
select 5863,'005863','1972-013B','11A57I','11A57I',27945,'Reentered','1972-03-10 00:00:00','1972-03-06 00:00:00',89.22,196,276,72.85,'Intermediate' from dual union all
select 5864,'005864','1972-013C','Sensor cover','',27945,'Reentered','1972-03-05 00:00:00','1972-03-04 00:00:00',88.83,193,241,72.82,'Intermediate' from dual union all
select 5865,'005865','1972-013D','Sensor cover','',27945,'Reentered','1972-03-05 00:00:00','1972-03-04 00:00:00',88.92,179,264,72.78,'Intermediate' from dual union all
select 5866,'005866','1971-106AD','deb Kosmos-462','',24924,'Reentered','1972-07-11 00:00:00','1972-07-05 00:00:00',90.49,173,425,65.78,'Intermediate' from dual union all
select 5867,'005867','1971-106AE','deb Kosmos-462','',24924,'Reentered','1972-03-22 00:00:00','1972-03-21 00:00:00',89.67,178,338,65.64,'Intermediate' from dual union all
select 5868,'005868','1969-082JZ','deb Agena D','',18349,'Reentered','1979-10-23 00:00:00','1972-03-08 00:00:00',100.34,658,892,69.76,'Intermediate' from dual union all
select 5869,'005869','1969-082KA','deb Agena D','',18349,'Reentered','1982-02-22 00:00:00','1972-03-08 00:00:00',102.45,824,925,70.15,'Intermediate' from dual union all
select 587,'000587','1963-018B','8A92E','8A92E',9249,'Reentered','1963-06-08 00:00:00','1963-05-31 00:00:00',89.12,199,263,64.95,'Intermediate' from dual union all
select 5870,'005870','1968-097DJ','deb Kosmos-252 [*]','',16824,'Reentered','1981-06-20 00:00:00','1969-11-27 00:00:00',115.72,763,2214,62.3,'Intermediate' from dual union all
select 5871,'005871','1968-097DK','deb Kosmos-252 [*]','',16824,'In Earth orbit','','1992-02-04 00:00:00',110.98,561,1983,62.81,'Intermediate' from dual union all
select 5872,'005872','1968-097DL','deb Kosmos-252 [*]','',16824,'Reentered','1990-02-16 00:00:00','1972-03-08 00:00:00',112.4,622,2052,62.34,'Intermediate' from dual union all
select 5873,'005873','1968-097DM','deb Kosmos-252','',16824,'Reentered','1979-06-16 00:00:00','1979-03-14 00:00:00',95.71,428,678,62.26,'Intermediate' from dual union all
select 5874,'005874','1968-097DN','deb Kosmos-252','',16824,'Reentered','1979-10-25 00:00:00','1979-10-15 00:00:00',104.27,497,1424,62.29,'Intermediate' from dual union all
select 5875,'005875','1968-097DP','deb Kosmos-252','',16824,'Reentered','1981-05-09 00:00:00','1981-04-10 00:00:00',96.56,441,747,62.22,'Intermediate' from dual union all
select 5876,'005876','61 Omicron 246','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-05 00:00:00',115.85,908,2080,65.68,'Intermediate' from dual union all
select 5877,'005877','61 Omicron 247','deb r/b Ablestar','',7273,'Reentered','1979-05-01 00:00:00','1972-03-08 00:00:00',101.03,673,941,66.67,'Intermediate' from dual union all
select 5878,'005878','1972-012C','Centaur AC-27','Centaur 24D',27944,'Reentered','1972-08-18 00:00:00','1972-03-03 00:00:00',1537.48,152,75345,31.48,'Very High' from dual union all
select 5879,'005879','1972-014A','TD-1A','TD-1A',27946,'Reentered','1980-01-09 00:00:00','1972-04-11 00:00:00',95.32,525,544,97.56,'Sun-Synch' from dual union all
select 588,'000588','1963-017G','S1','S1',9248,'Reentered','1964-04-02 00:00:00','1963-06-22 00:00:00',94.7,269,740,48.98,'Intermediate' from dual union all
select 5880,'005880','1972-014B','Delta 88','Delta DSV-3E-4',27946,'Reentered','1980-10-09 00:00:00','1972-04-11 00:00:00',95.25,524,538,97.55,'Sun-Synch' from dual union all
select 5881,'005881','1972-014C','deb TD-1A','',27946,'Reentered','1972-05-22 00:00:00','1972-04-12 00:00:00',94.68,496,511,97.54,'Sun-Synch' from dual union all
select 5882,'005882','1972-013E','Kapsula Kosmosa-477','Nauka 14KS No. 1',27945,'Reentered','1972-03-23 00:00:00','1972-03-14 00:00:00',89.38,197,290,72.82,'Intermediate' from dual union all
select 5883,'005883','1972-013F','Nauka part','',27945,'Reentered','1972-03-17 00:00:00','1972-03-14 00:00:00',89.12,195,267,72.89,'Intermediate' from dual union all
select 5884,'005884','1972-014D','TD-1A optics cover','',27946,'Reentered','1975-11-15 00:00:00','1972-04-13 00:00:00',95.3,523,543,97.56,'Sun-Synch' from dual union all
select 5885,'005885','1972-015A','Kosmos-478','Zenit-4M',27947,'Landed','1972-03-28 00:00:00','1972-03-17 00:00:00',89.06,175,280,65.39,'Intermediate' from dual union all
select 5886,'005886','1972-015B','11A57I','11A57I',27947,'Reentered','1972-03-21 00:00:00','1972-03-17 00:00:00',89.26,201,274,65.4,'Intermediate' from dual union all
select 5887,'005887','1972-015C','Sensor cover','Zenit-4M sensor cover',27947,'Reentered','1972-03-17 00:00:00','1972-03-15 00:00:00',89.46,203,293,65.39,'Intermediate' from dual union all
select 5888,'005888','1972-016A','OPS 1678','GAMBIT-3 Mission 4334',27948,'Reentered Attached','1972-04-11 00:00:00','1972-03-24 00:00:00',89.77,135,392,110.97,'Retrograde' from dual union all
select 5889,'005889','1968-097DQ','deb Kosmos-252','',16824,'Reentered','1982-11-19 00:00:00','1972-03-21 00:00:00',108.9,525,1827,62.36,'Intermediate' from dual union all
select 589,'000589','1963-014D','DASH 1','DASH 1',9245,'In Earth orbit','','1963-11-30 00:00:00',166.4,3611,3676,87.36,'Medium' from dual union all
select 5890,'005890','1968-097DR','deb Kosmos-252  [*]','',16824,'Reentered','1977-03-01 00:00:00','1972-03-21 00:00:00',115.87,807,2183,62.28,'Intermediate' from dual union all
select 5891,'005891','1968-097DS','deb Kosmos-252  [*]','',16824,'Reentered','1977-01-31 00:00:00','1979-09-05 00:00:00',115.1,768,2152,62.24,'Intermediate' from dual union all
select 5892,'005892','1964-038E','deb Elektron-3','',10324,'Reentered','1974-02-20 00:00:00','1972-03-21 00:00:00',155.75,393,6029,60.91,'Medium' from dual union all
select 5893,'005893','1964-038F','deb Elektron-3','',10324,'Reentered','1974-11-20 00:00:00','1972-02-22 00:00:00',160.31,409,6386,60.87,'Medium' from dual union all
select 5894,'005894','1972-017A','Kosmos-479','Tselina-OM',27949,'Reentered','1980-04-13 00:00:00','1972-04-21 00:00:00',95.13,512,538,74.05,'Intermediate' from dual union all
select 5895,'005895','1972-017B','S3M','S3M',27949,'Reentered','1980-05-12 00:00:00','1972-04-24 00:00:00',95.02,496,543,74.09,'Intermediate' from dual union all
select 5896,'005896','1970-089CP','deb Kosmos-374','',21566,'Reentered','1973-04-02 00:00:00','1972-03-20 00:00:00',109.97,514,1937,62.58,'Intermediate' from dual union all
select 5897,'005897','1970-089CQ','deb Kosmos-374 [*]','',21566,'Reentered','1981-07-22 00:00:00','1972-04-01 00:00:00',109.74,520,1910,62,'Intermediate' from dual union all
select 5898,'005898','1970-089CR','deb Kosmos-374','',21566,'Reentered','1981-11-05 00:00:00','1981-04-12 00:00:00',105.7,520,1535,62.21,'Intermediate' from dual union all
select 5899,'005899','1969-082KB','deb Agena D','',18349,'Reentered','1984-12-02 00:00:00','1984-06-25 00:00:00',102.08,478,1237,69.71,'Intermediate' from dual union all
select 59,'000059','60 Nu 2','Ablestar 005','AJ10-104',6327,'In Earth orbit','','1960-11-22 00:00:00',106.55,950,1184,28.3,'Intermediate' from dual union all
select 590,'000590','1963-019A','OPS 0954','CORONA M-21',9250,'Reentered Attached','1963-07-12 00:00:00','1963-06-21 00:00:00',90.37,181,405,81.81,'Intermediate' from dual union all
select 5900,'005900','1969-082KC','deb Agena D','',18349,'Reentered','1973-03-19 00:00:00','1972-03-21 00:00:00',101.51,257,1404,69.88,'Intermediate' from dual union all
select 5901,'005901','1972-017C','deb Kosmos-479','Tselina-O part',27949,'Reentered','1972-11-26 00:00:00','1972-04-23 00:00:00',94.7,488,521,74.13,'Intermediate' from dual union all
select 5902,'005902','1972-017D','deb Kosmos-479','Tselina-O part',27949,'Reentered','1972-11-01 00:00:00','1972-04-22 00:00:00',94.61,476,524,73.98,'Intermediate' from dual union all
select 5903,'005903','1972-018A','DMSP 5528','DMSP Block 5B F-2 (SV-1)',27950,'In Earth orbit','','1972-04-23 00:00:00',101.76,800,884,98.79,'Sun-Synch' from dual union all
select 5904,'005904','1972-018B','Burner IIA No. 15 Stage 2','Star 26B S/N 005',27950,'In Earth orbit','','1972-04-28 00:00:00',101.64,798,875,98.81,'Sun-Synch' from dual union all
select 5905,'005905','1972-019A','Kosmos-480','Sfera',27951,'In Earth orbit','','1972-04-29 00:00:00',109.14,1173,1201,82.98,'Intermediate' from dual union all
select 5906,'005906','1972-020A','Kosmos-481','DS-P1-Yu No. 46',27952,'Reentered','1972-09-02 00:00:00','1972-04-24 00:00:00',91.95,262,479,71.02,'Intermediate' from dual union all
select 5907,'005907','1972-019B','S3M','S3M',27951,'In Earth orbit','','1972-04-24 00:00:00',109,1164,1197,82.98,'Intermediate' from dual union all
select 5908,'005908','1972-020B','S1M','S1M',27952,'Reentered','1972-06-11 00:00:00','1972-04-24 00:00:00',91.33,254,426,71.04,'Intermediate' from dual union all
select 5909,'005909','1972-015D','Blenda','BZA',27947,'Reentered','1972-03-28 00:00:00','1972-03-25 00:00:00',89.65,180,335,65.4,'Intermediate' from dual union all
select 591,'000591','1963-020A','Vostok-5','Vostok-3A No. 7',9251,'Landed','1963-06-19 00:00:00','1963-06-18 00:00:00',87.7,158,163,64.92,'Intermediate' from dual union all
select 5910,'005910','1972-015E','KDU','KDU',27947,'Reentered','1972-03-30 00:00:00','1972-03-25 00:00:00',89.55,176,329,65.46,'Intermediate' from dual union all
select 5911,'005911','1972-017E','deb Kosmos-479','Tselina-O part',27949,'Reentered','1972-11-05 00:00:00','1972-04-21 00:00:00',94.61,475,525,74.07,'Intermediate' from dual union all
select 5912,'005912','1972-021A','Venera-8','4V-1 No. 670',27953,'Deep Space','1972-03-28 00:00:00','1972-03-27 00:00:00',88.89,194,246,51.79,'Intermediate' from dual union all
select 5913,'005913','1972-021B','11S510','11S510',27953,'Reentered','1972-03-29 00:00:00','1972-03-28 00:00:00',88.42,180,213,51.78,'Intermediate' from dual union all
select 5914,'005914','1972-021C','BOZ','BOZ',27953,'Reentered','1972-03-29 00:00:00','1972-03-28 00:00:00',88.56,180,227,51.69,'Intermediate' from dual union all
select 5915,'005915','1972-015F','PDTU cover','',27947,'Reentered','1972-03-30 00:00:00','1972-03-29 00:00:00',89.82,188,344,65.44,'Intermediate' from dual union all
select 5916,'005916','1963-014CV','deb West Ford','',9245,'Reentered','1972-04-26 00:00:00','1972-04-14 00:00:00',103.27,274,1552,83.89,'Intermediate' from dual union all
select 5917,'005917','1972-022A','Meteor','Meteor-MV No. 23',27954,'In Earth orbit','','1972-05-06 00:00:00',102.51,864,890,81.23,'Intermediate' from dual union all
select 5918,'005918','1972-022B','8A92ME','8A92ME',27954,'In Earth orbit','','1972-04-29 00:00:00',102.64,838,929,81.25,'Intermediate' from dual union all
select 5919,'005919','1972-023A','Kosmos-482','4V-1 No. 671',27955,'Reentered','1981-05-05 00:00:00','1972-05-03 00:00:00',200.47,205,9732,52.13,'Medium' from dual union all
select 592,'000592','1963-020B','8K72KE','8K72KE',9251,'Reentered','1963-06-16 00:00:00','1963-06-16 00:00:00',87.42,117,177,64.85,'Intermediate' from dual union all
select 5920,'005920','1972-023B','11S510','11S510',27955,'Reentered','1972-04-01 00:00:00','1972-03-31 00:00:00',88.46,190,207,51.79,'Intermediate' from dual union all
select 5921,'005921','1972-023C','BOZ','BOZ',27955,'Reentered','1972-04-02 00:00:00','1972-04-01 00:00:00',88.21,139,233,51.72,'Intermediate' from dual union all
select 5922,'005922','60 Pi 5','deb Echo 1                       ','[UNID047] RCS 0.0',6329,'In Earth orbit','','1972-02-21 00:00:00',97.31,572,688,48.52,'Intermediate' from dual union all
select 5923,'005923','1972-023D','Blok-NVL No. 64','Blok-L',27955,'Reentered','1983-02-20 00:00:00','1972-04-30 00:00:00',200.17,207,9706,52.15,'Medium' from dual union all
select 5924,'005924','1972-024A','Kosmos-483','Zenit-4M',27956,'Landed','1972-04-15 00:00:00','1972-04-05 00:00:00',89.32,175,307,72.85,'Intermediate' from dual union all
select 5925,'005925','1972-024B','11A57I','11A57I',27956,'Reentered','1972-04-11 00:00:00','1972-04-05 00:00:00',89.44,198,296,72.83,'Intermediate' from dual union all
select 5926,'005926','1972-024C','Sensor cover','Zenit-4M sensor cover',27956,'Reentered','1972-04-12 00:00:00','1972-04-03 00:00:00',89.23,198,275,72.84,'Intermediate' from dual union all
select 5927,'005927','1972-025A','Molniya-1','Molniya-1 11F67 No. 27',27957,'Reentered','1974-01-30 00:00:00','1972-05-10 00:00:00',717.7,439,39912,65.53,'GEO Transfer' from dual union all
select 5928,'005928','1972-025B','SRET 1','MAS/SRET',27957,'Reentered','1974-02-26 00:00:00','1972-04-05 00:00:00',91.59,231,475,65.53,'Intermediate' from dual union all
select 5929,'005929','1972-025C','11S510','11S510',27957,'Reentered','1972-04-28 00:00:00','1972-04-11 00:00:00',90.95,217,426,65.55,'Intermediate' from dual union all
select 593,'000593','1963-021A','WILD BILL 4','AFTRACK 33/Mission 7216',9252,'Reentered Attached','1963-08-07 00:00:00','1963-06-29 00:00:00',94.48,174,813,69.87,'Intermediate' from dual union all
select 5930,'005930','1972-025D','BOZ','BOZ',27957,'Reentered','1972-05-08 00:00:00','1972-04-11 00:00:00',91.33,227,453,65.52,'Intermediate' from dual union all
select 5931,'005931','1972-025E','deb Blok-I','',27957,'Reentered','1972-04-10 00:00:00','1972-04-06 00:00:00',90.71,206,413,65.54,'Intermediate' from dual union all
select 5932,'005932','1972-014E','TD-1A optics cover part','',27946,'Reentered','1975-10-27 00:00:00','1972-04-11 00:00:00',95.24,516,545,97.53,'Sun-Synch' from dual union all
select 5933,'005933','1972-026A','Kosmos-484','Zenit-2M',27958,'Landed','1972-04-18 00:00:00','1972-04-08 00:00:00',88.61,193,218,81.31,'Intermediate' from dual union all
select 5934,'005934','1972-026B','11A57I','11A57I',27958,'Reentered','1972-04-08 00:00:00','1972-04-07 00:00:00',88.32,167,216,81.29,'Intermediate' from dual union all
select 5935,'005935','1972-025F','deb Blok-I','',27957,'Reentered','1972-04-10 00:00:00','1972-04-06 00:00:00',91.39,204,481,65.54,'Intermediate' from dual union all
select 5936,'005936','1972-027A','Interkosmos-6','Energiya No. 1',27959,'Landed','1972-04-11 00:00:00','1972-04-09 00:00:00',88.95,202,243,51.76,'Intermediate' from dual;
insert into SATELLITE
select 5937,'005937','1972-027B','11A57I','11A57I',27959,'Reentered','1972-04-10 00:00:00','1972-04-09 00:00:00',88.4,166,224,51.79,'Intermediate' from dual union all
select 5938,'005938','1972-028A','Kosmos-485','DS-P1-Yu No. 58',27960,'Reentered','1972-08-30 00:00:00','1972-05-11 00:00:00',91.59,262,444,70.99,'Intermediate' from dual union all
select 5939,'005939','1972-028B','S1M','S1M',27960,'Reentered','1972-06-16 00:00:00','1972-05-11 00:00:00',90.96,261,383,70.98,'Intermediate' from dual union all
select 594,'000594','1963-022A','Transit VA-3','Transit VA-3',9253,'Reentered','1990-08-03 00:00:00','1963-07-21 00:00:00',99.63,728,753,90.02,'Polar' from dual union all
select 5940,'005940','1972-025G','Blok-L','Blok-L',27957,'Reentered','1974-03-08 00:00:00','1972-05-04 00:00:00',700.08,450,39028,65.56,'GEO Transfer' from dual union all
select 5941,'005941','1972-029A','Prognoz','SO-M No. 501',27961,'Deep Space','1972-04-14 00:00:00','1975-08-24 00:00:00',5773.98,467,200001,64.93,'Deep Space' from dual union all
select 5942,'005942','1972-029B','11S510','11S510',27961,'Reentered','1972-05-11 00:00:00','1972-04-21 00:00:00',90.7,219,399,64.97,'Intermediate' from dual union all
select 5943,'005943','1972-024D','KDU','KDU',27956,'Reentered','1972-04-19 00:00:00','1972-04-13 00:00:00',89.42,179,313,72.8,'Intermediate' from dual union all
select 5944,'005944','1972-024E','Blenda','BZA',27956,'Reentered','1972-04-18 00:00:00','1972-04-13 00:00:00',89.4,175,315,72.83,'Intermediate' from dual union all
select 5945,'005945','1972-030A','Kosmos-486','Zenit-4M',27962,'Landed','1972-04-27 00:00:00','1972-04-16 00:00:00',88.53,176,227,81.33,'Intermediate' from dual union all
select 5946,'005946','1972-029C','BOZ','BOZ',27961,'Reentered','1972-05-17 00:00:00','1972-04-21 00:00:00',91.06,225,429,64.97,'Intermediate' from dual union all
select 5947,'005947','1972-030B','11A57I','11A57I',27962,'Reentered','1972-04-17 00:00:00','1972-04-16 00:00:00',88.58,193,216,81.32,'Intermediate' from dual union all
select 5948,'005948','1972-030C','Sensor cover','Zenit-4M sensor cover',27962,'Reentered','1972-04-22 00:00:00','1972-04-18 00:00:00',87.72,138,186,81.33,'Intermediate' from dual union all
select 5949,'005949','1972-029D','deb Blok-I','',27961,'Reentered','1972-04-19 00:00:00','1972-04-17 00:00:00',89.09,213,246,64.96,'Intermediate' from dual union all
select 595,'000595','1963-023A','Vostok-6','Vostok-3A No. 8',9254,'Landed','1963-06-19 00:00:00','1963-06-18 00:00:00',88.03,163,191,64.92,'Intermediate' from dual union all
select 5950,'005950','1970-077E','deb Blok-L','',21554,'Reentered','1972-04-30 00:00:00','1972-04-14 00:00:00',161.37,131,6750,64.61,'Medium' from dual union all
select 5951,'005951','1967-001N','deb unknown                       ','[UNID048]',15256,'Reentered','1973-06-26 00:00:00','1972-03-17 00:00:00',431.25,191,24869,26.74,'Highly Ellip' from dual union all
select 5952,'005952','1970-109F','deb PEOLE','',21586,'Reentered','1976-10-30 00:00:00','1972-04-15 00:00:00',96.07,442,698,15.41,'Equatorial' from dual union all
select 5953,'005953','1963-014CW','deb West Ford','',9245,'Reentered','1985-09-21 00:00:00','1972-04-12 00:00:00',164.66,1653,5494,85.88,'Medium' from dual union all
select 5954,'005954','1963-014CX','deb West Ford','',9245,'Reentered','1982-11-22 00:00:00','1972-04-09 00:00:00',164.81,1715,5443,85.96,'Medium' from dual union all
select 5955,'005955','1965-082TY','deb Transtage 4','',11832,'Reentered','1978-02-22 00:00:00','1978-12-30 00:00:00',101.99,688,1018,31.21,'Intermediate' from dual union all
select 5956,'005956','1971-052F','deb Kosmos-426','',24870,'Reentered','1977-12-08 00:00:00','1972-04-13 00:00:00',106.79,378,1777,74.03,'Intermediate' from dual union all
select 5957,'005957','1970-025MN','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',104.64,932,1024,99.65,'Sun-Synch' from dual union all
select 5958,'005958','1970-025MP','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.51,1020,1110,99.43,'Sun-Synch' from dual union all
select 5959,'005959','1966-013L','deb Rubis','',13420,'Reentered','1978-11-21 00:00:00','1972-03-30 00:00:00',114.63,494,2384,34.08,'Intermediate' from dual union all
select 596,'000596','1963-023B','8K72KE','8K72KE',9254,'Reentered','1963-06-18 00:00:00','1963-06-16 00:00:00',88.35,163,223,65.08,'Intermediate' from dual union all
select 5960,'005960','1967-001P','deb unknown                       ','[UNID049]',15256,'Reentered','1973-06-26 00:00:00','1972-04-11 00:00:00',471.34,387,26982,26.51,'Highly Ellip' from dual union all
select 5961,'005961','1967-001Q','deb unknown                       ','[UNID050]',15256,'Reentered','1973-06-26 00:00:00','1972-04-13 00:00:00',436.96,396,24997,26.37,'Highly Ellip' from dual union all
select 5962,'005962','1971-119C','deb Aureole 1','',24937,'Reentered','1978-02-19 00:00:00','1972-04-15 00:00:00',113.47,379,2393,74.01,'Intermediate' from dual union all
select 5963,'005963','1970-089CS','deb Kosmos-374','',21566,'Reentered','1979-07-23 00:00:00','1972-04-13 00:00:00',107.71,539,1703,62.31,'Intermediate' from dual union all
select 5964,'005964','1969-082KD','deb Agena D','',18349,'Reentered','1981-08-13 00:00:00','1972-04-14 00:00:00',99.58,701,776,70.03,'Intermediate' from dual union all
select 5965,'005965','1969-082KE','deb Agena D','',18349,'Reentered','1989-07-12 00:00:00','1989-07-10 00:00:00',91.24,324,347,69.95,'Intermediate' from dual union all
select 5966,'005966','1968-097DT','deb Kosmos-252','',16824,'Reentered','1977-01-03 00:00:00','1971-11-30 00:00:00',120.83,537,2899,62.25,'Medium' from dual union all
select 5967,'005967','1970-025MQ','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',112.5,1240,1443,101.63,'Sun-Synch' from dual union all
select 5968,'005968','1965-082TZ','deb Transtage 4','',11832,'Reentered','1977-05-11 00:00:00','1977-04-09 00:00:00',94.96,228,806,31.35,'Intermediate' from dual union all
select 5969,'005969','1967-001R','deb unknown                       ','[UNID051]',15256,'Reentered','1975-09-14 00:00:00','1972-04-13 00:00:00',444.18,310,25502,26.77,'Highly Ellip' from dual union all
select 597,'000597','1963-021F','SURCAL','SURCAL',9252,'Reentered','1963-07-05 00:00:00','1963-06-22 00:00:00',93.65,139,768,69.69,'Intermediate' from dual union all
select 5970,'005970','1965-082UA','deb Transtage 4','',11832,'Reentered','1973-07-26 00:00:00','1972-04-14 00:00:00',104.86,604,1371,31.71,'Intermediate' from dual union all
select 5971,'005971','1967-006E','Altair yo weight','',15261,'In Earth orbit','','1972-03-22 00:00:00',113.13,1283,1458,102.04,'Sun-Synch' from dual union all
select 5972,'005972','61 Omicron 248','deb r/b Ablestar','',7273,'Reentered','1976-10-30 00:00:00','1972-04-01 00:00:00',113.93,916,1898,68.3,'Intermediate' from dual union all
select 5973,'005973','1971-096B','FW4S S/N 2223-16','FW4S',24914,'Reentered','1979-06-01 00:00:00','1972-03-20 00:00:00',458.75,270,26381,3.32,'Highly Ellip' from dual union all
select 5974,'005974','1971-009C','Star 37D yo weight','',24827,'Reentered','1975-08-10 00:00:00','1972-04-14 00:00:00',658.09,243,37125,24.8,'GEO Transfer' from dual union all
select 5975,'005975','1970-021C','Star 37D yo weight','',21498,'In Earth orbit','','1971-10-27 00:00:00',661.12,255,37267,25.05,'GEO Transfer' from dual union all
select 5976,'005976','1969-045B','Star 37D S/N 013','Star 37D S/N 00013',18312,'Reentered','1977-03-07 00:00:00','1969-05-22 00:00:00',654.15,280,36888,28.6,'GEO Transfer' from dual union all
select 5977,'005977','1969-011B','Star 37D S/N 012','Star 37D S/N 012',18278,'In Earth orbit','','1969-03-30 00:00:00',637.81,303,36030,29.66,'GEO Transfer' from dual union all
select 5978,'005978','1968-116B','Star 37D S/N 011','Star 37D S/N 00011',16843,'Reentered','2005-01-26 00:00:00','1999-08-05 00:00:00',311.06,244,17414,29.68,'Highly Ellip' from dual union all
select 5979,'005979','1971-109D','Ariel IV despin weight','',24927,'Reentered','1972-10-20 00:00:00','1972-01-28 00:00:00',95.16,404,649,83.14,'Intermediate' from dual union all
select 598,'000598','1963-021E','NRL PL130','POPPY 2 SV3',9252,'Reentered','1963-07-27 00:00:00','1963-06-24 00:00:00',94.63,173,828,69.92,'Intermediate' from dual union all
select 5980,'005980','1965-108P','Oscar IV','',11858,'Reentered','1976-04-12 00:00:00','1971-09-12 00:00:00',168.77,243,7234,26.36,'Medium' from dual union all
select 5981,'005981','1963-014CY','deb West Ford','RCS 0.12',9245,'Reentered','2001-12-19 00:00:00','1966-09-02 00:00:00',166.16,2671,4597,87.1,'Medium' from dual union all
select 5982,'005982','1965-082UB','deb Transtage 4','',11832,'Reentered','1973-01-31 00:00:00','1972-04-13 00:00:00',102.02,700,1009,32.75,'Intermediate' from dual union all
select 5983,'005983','1965-020FC','deb Blok Kosmos','',11770,'Reentered','1972-08-09 00:00:00','1972-04-13 00:00:00',130.03,497,3751,56.04,'Medium' from dual union all
select 5984,'005984','1968-091CD','deb Kosmos-249','',16818,'Reentered','2002-06-26 00:00:00','1969-04-24 00:00:00',112.18,504,2150,62.37,'Intermediate' from dual union all
select 5985,'005985','61 Omicron 249','deb r/b Ablestar','',7273,'Reentered','1976-12-07 00:00:00','1979-04-02 00:00:00',97.71,591,708,66.69,'Intermediate' from dual union all
select 5986,'005986','1965-108Q','deb Transtage 8','',11858,'In Earth orbit','','2004-10-03 00:00:00',436.87,479,24909,27.3,'Highly Ellip' from dual union all
select 5987,'005987','1967-001S','deb unknown                       ','[UNID052] RCS 0.2',15256,'In Earth orbit','','1971-10-30 00:00:00',532.36,468,30293,26.53,'Highly Ellip' from dual union all
select 5988,'005988','1967-001T','deb unknown                       ','[UNID053] RCS 0.7',15256,'Reentered','1993-01-29 00:00:00','1971-08-24 00:00:00',535.02,635,30271,26.47,'Highly Ellip' from dual union all
select 5989,'005989','1967-001U','deb unknown                       ','[UNID054]',15256,'Reentered','1976-11-07 00:00:00','1975-08-26 00:00:00',255.6,833,13093,26.24,'Medium' from dual union all
select 599,'000599','1963-021C','SR 6A','POPPY 2 SV1',9252,'Reentered','1963-08-01 00:00:00','1963-06-24 00:00:00',94.73,165,846,69.77,'Intermediate' from dual union all
select 5990,'005990','1967-001V','deb unknown                       ','[UNID055] RCS 0.1',15256,'Reentered','1992-10-14 00:00:00','1972-07-31 00:00:00',386.69,258,22150,26.45,'Highly Ellip' from dual union all
select 5991,'005991','1965-108R','deb Transtage 8','',11858,'In Earth orbit','','2004-09-28 00:00:00',547.75,1230,30366,27.38,'Highly Ellip' from dual union all
select 5992,'005992','1965-108S','deb Transtage 8','',11858,'In Earth orbit','','2004-07-18 00:00:00',540.71,1028,30187,26.81,'Highly Ellip' from dual union all
select 5993,'005993','1963-014CZ','deb West Ford','',9245,'Reentered','1974-08-20 00:00:00','1974-08-20 00:00:00',126.08,296,3606,83.35,'Medium' from dual union all
select 5994,'005994','1963-014DA','deb Pageos','[UNID056] RCS 22.2',9245,'Reentered','1995-09-21 00:00:00','1980-01-13 00:00:00',166.23,3031,4242,87.26,'Medium' from dual union all
select 5995,'005995','1963-014DB','deb West Ford','RCS 0.12',9245,'Reentered','1987-10-17 00:00:00','1986-11-03 00:00:00',179.02,2789,5498,87.27,'Medium' from dual union all
select 5996,'005996','1963-014DC','deb West Ford','',9245,'Reentered','1976-10-30 00:00:00','1978-12-22 00:00:00',166.28,3237,4040,87.29,'Medium' from dual union all
select 5997,'005997','1963-014DD','deb West Ford','',9245,'Reentered','1973-11-21 00:00:00','1973-02-17 00:00:00',165.46,2075,5136,86.14,'Medium' from dual union all
select 5998,'005998','1965-108T','deb Transtage 8','',11858,'In Earth orbit','','2004-09-28 00:00:00',485.81,569,27617,26.5,'Highly Ellip' from dual union all
select 5999,'005999','1968-081F','deb OV5-2','',16808,'Reentered','2001-01-01 00:00:00','1971-09-29 00:00:00',194.56,348,9141,26.44,'Medium' from dual union all
select 6,'000006','58 Gamma','Explorer 3','Explorer 3',4797,'Reentered Attached','1958-06-28 00:00:00','1958-03-26 00:00:00',116.03,195,2810,33.38,'Intermediate' from dual union all
select 60,'000060','60 Xi 1','Explorer 8','NASA S-30',6334,'Reentered','2012-03-28 00:00:00','1960-12-05 00:00:00',112.7,416,2286,49.98,'Intermediate' from dual union all
select 600,'000600','1963-021D','NRL PL112','POPPY 2 SV2',9252,'Reentered','1963-07-30 00:00:00','1963-06-24 00:00:00',94.73,174,837,69.94,'Intermediate' from dual union all
select 6000,'006000','1972-031A','Casper','Apollo CM-113',27963,'Deep Space','1972-04-17 00:00:00','1972-04-16 00:00:00',19753.7,202,471165,32.51,'Cislunar' from dual union all
select 6001,'006001','1972-031B','Saturn S-IVB-511','Saturn S-IVB-511',27963,'Deep Space','1972-04-17 00:00:00','1972-04-16 00:00:00',18541.8,147,451211,32.51,'Cislunar' from dual union all
select 6002,'006002','1972-026C','Kapsula Kosmosa-484','Nauka 5KS No. 3',27958,'Reentered','1972-04-18 00:00:00','1972-04-17 00:00:00',87.94,168,177,81.3,'Intermediate' from dual union all
select 6003,'006003','1972-032A','OPS 5640','CORONA CR-16',27964,'Reentered Attached','1972-05-12 00:00:00','1972-04-26 00:00:00',88.52,156,247,81.48,'Intermediate' from dual union all
select 6004,'006004','1963-047N','Centaur AC-2 blanket part','[UNID057] RCS 0.0',9278,'Reentered','1980-02-17 00:00:00','1972-04-20 00:00:00',105.87,578,1492,30.06,'Intermediate' from dual union all
select 6005,'006005','1972-031C','Orion','LM 11 AS',27963,'Deep Space Attached','1972-04-17 00:00:00','1972-04-16 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 6006,'006006','1972-033A','Kosmos-487','DS-P1-Yu No. 57',27965,'Reentered','1972-09-24 00:00:00','1972-05-21 00:00:00',91.84,262,467,70.97,'Intermediate' from dual union all
select 6007,'006007','1972-033B','S1M','S1M',27965,'Reentered','1972-07-03 00:00:00','1972-05-21 00:00:00',91.27,254,419,70.97,'Intermediate' from dual union all
select 6008,'006008','1972-029E','deb Blok-I','',27961,'Reentered','1972-06-01 00:00:00','1972-04-22 00:00:00',92.89,341,492,64.96,'Intermediate' from dual union all
select 6009,'006009','1972-031D','Apollo 16 Subsatellite','Apollo 16 Subsatellite',27963,'Deep Space','1972-04-17 00:00:00','1972-05-01 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 601,'000601','1963-021B','Lofti IIA','Lofti IIA',9252,'Reentered','1963-07-18 00:00:00','1963-06-23 00:00:00',94.64,169,833,69.86,'Intermediate' from dual union all
select 6010,'006010','1972-030D','KDU','KDU',27962,'Reentered','1972-04-27 00:00:00','1972-04-25 00:00:00',88.48,176,222,81.32,'Intermediate' from dual union all
select 6011,'006011','1972-030E','Blenda','BZA',27962,'Reentered','1972-04-26 00:00:00','1972-04-25 00:00:00',88.31,166,216,81.29,'Intermediate' from dual union all
select 6012,'006012','1972-030F','PTDU cover','',27962,'Reentered','1972-04-28 00:00:00','1972-04-27 00:00:00',89.48,183,315,81.27,'Intermediate' from dual union all
select 6013,'006013','1972-033C','deb Kosmos-487','',27965,'Reentered','1972-04-29 00:00:00','1972-04-21 00:00:00',88.44,194,201,70.9,'Intermediate' from dual union all
select 6014,'006014','1970-025MR','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.05,945,1049,100.02,'Sun-Synch' from dual union all
select 6015,'006015','1970-025MS','deb Agena D','',21502,'Reentered','1973-02-19 00:00:00','1972-04-30 00:00:00',106.87,1037,1126,99.76,'Sun-Synch' from dual union all
select 6016,'006016','1972-034A','Kosmos-488','Zenit-4MK',27966,'Landed','1972-05-18 00:00:00','1972-05-10 00:00:00',88.99,176,274,65.41,'Intermediate' from dual union all
select 6017,'006017','1972-034B','11A57I','11A57I',27966,'Reentered','1972-05-11 00:00:00','1972-05-09 00:00:00',88.74,186,239,65.39,'Intermediate' from dual union all
select 6018,'006018','1972-034C','Sensor cover','Zenit-4MK sensor cover',27966,'Reentered','1972-05-07 00:00:00','1972-05-06 00:00:00',88.83,137,297,65.27,'Intermediate' from dual union all
select 6019,'006019','1972-035A','Kosmos-489','Tsiklon',27967,'In Earth orbit','','1972-06-08 00:00:00',104.77,965,1003,74.02,'Intermediate' from dual union all
select 602,'000602','1963-014E','Midas solar array cover','Strongback [RCS 0.4]',9245,'In Earth orbit','','1963-06-17 00:00:00',165.99,3595,3659,87.36,'Medium' from dual union all
select 6020,'006020','1972-035B','S3M','S3M',27967,'In Earth orbit','','1972-06-06 00:00:00',104.61,965,988,74.02,'Intermediate' from dual union all
select 6021,'006021','1972-036A','Kosmos-490','Zenit-2M',27968,'Landed','1972-05-29 00:00:00','1972-05-20 00:00:00',89.34,203,281,65.41,'Intermediate' from dual union all
select 6022,'006022','1972-036B','11A57I','11A57I',27968,'Reentered','1972-05-23 00:00:00','1972-05-19 00:00:00',89.09,194,264,65.4,'Intermediate' from dual union all
select 6023,'006023','1972-034D','KDU part','KDU part',27966,'Reentered','1972-05-20 00:00:00','1972-05-17 00:00:00',89.2,153,316,65.34,'Intermediate' from dual union all
select 6024,'006024','1972-034E','KDU','KDU',27966,'Reentered','1972-05-23 00:00:00','1972-05-17 00:00:00',89.63,176,337,65.41,'Intermediate' from dual union all
select 6025,'006025','1972-034F','Blenda','Sunshade',27966,'Reentered','1972-05-20 00:00:00','1972-05-17 00:00:00',89.51,174,327,65.43,'Intermediate' from dual;
insert into SATELLITE
select 6026,'006026','1972-034G','KDU part','KDU part',27966,'Reentered','1972-05-20 00:00:00','1972-05-18 00:00:00',89.44,158,336,65.39,'Intermediate' from dual union all
select 6027,'006027','1972-034H','KDU part','KDU part',27966,'Reentered','1972-05-19 00:00:00','1972-05-18 00:00:00',89.16,166,300,65.36,'Intermediate' from dual union all
select 6028,'006028','1972-037B','11S510','11S510',27969,'Reentered','1972-06-08 00:00:00','1972-05-26 00:00:00',90.28,209,368,65.39,'Intermediate' from dual union all
select 6029,'006029','1972-037C','BOZ','BOZ',27969,'Reentered','1972-06-09 00:00:00','1972-05-26 00:00:00',90.62,200,409,65.4,'Intermediate' from dual union all
select 603,'000603','1963-022B','Altair X-248-A5S BSB-254','X-248',9253,'Reentered','2000-05-17 00:00:00','1963-07-16 00:00:00',99.62,723,757,90.02,'Polar' from dual union all
select 6030,'006030','1972-037D','deb Blok-I','',27969,'Reentered','1972-06-06 00:00:00','1972-05-26 00:00:00',90.5,215,384,65.41,'Intermediate' from dual union all
select 6031,'006031','1972-037A','Molniya-2','Molniya-2',27969,'Reentered','1977-03-22 00:00:00','1972-06-18 00:00:00',720.19,413,40061,65.45,'GEO Transfer' from dual union all
select 6032,'006032','1972-037E','deb Blok-I','',27969,'Reentered','1972-05-26 00:00:00','1972-05-22 00:00:00',90.23,212,360,65.41,'Intermediate' from dual union all
select 6033,'006033','1972-037F','deb Blok-I','',27969,'Reentered','1972-06-01 00:00:00','1972-05-22 00:00:00',90.5,206,393,65.44,'Intermediate' from dual union all
select 6034,'006034','1972-037G','Blok-L','Blok-L',27969,'Reentered','1977-06-01 00:00:00','1972-06-18 00:00:00',704.08,463,39214,65.32,'GEO Transfer' from dual union all
select 6035,'006035','1972-038A','Kosmos-491','Zenit-4M',27970,'Landed','1972-06-08 00:00:00','1972-05-27 00:00:00',89.43,174,319,65,'Intermediate' from dual union all
select 6036,'006036','1972-038B','11A57I','11A57I',27970,'Reentered','1972-05-30 00:00:00','1972-05-27 00:00:00',88.98,189,260,64.93,'Intermediate' from dual union all
select 6037,'006037','1972-039A','OPS 6371','CORONA CR-8R',27971,'Reentered Attached','1972-06-04 00:00:00','1972-05-27 00:00:00',89.22,157,314,96.35,'Sun-Synch' from dual union all
select 6038,'006038','1972-038C','Sensor cover','Zenit-4M sensor cover',27970,'Reentered','1972-05-29 00:00:00','1972-05-25 00:00:00',88.75,175,251,64.97,'Intermediate' from dual union all
select 6039,'006039','1972-036C','Nauka part','',27968,'Reentered','1972-05-30 00:00:00','1972-05-27 00:00:00',89.01,200,252,65.45,'Intermediate' from dual union all
select 604,'000604','1963-024A','Tiros 7','Tiros G',9255,'Reentered','1994-06-03 00:00:00','1963-07-24 00:00:00',97.41,614,655,58.21,'Intermediate' from dual union all
select 6040,'006040','1972-036D','Kapsula Kosmosa-490','Nauka 1KS No. 5',27968,'Reentered','1972-06-06 00:00:00','1972-05-27 00:00:00',89.17,209,258,65.37,'Intermediate' from dual union all
select 6041,'006041','1965-082UC','deb Transtage 4','',11832,'Reentered','1972-07-04 00:00:00','1972-05-31 00:00:00',89.56,195,310,31.67,'Intermediate' from dual union all
select 6042,'006042','1965-082UD','deb Transtage 4','',11832,'Reentered','1973-08-28 00:00:00','1972-05-29 00:00:00',101.03,644,971,32.8,'Intermediate' from dual union all
select 6043,'006043','1971-015CG','deb Kosmos-397','',24833,'Reentered','1979-07-31 00:00:00','1979-02-22 00:00:00',97.75,566,737,66.66,'Intermediate' from dual union all
select 6044,'006044','1969-082KF','deb Agena D','',18349,'Reentered','1981-12-03 00:00:00','1981-04-08 00:00:00',99.3,688,762,70.43,'Intermediate' from dual union all
select 6045,'006045','1968-091CE','deb Kosmos-249','',16818,'Reentered','1981-03-06 00:00:00','1976-12-03 00:00:00',110.3,559,1922,62.35,'Intermediate' from dual union all
select 6046,'006046','1972-038D','Blenda','BZA',27970,'Reentered','1972-06-09 00:00:00','1972-06-07 00:00:00',89.36,160,326,64.98,'Intermediate' from dual union all
select 6047,'006047','1972-038E','KDU','KDU',27970,'Reentered','1972-06-11 00:00:00','1972-06-07 00:00:00',89.47,171,326,64.97,'Intermediate' from dual union all
select 6048,'006048','1972-038F','PTDU cover','KDU part',27970,'Reentered','1972-06-10 00:00:00','1972-06-09 00:00:00',90.3,193,386,64.97,'Intermediate' from dual union all
select 6049,'006049','1972-040A','Kosmos-492','Zenit-4M',27972,'Landed','1972-06-22 00:00:00','1972-06-11 00:00:00',89.4,166,324,64.96,'Intermediate' from dual union all
select 605,'000605','1963-024B','Altair SV-355','Altair X-248-A5DM SV-35',9255,'Reentered','1978-12-23 00:00:00','1963-07-23 00:00:00',97.36,613,651,58.21,'Intermediate' from dual union all
select 6050,'006050','1972-040B','11A57I','11A57I',27972,'Reentered','1972-06-18 00:00:00','1972-06-11 00:00:00',89.42,199,293,64.97,'Intermediate' from dual union all
select 6051,'006051','1970-025MT','deb Agena D','',21502,'Reentered','1976-03-17 00:00:00','1976-03-01 00:00:00',89.82,266,266,99.95,'Retrograde' from dual union all
select 6052,'006052','1972-041A','INTELSAT IV F5','INTELSAT IV F5',27973,'In Earth orbit','','1977-02-24 00:00:00',1436.11,35786,35788,.1,'Stationary' from dual union all
select 6053,'006053','1972-042A','Kosmos-493','Zenit-2M',27974,'Landed','1972-07-03 00:00:00','1972-06-24 00:00:00',89.19,202,267,64.97,'Intermediate' from dual union all
select 6054,'006054','1972-042B','11A57I','11A57I',27974,'Reentered','1972-06-25 00:00:00','1972-06-23 00:00:00',88.79,187,243,65.02,'Intermediate' from dual union all
select 6055,'006055','1972-040C','Blenda','BZA',27972,'Reentered','1972-06-24 00:00:00','1972-06-21 00:00:00',89.18,175,293,65.01,'Intermediate' from dual union all
select 6056,'006056','1972-042C','Sensor cover','Sensor cover',27974,'Reentered','1972-06-23 00:00:00','1972-06-21 00:00:00',88.8,196,235,65.02,'Intermediate' from dual union all
select 6057,'006057','1972-040D','KDU','KDU',27972,'Reentered','1972-06-27 00:00:00','1972-06-21 00:00:00',89.32,178,303,64.97,'Intermediate' from dual union all
select 6058,'006058','1972-041B','Centaur AC-29','Centaur 26D',27973,'In Earth orbit','','1972-07-13 00:00:00',653.29,559,36565,27,'GEO Transfer' from dual union all
select 6059,'006059','1972-043A','Kosmos-494','Strela-2M',27975,'In Earth orbit','','1972-07-30 00:00:00',100.77,788,803,74.07,'Intermediate' from dual union all
select 606,'000606','1963-024C','Tiros VII despin weight','',9255,'Reentered','1988-06-09 00:00:00','1964-02-03 00:00:00',97.85,631,681,58.38,'Intermediate' from dual union all
select 6060,'006060','1972-044A','Kosmos-495','Zenit-4M',27976,'Landed','1972-07-06 00:00:00','1972-06-26 00:00:00',89.23,199,274,65.42,'Intermediate' from dual union all
select 6061,'006061','1972-043B','S3M','S3M',27975,'In Earth orbit','','1972-07-24 00:00:00',100.63,776,800,74.07,'Intermediate' from dual union all
select 6062,'006062','1972-044B','11A57I','11A57I',27976,'Reentered','1972-06-29 00:00:00','1972-06-25 00:00:00',88.93,192,251,65.41,'Intermediate' from dual union all
select 6063,'006063','1972-043C','deb Kosmos-494','Strela-2M debris',27975,'In Earth orbit','','1972-07-24 00:00:00',101.15,802,825,74.07,'Intermediate' from dual union all
select 6064,'006064','1972-040E','KDU part','KDU part',27972,'Reentered','1972-06-27 00:00:00','1972-06-24 00:00:00',89.24,204,270,64.96,'Intermediate' from dual union all
select 6065,'006065','1972-043D','deb Kosmos-494','Strela-2M debris',27975,'In Earth orbit','','1972-08-02 00:00:00',101.31,803,839,74.09,'Intermediate' from dual union all
select 6066,'006066','1972-045A','Kosmos-496','Soyuz 7K-T No. 33A',27977,'Landed','1972-07-02 00:00:00','1972-06-28 00:00:00',89.55,188,317,51.6,'Intermediate' from dual union all
select 6067,'006067','1972-045B','11S510','11S510',27977,'Reentered','1972-07-02 00:00:00','1972-06-28 00:00:00',89.31,187,293,51.57,'Intermediate' from dual union all
select 6068,'006068','1972-046A','Prognoz-2','SO-M No. 503',27978,'Deep Space','1972-06-29 00:00:00','1972-06-29 00:00:00',5845.25,418,201802,65.3,'Deep Space' from dual union all
select 6069,'006069','1972-046B','11S510','11S510',27978,'Reentered','1972-08-01 00:00:00','1972-07-06 00:00:00',90.83,225,406,64.93,'Intermediate' from dual union all
select 607,'000607','1963-024D','Tiros VII despin weight','',9255,'Reentered','1980-02-01 00:00:00','1963-08-07 00:00:00',96.89,573,646,58.1,'Intermediate' from dual union all
select 6070,'006070','1972-046C','BOZ','BOZ',27978,'Reentered','1972-08-13 00:00:00','1972-07-06 00:00:00',91.36,228,455,64.97,'Intermediate' from dual union all
select 6071,'006071','1972-046D','deb Blok-I','',27978,'Reentered','1972-07-19 00:00:00','1972-07-08 00:00:00',91.47,165,529,65.49,'Intermediate' from dual union all
select 6072,'006072','1970-103F','deb SOZ','Kosmos-382 part',21580,'Reentered','1977-01-03 00:00:00','1971-11-30 00:00:00',158.57,2300,4353,51.14,'Medium' from dual union all
select 6073,'006073','1972-023E','Kosmos-482 SA','SA',27955,'In Earth orbit','','1972-07-04 00:00:00',200.25,209,9710,52.14,'Medium' from dual union all
select 6074,'006074','1972-046E','deb Blok-I','',27978,'Reentered','1972-07-04 00:00:00','1972-07-02 00:00:00',92.31,244,532,64.89,'Intermediate' from dual union all
select 6075,'006075','1972-047A','Interkosmos-7','DS-U3-IK No. 3',27979,'Reentered','1972-10-05 00:00:00','1972-08-01 00:00:00',92.18,255,508,48.42,'Intermediate' from dual union all
select 6076,'006076','1972-048A','Kosmos-497','DS-P1-I No. 12',27980,'Reentered','1973-11-07 00:00:00','1972-07-30 00:00:00',94.98,270,766,71,'Intermediate' from dual union all
select 6077,'006077','1972-048B','S1M','S1M',27980,'Reentered','1973-03-17 00:00:00','1972-07-30 00:00:00',94.67,269,737,70.96,'Intermediate' from dual union all
select 6078,'006078','1972-047B','S1M','S1M',27979,'Reentered','1972-09-29 00:00:00','1972-07-30 00:00:00',91.88,262,472,48.42,'Intermediate' from dual union all
select 6079,'006079','1972-049A','Meteor','Meteor-MV No. 26',27981,'In Earth orbit','','1972-08-08 00:00:00',102.88,888,902,81.22,'Intermediate' from dual union all
select 608,'000608','1963-014C','TRS 6','ERS 6',9245,'In Earth orbit','','1964-03-17 00:00:00',166.41,3634,3653,87.35,'Medium' from dual union all
select 6080,'006080','1972-049B','8A92ME','8A92ME',27981,'In Earth orbit','','1972-08-07 00:00:00',102.98,862,938,81.24,'Intermediate' from dual union all
select 6081,'006081','1972-047C','deb Interkosmos-7','',27979,'Reentered','1972-09-01 00:00:00','1972-07-12 00:00:00',91.96,262,480,48.43,'Intermediate' from dual union all
select 6082,'006082','1972-048C','deb Kosmos-497','',27980,'Reentered','1976-01-01 00:00:00','1972-07-02 00:00:00',95.14,268,783,70.98,'Intermediate' from dual union all
select 6083,'006083','1972-045C','Kosmos-496 BO ','Bitovoy otsek',27977,'Reentered','1972-07-04 00:00:00','1972-07-02 00:00:00',88.63,173,241,51.6,'Intermediate' from dual union all
select 6084,'006084','1970-025MU','deb Agena D','',21502,'Reentered','1974-05-12 00:00:00','1974-05-06 00:00:00',91.18,304,361,99.94,'Retrograde' from dual union all
select 6085,'006085','1970-089CT','deb Kosmos-374 [*]','',21566,'Reentered','2001-06-13 00:00:00','1992-02-07 00:00:00',102.02,536,1172,62.78,'Intermediate' from dual union all
select 6086,'006086','1972-050A','Kosmos-498','DS-P1-Yu No. 56',27982,'Reentered','1972-11-25 00:00:00','1972-08-04 00:00:00',91.81,266,461,70.97,'Intermediate' from dual union all
select 6087,'006087','1972-050B','S1M','S1M',27982,'Reentered','1972-09-27 00:00:00','1972-08-04 00:00:00',91.4,265,421,70.96,'Intermediate' from dual union all
select 6088,'006088','1972-044C','Blenda','BZA',27976,'Reentered','1972-07-06 00:00:00','1972-07-05 00:00:00',88.56,158,248,65.41,'Intermediate' from dual union all
select 6089,'006089','1972-044D','KDU','KDU',27976,'Reentered','1972-07-08 00:00:00','1972-07-05 00:00:00',88.95,164,281,65.4,'Intermediate' from dual union all
select 609,'000609','1963-025A','OPS 0999','CORONA M-22',9256,'Reentered Attached','1963-07-26 00:00:00','1963-07-05 00:00:00',90.24,185,388,81.62,'Intermediate' from dual union all
select 6090,'006090','1972-051A','Kosmos-499','Zenit-4M',27983,'Landed','1972-07-17 00:00:00','1972-07-08 00:00:00',88.85,187,249,51.75,'Intermediate' from dual union all
select 6091,'006091','1972-051B','11A57I','11A57I',27983,'Reentered','1972-07-10 00:00:00','1972-07-08 00:00:00',88.69,182,238,51.72,'Intermediate' from dual union all
select 6092,'006092','1972-051C','Sensor cover','Zenit-4M sensor cover',27983,'Reentered','1972-07-10 00:00:00','1972-07-08 00:00:00',88.76,183,243,51.75,'Intermediate' from dual union all
select 6093,'006093','1972-044E','PTDU cover','KDU part',27976,'Reentered','1972-07-07 00:00:00','1972-07-06 00:00:00',89.82,174,358,65.41,'Intermediate' from dual union all
select 6094,'006094','1972-052A','OPS 7293','HEXAGON SV-3',27984,'Deorbited','1972-09-13 00:00:00','1972-08-06 00:00:00',88.76,175,252,96.87,'Sun-Synch' from dual union all
select 6095,'006095','1972-052B','Titan 23D-5 Stage 2','LR91AJ11 23D-5',27984,'Reentered','1972-07-09 00:00:00','1972-07-07 00:00:00',88.5,162,239,96.88,'Sun-Synch' from dual union all
select 6096,'006096','1972-052C','URSALA I','P-11 No. 4425',27984,'Reentered','1978-05-06 00:00:00','1972-08-06 00:00:00',94.55,493,501,96.14,'Sun-Synch' from dual union all
select 6097,'006097','1972-053A','Kosmos-500','Tselina-OM',27985,'Reentered','1980-03-29 00:00:00','1972-08-11 00:00:00',95.12,506,543,74.06,'Intermediate' from dual union all
select 6098,'006098','1972-053B','S3M','S3M',27985,'Reentered','1980-04-27 00:00:00','1972-08-12 00:00:00',95.01,496,543,74.07,'Intermediate' from dual union all
select 6099,'006099','1972-054A','Kosmos-501','DS-P1-Yu No. 50',27986,'Reentered','1974-05-09 00:00:00','1972-08-11 00:00:00',108.2,215,2072,48.44,'Intermediate' from dual union all
select 61,'000061','60 Omicron','Discoverer 17','CORONA C''-2',6328,'Reentered Attached','1960-12-29 00:00:00','1960-11-20 00:00:00',95.7,182,923,81.86,'Intermediate' from dual union all
select 610,'000610','1963-022C','Transit VA-3 despin weight','',9253,'Reentered','1991-11-15 00:00:00','1963-09-03 00:00:00',101.14,741,885,90.22,'Polar' from dual union all
select 6100,'006100','1972-054B','S1M','S1M',27986,'Reentered','1973-06-10 00:00:00','1972-08-12 00:00:00',107.65,217,2019,48.45,'Intermediate' from dual union all
select 6101,'006101','1972-053C','deb Kosmos-500','Tselina-O part',27985,'Reentered','1973-03-25 00:00:00','1972-08-09 00:00:00',94.73,472,539,74.04,'Intermediate' from dual union all
select 6102,'006102','1972-053D','deb Kosmos-500','Tselina-O part',27985,'Reentered','1973-04-15 00:00:00','1972-08-09 00:00:00',94.79,483,534,74.01,'Intermediate' from dual union all
select 6103,'006103','1972-054C','deb S1M','',27986,'Reentered','1972-09-05 00:00:00','1972-07-19 00:00:00',105.78,212,1850,48.43,'Intermediate' from dual union all
select 6104,'006104','1972-054D','deb S1M','',27986,'Reentered','1972-08-03 00:00:00','1972-07-19 00:00:00',103.02,231,1572,48.4,'Intermediate' from dual union all
select 6105,'006105','1972-055A','Kosmos-502','Zenit-4MT',27987,'Landed','1972-07-25 00:00:00','1972-07-15 00:00:00',89.13,200,263,65.4,'Intermediate' from dual union all
select 6106,'006106','1972-055B','11S510','11S510',27987,'Reentered','1972-07-18 00:00:00','1972-07-15 00:00:00',88.78,191,238,65.38,'Intermediate' from dual union all
select 6107,'006107','1972-054E','deb S1M','',27986,'Reentered','1972-08-07 00:00:00','1972-07-24 00:00:00',101.19,216,1414,48.38,'Intermediate' from dual union all
select 6108,'006108','1972-054F','deb S1M','',27986,'Reentered','1972-08-06 00:00:00','1972-07-24 00:00:00',99.38,204,1253,48.52,'Intermediate' from dual union all
select 6109,'006109','1972-054G','deb S1M','',27986,'Reentered','1972-07-25 00:00:00','1972-07-14 00:00:00',106.11,214,1879,48.47,'Intermediate' from dual union all
select 611,'000611','1963-022D','Transit VA-3 despin weight','',9253,'Reentered','1971-03-01 00:00:00','1963-08-09 00:00:00',98.12,580,758,89.83,'Polar' from dual union all
select 6110,'006110','1972-054H','deb S1M','',27986,'Reentered','1972-07-25 00:00:00','1972-07-14 00:00:00',105.6,225,1820,48.37,'Intermediate' from dual union all
select 6111,'006111','1972-055C','Sensor cover','Zenit-4MT sensor cover',27987,'Reentered','1972-07-15 00:00:00','1972-07-14 00:00:00',89.23,208,264,65.42,'Intermediate' from dual union all
select 6112,'006112','1972-051D','Blenda','BZA',27983,'Reentered','1972-07-19 00:00:00','1972-07-16 00:00:00',88.7,157,264,51.75,'Intermediate' from dual union all
select 6113,'006113','1972-051E','KDU','KDU',27983,'Reentered','1972-07-20 00:00:00','1972-07-16 00:00:00',88.93,171,273,51.77,'Intermediate' from dual union all
select 6114,'006114','1972-056A','Kosmos-503','Zenit-4M',27988,'Landed','1972-08-01 00:00:00','1972-07-21 00:00:00',88.99,186,264,65.43,'Intermediate' from dual union all
select 6115,'006115','1972-056B','11A57I','11A57I',27988,'Reentered','1972-07-27 00:00:00','1972-07-21 00:00:00',89.17,207,260,65.39,'Intermediate' from dual;
insert into SATELLITE
select 6116,'006116','1972-056C','Sensor cover','Zenit-4M sensor cover',27988,'Reentered','1972-07-23 00:00:00','1972-07-19 00:00:00',88.9,186,255,65.42,'Intermediate' from dual union all
select 6117,'006117','1972-057A','Kosmos-504','Strela-1M',27989,'In Earth orbit','','1972-08-22 00:00:00',113.99,1323,1495,74.03,'Intermediate' from dual union all
select 6118,'006118','1972-057B','Kosmos-505','Strela-1M',27989,'In Earth orbit','','1972-08-22 00:00:00',114.33,1353,1497,74.03,'Intermediate' from dual union all
select 6119,'006119','1972-057C','Kosmos-506','Strela-1M',27989,'In Earth orbit','','1972-08-22 00:00:00',114.65,1383,1496,74.03,'Intermediate' from dual union all
select 612,'000612','1963-026A','GRS','AFCRL A',9257,'Reentered Attached','1983-12-14 00:00:00','1963-07-30 00:00:00',102.13,421,1298,49.74,'Intermediate' from dual union all
select 6120,'006120','1972-057D','Kosmos-507','Strela-1M',27989,'In Earth orbit','','1972-08-22 00:00:00',114.98,1413,1496,74.03,'Intermediate' from dual union all
select 6121,'006121','1972-057E','Kosmos-508','Strela-1M',27989,'In Earth orbit','','1972-08-20 00:00:00',115.32,1444,1496,74.03,'Intermediate' from dual union all
select 6122,'006122','1972-057F','Kosmos-509','Strela-1M',27989,'In Earth orbit','','1972-08-24 00:00:00',115.67,1474,1498,74.03,'Intermediate' from dual union all
select 6123,'006123','1972-057G','Kosmos-510','Strela-1M',27989,'In Earth orbit','','1972-08-20 00:00:00',116.03,1495,1510,74.03,'Intermediate' from dual union all
select 6124,'006124','1972-057H','Kosmos-511','Strela-1M',27989,'In Earth orbit','','1972-08-22 00:00:00',116.43,1495,1546,74.03,'Intermediate' from dual union all
select 6125,'006125','1972-057J','S3M','S3M',27989,'In Earth orbit','','1972-08-26 00:00:00',117.04,1495,1600,74.03,'Intermediate' from dual union all
select 6126,'006126','1972-058A','ERTS 1','ERTS A',27990,'In Earth orbit','','1972-08-23 00:00:00',103.19,902,917,99.11,'Sun-Synch' from dual union all
select 6127,'006127','1972-058B','Delta 89','Delta DSV-3N-4 (main deb',27990,'In Earth orbit','','1972-08-25 00:00:00',100.27,633,910,98.49,'Sun-Synch' from dual union all
select 6128,'006128','1972-055D','KB electropitaniya','KBE',27987,'Reentered','1972-07-25 00:00:00','1972-07-25 00:00:00',88.53,175,229,65.4,'Intermediate' from dual union all
select 6129,'006129','1972-055E','Blenda','KBE part',27987,'Reentered','1972-07-26 00:00:00','1972-07-26 00:00:00',89.38,180,308,65.4,'Intermediate' from dual union all
select 613,'000613','1963-027A','OPS 1440','Project 698BK Group 1-D',9258,'Reentered Attached','1969-10-26 00:00:00','1963-08-02 00:00:00',94.67,485,521,82.31,'Intermediate' from dual union all
select 6130,'006130','1972-059A','Kosmos-512','Zenit-2M',27991,'Landed','1972-08-09 00:00:00','1972-07-30 00:00:00',89.23,202,270,65.38,'Intermediate' from dual union all
select 6131,'006131','1972-059B','11A57I','11A57I',27991,'Reentered','1972-08-01 00:00:00','1972-07-30 00:00:00',88.61,170,242,65.38,'Intermediate' from dual union all
select 6132,'006132','1972-056D','Blenda','BZA',27988,'Reentered','1972-08-01 00:00:00','1972-07-31 00:00:00',88.94,157,288,65.37,'Intermediate' from dual union all
select 6133,'006133','1972-054J','deb S1M','',27986,'Reentered','1972-09-18 00:00:00','1972-08-20 00:00:00',96.8,214,996,48.47,'Intermediate' from dual union all
select 6134,'006134','1972-056E','KDU','KDU',27988,'Reentered','1972-08-03 00:00:00','1972-07-31 00:00:00',89.04,162,293,65.46,'Intermediate' from dual union all
select 6135,'006135','1972-060A','Kosmos-513','Zenit-4M',27992,'Landed','1972-08-15 00:00:00','1972-08-04 00:00:00',89.34,173,311,64.97,'Intermediate' from dual union all
select 6136,'006136','1972-060B','11A57I','11A57I',27992,'Reentered','1972-08-09 00:00:00','1972-08-04 00:00:00',89.4,195,295,64.99,'Intermediate' from dual union all
select 6137,'006137','1972-060C','Sensor cover','Zenit-4M sensor cover',27992,'Reentered','1972-08-06 00:00:00','1972-08-04 00:00:00',89.66,201,315,65,'Intermediate' from dual union all
select 6138,'006138','1968-091CF','deb Kosmos-249','',16818,'Reentered','2004-09-16 00:00:00','1972-08-11 00:00:00',112.04,512,2129,62.38,'Intermediate' from dual union all
select 6139,'006139','1970-025MV','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',104.92,926,1056,99.36,'Sun-Synch' from dual union all
select 614,'000614','1963-025B','Hitchhiker 1','P-11 No. 4201',9256,'In Earth orbit','','1963-08-01 00:00:00',132.42,331,4125,82.14,'Medium' from dual union all
select 6140,'006140','1970-025MW','deb Agena D','',21502,'Reentered','1979-03-20 00:00:00','1979-03-14 00:00:00',99.75,606,888,98.33,'Sun-Synch' from dual union all
select 6141,'006141','1965-082UE','deb Transtage 4','',11832,'Reentered','1973-02-23 00:00:00','1972-08-12 00:00:00',104.86,701,1274,32.87,'Intermediate' from dual union all
select 6142,'006142','1972-061A','Explorer 46','Meteoroid Technology Sat',27993,'Reentered Attached','1979-11-02 00:00:00','1972-09-12 00:00:00',97.77,494,809,37.7,'Intermediate' from dual union all
select 6143,'006143','1972-060D','KDU','KDU',27992,'Reentered','1972-08-20 00:00:00','1972-08-14 00:00:00',89.79,165,364,64.84,'Intermediate' from dual union all
select 6144,'006144','1972-060E','Blenda','BZA',27992,'Reentered','1972-08-24 00:00:00','1972-08-15 00:00:00',89.43,166,327,64.97,'Intermediate' from dual union all
select 6145,'006145','1972-061B','Explorer 46 despin weight','',27993,'Reentered','1979-03-05 00:00:00','1972-09-12 00:00:00',97.94,490,831,37.69,'Intermediate' from dual union all
select 6146,'006146','1972-061C','Explorer 46 despin weight','',27993,'Reentered','1978-05-25 00:00:00','1972-09-12 00:00:00',97.47,457,819,37.68,'Intermediate' from dual union all
select 6147,'006147','1972-060F','PTDU cover','KDU part',27992,'Reentered','1972-08-18 00:00:00','1972-08-16 00:00:00',90.95,181,461,64.96,'Intermediate' from dual union all
select 6148,'006148','1972-062A','Kosmos-514','Tsiklon GVM',27994,'In Earth orbit','','1972-09-16 00:00:00',104.36,956,973,82.97,'Intermediate' from dual union all
select 6149,'006149','1972-062B','S3M','S3M',27994,'In Earth orbit','','1972-09-17 00:00:00',104.29,950,973,82.97,'Intermediate' from dual union all
select 615,'000615','1963-027B','deb USAF satellite/Agena B','',9258,'Reentered','1964-11-14 00:00:00','1963-08-01 00:00:00',94.6,490,509,82.31,'Intermediate' from dual union all
select 6150,'006150','1972-063A','Kosmos-515','Zenit-4MK',27995,'Landed','1972-08-31 00:00:00','1972-08-20 00:00:00',89.59,204,305,72.9,'Intermediate' from dual union all
select 6151,'006151','1972-063B','11A57I','11A57I',27995,'Reentered','1972-08-23 00:00:00','1972-08-20 00:00:00',88.79,183,246,72.85,'Intermediate' from dual union all
select 6152,'006152','1972-064A','Denpa','REXS',27996,'Reentered','1980-05-19 00:00:00','1972-09-18 00:00:00',156.81,236,6273,31.06,'Medium' from dual union all
select 6153,'006153','1972-065A','Copernicus','OAO-C',27997,'In Earth orbit','','1972-09-22 00:00:00',99.63,735,747,35.01,'Intermediate' from dual union all
select 6154,'006154','1972-066A','Kosmos-516','US-A reactor',27998,'In Earth orbit','','1972-09-22 00:00:00',89.62,247,265,64.97,'Intermediate' from dual union all
select 6155,'006155','1972-065B','Centaur D AC-22','Centaur 19D',27997,'In Earth orbit','','1972-09-20 00:00:00',99.59,698,780,35.01,'Intermediate' from dual union all
select 6156,'006156','1972-065C','PEP Sun Baffle Cover','Sun Baffle Cover',27997,'Reentered','1982-01-07 00:00:00','1972-09-22 00:00:00',99.6,732,747,35.01,'Intermediate' from dual union all
select 6157,'006157','1970-025MX','deb Agena D','',21502,'In Earth orbit','','1992-02-04 00:00:00',104.45,960,978,99.85,'Sun-Synch' from dual union all
select 6158,'006158','1963-047P','Centaur AC-2 part','Insulation blanket part',9278,'Reentered','1973-11-22 00:00:00','1972-08-24 00:00:00',103.53,577,1274,31.71,'Intermediate' from dual union all
select 6159,'006159','1970-025MY','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',113.22,1327,1422,101.29,'Sun-Synch' from dual union all
select 616,'000616','1963-027C','deb USAF satellite/Agena B','',9258,'Reentered','1963-07-10 00:00:00','1963-07-01 00:00:00',90.95,179,464,82.31,'Intermediate' from dual union all
select 6160,'006160','1970-025MZ','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',106.55,964,1170,100.47,'Sun-Synch' from dual union all
select 6161,'006161','1970-025NA','deb Agena D','',21502,'Reentered','1988-10-30 00:00:00','1988-10-29 00:00:00',92.24,381,387,101.9,'Retrograde' from dual union all
select 6162,'006162','1972-043E','deb Kosmos-494','',27975,'Reentered','1982-03-23 00:00:00','1972-08-26 00:00:00',100.85,775,823,74.05,'Intermediate' from dual union all
select 6163,'006163','1971-119D','deb Aureole 1','',24937,'Reentered','1973-10-02 00:00:00','1972-08-28 00:00:00',112.59,382,2310,73.96,'Intermediate' from dual union all
select 6164,'006164','1972-053E','deb Kosmos-500','Tselina-O part',27985,'Reentered','1973-03-29 00:00:00','1972-08-27 00:00:00',94.68,475,532,74.12,'Intermediate' from dual union all
select 6165,'006165','1972-063C','Blenda','Sunshade',27995,'Reentered','1972-09-03 00:00:00','1972-08-29 00:00:00',89.36,174,312,72.85,'Intermediate' from dual union all
select 6166,'006166','1972-063D','KDU','KDU',27995,'Reentered','1972-09-04 00:00:00','1972-08-29 00:00:00',89.55,188,317,72.84,'Intermediate' from dual union all
select 6167,'006167','1970-025NB','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',105.99,1016,1065,100.04,'Sun-Synch' from dual union all
select 6168,'006168','1972-067A','Kosmos-517','Zenit-2M',27999,'Landed','1972-09-11 00:00:00','1972-09-01 00:00:00',89.37,201,285,64.98,'Intermediate' from dual union all
select 6169,'006169','1972-067B','11A57I','11A57I',27999,'Reentered','1972-09-03 00:00:00','1972-09-01 00:00:00',88.73,181,243,65,'Intermediate' from dual union all
select 617,'000617','1963-021G','deb USN satellite/Agena D','',9252,'Reentered','1963-07-03 00:00:00','1963-07-02 00:00:00',89.78,159,369,69.82,'Intermediate' from dual union all
select 6170,'006170','1972-067C','Sensor cover','Sensor cover',27999,'Reentered','1972-09-08 00:00:00','1972-09-02 00:00:00',88.84,188,247,65,'Intermediate' from dual union all
select 6171,'006171','1972-063E','PTDU cover','KDU part',27995,'Reentered','1972-09-02 00:00:00','1972-08-31 00:00:00',89.49,181,318,72.85,'Intermediate' from dual union all
select 6172,'006172','1972-068A','OPS 8888','GAMBIT-3 Mission 4336',28000,'Reentered Attached','1972-09-30 00:00:00','1972-09-08 00:00:00',89.82,150,382,110.41,'Retrograde' from dual union all
select 6173,'006173','1972-069A','Triad OI-1X','Triad OI-1X',28001,'In Earth orbit','','1972-10-05 00:00:00',100.64,741,837,90.13,'Polar' from dual union all
select 6174,'006174','1972-053F','deb Kosmos-500','Tselina-O part',27985,'Reentered','1973-04-14 00:00:00','1972-09-02 00:00:00',94.68,478,528,74.01,'Intermediate' from dual union all
select 6175,'006175','1968-097DU','deb Kosmos-252','',16824,'Reentered','1975-09-27 00:00:00','1972-09-01 00:00:00',110.23,548,1927,62.53,'Intermediate' from dual union all
select 6176,'006176','1968-091CG','deb Kosmos-249','',16818,'In Earth orbit','','1992-02-06 00:00:00',110.61,883,1627,65.73,'Intermediate' from dual union all
select 6177,'006177','1970-089CU','deb Kosmos-374','',21566,'Reentered','1982-01-04 00:00:00','1972-09-01 00:00:00',106.07,532,1557,62.33,'Intermediate' from dual union all
select 6178,'006178','1969-082KG','deb Agena D','',18349,'Reentered','1979-12-16 00:00:00','1972-09-01 00:00:00',100.9,689,914,70.11,'Intermediate' from dual union all
select 6179,'006179','61 Omicron 250','deb r/b Ablestar','',7273,'Reentered','1976-11-20 00:00:00','1972-09-01 00:00:00',102.93,818,977,66.73,'Intermediate' from dual union all
select 618,'000618','1963-028A','OPS 1467','GAMBIT SV 951',9259,'Reentered','1963-07-18 00:00:00','1963-07-14 00:00:00',88.23,185,189,95.37,'Sun-Synch' from dual union all
select 6180,'006180','1972-069B','FW4S S/N 2376-1','FW4S',28001,'In Earth orbit','','1972-10-05 00:00:00',100.63,740,837,90.12,'Polar' from dual union all
select 6181,'006181','1971-076C','deb OPS 7681','',24894,'Reentered','1973-03-29 00:00:00','1972-09-05 00:00:00',93.95,456,479,75.16,'Intermediate' from dual union all
select 6182,'006182','1963-049L','deb Transit VBN-2','[UNID058] RCS 0.0',9280,'Reentered','1980-08-09 00:00:00','1972-09-07 00:00:00',104.91,830,1151,89.76,'Polar' from dual union all
select 6183,'006183','1965-082UF','deb Transtage 4','',11832,'Reentered','1974-09-15 00:00:00','1974-05-05 00:00:00',93.72,450,462,32.85,'Intermediate' from dual union all
select 6184,'006184','1970-025NC','deb Agena D','',21502,'Reentered','1990-11-06 00:00:00','1990-11-05 00:00:00',88.13,175,188,98.43,'Sun-Synch' from dual union all
select 6185,'006185','1970-025ND','deb Agena D','',21502,'Reentered','1981-06-08 00:00:00','1981-02-22 00:00:00',99.45,665,800,100.92,'Sun-Synch' from dual union all
select 6186,'006186','1972-070A','Kosmos-518','Zenit-2M',28002,'Landed','1972-09-24 00:00:00','1972-09-17 00:00:00',89.55,201,304,72.83,'Intermediate' from dual union all
select 6187,'006187','1972-070B','11A57I','11A57I',28002,'Reentered','1972-09-22 00:00:00','1972-09-17 00:00:00',89.21,191,279,72.9,'Intermediate' from dual union all
select 6188,'006188','1972-071A','Kosmos-519','Zenit-4M',28003,'Landed','1972-09-26 00:00:00','1972-09-18 00:00:00',90.14,205,358,71.34,'Intermediate' from dual union all
select 6189,'006189','1972-071B','11A57I','11A57I',28003,'Reentered','1972-09-24 00:00:00','1972-09-18 00:00:00',89.42,196,295,71.34,'Intermediate' from dual union all
select 619,'000619','1963-028B','AFP-206 sensor cover','',9259,'Reentered','1963-07-13 00:00:00','1963-07-13 00:00:00',87.87,123,216,95.32,'Sun-Synch' from dual union all
select 6190,'006190','1971-119E','deb Aureole 1','',24937,'Reentered','1977-10-25 00:00:00','1972-09-16 00:00:00',112.49,380,2302,74,'Intermediate' from dual union all
select 6191,'006191','1972-071C','Sensor cover','Zenit-4M sensor cover',28003,'Reentered','1972-09-19 00:00:00','1972-09-17 00:00:00',88.98,206,243,71.38,'Intermediate' from dual union all
select 6192,'006192','1972-072A','Kosmos-520','Oko',28004,'In Earth orbit','','1972-12-26 00:00:00',715.03,742,39477,62.88,'Molniya' from dual union all
select 6193,'006193','1972-072B','11S510','11S510',28004,'Reentered','1972-10-20 00:00:00','1972-09-26 00:00:00',92.43,197,590,62.79,'Intermediate' from dual union all
select 6194,'006194','1972-072C','BOZ','BOZ',28004,'Reentered','1972-11-05 00:00:00','1972-09-27 00:00:00',92.51,213,583,62.82,'Intermediate' from dual union all
select 6195,'006195','1972-072D','deb Blok-I','',28004,'Reentered','1972-09-30 00:00:00','1972-09-21 00:00:00',92.8,213,611,62.8,'Intermediate' from dual union all
select 6196,'006196','1972-066B','Kosmos-516 RLS','US-A RLS',27998,'Reentered','1972-09-25 00:00:00','1972-09-22 00:00:00',89.52,245,256,64.99,'Intermediate' from dual union all
select 6197,'006197','1972-073A','IMP 7','IMP H',28005,'Deep Space','1972-09-23 00:00:00','1972-09-25 00:00:00',17670.37,201100,235600,17.2,'Deep Space' from dual union all
select 6198,'006198','1972-070C','Kapsula Kosmosa-518','Nauka',28002,'Reentered','1972-09-27 00:00:00','1972-09-24 00:00:00',89.17,196,271,72.84,'Intermediate' from dual union all
select 6199,'006199','1972-066C','Kosmos-516 DU','US-A DU',27998,'Reentered','1972-10-20 00:00:00','1972-09-24 00:00:00',89.5,242,258,64.98,'Intermediate' from dual union all
select 62,'000062','60 Xi 2','Cluster 17 Assembly 4','RTV Motor',6334,'Reentered','1985-10-27 00:00:00','1960-12-13 00:00:00',112.65,416,2281,49.98,'Intermediate' from dual union all
select 620,'000620','1963-028C','Agena D 4702','Agena D',9259,'Reentered','1963-07-16 00:00:00','1963-07-14 00:00:00',88.08,175,184,95.37,'Sun-Synch' from dual union all
select 6200,'006200','1972-071D','KDU','KDU',28003,'Reentered','1972-10-14 00:00:00','1972-09-25 00:00:00',90.03,202,350,71.35,'Intermediate' from dual union all
select 6201,'006201','1972-071E','PTDU cover','KDU part',28003,'Reentered','1972-09-30 00:00:00','1972-09-27 00:00:00',91.57,215,489,71.35,'Intermediate' from dual union all
select 6202,'006202','1972-073B','Delta 90','Delta DSV-3N-4 S/N 20003',28005,'Reentered','1972-09-26 00:00:00','1972-09-23 00:00:00',90.15,153,411,28.8,'Intermediate' from dual union all
select 6203,'006203','1972-073C','Star 37C S/N 40018','Star 37C',28005,'Deep Space','1972-09-23 00:00:00','1972-09-23 00:00:00',7365.52,246,237796,28.6,'Deep Space' from dual union all
select 6204,'006204','1972-073D','Star 37C yo weight','',28005,'Deep Space','1972-09-23 00:00:00','1972-09-23 00:00:00',7365.52,246,237796,28.6,'Deep Space' from dual union all
select 6205,'006205','1972-073E','deb Delta 90','',28005,'Reentered','1972-09-26 00:00:00','1972-09-23 00:00:00',90.15,153,411,28.8,'Intermediate' from dual;
insert into SATELLITE
select 6206,'006206','1972-074A','Kosmos-521','DS-P1-M No. 4',28006,'In Earth orbit','','1972-10-29 00:00:00',104.94,976,1008,65.83,'Intermediate' from dual union all
select 6207,'006207','1972-074B','S3M','S3M',28006,'In Earth orbit','','1972-11-03 00:00:00',104.84,975,999,65.83,'Intermediate' from dual union all
select 6208,'006208','1972-075A','Molniya-2','Molniya-2',28007,'Reentered','1978-01-12 00:00:00','1972-11-01 00:00:00',717.44,473,39865,65.39,'GEO Transfer' from dual union all
select 6209,'006209','1972-075B','11S510','11S510',28007,'Reentered','1972-10-26 00:00:00','1972-10-07 00:00:00',90.98,211,435,65.41,'Intermediate' from dual union all
select 621,'000621','1963-029A','OPS 1266','CORONA M-23',9260,'Reentered Attached','1963-08-13 00:00:00','1963-07-26 00:00:00',90.05,188,365,82.87,'Intermediate' from dual union all
select 6210,'006210','1972-074C','deb Kosmos-521','DS-P1-M part [RCS 0.62]',28006,'In Earth orbit','','1972-10-30 00:00:00',104.99,978,1010,65.81,'Intermediate' from dual union all
select 6211,'006211','1972-075C','BOZ','BOZ',28007,'Reentered','1972-11-01 00:00:00','1972-10-11 00:00:00',91.07,223,432,65.38,'Intermediate' from dual union all
select 6212,'006212','1972-076A','Radcat','Radcat',28008,'Reentered','2012-08-02 00:00:00','1972-11-02 00:00:00',99.56,729,746,98.43,'Sun-Synch' from dual union all
select 6213,'006213','1966-087E','Altair yo weight','[UNID059] RCS 0.01',13494,'In Earth orbit','','1975-12-08 00:00:00',114.24,1376,1466,101.66,'Sun-Synch' from dual union all
select 6214,'006214','1966-016E','deb ESSA 2','',13423,'In Earth orbit','','1972-10-11 00:00:00',112.77,1343,1365,100.9,'Sun-Synch' from dual union all
select 6215,'006215','61 Sigma 5','Midas 3 debris','',7278,'Reentered','1976-10-30 00:00:00','1972-10-03 00:00:00',161.62,3356,3544,91.2,'Medium' from dual union all
select 6216,'006216','1970-025NE','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',102.37,644,1098,98.72,'Sun-Synch' from dual union all
select 6217,'006217','1972-076B','P72-1','Radsat',28008,'In Earth orbit','','1972-11-02 00:00:00',99.58,728,749,98.47,'Sun-Synch' from dual union all
select 6218,'006218','1972-076C','Burner II No. 23','Star 37B S/N 023',28008,'In Earth orbit','','1972-11-01 00:00:00',99.59,730,748,98.44,'Sun-Synch' from dual union all
select 6219,'006219','1972-077A','Kosmos-522','Zenit-4M',28009,'Landed','1972-10-17 00:00:00','1972-10-07 00:00:00',89.68,204,314,72.84,'Intermediate' from dual union all
select 622,'000622','1963-030A','Midas 9','Midas ETS-III F4',9261,'In Earth orbit','','1964-09-24 00:00:00',167.84,3670,3732,88.42,'Medium' from dual union all
select 6220,'006220','1972-077B','11A57I','11A57I',28009,'Reentered','1972-10-14 00:00:00','1972-10-09 00:00:00',88.87,182,256,72.84,'Intermediate' from dual union all
select 6221,'006221','1972-076D','P72-1 part','',28008,'Reentered','2002-02-07 00:00:00','1972-11-01 00:00:00',99.62,730,751,98.44,'Sun-Synch' from dual union all
select 6222,'006222','1972-078A','Kosmos-523','DS-P1-Yu No. 63',28010,'Reentered','1973-03-07 00:00:00','1972-11-04 00:00:00',91.68,264,450,71.01,'Intermediate' from dual union all
select 6223,'006223','1972-078B','S1M','S1M',28010,'Reentered','1972-12-07 00:00:00','1972-11-04 00:00:00',91.06,270,384,70.95,'Intermediate' from dual union all
select 6224,'006224','1972-076E','P72-1 part','',28008,'Reentered','1980-12-10 00:00:00','1972-11-03 00:00:00',99.54,725,748,98.44,'Sun-Synch' from dual union all
select 6225,'006225','1972-078C','deb S1M','',28010,'Reentered','1972-11-17 00:00:00','1972-10-12 00:00:00',92.96,354,485,70.57,'Intermediate' from dual union all
select 6226,'006226','1972-078D','deb S1M','',28010,'Reentered','1972-11-08 00:00:00','1972-10-12 00:00:00',93.62,338,566,71.2,'Intermediate' from dual union all
select 6227,'006227','1972-079A','OPS 8314','HEXAGON SV-4',28011,'Deorbited','1973-01-08 00:00:00','1972-11-09 00:00:00',88.79,160,270,96.43,'Sun-Synch' from dual union all
select 6228,'006228','1972-079B','Titan 23D-3 Stage 2','LR91AJ11 23D-3',28011,'Reentered','1972-10-12 00:00:00','1972-10-11 00:00:00',88.51,148,253,96.47,'Sun-Synch' from dual union all
select 6229,'006229','1972-080A','Kosmos-524','DS-P1-Yu No. 49',28012,'Reentered','1973-03-25 00:00:00','1972-11-11 00:00:00',91.89,259,476,70.99,'Intermediate' from dual union all
select 623,'000623','1963-029B','deb USAF satellite/Agena D','',9260,'Reentered','1963-07-29 00:00:00','1963-07-21 00:00:00',89.92,199,342,82.8,'Intermediate' from dual union all
select 6230,'006230','1972-080B','S1M','S1M',28012,'Reentered','1972-12-20 00:00:00','1972-11-10 00:00:00',91.2,249,417,71.01,'Intermediate' from dual union all
select 6231,'006231','1972-081A','Molniya-1','Molniya-1 11F67 No. 26',28013,'Reentered','1977-11-01 00:00:00','1972-11-16 00:00:00',717.76,638,39716,65.31,'GEO Transfer' from dual union all
select 6232,'006232','1972-081B','11S510','11S510',28013,'Reentered','1972-11-04 00:00:00','1972-10-21 00:00:00',90.78,204,422,65.44,'Intermediate' from dual union all
select 6233,'006233','1972-081C','BOZ','BOZ',28013,'Reentered','1972-11-16 00:00:00','1972-10-21 00:00:00',91.32,217,462,65.4,'Intermediate' from dual union all
select 6234,'006234','1972-081D','deb Blok-I','',28013,'Reentered','1972-11-14 00:00:00','1972-10-20 00:00:00',91.54,234,467,65.44,'Intermediate' from dual union all
select 6235,'006235','1972-082A','NOAA 2','ITOS D',28014,'In Earth orbit','','1972-11-15 00:00:00',114.94,1450,1456,101.77,'Sun-Synch' from dual union all
select 6236,'006236','1972-082B','AMSAT-OSCAR-6','OSCAR 6',28014,'In Earth orbit','','1972-11-15 00:00:00',114.93,1448,1457,101.75,'Sun-Synch' from dual union all
select 6237,'006237','1972-082C','Delta 91','Delta DSV-3N-4',28014,'In Earth orbit','','1972-12-11 00:00:00',109.3,917,1472,102.79,'Sun-Synch' from dual union all
select 6238,'006238','1972-077C','KDU','KDU',28009,'Reentered','1972-10-24 00:00:00','1972-10-16 00:00:00',89.98,179,367,72.82,'Intermediate' from dual union all
select 6239,'006239','1972-078E','deb S1M','',28010,'Reentered','1972-11-17 00:00:00','1972-10-15 00:00:00',93.71,369,543,70.12,'Intermediate' from dual union all
select 624,'000624','1963-030D','DASH 2','DASH 2',9261,'Reentered','1971-04-12 00:00:00','1963-09-05 00:00:00',167.91,3677,3731,88.41,'Medium' from dual union all
select 6240,'006240','1972-078F','deb S1M','',28010,'Reentered','1972-11-22 00:00:00','1972-10-17 00:00:00',94.53,354,638,70.62,'Intermediate' from dual union all
select 6241,'006241','1972-078G','deb S1M','',28010,'Reentered','1972-11-12 00:00:00','1972-10-14 00:00:00',94.12,350,602,70.62,'Intermediate' from dual union all
select 6242,'006242','1972-078H','deb S1M','',28010,'Reentered','1972-10-25 00:00:00','1972-10-17 00:00:00',90.06,197,358,71.21,'Intermediate' from dual union all
select 6243,'006243','1972-078J','deb S1M','',28010,'Reentered','1972-10-19 00:00:00','1972-10-15 00:00:00',89.33,172,311,71.17,'Intermediate' from dual union all
select 6244,'006244','1972-078K','deb S1M','',28010,'Reentered','1972-11-12 00:00:00','1972-10-15 00:00:00',95.16,373,680,70.9,'Intermediate' from dual union all
select 6245,'006245','1972-078L','deb S1M','',28010,'Reentered','1972-10-22 00:00:00','1972-10-14 00:00:00',90.04,194,359,71.11,'Intermediate' from dual union all
select 6246,'006246','1972-078M','deb S1M','',28010,'Reentered','1972-10-23 00:00:00','1972-10-15 00:00:00',90.53,242,359,71,'Intermediate' from dual union all
select 6247,'006247','1972-077D','PTDU cover','KDU part',28009,'Reentered','1972-10-20 00:00:00','1972-10-18 00:00:00',90.87,185,450,72.84,'Intermediate' from dual union all
select 6248,'006248','1972-083A','Kosmos-525','Zenit-2M',28015,'Landed','1972-10-29 00:00:00','1972-10-20 00:00:00',89.21,203,268,65.4,'Intermediate' from dual union all
select 6249,'006249','1972-083B','11A57I','11A57I',28015,'Reentered','1972-10-22 00:00:00','1972-10-20 00:00:00',88.81,187,245,65.39,'Intermediate' from dual union all
select 625,'000625','1963-031B','Altair','Altair X-248-A5DM',9262,'Reentered','1989-07-09 00:00:00','1963-11-19 00:00:00',627.8,256,35562,33.15,'GEO Transfer' from dual union all
select 6250,'006250','1972-069C','Triad OI-1X despin weight','',28001,'Reentered','2005-03-31 00:00:00','1972-10-22 00:00:00',100.24,709,831,89.74,'Polar' from dual union all
select 6251,'006251','62 A Alpha 5','deb Tiros V','',8136,'Reentered','2008-01-10 00:00:00','1972-10-21 00:00:00',99.69,581,906,58.1,'Intermediate' from dual union all
select 6252,'006252','1965-020FD','deb Blok Kosmos','',11770,'In Earth orbit','','1992-02-02 00:00:00',117.11,1403,1699,56.02,'Intermediate' from dual union all
select 6253,'006253','1965-082UG','deb Transtage 4','',11832,'Reentered','1983-04-15 00:00:00','1972-10-21 00:00:00',102.16,721,1001,32.39,'Intermediate' from dual union all
select 6254,'006254','1972-084A','Kosmos-526','DS-P1-Yu No. 61',28016,'Reentered','1973-04-08 00:00:00','1972-11-24 00:00:00',91.79,264,460,70.95,'Intermediate' from dual union all
select 6255,'006255','1972-084B','S1M','S1M',28016,'Reentered','1973-01-08 00:00:00','1972-11-24 00:00:00',91.2,258,409,70.96,'Intermediate' from dual union all
select 6256,'006256','1972-085A','Meteor','Meteor-M  No. 25',28017,'In Earth orbit','','1972-11-25 00:00:00',102.5,864,890,81.25,'Intermediate' from dual union all
select 6257,'006257','1972-085B','8A92ME','8A92ME',28017,'In Earth orbit','','1972-12-12 00:00:00',102.6,838,926,81.26,'Intermediate' from dual union all
select 6258,'006258','1972-083C','Kapsula Kosmosa-525','Nauka 16KS No. 1L',28015,'Reentered','1972-11-01 00:00:00','1972-10-27 00:00:00',88.98,191,257,65.37,'Intermediate' from dual union all
select 6259,'006259','1972-083D','Nauka part','',28015,'Reentered','1972-10-29 00:00:00','1972-10-27 00:00:00',88.62,189,224,65.4,'Intermediate' from dual union all
select 626,'000626','1963-032A','OPS 1370','LANYARD 3',9263,'Reentered Attached','1963-08-12 00:00:00','1963-07-31 00:00:00',90.87,168,467,74.9,'Intermediate' from dual union all
select 6260,'006260','1972-086A','Kosmos-527','Zenit-4MK',28018,'Landed','1972-11-13 00:00:00','1972-11-03 00:00:00',89.03,178,276,65.41,'Intermediate' from dual union all
select 6261,'006261','1972-086B','11A57I','11A57I',28018,'Reentered','1972-11-08 00:00:00','1972-11-02 00:00:00',89.3,190,289,65.4,'Intermediate' from dual union all
select 6262,'006262','1972-087A','Kosmos-528','Strela-1M',28019,'In Earth orbit','','1972-12-11 00:00:00',114.16,1367,1468,74.03,'Intermediate' from dual union all
select 6263,'006263','1972-086C','Sensor cover','Zenit-4MK sensor cover',28018,'Reentered','1972-11-03 00:00:00','1972-10-31 00:00:00',88.95,202,243,65.48,'Intermediate' from dual union all
select 6264,'006264','1972-087B','Kosmos-529','Strela-1M',28019,'In Earth orbit','','1972-12-10 00:00:00',114.56,1403,1467,74.04,'Intermediate' from dual union all
select 6265,'006265','1972-087C','Kosmos-530','Strela-1M',28019,'In Earth orbit','','1972-12-06 00:00:00',113.8,1334,1468,74.04,'Intermediate' from dual union all
select 6266,'006266','1972-087D','Kosmos-531','Strela-1M',28019,'In Earth orbit','','1972-12-07 00:00:00',114.78,1422,1468,74.04,'Intermediate' from dual union all
select 6267,'006267','1972-087E','Kosmos-532','Strela-1M',28019,'In Earth orbit','','1972-12-11 00:00:00',113.44,1301,1467,74.04,'Intermediate' from dual union all
select 6268,'006268','1972-087F','Kosmos-533','Strela-1M',28019,'In Earth orbit','','1972-12-11 00:00:00',113.62,1317,1469,74.04,'Intermediate' from dual union all
select 6269,'006269','1972-087G','Kosmos-534','Strela-1M',28019,'In Earth orbit','','1972-12-11 00:00:00',113.97,1350,1468,74.04,'Intermediate' from dual union all
select 627,'000627','1963-032B','LANYARD camera door','',9263,'Reentered','1963-08-10 00:00:00','1963-08-02 00:00:00',90.43,173,418,74.94,'Intermediate' from dual union all
select 6270,'006270','1972-087H','Kosmos-535','Strela-1M',28019,'In Earth orbit','','1972-12-05 00:00:00',114.36,1384,1469,74.04,'Intermediate' from dual union all
select 6271,'006271','1972-087J','S3M','S3M',28019,'In Earth orbit','','1972-12-10 00:00:00',116.65,1467,1594,74.04,'Intermediate' from dual union all
select 6272,'006272','1972-088A','Kosmos-536','Tselina-OM',28020,'Reentered','1980-07-20 00:00:00','1972-12-10 00:00:00',95.2,515,542,74.02,'Intermediate' from dual union all
select 6273,'006273','1972-088B','S3M','S3M',28020,'Reentered','1980-07-19 00:00:00','1972-12-08 00:00:00',95.11,500,547,74.03,'Intermediate' from dual union all
select 6274,'006274','1972-088C','deb Kosmos-536','Tselina-O part',28020,'Reentered','1973-12-19 00:00:00','1972-12-05 00:00:00',94.81,484,535,74.03,'Intermediate' from dual union all
select 6275,'006275','1972-089A','DMSP 6530','DMSP Block 5B F-3',28021,'In Earth orbit','','1972-12-09 00:00:00',101.72,810,871,98.64,'Sun-Synch' from dual union all
select 6276,'006276','1972-089B','Burner IIA No. 16 Stage 2','Star 26B S/N 003',28021,'In Earth orbit','','1972-12-10 00:00:00',101.75,812,870,98.63,'Sun-Synch' from dual union all
select 6277,'006277','1972-062C','deb Kosmos-514  [*]','',27994,'In Earth orbit','','1972-11-09 00:00:00',104.18,952,960,82.84,'Intermediate' from dual union all
select 6278,'006278','1972-090A','Anik A1','Anik A1',28022,'In Earth orbit','','1977-01-28 00:00:00',1436.09,35775,35798,.28,'Stationary' from dual union all
select 6279,'006279','1972-090B','Delta 92','Delta SSPS DSV-3P-5',28022,'Reentered','1973-05-07 00:00:00','1972-12-11 00:00:00',102.35,193,1547,28.56,'Intermediate' from dual union all
select 628,'000628','1963-014F','Midas solar array cover','Strongback  [RCS 1.0]',9245,'In Earth orbit','','1993-03-28 00:00:00',166.77,3588,3729,87.31,'Medium' from dual union all
select 6280,'006280','1972-086D','Blenda','Sunshade',28018,'Reentered','1972-11-15 00:00:00','1972-11-13 00:00:00',89.13,172,291,65.41,'Intermediate' from dual union all
select 6281,'006281','1972-086E','KDU','KDU',28018,'Reentered','1972-11-18 00:00:00','1972-11-14 00:00:00',89.33,177,306,65.42,'Intermediate' from dual union all
select 6282,'006282','1972-091A','SAS 2','SAS B',28023,'Reentered','1976-08-20 00:00:00','1972-11-15 00:00:00',95.4,444,631,1.9,'Equatorial' from dual union all
select 6283,'006283','1963-049M','deb Transit VBN-2','[UNID060] RCS 0.06',9280,'Reentered','1979-06-20 00:00:00','1972-11-15 00:00:00',104.79,823,1146,89.76,'Polar' from dual union all
select 6284,'006284','1972-069D','deb FW4S','',28001,'In Earth orbit','','2012-06-13 00:00:00',98.76,663,735,90.5,'Polar' from dual union all
select 6285,'006285','1972-092A','ESRO 4','ESRO 4',28024,'Reentered','1974-04-15 00:00:00','1973-01-03 00:00:00',98.39,240,1124,91.12,'Polar' from dual union all
select 6286,'006286','1972-092B','FW4S S/N 2223-13','FW4S',28024,'Reentered','1973-07-09 00:00:00','1972-12-25 00:00:00',97.96,239,1084,91.11,'Polar' from dual union all
select 6287,'006287','1972-093A','Kosmos-537','Zenit-2M',28025,'Landed','1972-12-07 00:00:00','1972-11-27 00:00:00',89.54,202,302,64.95,'Intermediate' from dual union all
select 6288,'006288','1972-093B','11A57I','11A57I',28025,'Reentered','1972-12-02 00:00:00','1972-11-27 00:00:00',89.17,190,277,64.96,'Intermediate' from dual union all
select 6289,'006289','1969-024C','deb Kosmos-272','',18291,'In Earth orbit','','1972-11-25 00:00:00',109.18,1183,1195,74.02,'Intermediate' from dual union all
select 629,'000629','1963-014G','West Ford telemetry section','-           [RCS 1.2]',9245,'In Earth orbit','','1991-04-08 00:00:00',166.38,3509,3776,87.35,'Medium' from dual union all
select 6290,'006290','1965-065S','deb NNS O-5','',11815,'Reentered','1981-06-02 00:00:00','1972-11-27 00:00:00',106.86,1029,1133,89.99,'Polar' from dual union all
select 6291,'006291','1972-094A','Interkosmos-8','DS-U1-IK No. 2',28026,'Reentered','1973-03-02 00:00:00','1973-01-02 00:00:00',92.16,197,564,71,'Intermediate' from dual union all
select 6292,'006292','1972-094B','S1M','S1M',28026,'Reentered','1973-01-12 00:00:00','1972-12-07 00:00:00',92.7,202,612,71.01,'Intermediate' from dual union all
select 6293,'006293','1972-094C','deb Interkosmos-8','',28026,'Reentered','1973-01-02 00:00:00','1972-12-08 00:00:00',92.16,157,604,71.08,'Intermediate' from dual union all
select 6294,'006294','1972-095A','Molniya-1','Molniya-1 11F67 No. 28',28027,'Reentered','1976-02-11 00:00:00','1973-01-08 00:00:00',717.72,473,39879,64.97,'GEO Transfer' from dual union all
select 6295,'006295','1972-095B','11S510','11S510',28027,'Reentered','1973-01-10 00:00:00','1972-12-13 00:00:00',90.94,226,416,64.99,'Intermediate' from dual union all
select 6296,'006296','1972-095C','BOZ','BOZ',28027,'Reentered','1973-02-02 00:00:00','1973-01-05 00:00:00',90.95,219,424,64.96,'Intermediate' from dual;
insert into SATELLITE
select 6297,'006297','1972-095D','deb Blok-I','',28027,'Reentered','1972-12-12 00:00:00','1972-12-04 00:00:00',91.12,225,434,64.96,'Intermediate' from dual union all
select 6298,'006298','1972-095E','deb Blok-I','',28027,'Reentered','1972-12-11 00:00:00','1972-12-05 00:00:00',91.1,210,446,64.97,'Intermediate' from dual union all
select 6299,'006299','1971-015CH','deb Kosmos-397','',24833,'Reentered','1990-10-01 00:00:00','1990-09-30 00:00:00',90.3,213,366,66.12,'Intermediate' from dual union all
select 63,'000063','60 Pi 1','Tiros 2','NASA A-2',6329,'Reentered','2014-05-18 00:00:00','1960-12-23 00:00:00',98.26,608,743,48.57,'Intermediate' from dual union all
select 630,'000630','1963-030C','Midas solar array cover','Strongback',9261,'In Earth orbit','','1963-08-21 00:00:00',167.42,3654,3715,88.37,'Medium' from dual union all
select 6300,'006300','1972-096A','America','Apollo CM-114',28028,'Deep Space','1972-12-07 00:00:00','1972-12-07 00:00:00',18915.79,201,457377,28.47,'Cislunar' from dual union all
select 6301,'006301','1972-096B','Saturn S-IVB-512','Saturn S-IVB-512',28028,'Deep Space','1972-12-07 00:00:00','1972-12-07 00:00:00',16940.51,68,424168,28.42,'Cislunar' from dual union all
select 6302,'006302','1972-072E','Blok-L','Blok-L',28004,'In Earth orbit','','1972-12-14 00:00:00',706.72,726,39082,62.95,'Molniya' from dual union all
select 6303,'006303','1972-075D','Blok-L','Blok-L',28007,'Reentered','1978-02-21 00:00:00','1973-01-02 00:00:00',700.5,569,38930,65.47,'GEO Transfer' from dual union all
select 6304,'006304','1972-081E','Blok-L','Blok-L',28013,'Reentered','1975-03-16 00:00:00','1972-11-21 00:00:00',701.46,275,39272,65.24,'GEO Transfer' from dual union all
select 6305,'006305','1972-097A','Nimbus 5','Nimbus E',28029,'In Earth orbit','','1973-01-10 00:00:00',107.22,1091,1105,99.95,'Sun-Synch' from dual union all
select 6306,'006306','1972-097B','Delta 93','Delta DSV-3N-4',28029,'In Earth orbit','','1973-01-10 00:00:00',111.81,1101,1519,99.81,'Sun-Synch' from dual union all
select 6307,'006307','1972-096C','Challenger','LM 12 AS',28028,'Deep Space Attached','1972-12-07 00:00:00','1972-12-07 00:00:00',18915.79,201,457377,28.47,'Cislunar' from dual union all
select 6308,'006308','1972-098A','Molniya-2','Molniya-2',28030,'Reentered','1975-01-22 00:00:00','1973-01-25 00:00:00',717.7,473,39878,65.44,'GEO Transfer' from dual union all
select 6309,'006309','1972-098B','BOZ','BOZ',28030,'Reentered','1973-01-13 00:00:00','1972-12-20 00:00:00',90.89,212,425,65.41,'Intermediate' from dual union all
select 631,'000631','1963-030E','Midas solar array cover','Strongback',9261,'In Earth orbit','','1963-08-20 00:00:00',168.2,3671,3760,88.52,'Medium' from dual union all
select 6310,'006310','1972-098C','11S510','11S510',28030,'Reentered','1973-01-19 00:00:00','1972-12-20 00:00:00',91.15,214,448,65.38,'Intermediate' from dual union all
select 6311,'006311','1972-099A','Kosmos-538','Zenit-4M',28031,'Landed','1972-12-27 00:00:00','1972-12-16 00:00:00',89.14,183,281,65.44,'Intermediate' from dual union all
select 6312,'006312','1972-099B','11A57I','11A57I',28031,'Reentered','1972-12-22 00:00:00','1972-12-16 00:00:00',89.08,192,266,65.4,'Intermediate' from dual union all
select 6313,'006313','1970-114G','deb Molniya-1','[UNID061]',21591,'Reentered','1973-02-07 00:00:00','1972-12-13 00:00:00',128.78,113,4026,65.16,'Medium' from dual union all
select 6314,'006314','1972-099C','Sensor cover','Zenit-4M sensor cover',28031,'Reentered','1972-12-18 00:00:00','1972-12-15 00:00:00',89.75,203,322,65.43,'Intermediate' from dual union all
select 6315,'006315','1972-100A','Aeros 1','GRS-B',28032,'Reentered','1973-08-22 00:00:00','1973-01-15 00:00:00',95.09,216,830,96.92,'Sun-Synch' from dual union all
select 6316,'006316','1972-100B','FW4S S/N 2376-2','FW4S',28032,'Reentered','1973-02-25 00:00:00','1973-01-16 00:00:00',94.01,214,728,96.93,'Sun-Synch' from dual union all
select 6317,'006317','1972-101A','OPS 9390','CANYON 5',28033,'In Earth orbit','','1972-12-31 00:00:00',1440.35,31012,40727,9.7,'Near-synch' from dual union all
select 6318,'006318','1972-101B','Agena D 2805','Agena D',28033,'In Earth orbit','','1972-12-31 00:00:00',1440.35,31012,40727,9.7,'Near-synch' from dual union all
select 6319,'006319','1972-102A','Kosmos-539','Sfera',28034,'In Earth orbit','','1973-01-22 00:00:00',112.93,1342,1380,74.01,'Intermediate' from dual union all
select 632,'000632','1963-033A','Kosmos-19','DS-P1 No. 3',9264,'Reentered','1964-03-30 00:00:00','1963-09-06 00:00:00',92.1,277,477,48.95,'Intermediate' from dual union all
select 6320,'006320','1972-102B','S3M','S3M',28034,'In Earth orbit','','1973-01-20 00:00:00',112.79,1338,1372,74.01,'Intermediate' from dual union all
select 6321,'006321','1972-103A','OPS 3978','GAMBIT-3 Mission 4337',28035,'Reentered Attached','1973-01-23 00:00:00','1972-12-28 00:00:00',89.75,135,389,110.43,'Retrograde' from dual union all
select 6322,'006322','1972-095F','Blok-L','Blok-L',28027,'Reentered','1975-02-25 00:00:00','1973-02-06 00:00:00',699.13,484,38947,65.01,'GEO Transfer' from dual union all
select 6323,'006323','1972-104A','Kosmos-540','Strela-2M',28036,'In Earth orbit','','1973-01-28 00:00:00',100.73,779,808,74.09,'Intermediate' from dual union all
select 6324,'006324','1972-104B','S3M','S3M',28036,'In Earth orbit','','1973-01-27 00:00:00',100.52,769,797,74.08,'Intermediate' from dual union all
select 6325,'006325','1972-099D','KDU','KDU part',28031,'Reentered','1973-01-04 00:00:00','1972-12-26 00:00:00',89.52,179,322,65.43,'Intermediate' from dual union all
select 6326,'006326','1972-105A','Kosmos-541','Zenit-4MT',28037,'Landed','1973-01-08 00:00:00','1972-12-29 00:00:00',90.13,216,346,81.33,'Intermediate' from dual union all
select 6327,'006327','1972-105B','11S510','11S510',28037,'Reentered','1973-01-12 00:00:00','1973-01-04 00:00:00',89.54,202,302,81.31,'Intermediate' from dual union all
select 6328,'006328','1972-106A','Kosmos-542','Tselina-D',28038,'Reentered','1983-10-09 00:00:00','1973-01-27 00:00:00',96.31,524,640,81.22,'Intermediate' from dual union all
select 6329,'006329','1972-106B','8A92ME','8A92ME',28038,'Reentered','1988-06-23 00:00:00','1973-01-28 00:00:00',96.33,507,658,81.23,'Intermediate' from dual union all
select 633,'000633','1963-033B','S1','S1',9264,'Reentered','1963-12-09 00:00:00','1963-09-06 00:00:00',91.9,274,462,48.95,'Intermediate' from dual union all
select 6330,'006330','1970-085C','deb Blok E','',21562,'Reentered','1990-03-10 00:00:00','1972-12-30 00:00:00',97.46,547,727,81.21,'Intermediate' from dual union all
select 6331,'006331','1972-099E','Blenda','BZA',28031,'Reentered','1973-01-04 00:00:00','1972-12-30 00:00:00',89.09,176,282,65.4,'Intermediate' from dual union all
select 6332,'006332','1972-064B','M-40','M-40',27996,'Reentered','1981-04-28 00:00:00','1973-01-02 00:00:00',157.86,234,6360,31.02,'Medium' from dual union all
select 6333,'006333','1973-001A','Luna-21','E-8 No. 204',30603,'Deep Space','1973-01-09 00:00:00','1973-01-08 00:00:00',15562.18,200,400000,51.56,'Cislunar' from dual union all
select 6334,'006334','1973-001B','SOZ','SOZ',30603,'Reentered','1973-01-12 00:00:00','1973-01-10 00:00:00',88.38,183,206,51.54,'Intermediate' from dual union all
select 6335,'006335','1973-001C','SOZ','SOZ',30603,'Reentered','1973-01-13 00:00:00','1973-01-10 00:00:00',88.42,178,215,51.52,'Intermediate' from dual union all
select 6336,'006336','1972-105C','Blenda','',28037,'Reentered','1973-01-11 00:00:00','1973-01-09 00:00:00',89.72,215,307,81.32,'Intermediate' from dual union all
select 6337,'006337','1972-105D','KBE part','',28037,'Reentered','1973-01-09 00:00:00','1973-01-09 00:00:00',89.42,211,280,81.16,'Intermediate' from dual union all
select 6338,'006338','1972-105E','KBE part','',28037,'Reentered','1973-01-12 00:00:00','1973-01-09 00:00:00',89.79,237,292,81.29,'Intermediate' from dual union all
select 6339,'006339','1973-002A','Kosmos-543','Zenit-4M',30604,'Landed','1973-01-25 00:00:00','1973-01-13 00:00:00',89.26,174,302,64.97,'Intermediate' from dual union all
select 634,'000634','1963-031A','Syncom 2','Syncom 2',9262,'In Earth orbit','','1963-12-10 00:00:00',1436.19,35784,35793,32.88,'Synchronous' from dual union all
select 6340,'006340','1973-002B','11A57I','11A57I',30604,'Reentered','1973-01-22 00:00:00','1973-01-14 00:00:00',89.29,200,279,64.97,'Intermediate' from dual union all
select 6341,'006341','1972-098D','Blok-L','Blok-L',28030,'Reentered','1974-10-07 00:00:00','1973-02-02 00:00:00',701.99,444,39129,65.4,'GEO Transfer' from dual union all
select 6342,'006342','1972-105F','KB electropitaniya','KBE',28037,'Reentered','1973-01-15 00:00:00','1973-01-14 00:00:00',89.91,219,321,81.34,'Intermediate' from dual union all
select 6343,'006343','1973-003A','Kosmos-544','Tselina-OM',30605,'Reentered','1980-06-15 00:00:00','1973-02-20 00:00:00',95.16,509,544,74.01,'Intermediate' from dual union all
select 6344,'006344','1973-003B','S3M','S3M',30605,'Reentered','1980-05-18 00:00:00','1973-02-22 00:00:00',95.02,499,541,74.04,'Intermediate' from dual union all
select 6345,'006345','1973-003C','deb Kosmos-544','Tselina-O part',30605,'Reentered','1975-10-14 00:00:00','1973-02-19 00:00:00',94.89,483,544,74.01,'Intermediate' from dual union all
select 6346,'006346','1973-002C','KDU','KDU',30604,'Reentered','1973-01-28 00:00:00','1973-01-23 00:00:00',89.04,166,289,64.98,'Intermediate' from dual union all
select 6347,'006347','1973-002D','Blenda','BZA',30604,'Reentered','1973-01-25 00:00:00','1973-01-23 00:00:00',88.76,165,261,64.97,'Intermediate' from dual union all
select 6348,'006348','1973-004A','Kosmos-545','DS-P1-Yu No. 62',30606,'Reentered','1973-07-31 00:00:00','1973-02-26 00:00:00',91.88,256,478,70.97,'Intermediate' from dual union all
select 6349,'006349','1973-004B','S1M','S1M',30606,'Reentered','1973-04-21 00:00:00','1973-02-25 00:00:00',91.39,262,424,71,'Intermediate' from dual union all
select 635,'000635','1963-030B','TRS 9','ERS 9',9261,'In Earth orbit','','1963-08-11 00:00:00',167.77,3663,3734,88.41,'Medium' from dual union all
select 6350,'006350','1973-005A','Kosmos-546','Tsiklon GVM',30607,'In Earth orbit','','1973-02-28 00:00:00',96.57,575,614,50.66,'Intermediate' from dual union all
select 6351,'006351','1973-005B','S3M','S3M',30607,'Reentered','1988-03-02 00:00:00','1973-02-26 00:00:00',96.47,566,614,50.66,'Intermediate' from dual union all
select 6352,'006352','1973-003D','deb Kosmos-544','Tselina-O part',30605,'Reentered','1975-03-21 00:00:00','1973-02-21 00:00:00',94.89,486,541,74.09,'Intermediate' from dual union all
select 6353,'006353','1973-006A','Kosmos-547','Zenit-2M',30608,'Landed','1973-02-13 00:00:00','1973-02-03 00:00:00',89.6,204,306,64.95,'Intermediate' from dual union all
select 6354,'006354','1973-006B','11A57I','11A57I',30608,'Reentered','1973-02-09 00:00:00','1973-02-03 00:00:00',89.29,203,276,64.97,'Intermediate' from dual union all
select 6355,'006355','1973-006C','Sensor cover','Zenit-2M sensor cover',30608,'Reentered','1973-02-02 00:00:00','1973-02-01 00:00:00',89.12,203,259,64.97,'Intermediate' from dual union all
select 6356,'006356','1973-007A','Molniya-1','Molniya-1 11F67 No. 31',30609,'Reentered','1977-10-23 00:00:00','1973-03-07 00:00:00',717.66,474,39875,65.03,'GEO Transfer' from dual union all
select 6357,'006357','1973-007B','11S510','11S510',30609,'Reentered','1973-03-13 00:00:00','1973-02-10 00:00:00',91.04,225,427,64.97,'Intermediate' from dual union all
select 6358,'006358','1973-007C','BOZ','BOZ',30609,'Reentered','1973-03-18 00:00:00','1973-02-10 00:00:00',91.34,226,455,64.95,'Intermediate' from dual union all
select 6359,'006359','1973-008A','Kosmos-548','Zenit-4M',30610,'Landed','1973-02-21 00:00:00','1973-02-10 00:00:00',89.52,204,298,65.38,'Intermediate' from dual union all
select 636,'000636','1963-034A','OPS 1419','CORONA J-1',9265,'Reentered Attached','1963-09-12 00:00:00','1963-08-25 00:00:00',90.72,184,436,75,'Intermediate' from dual union all
select 6360,'006360','1973-008B','11A57I','11A57I',30610,'Reentered','1973-02-16 00:00:00','1973-02-11 00:00:00',89.1,187,273,65.37,'Intermediate' from dual union all
select 6361,'006361','1973-008C','Sensor cover','Zenit-4M sensor cover',30610,'Reentered','1973-02-09 00:00:00','1973-02-08 00:00:00',89.07,204,253,65.38,'Intermediate' from dual union all
select 6362,'006362','1967-108F','deb Kosmos-189/536/544/549 ','[UNID062] Tselina-O part',15363,'Reentered','1974-02-27 00:00:00','1973-02-10 00:00:00',94.63,481,521,73.96,'Intermediate' from dual union all
select 6363,'006363','1973-007D','deb Blok-I','',30609,'Reentered','1973-03-10 00:00:00','1973-02-11 00:00:00',91.43,238,452,64.98,'Intermediate' from dual union all
select 6364,'006364','1973-009A','Prognoz-3','SO-M No. 502',30611,'Deep Space','1973-02-15 00:00:00','1976-07-06 00:00:00',5768.58,894,199442,65,'Deep Space' from dual union all
select 6365,'006365','1973-009B','11S510','11S510',30611,'Reentered','1973-03-22 00:00:00','1973-02-22 00:00:00',90.99,225,422,65,'Intermediate' from dual union all
select 6366,'006366','1973-009C','BOZ','BOZ',30611,'Reentered','1973-03-23 00:00:00','1973-02-22 00:00:00',91.26,213,460,65.01,'Intermediate' from dual union all
select 6367,'006367','1973-008D','KDU','KDU',30610,'Reentered','1973-02-25 00:00:00','1973-02-20 00:00:00',89.32,178,303,65.37,'Intermediate' from dual union all
select 6368,'006368','1973-007E','Blok-L','Blok-L',30609,'Reentered','1978-08-04 00:00:00','1973-03-11 00:00:00',702.49,527,39071,65.11,'GEO Transfer' from dual union all
select 6369,'006369','1973-008E','PTDU cover','KDU part',30610,'Reentered','1973-02-23 00:00:00','1973-02-21 00:00:00',90.89,190,447,65.38,'Intermediate' from dual union all
select 637,'000637','1963-035A','OPS 1561','ARGON 11',9266,'Reentered Attached','1963-11-07 00:00:00','1963-09-30 00:00:00',90.26,277,298,81.85,'Intermediate' from dual union all
select 6370,'006370','1973-008F','PTDU cover','KDU part',30610,'Reentered','1973-02-26 00:00:00','1973-02-21 00:00:00',90.66,197,417,65.32,'Intermediate' from dual union all
select 6371,'006371','1973-008G','Blenda','BZA',30610,'Reentered','1973-02-23 00:00:00','1973-02-22 00:00:00',89.75,194,331,65.38,'Intermediate' from dual union all
select 6372,'006372','1970-067E','NNS O-19 insulation','',21544,'Reentered','1981-04-13 00:00:00','1973-02-23 00:00:00',106.95,954,1217,90.06,'Polar' from dual union all
select 6373,'006373','1973-010A','Kosmos-549','Tselina-OM',30612,'Reentered','1980-06-29 00:00:00','1973-03-30 00:00:00',95.17,510,544,74.01,'Intermediate' from dual union all
select 6374,'006374','1973-010B','S3M','S3M',30612,'Reentered','1980-09-17 00:00:00','1973-03-30 00:00:00',95.08,495,550,74.05,'Intermediate' from dual union all
select 6375,'006375','1973-010C','deb Kosmos-549','Tselina-O part',30612,'Reentered','1974-06-24 00:00:00','1973-04-13 00:00:00',94.9,486,542,74.01,'Intermediate' from dual union all
select 6376,'006376','1973-011A','Kosmos-550','Zenit-4MK',30613,'Landed','1973-03-11 00:00:00','1973-03-04 00:00:00',89.35,184,301,65.41,'Intermediate' from dual union all
select 6377,'006377','1973-011B','11A57I','11A57I',30613,'Reentered','1973-03-09 00:00:00','1973-03-03 00:00:00',89.48,197,301,65.41,'Intermediate' from dual union all
select 6378,'006378','1973-012A','Kosmos-551','Zenit-4M',30614,'Landed','1973-03-20 00:00:00','1973-03-08 00:00:00',89.15,173,292,65.01,'Intermediate' from dual union all
select 6379,'006379','1973-012B','11A57I','11A57I',30614,'Reentered','1973-03-13 00:00:00','1973-03-10 00:00:00',88.78,190,239,65.01,'Intermediate' from dual union all
select 638,'000638','1963-035B','0.1 Square Meter Target','',9266,'Reentered','1963-09-29 00:00:00','1963-09-05 00:00:00',91.89,312,423,81.86,'Intermediate' from dual union all
select 6380,'006380','1973-013A','OPS 6063','RHYOLITE 2',30615,'In Earth orbit','','1973-04-01 00:00:00',1435.09,35679,35855,.2,'Drift GEO' from dual union all
select 6381,'006381','1973-012C','Sensor cover','Zenit-4M sensor cover',30614,'Reentered','1973-03-07 00:00:00','1973-03-06 00:00:00',88.88,194,245,65.01,'Intermediate' from dual union all
select 6382,'006382','1973-014A','OPS 8410','HEXAGON SV-5',30616,'Deorbited','1973-05-19 00:00:00','1973-04-08 00:00:00',88.79,165,265,95.67,'Sun-Synch' from dual union all
select 6383,'006383','1973-014B','Titan 23D-6 Stage 2','LR91AJ11 23D-6',30616,'Reentered','1973-03-11 00:00:00','1973-03-10 00:00:00',88.41,138,254,95.72,'Sun-Synch' from dual union all
select 6384,'006384','1973-011C','KDU','KDU',30613,'Reentered','1973-03-18 00:00:00','1973-03-10 00:00:00',89.21,183,288,65.43,'Intermediate' from dual union all
select 6385,'006385','1973-011D','Blenda','Sunshade',30613,'Reentered','1973-03-12 00:00:00','1973-03-10 00:00:00',88.98,186,263,65.44,'Intermediate' from dual union all
select 6386,'006386','1973-011E','PTDU cover','KDU part',30613,'Reentered','1973-03-12 00:00:00','1973-03-11 00:00:00',90.84,201,431,65.44,'Intermediate' from dual;
insert into SATELLITE
select 6387,'006387','1973-011F','PTDU cover','KDU part',30613,'Reentered','1973-03-12 00:00:00','1973-03-11 00:00:00',88.84,172,263,65.41,'Intermediate' from dual union all
select 6388,'006388','1973-013B','Agena D 1552','Agena D',30615,'Reentered','1977-03-07 00:00:00','1973-04-12 00:00:00',587.87,220,33516,28.3,'Highly Ellip' from dual union all
select 6389,'006389','1973-012D','KDU','KDU',30614,'Reentered','1973-03-22 00:00:00','1973-03-16 00:00:00',89.87,169,367,65.06,'Intermediate' from dual union all
select 639,'000639','1963-035C','deb USAF satellite','',9266,'Reentered','1963-09-02 00:00:00','1963-08-31 00:00:00',90.7,295,323,81.9,'Intermediate' from dual union all
select 6390,'006390','1973-012E','Blenda','BZA',30614,'Reentered','1973-03-19 00:00:00','1973-03-16 00:00:00',89.68,159,359,64.97,'Intermediate' from dual union all
select 6391,'006391','1972-104C','deb Kosmos-540','Strela-2M debris',28036,'Reentered','2014-08-04 00:00:00','1973-01-26 00:00:00',100.86,787,812,74.12,'Intermediate' from dual union all
select 6392,'006392','1973-015A','Meteor','Meteor-M  No. 29',30617,'In Earth orbit','','1973-04-19 00:00:00',102.57,871,890,81.25,'Intermediate' from dual union all
select 6393,'006393','1973-015B','8A92ME','8A92ME',30617,'In Earth orbit','','1973-04-19 00:00:00',102.7,841,932,81.25,'Intermediate' from dual union all
select 6394,'006394','1973-016A','Kosmos-552','Zenit-2M',30618,'Landed','1973-04-03 00:00:00','1973-03-24 00:00:00',89.61,203,308,72.87,'Intermediate' from dual union all
select 6395,'006395','1973-016B','11A57I','11A57I',30618,'Reentered','1973-03-30 00:00:00','1973-03-24 00:00:00',89.42,198,294,72.85,'Intermediate' from dual union all
select 6396,'006396','1972-104D','deb Kosmos-540','Strela-2M debris',28036,'In Earth orbit','','1973-01-30 00:00:00',101.12,795,829,74.14,'Intermediate' from dual union all
select 6397,'006397','1973-016C','Kapsula Kosmosa-552','Nauka 16KS No. 2L',30618,'Reentered','1973-04-09 00:00:00','1973-03-30 00:00:00',89.25,192,283,72.85,'Intermediate' from dual union all
select 6398,'006398','1973-017A','Salyut-2','Almaz OPS 1',30619,'Reentered','1973-05-28 00:00:00','1973-04-10 00:00:00',89.83,256,277,51.57,'Intermediate' from dual union all
select 6399,'006399','1973-017B','8S812','8S812',30619,'Reentered','1973-04-06 00:00:00','1973-04-04 00:00:00',88.8,192,239,51.48,'Intermediate' from dual union all
select 64,'000064','60 Pi 2','Altair','Altair X-248-A5 S/N 106',6329,'Reentered','1981-09-23 00:00:00','1961-01-04 00:00:00',98.19,608,737,48.57,'Intermediate' from dual union all
select 640,'000640','1963-035D','deb USAF satellite','',9266,'Reentered','1963-09-01 00:00:00','1963-08-31 00:00:00',90.23,277,295,81.89,'Intermediate' from dual union all
select 6400,'006400','1973-017C','deb 8S812','',30619,'Reentered','1973-04-10 00:00:00','1973-04-06 00:00:00',89.25,176,299,51.65,'Intermediate' from dual union all
select 6401,'006401','1973-017D','deb 8S812','',30619,'Reentered','1973-04-09 00:00:00','1973-04-06 00:00:00',89.03,203,251,51.62,'Intermediate' from dual union all
select 6402,'006402','1973-017E','deb 8S812','',30619,'Reentered','1973-04-12 00:00:00','1973-04-06 00:00:00',89.41,189,302,51.51,'Intermediate' from dual union all
select 6403,'006403','1973-017F','deb 8S812','',30619,'Reentered','1973-04-24 00:00:00','1973-04-22 00:00:00',91.98,353,391,51.67,'Intermediate' from dual union all
select 6404,'006404','1973-017G','deb 8S812','',30619,'Reentered','1973-04-30 00:00:00','1973-04-04 00:00:00',92.66,233,577,51.56,'Intermediate' from dual union all
select 6405,'006405','1973-017H','deb 8S812','',30619,'Reentered','1973-04-06 00:00:00','1973-04-06 00:00:00',87.83,83,252,52.9,'Intermediate' from dual union all
select 6406,'006406','1973-017J','deb 8S812','',30619,'Reentered','1973-04-26 00:00:00','1973-04-04 00:00:00',92.67,232,579,51.56,'Intermediate' from dual union all
select 6407,'006407','1973-017K','deb 8S812','',30619,'Reentered','1973-04-22 00:00:00','1973-04-04 00:00:00',91.27,197,477,52,'Intermediate' from dual union all
select 6408,'006408','1973-017L','deb 8S812','',30619,'Reentered','1973-04-08 00:00:00','1973-04-04 00:00:00',91.27,205,469,52.01,'Intermediate' from dual union all
select 6409,'006409','1973-017M','deb 8S812','',30619,'Reentered','1973-04-15 00:00:00','1973-04-04 00:00:00',89.71,213,308,51.6,'Intermediate' from dual union all
select 641,'000641','1963-036A','OPS 1947','GAMBIT SV 952',9267,'Deorbited','1963-09-13 00:00:00','1963-09-08 00:00:00',89,174,277,94.37,'Polar' from dual union all
select 6410,'006410','1973-017N','deb 8S812','',30619,'Reentered','1973-04-06 00:00:00','1973-04-04 00:00:00',88.51,188,213,51.67,'Intermediate' from dual union all
select 6411,'006411','1973-017P','deb 8S812','',30619,'Reentered','1973-04-08 00:00:00','1973-04-04 00:00:00',89.02,215,238,51.59,'Intermediate' from dual union all
select 6412,'006412','1973-017Q','deb 8S812','',30619,'Reentered','1973-04-08 00:00:00','1973-04-04 00:00:00',89.37,219,268,51.67,'Intermediate' from dual union all
select 6413,'006413','1973-017R','deb 8S812','',30619,'Reentered','1973-04-08 00:00:00','1973-04-04 00:00:00',89.29,231,248,51,'Intermediate' from dual union all
select 6414,'006414','1973-017S','deb 8S812','',30619,'Reentered','1973-04-06 00:00:00','1973-04-05 00:00:00',88.48,179,220,51.62,'Intermediate' from dual union all
select 6415,'006415','1973-017T','deb 8S812','',30619,'Reentered','1973-04-12 00:00:00','1973-04-08 00:00:00',89.38,228,260,51.62,'Intermediate' from dual union all
select 6416,'006416','1973-017U','deb 8S812','',30619,'Reentered','1973-04-08 00:00:00','1973-04-05 00:00:00',89.4,219,270,51.6,'Intermediate' from dual union all
select 6417,'006417','1973-017V','deb 8S812','',30619,'Reentered','1973-04-09 00:00:00','1973-04-05 00:00:00',89.49,232,267,51.6,'Intermediate' from dual union all
select 6418,'006418','1973-018A','Molniya-2','Molniya-2',30620,'Reentered','1979-01-06 00:00:00','1973-05-23 00:00:00',717.7,476,39875,65.43,'GEO Transfer' from dual union all
select 6419,'006419','1973-018B','BOZ','BOZ',30620,'Reentered','1973-04-24 00:00:00','1973-04-14 00:00:00',90.56,185,419,65.41,'Intermediate' from dual union all
select 642,'000642','1963-034B','Fairing Adapter','Fairing',9265,'Reentered','1963-09-08 00:00:00','1963-09-06 00:00:00',88.77,148,280,74.99,'Intermediate' from dual union all
select 6420,'006420','1973-018C','11S510','11S510',30620,'Reentered','1973-04-30 00:00:00','1973-04-13 00:00:00',90.75,210,413,65.4,'Intermediate' from dual union all
select 6421,'006421','1973-019A','Pioneer 11','Pioneer G',30621,'Deep Space','1973-04-06 00:00:00','1973-04-06 00:00:00',50.55,229,22043,34.9,'Earth Escape' from dual union all
select 6422,'006422','1973-017W','deb 8S812','',30619,'Reentered','1973-04-09 00:00:00','1973-04-05 00:00:00',89.33,210,272,51.61,'Intermediate' from dual union all
select 6423,'006423','1973-017X','deb 8S812','',30619,'Reentered','1973-04-10 00:00:00','1973-04-05 00:00:00',89.74,210,314,51.54,'Intermediate' from dual union all
select 6424,'006424','1973-017Y','deb 8S812','',30619,'Reentered','1973-04-10 00:00:00','1973-04-05 00:00:00',91.24,214,457,51.55,'Intermediate' from dual union all
select 6425,'006425','1973-019B','Star 37E S/N 40017','Star 37E',30621,'Deep Space','1973-04-06 00:00:00','1973-04-06 00:00:00',50.55,229,22043,34.9,'Earth Escape' from dual union all
select 6426,'006426','1973-019C','Centaur D-1A AC-30','Centaur D-1A AC-30 (01A0',30621,'Reentered','1974-11-04 00:00:00','1973-05-25 00:00:00',2302.18,512,102240,35.42,'Very High' from dual union all
select 6427,'006427','1973-020A','Kosmos-553','DS-P1-Yu No. 55',30622,'Reentered','1973-11-11 00:00:00','1973-05-12 00:00:00',91.88,264,470,70.96,'Intermediate' from dual union all
select 6428,'006428','1973-020B','S1M','S1M',30622,'Reentered','1973-08-07 00:00:00','1973-05-13 00:00:00',91.41,259,429,70.96,'Intermediate' from dual union all
select 6429,'006429','1973-017Z','Salyut-2 solar panel','',30619,'Reentered','1973-05-13 00:00:00','1973-04-10 00:00:00',90.36,211,373,51.67,'Intermediate' from dual union all
select 643,'000643','61 Omicron 187','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-07 00:00:00',109.66,923,1499,66.36,'Intermediate' from dual union all
select 6430,'006430','1973-017AA','Salyut-2 solar panel','',30619,'Reentered','1973-05-12 00:00:00','1973-04-15 00:00:00',89.72,247,275,51.63,'Intermediate' from dual union all
select 6431,'006431','1973-017AB','Salyut-2 solar panel','',30619,'Reentered','1973-04-17 00:00:00','1973-04-16 00:00:00',89.64,249,265,51.61,'Intermediate' from dual union all
select 6432,'006432','1973-021A','Kosmos-554','Zenit-4MK',30623,'Exploded','1973-05-06 00:00:00','1973-04-26 00:00:00',89.98,169,378,72.85,'Intermediate' from dual union all
select 6433,'006433','1973-022A','Interkosmos-9 Kopernik-500','DS-U2-IK Kopernik-500',30624,'Reentered','1973-10-15 00:00:00','1973-05-23 00:00:00',100.2,193,1343,48.32,'Intermediate' from dual union all
select 6434,'006434','1973-021B','11A57I','11A57I',30623,'Reentered','1973-04-26 00:00:00','1973-04-22 00:00:00',88.94,192,253,72.85,'Intermediate' from dual union all
select 6435,'006435','1973-022B','S1M','S1M',30624,'Reentered','1973-10-17 00:00:00','1973-05-19 00:00:00',100.23,200,1339,48.41,'Intermediate' from dual union all
select 6436,'006436','1973-021C','Sensor cover','Zenit-4MK sensor cover',30623,'Reentered','1973-04-25 00:00:00','1973-04-21 00:00:00',89.58,201,306,72.85,'Intermediate' from dual union all
select 6437,'006437','1973-023A','Anik A2','Anik A2',30625,'In Earth orbit','','1977-01-25 00:00:00',1436.06,35780,35792,.03,'Stationary' from dual union all
select 6438,'006438','1973-023B','Delta 94','Delta SSPS DSV-3P-5',30625,'Reentered','1975-10-18 00:00:00','1973-05-20 00:00:00',115.45,214,2738,29.5,'Intermediate' from dual union all
select 6439,'006439','1973-018D','Blok-L','Blok-L',30620,'Reentered','1979-12-05 00:00:00','1973-09-15 00:00:00',699.05,556,38871,65.58,'GEO Transfer' from dual union all
select 644,'000644','61 Omicron 188','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-05 00:00:00',106.86,952,1211,67.09,'Intermediate' from dual union all
select 6440,'006440','1973-024A','Kosmos-555','Zenit-2M',30626,'Landed','1973-05-07 00:00:00','1973-04-28 00:00:00',88.88,212,227,81.33,'Intermediate' from dual union all
select 6441,'006441','1973-024B','11A57I','11A57I',30626,'Reentered','1973-04-28 00:00:00','1973-04-27 00:00:00',88.39,192,198,81.32,'Intermediate' from dual union all
select 6442,'006442','1973-021D','Blenda','Sunshade',30623,'Reentered','1973-04-30 00:00:00','1973-04-26 00:00:00',89.44,166,328,72.88,'Intermediate' from dual union all
select 6443,'006443','1973-021E','KDU','KDU',30623,'Reentered','1973-05-01 00:00:00','1973-04-26 00:00:00',89.75,168,357,72.85,'Intermediate' from dual union all
select 6444,'006444','1973-024C','Sensor cover','Zenit-2M sensor cover',30626,'Reentered','1973-05-06 00:00:00','1973-05-04 00:00:00',88.51,196,206,81.33,'Intermediate' from dual union all
select 6445,'006445','1973-024D','Kapsula Kosmosa-555','Nauka 14KS No. 2',30626,'Reentered','1973-05-09 00:00:00','1973-05-06 00:00:00',88.45,192,203,81.33,'Intermediate' from dual union all
select 6446,'006446','1973-025A','Kosmos-556','Zenit-4MK',30627,'Landed','1973-05-14 00:00:00','1973-05-08 00:00:00',89.21,211,260,81.32,'Intermediate' from dual union all
select 6447,'006447','1973-025B','11A57I','11A57I',30627,'Reentered','1973-05-08 00:00:00','1973-05-07 00:00:00',88.38,188,201,81.3,'Intermediate' from dual union all
select 6448,'006448','1973-021F','deb Kosmos-554','',30623,'Reentered','1973-06-24 00:00:00','1973-06-10 00:00:00',90.53,222,378,72.73,'Intermediate' from dual union all
select 6449,'006449','1973-021G','deb Kosmos-554','',30623,'Reentered','1973-05-21 00:00:00','1973-05-19 00:00:00',90.55,247,356,72.79,'Intermediate' from dual union all
select 645,'000645','61 Omicron 189','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-03 00:00:00',110.85,946,1586,66.59,'Intermediate' from dual union all
select 6450,'006450','1973-021H','deb Kosmos-554','',30623,'Reentered','1973-05-16 00:00:00','1973-05-09 00:00:00',90.75,245,378,72.71,'Intermediate' from dual union all
select 6451,'006451','1973-021J','deb Kosmos-554','',30623,'Reentered','1973-05-24 00:00:00','1973-05-14 00:00:00',90.47,254,342,72.83,'Intermediate' from dual union all
select 6452,'006452','1973-021K','deb Kosmos-554','',30623,'Reentered','1973-05-16 00:00:00','1973-05-13 00:00:00',90.05,257,297,73.23,'Intermediate' from dual union all
select 6453,'006453','1973-021L','deb Kosmos-554','',30623,'Reentered','1973-05-20 00:00:00','1973-05-17 00:00:00',89.77,167,360,72.6,'Intermediate' from dual union all
select 6454,'006454','1973-021M','deb Kosmos-554','',30623,'Reentered','1973-05-27 00:00:00','1973-05-17 00:00:00',90.3,234,345,72.65,'Intermediate' from dual union all
select 6455,'006455','1973-021N','deb Kosmos-554','',30623,'Reentered','1973-05-12 00:00:00','1973-05-08 00:00:00',89.71,203,318,72.86,'Intermediate' from dual union all
select 6456,'006456','1973-021P','deb Kosmos-554','',30623,'Reentered','1973-05-17 00:00:00','1973-05-09 00:00:00',89.94,241,302,72.82,'Intermediate' from dual union all
select 6457,'006457','1973-021Q','deb Kosmos-554','',30623,'Reentered','1973-05-16 00:00:00','1973-05-13 00:00:00',89.24,181,293,72.78,'Intermediate' from dual union all
select 6458,'006458','1973-021R','deb Kosmos-554','',30623,'Reentered','1973-06-09 00:00:00','1973-05-08 00:00:00',90.38,236,351,72.84,'Intermediate' from dual union all
select 6459,'006459','1973-021S','deb Kosmos-554','',30623,'Reentered','1973-05-16 00:00:00','1973-05-16 00:00:00',88.16,14,381,73.62,'Suborbital' from dual union all
select 646,'000646','61 Omicron 190','deb r/b Ablestar','',7273,'Reentered','1972-06-07 00:00:00','1963-09-07 00:00:00',99.69,545,943,67.15,'Intermediate' from dual union all
select 6460,'006460','1973-021T','deb Kosmos-554','',30623,'Reentered','1973-05-12 00:00:00','1973-05-07 00:00:00',88.7,135,286,72.85,'Intermediate' from dual union all
select 6461,'006461','1973-021U','deb Kosmos-554','',30623,'Reentered','1973-05-20 00:00:00','1973-05-16 00:00:00',89.66,224,292,72.65,'Intermediate' from dual union all
select 6462,'006462','1973-021V','deb Kosmos-554','',30623,'Reentered','1973-05-12 00:00:00','1973-05-09 00:00:00',89.73,200,323,72.57,'Intermediate' from dual union all
select 6463,'006463','1973-021W','deb Kosmos-554','',30623,'Reentered','1973-05-21 00:00:00','1973-05-13 00:00:00',90.11,229,331,72.83,'Intermediate' from dual union all
select 6464,'006464','1973-021X','deb Kosmos-554','',30623,'Reentered','1973-05-18 00:00:00','1973-06-05 00:00:00',90.12,238,323,72.84,'Intermediate' from dual union all
select 6465,'006465','1973-021Y','deb Kosmos-554','',30623,'Reentered','1973-05-27 00:00:00','1973-06-12 00:00:00',90.67,237,377,72.72,'Intermediate' from dual union all
select 6466,'006466','1973-021Z','deb Kosmos-554','',30623,'Reentered','1973-06-04 00:00:00','1973-05-29 00:00:00',89.6,214,296,73.41,'Intermediate' from dual union all
select 6467,'006467','1973-021AA','deb Kosmos-554','',30623,'Reentered','1973-05-28 00:00:00','1973-05-30 00:00:00',88.7,86,335,73.03,'Intermediate' from dual union all
select 6468,'006468','1973-021AB','deb Kosmos-554','',30623,'Reentered','1973-07-06 00:00:00','1973-07-07 00:00:00',92.44,261,528,73.03,'Intermediate' from dual union all
select 6469,'006469','1973-021AC','deb Kosmos-554','',30623,'Reentered','1974-01-29 00:00:00','1973-06-01 00:00:00',94.73,291,720,73.16,'Intermediate' from dual union all
select 647,'000647','61 Omicron 191','deb r/b Ablestar','',7273,'Reentered','2001-04-30 00:00:00','1993-08-05 00:00:00',97.66,597,697,66.7,'Intermediate' from dual union all
select 6470,'006470','1973-021AD','deb Kosmos-554','',30623,'Reentered','1973-07-03 00:00:00','1973-06-05 00:00:00',91.72,270,448,72.9,'Intermediate' from dual union all
select 6471,'006471','1973-021AE','deb Kosmos-554','',30623,'Reentered','1973-11-29 00:00:00','1973-06-01 00:00:00',92.28,294,479,72.75,'Intermediate' from dual union all
select 6472,'006472','1973-021AF','deb Kosmos-554','',30623,'Reentered','1973-05-22 00:00:00','1973-05-12 00:00:00',90.32,253,328,72.8,'Intermediate' from dual union all
select 6473,'006473','1973-021AG','deb Kosmos-554','',30623,'Reentered','1973-05-30 00:00:00','1973-05-19 00:00:00',90.23,262,309,73.04,'Intermediate' from dual union all
select 6474,'006474','1973-021AH','deb Kosmos-554','',30623,'Reentered','1973-05-17 00:00:00','1973-05-09 00:00:00',91.1,227,430,72.78,'Intermediate' from dual union all
select 6475,'006475','1973-021AJ','deb Kosmos-554','',30623,'Reentered','1973-06-01 00:00:00','1973-05-16 00:00:00',90.39,244,343,73.06,'Intermediate' from dual union all
select 6476,'006476','1973-021AK','deb Kosmos-554','',30623,'Reentered','1973-05-19 00:00:00','1973-05-17 00:00:00',91.27,26,648,72.58,'ERROR - Unexpected value "TA"' from dual;
insert into SATELLITE
select 6477,'006477','1973-021AL','deb Kosmos-554','',30623,'Reentered','1973-05-16 00:00:00','1973-05-13 00:00:00',89.48,166,332,72.59,'Intermediate' from dual union all
select 6478,'006478','1973-021AM','deb Kosmos-554','',30623,'Reentered','1973-05-13 00:00:00','1973-05-10 00:00:00',89.02,122,331,72.97,'Intermediate' from dual union all
select 6479,'006479','1973-021AN','deb Kosmos-554','',30623,'Reentered','1973-05-27 00:00:00','1973-05-21 00:00:00',89.95,172,372,72.82,'Intermediate' from dual union all
select 648,'000648','61 Omicron 192','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-07 00:00:00',106.72,908,1241,65.89,'Intermediate' from dual union all
select 6480,'006480','1973-021AP','deb Kosmos-554','',30623,'Reentered','1973-05-15 00:00:00','1973-05-10 00:00:00',89.19,159,310,72.88,'Intermediate' from dual union all
select 6481,'006481','1973-021AQ','deb Kosmos-554','',30623,'Reentered','1973-05-18 00:00:00','1973-05-14 00:00:00',88.76,130,297,74.6,'Intermediate' from dual union all
select 6482,'006482','1973-021AR','deb Kosmos-554','',30623,'Reentered','1973-05-19 00:00:00','1973-05-14 00:00:00',90.06,213,342,72.83,'Intermediate' from dual union all
select 6483,'006483','1973-021AS','deb Kosmos-554','',30623,'Reentered','1973-05-15 00:00:00','1973-05-10 00:00:00',89.06,181,275,72.84,'Intermediate' from dual union all
select 6484,'006484','1973-021AT','deb Kosmos-554','',30623,'Reentered','1973-06-08 00:00:00','1973-05-22 00:00:00',91.31,210,468,72.98,'Intermediate' from dual union all
select 6485,'006485','1973-021AU','deb Kosmos-554','',30623,'Reentered','1973-05-19 00:00:00','1973-05-10 00:00:00',91,273,375,72.73,'Intermediate' from dual union all
select 6486,'006486','1973-021AV','deb Kosmos-554','',30623,'Reentered','1973-05-19 00:00:00','1973-05-09 00:00:00',92.07,256,497,72.45,'Intermediate' from dual union all
select 6487,'006487','1973-021AW','deb Kosmos-554','',30623,'Reentered','1973-05-18 00:00:00','1973-05-10 00:00:00',89.56,159,347,72.8,'Intermediate' from dual union all
select 6488,'006488','1973-021AX','deb Kosmos-554','',30623,'Reentered','1973-05-21 00:00:00','1973-05-20 00:00:00',88.75,179,247,72.9,'Intermediate' from dual union all
select 6489,'006489','1973-021AY','deb Kosmos-554','',30623,'Reentered','1973-06-13 00:00:00','1973-05-10 00:00:00',90.82,249,380,72.84,'Intermediate' from dual union all
select 649,'000649','61 Omicron 193','deb r/b Ablestar','',7273,'In Earth orbit','','1993-07-31 00:00:00',114.1,973,1856,66.26,'Intermediate' from dual union all
select 6490,'006490','1973-021AZ','deb Kosmos-554','',30623,'Reentered','1973-08-14 00:00:00','1973-05-19 00:00:00',91.73,240,479,72.73,'Intermediate' from dual union all
select 6491,'006491','1973-021BA','deb Kosmos-554','',30623,'Reentered','1973-07-14 00:00:00','1973-06-05 00:00:00',90.49,220,377,72.83,'Intermediate' from dual union all
select 6492,'006492','1973-021BB','deb Kosmos-554','',30623,'Reentered','1973-05-30 00:00:00','1973-05-10 00:00:00',91.35,240,442,72.8,'Intermediate' from dual union all
select 6493,'006493','1973-021BC','deb Kosmos-554','',30623,'Reentered','1973-07-07 00:00:00','1973-06-04 00:00:00',91.51,273,425,72.83,'Intermediate' from dual union all
select 6494,'006494','1973-021BD','deb Kosmos-554','',30623,'Reentered','1973-05-27 00:00:00','1973-05-19 00:00:00',89.83,224,309,73.09,'Intermediate' from dual union all
select 6495,'006495','1973-021BE','deb Kosmos-554','',30623,'Reentered','1973-05-25 00:00:00','1973-05-18 00:00:00',90.14,224,339,73.01,'Intermediate' from dual union all
select 6496,'006496','1973-021BF','deb Kosmos-554','',30623,'Reentered','1973-06-09 00:00:00','1973-05-09 00:00:00',90.89,218,419,73.11,'Intermediate' from dual union all
select 6497,'006497','1973-021BG','deb Kosmos-554','',30623,'Reentered','1973-05-27 00:00:00','1973-05-19 00:00:00',89.88,122,414,72.75,'Intermediate' from dual union all
select 6498,'006498','1973-026A','Kosmos-557','DOS 3',30628,'Reentered','1973-05-22 00:00:00','1973-05-14 00:00:00',88.94,208,237,51.59,'Intermediate' from dual union all
select 6499,'006499','1973-026B','8S812','8S812',30628,'Reentered','1973-05-17 00:00:00','1973-05-13 00:00:00',88.74,205,220,51.6,'Intermediate' from dual union all
select 65,'000065','60 Rho 1','Korabl''-Sputnik-3','Vostok-1 No. 3',6330,'Deorbited','1960-12-02 00:00:00','1960-12-01 00:00:00',88.57,171,237,65,'Intermediate' from dual union all
select 650,'000650','61 Omicron 194','deb r/b Ablestar','',7273,'Reentered','1982-05-27 00:00:00','1963-09-07 00:00:00',99.96,542,972,66.84,'Intermediate' from dual union all
select 6500,'006500','1973-021BH','deb Kosmos-554','',30623,'Reentered','1973-05-27 00:00:00','1973-05-11 00:00:00',90.67,230,384,72.93,'Intermediate' from dual union all
select 6501,'006501','1973-021BJ','deb Kosmos-554','',30623,'Reentered','1973-06-25 00:00:00','1973-05-19 00:00:00',92.84,291,537,72.91,'Intermediate' from dual union all
select 6502,'006502','1973-021BK','deb Kosmos-554','',30623,'Reentered','1974-01-02 00:00:00','1973-05-24 00:00:00',92.14,284,475,72.83,'Intermediate' from dual union all
select 6503,'006503','1973-021BL','deb Kosmos-554','',30623,'Reentered','1973-07-01 00:00:00','1973-05-23 00:00:00',90.84,252,380,73.05,'Intermediate' from dual union all
select 6504,'006504','1973-021BM','deb Kosmos-554','',30623,'Reentered','1973-05-18 00:00:00','1973-05-11 00:00:00',91.46,200,492,72.86,'Intermediate' from dual union all
select 6505,'006505','1973-021BN','deb Kosmos-554','',30623,'Reentered','1973-05-21 00:00:00','1973-05-13 00:00:00',90.71,252,366,72.85,'Intermediate' from dual union all
select 6506,'006506','1973-021BP','deb Kosmos-554','',30623,'Reentered','1973-05-17 00:00:00','1973-05-06 00:00:00',89.53,193,309,72.8,'Intermediate' from dual union all
select 6507,'006507','1973-021BQ','deb Kosmos-554','',30623,'Reentered','1973-05-15 00:00:00','1973-05-12 00:00:00',89.09,174,285,72.81,'Intermediate' from dual union all
select 6508,'006508','1973-021BR','deb Kosmos-554','',30623,'Reentered','1973-05-19 00:00:00','1973-05-17 00:00:00',89.53,164,339,72.97,'Intermediate' from dual union all
select 6509,'006509','1973-021BS','deb Kosmos-554','',30623,'Reentered','1973-06-18 00:00:00','1973-05-19 00:00:00',91.53,259,441,73.09,'Intermediate' from dual union all
select 651,'000651','61 Omicron 195','deb r/b Ablestar','',7273,'Reentered','1991-01-05 00:00:00','1991-01-04 00:00:00',89.56,246,259,66.81,'Intermediate' from dual union all
select 6510,'006510','1973-021BT','deb Kosmos-554','',30623,'Reentered','1973-06-06 00:00:00','1973-05-12 00:00:00',90.91,201,438,72.78,'Intermediate' from dual union all
select 6511,'006511','1973-021BU','deb Kosmos-554','',30623,'Reentered','1973-08-12 00:00:00','1973-05-22 00:00:00',90.52,227,372,73.02,'Intermediate' from dual union all
select 6512,'006512','1973-021BV','deb Kosmos-554','',30623,'Reentered','1973-05-26 00:00:00','1973-05-15 00:00:00',89.87,211,324,72.69,'Intermediate' from dual union all
select 6513,'006513','1973-021BW','deb Kosmos-554','',30623,'Reentered','1973-05-26 00:00:00','1973-05-16 00:00:00',90.18,261,305,72.72,'Intermediate' from dual union all
select 6514,'006514','1973-021BX','deb Kosmos-554','',30623,'Reentered','1973-05-18 00:00:00','1973-05-06 00:00:00',90.49,181,417,72,'Intermediate' from dual union all
select 6515,'006515','1973-021BY','deb Kosmos-554','',30623,'Reentered','1973-05-28 00:00:00','1973-05-17 00:00:00',89.8,226,304,72.93,'Intermediate' from dual union all
select 6516,'006516','1973-021BZ','deb Kosmos-554','',30623,'Reentered','1973-06-14 00:00:00','1973-05-15 00:00:00',90.43,111,481,73.37,'Intermediate' from dual union all
select 6517,'006517','1973-021CA','deb Kosmos-554','',30623,'Reentered','1973-05-26 00:00:00','1973-05-14 00:00:00',89.04,206,249,72.59,'Intermediate' from dual union all
select 6518,'006518','1973-021CB','deb Kosmos-554','',30623,'Reentered','1973-05-23 00:00:00','1973-05-13 00:00:00',90.05,154,400,73.9,'Intermediate' from dual union all
select 6519,'006519','1973-021CC','deb Kosmos-554','',30623,'Reentered','1973-07-03 00:00:00','1973-06-05 00:00:00',90.24,216,356,72.76,'Intermediate' from dual union all
select 652,'000652','61 Omicron 196','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-04 00:00:00',103.58,858,998,66.7,'Intermediate' from dual union all
select 6520,'006520','1973-021CD','deb Kosmos-554','',30623,'Reentered','1973-05-25 00:00:00','1973-05-18 00:00:00',89.47,212,285,72.73,'Intermediate' from dual union all
select 6521,'006521','1973-021CE','deb Kosmos-554','',30623,'Reentered','1973-06-06 00:00:00','1973-05-16 00:00:00',90.37,272,314,72.7,'Intermediate' from dual union all
select 6522,'006522','1973-021CF','deb Kosmos-554','',30623,'Reentered','1973-05-25 00:00:00','1973-05-13 00:00:00',89.83,230,303,72.82,'Intermediate' from dual union all
select 6523,'006523','1973-021CG','deb Kosmos-554','',30623,'Reentered','1973-05-22 00:00:00','1973-05-15 00:00:00',89.8,207,323,73.28,'Intermediate' from dual union all
select 6524,'006524','1973-021CH','deb Kosmos-554','',30623,'Reentered','1973-06-09 00:00:00','1973-06-03 00:00:00',92.56,269,531,73,'Intermediate' from dual union all
select 6525,'006525','1973-021CJ','deb Kosmos-554','',30623,'Reentered','1973-09-16 00:00:00','1973-05-19 00:00:00',91.02,249,401,72.63,'Intermediate' from dual union all
select 6526,'006526','1973-021CK','deb Kosmos-554','',30623,'Reentered','1973-05-26 00:00:00','1973-05-12 00:00:00',90.34,259,324,72.78,'Intermediate' from dual union all
select 6527,'006527','1973-021CL','deb Kosmos-554','',30623,'Reentered','1973-05-26 00:00:00','1973-05-12 00:00:00',89.01,189,263,72.82,'Intermediate' from dual union all
select 6528,'006528','1973-021CM','deb Kosmos-554','',30623,'Reentered','1973-05-26 00:00:00','1973-05-12 00:00:00',88.82,201,232,72.88,'Intermediate' from dual union all
select 6529,'006529','1973-021CN','deb Kosmos-554','',30623,'Reentered','1973-05-25 00:00:00','1973-05-12 00:00:00',90.2,218,350,72.89,'Intermediate' from dual union all
select 653,'000653','61 Omicron 197','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-07 00:00:00',106.06,925,1163,67.46,'Intermediate' from dual union all
select 6530,'006530','1973-021CP','deb Kosmos-554','',30623,'Reentered','1973-06-02 00:00:00','1973-05-13 00:00:00',91.16,216,447,72.85,'Intermediate' from dual union all
select 6531,'006531','1973-021CQ','deb Kosmos-554','',30623,'Reentered','1973-06-25 00:00:00','1973-05-19 00:00:00',91.59,275,431,72.77,'Intermediate' from dual union all
select 6532,'006532','1973-021CR','deb Kosmos-554','',30623,'Reentered','1973-05-17 00:00:00','1973-05-13 00:00:00',91.38,188,497,72.8,'Intermediate' from dual union all
select 6533,'006533','1973-021CS','deb Kosmos-554','',30623,'Reentered','1973-05-22 00:00:00','1973-05-16 00:00:00',90.47,227,369,72.72,'Intermediate' from dual union all
select 6534,'006534','1973-021CT','deb Kosmos-554','',30623,'Reentered','1973-05-26 00:00:00','1973-05-13 00:00:00',88.21,169,203,72.86,'Intermediate' from dual union all
select 6535,'006535','1973-021CU','deb Kosmos-554','',30623,'Reentered','1973-05-22 00:00:00','1973-05-11 00:00:00',90.31,230,350,72.85,'Intermediate' from dual union all
select 6536,'006536','1973-021CV','deb Kosmos-554','',30623,'Reentered','1973-06-30 00:00:00','1973-05-19 00:00:00',92.37,274,508,72.81,'Intermediate' from dual union all
select 6537,'006537','1973-021CW','deb Kosmos-554','',30623,'Reentered','1973-05-22 00:00:00','1973-05-06 00:00:00',91.65,196,515,72.82,'Intermediate' from dual union all
select 6538,'006538','1973-021CX','deb Kosmos-554','',30623,'Reentered','1973-05-25 00:00:00','1973-05-06 00:00:00',91.07,225,430,72.88,'Intermediate' from dual union all
select 6539,'006539','1973-021CY','deb Kosmos-554','',30623,'Reentered','1973-05-24 00:00:00','1973-05-06 00:00:00',91.67,304,408,72.85,'Intermediate' from dual union all
select 654,'000654','61 Omicron 198','deb r/b Ablestar','',7273,'Reentered','1968-07-03 00:00:00','1963-09-07 00:00:00',100.38,607,947,67.35,'Intermediate' from dual union all
select 6540,'006540','1973-021CZ','deb Kosmos-554','',30623,'Reentered','1973-12-31 00:00:00','1973-05-19 00:00:00',93.49,281,610,73.03,'Intermediate' from dual union all
select 6541,'006541','1973-021DA','deb Kosmos-554','',30623,'Reentered','1973-05-27 00:00:00','1973-05-06 00:00:00',90.9,178,460,72.8,'Intermediate' from dual union all
select 6542,'006542','1973-021DB','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',87.97,117,231,73.2,'Intermediate' from dual union all
select 6543,'006543','1973-021DC','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.33,162,222,72.9,'Intermediate' from dual union all
select 6544,'006544','1973-021DD','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.33,150,333,72.3,'Intermediate' from dual union all
select 6545,'006545','1973-021DE','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.79,195,334,73.3,'Intermediate' from dual union all
select 6546,'006546','1973-021DF','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.81,185,346,73.7,'Intermediate' from dual union all
select 6547,'006547','1973-021DG','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.91,156,384,72.9,'Intermediate' from dual union all
select 6548,'006548','1973-021DH','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',87.83,110,225,73.9,'Intermediate' from dual union all
select 6549,'006549','1973-021DJ','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.06,120,237,72.2,'Intermediate' from dual union all
select 655,'000655','61 Omicron 199','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-02 00:00:00',110.58,961,1546,67.25,'Intermediate' from dual union all
select 6550,'006550','1973-021DK','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.21,196,274,72.9,'Intermediate' from dual union all
select 6551,'006551','1973-021DL','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.53,188,314,72.8,'Intermediate' from dual union all
select 6552,'006552','1973-021DM','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.88,156,283,73,'Intermediate' from dual union all
select 6553,'006553','1973-021DN','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.16,201,265,72.8,'Intermediate' from dual union all
select 6554,'006554','1973-021DP','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.13,134,329,72.2,'Intermediate' from dual union all
select 6555,'006555','1973-021DQ','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.29,192,287,73.5,'Intermediate' from dual union all
select 6556,'006556','1973-021DR','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.94,137,406,72.7,'Intermediate' from dual union all
select 6557,'006557','1973-021DS','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',87.95,116,230,73,'Intermediate' from dual union all
select 6558,'006558','1973-021DT','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.6,205,305,71.8,'Intermediate' from dual union all
select 6559,'006559','1973-021DU','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.01,113,339,72.5,'Intermediate' from dual union all
select 656,'000656','61 Omicron 200','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-03 00:00:00',114.81,885,2009,66.29,'Intermediate' from dual union all
select 6560,'006560','1973-021DV','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.79,138,390,72.5,'Intermediate' from dual union all
select 6561,'006561','1973-021DW','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',87.83,127,208,72.8,'Intermediate' from dual union all
select 6562,'006562','1973-021DX','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.6,204,306,72.7,'Intermediate' from dual union all
select 6563,'006563','1973-021DY','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.71,193,228,73.3,'Intermediate' from dual union all
select 6564,'006564','1973-021DZ','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.25,198,277,72.7,'Intermediate' from dual union all
select 6565,'006565','1973-021EA','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.73,204,319,72.9,'Intermediate' from dual union all
select 6566,'006566','1973-021EB','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.3,144,336,72.7,'Intermediate' from dual;
insert into SATELLITE
select 6567,'006567','1973-021EC','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.66,188,327,73.3,'Intermediate' from dual union all
select 6568,'006568','1973-021ED','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.02,183,270,72,'Intermediate' from dual union all
select 6569,'006569','1973-021EE','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.72,204,219,73.1,'Intermediate' from dual union all
select 657,'000657','61 Omicron 201','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-08 00:00:00',109.98,970,1482,66.21,'Intermediate' from dual union all
select 6570,'006570','1973-021EF','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.96,204,341,72.9,'Intermediate' from dual union all
select 6571,'006571','1973-021EG','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.34,142,342,72.4,'Intermediate' from dual union all
select 6572,'006572','1973-021EH','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.94,179,266,73,'Intermediate' from dual union all
select 6573,'006573','1973-021EJ','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.84,194,241,72.9,'Intermediate' from dual union all
select 6574,'006574','1973-021EK','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',87.8,125,207,72.2,'Intermediate' from dual union all
select 6575,'006575','1973-021EL','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.6,131,379,74,'Intermediate' from dual union all
select 6576,'006576','1973-021EM','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89,177,274,72.5,'Intermediate' from dual union all
select 6577,'006577','1973-021EN','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.76,161,365,72.3,'Intermediate' from dual union all
select 6578,'006578','1973-021EP','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.95,173,273,72.9,'Intermediate' from dual union all
select 6579,'006579','1973-021EQ','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.85,174,262,72.1,'Intermediate' from dual union all
select 658,'000658','61 Omicron 202','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-03 00:00:00',108.57,871,1451,65.88,'Intermediate' from dual union all
select 6580,'006580','1973-021ER','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.47,173,224,73.2,'Intermediate' from dual union all
select 6581,'006581','1973-021ES','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.5,150,251,71.7,'Intermediate' from dual union all
select 6582,'006582','1973-021ET','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.78,110,418,73.4,'Intermediate' from dual union all
select 6583,'006583','1973-021EU','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.09,121,338,72.7,'Intermediate' from dual union all
select 6584,'006584','1973-021EV','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.31,155,227,73.1,'Intermediate' from dual union all
select 6585,'006585','1973-021EW','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.83,193,340,72.8,'Intermediate' from dual union all
select 6586,'006586','1973-021EX','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.78,188,241,71.9,'Intermediate' from dual union all
select 6587,'006587','1973-021EY','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.3,115,365,72.9,'Intermediate' from dual union all
select 6588,'006588','1973-021EZ','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.59,200,309,73.5,'Intermediate' from dual union all
select 6589,'006589','1973-021FA','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.62,132,281,72.3,'Intermediate' from dual union all
select 659,'000659','61 Omicron 203','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-07 00:00:00',114.13,948,1884,67.16,'Intermediate' from dual union all
select 6590,'006590','1973-021FB','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.65,202,312,73.4,'Intermediate' from dual union all
select 6591,'006591','1973-021FC','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.29,173,207,72.5,'Intermediate' from dual union all
select 6592,'006592','1973-021FD','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.83,203,330,72.4,'Intermediate' from dual union all
select 6593,'006593','1973-021FE','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.18,176,291,72.3,'Intermediate' from dual union all
select 6594,'006594','1973-021FF','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.45,171,324,73.1,'Intermediate' from dual union all
select 6595,'006595','1973-021FG','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.77,192,335,72.9,'Intermediate' from dual union all
select 6596,'006596','1973-021FH','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.43,178,216,72.5,'Intermediate' from dual union all
select 6597,'006597','1973-021FJ','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.84,132,303,72.4,'Intermediate' from dual union all
select 6598,'006598','1973-021FK','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.95,205,339,73.6,'Intermediate' from dual union all
select 6599,'006599','1973-021FL','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.43,186,306,73.1,'Intermediate' from dual union all
select 66,'000066','60 Rho 2','8K72E','8K72E',6330,'Reentered','1960-12-02 00:00:00','1960-12-01 00:00:00',88.56,171,235,65,'Intermediate' from dual union all
select 660,'000660','61 Omicron 204','deb r/b Ablestar','',7273,'In Earth orbit','','1993-08-06 00:00:00',100.97,759,851,66.54,'Intermediate' from dual union all
select 6600,'006600','1973-021FM','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.25,119,257,72.4,'Intermediate' from dual union all
select 6601,'006601','1973-021FN','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.93,170,274,72.1,'Intermediate' from dual union all
select 6602,'006602','1973-021FP','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.44,190,205,71.9,'Intermediate' from dual union all
select 6603,'006603','1973-021FQ','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.72,203,318,73,'Intermediate' from dual union all
select 6604,'006604','1973-021FR','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.11,172,289,73.5,'Intermediate' from dual union all
select 6605,'006605','1973-021FS','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.96,123,323,72.7,'Intermediate' from dual union all
select 6606,'006606','1973-021FT','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.62,154,358,72.9,'Intermediate' from dual union all
select 6607,'006607','1973-021FU','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.98,122,424,73.2,'Intermediate' from dual union all
select 6608,'006608','1973-021FV','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.03,117,236,72.9,'Intermediate' from dual union all
select 6609,'006609','1973-021FW','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.77,150,278,72.6,'Intermediate' from dual union all
select 661,'000661','61 Omicron 205','deb r/b Ablestar','',7273,'Reentered','2002-02-13 00:00:00','1993-08-07 00:00:00',101.64,789,884,66.92,'Intermediate' from dual union all
select 6610,'006610','1973-021FX','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.97,203,343,73.6,'Intermediate' from dual union all
select 6611,'006611','1973-021FY','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.74,192,332,72.9,'Intermediate' from dual union all
select 6612,'006612','1973-021FZ','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.68,154,265,72.5,'Intermediate' from dual union all
select 6613,'006613','1973-021GA','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.68,130,289,72.8,'Intermediate' from dual union all
select 6614,'006614','1973-021GB','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.75,181,245,71.9,'Intermediate' from dual union all
select 6615,'006615','1973-021GC','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.68,204,215,72.5,'Intermediate' from dual union all
select 6616,'006616','1973-021GD','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.36,152,235,73.6,'Intermediate' from dual union all
select 6617,'006617','1973-021GE','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.8,114,316,72.4,'Intermediate' from dual union all
select 6618,'006618','1973-021GF','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.13,150,213,73.6,'Intermediate' from dual union all
select 6619,'006619','1973-021GG','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.59,132,278,72.6,'Intermediate' from dual union all
select 662,'000662','61 Omicron 206','deb r/b Ablestar','',7273,'Reentered','1991-12-27 00:00:00','1991-12-26 00:00:00',89.12,186,276,66.63,'Intermediate' from dual union all
select 6620,'006620','1973-021GH','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.33,167,216,73.6,'Intermediate' from dual union all
select 6621,'006621','1973-021GJ','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.36,136,349,72.9,'Intermediate' from dual union all
select 6622,'006622','1973-021GK','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.11,203,258,73,'Intermediate' from dual union all
select 6623,'006623','1973-021GL','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.64,204,310,73.4,'Intermediate' from dual union all
select 6624,'006624','1973-021GM','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.47,169,229,72.8,'Intermediate' from dual union all
select 6625,'006625','1973-021GN','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.09,204,254,72.7,'Intermediate' from dual union all
select 6626,'006626','1973-021GP','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.42,151,341,72.4,'Intermediate' from dual union all
select 6627,'006627','1973-021GQ','deb Kosmos-554','RCS 0.31',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.74,194,329,73.1,'Intermediate' from dual union all
select 6628,'006628','1973-021GR','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',89.78,191,337,72.5,'Intermediate' from dual union all
select 6629,'006629','1973-021GS','deb Kosmos-554','',30623,'Reentered','1973-05-07 00:00:00','1973-05-06 00:00:00',88.85,142,293,73.6,'Intermediate' from dual union all
select 663,'000663','1963-036B','Agena D 4701','Agena D',9267,'Reentered','1963-09-14 00:00:00','1963-09-09 00:00:00',88.71,150,272,94.42,'Polar' from dual union all
select 6630,'006630','1973-025C','KDU','KDU',30627,'Reentered','1973-05-24 00:00:00','1973-05-13 00:00:00',89.1,208,251,81.32,'Intermediate' from dual union all
select 6631,'006631','1973-025D','Blenda','Sunshade',30627,'Reentered','1973-05-15 00:00:00','1973-05-13 00:00:00',89.15,206,259,81.32,'Intermediate' from dual union all
select 6632,'006632','1973-025E','KDU part','KDU part',30627,'Reentered','1973-05-17 00:00:00','1973-05-14 00:00:00',89.02,206,247,81.3,'Intermediate' from dual union all
select 6633,'006633','1973-027A','Skylab Orbital Workshop','Skylab I OWS',30629,'Reentered','1979-07-11 00:00:00','1974-01-09 00:00:00',93.22,421,444,50.04,'Intermediate' from dual union all
select 6634,'006634','1973-027B','Saturn S-II-13','Saturn S-II-13',30629,'Reentered','1975-01-11 00:00:00','1974-01-01 00:00:00',92.14,348,411,50.05,'Intermediate' from dual union all
select 6635,'006635','1973-027C','OWS Radiator cover','',30629,'Reentered','1973-09-10 00:00:00','1973-06-13 00:00:00',92.98,411,430,50,'Intermediate' from dual union all
select 6636,'006636','1973-027D','Skylab SAS-2 wing','',30629,'Reentered','1973-06-25 00:00:00','1973-05-22 00:00:00',92.88,405,427,49.99,'Intermediate' from dual union all
select 6637,'006637','1973-027E','Skylab Fairing section','Skylab Fairing',30629,'Reentered','1975-08-05 00:00:00','1974-01-01 00:00:00',92.72,396,420,50.05,'Intermediate' from dual union all
select 6638,'006638','1973-027F','Skylab Fairing section','Skylab Fairing',30629,'Reentered','1975-04-03 00:00:00','1974-01-25 00:00:00',92.94,417,420,50.05,'Intermediate' from dual union all
select 6639,'006639','1973-027G','SAS debris','debris',30629,'Reentered','1975-04-03 00:00:00','1973-06-13 00:00:00',92.64,374,434,49.96,'Intermediate' from dual union all
select 664,'000664','1963-036C','deb USAF satellite/Agena D','',9267,'Reentered','1963-09-10 00:00:00','1963-09-09 00:00:00',88.02,160,193,94.34,'Polar' from dual union all
select 6640,'006640','1973-028A','OPS 2093','GAMBIT-3 Mission 4338',30630,'Reentered Attached','1973-06-13 00:00:00','1973-05-23 00:00:00',89.77,135,392,110.51,'Retrograde' from dual union all
select 6641,'006641','1973-027H','Skylab Fairing section','Skylab Fairing',30629,'Reentered','1976-11-01 00:00:00','1974-01-18 00:00:00',93.13,426,430,50.05,'Intermediate' from dual union all
select 6642,'006642','1973-027J','SAS debris','debris',30629,'Reentered','1974-01-19 00:00:00','1973-06-15 00:00:00',93.38,416,464,49.9,'Intermediate' from dual union all
select 6643,'006643','1973-027K','Skylab Fairing section','Skylab Fairing',30629,'Reentered','1975-02-10 00:00:00','1974-01-01 00:00:00',92.65,394,415,50.04,'Intermediate' from dual union all
select 6644,'006644','1973-027L','SAS debris','debris',30629,'Reentered','1973-11-01 00:00:00','1973-06-18 00:00:00',93.12,419,436,50.04,'Intermediate' from dual union all
select 6645,'006645','1973-029A','Kosmos-558','DS-P1-Yu No. 65',30631,'Reentered','1973-12-22 00:00:00','1973-06-17 00:00:00',91.95,264,477,70.98,'Intermediate' from dual union all
select 6646,'006646','1973-029B','S1M','S1M',30631,'Reentered','1973-09-10 00:00:00','1973-06-16 00:00:00',91.53,257,443,70.97,'Intermediate' from dual union all
select 6647,'006647','1973-030A','Kosmos-559','Zenit-4MK',30632,'Landed','1973-05-23 00:00:00','1973-05-20 00:00:00',89.74,203,321,65.41,'Intermediate' from dual union all
select 6648,'006648','1973-030B','11S510','11S510',30632,'Reentered','1973-05-28 00:00:00','1973-05-20 00:00:00',89.61,204,306,65.39,'Intermediate' from dual union all
select 6649,'006649','1973-030C','Blenda','Sunshade',30632,'Reentered','1973-05-29 00:00:00','1973-05-28 00:00:00',89.29,205,273,65.43,'Intermediate' from dual union all
select 665,'000665','1963-036D','deb USAF satellite/Agena D','',9267,'Reentered','1963-09-08 00:00:00','1963-09-08 00:00:00',88.09,160,200,94.34,'Polar' from dual union all
select 6650,'006650','1973-030D','KDU','KDU',30632,'Reentered','1973-06-12 00:00:00','1973-05-26 00:00:00',89.52,200,302,65.41,'Intermediate' from dual union all
select 6651,'006651','1973-027M','Skylab Fairing debris','debris',30629,'Reentered','1975-06-12 00:00:00','1974-01-01 00:00:00',93.04,422,424,50.03,'Intermediate' from dual union all
select 6652,'006652','1973-031A','Kosmos-560','Zenit-4M',30633,'Landed','1973-06-05 00:00:00','1973-05-25 00:00:00',89.36,179,307,72.84,'Intermediate' from dual union all
select 6653,'006653','1973-031B','11A57I','11A57I',30633,'Reentered','1973-06-01 00:00:00','1973-05-25 00:00:00',89.37,198,288,72.84,'Intermediate' from dual union all
select 6654,'006654','1973-031C','Sensor cover','Zenit-4M sensor cover',30633,'Reentered','1973-06-01 00:00:00','1973-05-24 00:00:00',89.61,202,309,72.81,'Intermediate' from dual union all
select 6655,'006655','1973-032A','Skylab SL-2','Apollo CM-116',30634,'Landed','1973-06-22 00:00:00','1973-06-04 00:00:00',93.22,425,440,50.04,'Intermediate' from dual union all
select 6656,'006656','1973-032B','Saturn S-IVB-206','Saturn S-IVB-206',30634,'Deorbited','1973-05-25 00:00:00','1973-05-25 00:00:00',89.57,155,351,50.04,'Intermediate' from dual;
insert into SATELLITE
select 6657,'006657','1973-033A','Kosmos-561','Zenit-2M',30635,'Landed','1973-06-06 00:00:00','1973-05-27 00:00:00',89.47,204,293,65.41,'Intermediate' from dual union all
select 6658,'006658','1973-033B','11A57I','11A57I',30635,'Reentered','1973-06-02 00:00:00','1973-05-27 00:00:00',89.22,197,275,65.4,'Intermediate' from dual union all
select 6659,'006659','1973-034A','Meteor','Meteor-M  No. 27',30636,'In Earth orbit','','1973-06-28 00:00:00',102.41,850,896,81.22,'Intermediate' from dual union all
select 666,'000666','1963-036E','deb USAF satellite/Agena D','',9267,'Reentered','1963-09-11 00:00:00','1963-09-08 00:00:00',88.09,160,200,94.34,'Polar' from dual union all
select 6660,'006660','1973-034B','8A92ME','8A92ME',30636,'In Earth orbit','','1973-08-06 00:00:00',102.64,849,918,81.23,'Intermediate' from dual union all
select 6661,'006661','1973-033C','deb Kosmos-561','',30635,'Reentered','1973-06-07 00:00:00','1973-06-03 00:00:00',89.23,195,278,65.41,'Intermediate' from dual union all
select 6662,'006662','1973-033D','Kapsula Kosmosa-561','Nauka 9KS No. 1',30635,'Reentered','1973-06-20 00:00:00','1973-06-03 00:00:00',89.36,205,281,65.39,'Intermediate' from dual union all
select 6663,'006663','1973-031D','KDU','KDU',30633,'Reentered','1973-06-12 00:00:00','1973-06-04 00:00:00',89.34,171,313,72.88,'Intermediate' from dual union all
select 6664,'006664','1973-031E','KDU part','KDU part',30633,'Reentered','1973-06-06 00:00:00','1973-06-04 00:00:00',89.14,171,293,72.95,'Intermediate' from dual union all
select 6665,'006665','1973-035A','Kosmos-562','DS-P1-Yu No. 66',30637,'Reentered','1974-01-07 00:00:00','1973-07-05 00:00:00',91.85,267,464,70.99,'Intermediate' from dual union all
select 6666,'006666','1973-035B','S1M','S1M',30637,'Reentered','1973-09-29 00:00:00','1973-07-08 00:00:00',91.48,263,432,70.98,'Intermediate' from dual union all
select 6667,'006667','1973-036A','Kosmos-563','Zenit-4M',30638,'Landed','1973-06-18 00:00:00','1973-06-08 00:00:00',89.15,176,289,65.4,'Intermediate' from dual union all
select 6668,'006668','1973-036B','11A57I','11A57I',30638,'Reentered','1973-06-13 00:00:00','1973-06-08 00:00:00',89.27,195,282,65.39,'Intermediate' from dual union all
select 6669,'006669','1973-021GT','deb Kosmos-554','',30623,'Reentered','1973-09-23 00:00:00','1973-06-07 00:00:00',91.69,281,434,72.72,'Intermediate' from dual union all
select 667,'000667','1963-036F','deb USAF satellite/Agena D','',9267,'Reentered','1963-09-10 00:00:00','1963-09-09 00:00:00',88.45,170,226,94.37,'Polar' from dual union all
select 6670,'006670','1973-021GU','deb Kosmos-554','',30623,'Reentered','1973-06-26 00:00:00','1973-06-08 00:00:00',90.58,232,374,72.84,'Intermediate' from dual union all
select 6671,'006671','1973-021GV','deb Kosmos-554','',30623,'Reentered','1973-08-23 00:00:00','1973-08-19 00:00:00',90.06,207,347,72.71,'Intermediate' from dual union all
select 6672,'006672','1973-021GW','deb Kosmos-554','',30623,'Reentered','1973-06-25 00:00:00','1973-06-24 00:00:00',90.63,218,393,72.83,'Intermediate' from dual union all
select 6673,'006673','1973-021GX','deb Kosmos-554','',30623,'Reentered','1973-06-19 00:00:00','1973-06-08 00:00:00',89.84,214,319,73.04,'Intermediate' from dual union all
select 6674,'006674','1973-036C','Sensor cover','Zenit-4M sensor cover',30638,'Reentered','1973-06-15 00:00:00','1973-06-06 00:00:00',89.19,196,272,65.4,'Intermediate' from dual union all
select 6675,'006675','1973-037A','Kosmos-564','Strela-1M',30639,'In Earth orbit','','1973-07-08 00:00:00',114.61,1395,1481,74.03,'Intermediate' from dual union all
select 6676,'006676','1973-037B','Kosmos-565','Strela-1M',30639,'In Earth orbit','','1973-07-08 00:00:00',115.31,1449,1489,74.02,'Intermediate' from dual union all
select 6677,'006677','1973-037C','Kosmos-566','Strela-1M',30639,'In Earth orbit','','1973-07-08 00:00:00',115.06,1434,1482,74.02,'Intermediate' from dual union all
select 6678,'006678','1973-037D','Kosmos-567','Strela-1M',30639,'In Earth orbit','','1973-07-08 00:00:00',114.84,1413,1484,74.02,'Intermediate' from dual union all
select 6679,'006679','1973-037E','Kosmos-568','Strela-1M',30639,'In Earth orbit','','1973-07-08 00:00:00',114.4,1375,1481,74,'Intermediate' from dual union all
select 668,'000668','1963-037A','OPS 1353','CORONA J-2',9268,'Reentered Attached','1963-10-12 00:00:00','1963-10-01 00:00:00',89.98,177,369,74.96,'Intermediate' from dual union all
select 6680,'006680','1973-037F','Kosmos-569','Strela-1M',30639,'In Earth orbit','','1973-07-08 00:00:00',114.19,1357,1480,74.01,'Intermediate' from dual union all
select 6681,'006681','1973-037G','Kosmos-570','Strela-1M',30639,'In Earth orbit','','1973-07-08 00:00:00',113.99,1338,1480,74.02,'Intermediate' from dual union all
select 6682,'006682','1973-037H','Kosmos-571','Strela-1M',30639,'In Earth orbit','','1973-07-08 00:00:00',113.77,1320,1478,74.02,'Intermediate' from dual union all
select 6683,'006683','1973-037J','S3M','S3M',30639,'In Earth orbit','','1973-07-08 00:00:00',116.89,1477,1605,74.02,'Intermediate' from dual union all
select 6684,'006684','1973-038A','Kosmos-572','Zenit-4M',30640,'Landed','1973-06-23 00:00:00','1973-06-12 00:00:00',89.03,174,280,51.76,'Intermediate' from dual union all
select 6685,'006685','1973-038B','11A57I','11A57I',30640,'Reentered','1973-06-16 00:00:00','1973-06-13 00:00:00',88.75,181,245,51.64,'Intermediate' from dual union all
select 6686,'006686','1973-039A','RAE 2','RAE B',30641,'Deep Space','1973-06-11 00:00:00','1973-06-10 00:00:00',15012.97,182,390244,29.1,'Cislunar' from dual union all
select 6687,'006687','1973-039B','Star 37D S/N 024','Star 37D',30641,'Deep Space','1973-06-11 00:00:00','1973-06-10 00:00:00',15012.97,182,390244,29.1,'Cislunar' from dual union all
select 6688,'006688','1973-039C','Delta 95','Delta SSPS DSV-3P-5',30641,'Reentered','1974-01-29 00:00:00','1973-07-10 00:00:00',106.51,183,1947,29.33,'Intermediate' from dual union all
select 6689,'006689','1973-039D','deb Star 37D yo weight','',30641,'Deep Space','1973-06-11 00:00:00','1973-06-10 00:00:00',15012.97,182,390244,29.1,'Cislunar' from dual union all
select 669,'000669','1963-038A','Ablestar 013','AJ10-104D 013',9269,'In Earth orbit','','1963-11-02 00:00:00',107,1061,1114,89.9,'Polar' from dual union all
select 6690,'006690','1973-039E','Delta 95 clamp band half','',30641,'Reentered','1973-09-19 00:00:00','1973-07-10 00:00:00',105.15,186,1817,29.29,'Intermediate' from dual union all
select 6691,'006691','1973-040A','DSP F4','DSP 2',30642,'In Earth orbit','','1979-10-30 00:00:00',1436.11,35728,35845,4.99,'Inclined GEO' from dual union all
select 6692,'006692','1973-021GY','deb Kosmos-554','',30623,'Reentered','1973-06-15 00:00:00','1973-06-05 00:00:00',90.12,238,323,72.84,'Intermediate' from dual union all
select 6693,'006693','1973-021GZ','deb Kosmos-554','',30623,'Reentered','1973-06-24 00:00:00','1973-06-11 00:00:00',90.81,242,387,72.77,'Intermediate' from dual union all
select 6694,'006694','1973-041A','Kosmos-573','Soyuz 7K-T No. 36',30643,'Landed','1973-06-17 00:00:00','1973-06-16 00:00:00',89.5,192,308,51.57,'Intermediate' from dual union all
select 6695,'006695','1973-041B','11S510','11S510',30643,'Reentered','1973-06-21 00:00:00','1973-06-17 00:00:00',89.08,183,275,51.6,'Intermediate' from dual union all
select 6696,'006696','1967-108G','deb Kosmos-189','[UNID063] Tselina-O part',15363,'Reentered','1974-02-27 00:00:00','1973-01-31 00:00:00',94.73,486,525,73.99,'Intermediate' from dual union all
select 6697,'006697','1967-108H','deb Kosmos-189','Tselina-O part',15363,'Reentered','1977-03-29 00:00:00','1973-03-28 00:00:00',94.78,475,541,74.05,'Intermediate' from dual union all
select 6698,'006698','1967-108J','deb Kosmos-189','Tselina-O part',15363,'Reentered','1974-05-13 00:00:00','1973-02-27 00:00:00',94.71,471,539,74.07,'Intermediate' from dual union all
select 6699,'006699','1967-108K','deb Kosmos-189','Tselina-O part',15363,'Reentered','1974-03-29 00:00:00','1973-06-16 00:00:00',94.34,463,510,73.97,'Intermediate' from dual union all
select 67,'000067','60 Sigma','Discoverer 18','CORONA C''-3',6331,'Reentered Attached','1961-04-02 00:00:00','1961-01-11 00:00:00',92.64,272,536,81.48,'Intermediate' from dual union all
select 670,'000670','1963-038B','Transit VBN-1','Transit VBN-1',9269,'In Earth orbit','','1964-01-03 00:00:00',107.28,1071,1130,89.9,'Polar' from dual union all
select 6700,'006700','1972-088D','deb Kosmos-536','Tselina-O part',28020,'Reentered','1983-02-28 00:00:00','1973-06-27 00:00:00',95.32,519,550,74.06,'Intermediate' from dual union all
select 6701,'006701','1967-108L','deb Kosmos-189','[UNID064] Tselina-O part',15363,'Reentered','1974-02-10 00:00:00','1973-01-29 00:00:00',94.68,483,524,74.07,'Intermediate' from dual union all
select 6702,'006702','1973-041C','Kosmos-573 BO','Bitovoy otsek',30643,'Reentered','1973-06-21 00:00:00','1973-06-17 00:00:00',89.01,186,265,51.61,'Intermediate' from dual union all
select 6703,'006703','1973-036D','Blenda','BZA',30638,'Reentered','1973-06-25 00:00:00','1973-06-17 00:00:00',88.98,172,276,65.4,'Intermediate' from dual union all
select 6704,'006704','1973-027N','EVA debris','EVA debris',30629,'Reentered','1974-04-13 00:00:00','1974-01-04 00:00:00',93.17,418,442,50.04,'Intermediate' from dual union all
select 6705,'006705','1973-027P','SAS debris','EVA debris',30629,'Reentered','1973-09-24 00:00:00','1973-06-17 00:00:00',93.12,408,447,50.06,'Intermediate' from dual union all
select 6706,'006706','1973-036E','KDU','KDU',30638,'Reentered','1973-06-23 00:00:00','1973-06-18 00:00:00',89.06,169,286,65.41,'Intermediate' from dual union all
select 6707,'006707','1973-042A','Kosmos-574','Zaliv',30644,'In Earth orbit','','1973-07-22 00:00:00',105.06,982,1013,82.94,'Intermediate' from dual union all
select 6708,'006708','1973-042B','S3M','S3M',30644,'In Earth orbit','','1973-07-20 00:00:00',104.95,978,1007,82.95,'Intermediate' from dual union all
select 6709,'006709','1973-043A','Kosmos-575','Zenit-2M',30645,'Landed','1973-07-03 00:00:00','1973-06-23 00:00:00',89.22,203,269,65.42,'Intermediate' from dual union all
select 671,'000671','1963-038C','Transit VE-1','APL SN-39',9269,'In Earth orbit','','1964-01-05 00:00:00',107.26,1071,1129,89.9,'Polar' from dual union all
select 6710,'006710','1973-043B','11A57I','11A57I',30645,'Reentered','1973-06-27 00:00:00','1973-06-23 00:00:00',88.95,190,256,65.39,'Intermediate' from dual union all
select 6711,'006711','1973-038C','KDU','KDU',30640,'Reentered','1973-06-26 00:00:00','1973-06-22 00:00:00',88.99,170,279,51.65,'Intermediate' from dual union all
select 6712,'006712','1973-038D','Blenda','BZA',30640,'Reentered','1973-06-23 00:00:00','1973-06-22 00:00:00',88.82,168,265,51.66,'Intermediate' from dual union all
select 6713,'006713','1973-044A','Kosmos-576','Zenit-4MT',30646,'Landed','1973-07-09 00:00:00','1973-06-29 00:00:00',89.82,202,330,72.86,'Intermediate' from dual union all
select 6714,'006714','1973-044B','11S510','11S510',30646,'Reentered','1973-07-07 00:00:00','1973-06-29 00:00:00',89.49,195,304,72.85,'Intermediate' from dual union all
select 6715,'006715','1973-044C','Sensor cover','Zenit-4MT sensor cover',30646,'Reentered','1973-06-30 00:00:00','1973-06-29 00:00:00',89.82,202,330,72.86,'Intermediate' from dual union all
select 6716,'006716','1973-044D','Sensor cover','Zenit-4MT sensor cover',30646,'Reentered','1973-06-30 00:00:00','1973-06-28 00:00:00',88.94,164,281,72.85,'Intermediate' from dual union all
select 6717,'006717','1973-044E','Sensor cover','Zenit-4MT sensor cover',30646,'Reentered','1973-06-29 00:00:00','1973-06-28 00:00:00',89.3,180,300,72.85,'Intermediate' from dual union all
select 6718,'006718','1967-034F','NNS O-12 insulation','',15289,'Reentered','1980-03-06 00:00:00','1972-10-31 00:00:00',105.68,1019,1034,90.21,'Polar' from dual union all
select 6719,'006719','1973-044F','KB electropitaniya','KBE',30646,'Reentered','1973-07-11 00:00:00','1973-07-09 00:00:00',89.64,198,315,72.86,'Intermediate' from dual union all
select 672,'000672','1963-038D','deb Transit VBN-1','',9269,'In Earth orbit','','1963-11-23 00:00:00',107.36,1059,1150,89.92,'Polar' from dual union all
select 6720,'006720','1973-044G','KBE part','KBE part',30646,'Reentered','1973-07-11 00:00:00','1973-07-10 00:00:00',88.98,190,259,72.85,'Intermediate' from dual union all
select 6721,'006721','1973-044H','Blenda','KBE part (RAE Capsule)',30646,'Reentered','1973-07-14 00:00:00','1973-07-10 00:00:00',91.4,211,476,72.9,'Intermediate' from dual union all
select 6722,'006722','1973-045A','Molniya-2','Molniya-2',30647,'Reentered','1978-08-05 00:00:00','1973-08-13 00:00:00',717.96,391,39973,65.65,'GEO Transfer' from dual union all
select 6723,'006723','1973-045B','11S510','11S510',30647,'Reentered','1973-08-22 00:00:00','1973-07-18 00:00:00',91.08,216,439,65.41,'Intermediate' from dual union all
select 6724,'006724','1973-045C','BOZ','BOZ',30647,'Reentered','1973-08-03 00:00:00','1973-07-18 00:00:00',90.85,191,441,65.41,'Intermediate' from dual union all
select 6725,'006725','1973-039F','RAE Lunar Insertion Motor','Star 17',30641,'Deep Space Attached','1973-06-11 00:00:00','1973-06-11 00:00:00',15012.97,182,390244,29.1,'Cislunar' from dual union all
select 6726,'006726','1973-039G','RAE VCPS','Velocity Control Prop. S',30641,'Deep Space Attached','1973-06-11 00:00:00','1973-06-11 00:00:00',15012.97,182,390244,29.1,'Cislunar' from dual union all
select 6727,'006727','1973-046A','OPS 8261','HEXAGON SV-6',30648,'Deorbited','1973-10-12 00:00:00','1973-08-13 00:00:00',88.77,155,273,96.2,'Sun-Synch' from dual union all
select 6728,'006728','1973-046B','Titan 23D-7 Stage 2','LR91AJ11 23D-7',30648,'Reentered','1973-07-15 00:00:00','1973-07-15 00:00:00',88.03,142,212,96.21,'Sun-Synch' from dual union all
select 6729,'006729','1973-027Q','SAS debris','debris',30629,'Reentered','1973-10-16 00:00:00','1973-07-10 00:00:00',92.49,371,423,50.13,'Intermediate' from dual union all
select 673,'000673','1963-040A','Kosmos-20','Zenit-2 No. 13',9271,'Landed','1963-10-26 00:00:00','1963-10-18 00:00:00',89.49,202,297,64.89,'Intermediate' from dual union all
select 6730,'006730','1973-027R','SAS debris','debris',30629,'Reentered','1973-09-14 00:00:00','1973-07-09 00:00:00',92.07,353,400,50.1,'Intermediate' from dual union all
select 6731,'006731','1964-074D','Skylab part','[or Explorer 23 deb]',10360,'Reentered','1974-01-02 00:00:00','1973-07-14 00:00:00',95.69,384,720,50.93,'Intermediate' from dual union all
select 6732,'006732','1964-074E','Skylab part','[or Explorer 23 deb]',10360,'Reentered','1974-05-29 00:00:00','1973-07-11 00:00:00',96.93,429,795,51.42,'Intermediate' from dual union all
select 6733,'006733','1973-027S','Skylab part','debris',30629,'Reentered','1974-10-23 00:00:00','1973-07-09 00:00:00',96.89,436,784,48.98,'Intermediate' from dual union all
select 6734,'006734','1973-027U','Skylab part','debris',30629,'Reentered','1974-08-04 00:00:00','1973-07-09 00:00:00',97.5,405,874,49.06,'Intermediate' from dual union all
select 6735,'006735','1973-027T','Skylab part','debris',30629,'Reentered','1975-08-20 00:00:00','1974-01-01 00:00:00',98.43,409,959,50.62,'Intermediate' from dual union all
select 6736,'006736','1965-020FE','deb Blok Kosmos','',11770,'Reentered','1980-07-12 00:00:00','1965-03-15 00:00:00',111.35,738,1840,55,'Intermediate' from dual union all
select 6737,'006737','1973-021HA','deb Kosmos-554','',30623,'Reentered','1973-09-02 00:00:00','1973-07-17 00:00:00',93.28,273,598,73.1,'Intermediate' from dual union all
select 6738,'006738','1973-021HB','deb Kosmos-554','',30623,'Reentered','1973-10-14 00:00:00','1973-07-17 00:00:00',91.58,283,422,72.97,'Intermediate' from dual union all
select 6739,'006739','1973-021HC','deb Kosmos-554','',30623,'Reentered','1973-09-02 00:00:00','1973-07-17 00:00:00',90.63,234,377,72.79,'Intermediate' from dual union all
select 674,'000674','1963-039A','Vela 1A','Vela 1A',9270,'Deep Space','1980-01-01 00:00:00','1964-04-01 00:00:00',6486.19,101081,116581,38.71,'Very High' from dual union all
select 6740,'006740','1973-021HD','deb Kosmos-554','',30623,'Reentered','1973-09-20 00:00:00','1973-07-17 00:00:00',91.88,264,470,73.01,'Intermediate' from dual union all
select 6741,'006741','1973-045D','Blok-L','Blok-L',30647,'Reentered','1978-09-19 00:00:00','1973-08-19 00:00:00',702.59,425,39178,65.43,'GEO Transfer' from dual union all
select 6742,'006742','1973-047A','Mars-4','3MS No. 52S',30649,'Deep Space','1973-07-22 00:00:00','',null,null,null,null,'' from dual union all
select 6743,'006743','1973-047B','SOZ','SOZ',30649,'Reentered','1973-07-23 00:00:00','1973-07-21 00:00:00',87.99,150,200,51.52,'Intermediate' from dual union all
select 6744,'006744','1973-047C','SOZ','SOZ',30649,'Reentered','1973-07-26 00:00:00','1973-07-23 00:00:00',87.42,132,161,51.52,'Intermediate' from dual union all
select 6745,'006745','1973-048A','Kosmos-577','Zenit-4M',30650,'Landed','1973-08-07 00:00:00','1973-07-27 00:00:00',89.32,174,308,65.42,'Intermediate' from dual union all
select 6746,'006746','61 Omicron 251','deb r/b Ablestar','',7273,'Reentered','1974-12-21 00:00:00','1974-12-09 00:00:00',102.92,840,953,66.6,'Intermediate' from dual;
insert into SATELLITE
select 6747,'006747','61 Omicron 252','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-05 00:00:00',103.86,827,1055,66.6,'Intermediate' from dual union all
select 6748,'006748','61 Omicron 253','deb r/b Ablestar','',7273,'In Earth orbit','','1992-02-07 00:00:00',104.49,896,1046,66.92,'Intermediate' from dual union all
select 6749,'006749','1971-015CJ','deb Kosmos-397','',24833,'Reentered','1974-06-11 00:00:00','1973-02-25 00:00:00',112.21,572,2085,65.71,'Intermediate' from dual union all
select 675,'000675','1963-039B','ERS 12','ERS 12',9270,'Reentered','1966-02-05 00:00:00','1964-02-11 00:00:00',2319.36,952,102375,35.9,'Very High' from dual union all
select 6750,'006750','1969-082KH','deb Agena D','',18349,'Reentered','1981-01-31 00:00:00','1992-02-07 00:00:00',102,833,874,70.1,'Intermediate' from dual union all
select 6751,'006751','1968-091CH','deb Kosmos-249','',16818,'Reentered','1989-09-27 00:00:00','1989-09-26 00:00:00',89.49,239,260,62.21,'Intermediate' from dual union all
select 6752,'006752','1968-091CJ','deb Kosmos-249','',16818,'In Earth orbit','','1992-02-06 00:00:00',112.56,756,1933,62.29,'Intermediate' from dual union all
select 6753,'006753','1973-048B','11A57I','11A57I',30650,'Reentered','1973-08-01 00:00:00','1973-07-27 00:00:00',89.15,190,275,65.39,'Intermediate' from dual union all
select 6754,'006754','1973-049A','Mars-5','3MS No. 53S',30651,'Deep Space','1973-07-26 00:00:00','',null,null,null,null,'' from dual union all
select 6755,'006755','1973-049B','SOZ','SOZ',30651,'Reentered','1973-07-27 00:00:00','1973-07-26 00:00:00',87.88,152,187,51.55,'Intermediate' from dual union all
select 6756,'006756','1973-049C','SOZ','SOZ',30651,'Reentered','1973-07-27 00:00:00','1973-07-25 00:00:00',87.85,63,274,51.52,'ERROR - Unexpected value "TA"' from dual union all
select 6757,'006757','1973-050A','Skylab SL-3','Apollo CM-117',30652,'Landed','1973-09-25 00:00:00','1973-08-08 00:00:00',93.21,425,438,50.03,'Intermediate' from dual union all
select 6758,'006758','1973-050B','Saturn S-IVB-207','Saturn S-IVB-207',30652,'Deorbited','1973-07-28 00:00:00','1973-07-28 00:00:00',88.31,159,223,50.03,'Intermediate' from dual union all
select 6759,'006759','1973-051A','Kosmos-578','Zenit-2M',30653,'Landed','1973-08-13 00:00:00','1973-08-04 00:00:00',89.36,201,285,65.4,'Intermediate' from dual union all
select 676,'000676','1963-040B','8A92E','8A92E',9271,'Reentered','1963-10-30 00:00:00','1963-10-20 00:00:00',89.54,205,299,64.92,'Intermediate' from dual union all
select 6760,'006760','1973-051B','11A57I','11A57I',30653,'Reentered','1973-08-09 00:00:00','1973-08-04 00:00:00',89.07,187,270,65.35,'Intermediate' from dual union all
select 6761,'006761','1973-051C','Sensor cover','Zenit-2M sensor cover',30653,'Reentered','1973-08-05 00:00:00','1973-08-02 00:00:00',90.25,246,328,65.41,'Intermediate' from dual union all
select 6762,'006762','1973-051D','Sensor cover','Zenit-2M sensor cover',30653,'Reentered','1973-08-02 00:00:00','1973-08-02 00:00:00',88.53,202,202,65.4,'Intermediate' from dual union all
select 6763,'006763','1973-051E','Sensor cover','Zenit-2M sensor cover',30653,'Reentered','1973-08-02 00:00:00','1973-08-02 00:00:00',88.79,192,238,65.39,'Intermediate' from dual union all
select 6764,'006764','1968-091CK','deb Kosmos-249','',16818,'Reentered','1979-10-27 00:00:00','1998-09-23 00:00:00',116.55,929,2123,62.33,'Intermediate' from dual union all
select 6765,'006765','1970-025NG','deb Agena D','',21502,'In Earth orbit','','1992-02-05 00:00:00',106.6,1057,1082,99.81,'Sun-Synch' from dual union all
select 6766,'006766','1970-025NH','deb Agena D','',21502,'In Earth orbit','','1992-02-07 00:00:00',104.3,894,1029,100.14,'Sun-Synch' from dual union all
select 6767,'006767','1970-025NF','deb Agena D','',21502,'In Earth orbit','','1992-02-06 00:00:00',105.18,980,1026,99.51,'Sun-Synch' from dual union all
select 6768,'006768','1973-052A','Mars-6','3MP No. 50P',30654,'Deep Space','1973-08-06 00:00:00','',null,null,null,null,'' from dual union all
select 6769,'006769','1973-052B','SOZ','SOZ',30654,'Reentered','1973-08-07 00:00:00','1973-08-06 00:00:00',88.14,154,211,51.5,'Intermediate' from dual union all
select 677,'000677','1963-041A','OPS 2196','GAMBIT SV 953',9272,'Deorbited','1963-10-29 00:00:00','1963-10-27 00:00:00',88.6,119,292,99.08,'Retrograde' from dual union all
select 6770,'006770','1973-048C','Blenda','BZA',30650,'Reentered','1973-08-07 00:00:00','1973-08-06 00:00:00',88.81,168,264,65.38,'Intermediate' from dual union all
select 6771,'006771','1973-052C','SOZ','SOZ',30654,'Reentered','1973-08-07 00:00:00','1973-08-06 00:00:00',88.14,154,211,51.5,'Intermediate' from dual union all
select 6772,'006772','1973-048D','KDU','KDU',30650,'Reentered','1973-08-12 00:00:00','1973-08-06 00:00:00',88.99,170,279,65.41,'Intermediate' from dual union all
select 6773,'006773','1973-039H','Delta 95 clamp band half','',30641,'Reentered','1973-09-27 00:00:00','1973-07-10 00:00:00',104.87,182,1795,29.34,'Intermediate' from dual union all
select 6774,'006774','1973-027V','S-063 camera','S-063',30629,'Reentered','1973-09-20 00:00:00','1973-08-07 00:00:00',93.2,423,440,50.02,'Intermediate' from dual union all
select 6775,'006775','1973-048E','PDTU cover','KDU part',30650,'Reentered','1973-08-09 00:00:00','1973-08-07 00:00:00',90.06,178,377,65.41,'Intermediate' from dual union all
select 6776,'006776','1973-053A','Mars-7','3MP No. 51P',30655,'Deep Space','1973-08-10 00:00:00','',null,null,null,null,'' from dual union all
select 6777,'006777','1973-053B','SOZ','SOZ',30655,'Reentered','1973-08-11 00:00:00','1973-08-10 00:00:00',88.11,154,208,51.51,'Intermediate' from dual union all
select 6778,'006778','1973-053C','SOZ','SOZ',30655,'Reentered','1973-08-11 00:00:00','1973-08-09 00:00:00',88.11,154,208,51.51,'Intermediate' from dual union all
select 6779,'006779','1971-116B','Centaur AC-26','Centaur 23D',24934,'In Earth orbit','','1972-02-01 00:00:00',657.24,611,36714,27.98,'GEO Transfer' from dual union all
select 678,'000678','1963-041B','Agena D 4703','Agena D',9272,'Reentered','1963-10-29 00:00:00','1963-10-27 00:00:00',88.41,116,276,99.08,'Retrograde' from dual union all
select 6780,'006780','1973-027W','A-Frame part','EVA debris',30629,'Reentered','1974-12-12 00:00:00','1973-08-07 00:00:00',99.42,402,1060,50.55,'Intermediate' from dual union all
select 6781,'006781','1973-027X','A-Frame part','EVA debris',30629,'Reentered','1974-11-08 00:00:00','1973-08-11 00:00:00',99.41,396,1065,50.49,'Intermediate' from dual union all
select 6782,'006782','1973-027Y','A-Frame part','EVA debris',30629,'Reentered','1974-11-10 00:00:00','1973-08-11 00:00:00',98.35,411,948,49.25,'Intermediate' from dual union all
select 6783,'006783','1965-082UH','deb Transtage 4','',11832,'Reentered','1980-07-27 00:00:00','1980-07-13 00:00:00',92.76,401,419,32.55,'Intermediate' from dual union all
select 6784,'006784','1966-013M','deb Rubis','',13420,'Reentered','1974-03-15 00:00:00','1973-07-28 00:00:00',113.95,491,2325,34.06,'Intermediate' from dual union all
select 6785,'006785','1973-021HE','deb Kosmos-554','',30623,'Reentered','1973-11-18 00:00:00','1973-10-26 00:00:00',94.21,275,685,73.22,'Intermediate' from dual union all
select 6786,'006786','1973-027Z','A-Frame part','EVA debris',30629,'Reentered','1975-01-16 00:00:00','1974-01-01 00:00:00',92.85,407,422,50.04,'Intermediate' from dual union all
select 6787,'006787','1973-054A','DMSP 7529','DMSP Block 5B F-4',30656,'In Earth orbit','','1973-09-16 00:00:00',101.49,807,852,98.84,'Sun-Synch' from dual union all
select 6788,'006788','1973-054B','Burner IIA No. 21 Stage 2','Star 26B S/N 004',30656,'In Earth orbit','','1973-09-16 00:00:00',101.45,805,850,98.82,'Sun-Synch' from dual union all
select 6789,'006789','1973-055A','Kosmos-579','Zenit-4M',30657,'Landed','1973-09-03 00:00:00','1973-08-23 00:00:00',89.08,168,290,65.42,'Intermediate' from dual union all
select 679,'000679','1963-041C','deb USAF satellite','',9272,'Reentered','1963-10-29 00:00:00','1963-10-28 00:00:00',88.16,116,250,99.08,'Retrograde' from dual union all
select 6790,'006790','1973-055B','11A57I','11A57I',30657,'Reentered','1973-08-28 00:00:00','1973-08-23 00:00:00',89.21,193,278,65.42,'Intermediate' from dual union all
select 6791,'006791','1973-056A','OPS 7724','JUMPSEAT 3',30658,'In Earth orbit','','1973-08-24 00:00:00',701.89,362,39206,63.26,'Molniya' from dual union all
select 6792,'006792','1973-056B','Agena D','Agena D',30658,'In Earth orbit','','1974-01-27 00:00:00',697.42,438,38908,63.22,'Molniya' from dual union all
select 6793,'006793','1973-057A','Kosmos-580','DS-P1-Yu No. 59',30659,'Reentered','1974-04-01 00:00:00','1973-09-22 00:00:00',91.94,268,472,71,'Intermediate' from dual union all
select 6794,'006794','1973-057B','S1M','S1M',30659,'Reentered','1973-11-30 00:00:00','1973-09-23 00:00:00',91.55,264,438,71,'Intermediate' from dual union all
select 6795,'006795','1973-055C','Sensor cover','Zenit-4M sensor cover',30657,'Reentered','1973-08-25 00:00:00','1973-08-23 00:00:00',88.98,196,253,65.5,'Intermediate' from dual union all
select 6796,'006796','1973-058A','INTELSAT IV F7','INTELSAT IV F7',30660,'In Earth orbit','','1977-01-21 00:00:00',1436.09,35782,35791,.63,'Stationary' from dual union all
select 6797,'006797','1973-058B','Centaur D-1A AC-31','Centaur D-1A AC-31',30660,'In Earth orbit','','1973-09-24 00:00:00',655,610,36601,27.5,'GEO Transfer' from dual union all
select 6798,'006798','1973-059A','Kosmos-581','Zenit-4M',30661,'Landed','1973-09-06 00:00:00','1973-08-26 00:00:00',89.06,172,284,51.61,'Intermediate' from dual union all
select 6799,'006799','1973-059B','11A57I','11A57I',30661,'Reentered','1973-08-31 00:00:00','1973-08-26 00:00:00',89.12,199,263,51.63,'Intermediate' from dual union all
select 68,'000068','60 Tau','Discoverer 19','CORONA RM-1',6332,'Reentered Attached','1961-01-23 00:00:00','1960-12-27 00:00:00',92.39,206,578,83.4,'Intermediate' from dual union all
select 680,'000680','1963-041D','deb USAF satellite','',9272,'Reentered','1963-10-29 00:00:00','1963-10-27 00:00:00',88.34,116,269,99.08,'Retrograde' from dual union all
select 6800,'006800','1972-091B','FW4S S/N 2376-5','FW4S',28023,'Reentered','1979-05-01 00:00:00','1973-02-01 00:00:00',95.2,436,621,1.91,'Equatorial' from dual union all
select 6801,'006801','1973-059C','Sensor cover','Zenit-4M sensor cover',30661,'Reentered','1973-08-25 00:00:00','1973-08-24 00:00:00',88.94,200,244,51.57,'Intermediate' from dual union all
select 6802,'006802','1973-060A','Kosmos-582','Tselina-OM',30662,'Reentered','1980-09-05 00:00:00','1973-09-27 00:00:00',95.21,517,541,74.03,'Intermediate' from dual union all
select 6803,'006803','1973-060B','S3M','S3M',30662,'Reentered','1981-01-01 00:00:00','1973-10-08 00:00:00',95.14,509,541,74.04,'Intermediate' from dual union all
select 6804,'006804','1973-060C','deb Kosmos-582','Tselina-O part',30662,'Reentered','1982-06-20 00:00:00','1973-09-27 00:00:00',94.85,486,537,74.06,'Intermediate' from dual union all
select 6805,'006805','1973-061A','Molniya-1','Molniya-1',30663,'Reentered','1979-12-05 00:00:00','1973-09-30 00:00:00',717.68,466,39884,65.44,'GEO Transfer' from dual union all
select 6806,'006806','1973-061B','11S510','11S510',30663,'Reentered','1973-10-01 00:00:00','1973-09-06 00:00:00',90.98,214,432,65.42,'Intermediate' from dual union all
select 6807,'006807','1973-061C','BOZ','BOZ',30663,'Reentered','1973-10-08 00:00:00','1973-09-06 00:00:00',91.31,219,459,65.39,'Intermediate' from dual union all
select 6808,'006808','1973-061D','deb Blok-I','',30663,'Reentered','1973-09-06 00:00:00','1973-09-04 00:00:00',89.82,195,337,65.43,'Intermediate' from dual union all
select 6809,'006809','1973-062A','Kosmos-583','Zenit-2M',30664,'Landed','1973-09-12 00:00:00','1973-09-01 00:00:00',89.5,203,297,64.92,'Intermediate' from dual union all
select 681,'000681','1963-042A','OPS 2437','ARGON 6',9273,'Reentered Attached','1964-01-21 00:00:00','1963-11-29 00:00:00',90.34,275,308,89.91,'Polar' from dual union all
select 6810,'006810','1973-062B','11A57I','11A57I',30664,'Reentered','1973-09-06 00:00:00','1973-09-04 00:00:00',88.61,176,236,64.92,'Intermediate' from dual union all
select 6811,'006811','1973-062C','Sensor cover','Zenit-2M sensor cover',30664,'Reentered','1973-09-03 00:00:00','1973-08-30 00:00:00',88.8,194,237,64,'Intermediate' from dual union all
select 6812,'006812','1973-061E','deb Blok-I','',30663,'Reentered','1973-09-07 00:00:00','1973-09-03 00:00:00',91.06,176,478,65.4,'Intermediate' from dual union all
select 6813,'006813','1973-055D','KDU','KDU',30657,'Reentered','1973-09-05 00:00:00','1973-09-04 00:00:00',88.16,155,212,65.41,'Intermediate' from dual union all
select 6814,'006814','1973-055E','Blenda','BZA',30657,'Reentered','1973-09-04 00:00:00','1973-09-04 00:00:00',89.06,161,295,64.8,'Intermediate' from dual union all
select 6815,'006815','1973-061F','Blok-L','Blok-L',30663,'Reentered','1979-12-05 00:00:00','1973-12-01 00:00:00',678.22,478,37907,65.38,'GEO Transfer' from dual union all
select 6816,'006816','1973-059D','Blenda','BZA',30661,'Reentered','1973-09-06 00:00:00','1973-09-05 00:00:00',88.63,154,260,51.63,'Intermediate' from dual union all
select 6817,'006817','1973-059E','KDU','KDU',30661,'Reentered','1973-09-09 00:00:00','1973-09-05 00:00:00',88.91,176,266,51.6,'Intermediate' from dual union all
select 6818,'006818','1973-063A','Kosmos-584','Zenit-4M',30665,'Landed','1973-09-20 00:00:00','1973-09-09 00:00:00',89.88,204,333,72.85,'Intermediate' from dual union all
select 6819,'006819','1973-063B','11A57I','11A57I',30665,'Reentered','1973-09-17 00:00:00','1973-09-09 00:00:00',89.56,194,312,72.84,'Intermediate' from dual union all
select 682,'000682','1963-042B','PUNDIT I','P-11 No. 4001',9273,'Reentered','1965-05-23 00:00:00','1963-12-03 00:00:00',93.15,290,568,89.98,'Polar' from dual union all
select 6820,'006820','1973-063C','Sensor cover','Zenit-4M sensor cover',30665,'Reentered','1973-09-09 00:00:00','1973-09-07 00:00:00',89.47,190,307,72.72,'Intermediate' from dual union all
select 6821,'006821','1973-063D','Sensor cover','Zenit-4M sensor cover',30665,'Reentered','1973-09-07 00:00:00','1973-09-06 00:00:00',89.4,201,288,72.8,'Intermediate' from dual union all
select 6822,'006822','1972-079C','OPS 8314/2','P801-4',28011,'In Earth orbit','','1972-12-02 00:00:00',114.74,1420,1467,95.64,'Retrograde' from dual union all
select 6823,'006823','1972-079D','USAF satellite sensor cover','',28011,'In Earth orbit','','1972-12-02 00:00:00',114.83,1407,1489,95.79,'Retrograde' from dual union all
select 6824,'006824','1972-079E','USAF satellite sensor cover','',28011,'In Earth orbit','','1972-12-02 00:00:00',114.68,1435,1447,95.51,'Retrograde' from dual union all
select 6825,'006825','1973-064A','Kosmos-585','Geoid',30666,'In Earth orbit','','1973-10-10 00:00:00',113.56,1374,1406,74.03,'Intermediate' from dual union all
select 6826,'006826','1973-064B','S3M','S3M',30666,'In Earth orbit','','1973-10-08 00:00:00',113.43,1367,1400,74.02,'Intermediate' from dual union all
select 6827,'006827','1973-064C','deb Kosmos-585','',30666,'Reentered','1973-12-12 00:00:00','1974-04-11 00:00:00',113.59,1239,1544,74.07,'Intermediate' from dual union all
select 6828,'006828','1973-065A','Kosmos-586','Zaliv',30667,'In Earth orbit','','1973-10-14 00:00:00',104.82,966,1006,82.94,'Intermediate' from dual union all
select 6829,'006829','1973-065B','S3M','S3M',30667,'In Earth orbit','','1973-10-29 00:00:00',104.72,964,999,82.95,'Intermediate' from dual union all
select 683,'000683','1963-043A','Polyot','I-2B No. 102',9274,'Reentered','1982-10-16 00:00:00','1963-12-05 00:00:00',102.4,327,1418,58.88,'Intermediate' from dual union all
select 6830,'006830','1973-063E','KDU','KDU',30665,'Reentered','1973-09-28 00:00:00','1973-09-19 00:00:00',89.54,201,303,72.85,'Intermediate' from dual union all
select 6831,'006831','1973-063F','Blenda','BZA',30665,'Reentered','1973-09-25 00:00:00','1973-09-19 00:00:00',89.41,199,292,72.85,'Intermediate' from dual union all
select 6832,'006832','1973-066A','Kosmos-587','Zenit-4MK',30668,'Landed','1973-10-04 00:00:00','1973-09-23 00:00:00',89.52,204,298,65.43,'Intermediate' from dual union all
select 6833,'006833','1973-066B','11S510','11S510',30668,'Reentered','1973-09-28 00:00:00','1973-09-23 00:00:00',89.27,195,282,65.42,'Intermediate' from dual union all
select 6834,'006834','1973-066C','Zenit-4MK sensor cover','',30668,'Reentered','1973-09-23 00:00:00','1973-09-22 00:00:00',88.44,153,242,65.41,'Intermediate' from dual union all
select 6835,'006835','1973-063G','PTDU cover','KDU part',30665,'Reentered','1973-09-25 00:00:00','1973-09-22 00:00:00',90.22,210,361,72.83,'Intermediate' from dual union all
select 6836,'006836','1973-067A','Soyuz-12','Soyuz 7K-T No. 37',30669,'Landed','1973-09-29 00:00:00','1973-09-28 00:00:00',91.23,329,341,51.58,'Intermediate' from dual;
insert into SATELLITE
select 6837,'006837','1973-068A','OPS 6275','GAMBIT-3 Mission 4340',30670,'Reentered Attached','1973-10-29 00:00:00','1973-10-04 00:00:00',89.82,128,404,110.46,'Retrograde' from dual union all
select 6838,'006838','1973-067B','11S510','11S510',30669,'Reentered','1973-09-29 00:00:00','1973-09-28 00:00:00',88.41,181,211,51.58,'Intermediate' from dual union all
select 6839,'006839','1973-067C','Soyuz-12 BO','Bitovoy otsek',30669,'Reentered','1974-01-21 00:00:00','1973-10-27 00:00:00',90.86,300,334,51.58,'Intermediate' from dual union all
select 684,'000684','1963-043B','deb Polyot','',9274,'Reentered','1966-11-23 00:00:00','1963-12-06 00:00:00',102.24,326,1403,58.56,'Intermediate' from dual union all
select 6840,'006840','1973-067D','Soyuz-12 part','Soyuz part',30669,'Reentered','1973-11-03 00:00:00','1973-10-04 00:00:00',90.96,306,338,51.58,'Intermediate' from dual union all
select 6841,'006841','1965-060C','Apollo BP-9A part','',11810,'Reentered','1973-10-02 00:00:00','1973-09-27 00:00:00',92.58,397,405,28.88,'Intermediate' from dual union all
select 6842,'006842','61 Omicron 254','deb r/b Ablestar','',7273,'In Earth orbit','','1992-01-29 00:00:00',115.93,970,2025,66.73,'Intermediate' from dual union all
select 6843,'006843','1971-015CK','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-07 00:00:00',107.5,561,1661,65.69,'Intermediate' from dual union all
select 6844,'006844','1971-015CL','deb Kosmos-397','',24833,'In Earth orbit','','1992-02-05 00:00:00',108.42,551,1756,65.92,'Intermediate' from dual union all
select 6845,'006845','1973-069A','Kosmos-588','Strela-1M',30671,'In Earth orbit','','1973-11-01 00:00:00',115.34,1448,1494,74,'Intermediate' from dual union all
select 6846,'006846','1973-069B','Kosmos-589','Strela-1M',30671,'In Earth orbit','','1973-11-01 00:00:00',114.92,1416,1488,73.99,'Intermediate' from dual union all
select 6847,'006847','1973-069C','Kosmos-590','Strela-1M',30671,'In Earth orbit','','1973-11-01 00:00:00',115.12,1434,1488,74,'Intermediate' from dual union all
select 6848,'006848','1973-069D','Kosmos-591','Strela-1M',30671,'In Earth orbit','','1973-11-01 00:00:00',114.15,1348,1486,73.99,'Intermediate' from dual union all
select 6849,'006849','1973-069E','Kosmos-592','Strela-1M',30671,'In Earth orbit','','1973-11-01 00:00:00',113.95,1332,1484,74,'Intermediate' from dual union all
select 685,'000685','1963-043C','deb Polyot','',9274,'Reentered','1965-10-29 00:00:00','1963-12-03 00:00:00',100.89,298,1303,58.95,'Intermediate' from dual union all
select 6850,'006850','1973-069F','Kosmos-593','Strela-1M',30671,'In Earth orbit','','1973-11-02 00:00:00',114.34,1365,1486,73.99,'Intermediate' from dual union all
select 6851,'006851','1973-069G','Kosmos-594','Strela-1M',30671,'In Earth orbit','','1973-11-02 00:00:00',114.52,1382,1485,74,'Intermediate' from dual union all
select 6852,'006852','1973-069H','Kosmos-595','Strela-1M',30671,'In Earth orbit','','1973-11-02 00:00:00',114.71,1399,1485,74,'Intermediate' from dual union all
select 6853,'006853','1973-069J','S3M','S3M',30671,'In Earth orbit','','1973-11-13 00:00:00',117.16,1484,1623,74,'Intermediate' from dual union all
select 6854,'006854','1973-066D','KDU','KDU',30668,'Reentered','1973-10-08 00:00:00','1973-10-03 00:00:00',89.37,168,319,65.41,'Intermediate' from dual union all
select 6855,'006855','1973-066E','Blenda','Sunshade',30668,'Reentered','1973-10-04 00:00:00','1973-10-03 00:00:00',89.37,163,323,65.49,'Intermediate' from dual union all
select 6856,'006856','1973-070A','Kosmos-596','Zenit-2M',30672,'Landed','1973-10-09 00:00:00','1973-10-05 00:00:00',89.38,204,284,65.41,'Intermediate' from dual union all
select 6857,'006857','1973-070B','11A57I','11A57I',30672,'Reentered','1973-10-09 00:00:00','1973-10-05 00:00:00',89.08,191,267,65.41,'Intermediate' from dual union all
select 6858,'006858','1973-071A','Kosmos-597','Zenit-4MK',30673,'Landed','1973-10-12 00:00:00','1973-10-08 00:00:00',89.06,206,249,65.42,'Intermediate' from dual union all
select 6859,'006859','1973-071B','11A57I','11A57I',30673,'Reentered','1973-10-14 00:00:00','1973-10-08 00:00:00',89.19,201,268,65.4,'Intermediate' from dual union all
select 686,'000686','1963-043D','deb Polyot','',9274,'Reentered','1966-12-18 00:00:00','1963-12-02 00:00:00',101.89,327,1370,59.8,'Intermediate' from dual union all
select 6860,'006860','1973-071C','Sensor cover','Zenit-4M sensor cover',30673,'Reentered','1973-10-08 00:00:00','1973-10-07 00:00:00',89.35,209,276,65.46,'Intermediate' from dual union all
select 6861,'006861','1973-070C','FEU-170 cover','FEU-170 part',30672,'Reentered','1973-10-17 00:00:00','1973-10-09 00:00:00',89.34,204,279,65.39,'Intermediate' from dual union all
select 6862,'006862','1973-072A','Kosmos-598','Zenit-4M',30674,'Landed','1973-10-16 00:00:00','1973-10-13 00:00:00',88.97,207,241,72.84,'Intermediate' from dual union all
select 6863,'006863','1973-072B','11A57I','11A57I',30674,'Reentered','1973-10-21 00:00:00','1973-10-12 00:00:00',89.63,196,317,72.85,'Intermediate' from dual union all
select 6864,'006864','1973-072C','Sensor cover','Zenit-4M sensor cover',30674,'Reentered','1973-10-11 00:00:00','1973-10-10 00:00:00',89.4,194,296,72.84,'Intermediate' from dual union all
select 6865,'006865','1973-071D','KDU','KDU',30673,'Reentered','1973-10-16 00:00:00','1973-10-11 00:00:00',88.73,199,225,65.45,'Intermediate' from dual union all
select 6866,'006866','1973-071E','Blenda','Sunshade',30673,'Reentered','1973-10-18 00:00:00','1973-10-11 00:00:00',88.93,198,246,65.42,'Intermediate' from dual union all
select 6867,'006867','1973-073A','Kosmos-599','Zenit-2M',30675,'Landed','1973-10-28 00:00:00','1973-10-17 00:00:00',89.28,202,276,64.95,'Intermediate' from dual union all
select 6868,'006868','1973-073B','11A57I','11A57I',30675,'Reentered','1973-10-21 00:00:00','1973-10-17 00:00:00',88.93,190,254,64.96,'Intermediate' from dual union all
select 6869,'006869','1973-073C','Sensor cover','Zenit-2M sensor cover',30675,'Reentered','1973-10-24 00:00:00','1973-10-16 00:00:00',89.26,199,276,64.98,'Intermediate' from dual union all
select 687,'000687','1963-044A','Kosmos-21','3MV-1 No. 1',9275,'Reentered','1963-11-14 00:00:00','1963-11-12 00:00:00',88.48,180,219,64.78,'Intermediate' from dual union all
select 6870,'006870','1973-073D','Sensor cover','Zenit-2M sensor cover',30675,'Reentered','1973-10-24 00:00:00','1973-10-15 00:00:00',89.32,202,280,64.8,'Intermediate' from dual union all
select 6871,'006871','1973-072D','KDU','KDU',30674,'Reentered','1973-10-20 00:00:00','1973-10-15 00:00:00',88.87,198,240,72.87,'Intermediate' from dual union all
select 6872,'006872','1973-072E','Blenda','BZA',30674,'Reentered','1973-10-18 00:00:00','1973-10-15 00:00:00',88.7,189,231,72.84,'Intermediate' from dual union all
select 6873,'006873','1973-074A','Kosmos-600','Zenit-4M',30676,'Landed','1973-10-23 00:00:00','1973-10-18 00:00:00',89.92,204,337,72.84,'Intermediate' from dual union all
select 6874,'006874','1973-074B','11A57I','11A57I',30676,'Reentered','1973-10-28 00:00:00','1973-10-18 00:00:00',89.73,201,322,72.84,'Intermediate' from dual union all
select 6875,'006875','1973-075A','Kosmos-601','DS-P1-Yu No. 60',30677,'Reentered','1974-08-15 00:00:00','1973-11-24 00:00:00',100.88,197,1403,81.89,'Intermediate' from dual union all
select 6876,'006876','1973-075B','S1M','S1M',30677,'Reentered','1973-11-06 00:00:00','1973-10-23 00:00:00',101.24,205,1430,81.9,'Intermediate' from dual union all
select 6877,'006877','1973-076A','Molniya-2','Molniya-2',30678,'Reentered','1983-07-08 00:00:00','1973-12-12 00:00:00',717.78,602,39753,62.91,'Molniya' from dual union all
select 6878,'006878','1973-076B','11S510','11S510',30678,'Reentered','1973-12-03 00:00:00','1973-10-26 00:00:00',91.68,214,500,62.81,'Intermediate' from dual union all
select 6879,'006879','1973-076C','BOZ','BOZ',30678,'Reentered','1973-12-18 00:00:00','1973-11-19 00:00:00',91.17,201,463,62.83,'Intermediate' from dual union all
select 688,'000688','1963-044B','8K78I','8K78I',9275,'Reentered','1963-11-12 00:00:00','1963-11-11 00:00:00',88.35,182,204,65.08,'Intermediate' from dual union all
select 6880,'006880','1973-075C','deb Kosmos-601','',30677,'Reentered','1974-03-28 00:00:00','1973-11-21 00:00:00',99.71,206,1284,81.91,'Intermediate' from dual union all
select 6881,'006881','1973-075D','deb Kosmos-601','',30677,'Reentered','1973-12-04 00:00:00','1973-10-23 00:00:00',100.27,201,1341,81.69,'Intermediate' from dual union all
select 6882,'006882','1973-075E','deb Kosmos-601','',30677,'Reentered','1974-02-07 00:00:00','1973-12-03 00:00:00',97.2,194,1056,81.77,'Intermediate' from dual union all
select 6883,'006883','1973-075F','deb Kosmos-601','',30677,'Reentered','1973-12-04 00:00:00','1973-10-27 00:00:00',96.89,167,1053,81.83,'Intermediate' from dual union all
select 6884,'006884','1973-075G','deb Kosmos-601','',30677,'Reentered','1974-02-21 00:00:00','1973-11-23 00:00:00',101.3,242,1399,81.88,'Intermediate' from dual union all
select 6885,'006885','1973-077A','Kosmos-602','Zenit-4MK',30679,'Landed','1973-10-29 00:00:00','1973-10-22 00:00:00',89.28,169,308,72.86,'Intermediate' from dual union all
select 6886,'006886','1973-077B','11A57I','11A57I',30679,'Reentered','1973-10-31 00:00:00','1973-10-24 00:00:00',89.54,195,309,72.85,'Intermediate' from dual union all
select 6887,'006887','1973-074C','KDU','KDU',30676,'Reentered','1973-10-24 00:00:00','1973-10-22 00:00:00',89.04,191,264,72.82,'Intermediate' from dual union all
select 6888,'006888','1973-074D','Blenda','BZA',30676,'Reentered','1973-10-28 00:00:00','1973-10-22 00:00:00',89.22,198,274,72.81,'Intermediate' from dual union all
select 6889,'006889','1973-075H','deb Kosmos-601','',30677,'Reentered','1974-02-04 00:00:00','1973-11-19 00:00:00',96.33,161,1005,81.8,'Intermediate' from dual union all
select 689,'000689','1963-045A','Kosmos-22','Zenit-4 No. 1',9276,'Landed','1963-11-22 00:00:00','1963-11-18 00:00:00',90.26,194,381,64.9,'Intermediate' from dual union all
select 6890,'006890','1973-075J','deb Kosmos-601','',30677,'Reentered','1973-12-04 00:00:00','1973-10-25 00:00:00',98.98,202,1218,81.74,'Intermediate' from dual union all
select 6891,'006891','1973-075K','deb Kosmos-601','',30677,'Reentered','1974-03-01 00:00:00','1973-11-19 00:00:00',98.53,191,1185,81.84,'Intermediate' from dual union all
select 6892,'006892','1973-075L','deb Kosmos-601','',30677,'Reentered','1973-12-04 00:00:00','1973-10-25 00:00:00',99.52,196,1275,81.84,'Intermediate' from dual union all
select 6893,'006893','1973-078A','IMP 8','IMP J',30680,'Deep Space','1973-10-26 00:00:00','1975-07-25 00:00:00',17287.5,215092,215092,19.7,'Deep Space' from dual union all
select 6894,'006894','1973-078B','Delta 97 clamp band half','',30680,'Reentered','1973-10-29 00:00:00','1973-10-27 00:00:00',90.56,164,440,28.76,'Intermediate' from dual union all
select 6895,'006895','1973-078C','Delta 97','Delta DSV-3E-5',30680,'Reentered','1996-11-29 00:00:00','1973-11-27 00:00:00',112.53,362,2324,28.86,'Intermediate' from dual union all
select 6896,'006896','1973-078D','Star 37C S/N 40022','Star 37C',30680,'Deep Space','1973-10-26 00:00:00','1973-10-26 00:00:00',6971.03,196,228808,28.8,'Deep Space' from dual union all
select 6897,'006897','1973-075M','deb Kosmos-601','',30677,'Reentered','1974-01-18 00:00:00','1973-11-19 00:00:00',97.14,180,1063,81.82,'Intermediate' from dual union all
select 6898,'006898','1973-076D','Blok-L','Blok-L',30678,'Reentered','1983-06-22 00:00:00','1973-11-19 00:00:00',733.12,585,40524,62.91,'Molniya' from dual union all
select 6899,'006899','1973-074E','PTDU cover','KDU (RAE)',30676,'Reentered','1973-10-29 00:00:00','1973-10-27 00:00:00',90.41,248,342,72.85,'Intermediate' from dual union all
select 69,'000069','60 Xi 3','Explorer VIII despin weight','',6334,'Reentered','1970-02-16 00:00:00','1961-01-04 00:00:00',112.29,395,2269,49.36,'Intermediate' from dual union all
select 690,'000690','1963-045B','11A57I','11A57I',9276,'Reentered','1963-12-03 00:00:00','1963-11-24 00:00:00',89.52,187,315,64.87,'Intermediate' from dual union all
select 6900,'006900','1973-079A','Kosmos-603','Zenit-4M No. 287',30681,'Landed','1973-11-09 00:00:00','1973-10-29 00:00:00',90.09,203,354,72.87,'Intermediate' from dual union all
select 6901,'006901','1973-079B','11A57I','11A57I',30681,'Reentered','1973-11-09 00:00:00','1973-10-29 00:00:00',89.89,200,338,72.84,'Intermediate' from dual union all
select 6902,'006902','1973-078E','deb Kosmos-603','[UNID065]',30680,'Reentered','1973-10-27 00:00:00','1973-10-27 00:00:00',88.66,193,224,72.8,'Intermediate' from dual union all
select 6903,'006903','1973-079C','Sensor cover','Zenit-4M sensor cover',30681,'Reentered','1973-10-29 00:00:00','1973-10-28 00:00:00',88.62,191,222,72.79,'Intermediate' from dual union all
select 6904,'006904','1973-079D','Sensor cover','Zenit-4M sensor cover',30681,'Reentered','1973-10-29 00:00:00','1973-10-28 00:00:00',89.97,199,347,72.87,'Intermediate' from dual union all
select 6905,'006905','1973-077C','Blenda','Sunshade',30679,'Reentered','1973-10-31 00:00:00','1973-10-28 00:00:00',88.78,164,265,72.85,'Intermediate' from dual union all
select 6906,'006906','1973-077D','KDU','KDU',30679,'Reentered','1973-11-01 00:00:00','1973-10-28 00:00:00',88.64,167,248,72.87,'Intermediate' from dual union all
select 6907,'006907','1973-080A','Kosmos-604','Tselina-D',30682,'Reentered','1992-01-19 00:00:00','1973-11-29 00:00:00',97.16,609,637,81.22,'Intermediate' from dual union all
select 6908,'006908','1973-080B','8A92ME','8A92ME',30682,'Reentered','1992-09-30 00:00:00','1973-11-29 00:00:00',97.18,571,676,81.22,'Intermediate' from dual union all
select 6909,'006909','1973-081A','NNS O-20','NNS O-20',30683,'In Earth orbit','','1973-12-01 00:00:00',105.55,897,1144,90.19,'Polar' from dual union all
select 691,'000691','1963-042C','deb USAF satellite/Agena D','',9273,'Reentered','1963-11-27 00:00:00','1963-11-19 00:00:00',91.23,277,393,89.98,'Polar' from dual union all
select 6910,'006910','1973-081B','Altair 2 X-258E5 ABL-150','X-258E5',30683,'In Earth orbit','','1973-11-30 00:00:00',105.52,896,1142,90.18,'Polar' from dual union all
select 6911,'006911','1973-082A','Interkosmos-10','DS-U2-IK No. 3',30684,'Reentered','1977-07-01 00:00:00','1973-12-02 00:00:00',101.88,257,1439,74.02,'Intermediate' from dual union all
select 6912,'006912','1973-082B','S3M','S3M',30684,'Reentered','1977-10-08 00:00:00','1973-12-03 00:00:00',101.81,252,1437,74.03,'Intermediate' from dual union all
select 6913,'006913','1973-083A','Kosmos-605','Bion No. 1',30685,'Landed','1973-11-22 00:00:00','1973-11-07 00:00:00',90.61,212,397,62.81,'Intermediate' from dual union all
select 6914,'006914','1973-083B','11S510','11S510',30685,'Reentered','1973-11-28 00:00:00','1973-11-07 00:00:00',90.28,208,369,62.79,'Intermediate' from dual union all
select 6915,'006915','1973-082C','deb Interkosmos-10','',30684,'Reentered','1973-12-07 00:00:00','1973-11-06 00:00:00',101.19,255,1375,73.98,'Intermediate' from dual union all
select 6916,'006916','1973-084A','Kosmos-606','Oko',30686,'In Earth orbit','','1973-12-06 00:00:00',717.6,659,39687,62.77,'Molniya' from dual union all
select 6917,'006917','1973-084B','11S510','11S510',30686,'Reentered','1974-01-06 00:00:00','1973-12-02 00:00:00',91.19,210,456,62.81,'Intermediate' from dual union all
select 6918,'006918','1973-084C','BOZ','BOZ',30686,'Reentered','1974-01-29 00:00:00','1973-12-02 00:00:00',91.92,214,523,62.79,'Intermediate' from dual union all
select 6919,'006919','1973-085A','Mariner 10','Mariner 73J',30687,'Deep Space','1973-11-03 00:00:00','1973-11-03 00:00:00',532.92,189,56493,28.9,'Earth Escape' from dual union all
select 692,'000692','1963-039C','Vela 1B','Vela 1B',9270,'Deep Space','1980-01-01 00:00:00','1964-03-30 00:00:00',6519.62,101925,116528,37.82,'Very High' from dual union all
select 6920,'006920','1973-086A','NOAA 3','ITOS F',30688,'In Earth orbit','','1973-12-06 00:00:00',116.12,1501,1512,102.07,'Sun-Synch' from dual union all
select 6921,'006921','1973-086B','Delta 98','Delta DSV-3E-5 S/N 20009',30688,'In Earth orbit','','1973-11-13 00:00:00',116.1,1501,1510,102.06,'Sun-Synch' from dual union all
select 6922,'006922','1973-075N','deb Kosmos-601','',30677,'Reentered','1973-11-19 00:00:00','1973-11-05 00:00:00',95.12,184,865,81.69,'Intermediate' from dual union all
select 6923,'006923','1973-075P','deb Kosmos-601','',30677,'Reentered','1973-12-04 00:00:00','1973-11-05 00:00:00',94.86,166,858,81.77,'Intermediate' from dual union all
select 6924,'006924','1973-079E','Blenda','BZA',30681,'Reentered','1973-11-10 00:00:00','1973-11-08 00:00:00',88.9,170,271,72.86,'Intermediate' from dual union all
select 6925,'006925','1973-079F','KDU','KDU',30681,'Reentered','1973-11-13 00:00:00','1973-11-08 00:00:00',89.01,171,280,72.85,'Intermediate' from dual union all
select 6926,'006926','1973-087A','Kosmos-607','Zenit-4MK',30689,'Landed','1973-11-22 00:00:00','1973-11-12 00:00:00',89.91,204,336,72.83,'Intermediate' from dual;
insert into SATELLITE
select 6927,'006927','1973-087B','11A57I','11A57I',30689,'Reentered','1973-11-23 00:00:00','1973-11-12 00:00:00',89.71,201,320,72.82,'Intermediate' from dual union all
select 6928,'006928','1973-088A','OPS 6630','HEXAGON SV-7',30690,'Deorbited','1974-03-13 00:00:00','1973-12-10 00:00:00',88.72,160,263,96.91,'Sun-Synch' from dual union all
select 6929,'006929','1973-088B','Titan 23D-8 Stage 2','LR91AJ11 23D-8',30690,'Reentered','1973-11-13 00:00:00','1973-11-12 00:00:00',88.14,146,219,97,'Sun-Synch' from dual union all
select 693,'000693','1963-046A','IMP 1','IMP A',9277,'Deep Space','1963-11-27 00:00:00','1964-04-03 00:00:00',5599.48,2071,194079,35.29,'Deep Space' from dual union all
select 6930,'006930','1973-087C','Sensor cover','Zenit-4MK sensor cover',30689,'Reentered','1973-11-15 00:00:00','1973-11-11 00:00:00',88.82,195,238,72.81,'Intermediate' from dual union all
select 6931,'006931','1973-088C','URSALA II','P-11 No. 4426',30690,'Reentered','1978-12-26 00:00:00','1974-01-17 00:00:00',94.49,485,503,96.32,'Sun-Synch' from dual union all
select 6932,'006932','1973-089A','Molniya-1','Molniya-1 11F67 No. 32',30691,'Reentered','1979-05-26 00:00:00','1973-12-17 00:00:00',717.9,438,39923,64.97,'GEO Transfer' from dual union all
select 6933,'006933','1973-089B','11S510','11S510',30691,'Reentered','1974-01-02 00:00:00','1973-11-23 00:00:00',91.07,230,425,64.96,'Intermediate' from dual union all
select 6934,'006934','1973-089C','deb Blok-I','',30691,'Reentered','1973-11-26 00:00:00','1973-11-19 00:00:00',91.03,225,426,64.96,'Intermediate' from dual union all
select 6935,'006935','1973-089D','BOZ','BOZ',30691,'Reentered','1974-01-26 00:00:00','1973-12-16 00:00:00',90.98,229,417,64.91,'Intermediate' from dual union all
select 6936,'006936','1973-090A','Skylab SL-4','CM-118',30692,'Landed','1974-02-08 00:00:00','1974-01-21 00:00:00',93.21,420,443,50.04,'Intermediate' from dual union all
select 6937,'006937','1973-090B','Saturn S-IVB-208','Saturn S-IVB-208',30692,'Deorbited','1973-11-16 00:00:00','1973-11-16 00:00:00',88.26,150,227,50.05,'Intermediate' from dual union all
select 6938,'006938','1973-088D','OPS 6630 P/L 2','P801-5',30690,'In Earth orbit','','1974-02-09 00:00:00',114.59,1417,1457,96.94,'Sun-Synch' from dual union all
select 6939,'006939','1973-084D','Blok-L','Blok-L',30686,'Reentered','2015-06-29 00:00:00','1973-12-07 00:00:00',706.6,655,39147,62.75,'Molniya' from dual union all
select 694,'000694','1963-047A','AC-2 Instrumentation Ring','AC-2',9278,'In Earth orbit','','1989-08-18 00:00:00',105.36,469,1554,30.3,'Intermediate' from dual union all
select 6940,'006940','1973-089E','Blok-L','Blok-L',30691,'Reentered','1979-08-20 00:00:00','1973-12-14 00:00:00',698.67,435,38973,64.95,'GEO Transfer' from dual union all
select 6941,'006941','1973-091A','Kosmos-608','DS-P1-Yu No. 69',30693,'Reentered','1974-07-10 00:00:00','1973-12-22 00:00:00',92.05,265,486,70.97,'Intermediate' from dual union all
select 6942,'006942','1973-091B','S1M','S1M',30693,'Reentered','1974-03-23 00:00:00','1973-12-20 00:00:00',91.69,268,447,70.97,'Intermediate' from dual union all
select 6943,'006943','1973-092A','Kosmos-609','Zenit-4M',30694,'Landed','1973-12-04 00:00:00','1973-11-23 00:00:00',89.61,173,338,69.94,'Intermediate' from dual union all
select 6944,'006944','1973-092B','11A57I','11A57I',30694,'Reentered','1973-12-04 00:00:00','1973-11-23 00:00:00',89.73,205,318,69.95,'Intermediate' from dual union all
select 6945,'006945','1973-087D','KDU','KDU',30689,'Reentered','1973-11-28 00:00:00','1973-11-24 00:00:00',89.08,170,288,72.82,'Intermediate' from dual union all
select 6946,'006946','1973-087E','Blenda','Sunshade',30689,'Reentered','1973-11-23 00:00:00','1973-11-22 00:00:00',88.96,141,306,72.83,'Intermediate' from dual union all
select 6947,'006947','1973-083C','KB electropitaniya','KBE',30685,'Reentered','1974-01-16 00:00:00','1973-11-26 00:00:00',90.19,155,413,62.26,'Intermediate' from dual union all
select 6948,'006948','1973-083D','KBE part','KBE part',30685,'Reentered','1973-12-05 00:00:00','1973-11-24 00:00:00',90.24,207,365,62.81,'Intermediate' from dual union all
select 6949,'006949','1973-083E','KBE part','KBE part',30685,'Reentered','1974-01-09 00:00:00','1973-11-30 00:00:00',90.22,207,364,62.8,'Intermediate' from dual union all
select 695,'000695','1963-048A','OPS 2260','CORONA M-25',9279,'Reentered Attached','1963-12-15 00:00:00','1963-12-05 00:00:00',89.77,178,349,69.94,'Intermediate' from dual union all
select 6950,'006950','1973-093A','Kosmos-610','Tselina-OM',30695,'Reentered','1980-09-15 00:00:00','1973-12-28 00:00:00',95.21,513,545,74.04,'Intermediate' from dual union all
select 6951,'006951','1973-093B','S3M','S3M',30695,'Reentered','1980-11-06 00:00:00','1973-12-27 00:00:00',95.02,496,543,74.04,'Intermediate' from dual union all
select 6952,'006952','1973-094A','Kosmos-611','DS-P1-Yu No. 64',30696,'Reentered','1974-06-19 00:00:00','1973-12-30 00:00:00',91.84,264,466,70.97,'Intermediate' from dual union all
select 6953,'006953','1973-095A','Kosmos-612','Zenit-4MK',30697,'Landed','1973-12-11 00:00:00','1973-11-30 00:00:00',89.99,203,345,72.84,'Intermediate' from dual union all
select 6954,'006954','1973-095B','11A57I','11A57I',30697,'Reentered','1973-12-12 00:00:00','1973-11-30 00:00:00',89.8,201,328,72.84,'Intermediate' from dual union all
select 6955,'006955','1973-094B','S1M','S1M',30696,'Reentered','1974-03-26 00:00:00','1973-12-29 00:00:00',91.48,265,430,70.96,'Intermediate' from dual union all
select 6956,'006956','1973-083F','PTDU cover','KBE part',30685,'Reentered','1973-12-30 00:00:00','1973-12-01 00:00:00',91.99,306,439,62.8,'Intermediate' from dual union all
select 6957,'006957','1973-096A','Kosmos-613','Soyuz 7K-T No. 34A',30698,'Landed','1974-01-29 00:00:00','1973-12-31 00:00:00',90.86,251,382,51.59,'Intermediate' from dual union all
select 6958,'006958','1973-097A','Molniya-1','Molniya-1',30699,'Reentered','1985-06-09 00:00:00','1974-01-15 00:00:00',717.72,478,39874,62.7,'Molniya' from dual union all
select 6959,'006959','1973-097B','11S510','11S510',30699,'Reentered','1973-12-30 00:00:00','1973-12-08 00:00:00',90.5,212,387,62.79,'Intermediate' from dual union all
select 696,'000696','1963-047B','AC Short PLF','Centaur D PLF',9278,'Reentered','1992-03-20 00:00:00','1963-12-30 00:00:00',107.33,586,1621,30.08,'Intermediate' from dual union all
select 6960,'006960','1973-096B','11S510','11S510',30698,'Reentered','1973-12-04 00:00:00','1973-12-02 00:00:00',88.73,182,241,51.59,'Intermediate' from dual union all
select 6961,'006961','1973-097C','BOZ','BOZ',30699,'Reentered','1973-12-31 00:00:00','1973-12-07 00:00:00',90.67,203,411,62.81,'Intermediate' from dual union all
select 6962,'006962','1973-093C','deb Kosmos-610','Tselina-O part',30695,'Reentered','1975-06-29 00:00:00','1973-12-27 00:00:00',94.94,489,542,73.96,'Intermediate' from dual union all
select 6963,'006963','1973-092C','KDU','KDU',30694,'Reentered','1973-12-06 00:00:00','1973-12-03 00:00:00',88.88,171,268,69.96,'Intermediate' from dual union all
select 6964,'006964','1973-092D','Blenda','BZA',30694,'Reentered','1973-12-05 00:00:00','1973-12-03 00:00:00',88.73,168,256,69.94,'Intermediate' from dual union all
select 6965,'006965','1973-098A','Kosmos-614','Strela-2M',30700,'In Earth orbit','','1974-01-08 00:00:00',100.6,768,805,74.06,'Intermediate' from dual union all
select 6966,'006966','1973-098B','S3M','S3M',30700,'In Earth orbit','','1974-01-03 00:00:00',100.48,766,797,74.06,'Intermediate' from dual union all
select 6967,'006967','1973-098C','deb Kosmos-614','Strela-2M debris',30700,'Reentered','2005-10-11 00:00:00','1974-01-27 00:00:00',100.72,774,812,74.08,'Intermediate' from dual union all
select 6968,'006968','1973-095C','KDU','KDU',30697,'Reentered','1973-12-16 00:00:00','1973-12-10 00:00:00',89.11,166,294,72.84,'Intermediate' from dual union all
select 6969,'006969','1973-095D','Blenda','Sunshade',30697,'Reentered','1973-12-12 00:00:00','1973-12-11 00:00:00',88.63,161,252,72.84,'Intermediate' from dual union all
select 697,'000697','1963-047C','Insulation blanket','Insulation panel',9278,'Reentered','1989-06-24 00:00:00','1964-01-08 00:00:00',107.58,588,1642,30.07,'Intermediate' from dual union all
select 6970,'006970','1973-095E','KDU part','KDU part',30697,'Reentered','1973-12-11 00:00:00','1973-12-11 00:00:00',88.59,160,250,72.84,'Intermediate' from dual union all
select 6971,'006971','1973-099A','Kosmos-615','DS-P1-I No. 13',30701,'Reentered','1975-12-17 00:00:00','1974-01-25 00:00:00',95.49,267,817,71.01,'Intermediate' from dual union all
select 6972,'006972','1973-099B','S1M','S1M',30701,'Reentered','1975-01-24 00:00:00','1974-01-17 00:00:00',95.33,275,794,71.02,'Intermediate' from dual union all
select 6973,'006973','1973-100A','DSCS II F-3','DSCS II B-3',30702,'In Earth orbit','','1977-01-18 00:00:00',1436.09,35752,35821,.31,'Stationary' from dual union all
select 6974,'006974','1973-100B','DSCS II F-4','DSCS II B-4',30702,'In Earth orbit','','1977-01-22 00:00:00',1435.91,35623,35943,.44,'Stationary' from dual union all
select 6975,'006975','1973-100C','Titan 23C-8 Stage 2','LR91AJ11 23C-8',30702,'Reentered','1973-12-17 00:00:00','1973-12-16 00:00:00',89.1,149,311,28.59,'Intermediate' from dual union all
select 6976,'006976','1973-100D','Transtage 26','Transtage 26',30702,'In Earth orbit','','1977-01-30 00:00:00',1514.96,36071,38562,.46,'Very High' from dual union all
select 6977,'006977','1973-101A','AE-C','AE-C',30703,'Reentered','1978-12-12 00:00:00','1974-01-29 00:00:00',129.78,152,4074,68.07,'Medium' from dual union all
select 6978,'006978','1973-101B','Delta 99','Delta DSV-3E-5',30703,'Reentered','1974-08-14 00:00:00','1974-01-15 00:00:00',128.23,154,3937,68.06,'Medium' from dual union all
select 6979,'006979','1973-102A','Kosmos-616','Zenit-4MT',30704,'Landed','1973-12-28 00:00:00','1973-12-19 00:00:00',89.84,204,330,72.86,'Intermediate' from dual union all
select 698,'000698','1963-047D','Insulation blanket','Insulation panel',9278,'In Earth orbit','','1964-01-02 00:00:00',107.98,634,1633,29.94,'Intermediate' from dual union all
select 6980,'006980','1973-102B','11S510','11S510',30704,'Reentered','1973-12-31 00:00:00','1973-12-20 00:00:00',89.61,195,315,72.86,'Intermediate' from dual union all
select 6981,'006981','1973-102C','Sensor cover','Zenit-4MT sensor cover',30704,'Reentered','1974-01-14 00:00:00','1973-12-19 00:00:00',89.36,179,307,72.89,'Intermediate' from dual union all
select 6982,'006982','1973-103A','Soyuz-13','Soyuz 7K-T No. 33',30705,'Landed','1973-12-26 00:00:00','1973-12-20 00:00:00',89.24,221,253,51.57,'Intermediate' from dual union all
select 6983,'006983','1973-103B','11S510','11S510',30705,'Reentered','1973-12-22 00:00:00','1973-12-20 00:00:00',88.41,167,224,51.56,'Intermediate' from dual union all
select 6984,'006984','1973-103C','Orion-2 telescope cover','Orion-2 telescope cover',30705,'Reentered','1974-01-14 00:00:00','1973-12-18 00:00:00',88.94,163,282,51.56,'Intermediate' from dual union all
select 6985,'006985','1973-104A','Kosmos-617','Strela-1M',30706,'In Earth orbit','','1974-02-03 00:00:00',113.99,1335,1484,74.03,'Intermediate' from dual union all
select 6986,'006986','1973-104B','Kosmos-618','Strela-1M',30706,'In Earth orbit','','1974-01-28 00:00:00',115.23,1445,1487,74.03,'Intermediate' from dual union all
select 6987,'006987','1973-104C','Kosmos-619','Strela-1M',30706,'In Earth orbit','','1974-02-12 00:00:00',115.01,1425,1486,74.03,'Intermediate' from dual union all
select 6988,'006988','1973-104D','Kosmos-620','Strela-1M',30706,'In Earth orbit','','1974-01-20 00:00:00',115.46,1458,1495,74.03,'Intermediate' from dual union all
select 6989,'006989','1973-104E','Kosmos-621','Strela-1M',30706,'In Earth orbit','','1974-02-11 00:00:00',114.79,1407,1485,74.03,'Intermediate' from dual union all
select 699,'000699','1963-047E','Insulation blanket','Insulation panel',9278,'Reentered','1985-03-04 00:00:00','1985-03-01 00:00:00',89.93,258,284,30.41,'Intermediate' from dual union all
select 6990,'006990','1973-104F','Kosmos-622','Strela-1M',30706,'In Earth orbit','','1974-02-12 00:00:00',114.39,1371,1484,74.03,'Intermediate' from dual union all
select 6991,'006991','1973-104G','Kosmos-623','Strela-1M',30706,'In Earth orbit','','1974-01-30 00:00:00',114.58,1388,1485,74.02,'Intermediate' from dual union all
select 6992,'006992','1973-104H','Kosmos-624','Strela-1M',30706,'In Earth orbit','','1974-01-30 00:00:00',114.18,1352,1485,74.03,'Intermediate' from dual union all
select 6993,'006993','1973-104J','S3M','S3M',30706,'In Earth orbit','','1974-02-13 00:00:00',117.08,1475,1625,74.03,'Intermediate' from dual union all
select 6994,'006994','1973-102D','KBE part','KBE part',30704,'Reentered','1974-01-02 00:00:00','1973-12-18 00:00:00',89.39,197,292,72.85,'Intermediate' from dual union all
select 6995,'006995','1973-105A','Kosmos-625','Zenit-4MK',30707,'Landed','1974-01-03 00:00:00','1973-12-23 00:00:00',89.95,203,341,72.84,'Intermediate' from dual union all
select 6996,'006996','1973-105B','11A57I','11A57I',30707,'Reentered','1974-01-01 00:00:00','1973-12-23 00:00:00',89.5,198,302,72.83,'Intermediate' from dual union all
select 6997,'006997','1973-105C','Kosmos-625 sensor cover','',30707,'Reentered','1973-12-22 00:00:00','1973-12-21 00:00:00',89.3,200,280,72.8,'Intermediate' from dual union all
select 6998,'006998','1973-060D','deb Kosmos-582','Tselina-O part',30662,'Reentered','1975-03-05 00:00:00','1973-12-25 00:00:00',94.68,483,524,74.11,'Intermediate' from dual union all
select 6999,'006999','1973-060E','deb Kosmos-582','Tselina-O part',30662,'Reentered','1974-11-07 00:00:00','1973-12-25 00:00:00',94.62,479,522,73.96,'Intermediate' from dual union all
select 7,'000007','58 Delta 1','8K74A','8K74A',4794,'Reentered','1958-12-03 00:00:00','1958-05-15 00:00:00',105.91,214,1860,65.18,'Intermediate' from dual union all
select 70,'000070','61 Alpha 1','Samos 2 F-1','SAMOS F-1 No. 2',7260,'Reentered','1973-10-21 00:00:00','1961-03-02 00:00:00',94.89,474,553,97.4,'Sun-Synch' from dual union all
select 700,'000700','1963-047F','Insulation blanket','Insulation panel',9278,'In Earth orbit','','1993-03-29 00:00:00',107.9,573,1687,30.48,'Intermediate' from dual union all
select 7000,'007000','1973-106A','Molniya-2','Molniya-2',30708,'Reentered','1984-11-24 00:00:00','1974-01-27 00:00:00',718.02,436,39931,62.88,'Molniya' from dual union all
select 7001,'007001','1973-106B','11S510','11S510',30708,'Reentered','1974-01-24 00:00:00','1974-01-01 00:00:00',90.73,207,414,62.83,'Intermediate' from dual union all
select 7002,'007002','1973-106C','BOZ','BOZ',30708,'Reentered','1974-01-17 00:00:00','1974-01-01 00:00:00',90.47,189,407,62.83,'Intermediate' from dual union all
select 7003,'007003','1973-107A','Aureole 2','DS-U2-GKA No. 2',30709,'In Earth orbit','','1974-01-28 00:00:00',109.09,399,1971,74,'Intermediate' from dual union all
select 7004,'007004','1973-107B','S3M','S3M',30709,'In Earth orbit','','1974-02-01 00:00:00',108.97,389,1970,74,'Intermediate' from dual union all
select 7005,'007005','1973-108A','Kosmos-626','US-A reactor',30710,'In Earth orbit','','1974-01-30 00:00:00',89.63,250,262,65.01,'Intermediate' from dual union all
select 7006,'007006','1973-102E','KB electropitaniya','KBE',30704,'Reentered','1974-01-02 00:00:00','1973-12-28 00:00:00',89.46,196,300,72.86,'Intermediate' from dual union all
select 7007,'007007','1973-102F','Blenda','KBE part',30704,'Reentered','1974-01-02 00:00:00','1973-12-28 00:00:00',89.51,197,303,72.83,'Intermediate' from dual union all
select 7008,'007008','1973-109A','Kosmos-627','Zaliv',30711,'In Earth orbit','','1974-02-10 00:00:00',105,971,1018,82.96,'Intermediate' from dual union all
select 7009,'007009','1973-109B','S3M','S3M',30711,'In Earth orbit','','1974-02-13 00:00:00',104.84,963,1011,82.95,'Intermediate' from dual union all
select 701,'000701','1963-047G','Insulation blanket','Insulation panel',9278,'In Earth orbit','','1993-03-30 00:00:00',105.69,595,1459,30.01,'Intermediate' from dual union all
select 7010,'007010','1973-105D','Blenda','Sunshade',30707,'Reentered','1974-01-02 00:00:00','1973-12-30 00:00:00',89.17,159,307,72.83,'Intermediate' from dual union all
select 7011,'007011','1973-105E','KDU','KDU',30707,'Reentered','1974-01-06 00:00:00','1973-12-30 00:00:00',89.31,164,317,72.81,'Intermediate' from dual union all
select 7012,'007012','1973-086C','deb Delta 98','',30688,'In Earth orbit','','1974-01-02 00:00:00',115.56,1431,1530,102.1,'Sun-Synch' from dual union all
select 7013,'007013','1973-086D','deb Delta 98','',30688,'In Earth orbit','','1974-01-02 00:00:00',115.21,1400,1530,101.83,'Sun-Synch' from dual union all
select 7014,'007014','1973-086E','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',109.04,872,1493,101.74,'Sun-Synch' from dual union all
select 7015,'007015','1973-086F','deb Delta 98','',30688,'In Earth orbit','','1992-02-02 00:00:00',123.49,1507,2166,101.74,'Medium' from dual;
insert into SATELLITE
select 7016,'007016','1973-086G','deb Delta 98','',30688,'In Earth orbit','','1992-02-04 00:00:00',125.77,1494,2381,101.71,'Medium' from dual union all
select 7017,'007017','1973-086H','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',116.41,1508,1531,102.37,'Sun-Synch' from dual union all
select 7018,'007018','1973-086J','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',116.29,1502,1526,102.31,'Sun-Synch' from dual union all
select 7019,'007019','1973-086K','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',115.98,1445,1555,102.13,'Sun-Synch' from dual union all
select 702,'000702','1963-014H','West Ford package section','RCS 1.0',9245,'In Earth orbit','','1993-03-25 00:00:00',166.35,3425,3858,87.31,'Medium' from dual union all
select 7020,'007020','1973-086L','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',118.64,1490,1750,102.12,'Sun-Synch' from dual union all
select 7021,'007021','1973-086M','deb Delta 98','',30688,'In Earth orbit','','1992-01-31 00:00:00',117.94,1428,1749,101.67,'Sun-Synch' from dual union all
select 7022,'007022','1973-086N','deb Delta 98','',30688,'In Earth orbit','','1992-02-04 00:00:00',115.8,1339,1645,102.3,'Sun-Synch' from dual union all
select 7023,'007023','1973-086P','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',116.58,1505,1549,102.4,'Sun-Synch' from dual union all
select 7024,'007024','1973-086Q','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',113.44,1355,1413,101.9,'Sun-Synch' from dual union all
select 7025,'007025','1973-086R','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',124.74,1507,2277,101.95,'Medium' from dual union all
select 7026,'007026','1973-086S','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',114.7,1353,1531,102.08,'Sun-Synch' from dual union all
select 7027,'007027','1973-086T','deb Delta 98','',30688,'In Earth orbit','','1992-01-24 00:00:00',117.08,1498,1601,101.76,'Sun-Synch' from dual union all
select 7028,'007028','1973-086U','deb Delta 98','',30688,'In Earth orbit','','1992-01-27 00:00:00',117.63,1510,1639,102.67,'Sun-Synch' from dual union all
select 7029,'007029','1973-086V','deb Delta 98','',30688,'In Earth orbit','','1992-01-19 00:00:00',116.4,1509,1529,102.22,'Sun-Synch' from dual union all
select 703,'000703','1963-049A','Ablestar 015','AJ10-104D',9280,'In Earth orbit','','1964-03-18 00:00:00',106.79,1069,1087,89.95,'Polar' from dual union all
select 7030,'007030','1973-086W','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',117.71,1506,1649,102.48,'Sun-Synch' from dual union all
select 7031,'007031','1973-086X','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',116.3,1505,1523,101.84,'Sun-Synch' from dual union all
select 7032,'007032','1973-086Y','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',116.64,1507,1553,102.25,'Sun-Synch' from dual union all
select 7033,'007033','1973-086Z','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',116.01,1480,1522,101.59,'Sun-Synch' from dual union all
select 7034,'007034','1973-086AA','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',115.85,1480,1508,101.86,'Sun-Synch' from dual union all
select 7035,'007035','1973-086AB','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',121.87,1503,2026,101.79,'Medium' from dual union all
select 7036,'007036','1973-086AC','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',115.29,1434,1504,102.1,'Sun-Synch' from dual union all
select 7037,'007037','1973-086AD','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',121.41,1481,2007,102.03,'Medium' from dual union all
select 7038,'007038','1973-086AE','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',117.84,1483,1685,101.92,'Sun-Synch' from dual union all
select 7039,'007039','1973-086AF','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',115.38,1367,1579,102.52,'Sun-Synch' from dual union all
select 704,'000704','1963-049B','Transit VBN-2','Transit VBN-2',9280,'In Earth orbit','','1964-01-07 00:00:00',107.02,1064,1114,89.97,'Polar' from dual union all
select 7040,'007040','1973-086AG','deb Delta 98','',30688,'In Earth orbit','','1992-02-07 00:00:00',112.57,1193,1497,102.49,'Sun-Synch' from dual union all
select 7041,'007041','1973-086AH','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',117.78,1429,1733,101.59,'Sun-Synch' from dual union all
select 7042,'007042','1973-086AJ','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',108.13,878,1402,101.71,'Sun-Synch' from dual union all
select 7043,'007043','1973-086AK','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',112.22,1157,1501,102.08,'Sun-Synch' from dual union all
select 7044,'007044','1973-086AL','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',115.76,1423,1557,102.56,'Sun-Synch' from dual union all
select 7045,'007045','1973-086AM','deb Delta 98','',30688,'In Earth orbit','','1992-02-04 00:00:00',117.49,1509,1627,102.53,'Sun-Synch' from dual union all
select 7046,'007046','1973-086AN','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',117.17,1508,1600,102.45,'Sun-Synch' from dual union all
select 7047,'007047','1973-086AP','deb Delta 98','',30688,'Reentered','1996-11-15 00:00:00','1992-02-06 00:00:00',110.07,1037,1423,102.51,'Sun-Synch' from dual union all
select 7048,'007048','1973-086AQ','deb Delta 98','',30688,'In Earth orbit','','1992-01-20 00:00:00',125.15,1505,2315,101.64,'Medium' from dual union all
select 7049,'007049','1973-086AR','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',120.75,1513,1916,101.83,'Medium' from dual union all
select 705,'000705','1963-049C','Transit VE-3','Transit VE-3',9280,'In Earth orbit','','1964-11-14 00:00:00',107.1,1066,1118,89.95,'Polar' from dual union all
select 7050,'007050','1973-086AS','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',115.1,1433,1487,101.75,'Sun-Synch' from dual union all
select 7051,'007051','1973-086AT','deb Delta 98','',30688,'In Earth orbit','','1992-02-01 00:00:00',117.24,1504,1610,102.51,'Sun-Synch' from dual union all
select 7052,'007052','1973-086AU','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',116.44,1504,1537,102.36,'Sun-Synch' from dual union all
select 7053,'007053','1973-086AV','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',115.53,1444,1515,101.47,'Sun-Synch' from dual union all
select 7054,'007054','1973-086AW','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',110.6,1013,1496,101.53,'Sun-Synch' from dual union all
select 7055,'007055','1973-086AX','deb Delta 98','',30688,'In Earth orbit','','1992-02-02 00:00:00',121.63,1472,2036,102.58,'Medium' from dual union all
select 7056,'007056','1973-086AY','deb Delta 98','',30688,'In Earth orbit','','1992-01-31 00:00:00',117.69,1508,1646,102.7,'Sun-Synch' from dual union all
select 7057,'007057','1973-086AZ','deb Delta 98','',30688,'In Earth orbit','','1992-01-16 00:00:00',116.79,1510,1563,102.35,'Sun-Synch' from dual union all
select 7058,'007058','1973-086BA','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',115.33,1456,1485,101.89,'Sun-Synch' from dual union all
select 7059,'007059','1973-086BB','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',117.81,1499,1665,101.98,'Sun-Synch' from dual union all
select 706,'000706','1963-049D','Transit VBN-2 adapter','',9280,'In Earth orbit','','1993-03-29 00:00:00',106.54,1043,1090,90.07,'Polar' from dual union all
select 7060,'007060','1973-086BC','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',115.75,1470,1509,102.1,'Sun-Synch' from dual union all
select 7061,'007061','1973-086BD','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',114.88,1341,1558,102.45,'Sun-Synch' from dual union all
select 7062,'007062','1973-086BE','deb Delta 98','',30688,'Reentered','1982-09-10 00:00:00','1982-08-03 00:00:00',100.32,712,835,103.92,'Retrograde' from dual union all
select 7063,'007063','1973-086BF','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',110.9,1036,1501,102.43,'Sun-Synch' from dual union all
select 7064,'007064','1973-086BG','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',113.06,1234,1501,101.73,'Sun-Synch' from dual union all
select 7065,'007065','1973-086BH','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',117.03,1495,1599,101.94,'Sun-Synch' from dual union all
select 7066,'007066','1973-086BJ','deb Delta 98','',30688,'In Earth orbit','','1992-02-04 00:00:00',116.5,1464,1582,101.78,'Sun-Synch' from dual union all
select 7067,'007067','1973-086BK','deb Delta 98','',30688,'Reentered','1982-09-26 00:00:00','1982-09-15 00:00:00',101.04,502,1114,102.94,'Retrograde' from dual union all
select 7068,'007068','1973-086BL','deb Delta 98','',30688,'In Earth orbit','','1992-02-04 00:00:00',112.47,1173,1507,102.38,'Sun-Synch' from dual union all
select 7069,'007069','1973-086BM','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',108.36,914,1388,102.8,'Sun-Synch' from dual union all
select 707,'000707','1963-050A','Kosmos-23','Omega-1 No. 2',9281,'Reentered','1964-03-27 00:00:00','1964-01-17 00:00:00',92.2,237,528,48.96,'Intermediate' from dual union all
select 7070,'007070','1973-086BN','deb Delta 98','',30688,'Reentered','1981-11-04 00:00:00','1974-02-03 00:00:00',105.72,548,1508,101.8,'Sun-Synch' from dual union all
select 7071,'007071','1973-086BP','deb Delta 98','',30688,'In Earth orbit','','1992-02-01 00:00:00',116.03,1497,1508,101.7,'Sun-Synch' from dual union all
select 7072,'007072','1973-086BQ','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',115.93,1488,1508,101.79,'Sun-Synch' from dual union all
select 7073,'007073','1973-086BR','deb Delta 98','',30688,'In Earth orbit','','1992-02-02 00:00:00',117.24,1506,1608,102.59,'Sun-Synch' from dual union all
select 7074,'007074','1973-086BS','deb Delta 98','',30688,'In Earth orbit','','1992-02-02 00:00:00',113.21,1237,1511,101.95,'Sun-Synch' from dual union all
select 7075,'007075','1973-086BT','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',112.71,1211,1492,101.62,'Sun-Synch' from dual union all
select 7076,'007076','1973-086BU','deb Delta 98','',30688,'In Earth orbit','','1992-02-01 00:00:00',116.18,1498,1519,102.11,'Sun-Synch' from dual union all
select 7077,'007077','1973-086BV','deb Delta 98','',30688,'Reentered','1987-04-02 00:00:00','1974-01-15 00:00:00',119.3,1484,1815,101.75,'Sun-Synch' from dual union all
select 7078,'007078','1973-086BW','deb Delta 98','',30688,'Reentered','2004-12-10 00:00:00','1992-02-07 00:00:00',117.97,1156,2024,102.33,'Sun-Synch' from dual union all
select 7079,'007079','1973-086BX','deb Delta 98','',30688,'In Earth orbit','','1992-02-04 00:00:00',116.72,1507,1560,101.92,'Sun-Synch' from dual union all
select 708,'000708','1963-050B','S1','S1',9281,'Reentered','1964-03-06 00:00:00','1964-02-19 00:00:00',90.54,210,392,49.08,'Intermediate' from dual union all
select 7080,'007080','1973-086BY','deb Delta 98','',30688,'In Earth orbit','','1992-02-01 00:00:00',115.12,1424,1498,101.97,'Sun-Synch' from dual union all
select 7081,'007081','1973-086BZ','deb Delta 98','',30688,'In Earth orbit','','1992-02-07 00:00:00',113.56,1263,1517,102.34,'Sun-Synch' from dual union all
select 7082,'007082','1973-086CA','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',114.95,1423,1483,101.8,'Sun-Synch' from dual union all
select 7083,'007083','1973-086CB','deb Delta 98','',30688,'In Earth orbit','','1992-02-02 00:00:00',117.01,1492,1600,102.41,'Sun-Synch' from dual union all
select 7084,'007084','1973-086CC','deb Delta 98','',30688,'In Earth orbit','','1992-02-07 00:00:00',117.55,1447,1695,101.87,'Sun-Synch' from dual union all
select 7085,'007085','1973-086CD','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',114.49,1359,1506,102.07,'Sun-Synch' from dual union all
select 7086,'007086','1973-086CE','deb Delta 98','',30688,'In Earth orbit','','1992-01-26 00:00:00',116.17,1499,1518,101.98,'Sun-Synch' from dual union all
select 7087,'007087','1973-086CF','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',113.93,1095,1718,102.29,'Sun-Synch' from dual union all
select 7088,'007088','1973-086CG','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',112.89,1142,1576,102.37,'Sun-Synch' from dual union all
select 7089,'007089','1973-086CH','deb Delta 98','',30688,'Reentered','1997-03-06 00:00:00','1992-02-06 00:00:00',106.08,980,1109,102.67,'Sun-Synch' from dual union all
select 709,'000709','1963-050C','deb Kosmos-23','',9281,'Reentered','1964-01-03 00:00:00','1963-12-14 00:00:00',93.1,240,613,49,'Intermediate' from dual union all
select 7090,'007090','1973-086CJ','deb Delta 98','',30688,'Reentered','1982-01-27 00:00:00','1982-01-27 00:00:00',93.78,441,478,101.23,'Retrograde' from dual union all
select 7091,'007091','1973-086CK','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',120.47,1467,1937,102.17,'Medium' from dual union all
select 7092,'007092','1973-086CL','deb Delta 98','',30688,'In Earth orbit','','1992-02-02 00:00:00',110.53,1003,1500,101.87,'Sun-Synch' from dual union all
select 7093,'007093','1973-086CM','deb Delta 98','',30688,'In Earth orbit','','1992-02-01 00:00:00',117.39,1508,1619,102.57,'Sun-Synch' from dual union all
select 7094,'007094','1974-001A','Kosmos-628','Zaliv',33604,'In Earth orbit','','1974-02-16 00:00:00',104.79,956,1013,82.95,'Intermediate' from dual union all
select 7095,'007095','1974-001B','S3M','S3M',33604,'In Earth orbit','','1974-02-17 00:00:00',104.62,950,1004,82.95,'Intermediate' from dual union all
select 7096,'007096','1974-002A','Skynet IIA','Skynet IIA',33605,'Reentered','1974-01-25 00:00:00','1974-01-21 00:00:00',113.62,124,2662,37.65,'Intermediate' from dual union all
select 7097,'007097','1974-002B','Delta 100','Delta SSPS DSV-3P-5',33605,'Reentered','1974-01-20 00:00:00','1974-01-19 00:00:00',88.19,185,185,28.7,'Intermediate' from dual union all
select 7098,'007098','1974-002C','Delta 100 clamp band','',33605,'Reentered','1974-01-23 00:00:00','1974-01-19 00:00:00',88.01,170,181,28.66,'Intermediate' from dual union all
select 7099,'007099','1974-002D','Delta 100 clamp band','',33605,'Reentered','1974-01-23 00:00:00','1974-01-20 00:00:00',88.1,161,200,28.63,'Intermediate' from dual union all
select 71,'000071','61 Beta 1','Tyazholiy Sputnik','1VA No. 1',7261,'Reentered Attached','1961-02-26 00:00:00','1961-02-16 00:00:00',89.25,179,295,64.9,'Intermediate' from dual union all
select 710,'000710','1963-050D','deb Kosmos-23','',9281,'Reentered','1964-01-01 00:00:00','1963-12-22 00:00:00',92,228,518,48.96,'Intermediate' from dual union all
select 7100,'007100','1974-003A','Kosmos-629','Zenit-2M',33606,'Landed','1974-02-05 00:00:00','1974-01-26 00:00:00',89.33,196,287,62.81,'Intermediate' from dual union all
select 7101,'007101','1974-003B','11A57I','11A57I',33606,'Reentered','1974-02-01 00:00:00','1974-01-28 00:00:00',88.83,181,253,62.78,'Intermediate' from dual union all
select 7102,'007102','1974-003C','Sensor cover','Zenit-2M sensor cover',33606,'Reentered','1974-01-26 00:00:00','1974-01-25 00:00:00',89.57,197,310,62.86,'Intermediate' from dual union all
select 7103,'007103','1973-096C','Kosmos-613 BO ','Bitovoy otsek',30698,'Reentered','1974-03-09 00:00:00','1974-02-01 00:00:00',90.45,238,356,51.59,'Intermediate' from dual union all
select 7104,'007104','1974-004A','Kosmos-630','Zenit-4MK',33607,'Landed','1974-02-13 00:00:00','1974-02-02 00:00:00',89.96,201,344,72.84,'Intermediate' from dual union all
select 7105,'007105','1974-004B','11A57I','11A57I',33607,'Reentered','1974-02-14 00:00:00','1974-02-08 00:00:00',89.25,189,286,72.85,'Intermediate' from dual;
insert into SATELLITE
select 7106,'007106','1974-004C','Sensor cover','Zenit-4MK sensor cover',33607,'Reentered','1974-02-06 00:00:00','1974-02-01 00:00:00',88.96,200,247,72.84,'Intermediate' from dual union all
select 7107,'007107','1974-004D','Sensor cover','Zenit-4MK sensor cover',33607,'Reentered','1974-02-05 00:00:00','1974-01-31 00:00:00',89.92,204,337,72.89,'Intermediate' from dual union all
select 7108,'007108','1974-003D','FEU-170 part','FEU-170 part',33606,'Reentered','1974-02-09 00:00:00','1974-02-01 00:00:00',89.15,192,273,62.81,'Intermediate' from dual union all
select 7109,'007109','1974-005A','Kosmos-631','Tselina-OM',33608,'Reentered','1980-10-03 00:00:00','1974-03-08 00:00:00',95.25,518,544,74.04,'Intermediate' from dual union all
select 711,'000711','1963-051A','OPS 2372','GAMBIT SV 954',9282,'Reentered','1963-12-20 00:00:00','1963-12-19 00:00:00',88.46,126,271,97.9,'Sun-Synch' from dual union all
select 7110,'007110','1974-005B','S3M','S3M',33608,'Reentered','1980-12-07 00:00:00','1974-03-10 00:00:00',95.13,510,540,74.04,'Intermediate' from dual union all
select 7111,'007111','1974-005C','deb Kosmos-631','Tselina-O part',33608,'Reentered','1978-07-02 00:00:00','1974-03-08 00:00:00',95.02,500,540,73.96,'Intermediate' from dual union all
select 7112,'007112','1974-005D','deb Kosmos-631','Tselina-O part',33608,'Reentered','1975-10-11 00:00:00','1974-03-09 00:00:00',94.9,487,541,74.09,'Intermediate' from dual union all
select 7113,'007113','1973-108B','deb Kosmos-626','',30710,'Reentered','1974-02-17 00:00:00','1974-02-11 00:00:00',89.24,225,249,65.04,'Intermediate' from dual union all
select 7114,'007114','1973-108C','deb Kosmos-626','',30710,'Reentered','1974-02-23 00:00:00','1974-02-11 00:00:00',89.76,232,294,65.02,'Intermediate' from dual union all
select 7115,'007115','1973-108D','Kosmos-626 DU','US-A DU',30710,'Reentered','1974-03-22 00:00:00','1974-02-13 00:00:00',89.52,241,261,65.01,'Intermediate' from dual union all
select 7116,'007116','1973-108E','Kosmos-626 RLS','',30710,'Reentered','1974-02-16 00:00:00','1974-02-11 00:00:00',89.51,242,259,65,'Intermediate' from dual union all
select 7117,'007117','1974-006A','Kosmos-632','Zenit-4M',33609,'Landed','1974-02-26 00:00:00','1974-02-14 00:00:00',89.28,175,303,65,'Intermediate' from dual union all
select 7118,'007118','1974-006B','11A57I','11A57I',33609,'Reentered','1974-02-17 00:00:00','1974-02-14 00:00:00',88.84,173,262,65,'Intermediate' from dual union all
select 7119,'007119','1974-004E','Blenda','Sunshade',33607,'Reentered','1974-02-14 00:00:00','1974-02-12 00:00:00',88.62,147,266,72.84,'Intermediate' from dual union all
select 712,'000712','1963-052A','Kosmos-24','Zenit-2 No. 15',9283,'Landed','1963-12-28 00:00:00','1963-12-23 00:00:00',90.43,199,393,65.03,'Intermediate' from dual union all
select 7120,'007120','1974-004F','KDU','KDU',33607,'Reentered','1974-02-16 00:00:00','1974-02-12 00:00:00',88.93,166,278,72.86,'Intermediate' from dual union all
select 7121,'007121','1974-007A','OPS 6889','GAMBIT-3 Mission 4341',33610,'Reentered Attached','1974-03-17 00:00:00','1974-02-20 00:00:00',89.73,129,394,110.42,'Retrograde' from dual union all
select 7122,'007122','1974-008A','Tansei-2','MS-T2',33611,'Reentered','1983-01-22 00:00:00','1974-03-18 00:00:00',121.61,283,3222,31.24,'Medium' from dual union all
select 7123,'007123','1974-008B','M-3A','M-3A',33611,'Reentered','1983-01-21 00:00:00','1974-03-20 00:00:00',121.95,281,3255,31.24,'Medium' from dual union all
select 7124,'007124','1973-086CN','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',117.07,1507,1592,102.46,'Sun-Synch' from dual union all
select 7125,'007125','1973-086CP','deb Delta 98','',30688,'Reentered','1991-01-26 00:00:00','1991-01-26 00:00:00',91.91,343,394,102.7,'Retrograde' from dual union all
select 7126,'007126','1973-086CQ','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',116.54,1295,1756,102.08,'Sun-Synch' from dual union all
select 7127,'007127','1973-086CR','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',120.01,1469,1894,101.96,'Medium' from dual union all
select 7128,'007128','1973-086CS','deb Delta 98','',30688,'In Earth orbit','','1992-01-31 00:00:00',114.77,1430,1460,101.78,'Sun-Synch' from dual union all
select 7129,'007129','1973-086CT','deb Delta 98','',30688,'In Earth orbit','','1992-01-24 00:00:00',119.91,1477,1877,102.07,'Sun-Synch' from dual union all
select 713,'000713','1963-052B','8A92E','8A92E',9283,'Reentered','1964-01-25 00:00:00','1964-01-21 00:00:00',88.71,177,244,65.01,'Intermediate' from dual union all
select 7130,'007130','1973-086CU','deb Delta 98','',30688,'In Earth orbit','','1992-02-04 00:00:00',113.59,1334,1449,102.4,'Sun-Synch' from dual union all
select 7131,'007131','1973-086CV','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',119.03,1492,1783,101.9,'Sun-Synch' from dual union all
select 7132,'007132','1973-086CW','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',116.24,1496,1528,102.29,'Sun-Synch' from dual union all
select 7133,'007133','1973-086CX','deb Delta 98','',30688,'In Earth orbit','','1992-01-24 00:00:00',116.18,1474,1543,102.14,'Sun-Synch' from dual union all
select 7134,'007134','1973-086CY','deb Delta 98','',30688,'In Earth orbit','','1992-02-04 00:00:00',115.26,1456,1479,102.02,'Sun-Synch' from dual union all
select 7135,'007135','1973-086CZ','deb Delta 98','',30688,'In Earth orbit','','1992-02-02 00:00:00',114.69,1327,1556,102.25,'Sun-Synch' from dual union all
select 7136,'007136','1973-086DA','deb Delta 98','',30688,'In Earth orbit','','1992-02-02 00:00:00',115.18,1423,1504,101.66,'Sun-Synch' from dual union all
select 7137,'007137','1973-086DB','deb Delta 98','',30688,'In Earth orbit','','1992-02-07 00:00:00',115,1423,1488,101.9,'Sun-Synch' from dual union all
select 7138,'007138','1973-086DC','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',116.87,1499,1582,102.35,'Sun-Synch' from dual union all
select 7139,'007139','1973-086DD','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',117.04,1500,1596,102.02,'Sun-Synch' from dual union all
select 714,'000714','1963-053A','Explorer 19','AD-A',9284,'Reentered','1981-05-10 00:00:00','1964-01-20 00:00:00',115.72,588,2389,78.6,'Intermediate' from dual union all
select 7140,'007140','1973-086DE','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',116.55,1508,1543,102.41,'Sun-Synch' from dual union all
select 7141,'007141','1973-086DF','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',116.54,1502,1549,102.03,'Sun-Synch' from dual union all
select 7142,'007142','1973-086DG','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',116.9,1507,1576,102.32,'Sun-Synch' from dual union all
select 7143,'007143','1973-086DH','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',116.81,1499,1576,101.79,'Sun-Synch' from dual union all
select 7144,'007144','1973-086DJ','deb Delta 98','[UNID066] RCS 0.04',30688,'Reentered','1983-02-15 00:00:00','1983-02-10 00:00:00',92.92,393,443,102.02,'Retrograde' from dual union all
select 7145,'007145','1973-086DK','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',121.89,1507,2024,101.98,'Medium' from dual union all
select 7146,'007146','1973-086DL','deb Delta 98','',30688,'In Earth orbit','','1992-02-03 00:00:00',116.6,1491,1565,101.75,'Sun-Synch' from dual union all
select 7147,'007147','1973-086DM','deb Delta 98','',30688,'In Earth orbit','','1992-02-01 00:00:00',116.85,1495,1584,102.43,'Sun-Synch' from dual union all
select 7148,'007148','1973-086DN','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',117.49,1441,1696,102.45,'Sun-Synch' from dual union all
select 7149,'007149','1973-086DP','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',116.48,1483,1562,102.17,'Sun-Synch' from dual union all
select 715,'000715','1963-049E','deb Transit VBN-2','',9280,'In Earth orbit','','1993-03-29 00:00:00',105.82,1016,1049,90.06,'Polar' from dual union all
select 7150,'007150','1973-086DQ','deb Delta 98','',30688,'In Earth orbit','','1992-01-25 00:00:00',117.89,1485,1688,101.94,'Sun-Synch' from dual union all
select 7151,'007151','1973-086DR','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',118.52,1486,1743,102.61,'Sun-Synch' from dual union all
select 7152,'007152','1973-086DS','deb Delta 98','',30688,'In Earth orbit','','1992-02-02 00:00:00',117.24,1492,1622,102.17,'Sun-Synch' from dual union all
select 7153,'007153','1973-086DT','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',116.67,1505,1558,102.07,'Sun-Synch' from dual union all
select 7154,'007154','1974-009A','San Marco 4','San Marco C2',33612,'Reentered','1976-05-04 00:00:00','1974-03-20 00:00:00',96,236,898,2.9,'Equatorial' from dual union all
select 7155,'007155','1973-086DU','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',116.41,1417,1621,102.52,'Sun-Synch' from dual union all
select 7156,'007156','1973-086DV','deb Delta 98','',30688,'In Earth orbit','','1992-02-01 00:00:00',118.55,1506,1725,102.09,'Sun-Synch' from dual union all
select 7157,'007157','1974-009B','FW4S S/N 2376-4','FW4S',33612,'Reentered','1974-06-09 00:00:00','1974-03-21 00:00:00',95.04,234,807,2.88,'Equatorial' from dual union all
select 7158,'007158','1973-086DW','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',116.9,1509,1574,102.48,'Sun-Synch' from dual union all
select 7159,'007159','1973-086DX','deb Delta 98','',30688,'Reentered','1981-05-13 00:00:00','1981-04-09 00:00:00',97.17,513,734,102.37,'Retrograde' from dual union all
select 716,'000716','1963-054A','Tiros 8','Tiros H',9285,'In Earth orbit','','1964-01-24 00:00:00',99.33,697,757,58.49,'Intermediate' from dual union all
select 7160,'007160','1973-086DY','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',116.96,1478,1610,102.41,'Sun-Synch' from dual union all
select 7161,'007161','1973-086DZ','deb Delta 98','',30688,'In Earth orbit','','1992-02-02 00:00:00',116.04,1492,1514,102.22,'Sun-Synch' from dual union all
select 7162,'007162','1973-086EA','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',114.44,1378,1482,101.51,'Sun-Synch' from dual union all
select 7163,'007163','1973-086EB','deb Delta 98','',30688,'In Earth orbit','','1992-01-10 00:00:00',117,1495,1596,102.44,'Sun-Synch' from dual union all
select 7164,'007164','1973-086EC','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',113.55,1317,1461,101.3,'Sun-Synch' from dual union all
select 7165,'007165','1973-086ED','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',111.33,1135,1441,101.62,'Sun-Synch' from dual union all
select 7166,'007166','1973-086EE','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',116.18,1497,1521,102.16,'Sun-Synch' from dual union all
select 7167,'007167','1973-086EF','deb Delta 98','',30688,'In Earth orbit','','1992-01-31 00:00:00',115.66,1473,1498,101.96,'Sun-Synch' from dual union all
select 7168,'007168','1973-086EG','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',119.05,1495,1782,102.23,'Sun-Synch' from dual union all
select 7169,'007169','1973-086EH','deb Delta 98','',30688,'In Earth orbit','','1992-02-07 00:00:00',113.85,1307,1500,101.28,'Sun-Synch' from dual union all
select 717,'000717','1963-054B','Altair','Altair X-248-A5DM',9285,'Reentered','1990-10-05 00:00:00','1990-10-05 00:00:00',88.06,174,183,58.45,'Intermediate' from dual union all
select 7170,'007170','1973-086EJ','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',112.68,1278,1422,101.69,'Sun-Synch' from dual union all
select 7171,'007171','1974-006C','Blenda','BZA',33609,'Reentered','1974-02-27 00:00:00','1974-02-25 00:00:00',88.76,170,256,65.02,'Intermediate' from dual union all
select 7172,'007172','1974-006D','KDU','KDU',33609,'Reentered','1974-03-02 00:00:00','1974-02-26 00:00:00',88.88,169,270,65,'Intermediate' from dual union all
select 7173,'007173','1973-086EK','deb Delta 98','',30688,'In Earth orbit','','1992-02-04 00:00:00',117.65,1392,1759,102.23,'Sun-Synch' from dual union all
select 7174,'007174','1973-086EL','deb Delta 98','',30688,'In Earth orbit','','1992-02-01 00:00:00',119.62,1419,1909,102.05,'Sun-Synch' from dual union all
select 7175,'007175','1973-086EM','deb Delta 98','',30688,'In Earth orbit','','1992-02-02 00:00:00',112.1,1161,1486,102.61,'Sun-Synch' from dual union all
select 7176,'007176','1973-086EN','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',113,1225,1503,102.23,'Sun-Synch' from dual union all
select 7177,'007177','1973-086EP','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',110.6,1029,1480,102.02,'Sun-Synch' from dual union all
select 7178,'007178','1973-097D','Blok-L','Blok-L',30699,'Reentered','1986-04-17 00:00:00','1974-03-27 00:00:00',734.82,551,40641,62.8,'Molniya' from dual union all
select 7179,'007179','1973-086EQ','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',112.91,1261,1460,102.51,'Sun-Synch' from dual union all
select 718,'000718','1963-055A','OPS 1388','CORONA M-26',9286,'Reentered Attached','1964-01-08 00:00:00','1963-12-28 00:00:00',89.8,178,351,64.87,'Intermediate' from dual union all
select 7180,'007180','1973-086ER','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',113.45,1260,1510,102.21,'Sun-Synch' from dual union all
select 7181,'007181','1973-086ES','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',114.76,1362,1527,102.37,'Sun-Synch' from dual union all
select 7182,'007182','1973-086ET','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',113.55,1114,1665,102.46,'Sun-Synch' from dual union all
select 7183,'007183','1973-086EU','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',114.21,1310,1529,102.37,'Sun-Synch' from dual union all
select 7184,'007184','1973-086EV','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',111.57,1142,1456,102.29,'Sun-Synch' from dual union all
select 7185,'007185','1973-086EW','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',118.16,1484,1713,101.79,'Sun-Synch' from dual union all
select 7186,'007186','1973-086EX','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',111.7,1139,1470,101.8,'Sun-Synch' from dual union all
select 7187,'007187','1974-010A','Kosmos-633','DS-P1-Yu No. 71',33613,'Reentered','1974-10-04 00:00:00','1974-03-29 00:00:00',91.91,265,472,70.98,'Intermediate' from dual union all
select 7188,'007188','1974-010B','S1M','S1M',33613,'Reentered','1974-06-11 00:00:00','1974-03-29 00:00:00',91.51,257,440,70.97,'Intermediate' from dual union all
select 7189,'007189','1974-006E','PTDU cover','PTDU cover',33609,'Reentered','1974-03-10 00:00:00','1974-02-27 00:00:00',90.25,192,382,65.04,'Intermediate' from dual union all
select 719,'000719','1963-055B','PUNDIT II','P-11 No. 4101',9286,'Reentered','1964-11-07 00:00:00','1964-01-28 00:00:00',91.54,312,389,64.52,'Intermediate' from dual union all
select 7190,'007190','1973-086EY','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',107.75,895,1351,102.73,'Sun-Synch' from dual union all
select 7191,'007191','1973-086EZ','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',119.11,1434,1848,102.1,'Sun-Synch' from dual union all
select 7192,'007192','1973-086FA','deb Delta 98','',30688,'In Earth orbit','','1992-02-03 00:00:00',124.21,1503,2234,102.23,'Medium' from dual union all
select 7193,'007193','1973-086FB','deb Delta 98','',30688,'In Earth orbit','','1992-02-07 00:00:00',119.52,1408,1911,101.84,'Sun-Synch' from dual union all
select 7194,'007194','1973-086FC','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',125.53,1477,2377,102.65,'Medium' from dual union all
select 7195,'007195','1973-086FD','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',108.57,886,1435,103.23,'Sun-Synch' from dual union all
select 7196,'007196','1973-086FE','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',115.53,1438,1521,102.19,'Sun-Synch' from dual;
insert into SATELLITE
select 7197,'007197','1973-086FF','deb Delta 98','',30688,'Reentered','1980-06-09 00:00:00','1974-02-28 00:00:00',106.03,585,1500,101.98,'Sun-Synch' from dual union all
select 7198,'007198','1973-086FG','deb Delta 98','',30688,'In Earth orbit','','1992-02-01 00:00:00',112.66,1230,1468,102.38,'Sun-Synch' from dual union all
select 7199,'007199','1973-086FH','deb Delta 98','',30688,'Reentered','1974-07-17 00:00:00','1974-04-07 00:00:00',109.29,859,1529,102.1,'Sun-Synch' from dual union all
select 72,'000072','61 Beta 2','8K78I','8K78I',7261,'Reentered','1961-02-13 00:00:00','1961-02-09 00:00:00',89.16,209,257,64.9,'Intermediate' from dual union all
select 720,'000720','1963-054C','Tiros VIII despin weight','',9285,'In Earth orbit','','1993-03-29 00:00:00',100.08,667,858,58.47,'Intermediate' from dual union all
select 7200,'007200','1973-086FJ','deb Delta 98','',30688,'In Earth orbit','','1992-02-02 00:00:00',120.17,1448,1929,102.49,'Medium' from dual union all
select 7201,'007201','1973-086FK','deb Delta 98','',30688,'Reentered','1979-11-17 00:00:00','1979-11-09 00:00:00',96.73,450,755,101.92,'Retrograde' from dual union all
select 7202,'007202','1973-086FL','deb Delta 98','',30688,'In Earth orbit','','1992-02-06 00:00:00',109.82,987,1450,102.06,'Sun-Synch' from dual union all
select 7203,'007203','1973-086FM','deb Delta 98','',30688,'In Earth orbit','','1992-02-07 00:00:00',108.04,785,1488,101.99,'Sun-Synch' from dual union all
select 7204,'007204','1973-086FN','deb Delta 98','',30688,'Reentered','1977-12-13 00:00:00','1977-10-18 00:00:00',94.07,359,588,102.56,'Retrograde' from dual union all
select 7205,'007205','1973-086FP','deb Delta 98','',30688,'Reentered','1976-12-14 00:00:00','1976-11-09 00:00:00',91.62,309,398,102.64,'Retrograde' from dual union all
select 7206,'007206','1973-086FQ','deb Delta 98','',30688,'Reentered','1974-10-06 00:00:00','1974-02-28 00:00:00',101.61,305,1364,102.86,'Retrograde' from dual union all
select 7207,'007207','1973-086FR','deb Delta 98','',30688,'Reentered','1974-05-13 00:00:00','1974-04-21 00:00:00',100.32,318,1230,101.83,'Sun-Synch' from dual union all
select 7208,'007208','1973-086FS','deb Delta 98','',30688,'Reentered','1978-12-22 00:00:00','1978-11-06 00:00:00',102.19,512,1213,102.08,'Sun-Synch' from dual union all
select 7209,'007209','1974-011A','Meteor','Meteor-M  No. 30',33614,'In Earth orbit','','1974-04-05 00:00:00',102.16,829,892,81.24,'Intermediate' from dual union all
select 721,'000721','1963-053B','Altair 2 X-258B1 RH-61','X-258B1',9284,'In Earth orbit','','1964-10-15 00:00:00',115.79,593,2390,78.63,'Intermediate' from dual union all
select 7210,'007210','1974-011B','8A92ME','8A92ME',33614,'In Earth orbit','','1974-04-04 00:00:00',102.19,803,922,81.24,'Intermediate' from dual union all
select 7211,'007211','1974-012A','Kosmos-634','DS-P1-Yu No. 67',33615,'Reentered','1974-10-09 00:00:00','1974-04-07 00:00:00',91.85,266,465,70.92,'Intermediate' from dual union all
select 7212,'007212','1974-012B','S1M','S1M',33615,'Reentered','1974-06-17 00:00:00','1974-04-04 00:00:00',91.45,263,429,70.94,'Intermediate' from dual union all
select 7213,'007213','1974-013A','Miranda','X-4',33616,'In Earth orbit','','1974-04-09 00:00:00',101.14,709,917,97.81,'Sun-Synch' from dual union all
select 7214,'007214','1974-013D','X-4 despin weight','',33616,'Reentered','1983-05-04 00:00:00','1974-04-19 00:00:00',101.32,704,939,98.34,'Sun-Synch' from dual union all
select 7215,'007215','1974-013C','X-4 despin weight','',33616,'Reentered','1984-04-05 00:00:00','1974-04-11 00:00:00',101.08,714,905,97.26,'Sun-Synch' from dual union all
select 7216,'007216','1974-014A','Kosmos-635','Zenit-2M',33617,'Landed','1974-03-26 00:00:00','1974-03-16 00:00:00',89.75,202,322,72.83,'Intermediate' from dual union all
select 7217,'007217','1974-014B','11A57I','11A57I',33617,'Reentered','1974-03-25 00:00:00','1974-03-16 00:00:00',89.47,198,299,72.83,'Intermediate' from dual union all
select 7218,'007218','1974-015A','DMSP 8531','DMSP Block 5B F-5',33618,'In Earth orbit','','1974-04-15 00:00:00',101.46,779,877,98.93,'Sun-Synch' from dual union all
select 7219,'007219','1974-015B','Burner IIA No. 19 Stage 2','Star 26B S/N 006',33618,'In Earth orbit','','1974-04-16 00:00:00',101.57,781,885,98.93,'Sun-Synch' from dual union all
select 722,'000722','1963-053C','Canister part','Canister part',9284,'Reentered','2018-04-18 00:00:00','1964-10-15 00:00:00',115.85,603,2385,78.59,'Intermediate' from dual union all
select 7220,'007220','1974-014C','Nauka part','',33617,'Reentered','1974-03-23 00:00:00','1974-03-18 00:00:00',89.51,200,301,72.84,'Intermediate' from dual union all
select 7221,'007221','1974-014D','Nauka part','',33617,'Reentered','1974-03-20 00:00:00','1974-03-18 00:00:00',89.91,192,348,73.17,'Intermediate' from dual union all
select 7222,'007222','1974-014E','Kapsula Kosmosa-635','Nauka 17KS No. 1L',33617,'Reentered','1974-04-07 00:00:00','1974-03-22 00:00:00',89.51,198,302,72.84,'Intermediate' from dual union all
select 7223,'007223','1973-052D','SA Marsa-6','SA',30654,'Deep Space Attached','1973-08-06 00:00:00','1973-12-31 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 7224,'007224','1973-053D','SA Marsa-7','SA',30655,'Deep Space Attached','1973-08-10 00:00:00','1974-03-09 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 7225,'007225','1974-016A','Kosmos-636','Zenit-4MK',33619,'Landed','1974-04-03 00:00:00','1974-03-24 00:00:00',89.24,170,304,65.02,'Intermediate' from dual union all
select 7226,'007226','1974-016B','11S510I','11S510I',33619,'Reentered','1974-03-24 00:00:00','1974-03-22 00:00:00',89.16,164,302,65.04,'Intermediate' from dual union all
select 7227,'007227','1974-016C','Sensor cover','Zenit-4MK sensor cover',33619,'Reentered','1974-03-25 00:00:00','1974-03-20 00:00:00',88.93,129,315,65.07,'Intermediate' from dual union all
select 7228,'007228','1974-013B','FW4S S/N 2376-7','FW4S',33616,'In Earth orbit','','1974-04-10 00:00:00',101.15,711,915,97.81,'Sun-Synch' from dual union all
select 7229,'007229','1974-017A','Kosmos-637','11F638 GVM',33620,'In Earth orbit','','1974-04-29 00:00:00',1425.83,35402,35769,.29,'Drift GEO' from dual union all
select 723,'000723','1963-053D','Canister part','Canister part',9284,'Reentered','1982-11-12 00:00:00','1964-10-15 00:00:00',115.91,596,2398,78.62,'Intermediate' from dual union all
select 7230,'007230','1974-017B','Perekhodnik','Sredniy perekhodnik',33620,'Reentered','1974-03-27 00:00:00','1974-03-26 00:00:00',87.91,166,176,51,'Intermediate' from dual union all
select 7231,'007231','1974-017C','8S812','8S812',33620,'Reentered','1974-03-28 00:00:00','1974-03-27 00:00:00',87.87,160,179,51.5,'Intermediate' from dual union all
select 7232,'007232','1974-016D','KDU','KDU',33619,'Reentered','1974-04-03 00:00:00','1974-04-02 00:00:00',88.81,166,265,65.02,'Intermediate' from dual union all
select 7233,'007233','1974-016E','Blenda','Sunshade',33619,'Reentered','1974-04-04 00:00:00','1974-04-02 00:00:00',88.58,164,245,65.04,'Intermediate' from dual union all
select 7234,'007234','1974-018A','Kosmos-638','Soyuz 7K-TM No. 71-EPSA',33621,'Landed','1974-04-13 00:00:00','1974-04-05 00:00:00',89.55,188,317,51.78,'Intermediate' from dual union all
select 7235,'007235','1974-018B','11S510','11S510',33621,'Reentered','1974-04-09 00:00:00','1974-04-06 00:00:00',89.02,187,266,51.77,'Intermediate' from dual union all
select 7236,'007236','1974-018C','Kosmos-638 part','7K-TM part',33621,'Reentered','1974-04-06 00:00:00','1974-04-05 00:00:00',89.12,175,286,51.78,'Intermediate' from dual union all
select 7237,'007237','1974-018D','deb Kosmos-638','',33621,'Reentered','1974-04-05 00:00:00','1974-04-03 00:00:00',88.8,177,254,51.75,'Intermediate' from dual union all
select 7238,'007238','1974-018E','deb Kosmos-638','',33621,'Reentered','1974-04-06 00:00:00','1974-04-03 00:00:00',88.33,181,203,51.78,'Intermediate' from dual union all
select 7239,'007239','1974-016F','PTDU cover','',33619,'Reentered','1974-04-05 00:00:00','1974-04-03 00:00:00',90.54,182,419,65.01,'Intermediate' from dual union all
select 724,'000724','1963-053E','deb Explorer XIX','',9284,'In Earth orbit','','1964-10-15 00:00:00',115.96,624,2374,78.64,'Intermediate' from dual union all
select 7240,'007240','1974-019A','Kosmos-639','Zenit-4MK',33622,'Landed','1974-04-15 00:00:00','1974-04-06 00:00:00',88.77,181,247,81.31,'Intermediate' from dual union all
select 7241,'007241','1974-019B','11A57I','11A57I',33622,'Reentered','1974-04-07 00:00:00','1974-04-05 00:00:00',88.44,185,210,81.31,'Intermediate' from dual union all
select 7242,'007242','1974-020A','OPS 6245','HEXAGON SV-8',33623,'Deorbited','1974-07-28 00:00:00','1974-05-10 00:00:00',88.72,151,272,94.49,'Polar' from dual union all
select 7243,'007243','1974-020D','Titan 23D-9 Stage 2','LR91AJ11 23D-9',33623,'Reentered','1974-04-12 00:00:00','1974-04-11 00:00:00',88.7,121,300,94.5,'Polar' from dual union all
select 7244,'007244','1974-020B','S73-7','S73-7 canister',33623,'In Earth orbit','','1977-01-01 00:00:00',101,786,826,94.6,'Polar' from dual union all
select 7245,'007245','1974-021A','Kosmos-640','Zenit-2M',33624,'Landed','1974-04-23 00:00:00','1974-04-13 00:00:00',88.68,197,222,81.33,'Intermediate' from dual union all
select 7246,'007246','1974-021B','11A57I','11A57I',33624,'Reentered','1974-04-14 00:00:00','1974-04-13 00:00:00',88.27,171,207,81.31,'Intermediate' from dual union all
select 7247,'007247','1974-020C','TOPHAT II','P-11 No. 4428',33623,'Reentered','1980-02-22 00:00:00','1974-07-03 00:00:00',94.91,501,528,94,'Polar' from dual union all
select 7248,'007248','1974-017D','SOZ','SOZ',33620,'Reentered','1976-11-16 00:00:00','1974-04-28 00:00:00',630.85,263,35712,47.37,'GEO Transfer' from dual union all
select 7249,'007249','1974-017E','SOZ','SOZ',33620,'Reentered','1976-11-16 00:00:00','1974-04-27 00:00:00',632.28,322,35727,47.35,'GEO Transfer' from dual union all
select 725,'000725','1963-053F','deb Explorer XIX','',9284,'Reentered','1974-12-23 00:00:00','1964-10-15 00:00:00',115.87,591,2399,78.64,'Intermediate' from dual union all
select 7250,'007250','1974-022A','Westar 1','Westar 1',33625,'In Earth orbit','','1977-01-20 00:00:00',1436.06,35764,35808,.02,'Stationary' from dual union all
select 7251,'007251','1974-022B','Delta 101','Delta SSPU DSV-3P-4',33625,'Reentered','1974-05-25 00:00:00','1974-04-20 00:00:00',91.26,227,446,28.59,'Intermediate' from dual union all
select 7252,'007252','1974-022C','Star 37C S/N 40025','Star 37C',33625,'Reentered','1974-11-25 00:00:00','1975-06-10 00:00:00',629.95,241,35687,24.1,'GEO Transfer' from dual union all
select 7253,'007253','1974-019C','KDU part','KDU part',33622,'Reentered','1974-04-16 00:00:00','1974-04-14 00:00:00',88.32,167,216,81.31,'Intermediate' from dual union all
select 7254,'007254','1974-019D','KDU','KDU',33622,'Reentered','1974-04-17 00:00:00','1974-04-11 00:00:00',88.29,162,217,81.31,'Intermediate' from dual union all
select 7255,'007255','1974-019E','Blenda','Sunshade',33622,'Reentered','1974-04-16 00:00:00','1974-04-14 00:00:00',88.43,176,218,81.19,'Intermediate' from dual union all
select 7256,'007256','1974-019F','PTDU cover','KDU part',33622,'Reentered','1974-04-16 00:00:00','1974-04-15 00:00:00',89.96,143,402,81.31,'Intermediate' from dual union all
select 7257,'007257','1974-005E','Tselina-O part','[UNID067] RCS 0.0',33608,'Reentered','1978-01-11 00:00:00','1974-04-15 00:00:00',95.01,499,540,74.02,'Intermediate' from dual union all
select 7258,'007258','62 B Psi 5','deb Transit VA-1','',8177,'Reentered','1977-07-19 00:00:00','1974-04-15 00:00:00',98.36,661,700,90.63,'Polar' from dual union all
select 7259,'007259','1963-038K','deb Transit VBN-1','',9269,'Reentered','1982-11-06 00:00:00','1974-04-21 00:00:00',107.27,1077,1124,89.88,'Polar' from dual union all
select 726,'000726','1963-053G','deb Explorer XIX','',9284,'Reentered','2014-11-14 00:00:00','1964-10-15 00:00:00',115.81,592,2393,78.6,'Intermediate' from dual union all
select 7260,'007260','1974-023A','Molniya-1','Molniya-1',33626,'Reentered','1983-11-17 00:00:00','1974-06-04 00:00:00',717.68,625,39724,62.92,'Molniya' from dual union all
select 7261,'007261','1974-023B','11S510','11S510',33626,'Reentered','1974-06-20 00:00:00','1974-05-20 00:00:00',91.4,212,475,62.85,'Intermediate' from dual union all
select 7262,'007262','1974-023C','BOZ','BOZ',33626,'Reentered','1974-07-04 00:00:00','1974-05-20 00:00:00',91.9,212,523,62.82,'Intermediate' from dual union all
select 7263,'007263','1974-023D','deb Blok-I','',33626,'Reentered','1974-05-04 00:00:00','1974-04-23 00:00:00',92.86,215,615,62.83,'Intermediate' from dual union all
select 7264,'007264','1974-023E','Blok-L','Blok-L',33626,'Reentered','1983-12-21 00:00:00','1974-06-11 00:00:00',734.49,627,40549,62.94,'Molniya' from dual union all
select 7265,'007265','1974-024A','Kosmos-641','Strela-1M',33627,'In Earth orbit','','1974-05-25 00:00:00',114.55,1388,1481,74.01,'Intermediate' from dual union all
select 7266,'007266','1974-024B','Kosmos-642','Strela-1M',33627,'In Earth orbit','','1974-05-23 00:00:00',113.78,1319,1481,74.01,'Intermediate' from dual union all
select 7267,'007267','1974-024C','Kosmos-643','Strela-1M',33627,'In Earth orbit','','1974-05-23 00:00:00',114.16,1353,1481,74.01,'Intermediate' from dual union all
select 7268,'007268','1974-024D','Kosmos-644','Strela-1M',33627,'In Earth orbit','','1974-05-27 00:00:00',113.97,1335,1483,74.02,'Intermediate' from dual union all
select 7269,'007269','1974-024E','Kosmos-645','Strela-1M',33627,'In Earth orbit','','1974-05-23 00:00:00',114.35,1369,1483,74.02,'Intermediate' from dual union all
select 727,'000727','1964-001A','HAYLOFT','AFTRACK 37/Mission 7210',10287,'In Earth orbit','','1964-02-13 00:00:00',103.39,906,932,69.91,'Intermediate' from dual union all
select 7270,'007270','1974-024F','Kosmos-646','Strela-1M',33627,'In Earth orbit','','1974-05-23 00:00:00',114.76,1404,1484,74.01,'Intermediate' from dual union all
select 7271,'007271','1974-024G','Kosmos-647','Strela-1M',33627,'In Earth orbit','','1974-05-23 00:00:00',114.95,1422,1485,74.01,'Intermediate' from dual union all
select 7272,'007272','1974-024H','Kosmos-648','Strela-1M',33627,'In Earth orbit','','1974-05-27 00:00:00',115.17,1437,1490,74.01,'Intermediate' from dual union all
select 7273,'007273','1974-024J','S3M','S3M',33627,'In Earth orbit','','1974-05-24 00:00:00',116.94,1477,1609,74.02,'Intermediate' from dual union all
select 7274,'007274','1974-025A','Meteor','Meteor-M  No. 22',33628,'In Earth orbit','','1974-05-24 00:00:00',102.52,863,892,81.22,'Intermediate' from dual union all
select 7275,'007275','1974-025B','8A92ME','8A92ME',33628,'In Earth orbit','','1974-05-24 00:00:00',102.61,839,925,81.25,'Intermediate' from dual union all
select 7276,'007276','1974-026A','Molniya-2','Molniya-2',33629,'In Earth orbit','','1974-06-05 00:00:00',717.96,421,39943,62.88,'Molniya' from dual union all
select 7277,'007277','1974-026B','11S510','11S510',33629,'Reentered','1974-05-21 00:00:00','1974-05-03 00:00:00',90.62,212,398,62.85,'Intermediate' from dual union all
select 7278,'007278','1974-026C','BOZ','BOZ',33629,'Reentered','1974-05-17 00:00:00','1974-05-04 00:00:00',90.43,196,396,62.82,'Intermediate' from dual union all
select 7279,'007279','1974-026D','deb Blok-I','',33629,'Reentered','1974-04-28 00:00:00','1974-04-27 00:00:00',90.36,183,402,62.81,'Intermediate' from dual union all
select 728,'000728','1964-001B','GGSE 1','POPPY 3 SV3',10287,'In Earth orbit','','1964-02-12 00:00:00',103.43,907,935,69.93,'Intermediate' from dual union all
select 7280,'007280','1974-027A','Kosmos-649','Zenit-4MK',33630,'Landed','1974-05-11 00:00:00','1974-05-01 00:00:00',89.25,179,295,62.8,'Intermediate' from dual union all
select 7281,'007281','1974-028A','Kosmos-650','Geoid',33631,'In Earth orbit','','1974-05-30 00:00:00',113.44,1368,1401,74.04,'Intermediate' from dual union all
select 7282,'007282','1974-027B','11A57I','11A57I',33630,'Reentered','1974-05-03 00:00:00','1974-05-01 00:00:00',88.73,168,256,62.8,'Intermediate' from dual union all
select 7283,'007283','1974-027C','Sensor cover','Zenit-4MK sensor cover',33630,'Reentered','1974-04-30 00:00:00','1974-04-29 00:00:00',88.83,178,256,62.81,'Intermediate' from dual union all
select 7284,'007284','1974-028B','S3M','S3M',33631,'In Earth orbit','','1974-05-29 00:00:00',113.27,1363,1391,74.03,'Intermediate' from dual union all
select 7285,'007285','1969-082KJ','deb Agena D','',18349,'Reentered','1974-07-19 00:00:00','1974-05-06 00:00:00',95.43,261,817,70.93,'Intermediate' from dual union all
select 7286,'007286','1974-027D','KDU','KDU',33630,'Reentered','1974-05-16 00:00:00','1974-05-10 00:00:00',88.98,174,275,62.83,'Intermediate' from dual;
insert into SATELLITE
select 7287,'007287','1974-027E','Blenda','Sunshade',33630,'Reentered','1974-05-11 00:00:00','1974-05-10 00:00:00',88.8,175,256,62.86,'Intermediate' from dual union all
select 7288,'007288','1974-027F','PTDU cover','KDU part',33630,'Reentered','1974-05-10 00:00:00','1974-05-10 00:00:00',89.38,193,294,62.79,'Intermediate' from dual union all
select 7289,'007289','1974-027G','PTDU cover','KDU part',33630,'Reentered','1974-05-13 00:00:00','1974-05-11 00:00:00',88.82,170,263,62.81,'Intermediate' from dual union all
select 729,'000729','1964-001C','EGRS I','Secor Type II S/N 2',10287,'In Earth orbit','','1964-02-13 00:00:00',103.38,905,932,69.9,'Intermediate' from dual union all
select 7290,'007290','1974-012C','deb Kosmos-634','',33615,'Reentered','1974-09-10 00:00:00','1974-05-10 00:00:00',93.04,246,601,71.02,'Intermediate' from dual union all
select 7291,'007291','1974-029A','Kosmos-651','US-A reactor',33632,'In Earth orbit','','1974-06-14 00:00:00',89.62,249,263,64.96,'Intermediate' from dual union all
select 7292,'007292','1974-030A','Kosmos-652','Zenit-4MK',33633,'Landed','1974-05-23 00:00:00','1974-05-17 00:00:00',89.63,172,340,51.76,'Intermediate' from dual union all
select 7293,'007293','1974-031A','Kosmos-653','Zenit-2M',33634,'Landed','1974-05-27 00:00:00','1974-05-17 00:00:00',89.24,190,284,62.79,'Intermediate' from dual union all
select 7294,'007294','1974-030B','11S510I','11S510I',33633,'Reentered','1974-05-19 00:00:00','1974-05-18 00:00:00',88.57,157,251,51.75,'Intermediate' from dual union all
select 7295,'007295','1974-031B','11A57I','11A57I',33634,'Reentered','1974-05-20 00:00:00','1974-05-17 00:00:00',88.92,178,265,62.8,'Intermediate' from dual union all
select 7296,'007296','1974-031C','Sensor cover','Zenit-2M sensor cover',33634,'Reentered','1974-05-28 00:00:00','1974-05-16 00:00:00',89.36,195,291,62.79,'Intermediate' from dual union all
select 7297,'007297','1974-032A','Kosmos-654','US-A reactor',33635,'In Earth orbit','','1974-06-16 00:00:00',89.63,250,262,64.95,'Intermediate' from dual union all
select 7298,'007298','1974-033A','SMS 1','SMS A',33636,'In Earth orbit','','1977-01-26 00:00:00',1436.06,35705,35867,2.29,'Stationary' from dual union all
select 7299,'007299','1974-034A','Interkosmos-11','DS-U3-IK No. 4',33637,'Reentered','1979-09-06 00:00:00','1974-06-21 00:00:00',94.55,483,511,50.64,'Intermediate' from dual union all
select 73,'000073','61 Beta 3','BOZ','BOZ',7261,'Reentered','1961-03-17 00:00:00','1961-02-16 00:00:00',89.9,227,312,64.89,'Intermediate' from dual union all
select 730,'000730','1964-001D','SR 7A','POPPY 3 SV1',10287,'In Earth orbit','','1964-02-13 00:00:00',103.44,909,934,69.9,'Intermediate' from dual union all
select 7300,'007300','1974-033B','Delta 102','Delta SSPU DSV-3P-4',33636,'Reentered','1974-05-20 00:00:00','1974-05-19 00:00:00',88.35,162,224,28.32,'Intermediate' from dual union all
select 7301,'007301','1974-033C','Delta 102 clamp band','',33636,'Reentered','1974-05-21 00:00:00','1974-05-19 00:00:00',88.29,161,219,28.31,'Intermediate' from dual union all
select 7302,'007302','1974-034B','S3M','S3M',33637,'Reentered','1980-01-07 00:00:00','1974-06-16 00:00:00',94.43,465,517,50.69,'Intermediate' from dual union all
select 7303,'007303','1974-033D','Star 37E S/N 40020','Star 37E',33636,'Reentered','1974-11-12 00:00:00','1974-05-18 00:00:00',575.74,177,32917,24.52,'Highly Ellip' from dual union all
select 7304,'007304','1974-033E','Delta 102 clamp band','',33636,'Reentered','1974-05-18 00:00:00','1974-05-18 00:00:00',88.75,161,265,28.34,'Intermediate' from dual union all
select 7305,'007305','1974-034C','deb Interkosmos-11','',33637,'Reentered','1974-12-29 00:00:00','1974-06-16 00:00:00',94.02,444,499,50.62,'Intermediate' from dual union all
select 7306,'007306','1974-035A','Kosmos-655','Tselina-OM',33638,'Reentered','1980-11-19 00:00:00','1974-06-24 00:00:00',95.24,520,541,74.05,'Intermediate' from dual union all
select 7307,'007307','1974-035B','S3M','S3M',33638,'Reentered','1980-10-05 00:00:00','1974-06-20 00:00:00',95.15,512,540,74.06,'Intermediate' from dual union all
select 7308,'007308','1974-030C','KDU','KDU',33633,'Reentered','1974-05-27 00:00:00','1974-05-23 00:00:00',89.14,169,294,51.75,'Intermediate' from dual union all
select 7309,'007309','1974-030D','Blenda','Sunshade',33633,'Reentered','1974-05-25 00:00:00','1974-05-22 00:00:00',88.93,159,284,51.7,'Intermediate' from dual union all
select 731,'000731','1964-001E','GREB','POPPY 3 SV2',10287,'In Earth orbit','','1964-02-13 00:00:00',103.44,906,936,69.88,'Intermediate' from dual union all
select 7310,'007310','1974-030F','PTDU cover','KDU part',33633,'Reentered','1974-06-05 00:00:00','1974-05-23 00:00:00',90.88,219,417,51.78,'Intermediate' from dual union all
select 7311,'007311','1974-030E','PTDU cover','KDU part',33633,'Reentered','1974-06-05 00:00:00','1974-05-22 00:00:00',91.16,222,441,51.77,'Intermediate' from dual union all
select 7312,'007312','1974-030G','KDU part','KDU part',33633,'Reentered','1974-05-25 00:00:00','1974-05-23 00:00:00',88.68,163,256,51.77,'Intermediate' from dual union all
select 7313,'007313','1974-036A','Kosmos-656','Soyuz 7K-T(A9) No. 61',33639,'Landed','1974-05-29 00:00:00','1974-05-28 00:00:00',90.1,195,364,51.61,'Intermediate' from dual union all
select 7314,'007314','1974-036B','11S510','11S510',33639,'Reentered','1974-06-02 00:00:00','1974-05-29 00:00:00',89.22,179,292,51.57,'Intermediate' from dual union all
select 7315,'007315','1974-037A','Luna-22','E-8LS No. 206',33640,'Deep Space','1974-05-30 00:00:00','1974-05-29 00:00:00',15562.18,200,400000,51.5,'Cislunar' from dual union all
select 7316,'007316','1974-037B','SOZ','SOZ',33640,'Reentered','1974-06-02 00:00:00','1974-05-31 00:00:00',88.36,177,210,51.49,'Intermediate' from dual union all
select 7317,'007317','1974-038A','Kosmos-657','Zenit-4MK',33641,'Landed','1974-06-13 00:00:00','1974-06-01 00:00:00',89.34,175,308,62.79,'Intermediate' from dual union all
select 7318,'007318','1974-039A','ATS 6','ATS F',33642,'In Earth orbit','','1977-01-20 00:00:00',1436.09,35779,35794,.41,'Stationary' from dual union all
select 7319,'007319','1974-039B','Titan 23C-9 Stage 2','LR91AJ11 23C-9',33642,'Reentered','1974-06-04 00:00:00','1974-06-01 00:00:00',91.25,153,519,28.59,'Intermediate' from dual union all
select 732,'000732','1963-053H','deb Explorer XIX','',9284,'In Earth orbit','','1964-10-15 00:00:00',115.84,593,2394,78.6,'Intermediate' from dual union all
select 7320,'007320','1974-038B','11A57I','11A57I',33641,'Reentered','1974-06-02 00:00:00','1974-06-01 00:00:00',88.69,162,258,62.78,'Intermediate' from dual union all
select 7321,'007321','1974-038C','Sensor cover','Zenit-4MK sensor cover',33641,'Reentered','1974-06-02 00:00:00','1974-05-31 00:00:00',89.02,175,278,62.77,'Intermediate' from dual union all
select 7322,'007322','1974-037C','SOZ','SOZ',33640,'Reentered','1974-06-02 00:00:00','1974-05-31 00:00:00',88.51,176,225,51.56,'Intermediate' from dual union all
select 7323,'007323','1974-037D','deb Blok D','',33640,'Reentered','1974-06-02 00:00:00','1974-05-31 00:00:00',88.48,180,219,51.52,'Intermediate' from dual union all
select 7324,'007324','1974-039C','Transtage 27','Transtage 27',33642,'In Earth orbit','','1977-02-09 00:00:00',1430.45,35563,35789,.46,'Drift GEO' from dual union all
select 7325,'007325','1974-040A','Hawkeye 1','Injun 6',33643,'Reentered','1978-04-30 00:00:00','1976-02-29 00:00:00',3077.17,4430,122973,88.33,'Very High' from dual union all
select 7326,'007326','1974-040B','FW4S S/N 2376-3','FW4S',33643,'Reentered','1977-10-11 00:00:00','1974-07-04 00:00:00',95.81,330,786,89.71,'Polar' from dual union all
select 7327,'007327','1974-040C','Alcyone sep clamp','',33643,'Reentered','1974-11-10 00:00:00','1974-07-03 00:00:00',94.89,294,732,89.73,'Polar' from dual union all
select 7328,'007328','1974-041A','Kosmos-658','Zenit-2M',33644,'Landed','1974-06-18 00:00:00','1974-06-08 00:00:00',89.35,202,283,64.96,'Intermediate' from dual union all
select 7329,'007329','1974-041B','11A57I','11A57I',33644,'Reentered','1974-06-12 00:00:00','1974-06-08 00:00:00',89.11,188,273,64.97,'Intermediate' from dual union all
select 733,'000733','1964-002A','Agena D 2303','Agena D',10288,'In Earth orbit','','1964-07-03 00:00:00',101.3,792,849,99.06,'Sun-Synch' from dual union all
select 7330,'007330','1974-042A','OPS 1776','GAMBIT-3 Mission 4342',33645,'Reentered Attached','1974-07-23 00:00:00','1974-06-13 00:00:00',89.81,124,407,110.49,'Retrograde' from dual union all
select 7331,'007331','1974-040D','Alcyone sep clamp','',33643,'Reentered','1974-11-21 00:00:00','1974-07-03 00:00:00',94.98,298,738,89.73,'Polar' from dual union all
select 7332,'007332','1974-038D','KDU','KDU',33641,'Reentered','1974-06-18 00:00:00','1974-06-12 00:00:00',89.12,170,292,62.78,'Intermediate' from dual union all
select 7333,'007333','1974-038E','Blenda','Sunshade',33641,'Reentered','1974-06-14 00:00:00','1974-06-12 00:00:00',88.87,158,279,62.82,'Intermediate' from dual union all
select 7334,'007334','1974-043A','Kosmos-659','Zenit-4MK',33646,'Landed','1974-06-26 00:00:00','1974-06-15 00:00:00',89.62,177,335,62.81,'Intermediate' from dual union all
select 7335,'007335','1974-043B','11A57I','11A57I',33646,'Reentered','1974-06-18 00:00:00','1974-06-15 00:00:00',89.3,170,310,62.8,'Intermediate' from dual union all
select 7336,'007336','1974-043C','Sensor cover','Zenit-4MK sensor cover',33646,'Reentered','1974-06-16 00:00:00','1974-06-14 00:00:00',89.36,175,310,62.53,'Intermediate' from dual union all
select 7337,'007337','1974-044A','Kosmos-660','Vektor',33647,'In Earth orbit','','1974-07-18 00:00:00',109.03,395,1969,82.98,'Intermediate' from dual union all
select 7338,'007338','1974-044B','S3M','S3M',33647,'Reentered','2012-06-22 00:00:00','1974-07-18 00:00:00',108.9,390,1962,82.98,'Intermediate' from dual union all
select 7339,'007339','1974-045A','Kosmos-661','Tselina-OM',33648,'Reentered','1980-08-27 00:00:00','1974-07-22 00:00:00',95.18,510,544,74.04,'Intermediate' from dual union all
select 734,'000734','1964-002B','OPS 3367A','P35-6',10288,'In Earth orbit','','1964-03-02 00:00:00',101.19,808,821,99.05,'Sun-Synch' from dual union all
select 7340,'007340','1974-045B','S3M','S3M',33648,'Reentered','1980-10-16 00:00:00','1974-07-22 00:00:00',94.98,497,539,74.05,'Intermediate' from dual union all
select 7341,'007341','1974-045C','deb Kosmos-661','Tselina-O part',33648,'Reentered','1978-02-02 00:00:00','1974-10-28 00:00:00',94.6,473,525,74.09,'Intermediate' from dual union all
select 7342,'007342','1974-046A','Salyut-3','Almaz 2 (11F71 No. 101-2',33649,'Deorbited','1975-01-24 00:00:00','1974-07-24 00:00:00',89.84,264,270,51.58,'Intermediate' from dual union all
select 7343,'007343','1974-046B','8S812','8S812',33649,'Reentered','1974-07-03 00:00:00','1974-06-26 00:00:00',88.92,210,232,51.6,'Intermediate' from dual union all
select 7344,'007344','1974-046C','Salyut-3 shroud','',33649,'Reentered','1974-06-27 00:00:00','1974-06-25 00:00:00',88.77,202,225,51.58,'Intermediate' from dual union all
select 7345,'007345','1974-046D','Salyut-3 shroud','',33649,'Reentered','1974-06-28 00:00:00','1974-06-25 00:00:00',88.34,181,204,51.56,'Intermediate' from dual union all
select 7346,'007346','1974-043D','KDU','KDU',33646,'Reentered','1974-06-27 00:00:00','1974-06-25 00:00:00',89.06,145,311,62.81,'Intermediate' from dual union all
select 7347,'007347','1974-047A','Kosmos-662','DS-P1-I No. 14',33650,'Reentered','1976-08-28 00:00:00','1974-07-26 00:00:00',95.29,268,798,70.91,'Intermediate' from dual union all
select 7348,'007348','1974-047B','S1M','S1M',33650,'Reentered','1975-10-26 00:00:00','1974-07-26 00:00:00',95.14,274,777,70.92,'Intermediate' from dual union all
select 7349,'007349','1974-048A','Kosmos-663','Zaliv',33651,'In Earth orbit','','1974-07-27 00:00:00',104.8,967,1003,82.95,'Intermediate' from dual union all
select 735,'000735','1964-002C','OPS 3367B','P35-7',10288,'In Earth orbit','','1964-03-13 00:00:00',101.23,808,825,99.05,'Sun-Synch' from dual union all
select 7350,'007350','1974-048B','S3M','S3M',33651,'In Earth orbit','','1974-07-27 00:00:00',104.65,967,990,82.95,'Intermediate' from dual union all
select 7351,'007351','1974-049A','Kosmos-664','Zenit-4MT',33652,'Landed','1974-07-11 00:00:00','1974-07-01 00:00:00',89.93,203,339,72.85,'Intermediate' from dual union all
select 7352,'007352','1974-050A','Kosmos-665','Oko',33653,'Reentered','1990-07-06 00:00:00','1974-08-01 00:00:00',719.5,680,39760,62.91,'Molniya' from dual union all
select 7353,'007353','1974-050B','11S510','11S510',33653,'Reentered','1974-09-01 00:00:00','1974-07-29 00:00:00',91.55,211,491,62.81,'Intermediate' from dual union all
select 7354,'007354','1974-050C','Blok-L','Blok-L',33653,'Reentered','1990-02-21 00:00:00','1974-08-17 00:00:00',707.45,699,39145,62.93,'Molniya' from dual union all
select 7355,'007355','1974-049B','11S510','11S510',33652,'Reentered','1974-07-09 00:00:00','1974-07-01 00:00:00',89.65,193,322,72.85,'Intermediate' from dual union all
select 7356,'007356','1974-049C','Sensor cover','Zenit-4MT sensor cover',33652,'Reentered','1974-07-08 00:00:00','1974-06-30 00:00:00',89.59,191,318,72.82,'Intermediate' from dual union all
select 7357,'007357','1974-049D','Sensor cover','Zenit-4MT sensor cover',33652,'Reentered','1974-07-01 00:00:00','1974-06-30 00:00:00',89.89,197,341,72.85,'Intermediate' from dual union all
select 7358,'007358','1974-049E','Sensor cover','Zenit-4MT sensor cover',33652,'Reentered','1974-07-08 00:00:00','1974-06-30 00:00:00',88.99,186,264,72.82,'Intermediate' from dual union all
select 7359,'007359','1974-049F','Sensor cover','Zenit-4MT sensor cover',33652,'Reentered','1974-07-08 00:00:00','1974-06-30 00:00:00',89.72,193,329,72.79,'Intermediate' from dual union all
select 736,'000736','1963-054D','Tiros VIII despin weight','',9285,'Reentered','1981-11-25 00:00:00','1981-11-24 00:00:00',88.19,179,191,58.46,'Intermediate' from dual union all
select 7360,'007360','1974-050D','BOZ','BOZ',33653,'Reentered','1974-08-13 00:00:00','1974-07-07 00:00:00',92.44,194,595,62.9,'Intermediate' from dual union all
select 7361,'007361','1974-051A','Soyuz-14','Soyuz 7K-T(A9) No. 62',33654,'Landed','1974-07-19 00:00:00','1974-07-10 00:00:00',89.86,266,269,51.58,'Intermediate' from dual union all
select 7362,'007362','1974-051B','11S510','11S510',33654,'Reentered','1974-07-05 00:00:00','1974-07-04 00:00:00',88.51,189,213,51.6,'Intermediate' from dual union all
select 7363,'007363','1974-052A','Meteor','Meteor-Priroda No. 1',33655,'In Earth orbit','','1974-08-08 00:00:00',102.5,862,892,81.23,'Intermediate' from dual union all
select 7364,'007364','1974-052B','8A92ME','8A92ME',33655,'In Earth orbit','','1974-08-08 00:00:00',102.64,851,917,81.23,'Intermediate' from dual union all
select 7365,'007365','1974-049G','Blenda','Sunshade',33652,'Reentered','1974-07-13 00:00:00','1974-07-11 00:00:00',89.58,146,362,72.84,'Intermediate' from dual union all
select 7366,'007366','1974-049H','KB electropitaniya','KBE',33652,'Reentered','1974-07-13 00:00:00','1974-07-12 00:00:00',89.37,194,293,72.84,'Intermediate' from dual union all
select 7367,'007367','1974-053A','Kosmos-666','Zenit-4MK',33656,'Landed','1974-07-25 00:00:00','1974-07-14 00:00:00',89.58,180,328,62.81,'Intermediate' from dual union all
select 7368,'007368','1974-053B','11A57I','11A57I',33656,'Reentered','1974-07-18 00:00:00','1974-07-15 00:00:00',89.07,169,287,62.79,'Intermediate' from dual union all
select 7369,'007369','1974-054A','NTS 1','NTS 1',33657,'In Earth orbit','','1974-10-03 00:00:00',468.7,13473,13746,125.12,'Medium' from dual union all
select 737,'000737','1964-003A','Relay 2','Relay B',10289,'In Earth orbit','','1964-03-04 00:00:00',194.72,2090,7411,46.29,'Medium' from dual union all
select 7370,'007370','1974-054B','Star 37E S/N 40027','PTS',33657,'Reentered','1980-11-26 00:00:00','1974-08-14 00:00:00',251.19,190,13428,125.09,'Highly Ellip' from dual union all
select 7371,'007371','1974-055A','Aeros 2','Aeros B',33658,'Reentered','1975-09-25 00:00:00','1974-08-15 00:00:00',95.1,218,829,97.45,'Sun-Synch' from dual union all
select 7372,'007372','1973-106D','Blok-L','Blok-L',30708,'Reentered','1985-05-24 00:00:00','1974-07-02 00:00:00',733.59,488,40644,62.8,'Molniya' from dual union all
select 7373,'007373','1974-026E','Blok-L','Blok-L',33629,'In Earth orbit','','1974-07-10 00:00:00',732.71,293,40796,62.91,'Molniya' from dual union all
select 7374,'007374','1974-051C','Garbage bag','',33654,'Reentered','1974-07-17 00:00:00','1974-07-17 00:00:00',88.79,176,254,51.59,'Intermediate' from dual union all
select 7375,'007375','1974-055B','FW4S S/N 2376-6','FW4S',33658,'Reentered','1974-10-09 00:00:00','1974-08-16 00:00:00',94.23,215,747,97.44,'Sun-Synch' from dual union all
select 7376,'007376','1974-056A','Molniya-2','Molniya-2',33659,'In Earth orbit','','1974-09-02 00:00:00',717.78,447,39908,62.98,'Molniya' from dual;
insert into SATELLITE
select 7377,'007377','1974-056B','11S510','11S510',33659,'Reentered','1974-08-26 00:00:00','1974-07-30 00:00:00',90.79,213,414,62.84,'Intermediate' from dual union all
select 7378,'007378','1974-056C','BOZ','BOZ',33659,'Reentered','1974-08-28 00:00:00','1974-07-30 00:00:00',90.9,214,423,62.8,'Intermediate' from dual union all
select 7379,'007379','1974-053C','Blenda','Sunshade',33656,'Reentered','1974-07-25 00:00:00','1974-07-24 00:00:00',89.07,164,293,62.77,'Intermediate' from dual union all
select 738,'000738','1964-003B','Altair','Altair X-248-A5DM',10289,'In Earth orbit','','1964-03-19 00:00:00',194.77,2081,7424,46.35,'Medium' from dual union all
select 7380,'007380','1974-055C','Aeros 2 despin weight','',33658,'Reentered','1974-10-15 00:00:00','1974-08-09 00:00:00',95.11,224,824,97.42,'Sun-Synch' from dual union all
select 7381,'007381','1974-053D','KDU','KDU',33656,'Reentered','1974-07-30 00:00:00','1974-07-24 00:00:00',89.26,165,310,62.81,'Intermediate' from dual union all
select 7382,'007382','1974-056D','Blok-L','Blok-L',33659,'Reentered','2017-05-31 00:00:00','1974-08-29 00:00:00',733.94,465,40684,62.81,'Molniya' from dual union all
select 7383,'007383','1974-057A','Kosmos-667','Zenit-4M',33660,'Landed','1974-08-07 00:00:00','1974-07-27 00:00:00',89.42,175,316,64.97,'Intermediate' from dual union all
select 7384,'007384','1974-057B','11A57I','11A57I',33660,'Reentered','1974-07-30 00:00:00','1974-07-27 00:00:00',88.88,169,270,64.96,'Intermediate' from dual union all
select 7385,'007385','1974-058A','Kosmos-668','DS-P1-Yu No. 74',33661,'Reentered','1975-02-21 00:00:00','1974-08-24 00:00:00',91.95,266,475,70.95,'Intermediate' from dual union all
select 7386,'007386','1974-058B','S1M','S1M',33661,'Reentered','1974-11-15 00:00:00','1974-08-24 00:00:00',91.59,268,438,70.95,'Intermediate' from dual union all
select 7387,'007387','1974-029B','Kosmos-651 RLS','US-A RLS',33632,'Reentered','1974-07-30 00:00:00','1974-07-25 00:00:00',89.58,247,260,64.96,'Intermediate' from dual union all
select 7388,'007388','1974-029C','Kosmos-651 DU','US-A DU',33632,'Reentered','1974-09-05 00:00:00','1974-07-27 00:00:00',89.51,242,259,64.85,'Intermediate' from dual union all
select 7389,'007389','1974-059A','Kosmos-669','Zenit-2M',33662,'Landed','1974-08-08 00:00:00','1974-07-28 00:00:00',88.81,205,227,81.32,'Intermediate' from dual union all
select 739,'000739','1963-047H','Insulation blanket','Insulation panel',9278,'In Earth orbit','','1993-03-30 00:00:00',104.74,482,1482,30.42,'Intermediate' from dual union all
select 7390,'007390','1974-059B','11A57I','11A57I',33662,'Reentered','1974-07-30 00:00:00','1974-07-28 00:00:00',88.44,184,210,81.32,'Intermediate' from dual union all
select 7391,'007391','1974-059C','Sensor cover','Zenit-2M sensor cover',33662,'Reentered','1974-07-29 00:00:00','1974-07-27 00:00:00',88.68,194,225,81.31,'Intermediate' from dual union all
select 7392,'007392','1974-060A','Molniya-1S','Molniya-1 11F658 No. 38',33663,'In Earth orbit','','1977-08-20 00:00:00',1435.88,35751,35814,2.47,'Stationary' from dual union all
select 7393,'007393','1974-060B','8S812','8S812',33663,'Reentered','1974-08-01 00:00:00','1974-07-30 00:00:00',88.07,173,185,51.48,'Intermediate' from dual union all
select 7394,'007394','1974-060C','Perekhodnik','Sredniy perekhodnik',33663,'Reentered','1974-07-31 00:00:00','1974-07-30 00:00:00',87.1,131,131,51.47,'Intermediate' from dual union all
select 7395,'007395','1974-032B','Kosmos-654 RLS','US-A RLS',33635,'Reentered','1974-08-04 00:00:00','1974-07-30 00:00:00',89.53,245,258,64.98,'Intermediate' from dual union all
select 7396,'007396','1974-032C','deb Kosmos-654','',33635,'Reentered','1974-08-30 00:00:00','1974-07-30 00:00:00',90.21,191,378,65.1,'Intermediate' from dual union all
select 7397,'007397','1974-032D','Kosmos-654 DU','US-A DU',33635,'Reentered','1974-09-07 00:00:00','1974-08-01 00:00:00',89.52,240,261,64.98,'Intermediate' from dual union all
select 7398,'007398','1974-032E','deb Kosmos-654','',33635,'Reentered','1974-08-30 00:00:00','1974-07-30 00:00:00',89.97,237,309,65.11,'Intermediate' from dual union all
select 7399,'007399','1974-060D','SOZ','SOZ',33663,'Reentered','1979-01-26 00:00:00','1974-08-30 00:00:00',633.92,297,35836,47.23,'GEO Transfer' from dual union all
select 74,'000074','60 Pi 3','Tiros 2 despin weight','Despin weight',6329,'Reentered','1989-05-10 00:00:00','1961-04-29 00:00:00',98.19,611,734,48.52,'Intermediate' from dual union all
select 740,'000740','1964-004A','Echo 2','Echo C',10290,'Reentered','1969-06-07 00:00:00','1964-02-24 00:00:00',108.83,1008,1338,81.48,'Intermediate' from dual union all
select 7400,'007400','1974-060E','SOZ','SOZ',33663,'Reentered','1976-12-11 00:00:00','1974-08-31 00:00:00',635.12,358,35836,47.38,'GEO Transfer' from dual union all
select 7401,'007401','1974-059D','Nauka part','',33662,'Reentered','1974-08-07 00:00:00','1974-08-03 00:00:00',88.64,195,220,81.2,'Intermediate' from dual union all
select 7402,'007402','1974-059E','Nauka part','',33662,'Reentered','1974-08-08 00:00:00','1974-08-03 00:00:00',88.59,187,223,81.24,'Intermediate' from dual union all
select 7403,'007403','1974-059F','Nauka part','',33662,'Reentered','1974-08-07 00:00:00','1974-08-05 00:00:00',88.47,186,212,81.32,'Intermediate' from dual union all
select 7404,'007404','1974-059G','Kapsula Kosmosa-669','Nauka 8KS No. 3',33662,'Reentered','1974-08-11 00:00:00','1974-08-05 00:00:00',88.68,198,221,81.34,'Intermediate' from dual union all
select 7405,'007405','1974-061A','Kosmos-670','Soyuz 7K-S No. 1L',33664,'Landed','1974-08-09 00:00:00','1974-08-08 00:00:00',89.47,209,288,50.55,'Intermediate' from dual union all
select 7406,'007406','1974-061B','11S510','11S510',33664,'Reentered','1974-08-13 00:00:00','1974-08-08 00:00:00',89.12,199,263,50.54,'Intermediate' from dual union all
select 7407,'007407','1974-057C','Blenda','BZA',33660,'Reentered','1974-08-10 00:00:00','1974-08-06 00:00:00',88.83,167,267,64.97,'Intermediate' from dual union all
select 7408,'007408','1974-057D','KDU','KDU',33660,'Reentered','1974-08-18 00:00:00','1974-08-06 00:00:00',88.69,165,255,64.95,'Intermediate' from dual union all
select 7409,'007409','1974-062A','Kosmos-671','Zenit-4MK',33665,'Landed','1974-08-20 00:00:00','1974-08-09 00:00:00',89.29,168,311,62.82,'Intermediate' from dual union all
select 741,'000741','1964-004B','Agena B 6301','Agena B 6301',10290,'In Earth orbit','','1964-03-07 00:00:00',108.83,1039,1307,81.5,'Intermediate' from dual union all
select 7410,'007410','1974-062B','11A57I','11A57I',33665,'Reentered','1974-08-14 00:00:00','1974-08-09 00:00:00',89.51,173,328,62.79,'Intermediate' from dual union all
select 7411,'007411','1974-063A','DMSP 9532','DMSP Block 5C F-1',33666,'In Earth orbit','','1974-09-08 00:00:00',101.69,803,875,98.86,'Sun-Synch' from dual union all
select 7412,'007412','1974-063B','Burner IIA No. 22 Stage 2','Star 26B S/N 007',33666,'In Earth orbit','','1974-09-09 00:00:00',101.64,800,873,98.84,'Sun-Synch' from dual union all
select 7413,'007413','1974-064A','Kosmos-672','Soyuz 7K-TM No. 72-EPSA',33667,'Landed','1974-08-18 00:00:00','1974-08-14 00:00:00',88.95,223,223,51.76,'Intermediate' from dual union all
select 7414,'007414','1974-064B','11S510','11S510',33667,'Reentered','1974-08-14 00:00:00','1974-08-13 00:00:00',88.29,185,195,51.77,'Intermediate' from dual union all
select 7415,'007415','1974-064C','Kosmos-672 part','7K-TM part',33667,'Reentered','1974-08-16 00:00:00','1974-08-12 00:00:00',89.13,198,265,51.73,'Intermediate' from dual union all
select 7416,'007416','1974-065A','OPS 3004','GAMBIT-3 Mission 4343',33668,'Reentered Attached','1974-09-29 00:00:00','1974-08-21 00:00:00',89.84,123,411,110.51,'Retrograde' from dual union all
select 7417,'007417','1974-066A','Kosmos-673','Tselina-D',33669,'Reentered','1991-06-01 00:00:00','1974-09-16 00:00:00',97.09,604,635,81.22,'Intermediate' from dual union all
select 7418,'007418','1974-066B','8A92ME','8A92ME',33669,'Reentered','1999-04-13 00:00:00','1974-09-16 00:00:00',97.22,575,677,81.23,'Intermediate' from dual union all
select 7419,'007419','1974-062C','Blenda','Sunshade',33665,'Reentered','1974-08-23 00:00:00','1974-08-19 00:00:00',89.15,154,311,62.82,'Intermediate' from dual union all
select 742,'000742','1964-004C','A-12 canister half','A-12 canister',10290,'In Earth orbit','','1993-03-31 00:00:00',108.67,1035,1296,81.47,'Intermediate' from dual union all
select 7420,'007420','1974-062D','KDU','KDU',33665,'Reentered','1974-09-09 00:00:00','1974-08-19 00:00:00',89.27,164,313,62.84,'Intermediate' from dual union all
select 7421,'007421','1974-067A','Soyuz-15','Soyuz 7K-T(A9) No. 63',33670,'Landed','1974-08-28 00:00:00','1974-08-27 00:00:00',88.54,171,234,51.57,'Intermediate' from dual union all
select 7422,'007422','1974-067B','11S510','11S510',33670,'Reentered','1974-08-28 00:00:00','1974-08-27 00:00:00',88.47,188,210,51.61,'Intermediate' from dual union all
select 7423,'007423','1974-068A','Kosmos-674','Zenit-4MK',33671,'Landed','1974-09-07 00:00:00','1974-08-31 00:00:00',89.44,175,319,64.99,'Intermediate' from dual union all
select 7424,'007424','1974-069A','Kosmos-675','Geoid',33672,'In Earth orbit','','1974-10-05 00:00:00',113.65,1365,1422,74.06,'Intermediate' from dual union all
select 7425,'007425','1974-068B','11A57I','11A57I',33671,'Reentered','1974-09-04 00:00:00','1974-08-31 00:00:00',88.99,170,279,65,'Intermediate' from dual union all
select 7426,'007426','1974-069B','S3M','S3M',33672,'In Earth orbit','','1974-09-28 00:00:00',113.51,1354,1422,74.06,'Intermediate' from dual union all
select 7427,'007427','1974-070A','ANS','ANS',33673,'Reentered','1977-06-14 00:00:00','1974-09-30 00:00:00',98.86,256,1153,98.04,'Sun-Synch' from dual union all
select 7428,'007428','1974-070B','Star 20 E-14','Star 20 E-14',33673,'Reentered','1975-11-19 00:00:00','1974-09-29 00:00:00',98.65,254,1135,98.04,'Sun-Synch' from dual union all
select 7429,'007429','1974-070C','ANS telescope cover','Dust cover',33673,'Reentered','1975-07-20 00:00:00','1974-10-11 00:00:00',98.15,265,1075,97.65,'Sun-Synch' from dual union all
select 743,'000743','1964-004D','A-12 canister half','A-12 canister',10290,'In Earth orbit','','1993-03-25 00:00:00',108.64,1031,1297,81.53,'Intermediate' from dual union all
select 7430,'007430','1974-070D','ANS despin weight','',33673,'Reentered','1974-10-04 00:00:00','1974-09-06 00:00:00',98.68,262,1130,98.09,'Sun-Synch' from dual union all
select 7431,'007431','1974-068C','KDU','KDU',33671,'Reentered','1974-09-14 00:00:00','1974-09-06 00:00:00',89.28,172,306,65.04,'Intermediate' from dual union all
select 7432,'007432','1974-068D','Blenda','Sunshade',33671,'Reentered','1974-09-10 00:00:00','1974-09-06 00:00:00',89.1,163,297,65.04,'Intermediate' from dual union all
select 7433,'007433','1974-071A','Kosmos-676','Strela-2M',33674,'In Earth orbit','','1974-10-11 00:00:00',100.95,794,814,74.05,'Intermediate' from dual union all
select 7434,'007434','1974-071B','S3M','S3M',33674,'In Earth orbit','','1974-10-16 00:00:00',100.86,783,816,74.05,'Intermediate' from dual union all
select 7435,'007435','1974-072A','Kosmos-677','Strela-1M',33675,'In Earth orbit','','1974-10-22 00:00:00',114.48,1398,1466,74.03,'Intermediate' from dual union all
select 7436,'007436','1974-072B','Kosmos-678','Strela-1M',33675,'In Earth orbit','','1974-11-07 00:00:00',115.98,1466,1534,74.04,'Intermediate' from dual union all
select 7437,'007437','1974-072C','Kosmos-679','Strela-1M',33675,'In Earth orbit','','1974-10-22 00:00:00',115.74,1467,1511,74.03,'Intermediate' from dual union all
select 7438,'007438','1974-072D','Kosmos-680','Strela-1M',33675,'In Earth orbit','','1974-10-19 00:00:00',115.52,1466,1492,74.03,'Intermediate' from dual union all
select 7439,'007439','1974-072E','Kosmos-681','Strela-1M',33675,'In Earth orbit','','1974-10-19 00:00:00',115.29,1466,1471,74.03,'Intermediate' from dual union all
select 744,'000744','1964-005A','Saturn SA-5','Saturn S-IV SA-5',10291,'Reentered','1966-04-30 00:00:00','1964-03-03 00:00:00',94.61,258,741,31.44,'Intermediate' from dual union all
select 7440,'007440','1974-072F','Kosmos-682','Strela-1M',33675,'In Earth orbit','','1974-10-23 00:00:00',115.1,1453,1467,74.03,'Intermediate' from dual union all
select 7441,'007441','1974-072G','Kosmos-683','Strela-1M',33675,'In Earth orbit','','1974-10-29 00:00:00',114.9,1435,1467,74.04,'Intermediate' from dual union all
select 7442,'007442','1974-072H','Kosmos-684','Strela-1M',33675,'In Earth orbit','','1974-10-19 00:00:00',114.69,1416,1467,74.03,'Intermediate' from dual union all
select 7443,'007443','1974-072J','S3M','S3M',33675,'In Earth orbit','','1974-10-19 00:00:00',117.75,1471,1688,74.03,'Intermediate' from dual union all
select 7444,'007444','1974-073B','11A57I','11A57I',33676,'Reentered','1974-09-26 00:00:00','1974-09-22 00:00:00',89.04,194,261,65,'Intermediate' from dual union all
select 7445,'007445','1974-073A','Kosmos-685','Zenit-2M',33676,'Landed','1974-10-02 00:00:00','1974-09-22 00:00:00',89.35,202,283,64.98,'Intermediate' from dual union all
select 7446,'007446','1974-073C','Sensor cover','Zenit-2M sensor cover',33676,'Reentered','1974-09-25 00:00:00','1974-09-22 00:00:00',89.35,217,268,64.97,'Intermediate' from dual union all
select 7447,'007447','1974-074A','Kosmos-686','DS-P1-Yu No. 72',33677,'Reentered','1975-05-01 00:00:00','1974-10-27 00:00:00',91.81,266,461,70.99,'Intermediate' from dual union all
select 7448,'007448','1974-074B','S1M','S1M',33677,'Reentered','1974-10-29 00:00:00','1974-10-03 00:00:00',91.43,256,434,70.93,'Intermediate' from dual union all
select 7449,'007449','1974-074C','deb Kosmos-686','',33677,'Reentered','1975-03-19 00:00:00','1974-11-03 00:00:00',92.39,317,467,71.12,'Intermediate' from dual union all
select 745,'000745','1963-038E','deb Transit VBN-2','',9269,'In Earth orbit','','1993-03-25 00:00:00',106.59,1052,1086,89.95,'Polar' from dual union all
select 7450,'007450','1974-074D','deb Kosmos-686','',33677,'Reentered','1976-01-01 00:00:00','1974-10-29 00:00:00',92.69,315,498,71.15,'Intermediate' from dual union all
select 7451,'007451','1974-074E','deb Kosmos-686','',33677,'Reentered','1974-11-13 00:00:00','1974-10-03 00:00:00',92.25,284,485,71.24,'Intermediate' from dual union all
select 7452,'007452','1974-074F','deb Kosmos-686','',33677,'Reentered','1975-03-18 00:00:00','1974-10-27 00:00:00',92.27,317,455,71.14,'Intermediate' from dual union all
select 7453,'007453','1974-074G','deb Kosmos-686','',33677,'Reentered','1975-07-05 00:00:00','1974-10-27 00:00:00',92.64,310,498,71.21,'Intermediate' from dual union all
select 7454,'007454','1974-074H','deb Kosmos-686','',33677,'Reentered','1974-11-03 00:00:00','1974-10-09 00:00:00',92.62,315,490,71.14,'Intermediate' from dual union all
select 7455,'007455','1974-074J','deb Kosmos-686','',33677,'Reentered','1974-11-03 00:00:00','1974-10-04 00:00:00',92.91,313,522,71.2,'Intermediate' from dual union all
select 7456,'007456','1974-074K','deb Kosmos-686','',33677,'Reentered','1974-11-03 00:00:00','1974-10-04 00:00:00',92.25,305,465,71.12,'Intermediate' from dual union all
select 7457,'007457','1974-074L','deb Kosmos-686','',33677,'Reentered','1974-11-17 00:00:00','1974-10-04 00:00:00',91.93,309,430,71.26,'Intermediate' from dual union all
select 7458,'007458','1974-074M','deb Kosmos-686','',33677,'Reentered','1974-11-26 00:00:00','1974-10-26 00:00:00',91.84,303,426,71.11,'Intermediate' from dual union all
select 7459,'007459','1974-074N','deb Kosmos-686','',33677,'Reentered','1975-05-05 00:00:00','1974-10-26 00:00:00',92.29,304,470,71.14,'Intermediate' from dual union all
select 746,'000746','1964-006A','Elektron-1','2D No. 1',10292,'In Earth orbit','','1964-02-29 00:00:00',169.29,394,7123,60.83,'Medium' from dual union all
select 7460,'007460','1974-074P','deb Kosmos-686','',33677,'Reentered','1974-10-20 00:00:00','1974-10-03 00:00:00',92.63,331,476,71.32,'Intermediate' from dual union all
select 7461,'007461','1974-074Q','deb Kosmos-686','',33677,'Reentered','1974-10-18 00:00:00','1974-10-04 00:00:00',91.13,264,395,71.26,'Intermediate' from dual union all
select 7462,'007462','1974-074R','deb Kosmos-686','',33677,'Reentered','1974-10-23 00:00:00','1974-10-04 00:00:00',92.37,302,479,71.21,'Intermediate' from dual union all
select 7463,'007463','1974-074S','deb Kosmos-686','',33677,'Reentered','1974-11-10 00:00:00','1974-10-03 00:00:00',91.87,311,422,71.21,'Intermediate' from dual union all
select 7464,'007464','1974-074T','deb Kosmos-686','',33677,'Reentered','1974-11-01 00:00:00','1974-10-04 00:00:00',91.7,285,431,71.21,'Intermediate' from dual union all
select 7465,'007465','1974-074U','deb Kosmos-686','',33677,'Reentered','1975-02-21 00:00:00','1974-10-26 00:00:00',92.5,298,497,71.16,'Intermediate' from dual union all
select 7466,'007466','1974-075A','Westar 2','Westar 2',33678,'In Earth orbit','','1977-01-31 00:00:00',1436.06,35771,35801,.05,'Stationary' from dual;
insert into SATELLITE
select 7467,'007467','1974-075B','Delta 103','Delta SSPU DSV-3P-4',33678,'Reentered','1978-12-23 00:00:00','1974-11-09 00:00:00',123.24,230,3421,27.31,'Medium' from dual union all
select 7468,'007468','1974-075C','Star 37C S/N 40026','Star 37C',33678,'Reentered','1994-07-12 00:00:00','1974-11-20 00:00:00',642.77,241,36346,24.81,'GEO Transfer' from dual union all
select 7469,'007469','1974-076A','Kosmos-687','Vektor',33679,'Reentered','1978-02-05 00:00:00','1974-11-10 00:00:00',94.37,287,690,73.99,'Intermediate' from dual union all
select 747,'000747','1964-007A','Ranger 6','RA-6',10293,'Deep Space','1964-02-01 00:00:00','1964-01-30 00:00:00',22816.65,200,520000,29,'Cislunar' from dual union all
select 7470,'007470','1974-076B','S3M','S3M',33679,'Reentered','1977-01-04 00:00:00','1974-11-11 00:00:00',94.27,286,681,74,'Intermediate' from dual union all
select 7471,'007471','1974-077A','Ariel 5','UK 5',33680,'Reentered','1980-03-14 00:00:00','1974-11-15 00:00:00',95.21,506,552,2.88,'Equatorial' from dual union all
select 7472,'007472','1974-077B','Star 20 E-13','Star 20 E-13',33680,'Reentered','1979-05-13 00:00:00','1974-11-15 00:00:00',95.2,506,551,2.88,'Equatorial' from dual union all
select 7473,'007473','1974-078A','Kosmos-688','Zenit-4MK',33681,'Landed','1974-10-30 00:00:00','1974-10-20 00:00:00',89.75,179,345,62.82,'Intermediate' from dual union all
select 7474,'007474','1974-078B','11A57I','11A57I',33681,'Reentered','1974-10-24 00:00:00','1974-10-20 00:00:00',89.26,171,305,62.81,'Intermediate' from dual union all
select 7475,'007475','1974-078C','Sensor cover','Zenit-4MK sensor cover',33681,'Reentered','1974-10-23 00:00:00','1974-10-19 00:00:00',89.02,167,285,62.78,'Intermediate' from dual union all
select 7476,'007476','1974-079A','Kosmos-689','Zaliv',33682,'In Earth orbit','','1974-11-17 00:00:00',105.05,975,1019,82.97,'Intermediate' from dual union all
select 7477,'007477','1974-079B','S3M','S3M',33682,'In Earth orbit','','1974-11-17 00:00:00',104.94,974,1010,82.94,'Intermediate' from dual union all
select 7478,'007478','1974-080A','Kosmos-690','Bion No. 2',33683,'Landed','1974-11-12 00:00:00','1974-10-29 00:00:00',90.27,214,362,62.81,'Intermediate' from dual union all
select 7479,'007479','1974-080B','11S510','11S510',33683,'Reentered','1974-11-10 00:00:00','1974-10-29 00:00:00',89.88,209,327,62.8,'Intermediate' from dual union all
select 748,'000748','1964-006B','Elektron-2','2D No. 2',10292,'Reentered','1997-04-22 00:00:00','1964-03-25 00:00:00',1356.35,408,68014,60.46,'Highly Ellip' from dual union all
select 7480,'007480','1974-081A','Molniya-1','Molniya-1 No. 26',33684,'Reentered','1985-12-29 00:00:00','1974-11-25 00:00:00',717.76,672,39682,62.84,'Molniya' from dual union all
select 7481,'007481','1974-081B','11S510','11S510',33684,'Reentered','1974-12-31 00:00:00','1974-11-23 00:00:00',91.67,211,502,62.84,'Intermediate' from dual union all
select 7482,'007482','1974-081C','BOZ','BOZ',33684,'Reentered','1975-01-03 00:00:00','1974-11-23 00:00:00',91.88,204,530,62.81,'Intermediate' from dual union all
select 7483,'007483','1974-082A','Kosmos-691','Zenit-4MK',33685,'Landed','1974-11-06 00:00:00','1974-10-27 00:00:00',89.47,171,326,65.04,'Intermediate' from dual union all
select 7484,'007484','1974-082B','11S510I','11S510I',33685,'Reentered','1974-10-29 00:00:00','1974-10-27 00:00:00',89.03,168,285,65.04,'Intermediate' from dual union all
select 7485,'007485','1974-081D','Blok-L','Blok-L',33684,'Reentered','1986-10-27 00:00:00','1974-12-18 00:00:00',732,687,40367,62.91,'Molniya' from dual union all
select 7486,'007486','1974-081E','deb Blok-I','',33684,'Reentered','1974-11-04 00:00:00','1974-10-26 00:00:00',93.05,211,637,62.85,'Intermediate' from dual union all
select 7487,'007487','1973-086FT','deb Delta 98','',30688,'Reentered','2001-06-13 00:00:00','1992-02-07 00:00:00',102.57,699,1061,103.18,'Retrograde' from dual union all
select 7488,'007488','1973-086FU','deb Delta 98','',30688,'In Earth orbit','','1992-02-07 00:00:00',120.76,1254,2176,102.62,'Medium' from dual union all
select 7489,'007489','1973-086FV','deb Delta 98','',30688,'In Earth orbit','','1992-02-05 00:00:00',117.15,1504,1602,102.53,'Sun-Synch' from dual union all
select 749,'000749','1964-004E','Short Nimbus Shroud Half','Short Nimbus Clamshell',10290,'Reentered','1966-02-23 00:00:00','1964-02-26 00:00:00',100.56,299,1272,81.58,'Intermediate' from dual union all
select 7490,'007490','1974-083A','Meteor','Meteor-M  No. 33',33686,'In Earth orbit','','1974-11-27 00:00:00',102.4,840,905,81.19,'Intermediate' from dual union all
select 7491,'007491','1974-084A','Luna-23 KT','E-8-5M No. 410',33687,'Deep Space','1974-10-29 00:00:00','1974-10-28 00:00:00',15562.18,200,400000,51.5,'Cislunar' from dual union all
select 7492,'007492','1974-084B','SOZ','SOZ',33687,'Reentered','1974-11-01 00:00:00','1974-10-30 00:00:00',88.26,170,207,51.5,'Intermediate' from dual union all
select 7493,'007493','1974-083B','8A92ME','8A92ME',33686,'In Earth orbit','','1974-11-30 00:00:00',102.51,842,912,81.18,'Intermediate' from dual union all
select 7494,'007494','1974-084C','SOZ','SOZ',33687,'Reentered','1974-11-01 00:00:00','1974-10-30 00:00:00',88.46,172,224,51.54,'Intermediate' from dual union all
select 7495,'007495','1974-085A','OPS 7122','HEXAGON SV-9',33688,'Deorbited','1975-03-18 00:00:00','1974-11-28 00:00:00',88.77,155,273,96.66,'Sun-Synch' from dual union all
select 7496,'007496','1974-078D','KDU','KDU',33681,'Reentered','1974-10-31 00:00:00','1974-10-29 00:00:00',88.85,144,292,62.81,'Intermediate' from dual union all
select 7497,'007497','1974-085D','Titan 23D-4 Stage 2','LR91AJ11 23D-4',33688,'Reentered','1974-10-31 00:00:00','1974-10-30 00:00:00',88.62,154,259,96.7,'Sun-Synch' from dual union all
select 7498,'007498','1974-085B','RAQUEL I','P-11 No. 4429',33688,'Reentered','1980-01-23 00:00:00','1974-12-18 00:00:00',95.13,518,532,96.06,'Sun-Synch' from dual union all
select 7499,'007499','1974-085C','S3-1','S73-5',33688,'Reentered','1975-05-26 00:00:00','1974-11-30 00:00:00',123.03,148,3483,96.99,'Medium' from dual union all
select 75,'000075','60 Pi 4','Tiros 2 despin weight','Despin weight',6329,'Reentered','1990-01-25 00:00:00','1961-03-26 00:00:00',98.33,615,742,48.57,'Intermediate' from dual union all
select 750,'000750','1964-006C','Elektron-1 motor','Adapter',10292,'In Earth orbit','','1993-03-24 00:00:00',139.7,399,4682,60.83,'Medium' from dual union all
select 7500,'007500','1974-086A','Interkosmos-12','DS-U2-IK No. 4',33689,'Reentered','1975-07-11 00:00:00','1974-11-30 00:00:00',93.6,240,661,74.01,'Intermediate' from dual union all
select 7501,'007501','1974-086B','S3M','S3M',33689,'Reentered','1975-07-23 00:00:00','1974-12-01 00:00:00',93.56,227,671,74.01,'Intermediate' from dual union all
select 7502,'007502','1974-087A','Kosmos-692','Zenit-2M',33690,'Landed','1974-11-13 00:00:00','1974-11-03 00:00:00',89.39,194,295,62.81,'Intermediate' from dual union all
select 7503,'007503','1974-087B','11A57I','11A57I',33690,'Reentered','1974-11-06 00:00:00','1974-11-03 00:00:00',89.07,183,274,62.79,'Intermediate' from dual union all
select 7504,'007504','1974-087C','Sensor cover','Zenit-2M sensor cover',33690,'Reentered','1974-11-05 00:00:00','1974-11-02 00:00:00',89.68,194,323,62.79,'Intermediate' from dual union all
select 7505,'007505','1974-087D','Sensor cover','Zenit-2M sensor cover',33690,'Reentered','1974-11-05 00:00:00','1974-11-01 00:00:00',89.63,206,307,62.82,'Intermediate' from dual union all
select 7506,'007506','1974-087E','Sensor cover','Zenit-2M sensor cover',33690,'Reentered','1974-11-05 00:00:00','1974-11-01 00:00:00',89.04,190,265,62.77,'Intermediate' from dual union all
select 7507,'007507','1974-086C','deb Interkosmos-12','',33689,'Reentered','1974-11-11 00:00:00','1974-11-02 00:00:00',93.98,238,700,74.08,'Intermediate' from dual union all
select 7508,'007508','1974-086D','deb Interkosmos-12','',33689,'Reentered','1974-11-11 00:00:00','1974-11-02 00:00:00',93.71,237,675,74.03,'Intermediate' from dual union all
select 7509,'007509','1974-088A','Kosmos-693','Zenit-4MT',33691,'Landed','1974-11-16 00:00:00','1974-11-07 00:00:00',89,214,237,81.33,'Intermediate' from dual union all
select 751,'000751','1964-006D','8K72E','8K72KE',10292,'Reentered','1998-02-15 00:00:00','1991-09-22 00:00:00',1384.07,9924,59600,62.76,'Near-synch' from dual union all
select 7510,'007510','1974-088B','11S510','11S510',33691,'Reentered','1974-11-09 00:00:00','1974-11-06 00:00:00',88.72,203,220,81.32,'Intermediate' from dual union all
select 7511,'007511','1974-082C','KDU','KDU',33685,'Reentered','1974-11-10 00:00:00','1974-11-04 00:00:00',89.19,165,303,65.03,'Intermediate' from dual union all
select 7512,'007512','1974-082D','Blenda','Sunshade',33685,'Reentered','1974-11-06 00:00:00','1974-11-04 00:00:00',88.95,153,293,65.03,'Intermediate' from dual union all
select 7513,'007513','1974-088C','Sensor cover','Zenit-4MT sensor cover',33691,'Reentered','1974-11-06 00:00:00','1974-11-04 00:00:00',88.38,185,203,81.37,'Intermediate' from dual union all
select 7514,'007514','1974-088D','Sensor cover','Zenit-4MT sensor cover',33691,'Reentered','1974-11-16 00:00:00','1974-11-06 00:00:00',89.02,213,240,81.32,'Intermediate' from dual union all
select 7515,'007515','1974-035C','deb Kosmos-655','Tselina-O part',33638,'Reentered','1981-07-10 00:00:00','1974-10-01 00:00:00',95.26,517,545,73.99,'Intermediate' from dual union all
select 7516,'007516','1974-035D','deb Kosmos-655','Tselina-O part',33638,'Reentered','1981-04-22 00:00:00','1974-10-01 00:00:00',95.67,534,568,73.96,'Intermediate' from dual union all
select 7517,'007517','1974-035E','deb Kosmos-655','Tselina-O part',33638,'Reentered','1979-10-31 00:00:00','1974-09-30 00:00:00',95.07,491,553,74.03,'Intermediate' from dual union all
select 7518,'007518','1974-035F','deb Kosmos-655','Tselina-O part',33638,'Reentered','1979-10-21 00:00:00','1974-10-01 00:00:00',94.77,492,522,74.04,'Intermediate' from dual union all
select 7519,'007519','1974-035G','deb Kosmos-655','Tselina-O part',33638,'Reentered','1979-09-08 00:00:00','1974-10-01 00:00:00',94.76,488,526,74.04,'Intermediate' from dual union all
select 752,'000752','1964-008A','OPS 3444','CORONA J-5',10294,'Reentered Attached','1964-03-09 00:00:00','1964-02-26 00:00:00',90.43,182,410,74.95,'Intermediate' from dual union all
select 7520,'007520','1974-087F','FEU-170 part','FEU-170 part',33690,'Reentered','1974-11-23 00:00:00','1974-11-11 00:00:00',89.21,191,279,62.81,'Intermediate' from dual union all
select 7521,'007521','1974-080C','KBE part','KBE part',33683,'Reentered','1974-11-21 00:00:00','1974-11-12 00:00:00',89.97,208,338,62.79,'Intermediate' from dual union all
select 7522,'007522','1974-080D','Blenda','KBE part',33683,'Reentered','1974-12-03 00:00:00','1974-11-14 00:00:00',92.06,311,441,62.8,'Intermediate' from dual union all
select 7523,'007523','1974-080E','KB electropitaniya','KBE',33683,'Reentered','1974-12-14 00:00:00','1974-11-12 00:00:00',90.04,212,341,62.8,'Intermediate' from dual union all
select 7524,'007524','1974-080F','KBE part','KBE part',33683,'Reentered','1974-11-16 00:00:00','1974-11-12 00:00:00',91.84,309,421,63.16,'Intermediate' from dual union all
select 7525,'007525','1974-087G','FEU-170 part','FEU-170 part',33690,'Reentered','1974-11-18 00:00:00','1974-11-12 00:00:00',89.13,190,273,62.82,'Intermediate' from dual union all
select 7526,'007526','1974-074V','deb Kosmos-686','',33677,'Reentered','1974-11-27 00:00:00','1974-11-12 00:00:00',90.3,239,340,71.23,'Intermediate' from dual union all
select 7527,'007527','1974-080G','PTDU cover','KBE part',33683,'Reentered','1974-11-23 00:00:00','1974-11-12 00:00:00',93.8,326,595,62.87,'Intermediate' from dual union all
select 7528,'007528','1974-080H','PTDU cover','KBE part',33683,'Reentered','1974-11-24 00:00:00','1974-11-12 00:00:00',93.5,326,566,62.68,'Intermediate' from dual union all
select 7529,'007529','1974-089A','NOAA 4','ITOS G',33692,'In Earth orbit','','1974-12-15 00:00:00',114.93,1445,1460,101.74,'Sun-Synch' from dual union all
select 753,'000753','1963-049F','deb Transit VBN-2','',9280,'In Earth orbit','','1993-03-25 00:00:00',106.64,1045,1096,90.07,'Polar' from dual union all
select 7530,'007530','1974-089B','AMSAT-OSCAR-7','OSCAR 7',33692,'In Earth orbit','','1974-12-19 00:00:00',114.89,1440,1460,101.73,'Sun-Synch' from dual union all
select 7531,'007531','1974-089C','Intasat','Intasat',33692,'In Earth orbit','','1974-12-20 00:00:00',114.9,1442,1460,101.73,'Sun-Synch' from dual union all
select 7532,'007532','1974-089D','Delta 104','Delta SSPU DSV-3P-4',33692,'Exploded','1975-08-20 00:00:00','1974-12-20 00:00:00',114.93,1445,1460,101.73,'Sun-Synch' from dual union all
select 7533,'007533','1974-090A','Kosmos-694','Zenit-4MK',33693,'Landed','1974-11-29 00:00:00','1974-11-18 00:00:00',89.31,171,310,72.84,'Intermediate' from dual union all
select 7534,'007534','1974-088E','KB electropitaniya','KBE',33691,'Reentered','1974-11-17 00:00:00','1974-11-16 00:00:00',88.44,189,206,81.33,'Intermediate' from dual union all
select 7535,'007535','1974-090B','11A57I','11A57I',33693,'Reentered','1974-11-25 00:00:00','1974-11-18 00:00:00',89.5,199,301,72.83,'Intermediate' from dual union all
select 7536,'007536','1974-090C','Sensor cover','Zenit-4MK sensor cover',33693,'Reentered','1974-11-17 00:00:00','1974-11-16 00:00:00',89.25,193,282,72.8,'Intermediate' from dual union all
select 7537,'007537','1974-090D','Sensor cover','Zenit-4MK sensor cover',33693,'Reentered','1974-11-18 00:00:00','1974-11-17 00:00:00',89.04,177,278,72.81,'Intermediate' from dual union all
select 7538,'007538','1974-091A','Kosmos-695','DS-P1-Yu No. 73',33694,'Reentered','1975-07-15 00:00:00','1974-12-21 00:00:00',91.71,268,449,70.99,'Intermediate' from dual union all
select 7539,'007539','1974-091B','S1M','S1M',33694,'Reentered','1975-03-15 00:00:00','1974-12-20 00:00:00',91.41,265,423,70.98,'Intermediate' from dual union all
select 754,'000754','1964-009A','OPS 2423','GAMBIT SV 955',10295,'Deorbited','1964-03-01 00:00:00','1964-02-27 00:00:00',87.95,159,187,95.6,'Sun-Synch' from dual union all
select 7540,'007540','1974-092A','Molniya-3','Molniya-3 No. 11',33695,'Reentered','1986-05-15 00:00:00','1975-02-05 00:00:00',717.76,686,39668,62.87,'Molniya' from dual union all
select 7541,'007541','1974-092B','11S510','11S510',33695,'Reentered','1975-01-27 00:00:00','1974-12-21 00:00:00',91.49,208,487,62.83,'Intermediate' from dual union all
select 7542,'007542','1974-092C','BOZ','BOZ',33695,'Reentered','1975-01-11 00:00:00','1974-12-01 00:00:00',92.31,196,580,62.83,'Intermediate' from dual union all
select 7543,'007543','1974-092D','deb Blok-I','',33695,'Reentered','1975-01-10 00:00:00','1974-11-28 00:00:00',92.47,197,595,62.82,'Intermediate' from dual union all
select 7544,'007544','1974-093A','INTELSAT IV F8','INTELSAT IV F8',33696,'In Earth orbit','','1977-01-25 00:00:00',1435.96,35717,35851,.13,'Stationary' from dual union all
select 7545,'007545','1974-093B','Centaur D-1A AC-32','Centaur D-1A AC-32',33696,'In Earth orbit','','1974-12-28 00:00:00',654.27,569,36605,25.86,'GEO Transfer' from dual union all
select 7546,'007546','1974-092E','Blok-L','Blok-L',33695,'Reentered','1986-03-09 00:00:00','1974-12-22 00:00:00',733.77,629,40512,62.84,'Molniya' from dual union all
select 7547,'007547','1974-094A','Skynet IIB','Skynet IIB',33697,'In Earth orbit','','1977-03-01 00:00:00',1435.22,35757,35782,.82,'Drift GEO' from dual union all
select 7548,'007548','1974-094B','Delta 105 clamp band half','',33697,'Reentered','1975-01-07 00:00:00','1974-12-05 00:00:00',96.83,181,1033,28.17,'Intermediate' from dual union all
select 7549,'007549','1974-094C','Delta 105 clamp band half','',33697,'Reentered','1974-12-26 00:00:00','1974-12-08 00:00:00',96.63,179,1016,28.21,'Intermediate' from dual union all
select 755,'000755','1964-008B','Fairing Adapter','Fairing',10294,'Reentered','1964-02-26 00:00:00','1964-02-26 00:00:00',88.05,175,181,74.9,'Intermediate' from dual union all
select 7550,'007550','1974-094D','Delta 105','Delta SSPU DSV-3P-4',33697,'Reentered','1989-12-08 00:00:00','1974-12-23 00:00:00',123.27,311,3342,28.28,'Medium' from dual union all
select 7551,'007551','1974-095A','Kosmos-696','Zenit-2M',33698,'Landed','1974-12-09 00:00:00','1974-11-29 00:00:00',89.72,203,318,72.84,'Intermediate' from dual union all
select 7552,'007552','1974-095B','11A57I','11A57I',33698,'Reentered','1974-12-05 00:00:00','1974-11-29 00:00:00',89.46,182,314,72.85,'Intermediate' from dual union all
select 7553,'007553','1974-095C','Sensor cover','Zenit-2M sensor cover',33698,'Reentered','1974-11-29 00:00:00','1974-11-28 00:00:00',88.91,191,251,72.82,'Intermediate' from dual union all
select 7554,'007554','1974-095D','Sensor cover','Zenit-2M sensor cover',33698,'Reentered','1974-12-03 00:00:00','1974-11-29 00:00:00',90.03,198,354,72.84,'Intermediate' from dual union all
select 7555,'007555','1974-090E','KDU','KDU',33693,'Reentered','1974-12-03 00:00:00','1974-11-28 00:00:00',89.35,166,319,72.83,'Intermediate' from dual union all
select 7556,'007556','1974-090F','Blenda','Sunshade',33693,'Reentered','1974-11-30 00:00:00','1974-11-28 00:00:00',89.13,165,298,72.81,'Intermediate' from dual;
insert into SATELLITE
select 7557,'007557','1974-090G','KDU part','KDU part',33693,'Reentered','1974-12-03 00:00:00','1974-11-29 00:00:00',89.22,160,312,72.82,'Intermediate' from dual union all
select 7558,'007558','1966-076E','NNS O-10 insulation','',13483,'Reentered','1981-09-02 00:00:00','1974-11-27 00:00:00',106.7,1057,1091,88.85,'Polar' from dual union all
select 7559,'007559','1973-088E','deb USAF satellite','',30690,'In Earth orbit','','1974-01-02 00:00:00',114.71,1405,1480,96.8,'Sun-Synch' from dual union all
select 756,'000756','1964-009B','Agena D 4803','Agena D',10295,'Reentered','1964-02-26 00:00:00','1964-02-25 00:00:00',88.52,176,227,95.6,'Sun-Synch' from dual union all
select 7560,'007560','1972-062D','deb Kosmos-514','',27994,'In Earth orbit','','1972-12-05 00:00:00',103.26,856,969,82.98,'Intermediate' from dual union all
select 7561,'007561','1974-096A','Soyuz-16','Soyuz 7K-TM No. 73-EPSA',33699,'Landed','1974-12-08 00:00:00','1974-12-04 00:00:00',88.95,218,228,51.77,'Intermediate' from dual union all
select 7562,'007562','1974-096B','11S510','11S510',33699,'Reentered','1974-12-07 00:00:00','1974-12-04 00:00:00',88.95,180,266,51.8,'Intermediate' from dual union all
select 7563,'007563','1974-096C','Soyuz part','Soyuz part',33699,'Reentered','1974-12-08 00:00:00','1974-12-02 00:00:00',88.46,185,212,51.75,'Intermediate' from dual union all
select 7564,'007564','1974-096D','Soyuz part','Soyuz part',33699,'Reentered','1974-12-03 00:00:00','1974-12-02 00:00:00',92.15,185,575,51.75,'Intermediate' from dual union all
select 7565,'007565','1974-096E','APAS simulator','APAS',33699,'Reentered','1974-12-08 00:00:00','1974-12-07 00:00:00',88.55,187,219,51.8,'Intermediate' from dual union all
select 7566,'007566','1974-096F','APAS part','APAS',33699,'Reentered','1974-12-08 00:00:00','1974-12-07 00:00:00',88.52,191,212,51.76,'Intermediate' from dual union all
select 7567,'007567','1974-097A','Helios 1','Helios 1',33700,'Deep Space','1974-12-10 00:00:00','1974-12-10 00:00:00',49.31,289,21954,29.8,'Earth Escape' from dual union all
select 7568,'007568','1974-097B','Centaur TC-2','Centaur D-1T TC-2',33700,'Deep Space','1974-12-11 00:00:00','1974-12-10 00:00:00',4175.24,1764,157260,31.77,'Deep Space' from dual union all
select 7569,'007569','1974-097C','Star 37E S/N 40028','Star 37E',33700,'Deep Space','1974-12-10 00:00:00','1974-12-10 00:00:00',49.31,289,21954,29.8,'Earth Escape' from dual union all
select 757,'000757','1964-010A','Kosmos-25','DS-P1 No. 4',10296,'Reentered','1964-11-21 00:00:00','1964-03-29 00:00:00',92.09,260,494,49.05,'Intermediate' from dual union all
select 7570,'007570','1974-097D','Star 37 yo weight','',33700,'Deep Space','1974-12-10 00:00:00','1974-12-10 00:00:00',49.31,289,21954,29.8,'Earth Escape' from dual union all
select 7571,'007571','1974-098A','Kosmos-697','Yantar''-2K No. 2',33701,'Landed','1974-12-25 00:00:00','1974-12-15 00:00:00',90.1,172,387,62.8,'Intermediate' from dual union all
select 7572,'007572','1974-098B','11S510','11S510',33701,'Reentered','1974-12-21 00:00:00','1974-12-15 00:00:00',89.81,169,362,62.8,'Intermediate' from dual union all
select 7573,'007573','1974-098C','Kosmos-697 part','Feniks part',33701,'Reentered','1974-12-16 00:00:00','1974-12-15 00:00:00',88.89,157,283,62.83,'Intermediate' from dual union all
select 7574,'007574','1974-099A','Meteor','Meteor-M  No. 32',33702,'In Earth orbit','','1975-01-16 00:00:00',102.32,841,896,81.24,'Intermediate' from dual union all
select 7575,'007575','1974-099B','8A92ME','8A92ME',33702,'In Earth orbit','','1975-01-17 00:00:00',102.31,816,920,81.25,'Intermediate' from dual union all
select 7576,'007576','1974-100A','Kosmos-698','Tselina-OM',33703,'Reentered','1980-12-09 00:00:00','1975-01-19 00:00:00',95.26,512,551,74.04,'Intermediate' from dual union all
select 7577,'007577','1974-100B','S3M','S3M',33703,'Reentered','1980-11-08 00:00:00','1975-01-17 00:00:00',95.18,499,556,74.04,'Intermediate' from dual union all
select 7578,'007578','1974-101A','Symphonie 1','Symphonie MV1',33704,'In Earth orbit','','1977-11-10 00:00:00',1436.16,35781,35795,.15,'Stationary' from dual union all
select 7579,'007579','1974-101B','Delta 106','Delta SSPU DSV-3P-4',33704,'Reentered','1978-04-19 00:00:00','1975-01-23 00:00:00',98.53,284,1093,27,'Intermediate' from dual union all
select 758,'000758','1964-010B','S1','S1',10296,'Reentered','1964-06-18 00:00:00','1964-06-14 00:00:00',88.88,197,242,49.08,'Intermediate' from dual union all
select 7580,'007580','1974-101C','Delta 106 clamp band half','',33704,'Reentered','1977-04-24 00:00:00','1975-01-19 00:00:00',98.78,285,1116,27.03,'Intermediate' from dual union all
select 7581,'007581','1974-101D','Delta 106 clamp band half','',33704,'Reentered','1977-01-02 00:00:00','1975-01-18 00:00:00',98.51,287,1087,27.02,'Intermediate' from dual union all
select 7582,'007582','1974-100C','deb Kosmos-698','Tselina-O part',33703,'Reentered','1977-03-06 00:00:00','1975-01-18 00:00:00',94.91,484,545,74.07,'Intermediate' from dual union all
select 7583,'007583','1974-102A','Molniya-2','Molniya-2',33705,'Reentered','1988-07-07 00:00:00','1975-01-23 00:00:00',718.49,597,39793,62.89,'Molniya' from dual union all
select 7584,'007584','1974-102B','11S510','11S510',33705,'Reentered','1975-03-12 00:00:00','1975-01-20 00:00:00',91.69,212,503,62.85,'Intermediate' from dual union all
select 7585,'007585','1974-102C','BOZ','BOZ',33705,'Reentered','1975-03-08 00:00:00','1975-01-20 00:00:00',91.86,205,527,62.81,'Intermediate' from dual union all
select 7586,'007586','1974-102D','Blok-L','Blok-L',33705,'Reentered','1989-01-05 00:00:00','1975-02-01 00:00:00',734.04,588,40566,62.89,'Molniya' from dual union all
select 7587,'007587','1974-103A','Kosmos-699','US-P',33706,'Reentered','1977-10-16 00:00:00','1975-01-23 00:00:00',93.26,427,442,65.03,'Intermediate' from dual union all
select 7588,'007588','1974-103B','11S692','11S692',33706,'Reentered','1974-12-25 00:00:00','1974-12-24 00:00:00',89.2,113,356,65.06,'Intermediate' from dual union all
select 7589,'007589','1974-098D','Kosmos-697 part','Feniks AO part',33701,'Reentered','1974-12-26 00:00:00','1974-12-25 00:00:00',89.83,175,358,62.8,'Intermediate' from dual union all
select 759,'000759','1964-011A','OPS 3722','Project 698BK Group 1-A',10297,'Reentered Attached','1969-02-19 00:00:00','1964-03-30 00:00:00',94.63,493,508,82.07,'Intermediate' from dual union all
select 7590,'007590','1974-098E','Kosmos-697 part','Feniks AO part',33701,'Reentered','1974-12-26 00:00:00','1974-12-25 00:00:00',89.75,170,355,62.85,'Intermediate' from dual union all
select 7591,'007591','1974-104A','Salyut-4','DOS 4 (17K No. 124)',33707,'Deorbited','1977-02-02 00:00:00','1975-01-26 00:00:00',91.36,336,347,51.58,'Intermediate' from dual union all
select 7592,'007592','1974-104B','8S812','8S812',33707,'Reentered','1975-01-01 00:00:00','1974-12-29 00:00:00',88.65,205,211,51.58,'Intermediate' from dual union all
select 7593,'007593','1974-105A','Kosmos-700','Tsiklon-B',33708,'In Earth orbit','','1975-01-26 00:00:00',104.72,964,999,82.95,'Intermediate' from dual union all
select 7594,'007594','1974-105B','S3M','S3M',33708,'In Earth orbit','','1975-01-25 00:00:00',104.6,965,987,82.94,'Intermediate' from dual union all
select 7595,'007595','1974-104C','Salyut-4 part','Adapter',33707,'Reentered','1974-12-28 00:00:00','1974-12-27 00:00:00',89.32,225,257,51.6,'Intermediate' from dual union all
select 7596,'007596','1974-106A','Kosmos-701','Zenit-4MK',33709,'Landed','1975-01-09 00:00:00','1974-12-29 00:00:00',89.33,170,313,71.39,'Intermediate' from dual union all
select 7597,'007597','1974-106B','11A57I','11A57I',33709,'Reentered','1975-01-07 00:00:00','1974-12-29 00:00:00',89.51,198,302,71.39,'Intermediate' from dual union all
select 7598,'007598','1974-104D','Salyut-4 part','Telescope cover',33707,'Reentered','1975-01-22 00:00:00','1974-12-29 00:00:00',90.98,266,380,51.65,'Intermediate' from dual union all
select 7599,'007599','1974-106C','Sensor cover','Zenit-4MK sensor cover',33709,'Reentered','1975-01-02 00:00:00','1974-12-29 00:00:00',89.98,202,345,71.39,'Intermediate' from dual union all
select 76,'000076','61 Gamma 2','8K78I','8K78I',7266,'Reentered','1961-02-18 00:00:00','1961-02-15 00:00:00',88.96,210,237,65.01,'Intermediate' from dual union all
select 760,'000760','1964-011B','deb USAF satellite/Agena D','',10297,'Reentered','1965-08-17 00:00:00','1965-08-13 00:00:00',89.99,267,281,82.02,'Intermediate' from dual union all
select 7600,'007600','1974-106D','Sensor cover','Zenit-4MK sensor cover',33709,'Reentered','1975-01-07 00:00:00','1974-12-29 00:00:00',89.09,192,267,71.34,'Intermediate' from dual union all
select 7601,'007601','1974-106E','Blenda','Sunshade',33709,'Reentered','1975-01-11 00:00:00','1975-01-08 00:00:00',89.37,172,315,71.38,'Intermediate' from dual union all
select 7602,'007602','1974-106F','KDU','KDU',33709,'Reentered','1975-01-15 00:00:00','1975-01-08 00:00:00',89.5,172,328,71.43,'Intermediate' from dual union all
select 7603,'007603','1974-106G','KDU part','KDU part',33709,'Reentered','1975-01-15 00:00:00','1975-01-09 00:00:00',89.17,161,306,71.37,'Intermediate' from dual union all
select 7604,'007604','1975-001A','Soyuz-17','Soyuz 7K-T No. 38',36448,'Landed','1975-02-09 00:00:00','1975-01-17 00:00:00',91.37,336,348,51.57,'Intermediate' from dual union all
select 7605,'007605','1975-001B','11S510','11S510',36448,'Reentered','1975-01-14 00:00:00','1975-01-12 00:00:00',88.56,178,229,51.63,'Intermediate' from dual union all
select 7606,'007606','1975-002A','Kosmos-702','Zenit-2M',36449,'Landed','1975-01-29 00:00:00','1975-01-19 00:00:00',89.64,203,311,71.33,'Intermediate' from dual union all
select 7607,'007607','1975-002B','11A57I','11A57I',36449,'Reentered','1975-01-29 00:00:00','1975-01-19 00:00:00',89.53,199,303,71.36,'Intermediate' from dual union all
select 7608,'007608','1974-104E','Garbage bag','',33707,'Reentered','1975-04-29 00:00:00','1975-01-25 00:00:00',91.38,340,345,51.58,'Intermediate' from dual union all
select 7609,'007609','1975-002C','Sensor cover','Zenit-2M sensor cover',36449,'Reentered','1975-01-21 00:00:00','1975-01-18 00:00:00',88.85,189,247,71.34,'Intermediate' from dual union all
select 761,'000761','1964-011C','deb USAF satellite/Agena D','',10297,'Reentered','1965-09-21 00:00:00','1965-09-20 00:00:00',88.99,214,236,82.05,'Intermediate' from dual union all
select 7610,'007610','1975-001C','Garbage bag','',36448,'Reentered','1975-03-25 00:00:00','1975-01-25 00:00:00',91.28,331,344,51.58,'Intermediate' from dual union all
select 7611,'007611','1975-003A','Kosmos-703','DS-P1-Yu No. 70',36450,'Reentered','1975-11-20 00:00:00','1975-02-20 00:00:00',101.15,197,1430,81.95,'Intermediate' from dual union all
select 7612,'007612','1975-003B','S1M','S1M',36450,'Reentered','1975-08-22 00:00:00','1975-02-21 00:00:00',100.42,201,1356,81.97,'Intermediate' from dual union all
select 7613,'007613','1975-001D','Garbage bag','',36448,'Reentered','1975-04-12 00:00:00','1975-01-25 00:00:00',91.31,334,344,51.56,'Intermediate' from dual union all
select 7614,'007614','1975-001E','Garbage bag','',36448,'Reentered','1975-05-08 00:00:00','1975-01-25 00:00:00',91.39,338,348,51.58,'Intermediate' from dual union all
select 7615,'007615','1975-004A','Landsat 2','Landsat B',36451,'In Earth orbit','','1975-02-24 00:00:00',103.18,901,917,99.1,'Sun-Synch' from dual union all
select 7616,'007616','1975-004B','Delta 107','Delta SSPU DSV-3P-4',36451,'Exploded','1976-02-05 00:00:00','1975-02-26 00:00:00',101.44,741,913,97.82,'Sun-Synch' from dual union all
select 7617,'007617','1975-005A','Kosmos-704','Zenit-4MK',36452,'Landed','1975-02-06 00:00:00','1975-01-25 00:00:00',89.56,203,302,72.86,'Intermediate' from dual union all
select 7618,'007618','1975-005B','11A57I','11A57I',36452,'Reentered','1975-02-03 00:00:00','1975-01-25 00:00:00',89.38,199,289,72.86,'Intermediate' from dual union all
select 7619,'007619','1975-005C','Sensor cover','Zenit-4MK sensor cover',36452,'Reentered','1975-01-26 00:00:00','1975-01-23 00:00:00',89.03,210,244,72.79,'Intermediate' from dual union all
select 762,'000762','1964-010C','deb Kosmos-25','',10296,'Reentered','1964-03-04 00:00:00','1964-03-02 00:00:00',91.11,265,393,49.05,'Intermediate' from dual union all
select 7620,'007620','1975-005D','Sensor cover','Zenit-4MK sensor cover',36452,'Reentered','1975-01-26 00:00:00','1975-01-24 00:00:00',89.67,209,308,72.88,'Intermediate' from dual union all
select 7621,'007621','1975-005E','Sensor cover','Zenit-4MK sensor cover',36452,'Reentered','1975-01-26 00:00:00','1975-01-23 00:00:00',89.57,203,303,72.95,'Intermediate' from dual union all
select 7622,'007622','1975-001F','Garbage bag','',36448,'Reentered','1975-10-24 00:00:00','1975-01-25 00:00:00',91.29,332,344,51.58,'Intermediate' from dual union all
select 7623,'007623','1975-006A','Kosmos-705','DS-P1-Yu No. 75',36453,'Reentered','1975-11-18 00:00:00','1975-02-27 00:00:00',92.06,268,484,70.96,'Intermediate' from dual union all
select 7624,'007624','1975-006B','S1M','S1M',36453,'Reentered','1975-06-25 00:00:00','1975-02-27 00:00:00',91.73,267,452,70.94,'Intermediate' from dual union all
select 7625,'007625','1975-007A','Kosmos-706','Oko',36454,'In Earth orbit','','1975-03-04 00:00:00',717.66,660,39689,62.75,'Molniya' from dual union all
select 7626,'007626','1975-007B','11S510','11S510',36454,'Reentered','1975-04-04 00:00:00','1975-03-01 00:00:00',91.38,209,475,62.82,'Intermediate' from dual union all
select 7627,'007627','1975-007C','BOZ','BOZ',36454,'Reentered','1975-03-01 00:00:00','1975-02-11 00:00:00',91.7,174,542,62.88,'Intermediate' from dual union all
select 7628,'007628','1974-104F','Garbage bag','',33707,'Reentered','1975-04-08 00:00:00','1975-01-30 00:00:00',91.3,331,346,51.58,'Intermediate' from dual union all
select 7629,'007629','1975-007D','Blok-L','Blok-L',36454,'In Earth orbit','','1975-03-02 00:00:00',717.01,651,39666,62.79,'Molniya' from dual union all
select 763,'000763','1964-010D','deb Kosmos-25','',10296,'Reentered','1964-04-23 00:00:00','1964-04-18 00:00:00',89.98,256,291,49.35,'Intermediate' from dual union all
select 7630,'007630','1974-104G','Garbage bag','',33707,'Reentered','1975-04-15 00:00:00','1975-02-01 00:00:00',91.29,330,346,51.58,'Intermediate' from dual union all
select 7631,'007631','1975-005F','KDU','KDU',36452,'Reentered','1975-02-07 00:00:00','1975-02-02 00:00:00',89.51,168,332,72.84,'Intermediate' from dual union all
select 7632,'007632','1975-005G','Blenda','BZA',36452,'Reentered','1975-02-06 00:00:00','1975-02-02 00:00:00',89.3,167,313,72.84,'Intermediate' from dual union all
select 7633,'007633','1975-001G','Garbage bag','',36448,'Reentered','1975-04-15 00:00:00','1975-02-03 00:00:00',91.24,333,338,51.58,'Intermediate' from dual union all
select 7634,'007634','1974-104H','Garbage bag','',33707,'Reentered','1975-04-15 00:00:00','1975-02-04 00:00:00',91.25,327,345,51.57,'Intermediate' from dual union all
select 7635,'007635','1975-001H','Garbage bag','',36448,'Reentered','1975-03-23 00:00:00','1975-02-04 00:00:00',91.3,333,344,51.59,'Intermediate' from dual union all
select 7636,'007636','1974-100D','deb Kosmos-698','Tselina-O part',33703,'Reentered','1981-09-23 00:00:00','1975-02-12 00:00:00',94.82,461,559,74.03,'Intermediate' from dual union all
select 7637,'007637','1975-008A','Kosmos-707','Tselina-OM',36455,'Reentered','1980-09-07 00:00:00','1975-03-07 00:00:00',95.09,500,545,74.03,'Intermediate' from dual union all
select 7638,'007638','1975-008B','S3M','S3M',36455,'Reentered','1980-08-27 00:00:00','1975-03-09 00:00:00',94.98,489,546,74.03,'Intermediate' from dual union all
select 7639,'007639','1975-008C','deb Kosmos-707','Tselina-O part',36455,'Reentered','1976-12-14 00:00:00','1975-03-09 00:00:00',94.69,466,542,74.03,'Intermediate' from dual union all
select 764,'000764','1964-012A','OPS 3435','GAMBIT SV 956',10298,'Deorbited','1964-03-16 00:00:00','1964-03-13 00:00:00',88.06,161,196,95.77,'Sun-Synch' from dual union all
select 7640,'007640','1975-001J','Salyut-4 object','(long-lived)',36448,'Reentered','1975-10-27 00:00:00','1975-02-05 00:00:00',91.32,334,345,51.58,'Intermediate' from dual union all
select 7641,'007641','1975-009A','Molniya-2','Molniya-2',36456,'Reentered','1985-07-04 00:00:00','1975-03-16 00:00:00',717.76,633,39721,62.83,'Molniya' from dual union all
select 7642,'007642','1975-009B','11S510','11S510',36456,'Reentered','1975-04-25 00:00:00','1975-03-08 00:00:00',91.89,214,521,62.85,'Intermediate' from dual union all
select 7643,'007643','1975-009C','BOZ','BOZ',36456,'Reentered','1975-04-23 00:00:00','1975-03-08 00:00:00',92,209,537,62.81,'Intermediate' from dual union all
select 7644,'007644','1975-005H','PTDU cover','KDU part',36452,'Reentered','1975-02-08 00:00:00','1975-02-06 00:00:00',89.27,168,309,72.88,'Intermediate' from dual union all
select 7645,'007645','1975-008D','deb Kosmos-707','Tselina-O part',36455,'Reentered','1977-04-17 00:00:00','1975-04-16 00:00:00',94.81,482,537,73.95,'Intermediate' from dual union all
select 7646,'007646','1975-010A','Starlette','Starlette',36457,'In Earth orbit','','1989-09-25 00:00:00',104.18,804,1108,49.82,'Intermediate' from dual;
insert into SATELLITE
select 7647,'007647','1975-010B','P.068','P068',36457,'In Earth orbit','','1975-03-12 00:00:00',104.48,803,1137,49.82,'Intermediate' from dual union all
select 7648,'007648','1975-011A','SMS 2','SMS B',36458,'In Earth orbit','','1977-01-24 00:00:00',1435.96,35773,35795,.52,'Stationary' from dual union all
select 7649,'007649','1975-005J','PTDU cover','KDU part',36452,'Reentered','1975-02-09 00:00:00','1975-02-07 00:00:00',90.25,177,397,72.86,'Intermediate' from dual union all
select 765,'000765','1964-012B','Agena D 4804','Agena D',10298,'Reentered','1964-03-12 00:00:00','1964-03-11 00:00:00',88.67,184,233,95.69,'Sun-Synch' from dual union all
select 7650,'007650','1975-011B','Delta 108','Delta SSPU DSV-3P-4',36458,'Reentered','1982-04-06 00:00:00','1975-03-09 00:00:00',121.76,278,3241,27.63,'Medium' from dual union all
select 7651,'007651','1975-011C','Delta 108 clamp band half','',36458,'Reentered','1975-02-16 00:00:00','1975-02-08 00:00:00',92.12,175,582,28.07,'Intermediate' from dual union all
select 7652,'007652','1975-011D','Delta 108 clamp band half','',36458,'Reentered','1975-02-16 00:00:00','1975-02-08 00:00:00',92.07,174,579,28.06,'Intermediate' from dual union all
select 7653,'007653','1975-009D','Blok-L','Blok-L',36456,'Reentered','1985-09-30 00:00:00','1975-03-16 00:00:00',733.39,672,40449,62.79,'Molniya' from dual union all
select 7654,'007654','1975-010C','Starlette despin weight','',36457,'In Earth orbit','','1975-03-12 00:00:00',104.06,805,1095,49.85,'Intermediate' from dual union all
select 7655,'007655','1975-010D','Starlette despin weight','',36457,'In Earth orbit','','1975-03-14 00:00:00',104.12,804,1103,49.8,'Intermediate' from dual union all
select 7656,'007656','1974-104J','Garbage bag','',33707,'Reentered','1975-04-18 00:00:00','1975-02-10 00:00:00',91.21,332,336,51.59,'Intermediate' from dual union all
select 7657,'007657','1974-104K','Garbage bag','',33707,'Reentered','1975-04-26 00:00:00','1975-02-08 00:00:00',91.28,331,344,51.58,'Intermediate' from dual union all
select 7658,'007658','1974-104L','Garbage bag','',33707,'Reentered','1975-05-07 00:00:00','1975-02-08 00:00:00',91.28,330,345,51.57,'Intermediate' from dual union all
select 7659,'007659','1975-010E','Starlette/P068 adapter','',36457,'In Earth orbit','','1975-03-19 00:00:00',104.32,805,1121,49.82,'Intermediate' from dual union all
select 766,'000766','1964-013A','Kosmos-26','DS-MG No. 1',10299,'Reentered','1964-09-28 00:00:00','1964-04-20 00:00:00',90.82,263,367,48.97,'Intermediate' from dual union all
select 7660,'007660','1975-011E','deb Delta 108','',36458,'Reentered','1975-02-21 00:00:00','1975-02-12 00:00:00',91.3,137,540,28.1,'Intermediate' from dual union all
select 7661,'007661','1974-104M','Garbage bag','',33707,'Reentered','1975-04-13 00:00:00','1975-02-11 00:00:00',91.3,332,345,51.58,'Intermediate' from dual union all
select 7662,'007662','1974-104N','Soyuz-17 BO','Bitovoy otsek',33707,'Reentered','1975-09-10 00:00:00','1975-02-11 00:00:00',91.31,334,344,51.59,'Intermediate' from dual union all
select 7663,'007663','1975-012A','Kosmos-708','Geoid',36459,'In Earth orbit','','1975-03-16 00:00:00',113.55,1369,1410,69.24,'Intermediate' from dual union all
select 7664,'007664','1975-013A','Kosmos-709','Zenit-4MK',36460,'Landed','1975-02-25 00:00:00','1975-02-14 00:00:00',89.38,179,309,62.83,'Intermediate' from dual union all
select 7665,'007665','1975-012B','S3M','S3M',36459,'In Earth orbit','','1975-03-14 00:00:00',113.39,1366,1398,69.23,'Intermediate' from dual union all
select 7666,'007666','1975-013B','11A57I','11A57I',36460,'Reentered','1975-02-17 00:00:00','1975-02-14 00:00:00',89.08,172,286,62.82,'Intermediate' from dual union all
select 7667,'007667','1975-013C','Sensor cover','Zenit-4MK sensor cover',36460,'Reentered','1975-02-15 00:00:00','1975-02-12 00:00:00',89.39,179,310,62.83,'Intermediate' from dual union all
select 7668,'007668','1975-013D','Sensor cover','Zenit-4MK sensor cover',36460,'Reentered','1975-02-15 00:00:00','1975-02-12 00:00:00',89.39,179,310,62.83,'Intermediate' from dual union all
select 7669,'007669','1975-013E','Sensor cover','Zenit-4MK sensor cover',36460,'Reentered','1975-02-15 00:00:00','1975-02-12 00:00:00',89.39,179,310,62.83,'Intermediate' from dual union all
select 767,'000767','1964-013B','S1','S1',10299,'Reentered','1964-05-17 00:00:00','1964-05-13 00:00:00',88.94,208,237,48.99,'Intermediate' from dual union all
select 7670,'007670','1967-034G','NNS O-12 insulation','',15289,'Reentered','1981-05-02 00:00:00','1975-01-01 00:00:00',106.44,1048,1076,90.21,'Polar' from dual union all
select 7671,'007671','1975-014A','Taiyo','SRATS',36461,'Reentered','1980-06-29 00:00:00','1975-03-26 00:00:00',120.02,249,3115,31.56,'Medium' from dual union all
select 7672,'007672','1975-013F','KDU','KDU',36460,'Reentered','1975-03-01 00:00:00','1975-02-24 00:00:00',88.91,176,266,62.83,'Intermediate' from dual union all
select 7673,'007673','1975-013G','Blenda','Sunshade',36460,'Reentered','1975-02-26 00:00:00','1975-02-24 00:00:00',88.72,171,252,62.87,'Intermediate' from dual union all
select 7674,'007674','1975-014B','M-3A','M-3A',36461,'Reentered','1980-05-17 00:00:00','1975-03-26 00:00:00',120,250,3112,31.56,'Medium' from dual union all
select 7675,'007675','1975-015A','Kosmos-710','Zenit-4MK',36462,'Landed','1975-03-12 00:00:00','1975-02-28 00:00:00',89.34,173,311,64.98,'Intermediate' from dual union all
select 7676,'007676','1975-015B','11A57I','11A57I',36462,'Reentered','1975-03-03 00:00:00','1975-02-28 00:00:00',89.2,169,301,64.99,'Intermediate' from dual union all
select 7677,'007677','1975-015C','Sensor cover','Zenit-4MK sensor cover',36462,'Reentered','1975-03-03 00:00:00','1975-02-28 00:00:00',89.07,167,290,64.96,'Intermediate' from dual union all
select 7678,'007678','1975-016A','Kosmos-711','Strela-1M',36463,'In Earth orbit','','1975-04-27 00:00:00',115.48,1462,1493,74,'Intermediate' from dual union all
select 7679,'007679','1975-016B','Kosmos-712','Strela-1M',36463,'In Earth orbit','','1975-04-09 00:00:00',114.9,1412,1490,74.01,'Intermediate' from dual union all
select 768,'000768','1964-013C','deb Kosmos-26','',10299,'Reentered','1964-04-22 00:00:00','1964-03-24 00:00:00',90.64,247,365,48.9,'Intermediate' from dual union all
select 7680,'007680','1975-016C','Kosmos-713','Strela-1M',36463,'In Earth orbit','','1975-03-31 00:00:00',114.69,1396,1487,74,'Intermediate' from dual union all
select 7681,'007681','1975-016D','Kosmos-714','Strela-1M',36463,'In Earth orbit','','1975-04-26 00:00:00',115.27,1445,1491,74,'Intermediate' from dual union all
select 7682,'007682','1975-016E','Kosmos-715','Strela-1M',36463,'In Earth orbit','','1975-04-19 00:00:00',115.69,1470,1504,74,'Intermediate' from dual union all
select 7683,'007683','1975-016F','Kosmos-716','Strela-1M',36463,'In Earth orbit','','1975-04-26 00:00:00',115.91,1479,1515,74,'Intermediate' from dual union all
select 7684,'007684','1975-016G','Kosmos-717','Strela-1M',36463,'In Earth orbit','','1975-04-05 00:00:00',116.16,1479,1537,74,'Intermediate' from dual union all
select 7685,'007685','1975-016H','Kosmos-718','Strela-1M',36463,'In Earth orbit','','1975-04-07 00:00:00',115.09,1429,1489,74.01,'Intermediate' from dual union all
select 7686,'007686','1975-016J','S3M','S3M',36463,'In Earth orbit','','1975-03-30 00:00:00',118.03,1483,1702,74.01,'Intermediate' from dual union all
select 7687,'007687','1975-017A','OPS 2439','JUMPSEAT 4',36464,'In Earth orbit','','1975-03-12 00:00:00',708.96,342,39577,63.47,'Molniya' from dual union all
select 7688,'007688','1975-017B','Agena D','Agena D',36464,'In Earth orbit','','1975-03-13 00:00:00',699.75,287,39175,63.44,'Molniya' from dual union all
select 7689,'007689','1975-015D','Blenda','Sunshade',36462,'Reentered','1975-03-12 00:00:00','1975-03-12 00:00:00',87.67,143,176,64.97,'Intermediate' from dual union all
select 769,'000769','1964-013D','deb Kosmos-26','',10299,'Reentered','1964-03-25 00:00:00','1964-03-22 00:00:00',90.44,259,333,48.95,'Intermediate' from dual union all
select 7690,'007690','1975-015E','KDU','KDU',36462,'Reentered','1975-03-16 00:00:00','1975-03-11 00:00:00',88.95,168,278,64.99,'Intermediate' from dual union all
select 7691,'007691','1975-018A','Kosmos-719','Zenit-4MK',36465,'Landed','1975-03-25 00:00:00','1975-03-15 00:00:00',89.26,173,303,64.98,'Intermediate' from dual union all
select 7692,'007692','1975-018B','11A57I','11A57I',36465,'Reentered','1975-03-16 00:00:00','1975-03-14 00:00:00',88.77,166,262,64.99,'Intermediate' from dual union all
select 7693,'007693','1975-018C','Sensor cover','Zenit-4MK sensor cover',36465,'Reentered','1975-03-15 00:00:00','1975-03-13 00:00:00',88,151,200,64.97,'Intermediate' from dual union all
select 7694,'007694','1974-070E','ANS despin weight','',33673,'Reentered','1975-04-10 00:00:00','1975-03-14 00:00:00',91.67,223,490,98.45,'Sun-Synch' from dual union all
select 7695,'007695','1974-100E','deb Kosmos-698','Tselina-O part',33703,'Reentered','1977-09-29 00:00:00','1975-03-23 00:00:00',95.55,524,567,74.06,'Intermediate' from dual union all
select 7696,'007696','1975-019A','Kosmos-720','Zenit-4MT',36466,'Landed','1975-04-01 00:00:00','1975-03-24 00:00:00',89.32,211,271,62.81,'Intermediate' from dual union all
select 7697,'007697','1975-019B','11S510','11S510',36466,'Reentered','1975-03-30 00:00:00','1975-03-24 00:00:00',89.13,209,254,62.8,'Intermediate' from dual union all
select 7698,'007698','1975-019C','Sensor cover','Zenit-4MT sensor cover',36466,'Reentered','1975-03-22 00:00:00','1975-03-22 00:00:00',87.57,152,157,62.75,'Intermediate' from dual union all
select 7699,'007699','1975-019D','Sensor cover','Zenit-4MT sensor cover',36466,'Reentered','1975-03-25 00:00:00','1975-03-23 00:00:00',89.23,206,266,62.79,'Intermediate' from dual union all
select 77,'000077','61 Gamma 3','BOZ','BOZ',7266,'Reentered','1961-02-25 00:00:00','1961-02-16 00:00:00',89.48,197,301,65.01,'Intermediate' from dual union all
select 770,'000770','1964-014B','11S510','11S510',10300,'Reentered','1964-03-28 00:00:00','1964-03-27 00:00:00',88.27,158,220,64.75,'Intermediate' from dual union all
select 7700,'007700','1975-019E','Sensor cover','Zenit-4MT sensor cover',36466,'Reentered','1975-03-22 00:00:00','1975-03-21 00:00:00',88.88,195,244,62.79,'Intermediate' from dual union all
select 7701,'007701','1975-018D','Blenda','Sunshade',36465,'Reentered','1975-03-26 00:00:00','1975-03-24 00:00:00',89.01,169,283,64.96,'Intermediate' from dual union all
select 7702,'007702','1975-018E','KDU','KDU',36465,'Reentered','1975-03-30 00:00:00','1975-03-24 00:00:00',89.26,172,304,64.9,'Intermediate' from dual union all
select 7703,'007703','1975-018F','KDU part','KDU part',36465,'Reentered','1975-03-28 00:00:00','1975-03-25 00:00:00',89.08,169,289,64.99,'Intermediate' from dual union all
select 7704,'007704','1974-104P','Salyut-4 part','Unknown',33707,'Reentered','1975-06-16 00:00:00','1975-03-25 00:00:00',91.27,335,339,51.6,'Intermediate' from dual union all
select 7705,'007705','1975-020A','Kosmos-721','Zenit-2M',36467,'Landed','1975-04-07 00:00:00','1975-03-28 00:00:00',88.78,204,224,81.33,'Intermediate' from dual union all
select 7706,'007706','1975-020B','11A57I','11A57I',36467,'Reentered','1975-03-29 00:00:00','1975-03-28 00:00:00',88.19,171,199,81.32,'Intermediate' from dual union all
select 7707,'007707','1975-020C','Sensor cover','Zenit-2M sensor cover',36467,'Reentered','1975-03-29 00:00:00','1975-03-26 00:00:00',88.76,193,233,81.28,'Intermediate' from dual union all
select 7708,'007708','1975-020D','Sensor cover','Zenit-2M sensor cover',36467,'Reentered','1975-03-31 00:00:00','1975-03-28 00:00:00',88.93,193,251,81.32,'Intermediate' from dual union all
select 7709,'007709','1975-021A','Kosmos-722','Zenit-4MK',36468,'Landed','1975-04-09 00:00:00','1975-03-31 00:00:00',89.53,173,330,71.35,'Intermediate' from dual union all
select 771,'000771','1964-015A','Ariel 2','UK-C',10301,'Reentered','1967-11-18 00:00:00','1964-05-03 00:00:00',101.21,289,1343,51.65,'Intermediate' from dual union all
select 7710,'007710','1975-022A','Interkosmos-13','DS-U2-IK No. 5',36469,'Reentered','1980-09-02 00:00:00','1975-04-27 00:00:00',104.73,285,1679,82.96,'Intermediate' from dual union all
select 7711,'007711','1975-021B','11A57I','11A57I',36468,'Reentered','1975-04-09 00:00:00','1975-03-30 00:00:00',89.59,200,309,71.35,'Intermediate' from dual union all
select 7712,'007712','1975-021C','Sensor cover','Zenit-4MK sensor cover',36468,'Reentered','1975-03-29 00:00:00','1975-03-28 00:00:00',89.24,169,305,71.34,'Intermediate' from dual union all
select 7713,'007713','1975-022B','S3M','S3M',36469,'Reentered','1980-04-21 00:00:00','1975-04-26 00:00:00',104.59,279,1672,82.95,'Intermediate' from dual union all
select 7714,'007714','1975-023A','Meteor','Meteor-M  No. 31',36470,'In Earth orbit','','1975-05-05 00:00:00',102.54,864,894,81.22,'Intermediate' from dual union all
select 7715,'007715','1975-023B','8A92ME','8A92ME',36470,'In Earth orbit','','1975-05-01 00:00:00',102.57,841,920,81.23,'Intermediate' from dual union all
select 7716,'007716','1975-021D','Sensor cover','Zenit-4MK sensor cover',36468,'Reentered','1975-04-02 00:00:00','1975-04-01 00:00:00',89.47,169,328,71.36,'Intermediate' from dual union all
select 7717,'007717','1975-019F','KB electropitaniya','KBE',36466,'Reentered','1975-04-05 00:00:00','1975-04-02 00:00:00',89.13,205,258,62.79,'Intermediate' from dual union all
select 7718,'007718','1975-024A','Kosmos-723','US-A reactor',36471,'In Earth orbit','','1975-05-02 00:00:00',89.68,251,266,65.02,'Intermediate' from dual union all
select 7719,'007719','1975-019G','Blenda','[UNID068] KBE part',36466,'Reentered','1975-04-04 00:00:00','1975-04-02 00:00:00',102.53,866,891,81.22,'Intermediate' from dual union all
select 772,'000772','1964-014A','Kosmos-27','3MV-1 No. 3',10300,'Reentered Attached','1964-03-29 00:00:00','1964-03-27 00:00:00',88.54,197,208,64.8,'Intermediate' from dual union all
select 7720,'007720','1975-020E','Nauka part','Nauka part',36467,'Reentered','1975-04-07 00:00:00','1975-04-06 00:00:00',88.4,180,211,81.32,'Intermediate' from dual union all
select 7721,'007721','1975-020F','Kapsula Kosmosa-721','Nauka 5KSA No. 4',36467,'Reentered','1975-04-10 00:00:00','1975-04-06 00:00:00',88.58,194,215,81.32,'Intermediate' from dual union all
select 7722,'007722','1974-100F','deb Kosmos-698','Tselina-O part',33703,'Reentered','1979-10-20 00:00:00','1975-04-08 00:00:00',94.85,467,556,73.93,'Intermediate' from dual union all
select 7723,'007723','1974-100G','deb Kosmos-698','Tselina-O part',33703,'Reentered','1980-04-14 00:00:00','1975-04-18 00:00:00',95.33,515,555,73.92,'Intermediate' from dual union all
select 7724,'007724','1974-100H','deb Kosmos-698','Tselina-O part',33703,'Reentered','1978-03-02 00:00:00','1975-04-08 00:00:00',96.04,546,592,73.96,'Intermediate' from dual union all
select 7725,'007725','1975-019H','KBE part','KBE part',36466,'Reentered','1975-04-10 00:00:00','1975-04-05 00:00:00',90.65,231,382,62.8,'Intermediate' from dual union all
select 7726,'007726','1975-019J','KBE part','KBE part',36466,'Reentered','1975-04-09 00:00:00','1975-04-06 00:00:00',90.22,219,351,62.79,'Intermediate' from dual union all
select 7727,'007727','1975-025A','Kosmos-724','US-A reactor',36472,'In Earth orbit','','1975-05-07 00:00:00',89.62,249,263,64.97,'Intermediate' from dual union all
select 7728,'007728','1975-021E','Blenda','Sunshade',36468,'Reentered','1975-04-10 00:00:00','1975-04-08 00:00:00',89.14,160,304,71.35,'Intermediate' from dual union all
select 7729,'007729','1975-021F','PTDU cover','KDU part',36468,'Reentered','1975-04-12 00:00:00','1975-04-08 00:00:00',90.13,145,417,71.24,'Intermediate' from dual union all
select 773,'000773','1964-014C','BOZ','BOZ',10300,'Reentered','1964-03-31 00:00:00','1964-03-27 00:00:00',88.63,189,225,64.88,'Intermediate' from dual union all
select 7730,'007730','1975-026A','Kosmos-725','DS-P1-Yu No. 77',36473,'Reentered','1976-01-06 00:00:00','1975-05-08 00:00:00',91.88,267,467,70.99,'Intermediate' from dual union all
select 7731,'007731','1975-026B','S1M','S1M',36473,'Reentered','1975-09-09 00:00:00','1975-05-08 00:00:00',91.53,270,430,70.97,'Intermediate' from dual union all
select 7732,'007732','1975-021G','KDU','KDU',36468,'Reentered','1975-04-14 00:00:00','1975-04-09 00:00:00',89.21,172,299,71.35,'Intermediate' from dual union all
select 7733,'007733','1975-021H','KDU part','KDU part',36468,'Reentered','1975-04-10 00:00:00','1975-04-09 00:00:00',89.06,166,290,71.33,'Intermediate' from dual union all
select 7734,'007734','1975-027A','Geos 3','Geos C',36474,'In Earth orbit','','1988-06-09 00:00:00',101.67,821,855,114.99,'Retrograde' from dual union all
select 7735,'007735','1975-027B','Delta 109','Delta SSPU DSV-3P-4',36474,'In Earth orbit','','1975-05-10 00:00:00',101.71,837,843,115,'Retrograde' from dual union all
select 7736,'007736','1975-028A','Kosmos-726','Tsiklon-B',36475,'In Earth orbit','','1975-05-12 00:00:00',104.61,958,995,82.99,'Intermediate' from dual;
insert into SATELLITE
select 7737,'007737','1975-028B','S3M','S3M',36475,'In Earth orbit','','1975-05-11 00:00:00',104.49,958,984,83,'Intermediate' from dual union all
select 7738,'007738','1975-029A','Molniya-3','Molniya-3 No. 12',36476,'Reentered','1988-11-29 00:00:00','1975-05-18 00:00:00',717.48,615,39724,62.92,'Molniya' from dual union all
select 7739,'007739','1975-029B','11S510','11S510',36476,'Reentered','1975-06-20 00:00:00','1975-05-14 00:00:00',91.31,213,465,62.82,'Intermediate' from dual union all
select 774,'000774','1964-014D','deb Kosmos-27','',10300,'Reentered','1964-04-04 00:00:00','1964-03-31 00:00:00',88.68,187,231,64.81,'Intermediate' from dual union all
select 7740,'007740','1975-029C','BOZ','BOZ',36476,'Reentered','1975-06-04 00:00:00','1975-04-21 00:00:00',92.35,196,584,62.86,'Intermediate' from dual union all
select 7741,'007741','1975-029D','Blok-L','Blok-L',36476,'Reentered','1991-01-12 00:00:00','1975-05-17 00:00:00',733.14,611,40498,62.91,'Molniya' from dual union all
select 7742,'007742','1975-030A','Kosmos-727','Zenit-4MK',36477,'Landed','1975-04-28 00:00:00','1975-04-18 00:00:00',89.53,171,332,64.98,'Intermediate' from dual union all
select 7743,'007743','1975-030B','11S510I','11S510I',36477,'Reentered','1975-04-21 00:00:00','1975-04-18 00:00:00',89.06,167,288,64.99,'Intermediate' from dual union all
select 7744,'007744','1975-030C','Sensor cover','Zenit-4MK sensor cover',36477,'Reentered','1975-04-19 00:00:00','1975-04-16 00:00:00',88.72,162,261,64.99,'Intermediate' from dual union all
select 7745,'007745','1975-031A','Kosmos-728','Zenit-2M',36478,'Landed','1975-04-29 00:00:00','1975-04-20 00:00:00',89.74,200,323,72.83,'Intermediate' from dual union all
select 7746,'007746','1975-031B','11A57I','11A57I',36478,'Reentered','1975-04-30 00:00:00','1975-04-20 00:00:00',89.58,198,310,72.83,'Intermediate' from dual union all
select 7747,'007747','1975-032A','OPS 4883','GAMBIT-3 Mission 4344',36479,'Reentered Attached','1975-06-05 00:00:00','1975-04-25 00:00:00',89.75,125,400,110.52,'Retrograde' from dual union all
select 7748,'007748','1974-103C','deb Kosmos-699','',33706,'Reentered','1977-01-08 00:00:00','1975-04-18 00:00:00',93.22,422,443,65.04,'Intermediate' from dual union all
select 7749,'007749','1974-103D','deb Kosmos-699','',33706,'Reentered','1976-04-12 00:00:00','1975-04-25 00:00:00',93.51,435,458,64.93,'Intermediate' from dual union all
select 775,'000775','1964-015B','Altair X-248-A5 BSB-407A','X-248',10301,'Reentered','1967-04-13 00:00:00','1967-04-08 00:00:00',89.86,219,316,51.57,'Intermediate' from dual union all
select 7750,'007750','1974-103E','deb Kosmos-699','',33706,'Reentered','1977-02-27 00:00:00','1975-04-23 00:00:00',93.43,429,455,64.99,'Intermediate' from dual union all
select 7751,'007751','1975-031C','Sensor cover','Zenit-2M sensor cover',36478,'Reentered','1975-04-20 00:00:00','1975-04-19 00:00:00',89.29,198,281,72.83,'Intermediate' from dual union all
select 7752,'007752','1975-033A','Aryabhata','ISS',36480,'Reentered','1992-02-11 00:00:00','1975-05-19 00:00:00',96.46,568,610,50.68,'Intermediate' from dual union all
select 7753,'007753','1975-033B','S3M','S3M',36480,'Reentered','1988-11-06 00:00:00','1975-05-19 00:00:00',96.32,553,612,50.69,'Intermediate' from dual union all
select 7754,'007754','1974-103F','deb Kosmos-699','',33706,'Reentered','1975-06-01 00:00:00','1975-06-10 00:00:00',92.4,369,415,64.89,'Intermediate' from dual union all
select 7755,'007755','1974-103G','deb Kosmos-699','',33706,'Reentered','1976-01-20 00:00:00','1976-01-11 00:00:00',91.22,314,355,64.5,'Intermediate' from dual union all
select 7756,'007756','1974-103H','deb Kosmos-699','',33706,'Reentered','1976-12-23 00:00:00','1975-04-20 00:00:00',94.38,437,541,64.94,'Intermediate' from dual union all
select 7757,'007757','1974-103J','deb Kosmos-699','',33706,'Reentered','1978-10-18 00:00:00','1978-09-30 00:00:00',90.52,290,310,64.83,'Intermediate' from dual union all
select 7758,'007758','1974-103K','deb Kosmos-699','',33706,'Reentered','1975-08-16 00:00:00','1975-08-03 00:00:00',91.16,325,337,65,'Intermediate' from dual union all
select 7759,'007759','1974-103L','deb Kosmos-699','',33706,'Reentered','1976-06-06 00:00:00','1975-04-19 00:00:00',93.54,424,472,64.94,'Intermediate' from dual union all
select 776,'000776','1964-016A','BOZ','BOZ',10302,'Reentered','1964-04-03 00:00:00','1964-04-02 00:00:00',88.59,177,233,64.99,'Intermediate' from dual union all
select 7760,'007760','1974-103M','deb Kosmos-699','',33706,'Reentered','1975-10-30 00:00:00','1975-10-05 00:00:00',91.62,353,355,65,'Intermediate' from dual union all
select 7761,'007761','1975-031D','Sensor cover','Zenit-2M sensor cover',36478,'Reentered','1975-04-22 00:00:00','1975-04-20 00:00:00',89.28,191,287,72.81,'Intermediate' from dual union all
select 7762,'007762','1974-103N','deb Kosmos-699','',33706,'Reentered','1975-07-29 00:00:00','1975-07-26 00:00:00',90.45,204,390,64.89,'Intermediate' from dual union all
select 7763,'007763','1974-103P','deb Kosmos-699','',33706,'Reentered','1976-04-12 00:00:00','1975-04-20 00:00:00',93.36,433,444,64.95,'Intermediate' from dual union all
select 7764,'007764','1974-103Q','deb Kosmos-699','',33706,'Reentered','1975-08-26 00:00:00','1975-08-20 00:00:00',90.67,307,308,64.97,'Intermediate' from dual union all
select 7765,'007765','1974-103R','deb Kosmos-699','',33706,'Reentered','1977-09-23 00:00:00','1975-04-28 00:00:00',93.35,436,440,64.9,'Intermediate' from dual union all
select 7766,'007766','1974-103S','deb Kosmos-699','',33706,'Reentered','1975-07-25 00:00:00','1975-07-18 00:00:00',90.57,297,308,64.89,'Intermediate' from dual union all
select 7767,'007767','1974-103T','deb Kosmos-699','',33706,'Reentered','1975-07-09 00:00:00','1975-05-03 00:00:00',93.4,435,447,64.99,'Intermediate' from dual union all
select 7768,'007768','1975-034A','Kosmos-729','Zaliv',36481,'In Earth orbit','','1975-05-27 00:00:00',104.96,975,1011,82.96,'Intermediate' from dual union all
select 7769,'007769','1975-034B','S3M','S3M',36481,'In Earth orbit','','1975-05-24 00:00:00',104.87,974,1003,82.96,'Intermediate' from dual union all
select 777,'000777','1964-016B','11S510','11S510',10302,'Reentered','1964-04-02 00:00:00','1964-04-02 00:00:00',88.72,191,232,64.85,'Intermediate' from dual union all
select 7770,'007770','1975-035A','Kosmos-730','Zenit-4MK',36482,'Landed','1975-05-06 00:00:00','1975-04-26 00:00:00',88.88,208,231,81.33,'Intermediate' from dual union all
select 7771,'007771','1975-035B','11A57I','11A57I',36482,'Reentered','1975-04-29 00:00:00','1975-04-26 00:00:00',88.42,192,200,81.33,'Intermediate' from dual union all
select 7772,'007772','1975-035C','Sensor cover','Zenit-4MK sensor cover',36482,'Reentered','1975-05-01 00:00:00','1975-04-27 00:00:00',88.87,208,229,81.33,'Intermediate' from dual union all
select 7773,'007773','1975-035D','Sensor cover','Zenit-4MK sensor cover',36482,'Reentered','1975-04-28 00:00:00','1975-04-25 00:00:00',89.37,209,278,81.34,'Intermediate' from dual union all
select 7774,'007774','1975-030D','KDU','KDU',36477,'Reentered','1975-05-05 00:00:00','1975-04-27 00:00:00',89.3,167,313,64.98,'Intermediate' from dual union all
select 7775,'007775','1975-030E','Blenda','Sunshade',36477,'Reentered','1975-05-01 00:00:00','1975-04-27 00:00:00',89.11,168,293,64.97,'Intermediate' from dual union all
select 7776,'007776','1975-031E','Nauka part','',36478,'Reentered','1975-05-03 00:00:00','1975-04-27 00:00:00',89.51,197,303,72.84,'Intermediate' from dual union all
select 7777,'007777','1975-008E','deb Kosmos-707','Tselina-O part',36455,'Reentered','1977-04-29 00:00:00','1975-04-25 00:00:00',94.81,479,540,74.02,'Intermediate' from dual union all
select 7778,'007778','1975-031F','Nauka part','',36478,'Reentered','1975-04-28 00:00:00','1975-04-28 00:00:00',89.64,198,315,72.82,'Intermediate' from dual union all
select 7779,'007779','1975-031G','Kapsula Kosmosa-728','Nauka 15KS',36478,'Reentered','1975-05-16 00:00:00','1975-04-28 00:00:00',89.62,199,313,72.83,'Intermediate' from dual union all
select 778,'000778','1964-016C','deb Blok-I','',10302,'Reentered','1964-04-05 00:00:00','1964-04-02 00:00:00',88.72,191,232,64.85,'Intermediate' from dual union all
select 7780,'007780','1975-036A','Molniya-1','Molniya-1',36483,'In Earth orbit','','1975-06-03 00:00:00',717.72,446,39906,62.85,'Molniya' from dual union all
select 7781,'007781','1975-036B','11S510','11S510',36483,'Reentered','1975-05-29 00:00:00','1975-05-06 00:00:00',90.48,213,384,62.85,'Intermediate' from dual union all
select 7782,'007782','1975-036C','BOZ','BOZ',36483,'Reentered','1975-06-03 00:00:00','1975-05-07 00:00:00',90.89,209,428,62.82,'Intermediate' from dual union all
select 7783,'007783','1974-103U','deb Kosmos-699','',33706,'Reentered','1975-07-06 00:00:00','1975-07-02 00:00:00',90.6,287,321,64.91,'Intermediate' from dual union all
select 7784,'007784','1974-103V','deb Kosmos-699','',33706,'Reentered','1975-10-10 00:00:00','1975-09-24 00:00:00',92.79,405,418,65,'Intermediate' from dual union all
select 7785,'007785','1975-035E','KDU','KDU',36482,'Reentered','1975-05-07 00:00:00','1975-05-05 00:00:00',88.59,164,246,81.33,'Intermediate' from dual union all
select 7786,'007786','1975-035F','Blenda','Sunshade',36482,'Reentered','1975-05-06 00:00:00','1975-05-05 00:00:00',88.43,164,230,81.32,'Intermediate' from dual union all
select 7787,'007787','1975-035G','KDU part','KDU part',36482,'Reentered','1975-05-24 00:00:00','1975-05-05 00:00:00',88.35,114,271,81.34,'Intermediate' from dual union all
select 7788,'007788','1975-037A','SAS 3','SAS C',36484,'Reentered','1979-04-09 00:00:00','1975-06-24 00:00:00',94.73,502,509,3,'Equatorial' from dual union all
select 7789,'007789','1975-037B','Star 20 E-16','Star 20 E-16',36484,'Reentered','1979-09-09 00:00:00','1975-06-06 00:00:00',94.71,501,509,2.99,'Equatorial' from dual union all
select 779,'000779','1964-017A','Kosmos-28','Zenit-2 No. 16',10303,'Landed','1964-04-12 00:00:00','1964-04-06 00:00:00',90.34,205,377,65.02,'Intermediate' from dual union all
select 7790,'007790','1975-038A','Anik A3','Anik A3',36485,'In Earth orbit','','1977-01-26 00:00:00',1436.04,35774,35797,.01,'Stationary' from dual union all
select 7791,'007791','1975-038B','Delta 110','Delta SSPU DSV-3P-4',36485,'Reentered','1976-11-03 00:00:00','1975-06-26 00:00:00',108.37,210,2093,28.36,'Intermediate' from dual union all
select 7792,'007792','1975-038C','Delta 110 clamp band','',36485,'Reentered','1975-05-08 00:00:00','1975-05-08 00:00:00',88.24,146,229,28.73,'Intermediate' from dual union all
select 7793,'007793','1974-103W','deb Kosmos-699','',33706,'Reentered','1975-07-09 00:00:00','1975-07-05 00:00:00',90.32,158,423,64.2,'Intermediate' from dual union all
select 7794,'007794','1975-038D','Star 37C S/N 40033','Star 37C',36485,'Reentered','2014-06-24 00:00:00','1975-06-19 00:00:00',638,271,36072,24.74,'GEO Transfer' from dual union all
select 7795,'007795','1975-024B','Kosmos-723 RLS','US-A RLS',36471,'Reentered','1975-05-21 00:00:00','1975-05-16 00:00:00',89.57,249,258,65,'Intermediate' from dual union all
select 7796,'007796','1975-024C','deb Kosmos-723','',36471,'Reentered','1975-05-19 00:00:00','1975-05-16 00:00:00',89.15,203,262,65.07,'Intermediate' from dual union all
select 7797,'007797','1975-024D','Kosmos-723 DU','US-A',36471,'Reentered','1975-07-15 00:00:00','1975-05-15 00:00:00',89.64,253,261,65.01,'Intermediate' from dual union all
select 7798,'007798','1975-024E','deb Kosmos-723','',36471,'Reentered','1975-05-24 00:00:00','1975-05-16 00:00:00',88.77,197,231,64.99,'Intermediate' from dual union all
select 7799,'007799','1975-024F','deb Kosmos-723','',36471,'Reentered','1975-05-25 00:00:00','1975-05-16 00:00:00',89.63,247,266,65.03,'Intermediate' from dual union all
select 78,'000078','61 Gamma 4','deb Tyazhyoliy Sputnik','',7266,'Reentered','1961-02-18 00:00:00','1961-02-12 00:00:00',89.51,197,303,65.01,'Intermediate' from dual union all
select 780,'000780','1964-017B','8A92E','8A92E',10303,'Reentered','1964-05-03 00:00:00','1964-04-29 00:00:00',88.78,188,241,65.03,'Intermediate' from dual union all
select 7800,'007800','1975-036D','Blok-L','Blok-L',36483,'In Earth orbit','','1975-06-03 00:00:00',732.9,436,40662,62.92,'Molniya' from dual union all
select 7801,'007801','1975-039A','Pollux','D-5A',36486,'Reentered','1975-08-05 00:00:00','1975-06-16 00:00:00',100.25,271,1269,29.99,'Intermediate' from dual union all
select 7802,'007802','1975-039B','Castor','D-5B',36486,'Reentered','1979-02-18 00:00:00','1975-06-16 00:00:00',100.2,273,1263,29.97,'Intermediate' from dual union all
select 7803,'007803','1975-039C','P.068','P068',36486,'Reentered','1976-08-07 00:00:00','1975-06-16 00:00:00',99.72,269,1222,29.93,'Intermediate' from dual union all
select 7804,'007804','1975-039D','D-5 despin weight','',36486,'Reentered','1976-09-26 00:00:00','1975-11-29 00:00:00',97.58,268,1018,29.96,'Intermediate' from dual union all
select 7805,'007805','1975-039E','D-5 despin weight','',36486,'Reentered','1976-06-18 00:00:00','1975-06-16 00:00:00',99.83,269,1232,29.93,'Intermediate' from dual union all
select 7806,'007806','1975-039F','D-5A/B sep clamp','',36486,'Reentered','1976-09-30 00:00:00','1975-11-29 00:00:00',97.62,268,1022,29.96,'Intermediate' from dual union all
select 7807,'007807','1975-040A','DSCS II F-5','DSCS II B-5',36487,'Reentered','1975-05-26 00:00:00','1975-05-22 00:00:00',88.36,151,236,28.58,'Intermediate' from dual union all
select 7808,'007808','1975-040B','DSCS II F-6','DSCS II B-6',36487,'Reentered','1975-05-26 00:00:00','1975-05-22 00:00:00',88.36,151,236,28.58,'Intermediate' from dual union all
select 7809,'007809','1975-040C','Transtage 25','Transtage 25',36487,'Reentered','1975-05-21 00:00:00','1975-05-20 00:00:00',88.36,134,253,28.57,'Intermediate' from dual union all
select 781,'000781','1964-017C','deb Kosmos-28','',10303,'Reentered','1964-04-07 00:00:00','1964-04-05 00:00:00',89.75,205,320,65.09,'Intermediate' from dual union all
select 7810,'007810','1975-041A','Kosmos-731','Zenit-2M',36488,'Landed','1975-06-02 00:00:00','1975-05-23 00:00:00',89.46,202,294,64.96,'Intermediate' from dual union all
select 7811,'007811','1975-041B','11A57I','11A57I',36488,'Reentered','1975-05-30 00:00:00','1975-05-24 00:00:00',89.14,195,269,64.99,'Intermediate' from dual union all
select 7812,'007812','1975-041C','Sensor cover','Zenit-2M sensor cover',36488,'Reentered','1975-05-24 00:00:00','1975-05-23 00:00:00',88.83,131,303,64.77,'Intermediate' from dual union all
select 7813,'007813','1975-041D','Sensor cover','Zenit-2M sensor cover',36488,'Reentered','1975-05-26 00:00:00','1975-05-23 00:00:00',89.65,203,311,64.93,'Intermediate' from dual union all
select 7814,'007814','1974-103X','deb Kosmos-699','',33706,'Reentered','1975-09-09 00:00:00','1975-09-06 00:00:00',90.4,286,303,64.87,'Intermediate' from dual union all
select 7815,'007815','1975-042A','INTELSAT IV F1','INTELSAT IV F1',36489,'In Earth orbit','','1977-02-24 00:00:00',1436.09,35785,35788,.06,'Stationary' from dual union all
select 7816,'007816','1975-043A','DMSP 10533','DMSP Block 5C F-2',36490,'In Earth orbit','','1977-01-01 00:00:00',101.91,805,894,98.87,'Sun-Synch' from dual union all
select 7817,'007817','1975-043B','Burner IIA No. 24 Stage 2','Star 26B S/N 008',36490,'In Earth orbit','','1997-11-22 00:00:00',101.61,799,871,98.86,'Sun-Synch' from dual union all
select 7818,'007818','1975-044A','Soyuz-18','Soyuz 7K-T No. 40',36491,'Landed','1975-07-26 00:00:00','1975-06-24 00:00:00',91.35,334,348,51.59,'Intermediate' from dual union all
select 7819,'007819','1975-044B','11S510','11S510',36491,'Reentered','1975-05-27 00:00:00','1975-05-26 00:00:00',88.06,167,190,51.58,'Intermediate' from dual union all
select 782,'000782','1964-018A','Gemini I','Gemini SC1',10304,'Reentered Attached','1964-04-12 00:00:00','1964-04-10 00:00:00',88.76,155,271,32.56,'Intermediate' from dual union all
select 7820,'007820','1975-045A','Kosmos-732','Strela-1M',36492,'In Earth orbit','','1975-07-02 00:00:00',114.6,1404,1471,74.01,'Intermediate' from dual union all
select 7821,'007821','1975-046A','Kosmos-740','Zenit-4MK',36493,'Landed','1975-06-10 00:00:00','1975-05-30 00:00:00',89.45,173,321,64.97,'Intermediate' from dual union all
select 7822,'007822','1975-045B','Kosmos-733','Strela-1M',36492,'In Earth orbit','','1975-06-29 00:00:00',116.25,1472,1552,74.02,'Intermediate' from dual union all
select 7823,'007823','1975-045C','Kosmos-734','Strela-1M',36492,'In Earth orbit','','1975-06-27 00:00:00',115.05,1444,1472,74.01,'Intermediate' from dual union all
select 7824,'007824','1975-045D','Kosmos-735','Strela-1M',36492,'In Earth orbit','','1975-06-30 00:00:00',115.27,1462,1473,74.02,'Intermediate' from dual union all
select 7825,'007825','1975-045E','Kosmos-736','Strela-1M',36492,'In Earth orbit','','1975-06-27 00:00:00',115.5,1470,1487,74.02,'Intermediate' from dual union all
select 7826,'007826','1975-045F','Kosmos-737','Strela-1M',36492,'In Earth orbit','','1975-06-27 00:00:00',115.98,1470,1530,74.01,'Intermediate' from dual;
insert into SATELLITE
select 7827,'007827','1975-045G','Kosmos-738','Strela-1M',36492,'In Earth orbit','','1975-06-28 00:00:00',115.75,1469,1510,74.02,'Intermediate' from dual union all
select 7828,'007828','1975-045H','Kosmos-739','Strela-1M',36492,'In Earth orbit','','1975-06-28 00:00:00',114.83,1424,1472,74.01,'Intermediate' from dual union all
select 7829,'007829','1975-046B','11A57I','11A57I',36493,'Reentered','1975-06-03 00:00:00','1975-05-30 00:00:00',89.11,172,289,64.98,'Intermediate' from dual union all
select 783,'000783','1964-019B','Polyot-2 part','',10305,'Reentered','1964-04-30 00:00:00','1964-04-19 00:00:00',91.3,233,444,59.92,'Intermediate' from dual union all
select 7830,'007830','1975-046C','Sensor cover','Zenit-4MK sensor cover',36493,'Reentered','1975-06-04 00:00:00','1975-05-28 00:00:00',88.86,176,261,65.13,'Intermediate' from dual union all
select 7831,'007831','1975-045J','S3M','S3M',36492,'In Earth orbit','','1975-06-28 00:00:00',117.99,1482,1700,73.99,'Intermediate' from dual union all
select 7832,'007832','1972-058C','Delta 89 large debris','[AKA 6127]',27990,'Reentered','1977-02-12 00:00:00','1992-02-06 00:00:00',98.06,582,749,98.41,'Sun-Synch' from dual union all
select 7833,'007833','1972-058D','deb Delta 89','',27990,'Reentered','1981-06-29 00:00:00','1975-05-23 00:00:00',100.41,641,915,98.38,'Sun-Synch' from dual union all
select 7834,'007834','1972-058E','deb Delta 89','',27990,'Reentered','1981-05-09 00:00:00','1981-05-06 00:00:00',90.61,287,322,98.15,'Sun-Synch' from dual union all
select 7835,'007835','1972-058F','deb Delta 89','',27990,'Reentered','1977-02-23 00:00:00','1975-05-23 00:00:00',96.69,409,792,98.1,'Sun-Synch' from dual union all
select 7836,'007836','1972-058G','deb Delta 89','',27990,'In Earth orbit','','1992-02-05 00:00:00',104.06,683,1218,98.04,'Sun-Synch' from dual union all
select 7837,'007837','1972-058H','deb Delta 89','',27990,'Reentered','1981-09-25 00:00:00','1981-09-19 00:00:00',91.87,360,373,97.96,'Sun-Synch' from dual union all
select 7838,'007838','1972-058J','deb Delta 89','',27990,'In Earth orbit','','1992-02-05 00:00:00',103.8,673,1204,98.07,'Sun-Synch' from dual union all
select 7839,'007839','1972-058K','deb Delta 89','',27990,'In Earth orbit','','1992-02-06 00:00:00',101.21,633,999,98.38,'Sun-Synch' from dual union all
select 784,'000784','1964-019A','Polyot-2','I-1B No. 112',10305,'Reentered','1966-06-08 00:00:00','1964-05-25 00:00:00',92.3,303,472,58.06,'Intermediate' from dual union all
select 7840,'007840','1972-058L','deb Delta 89','',27990,'Reentered','1991-03-16 00:00:00','1991-03-16 00:00:00',88.08,171,188,98.32,'Sun-Synch' from dual union all
select 7841,'007841','1972-058M','deb Delta 89','',27990,'Reentered','1990-05-28 00:00:00','1975-05-23 00:00:00',100.94,664,942,98.14,'Sun-Synch' from dual union all
select 7842,'007842','1972-058N','deb Delta 89','',27990,'Reentered','2000-01-17 00:00:00','1992-02-06 00:00:00',96.99,545,685,97.99,'Sun-Synch' from dual union all
select 7843,'007843','1972-058P','deb Delta 89','',27990,'Reentered','1988-11-10 00:00:00','1988-11-09 00:00:00',89.7,257,263,98.07,'Sun-Synch' from dual union all
select 7844,'007844','1972-058Q','deb Delta 89','',27990,'Reentered','1982-03-27 00:00:00','1975-05-23 00:00:00',99.68,586,901,98.31,'Sun-Synch' from dual union all
select 7845,'007845','1972-058R','deb Delta 89','',27990,'Reentered','1988-01-08 00:00:00','1988-01-07 00:00:00',88.54,192,213,97.96,'Sun-Synch' from dual union all
select 7846,'007846','1972-058S','deb Delta 89','',27990,'In Earth orbit','','1992-02-05 00:00:00',99.95,634,879,98.78,'Sun-Synch' from dual union all
select 7847,'007847','1972-058T','deb Delta 89','',27990,'Reentered','2000-01-05 00:00:00','1992-02-07 00:00:00',96.97,551,677,98.43,'Sun-Synch' from dual union all
select 7848,'007848','1972-058U','deb Delta 89','',27990,'Reentered','1989-07-20 00:00:00','1989-07-19 00:00:00',88.11,168,193,97.97,'Sun-Synch' from dual union all
select 7849,'007849','1972-058V','deb Delta 89','',27990,'In Earth orbit','','1992-02-04 00:00:00',104.61,680,1273,98,'Sun-Synch' from dual union all
select 785,'000785','1964-016D','Zond','3MV-1 No. 4',10302,'Deep Space','1964-04-04 00:00:00','',null,null,null,null,'' from dual union all
select 7850,'007850','1972-058W','deb Delta 89','',27990,'In Earth orbit','','1992-02-06 00:00:00',102.39,654,1089,98.15,'Sun-Synch' from dual union all
select 7851,'007851','1972-058X','deb Delta 89','',27990,'In Earth orbit','','1992-02-05 00:00:00',101.23,630,1004,98.29,'Sun-Synch' from dual union all
select 7852,'007852','1972-058Y','deb Delta 89','',27990,'Reentered','2002-06-07 00:00:00','1992-02-06 00:00:00',99.69,624,863,98.05,'Sun-Synch' from dual union all
select 7853,'007853','1972-058Z','deb Delta 89','',27990,'Reentered','1999-11-30 00:00:00','1992-02-07 00:00:00',96.82,536,676,98.34,'Sun-Synch' from dual union all
select 7854,'007854','1972-058AA','deb Delta 89 (large)','',27990,'Reentered','1989-04-17 00:00:00','1989-04-16 00:00:00',89.27,224,253,97.92,'Sun-Synch' from dual union all
select 7855,'007855','1972-058AB','deb Delta 89','',27990,'Reentered','2002-02-14 00:00:00','1992-02-05 00:00:00',99.44,613,851,98.02,'Sun-Synch' from dual union all
select 7856,'007856','1972-058AC','deb Delta 89','',27990,'Reentered','1979-05-09 00:00:00','1979-04-21 00:00:00',94.85,507,516,98.13,'Sun-Synch' from dual union all
select 7857,'007857','1972-058AD','Delta 89 large debris','[AKA 7832, 6127]',27990,'Reentered','2004-12-14 00:00:00','1992-02-03 00:00:00',99.1,610,821,98.61,'Sun-Synch' from dual union all
select 7858,'007858','1972-058AE','deb Delta 89','',27990,'Reentered','1981-12-28 00:00:00','1981-12-26 00:00:00',90.32,279,302,98.1,'Sun-Synch' from dual union all
select 7859,'007859','1972-058AF','deb Delta 89','',27990,'Reentered','1980-04-09 00:00:00','1975-06-04 00:00:00',98.96,585,833,98.17,'Sun-Synch' from dual union all
select 786,'000786','1964-020A','OPS 3743','GAMBIT SV 957',10306,'Reentered','1964-04-29 00:00:00','1964-04-25 00:00:00',89.32,151,331,103.57,'Retrograde' from dual union all
select 7860,'007860','1972-058AG','deb Delta 89','',27990,'Reentered','1979-07-18 00:00:00','1979-07-03 00:00:00',92.16,370,391,98.15,'Sun-Synch' from dual union all
select 7861,'007861','1972-058AH','deb Delta 89','',27990,'Reentered','1979-07-30 00:00:00','1979-07-15 00:00:00',92.33,374,404,98.47,'Sun-Synch' from dual union all
select 7862,'007862','1972-058AJ','deb Delta 89','',27990,'Reentered','1980-07-11 00:00:00','1975-05-31 00:00:00',100.45,626,934,98.34,'Sun-Synch' from dual union all
select 7863,'007863','1972-058AK','deb Delta 89','',27990,'Reentered','1979-11-02 00:00:00','1979-10-17 00:00:00',95.74,518,591,97.99,'Sun-Synch' from dual union all
select 7864,'007864','1972-058AL','deb Delta 89','',27990,'Reentered','1981-08-14 00:00:00','1975-05-29 00:00:00',100.74,641,947,98.3,'Sun-Synch' from dual union all
select 7865,'007865','1972-058AM','deb Delta 89','',27990,'In Earth orbit','','1992-02-06 00:00:00',101.12,620,1004,98.05,'Sun-Synch' from dual union all
select 7866,'007866','1972-058AN','deb Delta 89','',27990,'Reentered','1990-08-26 00:00:00','1975-05-31 00:00:00',101.99,673,1033,98.27,'Sun-Synch' from dual union all
select 7867,'007867','1972-058AP','deb Delta 89','',27990,'Reentered','2001-10-31 00:00:00','1992-02-07 00:00:00',98.55,626,752,97.36,'Sun-Synch' from dual union all
select 7868,'007868','1972-058AQ','deb Delta 89','',27990,'Reentered','1991-08-31 00:00:00','1991-08-31 00:00:00',87.73,153,172,98.07,'Sun-Synch' from dual union all
select 7869,'007869','1972-058AR','deb Delta 89','',27990,'Reentered','1990-01-05 00:00:00','1975-06-04 00:00:00',101.21,644,988,98.28,'Sun-Synch' from dual union all
select 787,'000787','1964-020B','deb USAF satellite/Agena D','',10306,'Reentered','1964-04-26 00:00:00','1964-04-24 00:00:00',89.48,158,340,103.73,'Retrograde' from dual union all
select 7870,'007870','1972-058AS','deb Delta 89','',27990,'Reentered','1999-09-05 00:00:00','1992-02-07 00:00:00',96.63,538,657,98.25,'Sun-Synch' from dual union all
select 7871,'007871','1972-058AT','deb Delta 89','',27990,'Reentered','1990-05-07 00:00:00','1990-05-06 00:00:00',88.83,208,226,97.87,'Sun-Synch' from dual union all
select 7872,'007872','1972-058AU','deb Delta 89','',27990,'Reentered','1982-03-02 00:00:00','1975-06-04 00:00:00',99.84,654,848,98.34,'Sun-Synch' from dual union all
select 7873,'007873','1972-058AV','deb Delta 89','',27990,'Reentered','1978-12-31 00:00:00','1978-12-12 00:00:00',93.2,404,458,97.53,'Sun-Synch' from dual union all
select 7874,'007874','1972-058AW','deb Delta 89','',27990,'Reentered','2000-07-14 00:00:00','1992-02-06 00:00:00',96.93,556,668,98.22,'Sun-Synch' from dual union all
select 7875,'007875','1972-058AX','deb Delta 89','',27990,'Reentered','1989-03-25 00:00:00','1989-03-25 00:00:00',88.01,166,186,98.04,'Sun-Synch' from dual union all
select 7876,'007876','1972-058AY','deb Delta 89','',27990,'Reentered','1989-12-05 00:00:00','1989-12-05 00:00:00',90.01,264,286,99.89,'Retrograde' from dual union all
select 7877,'007877','1975-047A','Kosmos-741','Gektor-Priroda',36494,'Landed','1975-06-11 00:00:00','1975-06-01 00:00:00',88.84,206,228,81.33,'Intermediate' from dual union all
select 7878,'007878','1975-047B','11A57I','11A57I',36494,'Reentered','1975-06-05 00:00:00','1975-06-01 00:00:00',88.47,180,217,81.38,'Intermediate' from dual union all
select 7879,'007879','1975-041E','Nauka part','',36488,'Reentered','1975-06-06 00:00:00','1975-05-31 00:00:00',89.25,196,279,64.99,'Intermediate' from dual union all
select 788,'000788','1964-020C','deb USAF satellite/Agena D','',10306,'Reentered','1964-04-25 00:00:00','1964-04-24 00:00:00',89.24,151,322,103.55,'Retrograde' from dual union all
select 7880,'007880','1975-041F','Nauka part','',36488,'Reentered','1975-06-09 00:00:00','1975-05-31 00:00:00',89.82,149,383,65.12,'Intermediate' from dual union all
select 7881,'007881','1975-041G','Nauka part','',36488,'Reentered','1975-06-11 00:00:00','1975-05-31 00:00:00',89.76,221,305,65.4,'Intermediate' from dual union all
select 7882,'007882','1972-058AZ','deb Delta 89','',27990,'Reentered','1980-11-13 00:00:00','1980-11-11 00:00:00',90.45,284,310,98.15,'Sun-Synch' from dual union all
select 7883,'007883','1972-058BA','deb Delta 89','',27990,'Reentered','1981-11-04 00:00:00','1981-10-30 00:00:00',90.14,277,286,98.39,'Sun-Synch' from dual union all
select 7884,'007884','1972-058BB','deb Delta 89','',27990,'Reentered','1975-11-23 00:00:00','1975-09-27 00:00:00',97.2,491,759,98.84,'Sun-Synch' from dual union all
select 7885,'007885','1975-041H','Kapsula Kosmosa-731','Nauka 9KS No. 2',36488,'Reentered','1975-06-19 00:00:00','1975-06-01 00:00:00',89.35,199,286,64.96,'Intermediate' from dual union all
select 7886,'007886','1972-058BC','deb Delta 89','',27990,'Reentered','1980-05-03 00:00:00','1980-05-01 00:00:00',91.53,337,363,98.14,'Sun-Synch' from dual union all
select 7887,'007887','1972-058BD','deb Delta 89','',27990,'Reentered','1980-03-17 00:00:00','1975-06-09 00:00:00',98.87,542,868,98.53,'Sun-Synch' from dual union all
select 7888,'007888','1972-058BE','deb Delta 89','',27990,'Reentered','1989-12-14 00:00:00','1989-12-14 00:00:00',89.01,209,243,98.56,'Sun-Synch' from dual union all
select 7889,'007889','1972-058BF','deb Delta 89','',27990,'Reentered','1992-10-10 00:00:00','1992-02-07 00:00:00',95.84,490,629,98.6,'Sun-Synch' from dual union all
select 789,'000789','1964-020D','deb USAF satellite/Agena D','',10306,'Reentered','1964-04-25 00:00:00','1964-04-24 00:00:00',89.5,128,372,103.51,'Retrograde' from dual union all
select 7890,'007890','1972-058BG','deb Delta 89','',27990,'In Earth orbit','','1992-02-06 00:00:00',101.68,642,1035,98.69,'Sun-Synch' from dual union all
select 7891,'007891','1972-058BH','deb Delta 89','',27990,'Reentered','1981-09-25 00:00:00','1981-09-20 00:00:00',91.5,331,366,98.17,'Sun-Synch' from dual union all
select 7892,'007892','1972-058BJ','deb Delta 89','',27990,'In Earth orbit','','1992-02-06 00:00:00',99.52,630,842,98.75,'Sun-Synch' from dual union all
select 7893,'007893','1972-058BK','deb Delta 89','',27990,'In Earth orbit','','1992-02-06 00:00:00',105.34,693,1328,97.76,'Sun-Synch' from dual union all
select 7894,'007894','1972-058BL','deb Delta 89','',27990,'Reentered','1980-04-25 00:00:00','1980-04-18 00:00:00',92.49,386,408,98.16,'Sun-Synch' from dual union all
select 7895,'007895','1972-058BM','deb Delta 89','',27990,'Reentered','1979-10-24 00:00:00','1979-10-23 00:00:00',90.79,295,332,98.21,'Sun-Synch' from dual union all
select 7896,'007896','1972-058BN','deb Delta 89','',27990,'Reentered','1979-11-27 00:00:00','1975-06-02 00:00:00',98.18,574,770,97.61,'Sun-Synch' from dual union all
select 7897,'007897','1972-058BP','deb Delta 89','',27990,'Reentered','1980-04-12 00:00:00','1980-04-09 00:00:00',91.72,351,367,98.12,'Sun-Synch' from dual union all
select 7898,'007898','1972-058BQ','deb Delta 89','',27990,'Reentered','1981-07-09 00:00:00','1975-06-01 00:00:00',99,624,797,98.67,'Sun-Synch' from dual union all
select 7899,'007899','1972-058BR','deb Delta 89','',27990,'Reentered','1981-07-23 00:00:00','1975-05-31 00:00:00',99.63,575,907,98.5,'Sun-Synch' from dual union all
select 79,'000079','61 Alpha 2','Samos 2','SAMOS E-1 No. 2',7260,'Reentered Attached','1970-10-09 00:00:00','1961-03-02 00:00:00',94.89,474,553,97.4,'Sun-Synch' from dual union all
select 790,'000790','1964-020E','deb USAF satellite/Agena D','',10306,'Reentered','1964-04-25 00:00:00','1964-04-24 00:00:00',89.15,134,331,103.41,'Retrograde' from dual union all
select 7900,'007900','1975-048A','Kosmos-742','Zenit-4MK',36495,'Landed','1975-06-15 00:00:00','1975-06-05 00:00:00',89.28,145,333,62.82,'Intermediate' from dual union all
select 7901,'007901','1975-048B','11A57I','11A57I',36495,'Reentered','1975-06-11 00:00:00','1975-06-05 00:00:00',89.52,175,327,62.82,'Intermediate' from dual union all
select 7902,'007902','1975-042B','Centaur D-1A AC-35','Centaur D-1A AC-35',36489,'In Earth orbit','','1975-06-25 00:00:00',655.16,598,36621,26.13,'GEO Transfer' from dual union all
select 7903,'007903','1975-049A','Molniya-1','Molniya-1 No. 24',36496,'Reentered','1987-08-12 00:00:00','1975-07-19 00:00:00',717.86,476,39883,62.86,'Molniya' from dual union all
select 7904,'007904','1975-049C','11S510','11S510',36496,'Reentered','1975-07-16 00:00:00','1975-06-12 00:00:00',90.73,213,408,62.84,'Intermediate' from dual union all
select 7905,'007905','1975-049D','BOZ','BOZ',36496,'Reentered','1975-07-09 00:00:00','1975-06-13 00:00:00',90.54,199,403,62.83,'Intermediate' from dual union all
select 7906,'007906','1975-049E','deb Blok-I','',36496,'Reentered','1975-08-16 00:00:00','1975-06-05 00:00:00',91.14,220,440,62.83,'Intermediate' from dual union all
select 7907,'007907','1972-058BS','deb Delta 89','',27990,'In Earth orbit','','1992-02-05 00:00:00',103.5,689,1160,97.71,'Sun-Synch' from dual union all
select 7908,'007908','1972-058BT','deb Delta 89','',27990,'Reentered','1979-06-16 00:00:00','1979-04-20 00:00:00',95.14,479,572,97.86,'Sun-Synch' from dual union all
select 7909,'007909','1972-058BU','deb Delta 89','',27990,'Reentered','1979-12-06 00:00:00','1979-12-04 00:00:00',90.47,274,322,98.39,'Sun-Synch' from dual union all
select 791,'000791','1964-021A','Kosmos-29','Zenit-2 No. 19',10307,'Landed','1964-05-03 00:00:00','1964-04-28 00:00:00',89.47,202,295,65.01,'Intermediate' from dual union all
select 7910,'007910','1975-049B','SRET-2','SRET-2',36496,'Reentered','1988-07-10 00:00:00','1977-04-09 00:00:00',736.45,909,40363,62.95,'Molniya' from dual union all
select 7911,'007911','1975-044C','Salyut-4 object','(long-lived)',36491,'Reentered','1976-05-15 00:00:00','1975-06-06 00:00:00',91.31,330,348,51.61,'Intermediate' from dual union all
select 7912,'007912','1975-044D','Garbage bag','',36491,'Reentered','1975-09-20 00:00:00','1975-06-06 00:00:00',91.4,340,347,51.59,'Intermediate' from dual union all
select 7913,'007913','1975-048C','Sensor cover','Zenit-4MK sensor cover',36495,'Reentered','1975-06-12 00:00:00','1975-06-06 00:00:00',89.25,169,305,62.81,'Intermediate' from dual union all
select 7914,'007914','1972-058BV','deb Delta 89','',27990,'Reentered','1981-06-28 00:00:00','1981-06-03 00:00:00',92.69,366,446,98.13,'Sun-Synch' from dual union all
select 7915,'007915','1975-050A','Venera-9','4V-1 No. 660',36497,'Deep Space','1975-06-08 00:00:00','',null,null,null,null,'' from dual union all
select 7916,'007916','1975-050B','SOZ','SOZ',36497,'Reentered','1975-06-09 00:00:00','1975-06-08 00:00:00',88.02,165,188,51.48,'Intermediate' from dual;
insert into SATELLITE
select 7917,'007917','1975-050C','SOZ','SOZ',36497,'Reentered','1975-06-09 00:00:00','1975-06-08 00:00:00',88.09,163,197,51.54,'Intermediate' from dual union all
select 7918,'007918','1975-051A','OPS 6381','HEXAGON SV-10',36498,'Deorbited','1975-11-05 00:00:00','1975-07-08 00:00:00',88.69,156,264,96.36,'Sun-Synch' from dual union all
select 7919,'007919','1975-051B','Titan 23D-10 Stage 2','LR91AJ11 23D-10',36498,'Reentered','1975-06-11 00:00:00','1975-06-10 00:00:00',88.18,147,221,96.37,'Sun-Synch' from dual union all
select 792,'000792','1964-021B','8A92E','8A92E',10307,'Reentered','1964-05-11 00:00:00','1964-05-10 00:00:00',88.23,181,193,65.03,'Intermediate' from dual union all
select 7920,'007920','1975-046D','KDU','KDU',36493,'Reentered','1975-06-18 00:00:00','1975-06-09 00:00:00',89.22,169,303,64.93,'Intermediate' from dual union all
select 7921,'007921','1975-046E','Blenda','Sunshade',36493,'Reentered','1975-06-18 00:00:00','1975-06-09 00:00:00',89.01,170,282,64.96,'Intermediate' from dual union all
select 7922,'007922','1975-025B','Kosmos-724 DU','US-A DU',36472,'Reentered','1975-08-07 00:00:00','1975-06-12 00:00:00',89.56,246,259,64.95,'Intermediate' from dual union all
select 7923,'007923','1975-025C','Kosmos-724 RLS','US-A RLS',36472,'Reentered','1975-06-17 00:00:00','1975-06-12 00:00:00',89.53,246,257,64.97,'Intermediate' from dual union all
select 7924,'007924','1975-052A','Nimbus 6','Nimbus F',36499,'In Earth orbit','','1980-01-03 00:00:00',107.45,1103,1115,99.9,'Sun-Synch' from dual union all
select 7925,'007925','1975-053A','Kosmos-743','Zenit-4MK',36500,'Landed','1975-06-25 00:00:00','1975-06-14 00:00:00',89.75,180,345,62.82,'Intermediate' from dual union all
select 7926,'007926','1975-053B','11S510','11S510',36500,'Reentered','1975-06-20 00:00:00','1975-06-15 00:00:00',89.2,178,292,62.79,'Intermediate' from dual union all
select 7927,'007927','1975-053C','Sensor cover','Zenit-4MK sensor cover',36500,'Reentered','1975-06-18 00:00:00','1975-06-13 00:00:00',88.45,160,235,62.92,'Intermediate' from dual union all
select 7928,'007928','1972-058BW','deb Delta 89','',27990,'Reentered','1980-05-09 00:00:00','1980-05-05 00:00:00',91.31,322,356,98.14,'Sun-Synch' from dual union all
select 7929,'007929','1972-058BX','deb Delta 89','',27990,'Reentered','1980-07-16 00:00:00','1975-06-12 00:00:00',98.63,603,783,99.23,'Sun-Synch' from dual union all
select 793,'000793','1964-021C','deb Kosmos-29','',10307,'Reentered','1964-04-27 00:00:00','1964-04-26 00:00:00',88.68,199,220,65.06,'Intermediate' from dual union all
select 7930,'007930','1972-058BY','deb Delta 89','',27990,'Reentered','2015-10-18 00:00:00','1992-02-07 00:00:00',100.24,628,911,98.44,'Sun-Synch' from dual union all
select 7931,'007931','1972-058BZ','deb Delta 89','',27990,'Reentered','1988-10-30 00:00:00','1988-10-29 00:00:00',88.81,204,228,98.21,'Sun-Synch' from dual union all
select 7932,'007932','1972-058CA','deb Delta 89','',27990,'Reentered','1982-05-22 00:00:00','1975-06-16 00:00:00',98.96,599,818,98.05,'Sun-Synch' from dual union all
select 7933,'007933','1972-058CB','deb Delta 89','',27990,'In Earth orbit','','1992-02-02 00:00:00',102.9,628,1164,99.35,'Sun-Synch' from dual union all
select 7934,'007934','1972-058CC','deb Delta 89','',27990,'Reentered','1991-12-08 00:00:00','1991-12-05 00:00:00',90.32,279,302,98.08,'Sun-Synch' from dual union all
select 7935,'007935','1972-058CD','deb Delta 89','',27990,'Reentered','1992-04-17 00:00:00','1992-02-06 00:00:00',93.49,426,465,97.96,'Sun-Synch' from dual union all
select 7936,'007936','1972-058CE','deb Delta 89','',27990,'Reentered','1980-09-07 00:00:00','1980-09-03 00:00:00',91.6,338,369,98.14,'Sun-Synch' from dual union all
select 7937,'007937','1975-051C','SSU 1','P-226 No. 1',36498,'In Earth orbit','','1975-07-08 00:00:00',113.6,1387,1396,95.09,'Retrograde' from dual union all
select 7938,'007938','1975-051D','deb USAF satellite','',36498,'In Earth orbit','','1975-07-17 00:00:00',113.27,1348,1406,94.98,'Polar' from dual union all
select 7939,'007939','1975-051E','deb USAF satellite','',36498,'In Earth orbit','','1975-07-12 00:00:00',113.95,1386,1430,95.17,'Retrograde' from dual union all
select 794,'000794','1964-021D','deb Kosmos-29','',10307,'Reentered','1964-04-28 00:00:00','1964-04-26 00:00:00',89.37,205,282,64.98,'Intermediate' from dual union all
select 7940,'007940','1972-058CF','deb Delta 89','',27990,'In Earth orbit','','1992-02-05 00:00:00',103.77,655,1219,98.07,'Sun-Synch' from dual union all
select 7941,'007941','1972-058CG','deb Delta 89','',27990,'Reentered','2003-02-24 00:00:00','1992-02-07 00:00:00',98.18,599,745,98.43,'Sun-Synch' from dual union all
select 7942,'007942','1972-058CH','deb Delta 89','',27990,'Reentered','1993-04-16 00:00:00','1992-02-07 00:00:00',95.35,496,574,98.23,'Sun-Synch' from dual union all
select 7943,'007943','1972-058CJ','deb Delta 89','',27990,'Reentered','2003-04-01 00:00:00','1992-02-07 00:00:00',98.1,582,754,98.39,'Sun-Synch' from dual union all
select 7944,'007944','1972-058CK','deb Delta 89','',27990,'Reentered','1999-11-06 00:00:00','1992-02-06 00:00:00',96.63,545,650,98.22,'Sun-Synch' from dual union all
select 7945,'007945','1972-058CL','deb Delta 89','',27990,'Reentered','1990-03-03 00:00:00','1975-06-12 00:00:00',100.71,656,929,98.31,'Sun-Synch' from dual union all
select 7946,'007946','1975-052B','Delta 111','Delta SSPU DSV-3P-4',36499,'In Earth orbit','','1975-07-23 00:00:00',107.25,1094,1105,99.96,'Sun-Synch' from dual union all
select 7947,'007947','1975-054A','Venera-10','4V-1 No. 661',36501,'Deep Space','1975-06-14 00:00:00','',null,null,null,null,'' from dual union all
select 7948,'007948','1975-044E','Garbage bag','(long-lived)',36491,'Reentered','1976-03-28 00:00:00','1975-06-14 00:00:00',91.34,336,345,51.58,'Intermediate' from dual union all
select 7949,'007949','1975-044F','Garbage bag','',36491,'Reentered','1975-06-17 00:00:00','1975-06-14 00:00:00',91.37,339,345,51.59,'Intermediate' from dual union all
select 795,'000795','1964-021E','deb Kosmos-29','',10307,'Reentered','1964-04-27 00:00:00','1964-04-26 00:00:00',88.58,168,241,65.05,'Intermediate' from dual union all
select 7950,'007950','1975-054B','SOZ','SOZ',36501,'Reentered','1975-06-15 00:00:00','1975-06-14 00:00:00',87.99,157,193,51.54,'Intermediate' from dual union all
select 7951,'007951','1975-048D','KDU','KDU',36495,'Reentered','1975-06-16 00:00:00','1975-06-14 00:00:00',88.71,143,279,62.86,'Intermediate' from dual union all
select 7952,'007952','1975-048E','Blenda','Sunshade',36495,'Reentered','1975-06-15 00:00:00','1975-06-14 00:00:00',88.71,143,279,62.86,'Intermediate' from dual union all
select 7953,'007953','1975-044G','Garbage bag','',36491,'Reentered','1975-10-04 00:00:00','1975-06-14 00:00:00',91.35,336,346,51.6,'Intermediate' from dual union all
select 7954,'007954','1975-054C','SOZ','SOZ',36501,'Reentered','1975-06-15 00:00:00','1975-06-14 00:00:00',88.17,162,206,51.52,'Intermediate' from dual union all
select 7955,'007955','1975-044H','Garbage bag','',36491,'Reentered','1975-10-01 00:00:00','1975-06-16 00:00:00',91.34,336,345,51.6,'Intermediate' from dual union all
select 7956,'007956','1975-044J','Garbage bag','',36491,'Reentered','1975-09-24 00:00:00','1975-06-16 00:00:00',91.39,333,353,51.61,'Intermediate' from dual union all
select 7957,'007957','1972-058CM','deb Delta 89','',27990,'In Earth orbit','','1992-02-07 00:00:00',107.39,648,1563,99,'Sun-Synch' from dual union all
select 7958,'007958','1972-058CN','deb Delta 89','',27990,'In Earth orbit','','1992-02-06 00:00:00',104.47,695,1245,97.67,'Sun-Synch' from dual union all
select 7959,'007959','1972-058CP','deb Delta 89','',27990,'In Earth orbit','','1992-02-06 00:00:00',102.62,678,1088,97.99,'Sun-Synch' from dual union all
select 796,'000796','1964-022A','OPS 2921','CORONA J-8',10308,'Reentered Attached','1964-05-26 00:00:00','1964-05-04 00:00:00',90.53,175,426,79.93,'Intermediate' from dual union all
select 7960,'007960','1972-058CQ','deb Delta 89','',27990,'Reentered','1999-12-19 00:00:00','1992-02-07 00:00:00',96.76,547,660,98.22,'Sun-Synch' from dual union all
select 7961,'007961','1972-058CR','deb Delta 89','',27990,'Reentered','1980-01-12 00:00:00','1975-06-19 00:00:00',98.91,555,859,98.52,'Sun-Synch' from dual union all
select 7962,'007962','1975-044K','Garbage bag','',36491,'Reentered','1975-09-15 00:00:00','1975-06-17 00:00:00',91.35,335,347,51.61,'Intermediate' from dual union all
select 7963,'007963','1975-055A','OPS 4966','CANYON 6',36502,'In Earth orbit','','1975-06-30 00:00:00',1421.47,30200,40800,9,'Near-synch' from dual union all
select 7964,'007964','1975-055B','Agena D 2806','Agena D',36502,'In Earth orbit','','1975-06-30 00:00:00',1398.62,29700,40400,8,'Near-synch' from dual union all
select 7965,'007965','1975-052C','Spurious debris','[radar error]',36499,'Spurious','1981-07-01 00:00:00','1975-06-18 00:00:00',107.26,1091,1108,99.95,'Sun-Synch' from dual union all
select 7966,'007966','1972-058CS','deb Delta 89','',27990,'Reentered','1981-12-05 00:00:00','1981-12-03 00:00:00',90.6,285,323,98.42,'Sun-Synch' from dual union all
select 7967,'007967','1972-058CT','deb Delta 89','',27990,'Reentered','2000-10-12 00:00:00','1992-02-06 00:00:00',97.23,557,695,98.37,'Sun-Synch' from dual union all
select 7968,'007968','1975-056A','Kosmos-744','Tselina-D',36503,'Reentered','1991-10-12 00:00:00','1975-07-20 00:00:00',97.04,598,636,81.26,'Intermediate' from dual union all
select 7969,'007969','1975-056B','8A92ME','8A92ME',36503,'Reentered','2000-01-22 00:00:00','1975-07-20 00:00:00',97.22,584,668,81.27,'Intermediate' from dual union all
select 797,'000797','1964-023A','Kosmos-30','Zenit-4 No. 2',10309,'Landed','1964-05-26 00:00:00','1964-05-20 00:00:00',90.21,202,368,64.88,'Intermediate' from dual union all
select 7970,'007970','1975-057A','OSO 8','OSO I',36504,'Reentered','1986-07-09 00:00:00','1975-07-21 00:00:00',95.6,540,556,32.95,'Intermediate' from dual union all
select 7971,'007971','1975-057B','Delta 112','Delta SSPU DSV-3P-4',36504,'Reentered','1981-04-09 00:00:00','1975-07-23 00:00:00',95.6,541,555,32.95,'Intermediate' from dual union all
select 7972,'007972','1975-053D','KDU','KDU',36500,'Reentered','1975-06-29 00:00:00','1975-06-24 00:00:00',89.09,163,296,62.71,'Intermediate' from dual union all
select 7973,'007973','1972-058CU','deb Delta 89','',27990,'Reentered','1978-11-14 00:00:00','1978-10-31 00:00:00',92.73,377,440,98.07,'Sun-Synch' from dual union all
select 7974,'007974','1972-058CV','deb Delta 89','',27990,'Reentered','1989-06-04 00:00:00','1989-06-03 00:00:00',89.04,216,239,97.93,'Sun-Synch' from dual union all
select 7975,'007975','1972-058CW','deb Delta 89','',27990,'Reentered','1980-03-31 00:00:00','1980-03-22 00:00:00',92.68,389,423,98.21,'Sun-Synch' from dual union all
select 7976,'007976','1972-058CX','deb Delta 89','',27990,'Reentered','1994-12-09 00:00:00','1992-02-06 00:00:00',95.71,506,600,98.1,'Sun-Synch' from dual union all
select 7977,'007977','1972-058CY','deb Delta 89','',27990,'Reentered','1979-01-14 00:00:00','1975-06-25 00:00:00',96.08,366,776,97.66,'Sun-Synch' from dual union all
select 7978,'007978','1972-058CZ','deb Delta 89','',27990,'Reentered','1980-03-09 00:00:00','1975-06-24 00:00:00',97.8,578,729,98.78,'Sun-Synch' from dual union all
select 7979,'007979','1972-058DA','deb Delta 89','',27990,'In Earth orbit','','1992-02-03 00:00:00',105.76,707,1352,97.83,'Sun-Synch' from dual union all
select 798,'000798','1964-023B','11A57I','11A57I',10309,'Reentered','1964-06-07 00:00:00','1964-06-06 00:00:00',88.15,175,191,64.83,'Intermediate' from dual union all
select 7980,'007980','1972-058DB','deb Delta 89','',27990,'Reentered','1979-02-18 00:00:00','1979-02-15 00:00:00',92.49,380,414,98.19,'Sun-Synch' from dual union all
select 7981,'007981','1975-053E','Blenda','Sunshade',36500,'Reentered','1975-06-26 00:00:00','1975-06-24 00:00:00',88.87,166,272,62.83,'Intermediate' from dual union all
select 7982,'007982','1975-058A','Kosmos-745','DS-P1-Yu No. 76',36505,'Reentered','1976-03-12 00:00:00','1975-07-24 00:00:00',92.14,260,499,70.99,'Intermediate' from dual union all
select 7983,'007983','1975-058B','S1M','S1M',36505,'Reentered','1975-11-14 00:00:00','1975-07-24 00:00:00',91.98,256,487,70.99,'Intermediate' from dual union all
select 7984,'007984','1975-044L','Garbage bag','',36491,'Reentered','1975-10-12 00:00:00','1975-06-25 00:00:00',91.44,336,355,51.6,'Intermediate' from dual union all
select 7985,'007985','1975-059A','Kosmos-746','Zenit-4MK',36506,'Landed','1975-07-08 00:00:00','1975-06-28 00:00:00',89.37,169,318,62.8,'Intermediate' from dual union all
select 7986,'007986','1975-059B','11A57I','11A57I',36506,'Reentered','1975-07-01 00:00:00','1975-06-27 00:00:00',89.29,172,307,62.79,'Intermediate' from dual union all
select 7987,'007987','1975-053F','KDU part','KDU part',36500,'Reentered','1975-06-26 00:00:00','1975-06-25 00:00:00',88.58,161,248,62.81,'Intermediate' from dual union all
select 7988,'007988','1975-059C','Sensor cover','Zenit-4MK sensor cover',36506,'Reentered','1975-06-26 00:00:00','1975-06-25 00:00:00',89.44,176,317,62.78,'Intermediate' from dual union all
select 7989,'007989','1975-059D','Sensor cover','Zenit-4MK sensor cover',36506,'Reentered','1975-06-26 00:00:00','1975-06-25 00:00:00',89.1,169,291,62.72,'Intermediate' from dual union all
select 799,'000799','1964-024A','OPS 3592','GAMBIT SV 958',10310,'Deorbited','1964-05-22 00:00:00','1964-05-21 00:00:00',88.84,98,337,101.06,'Retrograde' from dual union all
select 7990,'007990','1975-060A','Kosmos-747','Zenit-2M',36507,'Landed','1975-07-09 00:00:00','1975-06-29 00:00:00',89.32,192,289,62.83,'Intermediate' from dual union all
select 7991,'007991','1975-060B','11A57I','11A57I',36507,'Reentered','1975-07-03 00:00:00','1975-06-29 00:00:00',89.04,175,280,62.81,'Intermediate' from dual union all
select 7992,'007992','1975-060C','Sensor cover','Zenit-2M sensor cover',36507,'Reentered','1975-06-28 00:00:00','1975-06-27 00:00:00',88.97,185,263,62.83,'Intermediate' from dual union all
select 7993,'007993','1975-060D','Sensor cover','Zenit-2M sensor cover',36507,'Reentered','1975-07-02 00:00:00','1975-06-29 00:00:00',89.6,195,315,62.84,'Intermediate' from dual union all
select 7994,'007994','1975-060E','Sensor cover','Zenit-2M sensor cover',36507,'Reentered','1975-06-29 00:00:00','1975-06-28 00:00:00',88.48,179,220,62.79,'Intermediate' from dual union all
select 7995,'007995','1974-104Q','Garbage bag','',33707,'Reentered','1975-09-16 00:00:00','1975-06-27 00:00:00',91.31,331,347,51.58,'Intermediate' from dual union all
select 7996,'007996','1974-104R','Garbage bag','',33707,'Reentered','1975-09-18 00:00:00','1975-06-27 00:00:00',91.32,333,346,51.59,'Intermediate' from dual union all
select 7997,'007997','1974-104S','Salyut-4 object','(long-lived)',33707,'Reentered','1976-03-25 00:00:00','1975-06-28 00:00:00',91.34,334,347,51.58,'Intermediate' from dual union all
select 7998,'007998','1974-104T','Garbage bag','',33707,'Reentered','1975-09-15 00:00:00','1975-06-27 00:00:00',91.35,335,347,51.6,'Intermediate' from dual union all
select 7999,'007999','1974-103Y','deb Kosmos-699','',33706,'Reentered','1978-12-06 00:00:00','1978-11-21 00:00:00',90.16,280,285,64.83,'Intermediate' from dual union all
select 8,'000008','58 Delta 2','3-y Sovetskiy ISZ','D-1 No. 2',4794,'Reentered','1960-04-06 00:00:00','1959-05-07 00:00:00',99.33,207,1247,65.12,'Intermediate' from dual union all
select 80,'000080','61 Gamma 1','AMS Venera','1VA No. 2',7266,'Deep Space','1961-02-14 00:00:00','',null,null,null,null,'' from dual union all
select 800,'000800','1964-025A','Apollo BP-13','Apollo BP-13 CM',10311,'Reentered Attached','1964-06-01 00:00:00','1964-05-30 00:00:00',88.26,178,199,31.77,'Intermediate' from dual union all
select 8000,'008000','1966-076F','NNS O-10 insulation','',13483,'Reentered','1981-09-11 00:00:00','1975-07-01 00:00:00',106.66,1054,1090,88.84,'Polar' from dual union all
select 8001,'008001','1972-058DC','deb Delta 89','',27990,'Reentered','1978-07-18 00:00:00','1978-06-18 00:00:00',92.89,397,436,98.76,'Sun-Synch' from dual union all
select 8002,'008002','1972-058DD','deb Delta 89','',27990,'Reentered','1980-08-27 00:00:00','1980-08-24 00:00:00',91.35,335,347,98.14,'Sun-Synch' from dual union all
select 8003,'008003','1972-058DE','deb Delta 89','',27990,'Reentered','1979-01-26 00:00:00','1979-01-13 00:00:00',91.24,311,360,98.85,'Sun-Synch' from dual union all
select 8004,'008004','1972-058DF','deb Delta 89','',27990,'In Earth orbit','','1993-08-04 00:00:00',102.54,673,1085,97.61,'Sun-Synch' from dual union all
select 8005,'008005','1972-058DG','deb Delta 89','',27990,'Reentered','1991-09-10 00:00:00','1991-09-09 00:00:00',88.28,171,208,98.33,'Sun-Synch' from dual;
insert into SATELLITE
select 8006,'008006','1975-061A','Kosmos-748','Zenit-4MK',36508,'Landed','1975-07-16 00:00:00','1975-07-05 00:00:00',89.8,177,353,62.82,'Intermediate' from dual union all
select 8007,'008007','1975-061B','11A57I','11A57I',36508,'Reentered','1975-07-09 00:00:00','1975-07-05 00:00:00',89.18,170,298,62.81,'Intermediate' from dual union all
select 8008,'008008','1975-061C','Sensor cover','Zenit-4MK sensor cover',36508,'Reentered','1975-07-15 00:00:00','1975-07-04 00:00:00',88.93,126,318,62.75,'Intermediate' from dual union all
select 8009,'008009','1975-062A','Kosmos-749','Tselina-OM',36509,'Reentered','1980-09-26 00:00:00','1975-08-07 00:00:00',95.2,508,549,74.03,'Intermediate' from dual union all
select 801,'000801','1964-026A','Transit VC','Transit VC',10312,'In Earth orbit','','1964-07-14 00:00:00',103.06,862,945,90.5,'Polar' from dual union all
select 8010,'008010','1975-062B','S3M','S3M',36509,'Reentered','1980-12-25 00:00:00','1975-08-03 00:00:00',95.1,499,548,74.05,'Intermediate' from dual union all
select 8011,'008011','1975-059E','KDU','KDU',36506,'Reentered','1975-07-12 00:00:00','1975-07-06 00:00:00',89.16,164,302,62.8,'Intermediate' from dual union all
select 8012,'008012','1975-059F','Blenda','Sunshade',36506,'Reentered','1975-07-09 00:00:00','1975-07-06 00:00:00',88.94,167,278,62.82,'Intermediate' from dual union all
select 8013,'008013','1975-060F','Kapsula Kosmosa-747','Nauka 22KS',36507,'Reentered','1975-07-17 00:00:00','1975-07-06 00:00:00',89.19,188,281,62.83,'Intermediate' from dual union all
select 8014,'008014','1975-060G','Nauka part','',36507,'Reentered','1975-07-09 00:00:00','1975-07-06 00:00:00',88.99,182,268,62.84,'Intermediate' from dual union all
select 8015,'008015','1975-063A','Molniya-2','Molniya-2',36510,'In Earth orbit','','1975-08-18 00:00:00',718.92,499,39912,62.85,'Molniya' from dual union all
select 8016,'008016','1975-063B','11S510','11S510',36510,'Reentered','1975-08-09 00:00:00','1975-07-17 00:00:00',90.19,209,359,62.83,'Intermediate' from dual union all
select 8017,'008017','1975-063C','BOZ','BOZ',36510,'Reentered','1975-08-09 00:00:00','1975-07-17 00:00:00',90.46,199,396,62.83,'Intermediate' from dual union all
select 8018,'008018','1975-063D','Blok-L','Blok-L',36510,'In Earth orbit','','1975-08-13 00:00:00',733.06,466,40640,62.89,'Molniya' from dual union all
select 8019,'008019','1975-044M','Garbage bag','',36491,'Reentered','1975-10-07 00:00:00','1975-07-10 00:00:00',91.27,329,345,51.57,'Intermediate' from dual union all
select 802,'000802','1964-027A','OPS 3483','CORONA J-9',10313,'Reentered Attached','1964-06-18 00:00:00','1964-06-07 00:00:00',90.26,146,429,79.96,'Intermediate' from dual union all
select 8020,'008020','1975-044N','Garbage bag','(long-lived)',36491,'Reentered','1976-03-06 00:00:00','1975-07-10 00:00:00',91.26,299,373,51,'Intermediate' from dual union all
select 8021,'008021','1975-044P','Garbage bag','',36491,'Reentered','1975-10-09 00:00:00','1975-07-10 00:00:00',91.3,333,344,51.59,'Intermediate' from dual union all
select 8022,'008022','1975-044Q','Garbage bag','',36491,'Reentered','1975-10-09 00:00:00','1975-07-11 00:00:00',91.34,333,348,51.59,'Intermediate' from dual union all
select 8023,'008023','1975-044R','Garbage bag','(long-lived)',36491,'Reentered','1976-03-29 00:00:00','1975-07-11 00:00:00',91.37,336,348,51.6,'Intermediate' from dual union all
select 8024,'008024','1975-044S','Garbage bag','(long-lived)',36491,'Reentered','1976-02-12 00:00:00','1975-07-10 00:00:00',91.38,336,349,51.6,'Intermediate' from dual union all
select 8025,'008025','1975-044T','Garbage bag','',36491,'Reentered','1975-10-05 00:00:00','1975-07-10 00:00:00',91.39,334,352,51.6,'Intermediate' from dual union all
select 8026,'008026','1975-064A','Meteor-2','Meteor-2 No. 1',36511,'In Earth orbit','','1975-08-10 00:00:00',102.4,856,889,81.29,'Intermediate' from dual union all
select 8027,'008027','1975-064B','8A92ME','8A92ME',36511,'In Earth orbit','','1975-08-14 00:00:00',102.52,837,919,81.3,'Intermediate' from dual union all
select 8028,'008028','1975-061D','Blenda','Sunshade',36508,'Reentered','1975-07-15 00:00:00','1975-07-13 00:00:00',88.76,167,260,62.82,'Intermediate' from dual union all
select 8029,'008029','1975-061E','KDU','KDU',36508,'Reentered','1975-07-16 00:00:00','1975-07-13 00:00:00',88.96,169,278,62.82,'Intermediate' from dual union all
select 803,'000803','1964-028A','Kosmos-31','DS-MT No. 2',10314,'Reentered','1964-10-20 00:00:00','1964-07-09 00:00:00',91.28,218,456,48.97,'Intermediate' from dual union all
select 8030,'008030','1975-065A','Soyuz-19','Soyuz 7K-TM No. 75',36512,'Landed','1975-07-21 00:00:00','1975-07-17 00:00:00',88.95,217,229,51.76,'Intermediate' from dual union all
select 8031,'008031','1975-065B','11S510','11S510',36512,'Reentered','1975-07-17 00:00:00','1975-07-16 00:00:00',88.32,184,199,51.78,'Intermediate' from dual union all
select 8032,'008032','1975-066A','Apollo-Soyuz Test Project','Apollo CM-111',36513,'Landed','1975-07-24 00:00:00','1975-07-17 00:00:00',88.35,162,224,51.76,'Intermediate' from dual union all
select 8033,'008033','1975-066B','Saturn S-IVB-210','Saturn S-IVB-210',36513,'Deorbited','1975-07-16 00:00:00','1975-07-15 00:00:00',87.76,155,173,51.8,'Intermediate' from dual union all
select 8034,'008034','1975-061F','PTDU cover','KDU',36508,'Reentered','1975-07-17 00:00:00','1975-07-16 00:00:00',88.86,165,272,62.84,'Intermediate' from dual union all
select 8035,'008035','1975-039G','D-5A/B sep clamp','',36486,'Reentered','1978-01-29 00:00:00','1975-07-16 00:00:00',98.61,192,1193,29.88,'Intermediate' from dual union all
select 8036,'008036','1975-067A','Kosmos-750','DS-P1-I No. 15',36514,'Reentered','1977-09-29 00:00:00','1975-08-17 00:00:00',95.25,269,793,71.04,'Intermediate' from dual union all
select 8037,'008037','1975-067B','S1M','S1M',36514,'Reentered','1976-11-17 00:00:00','1975-08-16 00:00:00',94.96,276,758,71.03,'Intermediate' from dual union all
select 8038,'008038','1975-044U','Garbage bag','',36491,'Reentered','1975-10-20 00:00:00','1975-07-20 00:00:00',91.27,328,346,51.58,'Intermediate' from dual union all
select 8039,'008039','1975-064C','deb Meteor-2','',36511,'In Earth orbit','','1975-08-29 00:00:00',102.39,853,891,81.3,'Intermediate' from dual union all
select 804,'000804','1964-028B','S1','S1',10314,'Reentered','1964-08-16 00:00:00','1964-08-16 00:00:00',87.99,157,193,48.95,'Intermediate' from dual union all
select 8040,'008040','1975-068A','Kosmos-751','Zenit-2M',36515,'Landed','1975-08-04 00:00:00','1975-07-25 00:00:00',89.6,196,314,62.81,'Intermediate' from dual union all
select 8041,'008041','1975-068B','11A57I','11A57I',36515,'Reentered','1975-07-30 00:00:00','1975-07-25 00:00:00',89.39,182,307,62.82,'Intermediate' from dual union all
select 8042,'008042','1975-066C','Docking Module 2','Apollo DM-2',36513,'Reentered','1975-08-02 00:00:00','1975-07-24 00:00:00',88.72,201,222,51.77,'Intermediate' from dual union all
select 8043,'008043','1975-069A','Kosmos-752','Vektor',36516,'Reentered','1981-02-28 00:00:00','1975-08-23 00:00:00',94.54,478,515,65.85,'Intermediate' from dual union all
select 8044,'008044','1975-069B','S3M','S3M',36516,'Reentered','1979-11-05 00:00:00','1975-09-04 00:00:00',94.43,467,515,65.86,'Intermediate' from dual union all
select 8045,'008045','1975-067C','deb Kosmos-750','',36514,'Reentered','1976-07-28 00:00:00','1975-08-19 00:00:00',96.83,333,880,70.87,'Intermediate' from dual union all
select 8046,'008046','1974-103Z','deb Kosmos-699','',33706,'Reentered','1975-11-01 00:00:00','1975-10-22 00:00:00',90.47,286,310,64.97,'Intermediate' from dual union all
select 8047,'008047','1974-103AA','deb Kosmos-699','',33706,'Reentered','1978-02-26 00:00:00','1978-01-07 00:00:00',91.45,345,347,64.91,'Intermediate' from dual union all
select 8048,'008048','1974-103AB','deb Kosmos-699','',33706,'Reentered','1976-03-22 00:00:00','1975-07-26 00:00:00',93.19,427,435,64.97,'Intermediate' from dual union all
select 8049,'008049','1974-103AC','deb Kosmos-699','',33706,'Reentered','1978-06-10 00:00:00','1978-04-30 00:00:00',91.38,341,344,64.9,'Intermediate' from dual union all
select 805,'000805','1964-026B','Transit VC despin weight','',10312,'In Earth orbit','','1964-07-06 00:00:00',103.87,901,982,90.19,'Polar' from dual union all
select 8050,'008050','1974-103AD','deb Kosmos-699','',33706,'Reentered','1975-11-12 00:00:00','1975-10-26 00:00:00',91.31,331,347,65.01,'Intermediate' from dual union all
select 8051,'008051','1974-103AE','deb Kosmos-699','',33706,'Reentered','1976-11-06 00:00:00','1975-07-26 00:00:00',93.22,426,439,64.97,'Intermediate' from dual union all
select 8052,'008052','1974-103AF','deb Kosmos-699','',33706,'Reentered','1975-08-04 00:00:00','1975-07-25 00:00:00',90.96,308,335,64.96,'Intermediate' from dual union all
select 8053,'008053','1975-070A','JSSW 3','JSSW',36517,'Reentered','1975-09-14 00:00:00','1975-08-02 00:00:00',90.88,184,452,69.01,'Intermediate' from dual union all
select 8054,'008054','1975-070B','FB-1 Stage 2','FB-1 Stage 2',36517,'Reentered','1975-08-25 00:00:00','1975-08-03 00:00:00',90.54,180,422,69.01,'Intermediate' from dual union all
select 8055,'008055','1974-104U','Soyuz-18 BO','Bitovoy otsek',33707,'Reentered','1975-10-27 00:00:00','1975-07-27 00:00:00',91.5,336,360,51.6,'Intermediate' from dual union all
select 8056,'008056','1974-103AG','deb Kosmos-699','',33706,'Reentered','1976-03-10 00:00:00','1975-07-31 00:00:00',92.65,394,415,65,'Intermediate' from dual union all
select 8057,'008057','1974-103AH','deb Kosmos-699','',33706,'Reentered','1978-11-21 00:00:00','1978-11-13 00:00:00',90.99,311,336,64.6,'Intermediate' from dual union all
select 8058,'008058','1975-033C','deb Blok Kosmos','',36480,'Reentered','1978-03-12 00:00:00','1975-07-31 00:00:00',96.26,552,607,50.69,'Intermediate' from dual union all
select 8059,'008059','1975-071A','Kosmos-753','Zenit-4MK',36518,'Landed','1975-08-13 00:00:00','1975-08-02 00:00:00',89.58,179,329,62.83,'Intermediate' from dual union all
select 806,'000806','1964-026C','Transit VC despin weight','',10312,'Reentered','2002-11-19 00:00:00','1993-03-29 00:00:00',98.95,668,749,90.81,'Polar' from dual union all
select 8060,'008060','1975-071B','11A57I','11A57I',36518,'Reentered','1975-08-07 00:00:00','1975-08-03 00:00:00',89.23,174,299,62.81,'Intermediate' from dual union all
select 8061,'008061','1975-071C','Sensor cover','Zenit-4MK sensor cover',36518,'Reentered','1975-08-01 00:00:00','1975-08-01 00:00:00',89.38,172,316,62.81,'Intermediate' from dual union all
select 8062,'008062','1975-072A','COS-B','COS-B',36519,'Reentered','1986-01-18 00:00:00','1976-01-01 00:00:00',2203.58,2536,96895,92.01,'Very High' from dual union all
select 8063,'008063','1975-072B','Delta 113','Delta SSPU DSV-3P-4',36519,'Reentered','2017-05-20 00:00:00','1975-09-10 00:00:00',139.56,333,4736,89.23,'Medium' from dual union all
select 8064,'008064','1966-056E','deb Pageos','RCS 64.9',13463,'Reentered','1990-01-03 00:00:00','1980-08-28 00:00:00',191.95,1452,7837,80.48,'Medium' from dual union all
select 8065,'008065','1966-056F','deb Pageos','RCS 0.34',13463,'Reentered','1978-04-18 00:00:00','1975-08-09 00:00:00',180.03,3023,5343,85.13,'Medium' from dual union all
select 8066,'008066','1966-056G','deb Pageos','RCS 263.0',13463,'Reentered','1989-11-26 00:00:00','1989-11-24 00:00:00',108.5,299,2016,81.87,'Intermediate' from dual union all
select 8067,'008067','1975-071D','KDU','KDU',36518,'Reentered','1975-08-16 00:00:00','1975-08-12 00:00:00',89.07,160,297,62.83,'Intermediate' from dual union all
select 8068,'008068','1975-071E','Blenda','Sunshade',36518,'Reentered','1975-08-13 00:00:00','1975-08-12 00:00:00',88.86,164,273,62.85,'Intermediate' from dual union all
select 8069,'008069','1975-073A','Kosmos-754','Zenit-4MK',36520,'Landed','1975-08-26 00:00:00','1975-08-15 00:00:00',89.36,171,315,71.37,'Intermediate' from dual union all
select 807,'000807','1964-029A','Kosmos-32','Zenit-2 No. 18',10315,'Landed','1964-06-18 00:00:00','1964-06-14 00:00:00',89.77,205,321,51.23,'Intermediate' from dual union all
select 8070,'008070','1975-073B','11A57I','11A57I',36520,'Reentered','1975-08-25 00:00:00','1975-08-15 00:00:00',89.54,202,302,71.38,'Intermediate' from dual union all
select 8071,'008071','1975-073C','Sensor cover','Zenit-4MK sensor cover',36520,'Reentered','1975-08-15 00:00:00','1975-08-14 00:00:00',88.9,199,242,71.37,'Intermediate' from dual union all
select 8072,'008072','1975-074A','Kosmos-755','Tsiklon-B',36521,'In Earth orbit','','1975-09-17 00:00:00',104.93,971,1012,82.9,'Intermediate' from dual union all
select 8073,'008073','1975-074B','S3M','S3M',36521,'In Earth orbit','','1975-09-13 00:00:00',104.78,968,1001,82.9,'Intermediate' from dual union all
select 8074,'008074','1966-056H','deb Pageos','RCS 34.34',13463,'Reentered','1996-10-08 00:00:00','1981-06-14 00:00:00',179.46,2740,5582,87.16,'Medium' from dual union all
select 8075,'008075','1966-056J','deb Pageos','RCS 0.70',13463,'Reentered','1977-02-19 00:00:00','1977-01-28 00:00:00',175.61,616,7404,81.43,'Medium' from dual union all
select 8076,'008076','1966-056K','deb Pageos','RCS 14.5',13463,'Reentered','1984-09-04 00:00:00','1984-09-04 00:00:00',105.67,222,1830,82.99,'Intermediate' from dual union all
select 8077,'008077','1966-056L','deb Pageos','RCS 0.82',13463,'Reentered','1977-04-01 00:00:00','1977-03-24 00:00:00',132.52,329,4136,83.02,'Medium' from dual union all
select 8078,'008078','1966-056M','deb Pageos','RCS 32.74',13463,'Reentered','1977-02-19 00:00:00','1977-02-08 00:00:00',172.17,393,7355,81.61,'Medium' from dual union all
select 8079,'008079','1966-056N','deb Pageos','RCS 0.40',13463,'Reentered','1986-01-12 00:00:00','1986-01-12 00:00:00',107.17,229,1963,80.03,'Intermediate' from dual union all
select 808,'000808','1964-029B','8A92E','8A92E',10315,'Reentered','1964-07-14 00:00:00','1964-07-13 00:00:00',88.07,179,179,51.23,'Intermediate' from dual union all
select 8080,'008080','1966-056P','deb Pageos','',13463,'Reentered','1977-01-21 00:00:00','1977-01-14 00:00:00',156.97,158,6364,81.48,'Medium' from dual union all
select 8081,'008081','1966-056Q','deb Pageos','RCS 0.18',13463,'Reentered','1977-05-22 00:00:00','1977-04-16 00:00:00',174.52,666,7267,82.99,'Medium' from dual union all
select 8082,'008082','1972-058DH','deb Delta 89','',27990,'Reentered','1980-01-11 00:00:00','1975-08-10 00:00:00',100.44,626,933,98.32,'Sun-Synch' from dual union all
select 8083,'008083','1972-058DJ','deb Delta 89','',27990,'Reentered','1991-03-22 00:00:00','1991-03-22 00:00:00',88.87,208,229,97.97,'Sun-Synch' from dual union all
select 8084,'008084','1972-058DK','deb Delta 89','',27990,'In Earth orbit','','1993-08-06 00:00:00',107.75,673,1573,98.64,'Sun-Synch' from dual union all
select 8085,'008085','1972-058DL','deb Delta 89','',27990,'Reentered','1978-12-23 00:00:00','1978-10-01 00:00:00',96.47,498,682,98.03,'Sun-Synch' from dual union all
select 8086,'008086','1972-058DM','deb Delta 89','',27990,'Reentered','1980-04-12 00:00:00','1980-03-20 00:00:00',93.56,421,477,98.22,'Sun-Synch' from dual union all
select 8087,'008087','1972-058DN','deb Delta 89','',27990,'Reentered','1977-05-05 00:00:00','1977-04-06 00:00:00',97.06,512,724,98.47,'Sun-Synch' from dual union all
select 8088,'008088','1972-058DP','deb Delta 89','',27990,'Reentered','1980-05-25 00:00:00','1980-01-12 00:00:00',94.37,445,532,98.2,'Sun-Synch' from dual union all
select 8089,'008089','1972-058DQ','deb Delta 89','',27990,'Reentered','1981-11-25 00:00:00','1975-08-14 00:00:00',99.05,608,819,98.7,'Sun-Synch' from dual union all
select 809,'000809','1964-026D','Altair 2 X-258C1 RH-55L','X-258C1',10312,'In Earth orbit','','1992-10-19 00:00:00',102.55,844,914,90.49,'Polar' from dual union all
select 8090,'008090','1972-058DR','deb Delta 89','',27990,'Reentered','1980-01-13 00:00:00','1982-04-30 00:00:00',91.26,333,340,97.98,'Sun-Synch' from dual union all
select 8091,'008091','1972-058DS','deb Delta 89','',27990,'Reentered','1979-02-07 00:00:00','1979-01-24 00:00:00',91.71,337,380,98.46,'Sun-Synch' from dual union all
select 8092,'008092','1972-058DT','deb Delta 89','',27990,'Reentered','2000-06-25 00:00:00','1993-08-06 00:00:00',97.25,575,680,98,'Sun-Synch' from dual union all
select 8093,'008093','1972-058DU','deb Delta 89','',27990,'Reentered','1978-07-18 00:00:00','1978-06-19 00:00:00',91.9,350,386,98.01,'Sun-Synch' from dual union all
select 8094,'008094','1972-058DV','deb Delta 89','',27990,'Reentered','1980-04-30 00:00:00','1980-04-21 00:00:00',92.42,388,399,98.07,'Sun-Synch' from dual union all
select 8095,'008095','1972-058DW','deb Delta 89','',27990,'Reentered','1983-06-09 00:00:00','1983-06-08 00:00:00',88.41,182,210,98.7,'Sun-Synch' from dual union all
select 8096,'008096','1972-058DX','deb Delta 89','',27990,'Reentered','1976-06-28 00:00:00','1976-06-18 00:00:00',91.23,298,372,97.91,'Sun-Synch' from dual;
insert into SATELLITE
select 8097,'008097','1972-058DY','deb Delta 89','',27990,'Reentered','1975-11-15 00:00:00','1975-09-01 00:00:00',91.97,335,408,98.03,'Sun-Synch' from dual union all
select 8098,'008098','1972-058DZ','deb Delta 89','',27990,'Reentered','1980-02-02 00:00:00','1975-08-15 00:00:00',102.92,500,1294,98.03,'Sun-Synch' from dual union all
select 8099,'008099','1972-058EA','deb Delta 89','',27990,'Reentered','1991-04-27 00:00:00','1991-04-27 00:00:00',88.48,193,205,98.58,'Sun-Synch' from dual union all
select 81,'000081','61 Delta 1','Explorer 9','NASA S-56A',7263,'Reentered','1964-04-09 00:00:00','1961-03-19 00:00:00',118.37,635,2581,38.86,'Intermediate' from dual union all
select 810,'000810','1964-029C','deb Kosmos-32','',10315,'Reentered','1964-06-12 00:00:00','1964-06-11 00:00:00',89.02,195,258,51.22,'Intermediate' from dual union all
select 8100,'008100','1974-103AJ','deb Kosmos-699','',33706,'Reentered','1975-09-20 00:00:00','1975-09-19 00:00:00',89.97,257,288,64.88,'Intermediate' from dual union all
select 8101,'008101','1974-103AK','deb Kosmos-699','',33706,'Reentered','1976-03-13 00:00:00','1975-08-16 00:00:00',93.28,426,444,65.01,'Intermediate' from dual union all
select 8102,'008102','1974-103AL','deb Kosmos-699','',33706,'Reentered','1978-03-12 00:00:00','1978-01-27 00:00:00',90.69,306,310,64.95,'Intermediate' from dual union all
select 8103,'008103','1974-103AM','deb Kosmos-699','',33706,'Reentered','1975-09-30 00:00:00','1975-09-17 00:00:00',91.44,279,412,64.13,'Intermediate' from dual union all
select 8104,'008104','1974-103AN','deb Kosmos-699','',33706,'Reentered','1976-01-29 00:00:00','1976-01-22 00:00:00',89.84,262,272,64.97,'Intermediate' from dual union all
select 8105,'008105','1974-103AP','deb Kosmos-699','',33706,'Reentered','1979-05-03 00:00:00','1979-03-17 00:00:00',90.68,303,313,65,'Intermediate' from dual union all
select 8106,'008106','1974-103AQ','deb Kosmos-699','',33706,'Reentered','1976-04-12 00:00:00','1975-08-17 00:00:00',92.65,399,409,64.99,'Intermediate' from dual union all
select 8107,'008107','1975-062C','deb Kosmos-749','Tselina-O part',36509,'Reentered','1981-08-27 00:00:00','1975-08-20 00:00:00',94.73,464,547,74.11,'Intermediate' from dual union all
select 8108,'008108','1975-075A','Viking Orbiter 1','Viking Orbiter 1',36522,'Deep Space','1975-08-21 00:00:00','1975-08-20 00:00:00',429.09,183,50629,29.3,'Earth Escape' from dual union all
select 8109,'008109','1974-103AR','deb Kosmos-699','',33706,'Reentered','1975-09-24 00:00:00','1975-09-18 00:00:00',90.43,290,302,64.84,'Intermediate' from dual union all
select 811,'000811','1964-030A','OPS 3236','ARGON 21',10316,'Reentered Attached','1965-06-02 00:00:00','1964-07-15 00:00:00',91.62,351,357,115,'Retrograde' from dual union all
select 8110,'008110','1975-064D','deb Meteor-2','',36511,'In Earth orbit','','1993-08-16 00:00:00',102.13,836,882,81.35,'Intermediate' from dual union all
select 8111,'008111','1975-075B','Centaur TC-4','Centaur D-1T TC-4',36522,'Deep Space','1975-08-21 00:00:00','1975-08-20 00:00:00',429.09,183,50629,29.3,'Earth Escape' from dual union all
select 8112,'008112','1974-103AS','deb Kosmos-699','',33706,'Reentered','1978-03-04 00:00:00','1978-02-15 00:00:00',90.26,286,289,64.96,'Intermediate' from dual union all
select 8113,'008113','1974-103AT','deb Kosmos-699','',33706,'Reentered','1978-07-15 00:00:00','1978-06-18 00:00:00',90.16,282,283,64.89,'Intermediate' from dual union all
select 8114,'008114','1974-103AU','deb Kosmos-699','',33706,'Reentered','1976-05-09 00:00:00','1975-08-16 00:00:00',91.35,341,341,64.83,'Intermediate' from dual union all
select 8115,'008115','1974-103AV','deb Kosmos-699','',33706,'Reentered','1976-01-28 00:00:00','1975-08-20 00:00:00',92.89,416,417,64.95,'Intermediate' from dual union all
select 8116,'008116','1974-103AW','deb Kosmos-699','',33706,'Reentered','1977-09-23 00:00:00','1975-08-17 00:00:00',92.85,359,470,65.05,'Intermediate' from dual union all
select 8117,'008117','1974-103AX','deb Kosmos-699','',33706,'Reentered','1976-04-14 00:00:00','1975-08-17 00:00:00',92.58,388,414,65.09,'Intermediate' from dual union all
select 8118,'008118','1974-103AY','deb Kosmos-699','',33706,'Reentered','1978-06-19 00:00:00','1978-05-10 00:00:00',91.7,349,366,64.99,'Intermediate' from dual union all
select 8119,'008119','1974-103AZ','deb Kosmos-699','',33706,'Reentered','1975-11-01 00:00:00','1975-10-30 00:00:00',89.97,273,273,64.98,'Intermediate' from dual union all
select 812,'000812','1964-031A','OPS 4467A','P35-8',10317,'In Earth orbit','','1964-07-19 00:00:00',101.55,829,836,99.82,'Sun-Synch' from dual union all
select 8120,'008120','1974-103BA','deb Kosmos-699','',33706,'Reentered','1978-04-10 00:00:00','1978-01-26 00:00:00',90.81,309,320,64.94,'Intermediate' from dual union all
select 8121,'008121','1974-103BB','deb Kosmos-699','',33706,'Reentered','1977-12-28 00:00:00','1975-08-19 00:00:00',93.27,416,454,65.01,'Intermediate' from dual union all
select 8122,'008122','1974-103BC','deb Kosmos-699','',33706,'Reentered','1978-06-28 00:00:00','1978-05-03 00:00:00',91.38,342,342,64.9,'Intermediate' from dual union all
select 8123,'008123','1972-058EB','deb Delta 89','',27990,'Reentered','1989-01-13 00:00:00','1975-08-19 00:00:00',100.41,634,922,98.31,'Sun-Synch' from dual union all
select 8124,'008124','1972-058EC','deb Delta 89','',27990,'Reentered','1978-11-09 00:00:00','1978-10-01 00:00:00',95.35,508,563,97.33,'Sun-Synch' from dual union all
select 8125,'008125','1972-058ED','deb Delta 89','',27990,'Reentered','1975-10-12 00:00:00','1975-10-01 00:00:00',96.58,487,703,98.17,'Sun-Synch' from dual union all
select 8126,'008126','1972-058EE','deb Delta 89','',27990,'Reentered','1982-06-10 00:00:00','1982-06-07 00:00:00',99.2,624,816,98.21,'Sun-Synch' from dual union all
select 8127,'008127','1975-076A','Kosmos-756','Tselina-D',36523,'Reentered','1992-11-05 00:00:00','1991-03-27 00:00:00',94.26,476,490,81.21,'Intermediate' from dual union all
select 8128,'008128','1975-076B','8A92ME','8A92ME',36523,'Reentered','2001-09-06 00:00:00','1991-03-27 00:00:00',95.54,518,572,81.24,'Intermediate' from dual union all
select 8129,'008129','1975-073D','KDU','KDU',36520,'Reentered','1975-08-31 00:00:00','1975-08-24 00:00:00',89.57,171,336,71.37,'Intermediate' from dual union all
select 813,'000813','1964-031B','OPS 4467B','P35-9',10317,'In Earth orbit','','1964-07-25 00:00:00',101.58,828,839,99.83,'Sun-Synch' from dual union all
select 8130,'008130','1975-073E','Blenda','Sunshade',36520,'Reentered','1975-08-27 00:00:00','1975-08-25 00:00:00',89.39,142,347,71.41,'Intermediate' from dual union all
select 8131,'008131','1975-073F','KDU part','KDU part',36520,'Reentered','1975-08-27 00:00:00','1975-08-26 00:00:00',89.41,171,319,71.33,'Intermediate' from dual union all
select 8132,'008132','1975-077A','Symphonie 2','Symphonie MV2',36524,'In Earth orbit','','1977-01-21 00:00:00',1436.16,35750,35826,.45,'Stationary' from dual union all
select 8133,'008133','1975-077B','Delta 114','Delta SSPU DSV-3P-4',36524,'Reentered','2013-07-14 00:00:00','1990-01-27 00:00:00',104.65,399,1557,25.33,'Intermediate' from dual union all
select 8134,'008134','1975-077C','Star 37E S/N 40038','Star 37E',36524,'In Earth orbit','','1976-03-04 00:00:00',678.99,394,38030,13.17,'GEO Transfer' from dual union all
select 8135,'008135','1975-077D','Delta 114 clamp band','',36524,'Reentered','1976-09-23 00:00:00','1975-11-30 00:00:00',97.32,285,975,27.02,'Intermediate' from dual union all
select 8136,'008136','1974-089E','deb Delta 104','',33692,'In Earth orbit','','1977-12-31 00:00:00',115.97,1270,1729,102.23,'Sun-Synch' from dual union all
select 8137,'008137','1974-089F','deb Delta 104','',33692,'In Earth orbit','','1993-07-29 00:00:00',113.4,1309,1456,101.36,'Sun-Synch' from dual union all
select 8138,'008138','1974-089G','deb Delta 104','',33692,'Reentered','1992-11-14 00:00:00','1992-11-14 00:00:00',91.2,311,355,101.5,'Retrograde' from dual union all
select 8139,'008139','1974-089H','deb Delta 104','',33692,'In Earth orbit','','1993-07-26 00:00:00',114.15,1377,1457,101.51,'Sun-Synch' from dual union all
select 814,'000814','1964-032A','OPS 3754','CORONA J-7',10318,'Reentered Attached','1964-07-16 00:00:00','1964-06-30 00:00:00',90.62,180,430,85.02,'Polar' from dual union all
select 8140,'008140','1974-089J','deb Delta 104','',33692,'In Earth orbit','','1993-07-26 00:00:00',115.52,1445,1512,102.16,'Sun-Synch' from dual union all
select 8141,'008141','1974-089K','deb Delta 104','',33692,'In Earth orbit','','1975-08-27 00:00:00',116.6,1455,1601,101.45,'Sun-Synch' from dual union all
select 8142,'008142','1974-089L','deb Delta 104','',33692,'In Earth orbit','','1975-08-27 00:00:00',122.17,1420,2136,101.28,'Medium' from dual union all
select 8143,'008143','1974-089M','deb Delta 104','',33692,'In Earth orbit','','1993-08-08 00:00:00',114.28,1268,1578,101.65,'Sun-Synch' from dual union all
select 8144,'008144','1974-089N','deb Delta 104','',33692,'In Earth orbit','','1975-08-27 00:00:00',115.04,1345,1570,101.45,'Sun-Synch' from dual union all
select 8145,'008145','1974-089P','deb Delta 104','',33692,'In Earth orbit','','1993-08-07 00:00:00',114.36,1406,1447,101.08,'Sun-Synch' from dual union all
select 8146,'008146','1974-089Q','deb Delta 104','',33692,'In Earth orbit','','1993-08-06 00:00:00',114.87,1304,1595,102.02,'Sun-Synch' from dual union all
select 8147,'008147','1975-078A','Kosmos-757','Zenit-4MK',36525,'Landed','1975-09-09 00:00:00','1975-08-29 00:00:00',89.24,166,308,62.83,'Intermediate' from dual union all
select 8148,'008148','1975-078B','11A57I','11A57I',36525,'Reentered','1975-09-03 00:00:00','1975-08-29 00:00:00',89.12,172,289,62.83,'Intermediate' from dual union all
select 8149,'008149','1975-078C','Sensor cover','Zenit-4MK sensor cover',36525,'Reentered','1975-09-02 00:00:00','1975-08-27 00:00:00',89.01,172,280,62.8,'Intermediate' from dual union all
select 815,'000815','1964-031C','Agena D 2304','Agena D',10317,'In Earth orbit','','1990-02-18 00:00:00',101.2,806,825,99.82,'Sun-Synch' from dual union all
select 8150,'008150','1975-078D','Sensor cover','Zenit-4MK sensor cover',36525,'Reentered','1975-09-02 00:00:00','1975-08-30 00:00:00',88.77,172,256,62.82,'Intermediate' from dual union all
select 8151,'008151','1974-089R','deb Delta 104','',33692,'In Earth orbit','','1993-08-01 00:00:00',112.57,1083,1607,101.79,'Sun-Synch' from dual union all
select 8152,'008152','1974-089S','deb Delta 104','',33692,'In Earth orbit','','1993-08-04 00:00:00',114.38,1382,1473,101.81,'Sun-Synch' from dual union all
select 8153,'008153','1974-089T','deb Delta 104','',33692,'In Earth orbit','','1993-07-30 00:00:00',119.04,1413,1863,100.95,'Sun-Synch' from dual union all
select 8154,'008154','1974-089U','deb Delta 104','',33692,'In Earth orbit','','1993-07-26 00:00:00',114.99,1450,1460,102.04,'Sun-Synch' from dual union all
select 8155,'008155','1974-089V','deb Delta 104','',33692,'Reentered','2002-04-25 00:00:00','1975-08-27 00:00:00',114.17,1295,1540,101.78,'Sun-Synch' from dual union all
select 8156,'008156','1974-089W','deb Delta 104','',33692,'In Earth orbit','','1975-08-27 00:00:00',114.01,1343,1478,101.9,'Sun-Synch' from dual union all
select 8157,'008157','1974-089X','deb Delta 104','',33692,'In Earth orbit','','1993-08-04 00:00:00',115.5,1373,1584,102.09,'Sun-Synch' from dual union all
select 8158,'008158','1974-089Y','deb Delta 104','',33692,'In Earth orbit','','1993-08-03 00:00:00',114.24,1377,1464,101.15,'Sun-Synch' from dual union all
select 8159,'008159','1974-089Z','deb Delta 104','',33692,'In Earth orbit','','1975-08-26 00:00:00',116.74,1280,1788,101.97,'Sun-Synch' from dual union all
select 816,'000816','1964-033A','Kosmos-33','Zenit-2 No. 20',10319,'Landed','1964-07-01 00:00:00','1964-07-02 00:00:00',89.26,209,267,65.03,'Intermediate' from dual union all
select 8160,'008160','1974-089AA','deb Delta 104','',33692,'In Earth orbit','','1993-08-01 00:00:00',114.03,1302,1520,101.98,'Sun-Synch' from dual union all
select 8161,'008161','1974-089AB','deb Delta 104','',33692,'In Earth orbit','','1993-08-08 00:00:00',120.51,1404,2004,101.18,'Medium' from dual union all
select 8162,'008162','1974-089AC','deb Delta 104','',33692,'In Earth orbit','','1993-08-07 00:00:00',117.25,1431,1684,100.85,'Sun-Synch' from dual union all
select 8163,'008163','1974-089AD','deb Delta 104','',33692,'In Earth orbit','','1993-08-04 00:00:00',117.64,1307,1843,102.07,'Sun-Synch' from dual union all
select 8164,'008164','1974-089AE','deb Delta 104','',33692,'In Earth orbit','','1993-07-30 00:00:00',117.18,1458,1651,101.58,'Sun-Synch' from dual union all
select 8165,'008165','1974-089AF','deb Delta 104','',33692,'In Earth orbit','','1993-08-04 00:00:00',115.25,1447,1487,102.02,'Sun-Synch' from dual union all
select 8166,'008166','1974-089AG','deb Delta 104','',33692,'In Earth orbit','','1991-06-05 00:00:00',652.91,602,36503,27.75,'GEO Transfer' from dual union all
select 8167,'008167','1974-089AH','deb Delta 104','',33692,'In Earth orbit','','1975-08-26 00:00:00',114.77,1247,1642,101.51,'Sun-Synch' from dual union all
select 8168,'008168','1974-089AJ','deb Delta 104','',33692,'In Earth orbit','','1975-08-27 00:00:00',115,1432,1478,101.48,'Sun-Synch' from dual union all
select 8169,'008169','1974-089AK','deb Delta 104','',33692,'In Earth orbit','','1993-08-08 00:00:00',113.25,1227,1525,102.53,'Sun-Synch' from dual union all
select 817,'000817','1964-033B','8A92E','8A92E',10319,'Reentered','1964-07-10 00:00:00','1964-07-01 00:00:00',89.13,208,255,64.95,'Intermediate' from dual union all
select 8170,'008170','1974-089AL','deb Delta 104','',33692,'In Earth orbit','','1993-08-09 00:00:00',110.22,1143,1331,101.78,'Sun-Synch' from dual union all
select 8171,'008171','1974-089AM','deb Delta 104','',33692,'Reentered','1982-05-08 00:00:00','1982-05-04 00:00:00',92.21,361,405,101.84,'Retrograde' from dual union all
select 8172,'008172','1974-089AN','deb Delta 104','',33692,'In Earth orbit','','1993-08-03 00:00:00',114.76,1403,1485,101.96,'Sun-Synch' from dual union all
select 8173,'008173','1974-089AP','deb Delta 104','',33692,'In Earth orbit','','1993-08-01 00:00:00',115.15,1370,1555,102.02,'Sun-Synch' from dual union all
select 8174,'008174','1974-089AQ','deb Delta 104','',33692,'In Earth orbit','','1993-08-02 00:00:00',114.99,1405,1505,101.84,'Sun-Synch' from dual union all
select 8175,'008175','1974-089AR','deb Delta 104','',33692,'In Earth orbit','','1993-08-08 00:00:00',117.15,1378,1728,101.64,'Sun-Synch' from dual union all
select 8176,'008176','1974-089AS','deb Delta 104','',33692,'Reentered','1989-10-17 00:00:00','1989-10-15 00:00:00',92.74,354,464,101.89,'Retrograde' from dual union all
select 8177,'008177','1974-089AT','deb Delta 104','',33692,'In Earth orbit','','1993-08-06 00:00:00',119.55,1450,1872,101.11,'Sun-Synch' from dual union all
select 8178,'008178','1974-089AU','deb Delta 104','',33692,'In Earth orbit','','1993-08-07 00:00:00',117.02,1370,1724,102.01,'Sun-Synch' from dual union all
select 8179,'008179','1974-089AV','deb Delta 104','',33692,'In Earth orbit','','1993-08-02 00:00:00',116.76,1456,1615,101.46,'Sun-Synch' from dual union all
select 818,'000818','1964-033C','deb Kosmos-33','',10319,'Reentered','1964-06-24 00:00:00','1964-06-23 00:00:00',88.96,185,262,65.1,'Intermediate' from dual union all
select 8180,'008180','1974-089AW','deb Delta 104','',33692,'In Earth orbit','','1993-08-05 00:00:00',116.27,1434,1591,100.86,'Sun-Synch' from dual union all
select 8181,'008181','1974-089AX','deb Delta 104','',33692,'In Earth orbit','','1993-08-05 00:00:00',116.19,1453,1566,101.52,'Sun-Synch' from dual union all
select 8182,'008182','1974-089AY','deb Delta 104','',33692,'In Earth orbit','','1975-08-26 00:00:00',116.75,1436,1634,100.52,'Sun-Synch' from dual union all
select 8183,'008183','1974-089AZ','deb Delta 104','',33692,'In Earth orbit','','1993-08-04 00:00:00',116.87,1457,1624,100.83,'Sun-Synch' from dual union all
select 8184,'008184','1974-089BA','deb Delta 104','',33692,'In Earth orbit','','1993-08-01 00:00:00',115.45,1426,1526,101.99,'Sun-Synch' from dual union all
select 8185,'008185','1974-089BB','deb Delta 104','',33692,'In Earth orbit','','1993-08-06 00:00:00',116.16,1453,1563,101.54,'Sun-Synch' from dual union all
select 8186,'008186','1974-089BC','deb Delta 104','',33692,'Reentered','1978-12-27 00:00:00','1978-11-01 00:00:00',98.73,408,987,101.87,'Retrograde' from dual;
insert into SATELLITE
select 8187,'008187','1975-079A','Molniya-1','Molniya-1',36526,'Reentered','1985-11-19 00:00:00','1975-10-18 00:00:00',717.78,613,39742,62.91,'Molniya' from dual union all
select 8188,'008188','1975-079B','11S510','11S510',36526,'Reentered','1975-11-10 00:00:00','1975-10-10 00:00:00',91.17,213,451,62.81,'Intermediate' from dual union all
select 8189,'008189','1975-079C','BOZ','BOZ',36526,'Reentered','1975-11-01 00:00:00','1975-10-04 00:00:00',91.3,209,467,62.87,'Intermediate' from dual union all
select 819,'000819','1964-033D','deb Kosmos-33','',10319,'Reentered','1964-06-24 00:00:00','1964-06-23 00:00:00',89.18,196,271,64.95,'Intermediate' from dual union all
select 8190,'008190','1975-079D','deb Blok-I','',36526,'Reentered','1975-10-12 00:00:00','1975-09-19 00:00:00',91.99,213,532,62.82,'Intermediate' from dual union all
select 8191,'008191','1975-080A','Kosmos-758','Yantar''-2K No. 3',36527,'Reentered','1975-09-25 00:00:00','1975-09-05 00:00:00',89.71,172,349,67.07,'Intermediate' from dual union all
select 8192,'008192','1975-080B','11S510','11S510',36527,'Reentered','1975-09-10 00:00:00','1975-09-07 00:00:00',89.09,168,291,67.14,'Intermediate' from dual union all
select 8193,'008193','1975-078E','Blenda','Sunshade',36525,'Reentered','1975-09-08 00:00:00','1975-09-06 00:00:00',89.07,154,303,62.82,'Intermediate' from dual union all
select 8194,'008194','1975-078F','KDU','KDU',36525,'Reentered','1975-09-11 00:00:00','1975-09-07 00:00:00',89.15,165,299,62.84,'Intermediate' from dual union all
select 8195,'008195','1975-081A','Molniya-2','Molniya-2',36528,'In Earth orbit','','1975-10-12 00:00:00',717.64,448,39900,62.88,'Molniya' from dual union all
select 8196,'008196','1975-081B','11S510','11S510',36528,'Reentered','1975-10-09 00:00:00','1975-09-18 00:00:00',90.41,209,381,62.86,'Intermediate' from dual union all
select 8197,'008197','1975-082A','Kiku-1','ETS 1',36529,'In Earth orbit','','1992-02-14 00:00:00',105.93,974,1102,47,'Intermediate' from dual union all
select 8198,'008198','1975-081C','BOZ','BOZ',36528,'Reentered','1975-10-18 00:00:00','1975-09-16 00:00:00',90.95,210,433,62.85,'Intermediate' from dual union all
select 8199,'008199','1975-083A','Viking Orbiter 2','Viking Orbiter 2',36530,'Deep Space','1975-09-10 00:00:00','1975-09-09 00:00:00',715.64,178,65940,29.3,'Earth Escape' from dual union all
select 82,'000082','61 Delta 2','Altair X-248-A5S SV-118','X-248',7263,'In Earth orbit','','1989-03-31 00:00:00',117.94,633,2544,38.9,'Intermediate' from dual union all
select 820,'000820','1964-030B','Starflash cover','deb CORONA 9065A',10316,'Reentered','1964-08-07 00:00:00','1964-06-28 00:00:00',91.92,352,386,114.97,'Retrograde' from dual union all
select 8200,'008200','1975-080C','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.32,171,311,67.2,'Intermediate' from dual union all
select 8201,'008201','1975-080D','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',90.14,173,389,67.1,'Intermediate' from dual union all
select 8202,'008202','1975-080E','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.45,173,321,67.3,'Intermediate' from dual union all
select 8203,'008203','1975-080F','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.89,171,367,66.9,'Intermediate' from dual union all
select 8204,'008204','1975-080G','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.07,173,284,66.9,'Intermediate' from dual union all
select 8205,'008205','1975-080H','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.45,171,324,67,'Intermediate' from dual union all
select 8206,'008206','1975-080J','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.06,166,290,67.1,'Intermediate' from dual union all
select 8207,'008207','1975-080K','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',87.9,159,183,66.9,'Intermediate' from dual union all
select 8208,'008208','1975-080L','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.11,170,291,67.2,'Intermediate' from dual union all
select 8209,'008209','1975-080M','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.92,165,278,67.1,'Intermediate' from dual union all
select 821,'000821','1964-032B','Fairing','Fairing',10318,'Reentered','1964-07-02 00:00:00','1964-06-29 00:00:00',90.18,161,406,85.18,'Polar' from dual union all
select 8210,'008210','1975-080N','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.1,163,297,67,'Intermediate' from dual union all
select 8211,'008211','1975-080P','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',90.07,173,383,67.2,'Intermediate' from dual union all
select 8212,'008212','1975-080Q','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.26,170,306,66.7,'Intermediate' from dual union all
select 8213,'008213','1975-080R','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.33,171,312,66.9,'Intermediate' from dual union all
select 8214,'008214','1975-080S','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.79,157,273,66.9,'Intermediate' from dual union all
select 8215,'008215','1975-080T','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.1,171,289,67,'Intermediate' from dual union all
select 8216,'008216','1975-080U','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.29,173,207,67,'Intermediate' from dual union all
select 8217,'008217','1975-080V','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.21,172,299,67.3,'Intermediate' from dual union all
select 8218,'008218','1975-080W','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.72,161,262,67.4,'Intermediate' from dual union all
select 8219,'008219','1975-080X','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.92,173,368,67.2,'Intermediate' from dual union all
select 822,'000822','1964-034A','Kosmos-34','Zenit-4 No. 3',10320,'Landed','1964-07-09 00:00:00','1964-07-03 00:00:00',89.97,201,345,64.89,'Intermediate' from dual union all
select 8220,'008220','1975-080Y','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.19,173,296,67,'Intermediate' from dual union all
select 8221,'008221','1975-080Z','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.1,138,223,67.3,'Intermediate' from dual union all
select 8222,'008222','1975-080AA','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.36,170,316,67.3,'Intermediate' from dual union all
select 8223,'008223','1975-080AB','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.65,172,343,66.9,'Intermediate' from dual union all
select 8224,'008224','1975-080AC','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.26,173,303,67.1,'Intermediate' from dual union all
select 8225,'008225','1975-080AD','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.89,173,267,67.1,'Intermediate' from dual union all
select 8226,'008226','1975-080AE','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.81,173,358,67,'Intermediate' from dual union all
select 8227,'008227','1975-080AF','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',90.11,171,389,66.8,'Intermediate' from dual union all
select 8228,'008228','1975-080AG','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.72,173,250,67.3,'Intermediate' from dual union all
select 8229,'008229','1975-080AH','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.95,173,370,67.3,'Intermediate' from dual union all
select 823,'000823','1964-034B','11A57I','11A57I',10320,'Reentered','1964-07-15 00:00:00','1964-07-05 00:00:00',89.48,180,318,64.89,'Intermediate' from dual union all
select 8230,'008230','1975-080AJ','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.64,173,242,67.1,'Intermediate' from dual union all
select 8231,'008231','1975-080AK','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.49,168,232,66.9,'Intermediate' from dual union all
select 8232,'008232','1975-080AL','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.68,170,348,67.1,'Intermediate' from dual union all
select 8233,'008233','1975-080AM','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.06,172,284,67.3,'Intermediate' from dual union all
select 8234,'008234','1975-080AN','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.83,173,261,67.3,'Intermediate' from dual union all
select 8235,'008235','1975-080AP','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.5,171,329,67.1,'Intermediate' from dual union all
select 8236,'008236','1975-080AQ','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.99,173,375,67.1,'Intermediate' from dual union all
select 8237,'008237','1975-080AR','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.57,173,334,66.9,'Intermediate' from dual union all
select 8238,'008238','1975-080AS','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.32,170,312,66.9,'Intermediate' from dual union all
select 8239,'008239','1975-080AT','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.97,172,275,66.7,'Intermediate' from dual union all
select 824,'000824','1964-035A','OPS 3395','Project 698BK Group 2-D',10321,'Reentered Attached','1969-08-07 00:00:00','1964-08-07 00:00:00',94.86,494,530,82.08,'Intermediate' from dual union all
select 8240,'008240','1975-080AU','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.9,172,367,67.1,'Intermediate' from dual union all
select 8241,'008241','1975-080AV','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.26,171,305,66.9,'Intermediate' from dual union all
select 8242,'008242','1975-080AW','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.96,169,375,67.3,'Intermediate' from dual union all
select 8243,'008243','1975-080AX','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.43,166,228,67.3,'Intermediate' from dual union all
select 8244,'008244','1975-080AY','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.34,173,311,67.1,'Intermediate' from dual union all
select 8245,'008245','1975-080AZ','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.72,173,349,67.1,'Intermediate' from dual union all
select 8246,'008246','1975-080BA','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.83,171,361,67.2,'Intermediate' from dual union all
select 8247,'008247','1975-080BB','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.2,167,303,67.1,'Intermediate' from dual union all
select 8248,'008248','1975-080BC','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.9,173,267,67.2,'Intermediate' from dual union all
select 8249,'008249','1975-080BD','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',90.09,173,384,67.3,'Intermediate' from dual union all
select 825,'000825','1964-036A','OPS 3684','GAMBIT SV 959',10322,'Deorbited','1964-07-08 00:00:00','1964-07-07 00:00:00',89.17,121,346,92.89,'Polar' from dual union all
select 8250,'008250','1975-080BE','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.14,172,291,67.3,'Intermediate' from dual union all
select 8251,'008251','1975-080BF','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.4,173,317,67.4,'Intermediate' from dual union all
select 8252,'008252','1975-080BG','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.93,173,369,66.9,'Intermediate' from dual union all
select 8253,'008253','1975-080BH','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',90.17,174,392,67,'Intermediate' from dual union all
select 8254,'008254','1975-080BJ','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.06,173,282,67.2,'Intermediate' from dual union all
select 8255,'008255','1975-080BK','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.68,166,253,67.5,'Intermediate' from dual union all
select 8256,'008256','1975-080BL','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.49,166,333,67.3,'Intermediate' from dual union all
select 8257,'008257','1975-080BM','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.85,166,270,67.1,'Intermediate' from dual union all
select 8258,'008258','1975-080BN','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.55,171,334,67.2,'Intermediate' from dual union all
select 8259,'008259','1975-080BP','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',90.03,173,379,67.1,'Intermediate' from dual union all
select 826,'000826','1964-036B','NOAH''S ARK','P-11 No. 4301',10322,'Reentered','1965-01-03 00:00:00','1964-08-05 00:00:00',91.29,289,387,92.99,'Polar' from dual union all
select 8260,'008260','1975-080BQ','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.07,165,292,67.1,'Intermediate' from dual union all
select 8261,'008261','1975-080BR','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',90.2,171,398,67.4,'Intermediate' from dual union all
select 8262,'008262','1975-080BS','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.4,173,317,66.9,'Intermediate' from dual union all
select 8263,'008263','1975-080BT','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',90.01,169,381,67,'Intermediate' from dual union all
select 8264,'008264','1975-080BU','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',90.12,170,391,67.4,'Intermediate' from dual union all
select 8265,'008265','1975-080BV','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.27,173,303,67.3,'Intermediate' from dual union all
select 8266,'008266','1975-080BW','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.49,170,230,67.3,'Intermediate' from dual union all
select 8267,'008267','1975-080BX','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.13,173,290,67.3,'Intermediate' from dual union all
select 8268,'008268','1975-080BY','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.17,169,298,67.1,'Intermediate' from dual union all
select 8269,'008269','1975-080BZ','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',89.78,172,356,67.1,'Intermediate' from dual union all
select 827,'000827','1964-036C','OPS 4923 sensor cover','',10322,'Reentered','1964-07-19 00:00:00','1964-07-07 00:00:00',91.37,304,380,92.84,'Polar' from dual union all
select 8270,'008270','1975-080CA','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',90.14,172,391,67.1,'Intermediate' from dual union all
select 8271,'008271','1975-080CB','deb Kosmos-758','',36527,'Reentered','1975-09-09 00:00:00','1975-09-07 00:00:00',88.86,169,268,66.9,'Intermediate' from dual union all
select 8272,'008272','1975-083B','Centaur TC-3','Centaur D-1T TC-3',36530,'Deep Space','1975-09-10 00:00:00','1975-09-09 00:00:00',715.64,178,65940,29.3,'Earth Escape' from dual union all
select 8273,'008273','1975-080CC','deb Kosmos-758','',36527,'Reentered','1975-09-17 00:00:00','1975-09-15 00:00:00',88.1,165,196,66.36,'Intermediate' from dual union all
select 8274,'008274','1975-079E','Blok-L','Blok-L',36526,'Reentered','1986-07-08 00:00:00','1975-10-07 00:00:00',735.96,715,40533,62.85,'Molniya' from dual union all
select 8275,'008275','1975-084A','Kosmos-759','Zenit-4MT',36531,'Landed','1975-09-23 00:00:00','1975-09-14 00:00:00',89.53,228,275,62.8,'Intermediate' from dual union all
select 8276,'008276','1975-084B','11S510','11S510',36531,'Reentered','1975-09-25 00:00:00','1975-09-14 00:00:00',89.37,237,249,62.79,'Intermediate' from dual;
insert into SATELLITE
select 8277,'008277','1975-084C','Sensor cover','Zenit-4MT sensor cover',36531,'Reentered','1975-09-14 00:00:00','1975-09-12 00:00:00',89.28,216,262,62.8,'Intermediate' from dual union all
select 8278,'008278','1975-084D','Sensor cover','Zenit-4MT sensor cover',36531,'Reentered','1975-09-17 00:00:00','1975-09-12 00:00:00',89.32,218,264,62.77,'Intermediate' from dual union all
select 8279,'008279','1975-084E','Sensor cover','Zenit-4MT sensor cover',36531,'Reentered','1975-09-16 00:00:00','1975-09-13 00:00:00',89.55,234,271,62.78,'Intermediate' from dual union all
select 828,'000828','1964-037A','OPS 3491','CORONA J-10',10323,'Reentered Attached','1964-08-06 00:00:00','1964-07-19 00:00:00',90.75,173,450,84.97,'Intermediate' from dual union all
select 8280,'008280','1975-080CD','deb Kosmos-758','',36527,'Reentered','1975-10-03 00:00:00','1975-10-02 00:00:00',87.83,152,183,67.47,'Intermediate' from dual union all
select 8281,'008281','1975-085A','Kosmos-760','Zenit-4MK',36532,'Landed','1975-09-30 00:00:00','1975-09-18 00:00:00',89.36,170,316,64.98,'Intermediate' from dual union all
select 8282,'008282','1975-085B','11A57I','11A57I',36532,'Reentered','1975-09-21 00:00:00','1975-09-19 00:00:00',88.95,167,279,64.97,'Intermediate' from dual union all
select 8283,'008283','1975-080CE','deb Kosmos-758','',36527,'Reentered','1975-09-26 00:00:00','1975-09-20 00:00:00',90.54,167,434,67.51,'Intermediate' from dual union all
select 8284,'008284','1975-085C','Sensor cover','Zenit-4MK sensor cover',36532,'Reentered','1975-09-18 00:00:00','1975-09-16 00:00:00',88.8,167,264,65,'Intermediate' from dual union all
select 8285,'008285','1975-086A','Kosmos-761','Strela-1M',36533,'In Earth orbit','','1992-02-14 00:00:00',114.67,1400,1481,74.01,'Intermediate' from dual union all
select 8286,'008286','1975-086B','Kosmos-762','Strela-1M',36533,'In Earth orbit','','1992-02-14 00:00:00',115.12,1438,1483,74.01,'Intermediate' from dual union all
select 8287,'008287','1975-086C','Kosmos-763','Strela-1M',36533,'In Earth orbit','','1992-02-14 00:00:00',115.8,1474,1510,74.01,'Intermediate' from dual union all
select 8288,'008288','1975-086D','Kosmos-764','Strela-1M',36533,'In Earth orbit','','1992-02-14 00:00:00',116.02,1479,1525,74.01,'Intermediate' from dual union all
select 8289,'008289','1975-086E','Kosmos-765','Strela-1M',36533,'In Earth orbit','','1992-02-14 00:00:00',116.29,1478,1550,74.01,'Intermediate' from dual union all
select 829,'000829','1964-038A','Elektron-3','2D No. 3',10324,'In Earth orbit','','1964-08-11 00:00:00',168.15,401,7026,60.8,'Medium' from dual union all
select 8290,'008290','1975-086F','Kosmos-766','Strela-1M',36533,'In Earth orbit','','1992-02-16 00:00:00',114.9,1419,1482,74.01,'Intermediate' from dual union all
select 8291,'008291','1975-086G','Kosmos-767','Strela-1M',36533,'In Earth orbit','','1992-02-16 00:00:00',115.34,1455,1487,74.01,'Intermediate' from dual union all
select 8292,'008292','1975-086H','Kosmos-768','Strela-1M',36533,'In Earth orbit','','1992-02-14 00:00:00',115.56,1470,1492,74.01,'Intermediate' from dual union all
select 8293,'008293','1975-087A','Meteor','Meteor-M  No. 28',36534,'In Earth orbit','','1992-02-15 00:00:00',102.08,802,913,81.26,'Intermediate' from dual union all
select 8294,'008294','1975-087B','8A92ME','8A92ME',36534,'In Earth orbit','','1992-02-14 00:00:00',102.3,824,911,81.27,'Intermediate' from dual union all
select 8295,'008295','1975-086J','S3M','S3M',36533,'In Earth orbit','','1992-02-14 00:00:00',117.81,1482,1682,73.99,'Intermediate' from dual union all
select 8296,'008296','1974-089BD','deb Delta 104','',33692,'In Earth orbit','','1975-09-10 00:00:00',117.96,1441,1737,102.23,'Sun-Synch' from dual union all
select 8297,'008297','1974-089BE','deb Delta 104','',33692,'Reentered','1991-01-10 00:00:00','1991-01-10 00:00:00',90.33,281,301,101.77,'Retrograde' from dual union all
select 8298,'008298','1974-089BF','deb Delta 104','',33692,'Reentered','2001-06-15 00:00:00','1993-08-09 00:00:00',107.52,1081,1142,100.69,'Sun-Synch' from dual union all
select 8299,'008299','1974-089BG','deb Delta 104','',33692,'In Earth orbit','','1975-09-17 00:00:00',112.3,1206,1458,102.17,'Sun-Synch' from dual union all
select 83,'000083','61 Epsilon 1','Discoverer 20','ARGON 1',7264,'Reentered Attached','1962-07-28 00:00:00','1961-03-20 00:00:00',95.16,283,769,80.91,'Intermediate' from dual union all
select 830,'000830','1964-038B','Elektron-4','2D No. 4',10324,'Reentered','1983-10-12 00:00:00','1964-08-14 00:00:00',1313.82,452,66264,60.63,'Highly Ellip' from dual union all
select 8300,'008300','1974-089BH','deb Delta 104','',33692,'In Earth orbit','','1993-08-06 00:00:00',120.4,1420,1978,100.91,'Medium' from dual union all
select 8301,'008301','1974-089BJ','deb Delta 104','',33692,'In Earth orbit','','1993-08-12 00:00:00',107.5,921,1300,103.1,'Sun-Synch' from dual union all
select 8302,'008302','1974-089BK','deb Delta 104','',33692,'Reentered','2011-10-24 00:00:00','1993-07-31 00:00:00',105.49,878,1156,101.2,'Sun-Synch' from dual union all
select 8303,'008303','1974-089BL','deb Delta 104','',33692,'In Earth orbit','','1993-08-06 00:00:00',115.29,1442,1496,101.34,'Sun-Synch' from dual union all
select 8304,'008304','1974-089BM','deb Delta 104','',33692,'Reentered','1989-11-10 00:00:00','1989-11-09 00:00:00',94.2,457,502,101.9,'Retrograde' from dual union all
select 8305,'008305','1974-089BN','deb Delta 104','',33692,'In Earth orbit','','1993-08-06 00:00:00',116.65,1455,1606,101.58,'Sun-Synch' from dual union all
select 8306,'008306','1974-089BP','deb Delta 104','',33692,'In Earth orbit','','1993-08-02 00:00:00',115.63,1428,1540,101.92,'Sun-Synch' from dual union all
select 8307,'008307','1974-089BQ','deb Delta 104','',33692,'In Earth orbit','','1993-08-01 00:00:00',122.64,1430,2168,101.16,'Medium' from dual union all
select 8308,'008308','1974-089BR','deb Delta 104','',33692,'In Earth orbit','','1993-08-01 00:00:00',115.64,1435,1533,101.97,'Sun-Synch' from dual union all
select 8309,'008309','1974-089BS','deb Delta 104','',33692,'Reentered','2002-01-17 00:00:00','1993-08-22 00:00:00',106.13,648,1447,101.96,'Sun-Synch' from dual union all
select 831,'000831','1964-038C','Elektron-3 motor','',10324,'In Earth orbit','','1964-08-22 00:00:00',168.59,402,7060,60.81,'Medium' from dual union all
select 8310,'008310','1974-089BT','deb Delta 104','',33692,'In Earth orbit','','1993-07-30 00:00:00',116.12,1455,1557,101.82,'Sun-Synch' from dual union all
select 8311,'008311','1974-089BU','deb Delta 104','',33692,'In Earth orbit','','1993-07-26 00:00:00',115.64,1455,1513,102.19,'Sun-Synch' from dual union all
select 8312,'008312','1972-090C','Star 37C S/N 40019','Star 37C S/N 40019',28022,'Reentered','1981-11-06 00:00:00','1972-11-10 00:00:00',644.17,189,36470,26.9,'GEO Transfer' from dual union all
select 8313,'008313','1972-058EF','deb Delta 89','',27990,'Reentered','1982-03-25 00:00:00','1982-03-24 00:00:00',90.14,266,297,98.13,'Sun-Synch' from dual union all
select 8314,'008314','1972-058EG','deb Delta 89','',27990,'In Earth orbit','','1993-08-03 00:00:00',99.03,614,811,98.54,'Sun-Synch' from dual union all
select 8315,'008315','1972-058EH','deb Delta 89','',27990,'Reentered','1979-11-06 00:00:00','1979-10-16 00:00:00',95.08,482,563,98.11,'Sun-Synch' from dual union all
select 8316,'008316','1972-058EJ','deb Delta 89','',27990,'Reentered','1981-05-12 00:00:00','1975-09-17 00:00:00',103.28,489,1339,97.92,'Sun-Synch' from dual union all
select 8317,'008317','1972-058EK','deb Delta 89','',27990,'Reentered','1981-03-08 00:00:00','1981-03-05 00:00:00',91.54,341,360,98.11,'Sun-Synch' from dual union all
select 8318,'008318','1972-058EL','deb Delta 89','',27990,'Reentered','1989-07-10 00:00:00','1989-07-10 00:00:00',88.97,209,239,97.98,'Sun-Synch' from dual union all
select 8319,'008319','1972-058EM','deb Delta 89','',27990,'In Earth orbit','','1993-08-02 00:00:00',99.84,637,865,98.77,'Sun-Synch' from dual union all
select 832,'000832','1964-038D','8K72E','8K72KE',10324,'Reentered','1983-08-24 00:00:00','1964-08-18 00:00:00',1341.29,441,67379,60.79,'Highly Ellip' from dual union all
select 8320,'008320','1972-058EN','deb Delta 89','',27990,'Reentered','1982-04-15 00:00:00','1975-09-13 00:00:00',100.42,638,918,98.31,'Sun-Synch' from dual union all
select 8321,'008321','1972-058EP','deb Delta 89','',27990,'Reentered','1979-04-14 00:00:00','1979-02-19 00:00:00',95.27,496,568,98.2,'Sun-Synch' from dual union all
select 8322,'008322','1975-088A','Kosmos-769','Zenit-2M',36535,'Landed','1975-10-05 00:00:00','1975-09-25 00:00:00',89.56,202,304,72.83,'Intermediate' from dual union all
select 8323,'008323','1975-088B','11A57I','11A57I',36535,'Reentered','1975-10-02 00:00:00','1975-09-25 00:00:00',89.38,198,290,72.84,'Intermediate' from dual union all
select 8324,'008324','1975-084F','KB electropitaniya','KBE',36531,'Reentered','1975-09-26 00:00:00','1975-09-24 00:00:00',89.32,220,261,62.81,'Intermediate' from dual union all
select 8325,'008325','1975-089A','Kosmos-770','Geoid',36536,'In Earth orbit','','1992-02-14 00:00:00',109.11,1165,1207,82.96,'Intermediate' from dual union all
select 8326,'008326','1975-089B','S3M','S3M',36536,'In Earth orbit','','1992-02-16 00:00:00',108.98,1162,1197,82.96,'Intermediate' from dual union all
select 8327,'008327','1975-090A','Kosmos-771','Fram',36537,'Landed','1975-10-08 00:00:00','1975-09-27 00:00:00',88.82,215,218,81.34,'Intermediate' from dual union all
select 8328,'008328','1975-090B','11S510','11S510',36537,'Reentered','1975-09-29 00:00:00','1975-09-27 00:00:00',88.49,197,203,81.32,'Intermediate' from dual union all
select 8329,'008329','1975-084G','KBE part','KBE part',36531,'Reentered','1975-09-26 00:00:00','1975-09-25 00:00:00',89.02,223,230,62.8,'Intermediate' from dual union all
select 833,'000833','1964-039A','Kosmos-35','Zenit-2 No. 21',10325,'Landed','1964-07-23 00:00:00','1964-07-18 00:00:00',89.25,212,263,51.26,'Intermediate' from dual union all
select 8330,'008330','1975-091A','INTELSAT IVA F1','INTELSAT IVA F1',36538,'In Earth orbit','','1977-01-17 00:00:00',1436.09,35785,35788,.05,'Stationary' from dual union all
select 8331,'008331','1975-091B','Centaur D-1AR AC-36','Centaur D-1AR AC-36',36538,'In Earth orbit','','1976-04-10 00:00:00',656.55,470,36820,21.6,'GEO Transfer' from dual union all
select 8332,'008332','1975-092A','Aura','D-2B',36539,'Reentered','1982-09-30 00:00:00','1975-10-27 00:00:00',96.82,501,712,37.16,'Intermediate' from dual union all
select 8333,'008333','1975-092B','P.068','P068',36539,'Reentered','1982-03-30 00:00:00','1975-10-28 00:00:00',96.9,501,720,37.16,'Intermediate' from dual union all
select 8334,'008334','1975-085D','KDU','KDU',36532,'Reentered','1975-10-05 00:00:00','1975-09-28 00:00:00',89.48,167,331,64.97,'Intermediate' from dual union all
select 8335,'008335','1975-085E','Blenda','Sunshade',36532,'Reentered','1975-09-30 00:00:00','1975-09-28 00:00:00',89.25,164,311,64.98,'Intermediate' from dual union all
select 8336,'008336','1975-092C','Aura despin weight','',36539,'Reentered','1978-10-17 00:00:00','1975-11-05 00:00:00',96.7,501,701,37.18,'Intermediate' from dual union all
select 8337,'008337','1975-092D','Aura despin weight','',36539,'Reentered','1978-05-16 00:00:00','1975-11-04 00:00:00',97.22,525,727,37.07,'Intermediate' from dual union all
select 8338,'008338','1975-093A','Kosmos-772','Soyuz 7K-S No. 2L',36540,'Landed','1975-10-02 00:00:00','1975-10-01 00:00:00',89.74,196,328,51.76,'Intermediate' from dual union all
select 8339,'008339','1975-093B','11S510','11S510',36540,'Reentered','1975-10-06 00:00:00','1975-10-03 00:00:00',88.78,183,246,51.81,'Intermediate' from dual union all
select 834,'000834','1964-039B','8A92E','8A92E',10325,'Reentered','1964-08-01 00:00:00','1964-07-24 00:00:00',88.99,205,245,51.27,'Intermediate' from dual union all
select 8340,'008340','1975-092E','Aura sep clamp','',36539,'Reentered','1978-10-27 00:00:00','1975-10-28 00:00:00',96.63,497,698,37.15,'Intermediate' from dual union all
select 8341,'008341','1975-092F','Aura sep clamp','',36539,'Reentered','1980-01-01 00:00:00','1975-11-06 00:00:00',96.82,503,710,37.16,'Intermediate' from dual union all
select 8342,'008342','1975-092G','Aura instrument cover','',36539,'Reentered','1979-11-20 00:00:00','1975-10-28 00:00:00',96.7,496,706,37.18,'Intermediate' from dual union all
select 8343,'008343','1975-094A','Kosmos-773','Strela-2M',36541,'In Earth orbit','','1992-02-12 00:00:00',100.53,775,793,74.05,'Intermediate' from dual union all
select 8344,'008344','1975-094B','S3M','S3M',36541,'In Earth orbit','','1991-03-28 00:00:00',100.37,759,793,74.05,'Intermediate' from dual union all
select 8345,'008345','1975-095A','Kosmos-774','Zenit-4MK',36542,'Landed','1975-10-15 00:00:00','1975-10-03 00:00:00',89.44,168,325,71.35,'Intermediate' from dual union all
select 8346,'008346','1975-094C','deb Kosmos-773','Strela-2M debris 1',36541,'Reentered','2004-06-18 00:00:00','1993-08-16 00:00:00',98.22,665,683,74.04,'Intermediate' from dual union all
select 8347,'008347','1975-085F','KDU part','KDU part',36532,'Reentered','1975-10-01 00:00:00','1975-09-30 00:00:00',88.67,168,250,64.97,'Intermediate' from dual union all
select 8348,'008348','1975-095B','11A57I','11A57I',36542,'Reentered','1975-10-10 00:00:00','1975-10-03 00:00:00',89.43,198,295,71.37,'Intermediate' from dual union all
select 8349,'008349','1975-088C','FEU-170 part','FEU-170 part',36535,'Reentered','1975-10-13 00:00:00','1975-10-01 00:00:00',89.46,199,297,72.84,'Intermediate' from dual union all
select 835,'000835','1964-039C','deb Kosmos-35','',10325,'Reentered','1964-07-31 00:00:00','1964-07-23 00:00:00',88.83,204,230,51.25,'Intermediate' from dual union all
select 8350,'008350','1975-095C','Sensor cover','Zenit-4MK sensor cover',36542,'Reentered','1975-10-08 00:00:00','1975-10-03 00:00:00',90,201,348,71.38,'Intermediate' from dual union all
select 8351,'008351','1975-095D','Sensor cover','Zenit-4MK sensor cover',36542,'Reentered','1975-10-08 00:00:00','1975-10-03 00:00:00',90.33,204,377,71.32,'Intermediate' from dual union all
select 8352,'008352','1975-082B','Star 37N S/N 14002','Star 37N',36529,'In Earth orbit','','1992-02-14 00:00:00',105.87,972,1099,47,'Intermediate' from dual union all
select 8353,'008353','1975-096A','AE-D','AE-D',36543,'Reentered','1976-03-12 00:00:00','1975-11-05 00:00:00',123.84,135,3569,90.08,'Medium' from dual union all
select 8354,'008354','1975-096B','Delta 115','Delta SSPU DSV-3P-4',36543,'Reentered','1976-04-01 00:00:00','1975-11-15 00:00:00',119.38,148,3159,90.06,'Polar' from dual union all
select 8355,'008355','1975-090C','KDU','KDU',36537,'Reentered','1975-10-12 00:00:00','1975-10-06 00:00:00',88.78,206,223,81.34,'Intermediate' from dual union all
select 8356,'008356','1975-090D','Blenda','Sunshade',36537,'Reentered','1975-10-08 00:00:00','1975-10-06 00:00:00',88.62,194,219,81.32,'Intermediate' from dual union all
select 8357,'008357','1975-097A','Kosmos-775','Oko',36544,'In Earth orbit','','1977-01-20 00:00:00',1434.3,35722,35780,1.05,'Drift GEO' from dual union all
select 8358,'008358','1975-097B','8S812','8S812',36544,'Reentered','1975-10-10 00:00:00','1975-10-09 00:00:00',87.65,155,161,51.49,'Intermediate' from dual union all
select 8359,'008359','1975-097C','Perekhodnik','Sredniy perekhodnik',36544,'Reentered','1975-10-09 00:00:00','1975-10-08 00:00:00',88.17,176,192,51.49,'Intermediate' from dual union all
select 836,'000836','1964-040A','Vela 2A','Vela 2A',10326,'Deep Space','1972-01-01 00:00:00','1964-09-02 00:00:00',6018.82,101986,104470,39.2,'Very High' from dual union all
select 8360,'008360','1975-098A','OPS 5499','GAMBIT-3 Mission 4345',36545,'Reentered Attached','1975-11-30 00:00:00','1975-10-16 00:00:00',89.26,119,357,96.4,'Sun-Synch' from dual union all
select 8361,'008361','1975-099A','TIP 2','TIP 2',36546,'Reentered','1991-05-26 00:00:00','1990-02-19 00:00:00',95.3,484,583,90.43,'Polar' from dual union all
select 8362,'008362','1975-095E','Blenda','Sunshade',36542,'Reentered','1975-10-14 00:00:00','1975-10-12 00:00:00',89.01,173,278,71.37,'Intermediate' from dual union all
select 8363,'008363','1975-095F','KDU','KDU',36542,'Reentered','1975-10-16 00:00:00','1975-10-12 00:00:00',89.11,167,294,71.35,'Intermediate' from dual union all
select 8364,'008364','1975-099B','Star 20 E-15','Star 20 E-15',36546,'Reentered','1978-03-28 00:00:00','1975-11-11 00:00:00',95.18,355,700,90.73,'Polar' from dual union all
select 8365,'008365','1975-095G','KDU part','KDU part',36542,'Reentered','1975-10-14 00:00:00','1975-10-14 00:00:00',88.33,152,231,71.38,'Intermediate' from dual union all
select 8366,'008366','1975-100A','GOES 1','GOES A',36547,'In Earth orbit','','1978-03-25 00:00:00',1436.27,35763,35817,.01,'Stationary' from dual;
insert into SATELLITE
select 8367,'008367','1975-100B','Delta 116','Delta SSPU DSV-3P-4',36547,'Reentered','1976-01-05 00:00:00','1975-11-19 00:00:00',94.19,182,777,28.27,'Intermediate' from dual union all
select 8368,'008368','1975-100C','Star 37E S/N 40024','Star 37E',36547,'Reentered','1998-12-06 00:00:00','1993-01-03 00:00:00',139.25,246,4796,23.35,'Medium' from dual union all
select 8369,'008369','1975-101A','Kosmos-776','Zenit-2M',36548,'Landed','1975-10-29 00:00:00','1975-10-19 00:00:00',89.34,198,286,62.84,'Intermediate' from dual union all
select 837,'000837','1964-040B','Vela 2B','Vela 2B',10326,'Deep Space','1969-01-01 00:00:00','1964-10-03 00:00:00',6002.73,94374,111691,40.68,'Very High' from dual union all
select 8370,'008370','1975-101B','11A57I','11A57I',36548,'Reentered','1975-10-23 00:00:00','1975-10-19 00:00:00',89.01,182,270,62.81,'Intermediate' from dual union all
select 8371,'008371','1975-100D','Delta 116 clamp band','',36547,'Reentered','1975-11-11 00:00:00','1975-10-24 00:00:00',92.71,182,633,28.32,'Intermediate' from dual union all
select 8372,'008372','1975-100E','Delta 116 clamp band','',36547,'Reentered','1975-11-01 00:00:00','1975-10-26 00:00:00',92.64,183,625,28.35,'Intermediate' from dual union all
select 8373,'008373','1975-101C','Sensor cover','Zenit-2M sensor cover',36548,'Reentered','1975-11-08 00:00:00','1975-10-24 00:00:00',89.13,196,266,62.81,'Intermediate' from dual union all
select 8374,'008374','1972-058EQ','deb Delta 89','',27990,'Reentered','1979-06-16 00:00:00','1979-03-14 00:00:00',99.22,672,771,100.02,'Sun-Synch' from dual union all
select 8375,'008375','1972-058ER','deb Delta 89','',27990,'Reentered','1981-05-27 00:00:00','1981-03-04 00:00:00',96.29,521,641,98.32,'Sun-Synch' from dual union all
select 8376,'008376','1972-058ES','deb Delta 89','',27990,'Reentered','1983-03-05 00:00:00','1989-01-12 00:00:00',88.63,202,212,98.37,'Sun-Synch' from dual union all
select 8377,'008377','1972-058ET','deb Delta 89','',27990,'Reentered','1982-11-22 00:00:00','1975-10-13 00:00:00',102.38,660,1083,98.01,'Sun-Synch' from dual union all
select 8378,'008378','1972-058EU','deb Delta 89','',27990,'Reentered','1978-03-28 00:00:00','1978-04-11 00:00:00',96.65,590,607,98.31,'Sun-Synch' from dual union all
select 8379,'008379','1972-058EV','deb Delta 89','',27990,'Reentered','1981-03-09 00:00:00','1981-03-05 00:00:00',90.64,294,318,98.88,'Sun-Synch' from dual union all
select 838,'000838','1964-040C','ERS 13','ERS 13',10326,'Reentered','1966-07-01 00:00:00','1964-10-30 00:00:00',2350.12,319,104032,38.3,'Very High' from dual union all
select 8380,'008380','1972-058EW','deb Delta 89','',27990,'Reentered','1979-03-30 00:00:00','1979-03-19 00:00:00',94.5,477,512,98.35,'Sun-Synch' from dual union all
select 8381,'008381','1972-058EX','deb Delta 89','',27990,'Reentered','1979-09-23 00:00:00','1979-07-16 00:00:00',96.43,526,649,98.36,'Sun-Synch' from dual union all
select 8382,'008382','1972-058EY','deb Delta 89','',27990,'Reentered','1980-10-13 00:00:00','1980-10-11 00:00:00',90.73,302,319,98.08,'Sun-Synch' from dual union all
select 8383,'008383','1972-058EZ','deb Delta 89','',27990,'Reentered','1979-10-09 00:00:00','1975-10-19 00:00:00',101.28,612,1027,98.61,'Sun-Synch' from dual union all
select 8384,'008384','1972-058FA','deb Delta 89','',27990,'Reentered','1979-12-07 00:00:00','1979-11-08 00:00:00',96.6,540,651,98.28,'Sun-Synch' from dual union all
select 8385,'008385','1972-058FB','deb Delta 89','',27990,'Reentered','1978-11-13 00:00:00','1978-01-04 00:00:00',93.93,327,607,97.3,'Sun-Synch' from dual union all
select 8386,'008386','1972-058FC','deb Delta 89','',27990,'Reentered','1993-12-08 00:00:00','1993-08-24 00:00:00',92.44,388,401,96.63,'Sun-Synch' from dual union all
select 8387,'008387','1972-058FD','deb Delta 89','',27990,'Reentered','1979-06-16 00:00:00','1979-04-25 00:00:00',95.9,509,616,98.66,'Sun-Synch' from dual union all
select 8388,'008388','1972-058FE','deb Delta 89','',27990,'Reentered','1981-06-25 00:00:00','1981-04-11 00:00:00',96.7,544,658,98.15,'Sun-Synch' from dual union all
select 8389,'008389','1972-058FF','deb Delta 89','',27990,'Reentered','1980-12-19 00:00:00','1980-12-16 00:00:00',91.31,326,352,98.17,'Sun-Synch' from dual union all
select 839,'000839','1964-039D','deb Kosmos-35','',10325,'Reentered','1964-07-25 00:00:00','1964-07-23 00:00:00',89.88,174,363,50.95,'Intermediate' from dual union all
select 8390,'008390','1972-058FG','deb Delta 89','',27990,'Reentered','1991-03-07 00:00:00','1991-03-06 00:00:00',88.5,196,205,98,'Sun-Synch' from dual union all
select 8391,'008391','1972-058FH','deb Delta 89','',27990,'Reentered','1980-03-30 00:00:00','1975-11-09 00:00:00',100.15,633,899,98.28,'Sun-Synch' from dual union all
select 8392,'008392','1972-058FJ','deb Delta 89','',27990,'Reentered','1982-11-04 00:00:00','1982-08-30 00:00:00',93.84,438,487,98.33,'Sun-Synch' from dual union all
select 8393,'008393','1972-058FK','deb Delta 89','',27990,'Reentered','1977-03-16 00:00:00','1977-03-10 00:00:00',91.84,339,391,98.19,'Sun-Synch' from dual union all
select 8394,'008394','1974-089BW','deb Delta 104','',33692,'In Earth orbit','','1993-07-28 00:00:00',118.09,1455,1735,102.13,'Sun-Synch' from dual union all
select 8395,'008395','1974-089BX','deb Delta 104','',33692,'In Earth orbit','','1993-08-03 00:00:00',115.57,1417,1546,100.73,'Sun-Synch' from dual union all
select 8396,'008396','1974-089BY','deb Delta 104','',33692,'In Earth orbit','','1993-08-07 00:00:00',115.96,1448,1550,101.92,'Sun-Synch' from dual union all
select 8397,'008397','1974-089BZ','deb Delta 104','',33692,'In Earth orbit','','1975-10-19 00:00:00',113.44,1312,1457,101.61,'Sun-Synch' from dual union all
select 8398,'008398','1974-089CA','deb Delta 104','',33692,'In Earth orbit','','1993-08-03 00:00:00',117.41,1503,1625,102.45,'Sun-Synch' from dual union all
select 8399,'008399','1974-089CB','deb Delta 104','',33692,'In Earth orbit','','1993-07-29 00:00:00',116.32,1394,1637,101.83,'Sun-Synch' from dual union all
select 84,'000084','61 Zeta','Discoverer 21','CORONA RM-2',7283,'Reentered Attached','1962-04-20 00:00:00','1961-03-20 00:00:00',97.4,243,1026,80.74,'Intermediate' from dual union all
select 840,'000840','60 Eta 4','Transit IIA despin weight','',6321,'Reentered','2003-08-19 00:00:00','1993-08-06 00:00:00',97.88,554,761,66.67,'Intermediate' from dual union all
select 8400,'008400','1974-089CC','deb Delta 104','',33692,'In Earth orbit','','1993-08-07 00:00:00',115.85,1449,1539,101.82,'Sun-Synch' from dual union all
select 8401,'008401','1974-089CD','deb Delta 104','',33692,'In Earth orbit','','1975-10-19 00:00:00',114,1357,1463,101.76,'Sun-Synch' from dual union all
select 8402,'008402','1974-089CE','deb Delta 104','',33692,'Reentered','2005-11-05 00:00:00','1975-10-01 00:00:00',119.46,1410,1904,101.37,'Sun-Synch' from dual union all
select 8403,'008403','1974-089CF','deb Delta 104','',33692,'In Earth orbit','','1993-08-05 00:00:00',115.58,1437,1527,101.38,'Sun-Synch' from dual union all
select 8404,'008404','1974-089CG','deb Delta 104','',33692,'In Earth orbit','','1975-10-18 00:00:00',116.13,1455,1559,101.63,'Sun-Synch' from dual union all
select 8405,'008405','1974-089CH','deb Delta 104','',33692,'In Earth orbit','','1993-08-06 00:00:00',127.73,1431,2616,102.65,'Medium' from dual union all
select 8406,'008406','1974-089CJ','deb Delta 104','',33692,'In Earth orbit','','1993-08-05 00:00:00',115.05,1442,1473,101.32,'Sun-Synch' from dual union all
select 8407,'008407','1974-089CK','deb Delta 104','',33692,'In Earth orbit','','1993-08-03 00:00:00',113.96,1225,1592,101.54,'Sun-Synch' from dual union all
select 8408,'008408','1972-058FL','deb Delta 89','',27990,'Reentered','1979-04-14 00:00:00','1979-02-28 00:00:00',94.89,479,547,98.86,'Sun-Synch' from dual union all
select 8409,'008409','1975-099C','TIP 2 despin weight','',36546,'Reentered','1978-11-12 00:00:00','1975-11-12 00:00:00',96.26,402,757,90.8,'Polar' from dual union all
select 841,'000841','60 Eta 5','Transit IIA despin weight','',6321,'Reentered','2002-11-29 00:00:00','1964-07-27 00:00:00',101.49,609,1050,66.69,'Intermediate' from dual union all
select 8410,'008410','1975-099D','TIP 2 despin weight','',36546,'Reentered','1976-05-31 00:00:00','1975-11-12 00:00:00',94.03,303,641,90.65,'Polar' from dual union all
select 8411,'008411','1975-050D','SA Veneri-9','SA',36497,'Deep Space Attached','1975-06-08 00:00:00','1975-10-20 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 8412,'008412','1975-101D','Kapsula Kosmosa-776','Nauka 20KS',36548,'Reentered','1975-11-01 00:00:00','1975-10-27 00:00:00',89.16,194,272,62.83,'Intermediate' from dual union all
select 8413,'008413','1975-101E','Nauka part','',36548,'Reentered','1975-10-31 00:00:00','1975-10-27 00:00:00',89.02,191,262,62.82,'Intermediate' from dual union all
select 8414,'008414','1975-097D','SOZ','SOZ',36544,'Reentered','1978-03-01 00:00:00','1975-11-27 00:00:00',631.72,291,35729,47.27,'GEO Transfer' from dual union all
select 8415,'008415','1975-097E','SOZ','SOZ',36544,'Reentered','1978-10-17 00:00:00','1975-11-12 00:00:00',632.94,327,35756,47.37,'GEO Transfer' from dual union all
select 8416,'008416','1975-102A','Kosmos-777','US-P',36549,'Reentered','1976-06-03 00:00:00','1975-11-28 00:00:00',93.28,429,441,65.01,'Intermediate' from dual union all
select 8417,'008417','1975-102B','11S692','11S692',36549,'Reentered','1975-10-30 00:00:00','1975-10-29 00:00:00',89.37,116,371,64.98,'Intermediate' from dual union all
select 8418,'008418','1975-081D','Blok-L','Blok-L',36528,'In Earth orbit','','1975-10-10 00:00:00',733.86,299,40846,62.58,'Molniya' from dual union all
select 8419,'008419','1975-103A','Kosmos-778','Tsiklon-B',36550,'In Earth orbit','','1975-12-06 00:00:00',104.87,973,1004,82.97,'Intermediate' from dual union all
select 842,'000842','1964-041A','Ranger 7','Ranger 7',10327,'Deep Space','1964-07-31 00:00:00','1964-07-28 00:00:00',23213.24,184,526173,28.96,'Cislunar' from dual union all
select 8420,'008420','1975-104A','Kosmos-779','Zenit-4MK',36551,'Landed','1975-11-18 00:00:00','1975-11-06 00:00:00',89.69,179,340,62.8,'Intermediate' from dual union all
select 8421,'008421','1975-103B','S3M','S3M',36550,'In Earth orbit','','1991-03-28 00:00:00',104.67,961,997,82.96,'Intermediate' from dual union all
select 8422,'008422','1975-104B','11A57I','11A57I',36551,'Reentered','1975-11-10 00:00:00','1975-11-06 00:00:00',89.12,170,292,62.79,'Intermediate' from dual union all
select 8423,'008423','1975-054D','SA Veneri-10','SA',36501,'Deep Space Attached','1975-06-14 00:00:00','1975-10-23 00:00:00',null,null,null,null,'Earth Escape' from dual union all
select 8424,'008424','1974-066C','deb Kosmos-673','',33669,'Reentered','1989-11-17 00:00:00','1975-11-07 00:00:00',97.1,606,634,81.19,'Intermediate' from dual union all
select 8425,'008425','1975-105A','Molniya-3','Molniya-3 No. 13',36552,'Reentered','2017-11-19 00:00:00','1975-12-18 00:00:00',717.6,500,39846,62.84,'Molniya' from dual union all
select 8426,'008426','1975-105B','11S510','11S510',36552,'Reentered','1975-12-15 00:00:00','1975-11-21 00:00:00',90.64,207,405,62.79,'Intermediate' from dual union all
select 8427,'008427','1975-105C','BOZ','BOZ',36552,'Reentered','1975-12-05 00:00:00','1975-11-21 00:00:00',90.47,186,410,62.79,'Intermediate' from dual union all
select 8428,'008428','1975-104C','KDU','KDU',36551,'Reentered','1975-11-19 00:00:00','1975-11-16 00:00:00',88.94,162,283,62.8,'Intermediate' from dual union all
select 8429,'008429','1975-104D','Blenda','Sunshade',36551,'Reentered','1975-11-17 00:00:00','1975-11-15 00:00:00',88.87,158,279,62.79,'Intermediate' from dual union all
select 843,'000843','1964-041B','Agena B 6009','Agena B 6009',10327,'Deep Space','1964-07-31 00:00:00','1964-07-28 00:00:00',17553.03,184,434524,28.94,'Cislunar' from dual union all
select 8430,'008430','1975-106A','Soyuz-20','Soyuz 7K-T(A9) No. 64',36553,'Landed','1976-02-16 00:00:00','1975-12-22 00:00:00',91.39,340,346,51.59,'Intermediate' from dual union all
select 8431,'008431','1975-106B','11S510','11S510',36553,'Reentered','1975-11-20 00:00:00','1975-11-18 00:00:00',88.46,180,217,51.63,'Intermediate' from dual union all
select 8432,'008432','1972-058FM','deb Delta 89','',27990,'Reentered','2001-02-09 00:00:00','1993-08-04 00:00:00',97.38,550,717,98.27,'Sun-Synch' from dual union all
select 8433,'008433','1972-058FN','deb Delta 89','',27990,'Reentered','1982-08-03 00:00:00','1982-07-27 00:00:00',103.37,655,1181,98.81,'Sun-Synch' from dual union all
select 8434,'008434','1972-058FP','deb Delta 89','',27990,'Reentered','1976-01-31 00:00:00','1976-01-27 00:00:00',89,210,240,97.78,'Sun-Synch' from dual union all
select 8435,'008435','1974-089CL','deb Delta 104','',33692,'In Earth orbit','','1993-08-06 00:00:00',117.97,1458,1722,101.6,'Sun-Synch' from dual union all
select 8436,'008436','1974-089CM','deb Delta 104','',33692,'In Earth orbit','','1993-08-02 00:00:00',116.56,1457,1596,101.57,'Sun-Synch' from dual union all
select 8437,'008437','1974-089CN','deb Delta 104','',33692,'In Earth orbit','','1993-07-27 00:00:00',114.5,1400,1466,101.89,'Sun-Synch' from dual union all
select 8438,'008438','1974-089CP','deb Delta 104','',33692,'In Earth orbit','','1993-08-01 00:00:00',108.07,1004,1271,101.62,'Sun-Synch' from dual union all
select 8439,'008439','1974-089CQ','deb Delta 104','',33692,'In Earth orbit','','1975-08-29 00:00:00',113.69,1330,1462,102.57,'Sun-Synch' from dual union all
select 844,'000844','1964-042A','Kosmos-36','DS-P1-Yu No. 1',10328,'Reentered','1965-02-28 00:00:00','1964-08-29 00:00:00',91.83,253,476,49,'Intermediate' from dual union all
select 8440,'008440','1975-107A','AE-E','AE-E',36554,'Reentered','1981-06-10 00:00:00','1975-12-21 00:00:00',115.55,142,2819,19.68,'Equatorial' from dual union all
select 8441,'008441','1975-107B','Delta 117','Delta SSPU DSV-3P-4',36554,'Reentered','1976-03-19 00:00:00','1975-12-21 00:00:00',112.43,156,2521,19.68,'Equatorial' from dual union all
select 8442,'008442','1975-108A','Kosmos-780','Zenit-2M',36555,'Landed','1975-12-03 00:00:00','1975-11-23 00:00:00',89.28,200,278,65.01,'Intermediate' from dual union all
select 8443,'008443','1975-108B','11A57I','11A57I',36555,'Reentered','1975-11-27 00:00:00','1975-11-23 00:00:00',88.99,197,253,65.02,'Intermediate' from dual union all
select 8444,'008444','1975-109A','Kosmos-781','Tselina-OK',36556,'Reentered','1980-11-26 00:00:00','1975-12-22 00:00:00',95.16,505,548,74.04,'Intermediate' from dual union all
select 8445,'008445','1975-109B','S3M','S3M',36556,'Reentered','1980-12-20 00:00:00','1975-12-22 00:00:00',95.06,489,554,74.05,'Intermediate' from dual union all
select 8446,'008446','1975-108C','Sensor cover','Zenit-2M sensor cover',36555,'Reentered','1975-11-22 00:00:00','1975-11-21 00:00:00',89.33,198,285,65.01,'Intermediate' from dual union all
select 8447,'008447','1975-109C','deb Kosmos-781','Tselina-O part',36556,'Reentered','1978-06-28 00:00:00','1975-12-22 00:00:00',94.94,487,545,73.99,'Intermediate' from dual union all
select 8448,'008448','1975-109D','deb Kosmos-781','Tselina-O part',36556,'Reentered','1977-11-25 00:00:00','1975-12-23 00:00:00',94.76,470,544,74.05,'Intermediate' from dual union all
select 8449,'008449','1975-109E','deb Kosmos-781','Tselina-O part',36556,'Reentered','1977-10-09 00:00:00','1975-12-21 00:00:00',94.85,478,544,74.12,'Intermediate' from dual union all
select 845,'000845','1964-042B','S1','S1',10328,'Reentered','1964-11-29 00:00:00','1964-09-07 00:00:00',91.6,248,459,48.99,'Intermediate' from dual union all
select 8450,'008450','1975-110A','Kosmos-782','Bion No. 3',36557,'Landed','1975-12-15 00:00:00','1975-12-02 00:00:00',90.47,216,379,62.8,'Intermediate' from dual union all
select 8451,'008451','1975-110B','11S510','11S510',36557,'Reentered','1975-12-23 00:00:00','1975-12-02 00:00:00',90.12,211,350,62.79,'Intermediate' from dual union all
select 8452,'008452','1975-111A','Fanhui Shi Weixing (01)','FSW No. 2',36558,'Reentered','1975-12-29 00:00:00','1975-12-04 00:00:00',90.7,178,440,62.95,'Intermediate' from dual union all
select 8453,'008453','1975-111B','CZ-2C Stage 2','CZ-2C Stage 2',36558,'Reentered','1975-12-22 00:00:00','1975-12-04 00:00:00',90.41,174,416,62.95,'Intermediate' from dual union all
select 8454,'008454','1975-111C','CZ-2C motor cover','',36558,'Reentered','1975-11-29 00:00:00','1975-11-26 00:00:00',92.09,140,614,63.05,'Intermediate' from dual union all
select 8455,'008455','1975-111D','CZ-2C motor cover','',36558,'Reentered','1975-11-28 00:00:00','1975-11-26 00:00:00',91.86,218,514,62.95,'Intermediate' from dual union all
select 8456,'008456','1975-111E','CZ-2C motor cover','',36558,'Reentered','1975-11-28 00:00:00','1975-11-27 00:00:00',89.43,175,318,62.95,'Intermediate' from dual;
insert into SATELLITE
select 8457,'008457','1975-111F','CZ-2C motor cover','',36558,'Reentered','1975-11-28 00:00:00','1975-11-27 00:00:00',90.08,170,387,62.93,'Intermediate' from dual union all
select 8458,'008458','1975-112A','Kosmos-783','Strela-2M',36559,'In Earth orbit','','1992-02-14 00:00:00',100.68,783,799,74.04,'Intermediate' from dual union all
select 8459,'008459','1975-112B','S3M','S3M',36559,'In Earth orbit','','1991-03-28 00:00:00',100.51,771,794,74.05,'Intermediate' from dual union all
select 846,'000846','1964-043A','OPS 3042','CORONA J-12',10329,'Reentered Attached','1964-09-01 00:00:00','1964-08-13 00:00:00',90.53,179,421,79.97,'Intermediate' from dual union all
select 8460,'008460','1975-108D','Kapsula Kosmosa-780','Nauka 20KS',36555,'Reentered','1975-12-11 00:00:00','1975-11-30 00:00:00',89.15,197,267,65.01,'Intermediate' from dual union all
select 8461,'008461','1975-108E','Nauka part','',36555,'Reentered','1975-12-02 00:00:00','1975-11-30 00:00:00',89.02,191,262,65.01,'Intermediate' from dual union all
select 8462,'008462','1975-105D','Blok-L','Blok-L',36552,'In Earth orbit','','1975-12-17 00:00:00',733.88,497,40649,62.81,'Molniya' from dual union all
select 8463,'008463','1975-113A','Kosmos-784','Zenit-2M',36560,'Landed','1975-12-15 00:00:00','1975-12-05 00:00:00',88.89,213,227,81.32,'Intermediate' from dual union all
select 8464,'008464','1975-113B','11A57I','11A57I',36560,'Reentered','1975-12-07 00:00:00','1975-12-05 00:00:00',88.55,200,205,81.33,'Intermediate' from dual union all
select 8465,'008465','1975-113C','Sensor cover','Zenit-2M sensor cover',36560,'Reentered','1975-12-04 00:00:00','1975-12-03 00:00:00',88.96,214,233,81.32,'Intermediate' from dual union all
select 8466,'008466','1975-113D','Sensor cover','Zenit-2M sensor cover',36560,'Reentered','1975-12-08 00:00:00','1975-12-05 00:00:00',89.12,209,253,81.32,'Intermediate' from dual union all
select 8467,'008467','1975-114A','OPS 4428','HEXAGON SV-11',36561,'Deorbited','1976-04-01 00:00:00','1976-01-03 00:00:00',88.31,153,229,96.25,'Sun-Synch' from dual union all
select 8468,'008468','1975-114B','S3-2','S3-2',36561,'Reentered','1978-05-01 00:00:00','1976-01-03 00:00:00',102.74,233,1544,96.28,'Sun-Synch' from dual union all
select 8469,'008469','1975-114C','Titan 23D-13 Stage 2','LR91AJ11 23D-13',36561,'Reentered','1975-12-06 00:00:00','1975-12-05 00:00:00',88.05,154,202,96.26,'Sun-Synch' from dual union all
select 847,'000847','1964-015C','Ariel II despin weight','',10301,'Reentered','1968-05-06 00:00:00','1964-08-05 00:00:00',104.13,370,1538,51.39,'Intermediate' from dual union all
select 8470,'008470','1975-114D','deb Star 17A','',36561,'Reentered','1976-06-03 00:00:00','1976-02-29 00:00:00',101.21,214,1418,95.97,'Sun-Synch' from dual union all
select 8471,'008471','1975-115A','Interkosmos-14','DS-U2-IK No. 6',36562,'Reentered','1983-02-27 00:00:00','1976-01-11 00:00:00',105.25,333,1680,73.99,'Intermediate' from dual union all
select 8472,'008472','1975-115B','S3M','S3M',36562,'Reentered','1982-11-28 00:00:00','1976-01-14 00:00:00',105.05,326,1667,74.01,'Intermediate' from dual union all
select 8473,'008473','1975-116A','Kosmos-785','US-A reactor',36563,'In Earth orbit','','1976-01-11 00:00:00',104.25,898,1021,65.08,'Intermediate' from dual union all
select 8474,'008474','1975-115C','deb Interkosmos-14','',36562,'Reentered','1978-11-11 00:00:00','1976-01-31 00:00:00',107.71,369,1873,73.05,'Intermediate' from dual union all
select 8475,'008475','1975-115D','deb Interkosmos-14','',36562,'Reentered','1982-05-30 00:00:00','1976-02-09 00:00:00',98.99,361,1060,73.98,'Intermediate' from dual union all
select 8476,'008476','1975-117A','Satcom 1','RCA-A',36564,'In Earth orbit','','1977-01-20 00:00:00',1436.01,35707,35863,.02,'Stationary' from dual union all
select 8477,'008477','1975-116B','Kosmos-785 RLS','US-A RLS',36563,'Reentered','1975-12-14 00:00:00','1975-12-13 00:00:00',89.74,241,283,64.96,'Intermediate' from dual union all
select 8478,'008478','1975-117B','Delta 118','Delta SSPU DSV-3P-4',36564,'Reentered','1976-08-07 00:00:00','1976-01-15 00:00:00',105.25,183,1830,28.39,'Intermediate' from dual union all
select 8479,'008479','1975-117C','Star 37E S/N 40039','Star 37E',36564,'Reentered','1982-11-10 00:00:00','1976-09-29 00:00:00',617.26,253,35020,26.87,'GEO Transfer' from dual union all
select 848,'000848','1964-044A','Kosmos-37','Zenit-2 No. 22',10330,'Landed','1964-08-22 00:00:00','1964-08-19 00:00:00',89.38,203,285,64.96,'Intermediate' from dual union all
select 8480,'008480','1975-116C','Kosmos-785 DU','US-A DU',36563,'Reentered','1976-02-05 00:00:00','1975-12-19 00:00:00',89.49,241,258,64.99,'Intermediate' from dual union all
select 8481,'008481','1975-113E','Nauka part','',36560,'Reentered','1975-12-15 00:00:00','1975-12-13 00:00:00',88.59,191,219,81.34,'Intermediate' from dual union all
select 8482,'008482','1975-118A','DSP F5','DSP 8',36565,'In Earth orbit','','1980-04-21 00:00:00',1433.38,35601,35866,3.31,'Inclined Drift' from dual union all
select 8483,'008483','1975-118B','Titan 23C-11 Stage 2','LR91AJ11 23C-11',36565,'Reentered','1975-12-19 00:00:00','1975-12-16 00:00:00',89.11,149,312,28.59,'Intermediate' from dual union all
select 8484,'008484','1975-113F','Nauka part','',36560,'Reentered','1975-12-20 00:00:00','1975-12-14 00:00:00',88.77,211,216,81.33,'Intermediate' from dual union all
select 8485,'008485','1975-113G','Kapsula Kosmosa-784','Nauka 22KS',36560,'Reentered','1975-12-21 00:00:00','1975-12-14 00:00:00',88.7,204,217,81.33,'Intermediate' from dual union all
select 8486,'008486','1975-110C','KBE part','KBE part',36557,'Reentered','1975-12-31 00:00:00','1975-12-15 00:00:00',90.19,214,354,62.81,'Intermediate' from dual union all
select 8487,'008487','1975-110D','KB electropitaniya','KBE',36557,'Reentered','1976-02-10 00:00:00','1975-12-26 00:00:00',90.15,214,350,62.8,'Intermediate' from dual union all
select 8488,'008488','1975-119A','JSSW 4','JSSW',36566,'Reentered','1976-01-27 00:00:00','1975-12-23 00:00:00',90.18,187,380,69,'Intermediate' from dual union all
select 8489,'008489','1975-120A','Kosmos-786','Zenit-4MK',36567,'Landed','1975-12-29 00:00:00','1975-12-18 00:00:00',89.46,172,324,65,'Intermediate' from dual union all
select 849,'000849','1964-044B','8A92E','8A92E',10330,'Reentered','1964-09-03 00:00:00','1964-08-21 00:00:00',89.33,216,267,64.97,'Intermediate' from dual union all
select 8490,'008490','1975-120B','11A57I','11A57I',36567,'Reentered','1975-12-21 00:00:00','1975-12-19 00:00:00',88.85,168,267,65.02,'Intermediate' from dual union all
select 8491,'008491','1975-119B','FB-1 Stage 2','FB-1 Stage 2',36566,'Reentered','1976-01-10 00:00:00','1975-12-23 00:00:00',89.84,182,352,69,'Intermediate' from dual union all
select 8492,'008492','1975-121A','Molniya-2','Molniya-2',36568,'Reentered','1987-03-07 00:00:00','1976-02-20 00:00:00',717.76,485,39869,62.89,'Molniya' from dual union all
select 8493,'008493','1975-121B','11S510','11S510',36568,'Reentered','1976-01-14 00:00:00','1975-12-24 00:00:00',90.26,209,366,62.81,'Intermediate' from dual union all
select 8494,'008494','1975-121C','BOZ','BOZ',36568,'Reentered','1976-01-11 00:00:00','1975-12-24 00:00:00',90.49,194,404,62.82,'Intermediate' from dual union all
select 8495,'008495','1972-058FQ','deb Delta 89','',27990,'In Earth orbit','','1993-08-07 00:00:00',118.49,1131,2096,101.06,'Sun-Synch' from dual union all
select 8496,'008496','1972-058FR','deb Delta 89','',27990,'Reentered','1981-04-13 00:00:00','1975-12-16 00:00:00',124.83,903,2889,99.61,'Medium' from dual union all
select 8497,'008497','1972-058FS','deb Delta 89','',27990,'Reentered','1992-03-16 00:00:00','1992-03-14 00:00:00',90.57,300,305,98.74,'Sun-Synch' from dual union all
select 8498,'008498','1972-058FT','deb Delta 89','',27990,'Reentered','1980-11-03 00:00:00','1975-12-16 00:00:00',100.78,605,987,98.59,'Sun-Synch' from dual union all
select 8499,'008499','1972-058FU','deb Delta 89','',27990,'Reentered','1979-11-12 00:00:00','1979-10-27 00:00:00',95.12,472,576,98.54,'Sun-Synch' from dual union all
select 85,'000085','61 Delta 3','S-56A Canister part','',7263,'In Earth orbit','','1969-07-21 00:00:00',117.05,639,2458,38.87,'Intermediate' from dual union all
select 850,'000850','1964-045A','OPS 3802','GAMBIT SV 960',10331,'Reentered','1964-08-23 00:00:00','1964-08-16 00:00:00',89.02,147,306,95.51,'Sun-Synch' from dual union all
select 8500,'008500','1972-058FV','deb Delta 89','',27990,'Reentered','1979-06-28 00:00:00','1979-06-24 00:00:00',95.77,508,604,98.49,'Sun-Synch' from dual union all
select 8501,'008501','1972-058FW','deb Delta 89','',27990,'Reentered','1979-11-21 00:00:00','1979-11-10 00:00:00',95.24,483,578,98.47,'Sun-Synch' from dual union all
select 8502,'008502','1972-058FX','deb Delta 89','',27990,'Reentered','1978-09-26 00:00:00','1978-06-19 00:00:00',96.51,513,670,98.58,'Sun-Synch' from dual union all
select 8503,'008503','1972-058FY','deb Delta 89','',27990,'Reentered','1978-03-21 00:00:00','1978-01-12 00:00:00',95.31,480,588,98.41,'Sun-Synch' from dual union all
select 8504,'008504','1972-058FZ','deb Delta 89','',27990,'Reentered','1981-03-21 00:00:00','1981-03-18 00:00:00',91.58,345,360,98.1,'Sun-Synch' from dual union all
select 8505,'008505','1972-058GA','deb Delta 89','',27990,'Reentered','1983-01-27 00:00:00','1975-12-09 00:00:00',100.96,655,954,98.32,'Sun-Synch' from dual union all
select 8506,'008506','1972-058GB','deb Delta 89','',27990,'Reentered','1977-02-23 00:00:00','1977-02-14 00:00:00',91.3,330,347,97.84,'Sun-Synch' from dual union all
select 8507,'008507','1972-058GC','deb Delta 89','',27990,'Reentered','1980-01-29 00:00:00','1975-12-09 00:00:00',99.76,650,845,97.99,'Sun-Synch' from dual union all
select 8508,'008508','1972-058GD','deb Delta 89','',27990,'Reentered','1980-08-24 00:00:00','1975-12-15 00:00:00',99.6,625,854,97.85,'Sun-Synch' from dual union all
select 8509,'008509','1972-058GE','deb Delta 89','',27990,'Reentered','1980-08-05 00:00:00','1975-12-22 00:00:00',102.64,667,1101,97.8,'Sun-Synch' from dual union all
select 851,'000851','1964-045B','Hitchhiker 2','P-11 No. 4202',10331,'Reentered','1979-03-08 00:00:00','1964-12-10 00:00:00',127.14,271,3724,95.69,'Medium' from dual union all
select 8510,'008510','1975-122A','Prognoz-4','SO-M No. 504',36569,'Deep Space','1975-12-22 00:00:00','1975-12-22 00:00:00',91.48,231,464,65.04,'Intermediate' from dual union all
select 8511,'008511','1975-122B','11S510','11S510',36569,'Reentered','1976-02-22 00:00:00','1976-01-22 00:00:00',90.77,219,406,64.98,'Intermediate' from dual union all
select 8512,'008512','1975-122C','BOZ','BOZ',36569,'Reentered','1976-02-09 00:00:00','1975-12-29 00:00:00',91.36,206,477,65.02,'Intermediate' from dual union all
select 8513,'008513','1975-123A','Raduga','Gran'' No. 11L',36570,'In Earth orbit','','1978-03-31 00:00:00',1435.98,35704,35865,1.47,'Stationary' from dual union all
select 8514,'008514','1975-123B','Perekhodnik','Sredniy perekhodnik',36570,'Reentered','1975-12-22 00:00:00','1975-12-22 00:00:00',87.93,146,198,51.46,'Intermediate' from dual union all
select 8515,'008515','1975-123C','8S812','8S812',36570,'Reentered','1975-12-25 00:00:00','1975-12-24 00:00:00',87.97,165,183,51.48,'Intermediate' from dual union all
select 8516,'008516','1975-118C','Transtage 29','Transtage 29',36565,'In Earth orbit','','1975-12-15 00:00:00',1431.67,35600,35800,2,'Drift GEO' from dual union all
select 8517,'008517','1975-118D','DSP 8 Aperture Cover','',36565,'In Earth orbit','','1975-12-15 00:00:00',1436.01,35780,35790,1,'Stationary' from dual union all
select 8518,'008518','1975-110E','PTDU cover','KBE part',36557,'Reentered','1976-01-26 00:00:00','1975-12-25 00:00:00',92.09,307,447,62.8,'Intermediate' from dual union all
select 8519,'008519','1975-124A','Meteor','Meteor No. 38',36571,'In Earth orbit','','1991-05-06 00:00:00',102.2,841,885,81.25,'Intermediate' from dual union all
select 852,'000852','1964-045C','Agena D 4808','Agena D',10331,'Reentered','1964-08-16 00:00:00','1964-08-16 00:00:00',88.36,142,245,95.5,'Sun-Synch' from dual union all
select 8520,'008520','1975-124B','8A92ME','8A92ME',36571,'In Earth orbit','','1992-02-14 00:00:00',102.28,831,903,81.26,'Intermediate' from dual union all
select 8521,'008521','1975-125A','Molniya-3','Molniya-3 No. 15',36572,'Reentered','1986-08-12 00:00:00','1976-02-29 00:00:00',717.72,506,39846,62.86,'Molniya' from dual union all
select 8522,'008522','1975-125B','11S510','11S510',36572,'Reentered','1976-01-23 00:00:00','1976-01-03 00:00:00',90.45,208,386,62.78,'Intermediate' from dual union all
select 8523,'008523','1975-120C','KDU','KDU',36567,'Reentered','1976-01-03 00:00:00','1975-12-27 00:00:00',89.09,170,289,65.04,'Intermediate' from dual union all
select 8524,'008524','1975-120D','Blenda','Sunshade',36567,'Reentered','1975-12-29 00:00:00','1975-12-27 00:00:00',88.86,168,269,65.02,'Intermediate' from dual union all
select 8525,'008525','1975-125C','BOZ','BOZ',36572,'Reentered','1976-01-19 00:00:00','1976-01-04 00:00:00',90.41,190,400,62.83,'Intermediate' from dual union all
select 8526,'008526','1975-125D','deb Blok-I','',36572,'Reentered','1975-12-28 00:00:00','1975-12-27 00:00:00',90.84,209,423,62.78,'Intermediate' from dual union all
select 8527,'008527','1975-125E','deb Blok-I','',36572,'Reentered','1976-01-08 00:00:00','1976-01-01 00:00:00',90.44,204,389,62.84,'Intermediate' from dual union all
select 8528,'008528','1975-120E','KDU part','KDU part',36567,'Reentered','1975-12-30 00:00:00','1975-12-29 00:00:00',88.66,159,258,65.01,'Intermediate' from dual union all
select 8529,'008529','1975-121D','Blok-L','Blok-L',36568,'Reentered','1986-10-13 00:00:00','1976-01-20 00:00:00',732.41,448,40626,62.79,'Molniya' from dual union all
select 853,'000853','1964-046A','Kosmos-38','Strela-1',10332,'Reentered','1964-11-08 00:00:00','1964-09-18 00:00:00',93,199,644,56.12,'Intermediate' from dual union all
select 8530,'008530','1976-001A','Kosmos-787','Tselina-OM',38953,'Reentered','1980-12-12 00:00:00','1976-02-08 00:00:00',95.25,516,546,74.03,'Intermediate' from dual union all
select 8531,'008531','1976-001B','S3M','S3M',38953,'Reentered','1981-03-15 00:00:00','1976-02-05 00:00:00',95.16,502,551,74.03,'Intermediate' from dual union all
select 8532,'008532','1972-058GF','deb Delta 89','',27990,'Reentered','1978-07-26 00:00:00','1978-01-27 00:00:00',95.57,453,640,99.04,'Sun-Synch' from dual union all
select 8533,'008533','1972-058GG','deb Delta 89','',27990,'In Earth orbit','','1993-08-04 00:00:00',101.46,704,952,98.46,'Sun-Synch' from dual union all
select 8534,'008534','1972-058GH','deb Delta 89','',27990,'Reentered','1980-11-16 00:00:00','1976-01-03 00:00:00',110.24,683,1792,98.55,'Sun-Synch' from dual union all
select 8535,'008535','1972-058GJ','deb Delta 89','',27990,'Reentered','1979-02-16 00:00:00','1979-02-01 00:00:00',94.3,437,533,98.65,'Sun-Synch' from dual union all
select 8536,'008536','1972-058GK','deb Delta 89','',27990,'Reentered','1978-01-22 00:00:00','1976-01-10 00:00:00',97.35,556,708,98.35,'Sun-Synch' from dual union all
select 8537,'008537','1972-058GL','deb Delta 89','',27990,'Reentered','1988-03-31 00:00:00','1988-03-29 00:00:00',89.56,241,265,98.23,'Sun-Synch' from dual union all
select 8538,'008538','1972-058GM','deb Delta 89','',27990,'Reentered','1999-05-23 00:00:00','1993-08-05 00:00:00',97.88,597,718,97.27,'Sun-Synch' from dual union all
select 8539,'008539','1972-058GN','deb Delta 89','',27990,'In Earth orbit','','1993-08-04 00:00:00',102.37,701,1040,97.31,'Sun-Synch' from dual union all
select 854,'000854','1964-046B','Kosmos-39','Strela-1',10332,'Reentered','1964-11-17 00:00:00','1964-09-19 00:00:00',93.28,204,667,56.15,'Intermediate' from dual union all
select 8540,'008540','1972-058GP','deb Delta 89','',27990,'Reentered','1977-11-02 00:00:00','1977-09-10 00:00:00',95.16,453,600,98.71,'Sun-Synch' from dual union all
select 8541,'008541','1972-058GQ','deb Delta 89','',27990,'Reentered','1979-01-07 00:00:00','1975-11-01 00:00:00',107.33,846,1360,98.29,'Sun-Synch' from dual union all
select 8542,'008542','1972-058GR','deb Delta 89','',27990,'In Earth orbit','','1993-08-02 00:00:00',104.53,694,1251,97.98,'Sun-Synch' from dual union all
select 8543,'008543','1972-058GS','deb Delta 89','',27990,'Reentered','1980-05-15 00:00:00','1976-01-05 00:00:00',107.28,780,1422,97.91,'Sun-Synch' from dual union all
select 8544,'008544','1972-058GT','deb Delta 89','',27990,'In Earth orbit','','1993-08-06 00:00:00',113.48,716,2057,97.42,'Sun-Synch' from dual union all
select 8545,'008545','1972-058GU','deb Delta 89','',27990,'Reentered','1988-07-29 00:00:00','1988-07-27 00:00:00',89.74,255,269,98.32,'Sun-Synch' from dual union all
select 8546,'008546','1975-123D','SOZ','SOZ',36570,'In Earth orbit','','1976-01-26 00:00:00',630.11,247,35690,47.11,'GEO Transfer' from dual;
insert into SATELLITE
select 8547,'008547','1975-123E','SOZ','SOZ',36570,'Reentered','1997-05-04 00:00:00','1992-02-16 00:00:00',326.56,174,18486,46.54,'Highly Ellip' from dual union all
select 8548,'008548','1975-049F','Blok-L','Blok-L',36496,'Reentered','1990-10-13 00:00:00','1976-01-05 00:00:00',730.61,698,40288,62.89,'Molniya' from dual union all
select 8549,'008549','1976-001C','deb Kosmos-787','Tselina-O part',38953,'Reentered','1977-10-20 00:00:00','1976-02-07 00:00:00',94.94,490,542,74.05,'Intermediate' from dual union all
select 855,'000855','1964-046C','Kosmos-40','Strela-1',10332,'Reentered','1964-11-18 00:00:00','1964-09-17 00:00:00',92.96,202,636,56.14,'Intermediate' from dual union all
select 8550,'008550','1976-001D','deb Kosmos-787','Tselina-O part',38953,'Reentered','1977-03-24 00:00:00','1976-02-06 00:00:00',94.82,486,534,74.04,'Intermediate' from dual union all
select 8551,'008551','1976-002A','Kosmos-788','Zenit-4MK',38954,'Landed','1976-01-20 00:00:00','1976-01-09 00:00:00',89.51,182,319,62.81,'Intermediate' from dual union all
select 8552,'008552','1976-002B','11A57I','11A57I',38954,'Reentered','1976-01-14 00:00:00','1976-01-09 00:00:00',89.33,178,304,62.8,'Intermediate' from dual union all
select 8553,'008553','1976-002C','Sensor cover','Zenit-4MK sensor cover',38954,'Reentered','1976-01-12 00:00:00','1976-01-09 00:00:00',89.35,175,310,62.75,'Intermediate' from dual union all
select 8554,'008554','1974-089CR','deb Delta 104','',33692,'In Earth orbit','','1993-08-07 00:00:00',114.71,1419,1466,101.93,'Sun-Synch' from dual union all
select 8555,'008555','1974-089CS','deb Delta 104','',33692,'In Earth orbit','','1993-08-07 00:00:00',115.32,1419,1521,101.39,'Sun-Synch' from dual union all
select 8556,'008556','1974-089CT','deb Delta 104','',33692,'In Earth orbit','','1993-08-03 00:00:00',115.05,1394,1522,101.25,'Sun-Synch' from dual union all
select 8557,'008557','1974-089CU','deb Delta 104','',33692,'Reentered','1982-06-10 00:00:00','1982-06-06 00:00:00',121.03,1437,2017,101.16,'Medium' from dual union all
select 8558,'008558','1974-089CV','deb Delta 104','',33692,'Reentered','1978-01-28 00:00:00','1976-01-04 00:00:00',129.32,1034,3152,100.1,'Medium' from dual union all
select 8559,'008559','1975-011G','Star 37E S/N 40023','Star 37E',36458,'Reentered','1998-12-06 00:00:00','1975-02-07 00:00:00',650.99,197,36810,23.9,'GEO Transfer' from dual union all
select 856,'000856','1964-046D','S3','S3',10332,'Reentered','1965-02-19 00:00:00','1964-09-18 00:00:00',94.5,204,785,56.14,'Intermediate' from dual union all
select 8560,'008560','1974-089CX','deb Delta 104','',33692,'In Earth orbit','','1993-08-04 00:00:00',108.64,846,1482,102.15,'Sun-Synch' from dual union all
select 8561,'008561','1974-089CY','deb Delta 104','',33692,'In Earth orbit','','1976-01-08 00:00:00',114.61,1375,1501,101.72,'Sun-Synch' from dual union all
select 8562,'008562','1974-089CZ','deb Delta 104','',33692,'In Earth orbit','','1993-08-06 00:00:00',118.57,1406,1828,101.68,'Sun-Synch' from dual union all
select 8563,'008563','1974-089DA','deb Delta 104','',33692,'In Earth orbit','','1993-07-30 00:00:00',115.12,1391,1531,101.32,'Sun-Synch' from dual union all
select 8564,'008564','1974-089DB','deb Delta 104','',33692,'In Earth orbit','','1993-08-05 00:00:00',114.32,1372,1477,101.21,'Sun-Synch' from dual union all
select 8565,'008565','1974-089DC','deb Delta 104','',33692,'Reentered','1988-05-16 00:00:00','1988-05-15 00:00:00',90.12,268,293,99.15,'Sun-Synch' from dual union all
select 8566,'008566','1974-089DD','deb Delta 104','',33692,'In Earth orbit','','1993-08-01 00:00:00',113.84,1347,1459,101.07,'Sun-Synch' from dual union all
select 8567,'008567','1974-089DE','deb Delta 104','',33692,'In Earth orbit','','1993-08-07 00:00:00',128.08,1422,2656,100.63,'Medium' from dual union all
select 8568,'008568','1972-058GV','deb Delta 89','',27990,'Reentered','1978-08-19 00:00:00','1978-07-17 00:00:00',99,559,863,98.66,'Sun-Synch' from dual union all
select 8569,'008569','1972-058GW','deb Delta 89','',27990,'Reentered','1986-03-15 00:00:00','1986-03-13 00:00:00',89.76,254,272,98.72,'Sun-Synch' from dual union all
select 857,'000857','1964-046E','deb Kosmos-38','',10332,'Reentered','1964-09-10 00:00:00','1964-09-01 00:00:00',94.43,209,773,56.14,'Intermediate' from dual union all
select 8570,'008570','1972-058GX','deb Delta 89','',27990,'Reentered','1982-10-06 00:00:00','1976-01-04 00:00:00',102.67,651,1118,98.65,'Sun-Synch' from dual union all
select 8571,'008571','1972-058GY','deb Delta 89','',27990,'Reentered','1981-03-05 00:00:00','1976-01-01 00:00:00',102.97,661,1138,98.1,'Sun-Synch' from dual union all
select 8572,'008572','1972-058GZ','deb Delta 89','',27990,'Reentered','1980-07-04 00:00:00','1980-10-23 00:00:00',93.07,400,450,98.24,'Sun-Synch' from dual union all
select 8573,'008573','1972-058HA','deb Delta 89','',27990,'Reentered','1979-11-18 00:00:00','1979-11-08 00:00:00',92.6,371,433,98.41,'Sun-Synch' from dual union all
select 8574,'008574','1972-058HB','deb Delta 89','',27990,'Reentered','1979-06-16 00:00:00','1979-05-19 00:00:00',94.06,462,484,98.07,'Sun-Synch' from dual union all
select 8575,'008575','1972-058HC','deb Delta 89','',27990,'Reentered','1978-01-02 00:00:00','1976-02-10 00:00:00',95.13,472,578,99.02,'Sun-Synch' from dual union all
select 8576,'008576','1972-058HD','deb Delta 89','',27990,'Reentered','1980-02-06 00:00:00','1976-01-10 00:00:00',102.34,689,1050,98.43,'Sun-Synch' from dual union all
select 8577,'008577','1972-058HE','deb Delta 89','',27990,'Reentered','2000-12-06 00:00:00','1993-08-03 00:00:00',96.74,531,675,98.17,'Sun-Synch' from dual union all
select 8578,'008578','1972-058HF','deb Delta 89','',27990,'Reentered','1980-05-01 00:00:00','1980-04-26 00:00:00',92.27,375,397,98.18,'Sun-Synch' from dual union all
select 8579,'008579','1972-058HG','deb Delta 89','',27990,'Reentered','1981-01-24 00:00:00','1976-01-08 00:00:00',100.44,644,915,98.3,'Sun-Synch' from dual union all
select 858,'000858','1964-047A','Syncom 3','Syncom C',10333,'In Earth orbit','','1964-09-25 00:00:00',1435.91,35640,35926,1.04,'Stationary' from dual union all
select 8580,'008580','1972-058HH','deb Delta 89','',27990,'Reentered','1979-11-28 00:00:00','1979-11-04 00:00:00',97.1,548,692,98.29,'Sun-Synch' from dual union all
select 8581,'008581','1972-058HJ','deb Delta 89','',27990,'Reentered','1980-02-08 00:00:00','1976-01-07 00:00:00',100.93,666,940,98.07,'Sun-Synch' from dual union all
select 8582,'008582','1976-003A','Helios 2','Helios 2',38955,'Deep Space','1976-01-15 00:00:00','1976-01-15 00:00:00',41.89,284,21032,30.3,'Earth Escape' from dual union all
select 8583,'008583','1976-003B','Centaur TC-5','Centaur D-1T TC-5',38955,'Deep Space','1976-01-15 00:00:00','1976-01-15 00:00:00',3643.46,1484,171109,28.47,'Earth Escape' from dual union all
select 8584,'008584','1976-003C','Star 37E S/N 40040','Star 37E',38955,'Deep Space','1976-01-15 00:00:00','1976-01-15 00:00:00',41.89,284,21032,30.3,'Earth Escape' from dual union all
select 8585,'008585','1976-004A','Hermes','CTS',38956,'In Earth orbit','','1977-01-20 00:00:00',1436.06,35759,35813,.06,'Stationary' from dual union all
select 8586,'008586','1976-004B','Delta 119','Delta SSPU DSV-3P-4',38956,'Reentered','1976-02-14 00:00:00','1976-01-24 00:00:00',91.78,177,546,28.64,'Intermediate' from dual union all
select 8587,'008587','1976-004C','Delta 119 clamp band','',38956,'Reentered','1976-04-14 00:00:00','1976-01-23 00:00:00',91.49,176,520,28.66,'Intermediate' from dual union all
select 8588,'008588','1976-002D','KDU','KDU',38954,'Reentered','1976-01-27 00:00:00','1976-01-19 00:00:00',89.46,166,330,62.8,'Intermediate' from dual union all
select 8589,'008589','1976-002E','Blenda','Sunshade',38954,'Reentered','1976-01-23 00:00:00','1976-01-20 00:00:00',89.57,162,345,62.8,'Intermediate' from dual union all
select 859,'000859','1964-046F','deb Kosmos-39','',10332,'Reentered','1964-09-06 00:00:00','1964-08-25 00:00:00',93.59,201,700,56.29,'Intermediate' from dual union all
select 8590,'008590','1976-002F','KDU part','KDU part',38954,'Reentered','1976-01-20 00:00:00','1976-01-07 00:00:00',88.93,174,270,62.8,'Intermediate' from dual union all
select 8591,'008591','1976-005A','Kosmos-789','Tsiklon-B',38957,'In Earth orbit','','1976-02-28 00:00:00',104.98,972,1014,82.98,'Intermediate' from dual union all
select 8592,'008592','1974-089DF','deb Delta 104','',33692,'In Earth orbit','','1993-08-04 00:00:00',116.98,1445,1646,101.73,'Sun-Synch' from dual union all
select 8593,'008593','1974-089DG','deb Delta 104','',33692,'In Earth orbit','','1993-07-14 00:00:00',113.31,1204,1553,102,'Sun-Synch' from dual union all
select 8594,'008594','1974-089DH','deb Delta 104','',33692,'In Earth orbit','','1993-08-08 00:00:00',113.25,1280,1472,101.64,'Sun-Synch' from dual union all
select 8595,'008595','1974-089DJ','deb Delta 104','',33692,'In Earth orbit','','1993-08-02 00:00:00',117.57,1453,1691,101.5,'Sun-Synch' from dual union all
select 8596,'008596','1974-089DK','deb Delta 104','',33692,'In Earth orbit','','1993-08-02 00:00:00',113.57,1321,1460,101.8,'Sun-Synch' from dual union all
select 8597,'008597','1976-005B','S3M','S3M',38957,'In Earth orbit','','1991-03-25 00:00:00',104.78,966,1002,82.97,'Intermediate' from dual union all
select 8598,'008598','1976-004D','Star 37E S/N 40032','Star 37E',38956,'Reentered','1977-05-17 00:00:00','1977-08-04 00:00:00',552.5,139,31713,25.49,'Highly Ellip' from dual union all
select 8599,'008599','1974-054C','Star 24','Star 24',33657,'In Earth orbit','','1993-09-29 00:00:00',468.74,13442,13779,125.08,'Medium' from dual union all
select 86,'000086','61 Delta 4','S-56A Canister part','',7263,'Reentered','1962-01-01 00:00:00','1961-03-01 00:00:00',118.31,630,2580,38.86,'Intermediate' from dual union all
select 860,'000860','1964-046G','deb Kosmos-40','',10332,'Reentered','1964-09-06 00:00:00','1964-09-03 00:00:00',94.76,244,770,56.02,'Intermediate' from dual union all
select 8600,'008600','1975-125F','Blok-L','Blok-L',36572,'Reentered','1986-07-30 00:00:00','1976-01-26 00:00:00',731.12,447,40564,62.8,'Molniya' from dual union all
select 8601,'008601','1976-006A','Molniya-1','Molniya-1 11F658 No. 40',38958,'In Earth orbit','','1976-02-25 00:00:00',717.7,534,39817,62.95,'Molniya' from dual union all
select 8602,'008602','1976-006B','11S510','11S510',38958,'Reentered','1976-04-02 00:00:00','1976-02-29 00:00:00',90.89,225,412,62.99,'Intermediate' from dual union all
select 8603,'008603','1976-006C','BOZ','BOZ',38958,'Reentered','1976-04-19 00:00:00','1976-02-22 00:00:00',91.37,234,450,63.05,'Intermediate' from dual union all
select 8604,'008604','1976-007A','Kosmos-790','Tselina-OM',38959,'Reentered','1980-11-12 00:00:00','1976-02-22 00:00:00',95.2,510,546,74.03,'Intermediate' from dual union all
select 8605,'008605','1976-007B','S3M','S3M',38959,'Reentered','1980-12-09 00:00:00','1976-02-26 00:00:00',95,498,540,74.04,'Intermediate' from dual union all
select 8606,'008606','1969-082KK','deb Agena D','',18349,'Reentered','1978-06-28 00:00:00','1978-05-17 00:00:00',91.59,274,432,71,'Intermediate' from dual union all
select 8607,'008607','1976-008A','Kosmos-791','Strela-1M',38960,'In Earth orbit','','1992-02-14 00:00:00',114.74,1401,1487,74.06,'Intermediate' from dual union all
select 8608,'008608','1976-008B','Kosmos-792','Strela-1M',38960,'In Earth orbit','','1992-12-20 00:00:00',115.15,1435,1489,74.05,'Intermediate' from dual union all
select 8609,'008609','1976-008C','Kosmos-793','Strela-1M',38960,'In Earth orbit','','1992-02-14 00:00:00',114.96,1417,1491,74.05,'Intermediate' from dual union all
select 861,'000861','1964-048A','OPS 2739','ARGON 22',10334,'Reentered Attached','1965-03-31 00:00:00','1964-09-20 00:00:00',91.28,329,345,114.99,'Retrograde' from dual union all
select 8610,'008610','1976-008D','Kosmos-794','Strela-1M',38960,'In Earth orbit','','1992-02-14 00:00:00',115.37,1452,1493,74.05,'Intermediate' from dual union all
select 8611,'008611','1976-008E','Kosmos-795','Strela-1M',38960,'In Earth orbit','','1992-02-11 00:00:00',115.59,1467,1498,74.05,'Intermediate' from dual union all
select 8612,'008612','1976-008F','Kosmos-796','Strela-1M',38960,'In Earth orbit','','1992-02-14 00:00:00',115.84,1472,1515,74.05,'Intermediate' from dual union all
select 8613,'008613','1976-008G','Kosmos-797','Strela-1M',38960,'In Earth orbit','','1992-02-16 00:00:00',116.06,1478,1529,74.05,'Intermediate' from dual union all
select 8614,'008614','1976-008H','Kosmos-798','Strela-1M',38960,'In Earth orbit','','1992-02-14 00:00:00',116.33,1478,1553,74.05,'Intermediate' from dual union all
select 8615,'008615','1976-008J','S3M','S3M',38960,'In Earth orbit','','1991-03-20 00:00:00',117.94,1482,1695,74.05,'Intermediate' from dual union all
select 8616,'008616','1976-009A','Kosmos-799','Zenit-2M',38961,'Landed','1976-02-10 00:00:00','1976-01-31 00:00:00',89.59,203,306,71.39,'Intermediate' from dual union all
select 8617,'008617','1976-009B','11A57I','11A57I',38961,'Reentered','1976-02-11 00:00:00','1976-02-01 00:00:00',89.36,215,271,71.42,'Intermediate' from dual union all
select 8618,'008618','1976-009C','Sensor cover','Zenit-2M sensor cover',38961,'Reentered','1976-01-31 00:00:00','1976-01-29 00:00:00',89.1,200,260,71.44,'Intermediate' from dual union all
select 8619,'008619','1976-009D','Sensor cover','Zenit-2M sensor cover',38961,'Reentered','1976-02-09 00:00:00','1976-01-31 00:00:00',89.47,193,304,71.33,'Intermediate' from dual union all
select 862,'000862','1964-047B','Altair 2 RH-67','X-258B2 RH-67',10333,'In Earth orbit','','1964-09-20 00:00:00',694.49,1113,38087,16.8,'GEO Transfer' from dual union all
select 8620,'008620','1976-010A','INTELSAT IVA F2','INTELSAT IVA F2',38962,'In Earth orbit','','1977-01-21 00:00:00',1436.14,35779,35796,.02,'Stationary' from dual union all
select 8621,'008621','1976-010B','Centaur D-1AR AC-37','Centaur D-1AR AC-37',38962,'In Earth orbit','','1976-02-29 00:00:00',655.14,595,36623,21.71,'GEO Transfer' from dual union all
select 8622,'008622','1975-102C','deb Kosmos-777','',36549,'Reentered','1976-12-04 00:00:00','1976-02-01 00:00:00',94.63,443,559,64.98,'Intermediate' from dual union all
select 8623,'008623','1975-102D','deb Kosmos-777','',36549,'Reentered','1976-12-13 00:00:00','1976-11-08 00:00:00',91.56,328,375,65.01,'Intermediate' from dual union all
select 8624,'008624','1975-102E','deb Kosmos-777','',36549,'Reentered','1976-11-15 00:00:00','1976-09-13 00:00:00',90.83,314,317,64.87,'Intermediate' from dual union all
select 8625,'008625','1975-102F','deb Kosmos-777','',36549,'Reentered','1976-03-15 00:00:00','1976-05-02 00:00:00',92.85,396,433,65.03,'Intermediate' from dual union all
select 8626,'008626','1975-102G','deb Kosmos-777','',36549,'Reentered','1977-01-08 00:00:00','1976-02-01 00:00:00',94.68,444,562,64.92,'Intermediate' from dual union all
select 8627,'008627','1975-102H','deb Kosmos-777','',36549,'Reentered','1977-02-23 00:00:00','1976-02-01 00:00:00',92.87,391,440,65.1,'Intermediate' from dual union all
select 8628,'008628','1975-102J','deb Kosmos-777','',36549,'Reentered','1976-11-15 00:00:00','1976-08-09 00:00:00',91.75,337,384,64.8,'Intermediate' from dual union all
select 8629,'008629','1975-102K','deb Kosmos-777','',36549,'Reentered','1976-11-15 00:00:00','1976-02-01 00:00:00',94.75,442,571,64.91,'Intermediate' from dual union all
select 863,'000863','1964-049A','8K78I','8K78I',10335,'Reentered','1964-09-15 00:00:00','1964-08-30 00:00:00',90.59,194,413,64.77,'Intermediate' from dual union all
select 8630,'008630','1975-102L','deb Kosmos-777','',36549,'Reentered','1976-08-19 00:00:00','1976-06-18 00:00:00',91.72,344,374,65.04,'Intermediate' from dual union all
select 8631,'008631','1975-102M','deb Kosmos-777','',36549,'Reentered','1977-08-21 00:00:00','1977-07-26 00:00:00',91.42,344,344,65.83,'Intermediate' from dual union all
select 8632,'008632','1975-102N','deb Kosmos-777','',36549,'Reentered','1977-07-20 00:00:00','1977-06-28 00:00:00',91.17,332,332,65.75,'Intermediate' from dual union all
select 8633,'008633','1975-102P','deb Kosmos-777','',36549,'Reentered','1976-12-09 00:00:00','1976-12-06 00:00:00',88.68,201,217,62.76,'Intermediate' from dual union all
select 8634,'008634','1975-102Q','deb Kosmos-777','',36549,'Reentered','1976-12-04 00:00:00','1976-02-01 00:00:00',94.88,442,583,64.85,'Intermediate' from dual union all
select 8635,'008635','1975-102R','deb Kosmos-777','',36549,'Reentered','1976-11-15 00:00:00','1976-08-23 00:00:00',91.54,331,370,64.93,'Intermediate' from dual union all
select 8636,'008636','1975-102S','deb Kosmos-777','',36549,'Reentered','1977-01-20 00:00:00','1976-02-01 00:00:00',93.15,414,443,65.07,'Intermediate' from dual;
insert into SATELLITE
select 8637,'008637','1975-102T','deb Kosmos-777','',36549,'Reentered','1977-04-04 00:00:00','1977-03-24 00:00:00',91.49,320,376,64.87,'Intermediate' from dual union all
select 8638,'008638','1975-102U','deb Kosmos-777','',36549,'Reentered','1976-03-14 00:00:00','1976-02-23 00:00:00',91.87,258,475,64.48,'Intermediate' from dual union all
select 8639,'008639','1975-102V','deb Kosmos-777','',36549,'Reentered','1976-12-04 00:00:00','1976-08-23 00:00:00',91.74,337,383,64.86,'Intermediate' from dual union all
select 864,'000864','1964-050A','Kosmos-42','Strela-1',10336,'Reentered','1965-12-19 00:00:00','1964-09-21 00:00:00',97.77,228,1076,49,'Intermediate' from dual union all
select 8640,'008640','1975-102W','deb Kosmos-777','',36549,'Reentered','1976-11-15 00:00:00','1976-07-26 00:00:00',91.56,338,365,64.83,'Intermediate' from dual union all
select 8641,'008641','1975-102X','deb Kosmos-777','',36549,'Reentered','1977-06-30 00:00:00','1976-08-03 00:00:00',93.96,430,507,65.8,'Intermediate' from dual union all
select 8642,'008642','1975-102Y','deb Kosmos-777','',36549,'Reentered','1976-11-15 00:00:00','1976-09-27 00:00:00',91.07,319,336,64.9,'Intermediate' from dual union all
select 8643,'008643','1975-102Z','deb Kosmos-777','',36549,'Reentered','1976-12-04 00:00:00','1976-02-01 00:00:00',93.47,436,453,65.08,'Intermediate' from dual union all
select 8644,'008644','1975-102AA','deb Kosmos-777','',36549,'Reentered','1977-01-08 00:00:00','1976-02-01 00:00:00',93.53,433,462,65.14,'Intermediate' from dual union all
select 8645,'008645','1976-011A','Kosmos-800','Zaliv',38963,'In Earth orbit','','1992-12-30 00:00:00',105,978,1011,82.97,'Intermediate' from dual union all
select 8646,'008646','1976-011B','S3M','S3M',38963,'In Earth orbit','','1991-03-28 00:00:00',104.88,985,993,82.97,'Intermediate' from dual union all
select 8647,'008647','1975-102AB','deb Kosmos-777','',36549,'Reentered','1978-06-05 00:00:00','1978-05-18 00:00:00',90.55,292,310,65.02,'Intermediate' from dual union all
select 8648,'008648','1975-102AC','deb Kosmos-777 solar array part','RCS 9m2',36549,'Reentered','1979-04-18 00:00:00','1976-02-27 00:00:00',93.3,431,442,64.99,'Intermediate' from dual union all
select 8649,'008649','1975-102AD','deb Kosmos-777','',36549,'Reentered','1977-06-18 00:00:00','1977-05-14 00:00:00',91.66,356,356,65.85,'Intermediate' from dual union all
select 865,'000865','1964-049B','BOZ','BOZ',10335,'Reentered','1964-09-28 00:00:00','1964-09-01 00:00:00',90.99,197,450,64.74,'Intermediate' from dual union all
select 8650,'008650','1975-102AE','deb Kosmos-777','',36549,'Reentered','1976-03-27 00:00:00','1976-05-02 00:00:00',92.84,407,421,65.03,'Intermediate' from dual union all
select 8651,'008651','1975-102AF','deb Kosmos-777','',36549,'Reentered','1977-03-16 00:00:00','1976-03-05 00:00:00',93.08,411,440,65.09,'Intermediate' from dual union all
select 8652,'008652','1975-102AG','deb Kosmos-777','',36549,'Reentered','1976-11-20 00:00:00','1976-10-04 00:00:00',90.07,278,278,65.05,'Intermediate' from dual union all
select 8653,'008653','1975-102AH','deb Kosmos-777','',36549,'Reentered','1976-12-13 00:00:00','1976-09-17 00:00:00',92.2,357,408,65.09,'Intermediate' from dual union all
select 8654,'008654','1975-102AJ','deb Kosmos-777','',36549,'Reentered','1976-11-15 00:00:00','1976-03-17 00:00:00',94.3,431,539,64.84,'Intermediate' from dual union all
select 8655,'008655','1975-102AK','deb Kosmos-777','',36549,'Reentered','1977-01-26 00:00:00','1976-02-02 00:00:00',93.96,441,496,64.88,'Intermediate' from dual union all
select 8656,'008656','1975-102AL','deb Kosmos-777','',36549,'Reentered','1977-10-30 00:00:00','1977-09-11 00:00:00',91.8,346,380,65.81,'Intermediate' from dual union all
select 8657,'008657','1975-102AM','deb Kosmos-777','',36549,'Reentered','1976-12-13 00:00:00','1976-09-19 00:00:00',91.35,320,361,64.86,'Intermediate' from dual union all
select 8658,'008658','1976-012A','Kosmos-801','DS-P1-I No. 16',38964,'Reentered','1978-01-05 00:00:00','1976-03-06 00:00:00',95.14,265,786,70.98,'Intermediate' from dual union all
select 8659,'008659','1976-012B','S1M','S1M',38964,'Reentered','1977-03-24 00:00:00','1976-03-06 00:00:00',94.93,280,751,71.03,'Intermediate' from dual union all
select 866,'000866','1964-050B','S1','S1',10336,'Reentered','1965-08-02 00:00:00','1964-09-22 00:00:00',97.49,226,1052,48.99,'Intermediate' from dual union all
select 8660,'008660','1976-012C','deb Blok Kosmos R-12','',38964,'Reentered','1977-04-14 00:00:00','1976-03-06 00:00:00',95.86,310,811,71.31,'Intermediate' from dual union all
select 8661,'008661','1976-012D','deb Blok Kosmos R-12','',38964,'Reentered','1976-07-28 00:00:00','1976-02-22 00:00:00',94.01,187,754,70.51,'Intermediate' from dual union all
select 8662,'008662','1976-012E','deb Blok Kosmos R-12','',38964,'Reentered','1976-10-11 00:00:00','1976-02-24 00:00:00',94.15,293,662,70.7,'Intermediate' from dual union all
select 8663,'008663','1976-012F','deb Blok Kosmos R-12','',38964,'Reentered','1976-04-05 00:00:00','1976-02-17 00:00:00',93.82,259,663,70.69,'Intermediate' from dual union all
select 8664,'008664','1976-012G','deb Blok Kosmos R-12','',38964,'Reentered','1976-10-11 00:00:00','1976-02-13 00:00:00',94.66,290,715,70.63,'Intermediate' from dual union all
select 8665,'008665','1976-012H','deb Blok Kosmos R-12','',38964,'Reentered','1976-08-17 00:00:00','1976-03-06 00:00:00',93.93,236,698,70.8,'Intermediate' from dual union all
select 8666,'008666','1976-012J','deb Blok Kosmos R-12','',38964,'Reentered','1976-06-14 00:00:00','1976-03-06 00:00:00',93.78,243,676,70.82,'Intermediate' from dual union all
select 8667,'008667','1976-012K','deb Blok Kosmos R-12','',38964,'Reentered','1976-03-18 00:00:00','1976-03-02 00:00:00',91.23,117,553,70.75,'Intermediate' from dual union all
select 8668,'008668','1976-012L','deb Blok Kosmos R-12','',38964,'Reentered','1976-10-11 00:00:00','1976-03-20 00:00:00',93.73,235,678,70.84,'Intermediate' from dual union all
select 8669,'008669','1976-012M','deb Blok Kosmos R-12','',38964,'Reentered','1976-10-11 00:00:00','1976-03-30 00:00:00',92.23,241,527,70.78,'Intermediate' from dual union all
select 867,'000867','1964-050C','Kosmos-43','Strela-1',10336,'Reentered','1965-12-27 00:00:00','1964-09-22 00:00:00',97.69,231,1066,49,'Intermediate' from dual union all
select 8670,'008670','1976-012N','deb Blok Kosmos R-12','',38964,'Reentered','1976-10-11 00:00:00','1976-02-07 00:00:00',94.49,268,720,70.72,'Intermediate' from dual union all
select 8671,'008671','1976-012P','deb Blok Kosmos R-12','',38964,'Reentered','1976-07-28 00:00:00','1976-02-08 00:00:00',93.6,231,671,70.9,'Intermediate' from dual union all
select 8672,'008672','1976-012Q','deb Blok Kosmos R-12','',38964,'Reentered','1976-10-11 00:00:00','1976-02-08 00:00:00',93.69,213,698,71.08,'Intermediate' from dual union all
select 8673,'008673','1976-012R','deb Blok Kosmos R-12','',38964,'Reentered','1976-09-25 00:00:00','1976-09-22 00:00:00',88.63,189,225,70.82,'Intermediate' from dual union all
select 8674,'008674','1976-012S','deb Blok Kosmos R-12','',38964,'Reentered','1976-06-21 00:00:00','1976-06-17 00:00:00',88.82,196,237,70.67,'Intermediate' from dual union all
select 8675,'008675','1975-004C','deb Delta 107','',36451,'Reentered','1990-09-18 00:00:00','1990-09-16 00:00:00',91.99,363,382,98.33,'Sun-Synch' from dual union all
select 8676,'008676','1975-004D','deb Delta 107','',36451,'Reentered','1979-11-05 00:00:00','1979-10-18 00:00:00',96.47,570,609,97.79,'Sun-Synch' from dual union all
select 8677,'008677','1975-004E','deb Delta 107','',36451,'Reentered','1979-09-14 00:00:00','1979-08-24 00:00:00',94.14,465,489,97.98,'Sun-Synch' from dual union all
select 8678,'008678','1975-004F','deb Delta 107','',36451,'In Earth orbit','','1993-08-05 00:00:00',102.43,740,1008,97.63,'Sun-Synch' from dual union all
select 8679,'008679','1975-004G','Unknown deb                       ','',36451,'In Earth orbit','','1993-08-05 00:00:00',102.36,616,1125,98.7,'Sun-Synch' from dual union all
select 868,'000868','1964-049C','deb Kosmos-41','',10335,'Reentered','1964-08-30 00:00:00','1964-08-25 00:00:00',91.4,157,530,64.82,'Intermediate' from dual union all
select 8680,'008680','1975-004H','deb Delta 107','',36451,'Reentered','1979-02-10 00:00:00','1976-03-10 00:00:00',99.72,651,839,97.99,'Sun-Synch' from dual union all
select 8681,'008681','1976-013A','Kosmos-802','Zenit-4MK',38965,'Landed','1976-02-25 00:00:00','1976-02-13 00:00:00',89.54,171,333,64.99,'Intermediate' from dual union all
select 8682,'008682','1975-004J','deb Delta 107','',36451,'In Earth orbit','','1976-07-07 00:00:00',102.72,745,1030,97.74,'Sun-Synch' from dual union all
select 8683,'008683','1975-004K','deb Delta 107','',36451,'Reentered','1977-04-05 00:00:00','1976-02-29 00:00:00',100.31,754,793,97.76,'Sun-Synch' from dual union all
select 8684,'008684','1975-004L','deb Delta 107','',36451,'Reentered','1979-12-28 00:00:00','1979-12-15 00:00:00',91.7,332,384,97.85,'Sun-Synch' from dual union all
select 8685,'008685','1975-004M','deb Delta 107','',36451,'In Earth orbit','','1976-02-11 00:00:00',101.12,727,897,97.75,'Sun-Synch' from dual union all
select 8686,'008686','1976-013B','11A57I','11A57I',38965,'Reentered','1976-02-17 00:00:00','1976-02-14 00:00:00',89,167,284,64.98,'Intermediate' from dual union all
select 8687,'008687','1975-004N','deb Delta 107','',36451,'Reentered','1977-02-18 00:00:00','1977-02-16 00:00:00',93.27,377,492,96.68,'Sun-Synch' from dual union all
select 8688,'008688','1976-014A','Kosmos-803','Lira',38966,'In Earth orbit','','1976-03-13 00:00:00',96.36,550,619,65.85,'Intermediate' from dual union all
select 8689,'008689','1976-014B','S3M','S3M',38966,'Reentered','1983-11-09 00:00:00','1976-03-13 00:00:00',96.27,544,616,65.86,'Intermediate' from dual union all
select 869,'000869','1964-049D','Kosmos-41','Molniya-1 11F67 No. 1',10335,'Reentered','2004-04-09 00:00:00','1964-09-22 00:00:00',714.5,424,39769,64.87,'GEO Transfer' from dual union all
select 8690,'008690','1976-014C','deb Kosmos-803','DS-P1-M part [RCS 0.48]',38966,'Reentered','1981-12-09 00:00:00','1976-03-19 00:00:00',96.35,550,617,65.86,'Intermediate' from dual union all
select 8691,'008691','1975-004P','deb Delta 107','',36451,'Reentered','1982-05-22 00:00:00','1976-02-13 00:00:00',100.95,726,882,97.75,'Sun-Synch' from dual union all
select 8692,'008692','1975-004Q','deb Delta 107','',36451,'Reentered','1989-08-28 00:00:00','1989-08-27 00:00:00',88.26,177,200,97.88,'Sun-Synch' from dual union all
select 8693,'008693','1975-004R','deb Delta 107','',36451,'Reentered','1981-08-29 00:00:00','1976-02-15 00:00:00',101.78,743,943,97.8,'Sun-Synch' from dual union all
select 8694,'008694','1976-015A','Kosmos-804','IS',38967,'Deorbited','1976-02-16 00:00:00','1976-02-16 00:00:00',93.06,148,701,65.15,'Intermediate' from dual union all
select 8695,'008695','1976-015B','11S692','11S692',38967,'Reentered','1976-02-26 00:00:00','1976-02-18 00:00:00',92.01,140,606,65.12,'Intermediate' from dual union all
select 8696,'008696','1976-016A','DMSP 11534','DMSP Block 5C F-3',38968,'Reentered','1976-02-19 00:00:00','1976-02-19 00:00:00',88.9,81,359,98.87,'Sun-Synch' from dual union all
select 8697,'008697','1976-017A','Marisat 101','Marisat A',38969,'In Earth orbit','','1977-01-25 00:00:00',1436.06,35743,35829,1.8,'Stationary' from dual union all
select 8698,'008698','1976-017B','Delta 120','Delta SSPU DSV-3P-4',38969,'Reentered','1976-03-16 00:00:00','1976-02-26 00:00:00',91.74,178,542,28.58,'Intermediate' from dual union all
select 8699,'008699','1976-018A','Kosmos-805','Yantar''-2K No. 4',38970,'Landed','1976-03-11 00:00:00','1976-02-27 00:00:00',89.43,169,323,67.13,'Intermediate' from dual union all
select 87,'000087','61 Eta','Transit 3B','Transit 3B',7265,'Reentered Attached','1961-03-30 00:00:00','1961-02-22 00:00:00',96.36,167,1002,28.38,'Intermediate' from dual union all
select 870,'000870','1964-051A','Explorer 20','NASA S-48',10337,'In Earth orbit','','1964-10-08 00:00:00',103.91,869,1018,79.9,'Intermediate' from dual union all
select 8700,'008700','1976-018B','11S510','11S510',38970,'Reentered','1976-02-27 00:00:00','1976-02-22 00:00:00',89.4,170,320,67.13,'Intermediate' from dual union all
select 8701,'008701','1976-006D','Blok-L','Blok-L',38958,'Reentered','2017-05-05 00:00:00','1976-02-27 00:00:00',695.42,508,38738,62.97,'Molniya' from dual union all
select 8702,'008702','1976-017C','Star 37E S/N 40041','Star 37E',38969,'Reentered','1982-11-27 00:00:00','1978-01-02 00:00:00',429.39,191,24760,24.51,'Highly Ellip' from dual union all
select 8703,'008703','1976-018C','deb Kosmos-805','Feniks part',38970,'Reentered','1976-02-22 00:00:00','1976-02-20 00:00:00',89.09,139,320,67.12,'Intermediate' from dual union all
select 8704,'008704','1976-017D','Delta 120 clamp band','',38969,'Reentered','1976-03-02 00:00:00','1976-02-23 00:00:00',91.84,179,551,28.59,'Intermediate' from dual union all
select 8705,'008705','1976-017E','Delta 120 clamp band','',38969,'Reentered','1976-02-29 00:00:00','1976-02-21 00:00:00',92.32,176,601,28.59,'Intermediate' from dual union all
select 8706,'008706','1976-013C','Blenda','Sunshade',38965,'Reentered','1976-02-25 00:00:00','1976-02-22 00:00:00',89.11,171,289,64.96,'Intermediate' from dual union all
select 8707,'008707','1976-013D','KDU','KDU',38965,'Reentered','1976-02-29 00:00:00','1976-02-22 00:00:00',89.24,163,311,64.92,'Intermediate' from dual union all
select 8708,'008708','1976-013E','KDU part','KDU part',38965,'Reentered','1976-02-26 00:00:00','1976-02-25 00:00:00',88.77,162,266,64.98,'Intermediate' from dual union all
select 8709,'008709','1976-019A','Ume','ISS',38971,'In Earth orbit','','1990-06-27 00:00:00',105.08,991,1005,69.67,'Intermediate' from dual union all
select 871,'000871','1964-051B','Altair 2 X-258C1 RH-68','X-258C1',10337,'In Earth orbit','','1964-10-12 00:00:00',103.85,869,1011,79.91,'Intermediate' from dual union all
select 8710,'008710','1976-019B','Star 37N S/N 14003','Star 37N',38971,'In Earth orbit','','1990-06-19 00:00:00',105.16,993,1011,69.67,'Intermediate' from dual union all
select 8711,'008711','1975-102AN','deb Kosmos-777','',36549,'Reentered','1976-12-14 00:00:00','1976-10-12 00:00:00',91.64,338,372,64.96,'Intermediate' from dual union all
select 8712,'008712','1975-102AP','deb Kosmos-777 solar array part','RCS 9m2',36549,'Reentered','1977-05-01 00:00:00','1976-03-01 00:00:00',94.47,439,547,64.9,'Intermediate' from dual union all
select 8713,'008713','1975-102AQ','deb Kosmos-777','',36549,'Reentered','1976-04-01 00:00:00','1976-05-04 00:00:00',93.25,404,463,64.9,'Intermediate' from dual union all
select 8714,'008714','1975-102AR','deb Kosmos-777','',36549,'Reentered','1976-11-20 00:00:00','1976-06-15 00:00:00',91.4,331,356,64.99,'Intermediate' from dual union all
select 8715,'008715','1975-102AS','deb Kosmos-777','',36549,'Reentered','1977-02-09 00:00:00','1977-01-29 00:00:00',91.51,322,376,64.85,'Intermediate' from dual union all
select 8716,'008716','1975-102AT','deb Kosmos-777','',36549,'Reentered','1976-12-04 00:00:00','1976-09-19 00:00:00',91.97,338,405,64.9,'Intermediate' from dual union all
select 8717,'008717','1975-102AU','deb Kosmos-777','',36549,'Reentered','1977-02-23 00:00:00','1976-02-29 00:00:00',92.85,404,425,65.01,'Intermediate' from dual union all
select 8718,'008718','1975-102AV','deb Kosmos-777','',36549,'Reentered','1976-12-13 00:00:00','1976-04-23 00:00:00',94.07,423,524,64.94,'Intermediate' from dual union all
select 8719,'008719','1975-102AW','deb Kosmos-777','',36549,'Reentered','1976-12-13 00:00:00','1976-10-24 00:00:00',91.6,340,367,64.92,'Intermediate' from dual union all
select 872,'000872','1964-052A','Nimbus 1','Nimbus A',10338,'Reentered','1974-05-16 00:00:00','1964-09-28 00:00:00',98.36,426,935,98.67,'Sun-Synch' from dual union all
select 8720,'008720','1975-102AX','deb Kosmos-777','',36549,'Reentered','1977-02-25 00:00:00','1977-01-30 00:00:00',92.15,345,414,65,'Intermediate' from dual union all
select 8721,'008721','1975-102AY','deb Kosmos-777','',36549,'Reentered','1976-03-03 00:00:00','1976-02-25 00:00:00',91.52,218,481,65.05,'Intermediate' from dual union all
select 8722,'008722','1975-102AZ','deb Kosmos-777','',36549,'Reentered','1976-12-07 00:00:00','1976-11-18 00:00:00',92.06,341,410,64.94,'Intermediate' from dual union all
select 8723,'008723','1975-102BA','deb Kosmos-777','',36549,'Reentered','1977-04-14 00:00:00','1977-04-21 00:00:00',93.48,408,482,65.8,'Intermediate' from dual union all
select 8724,'008724','1975-102BB','deb Kosmos-777','',36549,'Reentered','1976-11-15 00:00:00','1976-09-15 00:00:00',91.07,214,441,66.23,'Intermediate' from dual union all
select 8725,'008725','1975-102BC','deb Kosmos-777','',36549,'Reentered','1976-04-02 00:00:00','1976-05-06 00:00:00',93.97,431,507,64.95,'Intermediate' from dual union all
select 8726,'008726','1975-102BD','deb Kosmos-777','',36549,'Reentered','1976-04-20 00:00:00','1976-04-12 00:00:00',91.04,317,334,65.05,'Intermediate' from dual;
insert into SATELLITE
select 8727,'008727','1975-102BE','deb Kosmos-777','',36549,'Reentered','1976-11-15 00:00:00','1976-09-22 00:00:00',91.68,331,383,64.89,'Intermediate' from dual union all
select 8728,'008728','1975-102BF','deb Kosmos-777','',36549,'Reentered','1976-03-29 00:00:00','1976-05-02 00:00:00',93.48,407,482,64.83,'Intermediate' from dual union all
select 8729,'008729','1975-102BG','deb Kosmos-777','',36549,'Reentered','1977-01-08 00:00:00','1976-02-29 00:00:00',93.85,435,490,64.88,'Intermediate' from dual union all
select 873,'000873','1964-051C','deb Explorer XX','',10337,'Reentered','1968-04-18 00:00:00','1964-09-24 00:00:00',103.7,873,994,79.84,'Intermediate' from dual union all
select 8730,'008730','1975-102BH','deb Kosmos-777','',36549,'Reentered','1976-12-13 00:00:00','1976-09-22 00:00:00',92.09,351,403,64.88,'Intermediate' from dual union all
select 8731,'008731','1975-102BJ','deb Kosmos-777','',36549,'Reentered','1977-11-05 00:00:00','1976-04-12 00:00:00',92.92,405,431,65.02,'Intermediate' from dual union all
select 8732,'008732','1975-102BK','deb Kosmos-777','',36549,'Reentered','1976-09-08 00:00:00','1976-08-20 00:00:00',90.22,267,303,65.16,'Intermediate' from dual union all
select 8733,'008733','1975-102BL','deb Kosmos-777','',36549,'Reentered','1977-01-08 00:00:00','1976-03-07 00:00:00',93.81,434,488,65.06,'Intermediate' from dual union all
select 8734,'008734','1975-102BM','deb Kosmos-777','',36549,'Reentered','1977-01-27 00:00:00','1976-03-07 00:00:00',94.1,438,512,65.12,'Intermediate' from dual union all
select 8735,'008735','1975-102BN','deb Kosmos-777','',36549,'Reentered','1976-04-25 00:00:00','1976-04-01 00:00:00',91.51,255,443,65.13,'Intermediate' from dual union all
select 8736,'008736','1975-102BP','deb Kosmos-777','',36549,'Reentered','1977-06-09 00:00:00','1976-03-04 00:00:00',92.81,388,437,65.11,'Intermediate' from dual union all
select 8737,'008737','1976-020A','Kosmos-806','Zenit-4MK',38972,'Landed','1976-03-23 00:00:00','1976-03-12 00:00:00',89.59,177,332,71.37,'Intermediate' from dual union all
select 8738,'008738','1976-020B','11S510I','11S510I',38972,'Reentered','1976-03-15 00:00:00','1976-03-12 00:00:00',89.17,169,298,71.39,'Intermediate' from dual union all
select 8739,'008739','1974-101E','deb Delta 106','',33704,'Reentered','1977-09-08 00:00:00','1976-03-06 00:00:00',97.07,280,957,26.95,'Intermediate' from dual union all
select 874,'000874','1964-051D','deb Explorer XX','',10337,'Reentered','1968-06-17 00:00:00','1964-09-28 00:00:00',103.75,872,1000,79.83,'Intermediate' from dual union all
select 8740,'008740','1974-101F','deb Delta 106','',33704,'Reentered','1977-12-25 00:00:00','1976-03-10 00:00:00',97.79,285,1021,27.04,'Intermediate' from dual union all
select 8741,'008741','1976-021A','Molniya-1','Molniya-1',38973,'Reentered','1990-10-10 00:00:00','1976-04-11 00:00:00',717.78,486,39869,62.84,'Molniya' from dual union all
select 8742,'008742','1976-021B','11S510','11S510',38973,'Reentered','1976-04-13 00:00:00','1976-03-18 00:00:00',91.15,202,460,62.85,'Intermediate' from dual union all
select 8743,'008743','1976-021C','BOZ','BOZ',38973,'Reentered','1976-04-05 00:00:00','1976-03-20 00:00:00',90.99,192,455,62.83,'Intermediate' from dual union all
select 8744,'008744','1976-022A','Kosmos-807','Vektor',38974,'In Earth orbit','','1976-04-11 00:00:00',109.05,395,1971,82.96,'Intermediate' from dual union all
select 8745,'008745','1976-022B','S3M','S3M',38974,'Reentered','2005-06-27 00:00:00','1990-06-21 00:00:00',103.31,379,1452,82.94,'Intermediate' from dual union all
select 8746,'008746','1976-023A','LES 8','LES 8',38975,'In Earth orbit','','1977-01-24 00:00:00',1435.83,35751,35812,25.21,'Synchronous' from dual union all
select 8747,'008747','1976-023B','LES 9','LES 9',38975,'In Earth orbit','','1977-01-21 00:00:00',1436.19,35774,35803,25.2,'Synchronous' from dual union all
select 8748,'008748','1976-023C','SR 11A','SR 11A',38975,'In Earth orbit','','1976-03-18 00:00:00',7344.39,118383,119180,25.7,'Very High' from dual union all
select 8749,'008749','1976-023D','SR 11B','SR 11B',38975,'In Earth orbit','','1976-03-18 00:00:00',7116.82,115720,116645,25.6,'Very High' from dual union all
select 875,'000875','1964-051E','deb Explorer XX','',10337,'Reentered','1968-09-16 00:00:00','1964-09-29 00:00:00',103.81,864,1014,79.82,'Intermediate' from dual union all
select 8750,'008750','1976-023E','Titan 23C-12 Stage 2','LR91AJ11 23C-12',38975,'Reentered','1976-03-15 00:00:00','1976-03-15 00:00:00',87.6,150,162,28.6,'Intermediate' from dual union all
select 8751,'008751','1976-023F','Transtage 30','Transtage 30',38975,'In Earth orbit','','1976-12-03 00:00:00',1465.43,35743,36975,25.38,'Near-synch' from dual union all
select 8752,'008752','1976-023G','SR 11 PKM','Star 17A',38975,'In Earth orbit','','1976-03-16 00:00:00',3327.3,36599,98300,25.2,'Very High' from dual union all
select 8753,'008753','1976-023H','SR 11 AKM','Star 17',38975,'In Earth orbit','','1976-03-21 00:00:00',7284.49,117000,119200,25.7,'Very High' from dual union all
select 8754,'008754','1976-024A','Kosmos-808','Tselina-D',38976,'Reentered','1993-11-20 00:00:00','1991-03-28 00:00:00',94.63,490,512,81.22,'Intermediate' from dual union all
select 8755,'008755','1976-024B','8A92ME','8A92ME',38976,'Reentered','1999-05-08 00:00:00','1991-03-28 00:00:00',94.87,486,538,81.25,'Intermediate' from dual union all
select 8756,'008756','1974-071C','deb Kosmos-676','Strela-2M debris 1',33674,'In Earth orbit','','1993-08-16 00:00:00',99.55,732,742,74.06,'Intermediate' from dual union all
select 8757,'008757','1975-094D','deb Kosmos-773','Strela-2M debris 2',36541,'In Earth orbit','','1993-08-17 00:00:00',99.3,723,728,74.04,'Intermediate' from dual union all
select 8758,'008758','1976-025A','Kosmos-809','Zenit-2M',38977,'Landed','1976-03-30 00:00:00','1976-03-20 00:00:00',89.52,203,299,65.03,'Intermediate' from dual union all
select 8759,'008759','1976-025B','11S510I','11S510I',38977,'Reentered','1976-03-26 00:00:00','1976-03-20 00:00:00',89.24,195,279,65.03,'Intermediate' from dual union all
select 876,'000876','1964-053A','Kosmos-44','Meteor 11F614 No. 1',10339,'In Earth orbit','','1964-10-09 00:00:00',99.5,607,863,65.08,'Intermediate' from dual union all
select 8760,'008760','1976-025C','Sensor cover','Zenit-2M sensor cover',38977,'Reentered','1976-03-19 00:00:00','1976-03-18 00:00:00',89.14,197,267,65.05,'Intermediate' from dual union all
select 8761,'008761','1976-025D','Sensor cover','Zenit-2M sensor cover',38977,'Reentered','1976-03-19 00:00:00','1976-03-18 00:00:00',90.04,199,354,65.12,'Intermediate' from dual union all
select 8762,'008762','1976-026A','Molniya-1','Molniya-1 11F658 No. 42',38978,'Reentered','1985-05-15 00:00:00','1976-04-22 00:00:00',717.76,632,39722,63.01,'Molniya' from dual union all
select 8763,'008763','1976-026B','11S510','11S510',38978,'Reentered','1976-04-27 00:00:00','1976-03-26 00:00:00',90.86,228,406,63,'Intermediate' from dual union all
select 8764,'008764','1970-102E','deb Kosmos-381','',21579,'In Earth orbit','','1993-08-16 00:00:00',104.2,940,973,74.03,'Intermediate' from dual union all
select 8765,'008765','1975-115E','deb Interkosmos-14','',36562,'Reentered','1981-04-13 00:00:00','1976-01-31 00:00:00',107.4,411,1801,72.4,'Intermediate' from dual union all
select 8766,'008766','1975-115F','deb Interkosmos-14','',36562,'Reentered','1978-05-29 00:00:00','1976-01-31 00:00:00',106.29,375,1734,72.72,'Intermediate' from dual union all
select 8767,'008767','1976-026C','BOZ','BOZ',38978,'Reentered','1976-05-07 00:00:00','1976-03-28 00:00:00',91.5,215,482,63.02,'Intermediate' from dual union all
select 8768,'008768','1976-020C','KDU','KDU',38972,'Reentered','1976-03-30 00:00:00','1976-03-22 00:00:00',89.69,174,345,71.37,'Intermediate' from dual union all
select 8769,'008769','1976-020D','Blenda','Sunshade',38972,'Reentered','1976-03-26 00:00:00','1976-03-22 00:00:00',89.48,173,325,71.37,'Intermediate' from dual union all
select 877,'000877','1964-053B','8A92E','8A92E',10339,'In Earth orbit','','1964-10-11 00:00:00',99.58,686,791,65.09,'Intermediate' from dual union all
select 8770,'008770','1976-027A','OPS 7600','GAMBIT-3 Mission 4346',38979,'Reentered Attached','1976-05-18 00:00:00','1976-03-29 00:00:00',89.19,124,345,96.4,'Sun-Synch' from dual union all
select 8771,'008771','1976-020E','KDU part','KDU part',38972,'Reentered','1976-03-25 00:00:00','1976-03-23 00:00:00',89.62,171,341,71.38,'Intermediate' from dual union all
select 8772,'008772','1976-028A','Kosmos-810','Zenit-4MK',38980,'Landed','1976-04-08 00:00:00','1976-03-28 00:00:00',89.36,169,316,62.81,'Intermediate' from dual union all
select 8773,'008773','1976-028B','11A57I','11A57I',38980,'Reentered','1976-04-01 00:00:00','1976-03-28 00:00:00',89.18,173,295,62.8,'Intermediate' from dual union all
select 8774,'008774','1976-029A','Satcom 2','RCA-B',38981,'In Earth orbit','','1977-01-24 00:00:00',1436.06,35711,35860,.01,'Stationary' from dual union all
select 8775,'008775','1976-029B','Delta 121','Delta SSPU DSV-3P-4',38981,'Reentered','1976-12-02 00:00:00','1976-04-25 00:00:00',105.35,183,1839,28.38,'Intermediate' from dual union all
select 8776,'008776','1975-109F','Tselina-O part','[UNID069] RCS 0.0',36556,'Reentered','1977-11-18 00:00:00','1976-03-28 00:00:00',94.66,482,523,74.03,'Intermediate' from dual union all
select 8777,'008777','1975-109G','deb Kosmos-781','Tselina-O part',36556,'Reentered','1978-07-18 00:00:00','1976-03-26 00:00:00',94.71,476,534,74.1,'Intermediate' from dual union all
select 8778,'008778','1974-035H','deb Kosmos-655','Tselina-O part',33638,'Reentered','1978-01-11 00:00:00','1976-03-21 00:00:00',94.98,496,540,73.95,'Intermediate' from dual union all
select 8779,'008779','1971-075C','deb Kosmos-437','Tselina-O part',24893,'Reentered','1977-02-23 00:00:00','1976-03-24 00:00:00',93.74,444,471,74.11,'Intermediate' from dual union all
select 878,'000878','1964-052B','Agena B 6201','Agena B  6201',10338,'Reentered','1974-08-13 00:00:00','1964-10-10 00:00:00',98.36,424,937,98.68,'Sun-Synch' from dual union all
select 8780,'008780','1968-095G','Tselina-O part','[UNID070]',16822,'Reentered','1976-10-17 00:00:00','1976-09-16 00:00:00',92.49,325,469,74.11,'Intermediate' from dual union all
select 8781,'008781','1976-030A','Kosmos-811','Orion',38982,'Landed','1976-04-12 00:00:00','1976-04-02 00:00:00',89.89,204,334,72.85,'Intermediate' from dual union all
select 8782,'008782','1976-030B','11S510','11S510',38982,'Reentered','1976-04-10 00:00:00','1976-04-03 00:00:00',89.51,194,307,72.86,'Intermediate' from dual union all
select 8783,'008783','1976-030C','Sensor cover','Zenit-4MT sensor cover',38982,'Reentered','1976-04-13 00:00:00','1976-04-02 00:00:00',89.47,204,293,72.86,'Intermediate' from dual union all
select 8784,'008784','1974-089DL','deb Delta 104','',33692,'In Earth orbit','','1975-11-01 00:00:00',118.88,1449,1812,100.75,'Sun-Synch' from dual union all
select 8785,'008785','1974-089DM','deb Delta 104','',33692,'In Earth orbit','','1993-08-06 00:00:00',119.12,1441,1842,100.99,'Sun-Synch' from dual union all
select 8786,'008786','1974-089DN','deb Delta 104','',33692,'In Earth orbit','','1993-08-04 00:00:00',115.84,1451,1536,101.99,'Sun-Synch' from dual union all
select 8787,'008787','1974-089DP','deb Delta 104','',33692,'Reentered','1977-08-24 00:00:00','1976-04-01 00:00:00',121.14,944,2520,101.33,'Medium' from dual union all
select 8788,'008788','1974-089DQ','deb Delta 104','',33692,'In Earth orbit','','1993-08-01 00:00:00',116.23,1456,1567,102.02,'Sun-Synch' from dual union all
select 8789,'008789','1974-089DR','deb Delta 104','',33692,'In Earth orbit','','1993-08-02 00:00:00',116.23,1428,1594,102.31,'Sun-Synch' from dual union all
select 879,'000879','1964-054A','OGO 1','OGO A',10340,'Deep Space','1964-09-06 00:00:00','1964-10-25 00:00:00',3839.93,1288,148412,32.34,'Deep Space' from dual union all
select 8790,'008790','1974-089DS','deb Delta 104','',33692,'In Earth orbit','','1993-08-02 00:00:00',114.47,1393,1470,101.81,'Sun-Synch' from dual union all
select 8791,'008791','1966-087F','deb ESSA 3','[UNID071] RCS 0.01',13494,'In Earth orbit','','1975-10-01 00:00:00',114.62,1324,1553,101.53,'Sun-Synch' from dual union all
select 8792,'008792','1976-026D','Blok-L','Blok-L',38978,'Reentered','1978-04-02 00:00:00','1976-04-21 00:00:00',696.64,454,38853,62.98,'Molniya' from dual union all
select 8793,'008793','1976-029C','Star 37E S/N 40043','Star 37E',38981,'Reentered','1978-06-28 00:00:00','1976-07-04 00:00:00',625.78,241,35473,27.1,'GEO Transfer' from dual union all
select 8794,'008794','1976-031A','Kosmos-812','Tselina-OM',38983,'Reentered','1980-10-30 00:00:00','1976-05-13 00:00:00',95.16,507,546,74.03,'Intermediate' from dual union all
select 8795,'008795','1976-031B','S3M','S3M',38983,'Reentered','1981-01-05 00:00:00','1976-05-09 00:00:00',95.05,497,544,74.04,'Intermediate' from dual union all
select 8796,'008796','1976-028C','Blenda','Sunshade',38980,'Reentered','1976-04-07 00:00:00','1976-04-06 00:00:00',88.86,163,274,62.8,'Intermediate' from dual union all
select 8797,'008797','1976-028D','KDU','KDU',38980,'Reentered','1976-04-10 00:00:00','1976-04-06 00:00:00',89.07,160,297,62.71,'Intermediate' from dual union all
select 8798,'008798','1976-028E','KDU part','KDU part',38980,'Reentered','1976-04-07 00:00:00','1976-04-06 00:00:00',87.9,133,209,62.83,'Intermediate' from dual union all
select 8799,'008799','1976-032A','Meteor','Meteor No. 37',38984,'In Earth orbit','','1992-12-23 00:00:00',102.09,832,883,81.26,'Intermediate' from dual union all
select 88,'000088','61 Epsilon 2','Star 12','Star 12',7264,'Reentered','1961-03-30 00:00:00','1961-02-25 00:00:00',94.79,283,734,80.91,'Intermediate' from dual union all
select 880,'000880','1964-055A','Kosmos-45','Zenit-4 No. 5',10341,'Landed','1964-09-18 00:00:00','1964-09-14 00:00:00',89.66,200,316,64.91,'Intermediate' from dual union all
select 8800,'008800','1976-032B','8A92ME','8A92ME',38984,'In Earth orbit','','1993-08-15 00:00:00',102.2,793,932,81.25,'Intermediate' from dual union all
select 8801,'008801','1976-033A','Kosmos-813','Zenit-2M',38985,'Landed','1976-04-21 00:00:00','1976-04-11 00:00:00',88.88,206,233,81.35,'Intermediate' from dual union all
select 8802,'008802','1976-033B','11A57I','11A57I',38985,'Reentered','1976-04-14 00:00:00','1976-04-11 00:00:00',88.68,196,222,81.34,'Intermediate' from dual union all
select 8803,'008803','1976-030D','KBE part','KBE part',38982,'Reentered','1976-04-14 00:00:00','1976-04-12 00:00:00',89.68,201,317,72.85,'Intermediate' from dual union all
select 8804,'008804','1976-030E','KB electropitaniya','KBE',38982,'Reentered','1976-04-15 00:00:00','1976-04-12 00:00:00',89.69,201,318,72.85,'Intermediate' from dual union all
select 8805,'008805','1976-030F','KBE part','KBE part',38982,'Reentered','1976-04-15 00:00:00','1976-04-12 00:00:00',89.69,205,314,72.81,'Intermediate' from dual union all
select 8806,'008806','1976-034A','Kosmos-814','IS',38986,'Deorbited','1976-04-13 00:00:00','1976-04-13 00:00:00',90.46,116,479,65.07,'Intermediate' from dual union all
select 8807,'008807','1976-034B','11S692','11S692',38986,'Reentered','1976-04-16 00:00:00','1976-04-15 00:00:00',89.1,136,324,65.11,'Intermediate' from dual union all
select 8808,'008808','1976-035A','NATO 3A','NATO 3A',38987,'In Earth orbit','','1977-01-25 00:00:00',1436.09,35772,35801,2.53,'Stationary' from dual union all
select 8809,'008809','1976-035B','Delta 122','Delta SSPU DSV-3P-4',38987,'Reentered','1976-05-30 00:00:00','1976-05-02 00:00:00',92.73,177,640,28.22,'Intermediate' from dual union all
select 881,'000881','1964-055B','11A57I','11A57I',10341,'Reentered','1964-09-27 00:00:00','1964-09-15 00:00:00',89.55,201,304,64.89,'Intermediate' from dual union all
select 8810,'008810','1976-035C','Star 37E S/N 40037','Star 37E',38987,'Reentered','1977-10-18 00:00:00','1976-05-25 00:00:00',630.42,200,35753,26.91,'GEO Transfer' from dual union all
select 8811,'008811','1976-036A','Kosmos-815','Zenit-4MK',38988,'Landed','1976-05-11 00:00:00','1976-04-30 00:00:00',88.94,215,230,81.32,'Intermediate' from dual union all
select 8812,'008812','1976-037A','Kosmos-816','Romb 1',38989,'Reentered','1979-11-24 00:00:00','1976-05-29 00:00:00',94.54,478,515,65.83,'Intermediate' from dual union all
select 8813,'008813','1976-036B','11A57I','11A57I',38988,'Reentered','1976-05-02 00:00:00','1976-05-01 00:00:00',88.35,188,198,81.31,'Intermediate' from dual union all
select 8814,'008814','1976-037B','S3M','S3M',38989,'Reentered','1979-11-16 00:00:00','1976-05-29 00:00:00',94.44,468,515,65.84,'Intermediate' from dual union all
select 8815,'008815','1976-036C','Sensor cover','Zenit-4MK sensor cover',38988,'Reentered','1976-05-01 00:00:00','1976-04-28 00:00:00',88.48,194,205,81.31,'Intermediate' from dual union all
select 8816,'008816','1976-036D','Sensor cover','Zenit-4MK sensor cover',38988,'Reentered','1976-05-01 00:00:00','1976-04-29 00:00:00',88.73,208,216,81.29,'Intermediate' from dual;
insert into SATELLITE
select 8817,'008817','1976-036E','Sensor cover','Zenit-4MK sensor cover',38988,'Reentered','1976-05-03 00:00:00','1976-04-29 00:00:00',89.38,215,273,81.32,'Intermediate' from dual union all
select 8818,'008818','1976-038A','OPS 6431','PARCAE 1',38990,'In Earth orbit','','1976-12-30 00:00:00',107.47,1080,1140,63.44,'Intermediate' from dual union all
select 8819,'008819','1976-038B','MSD plume shield','',38990,'In Earth orbit','','1976-12-31 00:00:00',107.37,1079,1130,63.46,'Intermediate' from dual union all
select 882,'000882','1964-056A','OPS 3497','CORONA J-11',10342,'Reentered Attached','1964-10-06 00:00:00','1964-09-25 00:00:00',90.65,171,442,84.97,'Intermediate' from dual union all
select 8820,'008820','1976-039A','Lageos','Lageos',38991,'In Earth orbit','','1976-06-03 00:00:00',225.45,5834,5951,109.89,'Medium' from dual union all
select 8821,'008821','1976-039B','Star 37D S/N 028','Star 37D',38991,'Reentered','1991-09-22 00:00:00','1990-02-19 00:00:00',109.3,287,2102,109.59,'Retrograde' from dual union all
select 8822,'008822','1976-039C','Lageos AKM','Star 24',38991,'In Earth orbit','','1993-08-14 00:00:00',225.38,5837,5943,109.85,'Medium' from dual union all
select 8823,'008823','1976-040A','Kosmos-817','Zenit-4MK',38992,'Landed','1976-05-18 00:00:00','1976-05-07 00:00:00',89.42,172,320,64.97,'Intermediate' from dual union all
select 8824,'008824','1976-040B','11A57I','11A57I',38992,'Reentered','1976-05-10 00:00:00','1976-05-07 00:00:00',88.87,167,270,65.01,'Intermediate' from dual union all
select 8825,'008825','1976-040C','Sensor cover','Zenit-4MK sensor cover',38992,'Reentered','1976-05-06 00:00:00','1976-05-05 00:00:00',88.74,158,267,65,'Intermediate' from dual union all
select 8826,'008826','1971-120C','deb Meteor','',24938,'In Earth orbit','','1993-08-16 00:00:00',100.83,786,809,81.24,'Intermediate' from dual union all
select 8827,'008827','1971-120D','deb Meteor','',24938,'In Earth orbit','','1993-08-15 00:00:00',101.95,842,860,81.27,'Intermediate' from dual union all
select 8828,'008828','1970-106C','deb NOAA 1','[UNID072] RCS 0.01',21583,'In Earth orbit','','1993-08-13 00:00:00',116.39,1494,1543,102.37,'Sun-Synch' from dual union all
select 8829,'008829','1974-071D','deb Kosmos-676','Strela-2M debris 2',33674,'In Earth orbit','','1993-08-15 00:00:00',100.17,753,780,74.07,'Intermediate' from dual union all
select 883,'000883','1964-057A','Apollo BP-15','Apollo BP-15 CM',10343,'Reentered Attached','1964-09-22 00:00:00','1964-09-20 00:00:00',88.32,177,206,31.73,'Intermediate' from dual union all
select 8830,'008830','1976-036F','KDU','KDU',38988,'Reentered','1976-05-20 00:00:00','1976-05-10 00:00:00',88.78,208,221,81.3,'Intermediate' from dual union all
select 8831,'008831','1976-036G','Blenda','Sunshade',38988,'Reentered','1976-05-13 00:00:00','1976-05-10 00:00:00',88.59,192,218,81.31,'Intermediate' from dual union all
select 8832,'008832','1976-023J','P74-1 Upper Truss Assembly','',38975,'In Earth orbit','','1992-06-03 00:00:00',1465.46,35709,37009,18.63,'Inclined Drift' from dual union all
select 8833,'008833','1976-041A','Molniya-3','Molniya-3 No. 16',38993,'Reentered','1990-02-04 00:00:00','1976-06-13 00:00:00',717.72,614,39738,62.95,'Molniya' from dual union all
select 8834,'008834','1976-041B','11S510','11S510',38993,'Reentered','1976-08-05 00:00:00','1976-06-12 00:00:00',91.79,211,514,62.84,'Intermediate' from dual union all
select 8835,'008835','1976-038C','SSU 1','SSU',38990,'In Earth orbit','','1977-01-02 00:00:00',107.48,1087,1134,63.44,'Intermediate' from dual union all
select 8836,'008836','1976-038D','SSU 3','SSU',38990,'In Earth orbit','','1976-12-31 00:00:00',107.48,1086,1134,63.44,'Intermediate' from dual union all
select 8837,'008837','1976-041C','BOZ','BOZ',38993,'Reentered','1976-09-08 00:00:00','1976-06-11 00:00:00',92.32,214,562,62.87,'Intermediate' from dual union all
select 8838,'008838','1976-042A','Comstar D-1','Comstar D-1',38994,'In Earth orbit','','1977-01-25 00:00:00',1436.11,35776,35797,.05,'Stationary' from dual union all
select 8839,'008839','1976-038E','SSU 1 cover','SSU cover',38990,'In Earth orbit','','1977-01-02 00:00:00',107.65,1090,1146,63.44,'Intermediate' from dual union all
select 884,'000884','1964-058A','OPS 4262','GAMBIT SV 962',10344,'Reentered','1964-09-28 00:00:00','1964-09-25 00:00:00',88.03,121,233,92.91,'Polar' from dual union all
select 8840,'008840','1976-042B','Centaur D-1AR AC-38','Centaur D-1AR AC-38',38994,'In Earth orbit','','1976-06-16 00:00:00',649.34,652,36271,21.82,'GEO Transfer' from dual union all
select 8841,'008841','1976-036H','PTDU cover','KDU part',38988,'Reentered','1976-05-15 00:00:00','1976-05-14 00:00:00',88.72,192,230,81.38,'Intermediate' from dual union all
select 8842,'008842','1976-038F','deb USN satellite','',38990,'In Earth orbit','','1997-11-21 00:00:00',105.55,518,1522,63.37,'Intermediate' from dual union all
select 8843,'008843','1976-038G','SSU 2 cover','SSU cover',38990,'In Earth orbit','','1977-01-01 00:00:00',107.64,1089,1146,63.44,'Intermediate' from dual union all
select 8844,'008844','1976-041D','Blok-L','Blok-L',38993,'Reentered','1990-05-05 00:00:00','1976-06-14 00:00:00',733.55,624,40506,62.92,'Molniya' from dual union all
select 8845,'008845','1976-043A','Meteor','Meteor-Priroda No. 2-1',38995,'In Earth orbit','','1993-01-15 00:00:00',102.09,828,888,81.26,'Intermediate' from dual union all
select 8846,'008846','1976-043B','8A92ME','8A92ME',38995,'In Earth orbit','','1993-08-17 00:00:00',102.3,833,901,81.25,'Intermediate' from dual union all
select 8847,'008847','1976-041E','deb Blok-I','',38993,'Reentered','1976-07-04 00:00:00','1976-05-24 00:00:00',92.56,213,587,62.84,'Intermediate' from dual union all
select 8848,'008848','1976-040D','KDU','KDU',38992,'Reentered','1976-05-24 00:00:00','1976-05-16 00:00:00',89.16,170,296,64.99,'Intermediate' from dual union all
select 8849,'008849','1976-040E','Blenda','Sunshade',38992,'Reentered','1976-05-19 00:00:00','1976-05-16 00:00:00',88.96,165,281,65.02,'Intermediate' from dual union all
select 885,'000885','1964-059A','Kosmos-46','Zenit-2 No. 23',10345,'Landed','1964-10-02 00:00:00','1964-09-29 00:00:00',89.21,210,260,51.25,'Intermediate' from dual union all
select 8850,'008850','1976-040F','KDU part','KDU part',38992,'Reentered','1976-05-18 00:00:00','1976-05-18 00:00:00',88.94,170,274,64.96,'Intermediate' from dual union all
select 8851,'008851','1976-044A','Kosmos-818','DS-P1-Yu No. 78',38996,'Reentered','1977-03-07 00:00:00','1976-06-17 00:00:00',91.89,267,468,71.06,'Intermediate' from dual union all
select 8852,'008852','1976-044B','S1M','S1M',38996,'Reentered','1976-10-28 00:00:00','1976-06-17 00:00:00',91.52,270,429,71.04,'Intermediate' from dual union all
select 8853,'008853','1976-045A','Kosmos-819','Zenit-2M',38997,'Landed','1976-06-01 00:00:00','1976-05-24 00:00:00',89.38,200,288,65,'Intermediate' from dual union all
select 8854,'008854','1976-045B','11A57I','11A57I',38997,'Reentered','1976-05-30 00:00:00','1976-05-22 00:00:00',89.2,204,265,65.01,'Intermediate' from dual union all
select 8855,'008855','1976-045C','Sensor cover','Zenit-2M sensor cover',38997,'Reentered','1976-05-22 00:00:00','1976-05-20 00:00:00',89.43,204,289,65,'Intermediate' from dual union all
select 8856,'008856','1976-046A','Kosmos-820','Fram',38998,'Landed','1976-06-02 00:00:00','1976-05-23 00:00:00',88.87,206,232,81.36,'Intermediate' from dual union all
select 8857,'008857','1976-046B','11S510','11S510',38998,'Reentered','1976-05-24 00:00:00','1976-05-24 00:00:00',87.66,155,163,81.34,'Intermediate' from dual union all
select 8858,'008858','1976-046C','Sensor cover','Fram sensor cover',38998,'Reentered','1976-05-21 00:00:00','1976-05-21 00:00:00',87.91,86,256,81.67,'Intermediate' from dual union all
select 8859,'008859','1976-038H','SSU 3 cover','SSU cover',38990,'In Earth orbit','','1977-01-01 00:00:00',107.11,1074,1112,63.44,'Intermediate' from dual union all
select 886,'000886','1964-059B','8A92E','8A92E',10345,'Reentered','1964-10-07 00:00:00','1964-09-29 00:00:00',89.16,212,254,51.26,'Intermediate' from dual union all
select 8860,'008860','1976-047A','P76-5','NNS O-15',38999,'In Earth orbit','','1993-01-14 00:00:00',105.45,984,1047,99.61,'Sun-Synch' from dual union all
select 8861,'008861','1976-047B','Star 20 E-21','Star 20 E-21',38999,'In Earth orbit','','1993-08-16 00:00:00',105.5,987,1049,99.51,'Sun-Synch' from dual union all
select 8862,'008862','1976-048A','Kosmos-821','Zenit-4MK',39000,'Landed','1976-06-08 00:00:00','1976-05-29 00:00:00',89.53,167,336,72.83,'Intermediate' from dual union all
select 8863,'008863','1976-048B','11A57I','11A57I',39000,'Reentered','1976-06-07 00:00:00','1976-05-28 00:00:00',89.45,201,294,72.83,'Intermediate' from dual union all
select 8864,'008864','1976-048C','Sensor cover','Zenit-4MK sensor cover',39000,'Reentered','1976-06-02 00:00:00','1976-05-28 00:00:00',89.65,205,310,72.82,'Intermediate' from dual union all
select 8865,'008865','1976-049A','Kosmos-822','Vektor',39001,'Reentered','1978-08-08 00:00:00','1976-06-27 00:00:00',94.44,278,704,74.05,'Intermediate' from dual union all
select 8866,'008866','1976-049B','S3M','S3M',39001,'Reentered','1977-12-23 00:00:00','1976-06-30 00:00:00',94.28,273,695,74.06,'Intermediate' from dual union all
select 8867,'008867','1976-047C','P76-5 despin weight','',38999,'In Earth orbit','','1993-08-16 00:00:00',106.32,1000,1113,99.3,'Sun-Synch' from dual union all
select 8868,'008868','1976-047D','P76-5 despin weight','',38999,'In Earth orbit','','1993-08-15 00:00:00',104.58,937,1013,100.06,'Sun-Synch' from dual union all
select 8869,'008869','1976-046D','Blenda','Sunshade',38998,'Reentered','1976-06-04 00:00:00','1976-06-01 00:00:00',88.58,186,223,81.36,'Intermediate' from dual union all
select 887,'000887','1964-058B','Agena D 4809','Agena D',10344,'Reentered','1964-09-26 00:00:00','1964-09-23 00:00:00',88.93,142,302,92.9,'Polar' from dual union all
select 8870,'008870','1976-046E','KDU','KDU',38998,'Reentered','1976-06-10 00:00:00','1976-06-02 00:00:00',88.7,198,223,81.36,'Intermediate' from dual union all
select 8871,'008871','1976-050A','OPS 7837','QUASAR 1',39002,'In Earth orbit','','1976-06-02 00:00:00',700.58,311,39192,63.5,'Molniya' from dual union all
select 8872,'008872','1976-050B','Agena D','Agena D',39002,'In Earth orbit','','1976-06-02 00:00:00',700.58,311,39192,63.5,'Molniya' from dual union all
select 8873,'008873','1976-051A','Kosmos-823','Zaliv',39003,'In Earth orbit','','1993-01-10 00:00:00',104.9,973,1007,82.97,'Intermediate' from dual union all
select 8874,'008874','1976-051B','S3M','S3M',39003,'In Earth orbit','','1991-03-28 00:00:00',104.8,967,1003,82.96,'Intermediate' from dual union all
select 8875,'008875','1976-048D','Blenda','Sunshade',39000,'Reentered','1976-06-14 00:00:00','1976-06-07 00:00:00',89.43,167,326,72.82,'Intermediate' from dual union all
select 8876,'008876','1976-048E','KDU','KDU',39000,'Reentered','1976-06-09 00:00:00','1976-06-07 00:00:00',89.4,160,330,72.82,'Intermediate' from dual union all
select 8877,'008877','1976-052A','Kosmos-824','Zenit-4MK',39004,'Landed','1976-06-21 00:00:00','1976-06-10 00:00:00',89.34,168,316,71.37,'Intermediate' from dual union all
select 8878,'008878','1976-052B','11A57I','11A57I',39004,'Reentered','1976-06-20 00:00:00','1976-06-10 00:00:00',89.56,199,307,71.37,'Intermediate' from dual union all
select 8879,'008879','1976-048F','KDU part','KDU part',39000,'Reentered','1976-06-09 00:00:00','1976-06-08 00:00:00',89.11,164,297,72.83,'Intermediate' from dual union all
select 888,'000888','1964-056B','Fairing','Fairing',10342,'Reentered','1964-09-26 00:00:00','1964-09-24 00:00:00',90.66,172,441,85,'Polar' from dual union all
select 8880,'008880','1976-052C','Sensor cover','Zenit-4MK sensor cover',39004,'Reentered','1976-06-16 00:00:00','1976-06-10 00:00:00',90.13,202,360,71.37,'Intermediate' from dual union all
select 8881,'008881','1976-052D','Sensor cover','Zenit-4MK sensor cover',39004,'Reentered','1976-06-10 00:00:00','1976-06-09 00:00:00',88.7,195,226,71.38,'Intermediate' from dual union all
select 8882,'008882','1976-053A','Marisat 103','Marisat B',39005,'In Earth orbit','','1977-02-24 00:00:00',1436.14,35781,35794,2.28,'Stationary' from dual union all
select 8883,'008883','1976-053B','Delta 124','Delta SSPU DSV-3P-4',39005,'Reentered','1977-08-25 00:00:00','1976-07-12 00:00:00',93.75,273,643,28.55,'Intermediate' from dual union all
select 8884,'008884','1976-038J','SSU 2','SSU',38990,'In Earth orbit','','1976-12-31 00:00:00',107.48,1081,1140,63.44,'Intermediate' from dual union all
select 8885,'008885','1976-037C','Kosmos-816 SS 1','ESO',38989,'Reentered','1977-07-12 00:00:00','1976-06-12 00:00:00',94.53,475,517,65.83,'Intermediate' from dual union all
select 8886,'008886','1976-037D','Kosmos-816 SS 2','ESO',38989,'Reentered','1977-07-26 00:00:00','1976-06-14 00:00:00',94.55,476,517,65.83,'Intermediate' from dual union all
select 8887,'008887','1976-053C','Delta 124 clamp band','',39005,'Reentered','1976-07-02 00:00:00','1976-06-17 00:00:00',91.7,177,539,28.58,'Intermediate' from dual union all
select 8888,'008888','1976-053D','Delta 124 clamp band','',39005,'Reentered','1976-06-30 00:00:00','1976-06-15 00:00:00',91.82,172,555,28.56,'Intermediate' from dual union all
select 8889,'008889','1976-054A','Kosmos-825','Strela-1M',39006,'In Earth orbit','','1992-12-20 00:00:00',114.68,1395,1487,74,'Intermediate' from dual union all
select 889,'000889','1964-060A','IMP 2','IMP B',10346,'Reentered','1966-01-01 00:00:00','1964-11-29 00:00:00',2079.87,666,94526,33.82,'Very High' from dual union all
select 8890,'008890','1976-054B','Kosmos-826','Strela-1M',39006,'In Earth orbit','','1993-01-19 00:00:00',116.27,1482,1543,74,'Intermediate' from dual union all
select 8891,'008891','1976-054C','Kosmos-827','Strela-1M',39006,'In Earth orbit','','1992-12-23 00:00:00',114.9,1413,1488,74,'Intermediate' from dual union all
select 8892,'008892','1976-054D','Kosmos-828','Strela-1M',39006,'In Earth orbit','','1993-01-15 00:00:00',115.11,1433,1488,74,'Intermediate' from dual union all
select 8893,'008893','1976-054E','Kosmos-829','Strela-1M',39006,'In Earth orbit','','1992-12-22 00:00:00',115.32,1450,1490,74,'Intermediate' from dual union all
select 8894,'008894','1976-054F','Kosmos-830','Strela-1M',39006,'In Earth orbit','','1992-12-21 00:00:00',115.55,1468,1493,74,'Intermediate' from dual union all
select 8895,'008895','1976-054G','Kosmos-831','Strela-1M',39006,'In Earth orbit','','1992-12-20 00:00:00',115.78,1474,1507,74,'Intermediate' from dual union all
select 8896,'008896','1976-054H','Kosmos-832','Strela-1M',39006,'In Earth orbit','','1992-12-29 00:00:00',116,1482,1520,74,'Intermediate' from dual union all
select 8897,'008897','1976-054J','S3M','S3M',39006,'In Earth orbit','','1991-02-20 00:00:00',117.93,1488,1688,74,'Intermediate' from dual union all
select 8898,'008898','1976-055A','Kosmos-833','Zenit-4MK',39007,'Landed','1976-06-29 00:00:00','1976-06-18 00:00:00',89.41,178,313,62.82,'Intermediate' from dual union all
select 8899,'008899','1976-055B','11A57I','11A57I',39007,'Reentered','1976-06-23 00:00:00','1976-06-18 00:00:00',89.19,174,294,62.8,'Intermediate' from dual union all
select 89,'000089','61 Epsilon 3','SRV heat shield','SRV heat shield',7264,'Reentered','1961-04-20 00:00:00','1961-03-26 00:00:00',92.62,289,517,80.91,'Intermediate' from dual union all
select 890,'000890','1964-061A','OPS 3333','CORONA J-3',10347,'Reentered Attached','1964-10-26 00:00:00','1964-10-14 00:00:00',90.37,175,411,79.97,'Intermediate' from dual union all
select 8900,'008900','1976-055C','Sensor cover','Zenit-4MK sensor cover',39007,'Reentered','1976-06-17 00:00:00','1976-06-16 00:00:00',88.93,170,274,62.76,'Intermediate' from dual union all
select 8901,'008901','1976-055D','Sensor cover','Zenit-4MK sensor cover',39007,'Reentered','1976-06-21 00:00:00','1976-06-17 00:00:00',89.13,164,299,63.08,'Intermediate' from dual union all
select 8902,'008902','1976-053E','deb Delta 124','',39005,'Reentered','1976-07-01 00:00:00','1976-06-17 00:00:00',91.56,176,527,28.58,'Intermediate' from dual union all
select 8903,'008903','1976-056A','Interkosmos-15','AUOS-Z-T-IK',39008,'Reentered','1979-11-18 00:00:00','1976-07-19 00:00:00',94.6,483,516,74.05,'Intermediate' from dual union all
select 8904,'008904','1976-056B','S3M','S3M',39008,'Reentered','1980-01-15 00:00:00','1976-07-22 00:00:00',94.51,472,518,74.05,'Intermediate' from dual union all
select 8905,'008905','1976-052E','KDU','KDU',39004,'Reentered','1976-06-27 00:00:00','1976-06-20 00:00:00',89.33,166,317,71.38,'Intermediate' from dual union all
select 8906,'008906','1976-052F','Blenda','Sunshade',39004,'Reentered','1976-06-23 00:00:00','1976-06-20 00:00:00',89.14,158,306,71.32,'Intermediate' from dual;
insert into SATELLITE
select 8907,'008907','1976-052G','PTDU cover','KDU part',39004,'Reentered','1976-06-22 00:00:00','1976-06-22 00:00:00',88.52,165,238,71.3,'Intermediate' from dual union all
select 8908,'008908','1970-025NJ','deb Agena D','',21502,'In Earth orbit','','1993-08-07 00:00:00',109.01,1017,1344,101.68,'Sun-Synch' from dual union all
select 8909,'008909','1974-089DT','deb Delta 104','',33692,'Reentered','1981-11-15 00:00:00','1981-11-04 00:00:00',100.75,758,831,101.51,'Sun-Synch' from dual union all
select 891,'000891','1964-062A','Kosmos-47','Voskhod 3KV No. 2',10348,'Landed','1964-10-07 00:00:00','1964-10-06 00:00:00',90.08,174,383,64.6,'Intermediate' from dual union all
select 8910,'008910','1976-053F','Star 37E S/N 40042','Star 37E',39005,'Reentered','1982-11-27 00:00:00','1976-09-24 00:00:00',625.26,256,35431,25.5,'GEO Transfer' from dual union all
select 8911,'008911','1976-057A','Salyut-5','Almaz OPS 3',39009,'Deorbited','1977-08-08 00:00:00','1976-07-22 00:00:00',89.84,262,272,51.59,'Intermediate' from dual union all
select 8912,'008912','1976-057B','8S812','8S812',39009,'Reentered','1976-06-30 00:00:00','1976-06-24 00:00:00',88.78,208,221,51.61,'Intermediate' from dual union all
select 8913,'008913','1976-057C','deb Salyut-5','',39009,'Reentered','1976-06-24 00:00:00','1976-06-24 00:00:00',88.46,193,203,51.59,'Intermediate' from dual union all
select 8914,'008914','1976-058A','Kosmos-834','Zenit-2M',39010,'Landed','1976-07-06 00:00:00','1976-06-26 00:00:00',88.96,214,233,81.37,'Intermediate' from dual union all
select 8915,'008915','1976-058B','11S510','11S510',39010,'Reentered','1976-06-30 00:00:00','1976-06-27 00:00:00',88.59,201,209,81.38,'Intermediate' from dual union all
select 8916,'008916','1976-059A','DSP F6','DSP 7',39011,'In Earth orbit','','1979-11-06 00:00:00',1436.44,35768,35819,2.5,'Stationary' from dual union all
select 8917,'008917','1976-059B','Titan 23C-10 Stage 2','LR91AJ11 23C-10',39011,'Reentered','1976-06-30 00:00:00','1976-06-28 00:00:00',89.87,149,387,28.67,'Intermediate' from dual union all
select 8918,'008918','1976-059C','Transtage 28','Transtage 28',39011,'In Earth orbit','','1976-07-01 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 8919,'008919','1976-059D','DSP 7 Aperture Cover','',39011,'In Earth orbit','','1976-07-01 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 892,'000892','1964-062B','11A57I','11A57I',10348,'Reentered','1964-10-14 00:00:00','1964-10-10 00:00:00',89.34,167,317,64.7,'Intermediate' from dual union all
select 8920,'008920','1976-055E','KDU','KDU',39007,'Reentered','1976-07-08 00:00:00','1976-06-28 00:00:00',89.22,176,296,62.83,'Intermediate' from dual union all
select 8921,'008921','1976-055F','Blenda','Sunshade',39007,'Reentered','1976-07-01 00:00:00','1976-06-28 00:00:00',88.99,174,276,62.83,'Intermediate' from dual union all
select 8922,'008922','1976-060A','Kosmos-835','Zenit-4MK',39012,'Landed','1976-07-12 00:00:00','1976-07-01 00:00:00',89.36,173,313,64.96,'Intermediate' from dual union all
select 8923,'008923','1976-061A','Kosmos-836','Strela-2M',39013,'In Earth orbit','','1992-12-23 00:00:00',100.65,776,803,74.05,'Intermediate' from dual union all
select 8924,'008924','1976-061B','S3M','S3M',39013,'In Earth orbit','','1991-03-28 00:00:00',100.51,772,794,74.06,'Intermediate' from dual union all
select 8925,'008925','1976-060B','11A57I','11A57I',39012,'Reentered','1976-07-04 00:00:00','1976-07-01 00:00:00',89.04,173,282,64.97,'Intermediate' from dual union all
select 8926,'008926','1976-055G','PTDU cover','KDU part',39007,'Reentered','1976-06-30 00:00:00','1976-06-29 00:00:00',88.68,170,249,62.79,'Intermediate' from dual union all
select 8927,'008927','1976-062A','Kosmos-837','Molniya-2',39014,'Reentered','1983-11-18 00:00:00','1976-08-01 00:00:00',98.5,439,935,62.76,'Intermediate' from dual union all
select 8928,'008928','1976-062B','11S510','11S510',39014,'Reentered','1976-08-05 00:00:00','1976-07-11 00:00:00',90.27,210,366,62.8,'Intermediate' from dual union all
select 8929,'008929','1976-062C','BOZ','BOZ',39014,'Reentered','1976-07-21 00:00:00','1976-07-08 00:00:00',90.25,183,391,62.8,'Intermediate' from dual union all
select 893,'000893','1964-063A','Ablestar 016','AJ10-104D',10349,'In Earth orbit','','1964-11-16 00:00:00',106.3,1040,1071,89.91,'Polar' from dual union all
select 8930,'008930','1976-062D','deb Blok-I','',39014,'Reentered','1976-07-20 00:00:00','1976-07-09 00:00:00',90.6,210,398,62.81,'Intermediate' from dual union all
select 8931,'008931','1976-062E','Blok-L','Blok-L',39014,'Reentered','1982-09-26 00:00:00','1976-08-04 00:00:00',98.39,442,922,62.75,'Intermediate' from dual union all
select 8932,'008932','1976-063A','Kosmos-838','US-P',39015,'Reentered','1977-08-23 00:00:00','1976-08-01 00:00:00',93.27,427,442,65.05,'Intermediate' from dual union all
select 8933,'008933','1976-063B','11S692','11S692',39015,'Reentered','1976-07-03 00:00:00','1976-07-02 00:00:00',89.44,114,380,65.03,'Intermediate' from dual union all
select 8934,'008934','1976-064A','Soyuz-21','Soyuz 7K-T No. 41',39016,'Landed','1976-08-24 00:00:00','1976-07-19 00:00:00',89.84,262,272,51.59,'Intermediate' from dual union all
select 8935,'008935','1976-064B','11S510','11S510',39016,'Reentered','1976-07-09 00:00:00','1976-07-08 00:00:00',88.21,173,199,51.6,'Intermediate' from dual union all
select 8936,'008936','1975-004S','deb Delta 107','',36451,'Reentered','1978-11-20 00:00:00','1978-09-25 00:00:00',94.08,454,494,98.04,'Sun-Synch' from dual union all
select 8937,'008937','1975-004T','deb Delta 107','',36451,'Reentered','1979-12-09 00:00:00','1979-12-05 00:00:00',90.79,289,338,97.57,'Sun-Synch' from dual union all
select 8938,'008938','1975-004U','deb Delta 107','',36451,'Reentered','1978-04-18 00:00:00','1978-01-29 00:00:00',96.43,544,632,96.92,'Sun-Synch' from dual union all
select 8939,'008939','1975-004V','deb Delta 107','',36451,'Reentered','1981-05-05 00:00:00','1976-07-07 00:00:00',98.82,650,755,97.95,'Sun-Synch' from dual union all
select 894,'000894','1964-062C','deb Kosmos-47','',10348,'Reentered','1964-10-09 00:00:00','1964-10-06 00:00:00',90.01,180,370,64.7,'Intermediate' from dual union all
select 8940,'008940','1975-004W','deb Delta 107','',36451,'Reentered','1999-04-03 00:00:00','1993-08-23 00:00:00',95.16,519,534,98.3,'Sun-Synch' from dual union all
select 8941,'008941','1975-004X','deb Delta 107','',36451,'Reentered','1978-04-18 00:00:00','1978-02-26 00:00:00',95.67,495,606,99.36,'Sun-Synch' from dual union all
select 8942,'008942','1975-004Y','deb Delta 107','',36451,'Reentered','1981-09-26 00:00:00','1981-07-24 00:00:00',98.08,589,745,96.11,'Sun-Synch' from dual union all
select 8943,'008943','1975-004Z','deb Delta 107','',36451,'Reentered','1982-07-17 00:00:00','1982-07-16 00:00:00',91.92,342,396,98.3,'Sun-Synch' from dual union all
select 8944,'008944','1975-004AA','deb Delta 107','',36451,'Reentered','1980-02-18 00:00:00','1976-07-05 00:00:00',98.34,605,754,97.75,'Sun-Synch' from dual union all
select 8945,'008945','1975-004AB','deb Delta 107','',36451,'Reentered','1980-04-25 00:00:00','1976-07-04 00:00:00',98.54,611,767,98.01,'Sun-Synch' from dual union all
select 8946,'008946','1975-004AC','deb Delta 107','',36451,'Reentered','1978-07-15 00:00:00','1978-06-18 00:00:00',96.01,550,585,100.01,'Sun-Synch' from dual union all
select 8947,'008947','1975-004AD','deb Delta 107','',36451,'Reentered','1981-04-08 00:00:00','1976-07-07 00:00:00',98.74,528,868,98.54,'Sun-Synch' from dual union all
select 8948,'008948','1975-004AE','deb Delta 107','',36451,'Reentered','1981-08-10 00:00:00','1976-07-09 00:00:00',99.09,625,806,99.03,'Sun-Synch' from dual union all
select 8949,'008949','1975-004AF','deb Delta 107','',36451,'Reentered','1979-04-06 00:00:00','1976-07-03 00:00:00',96.22,405,750,97.28,'Sun-Synch' from dual union all
select 895,'000895','1964-062D','deb Kosmos-47','',10348,'Reentered','1964-10-12 00:00:00','1964-10-06 00:00:00',89.99,180,367,64.77,'Intermediate' from dual union all
select 8950,'008950','1975-004AG','deb Delta 107','',36451,'Reentered','1981-12-19 00:00:00','1981-12-16 00:00:00',91.85,346,384,97.86,'Sun-Synch' from dual union all
select 8951,'008951','1975-004AH','deb Delta 107','',36451,'In Earth orbit','','1993-08-06 00:00:00',99.23,693,751,97.84,'Sun-Synch' from dual union all
select 8952,'008952','1975-004AJ','deb Delta 107','',36451,'Reentered','1979-03-29 00:00:00','1979-03-23 00:00:00',92.33,373,405,97.83,'Sun-Synch' from dual union all
select 8953,'008953','1975-004AK','deb Delta 107','',36451,'Reentered','1989-06-07 00:00:00','1989-06-07 00:00:00',87.82,160,174,98.13,'Sun-Synch' from dual union all
select 8954,'008954','1975-004AL','deb Delta 107','',36451,'In Earth orbit','','1993-08-04 00:00:00',101.69,713,965,97.75,'Sun-Synch' from dual union all
select 8955,'008955','1975-004AM','deb Delta 107','',36451,'Reentered','1977-06-23 00:00:00','1976-07-05 00:00:00',96.29,320,842,98.32,'Sun-Synch' from dual union all
select 8956,'008956','1975-004AN','deb Delta 107','',36451,'In Earth orbit','','1993-08-05 00:00:00',98.97,679,739,97.96,'Sun-Synch' from dual union all
select 8957,'008957','1975-004AP','deb Delta 107','',36451,'Reentered','1979-11-04 00:00:00','1979-10-14 00:00:00',96.74,575,631,97.95,'Sun-Synch' from dual union all
select 8958,'008958','1975-004AQ','deb Delta 107','',36451,'In Earth orbit','','1993-08-04 00:00:00',99.6,708,771,97.88,'Sun-Synch' from dual union all
select 8959,'008959','1975-004AR','deb Delta 107','',36451,'In Earth orbit','','1993-08-08 00:00:00',100.26,719,823,98.28,'Sun-Synch' from dual union all
select 896,'000896','1964-062E','deb Kosmos-47','',10348,'Reentered','1964-10-09 00:00:00','1964-10-06 00:00:00',90.01,180,370,64.7,'Intermediate' from dual union all
select 8960,'008960','1975-004AS','deb Delta 107','',36451,'Reentered','1992-03-08 00:00:00','1992-03-08 00:00:00',88.64,200,215,97.39,'Sun-Synch' from dual union all
select 8961,'008961','1975-004AT','deb Delta 107','',36451,'Reentered','1979-04-24 00:00:00','1979-04-17 00:00:00',94.29,465,504,96.51,'Sun-Synch' from dual union all
select 8962,'008962','1975-004AU','deb Delta 107','',36451,'Reentered','1979-12-29 00:00:00','1979-12-17 00:00:00',94.58,475,522,97.57,'Sun-Synch' from dual union all
select 8963,'008963','1975-004AV','deb Delta 107','',36451,'Reentered','2002-06-14 00:00:00','1993-08-08 00:00:00',98.65,645,744,98.35,'Sun-Synch' from dual union all
select 8964,'008964','1975-004AW','deb Delta 107','',36451,'In Earth orbit','','1993-08-05 00:00:00',101.09,725,896,97.67,'Sun-Synch' from dual union all
select 8965,'008965','1975-004AX','deb Delta 107','',36451,'Reentered','1978-11-03 00:00:00','1978-10-25 00:00:00',91.85,328,403,98,'Sun-Synch' from dual union all
select 8966,'008966','1975-004AY','deb Delta 107','',36451,'Reentered','1978-11-09 00:00:00','1978-10-29 00:00:00',91.28,319,355,97.8,'Sun-Synch' from dual union all
select 8967,'008967','1975-004AZ','deb Delta 107','',36451,'Reentered','1989-10-16 00:00:00','1989-10-15 00:00:00',89.61,248,263,97.54,'Sun-Synch' from dual union all
select 8968,'008968','1975-004BA','deb Delta 107','',36451,'In Earth orbit','','1993-08-03 00:00:00',101.57,726,940,97.69,'Sun-Synch' from dual union all
select 8969,'008969','1975-004BB','deb Delta 107','',36451,'In Earth orbit','','1993-08-02 00:00:00',101.35,718,928,97.7,'Sun-Synch' from dual union all
select 897,'000897','1964-063B','NNS O-1','NNS O-1',10349,'In Earth orbit','','1964-12-21 00:00:00',106.57,1053,1083,89.92,'Polar' from dual union all
select 8970,'008970','1975-004BC','deb Delta 107','',36451,'In Earth orbit','','1993-08-03 00:00:00',101.32,715,928,97.3,'Sun-Synch' from dual union all
select 8971,'008971','1975-004BD','deb Delta 107','',36451,'Reentered','1979-10-23 00:00:00','1979-10-11 00:00:00',96.03,536,601,96.8,'Sun-Synch' from dual union all
select 8972,'008972','1975-004BE','deb Delta 107','',36451,'Reentered','1979-11-05 00:00:00','1979-09-17 00:00:00',97.85,573,739,98,'Sun-Synch' from dual union all
select 8973,'008973','1975-004BF','deb Delta 107','',36451,'Reentered','1980-01-13 00:00:00','1980-01-05 00:00:00',95.49,485,600,97.42,'Sun-Synch' from dual union all
select 8974,'008974','1975-004BG','deb Delta 107','',36451,'Reentered','1977-01-31 00:00:00','1976-07-07 00:00:00',96.83,440,774,98.73,'Sun-Synch' from dual union all
select 8975,'008975','1975-004BH','deb Delta 107','',36451,'Reentered','1979-09-06 00:00:00','1979-08-27 00:00:00',90.37,274,312,97.57,'Sun-Synch' from dual union all
select 8976,'008976','1975-004BJ','deb Delta 107','',36451,'Reentered','1979-11-21 00:00:00','1979-11-17 00:00:00',91.05,307,345,99,'Sun-Synch' from dual union all
select 8977,'008977','1975-004BK','deb Delta 107','',36451,'Reentered','1981-07-28 00:00:00','1976-07-07 00:00:00',99.08,611,819,98.91,'Sun-Synch' from dual union all
select 8978,'008978','1975-004BL','deb Delta 107','',36451,'In Earth orbit','','1993-08-04 00:00:00',98.72,666,729,98.05,'Sun-Synch' from dual union all
select 8979,'008979','1975-004BM','deb Delta 107','',36451,'Reentered','2000-11-10 00:00:00','1993-08-07 00:00:00',96.41,568,606,97.92,'Sun-Synch' from dual union all
select 898,'000898','1964-049E','Blok-L No. R103-36','Blok-L',10335,'Reentered','2004-05-07 00:00:00','1964-10-09 00:00:00',715.73,352,39902,64.99,'GEO Transfer' from dual union all
select 8980,'008980','1975-004BN','deb Delta 107','',36451,'In Earth orbit','','1993-08-06 00:00:00',99.41,691,769,97.9,'Sun-Synch' from dual union all
select 8981,'008981','1975-004BP','deb Delta 107','',36451,'In Earth orbit','','1993-08-08 00:00:00',101.52,707,955,97.73,'Sun-Synch' from dual union all
select 8982,'008982','1975-004BQ','deb Delta 107','',36451,'Reentered','1980-06-23 00:00:00','1980-06-21 00:00:00',92.11,355,401,96.01,'Sun-Synch' from dual union all
select 8983,'008983','1975-004BR','deb Delta 107','',36451,'Reentered','1979-09-11 00:00:00','1979-08-23 00:00:00',92.14,346,412,97.71,'Sun-Synch' from dual union all
select 8984,'008984','1975-004BS','deb Delta 107','',36451,'Reentered','1979-11-23 00:00:00','1979-11-18 00:00:00',91.02,310,340,98.03,'Sun-Synch' from dual union all
select 8985,'008985','1975-004BT','deb Delta 107','',36451,'Reentered','1979-05-30 00:00:00','1979-04-21 00:00:00',94.55,475,519,97.59,'Sun-Synch' from dual union all
select 8986,'008986','1975-004BU','deb Delta 107','',36451,'Reentered','1979-10-17 00:00:00','1979-10-08 00:00:00',95.35,509,562,97.75,'Sun-Synch' from dual union all
select 8987,'008987','1975-004BV','deb Delta 107','',36451,'Reentered','1979-12-16 00:00:00','1979-10-23 00:00:00',97.18,603,645,97.88,'Sun-Synch' from dual union all
select 8988,'008988','1975-004BW','deb Delta 107','',36451,'Reentered','1979-04-25 00:00:00','1979-03-31 00:00:00',96.27,563,597,97.58,'Sun-Synch' from dual union all
select 8989,'008989','1975-004BX','deb Delta 107','',36451,'Reentered','1978-08-16 00:00:00','1978-08-02 00:00:00',92.38,387,396,98.37,'Sun-Synch' from dual union all
select 899,'000899','1964-064A','Explorer 22','Beacon Explorer B',10350,'In Earth orbit','','1964-11-23 00:00:00',104.74,886,1079,79.69,'Intermediate' from dual union all
select 8990,'008990','1975-004BY','deb Delta 107','',36451,'Reentered','1982-02-27 00:00:00','1976-07-07 00:00:00',98.03,539,790,98.3,'Sun-Synch' from dual union all
select 8991,'008991','1975-004BZ','deb Delta 107','',36451,'Reentered','1983-12-09 00:00:00','1976-07-07 00:00:00',98.64,637,751,98.92,'Sun-Synch' from dual union all
select 8992,'008992','1975-004CA','deb Delta 107','',36451,'Reentered','1981-04-23 00:00:00','1981-04-21 00:00:00',90.55,288,315,98.03,'Sun-Synch' from dual union all
select 8993,'008993','1975-004CB','deb Delta 107','',36451,'Reentered','1986-12-01 00:00:00','1986-11-30 00:00:00',88.59,198,212,98.63,'Sun-Synch' from dual union all
select 8994,'008994','1975-004CC','deb Delta 107','',36451,'Reentered','1979-06-16 00:00:00','1979-04-04 00:00:00',94.4,481,497,97.81,'Sun-Synch' from dual union all
select 8995,'008995','1975-004CD','deb Delta 107','',36451,'Reentered','1986-10-11 00:00:00','1986-10-07 00:00:00',90.07,265,291,98.78,'Sun-Synch' from dual union all
select 8996,'008996','1975-004CE','deb Delta 107','',36451,'Reentered','1983-12-10 00:00:00','1976-07-07 00:00:00',99.9,729,779,98.21,'Sun-Synch' from dual union all
select 8997,'008997','1975-004CF','deb Delta 107','',36451,'Reentered','1998-01-18 00:00:00','1993-08-24 00:00:00',94.74,499,513,98.35,'Sun-Synch' from dual;
insert into SATELLITE
select 8998,'008998','1975-004CG','deb Delta 107','',36451,'Reentered','1979-03-24 00:00:00','1979-03-16 00:00:00',91.08,299,356,97.67,'Sun-Synch' from dual union all
select 8999,'008999','1975-004CH','deb Delta 107','',36451,'Reentered','1982-05-08 00:00:00','1976-07-07 00:00:00',99.1,634,797,98.19,'Sun-Synch' from dual union all
select 9,'000009','58 Epsilon','Explorer 4','Explorer 4',4795,'Reentered Attached','1959-10-23 00:00:00','1959-04-24 00:00:00',110.4,258,2233,50.4,'Intermediate' from dual union all
select 90,'000090','61 Epsilon 4','SRV 520','SRV',7264,'Reentered','1961-10-31 00:00:00','1961-03-23 00:00:00',96.26,395,764,80.91,'Intermediate' from dual union all
select 900,'000900','1964-063C','Dragsphere 1','Dragsphere',10349,'In Earth orbit','','1964-11-11 00:00:00',106.55,1053,1081,89.92,'Polar' from dual union all
select 9000,'009000','1975-004CJ','deb Delta 107','',36451,'Reentered','1991-12-13 00:00:00','1991-12-12 00:00:00',89.01,219,233,98.02,'Sun-Synch' from dual union all
select 9001,'009001','1975-004CK','deb Delta 107','',36451,'Reentered','1980-02-20 00:00:00','1980-02-09 00:00:00',91.87,351,381,98.01,'Sun-Synch' from dual union all
select 9002,'009002','1975-004CL','deb Delta 107','',36451,'Reentered','1980-12-26 00:00:00','1976-07-09 00:00:00',99.58,586,891,97.92,'Sun-Synch' from dual union all
select 9003,'009003','1975-004CM','deb Delta 107','',36451,'In Earth orbit','','1993-08-05 00:00:00',98.77,673,726,97.94,'Sun-Synch' from dual union all
select 9004,'009004','1975-004CN','deb Delta 107','',36451,'Reentered','1977-07-03 00:00:00','1976-07-09 00:00:00',95.84,350,769,97.47,'Sun-Synch' from dual union all
select 9005,'009005','1975-004CP','deb Delta 107','',36451,'Reentered','1980-12-26 00:00:00','1976-07-07 00:00:00',106.98,722,1452,97.8,'Sun-Synch' from dual union all
select 9006,'009006','1976-065A','OPS 4699','HEXAGON SV-12',39017,'Deorbited','1976-12-13 00:00:00','1976-08-07 00:00:00',88.44,157,238,96.98,'Sun-Synch' from dual union all
select 9007,'009007','1976-065B','S3-3','S3-3',39017,'Reentered','1986-04-24 00:00:00','1980-08-21 00:00:00',153.9,232,6038,97.46,'Medium' from dual union all
select 9008,'009008','1976-065C','URSALA III','P-11 No. 4430',39017,'Reentered','1990-01-01 00:00:00','1980-05-03 00:00:00',96.68,598,602,96.34,'Sun-Synch' from dual union all
select 9009,'009009','1976-066A','Palapa A1','Palapa A1',39018,'In Earth orbit','','1978-02-23 00:00:00',1436.01,35779,35790,.03,'Stationary' from dual union all
select 901,'000901','1964-063D','Adapter','',10349,'In Earth orbit','','1964-11-21 00:00:00',106.6,1061,1078,89.92,'Polar' from dual union all
select 9010,'009010','1976-065D','Titan 23D-14 Stage 2','LR91AJ11 23D-14',39017,'Reentered','1976-07-10 00:00:00','1976-07-09 00:00:00',88.05,149,206,96.99,'Sun-Synch' from dual union all
select 9011,'009011','1976-067A','Kosmos-839','Lira',39019,'In Earth orbit','','1976-08-08 00:00:00',116.85,983,2096,65.86,'Intermediate' from dual union all
select 9012,'009012','1976-066B','Delta 125','Delta SSPU DSV-3P-4',39018,'Reentered','1976-07-10 00:00:00','1976-07-09 00:00:00',88.16,148,219,28.73,'Intermediate' from dual union all
select 9013,'009013','1976-067B','S3M','S3M',39019,'In Earth orbit','','1993-07-28 00:00:00',116.58,956,2098,65.87,'Intermediate' from dual union all
select 9014,'009014','1976-060C','KDU','KDU',39012,'Reentered','1976-07-20 00:00:00','1976-07-12 00:00:00',89.07,169,287,64.96,'Intermediate' from dual union all
select 9015,'009015','1976-060D','Blenda','Sunshade',39012,'Reentered','1976-07-14 00:00:00','1976-07-11 00:00:00',88.95,174,272,64.89,'Intermediate' from dual union all
select 9016,'009016','1976-067C','deb Kosmos-839','DS-P1-M part [RCS 0.57]',39019,'In Earth orbit','','1993-08-03 00:00:00',116.84,978,2100,65.87,'Intermediate' from dual union all
select 9017,'009017','1976-066C','Star 37C S/N 40035','Star 37C',39018,'Reentered','2004-12-06 00:00:00','1976-08-11 00:00:00',640.52,251,36221,24.74,'GEO Transfer' from dual union all
select 9018,'009018','1976-060E','PTDU cover','KDU part',39012,'Reentered','1976-07-16 00:00:00','1976-07-13 00:00:00',89.89,156,381,64.99,'Intermediate' from dual union all
select 9019,'009019','1976-068A','Kosmos-840','Zenit-2M',39020,'Landed','1976-07-26 00:00:00','1976-07-16 00:00:00',89.66,201,315,72.87,'Intermediate' from dual union all
select 902,'000902','1964-063E','Dragsphere 2','Dragsphere',10349,'In Earth orbit','','1964-11-09 00:00:00',106.58,1058,1078,89.92,'Polar' from dual union all
select 9020,'009020','1976-068B','11S510','11S510',39020,'Reentered','1976-07-25 00:00:00','1976-07-17 00:00:00',89.32,188,294,72.87,'Intermediate' from dual union all
select 9021,'009021','1976-068C','Sensor cover','Zenit-2M sensor cover',39020,'Reentered','1976-07-17 00:00:00','1976-07-16 00:00:00',89.2,202,268,72.85,'Intermediate' from dual union all
select 9022,'009022','1976-069A','Kosmos-841','Strela-2M',39021,'In Earth orbit','','1992-12-30 00:00:00',100.48,771,791,74.04,'Intermediate' from dual union all
select 9023,'009023','1976-069B','S3M','S3M',39021,'In Earth orbit','','1991-03-28 00:00:00',100.36,763,788,74.03,'Intermediate' from dual union all
select 9024,'009024','1975-075C','Viking Lander 1','Viking Lander 1',36522,'Deep Space Attached','1975-08-21 00:00:00','1975-08-20 00:00:00',429.09,183,50629,29.3,'Earth Escape' from dual union all
select 9025,'009025','1976-070A','Kosmos-842','Geoid',39022,'In Earth orbit','','1992-12-24 00:00:00',104.83,966,1006,82.98,'Intermediate' from dual union all
select 9026,'009026','1975-004CQ','deb Delta 107','',36451,'Reentered','1978-11-09 00:00:00','1978-09-26 00:00:00',101.07,697,922,97.56,'Sun-Synch' from dual union all
select 9027,'009027','1975-004CR','deb Delta 107','',36451,'Reentered','1978-02-04 00:00:00','1976-07-20 00:00:00',96.09,378,765,96.75,'Sun-Synch' from dual union all
select 9028,'009028','1975-004CS','deb Delta 107','',36451,'Reentered','1978-05-01 00:00:00','1978-02-15 00:00:00',93.44,382,504,97.29,'Sun-Synch' from dual union all
select 9029,'009029','1975-004CT','deb Delta 107','',36451,'Reentered','1980-10-22 00:00:00','1980-10-15 00:00:00',91.17,324,339,97.74,'Sun-Synch' from dual union all
select 903,'000903','1964-063F','deb NNSS 30010','',10349,'In Earth orbit','','1964-11-09 00:00:00',106.61,1061,1079,89.92,'Polar' from dual union all
select 9030,'009030','1975-004CU','deb Delta 107','',36451,'Reentered','1980-01-11 00:00:00','1976-07-19 00:00:00',97.82,527,782,96.95,'Sun-Synch' from dual union all
select 9031,'009031','1975-004CV','deb Delta 107','',36451,'Reentered','1979-12-14 00:00:00','1976-07-19 00:00:00',98.53,623,754,97.73,'Sun-Synch' from dual union all
select 9032,'009032','1975-004CW','deb Delta 107','',36451,'Reentered','1977-02-04 00:00:00','1976-07-20 00:00:00',95.28,296,769,98.26,'Sun-Synch' from dual union all
select 9033,'009033','1975-004CX','deb Delta 107','',36451,'Reentered','1979-09-21 00:00:00','1979-08-22 00:00:00',92.12,333,424,98.15,'Sun-Synch' from dual union all
select 9034,'009034','1975-004CY','deb Delta 107','',36451,'Reentered','1981-04-09 00:00:00','1976-07-20 00:00:00',98.47,617,754,97.71,'Sun-Synch' from dual union all
select 9035,'009035','1975-004CZ','deb Delta 107','',36451,'Reentered','1978-10-12 00:00:00','1978-10-01 00:00:00',91.99,326,418,98.38,'Sun-Synch' from dual union all
select 9036,'009036','1975-004DA','deb Delta 107','',36451,'Reentered','1980-02-03 00:00:00','1976-07-19 00:00:00',97.75,544,759,97.6,'Sun-Synch' from dual union all
select 9037,'009037','1975-004DB','deb Delta 107','',36451,'Reentered','1979-10-03 00:00:00','1976-07-20 00:00:00',97.08,476,762,97.7,'Sun-Synch' from dual union all
select 9038,'009038','1975-004DC','deb Delta 107','',36451,'Reentered','1977-01-31 00:00:00','1976-06-30 00:00:00',96.8,448,763,98.25,'Sun-Synch' from dual union all
select 9039,'009039','1975-004DD','deb Delta 107','',36451,'Reentered','1978-10-06 00:00:00','1978-09-24 00:00:00',90.84,306,326,97.66,'Sun-Synch' from dual union all
select 904,'000904','1964-065A','Voskhod','Voskhod 3KV No. 3',10351,'Landed','1964-10-13 00:00:00','1964-10-12 00:00:00',89.63,177,336,64.72,'Intermediate' from dual union all
select 9040,'009040','1975-004DE','deb Delta 107','',36451,'In Earth orbit','','1993-08-06 00:00:00',102.64,745,1022,97.72,'Sun-Synch' from dual union all
select 9041,'009041','1975-004DF','deb Delta 107','',36451,'Reentered','1980-09-13 00:00:00','1980-09-10 00:00:00',88.66,199,218,98.48,'Sun-Synch' from dual union all
select 9042,'009042','1975-004DG','deb Delta 107','',36451,'In Earth orbit','','1993-08-06 00:00:00',100.32,725,822,98.01,'Sun-Synch' from dual union all
select 9043,'009043','1976-071A','Kosmos-843','IS',39023,'Deorbited','1976-07-21 00:00:00','1976-07-21 00:00:00',89.2,130,340,65.11,'Intermediate' from dual union all
select 9044,'009044','1976-070B','S3M','S3M',39022,'In Earth orbit','','1991-03-24 00:00:00',104.69,968,992,82.99,'Intermediate' from dual union all
select 9045,'009045','1976-071B','11S692','11S692',39023,'Reentered','1976-07-23 00:00:00','1976-07-22 00:00:00',88.17,125,243,65.1,'Intermediate' from dual union all
select 9046,'009046','1976-072A','Kosmos-844','Yantar''-2K No. 5',39024,'Reentered','1976-08-30 00:00:00','1976-08-08 00:00:00',88.76,161,266,67.15,'Intermediate' from dual union all
select 9047,'009047','1976-073A','Comstar D-2','Comstar D-2',39025,'In Earth orbit','','1977-01-27 00:00:00',1436.06,35758,35813,.05,'Stationary' from dual union all
select 9048,'009048','1976-072B','11S510','11S510',39024,'Reentered','1976-07-30 00:00:00','1976-07-25 00:00:00',89.49,169,330,67.14,'Intermediate' from dual union all
select 9049,'009049','1976-074A','Molniya-1','Molniya-1 11F658 No. 43',39026,'Reentered','1987-05-29 00:00:00','1976-08-25 00:00:00',717.68,498,39852,62.94,'Molniya' from dual union all
select 905,'000905','1964-065B','11A57I','11A57I',10351,'Reentered','1964-10-20 00:00:00','1964-10-15 00:00:00',89.47,169,328,64.77,'Intermediate' from dual union all
select 9050,'009050','1976-074B','11S510','11S510',39026,'Reentered','1976-10-02 00:00:00','1976-08-22 00:00:00',90.96,231,413,62.9,'Intermediate' from dual union all
select 9051,'009051','1976-074C','BOZ','BOZ',39026,'Reentered','1976-09-19 00:00:00','1976-07-30 00:00:00',91.64,213,497,62.89,'Intermediate' from dual union all
select 9052,'009052','1976-074D','deb Blok-I','',39026,'Reentered','1976-09-01 00:00:00','1976-07-23 00:00:00',91.8,216,510,62.89,'Intermediate' from dual union all
select 9053,'009053','1976-075A','Kosmos-845','Tselina-OM',39027,'Reentered','1980-11-15 00:00:00','1976-08-27 00:00:00',95.19,511,545,74.05,'Intermediate' from dual union all
select 9054,'009054','1976-075B','S3M','S3M',39027,'Reentered','1981-02-28 00:00:00','1976-08-27 00:00:00',95.09,503,543,74.05,'Intermediate' from dual union all
select 9055,'009055','1976-076A','Interkosmos-16','DS-U3-IK No. 6',39028,'Reentered','1979-07-10 00:00:00','1976-08-26 00:00:00',94.43,464,518,50.57,'Intermediate' from dual union all
select 9056,'009056','1976-076B','S3M','S3M',39028,'Reentered','1980-02-21 00:00:00','1976-08-26 00:00:00',94.3,455,515,50.58,'Intermediate' from dual union all
select 9057,'009057','1976-077A','NOAA 5','ITOS H',39029,'In Earth orbit','','1991-03-28 00:00:00',116.28,1506,1521,101.98,'Sun-Synch' from dual union all
select 9058,'009058','1976-075C','deb Kosmos-845','',39027,'Reentered','1978-07-18 00:00:00','1976-08-27 00:00:00',94.9,484,544,74.07,'Intermediate' from dual union all
select 9059,'009059','1976-057D','deb Salyut-5','',39009,'Reentered','1976-08-13 00:00:00','1976-07-29 00:00:00',89.71,257,264,51.6,'Intermediate' from dual union all
select 906,'000906','1964-065C','deb Voskhod','',10351,'Reentered','1964-10-13 00:00:00','1964-10-12 00:00:00',89.63,177,336,64.72,'Intermediate' from dual union all
select 9060,'009060','1976-057E','deb Salyut-5','',39009,'Reentered','1976-08-13 00:00:00','1976-07-29 00:00:00',89.75,258,267,51.59,'Intermediate' from dual union all
select 9061,'009061','1976-078A','Kosmos-846','Zaliv',39030,'In Earth orbit','','1993-01-09 00:00:00',104.67,948,1009,82.93,'Intermediate' from dual union all
select 9062,'009062','1976-078B','S3M','S3M',39030,'In Earth orbit','','1991-03-27 00:00:00',104.53,950,995,82.92,'Intermediate' from dual union all
select 9063,'009063','1976-077B','Delta 126','Delta SSPU DSV-3P-4',39029,'Exploded','','1976-09-03 00:00:00',116.25,1505,1520,102.07,'Sun-Synch' from dual union all
select 9064,'009064','1976-072C','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.16,171,394,67.1,'Intermediate' from dual union all
select 9065,'009065','1976-072D','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.23,179,294,67.3,'Intermediate' from dual union all
select 9066,'009066','1976-072E','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.06,181,374,67.2,'Intermediate' from dual union all
select 9067,'009067','1976-072F','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.21,184,386,67.1,'Intermediate' from dual union all
select 9068,'009068','1976-072G','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.95,153,293,67.3,'Intermediate' from dual union all
select 9069,'009069','1976-072H','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.61,184,327,67.3,'Intermediate' from dual union all
select 907,'000907','1964-064B','Altair 2 X-258C2 RH-72','X-258C1',10350,'In Earth orbit','','1964-12-19 00:00:00',104.73,890,1074,79.71,'Intermediate' from dual union all
select 9070,'009070','1976-072J','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.08,169,388,67.3,'Intermediate' from dual union all
select 9071,'009071','1976-072K','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.66,191,325,67.3,'Intermediate' from dual union all
select 9072,'009072','1976-072L','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.02,135,318,67.2,'Intermediate' from dual union all
select 9073,'009073','1976-072M','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.83,155,378,67.2,'Intermediate' from dual union all
select 9074,'009074','1976-072N','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.64,147,268,67.1,'Intermediate' from dual union all
select 9075,'009075','1976-072P','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.19,169,300,67.1,'Intermediate' from dual union all
select 9076,'009076','1976-072Q','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.62,147,266,67.1,'Intermediate' from dual union all
select 9077,'009077','1976-072R','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.81,184,347,67.2,'Intermediate' from dual union all
select 9078,'009078','1976-072S','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.63,144,369,67.2,'Intermediate' from dual union all
select 9079,'009079','1976-072T','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.62,147,365,67.2,'Intermediate' from dual union all
select 908,'000908','1964-066A','Kosmos-48','Zenit-2 No. 24',10352,'Landed','1964-10-20 00:00:00','1964-10-15 00:00:00',89.35,203,282,65.09,'Intermediate' from dual union all
select 9080,'009080','1976-072U','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.26,178,397,67.1,'Intermediate' from dual union all
select 9081,'009081','1976-072V','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.12,186,375,67.1,'Intermediate' from dual union all
select 9082,'009082','1976-072W','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.23,149,323,67.2,'Intermediate' from dual union all
select 9083,'009083','1976-072X','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.94,138,307,67.3,'Intermediate' from dual union all
select 9084,'009084','1976-072Y','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.25,191,383,67.2,'Intermediate' from dual union all
select 9085,'009085','1976-072Z','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.43,163,330,67.1,'Intermediate' from dual union all
select 9086,'009086','1976-072AA','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.4,152,338,67.1,'Intermediate' from dual;
insert into SATELLITE
select 9087,'009087','1976-072AB','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.03,192,360,67.2,'Intermediate' from dual union all
select 9088,'009088','1976-072AC','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.6,160,350,67.3,'Intermediate' from dual union all
select 9089,'009089','1976-072AD','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.83,145,289,67.3,'Intermediate' from dual union all
select 909,'000909','1964-066B','8A92E','8A92E',10352,'Reentered','1964-10-28 00:00:00','1964-10-17 00:00:00',89.44,215,279,65.03,'Intermediate' from dual union all
select 9090,'009090','1976-072AE','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.61,173,338,67,'Intermediate' from dual union all
select 9091,'009091','1976-072AF','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.12,178,383,67.4,'Intermediate' from dual union all
select 9092,'009092','1976-072AG','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.14,145,319,67.1,'Intermediate' from dual union all
select 9093,'009093','1976-072AH','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.74,189,335,67.2,'Intermediate' from dual union all
select 9094,'009094','1976-072AJ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.74,206,317,67.2,'Intermediate' from dual union all
select 9095,'009095','1976-072AK','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.7,180,340,67,'Intermediate' from dual union all
select 9096,'009096','1976-072AL','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.53,169,334,67,'Intermediate' from dual union all
select 9097,'009097','1976-072AM','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.07,177,379,67.2,'Intermediate' from dual union all
select 9098,'009098','1976-072AN','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.28,149,329,67.2,'Intermediate' from dual union all
select 9099,'009099','1976-072AP','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.72,146,376,67.2,'Intermediate' from dual union all
select 91,'000091','61 Theta 1','Korabl''-Sputnik-4','Vostok-3 No. 1',7280,'Landed','1961-03-09 00:00:00','1961-03-09 00:00:00',88.61,173,239,64.93,'Intermediate' from dual union all
select 910,'000910','1964-066C','deb Kosmos-48','',10352,'Reentered','1964-10-16 00:00:00','1964-10-15 00:00:00',89.35,203,282,65.09,'Intermediate' from dual union all
select 9100,'009100','1976-072AQ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.13,176,386,67.2,'Intermediate' from dual union all
select 9101,'009101','1976-072AR','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.79,152,376,67.2,'Intermediate' from dual union all
select 9102,'009102','1976-072AS','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.7,161,359,67.2,'Intermediate' from dual union all
select 9103,'009103','1976-072AT','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.19,186,283,67.3,'Intermediate' from dual union all
select 9104,'009104','1976-072AU','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.67,173,344,67.2,'Intermediate' from dual union all
select 9105,'009105','1976-072AV','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.6,187,323,67.1,'Intermediate' from dual union all
select 9106,'009106','1976-072AW','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.66,166,350,67.1,'Intermediate' from dual union all
select 9107,'009107','1976-072AX','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.68,183,334,67,'Intermediate' from dual union all
select 9108,'009108','1976-072AY','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.27,187,389,67.1,'Intermediate' from dual union all
select 9109,'009109','1976-072AZ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.38,160,327,67.3,'Intermediate' from dual union all
select 911,'000911','1964-067A','OPS 3559','CORONA J-13',10353,'Reentered Attached','1964-11-04 00:00:00','1964-10-25 00:00:00',90.13,170,392,74.99,'Intermediate' from dual union all
select 9110,'009110','1976-072BA','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.57,174,333,67.3,'Intermediate' from dual union all
select 9111,'009111','1976-072BB','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.47,166,331,67.1,'Intermediate' from dual union all
select 9112,'009112','1976-072BC','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.35,185,300,67,'Intermediate' from dual union all
select 9113,'009113','1976-072BD','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.53,192,310,67.3,'Intermediate' from dual union all
select 9114,'009114','1976-072BE','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.63,143,271,67.1,'Intermediate' from dual union all
select 9115,'009115','1976-072BF','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.43,177,315,67.1,'Intermediate' from dual union all
select 9116,'009116','1976-072BG','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.73,178,345,67.2,'Intermediate' from dual union all
select 9117,'009117','1976-072BH','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.93,200,341,67.1,'Intermediate' from dual union all
select 9118,'009118','1976-072BJ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.18,149,319,67.2,'Intermediate' from dual union all
select 9119,'009119','1976-072BK','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.8,160,271,67.3,'Intermediate' from dual union all
select 912,'000912','1964-068A','OPS 4384','GAMBIT SV 963',10354,'Deorbited','1964-10-28 00:00:00','1964-10-27 00:00:00',89.03,148,306,95.5,'Sun-Synch' from dual union all
select 9120,'009120','1976-072BL','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.76,179,347,67.1,'Intermediate' from dual union all
select 9121,'009121','1976-072BM','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.2,163,307,67,'Intermediate' from dual union all
select 9122,'009122','1976-072BN','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.79,176,353,67.1,'Intermediate' from dual union all
select 9123,'009123','1976-072BP','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.15,184,379,67.2,'Intermediate' from dual union all
select 9124,'009124','1976-072BQ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.21,195,375,67.2,'Intermediate' from dual union all
select 9125,'009125','1976-072BR','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.73,145,278,67.2,'Intermediate' from dual union all
select 9126,'009126','1976-072BS','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.38,193,393,67.2,'Intermediate' from dual union all
select 9127,'009127','1976-072BT','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.09,174,384,67.2,'Intermediate' from dual union all
select 9128,'009128','1976-072BU','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.56,126,281,67.2,'Intermediate' from dual union all
select 9129,'009129','1976-072BV','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.06,178,377,67.1,'Intermediate' from dual union all
select 913,'000913','1964-069A','Kosmos-49','DS-MG No. 2',10355,'Reentered','1965-08-21 00:00:00','1964-11-28 00:00:00',91.73,259,460,48.95,'Intermediate' from dual union all
select 9130,'009130','1976-072BW','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.7,147,373,67.2,'Intermediate' from dual union all
select 9131,'009131','1976-072BX','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.29,193,384,66.9,'Intermediate' from dual union all
select 9132,'009132','1976-072BY','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.96,177,368,67.1,'Intermediate' from dual union all
select 9133,'009133','1976-072BZ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.61,167,344,67.3,'Intermediate' from dual union all
select 9134,'009134','1976-072CA','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.65,145,370,67,'Intermediate' from dual union all
select 9135,'009135','1976-072CB','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.48,183,315,67.1,'Intermediate' from dual union all
select 9136,'009136','1976-072CC','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.82,166,267,67.1,'Intermediate' from dual union all
select 9137,'009137','1976-072CD','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.37,140,347,67,'Intermediate' from dual union all
select 9138,'009138','1976-072CE','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.77,165,263,67.1,'Intermediate' from dual union all
select 9139,'009139','1976-072CF','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.69,142,278,67.1,'Intermediate' from dual union all
select 914,'000914','1964-068B','STEP 13','P-11 No. 4302',10354,'Reentered','1965-02-23 00:00:00','1964-11-26 00:00:00',90.81,295,334,95.5,'Sun-Synch' from dual union all
select 9140,'009140','1976-072CG','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.44,182,312,67.2,'Intermediate' from dual union all
select 9141,'009141','1976-072CH','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.07,162,393,67.3,'Intermediate' from dual union all
select 9142,'009142','1976-072CJ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.28,181,396,67.2,'Intermediate' from dual union all
select 9143,'009143','1976-072CK','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.67,155,263,67.3,'Intermediate' from dual union all
select 9144,'009144','1976-072CL','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.16,175,390,67.2,'Intermediate' from dual union all
select 9145,'009145','1976-072CM','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.23,177,395,67,'Intermediate' from dual union all
select 9146,'009146','1976-072CN','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.6,134,276,67.1,'Intermediate' from dual union all
select 9147,'009147','1976-072CP','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.2,170,300,67.1,'Intermediate' from dual union all
select 9148,'009148','1976-072CQ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.43,195,396,67.1,'Intermediate' from dual union all
select 9149,'009149','1976-072CR','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.61,155,355,67.2,'Intermediate' from dual union all
select 915,'000915','1964-069B','S1','S1',10355,'Reentered','1965-02-10 00:00:00','1964-11-23 00:00:00',91.44,252,438,48.97,'Intermediate' from dual union all
select 9150,'009150','1976-072CS','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.97,134,314,67.1,'Intermediate' from dual union all
select 9151,'009151','1976-072CT','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.36,178,308,67.2,'Intermediate' from dual union all
select 9152,'009152','1976-072CU','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.9,198,340,67,'Intermediate' from dual union all
select 9153,'009153','1976-072CV','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.2,174,395,67.1,'Intermediate' from dual union all
select 9154,'009154','1976-072CW','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.77,190,337,67.1,'Intermediate' from dual union all
select 9155,'009155','1976-072CX','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.02,182,368,67,'Intermediate' from dual union all
select 9156,'009156','1976-072CY','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.69,174,345,67.1,'Intermediate' from dual union all
select 9157,'009157','1976-072CZ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.05,188,365,67.2,'Intermediate' from dual union all
select 9158,'009158','1976-072DA','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.39,134,255,67,'Intermediate' from dual union all
select 9159,'009159','1976-072DB','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.86,124,313,67.1,'Intermediate' from dual union all
select 916,'000916','1964-068C','Agena D 4811','Agena D',10354,'Reentered','1964-11-04 00:00:00','1964-10-27 00:00:00',88.07,149,208,95.5,'Sun-Synch' from dual union all
select 9160,'009160','1976-072DC','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.91,149,293,67.1,'Intermediate' from dual union all
select 9161,'009161','1976-072DD','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.44,144,350,67.1,'Intermediate' from dual union all
select 9162,'009162','1976-072DE','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.75,158,366,67.3,'Intermediate' from dual union all
select 9163,'009163','1976-072DF','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.79,175,353,67,'Intermediate' from dual union all
select 9164,'009164','1976-072DG','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.47,160,337,67.2,'Intermediate' from dual union all
select 9165,'009165','1976-072DH','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.92,168,372,67.1,'Intermediate' from dual union all
select 9166,'009166','1976-072DJ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.97,152,394,67.3,'Intermediate' from dual union all
select 9167,'009167','1976-072DK','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.36,177,309,67.2,'Intermediate' from dual union all
select 9168,'009168','1976-072DL','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.9,192,346,67.1,'Intermediate' from dual union all
select 9169,'009169','1976-072DM','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.26,150,325,67,'Intermediate' from dual union all
select 917,'000917','1964-069C','Kosmos-49 boom cover','',10355,'Reentered','1964-11-17 00:00:00','1964-10-31 00:00:00',91.11,258,400,48.91,'Intermediate' from dual union all
select 9170,'009170','1976-072DN','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.19,141,327,67,'Intermediate' from dual union all
select 9171,'009171','1976-072DP','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.68,173,345,67,'Intermediate' from dual union all
select 9172,'009172','1976-072DQ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.97,175,370,67.3,'Intermediate' from dual union all
select 9173,'009173','1976-072DR','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.18,175,391,67.2,'Intermediate' from dual union all
select 9174,'009174','1976-072DS','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.61,124,288,67.1,'Intermediate' from dual union all
select 9175,'009175','1976-072DT','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.26,184,291,67.1,'Intermediate' from dual union all
select 9176,'009176','1976-072DU','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.53,185,317,67.1,'Intermediate' from dual;
insert into SATELLITE
select 9177,'009177','1976-072DV','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.29,149,330,67,'Intermediate' from dual union all
select 9178,'009178','1976-072DW','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.96,124,323,67.2,'Intermediate' from dual union all
select 9179,'009179','1976-072DX','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.03,182,370,67.1,'Intermediate' from dual union all
select 918,'000918','1964-068D','SRV Mission 4013','',10354,'Reentered','1964-10-29 00:00:00','1964-10-27 00:00:00',88.89,140,300,95.5,'Sun-Synch' from dual union all
select 9180,'009180','1976-072DY','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.59,123,287,67.2,'Intermediate' from dual union all
select 9181,'009181','1976-072DZ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.71,167,354,67.1,'Intermediate' from dual union all
select 9182,'009182','1976-072EA','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.63,131,283,67.1,'Intermediate' from dual union all
select 9183,'009183','1976-072EB','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.89,185,353,67,'Intermediate' from dual union all
select 9184,'009184','1976-072EC','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.57,163,344,67.1,'Intermediate' from dual union all
select 9185,'009185','1976-072ED','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.35,157,328,67.1,'Intermediate' from dual union all
select 9186,'009186','1976-072EE','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.45,133,362,67.3,'Intermediate' from dual union all
select 9187,'009187','1976-072EF','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.78,173,355,67.1,'Intermediate' from dual union all
select 9188,'009188','1976-072EG','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.42,201,389,67.1,'Intermediate' from dual union all
select 9189,'009189','1976-072EH','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.24,180,393,67.3,'Intermediate' from dual union all
select 919,'000919','1964-070A','Kosmos-50','Zenit-2 No. 25',10356,'Exploded','1964-11-05 00:00:00','1964-10-29 00:00:00',88.69,187,233,51.23,'Intermediate' from dual union all
select 9190,'009190','1976-072EJ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.23,195,376,67.1,'Intermediate' from dual union all
select 9191,'009191','1976-072EK','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.85,150,286,67.2,'Intermediate' from dual union all
select 9192,'009192','1976-072EL','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.17,174,392,67,'Intermediate' from dual union all
select 9193,'009193','1976-072EM','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.06,178,377,67.2,'Intermediate' from dual union all
select 9194,'009194','1976-072EN','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.88,178,358,67,'Intermediate' from dual union all
select 9195,'009195','1976-072EP','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.9,192,346,67.2,'Intermediate' from dual union all
select 9196,'009196','1976-072EQ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.33,184,298,67,'Intermediate' from dual union all
select 9197,'009197','1976-072ER','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.64,128,287,67.1,'Intermediate' from dual union all
select 9198,'009198','1976-072ES','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.76,185,341,67.1,'Intermediate' from dual union all
select 9199,'009199','1976-072ET','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.39,181,308,67.1,'Intermediate' from dual union all
select 92,'000092','61 Theta 2','8K72KE','8K72KE',7280,'Reentered','1961-03-10 00:00:00','1961-03-09 00:00:00',88.19,185,185,64.93,'Intermediate' from dual union all
select 920,'000920','1964-070B','8A92E','8A92E',10356,'Reentered','1964-11-02 00:00:00','1964-10-31 00:00:00',88.48,181,217,51.24,'Intermediate' from dual union all
select 9200,'009200','1976-072EU','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.62,127,385,67.1,'Intermediate' from dual union all
select 9201,'009201','1976-072EV','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.56,164,342,67,'Intermediate' from dual union all
select 9202,'009202','1976-072EW','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.15,191,373,67.2,'Intermediate' from dual union all
select 9203,'009203','1976-072EX','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.03,173,280,67.2,'Intermediate' from dual union all
select 9204,'009204','1976-072EY','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.81,163,268,67.1,'Intermediate' from dual union all
select 9205,'009205','1976-072EZ','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.72,146,376,67.4,'Intermediate' from dual union all
select 9206,'009206','1976-072FA','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.37,140,347,67.2,'Intermediate' from dual union all
select 9207,'009207','1976-072FB','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.84,128,307,67.2,'Intermediate' from dual union all
select 9208,'009208','1976-072FC','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.16,177,387,67.1,'Intermediate' from dual union all
select 9209,'009209','1976-072FD','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.1,192,367,67.2,'Intermediate' from dual union all
select 921,'000921','1964-071A','OPS 5434','CORONA J-15',10357,'Reentered Attached','1964-11-28 00:00:00','1964-11-13 00:00:00',90.47,182,414,79.97,'Intermediate' from dual union all
select 9210,'009210','1976-072FE','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.35,185,399,67.1,'Intermediate' from dual union all
select 9211,'009211','1976-072FF','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',88.76,123,304,67.3,'Intermediate' from dual union all
select 9212,'009212','1976-072FG','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',89.66,189,326,67.1,'Intermediate' from dual union all
select 9213,'009213','1976-072FH','deb Kosmos-844','',39024,'Reentered','1976-08-02 00:00:00','1976-07-31 00:00:00',90.25,183,391,67.1,'Intermediate' from dual union all
select 9214,'009214','1976-079A','Kosmos-847','Zenit-4MK',39031,'Landed','1976-08-17 00:00:00','1976-08-06 00:00:00',89.47,180,316,62.8,'Intermediate' from dual union all
select 9215,'009215','1976-079B','11S510','11S510',39031,'Reentered','1976-08-11 00:00:00','1976-08-06 00:00:00',89.24,176,298,62.78,'Intermediate' from dual union all
select 9216,'009216','1976-079C','Sensor cover','Zenit-4MK sensor cover',39031,'Reentered','1976-08-05 00:00:00','1976-08-04 00:00:00',89.06,173,282,62.74,'Intermediate' from dual union all
select 9217,'009217','1976-072FJ','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.62,183,329,67.2,'Intermediate' from dual union all
select 9218,'009218','1976-072FK','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',90.04,166,386,67.2,'Intermediate' from dual union all
select 9219,'009219','1976-072FL','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.35,145,339,67.2,'Intermediate' from dual union all
select 922,'000922','1964-072A','OPS 3062','Project 698BK Group 3-D',10358,'Reentered Attached','1969-11-05 00:00:00','1964-12-14 00:00:00',94.95,507,526,82.04,'Intermediate' from dual union all
select 9220,'009220','1976-072FM','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.06,138,317,67,'Intermediate' from dual union all
select 9221,'009221','1976-072FN','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.81,175,355,67,'Intermediate' from dual union all
select 9222,'009222','1976-072FP','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.15,148,316,67.1,'Intermediate' from dual union all
select 9223,'009223','1976-072FQ','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.57,167,340,67.1,'Intermediate' from dual union all
select 9224,'009224','1976-072FR','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',90.22,176,395,67.1,'Intermediate' from dual union all
select 9225,'009225','1976-072FS','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.9,164,375,67.2,'Intermediate' from dual union all
select 9226,'009226','1976-072FT','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.89,176,362,67.2,'Intermediate' from dual union all
select 9227,'009227','1976-072FU','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',90.39,200,388,67.4,'Intermediate' from dual union all
select 9228,'009228','1976-072FV','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.82,151,380,67.1,'Intermediate' from dual union all
select 9229,'009229','1976-072FW','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.03,173,280,67.1,'Intermediate' from dual union all
select 923,'000923','1964-073A','Mariner 3','Mariner C-2',10359,'Deep Space','1964-11-06 00:00:00','',null,null,null,null,'' from dual union all
select 9230,'009230','1976-072FX','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.97,168,378,67.1,'Intermediate' from dual union all
select 9231,'009231','1976-072FY','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',90.11,190,370,67.1,'Intermediate' from dual union all
select 9232,'009232','1976-072FZ','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',90.12,174,387,67.2,'Intermediate' from dual union all
select 9233,'009233','1976-072GA','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.17,180,287,67.2,'Intermediate' from dual union all
select 9234,'009234','1976-072GB','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.12,184,278,67.1,'Intermediate' from dual union all
select 9235,'009235','1976-072GC','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.7,172,348,67.3,'Intermediate' from dual union all
select 9236,'009236','1976-072GD','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.23,203,270,67.2,'Intermediate' from dual union all
select 9237,'009237','1976-072GE','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',90.11,171,389,67.1,'Intermediate' from dual union all
select 9238,'009238','1976-072GF','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',88.62,132,281,67.1,'Intermediate' from dual union all
select 9239,'009239','1976-072GG','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',88.85,173,263,67.3,'Intermediate' from dual union all
select 924,'000924','1964-074A','Explorer 23','NASA S-55C',10360,'Reentered Attached','1983-06-29 00:00:00','1964-12-06 00:00:00',99.21,464,978,51.96,'Intermediate' from dual union all
select 9240,'009240','1976-072GH','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.6,174,336,67.1,'Intermediate' from dual union all
select 9241,'009241','1976-072GJ','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',88.6,134,276,67.1,'Intermediate' from dual union all
select 9242,'009242','1976-072GK','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',90.1,176,383,67.2,'Intermediate' from dual union all
select 9243,'009243','1976-072GL','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.52,196,305,67.1,'Intermediate' from dual union all
select 9244,'009244','1976-072GM','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.29,164,314,67.1,'Intermediate' from dual union all
select 9245,'009245','1976-072GN','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.38,187,301,67.3,'Intermediate' from dual union all
select 9246,'009246','1976-072GP','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',90.24,175,398,67.1,'Intermediate' from dual union all
select 9247,'009247','1976-072GQ','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',88.98,150,299,67,'Intermediate' from dual union all
select 9248,'009248','1976-072GR','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.25,175,299,67.4,'Intermediate' from dual union all
select 9249,'009249','1976-072GS','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89,133,318,67.2,'Intermediate' from dual union all
select 925,'000925','1964-072B','Ferret sensor cover','',10358,'Reentered','1967-10-02 00:00:00','1964-12-12 00:00:00',94.9,504,524,82.04,'Intermediate' from dual union all
select 9250,'009250','1976-072GT','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.9,167,372,67.2,'Intermediate' from dual union all
select 9251,'009251','1976-072GU','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.64,182,332,67.2,'Intermediate' from dual union all
select 9252,'009252','1976-072GV','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.44,157,337,67.3,'Intermediate' from dual union all
select 9253,'009253','1976-072GW','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.96,190,354,67,'Intermediate' from dual union all
select 9254,'009254','1976-072GX','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',90.43,194,398,67,'Intermediate' from dual union all
select 9255,'009255','1976-072GY','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',90.28,188,389,67.1,'Intermediate' from dual union all
select 9256,'009256','1976-072GZ','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.64,156,358,67.2,'Intermediate' from dual union all
select 9257,'009257','1976-072HA','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.93,190,352,67.3,'Intermediate' from dual union all
select 9258,'009258','1976-072HB','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',90.16,191,374,67.2,'Intermediate' from dual union all
select 9259,'009259','1976-072HC','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.62,162,349,67.3,'Intermediate' from dual union all
select 926,'000926','1964-072C','Ferret sensor cover','',10358,'Reentered','1966-06-30 00:00:00','1964-12-04 00:00:00',94.89,506,521,82.06,'Intermediate' from dual union all
select 9260,'009260','1976-072HD','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.98,165,381,67.3,'Intermediate' from dual union all
select 9261,'009261','1976-072HE','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.27,183,294,67.2,'Intermediate' from dual union all
select 9262,'009262','1976-072HF','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.4,172,318,66.9,'Intermediate' from dual union all
select 9263,'009263','1976-072HG','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.52,180,322,67.3,'Intermediate' from dual union all
select 9264,'009264','1976-072HH','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.01,138,314,67.1,'Intermediate' from dual union all
select 9265,'009265','1976-072HJ','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',90.16,172,393,67,'Intermediate' from dual union all
select 9266,'009266','1976-072HK','deb Kosmos-844','',39024,'Reentered','1976-08-05 00:00:00','1976-07-31 00:00:00',89.76,183,343,67.1,'Intermediate' from dual;
insert into SATELLITE
select 9267,'009267','1976-075D','deb Kosmos-845','Tselina-O part',39027,'Reentered','1978-10-14 00:00:00','1976-09-01 00:00:00',97.18,498,750,74.01,'Intermediate' from dual union all
select 9268,'009268','1976-075E','deb Kosmos-845','Tselina-O part',39027,'Reentered','1979-02-17 00:00:00','1976-08-26 00:00:00',97.2,508,741,74.02,'Intermediate' from dual union all
select 9269,'009269','1976-074E','Blok-L','Blok-L',39026,'Reentered','1991-09-09 00:00:00','1976-08-24 00:00:00',698.47,493,38905,62.91,'Molniya' from dual union all
select 927,'000927','1964-072D','Ferret sensor cover','',10358,'Reentered','1966-06-30 00:00:00','1964-12-11 00:00:00',94.9,506,522,82.02,'Intermediate' from dual union all
select 9270,'009270','1976-080A','OPS 7940','QUASAR 2',39032,'In Earth orbit','','1976-08-06 00:00:00',702.01,350,39224,62.5,'Molniya' from dual union all
select 9271,'009271','1976-080B','Agena D','Agena D',39032,'In Earth orbit','','1976-08-06 00:00:00',702.01,350,39224,62.5,'Molniya' from dual union all
select 9272,'009272','1976-081A','Luna-24 KT','E-8-5M No. 413',39033,'Deep Space','1976-08-10 00:00:00','1976-08-09 00:00:00',15562.18,200,400000,51.5,'Cislunar' from dual union all
select 9273,'009273','1976-081B','SOZ','SOZ',39033,'Reentered','1976-08-09 00:00:00','1976-08-09 00:00:00',88.91,194,248,51.54,'Intermediate' from dual union all
select 9274,'009274','1976-081C','SOZ','SOZ',39033,'Reentered','1976-08-15 00:00:00','1976-08-09 00:00:00',88.91,194,248,51.54,'Intermediate' from dual union all
select 9275,'009275','1976-057F','deb Salyut-5','',39009,'Reentered','1976-08-17 00:00:00','1976-08-09 00:00:00',89.28,239,239,51.61,'Intermediate' from dual union all
select 9276,'009276','1976-057G','deb Salyut-5','',39009,'Reentered','1976-09-05 00:00:00','1976-08-11 00:00:00',89.55,249,256,51.58,'Intermediate' from dual union all
select 9277,'009277','1976-057H','deb Salyut-5','',39009,'Reentered','1976-09-05 00:00:00','1976-08-10 00:00:00',89.6,252,258,51.6,'Intermediate' from dual union all
select 9278,'009278','1976-081D','Blok D-1 part','debris',39033,'Reentered','1976-08-14 00:00:00','1976-08-09 00:00:00',88.91,194,248,51.54,'Intermediate' from dual union all
select 9279,'009279','1969-029AN','deb Blok-E','',18296,'Reentered','1979-06-16 00:00:00','1979-04-25 00:00:00',94.4,411,568,81.19,'Intermediate' from dual union all
select 928,'000928','1964-070C','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.97,194,254,51.2,'Intermediate' from dual union all
select 9280,'009280','1976-082A','Kosmos-848','Zenit-2M',39034,'Landed','1976-08-25 00:00:00','1976-08-14 00:00:00',89.56,204,302,62.81,'Intermediate' from dual union all
select 9281,'009281','1976-082B','11S510','11S510',39034,'Reentered','1976-08-24 00:00:00','1976-08-14 00:00:00',89.41,201,290,62.8,'Intermediate' from dual union all
select 9282,'009282','1975-004DH','deb Delta 107','',36451,'Reentered','1979-12-10 00:00:00','1979-11-06 00:00:00',93.98,446,493,97.35,'Sun-Synch' from dual union all
select 9283,'009283','1975-004DJ','deb Delta 107','',36451,'Reentered','1979-12-02 00:00:00','1979-11-15 00:00:00',92.66,381,429,96.78,'Sun-Synch' from dual union all
select 9284,'009284','1975-004DK','deb Delta 107','',36451,'Reentered','1979-10-04 00:00:00','1979-08-11 00:00:00',95.68,515,588,96.43,'Sun-Synch' from dual union all
select 9285,'009285','1975-004DL','deb Delta 107','',36451,'Reentered','1977-01-31 00:00:00','1976-08-12 00:00:00',96.33,447,719,99.67,'Sun-Synch' from dual union all
select 9286,'009286','1975-004DM','deb Delta 107','',36451,'Reentered','1977-04-25 00:00:00','1977-04-19 00:00:00',93.06,411,437,99.41,'Sun-Synch' from dual union all
select 9287,'009287','1975-004DN','deb Delta 107','',36451,'Reentered','1979-04-20 00:00:00','1979-03-16 00:00:00',94.67,444,562,98.07,'Sun-Synch' from dual union all
select 9288,'009288','1975-004DP','deb Delta 107','',36451,'Reentered','1977-10-01 00:00:00','1977-09-07 00:00:00',91.15,306,356,97.7,'Sun-Synch' from dual union all
select 9289,'009289','1975-004DQ','deb Delta 107','',36451,'Reentered','1978-01-22 00:00:00','1976-08-12 00:00:00',96.32,390,775,97.7,'Sun-Synch' from dual union all
select 929,'000929','1964-070D','deb Kosmos-50','',10356,'Reentered','1964-11-15 00:00:00','1964-11-05 00:00:00',88.13,136,228,51.2,'Intermediate' from dual union all
select 9290,'009290','1975-004DR','deb Delta 107','',36451,'Reentered','1977-05-27 00:00:00','1977-05-15 00:00:00',91.42,301,388,97.48,'Sun-Synch' from dual union all
select 9291,'009291','1975-004DS','deb Delta 107','',36451,'Reentered','1980-06-30 00:00:00','1976-08-12 00:00:00',98.36,608,753,97.46,'Sun-Synch' from dual union all
select 9292,'009292','1975-004DT','deb Delta 107','',36451,'Reentered','1984-05-24 00:00:00','1976-08-11 00:00:00',99.68,608,878,97.42,'Sun-Synch' from dual union all
select 9293,'009293','1975-004DU','deb Delta 107','',36451,'In Earth orbit','','1993-08-02 00:00:00',99.52,666,806,97.89,'Sun-Synch' from dual union all
select 9294,'009294','1975-004DV','deb Delta 107','',36451,'Reentered','1979-02-26 00:00:00','1976-08-12 00:00:00',97.66,534,760,98.11,'Sun-Synch' from dual union all
select 9295,'009295','1975-004DW','deb Delta 107','',36451,'Reentered','1979-01-27 00:00:00','1976-08-12 00:00:00',98.9,655,758,97.83,'Sun-Synch' from dual union all
select 9296,'009296','1975-004DX','deb Delta 107','',36451,'Reentered','1981-05-18 00:00:00','1981-04-26 00:00:00',96.52,549,635,97.99,'Sun-Synch' from dual union all
select 9297,'009297','1975-004DY','deb Delta 107','',36451,'Reentered','1977-01-31 00:00:00','1976-08-09 00:00:00',96.46,450,729,98.9,'Sun-Synch' from dual union all
select 9298,'009298','1975-004DZ','deb Delta 107','',36451,'Reentered','1977-02-09 00:00:00','1976-08-12 00:00:00',96.45,455,723,99.15,'Sun-Synch' from dual union all
select 9299,'009299','1975-004EA','deb Delta 107','',36451,'Reentered','1978-06-30 00:00:00','1978-01-02 00:00:00',95.5,430,655,98.09,'Sun-Synch' from dual union all
select 93,'000093','61 Theta 3','deb Korabl''-Sputnik-4','',7280,'Reentered','1961-03-10 00:00:00','1961-03-09 00:00:00',87.93,172,172,64.93,'Intermediate' from dual union all
select 930,'000930','1964-075A','OPS 3360','CORONA J-16',10361,'Reentered Attached','1964-12-06 00:00:00','1964-11-27 00:00:00',89.5,174,326,70.01,'Intermediate' from dual union all
select 9300,'009300','1975-004EB','deb Delta 107','',36451,'Reentered','1982-11-19 00:00:00','1976-08-11 00:00:00',106.55,693,1441,98.45,'Sun-Synch' from dual union all
select 9301,'009301','1975-004EC','deb Delta 107','',36451,'Reentered','1979-01-04 00:00:00','1976-08-12 00:00:00',105.85,678,1391,98.92,'Sun-Synch' from dual union all
select 9302,'009302','1975-004ED','deb Delta 107','',36451,'Reentered','1979-04-15 00:00:00','1979-03-26 00:00:00',95.14,483,568,98.89,'Sun-Synch' from dual union all
select 9303,'009303','1975-004EE','deb Delta 107','',36451,'Reentered','1980-04-06 00:00:00','1976-08-11 00:00:00',100.33,622,927,98.29,'Sun-Synch' from dual union all
select 9304,'009304','1975-004EF','deb Delta 107','',36451,'Reentered','1980-09-05 00:00:00','1976-08-12 00:00:00',100.2,627,908,98.36,'Sun-Synch' from dual union all
select 9305,'009305','1975-004EG','deb Delta 107','',36451,'Reentered','1989-01-24 00:00:00','1989-01-23 00:00:00',89.29,236,242,98.15,'Sun-Synch' from dual union all
select 9306,'009306','1975-004EH','deb Delta 107','',36451,'Reentered','1980-12-21 00:00:00','1980-12-18 00:00:00',90.84,312,320,98.02,'Sun-Synch' from dual union all
select 9307,'009307','1975-004EJ','deb Delta 107','',36451,'Reentered','1979-10-21 00:00:00','1979-08-23 00:00:00',96.79,582,628,97.86,'Sun-Synch' from dual union all
select 9308,'009308','1975-004EK','deb Delta 107','',36451,'Reentered','1981-05-09 00:00:00','1981-04-12 00:00:00',95.48,528,556,97.91,'Sun-Synch' from dual union all
select 9309,'009309','1975-004EL','deb Delta 107','',36451,'Reentered','1981-08-11 00:00:00','1976-08-12 00:00:00',99.22,691,752,97.76,'Sun-Synch' from dual union all
select 931,'000931','1964-076A','Explorer 24','AD-B',10362,'Reentered','1968-10-18 00:00:00','1964-12-26 00:00:00',116.22,542,2480,81.35,'Intermediate' from dual union all
select 9310,'009310','1975-004EM','deb Delta 107','',36451,'Reentered','1977-07-07 00:00:00','1976-08-12 00:00:00',95.91,332,794,97.62,'Sun-Synch' from dual union all
select 9311,'009311','1975-004EN','deb Delta 107','',36451,'Reentered','1979-06-16 00:00:00','1979-04-06 00:00:00',97.6,594,693,97.6,'Sun-Synch' from dual union all
select 9312,'009312','1975-004EP','deb Delta 107','',36451,'Reentered','1978-05-08 00:00:00','1978-01-06 00:00:00',95.75,403,707,97.6,'Sun-Synch' from dual union all
select 9313,'009313','1975-004EQ','deb Delta 107','',36451,'In Earth orbit','','1993-08-01 00:00:00',100.46,712,849,97.55,'Sun-Synch' from dual union all
select 9314,'009314','1975-004ER','deb Delta 107','',36451,'Reentered','1982-12-01 00:00:00','1982-11-25 00:00:00',90.59,288,319,97.54,'Sun-Synch' from dual union all
select 9315,'009315','1975-004ES','deb Delta 107','',36451,'Reentered','1979-02-13 00:00:00','1979-01-17 00:00:00',93.62,404,500,97.69,'Sun-Synch' from dual union all
select 9316,'009316','1975-004ET','deb Delta 107','',36451,'Reentered','1977-12-28 00:00:00','1977-11-20 00:00:00',93.03,354,492,97.25,'Sun-Synch' from dual union all
select 9317,'009317','1975-004EU','deb Delta 107','',36451,'Reentered','1981-04-21 00:00:00','1981-04-18 00:00:00',90.11,272,288,97.96,'Sun-Synch' from dual union all
select 9318,'009318','1975-004EV','deb Delta 107','',36451,'Reentered','1979-06-04 00:00:00','1979-03-12 00:00:00',93.27,394,476,97.84,'Sun-Synch' from dual union all
select 9319,'009319','1975-004EW','deb Delta 107','',36451,'Reentered','1980-05-04 00:00:00','1976-08-13 00:00:00',99.41,525,936,97.92,'Sun-Synch' from dual union all
select 932,'000932','1964-076B','Explorer 25','Injun 4',10362,'In Earth orbit','','1964-12-21 00:00:00',116.2,528,2492,81.35,'Intermediate' from dual union all
select 9320,'009320','1975-004EX','deb Delta 107','',36451,'In Earth orbit','','1993-08-02 00:00:00',98.83,658,748,98.42,'Sun-Synch' from dual union all
select 9321,'009321','1975-004EY','deb Delta 107','',36451,'Reentered','1980-02-05 00:00:00','1980-02-02 00:00:00',94.09,439,510,100,'Sun-Synch' from dual union all
select 9322,'009322','1975-004EZ','deb Delta 107','',36451,'Reentered','1977-04-12 00:00:00','1977-04-07 00:00:00',92.83,370,456,99.13,'Sun-Synch' from dual union all
select 9323,'009323','1973-088F','deb USAF satellite','',30690,'In Earth orbit','','1993-08-08 00:00:00',114.4,1412,1445,97.03,'Sun-Synch' from dual union all
select 9324,'009324','1975-004FB','deb Delta 107','',36451,'Reentered','1979-09-08 00:00:00','1976-08-16 00:00:00',96.74,432,773,98.35,'Sun-Synch' from dual union all
select 9325,'009325','1975-004FC','deb Delta 107','',36451,'Reentered','1980-10-11 00:00:00','1976-08-28 00:00:00',100.15,631,901,98.35,'Sun-Synch' from dual union all
select 9326,'009326','1975-004FD','deb Delta 107','',36451,'Reentered','1978-04-17 00:00:00','1978-03-05 00:00:00',93.55,351,545,98.22,'Sun-Synch' from dual union all
select 9327,'009327','1975-004FE','deb Delta 107','',36451,'Reentered','1980-03-23 00:00:00','1976-08-12 00:00:00',98.12,548,789,97.29,'Sun-Synch' from dual union all
select 9328,'009328','1976-082C','deb Kosmos-848','',39034,'Reentered','1976-08-17 00:00:00','1976-08-14 00:00:00',89.18,205,263,62.81,'Intermediate' from dual union all
select 9329,'009329','1976-073B','Centaur AC-40','Centaur D-1AR AC-40',39025,'In Earth orbit','','1976-08-24 00:00:00',648.17,584,36279,21.86,'GEO Transfer' from dual union all
select 933,'000933','1964-076C','Altair 2 X-258C2 RH-74','X-258C2',10362,'In Earth orbit','','1965-01-05 00:00:00',116.23,531,2492,81.34,'Intermediate' from dual union all
select 9330,'009330','1974-101G','Star 37E S/N 40029','Star 37E',33704,'In Earth orbit','','1976-06-28 00:00:00',682.92,427,38194,12.65,'GEO Transfer' from dual union all
select 9331,'009331','1976-072HL','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',88.94,167,278,67.3,'Intermediate' from dual union all
select 9332,'009332','1976-072HM','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.38,178,310,67.1,'Intermediate' from dual union all
select 9333,'009333','1976-072HN','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.87,153,383,67.2,'Intermediate' from dual union all
select 9334,'009334','1976-072HP','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.22,198,274,67.1,'Intermediate' from dual union all
select 9335,'009335','1976-072HQ','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.33,188,295,67.2,'Intermediate' from dual union all
select 9336,'009336','1976-072HR','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.94,185,358,67.1,'Intermediate' from dual union all
select 9337,'009337','1976-072HS','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.6,168,341,67.2,'Intermediate' from dual union all
select 9338,'009338','1976-072HT','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.88,189,348,67,'Intermediate' from dual union all
select 9339,'009339','1976-072HU','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.58,158,350,67.2,'Intermediate' from dual union all
select 934,'000934','1964-076D','AD canister cover','',10362,'Reentered','1982-10-02 00:00:00','1964-12-25 00:00:00',116.32,535,2496,81.34,'Intermediate' from dual union all
select 9340,'009340','1976-072HV','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.96,173,372,67.2,'Intermediate' from dual union all
select 9341,'009341','1976-072HW','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.98,162,385,67.1,'Intermediate' from dual union all
select 9342,'009342','1976-072HX','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.71,149,371,67.2,'Intermediate' from dual union all
select 9343,'009343','1976-072HY','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',88.78,160,269,67.1,'Intermediate' from dual union all
select 9344,'009344','1976-072HZ','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',90.12,185,375,67.1,'Intermediate' from dual union all
select 9345,'009345','1976-072JA','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',88.53,140,264,67.2,'Intermediate' from dual union all
select 9346,'009346','1976-072JB','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.96,200,345,67,'Intermediate' from dual union all
select 9347,'009347','1976-072JC','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',88.77,140,288,67.1,'Intermediate' from dual union all
select 9348,'009348','1976-072JD','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.2,170,300,67.1,'Intermediate' from dual union all
select 9349,'009349','1976-072JE','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.57,154,353,67,'Intermediate' from dual union all
select 935,'000935','1964-076E','deb Explorer XXIV','',10362,'Reentered','1984-05-28 00:00:00','1964-12-23 00:00:00',116.32,532,2499,81.36,'Intermediate' from dual union all
select 9350,'009350','1976-072JF','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.53,157,346,67.1,'Intermediate' from dual union all
select 9351,'009351','1976-072JG','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.59,157,352,67.3,'Intermediate' from dual union all
select 9352,'009352','1976-072JH','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.67,168,349,67.2,'Intermediate' from dual union all
select 9353,'009353','1976-072JJ','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',90.27,189,387,67.1,'Intermediate' from dual union all
select 9354,'009354','1976-072JK','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.72,153,369,67.2,'Intermediate' from dual union all
select 9355,'009355','1976-072JL','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.17,162,305,67.1,'Intermediate' from dual union all
select 9356,'009356','1976-072JM','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',88.86,168,269,67.3,'Intermediate' from dual;
insert into SATELLITE
select 9357,'009357','1976-072JN','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.78,159,369,67.1,'Intermediate' from dual union all
select 9358,'009358','1976-072JP','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',90.36,203,381,67,'Intermediate' from dual union all
select 9359,'009359','1976-072JQ','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.12,142,320,67.1,'Intermediate' from dual union all
select 936,'000936','1964-076F','deb Explorer XXIV','',10362,'Reentered','1968-04-03 00:00:00','1964-12-27 00:00:00',116.25,555,2470,81.28,'Intermediate' from dual union all
select 9360,'009360','1976-072JR','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.79,163,366,67.1,'Intermediate' from dual union all
select 9361,'009361','1976-072JS','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.62,186,326,67.1,'Intermediate' from dual union all
select 9362,'009362','1976-072JT','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',90.1,170,389,67.2,'Intermediate' from dual union all
select 9363,'009363','1976-072JU','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',90.29,193,384,67.2,'Intermediate' from dual union all
select 9364,'009364','1976-072JV','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.19,154,315,67.3,'Intermediate' from dual union all
select 9365,'009365','1976-072JW','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',88.89,150,290,67.1,'Intermediate' from dual union all
select 9366,'009366','1976-072JX','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.66,202,314,67.1,'Intermediate' from dual union all
select 9367,'009367','1976-072JY','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.28,147,331,67.1,'Intermediate' from dual union all
select 9368,'009368','1976-072JZ','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.7,177,343,67.1,'Intermediate' from dual union all
select 9369,'009369','1976-072KA','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',88.68,139,280,67.2,'Intermediate' from dual union all
select 937,'000937','1964-076G','deb Explorer XXIV','',10362,'Reentered','1981-04-19 00:00:00','1965-05-27 00:00:00',116.34,536,2497,81.36,'Intermediate' from dual union all
select 9370,'009370','1976-072KB','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.7,178,342,67.2,'Intermediate' from dual union all
select 9371,'009371','1976-072KC','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.26,202,274,67,'Intermediate' from dual union all
select 9372,'009372','1976-072KD','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.83,167,366,67.2,'Intermediate' from dual union all
select 9373,'009373','1976-072KE','deb Kosmos-844','',39024,'Reentered','1976-08-13 00:00:00','1976-08-01 00:00:00',89.54,161,342,67.1,'Intermediate' from dual union all
select 9374,'009374','1976-082D','Kapsula Kosmosa-848','Nauka 19KS',39034,'Reentered','1976-09-01 00:00:00','1976-08-19 00:00:00',89.29,199,280,62.8,'Intermediate' from dual union all
select 9375,'009375','1976-082E','Nauka part','',39034,'Reentered','1976-08-24 00:00:00','1976-08-16 00:00:00',89.23,203,270,62.8,'Intermediate' from dual union all
select 9376,'009376','1976-079D','KDU','KDU',39031,'Reentered','1976-08-23 00:00:00','1976-08-16 00:00:00',89.2,165,305,62.8,'Intermediate' from dual union all
select 9377,'009377','1976-079E','Blenda','Sunshade',39031,'Reentered','1976-08-18 00:00:00','1976-08-16 00:00:00',89.02,164,289,62.8,'Intermediate' from dual union all
select 9378,'009378','1976-072KF','deb Kosmos-844','',39024,'Reentered','1976-08-17 00:00:00','1976-08-15 00:00:00',89.62,157,354,67.1,'Intermediate' from dual union all
select 9379,'009379','1976-072KG','deb Kosmos-844','',39024,'Reentered','1976-08-17 00:00:00','1976-08-15 00:00:00',89.92,192,349,67,'Intermediate' from dual union all
select 938,'000938','1964-077A','Mariner 4','Mariner C-3',10363,'Deep Space','1964-11-29 00:00:00','',null,null,null,null,'' from dual union all
select 9380,'009380','1976-072KH','deb Kosmos-844','',39024,'Reentered','1976-08-17 00:00:00','1976-08-15 00:00:00',90.25,186,388,67.2,'Intermediate' from dual union all
select 9381,'009381','1976-072KJ','deb Kosmos-844','',39024,'Reentered','1976-08-17 00:00:00','1976-08-15 00:00:00',88.98,133,316,67.2,'Intermediate' from dual union all
select 9382,'009382','1976-083A','Kosmos-849','DS-P1-I No. 17',39035,'Reentered','1978-04-24 00:00:00','1976-09-17 00:00:00',95.8,263,852,70.97,'Intermediate' from dual union all
select 9383,'009383','1976-083B','S1M','S1M',39035,'Reentered','1977-11-02 00:00:00','1976-09-17 00:00:00',95.44,269,811,70.96,'Intermediate' from dual union all
select 9384,'009384','1976-081E','Luna-24 VA','VA',39033,'Deep Space Attached','1976-08-10 00:00:00','1976-08-23 00:00:00',15550.88,0,400000,51.5,'Cislunar' from dual union all
select 9385,'009385','1976-057J','deb Salyut-5','',39009,'Reentered','1976-09-13 00:00:00','1976-08-18 00:00:00',89.56,249,257,51.61,'Intermediate' from dual union all
select 9386,'009386','1976-057K','deb Salyut-5','',39009,'Reentered','1976-08-22 00:00:00','1976-08-17 00:00:00',89.12,231,231,51.61,'Intermediate' from dual union all
select 9387,'009387','1976-084A','Kosmos-850','DS-P1-Yu No. 79',39036,'Reentered','1977-05-16 00:00:00','1976-09-25 00:00:00',92,267,479,70.94,'Intermediate' from dual union all
select 9388,'009388','1976-084B','S1M','S1M',39036,'Reentered','1977-01-14 00:00:00','1976-09-25 00:00:00',91.74,275,444,70.95,'Intermediate' from dual union all
select 9389,'009389','1976-085A','Kosmos-851','Tselina-D',39037,'Reentered','1989-08-05 00:00:00','1976-09-26 00:00:00',96.68,565,634,81.21,'Intermediate' from dual union all
select 939,'000939','1964-076H','deb Explorer XXIV','',10362,'Reentered','1968-04-30 00:00:00','1964-12-21 00:00:00',116.03,540,2465,81.28,'Intermediate' from dual union all
select 9390,'009390','1976-085B','8A92ME','8A92ME',39037,'Reentered','1992-03-22 00:00:00','1976-09-27 00:00:00',96.74,542,664,81.22,'Intermediate' from dual union all
select 9391,'009391','1976-086A','Kosmos-852','Zenit-4MK',39038,'Landed','1976-09-10 00:00:00','1976-08-30 00:00:00',89.5,171,329,64.99,'Intermediate' from dual union all
select 9392,'009392','1976-086B','11S510','11S510',39038,'Reentered','1976-09-02 00:00:00','1976-08-30 00:00:00',89.09,171,288,64.99,'Intermediate' from dual union all
select 9393,'009393','1976-086C','Sensor cover','Zenit-4MK sensor cover',39038,'Reentered','1976-08-29 00:00:00','1976-08-28 00:00:00',88.95,167,279,65.09,'Intermediate' from dual union all
select 9394,'009394','1976-087A','JSSW 5','JSSW',39039,'Reentered','1978-11-25 00:00:00','1976-10-01 00:00:00',108.47,198,2114,69.15,'Intermediate' from dual union all
select 9395,'009395','1976-087B','FB-1 Stage 2','FB-1 Stage 2',39039,'Reentered','1978-02-04 00:00:00','1976-09-29 00:00:00',108.02,193,2078,69.16,'Intermediate' from dual union all
select 9396,'009396','1976-037E','Kosmos-816 SS 3','ESO',38989,'Reentered','1977-06-12 00:00:00','1976-09-01 00:00:00',94.36,464,512,65.83,'Intermediate' from dual union all
select 9397,'009397','1976-037F','Kosmos-816 SS 4','ESO',38989,'Reentered','1977-09-17 00:00:00','1976-09-01 00:00:00',94.67,474,532,65.82,'Intermediate' from dual union all
select 9398,'009398','1976-088A','Kosmos-853','Molniya-2',39040,'Reentered','1976-12-31 00:00:00','1976-10-01 00:00:00',91.23,244,426,62.81,'Intermediate' from dual union all
select 9399,'009399','1976-088B','11S510','11S510',39040,'Reentered','1976-10-10 00:00:00','1976-09-08 00:00:00',90.99,212,434,62.83,'Intermediate' from dual union all
select 94,'000094','61 Theta 4','deb Korabl''-Sputnik-4','',7280,'Reentered','1961-03-10 00:00:00','1961-03-09 00:00:00',88.21,172,200,64.93,'Intermediate' from dual union all
select 940,'000940','1964-076N','deb Explorer XXIV','',10362,'Reentered','1981-03-13 00:00:00','1965-01-06 00:00:00',116.14,527,2488,81.34,'Intermediate' from dual union all
select 9400,'009400','1976-088C','Blok-L','Blok-L',39040,'Reentered','1977-05-20 00:00:00','1976-10-02 00:00:00',91.48,238,457,62.77,'Intermediate' from dual union all
select 9401,'009401','1966-076G','NNS O-10 insulation','',13483,'Reentered','1982-03-25 00:00:00','1976-08-30 00:00:00',106.7,1049,1098,88.86,'Polar' from dual union all
select 9402,'009402','1976-088D','BOZ','BOZ',39040,'Reentered','1976-12-09 00:00:00','1976-10-01 00:00:00',91.27,240,434,62.8,'Intermediate' from dual union all
select 9403,'009403','1976-089A','TIP 3','TIP 3',39041,'Reentered','1981-05-30 00:00:00','1976-10-02 00:00:00',95.86,343,778,90.32,'Polar' from dual union all
select 9404,'009404','1976-089B','Star 20 E-17','Star 20 E-17',39041,'Reentered','1978-06-29 00:00:00','1976-10-02 00:00:00',95.86,342,779,90.32,'Polar' from dual union all
select 9405,'009405','1976-090A','Kosmos-854','Zenit-4MK',39042,'Landed','1976-09-16 00:00:00','1976-09-05 00:00:00',88.93,165,279,81.36,'Intermediate' from dual union all
select 9406,'009406','1976-090B','11S510','11S510',39042,'Reentered','1976-09-07 00:00:00','1976-09-05 00:00:00',88.55,164,241,81.35,'Intermediate' from dual union all
select 9407,'009407','1976-083C','deb Kosmos-849','',39035,'Reentered','1977-02-16 00:00:00','1976-09-17 00:00:00',95.45,265,816,71.16,'Intermediate' from dual union all
select 9408,'009408','1975-083C','Viking Lander 2','Viking Lander 2',36530,'Deep Space Attached','1975-09-10 00:00:00','1975-09-09 00:00:00',715.64,178,65940,29.3,'Earth Escape' from dual union all
select 9409,'009409','1976-089C','TIP 3 despin weight','',39041,'Reentered','1978-03-01 00:00:00','1976-10-01 00:00:00',95.45,331,750,90.2,'Polar' from dual union all
select 941,'000941','1964-076J','deb Explorer XXIV','',10362,'Reentered','1979-02-22 00:00:00','1964-12-21 00:00:00',116.12,529,2484,81.34,'Intermediate' from dual union all
select 9410,'009410','1976-089D','TIP 3 despin weight','',39041,'Reentered','1978-05-17 00:00:00','1976-10-01 00:00:00',96.12,344,802,90.44,'Polar' from dual union all
select 9411,'009411','1976-021D','Blok-L','Blok-L',38973,'Reentered','1991-07-01 00:00:00','1976-09-19 00:00:00',731.12,449,40562,63.13,'Molniya' from dual union all
select 9412,'009412','1976-086D','KDU','KDU',39038,'Reentered','1976-09-16 00:00:00','1976-09-10 00:00:00',89.08,167,290,64.99,'Intermediate' from dual union all
select 9413,'009413','1976-086E','Blenda','Sunshade',39038,'Reentered','1976-09-11 00:00:00','1976-09-11 00:00:00',87.96,172,175,65.04,'Intermediate' from dual union all
select 9414,'009414','1976-086F','KDU part','KDU part',39038,'Reentered','1976-09-11 00:00:00','1976-09-10 00:00:00',89.16,164,302,65,'Intermediate' from dual union all
select 9415,'009415','1976-091A','DMSP F-1','DMSP Block 5D S-1',39043,'In Earth orbit','','1995-07-28 00:00:00',101.13,799,825,98.63,'Sun-Synch' from dual union all
select 9416,'009416','1976-092A','Raduga','Gran'' No. 12L',39044,'In Earth orbit','','1977-09-05 00:00:00',1436.21,35770,35808,.4,'Stationary' from dual union all
select 9417,'009417','1976-092B','8S812','8S812',39044,'Reentered','1976-09-14 00:00:00','1976-09-13 00:00:00',88.04,171,184,51.49,'Intermediate' from dual union all
select 9418,'009418','1976-092C','Perekhodnik','Sredniy perekhodnik',39044,'Reentered','1976-09-12 00:00:00','1976-09-12 00:00:00',88.07,155,202,51.48,'Intermediate' from dual union all
select 9419,'009419','1976-091B','DMSP F-1 solar array band','',39043,'Reentered','1985-01-01 00:00:00','1976-12-31 00:00:00',101.52,817,845,98.7,'Sun-Synch' from dual union all
select 942,'000942','1964-077B','Agena D 6932','Agena D 6932',10363,'Deep Space','1964-11-29 00:00:00','',null,null,null,null,'' from dual union all
select 9420,'009420','1976-091C','DMSP F-1 solar array band','',39043,'Reentered','1985-01-01 00:00:00','1977-01-01 00:00:00',101.53,818,845,98.7,'Sun-Synch' from dual union all
select 9421,'009421','1976-093A','Soyuz-22','Soyuz 7K-TM No. 74',39045,'Landed','1976-09-23 00:00:00','1976-09-18 00:00:00',89.53,246,257,64.75,'Intermediate' from dual union all
select 9422,'009422','1976-090C','KDU','KDU',39042,'Reentered','1976-09-19 00:00:00','1976-09-15 00:00:00',88.74,159,266,81.4,'Intermediate' from dual union all
select 9423,'009423','1976-090D','Blenda','Sunshade',39042,'Reentered','1976-09-17 00:00:00','1976-09-15 00:00:00',88.56,164,243,81.34,'Intermediate' from dual union all
select 9424,'009424','1976-093B','11S510','11S510',39045,'Reentered','1976-09-20 00:00:00','1976-09-17 00:00:00',88.94,180,265,64.76,'Intermediate' from dual union all
select 9425,'009425','1976-093C','MKF-6 cover','MKF-6 cover',39045,'Reentered','1976-09-16 00:00:00','1976-09-15 00:00:00',89.29,184,294,64.75,'Intermediate' from dual union all
select 9426,'009426','1976-094A','OPS 8533','GAMBIT-3 Mission 4347',39046,'Reentered Attached','1976-11-05 00:00:00','1976-10-02 00:00:00',89.14,131,333,96.4,'Sun-Synch' from dual union all
select 9427,'009427','1976-094B','OPS 8533 sensor cover','',39046,'Reentered','1976-09-17 00:00:00','1976-09-15 00:00:00',88.72,138,285,96.28,'Sun-Synch' from dual union all
select 9428,'009428','1976-037G','Kosmos-816 SS 5','ESO',38989,'Reentered','1977-08-09 00:00:00','1976-09-20 00:00:00',94.44,464,519,65.76,'Intermediate' from dual union all
select 9429,'009429','1976-037H','Kosmos-816 SS 6','ESO',38989,'Reentered','1977-08-10 00:00:00','1976-09-21 00:00:00',94.5,469,520,65.84,'Intermediate' from dual union all
select 943,'000943','1964-078A','8K78I','8K78I',10364,'Reentered','1964-12-01 00:00:00','1964-11-30 00:00:00',88.27,143,235,64.76,'Intermediate' from dual union all
select 9430,'009430','1976-037J','Kosmos-816 SS 7','ESO',38989,'Reentered','1977-08-29 00:00:00','1976-09-20 00:00:00',94.53,470,521,65.82,'Intermediate' from dual union all
select 9431,'009431','1976-037K','Kosmos-816 SS 8','ESO',38989,'Reentered','1977-08-24 00:00:00','1976-09-21 00:00:00',94.58,474,523,65.9,'Intermediate' from dual union all
select 9432,'009432','1976-037L','Kosmos-816 SS 9','ESO',38989,'Reentered','1977-09-20 00:00:00','1976-09-21 00:00:00',94.57,470,526,65.85,'Intermediate' from dual union all
select 9433,'009433','1976-095A','Kosmos-855','Orion',39047,'Landed','1976-10-03 00:00:00','1976-09-23 00:00:00',89.89,200,338,72.87,'Intermediate' from dual union all
select 9434,'009434','1976-095B','11S510','11S510',39047,'Reentered','1976-10-02 00:00:00','1976-09-23 00:00:00',89.63,192,321,72.86,'Intermediate' from dual union all
select 9435,'009435','1976-096A','Kosmos-856','Zenit-2M',39048,'Landed','1976-10-05 00:00:00','1976-09-26 00:00:00',89.48,201,297,65.01,'Intermediate' from dual union all
select 9436,'009436','1976-096B','11S510','11S510',39048,'Reentered','1976-09-30 00:00:00','1976-09-24 00:00:00',89.26,199,276,65.02,'Intermediate' from dual union all
select 9437,'009437','1976-096C','Sensor cover','Zenit-2M sensor cover',39048,'Reentered','1976-09-23 00:00:00','1976-09-22 00:00:00',89.14,208,256,65,'Intermediate' from dual union all
select 9438,'009438','1976-037M','Kosmos-816 SS 10','ESO',38989,'Reentered','1977-08-19 00:00:00','1976-09-25 00:00:00',94.46,468,516,65.81,'Intermediate' from dual union all
select 9439,'009439','1976-097A','Kosmos-857','Zenit-4MK',39049,'Landed','1976-10-07 00:00:00','1976-09-27 00:00:00',89.46,177,319,62.81,'Intermediate' from dual union all
select 944,'000944','1964-078B','BOZ','BOZ',10364,'Reentered','1964-12-02 00:00:00','1964-11-30 00:00:00',88.36,177,210,64.71,'Intermediate' from dual union all
select 9440,'009440','1976-092D','SOZ','SOZ',39044,'Reentered','1977-07-19 00:00:00','1976-10-24 00:00:00',633.88,316,35814,47.28,'GEO Transfer' from dual union all
select 9441,'009441','1976-092E','SOZ','SOZ',39044,'Reentered','1977-05-20 00:00:00','1976-11-14 00:00:00',631.84,169,35857,46.84,'GEO Transfer' from dual union all
select 9442,'009442','1976-097B','11S510','11S510',39049,'Reentered','1976-09-29 00:00:00','1976-09-26 00:00:00',89.14,170,294,62.79,'Intermediate' from dual union all
select 9443,'009443','1976-098A','Kosmos-858','Strela-2M',39050,'In Earth orbit','','1993-01-09 00:00:00',100.58,776,796,74.05,'Intermediate' from dual union all
select 9444,'009444','1976-098B','S3M','S3M',39050,'In Earth orbit','','1991-03-27 00:00:00',100.47,770,791,74.05,'Intermediate' from dual union all
select 9445,'009445','1976-095C','Blenda','KBE part',39047,'Reentered','1976-10-06 00:00:00','1976-10-03 00:00:00',89.66,196,320,72.87,'Intermediate' from dual union all
select 9446,'009446','1976-095D','KB electropitaniya','KBE',39047,'Reentered','1976-10-08 00:00:00','1976-10-03 00:00:00',89.79,199,330,72.87,'Intermediate' from dual;
insert into SATELLITE
select 9447,'009447','1976-096D','Nauka part','',39048,'Reentered','1976-10-08 00:00:00','1976-10-03 00:00:00',89.22,196,276,64.99,'Intermediate' from dual union all
select 9448,'009448','1976-096E','Kapsula Kosmosa-856','Nauka 23KS',39048,'Reentered','1976-10-16 00:00:00','1976-10-03 00:00:00',89.39,199,290,65.01,'Intermediate' from dual union all
select 9449,'009449','1976-097C','KDU','KDU',39049,'Reentered','1976-10-15 00:00:00','1976-10-06 00:00:00',89.48,175,323,62.8,'Intermediate' from dual union all
select 945,'000945','1964-078C','Zond-2','3MV-4A No. 2',10364,'Deep Space','1964-12-02 00:00:00','',null,null,null,null,'' from dual union all
select 9450,'009450','1976-097D','Blenda','Sunshade',39049,'Reentered','1976-10-08 00:00:00','1976-10-06 00:00:00',89.46,173,323,62.8,'Intermediate' from dual union all
select 9451,'009451','1976-097E','KDU part','KDU part',39049,'Reentered','1976-10-09 00:00:00','1976-10-06 00:00:00',89.23,173,300,62.81,'Intermediate' from dual union all
select 9452,'009452','1966-056R','deb Pageos','RCS 0.6',13463,'Reentered','1977-03-08 00:00:00','1977-01-06 00:00:00',174.54,541,7394,82.21,'Medium' from dual union all
select 9453,'009453','1966-056S','deb Pageos','RCS 12.3',13463,'Reentered','1981-04-13 00:00:00','1981-02-06 00:00:00',168.65,606,6860,83.52,'Medium' from dual union all
select 9454,'009454','1966-056T','deb Pageos','',13463,'Reentered','1977-02-08 00:00:00','1977-01-15 00:00:00',171.52,518,7178,81.85,'Medium' from dual union all
select 9455,'009455','1966-056U','deb Pageos','RCS 14.5',13463,'Reentered','1977-03-03 00:00:00','1977-02-14 00:00:00',174.43,415,7512,82.1,'Medium' from dual union all
select 9456,'009456','1966-056V','deb Pageos','RCS 0.0',13463,'Reentered','1977-02-14 00:00:00','1976-10-25 00:00:00',176.2,870,7196,82.44,'Medium' from dual union all
select 9457,'009457','1966-056W','deb Pageos','RCS 0.0',13463,'Reentered','1977-08-20 00:00:00','1977-07-26 00:00:00',174.42,541,7385,82.66,'Medium' from dual union all
select 9458,'009458','1966-056X','deb Pageos','RCS 0.5',13463,'Reentered','1977-06-29 00:00:00','1977-03-04 00:00:00',175.61,611,7409,82.45,'Medium' from dual union all
select 9459,'009459','1966-056Y','deb Pageos','RCS 0.5',13463,'Reentered','1977-07-02 00:00:00','1977-03-18 00:00:00',175.83,575,7462,82.68,'Medium' from dual union all
select 946,'000946','1964-079A','OPS 4439','GAMBIT SV 964',10365,'Reentered','1964-12-06 00:00:00','1965-12-05 00:00:00',89.65,158,357,97.02,'Sun-Synch' from dual union all
select 9460,'009460','1966-056Z','deb Pageos','RCS 7.4',13463,'Reentered','1990-01-09 00:00:00','1990-01-09 00:00:00',92.1,285,470,81.34,'Intermediate' from dual union all
select 9461,'009461','1966-056AA','deb Pageos','RCS 2.3',13463,'Reentered','1977-12-16 00:00:00','1977-06-13 00:00:00',150.78,303,5709,83.71,'Medium' from dual union all
select 9462,'009462','1966-056AB','deb Pageos','RCS 0.0',13463,'Reentered','1977-02-09 00:00:00','1977-02-07 00:00:00',174.55,420,7515,81.85,'Medium' from dual union all
select 9463,'009463','1966-056AC','deb Pageos','RCS 4.3',13463,'Reentered','1981-04-13 00:00:00','1976-10-07 00:00:00',177.01,1187,6943,83.38,'Medium' from dual union all
select 9464,'009464','1966-056AD','deb Pageos','RCS 0.8',13463,'Reentered','1980-08-22 00:00:00','1980-06-29 00:00:00',154.01,1591,4688,84.62,'Medium' from dual union all
select 9465,'009465','1966-056AE','deb Pageos','RCS 0.0',13463,'Reentered','1981-04-13 00:00:00','1976-10-07 00:00:00',177.49,1210,6958,83.39,'Medium' from dual union all
select 9466,'009466','1966-056AF','deb Pageos','RCS 0.8',13463,'Reentered','1989-02-02 00:00:00','1976-10-07 00:00:00',175.93,950,7094,83.28,'Medium' from dual union all
select 9467,'009467','1966-056AG','deb Pageos','',13463,'Reentered','1977-04-18 00:00:00','1977-03-05 00:00:00',173.85,483,7398,82.3,'Medium' from dual union all
select 9468,'009468','1966-056AH','deb Pageos','RCS 0.3',13463,'In Earth orbit','','1980-02-28 00:00:00',180.93,3433,5004,85.74,'Medium' from dual union all
select 9469,'009469','1966-056AJ','deb Pageos','RCS 0.8',13463,'Reentered','1982-11-25 00:00:00','1982-05-26 00:00:00',174.59,580,7358,84.64,'Medium' from dual union all
select 947,'000947','1964-080A','Kosmos-51','DS-MT No. 3',10366,'Reentered','1965-11-14 00:00:00','1965-01-11 00:00:00',92.44,259,529,48.76,'Intermediate' from dual union all
select 9470,'009470','1976-097F','PTDU cover','KDU part',39049,'Reentered','1976-10-11 00:00:00','1976-10-09 00:00:00',90.6,210,398,62.78,'Intermediate' from dual union all
select 9471,'009471','1976-099A','Kosmos-859','Zenit-4MK',39051,'Landed','1976-10-21 00:00:00','1976-10-12 00:00:00',89.35,171,314,64.99,'Intermediate' from dual union all
select 9472,'009472','1976-099B','11S510','11S510',39051,'Reentered','1976-10-15 00:00:00','1976-10-12 00:00:00',89.14,168,296,64.99,'Intermediate' from dual union all
select 9473,'009473','1976-099C','Sensor cover','Zenit-4MK sensor cover',39051,'Reentered','1976-10-11 00:00:00','1976-10-11 00:00:00',88.86,164,273,64.98,'Intermediate' from dual union all
select 9474,'009474','1976-091D','DMSP F-1 OLS Cover','',39043,'Reentered','1980-09-20 00:00:00','1977-01-01 00:00:00',101.52,814,848,98.73,'Sun-Synch' from dual union all
select 9475,'009475','1976-037N','Kosmos-816 SS 11','ESO',38989,'Reentered','1977-09-02 00:00:00','1976-10-10 00:00:00',94.48,466,520,65.76,'Intermediate' from dual union all
select 9476,'009476','1976-037P','Kosmos-816 SS 12','ESO',38989,'Reentered','1977-09-16 00:00:00','1976-10-10 00:00:00',94.55,472,522,65.87,'Intermediate' from dual union all
select 9477,'009477','1976-100A','Soyuz-23','Soyuz 7K-T(A9) No. 65',39052,'Landed','1976-10-16 00:00:00','1976-10-15 00:00:00',89.49,229,270,51.59,'Intermediate' from dual union all
select 9478,'009478','1976-101A','Marisat 102','Marisat C',39053,'In Earth orbit','','1977-01-18 00:00:00',1436.06,35781,35791,2.1,'Stationary' from dual union all
select 9479,'009479','1976-101B','Delta 127','Delta SSPU DSV-3P-4',39053,'Reentered','1976-11-15 00:00:00','1976-10-21 00:00:00',92.06,180,572,28.58,'Intermediate' from dual union all
select 948,'000948','1964-080B','S1','S1',10366,'Reentered','1965-05-11 00:00:00','1965-01-08 00:00:00',92.27,253,519,48.77,'Intermediate' from dual union all
select 9480,'009480','1976-100B','11S510','11S510',39052,'Reentered','1976-10-16 00:00:00','1976-10-15 00:00:00',88.51,183,218,51.61,'Intermediate' from dual union all
select 9481,'009481','1976-102A','Meteor','Meteor No. 35',39054,'In Earth orbit','','1993-01-14 00:00:00',102.23,839,890,81.25,'Intermediate' from dual union all
select 9482,'009482','1976-102B','8A92ME','8A92ME',39054,'In Earth orbit','','1993-08-17 00:00:00',102.38,829,914,81.27,'Intermediate' from dual union all
select 9483,'009483','1976-091E','DMSP F-1 OLS Cooler Cover','',39043,'Reentered','1983-08-24 00:00:00','1977-01-01 00:00:00',101.59,816,852,98.7,'Sun-Synch' from dual union all
select 9484,'009484','1976-091F','deb DMSP Block 5D F-1','',39043,'Reentered','1983-01-01 00:00:00','1977-01-01 00:00:00',101.58,815,852,98.7,'Sun-Synch' from dual union all
select 9485,'009485','1976-037Q','Kosmos-816 SS 13','ESO',38989,'Reentered','1977-10-28 00:00:00','1976-10-17 00:00:00',94.49,468,520,65.8,'Intermediate' from dual union all
select 9486,'009486','1976-103A','Kosmos-860','US-A reactor',39055,'In Earth orbit','','1976-11-16 00:00:00',104.31,918,1007,64.7,'Intermediate' from dual union all
select 9487,'009487','1976-037R','Kosmos-816 SS 14','ESO',38989,'Reentered','1977-09-07 00:00:00','1976-10-13 00:00:00',94.53,472,520,65.86,'Intermediate' from dual union all
select 9488,'009488','1976-037S','Kosmos-816 SS 15','ESO',38989,'Reentered','1977-09-27 00:00:00','1976-10-13 00:00:00',94.51,471,519,65.8,'Intermediate' from dual union all
select 9489,'009489','1976-037T','Kosmos-816 SS 16','ESO',38989,'Reentered','1977-09-24 00:00:00','1976-10-16 00:00:00',94.53,471,521,65.84,'Intermediate' from dual union all
select 949,'000949','1964-081A','Titan 3A Fairing','Lead Dummy Payload',10367,'Reentered Attached','1964-12-13 00:00:00','1964-12-10 00:00:00',87.99,168,182,32.15,'Intermediate' from dual union all
select 9490,'009490','1976-037U','Kosmos-816 SS 17','ESO',38989,'Reentered','1977-09-15 00:00:00','1976-10-16 00:00:00',94.49,470,518,65.81,'Intermediate' from dual union all
select 9491,'009491','1976-099D','KDU','KDU',39051,'Reentered','1976-10-24 00:00:00','1976-10-20 00:00:00',89.16,166,300,64.99,'Intermediate' from dual union all
select 9492,'009492','1976-099E','Blenda','Sunshade',39051,'Reentered','1976-10-25 00:00:00','1976-10-20 00:00:00',89.02,166,286,64.98,'Intermediate' from dual union all
select 9493,'009493','1976-101C','Delta 127 clamp band','',39053,'Reentered','1976-11-01 00:00:00','1976-10-21 00:00:00',91.47,177,517,28.59,'Intermediate' from dual union all
select 9494,'009494','1976-104A','Kosmos-861','US-A reactor',39056,'In Earth orbit','','1976-11-20 00:00:00',89.63,250,262,64.95,'Intermediate' from dual union all
select 9495,'009495','1976-105A','Kosmos-862','Oko',39057,'In Earth orbit','','1976-12-03 00:00:00',718.21,613,39763,62.93,'Molniya' from dual union all
select 9496,'009496','1976-105B','11S510','11S510',39057,'Reentered','1976-12-27 00:00:00','1976-11-21 00:00:00',91.08,214,441,62.77,'Intermediate' from dual union all
select 9497,'009497','1976-105C','BOZ','BOZ',39057,'Reentered','1976-12-10 00:00:00','1976-10-29 00:00:00',92.12,196,561,62.85,'Intermediate' from dual union all
select 9498,'009498','1976-101D','Delta 127 clamp band','',39053,'Reentered','1976-11-04 00:00:00','1976-10-21 00:00:00',91.46,179,514,28.57,'Intermediate' from dual union all
select 9499,'009499','1976-106A','Kosmos-863','Zenit-4MK',39058,'Landed','1976-11-05 00:00:00','1976-10-27 00:00:00',89.37,170,317,62.81,'Intermediate' from dual union all
select 95,'000095','61 Iota 1','Korabl''-Sputnik-5','Vostok-3 No. 2',7267,'Landed','1961-03-25 00:00:00','1961-03-25 00:00:00',88.41,163,229,64.9,'Intermediate' from dual union all
select 950,'000950','1964-080C','deb Kosmos-51','',10366,'Reentered','1964-12-29 00:00:00','1964-12-16 00:00:00',91.81,257,470,48.75,'Intermediate' from dual union all
select 9500,'009500','1976-106B','11S510','11S510',39058,'Reentered','1976-11-01 00:00:00','1976-10-27 00:00:00',89.5,176,324,62.81,'Intermediate' from dual union all
select 9501,'009501','1976-106C','Sensor cover','Zenit-4MK sensor cover',39058,'Reentered','1976-10-29 00:00:00','1976-10-27 00:00:00',89.69,174,345,62.8,'Intermediate' from dual union all
select 9502,'009502','1976-106D','Sensor cover','Zenit-4MK sensor cover',39058,'Reentered','1976-10-26 00:00:00','1976-10-26 00:00:00',88.78,163,265,62.77,'Intermediate' from dual union all
select 9503,'009503','1976-107A','Ekran','Ekran No. 11L',39059,'In Earth orbit','','1977-03-19 00:00:00',1436.11,35532,36042,.09,'Stationary' from dual union all
select 9504,'009504','1976-107B','8S812','8S812',39059,'Reentered','1976-10-29 00:00:00','1976-10-28 00:00:00',87.82,161,172,51.5,'Intermediate' from dual union all
select 9505,'009505','1976-107C','Perekhodnik','Sredniy perekhodnik',39059,'Reentered','1976-10-27 00:00:00','1976-10-26 00:00:00',88.16,182,185,51.47,'Intermediate' from dual union all
select 9506,'009506','1976-105D','Blok-L','Blok-L',39057,'Reentered','1993-11-16 00:00:00','1993-08-16 00:00:00',711.53,304,39742,62.77,'Molniya' from dual union all
select 9507,'009507','1976-037V','Kosmos-816 SS 18','ESO',38989,'Reentered','1977-08-01 00:00:00','1976-10-28 00:00:00',94.28,461,506,65.86,'Intermediate' from dual union all
select 9508,'009508','1976-037W','Kosmos-816 SS 19','ESO',38989,'Reentered','1977-09-17 00:00:00','1976-10-28 00:00:00',94.47,467,519,65.81,'Intermediate' from dual union all
select 9509,'009509','1976-108A','Kosmos-864','Parus',39060,'In Earth orbit','','1976-11-28 00:00:00',104.83,964,1008,82.94,'Intermediate' from dual union all
select 951,'000951','1964-082A','Surveyor Mass Model','Surveyor Model',10368,'Reentered Attached','1964-12-12 00:00:00','1964-12-11 00:00:00',87.91,165,178,30.7,'Intermediate' from dual union all
select 9510,'009510','1976-108B','S3M','S3M',39060,'In Earth orbit','','1991-03-27 00:00:00',104.61,957,996,82.94,'Intermediate' from dual union all
select 9511,'009511','1976-037X','Kosmos-816 SS 20','ESO',38989,'Reentered','1977-10-12 00:00:00','1976-10-31 00:00:00',94.6,473,525,65.81,'Intermediate' from dual union all
select 9512,'009512','1976-037Y','Kosmos-816 SS 21','ESO',38989,'Reentered','1977-09-03 00:00:00','1976-10-31 00:00:00',94.38,463,514,65.85,'Intermediate' from dual union all
select 9513,'009513','1976-037Z','Kosmos-816 SS 22','ESO',38989,'Reentered','1977-09-17 00:00:00','1976-10-31 00:00:00',94.42,465,516,65.82,'Intermediate' from dual union all
select 9514,'009514','1976-037AA','Kosmos-816 SS 23','ESO',38989,'Reentered','1977-10-17 00:00:00','1976-11-01 00:00:00',94.55,471,522,65.84,'Intermediate' from dual union all
select 9515,'009515','1976-109A','Kosmos-865','Zenit-2M',39061,'Landed','1976-11-13 00:00:00','1976-11-03 00:00:00',89.74,201,323,72.88,'Intermediate' from dual union all
select 9516,'009516','1976-109B','11S510','11S510',39061,'Reentered','1976-11-14 00:00:00','1976-11-03 00:00:00',89.52,207,295,72.87,'Intermediate' from dual union all
select 9517,'009517','1976-109C','Sensor cover','Zenit-2M sensor cover',39061,'Reentered','1976-11-05 00:00:00','1976-11-02 00:00:00',89.29,200,279,72.85,'Intermediate' from dual union all
select 9518,'009518','1976-091G','deb DMSP Block 5D F-1','',39043,'Reentered','1985-01-01 00:00:00','1977-01-01 00:00:00',101.52,816,846,98.7,'Sun-Synch' from dual union all
select 9519,'009519','1976-109D','Sensor cover','Zenit-2M sensor cover',39061,'Reentered','1976-11-07 00:00:00','1976-11-03 00:00:00',89.64,193,321,72.93,'Intermediate' from dual union all
select 952,'000952','1964-080D','deb Kosmos-51','',10366,'Reentered','1965-01-01 00:00:00','1964-12-17 00:00:00',91.76,251,471,48.75,'Intermediate' from dual union all
select 9520,'009520','1976-109E','Sensor cover','Zenit-2M sensor cover',39061,'Reentered','1976-11-22 00:00:00','1976-11-09 00:00:00',89.45,199,296,72.88,'Intermediate' from dual union all
select 9521,'009521','1976-106E','KDU','KDU',39058,'Reentered','1976-11-11 00:00:00','1976-11-05 00:00:00',89.35,165,320,62.8,'Intermediate' from dual union all
select 9522,'009522','1976-106F','Blenda','Sunshade',39058,'Reentered','1976-11-07 00:00:00','1976-11-04 00:00:00',89.3,156,324,62.81,'Intermediate' from dual union all
select 9523,'009523','1976-109F','Sensor cover','Zenit-2M sensor cover',39061,'Reentered','1976-11-10 00:00:00','1976-11-04 00:00:00',89.4,199,291,72.87,'Intermediate' from dual union all
select 9524,'009524','1976-106G','KDU part','KDU part',39058,'Reentered','1976-11-07 00:00:00','1976-11-05 00:00:00',89.35,164,321,62.81,'Intermediate' from dual union all
select 9525,'009525','1976-109G','Kosmos-865 subsatellite','MKA',39061,'Reentered','1977-02-10 00:00:00','1976-12-02 00:00:00',90.97,214,431,72.86,'Intermediate' from dual union all
select 9526,'009526','1976-109H','Kosmos-865 subsat part','MKA part',39061,'Reentered','1977-02-05 00:00:00','1976-12-01 00:00:00',90.86,214,420,72.85,'Intermediate' from dual union all
select 9527,'009527','1976-106H','PTDU cover','KDU part',39058,'Reentered','1976-11-09 00:00:00','1976-11-06 00:00:00',91.08,204,452,62.79,'Intermediate' from dual union all
select 9528,'009528','1976-106J','PTDU cover','KDU part',39058,'Reentered','1976-11-10 00:00:00','1976-11-06 00:00:00',91.2,203,464,62.72,'Intermediate' from dual union all
select 9529,'009529','1976-106K','Zenit-4MK part','KDU part',39058,'Reentered','1976-11-07 00:00:00','1976-11-05 00:00:00',89.27,204,272,63,'Intermediate' from dual union all
select 953,'000953','1964-083A','Ablestar 017','AJ10-104D',10369,'In Earth orbit','','1965-01-13 00:00:00',105.98,1017,1064,89.99,'Polar' from dual union all
select 9530,'009530','1974-094E','Star 37D S/N 026','Star 37D/Adapter',33697,'Reentered','2000-07-18 00:00:00','1999-09-21 00:00:00',110.44,233,2261,24.35,'Equatorial' from dual union all
select 9531,'009531','1976-103B','Kosmos-860 DU','US-A DU',39055,'Reentered','1976-12-29 00:00:00','1976-11-16 00:00:00',89.46,240,255,65.04,'Intermediate' from dual union all
select 9532,'009532','1976-110A','Kosmos-866','Zenit-4MK',39062,'Landed','1976-11-23 00:00:00','1976-11-13 00:00:00',89.41,176,315,64.98,'Intermediate' from dual union all
select 9533,'009533','1976-110B','11S510','11S510',39062,'Reentered','1976-11-14 00:00:00','1976-11-13 00:00:00',88.63,166,248,64.98,'Intermediate' from dual union all
select 9534,'009534','1976-110C','Sensor cover','Zenit-4MK sensor cover',39062,'Reentered','1976-11-16 00:00:00','1976-11-11 00:00:00',88.2,151,219,65.02,'Intermediate' from dual union all
select 9535,'009535','1976-110D','Sensor cover','Zenit-4MK sensor cover',39062,'Reentered','1976-11-14 00:00:00','1976-11-12 00:00:00',88.63,169,245,65.03,'Intermediate' from dual union all
select 9536,'009536','1976-110E','Sensor cover','Zenit-4MK sensor cover',39062,'Reentered','1976-11-13 00:00:00','1976-11-12 00:00:00',89.28,172,306,64.96,'Intermediate' from dual;
insert into SATELLITE
select 9537,'009537','1976-103C','deb Kosmos-860','',39055,'Reentered','1976-11-14 00:00:00','1976-11-11 00:00:00',89.5,244,256,65.02,'Intermediate' from dual union all
select 9538,'009538','1976-103D','Kosmos-860 RLS','US-A RLS',39055,'Reentered','1976-11-15 00:00:00','1976-11-12 00:00:00',89.48,242,255,65.03,'Intermediate' from dual union all
select 9539,'009539','1976-103E','deb Kosmos-860','',39055,'Reentered','1976-11-12 00:00:00','1976-11-12 00:00:00',89.47,240,256,65.1,'Intermediate' from dual union all
select 954,'000954','1964-080E','deb Kosmos-51','',10366,'Reentered','1965-01-01 00:00:00','1964-12-16 00:00:00',91.76,255,467,48.76,'Intermediate' from dual union all
select 9540,'009540','1976-107D','SOZ','SOZ',39059,'Reentered','1977-06-26 00:00:00','1976-11-27 00:00:00',626.93,279,35494,47.28,'GEO Transfer' from dual union all
select 9541,'009541','1976-107E','SOZ','SOZ',39059,'Reentered','1977-05-20 00:00:00','1976-11-27 00:00:00',625.55,228,35474,47.19,'GEO Transfer' from dual union all
select 9542,'009542','1976-101E','Star 37E S/N 40046','Star 37E',39053,'Reentered','1977-05-12 00:00:00','1976-11-19 00:00:00',643.35,199,36418,25.87,'GEO Transfer' from dual union all
select 9543,'009543','1975-004FF','deb Delta 107','',36451,'Reentered','1977-11-28 00:00:00','1977-11-07 00:00:00',94.76,491,522,98.21,'Sun-Synch' from dual union all
select 9544,'009544','1975-004FG','deb Delta 107','',36451,'Reentered','1979-05-12 00:00:00','1979-03-28 00:00:00',95.46,470,612,98.32,'Sun-Synch' from dual union all
select 9545,'009545','1975-004FH','deb Delta 107','',36451,'Reentered','1989-09-21 00:00:00','1989-09-20 00:00:00',89.64,249,265,98.28,'Sun-Synch' from dual union all
select 9546,'009546','1975-004FJ','deb Delta 107','',36451,'Reentered','1977-10-07 00:00:00','1977-09-07 00:00:00',95.5,532,554,98.11,'Sun-Synch' from dual union all
select 9547,'009547','1975-004FK','deb Delta 107','',36451,'Reentered','1980-01-19 00:00:00','1977-01-25 00:00:00',99.93,618,893,98.27,'Sun-Synch' from dual union all
select 9548,'009548','1975-004FL','deb Delta 107','',36451,'Reentered','1979-02-01 00:00:00','1976-11-15 00:00:00',98.14,523,816,97.52,'Sun-Synch' from dual union all
select 9549,'009549','1975-004FM','deb Delta 107','',36451,'Reentered','1978-07-08 00:00:00','1978-06-12 00:00:00',92.37,358,424,97.63,'Sun-Synch' from dual union all
select 955,'000955','1964-080F','deb Kosmos-51','',10366,'Reentered','1965-01-02 00:00:00','1964-12-17 00:00:00',91.72,254,464,48.77,'Intermediate' from dual union all
select 9550,'009550','1975-004FN','deb Delta 107','',36451,'Reentered','1979-04-14 00:00:00','1979-02-28 00:00:00',92.44,377,412,97.23,'Sun-Synch' from dual union all
select 9551,'009551','1976-110F','KDU','KDU',39062,'Reentered','1976-11-23 00:00:00','1976-11-23 00:00:00',87.83,161,174,64.98,'Intermediate' from dual union all
select 9552,'009552','1976-111A','Kosmos-867','Zenit-6',39063,'Landed','1976-12-06 00:00:00','1976-11-25 00:00:00',90.99,249,398,62.82,'Intermediate' from dual union all
select 9553,'009553','1976-110G','Blenda','Sunshade',39062,'Reentered','1976-11-24 00:00:00','1976-11-22 00:00:00',87.81,157,176,64.97,'Intermediate' from dual union all
select 9554,'009554','1976-111B','11S510','11S510',39063,'Reentered','1977-02-01 00:00:00','1976-12-23 00:00:00',90.39,241,346,62.81,'Intermediate' from dual union all
select 9555,'009555','1976-111C','Sensor cover','Zenit-6 sensor cover',39063,'Reentered','1976-12-16 00:00:00','1976-12-02 00:00:00',89.43,228,265,62.78,'Intermediate' from dual union all
select 9556,'009556','1976-111D','Sensor cover','Zenit-6 sensor cover',39063,'Reentered','1976-12-08 00:00:00','1976-11-30 00:00:00',90.53,242,359,62.83,'Intermediate' from dual union all
select 9557,'009557','1976-112A','Prognoz-5','SO-M No. 505',39064,'Deep Space','1976-11-25 00:00:00','1977-09-18 00:00:00',5728.71,3868,195485,66.58,'Deep Space' from dual union all
select 9558,'009558','1976-112B','11S510','11S510',39064,'Reentered','1977-01-23 00:00:00','1976-12-02 00:00:00',91.3,224,453,64.97,'Intermediate' from dual union all
select 9559,'009559','1976-112C','BOZ','BOZ',39064,'Reentered','1977-01-05 00:00:00','1976-12-13 00:00:00',90.76,200,424,65,'Intermediate' from dual union all
select 956,'000956','1964-083B','deb NNSS 30020','',10369,'In Earth orbit','','1965-01-13 00:00:00',106.28,1027,1081,89.99,'Polar' from dual union all
select 9560,'009560','1976-112D','deb Blok-I','',39064,'Reentered','1977-01-24 00:00:00','1977-01-09 00:00:00',90.15,214,350,64.96,'Intermediate' from dual union all
select 9561,'009561','1976-113A','Kosmos-868','US-P',39065,'Reentered','1978-07-08 00:00:00','1976-12-26 00:00:00',93.27,428,442,65.04,'Intermediate' from dual union all
select 9562,'009562','1976-113B','11S692','11S692',39065,'Reentered','1976-11-27 00:00:00','1976-11-26 00:00:00',88.98,112,337,65,'Intermediate' from dual union all
select 9563,'009563','1975-102BQ','deb Kosmos-777','',36549,'Reentered','1976-11-27 00:00:00','1976-11-28 00:00:00',89.28,230,248,65.04,'Intermediate' from dual union all
select 9564,'009564','1976-114A','Kosmos-869','Soyuz 7K-S No. 3L',39066,'Landed','1976-12-17 00:00:00','1976-12-06 00:00:00',90.84,264,368,51.79,'Intermediate' from dual union all
select 9565,'009565','1976-114B','11S510','11S510',39066,'Reentered','1976-12-06 00:00:00','1976-12-01 00:00:00',89.06,191,265,51.77,'Intermediate' from dual union all
select 9566,'009566','1976-111E','Sensor cover','Zenit-6 sensor cover',39063,'Reentered','1976-12-11 00:00:00','1976-12-02 00:00:00',90.4,236,352,62.82,'Intermediate' from dual union all
select 9567,'009567','1975-004FP','deb Delta 107','',36451,'In Earth orbit','','1993-08-05 00:00:00',101.53,738,925,97.93,'Sun-Synch' from dual union all
select 9568,'009568','1973-086FW','deb Delta 98','',30688,'In Earth orbit','','1993-08-07 00:00:00',120.12,1387,1986,101.51,'Medium' from dual union all
select 9569,'009569','1973-098D','deb Kosmos-407 [UNID]','Strela-2M debris',30700,'In Earth orbit','','1993-08-16 00:00:00',99.41,719,742,74.06,'Intermediate' from dual union all
select 957,'000957','1964-084A','San Marco 1','San Marco 1',10370,'Reentered','1965-09-11 00:00:00','1965-01-15 00:00:00',94.41,198,782,37.77,'Intermediate' from dual union all
select 9570,'009570','1973-086FX','deb Delta 98','',30688,'In Earth orbit','','1993-08-04 00:00:00',116.02,1449,1554,101.74,'Sun-Synch' from dual union all
select 9571,'009571','1975-004FQ','deb Delta 107','',36451,'Reentered','1978-12-31 00:00:00','1978-12-16 00:00:00',99.7,692,796,97.66,'Sun-Synch' from dual union all
select 9572,'009572','1975-112C','deb Kosmos-783','Strela-2M debris',36559,'In Earth orbit','','1993-08-16 00:00:00',99.11,712,721,74.05,'Intermediate' from dual union all
select 9573,'009573','1976-115A','Kosmos-870','Tselina-OM',39067,'Reentered','1980-12-20 00:00:00','1977-01-01 00:00:00',95.21,511,547,74,'Intermediate' from dual union all
select 9574,'009574','1976-116A','Molniya-2','Molniya-2',39068,'Reentered','1991-01-18 00:00:00','1977-01-02 00:00:00',717.66,599,39750,62.93,'Molniya' from dual union all
select 9575,'009575','1976-116B','11S510','11S510',39068,'Reentered','1977-03-04 00:00:00','1977-01-01 00:00:00',92.14,210,548,62.84,'Intermediate' from dual union all
select 9576,'009576','1976-115B','S3M','S3M',39067,'Reentered','1981-02-28 00:00:00','1977-01-01 00:00:00',95.11,496,552,74.01,'Intermediate' from dual union all
select 9577,'009577','1976-115C','deb Kosmos-870','Tselina-O part',39067,'Reentered','1978-02-10 00:00:00','1977-01-25 00:00:00',94.51,479,510,73.96,'Intermediate' from dual union all
select 9578,'009578','1976-116C','BOZ','BOZ',39068,'Reentered','1977-02-07 00:00:00','1977-01-02 00:00:00',91.6,192,515,62.86,'Intermediate' from dual union all
select 9579,'009579','1976-116D','Blok-L','Blok-L',39068,'Reentered','1991-09-19 00:00:00','1977-01-10 00:00:00',731.98,596,40457,62.98,'Molniya' from dual union all
select 958,'000958','1964-084B','Altair 2 X-258C2 RH-75','X-258C1',10370,'Reentered','1965-02-07 00:00:00','1964-12-24 00:00:00',94.13,199,754,37.78,'Intermediate' from dual union all
select 9580,'009580','1976-116E','deb Blok-I','',39068,'Reentered','1977-02-13 00:00:00','1976-12-05 00:00:00',92.37,198,584,62.86,'Intermediate' from dual union all
select 9581,'009581','1976-116F','deb Blok-I','',39068,'Reentered','1977-01-11 00:00:00','1976-12-07 00:00:00',92.87,210,621,62.84,'Intermediate' from dual union all
select 9582,'009582','1976-111F','KDU','KDU',39063,'Reentered','1978-01-07 00:00:00','1976-12-24 00:00:00',91.99,350,395,62.82,'Intermediate' from dual union all
select 9583,'009583','1976-111G','KDU part','KDU part',39063,'Reentered','1977-05-26 00:00:00','1976-12-23 00:00:00',91.8,347,379,62.82,'Intermediate' from dual union all
select 9584,'009584','1976-111H','KDU part','KDU part',39063,'Reentered','1977-03-20 00:00:00','1976-12-06 00:00:00',92.03,352,397,62.86,'Intermediate' from dual union all
select 9585,'009585','1976-111J','KDU part','KDU part',39063,'Reentered','1977-07-07 00:00:00','1977-03-13 00:00:00',91.58,245,459,62.8,'Intermediate' from dual union all
select 9586,'009586','1976-111K','KDU part','KDU part',39063,'Reentered','1977-03-05 00:00:00','1976-12-25 00:00:00',91.86,343,389,62.82,'Intermediate' from dual union all
select 9587,'009587','1976-117A','Fanhui Shi Weixing (02)','FSW No. 3',39069,'Reentered','1977-01-02 00:00:00','1976-12-14 00:00:00',90.63,173,438,59.45,'Intermediate' from dual union all
select 9588,'009588','1976-118A','Kosmos-871','Strela-1M',39070,'In Earth orbit','','1992-12-21 00:00:00',114.68,1417,1465,74.03,'Intermediate' from dual union all
select 9589,'009589','1976-118B','Kosmos-872','Strela-1M',39070,'In Earth orbit','','1992-12-23 00:00:00',114.46,1399,1462,74.03,'Intermediate' from dual union all
select 959,'000959','1964-083C','Transit VE-5','APL SN-43',10369,'In Earth orbit','','1965-01-18 00:00:00',106.3,1027,1084,89.99,'Polar' from dual union all
select 9590,'009590','1976-118C','Kosmos-873','Strela-1M',39070,'In Earth orbit','','1992-12-22 00:00:00',115.54,1464,1495,74.03,'Intermediate' from dual union all
select 9591,'009591','1976-118D','Kosmos-874','Strela-1M',39070,'In Earth orbit','','1992-12-22 00:00:00',115.76,1464,1516,74.03,'Intermediate' from dual union all
select 9592,'009592','1976-118E','Kosmos-875','Strela-1M',39070,'In Earth orbit','','1993-01-16 00:00:00',114.88,1436,1463,74.03,'Intermediate' from dual union all
select 9593,'009593','1976-118F','Kosmos-876','Strela-1M',39070,'In Earth orbit','','1992-12-25 00:00:00',116,1463,1539,74.03,'Intermediate' from dual union all
select 9594,'009594','1976-118G','Kosmos-877','Strela-1M',39070,'In Earth orbit','','1992-12-22 00:00:00',115.07,1454,1463,74.03,'Intermediate' from dual union all
select 9595,'009595','1976-118H','Kosmos-878','Strela-1M',39070,'In Earth orbit','','1992-12-21 00:00:00',115.29,1464,1474,74.03,'Intermediate' from dual union all
select 9596,'009596','1976-117B','CZ-2C Stage 2','CZ-2C Stage 2',39069,'Reentered','1976-12-29 00:00:00','1976-12-14 00:00:00',90.34,169,414,59.45,'Intermediate' from dual union all
select 9597,'009597','1976-117C','CZ-2C motor cover','',39069,'Reentered','1976-12-09 00:00:00','1976-12-07 00:00:00',90.53,152,449,59.42,'Intermediate' from dual union all
select 9598,'009598','1976-118J','S3M','S3M',39070,'In Earth orbit','','1990-02-05 00:00:00',117.63,1465,1684,74.02,'Intermediate' from dual union all
select 9599,'009599','1976-119A','Kosmos-879','Zenit-2M',39071,'Landed','1976-12-22 00:00:00','1976-12-12 00:00:00',88.78,211,218,81.38,'Intermediate' from dual union all
select 96,'000096','61 Iota 2','8K72KE','8K72KE',7267,'Reentered','1961-03-26 00:00:00','1961-03-25 00:00:00',88.2,167,203,65,'Intermediate' from dual union all
select 960,'000960','1964-076K','deb Explorer XXIV','',10362,'Reentered','1989-04-30 00:00:00','1965-01-03 00:00:00',116.41,537,2502,81.33,'Intermediate' from dual union all
select 9600,'009600','1976-119B','11S510','11S510',39071,'Reentered','1976-12-12 00:00:00','1976-12-11 00:00:00',88.29,170,210,81.38,'Intermediate' from dual union all
select 9601,'009601','1976-120A','Kosmos-880','Lira',39072,'Reentered','1979-10-08 00:00:00','1977-01-10 00:00:00',96.42,558,616,65.84,'Intermediate' from dual union all
select 9602,'009602','1976-119C','Sensor cover','Zenit-2M sensor cover',39071,'Reentered','1977-01-03 00:00:00','1976-12-17 00:00:00',88.69,207,213,81.38,'Intermediate' from dual union all
select 9603,'009603','1976-119D','Sensor cover','Zenit-2M sensor cover',39071,'Reentered','1976-12-15 00:00:00','1976-12-10 00:00:00',88.7,186,235,81.37,'Intermediate' from dual union all
select 9604,'009604','1976-120B','S3M','S3M',39072,'Reentered','1984-12-21 00:00:00','1977-01-08 00:00:00',96.3,548,615,65.85,'Intermediate' from dual union all
select 9605,'009605','1976-120C','deb Kosmos-880','DS-P1-M part [RCS 0.34]',39072,'Reentered','1981-11-01 00:00:00','1977-01-10 00:00:00',96.43,558,618,65.85,'Intermediate' from dual union all
select 9606,'009606','1976-121A','Kosmos-881','TKS VA No. 009A/1',39073,'Landed','1976-12-15 00:00:00','1976-12-15 00:00:00',88.9,200,240,51.6,'Intermediate' from dual union all
select 9607,'009607','1976-121B','Kosmos-882','TKS VA No. 009/1',39073,'Landed','1976-12-15 00:00:00','1976-12-15 00:00:00',88.51,189,213,51.6,'Intermediate' from dual union all
select 9608,'009608','1976-121C','8S812','8S812',39073,'Reentered','1976-12-20 00:00:00','1976-12-18 00:00:00',88.08,177,182,51.6,'Intermediate' from dual union all
select 9609,'009609','1976-121D','82LB72 LVI-1','TKS GVM',39073,'Reentered','1976-12-23 00:00:00','1976-12-19 00:00:00',88.49,188,211,51.59,'Intermediate' from dual union all
select 961,'000961','1964-085A','OPS 3358','CORONA J-17',10371,'Reentered Attached','1965-01-14 00:00:00','1965-01-05 00:00:00',89.88,185,352,74.99,'Intermediate' from dual union all
select 9610,'009610','1976-122A','Kosmos-883','Tsikada',39074,'In Earth orbit','','1977-01-16 00:00:00',104.79,960,1009,82.96,'Intermediate' from dual union all
select 9611,'009611','1976-121E','TKS VA cover','',39073,'Reentered','1976-12-26 00:00:00','1976-12-20 00:00:00',88.28,182,197,51.6,'Intermediate' from dual union all
select 9612,'009612','1976-121F','deb Kosmos-881','',39073,'Reentered','1976-12-21 00:00:00','1976-12-16 00:00:00',88.58,179,230,51.59,'Intermediate' from dual union all
select 9613,'009613','1976-122B','S3M','S3M',39074,'In Earth orbit','','1991-03-25 00:00:00',104.61,953,999,82.95,'Intermediate' from dual union all
select 9614,'009614','1976-123A','Kosmos-884','Zenit-4MK',39075,'In Earth orbit','1976-12-29 00:00:00','1976-12-19 00:00:00',89.31,165,316,65.02,'Intermediate' from dual union all
select 9615,'009615','1976-124A','Kosmos-885','Romb 2',39076,'Reentered','1979-10-14 00:00:00','1977-01-19 00:00:00',94.36,464,512,65.84,'Intermediate' from dual union all
select 9616,'009616','1975-004FR','deb Delta 107','',36451,'Reentered','1981-11-02 00:00:00','1981-10-11 00:00:00',96.22,543,612,98.15,'Sun-Synch' from dual union all
select 9617,'009617','1975-004FS','deb Delta 107','',36451,'Reentered','2013-06-10 00:00:00','1993-08-05 00:00:00',95.65,530,570,97.58,'Sun-Synch' from dual union all
select 9618,'009618','1975-004FT','deb Delta 107','',36451,'Reentered','1980-10-31 00:00:00','1976-12-13 00:00:00',100.11,632,896,98.32,'Sun-Synch' from dual union all
select 9619,'009619','1975-004FU','deb Delta 107','',36451,'In Earth orbit','','1993-07-31 00:00:00',101.36,717,929,97.45,'Sun-Synch' from dual union all
select 962,'000962','1964-084C','San Marco 1 attach clamp','',10370,'Reentered','1965-01-21 00:00:00','1964-12-26 00:00:00',93.54,220,676,37.34,'Intermediate' from dual union all
select 9620,'009620','1975-004FV','deb Delta 107','',36451,'Reentered','1991-05-01 00:00:00','1991-05-01 00:00:00',88.45,190,206,97.83,'Sun-Synch' from dual union all
select 9621,'009621','1975-004FW','deb Delta 107','',36451,'Reentered','1988-05-31 00:00:00','1988-05-31 00:00:00',88.59,193,217,98.07,'Sun-Synch' from dual union all
select 9622,'009622','1975-004FX','deb Delta 107','',36451,'Reentered','1979-01-18 00:00:00','1976-12-13 00:00:00',97.77,500,804,97.72,'Sun-Synch' from dual union all
select 9623,'009623','1976-123B','11S510','11S510',39075,'Reentered','1976-12-23 00:00:00','1976-12-19 00:00:00',89.23,170,302,65.02,'Intermediate' from dual union all
select 9624,'009624','1976-123C','Sensor cover','Zenit-4MK sensor cover',39075,'Reentered','1977-01-31 00:00:00','1976-12-17 00:00:00',89.51,169,331,65.03,'Intermediate' from dual union all
select 9625,'009625','1976-124B','S3M','S3M',39076,'Reentered','1979-09-05 00:00:00','1977-01-18 00:00:00',94.24,456,508,65.85,'Intermediate' from dual union all
select 9626,'009626','1976-124C','Kosmos-885 SS 1','ESO',39076,'Reentered','1977-10-29 00:00:00','1977-02-16 00:00:00',94.34,467,507,65.82,'Intermediate' from dual;
insert into SATELLITE
select 9627,'009627','1976-125A','OPS 5705','KENNEN 1',39077,'Deorbited','1979-01-28 00:00:00','1977-01-18 00:00:00',93.27,339,531,96.95,'Sun-Synch' from dual union all
select 9628,'009628','1976-125B','Titan 23D-15 Stage 2','LR91AJ11 23D-15',39077,'Reentered','1977-05-09 00:00:00','1977-05-08 00:00:00',87.61,150,163,96.88,'Sun-Synch' from dual union all
select 9629,'009629','1976-125C','KH-11 sensor cover','KH-11 sensor cover',39077,'Reentered','1977-02-23 00:00:00','1976-12-24 00:00:00',91.48,252,443,96.98,'Sun-Synch' from dual union all
select 963,'000963','1964-086A','Explorer 26','EPE-D',10372,'Reentered','1978-04-01 00:00:00','1965-01-22 00:00:00',456.32,310,26202,20.14,'Highly Ellip' from dual union all
select 9630,'009630','1976-125D','KH-11 sensor cover','',39077,'Reentered','1977-02-09 00:00:00','1976-12-26 00:00:00',91.43,239,450,96.87,'Sun-Synch' from dual union all
select 9631,'009631','1976-104C','Kosmos-861 DU','US-A DU',39056,'Reentered','1977-02-04 00:00:00','1976-12-20 00:00:00',89.56,244,262,64.95,'Intermediate' from dual union all
select 9632,'009632','1976-104D','Kosmos-861 RLS','US-A RLS',39056,'Reentered','1976-12-25 00:00:00','1976-12-20 00:00:00',89.45,233,262,64.95,'Intermediate' from dual union all
select 9633,'009633','1976-104B','Cataloging error','[Spurious]',39056,'Spurious','1976-10-21 00:00:00','1976-12-20 00:00:00',104.31,926,999,64.8,'Intermediate' from dual union all
select 9634,'009634','1976-126A','Kosmos-886','IS',39078,'In Earth orbit','','1977-01-07 00:00:00',114.93,595,2310,65.83,'Intermediate' from dual union all
select 9635,'009635','1976-127A','Molniya-3','Molniya-3 No. 17',39079,'Reentered','1990-02-06 00:00:00','1977-01-27 00:00:00',717.72,628,39724,62.9,'Molniya' from dual union all
select 9636,'009636','1976-127B','11S510','11S510',39079,'Reentered','1977-03-16 00:00:00','1977-02-05 00:00:00',91.5,210,487,62.83,'Intermediate' from dual union all
select 9637,'009637','1976-128A','Kosmos-887','Parus',39080,'In Earth orbit','','1977-01-27 00:00:00',104.76,951,1016,82.94,'Intermediate' from dual union all
select 9638,'009638','1976-128B','S3M','S3M',39080,'In Earth orbit','','1991-03-28 00:00:00',104.55,950,997,82.94,'Intermediate' from dual union all
select 9639,'009639','1976-123D','Blenda','Sunshade',39075,'Reentered','1976-12-31 00:00:00','1976-12-28 00:00:00',89.11,168,293,65.02,'Intermediate' from dual union all
select 964,'000964','1964-087A','FTV 2355','QUILL',10373,'Reentered','1965-01-11 00:00:00','1965-01-06 00:00:00',88.79,208,221,70.08,'Intermediate' from dual union all
select 9640,'009640','1976-123E','KDU','KDU',39075,'Reentered','1977-01-01 00:00:00','1976-12-28 00:00:00',89.28,167,310,65.03,'Intermediate' from dual union all
select 9641,'009641','1976-127C','BOZ','BOZ',39079,'Reentered','1977-02-15 00:00:00','1977-01-04 00:00:00',92.28,190,583,62.84,'Intermediate' from dual union all
select 9642,'009642','1976-127D','deb Blok-I','',39079,'Reentered','1977-02-23 00:00:00','1977-01-13 00:00:00',92.19,210,553,62.82,'Intermediate' from dual union all
select 9643,'009643','1976-126B','deb Kosmos-886','',39078,'Reentered','1977-01-14 00:00:00','1976-12-30 00:00:00',89.88,130,407,62.73,'Intermediate' from dual union all
select 9644,'009644','1976-126C','deb Kosmos-886','',39078,'Reentered','1976-12-31 00:00:00','1976-12-28 00:00:00',94.23,135,828,62.74,'Intermediate' from dual union all
select 9645,'009645','1976-126D','11S692','11S692',39078,'Reentered','1976-12-30 00:00:00','1976-12-30 00:00:00',95.88,140,982,62.78,'Intermediate' from dual union all
select 9646,'009646','1976-123F','KDU part','KDU part',39075,'Reentered','1977-01-03 00:00:00','1976-12-29 00:00:00',88.96,159,288,65.05,'Intermediate' from dual union all
select 9647,'009647','1976-127E','Blok-L','Blok-L',39079,'Reentered','1987-09-30 00:00:00','1977-02-10 00:00:00',732.41,627,40447,62.86,'Molniya' from dual union all
select 9648,'009648','1976-126E','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-05 00:00:00',105.81,473,1591,65.82,'Intermediate' from dual union all
select 9649,'009649','1976-126F','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-04 00:00:00',114.17,678,2158,65.93,'Intermediate' from dual union all
select 965,'000965','1964-083D','NNS O-2','NNS O-2',10369,'In Earth orbit','','1965-01-13 00:00:00',106.3,1027,1084,89.98,'Polar' from dual union all
select 9650,'009650','1976-126G','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-05 00:00:00',110.91,498,2040,65.82,'Intermediate' from dual union all
select 9651,'009651','1976-126H','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-07 00:00:00',111.1,552,2003,65.82,'Intermediate' from dual union all
select 9652,'009652','1976-126J','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-07 00:00:00',112.91,608,2113,65.84,'Intermediate' from dual union all
select 9653,'009653','1976-126K','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-07 00:00:00',111.45,574,2013,65.81,'Intermediate' from dual union all
select 9654,'009654','1976-126L','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-09 00:00:00',110.97,586,1957,65.81,'Intermediate' from dual union all
select 9655,'009655','1976-126M','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-06 00:00:00',109.11,487,1885,65.81,'Intermediate' from dual union all
select 9656,'009656','1976-126N','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-07 00:00:00',113.33,650,2109,65.83,'Intermediate' from dual union all
select 9657,'009657','1976-126P','deb Kosmos-886','',39078,'Reentered','1991-01-06 00:00:00','1991-01-05 00:00:00',90.95,315,328,65.67,'Intermediate' from dual union all
select 9658,'009658','1977-001A','Kosmos-888','Zenit-4MK',42129,'Landed','1977-01-19 00:00:00','1977-01-09 00:00:00',89.37,168,319,64.98,'Intermediate' from dual union all
select 9659,'009659','1977-001B','11S510','11S510',42129,'Reentered','1977-01-11 00:00:00','1977-01-08 00:00:00',89,163,288,64.99,'Intermediate' from dual union all
select 966,'000966','1964-083E','deb NNSS 30020','',10369,'Reentered','1982-06-10 00:00:00','1965-01-31 00:00:00',106.1,1015,1077,89.8,'Polar' from dual union all
select 9660,'009660','1977-001C','Sensor cover','Zenit-4MK sensor cover',42129,'Reentered','1977-01-11 00:00:00','1977-01-06 00:00:00',89.87,168,368,64.98,'Intermediate' from dual union all
select 9661,'009661','1977-002A','Meteor-2','Meteor-2 No. 2',42130,'In Earth orbit','','1990-03-31 00:00:00',102.76,882,897,81.26,'Intermediate' from dual union all
select 9662,'009662','1977-002B','8A92ME','8A92ME',42130,'In Earth orbit','','1993-08-17 00:00:00',102.85,856,931,81.28,'Intermediate' from dual union all
select 9663,'009663','1977-002C','deb Meteor-2','',42130,'In Earth orbit','','1993-08-17 00:00:00',102.73,882,894,81.26,'Intermediate' from dual union all
select 9664,'009664','1977-002D','deb Meteor-2','',42130,'In Earth orbit','','1993-08-17 00:00:00',102.77,883,897,81.26,'Intermediate' from dual union all
select 9665,'009665','1976-075F','deb Kosmos-845','Tselina-O part',39027,'Reentered','1978-07-18 00:00:00','1977-01-10 00:00:00',94.8,482,536,73.99,'Intermediate' from dual union all
select 9666,'009666','1970-025NK','deb Agena D','',21502,'In Earth orbit','','1993-08-01 00:00:00',107.56,1007,1221,99.27,'Sun-Synch' from dual union all
select 9667,'009667','1970-025NL','deb Agena D','',21502,'Lost','','1993-08-04 00:00:00',107.67,1080,1157,100.56,'Sun-Synch' from dual union all
select 9668,'009668','1970-025NM','deb Agena D','',21502,'In Earth orbit','','1993-08-05 00:00:00',102.51,796,959,100.15,'Sun-Synch' from dual union all
select 9669,'009669','1972-058HK','deb Delta 89','',27990,'Reentered','1979-02-24 00:00:00','1976-09-22 00:00:00',99.06,657,771,98.9,'Sun-Synch' from dual union all
select 967,'000967','1964-083F','deb NNSS 30020','',10369,'In Earth orbit','','1965-01-21 00:00:00',106.28,1025,1084,89.99,'Polar' from dual union all
select 9670,'009670','1972-058HL','deb Delta 89','',27990,'Reentered','1982-05-08 00:00:00','1977-01-10 00:00:00',100.11,672,856,98.67,'Sun-Synch' from dual union all
select 9671,'009671','1972-058HM','deb Delta 89','',27990,'Reentered','1980-03-14 00:00:00','1977-01-10 00:00:00',99.91,644,864,98.38,'Sun-Synch' from dual union all
select 9672,'009672','1972-058HN','deb Delta 89','',27990,'Reentered','1978-07-02 00:00:00','1978-01-03 00:00:00',94.93,454,577,99.57,'Sun-Synch' from dual union all
select 9673,'009673','1972-058HP','deb Delta 89','',27990,'Reentered','1979-06-16 00:00:00','1979-02-07 00:00:00',95.79,510,603,98.59,'Sun-Synch' from dual union all
select 9674,'009674','1972-058HQ','deb Delta 89','',27990,'In Earth orbit','','1993-08-04 00:00:00',98.18,577,767,98.35,'Sun-Synch' from dual union all
select 9675,'009675','1972-058HR','deb Delta 89','',27990,'Reentered','1979-05-23 00:00:00','1979-03-23 00:00:00',98.18,585,758,98,'Sun-Synch' from dual union all
select 9676,'009676','1972-058HS','deb Delta 89','',27990,'Reentered','1981-07-28 00:00:00','1977-01-10 00:00:00',98.84,650,757,99.02,'Sun-Synch' from dual union all
select 9677,'009677','1975-004FY','deb Delta 107','',36451,'Reentered','1981-06-11 00:00:00','1981-02-24 00:00:00',96.73,585,620,98.18,'Sun-Synch' from dual union all
select 9678,'009678','1975-004FZ','deb Delta 107','',36451,'Reentered','1979-06-14 00:00:00','1979-04-24 00:00:00',97.64,627,665,97.98,'Sun-Synch' from dual union all
select 9679,'009679','1975-004GA','deb Delta 107','',36451,'Reentered','1990-09-20 00:00:00','1977-01-10 00:00:00',99.7,691,798,97.91,'Sun-Synch' from dual union all
select 968,'000968','1965-001A','Kosmos-52','Zenit-2 No. 26',11751,'Landed','1965-01-19 00:00:00','1965-01-13 00:00:00',89.45,200,295,64.99,'Intermediate' from dual union all
select 9680,'009680','1975-004GB','deb Delta 107','',36451,'Reentered','1979-07-17 00:00:00','1979-07-14 00:00:00',99.65,718,766,97.98,'Sun-Synch' from dual union all
select 9681,'009681','1975-004GC','deb Delta 107','',36451,'In Earth orbit','','1993-07-30 00:00:00',100.18,724,810,97.95,'Sun-Synch' from dual union all
select 9682,'009682','1975-004GD','deb Delta 107','',36451,'Reentered','1992-01-16 00:00:00','1992-01-15 00:00:00',88.73,205,219,98.32,'Sun-Synch' from dual union all
select 9683,'009683','1975-004GE','deb Delta 107','',36451,'Reentered','1981-06-19 00:00:00','1981-04-10 00:00:00',95.63,505,594,97.92,'Sun-Synch' from dual union all
select 9684,'009684','1975-004GF','deb Delta 107','',36451,'Reentered','1979-02-20 00:00:00','1979-01-05 00:00:00',94.54,436,557,96.66,'Sun-Synch' from dual union all
select 9685,'009685','1975-004GG','deb Delta 107','',36451,'Reentered','1986-07-17 00:00:00','1986-07-03 00:00:00',90.32,284,297,98.71,'Sun-Synch' from dual union all
select 9686,'009686','1975-004GH','deb Delta 107','',36451,'Reentered','1979-06-16 00:00:00','1977-01-10 00:00:00',102.01,735,973,97.51,'Sun-Synch' from dual union all
select 9687,'009687','1975-004GJ','deb Delta 107','',36451,'Reentered','1979-11-30 00:00:00','1979-10-29 00:00:00',97.2,569,681,98.17,'Sun-Synch' from dual union all
select 9688,'009688','1975-004GK','deb Delta 107','',36451,'Reentered','1980-09-11 00:00:00','1980-09-08 00:00:00',90.77,303,321,98.13,'Sun-Synch' from dual union all
select 9689,'009689','1975-004GL','deb Delta 107','',36451,'Reentered','1990-09-12 00:00:00','1977-01-11 00:00:00',100.48,748,815,98.05,'Sun-Synch' from dual union all
select 969,'000969','1965-001B','8A92E','8A92E',11751,'Reentered','1965-01-29 00:00:00','1965-01-18 00:00:00',89.34,211,273,65,'Intermediate' from dual union all
select 9690,'009690','1975-004GM','deb Delta 107','',36451,'Reentered','1980-06-06 00:00:00','1978-06-09 00:00:00',93.04,367,480,97.64,'Sun-Synch' from dual union all
select 9691,'009691','1975-004GN','deb Delta 107','',36451,'Reentered','1978-06-21 00:00:00','1978-05-29 00:00:00',92.37,358,424,97.62,'Sun-Synch' from dual union all
select 9692,'009692','1969-029AP','deb Blok-E','',18296,'Reentered','1978-07-18 00:00:00','1978-05-04 00:00:00',92.51,361,435,81.19,'Intermediate' from dual union all
select 9693,'009693','1975-109H','deb Kosmos-781','Tselina-O part',36556,'Reentered','1977-09-11 00:00:00','1977-01-10 00:00:00',93.78,439,480,74.04,'Intermediate' from dual union all
select 9694,'009694','1963-014DE','deb West Ford','RCS 0.13',9245,'Reentered','1982-11-22 00:00:00','1977-01-11 00:00:00',162.74,882,6109,84.57,'Medium' from dual union all
select 9695,'009695','1974-089DU','deb Delta 104','',33692,'In Earth orbit','','1993-08-08 00:00:00',108.28,892,1402,103.04,'Sun-Synch' from dual union all
select 9696,'009696','1974-089DV','deb Delta 104','',33692,'In Earth orbit','','1993-08-03 00:00:00',118.21,1451,1750,100.85,'Sun-Synch' from dual union all
select 9697,'009697','1974-089DW','deb Delta 104','',33692,'In Earth orbit','','1993-08-06 00:00:00',115.02,1342,1570,101.81,'Sun-Synch' from dual union all
select 9698,'009698','1974-089DX','deb Delta 104','',33692,'Reentered','2004-11-14 00:00:00','1993-08-07 00:00:00',110.38,1072,1417,101.92,'Sun-Synch' from dual union all
select 9699,'009699','1974-089DY','deb Delta 104','',33692,'In Earth orbit','','1993-08-06 00:00:00',117.17,1456,1652,101.56,'Sun-Synch' from dual union all
select 97,'000097','61 Iota 3','deb Korabl''-Sputnik-5','',7267,'Reentered','1961-03-26 00:00:00','1961-03-25 00:00:00',88.52,167,236,65,'Intermediate' from dual union all
select 970,'000970','1965-001C','deb Kosmos-52','',11751,'Reentered','1965-01-14 00:00:00','1965-01-11 00:00:00',89.27,179,298,64.95,'Intermediate' from dual union all
select 9700,'009700','1973-086FY','deb Delta 98','',30688,'Reentered','1979-10-25 00:00:00','1979-10-14 00:00:00',103.69,716,1150,101.92,'Sun-Synch' from dual union all
select 9701,'009701','1973-086FZ','deb Delta 98','',30688,'In Earth orbit','','1993-08-08 00:00:00',113.99,1375,1444,102.79,'Sun-Synch' from dual union all
select 9702,'009702','1973-086GA','deb Delta 98','',30688,'In Earth orbit','','1993-08-07 00:00:00',127.46,1381,2642,103.21,'Medium' from dual union all
select 9703,'009703','1973-086GB','deb Delta 98','[UNID073] RCS 0.04',30688,'In Earth orbit','','1993-08-01 00:00:00',105.57,929,1114,65.72,'Intermediate' from dual union all
select 9704,'009704','1976-069C','deb Kosmos-841','Strela-2M debris',39021,'In Earth orbit','','1993-08-16 00:00:00',99.32,722,730,74.07,'Intermediate' from dual union all
select 9705,'009705','1975-004GP','deb Delta 107','',36451,'Reentered','1978-04-28 00:00:00','1978-03-27 00:00:00',92.57,326,475,98.23,'Sun-Synch' from dual union all
select 9706,'009706','1975-004GQ','deb Delta 107','',36451,'Reentered','1978-12-04 00:00:00','1978-10-30 00:00:00',92.71,369,445,97.76,'Sun-Synch' from dual union all
select 9707,'009707','1975-004GR','deb Delta 107','',36451,'Reentered','1979-11-16 00:00:00','1979-11-08 00:00:00',96.94,574,651,97.68,'Sun-Synch' from dual union all
select 9708,'009708','1975-004GS','deb Delta 107','',36451,'Reentered','1979-05-02 00:00:00','1979-04-27 00:00:00',93.78,444,475,97.61,'Sun-Synch' from dual union all
select 9709,'009709','1975-004GT','deb Delta 107','',36451,'Reentered','1978-03-12 00:00:00','1978-01-25 00:00:00',92.39,339,445,97.66,'Sun-Synch' from dual union all
select 971,'000971','1965-001D','deb Kosmos-52','',11751,'Reentered','1965-01-14 00:00:00','1965-01-12 00:00:00',89.4,208,282,65.06,'Intermediate' from dual union all
select 9710,'009710','1975-004GU','deb Delta 107','',36451,'In Earth orbit','','1993-08-08 00:00:00',101.98,730,975,97.7,'Sun-Synch' from dual union all
select 9711,'009711','1975-004GV','deb Delta 107','',36451,'Reentered','2002-10-23 00:00:00','1993-08-07 00:00:00',98.13,639,700,97.77,'Sun-Synch' from dual union all
select 9712,'009712','1975-004GW','deb Delta 107','',36451,'Reentered','1979-01-03 00:00:00','1977-01-12 00:00:00',98.55,631,748,97.79,'Sun-Synch' from dual union all
select 9713,'009713','1975-004GX','deb Delta 107','',36451,'Reentered','1978-07-18 00:00:00','1978-04-10 00:00:00',93.79,407,513,99.03,'Sun-Synch' from dual union all
select 9714,'009714','1975-004GY','deb Delta 107','',36451,'Reentered','1979-01-07 00:00:00','1977-01-13 00:00:00',100.72,749,837,97.69,'Sun-Synch' from dual union all
select 9715,'009715','1975-004GZ','deb Delta 107','',36451,'Reentered','1979-05-09 00:00:00','1979-02-10 00:00:00',96.97,524,704,97.55,'Sun-Synch' from dual union all
select 9716,'009716','1975-004HA','deb Delta 107','',36451,'Reentered','1977-11-25 00:00:00','1977-10-25 00:00:00',92.54,331,467,97.33,'Sun-Synch' from dual;
insert into SATELLITE
select 9717,'009717','1975-004HB','deb Delta 107','',36451,'Reentered','1979-04-11 00:00:00','1979-02-09 00:00:00',97.99,636,690,98.27,'Sun-Synch' from dual union all
select 9718,'009718','1976-075G','deb Kosmos-845','Tselina-O part',39027,'Reentered','1978-03-27 00:00:00','1977-01-13 00:00:00',94.77,479,536,74.11,'Intermediate' from dual union all
select 9719,'009719','1976-126Q','deb Kosmos-886','',39078,'Reentered','1978-07-11 00:00:00','1978-05-02 00:00:00',102.58,282,1480,65.89,'Intermediate' from dual union all
select 972,'000972','1965-002A','OPS 3928','CORONA J-18',11752,'Reentered Attached','1965-02-09 00:00:00','1965-01-25 00:00:00',90.39,179,409,74.94,'Intermediate' from dual union all
select 9720,'009720','1976-126R','deb Kosmos-886','',39078,'Reentered','1981-10-09 00:00:00','1977-02-01 00:00:00',112.38,405,2267,65.82,'Intermediate' from dual union all
select 9721,'009721','1976-126S','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-06 00:00:00',113.39,537,2228,65.84,'Intermediate' from dual union all
select 9722,'009722','1976-126T','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-07 00:00:00',116.58,838,2216,65.75,'Intermediate' from dual union all
select 9723,'009723','1976-126U','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-08 00:00:00',113.67,734,2056,65.77,'Intermediate' from dual union all
select 9724,'009724','1976-126V','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-08 00:00:00',111.74,594,2020,65.82,'Intermediate' from dual union all
select 9725,'009725','1976-126W','deb Kosmos-886','',39078,'Reentered','1992-12-21 00:00:00','1992-12-21 00:00:00',88.97,208,239,65.69,'Intermediate' from dual union all
select 9726,'009726','1973-086GC','deb Delta 98','',30688,'In Earth orbit','','1993-08-03 00:00:00',117.48,1503,1633,102.58,'Sun-Synch' from dual union all
select 9727,'009727','1972-058HT','deb Delta 89','',27990,'Reentered','2002-10-21 00:00:00','1993-08-06 00:00:00',97.05,596,638,98.36,'Sun-Synch' from dual union all
select 9728,'009728','1972-058HU','deb Delta 89','',27990,'Reentered','1979-04-10 00:00:00','1977-01-12 00:00:00',99.61,631,849,98.93,'Sun-Synch' from dual union all
select 9729,'009729','1972-058HV','deb Delta 89','',27990,'Reentered','1977-04-21 00:00:00','1977-04-16 00:00:00',92.98,420,421,99.02,'Sun-Synch' from dual union all
select 973,'000973','1965-003A','OPS 7040','DAPP 10',11753,'Reentered Attached','1979-07-13 00:00:00','1965-02-20 00:00:00',97.63,461,830,98.76,'Sun-Synch' from dual union all
select 9730,'009730','1974-089DZ','deb Delta 104','',33692,'In Earth orbit','','1993-08-02 00:00:00',111.64,1241,1364,101.8,'Sun-Synch' from dual union all
select 9731,'009731','1976-001E','deb Kosmos-787','Tselina-O part',38953,'Reentered','1978-07-15 00:00:00','1977-01-12 00:00:00',94.49,473,515,73.97,'Intermediate' from dual union all
select 9732,'009732','1977-001D','KDU','KDU',42129,'Reentered','1977-01-23 00:00:00','1977-01-18 00:00:00',89.09,164,295,65.02,'Intermediate' from dual union all
select 9733,'009733','1977-001E','Blenda','Sunshade',42129,'Reentered','1977-01-21 00:00:00','1977-01-18 00:00:00',89.27,166,310,64.97,'Intermediate' from dual union all
select 9734,'009734','1977-001F','KDU part','KDU part',42129,'Reentered','1977-01-20 00:00:00','1977-01-18 00:00:00',89.34,173,311,64.9,'Intermediate' from dual union all
select 9735,'009735','1977-003A','Kosmos-889','Zenit-2M',42131,'Landed','1977-02-01 00:00:00','1977-01-22 00:00:00',89.79,201,328,71.36,'Intermediate' from dual union all
select 9736,'009736','1977-003B','11S510','11S510',42131,'Reentered','1977-02-01 00:00:00','1977-01-22 00:00:00',89.47,204,293,71.39,'Intermediate' from dual union all
select 9737,'009737','1977-004A','Kosmos-890','Zaliv',42132,'In Earth orbit','','1992-12-28 00:00:00',105.02,976,1015,82.96,'Intermediate' from dual union all
select 9738,'009738','1977-004B','S3M','S3M',42132,'In Earth orbit','','1991-03-27 00:00:00',104.89,978,1001,82.95,'Intermediate' from dual union all
select 9739,'009739','1966-056AK','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 974,'000974','1965-003B','P35 despin weight','',11753,'Reentered','1965-06-17 00:00:00','1965-02-20 00:00:00',97.15,458,787,98.81,'Sun-Synch' from dual union all
select 9740,'009740','1966-056AL','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9741,'009741','1966-056AM','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9742,'009742','1966-056AN','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9743,'009743','1966-056AP','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9744,'009744','1966-056AQ','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9745,'009745','1966-056AR','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9746,'009746','1966-056AS','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9747,'009747','1966-056AT','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9748,'009748','1966-056AU','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9749,'009749','1966-056AV','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 975,'000975','1965-003C','P35 despin weight','',11753,'Reentered','1965-07-09 00:00:00','1965-02-18 00:00:00',97.37,467,798,98.69,'Sun-Synch' from dual union all
select 9750,'009750','1966-056AW','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9751,'009751','1966-056AX','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9752,'009752','1966-056AY','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9753,'009753','1966-056AZ','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9754,'009754','1966-056BA','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9755,'009755','1966-056BB','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9756,'009756','1966-056BC','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9757,'009757','1966-056BD','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9758,'009758','1966-056BE','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9759,'009759','1966-056BF','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 976,'000976','1964-064C','Explorer XXII despin weight','',10350,'In Earth orbit','','1965-01-20 00:00:00',104.04,842,1057,79.32,'Intermediate' from dual union all
select 9760,'009760','1966-056BG','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9761,'009761','1966-056BH','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9762,'009762','1966-056BJ','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9763,'009763','1966-056BK','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9764,'009764','1966-056BL','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9765,'009765','1966-056BM','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9766,'009766','1966-056BN','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9767,'009767','1966-056BP','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9768,'009768','1966-056BQ','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9769,'009769','1966-056BR','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 977,'000977','1964-064D','Explorer XXII despin weight','',10350,'In Earth orbit','','1965-01-20 00:00:00',105.48,914,1119,80.07,'Intermediate' from dual union all
select 9770,'009770','1966-056BS','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9771,'009771','1966-056BT','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9772,'009772','1966-056BU','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9773,'009773','1966-056BV','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9774,'009774','1966-056BW','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9775,'009775','1966-056BX','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9776,'009776','1966-056BY','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9777,'009777','1966-056BZ','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9778,'009778','1966-056CA','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9779,'009779','1966-056CB','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 978,'000978','1965-004A','Tiros 9','Tiros I',11754,'In Earth orbit','','1965-03-02 00:00:00',119.14,702,2583,96.41,'Retrograde' from dual union all
select 9780,'009780','1966-056CC','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9781,'009781','1966-056CD','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9782,'009782','1966-056CE','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9783,'009783','1966-056CF','deb Pageos','',13463,'Reentered','1977-01-20 00:00:00','1976-08-31 00:00:00',179.17,3000,5300,85.2,'Medium' from dual union all
select 9784,'009784','1970-025NN','deb Agena D','',21502,'Reentered','1989-11-24 00:00:00','1989-11-23 00:00:00',91.05,305,348,99.95,'Retrograde' from dual union all
select 9785,'009785','1977-005A','NATO 3B','NATO 3B',42133,'In Earth orbit','','1977-04-14 00:00:00',1435.83,35777,35786,2.72,'Stationary' from dual union all
select 9786,'009786','1977-005B','Delta 128','Delta SSPU DSV-3P-4',42133,'In Earth orbit','','1993-08-17 00:00:00',103.63,614,1247,28.01,'Intermediate' from dual union all
select 9787,'009787','1977-005C','Star 37E S/N 40045','Star 37E',42133,'Reentered','1981-02-06 00:00:00','1977-04-01 00:00:00',613.16,163,34897,26.76,'GEO Transfer' from dual union all
select 9788,'009788','1972-058HW','deb Delta 89','',27990,'Reentered','1981-04-13 00:00:00','1977-01-31 00:00:00',99.13,573,862,98.02,'Sun-Synch' from dual union all
select 9789,'009789','1976-075H','deb Kosmos-845','Tselina-O part',39027,'Reentered','1978-06-24 00:00:00','1977-01-31 00:00:00',94.87,487,538,74.03,'Intermediate' from dual union all
select 979,'000979','1965-004B','Altair 2 X-258C4 RH-79','X-258C4 RH-79',11754,'In Earth orbit','','1965-02-25 00:00:00',119.27,701,2596,96.43,'Retrograde' from dual union all
select 9790,'009790','1976-001F','deb Kosmos-787','Tselina-O part',38953,'Reentered','1977-08-05 00:00:00','1977-02-01 00:00:00',93.62,428,476,74.07,'Intermediate' from dual union all
select 9791,'009791','1976-001G','deb Kosmos-787','Tselina-O part',38953,'Reentered','1977-12-01 00:00:00','1977-01-31 00:00:00',94.25,458,507,74.08,'Intermediate' from dual union all
select 9792,'009792','1976-001H','deb Kosmos-787','Tselina-O part',38953,'Reentered','1977-12-12 00:00:00','1977-01-31 00:00:00',94.34,464,509,74.01,'Intermediate' from dual union all
select 9793,'009793','1974-100J','deb Kosmos-698','[UNID074] Tselina-O part',33703,'Reentered','1977-08-01 00:00:00','1977-01-30 00:00:00',92.77,395,426,74.01,'Intermediate' from dual union all
select 9794,'009794','1970-102F','deb S3M','',21579,'Reentered','1998-12-22 00:00:00','1993-08-15 00:00:00',98.21,664,683,74.03,'Intermediate' from dual union all
select 9795,'009795','1974-089EA','deb Delta 104','',33692,'In Earth orbit','','1993-08-05 00:00:00',114.4,1194,1663,101.97,'Sun-Synch' from dual union all
select 9796,'009796','1976-038K','deb USN satellite','',38990,'In Earth orbit','','1977-02-01 00:00:00',106.18,600,1500,63.4,'Intermediate' from dual union all
select 9797,'009797','1976-126X','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-07 00:00:00',113.9,650,2161,65.82,'Intermediate' from dual union all
select 9798,'009798','1976-126Y','deb Kosmos-886 [*]','',39078,'In Earth orbit','','1993-08-01 00:00:00',105.88,939,1133,65.75,'Intermediate' from dual union all
select 9799,'009799','1976-126Z','deb Kosmos-397','[UNID075] RCS 0.0',39078,'In Earth orbit','','1993-03-04 00:00:00',113.15,563,2180,65.6,'Intermediate' from dual union all
select 98,'000098','61 Kappa','Explorer 10','P-14',7268,'Deep Space','1961-03-26 00:00:00','1961-03-25 00:00:00',5010.03,220,180999,33,'Deep Space' from dual union all
select 980,'000980','1965-005A','OPS 4703','GAMBIT SV 965',11755,'Deorbited','1965-01-29 00:00:00','1965-01-25 00:00:00',88.83,141,292,102.54,'Retrograde' from dual union all
select 9800,'009800','1971-120E','deb Meteor','',24938,'Reentered','1984-04-24 00:00:00','1977-01-31 00:00:00',102.69,874,897,81.28,'Intermediate' from dual union all
select 9801,'009801','1977-006A','Kosmos-891','Vektor',42134,'Reentered','1981-02-04 00:00:00','1977-03-10 00:00:00',94.46,468,516,65.84,'Intermediate' from dual union all
select 9802,'009802','1977-006B','S3M','S3M',42134,'Reentered','1979-12-18 00:00:00','1977-03-04 00:00:00',94.38,462,516,65.84,'Intermediate' from dual union all
select 9803,'009803','1977-007A','DSP F7','DSP 9',42135,'In Earth orbit','','1979-10-31 00:00:00',1436.09,35777,35796,2.02,'Stationary' from dual union all
select 9804,'009804','1977-008A','Soyuz-24','Soyuz 7K-T(A9) No. 66',42136,'Landed','1977-02-25 00:00:00','1977-02-13 00:00:00',89.57,246,260,51.6,'Intermediate' from dual union all
select 9805,'009805','1977-008B','11S510','11S510',42136,'Reentered','1977-02-12 00:00:00','1977-02-09 00:00:00',89.19,172,297,51.62,'Intermediate' from dual union all
select 9806,'009806','1977-008C','Soyuz-24 part','Soyuz part',42136,'Reentered','1977-02-12 00:00:00','1977-02-09 00:00:00',89.38,175,313,51.61,'Intermediate' from dual;
insert into SATELLITE
select 9807,'009807','1976-126AA','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-07 00:00:00',108.27,532,1762,65.83,'Intermediate' from dual union all
select 9808,'009808','1977-007B','Titan 23C-5 Stage 2','LR91AJ11 23C-5',42135,'Reentered','1977-02-06 00:00:00','1977-02-06 00:00:00',90.01,150,400,28.6,'Intermediate' from dual union all
select 9809,'009809','1977-005D','Delta 128 clamp band','',42133,'Reentered','1980-01-01 00:00:00','1977-02-01 00:00:00',104.25,619,1300,28.01,'Intermediate' from dual union all
select 981,'000981','1965-005B','Agena D 4812','Agena D',11755,'Reentered','1965-01-25 00:00:00','1965-01-24 00:00:00',88.87,148,289,102.54,'Retrograde' from dual union all
select 9810,'009810','1977-005E','Delta 128 clamp band','',42133,'Reentered','1980-01-01 00:00:00','1977-02-01 00:00:00',104.25,619,1300,28.01,'Intermediate' from dual union all
select 9811,'009811','1977-005F','deb Delta','',42133,'Reentered','1980-01-01 00:00:00','1977-02-01 00:00:00',104.25,619,1300,28.01,'Intermediate' from dual union all
select 9812,'009812','1977-009A','Kosmos-892','Zenit-4MK',42137,'Landed','1977-02-22 00:00:00','1977-02-12 00:00:00',89.57,167,340,72.85,'Intermediate' from dual union all
select 9813,'009813','1977-009B','11S510','11S510',42137,'Reentered','1977-02-15 00:00:00','1977-02-11 00:00:00',89.97,155,391,72.86,'Intermediate' from dual union all
select 9814,'009814','1977-009C','Sensor cover','Zenit-4MK sensor cover',42137,'Reentered','1977-02-10 00:00:00','1977-02-10 00:00:00',89.12,149,312,72.85,'Intermediate' from dual union all
select 9815,'009815','1976-126AB','deb Kosmos-886','',39078,'Reentered','1999-10-04 00:00:00','1978-04-01 00:00:00',114.03,526,2297,65.89,'Intermediate' from dual union all
select 9816,'009816','1976-126AC','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-08 00:00:00',108.98,512,1848,65.82,'Intermediate' from dual union all
select 9817,'009817','1976-126AD','deb Kosmos-886','',39078,'Reentered','1982-07-12 00:00:00','1977-02-12 00:00:00',113.57,482,2299,65.83,'Intermediate' from dual union all
select 9818,'009818','1976-126AE','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-08 00:00:00',107.11,531,1654,65.79,'Intermediate' from dual union all
select 9819,'009819','1976-126AF','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-08 00:00:00',108.18,529,1757,65.93,'Intermediate' from dual union all
select 982,'000982','1965-002B','Fairing','Fairing',11752,'Reentered','1965-01-30 00:00:00','1965-01-25 00:00:00',88.13,182,182,74.9,'Intermediate' from dual union all
select 9820,'009820','1976-126AG','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-05 00:00:00',120.84,1017,2420,65.8,'Medium' from dual union all
select 9821,'009821','1976-126AH','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-06 00:00:00',107.59,533,1698,65.8,'Intermediate' from dual union all
select 9822,'009822','1976-126AJ','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-07 00:00:00',113,585,2144,65.8,'Intermediate' from dual union all
select 9823,'009823','1976-126AK','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-05 00:00:00',112.61,580,2113,65.8,'Intermediate' from dual union all
select 9824,'009824','1976-126AL','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-06 00:00:00',110.91,498,2040,65.82,'Intermediate' from dual union all
select 9825,'009825','1976-126AM','deb Kosmos-886','',39078,'Reentered','1982-03-12 00:00:00','1982-03-11 00:00:00',90.04,242,311,65.75,'Intermediate' from dual union all
select 9826,'009826','1976-126AN','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-07 00:00:00',110.74,550,1972,65.82,'Intermediate' from dual union all
select 9827,'009827','1976-126AP','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-06 00:00:00',107.61,540,1693,65.83,'Intermediate' from dual union all
select 9828,'009828','1976-126AQ','deb Kosmos-397','[UNID076] RCS 0.1',39078,'Reentered','2002-01-05 00:00:00','1993-08-07 00:00:00',102.66,527,1242,65.81,'Intermediate' from dual union all
select 9829,'009829','1977-010A','Molniya-2','Molniya-2',42138,'In Earth orbit','','1977-03-15 00:00:00',717.72,490,39862,62.85,'Molniya' from dual union all
select 983,'000983','1965-006A','Kosmos-53','DS-A1 No. 5',11756,'Reentered','1966-08-12 00:00:00','1965-03-03 00:00:00',98.39,221,1143,48.73,'Intermediate' from dual union all
select 9830,'009830','1977-010B','11S510','11S510',42138,'Reentered','1977-03-13 00:00:00','1977-02-18 00:00:00',90.74,204,418,62.84,'Intermediate' from dual union all
select 9831,'009831','1977-010C','BOZ','BOZ',42138,'Reentered','1977-03-02 00:00:00','1977-02-19 00:00:00',90.4,178,411,62.82,'Intermediate' from dual union all
select 9832,'009832','1977-010D','deb Blok-I','',42138,'Reentered','1977-02-27 00:00:00','1977-02-18 00:00:00',90.72,195,425,62.82,'Intermediate' from dual union all
select 9833,'009833','1977-011A','Kosmos-893','GVM DS-U2-IK',42139,'Reentered Attached','1984-10-06 00:00:00','1977-03-21 00:00:00',105.16,331,1673,74,'Intermediate' from dual union all
select 9834,'009834','1977-011B','deb Kosmos-893','',42139,'Reentered','1978-04-17 00:00:00','1977-03-17 00:00:00',104.68,327,1632,74,'Intermediate' from dual union all
select 9835,'009835','1973-086GD','deb Delta 98','',30688,'In Earth orbit','','1993-08-08 00:00:00',127.2,1436,2565,101.31,'Medium' from dual union all
select 9836,'009836','1976-031C','deb Kosmos-812','Tselina-O part',38983,'Reentered','1978-07-18 00:00:00','1977-02-16 00:00:00',94.11,445,506,74.05,'Intermediate' from dual union all
select 9837,'009837','1976-001J','deb Kosmos-787','Tselina-O part',38953,'Reentered','1977-08-27 00:00:00','1977-02-17 00:00:00',93.51,424,469,73.99,'Intermediate' from dual union all
select 9838,'009838','1976-001K','deb Kosmos-787','Tselina-O part',38953,'Reentered','1977-07-30 00:00:00','1977-02-16 00:00:00',93.43,420,465,74.01,'Intermediate' from dual union all
select 9839,'009839','1975-033D','deb Blok Kosmos','',36480,'Reentered','1977-06-21 00:00:00','1977-02-16 00:00:00',93.16,401,458,50.57,'Intermediate' from dual union all
select 984,'000984','1965-006B','S1','S1',11756,'Reentered','1966-01-11 00:00:00','1965-03-02 00:00:00',98.19,225,1120,48.74,'Intermediate' from dual union all
select 9840,'009840','1965-082UJ','deb Transtage 4','',11832,'Reentered','1978-05-21 00:00:00','1974-08-31 00:00:00',120.17,312,3065,31.02,'Medium' from dual union all
select 9841,'009841','1977-012A','Tansei-3','MS-T3',42140,'In Earth orbit','','1992-12-30 00:00:00',134.12,802,3801,65.75,'Medium' from dual union all
select 9842,'009842','1977-012B','M-3A','M-3A',42140,'Reentered','1979-03-21 00:00:00','1977-03-23 00:00:00',95.8,329,786,65.49,'Intermediate' from dual union all
select 9843,'009843','1977-012C','KM-A','KM-A',42140,'In Earth orbit','','1993-08-15 00:00:00',134.06,804,3794,65.75,'Medium' from dual union all
select 9844,'009844','1977-012D','deb M-3A','',42140,'Reentered','1978-05-15 00:00:00','1977-03-21 00:00:00',95.98,332,800,65,'Intermediate' from dual union all
select 9845,'009845','1977-009D','KDU','KDU',42137,'Reentered','1977-02-26 00:00:00','1977-02-22 00:00:00',89.37,166,321,72.86,'Intermediate' from dual union all
select 9846,'009846','1977-013A','Kosmos-894','Parus',42141,'In Earth orbit','','1992-12-28 00:00:00',104.85,964,1011,82.95,'Intermediate' from dual union all
select 9847,'009847','1977-009E','Blenda','Sunshade',42137,'Reentered','1977-02-24 00:00:00','1977-02-21 00:00:00',89.18,162,306,72.87,'Intermediate' from dual union all
select 9848,'009848','1977-013B','S3M','S3M',42141,'In Earth orbit','','1991-03-28 00:00:00',104.74,970,995,82.94,'Intermediate' from dual union all
select 9849,'009849','1977-009F','KDU part','KDU part',42137,'Reentered','1977-02-23 00:00:00','1977-02-22 00:00:00',89.39,165,324,72.86,'Intermediate' from dual union all
select 985,'000985','1965-006C','deb Kosmos-53','',11756,'Reentered','1965-03-05 00:00:00','1965-02-12 00:00:00',95.06,218,825,48.68,'Intermediate' from dual union all
select 9850,'009850','1977-010E','Blok-L','Blok-L',42138,'In Earth orbit','','1977-03-22 00:00:00',731.04,485,40522,62.87,'Molniya' from dual union all
select 9851,'009851','1977-009G','KDU part','KDU part',42137,'Reentered','1977-02-23 00:00:00','1977-02-22 00:00:00',89.03,163,291,72.85,'Intermediate' from dual union all
select 9852,'009852','1977-014A','Kiku 2','ETS-2',42142,'In Earth orbit','','1977-03-27 00:00:00',1436.14,35768,35806,.52,'Stationary' from dual union all
select 9853,'009853','1977-015A','Kosmos-895','Tselina-D',42143,'Reentered','1992-03-22 00:00:00','1977-03-28 00:00:00',97.12,609,633,81.19,'Intermediate' from dual union all
select 9854,'009854','1977-015B','8A92ME','8A92ME',42143,'Reentered','2000-02-14 00:00:00','1991-03-28 00:00:00',95.08,489,556,81.16,'Intermediate' from dual union all
select 9855,'009855','1977-007C','Transtage 23','Transtage 23',42135,'In Earth orbit','','1977-03-01 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 9856,'009856','1977-007D','DSP 9 Aperture Cover','',42135,'In Earth orbit','','1977-03-01 00:00:00',1431.67,35600,35800,1,'Drift GEO' from dual union all
select 9857,'009857','1977-016A','Kosmos-896','Zenit-6',42144,'Landed','1977-03-16 00:00:00','1977-03-05 00:00:00',88.48,193,205,72.88,'Intermediate' from dual union all
select 9858,'009858','1977-016B','11S510','11S510',42144,'Reentered','1977-03-04 00:00:00','1977-03-03 00:00:00',88.19,168,202,72.88,'Intermediate' from dual union all
select 9859,'009859','1977-014B','Star 37N S/N 14004','Star 37N',42142,'Reentered','1991-11-14 00:00:00','1977-04-06 00:00:00',628.36,205,35642,23.76,'GEO Transfer' from dual union all
select 986,'000986','1965-006D','deb Kosmos-53','',11756,'Reentered','1965-03-04 00:00:00','1965-02-08 00:00:00',96.32,220,945,48.78,'Intermediate' from dual union all
select 9860,'009860','1977-017A','Kosmos-897','Zenit-4MK',42145,'Landed','1977-03-23 00:00:00','1977-03-12 00:00:00',89.55,169,336,72.85,'Intermediate' from dual union all
select 9861,'009861','1977-017B','11S510','11S510',42145,'Reentered','1977-03-15 00:00:00','1977-03-12 00:00:00',89.11,165,296,72.85,'Intermediate' from dual union all
select 9862,'009862','1977-018A','Palapa A2','Palapa A2',42146,'In Earth orbit','','1979-12-26 00:00:00',1435.75,35751,35809,2.73,'Stationary' from dual union all
select 9863,'009863','1977-019A','OPS 4915','GAMBIT-3 Mission 4348',42147,'Reentered Attached','1977-05-26 00:00:00','1977-04-12 00:00:00',89.27,120,356,96.4,'Sun-Synch' from dual union all
select 9864,'009864','1977-018B','Star 37E S/N 40044','Star 37E',42146,'Reentered','1982-04-25 00:00:00','1998-09-30 00:00:00',319.41,292,17909,24.38,'Highly Ellip' from dual union all
select 9865,'009865','1977-016C','KDU','KDU',42144,'Reentered','1977-03-25 00:00:00','1977-03-16 00:00:00',89.42,173,318,72.88,'Intermediate' from dual union all
select 9866,'009866','1977-018C','Delta 129','Delta SSPU DSV-3P-4',42146,'Reentered','1977-12-17 00:00:00','1977-04-11 00:00:00',115.93,189,2807,28.7,'Intermediate' from dual union all
select 9867,'009867','1977-018D','Delta 129 clamp band','',42146,'Reentered','1977-10-13 00:00:00','1977-03-14 00:00:00',117.23,188,2925,28.7,'Intermediate' from dual union all
select 9868,'009868','1977-018E','Delta 129 clamp band','',42146,'Reentered','1978-02-15 00:00:00','1977-04-01 00:00:00',116.37,204,2831,28.69,'Intermediate' from dual union all
select 9869,'009869','1977-016D','Blenda','Sunshade',42144,'Reentered','1977-03-17 00:00:00','1977-03-16 00:00:00',88.39,158,232,72.89,'Intermediate' from dual union all
select 987,'000987','1965-007A','OSO 2','OSO B2',11757,'Reentered','1989-08-09 00:00:00','1965-03-10 00:00:00',96.44,549,628,32.85,'Intermediate' from dual union all
select 9870,'009870','1977-016E','KDU part','KDU part',42144,'Reentered','1977-03-17 00:00:00','1977-03-17 00:00:00',88.25,151,225,72.85,'Intermediate' from dual union all
select 9871,'009871','1977-020A','Kosmos-898','Zenit-2M',42148,'Landed','1977-03-30 00:00:00','1977-03-19 00:00:00',88.91,214,228,81.34,'Intermediate' from dual union all
select 9872,'009872','1977-020B','11S510','11S510',42148,'Reentered','1977-03-22 00:00:00','1977-03-20 00:00:00',88.47,199,199,81.35,'Intermediate' from dual union all
select 9873,'009873','1977-016F','KDU part','KDU part',42144,'Reentered','1977-03-24 00:00:00','1977-03-19 00:00:00',90.9,233,404,72.89,'Intermediate' from dual union all
select 9874,'009874','1977-016G','PTDU cover','',42144,'Reentered','1977-03-23 00:00:00','1977-03-18 00:00:00',91.14,234,427,72.93,'Intermediate' from dual union all
select 9875,'009875','1977-016H','PTDU cover','',42144,'Reentered','1977-03-21 00:00:00','1977-03-18 00:00:00',92.07,235,518,72.94,'Intermediate' from dual union all
select 9876,'009876','1977-017C','KDU','KDU',42145,'Reentered','1977-03-31 00:00:00','1977-03-23 00:00:00',90.1,166,393,72.85,'Intermediate' from dual union all
select 9877,'009877','1977-017D','Blenda','Sunshade',42145,'Reentered','1977-03-27 00:00:00','1977-03-22 00:00:00',89.97,169,377,72.87,'Intermediate' from dual union all
select 9878,'009878','1977-017E','KDU part','KDU part',42145,'Reentered','1977-03-26 00:00:00','1977-03-23 00:00:00',90.16,167,398,72.82,'Intermediate' from dual union all
select 9879,'009879','1977-017F','KDU part','KDU part',42145,'Reentered','1977-05-20 00:00:00','1977-03-25 00:00:00',89.88,160,376,72.86,'Intermediate' from dual union all
select 988,'000988','1965-007B','Altair 2 X-258C4','X-258C4',11757,'Reentered','1977-09-11 00:00:00','1965-03-17 00:00:00',96.54,553,633,32.87,'Intermediate' from dual union all
select 9880,'009880','1977-021A','Molniya-1','Molniya-1',42149,'In Earth orbit','','1977-05-04 00:00:00',717.44,462,39876,62.93,'Molniya' from dual union all
select 9881,'009881','1977-021B','11S510','11S510',42149,'Reentered','1977-04-19 00:00:00','1977-04-01 00:00:00',90.63,207,404,62.83,'Intermediate' from dual union all
select 9882,'009882','1977-021C','BOZ','BOZ',42149,'Reentered','1977-04-12 00:00:00','1977-03-31 00:00:00',90.5,189,410,62.83,'Intermediate' from dual union all
select 9883,'009883','1977-022A','Kosmos-899','Tselina-OM',42150,'Reentered','1980-10-19 00:00:00','1977-04-24 00:00:00',95.11,501,547,74.05,'Intermediate' from dual union all
select 9884,'009884','1977-022B','S3M','S3M',42150,'Reentered','1980-12-25 00:00:00','1977-04-28 00:00:00',94.98,493,542,74.06,'Intermediate' from dual union all
select 9885,'009885','1977-017G','PTDU cover','KDU part',42145,'Reentered','1977-03-27 00:00:00','1977-03-26 00:00:00',89.96,192,353,72.89,'Intermediate' from dual union all
select 9886,'009886','1977-020C','Nauka part','',42148,'Reentered','1977-03-30 00:00:00','1977-03-27 00:00:00',88.6,204,207,81.34,'Intermediate' from dual union all
select 9887,'009887','1977-020D','Kapsula Kosmosa-898','Nauka 25KS',42148,'Reentered','1977-04-03 00:00:00','1977-03-27 00:00:00',88.76,208,219,81.34,'Intermediate' from dual union all
select 9888,'009888','1976-105E','deb Kosmos-862','',39057,'Reentered','1994-06-12 00:00:00','1992-08-10 00:00:00',718.96,863,39550,64.41,'GEO Transfer' from dual union all
select 9889,'009889','1976-105F','deb Kosmos-862 baffle','Large deb RCS 2.5',39057,'Reentered','2014-05-29 00:00:00','1992-07-25 00:00:00',717.03,1058,39260,64.97,'GEO Transfer' from dual union all
select 989,'000989','1964-070E','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.53,181,223,51.1,'Intermediate' from dual union all
select 9890,'009890','1976-105G','deb Kosmos-862','',39057,'Lost','1979-04-10 00:00:00','1977-04-09 00:00:00',717.88,822,39538,63.16,'Molniya' from dual union all
select 9891,'009891','1976-105H','deb Kosmos-862','',39057,'Reentered','1999-07-26 00:00:00','1992-12-10 00:00:00',718.77,762,39642,64.28,'GEO Transfer' from dual union all
select 9892,'009892','1976-105J','Kosmos-862 large deb','RCS 2.5',39057,'Reentered','1995-02-13 00:00:00','1977-03-26 00:00:00',717.44,783,39555,63.1,'Molniya' from dual union all
select 9893,'009893','1976-105K','deb Kosmos-862','',39057,'Reentered','1994-06-10 00:00:00','1977-04-07 00:00:00',716.44,585,39704,62.95,'Molniya' from dual union all
select 9894,'009894','1976-105L','deb Kosmos-862','',39057,'Reentered','1993-05-22 00:00:00','1978-08-04 00:00:00',702.53,558,39042,63.87,'Molniya' from dual union all
select 9895,'009895','1976-105M','deb Kosmos-862','',39057,'Lost','1989-05-01 00:00:00','1979-04-09 00:00:00',718.71,1468,38933,65.85,'GEO Transfer' from dual union all
select 9896,'009896','1976-105N','deb Kosmos-862','',39057,'In Earth orbit','','1977-03-24 00:00:00',724.5,794,39892,63.14,'Molniya' from dual union all
select 9897,'009897','1976-124D','Kosmos-885 SS 2','ESO',39076,'Reentered','1977-11-22 00:00:00','1977-03-29 00:00:00',94.31,458,513,65.84,'Intermediate' from dual;
insert into SATELLITE
select 9898,'009898','1977-023A','Kosmos-900','AUOS-Z-R-O Oval',42151,'Reentered','1979-10-11 00:00:00','1977-04-29 00:00:00',94.34,455,519,82.95,'Intermediate' from dual union all
select 9899,'009899','1977-023B','S3M','S3M',42151,'Reentered','1979-09-06 00:00:00','1977-05-03 00:00:00',94.22,446,515,82.95,'Intermediate' from dual union all
select 99,'000099','60 Gamma 4','Transit 1B despin weight','Despin weight',6322,'Reentered','1979-07-17 00:00:00','1961-03-13 00:00:00',96.92,467,756,51.22,'Intermediate' from dual union all
select 990,'000990','1964-070F','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',88.38,190,199,51.3,'Intermediate' from dual union all
select 9900,'009900','1977-022C','deb Kosmos-899','Tselina-O part',42150,'Reentered','1978-04-11 00:00:00','1977-04-24 00:00:00',94.76,471,543,74.09,'Intermediate' from dual union all
select 9901,'009901','1976-124E','Kosmos-885 SS 3','ESO',39076,'Reentered','1977-11-11 00:00:00','1977-04-02 00:00:00',94.33,460,513,65.84,'Intermediate' from dual union all
select 9902,'009902','1976-105P','deb Kosmos-862','',39057,'Lost','1993-01-01 00:00:00','1993-01-04 00:00:00',727.15,1112,39704,64.56,'GEO Transfer' from dual union all
select 9903,'009903','1977-024A','Meteor','Meteor-M  No. 39',42152,'In Earth orbit','','1993-01-13 00:00:00',102.32,846,891,81.25,'Intermediate' from dual union all
select 9904,'009904','1977-024B','8A92ME','8A92ME',42152,'In Earth orbit','','1993-08-17 00:00:00',102.42,834,913,81.26,'Intermediate' from dual union all
select 9905,'009905','1977-025A','Kosmos-901','DS-P1-I No. 18',42153,'Reentered','1978-06-28 00:00:00','1977-05-09 00:00:00',95.32,267,802,70.99,'Intermediate' from dual union all
select 9906,'009906','1977-025B','S1M','S1M',42153,'Reentered','1978-03-25 00:00:00','1977-05-06 00:00:00',95.08,274,770,70.98,'Intermediate' from dual union all
select 9907,'009907','1977-024C','deb Meteor','',42152,'In Earth orbit','','1993-08-18 00:00:00',102.5,858,896,82.94,'Intermediate' from dual union all
select 9908,'009908','1977-026A','Kosmos-902','Zenit-4MK',42154,'Landed','1977-04-20 00:00:00','1977-04-11 00:00:00',88.81,166,265,81.39,'Intermediate' from dual union all
select 9909,'009909','1977-026B','11S510','11S510',42154,'Reentered','1977-04-09 00:00:00','1977-04-08 00:00:00',88.3,157,224,81.39,'Intermediate' from dual union all
select 991,'000991','1964-070G','deb Kosmos-50','',10356,'Reentered','1964-11-08 00:00:00','1964-11-05 00:00:00',88.7,166,255,51.2,'Intermediate' from dual union all
select 9910,'009910','1977-026C','Sensor cover','Zenit-4MK sensor cover',42154,'Reentered','1977-04-10 00:00:00','1977-04-09 00:00:00',88.24,163,212,81.4,'Intermediate' from dual union all
select 9911,'009911','1977-027A','Kosmos-903','Oko',42155,'Reentered','2014-08-04 00:00:00','1977-05-11 00:00:00',718,646,39720,62.84,'Molniya' from dual union all
select 9912,'009912','1977-027B','11S510','11S510',42155,'Reentered','1977-06-06 00:00:00','1977-04-18 00:00:00',92.18,212,551,62.82,'Intermediate' from dual union all
select 9913,'009913','1977-027C','BOZ','BOZ',42155,'Reentered','1977-05-01 00:00:00','1977-04-18 00:00:00',91.56,171,531,62.85,'Intermediate' from dual union all
select 9914,'009914','1976-124F','Kosmos-885 SS 4','ESO',39076,'Reentered','1977-12-09 00:00:00','1977-04-24 00:00:00',94.34,461,513,65.85,'Intermediate' from dual union all
select 9915,'009915','1976-124G','Kosmos-885 SS 5','ESO',39076,'Reentered','1977-11-29 00:00:00','1977-05-13 00:00:00',94.26,458,508,65.79,'Intermediate' from dual union all
select 9916,'009916','1976-124H','Kosmos-885 SS 6','ESO',39076,'Reentered','1977-11-16 00:00:00','1977-04-14 00:00:00',94.26,451,515,65.84,'Intermediate' from dual union all
select 9917,'009917','1976-124J','Kosmos-885 SS 7','ESO',39076,'Reentered','1977-11-05 00:00:00','1977-04-15 00:00:00',94.19,445,514,65.84,'Intermediate' from dual union all
select 9918,'009918','1976-124K','Kosmos-885 SS 8','ESO',39076,'Reentered','1977-11-20 00:00:00','1977-04-24 00:00:00',94.24,452,512,65.83,'Intermediate' from dual union all
select 9919,'009919','1976-124L','Kosmos-885 SS 9','ESO',39076,'Reentered','1977-11-10 00:00:00','1977-04-13 00:00:00',94.22,447,514,65.85,'Intermediate' from dual union all
select 992,'000992','1964-070H','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',88.23,148,226,51.1,'Intermediate' from dual union all
select 9920,'009920','1976-124M','Kosmos-885 SS 10','ESO',39076,'Reentered','1977-12-22 00:00:00','1977-04-15 00:00:00',94.49,473,515,65.84,'Intermediate' from dual union all
select 9921,'009921','1977-027D','Blok-L','Blok-L',42155,'In Earth orbit','','1977-05-19 00:00:00',723.91,655,40002,62.86,'Molniya' from dual union all
select 9922,'009922','1976-124N','Kosmos-885 SS 11','ESO',39076,'Reentered','1977-11-16 00:00:00','1977-04-30 00:00:00',94.22,451,511,65.89,'Intermediate' from dual union all
select 9923,'009923','1976-124P','Kosmos-885 SS 12','ESO',39076,'Reentered','1977-12-09 00:00:00','1977-04-16 00:00:00',94.45,470,514,65.83,'Intermediate' from dual union all
select 9924,'009924','1976-124Q','Kosmos-885 SS 13','ESO',39076,'Reentered','1977-12-14 00:00:00','1977-04-16 00:00:00',94.43,467,515,65.84,'Intermediate' from dual union all
select 9925,'009925','1976-124R','Kosmos-885 SS 14','ESO',39076,'Reentered','1977-11-22 00:00:00','1977-05-24 00:00:00',94.18,453,505,65.81,'Intermediate' from dual union all
select 9926,'009926','1976-124S','Kosmos-885 SS 15','ESO',39076,'Reentered','1977-12-01 00:00:00','1977-04-16 00:00:00',94.31,456,515,65.87,'Intermediate' from dual union all
select 9927,'009927','1977-021D','Blok-L','Blok-L',42149,'In Earth orbit','','1977-04-28 00:00:00',732.88,482,40614,62.85,'Molniya' from dual union all
select 9928,'009928','1977-026D','Blenda','Sunshade',42154,'Reentered','1977-04-20 00:00:00','1977-04-19 00:00:00',88.3,160,221,81.39,'Intermediate' from dual union all
select 9929,'009929','1977-026E','KDU','KDU',42154,'Reentered','1977-04-23 00:00:00','1977-04-20 00:00:00',88.7,167,254,81.39,'Intermediate' from dual union all
select 993,'000993','1964-070J','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',89.13,185,278,51,'Intermediate' from dual union all
select 9930,'009930','1977-028A','Kosmos-904','Zenit-2M',42156,'Landed','1977-05-04 00:00:00','1977-04-22 00:00:00',89.75,201,324,71.37,'Intermediate' from dual union all
select 9931,'009931','1977-029A','Geos 1','Geos 1',42157,'In Earth orbit','','1977-05-22 00:00:00',718.23,2083,38294,26.47,'GEO Transfer' from dual union all
select 9932,'009932','1977-029B','Delta 130','Delta SSPU DSV-3P-4',42157,'Reentered','1977-04-20 00:00:00','1977-04-20 00:00:00',88.38,157,232,28.74,'Intermediate' from dual union all
select 9933,'009933','1977-029C','Star 37E S/N 40048','Star 37E',42157,'Reentered','1989-12-17 00:00:00','1977-06-21 00:00:00',226.71,239,11636,26.06,'Medium' from dual union all
select 9934,'009934','1977-029D','Delta 130 clamp band','',42157,'Reentered','1977-04-20 00:00:00','1977-04-20 00:00:00',122,163,3378,28.34,'Medium' from dual union all
select 9935,'009935','1977-028B','11S510','11S510',42156,'Reentered','1977-04-29 00:00:00','1977-04-22 00:00:00',89.55,195,310,71.39,'Intermediate' from dual union all
select 9936,'009936','1977-028C','Sensor cover','Zenit-2M sensor cover',42156,'Reentered','1977-04-23 00:00:00','1977-04-20 00:00:00',88.91,208,234,71.3,'Intermediate' from dual union all
select 9937,'009937','1977-030A','Kosmos-905','Yantar''-2K No. 6',42158,'Landed','1977-05-26 00:00:00','1977-05-03 00:00:00',90.34,168,415,67.12,'Intermediate' from dual union all
select 9938,'009938','1977-031A','Kosmos-906','ER763-4',42159,'Reentered Attached','1980-03-23 00:00:00','1977-05-27 00:00:00',94.36,463,513,50.65,'Intermediate' from dual union all
select 9939,'009939','1977-030B','11S510','11S510',42158,'Reentered','1977-05-01 00:00:00','1977-04-29 00:00:00',88.9,163,278,67.12,'Intermediate' from dual union all
select 994,'000994','1964-070K','deb Kosmos-50','',10356,'Reentered','1964-11-13 00:00:00','1964-11-05 00:00:00',87.93,130,214,51.1,'Intermediate' from dual union all
select 9940,'009940','1977-030C','Kosmos-905 part','Feniks part',42158,'Reentered','1977-05-14 00:00:00','1977-05-03 00:00:00',89.39,176,313,67.12,'Intermediate' from dual union all
select 9941,'009941','1977-032A','Molniya-3','Molniya-3 No. 19',42160,'In Earth orbit','','1977-05-31 00:00:00',717.66,488,39861,62.86,'Molniya' from dual union all
select 9942,'009942','1977-032B','11S510','11S510',42160,'Reentered','1977-05-15 00:00:00','1977-05-08 00:00:00',89.55,203,302,62.78,'Intermediate' from dual union all
select 9943,'009943','1977-032C','BOZ','BOZ',42160,'Reentered','1977-05-15 00:00:00','1977-05-05 00:00:00',90.11,187,372,62.84,'Intermediate' from dual union all
select 9944,'009944','1977-033A','Kosmos-907','Zenit-4MK',42161,'Landed','1977-05-16 00:00:00','1977-05-08 00:00:00',89.26,167,308,62.8,'Intermediate' from dual union all
select 9945,'009945','1977-033B','11S510','11S510',42161,'Reentered','1977-05-13 00:00:00','1977-05-07 00:00:00',89.61,176,335,62.79,'Intermediate' from dual union all
select 9946,'009946','1977-033C','Sensor cover','Zenit-4MK sensor cover',42161,'Reentered','1977-05-06 00:00:00','1977-05-05 00:00:00',89.96,178,366,62.83,'Intermediate' from dual union all
select 9947,'009947','1977-033D','Sensor cover','Zenit-4MK sensor cover',42161,'Reentered','1977-05-06 00:00:00','1977-05-05 00:00:00',89.46,176,320,62.73,'Intermediate' from dual union all
select 9948,'009948','1975-004HC','deb Delta 107','',36451,'Reentered','1985-03-02 00:00:00','1985-02-28 00:00:00',91.44,327,364,101.08,'Retrograde' from dual union all
select 9949,'009949','1975-004HD','deb Delta 107','',36451,'Reentered','1981-10-26 00:00:00','1976-06-25 00:00:00',97.9,521,796,97.15,'Sun-Synch' from dual union all
select 995,'000995','1964-070L','deb Kosmos-50','',10356,'Reentered','1964-11-11 00:00:00','1964-11-05 00:00:00',89.34,186,298,51.3,'Intermediate' from dual union all
select 9950,'009950','1975-004HE','deb Delta 107','',36451,'Reentered','1979-07-14 00:00:00','1977-05-11 00:00:00',101.1,668,953,96.27,'Sun-Synch' from dual union all
select 9951,'009951','1976-126AR','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-06 00:00:00',111.32,565,2010,65.81,'Intermediate' from dual union all
select 9952,'009952','1976-126AS','deb Kosmos-886','',39078,'Reentered','1980-10-11 00:00:00','1977-05-12 00:00:00',113.71,494,2300,65.84,'Intermediate' from dual union all
select 9953,'009953','1976-126AT','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-05 00:00:00',112.05,577,2065,65.84,'Intermediate' from dual union all
select 9954,'009954','1976-126AU','deb Kosmos-397','[UNID077] RCS 0.01',39078,'In Earth orbit','','1993-08-04 00:00:00',111.46,562,2026,65.69,'Intermediate' from dual union all
select 9955,'009955','1976-126AV','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-08 00:00:00',114.27,585,2260,65.84,'Intermediate' from dual union all
select 9956,'009956','1976-126AW','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-03 00:00:00',116.61,784,2273,65.8,'Intermediate' from dual union all
select 9957,'009957','1976-126AX','deb Kosmos-886','',39078,'Reentered','2014-08-03 00:00:00','1993-08-07 00:00:00',108.55,577,1743,65.8,'Intermediate' from dual union all
select 9958,'009958','1976-126AY','deb Kosmos-886','',39078,'In Earth orbit','','1993-07-31 00:00:00',115.04,730,2185,65.87,'Intermediate' from dual union all
select 9959,'009959','1976-126AZ','deb Kosmos-886','',39078,'Reentered','1982-06-20 00:00:00','1977-05-10 00:00:00',114.26,531,2313,65.82,'Intermediate' from dual union all
select 996,'000996','1964-070M','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',89.43,192,301,51.1,'Intermediate' from dual union all
select 9960,'009960','1976-126BA','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-06 00:00:00',113.54,667,2111,65.87,'Intermediate' from dual union all
select 9961,'009961','1976-126BB','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-05 00:00:00',109.43,549,1852,65.83,'Intermediate' from dual union all
select 9962,'009962','1976-126BC','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-07 00:00:00',109.46,552,1852,65.79,'Intermediate' from dual union all
select 9963,'009963','1976-126BD','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-08 00:00:00',111.03,547,2002,65.82,'Intermediate' from dual union all
select 9964,'009964','1976-126BE','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-07 00:00:00',112.21,609,2048,65.82,'Intermediate' from dual union all
select 9965,'009965','1976-126BF','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-08 00:00:00',118.85,914,2345,65.8,'Intermediate' from dual union all
select 9966,'009966','1976-126BG','deb Kosmos-886','',39078,'In Earth orbit','','1993-08-07 00:00:00',110.44,591,1903,65.8,'Intermediate' from dual union all
select 9967,'009967','1976-126BH','deb Kosmos-886','',39078,'Reentered','1991-03-05 00:00:00','1991-03-05 00:00:00',89.71,249,271,65.71,'Intermediate' from dual union all
select 9968,'009968','61 Omicron 255','deb r/b Ablestar','',7273,'Reentered','2001-03-12 00:00:00','1993-08-08 00:00:00',99.02,699,725,66.52,'Intermediate' from dual union all
select 9969,'009969','61 Omicron 256','deb r/b Ablestar','',7273,'Reentered','1978-04-05 00:00:00','1977-05-12 00:00:00',103.65,815,1047,65.2,'Intermediate' from dual union all
select 997,'000997','1964-070N','deb Kosmos-50','',10356,'Reentered','1964-11-14 00:00:00','1964-11-05 00:00:00',88.39,163,227,51.4,'Intermediate' from dual union all
select 9970,'009970','1972-058HX','deb Delta 89','',27990,'Reentered','1980-10-10 00:00:00','1977-05-12 00:00:00',100.74,636,952,98.67,'Sun-Synch' from dual union all
select 9971,'009971','1972-058HY','deb Delta 89','',27990,'Reentered','1989-11-21 00:00:00','1989-11-14 00:00:00',91.03,315,336,98.25,'Sun-Synch' from dual union all
select 9972,'009972','1972-058HZ','deb Delta 89','',27990,'Reentered','1979-06-29 00:00:00','1979-02-21 00:00:00',97.77,614,690,98.91,'Sun-Synch' from dual union all
select 9973,'009973','1968-097DV','deb Kosmos-252','',16824,'In Earth orbit','','1993-08-08 00:00:00',108.85,592,1756,62.42,'Intermediate' from dual union all
select 9974,'009974','1968-097DW','deb Kosmos-252','',16824,'Reentered','1998-12-30 00:00:00','1993-08-23 00:00:00',97.73,459,842,62.18,'Intermediate' from dual union all
select 9975,'009975','1973-086GE','deb Delta 98','',30688,'In Earth orbit','','1993-08-01 00:00:00',116.38,1442,1594,102.54,'Sun-Synch' from dual union all
select 9976,'009976','1973-086GF','deb Delta 98','',30688,'In Earth orbit','','1993-08-01 00:00:00',116.73,1500,1568,102.41,'Sun-Synch' from dual union all
select 9977,'009977','1973-086GG','deb Delta 98','',30688,'In Earth orbit','','1993-08-08 00:00:00',115.4,1413,1535,101.76,'Sun-Synch' from dual union all
select 9978,'009978','1973-086GH','deb Delta 98','',30688,'In Earth orbit','','1993-08-02 00:00:00',116.93,1501,1585,102.52,'Sun-Synch' from dual union all
select 9979,'009979','1973-086GJ','deb Delta 98','',30688,'In Earth orbit','','1993-07-29 00:00:00',125.57,1417,2440,101.98,'Medium' from dual union all
select 998,'000998','1964-070P','deb Kosmos-50','',10356,'Reentered','1964-11-10 00:00:00','1964-11-05 00:00:00',89.67,187,330,51.5,'Intermediate' from dual union all
select 9980,'009980','1977-032D','Blok-L','Blok-L',42160,'Reentered','2017-04-22 00:00:00','1997-12-14 00:00:00',728.88,167,40734,62.05,'Molniya' from dual union all
select 9981,'009981','1977-012E','Tansei 3 despin weight','',42140,'In Earth orbit','','1993-08-12 00:00:00',133.28,798,3732,65.21,'Medium' from dual union all
select 9982,'009982','1977-012F','Tansei 3 despin weight','',42140,'In Earth orbit','','1993-08-16 00:00:00',133.51,777,3772,65.89,'Medium' from dual union all
select 9983,'009983','1977-012G','deb Tansei 3','',42140,'In Earth orbit','','1993-08-13 00:00:00',134.12,809,3794,65.64,'Medium' from dual union all
select 9984,'009984','1974-089EB','deb Delta 104','',33692,'In Earth orbit','','1993-07-30 00:00:00',116.11,1431,1581,102.3,'Sun-Synch' from dual union all
select 9985,'009985','1974-089EC','deb Delta 104','',33692,'In Earth orbit','','1993-08-04 00:00:00',115.36,1435,1509,102.1,'Sun-Synch' from dual union all
select 9986,'009986','1974-089ED','deb Delta 104','',33692,'In Earth orbit','','1993-08-03 00:00:00',112.39,908,1765,103.5,'Sun-Synch' from dual union all
select 9987,'009987','1974-089EE','deb Delta 104','',33692,'In Earth orbit','','1993-08-05 00:00:00',115.64,1430,1539,101.47,'Sun-Synch' from dual;
insert into SATELLITE
select 9988,'009988','1974-089EF','deb Delta 104','',33692,'In Earth orbit','','1993-07-30 00:00:00',106.61,815,1325,103.11,'Sun-Synch' from dual union all
select 9989,'009989','1974-089EG','deb Delta 104','',33692,'In Earth orbit','','1993-08-06 00:00:00',117.58,1431,1714,102.83,'Sun-Synch' from dual union all
select 999,'000999','1964-070Q','deb Kosmos-50','',10356,'Reentered','1964-11-12 00:00:00','1964-11-05 00:00:00',88.32,171,212,51.4,'Intermediate' from dual union all
select 9990,'009990','1976-031D','deb Kosmos-812','Tselina-O part',38983,'Reentered','1978-02-27 00:00:00','1977-05-16 00:00:00',94.31,468,503,73.98,'Intermediate' from dual union all
select 9991,'009991','1976-115D','deb Kosmos-870','Tselina-O part',39067,'Reentered','1978-04-30 00:00:00','1977-05-15 00:00:00',94.77,485,530,73.94,'Intermediate' from dual union all
select 9992,'009992','1976-115E','deb Kosmos-870','Tselina-O part',39067,'Reentered','1978-06-11 00:00:00','1977-05-12 00:00:00',94.8,487,531,73.97,'Intermediate' from dual union all
select 9993,'009993','1976-115F','deb Kosmos-870','Tselina-O part',39067,'Reentered','1980-03-07 00:00:00','1977-05-18 00:00:00',94.82,490,530,74.03,'Intermediate' from dual union all
select 9994,'009994','1975-109J','deb Kosmos-781','Tselina-O part',36556,'Reentered','1978-02-11 00:00:00','1977-05-11 00:00:00',94.2,460,500,74.02,'Intermediate' from dual union all
select 9995,'009995','1975-109K','Unknown Strela-2M debris','[UNID078] RCS 0.0',36556,'Reentered','1983-02-04 00:00:00','1977-05-10 00:00:00',97.41,356,914,74.01,'Intermediate' from dual union all
select 9996,'009996','1976-038L','deb USN satellite','',38990,'In Earth orbit','','1997-11-22 00:00:00',106.27,555,1553,63.4,'Intermediate' from dual union all
select 9997,'009997','1970-025NP','deb Agena D','',21502,'Reentered','1978-01-04 00:00:00','1977-05-12 00:00:00',96.15,424,725,99.99,'Sun-Synch' from dual union all
select 9998,'009998','1974-033F','SVM-5','SVM-5',33636,'In Earth orbit','','1974-06-30 00:00:00',146.39,363,5283,82.43,'Medium' from dual union all
select 9999,'009999','1976-124T','Kosmos-885 SS 16','ESO',39076,'Reentered','1977-10-17 00:00:00','1977-05-12 00:00:00',93.41,405,478,65.86,'Intermediate' from dual;


--------------------------------------------------------------------------------
-- SATELLITE_ORG
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing SATELLITE_ORG...');

CREATE TABLE "SATELLITE_ORG" 
 (	"SATELLITE_ID" NUMBER, 
	"OWNER_OPERATOR_ORG_CODE" VARCHAR2(4000), 
	 CONSTRAINT "SATELLITE_ORG_PK" PRIMARY KEY ("SATELLITE_ID", "OWNER_OPERATOR_ORG_CODE")
USING INDEX  ENABLE, 
	 CONSTRAINT "SATELLITE_ORG_SAT_FK" FOREIGN KEY ("SATELLITE_ID")
	  REFERENCES "SATELLITE" ("SATELLITE_ID") ENABLE, 
	 CONSTRAINT "SATELLITE_ORG_ORG_FK" FOREIGN KEY ("OWNER_OPERATOR_ORG_CODE")
	  REFERENCES "ORGANIZATION" ("ORG_CODE") ENABLE
 ) ;

insert into SATELLITE_ORG
select 1,'RVSN' from dual union all
select 10,'DOD' from dual union all
select 10,'USA' from dual union all
select 100,'USA' from dual union all
select 100,'USAF' from dual union all
select 1000,'USAF' from dual union all
select 10000,'DOD' from dual union all
select 10000,'USAF' from dual union all
select 10001,'DOD' from dual union all
select 10001,'USAF' from dual union all
select 10002,'USAF' from dual union all
select 10003,'USAF' from dual union all
select 10004,'RVSN' from dual union all
select 10005,'RVSN' from dual union all
select 10006,'RVSN' from dual union all
select 10007,'RVSN' from dual union all
select 10008,'RVSN' from dual union all
select 10009,'RVSN' from dual union all
select 1001,'USAF' from dual union all
select 10010,'PVO' from dual union all
select 10011,'RVSN' from dual union all
select 10012,'NRO' from dual union all
select 10013,'RVSN' from dual union all
select 10014,'PVO' from dual union all
select 10015,'RVSN' from dual union all
select 10016,'NRO' from dual union all
select 10017,'USAF' from dual union all
select 10018,'PVO' from dual union all
select 10019,'VTS' from dual union all
select 1002,'USAF' from dual union all
select 10020,'RVSN' from dual union all
select 10021,'MVD' from dual union all
select 10022,'RVSN' from dual union all
select 10023,'RVSN' from dual union all
select 10024,'INTEL' from dual union all
select 10025,'NASA' from dual union all
select 10026,'MVD' from dual union all
select 10027,'RVSN' from dual union all
select 10028,'PVO' from dual union all
select 10029,'RVSN' from dual union all
select 1003,'MO' from dual union all
select 10030,'RVSN' from dual union all
select 10031,'RVSN' from dual union all
select 10032,'RVSN' from dual union all
select 10033,'USAF' from dual union all
select 10034,'USAF' from dual union all
select 10035,'RVSN' from dual union all
select 10036,'RVSN' from dual union all
select 10037,'USAF' from dual union all
select 10038,'RVSN' from dual union all
select 10039,'RVSN' from dual union all
select 1004,'MO' from dual union all
select 10040,'VTS' from dual union all
select 10041,'RVSN' from dual union all
select 10042,'VTS' from dual union all
select 10043,'VTS' from dual union all
select 10044,'VTS' from dual union all
select 10045,'RVSN' from dual union all
select 10046,'TSSKB' from dual union all
select 10047,'VMF' from dual union all
select 10048,'VMF' from dual union all
select 10049,'VMF' from dual union all
select 1005,'MO' from dual union all
select 10050,'VMF' from dual union all
select 10051,'VMF' from dual union all
select 10052,'VMF' from dual union all
select 10053,'VMF' from dual union all
select 10054,'VMF' from dual union all
select 10055,'VMF' from dual union all
select 10056,'VMF' from dual union all
select 10057,'VMF' from dual union all
select 10058,'VMF' from dual union all
select 10059,'PVO' from dual union all
select 1006,'MO' from dual union all
select 10060,'RVSN' from dual union all
select 10061,'NOAA' from dual union all
select 10062,'NASA' from dual union all
select 10063,'NASA' from dual union all
select 10064,'CNES' from dual union all
select 10065,'PVO' from dual union all
select 10066,'RVSN' from dual union all
select 10067,'NASA' from dual union all
select 10068,'RVSN' from dual union all
select 10069,'RVSN' from dual union all
select 1007,'MO' from dual union all
select 10070,'PVO' from dual union all
select 10071,'RVSN' from dual union all
select 10072,'PVO' from dual union all
select 10073,'RVSN' from dual union all
select 10074,'RVSN' from dual union all
select 10075,'NASA' from dual union all
select 10076,'VMF' from dual union all
select 10077,'VMF' from dual union all
select 10078,'VMF' from dual union all
select 10079,'VMF' from dual union all
select 1008,'MO' from dual union all
select 10080,'VMF' from dual union all
select 10081,'VMF' from dual union all
select 10082,'VMF' from dual union all
select 10083,'VMF' from dual;
insert into SATELLITE_ORG
select 10084,'RVSN' from dual union all
select 10085,'USAF' from dual union all
select 10086,'RVSN' from dual union all
select 10087,'RVSN' from dual union all
select 10088,'RVSN' from dual union all
select 10089,'RVSN' from dual union all
select 1009,'MO' from dual union all
select 10090,'RVSN' from dual union all
select 10091,'USAF' from dual union all
select 10091,'USN' from dual union all
select 10092,'RVSN' from dual union all
select 10093,'RVSN' from dual union all
select 10094,'RVSN' from dual union all
select 10095,'YUZH' from dual union all
select 10096,'RVSN' from dual union all
select 10097,'VMF' from dual union all
select 10098,'VMF' from dual union all
select 10099,'VMF' from dual union all
select 101,'NASA' from dual union all
select 1010,'MO' from dual union all
select 10100,'VMF' from dual union all
select 10101,'VMF' from dual union all
select 10102,'VMF' from dual union all
select 10103,'VMF' from dual union all
select 10104,'VMF' from dual union all
select 10105,'VMF' from dual union all
select 10106,'VMF' from dual union all
select 10107,'VMF' from dual union all
select 10108,'VMF' from dual union all
select 10109,'VMF' from dual union all
select 1011,'MO' from dual union all
select 10110,'VMF' from dual union all
select 10111,'NRO' from dual union all
select 10112,'USAF' from dual union all
select 10113,'MVD' from dual union all
select 10114,'RVSN' from dual union all
select 10115,'RVSN' from dual union all
select 10116,'RVSN' from dual union all
select 10117,'RVSN' from dual union all
select 10118,'RVSN' from dual union all
select 10119,'RVSN' from dual union all
select 1012,'MO' from dual union all
select 10120,'RVSN' from dual union all
select 10121,'RVSN' from dual union all
select 10122,'VMF' from dual union all
select 10123,'VMF' from dual union all
select 10124,'VMF' from dual union all
select 10125,'VMF' from dual union all
select 10126,'VMF' from dual union all
select 10127,'RVSN' from dual union all
select 10128,'RVSN' from dual union all
select 10129,'VS' from dual union all
select 1013,'MO' from dual union all
select 10130,'RVSN' from dual union all
select 10131,'VS' from dual union all
select 10132,'VS' from dual union all
select 10133,'RVSN' from dual union all
select 10134,'VS' from dual union all
select 10135,'RVSN' from dual union all
select 10136,'NASA' from dual union all
select 10137,'MOM' from dual union all
select 10138,'RVSN' from dual union all
select 10139,'RVSN' from dual union all
select 1014,'MO' from dual union all
select 10140,'RVSN' from dual union all
select 10141,'VMF' from dual union all
select 10142,'RVSN' from dual union all
select 10143,'NASDA' from dual union all
select 10144,'NASA' from dual union all
select 10145,'NASA' from dual union all
select 10146,'KBSAL' from dual union all
select 10147,'RVSN' from dual union all
select 10148,'KBSAL' from dual union all
select 10149,'PVO' from dual union all
select 1015,'MO' from dual union all
select 10150,'PVO' from dual union all
select 10151,'RVSN' from dual union all
select 10152,'RVSN' from dual union all
select 10153,'RVSN' from dual union all
select 10154,'RVSN' from dual union all
select 10155,'RVSN' from dual union all
select 10156,'NASA' from dual union all
select 10157,'PVO' from dual union all
select 10158,'RVSN' from dual union all
select 10159,'RVSN' from dual union all
select 1016,'MO' from dual union all
select 10160,'RVSN' from dual union all
select 10161,'RVSN' from dual union all
select 10162,'RVSN' from dual union all
select 10163,'RVSN' from dual union all
select 10164,'RVSN' from dual union all
select 10165,'RVSN' from dual union all
select 10166,'RVSN' from dual union all
select 10167,'RVSN' from dual union all
select 10168,'RVSN' from dual union all
select 10169,'RVSN' from dual union all
select 1017,'MO' from dual union all
select 10170,'RVSN' from dual union all
select 10171,'RVSN' from dual union all
select 10172,'IMBP' from dual;
insert into SATELLITE_ORG
select 10172,'TSSKB' from dual union all
select 10173,'RVSN' from dual union all
select 10174,'RVSN' from dual union all
select 10175,'RVSN' from dual union all
select 10176,'NASA' from dual union all
select 10177,'NASA' from dual union all
select 10178,'NASA' from dual union all
select 10179,'NASA' from dual union all
select 1018,'MO' from dual union all
select 10180,'NASA' from dual union all
select 10181,'NASA' from dual union all
select 10182,'NASA' from dual union all
select 10183,'NASA' from dual union all
select 10184,'NASA' from dual union all
select 10185,'NASA' from dual union all
select 10186,'NASA' from dual union all
select 10187,'NASA' from dual union all
select 10188,'NASA' from dual union all
select 10189,'NASA' from dual union all
select 1019,'MO' from dual union all
select 10190,'NASA' from dual union all
select 10191,'NASA' from dual union all
select 10192,'NASA' from dual union all
select 10193,'NASA' from dual union all
select 10194,'NASA' from dual union all
select 10195,'RVSN' from dual union all
select 10196,'NASA' from dual union all
select 10197,'NASA' from dual union all
select 10198,'NASA' from dual union all
select 10199,'NASA' from dual union all
select 102,'CIA' from dual union all
select 102,'USAF' from dual union all
select 1020,'MO' from dual union all
select 10200,'NASA' from dual union all
select 10201,'NASA' from dual union all
select 10202,'NASA' from dual union all
select 10203,'NASA' from dual union all
select 10204,'NASA' from dual union all
select 10205,'NASA' from dual union all
select 10206,'NASA' from dual union all
select 10207,'NASA' from dual union all
select 10208,'NASA' from dual union all
select 10209,'NASA' from dual union all
select 1021,'MO' from dual union all
select 10210,'NASA' from dual union all
select 10211,'NASA' from dual union all
select 10212,'NASA' from dual union all
select 10213,'NASA' from dual union all
select 10214,'NASA' from dual union all
select 10215,'NASA' from dual union all
select 10216,'NASA' from dual union all
select 10217,'NASA' from dual union all
select 10218,'NASA' from dual union all
select 10219,'NASA' from dual union all
select 1022,'MO' from dual union all
select 10220,'NASA' from dual union all
select 10221,'NASA' from dual union all
select 10222,'NASA' from dual union all
select 10223,'NASA' from dual union all
select 10224,'NASA' from dual union all
select 10225,'NASA' from dual union all
select 10226,'NASA' from dual union all
select 10227,'NASA' from dual union all
select 10228,'NASA' from dual union all
select 10229,'NASA' from dual union all
select 1023,'MO' from dual union all
select 10230,'NASA' from dual union all
select 10231,'NASA' from dual union all
select 10232,'NASA' from dual union all
select 10233,'NASA' from dual union all
select 10234,'NASA' from dual union all
select 10235,'NASA' from dual union all
select 10236,'NASA' from dual union all
select 10237,'PVO' from dual union all
select 10238,'PVO' from dual union all
select 10239,'NASA' from dual union all
select 1024,'MO' from dual union all
select 10240,'NASA' from dual union all
select 10241,'NASA' from dual union all
select 10242,'NASA' from dual union all
select 10243,'NASA' from dual union all
select 10244,'NASA' from dual union all
select 10245,'NASA' from dual union all
select 10246,'NASA' from dual union all
select 10247,'NASA' from dual union all
select 10248,'NASA' from dual union all
select 10249,'NASA' from dual union all
select 1025,'MO' from dual union all
select 10250,'NASA' from dual union all
select 10251,'NASA' from dual union all
select 10252,'NASA' from dual union all
select 10253,'NASA' from dual union all
select 10254,'NASA' from dual union all
select 10255,'NASA' from dual union all
select 10256,'NASA' from dual union all
select 10257,'NASA' from dual union all
select 10258,'NASA' from dual union all
select 10259,'NASA' from dual union all
select 1026,'MO' from dual union all
select 10260,'NASA' from dual;
insert into SATELLITE_ORG
select 10261,'NASA' from dual union all
select 10262,'NASA' from dual union all
select 10263,'NASA' from dual union all
select 10264,'NASA' from dual union all
select 10265,'NASA' from dual union all
select 10266,'NASA' from dual union all
select 10267,'NASA' from dual union all
select 10268,'NASA' from dual union all
select 10269,'NASA' from dual union all
select 1027,'MO' from dual union all
select 10270,'NASA' from dual union all
select 10271,'JPL' from dual union all
select 10272,'NASA' from dual union all
select 10273,'NASA' from dual union all
select 10274,'RVSN' from dual union all
select 10275,'RVSN' from dual union all
select 10276,'RVSN' from dual union all
select 10277,'RVSN' from dual union all
select 10278,'VMF' from dual union all
select 10279,'RVSN' from dual union all
select 1028,'MO' from dual union all
select 10280,'RVSN' from dual union all
select 10281,'RVSN' from dual union all
select 10282,'RVSN' from dual union all
select 10283,'RVSN' from dual union all
select 10284,'RVSN' from dual union all
select 10285,'RVSN' from dual union all
select 10286,'RVSN' from dual union all
select 10287,'RVSN' from dual union all
select 10288,'RVSN' from dual union all
select 10289,'RVSN' from dual union all
select 1029,'MO' from dual union all
select 10290,'RVSN' from dual union all
select 10291,'RVSN' from dual union all
select 10292,'RVSN' from dual union all
select 10293,'RVSN' from dual union all
select 10294,'CNR' from dual union all
select 10295,'NASA' from dual union all
select 10296,'NASA' from dual union all
select 10297,'NASA' from dual union all
select 10298,'NASA' from dual union all
select 10299,'RVSN' from dual union all
select 103,'OKB1' from dual union all
select 1030,'MO' from dual union all
select 10300,'RVSN' from dual union all
select 10301,'RVSN' from dual union all
select 10302,'NASA' from dual union all
select 10303,'NASA' from dual union all
select 10304,'NASA' from dual union all
select 10305,'NASA' from dual union all
select 10306,'NASA' from dual union all
select 10307,'NASA' from dual union all
select 10308,'NASA' from dual union all
select 10309,'NASA' from dual union all
select 1031,'MO' from dual union all
select 10310,'NASA' from dual union all
select 10311,'NASA' from dual union all
select 10312,'NASA' from dual union all
select 10313,'NASA' from dual union all
select 10314,'NASA' from dual union all
select 10315,'RVSN' from dual union all
select 10316,'RVSN' from dual union all
select 10317,'RVSN' from dual union all
select 10318,'RVSN' from dual union all
select 10319,'MVD' from dual union all
select 1032,'MO' from dual union all
select 10320,'RVSN' from dual union all
select 10321,'JPL' from dual union all
select 10322,'NASA' from dual union all
select 10323,'NASA' from dual union all
select 10324,'RVSN' from dual union all
select 10325,'RVSN' from dual union all
select 10326,'TSSKB' from dual union all
select 10327,'RVSN' from dual union all
select 10328,'RVSN' from dual union all
select 10329,'PVO' from dual union all
select 1033,'MO' from dual union all
select 10330,'PVO' from dual union all
select 10331,'PVO' from dual union all
select 10332,'PVO' from dual union all
select 10333,'PVO' from dual union all
select 10334,'PVO' from dual union all
select 10335,'PVO' from dual union all
select 10336,'PVO' from dual union all
select 10337,'PVO' from dual union all
select 10338,'NASA' from dual union all
select 10339,'NASA' from dual union all
select 1034,'MO' from dual union all
select 10340,'NASA' from dual union all
select 10341,'NASA' from dual union all
select 10342,'NASA' from dual union all
select 10343,'NASA' from dual union all
select 10344,'NASA' from dual union all
select 10345,'NASA' from dual union all
select 10346,'NASA' from dual union all
select 10347,'NASA' from dual union all
select 10348,'NASA' from dual union all
select 10349,'PVO' from dual union all
select 1035,'MO' from dual union all
select 10350,'PVO' from dual;
insert into SATELLITE_ORG
select 10351,'RVSN' from dual union all
select 10352,'VMF' from dual union all
select 10353,'RVSN' from dual union all
select 10354,'RVSN' from dual union all
select 10355,'RVSN' from dual union all
select 10356,'RVSN' from dual union all
select 10357,'RVSN' from dual union all
select 10358,'VMF' from dual union all
select 10359,'RVSN' from dual union all
select 1036,'MO' from dual union all
select 10360,'RVSN' from dual union all
select 10361,'VMF' from dual union all
select 10362,'VS' from dual union all
select 10363,'RVSN' from dual union all
select 10364,'VS' from dual union all
select 10365,'MSVYAZ' from dual union all
select 10366,'RVSN' from dual union all
select 10367,'RVSN' from dual union all
select 10368,'RVSN' from dual union all
select 10369,'RVSN' from dual union all
select 1037,'MO' from dual union all
select 10370,'IKI' from dual union all
select 10370,'NPOL' from dual union all
select 10371,'RVSN' from dual union all
select 10372,'RVSN' from dual union all
select 10373,'RVSN' from dual union all
select 10374,'NRO' from dual union all
select 10375,'YUZH' from dual union all
select 10376,'IK' from dual union all
select 10377,'RVSN' from dual union all
select 10378,'RVSN' from dual union all
select 10379,'IK' from dual union all
select 1038,'MO' from dual union all
select 10380,'RVSN' from dual union all
select 10381,'RVSN' from dual union all
select 10382,'NPOE' from dual union all
select 10383,'RVSN' from dual union all
select 10384,'NPOE' from dual union all
select 10385,'RVSN' from dual union all
select 10386,'RVSN' from dual union all
select 10387,'RVSN' from dual union all
select 10388,'RVSN' from dual union all
select 10389,'NASA' from dual union all
select 1039,'MO' from dual union all
select 10390,'NASA' from dual union all
select 10391,'PVO' from dual union all
select 10392,'NASA' from dual union all
select 10393,'NASA' from dual union all
select 10394,'PVO' from dual union all
select 10395,'NASA' from dual union all
select 10396,'PVO' from dual union all
select 10397,'NASA' from dual union all
select 10398,'PVO' from dual union all
select 10399,'VMF' from dual union all
select 104,'RVSN' from dual union all
select 1040,'MO' from dual union all
select 10400,'VMF' from dual union all
select 10401,'NPOE' from dual union all
select 10402,'RVSN' from dual union all
select 10403,'RVSN' from dual union all
select 10404,'RVSN' from dual union all
select 10405,'RVSN' from dual union all
select 10406,'RVSN' from dual union all
select 10407,'RVSN' from dual union all
select 10408,'RVSN' from dual union all
select 10409,'NASA' from dual union all
select 1041,'MO' from dual union all
select 10410,'PVO' from dual union all
select 10411,'PVO' from dual union all
select 10412,'PVO' from dual union all
select 10413,'PVO' from dual union all
select 10414,'PVO' from dual union all
select 10415,'PVO' from dual union all
select 10416,'PVO' from dual union all
select 10417,'PVO' from dual union all
select 10418,'PVO' from dual union all
select 10419,'PVO' from dual union all
select 1042,'MO' from dual union all
select 10420,'RVSN' from dual union all
select 10421,'PVO' from dual union all
select 10422,'NASA' from dual union all
select 10423,'ESA' from dual union all
select 10424,'NASA' from dual union all
select 10425,'NASA' from dual union all
select 10426,'RVSN' from dual union all
select 10427,'RVSN' from dual union all
select 10428,'RVSN' from dual union all
select 10429,'RVSN' from dual union all
select 1043,'MO' from dual union all
select 10430,'VS' from dual union all
select 10431,'RVSN' from dual union all
select 10432,'VS' from dual union all
select 10433,'VS' from dual union all
select 10434,'PVO' from dual union all
select 10435,'RVSN' from dual union all
select 10436,'PVO' from dual union all
select 10437,'PVO' from dual union all
select 10438,'PVO' from dual union all
select 10439,'PVO' from dual union all
select 1044,'MO' from dual;
insert into SATELLITE_ORG
select 10440,'PVO' from dual union all
select 10441,'PVO' from dual union all
select 10442,'PVO' from dual union all
select 10443,'PVO' from dual union all
select 10444,'PVO' from dual union all
select 10445,'PVO' from dual union all
select 10446,'PVO' from dual union all
select 10447,'PVO' from dual union all
select 10448,'PVO' from dual union all
select 10449,'PVO' from dual union all
select 1045,'MO' from dual union all
select 10450,'PVO' from dual union all
select 10451,'PVO' from dual union all
select 10452,'PVO' from dual union all
select 10453,'PVO' from dual union all
select 10454,'PVO' from dual union all
select 10455,'MOM' from dual union all
select 10455,'MSVYAZ' from dual union all
select 10456,'RVSN' from dual union all
select 10457,'USN' from dual union all
select 10458,'RVSN' from dual union all
select 10459,'VMF' from dual union all
select 1046,'MO' from dual union all
select 10460,'RVSN' from dual union all
select 10461,'RVSN' from dual union all
select 10462,'NASA' from dual union all
select 10463,'PVO' from dual union all
select 10464,'PVO' from dual union all
select 10465,'PVO' from dual union all
select 10466,'PVO' from dual union all
select 10467,'PVO' from dual union all
select 10468,'NASA' from dual union all
select 10469,'NASA' from dual union all
select 1047,'MO' from dual union all
select 10470,'NASA' from dual union all
select 10471,'NASA' from dual union all
select 10472,'NASA' from dual union all
select 10473,'NASA' from dual union all
select 10474,'NASA' from dual union all
select 10475,'NASA' from dual union all
select 10476,'NASA' from dual union all
select 10477,'NASA' from dual union all
select 10478,'NASA' from dual union all
select 10479,'NASA' from dual union all
select 1048,'MO' from dual union all
select 10480,'NASA' from dual union all
select 10481,'NASA' from dual union all
select 10482,'NASA' from dual union all
select 10483,'NASA' from dual union all
select 10484,'RVSN' from dual union all
select 10485,'RVSN' from dual union all
select 10486,'RVSN' from dual union all
select 10487,'PVO' from dual union all
select 10488,'PVO' from dual union all
select 10489,'ESA' from dual union all
select 1049,'MO' from dual union all
select 10490,'NASA' from dual union all
select 10491,'VTS' from dual union all
select 10492,'RVSN' from dual union all
select 10493,'NASA' from dual union all
select 10494,'PVO' from dual union all
select 10495,'PVO' from dual union all
select 10496,'PVO' from dual union all
select 10497,'PVO' from dual union all
select 10498,'RVSN' from dual union all
select 10499,'RVSN' from dual union all
select 105,'NASA' from dual union all
select 1050,'MO' from dual union all
select 10500,'RVSN' from dual union all
select 10501,'PVO' from dual union all
select 10502,'NRO' from dual union all
select 10502,'USN' from dual union all
select 10503,'RVSN' from dual union all
select 10504,'USAF' from dual union all
select 10505,'PVO' from dual union all
select 10506,'NPOE' from dual union all
select 10507,'RVSN' from dual union all
select 10508,'CIA' from dual union all
select 10508,'NRO' from dual union all
select 10509,'USAF' from dual union all
select 1051,'MO' from dual union all
select 10510,'RVSN' from dual union all
select 10511,'RVSN' from dual union all
select 10512,'PVO' from dual union all
select 10513,'RVSN' from dual union all
select 10514,'GMS' from dual union all
select 10514,'RVSN' from dual union all
select 10515,'RVSN' from dual union all
select 10516,'NASDA' from dual union all
select 10517,'NASA' from dual union all
select 10518,'PVO' from dual union all
select 10519,'NASA' from dual union all
select 1052,'MO' from dual union all
select 10520,'RVSN' from dual union all
select 10521,'RVSN' from dual union all
select 10522,'RVSN' from dual union all
select 10523,'RVSN' from dual union all
select 10524,'RVSN' from dual union all
select 10525,'RVSN' from dual union all
select 10526,'RVSN' from dual;
insert into SATELLITE_ORG
select 10527,'RVSN' from dual union all
select 10528,'USN' from dual union all
select 10529,'NRO' from dual union all
select 10529,'USN' from dual union all
select 1053,'MO' from dual union all
select 10530,'RVSN' from dual union all
select 10531,'PVO' from dual union all
select 10532,'RVSN' from dual union all
select 10533,'NPOE' from dual union all
select 10534,'RVSN' from dual union all
select 10535,'PVO' from dual union all
select 10536,'VMF' from dual union all
select 10537,'RVSN' from dual union all
select 10538,'TSSKB' from dual union all
select 10539,'YUZH' from dual union all
select 1054,'MO' from dual union all
select 10540,'RVSN' from dual union all
select 10541,'RVSN' from dual union all
select 10542,'RVSN' from dual union all
select 10543,'RVSN' from dual union all
select 10544,'NRO' from dual union all
select 10544,'USN' from dual union all
select 10545,'NPOE' from dual union all
select 10546,'RVSN' from dual union all
select 10547,'NPOE' from dual union all
select 10548,'NPOE' from dual union all
select 10549,'NPOE' from dual union all
select 1055,'MO' from dual union all
select 10550,'RVSN' from dual union all
select 10551,'RVSN' from dual union all
select 10552,'RVSN' from dual union all
select 10553,'NPOE' from dual union all
select 10554,'RVSN' from dual union all
select 10555,'RVSN' from dual union all
select 10556,'RVSN' from dual union all
select 10557,'INTEL' from dual union all
select 10558,'TSSKB' from dual union all
select 10559,'RVSN' from dual union all
select 1056,'MO' from dual union all
select 10560,'NPOE' from dual union all
select 10561,'VS' from dual union all
select 10562,'PVO' from dual union all
select 10563,'PVO' from dual union all
select 10564,'PVO' from dual union all
select 10565,'PVO' from dual union all
select 10566,'PVO' from dual union all
select 10567,'PVO' from dual union all
select 10568,'PVO' from dual union all
select 10569,'PVO' from dual union all
select 1057,'MO' from dual union all
select 10570,'PVO' from dual union all
select 10571,'PVO' from dual union all
select 10572,'PVO' from dual union all
select 10573,'PVO' from dual union all
select 10574,'PVO' from dual union all
select 10575,'PVO' from dual union all
select 10576,'PVO' from dual union all
select 10577,'PVO' from dual union all
select 10578,'PVO' from dual union all
select 10579,'PVO' from dual union all
select 1058,'MO' from dual union all
select 10580,'PVO' from dual union all
select 10581,'RVSN' from dual union all
select 10582,'RVSN' from dual union all
select 10583,'RVSN' from dual union all
select 10584,'RVSN' from dual union all
select 10585,'RVSN' from dual union all
select 10586,'RVSN' from dual union all
select 10587,'RVSN' from dual union all
select 10588,'RVSN' from dual union all
select 10589,'RVSN' from dual union all
select 1059,'MO' from dual union all
select 10590,'RVSN' from dual union all
select 10591,'RVSN' from dual union all
select 10592,'RVSN' from dual union all
select 10593,'RVSN' from dual union all
select 10594,'NRO' from dual union all
select 10594,'USN' from dual union all
select 10595,'USN' from dual union all
select 10596,'NPOE' from dual union all
select 10597,'NPOE' from dual union all
select 10598,'NPOE' from dual union all
select 10599,'VMF' from dual union all
select 106,'CIA' from dual union all
select 106,'USAF' from dual union all
select 1060,'MO' from dual union all
select 10600,'RVSN' from dual union all
select 10601,'RVSN' from dual union all
select 10602,'RVSN' from dual union all
select 10603,'NPOE' from dual union all
select 10604,'RVSN' from dual union all
select 10605,'MOM' from dual union all
select 10605,'MSVYAZ' from dual union all
select 10606,'RVSN' from dual union all
select 10607,'RVSN' from dual union all
select 10608,'RVSN' from dual union all
select 10609,'RVSN' from dual union all
select 1061,'MO' from dual union all
select 10610,'RVSN' from dual union all
select 10611,'MAI' from dual;
insert into SATELLITE_ORG
select 10612,'MAI' from dual union all
select 10613,'PVO' from dual union all
select 10614,'PVO' from dual union all
select 10615,'PVO' from dual union all
select 10616,'PVO' from dual union all
select 10617,'PVO' from dual union all
select 10618,'PVO' from dual union all
select 10619,'PVO' from dual union all
select 1062,'MO' from dual union all
select 10620,'PVO' from dual union all
select 10621,'PVO' from dual union all
select 10622,'PVO' from dual union all
select 10623,'PVO' from dual union all
select 10624,'NASA' from dual union all
select 10625,'NASA' from dual union all
select 10626,'NASA' from dual union all
select 10627,'NASA' from dual union all
select 10628,'NASA' from dual union all
select 10629,'NASA' from dual union all
select 1063,'MO' from dual union all
select 10630,'NASA' from dual union all
select 10631,'NASA' from dual union all
select 10632,'NASA' from dual union all
select 10633,'NASA' from dual union all
select 10634,'NASA' from dual union all
select 10635,'NASA' from dual union all
select 10636,'NASA' from dual union all
select 10637,'ESA' from dual union all
select 10637,'NASA' from dual union all
select 10638,'NASA' from dual union all
select 10639,'RVSN' from dual union all
select 1064,'MO' from dual union all
select 10640,'RVSN' from dual union all
select 10641,'RVSN' from dual union all
select 10642,'NASA' from dual union all
select 10643,'NASA' from dual union all
select 10644,'NASA' from dual union all
select 10645,'NASA' from dual union all
select 10646,'NASA' from dual union all
select 10647,'NASA' from dual union all
select 10648,'NASA' from dual union all
select 10649,'NASA' from dual union all
select 1065,'MO' from dual union all
select 10650,'NASA' from dual union all
select 10651,'NASA' from dual union all
select 10652,'NASA' from dual union all
select 10653,'NASA' from dual union all
select 10654,'NASA' from dual union all
select 10655,'NASA' from dual union all
select 10656,'NASA' from dual union all
select 10657,'NASA' from dual union all
select 10658,'NASA' from dual union all
select 10659,'NASA' from dual union all
select 1066,'MO' from dual union all
select 10660,'NASA' from dual union all
select 10661,'NASA' from dual union all
select 10662,'NASA' from dual union all
select 10663,'NASA' from dual union all
select 10664,'ISAS' from dual union all
select 10665,'ISAS' from dual union all
select 10666,'VTS' from dual union all
select 10667,'RVSN' from dual union all
select 10668,'VTS' from dual union all
select 10669,'USN' from dual union all
select 1067,'MO' from dual union all
select 10670,'RVSN' from dual union all
select 10671,'RVSN' from dual union all
select 10672,'RVSN' from dual union all
select 10673,'RVSN' from dual union all
select 10674,'NASDA' from dual union all
select 10675,'NASDA' from dual union all
select 10676,'RVSN' from dual union all
select 10677,'RVSN' from dual union all
select 10678,'RVSN' from dual union all
select 10679,'RVSN' from dual union all
select 1068,'MO' from dual union all
select 10680,'RVSN' from dual union all
select 10681,'RVSN' from dual union all
select 10682,'PVO' from dual union all
select 10683,'PVO' from dual union all
select 10684,'USAF' from dual union all
select 10685,'NPOE' from dual union all
select 10686,'NPOE' from dual union all
select 10687,'NPOE' from dual union all
select 10688,'NRO' from dual union all
select 10689,'USAF' from dual union all
select 1069,'MO' from dual union all
select 10690,'RVSN' from dual union all
select 10691,'RVSN' from dual union all
select 10692,'VMF' from dual union all
select 10693,'RVSN' from dual union all
select 10694,'NPOE' from dual union all
select 10695,'RVSN' from dual union all
select 10696,'RVSN' from dual union all
select 10697,'RVSN' from dual union all
select 10698,'RVSN' from dual union all
select 10699,'RVSN' from dual union all
select 107,'NASA' from dual union all
select 1070,'MO' from dual union all
select 10700,'RVSN' from dual;
insert into SATELLITE_ORG
select 10701,'RVSN' from dual union all
select 10702,'NASA' from dual union all
select 10703,'AMNA' from dual union all
select 10704,'NASA' from dual union all
select 10705,'NASA' from dual union all
select 10706,'NASA' from dual union all
select 10707,'NASA' from dual union all
select 10708,'NASA' from dual union all
select 10709,'NASA' from dual union all
select 1071,'MO' from dual union all
select 10710,'NASA' from dual union all
select 10711,'NASA' from dual union all
select 10712,'NASA' from dual union all
select 10713,'NASA' from dual union all
select 10714,'NASA' from dual union all
select 10715,'NASA' from dual union all
select 10716,'NASA' from dual union all
select 10717,'NASA' from dual union all
select 10718,'NASA' from dual union all
select 10719,'NASA' from dual union all
select 1072,'MO' from dual union all
select 10720,'NASA' from dual union all
select 10721,'NASA' from dual union all
select 10722,'NASA' from dual union all
select 10723,'NASA' from dual union all
select 10724,'NASA' from dual union all
select 10725,'RVSN' from dual union all
select 10726,'RVSN' from dual union all
select 10727,'RVSN' from dual union all
select 10728,'NASA' from dual union all
select 10729,'NASA' from dual union all
select 1073,'MO' from dual union all
select 10730,'NASA' from dual union all
select 10731,'VMF' from dual union all
select 10732,'RVSN' from dual union all
select 10733,'NRO' from dual union all
select 10734,'NRO' from dual union all
select 10735,'RVSN' from dual union all
select 10736,'RVSN' from dual union all
select 10737,'USAF' from dual union all
select 10738,'NPOE' from dual union all
select 10739,'RVSN' from dual union all
select 1074,'MO' from dual union all
select 10740,'NPOE' from dual union all
select 10741,'PVO' from dual union all
select 10742,'PVO' from dual union all
select 10743,'NPOE' from dual union all
select 10744,'VMF' from dual union all
select 10745,'RVSN' from dual union all
select 10746,'NASA' from dual union all
select 10747,'NASA' from dual union all
select 10748,'NASA' from dual union all
select 10749,'NASA' from dual union all
select 1075,'MO' from dual union all
select 10750,'NASA' from dual union all
select 10751,'NASA' from dual union all
select 10752,'NASA' from dual union all
select 10753,'NASA' from dual union all
select 10754,'NASA' from dual union all
select 10755,'NASA' from dual union all
select 10756,'NASA' from dual union all
select 10757,'NASA' from dual union all
select 10758,'NASA' from dual union all
select 10759,'NASA' from dual union all
select 1076,'MO' from dual union all
select 10760,'NASA' from dual union all
select 10761,'NASA' from dual union all
select 10762,'NASA' from dual union all
select 10763,'NASA' from dual union all
select 10764,'NASA' from dual union all
select 10765,'NASA' from dual union all
select 10766,'NASA' from dual union all
select 10767,'NASA' from dual union all
select 10768,'NASA' from dual union all
select 10769,'NASA' from dual union all
select 1077,'MO' from dual union all
select 10770,'KBSAL' from dual union all
select 10771,'KBSAL' from dual union all
select 10772,'RVSN' from dual union all
select 10773,'RVSN' from dual union all
select 10774,'RVSN' from dual union all
select 10775,'KBSAL' from dual union all
select 10776,'MOM' from dual union all
select 10777,'RVSN' from dual union all
select 10778,'INTEL' from dual union all
select 10779,'NASA' from dual union all
select 1078,'MO' from dual union all
select 10780,'PVO' from dual union all
select 10781,'NPOE' from dual union all
select 10782,'PVO' from dual union all
select 10783,'NPOE' from dual union all
select 10784,'RVSN' from dual union all
select 10785,'RVSN' from dual union all
select 10786,'RVSN' from dual union all
select 10787,'CIA' from dual union all
select 10787,'NRO' from dual union all
select 10788,'USAF' from dual union all
select 10789,'PVO' from dual union all
select 1079,'MO' from dual union all
select 10790,'RVSN' from dual;
insert into SATELLITE_ORG
select 10791,'RVSN' from dual union all
select 10792,'NASDA' from dual union all
select 10793,'NASA' from dual union all
select 10794,'NASA' from dual union all
select 10795,'PVO' from dual union all
select 10796,'USN' from dual union all
select 10797,'PVO' from dual union all
select 10798,'RVSN' from dual union all
select 10799,'RVSN' from dual union all
select 108,'CIA' from dual union all
select 108,'USAF' from dual union all
select 1080,'MO' from dual union all
select 10800,'RVSN' from dual union all
select 10801,'USAF' from dual union all
select 10802,'RVSN' from dual union all
select 10803,'RVSN' from dual union all
select 10804,'NPOE' from dual union all
select 10805,'NPOE' from dual union all
select 10806,'NPOE' from dual union all
select 10807,'PVO' from dual union all
select 10808,'RVSN' from dual union all
select 10809,'PVO' from dual union all
select 1081,'MO' from dual union all
select 10810,'NPOE' from dual union all
select 10811,'RVSN' from dual union all
select 10812,'RVSN' from dual union all
select 10813,'RVSN' from dual union all
select 10814,'PVO' from dual union all
select 10815,'PVO' from dual union all
select 10816,'PVO' from dual union all
select 10817,'PVO' from dual union all
select 10818,'NASA' from dual union all
select 10819,'NASA' from dual union all
select 1082,'MO' from dual union all
select 10820,'USAF' from dual union all
select 10821,'RVSN' from dual union all
select 10822,'RVSN' from dual union all
select 10823,'NASA' from dual union all
select 10824,'NASA' from dual union all
select 10825,'NASA' from dual union all
select 10826,'NASA' from dual union all
select 10827,'NASA' from dual union all
select 10828,'NASA' from dual union all
select 10829,'NASA' from dual union all
select 1083,'MO' from dual union all
select 10830,'NASA' from dual union all
select 10831,'NASA' from dual union all
select 10832,'NASA' from dual union all
select 10833,'NASA' from dual union all
select 10834,'NASA' from dual union all
select 10835,'NASA' from dual union all
select 10836,'NASA' from dual union all
select 10837,'NASA' from dual union all
select 10838,'NASA' from dual union all
select 10839,'NASA' from dual union all
select 1084,'MO' from dual union all
select 10840,'NASA' from dual union all
select 10841,'NASA' from dual union all
select 10842,'NASA' from dual union all
select 10843,'NASA' from dual union all
select 10844,'NASA' from dual union all
select 10845,'NASA' from dual union all
select 10846,'RVSN' from dual union all
select 10847,'RVSN' from dual union all
select 10848,'RVSN' from dual union all
select 10849,'RVSN' from dual union all
select 1085,'NASA' from dual union all
select 10850,'RVSN' from dual union all
select 10851,'TSSKB' from dual union all
select 10852,'RVSN' from dual union all
select 10853,'USAF' from dual union all
select 10854,'USAF' from dual union all
select 10855,'ESA' from dual union all
select 10856,'NASA' from dual union all
select 10857,'NASA' from dual union all
select 10858,'PVO' from dual union all
select 10859,'PVO' from dual union all
select 1086,'JPL' from dual union all
select 1086,'NASA' from dual union all
select 10860,'VS' from dual union all
select 10861,'RVSN' from dual union all
select 10862,'PVO' from dual union all
select 10863,'RVSN' from dual union all
select 10864,'NASA' from dual union all
select 10865,'NASA' from dual union all
select 10866,'NASA' from dual union all
select 10867,'NASA' from dual union all
select 10868,'NASA' from dual union all
select 10869,'NASA' from dual union all
select 1087,'NASA' from dual union all
select 10870,'NASA' from dual union all
select 10871,'NASA' from dual union all
select 10872,'NASA' from dual union all
select 10873,'NASA' from dual union all
select 10874,'NASA' from dual union all
select 10875,'NASA' from dual union all
select 10876,'NASA' from dual union all
select 10877,'PVO' from dual union all
select 10878,'PVO' from dual union all
select 10879,'PVO' from dual;
insert into SATELLITE_ORG
select 1088,'NASA' from dual union all
select 10880,'PVO' from dual union all
select 10881,'PVO' from dual union all
select 10882,'PVO' from dual union all
select 10883,'PVO' from dual union all
select 10884,'PVO' from dual union all
select 10885,'PVO' from dual union all
select 10886,'PVO' from dual union all
select 10887,'PVO' from dual union all
select 10888,'PVO' from dual union all
select 10889,'PVO' from dual union all
select 1089,'OKB10' from dual union all
select 10890,'PVO' from dual union all
select 10891,'PVO' from dual union all
select 10892,'PVO' from dual union all
select 10893,'USAF' from dual union all
select 10894,'USAF' from dual union all
select 10895,'RVSN' from dual union all
select 10896,'RVSN' from dual union all
select 10897,'RVSN' from dual union all
select 10898,'VS' from dual union all
select 10899,'RVSN' from dual union all
select 109,'USAF' from dual union all
select 1090,'OKB10' from dual union all
select 10900,'VS' from dual union all
select 10901,'VS' from dual union all
select 10902,'VS' from dual union all
select 10903,'VS' from dual union all
select 10904,'PVO' from dual union all
select 10905,'RVSN' from dual union all
select 10906,'PVO' from dual union all
select 10907,'PVO' from dual union all
select 10908,'PVO' from dual union all
select 10909,'PVO' from dual union all
select 1091,'OKB10' from dual union all
select 10910,'PVO' from dual union all
select 10911,'NASA' from dual union all
select 10912,'NASA' from dual union all
select 10913,'USAF' from dual union all
select 10914,'USAF' from dual union all
select 10915,'MVD' from dual union all
select 10916,'RVSN' from dual union all
select 10917,'VMF' from dual union all
select 10918,'RVSN' from dual union all
select 10919,'RVSN' from dual union all
select 1092,'RVSN' from dual union all
select 10920,'RVSN' from dual union all
select 10921,'RVSN' from dual union all
select 10922,'RVSN' from dual union all
select 10923,'PVO' from dual union all
select 10924,'PVO' from dual union all
select 10925,'RVSN' from dual union all
select 10926,'RVSN' from dual union all
select 10927,'RVSN' from dual union all
select 10928,'RVSN' from dual union all
select 10929,'RVSN' from dual union all
select 1093,'OKB1' from dual union all
select 10930,'RVSN' from dual union all
select 10931,'RVSN' from dual union all
select 10932,'RVSN' from dual union all
select 10933,'RVSN' from dual union all
select 10934,'RVSN' from dual union all
select 10935,'RVSN' from dual union all
select 10936,'RVSN' from dual union all
select 10937,'RVSN' from dual union all
select 10938,'RVSN' from dual union all
select 10939,'RVSN' from dual union all
select 1094,'OKB10' from dual union all
select 10940,'RVSN' from dual union all
select 10941,'NRO' from dual union all
select 10942,'USAF' from dual union all
select 10943,'USAF' from dual union all
select 10944,'RVSN' from dual union all
select 10945,'RVSN' from dual union all
select 10946,'PVO' from dual union all
select 10947,'CIA' from dual union all
select 10947,'NRO' from dual union all
select 10948,'USAF' from dual union all
select 10949,'RVSN' from dual union all
select 1095,'RVSN' from dual union all
select 10950,'NASA' from dual union all
select 10951,'NRO' from dual union all
select 10952,'NPOE' from dual union all
select 10953,'NOAA' from dual union all
select 10954,'NASA' from dual union all
select 10955,'NASA' from dual union all
select 10956,'RVSN' from dual union all
select 10957,'NRO' from dual union all
select 10958,'NRO' from dual union all
select 10959,'NRO' from dual union all
select 1096,'CIA' from dual union all
select 1096,'NRO' from dual union all
select 10960,'USAF' from dual union all
select 10961,'RVSN' from dual union all
select 10962,'RVSN' from dual union all
select 10963,'RVSN' from dual union all
select 10964,'RVSN' from dual union all
select 10965,'RVSN' from dual union all
select 10966,'RVSN' from dual union all
select 10967,'JPL' from dual;
insert into SATELLITE_ORG
select 10967,'NASA' from dual union all
select 10968,'NPOE' from dual union all
select 10969,'RVSN' from dual union all
select 1097,'METP' from dual union all
select 10970,'PVO' from dual union all
select 10971,'RVSN' from dual union all
select 10972,'RVSN' from dual union all
select 10973,'YUZH' from dual union all
select 10974,'RVSN' from dual union all
select 10975,'COM' from dual union all
select 10976,'NASA' from dual union all
select 10977,'TSSKB' from dual union all
select 10978,'RVSN' from dual union all
select 10979,'NPOE' from dual union all
select 1098,'RVSN' from dual union all
select 10980,'RVSN' from dual union all
select 10981,'ESA' from dual union all
select 10982,'NASA' from dual union all
select 10983,'NASA' from dual union all
select 10984,'RVSN' from dual union all
select 10985,'RVSN' from dual union all
select 10986,'RVSN' from dual union all
select 10987,'RVSN' from dual union all
select 10988,'RVSN' from dual union all
select 10989,'RVSN' from dual union all
select 1099,'USN' from dual union all
select 10990,'NASA' from dual union all
select 10991,'VMF' from dual union all
select 10992,'RVSN' from dual union all
select 10993,'NRO' from dual union all
select 10994,'USAF' from dual union all
select 10995,'RVSN' from dual union all
select 10996,'RVSN' from dual union all
select 10997,'RVSN' from dual union all
select 10998,'RVSN' from dual union all
select 10999,'NPOE' from dual union all
select 11,'NASA' from dual union all
select 110,'NASA' from dual union all
select 110,'USAF' from dual union all
select 1100,'OKB1' from dual union all
select 11000,'RVSN' from dual union all
select 11001,'NASA' from dual union all
select 11002,'NASA' from dual union all
select 11003,'NASA' from dual union all
select 11004,'NASA' from dual union all
select 11005,'NASA' from dual union all
select 11006,'NASA' from dual union all
select 11007,'RVSN' from dual union all
select 11008,'RVSN' from dual union all
select 11009,'RVSN' from dual union all
select 1101,'OKB1' from dual union all
select 11010,'NPOE' from dual union all
select 11011,'RVSN' from dual union all
select 11012,'RVSN' from dual union all
select 11013,'RVSN' from dual union all
select 11014,'RVSN' from dual union all
select 11015,'PVO' from dual union all
select 11016,'RVSN' from dual union all
select 11017,'RVSN' from dual union all
select 11018,'RVSN' from dual union all
select 11019,'RVSN' from dual union all
select 1102,'OKB1' from dual union all
select 11020,'IKI' from dual union all
select 11020,'NPOL' from dual union all
select 11021,'RVSN' from dual union all
select 11022,'RVSN' from dual union all
select 11023,'RVSN' from dual union all
select 11024,'RVSN' from dual union all
select 11025,'IKI' from dual union all
select 11025,'NPOL' from dual union all
select 11026,'RVSN' from dual union all
select 11027,'ISAS' from dual union all
select 11028,'ISAS' from dual union all
select 11029,'RVSN' from dual union all
select 1103,'OKB1' from dual union all
select 11030,'RVSN' from dual union all
select 11031,'RVSN' from dual union all
select 11032,'NPOE' from dual union all
select 11033,'NPOE' from dual union all
select 11034,'NPOE' from dual union all
select 11035,'NPOE' from dual union all
select 11036,'NPOE' from dual union all
select 11037,'TSSKB' from dual union all
select 11038,'TSSKB' from dual union all
select 11039,'MVD' from dual union all
select 1104,'OKB1' from dual union all
select 11040,'NPOE' from dual union all
select 11041,'RVSN' from dual union all
select 11042,'RVSN' from dual union all
select 11043,'RVSN' from dual union all
select 11044,'RVSN' from dual union all
select 11045,'RVSN' from dual union all
select 11046,'RVSN' from dual union all
select 11047,'RVSN' from dual union all
select 11048,'RVSN' from dual union all
select 11049,'RVSN' from dual union all
select 1105,'OKB1' from dual union all
select 11050,'RVSN' from dual union all
select 11051,'RVSN' from dual union all
select 11052,'RVSN' from dual;
insert into SATELLITE_ORG
select 11053,'RVSN' from dual union all
select 11054,'USAF' from dual union all
select 11055,'VS' from dual union all
select 11056,'RVSN' from dual union all
select 11057,'MOM' from dual union all
select 11057,'MSVYAZ' from dual union all
select 11058,'RVSN' from dual union all
select 11059,'RVSN' from dual union all
select 1106,'OKB1' from dual union all
select 11060,'NASA' from dual union all
select 11061,'NASA' from dual union all
select 11062,'NASA' from dual union all
select 11063,'RVSN' from dual union all
select 11064,'RVSN' from dual union all
select 11065,'RVSN' from dual union all
select 11066,'RVSN' from dual union all
select 11067,'RVSN' from dual union all
select 11068,'RVSN' from dual union all
select 11069,'RVSN' from dual union all
select 1107,'OKB1' from dual union all
select 11070,'RVSN' from dual union all
select 11071,'RVSN' from dual union all
select 11072,'NASA' from dual union all
select 11073,'RVSN' from dual union all
select 11074,'RVSN' from dual union all
select 11075,'RVSN' from dual union all
select 11076,'RVSN' from dual union all
select 11077,'RVSN' from dual union all
select 11078,'USAF' from dual union all
select 11079,'RVSN' from dual union all
select 1108,'OKB1' from dual union all
select 11080,'NASA' from dual union all
select 11081,'NASA' from dual union all
select 11082,'IK' from dual union all
select 11083,'RVSN' from dual union all
select 11084,'GMS' from dual union all
select 11084,'RVSN' from dual union all
select 11085,'MAIO' from dual union all
select 11085,'MOM' from dual union all
select 11086,'MAIO' from dual union all
select 11086,'MOM' from dual union all
select 11087,'RVSN' from dual union all
select 11088,'IKI' from dual union all
select 11088,'NPOL' from dual union all
select 11089,'RVSN' from dual union all
select 1109,'OKB1' from dual union all
select 11090,'RVSN' from dual union all
select 11091,'NPOE' from dual union all
select 11092,'NPOE' from dual union all
select 11093,'NPOE' from dual union all
select 11094,'NPOE' from dual union all
select 11095,'NPOE' from dual union all
select 11096,'NPOE' from dual union all
select 11097,'NPOE' from dual union all
select 11098,'VTS' from dual union all
select 11099,'RVSN' from dual union all
select 111,'USA' from dual union all
select 1110,'OKB1' from dual union all
select 11100,'NPOE' from dual union all
select 11101,'NASA' from dual union all
select 11102,'NASA' from dual union all
select 11103,'RVSN' from dual union all
select 11104,'RVSN' from dual union all
select 11105,'RVSN' from dual union all
select 11106,'RVSN' from dual union all
select 11107,'RVSN' from dual union all
select 11108,'RVSN' from dual union all
select 11109,'RVSN' from dual union all
select 1111,'OKB1' from dual union all
select 11110,'CSSR' from dual union all
select 11111,'RVSN' from dual union all
select 11112,'RVSN' from dual union all
select 11113,'RVSN' from dual union all
select 11114,'RVSN' from dual union all
select 11115,'NATO' from dual union all
select 11116,'NASA' from dual union all
select 11117,'NASA' from dual union all
select 11118,'RVSN' from dual union all
select 11119,'RVSN' from dual union all
select 1112,'OKB1' from dual union all
select 11120,'RVSN' from dual union all
select 11121,'RVSN' from dual union all
select 11122,'RVSN' from dual union all
select 11123,'RVSN' from dual union all
select 11124,'RVSN' from dual union all
select 11125,'RVSN' from dual union all
select 11126,'RVSN' from dual union all
select 11127,'NPOE' from dual union all
select 11128,'RVSN' from dual union all
select 11129,'RVSN' from dual union all
select 1113,'OKB1' from dual union all
select 11130,'RVSN' from dual union all
select 11131,'RVSN' from dual union all
select 11132,'RVSN' from dual union all
select 11133,'RVSN' from dual union all
select 11134,'RVSN' from dual union all
select 11135,'RVSN' from dual union all
select 11136,'RVSN' from dual union all
select 11137,'RVSN' from dual union all
select 11138,'RVSN' from dual;
insert into SATELLITE_ORG
select 11139,'RVSN' from dual union all
select 1114,'OKB1' from dual union all
select 11140,'RVSN' from dual union all
select 11141,'USAF' from dual union all
select 11142,'USAF' from dual union all
select 11143,'RVSN' from dual union all
select 11144,'DOD' from dual union all
select 11144,'USAF' from dual union all
select 11145,'DOD' from dual union all
select 11145,'USAF' from dual union all
select 11146,'USAF' from dual union all
select 11147,'USAF' from dual union all
select 11148,'RVSN' from dual union all
select 11149,'RVSN' from dual union all
select 1115,'OKB1' from dual union all
select 11150,'VS' from dual union all
select 11151,'RVSN' from dual union all
select 11152,'VS' from dual union all
select 11153,'TCAN' from dual union all
select 11154,'NASA' from dual union all
select 11155,'VS' from dual union all
select 11156,'RVSN' from dual union all
select 11157,'RVSN' from dual union all
select 11158,'MSVYAZ' from dual union all
select 11159,'RVSN' from dual union all
select 1116,'OKB1' from dual union all
select 11160,'RVSN' from dual union all
select 11161,'VMF' from dual union all
select 11162,'RVSN' from dual union all
select 11163,'PVO' from dual union all
select 11164,'RVSN' from dual union all
select 11165,'METP' from dual union all
select 11165,'MOM' from dual union all
select 11166,'RVSN' from dual union all
select 11167,'TSSKB' from dual union all
select 11168,'VTS' from dual union all
select 11169,'RVSN' from dual union all
select 1117,'OKB1' from dual union all
select 11170,'RVSN' from dual union all
select 11171,'RVSN' from dual union all
select 11172,'RVSN' from dual union all
select 11173,'VTS' from dual union all
select 11174,'RVSN' from dual union all
select 11175,'VTS' from dual union all
select 11176,'VTS' from dual union all
select 11177,'RVSN' from dual union all
select 11178,'PVO' from dual union all
select 11179,'PVO' from dual union all
select 1118,'OKB1' from dual union all
select 11180,'PVO' from dual union all
select 11181,'PVO' from dual union all
select 11182,'PVO' from dual union all
select 11183,'PVO' from dual union all
select 11184,'PVO' from dual union all
select 11185,'PVO' from dual union all
select 11186,'PVO' from dual union all
select 11187,'PVO' from dual union all
select 11188,'PVO' from dual union all
select 11189,'PVO' from dual union all
select 1119,'OKB1' from dual union all
select 11190,'PVO' from dual union all
select 11191,'PVO' from dual union all
select 11192,'PVO' from dual union all
select 11193,'PVO' from dual union all
select 11194,'PVO' from dual union all
select 11195,'PVO' from dual union all
select 11196,'PVO' from dual union all
select 11197,'PVO' from dual union all
select 11198,'PVO' from dual union all
select 11199,'PVO' from dual union all
select 112,'OKB1' from dual union all
select 1120,'OKB1' from dual union all
select 11200,'PVO' from dual union all
select 11201,'PVO' from dual union all
select 11202,'PVO' from dual union all
select 11203,'PVO' from dual union all
select 11204,'PVO' from dual union all
select 11205,'PVO' from dual union all
select 11206,'PVO' from dual union all
select 11207,'PVO' from dual union all
select 11208,'PVO' from dual union all
select 11209,'PVO' from dual union all
select 1121,'OKB1' from dual union all
select 11210,'PVO' from dual union all
select 11211,'PVO' from dual union all
select 11212,'PVO' from dual union all
select 11213,'PVO' from dual union all
select 11214,'PVO' from dual union all
select 11215,'PVO' from dual union all
select 11216,'PVO' from dual union all
select 11217,'PVO' from dual union all
select 11218,'PVO' from dual union all
select 11219,'PVO' from dual union all
select 1122,'OKB1' from dual union all
select 11220,'PVO' from dual union all
select 11221,'PVO' from dual union all
select 11222,'PVO' from dual union all
select 11223,'PVO' from dual union all
select 11224,'PVO' from dual union all
select 11225,'PVO' from dual;
insert into SATELLITE_ORG
select 11226,'RVSN' from dual union all
select 11227,'RVSN' from dual union all
select 11228,'RVSN' from dual union all
select 11229,'RVSN' from dual union all
select 1123,'OKB1' from dual union all
select 11230,'RVSN' from dual union all
select 11231,'RVSN' from dual union all
select 11232,'RVSN' from dual union all
select 11233,'RVSN' from dual union all
select 11234,'RVSN' from dual union all
select 11235,'RVSN' from dual union all
select 11236,'RVSN' from dual union all
select 11237,'RVSN' from dual union all
select 11238,'VMF' from dual union all
select 11239,'RVSN' from dual union all
select 1124,'OKB1' from dual union all
select 11240,'MOM' from dual union all
select 11240,'MSVYAZ' from dual union all
select 11241,'RVSN' from dual union all
select 11242,'RVSN' from dual union all
select 11243,'PVO' from dual union all
select 11244,'PVO' from dual union all
select 11245,'TSSKB' from dual union all
select 11246,'RVSN' from dual union all
select 11247,'RVSN' from dual union all
select 11248,'PVO' from dual union all
select 11249,'PVO' from dual union all
select 1125,'OKB1' from dual union all
select 11250,'RVSN' from dual union all
select 11251,'MVD' from dual union all
select 11252,'RVSN' from dual union all
select 11253,'RVSN' from dual union all
select 11254,'RVSN' from dual union all
select 11255,'RVSN' from dual union all
select 11256,'USAF' from dual union all
select 11257,'RVSN' from dual union all
select 11258,'NASA' from dual union all
select 11259,'NPOE' from dual union all
select 1126,'OKB1' from dual union all
select 11260,'RVSN' from dual union all
select 11261,'NASDA' from dual union all
select 11262,'PVO' from dual union all
select 11263,'RVSN' from dual union all
select 11264,'RVSN' from dual union all
select 11265,'RVSN' from dual union all
select 11266,'GMS' from dual union all
select 11266,'YUZH' from dual union all
select 11267,'RVSN' from dual union all
select 11268,'VS' from dual union all
select 11269,'RVSN' from dual union all
select 1127,'OKB1' from dual union all
select 11270,'NASA' from dual union all
select 11271,'NASA' from dual union all
select 11272,'ISAS' from dual union all
select 11273,'MSVYAZ' from dual union all
select 11274,'RVSN' from dual union all
select 11275,'RVSN' from dual union all
select 11276,'RVSN' from dual union all
select 11277,'RVSN' from dual union all
select 11278,'USN' from dual union all
select 11279,'USAF' from dual union all
select 1128,'OKB1' from dual union all
select 11280,'ISAS' from dual union all
select 11281,'NPOE' from dual union all
select 11282,'RVSN' from dual union all
select 11283,'RVSN' from dual union all
select 11284,'RVSN' from dual union all
select 11285,'IK' from dual union all
select 11286,'RVSN' from dual union all
select 11287,'RVSN' from dual union all
select 11288,'GMS' from dual union all
select 11288,'RVSN' from dual union all
select 11289,'RVSN' from dual union all
select 1129,'OKB1' from dual union all
select 11290,'RVSN' from dual union all
select 11291,'USAF' from dual union all
select 11292,'NPOE' from dual union all
select 11293,'RVSN' from dual union all
select 11294,'RVSN' from dual union all
select 11295,'RVSN' from dual union all
select 11296,'RVSN' from dual union all
select 11297,'RVSN' from dual union all
select 11298,'RVSN' from dual union all
select 11299,'RVSN' from dual union all
select 113,'USA' from dual union all
select 1130,'OKB1' from dual union all
select 11300,'RVSN' from dual union all
select 11301,'RVSN' from dual union all
select 11302,'RVSN' from dual union all
select 11303,'RVSN' from dual union all
select 11304,'RVSN' from dual union all
select 11305,'NRO' from dual union all
select 11306,'NRO' from dual union all
select 11307,'USAF' from dual union all
select 11308,'VMF' from dual union all
select 11309,'RVSN' from dual union all
select 1131,'OKB1' from dual union all
select 11310,'RVSN' from dual union all
select 11311,'RVSN' from dual union all
select 11312,'RVSN' from dual;
insert into SATELLITE_ORG
select 11313,'RVSN' from dual union all
select 11314,'RVSN' from dual union all
select 11315,'NPOE' from dual union all
select 11316,'NPOE' from dual union all
select 11317,'NPOE' from dual union all
select 11318,'NPOE' from dual union all
select 11319,'NPOE' from dual union all
select 1132,'OKB1' from dual union all
select 11320,'VMF' from dual union all
select 11321,'RVSN' from dual union all
select 11322,'USAF' from dual union all
select 11323,'IK' from dual union all
select 11324,'NPOE' from dual union all
select 11325,'RVSN' from dual union all
select 11326,'MOM' from dual union all
select 11327,'RVSN' from dual union all
select 11328,'RVSN' from dual union all
select 11329,'RVSN' from dual union all
select 1133,'OKB1' from dual union all
select 11330,'RVSN' from dual union all
select 11331,'VS' from dual union all
select 11332,'RVSN' from dual union all
select 11333,'VMF' from dual union all
select 11334,'RVSN' from dual union all
select 11335,'RVSN' from dual union all
select 11336,'RVSN' from dual union all
select 11337,'RVSN' from dual union all
select 11338,'NPOE' from dual union all
select 11339,'NPOE' from dual union all
select 1134,'OKB1' from dual union all
select 11340,'NPOE' from dual union all
select 11341,'NPOE' from dual union all
select 11342,'NPOE' from dual union all
select 11343,'RVSN' from dual union all
select 11344,'RVSN' from dual union all
select 11345,'RVSN' from dual union all
select 11346,'VMF' from dual union all
select 11347,'RVSN' from dual union all
select 11348,'TSSKB' from dual union all
select 11349,'RVSN' from dual union all
select 1135,'OKB1' from dual union all
select 11350,'NPOE' from dual union all
select 11351,'RVSN' from dual union all
select 11352,'RVSN' from dual union all
select 11353,'USN' from dual union all
select 11354,'RVSN' from dual union all
select 11355,'RVSN' from dual union all
select 11356,'NPOE' from dual union all
select 11357,'RVSN' from dual union all
select 11358,'RVSN' from dual union all
select 11359,'RVSN' from dual union all
select 1136,'OKB1' from dual union all
select 11360,'MVD' from dual union all
select 11361,'RVSN' from dual union all
select 11362,'KBSAL' from dual union all
select 11363,'KBSAL' from dual union all
select 11364,'RVSN' from dual union all
select 11365,'KBSAL' from dual union all
select 11366,'KBSAL' from dual union all
select 11367,'VMF' from dual union all
select 11368,'MVD' from dual union all
select 11369,'RVSN' from dual union all
select 1137,'OKB1' from dual union all
select 11370,'RVSN' from dual union all
select 11371,'RVSN' from dual union all
select 11372,'NRO' from dual union all
select 11373,'RVSN' from dual union all
select 11374,'RVSN' from dual union all
select 11375,'RVSN' from dual union all
select 11376,'RVSN' from dual union all
select 11377,'RVSN' from dual union all
select 11378,'VMF' from dual union all
select 11379,'RVSN' from dual union all
select 1138,'OKB1' from dual union all
select 11380,'RVSN' from dual union all
select 11381,'RVSN' from dual union all
select 11382,'SRC' from dual union all
select 11383,'NASA' from dual union all
select 11384,'MOM' from dual union all
select 11384,'MSVYAZ' from dual union all
select 11385,'RVSN' from dual union all
select 11386,'RVSN' from dual union all
select 11387,'NPOE' from dual union all
select 11388,'RVSN' from dual union all
select 11389,'USAF' from dual union all
select 1139,'OKB1' from dual union all
select 11390,'TSSKB' from dual union all
select 11391,'RVSN' from dual union all
select 11392,'ISRO' from dual union all
select 11393,'RVSN' from dual union all
select 11394,'MVD' from dual union all
select 11395,'RVSN' from dual union all
select 11396,'RVSN' from dual union all
select 11397,'USAF' from dual union all
select 11398,'USAF' from dual union all
select 11399,'MVD' from dual union all
select 114,'OKB1' from dual union all
select 1140,'OKB1' from dual union all
select 11400,'RVSN' from dual union all
select 11401,'RVSN' from dual;
insert into SATELLITE_ORG
select 11402,'RVSN' from dual union all
select 11403,'USAF' from dual union all
select 11404,'RVSN' from dual union all
select 11405,'RVSN' from dual union all
select 11406,'RVSN' from dual union all
select 11407,'RVSN' from dual union all
select 11408,'USAF' from dual union all
select 11409,'RVSN' from dual union all
select 1141,'OKB1' from dual union all
select 11410,'USAF' from dual union all
select 11411,'USAF' from dual union all
select 11412,'USAF' from dual union all
select 11413,'MVD' from dual union all
select 11414,'RVSN' from dual union all
select 11415,'TSSKB' from dual union all
select 11416,'NOAA' from dual union all
select 11417,'PVO' from dual union all
select 11418,'RVSN' from dual union all
select 11419,'NASA' from dual union all
select 1142,'OKB1' from dual union all
select 11420,'RVSN' from dual union all
select 11421,'NPOE' from dual union all
select 11422,'RVSN' from dual union all
select 11423,'RVSN' from dual union all
select 11424,'PVO' from dual union all
select 11425,'RVSN' from dual union all
select 11426,'PVO' from dual union all
select 11427,'RVSN' from dual union all
select 11428,'RVSN' from dual union all
select 11429,'RVSN' from dual union all
select 1143,'OKB1' from dual union all
select 11430,'RVSN' from dual union all
select 11431,'NPOE' from dual union all
select 11432,'NPOE' from dual union all
select 11433,'NPOE' from dual union all
select 11434,'NPOE' from dual union all
select 11435,'NPOE' from dual union all
select 11436,'USAF' from dual union all
select 11437,'RVSN' from dual union all
select 11438,'RVSN' from dual union all
select 11439,'RVSN' from dual union all
select 1144,'OKB1' from dual union all
select 11440,'MSVYAZ' from dual union all
select 11441,'RVSN' from dual union all
select 11442,'RVSN' from dual union all
select 11443,'PVO' from dual union all
select 11444,'RVSN' from dual union all
select 11445,'PVO' from dual union all
select 11446,'PVO' from dual union all
select 11447,'RVSN' from dual union all
select 11448,'RVSN' from dual union all
select 11449,'VS' from dual union all
select 1145,'OKB1' from dual union all
select 11450,'RVSN' from dual union all
select 11451,'MVD' from dual union all
select 11452,'RVSN' from dual union all
select 11453,'RVSN' from dual union all
select 11454,'RVSN' from dual union all
select 11455,'RVSN' from dual union all
select 11456,'RVSN' from dual union all
select 11457,'VS' from dual union all
select 11458,'RVSN' from dual union all
select 11459,'RVSN' from dual union all
select 1146,'OKB1' from dual union all
select 11460,'RVSN' from dual union all
select 11461,'RVSN' from dual union all
select 11462,'RVSN' from dual union all
select 11463,'RVSN' from dual union all
select 11464,'RVSN' from dual union all
select 11465,'MVD' from dual union all
select 11466,'RVSN' from dual union all
select 11467,'NPOE' from dual union all
select 11468,'NPOE' from dual union all
select 11469,'NPOE' from dual union all
select 1147,'OKB1' from dual union all
select 11470,'NPOE' from dual union all
select 11471,'NPOE' from dual union all
select 11472,'NPOE' from dual union all
select 11473,'NPOE' from dual union all
select 11474,'RVSN' from dual union all
select 11475,'RVSN' from dual union all
select 11476,'RVSN' from dual union all
select 11477,'RVSN' from dual union all
select 11478,'VTS' from dual union all
select 11479,'RVSN' from dual union all
select 1148,'OKB1' from dual union all
select 11480,'NPOE' from dual union all
select 11481,'RVSN' from dual union all
select 11482,'RVSN' from dual union all
select 11483,'RVSN' from dual union all
select 11484,'WUTC' from dual union all
select 11485,'RVSN' from dual union all
select 11486,'RVSN' from dual union all
select 11487,'RVSN' from dual union all
select 11488,'RVSN' from dual union all
select 11489,'NPOE' from dual union all
select 1149,'OKB1' from dual union all
select 11490,'NASA' from dual union all
select 11491,'MVD' from dual union all
select 11492,'RVSN' from dual;
insert into SATELLITE_ORG
select 11493,'NPOE' from dual union all
select 11494,'NPOE' from dual union all
select 11495,'NPOE' from dual union all
select 11496,'MVD' from dual union all
select 11497,'RVSN' from dual union all
select 11498,'RVSN' from dual union all
select 11499,'RVSN' from dual union all
select 115,'NASA' from dual union all
select 1150,'OKB1' from dual union all
select 11500,'NPOE' from dual union all
select 11501,'NPOE' from dual union all
select 11502,'NPOE' from dual union all
select 11503,'NPOE' from dual union all
select 11504,'NPOE' from dual union all
select 11505,'RVSN' from dual union all
select 11506,'RVSN' from dual union all
select 11507,'RVSN' from dual union all
select 11508,'NPOE' from dual union all
select 11509,'PVO' from dual union all
select 1151,'OKB1' from dual union all
select 11510,'RVSN' from dual union all
select 11511,'RVSN' from dual union all
select 11512,'RVSN' from dual union all
select 11513,'RVSN' from dual union all
select 11514,'PVO' from dual union all
select 11515,'RVSN' from dual union all
select 11516,'RVSN' from dual union all
select 11517,'RVSN' from dual union all
select 11518,'RVSN' from dual union all
select 11519,'RVSN' from dual union all
select 1152,'OKB1' from dual union all
select 11520,'MVD' from dual union all
select 11521,'RVSN' from dual union all
select 11522,'RVSN' from dual union all
select 11523,'NPOE' from dual union all
select 11524,'RVSN' from dual union all
select 11525,'RVSN' from dual union all
select 11526,'RVSN' from dual union all
select 11527,'RVSN' from dual union all
select 11528,'RVSN' from dual union all
select 11529,'RVSN' from dual union all
select 1153,'OKB1' from dual union all
select 11530,'RVSN' from dual union all
select 11531,'RVSN' from dual union all
select 11532,'NASA' from dual union all
select 11533,'NASA' from dual union all
select 11534,'USAF' from dual union all
select 11535,'PVO' from dual union all
select 11536,'IMBP' from dual union all
select 11536,'TSSKB' from dual union all
select 11537,'RVSN' from dual union all
select 11538,'RVSN' from dual union all
select 11539,'RVSN' from dual union all
select 1154,'OKB1' from dual union all
select 11540,'RVSN' from dual union all
select 11541,'RVSN' from dual union all
select 11542,'RVSN' from dual union all
select 11543,'RVSN' from dual union all
select 11544,'RVSN' from dual union all
select 11545,'RVSN' from dual union all
select 11546,'RVSN' from dual union all
select 11547,'RVSN' from dual union all
select 11548,'RVSN' from dual union all
select 11549,'RVSN' from dual union all
select 1155,'OKB1' from dual union all
select 11550,'RVSN' from dual union all
select 11551,'RVSN' from dual union all
select 11552,'RVSN' from dual union all
select 11553,'RVSN' from dual union all
select 11554,'RVSN' from dual union all
select 11555,'RVSN' from dual union all
select 11556,'RVSN' from dual union all
select 11557,'PVO' from dual union all
select 11558,'NRO' from dual union all
select 11559,'USAF' from dual union all
select 1156,'OKB1' from dual union all
select 11560,'USAF' from dual union all
select 11561,'MSVYAZ' from dual union all
select 11562,'RVSN' from dual union all
select 11563,'NPOE' from dual union all
select 11564,'VTS' from dual union all
select 11565,'RVSN' from dual union all
select 11566,'VTS' from dual union all
select 11567,'RVSN' from dual union all
select 11568,'RVSN' from dual union all
select 11569,'RVSN' from dual union all
select 1157,'OKB1' from dual union all
select 11570,'RVSN' from dual union all
select 11571,'RVSN' from dual union all
select 11572,'NPOE' from dual union all
select 11573,'RVSN' from dual union all
select 11574,'RVSN' from dual union all
select 11575,'RVSN' from dual union all
select 11576,'RVSN' from dual union all
select 11577,'RVSN' from dual union all
select 11578,'RVSN' from dual union all
select 11579,'RVSN' from dual union all
select 1158,'OKB1' from dual union all
select 11580,'RVSN' from dual union all
select 11581,'RVSN' from dual;
insert into SATELLITE_ORG
select 11582,'RVSN' from dual union all
select 11583,'PVO' from dual union all
select 11584,'PVO' from dual union all
select 11585,'VMF' from dual union all
select 11586,'RVSN' from dual union all
select 11587,'RVSN' from dual union all
select 11588,'RVSN' from dual union all
select 11589,'RVSN' from dual union all
select 1159,'OKB1' from dual union all
select 11590,'RVSN' from dual union all
select 11591,'RVSN' from dual union all
select 11592,'RVSN' from dual union all
select 11593,'RVSN' from dual union all
select 11594,'PVO' from dual union all
select 11595,'PVO' from dual union all
select 11596,'PVO' from dual union all
select 11597,'PVO' from dual union all
select 11598,'PVO' from dual union all
select 11599,'PVO' from dual union all
select 116,'USN' from dual union all
select 1160,'OKB1' from dual union all
select 11600,'VS' from dual union all
select 11601,'RVSN' from dual union all
select 11602,'RVSN' from dual union all
select 11603,'PVO' from dual union all
select 11604,'NASA' from dual union all
select 11605,'GMS' from dual union all
select 11605,'RVSN' from dual union all
select 11606,'NASA' from dual union all
select 11607,'NASDA' from dual union all
select 11608,'RVSN' from dual union all
select 11609,'IK' from dual union all
select 1161,'OKB1' from dual union all
select 11610,'RVSN' from dual union all
select 11611,'RVSN' from dual union all
select 11612,'RVSN' from dual union all
select 11613,'RVSN' from dual union all
select 11614,'RVSN' from dual union all
select 11615,'RVSN' from dual union all
select 11616,'RVSN' from dual union all
select 11617,'PVO' from dual union all
select 11618,'PVO' from dual union all
select 11619,'PVO' from dual union all
select 1162,'OKB1' from dual union all
select 11620,'PVO' from dual union all
select 11621,'DOD' from dual union all
select 11621,'USAF' from dual union all
select 11622,'DOD' from dual union all
select 11622,'USAF' from dual union all
select 11623,'USAF' from dual union all
select 11624,'USAF' from dual union all
select 11625,'PVO' from dual union all
select 11626,'PVO' from dual union all
select 11627,'PVO' from dual union all
select 11628,'PVO' from dual union all
select 11629,'VS' from dual union all
select 1163,'OKB1' from dual union all
select 11630,'RVSN' from dual union all
select 11631,'USAF' from dual union all
select 11632,'PVO' from dual union all
select 11633,'RVSN' from dual union all
select 11634,'NASA' from dual union all
select 11635,'AMC' from dual union all
select 11636,'NASA' from dual union all
select 11637,'NASA' from dual union all
select 11638,'RVSN' from dual union all
select 11639,'RVSN' from dual union all
select 1164,'OKB1' from dual union all
select 11640,'NPOE' from dual union all
select 11641,'RVSN' from dual union all
select 11642,'NPOE' from dual union all
select 11643,'PVO' from dual union all
select 11644,'PVO' from dual union all
select 11645,'ESA' from dual union all
select 11646,'RVSN' from dual union all
select 11647,'RVSN' from dual union all
select 11648,'MSVYAZ' from dual union all
select 11649,'RVSN' from dual union all
select 1165,'OKB1' from dual union all
select 11650,'RVSN' from dual union all
select 11651,'RVSN' from dual union all
select 11652,'RVSN' from dual union all
select 11653,'RVSN' from dual union all
select 11654,'RVSN' from dual union all
select 11655,'RVSN' from dual union all
select 11656,'RVSN' from dual union all
select 11657,'RVSN' from dual union all
select 11658,'RVSN' from dual union all
select 11659,'ESA' from dual union all
select 1166,'OKB1' from dual union all
select 11660,'RVSN' from dual union all
select 11661,'RVSN' from dual union all
select 11662,'RVSN' from dual union all
select 11663,'RVSN' from dual union all
select 11664,'RVSN' from dual union all
select 11665,'RVSN' from dual union all
select 11666,'RVSN' from dual union all
select 11667,'VMF' from dual union all
select 11668,'RVSN' from dual union all
select 11669,'USN' from dual;
insert into SATELLITE_ORG
select 1167,'OKB1' from dual union all
select 11670,'RVSN' from dual union all
select 11671,'GMS' from dual union all
select 11671,'YUZH' from dual union all
select 11672,'RVSN' from dual union all
select 11673,'RVSN' from dual union all
select 11674,'RVSN' from dual union all
select 11675,'RVSN' from dual union all
select 11676,'RVSN' from dual union all
select 11677,'RVSN' from dual union all
select 11678,'RVSN' from dual union all
select 11679,'RVSN' from dual union all
select 1168,'OKB1' from dual union all
select 11680,'VMF' from dual union all
select 11681,'RVSN' from dual union all
select 11682,'VS' from dual union all
select 11683,'RVSN' from dual union all
select 11684,'RVSN' from dual union all
select 11685,'RVSN' from dual union all
select 11686,'RVSN' from dual union all
select 11687,'CIA' from dual union all
select 11687,'NRO' from dual union all
select 11688,'USAF' from dual union all
select 11689,'NRO' from dual union all
select 1169,'OKB1' from dual union all
select 11690,'USAF' from dual union all
select 11691,'RVSN' from dual union all
select 11692,'RVSN' from dual union all
select 11693,'RVSN' from dual union all
select 11694,'RVSN' from dual union all
select 11695,'RVSN' from dual union all
select 11696,'RVSN' from dual union all
select 11697,'RVSN' from dual union all
select 11698,'RVSN' from dual union all
select 11699,'RVSN' from dual union all
select 117,'USN' from dual union all
select 1170,'OKB1' from dual union all
select 11700,'PVO' from dual union all
select 11701,'RVSN' from dual union all
select 11702,'RVSN' from dual union all
select 11703,'NASA' from dual union all
select 11704,'NASA' from dual union all
select 11705,'USAF' from dual union all
select 11706,'ISAS' from dual union all
select 11707,'ISAS' from dual union all
select 11708,'RVSN' from dual union all
select 11709,'RVSN' from dual union all
select 1171,'OKB1' from dual union all
select 11710,'RVSN' from dual union all
select 11711,'RVSN' from dual union all
select 11712,'RVSN' from dual union all
select 11713,'RVSN' from dual union all
select 11714,'RVSN' from dual union all
select 11715,'NASDA' from dual union all
select 11716,'NASDA' from dual union all
select 11717,'RVSN' from dual union all
select 11718,'NASDA' from dual union all
select 11719,'NASA' from dual union all
select 1172,'OKB1' from dual union all
select 11720,'NRO' from dual union all
select 11720,'USN' from dual union all
select 11721,'USN' from dual union all
select 11722,'RVSN' from dual union all
select 11723,'RVSN' from dual union all
select 11724,'RVSN' from dual union all
select 11725,'RVSN' from dual union all
select 11726,'RVSN' from dual union all
select 11727,'RVSN' from dual union all
select 11728,'RVSN' from dual union all
select 11729,'VMF' from dual union all
select 1173,'OKB1' from dual union all
select 11730,'RVSN' from dual union all
select 11731,'NRO' from dual union all
select 11731,'USN' from dual union all
select 11732,'NRO' from dual union all
select 11732,'USN' from dual union all
select 11733,'USN' from dual union all
select 11734,'NRO' from dual union all
select 11734,'USN' from dual union all
select 11735,'VMF' from dual union all
select 11736,'RVSN' from dual union all
select 11737,'RVSN' from dual union all
select 11738,'RVSN' from dual union all
select 11739,'RVSN' from dual union all
select 1174,'OKB1' from dual union all
select 11740,'NPOE' from dual union all
select 11741,'PVO' from dual union all
select 11742,'RVSN' from dual union all
select 11743,'NPOE' from dual union all
select 11744,'RVSN' from dual union all
select 11745,'NRO' from dual union all
select 11745,'USN' from dual union all
select 11746,'USN' from dual union all
select 11747,'RVSN' from dual union all
select 11748,'RVSN' from dual union all
select 11749,'RVSN' from dual union all
select 1175,'OKB1' from dual union all
select 11750,'PVO' from dual union all
select 11751,'RVSN' from dual union all
select 11752,'PVO' from dual;
insert into SATELLITE_ORG
select 11753,'NPOE' from dual union all
select 11754,'RVSN' from dual union all
select 11755,'RVSN' from dual union all
select 11756,'RVSN' from dual union all
select 11757,'RVSN' from dual union all
select 11758,'PVO' from dual union all
select 11759,'RVSN' from dual union all
select 1176,'OKB1' from dual union all
select 11760,'RVSN' from dual union all
select 11761,'RVSN' from dual union all
select 11762,'RVSN' from dual union all
select 11763,'RVSN' from dual union all
select 11764,'RVSN' from dual union all
select 11765,'PVO' from dual union all
select 11766,'RVSN' from dual union all
select 11767,'RVSN' from dual union all
select 11768,'RVSN' from dual union all
select 11769,'MOM' from dual union all
select 11769,'MSVYAZ' from dual union all
select 1177,'OKB1' from dual union all
select 11770,'RVSN' from dual union all
select 11771,'PVO' from dual union all
select 11772,'PVO' from dual union all
select 11773,'PVO' from dual union all
select 11774,'PVO' from dual union all
select 11775,'PVO' from dual union all
select 11776,'PVO' from dual union all
select 11777,'PVO' from dual union all
select 11778,'PVO' from dual union all
select 11779,'PVO' from dual union all
select 1178,'OKB1' from dual union all
select 11780,'PVO' from dual union all
select 11781,'PVO' from dual union all
select 11782,'PVO' from dual union all
select 11783,'USAF' from dual union all
select 11784,'NPOE' from dual union all
select 11785,'RVSN' from dual union all
select 11786,'RVSN' from dual union all
select 11787,'RVSN' from dual union all
select 11788,'VMF' from dual union all
select 11789,'TSSKB' from dual union all
select 1179,'OKB1' from dual union all
select 11790,'RVSN' from dual union all
select 11791,'USAF' from dual union all
select 11792,'NASA' from dual union all
select 11793,'RVSN' from dual union all
select 11794,'RVSN' from dual union all
select 11795,'RVSN' from dual union all
select 11796,'PVO' from dual union all
select 11797,'RVSN' from dual union all
select 11798,'VTS' from dual union all
select 11799,'RVSN' from dual union all
select 118,'USAF' from dual union all
select 1180,'OKB1' from dual union all
select 11800,'RVSN' from dual union all
select 11801,'RVSN' from dual union all
select 11802,'RVSN' from dual union all
select 11803,'VMF' from dual union all
select 11804,'RVSN' from dual union all
select 11805,'RVSN' from dual union all
select 11806,'RVSN' from dual union all
select 11807,'RVSN' from dual union all
select 11808,'MVD' from dual union all
select 11809,'RVSN' from dual union all
select 1181,'OKB1' from dual union all
select 11810,'ISAS' from dual union all
select 11811,'NPOE' from dual union all
select 11812,'RVSN' from dual union all
select 11813,'RVSN' from dual union all
select 11814,'RVSN' from dual union all
select 11815,'RVSN' from dual union all
select 11816,'RVSN' from dual union all
select 11817,'RVSN' from dual union all
select 11818,'RVSN' from dual union all
select 11819,'NOAA' from dual union all
select 1182,'OKB1' from dual union all
select 11820,'NASA' from dual union all
select 11821,'VS' from dual union all
select 11822,'RVSN' from dual union all
select 11823,'RVSN' from dual union all
select 11824,'RVSN' from dual union all
select 11825,'NPOE' from dual union all
select 11826,'RVSN' from dual union all
select 11827,'MVD' from dual union all
select 11828,'RVSN' from dual union all
select 11829,'PVO' from dual union all
select 1183,'OKB1' from dual union all
select 11830,'RVSN' from dual union all
select 11831,'PVO' from dual union all
select 11832,'NPOE' from dual union all
select 11833,'RVSN' from dual union all
select 11834,'RVSN' from dual union all
select 11835,'RVSN' from dual union all
select 11836,'RVSN' from dual union all
select 11837,'RVSN' from dual union all
select 11838,'RVSN' from dual union all
select 11839,'RVSN' from dual union all
select 1184,'OKB1' from dual union all
select 11840,'RVSN' from dual union all
select 11841,'MSVYAZ' from dual;
insert into SATELLITE_ORG
select 11842,'RVSN' from dual union all
select 11843,'RVSN' from dual union all
select 11844,'PVO' from dual union all
select 11845,'RVSN' from dual union all
select 11846,'RVSN' from dual union all
select 11847,'RVSN' from dual union all
select 11848,'MVD' from dual union all
select 11849,'RVSN' from dual union all
select 1185,'OKB1' from dual union all
select 11850,'NRO' from dual union all
select 11851,'USAF' from dual union all
select 11852,'NRO' from dual union all
select 11852,'USAF' from dual union all
select 11853,'RVSN' from dual union all
select 11854,'RVSN' from dual union all
select 11855,'USN' from dual union all
select 11856,'RVSN' from dual union all
select 11857,'RVSN' from dual union all
select 11858,'RVSN' from dual union all
select 11859,'RVSN' from dual union all
select 1186,'OKB1' from dual union all
select 11860,'RVSN' from dual union all
select 11861,'RVSN' from dual union all
select 11862,'RVSN' from dual union all
select 11863,'RVSN' from dual union all
select 11864,'RVSN' from dual union all
select 11865,'RVSN' from dual union all
select 11866,'RVSN' from dual union all
select 11867,'NPOE' from dual union all
select 11868,'RVSN' from dual union all
select 11869,'RVSN' from dual union all
select 1187,'OKB1' from dual union all
select 11870,'RVSN' from dual union all
select 11871,'PVO' from dual union all
select 11872,'RVSN' from dual union all
select 11873,'RVSN' from dual union all
select 11874,'NASA' from dual union all
select 11875,'RVSN' from dual union all
select 11876,'RVSN' from dual union all
select 11877,'RVSN' from dual union all
select 11878,'RVSN' from dual union all
select 11879,'RVSN' from dual union all
select 1188,'OKB1' from dual union all
select 11880,'RVSN' from dual union all
select 11881,'RVSN' from dual union all
select 11882,'RVSN' from dual union all
select 11883,'RVSN' from dual union all
select 11884,'RVSN' from dual union all
select 11885,'RVSN' from dual union all
select 11886,'RVSN' from dual union all
select 11887,'RVSN' from dual union all
select 11888,'RVSN' from dual union all
select 11889,'RVSN' from dual union all
select 1189,'OKB1' from dual union all
select 11890,'MSVYAZ' from dual union all
select 11891,'RVSN' from dual union all
select 11892,'RVSN' from dual union all
select 11893,'NPOE' from dual union all
select 11894,'MVD' from dual union all
select 11895,'RVSN' from dual union all
select 11896,'MOM' from dual union all
select 11896,'MSVYAZ' from dual union all
select 11897,'RVSN' from dual union all
select 11898,'RVSN' from dual union all
select 11899,'ISRO' from dual union all
select 119,'USAF' from dual union all
select 1190,'OKB1' from dual union all
select 11900,'ISRO' from dual union all
select 11901,'RVSN' from dual union all
select 11902,'RVSN' from dual union all
select 11903,'RVSN' from dual union all
select 11904,'RVSN' from dual union all
select 11905,'NPOE' from dual union all
select 11906,'RVSN' from dual union all
select 11907,'RVSN' from dual union all
select 11908,'RVSN' from dual union all
select 11909,'RVSN' from dual union all
select 1191,'OKB1F3' from dual union all
select 11910,'RVSN' from dual union all
select 11911,'RVSN' from dual union all
select 11912,'PVO' from dual union all
select 11913,'PVO' from dual union all
select 11914,'RVSN' from dual union all
select 11915,'MVD' from dual union all
select 11916,'RVSN' from dual union all
select 11917,'PVO' from dual union all
select 11918,'RVSN' from dual union all
select 11919,'RVSN' from dual union all
select 1192,'RVSN' from dual union all
select 11920,'RVSN' from dual union all
select 11921,'RVSN' from dual union all
select 11922,'RVSN' from dual union all
select 11923,'RVSN' from dual union all
select 11924,'RVSN' from dual union all
select 11925,'RVSN' from dual union all
select 11926,'RVSN' from dual union all
select 11927,'RVSN' from dual union all
select 11928,'RVSN' from dual union all
select 11929,'RVSN' from dual union all
select 1193,'CIA' from dual;
insert into SATELLITE_ORG
select 1193,'NRO' from dual union all
select 11930,'NPOE' from dual union all
select 11931,'NPOE' from dual union all
select 11932,'VS' from dual union all
select 11933,'RVSN' from dual union all
select 11934,'NPOE' from dual union all
select 11935,'NPOE' from dual union all
select 11936,'NPOE' from dual union all
select 11937,'NPOE' from dual union all
select 11938,'MVD' from dual union all
select 11939,'RVSN' from dual union all
select 1194,'OKB1' from dual union all
select 11940,'USAF' from dual union all
select 11941,'RVSN' from dual union all
select 11942,'NPOE' from dual union all
select 11943,'RVSN' from dual union all
select 11944,'RVSN' from dual union all
select 11945,'RVSN' from dual union all
select 11946,'RVSN' from dual union all
select 11947,'NPOE' from dual union all
select 11948,'ISAS' from dual union all
select 11949,'RVSN' from dual union all
select 1195,'OKB1' from dual union all
select 11950,'MVD' from dual union all
select 11951,'RVSN' from dual union all
select 11952,'USN' from dual union all
select 11953,'USN' from dual union all
select 11954,'RVSN' from dual union all
select 11955,'NPOE' from dual union all
select 11956,'NPOE' from dual union all
select 11957,'NPOE' from dual union all
select 11958,'NPOE' from dual union all
select 11959,'RVSN' from dual union all
select 1196,'OKB1' from dual union all
select 11960,'PVO' from dual union all
select 11961,'PVO' from dual union all
select 11962,'GMS' from dual union all
select 11962,'RVSN' from dual union all
select 11963,'RVSN' from dual union all
select 11964,'NOAA' from dual union all
select 11965,'NASA' from dual union all
select 11966,'NPOE' from dual union all
select 11967,'NPOE' from dual union all
select 11968,'VMF' from dual union all
select 11969,'VMF' from dual union all
select 1197,'OKB1' from dual union all
select 11970,'NASA' from dual union all
select 11971,'VMF' from dual union all
select 11972,'NPOE' from dual union all
select 11973,'NPOE' from dual union all
select 11974,'NPOE' from dual union all
select 11975,'NPOE' from dual union all
select 11976,'RVSN' from dual union all
select 11977,'NPOE' from dual union all
select 11978,'RVSN' from dual union all
select 11979,'NPOE' from dual union all
select 1198,'OKB1' from dual union all
select 11980,'RVSN' from dual union all
select 11981,'RVSN' from dual union all
select 11982,'VTS' from dual union all
select 11983,'RVSN' from dual union all
select 11984,'NASA' from dual union all
select 11985,'MVD' from dual union all
select 11986,'RVSN' from dual union all
select 11987,'PVO' from dual union all
select 11988,'PVO' from dual union all
select 11989,'PVO' from dual union all
select 1199,'OKB1' from dual union all
select 11990,'USN' from dual union all
select 11991,'USN' from dual union all
select 11992,'PVO' from dual union all
select 11993,'NPOE' from dual union all
select 11994,'RVSN' from dual union all
select 11995,'RVSN' from dual union all
select 11996,'RVSN' from dual union all
select 11997,'RVSN' from dual union all
select 11998,'RVSN' from dual union all
select 11999,'RVSN' from dual union all
select 12,'NASA' from dual union all
select 120,'USAF' from dual union all
select 1200,'OKB1' from dual union all
select 12000,'NASA' from dual union all
select 12001,'RVSN' from dual union all
select 12002,'RVSN' from dual union all
select 12003,'RVSN' from dual union all
select 12004,'RVSN' from dual union all
select 12005,'RVSN' from dual union all
select 12006,'RVSN' from dual union all
select 12007,'RVSN' from dual union all
select 12008,'RVSN' from dual union all
select 12009,'RVSN' from dual union all
select 1201,'OKB1' from dual union all
select 12010,'PVO' from dual union all
select 12011,'PVO' from dual union all
select 12012,'RVSN' from dual union all
select 12013,'RVSN' from dual union all
select 12014,'PVO' from dual union all
select 12015,'PVO' from dual union all
select 12016,'VS' from dual union all
select 12017,'RVSN' from dual;
insert into SATELLITE_ORG
select 12018,'VS' from dual union all
select 12019,'RVSN' from dual union all
select 1202,'OKB1' from dual union all
select 12020,'RVSN' from dual union all
select 12021,'RVSN' from dual union all
select 12022,'RVSN' from dual union all
select 12023,'RVSN' from dual union all
select 12024,'RVSN' from dual union all
select 12025,'RVSN' from dual union all
select 12026,'VS' from dual union all
select 12027,'IKI' from dual union all
select 12027,'NPOL' from dual union all
select 12028,'IKI' from dual union all
select 12028,'NPOL' from dual union all
select 12029,'RVSN' from dual union all
select 1203,'OKB1' from dual union all
select 12030,'RVSN' from dual union all
select 12031,'RVSN' from dual union all
select 12032,'PVO' from dual union all
select 12033,'RVSN' from dual union all
select 12034,'RVSN' from dual union all
select 12035,'RVSN' from dual union all
select 12036,'PVO' from dual union all
select 12037,'PVO' from dual union all
select 12038,'PVO' from dual union all
select 12039,'TSSKB' from dual union all
select 1204,'OKB1' from dual union all
select 12040,'RVSN' from dual union all
select 12041,'RVSN' from dual union all
select 12042,'RVSN' from dual union all
select 12043,'RVSN' from dual union all
select 12044,'RVSN' from dual union all
select 12045,'RVSN' from dual union all
select 12046,'USN' from dual union all
select 12047,'PVO' from dual union all
select 12048,'PVO' from dual union all
select 12049,'PVO' from dual union all
select 1205,'OKB1' from dual union all
select 12050,'RVSN' from dual union all
select 12051,'RVSN' from dual union all
select 12052,'PVO' from dual union all
select 12053,'PVO' from dual union all
select 12054,'VMF' from dual union all
select 12055,'RVSN' from dual union all
select 12056,'PVO' from dual union all
select 12057,'PVO' from dual union all
select 12058,'RVSN' from dual union all
select 12059,'RVSN' from dual union all
select 1206,'OKB1' from dual union all
select 12060,'RVSN' from dual union all
select 12061,'RVSN' from dual union all
select 12062,'RVSN' from dual union all
select 12063,'RVSN' from dual union all
select 12064,'RVSN' from dual union all
select 12065,'SBS' from dual union all
select 12066,'RVSN' from dual union all
select 12067,'RVSN' from dual union all
select 12068,'RVSN' from dual union all
select 12069,'NASA' from dual union all
select 1207,'OKB1' from dual union all
select 12070,'RVSN' from dual union all
select 12071,'VS' from dual union all
select 12072,'RVSN' from dual union all
select 12073,'RVSN' from dual union all
select 12074,'PVO' from dual union all
select 12075,'PVO' from dual union all
select 12076,'RVSN' from dual union all
select 12077,'NPOE' from dual union all
select 12078,'PVO' from dual union all
select 12079,'RVSN' from dual union all
select 1208,'USA' from dual union all
select 12080,'RVSN' from dual union all
select 12081,'RVSN' from dual union all
select 12082,'RVSN' from dual union all
select 12083,'RVSN' from dual union all
select 12084,'RVSN' from dual union all
select 12085,'RVSN' from dual union all
select 12086,'RVSN' from dual union all
select 12087,'VMF' from dual union all
select 12088,'RVSN' from dual union all
select 12089,'INTEL' from dual union all
select 1209,'OKB1' from dual union all
select 12090,'NPOE' from dual union all
select 12091,'VMF' from dual union all
select 12092,'RVSN' from dual union all
select 12093,'NRO' from dual union all
select 12094,'USAF' from dual union all
select 12095,'RVSN' from dual union all
select 12096,'RVSN' from dual union all
select 12097,'RVSN' from dual union all
select 12098,'PVO' from dual union all
select 12099,'USAF' from dual union all
select 121,'USAF' from dual union all
select 1210,'OKB1' from dual union all
select 12100,'RVSN' from dual union all
select 12101,'RVSN' from dual union all
select 12102,'USAF' from dual union all
select 12103,'NASA' from dual union all
select 12104,'NASA' from dual union all
select 12105,'NASA' from dual;
insert into SATELLITE_ORG
select 12106,'NASA' from dual union all
select 12107,'RVSN' from dual union all
select 12108,'RVSN' from dual union all
select 12109,'RVSN' from dual union all
select 1211,'OKB1' from dual union all
select 12110,'RVSN' from dual union all
select 12111,'RVSN' from dual union all
select 12112,'RVSN' from dual union all
select 12113,'RVSN' from dual union all
select 12114,'RVSN' from dual union all
select 12115,'RVSN' from dual union all
select 12116,'IKI' from dual union all
select 12116,'NPOL' from dual union all
select 12117,'RVSN' from dual union all
select 12118,'RVSN' from dual union all
select 12119,'RVSN' from dual union all
select 1212,'OKB1' from dual union all
select 12120,'MSVYAZ' from dual union all
select 12121,'RVSN' from dual union all
select 12122,'RVSN' from dual union all
select 12123,'RVSN' from dual union all
select 12124,'RVSN' from dual union all
select 12125,'NASA' from dual union all
select 12126,'RVSN' from dual union all
select 12127,'RVSN' from dual union all
select 12128,'RVSN' from dual union all
select 12129,'RVSN' from dual union all
select 1213,'OKB1' from dual union all
select 12130,'MO' from dual union all
select 12131,'RVSN' from dual union all
select 12132,'MO' from dual union all
select 12133,'MOM' from dual union all
select 12133,'MSVYAZ' from dual union all
select 12134,'RVSN' from dual union all
select 12135,'RVSN' from dual union all
select 12136,'RVSN' from dual union all
select 12137,'RVSN' from dual union all
select 12138,'PVO' from dual union all
select 12139,'RVSN' from dual union all
select 1214,'OKB1' from dual union all
select 12140,'VTS' from dual union all
select 12141,'RVSN' from dual union all
select 12142,'USN' from dual union all
select 12143,'MO' from dual union all
select 12144,'RVSN' from dual union all
select 12145,'MO' from dual union all
select 12146,'MO' from dual union all
select 12147,'MO' from dual union all
select 12148,'MO' from dual union all
select 12149,'PVO' from dual union all
select 1215,'OKB1' from dual union all
select 12150,'RVSN' from dual union all
select 12151,'PVO' from dual union all
select 12152,'NPOE' from dual union all
select 12153,'RVSN' from dual union all
select 12154,'VS' from dual union all
select 12155,'RVSN' from dual union all
select 12156,'MO' from dual union all
select 12157,'RVSN' from dual union all
select 12158,'RVSN' from dual union all
select 12159,'RVSN' from dual union all
select 1216,'OKB1' from dual union all
select 12160,'PVO' from dual union all
select 12161,'RVSN' from dual union all
select 12162,'IK' from dual union all
select 12163,'RVSN' from dual union all
select 12164,'NASA' from dual union all
select 12165,'NASA' from dual union all
select 12166,'NASA' from dual union all
select 12167,'NASA' from dual union all
select 12168,'NASA' from dual union all
select 12169,'NASA' from dual union all
select 1217,'OKB1' from dual union all
select 12170,'NASA' from dual union all
select 12171,'NASA' from dual union all
select 12172,'NASA' from dual union all
select 12173,'NASA' from dual union all
select 12174,'NASA' from dual union all
select 12175,'NASA' from dual union all
select 12176,'NASA' from dual union all
select 12177,'NASA' from dual union all
select 12178,'NASA' from dual union all
select 12179,'NASA' from dual union all
select 1218,'OKB1' from dual union all
select 12180,'NASA' from dual union all
select 12181,'NASA' from dual union all
select 12182,'NASA' from dual union all
select 12183,'NASA' from dual union all
select 12184,'NASA' from dual union all
select 12185,'NASA' from dual union all
select 12186,'NASA' from dual union all
select 12187,'NASA' from dual union all
select 12188,'NASA' from dual union all
select 12189,'NASA' from dual union all
select 1219,'OKB1' from dual union all
select 12190,'NASA' from dual union all
select 12191,'NASA' from dual union all
select 12192,'NASA' from dual union all
select 12193,'NASA' from dual union all
select 12194,'NASA' from dual;
insert into SATELLITE_ORG
select 12195,'NASA' from dual union all
select 12196,'NASA' from dual union all
select 12197,'NASA' from dual union all
select 12198,'NASA' from dual union all
select 12199,'NASA' from dual union all
select 122,'USAF' from dual union all
select 1220,'OKB1' from dual union all
select 12200,'NASA' from dual union all
select 12201,'NASA' from dual union all
select 12202,'NASA' from dual union all
select 12203,'NASA' from dual union all
select 12204,'NASA' from dual union all
select 12205,'NASA' from dual union all
select 12206,'NASA' from dual union all
select 12207,'NASA' from dual union all
select 12208,'NASA' from dual union all
select 12209,'NASA' from dual union all
select 1221,'OKB1' from dual union all
select 12210,'NASA' from dual union all
select 12211,'NASA' from dual union all
select 12212,'NASA' from dual union all
select 12213,'NASA' from dual union all
select 12214,'NASA' from dual union all
select 12215,'NASA' from dual union all
select 12216,'NASA' from dual union all
select 12217,'NASA' from dual union all
select 12218,'NASA' from dual union all
select 12219,'NASA' from dual union all
select 1222,'OKB1' from dual union all
select 12220,'NASA' from dual union all
select 12221,'NASA' from dual union all
select 12222,'NASA' from dual union all
select 12223,'NASA' from dual union all
select 12224,'NASA' from dual union all
select 12225,'NASA' from dual union all
select 12226,'NASA' from dual union all
select 12227,'NASA' from dual union all
select 12228,'NASA' from dual union all
select 12229,'NASA' from dual union all
select 1223,'OKB1' from dual union all
select 12230,'NASA' from dual union all
select 12231,'NASA' from dual union all
select 12232,'NASA' from dual union all
select 12233,'NASA' from dual union all
select 12234,'NASA' from dual union all
select 12235,'NASA' from dual union all
select 12236,'NASA' from dual union all
select 12237,'NASA' from dual union all
select 12238,'NASA' from dual union all
select 12239,'NASA' from dual union all
select 1224,'OKB1' from dual union all
select 12240,'NASA' from dual union all
select 12241,'NASA' from dual union all
select 12242,'NASA' from dual union all
select 12243,'NASA' from dual union all
select 12244,'NASA' from dual union all
select 12245,'NASA' from dual union all
select 12246,'NASA' from dual union all
select 12247,'NASA' from dual union all
select 12248,'NASA' from dual union all
select 12249,'NASA' from dual union all
select 1225,'OKB1' from dual union all
select 12250,'NASA' from dual union all
select 12251,'NASA' from dual union all
select 12252,'NASA' from dual union all
select 12253,'NASA' from dual union all
select 12254,'NASA' from dual union all
select 12255,'NASA' from dual union all
select 12256,'NASA' from dual union all
select 12257,'NASA' from dual union all
select 12258,'NASA' from dual union all
select 12259,'NASA' from dual union all
select 1226,'OKB1' from dual union all
select 12260,'NASA' from dual union all
select 12261,'NASA' from dual union all
select 12262,'NASA' from dual union all
select 12263,'NASA' from dual union all
select 12264,'NASA' from dual union all
select 12265,'NASA' from dual union all
select 12266,'NASA' from dual union all
select 12267,'NASA' from dual union all
select 12268,'NASA' from dual union all
select 12269,'NASA' from dual union all
select 1227,'OKB1' from dual union all
select 12270,'NASA' from dual union all
select 12271,'NASA' from dual union all
select 12272,'NASA' from dual union all
select 12273,'NASA' from dual union all
select 12274,'NASA' from dual union all
select 12275,'NASA' from dual union all
select 12276,'NASA' from dual union all
select 12277,'NASA' from dual union all
select 12278,'NASA' from dual union all
select 12279,'NASA' from dual union all
select 1228,'USAF' from dual union all
select 12280,'NASA' from dual union all
select 12281,'NASA' from dual union all
select 12282,'NASA' from dual union all
select 12283,'NASA' from dual union all
select 12284,'NASA' from dual;
insert into SATELLITE_ORG
select 12285,'NASA' from dual union all
select 12286,'NASA' from dual union all
select 12287,'NASA' from dual union all
select 12288,'NASA' from dual union all
select 12289,'NASA' from dual union all
select 1229,'OKB1' from dual union all
select 12290,'NASA' from dual union all
select 12291,'NASA' from dual union all
select 12292,'NASA' from dual union all
select 12293,'NASA' from dual union all
select 12294,'NASA' from dual union all
select 12295,'NASDA' from dual union all
select 12296,'NASDA' from dual union all
select 12297,'VMF' from dual union all
select 12298,'RVSN' from dual union all
select 12299,'MO' from dual union all
select 123,'USAF' from dual union all
select 1230,'OKB1' from dual union all
select 12300,'RVSN' from dual union all
select 12301,'TSSKB' from dual union all
select 12302,'RVSN' from dual union all
select 12303,'PVO' from dual union all
select 12304,'RVSN' from dual union all
select 12305,'RVSN' from dual union all
select 12306,'RVSN' from dual union all
select 12307,'ISAS' from dual union all
select 12308,'ISAS' from dual union all
select 12309,'COM' from dual union all
select 1231,'OKB1' from dual union all
select 12310,'ISAS' from dual union all
select 12311,'RVSN' from dual union all
select 12312,'MO' from dual union all
select 12313,'MO' from dual union all
select 12314,'MO' from dual union all
select 12315,'NRO' from dual union all
select 12316,'MO' from dual union all
select 12317,'MO' from dual union all
select 12318,'RVSN' from dual union all
select 12319,'VMF' from dual union all
select 1232,'OKB1' from dual union all
select 12320,'MO' from dual union all
select 12321,'MO' from dual union all
select 12322,'MO' from dual union all
select 12323,'MO' from dual union all
select 12324,'MO' from dual union all
select 12325,'MO' from dual union all
select 12326,'MO' from dual union all
select 12327,'MO' from dual union all
select 12328,'RVSN' from dual union all
select 12329,'ISAS' from dual union all
select 1233,'OKB1' from dual union all
select 12330,'ISAS' from dual union all
select 12331,'ISAS' from dual union all
select 12332,'PVO' from dual union all
select 12333,'PVO' from dual union all
select 12334,'NPOE' from dual union all
select 12335,'RVSN' from dual union all
select 12336,'NASA' from dual union all
select 12337,'PVO' from dual union all
select 12338,'RVSN' from dual union all
select 12339,'USAF' from dual union all
select 1234,'OKB1' from dual union all
select 12340,'USAF' from dual union all
select 12341,'MO' from dual union all
select 12342,'RVSN' from dual union all
select 12343,'PVO' from dual union all
select 12344,'PVO' from dual union all
select 12345,'PVO' from dual union all
select 12346,'PVO' from dual union all
select 12347,'PVO' from dual union all
select 12348,'PVO' from dual union all
select 12349,'PVO' from dual union all
select 1235,'OKB1' from dual union all
select 12350,'MO' from dual union all
select 12351,'MO' from dual union all
select 12352,'RVSN' from dual union all
select 12353,'RVSN' from dual union all
select 12354,'PVO' from dual union all
select 12355,'PVO' from dual union all
select 12356,'PVO' from dual union all
select 12357,'PVO' from dual union all
select 12358,'PVO' from dual union all
select 12359,'PVO' from dual union all
select 1236,'OKB1' from dual union all
select 12360,'PVO' from dual union all
select 12361,'PVO' from dual union all
select 12362,'PVO' from dual union all
select 12363,'NASA' from dual union all
select 12364,'VMF' from dual union all
select 12365,'RVSN' from dual union all
select 12366,'NPOE' from dual union all
select 12367,'RVSN' from dual union all
select 12368,'MOM' from dual union all
select 12368,'MSVYAZ' from dual union all
select 12369,'RVSN' from dual union all
select 1237,'OKB1' from dual union all
select 12370,'RVSN' from dual union all
select 12371,'USAF' from dual union all
select 12372,'NASA' from dual union all
select 12373,'NASA' from dual;
insert into SATELLITE_ORG
select 12374,'NASA' from dual union all
select 12375,'NASA' from dual union all
select 12376,'PVO' from dual union all
select 12377,'RVSN' from dual union all
select 12378,'RVSN' from dual union all
select 12379,'MO' from dual union all
select 1238,'OKB1' from dual union all
select 12380,'MO' from dual union all
select 12381,'MO' from dual union all
select 12382,'MO' from dual union all
select 12383,'RVSN' from dual union all
select 12384,'RVSN' from dual union all
select 12385,'MO' from dual union all
select 12386,'RVSN' from dual union all
select 12387,'MO' from dual union all
select 12388,'PVO' from dual union all
select 12389,'RVSN' from dual union all
select 1239,'OKB1' from dual union all
select 12390,'NPOE' from dual union all
select 12391,'NPOE' from dual union all
select 12392,'NPOE' from dual union all
select 12393,'NPOE' from dual union all
select 12394,'NPOE' from dual union all
select 12395,'NPOE' from dual union all
select 12396,'RVSN' from dual union all
select 12397,'RVSN' from dual union all
select 12398,'NPOE' from dual union all
select 12399,'NASA' from dual union all
select 124,'USAF' from dual union all
select 1240,'OKB1' from dual union all
select 12400,'MO' from dual union all
select 12401,'RVSN' from dual union all
select 12402,'MO' from dual union all
select 12403,'RVSN' from dual union all
select 12404,'VMF' from dual union all
select 12405,'NASA' from dual union all
select 12406,'ISAS' from dual union all
select 12407,'PVO' from dual union all
select 12408,'PVO' from dual union all
select 12409,'VMF' from dual union all
select 1241,'OKB1' from dual union all
select 12410,'MO' from dual union all
select 12411,'MO' from dual union all
select 12412,'PVO' from dual union all
select 12413,'PVO' from dual union all
select 12414,'PVO' from dual union all
select 12415,'PVO' from dual union all
select 12416,'PVO' from dual union all
select 12417,'MO' from dual union all
select 12418,'NRO' from dual union all
select 12419,'KBSAL' from dual union all
select 1242,'OKB1' from dual union all
select 12420,'RVSN' from dual union all
select 12421,'MO' from dual union all
select 12422,'PVO' from dual union all
select 12423,'MO' from dual union all
select 12424,'RVSN' from dual union all
select 12425,'MO' from dual union all
select 12426,'MO' from dual union all
select 12427,'NASA' from dual union all
select 12428,'MO' from dual union all
select 12429,'VMF' from dual union all
select 1243,'OKB1' from dual union all
select 12430,'MO' from dual union all
select 12431,'MO' from dual union all
select 12432,'MO' from dual union all
select 12433,'MO' from dual union all
select 12434,'VMF' from dual union all
select 12435,'VMF' from dual union all
select 12436,'NPOE' from dual union all
select 12437,'NPOE' from dual union all
select 12438,'NPOE' from dual union all
select 12439,'NPOE' from dual union all
select 1244,'NRO' from dual union all
select 1244,'USN' from dual union all
select 12440,'NPOE' from dual union all
select 12441,'NPOE' from dual union all
select 12442,'MO' from dual union all
select 12443,'RVSN' from dual union all
select 12444,'NPOE' from dual union all
select 12445,'NASA' from dual union all
select 12446,'USAF' from dual union all
select 12447,'RVSN' from dual union all
select 12448,'MO' from dual union all
select 12449,'MO' from dual union all
select 1245,'USAF' from dual union all
select 12450,'MO' from dual union all
select 12451,'MO' from dual union all
select 12452,'MO' from dual union all
select 12453,'PVO' from dual union all
select 12454,'NPOE' from dual union all
select 12455,'RVSN' from dual union all
select 12456,'GMS' from dual union all
select 12456,'MO' from dual union all
select 12457,'RVSN' from dual union all
select 12458,'USN' from dual union all
select 12459,'PVO' from dual union all
select 1246,'OKB1' from dual union all
select 12460,'NASA' from dual union all
select 12461,'MO' from dual;
insert into SATELLITE_ORG
select 12462,'RVSN' from dual union all
select 12463,'MO' from dual union all
select 12464,'VS' from dual union all
select 12465,'RVSN' from dual union all
select 12466,'MO' from dual union all
select 12467,'RVSN' from dual union all
select 12468,'MO' from dual union all
select 12469,'MVD' from dual union all
select 1247,'NRO' from dual union all
select 12470,'RVSN' from dual union all
select 12471,'RVSN' from dual union all
select 12472,'NOAA' from dual union all
select 12473,'NASA' from dual union all
select 12474,'INTEL' from dual union all
select 12475,'NPOE' from dual union all
select 12476,'NPOE' from dual union all
select 12477,'NPOE' from dual union all
select 12478,'NPOE' from dual union all
select 12479,'NPOE' from dual union all
select 1248,'USN' from dual union all
select 12480,'PVO' from dual union all
select 12481,'PVO' from dual union all
select 12482,'PVO' from dual union all
select 12483,'PVO' from dual union all
select 12484,'PVO' from dual union all
select 12485,'PVO' from dual union all
select 12486,'PVO' from dual union all
select 12487,'PVO' from dual union all
select 12488,'NPOE' from dual union all
select 12489,'NPOE' from dual union all
select 1249,'USN' from dual union all
select 12490,'PVO' from dual union all
select 12491,'ISRO' from dual union all
select 12492,'ISRO' from dual union all
select 12493,'NPOE' from dual union all
select 12494,'NASA' from dual union all
select 12495,'MO' from dual union all
select 12496,'RVSN' from dual union all
select 12497,'NASA' from dual union all
select 12498,'MO' from dual union all
select 12499,'MO' from dual union all
select 125,'USAF' from dual union all
select 1250,'USA' from dual union all
select 12500,'MO' from dual union all
select 12501,'MO' from dual union all
select 12502,'MO' from dual union all
select 12503,'MO' from dual union all
select 12504,'VMF' from dual union all
select 12505,'MO' from dual union all
select 12506,'MO' from dual union all
select 12507,'MO' from dual union all
select 12508,'RVSN' from dual union all
select 12509,'PVO' from dual union all
select 1251,'USN' from dual union all
select 12510,'PVO' from dual union all
select 12511,'PVO' from dual union all
select 12512,'MOM' from dual union all
select 12512,'MSVYAZ' from dual union all
select 12513,'RVSN' from dual union all
select 12514,'RVSN' from dual union all
select 12515,'RVSN' from dual union all
select 12516,'USN' from dual union all
select 12517,'MVD' from dual union all
select 12518,'RVSN' from dual union all
select 12519,'RVSN' from dual union all
select 1252,'RVSN' from dual union all
select 12520,'MO' from dual union all
select 12521,'RVSN' from dual union all
select 12522,'MO' from dual union all
select 12523,'MO' from dual union all
select 12524,'MO' from dual union all
select 12525,'MO' from dual union all
select 12526,'MO' from dual union all
select 12527,'NASA' from dual union all
select 12528,'NASA' from dual union all
select 12529,'NASA' from dual union all
select 1253,'RVSN' from dual union all
select 12530,'NASA' from dual union all
select 12531,'NASA' from dual union all
select 12532,'NASA' from dual union all
select 12533,'NASA' from dual union all
select 12534,'NASA' from dual union all
select 12535,'NASA' from dual union all
select 12536,'NASA' from dual union all
select 12537,'NASA' from dual union all
select 12538,'NASA' from dual union all
select 12539,'NASA' from dual union all
select 1254,'OKB1' from dual union all
select 12540,'NASA' from dual union all
select 12541,'NASA' from dual union all
select 12542,'NASA' from dual union all
select 12543,'PVO' from dual union all
select 12544,'ESA' from dual union all
select 12545,'ISRO' from dual union all
select 12546,'ESA' from dual union all
select 12547,'PVO' from dual union all
select 12548,'RVSN' from dual union all
select 12549,'RVSN' from dual union all
select 1255,'OKB1' from dual union all
select 12550,'VMF' from dual;
insert into SATELLITE_ORG
select 12551,'VMF' from dual union all
select 12552,'VMF' from dual union all
select 12553,'NOAA' from dual union all
select 12554,'USN' from dual union all
select 12555,'USN' from dual union all
select 12556,'MO' from dual union all
select 12557,'RVSN' from dual union all
select 12558,'RVSN' from dual union all
select 12559,'NASA' from dual union all
select 1256,'OKB1' from dual union all
select 12560,'NASA' from dual union all
select 12561,'RVSN' from dual union all
select 12562,'ESA' from dual union all
select 12563,'RVSN' from dual union all
select 12564,'MSVYAZ' from dual union all
select 12565,'RVSN' from dual union all
select 12566,'RVSN' from dual union all
select 12567,'MO' from dual union all
select 12568,'MO' from dual union all
select 12569,'MO' from dual union all
select 1257,'OKB1' from dual union all
select 12570,'MO' from dual union all
select 12571,'MO' from dual union all
select 12572,'RVSN' from dual union all
select 12573,'MO' from dual union all
select 12574,'MO' from dual union all
select 12575,'MO' from dual union all
select 12576,'MO' from dual union all
select 12577,'MVD' from dual union all
select 12578,'RVSN' from dual union all
select 12579,'MO' from dual union all
select 1258,'OKB1' from dual union all
select 12580,'NASA' from dual union all
select 12581,'NASA' from dual union all
select 12582,'USN' from dual union all
select 12583,'MO' from dual union all
select 12584,'RVSN' from dual union all
select 12585,'MVD' from dual union all
select 12586,'RVSN' from dual union all
select 12587,'RVSN' from dual union all
select 12588,'MO' from dual union all
select 12589,'RVSN' from dual union all
select 1259,'USAF' from dual union all
select 12590,'MO' from dual union all
select 12591,'MO' from dual union all
select 12592,'MO' from dual union all
select 12593,'MO' from dual union all
select 12594,'MO' from dual union all
select 12595,'MO' from dual union all
select 12596,'MO' from dual union all
select 12597,'MO' from dual union all
select 12598,'MO' from dual union all
select 12599,'RVSN' from dual union all
select 126,'USAF' from dual union all
select 1260,'USAF' from dual union all
select 12600,'MO' from dual union all
select 12601,'MO' from dual union all
select 12602,'MO' from dual union all
select 12603,'RVSN' from dual union all
select 12604,'RVSN' from dual union all
select 12605,'VMF' from dual union all
select 12606,'VMF' from dual union all
select 12607,'VMF' from dual union all
select 12608,'VMF' from dual union all
select 12609,'VMF' from dual union all
select 1261,'OKB1' from dual union all
select 12610,'VMF' from dual union all
select 12611,'VMF' from dual union all
select 12612,'VMF' from dual union all
select 12613,'VMF' from dual union all
select 12614,'MO' from dual union all
select 12615,'VMF' from dual union all
select 12616,'VMF' from dual union all
select 12617,'RVSN' from dual union all
select 12618,'MO' from dual union all
select 12619,'RVSN' from dual union all
select 1262,'OKB1' from dual union all
select 12620,'RVSN' from dual union all
select 12621,'MO' from dual union all
select 12622,'MO' from dual union all
select 12623,'MO' from dual union all
select 12624,'NASA' from dual union all
select 12625,'NASA' from dual union all
select 12626,'NASA' from dual union all
select 12627,'PVO' from dual union all
select 12628,'NASA' from dual union all
select 12629,'RVSN' from dual union all
select 1263,'OKB1' from dual union all
select 12630,'RVSN' from dual union all
select 12631,'VMF' from dual union all
select 12632,'RVSN' from dual union all
select 12633,'RVSN' from dual union all
select 12634,'RVSN' from dual union all
select 12635,'USN' from dual union all
select 12636,'MO' from dual union all
select 12637,'MO' from dual union all
select 12638,'MO' from dual union all
select 12639,'MO' from dual union all
select 1264,'OKB1' from dual union all
select 12640,'MO' from dual;
insert into SATELLITE_ORG
select 12641,'MO' from dual union all
select 12642,'MO' from dual union all
select 12643,'MO' from dual union all
select 12644,'RVSN' from dual union all
select 12645,'IK' from dual union all
select 12646,'RVSN' from dual union all
select 12647,'NASA' from dual union all
select 12648,'VMF' from dual union all
select 12649,'VMF' from dual union all
select 1265,'OKB1' from dual union all
select 12650,'VMF' from dual union all
select 12651,'VMF' from dual union all
select 12652,'VMF' from dual union all
select 12653,'VMF' from dual union all
select 12654,'VMF' from dual union all
select 12655,'VMF' from dual union all
select 12656,'VMF' from dual union all
select 12657,'VMF' from dual union all
select 12658,'VMF' from dual union all
select 12659,'VMF' from dual union all
select 1266,'OKB1' from dual union all
select 12660,'VMF' from dual union all
select 12661,'VMF' from dual union all
select 12662,'VMF' from dual union all
select 12663,'VMF' from dual union all
select 12664,'VMF' from dual union all
select 12665,'VMF' from dual union all
select 12666,'VMF' from dual union all
select 12667,'VMF' from dual union all
select 12668,'VMF' from dual union all
select 12669,'VMF' from dual union all
select 1267,'OKB10' from dual union all
select 12670,'VMF' from dual union all
select 12671,'VMF' from dual union all
select 12672,'VMF' from dual union all
select 12673,'VMF' from dual union all
select 12674,'VMF' from dual union all
select 12675,'VMF' from dual union all
select 12676,'VMF' from dual union all
select 12677,'NASDA' from dual union all
select 12678,'NASDA' from dual union all
select 12679,'NASA' from dual union all
select 1268,'OKB10' from dual union all
select 12680,'RVSN' from dual union all
select 12681,'VMF' from dual union all
select 12682,'RVSN' from dual union all
select 12683,'MO' from dual union all
select 12684,'MO' from dual union all
select 12685,'MO' from dual union all
select 12686,'MO' from dual union all
select 12687,'MO' from dual union all
select 12688,'RVSN' from dual union all
select 12689,'VMF' from dual union all
select 1269,'OKB10' from dual union all
select 12690,'VMF' from dual union all
select 12691,'VMF' from dual union all
select 12692,'VMF' from dual union all
select 12693,'VMF' from dual union all
select 12694,'VMF' from dual union all
select 12695,'VMF' from dual union all
select 12696,'VMF' from dual union all
select 12697,'VMF' from dual union all
select 12698,'VMF' from dual union all
select 12699,'VMF' from dual union all
select 127,'USAF' from dual union all
select 1270,'RVSN' from dual union all
select 12700,'VMF' from dual union all
select 12701,'VMF' from dual union all
select 12702,'VMF' from dual union all
select 12703,'VMF' from dual union all
select 12704,'VMF' from dual union all
select 12705,'VMF' from dual union all
select 12706,'VMF' from dual union all
select 12707,'VMF' from dual union all
select 12708,'VMF' from dual union all
select 12709,'VMF' from dual union all
select 1271,'NRO' from dual union all
select 1271,'USN' from dual union all
select 12710,'VMF' from dual union all
select 12711,'VMF' from dual union all
select 12712,'VMF' from dual union all
select 12713,'VMF' from dual union all
select 12714,'VMF' from dual union all
select 12715,'VMF' from dual union all
select 12716,'MO' from dual union all
select 12717,'RVSN' from dual union all
select 12718,'MO' from dual union all
select 12719,'VMF' from dual union all
select 1272,'USN' from dual union all
select 12720,'VMF' from dual union all
select 12721,'VMF' from dual union all
select 12722,'VMF' from dual union all
select 12723,'VMF' from dual union all
select 12724,'VMF' from dual union all
select 12725,'VMF' from dual union all
select 12726,'VMF' from dual union all
select 12727,'VMF' from dual union all
select 12728,'VMF' from dual union all
select 12729,'VMF' from dual union all
select 1273,'NRO' from dual;
insert into SATELLITE_ORG
select 12730,'VMF' from dual union all
select 12731,'VMF' from dual union all
select 12732,'VMF' from dual union all
select 12733,'VMF' from dual union all
select 12734,'VMF' from dual union all
select 12735,'VMF' from dual union all
select 12736,'VMF' from dual union all
select 12737,'VMF' from dual union all
select 12738,'VMF' from dual union all
select 12739,'VMF' from dual union all
select 1274,'OKB1' from dual union all
select 12740,'VMF' from dual union all
select 12741,'VMF' from dual union all
select 12742,'VMF' from dual union all
select 12743,'VMF' from dual union all
select 12744,'VMF' from dual union all
select 12745,'VMF' from dual union all
select 12746,'VMF' from dual union all
select 12747,'VMF' from dual union all
select 12748,'VMF' from dual union all
select 12749,'VMF' from dual union all
select 1275,'OKB1' from dual union all
select 12750,'VMF' from dual union all
select 12751,'VMF' from dual union all
select 12752,'VMF' from dual union all
select 12753,'VMF' from dual union all
select 12754,'VMF' from dual union all
select 12755,'VMF' from dual union all
select 12756,'VMF' from dual union all
select 12757,'VMF' from dual union all
select 12758,'VMF' from dual union all
select 12759,'VMF' from dual union all
select 1276,'OKB1' from dual union all
select 12760,'VMF' from dual union all
select 12761,'VMF' from dual union all
select 12762,'VMF' from dual union all
select 12763,'VMF' from dual union all
select 12764,'VMF' from dual union all
select 12765,'VMF' from dual union all
select 12766,'VMF' from dual union all
select 12767,'VMF' from dual union all
select 12768,'VMF' from dual union all
select 12769,'VMF' from dual union all
select 1277,'OKB1' from dual union all
select 12770,'VMF' from dual union all
select 12771,'VMF' from dual union all
select 12772,'VMF' from dual union all
select 12773,'VMF' from dual union all
select 12774,'VMF' from dual union all
select 12775,'VMF' from dual union all
select 12776,'TSSKB' from dual union all
select 12777,'RVSN' from dual union all
select 12778,'NASA' from dual union all
select 12779,'VMF' from dual union all
select 1278,'OKB1' from dual union all
select 12780,'VMF' from dual union all
select 12781,'VMF' from dual union all
select 12782,'VMF' from dual union all
select 12783,'VMF' from dual union all
select 12784,'RVSN' from dual union all
select 12785,'VS' from dual union all
select 12786,'MO' from dual union all
select 12787,'NASDA' from dual union all
select 12788,'MVD' from dual union all
select 12789,'RVSN' from dual union all
select 1279,'OKB1' from dual union all
select 12790,'MO' from dual union all
select 12791,'MO' from dual union all
select 12792,'RVSN' from dual union all
select 12793,'MO' from dual union all
select 12794,'MO' from dual union all
select 12795,'MO' from dual union all
select 12796,'MO' from dual union all
select 12797,'MO' from dual union all
select 12798,'NASA' from dual union all
select 12799,'CIA' from dual union all
select 12799,'NRO' from dual union all
select 128,'USAF' from dual union all
select 1280,'OKB1' from dual union all
select 12800,'USAF' from dual union all
select 12801,'MO' from dual union all
select 12802,'RVSN' from dual union all
select 12803,'VMF' from dual union all
select 12804,'RVSN' from dual union all
select 12805,'MO' from dual union all
select 12806,'MO' from dual union all
select 12807,'VMF' from dual union all
select 12808,'VMF' from dual union all
select 12809,'VMF' from dual union all
select 1281,'OKB1' from dual union all
select 12810,'NASDA' from dual union all
select 12811,'MO' from dual union all
select 12812,'MO' from dual union all
select 12813,'MO' from dual union all
select 12814,'PVO' from dual union all
select 12815,'PVO' from dual union all
select 12816,'PVO' from dual union all
select 12817,'PVO' from dual union all
select 12818,'MOM' from dual union all
select 12818,'MSVYAZ' from dual;
insert into SATELLITE_ORG
select 12819,'RVSN' from dual union all
select 1282,'OKB1' from dual union all
select 12820,'RVSN' from dual union all
select 12821,'RVSN' from dual union all
select 12822,'RVSN' from dual union all
select 12823,'NASA' from dual union all
select 12824,'NASA' from dual union all
select 12825,'NASA' from dual union all
select 12826,'NASA' from dual union all
select 12827,'RVSN' from dual union all
select 12828,'VMF' from dual union all
select 12829,'RVSN' from dual union all
select 1283,'OKB1' from dual union all
select 12830,'MO' from dual union all
select 12831,'RVSN' from dual union all
select 12832,'MO' from dual union all
select 12833,'PVO' from dual union all
select 12834,'PVO' from dual union all
select 12835,'VMF' from dual union all
select 12836,'RVSN' from dual union all
select 12837,'VTS' from dual union all
select 12838,'RVSN' from dual union all
select 12839,'MO' from dual union all
select 1284,'OKB1' from dual union all
select 12840,'MO' from dual union all
select 12841,'MO' from dual union all
select 12842,'MAI' from dual union all
select 12843,'MAI' from dual union all
select 12844,'MAI' from dual union all
select 12845,'MAI' from dual union all
select 12846,'MAI' from dual union all
select 12847,'MAI' from dual union all
select 12848,'IK' from dual union all
select 12849,'MO' from dual union all
select 1285,'RVSN' from dual union all
select 12850,'RVSN' from dual union all
select 12851,'RVSN' from dual union all
select 12852,'PVO' from dual union all
select 12853,'RVSN' from dual union all
select 12854,'RVSN' from dual union all
select 12855,'SBS' from dual union all
select 12856,'USN' from dual union all
select 12857,'ISAS' from dual union all
select 12858,'USN' from dual union all
select 12859,'USN' from dual union all
select 1286,'OKB1' from dual union all
select 12860,'PVO' from dual union all
select 12861,'PVO' from dual union all
select 12862,'PVO' from dual union all
select 12863,'PVO' from dual union all
select 12864,'PVO' from dual union all
select 12865,'PVO' from dual union all
select 12866,'PVO' from dual union all
select 12867,'PVO' from dual union all
select 12868,'PVO' from dual union all
select 12869,'PVO' from dual union all
select 1287,'OKB1' from dual union all
select 12870,'PVO' from dual union all
select 12871,'PVO' from dual union all
select 12872,'RVSN' from dual union all
select 12873,'MO' from dual union all
select 12874,'NASA' from dual union all
select 12875,'MO' from dual union all
select 12876,'MO' from dual union all
select 12877,'MO' from dual union all
select 12878,'MO' from dual union all
select 12879,'MOM' from dual union all
select 1288,'USAF' from dual union all
select 12880,'MO' from dual union all
select 12881,'MO' from dual union all
select 12882,'RVSN' from dual union all
select 12883,'MO' from dual union all
select 12884,'MAI' from dual union all
select 12885,'MAI' from dual union all
select 12886,'VMF' from dual union all
select 12887,'JPL' from dual union all
select 12887,'UCO' from dual union all
select 12888,'UOS' from dual union all
select 12889,'NASA' from dual union all
select 1289,'USAF' from dual union all
select 12890,'MAI' from dual union all
select 12891,'UCO' from dual union all
select 12892,'PVO' from dual union all
select 12893,'PVO' from dual union all
select 12894,'PVO' from dual union all
select 12895,'MVD' from dual union all
select 12896,'RVSN' from dual union all
select 12897,'MO' from dual union all
select 12898,'RVSN' from dual union all
select 12899,'RVSN' from dual union all
select 129,'USAF' from dual union all
select 1290,'USAF' from dual union all
select 12900,'MO' from dual union all
select 12901,'RVSN' from dual union all
select 12902,'RVSN' from dual union all
select 12903,'VS' from dual union all
select 12904,'RVSN' from dual union all
select 12905,'MO' from dual union all
select 12906,'PVO' from dual union all
select 12907,'PVO' from dual;
insert into SATELLITE_ORG
select 12908,'NASA' from dual union all
select 12909,'PVO' from dual union all
select 1291,'NRO' from dual union all
select 1291,'USN' from dual union all
select 12910,'MO' from dual union all
select 12911,'MO' from dual union all
select 12912,'MO' from dual union all
select 12913,'RVSN' from dual union all
select 12914,'VMF' from dual union all
select 12915,'MOM' from dual union all
select 12915,'MSVYAZ' from dual union all
select 12916,'RVSN' from dual union all
select 12917,'RVSN' from dual union all
select 12918,'RVSN' from dual union all
select 12919,'PVO' from dual union all
select 1292,'NRO' from dual union all
select 1292,'USN' from dual union all
select 12920,'RVSN' from dual union all
select 12921,'MO' from dual union all
select 12922,'MO' from dual union all
select 12923,'MO' from dual union all
select 12924,'MO' from dual union all
select 12925,'MO' from dual union all
select 12926,'MO' from dual union all
select 12927,'IKI' from dual union all
select 12927,'NPOL' from dual union all
select 12928,'RVSN' from dual union all
select 12929,'RVSN' from dual union all
select 1293,'OSCAR' from dual union all
select 12930,'NRO' from dual union all
select 12931,'USAF' from dual union all
select 12932,'USAF' from dual union all
select 12933,'PVO' from dual union all
select 12934,'RVSN' from dual union all
select 12935,'RVSN' from dual union all
select 12936,'MO' from dual union all
select 12937,'RVSN' from dual union all
select 12938,'IKI' from dual union all
select 12938,'NPOL' from dual union all
select 12939,'RVSN' from dual union all
select 1294,'JPL' from dual union all
select 1294,'NASA' from dual union all
select 12940,'RVSN' from dual union all
select 12941,'RVSN' from dual union all
select 12942,'PVO' from dual union all
select 12943,'USN' from dual union all
select 12944,'USN' from dual union all
select 12945,'NASA' from dual union all
select 12946,'NASA' from dual union all
select 12947,'VMF' from dual union all
select 12948,'VMF' from dual union all
select 12949,'VMF' from dual union all
select 1295,'OKB1' from dual union all
select 12950,'VMF' from dual union all
select 12951,'VMF' from dual union all
select 12952,'VMF' from dual union all
select 12953,'NASA' from dual union all
select 12954,'MO' from dual union all
select 12955,'RVSN' from dual union all
select 12956,'MO' from dual union all
select 12957,'MO' from dual union all
select 12958,'MAI' from dual union all
select 12959,'MO' from dual union all
select 1296,'OKB1' from dual union all
select 12960,'RVSN' from dual union all
select 12961,'RVSN' from dual union all
select 12962,'VMF' from dual union all
select 12963,'VMF' from dual union all
select 12964,'VMF' from dual union all
select 12965,'VMF' from dual union all
select 12966,'VMF' from dual union all
select 12967,'AMC' from dual union all
select 12968,'ISRO' from dual union all
select 12969,'RVSN' from dual union all
select 1297,'OKB1' from dual union all
select 12970,'MO' from dual union all
select 12971,'MO' from dual union all
select 12972,'MO' from dual union all
select 12973,'MO' from dual union all
select 12974,'MO' from dual union all
select 12975,'MO' from dual union all
select 12976,'MO' from dual union all
select 12977,'MO' from dual union all
select 12978,'MO' from dual union all
select 12979,'MO' from dual union all
select 1298,'NASA' from dual union all
select 12980,'MO' from dual union all
select 12981,'MO' from dual union all
select 12982,'MO' from dual union all
select 12983,'RVSN' from dual union all
select 12984,'MO' from dual union all
select 12985,'MO' from dual union all
select 12986,'RVSN' from dual union all
select 12987,'VS' from dual union all
select 12988,'MO' from dual union all
select 12989,'MO' from dual union all
select 1299,'OKB1' from dual union all
select 12990,'RVSN' from dual union all
select 12991,'MO' from dual union all
select 12992,'MO' from dual;
insert into SATELLITE_ORG
select 12993,'MO' from dual union all
select 12994,'INTEL' from dual union all
select 12995,'RVSN' from dual union all
select 12996,'RVSN' from dual union all
select 12997,'MAIO' from dual union all
select 12997,'MOM' from dual union all
select 12998,'MAIO' from dual union all
select 12998,'MOM' from dual union all
select 12999,'MAIO' from dual union all
select 12999,'MOM' from dual union all
select 13,'CIA' from dual union all
select 13,'DOD' from dual union all
select 130,'USAF' from dual union all
select 1300,'OKB1' from dual union all
select 13000,'MAIO' from dual union all
select 13000,'MOM' from dual union all
select 13001,'MAIO' from dual union all
select 13001,'MOM' from dual union all
select 13002,'MAIO' from dual union all
select 13002,'MOM' from dual union all
select 13003,'RVSN' from dual union all
select 13004,'MO' from dual union all
select 13005,'MO' from dual union all
select 13006,'MO' from dual union all
select 13007,'NASA' from dual union all
select 13008,'MO' from dual union all
select 13009,'RVSN' from dual union all
select 1301,'NASA' from dual union all
select 13010,'ESA' from dual union all
select 13011,'ESA' from dual union all
select 13012,'MO' from dual union all
select 13013,'RVSN' from dual union all
select 13014,'RVSN' from dual union all
select 13015,'VMF' from dual union all
select 13016,'RVSN' from dual union all
select 13017,'VMF' from dual union all
select 13018,'VMF' from dual union all
select 13019,'VMF' from dual union all
select 1302,'USAF' from dual union all
select 13020,'VMF' from dual union all
select 13021,'VMF' from dual union all
select 13022,'VMF' from dual union all
select 13023,'VMF' from dual union all
select 13024,'VMF' from dual union all
select 13025,'ESA' from dual union all
select 13026,'VMF' from dual union all
select 13027,'MO' from dual union all
select 13028,'RVSN' from dual union all
select 13029,'MO' from dual union all
select 1303,'USN' from dual union all
select 13030,'MO' from dual union all
select 13031,'VTS' from dual union all
select 13032,'RVSN' from dual union all
select 13033,'VMF' from dual union all
select 13034,'RVSN' from dual union all
select 13035,'AMC' from dual union all
select 13036,'MO' from dual union all
select 13037,'RVSN' from dual union all
select 13038,'MO' from dual union all
select 13039,'MO' from dual union all
select 1304,'OKB1' from dual union all
select 13040,'NRO' from dual union all
select 13041,'NRO' from dual union all
select 13042,'PVO' from dual union all
select 13043,'RVSN' from dual union all
select 13044,'PVO' from dual union all
select 13045,'MO' from dual union all
select 13046,'RVSN' from dual union all
select 13047,'MO' from dual union all
select 13048,'MO' from dual union all
select 13049,'MO' from dual union all
select 1305,'MO' from dual union all
select 13050,'MO' from dual union all
select 13051,'MO' from dual union all
select 13052,'MO' from dual union all
select 13053,'VMF' from dual union all
select 13054,'PVO' from dual union all
select 13055,'PVO' from dual union all
select 13056,'MSVYAZ' from dual union all
select 13057,'RVSN' from dual union all
select 13058,'RVSN' from dual union all
select 13059,'RVSN' from dual union all
select 1306,'RVSN' from dual union all
select 13060,'RVSN' from dual union all
select 13061,'VMF' from dual union all
select 13062,'MO' from dual union all
select 13063,'MO' from dual union all
select 13064,'RVSN' from dual union all
select 13065,'VMF' from dual union all
select 13066,'RVSN' from dual union all
select 13067,'VS' from dual union all
select 13068,'RVSN' from dual union all
select 13069,'WUTC' from dual union all
select 1307,'CIA' from dual union all
select 1307,'NRO' from dual union all
select 13070,'MO' from dual union all
select 13071,'RVSN' from dual union all
select 13072,'RVSN' from dual union all
select 13073,'VMF' from dual union all
select 13074,'VMF' from dual;
insert into SATELLITE_ORG
select 13075,'RVSN' from dual union all
select 13076,'MO' from dual union all
select 13077,'MO' from dual union all
select 13078,'MO' from dual union all
select 13079,'MO' from dual union all
select 1308,'MO' from dual union all
select 13080,'PVO' from dual union all
select 13081,'RVSN' from dual union all
select 13082,'RVSN' from dual union all
select 13083,'INTEL' from dual union all
select 13084,'MO' from dual union all
select 13085,'RVSN' from dual union all
select 13086,'USAF' from dual union all
select 13087,'NASA' from dual union all
select 13088,'NASA' from dual union all
select 13089,'USAF' from dual union all
select 1309,'OKB1' from dual union all
select 13090,'RVSN' from dual union all
select 13091,'MO' from dual union all
select 13092,'MSVYAZ' from dual union all
select 13093,'RVSN' from dual union all
select 13094,'RVSN' from dual union all
select 13095,'RVSN' from dual union all
select 13096,'MO' from dual union all
select 13097,'RVSN' from dual union all
select 13098,'NASA' from dual union all
select 13099,'RVSN' from dual union all
select 131,'USAF' from dual union all
select 1310,'USN' from dual union all
select 13100,'MO' from dual union all
select 13101,'MO' from dual union all
select 13102,'MO' from dual union all
select 13103,'NRO' from dual union all
select 13104,'NRO' from dual union all
select 13105,'NRO' from dual union all
select 13106,'NASA' from dual union all
select 13107,'MOM' from dual union all
select 13107,'MSVYAZ' from dual union all
select 13108,'RVSN' from dual union all
select 13109,'RVSN' from dual union all
select 1311,'USAF' from dual union all
select 13110,'VMF' from dual union all
select 13111,'RVSN' from dual union all
select 13112,'RVSN' from dual union all
select 13113,'GMS' from dual union all
select 13113,'MO' from dual union all
select 13114,'MO' from dual union all
select 13115,'MO' from dual union all
select 13116,'MO' from dual union all
select 13117,'MO' from dual union all
select 13118,'VS' from dual union all
select 13119,'RVSN' from dual union all
select 1312,'NASA' from dual union all
select 13120,'VS' from dual union all
select 13121,'RVSN' from dual union all
select 13122,'TSSKB' from dual union all
select 13123,'RVSN' from dual union all
select 13124,'PVO' from dual union all
select 13125,'RVSN' from dual union all
select 13126,'RVSN' from dual union all
select 13127,'VMF' from dual union all
select 13128,'RVSN' from dual union all
select 13129,'ISRO' from dual union all
select 1313,'NASA' from dual union all
select 13130,'NASA' from dual union all
select 13131,'USAF' from dual union all
select 13132,'NASDA' from dual union all
select 13133,'ISAS' from dual union all
select 13134,'MO' from dual union all
select 13135,'RVSN' from dual union all
select 13136,'NASA' from dual union all
select 13137,'NASA' from dual union all
select 13138,'NPOE' from dual union all
select 13139,'RVSN' from dual union all
select 1314,'AEC' from dual union all
select 1314,'USAF' from dual union all
select 13140,'PVO' from dual union all
select 13141,'PVO' from dual union all
select 13142,'PVO' from dual union all
select 13143,'RVSN' from dual union all
select 13144,'MO' from dual union all
select 13145,'RVSN' from dual union all
select 13146,'MVD' from dual union all
select 13147,'RVSN' from dual union all
select 13148,'MO' from dual union all
select 13149,'RVSN' from dual union all
select 1315,'USA' from dual union all
select 13150,'VMF' from dual union all
select 13151,'MO' from dual union all
select 13152,'NRO' from dual union all
select 13153,'VS' from dual union all
select 13154,'RVSN' from dual union all
select 13155,'MO' from dual union all
select 13156,'MO' from dual union all
select 13157,'MO' from dual union all
select 13158,'MO' from dual union all
select 13159,'MO' from dual union all
select 1316,'USAF' from dual union all
select 13160,'MO' from dual union all
select 13161,'MO' from dual;
insert into SATELLITE_ORG
select 13162,'MO' from dual union all
select 13163,'MO' from dual union all
select 13164,'MO' from dual union all
select 13165,'MO' from dual union all
select 13166,'MO' from dual union all
select 13167,'MO' from dual union all
select 13168,'RVSN' from dual union all
select 13169,'RVSN' from dual union all
select 1317,'INTEL' from dual union all
select 13170,'NRO' from dual union all
select 13171,'USAF' from dual union all
select 13172,'NRO' from dual union all
select 13173,'NPOE' from dual union all
select 13174,'RVSN' from dual union all
select 13175,'VMF' from dual union all
select 13176,'MAIO' from dual union all
select 13176,'MOM' from dual union all
select 13177,'MO' from dual union all
select 13178,'RVSN' from dual union all
select 13179,'RVSN' from dual union all
select 1318,'NASA' from dual union all
select 13180,'VMF' from dual union all
select 13181,'VMF' from dual union all
select 13182,'VMF' from dual union all
select 13183,'VMF' from dual union all
select 13184,'VMF' from dual union all
select 13185,'VMF' from dual union all
select 13186,'VMF' from dual union all
select 13187,'VMF' from dual union all
select 13188,'VMF' from dual union all
select 13189,'VMF' from dual union all
select 1319,'USN' from dual union all
select 13190,'VMF' from dual union all
select 13191,'VMF' from dual union all
select 13192,'VMF' from dual union all
select 13193,'VMF' from dual union all
select 13194,'VMF' from dual union all
select 13195,'VMF' from dual union all
select 13196,'VMF' from dual union all
select 13197,'VMF' from dual union all
select 13198,'VMF' from dual union all
select 13199,'VMF' from dual union all
select 132,'USN' from dual union all
select 1320,'OKB1F3' from dual union all
select 13200,'VMF' from dual union all
select 13201,'VMF' from dual union all
select 13202,'VMF' from dual union all
select 13203,'VMF' from dual union all
select 13204,'MAIO' from dual union all
select 13204,'MOM' from dual union all
select 13205,'PVO' from dual union all
select 13206,'RVSN' from dual union all
select 13207,'RVSN' from dual union all
select 13208,'MO' from dual union all
select 13209,'RVSN' from dual union all
select 1321,'RVSN' from dual union all
select 13210,'NPOE' from dual union all
select 13211,'RVSN' from dual union all
select 13212,'MO' from dual union all
select 13213,'MVD' from dual union all
select 13214,'RVSN' from dual union all
select 13215,'RVSN' from dual union all
select 13216,'MO' from dual union all
select 13217,'RVSN' from dual union all
select 13218,'RVSN' from dual union all
select 13219,'TSSKB' from dual union all
select 1322,'OKB1F3' from dual union all
select 13220,'RVSN' from dual union all
select 13221,'VMF' from dual union all
select 13222,'VMF' from dual union all
select 13223,'VMF' from dual union all
select 13224,'VMF' from dual union all
select 13225,'VMF' from dual union all
select 13226,'VMF' from dual union all
select 13227,'VMF' from dual union all
select 13228,'VMF' from dual union all
select 13229,'VMF' from dual union all
select 1323,'USN' from dual union all
select 13230,'VMF' from dual union all
select 13231,'VMF' from dual union all
select 13232,'VMF' from dual union all
select 13233,'VMF' from dual union all
select 13234,'VMF' from dual union all
select 13235,'VMF' from dual union all
select 13236,'VMF' from dual union all
select 13237,'MO' from dual union all
select 13238,'RVSN' from dual union all
select 13239,'RVSN' from dual union all
select 1324,'OKB10' from dual union all
select 13240,'RVSN' from dual union all
select 13241,'MO' from dual union all
select 13242,'RVSN' from dual union all
select 13243,'VMF' from dual union all
select 13244,'MO' from dual union all
select 13245,'RVSN' from dual union all
select 13246,'PVO' from dual union all
select 13247,'PVO' from dual union all
select 13248,'PVO' from dual union all
select 13249,'PVO' from dual union all
select 1325,'RVSN' from dual;
insert into SATELLITE_ORG
select 13250,'PVO' from dual union all
select 13251,'PVO' from dual union all
select 13252,'PVO' from dual union all
select 13253,'RVSN' from dual union all
select 13254,'MO' from dual union all
select 13255,'MO' from dual union all
select 13256,'MO' from dual union all
select 13257,'MO' from dual union all
select 13257,'MOLN' from dual union all
select 13258,'RVSN' from dual union all
select 13259,'PVO' from dual union all
select 1326,'RVSN' from dual union all
select 13260,'RVSN' from dual union all
select 13261,'PVO' from dual union all
select 13262,'RVSN' from dual union all
select 13263,'MVD' from dual union all
select 13264,'RVSN' from dual union all
select 13265,'MO' from dual union all
select 13266,'RVSN' from dual union all
select 13267,'MO' from dual union all
select 13268,'MO' from dual union all
select 13269,'WUTC' from dual union all
select 1327,'NRO' from dual union all
select 13270,'NPOE' from dual union all
select 13271,'VS' from dual union all
select 13272,'MO' from dual union all
select 13273,'RVSN' from dual union all
select 13274,'NPOE' from dual union all
select 13275,'NPOE' from dual union all
select 13276,'MO' from dual union all
select 13277,'MO' from dual union all
select 13278,'MO' from dual union all
select 13279,'MO' from dual union all
select 1328,'NASA' from dual union all
select 13280,'MO' from dual union all
select 13280,'MOLN' from dual union all
select 13281,'PVO' from dual union all
select 13282,'VMF' from dual union all
select 13283,'MO' from dual union all
select 13284,'MO' from dual union all
select 13285,'RVSN' from dual union all
select 13286,'RVSN' from dual union all
select 13287,'NPOE' from dual union all
select 13288,'MO' from dual union all
select 13289,'MO' from dual union all
select 1329,'NRO' from dual union all
select 13290,'MO' from dual union all
select 13291,'NPOE' from dual union all
select 13292,'NPOE' from dual union all
select 13293,'RVSN' from dual union all
select 13294,'NASA' from dual union all
select 13295,'PVO' from dual union all
select 13296,'RVSN' from dual union all
select 13297,'RVSN' from dual union all
select 13298,'RVSN' from dual union all
select 13299,'NPOE' from dual union all
select 133,'USN' from dual union all
select 1330,'CIA' from dual union all
select 1330,'NRO' from dual union all
select 13300,'NASA' from dual union all
select 13301,'MOM' from dual union all
select 13302,'RVSN' from dual union all
select 13303,'MO' from dual union all
select 13304,'RVSN' from dual union all
select 13305,'MO' from dual union all
select 13306,'MO' from dual union all
select 13307,'MO' from dual union all
select 13308,'VMF' from dual union all
select 13309,'VMF' from dual union all
select 1331,'USAF' from dual union all
select 13310,'VMF' from dual union all
select 13311,'VMF' from dual union all
select 13312,'VMF' from dual union all
select 13313,'VMF' from dual union all
select 13314,'VMF' from dual union all
select 13315,'VMF' from dual union all
select 13316,'VMF' from dual union all
select 13317,'VMF' from dual union all
select 13318,'VMF' from dual union all
select 13319,'VMF' from dual union all
select 1332,'USAF' from dual union all
select 13320,'VMF' from dual union all
select 13321,'VMF' from dual union all
select 13322,'VMF' from dual union all
select 13323,'VMF' from dual union all
select 13324,'VMF' from dual union all
select 13325,'VMF' from dual union all
select 13326,'VMF' from dual union all
select 13327,'VMF' from dual union all
select 13328,'VMF' from dual union all
select 13329,'VMF' from dual union all
select 1333,'USAF' from dual union all
select 13330,'VMF' from dual union all
select 13331,'VMF' from dual union all
select 13332,'NPOE' from dual union all
select 13333,'VMF' from dual union all
select 13334,'VMF' from dual union all
select 13335,'VMF' from dual union all
select 13336,'VMF' from dual union all
select 13337,'VMF' from dual;
insert into SATELLITE_ORG
select 13338,'VMF' from dual union all
select 13339,'VMF' from dual union all
select 1334,'USAF' from dual union all
select 13340,'VMF' from dual union all
select 13341,'VMF' from dual union all
select 13342,'VMF' from dual union all
select 13343,'VMF' from dual union all
select 13344,'VMF' from dual union all
select 13345,'MO' from dual union all
select 13346,'RVSN' from dual union all
select 13347,'VMF' from dual union all
select 13348,'VMF' from dual union all
select 13349,'VMF' from dual union all
select 1335,'RVSN' from dual union all
select 13350,'VMF' from dual union all
select 13351,'VMF' from dual union all
select 13352,'VMF' from dual union all
select 13353,'VMF' from dual union all
select 13354,'RVSN' from dual union all
select 13355,'VMF' from dual union all
select 13356,'VMF' from dual union all
select 13357,'VMF' from dual union all
select 13358,'VMF' from dual union all
select 13359,'VMF' from dual union all
select 1336,'RVSN' from dual union all
select 13360,'USN' from dual union all
select 13361,'NPOE' from dual union all
select 13362,'RVSN' from dual union all
select 13363,'MO' from dual union all
select 13364,'USAF' from dual union all
select 13365,'MVD' from dual union all
select 13366,'RVSN' from dual union all
select 13367,'NASA' from dual union all
select 13368,'NASA' from dual union all
select 13369,'VMF' from dual union all
select 1337,'RVSN' from dual union all
select 13370,'MO' from dual union all
select 13371,'MO' from dual union all
select 13372,'MO' from dual union all
select 13373,'MO' from dual union all
select 13374,'MO' from dual union all
select 13375,'MO' from dual union all
select 13376,'MO' from dual union all
select 13377,'MO' from dual union all
select 13378,'MO' from dual union all
select 13379,'MO' from dual union all
select 1338,'RVSN' from dual union all
select 13380,'MO' from dual union all
select 13381,'MO' from dual union all
select 13382,'MO' from dual union all
select 13383,'MO' from dual union all
select 13384,'RVSN' from dual union all
select 13385,'RVSN' from dual union all
select 13386,'RVSN' from dual union all
select 13387,'VMF' from dual union all
select 13388,'MO' from dual union all
select 13389,'MO' from dual union all
select 1339,'RVSN' from dual union all
select 13390,'RVSN' from dual union all
select 13391,'MO' from dual union all
select 13392,'RVSN' from dual union all
select 13393,'VMF' from dual union all
select 13394,'PVO' from dual union all
select 13395,'RVSN' from dual union all
select 13396,'VTS' from dual union all
select 13397,'RVSN' from dual union all
select 13398,'NPOE' from dual union all
select 13399,'MO' from dual union all
select 134,'USAF' from dual union all
select 1340,'RVSN' from dual union all
select 13400,'RVSN' from dual union all
select 13401,'MO' from dual union all
select 13402,'VS' from dual union all
select 13403,'RVSN' from dual union all
select 13404,'VMF' from dual union all
select 13405,'JPL' from dual union all
select 13406,'MO' from dual union all
select 13407,'MO' from dual union all
select 13408,'VMF' from dual union all
select 13409,'NPOE' from dual union all
select 1341,'RVSN' from dual union all
select 13410,'NPOE' from dual union all
select 13411,'VMF' from dual union all
select 13412,'VMF' from dual union all
select 13413,'NASA' from dual union all
select 13414,'MO' from dual union all
select 13415,'MO' from dual union all
select 13416,'VMF' from dual union all
select 13417,'NPOE' from dual union all
select 13418,'NPOE' from dual union all
select 13419,'PVO' from dual union all
select 1342,'RVSN' from dual union all
select 13420,'PVO' from dual union all
select 13421,'PVO' from dual union all
select 13422,'PVO' from dual union all
select 13423,'NPOE' from dual union all
select 13424,'NPOE' from dual union all
select 13425,'NPOE' from dual union all
select 13426,'RVSN' from dual union all
select 13427,'MVD' from dual;
insert into SATELLITE_ORG
select 13428,'RVSN' from dual union all
select 13429,'PVO' from dual union all
select 1343,'RVSN' from dual union all
select 13430,'PVO' from dual union all
select 13431,'TCAN' from dual union all
select 13432,'MOM' from dual union all
select 13432,'MSVYAZ' from dual union all
select 13433,'RVSN' from dual union all
select 13434,'RVSN' from dual union all
select 13435,'NPOE' from dual union all
select 13436,'NPOE' from dual union all
select 13437,'RVSN' from dual union all
select 13438,'NASA' from dual union all
select 13439,'NPOE' from dual union all
select 1344,'RVSN' from dual union all
select 13440,'NPOE' from dual union all
select 13441,'VMF' from dual union all
select 13442,'VMF' from dual union all
select 13443,'VMF' from dual union all
select 13444,'VMF' from dual union all
select 13445,'VMF' from dual union all
select 13446,'RVSN' from dual union all
select 13447,'NASA' from dual union all
select 13448,'MO' from dual union all
select 13449,'MO' from dual union all
select 1345,'RVSN' from dual union all
select 13450,'RVSN' from dual union all
select 13451,'RVSN' from dual union all
select 13452,'MO' from dual union all
select 13453,'MO' from dual union all
select 13454,'MO' from dual union all
select 13455,'MO' from dual union all
select 13456,'MO' from dual union all
select 13457,'MO' from dual union all
select 13458,'MO' from dual union all
select 13459,'MO' from dual union all
select 1346,'RVSN' from dual union all
select 13460,'MO' from dual union all
select 13461,'MO' from dual union all
select 13462,'MO' from dual union all
select 13463,'MO' from dual union all
select 13464,'MO' from dual union all
select 13465,'MO' from dual union all
select 13466,'MO' from dual union all
select 13467,'MO' from dual union all
select 13468,'MO' from dual union all
select 13469,'MO' from dual union all
select 1347,'RVSN' from dual union all
select 13470,'MO' from dual union all
select 13471,'MO' from dual union all
select 13472,'MO' from dual union all
select 13473,'MO' from dual union all
select 13474,'MO' from dual union all
select 13475,'MO' from dual union all
select 13476,'MO' from dual union all
select 13477,'MO' from dual union all
select 13478,'MO' from dual union all
select 13479,'MO' from dual union all
select 1348,'RVSN' from dual union all
select 13480,'MO' from dual union all
select 13481,'MO' from dual union all
select 13482,'MO' from dual union all
select 13483,'MO' from dual union all
select 13484,'MO' from dual union all
select 13485,'MO' from dual union all
select 13486,'MO' from dual union all
select 13487,'MO' from dual union all
select 13488,'MO' from dual union all
select 13489,'MO' from dual union all
select 1349,'RVSN' from dual union all
select 13490,'MO' from dual union all
select 13491,'MO' from dual union all
select 13492,'NASDA' from dual union all
select 13493,'NASDA' from dual union all
select 13494,'MO' from dual union all
select 13495,'MO' from dual union all
select 13496,'MO' from dual union all
select 13497,'RVSN' from dual union all
select 13498,'MO' from dual union all
select 13499,'MO' from dual union all
select 135,'USAF' from dual union all
select 1350,'RVSN' from dual union all
select 13500,'RVSN' from dual union all
select 13501,'PVO' from dual union all
select 13502,'USAF' from dual union all
select 13503,'PVO' from dual union all
select 13504,'PVO' from dual union all
select 13505,'PVO' from dual union all
select 13506,'VMF' from dual union all
select 13507,'PVO' from dual union all
select 13508,'VMF' from dual union all
select 13509,'MO' from dual union all
select 1351,'RVSN' from dual union all
select 13510,'NASDA' from dual union all
select 13511,'USAF' from dual union all
select 13512,'USAF' from dual union all
select 13513,'USAF' from dual union all
select 13514,'USAF' from dual union all
select 13515,'PVO' from dual union all
select 13516,'PVO' from dual;
insert into SATELLITE_ORG
select 13517,'RVSN' from dual union all
select 13518,'NASA' from dual union all
select 13519,'MVD' from dual union all
select 1352,'RVSN' from dual union all
select 13520,'RVSN' from dual union all
select 13521,'MAI' from dual union all
select 13522,'NPOE' from dual union all
select 13523,'NPOE' from dual union all
select 13524,'NASA' from dual union all
select 13525,'NASA' from dual union all
select 13526,'NASA' from dual union all
select 13527,'NASA' from dual union all
select 13528,'NASA' from dual union all
select 13529,'NASA' from dual union all
select 1353,'RVSN' from dual union all
select 13530,'NASA' from dual union all
select 13531,'NASA' from dual union all
select 13532,'NASA' from dual union all
select 13533,'NASA' from dual union all
select 13534,'NASA' from dual union all
select 13535,'NASA' from dual union all
select 13536,'NASA' from dual union all
select 13537,'NASA' from dual union all
select 13538,'NASA' from dual union all
select 13539,'NASA' from dual union all
select 1354,'RVSN' from dual union all
select 13540,'MAI' from dual union all
select 13541,'NPOE' from dual union all
select 13542,'MO' from dual union all
select 13543,'MO' from dual union all
select 13544,'MO' from dual union all
select 13545,'MO' from dual union all
select 13546,'MO' from dual union all
select 13547,'RVSN' from dual union all
select 13548,'MO' from dual union all
select 13549,'MO' from dual union all
select 1355,'RVSN' from dual union all
select 13550,'MO' from dual union all
select 13551,'MO' from dual union all
select 13552,'VS' from dual union all
select 13553,'MO' from dual union all
select 13554,'MSVYAZ' from dual union all
select 13555,'RVSN' from dual union all
select 13556,'RVSN' from dual union all
select 13557,'MAI' from dual union all
select 13558,'NPOE' from dual union all
select 13559,'RVSN' from dual union all
select 1356,'RVSN' from dual union all
select 13560,'NPOE' from dual union all
select 13561,'NASA' from dual union all
select 13562,'NASA' from dual union all
select 13563,'NASA' from dual union all
select 13564,'NASA' from dual union all
select 13565,'NASA' from dual union all
select 13566,'NASA' from dual union all
select 13567,'NASA' from dual union all
select 13568,'NASA' from dual union all
select 13569,'NASA' from dual union all
select 1357,'NRO' from dual union all
select 13570,'NASA' from dual union all
select 13571,'NASA' from dual union all
select 13572,'NASA' from dual union all
select 13573,'NASA' from dual union all
select 13574,'NASA' from dual union all
select 13575,'NASA' from dual union all
select 13576,'NASA' from dual union all
select 13577,'NASA' from dual union all
select 13578,'NASA' from dual union all
select 13579,'NASA' from dual union all
select 1358,'NASA' from dual union all
select 13580,'NASA' from dual union all
select 13581,'MO' from dual union all
select 13582,'MO' from dual union all
select 13583,'RVSN' from dual union all
select 13584,'RVSN' from dual union all
select 13585,'PVO' from dual union all
select 13586,'RVSN' from dual union all
select 13587,'RVSN' from dual union all
select 13588,'VMF' from dual union all
select 13589,'MOM' from dual union all
select 1359,'USAF' from dual union all
select 13590,'MO' from dual union all
select 13591,'RVSN' from dual union all
select 13592,'VMF' from dual union all
select 13593,'VMF' from dual union all
select 13594,'VMF' from dual union all
select 13595,'INTEL' from dual union all
select 13596,'NPOE' from dual union all
select 13597,'MO' from dual union all
select 13598,'RVSN' from dual union all
select 13599,'NASA' from dual union all
select 136,'USAF' from dual union all
select 1360,'USAF' from dual union all
select 13600,'VMF' from dual union all
select 13601,'VMF' from dual union all
select 13602,'VMF' from dual union all
select 13603,'MO' from dual union all
select 13604,'RVSN' from dual union all
select 13605,'RVSN' from dual union all
select 13606,'MO' from dual;
insert into SATELLITE_ORG
select 13607,'MO' from dual union all
select 13608,'RVSN' from dual union all
select 13609,'RVSN' from dual union all
select 1361,'USAF' from dual union all
select 13610,'RVSN' from dual union all
select 13611,'MO' from dual union all
select 13612,'RVSN' from dual union all
select 13613,'MO' from dual union all
select 13614,'MO' from dual union all
select 13615,'VMF' from dual union all
select 13616,'MO' from dual union all
select 13617,'VMF' from dual union all
select 13618,'RVSN' from dual union all
select 13619,'MO' from dual union all
select 1362,'MO' from dual union all
select 13620,'VMF' from dual union all
select 13621,'VMF' from dual union all
select 13622,'VMF' from dual union all
select 13623,'VMF' from dual union all
select 13624,'MSVYAZ' from dual union all
select 13625,'RVSN' from dual union all
select 13626,'RVSN' from dual union all
select 13627,'PVO' from dual union all
select 13628,'RVSN' from dual union all
select 13629,'RVSN' from dual union all
select 1363,'RVSN' from dual union all
select 13630,'RVSN' from dual union all
select 13631,'AMC' from dual union all
select 13632,'NASA' from dual union all
select 13633,'MO' from dual union all
select 13634,'MO' from dual union all
select 13635,'VMF' from dual union all
select 13636,'DOD' from dual union all
select 13636,'USAF' from dual union all
select 13637,'DOD' from dual union all
select 13637,'USAF' from dual union all
select 13638,'NPOE' from dual union all
select 13639,'USAF' from dual union all
select 1364,'MO' from dual union all
select 13640,'RVSN' from dual union all
select 13641,'MO' from dual union all
select 13642,'RVSN' from dual union all
select 13643,'USAF' from dual union all
select 13644,'RVSN' from dual union all
select 13645,'VMF' from dual union all
select 13646,'VMF' from dual union all
select 13647,'VMF' from dual union all
select 13648,'MO' from dual union all
select 13649,'RVSN' from dual union all
select 1365,'MO' from dual union all
select 13650,'NASA' from dual union all
select 13651,'SBS' from dual union all
select 13652,'TCAN' from dual union all
select 13653,'VMF' from dual union all
select 13654,'MO' from dual union all
select 13655,'MO' from dual union all
select 13656,'MO' from dual union all
select 13657,'MO' from dual union all
select 13658,'SBS' from dual union all
select 13659,'CIA' from dual union all
select 13659,'NRO' from dual union all
select 1366,'OKB1' from dual union all
select 13660,'USAF' from dual union all
select 13661,'MO' from dual union all
select 13662,'RVSN' from dual union all
select 13663,'MAIO' from dual union all
select 13663,'MOM' from dual union all
select 13664,'MAIO' from dual union all
select 13665,'MO' from dual union all
select 13666,'TCAN' from dual union all
select 13667,'PVO' from dual union all
select 13668,'PVO' from dual union all
select 13669,'MO' from dual union all
select 1367,'CIA' from dual union all
select 1367,'NRO' from dual union all
select 13670,'RVSN' from dual union all
select 13671,'RVSN' from dual union all
select 13672,'MO' from dual union all
select 13673,'MO' from dual union all
select 13674,'MO' from dual union all
select 13675,'RVSN' from dual union all
select 13676,'RVSN' from dual union all
select 13677,'MO' from dual union all
select 13678,'RVSN' from dual union all
select 13679,'MO' from dual union all
select 1368,'RVSN' from dual union all
select 13680,'VMF' from dual union all
select 13681,'VMF' from dual union all
select 13682,'VMF' from dual union all
select 13683,'PVO' from dual union all
select 13684,'NASA' from dual union all
select 13685,'MO' from dual union all
select 13686,'RVSN' from dual union all
select 13687,'VMF' from dual union all
select 13688,'VMF' from dual union all
select 13689,'VMF' from dual union all
select 1369,'RVSN' from dual union all
select 13690,'VMF' from dual union all
select 13691,'VMF' from dual union all
select 13692,'RVSN' from dual;
insert into SATELLITE_ORG
select 13693,'RVSN' from dual union all
select 13694,'PVO' from dual union all
select 13695,'PVO' from dual union all
select 13696,'RVSN' from dual union all
select 13697,'RVSN' from dual union all
select 13698,'RVSN' from dual union all
select 13699,'RVSN' from dual union all
select 137,'USAF' from dual union all
select 1370,'RVSN' from dual union all
select 13700,'NPOE' from dual union all
select 13701,'RVSN' from dual union all
select 13702,'RVSN' from dual union all
select 13703,'RVSN' from dual union all
select 13704,'RVSN' from dual union all
select 13705,'RVSN' from dual union all
select 13706,'RVSN' from dual union all
select 13707,'RVSN' from dual union all
select 13708,'RVSN' from dual union all
select 13709,'RVSN' from dual union all
select 1371,'RVSN' from dual union all
select 13710,'RVSN' from dual union all
select 13711,'RVSN' from dual union all
select 13712,'RVSN' from dual union all
select 13713,'RVSN' from dual union all
select 13714,'RVSN' from dual union all
select 13715,'RVSN' from dual union all
select 13716,'RVSN' from dual union all
select 13717,'RVSN' from dual union all
select 13718,'GMS' from dual union all
select 13718,'MO' from dual union all
select 13719,'RVSN' from dual union all
select 1372,'RVSN' from dual union all
select 13720,'MO' from dual union all
select 13721,'RVSN' from dual union all
select 13722,'RVSN' from dual union all
select 13723,'PVO' from dual union all
select 13724,'PVO' from dual union all
select 13725,'MO' from dual union all
select 13726,'RVSN' from dual union all
select 13727,'MO' from dual union all
select 13728,'MO' from dual union all
select 13729,'PVO' from dual union all
select 1373,'RVSN' from dual union all
select 13730,'MO' from dual union all
select 13731,'PVO' from dual union all
select 13732,'PVO' from dual union all
select 13733,'PVO' from dual union all
select 13734,'PVO' from dual union all
select 13735,'PVO' from dual union all
select 13736,'USAF' from dual union all
select 13737,'USAF' from dual union all
select 13738,'USAF' from dual union all
select 13739,'MO' from dual union all
select 1374,'CIA' from dual union all
select 1374,'NRO' from dual union all
select 13740,'RVSN' from dual union all
select 13741,'MO' from dual union all
select 13742,'MO' from dual union all
select 13743,'PVO' from dual union all
select 13744,'PVO' from dual union all
select 13745,'TSSKB' from dual union all
select 13746,'RVSN' from dual union all
select 13747,'VMF' from dual union all
select 13748,'VMF' from dual union all
select 13749,'MO' from dual union all
select 1375,'RVSN' from dual union all
select 13750,'PVO' from dual union all
select 13751,'RVSN' from dual union all
select 13752,'MO' from dual union all
select 13753,'USAF' from dual union all
select 13754,'MO' from dual union all
select 13755,'MO' from dual union all
select 13756,'MO' from dual union all
select 13757,'VMF' from dual union all
select 13758,'RVSN' from dual union all
select 13759,'MO' from dual union all
select 1376,'USAF' from dual union all
select 13760,'MO' from dual union all
select 13761,'MO' from dual union all
select 13762,'MO' from dual union all
select 13763,'MO' from dual union all
select 13764,'MO' from dual union all
select 13765,'MO' from dual union all
select 13766,'MO' from dual union all
select 13767,'MO' from dual union all
select 13768,'MO' from dual union all
select 13769,'RVSN' from dual union all
select 1377,'USAF' from dual union all
select 13770,'VS' from dual union all
select 13771,'RVSN' from dual union all
select 13772,'PVO' from dual union all
select 13773,'USAF' from dual union all
select 13774,'USAF' from dual union all
select 13775,'PVO' from dual union all
select 13776,'PVO' from dual union all
select 13777,'JPL' from dual union all
select 13777,'NIVR' from dual union all
select 13778,'NASA' from dual union all
select 13779,'MO' from dual union all
select 1378,'USAF' from dual;
insert into SATELLITE_ORG
select 13780,'RVSN' from dual union all
select 13781,'VMF' from dual union all
select 13782,'NASDA' from dual union all
select 13783,'JPL' from dual union all
select 13784,'MO' from dual union all
select 13785,'RVSN' from dual union all
select 13786,'NASDA' from dual union all
select 13787,'MO' from dual union all
select 13788,'MO' from dual union all
select 13789,'MO' from dual union all
select 1379,'USAF' from dual union all
select 13790,'MO' from dual union all
select 13791,'NRO' from dual union all
select 13791,'USN' from dual union all
select 13792,'USN' from dual union all
select 13793,'MVD' from dual union all
select 13794,'RVSN' from dual union all
select 13795,'NASA' from dual union all
select 13796,'MO' from dual union all
select 13797,'MO' from dual union all
select 13798,'MO' from dual union all
select 13799,'MO' from dual union all
select 138,'USAF' from dual union all
select 1380,'USAF' from dual union all
select 13800,'MO' from dual union all
select 13801,'MO' from dual union all
select 13802,'MO' from dual union all
select 13803,'MO' from dual union all
select 13804,'MO' from dual union all
select 13805,'MO' from dual union all
select 13806,'MO' from dual union all
select 13807,'MO' from dual union all
select 13808,'MO' from dual union all
select 13809,'MO' from dual union all
select 1381,'NASA' from dual union all
select 13810,'MO' from dual union all
select 13811,'MO' from dual union all
select 13812,'MO' from dual union all
select 13813,'MO' from dual union all
select 13814,'MO' from dual union all
select 13815,'MO' from dual union all
select 13816,'MO' from dual union all
select 13817,'MO' from dual union all
select 13818,'VS' from dual union all
select 13819,'RVSN' from dual union all
select 1382,'OKB1F3' from dual union all
select 13820,'PVO' from dual union all
select 13821,'PVO' from dual union all
select 13822,'PVO' from dual union all
select 13823,'PVO' from dual union all
select 13824,'PVO' from dual union all
select 13825,'NASA' from dual union all
select 13826,'NASA' from dual union all
select 13827,'NASA' from dual union all
select 13828,'PVO' from dual union all
select 13829,'MONBU' from dual union all
select 1383,'RVSN' from dual union all
select 13830,'MONBU' from dual union all
select 13831,'PVO' from dual union all
select 13832,'PVO' from dual union all
select 13833,'PVO' from dual union all
select 13834,'USN' from dual union all
select 13835,'USN' from dual union all
select 13836,'PVO' from dual union all
select 13837,'PVO' from dual union all
select 13838,'PVO' from dual union all
select 13839,'PVO' from dual union all
select 1384,'OKB1F3' from dual union all
select 13840,'PVO' from dual union all
select 13841,'PVO' from dual union all
select 13842,'PVO' from dual union all
select 13843,'MO' from dual union all
select 13844,'NRO' from dual union all
select 13844,'USN' from dual union all
select 13845,'NRO' from dual union all
select 13845,'USN' from dual union all
select 13846,'MONBU' from dual union all
select 13847,'MO' from dual union all
select 13848,'MO' from dual union all
select 13849,'NRO' from dual union all
select 13849,'USN' from dual union all
select 1385,'NASA' from dual union all
select 13850,'MO' from dual union all
select 13851,'RVSN' from dual union all
select 13852,'PVO' from dual union all
select 13853,'PVO' from dual union all
select 13854,'PVO' from dual union all
select 13855,'PVO' from dual union all
select 13856,'PVO' from dual union all
select 13857,'PVO' from dual union all
select 13858,'PVO' from dual union all
select 13859,'PVO' from dual union all
select 1386,'NRO' from dual union all
select 13860,'PVO' from dual union all
select 13861,'PVO' from dual union all
select 13862,'PVO' from dual union all
select 13863,'PVO' from dual union all
select 13864,'PVO' from dual union all
select 13865,'PVO' from dual union all
select 13866,'PVO' from dual;
insert into SATELLITE_ORG
select 13867,'PVO' from dual union all
select 13868,'KBSAL' from dual union all
select 13869,'RVSN' from dual union all
select 1387,'OKB1F3' from dual union all
select 13870,'MO' from dual union all
select 13871,'RVSN' from dual union all
select 13872,'MO' from dual union all
select 13873,'MO' from dual union all
select 13874,'NRO' from dual union all
select 13874,'USN' from dual union all
select 13875,'MOM' from dual union all
select 13875,'MSVYAZ' from dual union all
select 13876,'RVSN' from dual union all
select 13877,'RVSN' from dual union all
select 13878,'MSVYAZ' from dual union all
select 13879,'RVSN' from dual union all
select 1388,'NASA' from dual union all
select 13880,'RVSN' from dual union all
select 13881,'RVSN' from dual union all
select 13882,'RVSN' from dual union all
select 13883,'MO' from dual union all
select 13883,'MOLN' from dual union all
select 13884,'RVSN' from dual union all
select 13885,'RVSN' from dual union all
select 13886,'MO' from dual union all
select 13887,'RVSN' from dual union all
select 13888,'RVSN' from dual union all
select 13889,'MOLN' from dual union all
select 1389,'USAF' from dual union all
select 13890,'MO' from dual union all
select 13891,'MO' from dual union all
select 13892,'MO' from dual union all
select 13893,'MO' from dual union all
select 13894,'MO' from dual union all
select 13895,'RVSN' from dual union all
select 13896,'RVSN' from dual union all
select 13897,'RVSN' from dual union all
select 13898,'RVSN' from dual union all
select 13899,'RVSN' from dual union all
select 139,'USAF' from dual union all
select 1390,'NASA' from dual union all
select 13900,'RVSN' from dual union all
select 13901,'IKI' from dual union all
select 13901,'NPOL' from dual union all
select 13902,'RVSN' from dual union all
select 13903,'RVSN' from dual union all
select 13904,'RVSN' from dual union all
select 13905,'UNK' from dual union all
select 13906,'USAF' from dual union all
select 13907,'ESA' from dual union all
select 13908,'RVSN' from dual union all
select 13909,'RVSN' from dual union all
select 1391,'USAF' from dual union all
select 13910,'RVSN' from dual union all
select 13911,'RVSN' from dual union all
select 13912,'USAF' from dual union all
select 13913,'RVSN' from dual union all
select 13914,'RVSN' from dual union all
select 13915,'NASA' from dual union all
select 13916,'MOM' from dual union all
select 13917,'RVSN' from dual union all
select 13918,'VMF' from dual union all
select 13919,'VMF' from dual union all
select 1392,'RVSN' from dual union all
select 13920,'VMF' from dual union all
select 13921,'VMF' from dual union all
select 13922,'VMF' from dual union all
select 13923,'NOAA' from dual union all
select 13924,'NASA' from dual union all
select 13925,'PVO' from dual union all
select 13926,'PVO' from dual union all
select 13927,'PVO' from dual union all
select 13928,'PVO' from dual union all
select 13929,'PVO' from dual union all
select 1393,'OKB1' from dual union all
select 13930,'PVO' from dual union all
select 13931,'PVO' from dual union all
select 13932,'PVO' from dual union all
select 13933,'NASA' from dual union all
select 13934,'NASA' from dual union all
select 13935,'NASA' from dual union all
select 13936,'PVO' from dual union all
select 13937,'RVSN' from dual union all
select 13938,'RVSN' from dual union all
select 13939,'NASA' from dual union all
select 1394,'RVSN' from dual union all
select 13940,'NASA' from dual union all
select 13941,'USAF' from dual union all
select 13942,'USAF' from dual union all
select 13943,'PVO' from dual union all
select 13944,'PVO' from dual union all
select 13945,'PVO' from dual union all
select 13946,'PVO' from dual union all
select 13947,'PVO' from dual union all
select 13948,'PVO' from dual union all
select 13949,'VMF' from dual union all
select 1395,'RVSN' from dual union all
select 13950,'RVSN' from dual union all
select 13951,'MO' from dual union all
select 13952,'MO' from dual;
insert into SATELLITE_ORG
select 13953,'MO' from dual union all
select 13954,'RVSN' from dual union all
select 13955,'MO' from dual union all
select 13956,'RVSN' from dual union all
select 13957,'USAF' from dual union all
select 13958,'USAF' from dual union all
select 13959,'RVSN' from dual union all
select 1396,'CIA' from dual union all
select 1396,'NRO' from dual union all
select 13960,'RVSN' from dual union all
select 13961,'MO' from dual union all
select 13962,'MO' from dual union all
select 13963,'MO' from dual union all
select 13964,'MO' from dual union all
select 13965,'RVSN' from dual union all
select 13966,'RVSN' from dual union all
select 13967,'RVSN' from dual union all
select 13968,'NASA' from dual union all
select 13969,'NASA' from dual union all
select 1397,'RVSN' from dual union all
select 13970,'NASA' from dual union all
select 13971,'NASA' from dual union all
select 13972,'PVO' from dual union all
select 13973,'RVSN' from dual union all
select 13974,'MO' from dual union all
select 13975,'MO' from dual union all
select 13976,'RVSN' from dual union all
select 13977,'RVSN' from dual union all
select 13978,'RVSN' from dual union all
select 13979,'RVSN' from dual union all
select 1398,'RVSN' from dual union all
select 13980,'RVSN' from dual union all
select 13981,'PVO' from dual union all
select 13982,'PVO' from dual union all
select 13983,'RVSN' from dual union all
select 13984,'AMC' from dual union all
select 13985,'NASA' from dual union all
select 13986,'PVO' from dual union all
select 13987,'PVO' from dual union all
select 13988,'PVO' from dual union all
select 13989,'PVO' from dual union all
select 1399,'USAF' from dual union all
select 13990,'NASA' from dual union all
select 13991,'MO' from dual union all
select 13992,'RVSN' from dual union all
select 13993,'MO' from dual union all
select 13994,'MO' from dual union all
select 13995,'MO' from dual union all
select 13996,'MO' from dual union all
select 13997,'MO' from dual union all
select 13998,'NASA' from dual union all
select 13999,'PVO' from dual union all
select 14,'CIA' from dual union all
select 14,'DOD' from dual union all
select 140,'USAF' from dual union all
select 1400,'RVSN' from dual union all
select 14000,'PVO' from dual union all
select 14001,'NRO' from dual union all
select 14002,'ISRO' from dual union all
select 14003,'ISRO' from dual union all
select 14004,'ISRO' from dual union all
select 14005,'RVSN' from dual union all
select 14006,'PVO' from dual union all
select 14007,'RVSN' from dual union all
select 14008,'PVO' from dual union all
select 14009,'PVO' from dual union all
select 1401,'RVSN' from dual union all
select 14010,'PVO' from dual union all
select 14011,'PVO' from dual union all
select 14012,'PVO' from dual union all
select 14013,'PVO' from dual union all
select 14014,'NPOE' from dual union all
select 14015,'RVSN' from dual union all
select 14016,'MO' from dual union all
select 14017,'MO' from dual union all
select 14018,'PVO' from dual union all
select 14019,'PVO' from dual union all
select 1402,'RVSN' from dual union all
select 14020,'PVO' from dual union all
select 14021,'PVO' from dual union all
select 14022,'PVO' from dual union all
select 14023,'PVO' from dual union all
select 14024,'PVO' from dual union all
select 14025,'PVO' from dual union all
select 14026,'MO' from dual union all
select 14027,'MO' from dual union all
select 14028,'RVSN' from dual union all
select 14029,'NPOE' from dual union all
select 1403,'RVSN' from dual union all
select 14030,'MO' from dual union all
select 14031,'PVO' from dual union all
select 14032,'VS' from dual union all
select 14033,'MO' from dual union all
select 14034,'PVO' from dual union all
select 14035,'PVO' from dual union all
select 14036,'RVSN' from dual union all
select 14037,'RVSN' from dual union all
select 14038,'RVSN' from dual union all
select 14039,'MO' from dual union all
select 1404,'MO' from dual;
insert into SATELLITE_ORG
select 14040,'RVSN' from dual union all
select 14041,'RVSN' from dual union all
select 14042,'NASA' from dual union all
select 14043,'RVSN' from dual union all
select 14044,'MVD' from dual union all
select 14045,'RVSN' from dual union all
select 14046,'MO' from dual union all
select 14047,'USAF' from dual union all
select 14048,'USAF' from dual union all
select 14049,'USAF' from dual union all
select 1405,'RVSN' from dual union all
select 14050,'NOAA' from dual union all
select 14051,'NASA' from dual union all
select 14052,'PVO' from dual union all
select 14053,'PVO' from dual union all
select 14054,'PVO' from dual union all
select 14055,'PVO' from dual union all
select 14056,'PVO' from dual union all
select 14057,'VMF' from dual union all
select 14058,'MO' from dual union all
select 14059,'RVSN' from dual union all
select 1406,'MO' from dual union all
select 14060,'RVSN' from dual union all
select 14061,'MO' from dual union all
select 14062,'MO' from dual union all
select 14063,'MO' from dual union all
select 14064,'VMF' from dual union all
select 14065,'MO' from dual union all
select 14066,'MO' from dual union all
select 14067,'MO' from dual union all
select 14068,'PVO' from dual union all
select 14069,'NASA' from dual union all
select 1407,'MO' from dual union all
select 14070,'RVSN' from dual union all
select 14071,'MVD' from dual union all
select 14072,'RVSN' from dual union all
select 14073,'MO' from dual union all
select 14074,'MO' from dual union all
select 14075,'PVO' from dual union all
select 14076,'RVSN' from dual union all
select 14077,'INTEL' from dual union all
select 14078,'MO' from dual union all
select 14079,'MO' from dual union all
select 1408,'MO' from dual union all
select 14080,'MO' from dual union all
select 14081,'NASA' from dual union all
select 14082,'MO' from dual union all
select 14083,'MO' from dual union all
select 14084,'VMF' from dual union all
select 14085,'RVSN' from dual union all
select 14086,'RVSN' from dual union all
select 14087,'PVO' from dual union all
select 14088,'RVSN' from dual union all
select 14089,'MO' from dual union all
select 1409,'RVSN' from dual union all
select 14090,'RVSN' from dual union all
select 14091,'PVO' from dual union all
select 14092,'PVO' from dual union all
select 14093,'PVO' from dual union all
select 14094,'PVO' from dual union all
select 14095,'ESA' from dual union all
select 14096,'NASA' from dual union all
select 14097,'MO' from dual union all
select 14098,'MO' from dual union all
select 14099,'MO' from dual union all
select 141,'USAF' from dual union all
select 1410,'RVSN' from dual union all
select 14100,'MO' from dual union all
select 14101,'RVSN' from dual union all
select 14102,'MO' from dual union all
select 14103,'MO' from dual union all
select 14104,'IKI' from dual union all
select 14104,'NPOL' from dual union all
select 14105,'RVSN' from dual union all
select 14106,'RVSN' from dual union all
select 14107,'IKI' from dual union all
select 14107,'NPOL' from dual union all
select 14108,'RVSN' from dual union all
select 14109,'RVSN' from dual union all
select 1411,'NASA' from dual union all
select 14110,'MVD' from dual union all
select 14111,'RVSN' from dual union all
select 14112,'NRO' from dual union all
select 14112,'USN' from dual union all
select 14113,'USN' from dual union all
select 14114,'RVSN' from dual union all
select 14115,'RVSN' from dual union all
select 14116,'MO' from dual union all
select 14117,'RVSN' from dual union all
select 14118,'MO' from dual union all
select 14119,'MO' from dual union all
select 1412,'USAF' from dual union all
select 14120,'MO' from dual union all
select 14121,'MO' from dual union all
select 14122,'PVO' from dual union all
select 14123,'MO' from dual union all
select 14124,'RVSN' from dual union all
select 14125,'ESA' from dual union all
select 14126,'MO' from dual union all
select 14127,'MO' from dual;
insert into SATELLITE_ORG
select 14128,'ESA' from dual union all
select 14129,'AMSAT' from dual union all
select 1413,'USAF' from dual union all
select 14130,'AE' from dual union all
select 14131,'RVSN' from dual union all
select 14132,'NASA' from dual union all
select 14133,'TCAN' from dual union all
select 14134,'PER' from dual union all
select 14135,'TCAN' from dual union all
select 14136,'PER' from dual union all
select 14137,'NRO' from dual union all
select 14138,'USAF' from dual union all
select 14139,'NRO' from dual union all
select 1414,'CIA' from dual union all
select 1414,'NRO' from dual union all
select 14140,'MO' from dual union all
select 14141,'MO' from dual union all
select 14142,'MBB' from dual union all
select 14143,'NRO' from dual union all
select 14143,'USN' from dual union all
select 14144,'NRO' from dual union all
select 14144,'USN' from dual union all
select 14145,'USN' from dual union all
select 14146,'NRO' from dual union all
select 14147,'VS' from dual union all
select 14148,'MO' from dual union all
select 14149,'MO' from dual union all
select 1415,'CIA' from dual union all
select 1415,'NRO' from dual union all
select 14150,'MO' from dual union all
select 14151,'AE' from dual union all
select 14152,'NPOE' from dual union all
select 14153,'RVSN' from dual union all
select 14154,'USAF' from dual union all
select 14155,'NASA' from dual union all
select 14156,'MO' from dual union all
select 14157,'RVSN' from dual union all
select 14158,'HEC' from dual union all
select 14159,'NASA' from dual union all
select 1416,'RVSN' from dual union all
select 14160,'MSVYAZ' from dual union all
select 14161,'RVSN' from dual union all
select 14162,'RVSN' from dual union all
select 14163,'IKI' from dual union all
select 14163,'NPOL' from dual union all
select 14164,'RVSN' from dual union all
select 14165,'RVSN' from dual union all
select 14166,'RVSN' from dual union all
select 14167,'RVSN' from dual union all
select 14168,'NASA' from dual union all
select 14169,'MO' from dual union all
select 1417,'RVSN' from dual union all
select 14170,'RVSN' from dual union all
select 14171,'MO' from dual union all
select 14172,'MO' from dual union all
select 14173,'MO' from dual union all
select 14174,'MO' from dual union all
select 14175,'MO' from dual union all
select 14176,'MO' from dual union all
select 14177,'MO' from dual union all
select 14178,'MO' from dual union all
select 14179,'RVSN' from dual union all
select 1418,'RVSN' from dual union all
select 14180,'NRO' from dual union all
select 14180,'USN' from dual union all
select 14181,'USN' from dual union all
select 14182,'PVO' from dual union all
select 14183,'RVSN' from dual union all
select 14184,'RVSN' from dual union all
select 14185,'MO' from dual union all
select 14186,'RVSN' from dual union all
select 14187,'MO' from dual union all
select 14188,'MO' from dual union all
select 14189,'USAF' from dual union all
select 1419,'RVSN' from dual union all
select 14190,'USAF' from dual union all
select 14191,'RVSN' from dual union all
select 14192,'RVSN' from dual union all
select 14193,'RVSN' from dual union all
select 14194,'RVSN' from dual union all
select 14195,'RVSN' from dual union all
select 14196,'NASA' from dual union all
select 14197,'MO' from dual union all
select 14198,'MO' from dual union all
select 14199,'MO' from dual union all
select 142,'USAF' from dual union all
select 1420,'USN' from dual union all
select 14200,'RVSN' from dual union all
select 14201,'RVSN' from dual union all
select 14202,'MO' from dual union all
select 14203,'MO' from dual union all
select 14204,'MVD' from dual union all
select 14205,'RVSN' from dual union all
select 14206,'RVSN' from dual union all
select 14207,'MVD' from dual union all
select 14208,'RVSN' from dual union all
select 14209,'MO' from dual union all
select 1421,'OKB1F3' from dual union all
select 14210,'MO' from dual union all
select 14211,'RVSN' from dual;
insert into SATELLITE_ORG
select 14212,'PVO' from dual union all
select 14213,'PVO' from dual union all
select 14214,'PVO' from dual union all
select 14215,'PVO' from dual union all
select 14216,'MO' from dual union all
select 14217,'MO' from dual union all
select 14218,'MO' from dual union all
select 14219,'MO' from dual union all
select 1422,'NRO' from dual union all
select 14220,'MO' from dual union all
select 14221,'MO' from dual union all
select 14222,'USAF' from dual union all
select 14223,'USAF' from dual union all
select 14224,'ISRO' from dual union all
select 14225,'ISRO' from dual union all
select 14226,'USN' from dual union all
select 14227,'MO' from dual union all
select 14228,'MO' from dual union all
select 14229,'MO' from dual union all
select 1423,'RVSN' from dual union all
select 14230,'MO' from dual union all
select 14231,'MO' from dual union all
select 14232,'MO' from dual union all
select 14233,'MO' from dual union all
select 14234,'ATT' from dual union all
select 14235,'NASA' from dual union all
select 14236,'NASA' from dual union all
select 14237,'NRO' from dual union all
select 14238,'USAF' from dual union all
select 14239,'MO' from dual union all
select 1424,'NRO' from dual union all
select 14240,'MO' from dual union all
select 14241,'RVSN' from dual union all
select 14242,'MO' from dual union all
select 14243,'MO' from dual union all
select 14244,'JPL' from dual union all
select 14245,'MVD' from dual union all
select 14246,'RVSN' from dual union all
select 14247,'MO' from dual union all
select 14248,'NASDA' from dual union all
select 14249,'MO' from dual union all
select 1425,'USN' from dual union all
select 14250,'NASDA' from dual union all
select 14251,'MO' from dual union all
select 14252,'RVSN' from dual union all
select 14253,'MO' from dual union all
select 14254,'MO' from dual union all
select 14255,'MO' from dual union all
select 14256,'MO' from dual union all
select 14257,'RVSN' from dual union all
select 14258,'MO' from dual union all
select 14259,'MO' from dual union all
select 1426,'USAF' from dual union all
select 14260,'MO' from dual union all
select 14261,'RVSN' from dual union all
select 14262,'RVSN' from dual union all
select 14263,'MO' from dual union all
select 14264,'RVSN' from dual union all
select 14265,'VMF' from dual union all
select 14266,'VMF' from dual union all
select 14267,'VMF' from dual union all
select 14268,'VMF' from dual union all
select 14269,'VMF' from dual union all
select 1427,'USAF' from dual union all
select 14270,'VMF' from dual union all
select 14271,'VMF' from dual union all
select 14272,'VMF' from dual union all
select 14273,'VMF' from dual union all
select 14274,'VMF' from dual union all
select 14275,'VMF' from dual union all
select 14276,'VMF' from dual union all
select 14277,'RVSN' from dual union all
select 14278,'RVSN' from dual union all
select 14279,'VMF' from dual union all
select 1428,'USAF' from dual union all
select 14280,'VMF' from dual union all
select 14281,'VMF' from dual union all
select 14282,'VMF' from dual union all
select 14283,'NPOE' from dual union all
select 14284,'RVSN' from dual union all
select 14285,'VMF' from dual union all
select 14286,'VMF' from dual union all
select 14287,'NASDA' from dual union all
select 14288,'MAI' from dual union all
select 14289,'MAI' from dual union all
select 1429,'OKB1F3' from dual union all
select 14290,'MAI' from dual union all
select 14291,'MO' from dual union all
select 14292,'MO' from dual union all
select 14293,'MAI' from dual union all
select 14294,'MAI' from dual union all
select 14295,'MO' from dual union all
select 14296,'MAI' from dual union all
select 14297,'MO' from dual union all
select 14298,'MO' from dual union all
select 14299,'MO' from dual union all
select 143,'USAF' from dual union all
select 1430,'ESSA' from dual union all
select 14300,'RVSN' from dual union all
select 14301,'MO' from dual;
insert into SATELLITE_ORG
select 14302,'MO' from dual union all
select 14303,'MO' from dual union all
select 14304,'MO' from dual union all
select 14305,'MO' from dual union all
select 14306,'MO' from dual union all
select 14307,'MO' from dual union all
select 14308,'RVSN' from dual union all
select 14309,'RVSN' from dual union all
select 1431,'YUZH' from dual union all
select 14310,'RVSN' from dual union all
select 14311,'RVSN' from dual union all
select 14312,'NASA' from dual union all
select 14313,'MOM' from dual union all
select 14313,'MSVYAZ' from dual union all
select 14314,'RVSN' from dual union all
select 14315,'RVSN' from dual union all
select 14316,'PVO' from dual union all
select 14317,'RVSN' from dual union all
select 14318,'ISRO' from dual union all
select 14319,'RVSN' from dual union all
select 1432,'RVSN' from dual union all
select 14320,'MVD' from dual union all
select 14321,'RVSN' from dual union all
select 14322,'MO' from dual union all
select 14323,'MO' from dual union all
select 14324,'MO' from dual union all
select 14325,'MO' from dual union all
select 14326,'MO' from dual union all
select 14327,'RVSN' from dual union all
select 14328,'AMC' from dual union all
select 14329,'NASA' from dual union all
select 1433,'NASA' from dual union all
select 14330,'MO' from dual union all
select 14331,'RVSN' from dual union all
select 14332,'NASA' from dual union all
select 14333,'RVSN' from dual union all
select 14334,'MVD' from dual union all
select 14335,'RVSN' from dual union all
select 14336,'MO' from dual union all
select 14337,'MO' from dual union all
select 14338,'MO' from dual union all
select 14339,'MO' from dual union all
select 1434,'YUZH' from dual union all
select 14340,'RVSN' from dual union all
select 14341,'MO' from dual union all
select 14342,'VMF' from dual union all
select 14343,'RVSN' from dual union all
select 14344,'MO' from dual union all
select 14345,'RVSN' from dual union all
select 14346,'MO' from dual union all
select 14347,'MO' from dual union all
select 14348,'VMF' from dual union all
select 14349,'VMF' from dual union all
select 1435,'USN' from dual union all
select 14350,'VMF' from dual union all
select 14351,'VMF' from dual union all
select 14352,'VMF' from dual union all
select 14353,'VMF' from dual union all
select 14354,'VMF' from dual union all
select 14355,'VMF' from dual union all
select 14356,'MO' from dual union all
select 14357,'MO' from dual union all
select 14358,'MO' from dual union all
select 14359,'MO' from dual union all
select 1436,'RVSN' from dual union all
select 14360,'MO' from dual union all
select 14361,'MO' from dual union all
select 14362,'MO' from dual union all
select 14363,'MO' from dual union all
select 14364,'MO' from dual union all
select 14365,'HEC' from dual union all
select 14366,'NASA' from dual union all
select 14367,'MO' from dual union all
select 14368,'MO' from dual union all
select 14369,'NASA' from dual union all
select 1437,'RVSN' from dual union all
select 14370,'MO' from dual union all
select 14371,'MO' from dual union all
select 14372,'GMS' from dual union all
select 14372,'YUZH' from dual union all
select 14373,'MO' from dual union all
select 14374,'MO' from dual union all
select 14375,'MO' from dual union all
select 14376,'MO' from dual union all
select 14377,'MSVYAZ' from dual union all
select 14378,'RVSN' from dual union all
select 14379,'RVSN' from dual union all
select 1438,'RVSN' from dual union all
select 14380,'PVO' from dual union all
select 14381,'RVSN' from dual union all
select 14382,'MO' from dual union all
select 14383,'MO' from dual union all
select 14384,'MO' from dual union all
select 14385,'MO' from dual union all
select 14386,'NPOE' from dual union all
select 14387,'NPOE' from dual union all
select 14388,'NPOE' from dual union all
select 14389,'RVSN' from dual union all
select 1439,'RVSN' from dual union all
select 14390,'RVSN' from dual;
insert into SATELLITE_ORG
select 14391,'MO' from dual union all
select 14392,'NPOE' from dual union all
select 14393,'NPOE' from dual union all
select 14394,'RVSN' from dual union all
select 14395,'PVO' from dual union all
select 14396,'RVSN' from dual union all
select 14397,'ISRO' from dual union all
select 14398,'MO' from dual union all
select 14399,'MO' from dual union all
select 144,'USAF' from dual union all
select 1440,'ESSA' from dual union all
select 14400,'MO' from dual union all
select 14401,'MO' from dual union all
select 14402,'RVSN' from dual union all
select 14403,'TSSKB' from dual union all
select 14404,'RVSN' from dual union all
select 14405,'VMF' from dual union all
select 14406,'VMF' from dual union all
select 14407,'VMF' from dual union all
select 14408,'VMF' from dual union all
select 14409,'VMF' from dual union all
select 1441,'OKB10' from dual union all
select 14410,'PVO' from dual union all
select 14411,'PVO' from dual union all
select 14412,'PVO' from dual union all
select 14413,'PVO' from dual union all
select 14414,'PVO' from dual union all
select 14415,'PVO' from dual union all
select 14416,'PVO' from dual union all
select 14417,'PVO' from dual union all
select 14418,'PVO' from dual union all
select 14419,'RVSN' from dual union all
select 1442,'OKB10' from dual union all
select 14420,'PVO' from dual union all
select 14421,'INTEL' from dual union all
select 14422,'NPOE' from dual union all
select 14423,'AE' from dual union all
select 14424,'RVSN' from dual union all
select 14425,'MO' from dual union all
select 14426,'RVSN' from dual union all
select 14427,'RVSN' from dual union all
select 14428,'RVSN' from dual union all
select 14429,'PVO' from dual union all
select 1443,'OKB10' from dual union all
select 14430,'PVO' from dual union all
select 14431,'VMF' from dual union all
select 14432,'VMF' from dual union all
select 14433,'VMF' from dual union all
select 14434,'RVSN' from dual union all
select 14435,'VMF' from dual union all
select 14436,'VMF' from dual union all
select 14437,'VMF' from dual union all
select 14438,'VMF' from dual union all
select 14439,'VMF' from dual union all
select 1444,'OKB10' from dual union all
select 14440,'VMF' from dual union all
select 14441,'VMF' from dual union all
select 14442,'PVO' from dual union all
select 14443,'PVO' from dual union all
select 14444,'PVO' from dual union all
select 14445,'PVO' from dual union all
select 14446,'PVO' from dual union all
select 14447,'PVO' from dual union all
select 14448,'PVO' from dual union all
select 14449,'VMF' from dual union all
select 1445,'OKB10' from dual union all
select 14450,'VMF' from dual union all
select 14451,'RVSN' from dual union all
select 14452,'GMS' from dual union all
select 14452,'MO' from dual union all
select 14453,'RVSN' from dual union all
select 14454,'MO' from dual union all
select 14455,'VMF' from dual union all
select 14456,'MO' from dual union all
select 14457,'NASA' from dual union all
select 14458,'NASA' from dual union all
select 14459,'NASA' from dual union all
select 1446,'RVSN' from dual union all
select 14460,'NASA' from dual union all
select 14461,'NASA' from dual union all
select 14462,'NASA' from dual union all
select 14463,'NASA' from dual union all
select 14464,'NASA' from dual union all
select 14465,'NASA' from dual union all
select 14466,'NASA' from dual union all
select 14467,'NASA' from dual union all
select 14468,'NASA' from dual union all
select 14469,'NASA' from dual union all
select 1447,'NRO' from dual union all
select 14470,'NASA' from dual union all
select 14471,'NASA' from dual union all
select 14472,'NASA' from dual union all
select 14473,'NASA' from dual union all
select 14474,'NASA' from dual union all
select 14475,'NASA' from dual union all
select 14476,'NASA' from dual union all
select 14477,'NASA' from dual union all
select 14478,'NASA' from dual union all
select 14479,'NASA' from dual union all
select 1448,'RVSN' from dual;
insert into SATELLITE_ORG
select 14480,'NASA' from dual union all
select 14481,'MO' from dual union all
select 14482,'MO' from dual union all
select 14483,'PVO' from dual union all
select 14484,'RVSN' from dual union all
select 14485,'NPOE' from dual union all
select 14486,'NPOE' from dual union all
select 14487,'NPOE' from dual union all
select 14488,'NPOE' from dual union all
select 14489,'NPOE' from dual union all
select 1449,'RVSN' from dual union all
select 14490,'MO' from dual union all
select 14491,'NASA' from dual union all
select 14492,'NASA' from dual union all
select 14493,'NASA' from dual union all
select 14494,'NASA' from dual union all
select 14495,'NASA' from dual union all
select 14496,'NASA' from dual union all
select 14497,'NASA' from dual union all
select 14498,'RVSN' from dual union all
select 14499,'USAF' from dual union all
select 145,'USAF' from dual union all
select 1450,'RVSN' from dual union all
select 14500,'USAF' from dual union all
select 14501,'PVO' from dual union all
select 14502,'PVO' from dual union all
select 14503,'PVO' from dual union all
select 14504,'VMF' from dual union all
select 14505,'CNES' from dual union all
select 14506,'USAF' from dual union all
select 14507,'NPOE' from dual union all
select 14508,'NPOE' from dual union all
select 14509,'NASA' from dual union all
select 1451,'RVSN' from dual union all
select 14510,'NASA' from dual union all
select 14511,'PVO' from dual union all
select 14512,'ISAS' from dual union all
select 14513,'MO' from dual union all
select 14514,'NASA' from dual union all
select 14515,'USAF' from dual union all
select 14516,'MO' from dual union all
select 14517,'RVSN' from dual union all
select 14518,'RVSN' from dual union all
select 14519,'NPOE' from dual union all
select 1452,'USAF' from dual union all
select 14520,'RVSN' from dual union all
select 14521,'MOM' from dual union all
select 14522,'MO' from dual union all
select 14523,'NASA' from dual union all
select 14524,'ISRO' from dual union all
select 14525,'NASA' from dual union all
select 14526,'NASA' from dual union all
select 14527,'PVO' from dual union all
select 14528,'NASA' from dual union all
select 14529,'NASA' from dual union all
select 1453,'RVSN' from dual union all
select 14530,'MO' from dual union all
select 14531,'RVSN' from dual union all
select 14532,'MSVYAZ' from dual union all
select 14533,'RVSN' from dual union all
select 14534,'RVSN' from dual union all
select 14535,'MO' from dual union all
select 14536,'MO' from dual union all
select 14537,'RVSN' from dual union all
select 14538,'RVSN' from dual union all
select 14539,'MO' from dual union all
select 1454,'OKB1' from dual union all
select 14540,'MO' from dual union all
select 14541,'MO' from dual union all
select 14542,'MO' from dual union all
select 14543,'RVSN' from dual union all
select 14544,'MO' from dual union all
select 14545,'MO' from dual union all
select 14546,'VMF' from dual union all
select 14547,'RVSN' from dual union all
select 14548,'RVSN' from dual union all
select 14549,'IMBP' from dual union all
select 14549,'TSSKB' from dual union all
select 1455,'USAF' from dual union all
select 14550,'RVSN' from dual union all
select 14551,'VS' from dual union all
select 14552,'MO' from dual union all
select 14553,'USAF' from dual union all
select 14554,'USAF' from dual union all
select 14555,'MO' from dual union all
select 14556,'MO' from dual union all
select 14557,'MO' from dual union all
select 14558,'MO' from dual union all
select 14559,'NASA' from dual union all
select 1456,'RVSN' from dual union all
select 14560,'MO' from dual union all
select 14561,'MO' from dual union all
select 14562,'MO' from dual union all
select 14563,'NASA' from dual union all
select 14564,'NASA' from dual union all
select 14565,'NASA' from dual union all
select 14566,'MONBU' from dual union all
select 14567,'MONBU' from dual union all
select 14568,'RVSN' from dual union all
select 14569,'NASDA' from dual;
insert into SATELLITE_ORG
select 1457,'CIA' from dual union all
select 1457,'NRO' from dual union all
select 14570,'MOM' from dual union all
select 14570,'MSVYAZ' from dual union all
select 14571,'RVSN' from dual union all
select 14572,'RVSN' from dual union all
select 14573,'MO' from dual union all
select 14574,'MO' from dual union all
select 14575,'MO' from dual union all
select 14576,'MO' from dual union all
select 14577,'PVO' from dual union all
select 14578,'PVO' from dual union all
select 14579,'MO' from dual union all
select 1458,'USAF' from dual union all
select 14580,'MO' from dual union all
select 14581,'MO' from dual union all
select 14582,'RVSN' from dual union all
select 14583,'TSSKB' from dual union all
select 14584,'RVSN' from dual union all
select 14585,'MO' from dual union all
select 14585,'MOLN' from dual union all
select 14586,'RVSN' from dual union all
select 14587,'PVO' from dual union all
select 14588,'RVSN' from dual union all
select 14589,'RVSN' from dual union all
select 1459,'USAF' from dual union all
select 14590,'MO' from dual union all
select 14591,'MO' from dual union all
select 14592,'MO' from dual union all
select 14593,'RVSN' from dual union all
select 14594,'RVSN' from dual union all
select 14595,'RVSN' from dual union all
select 14596,'RVSN' from dual union all
select 14597,'PVO' from dual union all
select 14598,'PVO' from dual union all
select 14599,'VMF' from dual union all
select 146,'USAF' from dual union all
select 1460,'USAF' from dual union all
select 14600,'VMF' from dual union all
select 14601,'VMF' from dual union all
select 14602,'VMF' from dual union all
select 14603,'VMF' from dual union all
select 14604,'VMF' from dual union all
select 14605,'VMF' from dual union all
select 14606,'VMF' from dual union all
select 14607,'RVSN' from dual union all
select 14608,'RVSN' from dual union all
select 14609,'USAF' from dual union all
select 1461,'USAF' from dual union all
select 14610,'USAF' from dual union all
select 14611,'MO' from dual union all
select 14612,'MO' from dual union all
select 14613,'MO' from dual union all
select 14614,'MO' from dual union all
select 14615,'MO' from dual union all
select 14616,'MO' from dual union all
select 14617,'MO' from dual union all
select 14618,'MO' from dual union all
select 14619,'RVSN' from dual union all
select 1462,'USAF' from dual union all
select 14620,'NASA' from dual union all
select 14621,'NASA' from dual union all
select 14622,'MO' from dual union all
select 14623,'RVSN' from dual union all
select 14624,'VMF' from dual union all
select 14625,'RVSN' from dual union all
select 14626,'VMF' from dual union all
select 14627,'VMF' from dual union all
select 14628,'VMF' from dual union all
select 14629,'VMF' from dual union all
select 1463,'USAF' from dual union all
select 14630,'VMF' from dual union all
select 14631,'MO' from dual union all
select 14632,'RVSN' from dual union all
select 14633,'CNES' from dual union all
select 14634,'MO' from dual union all
select 14635,'RVSN' from dual union all
select 14636,'VMF' from dual union all
select 14637,'VMF' from dual union all
select 14638,'VMF' from dual union all
select 14639,'VMF' from dual union all
select 1464,'PVO' from dual union all
select 1464,'YUZH' from dual union all
select 14640,'VMF' from dual union all
select 14641,'VMF' from dual union all
select 14642,'VMF' from dual union all
select 14643,'VMF' from dual union all
select 14644,'VMF' from dual union all
select 14645,'VMF' from dual union all
select 14646,'VMF' from dual union all
select 14647,'VMF' from dual union all
select 14648,'VMF' from dual union all
select 14649,'VMF' from dual union all
select 1465,'RVSN' from dual union all
select 14650,'VMF' from dual union all
select 14651,'VMF' from dual union all
select 14652,'VMF' from dual union all
select 14653,'VMF' from dual union all
select 14654,'VMF' from dual union all
select 14655,'VMF' from dual;
insert into SATELLITE_ORG
select 14656,'VMF' from dual union all
select 14657,'VMF' from dual union all
select 14658,'VMF' from dual union all
select 14659,'NASDA' from dual union all
select 1466,'OKB1' from dual union all
select 14660,'NASDA' from dual union all
select 14661,'MO' from dual union all
select 14662,'MO' from dual union all
select 14663,'MO' from dual union all
select 14664,'MO' from dual union all
select 14665,'NASDA' from dual union all
select 14666,'MO' from dual union all
select 14667,'RVSN' from dual union all
select 14668,'PVO' from dual union all
select 14669,'RVSN' from dual union all
select 1467,'NASA' from dual union all
select 14670,'MAI' from dual union all
select 14671,'MAI' from dual union all
select 14672,'MAI' from dual union all
select 14673,'MO' from dual union all
select 14674,'MO' from dual union all
select 14675,'NRO' from dual union all
select 14676,'USAF' from dual union all
select 14677,'USAF' from dual union all
select 14678,'MAI' from dual union all
select 14679,'VMF' from dual union all
select 1468,'NASA' from dual union all
select 14680,'RVSN' from dual union all
select 14681,'NASA' from dual union all
select 14682,'VMF' from dual union all
select 14683,'VMF' from dual union all
select 14684,'VMF' from dual union all
select 14685,'VMF' from dual union all
select 14686,'VMF' from dual union all
select 14687,'VMF' from dual union all
select 14688,'WUTC' from dual union all
select 14689,'NASA' from dual union all
select 1469,'MO' from dual union all
select 14690,'NRO' from dual union all
select 14690,'USN' from dual union all
select 14691,'USN' from dual union all
select 14692,'PER' from dual union all
select 14693,'PER' from dual union all
select 14694,'WUTC' from dual union all
select 14695,'NASA' from dual union all
select 14696,'NASA' from dual union all
select 14697,'VMF' from dual union all
select 14698,'PER' from dual union all
select 14699,'VS' from dual union all
select 147,'USAF' from dual union all
select 1470,'RVSN' from dual union all
select 14700,'MO' from dual union all
select 14701,'NPOE' from dual union all
select 14702,'RVSN' from dual union all
select 14703,'PVO' from dual union all
select 14704,'PVO' from dual union all
select 14705,'PVO' from dual union all
select 14706,'PVO' from dual union all
select 14707,'PVO' from dual union all
select 14708,'PVO' from dual union all
select 14709,'PVO' from dual union all
select 1471,'NRO' from dual union all
select 14710,'PVO' from dual union all
select 14711,'PVO' from dual union all
select 14712,'MO' from dual union all
select 14713,'MO' from dual union all
select 14714,'MO' from dual union all
select 14715,'USAF' from dual union all
select 14716,'USAF' from dual union all
select 14717,'USAF' from dual union all
select 14718,'PVO' from dual union all
select 14719,'PVO' from dual union all
select 1472,'NRO' from dual union all
select 14720,'PVO' from dual union all
select 14721,'PVO' from dual union all
select 14722,'MONBU' from dual union all
select 14723,'MONBU' from dual union all
select 14724,'MONBU' from dual union all
select 14725,'MO' from dual union all
select 14726,'RVSN' from dual union all
select 14727,'RVSN' from dual union all
select 14728,'NRO' from dual union all
select 14728,'USN' from dual union all
select 14729,'NRO' from dual union all
select 14729,'USN' from dual union all
select 1473,'RVSN' from dual union all
select 14730,'PVO' from dual union all
select 14731,'PVO' from dual union all
select 14732,'PVO' from dual union all
select 14733,'PVO' from dual union all
select 14734,'MO' from dual union all
select 14735,'MO' from dual union all
select 14736,'MO' from dual union all
select 14737,'MVD' from dual union all
select 14738,'RVSN' from dual union all
select 14739,'MO' from dual union all
select 1474,'USAF' from dual union all
select 14740,'MO' from dual union all
select 14741,'MO' from dual union all
select 14742,'RVSN' from dual;
insert into SATELLITE_ORG
select 14743,'RVSN' from dual union all
select 14744,'PVO' from dual union all
select 14745,'PVO' from dual union all
select 14746,'PVO' from dual union all
select 14747,'PVO' from dual union all
select 14748,'PVO' from dual union all
select 14749,'PVO' from dual union all
select 1475,'USAF' from dual union all
select 14750,'PVO' from dual union all
select 14751,'PVO' from dual union all
select 14752,'PVO' from dual union all
select 14753,'PVO' from dual union all
select 14754,'PVO' from dual union all
select 14755,'PVO' from dual union all
select 14756,'USAF' from dual union all
select 14757,'NPOE' from dual union all
select 14758,'RVSN' from dual union all
select 14759,'MO' from dual union all
select 1476,'RVSN' from dual union all
select 14760,'RVSN' from dual union all
select 14761,'USN' from dual union all
select 14762,'MO' from dual union all
select 14763,'MO' from dual union all
select 14764,'RVSN' from dual union all
select 14765,'PER' from dual union all
select 14766,'PER' from dual union all
select 14767,'PER' from dual union all
select 14768,'PER' from dual union all
select 14769,'PER' from dual union all
select 1477,'RVSN' from dual union all
select 14770,'WUTC' from dual union all
select 14771,'WUTC' from dual union all
select 14772,'WUTC' from dual union all
select 14773,'WUTC' from dual union all
select 14774,'MO' from dual union all
select 14775,'MO' from dual union all
select 14776,'MO' from dual union all
select 14777,'MO' from dual union all
select 14778,'MO' from dual union all
select 14779,'MO' from dual union all
select 1478,'RVSN' from dual union all
select 14780,'NOAA' from dual union all
select 14781,'UOS' from dual union all
select 14782,'NASA' from dual union all
select 14783,'MO' from dual union all
select 14784,'RVSN' from dual union all
select 14785,'RVSN' from dual union all
select 14786,'INTEL' from dual union all
select 14787,'AE' from dual union all
select 14788,'RVSN' from dual union all
select 14789,'RVSN' from dual union all
select 1479,'RVSN' from dual union all
select 14790,'PVO' from dual union all
select 14791,'RVSN' from dual union all
select 14792,'RVSN' from dual union all
select 14793,'MO' from dual union all
select 14794,'RVSN' from dual union all
select 14795,'NRO' from dual union all
select 14795,'USN' from dual union all
select 14796,'RVSN' from dual union all
select 14797,'TSSKB' from dual union all
select 14798,'RVSN' from dual union all
select 14799,'PVO' from dual union all
select 148,'USAF' from dual union all
select 1480,'RVSN' from dual union all
select 14800,'PVO' from dual union all
select 14801,'MO' from dual union all
select 14802,'RVSN' from dual union all
select 14803,'RVSN' from dual union all
select 14804,'RVSN' from dual union all
select 14805,'RVSN' from dual union all
select 14806,'RVSN' from dual union all
select 14807,'RVSN' from dual union all
select 14808,'RVSN' from dual union all
select 14809,'RVSN' from dual union all
select 1481,'RVSN' from dual union all
select 14810,'MO' from dual union all
select 14811,'MO' from dual union all
select 14812,'MO' from dual union all
select 14813,'MO' from dual union all
select 14814,'VMF' from dual union all
select 14815,'MO' from dual union all
select 14816,'MO' from dual union all
select 14817,'MO' from dual union all
select 14818,'RVSN' from dual union all
select 14819,'VS' from dual union all
select 1482,'RVSN' from dual union all
select 14820,'MO' from dual union all
select 14821,'MSVYAZ' from dual union all
select 14822,'RVSN' from dual union all
select 14823,'RVSN' from dual union all
select 14824,'PER' from dual union all
select 14825,'MO' from dual union all
select 14826,'RVSN' from dual union all
select 14827,'RVSN' from dual union all
select 14828,'RVSN' from dual union all
select 14829,'RVSN' from dual union all
select 1483,'RVSN' from dual union all
select 14830,'RVSN' from dual union all
select 14831,'VMF' from dual;
insert into SATELLITE_ORG
select 14832,'VMF' from dual union all
select 14833,'VMF' from dual union all
select 14834,'PVO' from dual union all
select 14835,'PVO' from dual union all
select 14836,'PVO' from dual union all
select 14837,'VMF' from dual union all
select 14838,'PVO' from dual union all
select 14839,'NASA' from dual union all
select 1484,'RVSN' from dual union all
select 14840,'PVO' from dual union all
select 14841,'NASA' from dual union all
select 14842,'USAF' from dual union all
select 14843,'USAF' from dual union all
select 14844,'CNES' from dual union all
select 14845,'CNES' from dual union all
select 14846,'NASA' from dual union all
select 14847,'MO' from dual union all
select 14848,'MO' from dual union all
select 14849,'MO' from dual union all
select 1485,'RVSN' from dual union all
select 14850,'RVSN' from dual union all
select 14851,'MO' from dual union all
select 14852,'MO' from dual union all
select 14853,'PVO' from dual union all
select 14854,'PVO' from dual union all
select 14855,'MO' from dual union all
select 14856,'MO' from dual union all
select 14857,'NPOE' from dual union all
select 14858,'NPOE' from dual union all
select 14859,'NPOE' from dual union all
select 1486,'RVSN' from dual union all
select 14860,'NPOE' from dual union all
select 14861,'NPOE' from dual union all
select 14862,'NPOE' from dual union all
select 14863,'NPOE' from dual union all
select 14864,'NPOE' from dual union all
select 14865,'MO' from dual union all
select 14866,'RVSN' from dual union all
select 14867,'PVO' from dual union all
select 14868,'RVSN' from dual union all
select 14869,'RVSN' from dual union all
select 1487,'RVSN' from dual union all
select 14870,'NPOE' from dual union all
select 14871,'NPOE' from dual union all
select 14872,'NPOE' from dual union all
select 14873,'RVSN' from dual union all
select 14874,'NASA' from dual union all
select 14875,'NASA' from dual union all
select 14876,'NASA' from dual union all
select 14877,'NASA' from dual union all
select 14878,'NASA' from dual union all
select 14879,'NASA' from dual union all
select 1488,'RVSN' from dual union all
select 14880,'NASA' from dual union all
select 14881,'NASA' from dual union all
select 14882,'NASA' from dual union all
select 14883,'NASA' from dual union all
select 14884,'PVO' from dual union all
select 14885,'RVSN' from dual union all
select 14886,'RVSN' from dual union all
select 14887,'RVSN' from dual union all
select 14888,'RVSN' from dual union all
select 14889,'MO' from dual union all
select 1489,'RVSN' from dual union all
select 14890,'MO' from dual union all
select 14891,'MO' from dual union all
select 14892,'MO' from dual union all
select 14893,'MO' from dual union all
select 14894,'RVSN' from dual union all
select 14895,'MO' from dual union all
select 14896,'MO' from dual union all
select 14897,'NASA' from dual union all
select 14898,'NASA' from dual union all
select 14899,'MAI' from dual union all
select 149,'USAF' from dual union all
select 1490,'RVSN' from dual union all
select 14900,'MAI' from dual union all
select 14901,'MO' from dual union all
select 14902,'MO' from dual union all
select 14903,'RVSN' from dual union all
select 14904,'NASA' from dual union all
select 14905,'NASA' from dual union all
select 14906,'PVO' from dual union all
select 14907,'PVO' from dual union all
select 14908,'NASA' from dual union all
select 14909,'NASA' from dual union all
select 1491,'RVSN' from dual union all
select 14910,'NASA' from dual union all
select 14911,'NASA' from dual union all
select 14912,'NASA' from dual union all
select 14913,'NASA' from dual union all
select 14914,'NASA' from dual union all
select 14915,'NASA' from dual union all
select 14916,'NPOE' from dual union all
select 14917,'NPOE' from dual union all
select 14918,'NPOE' from dual union all
select 14919,'PVO' from dual union all
select 1492,'RVSN' from dual union all
select 14920,'PVO' from dual union all
select 14921,'PVO' from dual;
insert into SATELLITE_ORG
select 14922,'PVO' from dual union all
select 14923,'PVO' from dual union all
select 14924,'VMF' from dual union all
select 14925,'MO' from dual union all
select 14926,'MO' from dual union all
select 14927,'NASA' from dual union all
select 14928,'MO' from dual union all
select 14929,'MO' from dual union all
select 1493,'RVSN' from dual union all
select 14930,'USAF' from dual union all
select 14931,'USAF' from dual union all
select 14932,'NPOE' from dual union all
select 14933,'RVSN' from dual union all
select 14934,'USN' from dual union all
select 14935,'NRO' from dual union all
select 14936,'PVO' from dual union all
select 14937,'PVO' from dual union all
select 14938,'MO' from dual union all
select 14939,'RVSN' from dual union all
select 1494,'RVSN' from dual union all
select 14940,'MSVYAZ' from dual union all
select 14941,'RVSN' from dual union all
select 14942,'RVSN' from dual union all
select 14943,'RVSN' from dual union all
select 14944,'RVSN' from dual union all
select 14945,'RVSN' from dual union all
select 14946,'NPOE' from dual union all
select 14947,'PVO' from dual union all
select 14948,'RVSN' from dual union all
select 14949,'NPOE' from dual union all
select 1495,'RVSN' from dual union all
select 14950,'RVSN' from dual union all
select 14951,'RVSN' from dual union all
select 14952,'MO' from dual union all
select 14953,'MO' from dual union all
select 14954,'MO' from dual union all
select 14955,'MO' from dual union all
select 14956,'MO' from dual union all
select 14957,'NPOE' from dual union all
select 14958,'NPOE' from dual union all
select 14959,'USAF' from dual union all
select 1496,'RVSN' from dual union all
select 14960,'NPOE' from dual union all
select 14961,'NPOE' from dual union all
select 14962,'RVSN' from dual union all
select 14963,'PVO' from dual union all
select 14964,'USN' from dual union all
select 14965,'VMF' from dual union all
select 14966,'RVSN' from dual union all
select 14967,'MO' from dual union all
select 14968,'RVSN' from dual union all
select 14969,'USN' from dual union all
select 1497,'RVSN' from dual union all
select 14970,'WUTC' from dual union all
select 14971,'TSSKB' from dual union all
select 14972,'RVSN' from dual union all
select 14973,'VMF' from dual union all
select 14974,'RVSN' from dual union all
select 14975,'NPOE' from dual union all
select 14976,'NPOE' from dual union all
select 14977,'MO' from dual union all
select 14978,'MO' from dual union all
select 14979,'MO' from dual union all
select 1498,'RVSN' from dual union all
select 14980,'RVSN' from dual union all
select 14981,'RVSN' from dual union all
select 14982,'MVD' from dual union all
select 14983,'RVSN' from dual union all
select 14984,'RVSN' from dual union all
select 14985,'GTE' from dual union all
select 14986,'MO' from dual union all
select 14987,'MO' from dual union all
select 14988,'MO' from dual union all
select 14989,'AE' from dual union all
select 1499,'RVSN' from dual union all
select 14990,'MO' from dual union all
select 14991,'MO' from dual union all
select 14992,'MO' from dual union all
select 14993,'MO' from dual union all
select 14994,'RVSN' from dual union all
select 14995,'NPOE' from dual union all
select 14996,'NPOE' from dual union all
select 14997,'RVSN' from dual union all
select 14998,'MO' from dual union all
select 14999,'MO' from dual union all
select 15,'NASA' from dual union all
select 150,'USAF' from dual union all
select 1500,'USAF' from dual union all
select 15000,'MO' from dual union all
select 15001,'MO' from dual union all
select 15002,'MO' from dual union all
select 15003,'MO' from dual union all
select 15004,'MO' from dual union all
select 15005,'MO' from dual union all
select 15006,'RVSN' from dual union all
select 15007,'PVO' from dual union all
select 15008,'PVO' from dual union all
select 15009,'VMF' from dual union all
select 1501,'USAF' from dual union all
select 15010,'RVSN' from dual;
insert into SATELLITE_ORG
select 15011,'MO' from dual union all
select 15012,'RVSN' from dual union all
select 15013,'PVO' from dual union all
select 15014,'PVO' from dual union all
select 15015,'PVO' from dual union all
select 15016,'PVO' from dual union all
select 15017,'PVO' from dual union all
select 15018,'PVO' from dual union all
select 15019,'PVO' from dual union all
select 1502,'NASA' from dual union all
select 15020,'PVO' from dual union all
select 15021,'MO' from dual union all
select 15022,'MO' from dual union all
select 15023,'MO' from dual union all
select 15024,'MO' from dual union all
select 15025,'MO' from dual union all
select 15026,'MO' from dual union all
select 15027,'PVO' from dual union all
select 15028,'RVSN' from dual union all
select 15029,'RVSN' from dual union all
select 1503,'JPL' from dual union all
select 1503,'NASA' from dual union all
select 15030,'RVSN' from dual union all
select 15031,'MO' from dual union all
select 15032,'RVSN' from dual union all
select 15033,'MO' from dual union all
select 15034,'INTEL' from dual union all
select 15035,'NASA' from dual union all
select 15036,'MO' from dual union all
select 15037,'RVSN' from dual union all
select 15038,'MO' from dual union all
select 15039,'USAF' from dual union all
select 1504,'USN' from dual union all
select 15040,'USAF' from dual union all
select 15041,'MO' from dual union all
select 15042,'MO' from dual union all
select 15043,'MO' from dual union all
select 15044,'MO' from dual union all
select 15045,'MO' from dual union all
select 15046,'MVD' from dual union all
select 15047,'RVSN' from dual union all
select 15048,'MO' from dual union all
select 15049,'MO' from dual union all
select 1505,'MO' from dual union all
select 15050,'MO' from dual union all
select 15051,'MO' from dual union all
select 15052,'RVSN' from dual union all
select 15053,'RVSN' from dual union all
select 15054,'RVSN' from dual union all
select 15055,'MOM' from dual union all
select 15056,'RVSN' from dual union all
select 15057,'MO' from dual union all
select 15058,'RVSN' from dual union all
select 15059,'RVSN' from dual union all
select 1506,'USA' from dual union all
select 15060,'MVD' from dual union all
select 15061,'RVSN' from dual union all
select 15062,'MO' from dual union all
select 15063,'NRO' from dual union all
select 15064,'USAF' from dual union all
select 15065,'MO' from dual union all
select 15066,'MO' from dual union all
select 15067,'MO' from dual union all
select 15068,'MO' from dual union all
select 15069,'MO' from dual union all
select 1507,'RVSN' from dual union all
select 15070,'MO' from dual union all
select 15071,'NRO' from dual union all
select 15072,'MO' from dual union all
select 15073,'RVSN' from dual union all
select 15074,'MO' from dual union all
select 15075,'RVSN' from dual union all
select 15076,'RVSN' from dual union all
select 15077,'VMF' from dual union all
select 15078,'RVSN' from dual union all
select 15079,'MO' from dual union all
select 1508,'USAF' from dual union all
select 15080,'PVO' from dual union all
select 15081,'RVSN' from dual union all
select 15082,'MO' from dual union all
select 15083,'MO' from dual union all
select 15084,'MO' from dual union all
select 15085,'VMF' from dual union all
select 15086,'MO' from dual union all
select 15087,'MO' from dual union all
select 15088,'MO' from dual union all
select 15089,'MO' from dual union all
select 1509,'RVSN' from dual union all
select 15090,'MO' from dual union all
select 15091,'RVSN' from dual union all
select 15092,'MO' from dual union all
select 15093,'MO' from dual union all
select 15094,'MO' from dual union all
select 15095,'PVO' from dual union all
select 15096,'RVSN' from dual union all
select 15097,'RVSN' from dual union all
select 15098,'RVSN' from dual union all
select 15099,'GMS' from dual union all
select 15099,'MO' from dual union all
select 151,'USAF' from dual;
insert into SATELLITE_ORG
select 1510,'USN' from dual union all
select 15100,'MO' from dual union all
select 15101,'MO' from dual union all
select 15102,'MO' from dual union all
select 15103,'MO' from dual union all
select 15104,'MO' from dual union all
select 15105,'NPOE' from dual union all
select 15106,'NPOE' from dual union all
select 15107,'NPOE' from dual union all
select 15108,'PVO' from dual union all
select 15109,'NPOE' from dual union all
select 1511,'USN' from dual union all
select 15110,'PVO' from dual union all
select 15111,'NPOE' from dual union all
select 15112,'NPOE' from dual union all
select 15113,'NPOE' from dual union all
select 15114,'NPOE' from dual union all
select 15115,'MO' from dual union all
select 15116,'MO' from dual union all
select 15117,'MO' from dual union all
select 15118,'MO' from dual union all
select 15119,'NPOE' from dual union all
select 1512,'USN' from dual union all
select 15120,'RVSN' from dual union all
select 15121,'MVD' from dual union all
select 15122,'RVSN' from dual union all
select 15123,'MO' from dual union all
select 15124,'RVSN' from dual union all
select 15125,'MO' from dual union all
select 15126,'MO' from dual union all
select 15127,'NPOE' from dual union all
select 15128,'NPOE' from dual union all
select 15129,'NPOE' from dual union all
select 1513,'CIA' from dual union all
select 1513,'NRO' from dual union all
select 15130,'NPOE' from dual union all
select 15131,'MO' from dual union all
select 15132,'RVSN' from dual union all
select 15133,'NPOE' from dual union all
select 15134,'NPOE' from dual union all
select 15135,'NPOE' from dual union all
select 15136,'NPOE' from dual union all
select 15137,'NPOE' from dual union all
select 15138,'NPOE' from dual union all
select 15139,'RVSN' from dual union all
select 1514,'USN' from dual union all
select 15140,'NPOE' from dual union all
select 15141,'RVSN' from dual union all
select 15142,'MO' from dual union all
select 15143,'RVSN' from dual union all
select 15144,'MSVYAZ' from dual union all
select 15145,'RVSN' from dual union all
select 15146,'RVSN' from dual union all
select 15147,'PVO' from dual union all
select 15148,'RVSN' from dual union all
select 15149,'RVSN' from dual union all
select 1515,'USN' from dual union all
select 15150,'MO' from dual union all
select 15151,'MO' from dual union all
select 15152,'NASDA' from dual union all
select 15153,'NASDA' from dual union all
select 15154,'MO' from dual union all
select 15155,'MO' from dual union all
select 15156,'RVSN' from dual union all
select 15157,'NASDA' from dual union all
select 15158,'EUTEL' from dual union all
select 15159,'FTEL' from dual union all
select 1516,'NASA' from dual union all
select 15160,'RVSN' from dual union all
select 15161,'RVSN' from dual union all
select 15162,'NASDA' from dual union all
select 15163,'MO' from dual union all
select 15164,'RVSN' from dual union all
select 15165,'AE' from dual union all
select 15166,'AE' from dual union all
select 15167,'VMF' from dual union all
select 15168,'RVSN' from dual union all
select 15169,'MO' from dual union all
select 1517,'USAF' from dual union all
select 15170,'MO' from dual union all
select 15171,'MOM' from dual union all
select 15172,'MO' from dual union all
select 15173,'MO' from dual union all
select 15174,'NPOE' from dual union all
select 15175,'NPOE' from dual union all
select 15176,'NPOE' from dual union all
select 15177,'NPOE' from dual union all
select 15178,'MO' from dual union all
select 15179,'MO' from dual union all
select 1518,'NASA' from dual union all
select 15180,'MO' from dual union all
select 15181,'RVSN' from dual union all
select 15182,'MO' from dual union all
select 15183,'RVSN' from dual union all
select 15184,'RVSN' from dual union all
select 15185,'MO' from dual union all
select 15186,'MO' from dual union all
select 15187,'MO' from dual union all
select 15188,'RVSN' from dual union all
select 15189,'NPOE' from dual;
insert into SATELLITE_ORG
select 1519,'NASA' from dual union all
select 15190,'NPOE' from dual union all
select 15191,'MO' from dual union all
select 15192,'NPOE' from dual union all
select 15193,'NPOE' from dual union all
select 15194,'RVSN' from dual union all
select 15195,'NPOE' from dual union all
select 15196,'NPOE' from dual union all
select 15197,'MVD' from dual union all
select 15198,'RVSN' from dual union all
select 15199,'NASA' from dual union all
select 152,'USAF' from dual union all
select 1520,'USN' from dual union all
select 15200,'DFVLR' from dual union all
select 15201,'SERC' from dual union all
select 15202,'NASA' from dual union all
select 15203,'PVO' from dual union all
select 15204,'PVO' from dual union all
select 15205,'NASA' from dual union all
select 15206,'NASA' from dual union all
select 15207,'NPOE' from dual union all
select 15208,'NPOE' from dual union all
select 15209,'NPOE' from dual union all
select 1521,'USN' from dual union all
select 15210,'NPOE' from dual union all
select 15211,'NPOE' from dual union all
select 15212,'NPOE' from dual union all
select 15213,'NPOE' from dual union all
select 15214,'NPOE' from dual union all
select 15215,'RVSN' from dual union all
select 15216,'RVSN' from dual union all
select 15217,'NPOE' from dual union all
select 15218,'NPOE' from dual union all
select 15219,'MSVYAZ' from dual union all
select 1522,'USN' from dual union all
select 15220,'RVSN' from dual union all
select 15221,'RVSN' from dual union all
select 15222,'MO' from dual union all
select 15223,'RVSN' from dual union all
select 15224,'RVSN' from dual union all
select 15225,'RVSN' from dual union all
select 15226,'NRO' from dual union all
select 15227,'USAF' from dual union all
select 15228,'MO' from dual union all
select 15229,'MO' from dual union all
select 1523,'MO' from dual union all
select 15230,'MO' from dual union all
select 15231,'MO' from dual union all
select 15232,'MVD' from dual union all
select 15233,'RVSN' from dual union all
select 15234,'NASA' from dual union all
select 15235,'SBS' from dual union all
select 15236,'HEC' from dual union all
select 15237,'ATT' from dual union all
select 15238,'MO' from dual union all
select 15239,'MO' from dual union all
select 1524,'RVSN' from dual union all
select 15240,'MO' from dual union all
select 15241,'MO' from dual union all
select 15242,'MO' from dual union all
select 15243,'MO' from dual union all
select 15244,'HEC' from dual union all
select 15245,'SBS' from dual union all
select 15246,'ATT' from dual union all
select 15247,'NPOE' from dual union all
select 15248,'NPOE' from dual union all
select 15249,'NPOE' from dual union all
select 1525,'MO' from dual union all
select 15250,'NPOE' from dual union all
select 15251,'NPOE' from dual union all
select 15252,'NPOE' from dual union all
select 15253,'NPOE' from dual union all
select 15254,'NPOE' from dual union all
select 15255,'NPOE' from dual union all
select 15256,'NPOE' from dual union all
select 15257,'MO' from dual union all
select 15258,'RVSN' from dual union all
select 15259,'MO' from dual union all
select 1526,'MO' from dual union all
select 15260,'MO' from dual union all
select 15261,'MO' from dual union all
select 15262,'RVSN' from dual union all
select 15263,'RVSN' from dual union all
select 15264,'RVSN' from dual union all
select 15265,'RVSN' from dual union all
select 15266,'RVSN' from dual union all
select 15267,'PVO' from dual union all
select 15268,'RVSN' from dual union all
select 15269,'RVSN' from dual union all
select 1527,'MO' from dual union all
select 15270,'RVSN' from dual union all
select 15271,'USAF' from dual union all
select 15272,'USAF' from dual union all
select 15273,'NPOE' from dual union all
select 15274,'NPOE' from dual union all
select 15275,'NPOE' from dual union all
select 15276,'NPOE' from dual union all
select 15277,'PVO' from dual union all
select 15278,'PVO' from dual union all
select 15279,'MAI' from dual;
insert into SATELLITE_ORG
select 1528,'USN' from dual union all
select 15280,'MAI' from dual union all
select 15281,'NPOE' from dual union all
select 15282,'NPOE' from dual union all
select 15283,'NPOE' from dual union all
select 15284,'NPOE' from dual union all
select 15285,'MAI' from dual union all
select 15286,'MAI' from dual union all
select 15287,'MVD' from dual union all
select 15288,'RVSN' from dual union all
select 15289,'MO' from dual union all
select 1529,'ESSA' from dual union all
select 15290,'MO' from dual union all
select 15291,'MO' from dual union all
select 15292,'VMF' from dual union all
select 15293,'RVSN' from dual union all
select 15294,'MO' from dual union all
select 15295,'MO' from dual union all
select 15296,'MO' from dual union all
select 15297,'NPOE' from dual union all
select 15298,'MO' from dual union all
select 15299,'MO' from dual union all
select 153,'USAF' from dual union all
select 1530,'RVSN' from dual union all
select 15300,'MO' from dual union all
select 15301,'MO' from dual union all
select 15302,'MO' from dual union all
select 15303,'NPOE' from dual union all
select 15304,'NPOE' from dual union all
select 15305,'NPOE' from dual union all
select 15306,'NPOE' from dual union all
select 15307,'NPOE' from dual union all
select 15308,'HEC' from dual union all
select 15309,'NASA' from dual union all
select 1531,'RVSN' from dual union all
select 15310,'NASA' from dual union all
select 15311,'NPOE' from dual union all
select 15312,'NPOE' from dual union all
select 15313,'NPOE' from dual union all
select 15314,'NPOE' from dual union all
select 15315,'NPOE' from dual union all
select 15316,'NPOE' from dual union all
select 15317,'NPOE' from dual union all
select 15318,'MO' from dual union all
select 15319,'RVSN' from dual union all
select 1532,'RVSN' from dual union all
select 15320,'MO' from dual union all
select 15321,'MO' from dual union all
select 15322,'MO' from dual union all
select 15323,'MO' from dual union all
select 15324,'MO' from dual union all
select 15325,'RVSN' from dual union all
select 15326,'PVO' from dual union all
select 15327,'VMF' from dual union all
select 15328,'VMF' from dual union all
select 15329,'RVSN' from dual union all
select 1533,'RVSN' from dual union all
select 15330,'VMF' from dual union all
select 15331,'GMS' from dual union all
select 15331,'YUZH' from dual union all
select 15332,'MO' from dual union all
select 15333,'TSN108' from dual union all
select 15334,'RVSN' from dual union all
select 15335,'RVSN' from dual union all
select 15336,'RVSN' from dual union all
select 15337,'RVSN' from dual union all
select 15338,'RVSN' from dual union all
select 15339,'USAF' from dual union all
select 1534,'RVSN' from dual union all
select 15340,'USAF' from dual union all
select 15341,'USN' from dual union all
select 15342,'PVO' from dual union all
select 15343,'PVO' from dual union all
select 15344,'MO' from dual union all
select 15345,'NPOE' from dual union all
select 15346,'NPOE' from dual union all
select 15347,'USN' from dual union all
select 15348,'USN' from dual union all
select 15349,'USN' from dual union all
select 1535,'RVSN' from dual union all
select 15350,'PVO' from dual union all
select 15351,'RVSN' from dual union all
select 15352,'RVSN' from dual union all
select 15353,'NASA' from dual union all
select 15354,'NASA' from dual union all
select 15355,'RVSN' from dual union all
select 15356,'MO' from dual union all
select 15357,'MO' from dual union all
select 15358,'MO' from dual union all
select 15359,'VMF' from dual union all
select 1536,'RVSN' from dual union all
select 15360,'RVSN' from dual union all
select 15361,'MO' from dual union all
select 15362,'USN' from dual union all
select 15363,'NASA' from dual union all
select 15364,'MO' from dual union all
select 15365,'NPOE' from dual union all
select 15366,'NPOE' from dual union all
select 15367,'NPOE' from dual union all
select 15368,'NPOE' from dual;
insert into SATELLITE_ORG
select 15369,'VS' from dual union all
select 1537,'RVSN' from dual union all
select 15370,'MO' from dual union all
select 15371,'USN' from dual union all
select 15372,'USN' from dual union all
select 15373,'MO' from dual union all
select 15374,'PVO' from dual union all
select 15375,'PVO' from dual union all
select 15376,'PVO' from dual union all
select 15377,'PVO' from dual union all
select 15378,'VMF' from dual union all
select 15379,'MO' from dual union all
select 1538,'RVSN' from dual union all
select 15380,'MO' from dual union all
select 15381,'MO' from dual union all
select 15382,'NASA' from dual union all
select 15383,'TCAN' from dual union all
select 15384,'HEC' from dual union all
select 15385,'GTE' from dual union all
select 15386,'ESA' from dual union all
select 15387,'TCAN' from dual union all
select 15388,'AE' from dual union all
select 15389,'AE' from dual union all
select 1539,'RVSN' from dual union all
select 15390,'HEC' from dual union all
select 15391,'NATO' from dual union all
select 15392,'NASA' from dual union all
select 15393,'TSSKB' from dual union all
select 15394,'RVSN' from dual union all
select 15395,'MO' from dual union all
select 15396,'RVSN' from dual union all
select 15397,'TSSKB' from dual union all
select 15398,'VMF' from dual union all
select 15399,'RVSN' from dual union all
select 154,'USAF' from dual union all
select 1540,'RVSN' from dual union all
select 15400,'MO' from dual union all
select 15401,'NPOE' from dual union all
select 15402,'NASA' from dual union all
select 15403,'MO' from dual union all
select 15404,'RVSN' from dual union all
select 15405,'MO' from dual union all
select 15406,'GMS' from dual union all
select 15407,'MO' from dual union all
select 15408,'MO' from dual union all
select 15409,'MO' from dual union all
select 1541,'RVSN' from dual union all
select 15410,'MO' from dual union all
select 15411,'MO' from dual union all
select 15412,'MO' from dual union all
select 15413,'MO' from dual union all
select 15414,'MO' from dual union all
select 15415,'RVSN' from dual union all
select 15416,'MO' from dual union all
select 15417,'MO' from dual union all
select 15418,'MO' from dual union all
select 15419,'MO' from dual union all
select 1542,'RVSN' from dual union all
select 15420,'NPOE' from dual union all
select 15421,'MO' from dual union all
select 15422,'NRO' from dual union all
select 15423,'CIA' from dual union all
select 15423,'NRO' from dual union all
select 15424,'USAF' from dual union all
select 15425,'PVO' from dual union all
select 15426,'PVO' from dual union all
select 15427,'NOAA' from dual union all
select 15428,'MO' from dual union all
select 15429,'MO' from dual union all
select 1543,'RVSN' from dual union all
select 15430,'RVSN' from dual union all
select 15431,'RVSN' from dual union all
select 15432,'IKI' from dual union all
select 15432,'NPOL' from dual union all
select 15433,'RVSN' from dual union all
select 15434,'RVSN' from dual union all
select 15435,'RVSN' from dual union all
select 15436,'RVSN' from dual union all
select 15437,'RVSN' from dual union all
select 15438,'RVSN' from dual union all
select 15439,'RVSN' from dual union all
select 1544,'RVSN' from dual union all
select 15440,'NASA' from dual union all
select 15441,'NASA' from dual union all
select 15442,'MO' from dual union all
select 15442,'MOLN' from dual union all
select 15443,'RVSN' from dual union all
select 15444,'MO' from dual union all
select 15445,'MO' from dual union all
select 15446,'PVO' from dual union all
select 15447,'RVSN' from dual union all
select 15448,'RVSN' from dual union all
select 15449,'IKI' from dual union all
select 15449,'NPOL' from dual union all
select 1545,'RVSN' from dual union all
select 15450,'RVSN' from dual union all
select 15451,'RVSN' from dual union all
select 15452,'RVSN' from dual union all
select 15453,'USAF' from dual union all
select 15454,'USAF' from dual;
insert into SATELLITE_ORG
select 15455,'USAF' from dual union all
select 15456,'MO' from dual union all
select 15457,'MO' from dual union all
select 15458,'MO' from dual union all
select 15459,'MO' from dual union all
select 1546,'RVSN' from dual union all
select 15460,'MO' from dual union all
select 15461,'MO' from dual union all
select 15462,'MO' from dual union all
select 15463,'MO' from dual union all
select 15464,'MONBU' from dual union all
select 15465,'MONBU' from dual union all
select 15466,'MONBU' from dual union all
select 15467,'MO' from dual union all
select 15468,'RVSN' from dual union all
select 15469,'MO' from dual union all
select 1547,'RVSN' from dual union all
select 15470,'MO' from dual union all
select 15471,'MO' from dual union all
select 15472,'MO' from dual union all
select 15473,'MO' from dual union all
select 15474,'MO' from dual union all
select 15475,'MO' from dual union all
select 15476,'MOM' from dual union all
select 15476,'MSVYAZ' from dual union all
select 15477,'RVSN' from dual union all
select 15478,'RVSN' from dual union all
select 15479,'MO' from dual union all
select 1548,'RVSN' from dual union all
select 15480,'RVSN' from dual union all
select 15481,'RVSN' from dual union all
select 15482,'MO' from dual union all
select 15483,'RVSN' from dual union all
select 15484,'MSVYAZ' from dual union all
select 15485,'RVSN' from dual union all
select 15486,'RVSN' from dual union all
select 15487,'RVSN' from dual union all
select 15488,'RVSN' from dual union all
select 15489,'RVSN' from dual union all
select 1549,'RVSN' from dual union all
select 15490,'MO' from dual union all
select 15491,'MO' from dual union all
select 15492,'VMF' from dual union all
select 15493,'MO' from dual union all
select 15494,'VS' from dual union all
select 15495,'MO' from dual union all
select 15496,'NASA' from dual union all
select 15497,'MO' from dual union all
select 15498,'MO' from dual union all
select 15499,'MO' from dual union all
select 155,'USAF' from dual union all
select 1550,'RVSN' from dual union all
select 15500,'MO' from dual union all
select 15501,'MO' from dual union all
select 15502,'VMF' from dual union all
select 15503,'VMF' from dual union all
select 15504,'VMF' from dual union all
select 15505,'VMF' from dual union all
select 15506,'RVSN' from dual union all
select 15507,'VMF' from dual union all
select 15508,'VMF' from dual union all
select 15509,'MO' from dual union all
select 1551,'RVSN' from dual union all
select 15510,'MO' from dual union all
select 15511,'VMF' from dual union all
select 15512,'PVO' from dual union all
select 15513,'PVO' from dual union all
select 15514,'MO' from dual union all
select 15515,'RVSN' from dual union all
select 15516,'GMS' from dual union all
select 15516,'MO' from dual union all
select 15517,'MO' from dual union all
select 15518,'PVO' from dual union all
select 15519,'PVO' from dual union all
select 1552,'RVSN' from dual union all
select 15520,'NASA' from dual union all
select 15521,'RVSN' from dual union all
select 15522,'NASA' from dual union all
select 15523,'PVO' from dual union all
select 15524,'NPOE' from dual union all
select 15525,'NPOE' from dual union all
select 15526,'NPOE' from dual union all
select 15527,'NPOE' from dual union all
select 15528,'MO' from dual union all
select 15529,'USAF' from dual union all
select 1553,'RVSN' from dual union all
select 15530,'USAF' from dual union all
select 15531,'CNES' from dual union all
select 15532,'NPOE' from dual union all
select 15533,'NPOE' from dual union all
select 15534,'PVO' from dual union all
select 15535,'PVO' from dual union all
select 15536,'PVO' from dual union all
select 15537,'NASA' from dual union all
select 15538,'PVO' from dual union all
select 15539,'NASA' from dual union all
select 1554,'RVSN' from dual union all
select 15540,'NASA' from dual union all
select 15541,'MO' from dual union all
select 15542,'MO' from dual;
insert into SATELLITE_ORG
select 15543,'NRO' from dual union all
select 15544,'USAF' from dual union all
select 15545,'USAF' from dual union all
select 15546,'NRO' from dual union all
select 15547,'USAF' from dual union all
select 15548,'USAF' from dual union all
select 15549,'USAF' from dual union all
select 1555,'RVSN' from dual union all
select 15550,'USAF' from dual union all
select 15551,'USAF' from dual union all
select 15552,'USAF' from dual union all
select 15553,'USAF' from dual union all
select 15554,'USAF' from dual union all
select 15555,'PVO' from dual union all
select 15556,'WUTC' from dual union all
select 15557,'WUTC' from dual union all
select 15558,'USAF' from dual union all
select 15559,'PVO' from dual union all
select 1556,'RVSN' from dual union all
select 15560,'ARAB' from dual union all
select 15561,'TELB' from dual union all
select 15562,'AE' from dual union all
select 15563,'AE' from dual union all
select 15564,'PVO' from dual union all
select 15565,'PVO' from dual union all
select 15566,'PVO' from dual union all
select 15567,'PVO' from dual union all
select 15568,'MO' from dual union all
select 15569,'MO' from dual union all
select 1557,'RVSN' from dual union all
select 15570,'MO' from dual union all
select 15571,'MO' from dual union all
select 15572,'MO' from dual union all
select 15573,'MO' from dual union all
select 15574,'PVO' from dual union all
select 15575,'RVSN' from dual union all
select 15576,'RVSN' from dual union all
select 15577,'RVSN' from dual union all
select 15578,'RVSN' from dual union all
select 15579,'MO' from dual union all
select 1558,'RVSN' from dual union all
select 15580,'MO' from dual union all
select 15581,'RVSN' from dual union all
select 15582,'MO' from dual union all
select 15583,'RVSN' from dual union all
select 15584,'PVO' from dual union all
select 15585,'RVSN' from dual union all
select 15586,'PVO' from dual union all
select 15587,'PVO' from dual union all
select 15588,'PVO' from dual union all
select 15589,'MO' from dual union all
select 1559,'RVSN' from dual union all
select 15590,'RVSN' from dual union all
select 15591,'MO' from dual union all
select 15592,'VS' from dual union all
select 15593,'MO' from dual union all
select 15594,'PVO' from dual union all
select 15595,'USN' from dual union all
select 15596,'USAF' from dual union all
select 15597,'VMF' from dual union all
select 15598,'RVSN' from dual union all
select 15599,'IKI' from dual union all
select 15599,'NPOL' from dual union all
select 156,'USAF' from dual union all
select 1560,'RVSN' from dual union all
select 15600,'IKI' from dual union all
select 15600,'NPOL' from dual union all
select 15601,'VMF' from dual union all
select 15602,'VMF' from dual union all
select 15603,'VMF' from dual union all
select 15604,'VMF' from dual union all
select 15605,'VMF' from dual union all
select 15606,'VMF' from dual union all
select 15607,'PVO' from dual union all
select 15608,'PVO' from dual union all
select 15609,'MO' from dual union all
select 1561,'RVSN' from dual union all
select 15610,'MO' from dual union all
select 15611,'MO' from dual union all
select 15612,'MO' from dual union all
select 15613,'USN' from dual union all
select 15614,'USN' from dual union all
select 15615,'USN' from dual union all
select 15616,'USN' from dual union all
select 15617,'MO' from dual union all
select 15618,'MO' from dual union all
select 15619,'MO' from dual union all
select 1562,'RVSN' from dual union all
select 15620,'MO' from dual union all
select 15621,'MO' from dual union all
select 15622,'MO' from dual union all
select 15623,'MO' from dual union all
select 15624,'MO' from dual union all
select 15625,'RVSN' from dual union all
select 15626,'MSVYAZ' from dual union all
select 15627,'RVSN' from dual union all
select 15628,'RVSN' from dual union all
select 15629,'INTEL' from dual union all
select 1563,'RVSN' from dual union all
select 15630,'RVSN' from dual;
insert into SATELLITE_ORG
select 15631,'NASA' from dual union all
select 15632,'RVSN' from dual union all
select 15633,'RVSN' from dual union all
select 15634,'MO' from dual union all
select 15635,'RVSN' from dual union all
select 15636,'MO' from dual union all
select 15637,'RVSN' from dual union all
select 15638,'MO' from dual union all
select 15639,'MO' from dual union all
select 1564,'RVSN' from dual union all
select 15640,'MO' from dual union all
select 15641,'NASA' from dual union all
select 15642,'TCAN' from dual union all
select 15643,'HEC' from dual union all
select 15644,'TCAN' from dual union all
select 15645,'TSSKB' from dual union all
select 15646,'RVSN' from dual union all
select 15647,'MO' from dual union all
select 15648,'MO' from dual union all
select 15649,'MO' from dual union all
select 1565,'RVSN' from dual union all
select 15650,'MO' from dual union all
select 15651,'MO' from dual union all
select 15652,'MO' from dual union all
select 15653,'VMF' from dual union all
select 15654,'MO' from dual union all
select 15655,'MO' from dual union all
select 15656,'RVSN' from dual union all
select 15657,'MO' from dual union all
select 15658,'MO' from dual union all
select 15659,'MO' from dual union all
select 1566,'RVSN' from dual union all
select 15660,'RVSN' from dual union all
select 15661,'IKI' from dual union all
select 15661,'NPOL' from dual union all
select 15662,'RVSN' from dual union all
select 15663,'RVSN' from dual union all
select 15664,'RVSN' from dual union all
select 15665,'NASA' from dual union all
select 15666,'WEB' from dual union all
select 15667,'MO' from dual union all
select 15668,'MO' from dual union all
select 15669,'MO' from dual union all
select 1567,'RVSN' from dual union all
select 15670,'MO' from dual union all
select 15671,'MO' from dual union all
select 15672,'MO' from dual union all
select 15673,'MO' from dual union all
select 15674,'MO' from dual union all
select 15675,'MO' from dual union all
select 15676,'MO' from dual union all
select 15677,'GTE' from dual union all
select 15678,'FTEL' from dual union all
select 15679,'AE' from dual union all
select 1568,'RVSN' from dual union all
select 15680,'AE' from dual union all
select 15681,'VMF' from dual union all
select 15682,'VMF' from dual union all
select 15683,'VMF' from dual union all
select 15684,'VMF' from dual union all
select 15685,'VMF' from dual union all
select 15686,'VMF' from dual union all
select 15687,'VMF' from dual union all
select 15688,'VMF' from dual union all
select 15689,'VMF' from dual union all
select 1569,'RVSN' from dual union all
select 15690,'VMF' from dual union all
select 15691,'VMF' from dual union all
select 15692,'VMF' from dual union all
select 15693,'RVSN' from dual union all
select 15694,'MO' from dual union all
select 15695,'RVSN' from dual union all
select 15696,'MO' from dual union all
select 15697,'MO' from dual union all
select 15698,'MO' from dual union all
select 15699,'MO' from dual union all
select 157,'USAF' from dual union all
select 1570,'OKB10' from dual union all
select 15700,'RVSN' from dual union all
select 15701,'RVSN' from dual union all
select 15702,'RVSN' from dual union all
select 15703,'VMF' from dual union all
select 15704,'VMF' from dual union all
select 15705,'VMF' from dual union all
select 15706,'VMF' from dual union all
select 15707,'VMF' from dual union all
select 15708,'VMF' from dual union all
select 15709,'VMF' from dual union all
select 1571,'OKB10' from dual union all
select 15710,'VMF' from dual union all
select 15711,'VMF' from dual union all
select 15712,'VMF' from dual union all
select 15713,'VMF' from dual union all
select 15714,'RVSN' from dual union all
select 15715,'RVSN' from dual union all
select 15716,'VMF' from dual union all
select 15717,'VMF' from dual union all
select 15718,'VMF' from dual union all
select 15719,'VMF' from dual union all
select 1572,'OKB10' from dual;
insert into SATELLITE_ORG
select 15720,'VMF' from dual union all
select 15721,'VMF' from dual union all
select 15722,'VMF' from dual union all
select 15723,'VMF' from dual union all
select 15724,'VMF' from dual union all
select 15725,'VMF' from dual union all
select 15726,'VMF' from dual union all
select 15727,'VMF' from dual union all
select 15728,'VMF' from dual union all
select 15729,'VMF' from dual union all
select 1573,'OKB10' from dual union all
select 15730,'VMF' from dual union all
select 15731,'VMF' from dual union all
select 15732,'MVD' from dual union all
select 15733,'RVSN' from dual union all
select 15734,'MO' from dual union all
select 15735,'RVSN' from dual union all
select 15736,'VMF' from dual union all
select 15737,'VMF' from dual union all
select 15738,'MOM' from dual union all
select 15738,'MSVYAZ' from dual union all
select 15739,'RVSN' from dual union all
select 1574,'OKB10' from dual union all
select 15740,'RVSN' from dual union all
select 15741,'RVSN' from dual union all
select 15742,'MO' from dual union all
select 15743,'MO' from dual union all
select 15744,'MO' from dual union all
select 15745,'MO' from dual union all
select 15746,'MO' from dual union all
select 15747,'MO' from dual union all
select 15748,'RVSN' from dual union all
select 15749,'RVSN' from dual union all
select 1575,'RVSN' from dual union all
select 15750,'RVSN' from dual union all
select 15751,'VMF' from dual union all
select 15752,'RVSN' from dual union all
select 15753,'MO' from dual union all
select 15754,'MO' from dual union all
select 15755,'TSN108' from dual union all
select 15756,'MO' from dual union all
select 15757,'MO' from dual union all
select 15758,'PVO' from dual union all
select 15759,'PVO' from dual union all
select 1576,'USN' from dual union all
select 15760,'USAF' from dual union all
select 15761,'PVO' from dual union all
select 15762,'PVO' from dual union all
select 15763,'VMF' from dual union all
select 15764,'USN' from dual union all
select 15765,'MO' from dual union all
select 15766,'MO' from dual union all
select 15767,'MO' from dual union all
select 15768,'MO' from dual union all
select 15769,'RVSN' from dual union all
select 1577,'USN' from dual union all
select 15770,'RVSN' from dual union all
select 15771,'RVSN' from dual union all
select 15772,'RVSN' from dual union all
select 15773,'RVSN' from dual union all
select 15774,'RVSN' from dual union all
select 15775,'PVO' from dual union all
select 15776,'PVO' from dual union all
select 15777,'PVO' from dual union all
select 15778,'NASA' from dual union all
select 15779,'NASA' from dual union all
select 1578,'MO' from dual union all
select 15780,'MO' from dual union all
select 15781,'PVO' from dual union all
select 15782,'PVO' from dual union all
select 15783,'USAF' from dual union all
select 15784,'RVSN' from dual union all
select 15785,'MO' from dual union all
select 15786,'RVSN' from dual union all
select 15787,'PVO' from dual union all
select 15788,'NASA' from dual union all
select 15789,'NASA' from dual union all
select 1579,'RVSN' from dual union all
select 15790,'MO' from dual union all
select 15791,'MO' from dual union all
select 15792,'MO' from dual union all
select 15793,'MO' from dual union all
select 15794,'MO' from dual union all
select 15795,'RVSN' from dual union all
select 15796,'VMF' from dual union all
select 15797,'VMF' from dual union all
select 15798,'VMF' from dual union all
select 15799,'VMF' from dual union all
select 158,'USAF' from dual union all
select 1580,'USAF' from dual union all
select 15800,'VMF' from dual union all
select 15801,'VMF' from dual union all
select 15802,'VMF' from dual union all
select 15803,'VMF' from dual union all
select 15804,'NPOE' from dual union all
select 15805,'RVSN' from dual union all
select 15806,'MVD' from dual union all
select 15807,'RVSN' from dual union all
select 15808,'PVO' from dual union all
select 15809,'RVSN' from dual;
insert into SATELLITE_ORG
select 1581,'USAF' from dual union all
select 15810,'RVSN' from dual union all
select 15811,'RVSN' from dual union all
select 15812,'VMF' from dual union all
select 15813,'VMF' from dual union all
select 15814,'VMF' from dual union all
select 15815,'VMF' from dual union all
select 15816,'VMF' from dual union all
select 15817,'VMF' from dual union all
select 15818,'MO' from dual union all
select 15819,'RVSN' from dual union all
select 1582,'USAF' from dual union all
select 15820,'MO' from dual union all
select 15821,'MOM' from dual union all
select 15822,'MO' from dual union all
select 15823,'NASA' from dual union all
select 15824,'SCT' from dual union all
select 15825,'ARAB' from dual union all
select 15826,'ATT' from dual union all
select 15827,'PVO' from dual union all
select 15828,'RVSN' from dual union all
select 15829,'RVSN' from dual union all
select 1583,'USAF' from dual union all
select 15830,'RVSN' from dual union all
select 15831,'NASA' from dual union all
select 15832,'SCT' from dual union all
select 15833,'PVO' from dual union all
select 15834,'RVSN' from dual union all
select 15835,'PVO' from dual union all
select 15836,'ARAB' from dual union all
select 15837,'ATT' from dual union all
select 15838,'NPOE' from dual union all
select 15839,'RVSN' from dual union all
select 1584,'OKB10' from dual union all
select 15840,'MVD' from dual union all
select 15841,'RVSN' from dual union all
select 15842,'MO' from dual union all
select 15843,'MO' from dual union all
select 15844,'MO' from dual union all
select 15845,'MO' from dual union all
select 15846,'MO' from dual union all
select 15847,'MO' from dual union all
select 15848,'MO' from dual union all
select 15849,'MO' from dual union all
select 1585,'OKB10' from dual union all
select 15850,'MO' from dual union all
select 15851,'MO' from dual union all
select 15852,'MO' from dual union all
select 15853,'MO' from dual union all
select 15854,'MO' from dual union all
select 15855,'MO' from dual union all
select 15856,'IKI' from dual union all
select 15856,'NPOL' from dual union all
select 15857,'IKI' from dual union all
select 15857,'NPOL' from dual union all
select 15858,'IKI' from dual union all
select 15858,'NPOL' from dual union all
select 15859,'IKI' from dual union all
select 15859,'NPOL' from dual union all
select 1586,'OKB10' from dual union all
select 15860,'MO' from dual union all
select 15861,'RVSN' from dual union all
select 15862,'PVO' from dual union all
select 15863,'PVO' from dual union all
select 15864,'PVO' from dual union all
select 15865,'PVO' from dual union all
select 15866,'MO' from dual union all
select 15867,'MO' from dual union all
select 15868,'MO' from dual union all
select 15869,'MO' from dual union all
select 1587,'OKB10' from dual union all
select 15870,'MO' from dual union all
select 15871,'MO' from dual union all
select 15872,'MO' from dual union all
select 15873,'INTEL' from dual union all
select 15874,'NASA' from dual union all
select 15875,'ESA' from dual union all
select 15876,'AE' from dual union all
select 15877,'MO' from dual union all
select 15878,'RVSN' from dual union all
select 15879,'MO' from dual union all
select 1588,'OKB10' from dual union all
select 15880,'MO' from dual union all
select 15881,'MO' from dual union all
select 15882,'MO' from dual union all
select 15883,'MO' from dual union all
select 15884,'MO' from dual union all
select 15885,'MO' from dual union all
select 15886,'MO' from dual union all
select 15887,'MO' from dual union all
select 15888,'MO' from dual union all
select 15889,'VS' from dual union all
select 1589,'RVSN' from dual union all
select 15890,'MO' from dual union all
select 15891,'IMBP' from dual union all
select 15891,'TSSKB' from dual union all
select 15892,'RVSN' from dual union all
select 15893,'MO' from dual union all
select 15894,'MO' from dual union all
select 15895,'MO' from dual;
insert into SATELLITE_ORG
select 15896,'MO' from dual union all
select 15897,'MO' from dual union all
select 15898,'MO' from dual union all
select 15899,'MO' from dual union all
select 159,'USAF' from dual union all
select 1590,'RVSN' from dual union all
select 15900,'MO' from dual union all
select 15901,'MO' from dual union all
select 15902,'MO' from dual union all
select 15903,'MO' from dual union all
select 15904,'MO' from dual union all
select 15905,'MO' from dual union all
select 15906,'MO' from dual union all
select 15907,'RVSN' from dual union all
select 15908,'MO' from dual union all
select 15909,'MO' from dual union all
select 15909,'MSVYAZ' from dual union all
select 1591,'RVSN' from dual union all
select 15910,'RVSN' from dual union all
select 15911,'RVSN' from dual union all
select 15912,'MO' from dual union all
select 15913,'MO' from dual union all
select 15914,'MO' from dual union all
select 15915,'MO' from dual union all
select 15916,'RVSN' from dual union all
select 15917,'MO' from dual union all
select 15918,'NPOE' from dual union all
select 15919,'RVSN' from dual union all
select 1592,'RVSN' from dual union all
select 15920,'MO' from dual union all
select 15921,'MO' from dual union all
select 15922,'MO' from dual union all
select 15923,'MO' from dual union all
select 15924,'MO' from dual union all
select 15925,'NASA' from dual union all
select 15926,'MO' from dual union all
select 15927,'MO' from dual union all
select 15928,'MO' from dual union all
select 15929,'NASA' from dual union all
select 1593,'RVSN' from dual union all
select 15930,'VMF' from dual union all
select 15931,'MO' from dual union all
select 15932,'RVSN' from dual union all
select 15933,'NPOE' from dual union all
select 15934,'NPOE' from dual union all
select 15935,'USN' from dual union all
select 15936,'USN' from dual union all
select 15937,'NPOE' from dual union all
select 15938,'NASA' from dual union all
select 15939,'NPOE' from dual union all
select 1594,'RVSN' from dual union all
select 15940,'MVD' from dual union all
select 15941,'RVSN' from dual union all
select 15942,'TSSKB' from dual union all
select 15943,'RVSN' from dual union all
select 15944,'VS' from dual union all
select 15945,'MO' from dual union all
select 15946,'MO' from dual union all
select 15947,'RVSN' from dual union all
select 15948,'RVSN' from dual union all
select 15949,'VTS' from dual union all
select 1595,'RVSN' from dual union all
select 15950,'USN' from dual union all
select 15951,'USN' from dual union all
select 15952,'PVO' from dual union all
select 15953,'RVSN' from dual union all
select 15954,'RVSN' from dual union all
select 15955,'RVSN' from dual union all
select 15956,'RVSN' from dual union all
select 15957,'RVSN' from dual union all
select 15958,'RVSN' from dual union all
select 15959,'MO' from dual union all
select 1596,'RVSN' from dual union all
select 15960,'RVSN' from dual union all
select 15961,'MO' from dual union all
select 15962,'MO' from dual union all
select 15963,'RVSN' from dual union all
select 15964,'MO' from dual union all
select 15965,'MO' from dual union all
select 15966,'MO' from dual union all
select 15967,'MONBU' from dual union all
select 15968,'MONBU' from dual union all
select 15969,'MONBU' from dual union all
select 1597,'RVSN' from dual union all
select 15970,'NPOE' from dual union all
select 15971,'MO' from dual union all
select 15972,'MO' from dual union all
select 15973,'MO' from dual union all
select 15974,'MO' from dual union all
select 15975,'MO' from dual union all
select 15976,'MO' from dual union all
select 15977,'MO' from dual union all
select 15978,'RVSN' from dual union all
select 15979,'RVSN' from dual union all
select 1598,'RVSN' from dual union all
select 15980,'PVO' from dual union all
select 15981,'PVO' from dual union all
select 15982,'PVO' from dual union all
select 15983,'RVSN' from dual union all
select 15984,'PVO' from dual;
insert into SATELLITE_ORG
select 15985,'PVO' from dual union all
select 15986,'VMF' from dual union all
select 15987,'PVO' from dual union all
select 15988,'PVO' from dual union all
select 15989,'PVO' from dual union all
select 1599,'RVSN' from dual union all
select 15990,'PVO' from dual union all
select 15991,'PVO' from dual union all
select 15992,'NASA' from dual union all
select 15993,'AUSS' from dual union all
select 15994,'CONTL' from dual union all
select 15995,'HEC' from dual union all
select 15996,'AUSS' from dual union all
select 15997,'MVD' from dual union all
select 15998,'RVSN' from dual union all
select 15999,'MO' from dual union all
select 16,'USN' from dual union all
select 160,'CIA' from dual union all
select 160,'USAF' from dual union all
select 1600,'RVSN' from dual union all
select 16000,'RVSN' from dual union all
select 16001,'HEC' from dual union all
select 16002,'MO' from dual union all
select 16003,'PVO' from dual union all
select 16004,'PVO' from dual union all
select 16005,'PVO' from dual union all
select 16006,'PVO' from dual union all
select 16007,'CONTL' from dual union all
select 16008,'PVO' from dual union all
select 16009,'PVO' from dual union all
select 1601,'RVSN' from dual union all
select 16010,'PVO' from dual union all
select 16011,'MO' from dual union all
select 16012,'RVSN' from dual union all
select 16013,'NASA' from dual union all
select 16014,'PVO' from dual union all
select 16015,'PVO' from dual union all
select 16016,'PVO' from dual union all
select 16017,'PVO' from dual union all
select 16018,'MVD' from dual union all
select 16019,'RVSN' from dual union all
select 1602,'CIA' from dual union all
select 1602,'NRO' from dual union all
select 16020,'USN' from dual union all
select 16021,'MO' from dual union all
select 16022,'MO' from dual union all
select 16023,'MO' from dual union all
select 16024,'MO' from dual union all
select 16025,'MO' from dual union all
select 16026,'MO' from dual union all
select 16027,'USAF' from dual union all
select 16028,'USAF' from dual union all
select 16029,'USAF' from dual union all
select 1603,'MO' from dual union all
select 16030,'USAF' from dual union all
select 16031,'USAF' from dual union all
select 16032,'USAF' from dual union all
select 16033,'USAF' from dual union all
select 16034,'USAF' from dual union all
select 16035,'USAF' from dual union all
select 16036,'USAF' from dual union all
select 16037,'USAF' from dual union all
select 16038,'USAF' from dual union all
select 16039,'USAF' from dual union all
select 1604,'RVSN' from dual union all
select 16040,'USAF' from dual union all
select 16041,'USAF' from dual union all
select 16042,'USAF' from dual union all
select 16043,'USAF' from dual union all
select 16044,'USAF' from dual union all
select 16045,'USAF' from dual union all
select 16046,'NPOE' from dual union all
select 16047,'NPOE' from dual union all
select 16048,'NPOE' from dual union all
select 16049,'NPOE' from dual union all
select 1605,'RVSN' from dual union all
select 16050,'NPOE' from dual union all
select 16051,'NPOE' from dual union all
select 16052,'RVSN' from dual union all
select 16053,'NPOE' from dual union all
select 16054,'VMF' from dual union all
select 16055,'MO' from dual union all
select 16056,'MO' from dual union all
select 16057,'RVSN' from dual union all
select 16058,'MO' from dual union all
select 16059,'MO' from dual union all
select 1606,'RVSN' from dual union all
select 16060,'MO' from dual union all
select 16061,'RVSN' from dual union all
select 16062,'NPOE' from dual union all
select 16063,'NPOE' from dual union all
select 16064,'PVO' from dual union all
select 16065,'RVSN' from dual union all
select 16066,'RVSN' from dual union all
select 16067,'PVO' from dual union all
select 16068,'PVO' from dual union all
select 16069,'PVO' from dual union all
select 1607,'RVSN' from dual union all
select 16070,'RVSN' from dual union all
select 16071,'USAF' from dual;
insert into SATELLITE_ORG
select 16072,'USAF' from dual union all
select 16073,'USAF' from dual union all
select 16074,'USAF' from dual union all
select 16075,'USAF' from dual union all
select 16076,'USAF' from dual union all
select 16077,'PVO' from dual union all
select 16078,'USAF' from dual union all
select 16079,'USAF' from dual union all
select 1608,'USN' from dual union all
select 16080,'USAF' from dual union all
select 16081,'USAF' from dual union all
select 16082,'USAF' from dual union all
select 16083,'USAF' from dual union all
select 16084,'USAF' from dual union all
select 16085,'USAF' from dual union all
select 16086,'USAF' from dual union all
select 16087,'USAF' from dual union all
select 16088,'MO' from dual union all
select 16089,'RVSN' from dual union all
select 1609,'NRO' from dual union all
select 16090,'RVSN' from dual union all
select 16091,'NPOE' from dual union all
select 16092,'NPOE' from dual union all
select 16093,'NPOE' from dual union all
select 16094,'NPOE' from dual union all
select 16095,'KBSAL' from dual union all
select 16096,'RVSN' from dual union all
select 16097,'NPOE' from dual union all
select 16098,'NPOE' from dual union all
select 16099,'NPOE' from dual union all
select 161,'USAF' from dual union all
select 1610,'OKB1' from dual union all
select 16100,'NPOE' from dual union all
select 16101,'INTEL' from dual union all
select 16102,'NASA' from dual union all
select 16103,'PVO' from dual union all
select 16104,'RVSN' from dual union all
select 16105,'RVSN' from dual union all
select 16106,'RVSN' from dual union all
select 16107,'PVO' from dual union all
select 16108,'RVSN' from dual union all
select 16109,'PVO' from dual union all
select 1611,'RVSN' from dual union all
select 16110,'MVD' from dual union all
select 16111,'RVSN' from dual union all
select 16112,'MO' from dual union all
select 16112,'MSVYAZ' from dual union all
select 16113,'RVSN' from dual union all
select 16114,'RVSN' from dual union all
select 16115,'NASA' from dual union all
select 16116,'DOD' from dual union all
select 16116,'USAF' from dual union all
select 16117,'DOD' from dual union all
select 16117,'USAF' from dual union all
select 16118,'USAF' from dual union all
select 16119,'USAF' from dual union all
select 1612,'RVSN' from dual union all
select 16120,'MO' from dual union all
select 16121,'MO' from dual union all
select 16122,'NPOE' from dual union all
select 16123,'MO' from dual union all
select 16124,'MO' from dual union all
select 16125,'RVSN' from dual union all
select 16126,'MO' from dual union all
select 16127,'MO' from dual union all
select 16128,'NPOE' from dual union all
select 16129,'USAF' from dual union all
select 1613,'USAF' from dual union all
select 16130,'USAF' from dual union all
select 16131,'USAF' from dual union all
select 16132,'USAF' from dual union all
select 16133,'USAF' from dual union all
select 16134,'MO' from dual union all
select 16135,'MO' from dual union all
select 16136,'MO' from dual union all
select 16137,'USAF' from dual union all
select 16138,'MO' from dual union all
select 16139,'MO' from dual union all
select 1614,'USAF' from dual union all
select 16140,'MO' from dual union all
select 16141,'MO' from dual union all
select 16142,'MO' from dual union all
select 16143,'MO' from dual union all
select 16144,'MO' from dual union all
select 16145,'MO' from dual union all
select 16146,'MO' from dual union all
select 16147,'USAF' from dual union all
select 16148,'USAF' from dual union all
select 16149,'USAF' from dual union all
select 1615,'CIA' from dual union all
select 1615,'NRO' from dual union all
select 16150,'USAF' from dual union all
select 16151,'USAF' from dual union all
select 16152,'USAF' from dual union all
select 16153,'USAF' from dual union all
select 16154,'USAF' from dual union all
select 16155,'USAF' from dual union all
select 16156,'USAF' from dual union all
select 16157,'USAF' from dual union all
select 16158,'USAF' from dual;
insert into SATELLITE_ORG
select 16159,'MO' from dual union all
select 1616,'USAF' from dual union all
select 16160,'MO' from dual union all
select 16161,'MO' from dual union all
select 16162,'MO' from dual union all
select 16163,'NPOE' from dual union all
select 16164,'NPOE' from dual union all
select 16165,'NPOE' from dual union all
select 16166,'NPOE' from dual union all
select 16167,'NPOE' from dual union all
select 16168,'NPOE' from dual union all
select 16169,'MO' from dual union all
select 1617,'RVSN' from dual union all
select 16170,'RVSN' from dual union all
select 16171,'NPOE' from dual union all
select 16172,'NPOE' from dual union all
select 16173,'NPOE' from dual union all
select 16174,'NPOE' from dual union all
select 16175,'NPOE' from dual union all
select 16176,'NPOE' from dual union all
select 16177,'MAI' from dual union all
select 16178,'MAI' from dual union all
select 16179,'MAI' from dual union all
select 1618,'RVSN' from dual union all
select 16180,'MAI' from dual union all
select 16181,'YUZH' from dual union all
select 16182,'MO' from dual union all
select 16183,'PVO' from dual union all
select 16184,'RVSN' from dual union all
select 16185,'RVSN' from dual union all
select 16186,'RVSN' from dual union all
select 16187,'MO' from dual union all
select 16188,'RVSN' from dual union all
select 16189,'RVSN' from dual union all
select 1619,'RVSN' from dual union all
select 16190,'RVSN' from dual union all
select 16191,'GMS' from dual union all
select 16192,'VMF' from dual union all
select 16193,'VMF' from dual union all
select 16194,'MO' from dual union all
select 16195,'VMF' from dual union all
select 16196,'VMF' from dual union all
select 16197,'RVSN' from dual union all
select 16198,'MO' from dual union all
select 16199,'MO' from dual union all
select 162,'NASA' from dual union all
select 1620,'NASA' from dual union all
select 16200,'RVSN' from dual union all
select 16201,'RVSN' from dual union all
select 16202,'PVO' from dual union all
select 16203,'PVO' from dual union all
select 16204,'PVO' from dual union all
select 16205,'PVO' from dual union all
select 16206,'PVO' from dual union all
select 16207,'PVO' from dual union all
select 16208,'PVO' from dual union all
select 16209,'PVO' from dual union all
select 1621,'OKB10' from dual union all
select 16210,'PVO' from dual union all
select 16211,'PVO' from dual union all
select 16212,'RVSN' from dual union all
select 16213,'VMF' from dual union all
select 16214,'RVSN' from dual union all
select 16215,'RVSN' from dual union all
select 16216,'RVSN' from dual union all
select 16217,'NPOE' from dual union all
select 16218,'NPOE' from dual union all
select 16219,'VMF' from dual union all
select 1622,'RVSN' from dual union all
select 16220,'MO' from dual union all
select 16221,'RVSN' from dual union all
select 16222,'RVSN' from dual union all
select 16223,'RVSN' from dual union all
select 16224,'RVSN' from dual union all
select 16225,'RVSN' from dual union all
select 16226,'RVSN' from dual union all
select 16227,'MO' from dual union all
select 16228,'MO' from dual union all
select 16229,'HEC' from dual union all
select 1623,'RVSN' from dual union all
select 16230,'NASA' from dual union all
select 16231,'USN' from dual union all
select 16232,'MO' from dual union all
select 16233,'MO' from dual union all
select 16234,'MO' from dual union all
select 16235,'PVO' from dual union all
select 16236,'RVSN' from dual union all
select 16237,'RVSN' from dual union all
select 16238,'NPOE' from dual union all
select 16239,'NPOE' from dual union all
select 1624,'USAF' from dual union all
select 16240,'NPOE' from dual union all
select 16241,'NPOE' from dual union all
select 16242,'NPOE' from dual union all
select 16243,'RVSN' from dual union all
select 16244,'NPOE' from dual union all
select 16245,'NPOE' from dual union all
select 16246,'NPOE' from dual union all
select 16247,'MO' from dual union all
select 16248,'RVSN' from dual;
insert into SATELLITE_ORG
select 16249,'NPOE' from dual union all
select 1625,'NASA' from dual union all
select 16250,'MO' from dual union all
select 16251,'RVSN' from dual union all
select 16252,'RVSN' from dual union all
select 16253,'NPOE' from dual union all
select 16254,'RVSN' from dual union all
select 16255,'RVSN' from dual union all
select 16256,'PVO' from dual union all
select 16257,'PVO' from dual union all
select 16258,'NPOE' from dual union all
select 16259,'NPOE' from dual union all
select 1626,'MO' from dual union all
select 16260,'NPOE' from dual union all
select 16261,'NPOE' from dual union all
select 16262,'VS' from dual union all
select 16263,'RVSN' from dual union all
select 16264,'RVSN' from dual union all
select 16265,'RVSN' from dual union all
select 16266,'RVSN' from dual union all
select 16267,'RVSN' from dual union all
select 16268,'RVSN' from dual union all
select 16269,'RVSN' from dual union all
select 1627,'RVSN' from dual union all
select 16270,'RVSN' from dual union all
select 16271,'RVSN' from dual union all
select 16272,'RVSN' from dual union all
select 16273,'NASA' from dual union all
select 16274,'SCT' from dual union all
select 16275,'AUSS' from dual union all
select 16276,'AMC' from dual union all
select 16277,'NASA' from dual union all
select 16278,'USAF' from dual union all
select 16279,'USAF' from dual union all
select 1628,'MO' from dual union all
select 16280,'USAF' from dual union all
select 16281,'USAF' from dual union all
select 16282,'USAF' from dual union all
select 16283,'USAF' from dual union all
select 16284,'USAF' from dual union all
select 16285,'MO' from dual union all
select 16286,'MO' from dual union all
select 16287,'MO' from dual union all
select 16288,'MO' from dual union all
select 16289,'MO' from dual union all
select 1629,'YUZH' from dual union all
select 16290,'MO' from dual union all
select 16291,'VMF' from dual union all
select 16292,'RVSN' from dual union all
select 16293,'SCT' from dual union all
select 16294,'AUSS' from dual union all
select 16295,'AMC' from dual union all
select 16296,'MO' from dual union all
select 16297,'RVSN' from dual union all
select 16298,'USAF' from dual union all
select 16299,'USAF' from dual union all
select 163,'USAF' from dual union all
select 1630,'RVSN' from dual union all
select 16300,'USAF' from dual union all
select 16301,'USAF' from dual union all
select 16302,'USAF' from dual union all
select 16303,'USAF' from dual union all
select 16304,'USAF' from dual union all
select 16305,'MO' from dual union all
select 16306,'MO' from dual union all
select 16307,'RVSN' from dual union all
select 16308,'USAF' from dual union all
select 16309,'USAF' from dual union all
select 1631,'YUZH' from dual union all
select 16310,'USAF' from dual union all
select 16311,'USAF' from dual union all
select 16312,'USAF' from dual union all
select 16313,'USAF' from dual union all
select 16314,'USAF' from dual union all
select 16315,'USAF' from dual union all
select 16316,'USAF' from dual union all
select 16317,'USAF' from dual union all
select 16318,'USAF' from dual union all
select 16319,'USAF' from dual union all
select 1632,'YUZH' from dual union all
select 16320,'USAF' from dual union all
select 16321,'USAF' from dual union all
select 16322,'USAF' from dual union all
select 16323,'USAF' from dual union all
select 16324,'USAF' from dual union all
select 16325,'USAF' from dual union all
select 16326,'VS' from dual union all
select 16327,'MO' from dual union all
select 16328,'USAF' from dual union all
select 16329,'USAF' from dual union all
select 1633,'RVSN' from dual union all
select 16330,'NASA' from dual union all
select 16331,'MVD' from dual union all
select 16332,'RVSN' from dual union all
select 16333,'USAF' from dual union all
select 16334,'USAF' from dual union all
select 16335,'USAF' from dual union all
select 16336,'USAF' from dual union all
select 16337,'USAF' from dual union all
select 16338,'USAF' from dual;
insert into SATELLITE_ORG
select 16339,'RVSN' from dual union all
select 1634,'RVSN' from dual union all
select 16340,'MO' from dual union all
select 16341,'USAF' from dual union all
select 16342,'USAF' from dual union all
select 16343,'USAF' from dual union all
select 16344,'USAF' from dual union all
select 16345,'USAF' from dual union all
select 16346,'USAF' from dual union all
select 16347,'USAF' from dual union all
select 16348,'USAF' from dual union all
select 16349,'USAF' from dual union all
select 1635,'RVSN' from dual union all
select 16350,'USAF' from dual union all
select 16351,'USAF' from dual union all
select 16352,'USAF' from dual union all
select 16353,'MO' from dual union all
select 16354,'MO' from dual union all
select 16355,'MO' from dual union all
select 16356,'MO' from dual union all
select 16357,'MO' from dual union all
select 16358,'MO' from dual union all
select 16359,'MO' from dual union all
select 1636,'MO' from dual union all
select 16360,'USAF' from dual union all
select 16361,'USAF' from dual union all
select 16362,'USAF' from dual union all
select 16363,'USAF' from dual union all
select 16364,'USAF' from dual union all
select 16365,'USAF' from dual union all
select 16366,'USAF' from dual union all
select 16367,'USAF' from dual union all
select 16368,'VMF' from dual union all
select 16369,'RVSN' from dual union all
select 1637,'CIA' from dual union all
select 1637,'NRO' from dual union all
select 16370,'MO' from dual union all
select 16371,'MO' from dual union all
select 16372,'MO' from dual union all
select 16373,'MO' from dual union all
select 16374,'MO' from dual union all
select 16375,'PVO' from dual union all
select 16376,'PVO' from dual union all
select 16377,'PVO' from dual union all
select 16378,'PVO' from dual union all
select 16379,'PVO' from dual union all
select 1638,'RVSN' from dual union all
select 16380,'PVO' from dual union all
select 16381,'PVO' from dual union all
select 16382,'PVO' from dual union all
select 16383,'PVO' from dual union all
select 16384,'PVO' from dual union all
select 16385,'PVO' from dual union all
select 16386,'PVO' from dual union all
select 16387,'PVO' from dual union all
select 16388,'MONBU' from dual union all
select 16389,'MO' from dual union all
select 1639,'USAF' from dual union all
select 16390,'MO' from dual union all
select 16391,'MO' from dual union all
select 16392,'MO' from dual union all
select 16393,'MO' from dual union all
select 16393,'MSVYAZ' from dual union all
select 16394,'RVSN' from dual union all
select 16395,'RVSN' from dual union all
select 16396,'MO' from dual union all
select 16397,'MO' from dual union all
select 16398,'MO' from dual union all
select 16399,'RVSN' from dual union all
select 164,'USAF' from dual union all
select 1640,'USAF' from dual union all
select 16400,'RVSN' from dual union all
select 16401,'RVSN' from dual union all
select 16402,'RVSN' from dual union all
select 16403,'RVSN' from dual union all
select 16404,'RVSN' from dual union all
select 16405,'RVSN' from dual union all
select 16406,'RVSN' from dual union all
select 16407,'RVSN' from dual union all
select 16408,'GMS' from dual union all
select 16408,'MO' from dual union all
select 16409,'MO' from dual union all
select 1641,'USAF' from dual union all
select 16410,'RVSN' from dual union all
select 16411,'RVSN' from dual union all
select 16412,'RVSN' from dual union all
select 16413,'USAF' from dual union all
select 16414,'USAF' from dual union all
select 16415,'USAF' from dual union all
select 16416,'USAF' from dual union all
select 16417,'USAF' from dual union all
select 16418,'USAF' from dual union all
select 16419,'USAF' from dual union all
select 1642,'USAF' from dual union all
select 16420,'USAF' from dual union all
select 16421,'USAF' from dual union all
select 16422,'USAF' from dual union all
select 16423,'USAF' from dual union all
select 16424,'USAF' from dual union all
select 16425,'USAF' from dual;
insert into SATELLITE_ORG
select 16426,'MO' from dual union all
select 16427,'MO' from dual union all
select 16428,'MO' from dual union all
select 16429,'TSSKB' from dual union all
select 1643,'USAF' from dual union all
select 16430,'RVSN' from dual union all
select 16431,'MO' from dual union all
select 16432,'MO' from dual union all
select 16433,'MO' from dual union all
select 16434,'TSN108' from dual union all
select 16435,'MO' from dual union all
select 16436,'MO' from dual union all
select 16437,'MO' from dual union all
select 16438,'MO' from dual union all
select 16439,'MO' from dual union all
select 1644,'USAF' from dual union all
select 16440,'MO' from dual union all
select 16441,'NPOE' from dual union all
select 16442,'NOAA' from dual union all
select 16443,'NOAA' from dual union all
select 16444,'NOAA' from dual union all
select 16445,'RVSN' from dual union all
select 16446,'RVSN' from dual union all
select 16447,'MO' from dual union all
select 16448,'RVSN' from dual union all
select 16449,'MO' from dual union all
select 1645,'USAF' from dual union all
select 16450,'MO' from dual union all
select 16451,'MO' from dual union all
select 16452,'MO' from dual union all
select 16453,'MO' from dual union all
select 16454,'MO' from dual union all
select 16455,'MO' from dual union all
select 16456,'MO' from dual union all
select 16457,'RVSN' from dual union all
select 16458,'USAF' from dual union all
select 16459,'USAF' from dual union all
select 1646,'USAF' from dual union all
select 16460,'USAF' from dual union all
select 16461,'USAF' from dual union all
select 16462,'USAF' from dual union all
select 16463,'USAF' from dual union all
select 16464,'USAF' from dual union all
select 16465,'USAF' from dual union all
select 16466,'USAF' from dual union all
select 16467,'USAF' from dual union all
select 16468,'USAF' from dual union all
select 16469,'USAF' from dual union all
select 1647,'USAF' from dual union all
select 16470,'USAF' from dual union all
select 16471,'USAF' from dual union all
select 16472,'USAF' from dual union all
select 16473,'USAF' from dual union all
select 16474,'USAF' from dual union all
select 16475,'USAF' from dual union all
select 16476,'USAF' from dual union all
select 16477,'USAF' from dual union all
select 16478,'USAF' from dual union all
select 16479,'USAF' from dual union all
select 1648,'USAF' from dual union all
select 16480,'USAF' from dual union all
select 16481,'NASA' from dual union all
select 16482,'AMCG' from dual union all
select 16483,'AMCG' from dual union all
select 16484,'USAF' from dual union all
select 16485,'USAF' from dual union all
select 16486,'USAF' from dual union all
select 16487,'USAF' from dual union all
select 16488,'USAF' from dual union all
select 16489,'USAF' from dual union all
select 1649,'USAF' from dual union all
select 16490,'MO' from dual union all
select 16491,'RVSN' from dual union all
select 16492,'MO' from dual union all
select 16493,'VMF' from dual union all
select 16494,'RVSN' from dual union all
select 16495,'VS' from dual union all
select 16496,'MO' from dual union all
select 16497,'MO' from dual union all
select 16498,'RVSN' from dual union all
select 16499,'RVSN' from dual union all
select 165,'NASA' from dual union all
select 1650,'USAF' from dual union all
select 16500,'RVSN' from dual union all
select 16501,'RVSN' from dual union all
select 16502,'NOAA' from dual union all
select 16503,'NOAA' from dual union all
select 16504,'NOAA' from dual union all
select 16505,'MO' from dual union all
select 16506,'MO' from dual union all
select 16507,'MO' from dual union all
select 16508,'MO' from dual union all
select 16509,'MO' from dual union all
select 1651,'USAF' from dual union all
select 16510,'VMF' from dual union all
select 16511,'RVSN' from dual union all
select 16512,'MO' from dual union all
select 16513,'RVSN' from dual union all
select 16514,'MO' from dual union all
select 16515,'USAF' from dual;
insert into SATELLITE_ORG
select 16516,'USAF' from dual union all
select 16517,'USAF' from dual union all
select 16518,'USAF' from dual union all
select 16519,'USAF' from dual union all
select 1652,'USAF' from dual union all
select 16520,'USAF' from dual union all
select 16521,'USAF' from dual union all
select 16522,'USAF' from dual union all
select 16523,'USAF' from dual union all
select 16524,'USAF' from dual union all
select 16525,'USAF' from dual union all
select 16526,'MAI' from dual union all
select 16527,'PVO' from dual union all
select 16528,'MAI' from dual union all
select 16529,'RVSN' from dual union all
select 1653,'USAF' from dual union all
select 16530,'RVSN' from dual union all
select 16531,'RVSN' from dual union all
select 16532,'RVSN' from dual union all
select 16533,'RVSN' from dual union all
select 16534,'USAF' from dual union all
select 16535,'USAF' from dual union all
select 16536,'USAF' from dual union all
select 16537,'USAF' from dual union all
select 16538,'USAF' from dual union all
select 16539,'USAF' from dual union all
select 1654,'USAF' from dual union all
select 16540,'MO' from dual union all
select 16541,'RVSN' from dual union all
select 16542,'USAF' from dual union all
select 16543,'USAF' from dual union all
select 16544,'RVSN' from dual union all
select 16545,'RVSN' from dual union all
select 16546,'RVSN' from dual union all
select 16547,'RVSN' from dual union all
select 16548,'RVSN' from dual union all
select 16549,'USAF' from dual union all
select 1655,'USAF' from dual union all
select 16550,'USAF' from dual union all
select 16551,'USAF' from dual union all
select 16552,'USAF' from dual union all
select 16553,'USAF' from dual union all
select 16554,'USAF' from dual union all
select 16555,'USAF' from dual union all
select 16556,'USAF' from dual union all
select 16557,'USAF' from dual union all
select 16558,'USAF' from dual union all
select 16559,'USAF' from dual union all
select 1656,'USAF' from dual union all
select 16560,'USAF' from dual union all
select 16561,'USAF' from dual union all
select 16562,'USAF' from dual union all
select 16563,'USAF' from dual union all
select 16564,'USAF' from dual union all
select 16565,'USAF' from dual union all
select 16566,'USAF' from dual union all
select 16567,'USAF' from dual union all
select 16568,'USAF' from dual union all
select 16569,'USAF' from dual union all
select 1657,'USAF' from dual union all
select 16570,'USAF' from dual union all
select 16571,'USAF' from dual union all
select 16572,'USAF' from dual union all
select 16573,'USAF' from dual union all
select 16574,'USAF' from dual union all
select 16575,'USAF' from dual union all
select 16576,'USAF' from dual union all
select 16577,'USAF' from dual union all
select 16578,'USAF' from dual union all
select 16579,'USAF' from dual union all
select 1658,'USAF' from dual union all
select 16580,'USAF' from dual union all
select 16581,'USAF' from dual union all
select 16582,'USAF' from dual union all
select 16583,'USAF' from dual union all
select 16584,'USAF' from dual union all
select 16585,'USAF' from dual union all
select 16586,'USAF' from dual union all
select 16587,'USAF' from dual union all
select 16588,'USAF' from dual union all
select 16589,'MO' from dual union all
select 1659,'USAF' from dual union all
select 16590,'RVSN' from dual union all
select 16591,'NRO' from dual union all
select 16591,'USN' from dual union all
select 16592,'USN' from dual union all
select 16593,'VTS' from dual union all
select 16594,'MO' from dual union all
select 16595,'MO' from dual union all
select 16596,'MO' from dual union all
select 16597,'NASDA' from dual union all
select 16598,'NASDA' from dual union all
select 16599,'MO' from dual union all
select 166,'NASA' from dual union all
select 1660,'USAF' from dual union all
select 16600,'NASDA' from dual union all
select 16601,'NASDA' from dual union all
select 16602,'MO' from dual union all
select 16603,'MO' from dual union all
select 16604,'MO' from dual;
insert into SATELLITE_ORG
select 16605,'PVO' from dual union all
select 16606,'PVO' from dual union all
select 16607,'PVO' from dual union all
select 16608,'PVO' from dual union all
select 16609,'NPOE' from dual union all
select 1661,'USAF' from dual union all
select 16610,'RVSN' from dual union all
select 16611,'VS' from dual union all
select 16612,'MO' from dual union all
select 16613,'CNES' from dual union all
select 16614,'SSC' from dual union all
select 16615,'AE' from dual union all
select 16616,'AE' from dual union all
select 16617,'VMF' from dual union all
select 16618,'MO' from dual union all
select 16619,'RVSN' from dual union all
select 1662,'USAF' from dual union all
select 16620,'VMF' from dual union all
select 16621,'MO' from dual union all
select 16622,'USN' from dual union all
select 16623,'USN' from dual union all
select 16624,'NRO' from dual union all
select 16624,'USN' from dual union all
select 16625,'NRO' from dual union all
select 16625,'USN' from dual union all
select 16626,'VMF' from dual union all
select 16627,'VMF' from dual union all
select 16628,'VMF' from dual union all
select 16629,'VMF' from dual union all
select 1663,'USAF' from dual union all
select 16630,'USN' from dual union all
select 16631,'NRO' from dual union all
select 16631,'USN' from dual union all
select 16632,'VMF' from dual union all
select 16633,'VMF' from dual union all
select 16634,'VMF' from dual union all
select 16635,'VMF' from dual union all
select 16636,'VMF' from dual union all
select 16637,'VMF' from dual union all
select 16638,'VMF' from dual union all
select 16639,'VMF' from dual union all
select 1664,'USAF' from dual union all
select 16640,'VMF' from dual union all
select 16641,'VMF' from dual union all
select 16642,'VMF' from dual union all
select 16643,'NPOE' from dual union all
select 16644,'RVSN' from dual union all
select 16645,'NPOE' from dual union all
select 16646,'RVSN' from dual union all
select 16647,'VMF' from dual union all
select 16648,'VMF' from dual union all
select 16649,'GTE' from dual union all
select 1665,'USAF' from dual union all
select 16650,'TELB' from dual union all
select 16651,'VMF' from dual union all
select 16652,'VMF' from dual union all
select 16653,'VMF' from dual union all
select 16654,'VMF' from dual union all
select 16655,'VMF' from dual union all
select 16656,'VMF' from dual union all
select 16657,'AE' from dual union all
select 16658,'AE' from dual union all
select 16659,'MO' from dual union all
select 1666,'USAF' from dual union all
select 16660,'USAF' from dual union all
select 16661,'USAF' from dual union all
select 16662,'USAF' from dual union all
select 16663,'USAF' from dual union all
select 16664,'VMF' from dual union all
select 16665,'VMF' from dual union all
select 16666,'VMF' from dual union all
select 16667,'MO' from dual union all
select 16668,'RVSN' from dual union all
select 16669,'RVSN' from dual union all
select 1667,'USAF' from dual union all
select 16670,'RVSN' from dual union all
select 16671,'RVSN' from dual union all
select 16672,'VMF' from dual union all
select 16673,'VMF' from dual union all
select 16674,'VMF' from dual union all
select 16675,'VMF' from dual union all
select 16676,'RVSN' from dual union all
select 16677,'MO' from dual union all
select 16678,'RVSN' from dual union all
select 16679,'MO' from dual union all
select 1668,'USAF' from dual union all
select 16680,'RVSN' from dual union all
select 16681,'MO' from dual union all
select 16682,'RVSN' from dual union all
select 16683,'MO' from dual union all
select 16683,'MSVYAZ' from dual union all
select 16684,'RVSN' from dual union all
select 16685,'RVSN' from dual union all
select 16686,'RVSN' from dual union all
select 16687,'NPOE' from dual union all
select 16688,'RVSN' from dual union all
select 16689,'VMF' from dual union all
select 1669,'USAF' from dual union all
select 16690,'VMF' from dual union all
select 16691,'VMF' from dual;
insert into SATELLITE_ORG
select 16692,'MO' from dual union all
select 16693,'MO' from dual union all
select 16694,'MO' from dual union all
select 16695,'MO' from dual union all
select 16696,'MO' from dual union all
select 16697,'MO' from dual union all
select 16698,'MO' from dual union all
select 16699,'MO' from dual union all
select 167,'NASA' from dual union all
select 1670,'USAF' from dual union all
select 16700,'MO' from dual union all
select 16701,'MO' from dual union all
select 16702,'VMF' from dual union all
select 16703,'VMF' from dual union all
select 16704,'VMF' from dual union all
select 16705,'VMF' from dual union all
select 16706,'VMF' from dual union all
select 16707,'VMF' from dual union all
select 16708,'VMF' from dual union all
select 16709,'VMF' from dual union all
select 1671,'USAF' from dual union all
select 16710,'VMF' from dual union all
select 16711,'VMF' from dual union all
select 16712,'VMF' from dual union all
select 16713,'VMF' from dual union all
select 16714,'MO' from dual union all
select 16715,'NASA' from dual union all
select 16716,'MO' from dual union all
select 16717,'MO' from dual union all
select 16718,'RVSN' from dual union all
select 16719,'VS' from dual union all
select 1672,'USAF' from dual union all
select 16720,'MO' from dual union all
select 16721,'NPOE' from dual union all
select 16722,'NPOE' from dual union all
select 16723,'RVSN' from dual union all
select 16724,'TSSKB' from dual union all
select 16725,'RVSN' from dual union all
select 16726,'USAF' from dual union all
select 16727,'VMF' from dual union all
select 16728,'RVSN' from dual union all
select 16729,'MSVYAZ' from dual union all
select 1673,'USAF' from dual union all
select 16730,'RVSN' from dual union all
select 16731,'RVSN' from dual union all
select 16732,'RVSN' from dual union all
select 16733,'RVSN' from dual union all
select 16734,'RVSN' from dual union all
select 16735,'GMS' from dual union all
select 16735,'MO' from dual union all
select 16736,'MO' from dual union all
select 16737,'MVD' from dual union all
select 16738,'RVSN' from dual union all
select 16739,'MO' from dual union all
select 1674,'USAF' from dual union all
select 16740,'MO' from dual union all
select 16741,'MO' from dual union all
select 16742,'MO' from dual union all
select 16743,'MO' from dual union all
select 16744,'MO' from dual union all
select 16745,'MO' from dual union all
select 16746,'RVSN' from dual union all
select 16747,'MO' from dual union all
select 16748,'NPOE' from dual union all
select 16749,'NPOE' from dual union all
select 1675,'USAF' from dual union all
select 16750,'NPOE' from dual union all
select 16751,'NPOE' from dual union all
select 16752,'NPOE' from dual union all
select 16753,'MO' from dual union all
select 16754,'MO' from dual union all
select 16755,'MO' from dual union all
select 16756,'MO' from dual union all
select 16757,'MO' from dual union all
select 16758,'MO' from dual union all
select 16759,'MO' from dual union all
select 1676,'USAF' from dual union all
select 16760,'MO' from dual union all
select 16761,'MO' from dual union all
select 16762,'MO' from dual union all
select 16763,'MO' from dual union all
select 16764,'MO' from dual union all
select 16765,'MO' from dual union all
select 16766,'RVSN' from dual union all
select 16767,'MO' from dual union all
select 16768,'RVSN' from dual union all
select 16769,'MSVYAZ' from dual union all
select 1677,'USAF' from dual union all
select 16770,'RVSN' from dual union all
select 16771,'RVSN' from dual union all
select 16772,'MO' from dual union all
select 16773,'RVSN' from dual union all
select 16774,'VMF' from dual union all
select 16775,'VMF' from dual union all
select 16776,'VMF' from dual union all
select 16777,'VMF' from dual union all
select 16778,'VMF' from dual union all
select 16779,'VMF' from dual union all
select 1678,'USAF' from dual union all
select 16780,'VMF' from dual;
insert into SATELLITE_ORG
select 16781,'VMF' from dual union all
select 16782,'VMF' from dual union all
select 16783,'MO' from dual union all
select 16784,'MO' from dual union all
select 16785,'MO' from dual union all
select 16786,'MO' from dual union all
select 16787,'MO' from dual union all
select 16788,'MO' from dual union all
select 16789,'RVSN' from dual union all
select 1679,'USAF' from dual union all
select 16790,'RVSN' from dual union all
select 16791,'VS' from dual union all
select 16792,'MO' from dual union all
select 16793,'MO' from dual union all
select 16794,'MO' from dual union all
select 16795,'MO' from dual union all
select 16796,'MO' from dual union all
select 16797,'RVSN' from dual union all
select 16798,'VMF' from dual union all
select 16799,'RVSN' from dual union all
select 168,'OKB1' from dual union all
select 1680,'USAF' from dual union all
select 16800,'MO' from dual union all
select 16801,'RVSN' from dual union all
select 16802,'MO' from dual union all
select 16802,'MSVYAZ' from dual union all
select 16803,'RVSN' from dual union all
select 16804,'RVSN' from dual union all
select 16805,'RVSN' from dual union all
select 16806,'VMF' from dual union all
select 16807,'VMF' from dual union all
select 16808,'VMF' from dual union all
select 16809,'VMF' from dual union all
select 1681,'USAF' from dual union all
select 16810,'NPOE' from dual union all
select 16811,'NPOE' from dual union all
select 16812,'NPOE' from dual union all
select 16813,'NPOE' from dual union all
select 16814,'NPOE' from dual union all
select 16815,'NPOE' from dual union all
select 16816,'MO' from dual union all
select 16817,'MO' from dual union all
select 16818,'MO' from dual union all
select 16819,'MO' from dual union all
select 1682,'USAF' from dual union all
select 16820,'MO' from dual union all
select 16821,'NPOE' from dual union all
select 16822,'NPOE' from dual union all
select 16823,'VMF' from dual union all
select 16824,'VMF' from dual union all
select 16825,'VMF' from dual union all
select 16826,'VMF' from dual union all
select 16827,'VMF' from dual union all
select 16828,'VMF' from dual union all
select 16829,'VMF' from dual union all
select 1683,'USAF' from dual union all
select 16830,'VMF' from dual union all
select 16831,'VMF' from dual union all
select 16832,'VMF' from dual union all
select 16833,'VMF' from dual union all
select 16834,'VMF' from dual union all
select 16835,'VMF' from dual union all
select 16836,'VMF' from dual union all
select 16837,'VMF' from dual union all
select 16838,'VMF' from dual union all
select 16839,'VMF' from dual union all
select 1684,'USAF' from dual union all
select 16840,'VMF' from dual union all
select 16841,'VMF' from dual union all
select 16842,'VMF' from dual union all
select 16843,'USAF' from dual union all
select 16844,'MO' from dual union all
select 16845,'MO' from dual union all
select 16846,'MO' from dual union all
select 16847,'MO' from dual union all
select 16848,'MO' from dual union all
select 16849,'PVO' from dual union all
select 1685,'USAF' from dual union all
select 16850,'RVSN' from dual union all
select 16851,'RVSN' from dual union all
select 16852,'NPOE' from dual union all
select 16853,'NPOE' from dual union all
select 16854,'RVSN' from dual union all
select 16855,'MVD' from dual union all
select 16856,'RVSN' from dual union all
select 16857,'VMF' from dual union all
select 16858,'VMF' from dual union all
select 16859,'VMF' from dual union all
select 1686,'USAF' from dual union all
select 16860,'MO' from dual union all
select 16861,'MO' from dual union all
select 16862,'RVSN' from dual union all
select 16863,'NPOE' from dual union all
select 16864,'RVSN' from dual union all
select 16865,'MO' from dual union all
select 16866,'MO' from dual union all
select 16867,'MO' from dual union all
select 16868,'VMF' from dual union all
select 16869,'RVSN' from dual union all
select 1687,'USAF' from dual;
insert into SATELLITE_ORG
select 16870,'RVSN' from dual union all
select 16871,'MO' from dual union all
select 16872,'MO' from dual union all
select 16873,'MO' from dual union all
select 16874,'MO' from dual union all
select 16875,'RVSN' from dual union all
select 16876,'MO' from dual union all
select 16877,'USAF' from dual union all
select 16878,'USAF' from dual union all
select 16879,'USAF' from dual union all
select 1688,'USAF' from dual union all
select 16880,'MO' from dual union all
select 16881,'GMS' from dual union all
select 16881,'YUZH' from dual union all
select 16882,'MO' from dual union all
select 16883,'YUZH' from dual union all
select 16884,'MO' from dual union all
select 16885,'MO' from dual union all
select 16886,'RVSN' from dual union all
select 16887,'RVSN' from dual union all
select 16888,'MO' from dual union all
select 16889,'RVSN' from dual union all
select 1689,'USAF' from dual union all
select 16890,'MVD' from dual union all
select 16891,'RVSN' from dual union all
select 16892,'MO' from dual union all
select 16893,'MO' from dual union all
select 16894,'MO' from dual union all
select 16895,'VMF' from dual union all
select 16896,'MO' from dual union all
select 16897,'MO' from dual union all
select 16898,'RVSN' from dual union all
select 16899,'MO' from dual union all
select 169,'RVSN' from dual union all
select 1690,'USAF' from dual union all
select 16900,'MO' from dual union all
select 16901,'MO' from dual union all
select 16902,'MO' from dual union all
select 16903,'MO' from dual union all
select 16904,'MO' from dual union all
select 16905,'MO' from dual union all
select 16906,'MO' from dual union all
select 16907,'MO' from dual union all
select 16908,'NASDA' from dual union all
select 16909,'JARL' from dual union all
select 1691,'USAF' from dual union all
select 16910,'NASDA' from dual union all
select 16911,'MO' from dual union all
select 16912,'MO' from dual union all
select 16913,'MO' from dual union all
select 16914,'MO' from dual union all
select 16915,'MO' from dual union all
select 16916,'MO' from dual union all
select 16917,'VMF' from dual union all
select 16918,'MO' from dual union all
select 16919,'RVSN' from dual union all
select 1692,'USAF' from dual union all
select 16920,'MO' from dual union all
select 16921,'RVSN' from dual union all
select 16922,'PVO' from dual union all
select 16923,'RVSN' from dual union all
select 16924,'RVSN' from dual union all
select 16925,'RVSN' from dual union all
select 16926,'MO' from dual union all
select 16927,'RVSN' from dual union all
select 16928,'PVO' from dual union all
select 16929,'RVSN' from dual union all
select 1693,'USAF' from dual union all
select 16930,'MO' from dual union all
select 16931,'MO' from dual union all
select 16932,'MO' from dual union all
select 16933,'MO' from dual union all
select 16934,'MO' from dual union all
select 16935,'RVSN' from dual union all
select 16936,'RVSN' from dual union all
select 16937,'SDIO' from dual union all
select 16938,'SDIO' from dual union all
select 16939,'RVSN' from dual union all
select 1694,'USAF' from dual union all
select 16940,'SDIO' from dual union all
select 16941,'SDIO' from dual union all
select 16942,'SDIO' from dual union all
select 16943,'SDIO' from dual union all
select 16944,'SDIO' from dual union all
select 16945,'SDIO' from dual union all
select 16946,'SDIO' from dual union all
select 16947,'SDIO' from dual union all
select 16948,'SDIO' from dual union all
select 16949,'SDIO' from dual union all
select 1695,'USAF' from dual union all
select 16950,'SDIO' from dual union all
select 16951,'SDIO' from dual union all
select 16952,'MO' from dual union all
select 16953,'RVSN' from dual union all
select 16954,'PVO' from dual union all
select 16955,'PVO' from dual union all
select 16956,'PVO' from dual union all
select 16957,'PVO' from dual union all
select 16958,'PVO' from dual union all
select 16959,'PVO' from dual;
insert into SATELLITE_ORG
select 1696,'USAF' from dual union all
select 16960,'PVO' from dual union all
select 16961,'MO' from dual union all
select 16962,'MO' from dual union all
select 16963,'MO' from dual union all
select 16964,'RVSN' from dual union all
select 16965,'RVSN' from dual union all
select 16966,'MO' from dual union all
select 16967,'RVSN' from dual union all
select 16968,'RVSN' from dual union all
select 16969,'NOAA' from dual union all
select 1697,'USAF' from dual union all
select 16970,'MO' from dual union all
select 16971,'MO' from dual union all
select 16972,'MO' from dual union all
select 16973,'MO' from dual union all
select 16974,'MO' from dual union all
select 16975,'MO' from dual union all
select 16976,'MO' from dual union all
select 16977,'MO' from dual union all
select 16978,'MO' from dual union all
select 16979,'MO' from dual union all
select 1698,'USAF' from dual union all
select 16980,'PVO' from dual union all
select 16981,'PVO' from dual union all
select 16982,'NASA' from dual union all
select 16983,'NASA' from dual union all
select 16984,'RVSN' from dual union all
select 16985,'RVSN' from dual union all
select 16986,'VS' from dual union all
select 16987,'MO' from dual union all
select 16988,'MO' from dual union all
select 16989,'MO' from dual union all
select 1699,'USAF' from dual union all
select 16990,'MO' from dual union all
select 16991,'MO' from dual union all
select 16992,'MO' from dual union all
select 16993,'PVO' from dual union all
select 16994,'RVSN' from dual union all
select 16995,'RVSN' from dual union all
select 16996,'RVSN' from dual union all
select 16997,'NASA' from dual union all
select 16998,'RVSN' from dual union all
select 16999,'NASA' from dual union all
select 17,'USAF' from dual union all
select 170,'NASA' from dual union all
select 1700,'USAF' from dual union all
select 17000,'NASA' from dual union all
select 17001,'MAI' from dual union all
select 17002,'MAI' from dual union all
select 17003,'TSSKB' from dual union all
select 17004,'RVSN' from dual union all
select 17005,'MAI' from dual union all
select 17006,'PVO' from dual union all
select 17007,'PVO' from dual union all
select 17008,'PVO' from dual union all
select 17009,'PVO' from dual union all
select 1701,'OKB1' from dual union all
select 17010,'VMF' from dual union all
select 17011,'VMF' from dual union all
select 17012,'VMF' from dual union all
select 17013,'VMF' from dual union all
select 17014,'VMF' from dual union all
select 17015,'VMF' from dual union all
select 17016,'VMF' from dual union all
select 17017,'VMF' from dual union all
select 17018,'PVO' from dual union all
select 17019,'SDIO' from dual union all
select 1702,'RVSN' from dual union all
select 17020,'SDIO' from dual union all
select 17021,'SDIO' from dual union all
select 17022,'SDIO' from dual union all
select 17023,'NASA' from dual union all
select 17024,'NASA' from dual union all
select 17025,'MO' from dual union all
select 17026,'MO' from dual union all
select 17027,'NASA' from dual union all
select 17028,'MAI' from dual union all
select 17029,'PVO' from dual union all
select 1703,'RVSN' from dual union all
select 17030,'VMF' from dual union all
select 17031,'PVO' from dual union all
select 17032,'RVSN' from dual union all
select 17033,'RVSN' from dual union all
select 17034,'VMF' from dual union all
select 17035,'VMF' from dual union all
select 17036,'VMF' from dual union all
select 17037,'RVSN' from dual union all
select 17038,'MO' from dual union all
select 17038,'MSVYAZ' from dual union all
select 17039,'RVSN' from dual union all
select 1704,'RVSN' from dual union all
select 17040,'RVSN' from dual union all
select 17041,'RVSN' from dual union all
select 17042,'PVO' from dual union all
select 17043,'MO' from dual union all
select 17044,'MO' from dual union all
select 17045,'RVSN' from dual union all
select 17046,'MO' from dual union all
select 17047,'RVSN' from dual;
insert into SATELLITE_ORG
select 17048,'RVSN' from dual union all
select 17049,'MO' from dual union all
select 1705,'OKB1' from dual union all
select 17050,'PVO' from dual union all
select 17051,'RVSN' from dual union all
select 17052,'RVSN' from dual union all
select 17053,'RVSN' from dual union all
select 17054,'MVD' from dual union all
select 17055,'RVSN' from dual union all
select 17056,'MO' from dual union all
select 17057,'RVSN' from dual union all
select 17058,'MO' from dual union all
select 17059,'PVO' from dual union all
select 1706,'YUZH' from dual union all
select 17060,'PVO' from dual union all
select 17061,'MO' from dual union all
select 17062,'MO' from dual union all
select 17063,'MO' from dual union all
select 17064,'MO' from dual union all
select 17065,'RVSN' from dual union all
select 17066,'VMF' from dual union all
select 17067,'RVSN' from dual union all
select 17068,'MO' from dual union all
select 17069,'RVSN' from dual union all
select 1707,'RVSN' from dual union all
select 17070,'USAF' from dual union all
select 17071,'NASA' from dual union all
select 17072,'MO' from dual union all
select 17073,'MO' from dual union all
select 17074,'MO' from dual union all
select 17075,'MO' from dual union all
select 17076,'MO' from dual union all
select 17077,'MO' from dual union all
select 17078,'MO' from dual union all
select 17079,'RVSN' from dual union all
select 1708,'RVSN' from dual union all
select 17080,'RVSN' from dual union all
select 17081,'RVSN' from dual union all
select 17082,'MO' from dual union all
select 17083,'MSVYAZ' from dual union all
select 17084,'RVSN' from dual union all
select 17085,'RVSN' from dual union all
select 17086,'MO' from dual union all
select 17087,'MO' from dual union all
select 17088,'MO' from dual union all
select 17089,'MO' from dual union all
select 1709,'USAF' from dual union all
select 17090,'PVO' from dual union all
select 17091,'VMF' from dual union all
select 17092,'MONBU' from dual union all
select 17093,'USAF' from dual union all
select 17094,'USAF' from dual union all
select 17095,'USAF' from dual union all
select 17096,'VMF' from dual union all
select 17097,'VMF' from dual union all
select 17098,'VMF' from dual union all
select 17099,'VMF' from dual union all
select 171,'USAF' from dual union all
select 1710,'USAF' from dual union all
select 17100,'USAF' from dual union all
select 17101,'USAF' from dual union all
select 17102,'PVO' from dual union all
select 17103,'PVO' from dual union all
select 17104,'PVO' from dual union all
select 17105,'PVO' from dual union all
select 17106,'PVO' from dual union all
select 17107,'PVO' from dual union all
select 17108,'PVO' from dual union all
select 17109,'NPOE' from dual union all
select 1711,'USAF' from dual union all
select 17110,'AE' from dual union all
select 17111,'AE' from dual union all
select 17112,'AE' from dual union all
select 17113,'AE' from dual union all
select 17114,'AE' from dual union all
select 17115,'AE' from dual union all
select 17116,'AE' from dual union all
select 17117,'AE' from dual union all
select 17118,'AE' from dual union all
select 17119,'AE' from dual union all
select 1712,'USAF' from dual union all
select 17120,'AE' from dual union all
select 17121,'AE' from dual union all
select 17122,'AE' from dual union all
select 17123,'AE' from dual union all
select 17124,'AE' from dual union all
select 17125,'RVSN' from dual union all
select 17126,'AE' from dual union all
select 17127,'AE' from dual union all
select 17128,'AE' from dual union all
select 17129,'AE' from dual union all
select 1713,'USAF' from dual union all
select 17130,'AE' from dual union all
select 17131,'AE' from dual union all
select 17132,'AE' from dual union all
select 17133,'AE' from dual union all
select 17134,'PVO' from dual union all
select 17135,'RVSN' from dual union all
select 17136,'RVSN' from dual union all
select 17137,'PVO' from dual;
insert into SATELLITE_ORG
select 17138,'MO' from dual union all
select 17139,'MO' from dual union all
select 1714,'USAF' from dual union all
select 17140,'MO' from dual union all
select 17141,'MO' from dual union all
select 17142,'MO' from dual union all
select 17143,'MO' from dual union all
select 17144,'MO' from dual union all
select 17145,'MO' from dual union all
select 17146,'RVSN' from dual union all
select 17147,'RVSN' from dual union all
select 17148,'RVSN' from dual union all
select 17149,'RVSN' from dual union all
select 1715,'USAF' from dual union all
select 17150,'AE' from dual union all
select 17151,'AE' from dual union all
select 17152,'AE' from dual union all
select 17153,'AE' from dual union all
select 17154,'AE' from dual union all
select 17155,'AE' from dual union all
select 17156,'AE' from dual union all
select 17157,'AE' from dual union all
select 17158,'PVO' from dual union all
select 17159,'VMF' from dual union all
select 1716,'USAF' from dual union all
select 17160,'RVSN' from dual union all
select 17161,'PVO' from dual union all
select 17162,'MONBU' from dual union all
select 17163,'VMF' from dual union all
select 17164,'USN' from dual union all
select 17165,'USAF' from dual union all
select 17166,'USAF' from dual union all
select 17167,'USAF' from dual union all
select 17168,'RVSN' from dual union all
select 17169,'NASA' from dual union all
select 1717,'USAF' from dual union all
select 17170,'NASA' from dual union all
select 17171,'VMF' from dual union all
select 17172,'USAF' from dual union all
select 17173,'USAF' from dual union all
select 17174,'USAF' from dual union all
select 17175,'MO' from dual union all
select 17176,'USN' from dual union all
select 17177,'VTS' from dual union all
select 17178,'MO' from dual union all
select 17179,'MO' from dual union all
select 1718,'USAF' from dual union all
select 17180,'RVSN' from dual union all
select 17181,'USN' from dual union all
select 17182,'NASA' from dual union all
select 17183,'AE' from dual union all
select 17184,'AE' from dual union all
select 17185,'AE' from dual union all
select 17186,'AE' from dual union all
select 17187,'AE' from dual union all
select 17188,'AE' from dual union all
select 17189,'AE' from dual union all
select 1719,'USAF' from dual union all
select 17190,'AE' from dual union all
select 17191,'VS' from dual union all
select 17192,'MO' from dual union all
select 17193,'AE' from dual union all
select 17194,'AE' from dual union all
select 17195,'AE' from dual union all
select 17196,'AE' from dual union all
select 17197,'AE' from dual union all
select 17198,'AE' from dual union all
select 17199,'AE' from dual union all
select 172,'USAF' from dual union all
select 1720,'USAF' from dual union all
select 17200,'AE' from dual union all
select 17201,'AE' from dual union all
select 17202,'AE' from dual union all
select 17203,'AE' from dual union all
select 17204,'AE' from dual union all
select 17205,'AE' from dual union all
select 17206,'AE' from dual union all
select 17207,'AE' from dual union all
select 17208,'AE' from dual union all
select 17209,'AE' from dual union all
select 1721,'USAF' from dual union all
select 17210,'AE' from dual union all
select 17211,'AE' from dual union all
select 17212,'AE' from dual union all
select 17213,'PVO' from dual union all
select 17214,'RVSN' from dual union all
select 17215,'RVSN' from dual union all
select 17216,'RVSN' from dual union all
select 17217,'MO' from dual union all
select 17218,'RVSN' from dual union all
select 17219,'AE' from dual union all
select 1722,'USAF' from dual union all
select 17220,'AE' from dual union all
select 17221,'AE' from dual union all
select 17222,'AE' from dual union all
select 17223,'AE' from dual union all
select 17224,'AE' from dual union all
select 17225,'AE' from dual union all
select 17226,'AE' from dual union all
select 17227,'AE' from dual;
insert into SATELLITE_ORG
select 17228,'AE' from dual union all
select 17229,'AE' from dual union all
select 1723,'USAF' from dual union all
select 17230,'AE' from dual union all
select 17231,'AE' from dual union all
select 17232,'AE' from dual union all
select 17233,'AE' from dual union all
select 17234,'AE' from dual union all
select 17235,'AE' from dual union all
select 17236,'AE' from dual union all
select 17237,'AE' from dual union all
select 17238,'AE' from dual union all
select 17239,'VMF' from dual union all
select 1724,'USAF' from dual union all
select 17240,'RVSN' from dual union all
select 17241,'IK' from dual union all
select 17242,'MO' from dual union all
select 17243,'MO' from dual union all
select 17244,'MO' from dual union all
select 17245,'MO' from dual union all
select 17246,'MO' from dual union all
select 17247,'USAF' from dual union all
select 17248,'VMF' from dual union all
select 17249,'VMF' from dual union all
select 1725,'USAF' from dual union all
select 17250,'VMF' from dual union all
select 17251,'VMF' from dual union all
select 17252,'MO' from dual union all
select 17253,'AE' from dual union all
select 17254,'AE' from dual union all
select 17255,'AE' from dual union all
select 17256,'MO' from dual union all
select 17257,'USAF' from dual union all
select 17258,'PVO' from dual union all
select 17259,'PVO' from dual union all
select 1726,'NASA' from dual union all
select 17260,'PVO' from dual union all
select 17261,'PVO' from dual union all
select 17262,'MO' from dual union all
select 17263,'RVSN' from dual union all
select 17264,'MO' from dual union all
select 17265,'RVSN' from dual union all
select 17266,'RVSN' from dual union all
select 17267,'RVSN' from dual union all
select 17268,'RVSN' from dual union all
select 17269,'RVSN' from dual union all
select 1727,'USAF' from dual union all
select 17270,'RVSN' from dual union all
select 17271,'RVSN' from dual union all
select 17272,'RVSN' from dual union all
select 17273,'RVSN' from dual union all
select 17274,'RVSN' from dual union all
select 17275,'VMF' from dual union all
select 17276,'RVSN' from dual union all
select 17277,'VMF' from dual union all
select 17278,'VMF' from dual union all
select 17279,'VMF' from dual union all
select 1728,'NRO' from dual union all
select 17280,'VMF' from dual union all
select 17281,'VMF' from dual union all
select 17282,'VMF' from dual union all
select 17283,'VMF' from dual union all
select 17284,'VMF' from dual union all
select 17285,'VMF' from dual union all
select 17286,'VMF' from dual union all
select 17287,'VMF' from dual union all
select 17288,'VMF' from dual union all
select 17289,'VMF' from dual union all
select 1729,'NASA' from dual union all
select 17290,'GMS' from dual union all
select 17290,'MO' from dual union all
select 17291,'MO' from dual union all
select 17292,'MO' from dual union all
select 17293,'RVSN' from dual union all
select 17294,'MO' from dual union all
select 17295,'VS' from dual union all
select 17296,'MO' from dual union all
select 17297,'MO' from dual union all
select 17298,'RVSN' from dual union all
select 17299,'NPOE' from dual union all
select 173,'JPL' from dual union all
select 173,'NASA' from dual union all
select 1730,'OKB1' from dual union all
select 17300,'RVSN' from dual union all
select 17301,'MO' from dual union all
select 17302,'AE' from dual union all
select 17303,'MO' from dual union all
select 17304,'RVSN' from dual union all
select 17305,'AE' from dual union all
select 17306,'AE' from dual union all
select 17307,'AE' from dual union all
select 17308,'AE' from dual union all
select 17309,'AE' from dual union all
select 1731,'RVSN' from dual union all
select 17310,'AE' from dual union all
select 17311,'AE' from dual union all
select 17312,'AE' from dual union all
select 17313,'AE' from dual union all
select 17314,'AE' from dual union all
select 17315,'AE' from dual;
insert into SATELLITE_ORG
select 17316,'AE' from dual union all
select 17317,'AE' from dual union all
select 17318,'AE' from dual union all
select 17319,'AE' from dual union all
select 1732,'RVSN' from dual union all
select 17320,'AE' from dual union all
select 17321,'AE' from dual union all
select 17322,'AE' from dual union all
select 17323,'AE' from dual union all
select 17324,'AE' from dual union all
select 17325,'AE' from dual union all
select 17326,'PVO' from dual union all
select 17327,'RVSN' from dual union all
select 17328,'MO' from dual union all
select 17328,'MSVYAZ' from dual union all
select 17329,'RVSN' from dual union all
select 1733,'OKB1' from dual union all
select 17330,'RVSN' from dual union all
select 17331,'AE' from dual union all
select 17332,'AE' from dual union all
select 17333,'RVSN' from dual union all
select 17334,'PVO' from dual union all
select 17335,'PVO' from dual union all
select 17336,'AE' from dual union all
select 17337,'AE' from dual union all
select 17338,'AE' from dual union all
select 17339,'AE' from dual union all
select 1734,'RVSN' from dual union all
select 17340,'AE' from dual union all
select 17341,'AE' from dual union all
select 17342,'AE' from dual union all
select 17343,'AE' from dual union all
select 17344,'AE' from dual union all
select 17345,'AE' from dual union all
select 17346,'AE' from dual union all
select 17347,'AE' from dual union all
select 17348,'AE' from dual union all
select 17349,'AE' from dual union all
select 1735,'RVSN' from dual union all
select 17350,'AE' from dual union all
select 17351,'AE' from dual union all
select 17352,'AE' from dual union all
select 17353,'AE' from dual union all
select 17354,'AE' from dual union all
select 17355,'AE' from dual union all
select 17356,'PVO' from dual union all
select 17357,'PVO' from dual union all
select 17358,'RVSNR' from dual union all
select 17359,'VMF' from dual union all
select 1736,'RVSN' from dual union all
select 17360,'RVSN' from dual union all
select 17361,'MO' from dual union all
select 17362,'MO' from dual union all
select 17363,'MO' from dual union all
select 17364,'MO' from dual union all
select 17365,'MSVYAZ' from dual union all
select 17366,'RVSN' from dual union all
select 17367,'RVSN' from dual union all
select 17368,'RVSN' from dual union all
select 17369,'VMF' from dual union all
select 1737,'RVSN' from dual union all
select 17370,'MO' from dual union all
select 17371,'AE' from dual union all
select 17372,'AE' from dual union all
select 17373,'AE' from dual union all
select 17374,'AE' from dual union all
select 17375,'AE' from dual union all
select 17376,'AE' from dual union all
select 17377,'AE' from dual union all
select 17378,'AE' from dual union all
select 17379,'AE' from dual union all
select 1738,'USN' from dual union all
select 17380,'AE' from dual union all
select 17381,'AE' from dual union all
select 17382,'AE' from dual union all
select 17383,'AE' from dual union all
select 17384,'AE' from dual union all
select 17385,'AE' from dual union all
select 17386,'AE' from dual union all
select 17387,'AE' from dual union all
select 17388,'AE' from dual union all
select 17389,'AE' from dual union all
select 1739,'NASA' from dual union all
select 17390,'AE' from dual union all
select 17391,'AE' from dual union all
select 17392,'AE' from dual union all
select 17393,'AE' from dual union all
select 17394,'AE' from dual union all
select 17395,'AE' from dual union all
select 17396,'AE' from dual union all
select 17397,'AE' from dual union all
select 17398,'MO' from dual union all
select 17399,'MO' from dual union all
select 174,'NASA' from dual union all
select 1740,'RVSN' from dual union all
select 17400,'MO' from dual union all
select 17401,'MO' from dual union all
select 17402,'MO' from dual union all
select 17403,'MO' from dual union all
select 17404,'MO' from dual;
insert into SATELLITE_ORG
select 17405,'MO' from dual union all
select 17406,'MO' from dual union all
select 17407,'MO' from dual union all
select 17408,'MO' from dual union all
select 17409,'MO' from dual union all
select 1741,'RVSN' from dual union all
select 17410,'MO' from dual union all
select 17411,'MO' from dual union all
select 17412,'MO' from dual union all
select 17413,'MO' from dual union all
select 17414,'MO' from dual union all
select 17415,'MO' from dual union all
select 17416,'MO' from dual union all
select 17417,'MO' from dual union all
select 17418,'MO' from dual union all
select 17419,'MO' from dual union all
select 1742,'OKB1' from dual union all
select 17420,'MO' from dual union all
select 17421,'AE' from dual union all
select 17422,'AE' from dual union all
select 17423,'AE' from dual union all
select 17424,'AE' from dual union all
select 17425,'MAI' from dual union all
select 17426,'AE' from dual union all
select 17427,'AE' from dual union all
select 17428,'AE' from dual union all
select 17429,'AE' from dual union all
select 1743,'RVSN' from dual union all
select 17430,'AE' from dual union all
select 17431,'AE' from dual union all
select 17432,'AE' from dual union all
select 17433,'AE' from dual union all
select 17434,'AE' from dual union all
select 17435,'AE' from dual union all
select 17436,'AE' from dual union all
select 17437,'AE' from dual union all
select 17438,'AE' from dual union all
select 17439,'AE' from dual union all
select 1744,'USAF' from dual union all
select 17440,'AE' from dual union all
select 17441,'AE' from dual union all
select 17442,'AE' from dual union all
select 17443,'AE' from dual union all
select 17444,'AE' from dual union all
select 17445,'AE' from dual union all
select 17446,'AE' from dual union all
select 17447,'AE' from dual union all
select 17448,'AE' from dual union all
select 17449,'AE' from dual union all
select 1745,'USAF' from dual union all
select 17450,'AE' from dual union all
select 17451,'AE' from dual union all
select 17452,'AE' from dual union all
select 17453,'AE' from dual union all
select 17454,'AE' from dual union all
select 17455,'AE' from dual union all
select 17456,'AE' from dual union all
select 17457,'AE' from dual union all
select 17458,'AE' from dual union all
select 17459,'AE' from dual union all
select 1746,'USAF' from dual union all
select 17460,'AE' from dual union all
select 17461,'AE' from dual union all
select 17462,'AE' from dual union all
select 17463,'AE' from dual union all
select 17464,'AE' from dual union all
select 17465,'AE' from dual union all
select 17466,'AE' from dual union all
select 17467,'AE' from dual union all
select 17468,'AE' from dual union all
select 17469,'AE' from dual union all
select 1747,'USAF' from dual union all
select 17470,'AE' from dual union all
select 17471,'AE' from dual union all
select 17472,'AE' from dual union all
select 17473,'AE' from dual union all
select 17474,'AE' from dual union all
select 17475,'AE' from dual union all
select 17476,'AE' from dual union all
select 17477,'AE' from dual union all
select 17478,'AE' from dual union all
select 17479,'AE' from dual union all
select 1748,'USAF' from dual union all
select 17480,'MONBU' from dual union all
select 17481,'MONBU' from dual union all
select 17482,'NPOE' from dual union all
select 17483,'RVSN' from dual union all
select 17484,'MO' from dual union all
select 17485,'RVSN' from dual union all
select 17486,'MO' from dual union all
select 17487,'MO' from dual union all
select 17488,'MO' from dual union all
select 17489,'MO' from dual union all
select 1749,'USAF' from dual union all
select 17490,'MO' from dual union all
select 17491,'MO' from dual union all
select 17492,'MO' from dual union all
select 17493,'MO' from dual union all
select 17494,'MO' from dual union all
select 17495,'MO' from dual;
insert into SATELLITE_ORG
select 17496,'MO' from dual union all
select 17497,'MO' from dual union all
select 17498,'MO' from dual union all
select 17499,'MO' from dual union all
select 175,'USAF' from dual union all
select 1750,'USAF' from dual union all
select 17500,'MO' from dual union all
select 17501,'MO' from dual union all
select 17502,'MO' from dual union all
select 17503,'MO' from dual union all
select 17504,'MO' from dual union all
select 17505,'MO' from dual union all
select 17506,'NRO' from dual union all
select 17507,'USAF' from dual union all
select 17508,'MO' from dual union all
select 17509,'MO' from dual union all
select 1751,'USAF' from dual union all
select 17510,'MO' from dual union all
select 17511,'MO' from dual union all
select 17512,'MO' from dual union all
select 17513,'MO' from dual union all
select 17514,'MO' from dual union all
select 17515,'MO' from dual union all
select 17516,'MO' from dual union all
select 17517,'MO' from dual union all
select 17518,'MO' from dual union all
select 17519,'MO' from dual union all
select 1752,'USAF' from dual union all
select 17520,'MO' from dual union all
select 17521,'MO' from dual union all
select 17522,'MO' from dual union all
select 17523,'YUZH' from dual union all
select 17524,'MO' from dual union all
select 17525,'VMF' from dual union all
select 17526,'RVSN' from dual union all
select 17527,'NASDA' from dual union all
select 17528,'NASDA' from dual union all
select 17529,'MO' from dual union all
select 1753,'USAF' from dual union all
select 17530,'MO' from dual union all
select 17531,'MO' from dual union all
select 17532,'MO' from dual union all
select 17533,'MO' from dual union all
select 17534,'RVSN' from dual union all
select 17535,'VTS' from dual union all
select 17536,'MO' from dual union all
select 17537,'PVO' from dual union all
select 17538,'PVO' from dual union all
select 17539,'AE' from dual union all
select 1754,'USAF' from dual union all
select 17540,'AE' from dual union all
select 17541,'AE' from dual union all
select 17542,'AE' from dual union all
select 17543,'AE' from dual union all
select 17544,'AE' from dual union all
select 17545,'AE' from dual union all
select 17546,'AE' from dual union all
select 17547,'AE' from dual union all
select 17548,'AE' from dual union all
select 17549,'AE' from dual union all
select 1755,'USAF' from dual union all
select 17550,'AE' from dual union all
select 17551,'AE' from dual union all
select 17552,'AE' from dual union all
select 17553,'AE' from dual union all
select 17554,'AE' from dual union all
select 17555,'AE' from dual union all
select 17556,'AE' from dual union all
select 17557,'AE' from dual union all
select 17558,'AE' from dual union all
select 17559,'MO' from dual union all
select 1756,'USAF' from dual union all
select 17560,'RVSN' from dual union all
select 17561,'NOAA' from dual union all
select 17562,'NASA' from dual union all
select 17563,'NASA' from dual union all
select 17564,'NPOE' from dual union all
select 17565,'RVSN' from dual union all
select 17566,'VS' from dual union all
select 17567,'MO' from dual union all
select 17568,'MO' from dual union all
select 17569,'MO' from dual union all
select 1757,'USAF' from dual union all
select 17570,'MO' from dual union all
select 17571,'MO' from dual union all
select 17572,'MO' from dual union all
select 17573,'MO' from dual union all
select 17574,'AE' from dual union all
select 17575,'AE' from dual union all
select 17576,'AE' from dual union all
select 17577,'MO' from dual union all
select 17578,'RVSN' from dual union all
select 17579,'MO' from dual union all
select 1758,'USAF' from dual union all
select 17580,'MO' from dual union all
select 17581,'MO' from dual union all
select 17582,'MO' from dual union all
select 17583,'MO' from dual union all
select 17584,'MO' from dual union all
select 17585,'MO' from dual;
insert into SATELLITE_ORG
select 17586,'MO' from dual union all
select 17587,'MO' from dual union all
select 17588,'MO' from dual union all
select 17589,'VS' from dual union all
select 1759,'USAF' from dual union all
select 17590,'MO' from dual union all
select 17591,'AE' from dual union all
select 17592,'AE' from dual union all
select 17593,'AE' from dual union all
select 17594,'AE' from dual union all
select 17595,'AE' from dual union all
select 17596,'AE' from dual union all
select 17597,'AE' from dual union all
select 17598,'AE' from dual union all
select 17599,'AE' from dual union all
select 176,'USAF' from dual union all
select 1760,'USAF' from dual union all
select 17600,'AE' from dual union all
select 17601,'AE' from dual union all
select 17602,'AE' from dual union all
select 17603,'AE' from dual union all
select 17604,'AE' from dual union all
select 17605,'AE' from dual union all
select 17606,'AE' from dual union all
select 17607,'AE' from dual union all
select 17608,'AE' from dual union all
select 17609,'AE' from dual union all
select 1761,'USAF' from dual union all
select 17610,'AE' from dual union all
select 17611,'MO' from dual union all
select 17612,'RVSN' from dual union all
select 17613,'RVSN' from dual union all
select 17614,'MO' from dual union all
select 17615,'MO' from dual union all
select 17616,'MO' from dual union all
select 17617,'MO' from dual union all
select 17618,'MO' from dual union all
select 17619,'MO' from dual union all
select 1762,'USAF' from dual union all
select 17620,'MO' from dual union all
select 17621,'MO' from dual union all
select 17622,'MO' from dual union all
select 17623,'MO' from dual union all
select 17624,'MO' from dual union all
select 17625,'MO' from dual union all
select 17626,'MO' from dual union all
select 17627,'MO' from dual union all
select 17628,'MO' from dual union all
select 17629,'MO' from dual union all
select 1763,'USAF' from dual union all
select 17630,'MO' from dual union all
select 17631,'MO' from dual union all
select 17632,'MO' from dual union all
select 17633,'MO' from dual union all
select 17634,'MO' from dual union all
select 17635,'MO' from dual union all
select 17636,'MO' from dual union all
select 17637,'MO' from dual union all
select 17638,'MO' from dual union all
select 17639,'MO' from dual union all
select 1764,'USAF' from dual union all
select 17640,'MO' from dual union all
select 17641,'MO' from dual union all
select 17642,'MO' from dual union all
select 17643,'VMF' from dual union all
select 17644,'VMF' from dual union all
select 17645,'VMF' from dual union all
select 17646,'VMF' from dual union all
select 17647,'VMF' from dual union all
select 17648,'USAF' from dual union all
select 17649,'USAF' from dual union all
select 1765,'USAF' from dual union all
select 17650,'USAF' from dual union all
select 17651,'NASA' from dual union all
select 17652,'NASA' from dual union all
select 17653,'USAF' from dual union all
select 17654,'VMF' from dual union all
select 17655,'VMF' from dual union all
select 17656,'VMF' from dual union all
select 17657,'VMF' from dual union all
select 17658,'USAF' from dual union all
select 17659,'USAF' from dual union all
select 1766,'USAF' from dual union all
select 17660,'USAF' from dual union all
select 17661,'USAF' from dual union all
select 17662,'PVO' from dual union all
select 17663,'PVO' from dual union all
select 17664,'PVO' from dual union all
select 17665,'NASA' from dual union all
select 17666,'USAF' from dual union all
select 17667,'PVO' from dual union all
select 17668,'PVO' from dual union all
select 17669,'PVO' from dual union all
select 1767,'USAF' from dual union all
select 17670,'PVO' from dual union all
select 17671,'VMF' from dual union all
select 17672,'AE' from dual union all
select 17673,'AE' from dual union all
select 17674,'AE' from dual union all
select 17675,'AE' from dual;
insert into SATELLITE_ORG
select 17676,'AE' from dual union all
select 17677,'AE' from dual union all
select 17678,'AE' from dual union all
select 17679,'AE' from dual union all
select 1768,'USAF' from dual union all
select 17680,'AE' from dual union all
select 17681,'AE' from dual union all
select 17682,'AE' from dual union all
select 17683,'AE' from dual union all
select 17684,'AE' from dual union all
select 17685,'AE' from dual union all
select 17686,'AE' from dual union all
select 17687,'AE' from dual union all
select 17688,'AE' from dual union all
select 17689,'AE' from dual union all
select 1769,'USAF' from dual union all
select 17690,'AE' from dual union all
select 17691,'AE' from dual union all
select 17692,'AE' from dual union all
select 17693,'AE' from dual union all
select 17694,'AE' from dual union all
select 17695,'AE' from dual union all
select 17696,'AE' from dual union all
select 17697,'AE' from dual union all
select 17698,'AE' from dual union all
select 17699,'AE' from dual union all
select 177,'USAF' from dual union all
select 1770,'USAF' from dual union all
select 17700,'AE' from dual union all
select 17701,'AE' from dual union all
select 17702,'AE' from dual union all
select 17703,'AE' from dual union all
select 17704,'AE' from dual union all
select 17705,'RVSN' from dual union all
select 17706,'PER' from dual union all
select 17707,'NASA' from dual union all
select 17708,'NASA' from dual union all
select 17709,'RVSN' from dual union all
select 1771,'OKB1' from dual union all
select 17710,'RVSN' from dual union all
select 17711,'USAF' from dual union all
select 17712,'USAF' from dual union all
select 17713,'USAF' from dual union all
select 17714,'USAF' from dual union all
select 17715,'NASA' from dual union all
select 17716,'NASA' from dual union all
select 17717,'NASA' from dual union all
select 17718,'NASA' from dual union all
select 17719,'NASA' from dual union all
select 1772,'OKB1' from dual union all
select 17720,'NASA' from dual union all
select 17721,'NASA' from dual union all
select 17722,'USAF' from dual union all
select 17723,'USN' from dual union all
select 17724,'USAF' from dual union all
select 17725,'PVO' from dual union all
select 17726,'PVO' from dual union all
select 17727,'NASA' from dual union all
select 17728,'NASA' from dual union all
select 17729,'NASA' from dual union all
select 1773,'OKB1' from dual union all
select 17730,'NASA' from dual union all
select 17731,'PVO' from dual union all
select 17732,'PVO' from dual union all
select 17733,'VMF' from dual union all
select 17734,'VMF' from dual union all
select 17735,'VMF' from dual union all
select 17736,'VMF' from dual union all
select 17737,'VMF' from dual union all
select 17738,'VMF' from dual union all
select 17739,'VMF' from dual union all
select 1774,'OKB1' from dual union all
select 17740,'VMF' from dual union all
select 17741,'VMF' from dual union all
select 17742,'VMF' from dual union all
select 17743,'VMF' from dual union all
select 17744,'VMF' from dual union all
select 17745,'VMF' from dual union all
select 17746,'VMF' from dual union all
select 17747,'VMF' from dual union all
select 17748,'VMF' from dual union all
select 17749,'VMF' from dual union all
select 1775,'OKB1' from dual union all
select 17750,'VMF' from dual union all
select 17751,'VMF' from dual union all
select 17752,'VMF' from dual union all
select 17753,'VMF' from dual union all
select 17754,'MO' from dual union all
select 17755,'MO' from dual union all
select 17756,'MO' from dual union all
select 17757,'MO' from dual union all
select 17758,'MO' from dual union all
select 17759,'MO' from dual union all
select 1776,'OKB1' from dual union all
select 17760,'MO' from dual union all
select 17761,'MO' from dual union all
select 17762,'MO' from dual union all
select 17763,'NASA' from dual union all
select 17764,'NASA' from dual union all
select 17765,'NASA' from dual;
insert into SATELLITE_ORG
select 17766,'NASA' from dual union all
select 17767,'NASA' from dual union all
select 17768,'NASA' from dual union all
select 17769,'NASA' from dual union all
select 1777,'YUZH' from dual union all
select 17770,'USAF' from dual union all
select 17771,'USAF' from dual union all
select 17772,'USAF' from dual union all
select 17773,'USAF' from dual union all
select 17774,'USAF' from dual union all
select 17775,'USAF' from dual union all
select 17776,'USAF' from dual union all
select 17777,'USAF' from dual union all
select 17778,'USAF' from dual union all
select 17779,'USAF' from dual union all
select 1778,'CNES' from dual union all
select 17780,'USAF' from dual union all
select 17781,'USAF' from dual union all
select 17782,'USAF' from dual union all
select 17783,'USAF' from dual union all
select 17784,'USAF' from dual union all
select 17785,'MO' from dual union all
select 17786,'MO' from dual union all
select 17787,'MO' from dual union all
select 17788,'MO' from dual union all
select 17789,'MO' from dual union all
select 1779,'RVSN' from dual union all
select 17790,'MO' from dual union all
select 17791,'MO' from dual union all
select 17792,'MO' from dual union all
select 17793,'MO' from dual union all
select 17794,'PVO' from dual union all
select 17795,'PVO' from dual union all
select 17796,'PVO' from dual union all
select 17797,'PVO' from dual union all
select 17798,'PVO' from dual union all
select 17799,'PVO' from dual union all
select 178,'USAF' from dual union all
select 1780,'MO' from dual union all
select 17800,'PVO' from dual union all
select 17801,'PVO' from dual union all
select 17802,'PVO' from dual union all
select 17803,'PVO' from dual union all
select 17804,'PVO' from dual union all
select 17805,'PVO' from dual union all
select 17806,'PVO' from dual union all
select 17807,'PVO' from dual union all
select 17808,'AE' from dual union all
select 17809,'AE' from dual union all
select 1781,'RVSN' from dual union all
select 17810,'AE' from dual union all
select 17811,'AE' from dual union all
select 17812,'AE' from dual union all
select 17813,'AE' from dual union all
select 17814,'AE' from dual union all
select 17815,'AE' from dual union all
select 17816,'AE' from dual union all
select 17817,'AE' from dual union all
select 17818,'PVO' from dual union all
select 17819,'PVO' from dual union all
select 1782,'MO' from dual union all
select 17820,'NPOE' from dual union all
select 17821,'NPOE' from dual union all
select 17822,'USAF' from dual union all
select 17823,'USAF' from dual union all
select 17824,'USAF' from dual union all
select 17825,'USAF' from dual union all
select 17826,'USAF' from dual union all
select 17827,'USAF' from dual union all
select 17828,'USAF' from dual union all
select 17829,'USAF' from dual union all
select 1783,'MO' from dual union all
select 17830,'USAF' from dual union all
select 17831,'USAF' from dual union all
select 17832,'USAF' from dual union all
select 17833,'VMF' from dual union all
select 17834,'VMF' from dual union all
select 17835,'VMF' from dual union all
select 17836,'VMF' from dual union all
select 17837,'VMF' from dual union all
select 17838,'PVO' from dual union all
select 17839,'WUTC' from dual union all
select 1784,'RVSN' from dual union all
select 17840,'VMF' from dual union all
select 17841,'VMF' from dual union all
select 17842,'MO' from dual union all
select 17843,'MO' from dual union all
select 17844,'RVSN' from dual union all
select 17845,'NPOE' from dual union all
select 17846,'RVSN' from dual union all
select 17847,'VMF' from dual union all
select 17848,'MO' from dual union all
select 17849,'MO' from dual union all
select 1785,'RVSN' from dual union all
select 17850,'RVSN' from dual union all
select 17851,'NPOE' from dual union all
select 17852,'MO' from dual union all
select 17853,'MO' from dual union all
select 17854,'MO' from dual union all
select 17855,'MO' from dual;
insert into SATELLITE_ORG
select 17856,'MO' from dual union all
select 17857,'MO' from dual union all
select 17858,'MO' from dual union all
select 17859,'MO' from dual union all
select 1786,'RVSN' from dual union all
select 17860,'MO' from dual union all
select 17861,'MO' from dual union all
select 17862,'MO' from dual union all
select 17863,'MO' from dual union all
select 17864,'MO' from dual union all
select 17865,'MO' from dual union all
select 17866,'MO' from dual union all
select 17867,'MO' from dual union all
select 17868,'MO' from dual union all
select 17869,'MO' from dual union all
select 1787,'RVSN' from dual union all
select 17870,'MO' from dual union all
select 17871,'MO' from dual union all
select 17872,'RVSN' from dual union all
select 17873,'RVSN' from dual union all
select 17874,'RVSN' from dual union all
select 17875,'RVSN' from dual union all
select 17876,'MO' from dual union all
select 17877,'RVSN' from dual union all
select 17878,'NPOE' from dual union all
select 17879,'RVSN' from dual union all
select 1788,'RVSN' from dual union all
select 17880,'MO' from dual union all
select 17881,'RVSN' from dual union all
select 17882,'MO' from dual union all
select 17883,'MO' from dual union all
select 17884,'MO' from dual union all
select 17885,'MO' from dual union all
select 17886,'MO' from dual union all
select 17887,'MO' from dual union all
select 17888,'MO' from dual union all
select 17889,'MO' from dual union all
select 1789,'RVSN' from dual union all
select 17890,'MO' from dual union all
select 17891,'MO' from dual union all
select 17892,'MO' from dual union all
select 17893,'MO' from dual union all
select 17894,'MO' from dual union all
select 17895,'MO' from dual union all
select 17896,'MO' from dual union all
select 17897,'MO' from dual union all
select 17898,'MO' from dual union all
select 17899,'MO' from dual union all
select 179,'USAF' from dual union all
select 1790,'RVSN' from dual union all
select 17900,'MO' from dual union all
select 17901,'MO' from dual union all
select 17902,'MO' from dual union all
select 17903,'MO' from dual union all
select 17904,'MO' from dual union all
select 17905,'RVSN' from dual union all
select 17906,'RVSN' from dual union all
select 17907,'TSSKB' from dual union all
select 17908,'RVSN' from dual union all
select 17909,'RVSN' from dual union all
select 1791,'RVSN' from dual union all
select 17910,'RVSN' from dual union all
select 17911,'VS' from dual union all
select 17912,'MO' from dual union all
select 17913,'RVSN' from dual union all
select 17914,'MO' from dual union all
select 17915,'MO' from dual union all
select 17916,'MO' from dual union all
select 17917,'MO' from dual union all
select 17918,'MO' from dual union all
select 17919,'MO' from dual union all
select 1792,'RVSN' from dual union all
select 17920,'MO' from dual union all
select 17921,'MO' from dual union all
select 17922,'MO' from dual union all
select 17923,'MO' from dual union all
select 17924,'MO' from dual union all
select 17925,'MO' from dual union all
select 17926,'MO' from dual union all
select 17927,'MO' from dual union all
select 17928,'MO' from dual union all
select 17929,'MO' from dual union all
select 1793,'RVSN' from dual union all
select 17930,'MO' from dual union all
select 17931,'MO' from dual union all
select 17932,'MO' from dual union all
select 17933,'MO' from dual union all
select 17934,'MO' from dual union all
select 17935,'MO' from dual union all
select 17936,'MO' from dual union all
select 17937,'MO' from dual union all
select 17938,'MO' from dual union all
select 17939,'RVSN' from dual union all
select 1794,'RVSN' from dual union all
select 17940,'MO' from dual union all
select 17941,'RVSN' from dual union all
select 17942,'AE' from dual union all
select 17943,'AE' from dual union all
select 17944,'AE' from dual union all
select 17945,'AE' from dual;
insert into SATELLITE_ORG
select 17946,'AE' from dual union all
select 17947,'AE' from dual union all
select 17948,'AE' from dual union all
select 17949,'AE' from dual union all
select 1795,'RVSN' from dual union all
select 17950,'AE' from dual union all
select 17951,'AE' from dual union all
select 17952,'AE' from dual union all
select 17953,'AE' from dual union all
select 17954,'AE' from dual union all
select 17955,'AE' from dual union all
select 17956,'AE' from dual union all
select 17957,'AE' from dual union all
select 17958,'AE' from dual union all
select 17959,'AE' from dual union all
select 1796,'RVSN' from dual union all
select 17960,'AE' from dual union all
select 17961,'AE' from dual union all
select 17962,'RVSN' from dual union all
select 17963,'MO' from dual union all
select 17964,'MO' from dual union all
select 17965,'MO' from dual union all
select 17966,'MO' from dual union all
select 17967,'MO' from dual union all
select 17968,'MO' from dual union all
select 17969,'MSVYAZ' from dual union all
select 1797,'RVSN' from dual union all
select 17970,'RVSN' from dual union all
select 17971,'RVSN' from dual union all
select 17972,'RVSN' from dual union all
select 17973,'VS' from dual union all
select 17974,'MO' from dual union all
select 17975,'MO' from dual union all
select 17976,'RVSN' from dual union all
select 17977,'AE' from dual union all
select 17978,'AE' from dual union all
select 17979,'AE' from dual union all
select 1798,'RVSN' from dual union all
select 17980,'AE' from dual union all
select 17981,'AE' from dual union all
select 17982,'AE' from dual union all
select 17983,'AE' from dual union all
select 17984,'AE' from dual union all
select 17985,'AE' from dual union all
select 17986,'AE' from dual union all
select 17987,'AE' from dual union all
select 17988,'AE' from dual union all
select 17989,'AE' from dual union all
select 1799,'RVSN' from dual union all
select 17990,'AE' from dual union all
select 17991,'AE' from dual union all
select 17992,'AE' from dual union all
select 17993,'AE' from dual union all
select 17994,'AE' from dual union all
select 17995,'AE' from dual union all
select 17996,'AE' from dual union all
select 17997,'NRO' from dual union all
select 17997,'USN' from dual union all
select 17998,'USN' from dual union all
select 17999,'NPOE' from dual union all
select 18,'CIA' from dual union all
select 18,'DOD' from dual union all
select 180,'NASA' from dual union all
select 1800,'RVSN' from dual union all
select 18000,'RVSN' from dual union all
select 18001,'MO' from dual union all
select 18002,'MO' from dual union all
select 18003,'MO' from dual union all
select 18004,'MVD' from dual union all
select 18005,'RVSN' from dual union all
select 18006,'MO' from dual union all
select 18007,'USN' from dual union all
select 18008,'USN' from dual union all
select 18009,'NRO' from dual union all
select 18009,'USN' from dual union all
select 1801,'RVSN' from dual union all
select 18010,'NRO' from dual union all
select 18010,'USN' from dual union all
select 18011,'MO' from dual union all
select 18012,'RVSN' from dual union all
select 18013,'MO' from dual union all
select 18014,'MO' from dual union all
select 18015,'MO' from dual union all
select 18016,'MO' from dual union all
select 18017,'MO' from dual union all
select 18018,'RVSN' from dual union all
select 18019,'MO' from dual union all
select 1802,'RVSN' from dual union all
select 18020,'MO' from dual union all
select 18021,'MO' from dual union all
select 18022,'MO' from dual union all
select 18023,'MO' from dual union all
select 18024,'USN' from dual union all
select 18025,'NRO' from dual union all
select 18025,'USN' from dual union all
select 18026,'PVO' from dual union all
select 18027,'PVO' from dual union all
select 18028,'MO' from dual union all
select 18029,'MO' from dual union all
select 1803,'MO' from dual;
insert into SATELLITE_ORG
select 18030,'MO' from dual union all
select 18031,'MO' from dual union all
select 18032,'MO' from dual union all
select 18033,'MO' from dual union all
select 18034,'MO' from dual union all
select 18035,'MO' from dual union all
select 18036,'MO' from dual union all
select 18037,'MO' from dual union all
select 18038,'MO' from dual union all
select 18039,'MO' from dual union all
select 1804,'DRTE' from dual union all
select 18040,'MO' from dual union all
select 18041,'MO' from dual union all
select 18042,'MO' from dual union all
select 18043,'MO' from dual union all
select 18044,'MO' from dual union all
select 18045,'MO' from dual union all
select 18046,'MO' from dual union all
select 18047,'MO' from dual union all
select 18048,'MO' from dual union all
select 18049,'MO' from dual union all
select 1805,'CNES' from dual union all
select 18050,'MO' from dual union all
select 18051,'MO' from dual union all
select 18052,'MO' from dual union all
select 18053,'MO' from dual union all
select 18054,'MO' from dual union all
select 18055,'MO' from dual union all
select 18056,'MO' from dual union all
select 18057,'MO' from dual union all
select 18058,'MO' from dual union all
select 18059,'MO' from dual union all
select 1806,'NASA' from dual union all
select 18060,'MO' from dual union all
select 18061,'MO' from dual union all
select 18062,'MO' from dual union all
select 18063,'MO' from dual union all
select 18064,'MO' from dual union all
select 18065,'MO' from dual union all
select 18066,'MO' from dual union all
select 18067,'MO' from dual union all
select 18068,'MO' from dual union all
select 18069,'MO' from dual union all
select 1807,'USAF' from dual union all
select 18070,'MO' from dual union all
select 18071,'MO' from dual union all
select 18072,'MO' from dual union all
select 18073,'MO' from dual union all
select 18074,'MO' from dual union all
select 18075,'MO' from dual union all
select 18076,'MO' from dual union all
select 18077,'MO' from dual union all
select 18078,'MO' from dual union all
select 18079,'MO' from dual union all
select 1808,'NASA' from dual union all
select 18080,'MO' from dual union all
select 18081,'MO' from dual union all
select 18082,'MO' from dual union all
select 18083,'PVO' from dual union all
select 18084,'RVSN' from dual union all
select 18085,'RVSN' from dual union all
select 18086,'RVSN' from dual union all
select 18087,'MO' from dual union all
select 18088,'MO' from dual union all
select 18089,'MO' from dual union all
select 1809,'RVSN' from dual union all
select 18090,'MO' from dual union all
select 18091,'MO' from dual union all
select 18092,'MO' from dual union all
select 18093,'MO' from dual union all
select 18094,'MO' from dual union all
select 18095,'MO' from dual union all
select 18096,'RVSN' from dual union all
select 18097,'MO' from dual union all
select 18098,'MO' from dual union all
select 18099,'MO' from dual union all
select 181,'CIA' from dual union all
select 1810,'OKB1' from dual union all
select 18100,'MO' from dual union all
select 18101,'MO' from dual union all
select 18102,'MO' from dual union all
select 18103,'PVO' from dual union all
select 18104,'RVSN' from dual union all
select 18105,'RVSN' from dual union all
select 18106,'RVSN' from dual union all
select 18107,'MO' from dual union all
select 18108,'MO' from dual union all
select 18109,'MO' from dual union all
select 1811,'RVSN' from dual union all
select 18110,'MO' from dual union all
select 18111,'RVSN' from dual union all
select 18112,'RVSN' from dual union all
select 18113,'MO' from dual union all
select 18114,'MO' from dual union all
select 18115,'MO' from dual union all
select 18116,'MO' from dual union all
select 18117,'MO' from dual union all
select 18118,'MO' from dual union all
select 18119,'MO' from dual union all
select 1812,'NASA' from dual;
insert into SATELLITE_ORG
select 18120,'MO' from dual union all
select 18121,'RVSN' from dual union all
select 18122,'VMF' from dual union all
select 18123,'USAF' from dual union all
select 18124,'MO' from dual union all
select 18125,'MO' from dual union all
select 18126,'MO' from dual union all
select 18127,'USAF' from dual union all
select 18128,'USAF' from dual union all
select 18129,'VMF' from dual union all
select 1813,'USAF' from dual union all
select 18130,'RVSN' from dual union all
select 18131,'MO' from dual union all
select 18132,'AE' from dual union all
select 18133,'AE' from dual union all
select 18134,'AE' from dual union all
select 18135,'AE' from dual union all
select 18136,'AE' from dual union all
select 18137,'AE' from dual union all
select 18138,'AE' from dual union all
select 18139,'AE' from dual union all
select 1814,'CNES' from dual union all
select 18140,'AE' from dual union all
select 18141,'AE' from dual union all
select 18142,'AE' from dual union all
select 18143,'AE' from dual union all
select 18144,'AE' from dual union all
select 18145,'AE' from dual union all
select 18146,'AE' from dual union all
select 18147,'AE' from dual union all
select 18148,'AE' from dual union all
select 18149,'AE' from dual union all
select 1815,'NASA' from dual union all
select 18150,'AE' from dual union all
select 18151,'AE' from dual union all
select 18152,'VS' from dual union all
select 18153,'MO' from dual union all
select 18154,'USAF' from dual union all
select 18155,'MO' from dual union all
select 18156,'RVSN' from dual union all
select 18157,'MO' from dual union all
select 18158,'MO' from dual union all
select 18159,'USAF' from dual union all
select 1816,'CIA' from dual union all
select 1816,'NRO' from dual union all
select 18160,'VMF' from dual union all
select 18161,'RVSN' from dual union all
select 18162,'TSSKB' from dual union all
select 18163,'RVSN' from dual union all
select 18164,'AE' from dual union all
select 18165,'AE' from dual union all
select 18166,'AE' from dual union all
select 18167,'AE' from dual union all
select 18168,'AE' from dual union all
select 18169,'AE' from dual union all
select 1817,'MO' from dual union all
select 18170,'AE' from dual union all
select 18171,'AE' from dual union all
select 18172,'AE' from dual union all
select 18173,'AE' from dual union all
select 18174,'AE' from dual union all
select 18175,'AE' from dual union all
select 18176,'AE' from dual union all
select 18177,'AE' from dual union all
select 18178,'AE' from dual union all
select 18179,'AE' from dual union all
select 1818,'RVSN' from dual union all
select 18180,'AE' from dual union all
select 18181,'AE' from dual union all
select 18182,'AE' from dual union all
select 18183,'AE' from dual union all
select 18184,'MO' from dual union all
select 18185,'RVSN' from dual union all
select 18186,'MO' from dual union all
select 18187,'VMF' from dual union all
select 18188,'MO' from dual union all
select 18189,'MO' from dual union all
select 1819,'USAF' from dual union all
select 18190,'MO' from dual union all
select 18191,'MO' from dual union all
select 18192,'PVO' from dual union all
select 18193,'RVSN' from dual union all
select 18194,'AE' from dual union all
select 18195,'AE' from dual union all
select 18196,'AE' from dual union all
select 18197,'AE' from dual union all
select 18198,'AE' from dual union all
select 18199,'AE' from dual union all
select 182,'CIA' from dual union all
select 182,'NRO' from dual union all
select 1820,'USAF' from dual union all
select 18200,'AE' from dual union all
select 18201,'AE' from dual union all
select 18202,'AE' from dual union all
select 18203,'AE' from dual union all
select 18204,'AE' from dual union all
select 18205,'AE' from dual union all
select 18206,'AE' from dual union all
select 18207,'AE' from dual union all
select 18208,'AE' from dual;
insert into SATELLITE_ORG
select 18209,'AE' from dual union all
select 1821,'USAF' from dual union all
select 18210,'AE' from dual union all
select 18211,'AE' from dual union all
select 18212,'AE' from dual union all
select 18213,'AE' from dual union all
select 18214,'GMS' from dual union all
select 18214,'YUZH' from dual union all
select 18215,'MO' from dual union all
select 18216,'MO' from dual union all
select 18217,'MO' from dual union all
select 18218,'MO' from dual union all
select 18219,'MO' from dual union all
select 1822,'USAF' from dual union all
select 18220,'MO' from dual union all
select 18221,'MO' from dual union all
select 18222,'NPOE' from dual union all
select 18223,'RVSN' from dual union all
select 18224,'MO' from dual union all
select 18225,'KBSAL' from dual union all
select 18226,'RVSN' from dual union all
select 18227,'MO' from dual union all
select 18228,'AE' from dual union all
select 18229,'MONBU' from dual union all
select 1823,'USAF' from dual union all
select 18230,'MO' from dual union all
select 18231,'AE' from dual union all
select 18232,'NASA' from dual union all
select 18233,'VMF' from dual union all
select 18234,'AE' from dual union all
select 18235,'MO' from dual union all
select 18236,'MO' from dual union all
select 18237,'VMF' from dual union all
select 18238,'MONBU' from dual union all
select 18239,'VMF' from dual union all
select 1824,'USAF' from dual union all
select 18240,'VMF' from dual union all
select 18241,'VMF' from dual union all
select 18242,'NPOE' from dual union all
select 18243,'VMF' from dual union all
select 18244,'VMF' from dual union all
select 18245,'VMF' from dual union all
select 18246,'VMF' from dual union all
select 18247,'VMF' from dual union all
select 18248,'VMF' from dual union all
select 18249,'VMF' from dual union all
select 1825,'USAF' from dual union all
select 18250,'USAF' from dual union all
select 18251,'USAF' from dual union all
select 18252,'USAF' from dual union all
select 18253,'USAF' from dual union all
select 18254,'USAF' from dual union all
select 18255,'USAF' from dual union all
select 18256,'USAF' from dual union all
select 18257,'MO' from dual union all
select 18258,'USAF' from dual union all
select 18259,'YUZH' from dual union all
select 1826,'USAF' from dual union all
select 18260,'MO' from dual union all
select 18261,'AE' from dual union all
select 18262,'AE' from dual union all
select 18263,'AE' from dual union all
select 18264,'AE' from dual union all
select 18265,'AE' from dual union all
select 18266,'AE' from dual union all
select 18267,'AE' from dual union all
select 18268,'USAF' from dual union all
select 18269,'USAF' from dual union all
select 1827,'USAF' from dual union all
select 18270,'USAF' from dual union all
select 18271,'USAF' from dual union all
select 18272,'PVO' from dual union all
select 18273,'MO' from dual union all
select 18274,'MO' from dual union all
select 18275,'MO' from dual union all
select 18276,'MO' from dual union all
select 18277,'RVSN' from dual union all
select 18278,'PVO' from dual union all
select 18279,'PVO' from dual union all
select 1828,'USAF' from dual union all
select 18280,'VMF' from dual union all
select 18281,'VMF' from dual union all
select 18282,'RVSN' from dual union all
select 18283,'NPOE' from dual union all
select 18284,'RVSN' from dual union all
select 18285,'MO' from dual union all
select 18286,'MO' from dual union all
select 18287,'MO' from dual union all
select 18288,'MO' from dual union all
select 18289,'AE' from dual union all
select 1829,'USAF' from dual union all
select 18290,'AE' from dual union all
select 18291,'AE' from dual union all
select 18292,'AE' from dual union all
select 18293,'AE' from dual union all
select 18294,'AE' from dual union all
select 18295,'AE' from dual union all
select 18296,'AE' from dual union all
select 18297,'AE' from dual union all
select 18298,'AE' from dual;
insert into SATELLITE_ORG
select 18299,'AE' from dual union all
select 183,'NASA' from dual union all
select 1830,'USAF' from dual union all
select 18300,'AE' from dual union all
select 18301,'NASA' from dual union all
select 18302,'NASA' from dual union all
select 18303,'USAF' from dual union all
select 18304,'USAF' from dual union all
select 18305,'MO' from dual union all
select 18306,'MAI' from dual union all
select 18307,'MAI' from dual union all
select 18308,'MO' from dual union all
select 18309,'MO' from dual union all
select 1831,'USAF' from dual union all
select 18310,'MO' from dual union all
select 18311,'MO' from dual union all
select 18312,'GMS' from dual union all
select 18312,'MO' from dual union all
select 18313,'MO' from dual union all
select 18314,'MO' from dual union all
select 18315,'RVSN' from dual union all
select 18316,'NASDA' from dual union all
select 18317,'NASDA' from dual union all
select 18318,'YUZH' from dual union all
select 18319,'MO' from dual union all
select 1832,'USAF' from dual union all
select 18320,'NASDA' from dual union all
select 18321,'MO' from dual union all
select 18322,'MO' from dual union all
select 18323,'MO' from dual union all
select 18324,'MO' from dual union all
select 18325,'MO' from dual union all
select 18326,'MO' from dual union all
select 18327,'RVSN' from dual union all
select 18328,'MSVYAZ' from dual union all
select 18329,'RVSN' from dual union all
select 1833,'USAF' from dual union all
select 18330,'RVSN' from dual union all
select 18331,'RVSN' from dual union all
select 18332,'RVSN' from dual union all
select 18333,'RVSN' from dual union all
select 18334,'MO' from dual union all
select 18335,'MO' from dual union all
select 18336,'MO' from dual union all
select 18337,'MO' from dual union all
select 18338,'MO' from dual union all
select 18339,'MO' from dual union all
select 1834,'USAF' from dual union all
select 18340,'MO' from dual union all
select 18341,'MAI' from dual union all
select 18342,'MAI' from dual union all
select 18343,'MO' from dual union all
select 18344,'RVSN' from dual union all
select 18345,'MO' from dual union all
select 18346,'MO' from dual union all
select 18347,'MO' from dual union all
select 18348,'MVD' from dual union all
select 18349,'RVSN' from dual union all
select 1835,'USAF' from dual union all
select 18350,'AUSS' from dual union all
select 18351,'EUTEL' from dual union all
select 18352,'AE' from dual union all
select 18353,'AE' from dual union all
select 18354,'VMF' from dual union all
select 18355,'MO' from dual union all
select 18356,'MO' from dual union all
select 18357,'MO' from dual union all
select 18358,'RVSN' from dual union all
select 18359,'RVSN' from dual union all
select 1836,'USAF' from dual union all
select 18360,'RVSN' from dual union all
select 18361,'USN' from dual union all
select 18362,'USN' from dual union all
select 18363,'NASA' from dual union all
select 18364,'AE' from dual union all
select 18365,'USN' from dual union all
select 18366,'MO' from dual union all
select 18367,'RVSN' from dual union all
select 18368,'MO' from dual union all
select 18369,'MO' from dual union all
select 1837,'USAF' from dual union all
select 18370,'MO' from dual union all
select 18371,'MO' from dual union all
select 18372,'MAI' from dual union all
select 18373,'MAI' from dual union all
select 18374,'RVSN' from dual union all
select 18375,'RVSN' from dual union all
select 18376,'NPOE' from dual union all
select 18377,'RVSN' from dual union all
select 18378,'VMF' from dual union all
select 18379,'VMF' from dual union all
select 1838,'CIA' from dual union all
select 1838,'NRO' from dual union all
select 18380,'IMBP' from dual union all
select 18380,'TSSKB' from dual union all
select 18381,'RVSN' from dual union all
select 18382,'VMF' from dual union all
select 18383,'VMF' from dual union all
select 18384,'MO' from dual union all
select 18385,'RVSN' from dual;
insert into SATELLITE_ORG
select 18386,'RVSN' from dual union all
select 18387,'RVSN' from dual union all
select 18388,'RVSN' from dual union all
select 18389,'MO' from dual union all
select 1839,'NASA' from dual union all
select 18390,'MO' from dual union all
select 18391,'MO' from dual union all
select 18392,'MO' from dual union all
select 18393,'MO' from dual union all
select 18394,'MO' from dual union all
select 18395,'RVSN' from dual union all
select 18396,'VMF' from dual union all
select 18397,'MO' from dual union all
select 18398,'MO' from dual union all
select 18399,'MO' from dual union all
select 184,'USAF' from dual union all
select 1840,'NASA' from dual union all
select 1840,'USAF' from dual union all
select 18400,'MO' from dual union all
select 18401,'MO' from dual union all
select 18402,'VMF' from dual union all
select 18403,'RVSN' from dual union all
select 18404,'AE' from dual union all
select 18405,'AE' from dual union all
select 18406,'AE' from dual union all
select 18407,'MONBU' from dual union all
select 18408,'MONBU' from dual union all
select 18409,'MONBU' from dual union all
select 1841,'NASA' from dual union all
select 18410,'MO' from dual union all
select 18411,'MO' from dual union all
select 18412,'MO' from dual union all
select 18413,'PVO' from dual union all
select 18414,'PVO' from dual union all
select 18415,'PVO' from dual union all
select 18416,'MO' from dual union all
select 18417,'MO' from dual union all
select 18418,'NASA' from dual union all
select 18419,'NASA' from dual union all
select 1842,'NASA' from dual union all
select 18420,'RVSN' from dual union all
select 18421,'VS' from dual union all
select 18422,'MO' from dual union all
select 18423,'USAF' from dual union all
select 18424,'USAF' from dual union all
select 18425,'USAF' from dual union all
select 18426,'USAF' from dual union all
select 18427,'USAF' from dual union all
select 18428,'PVO' from dual union all
select 18429,'NASA' from dual union all
select 1843,'METP' from dual union all
select 18430,'VMF' from dual union all
select 18431,'PVO' from dual union all
select 18432,'MO' from dual union all
select 18433,'RVSN' from dual union all
select 18434,'PVO' from dual union all
select 18435,'PVO' from dual union all
select 18436,'MO' from dual union all
select 18437,'MO' from dual union all
select 18438,'MO' from dual union all
select 18439,'MO' from dual union all
select 1844,'RVSN' from dual union all
select 18440,'MO' from dual union all
select 18441,'CIA' from dual union all
select 18441,'NRO' from dual union all
select 18442,'USAF' from dual union all
select 18443,'PVO' from dual union all
select 18444,'RVSN' from dual union all
select 18445,'RVSN' from dual union all
select 18446,'RVSN' from dual union all
select 18447,'RVSN' from dual union all
select 18448,'RVSN' from dual union all
select 18449,'MO' from dual union all
select 1845,'NASA' from dual union all
select 18450,'AE' from dual union all
select 18451,'AE' from dual union all
select 18452,'AE' from dual union all
select 18453,'AE' from dual union all
select 18454,'AE' from dual union all
select 18455,'AE' from dual union all
select 18456,'AE' from dual union all
select 18457,'AE' from dual union all
select 18458,'AE' from dual union all
select 18459,'AE' from dual union all
select 1846,'PVO' from dual union all
select 1846,'YUZH' from dual union all
select 18460,'AE' from dual union all
select 18461,'AE' from dual union all
select 18462,'AE' from dual union all
select 18463,'NASA' from dual union all
select 18464,'USAF' from dual union all
select 18465,'USAF' from dual union all
select 18466,'NASA' from dual union all
select 18467,'NASA' from dual union all
select 18468,'PVO' from dual union all
select 18469,'NASA' from dual union all
select 1847,'RVSN' from dual union all
select 18470,'NASA' from dual union all
select 18471,'PVO' from dual union all
select 18472,'NASA' from dual;
insert into SATELLITE_ORG
select 18473,'VMF' from dual union all
select 18474,'VMF' from dual union all
select 18475,'NASA' from dual union all
select 18476,'MO' from dual union all
select 18477,'PVO' from dual union all
select 18478,'PVO' from dual union all
select 18479,'USAF' from dual union all
select 1848,'USAF' from dual union all
select 18480,'VMF' from dual union all
select 18481,'VMF' from dual union all
select 18482,'VMF' from dual union all
select 18483,'VMF' from dual union all
select 18484,'VMF' from dual union all
select 18485,'VMF' from dual union all
select 18486,'USAF' from dual union all
select 18487,'USAF' from dual union all
select 18488,'PVO' from dual union all
select 18489,'PVO' from dual union all
select 1849,'USAF' from dual union all
select 18490,'PVO' from dual union all
select 18491,'MO' from dual union all
select 18492,'RVSN' from dual union all
select 18493,'VMF' from dual union all
select 18494,'PVO' from dual union all
select 18495,'VMF' from dual union all
select 18496,'USN' from dual union all
select 18497,'PVO' from dual union all
select 18498,'USAF' from dual union all
select 18499,'MO' from dual union all
select 185,'USAF' from dual union all
select 1850,'USAF' from dual union all
select 18500,'MO' from dual union all
select 18501,'RVSN' from dual union all
select 18502,'MO' from dual union all
select 18503,'USAF' from dual union all
select 18504,'MO' from dual union all
select 18505,'MO' from dual union all
select 18506,'VMF' from dual union all
select 18507,'PVO' from dual union all
select 18508,'NASA' from dual union all
select 18509,'MO' from dual union all
select 1851,'USAF' from dual union all
select 18510,'MO' from dual union all
select 18511,'NASA' from dual union all
select 18512,'RVSN' from dual union all
select 18513,'NASA' from dual union all
select 18514,'MO' from dual union all
select 18515,'MO' from dual union all
select 18516,'MO' from dual union all
select 18517,'PVO' from dual union all
select 18518,'PVO' from dual union all
select 18519,'MO' from dual union all
select 1852,'USAF' from dual union all
select 18520,'VMF' from dual union all
select 18521,'PVO' from dual union all
select 18522,'AE' from dual union all
select 18523,'MONBU' from dual union all
select 18524,'MO' from dual union all
select 18525,'USAF' from dual union all
select 18526,'MO' from dual union all
select 18527,'MO' from dual union all
select 18528,'RVSN' from dual union all
select 18529,'MO' from dual union all
select 1853,'USAF' from dual union all
select 18530,'USN' from dual union all
select 18531,'PVO' from dual union all
select 18532,'NASA' from dual union all
select 18533,'USAF' from dual union all
select 18534,'USAF' from dual union all
select 18535,'VTS' from dual union all
select 18536,'RVSN' from dual union all
select 18537,'VTS' from dual union all
select 18538,'USAF' from dual union all
select 18539,'NASA' from dual union all
select 1854,'USAF' from dual union all
select 18540,'NASA' from dual union all
select 18541,'AE' from dual union all
select 18542,'VMF' from dual union all
select 18543,'USAF' from dual union all
select 18544,'VMF' from dual union all
select 18545,'RVSN' from dual union all
select 18546,'USAF' from dual union all
select 18547,'NASA' from dual union all
select 18548,'PVO' from dual union all
select 18549,'PVO' from dual union all
select 1855,'USAF' from dual union all
select 18550,'MO' from dual union all
select 18551,'PVO' from dual union all
select 18552,'MO' from dual union all
select 18553,'AE' from dual union all
select 18554,'AE' from dual union all
select 18555,'AE' from dual union all
select 18556,'PVO' from dual union all
select 18557,'AE' from dual union all
select 18558,'AE' from dual union all
select 18559,'AE' from dual union all
select 1856,'USAF' from dual union all
select 18560,'PVO' from dual union all
select 18561,'USN' from dual union all
select 18562,'USN' from dual;
insert into SATELLITE_ORG
select 18563,'AE' from dual union all
select 18564,'USAF' from dual union all
select 18565,'RVSN' from dual union all
select 18566,'VMF' from dual union all
select 18567,'MO' from dual union all
select 18568,'NPOE' from dual union all
select 18569,'RVSN' from dual union all
select 1857,'USAF' from dual union all
select 18570,'DBP' from dual union all
select 18571,'AE' from dual union all
select 18572,'MO' from dual union all
select 18573,'MO' from dual union all
select 18574,'MO' from dual union all
select 18575,'MO' from dual union all
select 18576,'RVSN' from dual union all
select 18577,'RVSN' from dual union all
select 18578,'RVSN' from dual union all
select 18579,'RVSN' from dual union all
select 1858,'USAF' from dual union all
select 18580,'RVSN' from dual union all
select 18581,'MO' from dual union all
select 18582,'MO' from dual union all
select 18583,'USAF' from dual union all
select 18584,'USAF' from dual union all
select 18585,'MO' from dual union all
select 18586,'RVSN' from dual union all
select 18587,'MO' from dual union all
select 18588,'MO' from dual union all
select 18589,'RVSN' from dual union all
select 1859,'USAF' from dual union all
select 18590,'PVO' from dual union all
select 18591,'NASA' from dual union all
select 18592,'MO' from dual union all
select 18593,'MO' from dual union all
select 18594,'USN' from dual union all
select 18595,'MO' from dual union all
select 18596,'MO' from dual union all
select 18597,'PVO' from dual union all
select 18598,'RVSN' from dual union all
select 18599,'NASA' from dual union all
select 186,'CIA' from dual union all
select 186,'NRO' from dual union all
select 1860,'USAF' from dual union all
select 18600,'NASA' from dual union all
select 18601,'MO' from dual union all
select 18602,'PVO' from dual union all
select 18603,'USAF' from dual union all
select 18604,'USAF' from dual union all
select 18605,'NASA' from dual union all
select 18606,'PVO' from dual union all
select 18607,'AE' from dual union all
select 18608,'MO' from dual union all
select 18609,'MO' from dual union all
select 1861,'USAF' from dual union all
select 18610,'VMF' from dual union all
select 18611,'AE' from dual union all
select 18612,'USAF' from dual union all
select 18613,'PVO' from dual union all
select 18614,'AE' from dual union all
select 18615,'USAF' from dual union all
select 18616,'USAF' from dual union all
select 18617,'JPL' from dual union all
select 18618,'PVO' from dual union all
select 18619,'VMF' from dual union all
select 1862,'USAF' from dual union all
select 18620,'VMF' from dual union all
select 18621,'VMF' from dual union all
select 18622,'VMF' from dual union all
select 18623,'VMF' from dual union all
select 18624,'VMF' from dual union all
select 18625,'MO' from dual union all
select 18626,'RVSN' from dual union all
select 18627,'MO' from dual union all
select 18628,'VMF' from dual union all
select 18629,'VMF' from dual union all
select 1863,'USAF' from dual union all
select 18630,'VMF' from dual union all
select 18631,'MO' from dual union all
select 18632,'RVSN' from dual union all
select 18633,'RVSN' from dual union all
select 18634,'RVSN' from dual union all
select 18635,'RVSN' from dual union all
select 18636,'RVSN' from dual union all
select 18637,'MO' from dual union all
select 18638,'MO' from dual union all
select 18639,'MO' from dual union all
select 1864,'USN' from dual union all
select 18640,'MO' from dual union all
select 18641,'NASA' from dual union all
select 18642,'AE' from dual union all
select 18643,'NASA' from dual union all
select 18644,'PVO' from dual union all
select 18645,'NASA' from dual union all
select 18646,'USAF' from dual union all
select 18647,'NASA' from dual union all
select 18648,'PVO' from dual union all
select 18649,'JPL' from dual union all
select 1865,'NASA' from dual union all
select 18650,'RVSN' from dual union all
select 18651,'USAF' from dual;
insert into SATELLITE_ORG
select 18652,'VMF' from dual union all
select 18653,'VMF' from dual union all
select 18654,'VMF' from dual union all
select 18655,'VMF' from dual union all
select 18656,'VMF' from dual union all
select 18657,'VMF' from dual union all
select 18658,'VMF' from dual union all
select 18659,'VMF' from dual union all
select 1866,'CIA' from dual union all
select 1866,'NRO' from dual union all
select 18660,'VMF' from dual union all
select 18661,'VMF' from dual union all
select 18662,'VMF' from dual union all
select 18663,'VMF' from dual union all
select 18664,'VMF' from dual union all
select 18665,'VMF' from dual union all
select 18666,'MO' from dual union all
select 18667,'RVSN' from dual union all
select 18668,'PVO' from dual union all
select 18669,'RVSN' from dual union all
select 1867,'KB1' from dual union all
select 1867,'VMF' from dual union all
select 18670,'USAF' from dual union all
select 18671,'NASA' from dual union all
select 18672,'NASA' from dual union all
select 18673,'PVO' from dual union all
select 18674,'AE' from dual union all
select 18675,'USAF' from dual union all
select 18676,'AE' from dual union all
select 18677,'NASA' from dual union all
select 18678,'USAF' from dual union all
select 18679,'NASA' from dual union all
select 1868,'OKB10' from dual union all
select 18680,'RVSN' from dual union all
select 18681,'MO' from dual union all
select 18682,'MONBU' from dual union all
select 18683,'AE' from dual union all
select 18684,'AE' from dual union all
select 18685,'AE' from dual union all
select 18686,'RVSN' from dual union all
select 18687,'MO' from dual union all
select 18688,'USAF' from dual union all
select 18689,'PVO' from dual union all
select 1869,'RVSN' from dual union all
select 18690,'NASA' from dual union all
select 18691,'USAF' from dual union all
select 18692,'PVO' from dual union all
select 18693,'NASA' from dual union all
select 18694,'VMF' from dual union all
select 18695,'USAF' from dual union all
select 18696,'MO' from dual union all
select 18697,'MO' from dual union all
select 18698,'MO' from dual union all
select 18699,'NPOE' from dual union all
select 187,'USAF' from dual union all
select 1870,'USAF' from dual union all
select 18700,'RVSN' from dual union all
select 18701,'PVO' from dual union all
select 18702,'RVSN' from dual union all
select 18703,'RVSN' from dual union all
select 18704,'RVSN' from dual union all
select 18705,'MO' from dual union all
select 18706,'MO' from dual union all
select 18707,'MO' from dual union all
select 18708,'MO' from dual union all
select 18709,'VMF' from dual union all
select 1871,'RVSN' from dual union all
select 18710,'RVSN' from dual union all
select 18711,'MO' from dual union all
select 18712,'RVSN' from dual union all
select 18713,'MVD' from dual union all
select 18714,'RVSN' from dual union all
select 18715,'MSVYAZ' from dual union all
select 18716,'RVSN' from dual union all
select 18717,'RVSN' from dual union all
select 18718,'RVSN' from dual union all
select 18719,'RVSN' from dual union all
select 1872,'RVSN' from dual union all
select 18720,'MO' from dual union all
select 18721,'RVSN' from dual union all
select 18722,'RVSN' from dual union all
select 18723,'NPOE' from dual union all
select 18724,'MO' from dual union all
select 18725,'MO' from dual union all
select 18726,'MO' from dual union all
select 18727,'MO' from dual union all
select 18728,'MO' from dual union all
select 18729,'MO' from dual union all
select 1873,'RVSN' from dual union all
select 18730,'MO' from dual union all
select 18731,'MO' from dual union all
select 18732,'MO' from dual union all
select 18733,'MO' from dual union all
select 18734,'MO' from dual union all
select 18735,'MO' from dual union all
select 18736,'MO' from dual union all
select 18737,'MO' from dual union all
select 18738,'MO' from dual union all
select 18739,'MO' from dual union all
select 1874,'RVSN' from dual;
insert into SATELLITE_ORG
select 18740,'MO' from dual union all
select 18741,'MO' from dual union all
select 18742,'MO' from dual union all
select 18743,'MO' from dual union all
select 18744,'MO' from dual union all
select 18745,'MO' from dual union all
select 18746,'MO' from dual union all
select 18747,'MO' from dual union all
select 18748,'VS' from dual union all
select 18749,'MO' from dual union all
select 1875,'RVSN' from dual union all
select 18750,'MO' from dual union all
select 18751,'MO' from dual union all
select 18752,'MO' from dual union all
select 18753,'MO' from dual union all
select 18754,'MO' from dual union all
select 18755,'MO' from dual union all
select 18756,'MO' from dual union all
select 18757,'MO' from dual union all
select 18758,'MO' from dual union all
select 18759,'MO' from dual union all
select 1876,'RVSN' from dual union all
select 18760,'MO' from dual union all
select 18761,'MO' from dual union all
select 18762,'MO' from dual union all
select 18763,'MO' from dual union all
select 18764,'RVSN' from dual union all
select 18765,'RVSN' from dual union all
select 18766,'RVSN' from dual union all
select 18767,'RVSN' from dual union all
select 18768,'MO' from dual union all
select 18769,'MO' from dual union all
select 1877,'RVSN' from dual union all
select 18770,'MO' from dual union all
select 18771,'MO' from dual union all
select 18772,'MO' from dual union all
select 18773,'NASA' from dual union all
select 18774,'AE' from dual union all
select 18775,'NASA' from dual union all
select 18776,'USAF' from dual union all
select 18777,'RVSN' from dual union all
select 18778,'AE' from dual union all
select 18779,'NASA' from dual union all
select 1878,'RVSN' from dual union all
select 18780,'AE' from dual union all
select 18781,'AE' from dual union all
select 18782,'AE' from dual union all
select 18783,'USAF' from dual union all
select 18784,'USAF' from dual union all
select 18785,'NASA' from dual union all
select 18786,'NASA' from dual union all
select 18787,'AE' from dual union all
select 18788,'MO' from dual union all
select 18789,'MO' from dual union all
select 1879,'USAF' from dual union all
select 18790,'MO' from dual union all
select 18791,'MO' from dual union all
select 18792,'MO' from dual union all
select 18793,'MO' from dual union all
select 18794,'MO' from dual union all
select 18795,'NPOE' from dual union all
select 18796,'RVSN' from dual union all
select 18797,'VMF' from dual union all
select 18798,'PVO' from dual union all
select 18799,'VMF' from dual union all
select 188,'USAF' from dual union all
select 1880,'USAF' from dual union all
select 18800,'MO' from dual union all
select 18801,'MO' from dual union all
select 18802,'MO' from dual union all
select 18803,'MO' from dual union all
select 18804,'MO' from dual union all
select 18805,'MO' from dual union all
select 18806,'MO' from dual union all
select 18807,'MO' from dual union all
select 18808,'MO' from dual union all
select 18809,'MO' from dual union all
select 1881,'USAF' from dual union all
select 18810,'RVSN' from dual union all
select 18811,'NASA' from dual union all
select 18812,'VMF' from dual union all
select 18813,'AE' from dual union all
select 18814,'NASA' from dual union all
select 18815,'MO' from dual union all
select 18816,'ISAS' from dual union all
select 18817,'AE' from dual union all
select 18818,'USAF' from dual union all
select 18819,'RVSN' from dual union all
select 1882,'USAF' from dual union all
select 18820,'GMS' from dual union all
select 18820,'MO' from dual union all
select 18821,'MO' from dual union all
select 18822,'USAF' from dual union all
select 18823,'MO' from dual union all
select 18824,'RVSN' from dual union all
select 18825,'MO' from dual union all
select 18826,'MO' from dual union all
select 18827,'MO' from dual union all
select 18828,'MO' from dual union all
select 18829,'MO' from dual;
insert into SATELLITE_ORG
select 1883,'USAF' from dual union all
select 18830,'MO' from dual union all
select 18831,'MO' from dual union all
select 18832,'MO' from dual union all
select 18833,'MO' from dual union all
select 18834,'MO' from dual union all
select 18835,'MO' from dual union all
select 18836,'MO' from dual union all
select 18837,'MO' from dual union all
select 18838,'MO' from dual union all
select 18839,'MO' from dual union all
select 1884,'USAF' from dual union all
select 18840,'MO' from dual union all
select 18841,'MO' from dual union all
select 18842,'MO' from dual union all
select 18843,'MO' from dual union all
select 18844,'MO' from dual union all
select 18845,'USAF' from dual union all
select 18846,'USAF' from dual union all
select 18847,'SDIO' from dual union all
select 18848,'SDIO' from dual union all
select 18849,'SDIO' from dual union all
select 1885,'USAF' from dual union all
select 18850,'MO' from dual union all
select 18851,'MO' from dual union all
select 18852,'MO' from dual union all
select 18853,'MO' from dual union all
select 18854,'SDIO' from dual union all
select 18855,'PVO' from dual union all
select 18856,'PVO' from dual union all
select 18857,'MO' from dual union all
select 18858,'RVSN' from dual union all
select 18859,'RVSN' from dual union all
select 1886,'USAF' from dual union all
select 18860,'MVD' from dual union all
select 18861,'RVSN' from dual union all
select 18862,'MO' from dual union all
select 18863,'MO' from dual union all
select 18864,'NASA' from dual union all
select 18865,'AE' from dual union all
select 18866,'PVO' from dual union all
select 18867,'USAF' from dual union all
select 18868,'VMF' from dual union all
select 18869,'VMF' from dual union all
select 1887,'USAF' from dual union all
select 18870,'NASA' from dual union all
select 18871,'USAF' from dual union all
select 18872,'NASA' from dual union all
select 18873,'USAF' from dual union all
select 18874,'USAF' from dual union all
select 18875,'MO' from dual union all
select 18876,'RVSN' from dual union all
select 18877,'NASDA' from dual union all
select 18877,'TSCJ' from dual union all
select 18878,'NASDA' from dual union all
select 18879,'NASDA' from dual union all
select 1888,'USAF' from dual union all
select 18880,'MO' from dual union all
select 18881,'PVO' from dual union all
select 18882,'RVSN' from dual union all
select 18883,'RVSN' from dual union all
select 18884,'RVSN' from dual union all
select 18885,'NPOE' from dual union all
select 18886,'NPOE' from dual union all
select 18887,'NPOE' from dual union all
select 18888,'MO' from dual union all
select 18889,'NPOE' from dual union all
select 1889,'USAF' from dual union all
select 18890,'USAF' from dual union all
select 18891,'PVO' from dual union all
select 18892,'MO' from dual union all
select 18893,'MO' from dual union all
select 18894,'MO' from dual union all
select 18895,'MO' from dual union all
select 18896,'MO' from dual union all
select 18897,'MO' from dual union all
select 18898,'MO' from dual union all
select 18899,'MO' from dual union all
select 189,'CIA' from dual union all
select 189,'NRO' from dual union all
select 1890,'USAF' from dual union all
select 18900,'MO' from dual union all
select 18901,'NASA' from dual union all
select 18902,'MO' from dual union all
select 18903,'AE' from dual union all
select 18904,'HEC' from dual union all
select 18905,'AE' from dual union all
select 18906,'NASA' from dual union all
select 18907,'PVO' from dual union all
select 18908,'USAF' from dual union all
select 18909,'MO' from dual union all
select 1891,'USAF' from dual union all
select 18910,'MO' from dual union all
select 18911,'NASA' from dual union all
select 18912,'VMF' from dual union all
select 18913,'RVSN' from dual union all
select 18914,'AE' from dual union all
select 18915,'USAF' from dual union all
select 18916,'MO' from dual union all
select 18917,'MO' from dual;
insert into SATELLITE_ORG
select 18918,'MO' from dual union all
select 18919,'MO' from dual union all
select 1892,'USAF' from dual union all
select 18920,'MO' from dual union all
select 18921,'MO' from dual union all
select 18922,'CHIS' from dual union all
select 18923,'MAI' from dual union all
select 18924,'MO' from dual union all
select 18925,'MO' from dual union all
select 18926,'MO' from dual union all
select 18927,'MONBU' from dual union all
select 18928,'CNES' from dual union all
select 18929,'CNES' from dual union all
select 1893,'USAF' from dual union all
select 18930,'MO' from dual union all
select 18931,'MO' from dual union all
select 18932,'RVSN' from dual union all
select 18933,'MO' from dual union all
select 18934,'MO' from dual union all
select 18935,'MO' from dual union all
select 18936,'MO' from dual union all
select 18937,'MO' from dual union all
select 18938,'MO' from dual union all
select 18939,'MO' from dual union all
select 1894,'USAF' from dual union all
select 18940,'MO' from dual union all
select 18941,'MO' from dual union all
select 18942,'MO' from dual union all
select 18943,'MO' from dual union all
select 18944,'MO' from dual union all
select 18945,'RVSN' from dual union all
select 18946,'MO' from dual union all
select 18947,'RVSN' from dual union all
select 18948,'RVSN' from dual union all
select 18949,'RVSN' from dual union all
select 1895,'USAF' from dual union all
select 18950,'MO' from dual union all
select 18951,'GTE' from dual union all
select 18952,'FTEL' from dual union all
select 18953,'AE' from dual union all
select 18954,'AE' from dual union all
select 18955,'USAF' from dual union all
select 18956,'USAF' from dual union all
select 18957,'VMF' from dual union all
select 18958,'VS' from dual union all
select 18959,'MO' from dual union all
select 1896,'USAF' from dual union all
select 18960,'ISRO' from dual union all
select 18961,'RVSN' from dual union all
select 18962,'USAF' from dual union all
select 18963,'USAF' from dual union all
select 18964,'USAF' from dual union all
select 18965,'USAF' from dual union all
select 18966,'RVSN' from dual union all
select 18967,'USAF' from dual union all
select 18968,'MO' from dual union all
select 18969,'MO' from dual union all
select 1897,'USAF' from dual union all
select 18970,'MO' from dual union all
select 18971,'RVSN' from dual union all
select 18972,'MO' from dual union all
select 18973,'MO' from dual union all
select 18974,'MO' from dual union all
select 18975,'VMF' from dual union all
select 18976,'PVO' from dual union all
select 18977,'RVSN' from dual union all
select 18978,'USAF' from dual union all
select 18979,'USAF' from dual union all
select 1898,'USAF' from dual union all
select 18980,'MO' from dual union all
select 18981,'RVSN' from dual union all
select 18982,'RVSN' from dual union all
select 18983,'RVSN' from dual union all
select 18984,'USAF' from dual union all
select 18985,'VMF' from dual union all
select 18986,'RVSN' from dual union all
select 18987,'MO' from dual union all
select 18988,'MO' from dual union all
select 18989,'MO' from dual union all
select 1899,'USAF' from dual union all
select 18990,'MO' from dual union all
select 18991,'MO' from dual union all
select 18992,'NPOE' from dual union all
select 18993,'RVSN' from dual union all
select 18994,'MO' from dual union all
select 18995,'MO' from dual union all
select 18996,'JPL' from dual union all
select 18997,'PVO' from dual union all
select 18998,'PVO' from dual union all
select 18999,'USAF' from dual union all
select 19,'CIA' from dual union all
select 19,'DOD' from dual union all
select 190,'USAF' from dual union all
select 1900,'USAF' from dual union all
select 19000,'NASA' from dual union all
select 19001,'USAF' from dual union all
select 19002,'USAF' from dual union all
select 19003,'USAF' from dual union all
select 19004,'RVSN' from dual union all
select 19005,'MO' from dual;
insert into SATELLITE_ORG
select 19006,'MO' from dual union all
select 19007,'AE' from dual union all
select 19008,'NASA' from dual union all
select 19009,'AE' from dual union all
select 1901,'CIA' from dual union all
select 1901,'NRO' from dual union all
select 19010,'RVSN' from dual union all
select 19011,'MO' from dual union all
select 19012,'RVSN' from dual union all
select 19013,'ASI' from dual union all
select 19014,'NASA' from dual union all
select 19015,'MO' from dual union all
select 19016,'RVSN' from dual union all
select 19017,'MSVYAZ' from dual union all
select 19018,'RVSN' from dual union all
select 19019,'RVSN' from dual union all
select 1902,'OSCAR' from dual union all
select 19020,'RVSN' from dual union all
select 19021,'MO' from dual union all
select 19022,'MO' from dual union all
select 19023,'PVO' from dual union all
select 19024,'PVO' from dual union all
select 19025,'MO' from dual union all
select 19026,'MO' from dual union all
select 19027,'MO' from dual union all
select 19028,'USAF' from dual union all
select 19029,'USAF' from dual union all
select 1903,'MO' from dual union all
select 19030,'USAF' from dual union all
select 19031,'USAF' from dual union all
select 19032,'USAF' from dual union all
select 19033,'NASA' from dual union all
select 19034,'AE' from dual union all
select 19035,'MO' from dual union all
select 19036,'RVSN' from dual union all
select 19037,'RVSN' from dual union all
select 19038,'MO' from dual union all
select 19039,'RVSN' from dual union all
select 1904,'RVSN' from dual union all
select 19040,'MO' from dual union all
select 19041,'MO' from dual union all
select 19042,'RVSN' from dual union all
select 19043,'TSSKB' from dual union all
select 19044,'RVSN' from dual union all
select 19045,'MVD' from dual union all
select 19046,'RVSN' from dual union all
select 19047,'MO' from dual union all
select 19048,'RVSN' from dual union all
select 19049,'CNES' from dual union all
select 1905,'MO' from dual union all
select 19050,'AE' from dual union all
select 19051,'USAF' from dual union all
select 19052,'USAF' from dual union all
select 19053,'NASA' from dual union all
select 19054,'NASA' from dual union all
select 19055,'USAF' from dual union all
select 19056,'USAF' from dual union all
select 19057,'USAF' from dual union all
select 19058,'USAF' from dual union all
select 19059,'USAF' from dual union all
select 1906,'RVSN' from dual union all
select 19060,'USAF' from dual union all
select 19061,'USAF' from dual union all
select 19062,'USN' from dual union all
select 19063,'MO' from dual union all
select 19064,'PVO' from dual union all
select 19065,'MO' from dual union all
select 19066,'MO' from dual union all
select 19067,'MO' from dual union all
select 19068,'MO' from dual union all
select 19069,'MO' from dual union all
select 1907,'RVSN' from dual union all
select 19070,'USN' from dual union all
select 19071,'USN' from dual union all
select 19072,'NASA' from dual union all
select 19073,'METP' from dual union all
select 19074,'RVSN' from dual union all
select 19075,'RVSN' from dual union all
select 19076,'RVSN' from dual union all
select 19077,'USN' from dual union all
select 19078,'USN' from dual union all
select 19079,'MO' from dual union all
select 1908,'USAF' from dual union all
select 19080,'RVSN' from dual union all
select 19081,'MO' from dual union all
select 19082,'RVSN' from dual union all
select 19083,'RVSN' from dual union all
select 19084,'MO' from dual union all
select 19085,'MO' from dual union all
select 19086,'MO' from dual union all
select 19087,'MO' from dual union all
select 19088,'MO' from dual union all
select 19089,'NPOE' from dual union all
select 1909,'USAF' from dual union all
select 19090,'MSVYAZ' from dual union all
select 19091,'RVSN' from dual union all
select 19092,'RVSN' from dual union all
select 19093,'RVSN' from dual union all
select 19094,'RVSN' from dual union all
select 19095,'RVSN' from dual;
insert into SATELLITE_ORG
select 19096,'MO' from dual union all
select 19097,'MO' from dual union all
select 19098,'MO' from dual union all
select 19099,'MO' from dual union all
select 191,'USAF' from dual union all
select 1910,'USAF' from dual union all
select 19100,'MO' from dual union all
select 19101,'VMF' from dual union all
select 19102,'MO' from dual union all
select 19103,'JPL' from dual union all
select 19104,'MO' from dual union all
select 19105,'NASA' from dual union all
select 19106,'NASA' from dual union all
select 19107,'AE' from dual union all
select 19108,'USAF' from dual union all
select 19109,'USAF' from dual union all
select 1911,'USAF' from dual union all
select 19110,'NASA' from dual union all
select 19111,'RVSN' from dual union all
select 19112,'PVO' from dual union all
select 19113,'USAF' from dual union all
select 19114,'PVO' from dual union all
select 19115,'MO' from dual union all
select 19116,'RVSN' from dual union all
select 19117,'NPOE' from dual union all
select 19118,'RVSN' from dual union all
select 19119,'VS' from dual union all
select 1912,'USAF' from dual union all
select 19120,'MO' from dual union all
select 19121,'INTEL' from dual union all
select 19122,'AE' from dual union all
select 19123,'VTS' from dual union all
select 19124,'RVSN' from dual union all
select 19125,'MO' from dual union all
select 19126,'MO' from dual union all
select 19127,'MO' from dual union all
select 19128,'MO' from dual union all
select 19129,'MO' from dual union all
select 1913,'USAF' from dual union all
select 19130,'MO' from dual union all
select 19131,'MO' from dual union all
select 19132,'RVSN' from dual union all
select 19133,'RVSN' from dual union all
select 19134,'RVSN' from dual union all
select 19135,'RVSN' from dual union all
select 19136,'RVSN' from dual union all
select 19137,'RVSN' from dual union all
select 19138,'RVSN' from dual union all
select 19139,'RVSN' from dual union all
select 1914,'USAF' from dual union all
select 19140,'USN' from dual union all
select 19141,'MO' from dual union all
select 19142,'RVSN' from dual union all
select 19143,'RVSN' from dual union all
select 19144,'RVSN' from dual union all
select 19145,'MO' from dual union all
select 19146,'MO' from dual union all
select 19147,'MO' from dual union all
select 19148,'VMF' from dual union all
select 19149,'MO' from dual union all
select 1915,'USAF' from dual union all
select 19150,'NASA' from dual union all
select 19151,'MO' from dual union all
select 19152,'MO' from dual union all
select 19153,'MO' from dual union all
select 19154,'NASA' from dual union all
select 19155,'MO' from dual union all
select 19156,'USAF' from dual union all
select 19157,'USAF' from dual union all
select 19158,'AE' from dual union all
select 19159,'MO' from dual union all
select 1916,'USAF' from dual union all
select 19160,'VMF' from dual union all
select 19161,'VMF' from dual union all
select 19162,'VMF' from dual union all
select 19163,'MO' from dual union all
select 19164,'MO' from dual union all
select 19165,'MO' from dual union all
select 19166,'RVSN' from dual union all
select 19167,'RVSN' from dual union all
select 19168,'RVSN' from dual union all
select 19169,'RVSN' from dual union all
select 1917,'USAF' from dual union all
select 19170,'RVSN' from dual union all
select 19171,'VMF' from dual union all
select 19172,'VMF' from dual union all
select 19173,'RVSN' from dual union all
select 19174,'RVSN' from dual union all
select 19175,'RVSN' from dual union all
select 19176,'RVSN' from dual union all
select 19177,'VMF' from dual union all
select 19178,'RVSN' from dual union all
select 19179,'MO' from dual union all
select 1918,'USAF' from dual union all
select 19180,'MO' from dual union all
select 19181,'USN' from dual union all
select 19182,'USAF' from dual union all
select 19183,'NASA' from dual union all
select 19184,'MO' from dual union all
select 19185,'USAF' from dual;
insert into SATELLITE_ORG
select 19186,'NPOE' from dual union all
select 19187,'RVSN' from dual union all
select 19188,'MO' from dual union all
select 19189,'MO' from dual union all
select 19189,'MSVYAZ' from dual union all
select 1919,'USAF' from dual union all
select 19190,'RVSN' from dual union all
select 19191,'RVSN' from dual union all
select 19192,'RVSN' from dual union all
select 19193,'VMF' from dual union all
select 19194,'RVSN' from dual union all
select 19195,'VTS' from dual union all
select 19196,'MO' from dual union all
select 19197,'MVD' from dual union all
select 19198,'RVSN' from dual union all
select 19199,'MO' from dual union all
select 192,'USAF' from dual union all
select 1920,'USAF' from dual union all
select 19200,'MO' from dual union all
select 19201,'MO' from dual union all
select 19202,'MO' from dual union all
select 19203,'MO' from dual union all
select 19204,'NPOE' from dual union all
select 19205,'RVSN' from dual union all
select 19206,'MO' from dual union all
select 19207,'RVSN' from dual union all
select 19208,'MO' from dual union all
select 19209,'MO' from dual union all
select 1921,'USAF' from dual union all
select 19210,'VS' from dual union all
select 19211,'MO' from dual union all
select 19212,'MO' from dual union all
select 19213,'MO' from dual union all
select 19214,'MO' from dual union all
select 19215,'EUMET' from dual union all
select 19216,'AMSAT' from dual union all
select 19217,'PAN' from dual union all
select 19218,'AE' from dual union all
select 19219,'AE' from dual union all
select 1922,'USAF' from dual union all
select 19220,'AE' from dual union all
select 19221,'PVO' from dual union all
select 19222,'USN' from dual union all
select 19223,'USN' from dual union all
select 19224,'NASA' from dual union all
select 19225,'NPOE' from dual union all
select 19226,'RVSN' from dual union all
select 19227,'RVSN' from dual union all
select 19228,'RVSN' from dual union all
select 19229,'RVSN' from dual union all
select 1923,'USAF' from dual union all
select 19230,'RVSN' from dual union all
select 19231,'RVSN' from dual union all
select 19232,'RVSN' from dual union all
select 19233,'RVSN' from dual union all
select 19234,'RVSN' from dual union all
select 19235,'MO' from dual union all
select 19236,'MAIO' from dual union all
select 19236,'MOM' from dual union all
select 19237,'AE' from dual union all
select 19238,'NASA' from dual union all
select 19239,'RVSN' from dual union all
select 1924,'USAF' from dual union all
select 19240,'AE' from dual union all
select 19241,'MO' from dual union all
select 19242,'USAF' from dual union all
select 19243,'PVO' from dual union all
select 19244,'PVO' from dual union all
select 19245,'USN' from dual union all
select 19246,'AE' from dual union all
select 19247,'MO' from dual union all
select 19248,'USAF' from dual union all
select 19249,'MO' from dual union all
select 1925,'USAF' from dual union all
select 19250,'NASA' from dual union all
select 19251,'NPOE' from dual union all
select 19252,'USN' from dual union all
select 19253,'NPOE' from dual union all
select 19254,'NPOE' from dual union all
select 19255,'NPOE' from dual union all
select 19256,'MO' from dual union all
select 19257,'RVSN' from dual union all
select 19258,'MO' from dual union all
select 19259,'RVSN' from dual union all
select 1926,'USAF' from dual union all
select 19260,'MO' from dual union all
select 19261,'MO' from dual union all
select 19262,'MO' from dual union all
select 19263,'MO' from dual union all
select 19264,'RVSN' from dual union all
select 19265,'MO' from dual union all
select 19266,'MO' from dual union all
select 19267,'MO' from dual union all
select 19268,'MO' from dual union all
select 19269,'MO' from dual union all
select 1927,'USAF' from dual union all
select 19270,'MO' from dual union all
select 19271,'NPOE' from dual union all
select 19272,'NPOE' from dual union all
select 19273,'MO' from dual;
insert into SATELLITE_ORG
select 19274,'GMS' from dual union all
select 19274,'YUZH' from dual union all
select 19275,'MO' from dual union all
select 19276,'MVD' from dual union all
select 19277,'RVSN' from dual union all
select 19278,'MO' from dual union all
select 19279,'MO' from dual union all
select 1928,'USAF' from dual union all
select 19280,'MO' from dual union all
select 19281,'IKI' from dual union all
select 19281,'NPOL' from dual union all
select 19282,'RVSN' from dual union all
select 19283,'RVSN' from dual union all
select 19284,'MO' from dual union all
select 19285,'MO' from dual union all
select 19286,'MO' from dual union all
select 19287,'IKI' from dual union all
select 19287,'NPOL' from dual union all
select 19288,'RVSN' from dual union all
select 19289,'RVSN' from dual union all
select 1929,'USAF' from dual union all
select 19290,'NASA' from dual union all
select 19291,'NASA' from dual union all
select 19292,'RVSN' from dual union all
select 19293,'USAF' from dual union all
select 19294,'PVO' from dual union all
select 19295,'RVSN' from dual union all
select 19296,'VMF' from dual union all
select 19297,'VMF' from dual union all
select 19298,'USN' from dual union all
select 19299,'USAF' from dual union all
select 193,'USAF' from dual union all
select 1930,'USAF' from dual union all
select 19300,'USAF' from dual union all
select 19301,'PVO' from dual union all
select 19302,'USAF' from dual union all
select 19303,'VMF' from dual union all
select 19304,'USAF' from dual union all
select 19305,'VMF' from dual union all
select 19306,'NASA' from dual union all
select 19307,'AE' from dual union all
select 19308,'VMF' from dual union all
select 19309,'PVO' from dual union all
select 1931,'USAF' from dual union all
select 19310,'VMF' from dual union all
select 19311,'NASA' from dual union all
select 19312,'AE' from dual union all
select 19313,'AE' from dual union all
select 19314,'ISAS' from dual union all
select 19315,'AE' from dual union all
select 19316,'NASA' from dual union all
select 19317,'PVO' from dual union all
select 19318,'PVO' from dual union all
select 19319,'NASA' from dual union all
select 1932,'USAF' from dual union all
select 19320,'PVO' from dual union all
select 19321,'RVSN' from dual union all
select 19322,'NPOE' from dual union all
select 19323,'RVSN' from dual union all
select 19324,'VMF' from dual union all
select 19325,'RVSN' from dual union all
select 19326,'MO' from dual union all
select 19327,'MO' from dual union all
select 19328,'MO' from dual union all
select 19329,'MO' from dual union all
select 1933,'RVSN' from dual union all
select 19330,'ISRO' from dual union all
select 19331,'EUTEL' from dual union all
select 19332,'AE' from dual union all
select 19333,'MO' from dual union all
select 19334,'MO' from dual union all
select 19335,'AE' from dual union all
select 19336,'GMS' from dual union all
select 19337,'MO' from dual union all
select 19338,'PVO' from dual union all
select 19339,'RVSN' from dual union all
select 1934,'CNES' from dual union all
select 19340,'PVO' from dual union all
select 19341,'PVO' from dual union all
select 19342,'PVO' from dual union all
select 19343,'PVO' from dual union all
select 19344,'MO' from dual union all
select 19345,'RVSN' from dual union all
select 19346,'RVSN' from dual union all
select 19347,'RVSN' from dual union all
select 19348,'RVSN' from dual union all
select 19349,'RVSN' from dual union all
select 1935,'CNES' from dual union all
select 19350,'USAF' from dual union all
select 19351,'NASA' from dual union all
select 19352,'RVSN' from dual union all
select 19353,'NASA' from dual union all
select 19354,'MO' from dual union all
select 19355,'USAF' from dual union all
select 19356,'AE' from dual union all
select 19357,'AE' from dual union all
select 19358,'USAF' from dual union all
select 19359,'NASA' from dual union all
select 1936,'RVSN' from dual union all
select 19360,'NASA' from dual;
insert into SATELLITE_ORG
select 19361,'VMF' from dual union all
select 19362,'PVO' from dual union all
select 19363,'NASA' from dual union all
select 19364,'RVSN' from dual union all
select 19365,'NASA' from dual union all
select 19366,'USN' from dual union all
select 19367,'PVO' from dual union all
select 19368,'MAI' from dual union all
select 19369,'MAI' from dual union all
select 1937,'RVSN' from dual union all
select 19370,'MAI' from dual union all
select 19371,'MAI' from dual union all
select 19372,'MO' from dual union all
select 19373,'RVSN' from dual union all
select 19374,'MO' from dual union all
select 19375,'PVO' from dual union all
select 19376,'PVO' from dual union all
select 19377,'MO' from dual union all
select 19378,'RVSN' from dual union all
select 19379,'RVSN' from dual union all
select 1938,'CNES' from dual union all
select 19380,'RVSN' from dual union all
select 19381,'MAI' from dual union all
select 19382,'MAI' from dual union all
select 19383,'MAI' from dual union all
select 19384,'MO' from dual union all
select 19385,'RVSN' from dual union all
select 19386,'NPOE' from dual union all
select 19387,'NPOE' from dual union all
select 19388,'MO' from dual union all
select 19389,'USN' from dual union all
select 1939,'NRO' from dual union all
select 19390,'MO' from dual union all
select 19391,'NASA' from dual union all
select 19392,'MO' from dual union all
select 19393,'PVO' from dual union all
select 19394,'MO' from dual union all
select 19395,'AE' from dual union all
select 19396,'NASA' from dual union all
select 19397,'MSVYAZ' from dual union all
select 19398,'RVSN' from dual union all
select 19399,'RVSN' from dual union all
select 194,'USAF' from dual union all
select 1940,'USAF' from dual union all
select 19400,'RVSN' from dual union all
select 19401,'RVSN' from dual union all
select 19402,'RVSN' from dual union all
select 19403,'MO' from dual union all
select 19404,'MO' from dual union all
select 19405,'MO' from dual union all
select 19406,'MO' from dual union all
select 19407,'MO' from dual union all
select 19408,'NPOE' from dual union all
select 19409,'NPOE' from dual union all
select 1941,'USAF' from dual union all
select 19410,'PVO' from dual union all
select 19411,'PVO' from dual union all
select 19412,'MO' from dual union all
select 19413,'RVSN' from dual union all
select 19414,'MVD' from dual union all
select 19415,'RVSN' from dual union all
select 19416,'MO' from dual union all
select 19417,'MO' from dual union all
select 19418,'MO' from dual union all
select 19419,'USN' from dual union all
select 1942,'USAF' from dual union all
select 19420,'USN' from dual union all
select 19421,'NASA' from dual union all
select 19422,'NASA' from dual union all
select 19423,'NASA' from dual union all
select 19424,'RVSN' from dual union all
select 19425,'PVO' from dual union all
select 19426,'VMF' from dual union all
select 19427,'VMF' from dual union all
select 19428,'USAF' from dual union all
select 19429,'NASA' from dual union all
select 1943,'USAF' from dual union all
select 19430,'USAF' from dual union all
select 19431,'NASA' from dual union all
select 19432,'MO' from dual union all
select 19433,'PVO' from dual union all
select 19434,'RVSN' from dual union all
select 19435,'AE' from dual union all
select 19436,'NASA' from dual union all
select 19437,'MO' from dual union all
select 19438,'RVSN' from dual union all
select 19439,'PVO' from dual union all
select 1944,'NASA' from dual union all
select 19440,'MO' from dual union all
select 19441,'USN' from dual union all
select 19442,'NPOE' from dual union all
select 19443,'NPOE' from dual union all
select 19444,'RVSN' from dual union all
select 19445,'PVO' from dual union all
select 19446,'RVSN' from dual union all
select 19447,'RVSN' from dual union all
select 19448,'RVSN' from dual union all
select 19449,'RVSN' from dual union all
select 1945,'MO' from dual union all
select 19450,'RVSN' from dual;
insert into SATELLITE_ORG
select 19451,'RVSN' from dual union all
select 19452,'MO' from dual union all
select 19453,'PVO' from dual union all
select 19454,'NASA' from dual union all
select 19455,'NASA' from dual union all
select 19456,'MO' from dual union all
select 19457,'RVSN' from dual union all
select 19458,'NRO' from dual union all
select 19459,'USAF' from dual union all
select 1946,'RVSN' from dual union all
select 19460,'NRO' from dual union all
select 19461,'USAF' from dual union all
select 19462,'MO' from dual union all
select 19463,'RVSN' from dual union all
select 19464,'NPOE' from dual union all
select 19465,'MO' from dual union all
select 19466,'MO' from dual union all
select 19467,'CMA' from dual union all
select 19468,'MAI' from dual union all
select 19469,'MO' from dual union all
select 1947,'MO' from dual union all
select 19470,'MO' from dual union all
select 19471,'MO' from dual union all
select 19472,'MO' from dual union all
select 19473,'PVO' from dual union all
select 19474,'MO' from dual union all
select 19475,'PVO' from dual union all
select 19476,'PVO' from dual union all
select 19477,'AE' from dual union all
select 19478,'NASA' from dual union all
select 19479,'PVO' from dual union all
select 1948,'NASA' from dual union all
select 19480,'PVO' from dual union all
select 19481,'RVSN' from dual union all
select 19482,'USAF' from dual union all
select 19483,'GTE' from dual union all
select 19484,'STLC' from dual union all
select 19485,'AE' from dual union all
select 19486,'NPOE' from dual union all
select 19487,'AE' from dual union all
select 19488,'MVD' from dual union all
select 19489,'RVSN' from dual union all
select 1949,'PVO' from dual union all
select 1949,'YUZH' from dual union all
select 19490,'USAF' from dual union all
select 19491,'RVSN' from dual union all
select 19492,'MO' from dual union all
select 19493,'MO' from dual union all
select 19494,'MO' from dual union all
select 19495,'MO' from dual union all
select 19496,'RVSN' from dual union all
select 19497,'MO' from dual union all
select 19498,'MO' from dual union all
select 19499,'MO' from dual union all
select 195,'USAF' from dual union all
select 1950,'RVSN' from dual union all
select 19500,'MO' from dual union all
select 19501,'MO' from dual union all
select 19502,'MO' from dual union all
select 19503,'MO' from dual union all
select 19504,'RVSN' from dual union all
select 19505,'RVSN' from dual union all
select 19506,'PVO' from dual union all
select 19507,'PVO' from dual union all
select 19508,'NASDA' from dual union all
select 19508,'TSCJ' from dual union all
select 19509,'NASDA' from dual union all
select 1951,'NASA' from dual union all
select 19510,'AE' from dual union all
select 19511,'USAF' from dual union all
select 19512,'NASA' from dual union all
select 19513,'AE' from dual union all
select 19514,'PVO' from dual union all
select 19515,'USN' from dual union all
select 19516,'USN' from dual union all
select 19517,'VMF' from dual union all
select 19518,'USAF' from dual union all
select 19519,'ISA' from dual union all
select 1952,'USN' from dual union all
select 19520,'ISA' from dual union all
select 19521,'MO' from dual union all
select 19522,'RVSN' from dual union all
select 19523,'MO' from dual union all
select 19524,'MO' from dual union all
select 19525,'MO' from dual union all
select 19526,'MO' from dual union all
select 19527,'MO' from dual union all
select 19528,'MO' from dual union all
select 19529,'MO' from dual union all
select 1953,'NASA' from dual union all
select 19530,'MO' from dual union all
select 19531,'NOAA' from dual union all
select 19532,'NOAA' from dual union all
select 19533,'NPOE' from dual union all
select 19534,'NOAA' from dual union all
select 19535,'RVSN' from dual union all
select 19536,'NPOE' from dual union all
select 19537,'RVSN' from dual union all
select 19538,'NPOE' from dual union all
select 19539,'NPOE' from dual;
insert into SATELLITE_ORG
select 1954,'IKI' from dual union all
select 1954,'NPOL' from dual union all
select 19540,'NPOE' from dual union all
select 19541,'MO' from dual union all
select 19541,'MSVYAZ' from dual union all
select 19542,'RVSN' from dual union all
select 19543,'RVSN' from dual union all
select 19544,'RVSN' from dual union all
select 19545,'PVO' from dual union all
select 19546,'PVO' from dual union all
select 19547,'NASA' from dual union all
select 19548,'NASA' from dual union all
select 19549,'NASA' from dual union all
select 1955,'RVSN' from dual union all
select 19550,'NASA' from dual union all
select 19551,'PVO' from dual union all
select 19552,'PVO' from dual union all
select 19553,'VMF' from dual union all
select 19554,'PVO' from dual union all
select 19555,'RVSN' from dual union all
select 19556,'RVSN' from dual union all
select 19557,'RVSN' from dual union all
select 19558,'NASDA' from dual union all
select 19559,'USN' from dual union all
select 1956,'USAF' from dual union all
select 19560,'AE' from dual union all
select 19561,'PVO' from dual union all
select 19562,'MO' from dual union all
select 19563,'MO' from dual union all
select 19564,'MO' from dual union all
select 19565,'MO' from dual union all
select 19566,'MO' from dual union all
select 19567,'MO' from dual union all
select 19568,'NPOE' from dual union all
select 19569,'NPOE' from dual union all
select 1957,'USAF' from dual union all
select 19570,'NPOE' from dual union all
select 19571,'NPOE' from dual union all
select 19572,'MO' from dual union all
select 19573,'VS' from dual union all
select 19574,'MO' from dual union all
select 19575,'MO' from dual union all
select 19576,'USAF' from dual union all
select 19577,'USN' from dual union all
select 19578,'RVSN' from dual union all
select 19579,'RVSN' from dual union all
select 1958,'USAF' from dual union all
select 19580,'RVSN' from dual union all
select 19581,'USAF' from dual union all
select 19582,'MO' from dual union all
select 19583,'RVSN' from dual union all
select 19584,'NPOE' from dual union all
select 19585,'NPOE' from dual union all
select 19586,'NPOE' from dual union all
select 19587,'NPOE' from dual union all
select 19588,'NPOE' from dual union all
select 19589,'VMF' from dual union all
select 1959,'USAF' from dual union all
select 19590,'NASA' from dual union all
select 19591,'MO' from dual union all
select 19592,'USAF' from dual union all
select 19593,'USAF' from dual union all
select 19594,'NASA' from dual union all
select 19595,'USAF' from dual union all
select 19596,'MO' from dual union all
select 19597,'RVSN' from dual union all
select 19598,'RVSN' from dual union all
select 19599,'NASA' from dual union all
select 196,'USAF' from dual union all
select 1960,'USAF' from dual union all
select 19600,'RVSN' from dual union all
select 19601,'RVSN' from dual union all
select 19602,'PVO' from dual union all
select 19603,'PVO' from dual union all
select 19604,'NPOE' from dual union all
select 19605,'NPOE' from dual union all
select 19606,'NPOE' from dual union all
select 19607,'NPOE' from dual union all
select 19608,'PVO' from dual union all
select 19609,'RVSN' from dual union all
select 1961,'USAF' from dual union all
select 19610,'RVSN' from dual union all
select 19611,'RVSN' from dual union all
select 19612,'MO' from dual union all
select 19613,'RVSN' from dual union all
select 19614,'MO' from dual union all
select 19615,'MO' from dual union all
select 19616,'MO' from dual union all
select 19617,'MO' from dual union all
select 19618,'NPOE' from dual union all
select 19619,'NPOE' from dual union all
select 1962,'USAF' from dual union all
select 19620,'NPOE' from dual union all
select 19621,'FTEL' from dual union all
select 19622,'AE' from dual union all
select 19623,'MO' from dual union all
select 19624,'NPOE' from dual union all
select 19625,'CIA' from dual union all
select 19625,'NRO' from dual union all
select 19626,'USAF' from dual;
insert into SATELLITE_ORG
select 19627,'MO' from dual union all
select 19628,'MO' from dual union all
select 19629,'MO' from dual union all
select 1963,'USAF' from dual union all
select 19630,'MO' from dual union all
select 19631,'MO' from dual union all
select 19632,'NPOE' from dual union all
select 19633,'NPOE' from dual union all
select 19634,'NPOE' from dual union all
select 19635,'NPOE' from dual union all
select 19636,'NPOE' from dual union all
select 19637,'MOLN' from dual union all
select 19637,'MOM' from dual union all
select 19638,'NASA' from dual union all
select 19639,'USAF' from dual union all
select 1964,'USAF' from dual union all
select 19640,'MO' from dual union all
select 19641,'MO' from dual union all
select 19642,'AE' from dual union all
select 19643,'RVSN' from dual union all
select 19644,'MO' from dual union all
select 19645,'MO' from dual union all
select 19646,'RVSN' from dual union all
select 19647,'VMF' from dual union all
select 19648,'MO' from dual union all
select 19649,'VS' from dual union all
select 1965,'USAF' from dual union all
select 19650,'MO' from dual union all
select 19651,'MO' from dual union all
select 19652,'RVSN' from dual union all
select 19653,'MO' from dual union all
select 19654,'MO' from dual union all
select 19655,'MO' from dual union all
select 19656,'MO' from dual union all
select 19657,'MO' from dual union all
select 19658,'MO' from dual union all
select 19659,'MO' from dual union all
select 1966,'RVSN' from dual union all
select 19660,'NPOE' from dual union all
select 19661,'RVSN' from dual union all
select 19662,'MO' from dual union all
select 19663,'RVSN' from dual union all
select 19664,'USAF' from dual union all
select 19665,'MONBU' from dual union all
select 19666,'RVSN' from dual union all
select 19667,'NASA' from dual union all
select 19668,'VMF' from dual union all
select 19669,'RVSN' from dual union all
select 1967,'RVSN' from dual union all
select 19670,'NASA' from dual union all
select 19671,'CIA' from dual union all
select 19671,'NRO' from dual union all
select 19672,'MO' from dual union all
select 19673,'RVSN' from dual union all
select 19674,'MO' from dual union all
select 19675,'MO' from dual union all
select 19676,'MO' from dual union all
select 19677,'MO' from dual union all
select 19678,'MO' from dual union all
select 19679,'MO' from dual union all
select 1968,'CIA' from dual union all
select 1968,'NRO' from dual union all
select 19680,'CNES' from dual union all
select 19681,'CNES' from dual union all
select 19682,'CNES' from dual union all
select 19683,'MSVYAZ' from dual union all
select 19684,'RVSN' from dual union all
select 19685,'RVSN' from dual union all
select 19686,'RVSN' from dual union all
select 19687,'MOD' from dual union all
select 19688,'SES' from dual union all
select 19689,'AE' from dual union all
select 1969,'USAF' from dual union all
select 19690,'AE' from dual union all
select 19691,'RVSN' from dual union all
select 19692,'NPOE' from dual union all
select 19693,'NPOE' from dual union all
select 19694,'MO' from dual union all
select 19695,'MO' from dual union all
select 19696,'MO' from dual union all
select 19697,'MO' from dual union all
select 19698,'MO' from dual union all
select 19699,'RVSN' from dual union all
select 197,'CIA' from dual union all
select 197,'NRO' from dual union all
select 1970,'USAF' from dual union all
select 19700,'NPOE' from dual union all
select 19701,'NPOE' from dual union all
select 19702,'NPOE' from dual union all
select 19703,'NPOE' from dual union all
select 19704,'NPOE' from dual union all
select 19705,'MO' from dual union all
select 19706,'RVSN' from dual union all
select 19707,'NPOE' from dual union all
select 19708,'NPOE' from dual union all
select 19709,'MO' from dual union all
select 1971,'USAF' from dual union all
select 19710,'CHIS' from dual union all
select 19711,'PVO' from dual union all
select 19712,'PVO' from dual;
insert into SATELLITE_ORG
select 19713,'MO' from dual union all
select 19713,'MSVYAZ' from dual union all
select 19714,'RVSN' from dual union all
select 19715,'RVSN' from dual union all
select 19716,'RVSN' from dual union all
select 19717,'MO' from dual union all
select 19718,'MO' from dual union all
select 19719,'MO' from dual union all
select 1972,'USAF' from dual union all
select 19720,'MO' from dual union all
select 19721,'PVO' from dual union all
select 19722,'RVSN' from dual union all
select 19723,'RVSN' from dual union all
select 19724,'RVSN' from dual union all
select 19725,'RVSN' from dual union all
select 19726,'PVO' from dual union all
select 19727,'PVO' from dual union all
select 19728,'NPOE' from dual union all
select 19729,'RVSN' from dual union all
select 1973,'USAF' from dual union all
select 19730,'MO' from dual union all
select 19731,'RVSN' from dual union all
select 19732,'RVSN' from dual union all
select 19733,'RVSN' from dual union all
select 19734,'VTS' from dual union all
select 19735,'RVSN' from dual union all
select 19736,'NPOE' from dual union all
select 19737,'NPOE' from dual union all
select 19738,'NPOE' from dual union all
select 19739,'NPOE' from dual union all
select 1974,'USAF' from dual union all
select 19740,'NPOE' from dual union all
select 19741,'NPOE' from dual union all
select 19742,'RVSN' from dual union all
select 19743,'NPOE' from dual union all
select 19744,'NPOE' from dual union all
select 19745,'NPOE' from dual union all
select 19746,'NPOE' from dual union all
select 19747,'NPOE' from dual union all
select 19748,'RVSN' from dual union all
select 19749,'MO' from dual union all
select 1975,'USAF' from dual union all
select 19750,'MO' from dual union all
select 19751,'MOM' from dual union all
select 19752,'RVSN' from dual union all
select 19753,'MOM' from dual union all
select 19754,'RVSN' from dual union all
select 19755,'RVSN' from dual union all
select 19756,'MVD' from dual union all
select 19757,'RVSN' from dual union all
select 19758,'MO' from dual union all
select 19759,'RVSN' from dual union all
select 1976,'USAF' from dual union all
select 19760,'MO' from dual union all
select 19761,'MO' from dual union all
select 19762,'MO' from dual union all
select 19763,'PVO' from dual union all
select 19764,'MO' from dual union all
select 19765,'MSVYAZ' from dual union all
select 19766,'RVSN' from dual union all
select 19767,'RVSN' from dual union all
select 19768,'RVSN' from dual union all
select 19769,'MO' from dual union all
select 1977,'USAF' from dual union all
select 19770,'RVSN' from dual union all
select 19771,'RVSN' from dual union all
select 19772,'INTEL' from dual union all
select 19773,'AE' from dual union all
select 19774,'MO' from dual union all
select 19775,'RVSN' from dual union all
select 19776,'RVSN' from dual union all
select 19777,'RVSN' from dual union all
select 19778,'MO' from dual union all
select 19779,'MO' from dual union all
select 1978,'USAF' from dual union all
select 19780,'MO' from dual union all
select 19781,'MO' from dual union all
select 19782,'MO' from dual union all
select 19783,'NPOE' from dual union all
select 19784,'RVSN' from dual union all
select 19785,'MO' from dual union all
select 19786,'MO' from dual union all
select 19787,'MO' from dual union all
select 19788,'MO' from dual union all
select 19789,'MO' from dual union all
select 1979,'USAF' from dual union all
select 19790,'MO' from dual union all
select 19791,'MO' from dual union all
select 19792,'MO' from dual union all
select 19793,'RVSN' from dual union all
select 19794,'MO' from dual union all
select 19795,'MO' from dual union all
select 19796,'PVO' from dual union all
select 19797,'RVSN' from dual union all
select 19798,'RVSN' from dual union all
select 19799,'RVSN' from dual union all
select 198,'USAF' from dual union all
select 1980,'USAF' from dual union all
select 19800,'PVO' from dual union all
select 19801,'RVSN' from dual;
insert into SATELLITE_ORG
select 19802,'USAF' from dual union all
select 19803,'MDC' from dual union all
select 19804,'MDC' from dual union all
select 19805,'PVO' from dual union all
select 19806,'PVO' from dual union all
select 19807,'MO' from dual union all
select 19808,'RVSN' from dual union all
select 19809,'RVSN' from dual union all
select 1981,'USAF' from dual union all
select 19810,'RVSN' from dual union all
select 19811,'RVSN' from dual union all
select 19812,'AE' from dual union all
select 19813,'RVSN' from dual union all
select 19814,'USAF' from dual union all
select 19815,'MO' from dual union all
select 19816,'NPOE' from dual union all
select 19817,'PVO' from dual union all
select 19818,'MO' from dual union all
select 19819,'RVSN' from dual union all
select 1982,'ESSA' from dual union all
select 19820,'MO' from dual union all
select 19821,'PVO' from dual union all
select 19822,'MONBU' from dual union all
select 19823,'MONBU' from dual union all
select 19824,'MONBU' from dual union all
select 19825,'PVO' from dual union all
select 19826,'VMF' from dual union all
select 19827,'RVSN' from dual union all
select 19828,'USAF' from dual union all
select 19829,'PVO' from dual union all
select 1983,'NASA' from dual union all
select 19830,'RVSN' from dual union all
select 19831,'MO' from dual union all
select 19832,'PVO' from dual union all
select 19833,'MO' from dual union all
select 19834,'NASA' from dual union all
select 19835,'RVSN' from dual union all
select 19836,'MO' from dual union all
select 19837,'MO' from dual union all
select 19838,'USN' from dual union all
select 19839,'MO' from dual union all
select 1984,'USAF' from dual union all
select 19840,'MO' from dual union all
select 19841,'MONBU' from dual union all
select 19842,'MONBU' from dual union all
select 19843,'MONBU' from dual union all
select 19844,'MONBU' from dual union all
select 19845,'MONBU' from dual union all
select 19846,'PVO' from dual union all
select 19847,'PVO' from dual union all
select 19848,'PVO' from dual union all
select 19849,'PVO' from dual union all
select 1985,'USAF' from dual union all
select 19850,'PVO' from dual union all
select 19851,'GMS' from dual union all
select 19851,'MO' from dual union all
select 19852,'MO' from dual union all
select 19853,'MO' from dual union all
select 19854,'MO' from dual union all
select 19855,'AE' from dual union all
select 19856,'RVSN' from dual union all
select 19857,'AE' from dual union all
select 19858,'NASA' from dual union all
select 19859,'USAF' from dual union all
select 1986,'USAF' from dual union all
select 19860,'USAF' from dual union all
select 19861,'MO' from dual union all
select 19862,'MO' from dual union all
select 19863,'RVSN' from dual union all
select 19864,'MO' from dual union all
select 19865,'MO' from dual union all
select 19866,'MO' from dual union all
select 19867,'MO' from dual union all
select 19868,'MO' from dual union all
select 19869,'MO' from dual union all
select 1987,'USAF' from dual union all
select 19870,'MO' from dual union all
select 19871,'NPOE' from dual union all
select 19872,'NPOE' from dual union all
select 19873,'MO' from dual union all
select 19874,'JCSAT' from dual union all
select 19875,'MO' from dual union all
select 19876,'EUMET' from dual union all
select 19877,'AE' from dual union all
select 19878,'AE' from dual union all
select 19879,'PVO' from dual union all
select 1988,'USAF' from dual union all
select 19880,'USAF' from dual union all
select 19881,'PVO' from dual union all
select 19882,'NASA' from dual union all
select 19883,'NASA' from dual union all
select 19884,'NASA' from dual union all
select 19885,'PVO' from dual union all
select 19886,'PVO' from dual union all
select 19887,'AE' from dual union all
select 19888,'MO' from dual union all
select 19889,'MONBU' from dual union all
select 1989,'USAF' from dual union all
select 19890,'MO' from dual union all
select 19891,'USAF' from dual;
insert into SATELLITE_ORG
select 19892,'AE' from dual union all
select 19893,'MO' from dual union all
select 19894,'RVSN' from dual union all
select 19895,'NPOE' from dual union all
select 19896,'RVSN' from dual union all
select 19897,'MO' from dual union all
select 19898,'MO' from dual union all
select 19899,'MO' from dual union all
select 199,'USAF' from dual union all
select 1990,'USAF' from dual union all
select 19900,'MO' from dual union all
select 19901,'RVSN' from dual union all
select 19902,'MO' from dual union all
select 19903,'MO' from dual union all
select 19904,'MO' from dual union all
select 19905,'MO' from dual union all
select 19906,'MO' from dual union all
select 19907,'MO' from dual union all
select 19908,'MO' from dual union all
select 19909,'MO' from dual union all
select 1991,'USAF' from dual union all
select 19910,'RVSN' from dual union all
select 19911,'SDIO' from dual union all
select 19912,'PVO' from dual union all
select 19913,'NASA' from dual union all
select 19914,'MO' from dual union all
select 19915,'MO' from dual union all
select 19916,'MO' from dual union all
select 19917,'MO' from dual union all
select 19918,'MO' from dual union all
select 19919,'NSAB' from dual union all
select 1992,'USAF' from dual union all
select 19920,'AE' from dual union all
select 19921,'VMF' from dual union all
select 19922,'RVSN' from dual union all
select 19923,'MO' from dual union all
select 19924,'RVSN' from dual union all
select 19925,'MO' from dual union all
select 19926,'MO' from dual union all
select 19927,'MO' from dual union all
select 19928,'MO' from dual union all
select 19929,'RVSN' from dual union all
select 1993,'USAF' from dual union all
select 19930,'RVSN' from dual union all
select 19931,'RVSN' from dual union all
select 19932,'RVSN' from dual union all
select 19933,'RVSN' from dual union all
select 19934,'MO' from dual union all
select 19935,'MO' from dual union all
select 19936,'MO' from dual union all
select 19937,'MO' from dual union all
select 19938,'MO' from dual union all
select 19939,'RVSN' from dual union all
select 1994,'NASA' from dual union all
select 19940,'NPOE' from dual union all
select 19941,'TSSKB' from dual union all
select 19942,'RVSN' from dual union all
select 19943,'PVO' from dual union all
select 19944,'PVO' from dual union all
select 19945,'MO' from dual union all
select 19946,'USAF' from dual union all
select 19947,'VMF' from dual union all
select 19948,'NASA' from dual union all
select 19949,'PVO' from dual union all
select 1995,'NASA' from dual union all
select 19950,'USAF' from dual union all
select 19951,'AE' from dual union all
select 19952,'MONBU' from dual union all
select 19953,'NASA' from dual union all
select 19954,'USAF' from dual union all
select 19955,'MO' from dual union all
select 19956,'RVSN' from dual union all
select 19957,'MO' from dual union all
select 19958,'PVO' from dual union all
select 19959,'PVO' from dual union all
select 1996,'CNES' from dual union all
select 19960,'PVO' from dual union all
select 19961,'USAF' from dual union all
select 19962,'MONBU' from dual union all
select 19963,'MONBU' from dual union all
select 19964,'USAF' from dual union all
select 19965,'NASA' from dual union all
select 19966,'AE' from dual union all
select 19967,'NASA' from dual union all
select 19968,'NASA' from dual union all
select 19969,'JPL' from dual union all
select 19969,'NASA' from dual union all
select 1997,'NRO' from dual union all
select 19970,'NASA' from dual union all
select 19971,'NASA' from dual union all
select 19972,'MO' from dual union all
select 19973,'RVSN' from dual union all
select 19974,'MO' from dual union all
select 19975,'MO' from dual union all
select 19976,'NRO' from dual union all
select 19977,'USAF' from dual union all
select 19978,'MO' from dual union all
select 19979,'MO' from dual union all
select 1998,'MO' from dual union all
select 19980,'PVO' from dual;
insert into SATELLITE_ORG
select 19981,'USAF' from dual union all
select 19982,'MO' from dual union all
select 19983,'USAF' from dual union all
select 19984,'MO' from dual union all
select 19985,'MO' from dual union all
select 19986,'MO' from dual union all
select 19987,'RVSN' from dual union all
select 19988,'MO' from dual union all
select 19989,'MO' from dual union all
select 1999,'USAF' from dual union all
select 19990,'RVSN' from dual union all
select 19991,'RVSN' from dual union all
select 19992,'RVSN' from dual union all
select 19993,'RVSN' from dual union all
select 19994,'RVSN' from dual union all
select 19995,'RVSN' from dual union all
select 19996,'RVSN' from dual union all
select 19997,'RVSN' from dual union all
select 19998,'RVSN' from dual union all
select 19999,'AE' from dual union all
select 2,'OKB1' from dual union all
select 20,'NASA' from dual union all
select 200,'USAF' from dual union all
select 2000,'RVSN' from dual union all
select 20000,'VTS' from dual union all
select 20001,'RVSN' from dual union all
select 20002,'VTS' from dual union all
select 20003,'MO' from dual union all
select 20004,'MO' from dual union all
select 20005,'USAF' from dual union all
select 20006,'MVD' from dual union all
select 20007,'RVSN' from dual union all
select 20008,'NASA' from dual union all
select 20009,'USN' from dual union all
select 2001,'RVSN' from dual union all
select 20010,'RVSN' from dual union all
select 20011,'PVO' from dual union all
select 20012,'NASA' from dual union all
select 20013,'PVO' from dual union all
select 20014,'PVO' from dual union all
select 20015,'PVO' from dual union all
select 20016,'RVSN' from dual union all
select 20017,'USAF' from dual union all
select 20018,'USAF' from dual union all
select 20019,'MO' from dual union all
select 2002,'YUZH' from dual union all
select 20020,'MO' from dual union all
select 20021,'MONBU' from dual union all
select 20022,'USAF' from dual union all
select 20023,'USAF' from dual union all
select 20024,'MO' from dual union all
select 20025,'MO' from dual union all
select 20026,'MOM' from dual union all
select 20027,'RVSN' from dual union all
select 20028,'RVSN' from dual union all
select 20029,'NASA' from dual union all
select 2003,'USAF' from dual union all
select 20030,'PVO' from dual union all
select 20031,'USAF' from dual union all
select 20032,'PVO' from dual union all
select 20033,'USAF' from dual union all
select 20034,'MONBU' from dual union all
select 20035,'MO' from dual union all
select 20036,'RVSN' from dual union all
select 20037,'MO' from dual union all
select 20038,'MO' from dual union all
select 20039,'MO' from dual union all
select 2004,'USAF' from dual union all
select 20040,'SCC' from dual union all
select 20041,'DBP' from dual union all
select 20042,'AE' from dual union all
select 20043,'AE' from dual union all
select 20044,'MOM' from dual union all
select 20045,'VMF' from dual union all
select 20046,'RVSN' from dual union all
select 20047,'NASA' from dual union all
select 20048,'USAF' from dual union all
select 20049,'PVO' from dual union all
select 2005,'MO' from dual union all
select 20050,'AE' from dual union all
select 20051,'NASA' from dual union all
select 20052,'MO' from dual union all
select 20052,'MSVYAZ' from dual union all
select 20053,'RVSN' from dual union all
select 20054,'RVSN' from dual union all
select 20055,'RVSN' from dual union all
select 20056,'KAIK' from dual union all
select 20056,'MOM' from dual union all
select 20057,'RVSN' from dual union all
select 20058,'RVSN' from dual union all
select 20059,'RVSN' from dual union all
select 2006,'MO' from dual union all
select 20060,'KAIK' from dual union all
select 20060,'MOM' from dual union all
select 20061,'USAF' from dual union all
select 20062,'MDC' from dual union all
select 20063,'MDC' from dual union all
select 20064,'PVO' from dual union all
select 20065,'RVSN' from dual union all
select 20066,'USAF' from dual;
insert into SATELLITE_ORG
select 20067,'USAF' from dual union all
select 20068,'USAF' from dual union all
select 20069,'USAF' from dual union all
select 2007,'RVSN' from dual union all
select 20070,'MO' from dual union all
select 20071,'MO' from dual union all
select 20072,'MO' from dual union all
select 20073,'MO' from dual union all
select 20074,'RVSN' from dual union all
select 20075,'MO' from dual union all
select 20076,'MO' from dual union all
select 20077,'MO' from dual union all
select 20078,'MO' from dual union all
select 20079,'MO' from dual union all
select 2008,'YUZH' from dual union all
select 20080,'MO' from dual union all
select 20081,'RVSN' from dual union all
select 20082,'RVSN' from dual union all
select 20083,'MO' from dual union all
select 20084,'RVSN' from dual union all
select 20085,'RVSN' from dual union all
select 20086,'RVSN' from dual union all
select 20087,'MO' from dual union all
select 20088,'NASA' from dual union all
select 20089,'MO' from dual union all
select 2009,'USAF' from dual union all
select 20090,'PVO' from dual union all
select 20091,'PVO' from dual union all
select 20092,'MO' from dual union all
select 20093,'RVSN' from dual union all
select 20094,'RVSN' from dual union all
select 20095,'MVD' from dual union all
select 20096,'RVSN' from dual union all
select 20097,'PVO' from dual union all
select 20098,'PVO' from dual union all
select 20099,'PVO' from dual union all
select 201,'CIA' from dual union all
select 201,'NRO' from dual union all
select 2010,'USAF' from dual union all
select 20100,'MO' from dual union all
select 20101,'RVSN' from dual union all
select 20102,'USAF' from dual union all
select 20103,'MO' from dual union all
select 20103,'VMF' from dual union all
select 20104,'RVSN' from dual union all
select 20105,'MO' from dual union all
select 20106,'RVSN' from dual union all
select 20107,'MSVYAZ' from dual union all
select 20108,'RVSN' from dual union all
select 20109,'RVSN' from dual union all
select 2011,'NASA' from dual union all
select 20110,'RVSN' from dual union all
select 20111,'MO' from dual union all
select 20112,'MO' from dual union all
select 20113,'MO' from dual union all
select 20114,'MO' from dual union all
select 20115,'RVSN' from dual union all
select 20116,'RVSN' from dual union all
select 20117,'MO' from dual union all
select 20118,'MO' from dual union all
select 20119,'MO' from dual union all
select 2012,'NRO' from dual union all
select 20120,'MO' from dual union all
select 20121,'MO' from dual union all
select 20122,'ESA' from dual union all
select 20123,'AE' from dual union all
select 20124,'MO' from dual union all
select 20125,'RVSN' from dual union all
select 20126,'MO' from dual union all
select 20127,'AE' from dual union all
select 20128,'RVSN' from dual union all
select 20129,'RVSN' from dual union all
select 2013,'NASA' from dual union all
select 2013,'USN' from dual union all
select 20130,'MO' from dual union all
select 20131,'USAF' from dual union all
select 20132,'AE' from dual union all
select 20133,'VMF' from dual union all
select 20134,'MVD' from dual union all
select 20135,'RVSN' from dual union all
select 20136,'TSSKB' from dual union all
select 20137,'RVSN' from dual union all
select 20138,'MO' from dual union all
select 20139,'MO' from dual union all
select 2014,'USAF' from dual union all
select 20140,'MO' from dual union all
select 20141,'MO' from dual union all
select 20142,'MO' from dual union all
select 20143,'MO' from dual union all
select 20144,'MO' from dual union all
select 20145,'MO' from dual union all
select 20146,'RVSN' from dual union all
select 20147,'VMF' from dual union all
select 20148,'MO' from dual union all
select 20149,'VMF' from dual union all
select 2015,'USAF' from dual union all
select 20150,'RVSN' from dual union all
select 20151,'MO' from dual union all
select 20152,'RVSN' from dual union all
select 20153,'USAF' from dual;
insert into SATELLITE_ORG
select 20154,'MO' from dual union all
select 20155,'MO' from dual union all
select 20156,'MO' from dual union all
select 20157,'MO' from dual union all
select 20158,'MO' from dual union all
select 20159,'MO' from dual union all
select 2016,'CNES' from dual union all
select 20160,'KAIK' from dual union all
select 20160,'MOM' from dual union all
select 20161,'KAIK' from dual union all
select 20161,'MOM' from dual union all
select 20162,'MO' from dual union all
select 20163,'MO' from dual union all
select 20164,'NASA' from dual union all
select 20165,'MO' from dual union all
select 20166,'MO' from dual union all
select 20167,'NROC' from dual union all
select 20167,'USAF' from dual union all
select 20168,'DBP' from dual union all
select 20169,'ESA' from dual union all
select 2017,'CNES' from dual union all
select 20170,'AE' from dual union all
select 20171,'AE' from dual union all
select 20172,'DOD' from dual union all
select 20173,'AE' from dual union all
select 20174,'USAF' from dual union all
select 20175,'MVD' from dual union all
select 20176,'RVSN' from dual union all
select 20177,'MO' from dual union all
select 20178,'MO' from dual union all
select 20179,'MO' from dual union all
select 2018,'CNES' from dual union all
select 20180,'MO' from dual union all
select 20181,'PVO' from dual union all
select 20182,'PVO' from dual union all
select 20183,'PVO' from dual union all
select 20184,'PVO' from dual union all
select 20185,'USAF' from dual union all
select 20186,'MDC' from dual union all
select 20187,'MDC' from dual union all
select 20188,'MO' from dual union all
select 20189,'RVSN' from dual union all
select 2019,'MO' from dual union all
select 20190,'MO' from dual union all
select 20191,'NPOE' from dual union all
select 20192,'RVSN' from dual union all
select 20193,'BSB' from dual union all
select 20194,'MDC' from dual union all
select 20195,'MDC' from dual union all
select 20196,'VTS' from dual union all
select 20197,'MO' from dual union all
select 20198,'MO' from dual union all
select 20199,'MO' from dual union all
select 202,'USN' from dual union all
select 2020,'CNES' from dual union all
select 20200,'PVO' from dual union all
select 20201,'MO' from dual union all
select 20202,'DOD' from dual union all
select 20202,'USAF' from dual union all
select 20203,'DOD' from dual union all
select 20203,'USAF' from dual union all
select 20204,'USAF' from dual union all
select 20205,'USAF' from dual union all
select 20206,'MO' from dual union all
select 20207,'MO' from dual union all
select 20208,'MO' from dual union all
select 20209,'MO' from dual union all
select 2021,'CNES' from dual union all
select 20210,'MO' from dual union all
select 20211,'MO' from dual union all
select 20212,'MO' from dual union all
select 20213,'MO' from dual union all
select 20214,'MO' from dual union all
select 20215,'MO' from dual union all
select 20216,'MO' from dual union all
select 20217,'NASDA' from dual union all
select 20218,'NPOE' from dual union all
select 20219,'RVSN' from dual union all
select 2022,'RVSN' from dual union all
select 20220,'NRO' from dual union all
select 20221,'USAF' from dual union all
select 20222,'MVD' from dual union all
select 20223,'RVSN' from dual union all
select 20224,'RVSN' from dual union all
select 20225,'MO' from dual union all
select 20226,'MO' from dual union all
select 20227,'MO' from dual union all
select 20228,'MO' from dual union all
select 20229,'AE' from dual union all
select 2023,'CNES' from dual union all
select 20230,'NASDA' from dual union all
select 20231,'MO' from dual union all
select 20232,'MO' from dual union all
select 20233,'MO' from dual union all
select 20234,'MO' from dual union all
select 20235,'MO' from dual union all
select 20236,'MO' from dual union all
select 20237,'MO' from dual union all
select 20238,'MO' from dual union all
select 20239,'MO' from dual;
insert into SATELLITE_ORG
select 2024,'MO' from dual union all
select 20240,'MO' from dual union all
select 20241,'MO' from dual union all
select 20242,'IMBP' from dual union all
select 20242,'TSSKB' from dual union all
select 20243,'RVSN' from dual union all
select 20244,'MO' from dual union all
select 20245,'RVSN' from dual union all
select 20246,'MO' from dual union all
select 20247,'MO' from dual union all
select 20248,'MO' from dual union all
select 20249,'MO' from dual union all
select 2025,'USAF' from dual union all
select 20250,'MO' from dual union all
select 20251,'MO' from dual union all
select 20252,'MO' from dual union all
select 20253,'USN' from dual union all
select 20254,'NASA' from dual union all
select 20255,'MO' from dual union all
select 20256,'RVSN' from dual union all
select 20257,'RVSN' from dual union all
select 20258,'RVSN' from dual union all
select 20259,'VMF' from dual union all
select 2026,'USAF' from dual union all
select 20260,'MO' from dual union all
select 20261,'IK' from dual union all
select 20262,'MO' from dual union all
select 20263,'MSVYAZ' from dual union all
select 20264,'RVSN' from dual union all
select 20265,'RVSN' from dual union all
select 20266,'RVSN' from dual union all
select 20267,'MO' from dual union all
select 20268,'MO' from dual union all
select 20269,'MO' from dual union all
select 2027,'USAF' from dual union all
select 20270,'RVSN' from dual union all
select 20271,'RVSN' from dual union all
select 20272,'PVO' from dual union all
select 20273,'PVO' from dual union all
select 20274,'MO' from dual union all
select 20275,'MO' from dual union all
select 20276,'MO' from dual union all
select 20277,'MO' from dual union all
select 20278,'MO' from dual union all
select 20279,'MO' from dual union all
select 2028,'USAF' from dual union all
select 20280,'RVSN' from dual union all
select 20281,'CSSR' from dual union all
select 20282,'RVSN' from dual union all
select 20283,'PVO' from dual union all
select 20284,'RVSN' from dual union all
select 20285,'MO' from dual union all
select 20286,'PVO' from dual union all
select 20287,'PVO' from dual union all
select 20288,'PVO' from dual union all
select 20289,'PVO' from dual union all
select 2029,'USAF' from dual union all
select 20290,'PVO' from dual union all
select 20291,'PVO' from dual union all
select 20292,'MO' from dual union all
select 20293,'RVSN' from dual union all
select 20294,'MO' from dual union all
select 20295,'MO' from dual union all
select 20296,'MO' from dual union all
select 20297,'NASA' from dual union all
select 20298,'JPL' from dual union all
select 20298,'NASA' from dual union all
select 20299,'NASA' from dual union all
select 203,'USAF' from dual union all
select 2030,'USAF' from dual union all
select 20300,'NASA' from dual union all
select 20301,'MO' from dual union all
select 20302,'USAF' from dual union all
select 20303,'MDC' from dual union all
select 20304,'MDC' from dual union all
select 20305,'GMS' from dual union all
select 20306,'MO' from dual union all
select 20307,'RVSN' from dual union all
select 20308,'RVSN' from dual union all
select 20309,'NASA' from dual union all
select 2031,'USAF' from dual union all
select 20310,'MO' from dual union all
select 20311,'MO' from dual union all
select 20312,'MO' from dual union all
select 20313,'MO' from dual union all
select 20314,'MO' from dual union all
select 20315,'INTEL' from dual union all
select 20316,'AE' from dual union all
select 20317,'NASDA' from dual union all
select 20318,'RVSN' from dual union all
select 20319,'USAF' from dual union all
select 2032,'USAF' from dual union all
select 20320,'MO' from dual union all
select 20321,'RVSN' from dual union all
select 20322,'NASA' from dual union all
select 20323,'MDC' from dual union all
select 20324,'NASA' from dual union all
select 20325,'NASA' from dual union all
select 20326,'NASA' from dual union all
select 20327,'PVO' from dual;
insert into SATELLITE_ORG
select 20328,'NASA' from dual union all
select 20329,'NASA' from dual union all
select 2033,'USAF' from dual union all
select 20330,'PVO' from dual union all
select 20331,'RVSN' from dual union all
select 20332,'RVSN' from dual union all
select 20333,'RVSN' from dual union all
select 20334,'VMF' from dual union all
select 20335,'NPOE' from dual union all
select 20336,'RVSN' from dual union all
select 20337,'MO' from dual union all
select 20338,'MO' from dual union all
select 20338,'MSVYAZ' from dual union all
select 20339,'RVSN' from dual union all
select 2034,'USAF' from dual union all
select 20340,'RVSN' from dual union all
select 20341,'RVSN' from dual union all
select 20342,'PVO' from dual union all
select 20343,'PVO' from dual union all
select 20344,'USAF' from dual union all
select 20345,'PVO' from dual union all
select 20346,'PVO' from dual union all
select 20347,'RVSN' from dual union all
select 20348,'RVSN' from dual union all
select 20349,'NASA' from dual union all
select 2035,'USAF' from dual union all
select 20350,'MO' from dual union all
select 20351,'RVSN' from dual union all
select 20352,'IKI' from dual union all
select 20352,'NPOL' from dual union all
select 20353,'RVSN' from dual union all
select 20354,'RVSN' from dual union all
select 20355,'NRO' from dual union all
select 20356,'USAF' from dual union all
select 20357,'USAF' from dual union all
select 20358,'RVSN' from dual union all
select 20359,'RVSN' from dual union all
select 2036,'USAF' from dual union all
select 20360,'NASA' from dual union all
select 20361,'USAF' from dual union all
select 20362,'MDC' from dual union all
select 20363,'MDC' from dual union all
select 20364,'USAF' from dual union all
select 20365,'PVO' from dual union all
select 20366,'PVO' from dual union all
select 20367,'MO' from dual union all
select 20368,'RVSN' from dual union all
select 20369,'RVSN' from dual union all
select 2037,'USAF' from dual union all
select 20370,'RVSN' from dual union all
select 20371,'RVSN' from dual union all
select 20372,'RVSN' from dual union all
select 20373,'NPOE' from dual union all
select 20374,'RVSN' from dual union all
select 20375,'PVO' from dual union all
select 20376,'PVO' from dual union all
select 20377,'PVO' from dual union all
select 20378,'PVO' from dual union all
select 20379,'PVO' from dual union all
select 2038,'USAF' from dual union all
select 20380,'PVO' from dual union all
select 20381,'PVO' from dual union all
select 20382,'PVO' from dual union all
select 20383,'PVO' from dual union all
select 20384,'PVO' from dual union all
select 20385,'PVO' from dual union all
select 20386,'PVO' from dual union all
select 20387,'PVO' from dual union all
select 20388,'PVO' from dual union all
select 20389,'PVO' from dual union all
select 2039,'USAF' from dual union all
select 20390,'MO' from dual union all
select 20391,'MO' from dual union all
select 20392,'RVSN' from dual union all
select 20393,'RVSN' from dual union all
select 20394,'RVSN' from dual union all
select 20395,'PVO' from dual union all
select 20396,'PVO' from dual union all
select 20397,'PVO' from dual union all
select 20398,'PVO' from dual union all
select 20399,'RVSN' from dual union all
select 204,'USAF' from dual union all
select 2040,'USAF' from dual union all
select 20400,'RVSN' from dual union all
select 20401,'MOD' from dual union all
select 20402,'JCSAT' from dual union all
select 20403,'GD' from dual union all
select 20404,'MOD' from dual union all
select 20405,'GD' from dual union all
select 20406,'JCSAT' from dual union all
select 20407,'NPOE' from dual union all
select 20408,'PVO' from dual union all
select 20409,'NASA' from dual union all
select 2041,'USAF' from dual union all
select 20410,'HEC' from dual union all
select 20411,'HEC' from dual union all
select 20412,'VMF' from dual union all
select 20413,'RVSN' from dual union all
select 20414,'NPOE' from dual union all
select 20415,'NPOE' from dual;
insert into SATELLITE_ORG
select 20416,'NPOE' from dual union all
select 20417,'NPOE' from dual union all
select 20418,'NPOE' from dual union all
select 20419,'NPOE' from dual union all
select 2042,'USAF' from dual union all
select 20420,'NPOE' from dual union all
select 20421,'NPOE' from dual union all
select 20422,'PVO' from dual union all
select 20423,'PVO' from dual union all
select 20424,'PVO' from dual union all
select 20425,'PVO' from dual union all
select 20426,'MO' from dual union all
select 20427,'RVSN' from dual union all
select 20428,'MO' from dual union all
select 20429,'MO' from dual union all
select 2043,'USAF' from dual union all
select 20430,'PVO' from dual union all
select 20431,'PVO' from dual union all
select 20432,'MO' from dual union all
select 20433,'RVSN' from dual union all
select 20434,'MO' from dual union all
select 20435,'MO' from dual union all
select 20436,'CNES' from dual union all
select 20437,'SURREY' from dual union all
select 20438,'SURREY' from dual union all
select 20439,'AMNA' from dual union all
select 2044,'USAF' from dual union all
select 20440,'AMSAT' from dual union all
select 20441,'WEB' from dual union all
select 20442,'AMSAT' from dual union all
select 20443,'AE' from dual union all
select 20444,'MO' from dual union all
select 20444,'MSVYAZ' from dual union all
select 20445,'RVSN' from dual union all
select 20446,'RVSN' from dual union all
select 20447,'RVSN' from dual union all
select 20448,'ISASS' from dual union all
select 20449,'ISASS' from dual union all
select 2045,'USAF' from dual union all
select 20450,'MDC' from dual union all
select 20451,'ISASS' from dual union all
select 20452,'USAF' from dual union all
select 20453,'MDC' from dual union all
select 20454,'MO' from dual union all
select 20455,'USAF' from dual union all
select 20456,'NASA' from dual union all
select 20457,'MO' from dual union all
select 20458,'RVSN' from dual union all
select 20459,'NPOE' from dual union all
select 2046,'USAF' from dual union all
select 20460,'NPOE' from dual union all
select 20461,'NPOE' from dual union all
select 20462,'MO' from dual union all
select 20463,'MO' from dual union all
select 20464,'MO' from dual union all
select 20465,'VS' from dual union all
select 20466,'MO' from dual union all
select 20467,'PVO' from dual union all
select 20468,'PVO' from dual union all
select 20469,'NASA' from dual union all
select 2047,'USAF' from dual union all
select 20470,'USN' from dual union all
select 20471,'MO' from dual union all
select 20472,'NPOE' from dual union all
select 20473,'CHIS' from dual union all
select 20474,'MAI' from dual union all
select 20475,'NPOE' from dual union all
select 20476,'PVO' from dual union all
select 20477,'RVSN' from dual union all
select 20478,'NASDA' from dual union all
select 20479,'NASDA' from dual union all
select 2048,'USAF' from dual union all
select 20480,'JARL' from dual union all
select 20481,'PVO' from dual union all
select 20482,'PVO' from dual union all
select 20483,'PVO' from dual union all
select 20484,'PVO' from dual union all
select 20485,'PVO' from dual union all
select 20486,'PVO' from dual union all
select 20487,'PVO' from dual union all
select 20488,'AE' from dual union all
select 20489,'USAF' from dual union all
select 2049,'USAF' from dual union all
select 20490,'PVO' from dual union all
select 20491,'NASDA' from dual union all
select 20492,'PVO' from dual union all
select 20493,'PVO' from dual union all
select 20494,'NPOE' from dual union all
select 20495,'RVSN' from dual union all
select 20496,'SDIO' from dual union all
select 20497,'SDIO' from dual union all
select 20498,'MDC' from dual union all
select 20499,'MO' from dual union all
select 205,'USN' from dual union all
select 2050,'USAF' from dual union all
select 20500,'RVSN' from dual union all
select 20501,'RVSN' from dual union all
select 20502,'RVSN' from dual union all
select 20503,'PVO' from dual union all
select 20504,'USAF' from dual;
insert into SATELLITE_ORG
select 20505,'MO' from dual union all
select 20506,'RVSN' from dual union all
select 20507,'RVSN' from dual union all
select 20508,'MO' from dual union all
select 20508,'VMF' from dual union all
select 20509,'RVSN' from dual union all
select 2051,'USAF' from dual union all
select 20510,'GMS' from dual union all
select 20510,'YUZH' from dual union all
select 20511,'MO' from dual union all
select 20512,'NASA' from dual union all
select 20513,'NPOE' from dual union all
select 20514,'RVSN' from dual union all
select 20515,'PVO' from dual union all
select 20516,'NRO' from dual union all
select 20517,'NRO' from dual union all
select 20518,'NRO' from dual union all
select 20519,'NRO' from dual union all
select 2052,'USAF' from dual union all
select 20520,'NRO' from dual union all
select 20521,'NRO' from dual union all
select 20522,'PVO' from dual union all
select 20523,'INTEL' from dual union all
select 20524,'INTEL' from dual union all
select 20525,'VMF' from dual union all
select 20526,'MO' from dual union all
select 20527,'VMF' from dual union all
select 20528,'RVSN' from dual union all
select 20529,'MO' from dual union all
select 2053,'USAF' from dual union all
select 20530,'RVSN' from dual union all
select 20531,'PVO' from dual union all
select 20532,'PVO' from dual union all
select 20533,'USAF' from dual union all
select 20534,'MDC' from dual union all
select 20535,'MDC' from dual union all
select 20536,'PVO' from dual union all
select 20537,'RVSN' from dual union all
select 20538,'RVSN' from dual union all
select 20539,'RVSN' from dual union all
select 2054,'USAF' from dual union all
select 20540,'ISA' from dual union all
select 20541,'ISA' from dual union all
select 20542,'MO' from dual union all
select 20543,'MO' from dual union all
select 20544,'MO' from dual union all
select 20545,'MO' from dual union all
select 20546,'NASA' from dual union all
select 20547,'DOD' from dual union all
select 20547,'USN' from dual union all
select 20548,'MO' from dual union all
select 20549,'MO' from dual union all
select 2055,'USAF' from dual union all
select 20550,'MO' from dual union all
select 20551,'MO' from dual union all
select 20552,'MO' from dual union all
select 20553,'MO' from dual union all
select 20554,'MO' from dual union all
select 20555,'MO' from dual union all
select 20556,'MO' from dual union all
select 20557,'RVSN' from dual union all
select 20558,'ASIA' from dual union all
select 20559,'MAI' from dual union all
select 2056,'USAF' from dual union all
select 20560,'USAF' from dual union all
select 20561,'USAF' from dual union all
select 20562,'USAF' from dual union all
select 20562,'USN' from dual union all
select 20563,'USAF' from dual union all
select 20564,'USAF' from dual union all
select 20565,'USAF' from dual union all
select 20566,'TSSKB' from dual union all
select 20567,'RVSN' from dual union all
select 20568,'MO' from dual union all
select 20569,'RVSN' from dual union all
select 2057,'USAF' from dual union all
select 20570,'PER' from dual union all
select 20571,'MDC' from dual union all
select 20572,'MDC' from dual union all
select 20573,'MO' from dual union all
select 20574,'RVSN' from dual union all
select 20575,'USAF' from dual union all
select 20576,'USAF' from dual union all
select 20577,'VMF' from dual union all
select 20578,'RVSN' from dual union all
select 20579,'NASA' from dual union all
select 2058,'USAF' from dual union all
select 20580,'NASA' from dual union all
select 20581,'PVO' from dual union all
select 20582,'RVSN' from dual union all
select 20583,'MO' from dual union all
select 20584,'RVSN' from dual union all
select 20585,'RVSN' from dual union all
select 20586,'RVSN' from dual union all
select 20587,'PVO' from dual union all
select 20588,'MO' from dual union all
select 20589,'MO' from dual union all
select 2059,'USAF' from dual union all
select 20590,'MO' from dual union all
select 20591,'MO' from dual;
insert into SATELLITE_ORG
select 20592,'MO' from dual union all
select 20593,'MO' from dual union all
select 20594,'MO' from dual union all
select 20595,'MO' from dual union all
select 20596,'PVO' from dual union all
select 20597,'RVSN' from dual union all
select 20598,'RVSN' from dual union all
select 20599,'RVSN' from dual union all
select 206,'JPL' from dual union all
select 206,'NASA' from dual union all
select 2060,'USAF' from dual union all
select 20600,'RVSN' from dual union all
select 20601,'MO' from dual union all
select 20602,'NPOE' from dual union all
select 20603,'RVSN' from dual union all
select 20604,'MO' from dual union all
select 20605,'RVSN' from dual union all
select 20606,'MO' from dual union all
select 20607,'DOD' from dual union all
select 20608,'DOD' from dual union all
select 20609,'NASA' from dual union all
select 2061,'USAF' from dual union all
select 20610,'USAF' from dual union all
select 20611,'USAF' from dual union all
select 20612,'USAF' from dual union all
select 20613,'USAF' from dual union all
select 20614,'USAF' from dual union all
select 20615,'VTS' from dual union all
select 20616,'RVSN' from dual union all
select 20617,'VTS' from dual union all
select 20618,'ISASS' from dual union all
select 20619,'MO' from dual union all
select 2062,'USAF' from dual union all
select 20620,'MO' from dual union all
select 20621,'MO' from dual union all
select 20622,'RVSN' from dual union all
select 20623,'RVSN' from dual union all
select 20624,'VS' from dual union all
select 20625,'MO' from dual union all
select 20626,'MO' from dual union all
select 20627,'MO' from dual union all
select 20628,'MO' from dual union all
select 20629,'MO' from dual union all
select 2063,'USAF' from dual union all
select 20630,'RVSN' from dual union all
select 20631,'RVSN' from dual union all
select 20632,'MVD' from dual union all
select 20633,'RVSN' from dual union all
select 20634,'USAF' from dual union all
select 20635,'NPOE' from dual union all
select 20636,'RVSN' from dual union all
select 20637,'PVO' from dual union all
select 20638,'DLR' from dual union all
select 20639,'MDC' from dual union all
select 2064,'USAF' from dual union all
select 20640,'PVO' from dual union all
select 20641,'NRO' from dual union all
select 20641,'USN' from dual union all
select 20642,'USAF' from dual union all
select 20643,'ISRO' from dual union all
select 20644,'MDC' from dual union all
select 20645,'MDC' from dual union all
select 20646,'MO' from dual union all
select 20646,'MSVYAZ' from dual union all
select 20647,'RVSN' from dual union all
select 20648,'RVSN' from dual union all
select 20649,'RVSN' from dual union all
select 2065,'USAF' from dual union all
select 20650,'USAF' from dual union all
select 20651,'USAF' from dual union all
select 20652,'USN' from dual union all
select 20653,'MO' from dual union all
select 20654,'MO' from dual union all
select 20655,'MO' from dual union all
select 20656,'MO' from dual union all
select 20657,'MO' from dual union all
select 20658,'RVSN' from dual union all
select 20659,'MSVYAZ' from dual union all
select 2066,'USAF' from dual union all
select 20660,'RVSN' from dual union all
select 20661,'RVSN' from dual union all
select 20662,'RVSN' from dual union all
select 20663,'PVO' from dual union all
select 20664,'RVSN' from dual union all
select 20665,'RVSN' from dual union all
select 20666,'RVSN' from dual union all
select 20667,'INTEL' from dual union all
select 20668,'GD' from dual union all
select 20669,'INTEL' from dual union all
select 2067,'USAF' from dual union all
select 20670,'GMS' from dual union all
select 20670,'MO' from dual union all
select 20671,'MO' from dual union all
select 20672,'USAF' from dual union all
select 20673,'PVO' from dual union all
select 20674,'AE' from dual union all
select 20675,'MO' from dual union all
select 20676,'MO' from dual union all
select 20677,'MO' from dual union all
select 20678,'MO' from dual;
insert into SATELLITE_ORG
select 20679,'MO' from dual union all
select 2068,'USAF' from dual union all
select 20680,'NASA' from dual union all
select 20681,'NASA' from dual union all
select 20682,'NRO' from dual union all
select 20682,'USN' from dual union all
select 20683,'NPOE' from dual union all
select 20684,'RVSN' from dual union all
select 20685,'SUPA' from dual union all
select 20686,'MAI' from dual union all
select 20687,'MVD' from dual union all
select 20688,'RVSN' from dual union all
select 20689,'NPOE' from dual union all
select 2069,'USAF' from dual union all
select 20690,'NPOE' from dual union all
select 20691,'NRO' from dual union all
select 20691,'USN' from dual union all
select 20692,'NRO' from dual union all
select 20692,'USN' from dual union all
select 20693,'MO' from dual union all
select 20694,'RVSN' from dual union all
select 20695,'RVSN' from dual union all
select 20696,'RVSN' from dual union all
select 20697,'RVSN' from dual union all
select 20698,'RVSN' from dual union all
select 20699,'NPOE' from dual union all
select 207,'USAF' from dual union all
select 2070,'OKB1' from dual union all
select 20700,'NPOE' from dual union all
select 20701,'NPOE' from dual union all
select 20702,'MO' from dual union all
select 20703,'RVSN' from dual union all
select 20704,'RVSN' from dual union all
select 20705,'FTEL' from dual union all
select 20706,'DBP' from dual union all
select 20707,'PVO' from dual union all
select 20708,'RVSN' from dual union all
select 20709,'RVSN' from dual union all
select 2071,'RVSN' from dual union all
select 20710,'RVSN' from dual union all
select 20711,'RVSN' from dual union all
select 20712,'NASA' from dual union all
select 20712,'USAF' from dual union all
select 20713,'GD' from dual union all
select 20714,'MO' from dual union all
select 20715,'MO' from dual union all
select 20716,'RVSN' from dual union all
select 20717,'AE' from dual union all
select 20718,'AE' from dual union all
select 20719,'NPOE' from dual union all
select 2072,'USAF' from dual union all
select 20720,'VTS' from dual union all
select 20721,'MO' from dual union all
select 20722,'NPOE' from dual union all
select 20723,'RVSN' from dual union all
select 20724,'USAF' from dual union all
select 20725,'MDC' from dual union all
select 20726,'MDC' from dual union all
select 20727,'MO' from dual union all
select 20728,'MO' from dual union all
select 20729,'MO' from dual union all
select 2073,'USAF' from dual union all
select 20730,'MO' from dual union all
select 20731,'MO' from dual union all
select 20732,'MO' from dual union all
select 20733,'RVSN' from dual union all
select 20734,'MO' from dual union all
select 20735,'MO' from dual union all
select 20736,'MO' from dual union all
select 20737,'MO' from dual union all
select 20738,'MO' from dual union all
select 20739,'MO' from dual union all
select 2074,'USAF' from dual union all
select 20740,'MO' from dual union all
select 20741,'MO' from dual union all
select 20742,'MO' from dual union all
select 20743,'RVSN' from dual union all
select 20744,'RVSN' from dual union all
select 20745,'RVSN' from dual union all
select 20746,'PVO' from dual union all
select 20747,'PVO' from dual union all
select 20748,'PVO' from dual union all
select 20749,'PVO' from dual union all
select 2075,'USAF' from dual union all
select 20750,'PVO' from dual union all
select 20751,'PVO' from dual union all
select 20752,'NPOE' from dual union all
select 20753,'RVSN' from dual union all
select 20754,'MVD' from dual union all
select 20755,'RVSN' from dual union all
select 20756,'MO' from dual union all
select 20757,'MO' from dual union all
select 20758,'MO' from dual union all
select 20759,'USAF' from dual union all
select 2076,'USAF' from dual union all
select 20760,'USAF' from dual union all
select 20761,'MO' from dual union all
select 20762,'BSB' from dual union all
select 20763,'MDC' from dual union all
select 20764,'MDC' from dual;
insert into SATELLITE_ORG
select 20765,'VMF' from dual union all
select 20766,'MO' from dual union all
select 20767,'PVO' from dual union all
select 20768,'RVSN' from dual union all
select 20769,'RVSN' from dual union all
select 2077,'USAF' from dual union all
select 20770,'RVSN' from dual union all
select 20771,'NASDA' from dual union all
select 20771,'TSCJ' from dual union all
select 20772,'NASDA' from dual union all
select 20773,'NASDA' from dual union all
select 20774,'PVO' from dual union all
select 20775,'RVSN' from dual union all
select 20776,'MOD' from dual union all
select 20777,'EUTEL' from dual union all
select 20778,'AE' from dual union all
select 20779,'MO' from dual union all
select 2078,'USAF' from dual union all
select 20780,'RVSN' from dual union all
select 20781,'AE' from dual union all
select 20782,'MO' from dual union all
select 20783,'MO' from dual union all
select 20784,'MO' from dual union all
select 20785,'MO' from dual union all
select 20786,'MO' from dual union all
select 20787,'MO' from dual union all
select 20788,'CMA' from dual union all
select 20789,'MAI' from dual union all
select 2079,'USAF' from dual union all
select 20790,'MAI' from dual union all
select 20791,'MAI' from dual union all
select 20792,'MAI' from dual union all
select 20793,'MAI' from dual union all
select 20794,'MVD' from dual union all
select 20795,'MO' from dual union all
select 20796,'MO' from dual union all
select 20797,'MAI' from dual union all
select 20798,'MAI' from dual union all
select 20799,'NASA' from dual union all
select 208,'NASA' from dual union all
select 2080,'USAF' from dual union all
select 20800,'ESA' from dual union all
select 20801,'NASA' from dual union all
select 20802,'PVO' from dual union all
select 20803,'PVO' from dual union all
select 20804,'VMF' from dual union all
select 20805,'MO' from dual union all
select 20806,'MO' from dual union all
select 20807,'MO' from dual union all
select 20808,'MO' from dual union all
select 20809,'MO' from dual union all
select 2081,'USAF' from dual union all
select 20810,'MO' from dual union all
select 20811,'NASA' from dual union all
select 20812,'NPOE' from dual union all
select 20813,'MO' from dual union all
select 20813,'MSVYAZ' from dual union all
select 20814,'MO' from dual union all
select 20815,'MO' from dual union all
select 20816,'MO' from dual union all
select 20817,'MO' from dual union all
select 20818,'MO' from dual union all
select 20819,'MO' from dual union all
select 2082,'USAF' from dual union all
select 20820,'MO' from dual union all
select 20821,'PVO' from dual union all
select 20822,'PVO' from dual union all
select 20823,'PVO' from dual union all
select 20824,'NPOE' from dual union all
select 20825,'MO' from dual union all
select 20826,'GMS' from dual union all
select 20826,'MO' from dual union all
select 20827,'MO' from dual union all
select 20828,'TSSKB' from dual union all
select 20829,'RVSN' from dual union all
select 2083,'USAF' from dual union all
select 20830,'USAF' from dual union all
select 20831,'MDC' from dual union all
select 20832,'MDC' from dual union all
select 20833,'NASA' from dual union all
select 20834,'RVSN' from dual union all
select 20835,'NASA' from dual union all
select 20836,'RVSN' from dual union all
select 20837,'ESA' from dual union all
select 20838,'MAI' from dual union all
select 20839,'MAI' from dual union all
select 2084,'USAF' from dual union all
select 20840,'MAI' from dual union all
select 20841,'NASA' from dual union all
select 20842,'ESA' from dual union all
select 20843,'NASA' from dual union all
select 20844,'NASA' from dual union all
select 20845,'NASA' from dual union all
select 20846,'MAI' from dual union all
select 20847,'MAI' from dual union all
select 20848,'MAI' from dual union all
select 20849,'MAI' from dual union all
select 2085,'NASA' from dual union all
select 20850,'MAI' from dual union all
select 20851,'MAI' from dual;
insert into SATELLITE_ORG
select 20852,'MAI' from dual union all
select 20853,'MAI' from dual union all
select 20854,'MAI' from dual union all
select 20855,'MAI' from dual union all
select 20856,'MAI' from dual union all
select 20857,'MAI' from dual union all
select 20858,'MAI' from dual union all
select 20859,'MAI' from dual union all
select 2086,'USN' from dual union all
select 20860,'MAI' from dual union all
select 20861,'MAI' from dual union all
select 20862,'MAI' from dual union all
select 20863,'MAI' from dual union all
select 20864,'MAI' from dual union all
select 20865,'MAI' from dual union all
select 20866,'MAI' from dual union all
select 20867,'MAI' from dual union all
select 20868,'MAI' from dual union all
select 20869,'MAI' from dual union all
select 2087,'USAF' from dual union all
select 20870,'MAI' from dual union all
select 20871,'MAI' from dual union all
select 20872,'HEC' from dual union all
select 20873,'HEC' from dual union all
select 20874,'AE' from dual union all
select 20875,'AE' from dual union all
select 20876,'MAI' from dual union all
select 20877,'MAI' from dual union all
select 20878,'MAI' from dual union all
select 20879,'MAI' from dual union all
select 2088,'NASA' from dual union all
select 20880,'MAI' from dual union all
select 20881,'MAI' from dual union all
select 20882,'MAI' from dual union all
select 20883,'MAI' from dual union all
select 20884,'MAI' from dual union all
select 20885,'MAI' from dual union all
select 20886,'MAI' from dual union all
select 20887,'MAI' from dual union all
select 20888,'MAI' from dual union all
select 20889,'MAI' from dual union all
select 2089,'OKB1' from dual union all
select 20890,'MAI' from dual union all
select 20891,'MAI' from dual union all
select 20892,'MAI' from dual union all
select 20893,'MAI' from dual union all
select 20894,'MAI' from dual union all
select 20895,'MAI' from dual union all
select 20896,'MAI' from dual union all
select 20897,'MAI' from dual union all
select 20898,'MAI' from dual union all
select 20899,'MAI' from dual union all
select 209,'NASA' from dual union all
select 2090,'OKB1' from dual union all
select 20900,'MAI' from dual union all
select 20901,'MAI' from dual union all
select 20902,'MAI' from dual union all
select 20903,'MAI' from dual union all
select 20904,'MAI' from dual union all
select 20905,'MAI' from dual union all
select 20906,'MAI' from dual union all
select 20907,'MAI' from dual union all
select 20908,'MAI' from dual union all
select 20909,'MO' from dual union all
select 2091,'ESSA' from dual union all
select 20910,'RVSN' from dual union all
select 20911,'PVO' from dual union all
select 20912,'MAI' from dual union all
select 20913,'MAI' from dual union all
select 20914,'MAI' from dual union all
select 20915,'MAI' from dual union all
select 20916,'MAI' from dual union all
select 20917,'NPOE' from dual union all
select 20918,'INMAR' from dual union all
select 20919,'MDC' from dual union all
select 2092,'NASA' from dual union all
select 20920,'MDC' from dual union all
select 20921,'NPOE' from dual union all
select 20922,'NPOE' from dual union all
select 20923,'MSVYAZ' from dual union all
select 20924,'RVSN' from dual union all
select 20925,'RVSN' from dual union all
select 20926,'RVSN' from dual union all
select 20927,'RVSN' from dual union all
select 20928,'RVSN' from dual union all
select 20929,'USAF' from dual union all
select 2093,'OKB1' from dual union all
select 20930,'USAF' from dual union all
select 20931,'USAF' from dual union all
select 20932,'USAF' from dual union all
select 20933,'VMF' from dual union all
select 20934,'MO' from dual union all
select 20935,'NASA' from dual union all
select 20936,'MO' from dual union all
select 20937,'RVSN' from dual union all
select 20938,'MO' from dual union all
select 20939,'MO' from dual union all
select 2094,'RVSN' from dual union all
select 20940,'MO' from dual union all
select 20941,'PVO' from dual;
insert into SATELLITE_ORG
select 20942,'RVSN' from dual union all
select 20943,'RVSN' from dual union all
select 20944,'RVSN' from dual union all
select 20945,'AMCG' from dual union all
select 20946,'GTE' from dual union all
select 20947,'AE' from dual union all
select 20948,'AE' from dual union all
select 20949,'MO' from dual union all
select 2095,'RVSN' from dual union all
select 20950,'RVSN' from dual union all
select 20951,'RVSN' from dual union all
select 20952,'RVSN' from dual union all
select 20953,'MSVYAZ' from dual union all
select 20954,'RVSN' from dual union all
select 20955,'RVSN' from dual union all
select 20956,'MO' from dual union all
select 20957,'RVSN' from dual union all
select 20958,'RVSN' from dual union all
select 20959,'USAF' from dual union all
select 2096,'NASA' from dual union all
select 20960,'MDC' from dual union all
select 20961,'MDC' from dual union all
select 20962,'NASA' from dual union all
select 20963,'NRO' from dual union all
select 20964,'USAF' from dual union all
select 20965,'USAF' from dual union all
select 20966,'PVO' from dual union all
select 20967,'MO' from dual union all
select 20968,'MAI' from dual union all
select 20969,'MAI' from dual union all
select 2097,'USN' from dual union all
select 20970,'MAI' from dual union all
select 20971,'MAI' from dual union all
select 20972,'MAI' from dual union all
select 20973,'MAI' from dual union all
select 20974,'MAI' from dual union all
select 20975,'PVO' from dual union all
select 20976,'PVO' from dual union all
select 20977,'PVO' from dual union all
select 20978,'USAF' from dual union all
select 20979,'USAF' from dual union all
select 2098,'YUZH' from dual union all
select 20980,'NASA' from dual union all
select 20981,'NPOE' from dual union all
select 20982,'RVSN' from dual union all
select 20983,'USAF' from dual union all
select 20984,'USAF' from dual union all
select 20985,'VMF' from dual union all
select 20986,'MO' from dual union all
select 20987,'USAF' from dual union all
select 20988,'USAF' from dual union all
select 20989,'USAF' from dual union all
select 2099,'CIA' from dual union all
select 2099,'NRO' from dual union all
select 20990,'USAF' from dual union all
select 20991,'USAF' from dual union all
select 20992,'USAF' from dual union all
select 20993,'USAF' from dual union all
select 20994,'MO' from dual union all
select 20995,'MO' from dual union all
select 20996,'MO' from dual union all
select 20997,'USAF' from dual union all
select 20998,'USAF' from dual union all
select 20999,'USAF' from dual union all
select 21,'OKB1' from dual union all
select 210,'USAF' from dual union all
select 2100,'USAF' from dual union all
select 21000,'MO' from dual union all
select 21001,'RVSN' from dual union all
select 21002,'MO' from dual union all
select 21003,'MO' from dual union all
select 21004,'MO' from dual union all
select 21005,'MO' from dual union all
select 21006,'MO' from dual union all
select 21007,'MO' from dual union all
select 21008,'MO' from dual union all
select 21009,'RVSN' from dual union all
select 2101,'USAF' from dual union all
select 21010,'RVSN' from dual union all
select 21011,'RVSN' from dual union all
select 21012,'RVSN' from dual union all
select 21013,'RVSN' from dual union all
select 21014,'MO' from dual union all
select 21015,'RVSN' from dual union all
select 21016,'MO' from dual union all
select 21017,'RVSN' from dual union all
select 21018,'RVSN' from dual union all
select 21019,'RVSN' from dual union all
select 2102,'USAF' from dual union all
select 21020,'PVO' from dual union all
select 21021,'PVO' from dual union all
select 21022,'PVO' from dual union all
select 21023,'PVO' from dual union all
select 21024,'RVSN' from dual union all
select 21025,'RVSN' from dual union all
select 21026,'MO' from dual union all
select 21027,'RVSN' from dual union all
select 21028,'MO' from dual union all
select 21029,'MO' from dual union all
select 2103,'USAF' from dual;
insert into SATELLITE_ORG
select 21030,'MO' from dual union all
select 21031,'MO' from dual union all
select 21032,'MO' from dual union all
select 21033,'MO' from dual union all
select 21034,'MO' from dual union all
select 21035,'MO' from dual union all
select 21036,'RVSN' from dual union all
select 21037,'MO' from dual union all
select 21038,'MO' from dual union all
select 21039,'RVSN' from dual union all
select 2104,'NASA' from dual union all
select 21040,'RVSN' from dual union all
select 21041,'RVSN' from dual union all
select 21042,'PVO' from dual union all
select 21043,'PVO' from dual union all
select 21044,'RVSN' from dual union all
select 21045,'RVSN' from dual union all
select 21046,'RVSN' from dual union all
select 21047,'NATO' from dual union all
select 21048,'MDC' from dual union all
select 21049,'MDC' from dual union all
select 2105,'NASA' from dual union all
select 21050,'NPOE' from dual union all
select 21051,'NPOE' from dual union all
select 21052,'NASA' from dual union all
select 21053,'NPOE' from dual union all
select 21054,'RVSN' from dual union all
select 21055,'ASI' from dual union all
select 21056,'EUTEL' from dual union all
select 21057,'AE' from dual union all
select 21058,'AE' from dual union all
select 21059,'RVSNR' from dual union all
select 2106,'USAF' from dual union all
select 21060,'RVSN' from dual union all
select 21061,'MO' from dual union all
select 21062,'MO' from dual union all
select 21063,'MO' from dual union all
select 21064,'PVO' from dual union all
select 21065,'VMF' from dual union all
select 21066,'RVSNR' from dual union all
select 21067,'VMF' from dual union all
select 21068,'MO' from dual union all
select 21069,'PVO' from dual union all
select 2107,'MO' from dual union all
select 21070,'PVO' from dual union all
select 21071,'USAF' from dual union all
select 21072,'USAF' from dual union all
select 21073,'USAF' from dual union all
select 21074,'USAF' from dual union all
select 21075,'USAF' from dual union all
select 21076,'USAF' from dual union all
select 21077,'USAF' from dual union all
select 21078,'USAF' from dual union all
select 21079,'USAF' from dual union all
select 2108,'RVSN' from dual union all
select 21080,'USAF' from dual union all
select 21081,'PVO' from dual union all
select 21082,'NPOE' from dual union all
select 21083,'MAI' from dual union all
select 21084,'NPOE' from dual union all
select 21085,'NPOE' from dual union all
select 21086,'NPOE' from dual union all
select 21087,'MGON' from dual union all
select 21088,'RVSN' from dual union all
select 21089,'VMF' from dual union all
select 2109,'NRO' from dual union all
select 21090,'RVSN' from dual union all
select 21091,'VMF' from dual union all
select 21092,'RVSNR' from dual union all
select 21093,'RVSN' from dual union all
select 21094,'RVSNR' from dual union all
select 21095,'RVSNR' from dual union all
select 21096,'RVSNR' from dual union all
select 21097,'RVSNR' from dual union all
select 21098,'RVSNR' from dual union all
select 21099,'RVSNR' from dual union all
select 211,'USAF' from dual union all
select 2110,'MO' from dual union all
select 21100,'RVSNR' from dual union all
select 21101,'RVSNR' from dual union all
select 21102,'RVSNR' from dual union all
select 21103,'RVSNR' from dual union all
select 21104,'RVSNR' from dual union all
select 21105,'RVSNR' from dual union all
select 21106,'RVSNR' from dual union all
select 21107,'RVSNR' from dual union all
select 21108,'RVSN' from dual union all
select 21109,'RVSN' from dual union all
select 2111,'MO' from dual union all
select 21110,'RVSN' from dual union all
select 21111,'PVO' from dual union all
select 21112,'RVSN' from dual union all
select 21113,'RVSN' from dual union all
select 21114,'RVSN' from dual union all
select 21115,'RVSN' from dual union all
select 21116,'VTSR' from dual union all
select 21117,'RVSN' from dual union all
select 21118,'RVSNR' from dual union all
select 21119,'RVSN' from dual union all
select 2112,'USAF' from dual;
insert into SATELLITE_ORG
select 21120,'RVSN' from dual union all
select 21121,'RVSN' from dual union all
select 21122,'RVSN' from dual union all
select 21123,'VTSR' from dual union all
select 21124,'USAF' from dual union all
select 21125,'USAF' from dual union all
select 21126,'RVSN' from dual union all
select 21127,'USAF' from dual union all
select 21128,'PVO' from dual union all
select 21129,'RVSN' from dual union all
select 2113,'CIA' from dual union all
select 2113,'NRO' from dual union all
select 21130,'VMF' from dual union all
select 21131,'RVSN' from dual union all
select 21132,'RVSNR' from dual union all
select 21133,'RVSN' from dual union all
select 21134,'RVSN' from dual union all
select 21135,'RVSN' from dual union all
select 21136,'RVSN' from dual union all
select 21137,'RVSN' from dual union all
select 21138,'RVSN' from dual union all
select 21139,'SES' from dual union all
select 2114,'MO' from dual union all
select 21140,'EUMET' from dual union all
select 21141,'AE' from dual union all
select 21142,'AE' from dual union all
select 21143,'RVSNR' from dual union all
select 21144,'RVSN' from dual union all
select 21145,'RVSNR' from dual union all
select 21146,'RVSNR' from dual union all
select 21147,'CIA' from dual union all
select 21147,'NRO' from dual union all
select 21148,'USAF' from dual union all
select 21149,'INMAR' from dual union all
select 2115,'NASA' from dual union all
select 21150,'MDC' from dual union all
select 21151,'MDC' from dual union all
select 21152,'RVSNR' from dual union all
select 21152,'VMF' from dual union all
select 21153,'RVSN' from dual union all
select 21154,'RVSN' from dual union all
select 21155,'RVSN' from dual union all
select 21156,'RVSN' from dual union all
select 21157,'RVSN' from dual union all
select 21158,'RVSN' from dual union all
select 21159,'RVSN' from dual union all
select 2116,'RVSN' from dual union all
select 21160,'RVSN' from dual union all
select 21161,'RVSN' from dual union all
select 21162,'RVSN' from dual union all
select 21163,'RVSN' from dual union all
select 21164,'RVSN' from dual union all
select 21165,'RVSN' from dual union all
select 21166,'RVSN' from dual union all
select 21167,'RVSN' from dual union all
select 21168,'RVSN' from dual union all
select 21169,'RVSN' from dual union all
select 2117,'MO' from dual union all
select 21170,'RVSN' from dual union all
select 21171,'RVSN' from dual union all
select 21172,'RVSN' from dual union all
select 21173,'RVSN' from dual union all
select 21174,'RVSN' from dual union all
select 21175,'RVSN' from dual union all
select 21176,'RVSN' from dual union all
select 21177,'RVSN' from dual union all
select 21178,'RVSN' from dual union all
select 21179,'RVSN' from dual union all
select 2118,'NASA' from dual union all
select 21180,'RVSN' from dual union all
select 21181,'RVSN' from dual union all
select 21182,'RVSN' from dual union all
select 21183,'RVSN' from dual union all
select 21184,'RVSN' from dual union all
select 21185,'RVSN' from dual union all
select 21186,'RVSN' from dual union all
select 21187,'RVSN' from dual union all
select 21188,'NPOE' from dual union all
select 21189,'RVSN' from dual union all
select 2119,'USN' from dual union all
select 21190,'PVO' from dual union all
select 21191,'RVSN' from dual union all
select 21192,'RVSNR' from dual union all
select 21193,'RVSNR' from dual union all
select 21194,'RVSNR' from dual union all
select 21195,'RVSNR' from dual union all
select 21196,'MSVYAZ' from dual union all
select 21196,'RVSNR' from dual union all
select 21197,'RVSN' from dual union all
select 21198,'RVSN' from dual union all
select 21199,'RVSN' from dual union all
select 212,'USAF' from dual union all
select 2120,'NASA' from dual union all
select 21200,'RVSN' from dual union all
select 21201,'RVSN' from dual union all
select 21202,'RVSN' from dual union all
select 21203,'RVSNR' from dual union all
select 21204,'RVSN' from dual union all
select 21205,'PVO' from dual union all
select 21206,'PVO' from dual;
insert into SATELLITE_ORG
select 21207,'PVO' from dual union all
select 21208,'RVSN' from dual union all
select 21209,'RVSN' from dual union all
select 2121,'USAF' from dual union all
select 21210,'RVSN' from dual union all
select 21211,'RVSN' from dual union all
select 21212,'RVSN' from dual union all
select 21213,'KBSAL' from dual union all
select 21214,'RVSN' from dual union all
select 21215,'USAF' from dual union all
select 21216,'RVSNR' from dual union all
select 21217,'RVSNR' from dual union all
select 21218,'RVSNR' from dual union all
select 21219,'RVSN' from dual union all
select 2122,'USAF' from dual union all
select 21220,'RVSN' from dual union all
select 21221,'RVSN' from dual union all
select 21222,'TCAN' from dual union all
select 21223,'AE' from dual union all
select 21224,'NASA' from dual union all
select 21225,'NASA' from dual union all
select 21226,'RVSN' from dual union all
select 21227,'CONTL' from dual union all
select 21228,'MDC' from dual union all
select 21229,'MDC' from dual union all
select 2123,'USAF' from dual union all
select 21230,'VMF' from dual union all
select 21231,'RVSN' from dual union all
select 21232,'RGMS' from dual union all
select 21233,'RVSNR' from dual union all
select 21234,'RVSN' from dual union all
select 21235,'NPOE' from dual union all
select 21236,'NPOE' from dual union all
select 21237,'NPOE' from dual union all
select 21238,'NPOE' from dual union all
select 21239,'NPOE' from dual union all
select 2124,'USAF' from dual union all
select 21240,'NPOE' from dual union all
select 21241,'NPOE' from dual union all
select 21242,'NASA' from dual union all
select 21243,'NPOE' from dual union all
select 21244,'SDIO' from dual union all
select 21245,'SDIO' from dual union all
select 21246,'SDIO' from dual union all
select 21247,'SDIO' from dual union all
select 21248,'NPOE' from dual union all
select 21249,'RVSN' from dual union all
select 2125,'USAF' from dual union all
select 21250,'RVSN' from dual union all
select 21251,'RVSN' from dual union all
select 21252,'RVSN' from dual union all
select 21253,'RVSN' from dual union all
select 21254,'RVSN' from dual union all
select 21255,'MO' from dual union all
select 21256,'RVSN' from dual union all
select 21257,'VMF' from dual union all
select 21258,'PVO' from dual union all
select 21259,'PVO' from dual union all
select 2126,'TSKBEM' from dual union all
select 21260,'PVO' from dual union all
select 21261,'PVO' from dual union all
select 21262,'DOD' from dual union all
select 21263,'NOAA' from dual union all
select 21264,'PVO' from dual union all
select 21265,'PVO' from dual union all
select 21266,'PVO' from dual union all
select 21267,'NOAA' from dual union all
select 21268,'NASA' from dual union all
select 21269,'NASA' from dual union all
select 2127,'RVSN' from dual union all
select 21270,'NASA' from dual union all
select 21271,'NASA' from dual union all
select 21272,'NASA' from dual union all
select 21273,'NASA' from dual union all
select 21274,'NASA' from dual union all
select 21275,'NASA' from dual union all
select 21276,'NASA' from dual union all
select 21277,'NASA' from dual union all
select 21278,'NASA' from dual union all
select 21279,'NASA' from dual union all
select 2128,'RVSN' from dual union all
select 21280,'NASA' from dual union all
select 21281,'NASA' from dual union all
select 21282,'NASA' from dual union all
select 21283,'NASA' from dual union all
select 21284,'NASA' from dual union all
select 21285,'NASA' from dual union all
select 21286,'NASA' from dual union all
select 21287,'NASA' from dual union all
select 21288,'NASA' from dual union all
select 21289,'NASA' from dual union all
select 2129,'USAF' from dual union all
select 21290,'NASA' from dual union all
select 21291,'NASA' from dual union all
select 21292,'NASA' from dual union all
select 21293,'NASA' from dual union all
select 21294,'NASA' from dual union all
select 21295,'NASA' from dual union all
select 21296,'NASA' from dual union all
select 21297,'NASA' from dual;
insert into SATELLITE_ORG
select 21298,'NASA' from dual union all
select 21299,'RVSNR' from dual union all
select 213,'CIA' from dual union all
select 213,'NRO' from dual union all
select 2130,'RVSN' from dual union all
select 21300,'RVSNR' from dual union all
select 21301,'RVSNR' from dual union all
select 21302,'RVSNR' from dual union all
select 21303,'RVSNR' from dual union all
select 21304,'RVSNR' from dual union all
select 21305,'RVSNR' from dual union all
select 21306,'PVO' from dual union all
select 21307,'PVO' from dual union all
select 21308,'PVO' from dual union all
select 21309,'PVO' from dual union all
select 2131,'TSKBEM' from dual union all
select 21310,'PVO' from dual union all
select 21311,'NPOE' from dual union all
select 21312,'RVSN' from dual union all
select 21313,'MVD' from dual union all
select 21314,'RVSN' from dual union all
select 21315,'RVSNR' from dual union all
select 21316,'RVSN' from dual union all
select 21317,'NASA' from dual union all
select 21318,'NASA' from dual union all
select 21319,'NASA' from dual union all
select 2132,'TSKBEM' from dual union all
select 21320,'NASA' from dual union all
select 21321,'NASA' from dual union all
select 21322,'NASA' from dual union all
select 21323,'NASA' from dual union all
select 21324,'NASA' from dual union all
select 21325,'NASA' from dual union all
select 21326,'NASA' from dual union all
select 21327,'NASA' from dual union all
select 21328,'NASA' from dual union all
select 21329,'NASA' from dual union all
select 2133,'MO' from dual union all
select 21330,'NASA' from dual union all
select 21331,'NASA' from dual union all
select 21332,'RVSN' from dual union all
select 21333,'NASA' from dual union all
select 21334,'NASA' from dual union all
select 21335,'NASA' from dual union all
select 21336,'NASA' from dual union all
select 21337,'NASA' from dual union all
select 21338,'NASA' from dual union all
select 21339,'NASA' from dual union all
select 2134,'RVSN' from dual union all
select 21340,'NASA' from dual union all
select 21341,'NASA' from dual union all
select 21342,'NASA' from dual union all
select 21343,'NASA' from dual union all
select 21344,'NASA' from dual union all
select 21345,'NASA' from dual union all
select 21346,'NASA' from dual union all
select 21347,'NASA' from dual union all
select 21348,'NASA' from dual union all
select 21349,'NASA' from dual union all
select 2135,'MO' from dual union all
select 21350,'NASA' from dual union all
select 21351,'NASA' from dual union all
select 21352,'NASA' from dual union all
select 21353,'NASA' from dual union all
select 21354,'NASA' from dual union all
select 21355,'NASA' from dual union all
select 21356,'NASA' from dual union all
select 21357,'NASA' from dual union all
select 21358,'NASA' from dual union all
select 21359,'NASA' from dual union all
select 2136,'CIA' from dual union all
select 2136,'NRO' from dual union all
select 21360,'NASA' from dual union all
select 21361,'NASA' from dual union all
select 21362,'NASA' from dual union all
select 21363,'NASA' from dual union all
select 21364,'NASA' from dual union all
select 21365,'NASA' from dual union all
select 21366,'NASA' from dual union all
select 21367,'NASA' from dual union all
select 21368,'NASA' from dual union all
select 21369,'NASA' from dual union all
select 2137,'MO' from dual union all
select 21370,'NASA' from dual union all
select 21371,'NASA' from dual union all
select 21372,'NASA' from dual union all
select 21373,'NASA' from dual union all
select 21374,'NASA' from dual union all
select 21375,'NASA' from dual union all
select 21376,'NASA' from dual union all
select 21377,'NASA' from dual union all
select 21378,'NASA' from dual union all
select 21379,'NASA' from dual union all
select 2138,'MO' from dual union all
select 21380,'NASA' from dual union all
select 21381,'NASA' from dual union all
select 21382,'NASA' from dual union all
select 21383,'NASA' from dual union all
select 21384,'NASA' from dual union all
select 21385,'NASA' from dual;
insert into SATELLITE_ORG
select 21386,'NASA' from dual union all
select 21387,'NASA' from dual union all
select 21388,'NASA' from dual union all
select 21389,'NASA' from dual union all
select 2139,'JPL' from dual union all
select 2139,'NASA' from dual union all
select 21390,'NASA' from dual union all
select 21391,'NASA' from dual union all
select 21392,'AMCG' from dual union all
select 21393,'MDC' from dual union all
select 21394,'MDC' from dual union all
select 21395,'NPOE' from dual union all
select 21396,'RVSN' from dual union all
select 21397,'RGMS' from dual union all
select 21397,'YUZH' from dual union all
select 21398,'RVSNR' from dual union all
select 21399,'NASA' from dual union all
select 214,'OSCAR' from dual union all
select 2140,'USN' from dual union all
select 21400,'RVSN' from dual union all
select 21401,'RVSN' from dual union all
select 21402,'RVSN' from dual union all
select 21403,'RVSN' from dual union all
select 21404,'RVSN' from dual union all
select 21405,'RVSN' from dual union all
select 21406,'RVSN' from dual union all
select 21407,'RVSN' from dual union all
select 21408,'RVSN' from dual union all
select 21409,'NPOE' from dual union all
select 2141,'USN' from dual union all
select 21410,'NPOE' from dual union all
select 21411,'NPOE' from dual union all
select 21412,'NPOE' from dual union all
select 21413,'NPOE' from dual union all
select 21414,'NPOE' from dual union all
select 21415,'NPOE' from dual union all
select 21416,'NPOE' from dual union all
select 21417,'NPOE' from dual union all
select 21418,'RVSNR' from dual union all
select 21419,'RVSN' from dual union all
select 2142,'NASA' from dual union all
select 21420,'RVSNR' from dual union all
select 21421,'NPOE' from dual union all
select 21422,'MORF' from dual union all
select 21423,'RVSNR' from dual union all
select 21424,'NPOE' from dual union all
select 21425,'MAIO' from dual union all
select 21425,'MOM' from dual union all
select 21426,'RVSNR' from dual union all
select 21427,'RVSN' from dual union all
select 21428,'RVSN' from dual union all
select 21429,'RVSN' from dual union all
select 2143,'NASA' from dual union all
select 21430,'NASA' from dual union all
select 21431,'NASA' from dual union all
select 21432,'NASA' from dual union all
select 21433,'NASA' from dual union all
select 21434,'NASA' from dual union all
select 21435,'NASA' from dual union all
select 21436,'NASA' from dual union all
select 21437,'NASA' from dual union all
select 21438,'NASA' from dual union all
select 21439,'NASA' from dual union all
select 2144,'NASA' from dual union all
select 21440,'NASA' from dual union all
select 21441,'NASA' from dual union all
select 21442,'NASA' from dual union all
select 21443,'NASA' from dual union all
select 21444,'NASA' from dual union all
select 21445,'NASA' from dual union all
select 21446,'NASA' from dual union all
select 21447,'NASA' from dual union all
select 21448,'NASA' from dual union all
select 21449,'NASA' from dual union all
select 2145,'NASA' from dual union all
select 21450,'NASA' from dual union all
select 21451,'NASA' from dual union all
select 21452,'NASA' from dual union all
select 21453,'NASA' from dual union all
select 21454,'NASA' from dual union all
select 21455,'NASA' from dual union all
select 21456,'NASA' from dual union all
select 21457,'NASA' from dual union all
select 21458,'NASA' from dual union all
select 21459,'NASA' from dual union all
select 2146,'NRO' from dual union all
select 21460,'NASA' from dual union all
select 21461,'NASA' from dual union all
select 21462,'NASA' from dual union all
select 21463,'NASA' from dual union all
select 21464,'NASA' from dual union all
select 21465,'NASA' from dual union all
select 21466,'NASA' from dual union all
select 21467,'NASA' from dual union all
select 21468,'NASA' from dual union all
select 21469,'NASA' from dual union all
select 2147,'MO' from dual union all
select 21470,'NASA' from dual union all
select 21471,'NASA' from dual union all
select 21472,'NASA' from dual;
insert into SATELLITE_ORG
select 21473,'NASA' from dual union all
select 21474,'NASA' from dual union all
select 21475,'NASA' from dual union all
select 21476,'NPOE' from dual union all
select 21477,'NPOE' from dual union all
select 21478,'NPOE' from dual union all
select 21479,'RVSNR' from dual union all
select 2148,'USAF' from dual union all
select 21480,'RVSNR' from dual union all
select 21481,'NPOE' from dual union all
select 21482,'NPOE' from dual union all
select 21483,'NPOE' from dual union all
select 21484,'NPOE' from dual union all
select 21485,'RVSNR' from dual union all
select 21486,'NPOE' from dual union all
select 21487,'NPOE' from dual union all
select 21488,'NPOE' from dual union all
select 21489,'NPOE' from dual union all
select 2149,'RVSN' from dual union all
select 21490,'NPOE' from dual union all
select 21491,'NASA' from dual union all
select 21492,'NASA' from dual union all
select 21493,'NASA' from dual union all
select 21494,'NASA' from dual union all
select 21495,'NASA' from dual union all
select 21496,'NASA' from dual union all
select 21497,'NASA' from dual union all
select 21498,'NASA' from dual union all
select 21499,'NASA' from dual union all
select 215,'USAF' from dual union all
select 2150,'USAF' from dual union all
select 21500,'NASA' from dual union all
select 21501,'NASA' from dual union all
select 21502,'NASA' from dual union all
select 21503,'NASA' from dual union all
select 21504,'NASA' from dual union all
select 21505,'NASA' from dual union all
select 21506,'NASA' from dual union all
select 21507,'NASA' from dual union all
select 21508,'NASA' from dual union all
select 21509,'NASA' from dual union all
select 2151,'OKB10' from dual union all
select 21510,'NASA' from dual union all
select 21511,'NASA' from dual union all
select 21512,'NASA' from dual union all
select 21513,'NASA' from dual union all
select 21514,'NASA' from dual union all
select 21515,'NASA' from dual union all
select 21516,'NASA' from dual union all
select 21517,'NASA' from dual union all
select 21518,'NASA' from dual union all
select 21519,'NASA' from dual union all
select 2152,'PVO' from dual union all
select 2152,'YUZH' from dual union all
select 21520,'NASA' from dual union all
select 21521,'NASA' from dual union all
select 21522,'NASA' from dual union all
select 21523,'NASA' from dual union all
select 21524,'MVD' from dual union all
select 21525,'RVSN' from dual union all
select 21526,'NPOE' from dual union all
select 21527,'USAF' from dual union all
select 21528,'NASA' from dual union all
select 21529,'USAF' from dual union all
select 2153,'NASA' from dual union all
select 21530,'NPOE' from dual union all
select 21531,'NPOE' from dual union all
select 21532,'USAF' from dual union all
select 21533,'MSVYAZ' from dual union all
select 21534,'RVSN' from dual union all
select 21535,'RVSN' from dual union all
select 21536,'RVSN' from dual union all
select 21537,'PVO' from dual union all
select 21538,'RVSN' from dual union all
select 21539,'NASA' from dual union all
select 2154,'NASA' from dual union all
select 21540,'PVO' from dual union all
select 21541,'RVSN' from dual union all
select 21542,'NASA' from dual union all
select 21543,'NASA' from dual union all
select 21544,'NASA' from dual union all
select 21545,'NASA' from dual union all
select 21546,'NASA' from dual union all
select 21547,'NASA' from dual union all
select 21548,'NASA' from dual union all
select 21549,'NASA' from dual union all
select 2155,'RVSN' from dual union all
select 21550,'NASA' from dual union all
select 21551,'NASA' from dual union all
select 21552,'USAF' from dual union all
select 21553,'SDIO' from dual union all
select 21554,'MDC' from dual union all
select 21555,'MDC' from dual union all
select 21556,'NPOE' from dual union all
select 21557,'NPOE' from dual union all
select 21558,'RVSNR' from dual union all
select 21559,'RVSN' from dual union all
select 2156,'RVSN' from dual union all
select 21560,'TSSKB' from dual union all
select 21561,'RVSN' from dual;
insert into SATELLITE_ORG
select 21562,'NASA' from dual union all
select 21563,'NASA' from dual union all
select 21564,'NASA' from dual union all
select 21565,'NASA' from dual union all
select 21566,'NASA' from dual union all
select 21567,'NASA' from dual union all
select 21568,'NASA' from dual union all
select 21569,'NASA' from dual union all
select 2157,'RVSN' from dual union all
select 21570,'NASA' from dual union all
select 21571,'NPOE' from dual union all
select 21572,'NPOE' from dual union all
select 21573,'NPOE' from dual union all
select 21574,'ESA' from dual union all
select 21575,'SURREY' from dual union all
select 21576,'ORBC' from dual union all
select 21577,'TUB1' from dual union all
select 21578,'TARTU' from dual union all
select 21579,'NPOE' from dual union all
select 2158,'USAF' from dual union all
select 21580,'DOD' from dual union all
select 21581,'DOD' from dual union all
select 21582,'DOD' from dual union all
select 21583,'DOD' from dual union all
select 21584,'DOD' from dual union all
select 21585,'DOD' from dual union all
select 21586,'DOD' from dual union all
select 21587,'OSC' from dual union all
select 21588,'NASA' from dual union all
select 21589,'NASA' from dual union all
select 2159,'RVSN' from dual union all
select 21590,'NASA' from dual union all
select 21591,'NASA' from dual union all
select 21592,'NASA' from dual union all
select 21593,'NASA' from dual union all
select 21594,'NASA' from dual union all
select 21595,'NPOE' from dual union all
select 21596,'NPOE' from dual union all
select 21597,'NASA' from dual union all
select 21598,'NASA' from dual union all
select 21599,'NASA' from dual union all
select 216,'USAF' from dual union all
select 2160,'RVSN' from dual union all
select 21600,'NASA' from dual union all
select 21601,'NASA' from dual union all
select 21602,'NASA' from dual union all
select 21603,'NPOE' from dual union all
select 21604,'RVSNR' from dual union all
select 21605,'RVSNR' from dual union all
select 21606,'RVSNR' from dual union all
select 21607,'RVSNR' from dual union all
select 21608,'RVSNR' from dual union all
select 21609,'RVSNR' from dual union all
select 2161,'CNES' from dual union all
select 21610,'AE' from dual union all
select 21611,'MVD' from dual union all
select 21612,'RVSN' from dual union all
select 21613,'RVSNR' from dual union all
select 21614,'RVSNR' from dual union all
select 21615,'RVSNR' from dual union all
select 21616,'RVSNR' from dual union all
select 21617,'RVSNR' from dual union all
select 21618,'NPOE' from dual union all
select 21619,'NPOE' from dual union all
select 2162,'USAF' from dual union all
select 21620,'RVSN' from dual union all
select 21621,'RVSN' from dual union all
select 21622,'RVSN' from dual union all
select 21623,'RVSN' from dual union all
select 21624,'NPOE' from dual union all
select 21625,'NPOE' from dual union all
select 21626,'NPOE' from dual union all
select 21627,'NPOE' from dual union all
select 21628,'NPOE' from dual union all
select 21629,'NPOE' from dual union all
select 2163,'MO' from dual union all
select 21630,'RVSNR' from dual union all
select 21631,'RVSN' from dual union all
select 21632,'RVSN' from dual union all
select 21633,'RVSN' from dual union all
select 21634,'NASA' from dual union all
select 21635,'NASA' from dual union all
select 21636,'NASA' from dual union all
select 21637,'NASA' from dual union all
select 21638,'NASA' from dual union all
select 21639,'NASA' from dual union all
select 2164,'RVSN' from dual union all
select 21640,'NASA' from dual union all
select 21641,'NASA' from dual union all
select 21642,'NASA' from dual union all
select 21643,'RVSNR' from dual union all
select 21644,'RVSNR' from dual union all
select 21645,'RVSNR' from dual union all
select 21646,'RVSNR' from dual union all
select 21647,'RVSNR' from dual union all
select 21648,'RVSN' from dual union all
select 21649,'NPOE' from dual union all
select 2165,'MO' from dual union all
select 21650,'NPOE' from dual union all
select 21651,'NPOE' from dual;
insert into SATELLITE_ORG
select 21652,'NPOE' from dual union all
select 21653,'INTEL' from dual union all
select 21654,'AE' from dual union all
select 21655,'RGMS' from dual union all
select 21656,'RVSNR' from dual union all
select 21657,'RVSN' from dual union all
select 21658,'PVO' from dual union all
select 21659,'USAF' from dual union all
select 2166,'MO' from dual union all
select 21660,'USAF' from dual union all
select 21661,'NPOE' from dual union all
select 21662,'NPOE' from dual union all
select 21663,'RVSN' from dual union all
select 21664,'MVD' from dual union all
select 21665,'RVSN' from dual union all
select 21666,'VMF' from dual union all
select 21667,'RVSN' from dual union all
select 21668,'NASDA' from dual union all
select 21668,'TSCJ' from dual union all
select 21669,'NASDA' from dual union all
select 2167,'NASA' from dual union all
select 21670,'NASDA' from dual union all
select 21671,'NASA' from dual union all
select 21672,'NASA' from dual union all
select 21673,'NASA' from dual union all
select 21674,'NASA' from dual union all
select 21675,'NASA' from dual union all
select 21676,'NASA' from dual union all
select 21677,'NASA' from dual union all
select 21678,'NASA' from dual union all
select 21679,'NASA' from dual union all
select 2168,'METP' from dual union all
select 21680,'NASA' from dual union all
select 21681,'NASA' from dual union all
select 21682,'NASA' from dual union all
select 21683,'NASA' from dual union all
select 21684,'NASA' from dual union all
select 21685,'NASA' from dual union all
select 21686,'NASA' from dual union all
select 21687,'NASA' from dual union all
select 21688,'ISRO' from dual union all
select 21689,'RVSN' from dual union all
select 2169,'RVSN' from dual union all
select 21690,'USAF' from dual union all
select 21691,'USAF' from dual union all
select 21692,'USAF' from dual union all
select 21693,'USAF' from dual union all
select 21694,'ISASS' from dual union all
select 21695,'ISASS' from dual union all
select 21696,'ISASS' from dual union all
select 21697,'ISASS' from dual union all
select 21698,'ISASS' from dual union all
select 21699,'ISASS' from dual union all
select 217,'USAF' from dual union all
select 2170,'MO' from dual union all
select 21700,'NASA' from dual union all
select 21701,'NASA' from dual union all
select 21702,'PVO' from dual union all
select 21703,'RVSN' from dual union all
select 21704,'RVSN' from dual union all
select 21705,'RVSN' from dual union all
select 21706,'MSVYAZ' from dual union all
select 21706,'RVSNR' from dual union all
select 21707,'RVSN' from dual union all
select 21708,'RVSN' from dual union all
select 21709,'RVSN' from dual union all
select 2171,'NRO' from dual union all
select 21710,'RVSN' from dual union all
select 21711,'RVSNR' from dual union all
select 21712,'USAF' from dual union all
select 21713,'RVSNR' from dual union all
select 21714,'RVSN' from dual union all
select 21715,'RVSNR' from dual union all
select 21716,'RVSNR' from dual union all
select 21717,'RVSNR' from dual union all
select 21718,'RVSN' from dual union all
select 21719,'RVSN' from dual union all
select 2172,'NRO' from dual union all
select 21720,'RVSN' from dual union all
select 21721,'NASA' from dual union all
select 21722,'RVSN' from dual union all
select 21723,'PVO' from dual union all
select 21724,'RVSN' from dual union all
select 21725,'RVSN' from dual union all
select 21726,'TCAN' from dual union all
select 21727,'AE' from dual union all
select 21728,'RVSNR' from dual union all
select 21729,'RVSNR' from dual union all
select 2173,'NASA' from dual union all
select 21730,'RVSNR' from dual union all
select 21731,'RVSNR' from dual union all
select 21732,'RVSNR' from dual union all
select 21733,'RVSNR' from dual union all
select 21734,'RVSNR' from dual union all
select 21735,'NPOE' from dual union all
select 21736,'RVSN' from dual union all
select 21737,'TSSKB' from dual union all
select 21738,'RVSN' from dual union all
select 21739,'RVSN' from dual union all
select 2174,'NASA' from dual;
insert into SATELLITE_ORG
select 21740,'RVSN' from dual union all
select 21741,'TSSKB' from dual union all
select 21742,'RVSN' from dual union all
select 21743,'PVO' from dual union all
select 21744,'RVSN' from dual union all
select 21745,'RVSN' from dual union all
select 21746,'NPOE' from dual union all
select 21747,'RVSN' from dual union all
select 21748,'USAF' from dual union all
select 21749,'PVO' from dual union all
select 2175,'USAF' from dual union all
select 21750,'RVSN' from dual union all
select 21751,'RVSN' from dual union all
select 21752,'RVSN' from dual union all
select 21753,'RVSN' from dual union all
select 21754,'NASA' from dual union all
select 21755,'RVSNR' from dual union all
select 21756,'RVSNR' from dual union all
select 21757,'RVSNR' from dual union all
select 21758,'RVSNR' from dual union all
select 21759,'MSVYAZ' from dual union all
select 2176,'USN' from dual union all
select 21760,'RVSN' from dual union all
select 21761,'RVSN' from dual union all
select 21762,'RVSN' from dual union all
select 21763,'RVSN' from dual union all
select 21764,'RVSN' from dual union all
select 21765,'INTEL' from dual union all
select 21766,'AE' from dual union all
select 21767,'PVO' from dual union all
select 21768,'PVO' from dual union all
select 21769,'PVO' from dual union all
select 2177,'USAF' from dual union all
select 21770,'PVO' from dual union all
select 21771,'PVO' from dual union all
select 21772,'PVO' from dual union all
select 21773,'PVO' from dual union all
select 21774,'PVO' from dual union all
select 21775,'NRO' from dual union all
select 21775,'USN' from dual union all
select 21776,'USAF' from dual union all
select 21777,'NASA' from dual union all
select 21778,'NASA' from dual union all
select 21779,'RVSNR' from dual union all
select 2178,'USAF' from dual union all
select 21780,'RVSNR' from dual union all
select 21781,'RVSNR' from dual union all
select 21782,'RVSNR' from dual union all
select 21783,'RVSNR' from dual union all
select 21784,'RVSNR' from dual union all
select 21785,'RVSNR' from dual union all
select 21786,'ISASS' from dual union all
select 21787,'RVSNR' from dual union all
select 21788,'RVSN' from dual union all
select 21789,'RVSNR' from dual union all
select 2179,'USAF' from dual union all
select 21790,'RVSN' from dual union all
select 21791,'RVSN' from dual union all
select 21792,'RVSN' from dual union all
select 21793,'RVSN' from dual union all
select 21794,'RVSN' from dual union all
select 21795,'NASA' from dual union all
select 21796,'VMF' from dual union all
select 21797,'RVSN' from dual union all
select 21798,'USAF' from dual union all
select 21799,'NRO' from dual union all
select 21799,'USN' from dual union all
select 218,'NRO' from dual union all
select 2180,'NASA' from dual union all
select 21800,'USAF' from dual union all
select 21801,'USAF' from dual union all
select 21802,'ISASS' from dual union all
select 21803,'EUTEL' from dual union all
select 21804,'GD' from dual union all
select 21805,'USAF' from dual union all
select 21806,'USAF' from dual union all
select 21807,'USAF' from dual union all
select 21808,'NRO' from dual union all
select 21808,'USN' from dual union all
select 21809,'NRO' from dual union all
select 21809,'USN' from dual union all
select 2181,'CIA' from dual union all
select 2181,'NRO' from dual union all
select 21810,'NPOE' from dual union all
select 21811,'NPOE' from dual union all
select 21812,'NPOE' from dual union all
select 21813,'FTEL' from dual union all
select 21814,'INMAR' from dual union all
select 21815,'AE' from dual union all
select 21816,'VTSR' from dual union all
select 21817,'RVSNR' from dual union all
select 21818,'AE' from dual union all
select 21819,'IK' from dual union all
select 2182,'YUZH' from dual union all
select 21820,'RVSNR' from dual union all
select 21821,'RVSNR' from dual union all
select 21822,'RVSNR' from dual union all
select 21823,'RVSNR' from dual union all
select 21824,'RVSNR' from dual union all
select 21825,'USAF' from dual;
insert into SATELLITE_ORG
select 21826,'YUZH' from dual union all
select 21827,'YUZH' from dual union all
select 21828,'RVSNR' from dual union all
select 21829,'RVSNR' from dual union all
select 2183,'NASA' from dual union all
select 21830,'NPOE' from dual union all
select 21831,'NPOE' from dual union all
select 21832,'NPOE' from dual union all
select 21833,'CHIS' from dual union all
select 21834,'MAI' from dual union all
select 21835,'CSFR' from dual union all
select 21836,'USAF' from dual union all
select 21837,'NASA' from dual union all
select 21838,'NASA' from dual union all
select 21839,'NASA' from dual union all
select 2184,'NASA' from dual union all
select 21840,'NASA' from dual union all
select 21841,'NASA' from dual union all
select 21842,'YUZH' from dual union all
select 21843,'RVSNR' from dual union all
select 21844,'VKS' from dual union all
select 21845,'RVSNR' from dual union all
select 21846,'NASA' from dual union all
select 21847,'PVO' from dual union all
select 21848,'RVSNR' from dual union all
select 21849,'RVSNR' from dual union all
select 2185,'JPL' from dual union all
select 2185,'NASA' from dual union all
select 21850,'RVSNR' from dual union all
select 21851,'NPOE' from dual union all
select 21852,'RVSNR' from dual union all
select 21853,'VKS' from dual union all
select 21854,'VKS' from dual union all
select 21855,'VKS' from dual union all
select 21856,'RVSNR' from dual union all
select 21857,'RVSNR' from dual union all
select 21858,'RVSNR' from dual union all
select 21859,'AE' from dual union all
select 2186,'NASA' from dual union all
select 21860,'RVSN' from dual union all
select 21861,'RVSN' from dual union all
select 21862,'RVSNR' from dual union all
select 21863,'RVSNR' from dual union all
select 21864,'PVO' from dual union all
select 21865,'PVO' from dual union all
select 21866,'PVO' from dual union all
select 21867,'NASDA' from dual union all
select 21868,'NASDA' from dual union all
select 21869,'PVO' from dual union all
select 2187,'NASA' from dual union all
select 21870,'PVO' from dual union all
select 21871,'PVO' from dual union all
select 21872,'PVO' from dual union all
select 21873,'DOD' from dual union all
select 21873,'USAF' from dual union all
select 21874,'GD' from dual union all
select 21875,'VMFR' from dual union all
select 21876,'RVSNR' from dual union all
select 21877,'USAF' from dual union all
select 21878,'USN' from dual union all
select 21879,'NPOE' from dual union all
select 2188,'NASA' from dual union all
select 21880,'NPOE' from dual union all
select 21881,'NPOE' from dual union all
select 21882,'NPOE' from dual union all
select 21883,'NPOE' from dual union all
select 21884,'NPOE' from dual union all
select 21885,'NPOE' from dual union all
select 21886,'NPOE' from dual union all
select 21887,'NPOE' from dual union all
select 21888,'NPOE' from dual union all
select 21889,'NPOE' from dual union all
select 2189,'NASA' from dual union all
select 2189,'USAF' from dual union all
select 21890,'USAF' from dual union all
select 21891,'MDC' from dual union all
select 21892,'MDC' from dual union all
select 21893,'SCC' from dual union all
select 21894,'ARAB' from dual union all
select 21895,'AE' from dual union all
select 21896,'NPOE' from dual union all
select 21897,'VKS' from dual union all
select 21898,'RVSNR' from dual union all
select 21899,'RVSNR' from dual union all
select 219,'USAF' from dual union all
select 2190,'NASA' from dual union all
select 2190,'USAF' from dual union all
select 21900,'RVSNR' from dual union all
select 21901,'NPOE' from dual union all
select 21902,'VMFR' from dual union all
select 21903,'RVSNR' from dual union all
select 21904,'ESA' from dual union all
select 21905,'RVSNR' from dual union all
select 21906,'HEC' from dual union all
select 21907,'GD' from dual union all
select 21908,'NPOE' from dual union all
select 21909,'RVSNR' from dual union all
select 2191,'NASA' from dual union all
select 21910,'NASA' from dual union all
select 21911,'MAI' from dual;
insert into SATELLITE_ORG
select 21912,'VMFR' from dual union all
select 21913,'RVSN' from dual union all
select 21914,'NASA' from dual union all
select 21915,'NASA' from dual union all
select 21916,'NRO' from dual union all
select 21917,'NRO' from dual union all
select 21918,'RVSN' from dual union all
select 21919,'RVSN' from dual union all
select 2192,'NRO' from dual union all
select 21920,'VKS' from dual union all
select 21921,'RVSNR' from dual union all
select 21922,'GPKS' from dual union all
select 21923,'RVSNR' from dual union all
select 21924,'RVSNR' from dual union all
select 21925,'RVSNR' from dual union all
select 21926,'RVSNR' from dual union all
select 21927,'RVSNR' from dual union all
select 21928,'VKS' from dual union all
select 21929,'RVSNR' from dual union all
select 2193,'NASA' from dual union all
select 21930,'USAF' from dual union all
select 21931,'MDC' from dual union all
select 21932,'MDC' from dual union all
select 21933,'NASA' from dual union all
select 21934,'NASA' from dual union all
select 21935,'RVSN' from dual union all
select 21936,'RVSN' from dual union all
select 21937,'VMFR' from dual union all
select 21938,'RVSNR' from dual union all
select 21939,'FTEL' from dual union all
select 2194,'USAF' from dual union all
select 21940,'INMAR' from dual union all
select 21941,'AE' from dual union all
select 21942,'AE' from dual union all
select 21943,'RVSN' from dual union all
select 21944,'USAF' from dual union all
select 21945,'USN' from dual union all
select 21946,'NPOE' from dual union all
select 21947,'RVSNR' from dual union all
select 21948,'NASA' from dual union all
select 21949,'NRO' from dual union all
select 2195,'NASA' from dual union all
select 21950,'USAF' from dual union all
select 21951,'MVD' from dual union all
select 21952,'RVSNR' from dual union all
select 21953,'VTSR' from dual union all
select 21954,'RVSNR' from dual union all
select 21955,'RVSN' from dual union all
select 21956,'NRO' from dual union all
select 21957,'PVO' from dual union all
select 21958,'RVSN' from dual union all
select 21959,'RVSN' from dual union all
select 2196,'MO' from dual union all
select 21960,'RVSNR' from dual union all
select 21961,'YUZH' from dual union all
select 21962,'MO' from dual union all
select 21963,'NASA' from dual union all
select 21964,'TELK' from dual union all
select 21965,'MDC' from dual union all
select 21966,'MDC' from dual union all
select 21967,'ISRO' from dual union all
select 21968,'ISRO' from dual union all
select 21969,'NASA' from dual union all
select 2197,'RVSN' from dual union all
select 21970,'USAF' from dual union all
select 21971,'USAF' from dual union all
select 21972,'NASA' from dual union all
select 21973,'VKS' from dual union all
select 21974,'RVSNR' from dual union all
select 21975,'TSSKB' from dual union all
select 21976,'VKS' from dual union all
select 21977,'VKS' from dual union all
select 21978,'VKS' from dual union all
select 21979,'VKS' from dual union all
select 2198,'MO' from dual union all
select 21980,'VKS' from dual union all
select 21981,'VKS' from dual union all
select 21982,'VKS' from dual union all
select 21983,'VKS' from dual union all
select 21984,'RVSNR' from dual union all
select 21985,'NASA' from dual union all
select 21986,'USAF' from dual union all
select 21987,'NASA' from dual union all
select 21988,'MDC' from dual union all
select 21989,'INTEL' from dual union all
select 2199,'MO' from dual union all
select 21990,'GD' from dual union all
select 21991,'PVO' from dual union all
select 21992,'PVO' from dual union all
select 21993,'PVO' from dual union all
select 21994,'PVO' from dual union all
select 21995,'PVO' from dual union all
select 21996,'PVO' from dual union all
select 21997,'RVSN' from dual union all
select 21998,'MVD' from dual union all
select 21999,'RVSNR' from dual union all
select 22,'NASA' from dual union all
select 220,'USAF' from dual union all
select 2200,'USAF' from dual union all
select 22000,'NASA' from dual;
insert into SATELLITE_ORG
select 22001,'PVO' from dual union all
select 22002,'PVO' from dual union all
select 22003,'PVO' from dual union all
select 22004,'NPOE' from dual union all
select 22005,'RVSNR' from dual union all
select 22006,'VMFR' from dual union all
select 22007,'RVSNR' from dual union all
select 22008,'ISAS' from dual union all
select 22009,'DOD' from dual union all
select 22009,'USAF' from dual union all
select 2201,'USAF' from dual union all
select 22010,'GD' from dual union all
select 22011,'USAF' from dual union all
select 22012,'NASA' from dual union all
select 22013,'NASA' from dual union all
select 22014,'USAF' from dual union all
select 22015,'MDC' from dual union all
select 22016,'MDC' from dual union all
select 22017,'PVO' from dual union all
select 22018,'RVSNR' from dual union all
select 22019,'RVSNR' from dual union all
select 2202,'USAF' from dual union all
select 22020,'RVSNR' from dual union all
select 22021,'RVSNR' from dual union all
select 22022,'RVSNR' from dual union all
select 22023,'NPOE' from dual union all
select 22024,'NPOE' from dual union all
select 22025,'RVSN' from dual union all
select 22026,'TSSKB' from dual union all
select 22027,'ISRO' from dual union all
select 22028,'EUTEL' from dual union all
select 22029,'TSSKB' from dual union all
select 2203,'RVSN' from dual union all
select 22030,'TSSKB' from dual union all
select 22031,'TSSKB' from dual union all
select 22032,'AE' from dual union all
select 22033,'AE' from dual union all
select 22034,'VKS' from dual union all
select 22035,'VKS' from dual union all
select 22036,'VKS' from dual union all
select 22037,'VKS' from dual union all
select 22038,'VKS' from dual union all
select 22039,'VKS' from dual union all
select 2204,'RVSN' from dual union all
select 22040,'VKS' from dual union all
select 22041,'GPKS' from dual union all
select 22042,'RVSNR' from dual union all
select 22043,'RVSNR' from dual union all
select 22044,'RVSNR' from dual union all
select 22045,'NPOE' from dual union all
select 22046,'NPOE' from dual union all
select 22047,'RVSNR' from dual union all
select 22048,'RVSNR' from dual union all
select 22049,'ISASS' from dual union all
select 2205,'USA' from dual union all
select 22050,'MDC' from dual union all
select 22051,'MDC' from dual union all
select 22052,'VKS' from dual union all
select 22053,'RVSNR' from dual union all
select 22054,'NPOE' from dual union all
select 22055,'RVSNR' from dual union all
select 22056,'VKS' from dual union all
select 22057,'VKS' from dual union all
select 22058,'VKS' from dual union all
select 22059,'RVSNR' from dual union all
select 2206,'NASA' from dual union all
select 22060,'RVSNR' from dual union all
select 22061,'RVSNR' from dual union all
select 22062,'VKS' from dual union all
select 22063,'RVSNR' from dual union all
select 22064,'NASA' from dual union all
select 22065,'ESA' from dual union all
select 22066,'RVSNR' from dual union all
select 22067,'RVSNR' from dual union all
select 22068,'VKS' from dual union all
select 22069,'RVSNR' from dual union all
select 2207,'USAF' from dual union all
select 22070,'RVSNR' from dual union all
select 22071,'RVSNR' from dual union all
select 22072,'MAI' from dual union all
select 22073,'MAI' from dual union all
select 22074,'MAI' from dual union all
select 22075,'MAI' from dual union all
select 22076,'CNES' from dual union all
select 22076,'JPL' from dual union all
select 22077,'KAIST' from dual union all
select 22078,'CNES' from dual union all
select 22079,'AE' from dual union all
select 2208,'USAF' from dual union all
select 22080,'VKS' from dual union all
select 22081,'VKS' from dual union all
select 22082,'VKS' from dual union all
select 22083,'VKS' from dual union all
select 22084,'VKS' from dual union all
select 22085,'VKS' from dual union all
select 22086,'VKS' from dual union all
select 22087,'OPT' from dual union all
select 22088,'MAI' from dual union all
select 22089,'HEC' from dual union all
select 2209,'USAF' from dual;
insert into SATELLITE_ORG
select 22090,'NPOE' from dual union all
select 22091,'RVSNR' from dual union all
select 22092,'RVSNR' from dual union all
select 22093,'MVD' from dual union all
select 22094,'RVSNR' from dual union all
select 22095,'RVSNR' from dual union all
select 22096,'AMCG' from dual union all
select 22097,'MDC' from dual union all
select 22098,'MDC' from dual union all
select 22099,'RKA' from dual union all
select 22099,'SGAU' from dual union all
select 221,'JPL' from dual union all
select 221,'NASA' from dual union all
select 2210,'MO' from dual union all
select 22100,'RKA' from dual union all
select 22100,'SGAU' from dual union all
select 22101,'AE' from dual union all
select 22102,'TSSKB' from dual union all
select 22103,'TSSKB' from dual union all
select 22104,'TSSKB' from dual union all
select 22105,'TSSKB' from dual union all
select 22106,'NPOE' from dual union all
select 22107,'NPOE' from dual union all
select 22108,'USAF' from dual union all
select 22109,'MDC' from dual union all
select 2211,'RVSN' from dual union all
select 22110,'MDC' from dual union all
select 22111,'NPOE' from dual union all
select 22112,'PVO' from dual union all
select 22113,'RVSNR' from dual union all
select 22114,'RVSNR' from dual union all
select 22115,'RVSNR' from dual union all
select 22116,'HISP' from dual union all
select 22117,'AMCG' from dual union all
select 22118,'AE' from dual union all
select 22119,'AE' from dual union all
select 2212,'MO' from dual union all
select 22120,'NASA' from dual union all
select 22121,'NPOE' from dual union all
select 22122,'NPOE' from dual union all
select 22123,'NPOE' from dual union all
select 22124,'NPOE' from dual union all
select 22125,'RVSNR' from dual union all
select 22126,'RVSNR' from dual union all
select 22127,'NPOE' from dual union all
select 22128,'NPOE' from dual union all
select 22129,'NPOE' from dual union all
select 2213,'MO' from dual union all
select 22130,'NPOE' from dual union all
select 22131,'NPOE' from dual union all
select 22132,'RVSN' from dual union all
select 22133,'VKS' from dual union all
select 22134,'RVSNR' from dual union all
select 22135,'VKS' from dual union all
select 22136,'JPL' from dual union all
select 22137,'USAF' from dual union all
select 22138,'NASA' from dual union all
select 22138,'OSC' from dual union all
select 22139,'NASA' from dual union all
select 2214,'MO' from dual union all
select 22140,'RVSNR' from dual union all
select 22141,'RVSNR' from dual union all
select 22142,'RVSNR' from dual union all
select 22143,'RVSNR' from dual union all
select 22144,'RVSNR' from dual union all
select 22145,'RVSNR' from dual union all
select 22146,'RVSNR' from dual union all
select 22147,'RVSNR' from dual union all
select 22148,'RVSNR' from dual union all
select 22149,'RVSNR' from dual union all
select 2215,'USAF' from dual union all
select 22150,'RVSNR' from dual union all
select 22151,'RVSNR' from dual union all
select 22152,'RVSNR' from dual union all
select 22153,'RVSNR' from dual union all
select 22154,'RVSNR' from dual union all
select 22155,'RVSNR' from dual union all
select 22156,'RVSNR' from dual union all
select 22157,'RVSNR' from dual union all
select 22158,'RVSNR' from dual union all
select 22159,'RVSNR' from dual union all
select 2216,'USAF' from dual union all
select 22160,'RVSNR' from dual union all
select 22161,'SSC' from dual union all
select 22162,'MAI' from dual union all
select 22163,'MAI' from dual union all
select 22164,'MAI' from dual union all
select 22165,'MAI' from dual union all
select 22166,'MAI' from dual union all
select 22167,'MAI' from dual union all
select 22168,'MAI' from dual union all
select 22169,'MAI' from dual union all
select 2217,'USAF' from dual union all
select 22170,'MAI' from dual union all
select 22171,'NRO' from dual union all
select 22172,'USN' from dual union all
select 22173,'TSSKB' from dual union all
select 22174,'RVSNR' from dual union all
select 22175,'DBP' from dual union all
select 22176,'MDC' from dual;
insert into SATELLITE_ORG
select 22177,'MDC' from dual union all
select 22178,'GPKS' from dual union all
select 22178,'VKS' from dual union all
select 22179,'RVSNR' from dual union all
select 2218,'USAF' from dual union all
select 22180,'RVSNR' from dual union all
select 22181,'RVSNR' from dual union all
select 22182,'VKS' from dual union all
select 22183,'VKS' from dual union all
select 22184,'VKS' from dual union all
select 22185,'VKS' from dual union all
select 22186,'VKS' from dual union all
select 22187,'VKS' from dual union all
select 22188,'VKS' from dual union all
select 22189,'PVO' from dual union all
select 2219,'USAF' from dual union all
select 22190,'RVSNR' from dual union all
select 22191,'RVSNR' from dual union all
select 22192,'RVSNR' from dual union all
select 22193,'RVSNR' from dual union all
select 22194,'NASA' from dual union all
select 22195,'ASI' from dual union all
select 22196,'ASI' from dual union all
select 22197,'ASI' from dual union all
select 22198,'RVSN' from dual union all
select 22199,'TSSKB' from dual union all
select 222,'NASA' from dual union all
select 2220,'USAF' from dual union all
select 22200,'TSSKB' from dual union all
select 22201,'TSSKB' from dual union all
select 22202,'TSSKB' from dual union all
select 22203,'NPOE' from dual union all
select 22204,'RVSNR' from dual union all
select 22205,'HEC' from dual union all
select 22206,'AE' from dual union all
select 22207,'VMFR' from dual union all
select 22208,'VKS' from dual union all
select 22209,'NPOE' from dual union all
select 2221,'USAF' from dual union all
select 22210,'GPKS' from dual union all
select 22211,'RVSNR' from dual union all
select 22212,'RVSNR' from dual union all
select 22213,'RVSNR' from dual union all
select 22214,'CSA' from dual union all
select 22215,'RVSNR' from dual union all
select 22216,'RVSNR' from dual union all
select 22217,'TSSKB' from dual union all
select 22218,'RVSNR' from dual union all
select 22219,'MORF' from dual union all
select 2222,'USAF' from dual union all
select 22220,'VKS' from dual union all
select 22221,'VKS' from dual union all
select 22222,'VKS' from dual union all
select 22223,'VKS' from dual union all
select 22224,'VKS' from dual union all
select 22225,'NPOE' from dual union all
select 22226,'VKS' from dual union all
select 22227,'RVSNR' from dual union all
select 22228,'NPOE' from dual union all
select 22229,'SDIO' from dual union all
select 22229,'USAF' from dual union all
select 2223,'NASA' from dual union all
select 22230,'NASA' from dual union all
select 22231,'USAF' from dual union all
select 22232,'MDC' from dual union all
select 22233,'MDC' from dual union all
select 22234,'TSSKB' from dual union all
select 22235,'TSSKB' from dual union all
select 22236,'MORF' from dual union all
select 22237,'VKS' from dual union all
select 22238,'PVO' from dual union all
select 22239,'RVSNR' from dual union all
select 2224,'NASA' from dual union all
select 22240,'RVSNR' from dual union all
select 22241,'RVSNR' from dual union all
select 22242,'RVSNR' from dual union all
select 22243,'RVSNR' from dual union all
select 22244,'RVSNR' from dual union all
select 22245,'GPKS' from dual union all
select 22246,'RVSNR' from dual union all
select 22247,'RVSNR' from dual union all
select 22248,'RVSNR' from dual union all
select 22249,'RVSNR' from dual union all
select 2225,'USN' from dual union all
select 22250,'RVSNR' from dual union all
select 22251,'CIA' from dual union all
select 22251,'NRO' from dual union all
select 22252,'USAF' from dual union all
select 22253,'SCC' from dual union all
select 22254,'AE' from dual union all
select 22255,'GPKS' from dual union all
select 22255,'VKS' from dual union all
select 22256,'RVSNR' from dual union all
select 22257,'RVSNR' from dual union all
select 22258,'RVSNR' from dual union all
select 22259,'NASA' from dual union all
select 2226,'USN' from dual union all
select 22260,'VKS' from dual union all
select 22261,'RVSNR' from dual union all
select 22262,'RVSNR' from dual;
insert into SATELLITE_ORG
select 22263,'USAF' from dual union all
select 22264,'USAF' from dual union all
select 22265,'USAF' from dual union all
select 22266,'NASDA' from dual union all
select 22267,'USAF' from dual union all
select 22268,'USAF' from dual union all
select 22269,'PVO' from dual union all
select 2227,'CIA' from dual union all
select 2227,'NRO' from dual union all
select 22270,'RVSNR' from dual union all
select 22271,'RVSNR' from dual union all
select 22272,'RVSNR' from dual union all
select 22273,'RVSNR' from dual union all
select 22274,'RVSNR' from dual union all
select 22275,'USAF' from dual union all
select 22276,'MDC' from dual union all
select 22277,'MDC' from dual union all
select 22278,'HEC' from dual union all
select 22279,'MAI' from dual union all
select 2228,'NASA' from dual union all
select 2228,'USAF' from dual union all
select 22280,'VKS' from dual union all
select 22281,'RVSNR' from dual union all
select 22282,'VTSR' from dual union all
select 22283,'VKS' from dual union all
select 22284,'MORF' from dual union all
select 22285,'VKS' from dual union all
select 22286,'MORF' from dual union all
select 22287,'VKS' from dual union all
select 22288,'RVSNR' from dual union all
select 22289,'RVSNR' from dual union all
select 2229,'NASA' from dual union all
select 2229,'USAF' from dual union all
select 22290,'VKS' from dual union all
select 22291,'VKS' from dual union all
select 22292,'VKS' from dual union all
select 22293,'VKS' from dual union all
select 22294,'RVSNR' from dual union all
select 22295,'RVSNR' from dual union all
select 22296,'RVSNR' from dual union all
select 22297,'RVSNR' from dual union all
select 22298,'RVSNR' from dual union all
select 22299,'VKS' from dual union all
select 223,'USAF' from dual union all
select 2230,'NASA' from dual union all
select 2230,'USAF' from dual union all
select 22300,'IMBP' from dual union all
select 22300,'TSSKB' from dual union all
select 22301,'RVSNR' from dual union all
select 22302,'TSSKB' from dual union all
select 22303,'TSSKB' from dual union all
select 22304,'TSSKB' from dual union all
select 22305,'TSSKB' from dual union all
select 22306,'TSSKB' from dual union all
select 22307,'VMFR' from dual union all
select 22308,'VKS' from dual union all
select 22309,'VKS' from dual union all
select 2231,'NASA' from dual union all
select 2231,'USAF' from dual union all
select 22310,'RVSNR' from dual union all
select 22311,'RVSNR' from dual union all
select 22312,'RVSNR' from dual union all
select 22313,'NASA' from dual union all
select 22314,'NASA' from dual union all
select 22315,'NASA' from dual union all
select 22316,'NASA' from dual union all
select 22317,'VKS' from dual union all
select 22318,'RVSNR' from dual union all
select 22319,'NPOE' from dual union all
select 2232,'NASA' from dual union all
select 2232,'USAF' from dual union all
select 22320,'RVSNR' from dual union all
select 22321,'PVO' from dual union all
select 22322,'RVSNR' from dual union all
select 22323,'RVSNR' from dual union all
select 22324,'RVSNR' from dual union all
select 22325,'RVSNR' from dual union all
select 22326,'RVSNR' from dual union all
select 22327,'RVSNR' from dual union all
select 22328,'RVSNR' from dual union all
select 22329,'RVSNR' from dual union all
select 2233,'NASA' from dual union all
select 2233,'USAF' from dual union all
select 22330,'RVSNR' from dual union all
select 22331,'RVSNR' from dual union all
select 22332,'RVSNR' from dual union all
select 22333,'RVSNR' from dual union all
select 22334,'RVSNR' from dual union all
select 22335,'RVSNR' from dual union all
select 22336,'RVSNR' from dual union all
select 22337,'RVSNR' from dual union all
select 22338,'RVSNR' from dual union all
select 22339,'RVSNR' from dual union all
select 2234,'NASA' from dual union all
select 2234,'USAF' from dual union all
select 22340,'RVSNR' from dual union all
select 22341,'RVSNR' from dual union all
select 22342,'RVSNR' from dual union all
select 22343,'RVSNR' from dual union all
select 22344,'RVSNR' from dual;
insert into SATELLITE_ORG
select 22345,'RVSNR' from dual union all
select 22346,'RVSNR' from dual union all
select 22347,'RVSNR' from dual union all
select 22348,'RVSNR' from dual union all
select 22349,'RVSNR' from dual union all
select 2235,'NASA' from dual union all
select 2235,'USAF' from dual union all
select 22350,'RVSNR' from dual union all
select 22351,'RVSNR' from dual union all
select 22352,'RVSNR' from dual union all
select 22353,'RVSNR' from dual union all
select 22354,'RVSNR' from dual union all
select 22355,'RVSNR' from dual union all
select 22356,'RVSNR' from dual union all
select 22357,'RVSNR' from dual union all
select 22358,'RVSNR' from dual union all
select 22359,'RVSNR' from dual union all
select 2236,'NASA' from dual union all
select 2236,'USAF' from dual union all
select 22360,'RVSNR' from dual union all
select 22361,'RVSNR' from dual union all
select 22362,'RVSNR' from dual union all
select 22363,'RVSNR' from dual union all
select 22364,'RVSNR' from dual union all
select 22365,'RVSNR' from dual union all
select 22366,'RVSNR' from dual union all
select 22367,'RVSNR' from dual union all
select 22368,'RVSNR' from dual union all
select 22369,'RVSNR' from dual union all
select 2237,'NASA' from dual union all
select 2237,'USAF' from dual union all
select 22370,'RVSNR' from dual union all
select 22371,'RVSNR' from dual union all
select 22372,'RVSNR' from dual union all
select 22373,'RVSNR' from dual union all
select 22374,'RVSNR' from dual union all
select 22375,'RVSNR' from dual union all
select 22376,'RVSNR' from dual union all
select 22377,'RVSNR' from dual union all
select 22378,'RVSNR' from dual union all
select 22379,'RVSNR' from dual union all
select 2238,'NASA' from dual union all
select 2238,'USAF' from dual union all
select 22380,'RVSNR' from dual union all
select 22381,'RVSNR' from dual union all
select 22382,'RVSNR' from dual union all
select 22383,'RVSNR' from dual union all
select 22384,'RVSNR' from dual union all
select 22385,'RVSNR' from dual union all
select 22386,'RVSNR' from dual union all
select 22387,'RVSNR' from dual union all
select 22388,'RVSNR' from dual union all
select 22389,'RVSNR' from dual union all
select 2239,'NASA' from dual union all
select 2239,'USAF' from dual union all
select 22390,'RVSNR' from dual union all
select 22391,'RVSNR' from dual union all
select 22392,'RVSNR' from dual union all
select 22393,'RVSNR' from dual union all
select 22394,'RVSNR' from dual union all
select 22395,'RVSNR' from dual union all
select 22396,'RVSNR' from dual union all
select 22397,'RVSNR' from dual union all
select 22398,'RVSNR' from dual union all
select 22399,'RVSNR' from dual union all
select 224,'USAF' from dual union all
select 2240,'NASA' from dual union all
select 2240,'USAF' from dual union all
select 22400,'RVSNR' from dual union all
select 22401,'RVSNR' from dual union all
select 22402,'RVSNR' from dual union all
select 22403,'RVSNR' from dual union all
select 22404,'RVSNR' from dual union all
select 22405,'RVSNR' from dual union all
select 22406,'RVSNR' from dual union all
select 22407,'RVSNR' from dual union all
select 22408,'RVSNR' from dual union all
select 22409,'RVSNR' from dual union all
select 2241,'NASA' from dual union all
select 2241,'USAF' from dual union all
select 22410,'RVSNR' from dual union all
select 22411,'RVSNR' from dual union all
select 22412,'RVSNR' from dual union all
select 22413,'RVSNR' from dual union all
select 22414,'RVSNR' from dual union all
select 22415,'RVSNR' from dual union all
select 22416,'RVSNR' from dual union all
select 22417,'RVSNR' from dual union all
select 22418,'RVSNR' from dual union all
select 22419,'RVSNR' from dual union all
select 2242,'NASA' from dual union all
select 2242,'USAF' from dual union all
select 22420,'RVSNR' from dual union all
select 22421,'RVSNR' from dual union all
select 22422,'RVSNR' from dual union all
select 22423,'RVSNR' from dual union all
select 22424,'RVSNR' from dual union all
select 22425,'RVSNR' from dual union all
select 22426,'RVSNR' from dual union all
select 22427,'RVSNR' from dual;
insert into SATELLITE_ORG
select 22428,'RVSNR' from dual union all
select 22429,'RVSNR' from dual union all
select 2243,'NASA' from dual union all
select 2243,'USAF' from dual union all
select 22430,'RVSNR' from dual union all
select 22431,'RVSNR' from dual union all
select 22432,'RVSNR' from dual union all
select 22433,'RVSNR' from dual union all
select 22434,'RVSNR' from dual union all
select 22435,'RVSNR' from dual union all
select 22436,'RVSNR' from dual union all
select 22437,'RVSNR' from dual union all
select 22438,'RVSNR' from dual union all
select 22439,'RVSNR' from dual union all
select 2244,'NASA' from dual union all
select 2244,'USAF' from dual union all
select 22440,'RVSNR' from dual union all
select 22441,'RVSNR' from dual union all
select 22442,'RVSNR' from dual union all
select 22443,'RVSNR' from dual union all
select 22444,'RVSNR' from dual union all
select 22445,'RVSNR' from dual union all
select 22446,'USAF' from dual union all
select 22447,'MDC' from dual union all
select 22448,'MDC' from dual union all
select 22449,'NPOE' from dual union all
select 2245,'NASA' from dual union all
select 2245,'USAF' from dual union all
select 22450,'RVSNR' from dual union all
select 22451,'RVSNR' from dual union all
select 22452,'RVSNR' from dual union all
select 22453,'RVSNR' from dual union all
select 22454,'RVSNR' from dual union all
select 22455,'RVSNR' from dual union all
select 22456,'RVSNR' from dual union all
select 22457,'RVSNR' from dual union all
select 22458,'RVSNR' from dual union all
select 22459,'RVSNR' from dual union all
select 2246,'NASA' from dual union all
select 2246,'USAF' from dual union all
select 22460,'RVSNR' from dual union all
select 22461,'RVSNR' from dual union all
select 22462,'RVSNR' from dual union all
select 22463,'RVSNR' from dual union all
select 22464,'RVSNR' from dual union all
select 22465,'RVSNR' from dual union all
select 22466,'RVSNR' from dual union all
select 22467,'RVSNR' from dual union all
select 22468,'RVSNR' from dual union all
select 22469,'RVSNR' from dual union all
select 2247,'NASA' from dual union all
select 2247,'USAF' from dual union all
select 22470,'RVSNR' from dual union all
select 22471,'RVSNR' from dual union all
select 22472,'RVSNR' from dual union all
select 22473,'RVSNR' from dual union all
select 22474,'RVSNR' from dual union all
select 22475,'RVSNR' from dual union all
select 22476,'RVSNR' from dual union all
select 22477,'RVSNR' from dual union all
select 22478,'RVSNR' from dual union all
select 22479,'RVSNR' from dual union all
select 2248,'NASA' from dual union all
select 2248,'USAF' from dual union all
select 22480,'RVSNR' from dual union all
select 22481,'RVSNR' from dual union all
select 22482,'RVSNR' from dual union all
select 22483,'RVSNR' from dual union all
select 22484,'RVSNR' from dual union all
select 22485,'RVSNR' from dual union all
select 22486,'PVO' from dual union all
select 22487,'VMFR' from dual union all
select 22488,'VKS' from dual union all
select 22489,'ORBC' from dual union all
select 2249,'NASA' from dual union all
select 2249,'USAF' from dual union all
select 22490,'INPE' from dual union all
select 22491,'OSC' from dual union all
select 22492,'RVSNR' from dual union all
select 22493,'RVSNR' from dual union all
select 22494,'RVSNR' from dual union all
select 22495,'RVSNR' from dual union all
select 22496,'RVSNR' from dual union all
select 22497,'RVSNR' from dual union all
select 22498,'RVSNR' from dual union all
select 22499,'RVSNR' from dual union all
select 225,'USAF' from dual union all
select 2250,'NASA' from dual union all
select 2250,'USAF' from dual union all
select 22500,'RVSNR' from dual union all
select 22501,'RVSNR' from dual union all
select 22502,'RVSNR' from dual union all
select 22503,'RVSNR' from dual union all
select 22504,'RVSNR' from dual union all
select 22505,'RVSNR' from dual union all
select 22506,'RVSNR' from dual union all
select 22507,'RVSNR' from dual union all
select 22508,'RVSNR' from dual union all
select 22509,'RVSNR' from dual union all
select 2251,'NASA' from dual;
insert into SATELLITE_ORG
select 2251,'USAF' from dual union all
select 22510,'RVSNR' from dual union all
select 22511,'RVSNR' from dual union all
select 22512,'VKS' from dual union all
select 22513,'VKS' from dual union all
select 22514,'VKS' from dual union all
select 22515,'RVSNR' from dual union all
select 22516,'RVSNR' from dual union all
select 22517,'RVSNR' from dual union all
select 22518,'NROC' from dual union all
select 22518,'USAF' from dual union all
select 22519,'USAF' from dual union all
select 2252,'NASA' from dual union all
select 2252,'USAF' from dual union all
select 22520,'VKS' from dual union all
select 22521,'ISASS' from dual union all
select 22522,'ISASS' from dual union all
select 22523,'ISASS' from dual union all
select 22524,'RVSNR' from dual union all
select 22525,'VKS' from dual union all
select 22526,'VKS' from dual union all
select 22527,'VKS' from dual union all
select 22528,'RVSNR' from dual union all
select 22529,'VKS' from dual union all
select 2253,'NASA' from dual union all
select 22530,'NPOE' from dual union all
select 22531,'RVSNR' from dual union all
select 22532,'NASA' from dual union all
select 22533,'NASA' from dual union all
select 22534,'ISASS' from dual union all
select 22535,'ASI' from dual union all
select 22536,'VKS' from dual union all
select 22537,'RVSNR' from dual union all
select 22538,'RVSNR' from dual union all
select 22539,'RVSNR' from dual union all
select 2254,'METP' from dual union all
select 22540,'RVSNR' from dual union all
select 22541,'RVSNR' from dual union all
select 22542,'RVSNR' from dual union all
select 22543,'RVSNR' from dual union all
select 22544,'RVSNR' from dual union all
select 22545,'RVSNR' from dual union all
select 22546,'RVSNR' from dual union all
select 22547,'RVSNR' from dual union all
select 22548,'RVSNR' from dual union all
select 22549,'RVSNR' from dual union all
select 2255,'NASA' from dual union all
select 22550,'RVSNR' from dual union all
select 22551,'RVSNR' from dual union all
select 22552,'RVSNR' from dual union all
select 22553,'RVSNR' from dual union all
select 22554,'RVSNR' from dual union all
select 22555,'RVSNR' from dual union all
select 22556,'RVSNR' from dual union all
select 22557,'VKS' from dual union all
select 22558,'RVSNR' from dual union all
select 22559,'RVSNR' from dual union all
select 2256,'NASA' from dual union all
select 22560,'NASA' from dual union all
select 22561,'VKS' from dual union all
select 22562,'RVSNR' from dual union all
select 22563,'HEC' from dual union all
select 22564,'GD' from dual union all
select 22565,'MORF' from dual union all
select 22566,'VKS' from dual union all
select 22567,'RVSNR' from dual union all
select 22568,'RVSNR' from dual union all
select 22569,'RVSNR' from dual union all
select 2257,'RVSN' from dual union all
select 22570,'RVSNR' from dual union all
select 22571,'RVSNR' from dual union all
select 22572,'RVSNR' from dual union all
select 22573,'RVSNR' from dual union all
select 22574,'RVSNR' from dual union all
select 22575,'VKS' from dual union all
select 22576,'VKS' from dual union all
select 22577,'VKS' from dual union all
select 22578,'VKS' from dual union all
select 22579,'RVSNR' from dual union all
select 2258,'NASA' from dual union all
select 22580,'RVSNR' from dual union all
select 22581,'USAF' from dual union all
select 22582,'NASA' from dual union all
select 22583,'MDC' from dual union all
select 22584,'MDC' from dual union all
select 22585,'VMFR' from dual union all
select 22586,'VKS' from dual union all
select 22587,'ISASS' from dual union all
select 22588,'NPOE' from dual union all
select 22589,'RVSNR' from dual union all
select 2259,'NASA' from dual union all
select 22590,'VMFR' from dual union all
select 22591,'VKS' from dual union all
select 22592,'VKS' from dual union all
select 22593,'RVSNR' from dual union all
select 22594,'PVO' from dual union all
select 22595,'RVSNR' from dual union all
select 22596,'RVSNR' from dual union all
select 22597,'RVSNR' from dual union all
select 22598,'RVSNR' from dual;
insert into SATELLITE_ORG
select 22599,'RVSNR' from dual union all
select 226,'NASA' from dual union all
select 2260,'NASA' from dual union all
select 22600,'RVSNR' from dual union all
select 22601,'RVSNR' from dual union all
select 22602,'RVSNR' from dual union all
select 22603,'RVSNR' from dual union all
select 22604,'RVSNR' from dual union all
select 22605,'RVSNR' from dual union all
select 22606,'RVSNR' from dual union all
select 22607,'RVSNR' from dual union all
select 22608,'RVSNR' from dual union all
select 22609,'RVSNR' from dual union all
select 2261,'CIA' from dual union all
select 2261,'NRO' from dual union all
select 22610,'RVSNR' from dual union all
select 22611,'RVSNR' from dual union all
select 22612,'RVSNR' from dual union all
select 22613,'RVSNR' from dual union all
select 22614,'RVSNR' from dual union all
select 22615,'RVSNR' from dual union all
select 22616,'RVSNR' from dual union all
select 22617,'RVSNR' from dual union all
select 22618,'RVSNR' from dual union all
select 22619,'RVSNR' from dual union all
select 2262,'NASA' from dual union all
select 22620,'RVSNR' from dual union all
select 22621,'NASA' from dual union all
select 22622,'NPOE' from dual union all
select 22623,'NASA' from dual union all
select 22624,'RVSNR' from dual union all
select 22625,'NASA' from dual union all
select 22626,'MORF' from dual union all
select 22627,'VKS' from dual union all
select 22628,'ISASS' from dual union all
select 22629,'RVSNR' from dual union all
select 2263,'NASA' from dual union all
select 22630,'RVSNR' from dual union all
select 22631,'NPOE' from dual union all
select 22632,'NPOE' from dual union all
select 22633,'GPKS' from dual union all
select 22633,'VKS' from dual union all
select 22634,'RVSNR' from dual union all
select 22635,'RVSNR' from dual union all
select 22636,'RVSNR' from dual union all
select 22637,'NPOE' from dual union all
select 22638,'LANL' from dual union all
select 22638,'USAF' from dual union all
select 22639,'ORBC' from dual union all
select 2264,'NASA' from dual union all
select 2264,'USAF' from dual union all
select 22640,'NASA' from dual union all
select 22641,'VTSR' from dual union all
select 22642,'RVSNR' from dual union all
select 22643,'VMFR' from dual union all
select 22644,'VKS' from dual union all
select 22645,'RVSNR' from dual union all
select 22646,'VKS' from dual union all
select 22647,'VKS' from dual union all
select 22648,'VKS' from dual union all
select 22649,'VKS' from dual union all
select 2265,'NASA' from dual union all
select 2265,'USAF' from dual union all
select 22650,'VKS' from dual union all
select 22651,'VKS' from dual union all
select 22652,'VKS' from dual union all
select 22653,'SES' from dual union all
select 22654,'RACE' from dual union all
select 22655,'AE' from dual union all
select 22656,'AE' from dual union all
select 22657,'USAF' from dual union all
select 22658,'MDC' from dual union all
select 22659,'MDC' from dual union all
select 2266,'NASA' from dual union all
select 2266,'USAF' from dual union all
select 22660,'RVSNR' from dual union all
select 22661,'RVSNR' from dual union all
select 22662,'RVSNR' from dual union all
select 22663,'MVD' from dual union all
select 22664,'RVSNR' from dual union all
select 22665,'RVSNR' from dual union all
select 22666,'NPOE' from dual union all
select 22667,'RVSNR' from dual union all
select 22668,'USAF' from dual union all
select 22669,'USAF' from dual union all
select 2267,'NASA' from dual union all
select 2267,'USAF' from dual union all
select 22670,'AE' from dual union all
select 22671,'VKS' from dual union all
select 22672,'RVSNR' from dual union all
select 22673,'RVSNR' from dual union all
select 22674,'RVSNR' from dual union all
select 22675,'VKS' from dual union all
select 22676,'VKS' from dual union all
select 22677,'NPOE' from dual union all
select 22678,'NPOE' from dual union all
select 22679,'NPOE' from dual union all
select 2268,'NASA' from dual union all
select 2268,'USAF' from dual union all
select 22680,'NPOE' from dual;
insert into SATELLITE_ORG
select 22681,'TSSKB' from dual union all
select 22682,'TSSKB' from dual union all
select 22683,'NASA' from dual union all
select 22684,'NASA' from dual union all
select 22685,'TSSKB' from dual union all
select 22686,'TSSKB' from dual union all
select 22687,'VKS' from dual union all
select 22688,'VKS' from dual union all
select 22689,'VKS' from dual union all
select 2269,'NASA' from dual union all
select 2269,'USAF' from dual union all
select 22690,'VKS' from dual union all
select 22691,'VKS' from dual union all
select 22692,'VKS' from dual union all
select 22693,'VKS' from dual union all
select 22694,'HEC' from dual union all
select 22695,'NASA' from dual union all
select 22696,'MVD' from dual union all
select 22697,'RVSNR' from dual union all
select 22698,'USAF' from dual union all
select 22699,'NASA' from dual union all
select 227,'NASA' from dual union all
select 2270,'NASA' from dual union all
select 2270,'USAF' from dual union all
select 22700,'USAF' from dual union all
select 22701,'MDC' from dual union all
select 22702,'MDC' from dual union all
select 22703,'NPOE' from dual union all
select 22704,'NPOE' from dual union all
select 22705,'RVSNR' from dual union all
select 22706,'USAF' from dual union all
select 22707,'USAF' from dual union all
select 22708,'NPOE' from dual union all
select 22709,'VMFR' from dual union all
select 2271,'NASA' from dual union all
select 2271,'USAF' from dual union all
select 22710,'VKS' from dual union all
select 22711,'TSSKB' from dual union all
select 22712,'TSSKB' from dual union all
select 22713,'TSSKB' from dual union all
select 22714,'TSSKB' from dual union all
select 22715,'TSSKB' from dual union all
select 22716,'VKS' from dual union all
select 22717,'RVSNR' from dual union all
select 22718,'USAF' from dual union all
select 22719,'DOD' from dual union all
select 22719,'USAF' from dual union all
select 2272,'NASA' from dual union all
select 2272,'USAF' from dual union all
select 22720,'GD' from dual union all
select 22721,'MVD' from dual union all
select 22721,'VKS' from dual union all
select 22722,'RVSNR' from dual union all
select 22723,'HISP' from dual union all
select 22724,'ISRO' from dual union all
select 22725,'AE' from dual union all
select 22726,'AE' from dual union all
select 22727,'NASA' from dual union all
select 22728,'NASA' from dual union all
select 22729,'GPKS' from dual union all
select 22729,'VKS' from dual union all
select 2273,'NASA' from dual union all
select 2273,'USAF' from dual union all
select 22730,'RVSNR' from dual union all
select 22731,'RVSNR' from dual union all
select 22732,'RVSNR' from dual union all
select 22733,'TSSKB' from dual union all
select 22734,'TSSKB' from dual union all
select 22735,'TSSKB' from dual union all
select 22736,'TSSKB' from dual union all
select 22737,'TSSKB' from dual union all
select 22738,'USAF' from dual union all
select 22739,'NOAA' from dual union all
select 2274,'NASA' from dual union all
select 2274,'USAF' from dual union all
select 22740,'NASA' from dual union all
select 22741,'PVO' from dual union all
select 22742,'RVSNR' from dual union all
select 22743,'RVSNR' from dual union all
select 22744,'RVSNR' from dual union all
select 22745,'NPOE' from dual union all
select 22746,'RVSNR' from dual union all
select 22747,'NASA' from dual union all
select 22748,'NASA' from dual union all
select 22749,'NASA' from dual union all
select 2275,'NASA' from dual union all
select 2275,'USAF' from dual union all
select 22750,'NASA' from dual union all
select 22751,'NASA' from dual union all
select 22752,'NASA' from dual union all
select 22753,'NASA' from dual union all
select 22754,'NASA' from dual union all
select 22755,'NASA' from dual union all
select 22756,'NASA' from dual union all
select 22757,'NASA' from dual union all
select 22758,'NASA' from dual union all
select 22759,'NASA' from dual union all
select 2276,'NASA' from dual union all
select 2276,'USAF' from dual union all
select 22760,'NASA' from dual;
insert into SATELLITE_ORG
select 22761,'NASA' from dual union all
select 22762,'NASA' from dual union all
select 22763,'NASA' from dual union all
select 22764,'NASA' from dual union all
select 22765,'NASA' from dual union all
select 22766,'NASA' from dual union all
select 22767,'NASA' from dual union all
select 22768,'NASA' from dual union all
select 22769,'NASA' from dual union all
select 2277,'NASA' from dual union all
select 2277,'USAF' from dual union all
select 22770,'NASA' from dual union all
select 22771,'NASA' from dual union all
select 22772,'NASA' from dual union all
select 22773,'NASA' from dual union all
select 22774,'NASA' from dual union all
select 22775,'NASA' from dual union all
select 22776,'NASA' from dual union all
select 22777,'MVD' from dual union all
select 22778,'RVSNR' from dual union all
select 22779,'USAF' from dual union all
select 2278,'NASA' from dual union all
select 2278,'USAF' from dual union all
select 22780,'MDC' from dual union all
select 22781,'MDC' from dual union all
select 22782,'RGMS' from dual union all
select 22782,'VKS' from dual union all
select 22783,'TSPZ' from dual union all
select 22784,'VKS' from dual union all
select 22785,'USAF' from dual union all
select 22786,'USAF' from dual union all
select 22787,'HEC' from dual union all
select 22788,'GD' from dual union all
select 22789,'VKS' from dual union all
select 2279,'NASA' from dual union all
select 2279,'USAF' from dual union all
select 22790,'RVSNR' from dual union all
select 22791,'TSSKB' from dual union all
select 22792,'TSSKB' from dual union all
select 22793,'TSSKB' from dual union all
select 22794,'TSSKB' from dual union all
select 22795,'NASA' from dual union all
select 22796,'NASA' from dual union all
select 22797,'NASA' from dual union all
select 22797,'OSCD' from dual union all
select 22798,'DARA' from dual union all
select 22799,'NASA' from dual union all
select 228,'NASA' from dual union all
select 2280,'NASA' from dual union all
select 2280,'USAF' from dual union all
select 22800,'NASA' from dual union all
select 22801,'NASA' from dual union all
select 22802,'MORF' from dual union all
select 22803,'VKS' from dual union all
select 22804,'VKS' from dual union all
select 22805,'VKS' from dual union all
select 22806,'VKS' from dual union all
select 22807,'VKS' from dual union all
select 22808,'VMFR' from dual union all
select 22809,'VKS' from dual union all
select 2281,'NASA' from dual union all
select 2281,'USAF' from dual union all
select 22810,'NPOE' from dual union all
select 22811,'NPOE' from dual union all
select 22812,'RVSNR' from dual union all
select 22813,'NRO' from dual union all
select 22814,'NRO' from dual union all
select 22815,'NPOE' from dual union all
select 22816,'RVSNR' from dual union all
select 22817,'NPOE' from dual union all
select 22818,'USAF' from dual union all
select 22819,'NPOE' from dual union all
select 2282,'NASA' from dual union all
select 2282,'USAF' from dual union all
select 22820,'NASA' from dual union all
select 22821,'ISASS' from dual union all
select 22822,'ISASS' from dual union all
select 22823,'CNES' from dual union all
select 22824,'CNES' from dual union all
select 22825,'INFER' from dual union all
select 22826,'SATEL' from dual union all
select 22827,'AMSAT' from dual union all
select 22828,'KAIST' from dual union all
select 22829,'INETI' from dual union all
select 2283,'NASA' from dual union all
select 2283,'USAF' from dual union all
select 22830,'AE' from dual union all
select 22831,'NPOE' from dual union all
select 22832,'NPOE' from dual union all
select 22833,'NPOE' from dual union all
select 22834,'NPOE' from dual union all
select 22835,'NPOE' from dual union all
select 22836,'VKS' from dual union all
select 22837,'RVSNR' from dual union all
select 22838,'RVSNR' from dual union all
select 22839,'RVSNR' from dual union all
select 2284,'NASA' from dual union all
select 2284,'USAF' from dual union all
select 22840,'RVSN' from dual union all
select 22841,'RVSN' from dual;
insert into SATELLITE_ORG
select 22842,'RVSN' from dual union all
select 22843,'RVSN' from dual union all
select 22844,'ISASS' from dual union all
select 22845,'RVSNR' from dual union all
select 22846,'RVSNR' from dual union all
select 22847,'NASA' from dual union all
select 22848,'NASA' from dual union all
select 22849,'USAF' from dual union all
select 2285,'NASA' from dual union all
select 2285,'USAF' from dual union all
select 22850,'AE' from dual union all
select 22851,'AE' from dual union all
select 22852,'NASA' from dual union all
select 22853,'NASA' from dual union all
select 22854,'NASA' from dual union all
select 22855,'NASA' from dual union all
select 22856,'NASA' from dual union all
select 22857,'NASA' from dual union all
select 22858,'NASA' from dual union all
select 22859,'CASC' from dual union all
select 2286,'NASA' from dual union all
select 2286,'USAF' from dual union all
select 22860,'CNSA' from dual union all
select 22861,'CNSA' from dual union all
select 22862,'CNSA' from dual union all
select 22863,'CNSA' from dual union all
select 22864,'CNSA' from dual union all
select 22865,'CNSA' from dual union all
select 22866,'NPOE' from dual union all
select 22867,'NPOE' from dual union all
select 22868,'RVSNR' from dual union all
select 22869,'NASA' from dual union all
select 2287,'NASA' from dual union all
select 2287,'USAF' from dual union all
select 22870,'CASC' from dual union all
select 22871,'INTEL' from dual union all
select 22872,'NPOE' from dual union all
select 22873,'NPOE' from dual union all
select 22874,'NPOE' from dual union all
select 22875,'PVO' from dual union all
select 22876,'VKS' from dual union all
select 22877,'USAF' from dual union all
select 22878,'MDC' from dual union all
select 22879,'MDC' from dual union all
select 2288,'NASA' from dual union all
select 2288,'USAF' from dual union all
select 22880,'GPKS' from dual union all
select 22881,'RVSNR' from dual union all
select 22882,'RVSNR' from dual union all
select 22883,'RVSNR' from dual union all
select 22884,'RVSNR' from dual union all
select 22885,'RVSNR' from dual union all
select 22886,'NPOE' from dual union all
select 22887,'NPOE' from dual union all
select 22888,'VMFR' from dual union all
select 22889,'VKS' from dual union all
select 2289,'NASA' from dual union all
select 22890,'FKA' from dual union all
select 22891,'FKA' from dual union all
select 22892,'FKA' from dual union all
select 22893,'FKA' from dual union all
select 22894,'FKA' from dual union all
select 22895,'FKA' from dual union all
select 22896,'FKA' from dual union all
select 22897,'FKA' from dual union all
select 22898,'FKA' from dual union all
select 22899,'FKA' from dual union all
select 229,'NASA' from dual union all
select 2290,'TSKBEM' from dual union all
select 22900,'FKA' from dual union all
select 22901,'FKA' from dual union all
select 22902,'FKA' from dual union all
select 22903,'FKA' from dual union all
select 22904,'VKS' from dual union all
select 22905,'RVSNR' from dual union all
select 22906,'AE' from dual union all
select 22907,'RIMSAT' from dual union all
select 22908,'RVSNR' from dual union all
select 22909,'RVSNR' from dual union all
select 2291,'NASA' from dual union all
select 22910,'RVSNR' from dual union all
select 22911,'TELMX' from dual union all
select 22912,'EUMET' from dual union all
select 22913,'AE' from dual union all
select 22914,'AE' from dual union all
select 22915,'DOD' from dual union all
select 22915,'USAF' from dual union all
select 22916,'USAF' from dual union all
select 22917,'NASA' from dual union all
select 22918,'FKA' from dual union all
select 22919,'VMFR' from dual union all
select 2292,'RVSN' from dual union all
select 22920,'NASA' from dual union all
select 22921,'NATO' from dual union all
select 22922,'MDC' from dual union all
select 22923,'MDC' from dual union all
select 22924,'GD' from dual union all
select 22925,'RVSNR' from dual union all
select 22926,'RVSNR' from dual union all
select 22927,'ATT' from dual;
insert into SATELLITE_ORG
select 22928,'GD' from dual union all
select 22929,'RKA' from dual union all
select 2293,'RVSN' from dual union all
select 22930,'HEC' from dual union all
select 22931,'SHIN' from dual union all
select 22932,'AE' from dual union all
select 22933,'AE' from dual union all
select 22934,'FKA' from dual union all
select 22935,'FKA' from dual union all
select 22936,'FKA' from dual union all
select 22937,'FKA' from dual union all
select 22938,'FKA' from dual union all
select 22939,'FKA' from dual union all
select 2294,'RVSN' from dual union all
select 22940,'FKA' from dual union all
select 22941,'FKA' from dual union all
select 22942,'FKA' from dual union all
select 22943,'FKA' from dual union all
select 22944,'FKA' from dual union all
select 22945,'FKA' from dual union all
select 22946,'FKA' from dual union all
select 22947,'FKA' from dual union all
select 22948,'FKA' from dual union all
select 22949,'VKS' from dual union all
select 2295,'PVO' from dual union all
select 2295,'YUZH' from dual union all
select 22950,'RVSNR' from dual union all
select 22951,'RVSNR' from dual union all
select 22952,'RVSNR' from dual union all
select 22953,'VKS' from dual union all
select 22954,'NPOE' from dual union all
select 22955,'RVSNR' from dual union all
select 22956,'RVSNR' from dual union all
select 22957,'FKA' from dual union all
select 22958,'RVSNR' from dual union all
select 22959,'FKA' from dual union all
select 2296,'RVSN' from dual union all
select 22960,'FKA' from dual union all
select 22961,'FKA' from dual union all
select 22962,'FKA' from dual union all
select 22963,'INFK' from dual union all
select 22963,'LAND' from dual union all
select 22964,'RVSNR' from dual union all
select 22965,'RVSNR' from dual union all
select 22966,'RVSNR' from dual union all
select 22967,'RVSNR' from dual union all
select 22968,'RVSNR' from dual union all
select 22969,'RGMS' from dual union all
select 2297,'NASA' from dual union all
select 22970,'VKS' from dual union all
select 22971,'TUB1' from dual union all
select 22972,'NASA' from dual union all
select 22973,'BMDO' from dual union all
select 22974,'USAF' from dual union all
select 22975,'FKA' from dual union all
select 22976,'RVSNR' from dual union all
select 22977,'NASA' from dual union all
select 22978,'NAL' from dual union all
select 22978,'NASDA' from dual union all
select 22979,'NASDA' from dual union all
select 2298,'NASA' from dual union all
select 22980,'NASDA' from dual union all
select 22981,'VKS' from dual union all
select 22982,'RVSNR' from dual union all
select 22983,'RVSNR' from dual union all
select 22984,'RVSNR' from dual union all
select 22985,'RVSNR' from dual union all
select 22986,'RVSNR' from dual union all
select 22987,'BMDO' from dual union all
select 22988,'USAF' from dual union all
select 22989,'USAF' from dual union all
select 2299,'NASA' from dual union all
select 22990,'NASA' from dual union all
select 22991,'NASA' from dual union all
select 22992,'NASA' from dual union all
select 22993,'NASA' from dual union all
select 22994,'NASA' from dual union all
select 22995,'NASA' from dual union all
select 22996,'CASC' from dual union all
select 22997,'CNSA' from dual union all
select 22998,'BREM' from dual union all
select 22998,'DARA' from dual union all
select 22999,'VKS' from dual union all
select 23,'USA' from dual union all
select 230,'USAF' from dual union all
select 2300,'NASA' from dual union all
select 23000,'VKS' from dual union all
select 23001,'VKS' from dual union all
select 23002,'VKS' from dual union all
select 23003,'VKS' from dual union all
select 23004,'VKS' from dual union all
select 23005,'VKS' from dual union all
select 23006,'VKS' from dual union all
select 23007,'VKS' from dual union all
select 23008,'RVSNR' from dual union all
select 23009,'CASC' from dual union all
select 2301,'NASA' from dual union all
select 23010,'VKS' from dual union all
select 23011,'RVSNR' from dual union all
select 23012,'RVSNR' from dual;
insert into SATELLITE_ORG
select 23013,'RVSNR' from dual union all
select 23014,'RVSNR' from dual union all
select 23015,'RVSNR' from dual union all
select 23016,'HEC' from dual union all
select 23017,'MDC' from dual union all
select 23018,'MDC' from dual union all
select 23019,'IKI' from dual union all
select 23019,'YUZH' from dual union all
select 2302,'NASA' from dual union all
select 23020,'VKS' from dual union all
select 23021,'YUZH' from dual union all
select 23022,'YUZH' from dual union all
select 23023,'YUZH' from dual union all
select 23024,'YUZH' from dual union all
select 23025,'NASA' from dual union all
select 23026,'YUZH' from dual union all
select 23027,'USAF' from dual union all
select 23028,'MDC' from dual union all
select 23029,'MDC' from dual union all
select 2303,'NASA' from dual union all
select 23030,'USAF' from dual union all
select 23031,'DOD' from dual union all
select 23032,'OSC' from dual union all
select 23033,'VKS' from dual union all
select 23034,'RVSNR' from dual union all
select 23035,'FKA' from dual union all
select 23036,'RVSNR' from dual union all
select 23037,'USAF' from dual union all
select 23038,'USAF' from dual union all
select 23039,'USAF' from dual union all
select 2304,'NASA' from dual union all
select 23040,'USAF' from dual union all
select 23041,'USAF' from dual union all
select 23042,'NASA' from dual union all
select 23043,'VKS' from dual union all
select 23044,'VKS' from dual union all
select 23045,'VKS' from dual union all
select 23046,'RVSNR' from dual union all
select 23047,'RVSNR' from dual union all
select 23048,'RVSNR' from dual union all
select 23049,'RVSNR' from dual union all
select 2305,'NASA' from dual union all
select 23050,'RVSNR' from dual union all
select 23051,'NOAA' from dual union all
select 23052,'GD' from dual union all
select 23053,'NASA' from dual union all
select 23054,'NASA' from dual union all
select 23055,'NASA' from dual union all
select 23056,'NASA' from dual union all
select 23057,'NASA' from dual union all
select 23058,'NASA' from dual union all
select 23059,'NASA' from dual union all
select 2306,'NASA' from dual union all
select 23060,'NASA' from dual union all
select 23061,'NASA' from dual union all
select 23062,'NASA' from dual union all
select 23063,'NASA' from dual union all
select 23064,'NASA' from dual union all
select 23065,'NASA' from dual union all
select 23066,'NASA' from dual union all
select 23067,'NASA' from dual union all
select 23068,'NASA' from dual union all
select 23069,'NASA' from dual union all
select 2307,'NASA' from dual union all
select 23070,'NASA' from dual union all
select 23071,'NASA' from dual union all
select 23072,'NASA' from dual union all
select 23073,'NASA' from dual union all
select 23074,'NASA' from dual union all
select 23075,'NASA' from dual union all
select 23076,'NASA' from dual union all
select 23077,'NASA' from dual union all
select 23078,'NASA' from dual union all
select 23079,'NASA' from dual union all
select 2308,'NASA' from dual union all
select 23080,'NASA' from dual union all
select 23081,'NASA' from dual union all
select 23082,'NASA' from dual union all
select 23083,'NASA' from dual union all
select 23084,'NASA' from dual union all
select 23085,'NASA' from dual union all
select 23086,'NASA' from dual union all
select 23087,'MORF' from dual union all
select 23088,'VKS' from dual union all
select 23089,'VKS' from dual union all
select 2309,'NASA' from dual union all
select 23090,'VKS' from dual union all
select 23091,'VKS' from dual union all
select 23092,'VMFR' from dual union all
select 23093,'VKS' from dual union all
select 23094,'VKS' from dual union all
select 23095,'VKS' from dual union all
select 23096,'RVSNR' from dual union all
select 23097,'NROC' from dual union all
select 23097,'USAF' from dual union all
select 23098,'USAF' from dual union all
select 23099,'ISRO' from dual union all
select 231,'USAF' from dual union all
select 2310,'NASA' from dual union all
select 23100,'ISRO' from dual;
insert into SATELLITE_ORG
select 23101,'BMDO' from dual union all
select 23101,'USAF' from dual union all
select 23102,'NASA' from dual union all
select 23103,'USAF' from dual union all
select 23104,'VKS' from dual union all
select 23105,'USAF' from dual union all
select 23106,'OSC' from dual union all
select 23107,'AE' from dual union all
select 23108,'RIMSAT' from dual union all
select 23109,'RVSNR' from dual union all
select 2311,'NASA' from dual union all
select 23110,'RVSNR' from dual union all
select 23111,'RVSNR' from dual union all
select 23112,'VKS' from dual union all
select 23113,'VKS' from dual union all
select 23114,'FKA' from dual union all
select 23115,'RVSNR' from dual union all
select 23116,'RVSNR' from dual union all
select 23117,'RVSNR' from dual union all
select 23118,'ESA' from dual union all
select 23119,'MVD' from dual union all
select 23119,'VKS' from dual union all
select 2312,'NASA' from dual union all
select 23120,'RVSNR' from dual union all
select 23121,'TSSKB' from dual union all
select 23122,'TSSKB' from dual union all
select 23123,'RVSNR' from dual union all
select 23124,'INTEL' from dual union all
select 23125,'DRA' from dual union all
select 23126,'DRA' from dual union all
select 23127,'AE' from dual union all
select 23128,'USAF' from dual union all
select 23129,'USAF' from dual union all
select 2313,'NASA' from dual union all
select 23130,'USAF' from dual union all
select 23131,'USAF' from dual union all
select 23132,'HEC' from dual union all
select 23133,'MM' from dual union all
select 23134,'TSSKB' from dual union all
select 23135,'TSSKB' from dual union all
select 23136,'TSSKB' from dual union all
select 23137,'TSSKB' from dual union all
select 23138,'TSSKB' from dual union all
select 23139,'FKA' from dual union all
select 2314,'NASA' from dual union all
select 23140,'RVSNR' from dual union all
select 23141,'TSSKB' from dual union all
select 23142,'TSSKB' from dual union all
select 23143,'TSSKB' from dual union all
select 23144,'TSSKB' from dual union all
select 23145,'CASC' from dual union all
select 23146,'CNSA' from dual union all
select 23147,'CNSA' from dual union all
select 23148,'FKA' from dual union all
select 23149,'FKA' from dual union all
select 2315,'NASA' from dual union all
select 23150,'FKA' from dual union all
select 23151,'FKA' from dual union all
select 23152,'FKA' from dual union all
select 23153,'FKA' from dual union all
select 23154,'FKA' from dual union all
select 23155,'FKA' from dual union all
select 23156,'FKA' from dual union all
select 23157,'FKA' from dual union all
select 23158,'USAF' from dual union all
select 23159,'USAF' from dual union all
select 2316,'NASA' from dual union all
select 23160,'USAF' from dual union all
select 23161,'USAF' from dual union all
select 23162,'USAF' from dual union all
select 23163,'USAF' from dual union all
select 23164,'USAF' from dual union all
select 23165,'USAF' from dual union all
select 23166,'USAF' from dual union all
select 23167,'USAF' from dual union all
select 23168,'PVO' from dual union all
select 23169,'RVSNR' from dual union all
select 2317,'NASA' from dual union all
select 23170,'RVSNR' from dual union all
select 23171,'RVSNR' from dual union all
select 23172,'RVSNR' from dual union all
select 23173,'NASA' from dual union all
select 23174,'RVSNR' from dual union all
select 23175,'PAN' from dual union all
select 23176,'NHK' from dual union all
select 23177,'AE' from dual union all
select 23178,'AE' from dual union all
select 23179,'VKS' from dual union all
select 23179,'VMFR' from dual union all
select 2318,'NASA' from dual union all
select 23180,'VKS' from dual union all
select 23181,'CNSA' from dual union all
select 23182,'VKS' from dual union all
select 23183,'RVSNR' from dual union all
select 23184,'VKS' from dual union all
select 23185,'APT' from dual union all
select 23186,'CNSA' from dual union all
select 23187,'VTSR' from dual union all
select 23188,'RVSNR' from dual union all
select 23189,'TSSKB' from dual;
insert into SATELLITE_ORG
select 2319,'NASA' from dual union all
select 23190,'VKS' from dual union all
select 23191,'USAF' from dual union all
select 23192,'HEC' from dual union all
select 23193,'MM' from dual union all
select 23194,'PVO' from dual union all
select 23195,'RVSNR' from dual union all
select 23196,'RVSNR' from dual union all
select 23197,'RVSNR' from dual union all
select 23198,'OSC' from dual union all
select 23199,'TELB' from dual union all
select 232,'USAF' from dual union all
select 2320,'NASA' from dual union all
select 23200,'TURKP' from dual union all
select 23201,'AE' from dual union all
select 23202,'AE' from dual union all
select 23203,'VKS' from dual union all
select 23204,'VKS' from dual union all
select 23205,'VKS' from dual union all
select 23206,'RVSNR' from dual union all
select 23207,'RVSNR' from dual union all
select 23208,'RVSNR' from dual union all
select 23209,'RVSNR' from dual union all
select 2321,'NASA' from dual union all
select 23210,'RVSNR' from dual union all
select 23211,'GPKS' from dual union all
select 23211,'VKS' from dual union all
select 23212,'RVSNR' from dual union all
select 23213,'RVSNR' from dual union all
select 23214,'RVSNR' from dual union all
select 23215,'FKA' from dual union all
select 23216,'RVSNR' from dual union all
select 23217,'RVSNR' from dual union all
select 23218,'TSSKB' from dual union all
select 23219,'VKS' from dual union all
select 2322,'NRO' from dual union all
select 23220,'VKS' from dual union all
select 23221,'VKS' from dual union all
select 23222,'VKS' from dual union all
select 23223,'NROC' from dual union all
select 23223,'USAF' from dual union all
select 23224,'VKS' from dual union all
select 23225,'VKS' from dual union all
select 23226,'VKS' from dual union all
select 23227,'OPT' from dual union all
select 23228,'CNSA' from dual union all
select 23229,'HEC' from dual union all
select 2323,'USAF' from dual union all
select 23230,'NASDA' from dual union all
select 23231,'NASDA' from dual union all
select 23232,'AE' from dual union all
select 23233,'USAF' from dual union all
select 23234,'USAF' from dual union all
select 23235,'USAF' from dual union all
select 23236,'AE' from dual union all
select 23237,'AE' from dual union all
select 23238,'AE' from dual union all
select 23239,'AE' from dual union all
select 2324,'USAF' from dual union all
select 23240,'AE' from dual union all
select 23241,'AE' from dual union all
select 23242,'AE' from dual union all
select 23243,'AE' from dual union all
select 23244,'AE' from dual union all
select 23245,'AE' from dual union all
select 23246,'AE' from dual union all
select 23247,'USAF' from dual union all
select 23248,'NASDA' from dual union all
select 23249,'ATT' from dual union all
select 2325,'MO' from dual union all
select 23250,'USAF' from dual union all
select 23251,'NASA' from dual union all
select 23252,'FKA' from dual union all
select 23253,'NASA' from dual union all
select 23254,'MAI' from dual union all
select 23255,'MAI' from dual union all
select 23256,'MAI' from dual union all
select 23257,'MAI' from dual union all
select 23258,'MAI' from dual union all
select 23259,'MAI' from dual union all
select 2326,'RVSN' from dual union all
select 23260,'MAI' from dual union all
select 23261,'MAI' from dual union all
select 23262,'MAI' from dual union all
select 23263,'RVSN' from dual union all
select 23264,'RVSN' from dual union all
select 23265,'RVSN' from dual union all
select 23266,'FKA' from dual union all
select 23267,'VKS' from dual union all
select 23268,'RVSNR' from dual union all
select 23269,'RVSN' from dual union all
select 2327,'USAF' from dual union all
select 23270,'RVSNR' from dual union all
select 23271,'RVSNR' from dual union all
select 23272,'RVSNR' from dual union all
select 23273,'USAF' from dual union all
select 23274,'USAF' from dual union all
select 23275,'RVSNR' from dual union all
select 23276,'USAF' from dual union all
select 23277,'USAF' from dual;
insert into SATELLITE_ORG
select 23278,'PVO' from dual union all
select 23279,'VKS' from dual union all
select 2328,'USAF' from dual union all
select 23280,'GD' from dual union all
select 23281,'VKS' from dual union all
select 23282,'RVSNR' from dual union all
select 23283,'VKS' from dual union all
select 23284,'VKS' from dual union all
select 23285,'NASA' from dual union all
select 23286,'VKS' from dual union all
select 23287,'RVSNR' from dual union all
select 23288,'FKA' from dual union all
select 23289,'RVSNR' from dual union all
select 2329,'USAF' from dual union all
select 23290,'AE' from dual union all
select 23291,'RVSNR' from dual union all
select 23292,'RVSNR' from dual union all
select 23293,'RVSNR' from dual union all
select 23294,'RVSNR' from dual union all
select 23295,'RVSNR' from dual union all
select 23296,'RVSNR' from dual union all
select 23297,'RVSNR' from dual union all
select 23298,'RVSNR' from dual union all
select 23299,'MDC' from dual union all
select 233,'USAF' from dual union all
select 2330,'MO' from dual union all
select 23300,'MDC' from dual union all
select 23301,'AE' from dual union all
select 23302,'RVSNR' from dual union all
select 23303,'RVSNR' from dual union all
select 23304,'RVSNR' from dual union all
select 23305,'INTEL' from dual union all
select 23306,'MM' from dual union all
select 23307,'NASA' from dual union all
select 23308,'VMF' from dual union all
select 23309,'NASA' from dual union all
select 2331,'MO' from dual union all
select 23310,'NASA' from dual union all
select 23311,'RVSNR' from dual union all
select 23312,'MDC' from dual union all
select 23313,'TELMX' from dual union all
select 23314,'SHIN' from dual union all
select 23315,'AE' from dual union all
select 23316,'AE' from dual union all
select 23317,'RGMS' from dual union all
select 23317,'YUZH' from dual union all
select 23318,'VKS' from dual union all
select 23319,'GPKS' from dual union all
select 2332,'MO' from dual union all
select 23320,'RVSNR' from dual union all
select 23321,'RVSNR' from dual union all
select 23322,'RVSNR' from dual union all
select 23323,'ISRO' from dual union all
select 23324,'ISRO' from dual union all
select 23325,'RVSNR' from dual union all
select 23326,'RVSNR' from dual union all
select 23327,'FKA' from dual union all
select 23327,'RGMS' from dual union all
select 23328,'RVSNR' from dual union all
select 23329,'RVSNR' from dual union all
select 2333,'USAF' from dual union all
select 23330,'RVSNR' from dual union all
select 23331,'SES' from dual union all
select 23332,'AE' from dual union all
select 23333,'NASA' from dual union all
select 23334,'MDC' from dual union all
select 23335,'RVSNR' from dual union all
select 23336,'VMFR' from dual union all
select 23337,'VKS' from dual union all
select 23338,'RVSNR' from dual union all
select 23339,'RVSNR' from dual union all
select 2334,'RVSN' from dual union all
select 23340,'NASA' from dual union all
select 23341,'DLR' from dual union all
select 23342,'MVD' from dual union all
select 23343,'VKS' from dual union all
select 23344,'VKS' from dual union all
select 23345,'VKS' from dual union all
select 23346,'VKS' from dual union all
select 23347,'VKS' from dual union all
select 23348,'FKA' from dual union all
select 23349,'RVSNR' from dual union all
select 2335,'USN' from dual union all
select 23350,'MDC' from dual union all
select 23351,'USAF' from dual union all
select 23352,'USAF' from dual union all
select 23353,'USAF' from dual union all
select 23354,'USAF' from dual union all
select 23355,'USAF' from dual union all
select 23356,'USAF' from dual union all
select 23357,'USAF' from dual union all
select 23358,'USAF' from dual union all
select 23359,'USAF' from dual union all
select 2336,'NASA' from dual union all
select 23360,'USAF' from dual union all
select 23361,'USAF' from dual union all
select 23362,'USAF' from dual union all
select 23363,'USAF' from dual union all
select 23364,'USAF' from dual union all
select 23365,'USAF' from dual;
insert into SATELLITE_ORG
select 23366,'USAF' from dual union all
select 23367,'USAF' from dual union all
select 23368,'USAF' from dual union all
select 23369,'USAF' from dual union all
select 2337,'USAF' from dual union all
select 23370,'USAF' from dual union all
select 23371,'USAF' from dual union all
select 23372,'USAF' from dual union all
select 23373,'USAF' from dual union all
select 23374,'USAF' from dual union all
select 23375,'USAF' from dual union all
select 23376,'USAF' from dual union all
select 23377,'USAF' from dual union all
select 23378,'USAF' from dual union all
select 23379,'USAF' from dual union all
select 2338,'RVSN' from dual union all
select 23380,'USAF' from dual union all
select 23381,'USAF' from dual union all
select 23382,'USAF' from dual union all
select 23383,'USAF' from dual union all
select 23384,'USAF' from dual union all
select 23385,'USAF' from dual union all
select 23386,'USAF' from dual union all
select 23387,'USAF' from dual union all
select 23388,'USAF' from dual union all
select 23389,'USAF' from dual union all
select 2339,'RVSN' from dual union all
select 23390,'USAF' from dual union all
select 23391,'USAF' from dual union all
select 23392,'USAF' from dual union all
select 23393,'USAF' from dual union all
select 23394,'USAF' from dual union all
select 23395,'USAF' from dual union all
select 23396,'VKS' from dual union all
select 23397,'VKS' from dual union all
select 23398,'VKS' from dual union all
select 23399,'RVSNR' from dual union all
select 234,'USAF' from dual union all
select 2340,'USAF' from dual union all
select 23400,'RVSNR' from dual union all
select 23401,'RVSNR' from dual union all
select 23402,'RVSNR' from dual union all
select 23403,'RVSNR' from dual union all
select 23404,'MORF' from dual union all
select 23405,'VKS' from dual union all
select 23406,'VKS' from dual union all
select 23407,'VKS' from dual union all
select 23408,'VKS' from dual union all
select 23409,'VKS' from dual union all
select 2341,'NASA' from dual union all
select 23410,'VKS' from dual union all
select 23411,'VTSR' from dual union all
select 23412,'VKS' from dual union all
select 23413,'ORI' from dual union all
select 23414,'MM' from dual union all
select 23415,'CHIS' from dual union all
select 23416,'CNSA' from dual union all
select 23417,'VKS' from dual union all
select 23418,'VKS' from dual union all
select 23419,'VKS' from dual union all
select 2342,'NASA' from dual union all
select 23420,'VKS' from dual union all
select 23421,'RVSNR' from dual union all
select 23422,'RVSNR' from dual union all
select 23423,'RVSNR' from dual union all
select 23424,'USAF' from dual union all
select 23425,'NROC' from dual union all
select 23426,'VKS' from dual union all
select 23427,'RVSNR' from dual union all
select 23428,'RVSNR' from dual union all
select 23429,'RVSNR' from dual union all
select 2343,'NASA' from dual union all
select 23430,'RVSNR' from dual union all
select 23431,'VKS' from dual union all
select 23432,'VKS' from dual union all
select 23433,'VKS' from dual union all
select 23434,'VKS' from dual union all
select 23435,'USAF' from dual union all
select 23436,'USAF' from dual union all
select 23437,'USAF' from dual union all
select 23438,'USAF' from dual union all
select 23439,'FKA' from dual union all
select 2344,'NASA' from dual union all
select 23440,'VKS' from dual union all
select 23441,'VKS' from dual union all
select 23442,'VKS' from dual union all
select 23443,'VKS' from dual union all
select 23444,'VKS' from dual union all
select 23445,'VKS' from dual union all
select 23446,'VKS' from dual union all
select 23447,'VKS' from dual union all
select 23448,'VKS' from dual union all
select 23449,'RVSNR' from dual union all
select 2345,'NASA' from dual union all
select 23450,'RVSNR' from dual union all
select 23451,'RVSNR' from dual union all
select 23452,'RVSNR' from dual union all
select 23453,'VKS' from dual union all
select 23454,'RVSNR' from dual union all
select 23455,'NOAA' from dual;
insert into SATELLITE_ORG
select 23456,'VKS' from dual union all
select 23457,'NOAA' from dual union all
select 23458,'NOAA' from dual union all
select 23459,'RVSNR' from dual union all
select 2346,'NASA' from dual union all
select 23460,'RVSNR' from dual union all
select 23461,'INTEL' from dual union all
select 23462,'MM' from dual union all
select 23463,'VMFR' from dual union all
select 23464,'SSC' from dual union all
select 23465,'FAI' from dual union all
select 23466,'VKS' from dual union all
select 23467,'HEC' from dual union all
select 23468,'MM' from dual union all
select 23469,'NASA' from dual union all
select 2347,'NASA' from dual union all
select 23470,'USN' from dual union all
select 23471,'NASA' from dual union all
select 23472,'NASA' from dual union all
select 23473,'NASA' from dual union all
select 23474,'NASA' from dual union all
select 23475,'NASA' from dual union all
select 23476,'FKA' from dual union all
select 23477,'FKA' from dual union all
select 23478,'RVSNR' from dual union all
select 23479,'RVSNR' from dual union all
select 2348,'NASA' from dual union all
select 23480,'RVSNR' from dual union all
select 23481,'RVSN' from dual union all
select 23482,'RVSNR' from dual union all
select 23483,'RVSNR' from dual union all
select 23484,'RVSNR' from dual union all
select 23485,'RVSNR' from dual union all
select 23486,'RVSNR' from dual union all
select 23487,'RVSNR' from dual union all
select 23488,'RVSNR' from dual union all
select 23489,'RVSNR' from dual union all
select 2349,'NASA' from dual union all
select 23490,'RVSNR' from dual union all
select 23491,'RVSNR' from dual union all
select 23492,'RVSNR' from dual union all
select 23493,'RVSNR' from dual union all
select 23494,'RVSNR' from dual union all
select 23495,'RVSNR' from dual union all
select 23496,'RVSNR' from dual union all
select 23497,'TSSKB' from dual union all
select 23498,'RVSNR' from dual union all
select 23499,'TSSKB' from dual union all
select 235,'USAF' from dual union all
select 2350,'USAF' from dual union all
select 23500,'NASA' from dual union all
select 23501,'PVO' from dual union all
select 23502,'VKS' from dual union all
select 23503,'TSSKB' from dual union all
select 23504,'TSSKB' from dual union all
select 23505,'TSSKB' from dual union all
select 23506,'TSSKB' from dual union all
select 23507,'PVO' from dual union all
select 23508,'PVO' from dual union all
select 23509,'PVO' from dual union all
select 2351,'KB1' from dual union all
select 2351,'VMF' from dual union all
select 23510,'PVO' from dual union all
select 23511,'VKS' from dual union all
select 23512,'VKS' from dual union all
select 23513,'VKS' from dual union all
select 23514,'RVSNR' from dual union all
select 23515,'RVSNR' from dual union all
select 23516,'RVSNR' from dual union all
select 23517,'RVSNR' from dual union all
select 23518,'RVSNR' from dual union all
select 23519,'FKA' from dual union all
select 2352,'NASA' from dual union all
select 23520,'RVSNR' from dual union all
select 23521,'NASDA' from dual union all
select 23522,'NASDA' from dual union all
select 23523,'NASDA' from dual union all
select 23524,'NASDA' from dual union all
select 23525,'NASDA' from dual union all
select 23526,'VMFR' from dual union all
select 23527,'VKS' from dual union all
select 23528,'INTEL' from dual union all
select 23529,'LM' from dual union all
select 2353,'USN' from dual union all
select 23530,'VKS' from dual union all
select 23531,'RVSNR' from dual union all
select 23532,'USAF' from dual union all
select 23533,'USAF' from dual union all
select 23534,'USAF' from dual union all
select 23535,'USAF' from dual union all
select 23536,'TELB' from dual union all
select 23537,'EUTEL' from dual union all
select 23538,'AE' from dual union all
select 23539,'AE' from dual union all
select 2354,'NASA' from dual union all
select 23540,'FKA' from dual union all
select 23541,'VKS' from dual union all
select 23542,'VKS' from dual union all
select 23543,'VKS' from dual union all
select 23544,'VKS' from dual;
insert into SATELLITE_ORG
select 23545,'ORBC' from dual union all
select 23546,'ORBC' from dual union all
select 23547,'ORBI' from dual union all
select 23548,'OSC' from dual union all
select 23549,'ISA' from dual union all
select 2355,'NASA' from dual union all
select 23550,'ISA' from dual union all
select 23551,'FKA' from dual union all
select 23552,'USAF' from dual union all
select 23553,'AMSC' from dual union all
select 23554,'LM' from dual union all
select 23555,'FKA' from dual union all
select 23556,'RVSNR' from dual union all
select 23557,'FKA' from dual union all
select 23558,'GFZ' from dual union all
select 23559,'FKA' from dual union all
select 2356,'NASA' from dual union all
select 23560,'ESA' from dual union all
select 23561,'AE' from dual union all
select 23562,'USN' from dual union all
select 23563,'BOE' from dual union all
select 23564,'RVSNR' from dual union all
select 23565,'FKA' from dual union all
select 23566,'FKA' from dual union all
select 23567,'CIA' from dual union all
select 23567,'NROC' from dual union all
select 23568,'USAF' from dual union all
select 23569,'NASDA' from dual union all
select 2357,'NASA' from dual union all
select 23570,'NASDA' from dual union all
select 23571,'INTEL' from dual union all
select 23572,'AE' from dual union all
select 23573,'FKA' from dual union all
select 23574,'USAF' from dual union all
select 23575,'RVSNR' from dual union all
select 23576,'USAF' from dual union all
select 23577,'USAF' from dual union all
select 23578,'RVSNR' from dual union all
select 23579,'FKA' from dual union all
select 2358,'NASA' from dual union all
select 23580,'RVSNR' from dual union all
select 23581,'NOAA' from dual union all
select 23582,'LM' from dual union all
select 23583,'FKA' from dual union all
select 23584,'PVO' from dual union all
select 23585,'RVSNR' from dual union all
select 23586,'RVSNR' from dual union all
select 23587,'RVSNR' from dual union all
select 23588,'FKA' from dual union all
select 23589,'USN' from dual union all
select 2359,'USAF' from dual union all
select 23590,'LM' from dual union all
select 23591,'VKS' from dual union all
select 23592,'VKS' from dual union all
select 23593,'NASDA' from dual union all
select 23594,'USAF' from dual union all
select 23595,'USAF' from dual union all
select 23596,'VMFR' from dual union all
select 23597,'VKS' from dual union all
select 23598,'HEC' from dual union all
select 23599,'AE' from dual union all
select 236,'USAF' from dual union all
select 2360,'USAF' from dual union all
select 23600,'NASA' from dual union all
select 23601,'VKS' from dual union all
select 23602,'RVSNR' from dual union all
select 23603,'VKS' from dual union all
select 23603,'VMFR' from dual union all
select 23604,'VKS' from dual union all
select 23605,'CNES' from dual union all
select 23605,'DGA' from dual union all
select 23606,'CNES' from dual union all
select 23606,'DGA' from dual union all
select 23607,'UPM' from dual union all
select 23608,'AE' from dual union all
select 23609,'NROC' from dual union all
select 23609,'USAF' from dual union all
select 2361,'USAF' from dual union all
select 23610,'USAF' from dual union all
select 23611,'USAF' from dual union all
select 23612,'NASA' from dual union all
select 23613,'NASA' from dual union all
select 23614,'NASA' from dual union all
select 23615,'NASA' from dual union all
select 23616,'RVSNR' from dual union all
select 23617,'FKA' from dual union all
select 23618,'RVSNR' from dual union all
select 23619,'USAF' from dual union all
select 2362,'USAF' from dual union all
select 23620,'FKA' from dual union all
select 23620,'VKS' from dual union all
select 23621,'FKA' from dual union all
select 23621,'VKS' from dual union all
select 23622,'FKA' from dual union all
select 23622,'VKS' from dual union all
select 23623,'RVSNR' from dual union all
select 23624,'RVSNR' from dual union all
select 23625,'RVSNR' from dual union all
select 23626,'RVSNR' from dual union all
select 23627,'RVSNR' from dual;
insert into SATELLITE_ORG
select 23628,'DOD' from dual union all
select 23628,'USAF' from dual union all
select 23629,'LM' from dual union all
select 2363,'USAF' from dual union all
select 23630,'RVSNR' from dual union all
select 23631,'RVSNR' from dual union all
select 23632,'IKI' from dual union all
select 23632,'NPOL' from dual union all
select 23633,'RVSNR' from dual union all
select 23634,'RVSNR' from dual union all
select 23635,'RVSNR' from dual union all
select 23636,'PAN' from dual union all
select 23637,'AE' from dual union all
select 23638,'RVSNR' from dual union all
select 23639,'KTEL' from dual union all
select 2364,'USAF' from dual union all
select 23640,'MDC' from dual union all
select 23641,'MDC' from dual union all
select 23642,'GPKS' from dual union all
select 23642,'VKS' from dual union all
select 23643,'RVSNR' from dual union all
select 23644,'RVSNR' from dual union all
select 23645,'RVSNR' from dual union all
select 23646,'CZ' from dual union all
select 23647,'USAF' from dual union all
select 23648,'USAF' from dual union all
select 23649,'JSAT' from dual union all
select 2365,'USAF' from dual union all
select 23650,'ORBC' from dual union all
select 23651,'NTT' from dual union all
select 23652,'AE' from dual union all
select 23653,'VKS' from dual union all
select 23654,'RVSNR' from dual union all
select 23655,'RVSNR' from dual union all
select 23656,'RVSNR' from dual union all
select 23657,'NKAU' from dual union all
select 23658,'LM' from dual union all
select 23659,'VKS' from dual union all
select 2366,'USAF' from dual union all
select 23660,'RVSNR' from dual union all
select 23661,'RVSNR' from dual union all
select 23662,'FKA' from dual union all
select 23663,'RVSNR' from dual union all
select 23664,'RVSNR' from dual union all
select 23665,'FKA' from dual union all
select 23666,'RVSNR' from dual union all
select 23667,'NASA' from dual union all
select 23668,'NASA' from dual union all
select 23669,'SII' from dual union all
select 2367,'USAF' from dual union all
select 23670,'ATT' from dual union all
select 23671,'AE' from dual union all
select 23672,'MVD' from dual union all
select 23673,'RVSNR' from dual union all
select 23674,'VKS' from dual union all
select 23675,'RVSNR' from dual union all
select 23676,'VMFR' from dual union all
select 23677,'VKS' from dual union all
select 23678,'FKA' from dual union all
select 23679,'RVSNR' from dual union all
select 2368,'MO' from dual union all
select 23680,'VKS' from dual union all
select 23681,'RVSNR' from dual union all
select 23682,'RVSNR' from dual union all
select 23683,'RVSNR' from dual union all
select 23684,'RVSNR' from dual union all
select 23685,'RVSNR' from dual union all
select 23686,'SES' from dual union all
select 23687,'AE' from dual union all
select 23688,'NASA' from dual union all
select 23689,'FKA' from dual union all
select 2369,'RVSN' from dual union all
select 23690,'FKA' from dual union all
select 23691,'FKA' from dual union all
select 23692,'FKA' from dual union all
select 23693,'FKA' from dual union all
select 23694,'FKA' from dual union all
select 23695,'FKA' from dual union all
select 23696,'USN' from dual union all
select 23697,'LM' from dual union all
select 23698,'FKA' from dual union all
select 23699,'TSSKB' from dual union all
select 237,'USAF' from dual union all
select 2370,'MO' from dual union all
select 23700,'TSSKB' from dual union all
select 23701,'TSSKB' from dual union all
select 23702,'TSSKB' from dual union all
select 23703,'FKA' from dual union all
select 23704,'MORF' from dual union all
select 23705,'VKS' from dual union all
select 23706,'VKS' from dual union all
select 23707,'VKS' from dual union all
select 23708,'VKS' from dual union all
select 23709,'VKS' from dual union all
select 2371,'USAF' from dual union all
select 23710,'CSA' from dual union all
select 23711,'MDC' from dual union all
select 23712,'USAF' from dual union all
select 23713,'USAF' from dual union all
select 23714,'NASA' from dual;
insert into SATELLITE_ORG
select 23715,'ESA' from dual union all
select 23716,'AE' from dual union all
select 23717,'INFK' from dual union all
select 23717,'LAND' from dual union all
select 23718,'RVSNR' from dual union all
select 23719,'RVSNR' from dual union all
select 2372,'USAF' from dual union all
select 23720,'RVSNR' from dual union all
select 23721,'RVSNR' from dual union all
select 23722,'RVSNR' from dual union all
select 23723,'ASIA' from dual union all
select 23724,'CNSA' from dual union all
select 23725,'CASC' from dual union all
select 23726,'ESA' from dual union all
select 23727,'LM' from dual union all
select 23728,'CIA' from dual union all
select 23728,'NROC' from dual union all
select 23729,'USAF' from dual union all
select 2373,'USAF' from dual union all
select 23730,'FTEL' from dual union all
select 23731,'ISRO' from dual union all
select 23732,'AE' from dual union all
select 23733,'AE' from dual union all
select 23734,'FKA' from dual union all
select 23734,'VKS' from dual union all
select 23735,'FKA' from dual union all
select 23735,'VKS' from dual union all
select 23736,'FKA' from dual union all
select 23736,'VKS' from dual union all
select 23737,'RVSNR' from dual union all
select 23738,'RVSNR' from dual union all
select 23739,'RVSNR' from dual union all
select 2374,'USAF' from dual union all
select 23740,'RVSNR' from dual union all
select 23741,'HEC' from dual union all
select 23742,'LM' from dual union all
select 23743,'RVSNR' from dual union all
select 23744,'FKA' from dual union all
select 23745,'RVSNR' from dual union all
select 23746,'VKS' from dual union all
select 23747,'VKS' from dual union all
select 23748,'VMFR' from dual union all
select 23749,'MDC' from dual union all
select 2375,'USAF' from dual union all
select 23750,'VKS' from dual union all
select 23751,'ISRO' from dual union all
select 23752,'BMDO' from dual union all
select 23753,'RVSNR' from dual union all
select 23754,'ECHO' from dual union all
select 23755,'CNSA' from dual union all
select 23756,'CASC' from dual union all
select 23757,'NASA' from dual union all
select 23758,'MDC' from dual union all
select 23759,'NASA' from dual union all
select 2376,'NRO' from dual union all
select 23760,'NASA' from dual union all
select 23761,'NASA' from dual union all
select 23762,'NASA' from dual union all
select 23763,'NASA' from dual union all
select 23764,'PAN' from dual union all
select 23765,'BINA' from dual union all
select 23766,'AE' from dual union all
select 23767,'AE' from dual union all
select 23768,'KTEL' from dual union all
select 23769,'MDC' from dual union all
select 2377,'USAF' from dual union all
select 23770,'MDC' from dual union all
select 23771,'NASDA' from dual union all
select 23772,'NASDA' from dual union all
select 23773,'VMFR' from dual union all
select 23774,'VKS' from dual union all
select 23775,'GPKS' from dual union all
select 23776,'RVSNR' from dual union all
select 23777,'RVSNR' from dual union all
select 23778,'RVSNR' from dual union all
select 23779,'PSN' from dual union all
select 23779,'SINDO' from dual union all
select 2378,'USAF' from dual union all
select 23780,'LM' from dual union all
select 23781,'NTT' from dual union all
select 23782,'AE' from dual union all
select 23783,'FKA' from dual union all
select 23784,'NASA' from dual union all
select 23785,'MDC' from dual union all
select 23786,'MDC' from dual union all
select 23787,'FKA' from dual union all
select 23788,'FKA' from dual union all
select 23789,'FKA' from dual union all
select 2379,'USAF' from dual union all
select 23790,'VKS' from dual union all
select 23791,'VKS' from dual union all
select 23792,'VKS' from dual union all
select 23793,'VKS' from dual union all
select 23794,'VKS' from dual union all
select 23795,'RVSNR' from dual union all
select 23796,'RVSNR' from dual union all
select 23797,'RVSNR' from dual union all
select 23798,'FKA' from dual union all
select 23799,'RVSNR' from dual union all
select 238,'USAF' from dual;
insert into SATELLITE_ORG
select 2380,'USAF' from dual union all
select 23800,'RVSNR' from dual union all
select 23801,'NASA' from dual union all
select 23802,'NASA' from dual union all
select 23803,'MDC' from dual union all
select 23804,'MDC' from dual union all
select 23805,'ASI' from dual union all
select 23806,'USAF' from dual union all
select 23807,'USAF' from dual union all
select 23808,'USAF' from dual union all
select 23809,'USAF' from dual union all
select 2381,'USAF' from dual union all
select 23810,'USAF' from dual union all
select 23811,'USAF' from dual union all
select 23812,'USAF' from dual union all
select 23813,'RVSNR' from dual union all
select 23814,'USAF' from dual union all
select 23815,'OSC' from dual union all
select 23816,'INTEL' from dual union all
select 23817,'AE' from dual union all
select 23818,'VKS' from dual union all
select 23819,'RVSNR' from dual union all
select 2382,'NASA' from dual union all
select 23820,'USAF' from dual union all
select 23821,'USAF' from dual union all
select 23822,'FKA' from dual union all
select 23823,'FKA' from dual union all
select 23824,'RVSNR' from dual union all
select 23825,'FKA' from dual union all
select 23826,'FKA' from dual union all
select 23827,'ISRO' from dual union all
select 23828,'ISRO' from dual union all
select 23829,'USAF' from dual union all
select 2383,'NASA' from dual union all
select 23830,'USAF' from dual union all
select 23831,'NASA' from dual union all
select 23832,'USAF' from dual union all
select 23833,'USAF' from dual union all
select 23834,'MDC' from dual union all
select 23835,'MDC' from dual union all
select 23836,'VMF' from dual union all
select 23837,'USAF' from dual union all
select 23838,'USAF' from dual union all
select 23839,'INMAR' from dual union all
select 2384,'USAF' from dual union all
select 23840,'LM' from dual union all
select 23841,'USAF' from dual union all
select 23842,'SES' from dual union all
select 23843,'VKS' from dual union all
select 23844,'RVSNR' from dual union all
select 23845,'RVSNR' from dual union all
select 23846,'TMI' from dual union all
select 23847,'AE' from dual union all
select 23848,'FKA' from dual union all
select 23849,'RVSNR' from dual union all
select 2385,'USAF' from dual union all
select 23850,'FKA' from dual union all
select 23851,'BMDO' from dual union all
select 23852,'MDC' from dual union all
select 23853,'PVO' from dual union all
select 23854,'VKS' from dual union all
select 23855,'NROC' from dual union all
select 23855,'USAF' from dual union all
select 23856,'FKA' from dual union all
select 23857,'ASI' from dual union all
select 23858,'LM' from dual union all
select 23859,'BMDO' from dual union all
select 2386,'USN' from dual union all
select 23860,'FKA' from dual union all
select 23861,'RVSNR' from dual union all
select 23862,'NROC' from dual union all
select 23862,'USN' from dual union all
select 23863,'USAF' from dual union all
select 23864,'PSN' from dual union all
select 23864,'SINDO' from dual union all
select 23865,'IAI' from dual union all
select 23866,'AE' from dual union all
select 23867,'AE' from dual union all
select 23868,'BMDO' from dual union all
select 23869,'OSC' from dual union all
select 2387,'RVSN' from dual union all
select 23870,'NASA' from dual union all
select 23871,'JPL' from dual union all
select 23872,'JPL' from dual union all
select 23873,'FKA' from dual union all
select 23874,'FKA' from dual union all
select 23875,'FKA' from dual union all
select 23876,'NASA' from dual union all
select 23877,'HEC' from dual union all
select 23878,'MDC' from dual union all
select 23879,'MDC' from dual union all
select 2388,'NASA' from dual union all
select 23880,'GPKS' from dual union all
select 23881,'RVSNR' from dual union all
select 23882,'VKS' from dual union all
select 23883,'RVSNR' from dual union all
select 23884,'FKA' from dual union all
select 23885,'FKA' from dual union all
select 23886,'VKS' from dual union all
select 23887,'VKS' from dual;
insert into SATELLITE_ORG
select 23888,'FKA' from dual union all
select 23889,'FKA' from dual union all
select 2389,'USAF' from dual union all
select 23890,'FKA' from dual union all
select 23891,'FKA' from dual union all
select 23892,'FKA' from dual union all
select 23893,'NROC' from dual union all
select 23893,'USN' from dual union all
select 23894,'FKA' from dual union all
select 23895,'FKA' from dual union all
select 23896,'FKA' from dual union all
select 23897,'FKA' from dual union all
select 23898,'FKA' from dual union all
select 23899,'FKA' from dual union all
select 239,'USAF' from dual union all
select 2390,'USAF' from dual union all
select 23900,'FKA' from dual union all
select 23901,'FKA' from dual union all
select 23902,'FKA' from dual union all
select 23903,'FKA' from dual union all
select 23904,'FKA' from dual union all
select 23905,'JPL' from dual union all
select 23906,'USAF' from dual union all
select 23907,'NROC' from dual union all
select 23907,'USN' from dual union all
select 23908,'NROC' from dual union all
select 23908,'USN' from dual union all
select 23909,'VKS' from dual union all
select 2391,'MO' from dual union all
select 23910,'FKA' from dual union all
select 23911,'FKA' from dual union all
select 23912,'FKA' from dual union all
select 23913,'FKA' from dual union all
select 23914,'FKA' from dual union all
select 23915,'INTEL' from dual union all
select 23916,'AE' from dual union all
select 23917,'FKA' from dual union all
select 23918,'FKA' from dual union all
select 23919,'FKA' from dual union all
select 2392,'RVSN' from dual union all
select 23920,'FKA' from dual union all
select 23921,'FKA' from dual union all
select 23922,'FKA' from dual union all
select 23923,'FKA' from dual union all
select 23924,'FKA' from dual union all
select 23925,'FKA' from dual union all
select 23926,'FKA' from dual union all
select 23927,'FKA' from dual union all
select 23928,'FKA' from dual union all
select 23929,'FKA' from dual union all
select 2393,'CIA' from dual union all
select 2393,'NRO' from dual union all
select 23930,'JPL' from dual union all
select 23931,'NASA' from dual union all
select 23932,'FKA' from dual union all
select 23933,'USAF' from dual union all
select 23934,'FKA' from dual union all
select 23935,'NASA' from dual union all
select 23936,'NROC' from dual union all
select 23936,'USN' from dual union all
select 23937,'NROC' from dual union all
select 23937,'USN' from dual union all
select 23938,'NROC' from dual union all
select 23939,'NROC' from dual union all
select 2394,'NASA' from dual union all
select 23940,'NASA' from dual union all
select 23941,'OSC' from dual union all
select 23942,'OSC' from dual union all
select 23943,'APT' from dual union all
select 23944,'CNSA' from dual union all
select 23945,'NROC' from dual union all
select 23945,'USAF' from dual union all
select 23946,'USAF' from dual union all
select 23947,'USAF' from dual union all
select 23948,'ARAB' from dual union all
select 23949,'TURKT' from dual union all
select 2395,'NASA' from dual union all
select 23950,'AE' from dual union all
select 23951,'AE' from dual union all
select 23952,'USAF' from dual union all
select 23953,'USAF' from dual union all
select 23954,'MDC' from dual union all
select 23955,'MDC' from dual union all
select 23956,'OSC' from dual union all
select 23957,'OSC' from dual union all
select 23958,'OSC' from dual union all
select 23959,'OSC' from dual union all
select 2396,'NRO' from dual union all
select 23960,'OSC' from dual union all
select 23961,'OSC' from dual union all
select 23962,'OSC' from dual union all
select 23963,'OSC' from dual union all
select 23964,'OSC' from dual union all
select 23965,'OSC' from dual union all
select 23966,'AE' from dual union all
select 23967,'HEC' from dual union all
select 23968,'LM' from dual union all
select 23969,'OSC' from dual union all
select 2397,'NRO' from dual union all
select 23970,'OSC' from dual;
insert into SATELLITE_ORG
select 23971,'OSC' from dual union all
select 23972,'OSC' from dual union all
select 23973,'OSC' from dual union all
select 23974,'OSC' from dual union all
select 23975,'OSC' from dual union all
select 23976,'OSC' from dual union all
select 23977,'OSC' from dual union all
select 23978,'OSC' from dual union all
select 23979,'OSC' from dual union all
select 2398,'NASA' from dual union all
select 23980,'OSC' from dual union all
select 23981,'OSC' from dual union all
select 23982,'OSC' from dual union all
select 23983,'OSC' from dual union all
select 23984,'OSC' from dual union all
select 23985,'OSC' from dual union all
select 23986,'OSC' from dual union all
select 23987,'OSC' from dual union all
select 23988,'OSC' from dual union all
select 23989,'OSC' from dual union all
select 2399,'NASA' from dual union all
select 23990,'OSC' from dual union all
select 23991,'OSC' from dual union all
select 23992,'OSC' from dual union all
select 23993,'OSC' from dual union all
select 23994,'CNES' from dual union all
select 23995,'OSC' from dual union all
select 23996,'OSC' from dual union all
select 23997,'OSC' from dual union all
select 23998,'OSC' from dual union all
select 23999,'OSC' from dual union all
select 24,'CIA' from dual union all
select 24,'DOD' from dual union all
select 240,'NASA' from dual union all
select 2400,'USAF' from dual union all
select 24000,'OSC' from dual union all
select 24001,'OSC' from dual union all
select 24002,'OSC' from dual union all
select 24003,'OSC' from dual union all
select 24004,'OSC' from dual union all
select 24005,'OSC' from dual union all
select 24006,'OSC' from dual union all
select 24007,'OSC' from dual union all
select 24008,'OSC' from dual union all
select 24009,'OSC' from dual union all
select 2401,'USN' from dual union all
select 24010,'OSC' from dual union all
select 24011,'OSC' from dual union all
select 24012,'OSC' from dual union all
select 24013,'OSC' from dual union all
select 24014,'OSC' from dual union all
select 24015,'OSC' from dual union all
select 24016,'OSC' from dual union all
select 24017,'OSC' from dual union all
select 24018,'OSC' from dual union all
select 24019,'OSC' from dual union all
select 2402,'NASA' from dual union all
select 24020,'OSC' from dual union all
select 24021,'OSC' from dual union all
select 24022,'OSC' from dual union all
select 24023,'OSC' from dual union all
select 24024,'OSC' from dual union all
select 24025,'OSC' from dual union all
select 24026,'OSC' from dual union all
select 24027,'OSC' from dual union all
select 24028,'OSC' from dual union all
select 24029,'OSC' from dual union all
select 2403,'USAF' from dual union all
select 24030,'OSC' from dual union all
select 24031,'OSC' from dual union all
select 24032,'OSC' from dual union all
select 24033,'OSC' from dual union all
select 24034,'OSC' from dual union all
select 24035,'OSC' from dual union all
select 24036,'OSC' from dual union all
select 24037,'OSC' from dual union all
select 24038,'OSC' from dual union all
select 24039,'OSC' from dual union all
select 2404,'NASA' from dual union all
select 24040,'OSC' from dual union all
select 24041,'OSC' from dual union all
select 24042,'OSC' from dual union all
select 24043,'OSC' from dual union all
select 24044,'OSC' from dual union all
select 24045,'OSC' from dual union all
select 24046,'OSC' from dual union all
select 24047,'OSC' from dual union all
select 24048,'OSC' from dual union all
select 24049,'OSC' from dual union all
select 2405,'NASA' from dual union all
select 24050,'OSC' from dual union all
select 24051,'OSC' from dual union all
select 24052,'OSC' from dual union all
select 24053,'OSC' from dual union all
select 24054,'OSC' from dual union all
select 24055,'OSC' from dual union all
select 24056,'OSC' from dual union all
select 24057,'OSC' from dual union all
select 24058,'OSC' from dual union all
select 24059,'OSC' from dual;
insert into SATELLITE_ORG
select 2406,'IKI' from dual union all
select 2406,'NPOL' from dual union all
select 24060,'OSC' from dual union all
select 24061,'OSC' from dual union all
select 24062,'OSC' from dual union all
select 24063,'OSC' from dual union all
select 24064,'OSC' from dual union all
select 24065,'OSC' from dual union all
select 24066,'OSC' from dual union all
select 24067,'OSC' from dual union all
select 24068,'OSC' from dual union all
select 24069,'OSC' from dual union all
select 2407,'RVSN' from dual union all
select 24070,'OSC' from dual union all
select 24071,'FKA' from dual union all
select 24072,'RVSNR' from dual union all
select 24073,'OSC' from dual union all
select 24074,'OSC' from dual union all
select 24075,'OSC' from dual union all
select 24076,'OSC' from dual union all
select 24077,'OSC' from dual union all
select 24078,'OSC' from dual union all
select 24079,'OSC' from dual union all
select 2408,'RVSN' from dual union all
select 24080,'OSC' from dual union all
select 24081,'OSC' from dual union all
select 24082,'OSC' from dual union all
select 24083,'OSC' from dual union all
select 24084,'OSC' from dual union all
select 24085,'OSC' from dual union all
select 24086,'OSC' from dual union all
select 24087,'OSC' from dual union all
select 24088,'OSC' from dual union all
select 24089,'OSC' from dual union all
select 2409,'MO' from dual union all
select 24090,'OSC' from dual union all
select 24091,'OSC' from dual union all
select 24092,'OSC' from dual union all
select 24093,'OSC' from dual union all
select 24094,'OSC' from dual union all
select 24095,'OSC' from dual union all
select 24096,'OSC' from dual union all
select 24097,'OSC' from dual union all
select 24098,'OSC' from dual union all
select 24099,'OSC' from dual union all
select 241,'NASA' from dual union all
select 2410,'RVSN' from dual union all
select 24100,'OSC' from dual union all
select 24101,'OSC' from dual union all
select 24102,'OSC' from dual union all
select 24103,'OSC' from dual union all
select 24104,'OSC' from dual union all
select 24105,'OSC' from dual union all
select 24106,'OSC' from dual union all
select 24107,'OSC' from dual union all
select 24108,'OSC' from dual union all
select 24109,'OSC' from dual union all
select 2411,'USA' from dual union all
select 24110,'OSC' from dual union all
select 24111,'OSC' from dual union all
select 24112,'OSC' from dual union all
select 24113,'OSC' from dual union all
select 24114,'OSC' from dual union all
select 24115,'OSC' from dual union all
select 24116,'OSC' from dual union all
select 24117,'OSC' from dual union all
select 24118,'OSC' from dual union all
select 24119,'OSC' from dual union all
select 2412,'USAF' from dual union all
select 24120,'OSC' from dual union all
select 24121,'OSC' from dual union all
select 24122,'OSC' from dual union all
select 24123,'OSC' from dual union all
select 24124,'OSC' from dual union all
select 24125,'OSC' from dual union all
select 24126,'OSC' from dual union all
select 24127,'OSC' from dual union all
select 24128,'OSC' from dual union all
select 24129,'OSC' from dual union all
select 2413,'NASA' from dual union all
select 24130,'OSC' from dual union all
select 24131,'OSC' from dual union all
select 24132,'OSC' from dual union all
select 24133,'OSC' from dual union all
select 24134,'OSC' from dual union all
select 24135,'OSC' from dual union all
select 24136,'OSC' from dual union all
select 24137,'OSC' from dual union all
select 24138,'OSC' from dual union all
select 24139,'OSC' from dual union all
select 2414,'NASA' from dual union all
select 24140,'OSC' from dual union all
select 24141,'OSC' from dual union all
select 24142,'OSC' from dual union all
select 24143,'OSC' from dual union all
select 24144,'OSC' from dual union all
select 24145,'OSC' from dual union all
select 24146,'OSC' from dual union all
select 24147,'OSC' from dual union all
select 24148,'OSC' from dual;
insert into SATELLITE_ORG
select 24149,'OSC' from dual union all
select 2415,'NASA' from dual union all
select 24150,'OSC' from dual union all
select 24151,'OSC' from dual union all
select 24152,'OSC' from dual union all
select 24153,'OSC' from dual union all
select 24154,'OSC' from dual union all
select 24155,'OSC' from dual union all
select 24156,'OSC' from dual union all
select 24157,'OSC' from dual union all
select 24158,'OSC' from dual union all
select 24159,'OSC' from dual union all
select 2416,'USAF' from dual union all
select 24160,'OSC' from dual union all
select 24161,'OSC' from dual union all
select 24162,'OSC' from dual union all
select 24163,'OSC' from dual union all
select 24164,'OSC' from dual union all
select 24165,'OSC' from dual union all
select 24166,'OSC' from dual union all
select 24167,'OSC' from dual union all
select 24168,'OSC' from dual union all
select 24169,'OSC' from dual union all
select 2417,'NASA' from dual union all
select 24170,'OSC' from dual union all
select 24171,'OSC' from dual union all
select 24172,'OSC' from dual union all
select 24173,'OSC' from dual union all
select 24174,'OSC' from dual union all
select 24175,'OSC' from dual union all
select 24176,'OSC' from dual union all
select 24177,'OSC' from dual union all
select 24178,'OSC' from dual union all
select 24179,'OSC' from dual union all
select 2418,'USAF' from dual union all
select 24180,'OSC' from dual union all
select 24181,'OSC' from dual union all
select 24182,'OSC' from dual union all
select 24183,'OSC' from dual union all
select 24184,'OSC' from dual union all
select 24185,'OSC' from dual union all
select 24186,'OSC' from dual union all
select 24187,'OSC' from dual union all
select 24188,'OSC' from dual union all
select 24189,'OSC' from dual union all
select 2419,'NRO' from dual union all
select 24190,'OSC' from dual union all
select 24191,'OSC' from dual union all
select 24192,'OSC' from dual union all
select 24193,'OSC' from dual union all
select 24194,'OSC' from dual union all
select 24195,'OSC' from dual union all
select 24196,'OSC' from dual union all
select 24197,'OSC' from dual union all
select 24198,'OSC' from dual union all
select 24199,'OSC' from dual union all
select 242,'NRO' from dual union all
select 2420,'NRO' from dual union all
select 24200,'OSC' from dual union all
select 24201,'OSC' from dual union all
select 24202,'OSC' from dual union all
select 24203,'OSC' from dual union all
select 24204,'OSC' from dual union all
select 24205,'OSC' from dual union all
select 24206,'OSC' from dual union all
select 24207,'OSC' from dual union all
select 24208,'ASI' from dual union all
select 24209,'FTEL' from dual union all
select 2421,'NASA' from dual union all
select 24210,'AE' from dual union all
select 24211,'AE' from dual union all
select 24212,'OSC' from dual union all
select 24213,'OSC' from dual union all
select 24214,'OSC' from dual union all
select 24215,'OSC' from dual union all
select 24216,'OSC' from dual union all
select 24217,'OSC' from dual union all
select 24218,'OSC' from dual union all
select 24219,'OSC' from dual union all
select 2422,'USAF' from dual union all
select 24220,'OSC' from dual union all
select 24221,'OSC' from dual union all
select 24222,'OSC' from dual union all
select 24223,'OSC' from dual union all
select 24224,'OSC' from dual union all
select 24225,'OSC' from dual union all
select 24226,'OSC' from dual union all
select 24227,'OSC' from dual union all
select 24228,'OSC' from dual union all
select 24229,'OSC' from dual union all
select 2423,'USAF' from dual union all
select 24230,'OSC' from dual union all
select 24231,'OSC' from dual union all
select 24232,'OSC' from dual union all
select 24233,'OSC' from dual union all
select 24234,'OSC' from dual union all
select 24235,'OSC' from dual union all
select 24236,'OSC' from dual union all
select 24237,'OSC' from dual union all
select 24238,'OSC' from dual;
insert into SATELLITE_ORG
select 24239,'OSC' from dual union all
select 2424,'USAF' from dual union all
select 24240,'OSC' from dual union all
select 24241,'OSC' from dual union all
select 24242,'OSC' from dual union all
select 24243,'OSC' from dual union all
select 24244,'OSC' from dual union all
select 24245,'OSC' from dual union all
select 24246,'OSC' from dual union all
select 24247,'OSC' from dual union all
select 24248,'OSC' from dual union all
select 24249,'OSC' from dual union all
select 2425,'JPL' from dual union all
select 2425,'NASA' from dual union all
select 24250,'OSC' from dual union all
select 24251,'OSC' from dual union all
select 24252,'OSC' from dual union all
select 24253,'OSC' from dual union all
select 24254,'OSC' from dual union all
select 24255,'OSC' from dual union all
select 24256,'OSC' from dual union all
select 24257,'OSC' from dual union all
select 24258,'OSC' from dual union all
select 24259,'OSC' from dual union all
select 2426,'NASA' from dual union all
select 24260,'OSC' from dual union all
select 24261,'OSC' from dual union all
select 24262,'OSC' from dual union all
select 24263,'OSC' from dual union all
select 24264,'OSC' from dual union all
select 24265,'OSC' from dual union all
select 24266,'OSC' from dual union all
select 24267,'OSC' from dual union all
select 24268,'OSC' from dual union all
select 24269,'OSC' from dual union all
select 2427,'CIA' from dual union all
select 2427,'NRO' from dual union all
select 24270,'OSC' from dual union all
select 24271,'OSC' from dual union all
select 24272,'USN' from dual union all
select 24273,'VKS' from dual union all
select 24274,'RVSNR' from dual union all
select 24275,'RVSNR' from dual union all
select 24276,'RVSNR' from dual union all
select 24277,'NASDA' from dual union all
select 24278,'JARL' from dual union all
select 24279,'NASDA' from dual union all
select 2428,'USAF' from dual union all
select 24280,'FKA' from dual union all
select 24281,'VKS' from dual union all
select 24282,'CHIS' from dual union all
select 24283,'CASC' from dual union all
select 24284,'VKS' from dual union all
select 24285,'NASA' from dual union all
select 24286,'OSC' from dual union all
select 24287,'FKA' from dual union all
select 24288,'OSC' from dual union all
select 24289,'OSC' from dual union all
select 2429,'USAF' from dual union all
select 24290,'OSC' from dual union all
select 24291,'CORD' from dual union all
select 24292,'CZ' from dual union all
select 24293,'FKA' from dual union all
select 24293,'IKI' from dual union all
select 24294,'VKS' from dual union all
select 24295,'VKS' from dual union all
select 24296,'VKS' from dual union all
select 24297,'MORF' from dual union all
select 24298,'VKS' from dual union all
select 24299,'VKS' from dual union all
select 243,'USAF' from dual union all
select 2430,'USAF' from dual union all
select 24300,'VKS' from dual union all
select 24301,'VKS' from dual union all
select 24302,'VKS' from dual union all
select 24303,'VKS' from dual union all
select 24304,'VMFR' from dual union all
select 24305,'UNAM' from dual union all
select 24306,'VKS' from dual union all
select 24307,'INMAR' from dual union all
select 24308,'VKS' from dual union all
select 24309,'VKS' from dual union all
select 2431,'USAF' from dual union all
select 24310,'MDC' from dual union all
select 24311,'VKS' from dual union all
select 24312,'USAF' from dual union all
select 24313,'ECHO' from dual union all
select 24314,'AE' from dual union all
select 24315,'AMCG' from dual union all
select 24316,'LM' from dual union all
select 24317,'NASA' from dual union all
select 24318,'NASA' from dual union all
select 24319,'NASA' from dual union all
select 2432,'USN' from dual union all
select 24320,'USAF' from dual union all
select 24321,'MDC' from dual union all
select 24322,'OSC' from dual union all
select 24323,'MDC' from dual union all
select 24324,'NASA' from dual union all
select 24325,'OSC' from dual;
insert into SATELLITE_ORG
select 24326,'OSC' from dual union all
select 24327,'OSC' from dual union all
select 24328,'OSC' from dual union all
select 24329,'OSC' from dual union all
select 2433,'NRO' from dual union all
select 24330,'OSC' from dual union all
select 24331,'OSC' from dual union all
select 24332,'OSC' from dual union all
select 24333,'OSC' from dual union all
select 24334,'OSC' from dual union all
select 24335,'OSC' from dual union all
select 24336,'OSC' from dual union all
select 24337,'OSC' from dual union all
select 24338,'OSC' from dual union all
select 24339,'OSC' from dual union all
select 2434,'CIA' from dual union all
select 2434,'NRO' from dual union all
select 24340,'OSC' from dual union all
select 24341,'OSC' from dual union all
select 24342,'OSC' from dual union all
select 24343,'OSC' from dual union all
select 24344,'OSC' from dual union all
select 24345,'OSC' from dual union all
select 24346,'OSC' from dual union all
select 24347,'OSC' from dual union all
select 24348,'OSC' from dual union all
select 24349,'OSC' from dual union all
select 2435,'ESSA' from dual union all
select 24350,'OSC' from dual union all
select 24351,'OSC' from dual union all
select 24352,'OSC' from dual union all
select 24353,'OSC' from dual union all
select 24354,'OSC' from dual union all
select 24355,'OSC' from dual union all
select 24356,'OSC' from dual union all
select 24357,'OSC' from dual union all
select 24358,'OSC' from dual union all
select 24359,'OSC' from dual union all
select 2436,'NASA' from dual union all
select 24360,'OSC' from dual union all
select 24361,'OSC' from dual union all
select 24362,'OSC' from dual union all
select 24363,'OSC' from dual union all
select 24364,'OSC' from dual union all
select 24365,'OSC' from dual union all
select 24366,'OSC' from dual union all
select 24367,'OSC' from dual union all
select 24368,'OSC' from dual union all
select 24369,'OSC' from dual union all
select 2437,'RVSN' from dual union all
select 24370,'OSC' from dual union all
select 24371,'OSC' from dual union all
select 24372,'OSC' from dual union all
select 24373,'OSC' from dual union all
select 24374,'OSC' from dual union all
select 24375,'OSC' from dual union all
select 24376,'OSC' from dual union all
select 24377,'OSC' from dual union all
select 24378,'OSC' from dual union all
select 24379,'OSC' from dual union all
select 2438,'RVSN' from dual union all
select 24380,'OSC' from dual union all
select 24381,'OSC' from dual union all
select 24382,'OSC' from dual union all
select 24383,'OSC' from dual union all
select 24384,'OSC' from dual union all
select 24385,'OSC' from dual union all
select 24386,'OSC' from dual union all
select 24387,'OSC' from dual union all
select 24388,'OSC' from dual union all
select 24389,'OSC' from dual union all
select 2439,'RVSN' from dual union all
select 24390,'OSC' from dual union all
select 24391,'OSC' from dual union all
select 24392,'OSC' from dual union all
select 24393,'OSC' from dual union all
select 24394,'OSC' from dual union all
select 24395,'OSC' from dual union all
select 24396,'OSC' from dual union all
select 24397,'OSC' from dual union all
select 24398,'OSC' from dual union all
select 24399,'OSC' from dual union all
select 244,'USAF' from dual union all
select 2440,'RVSN' from dual union all
select 24400,'OSC' from dual union all
select 24401,'OSC' from dual union all
select 24402,'OSC' from dual union all
select 24403,'OSC' from dual union all
select 24404,'OSC' from dual union all
select 24405,'OSC' from dual union all
select 24406,'OSC' from dual union all
select 24407,'OSC' from dual union all
select 24408,'OSC' from dual union all
select 24409,'OSC' from dual union all
select 2441,'RVSN' from dual union all
select 24410,'OSC' from dual union all
select 24411,'OSC' from dual union all
select 24412,'OSC' from dual union all
select 24413,'OSC' from dual union all
select 24414,'OSC' from dual;
insert into SATELLITE_ORG
select 24415,'OSC' from dual union all
select 24416,'OSC' from dual union all
select 24417,'OSC' from dual union all
select 24418,'OSC' from dual union all
select 24419,'FKA' from dual union all
select 2442,'RVSN' from dual union all
select 24420,'OSC' from dual union all
select 24421,'OSC' from dual union all
select 24422,'OSC' from dual union all
select 24423,'OSC' from dual union all
select 24424,'OSC' from dual union all
select 24425,'OSC' from dual union all
select 24426,'OSC' from dual union all
select 24427,'OSC' from dual union all
select 24428,'OSC' from dual union all
select 24429,'OSC' from dual union all
select 2443,'RVSN' from dual union all
select 24430,'OSC' from dual union all
select 24431,'OSC' from dual union all
select 24432,'OSC' from dual union all
select 24433,'OSC' from dual union all
select 24434,'OSC' from dual union all
select 24435,'GPKS' from dual union all
select 24436,'VKS' from dual union all
select 24437,'VKS' from dual union all
select 24438,'VKS' from dual union all
select 24439,'OSC' from dual union all
select 2444,'RVSN' from dual union all
select 24440,'OSC' from dual union all
select 24441,'OSC' from dual union all
select 24442,'OSC' from dual union all
select 24443,'OSC' from dual union all
select 24444,'OSC' from dual union all
select 24445,'OSC' from dual union all
select 24446,'OSC' from dual union all
select 24447,'OSC' from dual union all
select 24448,'OSC' from dual union all
select 24449,'OSC' from dual union all
select 2445,'RVSN' from dual union all
select 24450,'OSC' from dual union all
select 24451,'OSC' from dual union all
select 24452,'OSC' from dual union all
select 24453,'OSC' from dual union all
select 24454,'VKS' from dual union all
select 24455,'VKS' from dual union all
select 24456,'VKS' from dual union all
select 24457,'OSC' from dual union all
select 24458,'OSC' from dual union all
select 24459,'OSC' from dual union all
select 2446,'RVSN' from dual union all
select 24460,'OSC' from dual union all
select 24461,'OSC' from dual union all
select 24462,'OSC' from dual union all
select 24463,'OSC' from dual union all
select 24464,'OSC' from dual union all
select 24465,'OSC' from dual union all
select 24466,'OSC' from dual union all
select 24467,'OSC' from dual union all
select 24468,'OSC' from dual union all
select 24469,'OSC' from dual union all
select 2447,'RVSN' from dual union all
select 24470,'OSC' from dual union all
select 24471,'OSC' from dual union all
select 24472,'OSC' from dual union all
select 24473,'OSC' from dual union all
select 24474,'OSC' from dual union all
select 24475,'OSC' from dual union all
select 24476,'OSC' from dual union all
select 24477,'OSC' from dual union all
select 24478,'OSC' from dual union all
select 24479,'OSC' from dual union all
select 2448,'RVSN' from dual union all
select 24480,'OSC' from dual union all
select 24481,'OSC' from dual union all
select 24482,'OSC' from dual union all
select 24483,'OSC' from dual union all
select 24484,'OSC' from dual union all
select 24485,'OSC' from dual union all
select 24486,'OSC' from dual union all
select 24487,'OSC' from dual union all
select 24488,'OSC' from dual union all
select 24489,'OSC' from dual union all
select 2449,'RVSN' from dual union all
select 24490,'OSC' from dual union all
select 24491,'OSC' from dual union all
select 24492,'OSC' from dual union all
select 24493,'OSC' from dual union all
select 24494,'OSC' from dual union all
select 24495,'OSC' from dual union all
select 24496,'OSC' from dual union all
select 24497,'OSC' from dual union all
select 24498,'OSC' from dual union all
select 24499,'OSC' from dual union all
select 245,'USAF' from dual union all
select 2450,'RVSN' from dual union all
select 24500,'OSC' from dual union all
select 24501,'OSC' from dual union all
select 24502,'OSC' from dual union all
select 24503,'VKS' from dual union all
select 24504,'OSC' from dual;
insert into SATELLITE_ORG
select 24505,'OSC' from dual union all
select 24506,'OSC' from dual union all
select 24507,'OSC' from dual union all
select 24508,'OSC' from dual union all
select 24509,'OSC' from dual union all
select 2451,'RVSN' from dual union all
select 24510,'OSC' from dual union all
select 24511,'OSC' from dual union all
select 24512,'OSC' from dual union all
select 24513,'OSC' from dual union all
select 24514,'OSC' from dual union all
select 24515,'OSC' from dual union all
select 24516,'OSC' from dual union all
select 24517,'OSC' from dual union all
select 24518,'OSC' from dual union all
select 24519,'OSC' from dual union all
select 2452,'RVSN' from dual union all
select 24520,'OSC' from dual union all
select 24521,'OSC' from dual union all
select 24522,'OSC' from dual union all
select 24523,'OSC' from dual union all
select 24524,'OSC' from dual union all
select 24525,'OSC' from dual union all
select 24526,'OSC' from dual union all
select 24527,'OSC' from dual union all
select 24528,'OSC' from dual union all
select 24529,'OSC' from dual union all
select 2453,'RVSN' from dual union all
select 24530,'OSC' from dual union all
select 24531,'OSC' from dual union all
select 24532,'OSC' from dual union all
select 24533,'OSC' from dual union all
select 24534,'OSC' from dual union all
select 24535,'OSC' from dual union all
select 24536,'OSC' from dual union all
select 24537,'OSC' from dual union all
select 24538,'OSC' from dual union all
select 24539,'OSC' from dual union all
select 2454,'RVSN' from dual union all
select 24540,'OSC' from dual union all
select 24541,'OSC' from dual union all
select 24542,'OSC' from dual union all
select 24543,'OSC' from dual union all
select 24544,'OSC' from dual union all
select 24545,'OSC' from dual union all
select 24546,'OSC' from dual union all
select 24547,'OSC' from dual union all
select 24548,'OSC' from dual union all
select 24549,'OSC' from dual union all
select 2455,'RVSN' from dual union all
select 24550,'OSC' from dual union all
select 24551,'OSC' from dual union all
select 24552,'OSC' from dual union all
select 24553,'OSC' from dual union all
select 24554,'OSC' from dual union all
select 24555,'OSC' from dual union all
select 24556,'OSC' from dual union all
select 24557,'OSC' from dual union all
select 24558,'OSC' from dual union all
select 24559,'OSC' from dual union all
select 2456,'RVSN' from dual union all
select 24560,'OSC' from dual union all
select 24561,'OSC' from dual union all
select 24562,'OSC' from dual union all
select 24563,'OSC' from dual union all
select 24564,'OSC' from dual union all
select 24565,'OSC' from dual union all
select 24566,'OSC' from dual union all
select 24567,'OSC' from dual union all
select 24568,'OSC' from dual union all
select 24569,'OSC' from dual union all
select 2457,'RVSN' from dual union all
select 24570,'OSC' from dual union all
select 24571,'OSC' from dual union all
select 24572,'OSC' from dual union all
select 24573,'OSC' from dual union all
select 24574,'OSC' from dual union all
select 24575,'OSC' from dual union all
select 24576,'OSC' from dual union all
select 24577,'OSC' from dual union all
select 24578,'OSC' from dual union all
select 24579,'OSC' from dual union all
select 2458,'RVSN' from dual union all
select 24580,'OSC' from dual union all
select 24581,'OSC' from dual union all
select 24582,'OSC' from dual union all
select 24583,'OSC' from dual union all
select 24584,'OSC' from dual union all
select 24585,'OSC' from dual union all
select 24586,'OSC' from dual union all
select 24587,'OSC' from dual union all
select 24588,'OSC' from dual union all
select 24589,'OSC' from dual union all
select 2459,'RVSN' from dual union all
select 24590,'OSC' from dual union all
select 24591,'OSC' from dual union all
select 24592,'OSC' from dual union all
select 24593,'OSC' from dual union all
select 24594,'OSC' from dual union all
select 24595,'OSC' from dual;
insert into SATELLITE_ORG
select 24596,'OSC' from dual union all
select 24597,'OSC' from dual union all
select 24598,'OSC' from dual union all
select 24599,'OSC' from dual union all
select 246,'USAF' from dual union all
select 2460,'RVSN' from dual union all
select 24600,'OSC' from dual union all
select 24601,'OSC' from dual union all
select 24602,'OSC' from dual union all
select 24603,'OSC' from dual union all
select 24604,'OSC' from dual union all
select 24605,'OSC' from dual union all
select 24606,'OSC' from dual union all
select 24607,'OSC' from dual union all
select 24608,'OSC' from dual union all
select 24609,'OSC' from dual union all
select 2461,'RVSN' from dual union all
select 24610,'OSC' from dual union all
select 24611,'OSC' from dual union all
select 24612,'OSC' from dual union all
select 24613,'OSC' from dual union all
select 24614,'OSC' from dual union all
select 24615,'OSC' from dual union all
select 24616,'OSC' from dual union all
select 24617,'OSC' from dual union all
select 24618,'OSC' from dual union all
select 24619,'OSC' from dual union all
select 2462,'RVSN' from dual union all
select 24620,'OSC' from dual union all
select 24621,'OSC' from dual union all
select 24622,'OSC' from dual union all
select 24623,'OSC' from dual union all
select 24624,'OSC' from dual union all
select 24625,'OSC' from dual union all
select 24626,'OSC' from dual union all
select 24627,'OSC' from dual union all
select 24628,'OSC' from dual union all
select 24629,'OSC' from dual union all
select 2463,'RVSN' from dual union all
select 24630,'OSC' from dual union all
select 24631,'OSC' from dual union all
select 24632,'OSC' from dual union all
select 24633,'OSC' from dual union all
select 24634,'CASC' from dual union all
select 24635,'CASC' from dual union all
select 24636,'USAF' from dual union all
select 24637,'NRO' from dual union all
select 24638,'NRO' from dual union all
select 24639,'NASA' from dual union all
select 2464,'RVSN' from dual union all
select 24640,'GPKS' from dual union all
select 24640,'VKS' from dual union all
select 24641,'RVSNR' from dual union all
select 24642,'RVSNR' from dual union all
select 24643,'RVSNR' from dual union all
select 24644,'CNSA' from dual union all
select 24645,'NASA' from dual union all
select 24646,'NASA' from dual union all
select 24647,'USAF' from dual union all
select 24648,'JPL' from dual union all
select 24648,'NASA' from dual union all
select 24649,'MDC' from dual union all
select 2465,'RVSN' from dual union all
select 24650,'MDC' from dual union all
select 24651,'MDC' from dual union all
select 24652,'ARAB' from dual union all
select 24653,'BINA' from dual union all
select 24654,'AE' from dual union all
select 24655,'AE' from dual union all
select 24656,'VKS' from dual union all
select 24657,'FKA' from dual union all
select 24658,'FKA' from dual union all
select 24659,'FKA' from dual union all
select 2466,'RVSN' from dual union all
select 24660,'NASA' from dual union all
select 24661,'DLR' from dual union all
select 24662,'SII' from dual union all
select 24663,'FKA' from dual union all
select 24664,'VKS' from dual union all
select 24665,'EUTEL' from dual union all
select 24666,'LM' from dual union all
select 24667,'JPL' from dual union all
select 24667,'NASA' from dual union all
select 24668,'BOE' from dual union all
select 24669,'BOE' from dual union all
select 2467,'RVSN' from dual union all
select 24670,'VMFR' from dual union all
select 24671,'VKS' from dual union all
select 24672,'FKA' from dual union all
select 24673,'FKA' from dual union all
select 24674,'INMAR' from dual union all
select 24675,'LM' from dual union all
select 24676,'USAF' from dual union all
select 24677,'VMFR' from dual union all
select 24678,'VKS' from dual union all
select 24679,'RVSNR' from dual union all
select 2468,'RVSN' from dual union all
select 24680,'NROC' from dual union all
select 24680,'USAF' from dual union all
select 24681,'USAF' from dual;
insert into SATELLITE_ORG
select 24682,'OSC' from dual union all
select 24683,'OSC' from dual union all
select 24684,'OSC' from dual union all
select 24685,'OSC' from dual union all
select 24686,'OSC' from dual union all
select 24687,'OSC' from dual union all
select 24688,'OSC' from dual union all
select 24689,'OSC' from dual union all
select 2469,'RVSN' from dual union all
select 24690,'OSC' from dual union all
select 24691,'OSC' from dual union all
select 24692,'OSC' from dual union all
select 24693,'OSC' from dual union all
select 24694,'OSC' from dual union all
select 24695,'OSC' from dual union all
select 24696,'OSC' from dual union all
select 24697,'OSC' from dual union all
select 24698,'OSC' from dual union all
select 24699,'OSC' from dual union all
select 247,'CIA' from dual union all
select 247,'NRO' from dual union all
select 2470,'RVSN' from dual union all
select 24700,'ESA' from dual union all
select 24701,'FKA' from dual union all
select 24701,'IMBP' from dual union all
select 24702,'VKS' from dual union all
select 24703,'RVSNR' from dual union all
select 24704,'USN' from dual union all
select 24705,'USN' from dual union all
select 24706,'TSSKB' from dual union all
select 24707,'TSSKB' from dual union all
select 24708,'FKA' from dual union all
select 24709,'FKA' from dual union all
select 2471,'RVSN' from dual union all
select 24710,'FKA' from dual union all
select 24711,'NASA' from dual union all
select 24712,'AE' from dual union all
select 24713,'AMCG' from dual union all
select 24714,'NAHU' from dual union all
select 24715,'AE' from dual union all
select 24716,'AE' from dual union all
select 24717,'FKA' from dual union all
select 24718,'VKS' from dual union all
select 24719,'NASA' from dual union all
select 2472,'RVSN' from dual union all
select 24720,'ISASS' from dual union all
select 24721,'ISASS' from dual union all
select 24722,'AE' from dual union all
select 24723,'AE' from dual union all
select 24724,'AE' from dual union all
select 24725,'VKS' from dual union all
select 24726,'VKS' from dual union all
select 24727,'VKS' from dual union all
select 24728,'FKA' from dual union all
select 24729,'FKA' from dual union all
select 2473,'RVSN' from dual union all
select 24730,'FKA' from dual union all
select 24731,'VKS' from dual union all
select 24732,'JSAT' from dual union all
select 24733,'LM' from dual union all
select 24734,'OSC' from dual union all
select 24735,'OSC' from dual union all
select 24736,'RVSNR' from dual union all
select 24737,'USAF' from dual union all
select 24738,'USAF' from dual union all
select 24739,'USAF' from dual union all
select 2474,'RVSN' from dual union all
select 24740,'USAF' from dual union all
select 24741,'AE' from dual union all
select 24742,'INTEL' from dual union all
select 24743,'AE' from dual union all
select 24744,'VKS' from dual union all
select 24745,'VKS' from dual union all
select 24746,'VKS' from dual union all
select 24747,'VKS' from dual union all
select 24748,'TCI' from dual union all
select 24749,'LM' from dual union all
select 2475,'RVSN' from dual union all
select 24750,'AE' from dual union all
select 24751,'AE' from dual union all
select 24752,'RVSNR' from dual union all
select 24753,'USAF' from dual union all
select 24754,'ESA' from dual union all
select 24755,'NASA' from dual union all
select 24756,'USAF' from dual union all
select 24757,'FKA' from dual union all
select 24758,'VKS' from dual union all
select 24759,'OSC' from dual union all
select 2476,'RVSN' from dual union all
select 24760,'OSC' from dual union all
select 24761,'PVO' from dual union all
select 24762,'VKS' from dual union all
select 24763,'VKS' from dual union all
select 24764,'VKS' from dual union all
select 24765,'USAF' from dual union all
select 24766,'OSC' from dual union all
select 24767,'OSC' from dual union all
select 24768,'SHIN' from dual union all
select 24769,'BSAT' from dual union all
select 2477,'RVSN' from dual;
insert into SATELLITE_ORG
select 24770,'AE' from dual union all
select 24771,'AE' from dual union all
select 24772,'VMFR' from dual union all
select 24773,'VKS' from dual union all
select 24774,'OSC' from dual union all
select 24775,'OSC' from dual union all
select 24776,'OSC' from dual union all
select 24777,'USAF' from dual union all
select 24778,'USAF' from dual union all
select 24779,'INTA' from dual union all
select 2478,'RVSN' from dual union all
select 24780,'OSC' from dual union all
select 24781,'PVO' from dual union all
select 24782,'PVO' from dual union all
select 24783,'PVO' from dual union all
select 24784,'PVO' from dual union all
select 24785,'PVO' from dual union all
select 24786,'NOAA' from dual union all
select 24787,'LM' from dual union all
select 24788,'OSC' from dual union all
select 24789,'PVO' from dual union all
select 2479,'RVSN' from dual union all
select 24790,'OSC' from dual union all
select 24791,'OSC' from dual union all
select 24792,'IRID' from dual union all
select 24793,'IRID' from dual union all
select 24794,'IRID' from dual union all
select 24795,'IRID' from dual union all
select 24796,'IRID' from dual union all
select 24797,'BOE' from dual union all
select 24798,'CHIS' from dual union all
select 24799,'CASC' from dual union all
select 248,'USAF' from dual union all
select 2480,'RVSN' from dual union all
select 24800,'PVO' from dual union all
select 24801,'VKS' from dual union all
select 24802,'VKS' from dual union all
select 24803,'VKS' from dual union all
select 24804,'NASA' from dual union all
select 24805,'VKS' from dual union all
select 24806,'VKS' from dual union all
select 24807,'USN' from dual union all
select 24808,'TNOR' from dual union all
select 24809,'BOE' from dual union all
select 2481,'USAF' from dual union all
select 24810,'BOE' from dual union all
select 24811,'USN' from dual union all
select 24812,'LORC' from dual union all
select 24813,'VKS' from dual union all
select 24814,'VKS' from dual union all
select 24815,'VKS' from dual union all
select 24816,'VKS' from dual union all
select 24817,'OSC' from dual union all
select 24818,'RVSN' from dual union all
select 24819,'INMAR' from dual union all
select 2482,'RVSN' from dual union all
select 24820,'ISRO' from dual union all
select 24821,'AE' from dual union all
select 24822,'AE' from dual union all
select 24823,'OSC' from dual union all
select 24824,'OSC' from dual union all
select 24825,'OSC' from dual union all
select 24826,'OSC' from dual union all
select 24827,'NPOL' from dual union all
select 24828,'VKS' from dual union all
select 24829,'VKS' from dual union all
select 2483,'RVSN' from dual union all
select 24830,'VKS' from dual union all
select 24831,'VKS' from dual union all
select 24832,'VKS' from dual union all
select 24833,'VKS' from dual union all
select 24834,'CMA' from dual union all
select 24835,'CASC' from dual union all
select 24836,'IRID' from dual union all
select 24837,'IRID' from dual union all
select 24838,'IRID' from dual union all
select 24839,'IRID' from dual union all
select 2484,'RVSN' from dual union all
select 24840,'IRID' from dual union all
select 24841,'IRID' from dual union all
select 24842,'IRID' from dual union all
select 24843,'VKS' from dual union all
select 24844,'OSC' from dual union all
select 24845,'FKA' from dual union all
select 24846,'INTEL' from dual union all
select 24847,'AE' from dual union all
select 24848,'FKA' from dual union all
select 24849,'NASA' from dual union all
select 2485,'RVSN' from dual union all
select 24850,'OSC' from dual union all
select 24851,'FKA' from dual union all
select 24852,'RVSN' from dual union all
select 24853,'VMFR' from dual union all
select 24854,'VMFR' from dual union all
select 24855,'VMFR' from dual union all
select 24856,'VMFR' from dual union all
select 24857,'VMFR' from dual union all
select 24858,'VMFR' from dual union all
select 24859,'VMFR' from dual union all
select 2486,'RVSN' from dual;
insert into SATELLITE_ORG
select 24860,'VMFR' from dual union all
select 24861,'VMFR' from dual union all
select 24862,'VMFR' from dual union all
select 24863,'VMFR' from dual union all
select 24864,'VMFR' from dual union all
select 24865,'USAF' from dual union all
select 24866,'RVSN' from dual union all
select 24867,'USAF' from dual union all
select 24868,'OSC' from dual union all
select 24869,'IRID' from dual union all
select 2487,'RVSN' from dual union all
select 24870,'IRID' from dual union all
select 24871,'IRID' from dual union all
select 24872,'IRID' from dual union all
select 24873,'IRID' from dual union all
select 24874,'BOE' from dual union all
select 24875,'VKS' from dual union all
select 24876,'USAF' from dual union all
select 24877,'BOE' from dual union all
select 24878,'BOE' from dual union all
select 24879,'VKS' from dual union all
select 2488,'RVSN' from dual union all
select 24880,'SCC' from dual union all
select 24881,'LM' from dual union all
select 24882,'BOE' from dual union all
select 24883,'ORBI' from dual union all
select 24884,'OSC' from dual union all
select 24885,'BOE' from dual union all
select 24886,'FKA' from dual union all
select 24887,'RVSN' from dual union all
select 24888,'FKA' from dual union all
select 24889,'NASA' from dual union all
select 2489,'NRO' from dual union all
select 24890,'DLR' from dual union all
select 24891,'PAN' from dual union all
select 24892,'AE' from dual union all
select 24893,'VKS' from dual union all
select 24894,'PVO' from dual union all
select 24895,'RVSN' from dual union all
select 24896,'RVSN' from dual union all
select 24897,'RVSN' from dual union all
select 24898,'BOE' from dual union all
select 24899,'RVSN' from dual union all
select 249,'USAF' from dual union all
select 2490,'USAF' from dual union all
select 24900,'RVSN' from dual union all
select 24901,'MPSC' from dual union all
select 24902,'CASC' from dual union all
select 24903,'IRID' from dual union all
select 24904,'IRID' from dual union all
select 24905,'IRID' from dual union all
select 24906,'IRID' from dual union all
select 24907,'IRID' from dual union all
select 24908,'BOE' from dual union all
select 24909,'NASA' from dual union all
select 24909,'TRW' from dual union all
select 2491,'MO' from dual union all
select 24910,'LM' from dual union all
select 24911,'NASA' from dual union all
select 24912,'NASA' from dual union all
select 24913,'BOE' from dual union all
select 24914,'NASA' from dual union all
select 24915,'NASA' from dual union all
select 24916,'PAN' from dual union all
select 24917,'RVSN' from dual union all
select 24918,'RVSN' from dual union all
select 24919,'RVSN' from dual union all
select 2492,'RVSN' from dual union all
select 24920,'USAF' from dual union all
select 24921,'OSC' from dual union all
select 24922,'BOE' from dual union all
select 24923,'BOE' from dual union all
select 24924,'BOE' from dual union all
select 24925,'CASC' from dual union all
select 24926,'CASC' from dual union all
select 24927,'CASC' from dual union all
select 24928,'CASC' from dual union all
select 24929,'CASC' from dual union all
select 2493,'USAF' from dual union all
select 24930,'CASC' from dual union all
select 24931,'EUTEL' from dual union all
select 24932,'EUMET' from dual union all
select 24933,'AE' from dual union all
select 24934,'AE' from dual union all
select 24935,'NASA' from dual union all
select 24936,'AMCG' from dual union all
select 24937,'LM' from dual union all
select 24938,'VKS' from dual union all
select 24939,'OSC' from dual union all
select 2494,'USAF' from dual union all
select 24940,'OSC' from dual union all
select 24941,'OSC' from dual union all
select 24942,'OSC' from dual union all
select 24943,'FKA' from dual union all
select 24944,'IRID' from dual union all
select 24945,'IRID' from dual union all
select 24946,'IRID' from dual union all
select 24947,'IRID' from dual union all
select 24948,'IRID' from dual union all
select 24949,'IRID' from dual;
insert into SATELLITE_ORG
select 2495,'USAF' from dual union all
select 24950,'IRID' from dual union all
select 24951,'RVSN' from dual union all
select 24952,'FKA' from dual union all
select 24953,'VMFR' from dual union all
select 24954,'FAI' from dual union all
select 24955,'RVSN' from dual union all
select 24956,'NASA' from dual union all
select 24957,'INTEL' from dual union all
select 24958,'FKA' from dual union all
select 24959,'USAF' from dual union all
select 2496,'USAF' from dual union all
select 24960,'RVSN' from dual union all
select 24961,'RVSN' from dual union all
select 24962,'RVSN' from dual union all
select 24963,'RVSN' from dual union all
select 24964,'NASA' from dual union all
select 24965,'IRID' from dual union all
select 24966,'IRID' from dual union all
select 24967,'IRID' from dual union all
select 24968,'IRID' from dual union all
select 24969,'IRID' from dual union all
select 2497,'USAF' from dual union all
select 24970,'BOE' from dual union all
select 24971,'ISRO' from dual union all
select 24972,'ISRO' from dual union all
select 24973,'OSC' from dual union all
select 24974,'VKS' from dual union all
select 24975,'OSC' from dual union all
select 24976,'OSC' from dual union all
select 24977,'OSC' from dual union all
select 24978,'OSC' from dual union all
select 24979,'OSC' from dual union all
select 2498,'USAF' from dual union all
select 24980,'OSC' from dual union all
select 24981,'OSC' from dual union all
select 24982,'OSC' from dual union all
select 24983,'OSC' from dual union all
select 24984,'OSC' from dual union all
select 24985,'OSC' from dual union all
select 24986,'OSC' from dual union all
select 24987,'OSC' from dual union all
select 24988,'OSC' from dual union all
select 24989,'OSC' from dual union all
select 2499,'USAF' from dual union all
select 24990,'VKS' from dual union all
select 24991,'OSC' from dual union all
select 24992,'OSC' from dual union all
select 24993,'OSC' from dual union all
select 24994,'OSC' from dual union all
select 24995,'OSC' from dual union all
select 24996,'OSC' from dual union all
select 24997,'OSC' from dual union all
select 24998,'OSC' from dual union all
select 24999,'OSC' from dual union all
select 25,'CIA' from dual union all
select 250,'USAF' from dual union all
select 2500,'USAF' from dual union all
select 25000,'USAF' from dual union all
select 25001,'USAF' from dual union all
select 25002,'FKA' from dual union all
select 25003,'RVSN' from dual union all
select 25004,'ECHO' from dual union all
select 25005,'LM' from dual union all
select 25006,'FKA' from dual union all
select 25007,'RVSN' from dual union all
select 25008,'JPL' from dual union all
select 25008,'NASA' from dual union all
select 25009,'NASA' from dual union all
select 2501,'OKB10' from dual union all
select 25010,'APT' from dual union all
select 25011,'CASC' from dual union all
select 25012,'VKS' from dual union all
select 25013,'USAF' from dual union all
select 25014,'OSC' from dual union all
select 25015,'FKA' from dual union all
select 25016,'FKA' from dual union all
select 25017,'CIA' from dual union all
select 25017,'NROC' from dual union all
select 25018,'USAF' from dual union all
select 25019,'DOD' from dual union all
select 25019,'USAF' from dual union all
select 2502,'MO' from dual union all
select 25020,'LM' from dual union all
select 25021,'USAF' from dual union all
select 25022,'VKS' from dual union all
select 25023,'ESA' from dual union all
select 25024,'ESA' from dual union all
select 25025,'ESA' from dual union all
select 25026,'ESA' from dual union all
select 25027,'VKS' from dual union all
select 25028,'VKS' from dual union all
select 25029,'MO' from dual union all
select 2503,'RVSN' from dual union all
select 25030,'USAF' from dual union all
select 25031,'BOE' from dual union all
select 25032,'BOE' from dual union all
select 25033,'MDC' from dual union all
select 25034,'NROC' from dual union all
select 25034,'USAF' from dual;
insert into SATELLITE_ORG
select 25035,'USAF' from dual union all
select 25036,'FKA' from dual union all
select 25037,'FKA' from dual union all
select 25038,'FKA' from dual union all
select 25039,'IRID' from dual union all
select 2504,'RVSN' from dual union all
select 25040,'IRID' from dual union all
select 25041,'IRID' from dual union all
select 25042,'IRID' from dual union all
select 25043,'IRID' from dual union all
select 25044,'BOE' from dual union all
select 25045,'TSBM' from dual union all
select 25046,'RVSN' from dual union all
select 25047,'RVSN' from dual union all
select 25048,'RVSN' from dual union all
select 25049,'NSAB' from dual union all
select 2505,'RVSN' from dual union all
select 25050,'INDO' from dual union all
select 25051,'AE' from dual union all
select 25052,'AE' from dual union all
select 25053,'RVSN' from dual union all
select 25054,'RVSN' from dual union all
select 25055,'JPL' from dual union all
select 25055,'NASA' from dual union all
select 25056,'USN' from dual union all
select 25057,'VMF' from dual union all
select 25058,'NASA' from dual union all
select 25059,'MVD' from dual union all
select 2506,'RVSN' from dual union all
select 25060,'RVSN' from dual union all
select 25061,'NASA' from dual union all
select 25062,'NASA' from dual union all
select 25063,'NASA' from dual union all
select 25064,'NASDA' from dual union all
select 25065,'NASDA' from dual union all
select 25066,'NASDA' from dual union all
select 25067,'JSAT' from dual union all
select 25068,'MPE' from dual union all
select 25069,'AE' from dual union all
select 2507,'RVSN' from dual union all
select 25070,'AE' from dual union all
select 25071,'SES' from dual union all
select 25072,'RVSN' from dual union all
select 25073,'RVSN' from dual union all
select 25074,'RVSN' from dual union all
select 25075,'OSC' from dual union all
select 25076,'OSC' from dual union all
select 25077,'IRID' from dual union all
select 25078,'IRID' from dual union all
select 25079,'CASC' from dual union all
select 2508,'IKI' from dual union all
select 2508,'NPOL' from dual union all
select 25080,'CASC' from dual union all
select 25081,'CASC' from dual union all
select 25082,'CASC' from dual union all
select 25083,'CASC' from dual union all
select 25084,'CASC' from dual union all
select 25085,'RVSNR' from dual union all
select 25086,'PAN' from dual union all
select 25087,'LM' from dual union all
select 25088,'VMFR' from dual union all
select 25089,'RVSN' from dual union all
select 2509,'RVSN' from dual union all
select 25090,'MDC' from dual union all
select 25091,'FKA' from dual union all
select 25092,'FKA' from dual union all
select 25093,'RVSN' from dual union all
select 25094,'RVSN' from dual union all
select 25095,'RVSN' from dual union all
select 25096,'RVSN' from dual union all
select 25097,'USAF' from dual union all
select 25098,'ESA' from dual union all
select 25099,'MDC' from dual union all
select 251,'USAF' from dual union all
select 2510,'RVSN' from dual union all
select 25100,'DASARI' from dual union all
select 25101,'USAF' from dual union all
select 25102,'FKA' from dual union all
select 25103,'RVSNR' from dual union all
select 25104,'IRID' from dual union all
select 25105,'IRID' from dual union all
select 25106,'IRID' from dual union all
select 25107,'IRID' from dual union all
select 25108,'IRID' from dual union all
select 25109,'BOE' from dual union all
select 2511,'NASA' from dual union all
select 25110,'INTEL' from dual union all
select 25111,'AE' from dual union all
select 25112,'ORBC' from dual union all
select 25113,'ORBC' from dual union all
select 25114,'ORBC' from dual union all
select 25115,'ORBC' from dual union all
select 25116,'ORBC' from dual union all
select 25117,'ORBC' from dual union all
select 25118,'ORBC' from dual union all
select 25119,'ORBC' from dual union all
select 2512,'JPL' from dual union all
select 2512,'NASA' from dual union all
select 25120,'OSC' from dual union all
select 25121,'OSC' from dual;
insert into SATELLITE_ORG
select 25122,'USAF' from dual union all
select 25123,'EWAT' from dual union all
select 25124,'RVSN' from dual union all
select 25125,'RVSN' from dual union all
select 25126,'ASIA' from dual union all
select 25127,'RVSN' from dual union all
select 25128,'RVSN' from dual union all
select 25129,'RVSN' from dual union all
select 2513,'NASA' from dual union all
select 25130,'PVO' from dual union all
select 25131,'NASA' from dual union all
select 25132,'LM' from dual union all
select 25133,'FKA' from dual union all
select 25134,'MOD' from dual union all
select 25135,'BOE' from dual union all
select 25136,'BOE' from dual union all
select 25137,'BOE' from dual union all
select 25138,'BOE' from dual union all
select 25139,'BOE' from dual union all
select 2514,'INTEL' from dual union all
select 25140,'FKA' from dual union all
select 25141,'IRID' from dual union all
select 25142,'FKA' from dual union all
select 25143,'NASA' from dual union all
select 25144,'RVSNR' from dual union all
select 25145,'NASA' from dual union all
select 25146,'FKA' from dual union all
select 25147,'RVSNR' from dual union all
select 25148,'NROC' from dual union all
select 25149,'LM' from dual union all
select 2515,'NASA' from dual union all
select 25150,'AE' from dual union all
select 25151,'AE' from dual union all
select 25152,'TELB' from dual union all
select 25153,'INMAR' from dual union all
select 25154,'AE' from dual union all
select 25155,'AE' from dual union all
select 25156,'USN' from dual union all
select 25157,'USN' from dual union all
select 25158,'ORBC' from dual union all
select 25159,'ORBC' from dual union all
select 2516,'USAF' from dual union all
select 25160,'OSC' from dual union all
select 25161,'OSC' from dual union all
select 25162,'GLOB' from dual union all
select 25163,'GLOB' from dual union all
select 25164,'GLOB' from dual union all
select 25165,'GLOB' from dual union all
select 25166,'BOE' from dual union all
select 25167,'VTSR' from dual union all
select 25168,'RVSNR' from dual union all
select 25169,'IRID' from dual union all
select 2517,'USAF' from dual union all
select 25170,'IRID' from dual union all
select 25171,'IRID' from dual union all
select 25172,'IRID' from dual union all
select 25173,'IRID' from dual union all
select 25174,'BOE' from dual union all
select 25175,'NASDA' from dual union all
select 25176,'NASDA' from dual union all
select 25177,'RVSN' from dual union all
select 25178,'RVSN' from dual union all
select 25179,'RVSN' from dual union all
select 2518,'NASA' from dual union all
select 25180,'RVSN' from dual union all
select 25181,'RVSN' from dual union all
select 25182,'RVSN' from dual union all
select 25183,'RVSN' from dual union all
select 25184,'RVSN' from dual union all
select 25185,'RVSN' from dual union all
select 25186,'RVSN' from dual union all
select 25187,'RVSN' from dual union all
select 25188,'RVSN' from dual union all
select 25189,'RVSN' from dual union all
select 2519,'NASA' from dual union all
select 25190,'RVSN' from dual union all
select 25191,'RVSN' from dual union all
select 25192,'RVSN' from dual union all
select 25193,'RVSN' from dual union all
select 25194,'RVSN' from dual union all
select 25195,'RVSN' from dual union all
select 25196,'RVSN' from dual union all
select 25197,'RVSN' from dual union all
select 25198,'RVSN' from dual union all
select 25199,'RVSN' from dual union all
select 252,'USAF' from dual union all
select 2520,'USA' from dual union all
select 25200,'RVSN' from dual union all
select 25201,'RVSN' from dual union all
select 25202,'RVSN' from dual union all
select 25203,'RVSN' from dual union all
select 25204,'RVSN' from dual union all
select 25205,'RVSN' from dual union all
select 25206,'RVSN' from dual union all
select 25207,'RVSN' from dual union all
select 25208,'RVSN' from dual union all
select 25209,'RVSN' from dual union all
select 2521,'USAF' from dual union all
select 25210,'RVSN' from dual union all
select 25211,'RVSN' from dual;
insert into SATELLITE_ORG
select 25212,'RVSN' from dual union all
select 25213,'RVSN' from dual union all
select 25214,'RVSN' from dual union all
select 25215,'RVSN' from dual union all
select 25216,'RVSN' from dual union all
select 25217,'RVSN' from dual union all
select 25218,'RVSN' from dual union all
select 25219,'RVSN' from dual union all
select 2522,'USAF' from dual union all
select 25220,'RVSN' from dual union all
select 25221,'RVSN' from dual union all
select 25222,'RVSN' from dual union all
select 25223,'RVSN' from dual union all
select 25224,'RVSN' from dual union all
select 25225,'RVSN' from dual union all
select 25226,'RVSN' from dual union all
select 25227,'RVSN' from dual union all
select 25228,'RVSN' from dual union all
select 25229,'RVSN' from dual union all
select 2523,'NRO' from dual union all
select 25230,'RVSN' from dual union all
select 25231,'RVSN' from dual union all
select 25232,'RVSN' from dual union all
select 25233,'NASA' from dual union all
select 25234,'TELD' from dual union all
select 25235,'OSC' from dual union all
select 25236,'OSC' from dual union all
select 25237,'EUTEL' from dual union all
select 25238,'AE' from dual union all
select 25239,'INTEL' from dual union all
select 2524,'USAF' from dual union all
select 25240,'LM' from dual union all
select 25241,'IRID' from dual union all
select 25242,'IRID' from dual union all
select 25243,'RVSN' from dual union all
select 25244,'RVSN' from dual union all
select 25245,'RVSN' from dual union all
select 25246,'RVSN' from dual union all
select 25247,'RVSN' from dual union all
select 25248,'RVSN' from dual union all
select 25249,'RVSN' from dual union all
select 2525,'USAF' from dual union all
select 25250,'RVSN' from dual union all
select 25251,'RVSN' from dual union all
select 25252,'RVSN' from dual union all
select 25253,'NASA' from dual union all
select 25254,'RVSNR' from dual union all
select 25255,'RVSNR' from dual union all
select 25256,'FKA' from dual union all
select 25257,'RVSNR' from dual union all
select 25258,'USN' from dual union all
select 25259,'LM' from dual union all
select 2526,'USAF' from dual union all
select 25260,'CNES' from dual union all
select 25261,'AE' from dual union all
select 25262,'IRID' from dual union all
select 25263,'IRID' from dual union all
select 25264,'CASC' from dual union all
select 25265,'CASC' from dual union all
select 25266,'CASC' from dual union all
select 25267,'CASC' from dual union all
select 25268,'CASC' from dual union all
select 25269,'CASC' from dual union all
select 2527,'USAF' from dual union all
select 25270,'NASA' from dual union all
select 25271,'OSC' from dual union all
select 25272,'IRID' from dual union all
select 25273,'IRID' from dual union all
select 25274,'IRID' from dual union all
select 25275,'IRID' from dual union all
select 25276,'IRID' from dual union all
select 25277,'BOE' from dual union all
select 25278,'RVSN' from dual union all
select 25279,'AE' from dual union all
select 2528,'USAF' from dual union all
select 25280,'NASA' from dual union all
select 25281,'OSC' from dual union all
select 25282,'AE' from dual union all
select 25283,'FKA' from dual union all
select 25284,'RVSNR' from dual union all
select 25285,'IRID' from dual union all
select 25286,'IRID' from dual union all
select 25287,'IRID' from dual union all
select 25288,'IRID' from dual union all
select 25289,'IRID' from dual union all
select 2529,'USAF' from dual union all
select 25290,'IRID' from dual union all
select 25291,'IRID' from dual union all
select 25292,'FKA' from dual union all
select 25293,'FKA' from dual union all
select 25294,'NPOE' from dual union all
select 25295,'RVSN' from dual union all
select 25296,'RVSN' from dual union all
select 25297,'NASA' from dual union all
select 25298,'IRID' from dual union all
select 25299,'IRID' from dual union all
select 253,'USAF' from dual union all
select 2530,'USAF' from dual union all
select 25300,'RVSN' from dual union all
select 25301,'FKA' from dual;
insert into SATELLITE_ORG
select 25302,'FKA' from dual union all
select 25303,'FKA' from dual union all
select 25304,'FKA' from dual union all
select 25305,'OSC' from dual union all
select 25306,'GLOB' from dual union all
select 25307,'GLOB' from dual union all
select 25308,'GLOB' from dual union all
select 25309,'GLOB' from dual union all
select 2531,'USAF' from dual union all
select 25310,'BOE' from dual union all
select 25311,'ERTU' from dual union all
select 25312,'BSAT' from dual union all
select 25313,'AE' from dual union all
select 25314,'AE' from dual union all
select 25315,'PVO' from dual union all
select 25316,'RVSNR' from dual union all
select 25317,'RVSNR' from dual union all
select 25318,'RVSNR' from dual union all
select 25319,'IRID' from dual union all
select 2532,'USAF' from dual union all
select 25320,'IRID' from dual union all
select 25321,'CASC' from dual union all
select 25322,'CASC' from dual union all
select 25323,'CASC' from dual union all
select 25324,'CASC' from dual union all
select 25325,'CASC' from dual union all
select 25326,'CASC' from dual union all
select 25327,'PVO' from dual union all
select 25328,'RVSNR' from dual union all
select 25329,'RVSNR' from dual union all
select 2533,'USAF' from dual union all
select 25330,'RVSNR' from dual union all
select 25331,'ECHO' from dual union all
select 25332,'RVSN' from dual union all
select 25333,'RVSN' from dual union all
select 25334,'RVSN' from dual union all
select 25335,'OSC' from dual union all
select 25336,'DOD' from dual union all
select 25336,'NROC' from dual union all
select 25337,'USAF' from dual union all
select 25338,'NOAA' from dual union all
select 25339,'VKS' from dual union all
select 2534,'NASA' from dual union all
select 25340,'FKA' from dual union all
select 25341,'RVSNR' from dual union all
select 25342,'IRID' from dual union all
select 25343,'IRID' from dual union all
select 25344,'IRID' from dual union all
select 25345,'IRID' from dual union all
select 25346,'IRID' from dual union all
select 25347,'BOE' from dual union all
select 25348,'IRID' from dual union all
select 25349,'USAF' from dual union all
select 2535,'USAF' from dual union all
select 25350,'BOE' from dual union all
select 25351,'RVSNR' from dual union all
select 25352,'RVSNR' from dual union all
select 25353,'EUMET' from dual union all
select 25354,'CHIS' from dual union all
select 25355,'CASC' from dual union all
select 25356,'NASA' from dual union all
select 25357,'OSC' from dual union all
select 25358,'TNOR' from dual union all
select 25359,'BOE' from dual union all
select 2536,'RVSN' from dual union all
select 25360,'BOE' from dual union all
select 25361,'OSC' from dual union all
select 25362,'OSC' from dual union all
select 25363,'RVSN' from dual union all
select 25364,'RVSN' from dual union all
select 25365,'RVSN' from dual union all
select 25366,'RVSN' from dual union all
select 25367,'RVSN' from dual union all
select 25368,'RVSN' from dual union all
select 25369,'RVSN' from dual union all
select 2537,'CIA' from dual union all
select 2537,'NRO' from dual union all
select 25370,'RVSN' from dual union all
select 25371,'INTEL' from dual union all
select 25372,'LM' from dual union all
select 25373,'RVSN' from dual union all
select 25374,'RVSNR' from dual union all
select 25375,'RVSN' from dual union all
select 25376,'RVSN' from dual union all
select 25377,'RVSNR' from dual union all
select 25378,'OSC' from dual union all
select 25379,'GPKS' from dual union all
select 25379,'RVSN' from dual union all
select 2538,'RVSN' from dual union all
select 25380,'RVSNR' from dual union all
select 25381,'RVSNR' from dual union all
select 25382,'RVSNR' from dual union all
select 25383,'ISASS' from dual union all
select 25384,'ISASS' from dual union all
select 25385,'ISASS' from dual union all
select 25386,'ISASS' from dual union all
select 25387,'RVSNR' from dual union all
select 25388,'ISASS' from dual union all
select 25389,'TUB1' from dual union all
select 2539,'RVSN' from dual;
insert into SATELLITE_ORG
select 25390,'TUB1' from dual union all
select 25391,'MAK' from dual union all
select 25392,'MAK' from dual union all
select 25393,'MAK' from dual union all
select 25394,'MVD' from dual union all
select 25395,'FACH' from dual union all
select 25396,'TMSAT' from dual union all
select 25397,'TECN' from dual union all
select 25398,'EOSAU' from dual union all
select 25399,'DLR' from dual union all
select 254,'USAF' from dual union all
select 2540,'RVSN' from dual union all
select 25400,'RVSN' from dual union all
select 25401,'RVSN' from dual union all
select 25402,'RVSN' from dual union all
select 25403,'RVSN' from dual union all
select 25404,'EURAS' from dual union all
select 25404,'SINO' from dual union all
select 25405,'CASC' from dual union all
select 25406,'MORF' from dual union all
select 25407,'RVSN' from dual union all
select 25408,'USN' from dual union all
select 25409,'RVSN' from dual union all
select 2541,'RVSN' from dual union all
select 25410,'RVSN' from dual union all
select 25411,'RVSN' from dual union all
select 25412,'RVSN' from dual union all
select 25413,'ORBC' from dual union all
select 25414,'ORBC' from dual union all
select 25415,'ORBC' from dual union all
select 25416,'ORBC' from dual union all
select 25417,'ORBC' from dual union all
select 25418,'ORBC' from dual union all
select 25419,'ORBC' from dual union all
select 2542,'RVSN' from dual union all
select 25420,'ORBC' from dual union all
select 25421,'OSC' from dual union all
select 25422,'OSC' from dual union all
select 25423,'VKS' from dual union all
select 25424,'NASDA' from dual union all
select 25425,'OSC' from dual union all
select 25426,'OSC' from dual union all
select 25427,'OSC' from dual union all
select 25428,'OSC' from dual union all
select 25429,'FKA' from dual union all
select 2543,'RVSN' from dual union all
select 25430,'RVSNR' from dual union all
select 25431,'IRID' from dual union all
select 25432,'IRID' from dual union all
select 25433,'CASC' from dual union all
select 25434,'CASC' from dual union all
select 25435,'CASC' from dual union all
select 25436,'CASC' from dual union all
select 25437,'CASC' from dual union all
select 25438,'CASC' from dual union all
select 25439,'FKA' from dual union all
select 2544,'RVSN' from dual union all
select 25440,'OSC' from dual union all
select 25441,'OSC' from dual union all
select 25442,'OSC' from dual union all
select 25443,'OSC' from dual union all
select 25444,'RVSN' from dual union all
select 25445,'OSC' from dual union all
select 25446,'OSC' from dual union all
select 25447,'OSC' from dual union all
select 25448,'OSC' from dual union all
select 25449,'OSC' from dual union all
select 2545,'RVSN' from dual union all
select 25450,'RVSN' from dual union all
select 25451,'RVSN' from dual union all
select 25452,'RVSN' from dual union all
select 25453,'RVSN' from dual union all
select 25454,'RVSN' from dual union all
select 25455,'RVSN' from dual union all
select 25456,'RVSN' from dual union all
select 25457,'RVSN' from dual union all
select 25458,'RVSN' from dual union all
select 25459,'RVSN' from dual union all
select 2546,'RVSN' from dual union all
select 25460,'STEL' from dual union all
select 25461,'AE' from dual union all
select 25462,'SES' from dual union all
select 25463,'RVSN' from dual union all
select 25464,'RVSN' from dual union all
select 25465,'RVSN' from dual union all
select 25466,'RVSNR' from dual union all
select 25467,'IRID' from dual union all
select 25468,'IRID' from dual union all
select 25469,'IRID' from dual union all
select 2547,'RVSN' from dual union all
select 25470,'IRID' from dual union all
select 25471,'IRID' from dual union all
select 25472,'BOE' from dual union all
select 25473,'PAN' from dual union all
select 25474,'AE' from dual union all
select 25475,'ORBC' from dual union all
select 25476,'ORBC' from dual union all
select 25477,'ORBC' from dual union all
select 25478,'ORBC' from dual union all
select 25479,'ORBC' from dual;
insert into SATELLITE_ORG
select 2548,'RVSN' from dual union all
select 25480,'ORBC' from dual union all
select 25481,'ORBC' from dual union all
select 25482,'ORBC' from dual union all
select 25483,'OSC' from dual union all
select 25484,'OSC' from dual union all
select 25485,'RVSN' from dual union all
select 25486,'RVSNR' from dual union all
select 25487,'RVSNR' from dual union all
select 25488,'RVSNR' from dual union all
select 25489,'NROC' from dual union all
select 2549,'RVSN' from dual union all
select 25490,'OSC' from dual union all
select 25491,'EUTEL' from dual union all
select 25492,'NSAB' from dual union all
select 25493,'AE' from dual union all
select 25494,'AE' from dual union all
select 25495,'EUTEL' from dual union all
select 25496,'LM' from dual union all
select 25497,'AE' from dual union all
select 25498,'AE' from dual union all
select 25499,'RVSNR' from dual union all
select 255,'NASA' from dual union all
select 2550,'RVSN' from dual union all
select 25500,'RVSNR' from dual union all
select 25501,'USN' from dual union all
select 25502,'LM' from dual union all
select 25503,'ESA' from dual union all
select 25504,'INPE' from dual union all
select 25505,'RVSN' from dual union all
select 25506,'OSC' from dual union all
select 25507,'OSC' from dual union all
select 25508,'JPL' from dual union all
select 25508,'NASA' from dual union all
select 25509,'SEDSH' from dual union all
select 2551,'RVSN' from dual union all
select 25510,'BOE' from dual union all
select 25511,'BOE' from dual union all
select 25512,'FKA' from dual union all
select 25513,'RVSNR' from dual union all
select 25514,'USN' from dual union all
select 25515,'WORSP' from dual union all
select 25516,'AMCG' from dual union all
select 25517,'AE' from dual union all
select 25518,'AE' from dual union all
select 25519,'NASA' from dual union all
select 2552,'RVSN' from dual union all
select 25520,'USN' from dual union all
select 25521,'NASA' from dual union all
select 25522,'PAN' from dual union all
select 25523,'RVSN' from dual union all
select 25524,'RVSN' from dual union all
select 25525,'RVSN' from dual union all
select 25526,'NASA' from dual union all
select 25527,'IRID' from dual union all
select 25528,'IRID' from dual union all
select 25529,'IRID' from dual union all
select 2553,'RVSN' from dual union all
select 25530,'IRID' from dual union all
select 25531,'IRID' from dual union all
select 25532,'BOE' from dual union all
select 25533,'ACF' from dual union all
select 25533,'RUAF' from dual union all
select 25534,'FKA' from dual union all
select 25535,'FKA' from dual union all
select 25536,'FKA' from dual union all
select 25537,'FKA' from dual union all
select 25538,'AE' from dual union all
select 25539,'AE' from dual union all
select 2554,'RVSN' from dual union all
select 25540,'AE' from dual union all
select 25541,'AE' from dual union all
select 25542,'AE' from dual union all
select 25543,'AE' from dual union all
select 25544,'NASA' from dual union all
select 25545,'RVSNR' from dual union all
select 25546,'BONUM' from dual union all
select 25547,'BOE' from dual union all
select 25548,'BOE' from dual union all
select 25549,'NASA' from dual union all
select 2555,'RVSN' from dual union all
select 25550,'CONAB' from dual union all
select 25551,'USAF' from dual union all
select 25552,'NPOE' from dual union all
select 25553,'NPOE' from dual union all
select 25554,'NPOE' from dual union all
select 25555,'NPOE' from dual union all
select 25556,'NPOE' from dual union all
select 25557,'NPOE' from dual union all
select 25558,'SAMX' from dual union all
select 25559,'AE' from dual union all
select 2556,'RVSN' from dual union all
select 25560,'NASA' from dual union all
select 25561,'OSC' from dual union all
select 25562,'OSC' from dual union all
select 25563,'MO' from dual union all
select 25564,'NASA' from dual union all
select 25565,'NASA' from dual union all
select 25566,'OSC' from dual union all
select 25567,'RVSN' from dual;
insert into SATELLITE_ORG
select 25567,'VMFR' from dual union all
select 25568,'SSC' from dual union all
select 25569,'RVSN' from dual union all
select 2557,'RVSN' from dual union all
select 25570,'NASA' from dual union all
select 25571,'JPL' from dual union all
select 25572,'BOE' from dual union all
select 25573,'BOE' from dual union all
select 25574,'AE' from dual union all
select 25575,'NASA' from dual union all
select 25576,'USN' from dual union all
select 25577,'IRID' from dual union all
select 25578,'IRID' from dual union all
select 25579,'CASC' from dual union all
select 2558,'RVSN' from dual union all
select 25580,'CASC' from dual union all
select 25581,'CASC' from dual union all
select 25582,'CASC' from dual union all
select 25583,'CASC' from dual union all
select 25584,'CASC' from dual union all
select 25585,'PAN' from dual union all
select 25586,'AE' from dual union all
select 25587,'USAF' from dual union all
select 25588,'USAF' from dual union all
select 25589,'USAF' from dual union all
select 2559,'RVSN' from dual union all
select 25590,'VMFR' from dual union all
select 25591,'NRO' from dual union all
select 25592,'RVSN' from dual union all
select 25593,'FKA' from dual union all
select 25593,'RVSN' from dual union all
select 25594,'FKA' from dual union all
select 25594,'RVSN' from dual union all
select 25595,'FKA' from dual union all
select 25595,'RVSN' from dual union all
select 25596,'RVSNR' from dual union all
select 25597,'RVSNR' from dual union all
select 25598,'RVSNR' from dual union all
select 25599,'RVSNR' from dual union all
select 256,'USAF' from dual union all
select 2560,'RVSN' from dual union all
select 25600,'RVSNR' from dual union all
select 25601,'RVSNR' from dual union all
select 25602,'RVSNR' from dual union all
select 25603,'RVSNR' from dual union all
select 25604,'RVSNR' from dual union all
select 25605,'JPL' from dual union all
select 25606,'BOE' from dual union all
select 25607,'BOE' from dual union all
select 25608,'USAF' from dual union all
select 25609,'NROC' from dual union all
select 2561,'RVSN' from dual union all
select 25610,'NROC' from dual union all
select 25611,'CASC' from dual union all
select 25612,'NASA' from dual union all
select 25613,'NASA' from dual union all
select 25614,'NASA' from dual union all
select 25615,'NROC' from dual union all
select 25615,'USN' from dual union all
select 25616,'NSPT' from dual union all
select 25617,'LM' from dual union all
select 25618,'JPL' from dual union all
select 25619,'BOE' from dual union all
select 2562,'RVSN' from dual union all
select 25620,'BOE' from dual union all
select 25621,'GLOB' from dual union all
select 25622,'GLOB' from dual union all
select 25623,'GLOB' from dual union all
select 25624,'GLOB' from dual union all
select 25625,'RVSNR' from dual union all
select 25626,'LORC' from dual union all
select 25627,'RVSNR' from dual union all
select 25628,'RVSNR' from dual union all
select 25629,'RVSNR' from dual union all
select 2563,'RVSN' from dual union all
select 25630,'JSAT' from dual union all
select 25631,'LM' from dual union all
select 25632,'FKA' from dual union all
select 25633,'RVSNR' from dual union all
select 25634,'USAF' from dual union all
select 25635,'DMI' from dual union all
select 25636,'STBO' from dual union all
select 25637,'BOE' from dual union all
select 25638,'ARAB' from dual union all
select 25639,'MOD' from dual union all
select 2564,'RVSN' from dual union all
select 25640,'AE' from dual union all
select 25641,'AE' from dual union all
select 25642,'RVSN' from dual union all
select 25643,'RVSNR' from dual union all
select 25644,'RVSNR' from dual union all
select 25645,'RVSNR' from dual union all
select 25646,'NASA' from dual union all
select 25647,'OSC' from dual union all
select 25648,'NASA' from dual union all
select 25649,'GLOB' from dual union all
select 2565,'NASA' from dual union all
select 25650,'GLOB' from dual union all
select 25651,'GLOB' from dual union all
select 25652,'GLOB' from dual;
insert into SATELLITE_ORG
select 25653,'RVSNR' from dual union all
select 25654,'RVSNR' from dual union all
select 25655,'RVSNR' from dual union all
select 25656,'RVSN' from dual union all
select 25657,'ASIA' from dual union all
select 25658,'RVSNR' from dual union all
select 25659,'RVSNR' from dual union all
select 2566,'NASA' from dual union all
select 25660,'RVSNR' from dual union all
select 25661,'SEALP' from dual union all
select 25662,'SEALC' from dual union all
select 25663,'OSC' from dual union all
select 25664,'FKA' from dual union all
select 25665,'RVSNR' from dual union all
select 25666,'ISRO' from dual union all
select 25667,'AE' from dual union all
select 25668,'AE' from dual union all
select 25669,'USAF' from dual union all
select 2567,'USAF' from dual union all
select 25670,'USAF' from dual union all
select 25671,'USAF' from dual union all
select 25672,'USAF' from dual union all
select 25673,'EUTEL' from dual union all
select 25674,'LM' from dual union all
select 25675,'USAF' from dual union all
select 25676,'GLOB' from dual union all
select 25677,'GLOB' from dual union all
select 25678,'GLOB' from dual union all
select 25679,'GLOB' from dual union all
select 2568,'MO' from dual union all
select 25680,'RVSNR' from dual union all
select 25681,'RVSNR' from dual union all
select 25682,'USGSR' from dual union all
select 25683,'BOE' from dual union all
select 25684,'RVSNR' from dual union all
select 25685,'AMRU' from dual union all
select 25686,'FKA' from dual union all
select 25687,'FKA' from dual union all
select 25688,'FKA' from dual union all
select 25689,'FKA' from dual union all
select 2569,'RVSN' from dual union all
select 25690,'FKA' from dual union all
select 25691,'FKA' from dual union all
select 25692,'FKA' from dual union all
select 25693,'SURREY' from dual union all
select 25694,'KTRAS' from dual union all
select 25695,'KTRAS' from dual union all
select 25696,'RVSNR' from dual union all
select 25697,'RVSNR' from dual union all
select 25698,'RVSNR' from dual union all
select 25699,'RVSNR' from dual union all
select 257,'NASA' from dual union all
select 2570,'RVSN' from dual union all
select 25700,'RVSNR' from dual union all
select 25701,'RVSNR' from dual union all
select 25702,'RVSNR' from dual union all
select 25703,'RVSNR' from dual union all
select 25704,'RVSNR' from dual union all
select 25705,'RVSNR' from dual union all
select 25706,'RVSNR' from dual union all
select 25707,'RVSNR' from dual union all
select 25708,'RVSNR' from dual union all
select 25709,'RVSNR' from dual union all
select 2571,'RVSN' from dual union all
select 25710,'RVSNR' from dual union all
select 25711,'RVSNR' from dual union all
select 25712,'RVSNR' from dual union all
select 25713,'RVSNR' from dual union all
select 25714,'RVSNR' from dual union all
select 25715,'RVSNR' from dual union all
select 25716,'RVSNR' from dual union all
select 25717,'RVSNR' from dual union all
select 25718,'RVSNR' from dual union all
select 25719,'RVSNR' from dual union all
select 2572,'NASA' from dual union all
select 25720,'RVSNR' from dual union all
select 25721,'DLR' from dual union all
select 25722,'MEGS' from dual union all
select 25723,'RVSNR' from dual union all
select 25724,'USAF' from dual union all
select 25725,'USAF' from dual union all
select 25726,'AE' from dual union all
select 25727,'LORC' from dual union all
select 25728,'BOE' from dual union all
select 25729,'BOE' from dual union all
select 2573,'USAF' from dual union all
select 25730,'CMA' from dual union all
select 25731,'CASC' from dual union all
select 25732,'CNSA' from dual union all
select 25733,'CASC' from dual union all
select 25734,'AE' from dual union all
select 25735,'NASA' from dual union all
select 25736,'USA' from dual union all
select 25737,'OSC' from dual union all
select 25738,'OSC' from dual union all
select 25739,'OSC' from dual union all
select 2574,'USAF' from dual union all
select 25740,'TCAN' from dual union all
select 25741,'RVSNR' from dual union all
select 25742,'RVSNR' from dual;
insert into SATELLITE_ORG
select 25743,'RVSNR' from dual union all
select 25744,'NROC' from dual union all
select 25745,'USAF' from dual union all
select 25746,'NROC' from dual union all
select 25747,'NROC' from dual union all
select 25748,'NROC' from dual union all
select 25749,'NROC' from dual union all
select 2575,'RVSN' from dual union all
select 25750,'OSC' from dual union all
select 25751,'NROC' from dual union all
select 25752,'NROC' from dual union all
select 25753,'NROC' from dual union all
select 25754,'NROC' from dual union all
select 25755,'NROC' from dual union all
select 25756,'KAIST' from dual union all
select 25757,'DLR' from dual union all
select 25758,'ISRO' from dual union all
select 25759,'ISRO' from dual union all
select 2576,'RVSN' from dual union all
select 25760,'NASA' from dual union all
select 25761,'RVSNR' from dual union all
select 25762,'RVSNR' from dual union all
select 25763,'ESA' from dual union all
select 25764,'ESA' from dual union all
select 25765,'ESA' from dual union all
select 25766,'ESA' from dual union all
select 25767,'ESA' from dual union all
select 25768,'ESA' from dual union all
select 25769,'NASA' from dual union all
select 2577,'NASA' from dual union all
select 25770,'GLOB' from dual union all
select 25771,'GLOB' from dual union all
select 25772,'GLOB' from dual union all
select 25773,'GLOB' from dual union all
select 25774,'BOE' from dual union all
select 25775,'USAF' from dual union all
select 25776,'NASDA' from dual union all
select 25777,'IRID' from dual union all
select 25778,'IRID' from dual union all
select 25779,'CASC' from dual union all
select 2578,'NASA' from dual union all
select 25780,'CASC' from dual union all
select 25781,'CASC' from dual union all
select 25782,'CASC' from dual union all
select 25783,'CASC' from dual union all
select 25784,'CASC' from dual union all
select 25785,'SES' from dual union all
select 25786,'RVSNR' from dual union all
select 25787,'RVSNR' from dual union all
select 25788,'RVSNR' from dual union all
select 25789,'NASA' from dual union all
select 2579,'NASA' from dual union all
select 25790,'USAF' from dual union all
select 25791,'NASA' from dual union all
select 25792,'BOE' from dual union all
select 25793,'USAF' from dual union all
select 25794,'RVSN' from dual union all
select 25795,'RVSN' from dual union all
select 25796,'RVSN' from dual union all
select 25797,'RVSNR' from dual union all
select 25798,'RVSNR' from dual union all
select 25799,'RVSNR' from dual union all
select 258,'USAF' from dual union all
select 2580,'USN' from dual union all
select 25800,'RVSNR' from dual union all
select 25801,'RVSNR' from dual union all
select 25802,'RVSNR' from dual union all
select 25803,'RVSNR' from dual union all
select 25804,'RVSNR' from dual union all
select 25805,'RVSNR' from dual union all
select 25806,'RVSNR' from dual union all
select 25807,'RVSNR' from dual union all
select 25808,'RVSNR' from dual union all
select 25809,'RVSNR' from dual union all
select 2581,'RVSN' from dual union all
select 25810,'RVSNR' from dual union all
select 25811,'RVSNR' from dual union all
select 25812,'RVSNR' from dual union all
select 25813,'RVSNR' from dual union all
select 25814,'RVSNR' from dual union all
select 25815,'RVSNR' from dual union all
select 25816,'RVSNR' from dual union all
select 25817,'RVSNR' from dual union all
select 25818,'RVSNR' from dual union all
select 25819,'ISRO' from dual union all
select 2582,'RVSN' from dual union all
select 25820,'VMFR' from dual union all
select 25821,'AE' from dual union all
select 25822,'AE' from dual union all
select 25823,'AE' from dual union all
select 25824,'AE' from dual union all
select 25825,'NASA' from dual union all
select 25826,'USAF' from dual union all
select 25827,'USAF' from dual union all
select 25828,'USAF' from dual union all
select 25829,'USAF' from dual union all
select 2583,'RVSN' from dual union all
select 25830,'USN' from dual union all
select 25831,'RVSN' from dual union all
select 25832,'RVSN' from dual;
insert into SATELLITE_ORG
select 25833,'RVSN' from dual union all
select 25834,'RVSN' from dual union all
select 25835,'RVSN' from dual union all
select 25836,'RVSN' from dual union all
select 25837,'ESA' from dual union all
select 25838,'RVSN' from dual union all
select 25839,'RVSN' from dual union all
select 2584,'RVSN' from dual union all
select 25840,'RVSN' from dual union all
select 25841,'RVSN' from dual union all
select 25842,'USAF' from dual union all
select 25843,'AE' from dual union all
select 25844,'AE' from dual union all
select 25845,'AE' from dual union all
select 25846,'NASA' from dual union all
select 25847,'GPKS' from dual union all
select 25847,'RVSN' from dual union all
select 25848,'RVSNR' from dual union all
select 25849,'RVSNR' from dual union all
select 2585,'RVSN' from dual union all
select 25850,'RVSNR' from dual union all
select 25851,'GLOB' from dual union all
select 25852,'GLOB' from dual union all
select 25853,'GLOB' from dual union all
select 25854,'GLOB' from dual union all
select 25855,'BOE' from dual union all
select 25856,'ISASS' from dual union all
select 25857,'ISASS' from dual union all
select 25858,'FKA' from dual union all
select 25859,'RVSNR' from dual union all
select 2586,'RVSN' from dual union all
select 25860,'RGMS' from dual union all
select 25860,'YUZH' from dual union all
select 25861,'RVSNR' from dual union all
select 25862,'RVSNR' from dual union all
select 25863,'RVSNR' from dual union all
select 25864,'RVSNR' from dual union all
select 25865,'RVSNR' from dual union all
select 25866,'NASA' from dual union all
select 25867,'NASA' from dual union all
select 25868,'NASA' from dual union all
select 25869,'NASA' from dual union all
select 2587,'RVSN' from dual union all
select 25870,'FKA' from dual union all
select 25871,'FKA' from dual union all
select 25872,'GLOB' from dual union all
select 25873,'GLOB' from dual union all
select 25874,'GLOB' from dual union all
select 25875,'GLOB' from dual union all
select 25876,'BOE' from dual union all
select 25877,'FKA' from dual union all
select 25878,'FKA' from dual union all
select 25879,'FKA' from dual union all
select 2588,'NASA' from dual union all
select 25880,'TELK' from dual union all
select 25881,'AE' from dual union all
select 25882,'CNSA' from dual union all
select 25883,'GLOB' from dual union all
select 25884,'GLOB' from dual union all
select 25885,'GLOB' from dual union all
select 25886,'GLOB' from dual union all
select 25887,'BOE' from dual union all
select 25888,'RVSNR' from dual union all
select 25889,'RVSN' from dual union all
select 2589,'NASA' from dual union all
select 25890,'RVSNR' from dual union all
select 25891,'RVSN' from dual union all
select 25892,'VMFR' from dual union all
select 25893,'RVSNR' from dual union all
select 25894,'KTEL' from dual union all
select 25895,'AE' from dual union all
select 25896,'GAZC' from dual union all
select 25897,'GAZC' from dual union all
select 25898,'KHRR' from dual union all
select 25899,'KHRR' from dual union all
select 259,'NRO' from dual union all
select 2590,'NASA' from dual union all
select 25900,'KHRR' from dual union all
select 25901,'KHRR' from dual union all
select 25902,'FKA' from dual union all
select 25903,'RVSNR' from dual union all
select 25904,'MDC' from dual union all
select 25905,'RVSNR' from dual union all
select 25906,'RVSNR' from dual union all
select 25907,'GLOB' from dual union all
select 25908,'GLOB' from dual union all
select 25909,'GLOB' from dual union all
select 2591,'NASA' from dual union all
select 25910,'GLOB' from dual union all
select 25911,'RVSNR' from dual union all
select 25912,'RVSNR' from dual union all
select 25913,'ECHO' from dual union all
select 25914,'LM' from dual union all
select 25915,'RVSNR' from dual union all
select 25916,'RVSNR' from dual union all
select 25917,'RVSNR' from dual union all
select 25918,'RVSNR' from dual union all
select 25919,'SIMG' from dual union all
select 2592,'NASA' from dual union all
select 25920,'LM' from dual;
insert into SATELLITE_ORG
select 25921,'LM' from dual union all
select 25922,'LORC' from dual union all
select 25923,'AE' from dual union all
select 25924,'LM' from dual union all
select 25925,'KHRR' from dual union all
select 25926,'KHRR' from dual union all
select 25927,'KHRR' from dual union all
select 25928,'AE' from dual union all
select 25929,'MVD' from dual union all
select 2593,'NASA' from dual union all
select 25930,'RVSNR' from dual union all
select 25931,'USAF' from dual union all
select 25932,'USAF' from dual union all
select 25933,'USAF' from dual union all
select 25934,'BOE' from dual union all
select 25935,'BOE' from dual union all
select 25936,'MAI' from dual union all
select 25937,'DTV' from dual union all
select 25938,'SEALC' from dual union all
select 25939,'ESA' from dual union all
select 2594,'NASA' from dual union all
select 25940,'CASC' from dual union all
select 25940,'INPE' from dual union all
select 25941,'INPE' from dual union all
select 25942,'CNSA' from dual union all
select 25943,'GLOB' from dual union all
select 25944,'GLOB' from dual union all
select 25945,'GLOB' from dual union all
select 25946,'GLOB' from dual union all
select 25947,'RVSNR' from dual union all
select 25948,'RVSNR' from dual union all
select 25949,'LORC' from dual union all
select 2595,'NASA' from dual union all
select 25950,'AE' from dual union all
select 25951,'FKA' from dual union all
select 25952,'FKA' from dual union all
select 25953,'FKA' from dual union all
select 25954,'AMCG' from dual union all
select 25955,'AE' from dual union all
select 25956,'PLA' from dual union all
select 25957,'CASC' from dual union all
select 25958,'CASC' from dual union all
select 25959,'CASC' from dual union all
select 2596,'NASA' from dual union all
select 25960,'PLA' from dual union all
select 25961,'GLOB' from dual union all
select 25962,'GLOB' from dual union all
select 25963,'GLOB' from dual union all
select 25964,'GLOB' from dual union all
select 25965,'RVSNR' from dual union all
select 25966,'RVSNR' from dual union all
select 25967,'USN' from dual union all
select 25968,'LM' from dual union all
select 25969,'RVSNR' from dual union all
select 2597,'NASA' from dual union all
select 25970,'RVSNR' from dual union all
select 25971,'RVSNR' from dual union all
select 25972,'RVSNR' from dual union all
select 25973,'RVSNR' from dual union all
select 25974,'RVSNR' from dual union all
select 25975,'RVSNR' from dual union all
select 25976,'RVSNR' from dual union all
select 25977,'CNES' from dual union all
select 25977,'DGA' from dual union all
select 25978,'CNES' from dual union all
select 25978,'DGA' from dual union all
select 25979,'AE' from dual union all
select 2598,'RVSN' from dual union all
select 25980,'ORBC' from dual union all
select 25981,'ORBC' from dual union all
select 25982,'ORBC' from dual union all
select 25983,'ORBC' from dual union all
select 25984,'ORBC' from dual union all
select 25985,'ORBC' from dual union all
select 25986,'ORBC' from dual union all
select 25987,'OSC' from dual union all
select 25988,'OSC' from dual union all
select 25989,'ESA' from dual union all
select 2599,'MO' from dual union all
select 25990,'ESA' from dual union all
select 25991,'USAF' from dual union all
select 25992,'USAF' from dual union all
select 25993,'USAF' from dual union all
select 25994,'NASA' from dual union all
select 25995,'LM' from dual union all
select 25996,'NASA' from dual union all
select 25997,'NASA' from dual union all
select 25998,'RVSNR' from dual union all
select 25999,'RVSNR' from dual union all
select 26,'CIA' from dual union all
select 260,'USAF' from dual union all
select 2600,'RVSN' from dual union all
select 26000,'RVSNR' from dual union all
select 26001,'RVSNR' from dual union all
select 26002,'RVSNR' from dual union all
select 26003,'RVSNR' from dual union all
select 26004,'RVSNR' from dual union all
select 26005,'RVSNR' from dual union all
select 26006,'RVSNR' from dual union all
select 26007,'RVSNR' from dual;
insert into SATELLITE_ORG
select 26008,'RVSNR' from dual union all
select 26009,'RVSNR' from dual union all
select 2601,'TSKBEM' from dual union all
select 26010,'RVSNR' from dual union all
select 26011,'RVSNR' from dual union all
select 26012,'RVSNR' from dual union all
select 26013,'RVSNR' from dual union all
select 26014,'RVSNR' from dual union all
select 26015,'RVSNR' from dual union all
select 26016,'RVSNR' from dual union all
select 26017,'RVSNR' from dual union all
select 26018,'RVSNR' from dual union all
select 26019,'RVSNR' from dual union all
select 2602,'RVSN' from dual union all
select 26020,'RVSNR' from dual union all
select 26021,'RVSNR' from dual union all
select 26022,'RVSNR' from dual union all
select 26023,'RVSNR' from dual union all
select 26024,'RVSNR' from dual union all
select 26025,'RVSNR' from dual union all
select 26026,'RVSNR' from dual union all
select 26027,'RVSNR' from dual union all
select 26028,'RVSNR' from dual union all
select 26029,'RVSNR' from dual union all
select 2603,'MO' from dual union all
select 26030,'RVSNR' from dual union all
select 26031,'AE' from dual union all
select 26032,'KARI' from dual union all
select 26033,'JPL' from dual union all
select 26033,'NASA' from dual union all
select 26034,'OSC' from dual union all
select 26035,'OSC' from dual union all
select 26036,'NASA' from dual union all
select 26037,'RVSNR' from dual union all
select 26038,'PAN' from dual union all
select 26039,'AE' from dual union all
select 2604,'RVSN' from dual union all
select 26040,'VMFR' from dual union all
select 26041,'RVSN' from dual union all
select 26042,'PVO' from dual union all
select 26043,'RVSNR' from dual union all
select 26044,'RVSNR' from dual union all
select 26045,'RVSNR' from dual union all
select 26046,'RVSNR' from dual union all
select 26047,'RVSNR' from dual union all
select 26048,'USAF' from dual union all
select 26049,'USAF' from dual union all
select 2605,'MO' from dual union all
select 26050,'USAF' from dual union all
select 26051,'OSC' from dual union all
select 26052,'DOD' from dual union all
select 26052,'USAF' from dual union all
select 26053,'LM' from dual union all
select 26054,'USAF' from dual union all
select 26055,'RVSNR' from dual union all
select 26056,'PAN' from dual union all
select 26057,'AE' from dual union all
select 26058,'PLA' from dual union all
select 26059,'CASC' from dual union all
select 2606,'NRO' from dual union all
select 26060,'RVSNR' from dual union all
select 26061,'WEB' from dual union all
select 26062,'USAF' from dual union all
select 26063,'STAN' from dual union all
select 26064,'USAF' from dual union all
select 26065,'ASU' from dual union all
select 26066,'OSC' from dual union all
select 26067,'FKA' from dual union all
select 26068,'RVSNR' from dual union all
select 26069,'MORF' from dual union all
select 2607,'USAF' from dual union all
select 26070,'RVSN' from dual union all
select 26071,'HISP' from dual union all
select 26072,'LM' from dual union all
select 26073,'RVSN' from dual union all
select 26074,'RVSN' from dual union all
select 26075,'RVSN' from dual union all
select 26076,'NASA' from dual union all
select 26077,'RVSN' from dual union all
select 26078,'NASA' from dual union all
select 26079,'USAF' from dual union all
select 2608,'NASA' from dual union all
select 26080,'DOD' from dual union all
select 26081,'GLOB' from dual union all
select 26082,'GLOB' from dual union all
select 26083,'GLOB' from dual union all
select 26084,'GLOB' from dual union all
select 26085,'BOE' from dual union all
select 26086,'NPOL' from dual union all
select 26087,'NPOL' from dual union all
select 26088,'NASA' from dual union all
select 26089,'PSN' from dual union all
select 2609,'NASA' from dual union all
select 26090,'KHRR' from dual union all
select 26091,'SCU' from dual union all
select 26092,'SCU' from dual union all
select 26093,'SCU' from dual union all
select 26094,'AMNA' from dual union all
select 26095,'SCC' from dual union all
select 26096,'AE' from dual;
insert into SATELLITE_ORG
select 26097,'STAN' from dual union all
select 26098,'GPKS' from dual union all
select 26098,'ISPUT' from dual union all
select 26099,'KHRR' from dual union all
select 261,'USAF' from dual union all
select 2610,'USAF' from dual union all
select 26100,'KHRR' from dual union all
select 26101,'KHRR' from dual union all
select 26102,'SANL' from dual union all
select 26103,'OSC' from dual union all
select 26104,'RVSNR' from dual union all
select 26105,'RVSNR' from dual union all
select 26106,'STSM' from dual union all
select 26107,'WORSP' from dual union all
select 26108,'ISRO' from dual union all
select 26109,'AE' from dual union all
select 2611,'USAF' from dual union all
select 26110,'AE' from dual union all
select 26111,'MO' from dual union all
select 26112,'CNSA' from dual union all
select 26113,'NASA' from dual union all
select 26114,'BOE' from dual union all
select 26115,'BOE' from dual union all
select 26116,'FKA' from dual union all
select 26117,'RVSNR' from dual union all
select 26118,'CNSA' from dual union all
select 26119,'CNSA' from dual union all
select 2612,'YUZH' from dual union all
select 26120,'CNSA' from dual union all
select 26121,'CNSA' from dual union all
select 26122,'CNSA' from dual union all
select 26123,'CNSA' from dual union all
select 26124,'CNSA' from dual union all
select 26125,'CNSA' from dual union all
select 26126,'CNSA' from dual union all
select 26127,'CNSA' from dual union all
select 26128,'CNSA' from dual union all
select 26129,'CNSA' from dual union all
select 2613,'USAF' from dual union all
select 26130,'CNSA' from dual union all
select 26131,'CNSA' from dual union all
select 26132,'CNSA' from dual union all
select 26133,'CNSA' from dual union all
select 26134,'CNSA' from dual union all
select 26135,'CNSA' from dual union all
select 26136,'CNSA' from dual union all
select 26137,'CNSA' from dual union all
select 26138,'CNSA' from dual union all
select 26139,'CNSA' from dual union all
select 2614,'USAF' from dual union all
select 26140,'CNSA' from dual union all
select 26141,'CNSA' from dual union all
select 26142,'CNSA' from dual union all
select 26143,'CNSA' from dual union all
select 26144,'CNSA' from dual union all
select 26145,'CNSA' from dual union all
select 26146,'USAF' from dual union all
select 26147,'USAF' from dual union all
select 26148,'CNSA' from dual union all
select 26149,'CNSA' from dual union all
select 2615,'RVSN' from dual union all
select 26150,'CNSA' from dual union all
select 26151,'CNSA' from dual union all
select 26152,'CNSA' from dual union all
select 26153,'CNSA' from dual union all
select 26154,'CNSA' from dual union all
select 26155,'CNSA' from dual union all
select 26156,'CNSA' from dual union all
select 26157,'CNSA' from dual union all
select 26158,'CNSA' from dual union all
select 26159,'CNSA' from dual union all
select 2616,'YUZH' from dual union all
select 26160,'CNSA' from dual union all
select 26161,'CNSA' from dual union all
select 26162,'CNSA' from dual union all
select 26163,'CNSA' from dual union all
select 26164,'CNSA' from dual union all
select 26165,'CNSA' from dual union all
select 26166,'CNSA' from dual union all
select 26167,'CNSA' from dual union all
select 26168,'CNSA' from dual union all
select 26169,'CNSA' from dual union all
select 2617,'YUZH' from dual union all
select 26170,'CNSA' from dual union all
select 26171,'CNSA' from dual union all
select 26172,'CNSA' from dual union all
select 26173,'CNSA' from dual union all
select 26174,'CNSA' from dual union all
select 26175,'CNSA' from dual union all
select 26176,'CNSA' from dual union all
select 26177,'CNSA' from dual union all
select 26178,'CNSA' from dual union all
select 26179,'CNSA' from dual union all
select 2618,'NRO' from dual union all
select 26180,'CNSA' from dual union all
select 26181,'CNSA' from dual union all
select 26182,'CNSA' from dual union all
select 26183,'CNSA' from dual union all
select 26184,'CNSA' from dual union all
select 26185,'CNSA' from dual;
insert into SATELLITE_ORG
select 26186,'CNSA' from dual union all
select 26187,'CNSA' from dual union all
select 26188,'CNSA' from dual union all
select 26189,'CNSA' from dual union all
select 2619,'NASA' from dual union all
select 26190,'CNSA' from dual union all
select 26191,'CNSA' from dual union all
select 26192,'CNSA' from dual union all
select 26193,'CNSA' from dual union all
select 26194,'CNSA' from dual union all
select 26195,'CNSA' from dual union all
select 26196,'CNSA' from dual union all
select 26197,'CNSA' from dual union all
select 26198,'CNSA' from dual union all
select 26199,'CNSA' from dual union all
select 262,'USAF' from dual union all
select 2620,'USN' from dual union all
select 26200,'CNSA' from dual union all
select 26201,'CNSA' from dual union all
select 26202,'CNSA' from dual union all
select 26203,'CNSA' from dual union all
select 26204,'CNSA' from dual union all
select 26205,'CNSA' from dual union all
select 26206,'CNSA' from dual union all
select 26207,'CNSA' from dual union all
select 26208,'CNSA' from dual union all
select 26209,'CNSA' from dual union all
select 2621,'USAF' from dual union all
select 26210,'CNSA' from dual union all
select 26211,'CNSA' from dual union all
select 26212,'CNSA' from dual union all
select 26213,'CNSA' from dual union all
select 26214,'CNSA' from dual union all
select 26215,'CNSA' from dual union all
select 26216,'CNSA' from dual union all
select 26217,'CNSA' from dual union all
select 26218,'CNSA' from dual union all
select 26219,'CNSA' from dual union all
select 2622,'USAF' from dual union all
select 26220,'CNSA' from dual union all
select 26221,'CNSA' from dual union all
select 26222,'CNSA' from dual union all
select 26223,'CNSA' from dual union all
select 26224,'CNSA' from dual union all
select 26225,'CNSA' from dual union all
select 26226,'CNSA' from dual union all
select 26227,'CNSA' from dual union all
select 26228,'CNSA' from dual union all
select 26229,'CNSA' from dual union all
select 2623,'NASA' from dual union all
select 26230,'CNSA' from dual union all
select 26231,'CNSA' from dual union all
select 26232,'CNSA' from dual union all
select 26233,'CNSA' from dual union all
select 26234,'CNSA' from dual union all
select 26235,'CNSA' from dual union all
select 26236,'CNSA' from dual union all
select 26237,'CNSA' from dual union all
select 26238,'CNSA' from dual union all
select 26239,'CNSA' from dual union all
select 2624,'MO' from dual union all
select 26240,'CNSA' from dual union all
select 26241,'CNSA' from dual union all
select 26242,'CNSA' from dual union all
select 26243,'EUTEL' from dual union all
select 26244,'KHRR' from dual union all
select 26245,'KHRR' from dual union all
select 26246,'KHRR' from dual union all
select 26247,'CNSA' from dual union all
select 26248,'CNSA' from dual union all
select 26249,'CNSA' from dual union all
select 2625,'RVSN' from dual union all
select 26250,'CNSA' from dual union all
select 26251,'CNSA' from dual union all
select 26252,'CNSA' from dual union all
select 26253,'CNSA' from dual union all
select 26254,'CNSA' from dual union all
select 26255,'CNSA' from dual union all
select 26256,'CNSA' from dual union all
select 26257,'CNSA' from dual union all
select 26258,'CNSA' from dual union all
select 26259,'CNSA' from dual union all
select 2626,'IKI' from dual union all
select 2626,'NPOL' from dual union all
select 26260,'CNSA' from dual union all
select 26261,'CNSA' from dual union all
select 26262,'CNSA' from dual union all
select 26263,'CNSA' from dual union all
select 26264,'CNSA' from dual union all
select 26265,'CNSA' from dual union all
select 26266,'CNSA' from dual union all
select 26267,'CNSA' from dual union all
select 26268,'CNSA' from dual union all
select 26269,'CNSA' from dual union all
select 2627,'YUZH' from dual union all
select 26270,'CNSA' from dual union all
select 26271,'CNSA' from dual union all
select 26272,'CNSA' from dual union all
select 26273,'CNSA' from dual union all
select 26274,'CNSA' from dual;
insert into SATELLITE_ORG
select 26275,'CNSA' from dual union all
select 26276,'CNSA' from dual union all
select 26277,'CNSA' from dual union all
select 26278,'CNSA' from dual union all
select 26279,'CNSA' from dual union all
select 2628,'RVSN' from dual union all
select 26280,'CNSA' from dual union all
select 26281,'CNSA' from dual union all
select 26282,'CNSA' from dual union all
select 26283,'CNSA' from dual union all
select 26284,'CNSA' from dual union all
select 26285,'CNSA' from dual union all
select 26286,'CNSA' from dual union all
select 26287,'CNSA' from dual union all
select 26288,'CNSA' from dual union all
select 26289,'CNSA' from dual union all
select 2629,'RVSN' from dual union all
select 26290,'CNSA' from dual union all
select 26291,'CNSA' from dual union all
select 26292,'CNSA' from dual union all
select 26293,'CNSA' from dual union all
select 26294,'CNSA' from dual union all
select 26295,'CNSA' from dual union all
select 26296,'CNSA' from dual union all
select 26297,'CNSA' from dual union all
select 26298,'PAN' from dual union all
select 26299,'AE' from dual union all
select 263,'USAF' from dual union all
select 2630,'RVSN' from dual union all
select 26300,'CNSA' from dual union all
select 26301,'FKA' from dual union all
select 26302,'RVSNR' from dual union all
select 26303,'NASA' from dual union all
select 26304,'CNSA' from dual union all
select 26305,'CNSA' from dual union all
select 26306,'CNSA' from dual union all
select 26307,'CNSA' from dual union all
select 26308,'CNSA' from dual union all
select 26309,'CNSA' from dual union all
select 2631,'NASA' from dual union all
select 26310,'CNSA' from dual union all
select 26311,'CNSA' from dual union all
select 26312,'CNSA' from dual union all
select 26313,'CNSA' from dual union all
select 26314,'CNSA' from dual union all
select 26315,'CNSA' from dual union all
select 26316,'CNSA' from dual union all
select 26317,'CNSA' from dual union all
select 26318,'CNSA' from dual union all
select 26319,'CNSA' from dual union all
select 2632,'NASA' from dual union all
select 26320,'CNSA' from dual union all
select 26321,'CNSA' from dual union all
select 26322,'CNSA' from dual union all
select 26323,'CNSA' from dual union all
select 26324,'CNSA' from dual union all
select 26325,'CNSA' from dual union all
select 26326,'CNSA' from dual union all
select 26327,'CNSA' from dual union all
select 26328,'CNSA' from dual union all
select 26329,'CNSA' from dual union all
select 2633,'YUZH' from dual union all
select 26330,'CNSA' from dual union all
select 26331,'CNSA' from dual union all
select 26332,'CNSA' from dual union all
select 26333,'CNSA' from dual union all
select 26334,'CNSA' from dual union all
select 26335,'CNSA' from dual union all
select 26336,'CNSA' from dual union all
select 26337,'CNSA' from dual union all
select 26338,'CNSA' from dual union all
select 26339,'CNSA' from dual union all
select 2634,'NRO' from dual union all
select 26340,'CNSA' from dual union all
select 26341,'CNSA' from dual union all
select 26342,'CNSA' from dual union all
select 26343,'CNSA' from dual union all
select 26344,'CNSA' from dual union all
select 26345,'CNSA' from dual union all
select 26346,'CNSA' from dual union all
select 26347,'CNSA' from dual union all
select 26348,'CNSA' from dual union all
select 26349,'CNSA' from dual union all
select 2635,'USAF' from dual union all
select 26350,'CNSA' from dual union all
select 26351,'CNSA' from dual union all
select 26352,'NOAA' from dual union all
select 26353,'LM' from dual union all
select 26354,'RVSN' from dual union all
select 26355,'RVSNR' from dual union all
select 26356,'USAF' from dual union all
select 26357,'USAF' from dual union all
select 26358,'USAF' from dual union all
select 26359,'USAF' from dual union all
select 2636,'USAF' from dual union all
select 26360,'USAF' from dual union all
select 26361,'BOE' from dual union all
select 26362,'BOE' from dual union all
select 26363,'FKA' from dual union all
select 26364,'FKA' from dual;
insert into SATELLITE_ORG
select 26365,'EUROK' from dual union all
select 26366,'EUROK' from dual union all
select 26367,'EUROK' from dual union all
select 26368,'NASA' from dual union all
select 26369,'EUTEL' from dual union all
select 2637,'RVSN' from dual union all
select 26370,'LM' from dual union all
select 26371,'AE' from dual union all
select 26372,'GPKS' from dual union all
select 26373,'KHRR' from dual union all
select 26374,'USAF' from dual union all
select 26375,'OSC' from dual union all
select 26376,'USAF' from dual union all
select 26377,'KHRR' from dual union all
select 26378,'GPKS' from dual union all
select 26378,'ISPUT' from dual union all
select 26379,'KHRR' from dual union all
select 2638,'USAF' from dual union all
select 26380,'KHRR' from dual union all
select 26381,'KHRR' from dual union all
select 26382,'CMA' from dual union all
select 26383,'CASC' from dual union all
select 26384,'RVSN' from dual union all
select 26384,'VMFR' from dual union all
select 26385,'TSHUA' from dual union all
select 26386,'SURREY' from dual union all
select 26387,'RVSNR' from dual union all
select 26388,'NASA' from dual union all
select 26389,'LM' from dual union all
select 2639,'INTEL' from dual union all
select 26390,'SIR' from dual union all
select 26391,'KHRR' from dual union all
select 26392,'KHRR' from dual union all
select 26393,'KHRR' from dual union all
select 26394,'RVSN' from dual union all
select 26395,'KHRR' from dual union all
select 26396,'KHRR' from dual union all
select 26397,'KHRR' from dual union all
select 26398,'RVSNR' from dual union all
select 26399,'RVSNR' from dual union all
select 264,'USAF' from dual union all
select 2640,'NASA' from dual union all
select 26400,'FKA' from dual union all
select 26401,'KHRR' from dual union all
select 26402,'ECHO' from dual union all
select 26403,'LM' from dual union all
select 26404,'ASI' from dual union all
select 26405,'GFZ' from dual union all
select 26406,'RVSNR' from dual union all
select 26407,'USAF' from dual union all
select 26408,'BOE' from dual union all
select 26409,'BOE' from dual union all
select 2641,'NASA' from dual union all
select 26410,'ESA' from dual union all
select 26411,'ESA' from dual union all
select 26412,'NPOL' from dual union all
select 26413,'AE' from dual union all
select 26414,'USAF' from dual union all
select 26415,'OSC' from dual union all
select 26416,'CNSA' from dual union all
select 26417,'CNSA' from dual union all
select 26418,'CNSA' from dual union all
select 26419,'CNSA' from dual union all
select 2642,'CIA' from dual union all
select 2642,'NRO' from dual union all
select 26420,'CNSA' from dual union all
select 26421,'CNSA' from dual union all
select 26422,'CNSA' from dual union all
select 26423,'CNSA' from dual union all
select 26424,'CNSA' from dual union all
select 26425,'CNSA' from dual union all
select 26426,'CNSA' from dual union all
select 26427,'CNSA' from dual union all
select 26428,'CNSA' from dual union all
select 26429,'CNSA' from dual union all
select 2643,'NASA' from dual union all
select 26430,'CNSA' from dual union all
select 26431,'CNSA' from dual union all
select 26432,'CNSA' from dual union all
select 26433,'CNSA' from dual union all
select 26434,'CNSA' from dual union all
select 26435,'CNSA' from dual union all
select 26436,'CNSA' from dual union all
select 26437,'CNSA' from dual union all
select 26438,'CNSA' from dual union all
select 26439,'CNSA' from dual union all
select 2644,'USN' from dual union all
select 26440,'CNSA' from dual union all
select 26441,'CNSA' from dual union all
select 26442,'CNSA' from dual union all
select 26443,'CNSA' from dual union all
select 26444,'CNSA' from dual union all
select 26445,'CNSA' from dual union all
select 26446,'CNSA' from dual union all
select 26447,'CNSA' from dual union all
select 26448,'CNSA' from dual union all
select 26449,'CNSA' from dual union all
select 2645,'USAF' from dual union all
select 26450,'CNSA' from dual union all
select 26451,'PAN' from dual;
insert into SATELLITE_ORG
select 26452,'SEALC' from dual union all
select 26453,'CNSA' from dual union all
select 26454,'CNSA' from dual union all
select 26455,'CNSA' from dual union all
select 26456,'CNSA' from dual union all
select 26457,'CNSA' from dual union all
select 26458,'CNSA' from dual union all
select 26459,'CNSA' from dual union all
select 2646,'MO' from dual union all
select 26460,'CASC' from dual union all
select 26461,'FKA' from dual union all
select 26462,'RVSNR' from dual union all
select 26463,'ESA' from dual union all
select 26464,'ESA' from dual union all
select 26465,'NPOL' from dual union all
select 26466,'AE' from dual union all
select 26467,'MO' from dual union all
select 26468,'FKA' from dual union all
select 26469,'TELB' from dual union all
select 2647,'MO' from dual union all
select 26470,'ERTU' from dual union all
select 26471,'AE' from dual union all
select 26472,'KHRR' from dual union all
select 26473,'NROC' from dual union all
select 26474,'USAF' from dual union all
select 26475,'BOE' from dual union all
select 26476,'BOE' from dual union all
select 26477,'RVSN' from dual union all
select 26478,'KHRR' from dual union all
select 26479,'KHRR' from dual union all
select 2648,'RVSN' from dual union all
select 26480,'KHRR' from dual union all
select 26481,'PLA' from dual union all
select 26482,'CNSA' from dual union all
select 26483,'SIR' from dual union all
select 26484,'KHRR' from dual union all
select 26485,'KHRR' from dual union all
select 26486,'KHRR' from dual union all
select 26487,'EUTEL' from dual union all
select 26488,'AE' from dual union all
select 26489,'NASA' from dual union all
select 2649,'USAF' from dual union all
select 26490,'AE' from dual union all
select 26491,'AE' from dual union all
select 26492,'NASA' from dual union all
select 26493,'NASA' from dual union all
select 26494,'SES' from dual union all
select 26495,'AMCG' from dual union all
select 26496,'AE' from dual union all
select 26497,'AE' from dual union all
select 26498,'MO' from dual union all
select 26499,'RVSNR' from dual union all
select 265,'USAF' from dual union all
select 2650,'USAF' from dual union all
select 26500,'RVSNR' from dual union all
select 26501,'RVSNR' from dual union all
select 26502,'RVSNR' from dual union all
select 26503,'VMF' from dual union all
select 26504,'VMF' from dual union all
select 26505,'VMF' from dual union all
select 26506,'VMF' from dual union all
select 26507,'VMF' from dual union all
select 26508,'VMF' from dual union all
select 26509,'VMF' from dual union all
select 2651,'USAF' from dual union all
select 26510,'VMF' from dual union all
select 26511,'VMF' from dual union all
select 26512,'VMF' from dual union all
select 26513,'VMF' from dual union all
select 26514,'VMF' from dual union all
select 26515,'VMF' from dual union all
select 26516,'VMF' from dual union all
select 26517,'VMF' from dual union all
select 26518,'VMF' from dual union all
select 26519,'VMF' from dual union all
select 2652,'USAF' from dual union all
select 26520,'VMF' from dual union all
select 26521,'VMF' from dual union all
select 26522,'VMF' from dual union all
select 26523,'VMF' from dual union all
select 26524,'USAF' from dual union all
select 26525,'USAF' from dual union all
select 26526,'USAF' from dual union all
select 26527,'USAF' from dual union all
select 26528,'USAF' from dual union all
select 26529,'USAF' from dual union all
select 2653,'USAF' from dual union all
select 26530,'USAF' from dual union all
select 26531,'USN' from dual union all
select 26532,'ESA' from dual union all
select 26533,'ESA' from dual union all
select 26534,'ESA' from dual union all
select 26535,'ESA' from dual union all
select 26536,'NOAA' from dual union all
select 26537,'VMF' from dual union all
select 26538,'RVSN' from dual union all
select 26539,'RVSN' from dual union all
select 2654,'USAF' from dual union all
select 26540,'RVSN' from dual union all
select 26541,'RVSN' from dual;
insert into SATELLITE_ORG
select 26542,'RVSN' from dual union all
select 26543,'RVSN' from dual union all
select 26544,'RVSN' from dual union all
select 26545,'BKSA' from dual union all
select 26546,'MEGS' from dual union all
select 26547,'ROSAP' from dual union all
select 26548,'KACST' from dual union all
select 26549,'KACST' from dual union all
select 2655,'USAF' from dual union all
select 26550,'KTRAS' from dual union all
select 26551,'KTRAS' from dual union all
select 26552,'VTSR' from dual union all
select 26553,'RVSNR' from dual union all
select 26554,'AMCG' from dual union all
select 26554,'GECAP' from dual union all
select 26555,'KHRR' from dual union all
select 26556,'KHRR' from dual union all
select 26557,'KHRR' from dual union all
select 26558,'NASA' from dual union all
select 26559,'SCC' from dual union all
select 2656,'RVSN' from dual union all
select 26560,'AE' from dual union all
select 26561,'NASA' from dual union all
select 26562,'OSC' from dual union all
select 26563,'NASA' from dual union all
select 26564,'FKA' from dual union all
select 26564,'RVSN' from dual union all
select 26565,'FKA' from dual union all
select 26565,'RVSN' from dual union all
select 26566,'FKA' from dual union all
select 26566,'RVSN' from dual union all
select 26567,'KHRR' from dual union all
select 26568,'KHRR' from dual union all
select 26569,'KHRR' from dual union all
select 2657,'ESSA' from dual union all
select 26570,'FKA' from dual union all
select 26571,'RVSNR' from dual union all
select 26572,'NASA' from dual union all
select 26573,'RVSNR' from dual union all
select 26574,'RVSNR' from dual union all
select 26575,'DOD' from dual union all
select 26575,'USAF' from dual union all
select 26576,'LM' from dual union all
select 26577,'USAF' from dual union all
select 26578,'THUR' from dual union all
select 26579,'SEALP' from dual union all
select 2658,'RVSN' from dual union all
select 26580,'AMCG' from dual union all
select 26581,'KHRR' from dual union all
select 26582,'KHRR' from dual union all
select 26583,'KHRR' from dual union all
select 26584,'PVO' from dual union all
select 26585,'PVO' from dual union all
select 26586,'PVO' from dual union all
select 26587,'PVO' from dual union all
select 26588,'PVO' from dual union all
select 26589,'PVO' from dual union all
select 2659,'RVSN' from dual union all
select 26590,'EUPST' from dual union all
select 26591,'AE' from dual union all
select 26592,'PVO' from dual union all
select 26593,'PVO' from dual union all
select 26594,'PVO' from dual union all
select 26595,'PVO' from dual union all
select 26596,'PVO' from dual union all
select 26597,'PVO' from dual union all
select 26598,'PVO' from dual union all
select 26599,'CNSA' from dual union all
select 266,'OKB586' from dual union all
select 2660,'USAF' from dual union all
select 26600,'CASC' from dual union all
select 26601,'RVSN' from dual union all
select 26602,'ISRO' from dual union all
select 26603,'FKA' from dual union all
select 26604,'RVSNR' from dual union all
select 26605,'USAF' from dual union all
select 26606,'BOE' from dual union all
select 26607,'BOE' from dual union all
select 26608,'PAN' from dual union all
select 26609,'AM' from dual union all
select 2661,'NASA' from dual union all
select 26610,'DERA' from dual union all
select 26611,'DERA' from dual union all
select 26612,'AE' from dual union all
select 26613,'AE' from dual union all
select 26614,'NASA' from dual union all
select 26615,'FKA' from dual union all
select 26616,'RVSNR' from dual union all
select 26617,'EWAT' from dual union all
select 26618,'NROC' from dual union all
select 26619,'NASA' from dual union all
select 2662,'CIA' from dual union all
select 2662,'NRO' from dual union all
select 26620,'CONAB' from dual union all
select 26621,'IRF' from dual union all
select 26622,'BOE' from dual union all
select 26623,'BOE' from dual union all
select 26624,'TCAN' from dual union all
select 26625,'AE' from dual union all
select 26626,'SIR' from dual;
insert into SATELLITE_ORG
select 26627,'KHRR' from dual union all
select 26628,'KHRR' from dual union all
select 26629,'KHRR' from dual union all
select 2663,'USAF' from dual union all
select 26630,'NASA' from dual union all
select 26631,'IMAG' from dual union all
select 26632,'PU' from dual union all
select 26633,'MO' from dual union all
select 26634,'VMF' from dual union all
select 26635,'NROC' from dual union all
select 26636,'LM' from dual union all
select 26637,'USAF' from dual union all
select 26638,'SES' from dual union all
select 26639,'AMCG' from dual union all
select 2664,'NRO' from dual union all
select 26640,'NASDA' from dual union all
select 26641,'AE' from dual union all
select 26642,'AE' from dual union all
select 26643,'CNSA' from dual union all
select 26644,'CASC' from dual union all
select 26645,'VMF' from dual union all
select 26646,'VMF' from dual union all
select 26647,'VMF' from dual union all
select 26648,'VMF' from dual union all
select 26649,'CNSA' from dual union all
select 2665,'USAF' from dual union all
select 26650,'CNSA' from dual union all
select 26651,'CNSA' from dual union all
select 26652,'CNSA' from dual union all
select 26653,'CNSA' from dual union all
select 26654,'CNSA' from dual union all
select 26655,'CNSA' from dual union all
select 26656,'CNSA' from dual union all
select 26657,'CNSA' from dual union all
select 26658,'CNSA' from dual union all
select 26659,'CNSA' from dual union all
select 2666,'NASA' from dual union all
select 26660,'CNSA' from dual union all
select 26661,'CNSA' from dual union all
select 26662,'CNSA' from dual union all
select 26663,'CNSA' from dual union all
select 26664,'PLA' from dual union all
select 26665,'CASC' from dual union all
select 26666,'EURSA' from dual union all
select 26667,'AE' from dual union all
select 26668,'FKA' from dual union all
select 26669,'NASA' from dual union all
select 2667,'TSKBEM' from dual union all
select 26670,'NASA' from dual union all
select 26671,'USAF' from dual union all
select 26672,'NASA' from dual union all
select 26673,'NASA' from dual union all
select 26674,'RVSN' from dual union all
select 26675,'PVO' from dual union all
select 26676,'PVO' from dual union all
select 26677,'PVO' from dual union all
select 26678,'PVO' from dual union all
select 26679,'PVO' from dual union all
select 2668,'RVSN' from dual union all
select 26680,'RVSN' from dual union all
select 26681,'AE' from dual union all
select 26682,'RVSNR' from dual union all
select 26683,'AE' from dual union all
select 26684,'AE' from dual union all
select 26685,'AE' from dual union all
select 26686,'AE' from dual union all
select 26687,'PLA' from dual union all
select 26688,'FKA' from dual union all
select 26689,'RVSNR' from dual union all
select 2669,'USAF' from dual union all
select 26690,'USAF' from dual union all
select 26691,'BOE' from dual union all
select 26692,'BOE' from dual union all
select 26693,'CNSA' from dual union all
select 26694,'MDDI' from dual union all
select 26695,'MOD' from dual union all
select 26696,'AE' from dual union all
select 26697,'AE' from dual union all
select 26698,'NASA' from dual union all
select 26699,'CNSA' from dual union all
select 267,'RVSN' from dual union all
select 2670,'MO' from dual union all
select 26700,'NASA' from dual union all
select 26701,'NASA' from dual union all
select 26702,'SSC' from dual union all
select 26703,'PU' from dual union all
select 26704,'PU' from dual union all
select 26705,'PU' from dual union all
select 26706,'PU' from dual union all
select 26707,'PU' from dual union all
select 26708,'PU' from dual union all
select 26709,'PU' from dual union all
select 2671,'CNES' from dual union all
select 26710,'RVSNR' from dual union all
select 26711,'RVSNR' from dual union all
select 26712,'PU' from dual union all
select 26713,'FKA' from dual union all
select 26714,'RVSNR' from dual union all
select 26715,'USAF' from dual union all
select 26716,'USAF' from dual;
insert into SATELLITE_ORG
select 26717,'AE' from dual union all
select 26718,'NASA' from dual union all
select 26719,'EUTEL' from dual union all
select 2672,'RVSN' from dual union all
select 26720,'BSAT' from dual union all
select 26721,'AE' from dual union all
select 26722,'AE' from dual union all
select 26723,'NASA' from dual union all
select 26724,'XMR' from dual union all
select 26725,'SEALP' from dual union all
select 26726,'AE' from dual union all
select 26727,'AE' from dual union all
select 26728,'AE' from dual union all
select 26729,'AE' from dual union all
select 2673,'CNES' from dual union all
select 26730,'SIMG' from dual union all
select 26731,'USAF' from dual union all
select 26732,'USAF' from dual union all
select 26733,'USAF' from dual union all
select 26734,'JPL' from dual union all
select 26735,'BOE' from dual union all
select 26736,'GPKS' from dual union all
select 26737,'KHRR' from dual union all
select 26738,'KHRR' from dual union all
select 26739,'BOE' from dual union all
select 2674,'CNES' from dual union all
select 26740,'OSC' from dual union all
select 26741,'AE' from dual union all
select 26742,'RVSNR' from dual union all
select 26743,'KVR' from dual union all
select 26744,'ISASS' from dual union all
select 26745,'ISRO' from dual union all
select 26746,'ISRO' from dual union all
select 26747,'NASA' from dual union all
select 26748,'RVSN' from dual union all
select 26749,'FKA' from dual union all
select 2675,'CNES' from dual union all
select 26750,'RVSNR' from dual union all
select 26751,'CNSA' from dual union all
select 26752,'CNSA' from dual union all
select 26753,'CNSA' from dual union all
select 26754,'CNSA' from dual union all
select 26755,'CNSA' from dual union all
select 26756,'CNSA' from dual union all
select 26757,'CNSA' from dual union all
select 26758,'CNSA' from dual union all
select 26759,'CNSA' from dual union all
select 2676,'CNES' from dual union all
select 26760,'RVSN' from dual union all
select 26761,'XMR' from dual union all
select 26762,'SEALP' from dual union all
select 26763,'USAF' from dual union all
select 26764,'USAF' from dual union all
select 26765,'USAF' from dual union all
select 26766,'PAN' from dual union all
select 26767,'KHRR' from dual union all
select 26768,'KHRR' from dual union all
select 26769,'KHRR' from dual union all
select 2677,'CNES' from dual union all
select 26770,'NROC' from dual union all
select 26771,'BOE' from dual union all
select 26772,'BOE' from dual union all
select 26773,'FKA' from dual union all
select 26774,'RVSNR' from dual union all
select 26775,'RVSN' from dual union all
select 26776,'RVSN' from dual union all
select 26777,'USAF' from dual union all
select 26778,'USAF' from dual union all
select 26779,'USAF' from dual union all
select 2678,'YUZH' from dual union all
select 26780,'USAF' from dual union all
select 26781,'USAF' from dual union all
select 26782,'USAF' from dual union all
select 26783,'USAF' from dual union all
select 26784,'RVSN' from dual union all
select 26785,'CNSA' from dual union all
select 26786,'MO' from dual union all
select 26787,'AE' from dual union all
select 26788,'MAI' from dual union all
select 26789,'BOE' from dual union all
select 2679,'RVSN' from dual union all
select 26790,'AE' from dual union all
select 26791,'MDC' from dual union all
select 26792,'MDC' from dual union all
select 26793,'MDC' from dual union all
select 26794,'AE' from dual union all
select 26795,'MO' from dual union all
select 26796,'KVR' from dual union all
select 26797,'KVR' from dual union all
select 26798,'KVR' from dual union all
select 26799,'AE' from dual union all
select 268,'USAF' from dual union all
select 2680,'CNES' from dual union all
select 26800,'AE' from dual union all
select 26801,'RVSNR' from dual union all
select 26802,'RVSNR' from dual union all
select 26803,'RVSNR' from dual union all
select 26804,'RVSNR' from dual union all
select 26805,'AE' from dual union all
select 26806,'AE' from dual;
insert into SATELLITE_ORG
select 26807,'AE' from dual union all
select 26808,'AE' from dual union all
select 26809,'AE' from dual union all
select 2681,'CNES' from dual union all
select 26810,'AE' from dual union all
select 26811,'AE' from dual union all
select 26812,'AE' from dual union all
select 26813,'RVSN' from dual union all
select 26814,'RVSN' from dual union all
select 26815,'USN' from dual union all
select 26816,'NASA' from dual union all
select 26817,'USN' from dual union all
select 26818,'VMFR' from dual union all
select 26819,'KVR' from dual union all
select 2682,'CNES' from dual union all
select 26820,'USAF' from dual union all
select 26821,'NASA' from dual union all
select 26822,'NASA' from dual union all
select 26823,'NASA' from dual union all
select 26824,'INTEL' from dual union all
select 26825,'AE' from dual union all
select 26826,'NASA' from dual union all
select 26827,'NASA' from dual union all
select 26828,'NASA' from dual union all
select 26829,'NASA' from dual union all
select 2683,'CNES' from dual union all
select 26830,'NASA' from dual union all
select 26831,'NASA' from dual union all
select 26832,'NASA' from dual union all
select 26833,'NASA' from dual union all
select 26834,'NASA' from dual union all
select 26835,'NASA' from dual union all
select 26836,'NASA' from dual union all
select 26837,'NASA' from dual union all
select 26838,'NASA' from dual union all
select 26839,'NASA' from dual union all
select 2684,'CNES' from dual union all
select 26840,'NASA' from dual union all
select 26841,'NASA' from dual union all
select 26842,'NASA' from dual union all
select 26843,'NASA' from dual union all
select 26844,'NASA' from dual union all
select 26845,'NASA' from dual union all
select 26846,'NASA' from dual union all
select 26847,'NASA' from dual union all
select 26848,'RVSN' from dual union all
select 26849,'RVSNR' from dual union all
select 2685,'CNES' from dual union all
select 26850,'AE' from dual union all
select 26851,'MDC' from dual union all
select 26852,'MDC' from dual union all
select 26853,'SES' from dual union all
select 26854,'KHRR' from dual union all
select 26855,'KHRR' from dual union all
select 26856,'KHRR' from dual union all
select 26857,'ICOH' from dual union all
select 26858,'LM' from dual union all
select 26859,'NASA' from dual union all
select 2686,'CIA' from dual union all
select 2686,'NRO' from dual union all
select 26860,'BOE' from dual union all
select 26861,'NASA' from dual union all
select 26862,'NASA' from dual union all
select 26863,'ESA' from dual union all
select 26864,'BSAT' from dual union all
select 26864,'OSCD' from dual union all
select 26865,'AE' from dual union all
select 26866,'AE' from dual union all
select 26867,'KS' from dual union all
select 26867,'KVR' from dual union all
select 26868,'KVR' from dual union all
select 26869,'KVR' from dual union all
select 2687,'NRO' from dual union all
select 26870,'KVR' from dual union all
select 26871,'NOAA' from dual union all
select 26872,'LM' from dual union all
select 26873,'IKI' from dual union all
select 26873,'YUZH' from dual union all
select 26874,'KVR' from dual union all
select 26875,'YUZH' from dual union all
select 26876,'YUZH' from dual union all
select 26877,'YUZH' from dual union all
select 26878,'YUZH' from dual union all
select 26879,'KVR' from dual union all
select 2688,'CNES' from dual union all
select 26880,'USAF' from dual union all
select 26881,'USAF' from dual union all
select 26882,'USAF' from dual union all
select 26883,'USAF' from dual union all
select 26884,'JPL' from dual union all
select 26884,'NASA' from dual union all
select 26885,'BOE' from dual union all
select 26886,'BOE' from dual union all
select 26887,'USAF' from dual union all
select 26888,'NASA' from dual union all
select 26889,'NASA' from dual union all
select 2689,'CNES' from dual union all
select 26890,'FKA' from dual union all
select 26891,'RVSNR' from dual union all
select 26892,'PVO' from dual;
insert into SATELLITE_ORG
select 26893,'KHRR' from dual union all
select 26894,'KHRR' from dual union all
select 26895,'KHRR' from dual union all
select 26896,'RVSNR' from dual union all
select 26897,'RVSNR' from dual union all
select 26898,'NASDA' from dual union all
select 26899,'NASDA' from dual union all
select 269,'OKB1' from dual union all
select 2690,'CNES' from dual union all
select 26900,'INTELB' from dual union all
select 26901,'AE' from dual union all
select 26902,'KVR' from dual union all
select 26903,'KVR' from dual union all
select 26904,'DOD' from dual union all
select 26905,'NROC' from dual union all
select 26905,'USN' from dual union all
select 26906,'LM' from dual union all
select 26907,'NROC' from dual union all
select 26907,'USN' from dual union all
select 26908,'FKA' from dual union all
select 26909,'FKA' from dual union all
select 2691,'CNES' from dual union all
select 26910,'USAF' from dual union all
select 26911,'VMF' from dual union all
select 26912,'USN' from dual union all
select 26913,'USAF' from dual union all
select 26914,'AE' from dual union all
select 26915,'RVSN' from dual union all
select 26916,'USAF' from dual union all
select 26917,'PVO' from dual union all
select 26918,'PVO' from dual union all
select 26919,'PVO' from dual union all
select 2692,'CNES' from dual union all
select 26920,'KVR' from dual union all
select 26921,'USAF' from dual union all
select 26922,'AE' from dual union all
select 26923,'USN' from dual union all
select 26924,'AE' from dual union all
select 26925,'AE' from dual union all
select 26926,'USAF' from dual union all
select 26927,'EUTSA' from dual union all
select 26928,'AE' from dual union all
select 26929,'NASA' from dual union all
select 2693,'MO' from dual union all
select 26930,'USAF' from dual union all
select 26931,'USAF' from dual union all
select 26931,'USN' from dual union all
select 26932,'USAF' from dual union all
select 26933,'LM' from dual union all
select 26934,'NROC' from dual union all
select 26935,'USAF' from dual union all
select 26936,'KVR' from dual union all
select 26937,'KHRR' from dual union all
select 26938,'KHRR' from dual union all
select 26939,'KHRR' from dual union all
select 2694,'RVSN' from dual union all
select 26940,'RVSNR' from dual union all
select 26941,'RVSNR' from dual union all
select 26942,'RVSNR' from dual union all
select 26943,'FKA' from dual union all
select 26944,'FKA' from dual union all
select 26945,'FKA' from dual union all
select 26946,'FKA' from dual union all
select 26947,'RVSNR' from dual union all
select 26948,'NROC' from dual union all
select 26949,'LM' from dual union all
select 2695,'METP' from dual union all
select 26950,'FKA' from dual union all
select 26951,'FKA' from dual union all
select 26952,'FKA' from dual union all
select 26953,'EWAT' from dual union all
select 26954,'BOE' from dual union all
select 26955,'FKA' from dual union all
select 26956,'RVSNR' from dual union all
select 26957,'ISRO' from dual union all
select 26958,'ESA' from dual union all
select 26959,'DLR' from dual union all
select 2696,'RVSN' from dual union all
select 26960,'ISRO' from dual union all
select 26961,'FKA' from dual union all
select 26962,'NASA' from dual union all
select 26963,'JPL' from dual union all
select 26964,'PVO' from dual union all
select 26965,'MDC' from dual union all
select 26966,'KVR' from dual union all
select 26967,'MDC' from dual union all
select 26968,'RVSNR' from dual union all
select 26969,'MDC' from dual union all
select 2697,'YUZH' from dual union all
select 26970,'KS' from dual union all
select 26970,'KVR' from dual union all
select 26971,'KVR' from dual union all
select 26972,'KVR' from dual union all
select 26973,'KVR' from dual union all
select 26974,'RVSN' from dual union all
select 26975,'RVSN' from dual union all
select 26976,'PVO' from dual union all
select 26977,'PVO' from dual union all
select 26978,'PVO' from dual union all
select 26979,'RVSNR' from dual;
insert into SATELLITE_ORG
select 2698,'RVSN' from dual union all
select 26980,'RVSNR' from dual union all
select 26981,'AE' from dual union all
select 26982,'RVSN' from dual union all
select 26983,'FKA' from dual union all
select 26984,'RVSNR' from dual union all
select 26985,'DTV' from dual union all
select 26986,'AE' from dual union all
select 26987,'FKA' from dual union all
select 26987,'KVR' from dual union all
select 26988,'FKA' from dual union all
select 26988,'KVR' from dual union all
select 26989,'FKA' from dual union all
select 26989,'KVR' from dual union all
select 2699,'USAF' from dual union all
select 26990,'KHRR' from dual union all
select 26991,'KHRR' from dual union all
select 26992,'KHRR' from dual union all
select 26993,'RVSNR' from dual union all
select 26994,'RVSNR' from dual union all
select 26995,'NASA' from dual union all
select 26996,'NASA' from dual union all
select 26997,'CNES' from dual union all
select 26997,'JPL' from dual union all
select 26998,'NASA' from dual union all
select 26999,'BOE' from dual union all
select 27,'NASA' from dual union all
select 270,'RVSN' from dual union all
select 2700,'NASA' from dual union all
select 27000,'BOE' from dual union all
select 27001,'FKA' from dual union all
select 27001,'RGMS' from dual union all
select 27002,'IZM' from dual union all
select 27002,'MAK' from dual union all
select 27003,'SUPA' from dual union all
select 27004,'CRTS' from dual union all
select 27005,'NIIPP' from dual union all
select 27005,'USAF' from dual union all
select 27006,'KVR' from dual union all
select 27007,'KVR' from dual union all
select 27008,'CNSA' from dual union all
select 27009,'KVR' from dual union all
select 2701,'USN' from dual union all
select 27010,'KVR' from dual union all
select 27011,'KVR' from dual union all
select 27012,'KVR' from dual union all
select 27013,'KVR' from dual union all
select 27014,'KVR' from dual union all
select 27015,'KVR' from dual union all
select 27016,'KVR' from dual union all
select 27017,'KVR' from dual union all
select 27018,'KVR' from dual union all
select 27019,'KVR' from dual union all
select 2702,'USN' from dual union all
select 27020,'KVR' from dual union all
select 27021,'KVR' from dual union all
select 27022,'KVR' from dual union all
select 27023,'KVR' from dual union all
select 27024,'KVR' from dual union all
select 27025,'AE' from dual union all
select 27026,'AE' from dual union all
select 27027,'AE' from dual union all
select 27028,'AE' from dual union all
select 27029,'AE' from dual union all
select 2703,'NASA' from dual union all
select 27030,'AE' from dual union all
select 27031,'AE' from dual union all
select 27032,'AE' from dual union all
select 27033,'AE' from dual union all
select 27034,'AE' from dual union all
select 27035,'AE' from dual union all
select 27036,'AE' from dual union all
select 27037,'AE' from dual union all
select 27038,'AE' from dual union all
select 27039,'AE' from dual union all
select 2704,'NASA' from dual union all
select 27040,'AE' from dual union all
select 27041,'AE' from dual union all
select 27042,'AE' from dual union all
select 27043,'AE' from dual union all
select 27044,'AE' from dual union all
select 27045,'AE' from dual union all
select 27046,'AE' from dual union all
select 27047,'AE' from dual union all
select 27048,'AE' from dual union all
select 27049,'AE' from dual union all
select 2705,'TSKBEM' from dual union all
select 27050,'AE' from dual union all
select 27051,'NASA' from dual union all
select 27052,'NASA' from dual union all
select 27053,'VMFR' from dual union all
select 27054,'KVR' from dual union all
select 27055,'KVR' from dual union all
select 27056,'KVR' from dual union all
select 27057,'KVR' from dual union all
select 27058,'FKA' from dual union all
select 27059,'FKA' from dual union all
select 2706,'NASA' from dual union all
select 27060,'FKA' from dual union all
select 27061,'KVR' from dual;
insert into SATELLITE_ORG
select 27062,'ISRO' from dual union all
select 27063,'ISRO' from dual union all
select 27064,'ISRO' from dual union all
select 27065,'ISRO' from dual union all
select 27066,'ISRO' from dual union all
select 27067,'ISRO' from dual union all
select 27068,'ISRO' from dual union all
select 27069,'ISRO' from dual union all
select 2707,'NASA' from dual union all
select 27070,'ISRO' from dual union all
select 27071,'ISRO' from dual union all
select 27072,'ISRO' from dual union all
select 27073,'ISRO' from dual union all
select 27074,'ISRO' from dual union all
select 27075,'ISRO' from dual union all
select 27076,'ISRO' from dual union all
select 27077,'ISRO' from dual union all
select 27078,'ISRO' from dual union all
select 27079,'ISRO' from dual union all
select 2708,'YUZH' from dual union all
select 27080,'ISRO' from dual union all
select 27081,'ISRO' from dual union all
select 27082,'ISRO' from dual union all
select 27083,'ISRO' from dual union all
select 27084,'ISRO' from dual union all
select 27085,'ISRO' from dual union all
select 27086,'ISRO' from dual union all
select 27087,'ISRO' from dual union all
select 27088,'ISRO' from dual union all
select 27089,'ISRO' from dual union all
select 2709,'RVSN' from dual union all
select 27090,'ISRO' from dual union all
select 27091,'ISRO' from dual union all
select 27092,'ISRO' from dual union all
select 27093,'ISRO' from dual union all
select 27094,'ISRO' from dual union all
select 27095,'ISRO' from dual union all
select 27096,'ISRO' from dual union all
select 27097,'ISRO' from dual union all
select 27098,'ISRO' from dual union all
select 27099,'ISRO' from dual union all
select 271,'USAF' from dual union all
select 2710,'MO' from dual union all
select 27100,'ISRO' from dual union all
select 27101,'ISRO' from dual union all
select 27102,'ISRO' from dual union all
select 27103,'ISRO' from dual union all
select 27104,'ISRO' from dual union all
select 27105,'ISRO' from dual union all
select 27106,'ISRO' from dual union all
select 27107,'ISRO' from dual union all
select 27108,'ISRO' from dual union all
select 27109,'ISRO' from dual union all
select 2711,'RVSN' from dual union all
select 27110,'ISRO' from dual union all
select 27111,'ISRO' from dual union all
select 27112,'ISRO' from dual union all
select 27113,'ISRO' from dual union all
select 27114,'ISRO' from dual union all
select 27115,'ISRO' from dual union all
select 27116,'ISRO' from dual union all
select 27117,'ISRO' from dual union all
select 27118,'ISRO' from dual union all
select 27119,'ISRO' from dual union all
select 2712,'PVO' from dual union all
select 2712,'YUZH' from dual union all
select 27120,'ISRO' from dual union all
select 27121,'ISRO' from dual union all
select 27122,'ISRO' from dual union all
select 27123,'ISRO' from dual union all
select 27124,'ISRO' from dual union all
select 27125,'ISRO' from dual union all
select 27126,'ISRO' from dual union all
select 27127,'ISRO' from dual union all
select 27128,'ISRO' from dual union all
select 27129,'ISRO' from dual union all
select 2713,'RVSN' from dual union all
select 27130,'ISRO' from dual union all
select 27131,'ISRO' from dual union all
select 27132,'ISRO' from dual union all
select 27133,'ISRO' from dual union all
select 27134,'ISRO' from dual union all
select 27135,'ISRO' from dual union all
select 27136,'ISRO' from dual union all
select 27137,'ISRO' from dual union all
select 27138,'ISRO' from dual union all
select 27139,'ISRO' from dual union all
select 2714,'YUZH' from dual union all
select 27140,'ISRO' from dual union all
select 27141,'ISRO' from dual union all
select 27142,'ISRO' from dual union all
select 27143,'ISRO' from dual union all
select 27144,'ISRO' from dual union all
select 27145,'ISRO' from dual union all
select 27146,'ISRO' from dual union all
select 27147,'ISRO' from dual union all
select 27148,'ISRO' from dual union all
select 27149,'ISRO' from dual union all
select 2715,'MO' from dual union all
select 27150,'ISRO' from dual;
insert into SATELLITE_ORG
select 27151,'ISRO' from dual union all
select 27152,'ISRO' from dual union all
select 27153,'ISRO' from dual union all
select 27154,'ISRO' from dual union all
select 27155,'ISRO' from dual union all
select 27156,'ISRO' from dual union all
select 27157,'ISRO' from dual union all
select 27158,'ISRO' from dual union all
select 27159,'ISRO' from dual union all
select 2716,'RVSN' from dual union all
select 27160,'ISRO' from dual union all
select 27161,'ISRO' from dual union all
select 27162,'ISRO' from dual union all
select 27163,'ISRO' from dual union all
select 27164,'ISRO' from dual union all
select 27165,'ISRO' from dual union all
select 27166,'FKA' from dual union all
select 27167,'FKA' from dual union all
select 27168,'USAF' from dual union all
select 27169,'USAF' from dual union all
select 2717,'INTEL' from dual union all
select 27170,'ISRO' from dual union all
select 27171,'ISRO' from dual union all
select 27172,'ISRO' from dual union all
select 27173,'ISRO' from dual union all
select 27174,'ISRO' from dual union all
select 27175,'ISRO' from dual union all
select 27176,'ISRO' from dual union all
select 27177,'ISRO' from dual union all
select 27178,'ISRO' from dual union all
select 27179,'ISRO' from dual union all
select 2718,'RVSN' from dual union all
select 27180,'ISRO' from dual union all
select 27181,'ISRO' from dual union all
select 27182,'ISRO' from dual union all
select 27183,'ISRO' from dual union all
select 27184,'ISRO' from dual union all
select 27185,'ISRO' from dual union all
select 27186,'ISRO' from dual union all
select 27187,'ISRO' from dual union all
select 27188,'ISRO' from dual union all
select 27189,'ISRO' from dual union all
select 2719,'NASA' from dual union all
select 27190,'ISRO' from dual union all
select 27191,'ISRO' from dual union all
select 27192,'ISRO' from dual union all
select 27193,'ISRO' from dual union all
select 27194,'ISRO' from dual union all
select 27195,'ISRO' from dual union all
select 27196,'ISRO' from dual union all
select 27197,'ISRO' from dual union all
select 27198,'ISRO' from dual union all
select 27199,'ISRO' from dual union all
select 272,'USAF' from dual union all
select 2720,'OKB10' from dual union all
select 27200,'ISRO' from dual union all
select 27201,'ISRO' from dual union all
select 27202,'ISRO' from dual union all
select 27203,'ISRO' from dual union all
select 27204,'ISRO' from dual union all
select 27205,'ISRO' from dual union all
select 27206,'ISRO' from dual union all
select 27207,'ISRO' from dual union all
select 27208,'ISRO' from dual union all
select 27209,'ISRO' from dual union all
select 2721,'RVSN' from dual union all
select 27210,'ISRO' from dual union all
select 27211,'ISRO' from dual union all
select 27212,'ISRO' from dual union all
select 27213,'ISRO' from dual union all
select 27214,'ISRO' from dual union all
select 27215,'ISRO' from dual union all
select 27216,'ISRO' from dual union all
select 27217,'ISRO' from dual union all
select 27218,'ISRO' from dual union all
select 27219,'ISRO' from dual union all
select 2722,'PVO' from dual union all
select 2722,'YUZH' from dual union all
select 27220,'ISRO' from dual union all
select 27221,'ISRO' from dual union all
select 27222,'ISRO' from dual union all
select 27223,'ISRO' from dual union all
select 27224,'ISRO' from dual union all
select 27225,'ISRO' from dual union all
select 27226,'ISRO' from dual union all
select 27227,'ISRO' from dual union all
select 27228,'ISRO' from dual union all
select 27229,'ISRO' from dual union all
select 2723,'RVSN' from dual union all
select 27230,'ISRO' from dual union all
select 27231,'ISRO' from dual union all
select 27232,'ISRO' from dual union all
select 27233,'ISRO' from dual union all
select 27234,'ISRO' from dual union all
select 27235,'ISRO' from dual union all
select 27236,'ISRO' from dual union all
select 27237,'ISRO' from dual union all
select 27238,'ISRO' from dual union all
select 27239,'ISRO' from dual union all
select 2724,'YUZH' from dual;
insert into SATELLITE_ORG
select 27240,'ISRO' from dual union all
select 27241,'ISRO' from dual union all
select 27242,'ISRO' from dual union all
select 27243,'ISRO' from dual union all
select 27244,'ISRO' from dual union all
select 27245,'ISRO' from dual union all
select 27246,'ISRO' from dual union all
select 27247,'ISRO' from dual union all
select 27248,'ISRO' from dual union all
select 27249,'ISRO' from dual union all
select 2725,'YUZH' from dual union all
select 27250,'ISRO' from dual union all
select 27251,'ISRO' from dual union all
select 27252,'ISRO' from dual union all
select 27253,'ISRO' from dual union all
select 27254,'ISRO' from dual union all
select 27255,'ISRO' from dual union all
select 27256,'ISRO' from dual union all
select 27257,'ISRO' from dual union all
select 27258,'ISRO' from dual union all
select 27259,'ISRO' from dual union all
select 2726,'YUZH' from dual union all
select 27260,'ISRO' from dual union all
select 27261,'ISRO' from dual union all
select 27262,'ISRO' from dual union all
select 27263,'ISRO' from dual union all
select 27264,'ISRO' from dual union all
select 27265,'ISRO' from dual union all
select 27266,'ISRO' from dual union all
select 27267,'ISRO' from dual union all
select 27268,'ISRO' from dual union all
select 27269,'ISRO' from dual union all
select 2727,'YUZH' from dual union all
select 27270,'ISRO' from dual union all
select 27271,'ISRO' from dual union all
select 27272,'ISRO' from dual union all
select 27273,'ISRO' from dual union all
select 27274,'ISRO' from dual union all
select 27275,'ISRO' from dual union all
select 27276,'ISRO' from dual union all
select 27277,'ISRO' from dual union all
select 27278,'ISRO' from dual union all
select 27279,'ISRO' from dual union all
select 2728,'YUZH' from dual union all
select 27280,'ISRO' from dual union all
select 27281,'ISRO' from dual union all
select 27282,'ISRO' from dual union all
select 27283,'ISRO' from dual union all
select 27284,'ISRO' from dual union all
select 27285,'ISRO' from dual union all
select 27286,'ISRO' from dual union all
select 27287,'ISRO' from dual union all
select 27288,'ISRO' from dual union all
select 27289,'ISRO' from dual union all
select 2729,'YUZH' from dual union all
select 27290,'ISRO' from dual union all
select 27291,'ISRO' from dual union all
select 27292,'ISRO' from dual union all
select 27293,'ISRO' from dual union all
select 27294,'ISRO' from dual union all
select 27295,'ISRO' from dual union all
select 27296,'ISRO' from dual union all
select 27297,'ISRO' from dual union all
select 27298,'ISRO' from dual union all
select 27299,'AE' from dual union all
select 273,'USAF' from dual union all
select 2730,'YUZH' from dual union all
select 27300,'ISRO' from dual union all
select 27301,'ISRO' from dual union all
select 27302,'ISRO' from dual union all
select 27303,'ISRO' from dual union all
select 27304,'ISRO' from dual union all
select 27305,'ISRO' from dual union all
select 27306,'ISRO' from dual union all
select 27307,'ISRO' from dual union all
select 27308,'ISRO' from dual union all
select 27309,'ISRO' from dual union all
select 2731,'YUZH' from dual union all
select 27310,'ISRO' from dual union all
select 27311,'ISRO' from dual union all
select 27312,'ISRO' from dual union all
select 27313,'ISRO' from dual union all
select 27314,'ISRO' from dual union all
select 27315,'ISRO' from dual union all
select 27316,'ISRO' from dual union all
select 27317,'ISRO' from dual union all
select 27318,'ISRO' from dual union all
select 27319,'ISRO' from dual union all
select 2732,'YUZH' from dual union all
select 27320,'ISRO' from dual union all
select 27321,'ISRO' from dual union all
select 27322,'ISRO' from dual union all
select 27323,'ISRO' from dual union all
select 27324,'ISRO' from dual union all
select 27325,'ISRO' from dual union all
select 27326,'ISRO' from dual union all
select 27327,'ISRO' from dual union all
select 27328,'FKA' from dual union all
select 27329,'ISRO' from dual union all
select 2733,'YUZH' from dual;
insert into SATELLITE_ORG
select 27330,'ISRO' from dual union all
select 27331,'ISRO' from dual union all
select 27332,'ISRO' from dual union all
select 27333,'ISRO' from dual union all
select 27334,'ISRO' from dual union all
select 27335,'ISRO' from dual union all
select 27336,'ISRO' from dual union all
select 27337,'ISRO' from dual union all
select 27338,'ISRO' from dual union all
select 27339,'ISRO' from dual union all
select 2734,'YUZH' from dual union all
select 27340,'ISRO' from dual union all
select 27341,'ISRO' from dual union all
select 27342,'ISRO' from dual union all
select 27343,'ISRO' from dual union all
select 27344,'ISRO' from dual union all
select 27345,'ISRO' from dual union all
select 27346,'ISRO' from dual union all
select 27347,'ISRO' from dual union all
select 27348,'ISRO' from dual union all
select 27349,'ISRO' from dual union all
select 2735,'RVSN' from dual union all
select 27350,'ISRO' from dual union all
select 27351,'ISRO' from dual union all
select 27352,'ISRO' from dual union all
select 27353,'ISRO' from dual union all
select 27354,'ISRO' from dual union all
select 27355,'ISRO' from dual union all
select 27356,'ISRO' from dual union all
select 27357,'ISRO' from dual union all
select 27358,'ISRO' from dual union all
select 27359,'ISRO' from dual union all
select 2736,'CIA' from dual union all
select 2736,'NRO' from dual union all
select 27360,'ISRO' from dual union all
select 27361,'ISRO' from dual union all
select 27362,'ISRO' from dual union all
select 27363,'ISRO' from dual union all
select 27364,'ISRO' from dual union all
select 27365,'ISRO' from dual union all
select 27366,'ISRO' from dual union all
select 27367,'NASDA' from dual union all
select 27368,'NASDA' from dual union all
select 27369,'NASDA' from dual union all
select 2737,'YUZH' from dual union all
select 27370,'NASA' from dual union all
select 27371,'OSC' from dual union all
select 27372,'IRIDS' from dual union all
select 27373,'IRIDS' from dual union all
select 27374,'IRIDS' from dual union all
select 27375,'IRIDS' from dual union all
select 27376,'IRIDS' from dual union all
select 27377,'BOE' from dual union all
select 27378,'ECHO' from dual union all
select 27379,'LM' from dual union all
select 2738,'YUZH' from dual union all
select 27380,'INTELB' from dual union all
select 27381,'AE' from dual union all
select 27382,'KVR' from dual union all
select 27383,'KVR' from dual union all
select 27384,'CNSA' from dual union all
select 27385,'CNSA' from dual union all
select 27386,'ESA' from dual union all
select 27387,'AE' from dual union all
select 27388,'NASA' from dual union all
select 27389,'NASA' from dual union all
select 2739,'YUZH' from dual union all
select 27390,'LM' from dual union all
select 27391,'GFZ' from dual union all
select 27391,'JPL' from dual union all
select 27392,'GFZ' from dual union all
select 27392,'JPL' from dual union all
select 27393,'EUROK' from dual union all
select 27394,'FKA' from dual union all
select 27395,'FKA' from dual union all
select 27396,'RVSNR' from dual union all
select 27397,'PLA' from dual union all
select 27398,'CASC' from dual union all
select 27399,'JSAT' from dual union all
select 274,'USAF' from dual union all
select 2740,'MO' from dual union all
select 27400,'SES' from dual union all
select 27401,'AE' from dual union all
select 27402,'AE' from dual union all
select 27403,'INTELB' from dual union all
select 27404,'KHRR' from dual union all
select 27405,'NASA' from dual union all
select 27406,'NASA' from dual union all
select 27407,'NASA' from dual union all
select 27408,'PLA' from dual union all
select 27409,'PVO' from dual union all
select 2741,'USAF' from dual union all
select 27410,'RVSNR' from dual union all
select 27411,'RVSNR' from dual union all
select 27412,'RVSNR' from dual union all
select 27413,'NASA' from dual union all
select 27414,'NSS' from dual union all
select 27415,'AE' from dual union all
select 27416,'FKA' from dual union all
select 27417,'RVSNR' from dual;
insert into SATELLITE_ORG
select 27418,'NRO' from dual union all
select 27419,'NRO' from dual union all
select 2742,'RVSN' from dual union all
select 27420,'NRO' from dual union all
select 27421,'CNES' from dual union all
select 27422,'AE' from dual union all
select 27423,'KVR' from dual union all
select 27424,'NASA' from dual union all
select 27425,'BOE' from dual union all
select 27426,'DTV' from dual union all
select 27427,'KHRR' from dual union all
select 27428,'KHRR' from dual union all
select 27429,'KHRR' from dual union all
select 2743,'NASA' from dual union all
select 27430,'CASC' from dual union all
select 27430,'GYZ' from dual union all
select 27431,'CMA' from dual union all
select 27432,'CNSA' from dual union all
select 27433,'CNSA' from dual union all
select 27434,'ISA' from dual union all
select 27435,'ISA' from dual union all
select 27436,'VMFR' from dual union all
select 27437,'KVR' from dual union all
select 27438,'INTELB' from dual union all
select 27439,'AE' from dual union all
select 2744,'NASA' from dual union all
select 27440,'NASA' from dual union all
select 27441,'KS' from dual union all
select 27442,'KHRR' from dual union all
select 27443,'KHRR' from dual union all
select 27444,'KHRR' from dual union all
select 27445,'PAN' from dual union all
select 27446,'SEALP' from dual union all
select 27447,'RVSNR' from dual union all
select 27448,'RVSNR' from dual union all
select 27449,'CNSA' from dual union all
select 2745,'RVSN' from dual union all
select 27450,'IRIDS' from dual union all
select 27451,'IRIDS' from dual union all
select 27452,'EUROK' from dual union all
select 27453,'NOAA' from dual union all
select 27454,'FKA' from dual union all
select 27455,'RVSNR' from dual union all
select 27456,'KVR' from dual union all
select 27457,'APL' from dual union all
select 27457,'NASA' from dual union all
select 27458,'BOE' from dual union all
select 27459,'BOE' from dual union all
select 2746,'TSKBEM' from dual union all
select 27460,'STELLT' from dual union all
select 27461,'NTT' from dual union all
select 27462,'AE' from dual union all
select 27463,'AE' from dual union all
select 27464,'KVR' from dual union all
select 27465,'KVR' from dual union all
select 27466,'KVR' from dual union all
select 27467,'RVSN' from dual union all
select 27468,'RVSN' from dual union all
select 27469,'USAF' from dual union all
select 2747,'RVSN' from dual union all
select 27470,'NPOL' from dual union all
select 27471,'KVR' from dual union all
select 27472,'KHRR' from dual union all
select 27473,'KVR' from dual union all
select 27474,'KVR' from dual union all
select 27475,'KVR' from dual union all
select 27476,'KVR' from dual union all
select 27477,'NOAA' from dual union all
select 27478,'USAF' from dual union all
select 27479,'USN' from dual union all
select 2748,'YUZH' from dual union all
select 27480,'NASA' from dual union all
select 27481,'NASA' from dual union all
select 27482,'MAI' from dual union all
select 27483,'MDC' from dual union all
select 27484,'AE' from dual union all
select 27485,'AE' from dual union all
select 27486,'AE' from dual union all
select 27487,'AE' from dual union all
select 27488,'ISASS' from dual union all
select 27489,'ISASS' from dual union all
select 2749,'YUZH' from dual union all
select 27490,'BOE' from dual union all
select 27491,'RVSN' from dual union all
select 27492,'CNSA' from dual union all
select 27493,'CNSA' from dual union all
select 27494,'KVR' from dual union all
select 27495,'VMF' from dual union all
select 27496,'ISRO' from dual union all
select 27497,'BOE' from dual union all
select 27498,'ISRO' from dual union all
select 27499,'EUTSA' from dual union all
select 275,'USAF' from dual union all
select 2750,'MO' from dual union all
select 27500,'LM' from dual union all
select 27501,'ECHO' from dual union all
select 27502,'ISRO' from dual union all
select 27503,'ISRO' from dual union all
select 27504,'KHRR' from dual union all
select 27505,'FKA' from dual;
insert into SATELLITE_ORG
select 27506,'FKA' from dual union all
select 27507,'FKA' from dual union all
select 27508,'EUTSA' from dual union all
select 27509,'EUMET' from dual union all
select 2751,'RVSN' from dual union all
select 27510,'AE' from dual union all
select 27511,'AE' from dual union all
select 27512,'ISRO' from dual union all
select 27513,'INTELB' from dual union all
select 27514,'AE' from dual union all
select 27515,'NASDA' from dual union all
select 27515,'USEF' from dual union all
select 27516,'NASDA' from dual union all
select 27517,'NASDA' from dual union all
select 27518,'NASDA' from dual union all
select 27519,'NASDA' from dual union all
select 2752,'MO' from dual union all
select 27520,'NASDA' from dual union all
select 27521,'FKA' from dual union all
select 27522,'FKA' from dual union all
select 27523,'FKA' from dual union all
select 27524,'FKA' from dual union all
select 27525,'ISRO' from dual union all
select 27526,'ISRO' from dual union all
select 27527,'NASDA' from dual union all
select 27528,'HISP' from dual union all
select 27529,'LM' from dual union all
select 2753,'MO' from dual union all
select 27530,'ISRO' from dual union all
select 27531,'FKA' from dual union all
select 27532,'RVSNR' from dual union all
select 27533,'KVR' from dual union all
select 27534,'KVR' from dual union all
select 27534,'VMFR' from dual union all
select 27535,'RVSNR' from dual union all
select 27536,'CNSA' from dual union all
select 27537,'NASA' from dual union all
select 27538,'ISRO' from dual union all
select 27539,'NASA' from dual union all
select 2754,'USN' from dual union all
select 27540,'ESA' from dual union all
select 27541,'KVR' from dual union all
select 27542,'KVR' from dual union all
select 27543,'KHRR' from dual union all
select 27544,'KHRR' from dual union all
select 27545,'USAF' from dual union all
select 27546,'KVR' from dual union all
select 27547,'KHRR' from dual union all
select 27548,'KHRR' from dual union all
select 27549,'KHRR' from dual union all
select 2755,'NASA' from dual union all
select 27550,'PLA' from dual union all
select 27551,'CNSA' from dual union all
select 27552,'FKA' from dual union all
select 27553,'RVSNR' from dual union all
select 27554,'EUTSA' from dual union all
select 27555,'BOE' from dual union all
select 27556,'NASA' from dual union all
select 27557,'SES' from dual union all
select 27558,'KHRR' from dual union all
select 27559,'ASAL' from dual union all
select 2756,'JPL' from dual union all
select 2756,'NASA' from dual union all
select 27560,'MOZHA' from dual union all
select 27561,'RVSNR' from dual union all
select 27562,'USAF' from dual union all
select 27563,'BOE' from dual union all
select 27564,'BOE' from dual union all
select 27565,'BOE' from dual union all
select 27566,'NASA' from dual union all
select 27567,'LM' from dual union all
select 27568,'VMF' from dual union all
select 27569,'VMF' from dual union all
select 2757,'ESSA' from dual union all
select 27570,'VMF' from dual union all
select 27571,'VMF' from dual union all
select 27572,'VMF' from dual union all
select 27573,'VMF' from dual union all
select 27574,'NASA' from dual union all
select 27575,'NASA' from dual union all
select 27576,'NASA' from dual union all
select 27577,'NASA' from dual union all
select 27578,'NASA' from dual union all
select 27579,'ISRO' from dual union all
select 2758,'NASA' from dual union all
select 27580,'ISRO' from dual union all
select 27581,'ISRO' from dual union all
select 27582,'ISRO' from dual union all
select 27583,'ISRO' from dual union all
select 27584,'ISRO' from dual union all
select 27585,'ISRO' from dual union all
select 27586,'ISRO' from dual union all
select 27587,'ISRO' from dual union all
select 27588,'ISRO' from dual union all
select 27589,'RVSNR' from dual union all
select 2759,'TSKBEM' from dual union all
select 27590,'ISRO' from dual union all
select 27591,'ISRO' from dual union all
select 27592,'ISRO' from dual union all
select 27593,'AE' from dual;
insert into SATELLITE_ORG
select 27594,'AE' from dual union all
select 27595,'AE' from dual union all
select 27596,'AE' from dual union all
select 27597,'NASDA' from dual union all
select 27598,'CRCSS' from dual union all
select 27599,'CHIBA' from dual union all
select 276,'CIA' from dual union all
select 276,'NRO' from dual union all
select 2760,'RVSN' from dual union all
select 27600,'NASDA' from dual union all
select 27601,'NASDA' from dual union all
select 27602,'AE' from dual union all
select 27603,'NSS' from dual union all
select 27604,'AE' from dual union all
select 27605,'FUCHS' from dual union all
select 27606,'SPQ' from dual union all
select 27607,'KACST' from dual union all
select 27608,'ROSAP' from dual union all
select 27609,'TRORB' from dual union all
select 2761,'CNR' from dual union all
select 27610,'KTRAS' from dual union all
select 27611,'KTRAS' from dual union all
select 27612,'SPQ' from dual union all
select 27613,'PVO' from dual union all
select 27614,'RVSNR' from dual union all
select 27615,'RVSNR' from dual union all
select 27616,'RVSNR' from dual union all
select 27617,'FKA' from dual union all
select 27617,'KVR' from dual union all
select 27618,'FKA' from dual union all
select 27618,'KVR' from dual union all
select 27619,'FKA' from dual union all
select 27619,'KVR' from dual union all
select 2762,'METP' from dual union all
select 27620,'KHRR' from dual union all
select 27621,'RVSNR' from dual union all
select 27622,'RVSNR' from dual union all
select 27623,'ISRO' from dual union all
select 27624,'NASA' from dual union all
select 27625,'AE' from dual union all
select 27626,'AE' from dual union all
select 27627,'AE' from dual union all
select 27628,'RVSNR' from dual union all
select 27629,'RVSNR' from dual union all
select 2763,'RVSN' from dual union all
select 27630,'PLA' from dual union all
select 27631,'CASC' from dual union all
select 27632,'TCAN' from dual union all
select 27633,'KHRR' from dual union all
select 27634,'PLA' from dual union all
select 27635,'USAF' from dual union all
select 27636,'USAF' from dual union all
select 27637,'NASA' from dual union all
select 27638,'VMF' from dual union all
select 27639,'RVSNR' from dual union all
select 2764,'NASA' from dual union all
select 27640,'USAF' from dual union all
select 27641,'USAF' from dual union all
select 27642,'NASA' from dual union all
select 27643,'NASA' from dual union all
select 27644,'BOE' from dual union all
select 27645,'BOE' from dual union all
select 27646,'KHRR' from dual union all
select 27647,'NASA' from dual union all
select 27648,'AE' from dual union all
select 27649,'AE' from dual union all
select 2765,'USAF' from dual union all
select 27650,'AE' from dual union all
select 27651,'NASA' from dual union all
select 27652,'OSC' from dual union all
select 27653,'ISRO' from dual union all
select 27654,'ISRO' from dual union all
select 27655,'ISRO' from dual union all
select 27656,'ISRO' from dual union all
select 27657,'ISRO' from dual union all
select 27658,'CRC' from dual union all
select 27659,'AE' from dual union all
select 2766,'USAF' from dual union all
select 27660,'AE' from dual union all
select 27661,'AE' from dual union all
select 27662,'AE' from dual union all
select 27663,'USAF' from dual union all
select 27664,'USAF' from dual union all
select 27665,'BOE' from dual union all
select 27666,'BOE' from dual union all
select 27667,'AE' from dual union all
select 27668,'AE' from dual union all
select 27669,'AE' from dual union all
select 2767,'USAF' from dual union all
select 27670,'AE' from dual union all
select 27671,'AE' from dual union all
select 27672,'AE' from dual union all
select 27673,'AE' from dual union all
select 27674,'AE' from dual union all
select 27675,'AE' from dual union all
select 27676,'CNSA' from dual union all
select 27677,'CNSA' from dual union all
select 27678,'CNSA' from dual union all
select 27679,'OSC' from dual union all
select 2768,'USAF' from dual;
insert into SATELLITE_ORG
select 27680,'USAF' from dual union all
select 27681,'FKA' from dual union all
select 27682,'RVSNR' from dual union all
select 27683,'INTELB' from dual union all
select 27684,'AE' from dual union all
select 27685,'AE' from dual union all
select 27686,'MDC' from dual union all
select 27687,'CNSA' from dual union all
select 27688,'CNSA' from dual union all
select 27689,'KVR' from dual union all
select 2769,'USAF' from dual union all
select 27690,'KVR' from dual union all
select 27691,'DOD' from dual union all
select 27691,'USAF' from dual union all
select 27692,'BOE' from dual union all
select 27693,'USAF' from dual union all
select 27694,'BOE' from dual union all
select 27695,'BOE' from dual union all
select 27696,'BOE' from dual union all
select 27697,'BOE' from dual union all
select 27698,'CSICE' from dual union all
select 27699,'CSICE' from dual union all
select 277,'USAF' from dual union all
select 2770,'USAF' from dual union all
select 27700,'NASDA' from dual union all
select 27701,'NASDA' from dual union all
select 27702,'NASDA' from dual union all
select 27703,'NASDA' from dual union all
select 27704,'USAF' from dual union all
select 27705,'BOE' from dual union all
select 27706,'BOE' from dual union all
select 27707,'KVR' from dual union all
select 27708,'RVSNR' from dual union all
select 27709,'RVSNR' from dual union all
select 2771,'NASA' from dual union all
select 27710,'RVSNR' from dual union all
select 27711,'USAF' from dual union all
select 27712,'USAF' from dual union all
select 27713,'NASA' from dual union all
select 27714,'ISRO' from dual union all
select 27715,'PAN' from dual union all
select 27716,'AE' from dual union all
select 27717,'AE' from dual union all
select 27718,'ASIA' from dual union all
select 27719,'LM' from dual union all
select 2772,'NASA' from dual union all
select 27720,'NASA' from dual union all
select 27721,'USAF' from dual union all
select 27722,'USAF' from dual union all
select 27723,'USAF' from dual union all
select 27724,'USAF' from dual union all
select 27725,'USAF' from dual union all
select 27726,'NASA' from dual union all
select 27727,'NASA' from dual union all
select 27728,'NASA' from dual union all
select 27729,'NASA' from dual union all
select 2773,'SRC' from dual union all
select 27730,'NASA' from dual union all
select 27731,'ISAS' from dual union all
select 27732,'ESA' from dual union all
select 27733,'MO' from dual union all
select 27734,'MO' from dual union all
select 27735,'MO' from dual union all
select 27736,'RVSN' from dual union all
select 27737,'RVSN' from dual union all
select 27738,'RVSN' from dual union all
select 27739,'VMF' from dual union all
select 2774,'NASA' from dual union all
select 27740,'AE' from dual union all
select 27741,'RVSNR' from dual union all
select 27742,'RVSN' from dual union all
select 27743,'RVSN' from dual union all
select 27744,'RVSN' from dual union all
select 27745,'RVSN' from dual union all
select 27746,'RVSN' from dual union all
select 27747,'RVSN' from dual union all
select 27748,'RVSN' from dual union all
select 27749,'RVSNR' from dual union all
select 2775,'NASA' from dual union all
select 27750,'RVSNR' from dual union all
select 27751,'RVSNR' from dual union all
select 27752,'RVSNR' from dual union all
select 27753,'RVSNR' from dual union all
select 27754,'MDC' from dual union all
select 27755,'OSC' from dual union all
select 27756,'OSC' from dual union all
select 27757,'OSC' from dual union all
select 27758,'OSC' from dual union all
select 27759,'OSC' from dual union all
select 2776,'OKB10' from dual union all
select 27760,'KVR' from dual union all
select 27761,'MDC' from dual union all
select 27762,'CNSA' from dual union all
select 27763,'NPOL' from dual union all
select 27764,'BOE' from dual union all
select 27765,'BOE' from dual union all
select 27766,'BOE' from dual union all
select 27767,'ISRO' from dual union all
select 27768,'RVSNR' from dual union all
select 27769,'RVSNR' from dual;
insert into SATELLITE_ORG
select 2777,'USN' from dual union all
select 27770,'RVSNR' from dual union all
select 27771,'BOE' from dual union all
select 27772,'GLOB' from dual union all
select 27773,'BOE' from dual union all
select 27774,'RVSNR' from dual union all
select 27775,'PVO' from dual union all
select 27776,'KHRR' from dual union all
select 27777,'KHRR' from dual union all
select 27778,'RVSNR' from dual union all
select 27779,'RVSNR' from dual union all
select 2778,'USN' from dual union all
select 27780,'KHRR' from dual union all
select 27781,'FKA' from dual union all
select 27782,'RVSNR' from dual union all
select 27783,'NASA' from dual union all
select 27784,'OSC' from dual union all
select 27785,'USAF' from dual union all
select 27786,'USAF' from dual union all
select 27787,'USAF' from dual union all
select 27788,'NASA' from dual union all
select 27789,'NASA' from dual union all
select 2779,'CIA' from dual union all
select 2779,'NRO' from dual union all
select 27790,'USAF' from dual union all
select 27791,'USAF' from dual union all
select 27792,'USAF' from dual union all
select 27793,'USAF' from dual union all
select 27794,'USAF' from dual union all
select 27795,'ESA' from dual union all
select 27796,'MO' from dual union all
select 27797,'MO' from dual union all
select 27798,'RVSN' from dual union all
select 27799,'RVSN' from dual union all
select 278,'USAF' from dual union all
select 2780,'NRO' from dual union all
select 27800,'RVSN' from dual union all
select 27801,'RVSN' from dual union all
select 27802,'RVSN' from dual union all
select 27803,'RVSN' from dual union all
select 27804,'RVSNR' from dual union all
select 27805,'RVSNR' from dual union all
select 27806,'CNSA' from dual union all
select 27807,'ISRO' from dual union all
select 27808,'ISRO' from dual union all
select 27809,'ISASS' from dual union all
select 2781,'MO' from dual union all
select 27810,'ISASS' from dual union all
select 27811,'HELSA' from dual union all
select 27812,'LM' from dual union all
select 27813,'CNSA' from dual union all
select 27814,'CASC' from dual union all
select 27815,'NASDA' from dual union all
select 27815,'USEF' from dual union all
select 27816,'ESA' from dual union all
select 27817,'NPOL' from dual union all
select 27818,'VMFR' from dual union all
select 27819,'KVR' from dual union all
select 2782,'RVSN' from dual union all
select 27820,'SES' from dual union all
select 27821,'KHRR' from dual union all
select 27822,'NASA' from dual union all
select 27823,'FKA' from dual union all
select 27824,'RVSNR' from dual union all
select 27825,'THUR' from dual union all
select 27826,'SEALP' from dual union all
select 27827,'JPL' from dual union all
select 27827,'NASA' from dual union all
select 27828,'BOE' from dual union all
select 27829,'BOE' from dual union all
select 2783,'MO' from dual union all
select 27830,'BSAT' from dual union all
select 27831,'OPTS' from dual union all
select 27832,'AE' from dual union all
select 27833,'AE' from dual union all
select 27834,'KS' from dual union all
select 27834,'KVR' from dual union all
select 27835,'KVR' from dual union all
select 27836,'KVR' from dual union all
select 27837,'KVR' from dual union all
select 27838,'ORBI' from dual union all
select 27839,'OSC' from dual union all
select 2784,'MO' from dual union all
select 27840,'EUROK' from dual union all
select 27841,'ANCZ' from dual union all
select 27842,'DTU' from dual union all
select 27843,'CSA' from dual union all
select 27844,'TIT' from dual union all
select 27845,'QUAFI' from dual union all
select 27846,'AAU' from dual union all
select 27847,'UTIAS0' from dual union all
select 27848,'TOK' from dual union all
select 27849,'JPL' from dual union all
select 27849,'NASA' from dual union all
select 2785,'MO' from dual union all
select 27850,'BOE' from dual union all
select 27851,'BOE' from dual union all
select 27852,'CABVS' from dual union all
select 27853,'LM' from dual union all
select 27854,'ECHO' from dual;
insert into SATELLITE_ORG
select 27855,'SEALP' from dual union all
select 27856,'KVR' from dual union all
select 27857,'KVR' from dual union all
select 27858,'CSA' from dual union all
select 27859,'OSC' from dual union all
select 2786,'USAF' from dual union all
select 27860,'FKA' from dual union all
select 27861,'FKA' from dual union all
select 27862,'FKA' from dual union all
select 27863,'FKA' from dual union all
select 27864,'USAF' from dual union all
select 27865,'USAF' from dual union all
select 27866,'USAF' from dual union all
select 27867,'USAF' from dual union all
select 27868,'KVR' from dual union all
select 27869,'KVR' from dual union all
select 2787,'USAF' from dual union all
select 27870,'KVR' from dual union all
select 27871,'JPL' from dual union all
select 27871,'NASA' from dual union all
select 27872,'BOE' from dual union all
select 27873,'FKA' from dual union all
select 27874,'RVSNR' from dual union all
select 27875,'DOD' from dual union all
select 27875,'USAF' from dual union all
select 27876,'BOE' from dual union all
select 27877,'USAF' from dual union all
select 27878,'USAF' from dual union all
select 27879,'USAF' from dual union all
select 2788,'USAF' from dual union all
select 27880,'USAF' from dual union all
select 27881,'KVR' from dual union all
select 27882,'PVO' from dual union all
select 27883,'PVO' from dual union all
select 27884,'PVO' from dual union all
select 27885,'PVO' from dual union all
select 27886,'PVO' from dual union all
select 27887,'PVO' from dual union all
select 27888,'PVO' from dual union all
select 27889,'JPL' from dual union all
select 27889,'NASA' from dual union all
select 2789,'USAF' from dual union all
select 27890,'PVO' from dual union all
select 27891,'PVO' from dual union all
select 27892,'PVO' from dual union all
select 27893,'PVO' from dual union all
select 27894,'PVO' from dual union all
select 27895,'PVO' from dual union all
select 27896,'PVO' from dual union all
select 27897,'PVO' from dual union all
select 27898,'PVO' from dual union all
select 27899,'PVO' from dual union all
select 279,'USAF' from dual union all
select 2790,'USAF' from dual union all
select 27900,'PVO' from dual union all
select 27901,'PVO' from dual union all
select 27902,'PVO' from dual union all
select 27903,'PVO' from dual union all
select 27904,'PVO' from dual union all
select 27905,'PVO' from dual union all
select 27906,'MO' from dual union all
select 27907,'MO' from dual union all
select 27908,'VMF' from dual union all
select 27909,'VMF' from dual union all
select 2791,'USAF' from dual union all
select 27910,'VMF' from dual union all
select 27911,'VMF' from dual union all
select 27912,'VMF' from dual union all
select 27913,'VMF' from dual union all
select 27914,'KVR' from dual union all
select 27915,'KVR' from dual union all
select 27916,'KVR' from dual union all
select 27917,'KVR' from dual union all
select 27918,'KVR' from dual union all
select 27919,'KVR' from dual union all
select 2792,'USAF' from dual union all
select 27920,'KVR' from dual union all
select 27921,'KVR' from dual union all
select 27922,'KVR' from dual union all
select 27923,'KVR' from dual union all
select 27924,'KVR' from dual union all
select 27925,'KVR' from dual union all
select 27926,'KVR' from dual union all
select 27927,'KVR' from dual union all
select 27928,'KVR' from dual union all
select 27929,'KVR' from dual union all
select 2793,'USAF' from dual union all
select 27930,'KVR' from dual union all
select 27931,'KVR' from dual union all
select 27932,'KVR' from dual union all
select 27933,'KVR' from dual union all
select 27934,'KVR' from dual union all
select 27935,'KVR' from dual union all
select 27936,'KVR' from dual union all
select 27937,'NROC' from dual union all
select 27938,'USAF' from dual union all
select 27939,'MOZHA' from dual union all
select 2794,'USAF' from dual union all
select 27940,'KVR' from dual union all
select 27941,'NSRDA' from dual;
insert into SATELLITE_ORG
select 27942,'BNSC' from dual union all
select 27943,'TUBT' from dual union all
select 27944,'KVR' from dual union all
select 27945,'KAIST' from dual union all
select 27946,'AE' from dual union all
select 27947,'NASA' from dual union all
select 27948,'EUTSA' from dual union all
select 27949,'ESA' from dual union all
select 2795,'USAF' from dual union all
select 27950,'AE' from dual union all
select 27951,'ISRO' from dual union all
select 27952,'AE' from dual union all
select 27953,'SEALP' from dual union all
select 27954,'PAN' from dual union all
select 27955,'BOE' from dual union all
select 27956,'JPL' from dual union all
select 27956,'NASA' from dual union all
select 27957,'KVR' from dual union all
select 27958,'KVR' from dual union all
select 27959,'KVR' from dual union all
select 2796,'USAF' from dual union all
select 27960,'KVR' from dual union all
select 27961,'KVR' from dual union all
select 27962,'KVR' from dual union all
select 27963,'KVR' from dual union all
select 27964,'RVSNR' from dual union all
select 27965,'RVSNR' from dual union all
select 27966,'RVSNR' from dual union all
select 27967,'RVSNR' from dual union all
select 27968,'RVSNR' from dual union all
select 27969,'RVSNR' from dual union all
select 2797,'OKB10' from dual union all
select 27970,'RVSNR' from dual union all
select 27971,'RVSNR' from dual union all
select 27972,'RVSNR' from dual union all
select 27973,'RVSNR' from dual union all
select 27974,'RVSNR' from dual union all
select 27975,'RVSNR' from dual union all
select 27976,'RVSNR' from dual union all
select 27977,'RVSNR' from dual union all
select 27978,'RVSNR' from dual union all
select 27979,'RVSNR' from dual union all
select 2798,'USN' from dual union all
select 27980,'RVSNR' from dual union all
select 27981,'RVSNR' from dual union all
select 27982,'RVSNR' from dual union all
select 27983,'RVSNR' from dual union all
select 27984,'RVSNR' from dual union all
select 27985,'RVSNR' from dual union all
select 27986,'RVSNR' from dual union all
select 27987,'RVSNR' from dual union all
select 27988,'RVSNR' from dual union all
select 27989,'RVSNR' from dual union all
select 2799,'USAF' from dual union all
select 27990,'RVSNR' from dual union all
select 27991,'RVSNR' from dual union all
select 27992,'RVSNR' from dual union all
select 27993,'RVSNR' from dual union all
select 27994,'RVSNR' from dual union all
select 27995,'RVSNR' from dual union all
select 27996,'RVSNR' from dual union all
select 27997,'RVSNR' from dual union all
select 27998,'RVSNR' from dual union all
select 27999,'RVSNR' from dual union all
select 28,'USAF' from dual union all
select 280,'JPL' from dual union all
select 280,'NASA' from dual union all
select 2800,'USAF' from dual union all
select 28000,'RVSNR' from dual union all
select 28001,'RVSNR' from dual union all
select 28002,'RVSNR' from dual union all
select 28003,'RVSNR' from dual union all
select 28004,'RVSNR' from dual union all
select 28005,'RVSNR' from dual union all
select 28006,'RVSNR' from dual union all
select 28007,'RVSNR' from dual union all
select 28008,'RVSNR' from dual union all
select 28009,'RVSNR' from dual union all
select 2801,'OKB10' from dual union all
select 28010,'RVSNR' from dual union all
select 28011,'RVSNR' from dual union all
select 28012,'RVSNR' from dual union all
select 28013,'RVSNR' from dual union all
select 28014,'RVSNR' from dual union all
select 28015,'RVSNR' from dual union all
select 28016,'RVSNR' from dual union all
select 28017,'RVSNR' from dual union all
select 28018,'RVSNR' from dual union all
select 28019,'RVSNR' from dual union all
select 2802,'RVSN' from dual union all
select 28020,'RVSNR' from dual union all
select 28021,'RVSNR' from dual union all
select 28022,'RVSNR' from dual union all
select 28023,'RVSNR' from dual union all
select 28024,'RVSNR' from dual union all
select 28025,'RVSNR' from dual union all
select 28026,'RVSNR' from dual union all
select 28027,'RVSNR' from dual union all
select 28028,'RVSNR' from dual union all
select 28029,'RVSNR' from dual;
insert into SATELLITE_ORG
select 2803,'RVSN' from dual union all
select 28030,'RVSNR' from dual union all
select 28031,'RVSNR' from dual union all
select 28032,'RVSNR' from dual union all
select 28033,'RVSNR' from dual union all
select 28034,'RVSNR' from dual union all
select 28035,'RVSNR' from dual union all
select 28036,'RVSNR' from dual union all
select 28037,'RVSNR' from dual union all
select 28038,'RVSNR' from dual union all
select 28039,'AE' from dual union all
select 2804,'RVSN' from dual union all
select 28040,'AE' from dual union all
select 28041,'AE' from dual union all
select 28042,'RVSNR' from dual union all
select 28043,'PLA' from dual union all
select 28044,'CASC' from dual union all
select 28045,'CASC' from dual union all
select 28046,'CASC' from dual union all
select 28047,'CASC' from dual union all
select 28048,'CASC' from dual union all
select 28049,'PLA' from dual union all
select 2805,'IKI' from dual union all
select 2805,'NPOL' from dual union all
select 28050,'ISRO' from dual union all
select 28051,'ISRO' from dual union all
select 28052,'FKA' from dual union all
select 28053,'RVSNR' from dual union all
select 28054,'USAF' from dual union all
select 28055,'USAF' from dual union all
select 28056,'USAF' from dual union all
select 28057,'GZB' from dual union all
select 28057,'INPE' from dual union all
select 28058,'CASC' from dual union all
select 28059,'CNSA' from dual union all
select 2806,'RVSN' from dual union all
select 28060,'USEF' from dual union all
select 28061,'EUROK' from dual union all
select 28062,'CASC' from dual union all
select 28063,'CASC' from dual union all
select 28064,'CNSA' from dual union all
select 28065,'AE' from dual union all
select 28066,'ESA' from dual union all
select 28067,'CNSA' from dual union all
select 28068,'CNSA' from dual union all
select 28069,'CNSA' from dual union all
select 2807,'USN' from dual union all
select 28070,'CNSA' from dual union all
select 28071,'CNSA' from dual union all
select 28072,'CNSA' from dual union all
select 28073,'CNSA' from dual union all
select 28074,'CNSA' from dual union all
select 28075,'CNSA' from dual union all
select 28076,'CASC' from dual union all
select 28077,'CASC' from dual union all
select 28078,'CASC' from dual union all
select 28079,'CASC' from dual union all
select 2808,'RVSN' from dual union all
select 28080,'USAF' from dual union all
select 28081,'USAF' from dual union all
select 28082,'PLA' from dual union all
select 28083,'CASC' from dual union all
select 28084,'KVR' from dual union all
select 28085,'KVR' from dual union all
select 28086,'KVR' from dual union all
select 28087,'KVR' from dual union all
select 28088,'KVR' from dual union all
select 28089,'GAZC' from dual union all
select 2809,'RVSN' from dual union all
select 28090,'KHRR' from dual union all
select 28091,'KHRR' from dual union all
select 28092,'RVSNR' from dual union all
select 28093,'RVSNR' from dual union all
select 28094,'GAZC' from dual union all
select 28095,'NROC' from dual union all
select 28095,'USN' from dual union all
select 28096,'LM' from dual union all
select 28097,'NROC' from dual union all
select 28097,'USN' from dual union all
select 28098,'NPOMA' from dual union all
select 28099,'NPOMA' from dual union all
select 281,'OKB1' from dual union all
select 2810,'RVSN' from dual union all
select 28100,'KVR' from dual union all
select 28101,'KVR' from dual union all
select 28102,'KVR' from dual union all
select 28103,'KVR' from dual union all
select 28104,'KVR' from dual union all
select 28105,'KVR' from dual union all
select 28106,'KVR' from dual union all
select 28107,'KVR' from dual union all
select 28108,'KVR' from dual union all
select 28109,'KVR' from dual union all
select 2811,'NASA' from dual union all
select 28110,'KVR' from dual union all
select 28111,'KVR' from dual union all
select 28112,'FKA' from dual union all
select 28112,'KVR' from dual union all
select 28113,'FKA' from dual union all
select 28113,'KVR' from dual;
insert into SATELLITE_ORG
select 28114,'FKA' from dual union all
select 28114,'KVR' from dual union all
select 28115,'KHRR' from dual union all
select 28116,'KHRR' from dual union all
select 28117,'USN' from dual union all
select 28118,'LM' from dual union all
select 28119,'KHRR' from dual union all
select 2812,'MO' from dual union all
select 28120,'KVR' from dual union all
select 28121,'KVR' from dual union all
select 28122,'KVR' from dual union all
select 28123,'KVR' from dual union all
select 28124,'KVR' from dual union all
select 28125,'KVR' from dual union all
select 28126,'KVR' from dual union all
select 28127,'KVR' from dual union all
select 28128,'KVR' from dual union all
select 28129,'USAF' from dual union all
select 2813,'NRO' from dual union all
select 28130,'BOE' from dual union all
select 28131,'BOE' from dual union all
select 28132,'IAI' from dual union all
select 28133,'NPOL' from dual union all
select 28134,'KS' from dual union all
select 28135,'KHRR' from dual union all
select 28136,'KHRR' from dual union all
select 28137,'LORC' from dual union all
select 28138,'SEALP' from dual union all
select 28139,'KHRR' from dual union all
select 2814,'RVSN' from dual union all
select 28140,'CASC' from dual union all
select 28141,'CASC' from dual union all
select 28142,'FKA' from dual union all
select 28143,'RVSNR' from dual union all
select 28144,'NASA' from dual union all
select 28145,'NASA' from dual union all
select 28146,'JPL' from dual union all
select 28146,'NASA' from dual union all
select 28147,'ESA' from dual union all
select 28148,'NASA' from dual union all
select 28149,'KVR' from dual union all
select 2815,'RVSN' from dual union all
select 28150,'KVR' from dual union all
select 28151,'KVR' from dual union all
select 28152,'RVSNR' from dual union all
select 28153,'RVSNR' from dual union all
select 28154,'SES' from dual union all
select 28155,'LM' from dual union all
select 28156,'USAF' from dual union all
select 28157,'USN' from dual union all
select 28158,'USAF' from dual union all
select 28159,'USAF' from dual union all
select 2816,'USAF' from dual union all
select 28160,'USAF' from dual union all
select 28161,'USAF' from dual union all
select 28162,'NASA' from dual union all
select 28163,'KVR' from dual union all
select 28164,'RVSNR' from dual union all
select 28165,'RVSNR' from dual union all
select 28166,'RVSNR' from dual union all
select 28167,'RVSNR' from dual union all
select 28168,'RVSNR' from dual union all
select 28169,'ESA' from dual union all
select 2817,'NASA' from dual union all
select 28170,'AE' from dual union all
select 28171,'KVR' from dual union all
select 28172,'KVR' from dual union all
select 28173,'KVR' from dual union all
select 28174,'KVR' from dual union all
select 28175,'KVR' from dual union all
select 28176,'KVR' from dual union all
select 28177,'KVR' from dual union all
select 28178,'KVR' from dual union all
select 28179,'KVR' from dual union all
select 2818,'RVSN' from dual union all
select 28180,'KVR' from dual union all
select 28181,'KVR' from dual union all
select 28182,'KVR' from dual union all
select 28183,'KVR' from dual union all
select 28184,'MBCO' from dual union all
select 28184,'SKTEL' from dual union all
select 28185,'LM' from dual union all
select 28186,'RVSNR' from dual union all
select 28187,'EUTSA' from dual union all
select 28188,'KHRR' from dual union all
select 28189,'BOE' from dual union all
select 2819,'RVSN' from dual union all
select 28190,'USAF' from dual union all
select 28191,'BOE' from dual union all
select 28192,'BOE' from dual union all
select 28193,'BOE' from dual union all
select 28194,'KVR' from dual union all
select 28195,'KHRR' from dual union all
select 28196,'KHRR' from dual union all
select 28197,'RVSNR' from dual union all
select 28198,'RVSNR' from dual union all
select 28199,'RVSNR' from dual union all
select 282,'NASA' from dual union all
select 2820,'RVSN' from dual union all
select 28200,'NASA' from dual;
insert into SATELLITE_ORG
select 28201,'NASA' from dual union all
select 28202,'NASA' from dual union all
select 28203,'NASA' from dual union all
select 28204,'NASA' from dual union all
select 28205,'NASA' from dual union all
select 28206,'NASA' from dual union all
select 28207,'NASA' from dual union all
select 28208,'NASA' from dual union all
select 28209,'NASA' from dual union all
select 2821,'RVSN' from dual union all
select 28210,'USN' from dual union all
select 28211,'VMF' from dual union all
select 28212,'KVR' from dual union all
select 28213,'KVR' from dual union all
select 28214,'KVR' from dual union all
select 28215,'KVR' from dual union all
select 28216,'KVR' from dual union all
select 28217,'BOE' from dual union all
select 28218,'SCC' from dual union all
select 28219,'LM' from dual union all
select 2822,'OKB10' from dual union all
select 28220,'CASC' from dual union all
select 28221,'CASC' from dual union all
select 28222,'CASC' from dual union all
select 28223,'CASC' from dual union all
select 28224,'CASC' from dual union all
select 28225,'CASC' from dual union all
select 28226,'CASC' from dual union all
select 28227,'CASC' from dual union all
select 28228,'FKA' from dual union all
select 28229,'RVSNR' from dual union all
select 2823,'RVSN' from dual union all
select 28230,'NASA' from dual union all
select 28231,'BOE' from dual union all
select 28232,'CASC' from dual union all
select 28233,'CASC' from dual union all
select 28234,'KS' from dual union all
select 28235,'KHRR' from dual union all
select 28236,'KHRR' from dual union all
select 28237,'NASDA' from dual union all
select 28238,'DTV' from dual union all
select 28239,'SEALP' from dual union all
select 2824,'CIA' from dual union all
select 2824,'NRO' from dual union all
select 28240,'KHRR' from dual union all
select 28241,'RVSNR' from dual union all
select 28242,'RVSNR' from dual union all
select 28243,'NASDA' from dual union all
select 28244,'BOE' from dual union all
select 28245,'BOE' from dual union all
select 28246,'BOE' from dual union all
select 28247,'BOE' from dual union all
select 28248,'USN' from dual union all
select 28249,'USN' from dual union all
select 2825,'USAF' from dual union all
select 28250,'USN' from dual union all
select 28251,'USAF' from dual union all
select 28252,'SES' from dual union all
select 28253,'LM' from dual union all
select 28254,'NSPT' from dual union all
select 28255,'OSC' from dual union all
select 28256,'RVSNR' from dual union all
select 28257,'RVSN' from dual union all
select 28258,'RVSN' from dual union all
select 28259,'RVSN' from dual union all
select 2826,'USN' from dual union all
select 28260,'RVSN' from dual union all
select 28261,'FKA' from dual union all
select 28262,'RVSNR' from dual union all
select 28263,'BOE' from dual union all
select 28264,'BOE' from dual union all
select 28265,'USAF' from dual union all
select 28266,'NASA' from dual union all
select 28267,'NASA' from dual union all
select 28268,'RVSN' from dual union all
select 28269,'MO' from dual union all
select 2827,'MO' from dual union all
select 28270,'MO' from dual union all
select 28271,'VMF' from dual union all
select 28272,'VMF' from dual union all
select 28273,'VMF' from dual union all
select 28274,'VMF' from dual union all
select 28275,'VMF' from dual union all
select 28276,'RVSNR' from dual union all
select 28277,'RVSNR' from dual union all
select 28278,'RVSNR' from dual union all
select 28279,'AE' from dual union all
select 2828,'NRO' from dual union all
select 2828,'USN' from dual union all
select 28280,'AE' from dual union all
select 28281,'RVSNR' from dual union all
select 28282,'RVSNR' from dual union all
select 28283,'RVSNR' from dual union all
select 28284,'RVSNR' from dual union all
select 28285,'RVSNR' from dual union all
select 28286,'RVSNR' from dual union all
select 28287,'RVSNR' from dual union all
select 28288,'RVSNR' from dual union all
select 28289,'RVSNR' from dual union all
select 2829,'RVSN' from dual;
insert into SATELLITE_ORG
select 28290,'RVSNR' from dual union all
select 28291,'RVSNR' from dual union all
select 28292,'RVSNR' from dual union all
select 28293,'RVSNR' from dual union all
select 28294,'USAF' from dual union all
select 28295,'USAF' from dual union all
select 28296,'USAF' from dual union all
select 28297,'USAF' from dual union all
select 28298,'USAF' from dual union all
select 28299,'USAF' from dual union all
select 283,'RVSN' from dual union all
select 2830,'MO' from dual union all
select 28300,'USAF' from dual union all
select 28301,'USAF' from dual union all
select 28302,'USAF' from dual union all
select 28303,'USAF' from dual union all
select 28304,'USAF' from dual union all
select 28305,'USAF' from dual union all
select 28306,'USAF' from dual union all
select 28307,'USAF' from dual union all
select 28308,'USAF' from dual union all
select 28309,'USAF' from dual union all
select 2831,'NRO' from dual union all
select 28310,'USAF' from dual union all
select 28311,'USAF' from dual union all
select 28312,'USAF' from dual union all
select 28313,'USAF' from dual union all
select 28314,'USAF' from dual union all
select 28315,'USAF' from dual union all
select 28316,'USAF' from dual union all
select 28317,'USAF' from dual union all
select 28318,'USAF' from dual union all
select 28319,'USAF' from dual union all
select 2832,'YUZH' from dual union all
select 28320,'USAF' from dual union all
select 28321,'USAF' from dual union all
select 28322,'USAF' from dual union all
select 28323,'USAF' from dual union all
select 28324,'USAF' from dual union all
select 28325,'USAF' from dual union all
select 28326,'USAF' from dual union all
select 28327,'USAF' from dual union all
select 28328,'USAF' from dual union all
select 28329,'USAF' from dual union all
select 2833,'RVSN' from dual union all
select 28330,'USAF' from dual union all
select 28331,'USAF' from dual union all
select 28332,'USAF' from dual union all
select 28333,'USAF' from dual union all
select 28334,'USAF' from dual union all
select 28335,'USAF' from dual union all
select 28336,'USAF' from dual union all
select 28337,'USAF' from dual union all
select 28338,'USAF' from dual union all
select 28339,'USAF' from dual union all
select 2834,'NRO' from dual union all
select 2834,'USN' from dual union all
select 28340,'USAF' from dual union all
select 28341,'USAF' from dual union all
select 28342,'USAF' from dual union all
select 28343,'USAF' from dual union all
select 28344,'USAF' from dual union all
select 28345,'USAF' from dual union all
select 28346,'USAF' from dual union all
select 28347,'USAF' from dual union all
select 28348,'USAF' from dual union all
select 28349,'USAF' from dual union all
select 2835,'USAF' from dual union all
select 28350,'VMFR' from dual union all
select 28351,'KVR' from dual union all
select 28352,'MORF' from dual union all
select 28353,'KVR' from dual union all
select 28354,'KVR' from dual union all
select 28355,'KVR' from dual union all
select 28356,'KVR' from dual union all
select 28357,'KVR' from dual union all
select 28358,'INTELB' from dual union all
select 28359,'KHRR' from dual union all
select 2836,'MO' from dual union all
select 28360,'KHRR' from dual union all
select 28361,'USAF' from dual union all
select 28362,'BOE' from dual union all
select 28363,'BOE' from dual union all
select 28364,'LORC' from dual union all
select 28365,'SEALP' from dual union all
select 28366,'SPQ' from dual union all
select 28367,'KTRAS' from dual union all
select 28368,'CNES' from dual union all
select 28369,'KACST' from dual union all
select 2837,'RVSN' from dual union all
select 28370,'KACST' from dual union all
select 28371,'KACST' from dual union all
select 28372,'SPQ' from dual union all
select 28373,'ROSAP' from dual union all
select 28374,'KTRAS' from dual union all
select 28375,'AMNA' from dual union all
select 28376,'NASA' from dual union all
select 28377,'BOE' from dual union all
select 28378,'TCAN' from dual union all
select 28379,'AE' from dual;
insert into SATELLITE_ORG
select 2838,'MO' from dual union all
select 28380,'VMFR' from dual union all
select 28381,'KVR' from dual union all
select 28382,'CASC' from dual union all
select 28383,'ISRO' from dual union all
select 28384,'NROC' from dual union all
select 28385,'LM' from dual union all
select 28386,'FKA' from dual union all
select 28387,'NASA' from dual union all
select 28388,'KVR' from dual union all
select 28389,'KVR' from dual union all
select 2839,'RVSN' from dual union all
select 28390,'BOE' from dual union all
select 28391,'APL' from dual union all
select 28392,'BOE' from dual union all
select 28393,'HISP' from dual union all
select 28394,'KHRR' from dual union all
select 28395,'KHRR' from dual union all
select 28396,'KVR' from dual union all
select 28397,'KVR' from dual union all
select 28398,'NASA' from dual union all
select 28399,'FKA' from dual union all
select 284,'RVSN' from dual union all
select 2840,'IKI' from dual union all
select 2840,'NPOL' from dual union all
select 28400,'RVSNR' from dual union all
select 28401,'PVO' from dual union all
select 28402,'CNSA' from dual union all
select 28403,'CNSA' from dual union all
select 28404,'CASC' from dual union all
select 28405,'CASC' from dual union all
select 28406,'NASA' from dual union all
select 28407,'NASA' from dual union all
select 28408,'FKA' from dual union all
select 28409,'FKA' from dual union all
select 2841,'RVSN' from dual union all
select 28410,'FKA' from dual union all
select 28411,'FKA' from dual union all
select 28412,'FKA' from dual union all
select 28413,'CASC' from dual union all
select 28414,'CASC' from dual union all
select 28415,'CNSA' from dual union all
select 28416,'CNSA' from dual union all
select 28417,'ISRO' from dual union all
select 28418,'ISRO' from dual union all
select 28419,'KVR' from dual union all
select 2842,'PVO' from dual union all
select 2842,'YUZH' from dual union all
select 28420,'KVR' from dual union all
select 28421,'KVR' from dual union all
select 28422,'NASA' from dual union all
select 28423,'NASA' from dual union all
select 28424,'CNSA' from dual union all
select 28425,'CNSA' from dual union all
select 28426,'CASC' from dual union all
select 28427,'CNSA' from dual union all
select 28428,'ISRO' from dual union all
select 28429,'ISRO' from dual union all
select 2843,'RVSN' from dual union all
select 28430,'ISRO' from dual union all
select 28431,'ISRO' from dual union all
select 28432,'ISRO' from dual union all
select 28433,'ISRO' from dual union all
select 28434,'ISRO' from dual union all
select 28435,'ISRO' from dual union all
select 28436,'KVR' from dual union all
select 28437,'KVR' from dual union all
select 28438,'CASC' from dual union all
select 28439,'CNSA' from dual union all
select 2844,'RVSN' from dual union all
select 28440,'CNSA' from dual union all
select 28441,'CNSA' from dual union all
select 28442,'CNSA' from dual union all
select 28443,'NASA' from dual union all
select 28444,'FKA' from dual union all
select 28445,'RVSNR' from dual union all
select 28446,'SES' from dual union all
select 28447,'KHRR' from dual union all
select 28448,'CASC' from dual union all
select 28449,'CASC' from dual union all
select 2845,'JPL' from dual union all
select 2845,'NASA' from dual union all
select 28450,'CASC' from dual union all
select 28451,'CMA' from dual union all
select 28452,'CASC' from dual union all
select 28453,'JPL' from dual union all
select 28453,'NASA' from dual union all
select 28454,'NASDA' from dual union all
select 28455,'NASDA' from dual union all
select 28456,'VMF' from dual union all
select 28457,'USAF' from dual union all
select 28458,'USAF' from dual union all
select 28459,'CNSA' from dual union all
select 2846,'NASA' from dual union all
select 28460,'CNSA' from dual union all
select 28461,'CNSA' from dual union all
select 28462,'CNSA' from dual union all
select 28463,'KS' from dual union all
select 28464,'KHRR' from dual union all
select 28465,'KHRR' from dual;
insert into SATELLITE_ORG
select 28466,'KHRR' from dual union all
select 28467,'RVSNR' from dual union all
select 28468,'RVSNR' from dual union all
select 28469,'RVSNR' from dual union all
select 2847,'USN' from dual union all
select 28470,'PLA' from dual union all
select 28471,'CNSA' from dual union all
select 28472,'SES' from dual union all
select 28473,'LM' from dual union all
select 28474,'USAF' from dual union all
select 28475,'BOE' from dual union all
select 28476,'BOE' from dual union all
select 28477,'KHRR' from dual union all
select 28478,'RVSNR' from dual union all
select 28479,'CASC' from dual union all
select 2848,'YUZH' from dual union all
select 28480,'CASC' from dual union all
select 28481,'CASC' from dual union all
select 28482,'CASC' from dual union all
select 28483,'CASC' from dual union all
select 28484,'CASC' from dual union all
select 28485,'NASA' from dual union all
select 28486,'BOE' from dual union all
select 28487,'RVSNR' from dual union all
select 28488,'CASC' from dual union all
select 28489,'RVSNR' from dual union all
select 2849,'RVSN' from dual union all
select 28490,'CNSA' from dual union all
select 28491,'CASC' from dual union all
select 28492,'CNES' from dual union all
select 28492,'DGA' from dual union all
select 28493,'INTA' from dual union all
select 28494,'CNES' from dual union all
select 28494,'DGA' from dual union all
select 28495,'CNES' from dual union all
select 28495,'DGA' from dual union all
select 28496,'CNES' from dual union all
select 28496,'DGA' from dual union all
select 28497,'CNES' from dual union all
select 28497,'DGA' from dual union all
select 28498,'CNES' from dual union all
select 28499,'AE' from dual union all
select 285,'BNC' from dual union all
select 2850,'CIA' from dual union all
select 2850,'NRO' from dual union all
select 28500,'BOE' from dual union all
select 28501,'KVR' from dual union all
select 28502,'KVR' from dual union all
select 28503,'FKA' from dual union all
select 28504,'RVSNR' from dual union all
select 28505,'NKAU' from dual union all
select 28506,'KVR' from dual union all
select 28507,'NKAU' from dual union all
select 28508,'FKA' from dual union all
select 28508,'KVR' from dual union all
select 28509,'FKA' from dual union all
select 28509,'KVR' from dual union all
select 2851,'NRO' from dual union all
select 28510,'FKA' from dual union all
select 28510,'KVR' from dual union all
select 28511,'KHRR' from dual union all
select 28512,'KHRR' from dual union all
select 28513,'KHRR' from dual union all
select 28514,'RVSNR' from dual union all
select 28515,'RVSNR' from dual union all
select 28516,'KVR' from dual union all
select 28517,'NASA' from dual union all
select 28518,'BOE' from dual union all
select 28519,'BOE' from dual union all
select 2852,'IKI' from dual union all
select 2852,'NPOL' from dual union all
select 28520,'NASA' from dual union all
select 28521,'VMFR' from dual union all
select 28522,'KVR' from dual union all
select 28523,'KVR' from dual union all
select 28523,'MGU' from dual union all
select 28524,'USAF' from dual union all
select 28525,'USAF' from dual union all
select 28526,'SES' from dual union all
select 28527,'KHRR' from dual union all
select 28528,'KHRR' from dual union all
select 28529,'USN' from dual union all
select 2853,'RVSN' from dual union all
select 28530,'USAF' from dual union all
select 28531,'USAF' from dual union all
select 28532,'USAF' from dual union all
select 28533,'USAF' from dual union all
select 28534,'USAF' from dual union all
select 28535,'USAF' from dual union all
select 28536,'USAF' from dual union all
select 28537,'NROC' from dual union all
select 28537,'USN' from dual union all
select 28538,'LM' from dual union all
select 28539,'USN' from dual union all
select 2854,'RVSN' from dual union all
select 28540,'USN' from dual union all
select 28541,'NROC' from dual union all
select 28541,'USN' from dual union all
select 28542,'HISP' from dual union all
select 28542,'XTAR' from dual;
insert into SATELLITE_ORG
select 28543,'ESA' from dual union all
select 28544,'ESA' from dual union all
select 28545,'ESA' from dual union all
select 28546,'BOE' from dual union all
select 28547,'FKA' from dual union all
select 28547,'NIIKP' from dual union all
select 28548,'BOE' from dual union all
select 28549,'BOE' from dual union all
select 2855,'RVSN' from dual union all
select 28550,'USAF' from dual union all
select 28551,'USAF' from dual union all
select 28552,'USAF' from dual union all
select 28553,'USAF' from dual union all
select 28554,'KVR' from dual union all
select 28555,'KVR' from dual union all
select 28556,'ESA' from dual union all
select 28557,'ESA' from dual union all
select 28558,'RVSN' from dual union all
select 28559,'RVSN' from dual union all
select 2856,'RVSN' from dual union all
select 28560,'RVSN' from dual union all
select 28561,'RVSN' from dual union all
select 28562,'AE' from dual union all
select 28563,'AE' from dual union all
select 28564,'AE' from dual union all
select 28565,'AE' from dual union all
select 28566,'AE' from dual union all
select 28567,'AE' from dual union all
select 28568,'AE' from dual union all
select 28569,'AE' from dual union all
select 2857,'RVSN' from dual union all
select 28570,'AE' from dual union all
select 28571,'KVR' from dual union all
select 28572,'KVR' from dual union all
select 28573,'AE' from dual union all
select 28574,'AE' from dual union all
select 28575,'AE' from dual union all
select 28576,'AE' from dual union all
select 28577,'AE' from dual union all
select 28578,'RVSNR' from dual union all
select 28579,'RVSNR' from dual union all
select 2858,'NRO' from dual union all
select 28580,'RVSNR' from dual union all
select 28581,'RVSNR' from dual union all
select 28582,'RVSNR' from dual union all
select 28583,'RVSNR' from dual union all
select 28584,'RVSNR' from dual union all
select 28585,'RVSNR' from dual union all
select 28586,'RVSNR' from dual union all
select 28587,'RVSNR' from dual union all
select 28588,'RVSNR' from dual union all
select 28589,'RVSNR' from dual union all
select 2859,'SRC' from dual union all
select 28590,'RVSNR' from dual union all
select 28591,'USAF' from dual union all
select 28592,'USAF' from dual union all
select 28593,'USAF' from dual union all
select 28594,'KVR' from dual union all
select 28595,'KVR' from dual union all
select 28596,'KVR' from dual union all
select 28597,'KVR' from dual union all
select 28598,'KVR' from dual union all
select 28599,'KVR' from dual union all
select 286,'USAF' from dual union all
select 2860,'SRC' from dual union all
select 28600,'KVR' from dual union all
select 28601,'KVR' from dual union all
select 28602,'KVR' from dual union all
select 28603,'VKS' from dual union all
select 28604,'NOAA' from dual union all
select 28605,'USAF' from dual union all
select 28606,'USAF' from dual union all
select 28607,'USAF' from dual union all
select 28608,'NASA' from dual union all
select 28609,'NASA' from dual union all
select 2861,'USA' from dual union all
select 28610,'NASA' from dual union all
select 28611,'NASA' from dual union all
select 28612,'NASA' from dual union all
select 28613,'NASA' from dual union all
select 28614,'NASA' from dual union all
select 28615,'NASA' from dual union all
select 28616,'NASA' from dual union all
select 28617,'NASA' from dual union all
select 28618,'NASA' from dual union all
select 28619,'NASA' from dual union all
select 2862,'USAF' from dual union all
select 28620,'NASA' from dual union all
select 28621,'NASA' from dual union all
select 28622,'NASDA' from dual union all
select 28623,'NASDA' from dual union all
select 28624,'FKA' from dual union all
select 28625,'RVSNR' from dual union all
select 28626,'XMR' from dual union all
select 28627,'SEALP' from dual union all
select 28628,'INMRL' from dual union all
select 28629,'KS' from dual union all
select 2863,'USAF' from dual union all
select 28630,'KHRR' from dual union all
select 28631,'KHRR' from dual;
insert into SATELLITE_ORG
select 28632,'RVSNR' from dual union all
select 28633,'RVSNR' from dual union all
select 28634,'KHRR' from dual union all
select 28635,'FKA' from dual union all
select 28636,'USAF' from dual union all
select 28637,'OSC' from dual union all
select 28638,'APT' from dual union all
select 28639,'CASC' from dual union all
select 2864,'USAF' from dual union all
select 28640,'FKA' from dual union all
select 28641,'RVSNR' from dual union all
select 28642,'NASA' from dual union all
select 28643,'OSC' from dual union all
select 28644,'DTV' from dual union all
select 28645,'SEALP' from dual union all
select 28646,'NROC' from dual union all
select 28647,'USAF' from dual union all
select 28648,'USAF' from dual union all
select 28649,'ISRO' from dual union all
select 2865,'USAF' from dual union all
select 28650,'AMVU' from dual union all
select 28651,'ISRO' from dual union all
select 28652,'ISRO' from dual union all
select 28653,'ISRO' from dual union all
select 28654,'NOAA' from dual union all
select 28655,'BOE' from dual union all
select 28656,'KHRR' from dual union all
select 28657,'USAF' from dual union all
select 28658,'USAF' from dual union all
select 28659,'DTV' from dual union all
select 2866,'USAF' from dual union all
select 28660,'KHRR' from dual union all
select 28661,'KHRR' from dual union all
select 28662,'NASA' from dual union all
select 28663,'USAF' from dual union all
select 28664,'USAF' from dual union all
select 28665,'NASA' from dual union all
select 28666,'NASA' from dual union all
select 28667,'NASA' from dual union all
select 28668,'NASA' from dual union all
select 28669,'NASA' from dual union all
select 2867,'USN' from dual union all
select 28670,'NASA' from dual union all
select 28671,'NASA' from dual union all
select 28672,'USN' from dual union all
select 28673,'USN' from dual union all
select 28674,'USN' from dual union all
select 28675,'USN' from dual union all
select 28676,'USN' from dual union all
select 28677,'RVSNR' from dual union all
select 28678,'RVSNR' from dual union all
select 28679,'RVSNR' from dual union all
select 2868,'USAF' from dual union all
select 28680,'RVSNR' from dual union all
select 28681,'RVSNR' from dual union all
select 28682,'RVSNR' from dual union all
select 28683,'RVSNR' from dual union all
select 28684,'RVSNR' from dual union all
select 28685,'RVSNR' from dual union all
select 28686,'FKA' from dual union all
select 28687,'KVR' from dual union all
select 28688,'KVR' from dual union all
select 28689,'KVR' from dual union all
select 2869,'MO' from dual union all
select 28690,'KVR' from dual union all
select 28691,'KVR' from dual union all
select 28692,'KVR' from dual union all
select 28693,'KVR' from dual union all
select 28694,'RVSNR' from dual union all
select 28695,'RVSNR' from dual union all
select 28696,'RVSNR' from dual union all
select 28697,'RVSNR' from dual union all
select 28698,'RVSNR' from dual union all
select 28699,'RVSNR' from dual union all
select 287,'OKB1' from dual union all
select 2870,'RVSN' from dual union all
select 28700,'FKA' from dual union all
select 28701,'RVSNR' from dual union all
select 28702,'INTELB' from dual union all
select 28703,'SEALP' from dual union all
select 28704,'KHRR' from dual union all
select 28705,'KVR' from dual union all
select 28706,'RVSNR' from dual union all
select 28707,'KS' from dual union all
select 28708,'KHRR' from dual union all
select 28709,'KHRR' from dual union all
select 2871,'MO' from dual union all
select 28710,'KHRR' from dual union all
select 28711,'KHRR' from dual union all
select 28712,'RVSNR' from dual union all
select 28713,'RVSNR' from dual union all
select 28714,'KHRR' from dual union all
select 28715,'KVR' from dual union all
select 28716,'KVR' from dual union all
select 28717,'KVR' from dual union all
select 28718,'KVR' from dual union all
select 28719,'KVR' from dual union all
select 2872,'USN' from dual union all
select 28720,'KVR' from dual union all
select 28721,'KVR' from dual;
insert into SATELLITE_ORG
select 28722,'KVR' from dual union all
select 28723,'KVR' from dual union all
select 28724,'KVR' from dual union all
select 28725,'KVR' from dual union all
select 28726,'KVR' from dual union all
select 28727,'KVR' from dual union all
select 28728,'KVR' from dual union all
select 28729,'KVR' from dual union all
select 2873,'NRO' from dual union all
select 2873,'USN' from dual union all
select 28730,'KVR' from dual union all
select 28731,'KVR' from dual union all
select 28732,'KVR' from dual union all
select 28733,'KVR' from dual union all
select 28734,'KVR' from dual union all
select 28735,'KVR' from dual union all
select 28736,'KVR' from dual union all
select 28737,'CASC' from dual union all
select 28738,'CASC' from dual union all
select 28739,'CASC' from dual union all
select 2874,'NRO' from dual union all
select 2874,'USN' from dual union all
select 28740,'CASC' from dual union all
select 28741,'CASC' from dual union all
select 28742,'CASC' from dual union all
select 28743,'KVR' from dual union all
select 28744,'KVR' from dual union all
select 28745,'KVR' from dual union all
select 28746,'KVR' from dual union all
select 28747,'KVR' from dual union all
select 28748,'KVR' from dual union all
select 28749,'KVR' from dual union all
select 2875,'JPL' from dual union all
select 2875,'NASA' from dual union all
select 28750,'KVR' from dual union all
select 28751,'KVR' from dual union all
select 28752,'KVR' from dual union all
select 28753,'KVR' from dual union all
select 28754,'KVR' from dual union all
select 28755,'KVR' from dual union all
select 28756,'KVR' from dual union all
select 28757,'KVR' from dual union all
select 28758,'KVR' from dual union all
select 28759,'KVR' from dual union all
select 2876,'USN' from dual union all
select 28760,'KVR' from dual union all
select 28761,'KVR' from dual union all
select 28762,'KVR' from dual union all
select 28763,'KVR' from dual union all
select 28764,'KVR' from dual union all
select 28765,'KVR' from dual union all
select 28766,'KVR' from dual union all
select 28767,'KVR' from dual union all
select 28768,'KVR' from dual union all
select 28769,'KVR' from dual union all
select 2877,'USAF' from dual union all
select 28770,'KVR' from dual union all
select 28771,'KVR' from dual union all
select 28772,'KVR' from dual union all
select 28773,'JAXA' from dual union all
select 28774,'JAXA' from dual union all
select 28775,'NASA' from dual union all
select 28776,'CASC' from dual union all
select 28777,'CASC' from dual union all
select 28778,'CASC' from dual union all
select 28779,'CASC' from dual union all
select 2878,'RVSN' from dual union all
select 28780,'CASC' from dual union all
select 28781,'CASC' from dual union all
select 28782,'CASC' from dual union all
select 28783,'CASC' from dual union all
select 28784,'CASC' from dual union all
select 28785,'CASC' from dual union all
select 28786,'SHIN' from dual union all
select 28787,'AE' from dual union all
select 28788,'JPL' from dual union all
select 28788,'NASA' from dual union all
select 28789,'LM' from dual union all
select 2879,'RVSN' from dual union all
select 28790,'PAN' from dual union all
select 28791,'NPOL' from dual union all
select 28792,'FKA' from dual union all
select 28793,'NASA' from dual union all
select 28794,'NASA' from dual union all
select 28795,'NASA' from dual union all
select 28796,'NASA' from dual union all
select 28797,'NASA' from dual union all
select 28798,'NASA' from dual union all
select 28799,'NASA' from dual union all
select 288,'NASA' from dual union all
select 2880,'RVSN' from dual union all
select 28800,'NASA' from dual union all
select 28801,'NASA' from dual union all
select 28802,'NASA' from dual union all
select 28803,'NASA' from dual union all
select 28804,'USAF' from dual union all
select 28805,'USAF' from dual union all
select 28806,'USAF' from dual union all
select 28807,'USAF' from dual union all
select 28808,'USAF' from dual;
insert into SATELLITE_ORG
select 28809,'JAXA' from dual union all
select 2881,'RVSN' from dual union all
select 28810,'JAXA' from dual union all
select 28811,'KTRAS' from dual union all
select 28812,'KTRAS' from dual union all
select 28813,'KTRAS' from dual union all
select 28814,'USAF' from dual union all
select 28815,'USAF' from dual union all
select 28816,'USAF' from dual union all
select 28817,'USAF' from dual union all
select 28818,'USAF' from dual union all
select 28819,'USAF' from dual union all
select 2882,'RVSN' from dual union all
select 28820,'USAF' from dual union all
select 28821,'USAF' from dual union all
select 28822,'KHRR' from dual union all
select 28823,'KVR' from dual union all
select 28824,'CASC' from dual union all
select 28825,'CASC' from dual union all
select 28826,'CASC' from dual union all
select 28827,'CASC' from dual union all
select 28828,'KTRAS' from dual union all
select 28829,'KTRAS' from dual union all
select 2883,'NASA' from dual union all
select 28830,'KTRAS' from dual union all
select 28831,'KTRAS' from dual union all
select 28832,'KTRAS' from dual union all
select 28833,'USAF' from dual union all
select 28834,'USAF' from dual union all
select 28835,'USAF' from dual union all
select 28836,'USAF' from dual union all
select 28837,'USAF' from dual union all
select 28838,'USAF' from dual union all
select 28839,'USAF' from dual union all
select 2884,'NASA' from dual union all
select 28840,'USAF' from dual union all
select 28841,'VTSR' from dual union all
select 28842,'RVSNR' from dual union all
select 28843,'JPL' from dual union all
select 28843,'NASA' from dual union all
select 28844,'AE' from dual union all
select 28845,'RVSNR' from dual union all
select 28846,'RVSNR' from dual union all
select 28847,'RVSNR' from dual union all
select 28848,'RVSNR' from dual union all
select 28849,'RVSNR' from dual union all
select 2885,'NASA' from dual union all
select 28850,'ISRO' from dual union all
select 28851,'ISRO' from dual union all
select 28852,'ISRO' from dual union all
select 28853,'ISRO' from dual union all
select 28854,'ISRO' from dual union all
select 28855,'ISRO' from dual union all
select 28856,'ISRO' from dual union all
select 28857,'ISRO' from dual union all
select 28858,'ISRO' from dual union all
select 28859,'ISRO' from dual union all
select 2886,'NASA' from dual union all
select 28860,'KVR' from dual union all
select 28861,'KVR' from dual union all
select 28862,'CNSA' from dual union all
select 28863,'CNSA' from dual union all
select 28864,'CNSA' from dual union all
select 28865,'CNSA' from dual union all
select 28866,'FKA' from dual union all
select 28867,'RVSNR' from dual union all
select 28868,'TCAN' from dual union all
select 28869,'KHRR' from dual union all
select 2887,'USAF' from dual union all
select 28870,'KHRR' from dual union all
select 28871,'DOD' from dual union all
select 28871,'USAF' from dual union all
select 28872,'OSC' from dual union all
select 28873,'USAF' from dual union all
select 28874,'USAF' from dual union all
select 28875,'BOE' from dual union all
select 28876,'BOE' from dual union all
select 28877,'FKA' from dual union all
select 28878,'RVSNR' from dual union all
select 28879,'PLA' from dual union all
select 2888,'NASA' from dual union all
select 28880,'CASC' from dual union all
select 28881,'CASC' from dual union all
select 28882,'CASC' from dual union all
select 28883,'PLA' from dual union all
select 28884,'PAN' from dual union all
select 28885,'DGA' from dual union all
select 28886,'AE' from dual union all
select 28887,'AE' from dual union all
select 28888,'NROC' from dual union all
select 28889,'USAF' from dual union all
select 2889,'USN' from dual union all
select 28890,'21AT' from dual union all
select 28891,'MOD' from dual union all
select 28892,'WURZ' from dual union all
select 28893,'IROST' from dual union all
select 28893,'ITRC' from dual union all
select 28894,'ESA' from dual union all
select 28895,'TOK' from dual union all
select 28896,'NASA' from dual;
insert into SATELLITE_ORG
select 28897,'ARR' from dual union all
select 28897,'NSC' from dual union all
select 28898,'KVR' from dual union all
select 28899,'INMRL' from dual union all
select 289,'OKB1' from dual union all
select 2890,'NRO' from dual union all
select 28900,'SEALP' from dual union all
select 28901,'ESA' from dual union all
select 28902,'TELK' from dual union all
select 28903,'DTV' from dual union all
select 28904,'ESA' from dual union all
select 28905,'ESA' from dual union all
select 28906,'FKA' from dual union all
select 28907,'RVSNR' from dual union all
select 28908,'KVR' from dual union all
select 28909,'FKA' from dual union all
select 2891,'USAF' from dual union all
select 28910,'KVR' from dual union all
select 28911,'ISRO' from dual union all
select 28912,'EUMET' from dual union all
select 28913,'AE' from dual union all
select 28914,'AE' from dual union all
select 28915,'FKA' from dual union all
select 28915,'KVR' from dual union all
select 28916,'FKA' from dual union all
select 28916,'KVR' from dual union all
select 28917,'FKA' from dual union all
select 28917,'KVR' from dual union all
select 28918,'KHRR' from dual union all
select 28919,'RVSNR' from dual union all
select 2892,'USAF' from dual union all
select 28920,'RVSNR' from dual union all
select 28921,'KHRR' from dual union all
select 28922,'ESA' from dual union all
select 28923,'NPOL' from dual union all
select 28924,'SES' from dual union all
select 28925,'KHRR' from dual union all
select 28926,'KHRR' from dual union all
select 28927,'JPL' from dual union all
select 28928,'APL' from dual union all
select 28928,'NASA' from dual union all
select 28929,'LM' from dual union all
select 2893,'USAF' from dual union all
select 28930,'APL' from dual union all
select 28930,'NASA' from dual union all
select 28931,'NASDA' from dual union all
select 28932,'NASDA' from dual union all
select 28933,'FKA' from dual union all
select 28934,'FKA' from dual union all
select 28935,'ECHO' from dual union all
select 28936,'SEALP' from dual union all
select 28937,'JCAB' from dual union all
select 28937,'JMA' from dual union all
select 28938,'NASDA' from dual union all
select 28939,'JAXA' from dual union all
select 2894,'USAF' from dual union all
select 28940,'JAXA' from dual union all
select 28941,'TIT' from dual union all
select 28942,'JAXA' from dual union all
select 28943,'ARAB' from dual union all
select 28943,'EADS' from dual union all
select 28944,'KHRR' from dual union all
select 28945,'HISP' from dual union all
select 28946,'EUTSA' from dual union all
select 28947,'AE' from dual union all
select 28948,'AE' from dual union all
select 28949,'CNES' from dual union all
select 2895,'NASA' from dual union all
select 28950,'RVSNR' from dual union all
select 28951,'RVSNR' from dual union all
select 28952,'RVSNR' from dual union all
select 28953,'RVSNR' from dual union all
select 28954,'RVSNR' from dual union all
select 28955,'RVSNR' from dual union all
select 28956,'RVSNR' from dual union all
select 28957,'RVSNR' from dual union all
select 28958,'RVSNR' from dual union all
select 28959,'RVSNR' from dual union all
select 2896,'NASA' from dual union all
select 28960,'RVSNR' from dual union all
select 28961,'RVSNR' from dual union all
select 28962,'RVSNR' from dual union all
select 28963,'RVSNR' from dual union all
select 28964,'RVSNR' from dual union all
select 28965,'RVSNR' from dual union all
select 28966,'RVSNR' from dual union all
select 28967,'RVSNR' from dual union all
select 28968,'RVSNR' from dual union all
select 28969,'RVSNR' from dual union all
select 2897,'USAF' from dual union all
select 28970,'RVSNR' from dual union all
select 28971,'RVSNR' from dual union all
select 28972,'KVR' from dual union all
select 28973,'KVR' from dual union all
select 28974,'KVR' from dual union all
select 28975,'KVR' from dual union all
select 28976,'USAF' from dual union all
select 28977,'NASA' from dual union all
select 28978,'NASA' from dual union all
select 28979,'NASA' from dual;
insert into SATELLITE_ORG
select 2898,'USAF' from dual union all
select 28980,'NASA' from dual union all
select 28981,'NASA' from dual union all
select 28982,'NASA' from dual union all
select 28983,'OSC' from dual union all
select 28984,'MAI' from dual union all
select 28985,'RVSNR' from dual union all
select 28986,'KVR' from dual union all
select 28987,'KVR' from dual union all
select 28988,'KVR' from dual union all
select 28989,'KVR' from dual union all
select 2899,'USAF' from dual union all
select 28990,'KVR' from dual union all
select 28991,'KVR' from dual union all
select 28992,'KVR' from dual union all
select 28993,'KVR' from dual union all
select 28994,'KVR' from dual union all
select 28995,'KVR' from dual union all
select 28996,'FKA' from dual union all
select 28997,'RVSNR' from dual union all
select 28998,'KHRR' from dual union all
select 28999,'NASDA' from dual union all
select 29,'NASA' from dual union all
select 290,'CIA' from dual union all
select 290,'NRO' from dual union all
select 2900,'CNES' from dual union all
select 29000,'USAF' from dual union all
select 29001,'RCA' from dual union all
select 29002,'RCA' from dual union all
select 29003,'RCA' from dual union all
select 29004,'RCA' from dual union all
select 29005,'NASA' from dual union all
select 29006,'NASA' from dual union all
select 29007,'FKA' from dual union all
select 29008,'OKB1' from dual union all
select 29009,'NASA' from dual union all
select 2901,'USAF' from dual union all
select 29010,'NASA' from dual union all
select 29011,'USAF' from dual union all
select 29012,'USAF' from dual union all
select 29013,'USAF' from dual union all
select 29014,'RVSN' from dual union all
select 29015,'ESA' from dual union all
select 29016,'ESA' from dual union all
select 29017,'ESA' from dual union all
select 29018,'ESA' from dual union all
select 29019,'VMF' from dual union all
select 2902,'RVSN' from dual union all
select 29020,'VMF' from dual union all
select 29021,'VMF' from dual union all
select 29022,'AE' from dual union all
select 29023,'AE' from dual union all
select 29024,'AE' from dual union all
select 29025,'RVSNR' from dual union all
select 29026,'AE' from dual union all
select 29027,'RVSNR' from dual union all
select 29028,'RVSNR' from dual union all
select 29029,'RVSNR' from dual union all
select 2903,'RVSN' from dual union all
select 29030,'RVSNR' from dual union all
select 29031,'RVSNR' from dual union all
select 29032,'RVSNR' from dual union all
select 29033,'OSC' from dual union all
select 29034,'KVR' from dual union all
select 29035,'KVR' from dual union all
select 29036,'KVR' from dual union all
select 29037,'NASA' from dual union all
select 29038,'CNSA' from dual union all
select 29039,'CNSA' from dual union all
select 2904,'RVSN' from dual union all
select 29040,'CNSA' from dual union all
select 29041,'BOE' from dual union all
select 29042,'CNSA' from dual union all
select 29043,'CNSA' from dual union all
select 29044,'KTRAS' from dual union all
select 29045,'JSAT' from dual union all
select 29046,'SEALP' from dual union all
select 29047,'NSPO' from dual union all
select 29048,'NSPO' from dual union all
select 29049,'NSPO' from dual union all
select 2905,'RVSN' from dual union all
select 29050,'NSPO' from dual union all
select 29051,'NSPO' from dual union all
select 29052,'NSPO' from dual union all
select 29053,'OSC' from dual union all
select 29054,'JAXA' from dual union all
select 29055,'SES' from dual union all
select 29056,'LM' from dual union all
select 29057,'FKA' from dual union all
select 29058,'RVSNR' from dual union all
select 29059,'USAF' from dual union all
select 2906,'RVSN' from dual union all
select 29060,'USAF' from dual union all
select 29061,'USAF' from dual union all
select 29062,'NASA' from dual union all
select 29063,'USAF' from dual union all
select 29064,'USAF' from dual union all
select 29065,'RVSN' from dual union all
select 29066,'KVR' from dual union all
select 29067,'KVR' from dual;
insert into SATELLITE_ORG
select 29068,'USAF' from dual union all
select 29069,'USAF' from dual union all
select 2907,'NASA' from dual union all
select 29070,'USAF' from dual union all
select 29071,'USAF' from dual union all
select 29072,'USAF' from dual union all
select 29073,'USAF' from dual union all
select 29074,'USAF' from dual union all
select 29075,'USAF' from dual union all
select 29076,'USAF' from dual union all
select 29077,'USAF' from dual union all
select 29078,'USAF' from dual union all
select 29079,'IMAG' from dual union all
select 2908,'NASA' from dual union all
select 29080,'PU' from dual union all
select 29081,'USAF' from dual union all
select 29082,'USAF' from dual union all
select 29083,'USAF' from dual union all
select 29084,'USAF' from dual union all
select 29085,'USAF' from dual union all
select 29086,'USAF' from dual union all
select 29087,'USAF' from dual union all
select 29088,'USAF' from dual union all
select 29089,'USAF' from dual union all
select 2909,'USN' from dual union all
select 29090,'USAF' from dual union all
select 29091,'USAF' from dual union all
select 29092,'PLA' from dual union all
select 29093,'CNSA' from dual union all
select 29094,'RVSNR' from dual union all
select 29095,'RVSNR' from dual union all
select 29096,'RVSNR' from dual union all
select 29097,'RVSNR' from dual union all
select 29098,'RVSNR' from dual union all
select 29099,'RVSNR' from dual union all
select 291,'USAF' from dual union all
select 2910,'CIA' from dual union all
select 2910,'NRO' from dual union all
select 29100,'RVSNR' from dual union all
select 29101,'RVSNR' from dual union all
select 29102,'RVSNR' from dual union all
select 29103,'RVSNR' from dual union all
select 29104,'ISRO' from dual union all
select 29105,'RVSNR' from dual union all
select 29106,'EUMET' from dual union all
select 29107,'NASA' from dual union all
select 29108,'CNES' from dual union all
select 29109,'BOE' from dual union all
select 2911,'RVSN' from dual union all
select 29110,'BOE' from dual union all
select 29111,'KVR' from dual union all
select 29112,'KVR' from dual union all
select 29113,'KVR' from dual union all
select 29114,'KVR' from dual union all
select 29115,'KVR' from dual union all
select 29116,'KVR' from dual union all
select 29117,'KVR' from dual union all
select 29118,'KVR' from dual union all
select 29119,'KVR' from dual union all
select 2912,'RVSN' from dual union all
select 29120,'KVR' from dual union all
select 29121,'KVR' from dual union all
select 29122,'KVR' from dual union all
select 29123,'KVR' from dual union all
select 29124,'KVR' from dual union all
select 29125,'KVR' from dual union all
select 29126,'KVR' from dual union all
select 29127,'KVR' from dual union all
select 29128,'KVR' from dual union all
select 29129,'KVR' from dual union all
select 2913,'RVSN' from dual union all
select 29130,'KVR' from dual union all
select 29131,'KVR' from dual union all
select 29132,'KVR' from dual union all
select 29133,'KVR' from dual union all
select 29134,'KVR' from dual union all
select 29135,'KVR' from dual union all
select 29136,'KVR' from dual union all
select 29137,'KVR' from dual union all
select 29138,'USN' from dual union all
select 29139,'USAF' from dual union all
select 2914,'MO' from dual union all
select 29140,'USN' from dual union all
select 29141,'KVR' from dual union all
select 29142,'KVR' from dual union all
select 29143,'KVR' from dual union all
select 29144,'KVR' from dual union all
select 29145,'KVR' from dual union all
select 29146,'KVR' from dual union all
select 29147,'KVR' from dual union all
select 29148,'KVR' from dual union all
select 29149,'KVR' from dual union all
select 2915,'RVSN' from dual union all
select 29150,'KVR' from dual union all
select 29151,'KVR' from dual union all
select 29152,'KVR' from dual union all
select 29153,'KVR' from dual union all
select 29154,'KVR' from dual union all
select 29155,'NOAA' from dual union all
select 29156,'BOE' from dual;
insert into SATELLITE_ORG
select 29157,'IZM' from dual union all
select 29157,'MAK' from dual union all
select 29158,'MAK' from dual union all
select 29159,'MAK' from dual union all
select 2916,'RVSN' from dual union all
select 29160,'MAK' from dual union all
select 29161,'MAK' from dual union all
select 29162,'SAMX' from dual union all
select 29163,'SHIN' from dual union all
select 29164,'AE' from dual union all
select 29165,'AE' from dual union all
select 29166,'NASA' from dual union all
select 29167,'NASA' from dual union all
select 29168,'NASA' from dual union all
select 29169,'NASA' from dual union all
select 2917,'RVSN' from dual union all
select 29170,'NASA' from dual union all
select 29171,'NASA' from dual union all
select 29172,'NASA' from dual union all
select 29173,'NASA' from dual union all
select 29174,'NASA' from dual union all
select 29175,'NASA' from dual union all
select 29176,'NASA' from dual union all
select 29177,'NASA' from dual union all
select 29178,'NASA' from dual union all
select 29179,'NASA' from dual union all
select 2918,'USAF' from dual union all
select 29180,'USN' from dual union all
select 29181,'NASA' from dual union all
select 29182,'NASA' from dual union all
select 29183,'NASA' from dual union all
select 29184,'NASA' from dual union all
select 29185,'NASA' from dual union all
select 29186,'NASA' from dual union all
select 29187,'NASA' from dual union all
select 29188,'NASA' from dual union all
select 29189,'NASA' from dual union all
select 2919,'NRO' from dual union all
select 29190,'NASA' from dual union all
select 29191,'FKA' from dual union all
select 29192,'KVR' from dual union all
select 29193,'KVR' from dual union all
select 29194,'VMFR' from dual union all
select 29195,'VMFR' from dual union all
select 29196,'VMFR' from dual union all
select 29197,'USAF' from dual union all
select 29198,'KVR' from dual union all
select 29199,'KVR' from dual union all
select 292,'NRO' from dual union all
select 292,'USA' from dual union all
select 2920,'USAF' from dual union all
select 29200,'KVR' from dual union all
select 29201,'KVR' from dual union all
select 29202,'MO' from dual union all
select 29203,'VMFR' from dual union all
select 29204,'VMFR' from dual union all
select 29205,'KVR' from dual union all
select 29206,'KVR' from dual union all
select 29207,'USN' from dual union all
select 29208,'USN' from dual union all
select 29209,'USN' from dual union all
select 2921,'PVO' from dual union all
select 2921,'YUZH' from dual union all
select 29210,'USN' from dual union all
select 29211,'USN' from dual union all
select 29212,'USN' from dual union all
select 29213,'CNSA' from dual union all
select 29214,'CNSA' from dual union all
select 29215,'CNSA' from dual union all
select 29216,'CNSA' from dual union all
select 29217,'CASC' from dual union all
select 29218,'KVR' from dual union all
select 29219,'KVR' from dual union all
select 2922,'RVSN' from dual union all
select 29220,'KVR' from dual union all
select 29221,'KVR' from dual union all
select 29222,'KVR' from dual union all
select 29223,'KVR' from dual union all
select 29224,'KVR' from dual union all
select 29225,'KVR' from dual union all
select 29226,'KVR' from dual union all
select 29227,'KVR' from dual union all
select 29228,'FKA' from dual union all
select 29229,'RVSNR' from dual union all
select 2923,'CIA' from dual union all
select 2923,'NRO' from dual union all
select 29230,'KAZK' from dual union all
select 29231,'KHRR' from dual union all
select 29232,'KHRR' from dual union all
select 29233,'KHRR' from dual union all
select 29234,'KHRR' from dual union all
select 29235,'KHRR' from dual union all
select 29236,'PAN' from dual union all
select 29237,'SEALP' from dual union all
select 29238,'KHRR' from dual union all
select 29239,'KHRR' from dual union all
select 2924,'RVSN' from dual union all
select 29240,'DOD' from dual union all
select 29241,'DOD' from dual union all
select 29242,'DOD' from dual;
insert into SATELLITE_ORG
select 29242,'USN' from dual union all
select 29243,'BOE' from dual union all
select 29244,'BOE' from dual union all
select 29245,'FKA' from dual union all
select 29246,'RVSNR' from dual union all
select 29247,'VMFR' from dual union all
select 29248,'KVR' from dual union all
select 29249,'NROC' from dual union all
select 2925,'MO' from dual union all
select 29250,'BOE' from dual union all
select 29251,'NASA' from dual union all
select 29252,'BIGEL' from dual union all
select 29253,'KTRAS' from dual union all
select 29254,'KHRR' from dual union all
select 29255,'USAF' from dual union all
select 29256,'KTRAS' from dual union all
select 29257,'NASA' from dual union all
select 29258,'KVR' from dual union all
select 29259,'KVR' from dual union all
select 2926,'RVSN' from dual union all
select 29260,'PVO' from dual union all
select 29261,'RVSNR' from dual union all
select 29262,'RVSNR' from dual union all
select 29263,'RVSNR' from dual union all
select 29264,'RVSNR' from dual union all
select 29265,'USN' from dual union all
select 29266,'RVSNR' from dual union all
select 29267,'RVSNR' from dual union all
select 29268,'KARI' from dual union all
select 29269,'EUROK' from dual union all
select 2927,'RVSN' from dual union all
select 29270,'EUTSA' from dual union all
select 29271,'KHRR' from dual union all
select 29272,'JSAT' from dual union all
select 29273,'DGA' from dual union all
select 29274,'AE' from dual union all
select 29275,'AE' from dual union all
select 29276,'USN' from dual union all
select 29277,'NASA' from dual union all
select 29278,'NASA' from dual union all
select 29279,'NASA' from dual union all
select 2928,'RVSN' from dual union all
select 29280,'RVSN' from dual union all
select 29281,'RVSN' from dual union all
select 29282,'RVSN' from dual union all
select 29283,'RVSN' from dual union all
select 29284,'RVSN' from dual union all
select 29285,'RVSN' from dual union all
select 29286,'RVSN' from dual union all
select 29287,'RVSN' from dual union all
select 29288,'RVSN' from dual union all
select 29289,'RVSN' from dual union all
select 2929,'USAF' from dual union all
select 29290,'RVSN' from dual union all
select 29291,'KVR' from dual union all
select 29292,'KVR' from dual union all
select 29293,'KVR' from dual union all
select 29294,'KVR' from dual union all
select 29295,'MO' from dual union all
select 29296,'RVSN' from dual union all
select 29297,'RGMS' from dual union all
select 29298,'RGMS' from dual union all
select 29299,'KVR' from dual union all
select 293,'USAF' from dual union all
select 2930,'USN' from dual union all
select 29300,'KVR' from dual union all
select 29301,'KVR' from dual union all
select 29302,'KVR' from dual union all
select 29303,'KVR' from dual union all
select 29304,'KVR' from dual union all
select 29305,'KVR' from dual union all
select 29306,'KVR' from dual union all
select 29307,'KVR' from dual union all
select 29308,'KVR' from dual union all
select 29309,'KVR' from dual union all
select 2931,'RVSN' from dual union all
select 29310,'KVR' from dual union all
select 29311,'KVR' from dual union all
select 29312,'KVR' from dual union all
select 29313,'KVR' from dual union all
select 29314,'KVR' from dual union all
select 29315,'KVR' from dual union all
select 29316,'KVR' from dual union all
select 29317,'KVR' from dual union all
select 29318,'KVR' from dual union all
select 29319,'KVR' from dual union all
select 2932,'USAF' from dual union all
select 29320,'KVR' from dual union all
select 29321,'KVR' from dual union all
select 29322,'KVR' from dual union all
select 29323,'KVR' from dual union all
select 29324,'KVR' from dual union all
select 29325,'KVR' from dual union all
select 29326,'KVR' from dual union all
select 29327,'KVR' from dual union all
select 29328,'KVR' from dual union all
select 29329,'KVR' from dual union all
select 2933,'USAF' from dual union all
select 29330,'KVR' from dual union all
select 29331,'KVR' from dual;
insert into SATELLITE_ORG
select 29332,'KVR' from dual union all
select 29333,'KVR' from dual union all
select 29334,'KVR' from dual union all
select 29335,'KVR' from dual union all
select 29336,'KVR' from dual union all
select 29337,'KVR' from dual union all
select 29338,'KVR' from dual union all
select 29339,'KVR' from dual union all
select 2934,'RVSN' from dual union all
select 29340,'KVR' from dual union all
select 29341,'KVR' from dual union all
select 29342,'KVR' from dual union all
select 29343,'KVR' from dual union all
select 29344,'KVR' from dual union all
select 29345,'KVR' from dual union all
select 29346,'KVR' from dual union all
select 29347,'KVR' from dual union all
select 29348,'KVR' from dual union all
select 29349,'ADD' from dual union all
select 29349,'KTEL' from dual union all
select 2935,'NASA' from dual union all
select 29350,'SEALP' from dual union all
select 29351,'MO' from dual union all
select 29352,'MO' from dual union all
select 29353,'MO' from dual union all
select 29354,'KVR' from dual union all
select 29355,'RVSN' from dual union all
select 29356,'RVSN' from dual union all
select 29357,'RVSN' from dual union all
select 29358,'RVSN' from dual union all
select 29359,'RVSN' from dual union all
select 2936,'NASA' from dual union all
select 29360,'RVSN' from dual union all
select 29361,'RVSN' from dual union all
select 29362,'USN' from dual union all
select 29363,'BOE' from dual union all
select 29364,'JAXA' from dual union all
select 29365,'JAXA' from dual union all
select 29366,'JAXA' from dual union all
select 29367,'JAXA' from dual union all
select 29368,'JAXA' from dual union all
select 29369,'JAXA' from dual union all
select 2937,'JPL' from dual union all
select 2937,'NASA' from dual union all
select 29370,'JAXA' from dual union all
select 29371,'JAXA' from dual union all
select 29372,'JAXA' from dual union all
select 29373,'JAXA' from dual union all
select 29374,'JAXA' from dual union all
select 29375,'JAXA' from dual union all
select 29376,'JAXA' from dual union all
select 29377,'JAXA' from dual union all
select 29378,'JAXA' from dual union all
select 29379,'JAXA' from dual union all
select 2938,'NASA' from dual union all
select 29380,'JAXA' from dual union all
select 29381,'JAXA' from dual union all
select 29382,'JAXA' from dual union all
select 29383,'JAXA' from dual union all
select 29384,'JAXA' from dual union all
select 29385,'CASC' from dual union all
select 29386,'CASC' from dual union all
select 29387,'CASC' from dual union all
select 29388,'CASC' from dual union all
select 29389,'CASC' from dual union all
select 2939,'MO' from dual union all
select 29390,'CASC' from dual union all
select 29391,'NASA' from dual union all
select 29392,'CASC' from dual union all
select 29393,'CSICE' from dual union all
select 29394,'JAXA' from dual union all
select 29395,'JAXA' from dual union all
select 29396,'JAXA' from dual union all
select 29397,'KVR' from dual union all
select 29398,'PLA' from dual union all
select 29399,'CASC' from dual union all
select 294,'USAF' from dual union all
select 2940,'USAF' from dual union all
select 29400,'FKA' from dual union all
select 29401,'RVSNR' from dual union all
select 29402,'KVR' from dual union all
select 29403,'KVR' from dual union all
select 29404,'KVR' from dual union all
select 29405,'KVR' from dual union all
select 29406,'KVR' from dual union all
select 29407,'KVR' from dual union all
select 29408,'KVR' from dual union all
select 29409,'KVR' from dual union all
select 2941,'RVSN' from dual union all
select 29410,'KVR' from dual union all
select 29411,'KVR' from dual union all
select 29412,'KVR' from dual union all
select 29413,'KVR' from dual union all
select 29414,'KVR' from dual union all
select 29415,'KVR' from dual union all
select 29416,'KVR' from dual union all
select 29417,'KVR' from dual union all
select 29418,'KVR' from dual union all
select 29419,'KVR' from dual union all
select 2942,'PVO' from dual;
insert into SATELLITE_ORG
select 2942,'YUZH' from dual union all
select 29420,'KVR' from dual union all
select 29421,'KVR' from dual union all
select 29422,'BOE' from dual union all
select 29423,'CNSA' from dual union all
select 29424,'KVR' from dual union all
select 29425,'KVR' from dual union all
select 29426,'KVR' from dual union all
select 29427,'KVR' from dual union all
select 29428,'KVR' from dual union all
select 29429,'KVR' from dual union all
select 2943,'MO' from dual union all
select 29430,'KVR' from dual union all
select 29431,'KVR' from dual union all
select 29432,'KVR' from dual union all
select 29433,'KVR' from dual union all
select 29434,'KVR' from dual union all
select 29435,'KVR' from dual union all
select 29436,'KVR' from dual union all
select 29437,'KVR' from dual union all
select 29438,'KVR' from dual union all
select 29439,'KVR' from dual union all
select 2944,'MO' from dual union all
select 29440,'KVR' from dual union all
select 29441,'KVR' from dual union all
select 29442,'KVR' from dual union all
select 29443,'KVR' from dual union all
select 29444,'VMF' from dual union all
select 29445,'CNSA' from dual union all
select 29446,'CNSA' from dual union all
select 29447,'MAI' from dual union all
select 29448,'MAI' from dual union all
select 29449,'KVR' from dual union all
select 2945,'RVSN' from dual union all
select 29450,'KVR' from dual union all
select 29451,'KVR' from dual union all
select 29452,'KVR' from dual union all
select 29453,'KVR' from dual union all
select 29454,'KVR' from dual union all
select 29455,'KVR' from dual union all
select 29456,'KVR' from dual union all
select 29457,'KVR' from dual union all
select 29458,'KVR' from dual union all
select 29459,'KVR' from dual union all
select 2946,'CIA' from dual union all
select 2946,'NRO' from dual union all
select 29460,'KVR' from dual union all
select 29461,'KVR' from dual union all
select 29462,'KVR' from dual union all
select 29463,'KVR' from dual union all
select 29464,'KVR' from dual union all
select 29465,'KVR' from dual union all
select 29466,'KVR' from dual union all
select 29467,'KVR' from dual union all
select 29468,'KVR' from dual union all
select 29469,'KVR' from dual union all
select 2947,'MO' from dual union all
select 29470,'KVR' from dual union all
select 29471,'KVR' from dual union all
select 29472,'KVR' from dual union all
select 29473,'KVR' from dual union all
select 29474,'KVR' from dual union all
select 29475,'KVR' from dual union all
select 29476,'KVR' from dual union all
select 29477,'KVR' from dual union all
select 29478,'KVR' from dual union all
select 29479,'JAXA' from dual union all
select 2948,'RVSN' from dual union all
select 29480,'JAXA' from dual union all
select 29481,'JAXA' from dual union all
select 29482,'JAXA' from dual union all
select 29483,'JAXA' from dual union all
select 29484,'HOKIT' from dual union all
select 29485,'JAXA' from dual union all
select 29486,'USAF' from dual union all
select 29487,'BOE' from dual union all
select 29488,'BOE' from dual union all
select 29489,'KVR' from dual union all
select 2949,'USAF' from dual union all
select 29490,'JAXA' from dual union all
select 29491,'JAXA' from dual union all
select 29492,'JAXA' from dual union all
select 29493,'JAXA' from dual union all
select 29494,'DTV' from dual union all
select 29495,'OPTS' from dual union all
select 29496,'JAXA' from dual union all
select 29497,'AE' from dual union all
select 29498,'AE' from dual union all
select 29499,'EUMET' from dual union all
select 295,'NASA' from dual union all
select 2950,'USAF' from dual union all
select 29500,'NPOL' from dual union all
select 29501,'AE' from dual union all
select 29502,'AE' from dual union all
select 29503,'FKA' from dual union all
select 29504,'RVSNR' from dual union all
select 29505,'CASC' from dual union all
select 29506,'CASC' from dual union all
select 29507,'CNSA' from dual union all
select 29508,'CNSA' from dual;
insert into SATELLITE_ORG
select 29509,'CNSA' from dual union all
select 2951,'RVSN' from dual union all
select 29510,'NASA' from dual union all
select 29511,'NASA' from dual union all
select 29512,'BOE' from dual union all
select 29513,'BOE' from dual union all
select 29514,'KVR' from dual union all
select 29515,'KVR' from dual union all
select 29516,'SINO' from dual union all
select 29517,'CASC' from dual union all
select 29518,'CASC' from dual union all
select 29519,'CASC' from dual union all
select 2952,'NRO' from dual union all
select 29520,'XMR' from dual union all
select 29521,'SEALP' from dual union all
select 29522,'USAF' from dual union all
select 29523,'BOE' from dual union all
select 29524,'USAF' from dual union all
select 29525,'USAF' from dual union all
select 29526,'ARAB' from dual union all
select 29527,'KHRR' from dual union all
select 29528,'KHRR' from dual union all
select 29529,'KVR' from dual union all
select 2953,'RVSN' from dual union all
select 29530,'NASA' from dual union all
select 29531,'NASA' from dual union all
select 29532,'NASA' from dual union all
select 29533,'NASA' from dual union all
select 29534,'BOE' from dual union all
select 29535,'OSC' from dual union all
select 29536,'CNSA' from dual union all
select 29537,'CNSA' from dual union all
select 29538,'CNSA' from dual union all
select 29539,'CNSA' from dual union all
select 2954,'RVSN' from dual union all
select 29540,'USAF' from dual union all
select 29541,'USAF' from dual union all
select 29542,'USAF' from dual union all
select 29543,'USAF' from dual union all
select 29544,'USAF' from dual union all
select 29545,'USAF' from dual union all
select 29546,'USAF' from dual union all
select 29547,'USAF' from dual union all
select 29548,'USAF' from dual union all
select 29549,'USAF' from dual union all
select 2955,'RVSN' from dual union all
select 29550,'USAF' from dual union all
select 29551,'USAF' from dual union all
select 29552,'USAF' from dual union all
select 29553,'USAF' from dual union all
select 29554,'USAF' from dual union all
select 29555,'USAF' from dual union all
select 29556,'USAF' from dual union all
select 29557,'USAF' from dual union all
select 29558,'USAF' from dual union all
select 29559,'USAF' from dual union all
select 2956,'MO' from dual union all
select 29560,'USAF' from dual union all
select 29561,'USAF' from dual union all
select 29562,'USAF' from dual union all
select 29563,'USAF' from dual union all
select 29564,'USAF' from dual union all
select 29565,'USAF' from dual union all
select 29566,'USAF' from dual union all
select 29567,'USAF' from dual union all
select 29568,'USAF' from dual union all
select 29569,'USAF' from dual union all
select 2957,'MO' from dual union all
select 29570,'USAF' from dual union all
select 29571,'USAF' from dual union all
select 29572,'USAF' from dual union all
select 29573,'USAF' from dual union all
select 29574,'USAF' from dual union all
select 29575,'USAF' from dual union all
select 29576,'USAF' from dual union all
select 29577,'USAF' from dual union all
select 29578,'USAF' from dual union all
select 29579,'USAF' from dual union all
select 2958,'MO' from dual union all
select 29580,'USAF' from dual union all
select 29581,'USAF' from dual union all
select 29582,'USAF' from dual union all
select 29583,'USAF' from dual union all
select 29584,'USAF' from dual union all
select 29585,'USAF' from dual union all
select 29586,'USAF' from dual union all
select 29587,'USAF' from dual union all
select 29588,'USAF' from dual union all
select 29589,'USAF' from dual union all
select 2959,'MO' from dual union all
select 29590,'USAF' from dual union all
select 29591,'USAF' from dual union all
select 29592,'USAF' from dual union all
select 29593,'USAF' from dual union all
select 29594,'USAF' from dual union all
select 29595,'USAF' from dual union all
select 29596,'USAF' from dual union all
select 29597,'USAF' from dual union all
select 29598,'USAF' from dual union all
select 29599,'USAF' from dual;
insert into SATELLITE_ORG
select 296,'NASA' from dual union all
select 2960,'MO' from dual union all
select 29600,'USAF' from dual union all
select 29601,'USAF' from dual union all
select 29602,'BOE' from dual union all
select 29603,'BOE' from dual union all
select 29604,'KVR' from dual union all
select 29605,'KVR' from dual union all
select 29606,'KVR' from dual union all
select 29607,'KVR' from dual union all
select 29608,'KVR' from dual union all
select 29609,'KVR' from dual union all
select 2961,'MO' from dual union all
select 29610,'KVR' from dual union all
select 29611,'KVR' from dual union all
select 29612,'KVR' from dual union all
select 29613,'KVR' from dual union all
select 29614,'KVR' from dual union all
select 29615,'KVR' from dual union all
select 29616,'KVR' from dual union all
select 29617,'KVR' from dual union all
select 29618,'KVR' from dual union all
select 29619,'KVR' from dual union all
select 2962,'RVSN' from dual union all
select 29620,'KVR' from dual union all
select 29621,'KVR' from dual union all
select 29622,'KVR' from dual union all
select 29623,'KVR' from dual union all
select 29624,'KVR' from dual union all
select 29625,'KVR' from dual union all
select 29626,'KVR' from dual union all
select 29627,'KVR' from dual union all
select 29628,'KVR' from dual union all
select 29629,'KVR' from dual union all
select 2963,'RVSN' from dual union all
select 29630,'KVR' from dual union all
select 29631,'KVR' from dual union all
select 29632,'USAF' from dual union all
select 29633,'USAF' from dual union all
select 29634,'KVR' from dual union all
select 29635,'KVR' from dual union all
select 29636,'CNSA' from dual union all
select 29637,'USAF' from dual union all
select 29638,'NASA' from dual union all
select 29639,'NASA' from dual union all
select 2964,'RVSN' from dual union all
select 29640,'CMA' from dual union all
select 29641,'CASC' from dual union all
select 29642,'CASC' from dual union all
select 29643,'WBLU' from dual union all
select 29644,'SES' from dual union all
select 29645,'AE' from dual union all
select 29646,'AE' from dual union all
select 29647,'NASA' from dual union all
select 29648,'MEAS' from dual union all
select 29649,'KHRR' from dual union all
select 2965,'USN' from dual union all
select 29650,'KHRR' from dual union all
select 29651,'NROC' from dual union all
select 29652,'BOE' from dual union all
select 29653,'USAF' from dual union all
select 29654,'OSC' from dual union all
select 29655,'NASA' from dual union all
select 29656,'JAXA' from dual union all
select 29657,'JAXA' from dual union all
select 29658,'BWEHR' from dual union all
select 29659,'KVR' from dual union all
select 2966,'MO' from dual union all
select 29660,'USAF' from dual union all
select 29661,'USN' from dual union all
select 29662,'USN' from dual union all
select 29663,'USAF' from dual union all
select 29663,'USN' from dual union all
select 29664,'USAF' from dual union all
select 29664,'USN' from dual union all
select 29665,'USAF' from dual union all
select 29665,'USN' from dual union all
select 29666,'USAF' from dual union all
select 29666,'USN' from dual union all
select 29667,'USAF' from dual union all
select 29667,'USN' from dual union all
select 29668,'KVR' from dual union all
select 29669,'NPOL' from dual union all
select 2967,'NASA' from dual union all
select 29670,'FKA' from dual union all
select 29670,'KVR' from dual union all
select 29671,'FKA' from dual union all
select 29671,'KVR' from dual union all
select 29672,'FKA' from dual union all
select 29672,'KVR' from dual union all
select 29673,'KHRR' from dual union all
select 29674,'KHRR' from dual union all
select 29675,'KHRR' from dual union all
select 29676,'EUMET' from dual union all
select 29677,'NASA' from dual union all
select 29678,'CNES' from dual union all
select 29679,'RVSNR' from dual union all
select 2968,'RVSN' from dual union all
select 29680,'RVSNR' from dual union all
select 29681,'NASA' from dual;
insert into SATELLITE_ORG
select 29682,'KVR' from dual union all
select 29683,'NASA' from dual union all
select 29684,'NASA' from dual union all
select 29685,'NASA' from dual union all
select 29686,'NASA' from dual union all
select 29687,'NASA' from dual union all
select 29688,'NASA' from dual union all
select 29689,'NASA' from dual union all
select 2969,'INTEL' from dual union all
select 29690,'NASA' from dual union all
select 29691,'NASA' from dual union all
select 29692,'NASA' from dual union all
select 29693,'NASA' from dual union all
select 29694,'NASA' from dual union all
select 29695,'NASA' from dual union all
select 29696,'NASA' from dual union all
select 29697,'NASA' from dual union all
select 29698,'NASA' from dual union all
select 29699,'NASA' from dual union all
select 297,'OKB1' from dual union all
select 2970,'NASA' from dual union all
select 29700,'NASA' from dual union all
select 29701,'NASA' from dual union all
select 29702,'NASA' from dual union all
select 29703,'NASA' from dual union all
select 29704,'NASA' from dual union all
select 29705,'NASA' from dual union all
select 29706,'NASA' from dual union all
select 29707,'NASA' from dual union all
select 29708,'USAF' from dual union all
select 29709,'LAPAN' from dual union all
select 2971,'NASA' from dual union all
select 29710,'ISRO' from dual union all
select 29711,'ISRO' from dual union all
select 29712,'AMSAT' from dual union all
select 29713,'ISRO' from dual union all
select 29714,'FKA' from dual union all
select 29715,'KVR' from dual union all
select 29716,'CASC' from dual union all
select 29717,'CASC' from dual union all
select 29718,'CASC' from dual union all
select 29719,'CASC' from dual union all
select 2972,'NASA' from dual union all
select 29720,'CASC' from dual union all
select 29721,'CASC' from dual union all
select 29722,'CASC' from dual union all
select 29723,'CASC' from dual union all
select 29724,'CASC' from dual union all
select 29725,'CASC' from dual union all
select 29726,'CASC' from dual union all
select 29727,'CASC' from dual union all
select 29728,'CASC' from dual union all
select 29729,'CASC' from dual union all
select 2973,'MO' from dual union all
select 29730,'CASC' from dual union all
select 29731,'CASC' from dual union all
select 29732,'CASC' from dual union all
select 29733,'CASC' from dual union all
select 29734,'CASC' from dual union all
select 29735,'CASC' from dual union all
select 29736,'CASC' from dual union all
select 29737,'CASC' from dual union all
select 29738,'CASC' from dual union all
select 29739,'CASC' from dual union all
select 2974,'RVSN' from dual union all
select 29740,'CASC' from dual union all
select 29741,'CASC' from dual union all
select 29742,'CASC' from dual union all
select 29743,'CASC' from dual union all
select 29744,'CASC' from dual union all
select 29745,'CASC' from dual union all
select 29746,'CASC' from dual union all
select 29747,'CASC' from dual union all
select 29748,'CASC' from dual union all
select 29749,'CASC' from dual union all
select 2975,'RVSN' from dual union all
select 29750,'CASC' from dual union all
select 29751,'CASC' from dual union all
select 29752,'CASC' from dual union all
select 29753,'CASC' from dual union all
select 29754,'CASC' from dual union all
select 29755,'CASC' from dual union all
select 29756,'CASC' from dual union all
select 29757,'CASC' from dual union all
select 29758,'CASC' from dual union all
select 29759,'CASC' from dual union all
select 2976,'ESSA' from dual union all
select 29760,'CASC' from dual union all
select 29761,'CASC' from dual union all
select 29762,'CASC' from dual union all
select 29763,'CASC' from dual union all
select 29764,'CASC' from dual union all
select 29765,'CASC' from dual union all
select 29766,'CASC' from dual union all
select 29767,'CASC' from dual union all
select 29768,'CASC' from dual union all
select 29769,'CASC' from dual union all
select 2977,'ESSA' from dual union all
select 29770,'CASC' from dual union all
select 29771,'CASC' from dual;
insert into SATELLITE_ORG
select 29772,'CASC' from dual union all
select 29773,'CASC' from dual union all
select 29774,'CASC' from dual union all
select 29775,'CASC' from dual union all
select 29776,'CASC' from dual union all
select 29777,'CASC' from dual union all
select 29778,'CASC' from dual union all
select 29779,'CASC' from dual union all
select 2978,'NASA' from dual union all
select 29780,'CASC' from dual union all
select 29781,'CASC' from dual union all
select 29782,'CASC' from dual union all
select 29783,'CASC' from dual union all
select 29784,'CASC' from dual union all
select 29785,'CASC' from dual union all
select 29786,'CASC' from dual union all
select 29787,'CASC' from dual union all
select 29788,'CASC' from dual union all
select 29789,'CASC' from dual union all
select 2979,'MO' from dual union all
select 29790,'CASC' from dual union all
select 29791,'CASC' from dual union all
select 29792,'CASC' from dual union all
select 29793,'CASC' from dual union all
select 29794,'CASC' from dual union all
select 29795,'CASC' from dual union all
select 29796,'CASC' from dual union all
select 29797,'CASC' from dual union all
select 29798,'CASC' from dual union all
select 29799,'CASC' from dual union all
select 298,'RVSN' from dual union all
select 2980,'USAF' from dual union all
select 29800,'CASC' from dual union all
select 29801,'CASC' from dual union all
select 29802,'CASC' from dual union all
select 29803,'CASC' from dual union all
select 29804,'CASC' from dual union all
select 29805,'CASC' from dual union all
select 29806,'CASC' from dual union all
select 29807,'CASC' from dual union all
select 29808,'CASC' from dual union all
select 29809,'CASC' from dual union all
select 2981,'MO' from dual union all
select 29810,'CASC' from dual union all
select 29811,'CASC' from dual union all
select 29812,'CASC' from dual union all
select 29813,'CASC' from dual union all
select 29814,'CASC' from dual union all
select 29815,'CASC' from dual union all
select 29816,'CASC' from dual union all
select 29817,'CASC' from dual union all
select 29818,'CASC' from dual union all
select 29819,'CASC' from dual union all
select 2982,'RVSN' from dual union all
select 29820,'CASC' from dual union all
select 29821,'CASC' from dual union all
select 29822,'CASC' from dual union all
select 29823,'CASC' from dual union all
select 29824,'CASC' from dual union all
select 29825,'CASC' from dual union all
select 29826,'CASC' from dual union all
select 29827,'CASC' from dual union all
select 29828,'CASC' from dual union all
select 29829,'CASC' from dual union all
select 2983,'MO' from dual union all
select 29830,'CASC' from dual union all
select 29831,'CASC' from dual union all
select 29832,'CASC' from dual union all
select 29833,'CASC' from dual union all
select 29834,'CASC' from dual union all
select 29835,'CASC' from dual union all
select 29836,'CASC' from dual union all
select 29837,'CASC' from dual union all
select 29838,'CASC' from dual union all
select 29839,'CASC' from dual union all
select 2984,'MO' from dual union all
select 29840,'CASC' from dual union all
select 29841,'CASC' from dual union all
select 29842,'CASC' from dual union all
select 29843,'CASC' from dual union all
select 29844,'CASC' from dual union all
select 29845,'CASC' from dual union all
select 29846,'CASC' from dual union all
select 29847,'CASC' from dual union all
select 29848,'CASC' from dual union all
select 29849,'CASC' from dual union all
select 2985,'USAF' from dual union all
select 29850,'CASC' from dual union all
select 29851,'CASC' from dual union all
select 29852,'CASC' from dual union all
select 29853,'CASC' from dual union all
select 29854,'CASC' from dual union all
select 29855,'CASC' from dual union all
select 29856,'CASC' from dual union all
select 29857,'CASC' from dual union all
select 29858,'CASC' from dual union all
select 29859,'CASC' from dual union all
select 2986,'USN' from dual union all
select 29860,'CASC' from dual union all
select 29861,'CASC' from dual;
insert into SATELLITE_ORG
select 29862,'CASC' from dual union all
select 29863,'CASC' from dual union all
select 29864,'CASC' from dual union all
select 29865,'CASC' from dual union all
select 29866,'CASC' from dual union all
select 29867,'CASC' from dual union all
select 29868,'CASC' from dual union all
select 29869,'CASC' from dual union all
select 2987,'NASA' from dual union all
select 29870,'CASC' from dual union all
select 29871,'CASC' from dual union all
select 29872,'CASC' from dual union all
select 29873,'CASC' from dual union all
select 29874,'CASC' from dual union all
select 29875,'CASC' from dual union all
select 29876,'CASC' from dual union all
select 29877,'CASC' from dual union all
select 29878,'CASC' from dual union all
select 29879,'CASC' from dual union all
select 2988,'USAF' from dual union all
select 29880,'CASC' from dual union all
select 29881,'CASC' from dual union all
select 29882,'CASC' from dual union all
select 29883,'CASC' from dual union all
select 29884,'CASC' from dual union all
select 29885,'CASC' from dual union all
select 29886,'CASC' from dual union all
select 29887,'CASC' from dual union all
select 29888,'CASC' from dual union all
select 29889,'CASC' from dual union all
select 2989,'USN' from dual union all
select 29890,'CASC' from dual union all
select 29891,'CASC' from dual union all
select 29892,'CASC' from dual union all
select 29893,'CASC' from dual union all
select 29894,'CASC' from dual union all
select 29895,'CASC' from dual union all
select 29896,'CASC' from dual union all
select 29897,'CASC' from dual union all
select 29898,'CASC' from dual union all
select 29899,'CASC' from dual union all
select 299,'USAF' from dual union all
select 2990,'CNES' from dual union all
select 29900,'CASC' from dual union all
select 29901,'CASC' from dual union all
select 29902,'CASC' from dual union all
select 29903,'CASC' from dual union all
select 29904,'CASC' from dual union all
select 29905,'CASC' from dual union all
select 29906,'CASC' from dual union all
select 29907,'CASC' from dual union all
select 29908,'CASC' from dual union all
select 29909,'CASC' from dual union all
select 2991,'USAF' from dual union all
select 29910,'CASC' from dual union all
select 29911,'CASC' from dual union all
select 29912,'CASC' from dual union all
select 29913,'CASC' from dual union all
select 29914,'CASC' from dual union all
select 29915,'CASC' from dual union all
select 29916,'CASC' from dual union all
select 29917,'CASC' from dual union all
select 29918,'CASC' from dual union all
select 29919,'CASC' from dual union all
select 2992,'USAF' from dual union all
select 29920,'CASC' from dual union all
select 29921,'CASC' from dual union all
select 29922,'CASC' from dual union all
select 29923,'CASC' from dual union all
select 29924,'CASC' from dual union all
select 29925,'CASC' from dual union all
select 29926,'CASC' from dual union all
select 29927,'CASC' from dual union all
select 29928,'CASC' from dual union all
select 29929,'CASC' from dual union all
select 2993,'CNES' from dual union all
select 29930,'CASC' from dual union all
select 29931,'CASC' from dual union all
select 29932,'CASC' from dual union all
select 29933,'CASC' from dual union all
select 29934,'CASC' from dual union all
select 29935,'CASC' from dual union all
select 29936,'CASC' from dual union all
select 29937,'CASC' from dual union all
select 29938,'CASC' from dual union all
select 29939,'CASC' from dual union all
select 2994,'USN' from dual union all
select 29940,'CASC' from dual union all
select 29941,'CASC' from dual union all
select 29942,'CASC' from dual union all
select 29943,'CASC' from dual union all
select 29944,'CASC' from dual union all
select 29945,'CASC' from dual union all
select 29946,'CASC' from dual union all
select 29947,'CASC' from dual union all
select 29948,'CASC' from dual union all
select 29949,'CASC' from dual union all
select 2995,'MO' from dual union all
select 29950,'CASC' from dual union all
select 29951,'CASC' from dual;
insert into SATELLITE_ORG
select 29952,'CASC' from dual union all
select 29953,'CASC' from dual union all
select 29954,'CASC' from dual union all
select 29955,'CASC' from dual union all
select 29956,'CASC' from dual union all
select 29957,'CASC' from dual union all
select 29958,'CASC' from dual union all
select 29959,'CASC' from dual union all
select 2996,'RVSN' from dual union all
select 29960,'CASC' from dual union all
select 29961,'CASC' from dual union all
select 29962,'CASC' from dual union all
select 29963,'CASC' from dual union all
select 29964,'CASC' from dual union all
select 29965,'CASC' from dual union all
select 29966,'CASC' from dual union all
select 29967,'CASC' from dual union all
select 29968,'CASC' from dual union all
select 29969,'CASC' from dual union all
select 2997,'USAF' from dual union all
select 29970,'CASC' from dual union all
select 29971,'CASC' from dual union all
select 29972,'CASC' from dual union all
select 29973,'CASC' from dual union all
select 29974,'CASC' from dual union all
select 29975,'CASC' from dual union all
select 29976,'CASC' from dual union all
select 29977,'CASC' from dual union all
select 29978,'CASC' from dual union all
select 29979,'CASC' from dual union all
select 2998,'USAF' from dual union all
select 29980,'CASC' from dual union all
select 29981,'CASC' from dual union all
select 29982,'CASC' from dual union all
select 29983,'CASC' from dual union all
select 29984,'CASC' from dual union all
select 29985,'CASC' from dual union all
select 29986,'CASC' from dual union all
select 29987,'CASC' from dual union all
select 29988,'CASC' from dual union all
select 29989,'CASC' from dual union all
select 2999,'MO' from dual union all
select 29990,'CASC' from dual union all
select 29991,'CASC' from dual union all
select 29992,'CASC' from dual union all
select 29993,'CASC' from dual union all
select 29994,'CASC' from dual union all
select 29995,'CASC' from dual union all
select 29996,'CASC' from dual union all
select 29997,'CASC' from dual union all
select 29998,'CASC' from dual union all
select 29999,'CASC' from dual union all
select 3,'OKB1' from dual union all
select 30,'USAF' from dual union all
select 300,'USAF' from dual union all
select 3000,'NASA' from dual union all
select 30000,'USAF' from dual union all
select 30001,'CASC' from dual union all
select 30002,'CASC' from dual union all
select 30003,'CASC' from dual union all
select 30004,'CASC' from dual union all
select 30005,'CASC' from dual union all
select 30006,'CASC' from dual union all
select 30007,'CASC' from dual union all
select 30008,'CASC' from dual union all
select 30009,'CASC' from dual union all
select 3001,'RVSN' from dual union all
select 30010,'CASC' from dual union all
select 30011,'CASC' from dual union all
select 30012,'CASC' from dual union all
select 30013,'CASC' from dual union all
select 30014,'CASC' from dual union all
select 30015,'CASC' from dual union all
select 30016,'CASC' from dual union all
select 30017,'CASC' from dual union all
select 30018,'CASC' from dual union all
select 30019,'CASC' from dual union all
select 3002,'RVSN' from dual union all
select 30020,'CASC' from dual union all
select 30021,'CASC' from dual union all
select 30022,'CASC' from dual union all
select 30023,'CASC' from dual union all
select 30024,'CASC' from dual union all
select 30025,'CASC' from dual union all
select 30026,'CASC' from dual union all
select 30027,'CASC' from dual union all
select 30028,'CASC' from dual union all
select 30029,'CASC' from dual union all
select 3003,'RVSN' from dual union all
select 30030,'CASC' from dual union all
select 30031,'CASC' from dual union all
select 30032,'CASC' from dual union all
select 30033,'CASC' from dual union all
select 30034,'CASC' from dual union all
select 30035,'CASC' from dual union all
select 30036,'CASC' from dual union all
select 30037,'CASC' from dual union all
select 30038,'CASC' from dual union all
select 30039,'CASC' from dual union all
select 3004,'NASA' from dual;
insert into SATELLITE_ORG
select 30040,'CASC' from dual union all
select 30041,'CASC' from dual union all
select 30042,'CASC' from dual union all
select 30043,'CASC' from dual union all
select 30044,'CASC' from dual union all
select 30045,'CASC' from dual union all
select 30046,'CASC' from dual union all
select 30047,'CASC' from dual union all
select 30048,'CASC' from dual union all
select 30049,'CASC' from dual union all
select 3005,'RVSN' from dual union all
select 30050,'CASC' from dual union all
select 30051,'CASC' from dual union all
select 30052,'CASC' from dual union all
select 30053,'CASC' from dual union all
select 30054,'CASC' from dual union all
select 30055,'CASC' from dual union all
select 30056,'CASC' from dual union all
select 30057,'CASC' from dual union all
select 30058,'CASC' from dual union all
select 30059,'CASC' from dual union all
select 3006,'RVSN' from dual union all
select 30060,'CASC' from dual union all
select 30061,'CASC' from dual union all
select 30062,'CASC' from dual union all
select 30063,'CASC' from dual union all
select 30064,'CASC' from dual union all
select 30065,'CASC' from dual union all
select 30066,'CASC' from dual union all
select 30067,'CASC' from dual union all
select 30068,'CASC' from dual union all
select 30069,'CASC' from dual union all
select 3007,'RVSN' from dual union all
select 30070,'CASC' from dual union all
select 30071,'CASC' from dual union all
select 30072,'CASC' from dual union all
select 30073,'CASC' from dual union all
select 30074,'CASC' from dual union all
select 30075,'CASC' from dual union all
select 30076,'CASC' from dual union all
select 30077,'CASC' from dual union all
select 30078,'CASC' from dual union all
select 30079,'CASC' from dual union all
select 3008,'MO' from dual union all
select 30080,'CASC' from dual union all
select 30081,'CASC' from dual union all
select 30082,'CASC' from dual union all
select 30083,'CASC' from dual union all
select 30084,'CASC' from dual union all
select 30085,'CASC' from dual union all
select 30086,'CASC' from dual union all
select 30087,'CASC' from dual union all
select 30088,'CASC' from dual union all
select 30089,'CASC' from dual union all
select 3009,'RVSN' from dual union all
select 30090,'CASC' from dual union all
select 30091,'CASC' from dual union all
select 30092,'CASC' from dual union all
select 30093,'CASC' from dual union all
select 30094,'CASC' from dual union all
select 30095,'CASC' from dual union all
select 30096,'CASC' from dual union all
select 30097,'CASC' from dual union all
select 30098,'CASC' from dual union all
select 30099,'CASC' from dual union all
select 301,'USAF' from dual union all
select 3010,'METP' from dual union all
select 30100,'CASC' from dual union all
select 30101,'CASC' from dual union all
select 30102,'CASC' from dual union all
select 30103,'CASC' from dual union all
select 30104,'CASC' from dual union all
select 30105,'CASC' from dual union all
select 30106,'CASC' from dual union all
select 30107,'CASC' from dual union all
select 30108,'CASC' from dual union all
select 30109,'CASC' from dual union all
select 3011,'RVSN' from dual union all
select 30110,'CASC' from dual union all
select 30111,'CASC' from dual union all
select 30112,'CASC' from dual union all
select 30113,'CASC' from dual union all
select 30114,'CASC' from dual union all
select 30115,'CASC' from dual union all
select 30116,'CASC' from dual union all
select 30117,'CASC' from dual union all
select 30118,'CASC' from dual union all
select 30119,'CASC' from dual union all
select 3012,'NRO' from dual union all
select 30120,'CASC' from dual union all
select 30121,'CASC' from dual union all
select 30122,'CASC' from dual union all
select 30123,'CASC' from dual union all
select 30124,'CASC' from dual union all
select 30125,'CASC' from dual union all
select 30126,'CASC' from dual union all
select 30127,'CASC' from dual union all
select 30128,'CASC' from dual union all
select 30129,'CASC' from dual union all
select 3013,'PVO' from dual;
insert into SATELLITE_ORG
select 30130,'CASC' from dual union all
select 30131,'CASC' from dual union all
select 30132,'CASC' from dual union all
select 30133,'CASC' from dual union all
select 30134,'CASC' from dual union all
select 30135,'CASC' from dual union all
select 30136,'CASC' from dual union all
select 30137,'CASC' from dual union all
select 30138,'CASC' from dual union all
select 30139,'CASC' from dual union all
select 3014,'TSKBEM' from dual union all
select 30140,'CASC' from dual union all
select 30141,'CASC' from dual union all
select 30142,'CASC' from dual union all
select 30143,'CASC' from dual union all
select 30144,'CASC' from dual union all
select 30145,'CASC' from dual union all
select 30146,'CASC' from dual union all
select 30147,'CASC' from dual union all
select 30148,'CASC' from dual union all
select 30149,'CASC' from dual union all
select 3015,'RVSN' from dual union all
select 30150,'CASC' from dual union all
select 30151,'CASC' from dual union all
select 30152,'CASC' from dual union all
select 30153,'CASC' from dual union all
select 30154,'CASC' from dual union all
select 30155,'CASC' from dual union all
select 30156,'CASC' from dual union all
select 30157,'CASC' from dual union all
select 30158,'CASC' from dual union all
select 30159,'CASC' from dual union all
select 3016,'RVSN' from dual union all
select 30160,'CASC' from dual union all
select 30161,'CASC' from dual union all
select 30162,'CASC' from dual union all
select 30163,'CASC' from dual union all
select 30164,'CASC' from dual union all
select 30165,'CASC' from dual union all
select 30166,'CASC' from dual union all
select 30167,'CASC' from dual union all
select 30168,'CASC' from dual union all
select 30169,'CASC' from dual union all
select 3017,'RVSN' from dual union all
select 30170,'CASC' from dual union all
select 30171,'CASC' from dual union all
select 30172,'CASC' from dual union all
select 30173,'CASC' from dual union all
select 30174,'CASC' from dual union all
select 30175,'CASC' from dual union all
select 30176,'CASC' from dual union all
select 30177,'CASC' from dual union all
select 30178,'CASC' from dual union all
select 30179,'CASC' from dual union all
select 3018,'RVSN' from dual union all
select 30180,'CASC' from dual union all
select 30181,'CASC' from dual union all
select 30182,'CASC' from dual union all
select 30183,'CASC' from dual union all
select 30184,'CASC' from dual union all
select 30185,'CASC' from dual union all
select 30186,'CASC' from dual union all
select 30187,'CASC' from dual union all
select 30188,'CASC' from dual union all
select 30189,'CASC' from dual union all
select 3019,'RVSN' from dual union all
select 30190,'CASC' from dual union all
select 30191,'CASC' from dual union all
select 30192,'CASC' from dual union all
select 30193,'CASC' from dual union all
select 30194,'CASC' from dual union all
select 30195,'CASC' from dual union all
select 30196,'CASC' from dual union all
select 30197,'CASC' from dual union all
select 30198,'CASC' from dual union all
select 30199,'CASC' from dual union all
select 302,'CIA' from dual union all
select 302,'NRO' from dual union all
select 3020,'TSKBEM' from dual union all
select 30200,'CASC' from dual union all
select 30201,'CASC' from dual union all
select 30202,'CASC' from dual union all
select 30203,'CASC' from dual union all
select 30204,'CASC' from dual union all
select 30205,'CASC' from dual union all
select 30206,'CASC' from dual union all
select 30207,'CASC' from dual union all
select 30208,'CASC' from dual union all
select 30209,'CASC' from dual union all
select 3021,'YUZH' from dual union all
select 30210,'CASC' from dual union all
select 30211,'CASC' from dual union all
select 30212,'CASC' from dual union all
select 30213,'CASC' from dual union all
select 30214,'CASC' from dual union all
select 30215,'CASC' from dual union all
select 30216,'CASC' from dual union all
select 30217,'CASC' from dual union all
select 30218,'CASC' from dual union all
select 30219,'CASC' from dual;
insert into SATELLITE_ORG
select 3022,'RVSN' from dual union all
select 30220,'CASC' from dual union all
select 30221,'CASC' from dual union all
select 30222,'CASC' from dual union all
select 30223,'CASC' from dual union all
select 30224,'CASC' from dual union all
select 30225,'CASC' from dual union all
select 30226,'CASC' from dual union all
select 30227,'CASC' from dual union all
select 30228,'CASC' from dual union all
select 30229,'CASC' from dual union all
select 3023,'RVSN' from dual union all
select 30230,'CASC' from dual union all
select 30231,'CASC' from dual union all
select 30232,'CASC' from dual union all
select 30233,'CASC' from dual union all
select 30234,'CASC' from dual union all
select 30235,'CASC' from dual union all
select 30236,'CASC' from dual union all
select 30237,'CASC' from dual union all
select 30238,'CASC' from dual union all
select 30239,'CASC' from dual union all
select 3024,'CIA' from dual union all
select 3024,'NRO' from dual union all
select 30240,'CASC' from dual union all
select 30241,'CASC' from dual union all
select 30242,'CASC' from dual union all
select 30243,'CASC' from dual union all
select 30244,'CASC' from dual union all
select 30245,'CASC' from dual union all
select 30246,'CASC' from dual union all
select 30247,'CASC' from dual union all
select 30248,'CASC' from dual union all
select 30249,'CASC' from dual union all
select 3025,'NRO' from dual union all
select 30250,'CASC' from dual union all
select 30251,'CASC' from dual union all
select 30252,'CASC' from dual union all
select 30253,'CASC' from dual union all
select 30254,'CASC' from dual union all
select 30255,'CASC' from dual union all
select 30256,'CASC' from dual union all
select 30257,'CASC' from dual union all
select 30258,'CASC' from dual union all
select 30259,'CASC' from dual union all
select 3026,'MO' from dual union all
select 30260,'CASC' from dual union all
select 30261,'CASC' from dual union all
select 30262,'CASC' from dual union all
select 30263,'CASC' from dual union all
select 30264,'CASC' from dual union all
select 30265,'CASC' from dual union all
select 30266,'CASC' from dual union all
select 30267,'CASC' from dual union all
select 30268,'CASC' from dual union all
select 30269,'CASC' from dual union all
select 3027,'RVSN' from dual union all
select 30270,'CASC' from dual union all
select 30271,'CASC' from dual union all
select 30272,'CASC' from dual union all
select 30273,'CASC' from dual union all
select 30274,'CASC' from dual union all
select 30275,'CASC' from dual union all
select 30276,'CASC' from dual union all
select 30277,'CASC' from dual union all
select 30278,'CASC' from dual union all
select 30279,'CASC' from dual union all
select 3028,'YUZH' from dual union all
select 30280,'CASC' from dual union all
select 30281,'CASC' from dual union all
select 30282,'CASC' from dual union all
select 30283,'CASC' from dual union all
select 30284,'CASC' from dual union all
select 30285,'CASC' from dual union all
select 30286,'CASC' from dual union all
select 30287,'CASC' from dual union all
select 30288,'CASC' from dual union all
select 30289,'CASC' from dual union all
select 3029,'NASA' from dual union all
select 30290,'CASC' from dual union all
select 30291,'CASC' from dual union all
select 30292,'CASC' from dual union all
select 30293,'CASC' from dual union all
select 30294,'CASC' from dual union all
select 30295,'CASC' from dual union all
select 30296,'CASC' from dual union all
select 30297,'CASC' from dual union all
select 30298,'CASC' from dual union all
select 30299,'CASC' from dual union all
select 303,'USAF' from dual union all
select 3030,'NASA' from dual union all
select 30300,'CASC' from dual union all
select 30301,'CASC' from dual union all
select 30302,'CASC' from dual union all
select 30303,'CASC' from dual union all
select 30304,'CASC' from dual union all
select 30305,'CASC' from dual union all
select 30306,'CASC' from dual union all
select 30307,'CASC' from dual union all
select 30308,'CASC' from dual;
insert into SATELLITE_ORG
select 30309,'CASC' from dual union all
select 3031,'JPL' from dual union all
select 3031,'NASA' from dual union all
select 30310,'CASC' from dual union all
select 30311,'CASC' from dual union all
select 30312,'CASC' from dual union all
select 30313,'CASC' from dual union all
select 30314,'CASC' from dual union all
select 30315,'CASC' from dual union all
select 30316,'CASC' from dual union all
select 30317,'CASC' from dual union all
select 30318,'CASC' from dual union all
select 30319,'CASC' from dual union all
select 3032,'NASA' from dual union all
select 30320,'CASC' from dual union all
select 30321,'CASC' from dual union all
select 30322,'CASC' from dual union all
select 30323,'CNSA' from dual union all
select 30324,'CASC' from dual union all
select 30325,'CASC' from dual union all
select 30326,'CASC' from dual union all
select 30327,'CASC' from dual union all
select 30328,'CASC' from dual union all
select 30329,'CASC' from dual union all
select 3033,'NASA' from dual union all
select 30330,'CASC' from dual union all
select 30331,'CASC' from dual union all
select 30332,'CASC' from dual union all
select 30333,'CASC' from dual union all
select 30334,'CASC' from dual union all
select 30335,'CASC' from dual union all
select 30336,'CASC' from dual union all
select 30337,'CASC' from dual union all
select 30338,'CASC' from dual union all
select 30339,'CASC' from dual union all
select 3034,'NASA' from dual union all
select 30340,'CASC' from dual union all
select 30341,'CASC' from dual union all
select 30342,'CASC' from dual union all
select 30343,'CASC' from dual union all
select 30344,'CASC' from dual union all
select 30345,'CASC' from dual union all
select 30346,'CASC' from dual union all
select 30347,'CASC' from dual union all
select 30348,'CASC' from dual union all
select 30349,'CASC' from dual union all
select 3035,'ESSA' from dual union all
select 30350,'CASC' from dual union all
select 30351,'CASC' from dual union all
select 30352,'CASC' from dual union all
select 30353,'CASC' from dual union all
select 30354,'CASC' from dual union all
select 30355,'CASC' from dual union all
select 30356,'CASC' from dual union all
select 30357,'CASC' from dual union all
select 30358,'CASC' from dual union all
select 30359,'CASC' from dual union all
select 3036,'NASA' from dual union all
select 30360,'CASC' from dual union all
select 30361,'CASC' from dual union all
select 30362,'CASC' from dual union all
select 30363,'CASC' from dual union all
select 30364,'CASC' from dual union all
select 30365,'CASC' from dual union all
select 30366,'CASC' from dual union all
select 30367,'CASC' from dual union all
select 30368,'CASC' from dual union all
select 30369,'CASC' from dual union all
select 3037,'USAF' from dual union all
select 30370,'CASC' from dual union all
select 30371,'CASC' from dual union all
select 30372,'CASC' from dual union all
select 30373,'CASC' from dual union all
select 30374,'CASC' from dual union all
select 30375,'CASC' from dual union all
select 30376,'CASC' from dual union all
select 30377,'CASC' from dual union all
select 30378,'CASC' from dual union all
select 30379,'CASC' from dual union all
select 3038,'RVSN' from dual union all
select 30380,'CASC' from dual union all
select 30381,'CASC' from dual union all
select 30382,'CASC' from dual union all
select 30383,'CASC' from dual union all
select 30384,'CASC' from dual union all
select 30385,'CASC' from dual union all
select 30386,'CASC' from dual union all
select 30387,'CASC' from dual union all
select 30388,'CASC' from dual union all
select 30389,'CASC' from dual union all
select 3039,'USAF' from dual union all
select 30390,'CASC' from dual union all
select 30391,'CASC' from dual union all
select 30392,'CASC' from dual union all
select 30393,'CASC' from dual union all
select 30394,'CASC' from dual union all
select 30395,'CASC' from dual union all
select 30396,'CASC' from dual union all
select 30397,'CASC' from dual union all
select 30398,'CASC' from dual;
insert into SATELLITE_ORG
select 30399,'CASC' from dual union all
select 304,'CIA' from dual union all
select 304,'NRO' from dual union all
select 3040,'USAF' from dual union all
select 30400,'CASC' from dual union all
select 30401,'CASC' from dual union all
select 30402,'CASC' from dual union all
select 30403,'CASC' from dual union all
select 30404,'CASC' from dual union all
select 30405,'CASC' from dual union all
select 30406,'CASC' from dual union all
select 30407,'CASC' from dual union all
select 30408,'CASC' from dual union all
select 30409,'CASC' from dual union all
select 3041,'YUZH' from dual union all
select 30410,'CASC' from dual union all
select 30411,'CASC' from dual union all
select 30412,'CASC' from dual union all
select 30413,'CASC' from dual union all
select 30414,'CASC' from dual union all
select 30415,'CASC' from dual union all
select 30416,'CASC' from dual union all
select 30417,'CASC' from dual union all
select 30418,'CASC' from dual union all
select 30419,'CASC' from dual union all
select 3042,'YUZH' from dual union all
select 30420,'CASC' from dual union all
select 30421,'CASC' from dual union all
select 30422,'CASC' from dual union all
select 30423,'CASC' from dual union all
select 30424,'CASC' from dual union all
select 30425,'NASA' from dual union all
select 30426,'NASA' from dual union all
select 30427,'NASA' from dual union all
select 30428,'NASA' from dual union all
select 30429,'CASC' from dual union all
select 3043,'PVO' from dual union all
select 3043,'YUZH' from dual union all
select 30430,'CASC' from dual union all
select 30431,'CASC' from dual union all
select 30432,'CASC' from dual union all
select 30433,'CASC' from dual union all
select 30434,'CASC' from dual union all
select 30435,'CASC' from dual union all
select 30436,'CASC' from dual union all
select 30437,'CASC' from dual union all
select 30438,'CASC' from dual union all
select 30439,'CASC' from dual union all
select 3044,'RVSN' from dual union all
select 30440,'CASC' from dual union all
select 30441,'CASC' from dual union all
select 30442,'CASC' from dual union all
select 30443,'CASC' from dual union all
select 30444,'CASC' from dual union all
select 30445,'CASC' from dual union all
select 30446,'CASC' from dual union all
select 30447,'CASC' from dual union all
select 30448,'CASC' from dual union all
select 30449,'CASC' from dual union all
select 3045,'USAF' from dual union all
select 30450,'CASC' from dual union all
select 30451,'CASC' from dual union all
select 30452,'CASC' from dual union all
select 30453,'CASC' from dual union all
select 30454,'CASC' from dual union all
select 30455,'CASC' from dual union all
select 30456,'CASC' from dual union all
select 30457,'CASC' from dual union all
select 30458,'CASC' from dual union all
select 30459,'CASC' from dual union all
select 3046,'USAF' from dual union all
select 30460,'CASC' from dual union all
select 30461,'CASC' from dual union all
select 30462,'CASC' from dual union all
select 30463,'CASC' from dual union all
select 30464,'CASC' from dual union all
select 30465,'CASC' from dual union all
select 30466,'CASC' from dual union all
select 30467,'CASC' from dual union all
select 30468,'CASC' from dual union all
select 30469,'CASC' from dual union all
select 3047,'OKB10' from dual union all
select 30470,'CASC' from dual union all
select 30471,'CASC' from dual union all
select 30472,'CASC' from dual union all
select 30473,'CASC' from dual union all
select 30474,'CASC' from dual union all
select 30475,'CASC' from dual union all
select 30476,'CASC' from dual union all
select 30477,'CASC' from dual union all
select 30478,'CASC' from dual union all
select 30479,'CNSA' from dual union all
select 3048,'RVSN' from dual union all
select 30480,'CASC' from dual union all
select 30481,'CASC' from dual union all
select 30482,'CASC' from dual union all
select 30483,'CASC' from dual union all
select 30484,'CASC' from dual union all
select 30485,'CASC' from dual union all
select 30486,'CASC' from dual;
insert into SATELLITE_ORG
select 30487,'CASC' from dual union all
select 30488,'CASC' from dual union all
select 30489,'CASC' from dual union all
select 3049,'USAF' from dual union all
select 30490,'CASC' from dual union all
select 30491,'CASC' from dual union all
select 30492,'CASC' from dual union all
select 30493,'CASC' from dual union all
select 30494,'CASC' from dual union all
select 30495,'CASC' from dual union all
select 30496,'CASC' from dual union all
select 30497,'CASC' from dual union all
select 30498,'CASC' from dual union all
select 30499,'CASC' from dual union all
select 305,'OSCAR' from dual union all
select 3050,'USAF' from dual union all
select 30500,'CASC' from dual union all
select 30501,'CASC' from dual union all
select 30502,'CASC' from dual union all
select 30503,'CASC' from dual union all
select 30504,'CASC' from dual union all
select 30505,'CASC' from dual union all
select 30506,'CASC' from dual union all
select 30507,'CASC' from dual union all
select 30508,'CASC' from dual union all
select 30509,'CASC' from dual union all
select 3051,'NASA' from dual union all
select 30510,'CASC' from dual union all
select 30511,'CASC' from dual union all
select 30512,'CASC' from dual union all
select 30513,'CASC' from dual union all
select 30514,'CASC' from dual union all
select 30515,'CASC' from dual union all
select 30516,'CASC' from dual union all
select 30517,'CASC' from dual union all
select 30518,'CASC' from dual union all
select 30519,'CASC' from dual union all
select 3052,'MO' from dual union all
select 30520,'CASC' from dual union all
select 30521,'CASC' from dual union all
select 30522,'CASC' from dual union all
select 30523,'CASC' from dual union all
select 30524,'CASC' from dual union all
select 30525,'CASC' from dual union all
select 30526,'CASC' from dual union all
select 30527,'CASC' from dual union all
select 30528,'CASC' from dual union all
select 30529,'CASC' from dual union all
select 3053,'RVSN' from dual union all
select 30530,'CASC' from dual union all
select 30531,'CASC' from dual union all
select 30532,'CASC' from dual union all
select 30533,'CASC' from dual union all
select 30534,'CASC' from dual union all
select 30535,'CASC' from dual union all
select 30536,'CASC' from dual union all
select 30537,'CASC' from dual union all
select 30538,'CASC' from dual union all
select 30539,'CASC' from dual union all
select 3054,'WRE' from dual union all
select 30540,'CASC' from dual union all
select 30541,'CASC' from dual union all
select 30542,'CASC' from dual union all
select 30543,'CASC' from dual union all
select 30544,'CASC' from dual union all
select 30545,'CASC' from dual union all
select 30546,'CASC' from dual union all
select 30547,'CASC' from dual union all
select 30548,'CASC' from dual union all
select 30549,'CASC' from dual union all
select 3055,'MO' from dual union all
select 30550,'CASC' from dual union all
select 30551,'CASC' from dual union all
select 30552,'CASC' from dual union all
select 30553,'CASC' from dual union all
select 30554,'CASC' from dual union all
select 30555,'CASC' from dual union all
select 30556,'CASC' from dual union all
select 30557,'CASC' from dual union all
select 30558,'CASC' from dual union all
select 30559,'CASC' from dual union all
select 3056,'RVSN' from dual union all
select 30560,'CASC' from dual union all
select 30561,'CASC' from dual union all
select 30562,'CASC' from dual union all
select 30563,'CASC' from dual union all
select 30564,'CASC' from dual union all
select 30565,'CASC' from dual union all
select 30566,'CASC' from dual union all
select 30567,'CASC' from dual union all
select 30568,'CASC' from dual union all
select 30569,'CASC' from dual union all
select 3057,'USAF' from dual union all
select 30570,'CASC' from dual union all
select 30571,'CASC' from dual union all
select 30572,'CASC' from dual union all
select 30573,'CASC' from dual union all
select 30574,'CASC' from dual union all
select 30575,'CASC' from dual union all
select 30576,'CASC' from dual;
insert into SATELLITE_ORG
select 30577,'CASC' from dual union all
select 30578,'CASC' from dual union all
select 30579,'CASC' from dual union all
select 3058,'NRO' from dual union all
select 30580,'NASA' from dual union all
select 30581,'NASA' from dual union all
select 30582,'NASA' from dual union all
select 30583,'NASA' from dual union all
select 30584,'CASC' from dual union all
select 30585,'BOE' from dual union all
select 30586,'CSICE' from dual union all
select 30587,'CSICE' from dual union all
select 30588,'JAXA' from dual union all
select 30589,'JAXA' from dual union all
select 3059,'NASA' from dual union all
select 30590,'JAXA' from dual union all
select 30591,'JAXA' from dual union all
select 30592,'CASC' from dual union all
select 30593,'CASC' from dual union all
select 30594,'CASC' from dual union all
select 30595,'CASC' from dual union all
select 30596,'CASC' from dual union all
select 30597,'CASC' from dual union all
select 30598,'CASC' from dual union all
select 30599,'CASC' from dual union all
select 306,'USAF' from dual union all
select 3060,'USAF' from dual union all
select 30600,'CASC' from dual union all
select 30601,'CASC' from dual union all
select 30602,'CASC' from dual union all
select 30603,'CASC' from dual union all
select 30604,'CASC' from dual union all
select 30605,'CASC' from dual union all
select 30606,'CASC' from dual union all
select 30607,'CASC' from dual union all
select 30608,'CASC' from dual union all
select 30609,'CASC' from dual union all
select 3061,'USAF' from dual union all
select 30610,'CASC' from dual union all
select 30611,'CASC' from dual union all
select 30612,'CASC' from dual union all
select 30613,'CASC' from dual union all
select 30614,'CASC' from dual union all
select 30615,'CASC' from dual union all
select 30616,'CASC' from dual union all
select 30617,'CASC' from dual union all
select 30618,'CASC' from dual union all
select 30619,'CASC' from dual union all
select 3062,'USAF' from dual union all
select 30620,'CASC' from dual union all
select 30621,'CASC' from dual union all
select 30622,'CASC' from dual union all
select 30623,'CASC' from dual union all
select 30624,'CASC' from dual union all
select 30625,'CASC' from dual union all
select 30626,'CASC' from dual union all
select 30627,'CASC' from dual union all
select 30628,'CASC' from dual union all
select 30629,'CASC' from dual union all
select 3063,'CIA' from dual union all
select 3063,'NRO' from dual union all
select 30630,'CASC' from dual union all
select 30631,'CASC' from dual union all
select 30632,'CASC' from dual union all
select 30633,'CASC' from dual union all
select 30634,'CASC' from dual union all
select 30635,'CASC' from dual union all
select 30636,'CASC' from dual union all
select 30637,'CASC' from dual union all
select 30638,'CASC' from dual union all
select 30639,'CASC' from dual union all
select 3064,'USAF' from dual union all
select 30640,'CASC' from dual union all
select 30641,'CASC' from dual union all
select 30642,'CASC' from dual union all
select 30643,'CASC' from dual union all
select 30644,'CASC' from dual union all
select 30645,'CASC' from dual union all
select 30646,'CASC' from dual union all
select 30647,'CASC' from dual union all
select 30648,'CASC' from dual union all
select 30649,'CASC' from dual union all
select 3065,'USAF' from dual union all
select 30650,'CASC' from dual union all
select 30651,'JAXA' from dual union all
select 30652,'CASC' from dual union all
select 30653,'CASC' from dual union all
select 30654,'CASC' from dual union all
select 30655,'CASC' from dual union all
select 30656,'CASC' from dual union all
select 30657,'CASC' from dual union all
select 30658,'CASC' from dual union all
select 30659,'CASC' from dual union all
select 3066,'NASA' from dual union all
select 30660,'CASC' from dual union all
select 30661,'CASC' from dual union all
select 30662,'CASC' from dual union all
select 30663,'CASC' from dual union all
select 30664,'CASC' from dual union all
select 30665,'CASC' from dual;
insert into SATELLITE_ORG
select 30666,'CASC' from dual union all
select 30667,'CASC' from dual union all
select 30668,'CASC' from dual union all
select 30669,'CASC' from dual union all
select 3067,'NASA' from dual union all
select 30670,'CASC' from dual union all
select 30671,'CASC' from dual union all
select 30672,'CASC' from dual union all
select 30673,'CASC' from dual union all
select 30674,'CASC' from dual union all
select 30675,'CASC' from dual union all
select 30676,'CASC' from dual union all
select 30677,'CASC' from dual union all
select 30678,'CASC' from dual union all
select 30679,'CASC' from dual union all
select 3068,'NASA' from dual union all
select 30680,'CASC' from dual union all
select 30681,'CASC' from dual union all
select 30682,'CASC' from dual union all
select 30683,'CASC' from dual union all
select 30684,'CASC' from dual union all
select 30685,'CASC' from dual union all
select 30686,'CASC' from dual union all
select 30687,'CASC' from dual union all
select 30688,'CASC' from dual union all
select 30689,'CASC' from dual union all
select 3069,'NASA' from dual union all
select 30690,'CASC' from dual union all
select 30691,'CASC' from dual union all
select 30692,'CASC' from dual union all
select 30693,'CASC' from dual union all
select 30694,'CASC' from dual union all
select 30695,'CASC' from dual union all
select 30696,'CASC' from dual union all
select 30697,'CASC' from dual union all
select 30698,'CASC' from dual union all
select 30699,'CASC' from dual union all
select 307,'USAF' from dual union all
select 3070,'NASA' from dual union all
select 30700,'CASC' from dual union all
select 30701,'CASC' from dual union all
select 30702,'CASC' from dual union all
select 30703,'CASC' from dual union all
select 30704,'CASC' from dual union all
select 30705,'CASC' from dual union all
select 30706,'CASC' from dual union all
select 30707,'CASC' from dual union all
select 30708,'CASC' from dual union all
select 30709,'CASC' from dual union all
select 3071,'MO' from dual union all
select 30710,'CASC' from dual union all
select 30711,'CASC' from dual union all
select 30712,'CASC' from dual union all
select 30713,'CASC' from dual union all
select 30714,'CASC' from dual union all
select 30715,'CASC' from dual union all
select 30716,'CASC' from dual union all
select 30717,'CASC' from dual union all
select 30718,'CASC' from dual union all
select 30719,'CASC' from dual union all
select 3072,'RVSN' from dual union all
select 30720,'CASC' from dual union all
select 30721,'CASC' from dual union all
select 30722,'CASC' from dual union all
select 30723,'CASC' from dual union all
select 30724,'CASC' from dual union all
select 30725,'CASC' from dual union all
select 30726,'CASC' from dual union all
select 30727,'CASC' from dual union all
select 30728,'CASC' from dual union all
select 30729,'CASC' from dual union all
select 3073,'MO' from dual union all
select 30730,'CASC' from dual union all
select 30731,'CASC' from dual union all
select 30732,'CASC' from dual union all
select 30733,'CASC' from dual union all
select 30734,'CASC' from dual union all
select 30735,'CASC' from dual union all
select 30736,'CASC' from dual union all
select 30737,'CASC' from dual union all
select 30738,'CASC' from dual union all
select 30739,'CASC' from dual union all
select 3074,'YUZH' from dual union all
select 30740,'CASC' from dual union all
select 30741,'CASC' from dual union all
select 30742,'CASC' from dual union all
select 30743,'CASC' from dual union all
select 30744,'CASC' from dual union all
select 30745,'CASC' from dual union all
select 30746,'CASC' from dual union all
select 30747,'CASC' from dual union all
select 30748,'CASC' from dual union all
select 30749,'CASC' from dual union all
select 3075,'RVSN' from dual union all
select 30750,'CASC' from dual union all
select 30751,'CASC' from dual union all
select 30752,'CASC' from dual union all
select 30753,'CASC' from dual union all
select 30754,'CASC' from dual union all
select 30755,'CASC' from dual;
insert into SATELLITE_ORG
select 30756,'CASC' from dual union all
select 30757,'CASC' from dual union all
select 30758,'CASC' from dual union all
select 30759,'CASC' from dual union all
select 3076,'YUZH' from dual union all
select 30760,'CASC' from dual union all
select 30761,'CASC' from dual union all
select 30762,'CASC' from dual union all
select 30763,'CASC' from dual union all
select 30764,'CASC' from dual union all
select 30765,'CASC' from dual union all
select 30766,'CASC' from dual union all
select 30767,'CASC' from dual union all
select 30768,'CASC' from dual union all
select 30769,'CASC' from dual union all
select 3077,'YUZH' from dual union all
select 30770,'CASC' from dual union all
select 30771,'CASC' from dual union all
select 30772,'DOD' from dual union all
select 30773,'USAF' from dual union all
select 30773,'USN' from dual union all
select 30774,'DOD' from dual union all
select 30775,'USAF' from dual union all
select 30776,'USAF' from dual union all
select 30777,'USAF' from dual union all
select 30778,'LM' from dual union all
select 30779,'CNSA' from dual union all
select 3078,'YUZH' from dual union all
select 30780,'CNSA' from dual union all
select 30781,'CNSA' from dual union all
select 30782,'CNSA' from dual union all
select 30783,'CNSA' from dual union all
select 30784,'CNSA' from dual union all
select 30785,'CNSA' from dual union all
select 30786,'CNSA' from dual union all
select 30787,'CNSA' from dual union all
select 30788,'CNSA' from dual union all
select 30789,'CNSA' from dual union all
select 3079,'YUZH' from dual union all
select 30790,'CNSA' from dual union all
select 30791,'CNSA' from dual union all
select 30792,'CNSA' from dual union all
select 30793,'ISRO' from dual union all
select 30794,'PARA' from dual union all
select 30795,'AE' from dual union all
select 30796,'AE' from dual union all
select 30797,'NASA' from dual union all
select 30798,'NASA' from dual union all
select 30799,'BOE' from dual union all
select 308,'NRO' from dual union all
select 3080,'RVSN' from dual union all
select 30800,'CASC' from dual union all
select 30801,'CASC' from dual union all
select 30802,'CASC' from dual union all
select 30803,'CASC' from dual union all
select 30804,'CASC' from dual union all
select 30805,'CASC' from dual union all
select 30806,'CASC' from dual union all
select 30807,'CASC' from dual union all
select 30808,'CASC' from dual union all
select 30809,'CASC' from dual union all
select 3081,'VMF' from dual union all
select 30810,'CASC' from dual union all
select 30811,'CASC' from dual union all
select 30812,'CASC' from dual union all
select 30813,'CASC' from dual union all
select 30814,'CASC' from dual union all
select 30815,'CASC' from dual union all
select 30816,'CASC' from dual union all
select 30817,'CASC' from dual union all
select 30818,'CASC' from dual union all
select 30819,'CASC' from dual union all
select 3082,'VMF' from dual union all
select 30820,'CASC' from dual union all
select 30821,'CASC' from dual union all
select 30822,'CASC' from dual union all
select 30823,'CASC' from dual union all
select 30824,'CASC' from dual union all
select 30825,'CASC' from dual union all
select 30826,'CASC' from dual union all
select 30827,'CASC' from dual union all
select 30828,'CASC' from dual union all
select 30829,'CASC' from dual union all
select 3083,'RVSN' from dual union all
select 30830,'CASC' from dual union all
select 30831,'CASC' from dual union all
select 30832,'CASC' from dual union all
select 30833,'CASC' from dual union all
select 30834,'CASC' from dual union all
select 30835,'CASC' from dual union all
select 30836,'CASC' from dual union all
select 30837,'CASC' from dual union all
select 30838,'CASC' from dual union all
select 30839,'CASC' from dual union all
select 3084,'RVSN' from dual union all
select 30840,'CASC' from dual union all
select 30841,'CASC' from dual union all
select 30842,'CASC' from dual union all
select 30843,'CASC' from dual union all
select 30844,'CASC' from dual;
insert into SATELLITE_ORG
select 30845,'CASC' from dual union all
select 30846,'CASC' from dual union all
select 30847,'CASC' from dual union all
select 30848,'CASC' from dual union all
select 30849,'CASC' from dual union all
select 3085,'RVSN' from dual union all
select 30850,'CASC' from dual union all
select 30851,'CASC' from dual union all
select 30852,'CASC' from dual union all
select 30853,'CASC' from dual union all
select 30854,'CASC' from dual union all
select 30855,'CASC' from dual union all
select 30856,'CASC' from dual union all
select 30857,'CASC' from dual union all
select 30858,'CASC' from dual union all
select 30859,'CASC' from dual union all
select 3086,'RVSN' from dual union all
select 30860,'CASC' from dual union all
select 30861,'CASC' from dual union all
select 30862,'CASC' from dual union all
select 30863,'CASC' from dual union all
select 30864,'CASC' from dual union all
select 30865,'CASC' from dual union all
select 30866,'CASC' from dual union all
select 30867,'CASC' from dual union all
select 30868,'CASC' from dual union all
select 30869,'CASC' from dual union all
select 3087,'RVSN' from dual union all
select 30870,'CASC' from dual union all
select 30871,'CASC' from dual union all
select 30872,'CASC' from dual union all
select 30873,'CASC' from dual union all
select 30874,'CASC' from dual union all
select 30875,'CASC' from dual union all
select 30876,'CASC' from dual union all
select 30877,'CASC' from dual union all
select 30878,'CASC' from dual union all
select 30879,'CASC' from dual union all
select 3088,'VMF' from dual union all
select 30880,'CASC' from dual union all
select 30881,'CASC' from dual union all
select 30882,'CASC' from dual union all
select 30883,'CASC' from dual union all
select 30884,'CASC' from dual union all
select 30885,'CASC' from dual union all
select 30886,'CASC' from dual union all
select 30887,'CASC' from dual union all
select 30888,'CASC' from dual union all
select 30889,'CASC' from dual union all
select 3089,'RVSN' from dual union all
select 30890,'CASC' from dual union all
select 30891,'CASC' from dual union all
select 30892,'CASC' from dual union all
select 30893,'CASC' from dual union all
select 30894,'CASC' from dual union all
select 30895,'CASC' from dual union all
select 30896,'CASC' from dual union all
select 30897,'CASC' from dual union all
select 30898,'CASC' from dual union all
select 30899,'CASC' from dual union all
select 309,'NASA' from dual union all
select 3090,'RVSN' from dual union all
select 30900,'CASC' from dual union all
select 30901,'CASC' from dual union all
select 30902,'CASC' from dual union all
select 30903,'CASC' from dual union all
select 30904,'CASC' from dual union all
select 30905,'CASC' from dual union all
select 30906,'CASC' from dual union all
select 30907,'CASC' from dual union all
select 30908,'CASC' from dual union all
select 30909,'CASC' from dual union all
select 3091,'JPL' from dual union all
select 3091,'NASA' from dual union all
select 30910,'CASC' from dual union all
select 30911,'CASC' from dual union all
select 30912,'CASC' from dual union all
select 30913,'CASC' from dual union all
select 30914,'CASC' from dual union all
select 30915,'CASC' from dual union all
select 30916,'CASC' from dual union all
select 30917,'CASC' from dual union all
select 30918,'CASC' from dual union all
select 30919,'CASC' from dual union all
select 3092,'NASA' from dual union all
select 30920,'CASC' from dual union all
select 30921,'CASC' from dual union all
select 30922,'CASC' from dual union all
select 30923,'CASC' from dual union all
select 30924,'CASC' from dual union all
select 30925,'CASC' from dual union all
select 30926,'CASC' from dual union all
select 30927,'CASC' from dual union all
select 30928,'CASC' from dual union all
select 30929,'CASC' from dual union all
select 3093,'NASA' from dual union all
select 30930,'CASC' from dual union all
select 30931,'CASC' from dual union all
select 30932,'CASC' from dual union all
select 30933,'CASC' from dual;
insert into SATELLITE_ORG
select 30934,'CASC' from dual union all
select 30935,'CASC' from dual union all
select 30936,'CASC' from dual union all
select 30937,'CASC' from dual union all
select 30938,'CASC' from dual union all
select 30939,'CASC' from dual union all
select 3094,'NASA' from dual union all
select 30940,'CASC' from dual union all
select 30941,'CASC' from dual union all
select 30942,'CASC' from dual union all
select 30943,'CASC' from dual union all
select 30944,'CASC' from dual union all
select 30945,'CASC' from dual union all
select 30946,'CASC' from dual union all
select 30947,'CASC' from dual union all
select 30948,'CASC' from dual union all
select 30949,'CASC' from dual union all
select 3095,'METP' from dual union all
select 30950,'CASC' from dual union all
select 30951,'CASC' from dual union all
select 30952,'CASC' from dual union all
select 30953,'CASC' from dual union all
select 30954,'CASC' from dual union all
select 30955,'CASC' from dual union all
select 30956,'CASC' from dual union all
select 30957,'CASC' from dual union all
select 30958,'CASC' from dual union all
select 30959,'CASC' from dual union all
select 3096,'RVSN' from dual union all
select 30960,'CASC' from dual union all
select 30961,'CASC' from dual union all
select 30962,'CASC' from dual union all
select 30963,'CASC' from dual union all
select 30964,'CASC' from dual union all
select 30965,'CASC' from dual union all
select 30966,'CASC' from dual union all
select 30967,'CASC' from dual union all
select 30968,'CASC' from dual union all
select 30969,'CASC' from dual union all
select 3097,'NRO' from dual union all
select 30970,'CASC' from dual union all
select 30971,'CASC' from dual union all
select 30972,'CASC' from dual union all
select 30973,'CASC' from dual union all
select 30974,'CASC' from dual union all
select 30975,'CASC' from dual union all
select 30976,'CASC' from dual union all
select 30977,'CASC' from dual union all
select 30978,'CASC' from dual union all
select 30979,'CASC' from dual union all
select 3098,'NRO' from dual union all
select 30980,'CASC' from dual union all
select 30981,'CASC' from dual union all
select 30982,'CASC' from dual union all
select 30983,'CASC' from dual union all
select 30984,'CASC' from dual union all
select 30985,'CASC' from dual union all
select 30986,'CASC' from dual union all
select 30987,'CASC' from dual union all
select 30988,'CASC' from dual union all
select 30989,'CASC' from dual union all
select 3099,'MO' from dual union all
select 30990,'CASC' from dual union all
select 30991,'CASC' from dual union all
select 30992,'CASC' from dual union all
select 30993,'CASC' from dual union all
select 30994,'CASC' from dual union all
select 30995,'CASC' from dual union all
select 30996,'CASC' from dual union all
select 30997,'CASC' from dual union all
select 30998,'CASC' from dual union all
select 30999,'CASC' from dual union all
select 31,'DOD' from dual union all
select 31,'USN' from dual union all
select 310,'NASA' from dual union all
select 3100,'TSN108' from dual union all
select 31000,'CASC' from dual union all
select 31001,'CASC' from dual union all
select 31002,'CASC' from dual union all
select 31003,'CASC' from dual union all
select 31004,'CASC' from dual union all
select 31005,'CASC' from dual union all
select 31006,'CASC' from dual union all
select 31007,'CASC' from dual union all
select 31008,'CASC' from dual union all
select 31009,'CASC' from dual union all
select 3101,'USAF' from dual union all
select 31010,'CASC' from dual union all
select 31011,'CASC' from dual union all
select 31012,'CASC' from dual union all
select 31013,'CASC' from dual union all
select 31014,'CASC' from dual union all
select 31015,'CASC' from dual union all
select 31016,'CASC' from dual union all
select 31017,'CASC' from dual union all
select 31018,'CASC' from dual union all
select 31019,'CASC' from dual union all
select 3102,'RVSN' from dual union all
select 31020,'CASC' from dual union all
select 31021,'CASC' from dual;
insert into SATELLITE_ORG
select 31022,'CASC' from dual union all
select 31023,'CASC' from dual union all
select 31024,'CASC' from dual union all
select 31025,'CASC' from dual union all
select 31026,'CASC' from dual union all
select 31027,'CASC' from dual union all
select 31028,'CASC' from dual union all
select 31029,'CASC' from dual union all
select 3103,'YUZH' from dual union all
select 31030,'CASC' from dual union all
select 31031,'CASC' from dual union all
select 31032,'CASC' from dual union all
select 31033,'CASC' from dual union all
select 31034,'CASC' from dual union all
select 31035,'CASC' from dual union all
select 31036,'CASC' from dual union all
select 31037,'CASC' from dual union all
select 31038,'CASC' from dual union all
select 31039,'CASC' from dual union all
select 3104,'YUZH' from dual union all
select 31040,'CASC' from dual union all
select 31041,'CASC' from dual union all
select 31042,'CASC' from dual union all
select 31043,'CASC' from dual union all
select 31044,'CASC' from dual union all
select 31045,'CASC' from dual union all
select 31046,'CASC' from dual union all
select 31047,'CASC' from dual union all
select 31048,'CASC' from dual union all
select 31049,'CASC' from dual union all
select 3105,'USAF' from dual union all
select 31050,'CASC' from dual union all
select 31051,'CASC' from dual union all
select 31052,'CASC' from dual union all
select 31053,'CASC' from dual union all
select 31054,'CASC' from dual union all
select 31055,'CASC' from dual union all
select 31056,'CASC' from dual union all
select 31057,'CASC' from dual union all
select 31058,'CASC' from dual union all
select 31059,'CASC' from dual union all
select 3106,'NASA' from dual union all
select 31060,'CASC' from dual union all
select 31061,'CASC' from dual union all
select 31062,'CASC' from dual union all
select 31063,'CASC' from dual union all
select 31064,'CASC' from dual union all
select 31065,'CASC' from dual union all
select 31066,'CASC' from dual union all
select 31067,'CASC' from dual union all
select 31068,'CASC' from dual union all
select 31069,'CASC' from dual union all
select 3107,'NASA' from dual union all
select 31070,'CASC' from dual union all
select 31071,'CASC' from dual union all
select 31072,'CASC' from dual union all
select 31073,'CASC' from dual union all
select 31074,'CASC' from dual union all
select 31075,'CASC' from dual union all
select 31076,'CASC' from dual union all
select 31077,'CASC' from dual union all
select 31078,'CASC' from dual union all
select 31079,'CASC' from dual union all
select 3108,'NASA' from dual union all
select 31080,'CASC' from dual union all
select 31081,'CASC' from dual union all
select 31082,'CASC' from dual union all
select 31083,'CASC' from dual union all
select 31084,'CASC' from dual union all
select 31085,'CASC' from dual union all
select 31086,'CASC' from dual union all
select 31087,'CASC' from dual union all
select 31088,'CASC' from dual union all
select 31089,'CASC' from dual union all
select 3109,'YUZH' from dual union all
select 31090,'CASC' from dual union all
select 31091,'CASC' from dual union all
select 31092,'CASC' from dual union all
select 31093,'CASC' from dual union all
select 31094,'CASC' from dual union all
select 31095,'CASC' from dual union all
select 31096,'CASC' from dual union all
select 31097,'CASC' from dual union all
select 31098,'CASC' from dual union all
select 31099,'CASC' from dual union all
select 311,'NASA' from dual union all
select 3110,'YUZH' from dual union all
select 31100,'FKA' from dual union all
select 31101,'KVR' from dual union all
select 31102,'TCAN' from dual union all
select 31103,'KHRR' from dual union all
select 31104,'KHRR' from dual union all
select 31105,'JAXA' from dual union all
select 31106,'JAXA' from dual union all
select 31107,'JAXA' from dual union all
select 31108,'JAXA' from dual union all
select 31109,'JAXA' from dual union all
select 3111,'YUZH' from dual union all
select 31110,'JAXA' from dual union all
select 31111,'JAXA' from dual;
insert into SATELLITE_ORG
select 31112,'JAXA' from dual union all
select 31113,'CASC' from dual union all
select 31113,'GYZ' from dual union all
select 31114,'CASC' from dual union all
select 31115,'CNSA' from dual union all
select 31116,'CASC' from dual union all
select 31117,'NARSS' from dual union all
select 31118,'KACST' from dual union all
select 31119,'KACST' from dual union all
select 3112,'YUZH' from dual union all
select 31120,'KTRAS' from dual union all
select 31121,'KACST' from dual union all
select 31122,'BOE' from dual union all
select 31123,'KTRAS' from dual union all
select 31124,'KACST' from dual union all
select 31125,'KACST' from dual union all
select 31126,'TUI' from dual union all
select 31127,'KACST' from dual union all
select 31128,'USERAR' from dual union all
select 31129,'CALP' from dual union all
select 3113,'CIA' from dual union all
select 3113,'NRO' from dual union all
select 31130,'ULLAF' from dual union all
select 31131,'CNSA' from dual union all
select 31132,'CALP' from dual union all
select 31133,'AERO' from dual union all
select 31134,'USAF' from dual union all
select 31135,'ASI' from dual union all
select 31136,'ISRO' from dual union all
select 31137,'ESA' from dual union all
select 31138,'RCA' from dual union all
select 31139,'CNSA' from dual union all
select 3114,'NRO' from dual union all
select 31140,'MDA' from dual union all
select 31141,'OSC' from dual union all
select 31142,'NASA' from dual union all
select 31143,'BOE' from dual union all
select 31144,'CASC' from dual union all
select 31145,'CASC' from dual union all
select 31146,'CASC' from dual union all
select 31147,'CASC' from dual union all
select 31148,'CASC' from dual union all
select 31149,'CASC' from dual union all
select 3115,'MO' from dual union all
select 31150,'CASC' from dual union all
select 31151,'CASC' from dual union all
select 31152,'CASC' from dual union all
select 31153,'CASC' from dual union all
select 31154,'CASC' from dual union all
select 31155,'CASC' from dual union all
select 31156,'CASC' from dual union all
select 31157,'CASC' from dual union all
select 31158,'CASC' from dual union all
select 31159,'CASC' from dual union all
select 3116,'USAF' from dual union all
select 31160,'CASC' from dual union all
select 31161,'CASC' from dual union all
select 31162,'CASC' from dual union all
select 31163,'CASC' from dual union all
select 31164,'CASC' from dual union all
select 31165,'CASC' from dual union all
select 31166,'CASC' from dual union all
select 31167,'CASC' from dual union all
select 31168,'CASC' from dual union all
select 31169,'CASC' from dual union all
select 3117,'MO' from dual union all
select 31170,'CASC' from dual union all
select 31171,'CASC' from dual union all
select 31172,'CASC' from dual union all
select 31173,'CASC' from dual union all
select 31174,'CASC' from dual union all
select 31175,'CASC' from dual union all
select 31176,'CASC' from dual union all
select 31177,'CASC' from dual union all
select 31178,'CASC' from dual union all
select 31179,'CASC' from dual union all
select 3118,'MO' from dual union all
select 31180,'CASC' from dual union all
select 31181,'CASC' from dual union all
select 31182,'CASC' from dual union all
select 31183,'CASC' from dual union all
select 31184,'CASC' from dual union all
select 31185,'CASC' from dual union all
select 31186,'CASC' from dual union all
select 31187,'CASC' from dual union all
select 31188,'CASC' from dual union all
select 31189,'CASC' from dual union all
select 3119,'RVSN' from dual union all
select 31190,'CASC' from dual union all
select 31191,'CASC' from dual union all
select 31192,'CASC' from dual union all
select 31193,'CASC' from dual union all
select 31194,'CASC' from dual union all
select 31195,'CASC' from dual union all
select 31196,'CASC' from dual union all
select 31197,'CASC' from dual union all
select 31198,'CASC' from dual union all
select 31199,'CASC' from dual union all
select 312,'NASA' from dual union all
select 3120,'CIA' from dual;
insert into SATELLITE_ORG
select 3120,'NRO' from dual union all
select 31200,'CASC' from dual union all
select 31201,'CASC' from dual union all
select 31202,'CASC' from dual union all
select 31203,'CASC' from dual union all
select 31204,'CASC' from dual union all
select 31205,'CASC' from dual union all
select 31206,'CASC' from dual union all
select 31207,'CASC' from dual union all
select 31208,'CASC' from dual union all
select 31209,'CASC' from dual union all
select 3121,'USAF' from dual union all
select 31210,'CASC' from dual union all
select 31211,'CASC' from dual union all
select 31212,'CASC' from dual union all
select 31213,'CASC' from dual union all
select 31214,'CASC' from dual union all
select 31215,'CASC' from dual union all
select 31216,'CASC' from dual union all
select 31217,'CASC' from dual union all
select 31218,'CASC' from dual union all
select 31219,'CASC' from dual union all
select 3122,'USN' from dual union all
select 31220,'CASC' from dual union all
select 31221,'CASC' from dual union all
select 31222,'CASC' from dual union all
select 31223,'CASC' from dual union all
select 31224,'CASC' from dual union all
select 31225,'CASC' from dual union all
select 31226,'CASC' from dual union all
select 31227,'CASC' from dual union all
select 31228,'CASC' from dual union all
select 31229,'CASC' from dual union all
select 3123,'NASA' from dual union all
select 31230,'CASC' from dual union all
select 31231,'CASC' from dual union all
select 31232,'CASC' from dual union all
select 31233,'CASC' from dual union all
select 31234,'CASC' from dual union all
select 31235,'CASC' from dual union all
select 31236,'CASC' from dual union all
select 31237,'CASC' from dual union all
select 31238,'CASC' from dual union all
select 31239,'CASC' from dual union all
select 3124,'USAF' from dual union all
select 31240,'CASC' from dual union all
select 31241,'CASC' from dual union all
select 31242,'CASC' from dual union all
select 31243,'CASC' from dual union all
select 31244,'CASC' from dual union all
select 31245,'CASC' from dual union all
select 31246,'CASC' from dual union all
select 31247,'CASC' from dual union all
select 31248,'CASC' from dual union all
select 31249,'CASC' from dual union all
select 3125,'USAF' from dual union all
select 31250,'CASC' from dual union all
select 31251,'CASC' from dual union all
select 31252,'CASC' from dual union all
select 31253,'CASC' from dual union all
select 31254,'CASC' from dual union all
select 31255,'CASC' from dual union all
select 31256,'CASC' from dual union all
select 31257,'CASC' from dual union all
select 31258,'CASC' from dual union all
select 31259,'CASC' from dual union all
select 3126,'NASA' from dual union all
select 31260,'CASC' from dual union all
select 31261,'CASC' from dual union all
select 31262,'CASC' from dual union all
select 31263,'CASC' from dual union all
select 31264,'CASC' from dual union all
select 31265,'CASC' from dual union all
select 31266,'CASC' from dual union all
select 31267,'CASC' from dual union all
select 31268,'CASC' from dual union all
select 31269,'CASC' from dual union all
select 3127,'NASA' from dual union all
select 31270,'CASC' from dual union all
select 31271,'CASC' from dual union all
select 31272,'CASC' from dual union all
select 31273,'CASC' from dual union all
select 31274,'CASC' from dual union all
select 31275,'CASC' from dual union all
select 31276,'CASC' from dual union all
select 31277,'CASC' from dual union all
select 31278,'CASC' from dual union all
select 31279,'CASC' from dual union all
select 3128,'YUZH' from dual union all
select 31280,'CASC' from dual union all
select 31281,'CASC' from dual union all
select 31282,'CASC' from dual union all
select 31283,'CASC' from dual union all
select 31284,'CASC' from dual union all
select 31285,'CASC' from dual union all
select 31286,'CASC' from dual union all
select 31287,'CASC' from dual union all
select 31288,'CASC' from dual union all
select 31289,'CASC' from dual union all
select 3129,'OKB10' from dual;
insert into SATELLITE_ORG
select 31290,'CASC' from dual union all
select 31291,'CASC' from dual union all
select 31292,'CASC' from dual union all
select 31293,'CASC' from dual union all
select 31294,'CASC' from dual union all
select 31295,'CASC' from dual union all
select 31296,'CASC' from dual union all
select 31297,'CASC' from dual union all
select 31298,'CASC' from dual union all
select 31299,'CASC' from dual union all
select 313,'NASA' from dual union all
select 3130,'RVSN' from dual union all
select 31300,'CASC' from dual union all
select 31301,'CASC' from dual union all
select 31302,'CASC' from dual union all
select 31303,'CASC' from dual union all
select 31304,'NASA' from dual union all
select 31305,'OSC' from dual union all
select 31306,'SES' from dual union all
select 31307,'INTELB' from dual union all
select 31308,'AE' from dual union all
select 31309,'AE' from dual union all
select 3131,'RVSN' from dual union all
select 31310,'CASC' from dual union all
select 31311,'CASC' from dual union all
select 31312,'CASC' from dual union all
select 31313,'CASC' from dual union all
select 31314,'CASC' from dual union all
select 31315,'CASC' from dual union all
select 31316,'CASC' from dual union all
select 31317,'CASC' from dual union all
select 31318,'CASC' from dual union all
select 31319,'CASC' from dual union all
select 3132,'USAF' from dual union all
select 31320,'CASC' from dual union all
select 31321,'CASC' from dual union all
select 31322,'CASC' from dual union all
select 31323,'CASC' from dual union all
select 31324,'CASC' from dual union all
select 31325,'CASC' from dual union all
select 31326,'CASC' from dual union all
select 31327,'CASC' from dual union all
select 31328,'CASC' from dual union all
select 31329,'CASC' from dual union all
select 3133,'USN' from dual union all
select 31330,'CASC' from dual union all
select 31331,'CASC' from dual union all
select 31332,'CASC' from dual union all
select 31333,'CASC' from dual union all
select 31334,'CASC' from dual union all
select 31335,'CASC' from dual union all
select 31336,'CASC' from dual union all
select 31337,'CASC' from dual union all
select 31338,'CASC' from dual union all
select 31339,'CASC' from dual union all
select 3134,'TSKBEM' from dual union all
select 31340,'CASC' from dual union all
select 31341,'CASC' from dual union all
select 31342,'CASC' from dual union all
select 31343,'CASC' from dual union all
select 31344,'CASC' from dual union all
select 31345,'CASC' from dual union all
select 31346,'CASC' from dual union all
select 31347,'CASC' from dual union all
select 31348,'CASC' from dual union all
select 31349,'CASC' from dual union all
select 3135,'RVSN' from dual union all
select 31350,'CASC' from dual union all
select 31351,'CASC' from dual union all
select 31352,'CASC' from dual union all
select 31353,'CASC' from dual union all
select 31354,'CASC' from dual union all
select 31355,'CASC' from dual union all
select 31356,'CASC' from dual union all
select 31357,'CASC' from dual union all
select 31358,'CASC' from dual union all
select 31359,'CASC' from dual union all
select 3136,'RVSN' from dual union all
select 31360,'CASC' from dual union all
select 31361,'CASC' from dual union all
select 31362,'CASC' from dual union all
select 31363,'CASC' from dual union all
select 31364,'CASC' from dual union all
select 31365,'CASC' from dual union all
select 31366,'CASC' from dual union all
select 31367,'CASC' from dual union all
select 31368,'CASC' from dual union all
select 31369,'CASC' from dual union all
select 3137,'NASA' from dual union all
select 31370,'CASC' from dual union all
select 31371,'CASC' from dual union all
select 31372,'CASC' from dual union all
select 31373,'CASC' from dual union all
select 31374,'CASC' from dual union all
select 31375,'CASC' from dual union all
select 31376,'CASC' from dual union all
select 31377,'CASC' from dual union all
select 31378,'CASC' from dual union all
select 31379,'CASC' from dual union all
select 3138,'NASA' from dual;
insert into SATELLITE_ORG
select 31380,'CASC' from dual union all
select 31381,'CASC' from dual union all
select 31382,'CASC' from dual union all
select 31383,'CASC' from dual union all
select 31384,'CASC' from dual union all
select 31385,'CASC' from dual union all
select 31386,'CASC' from dual union all
select 31387,'CASC' from dual union all
select 31388,'CASC' from dual union all
select 31389,'CASC' from dual union all
select 3139,'PVO' from dual union all
select 3139,'YUZH' from dual union all
select 31390,'JAXA' from dual union all
select 31391,'KTRAS' from dual union all
select 31392,'KTRAS' from dual union all
select 31393,'FKA' from dual union all
select 31394,'KVR' from dual union all
select 31395,'NGCOM' from dual union all
select 31396,'CASC' from dual union all
select 31397,'KVR' from dual union all
select 31398,'KVR' from dual union all
select 31399,'KVR' from dual union all
select 314,'USAF' from dual union all
select 3140,'MO' from dual union all
select 31400,'KVR' from dual union all
select 31401,'KVR' from dual union all
select 31402,'KVR' from dual union all
select 31403,'ISRO' from dual union all
select 31404,'KVR' from dual union all
select 31405,'NASA' from dual union all
select 31406,'KVR' from dual union all
select 31407,'USN' from dual union all
select 31408,'JPL' from dual union all
select 31408,'NASA' from dual union all
select 31409,'CASC' from dual union all
select 3141,'USN' from dual union all
select 31410,'CASC' from dual union all
select 31411,'CASC' from dual union all
select 31412,'CASC' from dual union all
select 31413,'CASC' from dual union all
select 31414,'CASC' from dual union all
select 31415,'CASC' from dual union all
select 31416,'CASC' from dual union all
select 31417,'CASC' from dual union all
select 31418,'CASC' from dual union all
select 31419,'CASC' from dual union all
select 3142,'RVSN' from dual union all
select 31420,'CASC' from dual union all
select 31421,'CASC' from dual union all
select 31422,'CASC' from dual union all
select 31423,'CASC' from dual union all
select 31424,'CASC' from dual union all
select 31425,'CASC' from dual union all
select 31426,'CASC' from dual union all
select 31427,'CASC' from dual union all
select 31428,'CASC' from dual union all
select 31429,'CASC' from dual union all
select 3143,'RVSN' from dual union all
select 31430,'CASC' from dual union all
select 31431,'CASC' from dual union all
select 31432,'CASC' from dual union all
select 31433,'CASC' from dual union all
select 31434,'CASC' from dual union all
select 31435,'CASC' from dual union all
select 31436,'CASC' from dual union all
select 31437,'CASC' from dual union all
select 31438,'CASC' from dual union all
select 31439,'CASC' from dual union all
select 3144,'TSKBEM' from dual union all
select 31440,'CASC' from dual union all
select 31441,'CASC' from dual union all
select 31442,'CASC' from dual union all
select 31443,'CASC' from dual union all
select 31444,'CASC' from dual union all
select 31445,'CASC' from dual union all
select 31446,'CASC' from dual union all
select 31447,'CASC' from dual union all
select 31448,'CASC' from dual union all
select 31449,'CASC' from dual union all
select 3145,'NASA' from dual union all
select 31450,'CASC' from dual union all
select 31451,'CASC' from dual union all
select 31452,'CASC' from dual union all
select 31453,'CASC' from dual union all
select 31454,'CASC' from dual union all
select 31455,'CASC' from dual union all
select 31456,'CASC' from dual union all
select 31457,'CASC' from dual union all
select 31458,'CASC' from dual union all
select 31459,'CASC' from dual union all
select 3146,'NASA' from dual union all
select 31460,'CASC' from dual union all
select 31461,'CASC' from dual union all
select 31462,'CASC' from dual union all
select 31463,'CASC' from dual union all
select 31464,'CASC' from dual union all
select 31465,'CASC' from dual union all
select 31466,'CASC' from dual union all
select 31467,'CASC' from dual union all
select 31468,'CASC' from dual;
insert into SATELLITE_ORG
select 31469,'CASC' from dual union all
select 3147,'OKB10' from dual union all
select 31470,'CASC' from dual union all
select 31471,'CASC' from dual union all
select 31472,'CASC' from dual union all
select 31473,'CASC' from dual union all
select 31474,'CASC' from dual union all
select 31475,'CASC' from dual union all
select 31476,'CASC' from dual union all
select 31477,'CASC' from dual union all
select 31478,'CASC' from dual union all
select 31479,'CASC' from dual union all
select 3148,'NRO' from dual union all
select 31480,'CASC' from dual union all
select 31481,'CASC' from dual union all
select 31482,'CASC' from dual union all
select 31483,'CASC' from dual union all
select 31484,'CASC' from dual union all
select 31485,'CASC' from dual union all
select 31486,'CASC' from dual union all
select 31487,'CASC' from dual union all
select 31488,'CASC' from dual union all
select 31489,'CASC' from dual union all
select 3149,'USAF' from dual union all
select 31490,'PLA' from dual union all
select 31491,'CASC' from dual union all
select 31492,'CASC' from dual union all
select 31493,'CASC' from dual union all
select 31494,'CASC' from dual union all
select 31495,'CASC' from dual union all
select 31496,'CASC' from dual union all
select 31497,'CASC' from dual union all
select 31498,'CASC' from dual union all
select 31499,'CASC' from dual union all
select 315,'CIA' from dual union all
select 315,'NRO' from dual union all
select 3150,'METP' from dual union all
select 31500,'CASC' from dual union all
select 31501,'CASC' from dual union all
select 31502,'CASC' from dual union all
select 31503,'CASC' from dual union all
select 31504,'CASC' from dual union all
select 31505,'CASC' from dual union all
select 31506,'CASC' from dual union all
select 31507,'CASC' from dual union all
select 31508,'CASC' from dual union all
select 31509,'CASC' from dual union all
select 3151,'RVSN' from dual union all
select 31510,'CASC' from dual union all
select 31511,'CASC' from dual union all
select 31512,'CASC' from dual union all
select 31513,'CASC' from dual union all
select 31514,'CASC' from dual union all
select 31515,'CASC' from dual union all
select 31516,'CASC' from dual union all
select 31517,'CASC' from dual union all
select 31518,'CASC' from dual union all
select 31519,'CASC' from dual union all
select 3152,'CIA' from dual union all
select 3152,'NRO' from dual union all
select 31520,'CASC' from dual union all
select 31521,'CASC' from dual union all
select 31522,'CASC' from dual union all
select 31523,'CASC' from dual union all
select 31524,'CASC' from dual union all
select 31525,'CASC' from dual union all
select 31526,'CASC' from dual union all
select 31527,'CASC' from dual union all
select 31528,'CASC' from dual union all
select 31529,'CASC' from dual union all
select 3153,'NRO' from dual union all
select 31530,'CASC' from dual union all
select 31531,'CASC' from dual union all
select 31532,'CASC' from dual union all
select 31533,'CASC' from dual union all
select 31534,'CASC' from dual union all
select 31535,'CASC' from dual union all
select 31536,'CASC' from dual union all
select 31537,'CASC' from dual union all
select 31538,'CASC' from dual union all
select 31539,'CASC' from dual union all
select 3154,'MO' from dual union all
select 31540,'CASC' from dual union all
select 31541,'CASC' from dual union all
select 31542,'CASC' from dual union all
select 31543,'CASC' from dual union all
select 31544,'CASC' from dual union all
select 31545,'CASC' from dual union all
select 31546,'CASC' from dual union all
select 31547,'CASC' from dual union all
select 31548,'CASC' from dual union all
select 31549,'CASC' from dual union all
select 3155,'RVSN' from dual union all
select 31550,'CASC' from dual union all
select 31551,'CASC' from dual union all
select 31552,'CASC' from dual union all
select 31553,'CASC' from dual union all
select 31554,'CASC' from dual union all
select 31555,'CASC' from dual union all
select 31556,'CASC' from dual;
insert into SATELLITE_ORG
select 31557,'CASC' from dual union all
select 31558,'CASC' from dual union all
select 31559,'CASC' from dual union all
select 3156,'TSKBEM' from dual union all
select 31560,'CASC' from dual union all
select 31561,'CASC' from dual union all
select 31562,'CASC' from dual union all
select 31563,'CASC' from dual union all
select 31564,'CASC' from dual union all
select 31565,'CASC' from dual union all
select 31566,'CASC' from dual union all
select 31567,'CASC' from dual union all
select 31568,'CASC' from dual union all
select 31569,'CASC' from dual union all
select 3157,'RVSN' from dual union all
select 31570,'CASC' from dual union all
select 31571,'GLOB' from dual union all
select 31572,'KVR' from dual union all
select 31573,'GLOB' from dual union all
select 31574,'GLOB' from dual union all
select 31575,'KVR' from dual union all
select 31576,'GLOB' from dual union all
select 31577,'SINO' from dual union all
select 31578,'CASC' from dual union all
select 31579,'NASA' from dual union all
select 3158,'VMF' from dual union all
select 31580,'RGMS' from dual union all
select 31581,'CNSA' from dual union all
select 31582,'CNSA' from dual union all
select 31583,'CNSA' from dual union all
select 31584,'CNSA' from dual union all
select 31585,'CNSA' from dual union all
select 31586,'CNSA' from dual union all
select 31587,'CNSA' from dual union all
select 31588,'CNSA' from dual union all
select 31589,'CNSA' from dual union all
select 3159,'USAF' from dual union all
select 31590,'CNSA' from dual union all
select 31591,'CNSA' from dual union all
select 31592,'CNSA' from dual union all
select 31593,'CNSA' from dual union all
select 31594,'CNSA' from dual union all
select 31595,'KVR' from dual union all
select 31596,'KVR' from dual union all
select 31597,'KVR' from dual union all
select 31598,'ASI' from dual union all
select 31599,'BOE' from dual union all
select 316,'CIA' from dual union all
select 316,'NRO' from dual union all
select 3160,'VMF' from dual union all
select 31600,'NASA' from dual union all
select 31601,'ISA' from dual union all
select 31602,'ISA' from dual union all
select 31603,'NASA' from dual union all
select 31604,'NASA' from dual union all
select 31605,'NASA' from dual union all
select 31606,'NASA' from dual union all
select 31607,'NASA' from dual union all
select 31608,'CASC' from dual union all
select 31609,'CASC' from dual union all
select 3161,'VMF' from dual union all
select 31610,'CASC' from dual union all
select 31611,'CASC' from dual union all
select 31612,'CASC' from dual union all
select 31613,'CASC' from dual union all
select 31614,'CASC' from dual union all
select 31615,'CASC' from dual union all
select 31616,'CASC' from dual union all
select 31617,'CASC' from dual union all
select 31618,'CASC' from dual union all
select 31619,'CASC' from dual union all
select 3162,'VMF' from dual union all
select 31620,'CASC' from dual union all
select 31621,'CASC' from dual union all
select 31622,'CASC' from dual union all
select 31623,'CASC' from dual union all
select 31624,'CASC' from dual union all
select 31625,'CASC' from dual union all
select 31626,'CASC' from dual union all
select 31627,'CASC' from dual union all
select 31628,'CASC' from dual union all
select 31629,'CASC' from dual union all
select 3163,'RVSN' from dual union all
select 31630,'CASC' from dual union all
select 31631,'CASC' from dual union all
select 31632,'CASC' from dual union all
select 31633,'CASC' from dual union all
select 31634,'CASC' from dual union all
select 31635,'CASC' from dual union all
select 31636,'CASC' from dual union all
select 31637,'CASC' from dual union all
select 31638,'CASC' from dual union all
select 31639,'CASC' from dual union all
select 3164,'USAF' from dual union all
select 31640,'CASC' from dual union all
select 31641,'CASC' from dual union all
select 31642,'CASC' from dual union all
select 31643,'CASC' from dual union all
select 31644,'CASC' from dual union all
select 31645,'CASC' from dual;
insert into SATELLITE_ORG
select 31646,'CASC' from dual union all
select 31647,'CASC' from dual union all
select 31648,'CASC' from dual union all
select 31649,'CASC' from dual union all
select 3165,'VMF' from dual union all
select 31650,'CASC' from dual union all
select 31651,'CASC' from dual union all
select 31652,'CASC' from dual union all
select 31653,'CASC' from dual union all
select 31654,'CASC' from dual union all
select 31655,'CASC' from dual union all
select 31656,'CASC' from dual union all
select 31657,'CASC' from dual union all
select 31658,'CASC' from dual union all
select 31659,'CASC' from dual union all
select 3166,'USN' from dual union all
select 31660,'CASC' from dual union all
select 31661,'CASC' from dual union all
select 31662,'CASC' from dual union all
select 31663,'CASC' from dual union all
select 31664,'CASC' from dual union all
select 31665,'CASC' from dual union all
select 31666,'CASC' from dual union all
select 31667,'CASC' from dual union all
select 31668,'CASC' from dual union all
select 31669,'CASC' from dual union all
select 3167,'TSKBEM' from dual union all
select 31670,'CASC' from dual union all
select 31671,'CASC' from dual union all
select 31672,'CASC' from dual union all
select 31673,'CASC' from dual union all
select 31674,'CASC' from dual union all
select 31675,'CASC' from dual union all
select 31676,'CASC' from dual union all
select 31677,'CASC' from dual union all
select 31678,'CASC' from dual union all
select 31679,'CASC' from dual union all
select 3168,'MO' from dual union all
select 31680,'CASC' from dual union all
select 31681,'CASC' from dual union all
select 31682,'CASC' from dual union all
select 31683,'CASC' from dual union all
select 31684,'CASC' from dual union all
select 31685,'CASC' from dual union all
select 31686,'CASC' from dual union all
select 31687,'CASC' from dual union all
select 31688,'CASC' from dual union all
select 31689,'CASC' from dual union all
select 3169,'RVSN' from dual union all
select 31690,'CASC' from dual union all
select 31691,'CASC' from dual union all
select 31692,'CASC' from dual union all
select 31693,'CASC' from dual union all
select 31694,'CASC' from dual union all
select 31695,'CASC' from dual union all
select 31696,'CASC' from dual union all
select 31697,'CASC' from dual union all
select 31698,'DLR' from dual union all
select 31698,'EADS' from dual union all
select 31699,'KTRAS' from dual union all
select 317,'USAF' from dual union all
select 3170,'NASA' from dual union all
select 31700,'KTRAS' from dual union all
select 31701,'NROC' from dual union all
select 31701,'USN' from dual union all
select 31702,'LM' from dual union all
select 31703,'USAF' from dual union all
select 31704,'USAF' from dual union all
select 31705,'USAF' from dual union all
select 31706,'USAF' from dual union all
select 31707,'USN' from dual union all
select 31708,'NROC' from dual union all
select 31708,'USN' from dual union all
select 31709,'CASC' from dual union all
select 3171,'NASA' from dual union all
select 31710,'CASC' from dual union all
select 31711,'CASC' from dual union all
select 31712,'CASC' from dual union all
select 31713,'CASC' from dual union all
select 31714,'CASC' from dual union all
select 31715,'CASC' from dual union all
select 31716,'CASC' from dual union all
select 31717,'CASC' from dual union all
select 31718,'CASC' from dual union all
select 31719,'CASC' from dual union all
select 3172,'MO' from dual union all
select 31720,'CASC' from dual union all
select 31721,'CASC' from dual union all
select 31722,'CASC' from dual union all
select 31723,'CASC' from dual union all
select 31724,'CASC' from dual union all
select 31725,'CASC' from dual union all
select 31726,'CASC' from dual union all
select 31727,'CASC' from dual union all
select 31728,'CASC' from dual union all
select 31729,'CASC' from dual union all
select 3173,'USAF' from dual union all
select 31730,'CASC' from dual union all
select 31731,'CASC' from dual union all
select 31732,'CASC' from dual;
insert into SATELLITE_ORG
select 31733,'CASC' from dual union all
select 31734,'CASC' from dual union all
select 31735,'CASC' from dual union all
select 31736,'CASC' from dual union all
select 31737,'CASC' from dual union all
select 31738,'CASC' from dual union all
select 31739,'CASC' from dual union all
select 3174,'USAF' from dual union all
select 31740,'CASC' from dual union all
select 31741,'CASC' from dual union all
select 31742,'CASC' from dual union all
select 31743,'CASC' from dual union all
select 31744,'CASC' from dual union all
select 31745,'CASC' from dual union all
select 31746,'CASC' from dual union all
select 31747,'CASC' from dual union all
select 31748,'CASC' from dual union all
select 31749,'CASC' from dual union all
select 3175,'USAF' from dual union all
select 31750,'CASC' from dual union all
select 31751,'CASC' from dual union all
select 31752,'CASC' from dual union all
select 31753,'CASC' from dual union all
select 31754,'CASC' from dual union all
select 31755,'CASC' from dual union all
select 31756,'CASC' from dual union all
select 31757,'CASC' from dual union all
select 31758,'CASC' from dual union all
select 31759,'CASC' from dual union all
select 3176,'USAF' from dual union all
select 31760,'CASC' from dual union all
select 31761,'CASC' from dual union all
select 31762,'CASC' from dual union all
select 31763,'CASC' from dual union all
select 31764,'CASC' from dual union all
select 31765,'CASC' from dual union all
select 31766,'CASC' from dual union all
select 31767,'CASC' from dual union all
select 31768,'CASC' from dual union all
select 31769,'CASC' from dual union all
select 3177,'USAF' from dual union all
select 31770,'CASC' from dual union all
select 31771,'CASC' from dual union all
select 31772,'CASC' from dual union all
select 31773,'CASC' from dual union all
select 31774,'CASC' from dual union all
select 31775,'CASC' from dual union all
select 31776,'CASC' from dual union all
select 31777,'CASC' from dual union all
select 31778,'CASC' from dual union all
select 31779,'CASC' from dual union all
select 3178,'IKI' from dual union all
select 3178,'NPOL' from dual union all
select 31780,'CASC' from dual union all
select 31781,'CASC' from dual union all
select 31782,'CASC' from dual union all
select 31783,'CASC' from dual union all
select 31784,'CASC' from dual union all
select 31785,'CASC' from dual union all
select 31786,'CASC' from dual union all
select 31787,'CASC' from dual union all
select 31788,'CASC' from dual union all
select 31789,'BIGEL' from dual union all
select 3179,'RVSN' from dual union all
select 31790,'KTRAS' from dual union all
select 31791,'KTRAS' from dual union all
select 31792,'MORF' from dual union all
select 31793,'KVR' from dual union all
select 31794,'KVR' from dual union all
select 31795,'KVR' from dual union all
select 31796,'KVR' from dual union all
select 31797,'BWEHR' from dual union all
select 31798,'KVR' from dual union all
select 31799,'KVR' from dual union all
select 318,'USAF' from dual union all
select 3180,'RVSN' from dual union all
select 31800,'CHISA' from dual union all
select 31801,'CASC' from dual union all
select 31802,'CASC' from dual union all
select 31803,'CASC' from dual union all
select 31804,'CASC' from dual union all
select 31805,'CASC' from dual union all
select 31806,'CASC' from dual union all
select 31807,'CASC' from dual union all
select 31808,'CASC' from dual union all
select 31809,'CASC' from dual union all
select 3181,'PVO' from dual union all
select 31810,'CASC' from dual union all
select 31811,'CASC' from dual union all
select 31812,'CASC' from dual union all
select 31813,'CASC' from dual union all
select 31814,'CASC' from dual union all
select 31815,'CASC' from dual union all
select 31816,'CASC' from dual union all
select 31817,'CASC' from dual union all
select 31818,'CASC' from dual union all
select 31819,'CASC' from dual union all
select 3182,'RVSN' from dual union all
select 31820,'CASC' from dual union all
select 31821,'CASC' from dual;
insert into SATELLITE_ORG
select 31822,'CASC' from dual union all
select 31823,'CASC' from dual union all
select 31824,'CASC' from dual union all
select 31825,'CASC' from dual union all
select 31826,'CASC' from dual union all
select 31827,'CASC' from dual union all
select 31828,'CASC' from dual union all
select 31829,'CASC' from dual union all
select 3183,'TSKBEM' from dual union all
select 31830,'CASC' from dual union all
select 31831,'CASC' from dual union all
select 31832,'CASC' from dual union all
select 31833,'CASC' from dual union all
select 31834,'CASC' from dual union all
select 31835,'CASC' from dual union all
select 31836,'CASC' from dual union all
select 31837,'CASC' from dual union all
select 31838,'CASC' from dual union all
select 31839,'CASC' from dual union all
select 3184,'RVSN' from dual union all
select 31840,'CASC' from dual union all
select 31841,'CASC' from dual union all
select 31842,'CASC' from dual union all
select 31843,'CASC' from dual union all
select 31844,'CASC' from dual union all
select 31845,'CASC' from dual union all
select 31846,'CASC' from dual union all
select 31847,'CASC' from dual union all
select 31848,'CASC' from dual union all
select 31849,'CASC' from dual union all
select 3185,'NASA' from dual union all
select 31850,'CASC' from dual union all
select 31851,'CASC' from dual union all
select 31852,'CASC' from dual union all
select 31853,'CASC' from dual union all
select 31854,'CASC' from dual union all
select 31855,'CASC' from dual union all
select 31856,'CASC' from dual union all
select 31857,'CASC' from dual union all
select 31858,'CASC' from dual union all
select 31859,'CASC' from dual union all
select 3186,'NASA' from dual union all
select 31860,'CASC' from dual union all
select 31861,'CASC' from dual union all
select 31862,'DTV' from dual union all
select 31863,'KHRR' from dual union all
select 31864,'KHRR' from dual union all
select 31865,'KTRAS' from dual union all
select 31866,'RVSNR' from dual union all
select 31867,'CNSA' from dual union all
select 31868,'CNSA' from dual union all
select 31869,'CNSA' from dual union all
select 3187,'NASA' from dual union all
select 31870,'CNSA' from dual union all
select 31871,'CNSA' from dual union all
select 31872,'CNSA' from dual union all
select 31873,'CNSA' from dual union all
select 31874,'CNSA' from dual union all
select 31875,'CNSA' from dual union all
select 31876,'CNSA' from dual union all
select 31877,'CASC' from dual union all
select 31878,'CASC' from dual union all
select 31879,'CASC' from dual union all
select 3188,'NASA' from dual union all
select 31880,'CASC' from dual union all
select 31881,'CASC' from dual union all
select 31882,'CASC' from dual union all
select 31883,'CASC' from dual union all
select 31884,'CASC' from dual union all
select 31885,'CASC' from dual union all
select 31886,'CASC' from dual union all
select 31887,'CASC' from dual union all
select 31888,'CASC' from dual union all
select 31889,'CASC' from dual union all
select 3189,'NASA' from dual union all
select 31890,'CASC' from dual union all
select 31891,'CASC' from dual union all
select 31892,'CASC' from dual union all
select 31893,'CASC' from dual union all
select 31894,'CASC' from dual union all
select 31895,'CASC' from dual union all
select 31896,'CASC' from dual union all
select 31897,'CASC' from dual union all
select 31898,'CASC' from dual union all
select 31899,'CASC' from dual union all
select 319,'USAF' from dual union all
select 3190,'NASA' from dual union all
select 31900,'CASC' from dual union all
select 31901,'CASC' from dual union all
select 31902,'CASC' from dual union all
select 31903,'CASC' from dual union all
select 31904,'CASC' from dual union all
select 31905,'CASC' from dual union all
select 31906,'CASC' from dual union all
select 31907,'CASC' from dual union all
select 31908,'CASC' from dual union all
select 31909,'CASC' from dual union all
select 3191,'NASA' from dual union all
select 31910,'CASC' from dual union all
select 31911,'CASC' from dual;
insert into SATELLITE_ORG
select 31912,'CASC' from dual union all
select 31913,'CASC' from dual union all
select 31914,'CASC' from dual union all
select 31915,'CASC' from dual union all
select 31916,'CASC' from dual union all
select 31917,'CASC' from dual union all
select 31918,'CASC' from dual union all
select 31919,'CASC' from dual union all
select 3192,'NASA' from dual union all
select 31920,'CASC' from dual union all
select 31921,'CASC' from dual union all
select 31922,'CASC' from dual union all
select 31923,'CASC' from dual union all
select 31924,'CASC' from dual union all
select 31925,'CASC' from dual union all
select 31926,'CASC' from dual union all
select 31927,'NASA' from dual union all
select 31928,'NASA' from dual union all
select 31929,'CASC' from dual union all
select 3193,'TSKBEM' from dual union all
select 31930,'CASC' from dual union all
select 31931,'CASC' from dual union all
select 31932,'CASC' from dual union all
select 31933,'CASC' from dual union all
select 31934,'CASC' from dual union all
select 31935,'CASC' from dual union all
select 31936,'CASC' from dual union all
select 31937,'CASC' from dual union all
select 31938,'CASC' from dual union all
select 31939,'CASC' from dual union all
select 3194,'RVSN' from dual union all
select 31940,'CASC' from dual union all
select 31941,'CASC' from dual union all
select 31942,'CASC' from dual union all
select 31943,'CASC' from dual union all
select 31944,'CASC' from dual union all
select 31945,'CASC' from dual union all
select 31946,'CASC' from dual union all
select 31947,'CASC' from dual union all
select 31948,'CASC' from dual union all
select 31949,'CASC' from dual union all
select 3195,'NASA' from dual union all
select 31950,'CASC' from dual union all
select 31951,'CASC' from dual union all
select 31952,'CASC' from dual union all
select 31953,'CASC' from dual union all
select 31954,'CASC' from dual union all
select 31955,'CASC' from dual union all
select 31956,'CASC' from dual union all
select 31957,'CASC' from dual union all
select 31958,'CASC' from dual union all
select 31959,'CASC' from dual union all
select 3196,'NASA' from dual union all
select 31960,'CASC' from dual union all
select 31961,'CASC' from dual union all
select 31962,'CASC' from dual union all
select 31963,'CASC' from dual union all
select 31964,'CASC' from dual union all
select 31965,'CASC' from dual union all
select 31966,'CASC' from dual union all
select 31967,'CASC' from dual union all
select 31968,'CASC' from dual union all
select 31969,'CASC' from dual union all
select 3197,'NASA' from dual union all
select 31970,'CASC' from dual union all
select 31971,'CASC' from dual union all
select 31972,'CASC' from dual union all
select 31973,'CASC' from dual union all
select 31974,'CASC' from dual union all
select 31975,'CASC' from dual union all
select 31976,'CASC' from dual union all
select 31977,'CASC' from dual union all
select 31978,'CASC' from dual union all
select 31979,'CASC' from dual union all
select 3198,'NASA' from dual union all
select 31980,'CASC' from dual union all
select 31981,'CASC' from dual union all
select 31982,'CASC' from dual union all
select 31983,'CASC' from dual union all
select 31984,'CASC' from dual union all
select 31985,'CASC' from dual union all
select 31986,'CASC' from dual union all
select 31987,'CASC' from dual union all
select 31988,'CASC' from dual union all
select 31989,'CASC' from dual union all
select 3199,'NRO' from dual union all
select 31990,'CASC' from dual union all
select 31991,'CASC' from dual union all
select 31992,'CASC' from dual union all
select 31993,'CASC' from dual union all
select 31994,'CASC' from dual union all
select 31995,'CASC' from dual union all
select 31996,'CASC' from dual union all
select 31997,'CASC' from dual union all
select 31998,'CASC' from dual union all
select 31999,'CRC' from dual union all
select 32,'CIA' from dual union all
select 32,'USAF' from dual union all
select 320,'USAF' from dual union all
select 3200,'NASA' from dual;
insert into SATELLITE_ORG
select 32000,'LM' from dual union all
select 32001,'FKA' from dual union all
select 32002,'KVR' from dual union all
select 32003,'JPL' from dual union all
select 32003,'NASA' from dual union all
select 32004,'BOE' from dual union all
select 32005,'RVSNR' from dual union all
select 32006,'RVSNR' from dual union all
select 32007,'BOE' from dual union all
select 32008,'NASA' from dual union all
select 32009,'JAXA' from dual union all
select 3201,'NASA' from dual union all
select 32010,'JAXA' from dual union all
select 32011,'JAXA' from dual union all
select 32012,'JAXA' from dual union all
select 32013,'JAXA' from dual union all
select 32014,'JAXA' from dual union all
select 32015,'JAXA' from dual union all
select 32016,'JAXA' from dual union all
select 32017,'JAXA' from dual union all
select 32018,'HEC' from dual union all
select 32019,'BSAT' from dual union all
select 3202,'NASA' from dual union all
select 32020,'AE' from dual union all
select 32021,'AE' from dual union all
select 32022,'KVR' from dual union all
select 32023,'KVR' from dual union all
select 32024,'KVR' from dual union all
select 32025,'KHRR' from dual union all
select 32026,'KHRR' from dual union all
select 32027,'KHRR' from dual union all
select 32028,'KHRR' from dual union all
select 32029,'KHRR' from dual union all
select 3203,'MO' from dual union all
select 32030,'KHRR' from dual union all
select 32031,'KHRR' from dual union all
select 32032,'KHRR' from dual union all
select 32033,'KHRR' from dual union all
select 32034,'KHRR' from dual union all
select 32035,'KHRR' from dual union all
select 32036,'KHRR' from dual union all
select 32037,'KHRR' from dual union all
select 32038,'KHRR' from dual union all
select 32039,'KHRR' from dual union all
select 3204,'RVSN' from dual union all
select 32040,'KHRR' from dual union all
select 32041,'KHRR' from dual union all
select 32042,'KHRR' from dual union all
select 32043,'KHRR' from dual union all
select 32044,'KHRR' from dual union all
select 32045,'KHRR' from dual union all
select 32046,'KHRR' from dual union all
select 32047,'KHRR' from dual union all
select 32048,'KVR' from dual union all
select 32049,'KVR' from dual union all
select 3205,'YUZH' from dual union all
select 32050,'ISRO' from dual union all
select 32051,'ISRO' from dual union all
select 32052,'VMFR' from dual union all
select 32053,'KVR' from dual union all
select 32054,'JAXA' from dual union all
select 32055,'JAXA' from dual union all
select 32056,'JAXA' from dual union all
select 32057,'JAXA' from dual union all
select 32058,'FKA' from dual union all
select 32059,'KVR' from dual union all
select 3206,'RVSN' from dual union all
select 32060,'DGLO' from dual union all
select 32061,'BOE' from dual union all
select 32062,'GZB' from dual union all
select 32062,'INPE' from dual union all
select 32063,'CNSA' from dual union all
select 32064,'CNSA' from dual union all
select 32065,'CNSA' from dual union all
select 32066,'CNSA' from dual union all
select 32067,'CNSA' from dual union all
select 32068,'CNSA' from dual union all
select 32069,'CNSA' from dual union all
select 3207,'MO' from dual union all
select 32070,'CNSA' from dual union all
select 32071,'CNSA' from dual union all
select 32072,'CNSA' from dual union all
select 32073,'CNSA' from dual union all
select 32074,'CNSA' from dual union all
select 32075,'CNSA' from dual union all
select 32076,'CNSA' from dual union all
select 32077,'CNSA' from dual union all
select 32078,'CNSA' from dual union all
select 32079,'CNSA' from dual union all
select 3208,'RVSN' from dual union all
select 32080,'CNSA' from dual union all
select 32081,'CNSA' from dual union all
select 32082,'CNSA' from dual union all
select 32083,'CNSA' from dual union all
select 32084,'CNSA' from dual union all
select 32085,'CNSA' from dual union all
select 32086,'CASC' from dual union all
select 32087,'CASC' from dual union all
select 32088,'CASC' from dual union all
select 32089,'CASC' from dual;
insert into SATELLITE_ORG
select 3209,'MO' from dual union all
select 32090,'CASC' from dual union all
select 32091,'CASC' from dual union all
select 32092,'CASC' from dual union all
select 32093,'CASC' from dual union all
select 32094,'CASC' from dual union all
select 32095,'CASC' from dual union all
select 32096,'CASC' from dual union all
select 32097,'CASC' from dual union all
select 32098,'CASC' from dual union all
select 32099,'CASC' from dual union all
select 321,'USAF' from dual union all
select 3210,'RVSN' from dual union all
select 32100,'CASC' from dual union all
select 32101,'CASC' from dual union all
select 32102,'CASC' from dual union all
select 32103,'CASC' from dual union all
select 32104,'CASC' from dual union all
select 32105,'CASC' from dual union all
select 32106,'CASC' from dual union all
select 32107,'CASC' from dual union all
select 32108,'CASC' from dual union all
select 32109,'CASC' from dual union all
select 3211,'RVSN' from dual union all
select 32110,'CASC' from dual union all
select 32111,'CASC' from dual union all
select 32112,'CASC' from dual union all
select 32113,'CASC' from dual union all
select 32114,'CASC' from dual union all
select 32115,'CASC' from dual union all
select 32116,'CASC' from dual union all
select 32117,'CASC' from dual union all
select 32118,'CASC' from dual union all
select 32119,'CASC' from dual union all
select 3212,'USAF' from dual union all
select 32120,'CASC' from dual union all
select 32121,'CASC' from dual union all
select 32122,'CASC' from dual union all
select 32123,'CASC' from dual union all
select 32124,'CASC' from dual union all
select 32125,'CASC' from dual union all
select 32126,'CASC' from dual union all
select 32127,'CASC' from dual union all
select 32128,'CASC' from dual union all
select 32129,'CASC' from dual union all
select 3213,'USN' from dual union all
select 32130,'CASC' from dual union all
select 32131,'CASC' from dual union all
select 32132,'CASC' from dual union all
select 32133,'CASC' from dual union all
select 32134,'CASC' from dual union all
select 32135,'CASC' from dual union all
select 32136,'CASC' from dual union all
select 32137,'CASC' from dual union all
select 32138,'CASC' from dual union all
select 32139,'CASC' from dual union all
select 3214,'USN' from dual union all
select 32140,'CASC' from dual union all
select 32141,'CASC' from dual union all
select 32142,'CASC' from dual union all
select 32143,'CASC' from dual union all
select 32144,'CASC' from dual union all
select 32145,'CASC' from dual union all
select 32146,'CASC' from dual union all
select 32147,'CASC' from dual union all
select 32148,'CASC' from dual union all
select 32149,'CASC' from dual union all
select 3215,'NASA' from dual union all
select 32150,'CASC' from dual union all
select 32151,'CASC' from dual union all
select 32152,'CASC' from dual union all
select 32153,'CASC' from dual union all
select 32154,'CASC' from dual union all
select 32155,'CASC' from dual union all
select 32156,'CASC' from dual union all
select 32157,'CASC' from dual union all
select 32158,'CASC' from dual union all
select 32159,'CASC' from dual union all
select 3216,'PVO' from dual union all
select 32160,'CASC' from dual union all
select 32161,'CASC' from dual union all
select 32162,'CASC' from dual union all
select 32163,'CASC' from dual union all
select 32164,'CASC' from dual union all
select 32165,'CASC' from dual union all
select 32166,'CASC' from dual union all
select 32167,'CASC' from dual union all
select 32168,'CASC' from dual union all
select 32169,'CASC' from dual union all
select 3217,'RVSN' from dual union all
select 32170,'CASC' from dual union all
select 32171,'CASC' from dual union all
select 32172,'CASC' from dual union all
select 32173,'CASC' from dual union all
select 32174,'CASC' from dual union all
select 32175,'CASC' from dual union all
select 32176,'CASC' from dual union all
select 32177,'CASC' from dual union all
select 32178,'CASC' from dual union all
select 32179,'CASC' from dual;
insert into SATELLITE_ORG
select 3218,'RVSN' from dual union all
select 32180,'CASC' from dual union all
select 32181,'CASC' from dual union all
select 32182,'CASC' from dual union all
select 32183,'CASC' from dual union all
select 32184,'CASC' from dual union all
select 32185,'CASC' from dual union all
select 32186,'CASC' from dual union all
select 32187,'CASC' from dual union all
select 32188,'CASC' from dual union all
select 32189,'CASC' from dual union all
select 3219,'RVSN' from dual union all
select 32190,'CASC' from dual union all
select 32191,'CASC' from dual union all
select 32192,'CASC' from dual union all
select 32193,'CASC' from dual union all
select 32194,'CASC' from dual union all
select 32195,'CASC' from dual union all
select 32196,'CASC' from dual union all
select 32197,'CASC' from dual union all
select 32198,'CASC' from dual union all
select 32199,'CASC' from dual union all
select 322,'USAF' from dual union all
select 3220,'YUZH' from dual union all
select 32200,'CASC' from dual union all
select 32201,'CASC' from dual union all
select 32202,'CASC' from dual union all
select 32203,'CASC' from dual union all
select 32204,'CASC' from dual union all
select 32205,'CASC' from dual union all
select 32206,'CASC' from dual union all
select 32207,'CASC' from dual union all
select 32208,'CASC' from dual union all
select 32209,'CASC' from dual union all
select 3221,'RVSN' from dual union all
select 32210,'CASC' from dual union all
select 32211,'CASC' from dual union all
select 32212,'CASC' from dual union all
select 32213,'CASC' from dual union all
select 32214,'CASC' from dual union all
select 32215,'CASC' from dual union all
select 32216,'CASC' from dual union all
select 32217,'CASC' from dual union all
select 32218,'CASC' from dual union all
select 32219,'CASC' from dual union all
select 3222,'RVSN' from dual union all
select 32220,'CASC' from dual union all
select 32221,'CASC' from dual union all
select 32222,'CASC' from dual union all
select 32223,'CASC' from dual union all
select 32224,'CASC' from dual union all
select 32225,'CASC' from dual union all
select 32226,'CASC' from dual union all
select 32227,'CASC' from dual union all
select 32228,'CASC' from dual union all
select 32229,'CASC' from dual union all
select 3223,'RVSN' from dual union all
select 32230,'CASC' from dual union all
select 32231,'CASC' from dual union all
select 32232,'CASC' from dual union all
select 32233,'CASC' from dual union all
select 32234,'CASC' from dual union all
select 32235,'CASC' from dual union all
select 32236,'CASC' from dual union all
select 32237,'CASC' from dual union all
select 32238,'CASC' from dual union all
select 32239,'CASC' from dual union all
select 3224,'RVSN' from dual union all
select 32240,'CASC' from dual union all
select 32241,'CASC' from dual union all
select 32242,'CASC' from dual union all
select 32243,'CASC' from dual union all
select 32244,'CASC' from dual union all
select 32245,'CASC' from dual union all
select 32246,'RVSNR' from dual union all
select 32247,'RVSNR' from dual union all
select 32248,'RVSNR' from dual union all
select 32249,'JPL' from dual union all
select 32249,'NASA' from dual union all
select 3225,'RVSN' from dual union all
select 32250,'BOE' from dual union all
select 32251,'BOE' from dual union all
select 32252,'OPTS' from dual union all
select 32253,'INTELB' from dual union all
select 32254,'AE' from dual union all
select 32255,'AE' from dual union all
select 32256,'FKA' from dual union all
select 32257,'KVR' from dual union all
select 32258,'USAF' from dual union all
select 32259,'LM' from dual union all
select 3226,'RVSN' from dual union all
select 32260,'USAF' from dual union all
select 32261,'BOE' from dual union all
select 32262,'BOE' from dual union all
select 32263,'GLOB' from dual union all
select 32264,'GLOB' from dual union all
select 32265,'GLOB' from dual union all
select 32266,'GLOB' from dual union all
select 32267,'KVR' from dual union all
select 32268,'PVO' from dual;
insert into SATELLITE_ORG
select 32269,'RVSNR' from dual union all
select 3227,'MO' from dual union all
select 32270,'RVSNR' from dual union all
select 32271,'RVSNR' from dual union all
select 32272,'NASA' from dual union all
select 32273,'CASC' from dual union all
select 32274,'CASC' from dual union all
select 32275,'FKA' from dual union all
select 32275,'KVR' from dual union all
select 32276,'FKA' from dual union all
select 32276,'KVR' from dual union all
select 32277,'FKA' from dual union all
select 32277,'KVR' from dual union all
select 32278,'KHRR' from dual union all
select 32279,'KHRR' from dual union all
select 3228,'CIA' from dual union all
select 3228,'NRO' from dual union all
select 32280,'RVSNR' from dual union all
select 32281,'KVR' from dual union all
select 32282,'RVSNR' from dual union all
select 32283,'BWEHR' from dual union all
select 32284,'KVR' from dual union all
select 32285,'NASA' from dual union all
select 32286,'NASA' from dual union all
select 32287,'USAF' from dual union all
select 32288,'BOE' from dual union all
select 32289,'PLA' from dual union all
select 3229,'OKB10' from dual union all
select 32290,'CNSA' from dual union all
select 32291,'NASA' from dual union all
select 32292,'NASA' from dual union all
select 32293,'TELB' from dual union all
select 32294,'PARA' from dual union all
select 32295,'AE' from dual union all
select 32296,'AE' from dual union all
select 32297,'NASA' from dual union all
select 32298,'NASA' from dual union all
select 32299,'SES' from dual union all
select 323,'USAF' from dual union all
select 3230,'RVSN' from dual union all
select 32300,'KHRR' from dual union all
select 32301,'KHRR' from dual union all
select 32302,'NASA' from dual union all
select 32303,'CASC' from dual union all
select 32304,'CASC' from dual union all
select 32305,'CASC' from dual union all
select 32306,'CASC' from dual union all
select 32307,'CASC' from dual union all
select 32308,'CASC' from dual union all
select 32309,'CASC' from dual union all
select 3231,'MO' from dual union all
select 32310,'CASC' from dual union all
select 32311,'CASC' from dual union all
select 32312,'CASC' from dual union all
select 32313,'CASC' from dual union all
select 32314,'CASC' from dual union all
select 32315,'CASC' from dual union all
select 32316,'CASC' from dual union all
select 32317,'CASC' from dual union all
select 32318,'CASC' from dual union all
select 32319,'CASC' from dual union all
select 3232,'CIA' from dual union all
select 3232,'NRO' from dual union all
select 32320,'CASC' from dual union all
select 32321,'CASC' from dual union all
select 32322,'CASC' from dual union all
select 32323,'CASC' from dual union all
select 32324,'CASC' from dual union all
select 32325,'CASC' from dual union all
select 32326,'CASC' from dual union all
select 32327,'CASC' from dual union all
select 32328,'CASC' from dual union all
select 32329,'CASC' from dual union all
select 3233,'ESTEC' from dual union all
select 32330,'CASC' from dual union all
select 32331,'CASC' from dual union all
select 32332,'CASC' from dual union all
select 32333,'CASC' from dual union all
select 32334,'CASC' from dual union all
select 32335,'CASC' from dual union all
select 32336,'CASC' from dual union all
select 32337,'CASC' from dual union all
select 32338,'CASC' from dual union all
select 32339,'CASC' from dual union all
select 3234,'NASA' from dual union all
select 32340,'CASC' from dual union all
select 32341,'CASC' from dual union all
select 32342,'CASC' from dual union all
select 32343,'CASC' from dual union all
select 32344,'CASC' from dual union all
select 32345,'CASC' from dual union all
select 32346,'CASC' from dual union all
select 32347,'CASC' from dual union all
select 32348,'CASC' from dual union all
select 32349,'CASC' from dual union all
select 3235,'USAF' from dual union all
select 32350,'CASC' from dual union all
select 32351,'CASC' from dual union all
select 32352,'CASC' from dual union all
select 32353,'CASC' from dual;
insert into SATELLITE_ORG
select 32354,'CASC' from dual union all
select 32355,'CASC' from dual union all
select 32356,'CASC' from dual union all
select 32357,'CASC' from dual union all
select 32358,'CASC' from dual union all
select 32359,'CASC' from dual union all
select 3236,'USAF' from dual union all
select 32360,'CASC' from dual union all
select 32361,'CASC' from dual union all
select 32362,'CASC' from dual union all
select 32363,'CASC' from dual union all
select 32364,'CASC' from dual union all
select 32365,'CASC' from dual union all
select 32366,'CASC' from dual union all
select 32367,'CASC' from dual union all
select 32368,'CASC' from dual union all
select 32369,'CASC' from dual union all
select 3237,'USAF' from dual union all
select 32370,'CASC' from dual union all
select 32371,'CASC' from dual union all
select 32372,'CASC' from dual union all
select 32373,'KVR' from dual union all
select 32374,'KHRR' from dual union all
select 32375,'KHRR' from dual union all
select 32376,'ASI' from dual union all
select 32377,'BOE' from dual union all
select 32378,'NROC' from dual union all
select 32379,'LM' from dual union all
select 3238,'USAF' from dual union all
select 32380,'CNSA' from dual union all
select 32381,'CNSA' from dual union all
select 32382,'CSA' from dual union all
select 32383,'KVR' from dual union all
select 32384,'USAF' from dual union all
select 32385,'BOE' from dual union all
select 32386,'BOE' from dual union all
select 32387,'I-RASC' from dual union all
select 32387,'RSQ' from dual union all
select 32388,'HORZ' from dual union all
select 32389,'AE' from dual union all
select 3239,'USAF' from dual union all
select 32390,'AE' from dual union all
select 32391,'FKA' from dual union all
select 32392,'KVR' from dual union all
select 32393,'FKA' from dual union all
select 32393,'KVR' from dual union all
select 32394,'FKA' from dual union all
select 32394,'KVR' from dual union all
select 32395,'FKA' from dual union all
select 32395,'KVR' from dual union all
select 32396,'KHRR' from dual union all
select 32397,'KHRR' from dual union all
select 32398,'KVR' from dual union all
select 32399,'KVR' from dual union all
select 324,'USAF' from dual union all
select 3240,'USAF' from dual union all
select 32400,'KHRR' from dual union all
select 32401,'KHRR' from dual union all
select 32402,'KHRR' from dual union all
select 32403,'KHRR' from dual union all
select 32404,'THUR' from dual union all
select 32405,'SEALP' from dual union all
select 32406,'CASC' from dual union all
select 32407,'CASC' from dual union all
select 32408,'CASC' from dual union all
select 32409,'CASC' from dual union all
select 3241,'USAF' from dual union all
select 32410,'CASC' from dual union all
select 32411,'CASC' from dual union all
select 32412,'CASC' from dual union all
select 32413,'CASC' from dual union all
select 32414,'CASC' from dual union all
select 32415,'CASC' from dual union all
select 32416,'CASC' from dual union all
select 32417,'CASC' from dual union all
select 32418,'CASC' from dual union all
select 32419,'CASC' from dual union all
select 3242,'USAF' from dual union all
select 32420,'CASC' from dual union all
select 32421,'CASC' from dual union all
select 32422,'CASC' from dual union all
select 32423,'CASC' from dual union all
select 32424,'CASC' from dual union all
select 32425,'CASC' from dual union all
select 32426,'CASC' from dual union all
select 32427,'CASC' from dual union all
select 32428,'CASC' from dual union all
select 32429,'CASC' from dual union all
select 3243,'USAF' from dual union all
select 32430,'CASC' from dual union all
select 32431,'CASC' from dual union all
select 32432,'CASC' from dual union all
select 32433,'CASC' from dual union all
select 32434,'CASC' from dual union all
select 32435,'CASC' from dual union all
select 32436,'CASC' from dual union all
select 32437,'CASC' from dual union all
select 32438,'CASC' from dual union all
select 32439,'CASC' from dual union all
select 3244,'USAF' from dual;
insert into SATELLITE_ORG
select 32440,'CASC' from dual union all
select 32441,'CASC' from dual union all
select 32442,'CASC' from dual union all
select 32443,'CASC' from dual union all
select 32444,'CASC' from dual union all
select 32445,'CASC' from dual union all
select 32446,'CASC' from dual union all
select 32447,'CASC' from dual union all
select 32448,'CASC' from dual union all
select 32449,'CASC' from dual union all
select 3245,'USAF' from dual union all
select 32450,'CASC' from dual union all
select 32451,'CASC' from dual union all
select 32452,'CASC' from dual union all
select 32453,'CASC' from dual union all
select 32454,'CASC' from dual union all
select 32455,'CASC' from dual union all
select 32456,'CASC' from dual union all
select 32457,'CASC' from dual union all
select 32458,'CASC' from dual union all
select 32459,'CASC' from dual union all
select 3246,'USAF' from dual union all
select 32460,'CASC' from dual union all
select 32461,'CASC' from dual union all
select 32462,'CASC' from dual union all
select 32463,'CASC' from dual union all
select 32464,'CASC' from dual union all
select 32465,'CASC' from dual union all
select 32466,'CNSA' from dual union all
select 32467,'CNSA' from dual union all
select 32468,'CNSA' from dual union all
select 32469,'CNSA' from dual union all
select 3247,'USAF' from dual union all
select 32470,'CNSA' from dual union all
select 32471,'NASA' from dual union all
select 32472,'NASA' from dual union all
select 32473,'CNSA' from dual union all
select 32474,'CNSA' from dual union all
select 32475,'NASA' from dual union all
select 32476,'IDF' from dual union all
select 32477,'ISRO' from dual union all
select 32478,'KS' from dual union all
select 32479,'KHRR' from dual union all
select 3248,'USAF' from dual union all
select 32480,'KHRR' from dual union all
select 32481,'KHRR' from dual union all
select 32482,'JPL' from dual union all
select 32482,'NASA' from dual union all
select 32483,'NASDA' from dual union all
select 32484,'FKA' from dual union all
select 32485,'KVR' from dual union all
select 32486,'NASA' from dual union all
select 32487,'TNOR' from dual union all
select 32488,'KHRR' from dual union all
select 32489,'KHRR' from dual union all
select 3249,'USAF' from dual union all
select 32490,'KHRR' from dual union all
select 32491,'KHRR' from dual union all
select 32492,'KHRR' from dual union all
select 32493,'KHRR' from dual union all
select 32494,'KHRR' from dual union all
select 32495,'KHRR' from dual union all
select 32496,'KHRR' from dual union all
select 32497,'KHRR' from dual union all
select 32498,'KHRR' from dual union all
select 32499,'KHRR' from dual union all
select 325,'USAF' from dual union all
select 3250,'USAF' from dual union all
select 32500,'JAXA' from dual union all
select 32501,'JAXA' from dual union all
select 32502,'NROC' from dual union all
select 32503,'NROC' from dual union all
select 32504,'NROC' from dual union all
select 32505,'NROC' from dual union all
select 32506,'NROC' from dual union all
select 32507,'NROC' from dual union all
select 32508,'NROC' from dual union all
select 32509,'NROC' from dual union all
select 3251,'USAF' from dual union all
select 32510,'NROC' from dual union all
select 32511,'NROC' from dual union all
select 32512,'NROC' from dual union all
select 32513,'NROC' from dual union all
select 32514,'NROC' from dual union all
select 32515,'NROC' from dual union all
select 32516,'NROC' from dual union all
select 32517,'NROC' from dual union all
select 32518,'NROC' from dual union all
select 32519,'NROC' from dual union all
select 3252,'USAF' from dual union all
select 32520,'NROC' from dual union all
select 32521,'NROC' from dual union all
select 32522,'NROC' from dual union all
select 32523,'NROC' from dual union all
select 32524,'NROC' from dual union all
select 32525,'NROC' from dual union all
select 32526,'NROC' from dual union all
select 32527,'NROC' from dual union all
select 32528,'NROC' from dual union all
select 32529,'NROC' from dual;
insert into SATELLITE_ORG
select 3253,'USAF' from dual union all
select 32530,'NROC' from dual union all
select 32531,'NROC' from dual union all
select 32532,'NROC' from dual union all
select 32533,'NROC' from dual union all
select 32534,'NROC' from dual union all
select 32535,'NROC' from dual union all
select 32536,'NROC' from dual union all
select 32537,'NROC' from dual union all
select 32538,'NROC' from dual union all
select 32539,'NROC' from dual union all
select 3254,'USAF' from dual union all
select 32540,'NROC' from dual union all
select 32541,'NROC' from dual union all
select 32542,'NROC' from dual union all
select 32543,'NROC' from dual union all
select 32544,'NROC' from dual union all
select 32545,'NROC' from dual union all
select 32546,'NROC' from dual union all
select 32547,'BOE' from dual union all
select 32548,'BOE' from dual union all
select 32549,'BOE' from dual union all
select 3255,'USAF' from dual union all
select 32550,'BOE' from dual union all
select 32551,'BOE' from dual union all
select 32552,'BOE' from dual union all
select 32553,'BOE' from dual union all
select 32554,'BOE' from dual union all
select 32555,'BOE' from dual union all
select 32556,'BOE' from dual union all
select 32557,'BOE' from dual union all
select 32558,'BOE' from dual union all
select 32559,'BOE' from dual union all
select 3256,'USAF' from dual union all
select 32560,'BOE' from dual union all
select 32561,'BOE' from dual union all
select 32562,'BOE' from dual union all
select 32563,'BOE' from dual union all
select 32564,'BOE' from dual union all
select 32565,'BOE' from dual union all
select 32566,'BOE' from dual union all
select 32567,'BOE' from dual union all
select 32568,'BOE' from dual union all
select 32569,'BOE' from dual union all
select 3257,'USAF' from dual union all
select 32570,'BOE' from dual union all
select 32571,'BOE' from dual union all
select 32572,'BOE' from dual union all
select 32573,'BOE' from dual union all
select 32574,'BOE' from dual union all
select 32575,'BOE' from dual union all
select 32576,'BOE' from dual union all
select 32577,'BOE' from dual union all
select 32578,'NROC' from dual union all
select 32579,'NROC' from dual union all
select 3258,'USAF' from dual union all
select 32580,'NROC' from dual union all
select 32581,'NROC' from dual union all
select 32582,'NROC' from dual union all
select 32583,'NROC' from dual union all
select 32584,'NROC' from dual union all
select 32585,'NROC' from dual union all
select 32586,'NROC' from dual union all
select 32587,'NROC' from dual union all
select 32588,'NROC' from dual union all
select 32589,'NROC' from dual union all
select 3259,'USAF' from dual union all
select 32590,'NROC' from dual union all
select 32591,'NROC' from dual union all
select 32592,'NROC' from dual union all
select 32593,'NROC' from dual union all
select 32594,'NROC' from dual union all
select 32595,'NROC' from dual union all
select 32596,'NROC' from dual union all
select 32597,'NROC' from dual union all
select 32598,'NROC' from dual union all
select 32599,'NROC' from dual union all
select 326,'USAF' from dual union all
select 3260,'USAF' from dual union all
select 32600,'NROC' from dual union all
select 32601,'NROC' from dual union all
select 32602,'NROC' from dual union all
select 32603,'NROC' from dual union all
select 32604,'NROC' from dual union all
select 32605,'NROC' from dual union all
select 32606,'NROC' from dual union all
select 32607,'NROC' from dual union all
select 32608,'NROC' from dual union all
select 32609,'NROC' from dual union all
select 3261,'USAF' from dual union all
select 32610,'NROC' from dual union all
select 32611,'NROC' from dual union all
select 32612,'NROC' from dual union all
select 32613,'NROC' from dual union all
select 32614,'NROC' from dual union all
select 32615,'NROC' from dual union all
select 32616,'NROC' from dual union all
select 32617,'NROC' from dual union all
select 32618,'NROC' from dual union all
select 32619,'NROC' from dual;
insert into SATELLITE_ORG
select 3262,'USAF' from dual union all
select 32620,'NROC' from dual union all
select 32621,'NROC' from dual union all
select 32622,'NROC' from dual union all
select 32623,'NROC' from dual union all
select 32624,'NROC' from dual union all
select 32625,'NROC' from dual union all
select 32626,'NROC' from dual union all
select 32627,'NROC' from dual union all
select 32628,'NROC' from dual union all
select 32629,'NROC' from dual union all
select 3263,'USAF' from dual union all
select 32630,'NROC' from dual union all
select 32631,'NROC' from dual union all
select 32632,'NROC' from dual union all
select 32633,'NROC' from dual union all
select 32634,'NROC' from dual union all
select 32635,'NROC' from dual union all
select 32636,'NROC' from dual union all
select 32637,'NROC' from dual union all
select 32638,'NROC' from dual union all
select 32639,'NROC' from dual union all
select 3264,'USAF' from dual union all
select 32640,'NROC' from dual union all
select 32641,'NROC' from dual union all
select 32642,'NROC' from dual union all
select 32643,'NROC' from dual union all
select 32644,'NROC' from dual union all
select 32645,'NROC' from dual union all
select 32646,'NROC' from dual union all
select 32647,'NROC' from dual union all
select 32648,'NROC' from dual union all
select 32649,'NROC' from dual union all
select 3265,'ESRO' from dual union all
select 32650,'NROC' from dual union all
select 32651,'NROC' from dual union all
select 32652,'NROC' from dual union all
select 32653,'NROC' from dual union all
select 32654,'NROC' from dual union all
select 32655,'NROC' from dual union all
select 32656,'NROC' from dual union all
select 32657,'NROC' from dual union all
select 32658,'NROC' from dual union all
select 32659,'NROC' from dual union all
select 3266,'USAF' from dual union all
select 32660,'NROC' from dual union all
select 32661,'NROC' from dual union all
select 32662,'NROC' from dual union all
select 32663,'NROC' from dual union all
select 32664,'NROC' from dual union all
select 32665,'NROC' from dual union all
select 32666,'NROC' from dual union all
select 32667,'NROC' from dual union all
select 32668,'NROC' from dual union all
select 32669,'NROC' from dual union all
select 3267,'USAF' from dual union all
select 32670,'NROC' from dual union all
select 32671,'NROC' from dual union all
select 32672,'NROC' from dual union all
select 32673,'NROC' from dual union all
select 32674,'NROC' from dual union all
select 32675,'NROC' from dual union all
select 32676,'NROC' from dual union all
select 32677,'NROC' from dual union all
select 32678,'NROC' from dual union all
select 32679,'NROC' from dual union all
select 3268,'USAF' from dual union all
select 32680,'NROC' from dual union all
select 32681,'NROC' from dual union all
select 32682,'NROC' from dual union all
select 32683,'NROC' from dual union all
select 32684,'NROC' from dual union all
select 32685,'NROC' from dual union all
select 32686,'ESA' from dual union all
select 32687,'AE' from dual union all
select 32688,'NROC' from dual union all
select 32689,'NROC' from dual union all
select 3269,'PVO' from dual union all
select 32690,'NROC' from dual union all
select 32691,'NROC' from dual union all
select 32692,'NROC' from dual union all
select 32693,'NROC' from dual union all
select 32694,'NROC' from dual union all
select 32695,'NROC' from dual union all
select 32696,'NROC' from dual union all
select 32697,'NROC' from dual union all
select 32698,'NROC' from dual union all
select 32699,'NASA' from dual union all
select 327,'USAF' from dual union all
select 3270,'RVSN' from dual union all
select 32700,'NROC' from dual union all
select 32701,'NROC' from dual union all
select 32702,'NROC' from dual union all
select 32703,'NROC' from dual union all
select 32704,'NROC' from dual union all
select 32705,'RVSN' from dual union all
select 32706,'NROC' from dual union all
select 32707,'LM' from dual union all
select 32708,'SES' from dual union all
select 32709,'KHRR' from dual;
insert into SATELLITE_ORG
select 3271,'USAF' from dual union all
select 32710,'KVR' from dual union all
select 32711,'USAF' from dual union all
select 32712,'BOE' from dual union all
select 32713,'BOE' from dual union all
select 32714,'NROC' from dual union all
select 32715,'KVR' from dual union all
select 32716,'KVR' from dual union all
select 32717,'KVR' from dual union all
select 32718,'KVR' from dual union all
select 32719,'KVR' from dual union all
select 3272,'PVO' from dual union all
select 32720,'KVR' from dual union all
select 32721,'KVR' from dual union all
select 32722,'KVR' from dual union all
select 32723,'KVR' from dual union all
select 32724,'KVR' from dual union all
select 32725,'KVR' from dual union all
select 32726,'KVR' from dual union all
select 32727,'KVR' from dual union all
select 32728,'KVR' from dual union all
select 32729,'DTV' from dual union all
select 3273,'RVSN' from dual union all
select 32730,'SEALP' from dual union all
select 32731,'NROC' from dual union all
select 32732,'KVR' from dual union all
select 32733,'KVR' from dual union all
select 32734,'KVR' from dual union all
select 32735,'KVR' from dual union all
select 32736,'KVR' from dual union all
select 32737,'KVR' from dual union all
select 32738,'KVR' from dual union all
select 32739,'KVR' from dual union all
select 3274,'MO' from dual union all
select 32740,'KVR' from dual union all
select 32741,'KVR' from dual union all
select 32742,'KVR' from dual union all
select 32743,'KVR' from dual union all
select 32744,'KVR' from dual union all
select 32745,'KVR' from dual union all
select 32746,'KVR' from dual union all
select 32747,'KVR' from dual union all
select 32748,'KVR' from dual union all
select 32749,'NROC' from dual union all
select 3275,'RVSN' from dual union all
select 32750,'BWEHR' from dual union all
select 32751,'KVR' from dual union all
select 32752,'KVR' from dual union all
select 32753,'KVR' from dual union all
select 32754,'KVR' from dual union all
select 32755,'NROC' from dual union all
select 32756,'FKA' from dual union all
select 32757,'KVR' from dual union all
select 32758,'KVR' from dual union all
select 32759,'KVR' from dual union all
select 3276,'MO' from dual union all
select 32760,'KVR' from dual union all
select 32761,'KVR' from dual union all
select 32762,'KVR' from dual union all
select 32763,'ICOH' from dual union all
select 32764,'LM' from dual union all
select 32765,'USAF' from dual union all
select 32766,'OSC' from dual union all
select 32767,'VNPT' from dual union all
select 32768,'TELB' from dual union all
select 32769,'AE' from dual union all
select 3277,'RVSN' from dual union all
select 32770,'AE' from dual union all
select 32771,'KVR' from dual union all
select 32772,'KVR' from dual union all
select 32773,'KVR' from dual union all
select 32774,'KVR' from dual union all
select 32775,'KVR' from dual union all
select 32776,'KVR' from dual union all
select 32777,'KVR' from dual union all
select 32778,'KVR' from dual union all
select 32779,'PLA' from dual union all
select 3278,'NRO' from dual union all
select 32780,'CASC' from dual union all
select 32781,'ESA' from dual union all
select 32782,'KVR' from dual union all
select 32783,'ISRO' from dual union all
select 32784,'COMDEV' from dual union all
select 32784,'UTIAS0' from dual union all
select 32785,'TIT' from dual union all
select 32786,'ISRO' from dual union all
select 32787,'AACH' from dual union all
select 32788,'AAU' from dual union all
select 32789,'DELFT' from dual union all
select 3279,'YUZH' from dual union all
select 32790,'UTIAS0' from dual union all
select 32791,'NIHON' from dual union all
select 32792,'ISRO' from dual union all
select 32793,'KVR' from dual union all
select 32794,'IAI' from dual union all
select 32795,'KVR' from dual union all
select 32796,'KVR' from dual union all
select 32797,'ISRO' from dual union all
select 32798,'KVR' from dual union all
select 32799,'KVR' from dual;
insert into SATELLITE_ORG
select 328,'USAF' from dual union all
select 3280,'RVSN' from dual union all
select 32800,'KVR' from dual union all
select 32801,'KVR' from dual union all
select 32802,'KVR' from dual union all
select 32803,'KVR' from dual union all
select 32804,'KVR' from dual union all
select 32805,'KVR' from dual union all
select 32806,'KVR' from dual union all
select 32807,'KVR' from dual union all
select 32808,'KVR' from dual union all
select 32809,'KVR' from dual union all
select 3281,'MO' from dual union all
select 32810,'KVR' from dual union all
select 32811,'KVR' from dual union all
select 32812,'KVR' from dual union all
select 32813,'KVR' from dual union all
select 32814,'KVR' from dual union all
select 32815,'KVR' from dual union all
select 32816,'KVR' from dual union all
select 32817,'KVR' from dual union all
select 32818,'KVR' from dual union all
select 32819,'KVR' from dual union all
select 3282,'GMS' from dual union all
select 3282,'MO' from dual union all
select 32820,'KVR' from dual union all
select 32821,'KVR' from dual union all
select 32822,'KVR' from dual union all
select 32823,'KVR' from dual union all
select 32824,'KVR' from dual union all
select 32825,'KVR' from dual union all
select 32826,'KVR' from dual union all
select 32827,'KVR' from dual union all
select 32828,'KVR' from dual union all
select 32829,'KVR' from dual union all
select 3283,'RVSN' from dual union all
select 32830,'KVR' from dual union all
select 32831,'KVR' from dual union all
select 32832,'KVR' from dual union all
select 32833,'KVR' from dual union all
select 32834,'KVR' from dual union all
select 32835,'KVR' from dual union all
select 32836,'KVR' from dual union all
select 32837,'KVR' from dual union all
select 32838,'KVR' from dual union all
select 32839,'KVR' from dual union all
select 3284,'USAF' from dual union all
select 32840,'KVR' from dual union all
select 32841,'KVR' from dual union all
select 32842,'KVR' from dual union all
select 32843,'KVR' from dual union all
select 32844,'KVR' from dual union all
select 32845,'KVR' from dual union all
select 32846,'KVR' from dual union all
select 32847,'FKA' from dual union all
select 32848,'KVR' from dual union all
select 32849,'KVR' from dual union all
select 3285,'USAF' from dual union all
select 32850,'KVR' from dual union all
select 32851,'KVR' from dual union all
select 32852,'KVR' from dual union all
select 32853,'KVR' from dual union all
select 32854,'KVR' from dual union all
select 32855,'KVR' from dual union all
select 32856,'KVR' from dual union all
select 32857,'KVR' from dual union all
select 32858,'KVR' from dual union all
select 32859,'KVR' from dual union all
select 3286,'USAF' from dual union all
select 32860,'KVR' from dual union all
select 32861,'KVR' from dual union all
select 32862,'KVR' from dual union all
select 32863,'KVR' from dual union all
select 32864,'KVR' from dual union all
select 32865,'KVR' from dual union all
select 32866,'KVR' from dual union all
select 32867,'KVR' from dual union all
select 32868,'KVR' from dual union all
select 32869,'KVR' from dual union all
select 3287,'USAF' from dual union all
select 32870,'KVR' from dual union all
select 32871,'KVR' from dual union all
select 32872,'KVR' from dual union all
select 32873,'KVR' from dual union all
select 32874,'KVR' from dual union all
select 32875,'KVR' from dual union all
select 32876,'KVR' from dual union all
select 32877,'KVR' from dual union all
select 32878,'KVR' from dual union all
select 32879,'KVR' from dual union all
select 3288,'USAF' from dual union all
select 32880,'KVR' from dual union all
select 32881,'KVR' from dual union all
select 32882,'KVR' from dual union all
select 32883,'KVR' from dual union all
select 32884,'KVR' from dual union all
select 32885,'KVR' from dual union all
select 32886,'KVR' from dual union all
select 32887,'KVR' from dual union all
select 32888,'KVR' from dual;
insert into SATELLITE_ORG
select 32889,'KVR' from dual union all
select 3289,'USAF' from dual union all
select 32890,'KVR' from dual union all
select 32891,'KVR' from dual union all
select 32892,'KVR' from dual union all
select 32893,'KVR' from dual union all
select 32894,'KVR' from dual union all
select 32895,'KVR' from dual union all
select 32896,'KVR' from dual union all
select 32897,'KVR' from dual union all
select 32898,'KVR' from dual union all
select 32899,'KVR' from dual union all
select 329,'USAF' from dual union all
select 3290,'USAF' from dual union all
select 32900,'KVR' from dual union all
select 32901,'KVR' from dual union all
select 32902,'KVR' from dual union all
select 32903,'KVR' from dual union all
select 32904,'KVR' from dual union all
select 32905,'KVR' from dual union all
select 32906,'KVR' from dual union all
select 32907,'KVR' from dual union all
select 32908,'KVR' from dual union all
select 32909,'KVR' from dual union all
select 3291,'USAF' from dual union all
select 32910,'KVR' from dual union all
select 32911,'KVR' from dual union all
select 32912,'KVR' from dual union all
select 32913,'KVR' from dual union all
select 32914,'KVR' from dual union all
select 32915,'KVR' from dual union all
select 32916,'KVR' from dual union all
select 32917,'KVR' from dual union all
select 32918,'KVR' from dual union all
select 32919,'KVR' from dual union all
select 3292,'USAF' from dual union all
select 32920,'KVR' from dual union all
select 32921,'KVR' from dual union all
select 32922,'KVR' from dual union all
select 32923,'KVR' from dual union all
select 32924,'KVR' from dual union all
select 32925,'KVR' from dual union all
select 32926,'KVR' from dual union all
select 32927,'KVR' from dual union all
select 32928,'KVR' from dual union all
select 32929,'KVR' from dual union all
select 3293,'MO' from dual union all
select 32930,'KVR' from dual union all
select 32931,'KVR' from dual union all
select 32932,'KVR' from dual union all
select 32933,'KVR' from dual union all
select 32934,'KVR' from dual union all
select 32935,'KVR' from dual union all
select 32936,'KVR' from dual union all
select 32937,'KVR' from dual union all
select 32938,'KVR' from dual union all
select 32939,'KVR' from dual union all
select 3294,'MO' from dual union all
select 32940,'KVR' from dual union all
select 32941,'KVR' from dual union all
select 32942,'KVR' from dual union all
select 32943,'KVR' from dual union all
select 32944,'KVR' from dual union all
select 32945,'KVR' from dual union all
select 32946,'KVR' from dual union all
select 32947,'KVR' from dual union all
select 32948,'KVR' from dual union all
select 32949,'KVR' from dual union all
select 3295,'RVSN' from dual union all
select 32950,'KVR' from dual union all
select 32951,'INTELB' from dual union all
select 32952,'SEALP' from dual union all
select 32953,'FKA' from dual union all
select 32953,'ROSTO' from dual union all
select 32954,'KVR' from dual union all
select 32955,'KVR' from dual union all
select 32956,'KVR' from dual union all
select 32957,'KVR' from dual union all
select 32958,'CMA' from dual union all
select 32959,'CNSA' from dual union all
select 3296,'CIA' from dual union all
select 3296,'NRO' from dual union all
select 32960,'NASA' from dual union all
select 32961,'PVO' from dual union all
select 32962,'PVO' from dual union all
select 32963,'PVO' from dual union all
select 32964,'PVO' from dual union all
select 32965,'PVO' from dual union all
select 32966,'PVO' from dual union all
select 32967,'PVO' from dual union all
select 32968,'PVO' from dual union all
select 32969,'PVO' from dual union all
select 3297,'NRO' from dual union all
select 32970,'PVO' from dual union all
select 32971,'PVO' from dual union all
select 32972,'PVO' from dual union all
select 32973,'PVO' from dual union all
select 32974,'PVO' from dual union all
select 32975,'PVO' from dual union all
select 32976,'PVO' from dual;
insert into SATELLITE_ORG
select 32977,'PVO' from dual union all
select 32978,'PVO' from dual union all
select 32979,'PVO' from dual union all
select 3298,'TSKBEM' from dual union all
select 32980,'PVO' from dual union all
select 32981,'PVO' from dual union all
select 32982,'PVO' from dual union all
select 32983,'RCA' from dual union all
select 32984,'RCA' from dual union all
select 32985,'RCA' from dual union all
select 32986,'RCA' from dual union all
select 32987,'RCA' from dual union all
select 32988,'RCA' from dual union all
select 32989,'RCA' from dual union all
select 3299,'MO' from dual union all
select 32990,'RCA' from dual union all
select 32991,'RCA' from dual union all
select 32992,'RCA' from dual union all
select 32993,'RCA' from dual union all
select 32994,'PVO' from dual union all
select 32995,'PVO' from dual union all
select 32996,'PVO' from dual union all
select 32997,'MO' from dual union all
select 32998,'AE' from dual union all
select 32999,'AE' from dual union all
select 33,'USN' from dual union all
select 330,'USAF' from dual union all
select 3300,'RVSN' from dual union all
select 33000,'NASA' from dual union all
select 33001,'KVR' from dual union all
select 33002,'KVR' from dual union all
select 33003,'KVR' from dual union all
select 33004,'KVR' from dual union all
select 33005,'KVR' from dual union all
select 33006,'KVR' from dual union all
select 33007,'KVR' from dual union all
select 33008,'KVR' from dual union all
select 33009,'KVR' from dual union all
select 3301,'MO' from dual union all
select 33010,'KVR' from dual union all
select 33011,'KVR' from dual union all
select 33012,'KVR' from dual union all
select 33013,'KVR' from dual union all
select 33014,'KVR' from dual union all
select 33015,'KVR' from dual union all
select 33016,'KVR' from dual union all
select 33017,'KVR' from dual union all
select 33018,'KVR' from dual union all
select 33019,'KVR' from dual union all
select 3302,'MO' from dual union all
select 33020,'KVR' from dual union all
select 33021,'KVR' from dual union all
select 33022,'KVR' from dual union all
select 33023,'KVR' from dual union all
select 33024,'KVR' from dual union all
select 33025,'KVR' from dual union all
select 33026,'KVR' from dual union all
select 33027,'KVR' from dual union all
select 33028,'KVR' from dual union all
select 33029,'KVR' from dual union all
select 3303,'MO' from dual union all
select 33030,'KVR' from dual union all
select 33031,'KVR' from dual union all
select 33032,'KVR' from dual union all
select 33033,'KVR' from dual union all
select 33034,'KVR' from dual union all
select 33035,'KVR' from dual union all
select 33036,'KVR' from dual union all
select 33037,'KVR' from dual union all
select 33038,'KVR' from dual union all
select 33039,'KVR' from dual union all
select 3304,'MO' from dual union all
select 33040,'KVR' from dual union all
select 33041,'KVR' from dual union all
select 33042,'KVR' from dual union all
select 33043,'KVR' from dual union all
select 33044,'KVR' from dual union all
select 33045,'KVR' from dual union all
select 33046,'KVR' from dual union all
select 33047,'KVR' from dual union all
select 33048,'KVR' from dual union all
select 33049,'KVR' from dual union all
select 3305,'RVSN' from dual union all
select 33050,'KVR' from dual union all
select 33051,'CHISA' from dual union all
select 33052,'CASC' from dual union all
select 33053,'NASA' from dual union all
select 33054,'BOE' from dual union all
select 33055,'PARA' from dual union all
select 33056,'TURKS' from dual union all
select 33057,'AE' from dual union all
select 33058,'AE' from dual union all
select 33059,'SEALP' from dual union all
select 3306,'TSKBEM' from dual union all
select 33060,'ORBC' from dual union all
select 33061,'ORBC' from dual union all
select 33062,'ORBC' from dual union all
select 33063,'ORBC' from dual union all
select 33064,'ORBC' from dual union all
select 33065,'ORBC' from dual;
insert into SATELLITE_ORG
select 33066,'KVR' from dual union all
select 33067,'CASC' from dual union all
select 33068,'KVR' from dual union all
select 33069,'KVR' from dual union all
select 3307,'NASA' from dual union all
select 33070,'KVR' from dual union all
select 33071,'KVR' from dual union all
select 33072,'KVR' from dual union all
select 33073,'KVR' from dual union all
select 33074,'KVR' from dual union all
select 33075,'KVR' from dual union all
select 33076,'KVR' from dual union all
select 33077,'KVR' from dual union all
select 33078,'KVR' from dual union all
select 33079,'KVR' from dual union all
select 3308,'YUZH' from dual union all
select 33080,'KVR' from dual union all
select 33081,'KVR' from dual union all
select 33082,'KVR' from dual union all
select 33083,'KVR' from dual union all
select 33084,'KVR' from dual union all
select 33085,'KVR' from dual union all
select 33086,'KVR' from dual union all
select 33087,'KVR' from dual union all
select 33088,'KVR' from dual union all
select 33089,'KVR' from dual union all
select 3309,'RVSN' from dual union all
select 33090,'KVR' from dual union all
select 33091,'KVR' from dual union all
select 33092,'KVR' from dual union all
select 33093,'KVR' from dual union all
select 33094,'KVR' from dual union all
select 33095,'KVR' from dual union all
select 33096,'KVR' from dual union all
select 33097,'KVR' from dual union all
select 33098,'KVR' from dual union all
select 33099,'KVR' from dual union all
select 331,'USAF' from dual union all
select 3310,'MO' from dual union all
select 33100,'KVR' from dual union all
select 33101,'KVR' from dual union all
select 33102,'KVR' from dual union all
select 33103,'KVR' from dual union all
select 33104,'KVR' from dual union all
select 33105,'CNES' from dual union all
select 33105,'JPL' from dual union all
select 33106,'BOE' from dual union all
select 33107,'KVR' from dual union all
select 33108,'PVO' from dual union all
select 33109,'KHRR' from dual union all
select 3311,'RVSN' from dual union all
select 33110,'KHRR' from dual union all
select 33111,'KHRR' from dual union all
select 33112,'RVSNR' from dual union all
select 33113,'RVSNR' from dual union all
select 33114,'KVR' from dual union all
select 33115,'KVR' from dual union all
select 33116,'KVR' from dual union all
select 33117,'KVR' from dual union all
select 33118,'KVR' from dual union all
select 33119,'KVR' from dual union all
select 3312,'RVSN' from dual union all
select 33120,'KVR' from dual union all
select 33121,'KVR' from dual union all
select 33122,'KVR' from dual union all
select 33123,'KVR' from dual union all
select 33124,'KVR' from dual union all
select 33125,'KVR' from dual union all
select 33126,'KVR' from dual union all
select 33127,'KVR' from dual union all
select 33128,'KVR' from dual union all
select 33129,'KVR' from dual union all
select 3313,'RVSN' from dual union all
select 33130,'KVR' from dual union all
select 33131,'KVR' from dual union all
select 33132,'KVR' from dual union all
select 33133,'KVR' from dual union all
select 33134,'KVR' from dual union all
select 33135,'KVR' from dual union all
select 33136,'KVR' from dual union all
select 33137,'KVR' from dual union all
select 33138,'KVR' from dual union all
select 33139,'KVR' from dual union all
select 3314,'RVSN' from dual union all
select 33140,'KVR' from dual union all
select 33141,'KVR' from dual union all
select 33142,'KVR' from dual union all
select 33143,'KVR' from dual union all
select 33144,'KVR' from dual union all
select 33145,'KVR' from dual union all
select 33146,'KVR' from dual union all
select 33147,'KVR' from dual union all
select 33148,'KVR' from dual union all
select 33149,'KVR' from dual union all
select 3315,'NASA' from dual union all
select 33150,'KVR' from dual union all
select 33151,'KVR' from dual union all
select 33152,'KVR' from dual union all
select 33153,'PROTO' from dual union all
select 33154,'ARAB' from dual;
insert into SATELLITE_ORG
select 33155,'AE' from dual union all
select 33156,'AE' from dual union all
select 33157,'KVR' from dual union all
select 33158,'KVR' from dual union all
select 33159,'KVR' from dual union all
select 3316,'MO' from dual union all
select 33160,'KVR' from dual union all
select 33161,'KVR' from dual union all
select 33162,'KVR' from dual union all
select 33163,'KVR' from dual union all
select 33164,'KVR' from dual union all
select 33165,'KVR' from dual union all
select 33166,'KVR' from dual union all
select 33167,'KVR' from dual union all
select 33168,'KVR' from dual union all
select 33169,'KVR' from dual union all
select 3317,'RVSN' from dual union all
select 33170,'KVR' from dual union all
select 33171,'KVR' from dual union all
select 33172,'KVR' from dual union all
select 33173,'KVR' from dual union all
select 33174,'KVR' from dual union all
select 33175,'KVR' from dual union all
select 33176,'KVR' from dual union all
select 33177,'KVR' from dual union all
select 33178,'KVR' from dual union all
select 33179,'KVR' from dual union all
select 3318,'USAF' from dual union all
select 33180,'KVR' from dual union all
select 33181,'KVR' from dual union all
select 33182,'KVR' from dual union all
select 33183,'KVR' from dual union all
select 33184,'KVR' from dual union all
select 33185,'KVR' from dual union all
select 33186,'KVR' from dual union all
select 33187,'KVR' from dual union all
select 33188,'KVR' from dual union all
select 33189,'KVR' from dual union all
select 3319,'USAF' from dual union all
select 33190,'KVR' from dual union all
select 33191,'KVR' from dual union all
select 33192,'KVR' from dual union all
select 33193,'KVR' from dual union all
select 33194,'KVR' from dual union all
select 33195,'KVR' from dual union all
select 33196,'KVR' from dual union all
select 33197,'KVR' from dual union all
select 33198,'KVR' from dual union all
select 33199,'KVR' from dual union all
select 332,'USAF' from dual union all
select 3320,'USAF' from dual union all
select 33200,'KVR' from dual union all
select 33201,'KVR' from dual union all
select 33202,'FKA' from dual union all
select 33203,'FKA' from dual union all
select 33204,'FKA' from dual union all
select 33205,'FKA' from dual union all
select 33206,'FKA' from dual union all
select 33207,'DISH' from dual union all
select 33208,'SEALP' from dual union all
select 33209,'KVR' from dual union all
select 3321,'USAF' from dual union all
select 33210,'KVR' from dual union all
select 33211,'KVR' from dual union all
select 33212,'KVR' from dual union all
select 33213,'KVR' from dual union all
select 33214,'KVR' from dual union all
select 33215,'KVR' from dual union all
select 33216,'KVR' from dual union all
select 33217,'KVR' from dual union all
select 33218,'KVR' from dual union all
select 33219,'KVR' from dual union all
select 3322,'MO' from dual union all
select 33220,'KVR' from dual union all
select 33221,'KVR' from dual union all
select 33222,'KVR' from dual union all
select 33223,'KVR' from dual union all
select 33224,'KVR' from dual union all
select 33225,'KVR' from dual union all
select 33226,'KVR' from dual union all
select 33227,'KVR' from dual union all
select 33228,'KVR' from dual union all
select 33229,'KVR' from dual union all
select 3323,'RVSN' from dual union all
select 33230,'KVR' from dual union all
select 33231,'KVR' from dual union all
select 33232,'KVR' from dual union all
select 33233,'KVR' from dual union all
select 33234,'KVR' from dual union all
select 33235,'KVR' from dual union all
select 33236,'KVR' from dual union all
select 33237,'KVR' from dual union all
select 33238,'KVR' from dual union all
select 33239,'KVR' from dual union all
select 3324,'USAF' from dual union all
select 33240,'KVR' from dual union all
select 33241,'KVR' from dual union all
select 33242,'KVR' from dual union all
select 33243,'FKA' from dual union all
select 33244,'BWEHR' from dual;
insert into SATELLITE_ORG
select 33245,'KVR' from dual union all
select 33246,'KVR' from dual union all
select 33247,'KVR' from dual union all
select 33248,'KVR' from dual union all
select 33249,'KVR' from dual union all
select 3325,'USAF' from dual union all
select 33250,'KVR' from dual union all
select 33251,'KVR' from dual union all
select 33252,'KVR' from dual union all
select 33253,'KVR' from dual union all
select 33254,'KVR' from dual union all
select 33255,'KVR' from dual union all
select 33256,'KVR' from dual union all
select 33257,'KVR' from dual union all
select 33258,'KVR' from dual union all
select 33259,'KVR' from dual union all
select 3326,'PVO' from dual union all
select 33260,'KVR' from dual union all
select 33261,'KVR' from dual union all
select 33262,'KVR' from dual union all
select 33263,'KVR' from dual union all
select 33264,'KVR' from dual union all
select 33265,'KVR' from dual union all
select 33266,'KVR' from dual union all
select 33267,'KVR' from dual union all
select 33268,'KVR' from dual union all
select 33269,'KVR' from dual union all
select 3327,'RVSN' from dual union all
select 33270,'KVR' from dual union all
select 33271,'KVR' from dual union all
select 33272,'KVR' from dual union all
select 33273,'KVR' from dual union all
select 33274,'SKPJ' from dual union all
select 33275,'SES' from dual union all
select 33276,'AE' from dual union all
select 33277,'AE' from dual union all
select 33278,'INMRL' from dual union all
select 33279,'KHRR' from dual union all
select 3328,'NASA' from dual union all
select 33280,'KHRR' from dual union all
select 33281,'RGMS' from dual union all
select 33282,'KVR' from dual union all
select 33283,'KVR' from dual union all
select 33284,'KVR' from dual union all
select 33285,'KVR' from dual union all
select 33286,'KVR' from dual union all
select 33287,'KVR' from dual union all
select 33288,'KVR' from dual union all
select 33289,'KVR' from dual union all
select 3329,'USAF' from dual union all
select 33290,'KVR' from dual union all
select 33291,'KVR' from dual union all
select 33292,'KVR' from dual union all
select 33293,'KVR' from dual union all
select 33294,'KVR' from dual union all
select 33295,'KVR' from dual union all
select 33296,'KVR' from dual union all
select 33297,'KVR' from dual union all
select 33298,'KVR' from dual union all
select 33299,'KVR' from dual union all
select 333,'USAF' from dual union all
select 3330,'USAF' from dual union all
select 33300,'KVR' from dual union all
select 33301,'KVR' from dual union all
select 33302,'KVR' from dual union all
select 33303,'KVR' from dual union all
select 33304,'KVR' from dual union all
select 33305,'KVR' from dual union all
select 33306,'KVR' from dual union all
select 33307,'KVR' from dual union all
select 33308,'KVR' from dual union all
select 33309,'KVR' from dual union all
select 3331,'USAF' from dual union all
select 33310,'KVR' from dual union all
select 33311,'KVR' from dual union all
select 33312,'REYE' from dual union all
select 33313,'REYE' from dual union all
select 33314,'REYE' from dual union all
select 33315,'REYE' from dual union all
select 33316,'REYE' from dual union all
select 33317,'KTRAS' from dual union all
select 33318,'KTRAS' from dual union all
select 33319,'KTRAS' from dual union all
select 3332,'MO' from dual union all
select 33320,'GZB' from dual union all
select 33321,'GZB' from dual union all
select 33322,'CASC' from dual union all
select 33323,'CASC' from dual union all
select 33324,'CASC' from dual union all
select 33325,'CASC' from dual union all
select 33326,'CASC' from dual union all
select 33327,'CASC' from dual union all
select 33328,'CASC' from dual union all
select 33329,'CASC' from dual union all
select 3333,'RVSN' from dual union all
select 33330,'CASC' from dual union all
select 33331,'GEYE' from dual union all
select 33332,'BOE' from dual union all
select 33333,'CASC' from dual union all
select 33334,'CASC' from dual;
insert into SATELLITE_ORG
select 33335,'CASC' from dual union all
select 33336,'CASC' from dual union all
select 33337,'CASC' from dual union all
select 33338,'CASC' from dual union all
select 33339,'CASC' from dual union all
select 3334,'NRO' from dual union all
select 33340,'FKA' from dual union all
select 33341,'KVR' from dual union all
select 33342,'CASC' from dual union all
select 33343,'CASC' from dual union all
select 33344,'CASC' from dual union all
select 33345,'CASC' from dual union all
select 33346,'CASC' from dual union all
select 33347,'CASC' from dual union all
select 33348,'KVR' from dual union all
select 33349,'KVR' from dual union all
select 3335,'NRO' from dual union all
select 33350,'KVR' from dual union all
select 33351,'KVR' from dual union all
select 33352,'KVR' from dual union all
select 33353,'KVR' from dual union all
select 33354,'KVR' from dual union all
select 33355,'KVR' from dual union all
select 33356,'KVR' from dual union all
select 33357,'KVR' from dual union all
select 33358,'KVR' from dual union all
select 33359,'KVR' from dual union all
select 3336,'CIA' from dual union all
select 3336,'NRO' from dual union all
select 33360,'KVR' from dual union all
select 33361,'KVR' from dual union all
select 33362,'KVR' from dual union all
select 33363,'KVR' from dual union all
select 33364,'KVR' from dual union all
select 33365,'KVR' from dual union all
select 33366,'KVR' from dual union all
select 33367,'KVR' from dual union all
select 33368,'KVR' from dual union all
select 33369,'KVR' from dual union all
select 3337,'NASA' from dual union all
select 33370,'KVR' from dual union all
select 33371,'KVR' from dual union all
select 33372,'CASC' from dual union all
select 33373,'TCAN' from dual union all
select 33374,'KHRR' from dual union all
select 33375,'KHRR' from dual union all
select 33376,'INTELB' from dual union all
select 33377,'SEALP' from dual union all
select 33378,'FKA' from dual union all
select 33378,'KVR' from dual union all
select 33379,'FKA' from dual union all
select 33379,'KVR' from dual union all
select 3338,'NASA' from dual union all
select 33380,'FKA' from dual union all
select 33380,'KVR' from dual union all
select 33381,'KHRR' from dual union all
select 33382,'KHRR' from dual union all
select 33383,'KHRR' from dual union all
select 33384,'KVR' from dual union all
select 33385,'KVR' from dual union all
select 33386,'PLA' from dual union all
select 33387,'CASC' from dual union all
select 33388,'CASC' from dual union all
select 33389,'CASC' from dual union all
select 3339,'MO' from dual union all
select 33390,'CASC' from dual union all
select 33391,'CASC' from dual union all
select 33392,'PLA' from dual union all
select 33393,'SPX' from dual union all
select 33394,'PLA' from dual union all
select 33395,'USN' from dual union all
select 33396,'GISTDA' from dual union all
select 33397,'KTRAS' from dual union all
select 33398,'KTRAS' from dual union all
select 33399,'FKA' from dual union all
select 334,'USAF' from dual union all
select 3340,'RVSN' from dual union all
select 33400,'KVR' from dual union all
select 33401,'NASA' from dual union all
select 33402,'OSC' from dual union all
select 33403,'NASA' from dual union all
select 33404,'NASA' from dual union all
select 33405,'ISRO' from dual union all
select 33406,'ISRO' from dual union all
select 33407,'ISRO' from dual union all
select 33408,'CASC' from dual union all
select 33409,'CASC' from dual union all
select 3341,'NASA' from dual union all
select 33410,'CNSA' from dual union all
select 33411,'CNSA' from dual union all
select 33412,'ASI' from dual union all
select 33413,'BOE' from dual union all
select 33414,'ABAE' from dual union all
select 33415,'CASC' from dual union all
select 33416,'KVR' from dual union all
select 33417,'KVR' from dual union all
select 33418,'KVR' from dual union all
select 33419,'KVR' from dual union all
select 3342,'NASA' from dual union all
select 33420,'KVR' from dual;
insert into SATELLITE_ORG
select 33421,'KVR' from dual union all
select 33422,'KVR' from dual union all
select 33423,'KVR' from dual union all
select 33424,'KVR' from dual union all
select 33425,'KVR' from dual union all
select 33426,'KVR' from dual union all
select 33427,'KVR' from dual union all
select 33428,'KVR' from dual union all
select 33429,'KVR' from dual union all
select 3343,'NASA' from dual union all
select 33430,'KVR' from dual union all
select 33431,'KVR' from dual union all
select 33432,'KVR' from dual union all
select 33433,'CASC' from dual union all
select 33434,'CASC' from dual union all
select 33435,'CNSA' from dual union all
select 33436,'SES' from dual union all
select 33437,'KHRR' from dual union all
select 33438,'KHRR' from dual union all
select 33439,'KVR' from dual union all
select 3344,'NASA' from dual union all
select 33440,'KVR' from dual union all
select 33441,'NASA' from dual union all
select 33442,'NASA' from dual union all
select 33443,'FKA' from dual union all
select 33444,'KVR' from dual union all
select 33445,'USAF' from dual union all
select 33446,'PLA' from dual union all
select 33447,'PVO' from dual union all
select 33448,'RVSNR' from dual union all
select 33449,'RVSNR' from dual union all
select 3345,'ESSA' from dual union all
select 33450,'RVSNR' from dual union all
select 33451,'RVSNR' from dual union all
select 33452,'CASC' from dual union all
select 33453,'CIEL' from dual union all
select 33454,'KHRR' from dual union all
select 33455,'KHRR' from dual union all
select 33456,'PLA' from dual union all
select 33457,'CNSA' from dual union all
select 33458,'CASC' from dual union all
select 33459,'EUTSA' from dual union all
select 3346,'NASA' from dual union all
select 33460,'EUTSA' from dual union all
select 33461,'AE' from dual union all
select 33462,'AE' from dual union all
select 33463,'CMA' from dual union all
select 33464,'CASC' from dual union all
select 33465,'CASC' from dual union all
select 33466,'FKA' from dual union all
select 33466,'KVR' from dual union all
select 33467,'FKA' from dual union all
select 33467,'KVR' from dual union all
select 33468,'FKA' from dual union all
select 33468,'KVR' from dual union all
select 33469,'KHRR' from dual union all
select 3347,'OKB10' from dual union all
select 33470,'KHRR' from dual union all
select 33471,'KHRR' from dual union all
select 33472,'KVR' from dual union all
select 33473,'KVR' from dual union all
select 33474,'KVR' from dual union all
select 33475,'KVR' from dual union all
select 33476,'KVR' from dual union all
select 33477,'KVR' from dual union all
select 33478,'KVR' from dual union all
select 33479,'KVR' from dual union all
select 3348,'MO' from dual union all
select 33480,'KVR' from dual union all
select 33481,'KVR' from dual union all
select 33482,'KVR' from dual union all
select 33483,'KVR' from dual union all
select 33484,'CASC' from dual union all
select 33485,'MO' from dual union all
select 33486,'MO' from dual union all
select 33487,'MO' from dual union all
select 33488,'MO' from dual union all
select 33489,'MO' from dual union all
select 3349,'RVSN' from dual union all
select 33490,'NROC' from dual union all
select 33491,'BOE' from dual union all
select 33492,'JAXA' from dual union all
select 33493,'TOK' from dual union all
select 33494,'TOHO' from dual union all
select 33495,'SORUN' from dual union all
select 33496,'JAXA' from dual union all
select 33496,'SOHLA' from dual union all
select 33497,'JAXA' from dual union all
select 33498,'KAGA' from dual union all
select 33499,'KOUKO' from dual union all
select 335,'USAF' from dual union all
select 3350,'RVSN' from dual union all
select 33500,'JAXA' from dual union all
select 33501,'JAXA' from dual union all
select 33502,'CASC' from dual union all
select 33503,'MO' from dual union all
select 33504,'FKA' from dual union all
select 33504,'IKI' from dual union all
select 33505,'KVR' from dual union all
select 33506,'IRSA' from dual;
insert into SATELLITE_ORG
select 33507,'IRSA' from dual union all
select 33508,'USN' from dual union all
select 33509,'USAF' from dual union all
select 3351,'TSKBEM' from dual union all
select 33510,'USAF' from dual union all
select 33511,'USAF' from dual union all
select 33512,'USAF' from dual union all
select 33513,'USAF' from dual union all
select 33514,'CRC' from dual union all
select 33515,'CRC' from dual union all
select 33516,'CRC' from dual union all
select 33517,'CRC' from dual union all
select 33518,'CRC' from dual union all
select 33519,'RVSN' from dual union all
select 3352,'RVSN' from dual union all
select 33520,'MO' from dual union all
select 33521,'NASA' from dual union all
select 33522,'RVSN' from dual union all
select 33523,'RVSN' from dual union all
select 33524,'RVSN' from dual union all
select 33525,'RVSN' from dual union all
select 33526,'RVSN' from dual union all
select 33527,'RVSN' from dual union all
select 33528,'RVSN' from dual union all
select 33529,'MO' from dual union all
select 3353,'MO' from dual union all
select 33530,'MO' from dual union all
select 33531,'MO' from dual union all
select 33532,'KVR' from dual union all
select 33533,'AE' from dual union all
select 33534,'AE' from dual union all
select 33535,'AE' from dual union all
select 33536,'AE' from dual union all
select 33537,'AE' from dual union all
select 33538,'AE' from dual union all
select 33539,'AE' from dual union all
select 3354,'RVSN' from dual union all
select 33540,'AE' from dual union all
select 33541,'AE' from dual union all
select 33542,'AE' from dual union all
select 33543,'AE' from dual union all
select 33544,'AE' from dual union all
select 33545,'AE' from dual union all
select 33546,'AE' from dual union all
select 33547,'AE' from dual union all
select 33548,'AE' from dual union all
select 33549,'AE' from dual union all
select 3355,'USAF' from dual union all
select 33550,'AE' from dual union all
select 33551,'AE' from dual union all
select 33552,'AE' from dual union all
select 33553,'AE' from dual union all
select 33554,'AE' from dual union all
select 33555,'AE' from dual union all
select 33556,'AE' from dual union all
select 33557,'AE' from dual union all
select 33558,'AE' from dual union all
select 33559,'AE' from dual union all
select 3356,'USAF' from dual union all
select 33560,'AE' from dual union all
select 33561,'AE' from dual union all
select 33562,'AE' from dual union all
select 33563,'AE' from dual union all
select 33564,'AE' from dual union all
select 33565,'AE' from dual union all
select 33566,'AE' from dual union all
select 33567,'AE' from dual union all
select 33568,'AE' from dual union all
select 33569,'AE' from dual union all
select 3357,'USAF' from dual union all
select 33570,'AE' from dual union all
select 33571,'AE' from dual union all
select 33572,'AE' from dual union all
select 33573,'AE' from dual union all
select 33574,'AE' from dual union all
select 33575,'AE' from dual union all
select 33576,'AE' from dual union all
select 33577,'AE' from dual union all
select 33578,'AE' from dual union all
select 33579,'AE' from dual union all
select 3358,'USAF' from dual union all
select 33580,'AE' from dual union all
select 33581,'AE' from dual union all
select 33582,'AE' from dual union all
select 33583,'AE' from dual union all
select 33584,'AE' from dual union all
select 33585,'KVR' from dual union all
select 33586,'KVR' from dual union all
select 33587,'KVR' from dual union all
select 33588,'AE' from dual union all
select 33589,'AE' from dual union all
select 3359,'USAF' from dual union all
select 33590,'AE' from dual union all
select 33591,'NOAA' from dual union all
select 33592,'BOE' from dual union all
select 33593,'FKA' from dual union all
select 33594,'KVR' from dual union all
select 33595,'KS' from dual union all
select 33596,'KS' from dual union all
select 33597,'KHRR' from dual;
insert into SATELLITE_ORG
select 33598,'KHRR' from dual union all
select 33599,'CASC' from dual union all
select 336,'USAF' from dual union all
select 3360,'USAF' from dual union all
select 33600,'CASC' from dual union all
select 33601,'CASC' from dual union all
select 33602,'CASC' from dual union all
select 33603,'CASC' from dual union all
select 33604,'CASC' from dual union all
select 33605,'CASC' from dual union all
select 33606,'CASC' from dual union all
select 33607,'CASC' from dual union all
select 33608,'CASC' from dual union all
select 33609,'CASC' from dual union all
select 3361,'USAF' from dual union all
select 33610,'CASC' from dual union all
select 33611,'CASC' from dual union all
select 33612,'CASC' from dual union all
select 33613,'CASC' from dual union all
select 33614,'CASC' from dual union all
select 33615,'CASC' from dual union all
select 33616,'CASC' from dual union all
select 33617,'CASC' from dual union all
select 33618,'CASC' from dual union all
select 33619,'CASC' from dual union all
select 3362,'USAF' from dual union all
select 33620,'CASC' from dual union all
select 33621,'CASC' from dual union all
select 33622,'CASC' from dual union all
select 33623,'CASC' from dual union all
select 33624,'CASC' from dual union all
select 33625,'CASC' from dual union all
select 33626,'CASC' from dual union all
select 33627,'CASC' from dual union all
select 33628,'CASC' from dual union all
select 33629,'CASC' from dual union all
select 3363,'USAF' from dual union all
select 33630,'CASC' from dual union all
select 33631,'CASC' from dual union all
select 33632,'CASC' from dual union all
select 33633,'CASC' from dual union all
select 33634,'CASC' from dual union all
select 33635,'CASC' from dual union all
select 33636,'CASC' from dual union all
select 33637,'CASC' from dual union all
select 33638,'CASC' from dual union all
select 33639,'CASC' from dual union all
select 3364,'USAF' from dual union all
select 33640,'CASC' from dual union all
select 33641,'CASC' from dual union all
select 33642,'CASC' from dual union all
select 33643,'CASC' from dual union all
select 33644,'CASC' from dual union all
select 33645,'CASC' from dual union all
select 33646,'CASC' from dual union all
select 33647,'CASC' from dual union all
select 33648,'CASC' from dual union all
select 33649,'CASC' from dual union all
select 3365,'USAF' from dual union all
select 33650,'CASC' from dual union all
select 33651,'CASC' from dual union all
select 33652,'CASC' from dual union all
select 33653,'CASC' from dual union all
select 33654,'CASC' from dual union all
select 33655,'CASC' from dual union all
select 33656,'CASC' from dual union all
select 33657,'CASC' from dual union all
select 33658,'CASC' from dual union all
select 33659,'CASC' from dual union all
select 3366,'USAF' from dual union all
select 33660,'CASC' from dual union all
select 33661,'CASC' from dual union all
select 33662,'CASC' from dual union all
select 33663,'CASC' from dual union all
select 33664,'CASC' from dual union all
select 33665,'CASC' from dual union all
select 33666,'CASC' from dual union all
select 33667,'CASC' from dual union all
select 33668,'CASC' from dual union all
select 33669,'CASC' from dual union all
select 3367,'USAF' from dual union all
select 33670,'CASC' from dual union all
select 33671,'CASC' from dual union all
select 33672,'CASC' from dual union all
select 33673,'CASC' from dual union all
select 33674,'CASC' from dual union all
select 33675,'CASC' from dual union all
select 33676,'CASC' from dual union all
select 33677,'CASC' from dual union all
select 33678,'CASC' from dual union all
select 33679,'CASC' from dual union all
select 3368,'USAF' from dual union all
select 33680,'CASC' from dual union all
select 33681,'CASC' from dual union all
select 33682,'CASC' from dual union all
select 33683,'CASC' from dual union all
select 33684,'CASC' from dual union all
select 33685,'CASC' from dual union all
select 33686,'CASC' from dual union all
select 33687,'CASC' from dual;
insert into SATELLITE_ORG
select 33688,'CASC' from dual union all
select 33689,'CASC' from dual union all
select 3369,'USAF' from dual union all
select 33690,'CASC' from dual union all
select 33691,'CASC' from dual union all
select 33692,'CASC' from dual union all
select 33693,'CASC' from dual union all
select 33694,'CASC' from dual union all
select 33695,'CASC' from dual union all
select 33696,'CASC' from dual union all
select 33697,'CASC' from dual union all
select 33698,'CASC' from dual union all
select 33699,'CASC' from dual union all
select 337,'USAF' from dual union all
select 3370,'USAF' from dual union all
select 33700,'CASC' from dual union all
select 33701,'CASC' from dual union all
select 33702,'CASC' from dual union all
select 33703,'CASC' from dual union all
select 33704,'CASC' from dual union all
select 33705,'CASC' from dual union all
select 33706,'CASC' from dual union all
select 33707,'CASC' from dual union all
select 33708,'CASC' from dual union all
select 33709,'CASC' from dual union all
select 3371,'USAF' from dual union all
select 33710,'CASC' from dual union all
select 33711,'CASC' from dual union all
select 33712,'CASC' from dual union all
select 33713,'CASC' from dual union all
select 33714,'CASC' from dual union all
select 33715,'CASC' from dual union all
select 33716,'CASC' from dual union all
select 33717,'CASC' from dual union all
select 33718,'CASC' from dual union all
select 33719,'CASC' from dual union all
select 3372,'USAF' from dual union all
select 33720,'CASC' from dual union all
select 33721,'CASC' from dual union all
select 33722,'CASC' from dual union all
select 33723,'CASC' from dual union all
select 33724,'CASC' from dual union all
select 33725,'CASC' from dual union all
select 33726,'CASC' from dual union all
select 33727,'CASC' from dual union all
select 33728,'CASC' from dual union all
select 33729,'CASC' from dual union all
select 3373,'USAF' from dual union all
select 33730,'CASC' from dual union all
select 33731,'CASC' from dual union all
select 33732,'CASC' from dual union all
select 33733,'CASC' from dual union all
select 33734,'CASC' from dual union all
select 33735,'CASC' from dual union all
select 33736,'CASC' from dual union all
select 33737,'CASC' from dual union all
select 33738,'CASC' from dual union all
select 33739,'CASC' from dual union all
select 3374,'USAF' from dual union all
select 33740,'CASC' from dual union all
select 33741,'CASC' from dual union all
select 33742,'CASC' from dual union all
select 33743,'CASC' from dual union all
select 33744,'CASC' from dual union all
select 33745,'CASC' from dual union all
select 33746,'CASC' from dual union all
select 33747,'CASC' from dual union all
select 33748,'CASC' from dual union all
select 33749,'SES' from dual union all
select 3375,'NRO' from dual union all
select 33750,'EUTSA' from dual union all
select 33751,'DGA' from dual union all
select 33752,'DGA' from dual union all
select 33753,'AE' from dual union all
select 33754,'AE' from dual union all
select 33755,'KVR' from dual union all
select 33756,'CNES' from dual union all
select 33757,'KVR' from dual union all
select 33758,'KVR' from dual union all
select 33759,'KVR' from dual union all
select 3376,'USAF' from dual union all
select 33760,'KVR' from dual union all
select 33761,'KVR' from dual union all
select 33762,'KVR' from dual union all
select 33763,'KVR' from dual union all
select 33764,'KVR' from dual union all
select 33765,'KVR' from dual union all
select 33766,'KVR' from dual union all
select 33767,'KVR' from dual union all
select 33768,'KVR' from dual union all
select 33769,'KVR' from dual union all
select 3377,'USAF' from dual union all
select 33770,'KVR' from dual union all
select 33771,'IRIDS' from dual union all
select 33772,'IRIDS' from dual union all
select 33773,'IRIDS' from dual union all
select 33774,'IRIDS' from dual union all
select 33775,'IRIDS' from dual union all
select 33776,'IRIDS' from dual union all
select 33777,'IRIDS' from dual;
insert into SATELLITE_ORG
select 33778,'IRIDS' from dual union all
select 33779,'KVR' from dual union all
select 3378,'USAF' from dual union all
select 33780,'KVR' from dual union all
select 33781,'KVR' from dual union all
select 33782,'KVR' from dual union all
select 33783,'KVR' from dual union all
select 33784,'KVR' from dual union all
select 33785,'KVR' from dual union all
select 33786,'KVR' from dual union all
select 33787,'KVR' from dual union all
select 33788,'KVR' from dual union all
select 33789,'KVR' from dual union all
select 3379,'USAF' from dual union all
select 33790,'KVR' from dual union all
select 33791,'KVR' from dual union all
select 33792,'KVR' from dual union all
select 33793,'KVR' from dual union all
select 33794,'KVR' from dual union all
select 33795,'KVR' from dual union all
select 33796,'KVR' from dual union all
select 33797,'KVR' from dual union all
select 33798,'KVR' from dual union all
select 33799,'KVR' from dual union all
select 338,'OKB586' from dual union all
select 338,'PVO' from dual union all
select 3380,'USAF' from dual union all
select 33800,'KVR' from dual union all
select 33801,'KVR' from dual union all
select 33802,'KVR' from dual union all
select 33803,'KVR' from dual union all
select 33804,'KVR' from dual union all
select 33805,'KVR' from dual union all
select 33806,'KVR' from dual union all
select 33807,'KVR' from dual union all
select 33808,'KVR' from dual union all
select 33809,'KVR' from dual union all
select 3381,'USAF' from dual union all
select 33810,'KVR' from dual union all
select 33811,'KVR' from dual union all
select 33812,'KVR' from dual union all
select 33813,'KVR' from dual union all
select 33814,'KVR' from dual union all
select 33815,'KVR' from dual union all
select 33816,'KVR' from dual union all
select 33817,'KVR' from dual union all
select 33818,'KVR' from dual union all
select 33819,'KVR' from dual union all
select 3382,'USAF' from dual union all
select 33820,'KVR' from dual union all
select 33821,'KVR' from dual union all
select 33822,'KVR' from dual union all
select 33823,'KVR' from dual union all
select 33824,'KVR' from dual union all
select 33825,'KVR' from dual union all
select 33826,'KVR' from dual union all
select 33827,'KVR' from dual union all
select 33828,'KVR' from dual union all
select 33829,'KVR' from dual union all
select 3383,'USAF' from dual union all
select 33830,'KVR' from dual union all
select 33831,'KVR' from dual union all
select 33832,'KVR' from dual union all
select 33833,'KVR' from dual union all
select 33834,'KVR' from dual union all
select 33835,'KVR' from dual union all
select 33836,'KVR' from dual union all
select 33837,'KVR' from dual union all
select 33838,'KVR' from dual union all
select 33839,'KVR' from dual union all
select 3384,'USAF' from dual union all
select 33840,'KVR' from dual union all
select 33841,'KVR' from dual union all
select 33842,'KVR' from dual union all
select 33843,'KVR' from dual union all
select 33844,'KVR' from dual union all
select 33845,'KVR' from dual union all
select 33846,'KVR' from dual union all
select 33847,'KVR' from dual union all
select 33848,'KVR' from dual union all
select 33849,'IRIDS' from dual union all
select 3385,'USAF' from dual union all
select 33850,'IRIDS' from dual union all
select 33851,'IRIDS' from dual union all
select 33852,'IRIDS' from dual union all
select 33853,'IRIDS' from dual union all
select 33854,'IRIDS' from dual union all
select 33855,'IRIDS' from dual union all
select 33856,'IRIDS' from dual union all
select 33857,'IRIDS' from dual union all
select 33858,'IRIDS' from dual union all
select 33859,'IRIDS' from dual union all
select 3386,'USAF' from dual union all
select 33860,'IRIDS' from dual union all
select 33861,'IRIDS' from dual union all
select 33862,'IRIDS' from dual union all
select 33863,'IRIDS' from dual union all
select 33864,'IRIDS' from dual union all
select 33865,'IRIDS' from dual union all
select 33866,'IRIDS' from dual;
insert into SATELLITE_ORG
select 33867,'IRIDS' from dual union all
select 33868,'IRIDS' from dual union all
select 33869,'IRIDS' from dual union all
select 3387,'USAF' from dual union all
select 33870,'IRIDS' from dual union all
select 33871,'IRIDS' from dual union all
select 33872,'IRIDS' from dual union all
select 33873,'IRIDS' from dual union all
select 33874,'IRIDS' from dual union all
select 33875,'IRIDS' from dual union all
select 33876,'IRIDS' from dual union all
select 33877,'IRIDS' from dual union all
select 33878,'IRIDS' from dual union all
select 33879,'IRIDS' from dual union all
select 3388,'MO' from dual union all
select 33880,'IRIDS' from dual union all
select 33881,'IRIDS' from dual union all
select 33882,'IRIDS' from dual union all
select 33883,'IRIDS' from dual union all
select 33884,'IRIDS' from dual union all
select 33885,'IRIDS' from dual union all
select 33886,'IRIDS' from dual union all
select 33887,'IRIDS' from dual union all
select 33888,'IRIDS' from dual union all
select 33889,'KVR' from dual union all
select 3389,'RVSN' from dual union all
select 33890,'KVR' from dual union all
select 33891,'KVR' from dual union all
select 33892,'KVR' from dual union all
select 33893,'KVR' from dual union all
select 33894,'KVR' from dual union all
select 33895,'KVR' from dual union all
select 33896,'KVR' from dual union all
select 33897,'KVR' from dual union all
select 33898,'KVR' from dual union all
select 33899,'KVR' from dual union all
select 339,'RVSN' from dual union all
select 3390,'NASA' from dual union all
select 33900,'KVR' from dual union all
select 33901,'KVR' from dual union all
select 33902,'KVR' from dual union all
select 33903,'KVR' from dual union all
select 33904,'KVR' from dual union all
select 33905,'KVR' from dual union all
select 33906,'KVR' from dual union all
select 33907,'KVR' from dual union all
select 33908,'KVR' from dual union all
select 33909,'KVR' from dual union all
select 3391,'NASA' from dual union all
select 33910,'KVR' from dual union all
select 33911,'KVR' from dual union all
select 33912,'KVR' from dual union all
select 33913,'KVR' from dual union all
select 33914,'KVR' from dual union all
select 33915,'KVR' from dual union all
select 33916,'KVR' from dual union all
select 33917,'KVR' from dual union all
select 33918,'KVR' from dual union all
select 33919,'KVR' from dual union all
select 3392,'NASA' from dual union all
select 33920,'KVR' from dual union all
select 33921,'KVR' from dual union all
select 33922,'KVR' from dual union all
select 33923,'KVR' from dual union all
select 33924,'KVR' from dual union all
select 33925,'KVR' from dual union all
select 33926,'KVR' from dual union all
select 33927,'KVR' from dual union all
select 33928,'KVR' from dual union all
select 33929,'KVR' from dual union all
select 3393,'NASA' from dual union all
select 33930,'KVR' from dual union all
select 33931,'KVR' from dual union all
select 33932,'KVR' from dual union all
select 33933,'KVR' from dual union all
select 33934,'KVR' from dual union all
select 33935,'KVR' from dual union all
select 33936,'KVR' from dual union all
select 33937,'KVR' from dual union all
select 33938,'KVR' from dual union all
select 33939,'KVR' from dual union all
select 3394,'TSKBEM' from dual union all
select 33940,'KVR' from dual union all
select 33941,'KVR' from dual union all
select 33942,'KVR' from dual union all
select 33943,'KVR' from dual union all
select 33944,'KVR' from dual union all
select 33945,'KVR' from dual union all
select 33946,'KVR' from dual union all
select 33947,'KVR' from dual union all
select 33948,'KVR' from dual union all
select 33949,'IRIDS' from dual union all
select 3395,'RVSN' from dual union all
select 33950,'IRIDS' from dual union all
select 33951,'IRIDS' from dual union all
select 33952,'IRIDS' from dual union all
select 33953,'IRIDS' from dual union all
select 33954,'IRIDS' from dual union all
select 33955,'IRIDS' from dual union all
select 33956,'IRIDS' from dual;
insert into SATELLITE_ORG
select 33957,'IRIDS' from dual union all
select 33958,'IRIDS' from dual union all
select 33959,'IRIDS' from dual union all
select 3396,'RVSN' from dual union all
select 33960,'IRIDS' from dual union all
select 33961,'IRIDS' from dual union all
select 33962,'IRIDS' from dual union all
select 33963,'IRIDS' from dual union all
select 33964,'IRIDS' from dual union all
select 33965,'IRIDS' from dual union all
select 33966,'IRIDS' from dual union all
select 33967,'IRIDS' from dual union all
select 33968,'IRIDS' from dual union all
select 33969,'KVR' from dual union all
select 3397,'TSKBEM' from dual union all
select 33970,'KVR' from dual union all
select 33971,'KVR' from dual union all
select 33972,'KVR' from dual union all
select 33973,'KVR' from dual union all
select 33974,'KVR' from dual union all
select 33975,'KVR' from dual union all
select 33976,'KVR' from dual union all
select 33977,'KVR' from dual union all
select 33978,'KVR' from dual union all
select 33979,'KVR' from dual union all
select 3398,'MO' from dual union all
select 33980,'KVR' from dual union all
select 33981,'KVR' from dual union all
select 33982,'KVR' from dual union all
select 33983,'KVR' from dual union all
select 33984,'KVR' from dual union all
select 33985,'KVR' from dual union all
select 33986,'KVR' from dual union all
select 33987,'KVR' from dual union all
select 33988,'KVR' from dual union all
select 33989,'KVR' from dual union all
select 3399,'RVSN' from dual union all
select 33990,'KVR' from dual union all
select 33991,'KVR' from dual union all
select 33992,'KVR' from dual union all
select 33993,'KVR' from dual union all
select 33994,'KVR' from dual union all
select 33995,'KVR' from dual union all
select 33996,'KVR' from dual union all
select 33997,'KVR' from dual union all
select 33998,'KVR' from dual union all
select 33999,'KVR' from dual union all
select 34,'OKB1' from dual union all
select 340,'ATT' from dual union all
select 3400,'USAF' from dual union all
select 34000,'KVR' from dual union all
select 34001,'KVR' from dual union all
select 34002,'KVR' from dual union all
select 34003,'KVR' from dual union all
select 34004,'KVR' from dual union all
select 34005,'KVR' from dual union all
select 34006,'KVR' from dual union all
select 34007,'KVR' from dual union all
select 34008,'KVR' from dual union all
select 34009,'KVR' from dual union all
select 3401,'USAF' from dual union all
select 34010,'KVR' from dual union all
select 34011,'KVR' from dual union all
select 34012,'KVR' from dual union all
select 34013,'KVR' from dual union all
select 34014,'KVR' from dual union all
select 34015,'KVR' from dual union all
select 34016,'KVR' from dual union all
select 34017,'KVR' from dual union all
select 34018,'KVR' from dual union all
select 34019,'KVR' from dual union all
select 3402,'USAF' from dual union all
select 34020,'KVR' from dual union all
select 34021,'KVR' from dual union all
select 34022,'KVR' from dual union all
select 34023,'KVR' from dual union all
select 34024,'KVR' from dual union all
select 34025,'KVR' from dual union all
select 34026,'KVR' from dual union all
select 34027,'KVR' from dual union all
select 34028,'KVR' from dual union all
select 34029,'KVR' from dual union all
select 3403,'USAF' from dual union all
select 34030,'KVR' from dual union all
select 34031,'KVR' from dual union all
select 34032,'KVR' from dual union all
select 34033,'KVR' from dual union all
select 34034,'KVR' from dual union all
select 34035,'KVR' from dual union all
select 34036,'KVR' from dual union all
select 34037,'KVR' from dual union all
select 34038,'KVR' from dual union all
select 34039,'KVR' from dual union all
select 3404,'USAF' from dual union all
select 34040,'KVR' from dual union all
select 34041,'KVR' from dual union all
select 34042,'KVR' from dual union all
select 34043,'KVR' from dual union all
select 34044,'KVR' from dual union all
select 34045,'KVR' from dual;
insert into SATELLITE_ORG
select 34046,'KVR' from dual union all
select 34047,'KVR' from dual union all
select 34048,'KVR' from dual union all
select 34049,'KVR' from dual union all
select 3405,'USAF' from dual union all
select 34050,'KVR' from dual union all
select 34051,'KVR' from dual union all
select 34052,'KVR' from dual union all
select 34053,'KVR' from dual union all
select 34054,'KVR' from dual union all
select 34055,'KVR' from dual union all
select 34056,'KVR' from dual union all
select 34057,'KVR' from dual union all
select 34058,'KVR' from dual union all
select 34059,'KVR' from dual union all
select 3406,'USAF' from dual union all
select 34060,'KVR' from dual union all
select 34061,'KVR' from dual union all
select 34062,'KVR' from dual union all
select 34063,'KVR' from dual union all
select 34064,'KVR' from dual union all
select 34065,'KVR' from dual union all
select 34066,'KVR' from dual union all
select 34067,'KVR' from dual union all
select 34068,'KVR' from dual union all
select 34069,'KVR' from dual union all
select 3407,'USAF' from dual union all
select 34070,'KVR' from dual union all
select 34071,'IRIDS' from dual union all
select 34072,'IRIDS' from dual union all
select 34073,'IRIDS' from dual union all
select 34074,'IRIDS' from dual union all
select 34075,'IRIDS' from dual union all
select 34076,'IRIDS' from dual union all
select 34077,'IRIDS' from dual union all
select 34078,'IRIDS' from dual union all
select 34079,'IRIDS' from dual union all
select 3408,'CIA' from dual union all
select 3408,'NRO' from dual union all
select 34080,'IRIDS' from dual union all
select 34081,'IRIDS' from dual union all
select 34082,'IRIDS' from dual union all
select 34083,'IRIDS' from dual union all
select 34084,'IRIDS' from dual union all
select 34085,'IRIDS' from dual union all
select 34086,'IRIDS' from dual union all
select 34087,'IRIDS' from dual union all
select 34088,'IRIDS' from dual union all
select 34089,'IRIDS' from dual union all
select 3409,'NRO' from dual union all
select 34090,'IRIDS' from dual union all
select 34091,'IRIDS' from dual union all
select 34092,'IRIDS' from dual union all
select 34093,'IRIDS' from dual union all
select 34094,'IRIDS' from dual union all
select 34095,'IRIDS' from dual union all
select 34096,'IRIDS' from dual union all
select 34097,'IRIDS' from dual union all
select 34098,'IRIDS' from dual union all
select 34099,'IRIDS' from dual union all
select 341,'NASA' from dual union all
select 3410,'USAF' from dual union all
select 34100,'IRIDS' from dual union all
select 34101,'IRIDS' from dual union all
select 34102,'IRIDS' from dual union all
select 34103,'IRIDS' from dual union all
select 34104,'IRIDS' from dual union all
select 34105,'IRIDS' from dual union all
select 34106,'IRIDS' from dual union all
select 34107,'IRIDS' from dual union all
select 34108,'IRIDS' from dual union all
select 34109,'IRIDS' from dual union all
select 3411,'USAF' from dual union all
select 34110,'IRIDS' from dual union all
select 34111,'TCAN' from dual union all
select 34112,'SIS' from dual union all
select 34113,'KVR' from dual union all
select 34114,'KVR' from dual union all
select 34115,'KVR' from dual union all
select 34116,'KVR' from dual union all
select 34117,'KVR' from dual union all
select 34118,'KVR' from dual union all
select 34119,'KVR' from dual union all
select 3412,'USAF' from dual union all
select 34120,'KVR' from dual union all
select 34121,'KVR' from dual union all
select 34122,'KVR' from dual union all
select 34123,'KVR' from dual union all
select 34124,'KVR' from dual union all
select 34125,'KVR' from dual union all
select 34126,'KVR' from dual union all
select 34127,'KVR' from dual union all
select 34128,'KVR' from dual union all
select 34129,'KVR' from dual union all
select 3413,'USAF' from dual union all
select 34130,'KVR' from dual union all
select 34131,'KVR' from dual union all
select 34132,'KVR' from dual union all
select 34133,'KVR' from dual union all
select 34134,'KVR' from dual;
insert into SATELLITE_ORG
select 34135,'KVR' from dual union all
select 34136,'KVR' from dual union all
select 34137,'KVR' from dual union all
select 34138,'KVR' from dual union all
select 34139,'KVR' from dual union all
select 3414,'PVO' from dual union all
select 3414,'YUZH' from dual union all
select 34140,'KVR' from dual union all
select 34141,'KVR' from dual union all
select 34142,'KVR' from dual union all
select 34143,'IRIDS' from dual union all
select 34144,'IRIDS' from dual union all
select 34145,'IRIDS' from dual union all
select 34146,'IRIDS' from dual union all
select 34147,'IRIDS' from dual union all
select 34148,'IRIDS' from dual union all
select 34149,'IRIDS' from dual union all
select 3415,'RVSN' from dual union all
select 34150,'IRIDS' from dual union all
select 34151,'IRIDS' from dual union all
select 34152,'IRIDS' from dual union all
select 34153,'IRIDS' from dual union all
select 34154,'IRIDS' from dual union all
select 34155,'IRIDS' from dual union all
select 34156,'IRIDS' from dual union all
select 34157,'IRIDS' from dual union all
select 34158,'IRIDS' from dual union all
select 34159,'IRIDS' from dual union all
select 3416,'NASA' from dual union all
select 34160,'IRIDS' from dual union all
select 34161,'IRIDS' from dual union all
select 34162,'IRIDS' from dual union all
select 34163,'USAF' from dual union all
select 34164,'USAF' from dual union all
select 34165,'USAF' from dual union all
select 34166,'USAF' from dual union all
select 34167,'USAF' from dual union all
select 34168,'USAF' from dual union all
select 34169,'USAF' from dual union all
select 3417,'NASA' from dual union all
select 34170,'USAF' from dual union all
select 34171,'USAF' from dual union all
select 34172,'USAF' from dual union all
select 34173,'USAF' from dual union all
select 34174,'KVR' from dual union all
select 34175,'KVR' from dual union all
select 34176,'KVR' from dual union all
select 34177,'KVR' from dual union all
select 34178,'AE' from dual union all
select 34179,'AE' from dual union all
select 3418,'TSKBEM' from dual union all
select 34180,'AE' from dual union all
select 34181,'AE' from dual union all
select 34182,'RVSNR' from dual union all
select 34183,'RVSNR' from dual union all
select 34184,'AE' from dual union all
select 34185,'AE' from dual union all
select 34186,'AE' from dual union all
select 34187,'AE' from dual union all
select 34188,'AE' from dual union all
select 34189,'AE' from dual union all
select 3419,'RVSN' from dual union all
select 34190,'AE' from dual union all
select 34191,'AE' from dual union all
select 34192,'AE' from dual union all
select 34193,'AE' from dual union all
select 34194,'AE' from dual union all
select 34195,'AE' from dual union all
select 34196,'AE' from dual union all
select 34197,'AE' from dual union all
select 34198,'AE' from dual union all
select 34199,'AE' from dual union all
select 342,'USAF' from dual union all
select 3420,'USAF' from dual union all
select 34200,'KVR' from dual union all
select 34201,'KVR' from dual union all
select 34202,'KVR' from dual union all
select 34203,'NASA' from dual union all
select 34204,'CNSA' from dual union all
select 34205,'CNSA' from dual union all
select 34206,'CNSA' from dual union all
select 34207,'CNSA' from dual union all
select 34208,'CNSA' from dual union all
select 34209,'CNSA' from dual union all
select 3421,'USAF' from dual union all
select 34210,'CNSA' from dual union all
select 34211,'CNSA' from dual union all
select 34212,'CNSA' from dual union all
select 34213,'CNSA' from dual union all
select 34214,'CNSA' from dual union all
select 34215,'CNSA' from dual union all
select 34216,'CNSA' from dual union all
select 34217,'CNSA' from dual union all
select 34218,'CNSA' from dual union all
select 34219,'ISRO' from dual union all
select 3422,'USAF' from dual union all
select 34220,'ISRO' from dual union all
select 34221,'ISRO' from dual union all
select 34222,'ISRO' from dual union all
select 34223,'CNSA' from dual;
insert into SATELLITE_ORG
select 34224,'KHRR' from dual union all
select 34225,'KHRR' from dual union all
select 34226,'KHRR' from dual union all
select 34227,'KHRR' from dual union all
select 34228,'KHRR' from dual union all
select 34229,'KHRR' from dual union all
select 3423,'USAF' from dual union all
select 34230,'KHRR' from dual union all
select 34231,'KHRR' from dual union all
select 34232,'KHRR' from dual union all
select 34233,'KHRR' from dual union all
select 34234,'KHRR' from dual union all
select 34235,'KHRR' from dual union all
select 34236,'KHRR' from dual union all
select 34237,'KHRR' from dual union all
select 34238,'KHRR' from dual union all
select 34239,'KHRR' from dual union all
select 3424,'USAF' from dual union all
select 34240,'KHRR' from dual union all
select 34241,'KHRR' from dual union all
select 34242,'KHRR' from dual union all
select 34243,'KHRR' from dual union all
select 34244,'KHRR' from dual union all
select 34245,'KHRR' from dual union all
select 34246,'KHRR' from dual union all
select 34247,'KHRR' from dual union all
select 34248,'KHRR' from dual union all
select 34249,'KHRR' from dual union all
select 3425,'USAF' from dual union all
select 34250,'KHRR' from dual union all
select 34251,'KHRR' from dual union all
select 34252,'KHRR' from dual union all
select 34253,'KHRR' from dual union all
select 34254,'KHRR' from dual union all
select 34255,'KHRR' from dual union all
select 34256,'KHRR' from dual union all
select 34257,'KHRR' from dual union all
select 34258,'KHRR' from dual union all
select 34259,'KHRR' from dual union all
select 3426,'USAF' from dual union all
select 34260,'KHRR' from dual union all
select 34261,'KHRR' from dual union all
select 34262,'KHRR' from dual union all
select 34263,'KHRR' from dual union all
select 34264,'KVR' from dual union all
select 34265,'RVSNR' from dual union all
select 34266,'KHRR' from dual union all
select 34267,'KHRR' from dual union all
select 34268,'RVSNR' from dual union all
select 34269,'RVSNR' from dual union all
select 3427,'USAF' from dual union all
select 34270,'KVR' from dual union all
select 34271,'KVR' from dual union all
select 34272,'KVR' from dual union all
select 34273,'KVR' from dual union all
select 34274,'KVR' from dual union all
select 34275,'KVR' from dual union all
select 34276,'KVR' from dual union all
select 34277,'KVR' from dual union all
select 34278,'KVR' from dual union all
select 34279,'KVR' from dual union all
select 3428,'USAF' from dual union all
select 34280,'KVR' from dual union all
select 34281,'KVR' from dual union all
select 34282,'KVR' from dual union all
select 34283,'KVR' from dual union all
select 34284,'KVR' from dual union all
select 34285,'KVR' from dual union all
select 34286,'KVR' from dual union all
select 34287,'KVR' from dual union all
select 34288,'KVR' from dual union all
select 34289,'KVR' from dual union all
select 3429,'USAF' from dual union all
select 34290,'KVR' from dual union all
select 34291,'KVR' from dual union all
select 34292,'KVR' from dual union all
select 34293,'KVR' from dual union all
select 34294,'KVR' from dual union all
select 34295,'KVR' from dual union all
select 34296,'KVR' from dual union all
select 34297,'KVR' from dual union all
select 34298,'KVR' from dual union all
select 34299,'KVR' from dual union all
select 343,'USAF' from dual union all
select 3430,'USAF' from dual union all
select 34300,'KVR' from dual union all
select 34301,'KVR' from dual union all
select 34302,'KVR' from dual union all
select 34303,'KVR' from dual union all
select 34304,'KVR' from dual union all
select 34305,'KVR' from dual union all
select 34306,'KVR' from dual union all
select 34307,'KVR' from dual union all
select 34308,'KVR' from dual union all
select 34309,'KVR' from dual union all
select 3431,'USAF' from dual union all
select 34310,'KVR' from dual union all
select 34311,'KVR' from dual union all
select 34312,'KVR' from dual union all
select 34313,'KVR' from dual;
insert into SATELLITE_ORG
select 34314,'KVR' from dual union all
select 34315,'KVR' from dual union all
select 34316,'KVR' from dual union all
select 34317,'KVR' from dual union all
select 34318,'KVR' from dual union all
select 34319,'KVR' from dual union all
select 3432,'USAF' from dual union all
select 34320,'KVR' from dual union all
select 34321,'KVR' from dual union all
select 34322,'KVR' from dual union all
select 34323,'KVR' from dual union all
select 34324,'KVR' from dual union all
select 34325,'KVR' from dual union all
select 34326,'KVR' from dual union all
select 34327,'KVR' from dual union all
select 34328,'KVR' from dual union all
select 34329,'KVR' from dual union all
select 3433,'USAF' from dual union all
select 34330,'KVR' from dual union all
select 34331,'KVR' from dual union all
select 34332,'KVR' from dual union all
select 34333,'KVR' from dual union all
select 34334,'KVR' from dual union all
select 34335,'KVR' from dual union all
select 34336,'KVR' from dual union all
select 34337,'KVR' from dual union all
select 34338,'KVR' from dual union all
select 34339,'KVR' from dual union all
select 3434,'USAF' from dual union all
select 34340,'KVR' from dual union all
select 34341,'KVR' from dual union all
select 34342,'KVR' from dual union all
select 34343,'KVR' from dual union all
select 34344,'KVR' from dual union all
select 34345,'KVR' from dual union all
select 34346,'KVR' from dual union all
select 34347,'KVR' from dual union all
select 34348,'KVR' from dual union all
select 34349,'KVR' from dual union all
select 3435,'USAF' from dual union all
select 34350,'IRIDS' from dual union all
select 34351,'IRIDS' from dual union all
select 34352,'IRIDS' from dual union all
select 34353,'IRIDS' from dual union all
select 34354,'IRIDS' from dual union all
select 34355,'IRIDS' from dual union all
select 34356,'IRIDS' from dual union all
select 34357,'IRIDS' from dual union all
select 34358,'IRIDS' from dual union all
select 34359,'IRIDS' from dual union all
select 3436,'USAF' from dual union all
select 34360,'IRIDS' from dual union all
select 34361,'IRIDS' from dual union all
select 34362,'IRIDS' from dual union all
select 34363,'IRIDS' from dual union all
select 34364,'IRIDS' from dual union all
select 34365,'IRIDS' from dual union all
select 34366,'IRIDS' from dual union all
select 34367,'IRIDS' from dual union all
select 34368,'IRIDS' from dual union all
select 34369,'IRIDS' from dual union all
select 3437,'USAF' from dual union all
select 34370,'IRIDS' from dual union all
select 34371,'IRIDS' from dual union all
select 34372,'IRIDS' from dual union all
select 34373,'IRIDS' from dual union all
select 34374,'IRIDS' from dual union all
select 34375,'IRIDS' from dual union all
select 34376,'IRIDS' from dual union all
select 34377,'IRIDS' from dual union all
select 34378,'IRIDS' from dual union all
select 34379,'IRIDS' from dual union all
select 3438,'USAF' from dual union all
select 34380,'NASA' from dual union all
select 34381,'BOE' from dual union all
select 34382,'BOE' from dual union all
select 34383,'KVR' from dual union all
select 34384,'KVR' from dual union all
select 34385,'KVR' from dual union all
select 34386,'KVR' from dual union all
select 34387,'KVR' from dual union all
select 34388,'KVR' from dual union all
select 34389,'KVR' from dual union all
select 3439,'USAF' from dual union all
select 34390,'KVR' from dual union all
select 34391,'KVR' from dual union all
select 34392,'KVR' from dual union all
select 34393,'KVR' from dual union all
select 34394,'KVR' from dual union all
select 34395,'KVR' from dual union all
select 34396,'KVR' from dual union all
select 34397,'KVR' from dual union all
select 34398,'KVR' from dual union all
select 34399,'KVR' from dual union all
select 344,'CIA' from dual union all
select 344,'NRO' from dual union all
select 3440,'USAF' from dual union all
select 34400,'KVR' from dual union all
select 34401,'KVR' from dual union all
select 34402,'KVR' from dual;
insert into SATELLITE_ORG
select 34403,'KVR' from dual union all
select 34404,'KVR' from dual union all
select 34405,'KVR' from dual union all
select 34406,'KVR' from dual union all
select 34407,'KVR' from dual union all
select 34408,'KVR' from dual union all
select 34409,'KVR' from dual union all
select 3441,'USAF' from dual union all
select 34410,'KVR' from dual union all
select 34411,'KVR' from dual union all
select 34412,'KVR' from dual union all
select 34413,'KVR' from dual union all
select 34414,'KVR' from dual union all
select 34415,'KVR' from dual union all
select 34416,'KVR' from dual union all
select 34417,'KVR' from dual union all
select 34418,'KVR' from dual union all
select 34419,'KVR' from dual union all
select 3442,'USAF' from dual union all
select 34420,'KVR' from dual union all
select 34421,'KVR' from dual union all
select 34422,'KVR' from dual union all
select 34423,'KVR' from dual union all
select 34424,'KVR' from dual union all
select 34425,'KVR' from dual union all
select 34426,'KVR' from dual union all
select 34427,'KVR' from dual union all
select 34428,'KVR' from dual union all
select 34429,'KVR' from dual union all
select 3443,'USAF' from dual union all
select 34430,'KVR' from dual union all
select 34431,'KVR' from dual union all
select 34432,'KVR' from dual union all
select 34433,'KVR' from dual union all
select 34434,'KVR' from dual union all
select 34435,'KVR' from dual union all
select 34436,'KVR' from dual union all
select 34437,'KVR' from dual union all
select 34438,'KVR' from dual union all
select 34439,'KVR' from dual union all
select 3444,'USAF' from dual union all
select 34440,'KVR' from dual union all
select 34441,'KVR' from dual union all
select 34442,'KVR' from dual union all
select 34443,'KVR' from dual union all
select 34444,'KVR' from dual union all
select 34445,'KVR' from dual union all
select 34446,'KVR' from dual union all
select 34447,'KVR' from dual union all
select 34448,'KVR' from dual union all
select 34449,'KVR' from dual union all
select 3445,'USAF' from dual union all
select 34450,'KVR' from dual union all
select 34451,'KVR' from dual union all
select 34452,'KVR' from dual union all
select 34453,'KVR' from dual union all
select 34454,'KVR' from dual union all
select 34455,'KVR' from dual union all
select 34456,'KVR' from dual union all
select 34457,'KVR' from dual union all
select 34458,'KVR' from dual union all
select 34459,'KVR' from dual union all
select 3446,'USAF' from dual union all
select 34460,'KVR' from dual union all
select 34461,'KVR' from dual union all
select 34462,'KVR' from dual union all
select 34463,'KVR' from dual union all
select 34464,'KVR' from dual union all
select 34465,'KVR' from dual union all
select 34466,'KVR' from dual union all
select 34467,'KVR' from dual union all
select 34468,'KVR' from dual union all
select 34469,'KVR' from dual union all
select 3447,'USAF' from dual union all
select 34470,'KVR' from dual union all
select 34471,'KVR' from dual union all
select 34472,'KVR' from dual union all
select 34473,'KVR' from dual union all
select 34474,'KVR' from dual union all
select 34475,'KVR' from dual union all
select 34476,'KVR' from dual union all
select 34477,'KVR' from dual union all
select 34478,'KVR' from dual union all
select 34479,'KVR' from dual union all
select 3448,'USAF' from dual union all
select 34480,'KVR' from dual union all
select 34481,'KVR' from dual union all
select 34482,'KVR' from dual union all
select 34483,'IRIDS' from dual union all
select 34484,'IRIDS' from dual union all
select 34485,'IRIDS' from dual union all
select 34486,'IRIDS' from dual union all
select 34487,'IRIDS' from dual union all
select 34488,'IRIDS' from dual union all
select 34489,'IRIDS' from dual union all
select 3449,'RVSN' from dual union all
select 34490,'IRIDS' from dual union all
select 34491,'IRIDS' from dual union all
select 34492,'IRIDS' from dual union all
select 34493,'IRIDS' from dual;
insert into SATELLITE_ORG
select 34494,'IRIDS' from dual union all
select 34495,'IRIDS' from dual union all
select 34496,'IRIDS' from dual union all
select 34497,'IRIDS' from dual union all
select 34498,'IRIDS' from dual union all
select 34499,'IRIDS' from dual union all
select 345,'CIA' from dual union all
select 345,'NRO' from dual union all
select 3450,'RVSN' from dual union all
select 34500,'IRIDS' from dual union all
select 34501,'IRIDS' from dual union all
select 34502,'IRIDS' from dual union all
select 34503,'IRIDS' from dual union all
select 34504,'IRIDS' from dual union all
select 34505,'IRIDS' from dual union all
select 34506,'IRIDS' from dual union all
select 34507,'IRIDS' from dual union all
select 34508,'IRIDS' from dual union all
select 34509,'IRIDS' from dual union all
select 3451,'RVSN' from dual union all
select 34510,'IRIDS' from dual union all
select 34511,'IRIDS' from dual union all
select 34512,'IRIDS' from dual union all
select 34513,'IRIDS' from dual union all
select 34514,'IRIDS' from dual union all
select 34515,'IRIDS' from dual union all
select 34516,'IRIDS' from dual union all
select 34517,'IRIDS' from dual union all
select 34518,'IRIDS' from dual union all
select 34519,'IRIDS' from dual union all
select 3452,'TSKBEM' from dual union all
select 34520,'IRIDS' from dual union all
select 34521,'IRIDS' from dual union all
select 34522,'IRIDS' from dual union all
select 34523,'IRIDS' from dual union all
select 34524,'IRIDS' from dual union all
select 34525,'IRIDS' from dual union all
select 34526,'IRIDS' from dual union all
select 34527,'IRIDS' from dual union all
select 34528,'IRIDS' from dual union all
select 34529,'IRIDS' from dual union all
select 3453,'USAF' from dual union all
select 34530,'IRIDS' from dual union all
select 34531,'IRIDS' from dual union all
select 34532,'IRIDS' from dual union all
select 34533,'IRIDS' from dual union all
select 34534,'IRIDS' from dual union all
select 34535,'IRIDS' from dual union all
select 34536,'IRIDS' from dual union all
select 34537,'IRIDS' from dual union all
select 34538,'IRIDS' from dual union all
select 34539,'IRIDS' from dual union all
select 3454,'USAF' from dual union all
select 34540,'IRIDS' from dual union all
select 34541,'NASA' from dual union all
select 34542,'KVR' from dual union all
select 34543,'KVR' from dual union all
select 34544,'KVR' from dual union all
select 34545,'KVR' from dual union all
select 34546,'KVR' from dual union all
select 34547,'KVR' from dual union all
select 34548,'KVR' from dual union all
select 34549,'KVR' from dual union all
select 3455,'USAF' from dual union all
select 34550,'KVR' from dual union all
select 34551,'KVR' from dual union all
select 34552,'KVR' from dual union all
select 34553,'KVR' from dual union all
select 34554,'KVR' from dual union all
select 34555,'KVR' from dual union all
select 34556,'KVR' from dual union all
select 34557,'KVR' from dual union all
select 34558,'KVR' from dual union all
select 34559,'KVR' from dual union all
select 3456,'USAF' from dual union all
select 34560,'KVR' from dual union all
select 34561,'KVR' from dual union all
select 34562,'KVR' from dual union all
select 34563,'KVR' from dual union all
select 34564,'KVR' from dual union all
select 34565,'KVR' from dual union all
select 34566,'KVR' from dual union all
select 34567,'KVR' from dual union all
select 34568,'KVR' from dual union all
select 34569,'KVR' from dual union all
select 3457,'PVO' from dual union all
select 34570,'KVR' from dual union all
select 34571,'KVR' from dual union all
select 34572,'KVR' from dual union all
select 34573,'KVR' from dual union all
select 34574,'KVR' from dual union all
select 34575,'KVR' from dual union all
select 34576,'KVR' from dual union all
select 34577,'KVR' from dual union all
select 34578,'KVR' from dual union all
select 34579,'KVR' from dual union all
select 3458,'RVSN' from dual union all
select 34580,'KVR' from dual union all
select 34581,'KVR' from dual union all
select 34582,'KVR' from dual;
insert into SATELLITE_ORG
select 34583,'KVR' from dual union all
select 34584,'KVR' from dual union all
select 34585,'KVR' from dual union all
select 34586,'KVR' from dual union all
select 34587,'KVR' from dual union all
select 34588,'KVR' from dual union all
select 34589,'KVR' from dual union all
select 3459,'ESRO' from dual union all
select 34590,'KVR' from dual union all
select 34591,'KVR' from dual union all
select 34592,'IRIDS' from dual union all
select 34593,'IRIDS' from dual union all
select 34594,'IRIDS' from dual union all
select 34595,'IRIDS' from dual union all
select 34596,'IRIDS' from dual union all
select 34597,'IRIDS' from dual union all
select 34598,'IRIDS' from dual union all
select 34599,'IRIDS' from dual union all
select 346,'OKB1' from dual union all
select 3460,'NASA' from dual union all
select 34600,'IRIDS' from dual union all
select 34601,'IRIDS' from dual union all
select 34602,'ESA' from dual union all
select 34603,'EUROK' from dual union all
select 34604,'JAXA' from dual union all
select 34605,'NASA' from dual union all
select 34606,'NASA' from dual union all
select 34607,'NASA' from dual union all
select 34608,'NASA' from dual union all
select 34609,'KVR' from dual union all
select 3461,'USAF' from dual union all
select 34610,'KVR' from dual union all
select 34611,'KVR' from dual union all
select 34612,'KVR' from dual union all
select 34613,'KVR' from dual union all
select 34614,'KVR' from dual union all
select 34615,'KVR' from dual union all
select 34616,'KVR' from dual union all
select 34617,'KVR' from dual union all
select 34618,'KVR' from dual union all
select 34619,'KVR' from dual union all
select 3462,'USAF' from dual union all
select 34620,'KVR' from dual union all
select 34621,'KVR' from dual union all
select 34622,'KVR' from dual union all
select 34623,'KVR' from dual union all
select 34624,'KVR' from dual union all
select 34625,'KVR' from dual union all
select 34626,'KVR' from dual union all
select 34627,'KVR' from dual union all
select 34628,'KVR' from dual union all
select 34629,'KVR' from dual union all
select 3463,'USAF' from dual union all
select 34630,'KVR' from dual union all
select 34631,'KVR' from dual union all
select 34632,'KVR' from dual union all
select 34633,'KVR' from dual union all
select 34634,'KVR' from dual union all
select 34635,'KVR' from dual union all
select 34636,'KVR' from dual union all
select 34637,'KVR' from dual union all
select 34638,'KVR' from dual union all
select 34639,'KVR' from dual union all
select 3464,'USAF' from dual union all
select 34640,'KVR' from dual union all
select 34641,'KVR' from dual union all
select 34642,'KVR' from dual union all
select 34643,'IRIDS' from dual union all
select 34644,'IRIDS' from dual union all
select 34645,'IRIDS' from dual union all
select 34646,'IRIDS' from dual union all
select 34647,'IRIDS' from dual union all
select 34648,'IRIDS' from dual union all
select 34649,'IRIDS' from dual union all
select 3465,'USAF' from dual union all
select 34650,'IRIDS' from dual union all
select 34651,'IRIDS' from dual union all
select 34652,'IRIDS' from dual union all
select 34653,'IRIDS' from dual union all
select 34654,'IRIDS' from dual union all
select 34655,'IRIDS' from dual union all
select 34656,'IRIDS' from dual union all
select 34657,'IRIDS' from dual union all
select 34658,'IRIDS' from dual union all
select 34659,'IRIDS' from dual union all
select 3466,'USAF' from dual union all
select 34660,'RVSNR' from dual union all
select 34661,'USAF' from dual union all
select 34662,'BOE' from dual union all
select 34663,'BOE' from dual union all
select 34664,'KVR' from dual union all
select 34665,'RVSNR' from dual union all
select 34666,'RVSNR' from dual union all
select 34667,'RVSNR' from dual union all
select 34668,'RVSNR' from dual union all
select 34669,'FKA' from dual union all
select 3467,'USAF' from dual union all
select 34670,'KVR' from dual union all
select 34671,'KVR' from dual union all
select 34672,'KVR' from dual;
insert into SATELLITE_ORG
select 34673,'KVR' from dual union all
select 34674,'KVR' from dual union all
select 34675,'KVR' from dual union all
select 34676,'KVR' from dual union all
select 34677,'KVR' from dual union all
select 34678,'KVR' from dual union all
select 34679,'KVR' from dual union all
select 3468,'USAF' from dual union all
select 34680,'KVR' from dual union all
select 34681,'KVR' from dual union all
select 34682,'KVR' from dual union all
select 34683,'KVR' from dual union all
select 34684,'KVR' from dual union all
select 34685,'KVR' from dual union all
select 34686,'KVR' from dual union all
select 34687,'KVR' from dual union all
select 34688,'KVR' from dual union all
select 34689,'KVR' from dual union all
select 3469,'MO' from dual union all
select 34690,'IRIDS' from dual union all
select 34691,'IRIDS' from dual union all
select 34692,'IRIDS' from dual union all
select 34693,'IRIDS' from dual union all
select 34694,'IRIDS' from dual union all
select 34695,'IRIDS' from dual union all
select 34696,'IRIDS' from dual union all
select 34697,'IRIDS' from dual union all
select 34698,'IRIDS' from dual union all
select 34699,'IRIDS' from dual union all
select 347,'RVSN' from dual union all
select 3470,'RVSN' from dual union all
select 34700,'IRIDS' from dual union all
select 34701,'IRIDS' from dual union all
select 34702,'IRIDS' from dual union all
select 34703,'IRIDS' from dual union all
select 34704,'IRIDS' from dual union all
select 34705,'IRIDS' from dual union all
select 34706,'IRIDS' from dual union all
select 34707,'IRIDS' from dual union all
select 34708,'IRIDS' from dual union all
select 34709,'IRIDS' from dual union all
select 3471,'RVSN' from dual union all
select 34710,'EUTSA' from dual union all
select 34711,'KHRR' from dual union all
select 34712,'KHRR' from dual union all
select 34713,'USAF' from dual union all
select 34714,'LM' from dual union all
select 34715,'KVR' from dual union all
select 34716,'KVR' from dual union all
select 34717,'KVR' from dual union all
select 34718,'KVR' from dual union all
select 34719,'KVR' from dual union all
select 3472,'NRO' from dual union all
select 34720,'KVR' from dual union all
select 34721,'KVR' from dual union all
select 34722,'KVR' from dual union all
select 34723,'KVR' from dual union all
select 34724,'KVR' from dual union all
select 34725,'KVR' from dual union all
select 34726,'KVR' from dual union all
select 34727,'KVR' from dual union all
select 34728,'KVR' from dual union all
select 34729,'KVR' from dual union all
select 3473,'MO' from dual union all
select 34730,'KVR' from dual union all
select 34731,'KVR' from dual union all
select 34732,'KVR' from dual union all
select 34733,'KVR' from dual union all
select 34734,'KVR' from dual union all
select 34735,'KVR' from dual union all
select 34736,'KVR' from dual union all
select 34737,'KVR' from dual union all
select 34738,'KVR' from dual union all
select 34739,'KVR' from dual union all
select 3474,'RVSN' from dual union all
select 34740,'KVR' from dual union all
select 34741,'KVR' from dual union all
select 34742,'KVR' from dual union all
select 34743,'KVR' from dual union all
select 34744,'KVR' from dual union all
select 34745,'KVR' from dual union all
select 34746,'KVR' from dual union all
select 34747,'KVR' from dual union all
select 34748,'KVR' from dual union all
select 34749,'KVR' from dual union all
select 3475,'USAF' from dual union all
select 34750,'KVR' from dual union all
select 34751,'KVR' from dual union all
select 34752,'KVR' from dual union all
select 34753,'KVR' from dual union all
select 34754,'KVR' from dual union all
select 34755,'KVR' from dual union all
select 34756,'KVR' from dual union all
select 34757,'KVR' from dual union all
select 34758,'KVR' from dual union all
select 34759,'KVR' from dual union all
select 3476,'USAF' from dual union all
select 34760,'KVR' from dual union all
select 34761,'KVR' from dual union all
select 34762,'KVR' from dual;
insert into SATELLITE_ORG
select 34763,'KVR' from dual union all
select 34764,'IRIDS' from dual union all
select 34765,'IRIDS' from dual union all
select 34766,'IRIDS' from dual union all
select 34767,'IRIDS' from dual union all
select 34768,'IRIDS' from dual union all
select 34769,'IRIDS' from dual union all
select 3477,'USAF' from dual union all
select 34770,'IRIDS' from dual union all
select 34771,'IRIDS' from dual union all
select 34772,'IRIDS' from dual union all
select 34773,'IRIDS' from dual union all
select 34774,'IRIDS' from dual union all
select 34775,'IRIDS' from dual union all
select 34776,'IRIDS' from dual union all
select 34777,'IRIDS' from dual union all
select 34778,'IRIDS' from dual union all
select 34779,'CNSA' from dual union all
select 3478,'USAF' from dual union all
select 34780,'CASC' from dual union all
select 34781,'KVR' from dual union all
select 34782,'KVR' from dual union all
select 34783,'KVR' from dual union all
select 34784,'KVR' from dual union all
select 34785,'KVR' from dual union all
select 34786,'KVR' from dual union all
select 34787,'KVR' from dual union all
select 34788,'KVR' from dual union all
select 34789,'KVR' from dual union all
select 3479,'ESRO' from dual union all
select 34790,'KVR' from dual union all
select 34791,'KVR' from dual union all
select 34792,'KVR' from dual union all
select 34793,'KVR' from dual union all
select 34794,'KVR' from dual union all
select 34795,'KVR' from dual union all
select 34796,'KVR' from dual union all
select 34797,'KVR' from dual union all
select 34798,'RVSNR' from dual union all
select 34799,'RVSNR' from dual union all
select 348,'OKB1' from dual union all
select 3480,'USAF' from dual union all
select 34800,'RVSNR' from dual union all
select 34801,'RVSNR' from dual union all
select 34802,'RVSNR' from dual union all
select 34803,'RVSNR' from dual union all
select 34804,'RVSNR' from dual union all
select 34805,'RVSNR' from dual union all
select 34806,'RVSNR' from dual union all
select 34807,'ISRO' from dual union all
select 34808,'ISRO' from dual union all
select 34809,'ISRO' from dual union all
select 3481,'USAF' from dual union all
select 34810,'MDDI' from dual union all
select 34811,'SEALP' from dual union all
select 34812,'KVR' from dual union all
select 34813,'KVR' from dual union all
select 34814,'KVR' from dual union all
select 34815,'KVR' from dual union all
select 34816,'KVR' from dual union all
select 34817,'KVR' from dual union all
select 34818,'KVR' from dual union all
select 34819,'KVR' from dual union all
select 3482,'USAF' from dual union all
select 34820,'KVR' from dual union all
select 34821,'KVR' from dual union all
select 34822,'KVR' from dual union all
select 34823,'KVR' from dual union all
select 34824,'IRIDS' from dual union all
select 34825,'IRIDS' from dual union all
select 34826,'IRIDS' from dual union all
select 34827,'IRIDS' from dual union all
select 34828,'IRIDS' from dual union all
select 34829,'IRIDS' from dual union all
select 3483,'USAF' from dual union all
select 34830,'IRIDS' from dual union all
select 34831,'IRIDS' from dual union all
select 34832,'IRIDS' from dual union all
select 34833,'IRIDS' from dual union all
select 34834,'IRIDS' from dual union all
select 34835,'IRIDS' from dual union all
select 34836,'SEALP' from dual union all
select 34837,'SEALP' from dual union all
select 34838,'SEALP' from dual union all
select 34839,'PLA' from dual union all
select 3484,'MO' from dual union all
select 34840,'CNSA' from dual union all
select 34841,'CNSA' from dual union all
select 34842,'CNSA' from dual union all
select 34843,'CNSA' from dual union all
select 34844,'CNSA' from dual union all
select 34845,'KVR' from dual union all
select 34846,'KVR' from dual union all
select 34847,'KVR' from dual union all
select 34848,'KVR' from dual union all
select 34849,'KVR' from dual union all
select 3485,'RVSN' from dual union all
select 34850,'KVR' from dual union all
select 34851,'KVR' from dual union all
select 34852,'KVR' from dual;
insert into SATELLITE_ORG
select 34853,'KVR' from dual union all
select 34854,'KVR' from dual union all
select 34855,'KVR' from dual union all
select 34856,'KVR' from dual union all
select 34857,'KVR' from dual union all
select 34858,'KVR' from dual union all
select 34859,'KVR' from dual union all
select 3486,'NASA' from dual union all
select 34860,'KVR' from dual union all
select 34861,'KVR' from dual union all
select 34862,'KVR' from dual union all
select 34863,'KVR' from dual union all
select 34864,'KVR' from dual union all
select 34865,'KVR' from dual union all
select 34866,'IRIDS' from dual union all
select 34867,'IRIDS' from dual union all
select 34868,'IRIDS' from dual union all
select 34869,'IRIDS' from dual union all
select 3487,'NASA' from dual union all
select 34870,'IRIDS' from dual union all
select 34871,'KVR' from dual union all
select 34872,'KVR' from dual union all
select 34873,'KVR' from dual union all
select 34874,'KVR' from dual union all
select 34875,'KVR' from dual union all
select 34876,'KVR' from dual union all
select 34877,'KVR' from dual union all
select 34878,'KVR' from dual union all
select 34879,'KVR' from dual union all
select 3488,'USAF' from dual union all
select 34880,'KVR' from dual union all
select 34881,'KVR' from dual union all
select 34882,'KVR' from dual union all
select 34883,'KVR' from dual union all
select 34884,'KVR' from dual union all
select 34885,'KVR' from dual union all
select 34886,'KVR' from dual union all
select 34887,'KVR' from dual union all
select 34888,'KVR' from dual union all
select 34889,'IRIDS' from dual union all
select 3489,'USAF' from dual union all
select 34890,'IRIDS' from dual union all
select 34891,'IRIDS' from dual union all
select 34892,'IRIDS' from dual union all
select 34893,'IRIDS' from dual union all
select 34894,'IRIDS' from dual union all
select 34895,'IRIDS' from dual union all
select 34896,'IRIDS' from dual union all
select 34897,'IRIDS' from dual union all
select 34898,'IRIDS' from dual union all
select 34899,'IRIDS' from dual union all
select 349,'OKB1' from dual union all
select 3490,'USAF' from dual union all
select 34900,'IRIDS' from dual union all
select 34901,'IRIDS' from dual union all
select 34902,'IRIDS' from dual union all
select 34903,'MDA' from dual union all
select 34904,'BOE' from dual union all
select 34905,'FKA' from dual union all
select 34906,'KVR' from dual union all
select 34907,'KVR' from dual union all
select 34908,'KVR' from dual union all
select 34909,'KVR' from dual union all
select 3491,'USAF' from dual union all
select 34910,'KVR' from dual union all
select 34911,'KVR' from dual union all
select 34912,'KVR' from dual union all
select 34913,'KVR' from dual union all
select 34914,'KVR' from dual union all
select 34915,'KVR' from dual union all
select 34916,'KVR' from dual union all
select 34917,'KVR' from dual union all
select 34918,'KVR' from dual union all
select 34919,'KVR' from dual union all
select 3492,'USAF' from dual union all
select 34920,'KVR' from dual union all
select 34921,'KVR' from dual union all
select 34922,'KVR' from dual union all
select 34923,'KVR' from dual union all
select 34924,'KVR' from dual union all
select 34925,'KVR' from dual union all
select 34926,'IRIDS' from dual union all
select 34927,'IRIDS' from dual union all
select 34928,'IRIDS' from dual union all
select 34929,'IRIDS' from dual union all
select 3493,'USAF' from dual union all
select 34930,'IRIDS' from dual union all
select 34931,'IRIDS' from dual union all
select 34932,'IRIDS' from dual union all
select 34933,'NASA' from dual union all
select 34934,'RVSNR' from dual union all
select 34935,'RVSNR' from dual union all
select 34936,'RVSNR' from dual union all
select 34937,'ESA' from dual union all
select 34938,'ESA' from dual union all
select 34939,'AE' from dual union all
select 3494,'USAF' from dual union all
select 34940,'AE' from dual union all
select 34941,'PROTO' from dual union all
select 34942,'KHRR' from dual;
insert into SATELLITE_ORG
select 34943,'KHRR' from dual union all
select 34944,'KVR' from dual union all
select 34945,'KVR' from dual union all
select 34946,'KVR' from dual union all
select 34947,'KVR' from dual union all
select 34948,'KVR' from dual union all
select 34949,'KVR' from dual union all
select 3495,'USAF' from dual union all
select 34950,'KVR' from dual union all
select 34951,'KVR' from dual union all
select 34952,'KVR' from dual union all
select 34953,'KVR' from dual union all
select 34954,'KVR' from dual union all
select 34955,'KVR' from dual union all
select 34956,'KVR' from dual union all
select 34957,'KVR' from dual union all
select 34958,'KVR' from dual union all
select 34959,'KVR' from dual union all
select 3496,'USAF' from dual union all
select 34960,'KVR' from dual union all
select 34961,'KVR' from dual union all
select 34962,'KVR' from dual union all
select 34963,'KVR' from dual union all
select 34964,'KVR' from dual union all
select 34965,'KVR' from dual union all
select 34966,'KVR' from dual union all
select 34967,'KVR' from dual union all
select 34968,'KVR' from dual union all
select 34969,'KVR' from dual union all
select 3497,'USAF' from dual union all
select 34970,'KVR' from dual union all
select 34971,'KVR' from dual union all
select 34972,'KVR' from dual union all
select 34973,'KVR' from dual union all
select 34974,'KVR' from dual union all
select 34975,'KVR' from dual union all
select 34976,'KVR' from dual union all
select 34977,'KVR' from dual union all
select 34978,'KVR' from dual union all
select 34979,'KVR' from dual union all
select 3498,'USAF' from dual union all
select 34980,'KVR' from dual union all
select 34981,'KVR' from dual union all
select 34982,'IRIDS' from dual union all
select 34983,'IRIDS' from dual union all
select 34984,'IRIDS' from dual union all
select 34985,'IRIDS' from dual union all
select 34986,'IRIDS' from dual union all
select 34987,'IRIDS' from dual union all
select 34988,'USAF' from dual union all
select 34989,'KVR' from dual union all
select 3499,'USAF' from dual union all
select 34990,'KVR' from dual union all
select 34991,'KVR' from dual union all
select 34992,'KVR' from dual union all
select 34993,'KVR' from dual union all
select 34994,'KVR' from dual union all
select 34995,'KVR' from dual union all
select 34996,'KVR' from dual union all
select 34997,'KVR' from dual union all
select 34998,'KVR' from dual union all
select 34999,'KVR' from dual union all
select 35,'RVSN' from dual union all
select 350,'USAF' from dual union all
select 3500,'USAF' from dual union all
select 35000,'KVR' from dual union all
select 35001,'USAF' from dual union all
select 35002,'NASA' from dual union all
select 35003,'CALP' from dual union all
select 35004,'HAWK' from dual union all
select 35005,'AERO' from dual union all
select 35006,'OSC' from dual union all
select 35007,'KVR' from dual union all
select 35008,'FKA' from dual union all
select 35009,'NPOL' from dual union all
select 3501,'USAF' from dual union all
select 35010,'FKA' from dual union all
select 35011,'KVR' from dual union all
select 35012,'KVR' from dual union all
select 35013,'KVR' from dual union all
select 35014,'KVR' from dual union all
select 35015,'KVR' from dual union all
select 35016,'KVR' from dual union all
select 35017,'KVR' from dual union all
select 35018,'KVR' from dual union all
select 35019,'KVR' from dual union all
select 3502,'USAF' from dual union all
select 35020,'KVR' from dual union all
select 35021,'KVR' from dual union all
select 35022,'KVR' from dual union all
select 35023,'KVR' from dual union all
select 35024,'KVR' from dual union all
select 35025,'KVR' from dual union all
select 35026,'KVR' from dual union all
select 35027,'KVR' from dual union all
select 35028,'KVR' from dual union all
select 35029,'KVR' from dual union all
select 3503,'PVO' from dual union all
select 35030,'KVR' from dual union all
select 35031,'KVR' from dual;
insert into SATELLITE_ORG
select 35032,'KVR' from dual union all
select 35033,'KVR' from dual union all
select 35034,'KVR' from dual union all
select 35035,'KVR' from dual union all
select 35036,'KVR' from dual union all
select 35037,'KVR' from dual union all
select 35038,'KVR' from dual union all
select 35039,'KVR' from dual union all
select 3504,'PVO' from dual union all
select 35040,'KVR' from dual union all
select 35041,'KVR' from dual union all
select 35042,'KVR' from dual union all
select 35043,'KVR' from dual union all
select 35044,'KVR' from dual union all
select 35045,'KVR' from dual union all
select 35046,'KVR' from dual union all
select 35047,'KVR' from dual union all
select 35048,'IRIDS' from dual union all
select 35049,'IRIDS' from dual union all
select 3505,'RVSN' from dual union all
select 35050,'IRIDS' from dual union all
select 35051,'IRIDS' from dual union all
select 35052,'IRIDS' from dual union all
select 35053,'IRIDS' from dual union all
select 35054,'IRIDS' from dual union all
select 35055,'IRIDS' from dual union all
select 35056,'RVSNR' from dual union all
select 35057,'RVSNR' from dual union all
select 35058,'KVR' from dual union all
select 35059,'KVR' from dual union all
select 3506,'PVO' from dual union all
select 35060,'KVR' from dual union all
select 35061,'KVR' from dual union all
select 35062,'KVR' from dual union all
select 35063,'KVR' from dual union all
select 35064,'KVR' from dual union all
select 35065,'KVR' from dual union all
select 35066,'KVR' from dual union all
select 35067,'KVR' from dual union all
select 35068,'KVR' from dual union all
select 35069,'KVR' from dual union all
select 3507,'PVO' from dual union all
select 35070,'KVR' from dual union all
select 35071,'KVR' from dual union all
select 35072,'KVR' from dual union all
select 35073,'KVR' from dual union all
select 35074,'KVR' from dual union all
select 35075,'KVR' from dual union all
select 35076,'KVR' from dual union all
select 35077,'IRIDS' from dual union all
select 35078,'IRIDS' from dual union all
select 35079,'IRIDS' from dual union all
select 3508,'PVO' from dual union all
select 35080,'IRIDS' from dual union all
select 35081,'IRIDS' from dual union all
select 35082,'RVSNR' from dual union all
select 35083,'RVSNR' from dual union all
select 35084,'RVSNR' from dual union all
select 35085,'RVSNR' from dual union all
select 35086,'CASC' from dual union all
select 35087,'CASC' from dual union all
select 35088,'CASC' from dual union all
select 35089,'CASC' from dual union all
select 3509,'PVO' from dual union all
select 35090,'CASC' from dual union all
select 35091,'CASC' from dual union all
select 35092,'CASC' from dual union all
select 35093,'CASC' from dual union all
select 35094,'CASC' from dual union all
select 35095,'CASC' from dual union all
select 35096,'CASC' from dual union all
select 35097,'CASC' from dual union all
select 35098,'CASC' from dual union all
select 35099,'CASC' from dual union all
select 351,'USAF' from dual union all
select 3510,'USAF' from dual union all
select 35100,'CASC' from dual union all
select 35101,'CASC' from dual union all
select 35102,'CASC' from dual union all
select 35103,'CASC' from dual union all
select 35104,'CASC' from dual union all
select 35105,'CASC' from dual union all
select 35106,'CASC' from dual union all
select 35107,'CASC' from dual union all
select 35108,'CASC' from dual union all
select 35109,'CASC' from dual union all
select 3511,'TSKBEM' from dual union all
select 35110,'CASC' from dual union all
select 35111,'CASC' from dual union all
select 35112,'CASC' from dual union all
select 35113,'CASC' from dual union all
select 35114,'CASC' from dual union all
select 35115,'CASC' from dual union all
select 35116,'CASC' from dual union all
select 35117,'CASC' from dual union all
select 35118,'CASC' from dual union all
select 35119,'CASC' from dual union all
select 3512,'RVSN' from dual union all
select 35120,'CASC' from dual union all
select 35121,'CASC' from dual;
insert into SATELLITE_ORG
select 35122,'CASC' from dual union all
select 35123,'CASC' from dual union all
select 35124,'CASC' from dual union all
select 35125,'CASC' from dual union all
select 35126,'CASC' from dual union all
select 35127,'CASC' from dual union all
select 35128,'CASC' from dual union all
select 35129,'CASC' from dual union all
select 3513,'RVSN' from dual union all
select 35130,'CASC' from dual union all
select 35131,'CASC' from dual union all
select 35132,'CASC' from dual union all
select 35133,'CASC' from dual union all
select 35134,'CASC' from dual union all
select 35135,'CASC' from dual union all
select 35136,'CASC' from dual union all
select 35137,'CASC' from dual union all
select 35138,'CASC' from dual union all
select 35139,'CASC' from dual union all
select 3514,'RVSN' from dual union all
select 35140,'CASC' from dual union all
select 35141,'CASC' from dual union all
select 35142,'CASC' from dual union all
select 35143,'CASC' from dual union all
select 35144,'CASC' from dual union all
select 35145,'CASC' from dual union all
select 35146,'CASC' from dual union all
select 35147,'CASC' from dual union all
select 35148,'CASC' from dual union all
select 35149,'CASC' from dual union all
select 3515,'RVSN' from dual union all
select 35150,'CASC' from dual union all
select 35151,'CASC' from dual union all
select 35152,'CASC' from dual union all
select 35153,'CASC' from dual union all
select 35154,'CASC' from dual union all
select 35155,'CASC' from dual union all
select 35156,'CASC' from dual union all
select 35157,'CASC' from dual union all
select 35158,'CASC' from dual union all
select 35159,'CASC' from dual union all
select 3516,'TSKBEM' from dual union all
select 35160,'CASC' from dual union all
select 35161,'CASC' from dual union all
select 35162,'CASC' from dual union all
select 35163,'CASC' from dual union all
select 35164,'CASC' from dual union all
select 35165,'CASC' from dual union all
select 35166,'CASC' from dual union all
select 35167,'CASC' from dual union all
select 35168,'CASC' from dual union all
select 35169,'CASC' from dual union all
select 3517,'RVSN' from dual union all
select 35170,'CASC' from dual union all
select 35171,'CASC' from dual union all
select 35172,'CASC' from dual union all
select 35173,'CASC' from dual union all
select 35174,'CASC' from dual union all
select 35175,'CASC' from dual union all
select 35176,'CASC' from dual union all
select 35177,'CASC' from dual union all
select 35178,'CASC' from dual union all
select 35179,'CASC' from dual union all
select 3518,'PVO' from dual union all
select 35180,'CASC' from dual union all
select 35181,'CASC' from dual union all
select 35182,'CASC' from dual union all
select 35183,'CASC' from dual union all
select 35184,'CASC' from dual union all
select 35185,'CASC' from dual union all
select 35186,'CASC' from dual union all
select 35187,'CASC' from dual union all
select 35188,'CASC' from dual union all
select 35189,'CASC' from dual union all
select 3519,'PVO' from dual union all
select 35190,'CASC' from dual union all
select 35191,'CASC' from dual union all
select 35192,'CASC' from dual union all
select 35193,'CASC' from dual union all
select 35194,'CASC' from dual union all
select 35195,'CASC' from dual union all
select 35196,'CASC' from dual union all
select 35197,'CASC' from dual union all
select 35198,'CASC' from dual union all
select 35199,'CASC' from dual union all
select 352,'USAF' from dual union all
select 3520,'PVO' from dual union all
select 35200,'CASC' from dual union all
select 35201,'CASC' from dual union all
select 35202,'CASC' from dual union all
select 35203,'CASC' from dual union all
select 35204,'CASC' from dual union all
select 35205,'CASC' from dual union all
select 35206,'CASC' from dual union all
select 35207,'CASC' from dual union all
select 35208,'CASC' from dual union all
select 35209,'CASC' from dual union all
select 3521,'PVO' from dual union all
select 35210,'CASC' from dual union all
select 35211,'CASC' from dual;
insert into SATELLITE_ORG
select 35212,'CASC' from dual union all
select 35213,'CASC' from dual union all
select 35214,'CASC' from dual union all
select 35215,'CASC' from dual union all
select 35216,'CASC' from dual union all
select 35217,'CASC' from dual union all
select 35218,'CASC' from dual union all
select 35219,'CASC' from dual union all
select 3522,'USAF' from dual union all
select 35220,'CASC' from dual union all
select 35221,'CASC' from dual union all
select 35222,'CASC' from dual union all
select 35223,'CASC' from dual union all
select 35224,'CASC' from dual union all
select 35225,'CASC' from dual union all
select 35226,'CASC' from dual union all
select 35227,'CASC' from dual union all
select 35228,'CASC' from dual union all
select 35229,'CASC' from dual union all
select 3523,'PVO' from dual union all
select 35230,'CASC' from dual union all
select 35231,'CASC' from dual union all
select 35232,'CASC' from dual union all
select 35233,'CASC' from dual union all
select 35234,'CASC' from dual union all
select 35235,'CASC' from dual union all
select 35236,'CASC' from dual union all
select 35237,'CASC' from dual union all
select 35238,'CASC' from dual union all
select 35239,'CASC' from dual union all
select 3524,'PVO' from dual union all
select 35240,'CASC' from dual union all
select 35241,'CASC' from dual union all
select 35242,'CASC' from dual union all
select 35243,'CASC' from dual union all
select 35244,'CASC' from dual union all
select 35245,'CASC' from dual union all
select 35246,'CASC' from dual union all
select 35247,'CASC' from dual union all
select 35248,'CNSA' from dual union all
select 35249,'CNSA' from dual union all
select 3525,'PVO' from dual union all
select 35250,'CNSA' from dual union all
select 35251,'CNSA' from dual union all
select 35252,'CNSA' from dual union all
select 35253,'KVR' from dual union all
select 35254,'KVR' from dual union all
select 35255,'KVR' from dual union all
select 35256,'KVR' from dual union all
select 35257,'KVR' from dual union all
select 35258,'ESA' from dual union all
select 35259,'ESA' from dual union all
select 3526,'TSN108' from dual union all
select 35260,'ESA' from dual union all
select 35261,'ESA' from dual union all
select 35262,'ESA' from dual union all
select 35263,'KVR' from dual union all
select 35264,'KVR' from dual union all
select 35265,'KVR' from dual union all
select 35266,'KVR' from dual union all
select 35267,'KVR' from dual union all
select 35268,'CNSA' from dual union all
select 35269,'CNSA' from dual union all
select 3527,'RVSN' from dual union all
select 35270,'CNSA' from dual union all
select 35271,'CASC' from dual union all
select 35272,'CASC' from dual union all
select 35273,'CASC' from dual union all
select 35274,'CASC' from dual union all
select 35275,'CASC' from dual union all
select 35276,'RVSNR' from dual union all
select 35277,'RVSNR' from dual union all
select 35278,'KVR' from dual union all
select 35279,'KVR' from dual union all
select 3528,'MO' from dual union all
select 35280,'KVR' from dual union all
select 35281,'KVR' from dual union all
select 35282,'KVR' from dual union all
select 35283,'KVR' from dual union all
select 35284,'KVR' from dual union all
select 35285,'KVR' from dual union all
select 35286,'KVR' from dual union all
select 35287,'KVR' from dual union all
select 35288,'KVR' from dual union all
select 35289,'KVR' from dual union all
select 3529,'RVSN' from dual union all
select 35290,'KVR' from dual union all
select 35291,'KVR' from dual union all
select 35292,'KVR' from dual union all
select 35293,'IRIDS' from dual union all
select 35294,'IRIDS' from dual union all
select 35295,'IRIDS' from dual union all
select 35296,'IRIDS' from dual union all
select 35297,'IRIDS' from dual union all
select 35298,'IRIDS' from dual union all
select 35299,'IRIDS' from dual union all
select 353,'USAF' from dual union all
select 3530,'PVO' from dual union all
select 35300,'IRIDS' from dual union all
select 35301,'IRIDS' from dual;
insert into SATELLITE_ORG
select 35302,'IRIDS' from dual union all
select 35303,'IRIDS' from dual union all
select 35304,'IRIDS' from dual union all
select 35305,'IRIDS' from dual union all
select 35306,'IRIDS' from dual union all
select 35307,'RVSNR' from dual union all
select 35308,'RVSNR' from dual union all
select 35309,'RVSNR' from dual union all
select 3531,'CIA' from dual union all
select 3531,'NRO' from dual union all
select 35310,'RVSNR' from dual union all
select 35311,'RVSNR' from dual union all
select 35312,'RVSNR' from dual union all
select 35313,'RVSNR' from dual union all
select 35314,'RVSNR' from dual union all
select 35315,'NASA' from dual union all
select 35316,'NASA' from dual union all
select 35317,'NASA' from dual union all
select 35318,'CNSA' from dual union all
select 35319,'CNSA' from dual union all
select 3532,'NRO' from dual union all
select 35320,'CNSA' from dual union all
select 35321,'CNSA' from dual union all
select 35322,'CNSA' from dual union all
select 35323,'CNSA' from dual union all
select 35324,'CNSA' from dual union all
select 35325,'CNSA' from dual union all
select 35326,'CNSA' from dual union all
select 35327,'CNSA' from dual union all
select 35328,'CNSA' from dual union all
select 35329,'CNSA' from dual union all
select 3533,'NASA' from dual union all
select 35330,'CNSA' from dual union all
select 35331,'CNSA' from dual union all
select 35332,'CNSA' from dual union all
select 35333,'CNSA' from dual union all
select 35334,'CNSA' from dual union all
select 35335,'ISRO' from dual union all
select 35336,'ISRO' from dual union all
select 35337,'ISRO' from dual union all
select 35338,'ISRO' from dual union all
select 35339,'ISRO' from dual union all
select 3534,'NASA' from dual union all
select 35340,'ISRO' from dual union all
select 35341,'ISRO' from dual union all
select 35342,'ISRO' from dual union all
select 35343,'ISRO' from dual union all
select 35344,'ISRO' from dual union all
select 35345,'ISRO' from dual union all
select 35346,'ISRO' from dual union all
select 35347,'ISRO' from dual union all
select 35348,'ISRO' from dual union all
select 35349,'ISRO' from dual union all
select 3535,'TSKBEM' from dual union all
select 35350,'ISRO' from dual union all
select 35351,'ISRO' from dual union all
select 35352,'RVSNR' from dual union all
select 35353,'RVSNR' from dual union all
select 35354,'RVSNR' from dual union all
select 35355,'RVSNR' from dual union all
select 35356,'RVSNR' from dual union all
select 35357,'RVSNR' from dual union all
select 35358,'RVSNR' from dual union all
select 35359,'RVSNR' from dual union all
select 3536,'RVSN' from dual union all
select 35360,'RVSNR' from dual union all
select 35361,'RVSNR' from dual union all
select 35362,'MEAS' from dual union all
select 35363,'SIS' from dual union all
select 35364,'KVR' from dual union all
select 35365,'KVR' from dual union all
select 35366,'KVR' from dual union all
select 35367,'KVR' from dual union all
select 35368,'KVR' from dual union all
select 35369,'KVR' from dual union all
select 3537,'RVSN' from dual union all
select 35370,'KVR' from dual union all
select 35371,'KVR' from dual union all
select 35372,'KVR' from dual union all
select 35373,'KVR' from dual union all
select 35374,'CNSA' from dual union all
select 35375,'CNSA' from dual union all
select 35376,'CNSA' from dual union all
select 35377,'CNSA' from dual union all
select 35378,'CNSA' from dual union all
select 35379,'CNSA' from dual union all
select 3538,'TSKBEM' from dual union all
select 35380,'CNSA' from dual union all
select 35381,'CNSA' from dual union all
select 35382,'CNSA' from dual union all
select 35383,'CNSA' from dual union all
select 35384,'CNSA' from dual union all
select 35385,'CNSA' from dual union all
select 35386,'CNSA' from dual union all
select 35387,'CNSA' from dual union all
select 35388,'CNSA' from dual union all
select 35389,'CNSA' from dual union all
select 3539,'TSKBEM' from dual union all
select 35390,'CNSA' from dual union all
select 35391,'ESA' from dual;
insert into SATELLITE_ORG
select 35392,'ESA' from dual union all
select 35393,'ESA' from dual union all
select 35394,'JPL' from dual union all
select 35395,'JPL' from dual union all
select 35396,'JPL' from dual union all
select 35397,'CNSA' from dual union all
select 35398,'CNSA' from dual union all
select 35399,'RVSNR' from dual union all
select 354,'USAF' from dual union all
select 3540,'MO' from dual union all
select 35400,'RVSNR' from dual union all
select 35401,'USAF' from dual union all
select 35402,'USAF' from dual union all
select 35403,'USAF' from dual union all
select 35404,'CNSA' from dual union all
select 35405,'CNSA' from dual union all
select 35406,'KVR' from dual union all
select 35407,'KVR' from dual union all
select 35408,'AE' from dual union all
select 35409,'AE' from dual union all
select 3541,'MO' from dual union all
select 35410,'NASA' from dual union all
select 35411,'USAF' from dual union all
select 35412,'KVR' from dual union all
select 35413,'KTRAS' from dual union all
select 35414,'NASA' from dual union all
select 35415,'USAF' from dual union all
select 35416,'USN' from dual union all
select 35417,'CASC' from dual union all
select 35418,'JAXA' from dual union all
select 35419,'USAF' from dual union all
select 3542,'MO' from dual union all
select 35420,'RGMS' from dual union all
select 35421,'USAF' from dual union all
select 35422,'KVR' from dual union all
select 35423,'KVR' from dual union all
select 35424,'KVR' from dual union all
select 35425,'NROC' from dual union all
select 35426,'KVR' from dual union all
select 35427,'KVR' from dual union all
select 35428,'BOE' from dual union all
select 35429,'KVR' from dual union all
select 3543,'RVSN' from dual union all
select 35430,'KVR' from dual union all
select 35431,'KVR' from dual union all
select 35432,'KVR' from dual union all
select 35433,'KVR' from dual union all
select 35434,'KVR' from dual union all
select 35435,'KVR' from dual union all
select 35436,'KVR' from dual union all
select 35437,'KVR' from dual union all
select 35438,'KVR' from dual union all
select 35439,'KVR' from dual union all
select 3544,'TSKBEM' from dual union all
select 35440,'KVR' from dual union all
select 35441,'KVR' from dual union all
select 35442,'KVR' from dual union all
select 35443,'KVR' from dual union all
select 35444,'KVR' from dual union all
select 35445,'KVR' from dual union all
select 35446,'KVR' from dual union all
select 35447,'KVR' from dual union all
select 35448,'KVR' from dual union all
select 35449,'KVR' from dual union all
select 3545,'RVSN' from dual union all
select 35450,'KVR' from dual union all
select 35451,'KVR' from dual union all
select 35452,'KVR' from dual union all
select 35453,'KVR' from dual union all
select 35454,'KVR' from dual union all
select 35455,'KVR' from dual union all
select 35456,'KVR' from dual union all
select 35457,'KVR' from dual union all
select 35458,'KVR' from dual union all
select 35459,'KVR' from dual union all
select 3546,'NASA' from dual union all
select 35460,'KVR' from dual union all
select 35461,'KVR' from dual union all
select 35462,'KVR' from dual union all
select 35463,'KVR' from dual union all
select 35464,'KVR' from dual union all
select 35465,'KVR' from dual union all
select 35466,'KVR' from dual union all
select 35467,'KVR' from dual union all
select 35468,'KVR' from dual union all
select 35469,'KVR' from dual union all
select 3547,'NASA' from dual union all
select 35470,'KVR' from dual union all
select 35471,'KVR' from dual union all
select 35472,'KVR' from dual union all
select 35473,'KVR' from dual union all
select 35474,'KVR' from dual union all
select 35475,'KVR' from dual union all
select 35476,'KVR' from dual union all
select 35477,'IRIDS' from dual union all
select 35478,'IRIDS' from dual union all
select 35479,'IRIDS' from dual union all
select 3548,'NASA' from dual union all
select 35480,'IRIDS' from dual union all
select 35481,'IRIDS' from dual;
insert into SATELLITE_ORG
select 35482,'IRIDS' from dual union all
select 35483,'IRIDS' from dual union all
select 35484,'IRIDS' from dual union all
select 35485,'IRIDS' from dual union all
select 35486,'IRIDS' from dual union all
select 35487,'IRIDS' from dual union all
select 35488,'IRIDS' from dual union all
select 35489,'IRIDS' from dual union all
select 3549,'PVO' from dual union all
select 35490,'IRIDS' from dual union all
select 35491,'NOAA' from dual union all
select 35492,'BOE' from dual union all
select 35493,'SIRX' from dual union all
select 35494,'KHRR' from dual union all
select 35495,'KHRR' from dual union all
select 35496,'TERRE' from dual union all
select 35497,'AE' from dual union all
select 35498,'KVR' from dual union all
select 35499,'KVR' from dual union all
select 355,'USAF' from dual union all
select 3550,'PVO' from dual union all
select 35500,'KVR' from dual union all
select 35501,'KVR' from dual union all
select 35502,'USAF' from dual union all
select 35503,'USAF' from dual union all
select 35504,'RVSN' from dual union all
select 35505,'RVSN' from dual union all
select 35506,'KVR' from dual union all
select 35507,'MO' from dual union all
select 35508,'MO' from dual union all
select 35509,'MO' from dual union all
select 3551,'PVO' from dual union all
select 35510,'MO' from dual union all
select 35511,'MO' from dual union all
select 35512,'MO' from dual union all
select 35513,'MO' from dual union all
select 35514,'MO' from dual union all
select 35515,'KVR' from dual union all
select 35516,'KVR' from dual union all
select 35517,'AE' from dual union all
select 35518,'KVR' from dual union all
select 35519,'RVSNR' from dual union all
select 3552,'PVO' from dual union all
select 35520,'RVSNR' from dual union all
select 35521,'RVSNR' from dual union all
select 35522,'RVSNR' from dual union all
select 35523,'RVSNR' from dual union all
select 35524,'RVSNR' from dual union all
select 35525,'RVSNR' from dual union all
select 35526,'USN' from dual union all
select 35527,'USN' from dual union all
select 35528,'USN' from dual union all
select 35529,'USN' from dual union all
select 3553,'PVO' from dual union all
select 35530,'RVSNR' from dual union all
select 35531,'USAF' from dual union all
select 35532,'USAF' from dual union all
select 35533,'NASA' from dual union all
select 35534,'USAF' from dual union all
select 35535,'NASA' from dual union all
select 35536,'ISRO' from dual union all
select 35537,'ISRO' from dual union all
select 35538,'KHRR' from dual union all
select 35539,'KHRR' from dual union all
select 3554,'PVO' from dual union all
select 35540,'KHRR' from dual union all
select 35541,'KHRR' from dual union all
select 35542,'KHRR' from dual union all
select 35543,'KHRR' from dual union all
select 35544,'KHRR' from dual union all
select 35545,'KHRR' from dual union all
select 35546,'KHRR' from dual union all
select 35547,'CNSA' from dual union all
select 35548,'CNSA' from dual union all
select 35549,'CNSA' from dual union all
select 3555,'PVO' from dual union all
select 35550,'CNSA' from dual union all
select 35551,'CNSA' from dual union all
select 35552,'CNSA' from dual union all
select 35553,'CNSA' from dual union all
select 35554,'CNSA' from dual union all
select 35555,'CNSA' from dual union all
select 35556,'CNSA' from dual union all
select 35557,'CNSA' from dual union all
select 35558,'CNSA' from dual union all
select 35559,'CNSA' from dual union all
select 3556,'PVO' from dual union all
select 35560,'CNSA' from dual union all
select 35561,'CNSA' from dual union all
select 35562,'CNSA' from dual union all
select 35563,'CNSA' from dual union all
select 35564,'CNSA' from dual union all
select 35565,'CNSA' from dual union all
select 35566,'CNSA' from dual union all
select 35567,'CNSA' from dual union all
select 35568,'BOE' from dual union all
select 35569,'CASC' from dual union all
select 3557,'PVO' from dual union all
select 35570,'GEYE' from dual union all
select 35571,'CASC' from dual;
insert into SATELLITE_ORG
select 35572,'CASC' from dual union all
select 35573,'CASC' from dual union all
select 35574,'CASC' from dual union all
select 35575,'CASC' from dual union all
select 35576,'CNSA' from dual union all
select 35577,'SIS' from dual union all
select 35578,'ATSB' from dual union all
select 35579,'SPX' from dual union all
select 3558,'PVO' from dual union all
select 35580,'KVR' from dual union all
select 35581,'KVR' from dual union all
select 35582,'KVR' from dual union all
select 35583,'KVR' from dual union all
select 35584,'KVR' from dual union all
select 35585,'KVR' from dual union all
select 35586,'KVR' from dual union all
select 35587,'KVR' from dual union all
select 35588,'KVR' from dual union all
select 35589,'KVR' from dual union all
select 3559,'PVO' from dual union all
select 35590,'KVR' from dual union all
select 35591,'KVR' from dual union all
select 35592,'KVR' from dual union all
select 35593,'KVR' from dual union all
select 35594,'KVR' from dual union all
select 35595,'KVR' from dual union all
select 35596,'KVR' from dual union all
select 35597,'KVR' from dual union all
select 35598,'KVR' from dual union all
select 35599,'KVR' from dual union all
select 356,'USAF' from dual union all
select 3560,'PVO' from dual union all
select 35600,'KVR' from dual union all
select 35601,'KVR' from dual union all
select 35602,'KVR' from dual union all
select 35603,'KVR' from dual union all
select 35604,'KVR' from dual union all
select 35605,'KVR' from dual union all
select 35606,'KVR' from dual union all
select 35607,'KVR' from dual union all
select 35608,'KVR' from dual union all
select 35609,'KVR' from dual union all
select 3561,'PVO' from dual union all
select 35610,'KVR' from dual union all
select 35611,'KVR' from dual union all
select 35612,'KVR' from dual union all
select 35613,'KVR' from dual union all
select 35614,'KVR' from dual union all
select 35615,'IRIDS' from dual union all
select 35616,'IRIDS' from dual union all
select 35617,'IRIDS' from dual union all
select 35618,'IRIDS' from dual union all
select 35619,'IRIDS' from dual union all
select 3562,'MO' from dual union all
select 35620,'IRIDS' from dual union all
select 35621,'IRIDS' from dual union all
select 35622,'IRIDS' from dual union all
select 35623,'IRIDS' from dual union all
select 35624,'IRIDS' from dual union all
select 35625,'IRIDS' from dual union all
select 35626,'IRIDS' from dual union all
select 35627,'IRIDS' from dual union all
select 35628,'IRIDS' from dual union all
select 35629,'IRIDS' from dual union all
select 3563,'RVSN' from dual union all
select 35630,'IRIDS' from dual union all
select 35631,'IRIDS' from dual union all
select 35632,'IRIDS' from dual union all
select 35633,'NASA' from dual union all
select 35634,'JAXA' from dual union all
select 35635,'VMFR' from dual union all
select 35636,'FKA' from dual union all
select 35637,'KVR' from dual union all
select 35638,'JAXA' from dual union all
select 35639,'JAXA' from dual union all
select 3564,'PVO' from dual union all
select 35640,'JAXA' from dual union all
select 35641,'FKA' from dual union all
select 35642,'KVR' from dual union all
select 35643,'KVR' from dual union all
select 35644,'KVR' from dual union all
select 35645,'KVR' from dual union all
select 35646,'KVR' from dual union all
select 35647,'KVR' from dual union all
select 35648,'KVR' from dual union all
select 35649,'KVR' from dual union all
select 3565,'PVO' from dual union all
select 35650,'KVR' from dual union all
select 35651,'KVR' from dual union all
select 35652,'KVR' from dual union all
select 35653,'KVR' from dual union all
select 35654,'KVR' from dual union all
select 35655,'KVR' from dual union all
select 35656,'KVR' from dual union all
select 35657,'KVR' from dual union all
select 35658,'KVR' from dual union all
select 35659,'KVR' from dual union all
select 3566,'PVO' from dual union all
select 35660,'KVR' from dual union all
select 35661,'KVR' from dual;
insert into SATELLITE_ORG
select 35662,'KVR' from dual union all
select 35663,'KVR' from dual union all
select 35664,'KVR' from dual union all
select 35665,'KVR' from dual union all
select 35666,'KVR' from dual union all
select 35667,'KVR' from dual union all
select 35668,'KVR' from dual union all
select 35669,'KVR' from dual union all
select 3567,'PVO' from dual union all
select 35670,'KVR' from dual union all
select 35671,'KVR' from dual union all
select 35672,'KVR' from dual union all
select 35673,'KVR' from dual union all
select 35674,'KVR' from dual union all
select 35675,'KVR' from dual union all
select 35676,'KVR' from dual union all
select 35677,'IRIDS' from dual union all
select 35678,'IRIDS' from dual union all
select 35679,'IRIDS' from dual union all
select 3568,'PVO' from dual union all
select 35680,'IRIDS' from dual union all
select 35681,'DEIMOS' from dual union all
select 35682,'EIAST' from dual union all
select 35683,'DMC' from dual union all
select 35684,'SPQ' from dual union all
select 35685,'INTA' from dual union all
select 35686,'SPQ' from dual union all
select 35687,'KTRAS' from dual union all
select 35688,'KTRAS' from dual union all
select 35689,'KTRAS' from dual union all
select 3569,'PVO' from dual union all
select 35690,'TAMU' from dual union all
select 35691,'USAF' from dual union all
select 35691,'USN' from dual union all
select 35692,'USAF' from dual union all
select 35692,'USN' from dual union all
select 35693,'USAF' from dual union all
select 35693,'USN' from dual union all
select 35694,'USAF' from dual union all
select 35694,'USN' from dual union all
select 35695,'USAF' from dual union all
select 35695,'USN' from dual union all
select 35696,'ASIA' from dual union all
select 35697,'KHRR' from dual union all
select 35698,'KHRR' from dual union all
select 35699,'KVR' from dual union all
select 357,'USAF' from dual union all
select 3570,'PVO' from dual union all
select 35700,'KVR' from dual union all
select 35701,'KVR' from dual union all
select 35702,'KVR' from dual union all
select 35703,'KVR' from dual union all
select 35704,'KVR' from dual union all
select 35705,'KVR' from dual union all
select 35706,'KVR' from dual union all
select 35707,'KVR' from dual union all
select 35708,'KVR' from dual union all
select 35709,'KVR' from dual union all
select 3571,'PVO' from dual union all
select 35710,'KVR' from dual union all
select 35711,'KVR' from dual union all
select 35712,'KVR' from dual union all
select 35713,'KVR' from dual union all
select 35714,'KVR' from dual union all
select 35715,'KVR' from dual union all
select 35716,'KVR' from dual union all
select 35717,'KVR' from dual union all
select 35718,'KVR' from dual union all
select 35719,'KVR' from dual union all
select 3572,'PVO' from dual union all
select 35720,'KVR' from dual union all
select 35721,'KVR' from dual union all
select 35722,'KVR' from dual union all
select 35723,'KVR' from dual union all
select 35724,'KVR' from dual union all
select 35725,'KVR' from dual union all
select 35726,'KVR' from dual union all
select 35727,'KVR' from dual union all
select 35728,'KVR' from dual union all
select 35729,'KVR' from dual union all
select 3573,'PVO' from dual union all
select 35730,'KVR' from dual union all
select 35731,'KVR' from dual union all
select 35732,'IRIDS' from dual union all
select 35733,'IRIDS' from dual union all
select 35734,'IRIDS' from dual union all
select 35735,'IRIDS' from dual union all
select 35736,'IRIDS' from dual union all
select 35737,'IRIDS' from dual union all
select 35738,'IRIDS' from dual union all
select 35739,'IRIDS' from dual union all
select 3574,'MO' from dual union all
select 35740,'IRIDS' from dual union all
select 35741,'IRIDS' from dual union all
select 35742,'IRIDS' from dual union all
select 35743,'IRIDS' from dual union all
select 35744,'IRIDS' from dual union all
select 35745,'IRIDS' from dual union all
select 35746,'IRIDS' from dual union all
select 35747,'IRIDS' from dual;
insert into SATELLITE_ORG
select 35748,'IRIDS' from dual union all
select 35749,'IRIDS' from dual union all
select 3575,'RVSN' from dual union all
select 35750,'IRIDS' from dual union all
select 35751,'IRIDS' from dual union all
select 35752,'USAF' from dual union all
select 35753,'BOE' from dual union all
select 35754,'BOE' from dual union all
select 35755,'SKPJ' from dual union all
select 35756,'OPTS' from dual union all
select 35757,'AE' from dual union all
select 35758,'AE' from dual union all
select 35759,'KVR' from dual union all
select 3576,'OKB10' from dual union all
select 35760,'KVR' from dual union all
select 35761,'KVR' from dual union all
select 35762,'KVR' from dual union all
select 35763,'KVR' from dual union all
select 35764,'KVR' from dual union all
select 35765,'KVR' from dual union all
select 35766,'KVR' from dual union all
select 35767,'KVR' from dual union all
select 35768,'KVR' from dual union all
select 35769,'KVR' from dual union all
select 3577,'RVSN' from dual union all
select 35770,'KVR' from dual union all
select 35771,'KVR' from dual union all
select 35772,'KVR' from dual union all
select 35773,'KVR' from dual union all
select 35774,'KVR' from dual union all
select 35775,'KVR' from dual union all
select 35776,'KVR' from dual union all
select 35777,'KVR' from dual union all
select 35778,'KVR' from dual union all
select 35779,'KVR' from dual union all
select 3578,'PVO' from dual union all
select 35780,'KVR' from dual union all
select 35781,'KVR' from dual union all
select 35782,'KVR' from dual union all
select 35783,'KVR' from dual union all
select 35784,'KVR' from dual union all
select 35785,'KVR' from dual union all
select 35786,'KVR' from dual union all
select 35787,'KVR' from dual union all
select 35788,'KVR' from dual union all
select 35789,'KVR' from dual union all
select 3579,'RVSN' from dual union all
select 35790,'KVR' from dual union all
select 35791,'KVR' from dual union all
select 35792,'KVR' from dual union all
select 35793,'KVR' from dual union all
select 35794,'KVR' from dual union all
select 35795,'KVR' from dual union all
select 35796,'KVR' from dual union all
select 35797,'IRIDS' from dual union all
select 35798,'IRIDS' from dual union all
select 35799,'IRIDS' from dual union all
select 358,'USAF' from dual union all
select 3580,'PVO' from dual union all
select 35800,'IRIDS' from dual union all
select 35801,'IRIDS' from dual union all
select 35802,'IRIDS' from dual union all
select 35803,'IRIDS' from dual union all
select 35804,'IRIDS' from dual union all
select 35805,'IRIDS' from dual union all
select 35806,'IRIDS' from dual union all
select 35807,'IRIDS' from dual union all
select 35808,'IRIDS' from dual union all
select 35809,'IRIDS' from dual union all
select 3581,'PVO' from dual union all
select 35810,'RVSNR' from dual union all
select 35811,'NASA' from dual union all
select 35812,'INDOS' from dual union all
select 35813,'CASC' from dual union all
select 35814,'NASA' from dual union all
select 35815,'NROC' from dual union all
select 35816,'LM' from dual union all
select 35817,'JAXA' from dual union all
select 35818,'JAXA' from dual union all
select 35819,'JAXA' from dual union all
select 3582,'PVO' from dual union all
select 35820,'JAXA' from dual union all
select 35821,'KVR' from dual union all
select 35822,'KVR' from dual union all
select 35823,'KVR' from dual union all
select 35824,'KVR' from dual union all
select 35825,'KVR' from dual union all
select 35826,'KVR' from dual union all
select 35827,'KVR' from dual union all
select 35828,'KVR' from dual union all
select 35829,'KVR' from dual union all
select 3583,'PVO' from dual union all
select 35830,'KVR' from dual union all
select 35831,'KVR' from dual union all
select 35832,'KVR' from dual union all
select 35833,'KVR' from dual union all
select 35834,'KVR' from dual union all
select 35835,'KVR' from dual union all
select 35836,'KVR' from dual union all
select 35837,'KVR' from dual;
insert into SATELLITE_ORG
select 35838,'KVR' from dual union all
select 35839,'KVR' from dual union all
select 3584,'PVO' from dual union all
select 35840,'KVR' from dual union all
select 35841,'KVR' from dual union all
select 35842,'KVR' from dual union all
select 35843,'KVR' from dual union all
select 35844,'IRIDS' from dual union all
select 35845,'IRIDS' from dual union all
select 35846,'IRIDS' from dual union all
select 35847,'IRIDS' from dual union all
select 35848,'IRIDS' from dual union all
select 35849,'IRIDS' from dual union all
select 3585,'PVO' from dual union all
select 35850,'IRIDS' from dual union all
select 35851,'IRIDS' from dual union all
select 35852,'IRIDS' from dual union all
select 35853,'IRIDS' from dual union all
select 35854,'IRIDS' from dual union all
select 35855,'IRIDS' from dual union all
select 35856,'IRIDS' from dual union all
select 35857,'IRIDS' from dual union all
select 35858,'IRIDS' from dual union all
select 35859,'IRIDS' from dual union all
select 3586,'PVO' from dual union all
select 35860,'IRIDS' from dual union all
select 35861,'IRIDS' from dual union all
select 35862,'IRIDS' from dual union all
select 35863,'IRIDS' from dual union all
select 35864,'CASC' from dual union all
select 35865,'FKA' from dual union all
select 35865,'RGMS' from dual union all
select 35866,'FKA' from dual union all
select 35867,'NPOL' from dual union all
select 35868,'FKA' from dual union all
select 35868,'MGU' from dual union all
select 35869,'UGATU' from dual union all
select 3587,'PVO' from dual union all
select 35870,'CSIR' from dual union all
select 35871,'NIIPP' from dual union all
select 35872,'RVSNR' from dual union all
select 35873,'TCAN' from dual union all
select 35874,'KHRR' from dual union all
select 35875,'KHRR' from dual union all
select 35876,'KVR' from dual union all
select 35877,'KVR' from dual union all
select 35878,'KVR' from dual union all
select 35879,'KVR' from dual union all
select 3588,'PVO' from dual union all
select 35880,'KVR' from dual union all
select 35881,'KVR' from dual union all
select 35882,'KVR' from dual union all
select 35883,'KVR' from dual union all
select 35884,'KVR' from dual union all
select 35885,'KVR' from dual union all
select 35886,'KVR' from dual union all
select 35887,'KVR' from dual union all
select 35888,'KVR' from dual union all
select 35889,'KVR' from dual union all
select 3589,'CNES' from dual union all
select 35890,'KVR' from dual union all
select 35891,'KVR' from dual union all
select 35892,'KVR' from dual union all
select 35893,'KVR' from dual union all
select 35894,'KVR' from dual union all
select 35895,'KVR' from dual union all
select 35896,'KVR' from dual union all
select 35897,'KVR' from dual union all
select 35898,'KVR' from dual union all
select 35899,'KVR' from dual union all
select 359,'USAF' from dual union all
select 3590,'USN' from dual union all
select 35900,'KVR' from dual union all
select 35901,'KVR' from dual union all
select 35902,'KVR' from dual union all
select 35903,'KVR' from dual union all
select 35904,'KVR' from dual union all
select 35905,'KVR' from dual union all
select 35906,'KVR' from dual union all
select 35907,'IRIDS' from dual union all
select 35908,'IRIDS' from dual union all
select 35909,'IRIDS' from dual union all
select 3591,'USN' from dual union all
select 35910,'IRIDS' from dual union all
select 35911,'IRIDS' from dual union all
select 35912,'IRIDS' from dual union all
select 35913,'IRIDS' from dual union all
select 35914,'IRIDS' from dual union all
select 35915,'IRIDS' from dual union all
select 35916,'IRIDS' from dual union all
select 35917,'IRIDS' from dual union all
select 35918,'IRIDS' from dual union all
select 35919,'IRIDS' from dual union all
select 3592,'USN' from dual union all
select 35920,'IRIDS' from dual union all
select 35921,'IRIDS' from dual union all
select 35922,'IRIDS' from dual union all
select 35923,'IRIDS' from dual union all
select 35924,'IRIDS' from dual union all
select 35925,'IRIDS' from dual;
insert into SATELLITE_ORG
select 35926,'IRIDS' from dual union all
select 35927,'IRIDS' from dual union all
select 35928,'IRIDS' from dual union all
select 35929,'IRIDS' from dual union all
select 3593,'PVO' from dual union all
select 35930,'IRIDS' from dual union all
select 35931,'ISRO' from dual union all
select 35932,'EPFL' from dual union all
select 35933,'TUB1' from dual union all
select 35934,'WURZ' from dual union all
select 35935,'ITUTR' from dual union all
select 35936,'ISRO' from dual union all
select 35937,'USAF' from dual union all
select 35938,'USAF' from dual union all
select 35939,'BOE' from dual union all
select 3594,'NRO' from dual union all
select 35940,'FKA' from dual union all
select 35941,'KVR' from dual union all
select 35942,'HISP' from dual union all
select 35943,'BWEHR' from dual union all
select 35944,'AE' from dual union all
select 35945,'AE' from dual union all
select 35946,'DGLO' from dual union all
select 35947,'BOE' from dual union all
select 35948,'FKA' from dual union all
select 35949,'KVR' from dual union all
select 3595,'ESRO' from dual union all
select 35950,'JAXA' from dual union all
select 35951,'USAF' from dual union all
select 35952,'LM' from dual union all
select 35953,'USAF' from dual union all
select 35954,'USAF' from dual union all
select 35955,'AE' from dual union all
select 35956,'KVR' from dual union all
select 35957,'KVR' from dual union all
select 35958,'KVR' from dual union all
select 35959,'KVR' from dual union all
select 3596,'NASA' from dual union all
select 35960,'KVR' from dual union all
select 35961,'KVR' from dual union all
select 35962,'KVR' from dual union all
select 35963,'KVR' from dual union all
select 35964,'KVR' from dual union all
select 35965,'KVR' from dual union all
select 35966,'KVR' from dual union all
select 35967,'KVR' from dual union all
select 35968,'KVR' from dual union all
select 35969,'KVR' from dual union all
select 3597,'NASA' from dual union all
select 35970,'KVR' from dual union all
select 35971,'KVR' from dual union all
select 35972,'KVR' from dual union all
select 35973,'KVR' from dual union all
select 35974,'KVR' from dual union all
select 35975,'KVR' from dual union all
select 35976,'KVR' from dual union all
select 35977,'KVR' from dual union all
select 35978,'KVR' from dual union all
select 35979,'KVR' from dual union all
select 3598,'NASA' from dual union all
select 35980,'KVR' from dual union all
select 35981,'KVR' from dual union all
select 35982,'KVR' from dual union all
select 35983,'KVR' from dual union all
select 35984,'KVR' from dual union all
select 35985,'KVR' from dual union all
select 35986,'KVR' from dual union all
select 35987,'KVR' from dual union all
select 35988,'KVR' from dual union all
select 35989,'KVR' from dual union all
select 3599,'PVO' from dual union all
select 35990,'KVR' from dual union all
select 35991,'KVR' from dual union all
select 35992,'KVR' from dual union all
select 35993,'KVR' from dual union all
select 35994,'KVR' from dual union all
select 35995,'KVR' from dual union all
select 35996,'KVR' from dual union all
select 35997,'KVR' from dual union all
select 35998,'KVR' from dual union all
select 35999,'KVR' from dual union all
select 36,'OKB1' from dual union all
select 360,'CIA' from dual union all
select 360,'NRO' from dual union all
select 3600,'PVO' from dual union all
select 36000,'KVR' from dual union all
select 36001,'KVR' from dual union all
select 36002,'KVR' from dual union all
select 36003,'KVR' from dual union all
select 36004,'KVR' from dual union all
select 36005,'KVR' from dual union all
select 36006,'KVR' from dual union all
select 36007,'KVR' from dual union all
select 36008,'KVR' from dual union all
select 36009,'IRIDS' from dual union all
select 3601,'PVO' from dual union all
select 36010,'IRIDS' from dual union all
select 36011,'IRIDS' from dual union all
select 36012,'IRIDS' from dual union all
select 36013,'IRIDS' from dual;
insert into SATELLITE_ORG
select 36014,'IRIDS' from dual union all
select 36015,'IRIDS' from dual union all
select 36016,'IRIDS' from dual union all
select 36017,'IRIDS' from dual union all
select 36018,'IRIDS' from dual union all
select 36019,'IRIDS' from dual union all
select 3602,'MO' from dual union all
select 36020,'IRIDS' from dual union all
select 36021,'IRIDS' from dual union all
select 36022,'IRIDS' from dual union all
select 36023,'IRIDS' from dual union all
select 36024,'IRIDS' from dual union all
select 36025,'IRIDS' from dual union all
select 36026,'IRIDS' from dual union all
select 36027,'IRIDS' from dual union all
select 36028,'IRIDS' from dual union all
select 36029,'IRIDS' from dual union all
select 3603,'RVSN' from dual union all
select 36030,'RVSNR' from dual union all
select 36031,'RVSNR' from dual union all
select 36032,'SES' from dual union all
select 36033,'TNOR' from dual union all
select 36034,'AE' from dual union all
select 36035,'AE' from dual union all
select 36036,'ESA' from dual union all
select 36037,'ESA' from dual union all
select 36038,'EUROK' from dual union all
select 36039,'EUROK' from dual union all
select 3604,'CIA' from dual union all
select 3604,'NRO' from dual union all
select 36040,'KVR' from dual union all
select 36041,'KVR' from dual union all
select 36042,'KVR' from dual union all
select 36043,'KVR' from dual union all
select 36044,'KVR' from dual union all
select 36045,'KVR' from dual union all
select 36046,'KVR' from dual union all
select 36047,'KVR' from dual union all
select 36048,'KVR' from dual union all
select 36049,'KVR' from dual union all
select 3605,'NRO' from dual union all
select 36050,'KVR' from dual union all
select 36051,'KVR' from dual union all
select 36052,'KVR' from dual union all
select 36053,'KVR' from dual union all
select 36054,'KVR' from dual union all
select 36055,'KVR' from dual union all
select 36056,'KVR' from dual union all
select 36057,'KVR' from dual union all
select 36058,'KVR' from dual union all
select 36059,'KVR' from dual union all
select 3606,'PVO' from dual union all
select 36060,'KVR' from dual union all
select 36061,'KVR' from dual union all
select 36062,'KVR' from dual union all
select 36063,'KVR' from dual union all
select 36064,'KVR' from dual union all
select 36065,'KVR' from dual union all
select 36066,'KVR' from dual union all
select 36067,'KVR' from dual union all
select 36068,'KVR' from dual union all
select 36069,'KVR' from dual union all
select 3607,'PVO' from dual union all
select 36070,'KVR' from dual union all
select 36071,'KVR' from dual union all
select 36072,'KVR' from dual union all
select 36073,'KVR' from dual union all
select 36074,'KVR' from dual union all
select 36075,'KVR' from dual union all
select 36076,'KVR' from dual union all
select 36077,'KVR' from dual union all
select 36078,'KVR' from dual union all
select 36079,'KVR' from dual union all
select 3608,'PVO' from dual union all
select 36080,'IRIDS' from dual union all
select 36081,'IRIDS' from dual union all
select 36082,'IRIDS' from dual union all
select 36083,'IRIDS' from dual union all
select 36084,'IRIDS' from dual union all
select 36085,'IRIDS' from dual union all
select 36086,'FKA' from dual union all
select 36087,'FKA' from dual union all
select 36088,'CASC' from dual union all
select 36089,'CASC' from dual union all
select 3609,'PVO' from dual union all
select 36090,'CASC' from dual union all
select 36091,'CASC' from dual union all
select 36092,'CASC' from dual union all
select 36093,'CASC' from dual union all
select 36094,'NASA' from dual union all
select 36095,'KVR' from dual union all
select 36096,'KVR' from dual union all
select 36097,'INTELB' from dual union all
select 36098,'LM' from dual union all
select 36099,'USAF' from dual union all
select 361,'USAF' from dual union all
select 3610,'PVO' from dual union all
select 36100,'USAF' from dual union all
select 36101,'EUTSA' from dual union all
select 36102,'KHRR' from dual;
insert into SATELLITE_ORG
select 36103,'KHRR' from dual union all
select 36104,'CSICE' from dual union all
select 36105,'JAXA' from dual union all
select 36106,'INTELB' from dual union all
select 36107,'SIS' from dual union all
select 36108,'USAF' from dual union all
select 36109,'BOE' from dual union all
select 3611,'USAF' from dual union all
select 36110,'PLA' from dual union all
select 36111,'FKA' from dual union all
select 36111,'KVR' from dual union all
select 36112,'FKA' from dual union all
select 36112,'KVR' from dual union all
select 36113,'FKA' from dual union all
select 36113,'KVR' from dual union all
select 36114,'KHRR' from dual union all
select 36115,'KVR' from dual union all
select 36116,'KVR' from dual union all
select 36117,'KHRR' from dual union all
select 36118,'KHRR' from dual union all
select 36119,'JPL' from dual union all
select 3612,'YUZH' from dual union all
select 36120,'BOE' from dual union all
select 36121,'PLA' from dual union all
select 36122,'CAMSAT' from dual union all
select 36123,'CNSA' from dual union all
select 36124,'CNES' from dual union all
select 36124,'DGA' from dual union all
select 36125,'AE' from dual union all
select 36126,'KVR' from dual union all
select 36127,'KVR' from dual union all
select 36128,'KVR' from dual union all
select 36129,'FKA' from dual union all
select 3613,'RVSN' from dual union all
select 36130,'KVR' from dual union all
select 36131,'DTV' from dual union all
select 36132,'KVR' from dual union all
select 36133,'KHRR' from dual union all
select 36134,'JPL' from dual union all
select 36135,'CASC' from dual union all
select 36136,'CASC' from dual union all
select 36137,'CASC' from dual union all
select 36138,'CASC' from dual union all
select 36139,'CASC' from dual union all
select 3614,'MO' from dual union all
select 36140,'CASC' from dual union all
select 36141,'CASC' from dual union all
select 36142,'CASC' from dual union all
select 36143,'CASC' from dual union all
select 36144,'CASC' from dual union all
select 36145,'CASC' from dual union all
select 36146,'CASC' from dual union all
select 36147,'CASC' from dual union all
select 36148,'CASC' from dual union all
select 36149,'CASC' from dual union all
select 3615,'ESSA' from dual union all
select 36150,'CASC' from dual union all
select 36151,'CASC' from dual union all
select 36152,'CASC' from dual union all
select 36153,'CASC' from dual union all
select 36154,'CASC' from dual union all
select 36155,'CASC' from dual union all
select 36156,'CASC' from dual union all
select 36157,'CASC' from dual union all
select 36158,'CASC' from dual union all
select 36159,'CASC' from dual union all
select 3616,'NASA' from dual union all
select 36160,'CASC' from dual union all
select 36161,'CASC' from dual union all
select 36162,'CASC' from dual union all
select 36163,'CASC' from dual union all
select 36164,'CASC' from dual union all
select 36165,'CASC' from dual union all
select 36166,'CASC' from dual union all
select 36167,'CASC' from dual union all
select 36168,'CASC' from dual union all
select 36169,'CASC' from dual union all
select 3617,'USAF' from dual union all
select 36170,'CASC' from dual union all
select 36171,'CASC' from dual union all
select 36172,'CASC' from dual union all
select 36173,'CASC' from dual union all
select 36174,'CASC' from dual union all
select 36175,'CASC' from dual union all
select 36176,'CASC' from dual union all
select 36177,'CASC' from dual union all
select 36178,'CASC' from dual union all
select 36179,'CASC' from dual union all
select 3618,'USAF' from dual union all
select 36180,'CASC' from dual union all
select 36181,'CASC' from dual union all
select 36182,'CASC' from dual union all
select 36183,'CASC' from dual union all
select 36184,'CASC' from dual union all
select 36185,'CASC' from dual union all
select 36186,'CASC' from dual union all
select 36187,'CASC' from dual union all
select 36188,'CASC' from dual union all
select 36189,'CASC' from dual union all
select 3619,'MO' from dual;
insert into SATELLITE_ORG
select 36190,'CASC' from dual union all
select 36191,'CASC' from dual union all
select 36192,'CASC' from dual union all
select 36193,'CASC' from dual union all
select 36194,'CASC' from dual union all
select 36195,'CASC' from dual union all
select 36196,'CASC' from dual union all
select 36197,'CASC' from dual union all
select 36198,'CASC' from dual union all
select 36199,'CASC' from dual union all
select 362,'USAF' from dual union all
select 3620,'RVSN' from dual union all
select 36200,'CASC' from dual union all
select 36201,'CASC' from dual union all
select 36202,'CASC' from dual union all
select 36203,'CASC' from dual union all
select 36204,'CASC' from dual union all
select 36205,'CASC' from dual union all
select 36206,'CASC' from dual union all
select 36207,'CASC' from dual union all
select 36208,'CASC' from dual union all
select 36209,'CASC' from dual union all
select 3621,'RVSN' from dual union all
select 36210,'CASC' from dual union all
select 36211,'CASC' from dual union all
select 36212,'CASC' from dual union all
select 36213,'CASC' from dual union all
select 36214,'CASC' from dual union all
select 36215,'CASC' from dual union all
select 36216,'CASC' from dual union all
select 36217,'CASC' from dual union all
select 36218,'CASC' from dual union all
select 36219,'CASC' from dual union all
select 3622,'RVSN' from dual union all
select 36220,'CASC' from dual union all
select 36221,'CASC' from dual union all
select 36222,'CASC' from dual union all
select 36223,'CASC' from dual union all
select 36224,'CASC' from dual union all
select 36225,'CASC' from dual union all
select 36226,'CASC' from dual union all
select 36227,'CASC' from dual union all
select 36228,'CASC' from dual union all
select 36229,'CASC' from dual union all
select 3623,'INTEL' from dual union all
select 36230,'CASC' from dual union all
select 36231,'CASC' from dual union all
select 36232,'CASC' from dual union all
select 36233,'CASC' from dual union all
select 36234,'CASC' from dual union all
select 36235,'CASC' from dual union all
select 36236,'CASC' from dual union all
select 36237,'CASC' from dual union all
select 36238,'CASC' from dual union all
select 36239,'CASC' from dual union all
select 3624,'YUZH' from dual union all
select 36240,'CASC' from dual union all
select 36241,'CASC' from dual union all
select 36242,'CASC' from dual union all
select 36243,'CASC' from dual union all
select 36244,'CASC' from dual union all
select 36245,'CASC' from dual union all
select 36246,'CASC' from dual union all
select 36247,'CASC' from dual union all
select 36248,'CASC' from dual union all
select 36249,'CASC' from dual union all
select 3625,'RVSN' from dual union all
select 36250,'CASC' from dual union all
select 36251,'CASC' from dual union all
select 36252,'CASC' from dual union all
select 36253,'CASC' from dual union all
select 36254,'CASC' from dual union all
select 36255,'CASC' from dual union all
select 36256,'CASC' from dual union all
select 36257,'CASC' from dual union all
select 36258,'CASC' from dual union all
select 36259,'CASC' from dual union all
select 3626,'NASA' from dual union all
select 36260,'CASC' from dual union all
select 36261,'CASC' from dual union all
select 36262,'CASC' from dual union all
select 36263,'CASC' from dual union all
select 36264,'CASC' from dual union all
select 36265,'CASC' from dual union all
select 36266,'CASC' from dual union all
select 36267,'CASC' from dual union all
select 36268,'CASC' from dual union all
select 36269,'CASC' from dual union all
select 3627,'NASA' from dual union all
select 36270,'CASC' from dual union all
select 36271,'CASC' from dual union all
select 36272,'CASC' from dual union all
select 36273,'CASC' from dual union all
select 36274,'CASC' from dual union all
select 36275,'CASC' from dual union all
select 36276,'CASC' from dual union all
select 36277,'CASC' from dual union all
select 36278,'CASC' from dual union all
select 36279,'CASC' from dual union all
select 3628,'MO' from dual;
insert into SATELLITE_ORG
select 36280,'CASC' from dual union all
select 36281,'CASC' from dual union all
select 36282,'CASC' from dual union all
select 36283,'CASC' from dual union all
select 36284,'CASC' from dual union all
select 36285,'FKA' from dual union all
select 36286,'FKA' from dual union all
select 36287,'CNSA' from dual union all
select 36288,'CASC' from dual union all
select 36289,'CASC' from dual union all
select 3629,'YUZH' from dual union all
select 36290,'CASC' from dual union all
select 36291,'CASC' from dual union all
select 36292,'CASC' from dual union all
select 36293,'ISRO' from dual union all
select 36294,'KVR' from dual union all
select 36295,'NASA' from dual union all
select 36296,'USAF' from dual union all
select 36297,'RVSNR' from dual union all
select 36298,'USAF' from dual union all
select 36299,'VMFR' from dual union all
select 363,'OKB1' from dual union all
select 3630,'RVSN' from dual union all
select 36300,'PVO' from dual union all
select 36301,'CRC' from dual union all
select 36302,'KVR' from dual union all
select 36303,'CNSA' from dual union all
select 36304,'CNSA' from dual union all
select 36305,'KVR' from dual union all
select 36306,'KVR' from dual union all
select 36307,'KVR' from dual union all
select 36308,'KVR' from dual union all
select 36309,'KVR' from dual union all
select 3631,'RVSN' from dual union all
select 36310,'JAXA' from dual union all
select 36311,'JAXA' from dual union all
select 36312,'JAXA' from dual union all
select 36313,'JAXA' from dual union all
select 36314,'JAXA' from dual union all
select 36315,'JAXA' from dual union all
select 36316,'JAXA' from dual union all
select 36317,'CNSA' from dual union all
select 36318,'RVSN' from dual union all
select 36319,'RVSN' from dual union all
select 3632,'RVSN' from dual union all
select 36320,'RVSN' from dual union all
select 36321,'RVSN' from dual union all
select 36322,'RVSN' from dual union all
select 36323,'RVSN' from dual union all
select 36324,'RVSN' from dual union all
select 36325,'RVSN' from dual union all
select 36326,'RVSN' from dual union all
select 36327,'RVSN' from dual union all
select 36328,'RVSN' from dual union all
select 36329,'RVSN' from dual union all
select 3633,'RVSN' from dual union all
select 36330,'RVSN' from dual union all
select 36331,'RVSN' from dual union all
select 36332,'RVSN' from dual union all
select 36333,'RVSN' from dual union all
select 36334,'RVSN' from dual union all
select 36335,'RVSN' from dual union all
select 36336,'RVSN' from dual union all
select 36337,'RVSN' from dual union all
select 36338,'RVSN' from dual union all
select 36339,'RVSN' from dual union all
select 3634,'RVSN' from dual union all
select 36340,'RVSN' from dual union all
select 36341,'RVSN' from dual union all
select 36342,'RVSN' from dual union all
select 36343,'RVSN' from dual union all
select 36344,'RVSN' from dual union all
select 36345,'RVSN' from dual union all
select 36346,'RVSN' from dual union all
select 36347,'RVSN' from dual union all
select 36348,'RVSN' from dual union all
select 36349,'RVSN' from dual union all
select 3635,'RVSN' from dual union all
select 36350,'RVSN' from dual union all
select 36351,'RVSN' from dual union all
select 36352,'RVSN' from dual union all
select 36353,'RVSN' from dual union all
select 36354,'RVSN' from dual union all
select 36355,'RVSN' from dual union all
select 36356,'RVSN' from dual union all
select 36357,'RVSN' from dual union all
select 36358,'KVR' from dual union all
select 36359,'KHRR' from dual union all
select 3636,'RVSN' from dual union all
select 36360,'KHRR' from dual union all
select 36361,'FKA' from dual union all
select 36362,'KVR' from dual union all
select 36363,'CNSA' from dual union all
select 36364,'CNSA' from dual union all
select 36365,'CNSA' from dual union all
select 36366,'KVR' from dual union all
select 36367,'KVR' from dual union all
select 36368,'KVR' from dual union all
select 36369,'KVR' from dual union all
select 3637,'RVSN' from dual;
insert into SATELLITE_ORG
select 36370,'KVR' from dual union all
select 36371,'KVR' from dual union all
select 36372,'KVR' from dual union all
select 36373,'KVR' from dual union all
select 36374,'KVR' from dual union all
select 36375,'KVR' from dual union all
select 36376,'KVR' from dual union all
select 36377,'KVR' from dual union all
select 36378,'KVR' from dual union all
select 36379,'KVR' from dual union all
select 3638,'RVSN' from dual union all
select 36380,'KVR' from dual union all
select 36381,'KVR' from dual union all
select 36382,'KVR' from dual union all
select 36383,'KVR' from dual union all
select 36384,'KVR' from dual union all
select 36385,'KVR' from dual union all
select 36386,'KVR' from dual union all
select 36387,'KVR' from dual union all
select 36388,'KVR' from dual union all
select 36389,'IRIDS' from dual union all
select 3639,'RVSN' from dual union all
select 36390,'IRIDS' from dual union all
select 36391,'IRIDS' from dual union all
select 36392,'IRIDS' from dual union all
select 36393,'IRIDS' from dual union all
select 36394,'NASA' from dual union all
select 36395,'NASA' from dual union all
select 36396,'LM' from dual union all
select 36397,'INTELS' from dual union all
select 36398,'KHRR' from dual union all
select 36399,'KHRR' from dual union all
select 364,'RVSN' from dual union all
select 3640,'RVSN' from dual union all
select 36400,'FKA' from dual union all
select 36400,'KVR' from dual union all
select 36401,'FKA' from dual union all
select 36401,'KVR' from dual union all
select 36402,'FKA' from dual union all
select 36402,'KVR' from dual union all
select 36403,'KHRR' from dual union all
select 36404,'KHRR' from dual union all
select 36405,'KHRR' from dual union all
select 36406,'KVR' from dual union all
select 36407,'KVR' from dual union all
select 36408,'KVR' from dual union all
select 36409,'KVR' from dual union all
select 3641,'RVSN' from dual union all
select 36410,'KVR' from dual union all
select 36411,'NOAA' from dual union all
select 36412,'BOE' from dual union all
select 36413,'PLA' from dual union all
select 36414,'PLA' from dual union all
select 36415,'PLA' from dual union all
select 36416,'CNSA' from dual union all
select 36417,'CNSA' from dual union all
select 36418,'CNSA' from dual union all
select 36419,'KVR' from dual union all
select 3642,'IKI' from dual union all
select 3642,'NPOL' from dual union all
select 36420,'KVR' from dual union all
select 36421,'KVR' from dual union all
select 36422,'KVR' from dual union all
select 36423,'KVR' from dual union all
select 36424,'KVR' from dual union all
select 36425,'KVR' from dual union all
select 36426,'KVR' from dual union all
select 36427,'KVR' from dual union all
select 36428,'KVR' from dual union all
select 36429,'KVR' from dual union all
select 3643,'RVSN' from dual union all
select 36430,'KVR' from dual union all
select 36431,'KVR' from dual union all
select 36432,'KVR' from dual union all
select 36433,'KVR' from dual union all
select 36434,'KVR' from dual union all
select 36435,'KVR' from dual union all
select 36436,'KVR' from dual union all
select 36437,'KVR' from dual union all
select 36438,'KVR' from dual union all
select 36439,'KVR' from dual union all
select 3644,'RVSN' from dual union all
select 36440,'KVR' from dual union all
select 36441,'KVR' from dual union all
select 36442,'KVR' from dual union all
select 36443,'KVR' from dual union all
select 36444,'KVR' from dual union all
select 36445,'KVR' from dual union all
select 36446,'KVR' from dual union all
select 36447,'KVR' from dual union all
select 36448,'KVR' from dual union all
select 36449,'KVR' from dual union all
select 3645,'RVSN' from dual union all
select 36450,'KVR' from dual union all
select 36451,'KVR' from dual union all
select 36452,'KVR' from dual union all
select 36453,'KVR' from dual union all
select 36454,'KVR' from dual union all
select 36455,'KVR' from dual union all
select 36456,'KVR' from dual;
insert into SATELLITE_ORG
select 36457,'KVR' from dual union all
select 36458,'KVR' from dual union all
select 36459,'KVR' from dual union all
select 3646,'RVSN' from dual union all
select 36460,'KVR' from dual union all
select 36461,'KVR' from dual union all
select 36462,'KVR' from dual union all
select 36463,'KVR' from dual union all
select 36464,'KVR' from dual union all
select 36465,'KVR' from dual union all
select 36466,'KVR' from dual union all
select 36467,'KVR' from dual union all
select 36468,'KVR' from dual union all
select 36469,'KVR' from dual union all
select 3647,'RVSN' from dual union all
select 36470,'KVR' from dual union all
select 36471,'KVR' from dual union all
select 36472,'KVR' from dual union all
select 36473,'KVR' from dual union all
select 36474,'KVR' from dual union all
select 36475,'KVR' from dual union all
select 36476,'KVR' from dual union all
select 36477,'KVR' from dual union all
select 36478,'KVR' from dual union all
select 36479,'KVR' from dual union all
select 3648,'IKI' from dual union all
select 3648,'NPOL' from dual union all
select 36480,'KVR' from dual union all
select 36481,'KVR' from dual union all
select 36482,'IRIDS' from dual union all
select 36483,'IRIDS' from dual union all
select 36484,'IRIDS' from dual union all
select 36485,'IRIDS' from dual union all
select 36486,'IRIDS' from dual union all
select 36487,'IRIDS' from dual union all
select 36488,'IRIDS' from dual union all
select 36489,'IRIDS' from dual union all
select 3649,'RVSN' from dual union all
select 36490,'IRIDS' from dual union all
select 36491,'IRIDS' from dual union all
select 36492,'IRIDS' from dual union all
select 36493,'IRIDS' from dual union all
select 36494,'IRIDS' from dual union all
select 36495,'IRIDS' from dual union all
select 36496,'IRIDS' from dual union all
select 36497,'IRIDS' from dual union all
select 36498,'IRIDS' from dual union all
select 36499,'DISH' from dual union all
select 365,'OKB1' from dual union all
select 3650,'RVSN' from dual union all
select 36500,'KHRR' from dual union all
select 36501,'KHRR' from dual union all
select 36502,'CNSA' from dual union all
select 36503,'CNSA' from dual union all
select 36504,'CNSA' from dual union all
select 36505,'FKA' from dual union all
select 36506,'KVR' from dual union all
select 36507,'NASA' from dual union all
select 36508,'ESA' from dual union all
select 36509,'KTRAS' from dual union all
select 3651,'MO' from dual union all
select 36510,'KTRAS' from dual union all
select 36511,'KVR' from dual union all
select 36512,'KVR' from dual union all
select 36513,'KVR' from dual union all
select 36514,'USAF' from dual union all
select 36515,'LM' from dual union all
select 36516,'SES' from dual union all
select 36517,'KHRR' from dual union all
select 36518,'KHRR' from dual union all
select 36519,'VMFR' from dual union all
select 3652,'RVSN' from dual union all
select 36520,'KVR' from dual union all
select 36521,'FKA' from dual union all
select 36522,'KVR' from dual union all
select 36523,'KVR' from dual union all
select 36524,'KVR' from dual union all
select 36525,'KVR' from dual union all
select 36526,'KVR' from dual union all
select 36527,'KVR' from dual union all
select 36528,'KVR' from dual union all
select 36529,'KVR' from dual union all
select 3653,'MO' from dual union all
select 36530,'KVR' from dual union all
select 36531,'KVR' from dual union all
select 36532,'KVR' from dual union all
select 36533,'KVR' from dual union all
select 36534,'KVR' from dual union all
select 36535,'KVR' from dual union all
select 36536,'KVR' from dual union all
select 36537,'KVR' from dual union all
select 36538,'KVR' from dual union all
select 36539,'KVR' from dual union all
select 3654,'TSKBEM' from dual union all
select 36540,'KVR' from dual union all
select 36541,'KVR' from dual union all
select 36542,'KVR' from dual union all
select 36543,'KVR' from dual union all
select 36544,'KVR' from dual union all
select 36545,'KVR' from dual;
insert into SATELLITE_ORG
select 36546,'KVR' from dual union all
select 36547,'KVR' from dual union all
select 36548,'KVR' from dual union all
select 36549,'KVR' from dual union all
select 3655,'RVSN' from dual union all
select 36550,'KVR' from dual union all
select 36551,'KVR' from dual union all
select 36552,'KVR' from dual union all
select 36553,'KVR' from dual union all
select 36554,'KVR' from dual union all
select 36555,'KVR' from dual union all
select 36556,'KVR' from dual union all
select 36557,'KVR' from dual union all
select 36558,'KVR' from dual union all
select 36559,'KVR' from dual union all
select 3656,'TSKBEM' from dual union all
select 36560,'KVR' from dual union all
select 36561,'KVR' from dual union all
select 36562,'IRIDS' from dual union all
select 36563,'IRIDS' from dual union all
select 36564,'IRIDS' from dual union all
select 36565,'IRIDS' from dual union all
select 36566,'IRIDS' from dual union all
select 36567,'IRIDS' from dual union all
select 36568,'IRIDS' from dual union all
select 36569,'IRIDS' from dual union all
select 3657,'RVSN' from dual union all
select 36570,'IRIDS' from dual union all
select 36571,'JAXA' from dual union all
select 36572,'NASA' from dual union all
select 36573,'KAGO' from dual union all
select 36574,'WASEDA' from dual union all
select 36575,'SOKA' from dual union all
select 36576,'JAXA' from dual union all
select 36577,'JAXA' from dual union all
select 36578,'UNIS' from dual union all
select 36579,'JAXA' from dual union all
select 3658,'TSKBEM' from dual union all
select 36580,'JAXA' from dual union all
select 36581,'SES' from dual union all
select 36582,'BWEHR' from dual union all
select 36583,'AE' from dual union all
select 36584,'AE' from dual union all
select 36585,'USAF' from dual union all
select 36586,'BOE' from dual union all
select 36587,'KHRR' from dual union all
select 36588,'USEF' from dual union all
select 36589,'EUROK' from dual union all
select 3659,'MO' from dual union all
select 36590,'CNSA' from dual union all
select 36591,'CASC' from dual union all
select 36592,'ARAB' from dual union all
select 36593,'KHRR' from dual union all
select 36594,'KHRR' from dual union all
select 36595,'SPX' from dual union all
select 36596,'CASC' from dual union all
select 36597,'CASC' from dual union all
select 36598,'CNES' from dual union all
select 36599,'SSC' from dual union all
select 366,'RVSN' from dual union all
select 3660,'MO' from dual union all
select 36600,'KTRAS' from dual union all
select 36601,'KTRAS' from dual union all
select 36602,'KTRAS' from dual union all
select 36603,'FKA' from dual union all
select 36604,'KVR' from dual union all
select 36605,'DLR' from dual union all
select 36605,'EADS' from dual union all
select 36606,'KTRAS' from dual union all
select 36607,'KTRAS' from dual union all
select 36608,'ISA' from dual union all
select 36609,'ISA' from dual union all
select 3661,'MO' from dual union all
select 36610,'KVR' from dual union all
select 36611,'KVR' from dual union all
select 36612,'KVR' from dual union all
select 36613,'KVR' from dual union all
select 36614,'KVR' from dual union all
select 36615,'KVR' from dual union all
select 36616,'KVR' from dual union all
select 36617,'KVR' from dual union all
select 36618,'KVR' from dual union all
select 36619,'KVR' from dual union all
select 3662,'MO' from dual union all
select 36620,'KVR' from dual union all
select 36621,'KVR' from dual union all
select 36622,'KVR' from dual union all
select 36623,'KVR' from dual union all
select 36624,'KVR' from dual union all
select 36625,'KVR' from dual union all
select 36626,'KVR' from dual union all
select 36627,'KVR' from dual union all
select 36628,'KVR' from dual union all
select 36629,'KVR' from dual union all
select 3663,'NASA' from dual union all
select 36630,'KVR' from dual union all
select 36631,'KVR' from dual union all
select 36632,'KVR' from dual union all
select 36633,'KVR' from dual union all
select 36634,'KVR' from dual;
insert into SATELLITE_ORG
select 36635,'KVR' from dual union all
select 36636,'KVR' from dual union all
select 36637,'KVR' from dual union all
select 36638,'KVR' from dual union all
select 36639,'KVR' from dual union all
select 3664,'NASA' from dual union all
select 36640,'IRIDS' from dual union all
select 36641,'IRIDS' from dual union all
select 36642,'IRIDS' from dual union all
select 36643,'IRIDS' from dual union all
select 36644,'CASC' from dual union all
select 36645,'CASC' from dual union all
select 36646,'CASC' from dual union all
select 36647,'CASC' from dual union all
select 36648,'CASC' from dual union all
select 36649,'CASC' from dual union all
select 3665,'NRO' from dual union all
select 36650,'CASC' from dual union all
select 36651,'CASC' from dual union all
select 36652,'CASC' from dual union all
select 36653,'CASC' from dual union all
select 36654,'CASC' from dual union all
select 36655,'CASC' from dual union all
select 36656,'CASC' from dual union all
select 36657,'CASC' from dual union all
select 36658,'CASC' from dual union all
select 36659,'CASC' from dual union all
select 3666,'USAF' from dual union all
select 36660,'CASC' from dual union all
select 36661,'CASC' from dual union all
select 36662,'CASC' from dual union all
select 36663,'CASC' from dual union all
select 36664,'CASC' from dual union all
select 36665,'CASC' from dual union all
select 36666,'CASC' from dual union all
select 36667,'CASC' from dual union all
select 36668,'CASC' from dual union all
select 36669,'CASC' from dual union all
select 3667,'TSKBEM' from dual union all
select 36670,'CASC' from dual union all
select 36671,'CASC' from dual union all
select 36672,'CASC' from dual union all
select 36673,'CASC' from dual union all
select 36674,'CASC' from dual union all
select 36675,'CASC' from dual union all
select 36676,'CASC' from dual union all
select 36677,'CASC' from dual union all
select 36678,'CASC' from dual union all
select 36679,'CASC' from dual union all
select 3668,'RVSN' from dual union all
select 36680,'CASC' from dual union all
select 36681,'CASC' from dual union all
select 36682,'CASC' from dual union all
select 36683,'CASC' from dual union all
select 36684,'CASC' from dual union all
select 36685,'CASC' from dual union all
select 36686,'CASC' from dual union all
select 36687,'CASC' from dual union all
select 36688,'CASC' from dual union all
select 36689,'CASC' from dual union all
select 3669,'CRC' from dual union all
select 36690,'CASC' from dual union all
select 36691,'CASC' from dual union all
select 36692,'CASC' from dual union all
select 36693,'CASC' from dual union all
select 36694,'CASC' from dual union all
select 36695,'CASC' from dual union all
select 36696,'CASC' from dual union all
select 36697,'CASC' from dual union all
select 36698,'CASC' from dual union all
select 36699,'CASC' from dual union all
select 367,'OKB586' from dual union all
select 3670,'NASA' from dual union all
select 36700,'CASC' from dual union all
select 36701,'CASC' from dual union all
select 36702,'CASC' from dual union all
select 36703,'CASC' from dual union all
select 36704,'CASC' from dual union all
select 36705,'CASC' from dual union all
select 36706,'CASC' from dual union all
select 36707,'CASC' from dual union all
select 36708,'CASC' from dual union all
select 36709,'CASC' from dual union all
select 3671,'MO' from dual union all
select 36710,'CASC' from dual union all
select 36711,'CASC' from dual union all
select 36712,'CASC' from dual union all
select 36713,'CASC' from dual union all
select 36714,'CASC' from dual union all
select 36715,'CASC' from dual union all
select 36716,'CASC' from dual union all
select 36717,'CASC' from dual union all
select 36718,'CASC' from dual union all
select 36719,'CASC' from dual union all
select 3672,'CIA' from dual union all
select 3672,'NRO' from dual union all
select 36720,'CASC' from dual union all
select 36721,'CASC' from dual union all
select 36722,'CASC' from dual union all
select 36723,'CASC' from dual;
insert into SATELLITE_ORG
select 36724,'CASC' from dual union all
select 36725,'CASC' from dual union all
select 36726,'CASC' from dual union all
select 36727,'CASC' from dual union all
select 36728,'CASC' from dual union all
select 36729,'CASC' from dual union all
select 3673,'NRO' from dual union all
select 36730,'CASC' from dual union all
select 36731,'CASC' from dual union all
select 36732,'CASC' from dual union all
select 36733,'CASC' from dual union all
select 36734,'CASC' from dual union all
select 36735,'CASC' from dual union all
select 36736,'CASC' from dual union all
select 36737,'CASC' from dual union all
select 36738,'CASC' from dual union all
select 36739,'CASC' from dual union all
select 3674,'INTEL' from dual union all
select 36740,'CASC' from dual union all
select 36741,'CASC' from dual union all
select 36742,'CASC' from dual union all
select 36743,'CASC' from dual union all
select 36744,'KARI' from dual union all
select 36745,'ARAB' from dual union all
select 36746,'AE' from dual union all
select 36747,'AE' from dual union all
select 36748,'FKA' from dual union all
select 36749,'KVR' from dual union all
select 3675,'PVO' from dual union all
select 36750,'KHRR' from dual union all
select 36751,'KHRR' from dual union all
select 36752,'KHRR' from dual union all
select 36753,'KHRR' from dual union all
select 36754,'KHRR' from dual union all
select 36755,'KHRR' from dual union all
select 36756,'KHRR' from dual union all
select 36757,'KHRR' from dual union all
select 36758,'KHRR' from dual union all
select 36759,'KHRR' from dual union all
select 3676,'RVSN' from dual union all
select 36760,'KHRR' from dual union all
select 36761,'KHRR' from dual union all
select 36762,'KHRR' from dual union all
select 36763,'KHRR' from dual union all
select 36764,'KHRR' from dual union all
select 36765,'KHRR' from dual union all
select 36766,'KHRR' from dual union all
select 36767,'KHRR' from dual union all
select 36768,'KHRR' from dual union all
select 36769,'KHRR' from dual union all
select 3677,'PVO' from dual union all
select 36770,'KHRR' from dual union all
select 36771,'KHRR' from dual union all
select 36772,'KHRR' from dual union all
select 36773,'KHRR' from dual union all
select 36774,'KHRR' from dual union all
select 36775,'KHRR' from dual union all
select 36776,'KHRR' from dual union all
select 36777,'KHRR' from dual union all
select 36778,'KHRR' from dual union all
select 36779,'KHRR' from dual union all
select 3678,'PVO' from dual union all
select 36780,'KHRR' from dual union all
select 36781,'KHRR' from dual union all
select 36782,'KHRR' from dual union all
select 36783,'KHRR' from dual union all
select 36784,'KHRR' from dual union all
select 36785,'KHRR' from dual union all
select 36786,'KHRR' from dual union all
select 36787,'KHRR' from dual union all
select 36788,'KHRR' from dual union all
select 36789,'KHRR' from dual union all
select 3679,'PVO' from dual union all
select 36790,'KHRR' from dual union all
select 36791,'KHRR' from dual union all
select 36792,'DISH' from dual union all
select 36793,'KHRR' from dual union all
select 36794,'KHRR' from dual union all
select 36795,'ISRO' from dual union all
select 36796,'NMIT' from dual union all
select 36797,'NSC' from dual union all
select 36798,'ASAL' from dual union all
select 36799,'SUPSI' from dual union all
select 368,'RVSN' from dual union all
select 3680,'PVO' from dual union all
select 36800,'ISRO' from dual union all
select 36801,'ISRO' from dual union all
select 36802,'KHRR' from dual union all
select 36803,'KHRR' from dual union all
select 36804,'KHRR' from dual union all
select 36805,'KHRR' from dual union all
select 36806,'KHRR' from dual union all
select 36807,'KHRR' from dual union all
select 36808,'KHRR' from dual union all
select 36809,'KHRR' from dual union all
select 3681,'PVO' from dual union all
select 36810,'KHRR' from dual union all
select 36811,'KHRR' from dual union all
select 36812,'KHRR' from dual union all
select 36813,'KHRR' from dual;
insert into SATELLITE_ORG
select 36814,'KHRR' from dual union all
select 36815,'KHRR' from dual union all
select 36816,'KHRR' from dual union all
select 36817,'KHRR' from dual union all
select 36818,'KHRR' from dual union all
select 36819,'KHRR' from dual union all
select 3682,'PVO' from dual union all
select 36820,'KHRR' from dual union all
select 36821,'KVR' from dual union all
select 36822,'KVR' from dual union all
select 36823,'KVR' from dual union all
select 36824,'FKA' from dual union all
select 36825,'FKA' from dual union all
select 36826,'FKA' from dual union all
select 36827,'SSC' from dual union all
select 36828,'CNSA' from dual union all
select 36829,'CASC' from dual union all
select 3683,'PVO' from dual union all
select 36830,'ERTU' from dual union all
select 36831,'I-RASC' from dual union all
select 36831,'RSQ' from dual union all
select 36832,'AE' from dual union all
select 36833,'AE' from dual union all
select 36834,'PLA' from dual union all
select 36835,'CNSA' from dual union all
select 36836,'KHRR' from dual union all
select 36837,'KHRR' from dual union all
select 36838,'KHRR' from dual union all
select 36839,'KHRR' from dual union all
select 3684,'PVO' from dual union all
select 36840,'KHRR' from dual union all
select 36841,'KHRR' from dual union all
select 36842,'KHRR' from dual union all
select 36843,'KHRR' from dual union all
select 36844,'KHRR' from dual union all
select 36845,'KHRR' from dual union all
select 36846,'KHRR' from dual union all
select 36847,'KHRR' from dual union all
select 36848,'KVR' from dual union all
select 36849,'KVR' from dual union all
select 3685,'PVO' from dual union all
select 36850,'KVR' from dual union all
select 36851,'KVR' from dual union all
select 36852,'KVR' from dual union all
select 36853,'KVR' from dual union all
select 36854,'USAF' from dual union all
select 36855,'USAF' from dual union all
select 36856,'USAF' from dual union all
select 36857,'KVR' from dual union all
select 36858,'KVR' from dual union all
select 36859,'KVR' from dual union all
select 3686,'PVO' from dual union all
select 36860,'KVR' from dual union all
select 36861,'KVR' from dual union all
select 36862,'KVR' from dual union all
select 36863,'KVR' from dual union all
select 36864,'KVR' from dual union all
select 36865,'KVR' from dual union all
select 36866,'KVR' from dual union all
select 36867,'KVR' from dual union all
select 36868,'USAF' from dual union all
select 36869,'LM' from dual union all
select 3687,'PVO' from dual union all
select 36870,'JAXA' from dual union all
select 36871,'JAXA' from dual union all
select 36872,'JAXA' from dual union all
select 36873,'JAXA' from dual union all
select 36874,'JAXA' from dual union all
select 36875,'KVR' from dual union all
select 36876,'KVR' from dual union all
select 36877,'KVR' from dual union all
select 36878,'KVR' from dual union all
select 36879,'KVR' from dual union all
select 3688,'PVO' from dual union all
select 36880,'USAF' from dual union all
select 36881,'USAF' from dual union all
select 36882,'RVSN' from dual union all
select 36883,'RVSN' from dual union all
select 36884,'KVR' from dual union all
select 36885,'KVR' from dual union all
select 36886,'KVR' from dual union all
select 36887,'KVR' from dual union all
select 36888,'KVR' from dual union all
select 36889,'KVR' from dual union all
select 3689,'PVO' from dual union all
select 36890,'KVR' from dual union all
select 36891,'KVR' from dual union all
select 36892,'KVR' from dual union all
select 36893,'KVR' from dual union all
select 36894,'KVR' from dual union all
select 36895,'KVR' from dual union all
select 36896,'KVR' from dual union all
select 36897,'KVR' from dual union all
select 36898,'KVR' from dual union all
select 36899,'KVR' from dual union all
select 369,'USAF' from dual union all
select 3690,'PVO' from dual union all
select 36900,'KVR' from dual union all
select 36901,'KVR' from dual union all
select 36902,'KVR' from dual;
insert into SATELLITE_ORG
select 36903,'KVR' from dual union all
select 36904,'KVR' from dual union all
select 36905,'KVR' from dual union all
select 36906,'KVR' from dual union all
select 36907,'KVR' from dual union all
select 36908,'KVR' from dual union all
select 36909,'KVR' from dual union all
select 3691,'USAF' from dual union all
select 36910,'KVR' from dual union all
select 36911,'KVR' from dual union all
select 36912,'KVR' from dual union all
select 36913,'KVR' from dual union all
select 36914,'KVR' from dual union all
select 36915,'KVR' from dual union all
select 36916,'KVR' from dual union all
select 36917,'KVR' from dual union all
select 36918,'KVR' from dual union all
select 36919,'KVR' from dual union all
select 3692,'USAF' from dual union all
select 36920,'KVR' from dual union all
select 36921,'KVR' from dual union all
select 36922,'KVR' from dual union all
select 36923,'KVR' from dual union all
select 36924,'KVR' from dual union all
select 36925,'KVR' from dual union all
select 36926,'KVR' from dual union all
select 36927,'KVR' from dual union all
select 36928,'KVR' from dual union all
select 36929,'KVR' from dual union all
select 3693,'PVO' from dual union all
select 36930,'KVR' from dual union all
select 36931,'KVR' from dual union all
select 36932,'KVR' from dual union all
select 36933,'KVR' from dual union all
select 36934,'KVR' from dual union all
select 36935,'KVR' from dual union all
select 36936,'KVR' from dual union all
select 36937,'KVR' from dual union all
select 36938,'KVR' from dual union all
select 36939,'KVR' from dual union all
select 3694,'PVO' from dual union all
select 36940,'KVR' from dual union all
select 36941,'KVR' from dual union all
select 36942,'KVR' from dual union all
select 36943,'KVR' from dual union all
select 36944,'KVR' from dual union all
select 36945,'KVR' from dual union all
select 36946,'KVR' from dual union all
select 36947,'KVR' from dual union all
select 36948,'KVR' from dual union all
select 36949,'KVR' from dual union all
select 3695,'PVO' from dual union all
select 36950,'KVR' from dual union all
select 36951,'KVR' from dual union all
select 36952,'KVR' from dual union all
select 36953,'KVR' from dual union all
select 36954,'KVR' from dual union all
select 36955,'KHRR' from dual union all
select 36956,'KHRR' from dual union all
select 36957,'KHRR' from dual union all
select 36958,'KHRR' from dual union all
select 36959,'KHRR' from dual union all
select 3696,'PVO' from dual union all
select 36960,'KHRR' from dual union all
select 36961,'KHRR' from dual union all
select 36962,'KHRR' from dual union all
select 36963,'KHRR' from dual union all
select 36964,'KHRR' from dual union all
select 36965,'CNSA' from dual union all
select 36966,'CNSA' from dual union all
select 36967,'CNSA' from dual union all
select 36968,'CNSA' from dual union all
select 36969,'CNSA' from dual union all
select 3697,'PVO' from dual union all
select 36970,'CNSA' from dual union all
select 36971,'CNSA' from dual union all
select 36972,'KVR' from dual union all
select 36973,'KVR' from dual union all
select 36974,'KVR' from dual union all
select 36975,'KHRR' from dual union all
select 36976,'KHRR' from dual union all
select 36977,'KHRR' from dual union all
select 36978,'KHRR' from dual union all
select 36979,'USAF' from dual union all
select 3698,'RVSN' from dual union all
select 36980,'USAF' from dual union all
select 36981,'USAF' from dual union all
select 36982,'USAF' from dual union all
select 36983,'USAF' from dual union all
select 36984,'USAF' from dual union all
select 36985,'GZB' from dual union all
select 36986,'CASC' from dual union all
select 36987,'CASC' from dual union all
select 36988,'CASC' from dual union all
select 36989,'CASC' from dual union all
select 3699,'PVO' from dual union all
select 36990,'CASC' from dual union all
select 36991,'CASC' from dual union all
select 36992,'CASC' from dual union all
select 36993,'CASC' from dual;
insert into SATELLITE_ORG
select 36994,'CASC' from dual union all
select 36995,'CASC' from dual union all
select 36996,'CASC' from dual union all
select 36997,'CASC' from dual union all
select 36998,'CASC' from dual union all
select 36999,'CASC' from dual union all
select 37,'OKB1' from dual union all
select 370,'USAF' from dual union all
select 3700,'RVSN' from dual union all
select 37000,'CASC' from dual union all
select 37001,'CASC' from dual union all
select 37002,'CASC' from dual union all
select 37003,'CASC' from dual union all
select 37004,'CASC' from dual union all
select 37005,'CASC' from dual union all
select 37006,'CASC' from dual union all
select 37007,'CASC' from dual union all
select 37008,'CASC' from dual union all
select 37009,'CASC' from dual union all
select 3701,'PVO' from dual union all
select 37010,'CASC' from dual union all
select 37011,'CASC' from dual union all
select 37012,'CASC' from dual union all
select 37013,'CASC' from dual union all
select 37014,'CASC' from dual union all
select 37015,'CASC' from dual union all
select 37016,'CASC' from dual union all
select 37017,'CASC' from dual union all
select 37018,'CASC' from dual union all
select 37019,'CASC' from dual union all
select 3702,'PVO' from dual union all
select 37020,'CASC' from dual union all
select 37021,'CASC' from dual union all
select 37022,'CASC' from dual union all
select 37023,'CASC' from dual union all
select 37024,'CASC' from dual union all
select 37025,'CASC' from dual union all
select 37026,'CASC' from dual union all
select 37027,'CASC' from dual union all
select 37028,'CASC' from dual union all
select 37029,'CASC' from dual union all
select 3703,'RVSN' from dual union all
select 37030,'CASC' from dual union all
select 37031,'CASC' from dual union all
select 37032,'CASC' from dual union all
select 37033,'CASC' from dual union all
select 37034,'CASC' from dual union all
select 37035,'CASC' from dual union all
select 37036,'CASC' from dual union all
select 37037,'CASC' from dual union all
select 37038,'CASC' from dual union all
select 37039,'CASC' from dual union all
select 3704,'PVO' from dual union all
select 37040,'CASC' from dual union all
select 37041,'CASC' from dual union all
select 37042,'CASC' from dual union all
select 37043,'CASC' from dual union all
select 37044,'CASC' from dual union all
select 37045,'CASC' from dual union all
select 37046,'CASC' from dual union all
select 37047,'CASC' from dual union all
select 37048,'CASC' from dual union all
select 37049,'CASC' from dual union all
select 3705,'RVSN' from dual union all
select 37050,'CASC' from dual union all
select 37051,'CASC' from dual union all
select 37052,'CASC' from dual union all
select 37053,'CASC' from dual union all
select 37054,'CASC' from dual union all
select 37055,'CASC' from dual union all
select 37056,'CASC' from dual union all
select 37057,'CASC' from dual union all
select 37058,'CASC' from dual union all
select 37059,'CASC' from dual union all
select 3706,'PVO' from dual union all
select 37060,'CASC' from dual union all
select 37061,'CASC' from dual union all
select 37062,'CASC' from dual union all
select 37063,'CASC' from dual union all
select 37064,'CASC' from dual union all
select 37065,'CASC' from dual union all
select 37066,'CASC' from dual union all
select 37067,'CASC' from dual union all
select 37068,'CASC' from dual union all
select 37069,'CASC' from dual union all
select 3707,'RVSN' from dual union all
select 37070,'CASC' from dual union all
select 37071,'CASC' from dual union all
select 37072,'CASC' from dual union all
select 37073,'CASC' from dual union all
select 37074,'CASC' from dual union all
select 37075,'CASC' from dual union all
select 37076,'CASC' from dual union all
select 37077,'CASC' from dual union all
select 37078,'CASC' from dual union all
select 37079,'CASC' from dual union all
select 3708,'RVSN' from dual union all
select 37080,'CASC' from dual union all
select 37081,'CASC' from dual union all
select 37082,'KHRR' from dual;
insert into SATELLITE_ORG
select 37083,'KHRR' from dual union all
select 37084,'KVR' from dual union all
select 37085,'KVR' from dual union all
select 37086,'KVR' from dual union all
select 37087,'KVR' from dual union all
select 37088,'KVR' from dual union all
select 37089,'KVR' from dual union all
select 3709,'RVSN' from dual union all
select 37090,'KVR' from dual union all
select 37091,'KVR' from dual union all
select 37092,'KVR' from dual union all
select 37093,'KVR' from dual union all
select 37094,'KVR' from dual union all
select 37095,'KVR' from dual union all
select 37096,'KVR' from dual union all
select 37097,'KVR' from dual union all
select 37098,'KVR' from dual union all
select 37099,'KVR' from dual union all
select 371,'OKB1' from dual union all
select 3710,'RVSN' from dual union all
select 37100,'KVR' from dual union all
select 37101,'KVR' from dual union all
select 37102,'KVR' from dual union all
select 37103,'KVR' from dual union all
select 37104,'KVR' from dual union all
select 37105,'KVR' from dual union all
select 37106,'KVR' from dual union all
select 37107,'KVR' from dual union all
select 37108,'KVR' from dual union all
select 37109,'KVR' from dual union all
select 3711,'PVO' from dual union all
select 37110,'KVR' from dual union all
select 37111,'KVR' from dual union all
select 37112,'KVR' from dual union all
select 37113,'KVR' from dual union all
select 37114,'KVR' from dual union all
select 37115,'KVR' from dual union all
select 37116,'KVR' from dual union all
select 37117,'KVR' from dual union all
select 37118,'KVR' from dual union all
select 37119,'KVR' from dual union all
select 3712,'PVO' from dual union all
select 37120,'KVR' from dual union all
select 37121,'KVR' from dual union all
select 37122,'KVR' from dual union all
select 37123,'KVR' from dual union all
select 37124,'KVR' from dual union all
select 37125,'KVR' from dual union all
select 37126,'KVR' from dual union all
select 37127,'KVR' from dual union all
select 37128,'KVR' from dual union all
select 37129,'KVR' from dual union all
select 3713,'PVO' from dual union all
select 37130,'KVR' from dual union all
select 37131,'KVR' from dual union all
select 37132,'KVR' from dual union all
select 37133,'KVR' from dual union all
select 37134,'IRIDS' from dual union all
select 37135,'IRIDS' from dual union all
select 37136,'IRIDS' from dual union all
select 37137,'FKA' from dual union all
select 37137,'KVR' from dual union all
select 37138,'FKA' from dual union all
select 37138,'KVR' from dual union all
select 37139,'FKA' from dual union all
select 37139,'KVR' from dual union all
select 3714,'PVO' from dual union all
select 37140,'KVR' from dual union all
select 37141,'KVR' from dual union all
select 37142,'KVR' from dual union all
select 37143,'KVR' from dual union all
select 37144,'KVR' from dual union all
select 37145,'KVR' from dual union all
select 37146,'KVR' from dual union all
select 37147,'KVR' from dual union all
select 37148,'NASA' from dual union all
select 37149,'KVR' from dual union all
select 3715,'PVO' from dual union all
select 37150,'CASC' from dual union all
select 37151,'CASC' from dual union all
select 37152,'FKA' from dual union all
select 37153,'KVR' from dual union all
select 37154,'KVR' from dual union all
select 37155,'KVR' from dual union all
select 37156,'FKA' from dual union all
select 37157,'KVR' from dual union all
select 37158,'JAXA' from dual union all
select 37158,'SPAC' from dual union all
select 37159,'JAXA' from dual union all
select 3716,'PVO' from dual union all
select 37160,'KHRR' from dual union all
select 37161,'KHRR' from dual union all
select 37162,'NROC' from dual union all
select 37163,'KHRR' from dual union all
select 37164,'KHRR' from dual union all
select 37165,'PLA' from dual union all
select 37166,'ZHEJ' from dual union all
select 37167,'ZHEJ' from dual union all
select 37168,'USAF' from dual union all
select 37169,'USAF' from dual;
insert into SATELLITE_ORG
select 3717,'PVO' from dual union all
select 37170,'PVO' from dual union all
select 37171,'RVSNR' from dual union all
select 37172,'RVSNR' from dual union all
select 37173,'RVSNR' from dual union all
select 37174,'CASC' from dual union all
select 37175,'CASC' from dual union all
select 37176,'RVSNR' from dual union all
select 37177,'KHRR' from dual union all
select 37178,'KHRR' from dual union all
select 37179,'CASC' from dual union all
select 3718,'USAF' from dual union all
select 37180,'CASC' from dual union all
select 37181,'CNSA' from dual union all
select 37182,'CNSA' from dual union all
select 37183,'FKA' from dual union all
select 37184,'KVR' from dual union all
select 37185,'SIRX' from dual union all
select 37186,'KHRR' from dual union all
select 37187,'KHRR' from dual union all
select 37188,'GLOB' from dual union all
select 37189,'GLOB' from dual union all
select 3719,'USAF' from dual union all
select 37190,'GLOB' from dual union all
select 37191,'GLOB' from dual union all
select 37192,'GLOB' from dual union all
select 37193,'GLOB' from dual union all
select 37194,'NPOL' from dual union all
select 37195,'NASA' from dual union all
select 37196,'FKA' from dual union all
select 37197,'KVR' from dual union all
select 37198,'KHRR' from dual union all
select 37199,'KHRR' from dual union all
select 372,'RVSN' from dual union all
select 3720,'USAF' from dual union all
select 37200,'KHRR' from dual union all
select 37201,'KHRR' from dual union all
select 37202,'KHRR' from dual union all
select 37203,'KHRR' from dual union all
select 37204,'KHRR' from dual union all
select 37205,'KHRR' from dual union all
select 37206,'EUTSA' from dual union all
select 37207,'BSAT' from dual union all
select 37208,'AE' from dual union all
select 37209,'AE' from dual union all
select 3721,'USAF' from dual union all
select 37210,'CNSA' from dual union all
select 37211,'CASC' from dual union all
select 37212,'KVR' from dual union all
select 37213,'NPOL' from dual union all
select 37214,'CMA' from dual union all
select 37215,'CNSA' from dual union all
select 37216,'ASI' from dual union all
select 37217,'BOE' from dual union all
select 37218,'SKYTC' from dual union all
select 37219,'KHRR' from dual union all
select 3722,'USAF' from dual union all
select 37220,'KHRR' from dual union all
select 37221,'FKA' from dual union all
select 37222,'USAF' from dual union all
select 37223,'NSF' from dual union all
select 37223,'UMI' from dual union all
select 37224,'NASA' from dual union all
select 37225,'USAF' from dual union all
select 37226,'USAF' from dual union all
select 37227,'USAF' from dual union all
select 37228,'USAF' from dual union all
select 37229,'USAF' from dual union all
select 3723,'USAF' from dual union all
select 37230,'USAF' from dual union all
select 37231,'USAF' from dual union all
select 37232,'NROC' from dual union all
select 37233,'BOE' from dual union all
select 37234,'PLA' from dual union all
select 37235,'CASC' from dual union all
select 37236,'KVR' from dual union all
select 37237,'AVANTI' from dual union all
select 37238,'INTELS' from dual union all
select 37239,'AE' from dual union all
select 3724,'USAF' from dual union all
select 37240,'AE' from dual union all
select 37241,'NOAA' from dual union all
select 37242,'NOAA' from dual union all
select 37243,'CNSA' from dual union all
select 37244,'SPX' from dual union all
select 37245,'NROC' from dual union all
select 37245,'USN' from dual union all
select 37246,'SMDC' from dual union all
select 37247,'DOD' from dual union all
select 37247,'LANL' from dual union all
select 37248,'DOD' from dual union all
select 37248,'LANL' from dual union all
select 37249,'NROC' from dual union all
select 37249,'USN' from dual union all
select 3725,'USAF' from dual union all
select 37250,'DOD' from dual union all
select 37250,'LANL' from dual union all
select 37251,'DOD' from dual union all
select 37251,'LANL' from dual union all
select 37252,'NGAS' from dual;
insert into SATELLITE_ORG
select 37252,'USCLA' from dual union all
select 37253,'SPX' from dual union all
select 37254,'FKA' from dual union all
select 37255,'KVR' from dual union all
select 37256,'CNSA' from dual union all
select 37257,'CASC' from dual union all
select 37258,'EUTSA' from dual union all
select 37259,'KHRR' from dual union all
select 3726,'USAF' from dual union all
select 37260,'KHRR' from dual union all
select 37261,'JAXA' from dual union all
select 37262,'JAXA' from dual union all
select 37263,'JAXA' from dual union all
select 37264,'HISP' from dual union all
select 37265,'KTEL' from dual union all
select 37266,'AE' from dual union all
select 37267,'AE' from dual union all
select 37268,'KVR' from dual union all
select 37269,'KVR' from dual union all
select 3727,'USAF' from dual union all
select 37270,'KVR' from dual union all
select 37271,'KVR' from dual union all
select 37272,'KVR' from dual union all
select 37273,'KVR' from dual union all
select 37274,'KVR' from dual union all
select 37275,'KVR' from dual union all
select 37276,'KVR' from dual union all
select 37277,'KVR' from dual union all
select 37278,'KVR' from dual union all
select 37279,'KVR' from dual union all
select 3728,'USAF' from dual union all
select 37280,'KVR' from dual union all
select 37281,'KVR' from dual union all
select 37282,'KVR' from dual union all
select 37283,'KVR' from dual union all
select 37284,'KVR' from dual union all
select 37285,'KVR' from dual union all
select 37286,'KVR' from dual union all
select 37287,'KVR' from dual union all
select 37288,'KVR' from dual union all
select 37289,'KVR' from dual union all
select 3729,'USAF' from dual union all
select 37290,'KVR' from dual union all
select 37291,'KVR' from dual union all
select 37292,'KVR' from dual union all
select 37293,'KVR' from dual union all
select 37294,'KVR' from dual union all
select 37295,'KVR' from dual union all
select 37296,'KVR' from dual union all
select 37297,'KVR' from dual union all
select 37298,'KVR' from dual union all
select 37299,'KVR' from dual union all
select 373,'RVSN' from dual union all
select 3730,'USAF' from dual union all
select 37300,'KVR' from dual union all
select 37301,'KVR' from dual union all
select 37302,'KVR' from dual union all
select 37303,'KVR' from dual union all
select 37304,'KVR' from dual union all
select 37305,'KVR' from dual union all
select 37306,'KVR' from dual union all
select 37307,'KVR' from dual union all
select 37308,'KVR' from dual union all
select 37309,'KVR' from dual union all
select 3731,'USAF' from dual union all
select 37310,'KVR' from dual union all
select 37311,'KVR' from dual union all
select 37312,'KVR' from dual union all
select 37313,'KVR' from dual union all
select 37314,'KVR' from dual union all
select 37315,'KVR' from dual union all
select 37316,'KVR' from dual union all
select 37317,'KVR' from dual union all
select 37318,'KVR' from dual union all
select 37319,'KVR' from dual union all
select 3732,'USAF' from dual union all
select 37320,'KVR' from dual union all
select 37321,'KVR' from dual union all
select 37322,'KVR' from dual union all
select 37323,'KVR' from dual union all
select 37324,'KVR' from dual union all
select 37325,'KVR' from dual union all
select 37326,'KVR' from dual union all
select 37327,'KVR' from dual union all
select 37328,'KVR' from dual union all
select 37329,'KVR' from dual union all
select 3733,'USAF' from dual union all
select 37330,'KVR' from dual union all
select 37331,'KVR' from dual union all
select 37332,'KVR' from dual union all
select 37333,'KVR' from dual union all
select 37334,'KVR' from dual union all
select 37335,'KVR' from dual union all
select 37336,'KVR' from dual union all
select 37337,'KVR' from dual union all
select 37338,'KVR' from dual union all
select 37339,'KVR' from dual union all
select 3734,'USAF' from dual union all
select 37340,'KVR' from dual union all
select 37341,'KVR' from dual;
insert into SATELLITE_ORG
select 37342,'KVR' from dual union all
select 37343,'KVR' from dual union all
select 37344,'NPOL' from dual union all
select 37344,'RGMS' from dual union all
select 37345,'NPOL' from dual union all
select 37346,'KVR' from dual union all
select 37347,'NPOL' from dual union all
select 37348,'NROC' from dual union all
select 37349,'FKA' from dual union all
select 3735,'USAF' from dual union all
select 37350,'FKA' from dual union all
select 37351,'JAXA' from dual union all
select 37352,'FKA' from dual union all
select 37353,'FKA' from dual union all
select 37354,'KVR' from dual union all
select 37355,'KVR' from dual union all
select 37356,'KVR' from dual union all
select 37357,'KVR' from dual union all
select 37358,'USAF' from dual union all
select 37359,'FKA' from dual union all
select 3736,'USAF' from dual union all
select 37360,'KVR' from dual union all
select 37361,'NASA' from dual union all
select 37362,'TSVS' from dual union all
select 37363,'KVR' from dual union all
select 37364,'NROC' from dual union all
select 37365,'OSC' from dual union all
select 37366,'NROC' from dual union all
select 37367,'NROC' from dual union all
select 37368,'ESA' from dual union all
select 37369,'AE' from dual union all
select 3737,'MO' from dual union all
select 37370,'FKA' from dual union all
select 37371,'NASA' from dual union all
select 37372,'FKA' from dual union all
select 37372,'KVR' from dual union all
select 37373,'NPOL' from dual union all
select 37374,'NASA' from dual union all
select 37375,'USAF' from dual union all
select 37376,'CASC' from dual union all
select 37377,'NROC' from dual union all
select 37378,'BOE' from dual union all
select 37379,'NASA' from dual union all
select 3738,'NASA' from dual union all
select 37380,'USAF' from dual union all
select 37381,'KHRR' from dual union all
select 37382,'FKA' from dual union all
select 37383,'KVR' from dual union all
select 37384,'CNSA' from dual union all
select 37385,'CASC' from dual union all
select 37386,'NROC' from dual union all
select 37386,'USN' from dual union all
select 37387,'ISRO' from dual union all
select 37388,'ISRO' from dual union all
select 37389,'NTU' from dual union all
select 3739,'NASA' from dual union all
select 37390,'ISRO' from dual union all
select 37391,'NROC' from dual union all
select 37391,'USN' from dual union all
select 37392,'INTELS' from dual union all
select 37393,'ALYAH' from dual union all
select 37394,'AE' from dual union all
select 37395,'AE' from dual union all
select 37396,'FKA' from dual union all
select 37397,'KVR' from dual union all
select 37398,'KVR' from dual union all
select 37399,'NPOL' from dual union all
select 374,'JPL' from dual union all
select 374,'NASA' from dual union all
select 3740,'USAF' from dual union all
select 37400,'CASC' from dual union all
select 37401,'CASC' from dual union all
select 37402,'CASC' from dual union all
select 37403,'CASC' from dual union all
select 37404,'CASC' from dual union all
select 37405,'CASC' from dual union all
select 37406,'CASC' from dual union all
select 37407,'CASC' from dual union all
select 37408,'CASC' from dual union all
select 37409,'CASC' from dual union all
select 3741,'NASA' from dual union all
select 37410,'CASC' from dual union all
select 37411,'CASC' from dual union all
select 37412,'CASC' from dual union all
select 37413,'CASC' from dual union all
select 37414,'CASC' from dual union all
select 37415,'CASC' from dual union all
select 37416,'CASC' from dual union all
select 37417,'CASC' from dual union all
select 37418,'CASC' from dual union all
select 37419,'CASC' from dual union all
select 3742,'CNES' from dual union all
select 37420,'CASC' from dual union all
select 37421,'CASC' from dual union all
select 37422,'CASC' from dual union all
select 37423,'CASC' from dual union all
select 37424,'CASC' from dual union all
select 37425,'CASC' from dual union all
select 37426,'CASC' from dual union all
select 37427,'CASC' from dual;
insert into SATELLITE_ORG
select 37428,'CASC' from dual union all
select 37429,'CASC' from dual union all
select 3743,'RVSN' from dual union all
select 37430,'CASC' from dual union all
select 37431,'CASC' from dual union all
select 37432,'CASC' from dual union all
select 37433,'CASC' from dual union all
select 37434,'CASC' from dual union all
select 37435,'CASC' from dual union all
select 37436,'CASC' from dual union all
select 37437,'CASC' from dual union all
select 37438,'CASC' from dual union all
select 37439,'CASC' from dual union all
select 3744,'USAF' from dual union all
select 37440,'CASC' from dual union all
select 37441,'CASC' from dual union all
select 37442,'CASC' from dual union all
select 37443,'CASC' from dual union all
select 37444,'CASC' from dual union all
select 37445,'CASC' from dual union all
select 37446,'CASC' from dual union all
select 37447,'CASC' from dual union all
select 37448,'CASC' from dual union all
select 37449,'CASC' from dual union all
select 3745,'RVSN' from dual union all
select 37450,'CASC' from dual union all
select 37451,'CASC' from dual union all
select 37452,'CASC' from dual union all
select 37453,'CASC' from dual union all
select 37454,'CASC' from dual union all
select 37455,'CASC' from dual union all
select 37456,'CASC' from dual union all
select 37457,'CASC' from dual union all
select 37458,'CASC' from dual union all
select 37459,'CASC' from dual union all
select 3746,'USAF' from dual union all
select 37460,'CASC' from dual union all
select 37461,'CASC' from dual union all
select 37462,'CASC' from dual union all
select 37463,'CASC' from dual union all
select 37464,'CASC' from dual union all
select 37465,'CASC' from dual union all
select 37466,'CASC' from dual union all
select 37467,'CASC' from dual union all
select 37468,'CASC' from dual union all
select 37469,'CASC' from dual union all
select 3747,'USAF' from dual union all
select 37470,'CASC' from dual union all
select 37471,'CASC' from dual union all
select 37472,'CASC' from dual union all
select 37473,'CASC' from dual union all
select 37474,'CASC' from dual union all
select 37475,'CASC' from dual union all
select 37476,'CASC' from dual union all
select 37477,'CASC' from dual union all
select 37478,'CASC' from dual union all
select 37479,'CASC' from dual union all
select 3748,'USAF' from dual union all
select 37480,'CASC' from dual union all
select 37481,'USAF' from dual union all
select 37482,'LM' from dual union all
select 37483,'KVR' from dual union all
select 37484,'KVR' from dual union all
select 37485,'KVR' from dual union all
select 37486,'KVR' from dual union all
select 37487,'KVR' from dual union all
select 37488,'KVR' from dual union all
select 37489,'KVR' from dual union all
select 3749,'RVSN' from dual union all
select 37490,'KVR' from dual union all
select 37491,'KVR' from dual union all
select 37492,'KVR' from dual union all
select 37493,'KVR' from dual union all
select 37494,'KVR' from dual union all
select 37495,'KVR' from dual union all
select 37496,'KVR' from dual union all
select 37497,'KVR' from dual union all
select 37498,'KVR' from dual union all
select 37499,'KVR' from dual union all
select 375,'NASA' from dual union all
select 3750,'NASA' from dual union all
select 37500,'KVR' from dual union all
select 37501,'KVR' from dual union all
select 37502,'KVR' from dual union all
select 37503,'KVR' from dual union all
select 37504,'KVR' from dual union all
select 37505,'KVR' from dual union all
select 37506,'KVR' from dual union all
select 37507,'KVR' from dual union all
select 37508,'KVR' from dual union all
select 37509,'KVR' from dual union all
select 3751,'USAF' from dual union all
select 37510,'KVR' from dual union all
select 37511,'KVR' from dual union all
select 37512,'KVR' from dual union all
select 37513,'KVR' from dual union all
select 37514,'KVR' from dual union all
select 37515,'KVR' from dual union all
select 37516,'KVR' from dual union all
select 37517,'KVR' from dual;
insert into SATELLITE_ORG
select 37518,'KVR' from dual union all
select 37519,'KVR' from dual union all
select 3752,'USAF' from dual union all
select 37520,'KVR' from dual union all
select 37521,'KVR' from dual union all
select 37522,'KVR' from dual union all
select 37523,'KVR' from dual union all
select 37524,'KVR' from dual union all
select 37525,'KVR' from dual union all
select 37526,'KVR' from dual union all
select 37527,'KVR' from dual union all
select 37528,'KVR' from dual union all
select 37529,'KVR' from dual union all
select 3753,'PVO' from dual union all
select 37530,'KVR' from dual union all
select 37531,'KVR' from dual union all
select 37532,'KVR' from dual union all
select 37533,'KVR' from dual union all
select 37534,'KVR' from dual union all
select 37535,'KVR' from dual union all
select 37536,'KVR' from dual union all
select 37537,'KVR' from dual union all
select 37538,'KVR' from dual union all
select 37539,'KVR' from dual union all
select 3754,'PVO' from dual union all
select 37540,'KVR' from dual union all
select 37541,'KVR' from dual union all
select 37542,'KVR' from dual union all
select 37543,'KVR' from dual union all
select 37544,'KVR' from dual union all
select 37545,'KVR' from dual union all
select 37546,'KVR' from dual union all
select 37547,'KVR' from dual union all
select 37548,'IRIDS' from dual union all
select 37549,'IRIDS' from dual union all
select 3755,'PVO' from dual union all
select 37550,'IRIDS' from dual union all
select 37551,'IRIDS' from dual union all
select 37552,'IRIDS' from dual union all
select 37553,'IRIDS' from dual union all
select 37554,'IRIDS' from dual union all
select 37555,'IRIDS' from dual union all
select 37556,'IRIDS' from dual union all
select 37557,'IRIDS' from dual union all
select 37558,'IRIDS' from dual union all
select 37559,'IRIDS' from dual union all
select 3756,'USAF' from dual union all
select 37560,'IRIDS' from dual union all
select 37561,'IRIDS' from dual union all
select 37562,'IRIDS' from dual union all
select 37563,'IRIDS' from dual union all
select 37564,'IRIDS' from dual union all
select 37565,'IRIDS' from dual union all
select 37566,'IRIDS' from dual union all
select 37567,'CNSA' from dual union all
select 37568,'CNSA' from dual union all
select 37569,'CNSA' from dual union all
select 3757,'PVO' from dual union all
select 37570,'CNSA' from dual union all
select 37571,'CNSA' from dual union all
select 37572,'CNSA' from dual union all
select 37573,'CNSA' from dual union all
select 37574,'CNSA' from dual union all
select 37575,'CNES' from dual union all
select 37576,'CNES' from dual union all
select 37577,'NASA' from dual union all
select 37578,'CASC' from dual union all
select 37579,'CASC' from dual union all
select 3758,'PVO' from dual union all
select 37580,'CASC' from dual union all
select 37581,'CASC' from dual union all
select 37582,'CNSA' from dual union all
select 37583,'CNSA' from dual union all
select 37584,'USAF' from dual union all
select 37585,'USAF' from dual union all
select 37586,'KTRAS' from dual union all
select 37587,'CASC' from dual union all
select 37588,'CASC' from dual union all
select 37589,'CASC' from dual union all
select 3759,'JPL' from dual union all
select 3759,'NASA' from dual union all
select 37590,'CASC' from dual union all
select 37591,'CASC' from dual union all
select 37592,'CASC' from dual union all
select 37593,'CASC' from dual union all
select 37594,'CASC' from dual union all
select 37595,'CASC' from dual union all
select 37596,'CASC' from dual union all
select 37597,'CASC' from dual union all
select 37598,'CASC' from dual union all
select 37599,'CASC' from dual union all
select 376,'OKB1' from dual union all
select 3760,'NASA' from dual union all
select 37600,'CASC' from dual union all
select 37601,'CASC' from dual union all
select 37602,'TCAN' from dual union all
select 37603,'KHRR' from dual union all
select 37604,'KHRR' from dual union all
select 37605,'ISRO' from dual union all
select 37606,'STEL' from dual;
insert into SATELLITE_ORG
select 37607,'AE' from dual union all
select 37608,'AE' from dual union all
select 37609,'MAI' from dual union all
select 3761,'MO' from dual union all
select 37610,'MAI' from dual union all
select 37611,'MAI' from dual union all
select 37612,'MAI' from dual union all
select 37613,'MAI' from dual union all
select 37614,'MAI' from dual union all
select 37615,'MAI' from dual union all
select 37616,'MAI' from dual union all
select 37617,'MAI' from dual union all
select 37618,'MAI' from dual union all
select 37619,'MAI' from dual union all
select 3762,'RVSN' from dual union all
select 37620,'MAI' from dual union all
select 37621,'MAI' from dual union all
select 37622,'MAI' from dual union all
select 37623,'MAI' from dual union all
select 37624,'MAI' from dual union all
select 37625,'MAI' from dual union all
select 37626,'MAI' from dual union all
select 37627,'NASA' from dual union all
select 37628,'NASA' from dual union all
select 37629,'NASA' from dual union all
select 3763,'MO' from dual union all
select 37630,'NASA' from dual union all
select 37631,'NASA' from dual union all
select 37632,'KVR' from dual union all
select 37633,'FKA' from dual union all
select 37634,'KVR' from dual union all
select 37635,'NASA' from dual union all
select 37636,'NASA' from dual union all
select 37637,'NASA' from dual union all
select 37638,'NASA' from dual union all
select 37639,'NASA' from dual union all
select 3764,'ESSA' from dual union all
select 37640,'NASA' from dual union all
select 37641,'NASA' from dual union all
select 37642,'NASA' from dual union all
select 37643,'NASA' from dual union all
select 37644,'NASA' from dual union all
select 37645,'NASA' from dual union all
select 37646,'NASA' from dual union all
select 37647,'NASA' from dual union all
select 37648,'NASA' from dual union all
select 37649,'NASA' from dual union all
select 3765,'MO' from dual union all
select 37650,'NASA' from dual union all
select 37651,'NASA' from dual union all
select 37652,'NASA' from dual union all
select 37653,'NASA' from dual union all
select 37654,'NASA' from dual union all
select 37655,'NASA' from dual union all
select 37656,'NASA' from dual union all
select 37657,'NASA' from dual union all
select 37658,'NASA' from dual union all
select 37659,'NASA' from dual union all
select 3766,'RVSN' from dual union all
select 37660,'NASA' from dual union all
select 37661,'NASA' from dual union all
select 37662,'NASA' from dual union all
select 37663,'NASA' from dual union all
select 37664,'NASA' from dual union all
select 37665,'NASA' from dual union all
select 37666,'NASA' from dual union all
select 37667,'NASA' from dual union all
select 37668,'NASA' from dual union all
select 37669,'NASA' from dual union all
select 3767,'NASA' from dual union all
select 37670,'NASA' from dual union all
select 37671,'NASA' from dual union all
select 37672,'NASA' from dual union all
select 37673,'CONAB' from dual union all
select 37674,'BOE' from dual union all
select 37675,'IRSA' from dual union all
select 37676,'IRSA' from dual union all
select 37677,'CASC' from dual union all
select 37678,'CASC' from dual union all
select 37679,'FKA' from dual union all
select 3768,'RVSN' from dual union all
select 37680,'KVR' from dual union all
select 37681,'CASC' from dual union all
select 37682,'CASC' from dual union all
select 37683,'CASC' from dual union all
select 37684,'CASC' from dual union all
select 37685,'CASC' from dual union all
select 37686,'CASC' from dual union all
select 37687,'CASC' from dual union all
select 37688,'CASC' from dual union all
select 37689,'CASC' from dual union all
select 3769,'NASA' from dual union all
select 37690,'CASC' from dual union all
select 37691,'CASC' from dual union all
select 37692,'CASC' from dual union all
select 37693,'CASC' from dual union all
select 37694,'CASC' from dual union all
select 37695,'CASC' from dual union all
select 37696,'CASC' from dual union all
select 37697,'CASC' from dual;
insert into SATELLITE_ORG
select 37698,'CASC' from dual union all
select 37699,'CASC' from dual union all
select 377,'CIA' from dual union all
select 377,'NRO' from dual union all
select 3770,'NASA' from dual union all
select 37700,'CASC' from dual union all
select 37701,'CASC' from dual union all
select 37702,'CASC' from dual union all
select 37703,'CASC' from dual union all
select 37704,'CASC' from dual union all
select 37705,'CASC' from dual union all
select 37706,'CASC' from dual union all
select 37707,'CASC' from dual union all
select 37708,'CASC' from dual union all
select 37709,'CASC' from dual union all
select 3771,'NASA' from dual union all
select 37710,'CASC' from dual union all
select 37711,'CASC' from dual union all
select 37712,'CASC' from dual union all
select 37713,'CASC' from dual union all
select 37714,'CASC' from dual union all
select 37715,'CASC' from dual union all
select 37716,'CASC' from dual union all
select 37717,'CASC' from dual union all
select 37718,'CASC' from dual union all
select 37719,'CASC' from dual union all
select 3772,'NRO' from dual union all
select 37720,'CASC' from dual union all
select 37721,'CASC' from dual union all
select 37722,'CASC' from dual union all
select 37723,'CASC' from dual union all
select 37724,'CASC' from dual union all
select 37725,'CASC' from dual union all
select 37726,'KVR' from dual union all
select 37727,'KVR' from dual union all
select 37728,'USAF' from dual union all
select 37729,'OSC' from dual union all
select 3773,'PVO' from dual union all
select 37730,'CASC' from dual union all
select 37731,'CASC' from dual union all
select 37732,'CASC' from dual union all
select 37733,'CASC' from dual union all
select 37734,'CASC' from dual union all
select 37735,'CASC' from dual union all
select 37736,'NASA' from dual union all
select 37737,'PLA' from dual union all
select 37738,'CASC' from dual union all
select 37739,'GLOB' from dual union all
select 3774,'RVSN' from dual union all
select 37740,'GLOB' from dual union all
select 37741,'GLOB' from dual union all
select 37742,'GLOB' from dual union all
select 37743,'GLOB' from dual union all
select 37744,'GLOB' from dual union all
select 37745,'NPOL' from dual union all
select 37746,'ISRO' from dual union all
select 37747,'ISRO' from dual union all
select 37748,'SES' from dual union all
select 37749,'KAZK' from dual union all
select 3775,'MO' from dual union all
select 37750,'KHRR' from dual union all
select 37751,'KHRR' from dual union all
select 37752,'USAF' from dual union all
select 37753,'USAF' from dual union all
select 37754,'BOE' from dual union all
select 37755,'AN' from dual union all
select 37755,'NPOL' from dual union all
select 37756,'KVR' from dual union all
select 37757,'KVR' from dual union all
select 37758,'KVR' from dual union all
select 37759,'KVR' from dual union all
select 3776,'RVSN' from dual union all
select 37760,'KVR' from dual union all
select 37761,'KVR' from dual union all
select 37762,'NASA' from dual union all
select 37763,'CNSA' from dual union all
select 37764,'CASC' from dual union all
select 37765,'CASC' from dual union all
select 37766,'CASC' from dual union all
select 37767,'CASC' from dual union all
select 37768,'CASC' from dual union all
select 37769,'CASC' from dual union all
select 3777,'MO' from dual union all
select 37770,'CASC' from dual union all
select 37771,'FKA' from dual union all
select 37772,'FKA' from dual union all
select 37773,'JPL' from dual union all
select 37773,'NASA' from dual union all
select 37774,'LM' from dual union all
select 37775,'SES' from dual union all
select 37776,'BSAT' from dual union all
select 37777,'AE' from dual union all
select 37778,'AE' from dual union all
select 37779,'SUPA' from dual union all
select 3778,'RVSN' from dual union all
select 37780,'CASC' from dual union all
select 37781,'CASC' from dual union all
select 37781,'GYZ' from dual union all
select 37782,'CNSA' from dual union all
select 37783,'USAF' from dual;
insert into SATELLITE_ORG
select 37784,'TSKBEM' from dual union all
select 37785,'MO' from dual union all
select 37786,'USAF' from dual union all
select 37787,'SES' from dual union all
select 37788,'ROSAP' from dual union all
select 37789,'NSRDA' from dual union all
select 3779,'RVSN' from dual union all
select 37790,'NSRDA' from dual union all
select 37791,'TUBT' from dual union all
select 37792,'SPQ' from dual union all
select 37793,'SPQ' from dual union all
select 37794,'NKAU' from dual union all
select 37795,'KTRAS' from dual union all
select 37796,'KTRAS' from dual union all
select 37797,'KTRAS' from dual union all
select 37798,'KS' from dual union all
select 37799,'KHRR' from dual union all
select 378,'USAF' from dual union all
select 3780,'NASA' from dual union all
select 37800,'KHRR' from dual union all
select 37801,'JPL' from dual union all
select 37801,'NASA' from dual union all
select 37802,'JPL' from dual union all
select 37802,'NASA' from dual union all
select 37803,'BOE' from dual union all
select 37804,'PLA' from dual union all
select 37805,'CASC' from dual union all
select 37806,'KVR' from dual union all
select 37807,'KHRR' from dual union all
select 37808,'KHRR' from dual union all
select 37809,'SES' from dual union all
select 3781,'RVSN' from dual union all
select 37810,'ARAB' from dual union all
select 37811,'AE' from dual union all
select 37812,'AE' from dual union all
select 37813,'CSICE' from dual union all
select 37814,'JAXA' from dual union all
select 37815,'JAXA' from dual union all
select 37816,'EUTSA' from dual union all
select 37817,'FKA' from dual union all
select 37818,'USN' from dual union all
select 37819,'USAF' from dual union all
select 3782,'RVSN' from dual union all
select 37820,'PLA' from dual union all
select 37821,'CASC' from dual union all
select 37822,'CASC' from dual union all
select 37823,'CASC' from dual union all
select 37824,'CASC' from dual union all
select 37825,'CASC' from dual union all
select 37826,'SES' from dual union all
select 37827,'KHRR' from dual union all
select 37828,'KHRR' from dual union all
select 37829,'FKA' from dual union all
select 37829,'KVR' from dual union all
select 3783,'PVO' from dual union all
select 37830,'NPOL' from dual union all
select 37831,'NASA' from dual union all
select 37832,'NASA' from dual union all
select 37833,'JAXA' from dual union all
select 37834,'INTELS' from dual union all
select 37835,'SIS' from dual union all
select 37836,'EUTSA' from dual union all
select 37837,'CASC' from dual union all
select 37838,'CNES' from dual union all
select 37838,'ISRO' from dual union all
select 37839,'IITK' from dual union all
select 3784,'PVO' from dual union all
select 37840,'FUCHS' from dual union all
select 37840,'ORBC' from dual union all
select 37841,'SRMU' from dual union all
select 37842,'ISRO' from dual union all
select 37843,'VIA' from dual union all
select 37844,'KHRR' from dual union all
select 37845,'KHRR' from dual union all
select 37846,'ESA' from dual union all
select 37846,'GSAEU' from dual union all
select 37847,'ESA' from dual union all
select 37847,'GSAEU' from dual union all
select 37848,'NPOL' from dual union all
select 37849,'NASA' from dual union all
select 37849,'NOAA' from dual union all
select 3785,'PVO' from dual union all
select 37850,'KVR' from dual union all
select 37851,'USU' from dual union all
select 37852,'USU' from dual union all
select 37853,'NSF' from dual union all
select 37853,'UMI' from dual union all
select 37854,'AUBU' from dual union all
select 37855,'MSU' from dual union all
select 37856,'BOE' from dual union all
select 37857,'FKA' from dual union all
select 37858,'KVR' from dual union all
select 37859,'PLA' from dual union all
select 3786,'PVO' from dual union all
select 37860,'CASC' from dual union all
select 37861,'CASC' from dual union all
select 37862,'NASA' from dual union all
select 37863,'MO' from dual union all
select 37864,'NASA' from dual union all
select 37865,'KVR' from dual;
insert into SATELLITE_ORG
select 37866,'KVR' from dual union all
select 37867,'FKA' from dual union all
select 37867,'KVR' from dual union all
select 37868,'FKA' from dual union all
select 37868,'KVR' from dual union all
select 37869,'FKA' from dual union all
select 37869,'KVR' from dual union all
select 3787,'PVO' from dual union all
select 37870,'KHRR' from dual union all
select 37871,'KHRR' from dual union all
select 37872,'IKI' from dual union all
select 37872,'NPOL' from dual union all
select 37873,'KVR' from dual union all
select 37874,'NUAA' from dual union all
select 37875,'PLA' from dual union all
select 37876,'CNSA' from dual union all
select 37877,'FKA' from dual union all
select 37878,'KVR' from dual union all
select 37879,'KVR' from dual union all
select 3788,'PVO' from dual union all
select 37880,'KVR' from dual union all
select 37881,'KVR' from dual union all
select 37882,'KVR' from dual union all
select 37883,'PLA' from dual union all
select 37884,'KVR' from dual union all
select 37885,'USAF' from dual union all
select 37886,'RVSN' from dual union all
select 37887,'RVSN' from dual union all
select 37888,'NASA' from dual union all
select 37889,'NASA' from dual union all
select 3789,'PVO' from dual union all
select 37890,'NASA' from dual union all
select 37891,'NASA' from dual union all
select 37892,'NASA' from dual union all
select 37893,'NASA' from dual union all
select 37894,'CASC' from dual union all
select 37895,'CASC' from dual union all
select 37896,'CASC' from dual union all
select 37897,'CASC' from dual union all
select 37898,'CASC' from dual union all
select 37899,'CASC' from dual union all
select 379,'OKB1' from dual union all
select 3790,'PVO' from dual union all
select 37900,'CASC' from dual union all
select 37901,'CASC' from dual union all
select 37902,'CASC' from dual union all
select 37903,'CASC' from dual union all
select 37904,'CASC' from dual union all
select 37905,'CASC' from dual union all
select 37906,'CASC' from dual union all
select 37907,'CASC' from dual union all
select 37908,'CASC' from dual union all
select 37909,'CASC' from dual union all
select 3791,'PVO' from dual union all
select 37910,'CASC' from dual union all
select 37911,'CASC' from dual union all
select 37912,'CASC' from dual union all
select 37913,'CASC' from dual union all
select 37914,'CASC' from dual union all
select 37915,'CASC' from dual union all
select 37916,'CASC' from dual union all
select 37917,'CASC' from dual union all
select 37918,'CASC' from dual union all
select 37919,'CASC' from dual union all
select 3792,'PVO' from dual union all
select 37920,'CASC' from dual union all
select 37921,'CASC' from dual union all
select 37922,'CASC' from dual union all
select 37923,'CASC' from dual union all
select 37924,'CASC' from dual union all
select 37925,'CASC' from dual union all
select 37926,'CASC' from dual union all
select 37927,'CASC' from dual union all
select 37928,'CASC' from dual union all
select 37929,'CASC' from dual union all
select 3793,'PVO' from dual union all
select 37930,'CASC' from dual union all
select 37931,'CASC' from dual union all
select 37932,'CNSA' from dual union all
select 37933,'ASIA' from dual union all
select 37934,'KHRR' from dual union all
select 37935,'KHRR' from dual union all
select 37936,'JPL' from dual union all
select 37936,'NASA' from dual union all
select 37937,'LM' from dual union all
select 37938,'FKA' from dual union all
select 37938,'KVR' from dual union all
select 37939,'NPOL' from dual union all
select 3794,'PVO' from dual union all
select 37940,'NPOL' from dual union all
select 37941,'PLA' from dual union all
select 37942,'CNSA' from dual union all
select 37943,'CNSA' from dual union all
select 37944,'CNSA' from dual union all
select 37945,'CNSA' from dual union all
select 37946,'CNSA' from dual union all
select 37947,'NPOL' from dual union all
select 37948,'CNSA' from dual union all
select 37949,'CASC' from dual union all
select 3795,'PVO' from dual;
insert into SATELLITE_ORG
select 37950,'IAI' from dual union all
select 37951,'FKA' from dual union all
select 37952,'KHRR' from dual union all
select 37953,'KHRR' from dual union all
select 37954,'CSICE' from dual union all
select 37955,'JAXA' from dual union all
select 37956,'KVR' from dual union all
select 37957,'KVR' from dual union all
select 37958,'KVR' from dual union all
select 37959,'KVR' from dual union all
select 3796,'PVO' from dual union all
select 37960,'KVR' from dual union all
select 37961,'KVR' from dual union all
select 37962,'KVR' from dual union all
select 37963,'KVR' from dual union all
select 37964,'KVR' from dual union all
select 37965,'KVR' from dual union all
select 37966,'KVR' from dual union all
select 37967,'KVR' from dual union all
select 37968,'KVR' from dual union all
select 37969,'KVR' from dual union all
select 3797,'PVO' from dual union all
select 37970,'KVR' from dual union all
select 37971,'KVR' from dual union all
select 37972,'KVR' from dual union all
select 37973,'KVR' from dual union all
select 37974,'KVR' from dual union all
select 37975,'KVR' from dual union all
select 37976,'KVR' from dual union all
select 37977,'KVR' from dual union all
select 37978,'KVR' from dual union all
select 37979,'KVR' from dual union all
select 3798,'RVSN' from dual union all
select 37980,'KVR' from dual union all
select 37981,'KVR' from dual union all
select 37982,'KVR' from dual union all
select 37983,'KVR' from dual union all
select 37984,'KVR' from dual union all
select 37985,'KVR' from dual union all
select 37986,'KVR' from dual union all
select 37987,'KVR' from dual union all
select 37988,'KVR' from dual union all
select 37989,'KVR' from dual union all
select 3799,'RVSN' from dual union all
select 37990,'KVR' from dual union all
select 37991,'KVR' from dual union all
select 37992,'KVR' from dual union all
select 37993,'KVR' from dual union all
select 37994,'KVR' from dual union all
select 37995,'KVR' from dual union all
select 37996,'KVR' from dual union all
select 37997,'KVR' from dual union all
select 37998,'KVR' from dual union all
select 37999,'KVR' from dual union all
select 38,'OKB1' from dual union all
select 380,'OKB1' from dual union all
select 3800,'RVSN' from dual union all
select 38000,'KVR' from dual union all
select 38001,'KVR' from dual union all
select 38002,'KVR' from dual union all
select 38003,'KVR' from dual union all
select 38004,'KVR' from dual union all
select 38005,'KVR' from dual union all
select 38006,'KVR' from dual union all
select 38007,'DGA' from dual union all
select 38008,'DGA' from dual union all
select 38009,'DGA' from dual union all
select 3801,'RVSN' from dual union all
select 38010,'DGA' from dual union all
select 38011,'FACH' from dual union all
select 38012,'CNES' from dual union all
select 38013,'CNES' from dual union all
select 38014,'NGCOM' from dual union all
select 38015,'CASC' from dual union all
select 38016,'IRIDS' from dual union all
select 38017,'IRIDS' from dual union all
select 38018,'IRIDS' from dual union all
select 38019,'IRIDS' from dual union all
select 3802,'RVSN' from dual union all
select 38020,'IRIDS' from dual union all
select 38021,'IRIDS' from dual union all
select 38022,'IRIDS' from dual union all
select 38023,'IRIDS' from dual union all
select 38024,'IRIDS' from dual union all
select 38025,'IRIDS' from dual union all
select 38026,'IRIDS' from dual union all
select 38027,'IRIDS' from dual union all
select 38028,'IRIDS' from dual union all
select 38029,'IRIDS' from dual union all
select 3803,'RVSN' from dual union all
select 38030,'IRIDS' from dual union all
select 38031,'IRIDS' from dual union all
select 38032,'IRIDS' from dual union all
select 38033,'IRIDS' from dual union all
select 38034,'IRIDS' from dual union all
select 38035,'IRIDS' from dual union all
select 38036,'FKA' from dual union all
select 38037,'VVKO' from dual union all
select 38038,'GZB' from dual union all
select 38039,'CNSA' from dual;
insert into SATELLITE_ORG
select 3804,'RVSN' from dual union all
select 38040,'GLOB' from dual union all
select 38041,'GLOB' from dual union all
select 38042,'GLOB' from dual union all
select 38043,'GLOB' from dual union all
select 38044,'GLOB' from dual union all
select 38045,'GLOB' from dual union all
select 38046,'GZB' from dual union all
select 38047,'FUCHS' from dual union all
select 38047,'ORBC' from dual union all
select 38048,'CNSA' from dual union all
select 38049,'CMA' from dual union all
select 3805,'RVSN' from dual union all
select 38050,'CASC' from dual union all
select 38051,'FKA' from dual union all
select 38052,'KVR' from dual union all
select 38053,'KVR' from dual union all
select 38054,'KVR' from dual union all
select 38055,'KVR' from dual union all
select 38056,'KVR' from dual union all
select 38057,'KVR' from dual union all
select 38058,'KVR' from dual union all
select 38059,'KVR' from dual union all
select 3806,'RVSN' from dual union all
select 38060,'KVR' from dual union all
select 38061,'KVR' from dual union all
select 38062,'KVR' from dual union all
select 38063,'KVR' from dual union all
select 38064,'KVR' from dual union all
select 38065,'KVR' from dual union all
select 38066,'KVR' from dual union all
select 38067,'KVR' from dual union all
select 38068,'KVR' from dual union all
select 38069,'KVR' from dual union all
select 3807,'MO' from dual union all
select 38070,'USAF' from dual union all
select 38071,'BOE' from dual union all
select 38072,'CASC' from dual union all
select 38073,'FKA' from dual union all
select 38074,'VVKO' from dual union all
select 38075,'IRSA' from dual union all
select 38076,'IRSA' from dual union all
select 38077,'ASI' from dual union all
select 38078,'BOL' from dual union all
select 38079,'TORP' from dual union all
select 3808,'RVSN' from dual union all
select 38080,'BUCU' from dual union all
select 38081,'BME' from dual union all
select 38082,'VIGO' from dual union all
select 38083,'PWAR' from dual union all
select 38084,'CNES' from dual union all
select 38084,'MONTP2' from dual union all
select 38085,'ROSAP' from dual union all
select 38086,'ASI' from dual union all
select 38087,'SES' from dual union all
select 38088,'KHRR' from dual union all
select 38089,'KHRR' from dual union all
select 3809,'USN' from dual union all
select 38090,'FKA' from dual union all
select 38091,'CNSA' from dual union all
select 38092,'CASC' from dual union all
select 38093,'USN' from dual union all
select 38094,'LM' from dual union all
select 38095,'CASC' from dual union all
select 38096,'ESA' from dual union all
select 38097,'AE' from dual union all
select 38098,'INTELS' from dual union all
select 38099,'KHRR' from dual union all
select 381,'OKB1' from dual union all
select 3810,'USN' from dual union all
select 38100,'KHRR' from dual union all
select 38101,'PVO' from dual union all
select 38102,'KHRR' from dual union all
select 38103,'KHRR' from dual union all
select 38104,'KHRR' from dual union all
select 38105,'RVSNR' from dual union all
select 38106,'RVSNR' from dual union all
select 38107,'APT' from dual union all
select 38108,'CASC' from dual union all
select 38109,'NROC' from dual union all
select 3811,'NASA' from dual union all
select 38110,'CASC' from dual union all
select 38111,'CASC' from dual union all
select 38112,'CASC' from dual union all
select 38113,'CASC' from dual union all
select 38114,'CASC' from dual union all
select 38115,'CASC' from dual union all
select 38116,'CASC' from dual union all
select 38117,'CASC' from dual union all
select 38118,'CASC' from dual union all
select 38119,'CASC' from dual union all
select 3812,'NASA' from dual union all
select 38120,'CASC' from dual union all
select 38121,'CASC' from dual union all
select 38122,'CASC' from dual union all
select 38123,'CASC' from dual union all
select 38124,'CASC' from dual union all
select 38125,'CASC' from dual union all
select 38126,'CASC' from dual union all
select 38127,'CASC' from dual;
insert into SATELLITE_ORG
select 38128,'CASC' from dual union all
select 38129,'CASC' from dual union all
select 3813,'PVO' from dual union all
select 38130,'CASC' from dual union all
select 38131,'CASC' from dual union all
select 38132,'CASC' from dual union all
select 38133,'CASC' from dual union all
select 38134,'CASC' from dual union all
select 38135,'CASC' from dual union all
select 38136,'CASC' from dual union all
select 38137,'CASC' from dual union all
select 38138,'CASC' from dual union all
select 38139,'CASC' from dual union all
select 3814,'PVO' from dual union all
select 38140,'CASC' from dual union all
select 38141,'CASC' from dual union all
select 38142,'CASC' from dual union all
select 38143,'CASC' from dual union all
select 38144,'CASC' from dual union all
select 38145,'CASC' from dual union all
select 38146,'CASC' from dual union all
select 38147,'CASC' from dual union all
select 38148,'CASC' from dual union all
select 38149,'CASC' from dual union all
select 3815,'PVO' from dual union all
select 38150,'CASC' from dual union all
select 38151,'CASC' from dual union all
select 38152,'CASC' from dual union all
select 38153,'CASC' from dual union all
select 38154,'CASC' from dual union all
select 38155,'CASC' from dual union all
select 38156,'CASC' from dual union all
select 38157,'CASC' from dual union all
select 38158,'CASC' from dual union all
select 38159,'CASC' from dual union all
select 3816,'PVO' from dual union all
select 38160,'CASC' from dual union all
select 38161,'CASC' from dual union all
select 38162,'CASC' from dual union all
select 38163,'CASC' from dual union all
select 38164,'CASC' from dual union all
select 38165,'CASC' from dual union all
select 38166,'CASC' from dual union all
select 38167,'CASC' from dual union all
select 38168,'CASC' from dual union all
select 38169,'CASC' from dual union all
select 3817,'PVO' from dual union all
select 38170,'CASC' from dual union all
select 38171,'CASC' from dual union all
select 38172,'CASC' from dual union all
select 38173,'CASC' from dual union all
select 38174,'CNSA' from dual union all
select 38175,'CNSA' from dual union all
select 38176,'CNSA' from dual union all
select 38177,'CNSA' from dual union all
select 38178,'CNSA' from dual union all
select 38179,'CNSA' from dual union all
select 3818,'OKB10' from dual union all
select 38180,'CNSA' from dual union all
select 38181,'NASA' from dual union all
select 38182,'NASA' from dual union all
select 38183,'NASA' from dual union all
select 38184,'CNSA' from dual union all
select 38185,'CNSA' from dual union all
select 38186,'CNSA' from dual union all
select 38187,'KHRR' from dual union all
select 38188,'KVR' from dual union all
select 38189,'KVR' from dual union all
select 3819,'RVSN' from dual union all
select 38190,'KVR' from dual union all
select 38191,'KVR' from dual union all
select 38192,'KVR' from dual union all
select 38193,'KVR' from dual union all
select 38194,'KVR' from dual union all
select 38195,'KVR' from dual union all
select 38196,'KVR' from dual union all
select 38197,'KVR' from dual union all
select 38198,'KVR' from dual union all
select 38199,'KVR' from dual union all
select 382,'RVSN' from dual union all
select 3820,'RVSN' from dual union all
select 38200,'KVR' from dual union all
select 38201,'KVR' from dual union all
select 38202,'KVR' from dual union all
select 38203,'KVR' from dual union all
select 38204,'KVR' from dual union all
select 38205,'KVR' from dual union all
select 38206,'KVR' from dual union all
select 38207,'KVR' from dual union all
select 38208,'KVR' from dual union all
select 38209,'KVR' from dual union all
select 3821,'RVSN' from dual union all
select 38210,'KVR' from dual union all
select 38211,'KVR' from dual union all
select 38212,'KVR' from dual union all
select 38213,'KVR' from dual union all
select 38214,'KVR' from dual union all
select 38215,'KVR' from dual union all
select 38216,'KVR' from dual union all
select 38217,'KVR' from dual;
insert into SATELLITE_ORG
select 38218,'KVR' from dual union all
select 38219,'KVR' from dual union all
select 3822,'RVSN' from dual union all
select 38220,'KVR' from dual union all
select 38221,'KVR' from dual union all
select 38222,'FKA' from dual union all
select 38223,'VVKO' from dual union all
select 38224,'IRIDS' from dual union all
select 38225,'IRIDS' from dual union all
select 38226,'IRIDS' from dual union all
select 38227,'IRIDS' from dual union all
select 38228,'IRIDS' from dual union all
select 38229,'IRIDS' from dual union all
select 3823,'USAF' from dual union all
select 38230,'IRIDS' from dual union all
select 38231,'IRIDS' from dual union all
select 38232,'IRIDS' from dual union all
select 38233,'IRIDS' from dual union all
select 38234,'IRIDS' from dual union all
select 38235,'IRIDS' from dual union all
select 38236,'IRIDS' from dual union all
select 38237,'IRIDS' from dual union all
select 38238,'IRIDS' from dual union all
select 38239,'IRIDS' from dual union all
select 3824,'USAF' from dual union all
select 38240,'IRIDS' from dual union all
select 38241,'IRIDS' from dual union all
select 38242,'IRIDS' from dual union all
select 38243,'IRIDS' from dual union all
select 38244,'IRIDS' from dual union all
select 38245,'ALYAH' from dual union all
select 38246,'KHRR' from dual union all
select 38247,'KHRR' from dual union all
select 38248,'ISRO' from dual union all
select 38249,'ISRO' from dual union all
select 3825,'USAF' from dual union all
select 38250,'CNSA' from dual union all
select 38251,'CNSA' from dual union all
select 38252,'CASC' from dual union all
select 38253,'CASC' from dual union all
select 38254,'USAF' from dual union all
select 38255,'LM' from dual union all
select 38256,'GZB' from dual union all
select 38257,'PLA' from dual union all
select 38258,'NUDTC' from dual union all
select 38259,'CNSA' from dual union all
select 3826,'USAF' from dual union all
select 38260,'CNSA' from dual union all
select 38261,'CNSA' from dual union all
select 38262,'CNSA' from dual union all
select 38263,'CNSA' from dual union all
select 38264,'CASC' from dual union all
select 38265,'CASC' from dual union all
select 38266,'CASC' from dual union all
select 38267,'CASC' from dual union all
select 38268,'CASC' from dual union all
select 38269,'CASC' from dual union all
select 3827,'USAF' from dual union all
select 38270,'CASC' from dual union all
select 38271,'KVR' from dual union all
select 38272,'KVR' from dual union all
select 38273,'KVR' from dual union all
select 38274,'KVR' from dual union all
select 38275,'KVR' from dual union all
select 38276,'KVR' from dual union all
select 38277,'KVR' from dual union all
select 38278,'KVR' from dual union all
select 38279,'KVR' from dual union all
select 3828,'USAF' from dual union all
select 38280,'KVR' from dual union all
select 38281,'KVR' from dual union all
select 38282,'KVR' from dual union all
select 38283,'KVR' from dual union all
select 38284,'KVR' from dual union all
select 38285,'KVR' from dual union all
select 38286,'KVR' from dual union all
select 38287,'KVR' from dual union all
select 38288,'KVR' from dual union all
select 38289,'KVR' from dual union all
select 3829,'CIA' from dual union all
select 3829,'NRO' from dual union all
select 38290,'KVR' from dual union all
select 38291,'FKA' from dual union all
select 38292,'VVKO' from dual union all
select 38293,'CNSA' from dual union all
select 38294,'CNSA' from dual union all
select 38295,'CNSA' from dual union all
select 38296,'CNSA' from dual union all
select 38297,'CNSA' from dual union all
select 38298,'CNSA' from dual union all
select 38299,'CNSA' from dual union all
select 383,'RVSN' from dual union all
select 3830,'NRO' from dual union all
select 38300,'CNSA' from dual union all
select 38301,'CNSA' from dual union all
select 38302,'CNSA' from dual union all
select 38303,'CNSA' from dual union all
select 38304,'CNSA' from dual union all
select 38305,'CNSA' from dual union all
select 38306,'CNSA' from dual;
insert into SATELLITE_ORG
select 38307,'CNSA' from dual union all
select 38308,'CNSA' from dual union all
select 38309,'CNSA' from dual union all
select 3831,'MO' from dual union all
select 38310,'CNSA' from dual union all
select 38311,'CNSA' from dual union all
select 38312,'CNSA' from dual union all
select 38313,'CNSA' from dual union all
select 38314,'CNSA' from dual union all
select 38315,'CNSA' from dual union all
select 38316,'CNSA' from dual union all
select 38317,'CNSA' from dual union all
select 38318,'CNSA' from dual union all
select 38319,'CNSA' from dual union all
select 3832,'RVSN' from dual union all
select 38320,'CNSA' from dual union all
select 38321,'CNSA' from dual union all
select 38322,'CNSA' from dual union all
select 38323,'CNSA' from dual union all
select 38324,'CNSA' from dual union all
select 38325,'CNSA' from dual union all
select 38326,'CNSA' from dual union all
select 38327,'CNSA' from dual union all
select 38328,'ISRO' from dual union all
select 38329,'USAF' from dual union all
select 3833,'MO' from dual union all
select 38330,'KVR' from dual union all
select 38331,'SKPJ' from dual union all
select 38332,'VNPT' from dual union all
select 38333,'AE' from dual union all
select 38334,'AE' from dual union all
select 38335,'KVR' from dual union all
select 38336,'KVR' from dual union all
select 38337,'JAXA' from dual union all
select 38338,'KARI' from dual union all
select 38339,'JAXA' from dual union all
select 3834,'RVSN' from dual union all
select 38340,'KYUT' from dual union all
select 38341,'JAXA' from dual union all
select 38342,'TCAN' from dual union all
select 38343,'KHRR' from dual union all
select 38344,'KHRR' from dual union all
select 38345,'JAXA' from dual union all
select 38346,'JAXA' from dual union all
select 38347,'JAXA' from dual union all
select 38348,'SPX' from dual union all
select 38349,'SPX' from dual union all
select 3835,'GMS' from dual union all
select 3835,'MO' from dual union all
select 38350,'SPX' from dual union all
select 38351,'SPX' from dual union all
select 38352,'PLA' from dual union all
select 38353,'CASC' from dual union all
select 38354,'PLA' from dual union all
select 38355,'CNSA' from dual union all
select 38356,'INTELS' from dual union all
select 38357,'FKA' from dual union all
select 38358,'JPL' from dual union all
select 38358,'NASA' from dual union all
select 38359,'OSC' from dual union all
select 3836,'RVSN' from dual union all
select 38360,'USAF' from dual union all
select 38361,'USAF' from dual union all
select 38362,'USAF' from dual union all
select 38363,'USAF' from dual union all
select 38364,'USAF' from dual union all
select 38365,'USAF' from dual union all
select 38366,'USAF' from dual union all
select 38367,'USAF' from dual union all
select 38368,'USAF' from dual union all
select 38369,'USAF' from dual union all
select 3837,'JPL' from dual union all
select 3837,'NASA' from dual union all
select 38370,'USAF' from dual union all
select 38371,'USAF' from dual union all
select 38372,'USAF' from dual union all
select 38373,'USAF' from dual union all
select 38374,'USAF' from dual union all
select 38375,'USAF' from dual union all
select 38376,'USAF' from dual union all
select 38377,'USAF' from dual union all
select 38378,'USAF' from dual union all
select 38379,'USAF' from dual union all
select 3838,'USAF' from dual union all
select 38380,'USAF' from dual union all
select 38381,'USAF' from dual union all
select 38382,'USAF' from dual union all
select 38383,'USAF' from dual union all
select 38384,'USAF' from dual union all
select 38385,'USAF' from dual union all
select 38386,'USAF' from dual union all
select 38387,'USAF' from dual union all
select 38388,'USAF' from dual union all
select 38389,'USAF' from dual union all
select 3839,'USAF' from dual union all
select 38390,'USAF' from dual union all
select 38391,'USAF' from dual union all
select 38392,'USAF' from dual union all
select 38393,'USAF' from dual union all
select 38394,'USAF' from dual;
insert into SATELLITE_ORG
select 38395,'USAF' from dual union all
select 38396,'USAF' from dual union all
select 38397,'USAF' from dual union all
select 38398,'USAF' from dual union all
select 38399,'USAF' from dual union all
select 384,'OKB1' from dual union all
select 3840,'USAF' from dual union all
select 38400,'USAF' from dual union all
select 38401,'USAF' from dual union all
select 38402,'USAF' from dual union all
select 38403,'USAF' from dual union all
select 38404,'USAF' from dual union all
select 38405,'USAF' from dual union all
select 38406,'USAF' from dual union all
select 38407,'USAF' from dual union all
select 38408,'USAF' from dual union all
select 38409,'USAF' from dual union all
select 3841,'USAF' from dual union all
select 38410,'USAF' from dual union all
select 38411,'USAF' from dual union all
select 38412,'USAF' from dual union all
select 38413,'USAF' from dual union all
select 38414,'USAF' from dual union all
select 38415,'USAF' from dual union all
select 38416,'USAF' from dual union all
select 38417,'USAF' from dual union all
select 38418,'USAF' from dual union all
select 38419,'USAF' from dual union all
select 3842,'USAF' from dual union all
select 38420,'USAF' from dual union all
select 38421,'USAF' from dual union all
select 38422,'USAF' from dual union all
select 38423,'USAF' from dual union all
select 38424,'USAF' from dual union all
select 38425,'USAF' from dual union all
select 38426,'USAF' from dual union all
select 38427,'USAF' from dual union all
select 38428,'USAF' from dual union all
select 38429,'USAF' from dual union all
select 3843,'PVO' from dual union all
select 38430,'USAF' from dual union all
select 38431,'USAF' from dual union all
select 38432,'USAF' from dual union all
select 38433,'USAF' from dual union all
select 38434,'USAF' from dual union all
select 38435,'USAF' from dual union all
select 38436,'USAF' from dual union all
select 38437,'USAF' from dual union all
select 38438,'USAF' from dual union all
select 38439,'USAF' from dual union all
select 3844,'RVSN' from dual union all
select 38440,'USAF' from dual union all
select 38441,'USAF' from dual union all
select 38442,'USAF' from dual union all
select 38443,'USAF' from dual union all
select 38444,'USAF' from dual union all
select 38445,'USAF' from dual union all
select 38446,'USAF' from dual union all
select 38447,'USAF' from dual union all
select 38448,'USAF' from dual union all
select 38449,'USAF' from dual union all
select 3845,'NASA' from dual union all
select 38450,'USAF' from dual union all
select 38451,'USAF' from dual union all
select 38452,'USAF' from dual union all
select 38453,'USAF' from dual union all
select 38454,'USAF' from dual union all
select 38455,'USAF' from dual union all
select 38456,'USAF' from dual union all
select 38457,'USAF' from dual union all
select 38458,'USAF' from dual union all
select 38459,'USAF' from dual union all
select 3846,'PVO' from dual union all
select 3846,'YUZH' from dual union all
select 38460,'USAF' from dual union all
select 38461,'PLA' from dual union all
select 38462,'CASC' from dual union all
select 38463,'CASC' from dual union all
select 38464,'CASC' from dual union all
select 38465,'CASC' from dual union all
select 38466,'NROC' from dual union all
select 38467,'CASC' from dual union all
select 38468,'IRIDS' from dual union all
select 38469,'IRIDS' from dual union all
select 3847,'RVSN' from dual union all
select 38470,'IRIDS' from dual union all
select 38471,'IRIDS' from dual union all
select 38472,'IRIDS' from dual union all
select 38473,'IRIDS' from dual union all
select 38474,'IRIDS' from dual union all
select 38475,'IRIDS' from dual union all
select 38476,'IRIDS' from dual union all
select 38477,'IRIDS' from dual union all
select 38478,'KVR' from dual union all
select 38479,'KVR' from dual union all
select 3848,'NASA' from dual union all
select 38480,'KVR' from dual union all
select 38481,'KVR' from dual union all
select 38482,'KVR' from dual union all
select 38483,'KVR' from dual;
insert into SATELLITE_ORG
select 38484,'KVR' from dual union all
select 38485,'KVR' from dual union all
select 38486,'KVR' from dual union all
select 38487,'KVR' from dual union all
select 38488,'CASC' from dual union all
select 38489,'CASC' from dual union all
select 3849,'RVSN' from dual union all
select 38490,'CASC' from dual union all
select 38491,'CASC' from dual union all
select 38492,'CASC' from dual union all
select 38493,'CASC' from dual union all
select 38494,'CASC' from dual union all
select 38495,'CASC' from dual union all
select 38496,'CASC' from dual union all
select 38497,'CASC' from dual union all
select 38498,'CASC' from dual union all
select 38499,'CASC' from dual union all
select 385,'NRO' from dual union all
select 385,'USA' from dual union all
select 3850,'RVSN' from dual union all
select 38500,'CASC' from dual union all
select 38501,'CASC' from dual union all
select 38502,'CASC' from dual union all
select 38503,'CASC' from dual union all
select 38504,'CASC' from dual union all
select 38505,'CASC' from dual union all
select 38506,'CASC' from dual union all
select 38507,'CASC' from dual union all
select 38508,'CASC' from dual union all
select 38509,'CASC' from dual union all
select 3851,'RVSN' from dual union all
select 38510,'CASC' from dual union all
select 38511,'CASC' from dual union all
select 38512,'CASC' from dual union all
select 38513,'CASC' from dual union all
select 38514,'CASC' from dual union all
select 38515,'CASC' from dual union all
select 38516,'CASC' from dual union all
select 38517,'CASC' from dual union all
select 38518,'CASC' from dual union all
select 38519,'CASC' from dual union all
select 3852,'RVSN' from dual union all
select 38520,'CASC' from dual union all
select 38521,'USN' from dual union all
select 38522,'USN' from dual union all
select 38523,'USN' from dual union all
select 38524,'USN' from dual union all
select 38525,'USN' from dual union all
select 38526,'USN' from dual union all
select 38527,'USN' from dual union all
select 38528,'NROC' from dual union all
select 38529,'BOE' from dual union all
select 3853,'RVSN' from dual union all
select 38530,'USN' from dual union all
select 38531,'OKB10' from dual union all
select 38532,'KVR' from dual union all
select 38533,'KVR' from dual union all
select 38534,'KVR' from dual union all
select 38535,'KVR' from dual union all
select 38536,'KVR' from dual union all
select 38537,'KVR' from dual union all
select 38538,'KVR' from dual union all
select 38539,'KVR' from dual union all
select 3854,'MO' from dual union all
select 38540,'KVR' from dual union all
select 38541,'KVR' from dual union all
select 38542,'KVR' from dual union all
select 38543,'KVR' from dual union all
select 38544,'KVR' from dual union all
select 38545,'NASA' from dual union all
select 38546,'NASA' from dual union all
select 38547,'NASA' from dual union all
select 38548,'RVSNR' from dual union all
select 38549,'NASA' from dual union all
select 3855,'PVO' from dual union all
select 38550,'PLA' from dual union all
select 38551,'ECHO' from dual union all
select 38552,'EUMET' from dual union all
select 38553,'AE' from dual union all
select 38554,'AE' from dual union all
select 38555,'NOAA' from dual union all
select 38556,'RVSN' from dual union all
select 38557,'KHRR' from dual union all
select 38558,'KHRR' from dual union all
select 38559,'KHRR' from dual union all
select 3856,'RVSN' from dual union all
select 38560,'KHRR' from dual union all
select 38561,'KHRR' from dual union all
select 38562,'KHRR' from dual union all
select 38563,'KHRR' from dual union all
select 38564,'KHRR' from dual union all
select 38565,'KHRR' from dual union all
select 38566,'KHRR' from dual union all
select 38567,'KHRR' from dual union all
select 38568,'KHRR' from dual union all
select 38569,'KHRR' from dual union all
select 3857,'RVSN' from dual union all
select 38570,'KHRR' from dual union all
select 38571,'KHRR' from dual union all
select 38572,'KHRR' from dual;
insert into SATELLITE_ORG
select 38573,'KHRR' from dual union all
select 38574,'KHRR' from dual union all
select 38575,'KHRR' from dual union all
select 38576,'KHRR' from dual union all
select 38577,'KHRR' from dual union all
select 38578,'KHRR' from dual union all
select 38579,'KHRR' from dual union all
select 3858,'RVSN' from dual union all
select 38580,'KHRR' from dual union all
select 38581,'KHRR' from dual union all
select 38582,'KHRR' from dual union all
select 38583,'KHRR' from dual union all
select 38584,'KHRR' from dual union all
select 38585,'KHRR' from dual union all
select 38586,'KHRR' from dual union all
select 38587,'KHRR' from dual union all
select 38588,'KHRR' from dual union all
select 38589,'KHRR' from dual union all
select 3859,'RVSN' from dual union all
select 38590,'KHRR' from dual union all
select 38591,'KHRR' from dual union all
select 38592,'KHRR' from dual union all
select 38593,'KHRR' from dual union all
select 38594,'KHRR' from dual union all
select 38595,'KHRR' from dual union all
select 38596,'KHRR' from dual union all
select 38597,'KHRR' from dual union all
select 38598,'KHRR' from dual union all
select 38599,'KHRR' from dual union all
select 386,'OKB1' from dual union all
select 3860,'RVSN' from dual union all
select 38600,'KHRR' from dual union all
select 38601,'KHRR' from dual union all
select 38602,'KHRR' from dual union all
select 38603,'KHRR' from dual union all
select 38604,'KHRR' from dual union all
select 38605,'KHRR' from dual union all
select 38606,'KHRR' from dual union all
select 38607,'KHRR' from dual union all
select 38608,'KHRR' from dual union all
select 38609,'KHRR' from dual union all
select 3861,'RVSN' from dual union all
select 38610,'KHRR' from dual union all
select 38611,'KHRR' from dual union all
select 38612,'KHRR' from dual union all
select 38613,'KHRR' from dual union all
select 38614,'KHRR' from dual union all
select 38615,'KHRR' from dual union all
select 38616,'KHRR' from dual union all
select 38617,'KHRR' from dual union all
select 38618,'KHRR' from dual union all
select 38619,'KHRR' from dual union all
select 3862,'RVSN' from dual union all
select 38620,'KHRR' from dual union all
select 38621,'KHRR' from dual union all
select 38622,'KHRR' from dual union all
select 38623,'KHRR' from dual union all
select 38624,'KHRR' from dual union all
select 38625,'KHRR' from dual union all
select 38626,'KHRR' from dual union all
select 38627,'KHRR' from dual union all
select 38628,'KHRR' from dual union all
select 38629,'KHRR' from dual union all
select 3863,'RVSN' from dual union all
select 38630,'KHRR' from dual union all
select 38631,'KHRR' from dual union all
select 38632,'KHRR' from dual union all
select 38633,'KHRR' from dual union all
select 38634,'KHRR' from dual union all
select 38635,'KHRR' from dual union all
select 38636,'KHRR' from dual union all
select 38637,'KHRR' from dual union all
select 38638,'KHRR' from dual union all
select 38639,'KHRR' from dual union all
select 3864,'RVSN' from dual union all
select 38640,'KHRR' from dual union all
select 38641,'KHRR' from dual union all
select 38642,'KHRR' from dual union all
select 38643,'KHRR' from dual union all
select 38644,'KHRR' from dual union all
select 38645,'KHRR' from dual union all
select 38646,'KHRR' from dual union all
select 38647,'KHRR' from dual union all
select 38648,'KHRR' from dual union all
select 38649,'KHRR' from dual union all
select 3865,'RVSN' from dual union all
select 38650,'KHRR' from dual union all
select 38651,'KHRR' from dual union all
select 38652,'SES' from dual union all
select 38653,'KHRR' from dual union all
select 38654,'KHRR' from dual union all
select 38655,'KVR' from dual union all
select 38656,'KVR' from dual union all
select 38657,'CNSA' from dual union all
select 38658,'CNSA' from dual union all
select 38659,'CNSA' from dual union all
select 3866,'RVSN' from dual union all
select 38660,'CNSA' from dual union all
select 38661,'CNSA' from dual union all
select 38662,'CNSA' from dual;
insert into SATELLITE_ORG
select 38663,'CNSA' from dual union all
select 38664,'CNSA' from dual union all
select 38665,'OKB10' from dual union all
select 38666,'OKB10' from dual union all
select 38667,'RVSN' from dual union all
select 38668,'PVO' from dual union all
select 38669,'PVO' from dual union all
select 3867,'RVSN' from dual union all
select 38670,'CNSA' from dual union all
select 38671,'FKA' from dual union all
select 38672,'VVKO' from dual union all
select 38673,'MAI' from dual union all
select 38674,'BOE' from dual union all
select 38675,'BOE' from dual union all
select 38676,'BOE' from dual union all
select 38677,'BOE' from dual union all
select 38678,'BOE' from dual union all
select 38679,'SIS' from dual union all
select 3868,'RVSN' from dual union all
select 38680,'RVSNR' from dual union all
select 38681,'RVSNR' from dual union all
select 38682,'NASDA' from dual union all
select 38683,'NASDA' from dual union all
select 38684,'NASDA' from dual union all
select 38685,'NASDA' from dual union all
select 38686,'RCA' from dual union all
select 38687,'RCA' from dual union all
select 38688,'RCA' from dual union all
select 38689,'RCA' from dual union all
select 3869,'RVSN' from dual union all
select 38690,'USAF' from dual union all
select 38691,'USAF' from dual union all
select 38692,'USAF' from dual union all
select 38693,'USAF' from dual union all
select 38694,'USAF' from dual union all
select 38695,'USAF' from dual union all
select 38696,'USAF' from dual union all
select 38697,'USAF' from dual union all
select 38698,'USAF' from dual union all
select 38699,'USAF' from dual union all
select 387,'OKB1' from dual union all
select 3870,'RVSN' from dual union all
select 38700,'USAF' from dual union all
select 38701,'USAF' from dual union all
select 38702,'USAF' from dual union all
select 38703,'USAF' from dual union all
select 38704,'USAF' from dual union all
select 38705,'USAF' from dual union all
select 38706,'JAXA' from dual union all
select 38707,'METP' from dual union all
select 38708,'ANBEL' from dual union all
select 38709,'COMDEV' from dual union all
select 3871,'RVSN' from dual union all
select 38710,'DLR' from dual union all
select 38711,'IKI' from dual union all
select 38711,'NPOL' from dual union all
select 38712,'KVR' from dual union all
select 38713,'KVR' from dual union all
select 38714,'CASC' from dual union all
select 38715,'CASC' from dual union all
select 38716,'CASC' from dual union all
select 38717,'CASC' from dual union all
select 38718,'CASC' from dual union all
select 38719,'CASC' from dual union all
select 3872,'PVO' from dual union all
select 38720,'CASC' from dual union all
select 38721,'CASC' from dual union all
select 38722,'CASC' from dual union all
select 38723,'CASC' from dual union all
select 38724,'CASC' from dual union all
select 38725,'CASC' from dual union all
select 38726,'CASC' from dual union all
select 38727,'CASC' from dual union all
select 38728,'CASC' from dual union all
select 38729,'CASC' from dual union all
select 3873,'PVO' from dual union all
select 38730,'PLA' from dual union all
select 38731,'CASC' from dual union all
select 38732,'NOAA' from dual union all
select 38733,'KVR' from dual union all
select 38734,'FKA' from dual union all
select 38735,'FKA' from dual union all
select 38736,'FKA' from dual union all
select 38737,'KVR' from dual union all
select 38738,'FKA' from dual union all
select 38739,'VVKO' from dual union all
select 3874,'PVO' from dual union all
select 38740,'INTELS' from dual union all
select 38741,'AVANTI' from dual union all
select 38742,'AE' from dual union all
select 38743,'AE' from dual union all
select 38744,'TELK' from dual union all
select 38745,'KS' from dual union all
select 38746,'KHRR' from dual union all
select 38747,'KHRR' from dual union all
select 38748,'CASC' from dual union all
select 38749,'INTELS' from dual union all
select 3875,'PVO' from dual union all
select 38750,'FKA' from dual union all
select 38751,'FKA' from dual;
insert into SATELLITE_ORG
select 38752,'APL' from dual union all
select 38752,'NASA' from dual union all
select 38753,'APL' from dual union all
select 38753,'NASA' from dual union all
select 38754,'LM' from dual union all
select 38755,'SPOT' from dual union all
select 38756,'OIT' from dual union all
select 38757,'ISRO' from dual union all
select 38758,'NROC' from dual union all
select 38758,'USN' from dual union all
select 38759,'SMDC' from dual union all
select 3876,'PVO' from dual union all
select 38760,'USCLA' from dual union all
select 38761,'UCO' from dual union all
select 38762,'MSUKY' from dual union all
select 38763,'CALP' from dual union all
select 38764,'UCB' from dual union all
select 38765,'LLNL' from dual union all
select 38766,'SMDC' from dual union all
select 38767,'AERO' from dual union all
select 38768,'AERO' from dual union all
select 38769,'AERO' from dual union all
select 3877,'RVSN' from dual union all
select 38770,'CALP' from dual union all
select 38770,'NROC' from dual union all
select 38771,'EUMET' from dual union all
select 38772,'NPOL' from dual union all
select 38773,'NROC' from dual union all
select 38773,'USN' from dual union all
select 38774,'CNSA' from dual union all
select 38775,'CNSA' from dual union all
select 38776,'CASC' from dual union all
select 38777,'CASC' from dual union all
select 38778,'SES' from dual union all
select 38779,'ISRO' from dual union all
select 3878,'RVSN' from dual union all
select 38780,'AE' from dual union all
select 38781,'AE' from dual union all
select 38782,'ABAE' from dual union all
select 38783,'CASC' from dual union all
select 38784,'CASC' from dual union all
select 38785,'CASC' from dual union all
select 38786,'CASC' from dual union all
select 38787,'CASC' from dual union all
select 38788,'CASC' from dual union all
select 38789,'CASC' from dual union all
select 3879,'RVSN' from dual union all
select 38790,'CASC' from dual union all
select 38791,'CASC' from dual union all
select 38792,'CASC' from dual union all
select 38793,'CASC' from dual union all
select 38794,'CASC' from dual union all
select 38795,'CASC' from dual union all
select 38796,'CASC' from dual union all
select 38797,'CASC' from dual union all
select 38798,'CASC' from dual union all
select 38799,'CASC' from dual union all
select 388,'NASA' from dual union all
select 3880,'RVSN' from dual union all
select 38800,'CASC' from dual union all
select 38801,'CASC' from dual union all
select 38802,'CASC' from dual union all
select 38803,'CASC' from dual union all
select 38804,'CASC' from dual union all
select 38805,'CASC' from dual union all
select 38806,'CASC' from dual union all
select 38807,'CASC' from dual union all
select 38808,'CASC' from dual union all
select 38809,'CASC' from dual union all
select 3881,'RVSN' from dual union all
select 38810,'CASC' from dual union all
select 38811,'CASC' from dual union all
select 38812,'CASC' from dual union all
select 38813,'CASC' from dual union all
select 38814,'CASC' from dual union all
select 38815,'CASC' from dual union all
select 38816,'CASC' from dual union all
select 38817,'CASC' from dual union all
select 38818,'CASC' from dual union all
select 38819,'CASC' from dual union all
select 3882,'RVSN' from dual union all
select 38820,'CASC' from dual union all
select 38821,'CASC' from dual union all
select 38822,'CASC' from dual union all
select 38823,'CASC' from dual union all
select 38824,'CASC' from dual union all
select 38825,'CASC' from dual union all
select 38826,'CASC' from dual union all
select 38827,'CASC' from dual union all
select 38828,'CASC' from dual union all
select 38829,'CASC' from dual union all
select 3883,'MO' from dual union all
select 38830,'CASC' from dual union all
select 38831,'CASC' from dual union all
select 38832,'CASC' from dual union all
select 38833,'USAF' from dual union all
select 38834,'BOE' from dual union all
select 38835,'MO' from dual union all
select 38836,'KVR' from dual union all
select 38837,'KVR' from dual;
insert into SATELLITE_ORG
select 38838,'KVR' from dual union all
select 38839,'NOAA' from dual union all
select 3884,'RVSN' from dual union all
select 38840,'NOAA' from dual union all
select 38841,'NOAA' from dual union all
select 38842,'ISRO' from dual union all
select 38843,'VMF' from dual union all
select 38844,'BWEHR' from dual union all
select 38845,'VMF' from dual union all
select 38846,'SPX' from dual union all
select 38847,'ORBC' from dual union all
select 38848,'SPX' from dual union all
select 38849,'SPX' from dual union all
select 3885,'MO' from dual union all
select 38850,'SPX' from dual union all
select 38851,'SPX' from dual union all
select 38852,'WAKA' from dual union all
select 38853,'FUKU' from dual union all
select 38854,'NASA' from dual union all
select 38854,'SJSU' from dual union all
select 38855,'FPTU' from dual union all
select 38856,'MEISEI' from dual union all
select 38857,'ESA' from dual union all
select 38857,'GSAEU' from dual union all
select 38858,'ESA' from dual union all
select 38858,'GSAEU' from dual union all
select 38859,'NPOL' from dual union all
select 3886,'RVSN' from dual union all
select 38860,'GZB' from dual union all
select 38861,'GZB' from dual union all
select 38862,'CNSA' from dual union all
select 38863,'CNSA' from dual union all
select 38864,'CASC' from dual union all
select 38865,'CNSA' from dual union all
select 38866,'CNSA' from dual union all
select 38867,'INTELS' from dual union all
select 38868,'KHRR' from dual union all
select 38869,'KHRR' from dual union all
select 3887,'RVSN' from dual union all
select 38870,'CNSA' from dual union all
select 38871,'FKA' from dual union all
select 38872,'VVKO' from dual union all
select 38873,'KHRR' from dual union all
select 38874,'KHRR' from dual union all
select 38875,'KHRR' from dual union all
select 38876,'KHRR' from dual union all
select 38877,'KHRR' from dual union all
select 38878,'KHRR' from dual union all
select 38879,'KHRR' from dual union all
select 3888,'RVSN' from dual union all
select 38880,'KHRR' from dual union all
select 38881,'KHRR' from dual union all
select 38882,'KHRR' from dual union all
select 38883,'KHRR' from dual union all
select 38884,'KHRR' from dual union all
select 38885,'KHRR' from dual union all
select 38886,'KHRR' from dual union all
select 38887,'KHRR' from dual union all
select 38888,'KHRR' from dual union all
select 38889,'KHRR' from dual union all
select 3889,'USAF' from dual union all
select 38890,'KHRR' from dual union all
select 38891,'KHRR' from dual union all
select 38892,'KHRR' from dual union all
select 38893,'KHRR' from dual union all
select 38894,'KHRR' from dual union all
select 38895,'KHRR' from dual union all
select 38896,'KHRR' from dual union all
select 38897,'KHRR' from dual union all
select 38898,'KHRR' from dual union all
select 38899,'KHRR' from dual union all
select 389,'OKB1' from dual union all
select 3890,'NASA' from dual union all
select 38900,'KHRR' from dual union all
select 38901,'KHRR' from dual union all
select 38902,'KHRR' from dual union all
select 38903,'KHRR' from dual union all
select 38904,'KHRR' from dual union all
select 38905,'KHRR' from dual union all
select 38906,'KHRR' from dual union all
select 38907,'KHRR' from dual union all
select 38908,'KHRR' from dual union all
select 38909,'KHRR' from dual union all
select 3891,'USA' from dual union all
select 38910,'KHRR' from dual union all
select 38911,'KHRR' from dual union all
select 38912,'KHRR' from dual union all
select 38913,'KHRR' from dual union all
select 38914,'KHRR' from dual union all
select 38915,'KHRR' from dual union all
select 38916,'KHRR' from dual union all
select 38917,'KHRR' from dual union all
select 38918,'KHRR' from dual union all
select 38919,'KHRR' from dual union all
select 3892,'NASA' from dual union all
select 38920,'KHRR' from dual union all
select 38921,'KHRR' from dual union all
select 38922,'KHRR' from dual union all
select 38923,'KHRR' from dual union all
select 38924,'KHRR' from dual;
insert into SATELLITE_ORG
select 38925,'KHRR' from dual union all
select 38926,'KHRR' from dual union all
select 38927,'KHRR' from dual union all
select 38928,'KHRR' from dual union all
select 38929,'KHRR' from dual union all
select 3893,'MO' from dual union all
select 38930,'KHRR' from dual union all
select 38931,'KHRR' from dual union all
select 38932,'KHRR' from dual union all
select 38933,'KHRR' from dual union all
select 38934,'KHRR' from dual union all
select 38935,'KHRR' from dual union all
select 38936,'KHRR' from dual union all
select 38937,'KHRR' from dual union all
select 38938,'KHRR' from dual union all
select 38939,'KHRR' from dual union all
select 3894,'RVSN' from dual union all
select 38940,'KHRR' from dual union all
select 38941,'KHRR' from dual union all
select 38942,'KHRR' from dual union all
select 38943,'KHRR' from dual union all
select 38944,'KHRR' from dual union all
select 38945,'KHRR' from dual union all
select 38946,'KHRR' from dual union all
select 38947,'KHRR' from dual union all
select 38948,'KHRR' from dual union all
select 38949,'KHRR' from dual union all
select 3895,'NRO' from dual union all
select 38950,'KHRR' from dual union all
select 38951,'KHRR' from dual union all
select 38952,'KHRR' from dual union all
select 38953,'CNSA' from dual union all
select 38954,'CASC' from dual union all
select 38955,'KHRR' from dual union all
select 38956,'KHRR' from dual union all
select 38957,'KHRR' from dual union all
select 38958,'KHRR' from dual union all
select 38959,'KHRR' from dual union all
select 3896,'PVO' from dual union all
select 38960,'KHRR' from dual union all
select 38961,'KHRR' from dual union all
select 38962,'KHRR' from dual union all
select 38963,'KHRR' from dual union all
select 38964,'KHRR' from dual union all
select 38965,'KHRR' from dual union all
select 38966,'KHRR' from dual union all
select 38967,'KHRR' from dual union all
select 38968,'KHRR' from dual union all
select 38969,'KHRR' from dual union all
select 3897,'PVO' from dual union all
select 38970,'KHRR' from dual union all
select 38971,'KHRR' from dual union all
select 38972,'KHRR' from dual union all
select 38973,'KHRR' from dual union all
select 38974,'KHRR' from dual union all
select 38975,'FKA' from dual union all
select 38976,'VVKO' from dual union all
select 38977,'FKA' from dual union all
select 38978,'GAZS' from dual union all
select 38979,'KHRR' from dual union all
select 3898,'PVO' from dual union all
select 38980,'KHRR' from dual union all
select 38981,'KHRR' from dual union all
select 38982,'KHRR' from dual union all
select 38983,'KHRR' from dual union all
select 38984,'KHRR' from dual union all
select 38985,'KHRR' from dual union all
select 38986,'KHRR' from dual union all
select 38987,'KHRR' from dual union all
select 38988,'KHRR' from dual union all
select 38989,'KHRR' from dual union all
select 3899,'PVO' from dual union all
select 38990,'KHRR' from dual union all
select 38991,'TELB' from dual union all
select 38992,'EUTSA' from dual union all
select 38993,'AE' from dual union all
select 38994,'AE' from dual union all
select 38995,'KVR' from dual union all
select 38996,'NPOL' from dual union all
select 38997,'GZB' from dual union all
select 38998,'CASC' from dual union all
select 38999,'CASC' from dual union all
select 39,'OKB1' from dual union all
select 390,'RVSN' from dual union all
select 3900,'PVO' from dual union all
select 39000,'CNSA' from dual union all
select 39001,'CNSA' from dual union all
select 39002,'JAXA' from dual union all
select 39003,'JAXA' from dual union all
select 39004,'JAXA' from dual union all
select 39005,'JAXA' from dual union all
select 39006,'JAXA' from dual union all
select 39007,'JAXA' from dual union all
select 39008,'ECHOC' from dual union all
select 39009,'KHRR' from dual union all
select 3901,'PVO' from dual union all
select 39010,'KHRR' from dual union all
select 39011,'PLA' from dual union all
select 39012,'PLA' from dual union all
select 39013,'PLA' from dual;
insert into SATELLITE_ORG
select 39014,'CNSA' from dual union all
select 39015,'CNSA' from dual union all
select 39016,'CNSA' from dual union all
select 39017,'CASC' from dual union all
select 39018,'CASC' from dual union all
select 39019,'CNES' from dual union all
select 3902,'PVO' from dual union all
select 39020,'EUTSA' from dual union all
select 39021,'FKA' from dual union all
select 39022,'GAZS' from dual union all
select 39023,'KHRR' from dual union all
select 39024,'KHRR' from dual union all
select 39025,'USAF' from dual union all
select 39026,'KCST' from dual union all
select 39027,'KCST' from dual union all
select 39028,'KCST' from dual union all
select 39029,'KCST' from dual union all
select 3903,'PVO' from dual union all
select 39030,'MSBTR' from dual union all
select 39031,'CASC' from dual union all
select 39032,'FKA' from dual union all
select 39033,'VVKO' from dual union all
select 39034,'PARA' from dual union all
select 39035,'SCT' from dual union all
select 39036,'AE' from dual union all
select 39037,'AE' from dual union all
select 39038,'CNSA' from dual union all
select 39039,'CNSA' from dual union all
select 3904,'PVO' from dual union all
select 39040,'CNSA' from dual union all
select 39041,'USAF' from dual union all
select 39042,'USAF' from dual union all
select 39043,'USAF' from dual union all
select 39044,'USAF' from dual union all
select 39045,'USAF' from dual union all
select 39046,'USAF' from dual union all
select 39047,'KVR' from dual union all
select 39048,'KVR' from dual union all
select 39049,'KVR' from dual union all
select 3905,'RVSN' from dual union all
select 39050,'KVR' from dual union all
select 39051,'KVR' from dual union all
select 39052,'CNSA' from dual union all
select 39053,'MO' from dual union all
select 39054,'RVSNR' from dual union all
select 39055,'MAI' from dual union all
select 39056,'MO' from dual union all
select 39057,'KVR' from dual union all
select 39058,'KVR' from dual union all
select 39059,'KVR' from dual union all
select 3906,'TSKBEM' from dual union all
select 39060,'KVR' from dual union all
select 39061,'CSICE' from dual union all
select 39062,'CSICE' from dual union all
select 39063,'JAXA' from dual union all
select 39064,'JAXA' from dual union all
select 39065,'JAXA' from dual union all
select 39066,'JAXA' from dual union all
select 39067,'JAXA' from dual union all
select 39068,'KARI' from dual union all
select 39069,'KARI' from dual union all
select 3907,'RVSN' from dual union all
select 39070,'NASA' from dual union all
select 39071,'LM' from dual union all
select 39072,'GLOB' from dual union all
select 39073,'GLOB' from dual union all
select 39074,'GLOB' from dual union all
select 39075,'GLOB' from dual union all
select 39076,'GLOB' from dual union all
select 39077,'GLOB' from dual union all
select 39078,'HISP' from dual union all
select 39079,'AZRITN' from dual union all
select 3908,'PVO' from dual union all
select 39080,'AE' from dual union all
select 39081,'AE' from dual union all
select 39082,'FKA' from dual union all
select 39083,'VVKO' from dual union all
select 39084,'NASA' from dual union all
select 39085,'LM' from dual union all
select 39086,'CNES' from dual union all
select 39086,'ISRO' from dual union all
select 39087,'AAU' from dual union all
select 39088,'DND' from dual union all
select 39088,'DRDC' from dual union all
select 39089,'CSA' from dual union all
select 3909,'PVO' from dual union all
select 39090,'SURREY' from dual union all
select 39091,'TUG' from dual union all
select 39092,'UTIAS0' from dual union all
select 39092,'UVIE' from dual union all
select 39093,'ISRO' from dual union all
select 39094,'ISRO' from dual union all
select 39095,'NASA' from dual union all
select 39096,'NASA' from dual union all
select 39097,'NASA' from dual union all
select 39098,'NASA' from dual union all
select 39099,'NASA' from dual union all
select 391,'RVSN' from dual union all
select 3910,'PVO' from dual union all
select 39100,'NASA' from dual;
insert into SATELLITE_ORG
select 39101,'NASA' from dual union all
select 39102,'NASA' from dual union all
select 39103,'AE' from dual union all
select 39104,'AE' from dual union all
select 39105,'AE' from dual union all
select 39106,'AE' from dual union all
select 39107,'AE' from dual union all
select 39108,'AE' from dual union all
select 39109,'KVR' from dual union all
select 3911,'PVO' from dual union all
select 39110,'KVR' from dual union all
select 39111,'KVR' from dual union all
select 39112,'CASC' from dual union all
select 39113,'CASC' from dual union all
select 39114,'CASC' from dual union all
select 39115,'SPX' from dual union all
select 39116,'SPX' from dual union all
select 39117,'SPX' from dual union all
select 39118,'SPX' from dual union all
select 39119,'FKA' from dual union all
select 3912,'PVO' from dual union all
select 39120,'USAF' from dual union all
select 39121,'LM' from dual union all
select 39122,'SAMX' from dual union all
select 39123,'KHRR' from dual union all
select 39124,'KHRR' from dual union all
select 39125,'FKA' from dual union all
select 39126,'VVKO' from dual union all
select 39127,'TCAN' from dual union all
select 39128,'KHRR' from dual union all
select 39129,'KHRR' from dual union all
select 3913,'RVSN' from dual union all
select 39130,'FKA' from dual union all
select 39131,'OSSI' from dual union all
select 39132,'COSMOG' from dual union all
select 39133,'SGAU' from dual union all
select 39134,'TUD' from dual union all
select 39135,'TUB1' from dual union all
select 39136,'TUB1' from dual union all
select 39137,'RVSNR' from dual union all
select 39138,'FKA' from dual union all
select 39139,'FKA' from dual union all
select 3914,'CIA' from dual union all
select 3914,'NRO' from dual union all
select 39140,'FKA' from dual union all
select 39141,'FKA' from dual union all
select 39142,'NASA' from dual union all
select 39143,'COSMOG' from dual union all
select 39144,'NASA' from dual union all
select 39145,'OSC' from dual union all
select 39146,'NASA' from dual union all
select 39147,'OSC' from dual union all
select 39148,'FKA' from dual union all
select 39149,'VVKO' from dual union all
select 3915,'NRO' from dual union all
select 39150,'CNSAS' from dual union all
select 39151,'EXA' from dual union all
select 39152,'ITUTR' from dual union all
select 39153,'MINCYT' from dual union all
select 39154,'CASC' from dual union all
select 39155,'FKA' from dual union all
select 39155,'KVR' from dual union all
select 39156,'NPOL' from dual union all
select 39157,'PLA' from dual union all
select 39158,'CASC' from dual union all
select 39159,'ESA' from dual union all
select 3916,'ESRO' from dual union all
select 39160,'VAST' from dual union all
select 39161,'ESSP' from dual union all
select 39162,'ESA' from dual union all
select 39163,'EUTSA' from dual union all
select 39164,'KHRR' from dual union all
select 39165,'KHRR' from dual union all
select 39166,'USAF' from dual union all
select 39167,'LM' from dual union all
select 39168,'USAF' from dual union all
select 39169,'BOE' from dual union all
select 3917,'RVSN' from dual union all
select 39170,'FKA' from dual union all
select 39171,'VVKO' from dual union all
select 39172,'SES' from dual union all
select 39173,'KHRR' from dual union all
select 39174,'KHRR' from dual union all
select 39175,'ESA' from dual union all
select 39176,'AE' from dual union all
select 39177,'KVR' from dual union all
select 39178,'KVR' from dual union all
select 39179,'PLA' from dual union all
select 3918,'RVSN' from dual union all
select 39180,'CASC' from dual union all
select 39181,'CASC' from dual union all
select 39182,'CASC' from dual union all
select 39183,'CASC' from dual union all
select 39184,'CASC' from dual union all
select 39185,'CASC' from dual union all
select 39186,'TSSKB' from dual union all
select 39187,'KVR' from dual union all
select 39188,'O3B' from dual union all
select 39189,'O3B' from dual union all
select 3919,'MO' from dual;
insert into SATELLITE_ORG
select 39190,'O3B' from dual union all
select 39191,'O3B' from dual union all
select 39192,'NPOL' from dual union all
select 39193,'PLA' from dual union all
select 39194,'NPOMA' from dual union all
select 39195,'NPOMA' from dual union all
select 39196,'FKA' from dual union all
select 39197,'NASA' from dual union all
select 39198,'OSC' from dual union all
select 39199,'ISRO' from dual union all
select 392,'OKB1' from dual union all
select 3920,'MO' from dual union all
select 39200,'ISRO' from dual union all
select 39201,'CNES' from dual union all
select 39202,'CNSA' from dual union all
select 39203,'CASC' from dual union all
select 39204,'CASC' from dual union all
select 39205,'CASC' from dual union all
select 39206,'USN' from dual union all
select 39207,'LM' from dual union all
select 39208,'CNSA' from dual union all
select 39209,'SIMIT' from dual union all
select 3921,'RVSN' from dual union all
select 39210,'CNSA' from dual union all
select 39211,'CNSA' from dual union all
select 39212,'CASC' from dual union all
select 39213,'CASC' from dual union all
select 39214,'CASC' from dual union all
select 39215,'ESA' from dual union all
select 39215,'INMRL' from dual union all
select 39216,'ISRO' from dual union all
select 39217,'AE' from dual union all
select 39218,'AE' from dual union all
select 39219,'FKA' from dual union all
select 3922,'NASA' from dual union all
select 39220,'VVKO' from dual union all
select 39221,'JAXA' from dual union all
select 39222,'USAF' from dual union all
select 39223,'BOE' from dual union all
select 39224,'CASC' from dual union all
select 39225,'CASC' from dual union all
select 39226,'TSSKB' from dual union all
select 39227,'KARI' from dual union all
select 39228,'KTRAS' from dual union all
select 39229,'KTRAS' from dual union all
select 3923,'PVO' from dual union all
select 39230,'ESA' from dual union all
select 39231,'CASC' from dual union all
select 39232,'NROC' from dual union all
select 39233,'EUTSA' from dual union all
select 39233,'ICTQ' from dual union all
select 39234,'INAV' from dual union all
select 39234,'ISRO' from dual union all
select 39235,'AE' from dual union all
select 39236,'AE' from dual union all
select 39237,'IAI' from dual union all
select 39238,'SIS' from dual union all
select 39239,'PLA' from dual union all
select 3924,'RVSN' from dual union all
select 39240,'PLA' from dual union all
select 39241,'PLA' from dual union all
select 39242,'CNSA' from dual union all
select 39243,'CNSA' from dual union all
select 39244,'CNSA' from dual union all
select 39245,'RKA' from dual union all
select 39246,'NASA' from dual union all
select 39247,'USN' from dual union all
select 39248,'OSC' from dual union all
select 39249,'KVR' from dual union all
select 3925,'USAF' from dual union all
select 39250,'KVR' from dual union all
select 39251,'KVR' from dual union all
select 39252,'KVR' from dual union all
select 39253,'JAXA' from dual union all
select 39254,'JAXA' from dual union all
select 39255,'JAXA' from dual union all
select 39256,'USAF' from dual union all
select 39257,'LM' from dual union all
select 39258,'OSC' from dual union all
select 39259,'OSC' from dual union all
select 3926,'USAF' from dual union all
select 39260,'CMA' from dual union all
select 39261,'CNSA' from dual union all
select 39262,'GYZ' from dual union all
select 39263,'FKA' from dual union all
select 39264,'VVKO' from dual union all
select 39265,'CSA' from dual union all
select 39266,'CORN' from dual union all
select 39267,'UCO' from dual union all
select 39268,'POPACS' from dual union all
select 39269,'POPACS' from dual union all
select 3927,'NASA' from dual union all
select 39270,'POPACS' from dual union all
select 39271,'SPX' from dual union all
select 39272,'SPX' from dual union all
select 39273,'SPX' from dual union all
select 39274,'SPX' from dual union all
select 39275,'SPX' from dual union all
select 39276,'SPX' from dual union all
select 39277,'SPX' from dual;
insert into SATELLITE_ORG
select 39278,'SPX' from dual union all
select 39279,'SPX' from dual union all
select 3928,'USAF' from dual union all
select 39280,'POPACS' from dual union all
select 39281,'SPX' from dual union all
select 39282,'SPX' from dual union all
select 39283,'SPX' from dual union all
select 39284,'SPX' from dual union all
select 39285,'SES' from dual union all
select 39286,'KHRR' from dual union all
select 39287,'KHRR' from dual union all
select 39288,'SPX' from dual union all
select 39289,'SPX' from dual union all
select 3929,'USAF' from dual union all
select 39290,'POPACS' from dual union all
select 39291,'SPX' from dual union all
select 39292,'POPACS' from dual union all
select 39293,'POPACS' from dual union all
select 39294,'NASA' from dual union all
select 39295,'NASA' from dual union all
select 39296,'USAF' from dual union all
select 39297,'USAF' from dual union all
select 39298,'USAF' from dual union all
select 39299,'USAF' from dual union all
select 393,'OKB1' from dual union all
select 3930,'USAF' from dual union all
select 39300,'RVSNR' from dual union all
select 39301,'RVSNR' from dual union all
select 39302,'RVSNR' from dual union all
select 39303,'RVSNR' from dual union all
select 39304,'RVSNR' from dual union all
select 39305,'RVSNR' from dual union all
select 39306,'OSC' from dual union all
select 39307,'OSC' from dual union all
select 39308,'OSC' from dual union all
select 39309,'OSC' from dual union all
select 3931,'RVSN' from dual union all
select 39310,'OSC' from dual union all
select 39311,'OSC' from dual union all
select 39312,'OSC' from dual union all
select 39313,'OSC' from dual union all
select 39314,'OSC' from dual union all
select 39315,'OSC' from dual union all
select 39316,'OSC' from dual union all
select 39317,'OSC' from dual union all
select 39318,'OSC' from dual union all
select 39319,'OSC' from dual union all
select 3932,'USAF' from dual union all
select 39320,'OSC' from dual union all
select 39321,'OSC' from dual union all
select 39322,'OSC' from dual union all
select 39323,'OSC' from dual union all
select 39324,'KHRR' from dual union all
select 39325,'KHRR' from dual union all
select 39326,'KHRR' from dual union all
select 39327,'KHRR' from dual union all
select 39328,'KHRR' from dual union all
select 39329,'KHRR' from dual union all
select 3933,'RVSN' from dual union all
select 39330,'KHRR' from dual union all
select 39331,'KHRR' from dual union all
select 39332,'CASC' from dual union all
select 39333,'CASC' from dual union all
select 39334,'CASC' from dual union all
select 39335,'CASC' from dual union all
select 39336,'CASC' from dual union all
select 39337,'CASC' from dual union all
select 39338,'CASC' from dual union all
select 39339,'CASC' from dual union all
select 3934,'PVO' from dual union all
select 39340,'CASC' from dual union all
select 39341,'CASC' from dual union all
select 39342,'CASC' from dual union all
select 39343,'CASC' from dual union all
select 39344,'CASC' from dual union all
select 39345,'CASC' from dual union all
select 39346,'CASC' from dual union all
select 39347,'CASC' from dual union all
select 39348,'CASC' from dual union all
select 39349,'CASC' from dual union all
select 3935,'CNES' from dual union all
select 39350,'CASC' from dual union all
select 39351,'CASC' from dual union all
select 39352,'CASC' from dual union all
select 39353,'CASC' from dual union all
select 39354,'CASC' from dual union all
select 39355,'CASC' from dual union all
select 39356,'CASC' from dual union all
select 39357,'CNSA' from dual union all
select 39358,'CMA' from dual union all
select 39359,'CNSA' from dual union all
select 3936,'USAF' from dual union all
select 39360,'SIRX' from dual union all
select 39361,'KHRR' from dual union all
select 39362,'KHRR' from dual union all
select 39363,'PLA' from dual union all
select 39364,'CNSA' from dual union all
select 39365,'CNSA' from dual union all
select 39366,'CNSA' from dual union all
select 39367,'CNSA' from dual;
insert into SATELLITE_ORG
select 39368,'CNSA' from dual union all
select 39369,'UCO' from dual union all
select 3937,'RVSN' from dual union all
select 39370,'ISRO' from dual union all
select 39371,'ISRO' from dual union all
select 39372,'CNSA' from dual union all
select 39373,'FKA' from dual union all
select 39374,'VVKO' from dual union all
select 39375,'KVR' from dual union all
select 39376,'KHRR' from dual union all
select 39377,'KHRR' from dual union all
select 39378,'NASA' from dual union all
select 39379,'LM' from dual union all
select 3938,'PVO' from dual union all
select 39380,'USAF' from dual union all
select 39381,'NASA' from dual union all
select 39382,'ULLAF' from dual union all
select 39383,'DREXEL' from dual union all
select 39383,'USN' from dual union all
select 39384,'KYSP' from dual union all
select 39385,'TJHS' from dual union all
select 39386,'SMDC' from dual union all
select 39387,'DOD' from dual union all
select 39387,'USAF' from dual union all
select 39388,'USAF' from dual union all
select 39389,'USN' from dual union all
select 3939,'MO' from dual union all
select 39390,'DOD' from dual union all
select 39390,'LANL' from dual union all
select 39391,'DOD' from dual union all
select 39391,'LANL' from dual union all
select 39392,'USAF' from dual union all
select 39393,'DOD' from dual union all
select 39393,'LANL' from dual union all
select 39394,'DOD' from dual union all
select 39394,'LANL' from dual union all
select 39395,'SLU' from dual union all
select 39396,'DOD' from dual union all
select 39396,'USAF' from dual union all
select 39397,'LLNL' from dual union all
select 39397,'NRO' from dual union all
select 39398,'USA' from dual union all
select 39399,'DOD' from dual union all
select 39399,'LANL' from dual union all
select 394,'OKB1' from dual union all
select 3940,'RVSN' from dual union all
select 39400,'UNMC' from dual union all
select 39401,'DOD' from dual union all
select 39401,'LANL' from dual union all
select 39402,'UFL' from dual union all
select 39403,'UHI' from dual union all
select 39404,'NASA' from dual union all
select 39404,'NSF' from dual union all
select 39405,'UAH' from dual union all
select 39406,'DOD' from dual union all
select 39406,'LANL' from dual union all
select 39407,'VTC' from dual union all
select 39408,'DOD' from dual union all
select 39408,'LANL' from dual union all
select 39409,'USAF' from dual union all
select 3941,'NASA' from dual union all
select 39410,'PLA' from dual union all
select 39411,'CNSA' from dual union all
select 39412,'NANSFI' from dual union all
select 39413,'VAST' from dual union all
select 39414,'NANSFI' from dual union all
select 39415,'NASA' from dual union all
select 39415,'SJSU' from dual union all
select 39416,'SPQ' from dual union all
select 39417,'CPUT' from dual union all
select 39418,'SKYBOX' from dual union all
select 39419,'EIAST' from dual union all
select 3942,'CIA' from dual union all
select 3942,'NRO' from dual union all
select 39420,'INTA' from dual union all
select 39421,'GAUSS' from dual union all
select 39422,'KARI' from dual union all
select 39423,'WNI' from dual union all
select 39424,'KHUS' from dual union all
select 39425,'SPQ' from dual union all
select 39426,'KHUS' from dual union all
select 39427,'ISISBV' from dual union all
select 39428,'DELFT' from dual union all
select 39429,'PLABS' from dual union all
select 3943,'NASA' from dual union all
select 39430,'GOMSP' from dual union all
select 39431,'CBKPL' from dual union all
select 39432,'ISTPK' from dual union all
select 39433,'VIGO' from dual union all
select 39434,'PUCP' from dual union all
select 39435,'STADK' from dual union all
select 39436,'MSUKY' from dual union all
select 39437,'MSUKY' from dual union all
select 39438,'NTU' from dual union all
select 39439,'TUM' from dual union all
select 3944,'MO' from dual union all
select 39440,'MINCYT' from dual union all
select 39441,'EXA' from dual union all
select 39442,'PUCP' from dual union all
select 39443,'UMBC' from dual;
insert into SATELLITE_ORG
select 39444,'AMSAT' from dual union all
select 39444,'AMUK' from dual union all
select 39445,'HIN' from dual union all
select 39446,'WURZ' from dual union all
select 39447,'PVO' from dual union all
select 39448,'KTRAS' from dual union all
select 39449,'KTRAS' from dual union all
select 3945,'RVSN' from dual union all
select 39450,'KTRAS' from dual union all
select 39451,'ESA' from dual union all
select 39452,'ESA' from dual union all
select 39453,'ESA' from dual union all
select 39454,'EUROK' from dual union all
select 39455,'GZB' from dual union all
select 39456,'FKA' from dual union all
select 39457,'VVKO' from dual union all
select 39458,'CASC' from dual union all
select 39459,'CASC' from dual union all
select 3946,'PVO' from dual union all
select 39460,'SES' from dual union all
select 39461,'SPX' from dual union all
select 39462,'NROC' from dual union all
select 39463,'MSU' from dual union all
select 39464,'MSU' from dual union all
select 39465,'AERO' from dual union all
select 39466,'AERO' from dual union all
select 39467,'USAF' from dual union all
select 39468,'SMDC' from dual union all
select 39469,'JPL' from dual union all
select 39469,'UMI' from dual union all
select 3947,'INTEL' from dual union all
select 39470,'CUNYME' from dual union all
select 39471,'CALP' from dual union all
select 39471,'JPL' from dual union all
select 39472,'SMDC' from dual union all
select 39473,'SMDC' from dual union all
select 39474,'SMDC' from dual union all
select 39475,'CALP' from dual union all
select 39475,'NROC' from dual union all
select 39476,'INMRL' from dual union all
select 39477,'KHRR' from dual union all
select 39478,'ESA' from dual union all
select 39479,'ESA' from dual union all
select 3948,'NASA' from dual union all
select 39480,'NPOL' from dual union all
select 39481,'ABE' from dual union all
select 39482,'CASC' from dual union all
select 39483,'KVR' from dual union all
select 39484,'KVR' from dual union all
select 39485,'KVR' from dual union all
select 39486,'KVR' from dual union all
select 39487,'KS' from dual union all
select 39488,'KHRR' from dual union all
select 39489,'KHRR' from dual union all
select 3949,'NASA' from dual union all
select 39490,'PVO' from dual union all
select 39491,'PVO' from dual union all
select 39492,'SGAU' from dual union all
select 39493,'RVSNR' from dual union all
select 39494,'KHRR' from dual union all
select 39495,'URTHE' from dual union all
select 39496,'FKA' from dual union all
select 39497,'FKA' from dual union all
select 39498,'ISRO' from dual union all
select 39499,'ISRO' from dual union all
select 395,'OKB1' from dual union all
select 3950,'USAF' from dual union all
select 39500,'THAIC' from dual union all
select 39501,'SPX' from dual union all
select 39502,'OSC' from dual union all
select 39503,'OSC' from dual union all
select 39504,'NASA' from dual union all
select 39505,'LM' from dual union all
select 39506,'FKA' from dual union all
select 39507,'VVKO' from dual union all
select 39508,'ABS' from dual union all
select 39509,'ASI' from dual union all
select 39509,'CNES' from dual union all
select 3951,'USAF' from dual union all
select 39510,'AE' from dual union all
select 39511,'AE' from dual union all
select 39512,'PLABS' from dual union all
select 39513,'PLABS' from dual union all
select 39514,'PLABS' from dual union all
select 39515,'PLABS' from dual union all
select 39516,'KVR' from dual union all
select 39517,'KVR' from dual union all
select 39518,'PLABS' from dual union all
select 39519,'PLABS' from dual union all
select 3952,'USAF' from dual union all
select 39520,'PLABS' from dual union all
select 39521,'PLABS' from dual union all
select 39522,'TURKS' from dual union all
select 39523,'KHRR' from dual union all
select 39524,'KHRR' from dual union all
select 39525,'PLABS' from dual union all
select 39526,'PLABS' from dual union all
select 39527,'PLABS' from dual union all
select 39528,'PLABS' from dual union all
select 39529,'PLABS' from dual;
insert into SATELLITE_ORG
select 3953,'PVO' from dual union all
select 39530,'PLABS' from dual union all
select 39531,'PLABS' from dual union all
select 39532,'PLABS' from dual union all
select 39533,'USAF' from dual union all
select 39534,'BOE' from dual union all
select 39535,'CASC' from dual union all
select 39536,'CASC' from dual union all
select 39537,'CASC' from dual union all
select 39538,'CASC' from dual union all
select 39539,'CASC' from dual union all
select 3954,'USAF' from dual union all
select 39540,'CASC' from dual union all
select 39541,'CASC' from dual union all
select 39542,'CASC' from dual union all
select 39543,'CASC' from dual union all
select 39544,'CASC' from dual union all
select 39545,'KVR' from dual union all
select 39546,'KVR' from dual union all
select 39547,'KVR' from dual union all
select 39548,'KVR' from dual union all
select 39549,'KVR' from dual union all
select 3955,'USAF' from dual union all
select 39550,'KVR' from dual union all
select 39551,'KVR' from dual union all
select 39552,'KVR' from dual union all
select 39553,'KVR' from dual union all
select 39554,'KVR' from dual union all
select 39555,'PLABS' from dual union all
select 39556,'PLABS' from dual union all
select 39557,'PLABS' from dual union all
select 39558,'PLABS' from dual union all
select 39559,'PLABS' from dual union all
select 3956,'USAF' from dual union all
select 39560,'PLABS' from dual union all
select 39561,'PLABS' from dual union all
select 39562,'PLABS' from dual union all
select 39563,'PLABS' from dual union all
select 39564,'PLABS' from dual union all
select 39565,'NANSFI' from dual union all
select 39566,'UAP' from dual union all
select 39567,'SSTARS' from dual union all
select 39568,'KTU' from dual union all
select 39569,'LKA' from dual union all
select 39569,'VILN' from dual union all
select 3957,'PVO' from dual union all
select 39570,'PLABS' from dual union all
select 39571,'PLABS' from dual union all
select 39572,'SHINSHU' from dual union all
select 39573,'TSUKU' from dual union all
select 39574,'NASA' from dual union all
select 39575,'OPU' from dual union all
select 39576,'TEIK' from dual union all
select 39577,'TAMA' from dual union all
select 39578,'KAGO' from dual union all
select 39579,'KAGA' from dual union all
select 3958,'RVSN' from dual union all
select 39580,'JAXA' from dual union all
select 39581,'JAXA' from dual union all
select 39582,'KVR' from dual union all
select 39583,'KVR' from dual union all
select 39584,'KVR' from dual union all
select 39585,'KVR' from dual union all
select 39586,'KVR' from dual union all
select 39587,'KVR' from dual union all
select 39588,'KVR' from dual union all
select 39589,'KVR' from dual union all
select 3959,'PVO' from dual union all
select 39590,'KVR' from dual union all
select 39591,'KVR' from dual union all
select 39592,'NASA' from dual union all
select 39593,'IRIDS' from dual union all
select 39594,'IRIDS' from dual union all
select 39595,'IRIDS' from dual union all
select 39596,'CNSA' from dual union all
select 39597,'ISRO' from dual union all
select 39598,'CNSA' from dual union all
select 39599,'KHRR' from dual union all
select 396,'CIA' from dual union all
select 396,'NRO' from dual union all
select 3960,'PVO' from dual union all
select 39600,'CASC' from dual union all
select 39601,'KHRR' from dual union all
select 39602,'KVR' from dual union all
select 39603,'KVR' from dual union all
select 39604,'KVR' from dual union all
select 39605,'KVR' from dual union all
select 39606,'KVR' from dual union all
select 39607,'KVR' from dual union all
select 39608,'KVR' from dual union all
select 39609,'USAF' from dual union all
select 3961,'PVO' from dual union all
select 39610,'ESA' from dual union all
select 39611,'KVR' from dual union all
select 39612,'KS' from dual union all
select 39613,'KS' from dual union all
select 39614,'KHRR' from dual union all
select 39615,'KHRR' from dual union all
select 39616,'HISP' from dual union all
select 39617,'SES' from dual;
insert into SATELLITE_ORG
select 39618,'AE' from dual union all
select 39619,'AE' from dual union all
select 3962,'PVO' from dual union all
select 39620,'FKA' from dual union all
select 39620,'KVR' from dual union all
select 39621,'NPOL' from dual union all
select 39622,'FKA' from dual union all
select 39623,'VVKO' from dual union all
select 39624,'CNSA' from dual union all
select 39625,'CASC' from dual union all
select 39626,'CASC' from dual union all
select 39627,'CASC' from dual union all
select 39628,'CASC' from dual union all
select 39629,'CASC' from dual union all
select 3963,'RVSN' from dual union all
select 39630,'USAF' from dual union all
select 39631,'LM' from dual union all
select 39632,'USAF' from dual union all
select 39633,'USAF' from dual union all
select 39634,'ESA' from dual union all
select 39635,'ISRO' from dual union all
select 39636,'ISRO' from dual union all
select 39637,'VMF' from dual union all
select 39638,'VMF' from dual union all
select 39639,'VMF' from dual union all
select 3964,'RVSN' from dual union all
select 39640,'VMF' from dual union all
select 39641,'VMF' from dual union all
select 39642,'VMF' from dual union all
select 39643,'VMF' from dual union all
select 39644,'VMF' from dual union all
select 39645,'VMF' from dual union all
select 39646,'VMF' from dual union all
select 39647,'VMF' from dual union all
select 39648,'FKA' from dual union all
select 39649,'VVKO' from dual union all
select 3965,'RVSN' from dual union all
select 39650,'IDF' from dual union all
select 39651,'ISA' from dual union all
select 39652,'NROC' from dual union all
select 39653,'LM' from dual union all
select 39654,'VMF' from dual union all
select 39655,'VMF' from dual union all
select 39656,'VMF' from dual union all
select 39657,'VMF' from dual union all
select 39658,'VMF' from dual union all
select 39659,'VMF' from dual union all
select 3966,'RVSN' from dual union all
select 39660,'VMF' from dual union all
select 39661,'VMF' from dual union all
select 39662,'VMF' from dual union all
select 39663,'VMF' from dual union all
select 39664,'VMF' from dual union all
select 39665,'VMF' from dual union all
select 39666,'VMF' from dual union all
select 39667,'VMF' from dual union all
select 39668,'VMF' from dual union all
select 39669,'VMF' from dual union all
select 3967,'RVSN' from dual union all
select 39670,'VMF' from dual union all
select 39671,'VMF' from dual union all
select 39672,'VMF' from dual union all
select 39673,'VMF' from dual union all
select 39674,'VMF' from dual union all
select 39675,'VMF' from dual union all
select 39676,'VMF' from dual union all
select 39677,'VMF' from dual union all
select 39678,'NARSS' from dual union all
select 39679,'VVKO' from dual union all
select 3968,'USAF' from dual union all
select 39680,'SPX' from dual union all
select 39681,'NASA' from dual union all
select 39682,'TAYLU' from dual union all
select 39683,'UCO' from dual union all
select 39684,'NASA' from dual union all
select 39685,'CORN' from dual union all
select 39685,'NASA' from dual union all
select 39686,'SPX' from dual union all
select 39687,'SPX' from dual union all
select 39688,'CRC' from dual union all
select 39689,'CRC' from dual union all
select 3969,'USAF' from dual union all
select 39690,'YUZH' from dual union all
select 39691,'RVSN' from dual union all
select 39692,'RVSN' from dual union all
select 39693,'RVSN' from dual union all
select 39694,'RVSN' from dual union all
select 39695,'RVSN' from dual union all
select 39696,'RVSN' from dual union all
select 39697,'RVSN' from dual union all
select 39698,'RVSN' from dual union all
select 39699,'RVSN' from dual union all
select 397,'NASA' from dual union all
select 3970,'CNES' from dual union all
select 39700,'RVSN' from dual union all
select 39701,'RVSN' from dual union all
select 39702,'RVSN' from dual union all
select 39703,'RVSN' from dual union all
select 39704,'AE' from dual union all
select 39705,'AE' from dual;
insert into SATELLITE_ORG
select 39706,'AE' from dual union all
select 39707,'RVSNR' from dual union all
select 39708,'RVSNR' from dual union all
select 39709,'RVSNR' from dual union all
select 3971,'MO' from dual union all
select 39710,'RVSNR' from dual union all
select 39711,'RVSNR' from dual union all
select 39712,'RVSNR' from dual union all
select 39713,'RVSNR' from dual union all
select 39714,'RVSNR' from dual union all
select 39715,'RVSNR' from dual union all
select 39716,'RVSNR' from dual union all
select 39717,'RVSNR' from dual union all
select 39718,'RVSNR' from dual union all
select 39719,'RVSNR' from dual union all
select 3972,'RVSN' from dual union all
select 39720,'RVSNR' from dual union all
select 39721,'RVSNR' from dual union all
select 39722,'RVSNR' from dual union all
select 39723,'RVSNR' from dual union all
select 39724,'RVSNR' from dual union all
select 39725,'RVSNR' from dual union all
select 39726,'RVSNR' from dual union all
select 39727,'FKA' from dual union all
select 39728,'KAZK' from dual union all
select 39729,'KHRR' from dual union all
select 3973,'MO' from dual union all
select 39730,'KHRR' from dual union all
select 39731,'KAZK' from dual union all
select 39732,'KVR' from dual union all
select 39733,'RVSNR' from dual union all
select 39734,'BOE' from dual union all
select 39735,'BOE' from dual union all
select 39736,'BOE' from dual union all
select 39737,'BOE' from dual union all
select 39738,'BOE' from dual union all
select 39739,'BOE' from dual union all
select 3974,'NASA' from dual union all
select 39740,'BOE' from dual union all
select 39741,'USAF' from dual union all
select 39742,'BOE' from dual union all
select 39743,'MORF' from dual union all
select 39744,'MORF' from dual union all
select 39745,'MORF' from dual union all
select 39746,'MORF' from dual union all
select 39747,'MORF' from dual union all
select 39748,'MORF' from dual union all
select 39749,'MORF' from dual union all
select 3975,'NASA' from dual union all
select 39750,'MORF' from dual union all
select 39751,'NROC' from dual union all
select 39752,'PVO' from dual union all
select 39753,'PVO' from dual union all
select 39754,'USAF' from dual union all
select 39755,'USAF' from dual union all
select 39756,'USAF' from dual union all
select 39757,'USAF' from dual union all
select 39758,'USAF' from dual union all
select 39759,'USAF' from dual union all
select 3976,'RVSN' from dual union all
select 39760,'USAF' from dual union all
select 39761,'KVR' from dual union all
select 39762,'KVR' from dual union all
select 39763,'KVR' from dual union all
select 39764,'KVR' from dual union all
select 39765,'FKA' from dual union all
select 39766,'NASDA' from dual union all
select 39767,'WAKA' from dual union all
select 39768,'KASH' from dual union all
select 39769,'TOHO' from dual union all
select 3977,'RVSN' from dual union all
select 39770,'NIHON' from dual union all
select 39771,'JAXA' from dual union all
select 39772,'JAXA' from dual union all
select 39773,'EUTSA' from dual union all
select 39774,'FKA' from dual union all
select 39775,'FKA' from dual union all
select 39776,'VVKO' from dual union all
select 39777,'IRIDS' from dual union all
select 39778,'IRIDS' from dual union all
select 39779,'IRIDS' from dual union all
select 3978,'USAF' from dual union all
select 39780,'IRIDS' from dual union all
select 39781,'IRIDS' from dual union all
select 39782,'IRIDS' from dual union all
select 39783,'IRIDS' from dual union all
select 39784,'IRIDS' from dual union all
select 39785,'IRIDS' from dual union all
select 39786,'IRIDS' from dual union all
select 39787,'IRIDS' from dual union all
select 39788,'IRIDS' from dual union all
select 39789,'IRIDS' from dual union all
select 3979,'PVO' from dual union all
select 39790,'IRIDS' from dual union all
select 39791,'IRIDS' from dual union all
select 39792,'IRIDS' from dual union all
select 39793,'IRIDS' from dual union all
select 39794,'IRIDS' from dual union all
select 39795,'IRIDS' from dual union all
select 39796,'IRIDS' from dual;
insert into SATELLITE_ORG
select 39797,'KVR' from dual union all
select 39798,'KVR' from dual union all
select 39799,'KVR' from dual union all
select 398,'NASA' from dual union all
select 3980,'USAF' from dual union all
select 39800,'KVR' from dual union all
select 39801,'KVR' from dual union all
select 39802,'KVR' from dual union all
select 39803,'KVR' from dual union all
select 39804,'KVR' from dual union all
select 39805,'RVSN' from dual union all
select 39806,'RVSN' from dual union all
select 39807,'RVSN' from dual union all
select 39808,'RVSN' from dual union all
select 39809,'RVSN' from dual union all
select 3981,'USAF' from dual union all
select 39810,'RVSN' from dual union all
select 39811,'RVSN' from dual union all
select 39812,'RVSN' from dual union all
select 39813,'RVSN' from dual union all
select 39814,'RVSN' from dual union all
select 39815,'RVSN' from dual union all
select 39816,'RVSN' from dual union all
select 39817,'RVSN' from dual union all
select 39818,'RVSN' from dual union all
select 39819,'RVSN' from dual union all
select 3982,'USAF' from dual union all
select 39820,'RVSN' from dual union all
select 39821,'RVSN' from dual union all
select 39822,'RVSN' from dual union all
select 39823,'RVSN' from dual union all
select 39824,'RVSN' from dual union all
select 39825,'RVSN' from dual union all
select 39826,'RVSN' from dual union all
select 39827,'RVSN' from dual union all
select 39828,'RVSN' from dual union all
select 39829,'RVSN' from dual union all
select 3983,'PVO' from dual union all
select 39830,'RVSN' from dual union all
select 39831,'RVSN' from dual union all
select 39832,'RVSN' from dual union all
select 39833,'RVSN' from dual union all
select 39834,'RVSN' from dual union all
select 39835,'RVSN' from dual union all
select 39836,'RVSN' from dual union all
select 39837,'RVSN' from dual union all
select 39838,'RVSN' from dual union all
select 39839,'RVSN' from dual union all
select 3984,'NRO' from dual union all
select 39840,'RVSN' from dual union all
select 39841,'RVSNR' from dual union all
select 39842,'RVSNR' from dual union all
select 39843,'RVSNR' from dual union all
select 39844,'RVSNR' from dual union all
select 39845,'RVSNR' from dual union all
select 39846,'RVSNR' from dual union all
select 39847,'RVSNR' from dual union all
select 39848,'RVSNR' from dual union all
select 39849,'RVSNR' from dual union all
select 3985,'RVSN' from dual union all
select 39850,'RVSNR' from dual union all
select 39851,'RVSNR' from dual union all
select 39852,'RVSNR' from dual union all
select 39853,'RVSNR' from dual union all
select 39854,'RVSNR' from dual union all
select 39855,'RVSNR' from dual union all
select 39856,'RVSNR' from dual union all
select 39857,'RVSNR' from dual union all
select 39858,'RVSNR' from dual union all
select 39859,'RVSNR' from dual union all
select 3986,'NASA' from dual union all
select 39860,'RVSNR' from dual union all
select 39861,'RVSNR' from dual union all
select 39862,'RVSNR' from dual union all
select 39863,'RVSNR' from dual union all
select 39864,'RVSNR' from dual union all
select 39865,'RVSNR' from dual union all
select 39866,'RVSNR' from dual union all
select 39867,'RVSNR' from dual union all
select 39868,'RVSNR' from dual union all
select 39869,'RVSNR' from dual union all
select 3987,'NASA' from dual union all
select 39870,'RVSNR' from dual union all
select 39871,'RVSNR' from dual union all
select 39872,'RVSNR' from dual union all
select 39873,'RVSNR' from dual union all
select 39874,'RVSNR' from dual union all
select 39875,'RVSNR' from dual union all
select 39876,'RVSNR' from dual union all
select 39877,'RVSNR' from dual union all
select 39878,'RVSNR' from dual union all
select 39879,'RVSNR' from dual union all
select 3988,'MO' from dual union all
select 39880,'RVSNR' from dual union all
select 39881,'RVSNR' from dual union all
select 39882,'RVSNR' from dual union all
select 39883,'RVSNR' from dual union all
select 39884,'RVSNR' from dual union all
select 39885,'RVSNR' from dual union all
select 39886,'RVSNR' from dual;
insert into SATELLITE_ORG
select 39887,'RVSNR' from dual union all
select 39888,'RVSNR' from dual union all
select 39889,'RVSNR' from dual union all
select 3989,'RVSN' from dual union all
select 39890,'RVSNR' from dual union all
select 39891,'RVSNR' from dual union all
select 39892,'RVSNR' from dual union all
select 39893,'RVSNR' from dual union all
select 39894,'RVSNR' from dual union all
select 39895,'RVSNR' from dual union all
select 39896,'RVSNR' from dual union all
select 39897,'RVSNR' from dual union all
select 39898,'RVSNR' from dual union all
select 39899,'RVSNR' from dual union all
select 399,'NASA' from dual union all
select 3990,'NASA' from dual union all
select 39900,'RVSNR' from dual union all
select 39901,'RVSNR' from dual union all
select 39902,'RVSNR' from dual union all
select 39903,'RVSNR' from dual union all
select 39904,'RVSNR' from dual union all
select 39905,'RVSNR' from dual union all
select 39906,'RVSNR' from dual union all
select 39907,'RVSNR' from dual union all
select 39908,'RVSNR' from dual union all
select 39909,'RVSNR' from dual union all
select 3991,'MO' from dual union all
select 39910,'RVSNR' from dual union all
select 39911,'RVSNR' from dual union all
select 39912,'OSC' from dual union all
select 39913,'OSC' from dual union all
select 39914,'OSC' from dual union all
select 39915,'OSC' from dual union all
select 39916,'OSC' from dual union all
select 39917,'OSC' from dual union all
select 39918,'OSC' from dual union all
select 39919,'OSC' from dual union all
select 3992,'RVSN' from dual union all
select 39920,'OSC' from dual union all
select 39921,'OSC' from dual union all
select 39922,'OSC' from dual union all
select 39923,'OSC' from dual union all
select 39924,'OSC' from dual union all
select 39925,'OSC' from dual union all
select 39926,'OSC' from dual union all
select 39927,'OSC' from dual union all
select 39928,'OSC' from dual union all
select 39929,'OSC' from dual union all
select 3993,'NASA' from dual union all
select 39930,'OSC' from dual union all
select 39931,'OSC' from dual union all
select 39932,'OSC' from dual union all
select 39933,'KHRR' from dual union all
select 39934,'KHRR' from dual union all
select 39935,'MO' from dual union all
select 39936,'MO' from dual union all
select 39937,'MO' from dual union all
select 39938,'MO' from dual union all
select 39939,'MO' from dual union all
select 3994,'MO' from dual union all
select 39940,'MO' from dual union all
select 39941,'MO' from dual union all
select 39942,'MO' from dual union all
select 39943,'MO' from dual union all
select 39944,'MO' from dual union all
select 39945,'MO' from dual union all
select 39946,'MO' from dual union all
select 39947,'MO' from dual union all
select 39948,'MO' from dual union all
select 39949,'MO' from dual union all
select 3995,'RVSN' from dual union all
select 39950,'MO' from dual union all
select 39951,'MO' from dual union all
select 39952,'MO' from dual union all
select 39953,'MO' from dual union all
select 39954,'MO' from dual union all
select 39955,'MO' from dual union all
select 39956,'MO' from dual union all
select 39957,'MO' from dual union all
select 39958,'MO' from dual union all
select 39959,'MO' from dual union all
select 3996,'USAF' from dual union all
select 39960,'MO' from dual union all
select 39961,'MO' from dual union all
select 39962,'MO' from dual union all
select 39963,'MO' from dual union all
select 39964,'MO' from dual union all
select 39965,'MO' from dual union all
select 39966,'MO' from dual union all
select 39967,'MO' from dual union all
select 39968,'MO' from dual union all
select 39969,'RVSNR' from dual union all
select 3997,'PVO' from dual union all
select 39970,'RVSNR' from dual union all
select 39971,'RVSNR' from dual union all
select 39972,'RVSNR' from dual union all
select 39973,'RVSNR' from dual union all
select 39974,'RVSNR' from dual union all
select 39975,'RVSNR' from dual union all
select 39976,'RVSNR' from dual;
insert into SATELLITE_ORG
select 39977,'RVSNR' from dual union all
select 39978,'RVSNR' from dual union all
select 39979,'RVSNR' from dual union all
select 3998,'USAF' from dual union all
select 39980,'RVSNR' from dual union all
select 39981,'RVSNR' from dual union all
select 39982,'RVSNR' from dual union all
select 39983,'RVSNR' from dual union all
select 39984,'RVSNR' from dual union all
select 39985,'RVSNR' from dual union all
select 39986,'RVSNR' from dual union all
select 39987,'KHRR' from dual union all
select 39988,'KHRR' from dual union all
select 39989,'KHRR' from dual union all
select 3999,'USAF' from dual union all
select 39990,'KHRR' from dual union all
select 39991,'KHRR' from dual union all
select 39992,'KHRR' from dual union all
select 39993,'KHRR' from dual union all
select 39994,'KHRR' from dual union all
select 39995,'KHRR' from dual union all
select 39996,'KHRR' from dual union all
select 39997,'KHRR' from dual union all
select 39998,'EUMET' from dual union all
select 39999,'EUMET' from dual union all
select 4,'JPL' from dual union all
select 4,'USA' from dual union all
select 40,'OKB1' from dual union all
select 400,'NASA' from dual union all
select 4000,'NASA' from dual union all
select 40000,'CMA' from dual union all
select 40001,'FKA' from dual union all
select 40001,'KVR' from dual union all
select 40002,'NPOL' from dual union all
select 40003,'USAF' from dual union all
select 40004,'USAF' from dual union all
select 40005,'USAF' from dual union all
select 40006,'USAF' from dual union all
select 40007,'USAF' from dual union all
select 40008,'USAF' from dual union all
select 40009,'USAF' from dual union all
select 4001,'NASA' from dual union all
select 40010,'KAZK' from dual union all
select 40011,'TOK' from dual union all
select 40012,'GAUSS' from dual union all
select 40013,'ELDEC' from dual union all
select 40014,'SATELOG' from dual union all
select 40015,'TOK' from dual union all
select 40016,'KACST' from dual union all
select 40017,'SPUT' from dual union all
select 40018,'SPQ' from dual union all
select 40019,'SPQ' from dual union all
select 4002,'PVO' from dual union all
select 40020,'CSA' from dual union all
select 40020,'UTIAS0' from dual union all
select 40021,'HSCIL' from dual union all
select 40022,'NCKU' from dual union all
select 40022,'VKI' from dual union all
select 40023,'PLABS' from dual union all
select 40024,'INPES' from dual union all
select 40024,'UFSM' from dual union all
select 40025,'QB50' from dual union all
select 40026,'PLABS' from dual union all
select 40027,'PLABS' from dual union all
select 40028,'MICSG' from dual union all
select 40029,'PLABS' from dual union all
select 4003,'USAF' from dual union all
select 40030,'DTU' from dual union all
select 40031,'PLABS' from dual union all
select 40032,'QB50' from dual union all
select 40033,'PLABS' from dual union all
select 40034,'ANTEL' from dual union all
select 40034,'FING' from dual union all
select 40035,'PLABS' from dual union all
select 40036,'PLABS' from dual union all
select 40037,'DAUR' from dual union all
select 40038,'PLABS' from dual union all
select 40039,'DAUR' from dual union all
select 4004,'USAF' from dual union all
select 40040,'PLABS' from dual union all
select 40041,'PLABS' from dual union all
select 40042,'KPI' from dual union all
select 40043,'IQMOST' from dual union all
select 40043,'ROSAP' from dual union all
select 40044,'NANSFI' from dual union all
select 40045,'AERO' from dual union all
select 40046,'AERO' from dual union all
select 40047,'KTRAS' from dual union all
select 40048,'KTRAS' from dual union all
select 40049,'KTRAS' from dual union all
select 4005,'USAF' from dual union all
select 40050,'RVSNR' from dual union all
select 40051,'JAXA' from dual union all
select 40052,'MORF' from dual union all
select 40053,'CNES' from dual union all
select 40054,'DLR' from dual union all
select 40055,'UTIAS0' from dual union all
select 40056,'UTIAS0' from dual union all
select 40057,'NTU' from dual union all
select 40058,'ISRO' from dual;
insert into SATELLITE_ORG
select 40059,'NASA' from dual union all
select 4006,'USAF' from dual union all
select 40060,'BOE' from dual union all
select 40061,'KVR' from dual union all
select 40062,'KVR' from dual union all
select 40063,'KVR' from dual union all
select 40064,'KVR' from dual union all
select 40065,'PLA' from dual union all
select 40066,'PLA' from dual union all
select 40067,'PLA' from dual union all
select 40068,'PLA' from dual union all
select 40069,'FKA' from dual union all
select 40069,'RGMS' from dual union all
select 4007,'USAF' from dual union all
select 40070,'IKI' from dual union all
select 40070,'NPOL' from dual union all
select 40071,'DAUR' from dual union all
select 40072,'SKYBOX' from dual union all
select 40073,'DND' from dual union all
select 40074,'UKSA' from dual union all
select 40075,'NSC' from dual union all
select 40076,'UKTSB' from dual union all
select 40077,'RVSNR' from dual union all
select 40078,'NPOL' from dual union all
select 40079,'O3B' from dual union all
select 4008,'PVO' from dual union all
select 40080,'O3B' from dual union all
select 40081,'O3B' from dual union all
select 40082,'O3B' from dual union all
select 40083,'NPOL' from dual union all
select 40084,'OSC' from dual union all
select 40085,'OSC' from dual union all
select 40086,'ORBC' from dual union all
select 40087,'ORBC' from dual union all
select 40088,'ORBC' from dual union all
select 40089,'ORBC' from dual union all
select 4009,'CNES' from dual union all
select 40090,'ORBC' from dual union all
select 40091,'ORBC' from dual union all
select 40092,'CNSA' from dual union all
select 40093,'CNSA' from dual union all
select 40094,'CNSA' from dual union all
select 40095,'FKA' from dual union all
select 40096,'RVSNR' from dual union all
select 40097,'FKA' from dual union all
select 40098,'VVKO' from dual union all
select 40099,'USAF' from dual union all
select 401,'USAF' from dual union all
select 4010,'PVO' from dual union all
select 40100,'USAF' from dual union all
select 40101,'USAF' from dual union all
select 40102,'BOE' from dual union all
select 40103,'ESA' from dual union all
select 40104,'RKA' from dual union all
select 40105,'USAF' from dual union all
select 40106,'LM' from dual union all
select 40107,'ASIA' from dual union all
select 40108,'SPX' from dual union all
select 40109,'PLA' from dual union all
select 4011,'PVO' from dual union all
select 40110,'PLA' from dual union all
select 40111,'PLA' from dual union all
select 40112,'CNSA' from dual union all
select 40113,'PLA' from dual union all
select 40114,'PLA' from dual union all
select 40115,'DGLO' from dual union all
select 40116,'LM' from dual union all
select 40117,'UNIPE' from dual union all
select 40118,'CNSAS' from dual union all
select 40119,'CBKPL' from dual union all
select 4012,'PVO' from dual union all
select 40120,'CNSA' from dual union all
select 40121,'KVR' from dual union all
select 40122,'PLABS' from dual union all
select 40123,'PLABS' from dual union all
select 40124,'PLABS' from dual union all
select 40125,'PLABS' from dual union all
select 40126,'PLABS' from dual union all
select 40127,'PLABS' from dual union all
select 40128,'GSAEU' from dual union all
select 40129,'GSAEU' from dual union all
select 4013,'PVO' from dual union all
select 40130,'NPOL' from dual union all
select 40131,'PLABS' from dual union all
select 40132,'PLABS' from dual union all
select 40133,'PLABS' from dual union all
select 40134,'PLABS' from dual union all
select 40135,'CNES' from dual union all
select 40135,'DGA' from dual union all
select 40136,'TSHUA' from dual union all
select 40137,'CASC' from dual union all
select 40138,'CNSA' from dual union all
select 40139,'PLABS' from dual union all
select 4014,'USAF' from dual union all
select 40140,'PLABS' from dual union all
select 40141,'ASIA' from dual union all
select 40142,'SPX' from dual union all
select 40143,'PLA' from dual union all
select 40144,'NUDTC' from dual union all
select 40145,'CNSA' from dual;
insert into SATELLITE_ORG
select 40146,'OPTS' from dual union all
select 40147,'MEAS' from dual union all
select 40148,'AE' from dual union all
select 40149,'AE' from dual union all
select 4015,'USAF' from dual union all
select 40150,'FKA' from dual union all
select 40151,'FKA' from dual union all
select 40152,'CNSA' from dual union all
select 40153,'CNSA' from dual union all
select 40154,'CNSA' from dual union all
select 40155,'CNSA' from dual union all
select 40156,'CNSA' from dual union all
select 40157,'CNSA' from dual union all
select 40158,'CASC' from dual union all
select 40159,'CASC' from dual union all
select 4016,'RVSN' from dual union all
select 40160,'CASC' from dual union all
select 40161,'CASC' from dual union all
select 40162,'CASC' from dual union all
select 40163,'CASC' from dual union all
select 40164,'CASC' from dual union all
select 40165,'CASC' from dual union all
select 40166,'CASC' from dual union all
select 40167,'CASC' from dual union all
select 40168,'CASC' from dual union all
select 40169,'CASC' from dual union all
select 4017,'USAF' from dual union all
select 40170,'CASC' from dual union all
select 40171,'CASC' from dual union all
select 40172,'CASC' from dual union all
select 40173,'CASC' from dual union all
select 40174,'CASC' from dual union all
select 40175,'CASC' from dual union all
select 40176,'CASC' from dual union all
select 40177,'CASC' from dual union all
select 40178,'CASC' from dual union all
select 40179,'CASC' from dual union all
select 4018,'PVO' from dual union all
select 40180,'CASC' from dual union all
select 40181,'CASC' from dual union all
select 40182,'CASC' from dual union all
select 40183,'CASC' from dual union all
select 40184,'CASC' from dual union all
select 40185,'CASC' from dual union all
select 40186,'CASC' from dual union all
select 40187,'CASC' from dual union all
select 40188,'CASC' from dual union all
select 40189,'CASC' from dual union all
select 4019,'NASA' from dual union all
select 40190,'CASC' from dual union all
select 40191,'CASC' from dual union all
select 40192,'CASC' from dual union all
select 40193,'CASC' from dual union all
select 40194,'CASC' from dual union all
select 40195,'CASC' from dual union all
select 40196,'CASC' from dual union all
select 40197,'CASC' from dual union all
select 40198,'CASC' from dual union all
select 40199,'CASC' from dual union all
select 402,'USAF' from dual union all
select 4020,'USAF' from dual union all
select 40200,'CASC' from dual union all
select 40201,'CASC' from dual union all
select 40202,'CASC' from dual union all
select 40203,'CASC' from dual union all
select 40204,'CASC' from dual union all
select 40205,'CASC' from dual union all
select 40206,'CASC' from dual union all
select 40207,'CASC' from dual union all
select 40208,'NROC' from dual union all
select 40209,'LM' from dual union all
select 4021,'USAF' from dual union all
select 40210,'SPX' from dual union all
select 40211,'SPX' from dual union all
select 40212,'SPX' from dual union all
select 40213,'KVR' from dual union all
select 40214,'KVR' from dual union all
select 40215,'KVR' from dual union all
select 40216,'USAF' from dual union all
select 40217,'GMS' from dual union all
select 40217,'YUZH' from dual union all
select 40218,'KVR' from dual union all
select 40219,'KVR' from dual union all
select 4022,'NASA' from dual union all
select 40220,'KVR' from dual union all
select 40221,'KVR' from dual union all
select 40222,'KVR' from dual union all
select 40223,'KVR' from dual union all
select 40224,'KVR' from dual union all
select 40225,'KVR' from dual union all
select 40226,'KVR' from dual union all
select 40227,'KVR' from dual union all
select 40228,'KVR' from dual union all
select 40229,'KVR' from dual union all
select 4023,'USAF' from dual union all
select 40230,'KVR' from dual union all
select 40231,'KVR' from dual union all
select 40232,'KVR' from dual union all
select 40233,'KVR' from dual union all
select 40234,'KVR' from dual;
insert into SATELLITE_ORG
select 40235,'KVR' from dual union all
select 40236,'KVR' from dual union all
select 40237,'KVR' from dual union all
select 40238,'KVR' from dual union all
select 40239,'KVR' from dual union all
select 4024,'PVO' from dual union all
select 40240,'OSC' from dual union all
select 40241,'OSC' from dual union all
select 40242,'OSC' from dual union all
select 40243,'OSC' from dual union all
select 40244,'OSC' from dual union all
select 40245,'KVR' from dual union all
select 40246,'FKA' from dual union all
select 40247,'VVKO' from dual union all
select 40248,'IRID' from dual union all
select 40249,'IRID' from dual union all
select 4025,'USAF' from dual union all
select 40250,'IRID' from dual union all
select 40251,'IRID' from dual union all
select 40252,'IRID' from dual union all
select 40253,'IRID' from dual union all
select 40254,'IRID' from dual union all
select 40255,'IRID' from dual union all
select 40256,'IRID' from dual union all
select 40257,'IRID' from dual union all
select 40258,'FSB' from dual union all
select 40259,'KHRR' from dual union all
select 4026,'NASA' from dual union all
select 40260,'KHRR' from dual union all
select 40261,'CNSA' from dual union all
select 40262,'CASC' from dual union all
select 40263,'CASC' from dual union all
select 40264,'CASC' from dual union all
select 40265,'CASC' from dual union all
select 40266,'CASC' from dual union all
select 40267,'JMA' from dual union all
select 40268,'JAXA' from dual union all
select 40269,'ISRO' from dual union all
select 4027,'USAF' from dual union all
select 40270,'ISRO' from dual union all
select 40271,'DTVA' from dual union all
select 40271,'INTELS' from dual union all
select 40272,'ARSAT' from dual union all
select 40273,'AE' from dual union all
select 40274,'AE' from dual union all
select 40275,'PLA' from dual union all
select 40276,'CNSA' from dual union all
select 40277,'KS' from dual union all
select 40278,'KHRR' from dual union all
select 40279,'KHRR' from dual union all
select 4028,'USAF' from dual union all
select 40280,'FKA' from dual union all
select 40281,'FKA' from dual union all
select 40282,'FKA' from dual union all
select 40283,'CNSA' from dual union all
select 40284,'CASC' from dual union all
select 40285,'FKA' from dual union all
select 40286,'CNSA' from dual union all
select 40287,'CASC' from dual union all
select 40288,'CASC' from dual union all
select 40289,'CASC' from dual union all
select 4029,'USAF' from dual union all
select 40290,'CASC' from dual union all
select 40291,'CASC' from dual union all
select 40292,'FKA' from dual union all
select 40293,'RVSNR' from dual union all
select 40294,'USAF' from dual union all
select 40295,'LM' from dual union all
select 40296,'KVR' from dual union all
select 40297,'NPOL' from dual union all
select 40298,'USEF' from dual union all
select 40299,'TOK' from dual union all
select 403,'USAF' from dual union all
select 4030,'PVO' from dual union all
select 40300,'NAG' from dual union all
select 40301,'KYUD' from dual union all
select 40302,'JAXA' from dual union all
select 40302,'TIT' from dual union all
select 40303,'KTRAS' from dual union all
select 40304,'KTRAS' from dual union all
select 40305,'PLA' from dual union all
select 40306,'CNSA' from dual union all
select 40307,'CNSA' from dual union all
select 40308,'CNSA' from dual union all
select 40309,'CNSA' from dual union all
select 4031,'PVO' from dual union all
select 40310,'PLA' from dual union all
select 40311,'GYZ' from dual union all
select 40312,'FKA' from dual union all
select 40313,'VVKO' from dual union all
select 40314,'USN' from dual union all
select 40315,'FKA' from dual union all
select 40315,'KVR' from dual union all
select 40316,'NPOL' from dual union all
select 40317,'KVR' from dual union all
select 40318,'KVR' from dual union all
select 40319,'JAXA' from dual union all
select 4032,'USAF' from dual union all
select 40320,'KYUT' from dual union all
select 40321,'TAMA' from dual;
insert into SATELLITE_ORG
select 40322,'JAXA' from dual union all
select 40322,'TOK' from dual union all
select 40323,'JAXA' from dual union all
select 40324,'IRIDS' from dual union all
select 40325,'IRIDS' from dual union all
select 40326,'IRIDS' from dual union all
select 40327,'IRIDS' from dual union all
select 40328,'PVO' from dual union all
select 40329,'NASA' from dual union all
select 4033,'USAF' from dual union all
select 40330,'BOE' from dual union all
select 40331,'BOE' from dual union all
select 40332,'ISRO' from dual union all
select 40333,'DTV' from dual union all
select 40334,'AE' from dual union all
select 40335,'AE' from dual union all
select 40336,'GZB' from dual union all
select 40336,'INPE' from dual union all
select 40337,'CNSA' from dual union all
select 40338,'PLA' from dual union all
select 40339,'PLA' from dual union all
select 4034,'MO' from dual union all
select 40340,'PLA' from dual union all
select 40341,'CNSA' from dual union all
select 40342,'PLA' from dual union all
select 40343,'PLA' from dual union all
select 40344,'NROC' from dual union all
select 40345,'GAZS' from dual union all
select 40346,'KHRR' from dual union all
select 40347,'KHRR' from dual union all
select 40348,'O3B' from dual union all
select 40349,'O3B' from dual union all
select 4035,'RVSN' from dual union all
select 40350,'O3B' from dual union all
select 40351,'O3B' from dual union all
select 40352,'NPOL' from dual union all
select 40353,'SANDF' from dual union all
select 40354,'NPOMA' from dual union all
select 40355,'KHRU' from dual union all
select 40356,'KHRR' from dual union all
select 40357,'RVSN' from dual union all
select 40358,'KVR' from dual union all
select 40359,'RVSNR' from dual union all
select 4036,'IKI' from dual union all
select 4036,'NPOL' from dual union all
select 40360,'TSSKB' from dual union all
select 40361,'KVR' from dual union all
select 40362,'PLA' from dual union all
select 40363,'CNSA' from dual union all
select 40364,'SES' from dual union all
select 40365,'KHRR' from dual union all
select 40366,'KHRR' from dual union all
select 40367,'CMA' from dual union all
select 40368,'CASC' from dual union all
select 40369,'CASC' from dual union all
select 4037,'RVSN' from dual union all
select 40370,'SPX' from dual union all
select 40371,'SPX' from dual union all
select 40372,'SPX' from dual union all
select 40373,'SPX' from dual union all
select 40374,'USN' from dual union all
select 40375,'LM' from dual union all
select 40376,'JPL' from dual union all
select 40376,'NASA' from dual union all
select 40377,'MSU' from dual union all
select 40377,'UNH' from dual union all
select 40378,'MSU' from dual union all
select 40378,'UNH' from dual union all
select 40379,'UMI' from dual union all
select 4038,'RVSN' from dual union all
select 40380,'CALP' from dual union all
select 40380,'NSF' from dual union all
select 40381,'CSICE' from dual union all
select 40382,'MHITO' from dual union all
select 40383,'MHITO' from dual union all
select 40384,'INMRL' from dual union all
select 40385,'KHRR' from dual union all
select 40386,'KHRR' from dual union all
select 40387,'IRSA' from dual union all
select 40388,'IRSA' from dual union all
select 40389,'AEB' from dual union all
select 4039,'NASA' from dual union all
select 40390,'NOAA' from dual union all
select 40391,'SPX' from dual union all
select 40392,'FKA' from dual union all
select 40393,'VVKO' from dual union all
select 40394,'USAF' from dual union all
select 40395,'USAF' from dual union all
select 40396,'USAF' from dual union all
select 40397,'USAF' from dual union all
select 40398,'USAF' from dual union all
select 40399,'USAF' from dual union all
select 404,'USAF' from dual union all
select 4040,'NASA' from dual union all
select 40400,'USAF' from dual union all
select 40401,'USAF' from dual union all
select 40402,'USAF' from dual union all
select 40403,'USAF' from dual union all
select 40404,'USAF' from dual union all
select 40405,'USAF' from dual;
insert into SATELLITE_ORG
select 40406,'USAF' from dual union all
select 40407,'USAF' from dual union all
select 40408,'USAF' from dual union all
select 40409,'USAF' from dual union all
select 4041,'NASA' from dual union all
select 40410,'USAF' from dual union all
select 40411,'USAF' from dual union all
select 40412,'USAF' from dual union all
select 40413,'USAF' from dual union all
select 40414,'USAF' from dual union all
select 40415,'USAF' from dual union all
select 40416,'USAF' from dual union all
select 40417,'USAF' from dual union all
select 40418,'USAF' from dual union all
select 40419,'USAF' from dual union all
select 4042,'MO' from dual union all
select 40420,'KVR' from dual union all
select 40421,'RVSNR' from dual union all
select 40422,'PLABS' from dual union all
select 40423,'PLABS' from dual union all
select 40424,'AGSL' from dual union all
select 40425,'EUTSA' from dual union all
select 40426,'SPX' from dual union all
select 40427,'PLABS' from dual union all
select 40428,'PLABS' from dual union all
select 40429,'PLABS' from dual union all
select 4043,'MO' from dual union all
select 40430,'PLABS' from dual union all
select 40431,'USAF' from dual union all
select 40432,'USAF' from dual union all
select 40433,'USAF' from dual union all
select 40434,'USAF' from dual union all
select 40435,'USAF' from dual union all
select 40436,'USAF' from dual union all
select 40437,'USAF' from dual union all
select 40438,'USAF' from dual union all
select 40439,'USAF' from dual union all
select 4044,'RVSN' from dual union all
select 40440,'USAF' from dual union all
select 40441,'USAF' from dual union all
select 40442,'USAF' from dual union all
select 40443,'USAF' from dual union all
select 40444,'USAF' from dual union all
select 40445,'USAF' from dual union all
select 40446,'USAF' from dual union all
select 40447,'USAF' from dual union all
select 40448,'USAF' from dual union all
select 40449,'USAF' from dual union all
select 4045,'RVSN' from dual union all
select 40450,'USAF' from dual union all
select 40451,'PLABS' from dual union all
select 40452,'PLABS' from dual union all
select 40453,'PLABS' from dual union all
select 40454,'PLABS' from dual union all
select 40455,'NASA' from dual union all
select 40455,'SJSU' from dual union all
select 40456,'NSL' from dual union all
select 40457,'MITLL' from dual union all
select 40458,'LAMBDA' from dual union all
select 40459,'PLABS' from dual union all
select 4046,'RVSN' from dual union all
select 40460,'PLABS' from dual union all
select 40461,'USAF' from dual union all
select 40462,'USAF' from dual union all
select 40463,'USAF' from dual union all
select 40464,'USAF' from dual union all
select 40465,'USAF' from dual union all
select 40466,'USAF' from dual union all
select 40467,'USAF' from dual union all
select 40468,'USAF' from dual union all
select 40469,'USAF' from dual union all
select 4047,'USAF' from dual union all
select 40470,'USAF' from dual union all
select 40471,'USAF' from dual union all
select 40472,'USAF' from dual union all
select 40473,'USAF' from dual union all
select 40474,'USAF' from dual union all
select 40475,'USAF' from dual union all
select 40476,'USAF' from dual union all
select 40477,'USAF' from dual union all
select 40478,'USAF' from dual union all
select 40479,'USAF' from dual union all
select 4048,'USAF' from dual union all
select 40480,'USAF' from dual union all
select 40481,'USAF' from dual union all
select 40482,'NASA' from dual union all
select 40483,'NASA' from dual union all
select 40484,'NASA' from dual union all
select 40485,'NASA' from dual union all
select 40486,'LM' from dual union all
select 40487,'USAF' from dual union all
select 40488,'USAF' from dual union all
select 40489,'USAF' from dual union all
select 4049,'RVSN' from dual union all
select 40490,'USAF' from dual union all
select 40491,'USAF' from dual union all
select 40492,'USAF' from dual union all
select 40493,'USAF' from dual union all
select 40494,'USAF' from dual union all
select 40495,'USAF' from dual;
insert into SATELLITE_ORG
select 40496,'USAF' from dual union all
select 40497,'USAF' from dual union all
select 40498,'USAF' from dual union all
select 40499,'USAF' from dual union all
select 405,'USAF' from dual union all
select 4050,'CIA' from dual union all
select 4050,'NRO' from dual union all
select 40500,'USAF' from dual union all
select 40501,'USAF' from dual union all
select 40502,'USAF' from dual union all
select 40503,'USAF' from dual union all
select 40504,'USAF' from dual union all
select 40505,'KS' from dual union all
select 40506,'KHRR' from dual union all
select 40507,'KHRR' from dual union all
select 40508,'USAF' from dual union all
select 40509,'USAF' from dual union all
select 4051,'INTEL' from dual union all
select 40510,'USAF' from dual union all
select 40511,'USAF' from dual union all
select 40512,'USAF' from dual union all
select 40513,'USAF' from dual union all
select 40514,'USAF' from dual union all
select 40515,'USAF' from dual union all
select 40516,'USAF' from dual union all
select 40517,'USAF' from dual union all
select 40518,'USAF' from dual union all
select 40519,'USAF' from dual union all
select 4052,'NASA' from dual union all
select 40520,'USAF' from dual union all
select 40521,'USAF' from dual union all
select 40522,'USAF' from dual union all
select 40523,'USAF' from dual union all
select 40524,'USAF' from dual union all
select 40525,'USAF' from dual union all
select 40526,'USAF' from dual union all
select 40527,'USAF' from dual union all
select 40528,'USAF' from dual union all
select 40529,'USAF' from dual union all
select 4053,'NASA' from dual union all
select 40530,'CNSA' from dual union all
select 40531,'CNSA' from dual union all
select 40532,'CNSA' from dual union all
select 40533,'CNSA' from dual union all
select 40534,'USAF' from dual union all
select 40535,'BOE' from dual union all
select 40536,'KARI' from dual union all
select 40537,'KTRAS' from dual union all
select 40538,'CSICE' from dual union all
select 40539,'JAXA' from dual union all
select 4054,'NRO' from dual union all
select 40540,'JAXA' from dual union all
select 40541,'KTRAS' from dual union all
select 40542,'FKA' from dual union all
select 40543,'VVKO' from dual union all
select 40544,'GSAEU' from dual union all
select 40545,'GSAEU' from dual union all
select 40546,'NPOL' from dual union all
select 40547,'ISRO' from dual union all
select 40548,'ISRO' from dual union all
select 40549,'CNSA' from dual union all
select 4055,'NASA' from dual union all
select 40550,'CASC' from dual union all
select 40551,'USAF' from dual union all
select 40552,'KVR' from dual union all
select 40553,'KVR' from dual union all
select 40554,'KVR' from dual union all
select 40555,'FKA' from dual union all
select 40556,'KVR' from dual union all
select 40557,'USAF' from dual union all
select 40558,'USAF' from dual union all
select 40559,'USAF' from dual union all
select 4056,'NASA' from dual union all
select 40560,'USAF' from dual union all
select 40561,'USAF' from dual union all
select 40562,'USAF' from dual union all
select 40563,'USAF' from dual union all
select 40564,'USAF' from dual union all
select 40565,'USAF' from dual union all
select 40566,'USAF' from dual union all
select 40567,'USAF' from dual union all
select 40568,'USAF' from dual union all
select 40569,'USAF' from dual union all
select 4057,'NASA' from dual union all
select 40570,'USAF' from dual union all
select 40571,'USAF' from dual union all
select 40572,'USAF' from dual union all
select 40573,'USAF' from dual union all
select 40574,'USAF' from dual union all
select 40575,'USAF' from dual union all
select 40576,'USAF' from dual union all
select 40577,'USAF' from dual union all
select 40578,'USAF' from dual union all
select 40579,'USAF' from dual union all
select 4058,'PVO' from dual union all
select 40580,'USAF' from dual union all
select 40581,'USAF' from dual union all
select 40582,'USAF' from dual union all
select 40583,'USAF' from dual union all
select 40584,'USAF' from dual;
insert into SATELLITE_ORG
select 40585,'USAF' from dual union all
select 40586,'USAF' from dual union all
select 40587,'USAF' from dual union all
select 40588,'SPX' from dual union all
select 40589,'SPX' from dual union all
select 4059,'NASA' from dual union all
select 40590,'SPX' from dual union all
select 40591,'SPX' from dual union all
select 40592,'NASA' from dual union all
select 40593,'CNSA' from dual union all
select 40594,'CNSA' from dual union all
select 40595,'CNSA' from dual union all
select 40596,'CNSA' from dual union all
select 40597,'BOE' from dual union all
select 40598,'BOE' from dual union all
select 40599,'BOE' from dual union all
select 406,'USAF' from dual union all
select 4060,'NASA' from dual union all
select 40600,'USAF' from dual union all
select 40601,'USAF' from dual union all
select 40602,'USAF' from dual union all
select 40603,'USAF' from dual union all
select 40604,'USAF' from dual union all
select 40605,'USAF' from dual union all
select 40606,'USAF' from dual union all
select 40607,'USAF' from dual union all
select 40608,'USAF' from dual union all
select 40609,'USAF' from dual union all
select 4061,'MO' from dual union all
select 40610,'USAF' from dual union all
select 40611,'USAF' from dual union all
select 40612,'USAF' from dual union all
select 40613,'TNOR' from dual union all
select 40614,'MDDI' from dual union all
select 40615,'AE' from dual union all
select 40616,'AE' from dual union all
select 40617,'SSIM' from dual union all
select 40617,'TMTEL' from dual union all
select 40618,'SPX' from dual union all
select 40619,'FKA' from dual union all
select 4062,'TSKBEM' from dual union all
select 40620,'RVSNR' from dual union all
select 40621,'FKA' from dual union all
select 40622,'FKA' from dual union all
select 40623,'FKA' from dual union all
select 40624,'FKA' from dual union all
select 40625,'FKA' from dual union all
select 40626,'FKA' from dual union all
select 40627,'FKA' from dual union all
select 40628,'FKA' from dual union all
select 40629,'FKA' from dual union all
select 4063,'RVSN' from dual union all
select 40630,'FKA' from dual union all
select 40631,'FKA' from dual union all
select 40632,'FKA' from dual union all
select 40633,'FKA' from dual union all
select 40634,'FKA' from dual union all
select 40635,'FKA' from dual union all
select 40636,'FKA' from dual union all
select 40637,'FKA' from dual union all
select 40638,'FKA' from dual union all
select 40639,'FKA' from dual union all
select 4064,'RVSN' from dual union all
select 40640,'FKA' from dual union all
select 40641,'NASA' from dual union all
select 40642,'NASA' from dual union all
select 40643,'NASA' from dual union all
select 40644,'NASA' from dual union all
select 40645,'NASA' from dual union all
select 40646,'NASA' from dual union all
select 40647,'NASA' from dual union all
select 40648,'NASA' from dual union all
select 40649,'BOE' from dual union all
select 4065,'NASA' from dual union all
select 40650,'KVR' from dual union all
select 40651,'USAF' from dual union all
select 40652,'USN' from dual union all
select 40653,'USAF' from dual union all
select 40654,'USN' from dual union all
select 40655,'USN' from dual union all
select 40656,'USAF' from dual union all
select 40657,'NSL' from dual union all
select 40657,'USAF' from dual union all
select 40658,'USAF' from dual union all
select 40659,'AERO' from dual union all
select 4066,'NASA' from dual union all
select 40660,'AERO' from dual union all
select 40661,'TPS' from dual union all
select 40662,'USN' from dual union all
select 40663,'DTV' from dual union all
select 40664,'DTV' from dual union all
select 40664,'DTVA' from dual union all
select 40665,'AE' from dual union all
select 40666,'AE' from dual union all
select 40667,'KVR' from dual union all
select 40668,'RVSNR' from dual union all
select 40669,'KVR' from dual union all
select 4067,'TSKBEM' from dual union all
select 40670,'KVR' from dual union all
select 40671,'KVR' from dual;
insert into SATELLITE_ORG
select 40672,'USAF' from dual union all
select 40673,'USAF' from dual union all
select 40674,'USAF' from dual union all
select 40675,'USAF' from dual union all
select 40676,'USAF' from dual union all
select 40677,'USAF' from dual union all
select 40678,'USAF' from dual union all
select 40679,'USAF' from dual union all
select 4068,'NASA' from dual union all
select 40680,'USAF' from dual union all
select 40681,'USAF' from dual union all
select 40682,'CASC' from dual union all
select 40683,'CASC' from dual union all
select 40684,'CASC' from dual union all
select 40685,'CASC' from dual union all
select 40686,'CASC' from dual union all
select 40687,'CASC' from dual union all
select 40688,'CASC' from dual union all
select 40689,'CASC' from dual union all
select 4069,'NASA' from dual union all
select 40690,'CASC' from dual union all
select 40691,'CASC' from dual union all
select 40692,'CASC' from dual union all
select 40693,'CASC' from dual union all
select 40694,'CASC' from dual union all
select 40695,'CASC' from dual union all
select 40696,'CASC' from dual union all
select 40697,'DGGROW' from dual union all
select 40697,'ESA' from dual union all
select 40698,'AE' from dual union all
select 40699,'KVR' from dual union all
select 407,'USAF' from dual union all
select 4070,'OKB10' from dual union all
select 40700,'KVR' from dual union all
select 40701,'PLA' from dual union all
select 40702,'CNSA' from dual union all
select 40703,'KVR' from dual union all
select 40704,'KVR' from dual union all
select 40705,'KVR' from dual union all
select 40706,'KVR' from dual union all
select 40707,'KVR' from dual union all
select 40708,'KVR' from dual union all
select 40709,'KVR' from dual union all
select 4071,'RVSN' from dual union all
select 40710,'KVR' from dual union all
select 40711,'KVR' from dual union all
select 40712,'KVR' from dual union all
select 40713,'FKA' from dual union all
select 40714,'VVKO' from dual union all
select 40715,'DMC' from dual union all
select 40716,'DMC' from dual union all
select 40717,'DMC' from dual union all
select 40718,'EADS' from dual union all
select 40719,'SURREY' from dual union all
select 4072,'TSKBEM' from dual union all
select 40720,'ISRO' from dual union all
select 40721,'USAF' from dual union all
select 40722,'PLABS' from dual union all
select 40723,'PLABS' from dual union all
select 40724,'PLABS' from dual union all
select 40725,'PLABS' from dual union all
select 40726,'PLABS' from dual union all
select 40727,'PLABS' from dual union all
select 40728,'PLABS' from dual union all
select 40729,'PLABS' from dual union all
select 4073,'RVSN' from dual union all
select 40730,'USAF' from dual union all
select 40731,'LM' from dual union all
select 40732,'EUMET' from dual union all
select 40733,'TELB' from dual union all
select 40734,'AE' from dual union all
select 40735,'AE' from dual union all
select 40736,'PLABS' from dual union all
select 40737,'PLABS' from dual union all
select 40738,'PLABS' from dual union all
select 40739,'PLABS' from dual union all
select 4074,'MO' from dual union all
select 40740,'PLABS' from dual union all
select 40741,'PLABS' from dual union all
select 40742,'PLRES' from dual union all
select 40743,'BOOZA' from dual union all
select 40744,'FKA' from dual union all
select 40745,'VVKO' from dual union all
select 40746,'USAF' from dual union all
select 40747,'BOE' from dual union all
select 40748,'CNSA' from dual union all
select 40749,'CNSA' from dual union all
select 4075,'RVSN' from dual union all
select 40750,'CASC' from dual union all
select 40751,'CASC' from dual union all
select 40752,'NASA' from dual union all
select 40753,'NASA' from dual union all
select 40754,'NASA' from dual union all
select 40755,'NASA' from dual union all
select 40756,'NASA' from dual union all
select 40757,'NASA' from dual union all
select 40758,'NASA' from dual union all
select 40759,'NASA' from dual union all
select 4076,'PVO' from dual union all
select 40760,'NASA' from dual;
insert into SATELLITE_ORG
select 40761,'NASA' from dual union all
select 40762,'NASA' from dual union all
select 40763,'NASA' from dual union all
select 40764,'NASA' from dual union all
select 40765,'NASA' from dual union all
select 40766,'NASA' from dual union all
select 40767,'NASA' from dual union all
select 40768,'NASA' from dual union all
select 40769,'NASA' from dual union all
select 4077,'RVSN' from dual union all
select 40770,'NASA' from dual union all
select 40771,'NASA' from dual union all
select 40772,'NASA' from dual union all
select 40773,'NASA' from dual union all
select 40774,'ESA' from dual union all
select 40775,'ESA' from dual union all
select 40776,'ESA' from dual union all
select 40777,'ESA' from dual union all
select 40778,'ESA' from dual union all
select 40779,'ESA' from dual union all
select 4078,'NRO' from dual union all
select 40780,'ESA' from dual union all
select 40781,'ESA' from dual union all
select 40782,'ESA' from dual union all
select 40783,'AE' from dual union all
select 40784,'AE' from dual union all
select 40785,'AE' from dual union all
select 40786,'AE' from dual union all
select 40787,'AE' from dual union all
select 40788,'AE' from dual union all
select 40789,'RVSNR' from dual union all
select 4079,'NASA' from dual union all
select 40790,'RVSNR' from dual union all
select 40791,'RVSNR' from dual union all
select 40792,'KVR' from dual union all
select 40793,'KVR' from dual union all
select 40794,'KVR' from dual union all
select 40795,'KVR' from dual union all
select 40796,'KVR' from dual union all
select 40797,'KVR' from dual union all
select 40798,'KVR' from dual union all
select 40799,'KVR' from dual union all
select 408,'USAF' from dual union all
select 4080,'MO' from dual union all
select 40800,'KVR' from dual union all
select 40801,'KVR' from dual union all
select 40802,'KVR' from dual union all
select 40803,'KVR' from dual union all
select 40804,'KVR' from dual union all
select 40805,'KVR' from dual union all
select 40806,'KVR' from dual union all
select 40807,'KVR' from dual union all
select 40808,'KVR' from dual union all
select 40809,'KVR' from dual union all
select 4081,'RVSN' from dual union all
select 40810,'KVR' from dual union all
select 40811,'KVR' from dual union all
select 40812,'ESA' from dual union all
select 40813,'CNSA' from dual union all
select 40814,'CASC' from dual union all
select 40815,'CASC' from dual union all
select 40816,'CASC' from dual union all
select 40817,'CASC' from dual union all
select 40818,'CASC' from dual union all
select 40819,'CASC' from dual union all
select 4082,'MO' from dual union all
select 40820,'CASC' from dual union all
select 40821,'CASC' from dual union all
select 40822,'CASC' from dual union all
select 40823,'CASC' from dual union all
select 40824,'CASC' from dual union all
select 40825,'CASC' from dual union all
select 40826,'CASC' from dual union all
select 40827,'CASC' from dual union all
select 40828,'CASC' from dual union all
select 40829,'CASC' from dual union all
select 4083,'RVSN' from dual union all
select 40830,'CASC' from dual union all
select 40831,'CASC' from dual union all
select 40832,'CASC' from dual union all
select 40833,'CASC' from dual union all
select 40834,'CASC' from dual union all
select 40835,'CASC' from dual union all
select 40836,'CASC' from dual union all
select 40837,'CASC' from dual union all
select 40838,'CASC' from dual union all
select 40839,'CASC' from dual union all
select 4084,'MO' from dual union all
select 40840,'CASC' from dual union all
select 40841,'CASC' from dual union all
select 40842,'CASC' from dual union all
select 40843,'CASC' from dual union all
select 40844,'CASC' from dual union all
select 40845,'CASC' from dual union all
select 40846,'CASC' from dual union all
select 40847,'CASC' from dual union all
select 40848,'CASC' from dual union all
select 40849,'CASC' from dual union all
select 4085,'NASA' from dual union all
select 40850,'CASC' from dual;
insert into SATELLITE_ORG
select 40851,'CASC' from dual union all
select 40852,'CASC' from dual union all
select 40853,'CASC' from dual union all
select 40854,'CASC' from dual union all
select 40855,'CASC' from dual union all
select 40856,'CASC' from dual union all
select 40857,'CASC' from dual union all
select 40858,'CASC' from dual union all
select 40859,'CASC' from dual union all
select 4086,'NASA' from dual union all
select 40860,'CASC' from dual union all
select 40861,'CASC' from dual union all
select 40862,'CASC' from dual union all
select 40863,'CASC' from dual union all
select 40864,'CASC' from dual union all
select 40865,'CASC' from dual union all
select 40866,'CASC' from dual union all
select 40867,'CASC' from dual union all
select 40868,'CASC' from dual union all
select 40869,'CASC' from dual union all
select 4087,'NASA' from dual union all
select 40870,'CASC' from dual union all
select 40871,'EUMET' from dual union all
select 40872,'EUMET' from dual union all
select 40873,'JAXA' from dual union all
select 40874,'INTELS' from dual union all
select 40875,'EUTSA' from dual union all
select 40876,'AE' from dual union all
select 40877,'AE' from dual union all
select 40878,'PLA' from dual union all
select 40879,'CNSA' from dual union all
select 4088,'NASA' from dual union all
select 40880,'ISRO' from dual union all
select 40881,'ISRO' from dual union all
select 40882,'INMRL' from dual union all
select 40883,'KHRR' from dual union all
select 40884,'KHRR' from dual union all
select 40885,'FKA' from dual union all
select 40886,'VVKO' from dual union all
select 40887,'USN' from dual union all
select 40888,'LM' from dual union all
select 40889,'GSAEU' from dual union all
select 4089,'NASA' from dual union all
select 40890,'GSAEU' from dual union all
select 40891,'NPOL' from dual union all
select 40892,'CASC' from dual union all
select 40893,'CASC' from dual union all
select 40894,'CNSAS' from dual union all
select 40895,'KS' from dual union all
select 40896,'KVR' from dual union all
select 40897,'AEB' from dual union all
select 40897,'UNB' from dual union all
select 40898,'CHIBA' from dual union all
select 40899,'NUDTC' from dual union all
select 409,'USAF' from dual union all
select 4090,'NASA' from dual union all
select 40900,'CAMSAT' from dual union all
select 40900,'NUDTC' from dual union all
select 40901,'ZHEJ' from dual union all
select 40902,'ZHEJ' from dual union all
select 40903,'CAMSAT' from dual union all
select 40904,'CASC' from dual union all
select 40905,'TSHUA' from dual union all
select 40906,'CAMSAT' from dual union all
select 40907,'CAMSAT' from dual union all
select 40908,'HARB' from dual union all
select 40909,'CAMSAT' from dual union all
select 4091,'NASA' from dual union all
select 40910,'CAMSAT' from dual union all
select 40911,'CAMSAT' from dual union all
select 40912,'CAMSAT' from dual union all
select 40912,'CASC' from dual union all
select 40913,'MAI' from dual union all
select 40914,'USAF' from dual union all
select 40915,'USAF' from dual union all
select 40916,'USAF' from dual union all
select 40917,'USAF' from dual union all
select 40918,'USAF' from dual union all
select 40919,'USAF' from dual union all
select 4092,'RVSN' from dual union all
select 40920,'KVR' from dual union all
select 40921,'KVR' from dual union all
select 40922,'KVR' from dual union all
select 40923,'EUROK' from dual union all
select 40924,'MO' from dual union all
select 40925,'CASC' from dual union all
select 40926,'NJUST' from dual union all
select 40927,'NJUST' from dual union all
select 40928,'SHANGT' from dual union all
select 40929,'CASC' from dual union all
select 4093,'RVSN' from dual union all
select 40930,'ISRO' from dual union all
select 40931,'LAPAN' from dual union all
select 40932,'SPIRE' from dual union all
select 40933,'SPIRE' from dual union all
select 40934,'SPIRE' from dual union all
select 40935,'SPIRE' from dual union all
select 40936,'COMDEV' from dual union all
select 40937,'ISRO' from dual union all
select 40938,'CNSA' from dual;
insert into SATELLITE_ORG
select 40939,'CASC' from dual union all
select 4094,'RVSN' from dual union all
select 40940,'NBN' from dual union all
select 40941,'ARSAT' from dual union all
select 40942,'AE' from dual union all
select 40943,'AE' from dual union all
select 40944,'FKA' from dual union all
select 40945,'VVKO' from dual union all
select 40946,'SCT' from dual union all
select 40947,'LM' from dual union all
select 40948,'AAU' from dual union all
select 40949,'GOMSP' from dual union all
select 4095,'RVSN' from dual union all
select 40950,'PLABS' from dual union all
select 40951,'PLABS' from dual union all
select 40952,'PLABS' from dual union all
select 40953,'PLABS' from dual union all
select 40954,'PLABS' from dual union all
select 40955,'PLABS' from dual union all
select 40956,'PLABS' from dual union all
select 40957,'PLABS' from dual union all
select 40958,'CAS' from dual union all
select 40959,'CAS' from dual union all
select 4096,'RVSN' from dual union all
select 40960,'CAS' from dual union all
select 40961,'CAS' from dual union all
select 40962,'PLABS' from dual union all
select 40963,'PLABS' from dual union all
select 40964,'NROC' from dual union all
select 40964,'USN' from dual union all
select 40965,'AERO' from dual union all
select 40966,'AERO' from dual union all
select 40967,'AMNA' from dual union all
select 40968,'SKC' from dual union all
select 40969,'UAF' from dual union all
select 4097,'MO' from dual union all
select 40970,'SMDC' from dual union all
select 40971,'JPL' from dual union all
select 40972,'SMDC' from dual union all
select 40973,'USN' from dual union all
select 40974,'SRII' from dual union all
select 40975,'SMDC' from dual union all
select 40976,'USN' from dual union all
select 40977,'SRII' from dual union all
select 40978,'CALP' from dual union all
select 40978,'NROC' from dual union all
select 40979,'PLABS' from dual union all
select 4098,'RVSN' from dual union all
select 40980,'PLABS' from dual union all
select 40981,'NROC' from dual union all
select 40981,'USN' from dual union all
select 40982,'APT' from dual union all
select 40983,'CASC' from dual union all
select 40984,'TURKS' from dual union all
select 40985,'KHRR' from dual union all
select 40986,'KHRR' from dual union all
select 40987,'CMA' from dual union all
select 40988,'GZB' from dual union all
select 40989,'EUMET' from dual union all
select 4099,'NASA' from dual union all
select 40990,'EUMET' from dual union all
select 40991,'LM' from dual union all
select 40992,'IRIDS' from dual union all
select 40993,'IRIDS' from dual union all
select 40994,'IRIDS' from dual union all
select 40995,'IRIDS' from dual union all
select 40996,'IRIDS' from dual union all
select 40997,'IRIDS' from dual union all
select 40998,'IRIDS' from dual union all
select 40999,'CASC' from dual union all
select 41,'OKB1' from dual union all
select 410,'USAF' from dual union all
select 4100,'NASA' from dual union all
select 41000,'CASC' from dual union all
select 41001,'CASC' from dual union all
select 41002,'CASC' from dual union all
select 41003,'CASC' from dual union all
select 41004,'CASC' from dual union all
select 41005,'CASC' from dual union all
select 41006,'CASC' from dual union all
select 41007,'CASC' from dual union all
select 41008,'CASC' from dual union all
select 41009,'CASC' from dual union all
select 4101,'NASA' from dual union all
select 41010,'CASC' from dual union all
select 41011,'CASC' from dual union all
select 41012,'CASC' from dual union all
select 41013,'CASC' from dual union all
select 41014,'CASC' from dual union all
select 41015,'CASC' from dual union all
select 41016,'CASC' from dual union all
select 41017,'CASC' from dual union all
select 41018,'CASC' from dual union all
select 41019,'USAF' from dual union all
select 4102,'CIA' from dual union all
select 4102,'NRO' from dual union all
select 41020,'LM' from dual union all
select 41021,'PLA' from dual union all
select 41022,'CASC' from dual union all
select 41023,'CASC' from dual;
insert into SATELLITE_ORG
select 41024,'CASC' from dual union all
select 41025,'CASC' from dual union all
select 41026,'PLA' from dual union all
select 41027,'CNSA' from dual union all
select 41028,'ISRO' from dual union all
select 41029,'ARAB' from dual union all
select 4103,'NRO' from dual union all
select 41030,'AE' from dual union all
select 41031,'AE' from dual union all
select 41032,'KVR' from dual union all
select 41033,'NPOL' from dual union all
select 41034,'CASC' from dual union all
select 41034,'LANAST' from dual union all
select 41035,'CASC' from dual union all
select 41036,'TCAN' from dual union all
select 41037,'JAXA' from dual union all
select 41038,'PLA' from dual union all
select 41039,'CNSA' from dual union all
select 4104,'IKI' from dual union all
select 4104,'NPOL' from dual union all
select 41040,'CNSA' from dual union all
select 41041,'CNSA' from dual union all
select 41042,'CNSA' from dual union all
select 41043,'ESA' from dual union all
select 41044,'AE' from dual union all
select 41045,'NOAA' from dual union all
select 41046,'NOAA' from dual union all
select 41047,'NOAA' from dual union all
select 41048,'NOAA' from dual union all
select 41049,'NOAA' from dual union all
select 4105,'RVSN' from dual union all
select 41050,'NOAA' from dual union all
select 41051,'NOAA' from dual union all
select 41052,'NOAA' from dual union all
select 41053,'NOAA' from dual union all
select 41054,'NOAA' from dual union all
select 41055,'NOAA' from dual union all
select 41056,'NOAA' from dual union all
select 41057,'NOAA' from dual union all
select 41058,'NOAA' from dual union all
select 41059,'NOAA' from dual union all
select 4106,'MO' from dual union all
select 41060,'NOAA' from dual union all
select 41061,'NOAA' from dual union all
select 41062,'NOAA' from dual union all
select 41063,'NOAA' from dual union all
select 41064,'NOAA' from dual union all
select 41065,'NOAA' from dual union all
select 41066,'NOAA' from dual union all
select 41067,'NOAA' from dual union all
select 41068,'NOAA' from dual union all
select 41069,'NOAA' from dual union all
select 4107,'RVSN' from dual union all
select 41070,'NOAA' from dual union all
select 41071,'NOAA' from dual union all
select 41072,'NOAA' from dual union all
select 41073,'NOAA' from dual union all
select 41074,'NOAA' from dual union all
select 41075,'NOAA' from dual union all
select 41076,'NOAA' from dual union all
select 41077,'NOAA' from dual union all
select 41078,'NOAA' from dual union all
select 41079,'NOAA' from dual union all
select 4108,'NASA' from dual union all
select 41080,'NOAA' from dual union all
select 41081,'NOAA' from dual union all
select 41082,'NOAA' from dual union all
select 41083,'NOAA' from dual union all
select 41084,'NOAA' from dual union all
select 41085,'NOAA' from dual union all
select 41086,'NOAA' from dual union all
select 41087,'NOAA' from dual union all
select 41088,'NOAA' from dual union all
select 41089,'NOAA' from dual union all
select 4109,'NASA' from dual union all
select 41090,'NOAA' from dual union all
select 41091,'NOAA' from dual union all
select 41092,'NOAA' from dual union all
select 41093,'NOAA' from dual union all
select 41094,'NOAA' from dual union all
select 41095,'NOAA' from dual union all
select 41096,'NOAA' from dual union all
select 41097,'NOAA' from dual union all
select 41098,'KVR' from dual union all
select 41099,'PVO' from dual union all
select 411,'USAF' from dual union all
select 4110,'NASA' from dual union all
select 41100,'RVSNR' from dual union all
select 41101,'OSC' from dual union all
select 41102,'RVSNR' from dual union all
select 41103,'PLA' from dual union all
select 41104,'CASC' from dual union all
select 41105,'NPOL' from dual union all
select 41105,'RGMS' from dual union all
select 41106,'NPOL' from dual union all
select 41107,'KVR' from dual union all
select 41108,'NPOL' from dual union all
select 41109,'KVR' from dual union all
select 4111,'NRO' from dual union all
select 41110,'KVR' from dual;
insert into SATELLITE_ORG
select 41111,'KVR' from dual union all
select 41112,'KVR' from dual union all
select 41113,'KVR' from dual union all
select 41114,'KVR' from dual union all
select 41115,'KVR' from dual union all
select 41116,'MO' from dual union all
select 41117,'MO' from dual union all
select 41118,'AE' from dual union all
select 41119,'CASC' from dual union all
select 4112,'USAF' from dual union all
select 41120,'CASC' from dual union all
select 41121,'KVR' from dual union all
select 41122,'KHRR' from dual union all
select 41123,'KHRR' from dual union all
select 41124,'FKA' from dual union all
select 41125,'VVKO' from dual union all
select 41126,'NOAA' from dual union all
select 41127,'NOAA' from dual union all
select 41128,'NOAA' from dual union all
select 41129,'NOAA' from dual union all
select 4113,'USAF' from dual union all
select 41130,'NOAA' from dual union all
select 41131,'NOAA' from dual union all
select 41132,'NOAA' from dual union all
select 41133,'NOAA' from dual union all
select 41134,'NOAA' from dual union all
select 41135,'NOAA' from dual union all
select 41136,'NOAA' from dual union all
select 41137,'NOAA' from dual union all
select 41138,'NOAA' from dual union all
select 41139,'NOAA' from dual union all
select 4114,'ESRO' from dual union all
select 41140,'NOAA' from dual union all
select 41141,'NOAA' from dual union all
select 41142,'NOAA' from dual union all
select 41143,'NOAA' from dual union all
select 41144,'NOAA' from dual union all
select 41145,'NOAA' from dual union all
select 41146,'NOAA' from dual union all
select 41147,'NOAA' from dual union all
select 41148,'NOAA' from dual union all
select 41149,'NOAA' from dual union all
select 4115,'USAF' from dual union all
select 41150,'NOAA' from dual union all
select 41151,'NOAA' from dual union all
select 41152,'NOAA' from dual union all
select 41153,'NOAA' from dual union all
select 41154,'NOAA' from dual union all
select 41155,'NOAA' from dual union all
select 41156,'NOAA' from dual union all
select 41157,'NOAA' from dual union all
select 41158,'NOAA' from dual union all
select 41159,'NOAA' from dual union all
select 4116,'ESRO' from dual union all
select 41160,'NOAA' from dual union all
select 41161,'NOAA' from dual union all
select 41162,'NOAA' from dual union all
select 41163,'NOAA' from dual union all
select 41164,'NOAA' from dual union all
select 41165,'NOAA' from dual union all
select 41166,'NTU' from dual union all
select 41167,'NUS' from dual union all
select 41168,'MICSG' from dual union all
select 41169,'STELEC' from dual union all
select 4117,'NASA' from dual union all
select 41170,'NUS' from dual union all
select 41171,'NTU' from dual union all
select 41172,'ISRO' from dual union all
select 41173,'CAS' from dual union all
select 41174,'GSAEU' from dual union all
select 41175,'GSAEU' from dual union all
select 41176,'NPOL' from dual union all
select 41177,'FKA' from dual union all
select 41178,'RVSNR' from dual union all
select 41179,'ORBC' from dual union all
select 4118,'ESRO' from dual union all
select 41180,'ORBC' from dual union all
select 41181,'ORBC' from dual union all
select 41182,'ORBC' from dual union all
select 41183,'ORBC' from dual union all
select 41184,'ORBC' from dual union all
select 41185,'ORBC' from dual union all
select 41186,'ORBC' from dual union all
select 41187,'ORBC' from dual union all
select 41188,'ORBC' from dual union all
select 41189,'ORBC' from dual union all
select 4119,'GMS' from dual union all
select 4119,'MO' from dual union all
select 41190,'VS' from dual union all
select 41191,'KS' from dual union all
select 41192,'KHRR' from dual union all
select 41193,'KHRR' from dual union all
select 41194,'CNSAS' from dual union all
select 41195,'CASC' from dual union all
select 41196,'NOAA' from dual union all
select 41197,'NOAA' from dual union all
select 41198,'NOAA' from dual union all
select 41199,'NOAA' from dual union all
select 412,'USAF' from dual union all
select 4120,'RVSN' from dual;
insert into SATELLITE_ORG
select 41200,'NOAA' from dual union all
select 41201,'NOAA' from dual union all
select 41202,'NOAA' from dual union all
select 41203,'NOAA' from dual union all
select 41204,'NOAA' from dual union all
select 41205,'NOAA' from dual union all
select 41206,'NOAA' from dual union all
select 41207,'NOAA' from dual union all
select 41208,'NOAA' from dual union all
select 41209,'NOAA' from dual union all
select 4121,'USAF' from dual union all
select 41210,'NOAA' from dual union all
select 41211,'NOAA' from dual union all
select 41212,'NOAA' from dual union all
select 41213,'NOAA' from dual union all
select 41214,'NOAA' from dual union all
select 41215,'NOAA' from dual union all
select 41216,'NOAA' from dual union all
select 41217,'NOAA' from dual union all
select 41218,'NOAA' from dual union all
select 41219,'NOAA' from dual union all
select 4122,'TSKBEM' from dual union all
select 41220,'NOAA' from dual union all
select 41221,'NOAA' from dual union all
select 41222,'NOAA' from dual union all
select 41223,'NOAA' from dual union all
select 41224,'NOAA' from dual union all
select 41225,'NOAA' from dual union all
select 41226,'NOAA' from dual union all
select 41227,'NOAA' from dual union all
select 41228,'NOAA' from dual union all
select 41229,'NOAA' from dual union all
select 4123,'RVSN' from dual union all
select 41230,'NOAA' from dual union all
select 41231,'NOAA' from dual union all
select 41232,'NOAA' from dual union all
select 41233,'NOAA' from dual union all
select 41234,'NOAA' from dual union all
select 41235,'NOAA' from dual union all
select 41236,'NOAA' from dual union all
select 41237,'NOAA' from dual union all
select 41238,'ZTZM' from dual union all
select 41239,'CASC' from dual union all
select 4124,'TSKBEM' from dual union all
select 41240,'CNES' from dual union all
select 41240,'NOAA' from dual union all
select 41241,'ISRO' from dual union all
select 41242,'ISRO' from dual union all
select 41243,'NOAA' from dual union all
select 41244,'NOAA' from dual union all
select 41245,'NOAA' from dual union all
select 41246,'NOAA' from dual union all
select 41247,'NOAA' from dual union all
select 41248,'NOAA' from dual union all
select 41249,'NOAA' from dual union all
select 4125,'RVSN' from dual union all
select 41250,'NOAA' from dual union all
select 41251,'NOAA' from dual union all
select 41252,'NOAA' from dual union all
select 41253,'NOAA' from dual union all
select 41254,'NOAA' from dual union all
select 41255,'NOAA' from dual union all
select 41256,'NOAA' from dual union all
select 41257,'NOAA' from dual union all
select 41258,'NOAA' from dual union all
select 41259,'NOAA' from dual union all
select 4126,'TSKBEM' from dual union all
select 41260,'NOAA' from dual union all
select 41261,'NOAA' from dual union all
select 41262,'NOAA' from dual union all
select 41263,'NOAA' from dual union all
select 41264,'NOAA' from dual union all
select 41265,'NOAA' from dual union all
select 41266,'NOAA' from dual union all
select 41267,'NOAA' from dual union all
select 41268,'NOAA' from dual union all
select 41269,'NOAA' from dual union all
select 4127,'RVSN' from dual union all
select 41270,'NOAA' from dual union all
select 41271,'NOAA' from dual union all
select 41272,'NOAA' from dual union all
select 41273,'NOAA' from dual union all
select 41274,'NOAA' from dual union all
select 41275,'NOAA' from dual union all
select 41276,'NOAA' from dual union all
select 41277,'NOAA' from dual union all
select 41278,'NOAA' from dual union all
select 41279,'NOAA' from dual union all
select 4128,'IK' from dual union all
select 41280,'NOAA' from dual union all
select 41281,'NOAA' from dual union all
select 41282,'NOAA' from dual union all
select 41283,'NOAA' from dual union all
select 41284,'NOAA' from dual union all
select 41285,'NOAA' from dual union all
select 41286,'NOAA' from dual union all
select 41287,'NOAA' from dual union all
select 41288,'NOAA' from dual union all
select 41289,'NOAA' from dual union all
select 4129,'RVSN' from dual;
insert into SATELLITE_ORG
select 41290,'NOAA' from dual union all
select 41291,'NOAA' from dual union all
select 41292,'NOAA' from dual union all
select 41293,'NOAA' from dual union all
select 41294,'NOAA' from dual union all
select 41295,'NOAA' from dual union all
select 41296,'NOAA' from dual union all
select 41297,'NOAA' from dual union all
select 41298,'NOAA' from dual union all
select 41299,'NOAA' from dual union all
select 413,'USAF' from dual union all
select 4130,'MO' from dual union all
select 41300,'NOAA' from dual union all
select 41301,'NOAA' from dual union all
select 41302,'NOAA' from dual union all
select 41303,'NOAA' from dual union all
select 41304,'NOAA' from dual union all
select 41305,'NOAA' from dual union all
select 41306,'NOAA' from dual union all
select 41307,'NOAA' from dual union all
select 41308,'INTELS' from dual union all
select 41309,'AE' from dual union all
select 4131,'RVSN' from dual union all
select 41310,'EUTSA' from dual union all
select 41311,'KHRR' from dual union all
select 41312,'KHRR' from dual union all
select 41313,'TAMU' from dual union all
select 41314,'UT' from dual union all
select 41315,'CNSA' from dual union all
select 41316,'CASC' from dual union all
select 41317,'KHRR' from dual union all
select 41318,'KHRR' from dual union all
select 41319,'KHRR' from dual union all
select 4132,'USN' from dual union all
select 41320,'KHRR' from dual union all
select 41321,'KHRR' from dual union all
select 41322,'KHRR' from dual union all
select 41323,'KHRR' from dual union all
select 41324,'KHRR' from dual union all
select 41325,'CASC' from dual union all
select 41326,'FKA' from dual union all
select 41327,'FKA' from dual union all
select 41328,'USAF' from dual union all
select 41329,'LM' from dual union all
select 4133,'USAF' from dual union all
select 41330,'FKA' from dual union all
select 41330,'KVR' from dual union all
select 41331,'NPOL' from dual union all
select 41332,'NADA' from dual union all
select 41333,'NADA' from dual union all
select 41334,'NROC' from dual union all
select 41335,'DGGROW' from dual union all
select 41335,'ESA' from dual union all
select 41336,'EUROK' from dual union all
select 41337,'ISAS' from dual union all
select 41337,'JAXA' from dual union all
select 41338,'NAG' from dual union all
select 41339,'NAG' from dual union all
select 4134,'USAF' from dual union all
select 41340,'KYUT' from dual union all
select 41341,'JAXA' from dual union all
select 41342,'JAXA' from dual union all
select 41343,'NOAA' from dual union all
select 41344,'NOAA' from dual union all
select 41345,'NOAA' from dual union all
select 41346,'NOAA' from dual union all
select 41347,'NOAA' from dual union all
select 41348,'NOAA' from dual union all
select 41349,'NOAA' from dual union all
select 4135,'USAF' from dual union all
select 41350,'NOAA' from dual union all
select 41351,'NOAA' from dual union all
select 41352,'NOAA' from dual union all
select 41353,'NOAA' from dual union all
select 41354,'NOAA' from dual union all
select 41355,'NOAA' from dual union all
select 41356,'NOAA' from dual union all
select 41357,'NOAA' from dual union all
select 41358,'NOAA' from dual union all
select 41359,'NOAA' from dual union all
select 4136,'PVO' from dual union all
select 41360,'NOAA' from dual union all
select 41361,'NOAA' from dual union all
select 41362,'NOAA' from dual union all
select 41363,'NOAA' from dual union all
select 41364,'NOAA' from dual union all
select 41365,'NOAA' from dual union all
select 41366,'NOAA' from dual union all
select 41367,'NOAA' from dual union all
select 41368,'NOAA' from dual union all
select 41369,'NOAA' from dual union all
select 4137,'RVSN' from dual union all
select 41370,'NOAA' from dual union all
select 41371,'NOAA' from dual union all
select 41372,'NOAA' from dual union all
select 41373,'NOAA' from dual union all
select 41374,'NOAA' from dual union all
select 41375,'NOAA' from dual union all
select 41376,'NOAA' from dual union all
select 41377,'NOAA' from dual;
insert into SATELLITE_ORG
select 41378,'NOAA' from dual union all
select 41379,'NOAA' from dual union all
select 4138,'OKB10' from dual union all
select 41380,'SES' from dual union all
select 41381,'SPX' from dual union all
select 41382,'EUTSA' from dual union all
select 41383,'AE' from dual union all
select 41384,'ISRO' from dual union all
select 41385,'ISRO' from dual union all
select 41386,'TSSKB' from dual union all
select 41387,'KVR' from dual union all
select 41388,'ESA' from dual union all
select 41389,'KHRR' from dual union all
select 4139,'RVSN' from dual union all
select 41390,'KHRR' from dual union all
select 41391,'FKA' from dual union all
select 41392,'VVKO' from dual union all
select 41393,'OSC' from dual union all
select 41394,'KVR' from dual union all
select 41395,'RVSNR' from dual union all
select 41396,'NOAA' from dual union all
select 41397,'NOAA' from dual union all
select 41398,'NOAA' from dual union all
select 41399,'NOAA' from dual union all
select 414,'USAF' from dual union all
select 4140,'USAF' from dual union all
select 41400,'NOAA' from dual union all
select 41401,'NOAA' from dual union all
select 41402,'NOAA' from dual union all
select 41403,'NOAA' from dual union all
select 41404,'NOAA' from dual union all
select 41405,'NOAA' from dual union all
select 41406,'NOAA' from dual union all
select 41407,'NOAA' from dual union all
select 41408,'NOAA' from dual union all
select 41409,'NOAA' from dual union all
select 4141,'USAF' from dual union all
select 41410,'NOAA' from dual union all
select 41411,'NOAA' from dual union all
select 41412,'NOAA' from dual union all
select 41413,'NOAA' from dual union all
select 41414,'NOAA' from dual union all
select 41415,'NOAA' from dual union all
select 41416,'NOAA' from dual union all
select 41417,'NOAA' from dual union all
select 41418,'NOAA' from dual union all
select 41419,'NOAA' from dual union all
select 4142,'USAF' from dual union all
select 41420,'NOAA' from dual union all
select 41421,'NOAA' from dual union all
select 41422,'NOAA' from dual union all
select 41423,'NOAA' from dual union all
select 41424,'NOAA' from dual union all
select 41425,'NOAA' from dual union all
select 41426,'NOAA' from dual union all
select 41427,'NOAA' from dual union all
select 41428,'NOAA' from dual union all
select 41429,'NOAA' from dual union all
select 4143,'USAF' from dual union all
select 41430,'NOAA' from dual union all
select 41431,'NOAA' from dual union all
select 41432,'NOAA' from dual union all
select 41433,'NOAA' from dual union all
select 41434,'CNSA' from dual union all
select 41435,'CASC' from dual union all
select 41436,'FKA' from dual union all
select 41437,'RVSNR' from dual union all
select 41438,'ISAS' from dual union all
select 41438,'JAXA' from dual union all
select 41439,'ISAS' from dual union all
select 41439,'JAXA' from dual union all
select 4144,'USAF' from dual union all
select 41440,'ISAS' from dual union all
select 41440,'JAXA' from dual union all
select 41441,'ISAS' from dual union all
select 41441,'JAXA' from dual union all
select 41442,'ISAS' from dual union all
select 41442,'JAXA' from dual union all
select 41443,'ISAS' from dual union all
select 41443,'JAXA' from dual union all
select 41444,'ISAS' from dual union all
select 41444,'JAXA' from dual union all
select 41445,'ISAS' from dual union all
select 41445,'JAXA' from dual union all
select 41446,'ISAS' from dual union all
select 41446,'JAXA' from dual union all
select 41447,'ISAS' from dual union all
select 41447,'JAXA' from dual union all
select 41448,'CASC' from dual union all
select 41449,'CASC' from dual union all
select 4145,'USAF' from dual union all
select 41450,'CASC' from dual union all
select 41451,'CASC' from dual union all
select 41452,'SPX' from dual union all
select 41453,'SPX' from dual union all
select 41454,'SPX' from dual union all
select 41455,'CASC' from dual union all
select 41456,'DGGROW' from dual union all
select 41456,'ESA' from dual union all
select 41457,'CNES' from dual;
insert into SATELLITE_ORG
select 41458,'LIEGE' from dual union all
select 41459,'TORP' from dual union all
select 4146,'USAF' from dual union all
select 41460,'AAU' from dual union all
select 41461,'CNES' from dual union all
select 41462,'CASC' from dual union all
select 41463,'DOST' from dual union all
select 41463,'UPD' from dual union all
select 41464,'MGU' from dual union all
select 41464,'RAKA' from dual union all
select 41465,'SGAU' from dual union all
select 41466,'SGAU' from dual union all
select 41467,'RVSNR' from dual union all
select 41468,'RVSNR' from dual union all
select 41469,'ISRO' from dual union all
select 4147,'USAF' from dual union all
select 41470,'ISRO' from dual union all
select 41471,'SKPJ' from dual union all
select 41472,'SPX' from dual union all
select 41473,'PLA' from dual union all
select 41474,'UCO' from dual union all
select 41475,'UMI' from dual union all
select 41476,'STMS' from dual union all
select 41477,'NASA' from dual union all
select 41478,'NASA' from dual union all
select 41479,'PLABS' from dual union all
select 4148,'USAF' from dual union all
select 41480,'PLABS' from dual union all
select 41481,'PLABS' from dual union all
select 41482,'PLABS' from dual union all
select 41483,'PLABS' from dual union all
select 41484,'PLABS' from dual union all
select 41485,'SPIRE' from dual union all
select 41486,'PLABS' from dual union all
select 41487,'PLABS' from dual union all
select 41488,'SPIRE' from dual union all
select 41489,'SPIRE' from dual union all
select 4149,'USAF' from dual union all
select 41490,'SPIRE' from dual union all
select 41491,'NOAA' from dual union all
select 41492,'NOAA' from dual union all
select 41493,'NOAA' from dual union all
select 41494,'NOAA' from dual union all
select 41495,'NOAA' from dual union all
select 41496,'NOAA' from dual union all
select 41497,'NOAA' from dual union all
select 41498,'NOAA' from dual union all
select 41499,'NOAA' from dual union all
select 415,'USAF' from dual union all
select 4150,'RVSN' from dual union all
select 41500,'NOAA' from dual union all
select 41501,'NOAA' from dual union all
select 41502,'NOAA' from dual union all
select 41503,'NOAA' from dual union all
select 41504,'NOAA' from dual union all
select 41505,'NOAA' from dual union all
select 41506,'NOAA' from dual union all
select 41507,'NOAA' from dual union all
select 41508,'NOAA' from dual union all
select 41509,'NOAA' from dual union all
select 4151,'RVSN' from dual union all
select 41510,'NOAA' from dual union all
select 41511,'NOAA' from dual union all
select 41512,'NOAA' from dual union all
select 41513,'NOAA' from dual union all
select 41514,'NOAA' from dual union all
select 41515,'NOAA' from dual union all
select 41516,'NOAA' from dual union all
select 41517,'NOAA' from dual union all
select 41518,'NOAA' from dual union all
select 41519,'NOAA' from dual union all
select 4152,'USAF' from dual union all
select 41520,'NOAA' from dual union all
select 41521,'NOAA' from dual union all
select 41522,'NOAA' from dual union all
select 41523,'NOAA' from dual union all
select 41524,'NOAA' from dual union all
select 41525,'NOAA' from dual union all
select 41526,'NOAA' from dual union all
select 41527,'NOAA' from dual union all
select 41528,'NOAA' from dual union all
select 41529,'NOAA' from dual union all
select 4153,'USAF' from dual union all
select 41530,'NOAA' from dual union all
select 41531,'NOAA' from dual union all
select 41532,'NOAA' from dual union all
select 41533,'NOAA' from dual union all
select 41534,'NOAA' from dual union all
select 41535,'NOAA' from dual union all
select 41536,'NOAA' from dual union all
select 41537,'NOAA' from dual union all
select 41538,'NOAA' from dual union all
select 41539,'NOAA' from dual union all
select 4154,'USAF' from dual union all
select 41540,'NOAA' from dual union all
select 41541,'TSSKB' from dual union all
select 41542,'TSSKB' from dual union all
select 41543,'KHRR' from dual union all
select 41544,'KHRR' from dual union all
select 41545,'KHRR' from dual;
insert into SATELLITE_ORG
select 41546,'KHRR' from dual union all
select 41547,'KHRR' from dual union all
select 41548,'KHRR' from dual union all
select 41549,'GSAEU' from dual union all
select 4155,'USAF' from dual union all
select 41550,'GSAEU' from dual union all
select 41551,'NPOL' from dual union all
select 41552,'THAIC' from dual union all
select 41553,'SPX' from dual union all
select 41554,'FKA' from dual union all
select 41554,'KVR' from dual union all
select 41555,'NPOL' from dual union all
select 41556,'GZB' from dual union all
select 41557,'SATELOG' from dual union all
select 41558,'SATELOG' from dual union all
select 41559,'CNSA' from dual union all
select 4156,'USAF' from dual union all
select 41560,'KVR' from dual union all
select 41561,'KVR' from dual union all
select 41562,'KVR' from dual union all
select 41563,'PLABS' from dual union all
select 41564,'PLABS' from dual union all
select 41565,'PLABS' from dual union all
select 41566,'PLABS' from dual union all
select 41567,'PLABS' from dual union all
select 41568,'PLABS' from dual union all
select 41569,'PLABS' from dual union all
select 4157,'USAF' from dual union all
select 41570,'PLABS' from dual union all
select 41571,'PLABS' from dual union all
select 41572,'PLABS' from dual union all
select 41573,'PLABS' from dual union all
select 41574,'PLABS' from dual union all
select 41575,'PLABS' from dual union all
select 41576,'PLABS' from dual union all
select 41577,'PLABS' from dual union all
select 41578,'PLABS' from dual union all
select 41579,'TSVS' from dual union all
select 4158,'USAF' from dual union all
select 41580,'EUROK' from dual union all
select 41581,'DTVA' from dual union all
select 41581,'INTELS' from dual union all
select 41582,'KHRR' from dual union all
select 41583,'KHRR' from dual union all
select 41584,'NROC' from dual union all
select 41585,'BOE' from dual union all
select 41586,'CNSA' from dual union all
select 41587,'CASC' from dual union all
select 41588,'ABS' from dual union all
select 41589,'EUTSA' from dual union all
select 4159,'USAF' from dual union all
select 41590,'SPX' from dual union all
select 41591,'BRID' from dual union all
select 41592,'DISH' from dual union all
select 41593,'AE' from dual union all
select 41594,'AE' from dual union all
select 41595,'SPIRE' from dual union all
select 41596,'SPIRE' from dual union all
select 41597,'SPIRE' from dual union all
select 41598,'SPIRE' from dual union all
select 41599,'ISRO' from dual union all
select 416,'USAF' from dual union all
select 4160,'USAF' from dual union all
select 41600,'SATHYA' from dual union all
select 41601,'TBELLA' from dual union all
select 41602,'UTIAS0' from dual union all
select 41603,'LAPAN' from dual union all
select 41604,'DLR' from dual union all
select 41605,'COMDEV' from dual union all
select 41605,'DND' from dual union all
select 41606,'PLABS' from dual union all
select 41607,'COEP' from dual union all
select 41608,'PLABS' from dual union all
select 41609,'PLABS' from dual union all
select 4161,'USAF' from dual union all
select 41610,'PLABS' from dual union all
select 41611,'PLABS' from dual union all
select 41612,'PLABS' from dual union all
select 41613,'PLABS' from dual union all
select 41614,'PLABS' from dual union all
select 41615,'PLABS' from dual union all
select 41616,'PLABS' from dual union all
select 41617,'PLABS' from dual union all
select 41618,'PLABS' from dual union all
select 41619,'TUB1' from dual union all
select 4162,'USAF' from dual union all
select 41620,'ISRO' from dual union all
select 41621,'ISRO' from dual union all
select 41622,'USN' from dual union all
select 41623,'LM' from dual union all
select 41624,'CASC' from dual union all
select 41625,'NWPU' from dual union all
select 41626,'CASC' from dual union all
select 41627,'CASC' from dual union all
select 41628,'MAI' from dual union all
select 41629,'CASC' from dual union all
select 4163,'USAF' from dual union all
select 41630,'MAI' from dual union all
select 41631,'MAI' from dual union all
select 41632,'MAI' from dual;
insert into SATELLITE_ORG
select 41633,'MAI' from dual union all
select 41634,'CMA' from dual union all
select 41635,'CNSA' from dual union all
select 41636,'CASC' from dual union all
select 41637,'CASC' from dual union all
select 41638,'CASC' from dual union all
select 41639,'FKA' from dual union all
select 4164,'USAF' from dual union all
select 41640,'VVKO' from dual union all
select 41641,'KVR' from dual union all
select 41642,'KVR' from dual union all
select 41643,'KVR' from dual union all
select 41644,'KVR' from dual union all
select 41645,'KVR' from dual union all
select 41646,'KVR' from dual union all
select 41647,'KVR' from dual union all
select 41648,'KVR' from dual union all
select 41649,'KVR' from dual union all
select 4165,'USAF' from dual union all
select 41650,'KVR' from dual union all
select 41651,'KVR' from dual union all
select 41652,'NOAA' from dual union all
select 41653,'NOAA' from dual union all
select 41654,'NOAA' from dual union all
select 41655,'NOAA' from dual union all
select 41656,'NOAA' from dual union all
select 41657,'NOAA' from dual union all
select 41658,'NOAA' from dual union all
select 41659,'NOAA' from dual union all
select 4166,'USN' from dual union all
select 41660,'NOAA' from dual union all
select 41661,'NOAA' from dual union all
select 41662,'NOAA' from dual union all
select 41663,'NOAA' from dual union all
select 41664,'NOAA' from dual union all
select 41665,'NOAA' from dual union all
select 41666,'NOAA' from dual union all
select 41667,'NOAA' from dual union all
select 41668,'NOAA' from dual union all
select 41669,'NOAA' from dual union all
select 4167,'USAF' from dual union all
select 41670,'FKA' from dual union all
select 41671,'VVKO' from dual union all
select 41672,'SPX' from dual union all
select 41673,'SPX' from dual union all
select 41674,'SPX' from dual union all
select 41675,'RVSNR' from dual union all
select 41676,'RVSNR' from dual union all
select 41677,'RVSNR' from dual union all
select 41678,'RVSNR' from dual union all
select 41679,'RVSNR' from dual union all
select 4168,'USN' from dual union all
select 41680,'RVSNR' from dual union all
select 41681,'RVSNR' from dual union all
select 41682,'RVSNR' from dual union all
select 41683,'RVSNR' from dual union all
select 41684,'RVSNR' from dual union all
select 41685,'RVSNR' from dual union all
select 41686,'RVSNR' from dual union all
select 41687,'RVSNR' from dual union all
select 41688,'RVSNR' from dual union all
select 41689,'RVSNR' from dual union all
select 4169,'USAF' from dual union all
select 41690,'RVSNR' from dual union all
select 41691,'RVSNR' from dual union all
select 41692,'RVSNR' from dual union all
select 41693,'RVSNR' from dual union all
select 41694,'RVSNR' from dual union all
select 41695,'KVR' from dual union all
select 41696,'KVR' from dual union all
select 41697,'KVR' from dual union all
select 41698,'KVR' from dual union all
select 41699,'KVR' from dual union all
select 417,'USAF' from dual union all
select 4170,'USAF' from dual union all
select 41700,'KVR' from dual union all
select 41701,'KVR' from dual union all
select 41702,'KVR' from dual union all
select 41703,'KVR' from dual union all
select 41704,'KVR' from dual union all
select 41705,'NOAA' from dual union all
select 41706,'NOAA' from dual union all
select 41707,'NOAA' from dual union all
select 41708,'NOAA' from dual union all
select 41709,'NOAA' from dual union all
select 4171,'USAF' from dual union all
select 41710,'NOAA' from dual union all
select 41711,'NOAA' from dual union all
select 41712,'NOAA' from dual union all
select 41713,'NOAA' from dual union all
select 41714,'NOAA' from dual union all
select 41715,'NOAA' from dual union all
select 41716,'NOAA' from dual union all
select 41717,'NOAA' from dual union all
select 41718,'KVR' from dual union all
select 41719,'KVR' from dual union all
select 4172,'USAF' from dual union all
select 41720,'KVR' from dual union all
select 41721,'KVR' from dual union all
select 41722,'KVR' from dual;
insert into SATELLITE_ORG
select 41723,'KVR' from dual union all
select 41724,'NROC' from dual union all
select 41725,'CASC' from dual union all
select 41726,'CASC' from dual union all
select 41727,'GHJ' from dual union all
select 41728,'CNSA' from dual union all
select 41729,'SKPJ' from dual union all
select 4173,'USAF' from dual union all
select 41730,'SPX' from dual union all
select 41731,'CAS' from dual union all
select 41732,'UPC' from dual union all
select 41733,'CAS' from dual union all
select 41734,'USN' from dual union all
select 41735,'DGLO' from dual union all
select 41736,'DGLO' from dual union all
select 41737,'DGLO' from dual union all
select 41738,'DGLO' from dual union all
select 41739,'DGLO' from dual union all
select 4174,'USAF' from dual union all
select 41740,'DGLO' from dual union all
select 41741,'DGLO' from dual union all
select 41742,'DGLO' from dual union all
select 41743,'DGLO' from dual union all
select 41744,'USAF' from dual union all
select 41745,'USAF' from dual union all
select 41746,'BOE' from dual union all
select 41747,'INTELS' from dual union all
select 41748,'INTELS' from dual union all
select 41749,'AE' from dual union all
select 4175,'USAF' from dual union all
select 41750,'AE' from dual union all
select 41751,'NASA' from dual union all
select 41752,'ISRO' from dual union all
select 41753,'ISRO' from dual union all
select 41754,'NASA' from dual union all
select 41755,'NASA' from dual union all
select 41756,'NASA' from dual union all
select 41757,'NASA' from dual union all
select 41758,'LM' from dual union all
select 41759,'ISA' from dual union all
select 4176,'USAF' from dual union all
select 41760,'ISA' from dual union all
select 41761,'PLABS' from dual union all
select 41762,'PLABS' from dual union all
select 41763,'PLABS' from dual union all
select 41764,'PLABS' from dual union all
select 41765,'PLA' from dual union all
select 41766,'CASC' from dual union all
select 41767,'CASC' from dual union all
select 41768,'KVR' from dual union all
select 41769,'PLABS' from dual union all
select 4177,'USAF' from dual union all
select 41770,'MDPE' from dual union all
select 41771,'TBELLA' from dual union all
select 41772,'TBELLA' from dual union all
select 41773,'TBELLA' from dual union all
select 41774,'TBELLA' from dual union all
select 41775,'ESA' from dual union all
select 41776,'PLABS' from dual union all
select 41777,'PLABS' from dual union all
select 41778,'CASC' from dual union all
select 41779,'CASC' from dual union all
select 4178,'USAF' from dual union all
select 41780,'CASC' from dual union all
select 41781,'CASC' from dual union all
select 41782,'PLABS' from dual union all
select 41783,'IITB' from dual union all
select 41784,'PESU' from dual union all
select 41785,'ASAL' from dual union all
select 41786,'ASAL' from dual union all
select 41787,'SFLIN' from dual union all
select 41788,'UTIAS0' from dual union all
select 41789,'ASAL' from dual union all
select 41789,'UKSA' from dual union all
select 4179,'USAF' from dual union all
select 41790,'ISRO' from dual union all
select 41791,'ISRO' from dual union all
select 41792,'ISRO' from dual union all
select 41793,'ISRO' from dual union all
select 41794,'NBN' from dual union all
select 41795,'AE' from dual union all
select 41796,'AE' from dual union all
select 41797,'ISRO' from dual union all
select 41798,'ESA' from dual union all
select 41799,'ESA' from dual union all
select 418,'USAF' from dual union all
select 4180,'USAF' from dual union all
select 41800,'ESA' from dual union all
select 41801,'ESA' from dual union all
select 41802,'ESA' from dual union all
select 41803,'ESA' from dual union all
select 41804,'ESA' from dual union all
select 41805,'KVR' from dual union all
select 41806,'KVR' from dual union all
select 41807,'KVR' from dual union all
select 41808,'KVR' from dual union all
select 41809,'KVR' from dual union all
select 4181,'USAF' from dual union all
select 41810,'KVR' from dual union all
select 41811,'KVR' from dual;
insert into SATELLITE_ORG
select 41812,'PLA' from dual union all
select 41813,'CASC' from dual union all
select 41814,'CASC' from dual union all
select 41815,'CASC' from dual union all
select 41816,'CASC' from dual union all
select 41817,'CASC' from dual union all
select 41818,'OSC' from dual union all
select 41819,'OSC' from dual union all
select 4182,'MO' from dual union all
select 41820,'FKA' from dual union all
select 41821,'VVKO' from dual union all
select 41822,'CASC' from dual union all
select 41823,'CASC' from dual union all
select 41824,'CASC' from dual union all
select 41825,'CASC' from dual union all
select 41826,'CASC' from dual union all
select 41827,'CASC' from dual union all
select 41828,'CASC' from dual union all
select 41829,'CASC' from dual union all
select 4183,'RVSN' from dual union all
select 41830,'CASC' from dual union all
select 41831,'CASC' from dual union all
select 41832,'CASC' from dual union all
select 41833,'CASC' from dual union all
select 41834,'PLA' from dual union all
select 41835,'USAF' from dual union all
select 41836,'JMA' from dual union all
select 41837,'JAXA' from dual union all
select 41838,'CASC' from dual union all
select 41839,'MAI' from dual union all
select 4184,'PVO' from dual union all
select 41840,'MAI' from dual union all
select 41841,'CASC' from dual union all
select 41842,'CTYK' from dual union all
select 41843,'ZDFH' from dual union all
select 41844,'ZDFH' from dual union all
select 41845,'ZHELZ' from dual union all
select 41846,'CASC' from dual union all
select 41847,'KCHT' from dual union all
select 41848,'DGLO' from dual union all
select 41849,'APL' from dual union all
select 41849,'NASA' from dual union all
select 4185,'RVSN' from dual union all
select 41850,'USAF' from dual union all
select 41851,'CALP' from dual union all
select 41851,'USAF' from dual union all
select 41852,'AERO' from dual union all
select 41853,'AERO' from dual union all
select 41854,'DOD' from dual union all
select 41855,'DOD' from dual union all
select 41856,'LM' from dual union all
select 41857,'CASC' from dual union all
select 41858,'CNSA' from dual union all
select 41859,'GSAEU' from dual union all
select 4186,'NRO' from dual union all
select 41860,'GSAEU' from dual union all
select 41861,'GSAEU' from dual union all
select 41862,'GSAEU' from dual union all
select 41863,'AE' from dual union all
select 41864,'FKA' from dual union all
select 41865,'VVKO' from dual union all
select 41866,'NOAA' from dual union all
select 41867,'LM' from dual union all
select 41868,'PLA' from dual union all
select 41869,'PLA' from dual union all
select 4187,'USAF' from dual union all
select 41870,'CASC' from dual union all
select 41871,'SPIRE' from dual union all
select 41872,'SPIRE' from dual union all
select 41873,'SPIRE' from dual union all
select 41874,'SPIRE' from dual union all
select 41875,'MSBTR' from dual union all
select 41876,'AE' from dual union all
select 41877,'ISRO' from dual union all
select 41878,'ISRO' from dual union all
select 41879,'USAF' from dual union all
select 4188,'USAF' from dual union all
select 41880,'BOE' from dual union all
select 41881,'JAXA' from dual union all
select 41882,'CMA' from dual union all
select 41883,'CASC' from dual union all
select 41884,'NASA' from dual union all
select 41884,'SWRI' from dual union all
select 41885,'NASA' from dual union all
select 41885,'SWRI' from dual union all
select 41886,'NASA' from dual union all
select 41886,'SWRI' from dual union all
select 41887,'NASA' from dual union all
select 41887,'SWRI' from dual union all
select 41888,'NASA' from dual union all
select 41888,'SWRI' from dual union all
select 41889,'NASA' from dual union all
select 41889,'SWRI' from dual union all
select 4189,'USAF' from dual union all
select 41890,'NASA' from dual union all
select 41890,'SWRI' from dual union all
select 41891,'NASA' from dual union all
select 41891,'SWRI' from dual union all
select 41892,'OSC' from dual union all
select 41893,'ECHO' from dual;
insert into SATELLITE_ORG
select 41894,'LM' from dual union all
select 41895,'SHIZ' from dual union all
select 41896,'JAXA' from dual union all
select 41897,'JAXA' from dual union all
select 41898,'CMA' from dual union all
select 41898,'SIMIT' from dual union all
select 41899,'SIMIT' from dual union all
select 419,'USAF' from dual union all
select 4190,'USAF' from dual union all
select 41900,'SIMIT' from dual union all
select 41901,'SIMIT' from dual union all
select 41902,'CNSA' from dual union all
select 41903,'SKPJ' from dual union all
select 41904,'TELB' from dual union all
select 41905,'AE' from dual union all
select 41906,'AE' from dual union all
select 41907,'BHSJ' from dual union all
select 41908,'BHSJ' from dual union all
select 41909,'BAYI' from dual union all
select 41909,'CASC' from dual union all
select 4191,'USAF' from dual union all
select 41910,'CNSA' from dual union all
select 41911,'CASC' from dual union all
select 41912,'CASC' from dual union all
select 41913,'CASIC' from dual union all
select 41914,'CAS' from dual union all
select 41915,'CATON' from dual union all
select 41916,'PLA2AC' from dual union all
select 41917,'IRID' from dual union all
select 41918,'IRID' from dual union all
select 41919,'IRID' from dual union all
select 4192,'USAF' from dual union all
select 41920,'IRID' from dual union all
select 41921,'IRID' from dual union all
select 41922,'IRID' from dual union all
select 41923,'IRID' from dual union all
select 41924,'IRID' from dual union all
select 41925,'IRID' from dual union all
select 41926,'IRID' from dual union all
select 41927,'USN' from dual union all
select 41928,'CASC' from dual union all
select 41929,'CASC' from dual union all
select 4193,'USAF' from dual union all
select 41930,'NAKA' from dual union all
select 41930,'TOHO' from dual union all
select 41931,'ETEC' from dual union all
select 41931,'INPE' from dual union all
select 41932,'TSUKU' from dual union all
select 41933,'WASEDA' from dual union all
select 41934,'TOK' from dual union all
select 41935,'KYUT' from dual union all
select 41935,'NTU' from dual union all
select 41936,'GAUSS' from dual union all
select 41937,'USAF' from dual union all
select 41938,'LM' from dual union all
select 41939,'OSN' from dual union all
select 4194,'USAF' from dual union all
select 41940,'DSNC' from dual union all
select 41941,'JAXA' from dual union all
select 41942,'HISP' from dual union all
select 41943,'NPOL' from dual union all
select 41944,'TELK' from dual union all
select 41945,'DTVA' from dual union all
select 41945,'INTELS' from dual union all
select 41946,'AE' from dual union all
select 41947,'AE' from dual union all
select 41948,'ISRO' from dual union all
select 41949,'ISRO' from dual union all
select 4195,'USAF' from dual union all
select 41950,'PLABS' from dual union all
select 41951,'PLABS' from dual union all
select 41952,'PLABS' from dual union all
select 41953,'PLABS' from dual union all
select 41954,'ISRO' from dual union all
select 41955,'PLABS' from dual union all
select 41956,'PLABS' from dual union all
select 41957,'PLABS' from dual union all
select 41958,'PLABS' from dual union all
select 41959,'PLABS' from dual union all
select 4196,'USAF' from dual union all
select 41960,'PLABS' from dual union all
select 41961,'PLABS' from dual union all
select 41962,'PLABS' from dual union all
select 41963,'PLABS' from dual union all
select 41964,'PLABS' from dual union all
select 41965,'PLABS' from dual union all
select 41966,'PLABS' from dual union all
select 41967,'PLABS' from dual union all
select 41968,'PLABS' from dual union all
select 41969,'PLABS' from dual union all
select 4197,'USAF' from dual union all
select 41970,'PLABS' from dual union all
select 41971,'PLABS' from dual union all
select 41972,'PLABS' from dual union all
select 41973,'PLABS' from dual union all
select 41974,'PLABS' from dual union all
select 41975,'PLABS' from dual union all
select 41976,'PLABS' from dual union all
select 41977,'PLABS' from dual union all
select 41978,'PLABS' from dual;
insert into SATELLITE_ORG
select 41979,'PLABS' from dual union all
select 4198,'USAF' from dual union all
select 41980,'PLABS' from dual union all
select 41981,'PLABS' from dual union all
select 41982,'PLABS' from dual union all
select 41983,'PLABS' from dual union all
select 41984,'PLABS' from dual union all
select 41985,'PLABS' from dual union all
select 41986,'PLABS' from dual union all
select 41987,'PLABS' from dual union all
select 41988,'PLABS' from dual union all
select 41989,'PLABS' from dual union all
select 4199,'USAF' from dual union all
select 41990,'PLABS' from dual union all
select 41991,'SPIRE' from dual union all
select 41992,'SPIRE' from dual union all
select 41993,'SPIRE' from dual union all
select 41994,'SPIRE' from dual union all
select 41995,'SPIRE' from dual union all
select 41996,'SPIRE' from dual union all
select 41997,'SPIRE' from dual union all
select 41998,'SPIRE' from dual union all
select 41999,'BGU' from dual union all
select 42,'OKB1' from dual union all
select 420,'USAF' from dual union all
select 4200,'USAF' from dual union all
select 42000,'SPHARS' from dual union all
select 42001,'PLABS' from dual union all
select 42002,'PLABS' from dual union all
select 42003,'PLABS' from dual union all
select 42004,'PLABS' from dual union all
select 42005,'PLABS' from dual union all
select 42006,'PLABS' from dual union all
select 42007,'PLABS' from dual union all
select 42008,'PLABS' from dual union all
select 42009,'PLABS' from dual union all
select 4201,'USAF' from dual union all
select 42010,'PLABS' from dual union all
select 42011,'PLABS' from dual union all
select 42012,'PLABS' from dual union all
select 42013,'PLABS' from dual union all
select 42014,'PLABS' from dual union all
select 42015,'ISISBV' from dual union all
select 42015,'TNO' from dual union all
select 42016,'KAZNU' from dual union all
select 42017,'EIAST' from dual union all
select 42018,'PLABS' from dual union all
select 42019,'PLABS' from dual union all
select 4202,'USAF' from dual union all
select 42020,'PLABS' from dual union all
select 42021,'PLABS' from dual union all
select 42022,'PLABS' from dual union all
select 42023,'PLABS' from dual union all
select 42024,'PLABS' from dual union all
select 42025,'PLABS' from dual union all
select 42026,'PLABS' from dual union all
select 42027,'PLABS' from dual union all
select 42028,'PLABS' from dual union all
select 42029,'PLABS' from dual union all
select 4203,'USAF' from dual union all
select 42030,'PLABS' from dual union all
select 42031,'PLABS' from dual union all
select 42032,'PLABS' from dual union all
select 42033,'PLABS' from dual union all
select 42034,'PLABS' from dual union all
select 42035,'PLABS' from dual union all
select 42036,'PLABS' from dual union all
select 42037,'PLABS' from dual union all
select 42038,'PLABS' from dual union all
select 42039,'PLABS' from dual union all
select 4204,'USAF' from dual union all
select 42040,'PLABS' from dual union all
select 42041,'PLABS' from dual union all
select 42042,'PLABS' from dual union all
select 42043,'PLABS' from dual union all
select 42044,'PLABS' from dual union all
select 42045,'PLABS' from dual union all
select 42046,'PLABS' from dual union all
select 42047,'PLABS' from dual union all
select 42048,'PLABS' from dual union all
select 42049,'PLABS' from dual union all
select 4205,'USAF' from dual union all
select 42050,'PLABS' from dual union all
select 42051,'PLABS' from dual union all
select 42052,'ISRO' from dual union all
select 42053,'SPX' from dual union all
select 42054,'SPX' from dual union all
select 42055,'SPX' from dual union all
select 42056,'FKA' from dual union all
select 42057,'VVKO' from dual union all
select 42058,'NROC' from dual union all
select 42058,'USN' from dual union all
select 42059,'SPIRE' from dual union all
select 4206,'USAF' from dual union all
select 42060,'VS' from dual union all
select 42061,'CASIC' from dual union all
select 42062,'CASIC' from dual union all
select 42063,'DGGROW' from dual union all
select 42063,'ESA' from dual union all
select 42064,'AE' from dual;
insert into SATELLITE_ORG
select 42065,'NROC' from dual union all
select 42065,'USN' from dual union all
select 42066,'NASA' from dual union all
select 42066,'SJSU' from dual union all
select 42067,'SPIRE' from dual union all
select 42068,'SPIRE' from dual union all
select 42069,'SPIRE' from dual union all
select 4207,'USAF' from dual union all
select 42070,'ECHOC' from dual union all
select 42071,'SPX' from dual union all
select 42072,'CSICE' from dual union all
select 42073,'MHITO' from dual union all
select 42074,'CSICE' from dual union all
select 42075,'USAF' from dual union all
select 42076,'BOE' from dual union all
select 42077,'CASIC' from dual union all
select 42078,'NASA' from dual union all
select 42079,'NASA' from dual union all
select 4208,'USAF' from dual union all
select 42080,'NASA' from dual union all
select 42081,'NASA' from dual union all
select 42082,'NASA' from dual union all
select 42083,'NASA' from dual union all
select 42084,'NASA' from dual union all
select 42085,'NASA' from dual union all
select 42086,'NASA' from dual union all
select 42087,'NASA' from dual union all
select 42088,'NASA' from dual union all
select 42089,'NASA' from dual union all
select 4209,'USAF' from dual union all
select 42090,'NASA' from dual union all
select 42091,'NASA' from dual union all
select 42092,'NASA' from dual union all
select 42093,'NASA' from dual union all
select 42094,'NASA' from dual union all
select 42095,'MO' from dual union all
select 42096,'MO' from dual union all
select 42097,'MO' from dual union all
select 42098,'MO' from dual union all
select 42099,'MO' from dual union all
select 421,'USAF' from dual union all
select 4210,'USAF' from dual union all
select 42100,'MO' from dual union all
select 42101,'MO' from dual union all
select 42102,'MO' from dual union all
select 42103,'MO' from dual union all
select 42104,'MO' from dual union all
select 42105,'MO' from dual union all
select 42106,'MO' from dual union all
select 42107,'MO' from dual union all
select 42108,'MO' from dual union all
select 42109,'MO' from dual union all
select 4211,'USAF' from dual union all
select 42110,'MO' from dual union all
select 42111,'MO' from dual union all
select 42112,'MO' from dual union all
select 42113,'MO' from dual union all
select 42114,'MO' from dual union all
select 42115,'MO' from dual union all
select 42116,'MO' from dual union all
select 42117,'MO' from dual union all
select 42118,'MO' from dual union all
select 42119,'MO' from dual union all
select 4212,'USAF' from dual union all
select 42120,'MO' from dual union all
select 42121,'MO' from dual union all
select 42122,'MO' from dual union all
select 42123,'MO' from dual union all
select 42124,'MO' from dual union all
select 42125,'MO' from dual union all
select 42126,'MO' from dual union all
select 42127,'MO' from dual union all
select 42128,'MO' from dual union all
select 42129,'MO' from dual union all
select 4213,'USAF' from dual union all
select 42130,'MO' from dual union all
select 42131,'MO' from dual union all
select 42132,'MO' from dual union all
select 42133,'MO' from dual union all
select 42134,'MO' from dual union all
select 42135,'MO' from dual union all
select 42136,'MO' from dual union all
select 42137,'MO' from dual union all
select 42138,'MO' from dual union all
select 42139,'MO' from dual union all
select 4214,'USAF' from dual union all
select 42140,'MO' from dual union all
select 42141,'MO' from dual union all
select 42142,'MO' from dual union all
select 42143,'MO' from dual union all
select 42144,'MO' from dual union all
select 42145,'MO' from dual union all
select 42146,'MO' from dual union all
select 42147,'MO' from dual union all
select 42148,'MO' from dual union all
select 42149,'MO' from dual union all
select 4215,'USAF' from dual union all
select 42150,'MO' from dual union all
select 42151,'MO' from dual union all
select 42152,'MO' from dual;
insert into SATELLITE_ORG
select 42153,'MO' from dual union all
select 42154,'MO' from dual union all
select 42155,'MO' from dual union all
select 42156,'MO' from dual union all
select 42157,'MO' from dual union all
select 42158,'MO' from dual union all
select 42159,'MO' from dual union all
select 4216,'USAF' from dual union all
select 42160,'MO' from dual union all
select 42161,'MO' from dual union all
select 42162,'MO' from dual union all
select 42163,'MO' from dual union all
select 42164,'MO' from dual union all
select 42165,'MO' from dual union all
select 42166,'MO' from dual union all
select 42167,'MO' from dual union all
select 42168,'MO' from dual union all
select 42169,'MO' from dual union all
select 4217,'USAF' from dual union all
select 42170,'MO' from dual union all
select 42171,'MO' from dual union all
select 42172,'MO' from dual union all
select 42173,'MO' from dual union all
select 42174,'MO' from dual union all
select 42175,'MO' from dual union all
select 42176,'MO' from dual union all
select 42177,'MO' from dual union all
select 42178,'MO' from dual union all
select 42179,'MO' from dual union all
select 4218,'USAF' from dual union all
select 42180,'MO' from dual union all
select 42181,'MO' from dual union all
select 42182,'MO' from dual union all
select 42183,'MO' from dual union all
select 42184,'MO' from dual union all
select 42185,'MO' from dual union all
select 42186,'MO' from dual union all
select 42187,'MO' from dual union all
select 42188,'MO' from dual union all
select 42189,'MO' from dual union all
select 4219,'PVO' from dual union all
select 4219,'YUZH' from dual union all
select 42190,'MO' from dual union all
select 42191,'MO' from dual union all
select 42192,'MO' from dual union all
select 42193,'MO' from dual union all
select 42194,'MO' from dual union all
select 42195,'MO' from dual union all
select 42196,'MO' from dual union all
select 42197,'MO' from dual union all
select 42198,'MO' from dual union all
select 42199,'MO' from dual union all
select 422,'OKB1' from dual union all
select 4220,'RVSN' from dual union all
select 42200,'MO' from dual union all
select 42201,'MO' from dual union all
select 42202,'MO' from dual union all
select 42203,'MO' from dual union all
select 42204,'MO' from dual union all
select 42205,'MO' from dual union all
select 42206,'MO' from dual union all
select 42207,'MO' from dual union all
select 42208,'MO' from dual union all
select 42209,'MO' from dual union all
select 4221,'DFVLR' from dual union all
select 42210,'MO' from dual union all
select 42211,'MO' from dual union all
select 42212,'MO' from dual union all
select 42213,'MO' from dual union all
select 42214,'MO' from dual union all
select 42215,'MO' from dual union all
select 42216,'MO' from dual union all
select 42217,'MO' from dual union all
select 42218,'MO' from dual union all
select 42219,'MO' from dual union all
select 4222,'NASA' from dual union all
select 42220,'MO' from dual union all
select 42221,'MO' from dual union all
select 42222,'MO' from dual union all
select 42223,'MO' from dual union all
select 42224,'MO' from dual union all
select 42225,'MO' from dual union all
select 42226,'MO' from dual union all
select 42227,'MO' from dual union all
select 42228,'MO' from dual union all
select 42229,'MO' from dual union all
select 4223,'MO' from dual union all
select 42230,'MO' from dual union all
select 42231,'MO' from dual union all
select 42232,'MO' from dual union all
select 42233,'MO' from dual union all
select 42234,'RVSNR' from dual union all
select 42235,'RVSNR' from dual union all
select 42236,'RVSNR' from dual union all
select 42237,'RVSNR' from dual union all
select 42238,'RVSNR' from dual union all
select 42239,'RVSNR' from dual union all
select 4224,'RVSN' from dual union all
select 42240,'RVSNR' from dual union all
select 42241,'RVSNR' from dual;
insert into SATELLITE_ORG
select 42242,'RVSNR' from dual union all
select 42243,'RVSNR' from dual union all
select 42244,'RVSNR' from dual union all
select 42245,'RVSNR' from dual union all
select 42246,'RVSNR' from dual union all
select 42247,'RVSNR' from dual union all
select 42248,'RVSNR' from dual union all
select 42249,'RVSNR' from dual union all
select 4225,'NASA' from dual union all
select 42250,'RVSNR' from dual union all
select 42251,'RVSNR' from dual union all
select 42252,'RVSNR' from dual union all
select 42253,'RVSNR' from dual union all
select 42254,'RVSNR' from dual union all
select 42255,'RVSNR' from dual union all
select 42256,'RVSNR' from dual union all
select 42257,'USAF' from dual union all
select 42258,'USAF' from dual union all
select 42259,'USAF' from dual union all
select 4226,'NASA' from dual union all
select 42260,'USAF' from dual union all
select 42261,'USAF' from dual union all
select 42262,'USAF' from dual union all
select 42263,'USAF' from dual union all
select 42264,'USAF' from dual union all
select 42265,'USAF' from dual union all
select 42266,'USAF' from dual union all
select 42267,'USAF' from dual union all
select 42268,'USAF' from dual union all
select 42269,'USAF' from dual union all
select 4227,'USAF' from dual union all
select 42270,'USAF' from dual union all
select 42271,'USAF' from dual union all
select 42272,'USAF' from dual union all
select 42273,'USAF' from dual union all
select 42274,'USAF' from dual union all
select 42275,'USAF' from dual union all
select 42276,'USAF' from dual union all
select 42277,'USAF' from dual union all
select 42278,'USAF' from dual union all
select 42279,'USAF' from dual union all
select 4228,'USAF' from dual union all
select 42280,'USAF' from dual union all
select 42281,'USAF' from dual union all
select 42282,'USAF' from dual union all
select 42283,'USAF' from dual union all
select 42284,'USAF' from dual union all
select 42285,'USAF' from dual union all
select 42286,'USAF' from dual union all
select 42287,'USAF' from dual union all
select 42288,'USAF' from dual union all
select 42289,'USAF' from dual union all
select 4229,'USAF' from dual union all
select 42290,'USAF' from dual union all
select 42291,'USAF' from dual union all
select 42292,'USAF' from dual union all
select 42293,'USAF' from dual union all
select 42294,'USAF' from dual union all
select 42295,'USAF' from dual union all
select 42296,'USAF' from dual union all
select 42297,'USAF' from dual union all
select 42298,'USAF' from dual union all
select 42299,'USAF' from dual union all
select 423,'RVSN' from dual union all
select 4230,'USAF' from dual union all
select 42300,'USAF' from dual union all
select 42301,'USAF' from dual union all
select 42302,'USAF' from dual union all
select 42303,'USAF' from dual union all
select 42304,'USAF' from dual union all
select 42305,'USAF' from dual union all
select 42306,'USAF' from dual union all
select 42307,'USAF' from dual union all
select 42308,'USAF' from dual union all
select 42309,'USAF' from dual union all
select 4231,'USAF' from dual union all
select 42310,'USAF' from dual union all
select 42311,'USAF' from dual union all
select 42312,'USAF' from dual union all
select 42313,'USAF' from dual union all
select 42314,'USAF' from dual union all
select 42315,'USAF' from dual union all
select 42316,'USAF' from dual union all
select 42317,'USAF' from dual union all
select 42318,'USAF' from dual union all
select 42319,'USAF' from dual union all
select 4232,'MO' from dual union all
select 42320,'USAF' from dual union all
select 42321,'USAF' from dual union all
select 42322,'USAF' from dual union all
select 42323,'USAF' from dual union all
select 42324,'USAF' from dual union all
select 42325,'USAF' from dual union all
select 42326,'USAF' from dual union all
select 42327,'USAF' from dual union all
select 42328,'USAF' from dual union all
select 42329,'USAF' from dual union all
select 4233,'RVSN' from dual union all
select 42330,'USAF' from dual union all
select 42331,'USAF' from dual;
insert into SATELLITE_ORG
select 42332,'NOAA' from dual union all
select 42333,'NOAA' from dual union all
select 42334,'NOAA' from dual union all
select 42335,'NOAA' from dual union all
select 42336,'NOAA' from dual union all
select 42337,'NOAA' from dual union all
select 42338,'NOAA' from dual union all
select 42339,'NOAA' from dual union all
select 4234,'MO' from dual union all
select 42340,'NOAA' from dual union all
select 42341,'NOAA' from dual union all
select 42342,'NOAA' from dual union all
select 42343,'NOAA' from dual union all
select 42344,'NOAA' from dual union all
select 42345,'NOAA' from dual union all
select 42346,'NOAA' from dual union all
select 42347,'NOAA' from dual union all
select 42348,'NOAA' from dual union all
select 42349,'NOAA' from dual union all
select 4235,'MO' from dual union all
select 42350,'NOAA' from dual union all
select 42351,'NOAA' from dual union all
select 42352,'NOAA' from dual union all
select 42353,'NOAA' from dual union all
select 42354,'NOAA' from dual union all
select 42355,'NOAA' from dual union all
select 42356,'NOAA' from dual union all
select 42357,'NOAA' from dual union all
select 42358,'NOAA' from dual union all
select 42359,'NOAA' from dual union all
select 4236,'TSKBEM' from dual union all
select 42360,'NOAA' from dual union all
select 42361,'NOAA' from dual union all
select 42362,'NOAA' from dual union all
select 42363,'NOAA' from dual union all
select 42364,'NOAA' from dual union all
select 42365,'NOAA' from dual union all
select 42366,'NOAA' from dual union all
select 42367,'NOAA' from dual union all
select 42368,'NOAA' from dual union all
select 42369,'NOAA' from dual union all
select 4237,'NRO' from dual union all
select 4237,'USN' from dual union all
select 42370,'NOAA' from dual union all
select 42371,'NOAA' from dual union all
select 42372,'NOAA' from dual union all
select 42373,'NOAA' from dual union all
select 42374,'NOAA' from dual union all
select 42375,'NOAA' from dual union all
select 42376,'NOAA' from dual union all
select 42377,'NOAA' from dual union all
select 42378,'NOAA' from dual union all
select 42379,'NOAA' from dual union all
select 4238,'USAF' from dual union all
select 42380,'NOAA' from dual union all
select 42381,'NOAA' from dual union all
select 42382,'NOAA' from dual union all
select 42383,'NOAA' from dual union all
select 42384,'NOAA' from dual union all
select 42385,'NOAA' from dual union all
select 42386,'NOAA' from dual union all
select 42387,'NOAA' from dual union all
select 42388,'NOAA' from dual union all
select 42389,'NOAA' from dual union all
select 4239,'USAF' from dual union all
select 42390,'NOAA' from dual union all
select 42391,'NOAA' from dual union all
select 42392,'NOAA' from dual union all
select 42393,'NOAA' from dual union all
select 42394,'NOAA' from dual union all
select 42395,'NOAA' from dual union all
select 42396,'NOAA' from dual union all
select 42397,'NOAA' from dual union all
select 42398,'NOAA' from dual union all
select 42399,'NOAA' from dual union all
select 424,'DRTE' from dual union all
select 4240,'USAF' from dual union all
select 42400,'NOAA' from dual union all
select 42401,'NOAA' from dual union all
select 42402,'NOAA' from dual union all
select 42403,'NOAA' from dual union all
select 42404,'NOAA' from dual union all
select 42405,'NOAA' from dual union all
select 42406,'NOAA' from dual union all
select 42407,'NOAA' from dual union all
select 42408,'NOAA' from dual union all
select 42409,'NOAA' from dual union all
select 4241,'USAF' from dual union all
select 42410,'NOAA' from dual union all
select 42411,'NOAA' from dual union all
select 42412,'NOAA' from dual union all
select 42413,'NOAA' from dual union all
select 42414,'NOAA' from dual union all
select 42415,'NOAA' from dual union all
select 42416,'NOAA' from dual union all
select 42417,'NOAA' from dual union all
select 42418,'NOAA' from dual union all
select 42419,'NOAA' from dual union all
select 4242,'DFVLR' from dual union all
select 42420,'NOAA' from dual;
insert into SATELLITE_ORG
select 42421,'NOAA' from dual union all
select 42422,'NOAA' from dual union all
select 42423,'NOAA' from dual union all
select 42424,'NOAA' from dual union all
select 42425,'NOAA' from dual union all
select 42426,'NOAA' from dual union all
select 42427,'NOAA' from dual union all
select 42428,'NOAA' from dual union all
select 42429,'NOAA' from dual union all
select 4243,'DFVLR' from dual union all
select 42430,'NOAA' from dual union all
select 42431,'NOAA' from dual union all
select 42432,'SES' from dual union all
select 42433,'SPX' from dual union all
select 42434,'NASA' from dual union all
select 42435,'NASA' from dual union all
select 42436,'NASA' from dual union all
select 42437,'NASA' from dual union all
select 42438,'NASA' from dual union all
select 42439,'NASA' from dual union all
select 4244,'USAF' from dual union all
select 42440,'NASA' from dual union all
select 42441,'NASA' from dual union all
select 42442,'NASA' from dual union all
select 42443,'JPL' from dual union all
select 42443,'NASA' from dual union all
select 42444,'JPL' from dual union all
select 42444,'NASA' from dual union all
select 42445,'MO' from dual union all
select 42446,'MO' from dual union all
select 42447,'MO' from dual union all
select 42448,'MO' from dual union all
select 42449,'MO' from dual union all
select 4245,'USAF' from dual union all
select 42450,'MO' from dual union all
select 42451,'MO' from dual union all
select 42452,'KVR' from dual union all
select 42453,'KVR' from dual union all
select 42454,'KVR' from dual union all
select 42455,'KVR' from dual union all
select 42456,'KVR' from dual union all
select 42457,'KVR' from dual union all
select 42458,'CASC' from dual union all
select 42459,'CASC' from dual union all
select 4246,'NASA' from dual union all
select 42460,'CASC' from dual union all
select 42461,'CASC' from dual union all
select 42462,'CASC' from dual union all
select 42463,'CASC' from dual union all
select 42464,'CASC' from dual union all
select 42465,'CASC' from dual union all
select 42466,'CASC' from dual union all
select 42467,'CASC' from dual union all
select 42468,'CASC' from dual union all
select 42469,'CASC' from dual union all
select 4247,'NRO' from dual union all
select 4247,'USN' from dual union all
select 42470,'CASC' from dual union all
select 42471,'CASC' from dual union all
select 42472,'CASC' from dual union all
select 42473,'CASC' from dual union all
select 42474,'CASC' from dual union all
select 42475,'CASC' from dual union all
select 42476,'CASC' from dual union all
select 42477,'CASC' from dual union all
select 42478,'CASC' from dual union all
select 42479,'CASC' from dual union all
select 4248,'USAF' from dual union all
select 42480,'CASC' from dual union all
select 42481,'CASC' from dual union all
select 42482,'AE' from dual union all
select 42483,'AE' from dual union all
select 42484,'AE' from dual union all
select 42485,'AE' from dual union all
select 42486,'AE' from dual union all
select 42487,'AE' from dual union all
select 42488,'AE' from dual union all
select 42489,'AE' from dual union all
select 4249,'USAF' from dual union all
select 42490,'NASA' from dual union all
select 42491,'NASA' from dual union all
select 42492,'NASA' from dual union all
select 42493,'NASA' from dual union all
select 42494,'NASA' from dual union all
select 42495,'NASA' from dual union all
select 42496,'NASA' from dual union all
select 42497,'NASA' from dual union all
select 42498,'NASA' from dual union all
select 42499,'NASA' from dual union all
select 425,'OKB1' from dual union all
select 4250,'MOD' from dual union all
select 42500,'NASA' from dual union all
select 42501,'NASA' from dual union all
select 42502,'NASA' from dual union all
select 42503,'NASA' from dual union all
select 42504,'NASA' from dual union all
select 42505,'NASA' from dual union all
select 42506,'NASA' from dual union all
select 42507,'NASA' from dual union all
select 42508,'NASA' from dual;
insert into SATELLITE_ORG
select 42509,'NASA' from dual union all
select 4251,'NASA' from dual union all
select 42510,'NASA' from dual union all
select 42511,'NASA' from dual union all
select 42512,'NASA' from dual union all
select 42513,'NASA' from dual union all
select 42514,'NASA' from dual union all
select 42515,'NASA' from dual union all
select 42516,'NASA' from dual union all
select 42517,'NASA' from dual union all
select 42518,'NASA' from dual union all
select 42519,'NASA' from dual union all
select 4252,'PVO' from dual union all
select 42520,'NASA' from dual union all
select 42521,'NASA' from dual union all
select 42522,'NASA' from dual union all
select 42523,'NASA' from dual union all
select 42524,'NASA' from dual union all
select 42525,'NASA' from dual union all
select 42526,'NASA' from dual union all
select 42527,'NASA' from dual union all
select 42528,'NASA' from dual union all
select 42529,'NASA' from dual union all
select 4253,'RVSN' from dual union all
select 42530,'NASA' from dual union all
select 42531,'NASA' from dual union all
select 42532,'NASA' from dual union all
select 42533,'NASA' from dual union all
select 42534,'NASA' from dual union all
select 42535,'NASA' from dual union all
select 42536,'NASA' from dual union all
select 42537,'NASA' from dual union all
select 42538,'NASA' from dual union all
select 42539,'NASA' from dual union all
select 4254,'OKB10' from dual union all
select 42540,'NASA' from dual union all
select 42541,'NASA' from dual union all
select 42542,'NASA' from dual union all
select 42543,'NASA' from dual union all
select 42544,'NASA' from dual union all
select 42545,'NASA' from dual union all
select 42546,'NASA' from dual union all
select 42547,'NASA' from dual union all
select 42548,'NASA' from dual union all
select 42549,'NASA' from dual union all
select 4255,'RVSN' from dual union all
select 42550,'NASA' from dual union all
select 42551,'NASA' from dual union all
select 42552,'NASA' from dual union all
select 42553,'NASA' from dual union all
select 42554,'NASA' from dual union all
select 42555,'NASA' from dual union all
select 42556,'NASA' from dual union all
select 42557,'NASA' from dual union all
select 42558,'NASA' from dual union all
select 42559,'NASA' from dual union all
select 4256,'USN' from dual union all
select 42560,'NASA' from dual union all
select 42561,'NASA' from dual union all
select 42562,'NASA' from dual union all
select 42563,'NASA' from dual union all
select 42564,'NASA' from dual union all
select 42565,'NASA' from dual union all
select 42566,'NASA' from dual union all
select 42567,'NASA' from dual union all
select 42568,'NASA' from dual union all
select 42569,'NASA' from dual union all
select 4257,'USN' from dual union all
select 42570,'NASA' from dual union all
select 42571,'NASA' from dual union all
select 42572,'NASA' from dual union all
select 42573,'NASA' from dual union all
select 42574,'NASA' from dual union all
select 42575,'NASA' from dual union all
select 42576,'NASA' from dual union all
select 42577,'NASA' from dual union all
select 42578,'NASA' from dual union all
select 42579,'NASA' from dual union all
select 4258,'USAF' from dual union all
select 42580,'NASA' from dual union all
select 42581,'NASA' from dual union all
select 42582,'NASA' from dual union all
select 42583,'NASA' from dual union all
select 42584,'NASA' from dual union all
select 42585,'NASA' from dual union all
select 42586,'NASA' from dual union all
select 42587,'NASA' from dual union all
select 42588,'NASA' from dual union all
select 42589,'NASA' from dual union all
select 4259,'NRO' from dual union all
select 4259,'USN' from dual union all
select 42590,'NASA' from dual union all
select 42591,'NASA' from dual union all
select 42592,'NASA' from dual union all
select 42593,'NASA' from dual union all
select 42594,'NASA' from dual union all
select 42595,'NASA' from dual union all
select 42596,'NASA' from dual union all
select 42597,'NASA' from dual union all
select 42598,'NASA' from dual;
insert into SATELLITE_ORG
select 42599,'NASA' from dual union all
select 426,'NASA' from dual union all
select 4260,'USAF' from dual union all
select 42600,'NASA' from dual union all
select 42601,'NASA' from dual union all
select 42602,'NASA' from dual union all
select 42603,'NASA' from dual union all
select 42604,'NASA' from dual union all
select 42605,'NASA' from dual union all
select 42606,'NASA' from dual union all
select 42607,'NASA' from dual union all
select 42608,'NASA' from dual union all
select 42609,'NASA' from dual union all
select 4261,'DFVLR' from dual union all
select 42610,'NASA' from dual union all
select 42611,'NASA' from dual union all
select 42612,'NASA' from dual union all
select 42613,'NASA' from dual union all
select 42614,'NASA' from dual union all
select 42615,'NASA' from dual union all
select 42616,'NASA' from dual union all
select 42617,'NASA' from dual union all
select 42618,'NASA' from dual union all
select 42619,'NASA' from dual union all
select 4262,'TSKBEM' from dual union all
select 42620,'NASA' from dual union all
select 42621,'NASA' from dual union all
select 42622,'NASA' from dual union all
select 42623,'NASA' from dual union all
select 42624,'NASA' from dual union all
select 42625,'NASA' from dual union all
select 42626,'NASA' from dual union all
select 42627,'NASA' from dual union all
select 42628,'NASA' from dual union all
select 42629,'NASA' from dual union all
select 4263,'RVSN' from dual union all
select 42630,'NASA' from dual union all
select 42631,'NASA' from dual union all
select 42632,'NASA' from dual union all
select 42633,'NASA' from dual union all
select 42634,'NASA' from dual union all
select 42635,'NASA' from dual union all
select 42636,'NASA' from dual union all
select 42637,'NASA' from dual union all
select 42638,'NASA' from dual union all
select 42639,'NASA' from dual union all
select 4264,'CIA' from dual union all
select 4264,'NRO' from dual union all
select 42640,'NASA' from dual union all
select 42641,'NASA' from dual union all
select 42642,'NASA' from dual union all
select 42643,'NASA' from dual union all
select 42644,'NASA' from dual union all
select 42645,'NASA' from dual union all
select 42646,'NASA' from dual union all
select 42647,'NASA' from dual union all
select 42648,'NASA' from dual union all
select 42649,'NASA' from dual union all
select 4265,'DFVLR' from dual union all
select 42650,'NASA' from dual union all
select 42651,'NASA' from dual union all
select 42652,'NASA' from dual union all
select 42653,'NASA' from dual union all
select 42654,'NASA' from dual union all
select 42655,'NASA' from dual union all
select 42656,'NASA' from dual union all
select 42657,'NASA' from dual union all
select 42658,'NASA' from dual union all
select 42659,'NASA' from dual union all
select 4266,'PVO' from dual union all
select 42660,'NASA' from dual union all
select 42661,'NASA' from dual union all
select 42662,'CASC' from dual union all
select 42663,'CASC' from dual union all
select 42664,'USAF' from dual union all
select 42665,'USAF' from dual union all
select 42666,'USN' from dual union all
select 42667,'USN' from dual union all
select 42668,'USN' from dual union all
select 42669,'ESA' from dual union all
select 4267,'RVSN' from dual union all
select 42670,'CNSA' from dual union all
select 42671,'CNSA' from dual union all
select 42672,'CNSA' from dual union all
select 42673,'CNSA' from dual union all
select 42674,'ISAS' from dual union all
select 42674,'JAXA' from dual union all
select 42675,'ISAS' from dual union all
select 42675,'JAXA' from dual union all
select 42676,'CNSA' from dual union all
select 42677,'CNSA' from dual union all
select 42678,'CNSA' from dual union all
select 42679,'CNSA' from dual union all
select 4268,'USAF' from dual union all
select 42680,'CNSA' from dual union all
select 42681,'OSC' from dual union all
select 42682,'FKA' from dual union all
select 42683,'VVKO' from dual union all
select 42684,'PLA' from dual union all
select 42685,'MAI' from dual;
insert into SATELLITE_ORG
select 42686,'MAI' from dual union all
select 42687,'MAI' from dual union all
select 42688,'MAI' from dual union all
select 42689,'NROC' from dual union all
select 4269,'USAF' from dual union all
select 42690,'SPX' from dual union all
select 42691,'KTELT' from dual union all
select 42692,'TELB' from dual union all
select 42693,'AE' from dual union all
select 42694,'AE' from dual union all
select 42695,'ISRO' from dual union all
select 42696,'ISRO' from dual union all
select 42697,'NASA' from dual union all
select 42698,'INMRL' from dual union all
select 42699,'SPX' from dual union all
select 427,'CIA' from dual union all
select 427,'NRO' from dual union all
select 4270,'USAF' from dual union all
select 42700,'TUD' from dual union all
select 42701,'HAVEL' from dual union all
select 42701,'ITUTR' from dual union all
select 42702,'QBUS' from dual union all
select 42702,'TURAB' from dual union all
select 42703,'KYSP' from dual union all
select 42704,'MSUKY' from dual union all
select 42705,'NASA' from dual union all
select 42706,'NCKU' from dual union all
select 42707,'POLY' from dual union all
select 42708,'LTU' from dual union all
select 42708,'OCOS' from dual union all
select 42709,'SES' from dual union all
select 4271,'USAF' from dual union all
select 42710,'NPOL' from dual union all
select 42711,'MILLEN' from dual union all
select 42712,'USAF' from dual union all
select 42713,'CUBESP' from dual union all
select 42713,'STBO' from dual union all
select 42714,'KAIST' from dual union all
select 42715,'CSUN' from dual union all
select 42715,'JPL' from dual union all
select 42716,'PATRAS' from dual union all
select 42717,'MINES' from dual union all
select 42718,'HSCIL' from dual union all
select 42719,'KVR' from dual union all
select 4272,'USAF' from dual union all
select 42720,'NPOL' from dual union all
select 42721,'QBUS' from dual union all
select 42721,'UCO' from dual union all
select 42722,'NJUST' from dual union all
select 42723,'UNSW' from dual union all
select 42724,'DUTH' from dual union all
select 42725,'HARB' from dual union all
select 42726,'SCSS' from dual union all
select 42727,'SNU' from dual union all
select 42728,'UPM' from dual union all
select 42729,'AALTO' from dual union all
select 4273,'TSN108' from dual union all
select 42730,'ADEL' from dual union all
select 42731,'USYD' from dual union all
select 42732,'KPI' from dual union all
select 42733,'SNU' from dual union all
select 42734,'ALB' from dual union all
select 42735,'NWPU' from dual union all
select 42736,'ITUTR' from dual union all
select 42736,'TUAF' from dual union all
select 42737,'QBUS' from dual union all
select 42737,'UMI' from dual union all
select 42738,'JAXA' from dual union all
select 42738,'SPAC' from dual union all
select 42739,'JAXA' from dual union all
select 4274,'RVSN' from dual union all
select 42740,'VIA' from dual union all
select 42741,'EUTSA' from dual union all
select 42742,'AE' from dual union all
select 42743,'AE' from dual union all
select 42744,'SPX' from dual union all
select 42745,'SPX' from dual union all
select 42746,'SPX' from dual union all
select 42747,'ISRO' from dual union all
select 42748,'ISRO' from dual union all
select 42749,'ECHO' from dual union all
select 4275,'YUZH' from dual union all
select 42750,'KHRR' from dual union all
select 42751,'KHRR' from dual union all
select 42752,'SPIRE' from dual union all
select 42753,'SPIRE' from dual union all
select 42754,'SPIRE' from dual union all
select 42755,'SPIRE' from dual union all
select 42756,'FKA' from dual union all
select 42757,'RVSNR' from dual union all
select 42758,'CNSA' from dual union all
select 42759,'ZHUORB' from dual union all
select 4276,'USAF' from dual union all
select 42760,'SATELOG' from dual union all
select 42761,'ZHUORB' from dual union all
select 42762,'CNSA' from dual union all
select 42763,'PLA' from dual union all
select 42764,'CASC' from dual union all
select 42765,'UCL' from dual union all
select 42766,'NIUTN' from dual;
insert into SATELLITE_ORG
select 42767,'ISRO' from dual union all
select 42768,'VILN' from dual union all
select 42769,'CANON' from dual union all
select 4277,'USAF' from dual union all
select 42770,'SURREY' from dual union all
select 42771,'SPIRE' from dual union all
select 42772,'SPIRE' from dual union all
select 42773,'SPIRE' from dual union all
select 42774,'SPIRE' from dual union all
select 42775,'AALTO' from dual union all
select 42776,'ROSAP' from dual union all
select 42777,'AACH' from dual union all
select 42778,'GOBBZ' from dual union all
select 42778,'GOBME' from dual union all
select 42779,'SPIRE' from dual union all
select 4278,'USAF' from dual union all
select 42780,'SPIRE' from dual union all
select 42781,'SPIRE' from dual union all
select 42782,'SPIRE' from dual union all
select 42783,'SASGA' from dual union all
select 42784,'FHWN' from dual union all
select 42785,'SASGA' from dual union all
select 42786,'SASGA' from dual union all
select 42787,'NUDTC' from dual union all
select 42788,'UCH' from dual union all
select 42789,'SOSA' from dual union all
select 4279,'YUZH' from dual union all
select 42790,'VZLU' from dual union all
select 42791,'VATP' from dual union all
select 42792,'MONTP2' from dual union all
select 42793,'GEOOPT' from dual union all
select 42794,'DORBIT' from dual union all
select 42795,'PACSCI' from dual union all
select 42796,'ISRO' from dual union all
select 42797,'ISRO' from dual union all
select 42798,'TSVS' from dual union all
select 42799,'RVSNR' from dual union all
select 428,'OKB1' from dual union all
select 4280,'TSKBEM' from dual union all
select 42800,'RVSNR' from dual union all
select 42801,'BULSAT' from dual union all
select 42802,'SPX' from dual union all
select 42803,'IRID' from dual union all
select 42804,'IRID' from dual union all
select 42805,'IRID' from dual union all
select 42806,'IRID' from dual union all
select 42807,'IRID' from dual union all
select 42808,'IRID' from dual union all
select 42809,'IRID' from dual union all
select 4281,'RVSN' from dual union all
select 42810,'IRID' from dual union all
select 42811,'IRID' from dual union all
select 42812,'IRID' from dual union all
select 42813,'NASA' from dual union all
select 42814,'HELSA' from dual union all
select 42815,'ISRO' from dual union all
select 42816,'AE' from dual union all
select 42817,'AE' from dual union all
select 42818,'INTELS' from dual union all
select 42819,'SPX' from dual union all
select 4282,'PVO' from dual union all
select 42820,'KYUT' from dual union all
select 42821,'ANUC' from dual union all
select 42822,'NUM' from dual union all
select 42823,'BRAC' from dual union all
select 42824,'FUTA' from dual union all
select 42825,'METP' from dual union all
select 42826,'NSC' from dual union all
select 42827,'MAIO' from dual union all
select 42828,'NSC' from dual union all
select 42829,'TUB1' from dual union all
select 4283,'PVO' from dual union all
select 42830,'MPU' from dual union all
select 42831,'IRSS' from dual union all
select 42832,'UTE' from dual union all
select 42832,'YUZGU' from dual union all
select 42833,'NPOL' from dual union all
select 42834,'RAKA' from dual union all
select 42835,'WNI' from dual union all
select 42836,'RAKA' from dual union all
select 42837,'SPIRE' from dual union all
select 42838,'SPIRE' from dual union all
select 42839,'SPIRE' from dual union all
select 4284,'RVSN' from dual union all
select 42840,'SPIRE' from dual union all
select 42841,'SPIRE' from dual union all
select 42842,'SPIRE' from dual union all
select 42843,'GEOOPT' from dual union all
select 42844,'TYVAK' from dual union all
select 42845,'SPIRE' from dual union all
select 42846,'ADIG' from dual union all
select 42847,'ADIG' from dual union all
select 42848,'GEOOPT' from dual union all
select 42849,'GEOOPT' from dual union all
select 4285,'IK' from dual union all
select 42850,'PLABS' from dual union all
select 42851,'PLABS' from dual union all
select 42852,'PLABS' from dual union all
select 42853,'PLABS' from dual union all
select 42854,'PLABS' from dual;
insert into SATELLITE_ORG
select 42855,'PLABS' from dual union all
select 42856,'PLABS' from dual union all
select 42857,'PLABS' from dual union all
select 42858,'PLABS' from dual union all
select 42859,'PLABS' from dual union all
select 4286,'RVSN' from dual union all
select 42860,'PLABS' from dual union all
select 42861,'PLABS' from dual union all
select 42862,'PLABS' from dual union all
select 42863,'PLABS' from dual union all
select 42864,'PLABS' from dual union all
select 42865,'PLABS' from dual union all
select 42866,'PLABS' from dual union all
select 42867,'PLABS' from dual union all
select 42868,'PLABS' from dual union all
select 42869,'PLABS' from dual union all
select 4287,'YUZH' from dual union all
select 42870,'PLABS' from dual union all
select 42871,'PLABS' from dual union all
select 42872,'PLABS' from dual union all
select 42873,'PLABS' from dual union all
select 42874,'PLABS' from dual union all
select 42875,'PLABS' from dual union all
select 42876,'PLABS' from dual union all
select 42877,'PLABS' from dual union all
select 42878,'PLABS' from dual union all
select 42879,'PLABS' from dual union all
select 4288,'USAF' from dual union all
select 42880,'PLABS' from dual union all
select 42881,'SPIRE' from dual union all
select 42882,'PLABS' from dual union all
select 42883,'PLABS' from dual union all
select 42884,'PLABS' from dual union all
select 42885,'PLABS' from dual union all
select 42886,'PLABS' from dual union all
select 42887,'PLABS' from dual union all
select 42888,'PLABS' from dual union all
select 42889,'PLABS' from dual union all
select 4289,'RVSN' from dual union all
select 42890,'PLABS' from dual union all
select 42891,'PLABS' from dual union all
select 42892,'PLABS' from dual union all
select 42893,'PLABS' from dual union all
select 42894,'PLABS' from dual union all
select 42895,'PLABS' from dual union all
select 42896,'PLABS' from dual union all
select 42897,'PLABS' from dual union all
select 42898,'FKA' from dual union all
select 42899,'VVKO' from dual union all
select 429,'OKB1' from dual union all
select 4290,'MO' from dual union all
select 42900,'MDDI' from dual union all
select 42901,'CNES' from dual union all
select 42901,'ISA' from dual union all
select 42902,'ESA' from dual union all
select 42903,'XCYS' from dual union all
select 42904,'SPX' from dual union all
select 42905,'SPX' from dual union all
select 42906,'SPX' from dual union all
select 42907,'KVR' from dual union all
select 42907,'VOENT' from dual union all
select 42908,'KHRR' from dual union all
select 42909,'KHRR' from dual union all
select 4291,'MO' from dual union all
select 42910,'TPU' from dual union all
select 42911,'YUZGU' from dual union all
select 42912,'YUZGU' from dual union all
select 42913,'FKA' from dual union all
select 42914,'FKA' from dual union all
select 42915,'NASA' from dual union all
select 42916,'LM' from dual union all
select 42917,'JAXA' from dual union all
select 42917,'SPAC' from dual union all
select 42918,'JAXA' from dual union all
select 42919,'KVR' from dual union all
select 4292,'MO' from dual union all
select 42920,'NSPO' from dual union all
select 42921,'USAF' from dual union all
select 42922,'DOD' from dual union all
select 42923,'DOD' from dual union all
select 42924,'DOD' from dual union all
select 42925,'OSC' from dual union all
select 42926,'OSC' from dual union all
select 42927,'OSC' from dual union all
select 42928,'ISRO' from dual union all
select 42929,'ISRO' from dual union all
select 4293,'RVSN' from dual union all
select 42930,'ISRO' from dual union all
select 42931,'MDDI' from dual union all
select 42932,'KVR' from dual union all
select 42933,'KVR' from dual union all
select 42934,'KVR' from dual union all
select 42935,'KVR' from dual union all
select 42936,'KVR' from dual union all
select 42937,'KVR' from dual union all
select 42938,'KVR' from dual union all
select 42939,'KVR' from dual union all
select 4294,'YUZH' from dual union all
select 42940,'KVR' from dual union all
select 42941,'SMDC' from dual;
insert into SATELLITE_ORG
select 42942,'NWURKS' from dual union all
select 42943,'KVR' from dual union all
select 42944,'RVSN' from dual union all
select 42945,'USAF' from dual union all
select 42946,'JPL' from dual union all
select 42946,'MIT' from dual union all
select 42947,'NASA' from dual union all
select 42948,'PSU' from dual union all
select 42949,'MO' from dual union all
select 4295,'NRO' from dual union all
select 4295,'USN' from dual union all
select 42950,'MO' from dual union all
select 42951,'ISRO' from dual union all
select 42952,'MO' from dual union all
select 42953,'RVSN' from dual union all
select 42954,'FKA' from dual union all
select 42955,'FKA' from dual union all
select 42956,'CASC' from dual union all
select 42957,'CASC' from dual union all
select 42958,'USAF' from dual union all
select 42959,'USAF' from dual union all
select 4296,'NRO' from dual union all
select 42960,'USAF' from dual union all
select 42961,'USAF' from dual union all
select 42962,'USAF' from dual union all
select 42963,'USN' from dual union all
select 42964,'USN' from dual union all
select 42965,'KVR' from dual union all
select 42966,'KVR' from dual union all
select 42967,'KVR' from dual union all
select 42968,'VMF' from dual union all
select 42969,'NASA' from dual union all
select 4297,'INTEL' from dual union all
select 42970,'NASA' from dual union all
select 42971,'KVR' from dual union all
select 42972,'KVR' from dual union all
select 42973,'KVR' from dual union all
select 42974,'KVR' from dual union all
select 42975,'KVR' from dual union all
select 42976,'KVR' from dual union all
select 42977,'MO' from dual union all
select 42978,'MO' from dual union all
select 42979,'MO' from dual union all
select 4298,'NASA' from dual union all
select 42980,'MO' from dual union all
select 42981,'MO' from dual union all
select 42982,'MO' from dual union all
select 42983,'KVR' from dual union all
select 42984,'KVR' from dual union all
select 42985,'KVR' from dual union all
select 42986,'RVSNR' from dual union all
select 42987,'MOM' from dual union all
select 42988,'RVSN' from dual union all
select 42989,'RVSN' from dual union all
select 4299,'PVO' from dual union all
select 42990,'RVSN' from dual union all
select 42991,'RVSN' from dual union all
select 42992,'RVSN' from dual union all
select 42993,'NASA' from dual union all
select 42994,'VTS' from dual union all
select 42995,'VTS' from dual union all
select 42996,'RVSN' from dual union all
select 42997,'VS' from dual union all
select 42998,'VMF' from dual union all
select 42999,'VMF' from dual union all
select 43,'USAF' from dual union all
select 430,'OKB1' from dual union all
select 4300,'RVSN' from dual union all
select 43000,'VMF' from dual union all
select 43001,'VMF' from dual union all
select 43002,'VMF' from dual union all
select 43003,'USAF' from dual union all
select 43004,'USN' from dual union all
select 43005,'USN' from dual union all
select 43006,'USN' from dual union all
select 43007,'ESA' from dual union all
select 43008,'RVSN' from dual union all
select 43009,'VMF' from dual union all
select 4301,'YUZH' from dual union all
select 43010,'VMF' from dual union all
select 43011,'NASA' from dual union all
select 43012,'VS' from dual union all
select 43013,'USAF' from dual union all
select 43014,'USAF' from dual union all
select 43015,'USAF' from dual union all
select 43016,'KVR' from dual union all
select 43017,'KVR' from dual union all
select 43018,'KVR' from dual union all
select 43019,'KVR' from dual union all
select 4302,'RVSN' from dual union all
select 43020,'ESA' from dual union all
select 43021,'RVSNR' from dual union all
select 43022,'RVSNR' from dual union all
select 43023,'RVSNR' from dual union all
select 43024,'CMA' from dual union all
select 43025,'BOE' from dual union all
select 43026,'BOE' from dual union all
select 43027,'CASC' from dual union all
select 43028,'CASC' from dual union all
select 43029,'NASA' from dual;
insert into SATELLITE_ORG
select 4303,'MO' from dual union all
select 43030,'NASA' from dual union all
select 43031,'NASA' from dual union all
select 43032,'NASA' from dual union all
select 43033,'NASA' from dual union all
select 43034,'NASA' from dual union all
select 43035,'NASA' from dual union all
select 43036,'NASA' from dual union all
select 43037,'NOAA' from dual union all
select 43038,'KVR' from dual union all
select 43039,'KVR' from dual union all
select 4304,'MO' from dual union all
select 43040,'NASDA' from dual union all
select 43041,'KVR' from dual union all
select 43042,'CASC' from dual union all
select 43043,'MORF' from dual union all
select 43044,'MORF' from dual union all
select 43045,'RVSNR' from dual union all
select 43046,'RVSNR' from dual union all
select 43047,'RVSNR' from dual union all
select 43048,'RVSNR' from dual union all
select 43049,'RVSNR' from dual union all
select 4305,'MO' from dual union all
select 43050,'KVR' from dual union all
select 43051,'ESA' from dual union all
select 43052,'ESA' from dual union all
select 43053,'ESA' from dual union all
select 43054,'SIS' from dual union all
select 43055,'DGLO' from dual union all
select 43056,'DGLO' from dual union all
select 43057,'DGLO' from dual union all
select 43058,'DGLO' from dual union all
select 43059,'DGLO' from dual union all
select 4306,'MO' from dual union all
select 43060,'DGLO' from dual union all
select 43061,'DGLO' from dual union all
select 43062,'USEF' from dual union all
select 43063,'USEF' from dual union all
select 43064,'NPOL' from dual union all
select 43065,'CMA' from dual union all
select 43066,'JAXA' from dual union all
select 43067,'JAXA' from dual union all
select 43068,'JAXA' from dual union all
select 43069,'KHRR' from dual union all
select 4307,'MO' from dual union all
select 43070,'KHRR' from dual union all
select 43071,'CNSA' from dual union all
select 43072,'CNSA' from dual union all
select 43073,'CNSA' from dual union all
select 43074,'MSBTR' from dual union all
select 43075,'MSBTR' from dual union all
select 43076,'KHRR' from dual union all
select 43077,'CNSA' from dual union all
select 43078,'CNSA' from dual union all
select 43079,'CNSA' from dual union all
select 4308,'YUZH' from dual union all
select 43080,'CNSA' from dual union all
select 43081,'CNSA' from dual union all
select 43082,'CNSA' from dual union all
select 43083,'KHRR' from dual union all
select 43084,'USAF' from dual union all
select 43085,'USAF' from dual union all
select 43086,'ESA' from dual union all
select 43087,'CNSA' from dual union all
select 43088,'CNSA' from dual union all
select 43089,'CNSA' from dual union all
select 4309,'RVSN' from dual union all
select 43090,'CNSA' from dual union all
select 43091,'CNSA' from dual union all
select 43092,'CNSA' from dual union all
select 43093,'CNSA' from dual union all
select 43094,'CNSA' from dual union all
select 43095,'CNSA' from dual union all
select 43096,'CNSA' from dual union all
select 43097,'CNSA' from dual union all
select 43098,'CNSA' from dual union all
select 43099,'CNSA' from dual union all
select 431,'OKB1' from dual union all
select 4310,'USAF' from dual union all
select 43100,'CNSA' from dual union all
select 43101,'USAF' from dual union all
select 43102,'USAF' from dual union all
select 43103,'USAF' from dual union all
select 43104,'USAF' from dual union all
select 43105,'USAF' from dual union all
select 43106,'USAF' from dual union all
select 43107,'USAF' from dual union all
select 43108,'USAF' from dual union all
select 43109,'VMF' from dual union all
select 4311,'USAF' from dual union all
select 43110,'VMF' from dual union all
select 43111,'VMF' from dual union all
select 43112,'PVO' from dual union all
select 43113,'CASC' from dual union all
select 4312,'USAF' from dual union all
select 4313,'MO' from dual union all
select 4314,'MO' from dual union all
select 4315,'MO' from dual union all
select 4316,'RVSN' from dual union all
select 4317,'USAF' from dual;
insert into SATELLITE_ORG
select 4318,'USAF' from dual union all
select 4319,'USAF' from dual union all
select 432,'NASA' from dual union all
select 4320,'NOAA' from dual union all
select 4321,'MELB' from dual union all
select 4322,'NASA' from dual union all
select 4323,'USAF' from dual union all
select 4324,'USAF' from dual union all
select 4325,'USAF' from dual union all
select 4326,'USAF' from dual union all
select 4327,'NASA' from dual union all
select 4328,'MO' from dual union all
select 4329,'RVSN' from dual union all
select 433,'NASA' from dual union all
select 4330,'ISAS' from dual union all
select 4331,'USAF' from dual union all
select 4332,'USAF' from dual union all
select 4333,'RVSN' from dual union all
select 4334,'RVSN' from dual union all
select 4335,'USAF' from dual union all
select 4336,'MO' from dual union all
select 4337,'RVSN' from dual union all
select 4338,'PVO' from dual union all
select 4339,'RVSN' from dual union all
select 434,'NASA' from dual union all
select 4340,'RVSN' from dual union all
select 4341,'RVSN' from dual union all
select 4342,'CIA' from dual union all
select 4342,'NRO' from dual union all
select 4343,'NRO' from dual union all
select 4344,'BMBW' from dual union all
select 4344,'CNES' from dual union all
select 4345,'CNES' from dual union all
select 4346,'RVSN' from dual union all
select 4347,'RVSN' from dual union all
select 4348,'USAF' from dual union all
select 4349,'GMS' from dual union all
select 4349,'RVSN' from dual union all
select 435,'OKB1' from dual union all
select 4350,'RVSN' from dual union all
select 4351,'PVO' from dual union all
select 4352,'RVSN' from dual union all
select 4353,'NATO' from dual union all
select 4354,'NASA' from dual union all
select 4355,'TSKBEM' from dual union all
select 4356,'RVSN' from dual union all
select 4357,'RVSN' from dual union all
select 4358,'RVSN' from dual union all
select 4359,'USAF' from dual union all
select 436,'NRO' from dual union all
select 436,'USA' from dual union all
select 4360,'TSN108' from dual union all
select 4361,'RVSN' from dual union all
select 4362,'NASA' from dual union all
select 4363,'USA' from dual union all
select 4364,'RVSN' from dual union all
select 4365,'RVSN' from dual union all
select 4366,'USAF' from dual union all
select 4367,'NASA' from dual union all
select 4368,'USAF' from dual union all
select 4369,'OKB10' from dual union all
select 437,'OKB1' from dual union all
select 4370,'RVSN' from dual union all
select 4371,'NASA' from dual union all
select 4372,'NASA' from dual union all
select 4373,'RVSN' from dual union all
select 4374,'RVSN' from dual union all
select 4375,'NRO' from dual union all
select 4376,'INTEL' from dual union all
select 4377,'NASA' from dual union all
select 4378,'PVO' from dual union all
select 4379,'RVSN' from dual union all
select 438,'RVSN' from dual union all
select 4380,'YUZH' from dual union all
select 4381,'RVSN' from dual union all
select 4382,'MAI' from dual union all
select 4383,'OKB10' from dual union all
select 4384,'OKB10' from dual union all
select 4385,'OKB10' from dual union all
select 4386,'OKB10' from dual union all
select 4387,'OKB10' from dual union all
select 4388,'OKB10' from dual union all
select 4389,'OKB10' from dual union all
select 439,'JPL' from dual union all
select 439,'NASA' from dual union all
select 4390,'OKB10' from dual union all
select 4391,'RVSN' from dual union all
select 4392,'MAI' from dual union all
select 4393,'GMS' from dual union all
select 4393,'RVSN' from dual union all
select 4394,'RVSN' from dual union all
select 4395,'RVSN' from dual union all
select 4396,'RVSN' from dual union all
select 4397,'RVSN' from dual union all
select 4398,'RVSN' from dual union all
select 4399,'RVSN' from dual union all
select 44,'USAF' from dual union all
select 440,'NASA' from dual union all
select 4400,'MAI' from dual union all
select 4401,'RVSN' from dual;
insert into SATELLITE_ORG
select 4402,'RVSN' from dual union all
select 4403,'RVSN' from dual union all
select 4404,'RVSN' from dual union all
select 4405,'CIA' from dual union all
select 4405,'NRO' from dual union all
select 4406,'NRO' from dual union all
select 4407,'TSKBEM' from dual union all
select 4408,'RVSN' from dual union all
select 4409,'RVSN' from dual union all
select 441,'OKB586' from dual union all
select 4410,'RVSN' from dual union all
select 4411,'PVO' from dual union all
select 4412,'RVSN' from dual union all
select 4413,'YUZH' from dual union all
select 4414,'RVSN' from dual union all
select 4415,'RVSN' from dual union all
select 4416,'RVSN' from dual union all
select 4417,'RVSN' from dual union all
select 4418,'CIA' from dual union all
select 4418,'NRO' from dual union all
select 4419,'GMS' from dual union all
select 4419,'RVSN' from dual union all
select 442,'RVSN' from dual union all
select 4420,'RVSN' from dual union all
select 4421,'PVO' from dual union all
select 4422,'NRO' from dual union all
select 4423,'RVSN' from dual union all
select 4424,'RVSN' from dual union all
select 4425,'RVSN' from dual union all
select 4426,'RVSN' from dual union all
select 4427,'PVO' from dual union all
select 4428,'RVSN' from dual union all
select 4429,'USAF' from dual union all
select 443,'OKB1' from dual union all
select 4430,'RVSN' from dual union all
select 4431,'RVSN' from dual union all
select 4432,'USAF' from dual union all
select 4433,'USAF' from dual union all
select 4434,'USAF' from dual union all
select 4435,'USAF' from dual union all
select 4436,'USAF' from dual union all
select 4437,'USAF' from dual union all
select 4438,'USAF' from dual union all
select 4439,'USAF' from dual union all
select 444,'CIA' from dual union all
select 444,'NRO' from dual union all
select 4440,'USAF' from dual union all
select 4441,'USAF' from dual union all
select 4442,'USAF' from dual union all
select 4443,'USAF' from dual union all
select 4444,'USAF' from dual union all
select 4445,'USAF' from dual union all
select 4446,'RVSN' from dual union all
select 4447,'RVSN' from dual union all
select 4448,'USAF' from dual union all
select 4449,'USAF' from dual union all
select 445,'NASA' from dual union all
select 4450,'USAF' from dual union all
select 4451,'USAF' from dual union all
select 4452,'USAF' from dual union all
select 4453,'USAF' from dual union all
select 4454,'USAF' from dual union all
select 4455,'RVSN' from dual union all
select 4456,'RVSN' from dual union all
select 4457,'USAF' from dual union all
select 4458,'USAF' from dual union all
select 4459,'USAF' from dual union all
select 446,'USN' from dual union all
select 4460,'USAF' from dual union all
select 4461,'USAF' from dual union all
select 4462,'USAF' from dual union all
select 4463,'USAF' from dual union all
select 4464,'USAF' from dual union all
select 4465,'USAF' from dual union all
select 4466,'USAF' from dual union all
select 4467,'USAF' from dual union all
select 4468,'USAF' from dual union all
select 4469,'USAF' from dual union all
select 447,'USAF' from dual union all
select 4470,'USAF' from dual union all
select 4471,'USAF' from dual union all
select 4472,'USAF' from dual union all
select 4473,'USAF' from dual union all
select 4474,'USAF' from dual union all
select 4475,'USAF' from dual union all
select 4476,'USAF' from dual union all
select 4477,'CIA' from dual union all
select 4477,'NRO' from dual union all
select 4478,'INTEL' from dual union all
select 4479,'RVSN' from dual union all
select 448,'RVSN' from dual union all
select 4480,'RVSN' from dual union all
select 4481,'RVSN' from dual union all
select 4482,'IK' from dual union all
select 4483,'RVSN' from dual union all
select 4484,'RVSN' from dual union all
select 4485,'RVSN' from dual union all
select 4486,'NASA' from dual union all
select 4487,'YUZH' from dual union all
select 4488,'RVSN' from dual;
insert into SATELLITE_ORG
select 4489,'IKI' from dual union all
select 4489,'NPOL' from dual union all
select 449,'RVSN' from dual union all
select 4490,'RVSN' from dual union all
select 4491,'RVSN' from dual union all
select 4492,'NRO' from dual union all
select 4493,'MOD' from dual union all
select 4494,'USAF' from dual union all
select 4495,'PVO' from dual union all
select 4496,'RVSN' from dual union all
select 4497,'PVO' from dual union all
select 4498,'RVSN' from dual union all
select 4499,'NASA' from dual union all
select 45,'USN' from dual union all
select 450,'OKB1' from dual union all
select 4500,'PVO' from dual union all
select 4501,'IKI' from dual union all
select 4501,'NPOL' from dual union all
select 4502,'RVSN' from dual union all
select 4503,'NRO' from dual union all
select 4504,'RVSN' from dual union all
select 4505,'MOM' from dual union all
select 4506,'RVSN' from dual union all
select 4507,'USN' from dual union all
select 4508,'RVSN' from dual union all
select 4509,'RVSN' from dual union all
select 451,'OKB1' from dual union all
select 4510,'USAF' from dual union all
select 4511,'USAF' from dual union all
select 4512,'USAF' from dual union all
select 4513,'USAF' from dual union all
select 4514,'PVO' from dual union all
select 4515,'NASA' from dual union all
select 4516,'RVSN' from dual union all
select 4517,'RVSN' from dual union all
select 4518,'USAF' from dual union all
select 4519,'USAF' from dual union all
select 452,'RVSN' from dual union all
select 4520,'USAF' from dual union all
select 4521,'USAF' from dual union all
select 4522,'USAF' from dual union all
select 4523,'USAF' from dual union all
select 4524,'RVSN' from dual union all
select 4525,'RVSN' from dual union all
select 4526,'RVSN' from dual union all
select 4527,'IKI' from dual union all
select 4527,'NPOL' from dual union all
select 4528,'RVSN' from dual union all
select 4529,'RVSN' from dual union all
select 453,'CIA' from dual union all
select 453,'NRO' from dual union all
select 4530,'USAF' from dual union all
select 4531,'USAF' from dual union all
select 4532,'USAF' from dual union all
select 4533,'USAF' from dual union all
select 4534,'USAF' from dual union all
select 4535,'USAF' from dual union all
select 4536,'PVO' from dual union all
select 4537,'RVSN' from dual union all
select 4538,'RVSN' from dual union all
select 4539,'RVSN' from dual union all
select 454,'RVSN' from dual union all
select 4540,'USAF' from dual union all
select 4541,'USAF' from dual union all
select 4542,'USAF' from dual union all
select 4543,'USAF' from dual union all
select 4544,'USAF' from dual union all
select 4545,'RVSN' from dual union all
select 4546,'RVSN' from dual union all
select 4547,'USAF' from dual union all
select 4548,'USAF' from dual union all
select 4549,'USAF' from dual union all
select 455,'USAF' from dual union all
select 4550,'USAF' from dual union all
select 4551,'USAF' from dual union all
select 4552,'USAF' from dual union all
select 4553,'TSKBEM' from dual union all
select 4554,'RVSN' from dual union all
select 4555,'USN' from dual union all
select 4556,'RVSN' from dual union all
select 4557,'RVSN' from dual union all
select 4558,'RVSN' from dual union all
select 4559,'RVSN' from dual union all
select 456,'RVSN' from dual union all
select 4560,'RVSN' from dual union all
select 4561,'RVSN' from dual union all
select 4562,'RVSN' from dual union all
select 4563,'RVSN' from dual union all
select 4564,'VMF' from dual union all
select 4565,'VMF' from dual union all
select 4566,'VMF' from dual union all
select 4567,'VMF' from dual union all
select 4568,'VMF' from dual union all
select 4569,'RVSN' from dual union all
select 457,'RVSN' from dual union all
select 4570,'RVSN' from dual union all
select 4571,'IMBP' from dual union all
select 4571,'RVSN' from dual union all
select 4572,'RVSN' from dual union all
select 4573,'PVO' from dual;
insert into SATELLITE_ORG
select 4574,'RVSN' from dual union all
select 4575,'TSKBEM' from dual union all
select 4576,'RVSN' from dual union all
select 4577,'RVSN' from dual union all
select 4578,'OKB10' from dual union all
select 4579,'RVSN' from dual union all
select 458,'RVSN' from dual union all
select 4580,'IK' from dual union all
select 4581,'RVSN' from dual union all
select 4582,'RVSN' from dual union all
select 4583,'GMS' from dual union all
select 4583,'RVSN' from dual union all
select 4584,'RVSN' from dual union all
select 4585,'TSKBEM' from dual union all
select 4586,'USN' from dual union all
select 4587,'USN' from dual union all
select 4588,'OKB10' from dual union all
select 4589,'RVSN' from dual union all
select 459,'OKB1' from dual union all
select 4590,'PVO' from dual union all
select 4591,'TSKBEM' from dual union all
select 4592,'RVSN' from dual union all
select 4593,'RVSN' from dual union all
select 4594,'PVO' from dual union all
select 4595,'RVSN' from dual union all
select 4596,'NRO' from dual union all
select 4597,'PVO' from dual union all
select 4598,'PVO' from dual union all
select 4599,'RVSN' from dual union all
select 46,'USN' from dual union all
select 460,'OKB1' from dual union all
select 4600,'RVSN' from dual union all
select 4601,'NASA' from dual union all
select 4602,'NASA' from dual union all
select 4603,'NASA' from dual union all
select 4604,'NASA' from dual union all
select 4605,'NASA' from dual union all
select 4606,'NASA' from dual union all
select 4607,'NASA' from dual union all
select 4608,'NASA' from dual union all
select 4609,'NASA' from dual union all
select 461,'USAF' from dual union all
select 4610,'NASA' from dual union all
select 4611,'NASA' from dual union all
select 4612,'NASA' from dual union all
select 4613,'NASA' from dual union all
select 4614,'NASA' from dual union all
select 4615,'NASA' from dual union all
select 4616,'PVO' from dual union all
select 4617,'PVO' from dual union all
select 4618,'PVO' from dual union all
select 4619,'PVO' from dual union all
select 462,'USAF' from dual union all
select 4620,'PVO' from dual union all
select 4621,'PVO' from dual union all
select 4622,'PVO' from dual union all
select 4623,'PVO' from dual union all
select 4624,'PVO' from dual union all
select 4625,'PVO' from dual union all
select 4626,'PVO' from dual union all
select 4627,'PVO' from dual union all
select 4628,'PVO' from dual union all
select 4629,'PVO' from dual union all
select 463,'USAF' from dual union all
select 4630,'USAF' from dual union all
select 4631,'USAF' from dual union all
select 4632,'USAF' from dual union all
select 4633,'PVO' from dual union all
select 4634,'PVO' from dual union all
select 4635,'PVO' from dual union all
select 4636,'NASA' from dual union all
select 4637,'NASA' from dual union all
select 4638,'NASA' from dual union all
select 4639,'NASA' from dual union all
select 464,'USAF' from dual union all
select 4640,'NASA' from dual union all
select 4641,'NASA' from dual union all
select 4642,'NASA' from dual union all
select 4643,'NASA' from dual union all
select 4644,'NASA' from dual union all
select 4645,'NASA' from dual union all
select 4646,'NASA' from dual union all
select 4647,'NASA' from dual union all
select 4648,'NASA' from dual union all
select 4649,'NASA' from dual union all
select 465,'USAF' from dual union all
select 4650,'NASA' from dual union all
select 4651,'NASA' from dual union all
select 4652,'NASA' from dual union all
select 4653,'NASA' from dual union all
select 4654,'NASA' from dual union all
select 4655,'NASA' from dual union all
select 4656,'NASA' from dual union all
select 4657,'NASA' from dual union all
select 4658,'NASA' from dual union all
select 4659,'NASA' from dual union all
select 466,'USAF' from dual union all
select 4660,'NASA' from dual union all
select 4661,'PVO' from dual union all
select 4662,'PVO' from dual;
insert into SATELLITE_ORG
select 4663,'PVO' from dual union all
select 4664,'PVO' from dual union all
select 4665,'PVO' from dual union all
select 4666,'PVO' from dual union all
select 4667,'PVO' from dual union all
select 4668,'PVO' from dual union all
select 4669,'PVO' from dual union all
select 467,'USAF' from dual union all
select 4670,'PVO' from dual union all
select 4671,'PVO' from dual union all
select 4672,'NASA' from dual union all
select 4673,'NASA' from dual union all
select 4674,'NASA' from dual union all
select 4675,'NASA' from dual union all
select 4676,'NASA' from dual union all
select 4677,'NASA' from dual union all
select 4678,'NASA' from dual union all
select 4679,'NASA' from dual union all
select 468,'USAF' from dual union all
select 4680,'NASA' from dual union all
select 4681,'NASA' from dual union all
select 4682,'NASA' from dual union all
select 4683,'NASA' from dual union all
select 4684,'NASA' from dual union all
select 4685,'NASA' from dual union all
select 4686,'NASA' from dual union all
select 4687,'NASA' from dual union all
select 4688,'NASA' from dual union all
select 4689,'NASA' from dual union all
select 469,'USAF' from dual union all
select 4690,'NASA' from dual union all
select 4691,'IKI' from dual union all
select 4691,'NPOL' from dual union all
select 4692,'NASA' from dual union all
select 4693,'NASA' from dual union all
select 4694,'RVSN' from dual union all
select 4695,'RVSN' from dual union all
select 4696,'RVSN' from dual union all
select 4697,'RVSN' from dual union all
select 4698,'PVO' from dual union all
select 4699,'PVO' from dual union all
select 47,'USAF' from dual union all
select 470,'USAF' from dual union all
select 4700,'RVSN' from dual union all
select 4701,'NASA' from dual union all
select 4702,'NASA' from dual union all
select 4703,'NASA' from dual union all
select 4704,'NASA' from dual union all
select 4705,'NASA' from dual union all
select 4706,'NASA' from dual union all
select 4707,'NASA' from dual union all
select 4708,'NASA' from dual union all
select 4709,'NASA' from dual union all
select 471,'USAF' from dual union all
select 4710,'NASA' from dual union all
select 4711,'PVO' from dual union all
select 4712,'PVO' from dual union all
select 4713,'YUZH' from dual union all
select 4714,'RVSN' from dual union all
select 4715,'NASA' from dual union all
select 4716,'NASA' from dual union all
select 4717,'NASA' from dual union all
select 4718,'NASA' from dual union all
select 4719,'NASA' from dual union all
select 472,'USAF' from dual union all
select 4720,'NASA' from dual union all
select 4721,'CIA' from dual union all
select 4721,'NRO' from dual union all
select 4722,'NRO' from dual union all
select 4723,'PVO' from dual union all
select 4724,'PVO' from dual union all
select 4725,'PVO' from dual union all
select 4726,'PVO' from dual union all
select 4727,'PVO' from dual union all
select 4728,'PVO' from dual union all
select 4729,'PVO' from dual union all
select 473,'USAF' from dual union all
select 4730,'PVO' from dual union all
select 4731,'PVO' from dual union all
select 4732,'NASA' from dual union all
select 4733,'NASA' from dual union all
select 4734,'NASA' from dual union all
select 4735,'NASA' from dual union all
select 4736,'NASA' from dual union all
select 4737,'NASA' from dual union all
select 4738,'NASA' from dual union all
select 4739,'NASA' from dual union all
select 474,'USAF' from dual union all
select 4740,'NASA' from dual union all
select 4741,'NASA' from dual union all
select 4742,'NASA' from dual union all
select 4743,'NASA' from dual union all
select 4744,'NASA' from dual union all
select 4745,'NASA' from dual union all
select 4746,'NASA' from dual union all
select 4747,'NASA' from dual union all
select 4748,'NASA' from dual union all
select 4749,'NASA' from dual union all
select 475,'USAF' from dual union all
select 4750,'NASA' from dual;
insert into SATELLITE_ORG
select 4751,'NASA' from dual union all
select 4752,'NASA' from dual union all
select 4753,'NASA' from dual union all
select 4754,'NASA' from dual union all
select 4755,'NASA' from dual union all
select 4756,'NASA' from dual union all
select 4757,'NASA' from dual union all
select 4758,'NASA' from dual union all
select 4759,'NASA' from dual union all
select 476,'USAF' from dual union all
select 4760,'TSKBEM' from dual union all
select 4761,'RVSN' from dual union all
select 4762,'PVO' from dual union all
select 4763,'RVSN' from dual union all
select 4764,'NASA' from dual union all
select 4765,'NASA' from dual union all
select 4766,'NASA' from dual union all
select 4767,'NASA' from dual union all
select 4768,'NASA' from dual union all
select 4769,'NASA' from dual union all
select 477,'USAF' from dual union all
select 4770,'NASA' from dual union all
select 4771,'NASA' from dual union all
select 4772,'NASA' from dual union all
select 4773,'NASA' from dual union all
select 4774,'NASA' from dual union all
select 4775,'NASA' from dual union all
select 4776,'NASA' from dual union all
select 4777,'NASA' from dual union all
select 4778,'NASA' from dual union all
select 4779,'RVSN' from dual union all
select 478,'USAF' from dual union all
select 4780,'TSKBEM' from dual union all
select 4781,'RVSN' from dual union all
select 4782,'RVSN' from dual union all
select 4783,'OKB10' from dual union all
select 4784,'RVSN' from dual union all
select 4785,'RVSN' from dual union all
select 4786,'TSKBEM' from dual union all
select 4787,'TSKBEM' from dual union all
select 4788,'RVSN' from dual union all
select 4789,'RVSN' from dual union all
select 479,'USAF' from dual union all
select 4790,'RVSN' from dual union all
select 4791,'RVSN' from dual union all
select 4792,'RVSN' from dual union all
select 4793,'NOAA' from dual union all
select 4794,'NASA' from dual union all
select 4795,'RVSN' from dual union all
select 4796,'RVSN' from dual union all
select 4797,'NASA' from dual union all
select 4798,'NASA' from dual union all
select 4799,'OKB10' from dual union all
select 48,'CIA' from dual union all
select 48,'USAF' from dual union all
select 480,'USAF' from dual union all
select 4800,'RVSN' from dual union all
select 4801,'CNES' from dual union all
select 4802,'CNES' from dual union all
select 4803,'CNES' from dual union all
select 4804,'RVSN' from dual union all
select 4805,'RVSN' from dual union all
select 4806,'TSN108' from dual union all
select 4807,'RVSN' from dual union all
select 4808,'RVSN' from dual union all
select 4809,'YUZH' from dual union all
select 481,'CIA' from dual union all
select 481,'NRO' from dual union all
select 4810,'TSKBEM' from dual union all
select 4811,'PVO' from dual union all
select 4812,'RVSN' from dual union all
select 4813,'TSN108' from dual union all
select 4814,'RVSN' from dual union all
select 4815,'YUZH' from dual union all
select 4816,'YUZH' from dual union all
select 4817,'TSKBEM' from dual union all
select 4818,'NASA' from dual union all
select 4819,'NASA' from dual union all
select 482,'OKB1' from dual union all
select 4820,'NASA' from dual union all
select 4821,'NASA' from dual union all
select 4822,'NASA' from dual union all
select 4823,'PVO' from dual union all
select 4824,'PVO' from dual union all
select 4825,'PVO' from dual union all
select 4826,'PVO' from dual union all
select 4827,'PVO' from dual union all
select 4828,'PVO' from dual union all
select 4829,'RVSN' from dual union all
select 483,'OKB1' from dual union all
select 4830,'RVSN' from dual union all
select 4831,'RVSN' from dual union all
select 4832,'RVSN' from dual union all
select 4833,'RVSN' from dual union all
select 4834,'RVSN' from dual union all
select 4835,'RVSN' from dual union all
select 4836,'RVSN' from dual union all
select 4837,'RVSN' from dual union all
select 4838,'RVSN' from dual union all
select 4839,'CNES' from dual;
insert into SATELLITE_ORG
select 484,'OKB1' from dual union all
select 4840,'RVSN' from dual union all
select 4841,'NASA' from dual union all
select 4842,'NASA' from dual union all
select 4843,'USN' from dual union all
select 4844,'NASA' from dual union all
select 4845,'RVSN' from dual union all
select 4846,'RVSN' from dual union all
select 4847,'PVO' from dual union all
select 4848,'RVSN' from dual union all
select 4849,'GMS' from dual union all
select 4849,'RVSN' from dual union all
select 485,'OKB1' from dual union all
select 4850,'RVSN' from dual union all
select 4851,'NASA' from dual union all
select 4852,'NASA' from dual union all
select 4853,'NASA' from dual union all
select 4854,'NASA' from dual union all
select 4855,'NASA' from dual union all
select 4856,'NASA' from dual union all
select 4857,'NASA' from dual union all
select 4858,'NASA' from dual union all
select 4859,'NASA' from dual union all
select 486,'OKB1' from dual union all
select 4860,'NASA' from dual union all
select 4861,'NASA' from dual union all
select 4862,'NASA' from dual union all
select 4863,'NASA' from dual union all
select 4864,'NASA' from dual union all
select 4865,'NASA' from dual union all
select 4866,'NASA' from dual union all
select 4867,'NASA' from dual union all
select 4868,'NASA' from dual union all
select 4869,'NASA' from dual union all
select 487,'OKB1' from dual union all
select 4870,'NASA' from dual union all
select 4871,'NASA' from dual union all
select 4872,'RVSN' from dual union all
select 4873,'RVSN' from dual union all
select 4874,'NRO' from dual union all
select 4875,'NRO' from dual union all
select 4876,'USAF' from dual union all
select 4877,'NASA' from dual union all
select 4878,'RVSN' from dual union all
select 4879,'RVSN' from dual union all
select 488,'OKB1' from dual union all
select 4880,'USAF' from dual union all
select 4881,'INTEL' from dual union all
select 4882,'NASA' from dual union all
select 4883,'RVSN' from dual union all
select 4884,'PVO' from dual union all
select 4885,'RVSN' from dual union all
select 4886,'RVSN' from dual union all
select 4887,'USAF' from dual union all
select 4888,'USAF' from dual union all
select 4889,'USAF' from dual union all
select 489,'OKB1' from dual union all
select 4890,'USAF' from dual union all
select 4891,'USAF' from dual union all
select 4892,'USAF' from dual union all
select 4893,'USAF' from dual union all
select 4894,'USAF' from dual union all
select 4895,'USAF' from dual union all
select 4896,'PVO' from dual union all
select 4897,'USAF' from dual union all
select 4898,'NASA' from dual union all
select 4899,'NASA' from dual union all
select 49,'NASA' from dual union all
select 490,'CIA' from dual union all
select 490,'NRO' from dual union all
select 4900,'NASA' from dual union all
select 4901,'UNK' from dual union all
select 4902,'NATO' from dual union all
select 4903,'NASA' from dual union all
select 4904,'NASA' from dual union all
select 4905,'NASA' from dual union all
select 4906,'USAF' from dual union all
select 4907,'USAF' from dual union all
select 4908,'NASA' from dual union all
select 4909,'NRO' from dual union all
select 491,'OKB1' from dual union all
select 4910,'NASA' from dual union all
select 4911,'NASA' from dual union all
select 4912,'NASA' from dual union all
select 4913,'NASA' from dual union all
select 4914,'NASA' from dual union all
select 4915,'NASA' from dual union all
select 4916,'NASA' from dual union all
select 4917,'NASA' from dual union all
select 4918,'NASA' from dual union all
select 4919,'NASA' from dual union all
select 492,'OKB1' from dual union all
select 4920,'NASA' from dual union all
select 4921,'NASA' from dual union all
select 4922,'PVO' from dual union all
select 4923,'RVSN' from dual union all
select 4924,'NASA' from dual union all
select 4925,'NASA' from dual union all
select 4926,'NASA' from dual union all
select 4927,'RVSN' from dual;
insert into SATELLITE_ORG
select 4928,'NASA' from dual union all
select 4929,'NASA' from dual union all
select 493,'OKB1' from dual union all
select 4930,'NASA' from dual union all
select 4931,'NASA' from dual union all
select 4932,'NASA' from dual union all
select 4933,'NASA' from dual union all
select 4934,'NASA' from dual union all
select 4935,'NASA' from dual union all
select 4936,'NASA' from dual union all
select 4937,'USAF' from dual union all
select 4938,'USAF' from dual union all
select 4939,'USAF' from dual union all
select 494,'OKB1' from dual union all
select 4940,'PVO' from dual union all
select 4941,'PVO' from dual union all
select 4942,'PVO' from dual union all
select 4943,'NASA' from dual union all
select 4944,'NASA' from dual union all
select 4945,'NASA' from dual union all
select 4946,'NASA' from dual union all
select 4947,'NASA' from dual union all
select 4948,'NASA' from dual union all
select 4949,'NASA' from dual union all
select 495,'OKB1' from dual union all
select 4950,'NASA' from dual union all
select 4951,'NASA' from dual union all
select 4952,'ISAS' from dual union all
select 4953,'USAF' from dual union all
select 4954,'USAF' from dual union all
select 4955,'TSN108' from dual union all
select 4956,'RVSN' from dual union all
select 4957,'USN' from dual union all
select 4958,'USN' from dual union all
select 4959,'RVSN' from dual union all
select 496,'OKB1' from dual union all
select 4960,'RVSN' from dual union all
select 4961,'RVSN' from dual union all
select 4962,'RVSN' from dual union all
select 4963,'USN' from dual union all
select 4964,'PVO' from dual union all
select 4965,'RVSN' from dual union all
select 4966,'TSKBEM' from dual union all
select 4967,'RVSN' from dual union all
select 4968,'NASA' from dual union all
select 4969,'NASA' from dual union all
select 497,'OKB1' from dual union all
select 4970,'NASA' from dual union all
select 4971,'NASA' from dual union all
select 4972,'NASA' from dual union all
select 4973,'NASA' from dual union all
select 4974,'NASA' from dual union all
select 4975,'NASA' from dual union all
select 4976,'NASA' from dual union all
select 4977,'NASA' from dual union all
select 4978,'NASA' from dual union all
select 4979,'NASA' from dual union all
select 498,'OKB1' from dual union all
select 4980,'NASA' from dual union all
select 4981,'NASA' from dual union all
select 4982,'NASA' from dual union all
select 4983,'NASA' from dual union all
select 4984,'NASA' from dual union all
select 4985,'NASA' from dual union all
select 4986,'NASA' from dual union all
select 4987,'NASA' from dual union all
select 4988,'NASA' from dual union all
select 4989,'NASA' from dual union all
select 499,'OKB1' from dual union all
select 4990,'NASA' from dual union all
select 4991,'NASA' from dual union all
select 4992,'NASA' from dual union all
select 4993,'NASA' from dual union all
select 4994,'NASA' from dual union all
select 4995,'NASA' from dual union all
select 4996,'NASA' from dual union all
select 4997,'NASA' from dual union all
select 4998,'NASA' from dual union all
select 4999,'NASA' from dual union all
select 5,'USN' from dual union all
select 50,'NASA' from dual union all
select 500,'OKB1' from dual union all
select 5000,'NASA' from dual union all
select 5001,'TSN108' from dual union all
select 5002,'TSN108' from dual union all
select 5003,'RVSN' from dual union all
select 5004,'RVSN' from dual union all
select 5005,'RVSN' from dual union all
select 5006,'RVSN' from dual union all
select 5007,'MAI' from dual union all
select 5008,'TSKBEM' from dual union all
select 5009,'PVO' from dual union all
select 501,'OKB1' from dual union all
select 5010,'PVO' from dual union all
select 5011,'PVO' from dual union all
select 5012,'PVO' from dual union all
select 5013,'PVO' from dual union all
select 5014,'PVO' from dual union all
select 5015,'PVO' from dual union all
select 5016,'PVO' from dual;
insert into SATELLITE_ORG
select 5017,'PVO' from dual union all
select 5018,'PVO' from dual union all
select 5019,'PVO' from dual union all
select 502,'NRO' from dual union all
select 502,'USN' from dual union all
select 5020,'PVO' from dual union all
select 5021,'PVO' from dual union all
select 5022,'PVO' from dual union all
select 5023,'NASA' from dual union all
select 5024,'NASA' from dual union all
select 5025,'NASA' from dual union all
select 5026,'NASA' from dual union all
select 5027,'NASA' from dual union all
select 5028,'PVO' from dual union all
select 5029,'PVO' from dual union all
select 503,'NASA' from dual union all
select 5030,'PVO' from dual union all
select 5031,'PVO' from dual union all
select 5032,'PVO' from dual union all
select 5033,'PVO' from dual union all
select 5034,'PVO' from dual union all
select 5035,'USAF' from dual union all
select 5036,'USN' from dual union all
select 5037,'NASA' from dual union all
select 5038,'NASA' from dual union all
select 5039,'TSKBEM' from dual union all
select 504,'USN' from dual union all
select 5040,'TSKBEM' from dual union all
select 5041,'MAI' from dual union all
select 5042,'PVO' from dual union all
select 5043,'NASA' from dual union all
select 5044,'NASA' from dual union all
select 5045,'NASA' from dual union all
select 5046,'NASA' from dual union all
select 5047,'RVSN' from dual union all
select 5048,'RVSN' from dual union all
select 5049,'RVSN' from dual union all
select 505,'CIA' from dual union all
select 505,'NRO' from dual union all
select 5050,'PVO' from dual union all
select 5051,'RVSN' from dual union all
select 5052,'PVO' from dual union all
select 5053,'NRO' from dual union all
select 5054,'USAF' from dual union all
select 5055,'RVSN' from dual union all
select 5056,'USAF' from dual union all
select 5057,'USAF' from dual union all
select 5058,'USAF' from dual union all
select 5059,'CIA' from dual union all
select 5059,'NRO' from dual union all
select 506,'NASA' from dual union all
select 5060,'NASA' from dual union all
select 5061,'NASA' from dual union all
select 5062,'NASA' from dual union all
select 5063,'NASA' from dual union all
select 5064,'NASA' from dual union all
select 5065,'NASA' from dual union all
select 5066,'NASA' from dual union all
select 5067,'NASA' from dual union all
select 5068,'PVO' from dual union all
select 5069,'PVO' from dual union all
select 507,'NRO' from dual union all
select 507,'USN' from dual union all
select 5070,'PVO' from dual union all
select 5071,'PVO' from dual union all
select 5072,'PVO' from dual union all
select 5073,'PVO' from dual union all
select 5074,'PVO' from dual union all
select 5075,'PVO' from dual union all
select 5076,'PVO' from dual union all
select 5077,'PVO' from dual union all
select 5078,'PVO' from dual union all
select 5079,'PVO' from dual union all
select 508,'USN' from dual union all
select 5080,'PVO' from dual union all
select 5081,'PVO' from dual union all
select 5082,'PVO' from dual union all
select 5083,'PVO' from dual union all
select 5084,'PVO' from dual union all
select 5085,'USAF' from dual union all
select 5086,'RVSN' from dual union all
select 5087,'RVSN' from dual union all
select 5088,'PVO' from dual union all
select 5089,'PVO' from dual union all
select 509,'USN' from dual union all
select 5090,'PVO' from dual union all
select 5091,'PVO' from dual union all
select 5092,'PVO' from dual union all
select 5093,'PVO' from dual union all
select 5094,'PVO' from dual union all
select 5095,'PVO' from dual union all
select 5096,'PVO' from dual union all
select 5097,'PVO' from dual union all
select 5098,'PVO' from dual union all
select 5099,'PVO' from dual union all
select 51,'NASA' from dual union all
select 510,'DRTE' from dual union all
select 5100,'PVO' from dual union all
select 5101,'PVO' from dual union all
select 5102,'PVO' from dual;
insert into SATELLITE_ORG
select 5103,'PVO' from dual union all
select 5104,'CRC' from dual union all
select 5105,'VMF' from dual union all
select 5106,'NASA' from dual union all
select 5107,'VMF' from dual union all
select 5108,'RVSN' from dual union all
select 5109,'VMF' from dual union all
select 511,'DRTE' from dual union all
select 5110,'RVSN' from dual union all
select 5111,'RVSN' from dual union all
select 5112,'NASA' from dual union all
select 5113,'PVO' from dual union all
select 5114,'RVSN' from dual union all
select 5115,'PVO' from dual union all
select 5116,'PVO' from dual union all
select 5117,'TSN108' from dual union all
select 5118,'RVSN' from dual union all
select 5119,'RVSN' from dual union all
select 512,'OKB1' from dual union all
select 5120,'RVSN' from dual union all
select 5121,'RVSN' from dual union all
select 5122,'UNK' from dual union all
select 5123,'UNK' from dual union all
select 5124,'RVSN' from dual union all
select 5125,'RVSN' from dual union all
select 5126,'ISAS' from dual union all
select 5127,'RVSN' from dual union all
select 5128,'CNES' from dual union all
select 5129,'NASA' from dual union all
select 513,'USN' from dual union all
select 5130,'PVO' from dual union all
select 5131,'PVO' from dual union all
select 5132,'PVO' from dual union all
select 5133,'NASA' from dual union all
select 5134,'RVSN' from dual union all
select 5135,'CNES' from dual union all
select 5136,'PVO' from dual union all
select 5137,'PVO' from dual union all
select 5138,'PVO' from dual union all
select 5139,'CNES' from dual union all
select 514,'USN' from dual union all
select 5140,'CNES' from dual union all
select 5141,'CNES' from dual union all
select 5142,'GMS' from dual union all
select 5142,'RVSN' from dual union all
select 5143,'RVSN' from dual union all
select 5144,'PVO' from dual union all
select 5145,'PVO' from dual union all
select 5146,'NASA' from dual union all
select 5147,'USAF' from dual union all
select 5148,'NASA' from dual union all
select 5149,'NASA' from dual union all
select 515,'NASA' from dual union all
select 5150,'NASA' from dual union all
select 5151,'NASA' from dual union all
select 5152,'NASA' from dual union all
select 5153,'NASA' from dual union all
select 5154,'NASA' from dual union all
select 5155,'PVO' from dual union all
select 5156,'PVO' from dual union all
select 5157,'PVO' from dual union all
select 5158,'USAF' from dual union all
select 5159,'USAF' from dual union all
select 516,'USAF' from dual union all
select 5160,'TSKBEM' from dual union all
select 5161,'RVSN' from dual union all
select 5162,'TSKBEM' from dual union all
select 5163,'TSKBEM' from dual union all
select 5164,'CNES' from dual union all
select 5165,'NASA' from dual union all
select 5166,'TSKBEM' from dual union all
select 5167,'TSKBEM' from dual union all
select 5168,'TSKBEM' from dual union all
select 5169,'RVSN' from dual union all
select 517,'OKB1' from dual union all
select 5170,'RVSN' from dual union all
select 5171,'NRO' from dual union all
select 5172,'TSKBEM' from dual union all
select 5173,'RVSN' from dual union all
select 5174,'OKB10' from dual union all
select 5175,'RVSN' from dual union all
select 5176,'CNR' from dual union all
select 5177,'PVO' from dual union all
select 5178,'RVSN' from dual union all
select 5179,'RVSN' from dual union all
select 518,'RVSN' from dual union all
select 5180,'OKB10' from dual union all
select 5181,'RVSN' from dual union all
select 5182,'USAF' from dual union all
select 5183,'NASA' from dual union all
select 5184,'NASA' from dual union all
select 5185,'NASA' from dual union all
select 5186,'NASA' from dual union all
select 5187,'PVO' from dual union all
select 5188,'PVO' from dual union all
select 5189,'PVO' from dual union all
select 519,'NASA' from dual union all
select 5190,'NASA' from dual union all
select 5191,'PVO' from dual union all
select 5192,'PVO' from dual;
insert into SATELLITE_ORG
select 5193,'PVO' from dual union all
select 5194,'PVO' from dual union all
select 5195,'USAF' from dual union all
select 5196,'PVO' from dual union all
select 5197,'PVO' from dual union all
select 5198,'PVO' from dual union all
select 5199,'PVO' from dual union all
select 52,'NASA' from dual union all
select 520,'NRO' from dual union all
select 5200,'PVO' from dual union all
select 5201,'PVO' from dual union all
select 5202,'PVO' from dual union all
select 5203,'NASA' from dual union all
select 5204,'USAF' from dual union all
select 5205,'USAF' from dual union all
select 5206,'USAF' from dual union all
select 5207,'RVSN' from dual union all
select 5208,'RVSN' from dual union all
select 5209,'CNES' from dual union all
select 521,'RVSN' from dual union all
select 5210,'OKB10' from dual union all
select 5211,'OKB10' from dual union all
select 5212,'OKB10' from dual union all
select 5213,'OKB10' from dual union all
select 5214,'OKB10' from dual union all
select 5215,'OKB10' from dual union all
select 5216,'OKB10' from dual union all
select 5217,'OKB10' from dual union all
select 5218,'RVSN' from dual union all
select 5219,'NASA' from dual union all
select 522,'OKB1' from dual union all
select 5220,'NASA' from dual union all
select 5221,'IKI' from dual union all
select 5221,'NPOL' from dual union all
select 5222,'PVO' from dual union all
select 5223,'PVO' from dual union all
select 5224,'PVO' from dual union all
select 5225,'RVSN' from dual union all
select 5226,'NASA' from dual union all
select 5227,'TSN108' from dual union all
select 5228,'TSKBEM' from dual union all
select 5229,'RVSN' from dual union all
select 523,'USN' from dual union all
select 5230,'RVSN' from dual union all
select 5231,'RVSN' from dual union all
select 5232,'PVO' from dual union all
select 5233,'RVSN' from dual union all
select 5234,'IKI' from dual union all
select 5234,'NPOL' from dual union all
select 5235,'RVSN' from dual union all
select 5236,'RVSN' from dual union all
select 5237,'NASA' from dual union all
select 5238,'OKB10' from dual union all
select 5239,'RVSN' from dual union all
select 524,'RVSN' from dual union all
select 5240,'PVO' from dual union all
select 5241,'PVO' from dual union all
select 5242,'PVO' from dual union all
select 5243,'PVO' from dual union all
select 5244,'PVO' from dual union all
select 5245,'UNK' from dual union all
select 5246,'PVO' from dual union all
select 5247,'RVSN' from dual union all
select 5248,'RVSN' from dual union all
select 5249,'RVSN' from dual union all
select 525,'CIA' from dual union all
select 525,'NRO' from dual union all
select 5250,'RVSN' from dual union all
select 5251,'RVSN' from dual union all
select 5252,'IKI' from dual union all
select 5252,'NPOL' from dual union all
select 5253,'VS' from dual union all
select 5254,'RVSN' from dual union all
select 5255,'RVSN' from dual union all
select 5256,'RVSN' from dual union all
select 5257,'VS' from dual union all
select 5258,'VS' from dual union all
select 5259,'VS' from dual union all
select 526,'USAF' from dual union all
select 5260,'RVSN' from dual union all
select 5261,'JPL' from dual union all
select 5261,'NASA' from dual union all
select 5262,'PVO' from dual union all
select 5263,'RVSN' from dual union all
select 5264,'CNES' from dual union all
select 5265,'USN' from dual union all
select 5266,'NASA' from dual union all
select 5267,'NASA' from dual union all
select 5268,'CNES' from dual union all
select 5269,'UNK' from dual union all
select 527,'NRO' from dual union all
select 5270,'USAF' from dual union all
select 5271,'USAF' from dual union all
select 5272,'PVO' from dual union all
select 5273,'PVO' from dual union all
select 5274,'PVO' from dual union all
select 5275,'USAF' from dual union all
select 5276,'PVO' from dual union all
select 5277,'PVO' from dual union all
select 5278,'PVO' from dual;
insert into SATELLITE_ORG
select 5279,'PVO' from dual union all
select 528,'USAF' from dual union all
select 5280,'PVO' from dual union all
select 5281,'YUZH' from dual union all
select 5282,'RVSN' from dual union all
select 5283,'TSKBEM' from dual union all
select 5284,'RVSN' from dual union all
select 5285,'USAF' from dual union all
select 5286,'RVSN' from dual union all
select 5287,'RVSN' from dual union all
select 5288,'RVSN' from dual union all
select 5289,'TSKBEM' from dual union all
select 529,'USAF' from dual union all
select 5290,'RVSN' from dual union all
select 5291,'TSKBEM' from dual union all
select 5292,'TSKBEM' from dual union all
select 5293,'PVO' from dual union all
select 5294,'PVO' from dual union all
select 5295,'PVO' from dual union all
select 5296,'PVO' from dual union all
select 5297,'NRO' from dual union all
select 5298,'USAF' from dual union all
select 5299,'VS' from dual union all
select 53,'NASA' from dual union all
select 530,'USAF' from dual union all
select 5300,'OKB10' from dual union all
select 5301,'OKB10' from dual union all
select 5302,'RVSN' from dual union all
select 5303,'RVSN' from dual union all
select 5304,'RVSN' from dual union all
select 5305,'RVSN' from dual union all
select 5306,'RVSN' from dual union all
select 5307,'RVSN' from dual union all
select 5308,'RVSN' from dual union all
select 5309,'NASA' from dual union all
select 531,'USAF' from dual union all
select 5310,'USAF' from dual union all
select 5311,'RVSN' from dual union all
select 5312,'USAF' from dual union all
select 5313,'RVSN' from dual union all
select 5314,'RVSN' from dual union all
select 5315,'TSKBEM' from dual union all
select 5316,'RVSN' from dual union all
select 5317,'USN' from dual union all
select 5318,'NASA' from dual union all
select 5319,'USN' from dual union all
select 532,'NASA' from dual union all
select 5320,'USN' from dual union all
select 5321,'PVO' from dual union all
select 5322,'PVO' from dual union all
select 5323,'PVO' from dual union all
select 5324,'PVO' from dual union all
select 5325,'PVO' from dual union all
select 5326,'RVSN' from dual union all
select 5327,'GMS' from dual union all
select 5327,'RVSN' from dual union all
select 5328,'RVSN' from dual union all
select 5329,'NRO' from dual union all
select 533,'USAF' from dual union all
select 5330,'RVSN' from dual union all
select 5331,'RVSN' from dual union all
select 5332,'RVSN' from dual union all
select 5333,'UNK' from dual union all
select 5334,'PVO' from dual union all
select 5335,'PVO' from dual union all
select 5336,'PVO' from dual union all
select 5337,'PVO' from dual union all
select 5338,'PVO' from dual union all
select 5339,'USAF' from dual union all
select 534,'NASA' from dual union all
select 5340,'RVSN' from dual union all
select 5341,'RVSN' from dual union all
select 5342,'RVSN' from dual union all
select 5343,'RVSN' from dual union all
select 5344,'USAF' from dual union all
select 5345,'USAF' from dual union all
select 5346,'NASA' from dual union all
select 5347,'NASA' from dual union all
select 5348,'NASA' from dual union all
select 5349,'NASA' from dual union all
select 535,'USAF' from dual union all
select 5350,'NASA' from dual union all
select 5351,'NASA' from dual union all
select 5352,'NASA' from dual union all
select 5353,'RVSN' from dual union all
select 5354,'RVSN' from dual union all
select 5355,'RVSN' from dual union all
select 5356,'RVSN' from dual union all
select 5357,'OKB10' from dual union all
select 5358,'OKB10' from dual union all
select 5359,'NASA' from dual union all
select 536,'USAF' from dual union all
select 5360,'CRC' from dual union all
select 5361,'USAF' from dual union all
select 5362,'USAF' from dual union all
select 5363,'USN' from dual union all
select 5364,'RVSN' from dual union all
select 5365,'RVSN' from dual union all
select 5366,'NASA' from dual union all
select 5367,'RVSN' from dual;
insert into SATELLITE_ORG
select 5368,'RVSN' from dual union all
select 5369,'RVSN' from dual union all
select 537,'USAF' from dual union all
select 5370,'RVSN' from dual union all
select 5371,'RVSN' from dual union all
select 5372,'USAF' from dual union all
select 5373,'RVSN' from dual union all
select 5374,'RVSN' from dual union all
select 5375,'USAF' from dual union all
select 5376,'USAF' from dual union all
select 5377,'NASA' from dual union all
select 5378,'RVSN' from dual union all
select 5379,'RVSN' from dual union all
select 538,'USAF' from dual union all
select 5380,'RVSN' from dual union all
select 5381,'RVSN' from dual union all
select 5382,'USAF' from dual union all
select 5383,'USAF' from dual union all
select 5384,'USAF' from dual union all
select 5385,'RVSN' from dual union all
select 5386,'NASA' from dual union all
select 5387,'NASA' from dual union all
select 5388,'NASA' from dual union all
select 5389,'NASA' from dual union all
select 539,'USAF' from dual union all
select 5390,'NASA' from dual union all
select 5391,'NASA' from dual union all
select 5392,'NASA' from dual union all
select 5393,'NASA' from dual union all
select 5394,'USAF' from dual union all
select 5395,'USAF' from dual union all
select 5396,'USAF' from dual union all
select 5397,'USAF' from dual union all
select 5398,'USA' from dual union all
select 5399,'USAF' from dual union all
select 54,'CIA' from dual union all
select 54,'USAF' from dual union all
select 540,'USAF' from dual union all
select 5400,'USAF' from dual union all
select 5401,'USAF' from dual union all
select 5402,'RVSN' from dual union all
select 5403,'RVSN' from dual union all
select 5404,'RVSN' from dual union all
select 5405,'USAF' from dual union all
select 5406,'USAF' from dual union all
select 5407,'TSKBEM' from dual union all
select 5408,'RVSN' from dual union all
select 5409,'NRO' from dual union all
select 541,'USAF' from dual union all
select 5410,'USAF' from dual union all
select 5411,'PVO' from dual union all
select 5412,'PVO' from dual union all
select 5413,'PVO' from dual union all
select 5414,'PVO' from dual union all
select 5415,'PVO' from dual union all
select 5416,'PVO' from dual union all
select 5417,'PVO' from dual union all
select 5418,'PVO' from dual union all
select 5419,'ISAS' from dual union all
select 542,'USAF' from dual union all
select 5420,'CNES' from dual union all
select 5421,'NASA' from dual union all
select 5422,'NASA' from dual union all
select 5423,'NASA' from dual union all
select 5424,'NASA' from dual union all
select 5425,'RVSN' from dual union all
select 5426,'CNES' from dual union all
select 5427,'TSKBEM' from dual union all
select 5428,'TSKBEM' from dual union all
select 5429,'USAF' from dual union all
select 543,'USAF' from dual union all
select 5430,'USAF' from dual union all
select 5431,'PVO' from dual union all
select 5432,'PVO' from dual union all
select 5433,'PVO' from dual union all
select 5434,'PVO' from dual union all
select 5435,'CNES' from dual union all
select 5436,'RVSN' from dual union all
select 5437,'RVSN' from dual union all
select 5438,'NASA' from dual union all
select 5439,'MO' from dual union all
select 544,'USAF' from dual union all
select 5440,'CNES' from dual union all
select 5441,'PVO' from dual union all
select 5442,'RVSN' from dual union all
select 5443,'NASA' from dual union all
select 5444,'USN' from dual union all
select 5445,'NASA' from dual union all
select 5446,'NASA' from dual union all
select 5447,'USN' from dual union all
select 5448,'IKI' from dual union all
select 5448,'NPOL' from dual union all
select 5449,'RVSN' from dual union all
select 545,'USAF' from dual union all
select 5450,'RVSN' from dual union all
select 5451,'RVSN' from dual union all
select 5452,'NRO' from dual union all
select 5453,'RVSN' from dual union all
select 5454,'PVO' from dual union all
select 5455,'PVO' from dual;
insert into SATELLITE_ORG
select 5456,'PVO' from dual union all
select 5457,'USAF' from dual union all
select 5458,'PVO' from dual union all
select 5459,'USAF' from dual union all
select 546,'USAF' from dual union all
select 5460,'PVO' from dual union all
select 5461,'VS' from dual union all
select 5462,'RVSN' from dual union all
select 5463,'VS' from dual union all
select 5464,'VS' from dual union all
select 5465,'VS' from dual union all
select 5466,'VS' from dual union all
select 5467,'RVSN' from dual union all
select 5468,'CIA' from dual union all
select 5468,'NRO' from dual union all
select 5469,'NRO' from dual union all
select 547,'USAF' from dual union all
select 5470,'VS' from dual union all
select 5471,'USAF' from dual union all
select 5472,'USAF' from dual union all
select 5473,'NASA' from dual union all
select 5474,'USAF' from dual union all
select 5475,'TSKBEM' from dual union all
select 5476,'RVSN' from dual union all
select 5477,'TSKBEM' from dual union all
select 5478,'TSKBEM' from dual union all
select 5479,'RVSN' from dual union all
select 548,'USAF' from dual union all
select 5480,'PVO' from dual union all
select 5481,'RVSN' from dual union all
select 5482,'TSKBEM' from dual union all
select 5483,'TSKBEM' from dual union all
select 5484,'TSKBEM' from dual union all
select 5485,'ISAS' from dual union all
select 5486,'RVSN' from dual union all
select 5487,'RVSN' from dual union all
select 5488,'IKI' from dual union all
select 5488,'NPOL' from dual union all
select 5489,'RVSN' from dual union all
select 549,'USAF' from dual union all
select 5490,'RVSN' from dual union all
select 5491,'NASA' from dual union all
select 5492,'NASA' from dual union all
select 5493,'RVSN' from dual union all
select 5494,'RVSN' from dual union all
select 5495,'RVSN' from dual union all
select 5496,'RVSN' from dual union all
select 5497,'RVSN' from dual union all
select 5498,'ISAS' from dual union all
select 5499,'NASA' from dual union all
select 55,'OKB1' from dual union all
select 550,'USAF' from dual union all
select 5500,'USAF' from dual union all
select 5501,'NASA' from dual union all
select 5502,'NASA' from dual union all
select 5503,'USAF' from dual union all
select 5504,'NASA' from dual union all
select 5505,'NASA' from dual union all
select 5506,'NASA' from dual union all
select 5507,'NASA' from dual union all
select 5508,'NASA' from dual union all
select 5509,'NASA' from dual union all
select 551,'USAF' from dual union all
select 5510,'NASA' from dual union all
select 5511,'USAF' from dual union all
select 5512,'USAF' from dual union all
select 5513,'USAF' from dual union all
select 5514,'PVO' from dual union all
select 5515,'USAF' from dual union all
select 5516,'PVO' from dual union all
select 5517,'PVO' from dual union all
select 5518,'PVO' from dual union all
select 5519,'USAF' from dual union all
select 552,'USAF' from dual union all
select 5520,'USAF' from dual union all
select 5521,'USAF' from dual union all
select 5522,'USAF' from dual union all
select 5523,'PVO' from dual union all
select 5524,'PVO' from dual union all
select 5525,'PVO' from dual union all
select 5526,'PVO' from dual union all
select 5527,'PVO' from dual union all
select 5528,'PVO' from dual union all
select 5529,'PVO' from dual union all
select 553,'NASA' from dual union all
select 5530,'PVO' from dual union all
select 5531,'PVO' from dual union all
select 5532,'PVO' from dual union all
select 5533,'PVO' from dual union all
select 5534,'PVO' from dual union all
select 5535,'NASA' from dual union all
select 5536,'RVSN' from dual union all
select 5537,'RVSN' from dual union all
select 5538,'RVSN' from dual union all
select 5539,'RVSN' from dual union all
select 554,'OKB1' from dual union all
select 5540,'USN' from dual union all
select 5541,'RVSN' from dual union all
select 5542,'RVSN' from dual union all
select 5543,'RVSN' from dual;
insert into SATELLITE_ORG
select 5544,'RVSN' from dual union all
select 5545,'RVSN' from dual union all
select 5546,'RVSN' from dual union all
select 5547,'OKB10' from dual union all
select 5548,'OKB10' from dual union all
select 5549,'OKB10' from dual union all
select 555,'RVSN' from dual union all
select 5550,'OKB10' from dual union all
select 5551,'OKB10' from dual union all
select 5552,'OKB10' from dual union all
select 5553,'OKB10' from dual union all
select 5554,'OKB10' from dual union all
select 5555,'RVSN' from dual union all
select 5556,'USAF' from dual union all
select 5557,'USAF' from dual union all
select 5558,'RVSN' from dual union all
select 5559,'RVSN' from dual union all
select 556,'USAF' from dual union all
select 5560,'USAF' from dual union all
select 5561,'RVSN' from dual union all
select 5562,'TSKBEM' from dual union all
select 5563,'PVO' from dual union all
select 5564,'RVSN' from dual union all
select 5565,'NASA' from dual union all
select 5566,'NASA' from dual union all
select 5567,'USAF' from dual union all
select 5568,'USAF' from dual union all
select 5569,'USAF' from dual union all
select 557,'USAF' from dual union all
select 5570,'NASA' from dual union all
select 5571,'USAF' from dual union all
select 5572,'USAF' from dual union all
select 5573,'RVSN' from dual union all
select 5574,'PVO' from dual union all
select 5575,'NRO' from dual union all
select 5576,'NASA' from dual union all
select 5577,'RVSN' from dual union all
select 5578,'RVSN' from dual union all
select 5579,'RVSN' from dual union all
select 558,'USAF' from dual union all
select 5580,'RAE' from dual union all
select 5581,'RAE' from dual union all
select 5582,'RAE' from dual union all
select 5583,'VS' from dual union all
select 5584,'VS' from dual union all
select 5585,'RVSN' from dual union all
select 5586,'RVSN' from dual union all
select 5587,'DOD' from dual union all
select 5587,'USAF' from dual union all
select 5588,'DOD' from dual union all
select 5588,'USAF' from dual union all
select 5589,'USAF' from dual union all
select 559,'USAF' from dual union all
select 5590,'RVSN' from dual union all
select 5591,'PVO' from dual union all
select 5592,'PVO' from dual union all
select 5593,'USAF' from dual union all
select 5594,'PVO' from dual union all
select 5595,'PVO' from dual union all
select 5596,'RVSN' from dual union all
select 5597,'PVO' from dual union all
select 5598,'NASA' from dual union all
select 5599,'USAF' from dual union all
select 56,'RVSN' from dual union all
select 560,'USAF' from dual union all
select 5600,'NASA' from dual union all
select 5601,'NASA' from dual union all
select 5602,'NASA' from dual union all
select 5603,'RVSN' from dual union all
select 5604,'RVSN' from dual union all
select 5605,'RVSN' from dual union all
select 5606,'NRO' from dual union all
select 5607,'RVSN' from dual union all
select 5608,'PVO' from dual union all
select 5609,'RVSN' from dual union all
select 561,'USAF' from dual union all
select 5610,'NASA' from dual union all
select 5611,'RVSN' from dual union all
select 5612,'RVSN' from dual union all
select 5613,'RVSN' from dual union all
select 5614,'OKB10' from dual union all
select 5615,'RVSN' from dual union all
select 5616,'RVSN' from dual union all
select 5617,'RVSN' from dual union all
select 5618,'RVSN' from dual union all
select 5619,'RVSN' from dual union all
select 562,'CIA' from dual union all
select 562,'NRO' from dual union all
select 5620,'RVSN' from dual union all
select 5621,'NASA' from dual union all
select 5622,'NASA' from dual union all
select 5623,'PVO' from dual union all
select 5624,'RVSN' from dual union all
select 5625,'PVO' from dual union all
select 5626,'RVSN' from dual union all
select 5627,'PVO' from dual union all
select 5628,'VS' from dual union all
select 5629,'RVSN' from dual union all
select 563,'RVSN' from dual union all
select 5630,'VS' from dual;
insert into SATELLITE_ORG
select 5631,'PVO' from dual union all
select 5632,'USAF' from dual union all
select 5633,'USAF' from dual union all
select 5634,'USAF' from dual union all
select 5635,'USAF' from dual union all
select 5636,'USAF' from dual union all
select 5637,'USAF' from dual union all
select 5638,'PVO' from dual union all
select 5639,'RVSN' from dual union all
select 564,'NASA' from dual union all
select 5640,'RVSN' from dual union all
select 5641,'IK' from dual union all
select 5642,'RVSN' from dual union all
select 5643,'YUZH' from dual union all
select 5644,'RVSN' from dual union all
select 5645,'RVSN' from dual union all
select 5646,'PVO' from dual union all
select 5647,'RVSN' from dual union all
select 5648,'RVSN' from dual union all
select 5649,'PVO' from dual union all
select 565,'NASA' from dual union all
select 5650,'RVSN' from dual union all
select 5651,'PVO' from dual union all
select 5652,'PVO' from dual union all
select 5653,'RVSN' from dual union all
select 5654,'PVO' from dual union all
select 5655,'PVO' from dual union all
select 5656,'PVO' from dual union all
select 5657,'RVSN' from dual union all
select 5658,'PVO' from dual union all
select 5659,'PVO' from dual union all
select 566,'OKB1' from dual union all
select 5660,'PVO' from dual union all
select 5661,'RVSN' from dual union all
select 5662,'RVSN' from dual union all
select 5663,'RVSN' from dual union all
select 5664,'PVO' from dual union all
select 5665,'PVO' from dual union all
select 5666,'PVO' from dual union all
select 5667,'IKI' from dual union all
select 5667,'NPOL' from dual union all
select 5668,'PVO' from dual union all
select 5669,'RVSN' from dual union all
select 567,'METP' from dual union all
select 5670,'RVSN' from dual union all
select 5671,'RVSN' from dual union all
select 5672,'PVO' from dual union all
select 5673,'PVO' from dual union all
select 5674,'RVSN' from dual union all
select 5675,'SRC' from dual union all
select 5676,'NASA' from dual union all
select 5677,'SRC' from dual union all
select 5678,'NRO' from dual union all
select 5678,'USN' from dual union all
select 5679,'USAF' from dual union all
select 568,'RVSN' from dual union all
select 5680,'NRO' from dual union all
select 5680,'USN' from dual union all
select 5681,'NRO' from dual union all
select 5681,'USN' from dual union all
select 5682,'NRO' from dual union all
select 5682,'USN' from dual union all
select 5683,'OKB10' from dual union all
select 5684,'RVSN' from dual union all
select 5685,'RVSN' from dual union all
select 5686,'RVSN' from dual union all
select 5687,'RVSN' from dual union all
select 5688,'RVSN' from dual union all
select 5689,'RVSN' from dual union all
select 569,'OKB1' from dual union all
select 5690,'NASA' from dual union all
select 5691,'NASA' from dual union all
select 5692,'NASA' from dual union all
select 5693,'NASA' from dual union all
select 5694,'NASA' from dual union all
select 5695,'NASA' from dual union all
select 5696,'NASA' from dual union all
select 5697,'NASA' from dual union all
select 5698,'NASA' from dual union all
select 5699,'NASA' from dual union all
select 57,'CIA' from dual union all
select 57,'USAF' from dual union all
select 570,'RVSN' from dual union all
select 5700,'NASA' from dual union all
select 5701,'NASA' from dual union all
select 5702,'USAF' from dual union all
select 5703,'USAF' from dual union all
select 5704,'PVO' from dual union all
select 5705,'OKB10' from dual union all
select 5706,'RVSN' from dual union all
select 5707,'RVSN' from dual union all
select 5708,'RVSN' from dual union all
select 5709,'INTEL' from dual union all
select 571,'OKB1' from dual union all
select 5710,'RVSN' from dual union all
select 5711,'USAF' from dual union all
select 5712,'RVSN' from dual union all
select 5713,'RVSN' from dual union all
select 5714,'PVO' from dual union all
select 5715,'USAF' from dual;
insert into SATELLITE_ORG
select 5716,'USAF' from dual union all
select 5717,'PVO' from dual union all
select 5718,'VS' from dual union all
select 5719,'USAF' from dual union all
select 572,'RVSN' from dual union all
select 5720,'PVO' from dual union all
select 5721,'VMF' from dual union all
select 5722,'RVSN' from dual union all
select 5723,'RVSN' from dual union all
select 5724,'RVSN' from dual union all
select 5725,'RVSN' from dual union all
select 5726,'RVSN' from dual union all
select 5727,'TSKBEM' from dual union all
select 5728,'RVSN' from dual union all
select 5729,'CNES' from dual union all
select 5729,'YUZH' from dual union all
select 573,'ATT' from dual union all
select 5730,'RVSN' from dual union all
select 5731,'GMS' from dual union all
select 5731,'RVSN' from dual union all
select 5732,'RVSN' from dual union all
select 5733,'RVSN' from dual union all
select 5734,'USAF' from dual union all
select 5735,'UNK' from dual union all
select 5736,'NASA' from dual union all
select 5737,'VMF' from dual union all
select 5738,'VMF' from dual union all
select 5739,'IKI' from dual union all
select 5739,'NPOL' from dual union all
select 574,'USAF' from dual union all
select 5740,'TSKBEM' from dual union all
select 5741,'TSKBEM' from dual union all
select 5742,'TSKBEM' from dual union all
select 5743,'NASA' from dual union all
select 5744,'USAF' from dual union all
select 5745,'USAF' from dual union all
select 5746,'USAF' from dual union all
select 5747,'USAF' from dual union all
select 5748,'USAF' from dual union all
select 5749,'PVO' from dual union all
select 575,'NASA' from dual union all
select 5750,'USAF' from dual union all
select 5751,'USAF' from dual union all
select 5752,'USAF' from dual union all
select 5753,'USAF' from dual union all
select 5754,'USAF' from dual union all
select 5755,'USAF' from dual union all
select 5756,'USAF' from dual union all
select 5757,'USAF' from dual union all
select 5758,'USAF' from dual union all
select 5759,'PVO' from dual union all
select 576,'NASA' from dual union all
select 5760,'PVO' from dual union all
select 5761,'PVO' from dual union all
select 5762,'PVO' from dual union all
select 5763,'PVO' from dual union all
select 5764,'RVSN' from dual union all
select 5765,'RVSN' from dual union all
select 5766,'UNK' from dual union all
select 5767,'RVSN' from dual union all
select 5768,'USAF' from dual union all
select 5769,'NRO' from dual union all
select 577,'NASA' from dual union all
select 5770,'USAF' from dual union all
select 5771,'NRO' from dual union all
select 5772,'NRO' from dual union all
select 5773,'RVSN' from dual union all
select 5774,'RVSN' from dual union all
select 5775,'INTEL' from dual union all
select 5776,'VS' from dual union all
select 5777,'VS' from dual union all
select 5778,'RVSN' from dual union all
select 5779,'VS' from dual union all
select 578,'CIA' from dual union all
select 578,'NRO' from dual union all
select 5780,'VS' from dual union all
select 5781,'VS' from dual union all
select 5782,'RVSN' from dual union all
select 5783,'VS' from dual union all
select 5784,'USAF' from dual union all
select 5785,'USAF' from dual union all
select 5786,'USAF' from dual union all
select 5787,'USAF' from dual union all
select 5788,'PVO' from dual union all
select 5789,'PVO' from dual union all
select 579,'USAF' from dual union all
select 5790,'PVO' from dual union all
select 5791,'USAF' from dual union all
select 5792,'USAF' from dual union all
select 5793,'USAF' from dual union all
select 5794,'USAF' from dual union all
select 5795,'PVO' from dual union all
select 5796,'PVO' from dual union all
select 5797,'PVO' from dual union all
select 5798,'PVO' from dual union all
select 5799,'PVO' from dual union all
select 58,'USA' from dual union all
select 580,'OKB586' from dual union all
select 5800,'PVO' from dual union all
select 5801,'PVO' from dual;
insert into SATELLITE_ORG
select 5802,'PVO' from dual union all
select 5803,'PVO' from dual union all
select 5804,'PVO' from dual union all
select 5805,'RVSN' from dual union all
select 5806,'RVSN' from dual union all
select 5807,'NASA' from dual union all
select 5808,'USAF' from dual union all
select 5809,'USAF' from dual union all
select 581,'OKB1' from dual union all
select 5810,'PVO' from dual union all
select 5811,'USAF' from dual union all
select 5812,'RVSN' from dual union all
select 5813,'RVSN' from dual union all
select 5814,'ESRO' from dual union all
select 5815,'NASA' from dual union all
select 5816,'NASA' from dual union all
select 5817,'NASA' from dual union all
select 5818,'NASA' from dual union all
select 5819,'NASA' from dual union all
select 582,'OKB586' from dual union all
select 5820,'NASA' from dual union all
select 5821,'RVSN' from dual union all
select 5822,'RVSN' from dual union all
select 5823,'USAF' from dual union all
select 5824,'NASA' from dual union all
select 5825,'NASA' from dual union all
select 5826,'USAF' from dual union all
select 5827,'PVO' from dual union all
select 5828,'PVO' from dual union all
select 5829,'PVO' from dual union all
select 583,'OKB586' from dual union all
select 5830,'PVO' from dual union all
select 5831,'PVO' from dual union all
select 5832,'PVO' from dual union all
select 5833,'PVO' from dual union all
select 5834,'PVO' from dual union all
select 5835,'IKI' from dual union all
select 5835,'NPOL' from dual union all
select 5836,'RVSN' from dual union all
select 5837,'RVSN' from dual union all
select 5838,'RVSN' from dual union all
select 5839,'RVSN' from dual union all
select 584,'OKB586' from dual union all
select 5840,'RVSN' from dual union all
select 5841,'IKI' from dual union all
select 5841,'NPOL' from dual union all
select 5842,'NASA' from dual union all
select 5843,'NASA' from dual union all
select 5844,'USAF' from dual union all
select 5845,'USAF' from dual union all
select 5846,'OKB10' from dual union all
select 5847,'RVSN' from dual union all
select 5848,'RVSN' from dual union all
select 5849,'RVSN' from dual union all
select 585,'OKB586' from dual union all
select 5850,'RVSN' from dual union all
select 5851,'USAF' from dual union all
select 5852,'TSN108' from dual union all
select 5853,'RVSN' from dual union all
select 5854,'USAF' from dual union all
select 5855,'USAF' from dual union all
select 5856,'USAF' from dual union all
select 5857,'USAF' from dual union all
select 5858,'RVSN' from dual union all
select 5859,'USAF' from dual union all
select 586,'OKB1' from dual union all
select 5860,'NASA' from dual union all
select 5861,'NASA' from dual union all
select 5862,'RVSN' from dual union all
select 5863,'RVSN' from dual union all
select 5864,'RVSN' from dual union all
select 5865,'RVSN' from dual union all
select 5866,'PVO' from dual union all
select 5867,'PVO' from dual union all
select 5868,'USAF' from dual union all
select 5869,'USAF' from dual union all
select 587,'RVSN' from dual union all
select 5870,'PVO' from dual union all
select 5871,'PVO' from dual union all
select 5872,'PVO' from dual union all
select 5873,'PVO' from dual union all
select 5874,'PVO' from dual union all
select 5875,'PVO' from dual union all
select 5876,'USAF' from dual union all
select 5877,'USAF' from dual union all
select 5878,'NASA' from dual union all
select 5879,'ESRO' from dual union all
select 588,'RVSN' from dual union all
select 5880,'NASA' from dual union all
select 5881,'ESRO' from dual union all
select 5882,'TSKBEM' from dual union all
select 5883,'RVSN' from dual union all
select 5884,'ESRO' from dual union all
select 5885,'RVSN' from dual union all
select 5886,'RVSN' from dual union all
select 5887,'RVSN' from dual union all
select 5888,'NRO' from dual union all
select 5889,'PVO' from dual union all
select 589,'USAF' from dual union all
select 5890,'PVO' from dual;
insert into SATELLITE_ORG
select 5891,'PVO' from dual union all
select 5892,'OKB1' from dual union all
select 5893,'OKB1' from dual union all
select 5894,'VS' from dual union all
select 5895,'RVSN' from dual union all
select 5896,'PVO' from dual union all
select 5897,'PVO' from dual union all
select 5898,'PVO' from dual union all
select 5899,'USAF' from dual union all
select 59,'USAF' from dual union all
select 590,'CIA' from dual union all
select 590,'NRO' from dual union all
select 5900,'USAF' from dual union all
select 5901,'VS' from dual union all
select 5902,'VS' from dual union all
select 5903,'USAF' from dual union all
select 5904,'USAF' from dual union all
select 5905,'OKB10' from dual union all
select 5906,'PVO' from dual union all
select 5907,'RVSN' from dual union all
select 5908,'RVSN' from dual union all
select 5909,'RVSN' from dual union all
select 591,'OKB1' from dual union all
select 5910,'RVSN' from dual union all
select 5911,'VS' from dual union all
select 5912,'IKI' from dual union all
select 5912,'NPOL' from dual union all
select 5913,'RVSN' from dual union all
select 5914,'RVSN' from dual union all
select 5915,'RVSN' from dual union all
select 5916,'USAF' from dual union all
select 5917,'GMS' from dual union all
select 5917,'RVSN' from dual union all
select 5918,'RVSN' from dual union all
select 5919,'IKI' from dual union all
select 5919,'NPOL' from dual union all
select 592,'RVSN' from dual union all
select 5920,'RVSN' from dual union all
select 5921,'RVSN' from dual union all
select 5922,'NASA' from dual union all
select 5923,'RVSN' from dual union all
select 5924,'RVSN' from dual union all
select 5925,'RVSN' from dual union all
select 5926,'RVSN' from dual union all
select 5927,'RVSN' from dual union all
select 5928,'CNES' from dual union all
select 5929,'RVSN' from dual union all
select 593,'NRO' from dual union all
select 5930,'RVSN' from dual union all
select 5931,'RVSN' from dual union all
select 5932,'ESRO' from dual union all
select 5933,'RVSN' from dual union all
select 5934,'RVSN' from dual union all
select 5935,'RVSN' from dual union all
select 5936,'TSKBEM' from dual union all
select 5937,'RVSN' from dual union all
select 5938,'PVO' from dual union all
select 5939,'RVSN' from dual union all
select 594,'USN' from dual union all
select 5940,'RVSN' from dual union all
select 5941,'IKI' from dual union all
select 5941,'NPOL' from dual union all
select 5942,'RVSN' from dual union all
select 5943,'RVSN' from dual union all
select 5944,'RVSN' from dual union all
select 5945,'RVSN' from dual union all
select 5946,'RVSN' from dual union all
select 5947,'RVSN' from dual union all
select 5948,'RVSN' from dual union all
select 5949,'RVSN' from dual union all
select 595,'OKB1' from dual union all
select 5950,'RVSN' from dual union all
select 5951,'UNK' from dual union all
select 5952,'CNES' from dual union all
select 5953,'USAF' from dual union all
select 5954,'USAF' from dual union all
select 5955,'USAF' from dual union all
select 5956,'RVSN' from dual union all
select 5957,'NASA' from dual union all
select 5958,'NASA' from dual union all
select 5959,'CNES' from dual union all
select 596,'RVSN' from dual union all
select 5960,'UNK' from dual union all
select 5961,'UNK' from dual union all
select 5962,'CNES' from dual union all
select 5963,'RVSN' from dual union all
select 5964,'USAF' from dual union all
select 5965,'USAF' from dual union all
select 5966,'PVO' from dual union all
select 5967,'NASA' from dual union all
select 5968,'USAF' from dual union all
select 5969,'UNK' from dual union all
select 597,'USN' from dual union all
select 5970,'USAF' from dual union all
select 5971,'NASA' from dual union all
select 5972,'USAF' from dual union all
select 5973,'NASA' from dual union all
select 5974,'NASA' from dual union all
select 5975,'NASA' from dual union all
select 5976,'NASA' from dual;
insert into SATELLITE_ORG
select 5977,'NASA' from dual union all
select 5978,'NASA' from dual union all
select 5979,'SRC' from dual union all
select 598,'NRO' from dual union all
select 598,'USN' from dual union all
select 5980,'USAF' from dual union all
select 5981,'USAF' from dual union all
select 5982,'USAF' from dual union all
select 5983,'RVSN' from dual union all
select 5984,'PVO' from dual union all
select 5985,'USAF' from dual union all
select 5986,'USAF' from dual union all
select 5987,'UNK' from dual union all
select 5988,'UNK' from dual union all
select 5989,'UNK' from dual union all
select 599,'NRO' from dual union all
select 599,'USN' from dual union all
select 5990,'UNK' from dual union all
select 5991,'USAF' from dual union all
select 5992,'USAF' from dual union all
select 5993,'USAF' from dual union all
select 5994,'USAF' from dual union all
select 5995,'USAF' from dual union all
select 5996,'USAF' from dual union all
select 5997,'USAF' from dual union all
select 5998,'USAF' from dual union all
select 5999,'USAF' from dual union all
select 6,'JPL' from dual union all
select 6,'USA' from dual union all
select 60,'NASA' from dual union all
select 600,'NRO' from dual union all
select 600,'USN' from dual union all
select 6000,'NASA' from dual union all
select 6001,'NASA' from dual union all
select 6002,'TSKBEM' from dual union all
select 6003,'CIA' from dual union all
select 6003,'NRO' from dual union all
select 6004,'NASA' from dual union all
select 6005,'NASA' from dual union all
select 6006,'PVO' from dual union all
select 6007,'RVSN' from dual union all
select 6008,'RVSN' from dual union all
select 6009,'NASA' from dual union all
select 601,'USN' from dual union all
select 6010,'RVSN' from dual union all
select 6011,'RVSN' from dual union all
select 6012,'RVSN' from dual union all
select 6013,'RVSN' from dual union all
select 6014,'NASA' from dual union all
select 6015,'NASA' from dual union all
select 6016,'RVSN' from dual union all
select 6017,'RVSN' from dual union all
select 6018,'RVSN' from dual union all
select 6019,'OKB10' from dual union all
select 602,'USAF' from dual union all
select 6020,'RVSN' from dual union all
select 6021,'RVSN' from dual union all
select 6022,'RVSN' from dual union all
select 6023,'RVSN' from dual union all
select 6024,'RVSN' from dual union all
select 6025,'RVSN' from dual union all
select 6026,'RVSN' from dual union all
select 6027,'RVSN' from dual union all
select 6028,'RVSN' from dual union all
select 6029,'RVSN' from dual union all
select 603,'NASA' from dual union all
select 6030,'RVSN' from dual union all
select 6031,'RVSN' from dual union all
select 6032,'RVSN' from dual union all
select 6033,'RVSN' from dual union all
select 6034,'RVSN' from dual union all
select 6035,'RVSN' from dual union all
select 6036,'RVSN' from dual union all
select 6037,'CIA' from dual union all
select 6037,'NRO' from dual union all
select 6038,'RVSN' from dual union all
select 6039,'RVSN' from dual union all
select 604,'NASA' from dual union all
select 6040,'TSKBEM' from dual union all
select 6041,'USAF' from dual union all
select 6042,'USAF' from dual union all
select 6043,'PVO' from dual union all
select 6044,'USAF' from dual union all
select 6045,'PVO' from dual union all
select 6046,'RVSN' from dual union all
select 6047,'RVSN' from dual union all
select 6048,'RVSN' from dual union all
select 6049,'RVSN' from dual union all
select 605,'NASA' from dual union all
select 6050,'RVSN' from dual union all
select 6051,'NASA' from dual union all
select 6052,'INTEL' from dual union all
select 6053,'RVSN' from dual union all
select 6054,'RVSN' from dual union all
select 6055,'RVSN' from dual union all
select 6056,'RVSN' from dual union all
select 6057,'RVSN' from dual union all
select 6058,'NASA' from dual union all
select 6059,'OKB10' from dual union all
select 606,'NASA' from dual;
insert into SATELLITE_ORG
select 6060,'RVSN' from dual union all
select 6061,'RVSN' from dual union all
select 6062,'RVSN' from dual union all
select 6063,'RVSN' from dual union all
select 6064,'RVSN' from dual union all
select 6065,'RVSN' from dual union all
select 6066,'TSKBEM' from dual union all
select 6067,'RVSN' from dual union all
select 6068,'IKI' from dual union all
select 6068,'NPOL' from dual union all
select 6069,'RVSN' from dual union all
select 607,'NASA' from dual union all
select 6070,'RVSN' from dual union all
select 6071,'RVSN' from dual union all
select 6072,'RVSN' from dual union all
select 6073,'IKI' from dual union all
select 6073,'NPOL' from dual union all
select 6074,'RVSN' from dual union all
select 6075,'IK' from dual union all
select 6076,'PVO' from dual union all
select 6077,'RVSN' from dual union all
select 6078,'RVSN' from dual union all
select 6079,'GMS' from dual union all
select 6079,'RVSN' from dual union all
select 608,'USAF' from dual union all
select 6080,'RVSN' from dual union all
select 6081,'RVSN' from dual union all
select 6082,'RVSN' from dual union all
select 6083,'TSKBEM' from dual union all
select 6084,'NASA' from dual union all
select 6085,'PVO' from dual union all
select 6086,'PVO' from dual union all
select 6087,'RVSN' from dual union all
select 6088,'RVSN' from dual union all
select 6089,'RVSN' from dual union all
select 609,'CIA' from dual union all
select 609,'NRO' from dual union all
select 6090,'RVSN' from dual union all
select 6091,'RVSN' from dual union all
select 6092,'RVSN' from dual union all
select 6093,'RVSN' from dual union all
select 6094,'NRO' from dual union all
select 6095,'USAF' from dual union all
select 6096,'NRO' from dual union all
select 6097,'VS' from dual union all
select 6098,'RVSN' from dual union all
select 6099,'PVO' from dual union all
select 61,'CIA' from dual union all
select 61,'USAF' from dual union all
select 610,'USN' from dual union all
select 6100,'RVSN' from dual union all
select 6101,'VS' from dual union all
select 6102,'VS' from dual union all
select 6103,'RVSN' from dual union all
select 6104,'RVSN' from dual union all
select 6105,'TSKBEM' from dual union all
select 6106,'RVSN' from dual union all
select 6107,'RVSN' from dual union all
select 6108,'RVSN' from dual union all
select 6109,'RVSN' from dual union all
select 611,'USN' from dual union all
select 6110,'RVSN' from dual union all
select 6111,'TSKBEM' from dual union all
select 6112,'RVSN' from dual union all
select 6113,'RVSN' from dual union all
select 6114,'RVSN' from dual union all
select 6115,'RVSN' from dual union all
select 6116,'RVSN' from dual union all
select 6117,'OKB10' from dual union all
select 6118,'OKB10' from dual union all
select 6119,'OKB10' from dual union all
select 612,'USAF' from dual union all
select 6120,'OKB10' from dual union all
select 6121,'OKB10' from dual union all
select 6122,'OKB10' from dual union all
select 6123,'OKB10' from dual union all
select 6124,'OKB10' from dual union all
select 6125,'RVSN' from dual union all
select 6126,'NASA' from dual union all
select 6127,'NASA' from dual union all
select 6128,'RVSN' from dual union all
select 6129,'RVSN' from dual union all
select 613,'NRO' from dual union all
select 6130,'RVSN' from dual union all
select 6131,'RVSN' from dual union all
select 6132,'RVSN' from dual union all
select 6133,'RVSN' from dual union all
select 6134,'RVSN' from dual union all
select 6135,'RVSN' from dual union all
select 6136,'RVSN' from dual union all
select 6137,'RVSN' from dual union all
select 6138,'PVO' from dual union all
select 6139,'NASA' from dual union all
select 614,'NRO' from dual union all
select 6140,'NASA' from dual union all
select 6141,'USAF' from dual union all
select 6142,'NASA' from dual union all
select 6143,'RVSN' from dual union all
select 6144,'RVSN' from dual union all
select 6145,'NASA' from dual;
insert into SATELLITE_ORG
select 6146,'NASA' from dual union all
select 6147,'RVSN' from dual union all
select 6148,'OKB10' from dual union all
select 6149,'RVSN' from dual union all
select 615,'USAF' from dual union all
select 6150,'RVSN' from dual union all
select 6151,'RVSN' from dual union all
select 6152,'ISAS' from dual union all
select 6153,'NASA' from dual union all
select 6154,'VMF' from dual union all
select 6155,'NASA' from dual union all
select 6156,'NASA' from dual union all
select 6157,'NASA' from dual union all
select 6158,'NASA' from dual union all
select 6159,'NASA' from dual union all
select 616,'USAF' from dual union all
select 6160,'NASA' from dual union all
select 6161,'NASA' from dual union all
select 6162,'RVSN' from dual union all
select 6163,'RVSN' from dual union all
select 6164,'VS' from dual union all
select 6165,'RVSN' from dual union all
select 6166,'RVSN' from dual union all
select 6167,'NASA' from dual union all
select 6168,'RVSN' from dual union all
select 6169,'RVSN' from dual union all
select 617,'USAF' from dual union all
select 6170,'RVSN' from dual union all
select 6171,'RVSN' from dual union all
select 6172,'NRO' from dual union all
select 6173,'USN' from dual union all
select 6174,'VS' from dual union all
select 6175,'PVO' from dual union all
select 6176,'PVO' from dual union all
select 6177,'PVO' from dual union all
select 6178,'USAF' from dual union all
select 6179,'USAF' from dual union all
select 618,'NRO' from dual union all
select 6180,'NASA' from dual union all
select 6181,'USAF' from dual union all
select 6182,'USN' from dual union all
select 6183,'USAF' from dual union all
select 6184,'NASA' from dual union all
select 6185,'NASA' from dual union all
select 6186,'RVSN' from dual union all
select 6187,'RVSN' from dual union all
select 6188,'RVSN' from dual union all
select 6189,'RVSN' from dual union all
select 619,'USAF' from dual union all
select 6190,'RVSN' from dual union all
select 6191,'RVSN' from dual union all
select 6192,'PVO' from dual union all
select 6193,'RVSN' from dual union all
select 6194,'RVSN' from dual union all
select 6195,'RVSN' from dual union all
select 6196,'VMF' from dual union all
select 6197,'NASA' from dual union all
select 6198,'TSKBEM' from dual union all
select 6199,'VMF' from dual union all
select 62,'NASA' from dual union all
select 620,'USAF' from dual union all
select 6200,'RVSN' from dual union all
select 6201,'RVSN' from dual union all
select 6202,'NASA' from dual union all
select 6203,'NASA' from dual union all
select 6204,'NASA' from dual union all
select 6205,'NASA' from dual union all
select 6206,'PVO' from dual union all
select 6207,'RVSN' from dual union all
select 6208,'RVSN' from dual union all
select 6209,'RVSN' from dual union all
select 621,'CIA' from dual union all
select 621,'NRO' from dual union all
select 6210,'RVSN' from dual union all
select 6211,'RVSN' from dual union all
select 6212,'USA' from dual union all
select 6213,'NASA' from dual union all
select 6214,'NASA' from dual union all
select 6215,'USAF' from dual union all
select 6216,'NASA' from dual union all
select 6217,'USAF' from dual union all
select 6218,'USAF' from dual union all
select 6219,'RVSN' from dual union all
select 622,'USAF' from dual union all
select 6220,'RVSN' from dual union all
select 6221,'USAF' from dual union all
select 6222,'PVO' from dual union all
select 6223,'RVSN' from dual union all
select 6224,'USAF' from dual union all
select 6225,'RVSN' from dual union all
select 6226,'RVSN' from dual union all
select 6227,'NRO' from dual union all
select 6228,'USAF' from dual union all
select 6229,'PVO' from dual union all
select 623,'USAF' from dual union all
select 6230,'RVSN' from dual union all
select 6231,'RVSN' from dual union all
select 6232,'RVSN' from dual union all
select 6233,'RVSN' from dual union all
select 6234,'RVSN' from dual;
insert into SATELLITE_ORG
select 6235,'NOAA' from dual union all
select 6236,'AMNA' from dual union all
select 6237,'NASA' from dual union all
select 6238,'RVSN' from dual union all
select 6239,'RVSN' from dual union all
select 624,'USAF' from dual union all
select 6240,'RVSN' from dual union all
select 6241,'RVSN' from dual union all
select 6242,'RVSN' from dual union all
select 6243,'RVSN' from dual union all
select 6244,'RVSN' from dual union all
select 6245,'RVSN' from dual union all
select 6246,'RVSN' from dual union all
select 6247,'RVSN' from dual union all
select 6248,'RVSN' from dual union all
select 6249,'RVSN' from dual union all
select 625,'NASA' from dual union all
select 6250,'USN' from dual union all
select 6251,'NASA' from dual union all
select 6252,'RVSN' from dual union all
select 6253,'USAF' from dual union all
select 6254,'PVO' from dual union all
select 6255,'RVSN' from dual union all
select 6256,'GMS' from dual union all
select 6256,'RVSN' from dual union all
select 6257,'RVSN' from dual union all
select 6258,'TSKBEM' from dual union all
select 6259,'RVSN' from dual union all
select 626,'CIA' from dual union all
select 626,'NRO' from dual union all
select 6260,'RVSN' from dual union all
select 6261,'RVSN' from dual union all
select 6262,'OKB10' from dual union all
select 6263,'RVSN' from dual union all
select 6264,'OKB10' from dual union all
select 6265,'OKB10' from dual union all
select 6266,'OKB10' from dual union all
select 6267,'OKB10' from dual union all
select 6268,'OKB10' from dual union all
select 6269,'OKB10' from dual union all
select 627,'USAF' from dual union all
select 6270,'OKB10' from dual union all
select 6271,'RVSN' from dual union all
select 6272,'VS' from dual union all
select 6273,'RVSN' from dual union all
select 6274,'VS' from dual union all
select 6275,'USAF' from dual union all
select 6276,'USAF' from dual union all
select 6277,'RVSN' from dual union all
select 6278,'TCAN' from dual union all
select 6279,'NASA' from dual union all
select 628,'USAF' from dual union all
select 6280,'RVSN' from dual union all
select 6281,'RVSN' from dual union all
select 6282,'NASA' from dual union all
select 6283,'USN' from dual union all
select 6284,'USN' from dual union all
select 6285,'ESRO' from dual union all
select 6286,'NASA' from dual union all
select 6287,'RVSN' from dual union all
select 6288,'RVSN' from dual union all
select 6289,'RVSN' from dual union all
select 629,'USAF' from dual union all
select 6290,'USN' from dual union all
select 6291,'IK' from dual union all
select 6292,'RVSN' from dual union all
select 6293,'RVSN' from dual union all
select 6294,'RVSN' from dual union all
select 6295,'RVSN' from dual union all
select 6296,'RVSN' from dual union all
select 6297,'RVSN' from dual union all
select 6298,'RVSN' from dual union all
select 6299,'PVO' from dual union all
select 63,'NASA' from dual union all
select 630,'USAF' from dual union all
select 6300,'NASA' from dual union all
select 6301,'NASA' from dual union all
select 6302,'RVSN' from dual union all
select 6303,'RVSN' from dual union all
select 6304,'RVSN' from dual union all
select 6305,'NASA' from dual union all
select 6306,'NASA' from dual union all
select 6307,'NASA' from dual union all
select 6308,'RVSN' from dual union all
select 6309,'RVSN' from dual union all
select 631,'USAF' from dual union all
select 6310,'RVSN' from dual union all
select 6311,'RVSN' from dual union all
select 6312,'RVSN' from dual union all
select 6313,'RVSN' from dual union all
select 6314,'RVSN' from dual union all
select 6315,'DFVLR' from dual union all
select 6316,'NASA' from dual union all
select 6317,'NRO' from dual union all
select 6318,'USAF' from dual union all
select 6319,'OKB10' from dual union all
select 632,'OKB586' from dual union all
select 632,'PVO' from dual union all
select 6320,'RVSN' from dual union all
select 6321,'NRO' from dual;
insert into SATELLITE_ORG
select 6322,'RVSN' from dual union all
select 6323,'OKB10' from dual union all
select 6324,'RVSN' from dual union all
select 6325,'RVSN' from dual union all
select 6326,'TSKBEM' from dual union all
select 6327,'RVSN' from dual union all
select 6328,'TSN108' from dual union all
select 6329,'RVSN' from dual union all
select 633,'RVSN' from dual union all
select 6330,'RVSN' from dual union all
select 6331,'RVSN' from dual union all
select 6332,'ISAS' from dual union all
select 6333,'IKI' from dual union all
select 6333,'NPOL' from dual union all
select 6334,'RVSN' from dual union all
select 6335,'RVSN' from dual union all
select 6336,'RVSN' from dual union all
select 6337,'RVSN' from dual union all
select 6338,'RVSN' from dual union all
select 6339,'RVSN' from dual union all
select 634,'NASA' from dual union all
select 6340,'RVSN' from dual union all
select 6341,'RVSN' from dual union all
select 6342,'RVSN' from dual union all
select 6343,'VS' from dual union all
select 6344,'RVSN' from dual union all
select 6345,'VS' from dual union all
select 6346,'RVSN' from dual union all
select 6347,'RVSN' from dual union all
select 6348,'PVO' from dual union all
select 6349,'RVSN' from dual union all
select 635,'USAF' from dual union all
select 6350,'OKB10' from dual union all
select 6351,'RVSN' from dual union all
select 6352,'VS' from dual union all
select 6353,'RVSN' from dual union all
select 6354,'RVSN' from dual union all
select 6355,'RVSN' from dual union all
select 6356,'RVSN' from dual union all
select 6357,'RVSN' from dual union all
select 6358,'RVSN' from dual union all
select 6359,'RVSN' from dual union all
select 636,'CIA' from dual union all
select 636,'NRO' from dual union all
select 6360,'RVSN' from dual union all
select 6361,'RVSN' from dual union all
select 6362,'VS' from dual union all
select 6363,'RVSN' from dual union all
select 6364,'IKI' from dual union all
select 6364,'NPOL' from dual union all
select 6365,'RVSN' from dual union all
select 6366,'RVSN' from dual union all
select 6367,'RVSN' from dual union all
select 6368,'RVSN' from dual union all
select 6369,'RVSN' from dual union all
select 637,'NRO' from dual union all
select 637,'USA' from dual union all
select 6370,'RVSN' from dual union all
select 6371,'RVSN' from dual union all
select 6372,'USN' from dual union all
select 6373,'VS' from dual union all
select 6374,'RVSN' from dual union all
select 6375,'VS' from dual union all
select 6376,'RVSN' from dual union all
select 6377,'RVSN' from dual union all
select 6378,'RVSN' from dual union all
select 6379,'RVSN' from dual union all
select 638,'USAF' from dual union all
select 6380,'CIA' from dual union all
select 6380,'NRO' from dual union all
select 6381,'RVSN' from dual union all
select 6382,'NRO' from dual union all
select 6383,'USAF' from dual union all
select 6384,'RVSN' from dual union all
select 6385,'RVSN' from dual union all
select 6386,'RVSN' from dual union all
select 6387,'RVSN' from dual union all
select 6388,'USAF' from dual union all
select 6389,'RVSN' from dual union all
select 639,'USAF' from dual union all
select 6390,'RVSN' from dual union all
select 6391,'RVSN' from dual union all
select 6392,'GMS' from dual union all
select 6392,'RVSN' from dual union all
select 6393,'RVSN' from dual union all
select 6394,'RVSN' from dual union all
select 6395,'RVSN' from dual union all
select 6396,'RVSN' from dual union all
select 6397,'TSKBEM' from dual union all
select 6398,'TSKBM' from dual union all
select 6399,'RVSN' from dual union all
select 64,'NASA' from dual union all
select 640,'USAF' from dual union all
select 6400,'RVSN' from dual union all
select 6401,'RVSN' from dual union all
select 6402,'RVSN' from dual union all
select 6403,'RVSN' from dual union all
select 6404,'RVSN' from dual union all
select 6405,'RVSN' from dual union all
select 6406,'RVSN' from dual;
insert into SATELLITE_ORG
select 6407,'RVSN' from dual union all
select 6408,'RVSN' from dual union all
select 6409,'RVSN' from dual union all
select 641,'NRO' from dual union all
select 6410,'RVSN' from dual union all
select 6411,'RVSN' from dual union all
select 6412,'RVSN' from dual union all
select 6413,'RVSN' from dual union all
select 6414,'RVSN' from dual union all
select 6415,'RVSN' from dual union all
select 6416,'RVSN' from dual union all
select 6417,'RVSN' from dual union all
select 6418,'RVSN' from dual union all
select 6419,'RVSN' from dual union all
select 642,'CIA' from dual union all
select 642,'NRO' from dual union all
select 6420,'RVSN' from dual union all
select 6421,'NASA' from dual union all
select 6422,'RVSN' from dual union all
select 6423,'RVSN' from dual union all
select 6424,'RVSN' from dual union all
select 6425,'NASA' from dual union all
select 6426,'NASA' from dual union all
select 6427,'PVO' from dual union all
select 6428,'RVSN' from dual union all
select 6429,'RVSN' from dual union all
select 643,'USAF' from dual union all
select 6430,'RVSN' from dual union all
select 6431,'RVSN' from dual union all
select 6432,'RVSN' from dual union all
select 6433,'IK' from dual union all
select 6434,'RVSN' from dual union all
select 6435,'RVSN' from dual union all
select 6436,'RVSN' from dual union all
select 6437,'TCAN' from dual union all
select 6438,'NASA' from dual union all
select 6439,'RVSN' from dual union all
select 644,'USAF' from dual union all
select 6440,'RVSN' from dual union all
select 6441,'RVSN' from dual union all
select 6442,'RVSN' from dual union all
select 6443,'RVSN' from dual union all
select 6444,'RVSN' from dual union all
select 6445,'TSKBEM' from dual union all
select 6446,'RVSN' from dual union all
select 6447,'RVSN' from dual union all
select 6448,'RVSN' from dual union all
select 6449,'RVSN' from dual union all
select 645,'USAF' from dual union all
select 6450,'RVSN' from dual union all
select 6451,'RVSN' from dual union all
select 6452,'RVSN' from dual union all
select 6453,'RVSN' from dual union all
select 6454,'RVSN' from dual union all
select 6455,'RVSN' from dual union all
select 6456,'RVSN' from dual union all
select 6457,'RVSN' from dual union all
select 6458,'RVSN' from dual union all
select 6459,'RVSN' from dual union all
select 646,'USAF' from dual union all
select 6460,'RVSN' from dual union all
select 6461,'RVSN' from dual union all
select 6462,'RVSN' from dual union all
select 6463,'RVSN' from dual union all
select 6464,'RVSN' from dual union all
select 6465,'RVSN' from dual union all
select 6466,'RVSN' from dual union all
select 6467,'RVSN' from dual union all
select 6468,'RVSN' from dual union all
select 6469,'RVSN' from dual union all
select 647,'USAF' from dual union all
select 6470,'RVSN' from dual union all
select 6471,'RVSN' from dual union all
select 6472,'RVSN' from dual union all
select 6473,'RVSN' from dual union all
select 6474,'RVSN' from dual union all
select 6475,'RVSN' from dual union all
select 6476,'RVSN' from dual union all
select 6477,'RVSN' from dual union all
select 6478,'RVSN' from dual union all
select 6479,'RVSN' from dual union all
select 648,'USAF' from dual union all
select 6480,'RVSN' from dual union all
select 6481,'RVSN' from dual union all
select 6482,'RVSN' from dual union all
select 6483,'RVSN' from dual union all
select 6484,'RVSN' from dual union all
select 6485,'RVSN' from dual union all
select 6486,'RVSN' from dual union all
select 6487,'RVSN' from dual union all
select 6488,'RVSN' from dual union all
select 6489,'RVSN' from dual union all
select 649,'USAF' from dual union all
select 6490,'RVSN' from dual union all
select 6491,'RVSN' from dual union all
select 6492,'RVSN' from dual union all
select 6493,'RVSN' from dual union all
select 6494,'RVSN' from dual union all
select 6495,'RVSN' from dual union all
select 6496,'RVSN' from dual;
insert into SATELLITE_ORG
select 6497,'RVSN' from dual union all
select 6498,'TSKBEM' from dual union all
select 6499,'RVSN' from dual union all
select 65,'OKB1' from dual union all
select 650,'USAF' from dual union all
select 6500,'RVSN' from dual union all
select 6501,'RVSN' from dual union all
select 6502,'RVSN' from dual union all
select 6503,'RVSN' from dual union all
select 6504,'RVSN' from dual union all
select 6505,'RVSN' from dual union all
select 6506,'RVSN' from dual union all
select 6507,'RVSN' from dual union all
select 6508,'RVSN' from dual union all
select 6509,'RVSN' from dual union all
select 651,'USAF' from dual union all
select 6510,'RVSN' from dual union all
select 6511,'RVSN' from dual union all
select 6512,'RVSN' from dual union all
select 6513,'RVSN' from dual union all
select 6514,'RVSN' from dual union all
select 6515,'RVSN' from dual union all
select 6516,'RVSN' from dual union all
select 6517,'RVSN' from dual union all
select 6518,'RVSN' from dual union all
select 6519,'RVSN' from dual union all
select 652,'USAF' from dual union all
select 6520,'RVSN' from dual union all
select 6521,'RVSN' from dual union all
select 6522,'RVSN' from dual union all
select 6523,'RVSN' from dual union all
select 6524,'RVSN' from dual union all
select 6525,'RVSN' from dual union all
select 6526,'RVSN' from dual union all
select 6527,'RVSN' from dual union all
select 6528,'RVSN' from dual union all
select 6529,'RVSN' from dual union all
select 653,'USAF' from dual union all
select 6530,'RVSN' from dual union all
select 6531,'RVSN' from dual union all
select 6532,'RVSN' from dual union all
select 6533,'RVSN' from dual union all
select 6534,'RVSN' from dual union all
select 6535,'RVSN' from dual union all
select 6536,'RVSN' from dual union all
select 6537,'RVSN' from dual union all
select 6538,'RVSN' from dual union all
select 6539,'RVSN' from dual union all
select 654,'USAF' from dual union all
select 6540,'RVSN' from dual union all
select 6541,'RVSN' from dual union all
select 6542,'RVSN' from dual union all
select 6543,'RVSN' from dual union all
select 6544,'RVSN' from dual union all
select 6545,'RVSN' from dual union all
select 6546,'RVSN' from dual union all
select 6547,'RVSN' from dual union all
select 6548,'RVSN' from dual union all
select 6549,'RVSN' from dual union all
select 655,'USAF' from dual union all
select 6550,'RVSN' from dual union all
select 6551,'RVSN' from dual union all
select 6552,'RVSN' from dual union all
select 6553,'RVSN' from dual union all
select 6554,'RVSN' from dual union all
select 6555,'RVSN' from dual union all
select 6556,'RVSN' from dual union all
select 6557,'RVSN' from dual union all
select 6558,'RVSN' from dual union all
select 6559,'RVSN' from dual union all
select 656,'USAF' from dual union all
select 6560,'RVSN' from dual union all
select 6561,'RVSN' from dual union all
select 6562,'RVSN' from dual union all
select 6563,'RVSN' from dual union all
select 6564,'RVSN' from dual union all
select 6565,'RVSN' from dual union all
select 6566,'RVSN' from dual union all
select 6567,'RVSN' from dual union all
select 6568,'RVSN' from dual union all
select 6569,'RVSN' from dual union all
select 657,'USAF' from dual union all
select 6570,'RVSN' from dual union all
select 6571,'RVSN' from dual union all
select 6572,'RVSN' from dual union all
select 6573,'RVSN' from dual union all
select 6574,'RVSN' from dual union all
select 6575,'RVSN' from dual union all
select 6576,'RVSN' from dual union all
select 6577,'RVSN' from dual union all
select 6578,'RVSN' from dual union all
select 6579,'RVSN' from dual union all
select 658,'USAF' from dual union all
select 6580,'RVSN' from dual union all
select 6581,'RVSN' from dual union all
select 6582,'RVSN' from dual union all
select 6583,'RVSN' from dual union all
select 6584,'RVSN' from dual union all
select 6585,'RVSN' from dual union all
select 6586,'RVSN' from dual;
insert into SATELLITE_ORG
select 6587,'RVSN' from dual union all
select 6588,'RVSN' from dual union all
select 6589,'RVSN' from dual union all
select 659,'USAF' from dual union all
select 6590,'RVSN' from dual union all
select 6591,'RVSN' from dual union all
select 6592,'RVSN' from dual union all
select 6593,'RVSN' from dual union all
select 6594,'RVSN' from dual union all
select 6595,'RVSN' from dual union all
select 6596,'RVSN' from dual union all
select 6597,'RVSN' from dual union all
select 6598,'RVSN' from dual union all
select 6599,'RVSN' from dual union all
select 66,'RVSN' from dual union all
select 660,'USAF' from dual union all
select 6600,'RVSN' from dual union all
select 6601,'RVSN' from dual union all
select 6602,'RVSN' from dual union all
select 6603,'RVSN' from dual union all
select 6604,'RVSN' from dual union all
select 6605,'RVSN' from dual union all
select 6606,'RVSN' from dual union all
select 6607,'RVSN' from dual union all
select 6608,'RVSN' from dual union all
select 6609,'RVSN' from dual union all
select 661,'USAF' from dual union all
select 6610,'RVSN' from dual union all
select 6611,'RVSN' from dual union all
select 6612,'RVSN' from dual union all
select 6613,'RVSN' from dual union all
select 6614,'RVSN' from dual union all
select 6615,'RVSN' from dual union all
select 6616,'RVSN' from dual union all
select 6617,'RVSN' from dual union all
select 6618,'RVSN' from dual union all
select 6619,'RVSN' from dual union all
select 662,'USAF' from dual union all
select 6620,'RVSN' from dual union all
select 6621,'RVSN' from dual union all
select 6622,'RVSN' from dual union all
select 6623,'RVSN' from dual union all
select 6624,'RVSN' from dual union all
select 6625,'RVSN' from dual union all
select 6626,'RVSN' from dual union all
select 6627,'RVSN' from dual union all
select 6628,'RVSN' from dual union all
select 6629,'RVSN' from dual union all
select 663,'USAF' from dual union all
select 6630,'RVSN' from dual union all
select 6631,'RVSN' from dual union all
select 6632,'RVSN' from dual union all
select 6633,'NASA' from dual union all
select 6634,'NASA' from dual union all
select 6635,'NASA' from dual union all
select 6636,'NASA' from dual union all
select 6637,'NASA' from dual union all
select 6638,'NASA' from dual union all
select 6639,'NASA' from dual union all
select 664,'USAF' from dual union all
select 6640,'NRO' from dual union all
select 6641,'NASA' from dual union all
select 6642,'NASA' from dual union all
select 6643,'NASA' from dual union all
select 6644,'NASA' from dual union all
select 6645,'PVO' from dual union all
select 6646,'RVSN' from dual union all
select 6647,'RVSN' from dual union all
select 6648,'RVSN' from dual union all
select 6649,'RVSN' from dual union all
select 665,'USAF' from dual union all
select 6650,'RVSN' from dual union all
select 6651,'NASA' from dual union all
select 6652,'RVSN' from dual union all
select 6653,'RVSN' from dual union all
select 6654,'RVSN' from dual union all
select 6655,'NASA' from dual union all
select 6656,'NASA' from dual union all
select 6657,'RVSN' from dual union all
select 6658,'RVSN' from dual union all
select 6659,'GMS' from dual union all
select 6659,'RVSN' from dual union all
select 666,'USAF' from dual union all
select 6660,'RVSN' from dual union all
select 6661,'RVSN' from dual union all
select 6662,'TSKBEM' from dual union all
select 6663,'RVSN' from dual union all
select 6664,'RVSN' from dual union all
select 6665,'PVO' from dual union all
select 6666,'RVSN' from dual union all
select 6667,'RVSN' from dual union all
select 6668,'RVSN' from dual union all
select 6669,'RVSN' from dual union all
select 667,'USAF' from dual union all
select 6670,'RVSN' from dual union all
select 6671,'RVSN' from dual union all
select 6672,'RVSN' from dual union all
select 6673,'RVSN' from dual union all
select 6674,'RVSN' from dual union all
select 6675,'RVSN' from dual;
insert into SATELLITE_ORG
select 6676,'RVSN' from dual union all
select 6677,'RVSN' from dual union all
select 6678,'RVSN' from dual union all
select 6679,'RVSN' from dual union all
select 668,'CIA' from dual union all
select 668,'NRO' from dual union all
select 6680,'RVSN' from dual union all
select 6681,'RVSN' from dual union all
select 6682,'RVSN' from dual union all
select 6683,'RVSN' from dual union all
select 6684,'RVSN' from dual union all
select 6685,'RVSN' from dual union all
select 6686,'NASA' from dual union all
select 6687,'NASA' from dual union all
select 6688,'NASA' from dual union all
select 6689,'NASA' from dual union all
select 669,'USAF' from dual union all
select 6690,'NASA' from dual union all
select 6691,'USAF' from dual union all
select 6692,'RVSN' from dual union all
select 6693,'RVSN' from dual union all
select 6694,'TSKBEM' from dual union all
select 6695,'RVSN' from dual union all
select 6696,'VS' from dual union all
select 6697,'VS' from dual union all
select 6698,'VS' from dual union all
select 6699,'VS' from dual union all
select 67,'CIA' from dual union all
select 67,'USAF' from dual union all
select 670,'USN' from dual union all
select 6700,'VS' from dual union all
select 6701,'VS' from dual union all
select 6702,'TSKBEM' from dual union all
select 6703,'RVSN' from dual union all
select 6704,'NASA' from dual union all
select 6705,'NASA' from dual union all
select 6706,'RVSN' from dual union all
select 6707,'VMF' from dual union all
select 6708,'RVSN' from dual union all
select 6709,'RVSN' from dual union all
select 671,'USN' from dual union all
select 6710,'RVSN' from dual union all
select 6711,'RVSN' from dual union all
select 6712,'RVSN' from dual union all
select 6713,'TSKBEM' from dual union all
select 6714,'RVSN' from dual union all
select 6715,'TSKBEM' from dual union all
select 6716,'TSKBEM' from dual union all
select 6717,'TSKBEM' from dual union all
select 6718,'USN' from dual union all
select 6719,'RVSN' from dual union all
select 672,'USN' from dual union all
select 6720,'RVSN' from dual union all
select 6721,'RVSN' from dual union all
select 6722,'RVSN' from dual union all
select 6723,'RVSN' from dual union all
select 6724,'RVSN' from dual union all
select 6725,'NASA' from dual union all
select 6726,'NASA' from dual union all
select 6727,'NRO' from dual union all
select 6728,'USAF' from dual union all
select 6729,'NASA' from dual union all
select 673,'OKB1' from dual union all
select 6730,'NASA' from dual union all
select 6731,'NASA' from dual union all
select 6732,'NASA' from dual union all
select 6733,'NASA' from dual union all
select 6734,'NASA' from dual union all
select 6735,'NASA' from dual union all
select 6736,'RVSN' from dual union all
select 6737,'RVSN' from dual union all
select 6738,'RVSN' from dual union all
select 6739,'RVSN' from dual union all
select 674,'USAF' from dual union all
select 6740,'RVSN' from dual union all
select 6741,'RVSN' from dual union all
select 6742,'IKI' from dual union all
select 6742,'NPOL' from dual union all
select 6743,'RVSN' from dual union all
select 6744,'RVSN' from dual union all
select 6745,'RVSN' from dual union all
select 6746,'USAF' from dual union all
select 6747,'USAF' from dual union all
select 6748,'USAF' from dual union all
select 6749,'PVO' from dual union all
select 675,'USAF' from dual union all
select 6750,'USAF' from dual union all
select 6751,'PVO' from dual union all
select 6752,'PVO' from dual union all
select 6753,'RVSN' from dual union all
select 6754,'IKI' from dual union all
select 6754,'NPOL' from dual union all
select 6755,'RVSN' from dual union all
select 6756,'RVSN' from dual union all
select 6757,'NASA' from dual union all
select 6758,'NASA' from dual union all
select 6759,'RVSN' from dual union all
select 676,'RVSN' from dual union all
select 6760,'RVSN' from dual union all
select 6761,'RVSN' from dual;
insert into SATELLITE_ORG
select 6762,'RVSN' from dual union all
select 6763,'RVSN' from dual union all
select 6764,'PVO' from dual union all
select 6765,'NASA' from dual union all
select 6766,'NASA' from dual union all
select 6767,'NASA' from dual union all
select 6768,'IKI' from dual union all
select 6768,'NPOL' from dual union all
select 6769,'RVSN' from dual union all
select 677,'NRO' from dual union all
select 6770,'RVSN' from dual union all
select 6771,'RVSN' from dual union all
select 6772,'RVSN' from dual union all
select 6773,'NASA' from dual union all
select 6774,'NASA' from dual union all
select 6775,'RVSN' from dual union all
select 6776,'IKI' from dual union all
select 6776,'NPOL' from dual union all
select 6777,'RVSN' from dual union all
select 6778,'RVSN' from dual union all
select 6779,'NASA' from dual union all
select 678,'USAF' from dual union all
select 6780,'NASA' from dual union all
select 6781,'NASA' from dual union all
select 6782,'NASA' from dual union all
select 6783,'USAF' from dual union all
select 6784,'CNES' from dual union all
select 6785,'RVSN' from dual union all
select 6786,'NASA' from dual union all
select 6787,'USAF' from dual union all
select 6788,'USAF' from dual union all
select 6789,'RVSN' from dual union all
select 679,'USAF' from dual union all
select 6790,'RVSN' from dual union all
select 6791,'NRO' from dual union all
select 6792,'USAF' from dual union all
select 6793,'PVO' from dual union all
select 6794,'RVSN' from dual union all
select 6795,'RVSN' from dual union all
select 6796,'INTEL' from dual union all
select 6797,'NASA' from dual union all
select 6798,'RVSN' from dual union all
select 6799,'RVSN' from dual union all
select 68,'DOD' from dual union all
select 68,'USAF' from dual union all
select 680,'USAF' from dual union all
select 6800,'NASA' from dual union all
select 6801,'RVSN' from dual union all
select 6802,'VS' from dual union all
select 6803,'RVSN' from dual union all
select 6804,'VS' from dual union all
select 6805,'RVSN' from dual union all
select 6806,'RVSN' from dual union all
select 6807,'RVSN' from dual union all
select 6808,'RVSN' from dual union all
select 6809,'RVSN' from dual union all
select 681,'NRO' from dual union all
select 681,'USA' from dual union all
select 6810,'RVSN' from dual union all
select 6811,'RVSN' from dual union all
select 6812,'RVSN' from dual union all
select 6813,'RVSN' from dual union all
select 6814,'RVSN' from dual union all
select 6815,'RVSN' from dual union all
select 6816,'RVSN' from dual union all
select 6817,'RVSN' from dual union all
select 6818,'RVSN' from dual union all
select 6819,'RVSN' from dual union all
select 682,'NRO' from dual union all
select 6820,'RVSN' from dual union all
select 6821,'RVSN' from dual union all
select 6822,'NRO' from dual union all
select 6823,'NRO' from dual union all
select 6824,'NRO' from dual union all
select 6825,'VTS' from dual union all
select 6826,'RVSN' from dual union all
select 6827,'RVSN' from dual union all
select 6828,'VMF' from dual union all
select 6829,'RVSN' from dual union all
select 683,'OKB52' from dual union all
select 6830,'RVSN' from dual union all
select 6831,'RVSN' from dual union all
select 6832,'RVSN' from dual union all
select 6833,'RVSN' from dual union all
select 6834,'RVSN' from dual union all
select 6835,'RVSN' from dual union all
select 6836,'TSKBEM' from dual union all
select 6837,'NRO' from dual union all
select 6838,'RVSN' from dual union all
select 6839,'TSKBEM' from dual union all
select 684,'OKB52' from dual union all
select 6840,'TSKBEM' from dual union all
select 6841,'NASA' from dual union all
select 6842,'USAF' from dual union all
select 6843,'PVO' from dual union all
select 6844,'PVO' from dual union all
select 6845,'RVSN' from dual union all
select 6846,'RVSN' from dual union all
select 6847,'RVSN' from dual union all
select 6848,'RVSN' from dual;
insert into SATELLITE_ORG
select 6849,'RVSN' from dual union all
select 685,'OKB52' from dual union all
select 6850,'RVSN' from dual union all
select 6851,'RVSN' from dual union all
select 6852,'RVSN' from dual union all
select 6853,'RVSN' from dual union all
select 6854,'RVSN' from dual union all
select 6855,'RVSN' from dual union all
select 6856,'RVSN' from dual union all
select 6857,'RVSN' from dual union all
select 6858,'RVSN' from dual union all
select 6859,'RVSN' from dual union all
select 686,'OKB52' from dual union all
select 6860,'RVSN' from dual union all
select 6861,'RVSN' from dual union all
select 6862,'RVSN' from dual union all
select 6863,'RVSN' from dual union all
select 6864,'RVSN' from dual union all
select 6865,'RVSN' from dual union all
select 6866,'RVSN' from dual union all
select 6867,'RVSN' from dual union all
select 6868,'RVSN' from dual union all
select 6869,'RVSN' from dual union all
select 687,'OKB1' from dual union all
select 6870,'RVSN' from dual union all
select 6871,'RVSN' from dual union all
select 6872,'RVSN' from dual union all
select 6873,'RVSN' from dual union all
select 6874,'RVSN' from dual union all
select 6875,'PVO' from dual union all
select 6876,'RVSN' from dual union all
select 6877,'RVSN' from dual union all
select 6878,'RVSN' from dual union all
select 6879,'RVSN' from dual union all
select 688,'RVSN' from dual union all
select 6880,'PVO' from dual union all
select 6881,'PVO' from dual union all
select 6882,'PVO' from dual union all
select 6883,'PVO' from dual union all
select 6884,'PVO' from dual union all
select 6885,'RVSN' from dual union all
select 6886,'RVSN' from dual union all
select 6887,'PVO' from dual union all
select 6888,'PVO' from dual union all
select 6889,'PVO' from dual union all
select 689,'OKB1F3' from dual union all
select 6890,'PVO' from dual union all
select 6891,'PVO' from dual union all
select 6892,'PVO' from dual union all
select 6893,'NASA' from dual union all
select 6894,'NASA' from dual union all
select 6895,'NASA' from dual union all
select 6896,'NASA' from dual union all
select 6897,'PVO' from dual union all
select 6898,'RVSN' from dual union all
select 6899,'RVSN' from dual union all
select 69,'NASA' from dual union all
select 690,'RVSN' from dual union all
select 6900,'RVSN' from dual union all
select 6901,'RVSN' from dual union all
select 6902,'RVSN' from dual union all
select 6903,'RVSN' from dual union all
select 6904,'RVSN' from dual union all
select 6905,'RVSN' from dual union all
select 6906,'RVSN' from dual union all
select 6907,'TSN108' from dual union all
select 6908,'RVSN' from dual union all
select 6909,'USN' from dual union all
select 691,'USAF' from dual union all
select 6910,'NASA' from dual union all
select 6911,'IK' from dual union all
select 6912,'RVSN' from dual union all
select 6913,'IMBP' from dual union all
select 6913,'TSKBEM' from dual union all
select 6914,'RVSN' from dual union all
select 6915,'RVSN' from dual union all
select 6916,'PVO' from dual union all
select 6917,'RVSN' from dual union all
select 6918,'RVSN' from dual union all
select 6919,'JPL' from dual union all
select 6919,'NASA' from dual union all
select 692,'USAF' from dual union all
select 6920,'NOAA' from dual union all
select 6921,'NASA' from dual union all
select 6922,'RVSN' from dual union all
select 6923,'RVSN' from dual union all
select 6924,'RVSN' from dual union all
select 6925,'RVSN' from dual union all
select 6926,'RVSN' from dual union all
select 6927,'RVSN' from dual union all
select 6928,'NRO' from dual union all
select 6929,'USAF' from dual union all
select 693,'NASA' from dual union all
select 6930,'RVSN' from dual union all
select 6931,'NRO' from dual union all
select 6932,'RVSN' from dual union all
select 6933,'RVSN' from dual union all
select 6934,'RVSN' from dual union all
select 6935,'RVSN' from dual union all
select 6936,'NASA' from dual;
insert into SATELLITE_ORG
select 6937,'NASA' from dual union all
select 6938,'NRO' from dual union all
select 6939,'RVSN' from dual union all
select 694,'NASA' from dual union all
select 6940,'RVSN' from dual union all
select 6941,'PVO' from dual union all
select 6942,'RVSN' from dual union all
select 6943,'RVSN' from dual union all
select 6944,'RVSN' from dual union all
select 6945,'RVSN' from dual union all
select 6946,'RVSN' from dual union all
select 6947,'RVSN' from dual union all
select 6948,'RVSN' from dual union all
select 6949,'RVSN' from dual union all
select 695,'CIA' from dual union all
select 695,'NRO' from dual union all
select 6950,'VS' from dual union all
select 6951,'RVSN' from dual union all
select 6952,'PVO' from dual union all
select 6953,'RVSN' from dual union all
select 6954,'RVSN' from dual union all
select 6955,'RVSN' from dual union all
select 6956,'RVSN' from dual union all
select 6957,'TSKBEM' from dual union all
select 6958,'RVSN' from dual union all
select 6959,'RVSN' from dual union all
select 696,'NASA' from dual union all
select 6960,'RVSN' from dual union all
select 6961,'RVSN' from dual union all
select 6962,'VS' from dual union all
select 6963,'RVSN' from dual union all
select 6964,'RVSN' from dual union all
select 6965,'OKB10' from dual union all
select 6966,'RVSN' from dual union all
select 6967,'RVSN' from dual union all
select 6968,'RVSN' from dual union all
select 6969,'RVSN' from dual union all
select 697,'NASA' from dual union all
select 6970,'RVSN' from dual union all
select 6971,'PVO' from dual union all
select 6972,'RVSN' from dual union all
select 6973,'DOD' from dual union all
select 6973,'USAF' from dual union all
select 6974,'DOD' from dual union all
select 6974,'USAF' from dual union all
select 6975,'USAF' from dual union all
select 6976,'USAF' from dual union all
select 6977,'NASA' from dual union all
select 6978,'NASA' from dual union all
select 6979,'TSKBEM' from dual union all
select 698,'NASA' from dual union all
select 6980,'RVSN' from dual union all
select 6981,'TSKBEM' from dual union all
select 6982,'TSKBEM' from dual union all
select 6983,'RVSN' from dual union all
select 6984,'TSKBEM' from dual union all
select 6985,'RVSN' from dual union all
select 6986,'RVSN' from dual union all
select 6987,'RVSN' from dual union all
select 6988,'RVSN' from dual union all
select 6989,'RVSN' from dual union all
select 699,'NASA' from dual union all
select 6990,'RVSN' from dual union all
select 6991,'RVSN' from dual union all
select 6992,'RVSN' from dual union all
select 6993,'RVSN' from dual union all
select 6994,'RVSN' from dual union all
select 6995,'RVSN' from dual union all
select 6996,'RVSN' from dual union all
select 6997,'RVSN' from dual union all
select 6998,'VS' from dual union all
select 6999,'VS' from dual union all
select 7,'RVSN' from dual union all
select 70,'USAF' from dual union all
select 700,'NASA' from dual union all
select 7000,'RVSN' from dual union all
select 7001,'RVSN' from dual union all
select 7002,'RVSN' from dual union all
select 7003,'CNES' from dual union all
select 7003,'MOM' from dual union all
select 7004,'RVSN' from dual union all
select 7005,'VMF' from dual union all
select 7006,'RVSN' from dual union all
select 7007,'RVSN' from dual union all
select 7008,'VMF' from dual union all
select 7009,'RVSN' from dual union all
select 701,'NASA' from dual union all
select 7010,'RVSN' from dual union all
select 7011,'RVSN' from dual union all
select 7012,'NASA' from dual union all
select 7013,'NASA' from dual union all
select 7014,'NASA' from dual union all
select 7015,'NASA' from dual union all
select 7016,'NASA' from dual union all
select 7017,'NASA' from dual union all
select 7018,'NASA' from dual union all
select 7019,'NASA' from dual union all
select 702,'USAF' from dual union all
select 7020,'NASA' from dual union all
select 7021,'NASA' from dual;
insert into SATELLITE_ORG
select 7022,'NASA' from dual union all
select 7023,'NASA' from dual union all
select 7024,'NASA' from dual union all
select 7025,'NASA' from dual union all
select 7026,'NASA' from dual union all
select 7027,'NASA' from dual union all
select 7028,'NASA' from dual union all
select 7029,'NASA' from dual union all
select 703,'USAF' from dual union all
select 7030,'NASA' from dual union all
select 7031,'NASA' from dual union all
select 7032,'NASA' from dual union all
select 7033,'NASA' from dual union all
select 7034,'NASA' from dual union all
select 7035,'NASA' from dual union all
select 7036,'NASA' from dual union all
select 7037,'NASA' from dual union all
select 7038,'NASA' from dual union all
select 7039,'NASA' from dual union all
select 704,'USN' from dual union all
select 7040,'NASA' from dual union all
select 7041,'NASA' from dual union all
select 7042,'NASA' from dual union all
select 7043,'NASA' from dual union all
select 7044,'NASA' from dual union all
select 7045,'NASA' from dual union all
select 7046,'NASA' from dual union all
select 7047,'NASA' from dual union all
select 7048,'NASA' from dual union all
select 7049,'NASA' from dual union all
select 705,'USN' from dual union all
select 7050,'NASA' from dual union all
select 7051,'NASA' from dual union all
select 7052,'NASA' from dual union all
select 7053,'NASA' from dual union all
select 7054,'NASA' from dual union all
select 7055,'NASA' from dual union all
select 7056,'NASA' from dual union all
select 7057,'NASA' from dual union all
select 7058,'NASA' from dual union all
select 7059,'NASA' from dual union all
select 706,'USN' from dual union all
select 7060,'NASA' from dual union all
select 7061,'NASA' from dual union all
select 7062,'NASA' from dual union all
select 7063,'NASA' from dual union all
select 7064,'NASA' from dual union all
select 7065,'NASA' from dual union all
select 7066,'NASA' from dual union all
select 7067,'NASA' from dual union all
select 7068,'NASA' from dual union all
select 7069,'NASA' from dual union all
select 707,'METP' from dual union all
select 7070,'NASA' from dual union all
select 7071,'NASA' from dual union all
select 7072,'NASA' from dual union all
select 7073,'NASA' from dual union all
select 7074,'NASA' from dual union all
select 7075,'NASA' from dual union all
select 7076,'NASA' from dual union all
select 7077,'NASA' from dual union all
select 7078,'NASA' from dual union all
select 7079,'NASA' from dual union all
select 708,'RVSN' from dual union all
select 7080,'NASA' from dual union all
select 7081,'NASA' from dual union all
select 7082,'NASA' from dual union all
select 7083,'NASA' from dual union all
select 7084,'NASA' from dual union all
select 7085,'NASA' from dual union all
select 7086,'NASA' from dual union all
select 7087,'NASA' from dual union all
select 7088,'NASA' from dual union all
select 7089,'NASA' from dual union all
select 709,'METP' from dual union all
select 7090,'NASA' from dual union all
select 7091,'NASA' from dual union all
select 7092,'NASA' from dual union all
select 7093,'NASA' from dual union all
select 7094,'VMF' from dual union all
select 7095,'RVSN' from dual union all
select 7096,'MOD' from dual union all
select 7097,'NASA' from dual union all
select 7098,'NASA' from dual union all
select 7099,'NASA' from dual union all
select 71,'OKB1' from dual union all
select 710,'METP' from dual union all
select 7100,'RVSN' from dual union all
select 7101,'RVSN' from dual union all
select 7102,'RVSN' from dual union all
select 7103,'FKA' from dual union all
select 7104,'RVSN' from dual union all
select 7105,'RVSN' from dual union all
select 7106,'RVSN' from dual union all
select 7107,'RVSN' from dual union all
select 7108,'RVSN' from dual union all
select 7109,'VS' from dual union all
select 711,'NRO' from dual union all
select 7110,'RVSN' from dual union all
select 7111,'VS' from dual;
insert into SATELLITE_ORG
select 7112,'VS' from dual union all
select 7113,'VMF' from dual union all
select 7114,'VMF' from dual union all
select 7115,'VMF' from dual union all
select 7116,'VMF' from dual union all
select 7117,'RVSN' from dual union all
select 7118,'RVSN' from dual union all
select 7119,'RVSN' from dual union all
select 712,'OKB1F3' from dual union all
select 7120,'RVSN' from dual union all
select 7121,'NRO' from dual union all
select 7122,'ISAS' from dual union all
select 7123,'ISAS' from dual union all
select 7124,'NASA' from dual union all
select 7125,'NASA' from dual union all
select 7126,'NASA' from dual union all
select 7127,'NASA' from dual union all
select 7128,'NASA' from dual union all
select 7129,'NASA' from dual union all
select 713,'RVSN' from dual union all
select 7130,'NASA' from dual union all
select 7131,'NASA' from dual union all
select 7132,'NASA' from dual union all
select 7133,'NASA' from dual union all
select 7134,'NASA' from dual union all
select 7135,'NASA' from dual union all
select 7136,'NASA' from dual union all
select 7137,'NASA' from dual union all
select 7138,'NASA' from dual union all
select 7139,'NASA' from dual union all
select 714,'NASA' from dual union all
select 7140,'NASA' from dual union all
select 7141,'NASA' from dual union all
select 7142,'NASA' from dual union all
select 7143,'NASA' from dual union all
select 7144,'NASA' from dual union all
select 7145,'NASA' from dual union all
select 7146,'NASA' from dual union all
select 7147,'NASA' from dual union all
select 7148,'NASA' from dual union all
select 7149,'NASA' from dual union all
select 715,'USN' from dual union all
select 7150,'NASA' from dual union all
select 7151,'NASA' from dual union all
select 7152,'NASA' from dual union all
select 7153,'NASA' from dual union all
select 7154,'CNR' from dual union all
select 7155,'NASA' from dual union all
select 7156,'NASA' from dual union all
select 7157,'NASA' from dual union all
select 7158,'NASA' from dual union all
select 7159,'NASA' from dual union all
select 716,'NASA' from dual union all
select 7160,'NASA' from dual union all
select 7161,'NASA' from dual union all
select 7162,'NASA' from dual union all
select 7163,'NASA' from dual union all
select 7164,'NASA' from dual union all
select 7165,'NASA' from dual union all
select 7166,'NASA' from dual union all
select 7167,'NASA' from dual union all
select 7168,'NASA' from dual union all
select 7169,'NASA' from dual union all
select 717,'NASA' from dual union all
select 7170,'NASA' from dual union all
select 7171,'RVSN' from dual union all
select 7172,'RVSN' from dual union all
select 7173,'NASA' from dual union all
select 7174,'NASA' from dual union all
select 7175,'NASA' from dual union all
select 7176,'NASA' from dual union all
select 7177,'NASA' from dual union all
select 7178,'RVSN' from dual union all
select 7179,'NASA' from dual union all
select 718,'CIA' from dual union all
select 718,'NRO' from dual union all
select 7180,'NASA' from dual union all
select 7181,'NASA' from dual union all
select 7182,'NASA' from dual union all
select 7183,'NASA' from dual union all
select 7184,'NASA' from dual union all
select 7185,'NASA' from dual union all
select 7186,'NASA' from dual union all
select 7187,'PVO' from dual union all
select 7188,'RVSN' from dual union all
select 7189,'RVSN' from dual union all
select 719,'NRO' from dual union all
select 7190,'NASA' from dual union all
select 7191,'NASA' from dual union all
select 7192,'NASA' from dual union all
select 7193,'NASA' from dual union all
select 7194,'NASA' from dual union all
select 7195,'NASA' from dual union all
select 7196,'NASA' from dual union all
select 7197,'NASA' from dual union all
select 7198,'NASA' from dual union all
select 7199,'NASA' from dual union all
select 72,'RVSN' from dual union all
select 720,'NASA' from dual union all
select 7200,'NASA' from dual;
insert into SATELLITE_ORG
select 7201,'NASA' from dual union all
select 7202,'NASA' from dual union all
select 7203,'NASA' from dual union all
select 7204,'NASA' from dual union all
select 7205,'NASA' from dual union all
select 7206,'NASA' from dual union all
select 7207,'NASA' from dual union all
select 7208,'NASA' from dual union all
select 7209,'GMS' from dual union all
select 7209,'RVSN' from dual union all
select 721,'NASA' from dual union all
select 7210,'RVSN' from dual union all
select 7211,'PVO' from dual union all
select 7212,'RVSN' from dual union all
select 7213,'RAE' from dual union all
select 7214,'RAE' from dual union all
select 7215,'RAE' from dual union all
select 7216,'RVSN' from dual union all
select 7217,'RVSN' from dual union all
select 7218,'USAF' from dual union all
select 7219,'USAF' from dual union all
select 722,'NASA' from dual union all
select 7220,'RVSN' from dual union all
select 7221,'RVSN' from dual union all
select 7222,'TSSKB' from dual union all
select 7223,'IKI' from dual union all
select 7223,'NPOL' from dual union all
select 7224,'IKI' from dual union all
select 7224,'NPOL' from dual union all
select 7225,'RVSN' from dual union all
select 7226,'RVSN' from dual union all
select 7227,'RVSN' from dual union all
select 7228,'NASA' from dual union all
select 7229,'OKB10' from dual union all
select 723,'NASA' from dual union all
select 7230,'RVSN' from dual union all
select 7231,'RVSN' from dual union all
select 7232,'RVSN' from dual union all
select 7233,'RVSN' from dual union all
select 7234,'FKA' from dual union all
select 7235,'RVSN' from dual union all
select 7236,'FKA' from dual union all
select 7237,'FKA' from dual union all
select 7238,'FKA' from dual union all
select 7239,'RVSN' from dual union all
select 724,'NASA' from dual union all
select 7240,'RVSN' from dual union all
select 7241,'RVSN' from dual union all
select 7242,'NRO' from dual union all
select 7243,'USAF' from dual union all
select 7244,'DOD' from dual union all
select 7244,'USAF' from dual union all
select 7245,'RVSN' from dual union all
select 7246,'RVSN' from dual union all
select 7247,'NRO' from dual union all
select 7248,'RVSN' from dual union all
select 7249,'RVSN' from dual union all
select 725,'NASA' from dual union all
select 7250,'WUTC' from dual union all
select 7251,'NASA' from dual union all
select 7252,'NASA' from dual union all
select 7253,'RVSN' from dual union all
select 7254,'RVSN' from dual union all
select 7255,'RVSN' from dual union all
select 7256,'RVSN' from dual union all
select 7257,'VS' from dual union all
select 7258,'USN' from dual union all
select 7259,'USN' from dual union all
select 726,'NASA' from dual union all
select 7260,'RVSN' from dual union all
select 7261,'RVSN' from dual union all
select 7262,'RVSN' from dual union all
select 7263,'RVSN' from dual union all
select 7264,'RVSN' from dual union all
select 7265,'RVSN' from dual union all
select 7266,'RVSN' from dual union all
select 7267,'RVSN' from dual union all
select 7268,'RVSN' from dual union all
select 7269,'RVSN' from dual union all
select 727,'NRO' from dual union all
select 7270,'RVSN' from dual union all
select 7271,'RVSN' from dual union all
select 7272,'RVSN' from dual union all
select 7273,'RVSN' from dual union all
select 7274,'GMS' from dual union all
select 7274,'RVSN' from dual union all
select 7275,'RVSN' from dual union all
select 7276,'RVSN' from dual union all
select 7277,'RVSN' from dual union all
select 7278,'RVSN' from dual union all
select 7279,'RVSN' from dual union all
select 728,'NRO' from dual union all
select 728,'USN' from dual union all
select 7280,'RVSN' from dual union all
select 7281,'VTS' from dual union all
select 7282,'RVSN' from dual union all
select 7283,'RVSN' from dual union all
select 7284,'RVSN' from dual union all
select 7285,'USAF' from dual union all
select 7286,'RVSN' from dual;
insert into SATELLITE_ORG
select 7287,'RVSN' from dual union all
select 7288,'RVSN' from dual union all
select 7289,'RVSN' from dual union all
select 729,'USA' from dual union all
select 7290,'RVSN' from dual union all
select 7291,'VMF' from dual union all
select 7292,'RVSN' from dual union all
select 7293,'RVSN' from dual union all
select 7294,'RVSN' from dual union all
select 7295,'RVSN' from dual union all
select 7296,'RVSN' from dual union all
select 7297,'VMF' from dual union all
select 7298,'NASA' from dual union all
select 7299,'IK' from dual union all
select 73,'RVSN' from dual union all
select 730,'NRO' from dual union all
select 730,'USN' from dual union all
select 7300,'NASA' from dual union all
select 7301,'NASA' from dual union all
select 7302,'RVSN' from dual union all
select 7303,'NASA' from dual union all
select 7304,'NASA' from dual union all
select 7305,'RVSN' from dual union all
select 7306,'VS' from dual union all
select 7307,'RVSN' from dual union all
select 7308,'RVSN' from dual union all
select 7309,'RVSN' from dual union all
select 731,'NRO' from dual union all
select 731,'USN' from dual union all
select 7310,'RVSN' from dual union all
select 7311,'RVSN' from dual union all
select 7312,'RVSN' from dual union all
select 7313,'FKA' from dual union all
select 7314,'RVSN' from dual union all
select 7315,'IKI' from dual union all
select 7315,'NPOL' from dual union all
select 7316,'RVSN' from dual union all
select 7317,'RVSN' from dual union all
select 7318,'NASA' from dual union all
select 7319,'NASA' from dual union all
select 732,'NASA' from dual union all
select 7320,'RVSN' from dual union all
select 7321,'RVSN' from dual union all
select 7322,'RVSN' from dual union all
select 7323,'RVSN' from dual union all
select 7324,'NASA' from dual union all
select 7325,'NASA' from dual union all
select 7326,'NASA' from dual union all
select 7327,'NASA' from dual union all
select 7328,'RVSN' from dual union all
select 7329,'RVSN' from dual union all
select 733,'USAF' from dual union all
select 7330,'NRO' from dual union all
select 7331,'NASA' from dual union all
select 7332,'RVSN' from dual union all
select 7333,'RVSN' from dual union all
select 7334,'RVSN' from dual union all
select 7335,'RVSN' from dual union all
select 7336,'RVSN' from dual union all
select 7337,'PVO' from dual union all
select 7338,'RVSN' from dual union all
select 7339,'VS' from dual union all
select 734,'USAF' from dual union all
select 7340,'RVSN' from dual union all
select 7341,'VS' from dual union all
select 7342,'TSKBM' from dual union all
select 7343,'RVSN' from dual union all
select 7344,'RVSN' from dual union all
select 7345,'RVSN' from dual union all
select 7346,'RVSN' from dual union all
select 7347,'PVO' from dual union all
select 7348,'RVSN' from dual union all
select 7349,'VMF' from dual union all
select 735,'USAF' from dual union all
select 7350,'RVSN' from dual union all
select 7351,'TSSKB' from dual union all
select 7352,'PVO' from dual union all
select 7353,'RVSN' from dual union all
select 7354,'RVSN' from dual union all
select 7355,'RVSN' from dual union all
select 7356,'TSSKB' from dual union all
select 7357,'TSSKB' from dual union all
select 7358,'TSSKB' from dual union all
select 7359,'TSSKB' from dual union all
select 736,'NASA' from dual union all
select 7360,'RVSN' from dual union all
select 7361,'NPOE' from dual union all
select 7362,'RVSN' from dual union all
select 7363,'MVD' from dual union all
select 7364,'RVSN' from dual union all
select 7365,'RVSN' from dual union all
select 7366,'RVSN' from dual union all
select 7367,'RVSN' from dual union all
select 7368,'RVSN' from dual union all
select 7369,'USAF' from dual union all
select 737,'NASA' from dual union all
select 7370,'USAF' from dual union all
select 7371,'DFVLR' from dual union all
select 7372,'RVSN' from dual union all
select 7373,'RVSN' from dual;
insert into SATELLITE_ORG
select 7374,'RVSN' from dual union all
select 7375,'NASA' from dual union all
select 7376,'RVSN' from dual union all
select 7377,'RVSN' from dual union all
select 7378,'RVSN' from dual union all
select 7379,'RVSN' from dual union all
select 738,'NASA' from dual union all
select 7380,'DFVLR' from dual union all
select 7381,'RVSN' from dual union all
select 7382,'RVSN' from dual union all
select 7383,'RVSN' from dual union all
select 7384,'RVSN' from dual union all
select 7385,'PVO' from dual union all
select 7386,'RVSN' from dual union all
select 7387,'VMF' from dual union all
select 7388,'VMF' from dual union all
select 7389,'RVSN' from dual union all
select 739,'NASA' from dual union all
select 7390,'RVSN' from dual union all
select 7391,'RVSN' from dual union all
select 7392,'RVSN' from dual union all
select 7393,'RVSN' from dual union all
select 7394,'RVSN' from dual union all
select 7395,'VMF' from dual union all
select 7396,'VMF' from dual union all
select 7397,'VMF' from dual union all
select 7398,'VMF' from dual union all
select 7399,'RVSN' from dual union all
select 74,'NASA' from dual union all
select 740,'NASA' from dual union all
select 7400,'RVSN' from dual union all
select 7401,'RVSN' from dual union all
select 7402,'RVSN' from dual union all
select 7403,'RVSN' from dual union all
select 7404,'TSSKB' from dual union all
select 7405,'NPOE' from dual union all
select 7406,'RVSN' from dual union all
select 7407,'RVSN' from dual union all
select 7408,'RVSN' from dual union all
select 7409,'RVSN' from dual union all
select 741,'NASA' from dual union all
select 7410,'RVSN' from dual union all
select 7411,'USAF' from dual union all
select 7412,'USAF' from dual union all
select 7413,'NPOE' from dual union all
select 7414,'RVSN' from dual union all
select 7415,'NPOE' from dual union all
select 7416,'NRO' from dual union all
select 7417,'TSN108' from dual union all
select 7418,'RVSN' from dual union all
select 7419,'RVSN' from dual union all
select 742,'NASA' from dual union all
select 7420,'RVSN' from dual union all
select 7421,'NPOE' from dual union all
select 7422,'RVSN' from dual union all
select 7423,'RVSN' from dual union all
select 7424,'VTS' from dual union all
select 7425,'RVSN' from dual union all
select 7426,'RVSN' from dual union all
select 7427,'NIVR' from dual union all
select 7428,'NASA' from dual union all
select 7429,'NIVR' from dual union all
select 743,'NASA' from dual union all
select 7430,'NIVR' from dual union all
select 7431,'RVSN' from dual union all
select 7432,'RVSN' from dual union all
select 7433,'RVSN' from dual union all
select 7434,'RVSN' from dual union all
select 7435,'RVSN' from dual union all
select 7436,'RVSN' from dual union all
select 7437,'RVSN' from dual union all
select 7438,'RVSN' from dual union all
select 7439,'RVSN' from dual union all
select 744,'NASA' from dual union all
select 7440,'RVSN' from dual union all
select 7441,'RVSN' from dual union all
select 7442,'RVSN' from dual union all
select 7443,'RVSN' from dual union all
select 7444,'RVSN' from dual union all
select 7445,'RVSN' from dual union all
select 7446,'RVSN' from dual union all
select 7447,'PVO' from dual union all
select 7448,'RVSN' from dual union all
select 7449,'PVO' from dual union all
select 745,'USN' from dual union all
select 7450,'PVO' from dual union all
select 7451,'PVO' from dual union all
select 7452,'PVO' from dual union all
select 7453,'PVO' from dual union all
select 7454,'PVO' from dual union all
select 7455,'PVO' from dual union all
select 7456,'PVO' from dual union all
select 7457,'PVO' from dual union all
select 7458,'PVO' from dual union all
select 7459,'PVO' from dual union all
select 746,'OKB1' from dual union all
select 7460,'PVO' from dual union all
select 7461,'PVO' from dual union all
select 7462,'PVO' from dual union all
select 7463,'PVO' from dual;
insert into SATELLITE_ORG
select 7464,'PVO' from dual union all
select 7465,'PVO' from dual union all
select 7466,'WUTC' from dual union all
select 7467,'NASA' from dual union all
select 7468,'NASA' from dual union all
select 7469,'PVO' from dual union all
select 747,'JPL' from dual union all
select 747,'NASA' from dual union all
select 7470,'RVSN' from dual union all
select 7471,'SRC' from dual union all
select 7472,'NASA' from dual union all
select 7473,'RVSN' from dual union all
select 7474,'RVSN' from dual union all
select 7475,'RVSN' from dual union all
select 7476,'VMF' from dual union all
select 7477,'RVSN' from dual union all
select 7478,'IMBP' from dual union all
select 7478,'TSSKB' from dual union all
select 7479,'RVSN' from dual union all
select 748,'OKB1' from dual union all
select 7480,'RVSN' from dual union all
select 7481,'RVSN' from dual union all
select 7482,'RVSN' from dual union all
select 7483,'RVSN' from dual union all
select 7484,'RVSN' from dual union all
select 7485,'RVSN' from dual union all
select 7486,'RVSN' from dual union all
select 7487,'NASA' from dual union all
select 7488,'NASA' from dual union all
select 7489,'NASA' from dual union all
select 749,'NASA' from dual union all
select 7490,'GMS' from dual union all
select 7490,'RVSN' from dual union all
select 7491,'IKI' from dual union all
select 7491,'NPOL' from dual union all
select 7492,'RVSN' from dual union all
select 7493,'RVSN' from dual union all
select 7494,'RVSN' from dual union all
select 7495,'NRO' from dual union all
select 7496,'RVSN' from dual union all
select 7497,'USAF' from dual union all
select 7498,'NRO' from dual union all
select 7499,'USAF' from dual union all
select 75,'NASA' from dual union all
select 750,'OKB1' from dual union all
select 7500,'IK' from dual union all
select 7501,'RVSN' from dual union all
select 7502,'RVSN' from dual union all
select 7503,'RVSN' from dual union all
select 7504,'RVSN' from dual union all
select 7505,'RVSN' from dual union all
select 7506,'RVSN' from dual union all
select 7507,'RVSN' from dual union all
select 7508,'RVSN' from dual union all
select 7509,'TSSKB' from dual union all
select 751,'RVSN' from dual union all
select 7510,'RVSN' from dual union all
select 7511,'RVSN' from dual union all
select 7512,'RVSN' from dual union all
select 7513,'TSSKB' from dual union all
select 7514,'TSSKB' from dual union all
select 7515,'VS' from dual union all
select 7516,'VS' from dual union all
select 7517,'VS' from dual union all
select 7518,'VS' from dual union all
select 7519,'VS' from dual union all
select 752,'CIA' from dual union all
select 752,'NRO' from dual union all
select 7520,'RVSN' from dual union all
select 7521,'RVSN' from dual union all
select 7522,'RVSN' from dual union all
select 7523,'RVSN' from dual union all
select 7524,'RVSN' from dual union all
select 7525,'RVSN' from dual union all
select 7526,'RVSN' from dual union all
select 7527,'RVSN' from dual union all
select 7528,'RVSN' from dual union all
select 7529,'NOAA' from dual union all
select 753,'USN' from dual union all
select 7530,'AMNA' from dual union all
select 7531,'INTA' from dual union all
select 7532,'NASA' from dual union all
select 7533,'RVSN' from dual union all
select 7534,'RVSN' from dual union all
select 7535,'RVSN' from dual union all
select 7536,'RVSN' from dual union all
select 7537,'RVSN' from dual union all
select 7538,'PVO' from dual union all
select 7539,'RVSN' from dual union all
select 754,'NRO' from dual union all
select 7540,'MSVYAZ' from dual union all
select 7540,'OKB10' from dual union all
select 7541,'RVSN' from dual union all
select 7542,'RVSN' from dual union all
select 7543,'RVSN' from dual union all
select 7544,'INTEL' from dual union all
select 7545,'NASA' from dual union all
select 7546,'RVSN' from dual union all
select 7547,'MOD' from dual union all
select 7548,'NASA' from dual;
insert into SATELLITE_ORG
select 7549,'NASA' from dual union all
select 755,'CIA' from dual union all
select 755,'NRO' from dual union all
select 7550,'NASA' from dual union all
select 7551,'RVSN' from dual union all
select 7552,'RVSN' from dual union all
select 7553,'RVSN' from dual union all
select 7554,'RVSN' from dual union all
select 7555,'RVSN' from dual union all
select 7556,'RVSN' from dual union all
select 7557,'RVSN' from dual union all
select 7558,'USN' from dual union all
select 7559,'USAF' from dual union all
select 756,'USAF' from dual union all
select 7560,'RVSN' from dual union all
select 7561,'NPOE' from dual union all
select 7562,'RVSN' from dual union all
select 7563,'NPOE' from dual union all
select 7564,'NPOE' from dual union all
select 7565,'NPOE' from dual union all
select 7566,'NPOE' from dual union all
select 7567,'DFVLR' from dual union all
select 7568,'NASA' from dual union all
select 7569,'NASA' from dual union all
select 757,'OKB586' from dual union all
select 757,'PVO' from dual union all
select 7570,'NASA' from dual union all
select 7571,'TSSKB' from dual union all
select 7572,'RVSN' from dual union all
select 7573,'RVSN' from dual union all
select 7574,'GMS' from dual union all
select 7574,'RVSN' from dual union all
select 7575,'RVSN' from dual union all
select 7576,'VS' from dual union all
select 7577,'RVSN' from dual union all
select 7578,'CNES' from dual union all
select 7578,'DFVLR' from dual union all
select 7579,'NASA' from dual union all
select 758,'RVSN' from dual union all
select 7580,'NASA' from dual union all
select 7581,'NASA' from dual union all
select 7582,'VS' from dual union all
select 7583,'RVSN' from dual union all
select 7584,'RVSN' from dual union all
select 7585,'RVSN' from dual union all
select 7586,'RVSN' from dual union all
select 7587,'VMF' from dual union all
select 7588,'RVSN' from dual union all
select 7589,'RVSN' from dual union all
select 759,'NRO' from dual union all
select 7590,'RVSN' from dual union all
select 7591,'NPOE' from dual union all
select 7592,'RVSN' from dual union all
select 7593,'OKB10' from dual union all
select 7594,'RVSN' from dual union all
select 7595,'NPOE' from dual union all
select 7596,'RVSN' from dual union all
select 7597,'RVSN' from dual union all
select 7598,'NPOE' from dual union all
select 7599,'RVSN' from dual union all
select 76,'RVSN' from dual union all
select 760,'USAF' from dual union all
select 7600,'RVSN' from dual union all
select 7601,'RVSN' from dual union all
select 7602,'RVSN' from dual union all
select 7603,'RVSN' from dual union all
select 7604,'NPOE' from dual union all
select 7605,'RVSN' from dual union all
select 7606,'RVSN' from dual union all
select 7607,'RVSN' from dual union all
select 7608,'NPOE' from dual union all
select 7609,'RVSN' from dual union all
select 761,'USAF' from dual union all
select 7610,'NPOE' from dual union all
select 7611,'PVO' from dual union all
select 7612,'RVSN' from dual union all
select 7613,'NPOE' from dual union all
select 7614,'NPOE' from dual union all
select 7615,'NASA' from dual union all
select 7616,'NASA' from dual union all
select 7617,'RVSN' from dual union all
select 7618,'RVSN' from dual union all
select 7619,'RVSN' from dual union all
select 762,'OKB586' from dual union all
select 7620,'RVSN' from dual union all
select 7621,'RVSN' from dual union all
select 7622,'NPOE' from dual union all
select 7623,'PVO' from dual union all
select 7624,'RVSN' from dual union all
select 7625,'PVO' from dual union all
select 7626,'RVSN' from dual union all
select 7627,'RVSN' from dual union all
select 7628,'NPOE' from dual union all
select 7629,'RVSN' from dual union all
select 763,'OKB586' from dual union all
select 7630,'NPOE' from dual union all
select 7631,'RVSN' from dual union all
select 7632,'RVSN' from dual union all
select 7633,'NPOE' from dual union all
select 7634,'NPOE' from dual;
insert into SATELLITE_ORG
select 7635,'NPOE' from dual union all
select 7636,'VS' from dual union all
select 7637,'VS' from dual union all
select 7638,'RVSN' from dual union all
select 7639,'VS' from dual union all
select 764,'NRO' from dual union all
select 7640,'RVSN' from dual union all
select 7641,'RVSN' from dual union all
select 7642,'RVSN' from dual union all
select 7643,'RVSN' from dual union all
select 7644,'RVSN' from dual union all
select 7645,'VS' from dual union all
select 7646,'CNES' from dual union all
select 7647,'CNES' from dual union all
select 7648,'NASA' from dual union all
select 7649,'RVSN' from dual union all
select 765,'USAF' from dual union all
select 7650,'NASA' from dual union all
select 7651,'NASA' from dual union all
select 7652,'NASA' from dual union all
select 7653,'RVSN' from dual union all
select 7654,'CNES' from dual union all
select 7655,'CNES' from dual union all
select 7656,'NPOE' from dual union all
select 7657,'NPOE' from dual union all
select 7658,'NPOE' from dual union all
select 7659,'CNES' from dual union all
select 766,'OKB586' from dual union all
select 7660,'NASA' from dual union all
select 7661,'NPOE' from dual union all
select 7662,'NPOE' from dual union all
select 7663,'VTS' from dual union all
select 7664,'RVSN' from dual union all
select 7665,'RVSN' from dual union all
select 7666,'RVSN' from dual union all
select 7667,'RVSN' from dual union all
select 7668,'RVSN' from dual union all
select 7669,'RVSN' from dual union all
select 767,'RVSN' from dual union all
select 7670,'USN' from dual union all
select 7671,'ISAS' from dual union all
select 7672,'RVSN' from dual union all
select 7673,'RVSN' from dual union all
select 7674,'ISAS' from dual union all
select 7675,'RVSN' from dual union all
select 7676,'RVSN' from dual union all
select 7677,'RVSN' from dual union all
select 7678,'RVSN' from dual union all
select 7679,'RVSN' from dual union all
select 768,'OKB586' from dual union all
select 7680,'RVSN' from dual union all
select 7681,'RVSN' from dual union all
select 7682,'RVSN' from dual union all
select 7683,'RVSN' from dual union all
select 7684,'RVSN' from dual union all
select 7685,'RVSN' from dual union all
select 7686,'RVSN' from dual union all
select 7687,'NRO' from dual union all
select 7688,'USAF' from dual union all
select 7689,'RVSN' from dual union all
select 769,'OKB586' from dual union all
select 7690,'RVSN' from dual union all
select 7691,'RVSN' from dual union all
select 7692,'RVSN' from dual union all
select 7693,'RVSN' from dual union all
select 7694,'NIVR' from dual union all
select 7695,'VS' from dual union all
select 7696,'TSSKB' from dual union all
select 7697,'RVSN' from dual union all
select 7698,'TSSKB' from dual union all
select 7699,'TSSKB' from dual union all
select 77,'RVSN' from dual union all
select 770,'RVSN' from dual union all
select 7700,'TSSKB' from dual union all
select 7701,'RVSN' from dual union all
select 7702,'RVSN' from dual union all
select 7703,'RVSN' from dual union all
select 7704,'NPOE' from dual union all
select 7705,'RVSN' from dual union all
select 7706,'RVSN' from dual union all
select 7707,'RVSN' from dual union all
select 7708,'RVSN' from dual union all
select 7709,'RVSN' from dual union all
select 771,'DSIR' from dual union all
select 7710,'IK' from dual union all
select 7711,'RVSN' from dual union all
select 7712,'RVSN' from dual union all
select 7713,'RVSN' from dual union all
select 7714,'GMS' from dual union all
select 7714,'RVSN' from dual union all
select 7715,'RVSN' from dual union all
select 7716,'RVSN' from dual union all
select 7717,'RVSN' from dual union all
select 7718,'VMF' from dual union all
select 7719,'RVSN' from dual union all
select 772,'OKB1' from dual union all
select 7720,'RVSN' from dual union all
select 7721,'TSSKB' from dual union all
select 7722,'VS' from dual union all
select 7723,'VS' from dual;
insert into SATELLITE_ORG
select 7724,'VS' from dual union all
select 7725,'RVSN' from dual union all
select 7726,'RVSN' from dual union all
select 7727,'VMF' from dual union all
select 7728,'RVSN' from dual union all
select 7729,'RVSN' from dual union all
select 773,'RVSN' from dual union all
select 7730,'PVO' from dual union all
select 7731,'RVSN' from dual union all
select 7732,'RVSN' from dual union all
select 7733,'RVSN' from dual union all
select 7734,'NASA' from dual union all
select 7735,'NASA' from dual union all
select 7736,'OKB10' from dual union all
select 7737,'RVSN' from dual union all
select 7738,'MSVYAZ' from dual union all
select 7738,'OKB10' from dual union all
select 7739,'RVSN' from dual union all
select 774,'OKB1' from dual union all
select 7740,'RVSN' from dual union all
select 7741,'RVSN' from dual union all
select 7742,'RVSN' from dual union all
select 7743,'RVSN' from dual union all
select 7744,'RVSN' from dual union all
select 7745,'RVSN' from dual union all
select 7746,'RVSN' from dual union all
select 7747,'NRO' from dual union all
select 7748,'VMF' from dual union all
select 7749,'VMF' from dual union all
select 775,'NASA' from dual union all
select 7750,'VMF' from dual union all
select 7751,'RVSN' from dual union all
select 7752,'ISRO' from dual union all
select 7753,'RVSN' from dual union all
select 7754,'VMF' from dual union all
select 7755,'VMF' from dual union all
select 7756,'VMF' from dual union all
select 7757,'VMF' from dual union all
select 7758,'VMF' from dual union all
select 7759,'VMF' from dual union all
select 776,'RVSN' from dual union all
select 7760,'VMF' from dual union all
select 7761,'RVSN' from dual union all
select 7762,'VMF' from dual union all
select 7763,'VMF' from dual union all
select 7764,'VMF' from dual union all
select 7765,'VMF' from dual union all
select 7766,'VMF' from dual union all
select 7767,'VMF' from dual union all
select 7768,'VMF' from dual union all
select 7769,'RVSN' from dual union all
select 777,'RVSN' from dual union all
select 7770,'RVSN' from dual union all
select 7771,'RVSN' from dual union all
select 7772,'RVSN' from dual union all
select 7773,'RVSN' from dual union all
select 7774,'RVSN' from dual union all
select 7775,'RVSN' from dual union all
select 7776,'RVSN' from dual union all
select 7777,'VS' from dual union all
select 7778,'RVSN' from dual union all
select 7779,'TSSKB' from dual union all
select 778,'RVSN' from dual union all
select 7780,'RVSN' from dual union all
select 7781,'RVSN' from dual union all
select 7782,'RVSN' from dual union all
select 7783,'VMF' from dual union all
select 7784,'VMF' from dual union all
select 7785,'RVSN' from dual union all
select 7786,'RVSN' from dual union all
select 7787,'RVSN' from dual union all
select 7788,'NASA' from dual union all
select 7789,'NASA' from dual union all
select 779,'OKB1F3' from dual union all
select 7790,'TCAN' from dual union all
select 7791,'NASA' from dual union all
select 7792,'NASA' from dual union all
select 7793,'VMF' from dual union all
select 7794,'NASA' from dual union all
select 7795,'VMF' from dual union all
select 7796,'VMF' from dual union all
select 7797,'VMF' from dual union all
select 7798,'VMF' from dual union all
select 7799,'VMF' from dual union all
select 78,'OKB1' from dual union all
select 780,'RVSN' from dual union all
select 7800,'RVSN' from dual union all
select 7801,'CNES' from dual union all
select 7802,'CNES' from dual union all
select 7803,'CNES' from dual union all
select 7804,'CNES' from dual union all
select 7805,'CNES' from dual union all
select 7806,'CNES' from dual union all
select 7807,'DOD' from dual union all
select 7807,'USAF' from dual union all
select 7808,'DOD' from dual union all
select 7808,'USAF' from dual union all
select 7809,'USAF' from dual union all
select 781,'OKB1F3' from dual union all
select 7810,'RVSN' from dual;
insert into SATELLITE_ORG
select 7811,'RVSN' from dual union all
select 7812,'RVSN' from dual union all
select 7813,'RVSN' from dual union all
select 7814,'VMF' from dual union all
select 7815,'INTEL' from dual union all
select 7816,'USAF' from dual union all
select 7817,'USAF' from dual union all
select 7818,'NPOE' from dual union all
select 7819,'RVSN' from dual union all
select 782,'NASA' from dual union all
select 7820,'RVSN' from dual union all
select 7821,'RVSN' from dual union all
select 7822,'RVSN' from dual union all
select 7823,'RVSN' from dual union all
select 7824,'RVSN' from dual union all
select 7825,'RVSN' from dual union all
select 7826,'RVSN' from dual union all
select 7827,'RVSN' from dual union all
select 7828,'RVSN' from dual union all
select 7829,'RVSN' from dual union all
select 783,'OKB52' from dual union all
select 7830,'RVSN' from dual union all
select 7831,'RVSN' from dual union all
select 7832,'NASA' from dual union all
select 7833,'NASA' from dual union all
select 7834,'NASA' from dual union all
select 7835,'NASA' from dual union all
select 7836,'NASA' from dual union all
select 7837,'NASA' from dual union all
select 7838,'NASA' from dual union all
select 7839,'NASA' from dual union all
select 784,'OKB52' from dual union all
select 7840,'NASA' from dual union all
select 7841,'NASA' from dual union all
select 7842,'NASA' from dual union all
select 7843,'NASA' from dual union all
select 7844,'NASA' from dual union all
select 7845,'NASA' from dual union all
select 7846,'NASA' from dual union all
select 7847,'NASA' from dual union all
select 7848,'NASA' from dual union all
select 7849,'NASA' from dual union all
select 785,'OKB52' from dual union all
select 7850,'NASA' from dual union all
select 7851,'NASA' from dual union all
select 7852,'NASA' from dual union all
select 7853,'NASA' from dual union all
select 7854,'NASA' from dual union all
select 7855,'NASA' from dual union all
select 7856,'NASA' from dual union all
select 7857,'NASA' from dual union all
select 7858,'NASA' from dual union all
select 7859,'NASA' from dual union all
select 786,'NRO' from dual union all
select 7860,'NASA' from dual union all
select 7861,'NASA' from dual union all
select 7862,'NASA' from dual union all
select 7863,'NASA' from dual union all
select 7864,'NASA' from dual union all
select 7865,'NASA' from dual union all
select 7866,'NASA' from dual union all
select 7867,'NASA' from dual union all
select 7868,'NASA' from dual union all
select 7869,'NASA' from dual union all
select 787,'USAF' from dual union all
select 7870,'NASA' from dual union all
select 7871,'NASA' from dual union all
select 7872,'NASA' from dual union all
select 7873,'NASA' from dual union all
select 7874,'NASA' from dual union all
select 7875,'NASA' from dual union all
select 7876,'NASA' from dual union all
select 7877,'MVD' from dual union all
select 7878,'RVSN' from dual union all
select 7879,'RVSN' from dual union all
select 788,'USAF' from dual union all
select 7880,'RVSN' from dual union all
select 7881,'RVSN' from dual union all
select 7882,'NASA' from dual union all
select 7883,'NASA' from dual union all
select 7884,'NASA' from dual union all
select 7885,'TSSKB' from dual union all
select 7886,'NASA' from dual union all
select 7887,'NASA' from dual union all
select 7888,'NASA' from dual union all
select 7889,'NASA' from dual union all
select 789,'USAF' from dual union all
select 7890,'NASA' from dual union all
select 7891,'NASA' from dual union all
select 7892,'NASA' from dual union all
select 7893,'NASA' from dual union all
select 7894,'NASA' from dual union all
select 7895,'NASA' from dual union all
select 7896,'NASA' from dual union all
select 7897,'NASA' from dual union all
select 7898,'NASA' from dual union all
select 7899,'NASA' from dual union all
select 79,'USAF' from dual union all
select 790,'USAF' from dual union all
select 7900,'RVSN' from dual;
insert into SATELLITE_ORG
select 7901,'RVSN' from dual union all
select 7902,'NASA' from dual union all
select 7903,'RVSN' from dual union all
select 7904,'RVSN' from dual union all
select 7905,'RVSN' from dual union all
select 7906,'RVSN' from dual union all
select 7907,'NASA' from dual union all
select 7908,'NASA' from dual union all
select 7909,'NASA' from dual union all
select 791,'OKB1F3' from dual union all
select 7910,'CNES' from dual union all
select 7911,'NPOE' from dual union all
select 7912,'NPOE' from dual union all
select 7913,'RVSN' from dual union all
select 7914,'NASA' from dual union all
select 7915,'IKI' from dual union all
select 7915,'NPOL' from dual union all
select 7916,'RVSN' from dual union all
select 7917,'RVSN' from dual union all
select 7918,'NRO' from dual union all
select 7919,'USAF' from dual union all
select 792,'RVSN' from dual union all
select 7920,'RVSN' from dual union all
select 7921,'RVSN' from dual union all
select 7922,'VMF' from dual union all
select 7923,'VMF' from dual union all
select 7924,'NASA' from dual union all
select 7925,'RVSN' from dual union all
select 7926,'RVSN' from dual union all
select 7927,'RVSN' from dual union all
select 7928,'NASA' from dual union all
select 7929,'NASA' from dual union all
select 793,'OKB1F3' from dual union all
select 7930,'NASA' from dual union all
select 7931,'NASA' from dual union all
select 7932,'NASA' from dual union all
select 7933,'NASA' from dual union all
select 7934,'NASA' from dual union all
select 7935,'NASA' from dual union all
select 7936,'NASA' from dual union all
select 7937,'NRO' from dual union all
select 7938,'NRO' from dual union all
select 7939,'NRO' from dual union all
select 794,'OKB1F3' from dual union all
select 7940,'NASA' from dual union all
select 7941,'NASA' from dual union all
select 7942,'NASA' from dual union all
select 7943,'NASA' from dual union all
select 7944,'NASA' from dual union all
select 7945,'NASA' from dual union all
select 7946,'NASA' from dual union all
select 7947,'IKI' from dual union all
select 7947,'NPOL' from dual union all
select 7948,'NPOE' from dual union all
select 7949,'NPOE' from dual union all
select 795,'OKB1F3' from dual union all
select 7950,'RVSN' from dual union all
select 7951,'RVSN' from dual union all
select 7952,'RVSN' from dual union all
select 7953,'NPOE' from dual union all
select 7954,'RVSN' from dual union all
select 7955,'NPOE' from dual union all
select 7956,'NPOE' from dual union all
select 7957,'NASA' from dual union all
select 7958,'NASA' from dual union all
select 7959,'NASA' from dual union all
select 796,'CIA' from dual union all
select 796,'NRO' from dual union all
select 7960,'NASA' from dual union all
select 7961,'NASA' from dual union all
select 7962,'NPOE' from dual union all
select 7963,'NRO' from dual union all
select 7964,'USAF' from dual union all
select 7965,'NASA' from dual union all
select 7966,'NASA' from dual union all
select 7967,'NASA' from dual union all
select 7968,'TSN108' from dual union all
select 7969,'RVSN' from dual union all
select 797,'OKB1F3' from dual union all
select 7970,'NASA' from dual union all
select 7971,'NASA' from dual union all
select 7972,'RVSN' from dual union all
select 7973,'NASA' from dual union all
select 7974,'NASA' from dual union all
select 7975,'NASA' from dual union all
select 7976,'NASA' from dual union all
select 7977,'NASA' from dual union all
select 7978,'NASA' from dual union all
select 7979,'NASA' from dual union all
select 798,'RVSN' from dual union all
select 7980,'NASA' from dual union all
select 7981,'RVSN' from dual union all
select 7982,'PVO' from dual union all
select 7983,'RVSN' from dual union all
select 7984,'NPOE' from dual union all
select 7985,'RVSN' from dual union all
select 7986,'RVSN' from dual union all
select 7987,'RVSN' from dual union all
select 7988,'RVSN' from dual union all
select 7989,'RVSN' from dual;
insert into SATELLITE_ORG
select 799,'NRO' from dual union all
select 7990,'RVSN' from dual union all
select 7991,'RVSN' from dual union all
select 7992,'RVSN' from dual union all
select 7993,'RVSN' from dual union all
select 7994,'RVSN' from dual union all
select 7995,'NPOE' from dual union all
select 7996,'NPOE' from dual union all
select 7997,'NPOE' from dual union all
select 7998,'NPOE' from dual union all
select 7999,'VMF' from dual union all
select 8,'OKB1' from dual union all
select 80,'OKB1' from dual union all
select 800,'NASA' from dual union all
select 8000,'USN' from dual union all
select 8001,'NASA' from dual union all
select 8002,'NASA' from dual union all
select 8003,'NASA' from dual union all
select 8004,'NASA' from dual union all
select 8005,'NASA' from dual union all
select 8006,'RVSN' from dual union all
select 8007,'RVSN' from dual union all
select 8008,'RVSN' from dual union all
select 8009,'VS' from dual union all
select 801,'USN' from dual union all
select 8010,'RVSN' from dual union all
select 8011,'RVSN' from dual union all
select 8012,'RVSN' from dual union all
select 8013,'TSSKB' from dual union all
select 8014,'RVSN' from dual union all
select 8015,'RVSN' from dual union all
select 8016,'RVSN' from dual union all
select 8017,'RVSN' from dual union all
select 8018,'RVSN' from dual union all
select 8019,'NPOE' from dual union all
select 802,'CIA' from dual union all
select 802,'NRO' from dual union all
select 8020,'NPOE' from dual union all
select 8021,'NPOE' from dual union all
select 8022,'NPOE' from dual union all
select 8023,'NPOE' from dual union all
select 8024,'NPOE' from dual union all
select 8025,'NPOE' from dual union all
select 8026,'GMS' from dual union all
select 8026,'RVSN' from dual union all
select 8027,'RVSN' from dual union all
select 8028,'RVSN' from dual union all
select 8029,'RVSN' from dual union all
select 803,'OKB586' from dual union all
select 8030,'NPOE' from dual union all
select 8031,'RVSN' from dual union all
select 8032,'NASA' from dual union all
select 8033,'NASA' from dual union all
select 8034,'RVSN' from dual union all
select 8035,'CNES' from dual union all
select 8036,'PVO' from dual union all
select 8037,'RVSN' from dual union all
select 8038,'NPOE' from dual union all
select 8039,'METP' from dual union all
select 804,'RVSN' from dual union all
select 8040,'RVSN' from dual union all
select 8041,'RVSN' from dual union all
select 8042,'NASA' from dual union all
select 8043,'PVO' from dual union all
select 8044,'RVSN' from dual union all
select 8045,'VMF' from dual union all
select 8046,'VMF' from dual union all
select 8047,'VMF' from dual union all
select 8048,'VMF' from dual union all
select 8049,'VMF' from dual union all
select 805,'USN' from dual union all
select 8050,'VMF' from dual union all
select 8051,'VMF' from dual union all
select 8052,'VMF' from dual union all
select 8053,'MAI' from dual union all
select 8054,'MAI' from dual union all
select 8055,'NPOE' from dual union all
select 8056,'VMF' from dual union all
select 8057,'VMF' from dual union all
select 8058,'RVSN' from dual union all
select 8059,'RVSN' from dual union all
select 806,'USN' from dual union all
select 8060,'RVSN' from dual union all
select 8061,'RVSN' from dual union all
select 8062,'ESA' from dual union all
select 8063,'NASA' from dual union all
select 8064,'NASA' from dual union all
select 8065,'NASA' from dual union all
select 8066,'NASA' from dual union all
select 8067,'RVSN' from dual union all
select 8068,'RVSN' from dual union all
select 8069,'RVSN' from dual union all
select 807,'OKB1F3' from dual union all
select 8070,'RVSN' from dual union all
select 8071,'RVSN' from dual union all
select 8072,'OKB10' from dual union all
select 8073,'RVSN' from dual union all
select 8074,'NASA' from dual union all
select 8075,'NASA' from dual union all
select 8076,'NASA' from dual;
insert into SATELLITE_ORG
select 8077,'NASA' from dual union all
select 8078,'NASA' from dual union all
select 8079,'NASA' from dual union all
select 808,'RVSN' from dual union all
select 8080,'NASA' from dual union all
select 8081,'NASA' from dual union all
select 8082,'NASA' from dual union all
select 8083,'NASA' from dual union all
select 8084,'NASA' from dual union all
select 8085,'NASA' from dual union all
select 8086,'NASA' from dual union all
select 8087,'NASA' from dual union all
select 8088,'NASA' from dual union all
select 8089,'NASA' from dual union all
select 809,'NASA' from dual union all
select 8090,'NASA' from dual union all
select 8091,'NASA' from dual union all
select 8092,'NASA' from dual union all
select 8093,'NASA' from dual union all
select 8094,'NASA' from dual union all
select 8095,'NASA' from dual union all
select 8096,'NASA' from dual union all
select 8097,'NASA' from dual union all
select 8098,'NASA' from dual union all
select 8099,'NASA' from dual union all
select 81,'NASA' from dual union all
select 810,'OKB1F3' from dual union all
select 8100,'VMF' from dual union all
select 8101,'VMF' from dual union all
select 8102,'VMF' from dual union all
select 8103,'VMF' from dual union all
select 8104,'VMF' from dual union all
select 8105,'VMF' from dual union all
select 8106,'VMF' from dual union all
select 8107,'VS' from dual union all
select 8108,'JPL' from dual union all
select 8109,'VMF' from dual union all
select 811,'NRO' from dual union all
select 811,'USA' from dual union all
select 8110,'RVSN' from dual union all
select 8111,'NASA' from dual union all
select 8112,'VMF' from dual union all
select 8113,'VMF' from dual union all
select 8114,'VMF' from dual union all
select 8115,'VMF' from dual union all
select 8116,'VMF' from dual union all
select 8117,'VMF' from dual union all
select 8118,'VMF' from dual union all
select 8119,'VMF' from dual union all
select 812,'USAF' from dual union all
select 8120,'VMF' from dual union all
select 8121,'VMF' from dual union all
select 8122,'VMF' from dual union all
select 8123,'NASA' from dual union all
select 8124,'NASA' from dual union all
select 8125,'NASA' from dual union all
select 8126,'NASA' from dual union all
select 8127,'TSN108' from dual union all
select 8128,'RVSN' from dual union all
select 8129,'RVSN' from dual union all
select 813,'USAF' from dual union all
select 8130,'RVSN' from dual union all
select 8131,'RVSN' from dual union all
select 8132,'CNES' from dual union all
select 8132,'DFVLR' from dual union all
select 8133,'NASA' from dual union all
select 8134,'NASA' from dual union all
select 8135,'NASA' from dual union all
select 8136,'NASA' from dual union all
select 8137,'NASA' from dual union all
select 8138,'NASA' from dual union all
select 8139,'NASA' from dual union all
select 814,'CIA' from dual union all
select 814,'NRO' from dual union all
select 8140,'NASA' from dual union all
select 8141,'NASA' from dual union all
select 8142,'NASA' from dual union all
select 8143,'NASA' from dual union all
select 8144,'NASA' from dual union all
select 8145,'NASA' from dual union all
select 8146,'NASA' from dual union all
select 8147,'RVSN' from dual union all
select 8148,'RVSN' from dual union all
select 8149,'RVSN' from dual union all
select 815,'USAF' from dual union all
select 8150,'RVSN' from dual union all
select 8151,'NASA' from dual union all
select 8152,'NASA' from dual union all
select 8153,'NASA' from dual union all
select 8154,'NASA' from dual union all
select 8155,'NASA' from dual union all
select 8156,'NASA' from dual union all
select 8157,'NASA' from dual union all
select 8158,'NASA' from dual union all
select 8159,'NASA' from dual union all
select 816,'OKB1F3' from dual union all
select 8160,'NASA' from dual union all
select 8161,'NASA' from dual union all
select 8162,'NASA' from dual union all
select 8163,'NASA' from dual;
insert into SATELLITE_ORG
select 8164,'NASA' from dual union all
select 8165,'NASA' from dual union all
select 8166,'NASA' from dual union all
select 8167,'NASA' from dual union all
select 8168,'NASA' from dual union all
select 8169,'NASA' from dual union all
select 817,'RVSN' from dual union all
select 8170,'NASA' from dual union all
select 8171,'NASA' from dual union all
select 8172,'NASA' from dual union all
select 8173,'NASA' from dual union all
select 8174,'NASA' from dual union all
select 8175,'NASA' from dual union all
select 8176,'NASA' from dual union all
select 8177,'NASA' from dual union all
select 8178,'NASA' from dual union all
select 8179,'NASA' from dual union all
select 818,'OKB1F3' from dual union all
select 8180,'NASA' from dual union all
select 8181,'NASA' from dual union all
select 8182,'NASA' from dual union all
select 8183,'NASA' from dual union all
select 8184,'NASA' from dual union all
select 8185,'NASA' from dual union all
select 8186,'NASA' from dual union all
select 8187,'RVSN' from dual union all
select 8188,'RVSN' from dual union all
select 8189,'RVSN' from dual union all
select 819,'OKB1F3' from dual union all
select 8190,'RVSN' from dual union all
select 8191,'TSSKB' from dual union all
select 8192,'RVSN' from dual union all
select 8193,'RVSN' from dual union all
select 8194,'RVSN' from dual union all
select 8195,'RVSN' from dual union all
select 8196,'RVSN' from dual union all
select 8197,'NASDA' from dual union all
select 8198,'RVSN' from dual union all
select 8199,'JPL' from dual union all
select 82,'NASA' from dual union all
select 820,'USAF' from dual union all
select 8200,'RVSN' from dual union all
select 8201,'RVSN' from dual union all
select 8202,'RVSN' from dual union all
select 8203,'RVSN' from dual union all
select 8204,'RVSN' from dual union all
select 8205,'RVSN' from dual union all
select 8206,'RVSN' from dual union all
select 8207,'RVSN' from dual union all
select 8208,'RVSN' from dual union all
select 8209,'RVSN' from dual union all
select 821,'CIA' from dual union all
select 821,'NRO' from dual union all
select 8210,'RVSN' from dual union all
select 8211,'RVSN' from dual union all
select 8212,'RVSN' from dual union all
select 8213,'RVSN' from dual union all
select 8214,'RVSN' from dual union all
select 8215,'RVSN' from dual union all
select 8216,'RVSN' from dual union all
select 8217,'RVSN' from dual union all
select 8218,'RVSN' from dual union all
select 8219,'RVSN' from dual union all
select 822,'OKB1F3' from dual union all
select 8220,'RVSN' from dual union all
select 8221,'RVSN' from dual union all
select 8222,'RVSN' from dual union all
select 8223,'RVSN' from dual union all
select 8224,'RVSN' from dual union all
select 8225,'RVSN' from dual union all
select 8226,'RVSN' from dual union all
select 8227,'RVSN' from dual union all
select 8228,'RVSN' from dual union all
select 8229,'RVSN' from dual union all
select 823,'RVSN' from dual union all
select 8230,'RVSN' from dual union all
select 8231,'RVSN' from dual union all
select 8232,'RVSN' from dual union all
select 8233,'RVSN' from dual union all
select 8234,'RVSN' from dual union all
select 8235,'RVSN' from dual union all
select 8236,'RVSN' from dual union all
select 8237,'RVSN' from dual union all
select 8238,'RVSN' from dual union all
select 8239,'RVSN' from dual union all
select 824,'NRO' from dual union all
select 8240,'RVSN' from dual union all
select 8241,'RVSN' from dual union all
select 8242,'RVSN' from dual union all
select 8243,'RVSN' from dual union all
select 8244,'RVSN' from dual union all
select 8245,'RVSN' from dual union all
select 8246,'RVSN' from dual union all
select 8247,'RVSN' from dual union all
select 8248,'RVSN' from dual union all
select 8249,'RVSN' from dual union all
select 825,'NRO' from dual union all
select 8250,'RVSN' from dual union all
select 8251,'RVSN' from dual union all
select 8252,'RVSN' from dual;
insert into SATELLITE_ORG
select 8253,'RVSN' from dual union all
select 8254,'RVSN' from dual union all
select 8255,'RVSN' from dual union all
select 8256,'RVSN' from dual union all
select 8257,'RVSN' from dual union all
select 8258,'RVSN' from dual union all
select 8259,'RVSN' from dual union all
select 826,'NRO' from dual union all
select 8260,'RVSN' from dual union all
select 8261,'RVSN' from dual union all
select 8262,'RVSN' from dual union all
select 8263,'RVSN' from dual union all
select 8264,'RVSN' from dual union all
select 8265,'RVSN' from dual union all
select 8266,'RVSN' from dual union all
select 8267,'RVSN' from dual union all
select 8268,'RVSN' from dual union all
select 8269,'RVSN' from dual union all
select 827,'USAF' from dual union all
select 8270,'RVSN' from dual union all
select 8271,'RVSN' from dual union all
select 8272,'NASA' from dual union all
select 8273,'RVSN' from dual union all
select 8274,'RVSN' from dual union all
select 8275,'TSSKB' from dual union all
select 8276,'RVSN' from dual union all
select 8277,'TSSKB' from dual union all
select 8278,'TSSKB' from dual union all
select 8279,'TSSKB' from dual union all
select 828,'CIA' from dual union all
select 828,'NRO' from dual union all
select 8280,'RVSN' from dual union all
select 8281,'RVSN' from dual union all
select 8282,'RVSN' from dual union all
select 8283,'RVSN' from dual union all
select 8284,'RVSN' from dual union all
select 8285,'RVSN' from dual union all
select 8286,'RVSN' from dual union all
select 8287,'RVSN' from dual union all
select 8288,'RVSN' from dual union all
select 8289,'RVSN' from dual union all
select 829,'OKB1' from dual union all
select 8290,'RVSN' from dual union all
select 8291,'RVSN' from dual union all
select 8292,'RVSN' from dual union all
select 8293,'GMS' from dual union all
select 8293,'RVSN' from dual union all
select 8294,'RVSN' from dual union all
select 8295,'RVSN' from dual union all
select 8296,'NASA' from dual union all
select 8297,'NASA' from dual union all
select 8298,'NASA' from dual union all
select 8299,'NASA' from dual union all
select 83,'USA' from dual union all
select 83,'USAF' from dual union all
select 830,'OKB1' from dual union all
select 8300,'NASA' from dual union all
select 8301,'NASA' from dual union all
select 8302,'NASA' from dual union all
select 8303,'NASA' from dual union all
select 8304,'NASA' from dual union all
select 8305,'NASA' from dual union all
select 8306,'NASA' from dual union all
select 8307,'NASA' from dual union all
select 8308,'NASA' from dual union all
select 8309,'NASA' from dual union all
select 831,'OKB1' from dual union all
select 8310,'NASA' from dual union all
select 8311,'NASA' from dual union all
select 8312,'NASA' from dual union all
select 8313,'NASA' from dual union all
select 8314,'NASA' from dual union all
select 8315,'NASA' from dual union all
select 8316,'NASA' from dual union all
select 8317,'NASA' from dual union all
select 8318,'NASA' from dual union all
select 8319,'NASA' from dual union all
select 832,'RVSN' from dual union all
select 8320,'NASA' from dual union all
select 8321,'NASA' from dual union all
select 8322,'RVSN' from dual union all
select 8323,'RVSN' from dual union all
select 8324,'RVSN' from dual union all
select 8325,'VTS' from dual union all
select 8326,'RVSN' from dual union all
select 8327,'MVD' from dual union all
select 8328,'RVSN' from dual union all
select 8329,'RVSN' from dual union all
select 833,'OKB1F3' from dual union all
select 8330,'INTEL' from dual union all
select 8331,'NASA' from dual union all
select 8332,'CNES' from dual union all
select 8333,'CNES' from dual union all
select 8334,'RVSN' from dual union all
select 8335,'RVSN' from dual union all
select 8336,'CNES' from dual union all
select 8337,'CNES' from dual union all
select 8338,'NPOE' from dual union all
select 8339,'RVSN' from dual union all
select 834,'RVSN' from dual;
insert into SATELLITE_ORG
select 8340,'CNES' from dual union all
select 8341,'CNES' from dual union all
select 8342,'CNES' from dual union all
select 8343,'RVSN' from dual union all
select 8344,'RVSN' from dual union all
select 8345,'RVSN' from dual union all
select 8346,'RVSN' from dual union all
select 8347,'RVSN' from dual union all
select 8348,'RVSN' from dual union all
select 8349,'RVSN' from dual union all
select 835,'OKB1F3' from dual union all
select 8350,'RVSN' from dual union all
select 8351,'RVSN' from dual union all
select 8352,'NASDA' from dual union all
select 8353,'NASA' from dual union all
select 8354,'NASA' from dual union all
select 8355,'RVSN' from dual union all
select 8356,'RVSN' from dual union all
select 8357,'PVO' from dual union all
select 8358,'RVSN' from dual union all
select 8359,'RVSN' from dual union all
select 836,'USAF' from dual union all
select 8360,'NRO' from dual union all
select 8361,'USN' from dual union all
select 8362,'RVSN' from dual union all
select 8363,'RVSN' from dual union all
select 8364,'NASA' from dual union all
select 8365,'RVSN' from dual union all
select 8366,'NOAA' from dual union all
select 8367,'NASA' from dual union all
select 8368,'NASA' from dual union all
select 8369,'RVSN' from dual union all
select 837,'USAF' from dual union all
select 8370,'RVSN' from dual union all
select 8371,'NASA' from dual union all
select 8372,'NASA' from dual union all
select 8373,'RVSN' from dual union all
select 8374,'NASA' from dual union all
select 8375,'NASA' from dual union all
select 8376,'NASA' from dual union all
select 8377,'NASA' from dual union all
select 8378,'NASA' from dual union all
select 8379,'NASA' from dual union all
select 838,'USAF' from dual union all
select 8380,'NASA' from dual union all
select 8381,'NASA' from dual union all
select 8382,'NASA' from dual union all
select 8383,'NASA' from dual union all
select 8384,'NASA' from dual union all
select 8385,'NASA' from dual union all
select 8386,'NASA' from dual union all
select 8387,'NASA' from dual union all
select 8388,'NASA' from dual union all
select 8389,'NASA' from dual union all
select 839,'OKB1F3' from dual union all
select 8390,'NASA' from dual union all
select 8391,'NASA' from dual union all
select 8392,'NASA' from dual union all
select 8393,'NASA' from dual union all
select 8394,'NASA' from dual union all
select 8395,'NASA' from dual union all
select 8396,'NASA' from dual union all
select 8397,'NASA' from dual union all
select 8398,'NASA' from dual union all
select 8399,'NASA' from dual union all
select 84,'DOD' from dual union all
select 84,'USAF' from dual union all
select 840,'USN' from dual union all
select 8400,'NASA' from dual union all
select 8401,'NASA' from dual union all
select 8402,'NASA' from dual union all
select 8403,'NASA' from dual union all
select 8404,'NASA' from dual union all
select 8405,'NASA' from dual union all
select 8406,'NASA' from dual union all
select 8407,'NASA' from dual union all
select 8408,'NASA' from dual union all
select 8409,'USN' from dual union all
select 841,'USN' from dual union all
select 8410,'USN' from dual union all
select 8411,'IKI' from dual union all
select 8411,'NPOL' from dual union all
select 8412,'TSSKB' from dual union all
select 8413,'RVSN' from dual union all
select 8414,'RVSN' from dual union all
select 8415,'RVSN' from dual union all
select 8416,'VMF' from dual union all
select 8417,'RVSN' from dual union all
select 8418,'RVSN' from dual union all
select 8419,'OKB10' from dual union all
select 842,'JPL' from dual union all
select 842,'NASA' from dual union all
select 8420,'RVSN' from dual union all
select 8421,'RVSN' from dual union all
select 8422,'RVSN' from dual union all
select 8423,'IKI' from dual union all
select 8423,'NPOL' from dual union all
select 8424,'RVSN' from dual union all
select 8425,'MSVYAZ' from dual union all
select 8425,'OKB10' from dual;
insert into SATELLITE_ORG
select 8426,'RVSN' from dual union all
select 8427,'RVSN' from dual union all
select 8428,'RVSN' from dual union all
select 8429,'RVSN' from dual union all
select 843,'NASA' from dual union all
select 8430,'NPOE' from dual union all
select 8431,'RVSN' from dual union all
select 8432,'NASA' from dual union all
select 8433,'NASA' from dual union all
select 8434,'NASA' from dual union all
select 8435,'NASA' from dual union all
select 8436,'NASA' from dual union all
select 8437,'NASA' from dual union all
select 8438,'NASA' from dual union all
select 8439,'NASA' from dual union all
select 844,'OKB586' from dual union all
select 844,'PVO' from dual union all
select 8440,'NASA' from dual union all
select 8441,'NASA' from dual union all
select 8442,'RVSN' from dual union all
select 8443,'RVSN' from dual union all
select 8444,'VS' from dual union all
select 8445,'RVSN' from dual union all
select 8446,'RVSN' from dual union all
select 8447,'VS' from dual union all
select 8448,'VS' from dual union all
select 8449,'VS' from dual union all
select 845,'RVSN' from dual union all
select 8450,'IMBP' from dual union all
select 8450,'TSSKB' from dual union all
select 8451,'RVSN' from dual union all
select 8452,'MAI' from dual union all
select 8453,'MAI' from dual union all
select 8454,'MAI' from dual union all
select 8455,'MAI' from dual union all
select 8456,'MAI' from dual union all
select 8457,'MAI' from dual union all
select 8458,'RVSN' from dual union all
select 8459,'RVSN' from dual union all
select 846,'CIA' from dual union all
select 846,'NRO' from dual union all
select 8460,'TSSKB' from dual union all
select 8461,'RVSN' from dual union all
select 8462,'RVSN' from dual union all
select 8463,'RVSN' from dual union all
select 8464,'RVSN' from dual union all
select 8465,'RVSN' from dual union all
select 8466,'RVSN' from dual union all
select 8467,'NRO' from dual union all
select 8468,'USAF' from dual union all
select 8469,'USAF' from dual union all
select 847,'DSIR' from dual union all
select 8470,'USAF' from dual union all
select 8471,'IK' from dual union all
select 8472,'RVSN' from dual union all
select 8473,'VMF' from dual union all
select 8474,'RVSN' from dual union all
select 8475,'RVSN' from dual union all
select 8476,'AMCR' from dual union all
select 8477,'VMF' from dual union all
select 8478,'NASA' from dual union all
select 8479,'NASA' from dual union all
select 848,'RVSN' from dual union all
select 8480,'VMF' from dual union all
select 8481,'RVSN' from dual union all
select 8482,'USAF' from dual union all
select 8483,'USAF' from dual union all
select 8484,'RVSN' from dual union all
select 8485,'TSSKB' from dual union all
select 8486,'RVSN' from dual union all
select 8487,'RVSN' from dual union all
select 8488,'MAI' from dual union all
select 8489,'RVSN' from dual union all
select 849,'RVSN' from dual union all
select 8490,'RVSN' from dual union all
select 8491,'MAI' from dual union all
select 8492,'RVSN' from dual union all
select 8493,'RVSN' from dual union all
select 8494,'RVSN' from dual union all
select 8495,'NASA' from dual union all
select 8496,'NASA' from dual union all
select 8497,'NASA' from dual union all
select 8498,'NASA' from dual union all
select 8499,'NASA' from dual union all
select 85,'NASA' from dual union all
select 850,'NRO' from dual union all
select 8500,'NASA' from dual union all
select 8501,'NASA' from dual union all
select 8502,'NASA' from dual union all
select 8503,'NASA' from dual union all
select 8504,'NASA' from dual union all
select 8505,'NASA' from dual union all
select 8506,'NASA' from dual union all
select 8507,'NASA' from dual union all
select 8508,'NASA' from dual union all
select 8509,'NASA' from dual union all
select 851,'NRO' from dual union all
select 8510,'IKI' from dual union all
select 8510,'NPOL' from dual union all
select 8511,'RVSN' from dual;
insert into SATELLITE_ORG
select 8512,'RVSN' from dual union all
select 8513,'RVSN' from dual union all
select 8514,'RVSN' from dual union all
select 8515,'RVSN' from dual union all
select 8516,'USAF' from dual union all
select 8517,'USAF' from dual union all
select 8518,'RVSN' from dual union all
select 8519,'GMS' from dual union all
select 8519,'RVSN' from dual union all
select 852,'USAF' from dual union all
select 8520,'RVSN' from dual union all
select 8521,'MSVYAZ' from dual union all
select 8521,'OKB10' from dual union all
select 8522,'RVSN' from dual union all
select 8523,'RVSN' from dual union all
select 8524,'RVSN' from dual union all
select 8525,'RVSN' from dual union all
select 8526,'RVSN' from dual union all
select 8527,'RVSN' from dual union all
select 8528,'RVSN' from dual union all
select 8529,'RVSN' from dual union all
select 853,'OKB10' from dual union all
select 8530,'VS' from dual union all
select 8531,'RVSN' from dual union all
select 8532,'NASA' from dual union all
select 8533,'NASA' from dual union all
select 8534,'NASA' from dual union all
select 8535,'NASA' from dual union all
select 8536,'NASA' from dual union all
select 8537,'NASA' from dual union all
select 8538,'NASA' from dual union all
select 8539,'NASA' from dual union all
select 854,'OKB10' from dual union all
select 8540,'NASA' from dual union all
select 8541,'NASA' from dual union all
select 8542,'NASA' from dual union all
select 8543,'NASA' from dual union all
select 8544,'NASA' from dual union all
select 8545,'NASA' from dual union all
select 8546,'RVSN' from dual union all
select 8547,'RVSN' from dual union all
select 8548,'RVSN' from dual union all
select 8549,'VS' from dual union all
select 855,'OKB10' from dual union all
select 8550,'VS' from dual union all
select 8551,'RVSN' from dual union all
select 8552,'RVSN' from dual union all
select 8553,'RVSN' from dual union all
select 8554,'NASA' from dual union all
select 8555,'NASA' from dual union all
select 8556,'NASA' from dual union all
select 8557,'NASA' from dual union all
select 8558,'NASA' from dual union all
select 8559,'NASA' from dual union all
select 856,'RVSN' from dual union all
select 8560,'NASA' from dual union all
select 8561,'NASA' from dual union all
select 8562,'NASA' from dual union all
select 8563,'NASA' from dual union all
select 8564,'NASA' from dual union all
select 8565,'NASA' from dual union all
select 8566,'NASA' from dual union all
select 8567,'NASA' from dual union all
select 8568,'NASA' from dual union all
select 8569,'NASA' from dual union all
select 857,'OKB10' from dual union all
select 8570,'NASA' from dual union all
select 8571,'NASA' from dual union all
select 8572,'NASA' from dual union all
select 8573,'NASA' from dual union all
select 8574,'NASA' from dual union all
select 8575,'NASA' from dual union all
select 8576,'NASA' from dual union all
select 8577,'NASA' from dual union all
select 8578,'NASA' from dual union all
select 8579,'NASA' from dual union all
select 858,'NASA' from dual union all
select 8580,'NASA' from dual union all
select 8581,'NASA' from dual union all
select 8582,'DFVLR' from dual union all
select 8583,'NASA' from dual union all
select 8584,'NASA' from dual union all
select 8585,'CRC' from dual union all
select 8586,'NASA' from dual union all
select 8587,'NASA' from dual union all
select 8588,'RVSN' from dual union all
select 8589,'RVSN' from dual union all
select 859,'OKB10' from dual union all
select 8590,'RVSN' from dual union all
select 8591,'OKB10' from dual union all
select 8592,'NASA' from dual union all
select 8593,'NASA' from dual union all
select 8594,'NASA' from dual union all
select 8595,'NASA' from dual union all
select 8596,'NASA' from dual union all
select 8597,'RVSN' from dual union all
select 8598,'NASA' from dual union all
select 8599,'USAF' from dual union all
select 86,'NASA' from dual union all
select 860,'OKB10' from dual;
insert into SATELLITE_ORG
select 8600,'RVSN' from dual union all
select 8601,'RVSN' from dual union all
select 8602,'RVSN' from dual union all
select 8603,'RVSN' from dual union all
select 8604,'VS' from dual union all
select 8605,'RVSN' from dual union all
select 8606,'USAF' from dual union all
select 8607,'RVSN' from dual union all
select 8608,'RVSN' from dual union all
select 8609,'RVSN' from dual union all
select 861,'NRO' from dual union all
select 861,'USA' from dual union all
select 8610,'RVSN' from dual union all
select 8611,'RVSN' from dual union all
select 8612,'RVSN' from dual union all
select 8613,'RVSN' from dual union all
select 8614,'RVSN' from dual union all
select 8615,'RVSN' from dual union all
select 8616,'RVSN' from dual union all
select 8617,'RVSN' from dual union all
select 8618,'RVSN' from dual union all
select 8619,'RVSN' from dual union all
select 862,'NASA' from dual union all
select 8620,'INTEL' from dual union all
select 8621,'NASA' from dual union all
select 8622,'VMF' from dual union all
select 8623,'VMF' from dual union all
select 8624,'VMF' from dual union all
select 8625,'VMF' from dual union all
select 8626,'VMF' from dual union all
select 8627,'VMF' from dual union all
select 8628,'VMF' from dual union all
select 8629,'VMF' from dual union all
select 863,'RVSN' from dual union all
select 8630,'VMF' from dual union all
select 8631,'VMF' from dual union all
select 8632,'VMF' from dual union all
select 8633,'VMF' from dual union all
select 8634,'VMF' from dual union all
select 8635,'VMF' from dual union all
select 8636,'VMF' from dual union all
select 8637,'VMF' from dual union all
select 8638,'VMF' from dual union all
select 8639,'VMF' from dual union all
select 864,'OKB10' from dual union all
select 8640,'VMF' from dual union all
select 8641,'VMF' from dual union all
select 8642,'VMF' from dual union all
select 8643,'VMF' from dual union all
select 8644,'VMF' from dual union all
select 8645,'VMF' from dual union all
select 8646,'RVSN' from dual union all
select 8647,'VMF' from dual union all
select 8648,'VMF' from dual union all
select 8649,'VMF' from dual union all
select 865,'RVSN' from dual union all
select 8650,'VMF' from dual union all
select 8651,'VMF' from dual union all
select 8652,'VMF' from dual union all
select 8653,'VMF' from dual union all
select 8654,'VMF' from dual union all
select 8655,'VMF' from dual union all
select 8656,'VMF' from dual union all
select 8657,'VMF' from dual union all
select 8658,'PVO' from dual union all
select 8659,'RVSN' from dual union all
select 866,'RVSN' from dual union all
select 8660,'RVSN' from dual union all
select 8661,'RVSN' from dual union all
select 8662,'RVSN' from dual union all
select 8663,'RVSN' from dual union all
select 8664,'RVSN' from dual union all
select 8665,'RVSN' from dual union all
select 8666,'RVSN' from dual union all
select 8667,'RVSN' from dual union all
select 8668,'RVSN' from dual union all
select 8669,'RVSN' from dual union all
select 867,'OKB10' from dual union all
select 8670,'RVSN' from dual union all
select 8671,'RVSN' from dual union all
select 8672,'RVSN' from dual union all
select 8673,'RVSN' from dual union all
select 8674,'RVSN' from dual union all
select 8675,'NASA' from dual union all
select 8676,'NASA' from dual union all
select 8677,'NASA' from dual union all
select 8678,'NASA' from dual union all
select 8679,'NASA' from dual union all
select 868,'OKB1' from dual union all
select 8680,'NASA' from dual union all
select 8681,'RVSN' from dual union all
select 8682,'NASA' from dual union all
select 8683,'NASA' from dual union all
select 8684,'NASA' from dual union all
select 8685,'NASA' from dual union all
select 8686,'RVSN' from dual union all
select 8687,'NASA' from dual union all
select 8688,'PVO' from dual union all
select 8689,'RVSN' from dual union all
select 869,'OKB1' from dual;
insert into SATELLITE_ORG
select 8690,'PVO' from dual union all
select 8691,'NASA' from dual union all
select 8692,'NASA' from dual union all
select 8693,'NASA' from dual union all
select 8694,'PVO' from dual union all
select 8695,'RVSN' from dual union all
select 8696,'USAF' from dual union all
select 8697,'COM' from dual union all
select 8698,'NASA' from dual union all
select 8699,'TSSKB' from dual union all
select 87,'USN' from dual union all
select 870,'NASA' from dual union all
select 8700,'RVSN' from dual union all
select 8701,'RVSN' from dual union all
select 8702,'NASA' from dual union all
select 8703,'RVSN' from dual union all
select 8704,'NASA' from dual union all
select 8705,'NASA' from dual union all
select 8706,'RVSN' from dual union all
select 8707,'RVSN' from dual union all
select 8708,'RVSN' from dual union all
select 8709,'NASDA' from dual union all
select 871,'NASA' from dual union all
select 8710,'NASDA' from dual union all
select 8711,'VMF' from dual union all
select 8712,'VMF' from dual union all
select 8713,'VMF' from dual union all
select 8714,'VMF' from dual union all
select 8715,'VMF' from dual union all
select 8716,'VMF' from dual union all
select 8717,'VMF' from dual union all
select 8718,'VMF' from dual union all
select 8719,'VMF' from dual union all
select 872,'NASA' from dual union all
select 8720,'VMF' from dual union all
select 8721,'VMF' from dual union all
select 8722,'VMF' from dual union all
select 8723,'VMF' from dual union all
select 8724,'VMF' from dual union all
select 8725,'VMF' from dual union all
select 8726,'VMF' from dual union all
select 8727,'VMF' from dual union all
select 8728,'VMF' from dual union all
select 8729,'VMF' from dual union all
select 873,'NASA' from dual union all
select 8730,'VMF' from dual union all
select 8731,'VMF' from dual union all
select 8732,'VMF' from dual union all
select 8733,'VMF' from dual union all
select 8734,'VMF' from dual union all
select 8735,'VMF' from dual union all
select 8736,'VMF' from dual union all
select 8737,'RVSN' from dual union all
select 8738,'RVSN' from dual union all
select 8739,'NASA' from dual union all
select 874,'NASA' from dual union all
select 8740,'NASA' from dual union all
select 8741,'RVSN' from dual union all
select 8742,'RVSN' from dual union all
select 8743,'RVSN' from dual union all
select 8744,'PVO' from dual union all
select 8745,'RVSN' from dual union all
select 8746,'USAF' from dual union all
select 8747,'USAF' from dual union all
select 8748,'USAF' from dual union all
select 8749,'USAF' from dual union all
select 875,'NASA' from dual union all
select 8750,'USAF' from dual union all
select 8751,'USAF' from dual union all
select 8752,'USAF' from dual union all
select 8753,'USAF' from dual union all
select 8754,'TSN108' from dual union all
select 8755,'RVSN' from dual union all
select 8756,'RVSN' from dual union all
select 8757,'RVSN' from dual union all
select 8758,'RVSN' from dual union all
select 8759,'RVSN' from dual union all
select 876,'METP' from dual union all
select 8760,'RVSN' from dual union all
select 8761,'RVSN' from dual union all
select 8762,'RVSN' from dual union all
select 8763,'RVSN' from dual union all
select 8764,'RVSN' from dual union all
select 8765,'RVSN' from dual union all
select 8766,'RVSN' from dual union all
select 8767,'RVSN' from dual union all
select 8768,'RVSN' from dual union all
select 8769,'RVSN' from dual union all
select 877,'RVSN' from dual union all
select 8770,'NRO' from dual union all
select 8771,'RVSN' from dual union all
select 8772,'RVSN' from dual union all
select 8773,'RVSN' from dual union all
select 8774,'AMC' from dual union all
select 8775,'NASA' from dual union all
select 8776,'VS' from dual union all
select 8777,'VS' from dual union all
select 8778,'VS' from dual union all
select 8779,'VS' from dual union all
select 878,'NASA' from dual;
insert into SATELLITE_ORG
select 8780,'VS' from dual union all
select 8781,'VTS' from dual union all
select 8782,'RVSN' from dual union all
select 8783,'RVSN' from dual union all
select 8784,'NASA' from dual union all
select 8785,'NASA' from dual union all
select 8786,'NASA' from dual union all
select 8787,'NASA' from dual union all
select 8788,'NASA' from dual union all
select 8789,'NASA' from dual union all
select 879,'NASA' from dual union all
select 8790,'NASA' from dual union all
select 8791,'NASA' from dual union all
select 8792,'RVSN' from dual union all
select 8793,'NASA' from dual union all
select 8794,'VS' from dual union all
select 8795,'RVSN' from dual union all
select 8796,'RVSN' from dual union all
select 8797,'RVSN' from dual union all
select 8798,'RVSN' from dual union all
select 8799,'GMS' from dual union all
select 8799,'RVSN' from dual union all
select 88,'CIA' from dual union all
select 88,'USAF' from dual union all
select 880,'OKB1F3' from dual union all
select 8800,'RVSN' from dual union all
select 8801,'RVSN' from dual union all
select 8802,'RVSN' from dual union all
select 8803,'RVSN' from dual union all
select 8804,'RVSN' from dual union all
select 8805,'RVSN' from dual union all
select 8806,'PVO' from dual union all
select 8807,'RVSN' from dual union all
select 8808,'NATO' from dual union all
select 8809,'NASA' from dual union all
select 881,'RVSN' from dual union all
select 8810,'NASA' from dual union all
select 8811,'RVSN' from dual union all
select 8812,'PVO' from dual union all
select 8813,'RVSN' from dual union all
select 8814,'RVSN' from dual union all
select 8815,'RVSN' from dual union all
select 8816,'RVSN' from dual union all
select 8817,'RVSN' from dual union all
select 8818,'NRO' from dual union all
select 8818,'USN' from dual union all
select 8819,'USN' from dual union all
select 882,'CIA' from dual union all
select 882,'NRO' from dual union all
select 8820,'NASA' from dual union all
select 8821,'NASA' from dual union all
select 8822,'NASA' from dual union all
select 8823,'RVSN' from dual union all
select 8824,'RVSN' from dual union all
select 8825,'RVSN' from dual union all
select 8826,'RVSN' from dual union all
select 8827,'RVSN' from dual union all
select 8828,'NOAA' from dual union all
select 8829,'RVSN' from dual union all
select 883,'NASA' from dual union all
select 8830,'RVSN' from dual union all
select 8831,'RVSN' from dual union all
select 8832,'USAF' from dual union all
select 8833,'MSVYAZ' from dual union all
select 8833,'OKB10' from dual union all
select 8834,'RVSN' from dual union all
select 8835,'NRO' from dual union all
select 8835,'USN' from dual union all
select 8836,'NRO' from dual union all
select 8836,'USN' from dual union all
select 8837,'RVSN' from dual union all
select 8838,'COM' from dual union all
select 8839,'USN' from dual union all
select 884,'NRO' from dual union all
select 8840,'NASA' from dual union all
select 8841,'RVSN' from dual union all
select 8842,'USN' from dual union all
select 8843,'USN' from dual union all
select 8844,'RVSN' from dual union all
select 8845,'MVD' from dual union all
select 8846,'RVSN' from dual union all
select 8847,'RVSN' from dual union all
select 8848,'RVSN' from dual union all
select 8849,'RVSN' from dual union all
select 885,'RVSN' from dual union all
select 8850,'RVSN' from dual union all
select 8851,'PVO' from dual union all
select 8852,'RVSN' from dual union all
select 8853,'RVSN' from dual union all
select 8854,'RVSN' from dual union all
select 8855,'RVSN' from dual union all
select 8856,'MVD' from dual union all
select 8857,'RVSN' from dual union all
select 8858,'RVSN' from dual union all
select 8859,'USN' from dual union all
select 886,'RVSN' from dual union all
select 8860,'USAF' from dual union all
select 8861,'NASA' from dual union all
select 8862,'RVSN' from dual union all
select 8863,'RVSN' from dual;
insert into SATELLITE_ORG
select 8864,'RVSN' from dual union all
select 8865,'PVO' from dual union all
select 8866,'RVSN' from dual union all
select 8867,'USAF' from dual union all
select 8868,'USAF' from dual union all
select 8869,'RVSN' from dual union all
select 887,'USAF' from dual union all
select 8870,'RVSN' from dual union all
select 8871,'NRO' from dual union all
select 8872,'USAF' from dual union all
select 8873,'VMF' from dual union all
select 8874,'RVSN' from dual union all
select 8875,'RVSN' from dual union all
select 8876,'RVSN' from dual union all
select 8877,'RVSN' from dual union all
select 8878,'RVSN' from dual union all
select 8879,'RVSN' from dual union all
select 888,'CIA' from dual union all
select 888,'NRO' from dual union all
select 8880,'RVSN' from dual union all
select 8881,'RVSN' from dual union all
select 8882,'COM' from dual union all
select 8883,'NASA' from dual union all
select 8884,'NRO' from dual union all
select 8884,'USN' from dual union all
select 8885,'PVO' from dual union all
select 8886,'PVO' from dual union all
select 8887,'NASA' from dual union all
select 8888,'NASA' from dual union all
select 8889,'RVSN' from dual union all
select 889,'NASA' from dual union all
select 8890,'RVSN' from dual union all
select 8891,'RVSN' from dual union all
select 8892,'RVSN' from dual union all
select 8893,'RVSN' from dual union all
select 8894,'RVSN' from dual union all
select 8895,'RVSN' from dual union all
select 8896,'RVSN' from dual union all
select 8897,'RVSN' from dual union all
select 8898,'RVSN' from dual union all
select 8899,'RVSN' from dual union all
select 89,'CIA' from dual union all
select 89,'USAF' from dual union all
select 890,'CIA' from dual union all
select 890,'NRO' from dual union all
select 8900,'RVSN' from dual union all
select 8901,'RVSN' from dual union all
select 8902,'NASA' from dual union all
select 8903,'IK' from dual union all
select 8904,'RVSN' from dual union all
select 8905,'RVSN' from dual union all
select 8906,'RVSN' from dual union all
select 8907,'RVSN' from dual union all
select 8908,'NASA' from dual union all
select 8909,'NASA' from dual union all
select 891,'OKB1' from dual union all
select 8910,'NASA' from dual union all
select 8911,'TSKBM' from dual union all
select 8912,'RVSN' from dual union all
select 8913,'RVSN' from dual union all
select 8914,'RVSN' from dual union all
select 8915,'RVSN' from dual union all
select 8916,'USAF' from dual union all
select 8917,'USAF' from dual union all
select 8918,'USAF' from dual union all
select 8919,'USAF' from dual union all
select 892,'RVSN' from dual union all
select 8920,'RVSN' from dual union all
select 8921,'RVSN' from dual union all
select 8922,'RVSN' from dual union all
select 8923,'RVSN' from dual union all
select 8924,'RVSN' from dual union all
select 8925,'RVSN' from dual union all
select 8926,'RVSN' from dual union all
select 8927,'RVSN' from dual union all
select 8928,'RVSN' from dual union all
select 8929,'RVSN' from dual union all
select 893,'USAF' from dual union all
select 8930,'RVSN' from dual union all
select 8931,'RVSN' from dual union all
select 8932,'VMF' from dual union all
select 8933,'RVSN' from dual union all
select 8934,'NPOE' from dual union all
select 8935,'RVSN' from dual union all
select 8936,'NASA' from dual union all
select 8937,'NASA' from dual union all
select 8938,'NASA' from dual union all
select 8939,'NASA' from dual union all
select 894,'OKB1' from dual union all
select 8940,'NASA' from dual union all
select 8941,'NASA' from dual union all
select 8942,'NASA' from dual union all
select 8943,'NASA' from dual union all
select 8944,'NASA' from dual union all
select 8945,'NASA' from dual union all
select 8946,'NASA' from dual union all
select 8947,'NASA' from dual union all
select 8948,'NASA' from dual union all
select 8949,'NASA' from dual union all
select 895,'OKB1' from dual;
insert into SATELLITE_ORG
select 8950,'NASA' from dual union all
select 8951,'NASA' from dual union all
select 8952,'NASA' from dual union all
select 8953,'NASA' from dual union all
select 8954,'NASA' from dual union all
select 8955,'NASA' from dual union all
select 8956,'NASA' from dual union all
select 8957,'NASA' from dual union all
select 8958,'NASA' from dual union all
select 8959,'NASA' from dual union all
select 896,'OKB1' from dual union all
select 8960,'NASA' from dual union all
select 8961,'NASA' from dual union all
select 8962,'NASA' from dual union all
select 8963,'NASA' from dual union all
select 8964,'NASA' from dual union all
select 8965,'NASA' from dual union all
select 8966,'NASA' from dual union all
select 8967,'NASA' from dual union all
select 8968,'NASA' from dual union all
select 8969,'NASA' from dual union all
select 897,'USN' from dual union all
select 8970,'NASA' from dual union all
select 8971,'NASA' from dual union all
select 8972,'NASA' from dual union all
select 8973,'NASA' from dual union all
select 8974,'NASA' from dual union all
select 8975,'NASA' from dual union all
select 8976,'NASA' from dual union all
select 8977,'NASA' from dual union all
select 8978,'NASA' from dual union all
select 8979,'NASA' from dual union all
select 898,'RVSN' from dual union all
select 8980,'NASA' from dual union all
select 8981,'NASA' from dual union all
select 8982,'NASA' from dual union all
select 8983,'NASA' from dual union all
select 8984,'NASA' from dual union all
select 8985,'NASA' from dual union all
select 8986,'NASA' from dual union all
select 8987,'NASA' from dual union all
select 8988,'NASA' from dual union all
select 8989,'NASA' from dual union all
select 899,'NASA' from dual union all
select 8990,'NASA' from dual union all
select 8991,'NASA' from dual union all
select 8992,'NASA' from dual union all
select 8993,'NASA' from dual union all
select 8994,'NASA' from dual union all
select 8995,'NASA' from dual union all
select 8996,'NASA' from dual union all
select 8997,'NASA' from dual union all
select 8998,'NASA' from dual union all
select 8999,'NASA' from dual union all
select 9,'JPL' from dual union all
select 9,'USA' from dual union all
select 90,'CIA' from dual union all
select 90,'USAF' from dual union all
select 900,'USN' from dual union all
select 9000,'NASA' from dual union all
select 9001,'NASA' from dual union all
select 9002,'NASA' from dual union all
select 9003,'NASA' from dual union all
select 9004,'NASA' from dual union all
select 9005,'NASA' from dual union all
select 9006,'NRO' from dual union all
select 9007,'USAF' from dual union all
select 9008,'NRO' from dual union all
select 9009,'PER' from dual union all
select 901,'USN' from dual union all
select 9010,'USAF' from dual union all
select 9011,'PVO' from dual union all
select 9012,'NASA' from dual union all
select 9013,'RVSN' from dual union all
select 9014,'RVSN' from dual union all
select 9015,'RVSN' from dual union all
select 9016,'RVSN' from dual union all
select 9017,'NASA' from dual union all
select 9018,'RVSN' from dual union all
select 9019,'RVSN' from dual union all
select 902,'USN' from dual union all
select 9020,'RVSN' from dual union all
select 9021,'RVSN' from dual union all
select 9022,'RVSN' from dual union all
select 9023,'RVSN' from dual union all
select 9024,'NASA' from dual union all
select 9025,'VTS' from dual union all
select 9026,'NASA' from dual union all
select 9027,'NASA' from dual union all
select 9028,'NASA' from dual union all
select 9029,'NASA' from dual union all
select 903,'USN' from dual union all
select 9030,'NASA' from dual union all
select 9031,'NASA' from dual union all
select 9032,'NASA' from dual union all
select 9033,'NASA' from dual union all
select 9034,'NASA' from dual union all
select 9035,'NASA' from dual union all
select 9036,'NASA' from dual union all
select 9037,'NASA' from dual;
insert into SATELLITE_ORG
select 9038,'NASA' from dual union all
select 9039,'NASA' from dual union all
select 904,'OKB1' from dual union all
select 9040,'NASA' from dual union all
select 9041,'NASA' from dual union all
select 9042,'NASA' from dual union all
select 9043,'PVO' from dual union all
select 9044,'RVSN' from dual union all
select 9045,'RVSN' from dual union all
select 9046,'TSSKB' from dual union all
select 9047,'COM' from dual union all
select 9048,'RVSN' from dual union all
select 9049,'RVSN' from dual union all
select 905,'RVSN' from dual union all
select 9050,'RVSN' from dual union all
select 9051,'RVSN' from dual union all
select 9052,'RVSN' from dual union all
select 9053,'VS' from dual union all
select 9054,'RVSN' from dual union all
select 9055,'IKI' from dual union all
select 9056,'RVSN' from dual union all
select 9057,'NOAA' from dual union all
select 9058,'RVSN' from dual union all
select 9059,'RVSN' from dual union all
select 906,'OKB1' from dual union all
select 9060,'RVSN' from dual union all
select 9061,'VMF' from dual union all
select 9062,'RVSN' from dual union all
select 9063,'NASA' from dual union all
select 9064,'TSSKB' from dual union all
select 9065,'TSSKB' from dual union all
select 9066,'TSSKB' from dual union all
select 9067,'TSSKB' from dual union all
select 9068,'TSSKB' from dual union all
select 9069,'TSSKB' from dual union all
select 907,'NASA' from dual union all
select 9070,'TSSKB' from dual union all
select 9071,'TSSKB' from dual union all
select 9072,'TSSKB' from dual union all
select 9073,'TSSKB' from dual union all
select 9074,'TSSKB' from dual union all
select 9075,'TSSKB' from dual union all
select 9076,'TSSKB' from dual union all
select 9077,'TSSKB' from dual union all
select 9078,'TSSKB' from dual union all
select 9079,'TSSKB' from dual union all
select 908,'MO' from dual union all
select 9080,'TSSKB' from dual union all
select 9081,'TSSKB' from dual union all
select 9082,'TSSKB' from dual union all
select 9083,'TSSKB' from dual union all
select 9084,'TSSKB' from dual union all
select 9085,'TSSKB' from dual union all
select 9086,'TSSKB' from dual union all
select 9087,'TSSKB' from dual union all
select 9088,'TSSKB' from dual union all
select 9089,'TSSKB' from dual union all
select 909,'RVSN' from dual union all
select 9090,'TSSKB' from dual union all
select 9091,'TSSKB' from dual union all
select 9092,'TSSKB' from dual union all
select 9093,'TSSKB' from dual union all
select 9094,'TSSKB' from dual union all
select 9095,'TSSKB' from dual union all
select 9096,'TSSKB' from dual union all
select 9097,'TSSKB' from dual union all
select 9098,'TSSKB' from dual union all
select 9099,'TSSKB' from dual union all
select 91,'OKB1' from dual union all
select 910,'MO' from dual union all
select 9100,'TSSKB' from dual union all
select 9101,'TSSKB' from dual union all
select 9102,'TSSKB' from dual union all
select 9103,'TSSKB' from dual union all
select 9104,'TSSKB' from dual union all
select 9105,'TSSKB' from dual union all
select 9106,'TSSKB' from dual union all
select 9107,'TSSKB' from dual union all
select 9108,'TSSKB' from dual union all
select 9109,'TSSKB' from dual union all
select 911,'CIA' from dual union all
select 911,'NRO' from dual union all
select 9110,'TSSKB' from dual union all
select 9111,'TSSKB' from dual union all
select 9112,'TSSKB' from dual union all
select 9113,'TSSKB' from dual union all
select 9114,'TSSKB' from dual union all
select 9115,'TSSKB' from dual union all
select 9116,'TSSKB' from dual union all
select 9117,'TSSKB' from dual union all
select 9118,'TSSKB' from dual union all
select 9119,'TSSKB' from dual union all
select 912,'NRO' from dual union all
select 9120,'TSSKB' from dual union all
select 9121,'TSSKB' from dual union all
select 9122,'TSSKB' from dual union all
select 9123,'TSSKB' from dual union all
select 9124,'TSSKB' from dual union all
select 9125,'TSSKB' from dual union all
select 9126,'TSSKB' from dual;
insert into SATELLITE_ORG
select 9127,'TSSKB' from dual union all
select 9128,'TSSKB' from dual union all
select 9129,'TSSKB' from dual union all
select 913,'OKB586' from dual union all
select 9130,'TSSKB' from dual union all
select 9131,'TSSKB' from dual union all
select 9132,'TSSKB' from dual union all
select 9133,'TSSKB' from dual union all
select 9134,'TSSKB' from dual union all
select 9135,'TSSKB' from dual union all
select 9136,'TSSKB' from dual union all
select 9137,'TSSKB' from dual union all
select 9138,'TSSKB' from dual union all
select 9139,'TSSKB' from dual union all
select 914,'NRO' from dual union all
select 9140,'TSSKB' from dual union all
select 9141,'TSSKB' from dual union all
select 9142,'TSSKB' from dual union all
select 9143,'TSSKB' from dual union all
select 9144,'TSSKB' from dual union all
select 9145,'TSSKB' from dual union all
select 9146,'TSSKB' from dual union all
select 9147,'TSSKB' from dual union all
select 9148,'TSSKB' from dual union all
select 9149,'TSSKB' from dual union all
select 915,'RVSN' from dual union all
select 9150,'TSSKB' from dual union all
select 9151,'TSSKB' from dual union all
select 9152,'TSSKB' from dual union all
select 9153,'TSSKB' from dual union all
select 9154,'TSSKB' from dual union all
select 9155,'TSSKB' from dual union all
select 9156,'TSSKB' from dual union all
select 9157,'TSSKB' from dual union all
select 9158,'TSSKB' from dual union all
select 9159,'TSSKB' from dual union all
select 916,'USAF' from dual union all
select 9160,'TSSKB' from dual union all
select 9161,'TSSKB' from dual union all
select 9162,'TSSKB' from dual union all
select 9163,'TSSKB' from dual union all
select 9164,'TSSKB' from dual union all
select 9165,'TSSKB' from dual union all
select 9166,'TSSKB' from dual union all
select 9167,'TSSKB' from dual union all
select 9168,'TSSKB' from dual union all
select 9169,'TSSKB' from dual union all
select 917,'OKB586' from dual union all
select 9170,'TSSKB' from dual union all
select 9171,'TSSKB' from dual union all
select 9172,'TSSKB' from dual union all
select 9173,'TSSKB' from dual union all
select 9174,'TSSKB' from dual union all
select 9175,'TSSKB' from dual union all
select 9176,'TSSKB' from dual union all
select 9177,'TSSKB' from dual union all
select 9178,'TSSKB' from dual union all
select 9179,'TSSKB' from dual union all
select 918,'NRO' from dual union all
select 9180,'TSSKB' from dual union all
select 9181,'TSSKB' from dual union all
select 9182,'TSSKB' from dual union all
select 9183,'TSSKB' from dual union all
select 9184,'TSSKB' from dual union all
select 9185,'TSSKB' from dual union all
select 9186,'TSSKB' from dual union all
select 9187,'TSSKB' from dual union all
select 9188,'TSSKB' from dual union all
select 9189,'TSSKB' from dual union all
select 919,'MO' from dual union all
select 9190,'TSSKB' from dual union all
select 9191,'TSSKB' from dual union all
select 9192,'TSSKB' from dual union all
select 9193,'TSSKB' from dual union all
select 9194,'TSSKB' from dual union all
select 9195,'TSSKB' from dual union all
select 9196,'TSSKB' from dual union all
select 9197,'TSSKB' from dual union all
select 9198,'TSSKB' from dual union all
select 9199,'TSSKB' from dual union all
select 92,'RVSN' from dual union all
select 920,'RVSN' from dual union all
select 9200,'TSSKB' from dual union all
select 9201,'TSSKB' from dual union all
select 9202,'TSSKB' from dual union all
select 9203,'TSSKB' from dual union all
select 9204,'TSSKB' from dual union all
select 9205,'TSSKB' from dual union all
select 9206,'TSSKB' from dual union all
select 9207,'TSSKB' from dual union all
select 9208,'TSSKB' from dual union all
select 9209,'TSSKB' from dual union all
select 921,'CIA' from dual union all
select 921,'NRO' from dual union all
select 9210,'TSSKB' from dual union all
select 9211,'TSSKB' from dual union all
select 9212,'TSSKB' from dual union all
select 9213,'TSSKB' from dual union all
select 9214,'RVSN' from dual union all
select 9215,'RVSN' from dual;
insert into SATELLITE_ORG
select 9216,'TSSKB' from dual union all
select 9217,'TSSKB' from dual union all
select 9218,'TSSKB' from dual union all
select 9219,'TSSKB' from dual union all
select 922,'NRO' from dual union all
select 9220,'TSSKB' from dual union all
select 9221,'TSSKB' from dual union all
select 9222,'TSSKB' from dual union all
select 9223,'TSSKB' from dual union all
select 9224,'TSSKB' from dual union all
select 9225,'TSSKB' from dual union all
select 9226,'TSSKB' from dual union all
select 9227,'TSSKB' from dual union all
select 9228,'TSSKB' from dual union all
select 9229,'TSSKB' from dual union all
select 923,'JPL' from dual union all
select 923,'NASA' from dual union all
select 9230,'TSSKB' from dual union all
select 9231,'TSSKB' from dual union all
select 9232,'TSSKB' from dual union all
select 9233,'TSSKB' from dual union all
select 9234,'TSSKB' from dual union all
select 9235,'TSSKB' from dual union all
select 9236,'TSSKB' from dual union all
select 9237,'TSSKB' from dual union all
select 9238,'TSSKB' from dual union all
select 9239,'TSSKB' from dual union all
select 924,'NASA' from dual union all
select 9240,'TSSKB' from dual union all
select 9241,'TSSKB' from dual union all
select 9242,'TSSKB' from dual union all
select 9243,'TSSKB' from dual union all
select 9244,'TSSKB' from dual union all
select 9245,'TSSKB' from dual union all
select 9246,'TSSKB' from dual union all
select 9247,'TSSKB' from dual union all
select 9248,'TSSKB' from dual union all
select 9249,'TSSKB' from dual union all
select 925,'USAF' from dual union all
select 9250,'TSSKB' from dual union all
select 9251,'TSSKB' from dual union all
select 9252,'TSSKB' from dual union all
select 9253,'TSSKB' from dual union all
select 9254,'TSSKB' from dual union all
select 9255,'TSSKB' from dual union all
select 9256,'TSSKB' from dual union all
select 9257,'TSSKB' from dual union all
select 9258,'TSSKB' from dual union all
select 9259,'TSSKB' from dual union all
select 926,'USAF' from dual union all
select 9260,'TSSKB' from dual union all
select 9261,'TSSKB' from dual union all
select 9262,'TSSKB' from dual union all
select 9263,'TSSKB' from dual union all
select 9264,'TSSKB' from dual union all
select 9265,'TSSKB' from dual union all
select 9266,'TSSKB' from dual union all
select 9267,'VS' from dual union all
select 9268,'VS' from dual union all
select 9269,'RVSN' from dual union all
select 927,'USAF' from dual union all
select 9270,'NRO' from dual union all
select 9271,'USAF' from dual union all
select 9272,'IKI' from dual union all
select 9272,'NPOL' from dual union all
select 9273,'RVSN' from dual union all
select 9274,'RVSN' from dual union all
select 9275,'RVSN' from dual union all
select 9276,'RVSN' from dual union all
select 9277,'RVSN' from dual union all
select 9278,'RVSN' from dual union all
select 9279,'RVSN' from dual union all
select 928,'MO' from dual union all
select 9280,'RVSN' from dual union all
select 9281,'RVSN' from dual union all
select 9282,'NASA' from dual union all
select 9283,'NASA' from dual union all
select 9284,'NASA' from dual union all
select 9285,'NASA' from dual union all
select 9286,'NASA' from dual union all
select 9287,'NASA' from dual union all
select 9288,'NASA' from dual union all
select 9289,'NASA' from dual union all
select 929,'MO' from dual union all
select 9290,'NASA' from dual union all
select 9291,'NASA' from dual union all
select 9292,'NASA' from dual union all
select 9293,'NASA' from dual union all
select 9294,'NASA' from dual union all
select 9295,'NASA' from dual union all
select 9296,'NASA' from dual union all
select 9297,'NASA' from dual union all
select 9298,'NASA' from dual union all
select 9299,'NASA' from dual union all
select 93,'OKB1' from dual union all
select 930,'CIA' from dual union all
select 930,'NRO' from dual union all
select 9300,'NASA' from dual union all
select 9301,'NASA' from dual union all
select 9302,'NASA' from dual;
insert into SATELLITE_ORG
select 9303,'NASA' from dual union all
select 9304,'NASA' from dual union all
select 9305,'NASA' from dual union all
select 9306,'NASA' from dual union all
select 9307,'NASA' from dual union all
select 9308,'NASA' from dual union all
select 9309,'NASA' from dual union all
select 931,'NASA' from dual union all
select 9310,'NASA' from dual union all
select 9311,'NASA' from dual union all
select 9312,'NASA' from dual union all
select 9313,'NASA' from dual union all
select 9314,'NASA' from dual union all
select 9315,'NASA' from dual union all
select 9316,'NASA' from dual union all
select 9317,'NASA' from dual union all
select 9318,'NASA' from dual union all
select 9319,'NASA' from dual union all
select 932,'NASA' from dual union all
select 9320,'NASA' from dual union all
select 9321,'NASA' from dual union all
select 9322,'NASA' from dual union all
select 9323,'USAF' from dual union all
select 9324,'NASA' from dual union all
select 9325,'NASA' from dual union all
select 9326,'NASA' from dual union all
select 9327,'NASA' from dual union all
select 9328,'TSSKB' from dual union all
select 9329,'NASA' from dual union all
select 933,'NASA' from dual union all
select 9330,'NASA' from dual union all
select 9331,'TSSKB' from dual union all
select 9332,'TSSKB' from dual union all
select 9333,'TSSKB' from dual union all
select 9334,'TSSKB' from dual union all
select 9335,'TSSKB' from dual union all
select 9336,'TSSKB' from dual union all
select 9337,'TSSKB' from dual union all
select 9338,'TSSKB' from dual union all
select 9339,'TSSKB' from dual union all
select 934,'NASA' from dual union all
select 9340,'TSSKB' from dual union all
select 9341,'TSSKB' from dual union all
select 9342,'TSSKB' from dual union all
select 9343,'TSSKB' from dual union all
select 9344,'TSSKB' from dual union all
select 9345,'TSSKB' from dual union all
select 9346,'TSSKB' from dual union all
select 9347,'TSSKB' from dual union all
select 9348,'TSSKB' from dual union all
select 9349,'TSSKB' from dual union all
select 935,'NASA' from dual union all
select 9350,'TSSKB' from dual union all
select 9351,'TSSKB' from dual union all
select 9352,'TSSKB' from dual union all
select 9353,'TSSKB' from dual union all
select 9354,'TSSKB' from dual union all
select 9355,'TSSKB' from dual union all
select 9356,'TSSKB' from dual union all
select 9357,'TSSKB' from dual union all
select 9358,'TSSKB' from dual union all
select 9359,'TSSKB' from dual union all
select 936,'NASA' from dual union all
select 9360,'TSSKB' from dual union all
select 9361,'TSSKB' from dual union all
select 9362,'TSSKB' from dual union all
select 9363,'TSSKB' from dual union all
select 9364,'TSSKB' from dual union all
select 9365,'TSSKB' from dual union all
select 9366,'TSSKB' from dual union all
select 9367,'TSSKB' from dual union all
select 9368,'TSSKB' from dual union all
select 9369,'TSSKB' from dual union all
select 937,'NASA' from dual union all
select 9370,'TSSKB' from dual union all
select 9371,'TSSKB' from dual union all
select 9372,'TSSKB' from dual union all
select 9373,'TSSKB' from dual union all
select 9374,'TSSKB' from dual union all
select 9375,'RVSN' from dual union all
select 9376,'RVSN' from dual union all
select 9377,'RVSN' from dual union all
select 9378,'TSSKB' from dual union all
select 9379,'TSSKB' from dual union all
select 938,'JPL' from dual union all
select 938,'NASA' from dual union all
select 9380,'TSSKB' from dual union all
select 9381,'TSSKB' from dual union all
select 9382,'PVO' from dual union all
select 9383,'RVSN' from dual union all
select 9384,'IKI' from dual union all
select 9384,'NPOL' from dual union all
select 9385,'RVSN' from dual union all
select 9386,'RVSN' from dual union all
select 9387,'PVO' from dual union all
select 9388,'RVSN' from dual union all
select 9389,'VS' from dual union all
select 939,'NASA' from dual union all
select 9390,'RVSN' from dual union all
select 9391,'RVSN' from dual;
insert into SATELLITE_ORG
select 9392,'RVSN' from dual union all
select 9393,'RVSN' from dual union all
select 9394,'MAI' from dual union all
select 9395,'MAI' from dual union all
select 9396,'PVO' from dual union all
select 9397,'PVO' from dual union all
select 9398,'RVSN' from dual union all
select 9399,'RVSN' from dual union all
select 94,'OKB1' from dual union all
select 940,'NASA' from dual union all
select 9400,'RVSN' from dual union all
select 9401,'USN' from dual union all
select 9402,'RVSN' from dual union all
select 9403,'USN' from dual union all
select 9404,'NASA' from dual union all
select 9405,'RVSN' from dual union all
select 9406,'RVSN' from dual union all
select 9407,'RVSN' from dual union all
select 9408,'NASA' from dual union all
select 9409,'USN' from dual union all
select 941,'NASA' from dual union all
select 9410,'USN' from dual union all
select 9411,'RVSN' from dual union all
select 9412,'RVSN' from dual union all
select 9413,'RVSN' from dual union all
select 9414,'RVSN' from dual union all
select 9415,'USAF' from dual union all
select 9416,'RVSN' from dual union all
select 9417,'RVSN' from dual union all
select 9418,'RVSN' from dual union all
select 9419,'USAF' from dual union all
select 942,'NASA' from dual union all
select 9420,'USAF' from dual union all
select 9421,'NPOE' from dual union all
select 9422,'RVSN' from dual union all
select 9423,'RVSN' from dual union all
select 9424,'RVSN' from dual union all
select 9425,'NPOE' from dual union all
select 9426,'NRO' from dual union all
select 9427,'USAF' from dual union all
select 9428,'PVO' from dual union all
select 9429,'PVO' from dual union all
select 943,'RVSN' from dual union all
select 9430,'PVO' from dual union all
select 9431,'PVO' from dual union all
select 9432,'PVO' from dual union all
select 9433,'VTS' from dual union all
select 9434,'RVSN' from dual union all
select 9435,'RVSN' from dual union all
select 9436,'RVSN' from dual union all
select 9437,'RVSN' from dual union all
select 9438,'PVO' from dual union all
select 9439,'RVSN' from dual union all
select 944,'RVSN' from dual union all
select 9440,'RVSN' from dual union all
select 9441,'RVSN' from dual union all
select 9442,'RVSN' from dual union all
select 9443,'RVSN' from dual union all
select 9444,'RVSN' from dual union all
select 9445,'RVSN' from dual union all
select 9446,'RVSN' from dual union all
select 9447,'RVSN' from dual union all
select 9448,'TSSKB' from dual union all
select 9449,'RVSN' from dual union all
select 945,'OKB1' from dual union all
select 9450,'RVSN' from dual union all
select 9451,'RVSN' from dual union all
select 9452,'NASA' from dual union all
select 9453,'NASA' from dual union all
select 9454,'NASA' from dual union all
select 9455,'NASA' from dual union all
select 9456,'NASA' from dual union all
select 9457,'NASA' from dual union all
select 9458,'NASA' from dual union all
select 9459,'NASA' from dual union all
select 946,'NRO' from dual union all
select 9460,'NASA' from dual union all
select 9461,'NASA' from dual union all
select 9462,'NASA' from dual union all
select 9463,'NASA' from dual union all
select 9464,'NASA' from dual union all
select 9465,'NASA' from dual union all
select 9466,'NASA' from dual union all
select 9467,'NASA' from dual union all
select 9468,'NASA' from dual union all
select 9469,'NASA' from dual union all
select 947,'OKB586' from dual union all
select 9470,'RVSN' from dual union all
select 9471,'RVSN' from dual union all
select 9472,'RVSN' from dual union all
select 9473,'RVSN' from dual union all
select 9474,'USAF' from dual union all
select 9475,'PVO' from dual union all
select 9476,'PVO' from dual union all
select 9477,'NPOE' from dual union all
select 9478,'COM' from dual union all
select 9479,'NASA' from dual union all
select 948,'RVSN' from dual union all
select 9480,'RVSN' from dual union all
select 9481,'GMS' from dual;
insert into SATELLITE_ORG
select 9481,'RVSN' from dual union all
select 9482,'RVSN' from dual union all
select 9483,'USAF' from dual union all
select 9484,'USAF' from dual union all
select 9485,'PVO' from dual union all
select 9486,'VMF' from dual union all
select 9487,'PVO' from dual union all
select 9488,'PVO' from dual union all
select 9489,'PVO' from dual union all
select 949,'USAF' from dual union all
select 9490,'PVO' from dual union all
select 9491,'RVSN' from dual union all
select 9492,'RVSN' from dual union all
select 9493,'NASA' from dual union all
select 9494,'VMF' from dual union all
select 9495,'PVO' from dual union all
select 9496,'RVSN' from dual union all
select 9497,'RVSN' from dual union all
select 9498,'NASA' from dual union all
select 9499,'RVSN' from dual union all
select 95,'OKB1' from dual union all
select 950,'OKB586' from dual union all
select 9500,'RVSN' from dual union all
select 9501,'RVSN' from dual union all
select 9502,'RVSN' from dual union all
select 9503,'MSVYAZ' from dual union all
select 9504,'RVSN' from dual union all
select 9505,'RVSN' from dual union all
select 9506,'RVSN' from dual union all
select 9507,'PVO' from dual union all
select 9508,'PVO' from dual union all
select 9509,'VMF' from dual union all
select 951,'JPL' from dual union all
select 951,'NASA' from dual union all
select 9510,'RVSN' from dual union all
select 9511,'PVO' from dual union all
select 9512,'PVO' from dual union all
select 9513,'PVO' from dual union all
select 9514,'PVO' from dual union all
select 9515,'RVSN' from dual union all
select 9516,'RVSN' from dual union all
select 9517,'RVSN' from dual union all
select 9518,'USAF' from dual union all
select 9519,'RVSN' from dual union all
select 952,'OKB586' from dual union all
select 9520,'RVSN' from dual union all
select 9521,'RVSN' from dual union all
select 9522,'RVSN' from dual union all
select 9523,'RVSN' from dual union all
select 9524,'RVSN' from dual union all
select 9525,'RVSN' from dual union all
select 9526,'RVSN' from dual union all
select 9527,'RVSN' from dual union all
select 9528,'RVSN' from dual union all
select 9529,'RVSN' from dual union all
select 953,'USAF' from dual union all
select 9530,'NASA' from dual union all
select 9531,'VMF' from dual union all
select 9532,'RVSN' from dual union all
select 9533,'RVSN' from dual union all
select 9534,'RVSN' from dual union all
select 9535,'RVSN' from dual union all
select 9536,'RVSN' from dual union all
select 9537,'VMF' from dual union all
select 9538,'VMF' from dual union all
select 9539,'VMF' from dual union all
select 954,'OKB586' from dual union all
select 9540,'RVSN' from dual union all
select 9541,'RVSN' from dual union all
select 9542,'NASA' from dual union all
select 9543,'NASA' from dual union all
select 9544,'NASA' from dual union all
select 9545,'NASA' from dual union all
select 9546,'NASA' from dual union all
select 9547,'NASA' from dual union all
select 9548,'NASA' from dual union all
select 9549,'NASA' from dual union all
select 955,'OKB586' from dual union all
select 9550,'NASA' from dual union all
select 9551,'RVSN' from dual union all
select 9552,'RVSN' from dual union all
select 9553,'RVSN' from dual union all
select 9554,'RVSN' from dual union all
select 9555,'RVSN' from dual union all
select 9556,'RVSN' from dual union all
select 9557,'IKI' from dual union all
select 9557,'NPOL' from dual union all
select 9558,'RVSN' from dual union all
select 9559,'RVSN' from dual union all
select 956,'USN' from dual union all
select 9560,'RVSN' from dual union all
select 9561,'VMF' from dual union all
select 9562,'RVSN' from dual union all
select 9563,'VMF' from dual union all
select 9564,'NPOE' from dual union all
select 9565,'RVSN' from dual union all
select 9566,'RVSN' from dual union all
select 9567,'NASA' from dual union all
select 9568,'NASA' from dual union all
select 9569,'RVSN' from dual;
insert into SATELLITE_ORG
select 957,'CNR' from dual union all
select 9570,'NASA' from dual union all
select 9571,'NASA' from dual union all
select 9572,'RVSN' from dual union all
select 9573,'VS' from dual union all
select 9574,'RVSN' from dual union all
select 9575,'RVSN' from dual union all
select 9576,'RVSN' from dual union all
select 9577,'VS' from dual union all
select 9578,'RVSN' from dual union all
select 9579,'RVSN' from dual union all
select 958,'NASA' from dual union all
select 9580,'RVSN' from dual union all
select 9581,'RVSN' from dual union all
select 9582,'RVSN' from dual union all
select 9583,'RVSN' from dual union all
select 9584,'RVSN' from dual union all
select 9585,'RVSN' from dual union all
select 9586,'RVSN' from dual union all
select 9587,'MAI' from dual union all
select 9588,'RVSN' from dual union all
select 9589,'RVSN' from dual union all
select 959,'USN' from dual union all
select 9590,'RVSN' from dual union all
select 9591,'RVSN' from dual union all
select 9592,'RVSN' from dual union all
select 9593,'RVSN' from dual union all
select 9594,'RVSN' from dual union all
select 9595,'RVSN' from dual union all
select 9596,'MAI' from dual union all
select 9597,'MAI' from dual union all
select 9598,'RVSN' from dual union all
select 9599,'RVSN' from dual union all
select 96,'RVSN' from dual union all
select 960,'NASA' from dual union all
select 9600,'RVSN' from dual union all
select 9601,'PVO' from dual union all
select 9602,'RVSN' from dual union all
select 9603,'RVSN' from dual union all
select 9604,'RVSN' from dual union all
select 9605,'PVO' from dual union all
select 9606,'KBSAL' from dual union all
select 9607,'KBSAL' from dual union all
select 9608,'RVSN' from dual union all
select 9609,'KBSAL' from dual union all
select 961,'CIA' from dual union all
select 961,'NRO' from dual union all
select 9610,'OKB10' from dual union all
select 9611,'KBSAL' from dual union all
select 9612,'KBSAL' from dual union all
select 9613,'RVSN' from dual union all
select 9614,'RVSN' from dual union all
select 9615,'PVO' from dual union all
select 9616,'NASA' from dual union all
select 9617,'NASA' from dual union all
select 9618,'NASA' from dual union all
select 9619,'NASA' from dual union all
select 962,'CNR' from dual union all
select 962,'NASA' from dual union all
select 9620,'NASA' from dual union all
select 9621,'NASA' from dual union all
select 9622,'NASA' from dual union all
select 9623,'RVSN' from dual union all
select 9624,'RVSN' from dual union all
select 9625,'RVSN' from dual union all
select 9626,'PVO' from dual union all
select 9627,'CIA' from dual union all
select 9627,'NRO' from dual union all
select 9628,'USAF' from dual union all
select 9629,'NRO' from dual union all
select 963,'NASA' from dual union all
select 9630,'NRO' from dual union all
select 9631,'VMF' from dual union all
select 9632,'VMF' from dual union all
select 9633,'VMF' from dual union all
select 9634,'PVO' from dual union all
select 9635,'MSVYAZ' from dual union all
select 9635,'OKB10' from dual union all
select 9636,'RVSN' from dual union all
select 9637,'VMF' from dual union all
select 9638,'RVSN' from dual union all
select 9639,'RVSN' from dual union all
select 964,'NRO' from dual union all
select 9640,'RVSN' from dual union all
select 9641,'RVSN' from dual union all
select 9642,'RVSN' from dual union all
select 9643,'PVO' from dual union all
select 9644,'PVO' from dual union all
select 9645,'RVSN' from dual union all
select 9646,'RVSN' from dual union all
select 9647,'RVSN' from dual union all
select 9648,'PVO' from dual union all
select 9649,'PVO' from dual union all
select 965,'USN' from dual union all
select 9650,'PVO' from dual union all
select 9651,'PVO' from dual union all
select 9652,'PVO' from dual union all
select 9653,'PVO' from dual union all
select 9654,'PVO' from dual union all
select 9655,'PVO' from dual;
insert into SATELLITE_ORG
select 9656,'PVO' from dual union all
select 9657,'PVO' from dual union all
select 9658,'RVSN' from dual union all
select 9659,'RVSN' from dual union all
select 966,'USN' from dual union all
select 9660,'RVSN' from dual union all
select 9661,'GMS' from dual union all
select 9661,'RVSN' from dual union all
select 9662,'RVSN' from dual union all
select 9663,'RVSN' from dual union all
select 9664,'RVSN' from dual union all
select 9665,'VS' from dual union all
select 9666,'NASA' from dual union all
select 9667,'NASA' from dual union all
select 9668,'NASA' from dual union all
select 9669,'NASA' from dual union all
select 967,'USN' from dual union all
select 9670,'NASA' from dual union all
select 9671,'NASA' from dual union all
select 9672,'NASA' from dual union all
select 9673,'NASA' from dual union all
select 9674,'NASA' from dual union all
select 9675,'NASA' from dual union all
select 9676,'NASA' from dual union all
select 9677,'NASA' from dual union all
select 9678,'NASA' from dual union all
select 9679,'NASA' from dual union all
select 968,'MO' from dual union all
select 9680,'NASA' from dual union all
select 9681,'NASA' from dual union all
select 9682,'NASA' from dual union all
select 9683,'NASA' from dual union all
select 9684,'NASA' from dual union all
select 9685,'NASA' from dual union all
select 9686,'NASA' from dual union all
select 9687,'NASA' from dual union all
select 9688,'NASA' from dual union all
select 9689,'NASA' from dual union all
select 969,'RVSN' from dual union all
select 9690,'NASA' from dual union all
select 9691,'NASA' from dual union all
select 9692,'RVSN' from dual union all
select 9693,'VS' from dual union all
select 9694,'USAF' from dual union all
select 9695,'NASA' from dual union all
select 9696,'NASA' from dual union all
select 9697,'NASA' from dual union all
select 9698,'NASA' from dual union all
select 9699,'NASA' from dual union all
select 97,'OKB1' from dual union all
select 970,'MO' from dual union all
select 9700,'NASA' from dual union all
select 9701,'NASA' from dual union all
select 9702,'NASA' from dual union all
select 9703,'NASA' from dual union all
select 9704,'RVSN' from dual union all
select 9705,'NASA' from dual union all
select 9706,'NASA' from dual union all
select 9707,'NASA' from dual union all
select 9708,'NASA' from dual union all
select 9709,'NASA' from dual union all
select 971,'MO' from dual union all
select 9710,'NASA' from dual union all
select 9711,'NASA' from dual union all
select 9712,'NASA' from dual union all
select 9713,'NASA' from dual union all
select 9714,'NASA' from dual union all
select 9715,'NASA' from dual union all
select 9716,'NASA' from dual union all
select 9717,'NASA' from dual union all
select 9718,'VS' from dual union all
select 9719,'PVO' from dual union all
select 972,'CIA' from dual union all
select 972,'NRO' from dual union all
select 9720,'PVO' from dual union all
select 9721,'PVO' from dual union all
select 9722,'PVO' from dual union all
select 9723,'PVO' from dual union all
select 9724,'PVO' from dual union all
select 9725,'PVO' from dual union all
select 9726,'NASA' from dual union all
select 9727,'NASA' from dual union all
select 9728,'NASA' from dual union all
select 9729,'NASA' from dual union all
select 973,'USAF' from dual union all
select 9730,'NASA' from dual union all
select 9731,'VS' from dual union all
select 9732,'RVSN' from dual union all
select 9733,'RVSN' from dual union all
select 9734,'RVSN' from dual union all
select 9735,'RVSN' from dual union all
select 9736,'RVSN' from dual union all
select 9737,'VMF' from dual union all
select 9738,'RVSN' from dual union all
select 9739,'NASA' from dual union all
select 974,'USAF' from dual union all
select 9740,'NASA' from dual union all
select 9741,'NASA' from dual union all
select 9742,'NASA' from dual union all
select 9743,'NASA' from dual;
insert into SATELLITE_ORG
select 9744,'NASA' from dual union all
select 9745,'NASA' from dual union all
select 9746,'NASA' from dual union all
select 9747,'NASA' from dual union all
select 9748,'NASA' from dual union all
select 9749,'NASA' from dual union all
select 975,'USAF' from dual union all
select 9750,'NASA' from dual union all
select 9751,'NASA' from dual union all
select 9752,'NASA' from dual union all
select 9753,'NASA' from dual union all
select 9754,'NASA' from dual union all
select 9755,'NASA' from dual union all
select 9756,'NASA' from dual union all
select 9757,'NASA' from dual union all
select 9758,'NASA' from dual union all
select 9759,'NASA' from dual union all
select 976,'NASA' from dual union all
select 9760,'NASA' from dual union all
select 9761,'NASA' from dual union all
select 9762,'NASA' from dual union all
select 9763,'NASA' from dual union all
select 9764,'NASA' from dual union all
select 9765,'NASA' from dual union all
select 9766,'NASA' from dual union all
select 9767,'NASA' from dual union all
select 9768,'NASA' from dual union all
select 9769,'NASA' from dual union all
select 977,'NASA' from dual union all
select 9770,'NASA' from dual union all
select 9771,'NASA' from dual union all
select 9772,'NASA' from dual union all
select 9773,'NASA' from dual union all
select 9774,'NASA' from dual union all
select 9775,'NASA' from dual union all
select 9776,'NASA' from dual union all
select 9777,'NASA' from dual union all
select 9778,'NASA' from dual union all
select 9779,'NASA' from dual union all
select 978,'NASA' from dual union all
select 9780,'NASA' from dual union all
select 9781,'NASA' from dual union all
select 9782,'NASA' from dual union all
select 9783,'NASA' from dual union all
select 9784,'NASA' from dual union all
select 9785,'NATO' from dual union all
select 9786,'NASA' from dual union all
select 9787,'NASA' from dual union all
select 9788,'NASA' from dual union all
select 9789,'VS' from dual union all
select 979,'NASA' from dual union all
select 9790,'VS' from dual union all
select 9791,'VS' from dual union all
select 9792,'VS' from dual union all
select 9793,'VS' from dual union all
select 9794,'RVSN' from dual union all
select 9795,'NASA' from dual union all
select 9796,'USN' from dual union all
select 9797,'PVO' from dual union all
select 9798,'PVO' from dual union all
select 9799,'PVO' from dual union all
select 98,'NASA' from dual union all
select 980,'NRO' from dual union all
select 9800,'RVSN' from dual union all
select 9801,'PVO' from dual union all
select 9802,'RVSN' from dual union all
select 9803,'USAF' from dual union all
select 9804,'NPOE' from dual union all
select 9805,'RVSN' from dual union all
select 9806,'NPOE' from dual union all
select 9807,'PVO' from dual union all
select 9808,'USAF' from dual union all
select 9809,'NASA' from dual union all
select 981,'USAF' from dual union all
select 9810,'NASA' from dual union all
select 9811,'NASA' from dual union all
select 9812,'RVSN' from dual union all
select 9813,'RVSN' from dual union all
select 9814,'RVSN' from dual union all
select 9815,'PVO' from dual union all
select 9816,'PVO' from dual union all
select 9817,'PVO' from dual union all
select 9818,'PVO' from dual union all
select 9819,'PVO' from dual union all
select 982,'CIA' from dual union all
select 982,'NRO' from dual union all
select 9820,'PVO' from dual union all
select 9821,'PVO' from dual union all
select 9822,'PVO' from dual union all
select 9823,'PVO' from dual union all
select 9824,'PVO' from dual union all
select 9825,'PVO' from dual union all
select 9826,'PVO' from dual union all
select 9827,'PVO' from dual union all
select 9828,'PVO' from dual union all
select 9829,'RVSN' from dual union all
select 983,'YUZH' from dual union all
select 9830,'RVSN' from dual union all
select 9831,'RVSN' from dual union all
select 9832,'RVSN' from dual;
insert into SATELLITE_ORG
select 9833,'YUZH' from dual union all
select 9834,'YUZH' from dual union all
select 9835,'NASA' from dual union all
select 9836,'VS' from dual union all
select 9837,'VS' from dual union all
select 9838,'VS' from dual union all
select 9839,'RVSN' from dual union all
select 984,'RVSN' from dual union all
select 9840,'USAF' from dual union all
select 9841,'ISAS' from dual union all
select 9842,'ISAS' from dual union all
select 9843,'ISAS' from dual union all
select 9844,'ISAS' from dual union all
select 9845,'RVSN' from dual union all
select 9846,'VMF' from dual union all
select 9847,'RVSN' from dual union all
select 9848,'RVSN' from dual union all
select 9849,'RVSN' from dual union all
select 985,'YUZH' from dual union all
select 9850,'RVSN' from dual union all
select 9851,'RVSN' from dual union all
select 9852,'NASDA' from dual union all
select 9853,'VS' from dual union all
select 9854,'RVSN' from dual union all
select 9855,'USAF' from dual union all
select 9856,'USAF' from dual union all
select 9857,'RVSN' from dual union all
select 9858,'RVSN' from dual union all
select 9859,'NASDA' from dual union all
select 986,'YUZH' from dual union all
select 9860,'RVSN' from dual union all
select 9861,'RVSN' from dual union all
select 9862,'PER' from dual union all
select 9863,'NRO' from dual union all
select 9864,'NASA' from dual union all
select 9865,'RVSN' from dual union all
select 9866,'NASA' from dual union all
select 9867,'NASA' from dual union all
select 9868,'NASA' from dual union all
select 9869,'RVSN' from dual union all
select 987,'NASA' from dual union all
select 9870,'RVSN' from dual union all
select 9871,'RVSN' from dual union all
select 9872,'RVSN' from dual union all
select 9873,'RVSN' from dual union all
select 9874,'RVSN' from dual union all
select 9875,'RVSN' from dual union all
select 9876,'RVSN' from dual union all
select 9877,'RVSN' from dual union all
select 9878,'RVSN' from dual union all
select 9879,'RVSN' from dual union all
select 988,'NASA' from dual union all
select 9880,'RVSN' from dual union all
select 9881,'RVSN' from dual union all
select 9882,'RVSN' from dual union all
select 9883,'VS' from dual union all
select 9884,'RVSN' from dual union all
select 9885,'RVSN' from dual union all
select 9886,'RVSN' from dual union all
select 9887,'TSSKB' from dual union all
select 9888,'PVO' from dual union all
select 9889,'PVO' from dual union all
select 989,'MO' from dual union all
select 9890,'PVO' from dual union all
select 9891,'PVO' from dual union all
select 9892,'PVO' from dual union all
select 9893,'PVO' from dual union all
select 9894,'PVO' from dual union all
select 9895,'PVO' from dual union all
select 9896,'PVO' from dual union all
select 9897,'PVO' from dual union all
select 9898,'IK' from dual union all
select 9898,'MOM' from dual union all
select 9899,'RVSN' from dual union all
select 99,'USN' from dual union all
select 990,'MO' from dual union all
select 9900,'VS' from dual union all
select 9901,'PVO' from dual union all
select 9902,'PVO' from dual union all
select 9903,'GMS' from dual union all
select 9903,'RVSN' from dual union all
select 9904,'RVSN' from dual union all
select 9905,'PVO' from dual union all
select 9906,'RVSN' from dual union all
select 9907,'RVSN' from dual union all
select 9908,'RVSN' from dual union all
select 9909,'RVSN' from dual union all
select 991,'MO' from dual union all
select 9910,'RVSN' from dual union all
select 9911,'PVO' from dual union all
select 9912,'RVSN' from dual union all
select 9913,'RVSN' from dual union all
select 9914,'PVO' from dual union all
select 9915,'PVO' from dual union all
select 9916,'PVO' from dual union all
select 9917,'PVO' from dual union all
select 9918,'PVO' from dual union all
select 9919,'PVO' from dual union all
select 992,'MO' from dual union all
select 9920,'PVO' from dual;
insert into SATELLITE_ORG
select 9921,'RVSN' from dual union all
select 9922,'PVO' from dual union all
select 9923,'PVO' from dual union all
select 9924,'PVO' from dual union all
select 9925,'PVO' from dual union all
select 9926,'PVO' from dual union all
select 9927,'RVSN' from dual union all
select 9928,'RVSN' from dual union all
select 9929,'RVSN' from dual union all
select 993,'MO' from dual union all
select 9930,'RVSN' from dual union all
select 9931,'ESA' from dual union all
select 9932,'NASA' from dual union all
select 9933,'NASA' from dual union all
select 9934,'NASA' from dual union all
select 9935,'RVSN' from dual union all
select 9936,'RVSN' from dual union all
select 9937,'TSSKB' from dual union all
select 9938,'RVSN' from dual union all
select 9939,'RVSN' from dual union all
select 994,'MO' from dual union all
select 9940,'TSSKB' from dual union all
select 9941,'MOM' from dual union all
select 9941,'MSVYAZ' from dual union all
select 9942,'RVSN' from dual union all
select 9943,'RVSN' from dual union all
select 9944,'RVSN' from dual union all
select 9945,'RVSN' from dual union all
select 9946,'RVSN' from dual union all
select 9947,'RVSN' from dual union all
select 9948,'NASA' from dual union all
select 9949,'NASA' from dual union all
select 995,'MO' from dual union all
select 9950,'NASA' from dual union all
select 9951,'PVO' from dual union all
select 9952,'PVO' from dual union all
select 9953,'PVO' from dual union all
select 9954,'PVO' from dual union all
select 9955,'PVO' from dual union all
select 9956,'PVO' from dual union all
select 9957,'PVO' from dual union all
select 9958,'PVO' from dual union all
select 9959,'PVO' from dual union all
select 996,'MO' from dual union all
select 9960,'PVO' from dual union all
select 9961,'PVO' from dual union all
select 9962,'PVO' from dual union all
select 9963,'PVO' from dual union all
select 9964,'PVO' from dual union all
select 9965,'PVO' from dual union all
select 9966,'PVO' from dual union all
select 9967,'PVO' from dual union all
select 9968,'USAF' from dual union all
select 9969,'USAF' from dual union all
select 997,'MO' from dual union all
select 9970,'NASA' from dual union all
select 9971,'NASA' from dual union all
select 9972,'NASA' from dual union all
select 9973,'PVO' from dual union all
select 9974,'PVO' from dual union all
select 9975,'NASA' from dual union all
select 9976,'NASA' from dual union all
select 9977,'NASA' from dual union all
select 9978,'NASA' from dual union all
select 9979,'NASA' from dual union all
select 998,'MO' from dual union all
select 9980,'RVSN' from dual union all
select 9981,'ISAS' from dual union all
select 9982,'ISAS' from dual union all
select 9983,'ISAS' from dual union all
select 9984,'NASA' from dual union all
select 9985,'NASA' from dual union all
select 9986,'NASA' from dual union all
select 9987,'NASA' from dual union all
select 9988,'NASA' from dual union all
select 9989,'NASA' from dual union all
select 999,'MO' from dual union all
select 9990,'VS' from dual union all
select 9991,'VS' from dual union all
select 9992,'VS' from dual union all
select 9993,'VS' from dual union all
select 9994,'VS' from dual union all
select 9995,'MO' from dual union all
select 9996,'USN' from dual union all
select 9997,'NASA' from dual union all
select 9998,'NASA' from dual union all
select 9999,'PVO' from dual;


--------------------------------------------------------------------------------
-- ENGINE
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing ENGINE...');

CREATE TABLE "ENGINE" 
 (	"ENGINE_ID" NUMBER, 
	"ENGINE_NAME" VARCHAR2(21), 
	"ENGINE_FAMILY" VARCHAR2(21), 
	"ENGINE_ALT_NAME" VARCHAR2(13), 
	"FIRST_LAUNCH_YEAR" NUMBER, 
	"USAGE" VARCHAR2(20), 
	"MASS" NUMBER, 
	"IMPULSE" NUMBER, 
	"THRUST" NUMBER, 
	"SPECIFIC_IMPULSE" NUMBER, 
	"DURATION" NUMBER, 
	"CHAMBER_COUNT" NUMBER, 
	 CONSTRAINT "ENGINE_PK" PRIMARY KEY ("ENGINE_ID")
USING INDEX  ENABLE
 ) ;

insert into ENGINE
select 1,'# Mk 7, 23, 25','','',null,'',null,null,null,null,null,null from dual union all
select 10,'11D458','11D458','',1977,'FGB DPS',null,null,.39,252,null,1 from dual union all
select 100,'9KS11000','BB3','',null,'BB3',265,440,49,220,9,1 from dual union all
select 1000,'Sirocco','LPC S-10','',1960,'Sirocco',65,111,14.8,null,null,1 from dual union all
select 1001,'Skybolt','','',null,'Skybolt',2500,null,164,245,40.7,1 from dual union all
select 1002,'Skybolt St2','','Skybolt.St2',1950,'Skybolt [2]',1230,null,81.8,245,41.3,1 from dual union all
select 1003,'Snipe','','',1974,'INTA 300 [2]',190,261,16.3,null,16,1 from dual union all
select 1004,'Sonda 1','','',1965,'Sonda 1',27.3,null,4.2,null,null,1 from dual union all
select 1005,'Sonda 1 Booster','','',1965,'Sonda 1',27.5,null,27,null,null,1 from dual union all
select 1006,'SpaceLoft','','',2006,'SpaceLoft XL',200,null,36.6,null,12,1 from dual union all
select 1007,'Spectre','','',null,'Blue Steel',null,null,35.6,null,null,1 from dual union all
select 1008,'Sprite','','',null,'RATO',null,null,22.2,null,null,1 from dual union all
select 1009,'Stable Super Loki','','',1973,'Stable Super Loki',27,44,21.2,228.7,2.1,1 from dual union all
select 101,'9KS1660','BB','',1969,'BB6',43,66,7.4,240,9,1 from dual union all
select 1010,'Star 10','','TE-M-195',1959,'SRV Retro Mk I',17,30,33.5,251,9,1 from dual union all
select 1011,'Star 12','','TE-M-236',1960,'SRV Mk IV Retro',27.8,46,5.6,252,7.5,1 from dual union all
select 1012,'Star 12A','','TE-M-236-3',1971,'Super SARV Retro',33.5,61,7.3,270,7.5,1 from dual union all
select 1013,'Star 13','','TE-M-458',null,'Star 13',36,84,4.1,273,21,1 from dual union all
select 1014,'Star 13A','','TE-M-516',null,'Star 13A',38,94,5.9,286.5,15.3,1 from dual union all
select 1015,'Star 13B','','TE-M-763',null,'Star 13B',47,116,7.6,285.7,14.8,1 from dual union all
select 1016,'Star 13C','','TE-M-345',null,'Star 13C',38,81,5,218,15,1 from dual union all
select 1017,'Star 13D','','TE-M-375',null,'Star 13D',35.3,76,5,223,15,1 from dual union all
select 1018,'Star 13E','','TE-M-385',null,'Star 13E',30.5,63,4.2,211,15,1 from dual union all
select 1019,'Star 13F','','TE-M-444',null,'Star 13F',40.1,94,6.3,240,15,1 from dual union all
select 102,'9M21','','',null,'Luna-M',2300,null,null,null,null,1 from dual union all
select 1020,'Star 17','','TE-M-479',null,'Star 17',79,198,11,286,17.6,1 from dual union all
select 1021,'Star 17A','','TE-M-521-5',null,'Star 17A',125.5,320,16,286.7,19.4,1 from dual union all
select 1022,'Star 20','Altair','TE-M-640-1',1970,'Scout G [4]',300,772,27.1,286.5,27.5,1 from dual union all
select 1023,'Star 24','','TE-M-604',1969,'Star 24',218,561,18.5,282.9,29.6,1 from dual union all
select 1024,'Star 24C','','TE-M-604-4',1978,'Star 24 IUE',239,614,21.5,282.3,28,1 from dual union all
select 1025,'Star 26','Star 26','TE-M-442',null,'Strypi IV',269,616,33.3,271,17.8,1 from dual union all
select 1026,'Star 26B','Star 26','TE-M-442-1',1974,'Burner 2A [2]',261,635,34.6,271.7,17.8,1 from dual union all
select 1027,'Star 26C','Star 26','TE-M-442-2',1980,'DOT',262,622,35,272.1,16.8,1 from dual union all
select 1028,'Star 27','','TE-M-616',1976,'Star 27',360.5,951,26.7,288,34.3,1 from dual union all
select 1029,'Star 30A','','TE-M-700-4',1980,'Star 30',492,1345,27,294.7,54,1 from dual union all
select 103,'9M723','','',null,'Iskander',3300,null,null,null,null,1 from dual union all
select 1030,'Star 30B','','TE-M-700-5',1980,'Star 30',537,1460,27,293,54,1 from dual union all
select 1031,'Star 30BP','','TE-M-700-20',1980,'Star 30BP',542,1460,27,292,54,1 from dual union all
select 1032,'Star 30C','','TE-M-700-18',1980,'Star 30C',629,1500,32.7,286.8,51,1 from dual union all
select 1033,'Star 30E','','TE-M-700-19',1980,'Star 30E',666,1788,35.3,290.5,49,1 from dual union all
select 1034,'Star 31','Antares','TE-M-762',null,'Scout G [3]',1392,3738,82.3,293.5,45,1 from dual union all
select 1035,'Star 37','TE-364','TE-M-364-1',1966,'Star 37',621,1585,null,260,null,1 from dual union all
select 1036,'Star 37B','TE-364','TE-M-364-2',1966,'Star 37B',718,1860,null,291,null,1 from dual union all
select 1037,'Star 37C','TE-364','TE-M-364-18',null,'Star 37C',1047,2708,null,285.5,null,1 from dual union all
select 1038,'Star 37D','TE-364','TE-M-364-3',null,'Star 37C',719,1860,45,266,null,1 from dual union all
select 1039,'Star 37E','TE-364','TE-M-364-4',null,'Star 37C',1123,2911,null,283.6,null,1 from dual union all
select 104,'9M76','','',null,'Temp',11900,null,null,null,null,1 from dual union all
select 1040,'Star 37F','TE-364','TE-M-364-19',null,'Star 37F',934,2445,null,286,null,1 from dual union all
select 1041,'Star 37FM','TE-364','TE-M-783',null,'Delta 7326 st 3',1147,3052,45.8,291.8,66.4,1 from dual union all
select 1042,'Star 37GV','TE-364','',null,'Delta 7326 st 3',1084,2825,87,290,49,1 from dual union all
select 1043,'Star 37N','TE-364','TE-M-364-14',null,'Star 37N',622,1591,null,290,null,1 from dual union all
select 1044,'Star 37S','TE-364','TE-M-364-15',null,'Star 37C',711,1870,null,287.3,null,1 from dual union all
select 1045,'Star 37XE','TE-364','TE-M-714-8',null,'Star 37XE',1401,null,56,289.6,66,1 from dual union all
select 1046,'Star 37XF','TE-364','TE-M-714-8',null,'Star 37XF',953,2340,56,290,66,1 from dual union all
select 1047,'Star 37XFP','TE-364','TE-M-714-16',1985,'Star 37XFP',955,2538,38,290,66,1 from dual union all
select 1048,'Star 48','','TE-M-711-3',1980,'Star 48',2113,null,67.2,286,88,1 from dual union all
select 1049,'Star 48B','','TE-M-711-17',1980,'STS PAM-D',2134,5673,67.2,286,84.1,1 from dual union all
select 105,'9M76B','','',null,'Temp-S',11900,null,null,null,null,1 from dual union all
select 1050,'Star 48B','','TE-M-711-18',1980,'Delta PAM-D',2140,5801,68.7,292.1,84.1,1 from dual union all
select 1051,'Star 48BV','','',null,'Conestoga',2168,5801,152,292,84.1,1 from dual union all
select 1052,'Star 48V','','TE-M-940-1',null,'Conestoga',2160,5801,68.8,292.1,84.1,1 from dual union all
select 1053,'Star 63D','','TU-936',1985,'PAM-D2',3500,9046,84.8,283,118,1 from dual union all
select 1054,'Star 63F','','TE-M-936-2',1993,'Star 63F',4591,12535,104.7,297.1,120,1 from dual union all
select 1055,'Start','Topol','',null,'Start-1 [4]',1000,null,100,295,207,1 from dual union all
select 1056,'Stentor 101','','',1950,'Blue Steel',null,null,112,null,null,1 from dual union all
select 1057,'Stonechat I','','',1969,'Stonechat',4780,8340,232,258.2,32.3,1 from dual union all
select 1058,'Stonechat II','','',1971,'Stonechat',5090,9610,242,293,32.8,1 from dual union all
select 1059,'Stromboli','','',1968,'Eridan [1]',942,1550,97,230,16,1 from dual union all
select 106,'9M79','','Tochka',1970,'Tochka',1530,null,null,null,null,1 from dual union all
select 1060,'Super Draco','','',2009,'Dragon',null,null,73,235,25,1 from dual union all
select 1061,'Super Draco','','',2014,'Dragon2',null,null,71,235,25,1 from dual union all
select 1062,'Super Loki','','',1968,'Super Loki',23,38,17.8,232,2.1,1 from dual union all
select 1063,'Sword 3KS5000','','',null,'Cleansweep III',42,60,16,null,3.8,1 from dual union all
select 1064,'T-22','','',1944,'Private A Booster',5,null,24,206,.2,1 from dual union all
select 1065,'T-40','','',1949,'TB1 [4]',62,89,13.3,null,6,1 from dual union all
select 1066,'T-41','Falcon','M-46',1949,'Falcon',12,null,null,null,null,1 from dual union all
select 1067,'T-55','','',1956,'Terrapin',28,null,21.4,null,1.3,1 from dual union all
select 1068,'T-7 LRE','','',1960,'T-7',1000,null,2,null,null,1 from dual union all
select 1069,'T-7 booster','','',null,'T-7',500,null,2,null,null,1 from dual union all
select 107,'9M79-1','','Tochka',1970,'Tochka-U',1530,null,null,null,null,1 from dual union all
select 1070,'T48','','',1954,'Nike T48 Ajax',900,658,193,null,3.4,1 from dual union all
select 1071,'TD-187','Bullpup A','',1950,'Bullpup',92,null,53,null,null,1 from dual union all
select 1072,'TD-2','','',2006,'TD-2',null,null,250,null,120,1 from dual union all
select 1073,'TD-232','Bullpup B','',1950,'Bullpup',255,null,53,null,null,1 from dual union all
select 1074,'TD-339','','',1966,'Surveyor Vernier',3,null,.46,287,null,1 from dual union all
select 1075,'TD1','','',1997,'TD1 [3]',252,null,18.3,250,27,1 from dual union all
select 1076,'TE-29-1A','Recruit','Recruit 1A',null,'Recruit',170,269,173,225,2.2,1 from dual union all
select 1077,'TE-307','Apache','',1961,'Apache',135,167,21.1,null,6,1 from dual union all
select 1078,'TE-388','Iroquois','',null,'Iroquois',100,null,null,null,null,1 from dual union all
select 1079,'TE-406','Tomahawk','Tomahawk',1965,'Tomahawk',220,417,53,null,9,1 from dual union all
select 108,'A-350 Booster','','',1968,'A-350 booster',15000,null,null,null,5,1 from dual union all
select 1080,'TE-416','Tomahawk','Tomahawk',1965,'Tomahawk',220,417,53,null,9,1 from dual union all
select 1081,'TE-473','Sandhawk','Sandhawk',null,'Sandhawk',600,1400,90,238,15,1 from dual union all
select 1082,'TE-M-380','LES Jettison','',1965,'Apollo LES Jettison',242,165,147,null,null,1 from dual union all
select 1083,'THAAD','','',2004,'THAAD',500,null,null,null,null,1 from dual union all
select 1084,'THAAD Proto','','',1995,'THAAD',500,null,null,null,null,1 from dual union all
select 1085,'TQS492-2','','',2011,'DFH-4 LAE',null,15000,.49,315.1,null,1 from dual union all
select 1086,'TR-1','','',1989,'TR-1',8558,32000,608,271.6,52,1 from dual union all
select 1087,'TR-1A','','',1989,'TR-1A',8558,32000,716,271.6,49,1 from dual union all
select 1088,'TR-201','','',null,'Delta 2910',null,null,41.9,301.3,null,1 from dual;
insert into ENGINE
select 1089,'TR-306','DM-LAE','',1991,'LM5000',null,null,.45,315,null,1 from dual union all
select 109,'A-350 Sustainer','5D22?','',null,'A-350 St2',null,null,131,null,40,1 from dual union all
select 1090,'TR-308','DM-LAE','',1999,'CXO',null,null,.47,322,3000,1 from dual union all
select 1091,'TR-500','SCAT','',2001,'Geolite',null,null,.04,null,null,1 from dual union all
select 1092,'TT-200','','',null,'TT-200',150,null,null,null,null,1 from dual union all
select 1093,'TT-210','','',null,'TT-210',150,null,null,null,null,1 from dual union all
select 1094,'TT-500','','',null,'TT-500',1300,null,116,null,null,1 from dual union all
select 1095,'TT-500-S2','','',null,'TT-500-S2',700,null,61,null,null,1 from dual union all
select 1096,'TU-122','Minuteman','M55E1',1962,'Minuteman 1B',23240,null,891,null,60,4 from dual union all
select 1097,'TU-122-ATK','Minuteman','M55E1',1962,'Minuteman 3 PRP',23240,null,891,null,60,4 from dual union all
select 1098,'TU-223','','M16E3',1955,'Mace Booster [0]',700,null,449,null,2.4,1 from dual union all
select 1099,'TU-289','Genie','Genie',1967,'Genie',300,null,162,null,null,1 from dual union all
select 11,'11D49','S5.23','',null,'S3M',null,null,157.5,303,335,1 from dual union all
select 110,'A-4','A-4','',1942,'A-4',null,null,312,203,68,1 from dual union all
select 1100,'TU-715','Genie','Ute',1971,'Ute',190,277,85,null,4.6,1 from dual union all
select 1101,'TU-716','Genie','Paiute',1972,'Paiute',308,520,186,null,3.7,1 from dual union all
select 1102,'TU-758','SAM-D','Malemute',1974,'Malemute (TX-516?)',619,1395,55.1,281.2,22,1 from dual union all
select 1103,'TU-904','MX','',1983,'Peacekeeper [1]',48625,null,1711,281.4,78.8,1 from dual union all
select 1104,'TX-12','','XM-100',1950,'Sergeant',3134,null,null,null,null,1 from dual union all
select 1105,'TX-135','','',1968,'Spartan [1]',5295,null,2000,null,4.5,1 from dual union all
select 1106,'TX-16-1','','M16F1',1955,'Matador [0]',618,null,254,null,2.4,1 from dual union all
select 1107,'TX-174','','Pershing',1960,'Pershing [1]',2269,null,117,null,39,1 from dual union all
select 1108,'TX-175','Pershing','',1960,'Pershing [2]',1473,null,85,null,39,1 from dual union all
select 1109,'TX-20','','XM-20?',null,'Sergeant',3200,null,220,null,23.8,1 from dual union all
select 111,'A1P-1','','',1960,'Polaris A1 [1]',8330,null,null,null,null,1 from dual union all
select 1110,'TX-238','','',null,'TX-238',3492,null,null,null,null,1 from dual union all
select 1111,'TX-239','','',1959,'Nike Zeus [3]',368,null,null,null,null,1 from dual union all
select 1112,'TX-261','','',null,'TX-261',1307,2761,244,null,11.3,1 from dual union all
select 1113,'TX-261-2','Pedro','Pedro',null,'Pedro',1307,2761,244,null,11.3,1 from dual union all
select 1114,'TX-261-3','Pedro','Pedro',null,'Pedro',1307,2761,244,null,11.3,1 from dual union all
select 1115,'TX-30','','',1960,'Nike Hercules [2]',1206,null,44.5,null,29,1 from dual union all
select 1116,'TX-33','Castor','Castor',null,'Castor I',4000,7311,240,254.3,27.4,1 from dual union all
select 1117,'TX-354-3','Castor','Castor 2A',null,'Scout [2]',4427,10301,275.1,281.9,39,1 from dual union all
select 1118,'TX-354-4','Castor','Castor 2',1965,'Castor II',4320,9732,259,null,37,1 from dual union all
select 1119,'TX-454','','',1967,'Spartan [2]',6454,null,null,null,null,1 from dual union all
select 112,'A1P-2','','',1960,'Polaris A1 [2]',4300,null,null,null,null,1 from dual union all
select 1120,'TX-486-1','SAM-D','Malemute',1974,'Patriot TX-486-1',619,1395,55.1,281.2,22,1 from dual union all
select 1121,'TX-500','','',1968,'Spartan[1]',9000,null,2000,null,null,1 from dual union all
select 1122,'TX-52','','Lacrosse',1958,'Doorknob',221,null,null,null,null,1 from dual union all
select 1123,'TX-526','Castor 4','',1971,'Athena H [1]',10534,23760,428,null,53.9,1 from dual union all
select 1124,'TX-664-4','Terrier','Mk 70',null,'SM-2 [1]',969,null,null,null,6.2,1 from dual union all
select 1125,'TX-77','','',null,'X-7A3 Booster',824,null,227,null,8.7,1 from dual union all
select 1126,'TX-780','Castor','Castor IVA',1980,'Atlas IIAS [-1]',11600,26460,433.7,237.8,56.2,1 from dual union all
select 1127,'TX-780XL','Castor','Castor IVAXL',1980,'Atlas',14851,null,599.8,269,60,1 from dual union all
select 1128,'TX-859','Castor','Castor IVB',1991,'Castor 4B',11476,26150,429,null,60.9,1 from dual union all
select 1129,'TX8-6','','',1958,'Able I motor',15,25,3.6,233,7,1 from dual union all
select 113,'A2P-1','','',1960,'Polaris A2 [1]',10100,null,null,null,null,1 from dual union all
select 1130,'Talos Mk 11 Mod 2','Talos','Talos',null,'Talos [X-251C1]',1500,2700,516,null,5.3,1 from dual union all
select 1131,'Taurus','M6','M31A1',1977,'Taurus Orion [1]',1343,1592,null,null,3.5,1 from dual union all
select 1132,'Terrier Mk12 Mod 0','Terrier','',null,'Terrier',845,1157,257.5,195.4,4.4,1 from dual union all
select 1133,'Terrier Mk12 Mod 1','Terrier','Adv Terr BT3',null,'Terrier  [X-256A1]',817,1134,257.5,195.4,4.4,1 from dual union all
select 1134,'Theodore','','',null,'Monica',10,18,1.8,null,10,1 from dual union all
select 1135,'Tianying 3C','','',2011,'Tianying 3C',300,null,null,null,null,1 from dual union all
select 1136,'Tianying 3F','','',2016,'Tianying 3F',400,null,null,null,null,1 from dual union all
select 1137,'Tiny Tim','','',1945,'WAC',345,130,220,null,.6,1 from dual union all
select 1138,'Topaze','','',null,'Topaze',null,null,150,260,40,1 from dual union all
select 1139,'Trident Motor','','',1967,'RMP-B',670,null,35,268,null,2 from dual union all
select 114,'A3P-1','','',1960,'Polaris A3 [1]',11100,null,null,null,null,1 from dual union all
select 1140,'Trident St 3','','',1977,'Trident [3]',1900,null,null,null,null,1 from dual union all
select 1141,'UA1205','T3SRM','',1965,'Titan 3 SRM',226200,null,5293.3,263,115,1 from dual union all
select 1142,'UA1206','T3SRM','',1989,'Titan 34D SRM',251400,null,6227,265,114,1 from dual union all
select 1143,'UA1207','T3SRM','',1989,'Titan 4 SRM',319300,null,7116.9,272,120,1 from dual union all
select 1144,'UM-129A','','',null,'H-I [3]',2200,null,77.5,291,68,1 from dual union all
select 1145,'UPS','','',1998,'COMETS LAE',null,null,1.7,321.4,null,1 from dual union all
select 1146,'Unha St3','Nodong','',null,'',null,null,null,null,null,1 from dual union all
select 1147,'Upstage','','',null,'Upstage',140,null,null,null,1,1 from dual union all
select 1148,'Urbion','','',1984,'INTA-100 [2]',33,null,1.5,null,34,1 from dual union all
select 1149,'V-750V','V-750','',null,'',null,null,30.4,null,null,1 from dual union all
select 115,'ABL JATO','JATO','3-DS-50000',1947,'JATO',350,null,222,null,3,1 from dual union all
select 1150,'V-825','','',null,'V-825 St 2',null,null,131,null,40,1 from dual union all
select 1151,'Valois','LRBA','',1970,'Diamant B [1]',null,39000,415,265,119,1 from dual union all
select 1152,'Vega','Belier','',null,'Belier III',320,null,20,null,23.8,1 from dual union all
select 1153,'Vela','','',1964,'Aero High [2]',118,133,26.7,null,5,1 from dual union all
select 1154,'Venus','Venus','',1961,'Centaure [1]',160,190,44,205,4.3,1 from dual union all
select 1155,'Vero P2','','',1952,'Veronique P2',500,null,10,null,null,1 from dual union all
select 1156,'Vero P6','','',1952,'Veronique P6',500,null,20,null,null,1 from dual union all
select 1157,'Veronique','LRBA','',1952,'Veronique',null,null,40,null,null,1 from dual union all
select 1158,'Veronique 61','LRBA','',1964,'Veronique 61',null,null,60,null,null,1 from dual union all
select 1159,'Veronique AGI','LRBA','',1959,'Veronique AGI',null,1900,40,null,49,1 from dual union all
select 116,'AJ-60A','','',2003,'Atlas V SRM',46200,null,1361,275,90,1 from dual union all
select 1160,'Veronique N','LRBA','',1952,'Veronique N',null,null,40,null,null,1 from dual union all
select 1161,'Vesta','LRBA','',1965,'Vesta',null,7900,140,null,56.4,1 from dual union all
select 1162,'Vexin','LRBA','',1964,'Emeraude',null,25500,274,null,93,1 from dual union all
select 1163,'Vexin A','','',null,'',null,null,68.6,277,null,1 from dual union all
select 1164,'Vexin B','','',null,'',null,null,75.4,251,null,1 from dual union all
select 1165,'Vexin C','','',null,'',null,null,99.1,251,null,1 from dual union all
select 1166,'Vikas','Viking','',1993,'PSLV PS2',null,109000,724,293,158,1 from dual union all
select 1167,'Vikas GSLV','Viking','',1993,'GSLV [1]',null,null,700,281,200,1 from dual union all
select 1168,'Viking IV','Viking','',1979,'Ariane 1 St2',null,null,713,295.6,136,1 from dual union all
select 1169,'Viking IVB','Viking','',null,'Ariane St2',null,null,785,293.5,126,1 from dual union all
select 117,'AJ10','AJ10','',null,'Vanguard [2]',null,null,33,null,122,1 from dual union all
select 1170,'Viking V','Viking','',1979,'Ariane 1 St1',null,null,693,280.6,142,1 from dual union all
select 1171,'Viking VC','Viking','',null,'Ariane St1',null,null,758.5,278.4,205,1 from dual union all
select 1172,'Viking VI','Viking','',null,'Ariane PAL',null,null,758.5,278,140,1 from dual union all
select 1173,'Viper IIIA','Viper','',1972,'Viper',30,59,24.7,null,2.2,1 from dual union all
select 1174,'Viper V','Viper','',1972,'Viper',60,142,27,null,5.3,1 from dual union all
select 1175,'Volga','','',2103,'Volga',null,null,2.9,307,null,1 from dual union all
select 1176,'Vulcain','','',1996,'Ariane EPC H158',null,null,1114,430,589,1 from dual union all
select 1177,'Vulcain 2','','',2002,'Ariane EPC H175',null,null,1300,434,589,1 from dual union all
select 1178,'WAC','WAC','',1945,'WAC',313,300,6.7,null,47,1 from dual;
insert into ENGINE
select 1179,'Wasserfall','','',1944,'Wasserfall',null,null,78,null,40,1 from dual union all
select 118,'AJ10-101','AJ10','',null,'',null,null,34.3,270,null,1 from dual union all
select 1180,'Waxwing','Waxwing','',1969,'Waxwing',351,845,28.8,276,40,1 from dual union all
select 1181,'X-202C3','','4DW105000',1950,'X-7A1 Booster',800,1900,470,null,4,1 from dual union all
select 1182,'X-209','JATO','',null,'M8 JATO (1980s)',71.8,null,4.4,null,14,1 from dual union all
select 1183,'X-216A2','Nike','2.5-DS-59000',null,'Nike 1249',630,640,262,null,2.5,1 from dual union all
select 1184,'X-220','Deacon','',1953,'Deacon',68,79,26,null,3,1 from dual union all
select 1185,'X-241','','',1958,'Pilot [3]',196,44,12.1,260.1,36,1 from dual union all
select 1186,'X-248','Altair','Altair I',1959,'Altair',296,561,14,null,40,1 from dual union all
select 1187,'X-248 [1]','Altair','Altair I',1957,'Altair  (Scout X-1 A',330,518,13.4,null,38.6,1 from dual union all
select 1188,'X-248A10','Altair','Altair I',1957,'Altair (TB2 st3)',296,516,13,null,39,1 from dual union all
select 1189,'X-248A3','Altair','Altair I',1958,'Thor Able [3]',232,518,14,250.5,39,1 from dual union all
select 119,'AJ10-101A','Able','',1958,'Able',null,null,34.7,null,null,1 from dual union all
select 1190,'X-248A5','Altair','Altair I',1957,'Altair/Scout X-1[4]',240,516,12.5,255,38,1 from dual union all
select 1191,'X-250','','DDT-70',1960,'Polaris A2 [2]',4200,null,null,null,null,1 from dual union all
select 1192,'X-251','Talos','Talos',null,'Talos',1922,2700,516,null,5.3,1 from dual union all
select 1193,'X-254','Antares','Antares 1A',1960,'Scout X-1[3]',1230,2330,60.5,255,39,1 from dual union all
select 1194,'X-254 (1)','Antares','Antares 1A',1960,'Antares  [Alway 2000',1300,2376,59.4,null,40,1 from dual union all
select 1195,'X-254 [1]','Antares','Antares 1A',1960,'Antares',1050,4200,62,255,39,1 from dual union all
select 1196,'X-256','Terrier','',null,'Terrier  [X-256]',820,1134,257.5,195.4,4.4,1 from dual union all
select 1197,'X-258','Altair','Altair II',null,'Altair II',300,623,24.2,279.1,31,1 from dual union all
select 1198,'X-258-B1','Altair','Altair II',null,'Scout B [4]',260,620,27.5,278.1,24,1 from dual union all
select 1199,'X-259','Antares','Antares 2',1962,'Antares 2',1275,null,96.5,null,null,1 from dual union all
select 12,'11D49 [Old]','S5.23','',null,'S3M [See N2O4]',null,null,157.5,303,335,1 from dual union all
select 120,'AJ10-104','AJ10','',null,'',null,null,35.1,278,null,1 from dual union all
select 1200,'X-259-B3','Antares','Antares 2',null,'Scout B [3]',1275,3225,93.1,281.4,35.9,1 from dual union all
select 1201,'X-260','','',1960,'Polaris A3 [2]',4900,null,null,null,null,1 from dual union all
select 1202,'X-265','','',null,'X-265',3000,null,null,null,null,1 from dual union all
select 1203,'X-271','','',null,'X-271',1000,null,null,null,null,1 from dual union all
select 1204,'X-405','X-405','',1957,'Vanguard St1',null,null,120,null,144,1 from dual union all
select 1205,'X21AL-2600','Nike','',1947,'Nike 46 Ajax [2]',null,null,11.4,null,21,1 from dual union all
select 1206,'XASR-1','Aerobee','',1948,'Aerobee',467,520,11.6,188.5,32.1,1 from dual union all
select 1207,'XLR-11','XLR-11','',1959,'X-15',null,null,36,null,100,4 from dual union all
select 1208,'XLR-43-NA-1','XLR-43','',null,'',null,null,333,null,null,1 from dual union all
select 1209,'XLR-83-NA-1','XLR-83','',null,'',null,null,602,273,null,1 from dual union all
select 121,'AJ10-118','AJ10','',null,'',null,null,33.7,271,null,1 from dual union all
select 1210,'XLR-99','XLR-99','TD-214',1959,'X-15',null,null,262.4,276,null,1 from dual union all
select 1211,'XLR-99','XLR-99','',1960,'X-15',null,null,257,276,100,1 from dual union all
select 1212,'XM-10','','Lacrosse',1958,'Lacrosse',221,null,null,null,null,1 from dual union all
select 1213,'XM-20','','XM-20',null,'Draco, X-17',3200,6350,198,null,32,1 from dual union all
select 1214,'XM-30','','',1959,'X-17, Draco',1200,null,54.3,null,37,1 from dual union all
select 1215,'XM-33-20-4','Castor','Castor',null,'Scout X-1 [2]',4360,7268,276,224,27,1 from dual union all
select 1216,'XM-33E2','Castor','TX-33-28',null,'Castor I',4000,7311,240,254.3,27.4,1 from dual union all
select 1217,'XM-33E3','Castor','TX-33-20',null,'Castor',4000,7311,240,254.3,27.4,1 from dual union all
select 1218,'XM-33E5','Castor','TX-33-33',null,'Castor',4000,7311,240,254.3,27.4,1 from dual union all
select 1219,'XM-33E6','Castor','Shotput',null,'Castor',4000,7311,240,254.3,27.4,1 from dual union all
select 122,'AJ10-118A','AJ10','',null,'Delta B Stage 2',null,5730,33.7,null,170,1 from dual union all
select 1220,'XM-33E8','Castor','Castor',null,'Castor I (mod)',3970,8272,236.7,254.3,35.3,1 from dual union all
select 1221,'XM-42','Nike M-42','',1956,'N Her booster (clust',2400,null,null,null,null,1 from dual union all
select 1222,'XM-45','TX-77','Skat',1964,'TB2 [2]',922,null,200,null,6,1 from dual union all
select 1223,'XM-51','Bomarc Booster','TX-131',1958,'Bomarc B [0]',2990,null,222,null,30,1 from dual union all
select 1224,'XM5','Nike','2.5-DS-59000',1950,'Nike (3-fin)',630,null,262,null,2.5,1 from dual union all
select 1225,'XR-5K18','','',2011,'XCOR Lynx',null,null,12.9,null,null,1 from dual union all
select 1226,'Xaero','','',null,'Xaero',null,null,13.2,null,null,1 from dual union all
select 1227,'YF-1','','',null,'CZ-1 [1]',null,null,255,240,130,1 from dual union all
select 1228,'YF-100','','',null,'CZ-5',null,null,1200,305,155,1 from dual union all
select 1229,'YF-115','','',null,'CZ-6 St 2',null,null,176.5,342,null,1 from dual union all
select 123,'AJ10-118D','AJ10','',null,'',null,null,33.7,278,null,1 from dual union all
select 1230,'YF-20','YF','',1973,'CZ-2C',null,null,696,289,128,4 from dual union all
select 1231,'YF-20B','YF','',1973,'CZ-3A St 1',null,null,740,289,166,4 from dual union all
select 1232,'YF-21','YF','',1970,'CZ-2 Stage 1 x4',null,null,740.4,260.7,122,1 from dual union all
select 1233,'YF-22','YF','',1970,'CZ-2 Stage 2',null,null,742,298,109,1 from dual union all
select 1234,'YF-22B','YF','',1990,'CZ-4 Stage 2',null,null,742,287.6,126.8,1 from dual union all
select 1235,'YF-22B''','YF','',1990,'CZ-4B Stage 2',null,null,742,298,126.8,1 from dual union all
select 1236,'YF-23','YF','',1990,'FB-1 stage 2',null,null,716,286.9,117.4,1 from dual union all
select 1237,'YF-23','','',1970,'CZ-2 Stage 2 vernier',null,null,11.8,296,347,1 from dual union all
select 1238,'YF-25','YF','',1973,'CZ-3B',null,null,696,289,128,4 from dual union all
select 1239,'YF-2A','YF-2','',null,'',null,null,306.1,268,null,1 from dual union all
select 124,'AJ10-118E','AJ10','',null,'',null,null,35.1,278,400,1 from dual union all
select 1240,'YF-3','','',null,'CZ-1 [2]',null,null,294,287,120,1 from dual union all
select 1241,'YF-40','YF','',1990,'CZ-4 Stage 3',null,null,98.1,303,303,1 from dual union all
select 1242,'YF-40','','',null,'CZ-4B (3)',null,null,49,295,null,1 from dual union all
select 1243,'YF-40A','','',null,'CZ-4C (3)',null,null,49,295,null,1 from dual union all
select 1244,'YF-50D','YZ-1','',null,'CZ-3C/YZ-1',null,null,6.5,315.3,null,1 from dual union all
select 1245,'YF-73','YF-73','',1984,'CZ-3 Stage 3',null,null,44.1,425,800,4 from dual union all
select 1246,'YF-75','YF-75','',null,'CZ-3A Stage 3',null,null,78.5,440,470,1 from dual union all
select 1247,'YF-75D','YF-75','',null,'CZ-5 Stage 2',null,null,88.3,442,700,1 from dual union all
select 1248,'YF-77','','',null,'CZ-5',null,null,673,438,500,1 from dual union all
select 1249,'YF-85','','',2015,'CZ-6',null,null,4000,null,null,1 from dual union all
select 125,'AJ10-118F','AJ10','',null,'',null,null,41.4,280,null,1 from dual union all
select 1250,'Yardbird','Recruit','TE-289',1959,'Yardbird',151,null,75,null,3.4,1 from dual union all
select 1251,'ZN-1','','',1970,'ZN-1',50,null,null,null,null,1 from dual union all
select 1252,'Zefiro','Zefiro','',1999,'Vega [2]',17260,41830,590,271,71,1 from dual union all
select 1253,'Zefiro 23','','',2012,'Vega [2]',26563,67776,935,289,71.7,1 from dual union all
select 1254,'Zefiro 9A','','',2012,'Vega [3]',9730,null,221,294,110,1 from dual union all
select 1255,'Zefiro-40C','','',2017,'Vega-C [2]',39270,103600,1120,293.5,92.5,1 from dual union all
select 1256,'Zenith','','',1967,'Zenith',600,900,45,null,21,1 from dual union all
select 1257,'Zeus A','','',null,'Zeus A',2000,null,null,null,null,1 from dual union all
select 126,'AJ10-118FJ','AJ10','',null,'N-2 Stage 2',null,null,44.1,319.2,430,1 from dual union all
select 127,'AJ10-118K','AJ10','',null,'Delta 2 Stage 2',null,null,43.7,319.2,431,1 from dual union all
select 128,'AJ10-138','AJ10','',1964,'Transtage x2',null,null,35.6,311,440,1 from dual union all
select 129,'AJ10-138 (early)','AJ10','',1964,'Transtage x2',null,null,35.6,302,430,1 from dual union all
select 13,'11D511','RD-108','',null,'Soyuz A',null,null,978,316,290,4 from dual union all
select 130,'AJ10-138A','AJ10','ITIP',1964,'Transtage x2 (late)',null,null,35.6,309.3,null,1 from dual union all
select 131,'AJ10-24','Aerobee','',1950,'Aerobee',467,586,17.8,193,32.1,1 from dual union all
select 132,'AJ10-25','Aerobee','',1950,'Aerobee',467,586,17.8,193,32.1,1 from dual union all
select 133,'AJ10-27','Aerobee','',1950,'Aerobee',467,586,17.8,193,32.1,1 from dual union all
select 134,'AJ10-34','Aerobee','',1950,'Aerobee',467,586,17.8,193,32.1,1 from dual union all
select 135,'AJ10-40','AJ10','',null,'Able',null,null,34.5,null,110,1 from dual union all
select 136,'AJ10-41','AJ10','',null,'Able',null,null,34.5,null,110,1 from dual;
insert into ENGINE
select 137,'AJ10-42','AJ10','',null,'Able',null,null,34.5,null,110,1 from dual union all
select 138,'AJ11-21','Aerobee','',1959,'Aerobee',670,928,17.8,199,52,1 from dual union all
select 139,'AJ11-6','Aerobee','AGVL-0113F',1959,'Aerobee',670,928,17.8,199,52,1 from dual union all
select 14,'11D512','RD-108','',null,'Soyuz BVGD',null,null,1000,314.5,120,4 from dual union all
select 140,'AJ26-62','','',null,'Taurus 2 [1]',null,null,1510,null,null,2 from dual union all
select 141,'AJ60-80','Aerobee Booster','Aerobee JATO',1947,'Aerobee [0]',260,200,80,null,2.5,1 from dual union all
select 142,'AJ60-92','Aerobee 200','',null,'Aerobee 200',576,1346,22.6,null,58.5,1 from dual union all
select 143,'AR2-3','','',null,'F-104, X-37B',null,null,31,245,null,1 from dual union all
select 144,'AS-4','','',1987,'ASLV [4]',512,null,35,281,33.4,1 from dual union all
select 145,'ASLV AS-1','','',null,'ASLV [1]',11800,null,702,259,45,1 from dual union all
select 146,'ASLV AS-2','','',null,'ASLV [2]',4400,null,304,275.9,36,1 from dual union all
select 147,'ASLV AS-3','','',null,'ASLV [3]',1710,null,90.7,277,45,1 from dual union all
select 148,'ASLV SRM','','',null,'ASLV [0]',11600,null,422,253,49,1 from dual union all
select 149,'ATACMS','ATACMS','',null,'ATACMS',1200,null,180,null,null,1 from dual union all
select 15,'11D55','RD-0106','RD-0110?',1965,'Blok I',null,null,298,326,250,4 from dual union all
select 150,'ATSM-13','Shaviyt','',1994,'Shaviyt 1 [1]',13000,null,775,250,55,1 from dual union all
select 151,'ATSM-9','Shaviyt','',1988,'Shaviyt [2]',9100,null,null,null,null,1 from dual union all
select 152,'AUS-51','','',1988,'Shaviyt [3]',2000,null,58,295,92,1 from dual union all
select 153,'Ababeel','','',2016,'Ababeel [3]',1000,null,null,null,null,1 from dual union all
select 154,'Abid','','',null,'Abid St 1',null,null,130,null,null,1 from dual union all
select 155,'Abid St3','','',null,'Abid St 3',500,null,null,null,null,1 from dual union all
select 156,'Acheron','','',1988,'Hades',1200,null,null,null,null,1 from dual union all
select 157,'Aerobee Jr','','',1958,'Aerobee 100',null,null,11.6,200,40,1 from dual union all
select 158,'Aerojet Jr','Aerojet Jr','',1961,'Aerojet Jr',3000,7100,254,null,28,1 from dual union all
select 159,'Aerojet Mk 8','','',null,'Bullpup',300,null,111,null,null,1 from dual union all
select 16,'11D58','RD-58','',1967,'Blok D',null,null,85,349,600,1 from dual union all
select 160,'Aestus','Aestus','',1996,'Ariane 5 EPS',null,null,29,324,null,1 from dual union all
select 161,'Agnes','','Elan',1967,'Belenos [1]',82,100,40,242,2.5,1 from dual union all
select 162,'Agni','','',null,'Agni [1]',11500,null,700,null,45,1 from dual union all
select 163,'Agni III','Agni3','',2006,'Agni 3 [1]',33000,null,1040,237,75,1 from dual union all
select 164,'Agni III S2','Agni3','',2006,'Agni 3 [2]',14000,null,315,294,110,1 from dual union all
select 165,'Agni St2','','',null,'Agni II [2]',4200,null,null,null,60,1 from dual union all
select 166,'Agni V S3','Agni3','',2006,'Agni 3 [2]',10000,null,315,294,110,1 from dual union all
select 167,'Ajax','','',null,'Nike Ajax [2]',null,null,11.6,null,30,1 from dual union all
select 168,'Ajax 1810','','',null,'Nike Hercules 1810 [',null,null,11.6,null,30,1 from dual union all
select 169,'Ajax-490','Nike','',1949,'Nike Ajax',null,null,11.4,null,21,1 from dual union all
select 17,'11D58M','RD-58','',1976,'Blok DM',null,null,85,352,680,1 from dual union all
select 170,'Al Fatah','','',null,'Black Sparrow',1200,1950,130,238,15,1 from dual union all
select 171,'AlKahir','','',1962,'Al Kahir',null,null,null,null,null,1 from dual union all
select 172,'Alcor IB','Alcor','Alcor IB',null,'Alcor IB',500,1148,42.8,null,26.5,1 from dual union all
select 173,'Alfa','','',1973,'Alfa [1]',6500,null,250,null,57,1 from dual union all
select 174,'Algol 3A','Algol III','',null,'Scout [1]',14136,32040,477,258.2,78,1 from dual union all
select 175,'Algol I','Algol','AJ Senior',1960,'Scout [1]',10710,18240,458.6,214,40,1 from dual union all
select 176,'Algol IA','Algol','AJ Senior',1960,'Scout [1]',10710,18240,458.6,214,40,1 from dual union all
select 177,'Algol IB','Algol','AJ Senior',1960,'Scout [1] ?',10710,18240,458.6,214,40,1 from dual union all
select 178,'Algol IC','Algol','AJ Senior',1960,'Scout [1] ?',10710,18240,458.6,214,40,1 from dual union all
select 179,'Algol ID','Algol','',1960,'Scout [1]',10700,18100,413,null,44,1 from dual union all
select 18,'11D58M','RD-58M','',1976,'Blok D-1',null,null,83.5,362,680,1 from dual union all
select 180,'Algol IIA','Algol','',1965,'Scout [1]',10709,24337,465.1,258.3,68.2,1 from dual union all
select 181,'Algol IIB','Algol','',1965,'Scout B [1]',10759,24352,449.2,258.9,80,1 from dual union all
select 182,'Algol IIC','Algol','',1965,'Scout [1]',10795,24390,436.7,258.9,76.1,1 from dual union all
select 183,'Arcon','Arcon','',1958,'Arcon',100,140,4.5,null,33,1 from dual union all
select 184,'Arcon booster','','',1958,'Arcon',12.4,5,14,null,.4,1 from dual union all
select 185,'Arrow II','','',1950,'Arrow',12,17,9,null,1.9,1 from dual union all
select 186,'Asp II','Asp','',1959,'Asp II',87,141,38.6,212,3.6,1 from dual union all
select 187,'Asp III','Asp','',1959,'Asp III',81,139,42.7,226,3.3,1 from dual union all
select 188,'Astra E1','','',2017,'Astra St 1',null,null,18,null,null,1 from dual union all
select 189,'Astris','','',1968,'Astris',null,null,22.5,297,361,1 from dual union all
select 19,'11D79','','',null,'',null,null,44,295,null,1 from dual union all
select 190,'Astrobee D','Astrobee','',1970,'Astrobee D',85,156,8.9,null,17,1 from dual union all
select 191,'Astrobee F','Astrobee','',1972,'Astrobee F',1300,2440,36.4,null,51.1,1 from dual union all
select 192,'Atea-1','','',2009,'Atea-1',null,100,6.9,null,14.5,1 from dual union all
select 193,'Aurore','','',null,'Belenos [2]',54,58,null,250,5.3,1 from dual union all
select 194,'BB Mk1','BB5','BB5',2007,'BB Mk1',1167,2251,89,230.2,26,1 from dual union all
select 195,'BB Mk3','BB5','BB5',2007,'BB Mk3',1167,2251,89,230.2,26,1 from dual union all
select 196,'BB7','','',1969,'BB 7',16,null,null,null,null,1 from dual union all
select 197,'BE-3','BE-3','Alcyone',null,'Alcyone',100,null,34,null,8,1 from dual union all
select 198,'BE-3 (BLOR)','','',null,'New Shepard',null,null,490,null,145,1 from dual union all
select 199,'BE-3A4','BE-3','',null,'Athena motor',100,null,34,null,8,1 from dual union all
select 2,'1.5KS35000','Recruit','Recruit XM19',1956,'Recruit',153,262,156,null,1.7,1 from dual union all
select 20,'12-AS-1000','F-80 JATO','',1945,'JATO',50,null,4.4,null,12,1 from dual union all
select 200,'BE-3A5','BE-3','',null,'SPARTA motor',100,null,34,null,8,1 from dual union all
select 201,'BE-3A9','BE-3','Alcyone',1969,'Scout E [5]',98,235,27.7,275.2,8.5,1 from dual union all
select 202,'BE-3B1','BE-3','Alcyone',null,'Alcyone',138,269,34.2,null,7.8,1 from dual union all
select 203,'BE-4','','',2019,'Atlas 5',null,null,2447,null,null,1 from dual union all
select 204,'BS.605','','',null,'Buccaneer RATO',null,null,35.6,null,null,null from dual union all
select 205,'BT-4','DMT','',2002,'NStar c LAE (STAR-2)',null,null,.5,329.6,null,1 from dual union all
select 206,'BUS-1 OAT','','',1973,'KH-11',null,null,1.1,null,null,1 from dual union all
select 207,'Badr-1','','',null,'Badr-1',1000,null,null,null,null,1 from dual union all
select 208,'Bantam','','',1962,'Skua',37,null,1.8,237,30,1 from dual union all
select 209,'Bantam 2','','',null,'Skua 2',37,null,1.8,237,30,1 from dual union all
select 21,'12KS10000','BB3','BB3B',1967,'BB3B',292,552,43.7,236.2,12,1 from dual union all
select 210,'Bantam 3','','',null,'Skua 3',33,null,1.7,237,34,1 from dual union all
select 211,'Belier II','Belier','',null,'Belier II',320,null,20,null,23.8,1 from dual union all
select 212,'Belisama','','',1968,'Belisama [2]',26,24,24,null,.9,1 from dual union all
select 213,'Bell 8048','Agena','',1959,'Agena A',null,null,69,276,120,1 from dual union all
select 214,'Bell 8081','Bell Agena','',null,'',null,null,71.2,285,null,1 from dual union all
select 215,'Bell 8096','Agena','',1960,'Agena B',null,null,71.2,289.8,null,1 from dual union all
select 216,'Bell 8096-37','Agena','',1960,'Agena D',null,null,71.2,289.8,null,1 from dual union all
select 217,'Bell 8096-39','Agena','',1960,'Agena D',null,null,71.2,299.7,null,1 from dual union all
select 218,'Black Sparrow','','',null,'Black Sparrow',1200,null,null,null,null,1 from dual union all
select 219,'Buran TDEO','','',1988,'Buran (x4)',null,35,28,null,null,1 from dual union all
select 22,'14-AS-1000','JATO','',1945,'JATO Mk 2?',50,null,4.4,null,14,1 from dual union all
select 220,'Buran UD','','17D15',1988,'Buran (x38)',null,null,4,280,null,1 from dual union all
select 221,'C3-1','','',1968,'Poseidon C-3 [1]',10000,null,null,null,null,1 from dual union all
select 222,'C3-2','','',1968,'Poseidon C-3 [2]',7000,null,null,null,null,1 from dual union all
select 223,'C4-1','','',null,'Trident C-4 [1]',12100,null,null,null,null,1 from dual union all
select 224,'C4-2','','',null,'Trident C-4 [1]',8000,null,null,null,null,1 from dual union all
select 225,'CE-20','','',2009,'C-25',null,null,186,443,550,1 from dual union all
select 226,'CPKM','','',1995,'CZ-2E/Asiasat',4635,11360,222.8,280,65.2,1 from dual;
insert into ENGINE
select 227,'CRAPEL','','',1969,'CRAPEL/Belier',null,52,null,null,10.5,1 from dual union all
select 228,'CZ-11-1','','',2016,'CZ-11 [1]',40000,null,null,248,null,1 from dual union all
select 229,'CZ-11-2','','',null,'CZ-11 [2]',15000,null,null,null,null,1 from dual union all
select 23,'14-NS-1000','JATO','',1945,'JATO',50,null,4.4,null,14,1 from dual union all
select 230,'CZ-11-3','','',null,'CZ-11 [3]',2000,null,null,null,null,1 from dual union all
select 231,'CZ-11-4','','',null,'CZ-11 [4]',2000,null,null,null,null,1 from dual union all
select 232,'Cajun','Deacon','',1957,'Cajun',75,108,43,null,3,1 from dual union all
select 233,'Canopus','','',1972,'Rigel',200,null,null,null,18,1 from dual union all
select 234,'Castor 120','MX','',1995,'Athena [1]',53100,null,1604,280,83.4,1 from dual union all
select 235,'Castor 30','','',null,'Taurus 2 [2]',14035,35200,259,301,136,1 from dual union all
select 236,'Castor 30B','','',null,'Taurus 2 [2]',13970,37300,293.4,304,127,1 from dual union all
select 237,'Castor 30XL','','',null,'Taurus 2 [2]',27400,58000,350,null,166,1 from dual union all
select 238,'Cedre St1','','',1962,'Cedre 3',null,null,138,null,null,1 from dual union all
select 239,'Cedre St2','','',1962,'Cedre 3',null,null,78,null,null,1 from dual union all
select 24,'15-KS-1000','JATO','',1945,'JATO Mk 6',63,null,4.4,null,15,1 from dual union all
select 240,'Cedre St3','','',1962,'Cedre 3',null,null,20,null,null,1 from dual union all
select 241,'Cedre-4 St1','','',1962,'Cedre 4',null,null,138,null,null,1 from dual union all
select 242,'Centenario','','',2013,'Centenario',224,null,39,null,11.2,1 from dual union all
select 243,'Cherokee','','TE-146',1963,'Squirt [2]',30,null,null,null,1.5,1 from dual union all
select 244,'Chick','','',null,'Chick',8,4,20,null,.2,1 from dual union all
select 245,'Cirrus I','','',null,'Cirrus',50,50,5,null,10,1 from dual union all
select 246,'Cirrus II','','',null,'Cirrus',50,90,18,null,5,1 from dual union all
select 247,'Coralie','','',1966,'Coralie',null,null,67,281,105,1 from dual union all
select 248,'Corporal','','',null,'Corporal',null,null,89,null,63,1 from dual union all
select 249,'Corporal E','','',1947,'Corporal E',null,null,89,null,null,1 from dual union all
select 25,'15D13','8D423, 5D13 type','',null,'UR-100 St 2',null,null,131,325,null,1 from dual union all
select 250,'Cuckoo 4','','',null,'Skylark 12 [3]',243,452,44,204,15,1 from dual union all
select 251,'Cuckoo IA','','',null,'Skylark',243,320,90,204,4.3,1 from dual union all
select 252,'Cuckoo IB','','',null,'BK',237,360,80.9,204,4.1,1 from dual union all
select 253,'Cuckoo II','','',null,'BK',226,396,36.5,204,10,1 from dual union all
select 254,'Curie','','',2017,'Electron St 3',null,null,.12,303,null,1 from dual union all
select 255,'Cygnus 10','','10" SM',null,'10" SM',15,null,10,null,3.2,1 from dual union all
select 256,'Cygnus 15','','TX-456-2?',null,'TB2 [4]',60,null,22.3,null,4.8,1 from dual union all
select 257,'Cygnus 20','','20" SM',1959,'20" SM',118,null,30,null,8.5,1 from dual union all
select 258,'Cygnus 25','','25" SM',null,'25" SM',239,null,33.3,null,16.5,1 from dual union all
select 259,'Cygnus 40','TE-M-186','40" SM',null,'40" SM',972,null,83,null,26,1 from dual union all
select 26,'15D14','8D419, 5D13 type','',null,'UR-100 vernier',null,null,null,null,null,1 from dual union all
select 260,'Cygnus 5','','',null,'TB2 [6]',2.5,null,2.3,236,1.6,1 from dual union all
select 261,'D-5-1','','',null,'Trident II [1]',39100,null,null,null,64,1 from dual union all
select 262,'D-5-2','','',null,'Trident II [2]',11800,null,null,null,65,1 from dual union all
select 263,'D-SAT-EES','','',null,'D-SAT',.9,null,.45,266.4,3.2,1 from dual union all
select 264,'DACRoc','','',1963,'DACRoc',250,null,50,220,8,1 from dual union all
select 265,'DF-1','R-2','',null,'DF-1',null,null,363,210,null,1 from dual union all
select 266,'DF-11','','',null,'DF-11',5400,null,93.2,null,null,1 from dual union all
select 267,'DF-15','','',null,'DF-15',6200,null,null,null,null,1 from dual union all
select 268,'DF-16','','',2015,'DF-16',4500,null,null,null,null,1 from dual union all
select 269,'DF-21-1','','',null,'DF-21 [1]',16621,null,null,239.7,65,1 from dual union all
select 27,'15D15','','15D151',1967,'RT-20P [1]',16700,null,600,260,65,4 from dual union all
select 270,'DF-21-2','','',null,'DF-21 [2]',8686,null,null,286.4,62,1 from dual union all
select 271,'DF-31-1','','',null,'DF-31 [1]',35000,null,null,null,null,1 from dual union all
select 272,'DF-31-2','','',null,'DF-31 [2]',10000,null,null,null,null,1 from dual union all
select 273,'DF-31-3','','',null,'DF-31 [3]',2800,null,null,null,null,1 from dual union all
select 274,'DF-41-1','','',null,'DF-31 [1]',35000,null,null,null,null,1 from dual union all
select 275,'DF-41-2','','',null,'DF-31 [2]',10000,null,null,null,null,1 from dual union all
select 276,'DF-41-3','','',null,'DF-31 [3]',2800,null,null,null,null,1 from dual union all
select 277,'DHX-200 Aurora','','',2015,'Stratos II+',85,190,8,null,23,1 from dual union all
select 278,'DIM','','',null,'DIM',30,null,null,null,null,1 from dual union all
select 279,'DM-16','Deacon','',1956,'Rockaire',60,33,30,210,2.1,1 from dual union all
select 28,'15D169','15D12 type','',null,'MR-UR-100 [2]',null,null,147,330,null,null from dual union all
select 280,'DMT-600','RDMT-100','',null,'',null,null,.6,301,null,1 from dual union all
select 281,'DMU 2000','','CORONA DMU',null,'CORONA',8.9,null,null,null,null,1 from dual union all
select 282,'DMU 3000','','CORONA DMU',null,'CORONA',13.4,null,null,null,null,1 from dual union all
select 283,'DS','Start DS','',null,'Start DS',null,null,null,null,210,1 from dual union all
select 284,'DST-100','DST-100','',null,'',null,null,.1,276,null,1 from dual union all
select 285,'DST-100A','DST-100','',null,'',null,null,.1,304,null,1 from dual union all
select 286,'DST-200','DST-200','',null,'',null,null,.2,280,null,1 from dual union all
select 287,'DST-200A','DST-200','',null,'',null,null,.2,300,null,1 from dual union all
select 288,'Demon','','',1962,'HAT',100,104,45.2,null,2.3,1 from dual union all
select 289,'Dolphin','','',1984,'Dolphin',7500,null,155,null,null,1 from dual union all
select 29,'15D171','','',null,'MR-UR-100 BR',500,null,null,null,null,1 from dual union all
select 290,'Dorado','','',1973,'Lorikeet',139.3,173,42,null,4.3,1 from dual union all
select 291,'Draco','','',2009,'DragonLab x 18',null,null,.4,null,600,1 from dual union all
select 292,'E-8-5 VA','','',null,'E-8-5 VA',481,null,null,null,null,1 from dual union all
select 293,'EAM','','',null,'Alphabus',null,null,.5,325,null,1 from dual union all
select 294,'EAP MPS P238','Ariane','P230',1996,'Ariane 5 EAP',268000,null,5000,275.4,129,1 from dual union all
select 295,'EAP MPS P240','Ariane','P230',1996,'Ariane 5 EAP',277500,null,5000,275.4,129,1 from dual union all
select 296,'EPBS','','',null,'',null,null,.4,215,null,1 from dual union all
select 297,'Emilie','','Elan',1967,'Grannos [1]',82,100,40,242,2.5,1 from dual union all
select 298,'Excalibur','','',2000,'Excalibur',39.2,61,20,null,2.5,1 from dual union all
select 299,'F-1','F-1','',1967,'Saturn V [1]',null,null,6680,260,null,1 from dual union all
select 3,'1.8KS7800','Sparrow','',null,'Sparrow',70,63,35,null,1.8,1 from dual union all
select 30,'15D206','','',null,'RT-23 [1]',53700,null,2060,null,null,1 from dual union all
select 300,'FFAR','FFAR','',1955,'Rockair',9,5,3.6,null,1.5,1 from dual union all
select 301,'FFK','','',null,'FFK',300,null,null,null,null,1 from dual union all
select 302,'FG-46','','',1990,'EPKM',5985,null,292,177,87,1 from dual union all
select 303,'FG-47','','',1997,'Smart Dispenser',160,null,null,null,36,1 from dual union all
select 304,'FTI','','',null,'FTB',60,null,null,null,null,1 from dual union all
select 305,'FW4','','',null,'FW-4D',299,null,26.7,null,32.4,1 from dual union all
select 306,'FW4S','','',1965,'FW-4S',301,766,26,284.1,31.5,1 from dual union all
select 307,'FY-25','FY-25','',1994,'DFH-3 LAE',null,null,.49,308.9,null,1 from dual union all
select 308,'Falcon','','M-58',1950,'Falcon',20,null,20,150,1.4,1 from dual union all
select 309,'Fastrac','Fastrac','',null,'X-34',null,null,265,null,null,1 from dual union all
select 31,'15D207','','',null,'RT-23 [2]',25000,null,1050,null,null,1 from dual union all
select 310,'G-38','','LR83-NA1',1956,'Navaho',null,null,534,282,null,1 from dual union all
select 311,'GEM 22','GEM 22 Oriole','',2000,'Oriole',980,null,92.1,null,null,1 from dual union all
select 312,'GEM-40','GEM','',null,'Delta II SRB',13080,31600,499.2,274,63.3,1 from dual union all
select 313,'GEM-46','GEM','',null,'Delta III SRB',19040,45730,608.1,274,75.2,1 from dual union all
select 314,'GEM-60','GEM','',2002,'Delta IV SRB',33200,null,758,275,90,1 from dual union all
select 315,'GEM-60','','',2015,'Atlas V SRM',46200,null,1361,275,90,1 from dual union all
select 316,'GF-01A','','',null,'',2000,null,29,230,140,1 from dual;
insert into ENGINE
select 317,'GF-02','','',1970,'CZ-1 [3]',2050,4500,29,253.9,38,1 from dual union all
select 318,'Gamma 2','Gamma','',1969,'Black Arrow [2]',null,null,68.2,251,113,2 from dual union all
select 319,'Gamma 201','Gamma','',1958,'Black Knight',null,null,71.2,226.9,null,4 from dual union all
select 32,'15D23','RT-2','',null,'RT-2 [1]',34550,null,893,null,75.4,1 from dual union all
select 320,'Gamma 301','Gamma','',1962,'Black Knight',null,null,96.1,249.7,null,4 from dual union all
select 321,'Gamma 8','Gamma','',1969,'Black Arrow',null,null,234.8,265,142,8 from dual union all
select 322,'Goldfinch II','','',1964,'Jabiru',410,700,175,225,3.5,1 from dual union all
select 323,'Goldfinch IIA','','',null,'Skylark',410,700,175,225,3.5,1 from dual union all
select 324,'Goose II','','',1969,'INTA 255',306,370,22,null,17,1 from dual union all
select 325,'Gosling','Gosling','',1960,'Jaguar 1 [1]',230,278,130,null,2.5,1 from dual union all
select 326,'Gosling 4','Gosling','',1974,'INTA 300 [1]',263,426,137,null,4.5,1 from dual union all
select 327,'Gosling IIN','Gosling','',1960,'Jaguar 1 [1]',292,400,130,null,3.5,1 from dual union all
select 328,'Gradicom 1','','',null,'Gradicom 1',400,null,null,null,null,1 from dual union all
select 329,'Gradicom 2','','',null,'Gradicom 2',100,null,null,null,null,1 from dual union all
select 33,'15D23P','RT-2','',null,'RT-2P [1]',34550,null,981,null,75.4,1 from dual union all
select 330,'H-1','H-1','',1961,'Saturn I',null,null,721,263,115,1 from dual union all
select 331,'H-II SRB','NASDA','',1994,'H2 SRB',70400,null,1550,273,94,1 from dual union all
select 332,'H-II SRB-A','NASDA','',null,'H2A SRB-A',75000,null,2250,280,100,1 from dual union all
select 333,'H-II SRB-A3','NASDA','',null,'H2A SRB-A',76500,null,2305,283.6,120,1 from dual union all
select 334,'H-II SSB','NASDA','',1995,'H2 SSB',10500,null,338,267,66,1 from dual union all
select 335,'H-II SSB','NASDA','',null,'H2A SSB',15500,null,745,282,60,1 from dual union all
select 336,'H500','','',1989,'AMROC',12200,23000,333,295.2,70,1 from dual union all
select 337,'HAEC FFAR','FFAR','',1961,'HAEC [2]',10,11,4,null,2.8,1 from dual union all
select 338,'HALO','','',1997,'HALO',20,null,1.9,240,null,1 from dual union all
select 339,'HBT-5','','',2012,'HTV-3',null,null,.5,322,null,1 from dual union all
select 34,'15D24','RT-2','',null,'RT-2 [2]',9600,null,432,null,59,1 from dual union all
select 340,'HIBEX','','',1965,'HIBEX',800,null,2180,null,1.1,1 from dual union all
select 341,'HIBEX-U','','',1965,'HIBEX',800,null,2180,null,1.1,1 from dual union all
select 342,'HM-7','','',1979,'Ariane H8',null,null,61.7,442.6,510.1,1 from dual union all
select 343,'HM-7B','','',null,'Ariane H10-3',null,null,64.8,445.1,745,1 from dual union all
select 344,'HOTROC','','',1958,'HOTROC',163,306,63.2,230,4.9,1 from dual union all
select 345,'HPAG','','',1950,'',30,null,12.6,null,2.7,1 from dual union all
select 346,'HS-15','','',null,'Hwasong-15',null,null,350,null,null,1 from dual union all
select 347,'HTR','','',2007,'HTR',null,520,26,202,20,1 from dual union all
select 348,'HVAR','','',1944,'HVAR',63,null,25,null,.9,1 from dual union all
select 349,'HYSR','','',2003,'HYSR',null,8000,270,290,33.4,1 from dual union all
select 35,'15D24P2','RT-2','',null,'RT-2P [2]',11280,null,440,null,60.6,4 from dual union all
select 350,'Hermes A-1','Hermes','',1950,'Hermes A-1',null,null,71,null,null,1 from dual union all
select 351,'Hermes A-3A','Hermes','',1953,'Hermes A-3A',null,null,80,242,null,1 from dual union all
select 352,'Hermes A-3B','Hermes','',1954,'Hermes A-3B',null,null,100,242,null,1 from dual union all
select 353,'Hopi 2.4KS5600','','',null,'Hopi III',34,58,24.9,227,2.4,1 from dual union all
select 354,'Hopi 3.0KS4000','','',null,'Hopi',33,53,18,null,3,1 from dual union all
select 355,'Hopi 9.0KS1800','','',null,'Hopi IV',40,72,7.7,null,9,1 from dual union all
select 356,'Hwasong-12','','',null,'Hwasong 12',null,null,470,null,128,1 from dual union all
select 357,'Hydac','','H-28',null,'Hydac',253,null,58.7,253,null,1 from dual union all
select 358,'Hydac','','LPC H-26',null,'Hydac',256,412,41.6,253,null,1 from dual union all
select 359,'Hyperion','','',1996,'eAc',47,93,6,null,15,1 from dual union all
select 36,'15D25','RT-2','',null,'RT-2 [3]',3500,null,215,null,45,1 from dual union all
select 360,'Hyunmoo 1','','',null,'',4000,null,null,null,null,1 from dual union all
select 361,'Hyunmoo-2B','','',null,'',4000,null,null,null,null,1 from dual union all
select 362,'IAI-Arrow-1','','',null,'Arrow 2',1500,null,null,null,null,1 from dual union all
select 363,'IAI-Arrow-2','','',null,'Arrow 2',1500,null,null,null,null,1 from dual union all
select 364,'IAI-Arrow-2-2','','',null,'Arrow 2',500,null,null,null,null,1 from dual union all
select 365,'IAI-Arrow-3','','',null,'Arrow 3',1500,null,null,null,null,1 from dual union all
select 366,'ICE','','',null,'GSLV 3',null,null,69.5,452,720,1 from dual union all
select 367,'INTA S-12','','',1984,'INTA-100 [1]',28,null,16.7,null,1.2,1 from dual union all
select 368,'IRIS','IRIS','',1992,'Lageos PKM',1720,4580,58,291,79,1 from dual union all
select 369,'ISPS','','',1970,'NRO Agena program (2',null,null,.45,272,null,1 from dual union all
select 37,'15D27','RT-2','',1965,'RT-15 [1]',11000,null,412,null,60,4 from dual union all
select 370,'ISRO-LAM','ISRO-LAM','',1992,'Insat 2',null,null,.44,null,null,1 from dual union all
select 371,'Icarus','','',2007,'Icarus [1]',35,null,15.8,239.9,6.4,1 from dual union all
select 372,'Icarus St2','','',2007,'Icarus [2]',13,null,4.1,241.5,7,1 from dual union all
select 373,'Imp Malemute','SAM-D','Malemute',1974,'Malemute (TX-486-1?)',619,1395,55.1,281.2,22,1 from dual union all
select 374,'J-2','J-2','',1966,'Saturn IVB',null,null,1023,427,null,1 from dual union all
select 375,'J-2X','J-2','',2010,'Ares I',null,null,1308,448,465,1 from dual union all
select 376,'JCR','','',null,'JCR',1300,null,133,null,null,1 from dual union all
select 377,'JCR-S2','','',null,'JCR',700,null,54,null,null,1 from dual union all
select 378,'JPL 2 lb Kick Motor','','',1958,'Deal III',1.5,null,null,246,null,1 from dual union all
select 379,'Javelin','','LPC J-22',null,'Javelin 2',155,null,178,243,null,1 from dual union all
select 38,'15D290','','',null,'RT-23 [2]',25000,null,1050,null,null,1 from dual union all
select 380,'Javelin 2','','LPC J-2',1965,'Javelin 2 [AWST]',155,257,140.4,null,null,1 from dual union all
select 381,'Javelin 3','','LPC J-33',1965,'Javelin 3',165,null,93,239,null,1 from dual union all
select 382,'Javelin 3 (old)','','LPC J-3',1965,'Javelin 3  [AWST]',165,273,47.6,239,null,1 from dual union all
select 383,'Jericho II','','',1959,'Daniel [2]',320,null,20,null,23.8,1 from dual union all
select 384,'Jericho IV','Belier','',1961,'Centaure [2]',317,460,20,235,23.4,1 from dual union all
select 385,'Judi-I 1.9KS2150','','1.9-KS-2150',1961,'Judi III',10,18,9.6,224,1.9,1 from dual union all
select 386,'Judi-III 1.9KS2100','','1.9-KS-2100',1961,'Judi III',12,null,9.3,null,1.9,1 from dual union all
select 387,'K-15 Booster','','',2007,'K-15 booster',1000,null,null,null,null,1 from dual union all
select 388,'K-15 Stage 2','','',2007,'K-15',2000,null,null,null,null,1 from dual union all
select 389,'K-4 Booster','','',2016,'K-4',5000,null,null,null,null,1 from dual union all
select 39,'15D305','','',null,'RT-23 [1]',53700,null,2060,null,null,1 from dual union all
select 390,'K-4 Stage 2','','',2016,'K-4',5000,null,null,null,null,1 from dual union all
select 391,'K-BATS','','',2014,'K-BATS',1200,null,null,null,null,1 from dual union all
select 392,'K10S','','',null,'K10S',100,null,null,null,null,1 from dual union all
select 393,'K150','','',1958,'Kappa 6',86,null,null,null,15.8,1 from dual union all
select 394,'K160','','',1958,'Kappa 8L [2]',94,null,null,null,5.3,1 from dual union all
select 395,'K220','','',1956,'Kappa 2',200,null,null,null,null,1 from dual union all
select 396,'K250','','',1958,'Kappa 6',240,null,34,null,9.3,1 from dual union all
select 397,'K250-8','','',1958,'Kappa 8',240,null,34,null,9.7,1 from dual union all
select 398,'K250H','','',1958,'Kappa 9M [2]',300,null,null,null,7.2,1 from dual union all
select 399,'K420','','',1958,'Kappa 8',1100,null,null,null,17,1 from dual union all
select 4,'10KS2500','','',1957,'Polaris FTV',80,111,11,null,10,1 from dual union all
select 40,'15D339','','',null,'RT-23 [2]',25000,null,1050,null,null,1 from dual union all
select 400,'K420(1/3)','','',1966,'Kappa 10',350,null,null,null,null,1 from dual union all
select 401,'K420H','','',1962,'Kappa 9',1100,null,null,null,17,1 from dual union all
select 402,'KDU-414','11D414','',1964,'Molniya-1',null,80,2,290,null,1 from dual union all
select 403,'KDU-426','11D426','',null,'',null,null,3.09,292,null,1 from dual union all
select 404,'KM-D','KM','',1985,'Mu 3S2 [4]',304,629,17.4,294,38,1 from dual union all
select 405,'KM-H','KM','',1977,'Mu 3H [4]',59,null,null,281,null,1 from dual union all
select 406,'KM-L','KM-L','',1990,'Hagoromo',4,null,null,null,null,1 from dual;
insert into ENGINE
select 407,'KM-M','KM','',1985,'Mu 3S2 [4]',550,1375,32,293,63,1 from dual union all
select 408,'KM-P','KM','',1985,'Mu 3S2 [4]',461,null,31.3,286,44,1 from dual union all
select 409,'KM-V1','KM','',1995,'Mu V [4]',1430,null,52,298,73,1 from dual union all
select 41,'15D92','RT-2','',null,'',4500,null,null,null,null,1 from dual union all
select 410,'KM-V2b','KM','',2013,'Epsilon',3300,null,99.8,301,90,1 from dual union all
select 411,'KM-V2c','KM','',2016,'Epsilon',2900,null,99.6,299,88,1 from dual union all
select 412,'KN-02 Motor','','Tochka',null,'KN-02',1530,null,null,null,null,1 from dual union all
select 413,'KPN-4D10','','',null,'Musudan',null,null,255,null,128,1 from dual union all
select 414,'KPN-Solid1','','',null,'KN-11-1',10000,null,null,null,null,1 from dual union all
select 415,'KPN-Solid2','','',null,'KN-11-2',5000,null,null,null,null,1 from dual union all
select 416,'KRD-442','11D442','',1977,'FGB DKS',null,null,4.1,null,null,1 from dual union all
select 417,'KRD-61','','',1969,'E-8 VA',null,null,18.8,313,53,1 from dual union all
select 418,'KRD-79','S5.79','',null,'Salyut-6 engine',null,null,3.09,null,null,1 from dual union all
select 419,'KSLV SRM','','',2009,'KSLV',1000,null,80,null,44,1 from dual union all
select 42,'15D94','RT-2','',null,'RT-2P [3]',4640,null,176,271,49,1 from dual union all
select 420,'KSR','','',1993,'KSR-I',1200,1700,86.3,235,20,1 from dual union all
select 421,'KSR Booster','','',1993,'KSR-II',1200,1250,211,null,5.9,1 from dual union all
select 422,'KSR-III','','',2002,'KSR-3',null,null,126,null,59,1 from dual union all
select 423,'KT1-3','','',null,'KT-1 [3]',2000,null,null,null,null,1 from dual union all
select 424,'KTDU-35','KTDU-35','',null,'',null,null,4.1,280,null,1 from dual union all
select 425,'KTDU-417','KTDU-417','',null,'',null,null,18.9,314,null,1 from dual union all
select 426,'KTDU-417','KTDU-425','',1969,'E-8 KT',null,null,18.9,314,650,1 from dual union all
select 427,'KTDU-417 verniers','','',1969,'E-8 KT',null,null,3.4,253.8,30,2 from dual union all
select 428,'KTDU-425','KTDU-417','',null,'',null,null,18.9,312,null,1 from dual union all
select 429,'KTDU-425A','KTDU-417','',null,'',null,null,18.9,315,null,1 from dual union all
select 43,'15KS25000','BB','',1957,'Black Brant I',970,1670,111,217,15,1 from dual union all
select 430,'KTDU-53','KTDU-35','',null,'',null,null,4.1,280,null,1 from dual union all
select 431,'KTDU-5A','KTDU-5','',null,'Luna E-6',null,null,45.5,287,null,1 from dual union all
select 432,'KTDU-66','KTDU-35','',null,'',null,null,4.1,280,null,1 from dual union all
select 433,'KTDU-80','S5.80','',null,'Soyuz TM engine, Pro',null,null,6.2,302,null,1 from dual union all
select 434,'KVD-1M','','RD-56M',null,'GSLV GS3',null,null,75,460,675,1 from dual union all
select 435,'KZ1-3','','',null,'KZ-1 [3]',2850,null,null,290.4,55,1 from dual union all
select 436,'KZ1-4','','',null,'KZ-1 [4]',null,null,null,null,null,1 from dual union all
select 437,'KZ2-3','','',null,'KZ-1 [3]',2850,null,null,290.4,55,1 from dual union all
select 438,'Kappa 1','','',1956,'Kappa 1',45,null,10,null,3,1 from dual union all
select 439,'Kappa 4','','',1956,'Kappa 4',45,null,10,null,3,1 from dual union all
select 44,'15Zh58A','Topol','',null,'Start-1 [1]',26000,null,980,263,60,1 from dual union all
select 440,'Kappa 4 Booster','','',null,'Kappa 4 Booster',150,null,105,null,3,1 from dual union all
select 441,'Kavoshgar','','',2007,'Kavoshgar',400,null,null,null,12,1 from dual union all
select 442,'Kestrel','','',null,'Falcon 1 [2]',null,null,33,317,378,1 from dual union all
select 443,'Kisha I 4.4KS4800','','',null,'Kisha',66,95,21,null,4.4,1 from dual union all
select 444,'Kiva 5.5KS6100','','',null,'Kiva',107,149,27,null,5.5,1 from dual union all
select 445,'L480S','','',1966,'Lambda [4]',102,null,7.8,254,31.5,1 from dual union all
select 446,'L500','L735','',1966,'Lambda [3]',832,null,65,249.3,27,1 from dual union all
select 447,'L735','L735','',1966,'Lambda [1]',5982,null,363,215,28.8,1 from dual union all
select 448,'L735(1/3)','L735','',1966,'Lambda [2]',2474,null,115,242.9,38.4,1 from dual union all
select 449,'LAPAN','','',1987,'LAPAN  (?)',300,310,52,null,6,1 from dual union all
select 45,'15Zh58B','Topol','',null,'Start-1 [2]',13000,null,490,280,64,1 from dual union all
select 450,'LAPSTAR','','',1957,'Aeolus [0]',52,37,13.7,null,2.7,1 from dual union all
select 451,'LE-1','LS-A','',1963,'LS-A',null,350,10,195,35,1 from dual union all
select 452,'LE-2','','',null,'LS-C',null,null,9.8,195,25,1 from dual union all
select 453,'LE-3','LE-3','',1975,'N-1 Stage 2',null,null,54,285,250,1 from dual union all
select 454,'LE-3 (T)','','',null,'LS-C 7',null,null,10,null,null,1 from dual union all
select 455,'LE-5','LE-5','',null,'H1 St2',null,null,107,448,370,1 from dual union all
select 456,'LE-5A','LE-5','',1994,'H2 St2',null,null,122,452,600,1 from dual union all
select 457,'LE-5B','LE-5','',null,'H2A St2',null,null,137,447,530,1 from dual union all
select 458,'LE-7','LE-5','',1994,'H2 St1',null,null,1080,445.6,346,1 from dual union all
select 459,'LE-7A','LE-5','',null,'H2A St1',null,null,1100,440,390,1 from dual union all
select 46,'15Zh58V','Topol','',null,'Start-1 [3]',6000,null,245,280,56,1 from dual union all
select 460,'LEO-1','','',2015,'Super Strypi',710,1845,50,null,51.5,1 from dual union all
select 461,'LEO-46','','',2015,'Super Strypi [1]',22102,56360,1265.4,null,76,1 from dual union all
select 462,'LEO-7','','Orbus 7',null,'Super Strypi [2]',3587,9266,105.1,null,57.4,1 from dual union all
select 463,'LK/LOK-40','LK/LOK-40','',null,'LK/LOK',null,null,.39,null,null,1 from dual union all
select 464,'LPC A-1','','',1965,'Apollo LES',2130,2292,690,null,null,1 from dual union all
select 465,'LPC-415','','SRAM',1969,'SRAM',1000,null,null,null,null,1 from dual union all
select 466,'LPC-A1','','',1966,'Apollo LES',2200,null,690,null,3,1 from dual union all
select 467,'LPR-006','','',null,'LPR-006',null,null,6,null,null,1 from dual union all
select 468,'LPR-3T','','',null,'LPR-3T',null,null,29,null,null,1 from dual union all
select 469,'LR101-NA-7','','',1980,'Upper stage vernier',null,null,5.2,251,null,1 from dual union all
select 47,'15Zh59A','','',null,'Kuryer [1]',5000,null,null,null,null,1 from dual union all
select 470,'LR2-RM-6','','',1945,'Lark',null,null,2.8,195,260,1 from dual union all
select 471,'LR59-AJ-13','','LR59-AG-13',1957,'Bomarc A',null,null,133,null,4,1 from dual union all
select 472,'LR87','LR87','',null,'Titan Stage 1 ?',null,null,1226,304,164,1 from dual union all
select 473,'LR87AJ11','LR87','',null,'Titan Stage 1',null,null,1226,304,164,1 from dual union all
select 474,'LR87AJ11A','LR87','',null,'Titan 4B Stage 1 x2',null,null,1226,302,164,1 from dual union all
select 475,'LR87AJ5','LR87','',null,'Titan 2 Stage 1 x2',null,null,1045,296,147,1 from dual union all
select 476,'LR87AJ9','LR87','',null,'Titan Stage 1  ?',null,null,1226,304,164,1 from dual union all
select 477,'LR91','LR91','',1962,'Titan 2 Stage 2',null,null,440,316,182,1 from dual union all
select 478,'LR91AJ11','LR91','',1990,'Titan  ??',null,null,472.2,316,223,1 from dual union all
select 479,'LR91AJ11A','LR91','',1990,'Titan 4 Stage 2',null,null,472.2,316,223,1 from dual union all
select 48,'15Zh59B','','',null,'Kuryer [2]',5000,null,null,null,null,1 from dual union all
select 480,'LR91AJ5','LR91','',1962,'Titan 2 Stage 2',null,null,440,316,182,1 from dual union all
select 481,'LR91AJ9','LR91','',1962,'Titan 2 Stage 2  ?',null,null,440,316,182,1 from dual union all
select 482,'LRE-4','','',2015,'Simorgh [2]',null,null,17.7,null,null,1 from dual union all
select 483,'LS-A booster','','',1964,'LS-A [1]',475,600,60,null,10,1 from dual union all
select 484,'LS-C Booster','','',1969,'LS-C',300,null,null,null,null,1 from dual union all
select 485,'LVS','LVS','',1993,'PSLV PS4 x 2',null,3200,7.4,308,null,1 from dual union all
select 486,'LaRC Plastic','','',1958,'Hi Ball',2.5,2,.4,100,3,1 from dual union all
select 487,'Lance','6.65-KS-38800','',1958,'X-7A3 Booster',761,1148,173,215,6.7,1 from dual union all
select 488,'Lapwing','','',1967,'Petrel',97.5,null,null,237,30,1 from dual union all
select 489,'Leros 1','','',1991,'Leros 1',null,null,.5,314,null,1 from dual union all
select 49,'15Zh59V','','',null,'Kuryer [3]',3000,null,null,null,null,1 from dual union all
select 490,'Leros 1b','','',1996,'Messenger, MGS',null,null,.67,318,null,1 from dual union all
select 491,'Lobster','','',null,'Jaguar 1 [2]',26,58,29,null,2,1 from dual union all
select 492,'Loki','Loki','',1956,'Loki Dart',8,10,15,213,.8,1 from dual union all
select 493,'Loki II','Loki','',1965,'Loki II Dart',11,17,9,223,2,1 from dual union all
select 494,'Lupus 1A','','',1969,'Kookaburra [1]',33.6,38,9.5,null,5.2,1 from dual union all
select 495,'Lupus 2','','',1972,'Kookaburra [1]',35.7,46,11.5,null,4,1 from dual union all
select 496,'Lupus 2D','','',1972,'Kookaburra',35.7,48,11.5,null,4,1 from dual union all
select 497,'Lupus 3','','',1974,'Kookaburra [1]',41,61,11.5,null,5.3,1 from dual;
insert into ENGINE
select 498,'Luzou','','',1975,'Eridan-ARAKS',7,null,3,null,1.7,1 from dual union all
select 499,'M-10','Mu','',1970,'Mu 4S [1]',26555,null,737,219,61,1 from dual union all
select 5,'11D416','','',null,'',null,null,16.4,null,null,1 from dual union all
select 50,'17D12','RD-58 (Buran ODU)','',1988,'Buran',null,null,86.3,362,680,1 from dual union all
select 500,'M-100','','',null,'M-100',250,null,null,null,3,1 from dual union all
select 501,'M-100 St2','','',null,'M-100 [2]',100,null,null,null,4.5,1 from dual union all
select 502,'M-13','Mu','',1977,'Mu 3H [1]',33407,null,954,239,70,1 from dual union all
select 503,'M-14','Mu','',1995,'Mu V [1]',84000,null,3786,276,49,1 from dual union all
select 504,'M-20','Mu','',1970,'Mu 4S [2]',9850,null,285,261,66,1 from dual union all
select 505,'M-22','Mu','',1972,'Mu 3C [2]',9833,null,285,273,70,1 from dual union all
select 506,'M-23','Mu','',1980,'Mu 3S [2]',13100,null,524,282,73,1 from dual union all
select 507,'M-24','Mu','',1995,'Mu V [2]',34000,null,1246,288,71,1 from dual union all
select 508,'M-25','Mu','',1995,'Mu V [2]',38000,null,1240,288,71,1 from dual union all
select 509,'M-30','Mu','',1970,'Mu 4S [3]',2781,null,128,265,42,1 from dual union all
select 51,'17D61','KRD 17D61','',1979,'Ikar; Yantar 1KFT',null,null,29.4,307.9,600,1 from dual union all
select 510,'M-34','M-34','',1997,'M-V [3]',11100,30000,null,null,null,1 from dual union all
select 511,'M-34','Mu','',1995,'Mu V [3]',11600,null,294,301,102,1 from dual union all
select 512,'M-34c','Mu','',1995,'Epsilon',11600,null,294,301,102,1 from dual union all
select 513,'M-35','Mu','',2016,'Epsilon',17200,null,445,295,129,1 from dual union all
select 514,'M-3A','Mu','',1972,'Mu 3C [3]',1245,null,61,283,54,1 from dual union all
select 515,'M-3B','Mu','',1985,'Mu 3S2 [3]',3603,null,132,293.1,87,1 from dual union all
select 516,'M-40','Mu','',1970,'Mu 4S [4]',445,null,26.5,270,40,1 from dual union all
select 517,'M-6','M6','Honest John',1963,'HJ [X-202]',1713,1800,373,null,5.2,1 from dual union all
select 518,'M-6 4DS105000','JATO M6','X-202E1',1950,'HJ 4DS105000',800,1870,470,null,4.4,1 from dual union all
select 519,'M112','Mk 56 type','',null,'Hawk  Taurus Orion [',395,null,null,null,null,1 from dual union all
select 52,'17KS12000','BB5','Nihka',1981,'BB10 [3]',402,915,53,284.9,17.8,1 from dual union all
select 520,'M112','','Hawk',1958,'Hawk',630,null,null,null,null,1 from dual union all
select 521,'M15A1 JATO','T60 JATO','16-NS-1000',1953,'M16A1 JATO',null,null,4.4,null,16,1 from dual union all
select 522,'M22E8','Orion','',1977,'Taurus Orion [2]',389,null,null,null,null,1 from dual union all
select 523,'M34 ZEL JATO','JATO','4-XX-130000',1955,'Aircraft JATO',null,null,578,null,4,1 from dual union all
select 524,'M3A2 JATO','JATO','',1980,'M3A2 5" JATO (HVAR?)',12.3,null,null,null,null,1 from dual union all
select 525,'M55A1','Minuteman','M55A1',1961,'Minuteman 1A XM-55',22930,54730,792,266.1,59.3,4 from dual union all
select 526,'M56A1','Aries','M56A1',1961,'Minuteman 1A XM-56 [',5240,12400,207,267.3,61,4 from dual union all
select 527,'M56A1 PRP','Aries','M56A1',1961,'Minuteman 1A XM-56 [',5480,12400,202,267.3,61,4 from dual union all
select 528,'M57A1','M57','M57A1',1961,'Minuteman 1A [3]',2030,4485,76,276,59,4 from dual union all
select 529,'M5E1','Nike','X-216',null,'Nike (Operational)',630,640,195.6,null,3.3,1 from dual union all
select 53,'1KS420','','',1959,'Explorer 6',2,2,22,null,1,1 from dual union all
select 530,'M5E1  (XM42)','Nike','M30? X216',1956,'Nike Hercules booste',984,null,null,null,null,1 from dual union all
select 531,'MA-1/B','MA-1','',1957,'Atlas B',null,null,1335,245,128,2 from dual union all
select 532,'MA-1/B-A','MA-1','',1957,'Atlas A',null,null,1210,245,125,2 from dual union all
select 533,'MA-1/S','MA-1','',1957,'Atlas B',null,null,267,210,253.6,1 from dual union all
select 534,'MA-2/B','MA-1','',1959,'Atlas D',null,null,1375,248,135,2 from dual union all
select 535,'MA-2/S','MA-1','',1959,'Atlas D',null,null,254,213,281,1 from dual union all
select 536,'MA-3/B','MA-1','',1959,'Atlas E',null,null,1468,250,null,2 from dual union all
select 537,'MA-3/S','MA-1','',null,'Atlas E',null,null,254,214,null,1 from dual union all
select 538,'MA-5/B','MA-1','',1962,'Atlas SLV',null,null,1677,259,null,2 from dual union all
select 539,'MA-5/S','MA-1','',null,'Atlas SLV',null,null,269.2,220,null,1 from dual union all
select 54,'2-AS-22000','Nike-46','',1946,'Nike-46',100,200,98,null,2,1 from dual union all
select 540,'MA-5A/B','MA-5','',1991,'Atlas 2',null,null,2065,293.4,165,2 from dual union all
select 541,'MA-5A/S','MA-1/H-1','',null,'Atlas 2',null,null,380.6,311,274,1 from dual union all
select 542,'MARC 13A1','','52KS3850',1960,'Iris',506,null,17,null,52,1 from dual union all
select 543,'MARC 14A1','','',null,'Arcas booster',16,null,12,null,.8,1 from dual union all
select 544,'MARC 14B1','','.8KS18800',1960,'Iris booster x 7',13.4,10,12,null,.8,1 from dual union all
select 545,'MARC 21A1','','17.5-KS-130',null,'Metroc',6.3,8,.6,null,17.5,1 from dual union all
select 546,'MARC 2B1','Arcas','',1960,'Arcas',34.4,43,1.5,null,29,1 from dual union all
select 547,'MARC 2B1','Arcas','',null,'Arcas',30,null,1.4,null,29.2,1 from dual union all
select 548,'MARC 2C2','Arcas','',null,'HV Arcas',26,null,1.4,null,29,1 from dual union all
select 549,'MARC 42A1','','Arcas Boost',1960,'Boosted Arcas 2 [0]',30.8,null,12.2,null,3,1 from dual union all
select 55,'2.5-AS-18000','Nike-47','',1946,'Nike-47',100,200,80,null,2.5,1 from dual union all
select 550,'MARC 42A2','','Arcas Boost',1960,'Boosted Arcas [0]',20,null,null,null,null,1 from dual union all
select 551,'MARC 60A','Arcas','',1960,'Super Arcas',38,60,1.6,null,32.6,1 from dual union all
select 552,'MB-1','Genie','2KS-36250',1956,'Genie',273,100,161,null,2,1 from dual union all
select 553,'MB-3','MB-3','LR79-NA9',1957,'Thor DM-18 [1]',null,116000,710,null,160,1 from dual union all
select 554,'MB-3 DM-21','MB-3','LR79-NA9',null,'Thor DM-21 [1]',null,null,779,null,170,1 from dual union all
select 555,'MB-3-2','MB-3','LR79-NA9',null,'Thor DM-21 [1]',null,null,760,null,170,1 from dual union all
select 556,'MB-3-3','MB-3','LR79-NA9',null,'Thor LTTAT [1]',null,null,779,null,170,1 from dual union all
select 557,'MCR-3','','',1970,'MCR-3/RH-125',20,null,null,null,null,1 from dual union all
select 558,'MERA','','',1992,'MERA',20,40,20,null,2,1 from dual union all
select 559,'MG-18','','',1962,'Scout X-2M [4]',248,591,null,null,null,1 from dual union all
select 56,'2.5-DS-59000','','',null,'Nike Deacon mod',700,null,265,null,2.5,1 from dual union all
select 560,'MLRS','','',null,'MLRS',150,170,53,null,3.2,1 from dual union all
select 561,'MMR-06','','',null,'MMR-06',80,null,null,null,9,1 from dual union all
select 562,'MR-103','MR-103','',1974,'AOCS 1N',null,null,0,227,null,1 from dual union all
select 563,'MR-104','MR-104','',1977,'AOCS 445N',null,null,.44,239,null,1 from dual union all
select 564,'MR-104H','MR-104','',2012,'AOCS 890N?',null,null,.89,null,null,1 from dual union all
select 565,'MR-106','MR-106','',1974,'AOCS 30N',null,null,.03,232,null,1 from dual union all
select 566,'MR-107','MR-107','',1990,'Pegasus AOCS, OAM, H',null,null,.26,236,550,1 from dual union all
select 567,'MR-111','MR-111','',1980,'AOCS 4N',null,null,0,229,null,1 from dual union all
select 568,'MR-12','','',null,'MR-12',500,null,null,null,null,1 from dual union all
select 569,'MR-125','MR-125','',1994,'Astrid',null,null,.11,null,30,1 from dual union all
select 57,'2.5KS18000','Aerobee Booster','Aerobee JATO',1947,'Aerobee [0]',260,200,80,null,2.5,1 from dual union all
select 570,'MR-20','','',null,'MR-20',500,null,null,null,null,1 from dual union all
select 571,'MR-25','','',null,'MR-25',500,null,null,null,null,1 from dual union all
select 572,'MR-30','','',null,'MR-30',500,null,null,null,null,1 from dual union all
select 573,'MR-50','MR-50','',1974,'AOCS 22N',null,null,.02,228,null,1 from dual union all
select 574,'MSBS 501','','',null,'MSBS M51 [1]',30000,null,900,null,60,1 from dual union all
select 575,'MSBS 502','','',null,'MSBS M51 [2]',10000,null,300,null,70,1 from dual union all
select 576,'MSBS 503','','',null,'MSBS M51 [3]',5000,null,100,null,40,1 from dual union all
select 577,'MT 27','','',1964,'Lex',75,200,10,200,20,1 from dual union all
select 578,'MT-135','','',1964,'MT-135',68,null,8.1,230,10.5,1 from dual union all
select 579,'MX-774','MX-774','',1948,'Hiroc',null,null,9,null,75,1 from dual union all
select 58,'21AL-2600','Nike','',1947,'Nike 47 Ajax [2]',null,null,11.4,null,21,1 from dual union all
select 580,'Mage 1','','',1977,'Mage 1',368,null,28.2,284.7,34,1 from dual union all
select 581,'Mage 1S','','',1980,'Mage 1S',450,null,33.5,291,48.5,1 from dual union all
select 582,'Mage 2','','',1980,'Mage 2',520,null,46.2,294,43,1 from dual union all
select 583,'Mayfly','','',1957,'Long Tom',230,224,62,null,3.6,1 from dual union all
select 584,'Melanie','','OPd 220',1959,'Daniel [3]',80,49,38,null,4.4,1 from dual union all
select 585,'Melusine','','',null,'Grannos',20,null,null,null,null,1 from dual union all
select 586,'Merlin 1A','','',null,'Falcon 1 [1]',null,null,378,304,169,1 from dual union all
select 587,'Merlin 1C','','',null,'Falcon 1 [1]',null,null,400,304,169,1 from dual;
insert into ENGINE
select 588,'Merlin 1C+','','',null,'Falcon 1e [1]',null,null,572,304,169,1 from dual union all
select 589,'Merlin 1C-Vac','','',null,'Falcon 9 [1]',null,null,427,342,169,1 from dual union all
select 59,'22.6KS1245','Sparrow','',1965,'Sparoair III [2]',60,124,5.5,null,22.6,1 from dual union all
select 590,'Merlin 1D','','',null,'Falcon 9 [1]',null,null,616,304.8,169,1 from dual union all
select 591,'Merlin 1D-4','','',null,'Falcon 9 [1]',null,null,780,304.8,169,1 from dual union all
select 592,'Merlin 1D-5','','',null,'Falcon 9 [1]',null,null,845,304.8,169,1 from dual union all
select 593,'Merlin 1D-FT','','',null,'Falcon 9 [1]',null,null,720,304.8,169,1 from dual union all
select 594,'Merlin 1D-Vac','','',null,'Falcon 9-1.1 [2]',null,null,801,345,372,1 from dual union all
select 595,'Meteor-1','','',1963,'Meteor-1',32.5,31,14,null,2.3,1 from dual union all
select 596,'Meteor-2','','',1968,'Meteor-2',368,423,24,null,18,1 from dual union all
select 597,'Mk 104','','',null,'SM-2 [2], Lynx',486,null,null,null,null,1 from dual union all
select 598,'Mk 117 JATO','JATO','',null,'JATO BQM74 5"',22.7,null,11.2,null,1.5,1 from dual union all
select 599,'Mk 12 ASROC','','',null,'ASROC',480,null,49,null,null,1 from dual union all
select 6,'11D428','11D428','',null,'Yantar URD-II',null,null,.11,null,null,1 from dual union all
select 60,'23KS11000','Alcor','Alcor IA',null,'Alcor IA',500,null,49,null,23,1 from dual union all
select 600,'Mk 125 JATO','JATO','',null,'JATO Pioneer UAV',9.2,null,null,null,null,1 from dual union all
select 601,'Mk 128 JATO','JATO','',null,'JATO C-130',64.4,null,4.4,null,15,1 from dual union all
select 602,'Mk 136','','',null,'ASAS XL',100,null,7,null,15,1 from dual union all
select 603,'Mk 15','Sidewinder','',null,'Sidewinder',45,null,17.7,null,2.1,1 from dual union all
select 604,'Mk 17','AIM-9B Sidewinder','',null,'Sidewinder',45,null,17.7,null,2.1,1 from dual union all
select 605,'Mk 22 Mod 2','','',null,'M58A1 munition',52,30,null,null,2.1,1 from dual union all
select 606,'Mk 30','','',null,'SM-1',400,null,null,null,null,1 from dual union all
select 607,'Mk 36','AIM-9D Sidewinder','',null,'Sidewinder',45,null,null,null,null,1 from dual union all
select 608,'Mk 39','','',1965,'Shrike',70,null,35,null,2,1 from dual union all
select 609,'Mk 40 LSFFAR','','',1980,'2.75-inch FFAR',5,5,3.2,null,1.6,1 from dual union all
select 61,'23KS20000','BB','',1957,'Black Brant IIB',1113,2050,89,217,23,1 from dual union all
select 610,'Mk 50 Mod 0','Sidewinder 1D','',1965,'Sidewinder (Chaparra',43.8,63,12.2,230,4.7,1 from dual union all
select 611,'Mk 56','','',null,'SM-1',600,null,null,null,null,1 from dual union all
select 612,'Mk 56 ALTR','','',null,'SM-1',600,null,null,null,null,1 from dual union all
select 613,'Mk 66 FFAR','','',null,'2.75-inch WrapAround',6.5,7,5.8,null,1,1 from dual union all
select 614,'Mk 72','Terrier','',null,'SM-3 [1]',710,null,null,null,null,1 from dual union all
select 615,'Mk 72 UTC','Terrier','',null,'SM-3 [1]',710,null,null,null,null,1 from dual union all
select 616,'Mk7','','3.25"',1948,'',13.1,9,9.8,null,.9,1 from dual union all
select 617,'Mockingbird','','',2001,'Mockingbird',null,null,165,null,72,1 from dual union all
select 618,'Momo','','',2017,'Momo',null,null,12,null,100,1 from dual union all
select 619,'Musca','','',1965,'Musca',10,19,5.1,null,3.7,1 from dual union all
select 62,'26KS20000','BB5','BB5',null,'BB5',1167,2251,89,230.2,26,1 from dual union all
select 620,'N-25 Snark JATO','Snark','X-226',1955,'Snark',800,800,209,null,4,1 from dual union all
select 621,'N-69A Snark JATO','Snark','',1955,'Snark',2500,1900,470,null,4,1 from dual union all
select 622,'N-69C Snark JATO','Snark','',1955,'Snark',2500,2300,580,null,4,1 from dual union all
select 623,'NA 801','Mamouth','',null,'Agate',1800,null,190,null,18,1 from dual union all
select 624,'NA 802 Soleil C','Soleil','',1962,'Topaze Court',1530,4680,120,245,39,4 from dual union all
select 625,'NA 803 Soleil','Soleil','',1963,'Topaze',2250,6460,147,null,44,4 from dual union all
select 626,'NA 804','Soleil','',1963,'Topaze',2250,6460,147,null,44,4 from dual union all
select 627,'NA 805','Soleil','',1963,'Topaze',2250,6460,147,null,44,4 from dual union all
select 628,'NAL-16','','',1965,'NAL-16',70,null,10.7,null,14,1 from dual union all
select 629,'NAL-25','','',1965,'S-A',100,null,null,null,null,1 from dual union all
select 63,'27D6','','',null,'Kontakt [1]',3000,null,null,null,null,1 from dual union all
select 630,'NAL-7','','',1965,'NAL-7',20,null,null,null,null,1 from dual union all
select 631,'NAL-735','','',2002,'NAL-735',3500,null,250,null,49,1 from dual union all
select 632,'NATIV','','',1947,'NATIV',null,null,11.6,null,null,1 from dual union all
select 633,'NII-612','','',null,'',null,null,6,286,null,1 from dual union all
select 634,'NK-15','NK','',1969,'N-1 St1',null,null,1677,284,125,1 from dual union all
select 635,'NK-15V','NK','',1969,'N-1 St2',null,null,1755,346,120,1 from dual union all
select 636,'NK-19','NK','',1969,'Blok G',null,null,445,353,443,1 from dual union all
select 637,'NK-21','NK','',1969,'N-1 St3',null,null,402,353,370,1 from dual union all
select 638,'NK-31','NK','',null,'N-1 St3 rev',null,null,402,null,400,1 from dual union all
select 639,'NK-33','','',null,'N-1 St1 rev',null,null,1510,null,110,1 from dual union all
select 64,'28D6','','',null,'Kontakt [2]',1000,null,null,null,null,1 from dual union all
select 640,'NK-33A','14D15','',null,'Soyuz-2-1V',null,null,1962,331.2,null,1 from dual union all
select 641,'NK-43','NK','',null,'N-1 St2 rev',null,null,1750,null,130,1 from dual union all
select 642,'NOTS 100A','','',1960,'BSJr [4]',50,null,40,null,20,1 from dual union all
select 643,'NOTS 124-C','','',1955,'X-17 HTV [3]',30,null,null,null,null,1 from dual union all
select 644,'NOTS 3SM','','',1958,'Pilot [5]',.6,1,.7,245,1,1 from dual union all
select 645,'NOTS 401A','NOTS 400','',1970,'NOTS 401A',500,null,null,null,7.2,1 from dual union all
select 646,'NOTS 500','','',1960,'Caleb [1]',900,null,null,null,45,1 from dual union all
select 647,'NOTS 8','','',1958,'Pilot [4]',14.9,29,5.1,245,5.7,1 from dual union all
select 648,'NOTS Mod 551-B','','',1965,'Terrier/551',371,831,null,270,28,1 from dual union all
select 649,'NP-110','','',null,'Fateh-110',2000,null,null,null,null,1 from dual union all
select 65,'29D6','','',null,'Kontakt [3]',500,null,null,null,null,1 from dual union all
select 650,'NRL Viking','NRL Viking','',1949,'Viking',null,null,93.5,195,70,1 from dual union all
select 651,'NRL-US','','',2006,'MITEX Stage',null,null,.4,null,null,1 from dual union all
select 652,'NSPO Booster','','',null,'NSPO',1000,null,null,null,6,1 from dual union all
select 653,'NSPO Sustainer','','',null,'NSPO',1000,null,null,null,30,1 from dual union all
select 654,'Neptune','','',1959,'Antares [2]',500,630,20,null,32,1 from dual union all
select 655,'Nodong','Nodong','',null,'TD1[1] x 4',null,null,144,255,170,1 from dual union all
select 656,'Nudol'' St 1','','',2015,'A-235',15000,null,null,null,5,1 from dual union all
select 657,'Nudol'' St 2','','',2015,'A-235',2000,null,null,null,5,1 from dual union all
select 658,'OMS','OMS','',1981,'STS OMS',null,null,26.7,316,90,1 from dual union all
select 659,'OTRAG','OTRAG','',1977,'OTRAG',null,null,58.7,250,126,1 from dual union all
select 66,'2S-SL','','',2017,'PDV Target [2]',1000,null,null,null,null,1 from dual union all
select 660,'Oka','','9M714',null,'',4000,null,null,null,null,1 from dual union all
select 661,'Orbus 1','','',1988,'Orbus 1',476,null,31,null,null,1 from dual union all
select 662,'Orbus 1A','','',2005,'Orbus 1',476,null,31,null,null,1 from dual union all
select 663,'Orbus 21D','Orbus','',1982,'IUS [1]',10619,28140,187,293,150,1 from dual union all
select 664,'Orbus 6E','Orbus','',1982,'IUS [2]',3865,null,76.5,303.8,104,1 from dual union all
select 665,'Oreste','','',null,'Monica',5,7,.9,null,8,1 from dual union all
select 666,'Orion 38','Pegasus','',1990,'Pegasus [3]',896,2185,36,289.6,69.6,1 from dual union all
select 667,'Orion 50','Pegasus','',1990,'Pegasus [2]',3370,8666,138,292.3,73.3,1 from dual union all
select 668,'Orion 50S','Orion','',1990,'Pegasus  [1]',13420,35108,485,294.4,72.4,1 from dual union all
select 669,'Orion 50SXL','Orion','',1994,'Pegasus XL [1]',16400,43586,594,295,73.4,1 from dual union all
select 67,'3-DS-47000','Nike','X-201A3 T-39',1950,'Nike X201 Ajax',700,null,209,null,3,1 from dual union all
select 670,'Orion 50SXLG','Orion','',2003,'Taurus Lite [1]',16400,43586,594,295,73.4,1 from dual union all
select 671,'Orion 50SXLT','Orion','',2003,'IRBM-T1 [1]',16400,43586,594,295,73.4,1 from dual union all
select 672,'Orion 50XL','Pegasus','',1994,'Pegasus XL [2]',4331,11218,196,289,72.5,1 from dual union all
select 673,'Orion 50XLT','Pegasus','',1994,'IRBM-T1[2]',4331,11218,196,289,72.5,1 from dual union all
select 674,'Orion LAS-AM','','',null,'Orion LAS',3464,null,1780,null,2.5,1 from dual union all
select 675,'Orion-1','','',1966,'Orion-1',140,null,null,null,12,1 from dual union all
select 676,'Orion-2','','',1966,'Orion-2',140,null,null,null,12,1 from dual union all
select 677,'P-15','S2.721','',null,'',null,null,11.9,null,null,1 from dual;
insert into ENGINE
select 678,'P064','Rubis','',1965,'Diamant A [3]',712,2300,52,null,45,1 from dual union all
select 679,'P068','Rubis','',1970,'Diamant B [3]',775,2300,52.5,280,46.5,1 from dual union all
select 68,'3.52KS8500','Viper','',1960,'Viper',87,142,37.8,null,3.5,1 from dual union all
select 680,'P120C','','',2017,'Vega-C',147000,368900,2774,277.5,133,1 from dual union all
select 681,'P4','Rita I','P-4',1966,'MSBS M1 [2]',4000,12000,176,null,65,1 from dual union all
select 682,'P4-1 Booster','','',null,'',null,null,2.4,232,null,1 from dual union all
select 683,'P4-1 Sustainer','','',null,'',null,null,.5,262,null,1 from dual union all
select 684,'P80FW','','',2012,'Vega [1]',97131,242721,2100,280,106.8,1 from dual union all
select 685,'P8E-9','Lance Sustainer','Dual Thrust',1964,'Lance Sustainer',null,null,20,227,114,1 from dual union all
select 686,'P8E-9 Boost','Lance Boost','Dual Thrust',1964,'Lance Booster',null,null,190,227,6.1,1 from dual union all
select 687,'PAP','Ariane','P9.5',1988,'Ariane 4 PAP',12600,null,650,241,42,1 from dual union all
select 688,'PPS-1350','','',null,'SMART-1/Stentor',null,null,.092,1800,null,1 from dual union all
select 689,'PRD-18','','',1957,'V-750 booster',300,null,490,null,4.5,1 from dual union all
select 69,'3.52KS8500','','Viper IIB',1968,'Viper',89,144,35.8,null,3.2,1 from dual union all
select 690,'PRD-33','','',1959,'V-1000 booster',2700,null,1962,null,4.5,1 from dual union all
select 691,'PS3','PS3','',1993,'PSLV PS3',8400,28000,324,294,83,1 from dual union all
select 692,'PSOM','PSOM','',1993,'PSLV PSOM',11000,20700,677,262,44.2,1 from dual union all
select 693,'PSOM-XL','PSOM','',2006,'PSLV PSOM',12400,20700,677,262,58,1 from dual union all
select 694,'PSSC','','',2011,'PSSC',2,40,null,null,null,4 from dual union all
select 695,'PTDU','','11D82M',1960,'Zenit/Resurs',405,736,29,277.4,23.3,1 from dual union all
select 696,'PWR35M','','',2009,'Taurus 2 Alt-2',null,null,null,null,null,1 from dual union all
select 697,'Paekkom','','',null,'Paekkom booster (1 o',1000,null,null,null,null,1 from dual union all
select 698,'Peregrine','MSFC','',2013,'Peregrine',1200,null,null,null,null,1 from dual union all
select 699,'Pershing 2 St1','','',1982,'Pershing 2 [1]',4110,null,172,276,58,1 from dual union all
select 7,'11D428A','11D428','',null,'Progress M/Soyuz TM/',null,null,.13,291,null,1 from dual union all
select 70,'3.8KS8040','','Viper II',1959,'Viper II',90,135,35.8,207,3.8,1 from dual union all
select 700,'Pershing 2 St2','','',1983,'Pershing 2 [2]',2600,null,134,279,46,1 from dual union all
select 701,'Pollux','Castor','XM-33E4',1959,'Pollux',3742,null,222,null,27,1 from dual union all
select 702,'Polyot','','',null,'IS',null,null,3.9,null,null,1 from dual union all
select 703,'Prithvi','','',1988,'Prithvi',null,null,100,null,null,1 from dual union all
select 704,'Proson 1','','',null,'Proson',60,null,null,null,null,1 from dual union all
select 705,'Proson 2','','',null,'Proson',30,null,null,null,null,1 from dual union all
select 706,'R-31 St 2','','',1973,'R-31 [2]',6500,null,233,null,73.6,1 from dual union all
select 707,'R-31BS','','',1976,'R-31 BS',300,null,1,null,101,1 from dual union all
select 708,'R-39 St 2','','',null,'RT-23 [2]',25000,null,1050,null,null,1 from dual union all
select 709,'R-39 St 3','','',null,'R-39',10000,null,null,null,null,1 from dual union all
select 71,'30AS1000','','',1944,'Private A',174,133,4.4,184,30,1 from dual union all
select 710,'R-4D-11','R-4D','',null,'HS-601 LAE',null,20000,.49,315.5,null,1 from dual union all
select 711,'R-6C','','',null,'ATS-1 AOCS?',null,null,.02,null,null,1 from dual union all
select 712,'RD 15A18-BR','','',null,'R-36M2 BR',null,null,18.6,null,600,1 from dual union all
select 713,'RD R-29 St 2','','',null,'R-29R St 2',null,null,100,null,94,1 from dual union all
select 714,'RD R-29RM St 2','','',null,'R-29RM St 2',null,null,100,null,94,1 from dual union all
select 715,'RD R-29RM St 3','','',null,'R-29RM St 3',null,null,100,null,87,1 from dual union all
select 716,'RD RT-1-3','','',null,'RT-1 [3]',3000,null,240,null,42,1 from dual union all
select 717,'RD RT-15-2','','',null,'RT-15',10000,null,500,null,30,1 from dual union all
select 718,'RD RT-21 (1)','','',null,'Temp-2S [1]',26000,null,null,null,63,1 from dual union all
select 719,'RD RT-21 St2','','',null,'Pioner [2]',8700,null,null,null,null,1 from dual union all
select 72,'30KS8000','Alcor','Alcor I',null,'Alcor I',478,1070,36,null,30,1 from dual union all
select 720,'RD RT-21-3','','',null,'RT-21 [3]',8700,null,1000,null,245,1 from dual union all
select 721,'RD-0106','RD-0106','8D715',1961,'R-9 St2',null,null,300,325,140,4 from dual union all
select 722,'RD-0107','RD-0106','8D715K',1962,'Blok I',null,null,297.9,326,250,4 from dual union all
select 723,'RD-0110','RD-0106','',1960,'Blok I',null,null,297.9,325,240,4 from dual union all
select 724,'RD-0120','11D122','',1987,'Energiya St2',null,null,1961,455,480,1 from dual union all
select 725,'RD-0124','RD-0106','14D23',2006,'Soyuz 2-1B St3',null,null,294.3,359,null,1 from dual union all
select 726,'RD-0124A','','',2014,'Angara URM-2',null,null,294.3,359,424,1 from dual union all
select 727,'RD-0203','RD-0203','',1963,'UR-200 [1] x 1?',null,null,560,null,null,1 from dual union all
select 728,'RD-0204','RD-0203','',1963,'UR-200 [1] x 3?',null,null,560,null,null,1 from dual union all
select 729,'RD-0206','RD-0203','',1963,'UR-200 [2]',null,null,590,null,null,1 from dual union all
select 73,'33KS2800','','',1957,'Vanguard [3]',250,347,12.5,null,33,1 from dual union all
select 730,'RD-0207','RD-0207','',1963,'UR-200 [2] vernier',null,null,30.9,297,null,1 from dual union all
select 731,'RD-0210','RD-0203','8D411K',1965,'UR-500 [2] x 1?',null,null,582.5,326.5,210,1 from dual union all
select 732,'RD-0211','RD-0203','8D412K',1965,'UR-500 [2] x 3?',null,null,582.5,326.5,210,1 from dual union all
select 733,'RD-0213','RD-0203','8D48',1967,'UR-500 [3]',null,null,583,326.5,230,1 from dual union all
select 734,'RD-0214','RD-0207','',1967,'UR-500 [3] vernier',null,null,31,293,null,1 from dual union all
select 735,'RD-0216','RD-0216','15D1',null,'UR-100 St 1',null,null,219,313,null,1 from dual union all
select 736,'RD-0217','RD-0216','15D2',null,'UR-100 St 1 center',null,null,219,313,null,1 from dual union all
select 737,'RD-0221','','',null,'N-1/L-3 RB',null,null,78,330,null,1 from dual union all
select 738,'RD-0225','RD-0225','',null,'',null,null,3.9,287,null,1 from dual union all
select 739,'RD-0228','','',null,'R-36M [2]',null,null,755,null,153,1 from dual union all
select 74,'3D17','','',1973,'R-31 [1]',16000,null,579,null,84,1 from dual union all
select 740,'RD-0231','','',null,'',null,null,29,275,null,1 from dual union all
select 741,'RD-0233','RD-0233','',1972,'UR-100N [1]',null,null,520,310,null,1 from dual union all
select 742,'RD-0235','RD-0235','',1972,'UR-100N [2]',null,null,240,320,null,1 from dual union all
select 743,'RD-0236','RD-0236','',1972,'UR-100N [2] vernier',null,null,15.8,293,null,4 from dual union all
select 744,'RD-0237','','',1972,'UR-100N BR',null,null,4.9,200,null,1 from dual union all
select 745,'RD-0242','','',null,'',null,null,123.6,303,null,1 from dual union all
select 746,'RD-0244','RD-0244','',null,'Shtil'' St 1',null,null,825.8,300,79,1 from dual union all
select 747,'RD-0245','','',null,'R29RM St 1 vernier',null,null,211,300,null,1 from dual union all
select 748,'RD-0255','RD-0203','',null,'R-36M2 [2]',null,null,760,null,153,1 from dual union all
select 749,'RD-0256','RD-0203','',null,'R-36M2 [2] ?',null,null,760,null,153,1 from dual union all
select 75,'3D65','','',null,'R-39 [1]',53700,null,2060,null,null,1 from dual union all
select 750,'RD-100','A-4','8D51',1948,'R-1',null,null,307,206,63,1 from dual union all
select 751,'RD-101','A-4','8D52',1949,'R-2',null,null,363,210,null,1 from dual union all
select 752,'RD-103','A-4','8D54',1953,'R-5',null,null,500,220,null,1 from dual union all
select 753,'RD-103M','A-4','8D71',1953,'R-5M',null,null,500,220,null,1 from dual union all
select 754,'RD-107','RD-108','',null,'Soyuz BVGD',null,null,991.7,310,120,4 from dual union all
select 755,'RD-107A','RD-108','14D22',null,'Soyuz FG BVGD',null,null,1000,310,120,4 from dual union all
select 756,'RD-108','RD-108','',null,'Soyuz A',null,null,997.1,311,286,4 from dual union all
select 757,'RD-108A','RD-108','14D21',null,'Soyuz FG A',null,null,1000,311,286,4 from dual union all
select 758,'RD-108PF','RD-108','11D512PF',null,'Soyuz U2 A',null,null,1009.4,311,286,4 from dual union all
select 759,'RD-111','RD-111','8D716',1961,'R-9 St 1',null,null,1381,312,108,4 from dual union all
select 76,'3KS33000','','Regulus JATO',1950,'Regulus 1',400,440,147,null,3,1 from dual union all
select 760,'RD-111 (1)','RD-111','8D717',1961,'R-9 St 1',null,null,1373,327,null,4 from dual union all
select 761,'RD-112','RD-112','',1960,'',null,null,1089,344,null,1 from dual union all
select 762,'RD-113','RD-112','',1960,'',null,null,1138,360,null,1 from dual union all
select 763,'RD-114','RD-112','',1961,'',null,null,1653,341,null,1 from dual union all
select 764,'RD-115','RD-112','',1961,'',null,null,1726,357,null,1 from dual union all
select 765,'RD-119','RD-109','8D710',1961,'63S1',null,null,105,352,null,1 from dual union all
select 766,'RD-120','11D123','',1985,'Zenit St 2',null,null,834,350,null,1 from dual union all
select 767,'RD-151','RD-170','',2009,'Naro-1',null,null,1670,337,229,1 from dual;
insert into ENGINE
select 768,'RD-170','RD-170','11D520',1985,'Energiya',null,null,7908,337,null,4 from dual union all
select 769,'RD-171','RD-170','',1985,'Zenit St 1',null,null,7908,337,null,4 from dual union all
select 77,'48N6','','',null,'48N6',1500,null,null,null,null,1 from dual union all
select 770,'RD-180','RD-170','',2000,'Atlas 3 [1]',null,null,4152,337.8,190,2 from dual union all
select 771,'RD-181','RD-170','',2015,'Antares [1]',null,null,1922,311.9,null,1 from dual union all
select 772,'RD-191','RD-170','',2009,'Angara URM',null,null,2079,337,150,1 from dual union all
select 773,'RD-214','RD-214','8D59',1957,'R-12',null,null,730,264,null,4 from dual union all
select 774,'RD-214 Early','RD-214','',1957,'R-12',null,null,625,210,null,4 from dual union all
select 775,'RD-215','RD-215 8D513','',1960,'R-14',null,null,864,291,null,1 from dual union all
select 776,'RD-216','RD-215','11D614',null,'Kosmos-3M St1',null,null,1728,291,130,4 from dual union all
select 777,'RD-217','RD-215 8D713','',1960,'R-16',null,null,865,289,106,1 from dual union all
select 778,'RD-219','RD-215 8D713','',1960,'R-16',null,null,882,293,80,1 from dual union all
select 779,'RD-224','RD-215','',null,'',null,null,1778,294,null,1 from dual union all
select 78,'4D10','','',null,'R-27',null,null,255,null,128,1 from dual union all
select 780,'RD-225','RD-215','',null,'',null,null,889,294,null,1 from dual union all
select 781,'RD-250','RD-250','',1963,'R-36 [1]',null,null,882,269.6,120,2 from dual union all
select 782,'RD-251','RD-251','',1965,'R-36 [1] = RD-250 x',null,null,2643,301,120,1 from dual union all
select 783,'RD-252','RD-250','',1963,'R-36 [2]',null,null,902,317.6,160,2 from dual union all
select 784,'RD-253','RD-253','11D48',1965,'UR-500 [1] x 6',null,null,1635,316,120,1 from dual union all
select 785,'RD-253 14D14','RD-253','14D14',1986,'Proton-M [1] x 6',null,null,1750,316,120,1 from dual union all
select 786,'RD-261','RD-251','',1965,'R-36 [1] = ? x 3',null,null,3032,270,120,2 from dual union all
select 787,'RD-262','RD-250','',1963,'R-36 [2]',null,null,941,318,160,2 from dual union all
select 788,'RD-263','RD-250','',null,'R-36M  [1] x 4 = RD-',null,null,1131,null,98,1 from dual union all
select 789,'RD-268','RD-250','',1970,'MR-UR-100 [1]',null,null,1236,319,null,1 from dual union all
select 79,'4DS105000','JATO','X-202C1',1950,'JATO/Talos',800,1900,470,null,4,1 from dual union all
select 790,'RD-273','RD-250','',null,'R-36MU [1] x 3',null,null,1239,null,98,1 from dual union all
select 791,'RD-275','RD-253','',1986,'Proton-M [1] x 6',null,null,1745,317,130,1 from dual union all
select 792,'RD-8','','',1985,'Zenit [2]',null,null,78.4,342,null,1 from dual union all
select 793,'RD-843','','',2012,'Vega [4] mdot=0.792',null,null,2.45,315.5,null,1 from dual union all
select 794,'RD-851','RD-851','',null,'',null,null,32.5,279,null,1 from dual union all
select 795,'RD-852','RD-851','',null,'',null,null,48.3,255,null,1 from dual union all
select 796,'RD-854','','',1965,'OGCh',null,null,75,312,null,1 from dual union all
select 797,'RD-855','','',null,'',null,null,285.4,292,null,1 from dual union all
select 798,'RD-856','','',null,'',null,null,54.2,281,null,1 from dual union all
select 799,'RD-857','15D12','',1967,'RT-20P [2]',null,null,147,330,215,1 from dual union all
select 8,'11D429','','',1974,'Molniya-1 mod',null,80,2,290,null,1 from dual union all
select 80,'4DS105000','JATO','X-202C6',1950,'HJ prototype',800,1870,470,null,4.4,1 from dual union all
select 800,'RD-858','','',null,'',null,null,20.1,315,null,1 from dual union all
select 801,'RD-859','','',null,'',null,null,20.1,312,null,1 from dual union all
select 802,'RD-861','S5M','D-25',1977,'S5M',null,null,78.7,314.4,130,1 from dual union all
select 803,'RD-861K','','',2008,'Tsiklon-4 [3]',null,null,77.6,330,470,1 from dual union all
select 804,'RD-862','','',null,'',null,null,142.6,331,null,1 from dual union all
select 805,'RD-863','','',null,'',null,null,276.8,301,null,1 from dual union all
select 806,'RD-864','','',null,'',null,null,20.2,309,null,1 from dual union all
select 807,'RD-866','','',null,'RT-23 [3]',15000,null,206,null,null,1 from dual union all
select 808,'RD-866','','',null,'',null,null,5.2,306,null,1 from dual union all
select 809,'RD-868P','','',2015,'Vega [4] mdot=0.792',null,null,2.45,315.5,null,1 from dual union all
select 81,'4KS115000','','Regulus JATO',1950,'Regulus 2',800,2000,512,null,4,1 from dual union all
select 810,'RD-869','','',2012,'Pre Vega',null,null,2.45,315.5,null,1 from dual union all
select 811,'RDMT-100','RDMT-100','',null,'',null,null,.1,260,null,1 from dual union all
select 812,'RDMT-135','RDMT-100','',null,'',null,null,.14,260,null,1 from dual union all
select 813,'RDMT-200','RDMT-100','',null,'',null,null,.2,255,null,1 from dual union all
select 814,'RDMT-400','RDMT-100','',null,'',null,null,.4,255,null,1 from dual union all
select 815,'REA 20-4','REA20-4','',null,'',null,null,.56,null,null,1 from dual union all
select 816,'RH-125','','',1970,'RH-125',30,30,10.5,null,2.9,1 from dual union all
select 817,'RH-125S','','',1970,'RH-125S',30,30,10.5,null,2.9,1 from dual union all
select 818,'RH-200','','',1970,'RH-200',60,96,16.9,null,5.7,1 from dual union all
select 819,'RH-200SV','','',1970,'RH-200SV',140,null,16.9,null,10,1 from dual union all
select 82,'5.6KS5400','','Viper IC',1950,'Viper I',91,134,24,209,5.6,1 from dual union all
select 820,'RH-300','','',null,'RH-300',300,null,33.3,null,16,1 from dual union all
select 821,'RH-300 Booster','','',null,'RH-300 booster',200,null,37.5,null,6,1 from dual union all
select 822,'RH-300-II','','',null,'RH-300-II',500,null,39.1,null,20.2,1 from dual union all
select 823,'RH-560','','',null,'RH-560',900,null,76,null,19,1 from dual union all
select 824,'RH-560S','','',null,'RH-560S',1300,null,119,null,19,1 from dual union all
select 825,'RH-75','','',1967,'',10,null,null,null,null,1 from dual union all
select 826,'RL10','RL10','',null,'Centaur I',null,null,66.7,433,470,1 from dual union all
select 827,'RL10A-1','RL10','',null,'Centaur',null,null,66.7,425,430,1 from dual union all
select 828,'RL10A-3','RL10','',null,'Centaur',null,null,65.6,444,470,1 from dual union all
select 829,'RL10A-3-3A','RL10','',null,'Centaur T',null,null,73.4,444,550,1 from dual union all
select 83,'500N AKE','','',2002,'DRTS',null,null,.55,319.5,null,1 from dual union all
select 830,'RL10A-3A','RL10','',null,'Centaur',null,null,73.4,444,550,1 from dual union all
select 831,'RL10A-4','RL10','',null,'Centaur IIA',null,null,92.6,449,370,1 from dual union all
select 832,'RL10A-4-1','RL10','',null,'Centaur IIA/IIIA',null,null,99.2,450.5,370,1 from dual union all
select 833,'RL10A-4-1B','RL10','',null,'Centaur IIIA',null,null,99.2,450.5,370,1 from dual union all
select 834,'RL10A-4-2','RL10','',null,'Centaur',null,null,99.2,451,370,1 from dual union all
select 835,'RL10B-2','RL10','',1998,'Delta III',null,null,110.1,465.5,700,1 from dual union all
select 836,'RL10C-1','RL10','',2014,'Delta IV',null,null,101.8,449.7,700,1 from dual union all
select 837,'RL10C-2','RL10','',2014,'SLS?',null,null,110,462.5,700,1 from dual union all
select 838,'RM-1','','',2003,'SS1',null,7200,90,null,80,1 from dual union all
select 839,'RM-1100','Asp','',1959,'Asp',85,141,26.6,215,5.3,1 from dual union all
select 84,'51T6 Motor','','',1968,'A-350 booster',15000,null,null,null,5,1 from dual union all
select 840,'RM-1400','Asp','',1960,'Asp IV',90,158,12.7,235,12,1 from dual union all
select 841,'RM-2','','',2007,'SS2',null,16000,270,null,60,1 from dual union all
select 842,'RM-3141','Scale Sergeant','',1959,'Ascamp [2]',25,48,7.6,222,6.3,1 from dual union all
select 843,'RO-5','','8D714',1958,'Blok E',null,null,49.9,316,450,1 from dual union all
select 844,'RO-7','','8D719',1959,'Blok E',null,null,54.5,323,430,1 from dual union all
select 845,'RS-14','','',1968,'Min 3 PBS',null,null,1.4,287.8,null,1 from dual union all
select 846,'RS-18','RS-18','',1968,'LMAS',null,null,15.6,309.9,null,1 from dual union all
select 847,'RS-2101A','','',1971,'Mariner 9',null,null,1.33,283,null,1 from dual union all
select 848,'RS-2101C','','',1975,'Viking',null,null,1.33,294,null,1 from dual union all
select 849,'RS-27','RS-27','',1974,'Thor',null,null,1023,295,274,1 from dual union all
select 85,'51T6B ZhRD','5D22?','',null,'51T6 [2]',null,null,131,null,40,1 from dual union all
select 850,'RS-27A','RS-27','',1990,'Thor',null,null,1086,302,261,1 from dual union all
select 851,'RS-34','','',1983,'Peacekeeper [4]',null,null,11.8,308,168,1 from dual union all
select 852,'RS-36','','',1982,'HOE Axial Engine',null,null,55.6,281,null,1 from dual union all
select 853,'RS-68','RS-68','',null,'Delta 4',null,null,2950,409,null,1 from dual union all
select 854,'RS-68A','RS-68','',2012,'Delta 4',null,null,3124,414,null,1 from dual union all
select 855,'RS-B-535','','',1972,'BLDT',null,null,null,null,null,1 from dual union all
select 856,'RSA-3-1','RSA','',1988,'RSA-3',10215,null,451,265,52,1 from dual union all
select 857,'RSA-3-2','RSA','',1988,'RSA-3',10971,null,477,277,52,1 from dual;
insert into ENGINE
select 858,'RSA-3-3','','',1988,'RSA-3',2048,null,59,298,94,1 from dual union all
select 859,'RT-1-1','','',null,'RT-1 [1]',19000,null,900,null,30,1 from dual union all
select 86,'53T6','','',null,'53T6',10000,null,null,null,null,1 from dual union all
select 860,'RTV','RTV Motor','',1956,'Juno I [4]',37,51,8.5,230,5.4,1 from dual union all
select 861,'RV-A-10','Sergeant','Hermes A-2',1953,'RV-A-10',2700,3600,157.5,null,24,1 from dual union all
select 862,'RX-250-LPN','','',1987,'RX-250-LPN',300,310,52,null,6,1 from dual union all
select 863,'RX-320','','',2008,'RX-320',500,510,null,null,null,1 from dual union all
select 864,'RX-420','','',2009,'RX-420',900,null,null,null,null,1 from dual union all
select 865,'RZ.2','','',1964,'Blue Streak (x2)',null,null,756,248,146,1 from dual union all
select 866,'Raven 1','Raven','',null,'Skylark [?]',1160,null,80,214.5,35,1 from dual union all
select 867,'Raven 11','Raven','',null,'Skylark 5',1280,2502,105,null,39,1 from dual union all
select 868,'Raven 2','Raven','',null,'Skylark [?]',1160,null,80,214.5,35,1 from dual union all
select 869,'Raven 5','Raven','',null,'Skylark [?]',1160,null,80,214.5,35,1 from dual union all
select 87,'5D22','5D13 type','',null,'A-350Zh St2',null,null,131,null,40,1 from dual union all
select 870,'Raven 6','Raven','',null,'Skylark',1160,null,80,214.5,35,1 from dual union all
select 871,'Raven 7','Raven','',null,'Skylark [?]',1160,null,80,214.5,35,1 from dual union all
select 872,'Raven 8','Raven','',null,'Skylark',1018,null,50,209.5,35,1 from dual union all
select 873,'Recruit','Recruit','Recruit 1A',null,'Squirt',170,269,173,225,2.2,1 from dual union all
select 874,'Redstone A-7','Redstone','',1953,'Juno I [1]',null,58000,416.2,265,155,1 from dual union all
select 875,'Rita II','P-6','P6',1973,'MSBS M2 [2]',6000,null,320,null,52,1 from dual union all
select 876,'Rook','Raven','',1960,'Rook',1057,1720,null,null,5.5,1 from dual union all
select 877,'Rook II','Raven','',1960,'Rook',1170,1735,null,null,7,1 from dual union all
select 878,'Rutherford','','',2017,'Electron St 1',null,null,20.3,303,null,1 from dual union all
select 879,'Rutherford Vac','','',2017,'Electron St 2',null,null,22,327,null,1 from dual union all
select 88,'5KS50000','TX-77?','',1958,'X-7A3 Booster',900,null,222,null,5,1 from dual union all
select 880,'S-160','','',1964,'S-160',90,null,null,null,null,1 from dual union all
select 881,'S-20','','',1966,'Sonda 2',316,612,36,257,20,1 from dual union all
select 882,'S-210','','',1969,'S-210',210,null,null,null,17,1 from dual union all
select 883,'S-23','','',1966,'Sonda 2M',182,null,18,null,null,1 from dual union all
select 884,'S-30','Sonda','',1976,'Sonda 3',1205,2256,102,266.3,22,1 from dual union all
select 885,'S-300','','',null,'S-300',400,null,null,null,null,1 from dual union all
select 886,'S-31','Sonda','',2004,'Sonda VSB-30',961,1730,136,261,12,1 from dual union all
select 887,'S-310','','',1975,'S-310',671,null,80,null,31.6,1 from dual union all
select 888,'S-3D','S-3D','',1957,'Jupiter [1]',null,120100,667,247,180,1 from dual union all
select 889,'S-40','Sonda','',1991,'VS-40 [1]',5340,11600,191,270.8,58.2,1 from dual union all
select 89,'5S24','','',null,'V-825 [1]',14000,null,null,null,null,1 from dual union all
select 890,'S-400-12','','',null,'European LAE',null,null,.44,318,null,1 from dual union all
select 891,'S-400-15','','',null,'European LAE',null,null,.44,321,null,1 from dual union all
select 892,'S-43','Sonda','',null,'VLS',8820,19500,308,279.1,62,1 from dual union all
select 893,'S-44','Sonda','',1991,'VLS [4]',917,2250,30.8,281.9,73,1 from dual union all
select 894,'S-50000','','',2004,'CSXT',272,430,71,null,13.4,1 from dual union all
select 895,'S-520','','',1980,'S-520',1796,3984,143.2,264.8,28.7,1 from dual union all
select 896,'S-A','','',1965,'S-A',36,29,2.5,164,12,1 from dual union all
select 897,'S-B','','',1965,'S-B',70,null,null,null,null,1 from dual union all
select 898,'S-C','','',1965,'S-C',70,null,null,null,null,1 from dual union all
select 899,'S09.29','S09.29','',null,'',null,null,88,null,null,1 from dual union all
select 9,'11D430','KRD 17D52','',null,'Yantar 2K',null,null,29.4,307,null,1 from dual union all
select 90,'5S47','','',1968,'A-350Zh booster',15000,null,null,null,5,1 from dual union all
select 900,'S1.5400','S1.5400','11D33',1965,'Blok L',null,null,66.7,340,250,1 from dual union all
select 901,'S125','PS1','',null,'GSLV S125',157300,486000,4801,266,93,1 from dual union all
select 902,'S139','PS1','',1993,'PSLV PS1',168000,null,4430,269,105,1 from dual union all
select 903,'S2.1100','S2.253','',1957,'Burya',null,null,673,null,null,4 from dual union all
select 904,'S2.1150','S2.1150','',1959,'Burya',null,null,671.2,250,null,1 from dual union all
select 905,'S2.253','S2.253','',1953,'R-11',null,null,80,251,78,1 from dual union all
select 906,'S2.253A','S2.253','',1953,'R-11FM',null,null,93,251,78,1 from dual union all
select 907,'S2.711','','',1957,'V-750',null,null,13.8,null,22,1 from dual union all
select 908,'S2.711V','','',1957,'V-750',null,null,30.4,null,25,1 from dual union all
select 909,'S2.713','S2.713','',null,'R-13',null,null,252,null,null,1 from dual union all
select 91,'5Ya26','','',null,'5Ya26',10000,null,null,null,null,1 from dual union all
select 910,'S2.720','S2.720','',null,'',null,null,34.3,233,null,1 from dual union all
select 911,'S2.721V','S2.721','',1956,'',null,null,11.8,null,null,1 from dual union all
select 912,'S2.726','S2.726','',1958,'V-1000',null,null,103,null,55,1 from dual union all
select 913,'S200','GSOM-200','',2009,'GSLV3 GSOM',207000,null,6000,274,105,1 from dual union all
select 914,'S250','','',1969,'SO-250',200,null,null,null,null,1 from dual union all
select 915,'S5.1','S5.1','',null,'',null,null,167,null,null,1 from dual union all
select 916,'S5.120','','',null,'Argon (Zenit-6U)',null,null,null,304,null,1 from dual union all
select 917,'S5.2','9D21','',null,'R-17',null,null,131.2,258,null,null from dual union all
select 918,'S5.3','S5.3','',null,'R-21',null,null,392,null,null,1 from dual union all
select 919,'S5.38','S5.3','',null,'R-21',null,null,392,null,null,1 from dual union all
select 92,'8-AS-1000','JATO','8-AS-1000',1945,'KAN-1 Litte Joe',null,null,4.4,null,8,1 from dual union all
select 920,'S5.3M','S5.3','',null,'',null,null,396.7,241,null,1 from dual union all
select 921,'S5.4','S5.4','',null,'',null,null,15.8,266,null,1 from dual union all
select 922,'S5.44','S5.44','',null,'',null,null,177,251,null,1 from dual union all
select 923,'S5.45','S5.45','',null,'',null,null,1961,267,null,1 from dual union all
select 924,'S5.5','KTDU-5','',null,'Luna E-6',null,null,44,287,null,1 from dual union all
select 925,'S5.51','','',null,'LOK PAO',null,null,33.2,314,null,1 from dual union all
select 926,'S5.92','S5','',1988,'Fobos, Briz, Fregat',null,20000,19.6,330,1000,1 from dual union all
select 927,'S5.92 (A)','S5','',1988,'Fregat [Umanskiy]',null,20000,19.91,333.4,1350,1 from dual union all
select 928,'S5.98','14D30','',1998,'Briz-M',null,null,19.62,331.9,3200,1 from dual union all
select 929,'S5.98M','','',null,'Briz',null,null,19.6,326,1000,1 from dual union all
select 93,'8D725','','',null,'',null,null,117,325,null,1 from dual union all
select 930,'S9','','',null,'S9',11800,null,702,259,45,1 from dual union all
select 931,'SA-II','','',1965,'SA-II',70,null,null,null,null,1 from dual union all
select 932,'SALVO Ventions','','',2014,'SALVO',null,null,4.4,null,null,1 from dual union all
select 933,'SARGE','LE23000FC','',2018,'SARGE',null,null,24.1,200,60,1 from dual union all
select 934,'SB-310','','',1966,'Lambda [0]',502,null,95,220,7.4,1 from dual union all
select 935,'SB-735','L735','',1985,'Mu 3SII [0]',5124,null,328,266,38,1 from dual union all
select 936,'SE5-2','SE5-2','',1962,'Agena APS',null,null,.21,272,null,2 from dual union all
select 937,'SEP 401','401','',1980,'MSBS M4 [1]',22500,null,900,null,60,1 from dual union all
select 938,'SEP 402','402','',1980,'MSBS M4 [2]',8800,null,300,null,70,1 from dual union all
select 939,'SEP 403','403','',1980,'MSBS M4 [3]',1500,null,100,null,40,1 from dual union all
select 94,'8D727','RD-108','RD-108MM',null,'Sputnik',null,null,833,308,340,4 from dual union all
select 940,'SEP 901','P-10','P-10',1966,'SSBS S01 [1]',10000,null,440,null,50,4 from dual union all
select 941,'SEP 902','P-16','P-10',1968,'SSBS S2 [1]',16000,null,540,null,null,4 from dual union all
select 942,'SEP 903','P-10','P-10',1965,'SSBS S2 [2]',10000,null,440,null,null,4 from dual union all
select 943,'SEP 904','P-10','P-10',1966,'MSBS M1 [1]',10000,null,450,null,50,4 from dual union all
select 944,'SEPR 685-4','','Mimosa',1959,'Antares [3]',150,167,33,null,5,1 from dual union all
select 945,'SEPR 734-1','','Vesuve',1959,'Antares [1]',800,1025,200,null,5,1 from dual union all
select 946,'SEPR 737','','',null,'Aigle',900,null,75,null,16,1 from dual union all
select 947,'SEPR 739','Berenice','BER',1960,'Berenice',1930,null,170,260,20,1 from dual;
insert into ENGINE
select 948,'SEPR 739-2','','Tacite',1965,'Tacite',1702,null,170,234,20,1 from dual union all
select 949,'SEPR 740','','',1962,'Berenice St 2',1110,null,null,238,20,1 from dual union all
select 95,'8D728','RD-108','RD-107MM',null,'Molniya',null,null,921,257,140,4 from dual union all
select 950,'SEPR 740-3','','',1962,'',1110,null,null,238,20,1 from dual union all
select 951,'SEPR P163','','',1963,'Emma [1]',50,null,null,null,35,1 from dual union all
select 952,'SEPR P167','','',null,'Berenice strapon',23,null,34,null,null,1 from dual union all
select 953,'SEPR P191','','',null,'Emma [0]',20,15,14.8,null,1,1 from dual union all
select 954,'SEPR-200','Tramontane','Tramontane',1962,'Berenice',280,333,50,null,6.7,1 from dual union all
select 955,'SEPR-732','','',null,'SE4400 booster',500,940,null,null,4.7,1 from dual union all
select 956,'SICBM','SICBM Stage 1','',1989,'SICBM [1]',16100,null,979,null,40,1 from dual union all
select 957,'SICBM2','SICBM Stage 2','',1989,'SICBM [2]',3260,12000,267,null,44,1 from dual union all
select 958,'SICBM3','SICBM Stage 3','',1989,'SICBM [3]',1550,null,88,null,null,1 from dual union all
select 959,'SICBM4','','',1989,'SICBM PBS',null,null,2,null,null,1 from dual union all
select 96,'8D74','RD-108','RD-107',null,'Sputnik',null,null,1000,256,140,4 from dual union all
select 960,'SLV-3-1','','',1979,'SLV-3 [1]',10800,null,422,253,49,1 from dual union all
select 961,'SLV-3-2','','',1979,'SLV-3 [2]',4900,null,267,266.9,40,1 from dual union all
select 962,'SLV-3-3','','',1979,'SLV-3 [2]',1500,null,90.7,276.9,45,1 from dual union all
select 963,'SLV-3-4','','',1979,'SLV-3 [4]',360,null,26.8,282.9,33,1 from dual union all
select 964,'SM-3-II-2','','',null,'SM-3-II-2',700,null,null,null,null,1 from dual union all
select 965,'SM-3-II-3','','',null,'SM-3-II-2',300,null,null,null,null,1 from dual union all
select 966,'SO-250','','',null,'SO-250',150,null,null,null,null,1 from dual union all
select 967,'SPRAN-50','','',1959,'Daniel',400,null,null,null,5.6,1 from dual union all
select 968,'SQ/DF-11','','',null,'DF-11',5400,null,93.2,null,null,1 from dual union all
select 969,'SR.116','AIM-9P Sidewinder','',null,'Sidewinder',45,null,17.7,null,2.1,1 from dual union all
select 97,'8D74PS','RD-108','RD-107',null,'Sputnik',null,null,1000,306,146,4 from dual union all
select 970,'SR119','MX Stage 2','',1983,'Peacekeeper [2]',27669,null,1365,304,54,1 from dual union all
select 971,'SR120','MX Stage 3','',1983,'Peacekeeper [3]',7983,null,329,306,62,1 from dual union all
select 972,'SR19','Minuteman-SR','',1964,'Minuteman 2/3 [2]',7280,null,270,287.5,65.5,1 from dual union all
select 973,'SR19-ATK','Minuteman-SR','',2004,'Minuteman 2/3 [2]',7033,null,268,287.5,65.5,1 from dual union all
select 974,'SR71-AD-1','','',1968,'Loki Dart PWN-8B',11,18,9,231,2,1 from dual union all
select 975,'SR73','Minuteman-SR','',1968,'Minuteman 3 [3]',3655,null,152,285,61,1 from dual union all
select 976,'SR73-ATK','Minuteman-SR','',1968,'Minuteman 3 PRP [3]',3655,null,152,285,61,1 from dual union all
select 977,'SRM','SRM','',1981,'STS SRB',590000,null,11790,267.3,123,1 from dual union all
select 978,'SRMU','SRMU','',1994,'Titan 4B',350000,null,7500,285.6,137.8,1 from dual union all
select 979,'SS-30','','',null,'FTB',60,null,null,null,null,1 from dual union all
select 98,'8D75','RD-108','RD-108',null,'Sputnik',null,null,941,308,340,4 from dual union all
select 980,'SS-50 Antares','','',1972,'Tauro',200,null,null,null,18,1 from dual union all
select 981,'SS-520','','',null,'SS-520 St 2',400,null,null,null,null,1 from dual union all
select 982,'SS-520 S3','','',null,'SS-520 St 3',78,null,null,null,null,1 from dual union all
select 983,'SSME','RS-24','',1981,'STS ET',null,null,2100,452.5,516,1 from dual union all
select 984,'ST-735-1','','',null,'ST-735 [1]',7400,null,250,null,49,1 from dual union all
select 985,'ST-735-2','','',null,'ST-735 [2]',4000,null,250,null,null,1 from dual union all
select 986,'STIG-A','','',2011,'STIG-A',null,null,22.5,200,50,1 from dual union all
select 987,'STIG-B','','',2011,'STIG-B',null,null,22.5,200,50,1 from dual union all
select 988,'SVM-2','','',1968,'Intelsat 3',159,400,14,285,28,1 from dual union all
select 989,'Safir','','',2008,'Safir [3]',1000,null,null,null,null,1 from dual union all
select 99,'8D75PS','RD-108','RD-108',null,'Sputnik',null,null,941,308,340,4 from dual union all
select 990,'Saluki B','','',1960,'Blue Water [1]',360,null,195.5,null,6,1 from dual union all
select 991,'Saluki S','','',1960,'Blue Water [2]',442,null,35.6,null,27,1 from dual union all
select 992,'Seagull','','',1963,'Seagull',8,null,10,null,2,1 from dual union all
select 993,'Sejjil','','',null,'Sejjil',13000,null,560,230,45,1 from dual union all
select 994,'Sejjil St 2','','',null,'Sejjil',5000,null,560,230,20,1 from dual union all
select 995,'Seliger 1','','',1962,'Seliger',1000,null,50,null,null,1 from dual union all
select 996,'Seliger 3','','',1962,'Seliger',500,null,50,null,null,1 from dual union all
select 997,'Shaheen-2','','',null,'Shaheen 2 [2]',4000,null,null,null,null,1 from dual union all
select 998,'Shourya','','',null,'Shourya [2]',5000,null,null,null,null,1 from dual union all
select 999,'Shourya Booster','','',null,'Shourya [1]',3000,null,null,null,null,1 from dual;


--------------------------------------------------------------------------------
-- STAGE
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing STAGE...');

CREATE TABLE "STAGE" 
 (	"STAGE_NAME" VARCHAR2(21), 
	"STAGE_FAMILY" VARCHAR2(21), 
	"STAGE_ALT_NAME" VARCHAR2(21), 
	"LENGTH" NUMBER, 
	"DIAMETER" NUMBER, 
	"LAUNCH_MASS" NUMBER, 
	"DRY_MASS" NUMBER, 
	"THRUST" NUMBER, 
	"DURATION" NUMBER, 
	"ENGINE_ID" NUMBER, 
	"ENGINE_COUNT" NUMBER, 
	 CONSTRAINT "STAGE_PK" PRIMARY KEY ("STAGE_NAME")
USING INDEX  ENABLE, 
	 CONSTRAINT "STAGE_ENGINE_FK" FOREIGN KEY ("ENGINE_ID")
	  REFERENCES "ENGINE" ("ENGINE_ID") ENABLE
 ) ;


CREATE INDEX "STAGE_IDX1" ON "STAGE" ("ENGINE_ID") 
;
insert into STAGE
select '10KS2500','10KS2500','',2.9,.35,.1,null,11,10,4,1 from dual union all
select '11A57I','Blok-I','Blok I',6.7,2.66,23.7,2350,298,240,723,1 from dual union all
select '11S510 (F)','Blok-I','',7.6,2.66,25.2,2410,298,240,723,1 from dual union all
select '11S510','Blok-I','',6.7,2.66,25.2,2410,298,240,723,1 from dual union all
select '11S59','R-7','Blok BVGD',19.8,2.68,44.5,3915,992,120,754,1 from dual union all
select '11S59A','Blok-A','Blok A',28,2.95,105.4,6875,997.1,286,756,1 from dual union all
select '11S59A-S','Blok-A','Blok A',28,2.95,105.4,6875,1009.4,286,758,1 from dual union all
select '11S691','R-36','',18.8,3,127,8300,3032,120,781,3 from dual union all
select '11S692','R-36.St2','',10.1,3,53.3,4800,1010,160,783,1 from dual union all
select '11S771','Zenit','',32.9,3.9,352,33900,7908,140,768,1 from dual union all
select '11S772','Zenit.St2','',11.1,3.9,90,9300,912,300,766,1 from dual union all
select '11S824','Blok-D','Blok D',5.5,3.7,13.4,1800,85,600,16,1 from dual union all
select '11S824F','Blok-D','Blok D-2',5.5,3.7,17.3,2400,85,600,16,1 from dual union all
select '11S824M','Blok-D','Blok D-1',5.5,3.7,17.3,2400,85,600,16,1 from dual union all
select '11S86','Blok-D','Blok DM',6.3,3.7,17.5,2440,83.4,680,17,1 from dual union all
select '11S861','Blok-D','Blok DM-2',6.3,3.7,17.5,2440,83.4,680,17,1 from dual union all
select '11S861-01','Blok-D','Blok DM-2M',6.3,3.7,17.5,2440,83.4,680,17,1 from dual union all
select '11S861-03','Blok-D','Blok DM-3',6.3,3.7,17.5,2440,83.4,680,17,1 from dual union all
select '12KRB','GS3','C12',8.7,2.8,15,2500,73.5,720,434,1 from dual union all
select '14S53','R-7','Blok BVGD',19.8,2.68,44.5,3915,992,120,755,1 from dual union all
select '14S53A','Blok-A','Blok A',28,2.95,105.4,6875,1009.4,286,757,1 from dual union all
select '14S54','Blok-I','',6.7,2.66,26,2800,294,359,725,1 from dual union all
select '14S54/RD-0110','Blok-I','',6.7,2.66,25.2,2410,298,240,723,1 from dual union all
select '15A14A','R-36','R-36M St 1',22.3,3,210,15000,4522,130,788,4 from dual union all
select '15A14B','R-36.St2','R-36M St 2',6,3,49,3000,760,153,739,1 from dual union all
select '15A18-BR','BR','R-36M St 3',1,3,8.2,900,18.6,600,712,1 from dual union all
select '15A18A','R-36','R-36M St 1',22.3,3,210.8,15000,4525,98,790,3 from dual union all
select '15A18B','R-36.St2','R-36M St 2',5.7,3,49.3,3000,760,153,748,1 from dual union all
select '15Zh44 St 1','R-39','',9.7,2.4,53.7,null,2060,null,30,1 from dual union all
select '15Zh44 St 2','RT-23.St2','',4.8,2.4,25,null,1050,null,31,1 from dual union all
select '15Zh52 St 1','R-39','',9.7,2.4,53.7,null,2060,null,30,1 from dual union all
select '15Zh52 St 2','RT-23.St2','',4.8,2.4,25,null,1050,null,31,1 from dual union all
select '15Zh58A','Topol','Topol''-1',8.5,1.8,26,3000,980,60,44,1 from dual union all
select '15Zh58B','Topol.St2','Topol''-2',6,1.55,13,1500,490,64,45,1 from dual union all
select '15Zh58MA','Topol','Topol''-M1',8.5,1.8,26,3000,980,60,44,1 from dual union all
select '15Zh58MB','Topol.St2','Topol''-M2',6,1.55,13,1500,490,64,45,1 from dual union all
select '15Zh58MV','Topol.St3','Topol''-M3',3.1,1.34,6,1000,245,56,46,1 from dual union all
select '15Zh58V','Topol.St3','Topol''-3',3.1,1.34,6,1000,245,56,46,1 from dual union all
select '15Zh59A','Kuryer','',3.1,1.36,6,null,null,null,47,1 from dual union all
select '15Zh59B','Kuryer','',3.1,1.36,5,null,null,null,48,1 from dual union all
select '15Zh59V','Kuryer.St3','',2.1,1.13,3,null,null,null,49,1 from dual union all
select '15Zh60 St 1','R-39','',9.7,2.4,53.7,null,2060,null,39,1 from dual union all
select '15Zh60 St 2','RT-23.St2','',4.8,2.4,25,null,1050,null,40,1 from dual union all
select '15Zh61 St 1','R-39','',9.7,2.4,53.7,null,2060,null,30,1 from dual union all
select '15Zh61 St 2','RT-23.St2','',4.8,2.4,25,null,1050,null,38,1 from dual union all
select '17S40','Blok-D','',6.3,3.7,17.5,2440,83.4,680,18,1 from dual union all
select '2.5-DS-59000','Nike','',3,.42,.7,null,265,2.5,56,1 from dual union all
select '22.6KS1245','Sparrow','',1.3,.2,.06,5,5.5,22.6,59,1 from dual union all
select '27D6','Kontakt','',6,.74,3,null,null,null,63,1 from dual union all
select '28D6','Kontakt','',2,.74,1,null,null,null,64,1 from dual union all
select '29D6','Kontakt','',1,.74,.5,null,null,null,65,1 from dual union all
select '374BL01A','R-7','Blok BVGD',19.8,2.68,44.5,3915,992,120,755,1 from dual union all
select '374BL01A-A','Blok-A','Blok A',28,2.95,105.4,6875,1009.4,286,757,1 from dual union all
select '374BL01B','R-7','Blok BVGD',19.8,2.68,44.5,3915,992,120,755,1 from dual union all
select '374BL01B-A','Blok-A','Blok A',28,2.95,105.4,6875,1009.4,286,757,1 from dual union all
select '374BL02A','Blok-I','',6.7,2.66,25.2,2410,298,240,723,1 from dual union all
select '374BL02B','Blok-I','',6.7,2.66,26,2800,294,359,725,1 from dual union all
select '48N6','48N6','',7.5,.5,1.8,null,null,null,77,1 from dual union all
select '53T6','PRS-1','',10,1,12,null,103,4,86,1 from dual union all
select '5Ya26','PRS-1','',10,1,12,null,103,4,91,1 from dual union all
select '8A92E','Blok-E','',3.6,2.58,8,1440,54.5,430,844,1 from dual union all
select '8A92ME','Blok-E','',3.6,2.58,8,1440,54.5,430,844,1 from dual union all
select '8K71','R-7','',19,2.7,10.8,850,250,120,97,1 from dual union all
select '8K71A','Blok-A','',28,2.95,93.5,7495,941,330,99,1 from dual union all
select '8K72E','Blok-E','',3.6,2.58,8,1440,49.4,450,843,1 from dual union all
select '8K72EL','Blok-E','',2.5,2.58,8.1,1110,49.4,450,843,1 from dual union all
select '8K72KE','Blok-E','',3.6,2.58,8,1440,54.5,430,844,1 from dual union all
select '8K74','R-7','',19,2.7,10.8,850,250,120,96,1 from dual union all
select '8K74A','Blok-A','',28,2.95,100,7000,950,330,98,1 from dual union all
select '8K75A','R-9','',14.6,2.7,61.6,4000,1600,108,759,1 from dual union all
select '8K75B','8K75B','',7,2.7,15.9,2500,300,140,721,1 from dual union all
select '8K78I','Blok-I','',9.4,2.66,25.5,2350,298,240,722,1 from dual union all
select '8K94','RT-20P.St2','',8.4,1.8,9.8,900,147,215,799,1 from dual union all
select '8K95A','RT-2','',4.8,1.95,19,null,900,30,859,1 from dual union all
select '8K95B','RT-15','',4,1.49,10,1500,500,30,717,1 from dual union all
select '8K95V','RT-1.St3','',2.8,1.4,3,null,240,42,716,1 from dual union all
select '8K96A (RT-2P)','RT-15','',5.1,1.49,11.3,1500,432,60.6,35,1 from dual union all
select '8K96A','RT-15','',4.7,1.49,11.3,1500,432,60.6,37,1 from dual union all
select '8K96B (RT-15)','RT-2.St3','RT-15 St2',4.6,1,4.6,1000,216,45,41,1 from dual union all
select '8K96B (RT-2P)','RT-2.St3','RT-2P St3',5.5,1,4.6,1000,176,45,42,1 from dual union all
select '8K96B','RT-2.St3','',4.6,.98,4.6,1000,176,49,42,1 from dual union all
select '8K98A','RT-2','',8.7,1.84,34.6,4000,893,75.4,32,1 from dual union all
select '8K98B','RT-15','',4.7,1.49,9.6,1500,432,59,34,1 from dual union all
select '8K98PA','RT-2','',9.2,1.84,34.6,4000,981,75.4,33,1 from dual union all
select '8K98V','RT-2.St3','',3.8,.98,3.5,1000,215,45,36,1 from dual union all
select '8K99A','RT-20P','',6.1,1.8,19,2450,600,65,27,1 from dual union all
select '8S641','R-16','R-16A',16,3,110,10000,2610,90,777,3 from dual union all
select '8S642','R-16.St2','R-16B',13,2.4,27,3000,882,125,778,1 from dual union all
select '8S671','R-36','R-36 St1',18.8,3,122,6400,2648,120,781,3 from dual union all
select '8S672','R-36.St2','R-36 St2',9.4,3,49,3700,902,160,783,1 from dual union all
select '8S691','R-36','R-36O St1',18.8,3,122,6400,2648,120,781,3 from dual union all
select '8S692','R-36.St2','R-36O St2',9.4,3,49,3700,902,160,783,1 from dual union all
select '8S810','Proton','Proton 8S810',21.2,7.4,450,31000,9500,120,784,6 from dual union all
select '8S810M','Proton','Proton 8S810',21.2,7.4,450,30600,10500,120,785,6 from dual union all
select '8S811','UR-200','Proton St2',17.1,4.1,167.8,11700,2330,210,732,4 from dual union all
select '8S812','Proton.St3','Proton St3',6.9,4.1,50.6,4185,583,230,731,1 from dual union all
select '8S812M','Proton.St3','Proton St3',4.1,4.1,50.3,3700,583,230,731,1 from dual union all
select '8S816','UR-100','',17.2,2,88,5000,2070,121,735,4 from dual union all
select '8S817','UR-100.St2','',3.9,2,15,2000,240,183,25,1 from dual union all
select 'A-1','V-2','GR',17,1.65,20,4000,363,85,751,1 from dual;
insert into STAGE
select 'A-350 Booster','A-350','',8,2.57,15,1000,null,5,108,1 from dual union all
select 'A-350','A-350','',10,2.57,15,1000,null,40,109,1 from dual union all
select 'A-350Zh Booster','A-350','',7.9,2.57,15,1000,null,5,90,1 from dual union all
select 'A-350Zh','A-350','',15,2.57,15,1000,null,40,87,1 from dual union all
select 'A-925 Booster','A-350','',8,2.57,15,1000,null,5,84,1 from dual union all
select 'A-925','A-350','',7,2.57,15,1000,null,40,85,1 from dual union all
select 'A1P Stage 2','Polaris.St2','Polaris A1P St2',2,1.37,4.3,951,null,null,112,1 from dual union all
select 'A1P','Polaris','Polaris A1P St1',3.8,1.37,8.3,1450,null,null,111,1 from dual union all
select 'A2E Stage 2','Polaris.St2','Polaris A2E St2',2.1,1.37,4.2,860,null,null,1191,1 from dual union all
select 'A2E','Polaris','Polaris A2E St1',4.3,1.37,10.1,1450,null,null,113,1 from dual union all
select 'A2P Stage 2','Polaris.St2','Polaris A2P St2',2.1,1.37,4.2,860,null,null,1191,1 from dual union all
select 'A2P','Polaris','Polaris A2P St1',4.3,1.37,10.1,1450,null,null,113,1 from dual union all
select 'A3P','Polaris','Polaris A3P St1',4.4,1.37,11.1,1270,null,null,114,1 from dual union all
select 'AAD','AAD','',6,.42,1,null,null,null,null,1 from dual union all
select 'AJ10','Able','Able',5,.81,1.9,429,33,122,117,1 from dual union all
select 'AJ10-101A','Able','Able I',5.8,.81,1.9,385,34.7,110,119,1 from dual union all
select 'AJ10-104','Ablestar','Ablestar',4.5,1.39,4.5,599,36,null,120,1 from dual union all
select 'AJ10-104D','Ablestar','Ablestar',4.5,1.39,4.5,599,36,null,120,1 from dual union all
select 'AJ10-118','Able','Delta',6,.81,2.2,694,33.7,115,121,1 from dual union all
select 'AJ10-118A','Able','Delta',6,.81,2.7,600,33.7,170,122,1 from dual union all
select 'AJ10-118D','Able','Delta',5.8,.81,2.7,545,33.7,170,123,1 from dual union all
select 'AJ10-118FJ','Delta2','Delta AJ10-118FJ',6.3,1.75,6,780,44.1,430,126,1 from dual union all
select 'AJ10-11B','Able','AJ10',5.7,.81,1.9,429,34.7,122,121,1 from dual union all
select 'AJ10-40','Able','Able',5.8,.81,1.9,429,34.5,110,135,1 from dual union all
select 'AJ10-41','Able','Able',5.8,.81,1.9,429,34.5,110,136,1 from dual union all
select 'AJ10-42','Able','Able II',5.8,.81,1.9,429,34.5,110,137,1 from dual union all
select 'AJ60-91','AJ60','Aerobee 350 St2',9.1,.56,3,1050,72,52.7,139,4 from dual union all
select 'AJ60-92','Aerobee','Aerobee 200 St2',5.5,.38,.7,131,22.6,61.1,142,1 from dual union all
select 'AMROC IRR','AMROC','IRR',17.7,1.29,12.2,null,315,70,336,1 from dual union all
select 'AS-0','ASLV','',10.7,1,11.6,2963,502,49,148,1 from dual union all
select 'AS-1','ASLV','ASLV',10.2,1,11.8,2900,702,45,145,1 from dual union all
select 'AS-2','ASLV.St2','St 2',6.4,.8,4.4,800,304,36,146,1 from dual union all
select 'AS-3','ASLV.St3','St 3',2.4,.8,1.7,650,90.7,45,147,1 from dual union all
select 'AS-4','ASLV.St4','St 4',1.4,.66,.5,195,35,33,144,1 from dual union all
select 'ASAS XL','SM3.TSRM','',.9,.53,.1,null,7,15,602,1 from dual union all
select 'ASAS','SM3.TSRM','SM-3 TSRM',.9,.33,.1,null,7,15,602,1 from dual union all
select 'ASAT','SRAM','',4.3,.46,1,null,null,null,465,1 from dual union all
select 'ASR SRB-A','H2.SRB','',11.7,2.6,76.5,8700,2271,116,333,1 from dual union all
select 'ASROC','HOTROC','',4.5,.34,.4,40,49,null,599,1 from dual union all
select 'ATACMS','ATACMS','',4,.61,1.2,null,180,null,149,1 from dual union all
select 'ATSM-9','Jericho','',5.3,1.35,9.1,1000,610,55,151,1 from dual union all
select 'AUS-51','AUS-51','',2.6,1.35,2,170,58,92,152,1 from dual union all
select 'AVUM','AVUM','',1.7,1.93,1,494,2.5,667,793,1 from dual union all
select 'AVUM-RD868P','AVUM','',1.7,1.93,1,494,2.5,667,809,1 from dual union all
select 'Ababeel St 3','DF-21','',3.3,1.4,4,null,null,null,153,1 from dual union all
select 'Abid St1','Abid','',8,2.64,25,3000,1500,90,154,5 from dual union all
select 'Abid St2','R-11','',8,.88,6,1600,null,null,917,1 from dual union all
select 'Abid St3','Abid.St3','',1,.5,null,null,null,null,155,1 from dual union all
select 'Aeolus Booster','LongTom','',1.8,.41,.3,null,80,2.7,450,7 from dual union all
select 'AeroHTV Stage 2','AeroHTV2','',1.7,.15,.1,null,null,null,492,4 from dual union all
select 'AeroHTV','AeroHTV','',1.7,.23,.1,null,null,null,492,7 from dual union all
select 'Aerobee 100','Aerobee','Aerobee Jr',3.6,.38,.5,119,11.5,40,157,1 from dual union all
select 'Aerobee 150','Aerobee','Aerobee',5.1,.38,.7,133,17.8,51.4,139,1 from dual union all
select 'Aerobee 150A','Aerobee','Aerobee',5.1,.38,.6,133,17.8,51.4,139,1 from dual union all
select 'Aerobee 75','Orion','',4,.35,.5,142,13,32,522,1 from dual union all
select 'Aerobee AJ10-24','Aerobee','',5.8,.38,.5,184,17.8,32.1,131,1 from dual union all
select 'Aerobee AJ10-25','Aerobee','',5.8,.38,.5,184,17.8,32.1,132,1 from dual union all
select 'Aerobee AJ10-27','Aerobee','',5.8,.38,.5,184,17.8,32.1,133,1 from dual union all
select 'Aerobee AJ10-34','Aerobee','',5.8,.38,.5,184,17.8,32.1,134,1 from dual union all
select 'Aerobee Booster','AerobeeB','',1.8,.33,.3,140,80,2.5,141,1 from dual union all
select 'Aerobee XASR-1','Aerobee','',5.8,.38,.7,184,11.6,32.1,1206,1 from dual union all
select 'Aerojet Jr','Sergeant','',5.7,.79,4,null,254,28,158,1 from dual union all
select 'Agate','Agate','VE110   Agate',4.5,.8,2,100,190,18,623,1 from dual union all
select 'Agena A','Agena','',3,1.52,3.8,400,69,120,213,1 from dual union all
select 'Agena B','Agena','',6.2,1.52,7,750,71,480,215,1 from dual union all
select 'Agena D','Agena','',6.2,1.52,7,750,71,480,215,1 from dual union all
select 'Agni 4 Stage 2','AgniII.St2','',6,1,4.2,null,null,60,165,1 from dual union all
select 'Agni II Stage 2','AgniII.St2','',6,1,4.2,null,null,60,165,1 from dual union all
select 'Agni III Stage 2','Agni3','',3.1,2,14,1820,314,110,164,1 from dual union all
select 'Agni III','Agni3','',7.7,2,33,2970,1040,75,163,1 from dual union all
select 'Agni V Stage 2','Agni3','',3.1,2,14,1820,314,110,164,1 from dual union all
select 'Agni V Stage 3','Agni3','',3,2,10,500,null,null,166,1 from dual union all
select 'Agni V','Agni3','',7.7,2,33,2970,1040,75,163,1 from dual union all
select 'Agni','ASLV','',11,1,11.5,2900,700,50,162,1 from dual union all
select 'Aigle','Stromboli','',3,.56,.9,246,75,16,946,1 from dual union all
select 'Ajax Cluster 1810','Ajax','',6,.76,1.2,null,50,71,168,4 from dual union all
select 'Ajax Cluster','Ajax','',6,.76,1.2,null,50,71,167,4 from dual union all
select 'Ajax','Ajax','',4.5,.3,.5,372,11.6,30,167,1 from dual union all
select 'Al Abbas','R-11','',10.9,.88,7.6,1600,null,null,917,1 from dual union all
select 'Al Fatah','Fatah','',4.6,.5,1,454,130,15,170,1 from dual union all
select 'Al Hussein','R-11','',9.5,.88,5.9,1600,null,null,917,1 from dual union all
select 'Al Samoud 2','Samoud','',5,.76,2,505,13.8,52,907,1 from dual union all
select 'Al Samoud','V-750','',6,.5,2,360,13.8,52,907,1 from dual union all
select 'AlKahir','AlKahir','',10,1.22,10,null,null,null,171,1 from dual union all
select 'Alcor IA','Alcor','',1.9,.52,.5,200,49,23,60,1 from dual union all
select 'Alcor IB','Alcor','',1.9,.52,.5,88,42,26,172,1 from dual union all
select 'Alcor','Alcor','',1.9,.52,.5,200,36,30,72,1 from dual union all
select 'Alcyone','BE3','',.6,.6,.1,39,34,8,202,1 from dual union all
select 'Alfa','Alfa','',3.8,1.37,6.5,1000,250,57,173,1 from dual union all
select 'Algol 1','Scout','',9.4,1.02,10.7,1380,413,44,175,1 from dual union all
select 'Algol 1A','Scout','',9.4,1.02,10.7,1200,420,44,176,1 from dual union all
select 'Algol 1B','Scout','',9.4,1.02,10.7,1200,420,44,177,1 from dual union all
select 'Algol 1C','Scout','',9.4,1.02,10.7,1200,440,44,178,1 from dual union all
select 'Algol 1D','Scout','',9.4,1.02,10.7,1200,440,44,179,1 from dual union all
select 'Algol 2A','Scout','',9.1,1.02,10.7,1200,380,60,180,1 from dual union all
select 'Algol 2B','Scout','',9.1,1.02,10.7,1170,400,80,181,1 from dual union all
select 'Algol 2C','Scout','',9.1,1.02,10.8,1200,436,76,182,1 from dual union all
select 'Algol 3A','ScoutD','',9.4,1.14,14.3,1600,471,56,174,1 from dual union all
select 'Altair 2','Altair','',1.5,.46,.3,33,27.2,28,1197,1 from dual union all
select 'Altair','Altair','',1.8,.46,.3,24,13,39,1188,1 from dual;
insert into STAGE
select 'Amethyste','Emeraude','',14,1.4,20.3,2500,415,119,1151,1 from dual union all
select 'Angara URM','Angara','',25.7,2.9,142.4,9800,2086,337.5,772,1 from dual union all
select 'Angara URM-1','Angara','',27.7,2.9,142.4,9800,2086,337.5,772,1 from dual union all
select 'Angara URM-2','URM-2','',5.5,3.6,39.8,4000,294.3,359,726,1 from dual union all
select 'Antares 2 Stage 1','Zenit','',27.6,3.9,260.7,18715,3844,235,771,2 from dual union all
select 'Antares 2','Antares','Antares II X-259',2.9,.76,1.3,107,93,36,1199,1 from dual union all
select 'Antares 2B','Antares','',2.9,.76,1.3,null,93,36,1199,1 from dual union all
select 'Antares 3','Antares','',2.9,.76,1.4,95,82.3,45,1034,1 from dual union all
select 'Antares Stage 1','Zenit','',27.6,3.9,260.7,18715,3260,235,140,2 from dual union all
select 'Antares','Antares','',3.3,.76,1.2,240,60,38,1193,1 from dual union all
select 'Apache','Deacon','',2.6,.166,.1,39,21,6,1077,1 from dual union all
select 'Apollo LES','Apollo.LES','LES',10.2,.66,2.5,null,690,3,464,1 from dual union all
select 'Arcas','Arcas','',1.8,.11,0,15,1.5,29,546,1 from dual union all
select 'Arcon booster','ArcasB','',.7,.1,0,8,14,.4,184,1 from dual union all
select 'Arcon','Arcon','',2.5,.15,.1,30,4.5,33,183,1 from dual union all
select 'Ares I-X','SRB','',50.9,3.77,590,110000,11790,123,977,1 from dual union all
select 'Ariane 5 EPC H158','Ariane5','',30.5,5.46,170,12700,1114,589,1176,1 from dual union all
select 'Ariane 5 EPC H175','Ariane5','',30.5,5.46,189,14200,1320,540,1177,1 from dual union all
select 'Ariane 5 EPS P2000','EPS','EPS L10',5.2,5.46,13,2850,29,1100,160,1 from dual union all
select 'Ariane 5 EPS','EPS','',4.9,5.46,12.5,2700,29,1100,160,1 from dual union all
select 'Ariane EAP P238','Vega','',31.6,3.01,277.6,39800,6719,129,294,1 from dual union all
select 'Ariane EAP P240','Vega','',31.6,3.01,280,40000,6719,130,295,1 from dual union all
select 'Ariane H10','Ariane.St3','H10',11.8,2.6,12.5,1740,62.7,720,343,1 from dual union all
select 'Ariane H10+','Ariane.St3','H10+',11.8,2.6,12.8,1740,62.7,750,343,1 from dual union all
select 'Ariane H10-3','Ariane.St3','H10-3',12.1,2.6,13.5,1780,62.7,780,343,1 from dual union all
select 'Ariane H8','Ariane.St3','H8',10.2,2.6,9.7,1450,61.6,563,342,1 from dual union all
select 'Ariane L144','Ariane','',18.4,3.8,160,13750,2462,146,1170,4 from dual union all
select 'Ariane L220','Ariane','',25.4,3.8,246,17900,3034,205,1171,4 from dual union all
select 'Ariane L33','Ariane.St2','L33',11.6,2.6,38.9,3500,785,126,1169,1 from dual union all
select 'Arrow 3 Stage 2','Arrow3','',3.5,.7,.5,null,null,null,365,1 from dual union all
select 'Arrow 3','Arrow3','',6.5,.7,1,null,null,null,365,1 from dual union all
select 'Arrow II','Loki','',1.5,.08,.01,2,10,2,185,1 from dual union all
select 'Asp II','Asp','',2.7,.165,.1,18,26,3.6,186,1 from dual union all
select 'Asp III','Asp','',2.7,.165,.1,18,26,3.3,187,1 from dual union all
select 'Asp IV','Asp','Asp',2.8,.165,.094,27,12.7,12,840,1 from dual union all
select 'Asp','Asp','',2.7,.165,.1,18,26,5.3,839,1 from dual union all
select 'Astra Stage 1','Astra','',8,1,4,null,null,null,188,4 from dual union all
select 'Astra Stage 2','Astra','',2,1,2,null,null,null,188,1 from dual union all
select 'Astrid','Astrid','',1.9,.17,.02,8,.4,30,569,4 from dual union all
select 'Astris','Astris','',3.8,2.03,3.6,800,22.5,361,189,1 from dual union all
select 'Astrobee D','AstrobeeD','',2.8,.15,.1,20,8.9,17,190,1 from dual union all
select 'Astrobee F','AstrobeeF','',7.3,.38,1.3,250,36.4,51.1,191,1 from dual union all
select 'Atea-1','Atea','',3,.15,.06,null,6.9,14.5,192,1 from dual union all
select 'Atlas A Booster','Atlas.Booster','',4.5,3.05,3.5,3500,1335,128,532,1 from dual union all
select 'Atlas A','Atlas','',21.8,3.05,82,7000,0,125,null,1 from dual union all
select 'Atlas B Booster','Atlas.Booster','',4.5,3.05,3.5,3500,1335,128,531,1 from dual union all
select 'Atlas B','Atlas','',21.9,3.05,108,3850,267,253.6,533,1 from dual union all
select 'Atlas Booster','Atlas.Booster','',4.5,3.05,3.2,3200,2065,165,540,1 from dual union all
select 'Atlas C Booster','Atlas.Booster','',4.5,3.05,3.5,3500,1335,128,531,1 from dual union all
select 'Atlas C','Atlas','',21.9,3.05,108,3850,267,253.6,533,1 from dual union all
select 'Atlas CCB','Atlas5','',32.5,3.8,304,20890,3800,240,770,1 from dual union all
select 'Atlas D Booster','Atlas.Booster','',4.5,3.05,3.4,3400,1375,135,534,1 from dual union all
select 'Atlas D','Atlas','',22,3.05,115,3400,253,281,535,1 from dual union all
select 'Atlas E','Atlas','',20,3.05,116,3500,253,288,535,1 from dual union all
select 'Atlas E/F Booster','Atlas.Booster','',4.5,3.05,3.2,3200,1468,131,536,1 from dual union all
select 'Atlas F','Atlas','',20,3.05,116,3500,253,312,535,1 from dual union all
select 'Atlas G Booster','Atlas.Booster','',4.5,3.05,3.2,3200,1682,140,538,1 from dual union all
select 'Atlas G','Atlas','',22.2,3.05,142.5,4240,386.5,266,539,1 from dual union all
select 'Atlas H','Atlas','',21.2,3.05,142.5,4240,380.6,266,539,1 from dual union all
select 'Atlas I','Atlas','',22.2,3.05,142.5,4240,380.6,266,539,1 from dual union all
select 'Atlas II Booster','Atlas.Booster','',4.5,3.05,3.2,3200,1909,140,540,1 from dual union all
select 'Atlas II','Atlas','',24.9,3.05,163.5,10350,380.6,274,539,1 from dual union all
select 'Atlas III','Atlas','',29,3.05,195,13725,4152,190,770,1 from dual union all
select 'Atlas MA-5 Booster','Atlas.Booster','',4.5,3.05,3.2,3200,1870,140,538,1 from dual union all
select 'Atlas SLV-3','Atlas','SLV-3',21,3.05,118,3000,250,312,539,1 from dual union all
select 'Atlas SLV-3A','Atlas','',24,3.05,142,3700,386,175,539,1 from dual union all
select 'Atlas SLV-3C','Atlas','',21.9,3.05,128.5,4000,267,260,539,1 from dual union all
select 'Atlas SLV-3D','Atlas','',21.2,3.05,128.5,4000,386.5,260,539,1 from dual union all
select 'Atlas V SRM','AtlasVSRM','',19.5,1.55,46.2,3600,1361,91,116,1 from dual union all
select 'Aurore','Aurore','',2.1,.13,.05,20,10.5,5.3,193,1 from dual union all
select 'B-47 Stratojet','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'B-52H','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'B-57A Canberra','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'B-58 Hustler','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'B-611','DF-11','',9.4,.88,2,1645,93.2,null,266,1 from dual union all
select 'BE-3','BE3','',.6,.6,.1,28,34,9,197,1 from dual union all
select 'BE-3A4','BE3','',.6,.6,.1,28,34,9,199,1 from dual union all
select 'BE-3A5','BE3','',.6,.6,.1,28,34,9,200,1 from dual union all
select 'BE-3A9','BE3','',.6,.6,.1,28,34,9,201,1 from dual union all
select 'BER','Stromboli','',4.7,.56,1.9,700,170,20,947,1 from dual union all
select 'BLDT','BLDT','',0,0,0,0,0,0,855,4 from dual union all
select 'BOZ','BOZ','',0,0,0,0,0,0,null,0 from dual union all
select 'BT-310','S-310','S310',6,.31,.6,230,80,31.6,887,1 from dual union all
select 'Baby Sergeant','BabySergeant','',1.2,.15,.025,4,8.6,7.6,842,1 from dual union all
select 'Badr-1','Grad','',5,.6,1,null,null,null,207,1 from dual union all
select 'Balloon','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'Belier II','Belier','',2.9,.31,.3,80,20,23.4,211,1 from dual union all
select 'Belier III','Belier','',2.9,.31,.3,100,20,23.4,1152,1 from dual union all
select 'Belier','Belier','',2.9,.31,.3,100,20,23.4,384,1 from dual union all
select 'Belisama','Belisama','',1.6,.1,.03,14,24,.9,212,1 from dual union all
select 'Berenice St2','Stromboli','',3.8,.56,1.1,372,170,20,949,1 from dual union all
select 'Black Arrow St2','BlackArrow.St2','St 2',3.2,1.37,3.5,535,69,113,318,1 from dual union all
select 'Black Arrow','BlackArrow','',5.8,2,14.1,1070,251,125,321,1 from dual union all
select 'Black Brant I','BB5','Black Brant 1',5.3,.44,1.1,null,110,15,43,1 from dual union all
select 'Black Brant IIB','BB5','Black Brant 2B',5.2,.44,1,150,89,23,61,1 from dual union all
select 'Black Brant IIIA','BB3','BB3',3.7,.26,.3,50,49,9,100,1 from dual union all
select 'Black Brant IIIB','BB3','BB3B',3.9,.26,.3,50,45,12,21,1 from dual union all
select 'Black Brant Mk1','BB5','Black Brant 5C',5.3,.44,1.2,240,68.2,33,194,1 from dual union all
select 'Black Brant Mk3','BB5','Black Brant 5C',5.3,.44,1.2,240,68.2,33,195,1 from dual union all
select 'Black Brant Mk4','BB5','Black Brant 5C',5.3,.44,1.2,240,68.2,33,195,1 from dual;
insert into STAGE
select 'Black Brant VA','BB5','Black Brant 2A',5.2,.44,1,150,111,15,43,1 from dual union all
select 'Black Brant VB','BB5','Black Brant 5B',5.3,.44,1.2,151,89,32.4,62,1 from dual union all
select 'Black Brant VC','BB5','Black Brant 5C',5.3,.44,1.2,240,68.2,33,62,1 from dual union all
select 'Black Brant VI','BB6','Black Brant 6',2,.12,.1,20,7.4,9,101,1 from dual union all
select 'Black Brant VII','BB7','',1.7,.09,0,10,1,4,196,1 from dual union all
select 'Black Knight 301','BlackKnight','',9,.91,5.4,540,75,140,320,1 from dual union all
select 'Black Knight','BlackKnight','',9,.91,5.4,540,75,140,319,1 from dual union all
select 'Black Sparrow','BlackSparrow','',4.8,.53,1.2,null,null,null,218,1 from dual union all
select 'Blok DM-SL','Blok-D','',5.6,3.7,17.3,2720,85,650,17,1 from dual union all
select 'Blok DM-SLB','Blok-D','',5.6,3.7,17.3,2720,85,650,17,1 from dual union all
select 'Blok DM1','Blok-D','',6.3,3.7,17.5,2440,83.4,680,17,1 from dual union all
select 'Blok DM3','Blok-D','',6.3,3.7,17.5,2440,83.4,680,17,1 from dual union all
select 'Blok DM4','Blok-D','',6.3,3.7,17.5,2440,83.4,680,17,1 from dual union all
select 'Blok SO-L','Blok-L','',3.2,2.4,6.2,999,66.7,250,900,1 from dual union all
select 'Blok-2BL','Blok-L','',3.2,2.4,5.8,902,66.7,250,900,1 from dual union all
select 'Blok-L','Blok-L','',3.2,2.4,7.3,1160,66.7,250,900,1 from dual union all
select 'Blok-ML','Blok-L','',3.2,2.4,6.2,999,66.7,250,900,1 from dual union all
select 'Blok-NVL','Blok-L','',3.2,2.4,5.8,902,66.7,250,900,1 from dual union all
select 'Blok-VL','Blok-L','',3.2,2.4,5.8,902,66.7,250,900,1 from dual union all
select 'Blue Sparrow','BlackSparrow','',4.8,.53,1.2,null,null,null,218,1 from dual union all
select 'Blue Streak','BlueStreak','',17.7,3.05,95,6800,1512,161,865,2 from dual union all
select 'Bomarc A','Bomarc','',0,0,0,0,0,0,471,0 from dual union all
select 'Bomarc B','Bomarc','',0,0,0,0,0,0,1223,0 from dual union all
select 'Briz-K','Briz','14S12',2.5,2,6.7,1700,19.6,1000,929,1 from dual union all
select 'Briz-KM','Briz','',2.5,2.5,6.7,1700,19.6,1000,929,1 from dual union all
select 'Briz-M','Briz','',2.8,4.1,22.1,2370,19.6,3000,929,1 from dual union all
select 'Bull Goose','BullGoose','',0,0,0,0,0,0,null,0 from dual union all
select 'Bullpup','Bullpup','',3,.3,.3,null,111,null,159,1 from dual union all
select 'Burya booster','Burya','',18.9,1.5,27,4000,765,null,904,1 from dual union all
select 'Burya','Burya','La-350',18,2.2,40.8,13000,null,0,null,1 from dual union all
select 'C-130','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'C-17','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'C-3 Stage 1','Poseidon','',4.8,1.88,10,null,null,null,221,1 from dual union all
select 'C-3 Stage 2','Poseidon.St2','',2.5,1.88,7,null,null,null,222,1 from dual union all
select 'CB-250','CB-250','',1,.25,.3,null,null,null,825,7 from dual union all
select 'CPKM','SD','',2.9,1.4,4.6,496,222.8,65.2,226,1 from dual union all
select 'CSXT','CSXT','',6.4,.25,.3,96,71,13.4,894,1 from dual union all
select 'CUS12','GS3','',8.7,2.8,14.6,2200,73.6,720,366,1 from dual union all
select 'CUS15','GS3','',8.7,2.8,14.6,2200,73.6,720,366,1 from dual union all
select 'CZ Booster','CZ.LB','LB40-YF25',15.3,2.25,37.7,3200,740.4,128,1238,1 from dual union all
select 'CZ-11-1','DF-31','',8,2,40,5000,null,null,228,1 from dual union all
select 'CZ-11-2','DF-31','',6,2,15,null,null,null,229,1 from dual union all
select 'CZ-11-3','CZ11-3','',2,1.8,4,null,null,null,230,1 from dual union all
select 'CZ-11-4','CZ11-4','',1.5,1.5,2,null,null,null,231,1 from dual union all
select 'CZ-2 Stage 1','DF5','L140-YF21',20.5,3.35,153,10000,2961,122,1232,4 from dual union all
select 'CZ-2 Stage 2','CZ.St2','LF35-YF24',7.5,3.35,39,4000,742,109,1233,1 from dual union all
select 'CZ-2C Stage 1','DF5','L140-YF21',20.5,3.35,153,10000,2961,122,1232,4 from dual union all
select 'CZ-2C Stage 2','CZ.St2','LF35-YF24',7.5,3.35,39,4000,742,109,1233,1 from dual union all
select 'CZ-2Cj Stage 2','CZ.St2','LF35-YF24',7.5,3.35,39,4000,742,109,1233,1 from dual union all
select 'CZ-2E Stage 1','DF5','L180-YF21B',23.7,3.35,196.5,9500,2961,166,1232,4 from dual union all
select 'CZ-2E Stage 2','CZ.St2','L90-YF24B',15.5,3.35,91.5,5500,742,295,1233,1 from dual union all
select 'CZ-3 Stage 1','DF5','L140-YF21',20.2,3.35,151,9000,2785,132,1232,4 from dual union all
select 'CZ-3 Stage 2','CZ.St2','LF35-YF24',9.7,3.35,39,4000,720,180,1233,1 from dual union all
select 'CZ-3 Stage 3','CZ3.St3','H8-YF73',7.5,2.25,10.5,2000,44.1,800,1245,1 from dual union all
select 'CZ-3A Stage 1','DF5','L180-YF21B',23.3,3.35,180,9500,2961,145,1232,4 from dual union all
select 'CZ-3A Stage 2','CZ.St2','L35-YF24B',9.9,3.35,33.6,4000,742,180,1233,1 from dual union all
select 'CZ-3A Stage 3','CZ3.St3','H18-YF75',12.4,3,21,2800,157,470,1246,2 from dual union all
select 'CZ-3B Stage 2','CZ.St2','L35-YF24B',9.9,3.35,44,4000,742,180,1233,1 from dual union all
select 'CZ-3BE Booster','CZ.LB','LB41-YF25',16.1,2.25,41,3200,740.4,128,1238,1 from dual union all
select 'CZ-3BE Stage 1','DF5','L180-YF21B',24.8,3.35,186.2,9500,2961,145,1232,4 from dual union all
select 'CZ-4 Stage 1','DF5','L180-YF21B',24.7,3.35,192.7,9500,2961,170,1232,4 from dual union all
select 'CZ-4 Stage 2','CZ.St2','L35-YF24B',10.4,3.35,39.6,4000,742,127,1234,1 from dual union all
select 'CZ-4 Stage 2j','CZ.St2','L35-YF24B',10.4,3.35,39.6,4000,742,127,1234,1 from dual union all
select 'CZ-4 Stage 3','L14','L14-YF40',1.9,2.9,15.2,1000,98.1,303,1241,1 from dual union all
select 'CZ-4B Stage 2','CZ.St2','L35-YF24B',10.4,3.35,39.6,4000,742,127,1235,1 from dual union all
select 'CZ-4B Stage 3','L14','L14-YF40',1.9,2.9,15.2,1000,98.1,303,1241,1 from dual union all
select 'CZ-4C Stage 3','L14','L14-YF40',1.9,2.9,15.2,1000,98.1,303,1243,1 from dual union all
select 'CZ-5 Booster','DF5','K3-1',27.6,3.35,155.7,12000,2400,180,1228,2 from dual union all
select 'CZ-5 Stage 1','CZ5','CZ-H5-1',31.7,5,175.6,17300,1400,480,1248,2 from dual union all
select 'CZ-5 Stage 2','CZ5','CZ-H5-2',10.6,5,22.2,5100,176.5,700,1247,2 from dual union all
select 'CZ-7 Stage 1','DF5','K3-1',26,3.35,186,12000,2400,215,1228,2 from dual union all
select 'CZ-7 Stage 2','CZ.St2','K3-2',11.5,3.35,71,6000,720,null,1229,4 from dual union all
select 'CZ-K2-1','CZ.LB','',26.5,2.25,81.5,6000,1200,185,1228,1 from dual union all
select 'CZ-K2-2','CZ1.St2','',8,2.25,15,3000,null,null,1229,1 from dual union all
select 'CZ-K2-3','CZ1.St2','',4,2.25,null,1000,null,null,1249,4 from dual union all
select 'CZ6-C','DF5','',15,3.35,84,4100,1224,null,1228,1 from dual union all
select 'Cajun','Deacon','',2.6,.166,.1,22,36,3,232,1 from dual union all
select 'Canopus','Rigel','',2,.28,.2,null,null,18,233,1 from dual union all
select 'Castor 1','Sergeant','',6,.79,4.2,783,235,38,1116,1 from dual union all
select 'Castor 120','MX','',10.7,2.3,53.1,4450,1604,83.4,234,1 from dual union all
select 'Castor 2','Sergeant','',6,.79,4,565,259,37,1118,1 from dual union all
select 'Castor 2A','Sergeant','',6,.79,4,565,259,37,1117,1 from dual union all
select 'Castor 30','MX','',3.5,2.35,14,1220,260,136,235,1 from dual union all
select 'Castor 30B','MX','',4.2,2.35,14,1083,293.4,128,236,1 from dual union all
select 'Castor 30XL','MX','',6,2.35,26.3,1220,260,166,237,1 from dual union all
select 'Castor 4','Castor4','',9.7,1.02,10.5,1200,428,53.9,1123,1 from dual union all
select 'Castor 4A','Castor4','',13.6,1.02,11.6,1400,433.7,56.2,1126,1 from dual union all
select 'Castor 4AXL','Castor4','',15,1.02,15.4,2500,745,60,1127,1 from dual union all
select 'Castor 4B','Castor4','',8.8,1.02,11.5,1420,429,60.9,1128,1 from dual union all
select 'Castor','Sergeant','',6,.79,4.2,783,235,38,1116,1 from dual union all
select 'Cedre St1','Cedre','',2.3,.52,1,null,138,null,238,1 from dual union all
select 'Cedre St2','Cedre.St2','',2.1,.21,.4,null,78,null,239,1 from dual union all
select 'Cedre St3','Cedre.St3','',2.1,.08,.2,null,20,null,240,1 from dual union all
select 'Cedre-4 St1','Cedre','',3,.21,1,null,138,null,241,1 from dual union all
select 'Centaur D','Centaur','Centaur',9.1,3.05,15.6,2000,134,430,827,2 from dual union all
select 'Centaur D-1A','Centaur','Centaur D1AR',9.6,3.05,16.3,2630,134,430,828,2 from dual union all
select 'Centaur D-1T','Centaur','',9.6,3.05,16.3,2630,134,470,828,2 from dual union all
select 'Centaur I','Centaur','Centaur',9.2,3.05,15,1700,147,402,830,2 from dual union all
select 'Centaur II','Centaur','Centaur',10.1,3.05,18.8,2053,147,488,830,2 from dual union all
select 'Centaur IIA','Centaur','Centaur',10.1,3.05,19,2200,198.4,370,832,2 from dual;
insert into STAGE
select 'Centaur IIIA','Centaur','Centaur',10.1,3.05,18.6,1720,99.2,740,834,1 from dual union all
select 'Centaur IIIB/C-1','Centaur','Centaur',11.7,3.05,22.9,2130,101.8,740,836,1 from dual union all
select 'Centaur IIIB/DEC','Centaur','Centaur',11.7,3.05,22.9,2130,198.4,740,833,2 from dual union all
select 'Centaur IIIB/SEC','Centaur','Centaur',11.7,3.05,22.9,2130,99.2,740,834,1 from dual union all
select 'Centaur T','Centaur','Centaur',9,4.3,23.8,2775,146,600,829,2 from dual union all
select 'Centaur','Centaur','Centaur',9.1,3.05,15.6,2000,134,430,827,2 from dual union all
select 'Centenario','Rigel','',2.4,.28,.2,null,39,11.2,242,1 from dual union all
select 'Cetus','Cetus','',.5,.38,.1,10,40,20,642,1 from dual union all
select 'Chick x3','Chick','',.6,.1,0,6,60,.2,244,1 from dual union all
select 'Chick x4','Chick','',.6,.1,0,6,80,.2,244,1 from dual union all
select 'Chick','Chick','',.6,.1,.05,6,20,.2,244,1 from dual union all
select 'Cirrus I','Cirrus','',2,.2,.1,null,5,null,245,1 from dual union all
select 'Cirrus II','Cirrus','',2,.2,.1,null,18,null,246,1 from dual union all
select 'Cluster Stage 2','Cluster','11xBaby Sergeant',1.2,.89,.5,223,80,6,860,11 from dual union all
select 'Cluster Stage 3','BabySergeant','3xBaby Sergeant',1.2,.3,.1,63,22,6,860,3 from dual union all
select 'Coralie','Coralie','',6.5,2.01,12,2400,268,105,247,4 from dual union all
select 'Corporal E','Corporal','Corporal',13.7,.76,4.5,null,89,63,249,1 from dual union all
select 'Corporal Type 1','Corporal','Corporal',13.7,.76,5.2,null,89,63,248,1 from dual union all
select 'Cuckoo 4','Raven','',1.2,.44,.2,null,null,4.3,250,1 from dual union all
select 'Cuckoo IA','Raven','',1,.44,.2,null,80,4,251,1 from dual union all
select 'Cuckoo IB','Raven','',1.2,.44,.2,57,80.9,4.1,252,1 from dual union all
select 'Cuckoo II','Raven','',1.2,.44,.2,37,36.5,10,253,1 from dual union all
select 'Cygnus 10','10SM','',.4,.25,0,1,10,3.2,255,1 from dual union all
select 'Cygnus 15','Cygnus','',.4,.38,.1,5,22,4.8,256,1 from dual union all
select 'Cygnus 20','20SM','',.7,.51,.1,10,30,8.5,257,1 from dual union all
select 'Cygnus 25','25SM','',1.1,.64,.2,19,33.3,16.5,258,1 from dual union all
select 'Cygnus 40','40SM','',1.1,1.01,1,74,83,26,259,1 from dual union all
select 'Cygnus 5','5inSM','5"SM',.2,.13,0,1,2.3,1.6,260,1 from dual union all
select 'DAC Roc','DACRoc','',3,.25,.3,null,50,10,264,1 from dual union all
select 'DC-X','DC-X','',12.6,4.95,18.8,10000,242,263,826,4 from dual union all
select 'DC-XA','DC-X','Clipper Graham',12.6,4.95,18.8,10000,242,263,826,4 from dual union all
select 'DF-1','V-2','',14.1,1.65,14,4000,270,65,265,1 from dual union all
select 'DF-11','DF-11','',7.4,.88,5.4,1645,93.2,null,266,1 from dual union all
select 'DF-11M','DF-11','',9.4,.88,5.4,1645,93.2,null,266,1 from dual union all
select 'DF-15','DF-15','M-9',9.1,1,6.2,null,null,null,267,1 from dual union all
select 'DF-16','DF-16','',7.5,1.2,12,null,null,null,268,1 from dual union all
select 'DF-2','V-2','',20,1.65,30,4000,300,null,1227,1 from dual union all
select 'DF-21 St2','DF-21','CSSNX3',4,1.4,8.7,null,null,62,270,1 from dual union all
select 'DF-21','DF-21','CSSNX3',6,1.4,16.6,null,null,65,269,1 from dual union all
select 'DF-2A','V-2','',20,1.65,30,4000,300,null,1227,1 from dual union all
select 'DF-31 St1','DF-31','',8,2,35,null,null,null,271,1 from dual union all
select 'DF-31 St2','DF-31','',4,2,10,null,null,null,272,1 from dual union all
select 'DF-31 St3','DF-31','',2,2,2,null,null,null,273,1 from dual union all
select 'DF-41 St1','DF-31','',8,2,35,null,null,null,274,1 from dual union all
select 'DF-41 St2','DF-31','',4,2,10,null,null,null,275,1 from dual union all
select 'DF-41 St3','DF-31','',2,2,2,null,null,null,276,1 from dual union all
select 'DF-5 Stage 1','DF5','LF140-YF21',20.2,3.35,151,9000,2785,132,1232,4 from dual union all
select 'DF-5 Stage 2','CZ.St2','LF35-YF24',7.5,3.35,39,4000,742,110,1233,1 from dual union all
select 'DF3-YF2A','DF3','',18,2.25,64.1,4100,1224,130,1239,4 from dual union all
select 'DF4-YF3','CZ1.St2','',5.4,2.25,15,2700,306,110,1240,1 from dual union all
select 'DIM','DIM','',1.9,.125,.04,null,null,null,278,1 from dual union all
select 'DS','DS','',.5,1,.5,50,null,250,283,1 from dual union all
select 'DSV-3E-3','Ablestar','Delta AJ10-118E',5.2,1.39,6,785,33.7,400,124,1 from dual union all
select 'DSV-3N-4','Ablestar','Delta AJ10-118F',6.3,1.39,5.6,784,41.4,335,125,1 from dual union all
select 'DSV-3P-4','Ablestar','Delta AJ10-118F',6,1.39,5.6,784,41.4,335,125,1 from dual union all
select 'DSV-3P-4B','Ablestar','Delta TR-201',6,1.39,5.4,845,42,322,1088,1 from dual union all
select 'Deacon','Deacon','',2.8,.152,.1,24,26,3,1184,1 from dual union all
select 'Delta CBC RS-68A','Delta4','',40.8,5,226.4,26760,3124,249,854,1 from dual union all
select 'Delta CBC','Delta4','',40.8,5,226.4,26760,2891,249,853,1 from dual union all
select 'Delta D4SS-4','D3SS','Delta/0030',12,4,24.2,2850,110,850,835,1 from dual union all
select 'Delta D4SS-5','D4SS','Delta-5m',12,5,30.7,3490,110,1125,835,1 from dual union all
select 'Delta III Stage 1','Thor','',20,2.44,102.4,6820,1085.8,261,850,1 from dual union all
select 'Delta III Stage 2','D3SS','Delta/0030',8.8,4,19.3,2475,110,700,835,1 from dual union all
select 'Delta/0020','Delta2','Delta AJ10-118K',6,1.75,6.9,950,43.7,431,127,1 from dual union all
select 'Demon','Demon','',2.8,.18,0,null,null,2.3,288,1 from dual union all
select 'Dolphin','Dolphin','',15.5,1.07,7.5,null,155,null,289,1 from dual union all
select 'Dorado','Demon','',3.1,.18,.14,66,42,4.1,290,1 from dual union all
select 'Dragon V2','Dragon','',6.1,3.7,4.2,4200,584,5,1061,8 from dual union all
select 'Dummy stage','Generic','',0,0,0,0,0,0,null,0 from dual union all
select 'E-8-5 VA','E8','',2.3,.5,.4,null,null,null,292,1 from dual union all
select 'EELT Thor','Thor','',26.1,2.44,101.8,5680,1085.8,261,850,1 from dual union all
select 'ELT Thor DSV-2L','Thor','',22.4,2.44,84.1,4059,860,222,556,1 from dual union all
select 'ELT Thor DSV-2L-1D','Thor','',22.4,2.44,84.1,4059,860,222,556,1 from dual union all
select 'ELT Thor DSV-2L-1E','Thor','',22.4,2.44,84.1,4059,860,222,556,1 from dual union all
select 'ELT Thor DSV-2W','Thor','ELT Thor 3900',22.4,2.44,84.4,4360,1030,223,849,1 from dual union all
select 'ELT Thor DSV-2W-1','Thor','',22.8,2.44,84.4,4360,912,223,849,1 from dual union all
select 'ELT Thor N','Thor','',22.4,2.44,85.8,4400,860,270,556,1 from dual union all
select 'EPKM','SD','',2.8,1.7,6,541,177,87,302,1 from dual union all
select 'ESC-A','EPS','',7,5.43,14,2000,64,945,343,1 from dual union all
select 'ET','STS','',47,8.4,748,27000,6300,516,983,3 from dual union all
select 'Elan','Belenos','',1.6,.21,.08,32,null,2.5,161,1 from dual union all
select 'Electron Stage 1','Electron','',12.1,1.2,10.2,950,162,null,878,9 from dual union all
select 'Electron Stage 2','Electron','',2.4,1.2,2.3,250,22,null,879,1 from dual union all
select 'Electron Stage 3','Electron','',1,1,1,null,null,null,254,1 from dual union all
select 'Emad','Nodong','',16,1.25,15.1,2000,255,90,655,1 from dual union all
select 'Emeraude','Emeraude','',9,1.4,14.7,1500,274,93,1162,1 from dual union all
select 'Emilie','Belenos','',1.6,.21,.08,32,null,2.5,297,1 from dual union all
select 'Emma Booster','Emma','',.8,.13,.02,13,14.8,1,953,1 from dual union all
select 'Emma','Emma','',1.7,.13,.05,11,1.4,37,951,1 from dual union all
select 'Energiya Blok A','Zenit','',38.3,3.9,360,60000,7908,145,768,1 from dual union all
select 'Energiya','Energiya','',58.1,7.75,780,80000,7850,480,724,4 from dual union all
select 'Epsilon PBS','EPBS','',1.2,1.5,.3,200,.4,1100,296,1 from dual union all
select 'Excalibur','Excalibur','',2.4,.11,.04,13,25,2.5,298,1 from dual union all
select 'F-104A Starfighter','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'F-15D','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'F2H2 Banshee','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'F3H Cougar','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'F4B Phantom','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'F4D Phantom','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'F4D-1 Skyray','Air','',0,0,0,0,0,0,null,0 from dual;
insert into STAGE
select 'F4H Phantom','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'F86D Sabrejet','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'FB-1 Stage 1','DF5','LF140-YF21',20.2,3.35,151,9000,2785,132,1232,4 from dual union all
select 'FB-1 Stage 2','CZ.St2','LF35-YF24',7.5,3.35,39,4000,742,110,1236,1 from dual union all
select 'FFAR','FFAR','',1.2,.07,0,5,3.6,1.5,300,1 from dual union all
select 'FFK','FFK','',4,.25,.3,null,null,null,301,1 from dual union all
select 'FTB','FTB','',3,.13,.07,1,null,null,979,1 from dual union all
select 'FTI','FTI','',5,.3,.07,1,null,null,304,1 from dual union all
select 'FTV','ISRO-TV','',2,.25,.3,null,null,null,null,1 from dual union all
select 'FW4D','FW4','',1.5,.5,.3,25,25,30.5,305,1 from dual union all
select 'FW4S','FW4','',1.5,.5,.3,32,25,30.5,305,1 from dual union all
select 'Falcon 1 St 2','Falcon1','',4.5,1.7,4.5,544,33,418,442,1 from dual union all
select 'Falcon 1','Falcon1','',13,1.7,22.9,1360,378,169,587,1 from dual union all
select 'Falcon 9 St 1','Falcon9','',29,3.65,302,17726,3680,174,587,9 from dual union all
select 'Falcon 9 St 1/1D','Falcon9','',42,3.65,390,18000,5886,180,590,9 from dual union all
select 'Falcon 9 St 1/1D-4','Falcon9','',43.5,3.65,390,18000,6770,180,591,9 from dual union all
select 'Falcon 9 St 1/1D-5','Falcon9','',43.5,3.65,390,18000,6770,180,592,9 from dual union all
select 'Falcon 9 St 1/1D-FT','Falcon9','',43.5,3.65,390,18000,6770,180,593,9 from dual union all
select 'Falcon 9 St 2','Falcon9.St2','',10,3.65,52,2957,427,345,589,1 from dual union all
select 'Falcon 9 St 2/1D','Falcon9.St2','',12.6,3.65,75.7,4900,801,372,594,1 from dual union all
select 'Falcon 9 St 2/1D-4','Falcon9.St2','',12.6,3.65,75.7,4900,801,372,594,1 from dual union all
select 'Falcon 9 St 2/1D-5','Falcon9.St2','',12.6,3.65,75.7,4900,801,372,594,1 from dual union all
select 'Falcon 9-1D','Falcon9','',33,3.65,302,18000,547,174,590,1 from dual union all
select 'Falcon','Falcon','',1,.16,.02,0,0,0,308,1 from dual union all
select 'FalconLaunch','SuperLoki','',2,.1,.03,6,18,2.1,1062,1 from dual union all
select 'Fateh-110','Zelzal','',9,.61,1,null,null,null,649,1 from dual union all
select 'Frangible Arcas','Arcas','',1.8,.11,0,15,1.5,29,546,1 from dual union all
select 'Fregat','Fregat','',1.5,3.35,6.5,1100,19.6,877,926,1 from dual union all
select 'Fregat-SB','Fregat','',2.4,3.35,8.5,1050,19.6,1200,926,1 from dual union all
select 'GCA/Orion 38','Pegasus.St3','',2.1,2.35,1.2,500,36,69.6,666,1 from dual union all
select 'GCR 33KS2800','GCR','',1.4,.46,.2,30,12.5,33,73,1 from dual union all
select 'GEM-40','GEM','',13,1.02,13.1,1315,499.2,63.3,312,1 from dual union all
select 'GEM-46','GEM','',14.7,1.17,19.3,2035,608.1,75.2,313,1 from dual union all
select 'GEM-60','GEM60','',16.2,1.52,33.2,3850,851.5,90,315,1 from dual union all
select 'GF-01A','GF-02','',2,.77,2,200,29,140,317,1 from dual union all
select 'GF-02','GF-02','',3.9,.77,2,250,118,38,316,1 from dual union all
select 'GLCM','GLCM','',6.4,.53,1.8,null,null,null,null,0 from dual union all
select 'GS1-L110','GSLV3','',17,4,110,null,1400,200,1167,2 from dual union all
select 'GS1-S125','PSLV','',20.3,2.8,156,27000,4700,100,901,1 from dual union all
select 'GS1-S139','PSLV','PS-1',20,2.8,177,30000,4910,105,902,1 from dual union all
select 'GS2','PSLV.St2','',11.6,2.8,42.8,4800,799,136,1166,1 from dual union all
select 'GS2-C25','GSLV3.St2','',8.2,4,27,null,200,580,225,1 from dual union all
select 'GSOM','PSLV.St2','L40',19.7,2.1,46,6000,680,160,1166,1 from dual union all
select 'GSOM-S200','GSOM','',22,3.2,207,null,4800,130,913,1 from dual union all
select 'GammaCen Booster','GammaCen','',.8,.13,.03,null,null,null,null,1 from dual union all
select 'GammaCen Sustainer','GammaCenS','',1.6,.09,null,null,null,null,null,1 from dual union all
select 'Genie','Genie','MB-1',1.8,.38,.3,null,156,1.5,552,1 from dual union all
select 'Genie-Alarr','Genie','',3.1,.38,.3,null,null,null,552,1 from dual union all
select 'Ghauri','Nodong','Ghauri',16,1.25,16,3000,255,110,655,1 from dual union all
select 'Ghaznavi','DF-11','',9.4,.88,5.4,1645,93.2,null,266,1 from dual union all
select 'Goldfinch II','Raven','Goldfinch',3,.44,.4,89,null,3.5,322,1 from dual union all
select 'Goldfinch','Raven','',2.5,.44,.4,94,175,3.5,323,1 from dual union all
select 'Goose','Gosling','',3.4,.26,.3,95,22,17,324,1 from dual union all
select 'Gosling 4T','Gosling','',3.6,.26,.3,89,137,3.1,326,1 from dual union all
select 'Gosling IIN','Gosling','',3.5,.26,.2,null,140,3,325,1 from dual union all
select 'Gosling IV','Gosling','',3.5,.26,.3,null,140,3,326,1 from dual union all
select 'Gosling','Gosling','',3.3,.26,.2,null,152,2.5,325,1 from dual union all
select 'Grad','Grad','',5,.6,1,null,null,null,207,1 from dual union all
select 'Gradicom 1','Gradicom','',2.5,.32,.4,null,null,null,328,1 from dual union all
select 'Gradicom 2','Gradicom','',2.5,.32,.2,null,null,null,329,1 from dual union all
select 'Gun Projectile 16','Gun16','',1,.41,.1,null,null,null,null,0 from dual union all
select 'Gun Projectile 5','Gun05','',1,.13,.1,null,null,null,null,0 from dual union all
select 'Gun Projectile 7','Gun07','',1,.18,.1,null,null,null,null,0 from dual union all
select 'H-II SRB','H2.SRB','',23.4,1.8,70.4,11200,1550,94,331,1 from dual union all
select 'H-II SRB-A','H2.SRB','',15.2,2.5,75,10000,2250,100,332,1 from dual union all
select 'H-II SRB-A3','H2.SRB','',15.1,2.5,76.5,10500,2305,120,333,1 from dual union all
select 'H-II SSB','H2.SSB','',9,1.1,10.5,2100,338,66,334,1 from dual union all
select 'H-II Stage 1','H2','H-II',34.4,4,98,11700,1080,346,458,1 from dual union all
select 'H-II Stage 2','H2.St2','H-II St2',11,4,16.7,2700,122,600,456,1 from dual union all
select 'H-IIA Stage 1','H2','H-II',37.2,4,114,13000,1100,390,459,1 from dual union all
select 'H-IIA Stage 2 U','H2.St2','H-II St2',9.2,4,20,4000,137,600,457,1 from dual union all
select 'H-IIA Stage 2','H2.St2','H-II St2',9.2,4,20,4000,137,600,457,1 from dual union all
select 'H-IIB Stage 1','H2B','H-II',38,5.2,202,24000,2196,352,459,2 from dual union all
select 'HAEC FFAR','FFAR','',1.4,.08,0,null,null,2.8,337,1 from dual union all
select 'HALO','HALO','',1.6,.13,.02,10,.2,null,338,1 from dual union all
select 'HAPS Stage','HAPS','',.8,1,.1,50,.7,180,566,3 from dual union all
select 'HAPS','HAPS','',.8,1,.1,50,.7,180,566,3 from dual union all
select 'HAT','LongTom','',1.8,.41,.3,null,90.4,2.3,288,2 from dual union all
select 'HIBEX','HIBEX','',3.1,.79,6,null,null,null,340,1 from dual union all
select 'HIBEX-U','HIBEX','',3.1,.79,6,null,null,null,341,1 from dual union all
select 'HOTROC','HOTROC','',1.8,.3,.1,27,63.2,4.9,344,1 from dual union all
select 'HPAG','Sidewinder','',1.8,.13,.03,12,12.5,2.7,345,1 from dual union all
select 'HPS3','PSLV.St3','PS-3',3.6,2,8.4,1100,324,83,691,1 from dual union all
select 'HPS4','PSLV.St4','PS-4',2.6,2,2.9,920,14.8,425,485,2 from dual union all
select 'HTR','HTR','',10,.25,.5,null,30,20,347,1 from dual union all
select 'HV Arcas','Arcas','',1.6,.11,0,15,1.5,29,548,1 from dual union all
select 'HVAR','HVAR','',1.7,.13,.04,17,25,.9,348,1 from dual union all
select 'HYSR','HYSR','',15,.61,null,null,270,null,349,1 from dual union all
select 'Hades','Hades','',5,.53,1.8,null,null,null,156,1 from dual union all
select 'Hatf I','Stromboli','',3,.56,.9,246,75,16,1059,1 from dual union all
select 'Hawk','Orion','',5,.35,.6,null,13,32,522,1 from dual union all
select 'Hercules Booster','Hercules','',4.3,1.06,2.4,null,772,3.4,1221,4 from dual union all
select 'Hermes A-1','Hermes','',7.8,.86,5,null,71,null,350,1 from dual union all
select 'Hermes A-3A','HermesA3','',8.7,1.02,3.8,null,80,null,351,1 from dual union all
select 'Hermes A-3B','HermesA3','',10.1,1.19,5.3,2000,100,null,352,1 from dual union all
select 'Heron','Gosling','',3.6,.26,.3,72,137,3.1,326,1 from dual union all
select 'Hopi III','Hopi','',2,.11,.03,8,28,2.1,353,1 from dual union all
select 'Hopi IV','Hopi','',2.4,.11,.04,9,7.7,9,355,1 from dual union all
select 'Hopi','Hopi','',2.1,.11,.03,7,18,3,354,1 from dual union all
select 'Hound Dog','HoundDog','',0,0,0,0,0,0,null,0 from dual;
insert into STAGE
select 'Hwasong 10','R-27','',8,1.5,10,null,255,128,413,1 from dual union all
select 'Hwasong 5','R-11','',10,.88,5,null,93,null,905,1 from dual union all
select 'Hwasong 6','R-11','',10,.88,5,null,93,null,905,1 from dual union all
select 'Hwasong 7','R-11','',10,.88,5,null,93,null,905,1 from dual union all
select 'Hwasong ER','R-11','',12,.88,5,null,93,null,905,1 from dual union all
select 'Hwasong-12 Stage 1','Hwasong-12','',11.8,1.75,10,null,null,151,356,1 from dual union all
select 'Hwasong-14 Stage 2','Nodong','',2.1,1.25,3,null,null,224,413,1 from dual union all
select 'Hwasong-15 Stage 1','Hwasong-15','',2.1,2.25,3,null,null,224,346,1 from dual union all
select 'Hwasong-15 Stage 2','Hwasong-15','',2.1,2.25,3,null,null,224,346,1 from dual union all
select 'Hwasong-7','Nodong','',16,1.25,15.1,2000,255,110,655,1 from dual union all
select 'Hydac','Javelin','',3.7,.23,.3,null,58.7,null,358,1 from dual union all
select 'Hyperion','AMROC','',5.8,.147,.1,46,6,15,359,1 from dual union all
select 'Hyunmoo Booster','Hyunmoo','',4.2,1.2,2.4,null,800,null,360,1 from dual union all
select 'Hyunmoo-2A','Hyunmoo','',7,1.2,4,null,null,null,361,1 from dual union all
select 'Hyunmoo-2B','Hyunmoo','',7,1.2,4,null,null,null,361,1 from dual union all
select 'Hyunmoo-2C','Hyunmoo','',7,1.2,4,null,null,null,361,1 from dual union all
select 'IAI Arrow St2','IAI.Arrow.St2','Arrow St2',2.5,.5,.5,null,null,null,364,1 from dual union all
select 'IAI Arrow','Arrow2','Arrow',5,.8,1.5,null,null,null,363,1 from dual union all
select 'IAI Arrow-1','Jericho','Arrow',5,1.35,1.5,null,null,null,362,1 from dual union all
select 'INTA S-12','INTA-100','',1.6,.1,.03,null,16.7,1.2,367,1 from dual union all
select 'INTA-100','INTA-100','',2.5,.12,.03,null,1.5,32,1148,1 from dual union all
select 'IRIS Stage','IRIS.Fiat','IRIS',1.7,1.31,1.7,146,58,79,368,1 from dual union all
select 'IUS SRM-1','Orbus21','Orbus 21',3.5,2.3,10.9,1115,186,153,663,1 from dual union all
select 'IUS SRM-2','Orbus6','Orbus 6E',2.1,2.9,3.9,1005,76.5,104,664,1 from dual union all
select 'Icarus Stage 2','Icarus2','',1.4,.11,.01,4,4.1,7,372,1 from dual union all
select 'Icarus','Icarus','',2.3,.15,.04,4,15.8,6.4,371,1 from dual union all
select 'Ikar','Ikar','',2.6,2.72,3.1,2344,29.4,600,51,1 from dual union all
select 'Imp Malemute','Malemute','',3.3,.41,.6,100,55.1,22,373,1 from dual union all
select 'Improved Orion','Orion','Orion',1.8,.35,.4,null,13,32,520,1 from dual union all
select 'Iraqi Gun Projectile','IraqGun350','',1.5,.35,.07,67,null,null,null,0 from dual union all
select 'Iris','Iris','',4,.3,.5,112,17,52,542,1 from dual union all
select 'Iroquois','Iroquois','TE-M-388',2.7,.2,.1,43,24.4,7.7,1078,1 from dual union all
select 'Iskander','Oka','',7.3,.92,3.8,null,null,null,103,1 from dual union all
select 'Iskander-E','Oka','',7.3,.92,3.8,null,null,null,103,1 from dual union all
select 'Iskander-M','Oka','',7.3,.92,3.8,null,null,null,103,1 from dual union all
select 'Ivanov Booster','Ivanov','',.4,.14,.1,null,null,null,null,1 from dual union all
select 'Ivanov','Ivanov','',1,.14,.1,null,null,null,null,1 from dual union all
select 'JCR Booster','JCR','',5,.51,.7,null,133,null,376,1 from dual union all
select 'JCR','JCR','',5,.51,.7,null,54,null,377,1 from dual union all
select 'Javelin 3','Javelin','',2.6,.23,.2,null,93,null,381,1 from dual union all
select 'Jericho II','Belier','',2.7,.31,.3,null,20,23.8,383,1 from dual union all
select 'Jericho IIIA1','Jericho','',7.5,1.35,10,null,null,null,null,1 from dual union all
select 'Jericho IIIA2','Jericho','',7.5,1.35,10,null,null,null,null,1 from dual union all
select 'Jericho-1','Arrow2','',4.5,.8,2,null,null,null,626,1 from dual union all
select 'Judi I','Loki','',1.7,.08,.01,2,9,1.9,385,1 from dual union all
select 'Judi III','Loki','',2.3,.08,0,2,9,1.9,386,1 from dual union all
select 'Jupiter','Jupiter','',14.6,2.67,54.4,5400,660,180,888,1 from dual union all
select 'K-15 Booster','K-15','',2,.5,1,null,null,null,387,1 from dual union all
select 'K-15 Stage 2','K-15','',5,.5,2,null,null,null,388,1 from dual union all
select 'K-4 Booster','K-4','',5,1.3,8,null,null,null,389,1 from dual union all
select 'K-4 Stage 2','K-4','',5,1.3,8,null,null,null,390,1 from dual union all
select 'K-BATS','ATACMS','',4,.61,1.2,null,null,null,391,1 from dual union all
select 'K10S','K10S','',.5,.3,.1,null,null,null,392,1 from dual union all
select 'K150','K150','',3.5,.15,.1,10,19,15.8,393,1 from dual union all
select 'K245','K250','',1,.25,.1,40,34,9.3,396,1 from dual union all
select 'K250','K250','',4.6,.25,.3,50,43,7.6,396,1 from dual union all
select 'K420','K420','',5.8,.42,1.2,200,110,17,399,1 from dual union all
select 'K420(1/3)','K420','',2,.42,null,null,45,null,400,1 from dual union all
select 'K420H','K420','',5.8,.42,1.2,200,110,17,401,1 from dual union all
select 'KM Rockoon','KM-D','',1.3,.7,.3,50,null,null,404,1 from dual union all
select 'KM-D','KM-D','',1,.63,.3,24,17.4,38,404,1 from dual union all
select 'KM-H','KM-H','',1.4,.93,.1,13,null,null,405,1 from dual union all
select 'KM-L','KM-L','',.1,.1,.004,1,null,null,406,1 from dual union all
select 'KM-M','M-40','',1.3,.8,.6,45,32,63,407,1 from dual union all
select 'KM-P','M-40','',1.2,.79,.5,42,31.3,44,408,1 from dual union all
select 'KM-V1','KM-V','',1.5,1.2,1.4,118,52,73,409,1 from dual union all
select 'KM-V2b','KM-V2','',2.3,1.4,2.9,800,99.8,90,410,1 from dual union all
select 'KM-V2c','KM-V2','',2.2,1.4,2.9,400,99.6,88,411,1 from dual union all
select 'KN-02','Tochka','',6.4,.65,2,null,null,null,412,1 from dual union all
select 'KN-11','Bukguekseong','',4,1.5,10,null,255,128,414,1 from dual union all
select 'KN-11-2','Bukguekseong','',2.8,1.5,7,null,255,128,415,1 from dual union all
select 'KSLV-1 Stage 2','KSLV.St2','',1.6,1,1.4,200,80,44,419,1 from dual union all
select 'KSR Booster','KSR','',3.6,.42,.8,null,211,5.9,421,1 from dual union all
select 'KSR','KSR','',4,.42,1.2,null,86,20,420,1 from dual union all
select 'KSR-III','KSR-3','',10,1,5.6,null,126,59,422,1 from dual union all
select 'KT-1 St3','KT1-3','',2,1.2,2,null,null,null,423,1 from dual union all
select 'KZ-1 St3','KT1-3','',2,1.2,3.2,null,null,55,435,1 from dual union all
select 'KZ-1 St4','KT1-3','',1.5,1.1,2,null,null,null,436,1 from dual union all
select 'KZ-1A St1','DF-21','',6,1.4,16.6,null,null,65,269,1 from dual union all
select 'KZ-2 St3','KZ2-3','',2,1.4,3.2,null,null,55,437,1 from dual union all
select 'Kappa 1','Kappa','',2.7,.13,.04,20,10,3,438,1 from dual union all
select 'Kappa 3 Booster','K250','',2.5,.22,.2,null,null,null,395,1 from dual union all
select 'Kappa 4 Booster','Kappa4','',3.3,.33,.15,null,105,3,440,1 from dual union all
select 'Kappa 4','Kappa','',2.7,.13,.04,20,10,3,439,1 from dual union all
select 'Kappa 6','K250','',2.3,.25,.2,40,34,9.3,396,1 from dual union all
select 'Kappa 6H','K250','',3.6,.25,.3,50,null,7.2,398,1 from dual union all
select 'Kappa 8','K250','',3.1,.25,.3,50,34,9.7,397,1 from dual union all
select 'Kappa 8L','Kappa','',2.4,.16,.1,20,null,5.3,394,1 from dual union all
select 'Kavoshgar','Kavoshgar','Iran',4,.36,.3,0,0,null,441,1 from dual union all
select 'Khorramshahr','R-27','',8,1.5,10,null,255,128,413,1 from dual union all
select 'Kisha','Deacon','',2,.166,.1,null,21,4.4,443,1 from dual union all
select 'Kiva','Kiva','',2.6,.166,.1,24,27,5.5,444,1 from dual union all
select 'L-1011','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'L-3 Blok G','BlokG','',9.1,4.4,61.8,6000,445,443,636,1 from dual union all
select 'L140-YF21','DF5','',20.2,3.35,151,9000,2785,132,1232,4 from dual union all
select 'L180-YF21B','DF5','L180-YF21B',23.1,3.35,180,10000,2961,130,1231,4 from dual union all
select 'L35-YF24','CZ.St2','L35-YF24',9.7,3.35,39,4000,765.8,110,1233,1 from dual union all
select 'L35-YF24B','CZ.St2','L35-YF24B',11.5,3.35,34,4000,788.4,110,1234,1 from dual union all
select 'L480S','L480S','',.5,.48,.1,15,7.8,31.5,445,1 from dual union all
select 'L500','L500','',2.5,.5,.8,null,69,27,446,1 from dual;
insert into STAGE
select 'L735','Lambda','',8.4,.74,5,2945,410,28.8,447,1 from dual union all
select 'L735(1/3)','Lambda.St2','L735(1-3)',3.9,.74,2.5,null,118,38.4,448,1 from dual union all
select 'LAPSTAR','Lupus','',1.5,.13,.05,15,13.7,2.7,450,1 from dual union all
select 'LE-3','LE-3','',5.4,1.62,5.8,500,54,250,453,1 from dual union all
select 'LE-5','LE-5','',9.3,2.44,10.6,1800,103,370,455,1 from dual union all
select 'LEO-1','Orbus1','',1.2,.7,.7,45,50,51.5,460,1 from dual union all
select 'LEO-46','LEO-46','',11.2,1.17,22.1,1700,1265,76,461,1 from dual union all
select 'LEO-7','LEO-7','',2.7,1.17,3.6,350,195.1,62,462,1 from dual union all
select 'LM AS','LMAS','',2.5,3.76,4.9,2000,15.6,480,846,1 from dual union all
select 'LPR-006','LPR-006','',3,.13,.2,null,6,null,467,1 from dual union all
select 'LPR-3T','ISRO-TV','',2,.25,.3,null,null,null,468,1 from dual union all
select 'LR87','Titan','',16,3.05,76.2,4000,1470,138,472,2 from dual union all
select 'LR87AJ11','Titan','',23.8,3.05,116.6,5500,2353,147,473,2 from dual union all
select 'LR87AJ11A','Titan','',26.4,3.05,163,8000,2452,164,474,2 from dual union all
select 'LR87AJ5','Titan','',21.5,3.05,122,4000,2090,147,475,2 from dual union all
select 'LR87AJ9','Titan','',23.8,3.05,116.6,5500,2353,147,473,2 from dual union all
select 'LR91','Titan.St2','',8,3.05,28.9,1725,356.1,225,477,1 from dual union all
select 'LR91AJ11','Titan.St2','',9.6,3.05,29.2,2650,449.2,205,478,1 from dual union all
select 'LR91AJ11A','Titan.St2','',10,3.05,39.5,4500,472.2,223,479,1 from dual union all
select 'LR91AJ5','Titan.St2','',7.3,3.05,28.9,2400,440,182,480,1 from dual union all
select 'LR91AJ9','Titan.St2','',9.6,3.05,29.2,2650,449.2,205,481,1 from dual union all
select 'LS-A Booster','STA','',3,.35,.4,null,60,10,483,1 from dual union all
select 'LS-A','LS-C','',4.5,.3,.3,null,10,35,451,1 from dual union all
select 'LS-C Booster','LS-C','',2.5,.6,.3,null,null,null,484,1 from dual union all
select 'LS-C','LS-C','',4,.6,1,null,null,null,452,1 from dual union all
select 'LS-C-7','LS-C','',4,.6,1,null,null,null,454,1 from dual union all
select 'LT Thor DSV-2L-1A','Thor','LT Thor DSV-2L',21,2.44,70.3,3715,860,215,556,1 from dual union all
select 'LT Thor DSV-2L-1B','Thor','LT Thor DSV-2L',20.3,2.44,70.3,3715,860,215,556,1 from dual union all
select 'LT Thor DSV-2L-1C','Thor','LT Thor DSV-2L',20.3,2.44,70.3,3715,860,215,556,1 from dual union all
select 'LT Thor N','Thor','',20.3,2.44,70.3,3715,860,215,556,1 from dual union all
select 'LaRC Plastic','Plastic','LaRC Plastic rocket',.5,.09,0,1,.3,3,486,1 from dual union all
select 'Lacrosse','Lacrosse','',2.1,.4,.3,null,168,2.6,1122,1 from dual union all
select 'Lambda2-2','K420','',8.7,.42,1.4,300,110,17,399,1 from dual union all
select 'Lambda3-2','K420','',11,.42,1.8,300,110,17,399,1 from dual union all
select 'Lambda3-3','K420','',4.8,.42,.5,100,110,null,399,1 from dual union all
select 'Lance Missile','Lance','',6.2,.56,null,null,20,114,685,1 from dual union all
select 'Lance XM-45','LanceMotor','',4.9,.38,.8,null,222,5,1125,1 from dual union all
select 'Lance','LanceMotor','',4.6,.38,.8,217,174,6.7,487,1 from dual union all
select 'Lark','Lark','',4.2,.46,.5,null,2.8,260,470,1 from dual union all
select 'Lex','Lex','',2.6,.16,.1,20,10,20,577,1 from dual union all
select 'Lobster I','Lobster','',1.9,.18,.03,null,29,2,491,1 from dual union all
select 'Loki II','Loki','Loki',1.7,.08,.011,2,9,2,493,1 from dual union all
select 'Loki SR71-AD-1','Loki','',1.7,.08,.011,2,9,2,974,1 from dual union all
select 'Loki','Loki','Loki',1.7,.08,0,3,15,.8,492,1 from dual union all
select 'Loki-Dart','Loki','Loki II Dart',1.7,.08,0,3,15,.8,492,1 from dual union all
select 'Long Tom','LongTom','',3.1,.47,.6,null,190,3.6,583,4 from dual union all
select 'Luna-M','Luna','',9.4,.54,2.3,null,null,null,102,1 from dual union all
select 'Lupus 2','Lupus','',1.7,.13,.04,16,9.5,4,495,1 from dual union all
select 'Lupus 3','Lupus','',2,.13,.04,16,9.5,4,497,1 from dual union all
select 'Lupus','Lupus','',1.7,.13,.03,16,9.5,5.2,494,1 from dual union all
select 'Luzou','Luzou','',.3,.15,.01,3,3,1.7,498,1 from dual union all
select 'Lynx','Standard','',3,.35,.5,null,null,null,597,1 from dual union all
select 'M-10','Mu','',13,1.41,26.6,5907,737,61,499,1 from dual union all
select 'M-100 St2','M-100','',1.6,.25,.1,55,null,4.5,501,1 from dual union all
select 'M-100','M-100','',3.5,.25,.3,110,null,5,500,1 from dual union all
select 'M-13','Mu','',14.8,1.41,33.4,6290,954,66,502,1 from dual union all
select 'M-13TVC','Mu','',14.8,1.41,34.2,6987,1020,66,502,1 from dual union all
select 'M-14','MV','',14.1,2.51,87,12070,3780,49,503,1 from dual union all
select 'M-20','M-20','',4.7,1.41,9.9,2779,285,66,504,1 from dual union all
select 'M-22TVC','M-20','',5,1.41,9.8,2607,285,66,505,1 from dual union all
select 'M-23','M-20','',6.2,1.41,13,2742,523,73,506,1 from dual union all
select 'M-24','M-24','',6.6,2.51,38,3410,1240,71,507,1 from dual union all
select 'M-25','M-24','',6.6,2.51,38,3410,1240,71,508,1 from dual union all
select 'M-250','MR-12','',8.8,.45,1,null,null,null,571,1 from dual union all
select 'M-30','M-30','',4,.86,2.8,773,128,42,509,1 from dual union all
select 'M-34','M-34','',3.4,2.2,11.6,1000,290,102,511,1 from dual union all
select 'M-34c','M-34','',4.3,2.2,12.3,1500,327,105,512,1 from dual union all
select 'M-35','M-35','',4,2.6,17.2,2200,445,129,513,1 from dual union all
select 'M-3A','M-40','',2.3,1.14,1.1,130,61,54,514,1 from dual union all
select 'M-3B','M-40','',2.7,1.5,3.6,310,132,87,515,1 from dual union all
select 'M-40','M-40','',1.8,.79,.4,73,26.5,40,516,1 from dual union all
select 'M-6','HJ','M6 Honest John',4.9,.58,1.9,954,365,5,517,1 from dual union all
select 'M55A1','Minuteman','',7.5,1.68,22.9,2500,792,59.3,525,1 from dual union all
select 'M55E1','Minuteman','',7.5,1.68,22.9,2320,892,60,1096,1 from dual union all
select 'M55E1-ATK','Minuteman','',7.5,1.68,22.9,2522,892,60,1097,1 from dual union all
select 'M56A1','Aries','Aries',3.8,1.12,5.2,750,200,60,526,1 from dual union all
select 'M57A1','Minuteman.St3','',2.1,.97,1.9,300,76,59,528,1 from dual union all
select 'MARC 14A1','ArcasB','',.7,.11,.016,11,12,.8,543,1 from dual union all
select 'MARC 14B1','TTB','',.8,.3,.1,59,84,.8,544,7 from dual union all
select 'MARC 42A1','ArcasB','',1.6,.11,.03,15,12.2,3,549,1 from dual union all
select 'MARC 42A2','ArcasB','',.9,.11,0,null,null,null,550,1 from dual union all
select 'MCR-3','RH-125','',1,.12,.02,null,null,null,557,1 from dual union all
select 'MD-620','Agate','',4.1,.8,2,null,null,null,626,1 from dual union all
select 'MERA','Mera','',1,.15,.02,4,null,null,558,1 from dual union all
select 'MG-18','Altair','M-2',1.5,.46,.3,null,null,null,559,1 from dual union all
select 'MLRS','MLRS','',2,.23,.3,null,53,3.2,560,1 from dual union all
select 'MMR-06','MMR-06','',2.5,.2,.1,20,null,9,561,1 from dual union all
select 'MR-1','Sparrow','',1.6,.41,.22,124,102,1.9,3,3 from dual union all
select 'MR-12','MR-12','',8.8,.45,1,null,null,null,568,1 from dual union all
select 'MR-20','MR-12','',8.8,.45,1,null,null,null,570,1 from dual union all
select 'MR-30','MR-12','',8.8,.45,1,null,null,null,572,1 from dual union all
select 'MR-UR-100 BR','BR','',1,2.15,1,500,null,null,29,1 from dual union all
select 'MR-UR-100 St 1','MR-UR-100','',14.3,2.25,59,5000,1236,null,789,1 from dual union all
select 'MR-UR-100 St 2','MR-UR-100.St2','',3.2,2.15,10,1000,142,195,28,1 from dual union all
select 'MSBS 401','MSBS','',5.4,1.93,22.5,2500,900,60,937,1 from dual union all
select 'MSBS 402','MSBS.St2','',2,1.93,8.8,800,300,70,938,1 from dual union all
select 'MSBS 403','MSBS.St3','',1.1,1.93,1.5,100,100,40,939,1 from dual union all
select 'MSBS 501','M51','',6.5,2.3,30,2500,900,60,574,1 from dual union all
select 'MSBS 502','M51','',2.3,2.3,10,800,300,70,575,1 from dual union all
select 'MSBS 503','M51','',1.3,2.3,5,100,100,40,576,1 from dual;
insert into STAGE
select 'MSD','Star20','',1.5,.5,.3,26,26.5,27.5,1022,1 from dual union all
select 'MT-135P','MT-135','',3.3,.14,0,null,12,10.5,578,1 from dual union all
select 'MX-774','MX-774','',5.7,.76,1.1,500,35,75,579,4 from dual union all
select 'MX-775','Snark','',5.1,.69,1.2,0,580,4,620,2 from dual union all
select 'Mace','Matador','',0,0,0,0,0,0,1098,0 from dual union all
select 'Malemute','Malemute','',3.3,.41,.6,100,55.1,22,1102,1 from dual union all
select 'Martlet 1','Gun16','',1.5,.13,.2,170,0,0,null,0 from dual union all
select 'Martlet 2','Gun16','',1.5,.13,.2,170,0,0,null,0 from dual union all
select 'Martlet 3A','Gun16','',1.5,.13,.2,170,0,0,null,0 from dual union all
select 'Martlet 3B','Gun16','',1.5,.13,.2,170,0,0,null,0 from dual union all
select 'Matador','Matador','',0,0,0,0,0,0,1106,0 from dual union all
select 'Mayfly','LongTom','',3.1,.23,.25,null,62,3.6,583,1 from dual union all
select 'Melanie 1','Monica','',1.1,.16,.01,null,null,null,584,1 from dual union all
select 'Melanie','Berenice','',3,.22,.1,40,38,4.4,584,1 from dual union all
select 'Melusine','Monica','',1.9,.16,0,29,null,8,585,1 from dual union all
select 'Mesquito','MLRS','',2,.23,.3,null,53,3.2,560,1 from dual union all
select 'Meteo','Meteo','',3.4,.15,.02,null,null,null,null,1 from dual union all
select 'Meteor 1','Meteor','',1.8,.11,.03,10,14,2.3,595,1 from dual union all
select 'Meteor 3 Booster','Meteor','',1.8,.11,.03,10,14,2.3,595,1 from dual union all
select 'Meteor 3B Booster','Meteor','',1,.2,.06,null,null,2.3,595,3 from dual union all
select 'Meteor SRB','Meteor','',1.7,.11,0,15,14,2.3,595,1 from dual union all
select 'Meteor-2','Meteor2','',4.2,.35,.4,126,24,18,596,1 from dual union all
select 'Metroc','Loki','',1.4,.08,.01,0,.6,17.5,545,1 from dual union all
select 'MiG31D 072','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'Mimosa','Mimosa','',2.6,.2,.2,40,33,5,944,1 from dual union all
select 'Mk 104 (ARC)','Standard','',2.9,.35,.5,128,null,null,597,1 from dual union all
select 'Mk 104','Standard','',2.9,.35,.5,128,null,null,597,1 from dual union all
select 'Mk 30','Standard','',3.5,.35,.4,null,null,null,606,1 from dual union all
select 'Mk 56','Standard','',4.5,.35,.6,null,null,null,611,1 from dual union all
select 'Mk 72','Mk72','',1.7,.53,.7,243,null,null,614,1 from dual union all
select 'Mk7','FFAR','',1.2,.07,.01,9,9.8,.9,616,1 from dual union all
select 'Momo','Momo','',6.2,.5,.8,null,null,null,618,1 from dual union all
select 'Musca','Musca','',1.7,.09,.01,null,5.1,3.7,619,1 from dual union all
select 'N-1 Blok-A','N-1','N-1 Blok A',30,14,1870,130000,45300,110,634,30 from dual union all
select 'N-1 Blok-B','11A52B','Blok B',20.5,10.3,540,56000,14000,130,635,8 from dual union all
select 'N-1 Blok-V','11A52V','Blok V',12.5,6.4,185,13700,1610,400,637,4 from dual union all
select 'NA-805','Agate','',5,.8,4.1,null,null,null,627,1 from dual union all
select 'NAL-16','STA','',3.9,.165,.1,40,10,14,628,1 from dual union all
select 'NAL-25','STA','',4,.25,0,null,null,null,629,1 from dual union all
select 'NAL-7','STA','',2,.07,0,null,5,1,630,1 from dual union all
select 'NAL-735','SB-735','',9,.74,4.7,null,250,49,631,1 from dual union all
select 'NB-52','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'NOTS 124-C','Sidewinder','',1,.13,.03,null,null,null,643,1 from dual union all
select 'NOTS 3in Sph.','NOTS.3inSM','',.1,.08,0,1,.7,1,644,1 from dual union all
select 'NOTS 401A','NOTS400','',.8,.5,null,null,null,7.2,645,1 from dual union all
select 'NOTS 500','NOTS500','',2.5,.61,.9,null,null,45,646,1 from dual union all
select 'NOTS 551','NOTS551','',3,.34,.4,85,null,28,648,1 from dual union all
select 'NOTS 8in','NOTS.8in','',.5,.2,.01,3,5.1,5.7,647,1 from dual union all
select 'NSPO Booster','NSPOB','',2.4,.55,1,null,null,6,652,1 from dual union all
select 'NSPO Stage 2','NSPO','',3.9,.42,1,null,null,30,653,1 from dual union all
select 'Naro Stage 1','Angara','',25.7,2.9,142.4,9800,1670,337.5,767,1 from dual union all
select 'Navaho Booster','Navaho','',23.2,null,null,null,0,0,310,2 from dual union all
select 'Navaho Missile','X-10','',21.3,null,null,null,0,0,null,0 from dual union all
select 'Nazeat','Kavoshgar','Iran',4,.36,.3,0,0,null,441,1 from dual union all
select 'Neptune','Neptune','',3.2,.39,.5,150,20,32,654,1 from dual union all
select 'New Shepard PM','NewShepard','',12,3,75,10000,1020,145,198,4 from dual union all
select 'Nihka','BB5','',2,.44,.4,79,49.9,17.8,52,1 from dual union all
select 'Nike','Nike','',3.6,.42,.7,256,195.6,3.4,529,1 from dual union all
select 'Nike-46 Sustainer','Ajax','',6,.3,.4,null,11.4,21,1205,1 from dual union all
select 'Nike-46','Nike-46','',1,.4,1,null,392,2,54,4 from dual union all
select 'Nike-47 Sustainer','Ajax','',6,.3,.4,null,11.4,21,58,1 from dual union all
select 'Nike-47','Nike-46','',1,.4,1.1,null,392,2,55,4 from dual union all
select 'Nike-484 Sustainer','Ajax','',6,.3,.4,null,11.4,21,58,1 from dual union all
select 'Nike-490 Sustainer','Ajax','',6,.3,.4,null,13.8,71,169,1 from dual union all
select 'Nudol'' Booster','A-350','',8,2.6,15,null,null,null,656,1 from dual union all
select 'Nudol''','A-350','',7,2.6,15,null,null,null,657,1 from dual union all
select 'OAM','OAM','',1,2.3,.7,360,.9,550,566,4 from dual union all
select 'OIS','Star27','',1.2,.69,.4,24,26.7,34.3,1028,1 from dual union all
select 'OS-X0','DF-11','',7.4,.88,5.4,1645,93.2,null,266,1 from dual union all
select 'OTRAG','OTRAG','',7,.76,null,null,120,null,659,4 from dual union all
select 'OV','STS.OV','',37.2,5.49,110,80000,54,90,658,2 from dual union all
select 'Oka','Oka','',7.5,.92,4.6,null,null,null,660,1 from dual union all
select 'Orbus 1','Orbus1','',1.2,.7,.5,50,88,null,661,1 from dual union all
select 'Orbus 1A','Orbus1','',1.2,.7,.5,50,88,null,662,1 from dual union all
select 'Orbus 21D','Orbus21','ESBM',3,2.3,10.8,1030,187,150,663,1 from dual union all
select 'Oreste','Monica','',.5,.13,.01,null,.9,8,665,1 from dual union all
select 'Oriole','Oriole','',3.9,.56,1,198,92.1,null,311,1 from dual union all
select 'Orion 38','Pegasus.St3','Pegasus 3',1.3,.97,.9,126,36,69.6,666,1 from dual union all
select 'Orion 50','Pegasus.St2','Pegasus 2',2.7,1.27,3.4,345,118,73.3,667,1 from dual union all
select 'Orion 50S','Pegasus','Pegasus 1',8.9,1.27,13.4,1900,485,72.4,668,1 from dual union all
select 'Orion 50SG','Pegasus','Taurus-Lite',10,1.27,15,null,null,null,670,1 from dual union all
select 'Orion 50SXLG','Pegasus','Taurus-Lite',10,1.27,15,null,704,null,670,1 from dual union all
select 'Orion 50SXLT','Pegasus','',10,1.27,15,null,704,null,671,1 from dual union all
select 'Orion 50XL','Pegasus.St2','Pegasus XL Stage 2',3.7,1.27,4,420,196,72.5,672,1 from dual union all
select 'Orion 50XLT','Pegasus.St2','',3.7,1.27,4,420,196,72.5,673,1 from dual union all
select 'Orion LAS','Orion.LAS','',10,1,6,null,null,null,674,1 from dual union all
select 'Orion','Orion','',2.7,.35,.4,111,13,32,522,1 from dual union all
select 'Orion-1','IIAE.Orion','',3,.21,.1,55,null,12,675,1 from dual union all
select 'Orion-2','IIAE.Orion','',3,.21,.1,55,null,12,676,1 from dual union all
select 'P064','Rubis','Rubis',1.5,.65,.7,89,52,45,678,1 from dual union all
select 'P068','P068','P0.68',1.7,.8,.8,62,52.5,46.5,679,1 from dual union all
select 'PAL','PAL','',18.6,2.17,44,4500,758.5,140,1172,1 from dual union all
select 'PAM-D2','PAM-D2','',2.7,1.6,3.5,230,85,118,1053,1 from dual union all
select 'PAP','PAP','',11.5,1.07,12.6,3100,650,42,687,1 from dual union all
select 'PBS','PBS','',1,1.32,1,null,1.4,null,845,1 from dual union all
select 'PDV Target St2','Prithvi','',2,1.1,2,null,null,null,66,1 from dual union all
select 'PS2','PSLV.St2','PS-2',12.5,2.8,45.8,5300,724,158,1166,1 from dual union all
select 'PS3','PSLV.St3','PS-3',3.6,2,8.4,1100,324,83,691,1 from dual union all
select 'PS4','PSLV.St4','PS-4',2.6,2,2.9,920,14.8,425,485,2 from dual union all
select 'PSOM','ASLV','',11.3,1,11,2000,677,44.2,692,1 from dual;
insert into STAGE
select 'PSOM-XL','ASLV','',13.5,1,13.5,2000,720,49,692,1 from dual union all
select 'Paekkom Booster','Hercules','',4.3,1.06,2.4,null,null,null,697,4 from dual union all
select 'Paiute','Genie','',1.9,.38,.3,81,186,3.7,1101,1 from dual union all
select 'Patriot','Malemute','',3.3,.41,.6,100,55.1,22,1120,1 from dual union all
select 'Peacekeeper PBPS','MX.St4','',1.1,2.35,1.3,550,11.8,168,851,1 from dual union all
select 'Pedro','Pedro','',3.1,.71,1.5,250,244,11.3,1114,1 from dual union all
select 'Pegasus XL Stage 1','Pegasus','Pegasus XL1',10.3,1.27,17,2000,594,73.4,669,1 from dual union all
select 'Peregrine','Peregrine','',6.2,.44,1.2,null,null,null,698,1 from dual union all
select 'Perekhodnik','Perekhodnik','',0,0,0,0,0,0,null,0 from dual union all
select 'Perimetr Stage 1','MR-UR-100','',14.9,2.25,59,5000,1236,null,789,1 from dual union all
select 'Perimetr Stage 2','MR-UR-100.St2','',3.2,2.15,10,1000,142,195,28,1 from dual union all
select 'Pershing 2 St1','Pershing','',4,1.02,4.1,530,172,58,699,1 from dual union all
select 'Pershing 2 St2','Pershing','',2.8,1.02,5.8,350,134,46,700,1 from dual union all
select 'Petrel 2','Petrel','Petrel',1.8,.19,.1,26,4.5,30,488,1 from dual union all
select 'Petrel','Petrel','',1.8,.19,.1,26,4.5,30,488,1 from dual union all
select 'Polaris TV','Polaris','-  [0.78m?]',6,.79,8.3,1450,285,23,1116,1 from dual union all
select 'Pollux','Sergeant','',5.7,.79,4.2,null,null,38,701,1 from dual union all
select 'Prithvi','Prithvi','',8.5,1.1,4.5,null,null,null,703,1 from dual union all
select 'Private A Booster','PrivateBooster','',1,.47,.07,null,98,.2,1064,4 from dual union all
select 'Private A','PrivateA','',2.3,.25,.3,130,4.4,30,71,1 from dual union all
select 'Private F','PrivateA','',2.3,.25,.3,130,4.4,30,71,1 from dual union all
select 'Proson St1','Proson','',1.5,.2,.1,null,null,null,704,1 from dual union all
select 'Proson St2','Proson2','',1.6,.106,.03,null,null,null,705,1 from dual union all
select 'Prosper','Monica','',1.2,.16,.01,null,null,null,584,1 from dual union all
select 'R-1','V-2','',14.1,1.65,13.4,4000,267,65,750,1 from dual union all
select 'R-11','R-11','',7.4,.88,4.6,955,93,null,905,1 from dual union all
select 'R-11FM','R-11','',7.4,.88,4.5,1080,93,63,906,1 from dual union all
select 'R-12 11K63A','R-12','',18,1.65,39.5,4810,625.6,125,773,1 from dual union all
select 'R-12 63S1A','R-12','',18,1.65,39.5,4810,625.6,125,773,1 from dual union all
select 'R-12 63S1MA','R-12','',18,1.65,39.5,4810,625.6,125,773,1 from dual union all
select 'R-12','R-12','',18,1.65,34.6,4810,625.6,125,773,1 from dual union all
select 'R-13','R-13','',10,1.3,13.7,null,null,null,909,1 from dual union all
select 'R-14 11K65A','R-14','',22.4,2.4,87.2,5300,1745,130,776,1 from dual union all
select 'R-14 11K65MA','R-14','',22.4,2.4,87.2,5300,1745,130,776,1 from dual union all
select 'R-14 65S3A','R-14','',22.4,2.4,87.2,5300,1745,130,776,1 from dual union all
select 'R-14','R-14','',22.4,2.4,87.2,5300,1745,130,776,1 from dual union all
select 'R-17','R-11','Scud B',7.4,.88,5.4,1645,93.2,null,917,1 from dual union all
select 'R-2','V-2','',17,1.65,20.3,4380,363,85,751,1 from dual union all
select 'R-21','R-21','',12,1.3,16.6,null,null,null,919,1 from dual union all
select 'R-27','R-27','',8,1.5,10,null,255,128,78,1 from dual union all
select 'R-29 St 1','R-29','',7.7,1.9,20,null,682,74,746,1 from dual union all
select 'R-29 St 2','R-29.St2','',3.1,1.8,10,null,null,null,713,1 from dual union all
select 'R-29R St 1','R-29','',7.7,1.9,20,null,682,74,746,1 from dual union all
select 'R-29R St 2','R-29.St2','',3.1,1.8,10,null,null,null,713,1 from dual union all
select 'R-29RM St1','R-29','',7.7,1.9,20,null,682,74,746,1 from dual union all
select 'R-29RM St2','R-29.St2','',4.9,1.9,10,null,null,94,714,1 from dual union all
select 'R-29RM St3','R-29.St3','',2.8,1.9,10,null,null,87,715,1 from dual union all
select 'R-31 BS','BR','',2.5,1.54,1.3,200,4,101,707,4 from dual union all
select 'R-31 St 1','R-31','',6.3,1.54,17.5,2000,579,84,74,1 from dual union all
select 'R-31 St 2','R-31.St2','',3.1,1.54,6.9,800,233,74,706,1 from dual union all
select 'R-39 St 1','R-39','',9.7,2.4,53.7,null,2060,null,75,1 from dual union all
select 'R-39 St 2','R-39.St2','',5,2.4,25,null,null,null,708,1 from dual union all
select 'R-39 St 3','R-39.St3','',4,2.4,10,null,null,null,709,1 from dual union all
select 'R-5','R-5','',20.7,1.65,28.6,4400,500,115,752,1 from dual union all
select 'R-UNK','UNK','',0,0,null,null,null,null,null,1 from dual union all
select 'RB-57B','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'RH-125','RH-125','',1.4,.12,.02,11,8.4,2.9,816,1 from dual union all
select 'RH-125S','RH-125','',1.6,.12,.02,14,1.3,32,817,1 from dual union all
select 'RH-200','RH-200','',1.4,.2,.1,null,16.9,5.7,818,1 from dual union all
select 'RH-200SV','RH-200','',1.4,.21,.14,48,16.9,10,819,1 from dual union all
select 'RH-300 Booster','Venus','',2,.28,.2,65,37.5,6,821,1 from dual union all
select 'RH-300 Mk II','Belier','',3.4,.31,.5,null,39.1,20.2,822,1 from dual union all
select 'RH-300','Belier','',2.8,.31,.3,79,33.3,16,820,1 from dual union all
select 'RH-560','Stromboli','',3.9,.56,.9,277,76,19,823,1 from dual union all
select 'RH-560S','Stromboli','',4.1,.56,1.3,262,119,19,824,1 from dual union all
select 'RH-75','RH-75','',1.2,.08,.01,null,null,null,825,1 from dual union all
select 'RH-ATV','Stromboli','',3.7,.56,.9,277,76,19,823,1 from dual union all
select 'RSA-3-1','Jericho','',6.3,1.35,10.2,1100,456,52,856,1 from dual union all
select 'RSA-3-2','Jericho','',6.4,1.35,11,1771,477,52,857,1 from dual union all
select 'RSA-3-3','AUS-51','',2.1,1.35,2,170,59,94,858,1 from dual union all
select 'RSRM','SRB','',45.5,3.77,590,88000,11790,123,977,1 from dual union all
select 'RT-21 St1','Temp-2S','',8.6,1.79,26.6,null,null,63,718,1 from dual union all
select 'RT-21 St2','RT-21.St2','',4.4,1.47,8.7,null,null,null,719,1 from dual union all
select 'RT-21 St3','Topol.St3','',3.9,1.34,8.7,1000,245,null,720,1 from dual union all
select 'RT-23 St 3','RT-23.St3','',3.6,2.4,15,null,206,null,807,1 from dual union all
select 'RTV Motor','BabySergeant','Baby Sergeant',1.2,.15,.025,6,8.6,7.6,860,1 from dual union all
select 'RTV-A-3','RTV','',2.5,.46,.6,280,12,null,632,1 from dual union all
select 'RV-A-10','Sergeant','',4,.79,3.5,800,140,30,861,1 from dual union all
select 'RX-250-LPN','RX-250','',5.3,.25,.3,null,52,6,862,1 from dual union all
select 'RX-320','RX-320','',4.7,.32,.5,null,52,null,863,1 from dual union all
select 'RX-420','RX-420','',6,.42,.5,null,100,null,864,1 from dual union all
select 'Raven 1/1A/2A','Raven','',5.1,.44,1,240,44.5,30,866,1 from dual union all
select 'Raven 11','Raven','Raven',5.5,.44,1.3,290,105,39,867,1 from dual union all
select 'Raven 2','Raven','',5.1,.44,1,240,44.5,30,868,1 from dual union all
select 'Raven 5','Raven','',5.1,.44,1,240,44.5,30,869,1 from dual union all
select 'Raven 6','Raven','Raven',5.4,.44,1.2,242,52.5,36,870,1 from dual union all
select 'Raven 7','Raven','',5.1,.44,1,240,44.5,30,871,1 from dual union all
select 'Raven 8','Raven','Raven',5.4,.44,1.1,230,50.1,30,872,1 from dual union all
select 'Recruit 1A','Recruit','TE29-1A',3.1,.23,.2,67,132,2.2,1076,1 from dual union all
select 'Recruit','Recruit','XM-19',2.6,.23,.2,null,156,1.5,2,1 from dual union all
select 'Redstone MRLV','Redstone','',15.5,1.78,33,3000,370,175,874,1 from dual union all
select 'Redstone','Redstone','',13.4,1.78,33,3000,370,155,874,1 from dual union all
select 'Regulus II','Regulus','',20.5,1.3,10.2,null,0,0,null,0 from dual union all
select 'Regulus','Regulus','',10.1,1.4,6.6,null,0,0,null,0 from dual union all
select 'Rita I','Rita','Rita 1',2.3,1.51,4.8,350,180,62,681,1 from dual union all
select 'Rita II','Rita','Rita 2',3,1.51,6,null,320,52,875,1 from dual union all
select 'Rocketsonde','Rocketsonde','',2,.11,0,15,null,null,null,1 from dual union all
select 'Rocketsonde2','Rocketsonde2','',2,.11,0,15,null,null,null,1 from dual union all
select 'Rook II','Raven','Rook',5.4,.44,1.2,304,300,7,876,1 from dual union all
select 'Rook IIIA','Raven','Rook',5.4,.44,1.1,186,300,5.5,876,1 from dual;
insert into STAGE
select 'Rook IIIB','Raven','Rook',5.4,.44,1.1,186,300,5.5,876,1 from dual union all
select 'Rook','Raven','Rook',5.4,.44,1.1,186,300,5.5,876,1 from dual union all
select 'S-160','S-160','S160',3.9,.16,.1,null,null,null,880,1 from dual union all
select 'S-210','S-210','S210',5.1,.21,.26,null,null,17,882,1 from dual union all
select 'S-250','K250','',2,.25,.2,null,null,null,966,1 from dual union all
select 'S-300','S-300','S300',6,.3,.5,null,null,null,885,1 from dual union all
select 'S-310','S-310','S310',5,.31,.6,230,80,31.6,887,1 from dual union all
select 'S-40','S-40','S-40',5.3,1.01,5.6,1300,252,40,889,1 from dual union all
select 'S-40TM','S-40','',5.1,1.01,5.4,1190,202,58,889,1 from dual union all
select 'S-43 VLR-R2','S-40','S-43',8.1,1.01,8.4,1250,327,40,892,1 from dual union all
select 'S-43','S-40','S-43',8.1,1.01,8.4,1250,327,40,892,1 from dual union all
select 'S-43TM','S-40','S-43',8.1,1.01,8.4,1250,327,40,892,1 from dual union all
select 'S-44','S-44','',1.7,1.01,1,127,33,66,893,1 from dual union all
select 'S-44M','S-44','',1.7,1.01,1,127,33,66,893,1 from dual union all
select 'S-520','S-520','S520',5.9,.52,2.3,400,143.1,28.7,895,1 from dual union all
select 'S-A','STA','',2.8,.16,.037,null,2.5,null,896,1 from dual union all
select 'S-B','STA','',2.8,.16,.1,null,null,null,897,1 from dual union all
select 'S-C','STA','',2.8,.16,.1,null,null,null,898,1 from dual union all
select 'S-II Interstage','Interstage','',0,0,0,0,0,0,null,0 from dual union all
select 'S1','Kosmos-2.St2','',7.8,1.65,8.6,840,105,250,765,1 from dual union all
select 'S125','PSLV','PS-1',20,2.8,168,30000,4430,105,901,1 from dual union all
select 'S139','PSLV','PS-1',20,2.8,177,30000,4910,105,902,1 from dual union all
select 'S1M','Kosmos-2.St2','',7.8,1.65,8.6,840,105,250,765,1 from dual union all
select 'S3','Kosmos-3.St2','',6,2.4,20.1,1500,157,335,11,1 from dual union all
select 'S31','Sonda3','',3.2,.56,.9,284,136,11,886,1 from dual union all
select 'S3M','Kosmos-3.St2','',6,2.4,20.1,1435,157,335,11,1 from dual union all
select 'S5M','S5M','',2.8,2.2,4.6,1600,79,125,802,1 from dual union all
select 'S9','ASLV','',10.7,1,11,2900,500,49,930,1 from dual union all
select 'SA-II','STA','',1,.1,0,null,null,null,931,1 from dual union all
select 'SARGE','STIG-B','',11,.51,1,363,24.1,60,933,1 from dual union all
select 'SB-310','SB-310','',5.8,.31,.5,190,95,7.4,934,1 from dual union all
select 'SB-735','SB-735','M-3S SRB',9.1,.74,5.1,1100,328,38,935,1 from dual union all
select 'SDC Viper','Viper','Viper IIB',2.7,.166,.09,20,35.8,3.2,69,1 from dual union all
select 'SEGPRO','ISRO-TV','',2,.25,.3,null,null,null,null,1 from dual union all
select 'SEP 901','P-10','P-10',4.5,1.51,10,null,440,null,940,1 from dual union all
select 'SEP 902','P-16','P-16',6.9,1.51,16,null,540,null,941,1 from dual union all
select 'SEP 903','P-10','P-10',4.7,1.51,10,null,440,null,942,1 from dual union all
select 'SEP 904','P-10','P-10',4.5,1.51,10,null,450,50,943,1 from dual union all
select 'SEPR P167','SEPR167','',1,.1,0,null,null,null,952,1 from dual union all
select 'SEPR-732','Stromboli','',2.5,.56,1.5,null,200,null,955,1 from dual union all
select 'SICBM PBS','SICBM4','',.8,1.17,.5,null,null,null,959,1 from dual union all
select 'SICBM St 2','SICBM2','',3.2,1.17,3.3,null,267,44,957,1 from dual union all
select 'SICBM St 3','SICBM3','',2,1.17,1.5,null,88,null,958,1 from dual union all
select 'SICBM','SICBM','',7.5,1.17,11.6,null,979,40,956,1 from dual union all
select 'SLV-3 Booster','Atlas.Booster','',4.5,3.05,3.2,3200,1495,140,538,1 from dual union all
select 'SLV-3-1','ASLV','',9.8,1,10.8,2140,420,53,960,1 from dual union all
select 'SLV-3-2','ASLV.St2','',6.4,.8,4.9,1750,200,42,961,1 from dual union all
select 'SLV-3-3','ASLV.St3','',2.4,.8,1.5,440,60,46,962,1 from dual union all
select 'SLV-3-4','ASLV.St4','',1.5,.66,.4,98,27,33,963,1 from dual union all
select 'SLV-3C Booster','Atlas.Booster','',4.5,3.05,3.2,3200,1646,140,538,1 from dual union all
select 'SLV-3D Booster','Atlas.Booster','',4.5,3.05,3.2,3200,1557,140,538,1 from dual union all
select 'SM-3-II-2','SM-3-II','',2.5,.53,null,null,null,null,964,1 from dual union all
select 'SM-3-II-3','SM-3-II','',1,.53,null,null,null,null,965,1 from dual union all
select 'SOZ','SOZ','',0,0,0,0,0,0,null,0 from dual union all
select 'SPRAN-50','Daniel','',2.5,.4,.4,null,null,5.6,967,1 from dual union all
select 'SQ-1S','DF-11','',7.4,.88,5.4,1645,93.2,null,266,1 from dual union all
select 'SR118','MX','TU904',8.4,2.35,53.4,4400,2225,78.8,1103,1 from dual union all
select 'SR119','MX.St2','SR119',5.3,2.35,27.7,3175,1223,54,970,1 from dual union all
select 'SR120','MX.St3','SR120',2.3,2.35,7.8,635,289,60.7,971,1 from dual union all
select 'SR19','SR19','',4.1,1.32,7.3,1070,267,65.5,972,1 from dual union all
select 'SR19AJ1','SR19','SR19',4.1,1.32,7.3,800,267,65.5,972,1 from dual union all
select 'SR19AJ1-ATK','SR19','',4.1,1.32,7.3,1070,267,65.5,973,1 from dual union all
select 'SR73AJ1','SR73','SR73',2.3,1.32,3.7,410,152,61,975,1 from dual union all
select 'SR73AJ1-ATK','SR73','SR73',2.3,1.32,3.7,410,152,61,976,1 from dual union all
select 'SRAM','SRAM','',4.3,.46,1,null,null,null,465,1 from dual union all
select 'SRM','SRB','',45.5,3.77,590,88000,11790,123,977,1 from dual union all
select 'SRMU','SRMU','',34.3,3.2,350,37000,7500,137.8,978,1 from dual union all
select 'SS-50','Rigel','',2,.28,.2,null,null,18,980,1 from dual union all
select 'SS-520 St 2','S-520','',1.7,.52,.4,111,null,null,981,1 from dual union all
select 'SS-520 St 3','SS-520-4','',.6,.45,.078,null,null,null,982,1 from dual union all
select 'ST','STA','',1,.1,0,null,null,null,897,1 from dual union all
select 'ST-735 Stage 2','SB-735','',2,.74,4,null,null,null,985,1 from dual union all
select 'ST-735','SB-735','',4.6,.74,7.4,null,250,49,984,1 from dual union all
select 'ST-735-SB','SB-310','',4.7,.38,.3,356,0,0,null,1 from dual union all
select 'STIG-A','STIG','',9.7,.39,.7,null,22.5,50,986,1 from dual union all
select 'STIG-B','STIG-B','',10,.51,.7,null,22.5,50,987,1 from dual union all
select 'Safir Stage 2','Nodong','',3,1.25,9,null,18,60,482,1 from dual union all
select 'Saluki B','Raven','',1,.44,.56,null,195.5,6,990,1 from dual union all
select 'Saluki S','Raven','',1,.44,.44,null,35.6,27,991,1 from dual union all
select 'Sandhawk','Sandhawk','',5.1,.33,.7,200,90,14,1081,1 from dual union all
select 'Saturn S-I','Saturn','',22.5,6.53,432.7,45250,5768,115,330,8 from dual union all
select 'Saturn S-IB','Saturn','',24.4,6.53,448.7,41594,8240,155,330,8 from dual union all
select 'Saturn S-IC','SaturnV','',42.1,10.06,2286,135200,33500,170,299,5 from dual union all
select 'Saturn S-II','SaturnII','S-II',24.8,10.06,490.8,39000,4500,270,374,5 from dual union all
select 'Saturn S-IV','SaturnIV','S-IV',9.8,5.59,50.6,5217,400.2,482,826,6 from dual union all
select 'Saturn S-IVB','SaturnIVB','S-IVB',18.7,6.6,120,13300,890,500,374,1 from dual union all
select 'Saturn S-IVD','SaturnIV','Dummy S-IV',9.8,5.59,50.6,5217,400.2,482,826,6 from dual union all
select 'Saturn S-V','Centaur','S-V',6,3.05,15.6,2000,0,0,826,2 from dual union all
select 'Scud B','R-11','',8.6,.88,5.6,1600,null,null,917,1 from dual union all
select 'Scud D','Nodong','',16,1.25,15.1,2000,255,110,655,1 from dual union all
select 'Scud','R-11','',8,.88,6.3,1600,null,null,917,1 from dual union all
select 'Seagull','','',1.5,.08,.01,1,10,2,992,1 from dual union all
select 'Sejjil Stage 2','Sejjil','',8,1.25,5.5,2000,560,90,994,1 from dual union all
select 'Sejjil','Sejjil','',8,1.25,13.5,2000,560,90,993,1 from dual union all
select 'Seliger 1','Seliger','',3,.6,.5,null,49,null,995,1 from dual union all
select 'Seliger 3','Seliger','',6,.6,1,null,50,null,996,1 from dual union all
select 'Sep Motor Cover','Perekhodnik','',0,0,0,0,0,0,null,0 from dual union all
select 'Sergeant','Sergeant','',4.4,.79,3.2,null,220,23.8,1115,1 from dual union all
select 'Shahab 3','Nodong','',16,1.25,15.1,2000,255,90,655,1 from dual union all
select 'Shaheen 2 St 1','DF-21','CSSNX3',9.5,1.4,20,null,null,65,269,1 from dual;
insert into STAGE
select 'Shaheen 3 St 2','DF-21','',4.4,1.4,4,null,null,null,997,1 from dual union all
select 'Shaheen St 2','DF-21','',4.4,1.4,4,null,null,null,997,1 from dual union all
select 'Shaheen','DF-15','M-9',9.1,1,6.2,null,null,null,267,1 from dual union all
select 'Shaviyt 1','Shaviyt','',7.5,1.35,13,1200,775,55,150,1 from dual union all
select 'Short Al Hussein','R-11','',9,.88,5.9,1600,null,null,917,1 from dual union all
select 'Shourya Booster','K-15','',2.2,.75,3,null,null,null,999,1 from dual union all
select 'Shourya','K-15','',5.6,.75,5,null,null,null,998,1 from dual union all
select 'Shrike','Sparrow','',2,.2,null,null,null,null,608,1 from dual union all
select 'Sidewinder 1A','Sidewinder','',1.9,.13,.04,18,17.7,2.1,604,1 from dual union all
select 'Sidewinder 1A-15','Sidewinder','',1.9,.13,.04,18,17.7,2.1,603,1 from dual union all
select 'Sidewinder 1D','Sidewinder','',1.8,.13,.044,16,12.2,4.7,610,1 from dual union all
select 'Sidewinder','Sidewinder','',1.8,.13,.04,18,null,null,607,1 from dual union all
select 'Silver Sparrow','BlackSparrow','',7,.53,2.5,null,null,null,218,1 from dual union all
select 'Simorgh Stage 1','Unha','',17.8,2.4,60,null,1400,null,null,1 from dual union all
select 'Simorgh Stage 2','Unha.St2','',8.2,1.5,8,null,71,null,482,1 from dual union all
select 'Sirocco','Sirocco','',2.8,.14,.1,20,null,null,1000,1 from dual union all
select 'Skat','LanceMotor','',4.5,.38,.9,377,200,6,1222,1 from dual union all
select 'Skua 2','Skua','Skua',2.4,.13,.04,10,1.8,30,209,1 from dual union all
select 'Skua 3','Skua','Skua',2.4,.13,.08,12,1.7,34,210,1 from dual union all
select 'Skua','Skua','',2.2,.13,.04,10,1.8,30,208,1 from dual union all
select 'Skybolt St1','Skybolt','Skybolt',3.8,.91,2.5,null,164,40.7,1001,1 from dual union all
select 'Skybolt St2','Skybolt.St2','',2.4,.91,1.2,null,81.8,41.3,1002,1 from dual union all
select 'Smart Dispenser','SD','',1,3.35,.2,20,10.8,36,303,1 from dual union all
select 'Snark Booster','Snark','',0,0,0,0,0,0,622,2 from dual union all
select 'Snipe','Gosling','',2.1,.26,.2,58,16,16,1003,1 from dual union all
select 'Sonda 1-1','Sonda','',1.5,.13,.03,15,27,null,1005,1 from dual union all
select 'Sonda 1-2','Sonda','',2.1,.11,.03,14,4.2,32,1004,1 from dual union all
select 'Sonda 2A','Sonda','',4.2,.3,.3,77,36,20,881,1 from dual union all
select 'Sonda 2M','Sonda','',2.2,.3,.2,69,18,20,883,1 from dual union all
select 'Sonda 3-1','Sonda3','',3.3,.56,1.2,341,102,20,884,1 from dual union all
select 'Soyuz-2-1V 132KS','Blok-A','',26.9,2.95,129,9300,1754,292.1,640,1 from dual union all
select 'SpaceLoft XL','SpaceLoft','',4,.27,.2,null,45,13.5,1006,1 from dual union all
select 'Spaceship One','SS1','',5,1.5,4,3000,null,null,null,1 from dual union all
select 'Spaceship Two','SS2','',18.3,2.3,9.7,null,null,null,null,1 from dual union all
select 'Sparrow','Sparrow','Mk 6 Mod 3',1.3,.2,.1,null,35,1.8,3,1 from dual union all
select 'Squirt St 1','Squirt','',2,1.37,1,null,null,1.2,873,7 from dual union all
select 'Squirt St 2','Squirt','',1,1.37,1,null,null,1.5,243,7 from dual union all
select 'Stable Super Loki','SuperLoki','',2.3,.1,.027,6,21,2.1,1009,1 from dual union all
select 'Star 20','Star20','',1.5,.5,.3,26,26.5,27.5,1022,1 from dual union all
select 'Star 26','Star26','',.8,.66,.3,38,33,17.8,1025,1 from dual union all
select 'Star 26B','Star26','',.8,.66,.3,23,35,17.8,1026,1 from dual union all
select 'Star 26C','Star26','',.8,.66,.3,29,35,16.8,1027,1 from dual union all
select 'Star 27','Star27','',1.2,.69,.4,24,26.7,34.3,1028,1 from dual union all
select 'Star 37B','Star37','Star 37',1.5,.94,.7,65,45,null,1036,1 from dual union all
select 'Star 37C','Star37','',1.5,.94,1,83,45,null,1037,1 from dual union all
select 'Star 37D','Star37','',1.5,.94,.7,65,45,null,1038,1 from dual union all
select 'Star 37E','Star37','',1.7,.94,1.1,83,68,42,1039,1 from dual union all
select 'Star 37F','Star37','',1.7,.94,.9,67,55.5,42,1040,1 from dual union all
select 'Star 37FM','Star37','',1.7,.94,1.1,74,45.8,66.4,1041,1 from dual union all
select 'Star 37FMV','Star37','',1.7,.94,1.1,74,45.8,66.4,1041,1 from dual union all
select 'Star 37N','Star37','Star 37E',1.7,.94,.6,64,45,null,1043,1 from dual union all
select 'Star 37S','Star37','',1.7,.94,.7,53,45,null,1044,1 from dual union all
select 'Star 37XE','Star37','',1.7,.94,.9,67,45,66,1045,1 from dual union all
select 'Star 37XF','Star37','',1.7,.94,.9,67,45,null,1046,1 from dual union all
select 'Star 37XFP','Star37','',1.7,.94,1,72,38,66,1047,1 from dual union all
select 'Star 48','Star48','Star 48',2,1.22,2.2,208,66.4,84.1,1048,1 from dual union all
select 'Star 48B AKM','Star48','Star 48',2,1.22,2.2,116,66.4,84.1,1050,1 from dual union all
select 'Star 48B','Star48','Star 48',2,1.22,2.2,208,66.4,84.1,1050,1 from dual union all
select 'Star 48BV','Star48','',2,1.22,2.2,140,68.7,84.1,1052,1 from dual union all
select 'Star 48V','Star48','',2,1.22,2.2,140,68.7,84.1,1052,1 from dual union all
select 'Start','Start','',2.6,1,1,300,100,53,1055,1 from dual union all
select 'Stonechat I','Stonechat','',4.5,.92,4.8,750,232,32.3,1057,1 from dual union all
select 'Stonechat II','Stonechat','',4.5,.92,5.1,750,242,32.8,1058,1 from dual union all
select 'Stratos II+','Stratos2','',7,.2,.19,100,8,23,277,1 from dual union all
select 'Strela APB','UR-100NU.St3','',1.3,2.4,null,null,null,null,null,1 from dual union all
select 'Stromboli','Stromboli','',3,.56,.9,246,75,16,1059,1 from dual union all
select 'Super Arcas','Arcas','',2,.11,.04,15,1.5,40,551,1 from dual union all
select 'Super Loki','SuperLoki','',2,.1,.03,6,18,2.1,1062,1 from dual union all
select 'Sword','Asp','',1.2,.166,.1,12,16,3.8,1063,1 from dual union all
select 'T-40','T-40','T40',1.4,.2,.1,15,13.3,6,1065,1 from dual union all
select 'T-55','T-55','T55',1.5,.15,0,12,21,1.3,1067,1 from dual union all
select 'T-7 booster','T-7B','',1,.45,.5,null,2,null,1069,1 from dual union all
select 'T-7','T-7','',6,.45,1,null,2,null,1068,1 from dual union all
select 'T48','T48','',4,.46,1,null,193,3.4,1070,1 from dual union all
select 'THAAD Proto','THAAD','',3.8,.34,.6,null,null,null,1083,1 from dual union all
select 'THAAD','THAAD','',3.8,.34,.6,null,null,null,1083,1 from dual union all
select 'TR-1','TR-1','',7.1,1.13,10.8,3900,608,51.3,1086,1 from dual union all
select 'TR-1A','TR-1','TR-1A',7.5,1.13,8.8,1784,608,52,1087,1 from dual union all
select 'TT-200','TT-210','',5,.2,.2,null,null,null,1092,1 from dual union all
select 'TT-210','TT-210','',5.3,.21,.2,null,null,null,1093,1 from dual union all
select 'TT-500','JCR','',7,.5,1.3,300,116,null,1094,1 from dual union all
select 'TT-500-S2','JCR','',3,.5,.7,200,61,null,1095,1 from dual union all
select 'TX-135','Spartan','',5.2,1.09,5.3,932,2000,4.5,1105,1 from dual union all
select 'TX-174','Pershing','',2.8,1.02,2.3,251,117,39,1107,1 from dual union all
select 'TX-175','Pershing','',2.7,1.02,1.5,210,85,39,1108,1 from dual union all
select 'TX-20 Sergeant','Sergeant','',6.1,.79,4,null,213,24,1109,1 from dual union all
select 'TX-238','TX-238','',4.7,.91,3.5,417,null,null,1110,1 from dual union all
select 'TX-239','TX-239','',1.1,.74,.37,null,null,null,1111,1 from dual union all
select 'TX-261','Pedro','',3.1,.71,1.5,250,244,11.3,1114,1 from dual union all
select 'TX-261-2','Pedro','',3.1,.71,1.5,250,244,11.3,1113,1 from dual union all
select 'TX-30','Sergeant','',5.4,.79,2,null,44.5,29,1115,1 from dual union all
select 'TX-454','Spartan','',5,1.09,6.5,753,null,null,1119,1 from dual union all
select 'TX-500','Spartan','Spartan TX-500',4.9,1.09,5.5,1030,2000,null,1121,1 from dual union all
select 'Tacite','Stromboli','',4.7,.56,1.7,455,170,20,948,1 from dual union all
select 'Taepo Dong 1 Stage 2','R-11','',12,.88,7.5,1500,140,176,655,1 from dual union all
select 'Taepo Dong 1 Stage 3','TD3','',1.8,.3,.2,50,18,27,1075,1 from dual union all
select 'Taepo Dong 1','Nodong','',12,1.25,25.6,3700,570,95,655,1 from dual union all
select 'Taepo Dong 2','Unha','',14,2.4,60,5000,1200,120,1072,4 from dual union all
select 'Talos','Talos','',3.4,.76,1.5,null,516,5.2,1130,1 from dual union all
select 'Taurion','HJ','',4.3,.58,1.4,null,13,32,520,1 from dual;
insert into STAGE
select 'Taurus','HJ','',4.3,.58,1.4,null,457,3.5,1131,1 from dual union all
select 'Temp','Temp','',12.4,1.01,11.9,120,null,null,104,1 from dual union all
select 'Temp-S','Temp','',12.4,1.01,11.9,120,null,null,105,1 from dual union all
select 'Terrier 12-0','Terrier','',3.9,.46,.8,322,257.5,4.4,1132,1 from dual union all
select 'Terrier Mk 12','Terrier','',3.9,.46,.9,322,257.5,4.4,1133,1 from dual union all
select 'Terrier Mk 70','Terrier','',3.9,.46,1,290,null,6.2,1124,1 from dual union all
select 'Terrier','Terrier','ABL X256',3.9,.46,.9,322,257.5,4.4,1196,1 from dual union all
select 'Theodore','Monica','',.8,.13,.01,null,1.8,10,1134,1 from dual union all
select 'Thor DM-18','Thor','',18.2,2.44,48.8,3125,667,160,553,1 from dual union all
select 'Thor DM-18A','Thor','',18.6,2.44,48.8,3125,667,160,553,1 from dual union all
select 'Thor DM-18C','Thor','',18.6,2.44,48.8,3125,734,170,555,1 from dual union all
select 'Thor DM-21','Thor','',17.1,2.44,47.8,3125,779,170,555,1 from dual union all
select 'Thor DSV-2A','Thor','',17.1,2.44,47.8,3125,779,170,555,1 from dual union all
select 'Thor DSV-2C','Thor','TA Thor DSV-2C',17.1,2.44,48,3175,760,170,555,1 from dual union all
select 'Tianying-3C','Tianying','',5,.45,1.1,0,0,null,1135,1 from dual union all
select 'Tianying-3F Booster','Tianying','',2.7,.45,.5,0,0,null,1136,1 from dual union all
select 'Tiny Tim','TTB','',2.4,.3,.3,278,222,.6,1137,1 from dual union all
select 'Tochka','Tochka','',6.4,.65,2,null,null,null,106,1 from dual union all
select 'Tochka-U','Tochka','',6.4,.65,2,null,null,null,107,1 from dual union all
select 'Tomahawk 1','Tomahawk','',3.5,.22,.2,60,53,9,1079,1 from dual union all
select 'Tomahawk','Tomahawk','',3.5,.22,.2,60,53,9.5,1080,1 from dual union all
select 'Topaze VE111C','Agate','',2.9,.8,1.5,null,120,39,624,1 from dual union all
select 'Topaze VE111L','Agate','',3,.8,2.2,null,156,44,625,1 from dual union all
select 'Topaze','Agate','',4.6,.8,2.2,null,156,44,625,1 from dual union all
select 'Tramontane','Berenice','',2.4,.33,.3,124,50,6.7,954,1 from dual union all
select 'Transtage','Transtage','',4.6,3.05,12.5,2000,71.1,440,128,2 from dual union all
select 'Trident C-4 Stage 1','Trident','',4.4,1.88,12.1,null,null,null,223,1 from dual union all
select 'Trident C-4 Stage 2','Trident.St2','',2.5,1.88,8,null,null,null,224,1 from dual union all
select 'Trident C-4 Stage 3','Trident.St3','',3.3,.86,1.9,150,null,40,1140,1 from dual union all
select 'Trident D-5 Stage 1','Trident','',7.2,2.11,39.1,2242,null,64,261,1 from dual union all
select 'Trident D-5 Stage 2','Trident.St2','',2.9,2.11,11.8,790,null,65,262,2 from dual union all
select 'Trident D-5 Stage 3','Trident.St3','',3.3,.86,2.2,159,null,40,1140,1 from dual union all
select 'Trident','Trident.St3','',2,2.13,1.1,700,35,null,1139,1 from dual union all
select 'UA1205','Titan.SRM','SRM',25.9,3.11,226.2,33800,6227.2,115,1141,1 from dual union all
select 'UA1206','Titan.SRM','SRM',27.6,3.11,251.2,40800,6227.2,114,1142,1 from dual union all
select 'UA1207','Titan.SRM','SRM1207',34.1,3.11,319.3,51200,7120,120,1143,1 from dual union all
select 'UM129A','UM129A','',2.3,1.32,2.2,360,77.5,68,1144,1 from dual union all
select 'UR-100NA','UR-100N','',17.2,2.5,88,5000,2070,121,741,4 from dual union all
select 'UR-100NB','UR-100N.St2','',3.9,2.5,15,2000,240,183,742,1 from dual union all
select 'UR-100NU APB','UR-100NU.St3','',.5,2.4,null,null,null,null,null,1 from dual union all
select 'Unha-1 Stage 2','Unha.St2','',8,1.5,25.6,3700,570,95,655,1 from dual union all
select 'Unha-2 Stage 2','Unha.St2','',8,1.5,7.5,1500,140,176,655,1 from dual union all
select 'Unha-2 Stage 3','Unha.St3','',3,1.25,3,300,18,27,1146,1 from dual union all
select 'Unha-3 Stage 3','Unha.St3','',3.8,1.25,4,500,18,27,1146,1 from dual union all
select 'Upstage','HIBEX','',1,.5,.2,null,null,null,1147,1 from dual union all
select 'Ute','Genie','',1.3,.38,.2,62,85,4.6,1100,1 from dual union all
select 'V-1000 Booster','V-1000.Booster','',3.1,1,3,null,1962,4.5,690,1 from dual union all
select 'V-1000','V-1000','',8,.55,null,null,103,null,912,1 from dual union all
select 'V-2','V-2','',12,1.65,12.8,4000,312,70,110,1 from dual union all
select 'V-2-Ram','V-2','',12,1.65,12.8,4000,312,70,110,1 from dual union all
select 'V-750 Booster','V-750','',2.4,.65,.3,null,null,4.5,689,1 from dual union all
select 'V-750','V-750','',8.1,.5,2,null,13.8,25,907,1 from dual union all
select 'V-825 Booster','V-825','5Ya27',10,1.3,14,null,null,null,89,1 from dual union all
select 'V-825','V-825','5Ya27',9,1.3,3.9,null,null,null,1150,1 from dual union all
select 'VAM Booster','AerobeeB','',1.8,.33,.3,140,80,2.5,57,1 from dual union all
select 'VAM-17','AerobeeB','',1.8,.33,.3,140,80,2.5,57,1 from dual union all
select 'VAM-20','AerobeeB','',1.8,.33,.3,140,80,2.5,57,1 from dual union all
select 'VLS-R1 Strapon','Sonda','',2,.3,.5,0,18,20,881,1 from dual union all
select 'VLS-R1','Sonda','',2.5,.3,.5,0,18,20,881,1 from dual union all
select 'VLS-R2 Strapon','Sonda','',2,.3,.5,0,18,20,881,1 from dual union all
select 'VLS-R2','Sonda','',3,.3,.5,0,18,20,881,1 from dual union all
select 'Vanguard','Viking','',11.4,1.14,7.7,811,120,144,1204,1 from dual union all
select 'Vega P80FW','Vega','',11.2,3.01,97.1,8790,2100,106.8,684,1 from dual union all
select 'Vela','Vela','',3.1,.21,.12,59,26.7,5,1153,1 from dual union all
select 'Venus','Venus','',2,.28,.2,66,44,4.3,1154,1 from dual union all
select 'Veronique 61','Veronique','Veronique',9.5,.55,1.9,null,60,54,1158,1 from dual union all
select 'Veronique 61M','Veronique','Veronique',11.7,.55,2,null,60,56,1158,1 from dual union all
select 'Veronique AGI','Veronique','Veronique',7.3,.55,1.3,278,40,49,1159,1 from dual union all
select 'Veronique P2','Veronique','',6,.55,.5,null,10,null,1155,1 from dual union all
select 'Veronique P6','Veronique','',6,.55,.5,null,20,null,1156,1 from dual union all
select 'Veronique R','Veronique','',5.8,.55,.5,385,40,6.5,1157,1 from dual union all
select 'Veronique','Veronique','',7.3,.55,1.3,278,40,45,1157,1 from dual union all
select 'Vesta','Vesta','',7.2,1,5,780,140,56.4,1161,1 from dual union all
select 'Vesuve','Stromboli','',3.5,.56,.8,250,200,5,945,1 from dual union all
select 'Viking Type 9','Viking','Viking',12.8,1.14,6.8,1000,93,105,650,1 from dual union all
select 'Viking','Viking','',14.9,.81,4.5,800,93,70,650,1 from dual union all
select 'Viper I','Viper','Viper',2.7,.166,.09,25,24,5.6,82,1 from dual union all
select 'Viper IC','Viper','Viper',2.7,.166,.09,25,24,5.6,82,1 from dual union all
select 'Viper II','Viper','Viper',2.7,.166,.09,25,35.8,3.8,70,1 from dual union all
select 'Viper IIIA','Viper3','',2.4,.11,.04,9,24.7,2.2,1173,1 from dual union all
select 'Viper V','Viper5','Viper',2.2,.18,.1,null,27,5.2,1174,1 from dual union all
select 'Volga','Ikar','',1,2.7,1.8,900,2.9,882,1175,1 from dual union all
select 'WAC Corporal','WAC','',4.9,.3,.3,125,6.7,45,1178,1 from dual union all
select 'Wasserfall','Hermes','',7.8,.88,3.5,1700,80,40,1179,1 from dual union all
select 'Waxwing','Waxwing','',1.5,.71,.3,39,28.8,37.2,1180,1 from dual union all
select 'White Knight 2','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'White Knight','Air','',0,0,0,0,0,0,null,0 from dual union all
select 'X-10','X-10','',21.3,null,null,null,0,0,null,0 from dual union all
select 'X-15','X-15','',15.3,3.55,14.2,5150,257,100,1210,1 from dual union all
select 'X-201','Nike','',3,.42,.7,null,209,3,67,1 from dual union all
select 'X-241','X-241','ABL X241',1.5,.46,.2,25,12.1,36,1185,1 from dual union all
select 'X-248','Altair','',1.8,.46,.3,24,13.4,38.6,1186,1 from dual union all
select 'X-260','PolarisA3.St2','',2.3,1.37,4.9,815,null,null,1201,1 from dual union all
select 'X-265','Sprint','Sprint X265',3,1.37,4,null,null,1.2,1202,1 from dual union all
select 'X-271','X-271','Sprint X271',5,.82,null,null,null,null,1203,1 from dual union all
select 'X-51A','X-51','',4.5,.61,0,0,0,0,null,0 from dual union all
select 'X-7','X-7','',0,0,0,0,0,0,null,0 from dual union all
select 'X-7A-1 Booster','HJ','',6,.58,1.9,1474,467,4,1181,1 from dual union all
select 'Xaero','Xaero','',4.6,.77,1,null,13.2,null,1226,1 from dual union all
select 'YZ-1','YZ-1','YZ-1',5,3,0,0,0,0,1244,2 from dual;
insert into STAGE
select 'YZ-1A','YZ-1','YZ-1',5,3,0,0,0,0,1244,2 from dual union all
select 'YZ-2','YZ-1','YZ-2',5,3,0,0,0,0,1244,2 from dual union all
select 'Yardbird','Recruit','',2.5,.23,.2,29,75,3.4,1250,1 from dual union all
select 'ZN-1','ZN','',2.7,.15,.1,null,null,null,1251,1 from dual union all
select 'ZUR-215','V-750','',8.1,.5,2,null,13.8,25,907,1 from dual union all
select 'Zefiro 23','Zefiro','',8.5,1.91,26.5,2653,871,77,1253,1 from dual union all
select 'Zefiro 9A','Zefiro','',4.4,1.91,11.5,1386,260,120,1254,1 from dual union all
select 'Zefiro','Zefiro','',4.9,1.14,17.3,1500,590,71,1252,1 from dual union all
select 'Zelzal','Zelzal','Iran',6,.61,.5,0,0,null,649,1 from dual union all
select 'Zenit','Zenith','',5.6,.42,.6,150,45,21,1256,1 from dual union all
select 'Zeus A','Zeus-A','',7,.5,2,null,null,null,1257,1 from dual union all
select 'Zulfiqar','Zelzal','Iran',6,.61,.5,0,0,null,649,1 from dual;


--------------------------------------------------------------------------------
-- LAUNCH_VEHICLE_STAGE
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing LAUNCH_VEHICLE_STAGE...');

CREATE TABLE "LAUNCH_VEHICLE_STAGE" 
 (	"LV_ID" NUMBER, 
	"STAGE_NAME" VARCHAR2(21), 
	"STAGE_NO" VARCHAR2(3), 
	"IS_DUMMY" NUMBER, 
	"MULTIPLICITY" NUMBER, 
	"STAGE_IMPULSE" NUMBER, 
	"STAGE_APOGEE" NUMBER, 
	"STAGE_PERIGEE" NUMBER, 
	 CONSTRAINT "LAUNCH_VEHICLE_STAGE_PK" PRIMARY KEY ("LV_ID", "STAGE_NAME", "STAGE_NO")
USING INDEX  ENABLE, 
	 CONSTRAINT "LAUNCH_VEHICLE_STAGE_FK1" FOREIGN KEY ("LV_ID")
	  REFERENCES "LAUNCH_VEHICLE" ("LV_ID") ENABLE, 
	 CONSTRAINT "LAUNCH_VEHICLE_STAGE_FK2" FOREIGN KEY ("STAGE_NAME")
	  REFERENCES "STAGE" ("STAGE_NAME") ENABLE
 ) ;


CREATE INDEX "LAUNCH_VEHICLE_STAGE_IDX1" ON "LAUNCH_VEHICLE_STAGE" ("LV_ID") 
;

CREATE INDEX "LAUNCH_VEHICLE_STAGE_IDX2" ON "LAUNCH_VEHICLE_STAGE" ("STAGE_NAME") 
;
insert into LAUNCH_VEHICLE_STAGE
select 1,'V-1000 Booster','0',0,1,null,2,-6378 from dual union all
select 1,'V-1000','1',0,1,null,25,-6300 from dual union all
select 10,'A-925 Booster','1',0,1,null,10,-6370 from dual union all
select 10,'A-925','2',0,1,null,100,-6300 from dual union all
select 100,'Ariane H10','3',0,1,48906,35787,200 from dual union all
select 100,'Ariane L220','1',0,1,621970,80,-5800 from dual union all
select 100,'Ariane L33','2',0,1,98910,180,-3700 from dual union all
select 100,'PAP','0',0,2,54600,5,-6300 from dual union all
select 1000,'8S671','1',0,1,null,100,-6100 from dual union all
select 1000,'8S672','2',0,1,null,1000,-4000 from dual union all
select 1001,'8S671','1',0,1,null,100,-6100 from dual union all
select 1001,'8S672','2',0,1,null,1000,-4000 from dual union all
select 1002,'8S671','1',0,1,null,100,-6100 from dual union all
select 1002,'8S672','2',0,1,null,1000,-4000 from dual union all
select 1003,'8S671','1',0,1,null,100,-6100 from dual union all
select 1003,'8S672','2',0,1,null,1000,-4000 from dual union all
select 1004,'15A14A','1',0,1,null,100,-6100 from dual union all
select 1004,'15A14B','2',0,1,null,1000,-4000 from dual union all
select 1005,'15A18-BR','3',0,1,null,1000,-4000 from dual union all
select 1005,'15A18A','1',0,1,null,100,-6100 from dual union all
select 1005,'15A18B','2',0,1,null,1000,-4000 from dual union all
select 1006,'15A18-BR','3',0,1,null,1000,-4000 from dual union all
select 1006,'15A18A','1',0,1,null,100,-6100 from dual union all
select 1006,'15A18B','2',0,1,null,1000,-4000 from dual union all
select 1007,'8S691','1',0,1,null,100,-6100 from dual union all
select 1007,'8S692','2',0,1,null,200,200 from dual union all
select 1008,'8S691','1',0,1,null,100,-6100 from dual union all
select 1008,'8S692','2',0,1,null,200,200 from dual union all
select 1009,'R-5','1',0,1,null,500,-6200 from dual union all
select 101,'Ariane H10+','3',0,1,47025,35787,200 from dual union all
select 101,'Ariane L220','1',0,1,621970,80,-5800 from dual union all
select 101,'Ariane L33','2',0,1,98910,180,-3700 from dual union all
select 101,'PAP','0',0,2,54600,5,-6300 from dual union all
select 1010,'R-5','1',0,1,null,500,-6200 from dual union all
select 1011,'R-5','1',0,1,null,500,-6200 from dual union all
select 1012,'R-5','1',0,1,null,500,-6200 from dual union all
select 1013,'R-5','1',0,1,null,500,-6200 from dual union all
select 1014,'R-5','1',0,1,null,500,-6200 from dual union all
select 1015,'8K71','1',0,4,null,75,-5900 from dual union all
select 1015,'8K71A','2',0,1,null,1350,-4000 from dual union all
select 1016,'8K71','1',0,4,null,75,-5900 from dual union all
select 1016,'8K71A','2',0,1,null,1350,-4000 from dual union all
select 1017,'8K74','1',0,4,null,75,-5900 from dual union all
select 1017,'8K74A','2',0,1,null,1350,-4000 from dual union all
select 1018,'8K75A','1',0,1,null,100,-6100 from dual union all
select 1018,'8K75B','2',0,1,null,1000,-4000 from dual union all
select 1019,'R-UNK','1',0,1,null,400,-6378 from dual union all
select 102,'Ariane H10-3','3',0,1,48906,35787,200 from dual union all
select 102,'Ariane L220','1',0,1,621970,80,-5800 from dual union all
select 102,'Ariane L33','2',0,1,98910,200,-3700 from dual union all
select 102,'PAP','0',0,2,54600,5,-6300 from dual union all
select 1020,'Castor','1',0,1,null,50,-6300 from dual union all
select 1020,'Recruit','0',0,2,null,1,-6378 from dual union all
select 1020,'Recruit','4',0,1,null,1280,-5340 from dual union all
select 1020,'Skat','2',0,1,null,120,-6300 from dual union all
select 1020,'Skat','3',0,1,null,200,-6200 from dual union all
select 1021,'Alcor','3',0,1,null,88,-4200 from dual union all
select 1021,'Antares','2',0,1,null,50,-5900 from dual union all
select 1021,'Castor','1',0,1,null,25,-6300 from dual union all
select 1022,'RH-125','1',0,1,null,20,-6378 from dual union all
select 1023,'RH-125','2',0,1,null,70,-6370 from dual union all
select 1023,'RH-200','1',0,1,null,40,-6370 from dual union all
select 1024,'RH-200SV','1',0,1,null,40,-6370 from dual union all
select 1025,'RH-300','1',0,1,null,100,-6370 from dual union all
select 1026,'RH-300 Mk II','1',0,1,null,135,-6370 from dual union all
select 1027,'RH-300 Booster','1',0,1,null,10,-6300 from dual union all
select 1027,'RH-300','2',0,1,null,150,-6300 from dual union all
select 1028,'Belier','2',0,1,null,320,-6300 from dual union all
select 1028,'RH-560','1',0,1,null,60,-6370 from dual union all
select 1029,'RH-300 Mk II','2',0,1,null,420,-6300 from dual union all
select 1029,'RH-560','1',0,1,null,60,-6370 from dual union all
select 103,'Ariane H10','3',0,1,45144,35787,200 from dual union all
select 103,'Ariane L220','1',0,1,621970,90,-5720 from dual union all
select 103,'Ariane L33','2',0,1,98910,180,-3700 from dual union all
select 103,'PAL','0',0,4,424760,50,-6170 from dual union all
select 1030,'LPR-3T','2',0,1,null,20,-6378 from dual union all
select 1030,'RH-560','1',0,1,null,20,-6378 from dual union all
select 1031,'RH-560S','1',0,1,null,100,-6370 from dual union all
select 1032,'RH-75','1',0,1,null,10,-6378 from dual union all
select 1033,'RSA-3-1','1',0,1,null,100,-6300 from dual union all
select 1033,'RSA-3-2','2',1,1,null,100,-6300 from dual union all
select 1033,'RSA-3-3','3',1,1,null,100,-6300 from dual union all
select 1034,'RSA-3-1','1',0,1,null,40,-6370 from dual union all
select 1034,'RSA-3-2','2',0,1,null,100,-6300 from dual union all
select 1034,'RSA-3-3','3',1,1,null,300,-6300 from dual union all
select 1035,'8K95A','1',0,1,null,40,-6370 from dual union all
select 1035,'8K95B','2',0,1,null,100,-6300 from dual union all
select 1035,'8K95V','3',0,1,null,500,-6000 from dual union all
select 1036,'8K96A','1',0,1,null,100,-6300 from dual union all
select 1036,'8K96B','2',0,1,null,1000,-5000 from dual union all
select 1037,'8K98A','1',0,1,null,40,-6370 from dual union all
select 1037,'8K98B','2',0,1,null,200,-6000 from dual union all
select 1037,'8K98V','3',0,1,null,1000,-4000 from dual union all
select 1038,'8K94','2',0,1,null,1000,-4000 from dual union all
select 1038,'8K99A','1',0,1,null,100,-6100 from dual union all
select 1039,'15Zh44 St 1','1',0,1,null,50,-6370 from dual union all
select 1039,'15Zh44 St 2','2',0,1,null,200,-6000 from dual union all
select 1039,'RT-23 St 3','3',0,1,null,1000,-4000 from dual union all
select 104,'Ariane H10+','3',0,1,47025,35787,200 from dual union all
select 104,'Ariane L220','1',0,1,621970,100,-5720 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 104,'Ariane L33','2',0,1,98910,200,-3700 from dual union all
select 104,'PAL','0',0,4,424760,50,-6170 from dual union all
select 1040,'15Zh52 St 1','1',0,1,null,50,-6370 from dual union all
select 1040,'15Zh52 St 2','2',0,1,null,200,-6000 from dual union all
select 1040,'RT-23 St 3','3',0,1,null,1000,-4000 from dual union all
select 1041,'15Zh60 St 1','1',0,1,null,50,-6370 from dual union all
select 1041,'15Zh60 St 2','2',0,1,null,200,-6000 from dual union all
select 1041,'RT-23 St 3','3',0,1,null,1000,-4000 from dual union all
select 1042,'15Zh61 St 1','1',0,1,null,50,-6370 from dual union all
select 1042,'15Zh61 St 2','2',0,1,null,200,-6000 from dual union all
select 1042,'RT-23 St 3','3',0,1,null,1000,-4000 from dual union all
select 1043,'8K96A','2',0,1,null,200,-6000 from dual union all
select 1043,'8K96B','3',0,1,null,1000,-4000 from dual union all
select 1043,'8K98PA','1',0,1,null,50,-6370 from dual union all
select 1044,'8K96A','2',0,1,null,200,-6000 from dual union all
select 1044,'8K96B','3',0,1,null,1000,-4000 from dual union all
select 1044,'8K98PA','1',0,1,null,50,-6370 from dual union all
select 1045,'MX-774','1',0,1,null,50,-6370 from dual union all
select 1046,'RV-A-10','1',0,1,null,50,-6370 from dual union all
select 1047,'RX-250-LPN','1',0,1,null,70,-6370 from dual union all
select 1048,'RX-320','1',0,1,null,70,-6370 from dual union all
select 1049,'RX-420','1',0,1,null,70,-6370 from dual union all
select 105,'Ariane H10-3','3',0,1,48906,35787,200 from dual union all
select 105,'Ariane L220','1',0,1,621970,100,-5720 from dual union all
select 105,'Ariane L33','2',0,1,98910,200,-3700 from dual union all
select 105,'PAL','0',0,4,424760,50,-6170 from dual union all
select 1050,'Hopi IV','1',0,1,null,60,-6370 from dual union all
select 1051,'Recruit','1',0,1,null,4,-6378 from dual union all
select 1052,'B-57A Canberra','0',0,1,null,14,-6378 from dual union all
select 1052,'Recruit','1',0,1,null,14,-6378 from dual union all
select 1052,'T-55','2',0,1,null,14,-6378 from dual union all
select 1053,'Redstone','1',0,1,null,90,-6230 from dual union all
select 1054,'Redstone MRLV','1',0,1,null,200,-6200 from dual union all
select 1055,'Regulus','1',0,1,null,20,-6378 from dual union all
select 1056,'Regulus II','1',0,1,null,20,-6378 from dual union all
select 1057,'R-39 St 1','1',0,1,null,40,-6370 from dual union all
select 1057,'R-39 St 2','2',0,1,null,200,-6000 from dual union all
select 1057,'R-39 St 3','3',0,1,null,1000,-4000 from dual union all
select 1058,'Canopus','1',0,1,null,70,-6300 from dual union all
select 1058,'Orion-2','2',0,1,null,250,-6200 from dual union all
select 1059,'F2H2 Banshee','0',0,1,null,9,-6378 from dual union all
select 1059,'FFAR','1',0,1,null,55,-6370 from dual union all
select 106,'Ariane H10','3',0,1,45144,35787,200 from dual union all
select 106,'Ariane L220','1',0,1,621970,90,-5800 from dual union all
select 106,'Ariane L33','2',0,1,98910,180,-3800 from dual union all
select 106,'PAL','-1',0,2,212380,50,-6200 from dual union all
select 106,'PAP','0',0,2,54600,5,-6370 from dual union all
select 1060,'Deacon','1',0,1,null,40,-6370 from dual union all
select 1060,'F86D Sabrejet','0',0,1,null,9,-6378 from dual union all
select 1061,'Rocketsonde','1',0,1,null,60,-6370 from dual union all
select 1062,'Rocketsonde2','1',0,1,null,60,-6370 from dual union all
select 1063,'Briz-K','3',0,1,null,700,200 from dual union all
select 1063,'UR-100NA','1',0,1,null,130,-5900 from dual union all
select 1063,'UR-100NB','2',0,1,null,240,-4020 from dual union all
select 1064,'Briz-KM','3',0,1,null,700,200 from dual union all
select 1064,'UR-100NA','1',0,1,null,130,-5900 from dual union all
select 1064,'UR-100NB','2',0,1,null,240,-4020 from dual union all
select 1065,'Rook II','1',0,1,null,20,-6378 from dual union all
select 1066,'Rook IIIA','1',0,1,null,20,-6378 from dual union all
select 1067,'15Zh58MA','1',0,1,null,70,-6300 from dual union all
select 1067,'15Zh58MB','2',0,1,null,200,-6000 from dual union all
select 1068,'Agate','1',0,1,null,30,-6200 from dual union all
select 1068,'P064','2',0,1,null,2000,-5900 from dual union all
select 1069,'S-160','1',0,1,null,90,-6370 from dual union all
select 107,'Ariane H10+','3',0,1,47025,35787,200 from dual union all
select 107,'Ariane L220','1',0,1,621970,100,-5700 from dual union all
select 107,'Ariane L33','2',0,1,98910,200,-3700 from dual union all
select 107,'PAL','-1',0,2,212380,50,-6200 from dual union all
select 107,'PAP','0',0,2,54600,5,-6370 from dual union all
select 1070,'S-210','1',0,1,null,100,-6370 from dual union all
select 1071,'S-250','1',0,1,null,100,-6370 from dual union all
select 1072,'S-300','1',0,1,null,160,-6370 from dual union all
select 1073,'S-310','1',0,1,null,200,-6370 from dual union all
select 1074,'S-520','1',0,1,null,300,-6200 from dual union all
select 1075,'S-A','1',0,1,null,50,-6378 from dual union all
select 1076,'S-B','1',0,1,null,75,-6370 from dual union all
select 1077,'S-C','1',0,1,null,85,-6370 from dual union all
select 1078,'SA-II','1',0,1,null,20,-6378 from dual union all
select 1079,'SARGE','1',0,1,null,100,-6378 from dual union all
select 108,'Ariane H10-3','3',0,1,48906,35787,200 from dual union all
select 108,'Ariane L220','1',0,1,621970,100,-5700 from dual union all
select 108,'Ariane L33','2',0,1,98910,200,-3700 from dual union all
select 108,'PAL','-1',0,2,212380,50,-6200 from dual union all
select 108,'PAP','0',0,2,54600,5,-6370 from dual union all
select 1080,'SEPR-732','1',0,1,null,60,-6378 from dual union all
select 1081,'SEGPRO','1',0,1,null,20,-6378 from dual union all
select 1082,'SICBM PBS','4',0,1,null,1000,-4000 from dual union all
select 1082,'SICBM St 2','2',0,1,null,300,-5500 from dual union all
select 1082,'SICBM St 3','3',0,1,null,1000,-4000 from dual union all
select 1082,'SICBM','1',0,1,null,70,-6000 from dual union all
select 1083,'AS-4','4',0,1,null,850,390 from dual union all
select 1083,'SLV-3-1','1',0,1,null,40,-6300 from dual union all
select 1083,'SLV-3-2','2',0,1,null,100,-6000 from dual union all
select 1083,'SLV-3-3','3',0,1,null,500,-4150 from dual union all
select 1084,'SLV-3-1','1',0,1,null,40,-6300 from dual union all
select 1084,'SLV-3-2','2',0,1,null,100,-6000 from dual union all
select 1084,'SLV-3-3','3',0,1,null,330,-4700 from dual union all
select 1084,'SLV-3-4','4',0,1,null,300,200 from dual union all
select 1085,'Mk 30','2',0,1,null,20,-6378 from dual union all
select 1085,'Terrier Mk 12','1',0,1,null,10,-6378 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 1086,'Mk 56','1',0,1,null,20,-6378 from dual union all
select 1087,'Mk 30','2',0,1,null,20,-6370 from dual union all
select 1087,'Terrier Mk 70','1',0,1,null,10,-6378 from dual union all
select 1088,'Mk 104','2',0,1,null,50,-6350 from dual union all
select 1088,'Mk 72','1',0,1,null,10,-6370 from dual union all
select 1089,'Mk 104','1',0,1,null,20,-6370 from dual union all
select 109,'Ariane H10','3',0,1,45144,35787,200 from dual union all
select 109,'Ariane L220','1',0,1,621970,100,-5700 from dual union all
select 109,'Ariane L33','2',0,1,98910,200,-3700 from dual union all
select 109,'PAP','0',0,4,102900,5,-6370 from dual union all
select 1090,'ASAS','3',0,1,null,150,-6300 from dual union all
select 1090,'Mk 104','2',0,1,null,50,-6350 from dual union all
select 1090,'Mk 72','1',0,1,null,10,-6370 from dual union all
select 1091,'ASAS XL','3',0,1,null,150,-6300 from dual union all
select 1091,'Mk 104','2',0,1,null,50,-6350 from dual union all
select 1091,'Mk 72','1',0,1,null,10,-6370 from dual union all
select 1092,'Mk 72','1',0,1,null,10,-6370 from dual union all
select 1092,'SM-3-II-2','2',0,1,null,50,-6350 from dual union all
select 1092,'SM-3-II-3','3',0,1,null,150,-6300 from dual union all
select 1093,'Mk 104','2',0,1,null,50,-6350 from dual union all
select 1093,'Mk 72','1',0,1,null,10,-6370 from dual union all
select 1094,'RH-200','0',0,2,null,10,-6370 from dual union all
select 1094,'RH-300','1',0,1,null,50,-6370 from dual union all
select 1095,'Antares 2','2',0,1,null,190,-5500 from dual union all
select 1095,'BE-3A5','3',0,1,null,190,-5500 from dual union all
select 1095,'Redstone','1',0,1,null,90,-6100 from dual union all
select 1096,'SR19AJ1','1',0,1,null,100,-6300 from dual union all
select 1097,'MLRS','0',0,2,null,5,-6100 from dual union all
select 1097,'SR19','1',0,1,null,70,-6100 from dual union all
select 1097,'SR19','2',0,1,null,300,-5500 from dual union all
select 1098,'C-17','0',0,1,null,10,-6378 from dual union all
select 1098,'SR19','1',0,1,null,50,-6300 from dual union all
select 1099,'SRAM','1',0,1,null,10,-6378 from dual union all
select 11,'AAD','1',0,1,null,40,-6370 from dual union all
select 110,'Ariane H10-3','3',0,1,48906,35787,200 from dual union all
select 110,'Ariane L220','1',0,1,621970,100,-5700 from dual union all
select 110,'Ariane L33','2',0,1,98910,200,-3700 from dual union all
select 110,'PAP','0',0,4,102900,5,-6370 from dual union all
select 1100,'S-520','1',0,1,null,50,-6378 from dual union all
select 1100,'SS-520 St 2','2',0,1,null,1000,-6373 from dual union all
select 1100,'SS-520 St 3','3',0,1,null,1000,180 from dual union all
select 1101,'S-520','1',0,1,null,50,-6378 from dual union all
select 1101,'SS-520 St 2','2',0,1,null,1000,-6373 from dual union all
select 1102,'SEP 901','1',0,1,null,50,-6300 from dual union all
select 1102,'SEP 903','2',0,1,null,500,-6000 from dual union all
select 1103,'SEP 902','1',0,1,null,100,-6300 from dual union all
select 1103,'SEP 903','2',0,1,null,1000,-6000 from dual union all
select 1104,'SEP 901','1',0,1,null,50,-6300 from dual union all
select 1104,'SEP 903','2',1,1,null,50,-6300 from dual union all
select 1105,'SEP 902','1',0,1,null,100,-6300 from dual union all
select 1105,'SEP 903','2',0,1,null,1000,-6000 from dual union all
select 1106,'Rita II','2',0,1,null,1000,-6000 from dual union all
select 1106,'SEP 902','1',0,1,null,100,-6300 from dual union all
select 1107,'ST','1',0,1,null,20,-6378 from dual union all
select 1108,'ST-735','1',0,1,null,50,-6378 from dual union all
select 1108,'ST-735-SB','0',0,2,null,5,-6378 from dual union all
select 1109,'ST-735 Stage 2','2',0,1,null,100,-6378 from dual union all
select 1109,'ST-735','1',0,1,null,30,-6378 from dual union all
select 111,'Ariane 5 EPC H175','1',0,1,656146,1500,50 from dual union all
select 111,'Ariane EAP P240','0',0,2,1262652,75,-6100 from dual union all
select 111,'ESC-A','2',0,1,null,35780,200 from dual union all
select 1110,'Super Loki','1',0,1,null,82,-6370 from dual union all
select 1111,'A3P','1',0,1,null,20,-6378 from dual union all
select 1111,'Orbus 1','3',0,1,null,1000,-5500 from dual union all
select 1111,'X-260','2',0,1,null,500,-6000 from dual union all
select 1112,'STIG-A','1',0,1,null,80,-6378 from dual union all
select 1113,'STIG-B','1',0,1,null,80,-6378 from dual union all
select 1114,'Safir Stage 2','2',0,1,null,250,-6300 from dual union all
select 1114,'Shahab 3','1',0,1,null,150,-6300 from dual union all
select 1115,'Sandhawk','1',0,1,null,100,-6378 from dual union all
select 1116,'Sandhawk','1',0,1,null,50,-6378 from dual union all
select 1116,'Tomahawk','2',0,1,null,500,-6378 from dual union all
select 1117,'Emeraude','1',0,1,null,50,-6300 from dual union all
select 1117,'Topaze','2',0,1,null,1000,-5000 from dual union all
select 1118,'Saturn S-I','1',0,1,null,130,-6300 from dual union all
select 1118,'Saturn S-IVD','2',1,1,null,130,-6300 from dual union all
select 1118,'Saturn S-V','3',1,1,null,130,-6300 from dual union all
select 1119,'Saturn S-I','1',0,1,null,160,-5940 from dual union all
select 1119,'Saturn S-IV','2',0,1,null,180,180 from dual union all
select 112,'Ariane 5 EPC H175','1',0,1,656146,1500,50 from dual union all
select 112,'Ariane 5 EPS P2000','2',0,1,31900,35780,200 from dual union all
select 112,'Ariane EAP P240','0',0,2,1262652,75,-6100 from dual union all
select 1120,'Saturn S-I','1',0,1,null,130,-6250 from dual union all
select 1120,'Saturn S-IVD','2',1,1,null,130,-6250 from dual union all
select 1120,'Saturn S-V','3',1,1,null,130,-6250 from dual union all
select 1121,'Saturn S-IB','1',0,1,null,120,-6145 from dual union all
select 1121,'Saturn S-IVB','2',0,1,null,220,220 from dual union all
select 1122,'S-II Interstage','2',0,1,null,427,427 from dual union all
select 1122,'Saturn S-IC','1',0,1,null,206,-6055 from dual union all
select 1122,'Saturn S-II','3',0,1,null,427,427 from dual union all
select 1123,'S-II Interstage','2',0,1,null,120,-5800 from dual union all
select 1123,'Saturn S-IC','1',0,1,null,114,-5998 from dual union all
select 1123,'Saturn S-II','3',0,1,null,189,-2112 from dual union all
select 1123,'Saturn S-IVB','4',0,1,null,188,188 from dual union all
select 1124,'Aerojet Jr','1',0,1,null,30,-6370 from dual union all
select 1124,'Altair','3',0,1,null,700,-6000 from dual union all
select 1124,'Lance','2',0,1,null,100,-6200 from dual union all
select 1124,'Recruit','0',0,2,null,5,-6378 from dual union all
select 1125,'Algol 2A','1',0,1,null,70,-6300 from dual union all
select 1125,'Altair 2','4',0,1,null,1000,1000 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 1125,'Antares 2','3',0,1,null,1000,-5000 from dual union all
select 1125,'Castor 2A','2',0,1,null,280,-6000 from dual union all
select 1126,'Algol 2C','1',0,1,null,70,-6300 from dual union all
select 1126,'Altair 2','4',0,1,null,1000,1000 from dual union all
select 1126,'Antares 2','3',0,1,null,1000,-5000 from dual union all
select 1126,'Castor 2A','2',0,1,null,280,-6000 from dual union all
select 1127,'Algol 2A','1',0,1,null,70,-6300 from dual union all
select 1127,'Antares 2','3',0,1,null,500,-5000 from dual union all
select 1127,'Castor 2A','2',0,1,null,280,-6100 from dual union all
select 1127,'FW4S','4',0,1,null,500,500 from dual union all
select 1128,'Algol 2B','1',0,1,null,70,-6300 from dual union all
select 1128,'Antares 2','3',0,1,null,1000,-5000 from dual union all
select 1128,'Castor 2A','2',0,1,null,280,-6000 from dual union all
select 1128,'FW4S','4',0,1,null,1000,1000 from dual union all
select 1129,'Algol 2C','1',0,1,null,70,-6300 from dual union all
select 1129,'Antares 2','3',0,1,null,500,-5000 from dual union all
select 1129,'Castor 2A','2',0,1,null,280,-6100 from dual union all
select 1129,'FW4S','4',0,1,null,500,500 from dual union all
select 113,'Ariane 5 EPC H175','1',0,1,656146,1500,50 from dual union all
select 113,'Ariane 5 EPS P2000','2',0,1,31900,35780,200 from dual union all
select 113,'Ariane EAP P240','0',0,2,1262652,75,-6100 from dual union all
select 1130,'Algol 2C','1',0,1,null,70,-6300 from dual union all
select 1130,'Antares 2','3',0,1,null,500,-5000 from dual union all
select 1130,'Castor 2A','2',0,1,null,280,-6100 from dual union all
select 1130,'Star 20','4',0,1,null,500,500 from dual union all
select 1131,'Algol 3A','1',0,1,null,83,-6317 from dual union all
select 1131,'Antares 2','3',0,1,null,275,-5535 from dual union all
select 1131,'Castor 2A','2',0,1,null,280,-5925 from dual union all
select 1131,'FW4S','4',0,1,null,1000,275 from dual union all
select 1132,'Algol 3A','1',0,1,null,83,-6317 from dual union all
select 1132,'Antares 2','3',0,1,null,275,-4435 from dual union all
select 1132,'Castor 2A','2',0,1,null,280,-5925 from dual union all
select 1132,'Star 20','4',0,1,null,1000,275 from dual union all
select 1133,'Algol 3A','1',0,1,null,70,-6300 from dual union all
select 1133,'Antares 2','3',0,1,null,340,-5000 from dual union all
select 1133,'BE-3','5',0,1,null,800,340 from dual union all
select 1133,'Castor 2A','2',0,1,null,280,-6100 from dual union all
select 1133,'FW4S','4',0,1,null,800,340 from dual union all
select 1134,'Algol 3A','1',0,1,null,70,-6300 from dual union all
select 1134,'Antares 2B','3',0,1,null,350,-4330 from dual union all
select 1134,'Castor 2A','2',0,1,null,280,-6100 from dual union all
select 1134,'Star 20','4',0,1,null,350,350 from dual union all
select 1135,'Algol 3A','1',0,1,null,70,-6300 from dual union all
select 1135,'Antares 3','3',0,1,null,370,-5000 from dual union all
select 1135,'Castor 2A','2',0,1,null,250,-6200 from dual union all
select 1135,'Star 20','4',0,1,null,1000,1000 from dual union all
select 1136,'Zefiro','1',0,1,null,1,-6378 from dual union all
select 1137,'Algol 1A','1',0,1,null,48,-6370 from dual union all
select 1137,'Altair','4',1,1,null,48,-6370 from dual union all
select 1137,'Antares','3',0,1,null,48,-6370 from dual union all
select 1137,'Castor','2',1,1,null,48,-6370 from dual union all
select 1138,'Algol 1B','1',0,1,null,77,-6353 from dual union all
select 1138,'Altair','4',0,1,null,200,200 from dual union all
select 1138,'Antares','3',0,1,null,640,-5050 from dual union all
select 1138,'Castor','2',0,1,null,433,-6216 from dual union all
select 1139,'Algol 1B','1',0,1,null,77,-6300 from dual union all
select 1139,'Altair','4',0,1,null,349,-4198 from dual union all
select 1139,'Antares','3',0,1,null,214,-5000 from dual union all
select 1139,'Castor','2',0,1,null,213,-6200 from dual union all
select 1139,'Cetus','5',0,1,null,1084,-3052 from dual union all
select 114,'Ariane 5 EPC H158','1',0,1,656146,1500,50 from dual union all
select 114,'Ariane 5 EPS','2',0,1,31900,35780,200 from dual union all
select 114,'Ariane EAP P238','0',0,2,1262652,75,-6000 from dual union all
select 1140,'Algol 1C','1',0,1,null,75,-6300 from dual union all
select 1140,'Altair','4',0,1,null,7233,-4400 from dual union all
select 1140,'Antares 2','3',0,1,null,1400,-5800 from dual union all
select 1140,'Castor','2',0,1,null,280,-6200 from dual union all
select 1141,'Algol 1D','1',0,1,null,75,-6350 from dual union all
select 1141,'Altair','4',0,1,null,900,900 from dual union all
select 1141,'Antares 2','3',0,1,null,900,-5900 from dual union all
select 1141,'Castor','2',0,1,null,400,-6200 from dual union all
select 1142,'Algol 1D','1',0,1,null,75,-6300 from dual union all
select 1142,'Altair 2','4',0,1,null,730,-5000 from dual union all
select 1142,'Antares 2','3',0,1,null,730,-5000 from dual union all
select 1142,'Castor','2',0,1,null,280,-6200 from dual union all
select 1143,'Algol 1D','1',0,1,null,75,-6300 from dual union all
select 1143,'Antares 2','3',0,1,null,700,-5000 from dual union all
select 1143,'Castor','2',0,1,null,280,-6200 from dual union all
select 1143,'MG-18','4',0,1,null,700,700 from dual union all
select 1144,'Algol 2A','1',0,1,null,60,-6330 from dual union all
select 1144,'Altair','4',0,1,null,1100,740 from dual union all
select 1144,'Antares 2','3',0,1,null,740,-4500 from dual union all
select 1144,'Castor','2',0,1,null,220,-6030 from dual union all
select 1145,'Algol 2A','1',0,1,null,60,-6330 from dual union all
select 1145,'Altair','4',0,1,null,220,-6030 from dual union all
select 1145,'Antares 2','3',0,1,null,220,-6030 from dual union all
select 1145,'Castor','2',0,1,null,220,-6030 from dual union all
select 1145,'Cetus','5',0,1,null,7000,-6000 from dual union all
select 1146,'Algol 2A','1',0,1,null,60,-6330 from dual union all
select 1146,'Antares 2','3',0,1,null,183,-3620 from dual union all
select 1146,'Castor','2',0,1,null,136,-6000 from dual union all
select 1147,'Algol 2A','1',0,1,null,75,-6300 from dual union all
select 1147,'Antares 2','3',0,1,null,700,-5000 from dual union all
select 1147,'Castor','2',0,1,null,280,-6200 from dual union all
select 1147,'MG-18','4',0,1,null,700,700 from dual union all
select 1148,'Algol 2A','1',0,1,null,77,-6319 from dual union all
select 1148,'Altair 2','4',0,1,null,1000,1000 from dual union all
select 1148,'Antares 2','3',0,1,null,1000,-4870 from dual union all
select 1148,'Castor','2',0,1,null,280,-6006 from dual union all
select 1149,'Algol 2A','1',0,1,null,70,-6300 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 1149,'Altair 2','4',0,1,null,175,-6000 from dual union all
select 1149,'Antares 2','3',0,1,null,175,-6000 from dual union all
select 1149,'Castor','2',0,1,null,175,-6000 from dual union all
select 1149,'Cetus','5',0,1,null,1709,-414 from dual union all
select 115,'Ariane 5 EPC H158','1',0,1,656146,1500,50 from dual union all
select 115,'Ariane 5 EPS P2000','2',0,1,31900,35780,200 from dual union all
select 115,'Ariane EAP P238','0',0,2,1262652,75,-6000 from dual union all
select 1150,'Algol 2B','1',0,1,null,70,-6300 from dual union all
select 1150,'Antares 2','3',0,1,null,664,-3866 from dual union all
select 1150,'Castor 2','2',0,1,null,170,-6000 from dual union all
select 1151,'Scud','1',0,1,null,100,-6200 from dual union all
select 1152,'Scud B','1',0,1,null,100,-6200 from dual union all
select 1153,'Scud D','1',0,1,null,100,-6200 from dual union all
select 1154,'Aerobee 100','1',0,1,null,1,-6370 from dual union all
select 1155,'Seagull','1',0,1,null,50,-6370 from dual union all
select 1156,'Sejjil Stage 2','2',0,1,null,200,-6300 from dual union all
select 1156,'Sejjil','1',0,1,null,50,-6300 from dual union all
select 1157,'Sejjil Stage 2','2',0,1,null,200,-6300 from dual union all
select 1157,'Sejjil','1',0,1,null,50,-6300 from dual union all
select 1158,'Seliger 1','1',0,1,null,50,-6378 from dual union all
select 1159,'Seliger 1','1',0,1,null,20,-6378 from dual union all
select 1159,'Seliger 1','2',0,1,null,50,-6378 from dual union all
select 116,'Ariane 5 EPC H158','1',0,1,656146,1500,50 from dual union all
select 116,'Ariane 5 EPS P2000','2',0,1,31900,35780,200 from dual union all
select 116,'Ariane EAP P240','0',0,2,1262652,75,-6100 from dual union all
select 1160,'Seliger 1','1',0,1,null,20,-6378 from dual union all
select 1160,'Seliger 1','2',0,1,null,50,-6378 from dual union all
select 1160,'Seliger 3','3',0,1,null,120,-6350 from dual union all
select 1161,'Recruit','0',0,2,null,1,-6200 from dual union all
select 1161,'Sergeant','1',0,1,null,100,-6200 from dual union all
select 1162,'Sergeant','1',0,1,null,100,-6200 from dual union all
select 1163,'Lance','2',0,1,null,10,-6378 from dual union all
select 1163,'Lance','3',0,1,null,10,-6378 from dual union all
select 1163,'Recruit','4',0,1,null,30,-6300 from dual union all
select 1163,'Sergeant','1',0,1,null,10,-6378 from dual union all
select 1163,'T-55','5',0,1,null,34,-6200 from dual union all
select 1164,'Hydac','2',0,1,null,400,-6300 from dual union all
select 1164,'Sergeant','1',0,1,null,50,-6370 from dual union all
select 1165,'Super Loki','1',0,1,null,120,-6370 from dual union all
select 1166,'Hwasong 5','1',0,1,null,200,-6250 from dual union all
select 1167,'Hwasong 6','1',0,1,null,200,-6250 from dual union all
select 1168,'Shahab 3','1',0,1,null,150,-6300 from dual union all
select 1169,'Shaheen','1',0,1,null,100,-6370 from dual union all
select 117,'M56A1','1',0,1,null,300,-6350 from dual union all
select 1170,'Shaheen','1',0,1,null,100,-6370 from dual union all
select 1171,'Shaheen 2 St 1','1',0,1,null,80,-6370 from dual union all
select 1171,'Shaheen St 2','2',0,1,null,300,-6300 from dual union all
select 1172,'Shaheen 2 St 1','1',0,1,null,80,-6370 from dual union all
select 1172,'Shaheen 3 St 2','2',0,1,null,300,-6300 from dual union all
select 1173,'ATSM-9','1',0,1,null,100,-6370 from dual union all
select 1173,'ATSM-9','2',0,1,null,250,-5740 from dual union all
select 1173,'AUS-51','3',0,1,null,700,250 from dual union all
select 1174,'ATSM-9','2',0,1,null,250,-5740 from dual union all
select 1174,'AUS-51','3',0,1,null,700,250 from dual union all
select 1174,'Shaviyt 1','1',0,1,null,100,-6370 from dual union all
select 1175,'SQ-1S','1',0,1,null,100,-6378 from dual union all
select 1176,'Short Al Hussein','1',0,1,null,100,-6200 from dual union all
select 1177,'Altair','2',0,1,null,400,-6160 from dual union all
select 1177,'Pollux','1',0,1,null,70,-6350 from dual union all
select 1177,'Recruit','0',0,2,null,1,-6378 from dual union all
select 1178,'Shourya Booster','1',0,1,null,10,-6370 from dual union all
select 1178,'Shourya','2',0,1,null,200,-6370 from dual union all
select 1179,'R-29RM St1','1',0,1,null,70,-6100 from dual union all
select 1179,'R-29RM St2','2',0,1,null,500,-5000 from dual union all
select 1179,'R-29RM St3','3',0,1,null,1000,-3000 from dual union all
select 118,'IAI Arrow-1','1',0,1,null,20,-6378 from dual union all
select 1180,'R-29RM St1','1',0,1,null,70,-6100 from dual union all
select 1180,'R-29RM St2','2',0,1,null,500,-5000 from dual union all
select 1180,'R-29RM St3','3',0,1,null,750,300 from dual union all
select 1181,'HV Arcas','2',0,1,null,90,-6347 from dual union all
select 1181,'Sidewinder 1A','1',0,1,null,3,-6378 from dual union all
select 1182,'Hopi IV','2',0,1,null,110,-6350 from dual union all
select 1182,'Sidewinder 1A-15','1',0,1,null,3,-6378 from dual union all
select 1183,'F-15D','0',0,1,null,10,-6378 from dual union all
select 1183,'Silver Sparrow','1',0,1,null,100,-6378 from dual union all
select 1184,'Simorgh Stage 1','1',0,1,null,70,-5000 from dual union all
select 1184,'Simorgh Stage 2','2',0,1,null,500,500 from dual union all
select 1185,'R-29RM St1','1',0,1,null,70,-6100 from dual union all
select 1185,'R-29RM St2','2',0,1,null,500,-5000 from dual union all
select 1185,'R-29RM St3','3',0,1,null,1000,-3000 from dual union all
select 1186,'Sirocco','1',0,1,null,60,-6370 from dual union all
select 1187,'15Zh58A','1',0,1,null,70,-6300 from dual union all
select 1187,'15Zh58B','2',0,1,null,200,-6000 from dual union all
select 1188,'Chick x3','0',0,1,null,2,-6378 from dual union all
select 1188,'Skua','1',0,1,null,100,-6370 from dual union all
select 1189,'Chick x4','0',0,1,null,2,-6378 from dual union all
select 1189,'Skua 2','1',0,1,null,100,-6370 from dual union all
select 119,'IAI Arrow St2','2',0,1,null,80,-6378 from dual union all
select 119,'IAI Arrow','1',0,1,null,10,-6378 from dual union all
select 1190,'Chick x4','0',0,1,null,2,-6378 from dual union all
select 1190,'Skua 3','1',0,1,null,100,-6370 from dual union all
select 1191,'Chick x4','0',0,1,null,2,-6378 from dual union all
select 1191,'Skua 3','1',0,1,null,100,-6370 from dual union all
select 1192,'Skybolt St1','1',0,1,null,20,-6378 from dual union all
select 1192,'Skybolt St2','2',0,1,null,20,-6370 from dual union all
select 1193,'Raven 1/1A/2A','1',0,1,null,180,-6300 from dual union all
select 1194,'Raven 2','1',0,1,null,180,-6300 from dual union all
select 1195,'Cuckoo IA','1',0,1,null,3,-6378 from dual union all
select 1195,'Raven 2','2',0,1,null,180,-6300 from dual union all
select 1196,'Raven 5','1',0,1,null,180,-6300 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 1197,'Cuckoo IA','1',0,1,null,3,-6378 from dual union all
select 1197,'Raven 5','2',0,1,null,180,-6300 from dual union all
select 1198,'Raven 7','1',0,1,null,180,-6300 from dual union all
select 1199,'Cuckoo IA','1',0,1,null,3,-6378 from dual union all
select 1199,'Raven 7','2',0,1,null,180,-6300 from dual union all
select 12,'Lupus','1',0,1,null,20,-6378 from dual union all
select 120,'Arrow 3 Stage 2','2',0,1,null,80,-6378 from dual union all
select 120,'Arrow 3','1',0,1,null,10,-6378 from dual union all
select 1200,'Raven 8','1',0,1,null,100,-6300 from dual union all
select 1201,'Goldfinch','1',0,1,null,5,-6378 from dual union all
select 1201,'Gosling 4T','2',0,1,null,240,-6300 from dual union all
select 1202,'Cuckoo 4','3',0,1,null,800,-6000 from dual union all
select 1202,'Goldfinch','1',0,1,null,5,-6378 from dual union all
select 1202,'Raven 11','2',0,1,null,150,-6300 from dual union all
select 1203,'Cuckoo 4','3',0,1,null,800,-6000 from dual union all
select 1203,'Goldfinch II','1',0,1,null,5,-6378 from dual union all
select 1203,'Raven 11','2',0,1,null,150,-6300 from dual union all
select 1204,'Cuckoo IA','1',0,1,null,3,-6378 from dual union all
select 1204,'Raven 8','2',0,1,null,200,-6300 from dual union all
select 1205,'Cuckoo IA','1',0,1,null,3,-6378 from dual union all
select 1205,'Raven 8','2',0,1,null,200,-6300 from dual union all
select 1206,'Cuckoo IA','1',0,1,null,3,-6378 from dual union all
select 1206,'Raven 6','2',0,1,null,200,-6300 from dual union all
select 1207,'Cuckoo IA','1',0,1,null,3,-6378 from dual union all
select 1207,'Raven 6','2',0,1,null,260,-6300 from dual union all
select 1208,'Goldfinch','1',0,1,null,3,-6378 from dual union all
select 1208,'Raven 8','2',0,1,null,240,-6300 from dual union all
select 1209,'Goldfinch','1',0,1,null,3,-6378 from dual union all
select 1209,'Raven 8','2',0,1,null,240,-6300 from dual union all
select 121,'Sejjil Stage 2','2',0,1,null,50,-6300 from dual union all
select 121,'Sejjil','1',0,1,null,50,-6300 from dual union all
select 1210,'Goldfinch II','1',0,1,null,3,-6378 from dual union all
select 1210,'Raven 6','2',0,1,null,240,-6300 from dual union all
select 1211,'Goldfinch II','1',0,1,null,3,-6378 from dual union all
select 1211,'Raven 6','2',0,1,null,240,-6300 from dual union all
select 1212,'Goldfinch II','1',0,1,null,3,-6378 from dual union all
select 1212,'Raven 11','2',0,1,null,240,-6300 from dual union all
select 1213,'Goldfinch II','1',0,1,null,3,-6378 from dual union all
select 1213,'Raven 11','2',0,1,null,240,-6300 from dual union all
select 1214,'Snark Booster','1',0,1,null,20,-6378 from dual union all
select 1215,'Sonda 1-1','1',0,1,null,5,-6378 from dual union all
select 1215,'Sonda 1-2','2',0,1,null,65,-6370 from dual union all
select 1216,'Sonda 2A','1',0,1,null,100,-6370 from dual union all
select 1217,'Sonda 2M','2',0,1,null,500,-6300 from dual union all
select 1217,'Sonda 3-1','1',0,1,null,50,-6370 from dual union all
select 1218,'Sonda 2M','2',0,1,null,500,-6300 from dual union all
select 1218,'Sonda 3-1','1',0,1,null,50,-6370 from dual union all
select 1219,'Sonda 2A','2',0,1,null,500,-6300 from dual union all
select 1219,'Sonda 3-1','1',0,1,null,50,-6370 from dual union all
select 122,'Asp','1',0,1,null,30,-6370 from dual union all
select 1220,'S-43','1',0,1,null,130,-6370 from dual union all
select 1220,'Sonda 3-1','2',0,1,null,800,-6300 from dual union all
select 1221,'NSPO Booster','1',0,1,null,10,-6370 from dual union all
select 1221,'NSPO Stage 2','2',0,1,null,200,-6370 from dual union all
select 1222,'11S59','1',0,4,null,75,-5900 from dual union all
select 1222,'11S59A','2',0,1,null,200,-1000 from dual union all
select 1223,'11S510','3',0,1,null,200,200 from dual union all
select 1223,'11S59','1',0,4,null,75,-5900 from dual union all
select 1223,'11S59A','2',0,1,null,170,-4400 from dual union all
select 1224,'11S510','3',0,1,null,200,200 from dual union all
select 1224,'11S59','1',0,4,null,75,-5900 from dual union all
select 1224,'11S59A','2',0,1,null,175,-4400 from dual union all
select 1225,'11S510','3',0,1,null,200,200 from dual union all
select 1225,'11S59','1',0,4,null,75,-5900 from dual union all
select 1225,'11S59A','2',0,1,null,175,-4400 from dual union all
select 1226,'14S53','1',0,4,null,75,-5900 from dual union all
select 1226,'14S53A','2',0,1,null,120,-5700 from dual union all
select 1226,'14S54/RD-0110','3',0,1,null,200,200 from dual union all
select 1226,'Fregat','4',0,1,null,200,200 from dual union all
select 1227,'14S53','1',0,4,null,75,-5900 from dual union all
select 1227,'14S53A','2',0,1,null,120,-5700 from dual union all
select 1227,'14S54/RD-0110','3',0,1,null,200,200 from dual union all
select 1228,'14S53','1',0,4,null,75,-5900 from dual union all
select 1228,'14S53A','2',0,1,null,120,-5700 from dual union all
select 1228,'14S54','3',0,1,null,200,200 from dual union all
select 1228,'Fregat','4',0,1,null,200,200 from dual union all
select 1229,'14S53','1',0,4,null,75,-5900 from dual union all
select 1229,'14S53A','2',0,1,null,120,-5700 from dual union all
select 1229,'14S54','3',0,1,null,200,200 from dual union all
select 123,'Asp II','1',0,1,null,30,-6370 from dual union all
select 1230,'14S54','2',0,1,null,300,300 from dual union all
select 1230,'Soyuz-2-1V 132KS','1',0,1,null,300,-5500 from dual union all
select 1230,'Volga','3',0,1,null,300,300 from dual union all
select 1231,'14S54','2',0,1,null,300,300 from dual union all
select 1231,'Soyuz-2-1V 132KS','1',0,1,null,300,-5500 from dual union all
select 1232,'11S510','3',0,1,null,100,100 from dual union all
select 1232,'11S59','1',0,4,null,75,-5900 from dual union all
select 1232,'11S59A','2',0,1,null,100,-5700 from dual union all
select 1232,'Fregat','4',0,1,null,200,200 from dual union all
select 1233,'11S510','3',0,1,null,200,200 from dual union all
select 1233,'11S59','1',0,4,null,75,-5900 from dual union all
select 1233,'11S59A','2',0,1,null,100,-5700 from dual union all
select 1234,'374BL01A','1',0,4,null,75,-5900 from dual union all
select 1234,'374BL01A-A','2',0,1,null,120,-5700 from dual union all
select 1234,'374BL02A','3',0,1,null,200,200 from dual union all
select 1234,'Fregat','4',0,1,null,200,200 from dual union all
select 1235,'374BL01A','1',0,4,null,75,-5900 from dual union all
select 1235,'374BL01A-A','2',0,1,null,120,-5700 from dual union all
select 1235,'374BL02A','3',0,1,null,200,200 from dual union all
select 1236,'374BL01B','1',0,4,null,75,-5900 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 1236,'374BL01B-A','2',0,1,null,120,-5700 from dual union all
select 1236,'374BL02B','3',0,1,null,200,200 from dual union all
select 1236,'Fregat','4',0,1,null,200,200 from dual union all
select 1237,'374BL01B','1',0,4,null,75,-5900 from dual union all
select 1237,'374BL01B-A','2',0,1,null,120,-5700 from dual union all
select 1237,'374BL02B','3',0,1,null,200,200 from dual union all
select 1238,'11S510','3',0,1,null,200,200 from dual union all
select 1238,'11S59','1',0,4,null,75,-5900 from dual union all
select 1238,'11S59A','2',0,1,null,175,-4400 from dual union all
select 1239,'11S510 (F)','3',0,1,null,180,-1200 from dual union all
select 1239,'11S59','1',0,4,null,75,-5900 from dual union all
select 1239,'11S59A','2',0,1,null,100,-5700 from dual union all
select 1239,'Fregat','4',0,1,null,200,200 from dual union all
select 124,'Asp II','1',0,1,null,50,-6370 from dual union all
select 124,'Loki','0',0,4,null,1,-6378 from dual union all
select 1240,'11S510','3',0,1,null,200,900 from dual union all
select 1240,'11S59','1',0,4,null,75,-5900 from dual union all
select 1240,'11S59A','2',0,1,null,230,-5500 from dual union all
select 1240,'Ikar','4',0,1,null,900,900 from dual union all
select 1241,'11S510','3',0,1,null,200,200 from dual union all
select 1241,'11S59','1',0,4,null,75,-5900 from dual union all
select 1241,'11S59A','2',0,1,null,175,-4400 from dual union all
select 1242,'11S510','3',0,1,null,200,200 from dual union all
select 1242,'11S59','1',0,4,null,75,-5900 from dual union all
select 1242,'11S59A-S','2',0,1,null,175,-4400 from dual union all
select 1243,'ET','2',0,1,null,280,80 from dual union all
select 1243,'OV','3',0,1,null,280,80 from dual union all
select 1243,'SRM','1',0,2,null,65,-6250 from dual union all
select 1244,'SpaceLoft XL','1',0,1,null,100,-6370 from dual union all
select 1245,'Spaceship One','1',0,1,null,110,-6377 from dual union all
select 1245,'White Knight','0',0,1,null,12,-6378 from dual union all
select 1246,'Spaceship Two','1',0,1,null,110,-6377 from dual union all
select 1246,'White Knight 2','0',0,1,null,12,-6378 from dual union all
select 1247,'F3H Cougar','0',0,1,null,10,-6378 from dual union all
select 1247,'Sparrow','1',0,1,null,20,-6370 from dual union all
select 1247,'Sparrow','2',0,1,null,100,-6100 from dual union all
select 1248,'F3H Cougar','0',0,1,null,10,-6378 from dual union all
select 1248,'Sparrow','1',0,1,null,20,-6370 from dual union all
select 1248,'Sparrow','2',0,1,null,100,-6100 from dual union all
select 1249,'22.6KS1245','2',0,1,null,30,-6370 from dual union all
select 1249,'F4B Phantom','0',0,1,null,10,-6378 from dual union all
select 1249,'Sparrow','1',0,1,null,20,-6370 from dual union all
select 125,'Asp II','1',0,1,null,50,-6370 from dual union all
select 125,'Loki','0',0,2,null,1,-6378 from dual union all
select 1250,'HV Arcas','2',0,1,null,100,-6370 from dual union all
select 1250,'Sparrow','1',0,1,null,20,-6378 from dual union all
select 1251,'TX-239','3',0,1,null,100,-6370 from dual union all
select 1251,'TX-454','2',0,1,null,30,-6370 from dual union all
select 1251,'TX-500','1',0,1,null,10,-6378 from dual union all
select 1252,'X-265','1',0,1,null,1,-6378 from dual union all
select 1252,'X-271','2',0,1,null,10,-6378 from dual union all
select 1253,'8K74','1',0,4,null,75,-5900 from dual union all
select 1253,'8K74A','2',0,1,null,300,-500 from dual union all
select 1254,'8K74','1',0,4,null,75,-5900 from dual union all
select 1254,'8K74A','2',0,1,null,1000,300 from dual union all
select 1255,'8K71','1',0,4,null,75,-5900 from dual union all
select 1255,'8K71A','2',0,1,null,1000,300 from dual union all
select 1256,'Squirt St 1','1',0,1,null,1,-6378 from dual union all
select 1256,'Squirt St 2','2',0,1,null,1,-6378 from dual union all
select 1257,'Stable Super Loki','1',0,1,null,120,-6370 from dual union all
select 1258,'Orbus 1','3',0,1,null,100,-6300 from dual union all
select 1258,'Orbus 1','4',0,1,null,100,-6250 from dual union all
select 1258,'Sergeant','2',0,1,null,30,-6378 from dual union all
select 1258,'Talos','1',0,1,null,10,-6378 from dual union all
select 1259,'15Zh58A','1',0,1,null,50,-6300 from dual union all
select 1259,'15Zh58B','2',0,1,null,100,-5200 from dual union all
select 1259,'15Zh58B','3',0,1,null,150,-4500 from dual union all
select 1259,'15Zh58V','4',0,1,null,600,-2500 from dual union all
select 1259,'DS','6',0,1,null,600,600 from dual union all
select 1259,'Start','5',0,1,null,600,600 from dual union all
select 126,'Asp IV','1',0,1,null,80,-6370 from dual union all
select 1260,'15Zh58A','1',0,1,null,55,-6320 from dual union all
select 1260,'15Zh58B','2',0,1,null,150,-5800 from dual union all
select 1260,'15Zh58V','3',0,1,null,600,-4500 from dual union all
select 1260,'DS','5',0,1,null,600,600 from dual union all
select 1260,'Start','4',0,1,null,600,600 from dual union all
select 1261,'15Zh58A','1',0,1,null,55,-6300 from dual union all
select 1261,'15Zh58B','2',0,1,null,150,-5800 from dual union all
select 1261,'15Zh58V','3',0,1,null,500,-4500 from dual union all
select 1261,'DS','5',0,1,null,500,500 from dual union all
select 1261,'Start','4',0,1,null,500,500 from dual union all
select 1262,'M57A1','2',0,1,null,200,-6350 from dual union all
select 1262,'Sergeant','1',0,1,null,50,-6370 from dual union all
select 1263,'SR19AJ1','1',0,1,null,100,-6350 from dual union all
select 1264,'Stratos II+','1',0,1,null,50,-6300 from dual union all
select 1265,'Strela APB','3',0,1,null,450,400 from dual union all
select 1265,'UR-100NA','1',0,1,null,130,-5800 from dual union all
select 1265,'UR-100NB','2',0,1,null,450,-800 from dual union all
select 1266,'M-6','1',0,1,null,10,-6378 from dual union all
select 1266,'Nike','2',0,1,null,40,-6370 from dual union all
select 1266,'Nike','3',0,1,null,75,-6300 from dual union all
select 1266,'Recruit','4',0,1,null,75,-6300 from dual union all
select 1267,'Baby Sergeant','5',0,1,null,1000,-6000 from dual union all
select 1267,'M-6','1',0,1,null,10,-6378 from dual union all
select 1267,'Nike','2',0,1,null,40,-6370 from dual union all
select 1267,'Nike','3',0,1,null,75,-6300 from dual union all
select 1267,'Yardbird','4',0,1,null,500,-6200 from dual union all
select 1268,'Castor','1',0,1,null,220,-6100 from dual union all
select 1268,'Recruit','0',0,2,null,1,-6378 from dual union all
select 1269,'Castor','1',0,1,null,250,-6300 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 1269,'Recruit','0',0,2,null,2,-6378 from dual union all
select 127,'Astra Stage 1','1',0,1,null,200,-6378 from dual union all
select 127,'Astra Stage 2','2',0,1,null,200,200 from dual union all
select 1270,'Castor 2','1',0,1,null,250,-6300 from dual union all
select 1270,'Recruit','0',0,2,null,2,-6378 from dual union all
select 1271,'Castor','1',0,1,null,250,-6300 from dual union all
select 1271,'Recruit','0',0,2,null,2,-6378 from dual union all
select 1272,'Castor','1',0,1,null,200,-6300 from dual union all
select 1272,'Recruit','0',0,2,null,2,-6378 from dual union all
select 1272,'Star 26','2',0,1,null,200,-6000 from dual union all
select 1273,'Castor','1',0,1,null,200,-6300 from dual union all
select 1273,'Recruit','0',0,2,null,2,-6378 from dual union all
select 1273,'Star 26','2',0,1,null,200,-6000 from dual union all
select 1274,'Castor','1',0,1,null,200,-6300 from dual union all
select 1274,'Recruit','0',0,2,null,2,-6378 from dual union all
select 1274,'Star 26','2',0,1,null,200,-6000 from dual union all
select 1275,'Antares 2','2',0,1,null,200,-5000 from dual union all
select 1275,'Castor','1',0,1,null,200,-6300 from dual union all
select 1276,'Alcor','2',0,1,null,200,-6100 from dual union all
select 1276,'Alcyone','3',0,1,null,250,-6000 from dual union all
select 1276,'Castor','1',0,1,null,200,-6300 from dual union all
select 1276,'Recruit','0',0,2,null,2,-6378 from dual union all
select 1277,'Alcor','2',0,1,null,170,-6300 from dual union all
select 1277,'Alcyone','3',0,1,null,500,-5000 from dual union all
select 1277,'Castor 2','1',0,1,null,40,-6378 from dual union all
select 1277,'Recruit','0',0,2,null,2,-6378 from dual union all
select 1278,'Alcor IB','2',0,1,1148,170,-6300 from dual union all
select 1278,'Alcyone','3',0,1,269,1000,-4300 from dual union all
select 1278,'Castor','1',0,1,7311,40,-6378 from dual union all
select 1278,'Recruit','0',0,2,582,1,-6378 from dual union all
select 1279,'Antares 2','2',0,1,null,200,-6300 from dual union all
select 1279,'Castor','1',0,1,null,50,-6378 from dual union all
select 1279,'Recruit','0',0,2,null,2,-6378 from dual union all
select 1279,'Star 27','3',0,1,null,200,-6000 from dual union all
select 128,'Astrid','1',0,1,null,2,-6378 from dual union all
select 1280,'Castor','1',0,1,null,50,-6370 from dual union all
select 1280,'Orbus 1','2',0,1,null,100,-6370 from dual union all
select 1280,'Recruit','0',0,2,null,2,-6378 from dual union all
select 1281,'Super Arcas','1',0,1,null,80,-6378 from dual union all
select 1282,'Sergeant','2',0,1,null,300,-6350 from dual union all
select 1282,'Talos','1',0,1,null,20,-6378 from dual union all
select 1283,'Castor','2',0,1,null,300,-6350 from dual union all
select 1283,'Talos','1',0,1,null,20,-6378 from dual union all
select 1284,'Super Loki','1',0,1,null,120,-6370 from dual union all
select 1285,'LEO-1','3',0,1,null,300,300 from dual union all
select 1285,'LEO-46','1',0,1,null,10,-6300 from dual union all
select 1285,'LEO-7','2',0,1,null,300,-6300 from dual union all
select 1286,'Antares','2',0,1,null,1200,-6100 from dual union all
select 1286,'Castor','1',0,1,null,100,-6100 from dual union all
select 1286,'Recruit','0',0,2,null,1,-6378 from dual union all
select 1287,'T-7 booster','0',0,1,null,10,-6378 from dual union all
select 1287,'T-7','1',0,1,null,100,-6370 from dual union all
select 1288,'GF-01A','2',0,1,null,300,-6200 from dual union all
select 1288,'T-7 booster','0',0,1,null,10,-6378 from dual union all
select 1288,'T-7','1',0,1,null,100,-6300 from dual union all
select 1289,'T-7 booster','0',0,1,null,10,-6378 from dual union all
select 1289,'T-7','1',0,1,null,100,-6370 from dual union all
select 129,'Aerojet Jr','1',0,1,null,278,-6300 from dual union all
select 129,'Alcor','2',0,1,null,2000,-6065 from dual union all
select 129,'Recruit','0',0,2,null,2,-6378 from dual union all
select 1290,'T-7 booster','0',0,1,null,10,-6378 from dual union all
select 1290,'T-7','1',0,1,null,100,-6370 from dual union all
select 1291,'T-7 booster','0',0,1,null,10,-6378 from dual union all
select 1291,'T-7','1',0,1,null,100,-6370 from dual union all
select 1292,'T-7 booster','0',0,1,null,10,-6378 from dual union all
select 1292,'T-7','1',0,1,null,100,-6370 from dual union all
select 1293,'THAAD','1',0,1,null,100,-6050 from dual union all
select 1294,'THAAD Proto','1',0,1,null,100,-6050 from dual union all
select 1295,'TR-1','1',0,1,null,80,-6370 from dual union all
select 1296,'TR-1A','1',0,1,31616,280,-6370 from dual union all
select 1297,'TT-200','1',0,1,null,100,-6370 from dual union all
select 1298,'TT-210','1',0,1,null,100,-6370 from dual union all
select 1299,'TT-500','1',0,1,null,70,-6370 from dual union all
select 1299,'TT-500-S2','2',0,1,null,300,-6300 from dual union all
select 13,'F4D Phantom','0',0,1,null,0,-6378 from dual union all
select 13,'Genie-Alarr','1',0,1,null,100,-6300 from dual union all
select 130,'Alcor','2',0,1,null,300,-6200 from dual union all
select 130,'Nike','1',0,1,null,20,-6370 from dual union all
select 1300,'TT-500','1',0,1,null,70,-6370 from dual union all
select 1300,'TT-500-S2','2',0,1,null,300,-6300 from dual union all
select 1301,'Tacite','1',0,1,null,140,-6370 from dual union all
select 1302,'Talos','1',0,1,null,10,-6378 from dual union all
select 1303,'Castor','2',0,1,null,330,-6370 from dual union all
select 1303,'Talos','1',0,1,null,10,-6378 from dual union all
select 1304,'Hydac','3',0,1,null,500,-6000 from dual union all
select 1304,'Sergeant','2',0,1,null,50,-6300 from dual union all
select 1304,'Talos','1',0,1,null,10,-6378 from dual union all
select 1305,'Talos','1',0,1,null,1,-6378 from dual union all
select 1305,'Terrier','2',0,1,null,12,-6378 from dual union all
select 1306,'Oriole','3',0,1,null,300,-6378 from dual union all
select 1306,'Talos','1',0,1,null,1,-6378 from dual union all
select 1306,'Terrier Mk 70','2',0,1,null,12,-6378 from dual union all
select 1307,'Peregrine','3',0,1,null,300,-6378 from dual union all
select 1307,'Talos','1',0,1,null,1,-6378 from dual union all
select 1307,'Terrier Mk 70','2',0,1,null,12,-6378 from dual union all
select 1308,'Deacon','1',0,2,null,10,-6378 from dual union all
select 1308,'Deacon','2',0,2,null,10,-6378 from dual union all
select 1309,'Recruit','3',0,1,null,14,-6378 from dual union all
select 1309,'Talos','1',0,1,null,1,-6378 from dual union all
select 1309,'Terrier','2',0,1,null,5,-6378 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 131,'Alcor','2',0,1,null,0,-6378 from dual union all
select 131,'Asp','3',0,1,null,0,-6378 from dual union all
select 131,'Genie','1',0,1,null,0,-6378 from dual union all
select 1310,'Taurion','1',0,1,null,2,-6100 from dual union all
select 1311,'Canopus','2',0,1,null,150,-6350 from dual union all
select 1311,'SS-50','1',0,1,null,10,-6350 from dual union all
select 1312,'Taurus','1',0,1,null,10,-6378 from dual union all
select 1314,'Castor 120','1',0,1,null,50,-6200 from dual union all
select 1314,'Orion 38','4',0,1,null,740,740 from dual union all
select 1314,'Orion 50','2',0,1,null,180,-5000 from dual union all
select 1314,'Orion 50','3',0,1,null,740,-3000 from dual union all
select 1315,'Castor 120','1',0,1,null,50,-6200 from dual union all
select 1315,'Orion 38','4',0,1,null,740,740 from dual union all
select 1315,'Orion 50','2',0,1,null,180,-5000 from dual union all
select 1315,'Orion 50','3',0,1,null,740,-3000 from dual union all
select 1316,'Castor 120','1',0,1,null,50,-6200 from dual union all
select 1316,'Orion 38','4',0,1,null,740,740 from dual union all
select 1316,'Orion 50SXLG','2',0,1,null,180,-5000 from dual union all
select 1316,'Orion 50XL','3',0,1,null,714,-2836 from dual union all
select 1317,'Castor 120','1',0,1,null,50,-6200 from dual union all
select 1317,'Orion 38','4',0,1,null,740,740 from dual union all
select 1317,'Orion 50SXLG','2',0,1,null,180,-5000 from dual union all
select 1317,'Orion 50XL','3',0,1,null,714,-2836 from dual union all
select 1318,'Nike','2',0,1,null,40,-6378 from dual union all
select 1318,'Taurus','1',0,1,null,10,-6378 from dual union all
select 1318,'Tomahawk','3',0,1,null,500,-6360 from dual union all
select 1319,'Orion','2',0,1,null,200,-6360 from dual union all
select 1319,'Taurus','1',0,1,null,20,-6378 from dual union all
select 132,'Astrobee D','1',0,1,null,120,-6370 from dual union all
select 1320,'Taurus','1',0,1,null,20,-6378 from dual union all
select 1320,'Tomahawk','2',0,1,null,500,-6100 from dual union all
select 1321,'Temp','1',0,1,null,150,-6378 from dual union all
select 1322,'RT-21 St1','1',0,1,null,70,-6100 from dual union all
select 1322,'RT-21 St2','2',0,1,null,300,-5500 from dual union all
select 1322,'RT-21 St3','3',0,1,null,1000,-4000 from dual union all
select 1323,'Temp-S','1',0,1,null,150,-6378 from dual union all
select 1324,'Deacon','1',0,1,null,10,-6378 from dual union all
select 1324,'T-55','2',0,1,null,100,-6370 from dual union all
select 1325,'Terrier','1',0,1,null,10,-6378 from dual union all
select 1326,'ASAS XL','2',0,1,null,30,-6378 from dual union all
select 1326,'Terrier Mk 70','1',0,1,null,10,-6378 from dual union all
select 1327,'ASROC','2',0,1,null,30,-6378 from dual union all
select 1327,'Cajun','3',0,1,null,100,-6370 from dual union all
select 1327,'Terrier','1',0,1,null,10,-6378 from dual union all
select 1328,'Asp','2',0,1,null,100,-6370 from dual union all
select 1328,'Terrier','1',0,1,null,20,-6378 from dual union all
select 1329,'Asp IV','2',0,1,null,150,-6370 from dual union all
select 1329,'Terrier','1',0,1,null,20,-6378 from dual union all
select 133,'Astrobee F','1',0,1,null,250,-6370 from dual union all
select 1330,'Hydac','2',0,1,null,400,-6300 from dual union all
select 1330,'Terrier','1',0,1,null,10,-6370 from dual union all
select 1331,'Imp Malemute','2',0,1,1210,500,-6370 from dual union all
select 1331,'Terrier','1',0,1,1133,10,-6378 from dual union all
select 1332,'Imp Malemute','2',0,1,1210,500,-6370 from dual union all
select 1332,'Terrier Mk 70','1',0,1,1133,10,-6378 from dual union all
select 1333,'Imp Malemute','2',0,1,1210,500,-6370 from dual union all
select 1333,'Terrier','1',0,1,1133,10,-6378 from dual union all
select 1334,'Improved Orion','2',0,1,null,200,-6370 from dual union all
select 1334,'Terrier','1',0,1,null,10,-6378 from dual union all
select 1335,'ASAS','3',0,1,null,30,-6378 from dual union all
select 1335,'Mk 30','2',0,1,null,30,-6378 from dual union all
select 1335,'Terrier Mk 70','1',0,1,null,10,-6378 from dual union all
select 1336,'Lynx','2',0,1,null,300,-6370 from dual union all
select 1336,'Terrier','1',0,1,null,20,-6378 from dual union all
select 1337,'Malemute','2',0,1,1210,500,-6370 from dual union all
select 1337,'Terrier','1',0,1,1133,10,-6378 from dual union all
select 1338,'Improved Orion','2',0,1,null,200,-6370 from dual union all
select 1338,'Terrier','1',0,1,null,10,-6378 from dual union all
select 1339,'Improved Orion','2',0,1,null,200,-6370 from dual union all
select 1339,'Terrier Mk 70','1',0,1,null,10,-6378 from dual union all
select 134,'Atea-1','1',0,1,null,100,-6378 from dual union all
select 1340,'Oriole','2',0,1,null,368,-6378 from dual union all
select 1340,'Terrier Mk 70','1',0,1,null,10,-6378 from dual union all
select 1341,'Improved Orion','2',0,1,null,200,-6370 from dual union all
select 1341,'Terrier Mk 70','1',0,1,null,10,-6378 from dual union all
select 1342,'Peregrine','2',0,1,null,300,-6378 from dual union all
select 1342,'Terrier Mk 70','1',0,1,null,12,-6378 from dual union all
select 1343,'Nike','2',0,1,null,2,-6378 from dual union all
select 1343,'Terrier','1',0,1,null,1,-6378 from dual union all
select 1344,'Oriole','2',0,1,null,368,-6378 from dual union all
select 1344,'Terrier','1',0,1,null,10,-6378 from dual union all
select 1345,'Orion','2',0,1,null,150,-6370 from dual union all
select 1345,'Terrier','1',0,1,null,10,-6378 from dual union all
select 1346,'Recruit','2',0,1,null,10,-6378 from dual union all
select 1346,'Terrier','1',0,1,null,10,-6378 from dual union all
select 1347,'Sandhawk','2',0,1,null,400,-6370 from dual union all
select 1347,'Terrier','1',0,1,null,10,-6378 from dual union all
select 1348,'Oriole','3',0,1,null,368,-6378 from dual union all
select 1348,'Terrier','1',0,1,null,10,-6378 from dual union all
select 1348,'Terrier','2',0,1,null,10,-6378 from dual union all
select 1349,'Terrier','1',0,1,null,10,-6378 from dual union all
select 1349,'Tomahawk','2',0,1,null,400,-6370 from dual union all
select 135,'Alcor','3',0,1,null,600,-5100 from dual union all
select 135,'BE-3A4','4',0,1,null,2000,-4000 from dual union all
select 135,'Castor','1',0,1,null,45,-6300 from dual union all
select 135,'Recruit','0',0,2,null,1,-6378 from dual union all
select 135,'TX-261-2','2',0,1,null,200,-6250 from dual union all
select 1350,'Terrier','1',0,1,null,20,-6370 from dual union all
select 1350,'Viper IC','2',0,1,null,150,-6360 from dual union all
select 1351,'NOTS 551','2',0,1,null,400,-6378 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 1351,'Terrier','1',0,1,null,5,-6378 from dual union all
select 1352,'AJ10-40','2',0,1,null,1000,-5800 from dual union all
select 1352,'Thor DM-18A','1',0,1,null,500,-6000 from dual union all
select 1353,'AJ10-41','2',0,1,null,2500,-3500 from dual union all
select 1353,'Altair','3',0,1,null,300000,-1300 from dual union all
select 1353,'Thor DM-18A','1',0,1,null,1000,-5500 from dual union all
select 1354,'AJ10-42','2',0,1,null,700,-4300 from dual union all
select 1354,'Altair','3',0,1,null,700,700 from dual union all
select 1354,'Thor DM-18A','1',0,1,null,200,-6100 from dual union all
select 1355,'AJ10-42','2',0,1,null,500,-6000 from dual union all
select 1355,'Thor DM-18A','1',0,1,null,500,-6000 from dual union all
select 1356,'AJ10','2',0,1,null,250,-1000 from dual union all
select 1356,'Altair','3',0,1,null,42000,250 from dual union all
select 1356,'Thor DM-18A','1',0,1,null,500,-5500 from dual union all
select 1357,'AJ10','2',0,1,null,500,-1000 from dual union all
select 1357,'Altair','3',0,1,null,200,200 from dual union all
select 1357,'Thor DM-18A','1',0,1,null,500,-5500 from dual union all
select 1358,'AJ10-104D','2',0,1,null,900,900 from dual union all
select 1358,'Thor DSV-2A','1',0,1,null,200,-5500 from dual union all
select 1359,'AJ10-104','2',0,1,null,900,900 from dual union all
select 1359,'Thor DM-21','1',0,1,null,200,-5500 from dual union all
select 136,'Alcor IA','3',0,1,null,2000,-4000 from dual union all
select 136,'Antares 2','2',0,1,null,200,-6250 from dual union all
select 136,'Castor 4','1',0,1,null,60,-6300 from dual union all
select 136,'Recruit','0',0,4,null,1,-6378 from dual union all
select 1360,'Agena A','2',0,1,null,400,200 from dual union all
select 1360,'Thor DM-18A','1',0,1,null,200,-5450 from dual union all
select 1361,'Agena B','2',0,1,null,200,200 from dual union all
select 1361,'Thor DM-21','1',0,1,null,150,-5500 from dual union all
select 1362,'Agena D','2',0,1,null,200,200 from dual union all
select 1362,'Thor DM-21','1',0,1,null,150,-5500 from dual union all
select 1363,'FW4S','2',0,1,null,800,800 from dual union all
select 1363,'Thor DM-18A','1',0,1,null,800,-5400 from dual union all
select 1364,'Star 37B','2',0,1,null,850,850 from dual union all
select 1364,'Thor DM-18A','1',0,1,null,850,-5650 from dual union all
select 1365,'Star 26B','3',0,1,null,800,800 from dual union all
select 1365,'Star 37B','2',0,1,null,800,-3000 from dual union all
select 1365,'Thor DM-18A','1',0,1,null,500,-5050 from dual union all
select 1366,'Thor DM-18','1',0,1,null,500,-5700 from dual union all
select 1367,'Thor DM-18A','1',0,1,null,500,-5700 from dual union all
select 1368,'Thor DM-18C','1',0,1,null,500,-5300 from dual union all
select 1369,'Thor DM-21','1',0,1,null,1400,-6300 from dual union all
select 137,'Castor 120','1',0,1,133774,140,-5900 from dual union all
select 137,'OAM','3',0,1,500,800,240 from dual union all
select 137,'Orbus 21D','2',0,1,28050,200,-340 from dual union all
select 1370,'Thor DM-18A','1',0,1,null,500,-6200 from dual union all
select 1371,'Thor DM-18A','1',0,1,null,50,-5200 from dual union all
select 1372,'AJ10-118','2',0,1,null,60,-4000 from dual union all
select 1372,'Thor DM-18A','1',0,1,null,50,-5200 from dual union all
select 1373,'Thor DM-18A','1',0,1,null,500,-6200 from dual union all
select 1374,'Star 37XE','2',0,1,null,820,-4130 from dual union all
select 1374,'Thor DM-18A','1',0,1,null,760,-5480 from dual union all
select 1375,'AJ10-118','2',0,1,null,300,-2000 from dual union all
select 1375,'Altair','3',0,1,null,10000,300 from dual union all
select 1375,'Thor DM-18A','1',0,1,null,200,-5030 from dual union all
select 1376,'AJ10-118','2',0,1,null,300,-2400 from dual union all
select 1376,'Altair','3',0,1,null,300,300 from dual union all
select 1376,'Thor DM-21','1',0,1,null,200,-5030 from dual union all
select 1377,'AJ10-118A','2',0,1,null,1200,-5000 from dual union all
select 1377,'Altair','3',0,1,null,1200,1200 from dual union all
select 1377,'Thor DM-21','1',0,1,null,150,-5500 from dual union all
select 1378,'AJ10-118D','2',0,1,null,200,-1000 from dual union all
select 1378,'Altair 2','3',0,1,null,200,200 from dual union all
select 1378,'Thor DSV-2A','1',0,1,null,150,-5500 from dual union all
select 1379,'AJ10-118D','2',0,1,null,200,-1000 from dual union all
select 1379,'FW4D','3',0,1,null,200,200 from dual union all
select 1379,'Thor DSV-2A','1',0,1,null,150,-5500 from dual union all
select 138,'Castor 120','1',0,1,133774,20,-6300 from dual union all
select 138,'Castor 120','2',0,1,133774,180,-5600 from dual union all
select 138,'OAM','4',0,1,500,600,200 from dual union all
select 138,'Orbus 21D','3',0,1,28050,400,200 from dual union all
select 1380,'AJ10-118D','2',0,1,null,1400,-1000 from dual union all
select 1380,'Altair 2','3',0,1,null,35780,200 from dual union all
select 1380,'Castor','0',0,3,null,30,-6340 from dual union all
select 1380,'Thor DSV-2C','1',0,1,null,150,-5500 from dual union all
select 1381,'Altair 2','3',0,1,null,35780,200 from dual union all
select 1381,'Castor','0',0,3,null,30,-6340 from dual union all
select 1381,'DSV-3E-3','2',0,1,null,200,200 from dual union all
select 1381,'Thor DSV-2C','1',0,1,null,200,-5350 from dual union all
select 1382,'Castor','0',0,3,null,30,-6340 from dual union all
select 1382,'DSV-3E-3','2',0,1,null,200,200 from dual union all
select 1382,'FW4D','3',0,1,null,35780,200 from dual union all
select 1382,'Thor DSV-2C','1',0,1,null,200,-5350 from dual union all
select 1383,'Castor','0',0,3,null,30,-6340 from dual union all
select 1383,'DSV-3E-3','2',0,1,null,200,200 from dual union all
select 1383,'Thor DSV-2C','1',0,1,null,200,-5350 from dual union all
select 1384,'Castor','0',0,3,null,30,-6340 from dual union all
select 1384,'DSV-3E-3','2',0,1,null,640,-6300 from dual union all
select 1384,'Star 37D','3',0,1,null,5000,640 from dual union all
select 1384,'Thor DSV-2C','1',0,1,null,200,-5350 from dual union all
select 1385,'Castor 2','0',0,3,null,30,-6340 from dual union all
select 1385,'DSV-3E-3','2',0,1,null,200,200 from dual union all
select 1385,'FW4D','3',0,1,null,200,200 from dual union all
select 1385,'LT Thor DSV-2L-1B','1',0,1,null,150,-4800 from dual union all
select 1386,'Castor 2','0',0,3,null,30,-6340 from dual union all
select 1386,'DSV-3E-3','2',0,1,null,200,200 from dual union all
select 1386,'LT Thor DSV-2L-1B','1',0,1,null,150,-4800 from dual union all
select 1386,'Star 37D','3',0,1,null,37800,200 from dual union all
select 1387,'Castor 2','0',0,6,null,30,-6340 from dual union all
select 1387,'DSV-3E-3','2',0,1,null,200,-1000 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 1387,'LT Thor DSV-2L-1C','1',0,1,null,150,-4800 from dual union all
select 1387,'Star 37D','3',0,1,null,37800,200 from dual union all
select 1388,'Castor 2','0',0,3,null,30,-6340 from dual union all
select 1388,'DSV-3E-3','2',0,1,null,200,200 from dual union all
select 1388,'LT Thor DSV-2L-1B','1',0,1,null,150,-4800 from dual union all
select 1389,'Castor 2','0',0,6,null,30,-6340 from dual union all
select 1389,'DSV-3E-3','2',0,1,null,200,200 from dual union all
select 1389,'LT Thor DSV-2L-1C','1',0,1,null,150,-4800 from dual union all
select 139,'Atlas III','1',0,1,788880,200,-5800 from dual union all
select 139,'Centaur IIIA','2',0,1,73408,2000,200 from dual union all
select 1390,'MG-18','2',0,1,null,700,700 from dual union all
select 1390,'Thor DM-18A','1',0,1,null,700,-5200 from dual union all
select 1391,'Agena B','2',0,1,null,200,200 from dual union all
select 1391,'Thor DSV-2A','1',0,1,null,150,-5800 from dual union all
select 1392,'Agena D','2',0,1,null,200,200 from dual union all
select 1392,'Thor DSV-2A','1',0,1,null,150,-5800 from dual union all
select 1393,'Agena B','2',0,1,null,200,200 from dual union all
select 1393,'Castor 1','0',0,3,null,45,-6000 from dual union all
select 1393,'Thor DSV-2C','1',0,1,null,150,-5700 from dual union all
select 1394,'Agena D','2',0,1,null,200,200 from dual union all
select 1394,'Castor 1','0',0,3,null,45,-6000 from dual union all
select 1394,'Thor DSV-2C','1',0,1,null,150,-5700 from dual union all
select 1395,'Agena D','2',0,1,null,200,200 from dual union all
select 1395,'Castor 2','0',0,3,null,45,-6000 from dual union all
select 1395,'LT Thor DSV-2L-1A','1',0,1,null,150,-5520 from dual union all
select 1396,'Agena D','2',0,1,null,200,200 from dual union all
select 1396,'Castor 2','0',0,3,null,45,-6000 from dual union all
select 1396,'LT Thor DSV-2L-1A','1',0,1,null,150,-5520 from dual union all
select 1397,'Tianying-3C','1',0,1,null,200,-6378 from dual union all
select 1398,'Tianying-3C','1',0,1,null,200,-6378 from dual union all
select 1399,'Tianying-3C','2',0,1,null,320,-6378 from dual union all
select 1399,'Tianying-3F Booster','1',0,1,null,10,-6378 from dual union all
select 14,'F-104A Starfighter','0',0,1,null,0,-6378 from dual union all
select 14,'Viper I','1',0,1,null,70,-6378 from dual union all
select 140,'Atlas III','1',0,1,null,200,-5800 from dual union all
select 140,'Centaur IIIB/DEC','2',0,1,null,2000,200 from dual union all
select 1400,'BER','1',0,1,null,20,-6370 from dual union all
select 1400,'P064','3',0,1,null,150,-5000 from dual union all
select 1400,'Tacite','2',0,1,null,150,-6300 from dual union all
select 1401,'Tiny Tim','1',0,1,null,1,-6378 from dual union all
select 1402,'Agena D','3',0,1,null,200,200 from dual union all
select 1402,'LR87AJ11','1',0,1,null,90,-6300 from dual union all
select 1402,'LR91AJ11','2',0,1,null,130,-5000 from dual union all
select 1403,'Agena D','3',0,1,null,200,200 from dual union all
select 1403,'LR87AJ11','1',0,1,null,90,-6300 from dual union all
select 1403,'LR91AJ11','2',0,1,null,130,-5000 from dual union all
select 1404,'Agena D','3',0,1,null,200,200 from dual union all
select 1404,'LR87AJ11','1',0,1,null,90,-6300 from dual union all
select 1404,'LR91AJ11','2',0,1,null,130,-5000 from dual union all
select 1405,'Agena D','3',0,1,null,200,200 from dual union all
select 1405,'LR87AJ11','1',0,1,null,90,-6300 from dual union all
select 1405,'LR91AJ11','2',0,1,null,130,-5000 from dual union all
select 1406,'LR87AJ11','1',0,1,null,140,-5000 from dual union all
select 1406,'LR91AJ11','2',0,1,null,200,200 from dual union all
select 1406,'UA1206','0',0,2,null,45,-6200 from dual union all
select 1407,'IUS SRM-1','3',0,1,null,200,200 from dual union all
select 1407,'IUS SRM-2','4',0,1,null,200,200 from dual union all
select 1407,'LR87AJ11','1',0,1,null,140,-5000 from dual union all
select 1407,'LR91AJ11','2',0,1,null,200,200 from dual union all
select 1407,'UA1206','0',0,2,null,45,-6200 from dual union all
select 1408,'LR87AJ11','1',0,1,null,140,-5000 from dual union all
select 1408,'LR91AJ11','2',0,1,null,200,200 from dual union all
select 1408,'Transtage','3',0,1,null,200,200 from dual union all
select 1408,'UA1206','0',0,2,null,45,-6200 from dual union all
select 1409,'Centaur T','3',0,1,null,200,200 from dual union all
select 1409,'LR87AJ11A','1',0,1,null,160,-5000 from dual union all
select 1409,'LR91AJ11A','2',0,1,null,220,-3000 from dual union all
select 1409,'UA1207','0',0,2,null,60,-6200 from dual union all
select 141,'Atlas III','1',0,1,null,200,-5800 from dual union all
select 141,'Centaur IIIB/SEC','2',0,1,null,2000,200 from dual union all
select 1410,'Centaur T','3',0,1,null,220,220 from dual union all
select 1410,'LR87AJ11A','1',0,1,null,185,-5000 from dual union all
select 1410,'LR91AJ11A','2',0,1,null,220,-2200 from dual union all
select 1410,'SRMU','0',0,2,null,75,-6200 from dual union all
select 1411,'IUS SRM-1','3',0,1,null,200,200 from dual union all
select 1411,'IUS SRM-2','4',0,1,null,200,200 from dual union all
select 1411,'LR87AJ11A','1',0,1,null,160,-5000 from dual union all
select 1411,'LR91AJ11A','2',0,1,null,200,200 from dual union all
select 1411,'UA1207','0',0,2,null,60,-6200 from dual union all
select 1412,'IUS SRM-1','3',0,1,null,200,200 from dual union all
select 1412,'IUS SRM-2','4',0,1,null,200,200 from dual union all
select 1412,'LR87AJ11A','1',0,1,null,180,-5000 from dual union all
select 1412,'LR91AJ11A','2',0,1,null,200,200 from dual union all
select 1412,'SRMU','0',0,2,null,75,-6200 from dual union all
select 1413,'LR87AJ11A','1',0,1,null,160,-5000 from dual union all
select 1413,'LR91AJ11A','2',0,1,null,200,200 from dual union all
select 1413,'UA1207','0',0,2,null,60,-6200 from dual union all
select 1414,'LR87AJ11A','1',0,1,null,160,-5000 from dual union all
select 1414,'LR91AJ11A','2',0,1,null,200,200 from dual union all
select 1414,'SRMU','0',0,2,null,75,-6200 from dual union all
select 1415,'LR87AJ11A','1',0,1,null,160,-5000 from dual union all
select 1415,'LR91AJ11A','2',0,1,null,200,200 from dual union all
select 1415,'UA1207','0',0,2,null,60,-6200 from dual union all
select 1416,'LR87AJ11A','1',0,1,null,160,-5000 from dual union all
select 1416,'LR91AJ11A','2',0,1,null,200,200 from dual union all
select 1416,'SRMU','0',0,2,null,75,-6200 from dual union all
select 1417,'LR87AJ11A','1',0,1,null,160,-5000 from dual union all
select 1417,'LR91AJ11A','2',0,1,null,200,200 from dual union all
select 1417,'UA1207','0',0,2,null,60,-6200 from dual union all
select 1418,'LR87AJ11A','1',0,1,null,160,-5000 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 1418,'LR91AJ11A','2',0,1,null,200,200 from dual union all
select 1418,'SRMU','0',0,2,null,75,-6200 from dual union all
select 1419,'LR87','1',0,1,null,100,-5950 from dual union all
select 1419,'LR91','2',0,1,null,1000,-3200 from dual union all
select 142,'Atlas A Booster','0',0,1,null,120,-6100 from dual union all
select 142,'Atlas A','1',0,1,null,120,-6100 from dual union all
select 1420,'LR87AJ5','1',0,1,null,110,-5950 from dual union all
select 1420,'LR91AJ5','2',0,1,null,1300,-3000 from dual union all
select 1421,'LR87AJ5','1',0,1,null,120,-5950 from dual union all
select 1421,'LR91AJ5','2',0,1,null,180,180 from dual union all
select 1422,'LR87AJ5','1',0,1,null,120,-5950 from dual union all
select 1422,'LR91AJ5','2',0,1,null,200,200 from dual union all
select 1423,'LR87AJ9','1',0,1,null,90,-6300 from dual union all
select 1423,'LR91AJ9','2',0,1,null,160,-5500 from dual union all
select 1423,'Transtage','3',0,1,null,200,200 from dual union all
select 1424,'Agena D','3',0,1,null,200,200 from dual union all
select 1424,'LR87AJ9','1',0,1,null,90,-6300 from dual union all
select 1424,'LR91AJ9','2',0,1,null,130,-5000 from dual union all
select 1425,'LR87AJ11','1',0,1,null,120,-5060 from dual union all
select 1425,'LR91AJ11','2',0,1,null,160,-500 from dual union all
select 1425,'Transtage','3',0,1,null,200,200 from dual union all
select 1425,'UA1205','0',0,2,null,55,-6300 from dual union all
select 1426,'LR87AJ11','1',0,1,null,120,-5060 from dual union all
select 1426,'LR91AJ11','2',0,1,null,200,200 from dual union all
select 1426,'UA1205','0',0,2,null,55,-6300 from dual union all
select 1427,'Centaur D-1T','3',0,1,null,200,200 from dual union all
select 1427,'LR87AJ11','1',0,1,null,120,-5000 from dual union all
select 1427,'LR91AJ11','2',0,1,null,180,-3000 from dual union all
select 1427,'UA1205','0',0,2,null,50,-6300 from dual union all
select 1428,'BER','1',0,1,null,20,-6370 from dual union all
select 1428,'Berenice St2','2',0,1,null,250,-6300 from dual union all
select 1429,'Tochka','1',0,1,null,20,-6378 from dual union all
select 143,'AJ10-101A','2',0,1,null,1290,-1000 from dual union all
select 143,'Altair','3',0,1,null,1000,200 from dual union all
select 143,'Atlas D Booster','0',0,1,null,120,-6100 from dual union all
select 143,'Atlas D','1',0,1,null,1290,-1000 from dual union all
select 1430,'Tochka-U','1',0,1,null,20,-6378 from dual union all
select 1431,'Tomahawk','1',0,1,null,100,-6370 from dual union all
select 1432,'Topaze VE111C','1',0,1,null,100,-6370 from dual union all
select 1433,'Topaze VE111L','1',0,1,null,100,-6370 from dual union all
select 1434,'15Zh58A','1',0,1,null,70,-6300 from dual union all
select 1434,'15Zh58B','2',0,1,null,200,-6000 from dual union all
select 1434,'15Zh58V','3',0,1,null,1000,-4000 from dual union all
select 1435,'15Zh58A','1',0,1,null,70,-6300 from dual union all
select 1435,'15Zh58B','2',0,1,null,200,-6000 from dual union all
select 1435,'15Zh58V','3',0,1,null,1000,-4000 from dual union all
select 1436,'15Zh58MA','1',0,1,null,70,-6300 from dual union all
select 1436,'15Zh58MB','2',0,1,null,200,-6000 from dual union all
select 1436,'15Zh58MV','3',0,1,null,1000,-4000 from dual union all
select 1437,'15Zh58MA','1',0,1,null,70,-6300 from dual union all
select 1437,'15Zh58MB','2',0,1,null,200,-6000 from dual union all
select 1437,'15Zh58MV','3',0,1,null,1000,-4000 from dual union all
select 1438,'Cygnus 5','6',0,1,null,3331,-6289 from dual union all
select 1438,'Lance','3',0,1,null,300,-6338 from dual union all
select 1438,'M-6','1',0,1,null,12,-6378 from dual union all
select 1438,'Nike','2',0,1,null,40,-6378 from dual union all
select 1438,'T-40','4',0,1,null,425,-6361 from dual union all
select 1438,'T-55','5',0,1,null,943,-6369 from dual union all
select 1439,'Altair','3',0,1,507,700,-6370 from dual union all
select 1439,'Castor','1',0,1,8930,58,-6350 from dual union all
select 1439,'Cygnus 15','4',0,1,106,2300,-6320 from dual union all
select 1439,'Cygnus 5','5',0,1,3.8,2000,-6328 from dual union all
select 1439,'Recruit','0',0,2,530,1,-6378 from dual union all
select 1439,'Skat','2',0,1,null,305,-6310 from dual union all
select 144,'Agena A','2',0,1,null,3360,140 from dual union all
select 144,'Atlas D Booster','0',0,1,null,120,-6000 from dual union all
select 144,'Atlas D','1',0,1,null,173,-2226 from dual union all
select 1440,'Altair','3',0,1,507,700,-6368 from dual union all
select 1440,'Castor','1',0,1,8930,58,-6350 from dual union all
select 1440,'Cygnus 15','4',0,1,null,2670,-6360 from dual union all
select 1440,'Lance','2',0,1,null,300,-6310 from dual union all
select 1440,'Recruit','0',0,2,530,1,-6378 from dual union all
select 1441,'Deacon','1',0,3,null,5,-6378 from dual union all
select 1441,'Deacon','2',0,1,null,5,-6378 from dual union all
select 1441,'HPAG','3',0,1,null,10,-6378 from dual union all
select 1442,'Trident C-4 Stage 1','1',0,1,null,40,-6370 from dual union all
select 1442,'Trident C-4 Stage 2','2',0,1,null,150,-6000 from dual union all
select 1442,'Trident C-4 Stage 3','3',0,1,null,1000,-4500 from dual union all
select 1443,'Trident D-5 Stage 1','1',0,1,null,40,-6370 from dual union all
select 1443,'Trident D-5 Stage 2','2',0,1,null,150,-6000 from dual union all
select 1443,'Trident D-5 Stage 3','3',0,1,null,1000,-4500 from dual union all
select 1444,'Deacon','1',0,3,null,10,-6378 from dual union all
select 1445,'Deacon','1',0,3,null,10,-6378 from dual union all
select 1445,'HPAG','2',0,1,null,20,-6378 from dual union all
select 1446,'11S691','1',0,1,null,70,-6000 from dual union all
select 1446,'11S692','2',0,1,null,400,100 from dual union all
select 1447,'8S671','1',0,1,null,70,-6000 from dual union all
select 1447,'8S672','2',0,1,null,400,100 from dual union all
select 1448,'11S691','1',0,1,null,70,-6100 from dual union all
select 1448,'11S692','2',0,1,null,200,-2700 from dual union all
select 1448,'S5M','3',0,1,null,400,400 from dual union all
select 1449,'8S816','1',0,1,null,100,-6100 from dual union all
select 1449,'8S817','2',0,1,null,1000,-4500 from dual union all
select 145,'Agena B','2',0,1,null,500,200 from dual union all
select 145,'Atlas D Booster','0',0,1,null,120,-6100 from dual union all
select 145,'Atlas D','1',0,1,null,180,-5000 from dual union all
select 1450,'8S816','1',0,1,null,100,-6100 from dual union all
select 1450,'8S817','2',0,1,null,1000,-4500 from dual union all
select 1451,'8S816','1',0,1,null,100,-6100 from dual union all
select 1451,'8S817','2',0,1,null,1000,-4500 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 1452,'UR-100NA','1',0,1,null,100,-6100 from dual union all
select 1452,'UR-100NB','2',0,1,null,1000,-4500 from dual union all
select 1453,'UR-100NA','1',0,1,null,100,-6100 from dual union all
select 1453,'UR-100NB','2',0,1,null,1000,-4500 from dual union all
select 1453,'UR-100NU APB','3',0,1,null,1000,-4500 from dual union all
select 1454,'UR-100NA','1',0,1,null,100,-6100 from dual union all
select 1454,'UR-100NB','2',0,1,null,1000,-4500 from dual union all
select 1454,'UR-100NU APB','3',0,1,null,1000,-4500 from dual union all
select 1455,'UR-100NA','1',0,1,null,100,-6100 from dual union all
select 1455,'UR-100NB','2',0,1,null,1000,-4500 from dual union all
select 1455,'UR-100NU APB','3',0,1,null,1000,-4500 from dual union all
select 1456,'8S816','1',0,1,null,100,-6100 from dual union all
select 1456,'8S817','2',0,1,null,1000,-4500 from dual union all
select 1457,'8S811','1',0,1,null,500,-6100 from dual union all
select 1457,'8S812','2',1,1,null,500,-6100 from dual union all
select 1458,'8S810','1',0,1,null,60,-6200 from dual union all
select 1458,'8S811','2',0,1,null,130,-5000 from dual union all
select 146,'Agena D','2',0,1,null,300,300 from dual union all
select 146,'Atlas D Booster','0',0,1,null,130,-5950 from dual union all
select 146,'Atlas D','1',0,1,null,300,-4400 from dual union all
select 1461,'Taepo Dong 2','1',0,1,null,70,-6100 from dual union all
select 1461,'Unha-1 Stage 2','2',0,1,null,1000,-5000 from dual union all
select 1462,'Taepo Dong 2','1',0,1,null,70,-6100 from dual union all
select 1462,'Unha-2 Stage 2','2',0,1,null,220,-5000 from dual union all
select 1462,'Unha-2 Stage 3','3',0,1,null,220,-5000 from dual union all
select 1463,'Taepo Dong 2','1',0,1,null,70,-6100 from dual union all
select 1463,'Unha-2 Stage 2','2',0,1,null,220,-5000 from dual union all
select 1463,'Unha-3 Stage 3','3',0,1,null,220,-5000 from dual union all
select 1466,'HIBEX-U','1',0,1,null,20,-6300 from dual union all
select 1466,'Upstage','2',0,1,null,20,-6300 from dual union all
select 1467,'Apache','2',0,1,null,200,-6300 from dual union all
select 1467,'Ute','1',0,1,null,10,-6378 from dual union all
select 1468,'Tomahawk','2',0,1,null,200,-6300 from dual union all
select 1468,'Ute','1',0,1,null,10,-6378 from dual union all
select 1469,'V-1000 Booster','0',0,1,null,2,-6378 from dual union all
select 1469,'V-1000','1',0,1,null,25,-6300 from dual union all
select 147,'Atlas B Booster','0',0,1,null,120,-6100 from dual union all
select 147,'Atlas B','1',0,1,null,900,-4000 from dual union all
select 1470,'V-2','1',0,1,null,100,-6260 from dual union all
select 1471,'V-750 Booster','0',0,1,null,10,-6378 from dual union all
select 1471,'V-750','1',0,1,null,27,-6378 from dual union all
select 1472,'V-825 Booster','0',0,1,null,20,-6370 from dual union all
select 1472,'V-825','1',0,1,null,180,-6300 from dual union all
select 1473,'Aigle','1',0,1,null,20,-6378 from dual union all
select 1474,'Aigle','1',0,1,null,20,-6378 from dual union all
select 1475,'S-40TM','2',0,1,null,800,-5000 from dual union all
select 1475,'S-43','0',0,4,null,40,-6310 from dual union all
select 1475,'S-43TM','1',0,1,null,120,-6100 from dual union all
select 1475,'S-44','3',0,1,null,800,800 from dual union all
select 1476,'VLS-R1 Strapon','1',0,4,null,10,-6370 from dual union all
select 1476,'VLS-R1','2',1,1,null,10,-6370 from dual union all
select 1477,'VLS-R2 Strapon','0',0,4,null,10,-6370 from dual union all
select 1477,'VLS-R2','1',0,1,null,50,-6370 from dual union all
select 1478,'Sonda 3-1','1',0,1,null,120,-6370 from dual union all
select 1479,'Improved Orion','2',0,1,null,400,-6300 from dual union all
select 1479,'Sonda 3-1','1',0,1,null,120,-6370 from dual union all
select 148,'Atlas Booster','0',0,1,null,120,-6100 from dual union all
select 148,'Atlas SLV-3','1',0,1,null,600,-4000 from dual union all
select 148,'Star 37B','2',0,1,null,600,-2000 from dual union all
select 1480,'Imp Malemute','2',0,1,null,400,-6300 from dual union all
select 1480,'S31','1',0,1,null,20,-6378 from dual union all
select 1481,'Improved Orion','2',0,1,null,400,-6300 from dual union all
select 1481,'S31','1',0,1,null,20,-6378 from dual union all
select 1482,'S-40TM','1',0,1,null,100,-6300 from dual union all
select 1482,'S-44','2',0,1,null,950,-6000 from dual union all
select 1483,'S-40TM','1',0,1,null,100,-6300 from dual union all
select 1483,'S-44M','2',0,1,null,950,-6000 from dual union all
select 1484,'S31','1',0,1,null,20,-6378 from dual union all
select 1484,'Sonda 3-1','2',0,1,null,100,-6370 from dual union all
select 1485,'AJ10','2',0,1,null,650,-5600 from dual union all
select 1485,'Vanguard','1',0,1,null,165,-6000 from dual union all
select 1485,'X-248','3',0,1,null,4000,650 from dual union all
select 1486,'AJ10','2',0,1,null,650,-5600 from dual union all
select 1486,'GCR 33KS2800','3',0,1,null,4000,650 from dual union all
select 1486,'Vanguard','1',0,1,null,165,-6000 from dual union all
select 1487,'AVUM-RD868P','4',0,1,null,700,700 from dual union all
select 1487,'Vega P80FW','1',0,1,null,50,-6000 from dual union all
select 1487,'Zefiro 23','2',0,1,null,150,-5000 from dual union all
select 1487,'Zefiro 9A','3',0,1,null,160,160 from dual union all
select 1488,'AVUM','4',0,1,null,700,700 from dual union all
select 1488,'Vega P80FW','1',0,1,null,50,-6000 from dual union all
select 1488,'Zefiro 23','2',0,1,null,150,-5000 from dual union all
select 1488,'Zefiro 9A','3',0,1,null,160,160 from dual union all
select 1489,'Vesuve','1',0,1,null,20,-6378 from dual union all
select 149,'Atlas Booster','0',0,1,null,120,-6100 from dual union all
select 149,'Atlas F','1',0,1,null,300,-3300 from dual union all
select 149,'Star 26B','3',0,1,null,740,740 from dual union all
select 149,'Star 37B','2',0,1,null,740,-1240 from dual union all
select 1490,'Veronique','1',0,1,null,150,-6300 from dual union all
select 1491,'Veronique 61','1',0,1,null,200,-6300 from dual union all
select 1492,'Veronique 61M','1',0,1,null,200,-6300 from dual union all
select 1493,'Veronique AGI','1',0,1,null,150,-6300 from dual union all
select 1494,'Veronique P2','1',0,1,null,2,-6378 from dual union all
select 1495,'Veronique P6','1',0,1,null,2,-6378 from dual union all
select 1496,'Veronique R','1',0,1,null,2,-6378 from dual union all
select 1497,'R-14','1',0,1,null,1500,-6100 from dual union all
select 1498,'Vesta','1',0,1,null,200,-6300 from dual union all
select 1499,'Viking','1',0,1,null,200,-6300 from dual union all
select 15,'Orion 50S','1',0,1,null,33,-6378 from dual union all
select 15,'Star 37FMV','2',0,1,null,200,-6378 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 150,'Atlas C Booster','0',0,1,null,120,-6100 from dual union all
select 150,'Atlas C','1',0,1,null,900,-4000 from dual union all
select 1500,'Viking Type 9','1',0,1,null,200,-6300 from dual union all
select 1501,'Viper II','1',0,1,null,75,-6370 from dual union all
select 1502,'Viper IIIA','1',0,1,null,100,-6370 from dual union all
select 1503,'Falcon','2',0,1,null,150,-6300 from dual union all
select 1503,'Viper I','1',0,1,null,50,-6370 from dual union all
select 1504,'Viper V','1',0,1,null,100,-6370 from dual union all
select 1505,'Arrow II','2',0,1,null,100,-6370 from dual union all
select 1505,'Viper II','1',0,1,null,10,-6370 from dual union all
select 1506,'Viper II','1',0,1,null,123,-6370 from dual union all
select 1507,'R-29R St 1','1',0,1,null,200,-6100 from dual union all
select 1507,'R-29R St 2','2',0,1,null,1000,-5000 from dual union all
select 1508,'11A57I','3',0,1,null,200,200 from dual union all
select 1508,'8K74','1',0,4,null,60,-5900 from dual union all
select 1508,'8K74A','2',0,1,null,120,-5000 from dual union all
select 1509,'8A92E','3',0,1,null,200,200 from dual union all
select 1509,'8K74','1',0,4,null,60,-5900 from dual union all
select 1509,'8K74A','2',0,1,null,120,-5000 from dual union all
select 151,'AJ10-101A','2',0,1,null,0,-6378 from dual union all
select 151,'Altair','3',0,1,null,0,-6378 from dual union all
select 151,'Atlas C Booster','0',0,1,null,0,-6378 from dual union all
select 151,'Atlas C','1',0,1,null,0,-6378 from dual union all
select 1510,'8A92ME','3',0,1,null,200,200 from dual union all
select 1510,'8K74','1',0,4,null,60,-5900 from dual union all
select 1510,'8K74A','2',0,1,null,120,-5000 from dual union all
select 1511,'8K71','1',0,4,null,60,-5900 from dual union all
select 1511,'8K71A','2',0,1,null,120,-5000 from dual union all
select 1511,'8K72E','3',0,1,null,200,200 from dual union all
select 1512,'8K71','1',0,4,null,60,-5900 from dual union all
select 1512,'8K71A','2',0,1,null,120,-5000 from dual union all
select 1512,'8K72KE','3',0,1,null,200,200 from dual union all
select 1513,'8K71','1',0,4,null,60,-5900 from dual union all
select 1513,'8K71A','2',0,1,null,120,-5000 from dual union all
select 1513,'8K72E','3',0,1,null,200,200 from dual union all
select 1514,'R-29 St 1','1',0,1,null,100,-6100 from dual union all
select 1514,'R-29 St 2','2',0,1,null,1000,-5000 from dual union all
select 1515,'Tiny Tim','0',0,1,null,2,-6378 from dual union all
select 1515,'WAC Corporal','1',1,1,null,2,-6378 from dual union all
select 1516,'Tiny Tim','0',0,1,null,2,-6378 from dual union all
select 1516,'WAC Corporal','1',0,1,null,40,-6378 from dual union all
select 1517,'Tiny Tim','0',0,1,null,2,-6378 from dual union all
select 1517,'WAC Corporal','1',0,1,null,40,-6378 from dual union all
select 1518,'Antares','2',0,1,null,238,-6370 from dual union all
select 1518,'Pollux','1',0,1,null,50,-6370 from dual union all
select 1518,'Recruit','0',0,2,null,2,-6378 from dual union all
select 1519,'Wasserfall','1',0,1,null,10,-6378 from dual union all
select 152,'Atlas D Booster','0',0,1,null,100,-6000 from dual union all
select 152,'Atlas D','1',0,1,null,250,-5800 from dual union all
select 152,'Centaur','2',0,1,null,250,250 from dual union all
select 1520,'NB-52','0',0,1,null,14,-6378 from dual union all
select 1520,'X-15','1',0,1,null,100,-6200 from dual union all
select 1521,'Recruit','2',0,3,null,140,-6000 from dual union all
select 1521,'Recruit','3',0,1,null,140,-5800 from dual union all
select 1521,'TX-20 Sergeant','1',0,1,null,140,-6300 from dual union all
select 1522,'Dummy stage','2',0,1,null,20,-6378 from dual union all
select 1522,'Dummy stage','3',0,1,null,20,-6378 from dual union all
select 1522,'X-201','1',0,1,null,20,-6378 from dual union all
select 1523,'NOTS 124-C','2',0,3,null,20,-6378 from dual union all
select 1523,'NOTS 124-C','3',0,1,null,20,-6378 from dual union all
select 1523,'X-201','1',0,1,null,20,-6378 from dual union all
select 1524,'Dummy stage','2',0,1,null,11,-6378 from dual union all
select 1524,'Dummy stage','3',0,1,null,11,-6378 from dual union all
select 1524,'T-40','1',0,1,null,11,-6378 from dual union all
select 1525,'ATACMS','1',0,1,null,15,-6378 from dual union all
select 1525,'B-52H','0',0,1,null,15,-6378 from dual union all
select 1525,'X-51A','2',0,1,null,20,-6378 from dual union all
select 1526,'X-7','2',0,1,null,30,-6378 from dual union all
select 1526,'X-7A-1 Booster','1',0,1,null,0,-6378 from dual union all
select 1527,'Lance','0',0,2,null,0,-6378 from dual union all
select 1527,'X-7','1',0,1,null,30,-6378 from dual union all
select 1528,'Lance','0',0,2,null,0,-6378 from dual union all
select 1528,'X-7','1',0,1,null,30,-6378 from dual union all
select 1529,'Xaero','1',0,1,null,30,-6378 from dual union all
select 153,'Atlas D Booster','0',0,1,null,100,-6000 from dual union all
select 153,'Atlas D','1',0,1,null,250,-5800 from dual union all
select 153,'Centaur D','2',0,1,null,250,250 from dual union all
select 1530,'15Zh58MA','1',0,1,null,70,-6300 from dual union all
select 1530,'15Zh58MB','2',0,1,null,200,-6000 from dual union all
select 1530,'15Zh58MV','3',0,1,null,1000,-4000 from dual union all
select 1531,'15Zh58MA','1',0,1,null,70,-6300 from dual union all
select 1531,'15Zh58MB','2',0,1,null,200,-6000 from dual union all
select 1531,'15Zh58MV','3',0,1,null,1000,-4000 from dual union all
select 1532,'15Zh58MA','1',0,1,null,70,-6300 from dual union all
select 1532,'15Zh58MB','2',0,1,null,200,-6000 from dual union all
select 1532,'15Zh58MV','3',0,1,null,1000,-4000 from dual union all
select 1533,'ZUR-215','1',0,1,null,27,-6378 from dual union all
select 1534,'Zelzal','1',0,1,null,60,-6378 from dual union all
select 1535,'Cuckoo IA','0',0,1,null,10,-6378 from dual union all
select 1535,'Zenit','1',0,1,null,140,-6370 from dual union all
select 1536,'Zenit','1',0,1,null,145,-6370 from dual union all
select 1537,'11S771','1',0,1,null,150,-6100 from dual union all
select 1537,'11S772','2',0,1,null,700,700 from dual union all
select 1538,'11S771','1',0,1,null,100,-6200 from dual union all
select 1538,'11S772','2',0,1,null,900,900 from dual union all
select 1539,'11S771','1',0,1,null,150,-6100 from dual union all
select 1539,'11S772','2',0,1,null,700,700 from dual union all
select 154,'Atlas D Booster','0',0,1,null,120,-6100 from dual union all
select 154,'Atlas D','1',0,1,null,1000,-2700 from dual union all
select 1540,'11S771','1',0,1,null,150,-6100 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 1540,'11S772','2',0,1,null,700,700 from dual union all
select 1541,'11S771','1',0,1,null,100,-6200 from dual union all
select 1541,'11S772','2',0,1,null,200,-2220 from dual union all
select 1541,'Fregat-SB','3',0,1,null,2000,200 from dual union all
select 1542,'11S771','1',0,1,null,100,-6200 from dual union all
select 1542,'11S772','2',0,1,null,200,-2220 from dual union all
select 1542,'Blok DM-SL','3',0,1,null,2000,200 from dual union all
select 1543,'11S771','1',0,1,null,100,-6200 from dual union all
select 1543,'11S772','2',0,1,null,200,-2220 from dual union all
select 1543,'Blok DM-SLB','3',0,1,null,2000,200 from dual union all
select 1545,'ZN-1','1',0,1,null,70,-6370 from dual union all
select 1546,'ATACMS','1',0,1,null,120,-6300 from dual union all
select 1547,'Zulfiqar','1',0,1,null,60,-6378 from dual union all
select 1548,'R-27','1',0,1,null,500,-6000 from dual union all
select 1549,'R-27','1',0,1,null,500,-6000 from dual union all
select 155,'Atlas E','1',0,1,null,1017,-3500 from dual union all
select 155,'Atlas E/F Booster','0',0,1,null,172,-6031 from dual union all
select 1550,'R-27','1',0,1,null,500,-6000 from dual union all
select 1551,'C-17','0',0,1,null,10,-6378 from dual union all
select 1551,'SR19','1',0,1,null,70,-6100 from dual union all
select 1551,'SR19','2',0,1,null,300,-5500 from dual union all
select 1552,'MLRS','0',0,2,null,5,-6100 from dual union all
select 1552,'SR19','1',0,1,null,70,-6100 from dual union all
select 1552,'SR19','2',0,1,null,300,-5500 from dual union all
select 156,'Atlas E','1',0,1,null,740,-3550 from dual union all
select 156,'Atlas E/F Booster','0',0,1,null,120,-6100 from dual union all
select 156,'Star 20','2',0,1,null,740,740 from dual union all
select 157,'Atlas E','1',0,1,null,1050,-800 from dual union all
select 157,'Atlas E/F Booster','0',0,1,null,120,-6100 from dual union all
select 157,'MSD','2',0,1,null,1000,1000 from dual union all
select 158,'Atlas E','1',0,1,null,800,-2000 from dual union all
select 158,'Atlas E/F Booster','0',0,1,null,120,-6100 from dual union all
select 158,'OIS','2',0,1,null,800,600 from dual union all
select 159,'Atlas E','1',0,1,null,550,-4850 from dual union all
select 159,'Atlas E/F Booster','0',0,1,null,120,-6100 from dual union all
select 159,'Star 48','2',0,1,null,550,-2800 from dual union all
select 159,'Star 48','3',0,1,null,20900,550 from dual union all
select 16,'AMROC IRR','1',0,1,null,1,-6378 from dual union all
select 160,'Atlas E','1',0,1,null,750,-2800 from dual union all
select 160,'Atlas E/F Booster','0',0,1,null,120,-6100 from dual union all
select 160,'Star 37E','2',0,1,null,1800,-1000 from dual union all
select 160,'Star 37E','3',0,1,null,20200,190 from dual union all
select 161,'Atlas E','1',0,1,null,1000,-3000 from dual union all
select 161,'Atlas E/F Booster','0',0,1,null,120,-6100 from dual union all
select 161,'Trident','2',0,1,null,1000,-3000 from dual union all
select 162,'Atlas E/F Booster','0',0,1,null,143,-6110 from dual union all
select 162,'Atlas F','1',0,1,null,1000,-3450 from dual union all
select 163,'Agena D','2',0,1,null,780,100 from dual union all
select 163,'Atlas E/F Booster','0',0,1,null,120,-6100 from dual union all
select 163,'Atlas F','1',0,1,null,780,-3500 from dual union all
select 164,'Atlas E/F Booster','0',0,1,null,120,-6100 from dual union all
select 164,'Atlas F','1',0,1,null,1050,-800 from dual union all
select 164,'MSD','2',0,1,null,1000,1000 from dual union all
select 165,'Atlas E/F Booster','0',0,1,null,120,-6100 from dual union all
select 165,'Atlas F','1',0,1,null,600,-1900 from dual union all
select 165,'OIS','2',0,1,null,600,600 from dual union all
select 166,'Atlas E/F Booster','0',0,1,null,120,-6100 from dual union all
select 166,'Atlas F','1',0,1,null,200,-2450 from dual union all
select 166,'Star 37E','2',0,1,null,13600,190 from dual union all
select 167,'Atlas E/F Booster','0',0,1,null,120,-6100 from dual union all
select 167,'Atlas F','1',0,1,null,750,-2800 from dual union all
select 167,'Star 37E','2',0,1,null,1800,-1000 from dual union all
select 167,'Star 37E','3',0,1,null,20200,190 from dual union all
select 168,'Atlas E/F Booster','0',0,1,null,120,-6100 from dual union all
select 168,'Atlas F','1',0,1,null,1000,-3000 from dual union all
select 168,'Trident','2',0,1,null,1000,-3000 from dual union all
select 169,'Atlas Booster','0',0,1,null,120,-5940 from dual union all
select 169,'Atlas G','1',0,1,null,200,-5300 from dual union all
select 169,'Centaur D-1A','2',0,1,null,200,200 from dual union all
select 17,'Oriole','2',0,1,null,368,-6378 from dual union all
select 17,'Star 20','3',0,1,null,368,-6378 from dual union all
select 17,'Terrier Mk 70','1',0,1,null,10,-6378 from dual union all
select 170,'Atlas Booster','0',0,1,null,120,-6100 from dual union all
select 170,'Atlas H','1',0,1,null,1000,-800 from dual union all
select 171,'Atlas Booster','0',0,1,null,100,-6000 from dual union all
select 171,'Atlas I','1',0,1,null,150,-5900 from dual union all
select 171,'Centaur I','2',0,1,null,200,200 from dual union all
select 172,'Atlas Booster','0',0,1,340725,100,-6000 from dual union all
select 172,'Atlas II','1',0,1,104284,150,-5900 from dual union all
select 172,'Centaur II','2',0,1,null,200,200 from dual union all
select 173,'Atlas Booster','0',0,1,340725,100,-6000 from dual union all
select 173,'Atlas II','1',0,1,104284,150,-5900 from dual union all
select 173,'Centaur IIA','2',0,1,73408,200,200 from dual union all
select 174,'Atlas Booster','-1',0,1,340725,90,-5900 from dual union all
select 174,'Atlas II','1',0,1,104284,200,-5550 from dual union all
select 174,'Castor 4A','0',0,4,97495,50,-6000 from dual union all
select 174,'Centaur IIA','2',0,1,73408,200,200 from dual union all
select 175,'Atlas Booster','0',0,1,null,200,-6150 from dual union all
select 175,'Atlas SLV-3','1',0,1,null,200,200 from dual union all
select 176,'Agena B','2',0,1,null,500,200 from dual union all
select 176,'Atlas Booster','0',0,1,null,120,-6100 from dual union all
select 176,'Atlas SLV-3','1',0,1,null,180,-5000 from dual union all
select 177,'Agena D','2',0,1,null,500,200 from dual union all
select 177,'Atlas Booster','0',0,1,null,120,-6100 from dual union all
select 177,'Atlas SLV-3','1',0,1,null,180,-5000 from dual union all
select 178,'Agena D','2',0,1,null,500,200 from dual union all
select 178,'Atlas Booster','0',0,1,null,120,-6100 from dual union all
select 178,'Atlas SLV-3A','1',0,1,null,180,-5000 from dual union all
select 179,'Atlas Booster','0',0,1,null,100,-6000 from dual union all
select 179,'Atlas SLV-3C','1',0,1,null,250,-5800 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 179,'Centaur D','2',0,1,null,200,200 from dual union all
select 18,'Orion 38','4',0,1,null,740,740 from dual union all
select 18,'Orion 50','2',0,1,null,180,-5000 from dual union all
select 18,'Orion 50','3',0,1,null,714,-3000 from dual union all
select 18,'SR118','1',0,1,null,50,-6200 from dual union all
select 180,'Atlas Booster','0',0,1,null,100,-6000 from dual union all
select 180,'Atlas SLV-3D','1',0,1,null,250,-5800 from dual union all
select 180,'Centaur D-1A','2',0,1,null,200,200 from dual union all
select 181,'Atlas CCB','1',0,1,null,200,-5000 from dual union all
select 181,'Centaur IIIB/C-1','2',0,1,null,1500,200 from dual union all
select 182,'Atlas CCB','1',0,1,null,200,-5000 from dual union all
select 182,'Centaur IIIB/SEC','2',0,1,null,1500,200 from dual union all
select 183,'Atlas CCB','1',0,1,null,350,-3850 from dual union all
select 183,'Atlas V SRM','0',0,1,null,75,-6100 from dual union all
select 183,'Centaur IIIB/C-1','2',0,1,null,4000,200 from dual union all
select 184,'Atlas CCB','1',0,1,null,350,-3850 from dual union all
select 184,'Atlas V SRM','0',0,1,null,75,-6100 from dual union all
select 184,'Centaur IIIB/SEC','2',0,1,null,4000,200 from dual union all
select 185,'Atlas CCB','1',0,1,null,350,-3850 from dual union all
select 185,'Atlas V SRM','0',0,2,null,75,-6100 from dual union all
select 185,'Centaur IIIB/C-1','2',0,1,null,4000,200 from dual union all
select 186,'Atlas CCB','1',0,1,null,350,-3850 from dual union all
select 186,'Atlas V SRM','0',0,2,null,75,-6100 from dual union all
select 186,'Centaur IIIB/SEC','2',0,1,null,4000,200 from dual union all
select 187,'Atlas CCB','1',0,1,null,350,-3850 from dual union all
select 187,'Atlas V SRM','0',0,3,null,75,-6100 from dual union all
select 187,'Centaur IIIB/C-1','2',0,1,null,4000,200 from dual union all
select 188,'Atlas CCB','1',0,1,null,350,-3850 from dual union all
select 188,'Atlas V SRM','0',0,3,null,75,-6100 from dual union all
select 188,'Centaur IIIB/SEC','2',0,1,null,4000,200 from dual union all
select 189,'Atlas CCB','1',0,1,null,200,-5000 from dual union all
select 189,'Centaur IIIB/C-1','2',0,1,null,1500,200 from dual union all
select 19,'Castor','1',0,1,null,30,-6300 from dual union all
select 19,'Recruit','0',0,2,null,2,-6378 from dual union all
select 19,'TX-261','2',0,1,null,30,-6300 from dual union all
select 190,'Atlas CCB','1',0,1,null,200,-5000 from dual union all
select 190,'Centaur IIIB/SEC','2',0,1,null,1500,200 from dual union all
select 191,'Atlas CCB','1',0,1,null,350,-3850 from dual union all
select 191,'Atlas V SRM','0',0,2,null,75,-6100 from dual union all
select 191,'Centaur IIIB/SEC','2',0,1,null,4000,200 from dual union all
select 192,'Atlas CCB','1',0,1,null,350,-3850 from dual union all
select 192,'Atlas V SRM','0',0,3,null,75,-6100 from dual union all
select 192,'Centaur IIIB/SEC','2',0,1,null,4000,200 from dual union all
select 193,'Atlas CCB','1',0,1,null,350,-3850 from dual union all
select 193,'Atlas V SRM','0',0,4,null,75,-6100 from dual union all
select 193,'Centaur IIIB/C-1','2',0,1,null,4000,200 from dual union all
select 194,'Atlas CCB','1',0,1,null,350,-3850 from dual union all
select 194,'Atlas V SRM','0',0,4,null,75,-6100 from dual union all
select 194,'Centaur IIIB/SEC','2',0,1,null,4000,200 from dual union all
select 195,'Atlas CCB','1',0,1,null,350,-3850 from dual union all
select 195,'Atlas V SRM','0',0,5,null,75,-6100 from dual union all
select 195,'Centaur IIIB/C-1','2',0,1,null,4000,200 from dual union all
select 196,'Atlas CCB','1',0,1,null,350,-3850 from dual union all
select 196,'Atlas V SRM','0',0,5,null,75,-6100 from dual union all
select 196,'Centaur IIIB/SEC','2',0,1,null,4000,200 from dual union all
select 197,'Aurore','1',0,1,null,40,-6370 from dual union all
select 198,'B-611','1',0,1,null,100,-6370 from dual union all
select 199,'BLDT','1',0,1,null,10,-6378 from dual union all
select 2,'48N6','1',0,1,null,40,-6378 from dual union all
select 20,'ASAT','1',0,1,null,70,-6370 from dual union all
select 20,'Star 20','2',0,1,null,1000,-6300 from dual union all
select 200,'BT-310','1',0,1,null,1,-6378 from dual union all
select 201,'Badr-1','1',0,1,null,20,-6378 from dual union all
select 202,'Aurore','2',0,1,null,120,-6300 from dual union all
select 202,'Elan','1',0,1,null,15,-6370 from dual union all
select 203,'Belier','1',0,1,null,60,-6300 from dual union all
select 204,'Belier III','1',0,1,null,120,-6300 from dual union all
select 205,'Belisama','2',0,1,null,90,-6370 from dual union all
select 205,'Elan','1',0,1,null,10,-6370 from dual union all
select 206,'BER','1',0,1,null,8,-6370 from dual union all
select 206,'Berenice St2','2',0,1,null,250,-6300 from dual union all
select 206,'Melanie','4',0,1,null,250,-6300 from dual union all
select 206,'Tramontane','3',0,1,null,250,-6300 from dual union all
select 207,'Black Arrow St2','2',0,1,null,550,-5040 from dual union all
select 207,'Black Arrow','1',0,1,null,92,-6291 from dual union all
select 207,'Waxwing','3',1,1,null,550,-5040 from dual union all
select 208,'Black Arrow St2','2',0,1,null,550,-5040 from dual union all
select 208,'Black Arrow','1',0,1,null,92,-6291 from dual union all
select 208,'Waxwing','3',0,1,null,1580,550 from dual union all
select 209,'Black Brant VC','2',0,1,null,250,-6360 from dual union all
select 209,'Nihka','3',0,1,null,1000,-6300 from dual union all
select 209,'Terrier Mk 70','1',0,1,null,5,-6378 from dual union all
select 21,'Asp','1',0,1,null,30,-6370 from dual union all
select 21,'Baby Sergeant','2',0,1,null,100,-6370 from dual union all
select 210,'Black Brant VB','2',0,1,null,340,-6350 from dual union all
select 210,'Nike','1',0,1,null,5,-6378 from dual union all
select 211,'Black Brant VC','2',0,1,null,340,-6350 from dual union all
select 211,'Nike','1',0,1,null,5,-6378 from dual union all
select 212,'Black Brant VB','2',0,1,null,350,-6350 from dual union all
select 212,'Terrier Mk 70','1',0,1,null,5,-6378 from dual union all
select 213,'Black Brant VC','2',0,1,null,350,-6350 from dual union all
select 213,'Terrier Mk 70','1',0,1,null,5,-6378 from dual union all
select 214,'Black Brant I','1',0,1,null,130,-6370 from dual union all
select 215,'Black Brant VA','1',0,1,null,130,-6370 from dual union all
select 216,'Black Brant IIB','1',0,1,null,130,-6370 from dual union all
select 217,'Black Brant IIIA','1',0,1,null,180,-6370 from dual union all
select 218,'Black Brant IIIB','1',0,1,null,180,-6370 from dual union all
select 219,'Black Brant IIIA','2',0,1,null,800,-6350 from dual union all
select 219,'Black Brant VA','1',0,1,null,100,-6370 from dual union all
select 22,'AS-0','0',0,2,null,30,-6300 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 22,'AS-1','1',0,1,null,50,-6300 from dual union all
select 22,'AS-2','2',0,1,null,110,-5000 from dual union all
select 22,'AS-3','3',0,1,null,430,-3580 from dual union all
select 22,'AS-4','4',0,1,null,430,400 from dual union all
select 220,'Black Brant IIIB','2',0,1,null,800,-6350 from dual union all
select 220,'Black Brant VA','1',0,1,null,100,-6370 from dual union all
select 221,'Black Brant Mk1','2',0,1,null,350,-6350 from dual union all
select 221,'Terrier Mk 70','1',0,1,null,5,-6378 from dual union all
select 222,'Black Brant Mk3','2',0,1,null,350,-6350 from dual union all
select 222,'Terrier Mk 70','1',0,1,null,5,-6378 from dual union all
select 223,'Black Brant Mk4','2',0,1,null,350,-6350 from dual union all
select 223,'Terrier Mk 70','1',0,1,null,5,-6378 from dual union all
select 224,'Black Brant VC','2',0,1,null,350,-6350 from dual union all
select 224,'Terrier','1',0,1,null,5,-6378 from dual union all
select 225,'Black Brant VB','2',0,1,null,350,-6350 from dual union all
select 225,'Terrier','1',0,1,null,5,-6378 from dual union all
select 226,'Black Brant VA','1',0,1,null,180,-6370 from dual union all
select 227,'Black Brant VB','1',0,1,null,180,-6370 from dual union all
select 228,'Black Brant VC','1',0,1,null,180,-6370 from dual union all
select 229,'Black Brant VI','1',0,1,null,80,-6378 from dual union all
select 23,'ATACMS','1',0,1,null,120,-6300 from dual union all
select 230,'Black Brant VII','1',0,1,null,40,-6378 from dual union all
select 231,'Black Brant VB','2',0,1,null,250,-6360 from dual union all
select 231,'Nihka','3',0,1,null,1000,-6300 from dual union all
select 231,'Terrier','1',0,1,null,5,-6378 from dual union all
select 232,'Black Brant VB','2',0,1,null,250,-6360 from dual union all
select 232,'Nihka','3',0,1,null,1000,-6300 from dual union all
select 232,'Terrier','1',0,1,null,5,-6378 from dual union all
select 233,'Black Brant Mk1','3',0,1,null,300,-6350 from dual union all
select 233,'Talos','1',0,1,null,5,-6378 from dual union all
select 233,'Taurus','2',0,1,null,19,-6370 from dual union all
select 234,'Black Brant VB','3',0,1,null,300,-6350 from dual union all
select 234,'Talos','1',0,1,null,5,-6378 from dual union all
select 234,'Taurus','2',0,1,null,19,-6370 from dual union all
select 235,'Black Brant Mk1','3',0,1,null,340,-6340 from dual union all
select 235,'Talos','1',0,1,null,5,-6378 from dual union all
select 235,'Terrier Mk 70','2',0,1,null,19,-6370 from dual union all
select 236,'Black Brant Mk1','3',0,1,null,340,-6340 from dual union all
select 236,'Nihka','4',0,1,null,800,-6270 from dual union all
select 236,'Talos','1',0,1,null,5,-6378 from dual union all
select 236,'Taurus','2',0,1,null,19,-6370 from dual union all
select 237,'Black Brant VB','3',0,1,null,340,-6340 from dual union all
select 237,'Nihka','4',0,1,null,800,-6270 from dual union all
select 237,'Talos','1',0,1,null,5,-6378 from dual union all
select 237,'Taurus','2',0,1,null,19,-6370 from dual union all
select 238,'Black Brant Mk1','3',0,1,null,340,-6340 from dual union all
select 238,'Nihka','4',0,1,null,800,-6270 from dual union all
select 238,'Talos','1',0,1,null,5,-6378 from dual union all
select 238,'Terrier Mk 70','2',0,1,null,19,-6370 from dual union all
select 239,'Black Knight','1',0,1,null,600,-6350 from dual union all
select 24,'RH-ATV','1',0,1,null,60,-6370 from dual union all
select 24,'RH-ATV','2',0,1,null,400,-6370 from dual union all
select 240,'Black Knight','1',0,1,null,600,-6350 from dual union all
select 240,'Cuckoo IB','2',0,1,null,600,-6350 from dual union all
select 241,'Black Knight','1',0,1,null,600,-6350 from dual union all
select 241,'Cuckoo II','2',0,1,null,600,-6350 from dual union all
select 242,'Black Knight 301','1',0,1,null,600,-6350 from dual union all
select 242,'Cuckoo IB','2',0,1,null,600,-6350 from dual union all
select 243,'Black Knight 301','1',0,1,null,600,-6350 from dual union all
select 243,'Cuckoo II','2',0,1,null,600,-6350 from dual union all
select 244,'Black Sparrow','1',0,1,null,40,-6378 from dual union all
select 244,'F-15D','0',0,1,null,10,-6378 from dual union all
select 245,'Algol 1','1',0,1,null,70,-6350 from dual union all
select 245,'Antares','3',0,1,null,1300,-6168 from dual union all
select 245,'Castor','2',0,1,null,270,-6350 from dual union all
select 246,'Algol 1','1',0,1,null,70,-6350 from dual union all
select 246,'Altair','4',0,1,null,2500,-5000 from dual union all
select 246,'Antares','3',0,1,null,900,-6200 from dual union all
select 246,'Castor','2',0,1,null,170,-6340 from dual union all
select 247,'Alcor','3',0,1,null,1000,-4300 from dual union all
select 247,'Antares','2',0,1,null,200,-6300 from dual union all
select 247,'Castor','1',0,1,null,50,-6350 from dual union all
select 247,'Cetus','4',0,1,null,20000,-2000 from dual union all
select 248,'Alcor','3',0,1,null,2200,-6120 from dual union all
select 248,'Antares','2',0,1,null,200,-6300 from dual union all
select 248,'Castor','1',0,1,null,50,-6350 from dual union all
select 249,'Altair','3',0,1,null,1000,-4600 from dual union all
select 249,'Antares','2',0,1,null,200,-6300 from dual union all
select 249,'Castor','1',0,1,null,50,-6350 from dual union all
select 25,'Ababeel St 3','3',0,1,null,300,-6300 from dual union all
select 25,'DF-21','1',0,1,null,80,-6370 from dual union all
select 25,'Shaheen 3 St 2','2',0,1,null,300,-6300 from dual union all
select 250,'Blue Sparrow','1',0,1,null,100,-6378 from dual union all
select 250,'F-15D','0',0,1,null,10,-6378 from dual union all
select 251,'Blue Streak','1',0,1,null,240,-5800 from dual union all
select 252,'Saluki B','1',0,1,null,10,-6370 from dual union all
select 252,'Saluki S','2',0,1,null,10,-6370 from dual union all
select 253,'B-47 Stratojet','0',0,1,null,10,-6378 from dual union all
select 253,'Sergeant','1',0,1,null,100,-6350 from dual union all
select 254,'Altair','2',0,1,null,200,-6300 from dual union all
select 254,'B-47 Stratojet','0',0,1,null,10,-6378 from dual union all
select 254,'Sergeant','1',0,1,null,90,-6350 from dual union all
select 255,'Bomarc A','1',0,1,null,20,-6378 from dual union all
select 256,'Bomarc B','1',0,1,null,20,-6378 from dual union all
select 257,'Arcas','2',0,1,null,100,-6370 from dual union all
select 257,'MARC 14A1','1',0,1,null,2,-6378 from dual union all
select 258,'Arcas','2',0,1,null,100,-6370 from dual union all
select 258,'MARC 42A1','1',0,1,null,2,-6378 from dual union all
select 259,'KN-11','1',0,1,null,1000,-6100 from dual union all
select 259,'KN-11-2','2',0,1,null,1000,-6100 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 26,'Aeolus Booster','1',0,1,null,5,-6370 from dual union all
select 26,'Mayfly','2',0,1,null,80,-6370 from dual union all
select 260,'KN-11','1',0,1,null,1000,-6100 from dual union all
select 260,'KN-11-2','2',0,1,null,1000,-6100 from dual union all
select 261,'15Zh58MA','1',0,1,null,70,-6300 from dual union all
select 261,'15Zh58MB','2',0,1,null,200,-6000 from dual union all
select 261,'15Zh58MV','3',0,1,null,1000,-4000 from dual union all
select 262,'Bull Goose','1',0,1,null,20,-6378 from dual union all
select 263,'Apache','2',0,1,null,41,-6378 from dual union all
select 263,'Bullpup','1',0,1,null,2,-6378 from dual union all
select 264,'Bullpup','1',0,1,null,2,-6378 from dual union all
select 264,'Cajun','2',0,1,null,50,-6378 from dual union all
select 265,'V-2','1',0,1,null,100,-6370 from dual union all
select 265,'WAC Corporal','2',0,1,null,300,-6370 from dual union all
select 266,'Hwasong 6','1',0,1,null,200,-6350 from dual union all
select 267,'Hwasong 6','1',0,1,null,200,-6350 from dual union all
select 268,'Burya booster','1',0,2,null,2,-6378 from dual union all
select 268,'Burya','2',0,1,null,15,-6378 from dual union all
select 269,'CB-250','0',0,1,null,10,-6378 from dual union all
select 269,'LPR-006','1',0,1,null,20,-6378 from dual union all
select 27,'Gosling IV','1',0,1,null,10,-6370 from dual union all
select 27,'Vela','2',0,1,null,150,-6370 from dual union all
select 270,'CB-250','0',0,1,null,10,-6378 from dual union all
select 270,'RH-125','1',0,1,null,20,-6378 from dual union all
select 271,'CSXT','1',0,1,null,100,-6370 from dual union all
select 272,'Cajun','1',0,1,12,12,-6374 from dual union all
select 273,'Cajun','1',0,1,12,12,-6374 from dual union all
select 274,'F4H Phantom','0',0,1,null,10,-6378 from dual union all
select 274,'NOTS 500','1',0,1,null,45,-6378 from dual union all
select 274,'X-248','2',0,1,null,1100,-6300 from dual union all
select 275,'Canopus','1',0,1,null,150,-6350 from dual union all
select 276,'Canopus','1',0,1,null,150,-6350 from dual union all
select 277,'Castor','1',0,1,null,200,-6300 from dual union all
select 277,'Recruit','0',0,2,null,1,-6378 from dual union all
select 278,'Castor','1',0,1,null,100,-6378 from dual union all
select 279,'Canopus','1',0,4,null,70,-6300 from dual union all
select 279,'Canopus','2',0,1,null,400,-6200 from dual union all
select 28,'Aerobee 150','2',0,1,null,100,-6370 from dual union all
select 28,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 280,'Castor 4A','1',0,1,null,500,-6300 from dual union all
select 281,'Castor 4B','1',0,1,null,300,-6370 from dual union all
select 282,'Castor','1',0,1,null,50,-6350 from dual union all
select 282,'Lance','2',0,1,null,500,-6350 from dual union all
select 282,'Recruit','0',0,2,null,1,-6378 from dual union all
select 283,'Castor 4B','1',0,1,null,100,-6370 from dual union all
select 283,'M57A1','2',0,1,null,1060,-6300 from dual union all
select 284,'Castor','1',0,1,null,50,-6370 from dual union all
select 284,'Recruit','2',0,1,null,150,-6300 from dual union all
select 285,'Cedre St1','1',0,1,null,10,-6378 from dual union all
select 285,'Cedre St2','2',0,1,null,10,-6378 from dual union all
select 285,'Cedre St3','3',0,1,null,200,-6378 from dual union all
select 286,'Cedre St2','2',0,1,null,10,-6378 from dual union all
select 286,'Cedre St3','3',0,1,null,200,-6378 from dual union all
select 286,'Cedre-4 St1','1',0,1,null,10,-6378 from dual union all
select 288,'Belier','2',0,1,null,150,-6300 from dual union all
select 288,'Venus','1',0,1,null,10,-6300 from dual union all
select 289,'Belier II','2',0,1,null,150,-6300 from dual union all
select 289,'Venus','1',0,1,null,10,-6300 from dual union all
select 29,'Aerobee 100','2',0,1,null,90,-6370 from dual union all
select 29,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 290,'Belier II','2',0,1,null,150,-6300 from dual union all
select 290,'Venus','1',0,1,null,10,-6300 from dual union all
select 291,'Belier II','2',0,1,null,150,-6300 from dual union all
select 291,'Venus','1',0,1,null,10,-6300 from dual union all
select 292,'Centenario','1',0,1,null,50,-6378 from dual union all
select 293,'DF3-YF2A','1',0,1,null,90,-6200 from dual union all
select 293,'DF4-YF3','2',0,1,null,400,-5300 from dual union all
select 293,'GF-02','3',0,1,null,2400,440 from dual union all
select 294,'CZ-11-1','1',0,1,null,200,-6370 from dual union all
select 294,'CZ-11-2','2',0,1,null,200,-6370 from dual union all
select 294,'CZ-11-3','3',0,1,null,200,-6370 from dual union all
select 294,'CZ-11-4','4',0,1,null,700,700 from dual union all
select 295,'DF3-YF2A','1',0,1,null,90,-6200 from dual union all
select 295,'DF4-YF3','2',0,1,null,500,-5300 from dual union all
select 295,'GF-02','3',0,1,null,500,-5300 from dual union all
select 296,'CZ-2 Stage 1','1',0,1,null,100,-5500 from dual union all
select 296,'CZ-2 Stage 2','2',0,1,null,300,200 from dual union all
select 297,'CZ-2C Stage 1','1',0,1,null,100,-5500 from dual union all
select 297,'CZ-2Cj Stage 2','2',0,1,null,300,200 from dual union all
select 298,'CZ-2C Stage 1','1',0,1,null,100,-5500 from dual union all
select 298,'CZ-2Cj Stage 2','2',0,1,null,300,200 from dual union all
select 299,'CZ-2C Stage 1','1',0,1,null,100,-5500 from dual union all
select 299,'CZ-2Cj Stage 2','2',0,1,null,300,200 from dual union all
select 3,'53T6','1',0,1,null,100,-6300 from dual union all
select 30,'Aerobee 150','2',0,1,null,200,-6370 from dual union all
select 30,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 300,'CZ-2C Stage 1','1',0,1,null,100,-5500 from dual union all
select 300,'CZ-2Cj Stage 2','2',0,1,null,300,200 from dual union all
select 301,'CZ-2C Stage 1','1',0,1,null,100,-5500 from dual union all
select 301,'CZ-2C Stage 2','2',0,1,null,300,200 from dual union all
select 302,'CZ-2C Stage 1','1',0,1,null,85,-6180 from dual union all
select 302,'CZ-2C Stage 2','2',0,1,null,600,180 from dual union all
select 302,'Smart Dispenser','3',0,1,null,600,600 from dual union all
select 303,'CZ-4 Stage 1','1',0,1,null,120,-5500 from dual union all
select 303,'CZ-4 Stage 2j','2',0,1,null,300,170 from dual union all
select 304,'CZ-4 Stage 1','1',0,1,null,120,-5500 from dual union all
select 304,'CZ-4 Stage 2','2',0,1,null,300,170 from dual union all
select 305,'CZ Booster','0',0,4,null,60,-6300 from dual union all
select 305,'CZ-2E Stage 1','1',0,1,null,120,-5500 from dual union all
select 305,'CZ-2E Stage 2','2',0,1,null,1000,200 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 306,'CZ Booster','0',0,4,null,60,-6300 from dual union all
select 306,'CZ-2E Stage 1','1',0,1,null,120,-5500 from dual union all
select 306,'CZ-2E Stage 2','2',0,1,null,200,200 from dual union all
select 307,'CZ Booster','0',0,4,null,60,-6300 from dual union all
select 307,'CZ-2E Stage 1','1',0,1,null,120,-5500 from dual union all
select 307,'CZ-2E Stage 2','2',0,1,null,200,200 from dual union all
select 308,'CZ-3 Stage 1','1',0,1,null,90,-6000 from dual union all
select 308,'CZ-3 Stage 2','2',0,1,null,170,-5100 from dual union all
select 308,'CZ-3 Stage 3','3',0,1,null,200,200 from dual union all
select 309,'CZ-3A Stage 1','1',0,1,null,90,-6000 from dual union all
select 309,'CZ-3A Stage 2','2',0,1,null,170,-5130 from dual union all
select 309,'CZ-3A Stage 3','3',0,1,null,200,200 from dual union all
select 31,'Aerobee 150','2',0,1,null,200,-6370 from dual union all
select 31,'VAM-17','1',0,1,null,10,-6370 from dual union all
select 310,'CZ-3A Stage 3','3',0,1,null,200,200 from dual union all
select 310,'CZ-3B Stage 2','2',0,1,null,200,-4120 from dual union all
select 310,'CZ-3BE Booster','0',0,4,null,105,-6040 from dual union all
select 310,'CZ-3BE Stage 1','1',0,1,null,135,-5900 from dual union all
select 311,'CZ-3A Stage 3','3',0,1,null,200,200 from dual union all
select 311,'CZ-3B Stage 2','2',0,1,null,200,-4120 from dual union all
select 311,'CZ-3BE Booster','0',0,4,null,105,-6040 from dual union all
select 311,'CZ-3BE Stage 1','1',0,1,null,135,-5900 from dual union all
select 312,'CZ-3A Stage 3','3',0,1,null,200,200 from dual union all
select 312,'CZ-3B Stage 2','2',0,1,null,200,-4120 from dual union all
select 312,'CZ-3BE Booster','0',0,4,null,105,-6040 from dual union all
select 312,'CZ-3BE Stage 1','1',0,1,null,135,-5900 from dual union all
select 313,'CZ Booster','0',0,4,null,105,-6040 from dual union all
select 313,'CZ-3A Stage 1','1',0,1,null,135,-5900 from dual union all
select 313,'CZ-3A Stage 3','3',0,1,null,200,200 from dual union all
select 313,'CZ-3B Stage 2','2',0,1,null,200,-4120 from dual union all
select 314,'CZ-3A Stage 3','3',0,1,null,200,200 from dual union all
select 314,'CZ-3B Stage 2','2',0,1,null,200,-4120 from dual union all
select 314,'CZ-3BE Booster','0',0,4,null,105,-6040 from dual union all
select 314,'CZ-3BE Stage 1','1',0,1,null,135,-5900 from dual union all
select 314,'YZ-1','4',0,1,null,200,35780 from dual union all
select 315,'CZ-3A Stage 3','3',0,1,null,200,200 from dual union all
select 315,'CZ-3B Stage 2','2',0,1,null,200,-4120 from dual union all
select 315,'CZ-3BE Booster','0',0,2,null,105,-6040 from dual union all
select 315,'CZ-3BE Stage 1','1',0,1,null,135,-5900 from dual union all
select 316,'CZ Booster','0',0,2,null,105,-6040 from dual union all
select 316,'CZ-3A Stage 1','1',0,1,null,135,-5900 from dual union all
select 316,'CZ-3A Stage 3','3',0,1,null,200,200 from dual union all
select 316,'CZ-3B Stage 2','2',0,1,null,200,-4120 from dual union all
select 317,'CZ-3A Stage 3','3',0,1,null,200,200 from dual union all
select 317,'CZ-3B Stage 2','2',0,1,null,200,-4120 from dual union all
select 317,'CZ-3BE Booster','0',0,2,null,105,-6040 from dual union all
select 317,'CZ-3BE Stage 1','1',0,1,null,135,-5900 from dual union all
select 317,'YZ-1','4',0,1,null,200,35780 from dual union all
select 318,'CZ-4 Stage 1','1',0,1,null,100,-6100 from dual union all
select 318,'CZ-4 Stage 2','2',0,1,null,300,-5000 from dual union all
select 318,'CZ-4 Stage 3','3',0,1,null,900,900 from dual union all
select 319,'CZ-4 Stage 1','1',0,1,null,100,-6100 from dual union all
select 319,'CZ-4B Stage 2','2',0,1,null,300,-5000 from dual union all
select 319,'CZ-4B Stage 3','3',0,1,null,900,900 from dual union all
select 32,'Aerobee 150','2',0,1,null,200,-6370 from dual union all
select 32,'VAM-20','1',0,1,null,10,-6370 from dual union all
select 320,'CZ-4 Stage 1','1',0,1,null,100,-6100 from dual union all
select 320,'CZ-4B Stage 2','2',0,1,null,300,-5000 from dual union all
select 320,'CZ-4C Stage 3','3',0,1,null,900,900 from dual union all
select 321,'CZ-5 Booster','0',0,2,null,50,-6370 from dual union all
select 321,'CZ-5 Stage 1','1',0,1,null,50,-6370 from dual union all
select 321,'CZ-5 Stage 2','2',0,1,null,200,200 from dual union all
select 321,'CZ-K2-1','-1',0,2,null,50,-6370 from dual union all
select 322,'CZ-5 Booster','0',0,4,null,50,-6370 from dual union all
select 322,'CZ-5 Stage 1','1',0,1,null,50,-6370 from dual union all
select 322,'CZ-5 Stage 2','2',0,1,null,200,200 from dual union all
select 323,'CZ-5 Booster','0',0,4,null,50,-6370 from dual union all
select 323,'CZ-5 Stage 1','1',0,1,null,50,-6370 from dual union all
select 323,'CZ-5 Stage 2','2',0,1,null,200,200 from dual union all
select 323,'YZ-2','3',0,1,null,200,35780 from dual union all
select 324,'CZ-K2-2','2',0,1,null,200,200 from dual union all
select 324,'CZ-K2-3','3',0,1,null,200,200 from dual union all
select 324,'CZ6-C','1',0,1,null,50,-6370 from dual union all
select 325,'CZ-7 Stage 1','1',0,1,null,50,-6370 from dual union all
select 325,'CZ-7 Stage 2','2',0,1,null,200,200 from dual union all
select 325,'CZ-K2-1','0',0,4,null,50,-6370 from dual union all
select 326,'CZ-7 Stage 1','1',0,1,null,50,-6370 from dual union all
select 326,'CZ-7 Stage 2','2',0,1,null,200,200 from dual union all
select 326,'CZ-K2-1','0',0,4,null,50,-6370 from dual union all
select 326,'YZ-1A','3',0,1,null,200,200 from dual union all
select 327,'Cirrus I','1',0,1,null,5,-6378 from dual union all
select 327,'Cirrus I','2',0,1,null,35,-6378 from dual union all
select 328,'Cirrus I','2',0,1,null,50,-6378 from dual union all
select 328,'Cirrus II','1',0,1,null,10,-6378 from dual union all
select 329,'Sword','1',0,1,null,40,-6300 from dual union all
select 33,'Aerobee 150','2',0,1,null,200,-6370 from dual union all
select 33,'VAM Booster','1',0,1,null,10,-6370 from dual union all
select 330,'RB-57B','0',0,1,null,10,-6300 from dual union all
select 330,'Shrike','1',0,1,null,51,-6300 from dual union all
select 331,'Gosling','1',0,1,null,20,-6300 from dual union all
select 331,'Lupus','2',0,1,null,150,-6300 from dual union all
select 332,'Gosling','1',0,1,null,20,-6300 from dual union all
select 332,'Lupus 2','2',0,1,null,150,-6300 from dual union all
select 333,'Gosling IV','1',0,1,null,20,-6300 from dual union all
select 333,'Lupus 2','2',0,1,null,150,-6300 from dual union all
select 334,'Gosling IV','1',0,1,null,20,-6300 from dual union all
select 334,'Lupus','2',0,1,null,150,-6300 from dual union all
select 335,'LR87AJ11','1',0,1,null,140,-5000 from dual union all
select 335,'LR91AJ11','2',0,1,null,200,200 from dual union all
select 335,'UA1205','0',0,2,null,45,-6200 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 336,'Castor 4B','1',0,4,null,10,-6370 from dual union all
select 336,'Castor 4B','2',0,2,null,10,-6370 from dual union all
select 336,'Castor 4B','3',0,1,null,10,-6370 from dual union all
select 336,'Star 48V','4',0,1,null,10,-6370 from dual union all
select 337,'Astris','2',1,1,null,55,-6300 from dual union all
select 337,'Coralie','1',0,1,null,55,-6300 from dual union all
select 338,'Dorado','2',0,1,null,200,-6300 from dual union all
select 338,'Gosling IV','1',0,1,null,20,-6370 from dual union all
select 339,'Corporal Type 1','1',0,1,null,30,-6370 from dual union all
select 34,'Aerobee 150','2',0,1,null,200,-6370 from dual union all
select 34,'VAM-20','1',0,1,null,10,-6370 from dual union all
select 340,'Corporal E','1',0,1,null,40,-6370 from dual union all
select 341,'Corporal Type 1','1',0,1,null,30,-6370 from dual union all
select 342,'Terrier Mk 70','1',0,1,null,1,-6378 from dual union all
select 343,'Tomahawk','1',0,1,null,120,-6300 from dual union all
select 344,'DAC Roc','1',0,1,null,50,-6378 from dual union all
select 345,'DC-X','1',0,1,null,2,-6378 from dual union all
select 346,'DC-XA','1',0,1,null,2,-6378 from dual union all
select 347,'DF-1','1',0,1,null,100,-6370 from dual union all
select 348,'DF-11','1',0,1,null,100,-6370 from dual union all
select 349,'DF-11M','1',0,1,null,100,-6370 from dual union all
select 35,'Aerobee 150A','2',0,1,null,200,-6370 from dual union all
select 35,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 350,'DF-15','1',0,1,null,100,-6370 from dual union all
select 351,'DF-16','1',0,1,null,300,-6378 from dual union all
select 352,'DF-16','1',0,1,null,300,-6378 from dual union all
select 353,'DF-2','1',0,1,null,200,-6370 from dual union all
select 354,'DF-21 St2','2',0,1,null,1000,-5000 from dual union all
select 354,'DF-21','1',0,1,null,100,-6300 from dual union all
select 355,'DF-2A','1',0,1,null,200,-6370 from dual union all
select 356,'DF3-YF2A','1',0,1,null,500,-6300 from dual union all
select 357,'DF-31 St1','1',0,1,null,50,-6300 from dual union all
select 357,'DF-31 St2','2',0,1,null,200,-5500 from dual union all
select 357,'DF-31 St3','3',0,1,null,1000,-4000 from dual union all
select 358,'DF3-YF2A','1',0,1,null,500,-6300 from dual union all
select 359,'DF3-YF2A','1',0,1,null,50,-6370 from dual union all
select 359,'DF4-YF3','2',0,1,null,500,-6300 from dual union all
select 36,'Aerobee 150','2',0,1,null,200,-6370 from dual union all
select 36,'VAM Booster','1',0,1,null,10,-6370 from dual union all
select 360,'DF-41 St1','1',0,1,null,50,-6300 from dual union all
select 360,'DF-41 St2','2',0,1,null,200,-5500 from dual union all
select 360,'DF-41 St3','3',0,1,null,1000,-4000 from dual union all
select 361,'DIM','1',0,1,null,70,-6378 from dual union all
select 362,'Castor','1',0,1,null,100,-6370 from dual union all
select 362,'Recruit','0',0,2,null,5,-6378 from dual union all
select 362,'Star 26C','2',0,1,null,300,-6300 from dual union all
select 363,'MD-620','2',0,1,null,50,-6378 from dual union all
select 363,'Melanie','3',0,1,null,130,-6370 from dual union all
select 363,'SPRAN-50','1',0,1,null,10,-6378 from dual union all
select 364,'Stromboli','1',0,1,null,110,-6370 from dual union all
select 365,'Deacon','1',0,1,null,10,-6378 from dual union all
select 366,'Arrow II','2',0,1,null,85,-6378 from dual union all
select 366,'Deacon','1',0,1,null,10,-6378 from dual union all
select 367,'Deacon','1',0,1,null,5,-6378 from dual union all
select 367,'Deacon','2',0,1,null,10,-6378 from dual union all
select 368,'Deacon','1',0,1,null,10,-6378 from dual union all
select 368,'HVAR','2',0,1,null,10,-6378 from dual union all
select 369,'Deacon','1',0,1,null,10,-6378 from dual union all
select 369,'Judi III','2',0,1,null,90,-6378 from dual union all
select 37,'Aerobee 150','2',0,1,928,200,-6370 from dual union all
select 37,'Nike','1',0,1,640,10,-6370 from dual union all
select 370,'Balloon','0',0,1,null,10,-6378 from dual union all
select 370,'Deacon','1',0,1,null,120,-6370 from dual union all
select 371,'Deacon','1',0,1,null,10,-6378 from dual union all
select 371,'HPAG','2',0,1,null,10,-6378 from dual union all
select 372,'Balloon','0',0,1,null,10,-6378 from dual union all
select 372,'Deacon','1',0,1,null,30,-6378 from dual union all
select 372,'Loki','2',0,1,null,100,-6378 from dual union all
select 373,'Castor 2','0',0,3,null,30,-6340 from dual union all
select 373,'DSV-3N-4','2',0,1,null,200,200 from dual union all
select 373,'LT Thor DSV-2L-1C','1',0,1,null,150,-4800 from dual union all
select 374,'Castor 2','0',0,9,null,30,-6340 from dual union all
select 374,'DSV-3N-4','2',0,1,null,200,200 from dual union all
select 374,'LT Thor DSV-2L-1C','1',0,1,null,150,-4800 from dual union all
select 375,'Castor 2','0',0,4,null,30,-6340 from dual union all
select 375,'DSV-3P-4B','2',0,1,null,200,200 from dual union all
select 375,'ELT Thor DSV-2L-1E','1',0,1,null,150,-4800 from dual union all
select 376,'Castor 2','0',0,6,null,30,-6340 from dual union all
select 376,'DSV-3N-4','2',0,1,null,150,150 from dual union all
select 376,'ELT Thor DSV-2L-1D','1',0,1,null,150,-4800 from dual union all
select 376,'Star 37C','3',0,1,null,37800,200 from dual union all
select 377,'Castor 2','0',0,9,null,30,-6340 from dual union all
select 377,'DSV-3N-4','2',0,1,null,200,200 from dual union all
select 377,'ELT Thor DSV-2L-1D','1',0,1,null,150,-4800 from dual union all
select 378,'Castor 2','0',0,9,null,30,-6340 from dual union all
select 378,'DSV-3P-4','2',0,1,null,500,500 from dual union all
select 378,'ELT Thor DSV-2L-1E','1',0,1,null,150,-4800 from dual union all
select 379,'Castor 2','0',0,9,null,30,-6340 from dual union all
select 379,'DSV-3P-4','2',0,1,null,200,-1000 from dual union all
select 379,'ELT Thor DSV-2L-1E','1',0,1,null,150,-4800 from dual union all
select 379,'Star 37D','3',0,1,null,37800,200 from dual union all
select 38,'Aerobee 150','2',0,1,null,200,-6370 from dual union all
select 38,'Nike','1',0,1,null,10,-6370 from dual union all
select 380,'Castor 2','0',0,9,null,30,-6340 from dual union all
select 380,'DSV-3P-4','2',0,1,null,200,-1000 from dual union all
select 380,'ELT Thor DSV-2L-1E','1',0,1,null,150,-4800 from dual union all
select 380,'Star 37C','3',0,1,null,37800,200 from dual union all
select 381,'Castor 2','0',0,3,null,30,-6340 from dual union all
select 381,'DSV-3P-4B','2',0,1,null,1000,1000 from dual union all
select 381,'ELT Thor DSV-2W-1','1',0,1,null,150,-4800 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 382,'Castor 2','0',0,3,null,32,-6340 from dual union all
select 382,'DSV-3P-4B','2',0,1,null,200,-500 from dual union all
select 382,'ELT Thor DSV-2W-1','1',0,1,null,150,-4800 from dual union all
select 382,'Star 37D','3',0,1,null,37800,200 from dual union all
select 383,'Castor 2','0',0,9,null,32,-6340 from dual union all
select 383,'DSV-3P-4B','2',0,1,null,200,200 from dual union all
select 383,'ELT Thor DSV-2W-1','1',0,1,null,150,-4800 from dual union all
select 384,'Castor 2','0',0,9,null,32,-6340 from dual union all
select 384,'DSV-3P-4B','2',0,1,null,200,200 from dual union all
select 384,'ELT Thor DSV-2W-1','1',0,1,null,160,-4830 from dual union all
select 384,'Star 37D','3',0,1,null,200,200 from dual union all
select 385,'Castor 2','0',0,9,null,32,-6340 from dual union all
select 385,'DSV-3P-4B','2',0,1,null,170,-4250 from dual union all
select 385,'ELT Thor DSV-2W-1','1',0,1,null,200,-1000 from dual union all
select 385,'Star 37C','3',0,1,null,35780,230 from dual union all
select 386,'Castor 2','0',0,9,null,32,-6340 from dual union all
select 386,'DSV-3P-4B','2',0,1,null,231,-500 from dual union all
select 386,'ELT Thor DSV-2W-1','1',0,1,null,160,-4830 from dual union all
select 386,'Star 37E','3',0,1,null,35780,230 from dual union all
select 387,'Castor 4','0',0,9,null,70,-6100 from dual union all
select 387,'DSV-3P-4B','2',0,1,null,200,200 from dual union all
select 387,'ELT Thor DSV-2W','1',0,1,null,150,-4100 from dual union all
select 388,'Castor 4','0',0,9,null,75,-6100 from dual union all
select 388,'DSV-3P-4B','2',0,1,null,300,-750 from dual union all
select 388,'ELT Thor DSV-2W','1',0,1,null,150,-4200 from dual union all
select 388,'Star 48B','3',0,1,null,35900,300 from dual union all
select 389,'Castor 4','0',0,9,null,70,-6100 from dual union all
select 389,'DSV-3P-4B','2',0,1,null,200,200 from dual union all
select 389,'ELT Thor DSV-2W','1',0,1,null,150,-4100 from dual union all
select 389,'Star 37N','3',0,1,null,24900,200 from dual union all
select 39,'Aerobee 150','2',0,1,null,200,-6370 from dual union all
select 39,'Nike','1',0,1,null,10,-6370 from dual union all
select 390,'Castor 4','0',0,9,null,70,-6100 from dual union all
select 390,'DSV-3P-4B','2',0,1,null,200,200 from dual union all
select 390,'ELT Thor DSV-2W','1',0,1,null,150,-4100 from dual union all
select 390,'Star 37E','3',0,1,null,35780,200 from dual union all
select 391,'Castor 4','0',0,9,null,70,-6100 from dual union all
select 391,'Delta/0020','2',0,1,null,200,200 from dual union all
select 391,'ELT Thor DSV-2W','1',0,1,null,150,-4100 from dual union all
select 392,'Castor 4','0',0,9,null,70,-6100 from dual union all
select 392,'Delta/0020','2',0,1,null,200,200 from dual union all
select 392,'ELT Thor DSV-2W','1',0,1,null,150,-4100 from dual union all
select 393,'Castor 4','0',0,9,null,70,-6100 from dual union all
select 393,'Delta/0020','2',0,1,null,200,200 from dual union all
select 393,'ELT Thor DSV-2W','1',0,1,null,150,-4100 from dual union all
select 393,'Star 48B','3',0,1,null,35780,200 from dual union all
select 394,'Castor 4','0',0,9,null,75,-6100 from dual union all
select 394,'Delta/0020','2',0,1,null,500,170 from dual union all
select 394,'ELT Thor DSV-2W','1',0,1,null,145,-4200 from dual union all
select 394,'Star 37E','3',0,1,null,35786,170 from dual union all
select 395,'Castor 4A','0',0,9,null,70,-6100 from dual union all
select 395,'Delta/0020','2',0,1,null,200,200 from dual union all
select 395,'ELT Thor DSV-2L','1',0,1,null,150,-4100 from dual union all
select 395,'Star 48B','3',0,1,null,35780,200 from dual union all
select 396,'Delta CBC RS-68A','0',0,2,null,100,-5800 from dual union all
select 396,'Delta CBC RS-68A','1',0,1,null,200,-4400 from dual union all
select 396,'Delta D4SS-5','2',0,1,null,200,200 from dual union all
select 397,'Delta CBC','0',0,2,null,100,-5800 from dual union all
select 397,'Delta CBC','1',0,1,null,200,-4400 from dual union all
select 397,'Delta D4SS-5','2',0,1,null,200,200 from dual union all
select 398,'Delta CBC','1',0,1,null,180,-4900 from dual union all
select 398,'Delta D4SS-4','2',0,1,null,200,200 from dual union all
select 399,'Delta CBC RS-68A','1',0,1,null,180,-4900 from dual union all
select 399,'Delta D4SS-4','2',0,1,null,200,200 from dual union all
select 399,'GEM-60','0',0,2,null,40,-6370 from dual union all
select 40,'AJ60-92','2',0,1,null,250,-6370 from dual union all
select 40,'Nike','1',0,1,null,10,-6370 from dual union all
select 400,'Delta CBC','1',0,1,null,180,-4900 from dual union all
select 400,'Delta D4SS-4','2',0,1,null,200,200 from dual union all
select 400,'GEM-60','0',0,2,null,40,-6370 from dual union all
select 401,'Delta CBC RS-68A','1',0,1,null,180,-4900 from dual union all
select 401,'Delta D4SS-5','2',0,1,null,200,200 from dual union all
select 401,'GEM-60','0',0,2,null,40,-6370 from dual union all
select 402,'Delta CBC','1',0,1,null,180,-4900 from dual union all
select 402,'Delta D4SS-5','2',0,1,null,200,200 from dual union all
select 402,'GEM-60','0',0,2,null,40,-6370 from dual union all
select 403,'Delta CBC RS-68A','1',0,1,null,180,-4900 from dual union all
select 403,'Delta D4SS-5','2',0,1,null,200,200 from dual union all
select 403,'GEM-60','0',0,4,null,40,-6370 from dual union all
select 404,'Delta CBC','1',0,1,null,180,-4900 from dual union all
select 404,'Delta D4SS-5','2',0,1,null,200,200 from dual union all
select 404,'GEM-60','0',0,4,null,40,-6370 from dual union all
select 405,'Castor 4A','0',0,9,null,70,-6100 from dual union all
select 405,'Delta/0020','2',0,1,null,200,200 from dual union all
select 405,'ELT Thor DSV-2W','1',0,1,null,150,-4100 from dual union all
select 406,'Castor 4A','0',0,9,null,70,-6100 from dual union all
select 406,'Delta/0020','2',0,1,null,200,200 from dual union all
select 406,'EELT Thor','1',0,1,null,150,-4100 from dual union all
select 407,'Castor 4A','0',0,9,null,70,-6100 from dual union all
select 407,'Delta/0020','2',0,1,null,200,200 from dual union all
select 407,'EELT Thor','1',0,1,null,150,-4100 from dual union all
select 408,'Castor 4A','0',0,9,null,70,-6100 from dual union all
select 408,'Delta/0020','2',0,1,null,200,200 from dual union all
select 408,'EELT Thor','1',0,1,null,150,-4100 from dual union all
select 408,'Star 48B','3',0,1,null,35780,200 from dual union all
select 409,'Castor 4A','0',0,9,null,70,-6100 from dual union all
select 409,'Delta/0020','2',0,1,null,200,200 from dual union all
select 409,'EELT Thor','1',0,1,null,150,-4100 from dual union all
select 409,'Star 48B','3',0,1,null,35780,200 from dual union all
select 41,'AJ60-92','2',0,1,null,250,-6370 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 41,'Nike','1',0,1,null,10,-6370 from dual union all
select 410,'Delta/0020','2',0,1,null,590,590 from dual union all
select 410,'EELT Thor','1',0,1,283500,150,-4900 from dual union all
select 410,'GEM-40','0',0,3,94800,35,-6350 from dual union all
select 411,'Delta/0020','2',0,1,null,590,590 from dual union all
select 411,'EELT Thor','1',0,1,283500,150,-4900 from dual union all
select 411,'GEM-40','0',0,3,94800,35,-6350 from dual union all
select 412,'Delta/0020','2',0,1,null,1000,185 from dual union all
select 412,'EELT Thor','1',0,1,283500,170,-4100 from dual union all
select 412,'GEM-40','0',0,3,94800,21,-6340 from dual union all
select 412,'Star 37FM','3',0,1,null,45000,1000 from dual union all
select 413,'Delta/0020','2',0,1,null,1200,150 from dual union all
select 413,'EELT Thor','1',0,1,283500,150,-4500 from dual union all
select 413,'GEM-40','0',0,4,126400,30,-6330 from dual union all
select 414,'Delta/0020','2',0,1,null,180,180 from dual union all
select 414,'EELT Thor','1',0,1,283500,160,-4470 from dual union all
select 414,'GEM-40','0',0,4,126400,28,-6320 from dual union all
select 414,'Star 48B','3',0,1,null,180,180 from dual union all
select 415,'Delta/0020','2',0,1,null,200,200 from dual union all
select 415,'EELT Thor','1',0,1,283500,150,-4500 from dual union all
select 415,'GEM-40','0',0,4,126400,20,-6300 from dual union all
select 415,'Star 48B','3',0,1,null,37800,200 from dual union all
select 416,'Delta/0020','2',0,1,null,180,180 from dual union all
select 416,'EELT Thor','1',0,1,283500,170,-4200 from dual union all
select 416,'GEM-40','0',0,4,126400,30,-6330 from dual union all
select 416,'Star 37FM','3',0,1,null,180,180 from dual union all
select 417,'Delta/0020','2',0,1,null,170,170 from dual union all
select 417,'EELT Thor','1',0,1,null,170,-4140 from dual union all
select 417,'GEM-40','0',0,9,null,75,-6200 from dual union all
select 418,'Delta/0020','2',0,1,null,170,170 from dual union all
select 418,'EELT Thor','1',0,1,283500,170,-4200 from dual union all
select 418,'GEM-40','0',0,9,284400,75,-6200 from dual union all
select 419,'Delta/0020','2',0,1,null,170,170 from dual union all
select 419,'EELT Thor','1',0,1,283500,170,-4200 from dual union all
select 419,'GEM-40','0',0,9,284400,75,-6200 from dual union all
select 42,'Aerobee 150','2',0,1,null,150,-6370 from dual union all
select 42,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 42,'Sparrow','3',0,1,null,300,-6370 from dual union all
select 420,'Delta/0020','2',0,1,null,185,185 from dual union all
select 420,'EELT Thor','1',0,1,283500,185,-2700 from dual union all
select 420,'GEM-40','0',0,9,284400,70,-6200 from dual union all
select 421,'Delta/0020','2',0,1,null,160,160 from dual union all
select 421,'EELT Thor','1',0,1,283500,160,-3400 from dual union all
select 421,'GEM-46','0',0,9,284400,75,-6100 from dual union all
select 422,'Delta/0020','2',0,1,null,180,180 from dual union all
select 422,'EELT Thor','1',0,1,283500,170,-3700 from dual union all
select 422,'GEM-40','0',0,9,284400,85,-6100 from dual union all
select 422,'Star 48B','3',0,1,null,37800,200 from dual union all
select 423,'Delta/0020','2',0,1,null,170,170 from dual union all
select 423,'EELT Thor','1',0,1,283500,170,-4250 from dual union all
select 423,'GEM-40','0',0,9,284400,65,-6180 from dual union all
select 423,'Star 48B','3',0,1,null,50550,185 from dual union all
select 424,'Delta/0020','2',0,1,null,170,170 from dual union all
select 424,'EELT Thor','1',0,1,283500,170,-4250 from dual union all
select 424,'GEM-40','0',0,9,284400,65,-6180 from dual union all
select 424,'Star 48B','3',0,1,null,50550,185 from dual union all
select 425,'Delta/0020','2',0,1,null,170,170 from dual union all
select 425,'EELT Thor','1',0,1,283500,170,-4250 from dual union all
select 425,'GEM-40','0',0,9,284400,65,-6180 from dual union all
select 425,'Star 48B','3',0,1,null,50550,185 from dual union all
select 426,'Delta/0020','2',0,1,null,180,180 from dual union all
select 426,'EELT Thor','1',0,1,283500,170,-3700 from dual union all
select 426,'GEM-40','0',0,9,284400,85,-6100 from dual union all
select 426,'Star 48B','3',0,1,null,37800,200 from dual union all
select 427,'Delta/0020','2',0,1,null,180,180 from dual union all
select 427,'EELT Thor','1',0,1,283500,170,-3700 from dual union all
select 427,'GEM-40','0',0,9,284400,85,-6100 from dual union all
select 427,'Star 48B','3',0,1,null,37800,200 from dual union all
select 428,'Delta/0020','2',0,1,null,160,160 from dual union all
select 428,'EELT Thor','1',0,1,283500,120,-3400 from dual union all
select 428,'GEM-46','0',0,9,284400,85,-5800 from dual union all
select 428,'Star 48B','3',0,1,null,160,160 from dual union all
select 429,'Delta III Stage 1','1',0,1,283500,175,-4830 from dual union all
select 429,'Delta III Stage 2','2',0,1,77000,170,170 from dual union all
select 429,'GEM-46','0',0,9,411562,90,-5950 from dual union all
select 43,'Aerobee 150','2',0,1,null,150,-6370 from dual union all
select 43,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 43,'Sparrow','3',0,1,null,340,-6370 from dual union all
select 430,'Prithvi','1',0,1,null,50,-6370 from dual union all
select 431,'Emeraude','1',0,1,null,120,-6260 from dual union all
select 431,'P064','3',0,1,null,1800,530 from dual union all
select 431,'Topaze','2',0,1,null,530,-5830 from dual union all
select 432,'Amethyste','1',0,1,null,120,-6260 from dual union all
select 432,'P068','3',0,1,null,1200,270 from dual union all
select 432,'Topaze','2',0,1,null,270,-5800 from dual union all
select 433,'Amethyste','1',0,1,null,120,-6260 from dual union all
select 433,'P068','3',0,1,null,1200,270 from dual union all
select 433,'Rita I','2',0,1,null,270,-5600 from dual union all
select 434,'15A18-BR','3',0,1,11160,600,600 from dual union all
select 434,'15A18A','1',0,1,443450,100,-6300 from dual union all
select 434,'15A18B','2',0,1,116280,350,-1000 from dual union all
select 435,'Dolphin','1',0,1,null,0,-6378 from dual union all
select 436,'DF-5 Stage 1','1',0,1,null,100,-5500 from dual union all
select 436,'DF-5 Stage 2','2',0,1,null,1000,-4000 from dual union all
select 437,'DF-5 Stage 1','1',0,1,null,100,-5500 from dual union all
select 437,'DF-5 Stage 2','2',0,1,null,1000,-4000 from dual union all
select 438,'DF-5 Stage 1','1',0,1,null,100,-5500 from dual union all
select 438,'DF-5 Stage 2','2',0,1,null,1000,-4000 from dual union all
select 439,'Lacrosse','1',0,1,null,24,-6378 from dual union all
select 44,'AJ60-91','2',0,1,null,300,-6370 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 44,'Nike','1',0,1,null,10,-6370 from dual union all
select 440,'Lacrosse','1',0,1,null,20,-6378 from dual union all
select 440,'Lacrosse','2',0,1,null,83,-6378 from dual union all
select 441,'Cajun','1',0,2,null,10,-6378 from dual union all
select 441,'T-40','2',0,1,null,10,-6378 from dual union all
select 442,'Deacon','1',0,2,null,10,-6378 from dual union all
select 443,'Mk7','1',0,1,null,2,-6378 from dual union all
select 443,'Mk7','2',0,1,null,5,-6378 from dual union all
select 444,'Deacon','2',0,1,null,10,-6378 from dual union all
select 444,'HPAG','1',0,2,null,2,-6378 from dual union all
select 445,'TX-20 Sergeant','1',0,1,null,10,-6378 from dual union all
select 445,'TX-30','2',0,1,null,30,-6378 from dual union all
select 446,'Belier','2',0,1,null,400,-6300 from dual union all
select 446,'Stromboli','1',0,1,null,30,-6370 from dual union all
select 447,'Belier II','2',0,1,null,400,-6300 from dual union all
select 447,'Stromboli','1',0,1,null,30,-6370 from dual union all
select 448,'Belier III','2',0,1,null,400,-6300 from dual union all
select 448,'Stromboli','1',0,1,null,30,-6370 from dual union all
select 449,'Dragon V2','1',0,1,null,1,-6300 from dual union all
select 45,'Aerobee 75','1',0,1,null,60,-6370 from dual union all
select 450,'E-8-5 VA','1',0,1,null,0,0 from dual union all
select 451,'C-17','0',0,1,null,10,-6378 from dual union all
select 451,'Orbus 1A','3',0,1,null,300,-6300 from dual union all
select 451,'SR19','1',0,1,null,50,-6300 from dual union all
select 451,'SR19','2',0,1,null,300,-6300 from dual union all
select 452,'Astris','3',0,1,null,250,250 from dual union all
select 452,'Blue Streak','1',0,1,null,200,-5800 from dual union all
select 452,'Coralie','2',0,1,null,250,-5200 from dual union all
select 453,'M56A1','1',0,1,null,300,-6100 from dual union all
select 453,'M57A1','2',0,1,null,300,-6100 from dual union all
select 454,'LE-3','2',0,1,null,150,-6300 from dual union all
select 454,'M-10','1',0,1,null,50,-6300 from dual union all
select 454,'SB-310','0',0,8,null,10,-6370 from dual union all
select 455,'Scud B','1',0,1,null,90,-6370 from dual union all
select 456,'Electron Stage 1','1',0,1,null,50,-6370 from dual union all
select 456,'Electron Stage 2','2',0,1,null,200,200 from dual union all
select 456,'Electron Stage 3','3',0,1,null,200,200 from dual union all
select 457,'Emad','1',0,1,null,150,-6300 from dual union all
select 458,'Emeraude','1',0,1,null,200,-6370 from dual union all
select 459,'Emma Booster','0',0,1,null,5,-6378 from dual union all
select 459,'Emma','1',0,1,null,60,-6378 from dual union all
select 46,'Aerobee AJ10-25','2',0,1,null,150,-6370 from dual union all
select 46,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 460,'Energiya Blok A','0',0,4,null,75,-6300 from dual union all
select 460,'Energiya','1',0,1,null,155,-15 from dual union all
select 461,'11S771','0',0,4,null,75,-6300 from dual union all
select 461,'Energiya','1',0,1,null,154,11 from dual union all
select 462,'Belisama','1',0,1,null,15,-6378 from dual union all
select 462,'Belisama','2',0,1,null,60,-6378 from dual union all
select 463,'ASR SRB-A','1',0,1,null,100,-6000 from dual union all
select 463,'KM-V2c','3',0,1,null,200,200 from dual union all
select 463,'M-35','2',0,1,null,200,-5000 from dual union all
select 464,'ASR SRB-A','1',0,1,null,100,-6000 from dual union all
select 464,'Epsilon PBS','4',0,1,null,200,200 from dual union all
select 464,'KM-V2c','3',0,1,null,200,200 from dual union all
select 464,'M-35','2',0,1,null,200,-5000 from dual union all
select 465,'ASR SRB-A','1',0,1,null,100,-6000 from dual union all
select 465,'Epsilon PBS','4',0,1,null,200,200 from dual union all
select 465,'KM-V2b','3',0,1,null,200,200 from dual union all
select 465,'M-34c','2',0,1,null,200,-5000 from dual union all
select 466,'Stromboli','1',0,1,null,50,-6370 from dual union all
select 466,'Stromboli','2',0,1,null,400,-6370 from dual union all
select 467,'Astris','3',0,1,null,250,250 from dual union all
select 467,'Blue Streak','1',0,1,null,200,-5800 from dual union all
select 467,'Coralie','2',0,1,null,250,-5200 from dual union all
select 468,'Astris','3',0,1,null,250,250 from dual union all
select 468,'Blue Streak','1',0,1,null,27,-6370 from dual union all
select 468,'Coralie','2',0,1,null,250,-5200 from dual union all
select 468,'P068','4',0,1,null,250,250 from dual union all
select 469,'Excalibur','1',0,1,null,100,-6378 from dual union all
select 47,'Aerobee AJ10-27','2',0,1,null,150,-6370 from dual union all
select 47,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 470,'M-6','1',0,1,null,14,-6378 from dual union all
select 470,'Nike','2',0,1,null,37,-6370 from dual union all
select 470,'Yardbird','3',0,1,null,600,-6370 from dual union all
select 471,'Mk7','1',0,1,null,5,-6378 from dual union all
select 472,'FTB','1',0,1,null,40,-6378 from dual union all
select 473,'FTI','1',0,1,null,60,-6378 from dual union all
select 474,'FTV','1',0,1,null,20,-6378 from dual union all
select 475,'Falcon 1 St 2','2',0,1,null,600,600 from dual union all
select 475,'Falcon 1','1',0,1,null,100,-5500 from dual union all
select 476,'Falcon 9 St 1/1D','1',0,1,null,100,-6100 from dual union all
select 476,'Falcon 9 St 2/1D','2',0,1,null,200,200 from dual union all
select 477,'Falcon 9 St 1/1D-FT','1',0,1,null,100,-6100 from dual union all
select 477,'Falcon 9 St 2/1D','2',0,1,null,200,200 from dual union all
select 478,'Falcon 9 St 1/1D-FT','1',0,1,null,100,-6100 from dual union all
select 478,'Falcon 9 St 2/1D-4','2',0,1,null,200,200 from dual union all
select 479,'Falcon 9 St 1/1D-4','1',0,1,null,100,-6100 from dual union all
select 479,'Falcon 9 St 2/1D-4','2',0,1,null,200,200 from dual union all
select 48,'Aerobee AJ10-34','2',0,1,null,150,-6370 from dual union all
select 48,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 480,'Falcon 9 St 1/1D-4','1',0,1,null,100,-6100 from dual union all
select 480,'Falcon 9 St 2/1D-5','2',0,1,null,200,200 from dual union all
select 481,'Falcon 9 St 1/1D-5','1',0,1,null,100,-6100 from dual union all
select 481,'Falcon 9 St 2/1D-5','2',0,1,null,200,200 from dual union all
select 482,'Falcon 9 St 1','1',0,1,null,100,-6100 from dual union all
select 482,'Falcon 9 St 2','2',0,1,null,200,200 from dual union all
select 483,'Falcon 9 St 1/1D','1',0,1,null,10,-6370 from dual union all
select 484,'Falcon 9 St 1','0',0,2,null,100,-6100 from dual union all
select 484,'Falcon 9 St 1','1',0,1,null,100,-6100 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 484,'Falcon 9 St 2','2',0,1,null,200,200 from dual union all
select 485,'FalconLaunch','1',0,1,null,120,-6370 from dual union all
select 486,'Stonechat I','1',0,1,null,30,-6378 from dual union all
select 487,'Stonechat II','1',0,1,null,150,-6370 from dual union all
select 488,'Arrow II','3',0,4,null,100,-6300 from dual union all
select 488,'Arrow II','4',0,1,null,5000,-6300 from dual union all
select 488,'Recruit','1',0,4,null,20,-6370 from dual union all
select 488,'Recruit','2',0,1,null,50,-6370 from dual union all
select 489,'Fateh-110','1',0,1,null,60,-6378 from dual union all
select 49,'Aerobee 150','2',0,1,null,150,-6370 from dual union all
select 49,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 490,'Deacon','0',0,6,null,1,-6370 from dual union all
select 490,'M-6','1',0,1,null,2,-6370 from dual union all
select 491,'Deacon','0',0,8,null,1,-6370 from dual union all
select 491,'M-6','1',0,1,null,2,-6370 from dual union all
select 492,'FB-1 Stage 1','1',0,1,null,100,-6000 from dual union all
select 492,'FB-1 Stage 2','2',0,1,null,200,200 from dual union all
select 493,'FFK','1',0,1,null,120,-6300 from dual union all
select 494,'Frangible Arcas','1',0,1,null,90,-6300 from dual union all
select 495,'Heron','1',0,1,null,15,-6378 from dual union all
select 495,'Snipe','2',0,1,null,150,-6370 from dual union all
select 496,'Orion 50','2',0,1,null,300,-5000 from dual union all
select 496,'Orion 50SG','1',0,1,null,70,-6000 from dual union all
select 497,'Orion 38','3',0,1,null,1800,-4500 from dual union all
select 497,'Orion 50','2',0,1,null,300,-5000 from dual union all
select 497,'Orion 50SG','1',0,1,null,70,-6000 from dual union all
select 498,'GEM-40','1',0,1,null,70,-6370 from dual union all
select 498,'Orbus 1','2',0,1,null,100,-6300 from dual union all
select 498,'Orbus 1','3',0,1,null,200,-6000 from dual union all
select 499,'GEM-40','1',0,1,null,70,-6370 from dual union all
select 499,'Orbus 1','2',0,1,null,100,-6300 from dual union all
select 499,'Orbus 1','3',0,1,null,200,-6000 from dual union all
select 5,'A-1','1',0,1,null,100,-6300 from dual union all
select 50,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 50,'Aerobee XASR-1','2',0,1,null,150,-6370 from dual union all
select 500,'GLCM','1',0,1,null,20,-6378 from dual union all
select 501,'12KRB','3',0,1,null,35000,200 from dual union all
select 501,'GS1-S125','1',0,1,null,85,-6200 from dual union all
select 501,'GS2','2',0,1,null,150,-4500 from dual union all
select 501,'GSOM','0',0,4,null,85,-6200 from dual union all
select 502,'12KRB','3',0,1,null,35000,200 from dual union all
select 502,'GS1-S139','1',0,1,null,85,-6200 from dual union all
select 502,'GS2','2',0,1,null,150,-4500 from dual union all
select 502,'GSOM','0',0,4,null,85,-6200 from dual union all
select 503,'CUS15','3',0,1,null,35000,200 from dual union all
select 503,'GS1-S139','1',0,1,null,85,-6200 from dual union all
select 503,'GS2','2',0,1,null,150,-4500 from dual union all
select 503,'GSOM','0',0,4,null,85,-6200 from dual union all
select 504,'CUS12','3',0,1,null,35000,200 from dual union all
select 504,'GS1-S139','1',0,1,null,85,-6200 from dual union all
select 504,'GS2','2',0,1,null,150,-4500 from dual union all
select 504,'GSOM','0',0,4,null,85,-6200 from dual union all
select 505,'GS1-L110','1',0,1,null,50,-6300 from dual union all
select 505,'GS2-C25','2',0,1,null,500,35700 from dual union all
select 505,'GSOM-S200','0',0,2,null,50,-6300 from dual union all
select 506,'GammaCen Booster','1',0,1,null,1,-6378 from dual union all
select 506,'GammaCen Sustainer','2',0,1,null,35,-6378 from dual union all
select 507,'Genie','1',0,1,null,10,-6300 from dual union all
select 508,'Shahab 3','1',0,1,null,150,-6300 from dual union all
select 509,'Ghauri','1',0,1,null,150,-6370 from dual union all
select 51,'Aerobee AJ10-25','2',0,1,null,150,-6370 from dual union all
select 51,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 510,'Ghaznavi','1',0,1,null,100,-6370 from dual union all
select 511,'Bull Goose','1',0,1,null,20,-6378 from dual union all
select 512,'Gosling','1',0,1,null,10,-6378 from dual union all
select 512,'Lobster I','2',0,1,null,20,-6378 from dual union all
select 513,'Grad','1',0,1,null,20,-6378 from dual union all
select 514,'Gradicom 1','1',0,1,null,40,-6378 from dual union all
select 515,'Gradicom 1','2',0,1,null,100,-6378 from dual union all
select 515,'Gradicom 2','1',0,1,null,10,-6378 from dual union all
select 516,'Emilie','1',0,1,null,15,-6378 from dual union all
select 516,'Melusine','2',0,1,null,130,-6370 from dual union all
select 517,'Falcon 9-1D','1',0,1,null,0,-6378 from dual union all
select 518,'R-39 St 1','1',0,1,null,0,-6378 from dual union all
select 518,'R-39 St 2','2',0,1,null,0,-6378 from dual union all
select 518,'R-39 St 3','3',0,1,null,0,-6378 from dual union all
select 519,'Gun Projectile 16','1',0,1,null,100,-6370 from dual union all
select 52,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 52,'Aerobee XASR-1','2',0,1,null,150,-6370 from dual union all
select 520,'Gun Projectile 5','1',0,1,null,100,-6370 from dual union all
select 521,'Gun Projectile 7','1',0,1,null,100,-6370 from dual union all
select 522,'Castor 2','0',0,9,null,50,-6100 from dual union all
select 522,'ELT Thor N','1',0,1,null,150,-5500 from dual union all
select 522,'LE-5','2',0,1,null,200,200 from dual union all
select 523,'Castor 2','0',0,6,null,50,-6100 from dual union all
select 523,'ELT Thor N','1',0,1,null,150,-5500 from dual union all
select 523,'LE-5','2',0,1,null,220,-1000 from dual union all
select 523,'UM129A','3',0,1,null,35800,200 from dual union all
select 524,'Castor 2','0',0,9,null,50,-6100 from dual union all
select 524,'ELT Thor N','1',0,1,null,150,-5500 from dual union all
select 524,'LE-5','2',0,1,null,600,100 from dual union all
select 524,'UM129A','3',0,1,null,35800,200 from dual union all
select 525,'H-II SRB','0',0,2,291400,60,-6250 from dual union all
select 525,'H-II SSB','-1',0,2,null,50,-6250 from dual union all
select 525,'H-II Stage 1','1',0,1,373680,250,-4400 from dual union all
select 525,'H-II Stage 2','2',0,1,73200,400,400 from dual union all
select 526,'H-II SRB','0',0,2,291400,60,-6260 from dual union all
select 526,'H-II Stage 1','1',0,1,373680,300,-4600 from dual union all
select 526,'H-II Stage 2','2',0,1,73200,400,400 from dual union all
select 527,'H-II SRB-A3','0',0,2,null,70,-6260 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 527,'H-IIA Stage 1','1',0,1,null,250,-4400 from dual union all
select 527,'H-IIA Stage 2','2',0,1,null,400,400 from dual union all
select 528,'H-II SRB-A','0',0,2,null,70,-6260 from dual union all
select 528,'H-IIA Stage 1','1',0,1,null,250,-4400 from dual union all
select 528,'H-IIA Stage 2','2',0,1,null,400,400 from dual union all
select 529,'Castor 4AXL','-1',0,2,null,75,-6250 from dual union all
select 529,'H-II SRB-A','0',0,2,null,75,-6250 from dual union all
select 529,'H-IIA Stage 1','1',0,1,null,250,-4500 from dual union all
select 529,'H-IIA Stage 2','2',0,1,null,500,500 from dual union all
select 53,'Aerobee AJ10-25','1',0,1,null,150,-6370 from dual union all
select 530,'Castor 4AXL','-1',0,4,null,90,-6190 from dual union all
select 530,'H-II SRB-A3','0',0,2,null,100,-6190 from dual union all
select 530,'H-IIA Stage 1','1',0,1,null,330,-4340 from dual union all
select 530,'H-IIA Stage 2','2',0,1,null,500,500 from dual union all
select 531,'Castor 4AXL','-1',0,4,null,90,-6190 from dual union all
select 531,'H-II SRB-A','0',0,2,null,100,-6190 from dual union all
select 531,'H-IIA Stage 1','1',0,1,null,330,-4340 from dual union all
select 531,'H-IIA Stage 2','2',0,1,null,500,500 from dual union all
select 532,'H-II SRB-A3','0',0,4,null,70,-6260 from dual union all
select 532,'H-IIA Stage 1','1',0,1,null,250,-4400 from dual union all
select 532,'H-IIA Stage 2 U','2',0,1,null,400,400 from dual union all
select 533,'H-II SRB-A','0',0,4,null,70,-6260 from dual union all
select 533,'H-IIA Stage 1','1',0,1,null,250,-4400 from dual union all
select 533,'H-IIA Stage 2','2',0,1,null,400,400 from dual union all
select 534,'H-II SRB-A3','0',0,4,null,50,-6378 from dual union all
select 534,'H-IIA Stage 2','2',0,1,null,300,200 from dual union all
select 534,'H-IIB Stage 1','1',0,1,null,100,-6000 from dual union all
select 535,'Gosling','1',0,1,null,10,-6370 from dual union all
select 535,'LAPSTAR','2',0,1,null,100,-6370 from dual union all
select 536,'Demon','1',0,1,null,10,-6370 from dual union all
select 536,'HAEC FFAR','2',0,1,null,10,-6370 from dual union all
select 537,'HALO','1',0,1,null,60,-6378 from dual union all
select 538,'HAT','1',0,1,null,10,-6370 from dual union all
select 538,'LAPSTAR','2',0,1,null,80,-6370 from dual union all
select 539,'X-265','1',0,1,null,1,-6370 from dual union all
select 539,'X-271','2',0,1,null,10,-6370 from dual union all
select 54,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 54,'Aerobee XASR-1','2',0,1,null,150,-6370 from dual union all
select 540,'HIBEX','1',0,1,null,20,-6300 from dual union all
select 541,'Balloon','0',0,1,null,18,-6378 from dual union all
select 541,'KM Rockoon','1',0,1,null,70,-6378 from dual union all
select 542,'M-6','1',0,1,null,10,-6370 from dual union all
select 543,'Hydac','2',0,1,null,170,-6370 from dual union all
select 543,'M-6','1',0,1,null,10,-6370 from dual union all
select 544,'Javelin 3','2',0,1,null,300,-6370 from dual union all
select 544,'M-6','1',0,1,null,15,-6370 from dual union all
select 545,'M-6','1',0,1,null,10,-6370 from dual union all
select 545,'Nike','2',0,1,null,100,-6370 from dual union all
select 546,'Gosling','3',0,1,null,43,-6276 from dual union all
select 546,'M-6','1',0,1,null,24,-6365 from dual union all
select 546,'Nike','2',0,1,null,37,-6290 from dual union all
select 547,'Hydac','3',0,1,null,500,-6370 from dual union all
select 547,'M-6','1',0,1,null,10,-6370 from dual union all
select 547,'Nike','2',0,1,null,50,-6370 from dual union all
select 548,'Javelin 3','3',0,1,null,300,-6370 from dual union all
select 548,'M-6','1',0,1,null,10,-6370 from dual union all
select 548,'Nike','2',0,1,null,40,-6370 from dual union all
select 549,'M-6','1',0,1,null,10,-6370 from dual union all
select 549,'Nike','2',0,1,null,40,-6300 from dual union all
select 549,'Nike','3',0,1,null,120,-6300 from dual union all
select 55,'Aerobee AJ10-25','2',0,1,null,150,-6370 from dual union all
select 55,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 550,'Cygnus 20','4',0,1,null,1380,-6100 from dual union all
select 550,'M-6','1',0,1,null,15,-6360 from dual union all
select 550,'Nike','2',0,1,null,40,-6300 from dual union all
select 550,'Nike','3',0,1,null,90,-6300 from dual union all
select 551,'M-6','1',0,1,null,20,-6364 from dual union all
select 551,'Nike','2',0,1,null,30,-6338 from dual union all
select 551,'Nike','3',0,1,null,35,-6300 from dual union all
select 551,'Recruit','4',0,1,null,35,-6300 from dual union all
select 552,'M-6','1',0,1,null,30,-6300 from dual union all
select 552,'Nike','2',0,1,null,30,-6300 from dual union all
select 552,'Nike','3',0,1,null,30,-6300 from dual union all
select 552,'Recruit','4',0,1,null,30,-6300 from dual union all
select 552,'T-55','5',0,1,null,30,-6300 from dual union all
select 553,'M-6','1',0,1,null,10,-6378 from dual union all
select 553,'Nike','2',0,1,null,20,-6378 from dual union all
select 553,'T-40','3',0,1,null,30,-6378 from dual union all
select 554,'M-6','1',0,1,null,10,-6378 from dual union all
select 554,'Nike','2',0,1,null,20,-6378 from dual union all
select 554,'T-40','3',0,1,null,30,-6378 from dual union all
select 554,'T-55','4',0,1,null,30,-6378 from dual union all
select 555,'Deacon','3',0,3,null,30,-6378 from dual union all
select 555,'M-6','1',0,1,null,10,-6378 from dual union all
select 555,'Nike','2',0,1,null,20,-6378 from dual union all
select 555,'T-40','4',0,1,null,30,-6378 from dual union all
select 556,'M-6','1',0,1,null,20,-6370 from dual union all
select 556,'Orion','2',0,1,null,120,-6370 from dual union all
select 557,'M-6','1',0,1,null,10,-6370 from dual union all
select 557,'Nike','2',0,1,null,23,-6362 from dual union all
select 557,'Recruit','4',0,1,null,64,-5738 from dual union all
select 557,'Skat','3',0,1,null,34,-6157 from dual union all
select 558,'M55E1','1',0,1,null,70,-6370 from dual union all
select 558,'M56A1','2',0,1,null,200,-6370 from dual union all
select 559,'HPAG','1',0,1,null,10,-6378 from dual union all
select 56,'Aerobee AJ10-24','2',0,1,null,150,-6370 from dual union all
select 56,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 560,'Deacon','2',0,1,null,10,-6378 from dual union all
select 560,'HPAG','1',0,1,null,10,-6378 from dual union all
select 561,'M56A1','2',0,1,null,400,-6300 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 561,'Talos','1',0,1,null,5,-6378 from dual union all
select 562,'S9','1',0,1,null,70,-6300 from dual union all
select 563,'AeroHTV Stage 2','2',0,1,null,10,-6378 from dual union all
select 563,'AeroHTV','1',0,1,null,1,-6378 from dual union all
select 564,'HVAR','1',0,1,null,5,-6378 from dual union all
select 565,'HVAR','1',0,1,null,5,-6378 from dual union all
select 565,'Mk7','2',0,1,null,5,-6378 from dual union all
select 566,'HYSR','1',0,1,null,70,-6370 from dual union all
select 567,'Hades','1',0,1,null,150,-6370 from dual union all
select 568,'Hatf I','1',0,1,null,50,-6370 from dual union all
select 569,'Hatf I','1',0,1,null,50,-6370 from dual union all
select 569,'Hatf I','2',0,1,null,250,-6370 from dual union all
select 57,'Aerobee AJ10-34','2',0,1,null,150,-6370 from dual union all
select 57,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 570,'M57A1','2',0,1,null,300,-6370 from dual union all
select 570,'SR19AJ1','1',0,1,null,70,-6370 from dual union all
select 571,'Hermes A-1','1',0,1,null,30,-6370 from dual union all
select 572,'Hermes A-3A','1',0,1,null,30,-6370 from dual union all
select 573,'Hermes A-3B','1',0,1,null,35,-6370 from dual union all
select 574,'V-2-Ram','1',0,1,null,150,-6370 from dual union all
select 575,'B-58 Hustler','0',0,1,null,10,-6378 from dual union all
select 575,'TX-20 Sergeant','1',0,1,null,76,-6370 from dual union all
select 576,'M-6','1',0,1,null,10,-6370 from dual union all
select 577,'Hopi III','1',0,1,null,90,-6370 from dual union all
select 578,'Cajun','2',0,1,null,100,-6378 from dual union all
select 578,'Terrier','1',0,1,null,5,-6378 from dual union all
select 579,'Hwasong 5','1',0,1,null,200,-6350 from dual union all
select 58,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 58,'Aerobee XASR-1','2',0,1,null,150,-6370 from dual union all
select 580,'Hwasong 6','1',0,1,null,200,-6350 from dual union all
select 581,'Hwasong ER','1',0,1,null,150,-6300 from dual union all
select 582,'Hwasong 10','1',0,1,null,1000,-6100 from dual union all
select 583,'Hwasong-12 Stage 1','1',0,1,null,2000,-6370 from dual union all
select 584,'Hwasong-12 Stage 1','1',0,1,null,50,-6370 from dual union all
select 584,'Hwasong-14 Stage 2','2',0,1,null,3000,-6370 from dual union all
select 585,'Hwasong-15 Stage 1','1',0,1,null,50,-6370 from dual union all
select 585,'Hwasong-15 Stage 2','2',0,1,null,3000,-6370 from dual union all
select 586,'Hwasong-7','1',0,1,null,150,-6300 from dual union all
select 587,'HTR','1',0,1,null,30,-6378 from dual union all
select 588,'NOTS 401A','1',0,1,null,20,-6378 from dual union all
select 588,'Sandhawk','2',0,1,null,250,-6300 from dual union all
select 589,'Iris','1',0,1,null,200,-6378 from dual union all
select 589,'MR-1','0',0,1,null,3,-6378 from dual union all
select 59,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 59,'Aerobee XASR-1','2',0,1,null,150,-6370 from dual union all
select 590,'NB-52','0',0,1,null,10,-6378 from dual union all
select 590,'Orion 50S','1',0,1,null,33,-6378 from dual union all
select 591,'Hyperion','1',0,1,null,36,-6378 from dual union all
select 592,'Hyunmoo Booster','1',0,1,null,2,-6378 from dual union all
select 592,'TX-30','2',0,1,null,20,-6378 from dual union all
select 593,'Hyunmoo-2A','1',0,1,null,100,-6378 from dual union all
select 594,'Hyunmoo-2B','1',0,1,null,100,-6378 from dual union all
select 595,'Hyunmoo-2C','1',0,1,null,100,-6378 from dual union all
select 596,'Orion 50','3',0,1,null,300,-5000 from dual union all
select 596,'Orion 50SXLT','2',0,1,null,70,-6000 from dual union all
select 596,'Trident C-4 Stage 1','1',0,1,null,40,-6370 from dual union all
select 597,'INTA S-12','1',0,1,null,1,-6378 from dual union all
select 597,'INTA-100','2',0,1,null,100,-6378 from dual union all
select 598,'Chick','0',0,4,null,1,-6378 from dual union all
select 598,'Goose','1',0,1,null,130,-6378 from dual union all
select 599,'Heron','1',0,1,null,2,-6378 from dual union all
select 599,'Snipe','2',0,1,null,200,-6370 from dual union all
select 6,'A-350 Booster','1',0,1,null,10,-6370 from dual union all
select 6,'A-350','2',0,1,null,300,-6300 from dual union all
select 60,'Aerobee Booster','1',0,1,null,10,-6370 from dual union all
select 60,'Aerobee XASR-1','2',0,1,null,150,-6370 from dual union all
select 600,'Heron','1',0,1,null,2,-6378 from dual union all
select 600,'Snipe','2',0,1,null,150,-6370 from dual union all
select 601,'Orion 50SXLT','1',0,1,null,70,-6000 from dual union all
select 601,'Orion 50XLT','2',0,1,null,300,-5000 from dual union all
select 602,'Icarus Stage 2','2',0,1,null,60,-6378 from dual union all
select 602,'Icarus','1',0,1,null,1,-6378 from dual union all
select 603,'Taurus','1',0,1,null,2,-6370 from dual union all
select 604,'Imp Malemute','1',0,1,null,120,-6300 from dual union all
select 605,'Improved Orion','1',0,1,null,88,-6370 from dual union all
select 606,'Iraqi Gun Projectile','1',0,1,null,100,-6370 from dual union all
select 607,'Iris','1',0,1,null,200,-6300 from dual union all
select 607,'MARC 14B1','0',0,1,null,1,-6378 from dual union all
select 608,'Iskander','1',0,1,null,150,-6378 from dual union all
select 609,'Iskander-E','1',0,1,null,150,-6378 from dual union all
select 61,'Agate','1',0,1,null,20,-6300 from dual union all
select 610,'Iskander-M','1',0,1,null,150,-6378 from dual union all
select 611,'Ivanov Booster','0',0,1,null,1,-6378 from dual union all
select 611,'Ivanov','1',0,1,null,5,-6378 from dual union all
select 611,'Ivanov','2',0,1,null,10,-6378 from dual union all
select 611,'Ivanov','3',0,1,null,35,-6378 from dual union all
select 612,'H-II SRB','1',0,1,null,95,-6360 from dual union all
select 612,'M-23','2',0,1,null,110,-5450 from dual union all
select 613,'JCR Booster','1',0,1,null,10,-6370 from dual union all
select 613,'JCR','2',0,1,null,200,-6370 from dual union all
select 614,'DF-21 St2','2',0,1,null,500,-6300 from dual union all
select 614,'DF-21','1',0,1,null,100,-6300 from dual union all
select 615,'DF-31 St1','1',0,1,null,50,-6300 from dual union all
select 615,'DF-31 St2','2',0,1,null,200,-5500 from dual union all
select 615,'DF-31 St3','3',0,1,null,1000,-4000 from dual union all
select 616,'Goldfinch II','2',0,1,null,30,-6370 from dual union all
select 616,'Gosling IV','3',0,1,null,30,-6370 from dual union all
select 616,'Rook IIIA','1',0,1,null,30,-6370 from dual union all
select 617,'Goldfinch II','2',0,1,null,30,-6370 from dual union all
select 617,'Gosling IV','3',0,1,null,30,-6370 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 617,'Rook IIIA','1',0,1,null,30,-6370 from dual union all
select 618,'Rook IIIA','1',0,1,null,30,-6370 from dual union all
select 618,'Rook IIIB','2',0,1,null,30,-6370 from dual union all
select 619,'B-57A Canberra','0',0,1,null,10,-6378 from dual union all
select 619,'Baby Sergeant','3',0,1,null,800,-6300 from dual union all
select 619,'Recruit','1',0,3,null,20,-6300 from dual union all
select 619,'Recruit','2',0,1,null,100,-6300 from dual union all
select 62,'Agni','1',0,1,null,40,-6300 from dual union all
select 620,'Gosling IIN','2',0,1,null,30,-6370 from dual union all
select 620,'Lobster I','3',0,1,null,30,-6370 from dual union all
select 620,'Rook II','1',0,1,null,30,-6370 from dual union all
select 621,'Goldfinch II','2',0,1,null,30,-6370 from dual union all
select 621,'Gosling IV','3',0,1,null,30,-6370 from dual union all
select 621,'Rook IIIA','1',0,1,null,30,-6370 from dual union all
select 622,'MLRS','0',0,2,null,5,-6100 from dual union all
select 622,'SR19','1',0,1,null,70,-6370 from dual union all
select 622,'SR19','2',0,1,null,300,-6370 from dual union all
select 623,'M-6','1',0,1,null,5,-6370 from dual union all
select 623,'Nike','2',0,1,null,14,-6370 from dual union all
select 623,'Nike','3',0,1,null,50,-6300 from dual union all
select 623,'Recruit','4',0,1,null,200,-6250 from dual union all
select 623,'T-55','5',0,1,null,800,-6200 from dual union all
select 624,'Altair','4',0,1,null,1300,-6150 from dual union all
select 624,'M-6','1',0,1,null,5,-6370 from dual union all
select 624,'Nike','2',0,1,null,14,-6370 from dual union all
select 624,'Nike','3',0,1,null,95,-6300 from dual union all
select 625,'Javelin 3','1',0,1,null,80,-6370 from dual union all
select 626,'MD-620','1',0,1,null,100,-6370 from dual union all
select 626,'NA-805','2',0,1,null,100,-6370 from dual union all
select 627,'Jericho II','1',0,1,null,300,-6370 from dual union all
select 628,'Jericho IIIA1','1',0,1,null,100,-6370 from dual union all
select 628,'Jericho IIIA2','2',0,1,null,300,-6370 from dual union all
select 629,'Jericho-1','1',0,1,null,100,-6370 from dual union all
select 63,'Agni II Stage 2','2',0,1,null,400,-5000 from dual union all
select 63,'Agni','1',0,1,null,40,-6300 from dual union all
select 630,'Altair','4',0,1,null,1100,-6250 from dual union all
select 630,'Lance','2',0,1,null,70,-6378 from dual union all
select 630,'Lance','3',0,1,null,300,-6300 from dual union all
select 630,'Recruit','0',0,2,null,1,-6378 from dual union all
select 630,'Sergeant','1',0,1,null,30,-6378 from dual union all
select 631,'Judi I','1',0,1,null,75,-6370 from dual union all
select 632,'M57A1','2',0,1,null,300,-6370 from dual union all
select 632,'SR19AJ1','1',0,1,null,70,-6370 from dual union all
select 633,'Cluster Stage 2','2',0,1,null,830,-4000 from dual union all
select 633,'Cluster Stage 3','3',0,1,null,830,-4000 from dual union all
select 633,'Jupiter','1',0,1,null,442,-5179 from dual union all
select 634,'Cluster Stage 2','2',0,1,null,442,-3868 from dual union all
select 634,'Cluster Stage 3','3',0,1,null,442,-1883 from dual union all
select 634,'Jupiter','1',0,1,null,442,-5179 from dual union all
select 634,'RTV Motor','4',0,1,null,872,440 from dual union all
select 635,'Redstone','1',0,1,null,90,-6230 from dual union all
select 636,'Cluster Stage 2','2',0,1,null,360,-5500 from dual union all
select 636,'Cluster Stage 3','3',0,1,null,360,-3600 from dual union all
select 636,'RTV Motor','4',0,1,null,2500,360 from dual union all
select 636,'Redstone','1',0,1,null,360,-6200 from dual union all
select 637,'Cluster Stage 2','2',0,1,null,363,-5500 from dual union all
select 637,'Cluster Stage 3','3',0,1,null,1000,-4094 from dual union all
select 637,'Redstone','1',0,1,null,360,-6200 from dual union all
select 638,'Jupiter','1',0,1,null,500,-5700 from dual union all
select 639,'K-15 Booster','1',0,1,null,5,-6378 from dual union all
select 639,'K-15 Stage 2','2',0,1,null,200,-6300 from dual union all
select 64,'Agni II Stage 2','2',0,1,null,400,-5000 from dual union all
select 64,'Agni','1',0,1,null,40,-6300 from dual union all
select 640,'K-4 Booster','1',0,1,null,5,-6378 from dual union all
select 640,'K-4 Stage 2','2',0,1,null,200,-6300 from dual union all
select 641,'K-BATS','1',0,1,null,50,-6370 from dual union all
select 642,'K150','1',0,1,null,5,-6378 from dual union all
select 643,'K245','1',0,1,null,1,-6378 from dual union all
select 644,'R-12','1',0,1,null,500,-6200 from dual union all
select 645,'R-12','1',0,1,null,500,-6200 from dual union all
select 646,'R-14','1',0,1,null,500,-5800 from dual union all
select 646,'S3M','2',0,1,null,500,-5800 from dual union all
select 647,'R-14','1',0,1,null,500,-5800 from dual union all
select 647,'S3M','2',0,1,null,500,-5800 from dual union all
select 648,'KN-02','1',0,1,null,20,-6378 from dual union all
select 649,'KSR','1',0,1,null,40,-6370 from dual union all
select 65,'Agni III Stage 2','2',0,1,null,400,-5000 from dual union all
select 65,'Agni III','1',0,1,null,40,-6300 from dual union all
select 650,'KSR Booster','1',0,1,null,35,-6370 from dual union all
select 650,'KSR','2',0,1,null,150,-6370 from dual union all
select 651,'KSR-III','1',0,1,null,100,-6300 from dual union all
select 652,'DF-21 St2','2',0,1,null,200,-5000 from dual union all
select 652,'DF-21','1',0,1,null,50,-6300 from dual union all
select 652,'KT-1 St3','3',0,1,null,200,200 from dual union all
select 653,'DF-31 St1','1',0,1,null,50,-6300 from dual union all
select 653,'DF-31 St2','2',0,1,null,200,-5500 from dual union all
select 653,'DF-31 St3','3',0,1,null,1000,-4000 from dual union all
select 654,'Kappa 1','1',0,1,null,5,-6378 from dual union all
select 655,'K420(1/3)','2',0,1,null,250,-6300 from dual union all
select 655,'K420H','1',0,1,null,30,-6378 from dual union all
select 656,'K420(1/3)','2',0,1,null,300,-6300 from dual union all
select 656,'K420H','1',0,1,null,30,-6378 from dual union all
select 657,'K10S','3',0,1,null,740,-6370 from dual union all
select 657,'K420(1/3)','2',0,1,null,250,-6300 from dual union all
select 657,'K420H','1',0,1,null,30,-6378 from dual union all
select 658,'Kappa 1','2',1,1,null,5,-6378 from dual union all
select 658,'Kappa 3 Booster','1',0,1,null,5,-6378 from dual union all
select 659,'Kappa 1','2',0,1,null,30,-6378 from dual union all
select 659,'Kappa 3 Booster','1',0,1,null,5,-6378 from dual union all
select 66,'Agni 4 Stage 2','2',0,1,null,400,-5000 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 66,'Agni','1',0,1,null,40,-6300 from dual union all
select 660,'Kappa 4 Booster','1',0,1,null,5,-6378 from dual union all
select 660,'Kappa 4','2',0,1,null,40,-6378 from dual union all
select 661,'K150','2',0,1,null,15,-6378 from dual union all
select 661,'Kappa 6','1',0,1,null,5,-6378 from dual union all
select 662,'K150','2',0,1,null,85,-6350 from dual union all
select 662,'Kappa 6','1',0,1,null,50,-6379 from dual union all
select 663,'K150','2',0,1,null,70,-6350 from dual union all
select 663,'Kappa 6H','1',0,1,null,40,-6350 from dual union all
select 664,'K420','1',0,1,null,53,-6370 from dual union all
select 665,'K420','1',0,1,null,50,-6370 from dual union all
select 665,'Kappa 8','2',0,1,null,200,-6370 from dual union all
select 666,'Kappa 6H','1',0,1,null,50,-6378 from dual union all
select 666,'Kappa 8L','2',0,1,null,150,-6340 from dual union all
select 667,'K150','3',0,1,null,350,-6378 from dual union all
select 667,'K420','1',0,1,null,50,-6378 from dual union all
select 667,'Kappa 6','2',0,1,null,100,-6378 from dual union all
select 668,'K250','2',0,1,null,250,-6370 from dual union all
select 668,'K420H','1',0,1,null,60,-6370 from dual union all
select 669,'Zelzal','1',0,1,null,60,-6378 from dual union all
select 67,'Agni V Stage 2','2',0,1,null,400,-5000 from dual union all
select 67,'Agni V Stage 3','3',0,1,null,400,-5000 from dual union all
select 67,'Agni V','1',0,1,null,40,-6300 from dual union all
select 670,'Kavoshgar','1',0,1,null,15,-6370 from dual union all
select 671,'Khorramshahr','1',0,1,null,1000,-6100 from dual union all
select 672,'Judi III','2',0,1,null,100,-6370 from dual union all
select 672,'Kisha','1',0,1,null,20,-6378 from dual union all
select 673,'Hopi','2',0,1,null,300,-6378 from dual union all
select 673,'Kiva','1',0,1,null,30,-6378 from dual union all
select 674,'27D6','1',0,1,null,20,-6300 from dual union all
select 674,'28D6','2',0,1,null,100,-6300 from dual union all
select 674,'29D6','3',0,1,null,500,-6300 from dual union all
select 674,'MiG31D 072','0',0,1,null,17,-6300 from dual union all
select 675,'Lupus','1',0,1,null,20,-6378 from dual union all
select 675,'Musca','2',0,1,null,50,-6378 from dual union all
select 676,'Lupus 2','1',0,1,null,20,-6378 from dual union all
select 676,'Musca','2',0,1,null,50,-6378 from dual union all
select 677,'Lupus 3','1',0,1,null,20,-6378 from dual union all
select 677,'Musca','2',0,1,null,50,-6378 from dual union all
select 678,'R-12','1',0,1,null,100,-6000 from dual union all
select 678,'S1M','2',0,1,null,200,200 from dual union all
select 679,'R-14','1',0,1,null,140,-6100 from dual union all
select 679,'S3','2',0,1,null,200,200 from dual union all
select 68,'Agni','1',0,1,null,40,-6300 from dual union all
select 68,'Prithvi','2',0,1,null,300,-5000 from dual union all
select 680,'R-14 11K65MA','1',0,1,226900,140,-6100 from dual union all
select 680,'S3M','2',0,1,52600,200,200 from dual union all
select 681,'R-12','1',0,1,null,100,-6000 from dual union all
select 681,'S1','2',0,1,null,200,200 from dual union all
select 682,'R-14','1',0,1,null,100,-6100 from dual union all
select 682,'S3','2',0,1,null,200,200 from dual union all
select 683,'DF-21 St2','2',0,1,null,200,-5000 from dual union all
select 683,'KZ-1 St3','3',0,1,null,200,200 from dual union all
select 683,'KZ-1 St4','4',0,1,null,200,200 from dual union all
select 683,'KZ-1A St1','1',0,1,null,50,-6300 from dual union all
select 684,'15Zh59A','1',0,1,null,10,-6378 from dual union all
select 684,'15Zh59B','2',0,1,null,100,-6378 from dual union all
select 684,'15Zh59V','3',0,1,null,1000,-6378 from dual union all
select 686,'M57A1','3',0,1,null,400,-6000 from dual union all
select 686,'Sergeant','2',0,1,null,90,-6300 from dual union all
select 686,'Talos','1',0,1,null,10,-6370 from dual union all
select 687,'Balloon','0',0,1,null,18,-6378 from dual union all
select 687,'Star 48B','1',0,1,null,55,-6378 from dual union all
select 689,'LM AS','1',0,1,null,0,0 from dual union all
select 69,'Al Abbas','1',0,1,null,100,-6200 from dual union all
select 691,'C-17','0',0,1,null,10,-6378 from dual union all
select 691,'SR19','1',0,1,null,50,-6300 from dual union all
select 691,'SR19','2',0,1,null,300,-6300 from dual union all
select 692,'LS-A Booster','1',0,1,null,5,-6378 from dual union all
select 692,'LS-A','2',0,1,null,150,-6370 from dual union all
select 693,'LS-C Booster','1',0,1,null,5,-6378 from dual union all
select 693,'LS-C-7','2',0,1,null,100,-6370 from dual union all
select 694,'LS-C Booster','1',0,1,null,5,-6378 from dual union all
select 694,'LS-C','2',0,1,null,100,-6370 from dual union all
select 695,'Trident C-4 Stage 1','1',0,1,null,40,-6370 from dual union all
select 695,'Trident C-4 Stage 2','2',0,1,null,150,-6000 from dual union all
select 696,'L735','1',0,1,11800,25,-6370 from dual union all
select 696,'Lambda2-2','2',0,1,null,410,-6360 from dual union all
select 696,'SB-310','0',0,2,1406,3,-6370 from dual union all
select 697,'L735','1',0,1,11800,30,-6370 from dual union all
select 697,'Lambda3-2','2',0,1,null,300,-6370 from dual union all
select 697,'Lambda3-3','3',0,1,null,1000,-6344 from dual union all
select 697,'SB-310','0',0,2,1406,3,-6370 from dual union all
select 698,'L500','3',0,1,1863,2000,-6340 from dual union all
select 698,'L735','1',0,1,11800,25,-6370 from dual union all
select 698,'L735(1/3)','2',0,1,4531,300,-6370 from dual union all
select 698,'SB-310','0',0,2,1406,3,-6370 from dual union all
select 699,'L480S','4',0,1,245,4000,500 from dual union all
select 699,'L500','3',0,1,1863,525,-4800 from dual union all
select 699,'L735','1',0,1,11800,50,-6360 from dual union all
select 699,'L735(1/3)','2',0,1,4531,300,-6300 from dual union all
select 699,'SB-310','0',0,2,1406,3,-6370 from dual union all
select 7,'A-350Zh Booster','1',0,1,null,10,-6370 from dual union all
select 7,'A-350Zh','2',0,1,null,100,-6300 from dual union all
select 70,'Abid St1','1',0,1,null,30,-6300 from dual union all
select 70,'Abid St2','2',1,1,null,50,-6000 from dual union all
select 70,'Abid St3','3',1,1,null,50,-6000 from dual union all
select 700,'L480S','4',0,1,245,300,-6000 from dual union all
select 700,'L500','3',0,1,1863,300,-6000 from dual union all
select 700,'L735','1',0,1,11800,50,-6360 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 700,'L735(1/3)','2',0,1,4531,300,-6300 from dual union all
select 700,'SB-310','0',0,2,1406,3,-6370 from dual union all
select 701,'L480S','4',0,1,245,400,-2500 from dual union all
select 701,'L500','3',0,1,1863,400,-5000 from dual union all
select 701,'L735','1',0,1,11800,50,-6360 from dual union all
select 701,'L735(1/3)','2',0,1,4531,300,-6300 from dual union all
select 701,'SB-310','0',0,2,1406,3,-6370 from dual union all
select 702,'Lance Missile','1',0,1,null,45,-6378 from dual union all
select 703,'Lark','1',0,1,null,8,-6378 from dual union all
select 704,'R-29RM St1','1',0,1,null,70,-6100 from dual union all
select 704,'R-29RM St2','2',0,1,null,500,-5000 from dual union all
select 704,'R-29RM St3','3',0,1,null,1000,-3000 from dual union all
select 705,'Gosling','2',0,1,null,20,-6378 from dual union all
select 705,'Rook','1',0,1,null,5,-6378 from dual union all
select 706,'Lex','1',0,1,null,100,-6370 from dual union all
select 707,'Castor','1',0,2,null,25,-6368 from dual union all
select 707,'Recruit','0',0,4,null,25,-6368 from dual union all
select 708,'Pollux','1',0,2,null,25,-6368 from dual union all
select 708,'Recruit','0',0,4,null,25,-6368 from dual union all
select 709,'Castor','1',0,4,null,25,-6368 from dual union all
select 709,'Recruit','0',0,4,null,25,-6368 from dual union all
select 71,'Al Fatah','1',0,1,null,100,-6378 from dual union all
select 710,'Pollux','1',0,4,null,25,-6368 from dual union all
select 710,'Recruit','0',0,4,null,25,-6368 from dual union all
select 711,'Algol 1D','1',0,6,null,25,-6368 from dual union all
select 712,'Algol 1D','1',0,2,null,25,-6368 from dual union all
select 712,'Recruit','0',0,4,null,25,-6368 from dual union all
select 713,'Algol 1D','1',0,4,null,25,-6368 from dual union all
select 713,'Recruit','0',0,5,null,25,-6368 from dual union all
select 714,'Algol 1D','1',0,1,null,25,-6368 from dual union all
select 714,'Recruit','0',0,6,null,25,-6368 from dual union all
select 715,'Loki','1',0,1,null,75,-6378 from dual union all
select 716,'Loki-Dart','1',0,1,null,75,-6378 from dual union all
select 717,'Loki SR71-AD-1','1',0,1,null,75,-6378 from dual union all
select 718,'Balloon','0',0,1,null,10,-6378 from dual union all
select 718,'Loki','1',0,1,null,100,-6378 from dual union all
select 719,'Gosling','1',0,1,null,30,-6378 from dual union all
select 719,'Mayfly','2',0,1,null,100,-6378 from dual union all
select 72,'Al Hussein','1',0,1,null,100,-6200 from dual union all
select 720,'Long Tom','1',0,1,null,30,-6378 from dual union all
select 720,'Mayfly','2',0,1,null,100,-6378 from dual union all
select 721,'Dorado','1',0,1,null,30,-6378 from dual union all
select 721,'Lupus','2',0,1,null,75,-6378 from dual union all
select 722,'Dorado','1',0,1,null,30,-6378 from dual union all
select 722,'Lupus 3','2',0,1,null,140,-6378 from dual union all
select 723,'Luna-M','1',0,1,null,70,-6378 from dual union all
select 724,'Lynx','1',0,1,null,10,-6378 from dual union all
select 725,'M-100 St2','2',0,1,null,85,-6370 from dual union all
select 725,'M-100','1',0,1,null,10,-6378 from dual union all
select 726,'M-100 St2','2',0,1,null,85,-6370 from dual union all
select 726,'M-100','1',0,1,null,10,-6378 from dual union all
select 727,'M-100 St2','2',0,1,null,85,-6370 from dual union all
select 727,'M-100','1',0,1,null,10,-6378 from dual union all
select 728,'M-100 St2','2',0,1,null,85,-6370 from dual union all
select 728,'M-100','1',0,1,null,10,-6378 from dual union all
select 729,'Fateh-110','1',0,1,null,60,-6378 from dual union all
select 73,'Al Hussein','1',0,1,null,100,-6200 from dual union all
select 730,'M-14','1',0,1,185220,70,-6200 from dual union all
select 730,'M-25','2',0,1,88040,230,-5000 from dual union all
select 730,'M-34','3',0,1,29580,200,200 from dual union all
select 731,'M-14','1',0,1,185220,70,-6200 from dual union all
select 731,'M-24','2',0,1,88040,230,-5000 from dual union all
select 731,'M-34','3',0,1,29580,200,200 from dual union all
select 732,'M56A1','1',0,1,null,300,-6350 from dual union all
select 733,'MCR-3','0',0,1,null,10,-6378 from dual union all
select 733,'RH-125','1',0,1,null,50,-6378 from dual union all
select 734,'Meteo','1',0,1,null,50,-6378 from dual union all
select 735,'MD-620','1',0,1,null,100,-6370 from dual union all
select 735,'NA-805','2',0,1,null,100,-6370 from dual union all
select 736,'MD-620','1',0,1,null,100,-6370 from dual union all
select 737,'MERA','1',0,1,null,10,-6378 from dual union all
select 737,'MERA','2',0,1,null,80,-6370 from dual union all
select 738,'MMR-06','1',0,1,null,60,-6378 from dual union all
select 74,'AlKahir','1',0,1,null,50,-6378 from dual union all
select 740,'MR-12','1',0,1,null,170,-6370 from dual union all
select 741,'MR-20','1',0,1,null,150,-6370 from dual union all
select 742,'M-250','1',0,1,null,150,-6370 from dual union all
select 743,'MR-30','1',0,1,null,300,-6370 from dual union all
select 744,'MR-UR-100 BR','3',0,1,null,1000,-4500 from dual union all
select 744,'MR-UR-100 St 1','1',0,1,null,70,-6300 from dual union all
select 744,'MR-UR-100 St 2','2',0,1,null,200,-5500 from dual union all
select 745,'MR-UR-100 BR','3',0,1,null,1000,-4500 from dual union all
select 745,'MR-UR-100 St 1','1',0,1,null,70,-6300 from dual union all
select 745,'MR-UR-100 St 2','2',0,1,null,200,-5500 from dual union all
select 746,'SR19','1',0,1,null,70,-6100 from dual union all
select 746,'SR19','2',0,1,null,300,-5500 from dual union all
select 747,'SR19','1',0,1,null,5,-6100 from dual union all
select 747,'SR73AJ1-ATK','2',0,1,null,70,-6100 from dual union all
select 748,'Castor 4B','1',0,1,null,300,-6370 from dual union all
select 749,'Rita I','2',1,1,null,50,-6370 from dual union all
select 749,'SEP 904','1',0,1,null,50,-6370 from dual union all
select 75,'Al Samoud','1',0,1,null,50,-6378 from dual union all
select 750,'Rita I','2',0,1,null,500,-6000 from dual union all
select 750,'SEP 904','1',0,1,null,50,-6370 from dual union all
select 751,'Rita I','2',0,1,null,500,-6000 from dual union all
select 751,'SEP 904','1',0,1,null,50,-6370 from dual union all
select 752,'Rita I','2',0,1,null,500,-5500 from dual union all
select 752,'SEP 904','1',0,1,null,50,-6370 from dual union all
select 753,'Rita I','2',1,1,null,50,-6370 from dual union all
select 753,'SEP 901','1',0,1,null,50,-6370 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 754,'Rita II','2',0,1,null,600,-5500 from dual union all
select 754,'SEP 904','1',0,1,null,50,-6370 from dual union all
select 755,'Rita II','2',0,1,null,600,-5300 from dual union all
select 755,'SEP 904','1',0,1,null,50,-6370 from dual union all
select 756,'MSBS 401','1',0,1,null,50,-6370 from dual union all
select 756,'MSBS 402','2',0,1,null,100,-6300 from dual union all
select 756,'MSBS 403','3',0,1,null,800,-5500 from dual union all
select 757,'MSBS 401','1',0,1,null,50,-6370 from dual union all
select 757,'MSBS 402','2',0,1,null,100,-6300 from dual union all
select 757,'MSBS 403','3',0,1,null,800,-5500 from dual union all
select 758,'MSBS 501','1',0,1,null,50,-6370 from dual union all
select 758,'MSBS 502','2',0,1,null,100,-6300 from dual union all
select 758,'MSBS 503','3',0,1,null,1000,-5500 from dual union all
select 759,'MT-135P','1',0,1,null,55,-6378 from dual union all
select 76,'Al Samoud 2','1',0,1,null,100,-6378 from dual union all
select 760,'MT-135P','1',0,1,null,71,-6378 from dual union all
select 761,'MT-135P','1',0,1,null,65,-6378 from dual union all
select 762,'AeroHTV Stage 2','2',0,1,null,10,-6378 from dual union all
select 762,'AeroHTV','1',0,1,null,1,-6378 from dual union all
select 763,'MX-775','1',0,1,null,10,-6378 from dual union all
select 764,'Mace','1',0,1,null,12,-6378 from dual union all
select 765,'Malemute','1',0,1,null,150,-6370 from dual union all
select 766,'Martlet 1','1',0,1,null,100,-6370 from dual union all
select 767,'Martlet 2','1',0,1,null,100,-6370 from dual union all
select 768,'Martlet 2','1',0,1,null,100,-6370 from dual union all
select 769,'Martlet 3A','1',0,1,null,100,-6370 from dual union all
select 77,'Alfa','1',0,1,null,50,-6300 from dual union all
select 770,'Martlet 3B','1',0,1,null,100,-6370 from dual union all
select 771,'Matador','1',0,1,null,12,-6378 from dual union all
select 772,'Castor 4B','1',0,1,null,700,-6370 from dual union all
select 773,'RH-125','1',0,1,null,20,-6378 from dual union all
select 773,'RH-125S','2',0,1,null,60,-6370 from dual union all
select 774,'Mesquito','1',0,1,null,50,-6378 from dual union all
select 775,'Meteo','1',0,1,null,50,-6378 from dual union all
select 776,'Meteor 1','1',0,1,null,40,-6378 from dual union all
select 777,'Meteor-2','1',0,1,null,68,-6378 from dual union all
select 778,'Meteor SRB','0',0,2,null,5,-6378 from dual union all
select 778,'Meteor-2','1',0,1,null,90,-6370 from dual union all
select 779,'Meteor 1','2',0,1,null,60,-6378 from dual union all
select 779,'Meteor 3 Booster','1',0,1,null,30,-6378 from dual union all
select 78,'SR19AJ1','1',0,1,null,100,-6300 from dual union all
select 780,'Meteor 1','2',0,1,null,60,-6378 from dual union all
select 780,'Meteor 3B Booster','1',0,1,null,30,-6378 from dual union all
select 781,'Metroc','1',0,1,null,50,-6378 from dual union all
select 782,'Super Loki','1',0,1,null,82,-6370 from dual union all
select 783,'M55E1','1',0,1,null,70,-6300 from dual union all
select 783,'Orion 38','4',0,1,null,700,700 from dual union all
select 783,'Orion 50XL','3',0,1,null,700,-4300 from dual union all
select 783,'SR19AJ1','2',0,1,null,300,-6120 from dual union all
select 784,'M55E1','1',0,1,null,75,-6250 from dual union all
select 784,'M57A1','3',0,1,null,1300,-3500 from dual union all
select 784,'SR19AJ1','2',0,1,null,350,-5600 from dual union all
select 785,'Castor 120','1',0,1,null,50,-6200 from dual union all
select 785,'Orion 38','4',0,1,null,740,740 from dual union all
select 785,'Orion 50SXLG','2',0,1,null,180,-5000 from dual union all
select 785,'Orion 50XL','3',0,1,null,714,-2836 from dual union all
select 786,'GCA/Orion 38','4',0,1,null,500,500 from dual union all
select 786,'GCA/Orion 38','5',0,1,null,500,500 from dual union all
select 786,'SR118','1',0,1,null,34,-6320 from dual union all
select 786,'SR119','2',0,1,null,120,-6050 from dual union all
select 786,'SR120','3',0,1,null,920,-3800 from dual union all
select 787,'GCA/Orion 38','4',0,1,null,500,500 from dual union all
select 787,'HAPS Stage','5',0,1,null,500,500 from dual union all
select 787,'SR118','1',0,1,null,34,-6320 from dual union all
select 787,'SR119','2',0,1,null,120,-6050 from dual union all
select 787,'SR120','3',0,1,null,920,-3800 from dual union all
select 788,'GCA/Orion 38','4',0,1,null,500,500 from dual union all
select 788,'SR118','1',0,1,null,34,-6320 from dual union all
select 788,'SR119','2',0,1,null,120,-6050 from dual union all
select 788,'SR120','3',0,1,null,920,-3800 from dual union all
select 789,'SR118','1',0,1,null,34,-6320 from dual union all
select 789,'SR119','2',0,1,null,120,-6050 from dual union all
select 789,'SR120','3',0,1,null,120,-3800 from dual union all
select 79,'Angara URM-1','0',0,4,null,200,-6000 from dual union all
select 79,'Angara URM-1','1',0,1,null,200,-6000 from dual union all
select 79,'Angara URM-2','2',0,1,null,200,200 from dual union all
select 79,'Briz-M','3',0,1,null,200,35780 from dual union all
select 790,'SR118','1',0,1,null,34,-6320 from dual union all
select 790,'SR119','2',0,1,null,120,-6050 from dual union all
select 790,'SR120','3',0,1,null,920,-3800 from dual union all
select 790,'Star 48V','4',0,1,null,500,500 from dual union all
select 791,'SR118','1',0,1,null,34,-6320 from dual union all
select 791,'SR119','2',0,1,null,120,-6050 from dual union all
select 791,'SR120','3',0,1,null,920,-3800 from dual union all
select 791,'Star 37FM','5',0,1,null,200,20000 from dual union all
select 791,'Star 48V','4',0,1,null,500,500 from dual union all
select 792,'M55E1','1',0,1,null,50,-6370 from dual union all
select 792,'M57A1','3',1,1,null,50,-6370 from dual union all
select 792,'SR19AJ1','2',1,1,null,50,-6370 from dual union all
select 793,'M55A1','1',0,1,null,1,-6378 from dual union all
select 794,'M55A1','1',0,1,null,72,-6249 from dual union all
select 794,'M56A1','2',0,1,null,328,-5630 from dual union all
select 794,'M57A1','3',0,1,null,1100,-3470 from dual union all
select 795,'M55E1','1',0,1,null,72,-6249 from dual union all
select 795,'M56A1','2',0,1,null,328,-5630 from dual union all
select 795,'M57A1','3',0,1,null,1100,-3470 from dual union all
select 796,'M55E1','1',0,1,null,75,-6250 from dual union all
select 796,'M57A1','3',0,1,null,1300,-3500 from dual union all
select 796,'SR19AJ1','2',0,1,null,350,-5600 from dual union all
select 797,'M57A1','2',0,1,null,230,-5500 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 797,'SR19AJ1','1',0,1,null,70,-6200 from dual union all
select 798,'M55E1-ATK','1',0,1,null,75,-6250 from dual union all
select 798,'PBS','4',0,1,null,1300,-3500 from dual union all
select 798,'SR19AJ1-ATK','2',0,1,null,350,-5600 from dual union all
select 798,'SR73AJ1-ATK','3',0,1,null,1200,-3500 from dual union all
select 799,'M55E1','1',0,1,null,75,-6250 from dual union all
select 799,'PBS','4',0,1,null,1300,-3500 from dual union all
select 799,'SR19AJ1','2',0,1,null,350,-5600 from dual union all
select 799,'SR73AJ1','3',0,1,null,1200,-3500 from dual union all
select 8,'V-2','1',0,1,null,100,-6300 from dual union all
select 80,'Angara URM-1','1',0,1,null,200,-6000 from dual union all
select 80,'Angara URM-2','2',0,1,null,200,-100 from dual union all
select 800,'8K74','1',0,4,null,75,-5900 from dual union all
select 800,'8K74A','2',0,1,null,200,-4500 from dual union all
select 800,'8K78I','3',0,1,null,200,200 from dual union all
select 800,'BOZ','4',0,1,null,200,200 from dual union all
select 800,'Blok-L','5',0,1,null,200,200 from dual union all
select 801,'8K74','1',0,4,null,75,-5900 from dual union all
select 801,'8K74A','2',0,1,null,200,-4500 from dual union all
select 801,'8K78I','3',0,1,null,200,200 from dual union all
select 801,'BOZ','4',0,1,null,200,200 from dual union all
select 801,'Blok-L','5',0,1,null,200,200 from dual union all
select 802,'8K74','1',0,4,null,75,-5900 from dual union all
select 802,'8K74A','2',0,1,null,200,-4500 from dual union all
select 802,'8K78I','3',0,1,null,200,200 from dual union all
select 802,'BOZ','4',0,1,null,200,200 from dual union all
select 802,'Blok-L','5',0,1,null,200,200 from dual union all
select 803,'11S510','3',0,1,null,200,200 from dual union all
select 803,'11S59','1',0,4,null,75,-5900 from dual union all
select 803,'11S59A','2',0,1,null,200,-4500 from dual union all
select 803,'BOZ','4',0,1,null,200,200 from dual union all
select 803,'Blok-2BL','5',0,1,null,200,200 from dual union all
select 804,'11S510','3',0,1,null,200,200 from dual union all
select 804,'11S59','1',0,4,null,75,-5900 from dual union all
select 804,'11S59A','2',0,1,null,200,-4500 from dual union all
select 804,'BOZ','4',0,1,null,200,200 from dual union all
select 804,'Blok-L','5',0,1,null,200,200 from dual union all
select 805,'11S510','3',0,1,null,200,200 from dual union all
select 805,'11S59','1',0,4,null,75,-5900 from dual union all
select 805,'11S59A','2',0,1,null,200,-4500 from dual union all
select 805,'BOZ','4',0,1,null,200,200 from dual union all
select 805,'Blok-ML','5',0,1,null,200,200 from dual union all
select 806,'11S510','3',0,1,null,200,200 from dual union all
select 806,'11S59','1',0,4,null,75,-5900 from dual union all
select 806,'11S59A','2',0,1,null,200,-4500 from dual union all
select 806,'BOZ','4',0,1,null,200,200 from dual union all
select 806,'Blok-NVL','5',0,1,null,200,200 from dual union all
select 807,'11S510','3',0,1,null,200,200 from dual union all
select 807,'11S59','1',0,4,null,75,-5900 from dual union all
select 807,'11S59A','2',0,1,null,200,-4500 from dual union all
select 807,'BOZ','4',0,1,null,200,200 from dual union all
select 807,'Blok SO-L','5',0,1,null,200,200 from dual union all
select 808,'11S510','3',0,1,null,200,200 from dual union all
select 808,'11S59','1',0,4,null,75,-5900 from dual union all
select 808,'11S59A','2',0,1,null,200,-4500 from dual union all
select 808,'BOZ','4',0,1,null,200,200 from dual union all
select 808,'Blok-VL','5',0,1,null,200,200 from dual union all
select 809,'11S510','3',0,1,null,200,200 from dual union all
select 809,'11S59','1',0,4,null,75,-5900 from dual union all
select 809,'11S59A','2',0,1,null,200,-4500 from dual union all
select 809,'BOZ','4',0,1,null,200,200 from dual union all
select 809,'Blok-L','5',0,1,null,200,200 from dual union all
select 81,'Melanie','4',0,1,null,150,-6100 from dual union all
select 81,'Mimosa','3',0,1,null,150,-6300 from dual union all
select 81,'Neptune','2',0,1,null,50,-6350 from dual union all
select 81,'Vesuve','1',0,1,null,20,-6370 from dual union all
select 810,'11S510','3',0,1,null,200,200 from dual union all
select 810,'11S59','1',0,4,null,75,-5900 from dual union all
select 810,'11S59A','2',0,1,null,200,-4500 from dual union all
select 810,'BOZ','4',0,1,null,200,200 from dual union all
select 810,'Blok-2BL','5',0,1,null,200,200 from dual union all
select 811,'11S510','3',0,1,null,200,200 from dual union all
select 811,'11S59','1',0,4,null,75,-5900 from dual union all
select 811,'11S59A','2',0,1,null,200,-4500 from dual union all
select 811,'BOZ','4',0,1,null,200,200 from dual union all
select 811,'Blok-ML','5',0,1,null,200,200 from dual union all
select 812,'Momo','1',0,1,null,100,-6378 from dual union all
select 813,'Melanie 1','1',0,1,null,5,-6378 from dual union all
select 813,'Oreste','3',0,1,null,50,-6370 from dual union all
select 813,'Theodore','2',0,1,null,20,-6370 from dual union all
select 814,'Melanie 1','1',0,1,null,5,-6378 from dual union all
select 814,'Prosper','2',0,1,null,30,-6370 from dual union all
select 814,'Prosper','3',0,1,null,90,-6370 from dual union all
select 815,'M-10','1',0,1,null,34,-6370 from dual union all
select 815,'M-20','2',1,1,null,34,-6370 from dual union all
select 815,'M-30','3',1,1,null,34,-6370 from dual union all
select 815,'M-40','4',1,1,null,34,-6370 from dual union all
select 815,'SB-310','0',0,8,null,10,-6378 from dual union all
select 816,'M-10','1',0,1,null,120,-6150 from dual union all
select 816,'M-22TVC','2',0,1,null,290,-5300 from dual union all
select 816,'M-3A','3',0,1,null,3000,280 from dual union all
select 816,'SB-310','0',0,8,null,10,-6378 from dual union all
select 817,'M-10','1',0,1,null,75,-6300 from dual union all
select 817,'M-20','2',0,1,null,160,-5500 from dual union all
select 817,'M-30','3',1,1,null,160,-5500 from dual union all
select 817,'M-40','4',0,1,null,160,-5500 from dual union all
select 817,'SB-310','0',0,8,null,10,-6378 from dual union all
select 818,'KM-H','4',0,1,null,3800,800 from dual union all
select 818,'M-13','1',0,1,null,120,-6000 from dual union all
select 818,'M-22TVC','2',0,1,null,328,-4450 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 818,'M-3A','3',0,1,null,800,300 from dual union all
select 818,'SB-310','0',0,8,null,10,-6378 from dual union all
select 819,'M-13TVC','1',0,1,null,120,-6000 from dual union all
select 819,'M-22TVC','2',0,1,null,520,-4900 from dual union all
select 819,'M-3A','3',0,1,null,500,500 from dual union all
select 819,'SB-310','0',0,8,null,10,-6378 from dual union all
select 82,'Antares Stage 1','1',0,1,null,-5200,200 from dual union all
select 82,'Castor 30','2',0,1,null,200,200 from dual union all
select 820,'M-13','1',0,1,null,120,-6000 from dual union all
select 820,'M-23','2',0,1,null,240,-4900 from dual union all
select 820,'M-3B','3',0,1,null,400,200 from dual union all
select 820,'SB-735','0',0,2,null,10,-6378 from dual union all
select 821,'M-10','1',0,1,44950,75,-6300 from dual union all
select 821,'M-20','2',0,1,18810,280,-5000 from dual union all
select 821,'M-30','3',0,1,5376,1000,-1670 from dual union all
select 821,'M-40','4',0,1,1060,1000,1000 from dual union all
select 821,'SB-310','0',0,8,5624,10,-6370 from dual union all
select 822,'Castor 2','0',0,3,null,50,-6100 from dual union all
select 822,'LE-3','2',0,1,null,1000,-3200 from dual union all
select 822,'LT Thor N','1',0,1,null,150,-5500 from dual union all
select 822,'Star 37N','3',0,1,null,1000,1000 from dual union all
select 823,'N-1 Blok-A','1',0,1,4980000,50,-6300 from dual union all
select 823,'N-1 Blok-B','2',0,1,1820000,100,-6300 from dual union all
select 823,'N-1 Blok-V','3',0,1,642000,200,-6300 from dual union all
select 824,'AJ10-118FJ','2',0,1,null,1000,1000 from dual union all
select 824,'Castor 2','0',0,9,null,50,-6100 from dual union all
select 824,'ELT Thor N','1',0,1,null,150,-5500 from dual union all
select 825,'AJ10-118FJ','2',0,1,null,500,170 from dual union all
select 825,'Castor 2','0',0,9,null,50,-6100 from dual union all
select 825,'ELT Thor N','1',0,1,null,150,-5500 from dual union all
select 825,'Star 37E','3',0,1,null,35800,180 from dual union all
select 826,'NAL-16','1',0,1,null,100,-6370 from dual union all
select 827,'NAL-25','1',0,1,null,50,-6370 from dual union all
select 828,'NAL-7','1',0,1,null,20,-6370 from dual union all
select 829,'NAL-735','1',0,1,null,10,-6378 from dual union all
select 83,'Antares Stage 1','1',0,1,null,-5200,200 from dual union all
select 83,'Castor 30B','2',0,1,null,200,200 from dual union all
select 830,'RTV-A-3','1',0,1,null,15,-6378 from dual union all
select 831,'KSLV-1 Stage 2','2',0,1,null,200,200 from dual union all
select 831,'Naro Stage 1','1',0,1,null,200,-6000 from dual union all
select 832,'Navaho Booster','1',0,1,null,25,-6378 from dual union all
select 832,'Navaho Missile','2',0,1,null,25,-6378 from dual union all
select 833,'Nazeat','1',0,1,null,60,-6378 from dual union all
select 834,'New Shepard PM','1',0,1,null,100,-6300 from dual union all
select 835,'Nike','1',0,1,null,10,-6378 from dual union all
select 836,'Ajax','2',0,1,null,30,-6378 from dual union all
select 836,'Nike','1',0,1,null,2,-6378 from dual union all
select 837,'Apache','2',0,1,null,110,-6300 from dual union all
select 837,'Nike','1',0,1,null,7,-6378 from dual union all
select 838,'Asp IV','2',0,1,null,200,-6370 from dual union all
select 838,'Nike','1',0,1,null,24,-6378 from dual union all
select 839,'Cajun','2',0,1,null,120,-6370 from dual union all
select 839,'Nike','1',0,1,null,24,-6378 from dual union all
select 84,'Antares Stage 1','1',0,1,null,-5200,200 from dual union all
select 84,'Castor 30XL','2',0,1,null,200,200 from dual union all
select 840,'2.5-DS-59000','1',0,1,null,20,-6378 from dual union all
select 840,'Deacon','2',0,1,null,120,-6370 from dual union all
select 841,'Deacon','2',0,1,null,120,-6370 from dual union all
select 841,'Nike','1',0,1,null,20,-6378 from dual union all
select 842,'Hawk','2',0,1,null,162,-6370 from dual union all
select 842,'Nike','1',0,1,null,20,-6378 from dual union all
select 843,'Hercules Booster','1',0,1,null,2,-6378 from dual union all
select 843,'TX-30','2',0,1,null,20,-6378 from dual union all
select 844,'Ajax Cluster 1810','2',0,1,null,20,-6378 from dual union all
select 844,'Hercules Booster','1',0,1,null,2,-6378 from dual union all
select 845,'Ajax Cluster','2',0,1,null,20,-6378 from dual union all
select 845,'Hercules Booster','1',0,1,null,2,-6378 from dual union all
select 846,'Hydac','2',0,1,null,150,-6370 from dual union all
select 846,'Nike','1',0,1,null,15,-6378 from dual union all
select 847,'Improved Orion','2',0,1,416,140,-6370 from dual union all
select 847,'Nike','1',0,1,665,25,-6378 from dual union all
select 848,'Iroquois','2',0,1,188,200,-6370 from dual union all
select 848,'Nike','1',0,1,null,20,-6378 from dual union all
select 849,'Hercules Booster','1',0,1,null,2,-6378 from dual union all
select 849,'TX-30','2',0,1,null,20,-6378 from dual union all
select 85,'Antares 2 Stage 1','1',0,1,null,-5200,200 from dual union all
select 85,'Castor 30XL','2',0,1,null,200,200 from dual union all
select 850,'Javelin 3','2',0,1,null,130,-6370 from dual union all
select 850,'Nike','1',0,1,null,20,-6378 from dual union all
select 851,'Malemute','2',0,1,1210,500,-6370 from dual union all
select 851,'Nike','1',0,1,665,20,-6378 from dual union all
select 852,'Nike','1',0,1,null,20,-6378 from dual union all
select 852,'Nike','2',0,1,null,50,-6370 from dual union all
select 853,'Cajun','3',0,1,null,120,-6370 from dual union all
select 853,'Nike','1',0,1,null,24,-6378 from dual union all
select 853,'Nike','2',0,1,null,24,-6378 from dual union all
select 854,'Deacon','3',0,1,null,13,-6378 from dual union all
select 854,'Nike','1',0,1,null,10,-6378 from dual union all
select 854,'Nike','2',0,1,null,13,-6378 from dual union all
select 855,'HPAG','3',0,1,null,10,-6378 from dual union all
select 855,'Nike','1',0,1,null,5,-6378 from dual union all
select 855,'Nike','2',0,1,null,10,-6378 from dual union all
select 856,'Nike','1',0,1,null,5,-6378 from dual union all
select 856,'Nike','2',0,1,null,20,-6378 from dual union all
select 856,'Recruit','3',0,1,null,120,-6350 from dual union all
select 857,'Nike','1',0,1,null,10,-6378 from dual union all
select 857,'Nike','2',0,1,null,25,-6378 from dual union all
select 857,'T-40','3',0,1,null,50,-6378 from dual union all
select 858,'Nike','1',0,1,null,15,-6378 from dual union all
select 858,'Nike','2',0,1,null,40,-6378 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 858,'T-40','3',0,1,null,100,-6370 from dual union all
select 858,'T-55','4',0,1,null,352,-6370 from dual union all
select 859,'Deacon','3',0,3,null,18,-6378 from dual union all
select 859,'Nike','1',0,1,null,10,-6378 from dual union all
select 859,'Nike','2',0,1,null,18,-6378 from dual union all
select 859,'T-40','4',0,1,null,18,-6378 from dual union all
select 86,'Apache','1',0,1,null,60,-6378 from dual union all
select 860,'Nike','1',0,1,665,25,-6378 from dual union all
select 860,'Orion','2',0,1,416,140,-6370 from dual union all
select 861,'Nike','1',0,1,null,5,-6378 from dual union all
select 861,'Recruit','2',0,1,null,10,-6378 from dual union all
select 862,'Nike','1',0,1,null,5,-6378 from dual union all
select 862,'T-40','2',0,1,null,10,-6378 from dual union all
select 862,'T-55','3',0,1,null,10,-6378 from dual union all
select 863,'Nike-490 Sustainer','2',0,1,null,40,-6370 from dual union all
select 863,'T48','1',0,1,null,5,-6370 from dual union all
select 864,'Nike','1',0,1,665,28,-6364 from dual union all
select 864,'Tomahawk','2',0,1,503,290,-6321 from dual union all
select 865,'Nike','1',0,1,665,28,-6364 from dual union all
select 865,'Tomahawk','2',0,1,503,290,-6321 from dual union all
select 866,'Nike','1',0,1,665,28,-6364 from dual union all
select 866,'Tomahawk 1','2',0,1,503,290,-6321 from dual union all
select 867,'Nike','1',0,1,665,28,-6364 from dual union all
select 867,'Tomahawk','2',0,1,503,290,-6321 from dual union all
select 868,'Nike','1',0,1,null,20,-6370 from dual union all
select 868,'Viper I','2',0,1,null,80,-6360 from dual union all
select 869,'Nike-490 Sustainer','2',0,1,null,40,-6370 from dual union all
select 869,'X-201','1',0,1,null,5,-6370 from dual union all
select 87,'Apollo LES','1',0,1,null,2,-6300 from dual union all
select 870,'Nike','1',0,1,null,24,-6378 from dual union all
select 870,'Yardbird','2',0,1,null,120,-6370 from dual union all
select 871,'TX-135','1',0,1,null,15,-6370 from dual union all
select 871,'TX-238','2',0,1,null,150,-6350 from dual union all
select 871,'TX-239','3',0,1,null,200,-6350 from dual union all
select 872,'TX-135','1',0,1,null,15,-6370 from dual union all
select 872,'TX-238','2',0,1,null,150,-6350 from dual union all
select 873,'TX-135','1',0,1,null,10,-6378 from dual union all
select 873,'Zeus A','2',0,1,null,50,-6370 from dual union all
select 874,'TX-135','1',0,1,null,15,-6370 from dual union all
select 874,'TX-238','2',0,1,null,150,-6350 from dual union all
select 874,'TX-239','3',0,1,null,200,-6350 from dual union all
select 875,'Nike-46 Sustainer','2',0,1,null,40,-6370 from dual union all
select 875,'Nike-46','1',0,1,null,5,-6370 from dual union all
select 876,'Nike-47 Sustainer','2',0,1,null,40,-6370 from dual union all
select 876,'Nike-47','1',0,1,null,5,-6370 from dual union all
select 877,'Nike-47 Sustainer','2',0,1,null,40,-6370 from dual union all
select 877,'X-201','1',0,1,null,5,-6370 from dual union all
select 878,'Nike-46','1',0,1,null,5,-6370 from dual union all
select 878,'Nike-47 Sustainer','2',0,1,null,40,-6370 from dual union all
select 879,'Nike-47 Sustainer','2',1,1,null,40,-6370 from dual union all
select 879,'X-201','1',0,1,null,5,-6370 from dual union all
select 88,'Arcas','1',0,1,null,90,-6300 from dual union all
select 880,'Nike-484 Sustainer','2',0,1,null,40,-6370 from dual union all
select 880,'X-201','1',0,1,null,5,-6370 from dual union all
select 881,'Nike-490 Sustainer','2',0,1,null,40,-6370 from dual union all
select 881,'X-201','1',0,1,null,5,-6370 from dual union all
select 882,'Nudol'' Booster','1',0,1,null,10,-6370 from dual union all
select 882,'Nudol''','2',0,1,null,100,-6300 from dual union all
select 883,'Melanie','4',0,1,null,150,-6300 from dual union all
select 883,'Mimosa','3',0,1,null,150,-6300 from dual union all
select 883,'Neptune','2',0,1,null,50,-6350 from dual union all
select 883,'Vesuve','1',0,1,null,20,-6370 from dual union all
select 884,'OS-X0','1',0,1,null,40,-6378 from dual union all
select 885,'OTRAG','1',0,1,null,50,-6370 from dual union all
select 886,'Oka','1',0,1,null,80,-6370 from dual union all
select 887,'Oriole','2',0,1,null,368,-6378 from dual union all
select 887,'Terrier','1',0,1,null,10,-6378 from dual union all
select 888,'Oriole','3',0,1,null,300,-6378 from dual union all
select 888,'Talos','1',0,1,null,1,-6378 from dual union all
select 888,'Terrier','2',0,1,null,12,-6378 from dual union all
select 889,'Nihka','3',0,1,null,800,-6378 from dual union all
select 889,'Oriole','2',0,1,null,300,-6378 from dual union all
select 889,'Terrier','1',0,1,null,12,-6378 from dual union all
select 89,'Arcon booster','0',0,1,null,1,-6300 from dual union all
select 89,'Arcon','1',0,1,null,60,-6300 from dual union all
select 890,'Nihka','4',0,1,null,800,-6378 from dual union all
select 890,'Oriole','3',0,1,null,300,-6378 from dual union all
select 890,'Talos','1',0,1,null,1,-6378 from dual union all
select 890,'Terrier Mk 70','2',0,1,null,12,-6378 from dual union all
select 891,'Orion','1',0,1,null,88,-6370 from dual union all
select 892,'Orion LAS','1',0,1,null,2,-6300 from dual union all
select 893,'Orion-1','1',0,1,null,90,-6370 from dual union all
select 894,'Orion-2','1',0,1,null,90,-6370 from dual union all
select 895,'Prithvi','1',0,1,null,100,-6370 from dual union all
select 896,'PDV Target St2','2',0,1,null,100,-6370 from dual union all
select 896,'Prithvi','1',0,1,null,100,-6370 from dual union all
select 897,'5Ya26','1',0,1,null,100,-6300 from dual union all
select 898,'53T6','1',0,1,null,100,-6300 from dual union all
select 899,'HPS3','3',0,1,null,800,-4000 from dual union all
select 899,'PS2','2',0,1,null,350,-5500 from dual union all
select 899,'PS4','4',0,1,null,800,800 from dual union all
select 899,'PSOM','0',0,6,null,50,-6300 from dual union all
select 899,'S139','1',0,1,null,90,-6100 from dual union all
select 9,'V-2','1',0,1,null,80,-6300 from dual union all
select 90,'Ares I-X','1',0,1,null,45,-6200 from dual union all
select 900,'HPS3','3',0,1,null,800,-4000 from dual union all
select 900,'PS2','2',0,1,null,350,-5500 from dual union all
select 900,'PS4','4',0,1,null,800,800 from dual union all
select 900,'S139','1',0,1,null,90,-6100 from dual union all
select 901,'PS2','2',0,1,null,350,-5500 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 901,'PS3','3',0,1,null,800,-4000 from dual union all
select 901,'PS4','4',0,1,null,800,800 from dual union all
select 901,'PSOM','0',0,6,null,50,-6300 from dual union all
select 901,'S125','1',0,1,null,90,-6100 from dual union all
select 902,'PS2','2',0,1,null,350,-5500 from dual union all
select 902,'PS3','3',0,1,null,800,-4000 from dual union all
select 902,'PS4','4',0,1,null,800,800 from dual union all
select 902,'PSOM-XL','0',0,6,null,50,-6300 from dual union all
select 902,'S139','1',0,1,null,90,-6100 from dual union all
select 903,'Hercules Booster','1',0,1,null,2,-6378 from dual union all
select 903,'TX-30','2',0,1,null,20,-6378 from dual union all
select 904,'Taepo Dong 1 Stage 2','2',0,1,null,220,-5000 from dual union all
select 904,'Taepo Dong 1 Stage 3','3',0,1,null,220,-5000 from dual union all
select 904,'Taepo Dong 1','1',0,1,null,70,-6100 from dual union all
select 905,'Apache','2',0,1,null,200,-6370 from dual union all
select 905,'Paiute','1',0,1,null,20,-6378 from dual union all
select 906,'Paiute','1',0,1,null,20,-6378 from dual union all
select 906,'Tomahawk','2',0,1,null,200,-6370 from dual union all
select 907,'Peacekeeper PBPS','4',0,1,null,1000,-3700 from dual union all
select 907,'SR118','1',0,1,null,34,-6320 from dual union all
select 907,'SR119','2',0,1,null,120,-6050 from dual union all
select 907,'SR120','3',0,1,null,920,-3800 from dual union all
select 908,'Pedro','1',0,1,null,17,-6378 from dual union all
select 908,'Recruit','2',0,1,null,17,-6378 from dual union all
select 909,'NB-52','0',0,1,null,14,-6378 from dual union all
select 909,'Orion 38','3',0,1,null,740,740 from dual union all
select 909,'Orion 50','2',0,1,null,740,-4500 from dual union all
select 909,'Orion 50S','1',0,1,null,90,-6100 from dual union all
select 91,'Ariane H8','3',0,1,null,37800,200 from dual union all
select 91,'Ariane L144','1',0,1,null,80,-6130 from dual union all
select 91,'Ariane L33','2',0,1,null,160,-4530 from dual union all
select 910,'L-1011','0',0,1,null,14,-6378 from dual union all
select 910,'Orion 38','3',0,1,null,740,740 from dual union all
select 910,'Orion 50','2',0,1,null,740,-4500 from dual union all
select 910,'Orion 50S','1',0,1,null,90,-6100 from dual union all
select 911,'L-1011','0',0,1,null,14,-6378 from dual union all
select 911,'Orion 38','3',0,1,null,700,700 from dual union all
select 911,'Orion 50XL','2',0,1,null,700,-4500 from dual union all
select 911,'Pegasus XL Stage 1','1',0,1,null,90,-6100 from dual union all
select 912,'HAPS','4',0,1,null,700,700 from dual union all
select 912,'L-1011','0',0,1,null,14,-6378 from dual union all
select 912,'Orion 38','3',0,1,null,700,120 from dual union all
select 912,'Orion 50XL','2',0,1,null,120,-4400 from dual union all
select 912,'Pegasus XL Stage 1','1',0,1,null,70,-6100 from dual union all
select 913,'HAPS','4',0,1,null,700,700 from dual union all
select 913,'NB-52','0',0,1,null,14,-6378 from dual union all
select 913,'Orion 38','3',0,1,null,700,120 from dual union all
select 913,'Orion 50','2',0,1,null,120,-4400 from dual union all
select 913,'Orion 50S','1',0,1,null,70,-6100 from dual union all
select 914,'Perimetr Stage 1','1',0,1,null,100,-6100 from dual union all
select 914,'Perimetr Stage 2','2',0,1,null,1000,-5000 from dual union all
select 915,'15Zh58A','1',0,1,null,70,-6300 from dual union all
select 915,'15Zh58B','2',0,1,null,200,-6000 from dual union all
select 915,'15Zh58V','3',0,1,null,1000,-4000 from dual union all
select 916,'TX-174','1',0,1,null,40,-6300 from dual union all
select 916,'TX-175','2',0,1,null,250,-6000 from dual union all
select 917,'TX-174','1',0,1,null,40,-6300 from dual union all
select 917,'TX-175','2',0,1,null,250,-6000 from dual union all
select 918,'Pershing 2 St1','1',0,1,null,40,-6300 from dual union all
select 918,'Pershing 2 St1','2',1,1,null,40,-6300 from dual union all
select 919,'Pershing 2 St1','1',0,1,null,40,-6300 from dual union all
select 919,'Pershing 2 St2','2',0,1,null,250,-6000 from dual union all
select 92,'Ariane H10','3',0,1,null,37800,200 from dual union all
select 92,'Ariane L144','1',0,1,null,75,-6120 from dual union all
select 92,'Ariane L33','2',0,1,null,160,-4760 from dual union all
select 920,'Chick x3','0',0,1,null,20,-6370 from dual union all
select 920,'Petrel','1',0,1,null,150,-6370 from dual union all
select 921,'Chick x4','0',0,1,null,20,-6370 from dual union all
select 921,'Petrel 2','1',0,1,null,150,-6370 from dual union all
select 922,'RT-21 St1','1',0,1,null,70,-6370 from dual union all
select 922,'RT-21 St2','2',0,1,null,500,-6300 from dual union all
select 923,'RT-21 St1','1',0,1,null,70,-6370 from dual union all
select 923,'RT-21 St2','2',0,1,null,500,-6300 from dual union all
select 924,'RT-21 St1','1',0,1,null,70,-6370 from dual union all
select 924,'RT-21 St2','2',0,1,null,500,-6300 from dual union all
select 925,'Deacon','1',0,1,null,25,-6370 from dual union all
select 926,'Cajun','1',0,1,null,25,-6370 from dual union all
select 927,'Asp','1',0,1,null,25,-6370 from dual union all
select 928,'A1P Stage 2','2',0,1,null,50,-6370 from dual union all
select 928,'A1P','1',0,1,null,10,-6378 from dual union all
select 929,'A2P Stage 2','2',0,1,null,20,-6378 from dual union all
select 929,'A2P','1',0,1,null,500,-6300 from dual union all
select 93,'Ariane H10','3',0,1,null,37800,200 from dual union all
select 93,'Ariane L144','1',0,1,null,87,-6060 from dual union all
select 93,'Ariane L33','2',0,1,null,160,-4500 from dual union all
select 93,'PAP','0',0,2,null,4,-6370 from dual union all
select 930,'A2E Stage 2','2',0,1,null,1000,-6000 from dual union all
select 930,'A2E','1',0,1,null,70,-6300 from dual union all
select 931,'A3P','1',0,1,null,50,-6370 from dual union all
select 931,'X-260','2',0,1,null,1000,-6000 from dual union all
select 932,'A3P','1',0,1,null,50,-6370 from dual union all
select 932,'X-260','2',0,1,null,1000,-6000 from dual union all
select 933,'A1P Stage 2','2',0,1,null,200,-6370 from dual union all
select 933,'A1P','1',0,1,null,10,-6378 from dual union all
select 934,'Sergeant','1',0,1,null,140,-6300 from dual union all
select 935,'Recruit','2',0,3,null,140,-6000 from dual union all
select 935,'Recruit','3',0,1,null,140,-5800 from dual union all
select 935,'Sergeant','1',0,1,null,140,-6300 from dual union all
select 936,'10KS2500','1',0,1,null,10,-6378 from dual union all
select 937,'10KS2500','2',0,1,null,40,-6378 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 937,'TX-20 Sergeant','1',0,1,null,10,-6378 from dual union all
select 938,'Polaris TV','1',0,1,null,500,-6370 from dual union all
select 939,'C-3 Stage 1','1',0,1,null,50,-6370 from dual union all
select 939,'C-3 Stage 2','2',0,1,null,500,-6300 from dual union all
select 94,'Ariane H10','3',0,1,45144,800,800 from dual union all
select 94,'Ariane L220','1',0,1,621970,80,-5800 from dual union all
select 94,'Ariane L33','2',0,1,98910,180,-3700 from dual union all
select 940,'AAD','1',0,1,null,40,-6370 from dual union all
select 941,'Prithvi','1',0,1,null,100,-6370 from dual union all
select 942,'Prithvi','1',0,1,null,100,-6370 from dual union all
select 943,'Private A Booster','0',0,1,null,1,-6378 from dual union all
select 943,'Private A','1',0,1,null,1,-6378 from dual union all
select 944,'Private A Booster','0',0,1,null,1,-6378 from dual union all
select 944,'Private F','1',0,1,null,1,-6378 from dual union all
select 945,'HOTROC','1',0,2,null,40,-6370 from dual union all
select 945,'HOTROC','2',1,2,null,40,-6370 from dual union all
select 945,'NOTS 3in Sph.','5',1,1,null,40,-6370 from dual union all
select 945,'NOTS 8in','4',1,1,null,40,-6370 from dual union all
select 945,'X-241','3',1,1,null,40,-6370 from dual union all
select 946,'F4D-1 Skyray','0',0,1,null,12,-6378 from dual union all
select 946,'HOTROC','1',0,2,null,48,-6360 from dual union all
select 946,'HOTROC','2',0,2,null,81,-6300 from dual union all
select 946,'NOTS 3in Sph.','5',0,1,null,2500,90 from dual union all
select 946,'NOTS 8in','4',0,1,null,730,-330 from dual union all
select 946,'X-241','3',0,1,null,83,-4300 from dual union all
select 947,'Proson St1','1',0,1,null,1,-6300 from dual union all
select 947,'Proson St2','2',0,1,null,1,-6300 from dual union all
select 948,'Castor 4A','1',0,1,null,1,-6378 from dual union all
select 949,'8S810','1',0,1,null,60,-6200 from dual union all
select 949,'8S811','2',0,1,null,140,-4900 from dual union all
select 949,'8S812','3',0,1,null,180,180 from dual union all
select 95,'Ariane H10+','3',0,1,47025,800,800 from dual union all
select 95,'Ariane L220','1',0,1,621970,80,-5800 from dual union all
select 95,'Ariane L33','2',0,1,98910,180,-3700 from dual union all
select 950,'17S40','5',0,1,null,180,180 from dual union all
select 950,'8S810','1',0,1,null,60,-6200 from dual union all
select 950,'8S811','2',0,1,null,130,-5000 from dual union all
select 950,'8S812','3',0,1,null,180,180 from dual union all
select 950,'Perekhodnik','4',0,1,null,180,180 from dual union all
select 950,'SOZ','6',0,2,null,180,180 from dual union all
select 951,'17S40','5',0,1,null,180,180 from dual union all
select 951,'8S810','1',0,1,null,60,-6200 from dual union all
select 951,'8S811','2',0,1,null,130,-5000 from dual union all
select 951,'8S812','3',0,1,null,180,180 from dual union all
select 951,'Perekhodnik','4',0,1,null,180,180 from dual union all
select 951,'SOZ','6',0,2,null,180,180 from dual union all
select 952,'8S810','1',0,1,null,60,-6200 from dual union all
select 952,'8S811','2',0,1,null,130,-5000 from dual union all
select 952,'8S812','3',0,1,null,160,-1000 from dual union all
select 952,'Briz-M','4',0,1,null,200,200 from dual union all
select 953,'11S824','5',0,1,null,180,180 from dual union all
select 953,'8S810','1',0,1,null,60,-6200 from dual union all
select 953,'8S811','2',0,1,null,130,-5000 from dual union all
select 953,'8S812','3',0,1,null,160,-1000 from dual union all
select 953,'Perekhodnik','4',0,1,null,160,-1000 from dual union all
select 953,'SOZ','6',0,2,null,180,180 from dual union all
select 954,'11S824M','5',0,1,null,180,180 from dual union all
select 954,'8S810','1',0,1,null,60,-6200 from dual union all
select 954,'8S811','2',0,1,null,130,-5000 from dual union all
select 954,'8S812','3',0,1,null,160,-1000 from dual union all
select 954,'Perekhodnik','4',0,1,null,160,-1000 from dual union all
select 954,'SOZ','6',0,2,null,180,180 from dual union all
select 955,'11S824F','5',0,1,null,180,180 from dual union all
select 955,'8S810','1',0,1,null,60,-6200 from dual union all
select 955,'8S811','2',0,1,null,130,-5000 from dual union all
select 955,'8S812','3',0,1,null,160,-900 from dual union all
select 955,'Perekhodnik','4',0,1,null,160,-900 from dual union all
select 955,'SOZ','6',0,2,null,180,180 from dual union all
select 956,'11S86','5',0,1,null,180,180 from dual union all
select 956,'8S810','1',0,1,null,60,-6200 from dual union all
select 956,'8S811','2',0,1,null,130,-5000 from dual union all
select 956,'8S812','3',0,1,null,180,180 from dual union all
select 956,'Perekhodnik','4',0,1,null,180,180 from dual union all
select 956,'SOZ','6',0,2,null,180,180 from dual union all
select 957,'8S810','1',0,1,null,60,-6200 from dual union all
select 957,'8S811','2',0,1,null,130,-5000 from dual union all
select 957,'8S812','3',0,1,null,180,180 from dual union all
select 957,'Blok DM1','5',0,1,null,180,180 from dual union all
select 957,'Perekhodnik','4',0,1,null,180,180 from dual union all
select 957,'SOZ','6',0,2,null,180,180 from dual union all
select 958,'11S861','5',0,1,null,180,180 from dual union all
select 958,'8S810','1',0,1,null,60,-6200 from dual union all
select 958,'8S811','2',0,1,null,130,-5000 from dual union all
select 958,'8S812','3',0,1,null,180,180 from dual union all
select 958,'Perekhodnik','4',0,1,null,180,180 from dual union all
select 958,'SOZ','6',0,2,null,180,180 from dual union all
select 959,'11S861-01','5',0,1,null,180,180 from dual union all
select 959,'8S810','1',0,1,null,60,-6200 from dual union all
select 959,'8S811','2',0,1,null,130,-5000 from dual union all
select 959,'8S812','3',0,1,null,180,180 from dual union all
select 959,'Perekhodnik','4',0,1,null,180,180 from dual union all
select 959,'SOZ','6',0,2,null,37800,180 from dual union all
select 96,'Ariane H10-3','3',0,1,48906,800,800 from dual union all
select 96,'Ariane L220','1',0,1,621970,80,-5800 from dual union all
select 96,'Ariane L33','2',0,1,98910,180,-3700 from dual union all
select 960,'8S810','1',0,1,null,60,-6200 from dual union all
select 960,'8S811','2',0,1,null,130,-5000 from dual union all
select 960,'8S812','3',0,1,null,180,180 from dual union all
select 960,'Blok DM3','5',0,1,null,180,180 from dual union all
select 960,'Perekhodnik','4',0,1,null,180,180 from dual;
insert into LAUNCH_VEHICLE_STAGE
select 960,'SOZ','6',0,2,null,180,180 from dual union all
select 961,'8S810','1',0,1,null,60,-6200 from dual union all
select 961,'8S811','2',0,1,null,130,-5000 from dual union all
select 961,'8S812','3',0,1,null,180,180 from dual union all
select 961,'Blok DM4','5',0,1,null,180,180 from dual union all
select 961,'Perekhodnik','4',0,1,null,180,180 from dual union all
select 961,'SOZ','6',0,2,null,180,180 from dual union all
select 962,'8S810M','1',0,1,null,60,-6200 from dual union all
select 962,'8S811','2',0,1,null,130,-5000 from dual union all
select 962,'8S812M','3',0,1,null,180,180 from dual union all
select 962,'Briz-M','4',0,1,null,180,180 from dual union all
select 963,'11S861','5',0,1,null,180,180 from dual union all
select 963,'8S810M','1',0,1,null,60,-6200 from dual union all
select 963,'8S811','2',0,1,null,130,-5000 from dual union all
select 963,'8S812M','3',0,1,null,180,180 from dual union all
select 963,'Perekhodnik','4',0,1,null,180,180 from dual union all
select 963,'SOZ','6',0,2,null,180,180 from dual union all
select 964,'11S861-03','5',0,1,null,180,180 from dual union all
select 964,'8S810M','1',0,1,null,60,-6200 from dual union all
select 964,'8S811','2',0,1,null,130,-5000 from dual union all
select 964,'8S812M','3',0,1,null,180,180 from dual union all
select 964,'Perekhodnik','4',0,1,null,180,180 from dual union all
select 964,'SOZ','6',0,2,null,180,180 from dual union all
select 965,'V-750 Booster','0',0,1,null,10,-6378 from dual union all
select 965,'V-750','1',0,1,null,27,-6378 from dual union all
select 966,'V-750 Booster','0',0,1,null,10,-6378 from dual union all
select 966,'V-750','1',0,1,null,27,-6378 from dual union all
select 967,'Hwasong 6','1',0,1,null,200,-6250 from dual union all
select 968,'Deacon','1',0,4,null,10,-6378 from dual union all
select 969,'R-1','1',0,1,null,100,-6280 from dual union all
select 97,'Ariane H10','3',0,1,45144,35787,200 from dual union all
select 97,'Ariane L220','1',0,1,621970,90,-5720 from dual union all
select 97,'Ariane L33','2',0,1,98910,180,-3700 from dual union all
select 97,'PAL','0',0,2,212380,5,-6300 from dual union all
select 970,'R-1','1',0,1,null,100,-6280 from dual union all
select 971,'R-11','1',0,1,null,200,-6250 from dual union all
select 972,'R-11','1',0,1,null,200,-6250 from dual union all
select 973,'R-11','1',0,1,null,200,-6250 from dual union all
select 974,'R-11FM','1',0,1,null,200,-6250 from dual union all
select 975,'R-11','1',0,1,null,200,-6250 from dual union all
select 976,'R-12','1',0,1,null,500,-6200 from dual union all
select 977,'R-12','1',0,1,null,500,-6200 from dual union all
select 978,'R-13','1',0,1,null,150,-6300 from dual union all
select 979,'R-14','1',0,1,null,670,-5300 from dual union all
select 98,'Ariane H10+','3',0,1,47025,35787,200 from dual union all
select 98,'Ariane L220','1',0,1,621970,100,-5700 from dual union all
select 98,'Ariane L33','2',0,1,98910,200,-3700 from dual union all
select 98,'PAL','0',0,2,212380,5,-6300 from dual union all
select 980,'R-14','1',0,1,null,670,-5300 from dual union all
select 981,'8S641','1',0,1,null,100,-6200 from dual union all
select 981,'8S642','2',0,1,null,1000,-6000 from dual union all
select 982,'8S641','1',0,1,null,100,-6200 from dual union all
select 982,'8S642','2',0,1,null,1000,-5000 from dual union all
select 983,'R-17','1',0,1,null,200,-6250 from dual union all
select 984,'R-17','1',0,1,null,200,-6250 from dual union all
select 985,'R-1','1',0,1,null,100,-6280 from dual union all
select 986,'R-1','1',0,1,null,100,-6280 from dual union all
select 987,'R-1','1',0,1,null,100,-6280 from dual union all
select 988,'R-1','1',0,1,null,100,-6280 from dual union all
select 989,'R-1','1',0,1,null,100,-6280 from dual union all
select 99,'Ariane H10-3','3',0,1,48906,35787,200 from dual union all
select 99,'Ariane L220','1',0,1,621970,100,-5700 from dual union all
select 99,'Ariane L33','2',0,1,98910,200,-3700 from dual union all
select 99,'PAL','0',0,2,212380,5,-6300 from dual union all
select 990,'R-1','1',0,1,null,100,-6280 from dual union all
select 991,'R-1','1',0,1,null,100,-6280 from dual union all
select 992,'R-2','1',0,1,null,200,-6250 from dual union all
select 993,'R-21','1',0,1,null,300,-6200 from dual union all
select 994,'R-2','1',0,1,null,200,-6250 from dual union all
select 995,'R-2','1',0,1,null,200,-6250 from dual union all
select 996,'R-2','1',0,1,null,200,-6250 from dual union all
select 997,'R-31 BS','3',0,1,null,1000,-4000 from dual union all
select 997,'R-31 St 1','1',0,1,null,40,-6300 from dual union all
select 997,'R-31 St 2','2',0,1,null,100,-6100 from dual union all
select 998,'8S671','1',0,1,null,100,-6100 from dual union all
select 998,'8S672','2',0,1,null,1000,-4000 from dual union all
select 999,'8S671','1',0,1,null,100,-6100 from dual union all
select 999,'8S672','2',0,1,null,1000,-4000 from dual;


--------------------------------------------------------------------------------
-- STAGE_MANUFACTURER
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing STAGE_MANUFACTURER...');

CREATE TABLE "STAGE_MANUFACTURER" 
 (	"STAGE_NAME" VARCHAR2(21), 
	"MANUFACTURER_ORG_CODE" VARCHAR2(4000), 
	 CONSTRAINT "STAGE_MANUFACTURER_PK" PRIMARY KEY ("STAGE_NAME", "MANUFACTURER_ORG_CODE")
USING INDEX  ENABLE, 
	 CONSTRAINT "STAGE_MANUFACTURER_STAGE_FK" FOREIGN KEY ("STAGE_NAME")
	  REFERENCES "STAGE" ("STAGE_NAME") ENABLE, 
	 CONSTRAINT "STAGE_MANUFACTURER_MANUF_FK" FOREIGN KEY ("MANUFACTURER_ORG_CODE")
	  REFERENCES "ORGANIZATION" ("ORG_CODE") ENABLE
 ) ;


CREATE INDEX "STAGE_MANUFACTURER_IDX1" ON "STAGE_MANUFACTURER" ("MANUFACTURER_ORG_CODE") 
;
insert into STAGE_MANUFACTURER
select '10KS2500','THKE' from dual union all
select '11A57I','OKB1' from dual union all
select '11S510 (F)','OKB1' from dual union all
select '11S510','OKB1' from dual union all
select '11S59','OKB1' from dual union all
select '11S59A','OKB1' from dual union all
select '11S59A-S','OKB1' from dual union all
select '11S691','YUZH' from dual union all
select '11S692','OKB586' from dual union all
select '11S771','YUZH' from dual union all
select '11S772','YUZH' from dual union all
select '11S824','NPOE' from dual union all
select '11S824F','NPOE' from dual union all
select '11S824M','NPOE' from dual union all
select '11S86','NPOE' from dual union all
select '11S861','NPOE' from dual union all
select '11S861-01','NPOE' from dual union all
select '11S861-03','NPOE' from dual union all
select '12KRB','KHRU' from dual union all
select '14S53','OKB1' from dual union all
select '14S53A','OKB1' from dual union all
select '14S54','OKB1' from dual union all
select '14S54/RD-0110','OKB1' from dual union all
select '15A14A','YUZH' from dual union all
select '15A14B','YUZH' from dual union all
select '15A18-BR','YUZH' from dual union all
select '15A18A','YUZH' from dual union all
select '15A18B','YUZH' from dual union all
select '15Zh44 St 1','PAVKH' from dual union all
select '15Zh44 St 2','PAVKH' from dual union all
select '15Zh52 St 1','PAVKH' from dual union all
select '15Zh52 St 2','PAVKH' from dual union all
select '15Zh58A','MITT' from dual union all
select '15Zh58B','MITT' from dual union all
select '15Zh58MA','MITT' from dual union all
select '15Zh58MB','MITT' from dual union all
select '15Zh58MV','MITT' from dual union all
select '15Zh58V','MITT' from dual union all
select '15Zh59A','MITT' from dual union all
select '15Zh59B','MITT' from dual union all
select '15Zh59V','MITT' from dual union all
select '15Zh60 St 1','PAVKH' from dual union all
select '15Zh60 St 2','PAVKH' from dual union all
select '15Zh61 St 1','PAVKH' from dual union all
select '15Zh61 St 2','PAVKH' from dual union all
select '17S40','NPOE' from dual union all
select '2.5-DS-59000','ABL' from dual union all
select '22.6KS1245','HUMS' from dual union all
select '27D6','FAKEL' from dual union all
select '28D6','FAKEL' from dual union all
select '29D6','FAKEL' from dual union all
select '374BL01A','OKB1' from dual union all
select '374BL01A-A','OKB1' from dual union all
select '374BL01B','OKB1' from dual union all
select '374BL01B-A','OKB1' from dual union all
select '374BL02A','OKB1' from dual union all
select '374BL02B','OKB1' from dual union all
select '48N6','FAKEL' from dual union all
select '53T6','NOVTR' from dual union all
select '5Ya26','NOVTR' from dual union all
select '8A92E','OKB1' from dual union all
select '8A92ME','OKB1' from dual union all
select '8K71','OKB1' from dual union all
select '8K71A','OKB1' from dual union all
select '8K72E','OKB1' from dual union all
select '8K72EL','OKB1' from dual union all
select '8K72KE','OKB1' from dual union all
select '8K74','OKB1' from dual union all
select '8K74A','OKB1' from dual union all
select '8K75A','OKB1' from dual union all
select '8K75B','OKB1' from dual union all
select '8K78I','OKB1' from dual union all
select '8K94','YUZH' from dual union all
select '8K95A','OKB1' from dual union all
select '8K95B','OKB1' from dual union all
select '8K95V','OKB1' from dual union all
select '8K96A (RT-2P)','ARSL' from dual union all
select '8K96A','ARSL' from dual union all
select '8K96B (RT-15)','ARSL' from dual union all
select '8K96B (RT-15)','PERM' from dual union all
select '8K96B (RT-2P)','ARSL' from dual union all
select '8K96B (RT-2P)','PERM' from dual union all
select '8K96B','ARSL' from dual union all
select '8K96B','PERM' from dual union all
select '8K98A','OKB1' from dual union all
select '8K98A','PERM' from dual union all
select '8K98B','OKB1' from dual union all
select '8K98B','PERM' from dual union all
select '8K98PA','ARSL' from dual union all
select '8K98V','ARSL' from dual union all
select '8K98V','PERM' from dual union all
select '8K99A','YUZH' from dual union all
select '8S641','OKB586' from dual union all
select '8S642','OKB586' from dual union all
select '8S671','YUZH' from dual union all
select '8S672','OKB586' from dual union all
select '8S691','YUZH' from dual union all
select '8S692','OKB586' from dual union all
select '8S810','OKB52' from dual union all
select '8S810M','OKB52' from dual;
insert into STAGE_MANUFACTURER
select '8S811','OKB52' from dual union all
select '8S812','OKB52' from dual union all
select '8S812M','OKB52' from dual union all
select '8S816','KHRU' from dual union all
select '8S817','KHRU' from dual union all
select 'A-1','OKB1' from dual union all
select 'A-350 Booster','SKB30' from dual union all
select 'A-350','SKB30' from dual union all
select 'A-350Zh Booster','SKB30' from dual union all
select 'A-350Zh','SKB30' from dual union all
select 'A-925 Booster','FAKEL' from dual union all
select 'A-925','FAKEL' from dual union all
select 'A1P Stage 2','AJSP' from dual union all
select 'A1P Stage 2','LMSC' from dual union all
select 'A1P','AJSP' from dual union all
select 'A1P','LMSC' from dual union all
select 'A2E Stage 2','HERC' from dual union all
select 'A2E Stage 2','LMSC' from dual union all
select 'A2E','AJSP' from dual union all
select 'A2E','LMSC' from dual union all
select 'A2P Stage 2','ABL' from dual union all
select 'A2P Stage 2','LMSC' from dual union all
select 'A2P','AJSP' from dual union all
select 'A2P','LMSC' from dual union all
select 'A3P','AJSP' from dual union all
select 'A3P','LMSC' from dual union all
select 'AAD','HAL' from dual union all
select 'AJ10','AJGC' from dual union all
select 'AJ10-101A','AJGC' from dual union all
select 'AJ10-104','AJGC' from dual union all
select 'AJ10-104D','AJGC' from dual union all
select 'AJ10-118','AJGC' from dual union all
select 'AJ10-118A','AJGC' from dual union all
select 'AJ10-118D','AJGC' from dual union all
select 'AJ10-118FJ','AJGC' from dual union all
select 'AJ10-118FJ','MDAC' from dual union all
select 'AJ10-11B','AJGC' from dual union all
select 'AJ10-40','AJGC' from dual union all
select 'AJ10-41','AJGC' from dual union all
select 'AJ10-42','AJGC' from dual union all
select 'AJ60-91','AJGC' from dual union all
select 'AJ60-92','AJGC' from dual union all
select 'AMROC IRR','AMROC' from dual union all
select 'AS-0','VSSC' from dual union all
select 'AS-1','VSSC' from dual union all
select 'AS-2','VSSC' from dual union all
select 'AS-3','VSSC' from dual union all
select 'AS-4','VSSC' from dual union all
select 'ASAS XL','THKE' from dual union all
select 'ASAS','THKE' from dual union all
select 'ASAT','BOUT' from dual union all
select 'ASR SRB-A','NISST' from dual union all
select 'ASROC','UNK' from dual union all
select 'ATACMS','AJCA' from dual union all
select 'ATSM-9','IAI' from dual union all
select 'AUS-51','IAI' from dual union all
select 'AVUM','AVIO' from dual union all
select 'AVUM-RD868P','AVIO' from dual union all
select 'Ababeel St 3','PKNDC' from dual union all
select 'Abid St1','IQ' from dual union all
select 'Abid St2','IQ' from dual union all
select 'Abid St3','IQ' from dual union all
select 'Aeolus Booster','WRE' from dual union all
select 'AeroHTV Stage 2','AEROPH' from dual union all
select 'AeroHTV','AEROPH' from dual union all
select 'Aerobee 100','AJ' from dual union all
select 'Aerobee 150','AJGC' from dual union all
select 'Aerobee 150A','AJGC' from dual union all
select 'Aerobee 75','AJSP' from dual union all
select 'Aerobee AJ10-24','AJ' from dual union all
select 'Aerobee AJ10-25','AJ' from dual union all
select 'Aerobee AJ10-27','AJ' from dual union all
select 'Aerobee AJ10-34','AJ' from dual union all
select 'Aerobee Booster','AJ' from dual union all
select 'Aerobee XASR-1','AJ' from dual union all
select 'Aerojet Jr','AJ' from dual union all
select 'Agate','SEREB' from dual union all
select 'Agena A','LMSC' from dual union all
select 'Agena B','LMSC' from dual union all
select 'Agena D','LMSC' from dual union all
select 'Agni 4 Stage 2','DRDO' from dual union all
select 'Agni II Stage 2','DRDO' from dual union all
select 'Agni III Stage 2','DRDO' from dual union all
select 'Agni III','DRDO' from dual union all
select 'Agni V Stage 2','DRDO' from dual union all
select 'Agni V Stage 3','DRDO' from dual union all
select 'Agni V','DRDO' from dual union all
select 'Agni','DRDO' from dual union all
select 'Aigle','ONERA' from dual union all
select 'Ajax Cluster 1810','AJ' from dual union all
select 'Ajax Cluster','AJ' from dual union all
select 'Ajax','BELL' from dual union all
select 'Al Abbas','MAK' from dual union all
select 'Al Fatah','FATAH' from dual union all
select 'Al Hussein','MAK' from dual union all
select 'Al Samoud 2','IQMIC' from dual union all
select 'Al Samoud','IQMIC' from dual union all
select 'AlKahir','EGAO' from dual union all
select 'Alcor IA','AJSP' from dual union all
select 'Alcor IB','AJSP' from dual;
insert into STAGE_MANUFACTURER
select 'Alcor','AJSP' from dual union all
select 'Alcyone','HERC' from dual union all
select 'Alfa','AERIT' from dual union all
select 'Algol 1','LTV' from dual union all
select 'Algol 1A','LTV' from dual union all
select 'Algol 1B','LTV' from dual union all
select 'Algol 1C','LTV' from dual union all
select 'Algol 1D','LTV' from dual union all
select 'Algol 2A','LTV' from dual union all
select 'Algol 2B','LTV' from dual union all
select 'Algol 2C','LTV' from dual union all
select 'Algol 3A','LTV' from dual union all
select 'Altair 2','ABL' from dual union all
select 'Altair','ABL' from dual union all
select 'Amethyste','LRBA' from dual union all
select 'Angara URM','KHRU' from dual union all
select 'Angara URM-1','KHRU' from dual union all
select 'Angara URM-2','KHRU' from dual union all
select 'Antares 2 Stage 1','OSCW' from dual union all
select 'Antares 2 Stage 1','YUZH' from dual union all
select 'Antares 2','ABL' from dual union all
select 'Antares 2B','ABL' from dual union all
select 'Antares 3','THKE' from dual union all
select 'Antares Stage 1','OSCW' from dual union all
select 'Antares Stage 1','YUZH' from dual union all
select 'Antares','ABL' from dual union all
select 'Apache','THKE' from dual union all
select 'Apollo LES','NAAD' from dual union all
select 'Arcas','ATLR' from dual union all
select 'Arcon booster','ATLR' from dual union all
select 'Arcon','ATLR' from dual union all
select 'Ares I-X','THKU' from dual union all
select 'Ariane 5 EPC H158','NORLM' from dual union all
select 'Ariane 5 EPC H175','NORLM' from dual union all
select 'Ariane 5 EPS P2000','DORN' from dual union all
select 'Ariane 5 EPS','DORN' from dual union all
select 'Ariane EAP P238','EADSL' from dual union all
select 'Ariane EAP P240','EADSL' from dual union all
select 'Ariane H10','AELM' from dual union all
select 'Ariane H10+','AELM' from dual union all
select 'Ariane H10-3','AELM' from dual union all
select 'Ariane H8','AELM' from dual union all
select 'Ariane L144','AELM' from dual union all
select 'Ariane L220','EADSL' from dual union all
select 'Ariane L33','DASA' from dual union all
select 'Ariane L33','ERNO' from dual union all
select 'Arrow 3 Stage 2','IAI' from dual union all
select 'Arrow 3','IAI' from dual union all
select 'Arrow II','GCR' from dual union all
select 'Asp II','COOP' from dual union all
select 'Asp III','COOP' from dual union all
select 'Asp IV','COOP' from dual union all
select 'Asp','COOP' from dual union all
select 'Astra Stage 1','ASTRV' from dual union all
select 'Astra Stage 2','ASTRV' from dual union all
select 'Astrid','LLNL' from dual union all
select 'Astris','ERNO' from dual union all
select 'Astrobee D','AJSP' from dual union all
select 'Astrobee F','AJSP' from dual union all
select 'Atea-1','RLABN' from dual union all
select 'Atlas A Booster','GDA' from dual union all
select 'Atlas A','GDA' from dual union all
select 'Atlas B Booster','GDA' from dual union all
select 'Atlas B','GDA' from dual union all
select 'Atlas Booster','GDA' from dual union all
select 'Atlas C Booster','GDA' from dual union all
select 'Atlas C','GDA' from dual union all
select 'Atlas CCB','LMA' from dual union all
select 'Atlas D Booster','GDA' from dual union all
select 'Atlas D','GDA' from dual union all
select 'Atlas E','GDA' from dual union all
select 'Atlas E/F Booster','GDA' from dual union all
select 'Atlas F','GDA' from dual union all
select 'Atlas G Booster','LMA' from dual union all
select 'Atlas G','GDA' from dual union all
select 'Atlas H','GDA' from dual union all
select 'Atlas I','GDA' from dual union all
select 'Atlas II Booster','LMA' from dual union all
select 'Atlas II','GDA' from dual union all
select 'Atlas III','LMA' from dual union all
select 'Atlas MA-5 Booster','GDA' from dual union all
select 'Atlas SLV-3','GDA' from dual union all
select 'Atlas SLV-3A','GDA' from dual union all
select 'Atlas SLV-3C','GDA' from dual union all
select 'Atlas SLV-3D','GDA' from dual union all
select 'Atlas V SRM','AJSP' from dual union all
select 'Aurore','SNECMA' from dual union all
select 'B-611','SANJ' from dual union all
select 'BE-3','HERC' from dual union all
select 'BE-3A4','HERC' from dual union all
select 'BE-3A5','HERC' from dual union all
select 'BE-3A9','HERC' from dual union all
select 'BER','ONERA' from dual union all
select 'BOZ','PROG' from dual union all
select 'BT-310','ISAS' from dual union all
select 'Baby Sergeant','GCR' from dual union all
select 'Baby Sergeant','JPL' from dual union all
select 'Badr-1','ANSAL' from dual union all
select 'Belier II','SUD' from dual union all
select 'Belier III','SUD' from dual;
insert into STAGE_MANUFACTURER
select 'Belier','SUD' from dual union all
select 'Belisama','CFTHB' from dual union all
select 'Berenice St2','ONERA' from dual union all
select 'Black Arrow St2','SROE' from dual union all
select 'Black Arrow','RAE' from dual union all
select 'Black Arrow','SROE' from dual union all
select 'Black Brant I','BAL' from dual union all
select 'Black Brant IIB','BAL' from dual union all
select 'Black Brant IIIA','BAL' from dual union all
select 'Black Brant IIIB','BAL' from dual union all
select 'Black Brant Mk1','BAL' from dual union all
select 'Black Brant Mk3','BAL' from dual union all
select 'Black Brant Mk4','BAL' from dual union all
select 'Black Brant VA','BAL' from dual union all
select 'Black Brant VB','BAL' from dual union all
select 'Black Brant VC','BAL' from dual union all
select 'Black Brant VI','BAL' from dual union all
select 'Black Brant VII','BAL' from dual union all
select 'Black Knight 301','SROE' from dual union all
select 'Black Knight','SROE' from dual union all
select 'Black Sparrow','IAI' from dual union all
select 'Blok DM-SL','NPOE' from dual union all
select 'Blok DM-SLB','NPOE' from dual union all
select 'Blok DM1','NPOE' from dual union all
select 'Blok DM3','NPOE' from dual union all
select 'Blok DM4','NPOE' from dual union all
select 'Blok SO-L','NPOL' from dual union all
select 'Blok-2BL','NPOL' from dual union all
select 'Blok-L','OKB1' from dual union all
select 'Blok-ML','NPOL' from dual union all
select 'Blok-NVL','NPOL' from dual union all
select 'Blok-VL','NPOL' from dual union all
select 'Blue Sparrow','IAI' from dual union all
select 'Blue Streak','DEHAVS' from dual union all
select 'Bomarc A','BOC' from dual union all
select 'Bomarc B','BOC' from dual union all
select 'Briz-K','KHRU' from dual union all
select 'Briz-KM','KHRU' from dual union all
select 'Briz-M','KHRU' from dual union all
select 'Bull Goose','FAIRA' from dual union all
select 'Bullpup','THKE' from dual union all
select 'Burya booster','NPOL' from dual union all
select 'Burya','NPOL' from dual union all
select 'C-3 Stage 1','LMSC' from dual union all
select 'C-3 Stage 2','LMSC' from dual union all
select 'CB-250','VSSC' from dual union all
select 'CPKM','ARMT' from dual union all
select 'CSXT','CSXT' from dual union all
select 'CUS12','VSSC' from dual union all
select 'CUS15','VSSC' from dual union all
select 'CZ Booster','CALT' from dual union all
select 'CZ-11-1','SLREC' from dual union all
select 'CZ-11-2','ARMT' from dual union all
select 'CZ-11-3','ARMT' from dual union all
select 'CZ-11-4','ARMT' from dual union all
select 'CZ-2 Stage 1','CALT' from dual union all
select 'CZ-2 Stage 2','CALT' from dual union all
select 'CZ-2C Stage 1','CALT' from dual union all
select 'CZ-2C Stage 2','CALT' from dual union all
select 'CZ-2Cj Stage 2','CALT' from dual union all
select 'CZ-2E Stage 1','CALT' from dual union all
select 'CZ-2E Stage 2','CALT' from dual union all
select 'CZ-3 Stage 1','SAST' from dual union all
select 'CZ-3 Stage 2','SAST' from dual union all
select 'CZ-3 Stage 3','CALT' from dual union all
select 'CZ-3A Stage 1','SAST' from dual union all
select 'CZ-3A Stage 2','SAST' from dual union all
select 'CZ-3A Stage 3','CALT' from dual union all
select 'CZ-3B Stage 2','SAST' from dual union all
select 'CZ-3BE Booster','CALT' from dual union all
select 'CZ-3BE Stage 1','SAST' from dual union all
select 'CZ-4 Stage 1','SAST' from dual union all
select 'CZ-4 Stage 2','SAST' from dual union all
select 'CZ-4 Stage 2j','SAST' from dual union all
select 'CZ-4 Stage 3','SAST' from dual union all
select 'CZ-4B Stage 2','SAST' from dual union all
select 'CZ-4B Stage 3','SAST' from dual union all
select 'CZ-4C Stage 3','SAST' from dual union all
select 'CZ-5 Booster','CALT' from dual union all
select 'CZ-5 Stage 1','CALT' from dual union all
select 'CZ-5 Stage 2','CALT' from dual union all
select 'CZ-7 Stage 1','CALT' from dual union all
select 'CZ-7 Stage 2','CALT' from dual union all
select 'CZ-K2-1','CALT' from dual union all
select 'CZ-K2-2','CALT' from dual union all
select 'CZ-K2-3','CALT' from dual union all
select 'CZ6-C','CALT' from dual union all
select 'Cajun','THKE' from dual union all
select 'Canopus','IIAE' from dual union all
select 'Castor 1','THKX' from dual union all
select 'Castor 120','THKU' from dual union all
select 'Castor 2','THKX' from dual union all
select 'Castor 2A','THKX' from dual union all
select 'Castor 30','ATK' from dual union all
select 'Castor 30B','ATK' from dual union all
select 'Castor 30XL','ATK' from dual union all
select 'Castor 4','THKX' from dual union all
select 'Castor 4A','THKX' from dual union all
select 'Castor 4AXL','THKX' from dual union all
select 'Castor 4B','THKX' from dual;
insert into STAGE_MANUFACTURER
select 'Castor','THKX' from dual union all
select 'Cedre St1','LRS' from dual union all
select 'Cedre St2','LRS' from dual union all
select 'Cedre St3','LRS' from dual union all
select 'Cedre-4 St1','LRS' from dual union all
select 'Centaur D','GDA' from dual union all
select 'Centaur D-1A','GDA' from dual union all
select 'Centaur D-1T','GDA' from dual union all
select 'Centaur I','GDA' from dual union all
select 'Centaur II','GDA' from dual union all
select 'Centaur IIA','GDA' from dual union all
select 'Centaur IIIA','LMA' from dual union all
select 'Centaur IIIB/C-1','LMA' from dual union all
select 'Centaur IIIB/DEC','LMA' from dual union all
select 'Centaur IIIB/SEC','LMA' from dual union all
select 'Centaur T','GDA' from dual union all
select 'Centaur','GDA' from dual union all
select 'Centenario','IIAE' from dual union all
select 'Cetus','LARC' from dual union all
select 'Cetus','THKE' from dual union all
select 'Chick x3','RORD' from dual union all
select 'Chick x4','RORD' from dual union all
select 'Chick','RORD' from dual union all
select 'Cirrus I','DRG' from dual union all
select 'Cirrus II','DRG' from dual union all
select 'Cluster Stage 2','JPL' from dual union all
select 'Cluster Stage 3','JPL' from dual union all
select 'Coralie','SEREB' from dual union all
select 'Corporal E','DAC' from dual union all
select 'Corporal E','JPL' from dual union all
select 'Corporal Type 1','FIRES' from dual union all
select 'Cuckoo 4','RORD' from dual union all
select 'Cuckoo IA','RORD' from dual union all
select 'Cuckoo IB','RORD' from dual union all
select 'Cuckoo II','RORD' from dual union all
select 'Cygnus 10','PARD' from dual union all
select 'Cygnus 15','LARC' from dual union all
select 'Cygnus 15','THKE' from dual union all
select 'Cygnus 20','PARD' from dual union all
select 'Cygnus 20','THKE' from dual union all
select 'Cygnus 25','PARD' from dual union all
select 'Cygnus 40','LARC' from dual union all
select 'Cygnus 40','THKE' from dual union all
select 'Cygnus 5','LARC' from dual union all
select 'DAC Roc','DAC' from dual union all
select 'DC-X','MDAC' from dual union all
select 'DC-XA','MDAC' from dual union all
select 'DF-1','CALT' from dual union all
select 'DF-11','SANJ' from dual union all
select 'DF-11M','SANJ' from dual union all
select 'DF-15','ARMT' from dual union all
select 'DF-16','CASIC' from dual union all
select 'DF-2','CALT' from dual union all
select 'DF-21 St2','ARMT' from dual union all
select 'DF-21','ARMT' from dual union all
select 'DF-2A','CALT' from dual union all
select 'DF-31 St1','ARMT' from dual union all
select 'DF-31 St2','ARMT' from dual union all
select 'DF-31 St3','ARMT' from dual union all
select 'DF-41 St1','ARMT' from dual union all
select 'DF-41 St2','ARMT' from dual union all
select 'DF-41 St3','ARMT' from dual union all
select 'DF-5 Stage 1','CALT' from dual union all
select 'DF-5 Stage 2','CALT' from dual union all
select 'DF3-YF2A','CALT' from dual union all
select 'DF4-YF3','CALT' from dual union all
select 'DIM','IIAE' from dual union all
select 'DS','MITT' from dual union all
select 'DSV-3E-3','AJGC' from dual union all
select 'DSV-3N-4','AJGC' from dual union all
select 'DSV-3P-4','MDAC' from dual union all
select 'DSV-3P-4B','MDAC' from dual union all
select 'Deacon','ABL' from dual union all
select 'Delta CBC RS-68A','BOHB' from dual union all
select 'Delta CBC','BOHB' from dual union all
select 'Delta D4SS-4','BOHB' from dual union all
select 'Delta D4SS-5','BOHB' from dual union all
select 'Delta III Stage 1','BOHB' from dual union all
select 'Delta III Stage 2','BOHB' from dual union all
select 'Delta/0020','MDAC' from dual union all
select 'Demon','RORD' from dual union all
select 'Dolphin','STST' from dual union all
select 'Dorado','WRE' from dual union all
select 'Dragon V2','SPX' from dual union all
select 'E-8-5 VA','NPOL' from dual union all
select 'EELT Thor','DACHB' from dual union all
select 'ELT Thor DSV-2L','DACHB' from dual union all
select 'ELT Thor DSV-2L-1D','DACHB' from dual union all
select 'ELT Thor DSV-2L-1E','DACHB' from dual union all
select 'ELT Thor DSV-2W','DACHB' from dual union all
select 'ELT Thor DSV-2W-1','DACHB' from dual union all
select 'ELT Thor N','DACHB' from dual union all
select 'EPKM','CALT' from dual union all
select 'ESC-A','ASTD' from dual union all
select 'ET','MMMI' from dual union all
select 'Elan','CFTHB' from dual union all
select 'Electron Stage 1','RLABN' from dual union all
select 'Electron Stage 2','RLABN' from dual union all
select 'Electron Stage 3','RLABN' from dual union all
select 'Emad','IR' from dual;
insert into STAGE_MANUFACTURER
select 'Emeraude','LRBA' from dual union all
select 'Emilie','CFTHB' from dual union all
select 'Emma Booster','MATRA' from dual union all
select 'Emma','MATRA' from dual union all
select 'Energiya Blok A','YUZH' from dual union all
select 'Energiya','RKKE' from dual union all
select 'Epsilon PBS','JAXA' from dual union all
select 'Excalibur','BAL' from dual union all
select 'FB-1 Stage 1','SAST' from dual union all
select 'FB-1 Stage 2','SAST' from dual union all
select 'FFAR','PARD' from dual union all
select 'FFK','ERNO' from dual union all
select 'FTB','AVIB' from dual union all
select 'FTI','AVIB' from dual union all
select 'FTV','VSSC' from dual union all
select 'FW4D','UTC' from dual union all
select 'FW4S','UTC' from dual union all
select 'Falcon 1 St 2','SPX' from dual union all
select 'Falcon 1','SPX' from dual union all
select 'Falcon 9 St 1','SPX' from dual union all
select 'Falcon 9 St 1/1D','SPX' from dual union all
select 'Falcon 9 St 1/1D-4','SPX' from dual union all
select 'Falcon 9 St 1/1D-5','SPX' from dual union all
select 'Falcon 9 St 1/1D-FT','SPX' from dual union all
select 'Falcon 9 St 2','SPX' from dual union all
select 'Falcon 9 St 2/1D','SPX' from dual union all
select 'Falcon 9 St 2/1D-4','SPX' from dual union all
select 'Falcon 9 St 2/1D-5','SPX' from dual union all
select 'Falcon 9-1D','SPX' from dual union all
select 'Falcon','THKX' from dual union all
select 'FalconLaunch','USAFA' from dual union all
select 'Fateh-110','IR' from dual union all
select 'Frangible Arcas','ATLR' from dual union all
select 'Fregat','NPOL' from dual union all
select 'Fregat-SB','NPOL' from dual union all
select 'GCA/Orion 38','ATKH' from dual union all
select 'GCR 33KS2800','GCR' from dual union all
select 'GEM-40','ATKH' from dual union all
select 'GEM-46','ATKH' from dual union all
select 'GEM-60','ATKH' from dual union all
select 'GF-01A','ARMT' from dual union all
select 'GF-02','ARMT' from dual union all
select 'GLCM','UNK' from dual union all
select 'GS1-L110','LPSCM' from dual union all
select 'GS1-S125','VSSC' from dual union all
select 'GS1-S139','VSSC' from dual union all
select 'GS2','VSSC' from dual union all
select 'GS2-C25','LPSCM' from dual union all
select 'GSOM','VSSC' from dual union all
select 'GSOM-S200','SDSC' from dual union all
select 'GammaCen Booster','IIAE' from dual union all
select 'GammaCen Sustainer','IAE' from dual union all
select 'Genie','THKU' from dual union all
select 'Genie-Alarr','THKU' from dual union all
select 'Ghauri','KRLP' from dual union all
select 'Ghaznavi','PKPMO' from dual union all
select 'Goldfinch II','BAJ' from dual union all
select 'Goldfinch','BAJ' from dual union all
select 'Goose','RORD' from dual union all
select 'Gosling 4T','RORD' from dual union all
select 'Gosling IIN','RORD' from dual union all
select 'Gosling IV','RORD' from dual union all
select 'Gosling','RORD' from dual union all
select 'Grad','ANSAL' from dual union all
select 'Gradicom 1','CITED' from dual union all
select 'Gradicom 2','CITED' from dual union all
select 'Gun Projectile 16','BRL' from dual union all
select 'Gun Projectile 5','BRL' from dual union all
select 'Gun Projectile 7','BRL' from dual union all
select 'H-II SRB','NISST' from dual union all
select 'H-II SRB-A','NISST' from dual union all
select 'H-II SRB-A3','NISST' from dual union all
select 'H-II SSB','NISST' from dual union all
select 'H-II Stage 1','MHITO' from dual union all
select 'H-II Stage 2','MHITO' from dual union all
select 'H-IIA Stage 1','MHITO' from dual union all
select 'H-IIA Stage 2 U','MHITO' from dual union all
select 'H-IIA Stage 2','MHITO' from dual union all
select 'H-IIB Stage 1','MHITO' from dual union all
select 'HAEC FFAR','UNK' from dual union all
select 'HALO','HAL5' from dual union all
select 'HAPS Stage','PRIMX' from dual union all
select 'HAPS','PRIMX' from dual union all
select 'HAT','WRE' from dual union all
select 'HIBEX','BOKT' from dual union all
select 'HIBEX-U','BOKT' from dual union all
select 'HOTROC','NOTS' from dual union all
select 'HPAG','NOTS' from dual union all
select 'HPS3','VSSC' from dual union all
select 'HPS4','VSSC' from dual union all
select 'HTR','NAMMO' from dual union all
select 'HV Arcas','ATLR' from dual union all
select 'HVAR','CIT' from dual union all
select 'HVAR','NOTS' from dual union all
select 'HYSR','LMMI' from dual union all
select 'Hades','AELM' from dual union all
select 'Hatf I','SUPA' from dual union all
select 'Hawk','AJSP' from dual union all
select 'Hercules Booster','DAC' from dual union all
select 'Hercules Booster','HERCR' from dual;
insert into STAGE_MANUFACTURER
select 'Hermes A-1','GEMTS' from dual union all
select 'Hermes A-3A','GEMTS' from dual union all
select 'Hermes A-3B','GEMTS' from dual union all
select 'Heron','RORD' from dual union all
select 'Hopi III','RPI' from dual union all
select 'Hopi IV','RPI' from dual union all
select 'Hopi','RPI' from dual union all
select 'Hound Dog','NAA' from dual union all
select 'Hwasong 10','KPA' from dual union all
select 'Hwasong 5','KPA' from dual union all
select 'Hwasong 6','KPA' from dual union all
select 'Hwasong 7','KPA' from dual union all
select 'Hwasong ER','KPA' from dual union all
select 'Hwasong-12 Stage 1','KPA' from dual union all
select 'Hwasong-14 Stage 2','KPA' from dual union all
select 'Hwasong-15 Stage 1','KPA' from dual union all
select 'Hwasong-15 Stage 2','KPA' from dual union all
select 'Hwasong-7','KPA' from dual union all
select 'Hydac','SDCO' from dual union all
select 'Hyperion','EAC' from dual union all
select 'Hyunmoo Booster','DAC' from dual union all
select 'Hyunmoo Booster','HERCR' from dual union all
select 'Hyunmoo-2A','ADD' from dual union all
select 'Hyunmoo-2B','ADD' from dual union all
select 'Hyunmoo-2C','ADD' from dual union all
select 'IAI Arrow St2','IAI' from dual union all
select 'IAI Arrow','IAI' from dual union all
select 'IAI Arrow-1','IAI' from dual union all
select 'INTA S-12','INTA' from dual union all
select 'INTA-100','INTA' from dual union all
select 'IRIS Stage','FIATA' from dual union all
select 'IUS SRM-1','UTC' from dual union all
select 'IUS SRM-2','UTC' from dual union all
select 'Icarus Stage 2','ERAU' from dual union all
select 'Icarus','ERAU' from dual union all
select 'Ikar','PROG' from dual union all
select 'Imp Malemute','THKU' from dual union all
select 'Improved Orion','AJSP' from dual union all
select 'Iraqi Gun Projectile','SRCI' from dual union all
select 'Iris','ATLR' from dual union all
select 'Iroquois','THKE' from dual union all
select 'Iskander','KBM' from dual union all
select 'Iskander-E','KBM' from dual union all
select 'Iskander-M','KBM' from dual union all
select 'Ivanov Booster','FIAN' from dual union all
select 'Ivanov','FIAN' from dual union all
select 'JCR Booster','NISST' from dual union all
select 'JCR','NISST' from dual union all
select 'Javelin 3','SDCO' from dual union all
select 'Jericho II','SUD' from dual union all
select 'Jericho IIIA1','IAI' from dual union all
select 'Jericho IIIA2','IAI' from dual union all
select 'Jericho-1','IAI' from dual union all
select 'Judi I','RPI' from dual union all
select 'Judi III','RPI' from dual union all
select 'Jupiter','ABMA' from dual union all
select 'Jupiter','CHRY' from dual union all
select 'K-15 Booster','DRDO' from dual union all
select 'K-15 Stage 2','DRDO' from dual union all
select 'K-4 Booster','DRDO' from dual union all
select 'K-4 Stage 2','DRDO' from dual union all
select 'K-BATS','HANWHA' from dual union all
select 'K10S','ISAS' from dual union all
select 'K150','ISAS' from dual union all
select 'K245','ISAS' from dual union all
select 'K250','ISAS' from dual union all
select 'K420','ISAS' from dual union all
select 'K420(1/3)','ISAS' from dual union all
select 'K420H','ISAS' from dual union all
select 'KM Rockoon','NISST' from dual union all
select 'KM-D','NISST' from dual union all
select 'KM-H','NISST' from dual union all
select 'KM-L','NISST' from dual union all
select 'KM-M','NISST' from dual union all
select 'KM-P','NISST' from dual union all
select 'KM-V1','NISST' from dual union all
select 'KM-V2b','NISST' from dual union all
select 'KM-V2c','NISST' from dual union all
select 'KN-02','KBM' from dual union all
select 'KN-11','KPA' from dual union all
select 'KN-11-2','KPA' from dual union all
select 'KSLV-1 Stage 2','KARI' from dual union all
select 'KSR Booster','KARI' from dual union all
select 'KSR','KARI' from dual union all
select 'KSR-III','KARI' from dual union all
select 'KT-1 St3','HEXI' from dual union all
select 'KZ-1 St3','HEXI' from dual union all
select 'KZ-1 St4','HEXI' from dual union all
select 'KZ-1A St1','ARMT' from dual union all
select 'KZ-2 St3','HEXI' from dual union all
select 'Kappa 1','ISAS' from dual union all
select 'Kappa 3 Booster','ISAS' from dual union all
select 'Kappa 4 Booster','ISAS' from dual union all
select 'Kappa 4','ISAS' from dual union all
select 'Kappa 6','ISAS' from dual union all
select 'Kappa 6H','ISAS' from dual union all
select 'Kappa 8','ISAS' from dual union all
select 'Kappa 8L','ISAS' from dual union all
select 'Kavoshgar','IRSA' from dual union all
select 'Khorramshahr','IR' from dual;
insert into STAGE_MANUFACTURER
select 'Kisha','RPI' from dual union all
select 'Kiva','RPI' from dual union all
select 'L-3 Blok G','OKB1' from dual union all
select 'L140-YF21','CALT' from dual union all
select 'L180-YF21B','CALT' from dual union all
select 'L35-YF24','CALT' from dual union all
select 'L35-YF24B','CALT' from dual union all
select 'L480S','ISAS' from dual union all
select 'L500','ISAS' from dual union all
select 'L735','ISAS' from dual union all
select 'L735(1/3)','ISAS' from dual union all
select 'LAPSTAR','WRE' from dual union all
select 'LE-3','MHI' from dual union all
select 'LE-5','MHITO' from dual union all
select 'LEO-1','AJSP' from dual union all
select 'LEO-46','AJSP' from dual union all
select 'LEO-7','AJSP' from dual union all
select 'LM AS','GRUM' from dual union all
select 'LPR-006','VSSC' from dual union all
select 'LPR-3T','VSSC' from dual union all
select 'LR87','MARTD' from dual union all
select 'LR87AJ11','MARTD' from dual union all
select 'LR87AJ11A','MARTD' from dual union all
select 'LR87AJ5','MARTB' from dual union all
select 'LR87AJ9','MARTD' from dual union all
select 'LR91','MARTD' from dual union all
select 'LR91AJ11','MARTD' from dual union all
select 'LR91AJ11A','MARTD' from dual union all
select 'LR91AJ5','MARTB' from dual union all
select 'LR91AJ9','MARTD' from dual union all
select 'LS-A Booster','MHITO' from dual union all
select 'LS-A','MHITO' from dual union all
select 'LS-C Booster','MHITO' from dual union all
select 'LS-C','MHITO' from dual union all
select 'LS-C-7','MHITO' from dual union all
select 'LT Thor DSV-2L-1A','DACHB' from dual union all
select 'LT Thor DSV-2L-1B','DACHB' from dual union all
select 'LT Thor DSV-2L-1C','DACHB' from dual union all
select 'LT Thor N','DACHB' from dual union all
select 'LaRC Plastic','LARC' from dual union all
select 'Lacrosse','MARTO' from dual union all
select 'Lambda2-2','ISAS' from dual union all
select 'Lambda3-2','ISAS' from dual union all
select 'Lambda3-3','ISAS' from dual union all
select 'Lance Missile','LTVM' from dual union all
select 'Lance XM-45','THKX' from dual union all
select 'Lance','GCR' from dual union all
select 'Lark','FAIRA' from dual union all
select 'Lex','SNECMA' from dual union all
select 'Lobster I','RORD' from dual union all
select 'Loki II','GCR' from dual union all
select 'Loki SR71-AD-1','SDCO' from dual union all
select 'Loki','GCR' from dual union all
select 'Loki-Dart','GCR' from dual union all
select 'Long Tom','WRE' from dual union all
select 'Luna-M','MIT' from dual union all
select 'Lupus 2','WRE' from dual union all
select 'Lupus 3','WRE' from dual union all
select 'Lupus','WRE' from dual union all
select 'Luzou','SEP' from dual union all
select 'Lynx','NAWC' from dual union all
select 'M-10','NISST' from dual union all
select 'M-100 St2','TAIFUN' from dual union all
select 'M-100','TAIFUN' from dual union all
select 'M-13','NISST' from dual union all
select 'M-13TVC','NISST' from dual union all
select 'M-14','NISST' from dual union all
select 'M-20','NISST' from dual union all
select 'M-22TVC','NISST' from dual union all
select 'M-23','NISST' from dual union all
select 'M-24','NISST' from dual union all
select 'M-25','NISST' from dual union all
select 'M-250','TAIFUN' from dual union all
select 'M-30','NISST' from dual union all
select 'M-34','NISST' from dual union all
select 'M-34c','NISST' from dual union all
select 'M-35','NISST' from dual union all
select 'M-3A','NISST' from dual union all
select 'M-3B','NISST' from dual union all
select 'M-40','NISST' from dual union all
select 'M-6','ABL' from dual union all
select 'M55A1','THKU' from dual union all
select 'M55E1','THKU' from dual union all
select 'M55E1-ATK','THKU' from dual union all
select 'M56A1','SVC' from dual union all
select 'M57A1','HERC' from dual union all
select 'MARC 14A1','ATLR' from dual union all
select 'MARC 14B1','USN' from dual union all
select 'MARC 42A1','ATLR' from dual union all
select 'MARC 42A2','ATLR' from dual union all
select 'MCR-3','VSSC' from dual union all
select 'MD-620','DASST' from dual union all
select 'MERA','KBPT' from dual union all
select 'MG-18','LPC' from dual union all
select 'MLRS','LMCAM' from dual union all
select 'MMR-06','GMS' from dual union all
select 'MR-1','NMC' from dual union all
select 'MR-12','TAIFUN' from dual union all
select 'MR-20','TAIFUN' from dual union all
select 'MR-30','TAIFUN' from dual;
insert into STAGE_MANUFACTURER
select 'MR-UR-100 BR','YUZH' from dual union all
select 'MR-UR-100 St 1','YUZH' from dual union all
select 'MR-UR-100 St 2','YUZH' from dual union all
select 'MSBS 401','AELM' from dual union all
select 'MSBS 402','AELM' from dual union all
select 'MSBS 403','AELM' from dual union all
select 'MSBS 501','AELM' from dual union all
select 'MSBS 502','AELM' from dual union all
select 'MSBS 503','AELM' from dual union all
select 'MSD','NRL' from dual union all
select 'MT-135P','NISST' from dual union all
select 'MX-774','CONV' from dual union all
select 'MX-775','ABL' from dual union all
select 'Mace','MARTB' from dual union all
select 'Malemute','THKU' from dual union all
select 'Martlet 1','SRCI' from dual union all
select 'Martlet 2','SRCI' from dual union all
select 'Martlet 3A','SRCI' from dual union all
select 'Martlet 3B','SRCI' from dual union all
select 'Matador','MARTB' from dual union all
select 'Mayfly','WRE' from dual union all
select 'Melanie 1','ATEF' from dual union all
select 'Melanie','ONERA' from dual union all
select 'Melusine','CFTHB' from dual union all
select 'Mesquito','LMCAM' from dual union all
select 'Meteo','SUD' from dual union all
select 'Meteor 1','PIHM' from dual union all
select 'Meteor 3 Booster','PIHM' from dual union all
select 'Meteor 3B Booster','PIHM' from dual union all
select 'Meteor SRB','PIHM' from dual union all
select 'Meteor-2','PAI' from dual union all
select 'Metroc','ATLR' from dual union all
select 'Mimosa','ONERA' from dual union all
select 'Mk 104 (ARC)','ATLCA' from dual union all
select 'Mk 104','THKE' from dual union all
select 'Mk 30','HUMS' from dual union all
select 'Mk 56','ATLR' from dual union all
select 'Mk 72','UTC' from dual union all
select 'Mk7','PARD' from dual union all
select 'Momo','IST' from dual union all
select 'Musca','WRE' from dual union all
select 'N-1 Blok-A','OKB1' from dual union all
select 'N-1 Blok-B','OKB1' from dual union all
select 'N-1 Blok-V','OKB1' from dual union all
select 'NA-805','DASST' from dual union all
select 'NAL-16','MHITO' from dual union all
select 'NAL-25','MHITO' from dual union all
select 'NAL-7','MHITO' from dual union all
select 'NAL-735','NASDA' from dual union all
select 'NOTS 124-C','NOTS' from dual union all
select 'NOTS 3in Sph.','NOTS' from dual union all
select 'NOTS 401A','NOTS' from dual union all
select 'NOTS 500','NOTS' from dual union all
select 'NOTS 551','NOTS' from dual union all
select 'NOTS 8in','NOTS' from dual union all
select 'NSPO Booster','NSPO' from dual union all
select 'NSPO Stage 2','NSPO' from dual union all
select 'Naro Stage 1','KHRU' from dual union all
select 'Navaho Booster','NAAD' from dual union all
select 'Navaho Missile','NAAD' from dual union all
select 'Nazeat','IR' from dual union all
select 'Neptune','ONERA' from dual union all
select 'New Shepard PM','BLOR' from dual union all
select 'Nihka','BAL' from dual union all
select 'Nike','ABL' from dual union all
select 'Nike-46 Sustainer','AJ' from dual union all
select 'Nike-46','AJ' from dual union all
select 'Nike-47 Sustainer','AJ' from dual union all
select 'Nike-47','AJ' from dual union all
select 'Nike-484 Sustainer','AJ' from dual union all
select 'Nike-490 Sustainer','AJ' from dual union all
select 'Nudol'' Booster','ALMAN' from dual union all
select 'Nudol''','ALMAN' from dual union all
select 'OAM','PRIMX' from dual union all
select 'OIS','FAIR' from dual union all
select 'OS-X0','SANJ' from dual union all
select 'OTRAG','OTR' from dual union all
select 'OV','RWI' from dual union all
select 'Oka','KBM' from dual union all
select 'Orbus 1','UTC' from dual union all
select 'Orbus 1A','ATKE' from dual union all
select 'Orbus 21D','UTC' from dual union all
select 'Oreste','ATEF' from dual union all
select 'Oriole','DTI' from dual union all
select 'Orion 38','ATKH' from dual union all
select 'Orion 50','OSCC' from dual union all
select 'Orion 50S','OSCC' from dual union all
select 'Orion 50SG','OSCC' from dual union all
select 'Orion 50SXLG','OSCC' from dual union all
select 'Orion 50SXLT','OSCC' from dual union all
select 'Orion 50XL','OSCC' from dual union all
select 'Orion 50XLT','OSCC' from dual union all
select 'Orion LAS','OSCC' from dual union all
select 'Orion','AJSP' from dual union all
select 'Orion-1','IIAE' from dual union all
select 'Orion-2','IIAE' from dual union all
select 'P064','SUD' from dual union all
select 'P068','SEP' from dual union all
select 'PAL','DASA' from dual union all
select 'PAM-D2','THKU' from dual;
insert into STAGE_MANUFACTURER
select 'PAP','BPD' from dual union all
select 'PBS','BELT' from dual union all
select 'PBS','RDYNE' from dual union all
select 'PDV Target St2','HAL' from dual union all
select 'PS2','VSSC' from dual union all
select 'PS3','VSSC' from dual union all
select 'PS4','VSSC' from dual union all
select 'PSOM','VSSC' from dual union all
select 'PSOM-XL','VSSC' from dual union all
select 'Paekkom Booster','ADD' from dual union all
select 'Paiute','THKU' from dual union all
select 'Patriot','THKU' from dual union all
select 'Peacekeeper PBPS','RDYNE' from dual union all
select 'Pedro','THKX' from dual union all
select 'Pegasus XL Stage 1','OSCC' from dual union all
select 'Peregrine','MSFC' from dual union all
select 'Perimetr Stage 1','YUZH' from dual union all
select 'Perimetr Stage 2','YUZH' from dual union all
select 'Pershing 2 St1','MARTO' from dual union all
select 'Pershing 2 St2','MARTO' from dual union all
select 'Petrel 2','BAJUK' from dual union all
select 'Petrel','BAJUK' from dual union all
select 'Polaris TV','LMSC' from dual union all
select 'Pollux','THKX' from dual union all
select 'Prithvi','HAL' from dual union all
select 'Private A Booster','JPL' from dual union all
select 'Private A','JPL' from dual union all
select 'Private F','JPL' from dual union all
select 'Proson St1','CITEF' from dual union all
select 'Proson St2','CITEF' from dual union all
select 'Prosper','ATEF' from dual union all
select 'R-1','NII88' from dual union all
select 'R-11','OKB1' from dual union all
select 'R-11FM','OKB1' from dual union all
select 'R-12 11K63A','OKB586' from dual union all
select 'R-12 63S1A','OKB586' from dual union all
select 'R-12 63S1MA','OKB586' from dual union all
select 'R-12','OKB586' from dual union all
select 'R-13','SKB385' from dual union all
select 'R-14 11K65A','YUZH' from dual union all
select 'R-14 11K65MA','POL' from dual union all
select 'R-14 65S3A','YUZH' from dual union all
select 'R-14','YUZH' from dual union all
select 'R-17','MAK' from dual union all
select 'R-2','OKB1' from dual union all
select 'R-21','SKB385' from dual union all
select 'R-27','MAK' from dual union all
select 'R-29 St 1','MAK' from dual union all
select 'R-29 St 2','MAK' from dual union all
select 'R-29R St 1','MAK' from dual union all
select 'R-29R St 2','MAK' from dual union all
select 'R-29RM St1','MAK' from dual union all
select 'R-29RM St2','MAK' from dual union all
select 'R-29RM St3','MAK' from dual union all
select 'R-31 BS','ARSL' from dual union all
select 'R-31 St 1','ARSL' from dual union all
select 'R-31 St 2','ARSL' from dual union all
select 'R-39 St 1','PAVKH' from dual union all
select 'R-39 St 1','YUZH' from dual union all
select 'R-39 St 2','MAK' from dual union all
select 'R-39 St 3','MAK' from dual union all
select 'R-5','OKB1' from dual union all
select 'R-UNK','UNK' from dual union all
select 'RH-125','VSSC' from dual union all
select 'RH-125S','VSSC' from dual union all
select 'RH-200','VSSC' from dual union all
select 'RH-200SV','VSSC' from dual union all
select 'RH-300 Booster','ISRO' from dual union all
select 'RH-300 Mk II','VSSC' from dual union all
select 'RH-300','VSSC' from dual union all
select 'RH-560','VSSC' from dual union all
select 'RH-560S','VSSC' from dual union all
select 'RH-75','VSSC' from dual union all
select 'RH-ATV','VSSC' from dual union all
select 'RSA-3-1','IAI' from dual union all
select 'RSA-3-2','IAI' from dual union all
select 'RSA-3-3','HOUW' from dual union all
select 'RSRM','THKU' from dual union all
select 'RT-21 St1','MITT' from dual union all
select 'RT-21 St2','MITT' from dual union all
select 'RT-21 St3','MITT' from dual union all
select 'RT-23 St 3','ISKRP' from dual union all
select 'RTV Motor','JPL' from dual union all
select 'RTV-A-3','NAA' from dual union all
select 'RV-A-10','GEMTS' from dual union all
select 'RX-250-LPN','LAPAN' from dual union all
select 'RX-320','LAPAN' from dual union all
select 'RX-420','LAPAN' from dual union all
select 'Raven 1/1A/2A','RORD' from dual union all
select 'Raven 11','RORD' from dual union all
select 'Raven 2','RORD' from dual union all
select 'Raven 5','RORD' from dual union all
select 'Raven 6','RORD' from dual union all
select 'Raven 7','RORD' from dual union all
select 'Raven 8','RORD' from dual union all
select 'Recruit 1A','THKE' from dual union all
select 'Recruit','THKE' from dual union all
select 'Redstone MRLV','CHRY' from dual union all
select 'Redstone','CHRY' from dual union all
select 'Regulus II','VOUA' from dual;
insert into STAGE_MANUFACTURER
select 'Regulus','VOUA' from dual union all
select 'Rita I','SEP' from dual union all
select 'Rita I','SUD' from dual union all
select 'Rita II','SEP' from dual union all
select 'Rita II','SUD' from dual union all
select 'Rocketsonde','UNK' from dual union all
select 'Rocketsonde2','UNK' from dual union all
select 'Rook II','RORD' from dual union all
select 'Rook IIIA','RORD' from dual union all
select 'Rook IIIB','RORD' from dual union all
select 'Rook','RORD' from dual union all
select 'S-160','ISAS' from dual union all
select 'S-210','ISAS' from dual union all
select 'S-250','ISAS' from dual union all
select 'S-300','ISAS' from dual union all
select 'S-310','ISAS' from dual union all
select 'S-40','IAE' from dual union all
select 'S-40TM','IAE' from dual union all
select 'S-43 VLR-R2','IAE' from dual union all
select 'S-43','IAE' from dual union all
select 'S-43TM','IAE' from dual union all
select 'S-44','IAE' from dual union all
select 'S-44M','IAE' from dual union all
select 'S-520','ISAS' from dual union all
select 'S-A','MHITO' from dual union all
select 'S-B','MHITO' from dual union all
select 'S-C','MHITO' from dual union all
select 'S1','YUZH' from dual union all
select 'S125','VSSC' from dual union all
select 'S139','VSSC' from dual union all
select 'S1M','YUZH' from dual union all
select 'S3','YUZH' from dual union all
select 'S31','IAE' from dual union all
select 'S3M','POL' from dual union all
select 'S5M','YUZH' from dual union all
select 'S9','VSSC' from dual union all
select 'SA-II','MHITO' from dual union all
select 'SARGE','EXOSA' from dual union all
select 'SB-310','ISAS' from dual union all
select 'SB-735','ISAS' from dual union all
select 'SDC Viper','SDCO' from dual union all
select 'SEGPRO','VSSC' from dual union all
select 'SEP 901','SEP' from dual union all
select 'SEP 901','SUD' from dual union all
select 'SEP 902','SEP' from dual union all
select 'SEP 902','SUD' from dual union all
select 'SEP 903','SEP' from dual union all
select 'SEP 904','SEP' from dual union all
select 'SEP 904','SUD' from dual union all
select 'SEPR P167','SEP' from dual union all
select 'SEPR-732','SUD' from dual union all
select 'SICBM PBS','MMD' from dual union all
select 'SICBM St 2','AJSP' from dual union all
select 'SICBM St 3','HERC' from dual union all
select 'SICBM','THKU' from dual union all
select 'SLV-3 Booster','GDA' from dual union all
select 'SLV-3-1','VSSC' from dual union all
select 'SLV-3-2','VSSC' from dual union all
select 'SLV-3-3','VSSC' from dual union all
select 'SLV-3-4','VSSC' from dual union all
select 'SLV-3C Booster','GDA' from dual union all
select 'SLV-3D Booster','GDA' from dual union all
select 'SM-3-II-2','MHI' from dual union all
select 'SM-3-II-3','MHI' from dual union all
select 'SOZ','PROG' from dual union all
select 'SPRAN-50','ONERA' from dual union all
select 'SQ-1S','SANJ' from dual union all
select 'SR118','THKU' from dual union all
select 'SR119','AJSP' from dual union all
select 'SR120','HERC' from dual union all
select 'SR19','AJSP' from dual union all
select 'SR19AJ1','AJSP' from dual union all
select 'SR19AJ1-ATK','ATKH' from dual union all
select 'SR73AJ1','AJSP' from dual union all
select 'SR73AJ1-ATK','ATKH' from dual union all
select 'SRAM','BOUT' from dual union all
select 'SRM','THKU' from dual union all
select 'SRMU','ATKP' from dual union all
select 'SS-50','IIAE' from dual union all
select 'SS-520 St 2','ISAS' from dual union all
select 'SS-520 St 3','ISAS' from dual union all
select 'ST','MHITO' from dual union all
select 'ST-735 Stage 2','ISAS' from dual union all
select 'ST-735','ISAS' from dual union all
select 'ST-735-SB','NISST' from dual union all
select 'STIG-A','ARMAD' from dual union all
select 'STIG-B','ARMAD' from dual union all
select 'Safir Stage 2','IR' from dual union all
select 'Saluki B','IMIS' from dual union all
select 'Saluki S','IMIS' from dual union all
select 'Sandhawk','SAND' from dual union all
select 'Saturn S-I','MSFC' from dual union all
select 'Saturn S-IB','CHRMI' from dual union all
select 'Saturn S-IC','BOMI' from dual union all
select 'Saturn S-II','NAASB' from dual union all
select 'Saturn S-IV','MSFC' from dual union all
select 'Saturn S-IVB','DACHB' from dual union all
select 'Saturn S-IVD','MSFC' from dual union all
select 'Saturn S-V','GDA' from dual union all
select 'Scud B','MAK' from dual;
insert into STAGE_MANUFACTURER
select 'Scud D','KPA' from dual union all
select 'Scud','MAK' from dual union all
select 'Seagull','USN' from dual union all
select 'Sejjil Stage 2','IR' from dual union all
select 'Sejjil','IR' from dual union all
select 'Seliger 1','DRG' from dual union all
select 'Seliger 3','DRG' from dual union all
select 'Sergeant','THKX' from dual union all
select 'Shahab 3','IR' from dual union all
select 'Shaheen 2 St 1','ARMT' from dual union all
select 'Shaheen 3 St 2','PKNDC' from dual union all
select 'Shaheen St 2','PKNDC' from dual union all
select 'Shaheen','ARMT' from dual union all
select 'Shaviyt 1','IAI' from dual union all
select 'Short Al Hussein','MAK' from dual union all
select 'Shourya Booster','DRDO' from dual union all
select 'Shourya','DRDO' from dual union all
select 'Shrike','NWC' from dual union all
select 'Sidewinder 1A','NOTS' from dual union all
select 'Sidewinder 1A','THKX' from dual union all
select 'Sidewinder 1A-15','NOTS' from dual union all
select 'Sidewinder 1A-15','THKX' from dual union all
select 'Sidewinder 1D','THKX' from dual union all
select 'Sidewinder','THKX' from dual union all
select 'Silver Sparrow','IAI' from dual union all
select 'Simorgh Stage 1','IR' from dual union all
select 'Simorgh Stage 2','IR' from dual union all
select 'Sirocco','SDCO' from dual union all
select 'Skat','THKX' from dual union all
select 'Skua 2','BAJUK' from dual union all
select 'Skua 3','BAJUK' from dual union all
select 'Skua','BAJUK' from dual union all
select 'Skybolt St1','DAC' from dual union all
select 'Skybolt St2','DAC' from dual union all
select 'Smart Dispenser','CALT' from dual union all
select 'Snark Booster','NORTH' from dual union all
select 'Snipe','RORD' from dual union all
select 'Sonda 1-1','IAE' from dual union all
select 'Sonda 1-2','IAE' from dual union all
select 'Sonda 2A','AVIB' from dual union all
select 'Sonda 2M','AVIB' from dual union all
select 'Sonda 3-1','IAE' from dual union all
select 'Soyuz-2-1V 132KS','PROG' from dual union all
select 'SpaceLoft XL','UPAE' from dual union all
select 'Spaceship One','SCAL' from dual union all
select 'Spaceship Two','SCAL' from dual union all
select 'Sparrow','HUMS' from dual union all
select 'Squirt St 1','MARTO' from dual union all
select 'Squirt St 2','MARTO' from dual union all
select 'Stable Super Loki','SDCO' from dual union all
select 'Star 20','THKE' from dual union all
select 'Star 26','ATKE' from dual union all
select 'Star 26B','ATKE' from dual union all
select 'Star 26C','ATKE' from dual union all
select 'Star 27','ATKE' from dual union all
select 'Star 37B','THKE' from dual union all
select 'Star 37C','THKE' from dual union all
select 'Star 37D','THKE' from dual union all
select 'Star 37E','THKE' from dual union all
select 'Star 37F','THKE' from dual union all
select 'Star 37FM','THKE' from dual union all
select 'Star 37FMV','THKE' from dual union all
select 'Star 37N','THKE' from dual union all
select 'Star 37S','THKE' from dual union all
select 'Star 37XE','THKE' from dual union all
select 'Star 37XF','THKE' from dual union all
select 'Star 37XFP','THKE' from dual union all
select 'Star 48','THKE' from dual union all
select 'Star 48B AKM','THKE' from dual union all
select 'Star 48B','THKE' from dual union all
select 'Star 48BV','THKE' from dual union all
select 'Star 48V','THKE' from dual union all
select 'Start','MITT' from dual union all
select 'Stonechat I','RORD' from dual union all
select 'Stonechat II','RORD' from dual union all
select 'Stratos II+','DARE' from dual union all
select 'Strela APB','KHRU' from dual union all
select 'Stromboli','ONERA' from dual union all
select 'Super Arcas','ATLR' from dual union all
select 'Super Loki','SDCO' from dual union all
select 'Sword','LPC' from dual union all
select 'T-40','THKE' from dual union all
select 'T-55','THKE' from dual union all
select 'T-7 booster','SIMED' from dual union all
select 'T-7','SIMED' from dual union all
select 'T48','MARTB' from dual union all
select 'THAAD Proto','LMCO' from dual union all
select 'THAAD','LMPCO' from dual union all
select 'TR-1','NASDA' from dual union all
select 'TR-1A','NASDA' from dual union all
select 'TT-200','NASDA' from dual union all
select 'TT-210','NASDA' from dual union all
select 'TT-500','NASDA' from dual union all
select 'TT-500-S2','NASDA' from dual union all
select 'TX-135','THKXM' from dual union all
select 'TX-174','MARTO' from dual union all
select 'TX-175','MARTO' from dual union all
select 'TX-20 Sergeant','THKX' from dual union all
select 'TX-238','THKXM' from dual union all
select 'TX-239','THKXM' from dual;
insert into STAGE_MANUFACTURER
select 'TX-261','THKX' from dual union all
select 'TX-261-2','THKX' from dual union all
select 'TX-30','THKX' from dual union all
select 'TX-454','THKXM' from dual union all
select 'TX-500','THKXM' from dual union all
select 'Tacite','ONERA' from dual union all
select 'Taepo Dong 1 Stage 2','KPA' from dual union all
select 'Taepo Dong 1 Stage 3','KPA' from dual union all
select 'Taepo Dong 1','KPA' from dual union all
select 'Taepo Dong 2','KPA' from dual union all
select 'Talos','ABL' from dual union all
select 'Taurion','WFF' from dual union all
select 'Taurus','ABL' from dual union all
select 'Taurus','WFF' from dual union all
select 'Temp','MIT' from dual union all
select 'Temp-S','MIT' from dual union all
select 'Terrier 12-0','CONVPO' from dual union all
select 'Terrier Mk 12','HERCR' from dual union all
select 'Terrier Mk 70','THKX' from dual union all
select 'Terrier','HERCR' from dual union all
select 'Theodore','ATEF' from dual union all
select 'Thor DM-18','DAC' from dual union all
select 'Thor DM-18A','DAC' from dual union all
select 'Thor DM-18C','DAC' from dual union all
select 'Thor DM-21','DAC' from dual union all
select 'Thor DSV-2A','DACHB' from dual union all
select 'Thor DSV-2C','DACHB' from dual union all
select 'Tianying-3C','CSSAR' from dual union all
select 'Tianying-3F Booster','CSSAR' from dual union all
select 'Tiny Tim','USN' from dual union all
select 'Tochka','KBM' from dual union all
select 'Tochka-U','KBM' from dual union all
select 'Tomahawk 1','THKE' from dual union all
select 'Tomahawk','THKE' from dual union all
select 'Topaze VE111C','SEREB' from dual union all
select 'Topaze VE111L','SEREB' from dual union all
select 'Topaze','SEREB' from dual union all
select 'Tramontane','ONERA' from dual union all
select 'Transtage','MARTD' from dual union all
select 'Trident C-4 Stage 1','LMSC' from dual union all
select 'Trident C-4 Stage 2','HERC' from dual union all
select 'Trident C-4 Stage 2','LMSC' from dual union all
select 'Trident C-4 Stage 3','UTC' from dual union all
select 'Trident D-5 Stage 1','LMSC' from dual union all
select 'Trident D-5 Stage 2','LMSC' from dual union all
select 'Trident D-5 Stage 3','UTC' from dual union all
select 'Trident','UTC' from dual union all
select 'UA1205','UTC' from dual union all
select 'UA1206','UTC' from dual union all
select 'UA1207','UTC' from dual union all
select 'UM129A','NISST' from dual union all
select 'UR-100NA','KHRU' from dual union all
select 'UR-100NB','KHRU' from dual union all
select 'UR-100NU APB','KHRU' from dual union all
select 'Unha-1 Stage 2','KPA' from dual union all
select 'Unha-2 Stage 2','KPA' from dual union all
select 'Unha-2 Stage 3','KPA' from dual union all
select 'Unha-3 Stage 3','KPA' from dual union all
select 'Upstage','BOC' from dual union all
select 'Ute','THKU' from dual union all
select 'V-1000 Booster','SKB30' from dual union all
select 'V-1000','SKB30' from dual union all
select 'V-2','WEHR' from dual union all
select 'V-2-Ram','ABMA' from dual union all
select 'V-750 Booster','ISKRM' from dual union all
select 'V-750','FAKEL' from dual union all
select 'V-825 Booster','FAKEL' from dual union all
select 'V-825','FAKEL' from dual union all
select 'VAM Booster','AJ' from dual union all
select 'VAM-17','AJ' from dual union all
select 'VAM-20','AJ' from dual union all
select 'VLS-R1 Strapon','AVIB' from dual union all
select 'VLS-R1','AVIB' from dual union all
select 'VLS-R2 Strapon','AVIB' from dual union all
select 'VLS-R2','AVIB' from dual union all
select 'Vanguard','MARTB' from dual union all
select 'Vega P80FW','AVIO' from dual union all
select 'Vela','WRE' from dual union all
select 'Venus','SUD' from dual union all
select 'Veronique 61','LRBA' from dual union all
select 'Veronique 61M','LRBA' from dual union all
select 'Veronique AGI','LRBA' from dual union all
select 'Veronique P2','LRBA' from dual union all
select 'Veronique P6','LRBA' from dual union all
select 'Veronique R','LRBA' from dual union all
select 'Veronique','LRBA' from dual union all
select 'Vesta','LRBA' from dual union all
select 'Vesuve','ONERA' from dual union all
select 'Viking Type 9','MARTB' from dual union all
select 'Viking','MARTB' from dual union all
select 'Viper I','LPC' from dual union all
select 'Viper IC','LPC' from dual union all
select 'Viper II','LPC' from dual union all
select 'Viper IIIA','SDCO' from dual union all
select 'Viper V','OSCC' from dual union all
select 'Volga','PROG' from dual union all
select 'WAC Corporal','DAC' from dual union all
select 'WAC Corporal','JPL' from dual union all
select 'Wasserfall','WEHR' from dual union all
select 'Waxwing','RORD' from dual;
insert into STAGE_MANUFACTURER
select 'X-10','NAAD' from dual union all
select 'X-15','NAA' from dual union all
select 'X-201','ABL' from dual union all
select 'X-241','ABL' from dual union all
select 'X-248','ABL' from dual union all
select 'X-260','ABL' from dual union all
select 'X-260','LMSC' from dual union all
select 'X-265','MARTO' from dual union all
select 'X-271','ABL' from dual union all
select 'X-51A','BNAP' from dual union all
select 'X-7','LVN' from dual union all
select 'X-7A-1 Booster','ABL' from dual union all
select 'X-7A-1 Booster','LVN' from dual union all
select 'Xaero','MASTEN' from dual union all
select 'YZ-1','CALT' from dual union all
select 'YZ-1A','CALT' from dual union all
select 'YZ-2','CALT' from dual union all
select 'Yardbird','THKE' from dual union all
select 'ZN-1','NUDTC' from dual union all
select 'ZUR-215','NPOL' from dual union all
select 'Zefiro 23','AVIO' from dual union all
select 'Zefiro 9A','AVIO' from dual union all
select 'Zefiro','FIATA' from dual union all
select 'Zelzal','IR' from dual union all
select 'Zenit','CONTR' from dual union all
select 'Zeus A','GCR' from dual union all
select 'Zulfiqar','IR' from dual;


--------------------------------------------------------------------------------
-- PROPELLANT
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing PROPELLANT...');

CREATE TABLE "PROPELLANT" 
 (	"PROPELLANT_ID" NUMBER, 
	"PROPELLANT_NAME" VARCHAR2(4000), 
	 CONSTRAINT "PROPELLANT_PK" PRIMARY KEY ("PROPELLANT_ID")
USING INDEX  ENABLE, 
	 CONSTRAINT "PROPELLANT_UQ" UNIQUE ("PROPELLANT_NAME")
USING INDEX  ENABLE
 ) ;

insert into PROPELLANT
select 1,'AP' from dual union all
select 10,'Amine TG-02' from dual union all
select 100,'Solid' from dual union all
select 101,'Solid ANP2872JM-IV' from dual union all
select 102,'Solid AP' from dual union all
select 103,'Solid Arcite' from dual union all
select 104,'Solid DB' from dual union all
select 105,'Solid DB N-4' from dual union all
select 106,'Solid DB T6' from dual union all
select 107,'Solid GALCIT' from dual union all
select 108,'Solid JPL-131' from dual union all
select 109,'Solid JPL-132A' from dual union all
select 11,'Ammonia' from dual union all
select 110,'Solid PBAA' from dual union all
select 111,'Solid PS' from dual union all
select 112,'Solid RD2409' from dual union all
select 113,'Solid RD2410' from dual union all
select 114,'Solid RD2435' from dual union all
select 115,'Solid T13E1' from dual union all
select 116,'Solid T14E1' from dual union all
select 117,'Solid T9-BK-6' from dual union all
select 118,'Solid TP-H 1205C' from dual union all
select 119,'Sorbitol' from dual union all
select 12,'Aniline' from dual union all
select 120,'Triethylamine' from dual union all
select 121,'Turpentine' from dual union all
select 122,'UDMH' from dual union all
select 123,'UDMH UH25' from dual union all
select 124,'UDMH USO' from dual union all
select 125,'UDMH)' from dual union all
select 126,'UMDH' from dual union all
select 127,'UP Polyurethane' from dual union all
select 128,'USO' from dual union all
select 129,'Vinyl Isobutyl ether' from dual union all
select 13,'Aniline (ANFA)' from dual union all
select 130,'WFNA' from dual union all
select 131,'WIFNA' from dual union all
select 132,'Xe' from dual union all
select 133,'Xylidiene' from dual union all
select 14,'BTTN' from dual union all
select 15,'Butalane' from dual union all
select 16,'CMDB' from dual union all
select 17,'CTPB' from dual union all
select 18,'CTPB 16.12' from dual union all
select 19,'Diesel oil' from dual union all
select 2,'AZ-50' from dual union all
select 20,'E107M' from dual union all
select 21,'E107M polyurethane' from dual union all
select 22,'EP polyesther' from dual union all
select 23,'Elec SPT' from dual union all
select 24,'Epictete' from dual union all
select 25,'Ethanol' from dual union all
select 26,'GOX' from dual union all
select 27,'Gasoline' from dual union all
select 28,'H2O2' from dual union all
select 29,'HDA' from dual union all
select 3,'Al' from dual union all
select 30,'HEF-20' from dual union all
select 31,'HTBP' from dual union all
select 32,'HTPB' from dual union all
select 33,'HTPB 1813' from dual union all
select 34,'HTPB 1814' from dual union all
select 35,'HTPB BP-201J' from dual union all
select 36,'HTPB BP-204J' from dual union all
select 37,'HTPB BP-205J' from dual union all
select 38,'HTPB-2013' from dual union all
select 39,'Hydrazine' from dual union all
select 4,'Al E-107M' from dual union all
select 40,'Hydyne' from dual union all
select 41,'IRFMA' from dual union all
select 42,'IRFNA' from dual union all
select 43,'Isolane' from dual union all
select 44,'JPL 136' from dual union all
select 45,'JPX (JP-4' from dual union all
select 46,'Kero' from dual union all
select 47,'Kero RG-1' from dual union all
select 48,'Kero RJ-1' from dual union all
select 49,'Kero RP-1' from dual union all
select 5,'Al TP-G-3012' from dual union all
select 50,'Kero Sintin' from dual union all
select 51,'Kero T-1' from dual union all
select 52,'Kerosene' from dual union all
select 53,'Kerosene JP-4' from dual union all
select 54,'Kerosene RP-1' from dual union all
select 55,'Kerosene T-1' from dual union all
select 56,'Kerosene TG-02' from dual union all
select 57,'LH2' from dual union all
select 58,'LOX' from dual union all
select 59,'MAF-1' from dual union all
select 6,'Al TP-H-3340' from dual union all
select 60,'MMH' from dual union all
select 61,'MMH-H2O' from dual union all
select 62,'MON' from dual union all
select 63,'MON-3' from dual union all
select 64,'MON-I' from dual union all
select 65,'Methane' from dual union all
select 66,'Methane LNG' from dual union all
select 67,'N2' from dual union all
select 68,'N204' from dual union all
select 69,'N2H4' from dual;
insert into PROPELLANT
select 7,'AlTP-H-3062M' from dual union all
select 70,'N2O' from dual union all
select 71,'N2O4' from dual union all
select 72,'NA' from dual union all
select 73,'NA AK-20' from dual union all
select 74,'NA AK-20I' from dual union all
select 75,'NA AK-27' from dual union all
select 76,'NA AK-27I' from dual union all
select 77,'NA27S' from dual union all
select 78,'NEPE Polyethane' from dual union all
select 79,'NG' from dual union all
select 8,'Alcohol' from dual union all
select 80,'NH3 (Ammonia)' from dual union all
select 81,'PB' from dual union all
select 82,'PBAA' from dual union all
select 83,'PBAA TPH-3117' from dual union all
select 84,'PBAN' from dual union all
select 85,'PBHL' from dual union all
select 86,'PEG' from dual union all
select 87,'PS' from dual union all
select 88,'Paraf' from dual union all
select 89,'Plastolane' from dual union all
select 9,'Amine' from dual union all
select 90,'Plastolite' from dual union all
select 91,'Polysulfide' from dual union all
select 92,'Polysulphide' from dual union all
select 93,'Polyurethane' from dual union all
select 94,'Polyurethane DREV' from dual union all
select 95,'Polyurethane-AP' from dual union all
select 96,'Polyvinyl' from dual union all
select 97,'RFNA' from dual union all
select 98,'RFNA AK-20F' from dual union all
select 99,'SP Polysulphide' from dual;


--------------------------------------------------------------------------------
-- ENGINE_PROPELLANT
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing ENGINE_PROPELLANT...');

CREATE TABLE "ENGINE_PROPELLANT" 
 (	"ENGINE_ID" NUMBER, 
	"PROPELLANT_ID" NUMBER, 
	"OXIDIZER_OR_FUEL" VARCHAR2(8), 
	 CONSTRAINT "ENGINE_PROPELLANT_PK" PRIMARY KEY ("ENGINE_ID", "PROPELLANT_ID", "OXIDIZER_OR_FUEL")
USING INDEX  ENABLE, 
	 CONSTRAINT "ENGINE_PROPELLANT_ENGINE_FK" FOREIGN KEY ("ENGINE_ID")
	  REFERENCES "ENGINE" ("ENGINE_ID") ENABLE, 
	 CONSTRAINT "ENGINE_PROPELLANT_PROP_FK" FOREIGN KEY ("PROPELLANT_ID")
	  REFERENCES "PROPELLANT" ("PROPELLANT_ID") ENABLE
 ) ;


CREATE INDEX "ENGINE_PROPELLANT_IDX1" ON "ENGINE_PROPELLANT" ("PROPELLANT_ID") 
;
insert into ENGINE_PROPELLANT
select 10,122,'fuel' from dual union all
select 10,71,'oxidizer' from dual union all
select 100,94,'fuel' from dual union all
select 1000,82,'fuel' from dual union all
select 1001,100,'fuel' from dual union all
select 1002,100,'fuel' from dual union all
select 1003,100,'fuel' from dual union all
select 1004,100,'fuel' from dual union all
select 1005,100,'fuel' from dual union all
select 1006,100,'fuel' from dual union all
select 1007,28,'oxidizer' from dual union all
select 1007,52,'fuel' from dual union all
select 1008,28,'oxidizer' from dual union all
select 1008,52,'fuel' from dual union all
select 1009,1,'fuel' from dual union all
select 1009,87,'fuel' from dual union all
select 101,81,'fuel' from dual union all
select 1010,1,'fuel' from dual union all
select 1010,5,'fuel' from dual union all
select 1010,81,'fuel' from dual union all
select 1011,1,'fuel' from dual union all
select 1011,3,'fuel' from dual union all
select 1011,93,'fuel' from dual union all
select 1012,1,'fuel' from dual union all
select 1012,3,'fuel' from dual union all
select 1012,93,'fuel' from dual union all
select 1013,100,'fuel' from dual union all
select 1014,1,'fuel' from dual union all
select 1014,17,'fuel' from dual union all
select 1014,3,'fuel' from dual union all
select 1015,1,'fuel' from dual union all
select 1015,17,'fuel' from dual union all
select 1015,3,'fuel' from dual union all
select 1016,100,'fuel' from dual union all
select 1017,100,'fuel' from dual union all
select 1018,100,'fuel' from dual union all
select 1019,100,'fuel' from dual union all
select 102,100,'fuel' from dual union all
select 1020,1,'fuel' from dual union all
select 1020,17,'fuel' from dual union all
select 1020,3,'fuel' from dual union all
select 1021,1,'fuel' from dual union all
select 1021,17,'fuel' from dual union all
select 1021,3,'fuel' from dual union all
select 1022,1,'fuel' from dual union all
select 1022,17,'fuel' from dual union all
select 1022,7,'fuel' from dual union all
select 1023,1,'fuel' from dual union all
select 1023,17,'fuel' from dual union all
select 1023,3,'fuel' from dual union all
select 1024,1,'fuel' from dual union all
select 1024,17,'fuel' from dual union all
select 1024,3,'fuel' from dual union all
select 1025,1,'fuel' from dual union all
select 1025,17,'fuel' from dual union all
select 1025,3,'fuel' from dual union all
select 1026,1,'fuel' from dual union all
select 1026,17,'fuel' from dual union all
select 1026,3,'fuel' from dual union all
select 1027,1,'fuel' from dual union all
select 1027,17,'fuel' from dual union all
select 1027,3,'fuel' from dual union all
select 1028,1,'fuel' from dual union all
select 1028,17,'fuel' from dual union all
select 1028,3,'fuel' from dual union all
select 1029,1,'fuel' from dual union all
select 1029,3,'fuel' from dual union all
select 1029,32,'fuel' from dual union all
select 103,100,'fuel' from dual union all
select 1030,1,'fuel' from dual union all
select 1030,3,'fuel' from dual union all
select 1030,32,'fuel' from dual union all
select 1031,1,'fuel' from dual union all
select 1031,3,'fuel' from dual union all
select 1031,32,'fuel' from dual union all
select 1032,1,'fuel' from dual union all
select 1032,3,'fuel' from dual union all
select 1032,32,'fuel' from dual union all
select 1033,1,'fuel' from dual union all
select 1033,3,'fuel' from dual union all
select 1033,32,'fuel' from dual union all
select 1034,1,'fuel' from dual union all
select 1034,32,'fuel' from dual union all
select 1034,6,'fuel' from dual union all
select 1035,100,'fuel' from dual union all
select 1036,100,'fuel' from dual union all
select 1037,100,'fuel' from dual union all
select 1038,100,'fuel' from dual union all
select 1039,100,'fuel' from dual union all
select 104,100,'fuel' from dual union all
select 1040,100,'fuel' from dual union all
select 1041,32,'fuel' from dual union all
select 1042,32,'fuel' from dual union all
select 1043,100,'fuel' from dual union all
select 1044,100,'fuel' from dual union all
select 1045,32,'fuel' from dual union all
select 1046,32,'fuel' from dual union all
select 1047,32,'fuel' from dual union all
select 1048,32,'fuel' from dual union all
select 1049,32,'fuel' from dual;
insert into ENGINE_PROPELLANT
select 105,100,'fuel' from dual union all
select 1050,32,'fuel' from dual union all
select 1051,32,'fuel' from dual union all
select 1052,32,'fuel' from dual union all
select 1053,32,'fuel' from dual union all
select 1054,32,'fuel' from dual union all
select 1055,100,'fuel' from dual union all
select 1056,28,'oxidizer' from dual union all
select 1056,52,'fuel' from dual union all
select 1057,100,'fuel' from dual union all
select 1058,100,'fuel' from dual union all
select 1059,90,'fuel' from dual union all
select 106,100,'fuel' from dual union all
select 1060,60,'fuel' from dual union all
select 1060,71,'oxidizer' from dual union all
select 1061,60,'fuel' from dual union all
select 1061,71,'oxidizer' from dual union all
select 1062,1,'fuel' from dual union all
select 1062,87,'fuel' from dual union all
select 1063,100,'fuel' from dual union all
select 1064,100,'fuel' from dual union all
select 1065,91,'fuel' from dual union all
select 1066,91,'fuel' from dual union all
select 1067,100,'fuel' from dual union all
select 1069,100,'fuel' from dual union all
select 107,100,'fuel' from dual union all
select 1070,100,'fuel' from dual union all
select 1071,42,'oxidizer' from dual union all
select 1071,59,'fuel' from dual union all
select 1072,52,'fuel' from dual union all
select 1072,97,'oxidizer' from dual union all
select 1073,42,'oxidizer' from dual union all
select 1073,59,'fuel' from dual union all
select 1074,61,'fuel' from dual union all
select 1074,62,'oxidizer' from dual union all
select 1075,100,'fuel' from dual union all
select 1076,100,'fuel' from dual union all
select 1077,100,'fuel' from dual union all
select 1078,83,'fuel' from dual union all
select 1079,82,'fuel' from dual union all
select 108,100,'fuel' from dual union all
select 1080,82,'fuel' from dual union all
select 1081,100,'fuel' from dual union all
select 1082,100,'fuel' from dual union all
select 1083,100,'fuel' from dual union all
select 1084,100,'fuel' from dual union all
select 1085,60,'fuel' from dual union all
select 1085,64,'oxidizer' from dual union all
select 1086,35,'fuel' from dual union all
select 1087,35,'fuel' from dual union all
select 1088,2,'fuel' from dual union all
select 1088,71,'oxidizer' from dual union all
select 1089,62,'oxidizer' from dual union all
select 1089,69,'fuel' from dual union all
select 109,122,'fuel' from dual union all
select 109,71,'oxidizer' from dual union all
select 1090,69,'fuel' from dual union all
select 1090,71,'oxidizer' from dual union all
select 1091,69,'fuel' from dual union all
select 1091,71,'oxidizer' from dual union all
select 1092,100,'fuel' from dual union all
select 1093,100,'fuel' from dual union all
select 1094,100,'fuel' from dual union all
select 1095,100,'fuel' from dual union all
select 1096,84,'fuel' from dual union all
select 1097,84,'fuel' from dual union all
select 1098,100,'fuel' from dual union all
select 1099,17,'fuel' from dual union all
select 11,122,'fuel' from dual union all
select 11,71,'oxidizer' from dual union all
select 110,58,'oxidizer' from dual union all
select 110,8,'fuel' from dual union all
select 1100,17,'fuel' from dual union all
select 1101,17,'fuel' from dual union all
select 1102,100,'fuel' from dual union all
select 1103,32,'fuel' from dual union all
select 1104,92,'fuel' from dual union all
select 1105,17,'fuel' from dual union all
select 1106,100,'fuel' from dual union all
select 1107,82,'fuel' from dual union all
select 1108,82,'fuel' from dual union all
select 1109,111,'fuel' from dual union all
select 111,1,'fuel' from dual union all
select 111,93,'fuel' from dual union all
select 1110,82,'fuel' from dual union all
select 1111,110,'fuel' from dual union all
select 1112,82,'fuel' from dual union all
select 1113,82,'fuel' from dual union all
select 1114,82,'fuel' from dual union all
select 1115,91,'fuel' from dual union all
select 1116,82,'fuel' from dual union all
select 1117,17,'fuel' from dual union all
select 1118,17,'fuel' from dual union all
select 1119,17,'fuel' from dual union all
select 112,1,'fuel' from dual union all
select 112,93,'fuel' from dual union all
select 1120,100,'fuel' from dual union all
select 1121,17,'fuel' from dual union all
select 1122,91,'fuel' from dual union all
select 1123,82,'fuel' from dual;
insert into ENGINE_PROPELLANT
select 1124,32,'fuel' from dual union all
select 1125,100,'fuel' from dual union all
select 1126,32,'fuel' from dual union all
select 1127,32,'fuel' from dual union all
select 1128,32,'fuel' from dual union all
select 1129,100,'fuel' from dual union all
select 113,1,'fuel' from dual union all
select 113,93,'fuel' from dual union all
select 1130,100,'fuel' from dual union all
select 1131,100,'fuel' from dual union all
select 1132,100,'fuel' from dual union all
select 1133,100,'fuel' from dual union all
select 1134,24,'fuel' from dual union all
select 1135,100,'fuel' from dual union all
select 1136,100,'fuel' from dual union all
select 1137,100,'fuel' from dual union all
select 1138,122,'fuel' from dual union all
select 1138,71,'oxidizer' from dual union all
select 1139,100,'fuel' from dual union all
select 114,1,'fuel' from dual union all
select 114,93,'fuel' from dual union all
select 1140,100,'fuel' from dual union all
select 1141,100,'fuel' from dual union all
select 1142,100,'fuel' from dual union all
select 1143,100,'fuel' from dual union all
select 1144,100,'fuel' from dual union all
select 1145,69,'fuel' from dual union all
select 1145,71,'oxidizer' from dual union all
select 1146,122,'fuel' from dual union all
select 1146,72,'oxidizer' from dual union all
select 1147,100,'fuel' from dual union all
select 1148,100,'fuel' from dual union all
select 1149,72,'oxidizer' from dual union all
select 1149,9,'fuel' from dual union all
select 115,100,'fuel' from dual union all
select 1150,122,'fuel' from dual union all
select 1150,71,'oxidizer' from dual union all
select 1151,122,'fuel' from dual union all
select 1151,71,'oxidizer' from dual union all
select 1152,93,'fuel' from dual union all
select 1153,100,'fuel' from dual union all
select 1154,100,'fuel' from dual union all
select 1155,100,'fuel' from dual union all
select 1156,100,'fuel' from dual union all
select 1157,130,'oxidizer' from dual union all
select 1157,19,'fuel' from dual union all
select 1158,121,'fuel' from dual union all
select 1158,72,'oxidizer' from dual union all
select 1159,121,'fuel' from dual union all
select 1159,130,'oxidizer' from dual union all
select 116,100,'fuel' from dual union all
select 1160,52,'fuel' from dual union all
select 1160,72,'oxidizer' from dual union all
select 1161,121,'fuel' from dual union all
select 1161,72,'oxidizer' from dual union all
select 1162,121,'fuel' from dual union all
select 1162,72,'oxidizer' from dual union all
select 1163,122,'fuel' from dual union all
select 1163,71,'oxidizer' from dual union all
select 1164,122,'fuel' from dual union all
select 1164,71,'oxidizer' from dual union all
select 1165,122,'fuel' from dual union all
select 1165,71,'oxidizer' from dual union all
select 1166,122,'fuel' from dual union all
select 1166,71,'oxidizer' from dual union all
select 1167,122,'fuel' from dual union all
select 1167,71,'oxidizer' from dual union all
select 1168,122,'fuel' from dual union all
select 1168,71,'oxidizer' from dual union all
select 1169,123,'fuel' from dual union all
select 1169,71,'oxidizer' from dual union all
select 117,122,'fuel' from dual union all
select 117,130,'oxidizer' from dual union all
select 1170,122,'fuel' from dual union all
select 1170,71,'oxidizer' from dual union all
select 1171,123,'fuel' from dual union all
select 1171,71,'oxidizer' from dual union all
select 1172,123,'fuel' from dual union all
select 1172,71,'oxidizer' from dual union all
select 1173,100,'fuel' from dual union all
select 1174,100,'fuel' from dual union all
select 1175,122,'fuel' from dual union all
select 1175,71,'oxidizer' from dual union all
select 1176,57,'fuel' from dual union all
select 1176,58,'oxidizer' from dual union all
select 1177,57,'fuel' from dual union all
select 1177,58,'oxidizer' from dual union all
select 1178,12,'fuel' from dual union all
select 1178,8,'fuel' from dual union all
select 1178,97,'oxidizer' from dual union all
select 1179,129,'fuel' from dual union all
select 1179,72,'oxidizer' from dual union all
select 118,122,'fuel' from dual union all
select 118,72,'oxidizer' from dual union all
select 1180,114,'fuel' from dual union all
select 1181,100,'fuel' from dual union all
select 1182,100,'fuel' from dual union all
select 1183,100,'fuel' from dual union all
select 1184,100,'fuel' from dual union all
select 1185,100,'fuel' from dual;
insert into ENGINE_PROPELLANT
select 1186,100,'fuel' from dual union all
select 1187,100,'fuel' from dual union all
select 1188,100,'fuel' from dual union all
select 1189,100,'fuel' from dual union all
select 119,122,'fuel' from dual union all
select 119,131,'oxidizer' from dual union all
select 1190,100,'fuel' from dual union all
select 1191,1,'fuel' from dual union all
select 1191,104,'fuel' from dual union all
select 1192,100,'fuel' from dual union all
select 1193,100,'fuel' from dual union all
select 1194,100,'fuel' from dual union all
select 1195,100,'fuel' from dual union all
select 1196,100,'fuel' from dual union all
select 1197,100,'fuel' from dual union all
select 1198,100,'fuel' from dual union all
select 1199,100,'fuel' from dual union all
select 12,122,'fuel' from dual union all
select 12,75,'oxidizer' from dual union all
select 120,122,'fuel' from dual union all
select 120,72,'oxidizer' from dual union all
select 1200,100,'fuel' from dual union all
select 1201,1,'fuel' from dual union all
select 1201,104,'fuel' from dual union all
select 1202,100,'fuel' from dual union all
select 1203,100,'fuel' from dual union all
select 1204,46,'fuel' from dual union all
select 1204,58,'oxidizer' from dual union all
select 1205,12,'fuel' from dual union all
select 1205,8,'fuel' from dual union all
select 1205,97,'oxidizer' from dual union all
select 1206,12,'fuel' from dual union all
select 1206,8,'fuel' from dual union all
select 1206,97,'oxidizer' from dual union all
select 1207,11,'fuel' from dual union all
select 1207,58,'oxidizer' from dual union all
select 1208,58,'oxidizer' from dual union all
select 1208,8,'fuel' from dual union all
select 1209,58,'oxidizer' from dual union all
select 1209,8,'fuel' from dual union all
select 121,122,'fuel' from dual union all
select 121,72,'oxidizer' from dual union all
select 1210,58,'oxidizer' from dual union all
select 1210,80,'fuel' from dual union all
select 1211,11,'fuel' from dual union all
select 1211,58,'oxidizer' from dual union all
select 1212,91,'fuel' from dual union all
select 1213,111,'fuel' from dual union all
select 1214,100,'fuel' from dual union all
select 1215,82,'fuel' from dual union all
select 1216,82,'fuel' from dual union all
select 1217,82,'fuel' from dual union all
select 1218,82,'fuel' from dual union all
select 1219,82,'fuel' from dual union all
select 122,122,'fuel' from dual union all
select 122,72,'oxidizer' from dual union all
select 1220,82,'fuel' from dual union all
select 1221,100,'fuel' from dual union all
select 1222,100,'fuel' from dual union all
select 1223,100,'fuel' from dual union all
select 1224,100,'fuel' from dual union all
select 1225,46,'fuel' from dual union all
select 1225,58,'oxidizer' from dual union all
select 1226,58,'oxidizer' from dual union all
select 1226,8,'fuel' from dual union all
select 1227,122,'fuel' from dual union all
select 1227,77,'oxidizer' from dual union all
select 1228,46,'fuel' from dual union all
select 1228,58,'oxidizer' from dual union all
select 1229,46,'fuel' from dual union all
select 1229,58,'oxidizer' from dual union all
select 123,122,'fuel' from dual union all
select 123,72,'oxidizer' from dual union all
select 1230,122,'fuel' from dual union all
select 1230,71,'oxidizer' from dual union all
select 1231,122,'fuel' from dual union all
select 1231,71,'oxidizer' from dual union all
select 1232,122,'fuel' from dual union all
select 1232,71,'oxidizer' from dual union all
select 1233,122,'fuel' from dual union all
select 1233,71,'oxidizer' from dual union all
select 1234,122,'fuel' from dual union all
select 1234,71,'oxidizer' from dual union all
select 1235,122,'fuel' from dual union all
select 1235,71,'oxidizer' from dual union all
select 1236,122,'fuel' from dual union all
select 1236,71,'oxidizer' from dual union all
select 1237,122,'fuel' from dual union all
select 1237,71,'oxidizer' from dual union all
select 1238,122,'fuel' from dual union all
select 1238,71,'oxidizer' from dual union all
select 1239,122,'fuel' from dual union all
select 1239,72,'oxidizer' from dual union all
select 124,122,'fuel' from dual union all
select 124,72,'oxidizer' from dual union all
select 1240,122,'fuel' from dual union all
select 1240,77,'oxidizer' from dual union all
select 1241,122,'fuel' from dual union all
select 1241,71,'oxidizer' from dual union all
select 1242,122,'fuel' from dual;
insert into ENGINE_PROPELLANT
select 1242,71,'oxidizer' from dual union all
select 1243,122,'fuel' from dual union all
select 1243,71,'oxidizer' from dual union all
select 1244,122,'fuel' from dual union all
select 1244,71,'oxidizer' from dual union all
select 1245,57,'fuel' from dual union all
select 1245,58,'oxidizer' from dual union all
select 1246,57,'fuel' from dual union all
select 1246,58,'oxidizer' from dual union all
select 1247,57,'fuel' from dual union all
select 1247,58,'oxidizer' from dual union all
select 1248,57,'fuel' from dual union all
select 1248,58,'oxidizer' from dual union all
select 1249,28,'oxidizer' from dual union all
select 1249,52,'fuel' from dual union all
select 125,2,'fuel' from dual union all
select 125,71,'oxidizer' from dual union all
select 1250,100,'fuel' from dual union all
select 1251,100,'fuel' from dual union all
select 1252,34,'fuel' from dual union all
select 1253,32,'fuel' from dual union all
select 1254,32,'fuel' from dual union all
select 1255,38,'fuel' from dual union all
select 1256,100,'fuel' from dual union all
select 1257,100,'fuel' from dual union all
select 126,2,'fuel' from dual union all
select 126,71,'oxidizer' from dual union all
select 127,2,'fuel' from dual union all
select 127,71,'oxidizer' from dual union all
select 128,2,'fuel' from dual union all
select 128,71,'oxidizer' from dual union all
select 129,2,'fuel' from dual union all
select 129,71,'oxidizer' from dual union all
select 13,51,'fuel' from dual union all
select 13,58,'oxidizer' from dual union all
select 130,2,'fuel' from dual union all
select 130,71,'oxidizer' from dual union all
select 131,12,'fuel' from dual union all
select 131,8,'fuel' from dual union all
select 131,97,'oxidizer' from dual union all
select 132,12,'fuel' from dual union all
select 132,8,'fuel' from dual union all
select 132,97,'oxidizer' from dual union all
select 133,12,'fuel' from dual union all
select 133,8,'fuel' from dual union all
select 133,97,'oxidizer' from dual union all
select 134,12,'fuel' from dual union all
select 134,8,'fuel' from dual union all
select 134,97,'oxidizer' from dual union all
select 135,122,'fuel' from dual union all
select 135,72,'oxidizer' from dual union all
select 136,122,'fuel' from dual union all
select 136,72,'oxidizer' from dual union all
select 137,122,'fuel' from dual union all
select 137,72,'oxidizer' from dual union all
select 138,12,'fuel' from dual union all
select 138,8,'fuel' from dual union all
select 138,97,'oxidizer' from dual union all
select 139,12,'fuel' from dual union all
select 139,8,'fuel' from dual union all
select 139,97,'oxidizer' from dual union all
select 14,51,'fuel' from dual union all
select 14,58,'oxidizer' from dual union all
select 140,46,'fuel' from dual union all
select 140,58,'oxidizer' from dual union all
select 141,100,'fuel' from dual union all
select 142,13,'fuel' from dual union all
select 142,41,'oxidizer' from dual union all
select 143,28,'oxidizer' from dual union all
select 143,53,'fuel' from dual union all
select 144,30,'fuel' from dual union all
select 145,100,'fuel' from dual union all
select 146,100,'fuel' from dual union all
select 147,100,'fuel' from dual union all
select 148,100,'fuel' from dual union all
select 149,100,'fuel' from dual union all
select 15,47,'fuel' from dual union all
select 15,58,'oxidizer' from dual union all
select 150,32,'fuel' from dual union all
select 151,32,'fuel' from dual union all
select 152,32,'fuel' from dual union all
select 153,100,'fuel' from dual union all
select 154,52,'fuel' from dual union all
select 154,72,'oxidizer' from dual union all
select 155,100,'fuel' from dual union all
select 156,100,'fuel' from dual union all
select 157,53,'fuel' from dual union all
select 157,97,'oxidizer' from dual union all
select 158,100,'fuel' from dual union all
select 159,100,'fuel' from dual union all
select 16,46,'fuel' from dual union all
select 16,58,'oxidizer' from dual union all
select 160,60,'fuel' from dual union all
select 160,71,'oxidizer' from dual union all
select 161,96,'fuel' from dual union all
select 162,100,'fuel' from dual union all
select 163,1,'fuel' from dual union all
select 163,3,'fuel' from dual union all
select 163,32,'fuel' from dual union all
select 164,1,'fuel' from dual;
insert into ENGINE_PROPELLANT
select 164,3,'fuel' from dual union all
select 164,31,'fuel' from dual union all
select 165,100,'fuel' from dual union all
select 166,1,'fuel' from dual union all
select 166,3,'fuel' from dual union all
select 166,31,'fuel' from dual union all
select 167,42,'oxidizer' from dual union all
select 167,53,'fuel' from dual union all
select 168,125,'fuel' from dual union all
select 168,42,'oxidizer' from dual union all
select 168,45,'fuel' from dual union all
select 169,12,'fuel' from dual union all
select 169,8,'fuel' from dual union all
select 169,97,'oxidizer' from dual union all
select 17,46,'fuel' from dual union all
select 17,58,'oxidizer' from dual union all
select 170,100,'fuel' from dual union all
select 171,52,'fuel' from dual union all
select 171,72,'oxidizer' from dual union all
select 172,81,'fuel' from dual union all
select 173,100,'fuel' from dual union all
select 174,100,'fuel' from dual union all
select 175,100,'fuel' from dual union all
select 176,100,'fuel' from dual union all
select 177,100,'fuel' from dual union all
select 178,100,'fuel' from dual union all
select 179,100,'fuel' from dual union all
select 18,50,'fuel' from dual union all
select 18,58,'oxidizer' from dual union all
select 180,100,'fuel' from dual union all
select 181,100,'fuel' from dual union all
select 182,100,'fuel' from dual union all
select 183,100,'fuel' from dual union all
select 184,100,'fuel' from dual union all
select 185,100,'fuel' from dual union all
select 186,108,'fuel' from dual union all
select 187,108,'fuel' from dual union all
select 188,46,'fuel' from dual union all
select 188,58,'oxidizer' from dual union all
select 189,2,'fuel' from dual union all
select 189,71,'oxidizer' from dual union all
select 19,122,'fuel' from dual union all
select 19,71,'oxidizer' from dual union all
select 190,32,'fuel' from dual union all
select 191,32,'fuel' from dual union all
select 192,100,'fuel' from dual union all
select 192,70,'oxidizer' from dual union all
select 193,100,'fuel' from dual union all
select 194,32,'fuel' from dual union all
select 195,32,'fuel' from dual union all
select 196,100,'fuel' from dual union all
select 197,100,'fuel' from dual union all
select 198,57,'fuel' from dual union all
select 198,58,'oxidizer' from dual union all
select 199,100,'fuel' from dual union all
select 2,100,'fuel' from dual union all
select 20,100,'fuel' from dual union all
select 200,100,'fuel' from dual union all
select 201,100,'fuel' from dual union all
select 202,100,'fuel' from dual union all
select 203,58,'oxidizer' from dual union all
select 203,66,'fuel' from dual union all
select 204,28,'oxidizer' from dual union all
select 204,52,'fuel' from dual union all
select 205,69,'fuel' from dual union all
select 205,71,'oxidizer' from dual union all
select 206,60,'fuel' from dual union all
select 206,71,'oxidizer' from dual union all
select 207,100,'fuel' from dual union all
select 208,100,'fuel' from dual union all
select 209,100,'fuel' from dual union all
select 21,101,'fuel' from dual union all
select 210,100,'fuel' from dual union all
select 211,96,'fuel' from dual union all
select 212,100,'fuel' from dual union all
select 213,122,'fuel' from dual union all
select 213,42,'oxidizer' from dual union all
select 214,122,'fuel' from dual union all
select 214,72,'oxidizer' from dual union all
select 215,122,'fuel' from dual union all
select 215,42,'oxidizer' from dual union all
select 216,122,'fuel' from dual union all
select 216,42,'oxidizer' from dual union all
select 217,124,'fuel' from dual union all
select 217,29,'oxidizer' from dual union all
select 218,100,'fuel' from dual union all
select 219,100,'fuel' from dual union all
select 22,100,'fuel' from dual union all
select 220,26,'oxidizer' from dual union all
select 220,50,'fuel' from dual union all
select 221,100,'fuel' from dual union all
select 222,100,'fuel' from dual union all
select 223,100,'fuel' from dual union all
select 224,100,'fuel' from dual union all
select 225,57,'fuel' from dual union all
select 225,58,'oxidizer' from dual union all
select 226,100,'fuel' from dual union all
select 227,100,'fuel' from dual union all
select 228,100,'fuel' from dual union all
select 229,100,'fuel' from dual;
insert into ENGINE_PROPELLANT
select 23,100,'fuel' from dual union all
select 230,100,'fuel' from dual union all
select 231,100,'fuel' from dual union all
select 232,100,'fuel' from dual union all
select 233,100,'fuel' from dual union all
select 234,32,'fuel' from dual union all
select 235,100,'fuel' from dual union all
select 236,100,'fuel' from dual union all
select 237,100,'fuel' from dual union all
select 238,100,'fuel' from dual union all
select 239,100,'fuel' from dual union all
select 24,100,'fuel' from dual union all
select 240,100,'fuel' from dual union all
select 241,100,'fuel' from dual union all
select 242,100,'fuel' from dual union all
select 243,100,'fuel' from dual union all
select 244,100,'fuel' from dual union all
select 245,100,'fuel' from dual union all
select 246,100,'fuel' from dual union all
select 247,122,'fuel' from dual union all
select 247,71,'oxidizer' from dual union all
select 248,12,'fuel' from dual union all
select 248,42,'oxidizer' from dual union all
select 249,12,'fuel' from dual union all
select 249,97,'oxidizer' from dual union all
select 25,122,'fuel' from dual union all
select 25,71,'oxidizer' from dual union all
select 250,100,'fuel' from dual union all
select 251,100,'fuel' from dual union all
select 252,112,'fuel' from dual union all
select 253,113,'fuel' from dual union all
select 254,46,'fuel' from dual union all
select 254,58,'oxidizer' from dual union all
select 255,100,'fuel' from dual union all
select 256,100,'fuel' from dual union all
select 257,100,'fuel' from dual union all
select 258,100,'fuel' from dual union all
select 259,100,'fuel' from dual union all
select 26,122,'fuel' from dual union all
select 26,71,'oxidizer' from dual union all
select 260,100,'fuel' from dual union all
select 261,100,'fuel' from dual union all
select 262,100,'fuel' from dual union all
select 263,100,'fuel' from dual union all
select 264,100,'fuel' from dual union all
select 265,58,'oxidizer' from dual union all
select 265,8,'fuel' from dual union all
select 266,100,'fuel' from dual union all
select 267,100,'fuel' from dual union all
select 268,100,'fuel' from dual union all
select 269,100,'fuel' from dual union all
select 27,100,'fuel' from dual union all
select 270,100,'fuel' from dual union all
select 271,100,'fuel' from dual union all
select 272,100,'fuel' from dual union all
select 273,100,'fuel' from dual union all
select 274,100,'fuel' from dual union all
select 275,100,'fuel' from dual union all
select 276,100,'fuel' from dual union all
select 277,119,'fuel' from dual union all
select 277,3,'fuel' from dual union all
select 277,70,'oxidizer' from dual union all
select 277,88,'fuel' from dual union all
select 278,100,'fuel' from dual union all
select 279,100,'fuel' from dual union all
select 28,122,'fuel' from dual union all
select 28,71,'oxidizer' from dual union all
select 280,122,'fuel' from dual union all
select 280,71,'oxidizer' from dual union all
select 281,100,'fuel' from dual union all
select 282,100,'fuel' from dual union all
select 283,67,'fuel' from dual union all
select 284,122,'fuel' from dual union all
select 284,71,'oxidizer' from dual union all
select 285,122,'fuel' from dual union all
select 285,71,'oxidizer' from dual union all
select 286,122,'fuel' from dual union all
select 286,71,'oxidizer' from dual union all
select 287,122,'fuel' from dual union all
select 287,71,'oxidizer' from dual union all
select 288,100,'fuel' from dual union all
select 289,58,'oxidizer' from dual union all
select 289,81,'fuel' from dual union all
select 29,100,'fuel' from dual union all
select 290,100,'fuel' from dual union all
select 291,60,'fuel' from dual union all
select 291,71,'oxidizer' from dual union all
select 292,100,'fuel' from dual union all
select 293,60,'fuel' from dual union all
select 293,62,'oxidizer' from dual union all
select 293,71,'oxidizer' from dual union all
select 294,32,'fuel' from dual union all
select 295,32,'fuel' from dual union all
select 296,39,'fuel' from dual union all
select 297,96,'fuel' from dual union all
select 298,100,'fuel' from dual union all
select 299,54,'fuel' from dual union all
select 299,58,'oxidizer' from dual union all
select 3,100,'fuel' from dual union all
select 30,100,'fuel' from dual;
insert into ENGINE_PROPELLANT
select 300,100,'fuel' from dual union all
select 301,100,'fuel' from dual union all
select 302,100,'fuel' from dual union all
select 303,100,'fuel' from dual union all
select 304,100,'fuel' from dual union all
select 305,100,'fuel' from dual union all
select 306,100,'fuel' from dual union all
select 307,60,'fuel' from dual union all
select 307,71,'oxidizer' from dual union all
select 308,1,'fuel' from dual union all
select 308,87,'fuel' from dual union all
select 309,54,'fuel' from dual union all
select 309,58,'oxidizer' from dual union all
select 31,100,'fuel' from dual union all
select 310,46,'fuel' from dual union all
select 310,58,'oxidizer' from dual union all
select 311,32,'fuel' from dual union all
select 312,32,'fuel' from dual union all
select 313,32,'fuel' from dual union all
select 314,32,'fuel' from dual union all
select 315,100,'fuel' from dual union all
select 316,100,'fuel' from dual union all
select 317,100,'fuel' from dual union all
select 318,28,'oxidizer' from dual union all
select 318,52,'fuel' from dual union all
select 319,28,'oxidizer' from dual union all
select 319,52,'fuel' from dual union all
select 32,100,'fuel' from dual union all
select 320,28,'oxidizer' from dual union all
select 320,52,'fuel' from dual union all
select 321,28,'oxidizer' from dual union all
select 321,52,'fuel' from dual union all
select 322,100,'fuel' from dual union all
select 323,100,'fuel' from dual union all
select 324,100,'fuel' from dual union all
select 325,100,'fuel' from dual union all
select 326,100,'fuel' from dual union all
select 327,100,'fuel' from dual union all
select 328,100,'fuel' from dual union all
select 329,100,'fuel' from dual union all
select 33,100,'fuel' from dual union all
select 330,46,'fuel' from dual union all
select 330,58,'oxidizer' from dual union all
select 331,32,'fuel' from dual union all
select 332,32,'fuel' from dual union all
select 333,32,'fuel' from dual union all
select 334,32,'fuel' from dual union all
select 335,32,'fuel' from dual union all
select 336,58,'oxidizer' from dual union all
select 336,81,'fuel' from dual union all
select 337,100,'fuel' from dual union all
select 338,32,'fuel' from dual union all
select 338,70,'oxidizer' from dual union all
select 339,60,'fuel' from dual union all
select 339,63,'oxidizer' from dual union all
select 34,100,'fuel' from dual union all
select 340,100,'fuel' from dual union all
select 341,100,'fuel' from dual union all
select 342,57,'fuel' from dual union all
select 342,58,'oxidizer' from dual union all
select 343,57,'fuel' from dual union all
select 343,58,'oxidizer' from dual union all
select 344,100,'fuel' from dual union all
select 345,100,'fuel' from dual union all
select 346,122,'fuel' from dual union all
select 346,71,'oxidizer' from dual union all
select 347,100,'fuel' from dual union all
select 347,58,'oxidizer' from dual union all
select 348,100,'fuel' from dual union all
select 349,32,'fuel' from dual union all
select 349,58,'oxidizer' from dual union all
select 35,100,'fuel' from dual union all
select 350,58,'oxidizer' from dual union all
select 350,8,'fuel' from dual union all
select 351,58,'oxidizer' from dual union all
select 351,8,'fuel' from dual union all
select 352,58,'oxidizer' from dual union all
select 352,8,'fuel' from dual union all
select 353,100,'fuel' from dual union all
select 354,100,'fuel' from dual union all
select 355,100,'fuel' from dual union all
select 356,122,'fuel' from dual union all
select 356,71,'oxidizer' from dual union all
select 357,82,'fuel' from dual union all
select 358,82,'fuel' from dual union all
select 359,32,'fuel' from dual union all
select 359,70,'oxidizer' from dual union all
select 36,100,'fuel' from dual union all
select 360,100,'fuel' from dual union all
select 361,100,'fuel' from dual union all
select 362,100,'fuel' from dual union all
select 363,100,'fuel' from dual union all
select 364,100,'fuel' from dual union all
select 365,100,'fuel' from dual union all
select 366,57,'fuel' from dual union all
select 366,58,'oxidizer' from dual union all
select 367,100,'fuel' from dual union all
select 368,33,'fuel' from dual union all
select 369,128,'fuel' from dual union all
select 369,29,'oxidizer' from dual;
insert into ENGINE_PROPELLANT
select 37,117,'fuel' from dual union all
select 370,60,'fuel' from dual union all
select 370,71,'oxidizer' from dual union all
select 371,1,'fuel' from dual union all
select 371,3,'fuel' from dual union all
select 372,1,'fuel' from dual union all
select 372,3,'fuel' from dual union all
select 373,100,'fuel' from dual union all
select 374,57,'fuel' from dual union all
select 374,58,'oxidizer' from dual union all
select 375,57,'fuel' from dual union all
select 375,58,'oxidizer' from dual union all
select 376,100,'fuel' from dual union all
select 377,100,'fuel' from dual union all
select 378,44,'fuel' from dual union all
select 379,1,'fuel' from dual union all
select 379,82,'fuel' from dual union all
select 38,100,'fuel' from dual union all
select 380,1,'fuel' from dual union all
select 380,82,'fuel' from dual union all
select 381,1,'fuel' from dual union all
select 381,82,'fuel' from dual union all
select 382,1,'fuel' from dual union all
select 382,82,'fuel' from dual union all
select 383,104,'fuel' from dual union all
select 384,104,'fuel' from dual union all
select 385,100,'fuel' from dual union all
select 386,100,'fuel' from dual union all
select 387,100,'fuel' from dual union all
select 388,100,'fuel' from dual union all
select 389,100,'fuel' from dual union all
select 39,100,'fuel' from dual union all
select 390,100,'fuel' from dual union all
select 391,100,'fuel' from dual union all
select 392,100,'fuel' from dual union all
select 393,22,'fuel' from dual union all
select 394,22,'fuel' from dual union all
select 395,100,'fuel' from dual union all
select 396,22,'fuel' from dual union all
select 397,22,'fuel' from dual union all
select 398,127,'fuel' from dual union all
select 399,99,'fuel' from dual union all
select 4,100,'fuel' from dual union all
select 40,100,'fuel' from dual union all
select 400,127,'fuel' from dual union all
select 401,127,'fuel' from dual union all
select 402,122,'fuel' from dual union all
select 402,73,'oxidizer' from dual union all
select 403,122,'fuel' from dual union all
select 403,71,'oxidizer' from dual union all
select 404,32,'fuel' from dual union all
select 405,100,'fuel' from dual union all
select 406,32,'fuel' from dual union all
select 407,32,'fuel' from dual union all
select 408,32,'fuel' from dual union all
select 409,32,'fuel' from dual union all
select 41,100,'fuel' from dual union all
select 410,32,'fuel' from dual union all
select 411,32,'fuel' from dual union all
select 412,100,'fuel' from dual union all
select 413,122,'fuel' from dual union all
select 413,71,'oxidizer' from dual union all
select 414,100,'fuel' from dual union all
select 415,100,'fuel' from dual union all
select 416,122,'fuel' from dual union all
select 416,71,'oxidizer' from dual union all
select 417,122,'fuel' from dual union all
select 417,68,'oxidizer' from dual union all
select 418,122,'fuel' from dual union all
select 418,71,'oxidizer' from dual union all
select 419,100,'fuel' from dual union all
select 42,100,'fuel' from dual union all
select 420,100,'fuel' from dual union all
select 421,100,'fuel' from dual union all
select 422,46,'fuel' from dual union all
select 422,58,'oxidizer' from dual union all
select 423,100,'fuel' from dual union all
select 424,122,'fuel' from dual union all
select 424,72,'oxidizer' from dual union all
select 425,122,'fuel' from dual union all
select 425,72,'oxidizer' from dual union all
select 426,122,'fuel' from dual union all
select 426,68,'oxidizer' from dual union all
select 427,122,'fuel' from dual union all
select 427,71,'oxidizer' from dual union all
select 428,122,'fuel' from dual union all
select 428,71,'oxidizer' from dual union all
select 429,122,'fuel' from dual union all
select 429,71,'oxidizer' from dual union all
select 43,93,'fuel' from dual union all
select 430,122,'fuel' from dual union all
select 430,72,'oxidizer' from dual union all
select 431,72,'oxidizer' from dual union all
select 431,9,'fuel' from dual union all
select 432,122,'fuel' from dual union all
select 432,72,'oxidizer' from dual union all
select 433,122,'fuel' from dual union all
select 433,71,'oxidizer' from dual union all
select 434,57,'fuel' from dual union all
select 434,58,'oxidizer' from dual;
insert into ENGINE_PROPELLANT
select 435,100,'fuel' from dual union all
select 436,60,'fuel' from dual union all
select 436,71,'oxidizer' from dual union all
select 437,100,'fuel' from dual union all
select 438,100,'fuel' from dual union all
select 439,100,'fuel' from dual union all
select 44,100,'fuel' from dual union all
select 440,100,'fuel' from dual union all
select 441,100,'fuel' from dual union all
select 442,46,'fuel' from dual union all
select 442,58,'oxidizer' from dual union all
select 443,100,'fuel' from dual union all
select 444,100,'fuel' from dual union all
select 445,81,'fuel' from dual union all
select 446,81,'fuel' from dual union all
select 447,93,'fuel' from dual union all
select 448,93,'fuel' from dual union all
select 449,32,'fuel' from dual union all
select 45,100,'fuel' from dual union all
select 450,100,'fuel' from dual union all
select 451,130,'oxidizer' from dual union all
select 451,52,'fuel' from dual union all
select 452,122,'fuel' from dual union all
select 452,130,'oxidizer' from dual union all
select 453,2,'fuel' from dual union all
select 453,71,'oxidizer' from dual union all
select 454,122,'fuel' from dual union all
select 454,130,'oxidizer' from dual union all
select 455,57,'fuel' from dual union all
select 455,58,'oxidizer' from dual union all
select 456,57,'fuel' from dual union all
select 456,58,'oxidizer' from dual union all
select 457,57,'fuel' from dual union all
select 457,58,'oxidizer' from dual union all
select 458,57,'fuel' from dual union all
select 458,58,'oxidizer' from dual union all
select 459,57,'fuel' from dual union all
select 459,58,'oxidizer' from dual union all
select 46,100,'fuel' from dual union all
select 460,100,'fuel' from dual union all
select 461,32,'fuel' from dual union all
select 462,100,'fuel' from dual union all
select 463,122,'fuel' from dual union all
select 463,71,'oxidizer' from dual union all
select 464,100,'fuel' from dual union all
select 465,100,'fuel' from dual union all
select 466,100,'fuel' from dual union all
select 467,12,'fuel' from dual union all
select 467,97,'oxidizer' from dual union all
select 468,12,'fuel' from dual union all
select 468,97,'oxidizer' from dual union all
select 469,60,'fuel' from dual union all
select 469,71,'oxidizer' from dual union all
select 47,100,'fuel' from dual union all
select 470,12,'fuel' from dual union all
select 470,97,'oxidizer' from dual union all
select 471,122,'fuel' from dual union all
select 471,27,'fuel' from dual union all
select 471,42,'oxidizer' from dual union all
select 472,2,'fuel' from dual union all
select 472,71,'oxidizer' from dual union all
select 473,2,'fuel' from dual union all
select 473,71,'oxidizer' from dual union all
select 474,2,'fuel' from dual union all
select 474,71,'oxidizer' from dual union all
select 475,2,'fuel' from dual union all
select 475,71,'oxidizer' from dual union all
select 476,2,'fuel' from dual union all
select 476,71,'oxidizer' from dual union all
select 477,2,'fuel' from dual union all
select 477,71,'oxidizer' from dual union all
select 478,2,'fuel' from dual union all
select 478,71,'oxidizer' from dual union all
select 479,2,'fuel' from dual union all
select 479,71,'oxidizer' from dual union all
select 48,100,'fuel' from dual union all
select 480,2,'fuel' from dual union all
select 480,71,'oxidizer' from dual union all
select 481,2,'fuel' from dual union all
select 481,71,'oxidizer' from dual union all
select 482,122,'fuel' from dual union all
select 482,72,'oxidizer' from dual union all
select 483,100,'fuel' from dual union all
select 484,100,'fuel' from dual union all
select 485,60,'fuel' from dual union all
select 485,63,'oxidizer' from dual union all
select 486,115,'fuel' from dual union all
select 487,100,'fuel' from dual union all
select 488,100,'fuel' from dual union all
select 489,62,'oxidizer' from dual union all
select 489,69,'fuel' from dual union all
select 49,100,'fuel' from dual union all
select 490,62,'oxidizer' from dual union all
select 490,69,'fuel' from dual union all
select 491,100,'fuel' from dual union all
select 492,100,'fuel' from dual union all
select 493,100,'fuel' from dual union all
select 494,100,'fuel' from dual union all
select 495,100,'fuel' from dual union all
select 496,100,'fuel' from dual;
insert into ENGINE_PROPELLANT
select 497,100,'fuel' from dual union all
select 498,100,'fuel' from dual union all
select 499,100,'fuel' from dual union all
select 5,122,'fuel' from dual union all
select 5,71,'oxidizer' from dual union all
select 50,50,'fuel' from dual union all
select 50,58,'oxidizer' from dual union all
select 500,100,'fuel' from dual union all
select 501,100,'fuel' from dual union all
select 502,17,'fuel' from dual union all
select 503,36,'fuel' from dual union all
select 504,100,'fuel' from dual union all
select 505,100,'fuel' from dual union all
select 506,32,'fuel' from dual union all
select 507,36,'fuel' from dual union all
select 508,36,'fuel' from dual union all
select 509,100,'fuel' from dual union all
select 51,122,'fuel' from dual union all
select 51,71,'oxidizer' from dual union all
select 510,32,'fuel' from dual union all
select 511,37,'fuel' from dual union all
select 512,37,'fuel' from dual union all
select 513,32,'fuel' from dual union all
select 514,100,'fuel' from dual union all
select 515,32,'fuel' from dual union all
select 516,100,'fuel' from dual union all
select 517,100,'fuel' from dual union all
select 518,100,'fuel' from dual union all
select 519,100,'fuel' from dual union all
select 52,100,'fuel' from dual union all
select 520,100,'fuel' from dual union all
select 521,100,'fuel' from dual union all
select 522,100,'fuel' from dual union all
select 523,100,'fuel' from dual union all
select 524,106,'fuel' from dual union all
select 525,84,'fuel' from dual union all
select 526,93,'fuel' from dual union all
select 527,93,'fuel' from dual union all
select 528,16,'fuel' from dual union all
select 529,100,'fuel' from dual union all
select 53,100,'fuel' from dual union all
select 530,100,'fuel' from dual union all
select 531,54,'fuel' from dual union all
select 531,58,'oxidizer' from dual union all
select 532,54,'fuel' from dual union all
select 532,58,'oxidizer' from dual union all
select 533,54,'fuel' from dual union all
select 533,58,'oxidizer' from dual union all
select 534,54,'fuel' from dual union all
select 534,58,'oxidizer' from dual union all
select 535,54,'fuel' from dual union all
select 535,58,'oxidizer' from dual union all
select 536,54,'fuel' from dual union all
select 536,58,'oxidizer' from dual union all
select 537,54,'fuel' from dual union all
select 537,58,'oxidizer' from dual union all
select 538,54,'fuel' from dual union all
select 538,58,'oxidizer' from dual union all
select 539,54,'fuel' from dual union all
select 539,58,'oxidizer' from dual union all
select 54,100,'fuel' from dual union all
select 540,54,'fuel' from dual union all
select 540,58,'oxidizer' from dual union all
select 541,54,'fuel' from dual union all
select 541,58,'oxidizer' from dual union all
select 542,100,'fuel' from dual union all
select 543,100,'fuel' from dual union all
select 544,100,'fuel' from dual union all
select 545,103,'fuel' from dual union all
select 546,100,'fuel' from dual union all
select 547,100,'fuel' from dual union all
select 548,100,'fuel' from dual union all
select 549,100,'fuel' from dual union all
select 55,100,'fuel' from dual union all
select 550,100,'fuel' from dual union all
select 551,100,'fuel' from dual union all
select 552,100,'fuel' from dual union all
select 553,48,'fuel' from dual union all
select 553,58,'oxidizer' from dual union all
select 554,48,'fuel' from dual union all
select 554,58,'oxidizer' from dual union all
select 555,48,'fuel' from dual union all
select 555,58,'oxidizer' from dual union all
select 556,48,'fuel' from dual union all
select 556,58,'oxidizer' from dual union all
select 557,100,'fuel' from dual union all
select 558,100,'fuel' from dual union all
select 559,100,'fuel' from dual union all
select 56,100,'fuel' from dual union all
select 560,100,'fuel' from dual union all
select 561,100,'fuel' from dual union all
select 562,39,'fuel' from dual union all
select 563,39,'fuel' from dual union all
select 564,39,'fuel' from dual union all
select 565,39,'fuel' from dual union all
select 566,39,'fuel' from dual union all
select 567,39,'fuel' from dual union all
select 568,100,'fuel' from dual union all
select 569,39,'fuel' from dual union all
select 57,100,'fuel' from dual;
insert into ENGINE_PROPELLANT
select 570,100,'fuel' from dual union all
select 571,100,'fuel' from dual union all
select 572,100,'fuel' from dual union all
select 573,39,'fuel' from dual union all
select 574,100,'fuel' from dual union all
select 575,100,'fuel' from dual union all
select 576,100,'fuel' from dual union all
select 577,100,'fuel' from dual union all
select 577,71,'oxidizer' from dual union all
select 577,72,'oxidizer' from dual union all
select 578,100,'fuel' from dual union all
select 579,58,'oxidizer' from dual union all
select 579,8,'fuel' from dual union all
select 58,12,'fuel' from dual union all
select 58,8,'fuel' from dual union all
select 58,97,'oxidizer' from dual union all
select 580,18,'fuel' from dual union all
select 581,18,'fuel' from dual union all
select 582,18,'fuel' from dual union all
select 583,100,'fuel' from dual union all
select 584,100,'fuel' from dual union all
select 585,90,'fuel' from dual union all
select 586,46,'fuel' from dual union all
select 586,58,'oxidizer' from dual union all
select 587,46,'fuel' from dual union all
select 587,58,'oxidizer' from dual union all
select 588,46,'fuel' from dual union all
select 588,58,'oxidizer' from dual union all
select 589,46,'fuel' from dual union all
select 589,58,'oxidizer' from dual union all
select 59,20,'fuel' from dual union all
select 590,46,'fuel' from dual union all
select 590,58,'oxidizer' from dual union all
select 591,46,'fuel' from dual union all
select 591,58,'oxidizer' from dual union all
select 592,46,'fuel' from dual union all
select 592,58,'oxidizer' from dual union all
select 593,46,'fuel' from dual union all
select 593,58,'oxidizer' from dual union all
select 594,46,'fuel' from dual union all
select 594,58,'oxidizer' from dual union all
select 595,100,'fuel' from dual union all
select 596,100,'fuel' from dual union all
select 597,118,'fuel' from dual union all
select 598,100,'fuel' from dual union all
select 599,100,'fuel' from dual union all
select 6,122,'fuel' from dual union all
select 6,71,'oxidizer' from dual union all
select 60,81,'fuel' from dual union all
select 600,100,'fuel' from dual union all
select 601,100,'fuel' from dual union all
select 602,100,'fuel' from dual union all
select 603,100,'fuel' from dual union all
select 604,100,'fuel' from dual union all
select 605,105,'fuel' from dual union all
select 606,100,'fuel' from dual union all
select 607,32,'fuel' from dual union all
select 608,100,'fuel' from dual union all
select 609,100,'fuel' from dual union all
select 61,93,'fuel' from dual union all
select 610,100,'fuel' from dual union all
select 611,100,'fuel' from dual union all
select 612,100,'fuel' from dual union all
select 613,100,'fuel' from dual union all
select 614,32,'fuel' from dual union all
select 615,32,'fuel' from dual union all
select 616,100,'fuel' from dual union all
select 617,28,'oxidizer' from dual union all
select 617,52,'fuel' from dual union all
select 618,25,'fuel' from dual union all
select 618,58,'oxidizer' from dual union all
select 619,100,'fuel' from dual union all
select 62,1,'fuel' from dual union all
select 62,3,'fuel' from dual union all
select 62,93,'fuel' from dual union all
select 620,100,'fuel' from dual union all
select 621,100,'fuel' from dual union all
select 622,100,'fuel' from dual union all
select 623,100,'fuel' from dual union all
select 624,43,'fuel' from dual union all
select 625,43,'fuel' from dual union all
select 626,43,'fuel' from dual union all
select 627,43,'fuel' from dual union all
select 628,100,'fuel' from dual union all
select 629,100,'fuel' from dual union all
select 63,100,'fuel' from dual union all
select 630,100,'fuel' from dual union all
select 631,100,'fuel' from dual union all
select 632,12,'fuel' from dual union all
select 632,72,'oxidizer' from dual union all
select 633,122,'fuel' from dual union all
select 633,71,'oxidizer' from dual union all
select 634,46,'fuel' from dual union all
select 634,58,'oxidizer' from dual union all
select 635,46,'fuel' from dual union all
select 635,58,'oxidizer' from dual union all
select 636,46,'fuel' from dual union all
select 636,58,'oxidizer' from dual union all
select 637,46,'fuel' from dual union all
select 637,58,'oxidizer' from dual;
insert into ENGINE_PROPELLANT
select 638,46,'fuel' from dual union all
select 638,58,'oxidizer' from dual union all
select 639,46,'fuel' from dual union all
select 639,58,'oxidizer' from dual union all
select 64,100,'fuel' from dual union all
select 640,46,'fuel' from dual union all
select 640,58,'oxidizer' from dual union all
select 641,46,'fuel' from dual union all
select 641,58,'oxidizer' from dual union all
select 642,100,'fuel' from dual union all
select 643,100,'fuel' from dual union all
select 644,100,'fuel' from dual union all
select 645,100,'fuel' from dual union all
select 646,95,'fuel' from dual union all
select 647,100,'fuel' from dual union all
select 648,21,'fuel' from dual union all
select 649,100,'fuel' from dual union all
select 65,100,'fuel' from dual union all
select 650,58,'oxidizer' from dual union all
select 650,8,'fuel' from dual union all
select 651,60,'fuel' from dual union all
select 651,71,'oxidizer' from dual union all
select 652,100,'fuel' from dual union all
select 653,100,'fuel' from dual union all
select 654,100,'fuel' from dual union all
select 655,122,'fuel' from dual union all
select 655,72,'oxidizer' from dual union all
select 656,100,'fuel' from dual union all
select 657,100,'fuel' from dual union all
select 658,60,'fuel' from dual union all
select 658,71,'oxidizer' from dual union all
select 659,52,'fuel' from dual union all
select 659,72,'oxidizer' from dual union all
select 66,100,'fuel' from dual union all
select 660,100,'fuel' from dual union all
select 661,100,'fuel' from dual union all
select 662,100,'fuel' from dual union all
select 663,32,'fuel' from dual union all
select 664,32,'fuel' from dual union all
select 665,24,'fuel' from dual union all
select 666,32,'fuel' from dual union all
select 667,32,'fuel' from dual union all
select 668,32,'fuel' from dual union all
select 669,32,'fuel' from dual union all
select 67,100,'fuel' from dual union all
select 670,32,'fuel' from dual union all
select 671,32,'fuel' from dual union all
select 672,32,'fuel' from dual union all
select 673,32,'fuel' from dual union all
select 674,100,'fuel' from dual union all
select 675,100,'fuel' from dual union all
select 676,100,'fuel' from dual union all
select 677,72,'oxidizer' from dual union all
select 677,9,'fuel' from dual union all
select 678,100,'fuel' from dual union all
select 679,100,'fuel' from dual union all
select 68,100,'fuel' from dual union all
select 680,38,'fuel' from dual union all
select 681,43,'fuel' from dual union all
select 682,40,'fuel' from dual union all
select 682,42,'oxidizer' from dual union all
select 683,40,'fuel' from dual union all
select 683,42,'oxidizer' from dual union all
select 684,32,'fuel' from dual union all
select 685,122,'fuel' from dual union all
select 685,42,'oxidizer' from dual union all
select 686,122,'fuel' from dual union all
select 686,42,'oxidizer' from dual union all
select 687,17,'fuel' from dual union all
select 688,132,'fuel' from dual union all
select 688,23,'oxidizer' from dual union all
select 689,100,'fuel' from dual union all
select 69,100,'fuel' from dual union all
select 690,100,'fuel' from dual union all
select 691,32,'fuel' from dual union all
select 692,32,'fuel' from dual union all
select 693,32,'fuel' from dual union all
select 694,100,'fuel' from dual union all
select 695,100,'fuel' from dual union all
select 696,58,'oxidizer' from dual union all
select 696,65,'fuel' from dual union all
select 697,100,'fuel' from dual union all
select 698,100,'fuel' from dual union all
select 699,32,'fuel' from dual union all
select 7,122,'fuel' from dual union all
select 7,71,'oxidizer' from dual union all
select 70,100,'fuel' from dual union all
select 700,32,'fuel' from dual union all
select 701,91,'fuel' from dual union all
select 702,122,'fuel' from dual union all
select 702,71,'oxidizer' from dual union all
select 703,120,'fuel' from dual union all
select 703,133,'fuel' from dual union all
select 703,97,'oxidizer' from dual union all
select 704,100,'fuel' from dual union all
select 705,100,'fuel' from dual union all
select 706,100,'fuel' from dual union all
select 707,100,'fuel' from dual union all
select 708,100,'fuel' from dual union all
select 709,100,'fuel' from dual;
insert into ENGINE_PROPELLANT
select 71,107,'fuel' from dual union all
select 710,60,'fuel' from dual union all
select 710,63,'oxidizer' from dual union all
select 711,60,'fuel' from dual union all
select 711,71,'oxidizer' from dual union all
select 712,122,'fuel' from dual union all
select 712,71,'oxidizer' from dual union all
select 713,122,'fuel' from dual union all
select 713,71,'oxidizer' from dual union all
select 714,122,'fuel' from dual union all
select 714,71,'oxidizer' from dual union all
select 715,122,'fuel' from dual union all
select 715,71,'oxidizer' from dual union all
select 716,100,'fuel' from dual union all
select 717,100,'fuel' from dual union all
select 718,100,'fuel' from dual union all
select 719,100,'fuel' from dual union all
select 72,93,'fuel' from dual union all
select 720,100,'fuel' from dual union all
select 721,46,'fuel' from dual union all
select 721,58,'oxidizer' from dual union all
select 722,46,'fuel' from dual union all
select 722,58,'oxidizer' from dual union all
select 723,46,'fuel' from dual union all
select 723,58,'oxidizer' from dual union all
select 724,57,'fuel' from dual union all
select 724,58,'oxidizer' from dual union all
select 725,46,'fuel' from dual union all
select 725,58,'oxidizer' from dual union all
select 726,46,'fuel' from dual union all
select 726,58,'oxidizer' from dual union all
select 727,122,'fuel' from dual union all
select 727,71,'oxidizer' from dual union all
select 728,122,'fuel' from dual union all
select 728,71,'oxidizer' from dual union all
select 729,122,'fuel' from dual union all
select 729,71,'oxidizer' from dual union all
select 73,100,'fuel' from dual union all
select 730,122,'fuel' from dual union all
select 730,71,'oxidizer' from dual union all
select 731,122,'fuel' from dual union all
select 731,71,'oxidizer' from dual union all
select 732,122,'fuel' from dual union all
select 732,71,'oxidizer' from dual union all
select 733,122,'fuel' from dual union all
select 733,71,'oxidizer' from dual union all
select 734,122,'fuel' from dual union all
select 734,71,'oxidizer' from dual union all
select 735,122,'fuel' from dual union all
select 735,71,'oxidizer' from dual union all
select 736,122,'fuel' from dual union all
select 736,71,'oxidizer' from dual union all
select 737,122,'fuel' from dual union all
select 737,71,'oxidizer' from dual union all
select 738,122,'fuel' from dual union all
select 738,71,'oxidizer' from dual union all
select 739,126,'fuel' from dual union all
select 739,71,'oxidizer' from dual union all
select 74,100,'fuel' from dual union all
select 740,122,'fuel' from dual union all
select 740,71,'oxidizer' from dual union all
select 741,122,'fuel' from dual union all
select 741,71,'oxidizer' from dual union all
select 742,122,'fuel' from dual union all
select 742,71,'oxidizer' from dual union all
select 743,122,'fuel' from dual union all
select 743,71,'oxidizer' from dual union all
select 744,122,'fuel' from dual union all
select 744,71,'oxidizer' from dual union all
select 745,122,'fuel' from dual union all
select 745,71,'oxidizer' from dual union all
select 746,122,'fuel' from dual union all
select 746,71,'oxidizer' from dual union all
select 747,122,'fuel' from dual union all
select 747,71,'oxidizer' from dual union all
select 748,122,'fuel' from dual union all
select 748,71,'oxidizer' from dual union all
select 749,122,'fuel' from dual union all
select 749,71,'oxidizer' from dual union all
select 75,100,'fuel' from dual union all
select 750,58,'oxidizer' from dual union all
select 750,8,'fuel' from dual union all
select 751,58,'oxidizer' from dual union all
select 751,8,'fuel' from dual union all
select 752,58,'oxidizer' from dual union all
select 752,8,'fuel' from dual union all
select 753,58,'oxidizer' from dual union all
select 753,8,'fuel' from dual union all
select 754,51,'fuel' from dual union all
select 754,58,'oxidizer' from dual union all
select 755,51,'fuel' from dual union all
select 755,58,'oxidizer' from dual union all
select 756,51,'fuel' from dual union all
select 756,58,'oxidizer' from dual union all
select 757,51,'fuel' from dual union all
select 757,58,'oxidizer' from dual union all
select 758,50,'fuel' from dual union all
select 758,58,'oxidizer' from dual union all
select 759,46,'fuel' from dual union all
select 759,58,'oxidizer' from dual;
insert into ENGINE_PROPELLANT
select 76,100,'fuel' from dual union all
select 760,46,'fuel' from dual union all
select 760,58,'oxidizer' from dual union all
select 761,122,'fuel' from dual union all
select 761,58,'oxidizer' from dual union all
select 762,122,'fuel' from dual union all
select 762,58,'oxidizer' from dual union all
select 763,122,'fuel' from dual union all
select 763,58,'oxidizer' from dual union all
select 764,122,'fuel' from dual union all
select 764,58,'oxidizer' from dual union all
select 765,122,'fuel' from dual union all
select 765,58,'oxidizer' from dual union all
select 766,46,'fuel' from dual union all
select 766,58,'oxidizer' from dual union all
select 767,46,'fuel' from dual union all
select 767,58,'oxidizer' from dual union all
select 768,46,'fuel' from dual union all
select 768,58,'oxidizer' from dual union all
select 769,46,'fuel' from dual union all
select 769,58,'oxidizer' from dual union all
select 77,100,'fuel' from dual union all
select 770,54,'fuel' from dual union all
select 770,58,'oxidizer' from dual union all
select 771,54,'fuel' from dual union all
select 771,58,'oxidizer' from dual union all
select 772,46,'fuel' from dual union all
select 772,58,'oxidizer' from dual union all
select 773,52,'fuel' from dual union all
select 773,72,'oxidizer' from dual union all
select 774,52,'fuel' from dual union all
select 774,72,'oxidizer' from dual union all
select 775,122,'fuel' from dual union all
select 775,72,'oxidizer' from dual union all
select 776,122,'fuel' from dual union all
select 776,75,'oxidizer' from dual union all
select 777,122,'fuel' from dual union all
select 777,72,'oxidizer' from dual union all
select 778,122,'fuel' from dual union all
select 778,72,'oxidizer' from dual union all
select 779,122,'fuel' from dual union all
select 779,72,'oxidizer' from dual union all
select 78,122,'fuel' from dual union all
select 78,71,'oxidizer' from dual union all
select 780,122,'fuel' from dual union all
select 780,72,'oxidizer' from dual union all
select 781,122,'fuel' from dual union all
select 781,71,'oxidizer' from dual union all
select 782,122,'fuel' from dual union all
select 782,71,'oxidizer' from dual union all
select 783,122,'fuel' from dual union all
select 783,71,'oxidizer' from dual union all
select 784,122,'fuel' from dual union all
select 784,71,'oxidizer' from dual union all
select 785,122,'fuel' from dual union all
select 785,71,'oxidizer' from dual union all
select 786,122,'fuel' from dual union all
select 786,71,'oxidizer' from dual union all
select 787,122,'fuel' from dual union all
select 787,71,'oxidizer' from dual union all
select 788,122,'fuel' from dual union all
select 788,71,'oxidizer' from dual union all
select 789,122,'fuel' from dual union all
select 789,71,'oxidizer' from dual union all
select 79,100,'fuel' from dual union all
select 790,122,'fuel' from dual union all
select 790,71,'oxidizer' from dual union all
select 791,122,'fuel' from dual union all
select 791,71,'oxidizer' from dual union all
select 792,46,'fuel' from dual union all
select 792,58,'oxidizer' from dual union all
select 793,122,'fuel' from dual union all
select 793,71,'oxidizer' from dual union all
select 794,122,'fuel' from dual union all
select 794,72,'oxidizer' from dual union all
select 795,122,'fuel' from dual union all
select 795,72,'oxidizer' from dual union all
select 796,122,'fuel' from dual union all
select 796,71,'oxidizer' from dual union all
select 797,122,'fuel' from dual union all
select 797,71,'oxidizer' from dual union all
select 798,122,'fuel' from dual union all
select 798,71,'oxidizer' from dual union all
select 799,122,'fuel' from dual union all
select 799,71,'oxidizer' from dual union all
select 8,122,'fuel' from dual union all
select 8,73,'oxidizer' from dual union all
select 80,100,'fuel' from dual union all
select 800,122,'fuel' from dual union all
select 800,71,'oxidizer' from dual union all
select 801,122,'fuel' from dual union all
select 801,71,'oxidizer' from dual union all
select 802,122,'fuel' from dual union all
select 802,71,'oxidizer' from dual union all
select 803,122,'fuel' from dual union all
select 803,71,'oxidizer' from dual union all
select 804,122,'fuel' from dual union all
select 804,71,'oxidizer' from dual union all
select 805,122,'fuel' from dual union all
select 805,71,'oxidizer' from dual;
insert into ENGINE_PROPELLANT
select 806,122,'fuel' from dual union all
select 806,71,'oxidizer' from dual union all
select 807,100,'fuel' from dual union all
select 808,122,'fuel' from dual union all
select 808,71,'oxidizer' from dual union all
select 809,122,'fuel' from dual union all
select 809,71,'oxidizer' from dual union all
select 81,100,'fuel' from dual union all
select 810,122,'fuel' from dual union all
select 810,71,'oxidizer' from dual union all
select 811,122,'fuel' from dual union all
select 811,71,'oxidizer' from dual union all
select 812,122,'fuel' from dual union all
select 812,71,'oxidizer' from dual union all
select 813,122,'fuel' from dual union all
select 813,71,'oxidizer' from dual union all
select 814,122,'fuel' from dual union all
select 814,71,'oxidizer' from dual union all
select 815,39,'fuel' from dual union all
select 816,100,'fuel' from dual union all
select 817,100,'fuel' from dual union all
select 818,100,'fuel' from dual union all
select 819,100,'fuel' from dual union all
select 82,100,'fuel' from dual union all
select 820,100,'fuel' from dual union all
select 821,100,'fuel' from dual union all
select 822,100,'fuel' from dual union all
select 823,100,'fuel' from dual union all
select 824,100,'fuel' from dual union all
select 825,100,'fuel' from dual union all
select 826,57,'fuel' from dual union all
select 826,58,'oxidizer' from dual union all
select 827,57,'fuel' from dual union all
select 827,58,'oxidizer' from dual union all
select 828,57,'fuel' from dual union all
select 828,58,'oxidizer' from dual union all
select 829,57,'fuel' from dual union all
select 829,58,'oxidizer' from dual union all
select 83,63,'fuel' from dual union all
select 83,71,'oxidizer' from dual union all
select 830,57,'fuel' from dual union all
select 830,58,'oxidizer' from dual union all
select 831,57,'fuel' from dual union all
select 831,58,'oxidizer' from dual union all
select 832,57,'fuel' from dual union all
select 832,58,'oxidizer' from dual union all
select 833,57,'fuel' from dual union all
select 833,58,'oxidizer' from dual union all
select 834,57,'fuel' from dual union all
select 834,58,'oxidizer' from dual union all
select 835,57,'fuel' from dual union all
select 835,58,'oxidizer' from dual union all
select 836,57,'fuel' from dual union all
select 836,58,'oxidizer' from dual union all
select 837,57,'fuel' from dual union all
select 837,58,'oxidizer' from dual union all
select 838,32,'fuel' from dual union all
select 838,70,'oxidizer' from dual union all
select 839,109,'fuel' from dual union all
select 84,100,'fuel' from dual union all
select 840,102,'fuel' from dual union all
select 840,4,'fuel' from dual union all
select 841,32,'fuel' from dual union all
select 841,70,'oxidizer' from dual union all
select 842,100,'fuel' from dual union all
select 843,46,'fuel' from dual union all
select 843,58,'oxidizer' from dual union all
select 844,46,'fuel' from dual union all
select 844,58,'oxidizer' from dual union all
select 845,60,'fuel' from dual union all
select 845,71,'oxidizer' from dual union all
select 846,2,'fuel' from dual union all
select 846,71,'oxidizer' from dual union all
select 847,60,'fuel' from dual union all
select 847,71,'oxidizer' from dual union all
select 848,60,'fuel' from dual union all
select 848,71,'oxidizer' from dual union all
select 849,54,'fuel' from dual union all
select 849,58,'oxidizer' from dual union all
select 85,122,'fuel' from dual union all
select 85,71,'oxidizer' from dual union all
select 850,54,'fuel' from dual union all
select 850,58,'oxidizer' from dual union all
select 851,60,'fuel' from dual union all
select 851,71,'oxidizer' from dual union all
select 852,60,'fuel' from dual union all
select 852,71,'oxidizer' from dual union all
select 853,57,'fuel' from dual union all
select 853,58,'oxidizer' from dual union all
select 854,57,'fuel' from dual union all
select 854,58,'oxidizer' from dual union all
select 855,100,'fuel' from dual union all
select 856,100,'fuel' from dual union all
select 857,100,'fuel' from dual union all
select 858,100,'fuel' from dual union all
select 859,100,'fuel' from dual union all
select 86,100,'fuel' from dual union all
select 860,44,'fuel' from dual union all
select 861,116,'fuel' from dual union all
select 862,32,'fuel' from dual;
insert into ENGINE_PROPELLANT
select 863,100,'fuel' from dual union all
select 864,100,'fuel' from dual union all
select 865,46,'fuel' from dual union all
select 865,58,'oxidizer' from dual union all
select 866,100,'fuel' from dual union all
select 867,100,'fuel' from dual union all
select 868,100,'fuel' from dual union all
select 869,100,'fuel' from dual union all
select 87,122,'fuel' from dual union all
select 87,71,'oxidizer' from dual union all
select 870,100,'fuel' from dual union all
select 871,100,'fuel' from dual union all
select 872,1,'fuel' from dual union all
select 873,100,'fuel' from dual union all
select 874,40,'fuel' from dual union all
select 874,58,'oxidizer' from dual union all
select 875,43,'fuel' from dual union all
select 876,100,'fuel' from dual union all
select 877,100,'fuel' from dual union all
select 878,46,'fuel' from dual union all
select 878,58,'oxidizer' from dual union all
select 879,46,'fuel' from dual union all
select 879,58,'oxidizer' from dual union all
select 88,100,'fuel' from dual union all
select 880,127,'fuel' from dual union all
select 881,100,'fuel' from dual union all
select 882,81,'fuel' from dual union all
select 883,100,'fuel' from dual union all
select 884,100,'fuel' from dual union all
select 885,100,'fuel' from dual union all
select 886,100,'fuel' from dual union all
select 887,81,'fuel' from dual union all
select 888,49,'fuel' from dual union all
select 888,58,'oxidizer' from dual union all
select 889,1,'fuel' from dual union all
select 889,3,'fuel' from dual union all
select 889,85,'fuel' from dual union all
select 89,100,'fuel' from dual union all
select 890,60,'fuel' from dual union all
select 890,62,'oxidizer' from dual union all
select 890,71,'oxidizer' from dual union all
select 891,60,'fuel' from dual union all
select 891,62,'oxidizer' from dual union all
select 891,71,'oxidizer' from dual union all
select 892,100,'fuel' from dual union all
select 893,100,'fuel' from dual union all
select 894,100,'fuel' from dual union all
select 895,32,'fuel' from dual union all
select 896,100,'fuel' from dual union all
select 897,100,'fuel' from dual union all
select 898,100,'fuel' from dual union all
select 899,72,'oxidizer' from dual union all
select 899,9,'fuel' from dual union all
select 9,122,'fuel' from dual union all
select 9,71,'oxidizer' from dual union all
select 90,100,'fuel' from dual union all
select 900,46,'fuel' from dual union all
select 900,58,'oxidizer' from dual union all
select 901,32,'fuel' from dual union all
select 902,32,'fuel' from dual union all
select 903,55,'fuel' from dual union all
select 903,76,'oxidizer' from dual union all
select 904,10,'fuel' from dual union all
select 904,76,'oxidizer' from dual union all
select 905,10,'fuel' from dual union all
select 905,72,'oxidizer' from dual union all
select 906,52,'fuel' from dual union all
select 906,72,'oxidizer' from dual union all
select 907,52,'fuel' from dual union all
select 907,97,'oxidizer' from dual union all
select 908,56,'fuel' from dual union all
select 908,98,'oxidizer' from dual union all
select 909,10,'fuel' from dual union all
select 909,76,'oxidizer' from dual union all
select 91,100,'fuel' from dual union all
select 910,72,'oxidizer' from dual union all
select 910,9,'fuel' from dual union all
select 911,72,'oxidizer' from dual union all
select 911,9,'fuel' from dual union all
select 912,10,'fuel' from dual union all
select 912,74,'oxidizer' from dual union all
select 913,32,'fuel' from dual union all
select 914,100,'fuel' from dual union all
select 915,72,'oxidizer' from dual union all
select 915,9,'fuel' from dual union all
select 916,122,'fuel' from dual union all
select 916,71,'oxidizer' from dual union all
select 917,52,'fuel' from dual union all
select 917,72,'oxidizer' from dual union all
select 918,72,'oxidizer' from dual union all
select 918,9,'fuel' from dual union all
select 919,72,'oxidizer' from dual union all
select 919,9,'fuel' from dual union all
select 92,100,'fuel' from dual union all
select 920,72,'oxidizer' from dual union all
select 920,9,'fuel' from dual union all
select 921,72,'oxidizer' from dual union all
select 921,9,'fuel' from dual union all
select 922,72,'oxidizer' from dual union all
select 922,9,'fuel' from dual;
insert into ENGINE_PROPELLANT
select 923,122,'fuel' from dual union all
select 923,72,'oxidizer' from dual union all
select 924,10,'fuel' from dual union all
select 924,76,'oxidizer' from dual union all
select 925,122,'fuel' from dual union all
select 925,71,'oxidizer' from dual union all
select 926,122,'fuel' from dual union all
select 926,71,'oxidizer' from dual union all
select 927,122,'fuel' from dual union all
select 927,71,'oxidizer' from dual union all
select 928,122,'fuel' from dual union all
select 928,71,'oxidizer' from dual union all
select 929,122,'fuel' from dual union all
select 929,71,'oxidizer' from dual union all
select 93,122,'fuel' from dual union all
select 93,71,'oxidizer' from dual union all
select 930,100,'fuel' from dual union all
select 931,100,'fuel' from dual union all
select 932,46,'fuel' from dual union all
select 932,58,'oxidizer' from dual union all
select 933,25,'fuel' from dual union all
select 933,58,'oxidizer' from dual union all
select 934,93,'fuel' from dual union all
select 935,17,'fuel' from dual union all
select 936,60,'fuel' from dual union all
select 936,69,'fuel' from dual union all
select 936,71,'oxidizer' from dual union all
select 937,15,'fuel' from dual union all
select 938,15,'fuel' from dual union all
select 939,15,'fuel' from dual union all
select 94,51,'fuel' from dual union all
select 94,58,'oxidizer' from dual union all
select 940,43,'fuel' from dual union all
select 941,43,'fuel' from dual union all
select 942,43,'fuel' from dual union all
select 943,43,'fuel' from dual union all
select 944,100,'fuel' from dual union all
select 945,100,'fuel' from dual union all
select 946,100,'fuel' from dual union all
select 947,89,'fuel' from dual union all
select 948,100,'fuel' from dual union all
select 949,89,'fuel' from dual union all
select 95,51,'fuel' from dual union all
select 95,58,'oxidizer' from dual union all
select 950,89,'fuel' from dual union all
select 951,100,'fuel' from dual union all
select 952,100,'fuel' from dual union all
select 953,100,'fuel' from dual union all
select 954,100,'fuel' from dual union all
select 955,100,'fuel' from dual union all
select 956,14,'fuel' from dual union all
select 956,32,'fuel' from dual union all
select 956,79,'fuel' from dual union all
select 957,14,'fuel' from dual union all
select 957,79,'fuel' from dual union all
select 957,86,'fuel' from dual union all
select 958,14,'fuel' from dual union all
select 958,79,'fuel' from dual union all
select 958,86,'fuel' from dual union all
select 959,60,'fuel' from dual union all
select 959,71,'oxidizer' from dual union all
select 96,51,'fuel' from dual union all
select 96,58,'oxidizer' from dual union all
select 960,100,'fuel' from dual union all
select 961,100,'fuel' from dual union all
select 962,100,'fuel' from dual union all
select 963,30,'fuel' from dual union all
select 964,100,'fuel' from dual union all
select 965,100,'fuel' from dual union all
select 966,100,'fuel' from dual union all
select 967,100,'fuel' from dual union all
select 968,100,'fuel' from dual union all
select 969,100,'fuel' from dual union all
select 97,51,'fuel' from dual union all
select 97,58,'oxidizer' from dual union all
select 970,32,'fuel' from dual union all
select 971,78,'fuel' from dual union all
select 972,17,'fuel' from dual union all
select 973,17,'fuel' from dual union all
select 974,100,'fuel' from dual union all
select 975,17,'fuel' from dual union all
select 976,17,'fuel' from dual union all
select 977,84,'fuel' from dual union all
select 978,32,'fuel' from dual union all
select 979,100,'fuel' from dual union all
select 98,51,'fuel' from dual union all
select 98,58,'oxidizer' from dual union all
select 980,100,'fuel' from dual union all
select 981,100,'fuel' from dual union all
select 982,100,'fuel' from dual union all
select 983,57,'fuel' from dual union all
select 983,58,'oxidizer' from dual union all
select 984,100,'fuel' from dual union all
select 985,100,'fuel' from dual union all
select 986,25,'fuel' from dual union all
select 986,58,'oxidizer' from dual union all
select 987,25,'fuel' from dual union all
select 987,58,'oxidizer' from dual union all
select 988,100,'fuel' from dual union all
select 989,100,'fuel' from dual;
insert into ENGINE_PROPELLANT
select 99,51,'fuel' from dual union all
select 99,58,'oxidizer' from dual union all
select 990,100,'fuel' from dual union all
select 991,100,'fuel' from dual union all
select 992,100,'fuel' from dual union all
select 993,100,'fuel' from dual union all
select 994,100,'fuel' from dual union all
select 995,100,'fuel' from dual union all
select 996,100,'fuel' from dual union all
select 997,100,'fuel' from dual union all
select 998,100,'fuel' from dual union all
select 999,100,'fuel' from dual;


--------------------------------------------------------------------------------
-- ENGINE_MANUFACTURER
--------------------------------------------------------------------------------

exec dbms_output.put_line('Installing ENGINE_MANUFACTURER...');

CREATE TABLE "ENGINE_MANUFACTURER" 
 (	"ENGINE_ID" NUMBER, 
	"MANUFACTURER_ORG_CODE" VARCHAR2(4000), 
	 CONSTRAINT "ENGINE_MANUFACTURER_PK" PRIMARY KEY ("ENGINE_ID", "MANUFACTURER_ORG_CODE")
USING INDEX  ENABLE, 
	 CONSTRAINT "ENGINE_MANUFACTURER_ENGINE_FK" FOREIGN KEY ("ENGINE_ID")
	  REFERENCES "ENGINE" ("ENGINE_ID") ENABLE, 
	 CONSTRAINT "ENGINE_MANUFACTURER_MANUF_FK" FOREIGN KEY ("MANUFACTURER_ORG_CODE")
	  REFERENCES "ORGANIZATION" ("ORG_CODE") ENABLE
 ) ;


CREATE INDEX "ENGINE_MANUFACTURER_IDX1" ON "ENGINE_MANUFACTURER" ("MANUFACTURER_ORG_CODE") 
;
insert into ENGINE_MANUFACTURER
select 10,'KHMM' from dual union all
select 100,'BAL' from dual union all
select 1000,'GCR' from dual union all
select 1001,'AJSP' from dual union all
select 1002,'AJSP' from dual union all
select 1003,'RORD' from dual union all
select 1004,'IAE' from dual union all
select 1005,'IAE' from dual union all
select 1006,'UPAE' from dual union all
select 1007,'DEHAVE' from dual union all
select 1008,'DEHAVE' from dual union all
select 1009,'SDC' from dual union all
select 101,'BAL' from dual union all
select 1010,'THKE' from dual union all
select 1011,'THKE' from dual union all
select 1012,'THKE' from dual union all
select 1013,'THKE' from dual union all
select 1014,'THKE' from dual union all
select 1015,'THKE' from dual union all
select 1016,'THKE' from dual union all
select 1017,'THKE' from dual union all
select 1018,'THKE' from dual union all
select 1019,'THKE' from dual union all
select 102,'VOTK' from dual union all
select 1020,'THKE' from dual union all
select 1021,'THKE' from dual union all
select 1022,'THKE' from dual union all
select 1023,'THKE' from dual union all
select 1024,'THKE' from dual union all
select 1025,'THKE' from dual union all
select 1026,'THKE' from dual union all
select 1027,'THKE' from dual union all
select 1028,'THKE' from dual union all
select 1029,'THKE' from dual union all
select 103,'VOTK' from dual union all
select 1030,'THKE' from dual union all
select 1031,'THKE' from dual union all
select 1032,'THKE' from dual union all
select 1033,'THKE' from dual union all
select 1034,'THKE' from dual union all
select 1035,'THKE' from dual union all
select 1036,'THKE' from dual union all
select 1037,'THKE' from dual union all
select 1038,'THKE' from dual union all
select 1039,'THKE' from dual union all
select 104,'VOTK' from dual union all
select 1040,'THKE' from dual union all
select 1041,'THKE' from dual union all
select 1042,'THKE' from dual union all
select 1043,'THKE' from dual union all
select 1044,'THKE' from dual union all
select 1045,'THKE' from dual union all
select 1046,'THKE' from dual union all
select 1047,'THKE' from dual union all
select 1048,'THKE' from dual union all
select 1049,'THKE' from dual union all
select 105,'VOTK' from dual union all
select 1050,'THKE' from dual union all
select 1051,'THKE' from dual union all
select 1052,'THKE' from dual union all
select 1053,'THKU' from dual union all
select 1054,'THKE' from dual union all
select 1055,'MITT' from dual union all
select 1056,'ASID' from dual union all
select 1057,'RPEW' from dual union all
select 1058,'RPEW' from dual union all
select 1059,'SDP' from dual union all
select 106,'VOTK' from dual union all
select 1060,'SPXTX' from dual union all
select 1061,'SPXTX' from dual union all
select 1062,'SDC' from dual union all
select 1063,'LPC' from dual union all
select 1064,'JPL' from dual union all
select 1065,'THKE' from dual union all
select 1066,'THKX' from dual union all
select 1067,'THKE' from dual union all
select 1068,'SIMED' from dual union all
select 1069,'SIMED' from dual union all
select 107,'VOTK' from dual union all
select 1070,'HERCR' from dual union all
select 1070,'MARTB' from dual union all
select 1071,'RMD' from dual union all
select 1072,'KPA' from dual union all
select 1073,'RMD' from dual union all
select 1074,'RMD' from dual union all
select 1075,'KPA' from dual union all
select 1076,'THKE' from dual union all
select 1077,'THKE' from dual union all
select 1078,'THKE' from dual union all
select 1079,'THKE' from dual union all
select 108,'KAZSOY' from dual union all
select 1080,'THKE' from dual union all
select 1081,'THKE' from dual union all
select 1082,'THKE' from dual union all
select 1083,'AJSP' from dual union all
select 1084,'UTC' from dual union all
select 1085,'SISP' from dual union all
select 1086,'NASDA' from dual union all
select 1087,'NASDA' from dual union all
select 1088,'TRW' from dual;
insert into ENGINE_MANUFACTURER
select 1089,'TRW' from dual union all
select 109,'KLIMOV' from dual union all
select 1090,'TRW' from dual union all
select 1091,'TRW' from dual union all
select 1092,'NASDA' from dual union all
select 1093,'NASDA' from dual union all
select 1094,'NASDA' from dual union all
select 1095,'NASDA' from dual union all
select 1096,'THKU' from dual union all
select 1097,'ATKP' from dual union all
select 1098,'THKU' from dual union all
select 1099,'THKU' from dual union all
select 11,'KHMM' from dual union all
select 110,'HVP' from dual union all
select 1100,'THKU' from dual union all
select 1101,'THKU' from dual union all
select 1102,'THKU' from dual union all
select 1103,'THKU' from dual union all
select 1104,'THKX' from dual union all
select 1105,'THKX' from dual union all
select 1106,'THKX' from dual union all
select 1107,'THKXM' from dual union all
select 1108,'THKXM' from dual union all
select 1109,'THKX' from dual union all
select 111,'AJSP' from dual union all
select 1110,'THKXM' from dual union all
select 1111,'THKX' from dual union all
select 1112,'THKX' from dual union all
select 1113,'THKX' from dual union all
select 1114,'THKX' from dual union all
select 1115,'THKXM' from dual union all
select 1116,'THKX' from dual union all
select 1117,'THKX' from dual union all
select 1118,'THKX' from dual union all
select 1119,'THKX' from dual union all
select 1119,'THKXM' from dual union all
select 112,'AJSP' from dual union all
select 1120,'THKX' from dual union all
select 1121,'THKXM' from dual union all
select 1122,'THKX' from dual union all
select 1123,'THKX' from dual union all
select 1124,'THKXM' from dual union all
select 1125,'THKX' from dual union all
select 1126,'THKX' from dual union all
select 1127,'THKX' from dual union all
select 1128,'THKX' from dual union all
select 1129,'THKX' from dual union all
select 113,'AJSP' from dual union all
select 1130,'ABL' from dual union all
select 1131,'UNK' from dual union all
select 1132,'UNK' from dual union all
select 1133,'ABL' from dual union all
select 1134,'ATEF' from dual union all
select 1135,'SZRTC' from dual union all
select 1136,'SZRTC' from dual union all
select 1137,'USN' from dual union all
select 1138,'SEP' from dual union all
select 1139,'NORTHC' from dual union all
select 114,'AJSP' from dual union all
select 1140,'UTC' from dual union all
select 1141,'UTC' from dual union all
select 1142,'UTC' from dual union all
select 1143,'UTC' from dual union all
select 1144,'NISST' from dual union all
select 1145,'IHI' from dual union all
select 1145,'NASDA' from dual union all
select 1146,'KPA' from dual union all
select 1147,'MDAC' from dual union all
select 1148,'INTA' from dual union all
select 1149,'ISAEV' from dual union all
select 115,'ABL' from dual union all
select 1150,'KLIMOV' from dual union all
select 1151,'LRBA' from dual union all
select 1152,'SDP' from dual union all
select 1153,'WRE' from dual union all
select 1154,'SUD' from dual union all
select 1155,'LRBA' from dual union all
select 1156,'LRBA' from dual union all
select 1157,'LRBA' from dual union all
select 1158,'LRBA' from dual union all
select 1159,'LRBA' from dual union all
select 116,'AJSP' from dual union all
select 1160,'LRBA' from dual union all
select 1161,'LRBA' from dual union all
select 1162,'LRBA' from dual union all
select 1163,'LRBA' from dual union all
select 1164,'LRBA' from dual union all
select 1165,'LRBA' from dual union all
select 1166,'VSSC' from dual union all
select 1167,'VSSC' from dual union all
select 1168,'SEP' from dual union all
select 1169,'SEP' from dual union all
select 117,'AJGC' from dual union all
select 1170,'SEP' from dual union all
select 1171,'SEP' from dual union all
select 1172,'SEP' from dual union all
select 1173,'SDCO' from dual union all
select 1174,'OSCC' from dual union all
select 1175,'PROG' from dual union all
select 1176,'SEP' from dual;
insert into ENGINE_MANUFACTURER
select 1177,'SEP' from dual union all
select 1178,'AJGC' from dual union all
select 1179,'HVP' from dual union all
select 118,'AJGC' from dual union all
select 1180,'RPEW' from dual union all
select 1181,'ABL' from dual union all
select 1182,'ABL' from dual union all
select 1183,'ABL' from dual union all
select 1184,'ABL' from dual union all
select 1185,'ABL' from dual union all
select 1186,'ABL' from dual union all
select 1187,'ABL' from dual union all
select 1188,'ABL' from dual union all
select 1189,'ABL' from dual union all
select 119,'AJGC' from dual union all
select 1190,'ABL' from dual union all
select 1191,'ABL' from dual union all
select 1192,'ABL' from dual union all
select 1193,'ABL' from dual union all
select 1194,'ABL' from dual union all
select 1195,'ABL' from dual union all
select 1196,'ABL' from dual union all
select 1197,'ABL' from dual union all
select 1198,'ABL' from dual union all
select 1199,'ABL' from dual union all
select 12,'KBKHA' from dual union all
select 120,'AJGC' from dual union all
select 1200,'ABL' from dual union all
select 1201,'ABL' from dual union all
select 1202,'HERC' from dual union all
select 1203,'HERC' from dual union all
select 1204,'GEMTS' from dual union all
select 1205,'AJGC' from dual union all
select 1206,'AJGC' from dual union all
select 1207,'RMI' from dual union all
select 1208,'NAADR' from dual union all
select 1209,'NAADR' from dual union all
select 121,'AJGC' from dual union all
select 1210,'RMD' from dual union all
select 1211,'RMD' from dual union all
select 1212,'THKX' from dual union all
select 1213,'THKX' from dual union all
select 1214,'THKXM' from dual union all
select 1215,'THKX' from dual union all
select 1216,'THKX' from dual union all
select 1217,'THKX' from dual union all
select 1218,'THKX' from dual union all
select 1219,'THKX' from dual union all
select 122,'AJGC' from dual union all
select 1220,'THKX' from dual union all
select 1221,'HERCR' from dual union all
select 1222,'THKX' from dual union all
select 1223,'THKU' from dual union all
select 1223,'THKX' from dual union all
select 1224,'ABL' from dual union all
select 1225,'XCOR' from dual union all
select 1226,'MASTEN' from dual union all
select 1227,'CALT' from dual union all
select 1228,'TIANJ' from dual union all
select 1229,'TIANJ' from dual union all
select 123,'AJGC' from dual union all
select 1230,'SLREC' from dual union all
select 1231,'SLREC' from dual union all
select 1232,'SLREC' from dual union all
select 1233,'SLREC' from dual union all
select 1234,'SLREC' from dual union all
select 1235,'SLREC' from dual union all
select 1236,'SLREC' from dual union all
select 1237,'SLREC' from dual union all
select 1238,'SLREC' from dual union all
select 1239,'CALT' from dual union all
select 124,'AJGC' from dual union all
select 1240,'CALT' from dual union all
select 1241,'SLREC' from dual union all
select 1242,'CALT' from dual union all
select 1243,'CALT' from dual union all
select 1244,'CALT' from dual union all
select 1245,'SLREC' from dual union all
select 1246,'SLREC' from dual union all
select 1247,'SLREC' from dual union all
select 1248,'CALT' from dual union all
select 1249,'TIANJ' from dual union all
select 125,'AJGC' from dual union all
select 1250,'THKE' from dual union all
select 1251,'NUDTC' from dual union all
select 1252,'FIATA' from dual union all
select 1253,'AVIO' from dual union all
select 1254,'AVIO' from dual union all
select 1255,'AVIO' from dual union all
select 1256,'CONTR' from dual union all
select 1257,'GCR' from dual union all
select 126,'AJGC' from dual union all
select 127,'AJGC' from dual union all
select 128,'AJGC' from dual union all
select 129,'AJGC' from dual union all
select 13,'EMASH' from dual union all
select 130,'AJGC' from dual union all
select 131,'AJGC' from dual union all
select 132,'AJGC' from dual union all
select 133,'AJGC' from dual;
insert into ENGINE_MANUFACTURER
select 134,'AJGC' from dual union all
select 135,'AJGC' from dual union all
select 136,'AJGC' from dual union all
select 137,'AJGC' from dual union all
select 138,'AJGC' from dual union all
select 139,'AJGC' from dual union all
select 14,'EMASH' from dual union all
select 140,'AJ' from dual union all
select 140,'NKE' from dual union all
select 141,'AJ' from dual union all
select 142,'AJGC' from dual union all
select 143,'RDYNE' from dual union all
select 144,'VSSC' from dual union all
select 145,'VSSC' from dual union all
select 146,'VSSC' from dual union all
select 147,'VSSC' from dual union all
select 148,'VSSC' from dual union all
select 149,'AJCA' from dual union all
select 15,'KBKHA' from dual union all
select 150,'IAI' from dual union all
select 151,'IAI' from dual union all
select 152,'RAFL' from dual union all
select 153,'PKNDC' from dual union all
select 154,'IQ' from dual union all
select 155,'IQ' from dual union all
select 156,'UNK' from dual union all
select 157,'AJGC' from dual union all
select 158,'AJSP' from dual union all
select 159,'AJ' from dual union all
select 16,'EMASH' from dual union all
select 160,'ASTO' from dual union all
select 161,'CFTHB' from dual union all
select 162,'DRDO' from dual union all
select 163,'DRDO' from dual union all
select 164,'DRDO' from dual union all
select 165,'DRDO' from dual union all
select 166,'DRDO' from dual union all
select 167,'AJ' from dual union all
select 168,'AJ' from dual union all
select 169,'AJGC' from dual union all
select 17,'VORO' from dual union all
select 170,'FATAH' from dual union all
select 171,'EGAO' from dual union all
select 172,'AJ' from dual union all
select 173,'BPD' from dual union all
select 174,'UTC' from dual union all
select 175,'AJSP' from dual union all
select 176,'AJSP' from dual union all
select 177,'AJSP' from dual union all
select 178,'AJSP' from dual union all
select 179,'AJSP' from dual union all
select 18,'VORO' from dual union all
select 180,'AJSP' from dual union all
select 181,'AJSP' from dual union all
select 182,'AJSP' from dual union all
select 183,'ATLR' from dual union all
select 184,'ARC' from dual union all
select 185,'GCR' from dual union all
select 186,'COOP' from dual union all
select 187,'COOP' from dual union all
select 188,'ASTRV' from dual union all
select 189,'MBB' from dual union all
select 19,'SOYUZ' from dual union all
select 190,'AJSP' from dual union all
select 191,'AJSP' from dual union all
select 192,'RLABN' from dual union all
select 193,'SNECMA' from dual union all
select 194,'BAL' from dual union all
select 195,'BAL' from dual union all
select 196,'BAL' from dual union all
select 197,'HERC' from dual union all
select 198,'BLOR' from dual union all
select 199,'HERC' from dual union all
select 2,'THKE' from dual union all
select 20,'AJP' from dual union all
select 200,'HERC' from dual union all
select 201,'HERC' from dual union all
select 202,'HERC' from dual union all
select 203,'BLORTX' from dual union all
select 204,'BSELC' from dual union all
select 205,'IHI' from dual union all
select 205,'OSCD' from dual union all
select 206,'TRW' from dual union all
select 207,'ANSAL' from dual union all
select 208,'BAJUK' from dual union all
select 209,'BAJUK' from dual union all
select 21,'BAL' from dual union all
select 210,'BAJUK' from dual union all
select 211,'SDP' from dual union all
select 212,'CFTHB' from dual union all
select 213,'BELL' from dual union all
select 214,'BELL' from dual union all
select 215,'BELL' from dual union all
select 216,'BELL' from dual union all
select 217,'BELL' from dual union all
select 218,'IAI' from dual union all
select 219,'UNK' from dual union all
select 22,'AJP' from dual union all
select 220,'OKB1' from dual union all
select 221,'HERC' from dual;
insert into ENGINE_MANUFACTURER
select 221,'THKU' from dual union all
select 222,'HERC' from dual union all
select 223,'HERC' from dual union all
select 223,'THKU' from dual union all
select 224,'HERC' from dual union all
select 225,'VSSC' from dual union all
select 226,'ARMT' from dual union all
select 227,'ONERA' from dual union all
select 228,'SLREC' from dual union all
select 229,'CALT' from dual union all
select 23,'AJP' from dual union all
select 230,'CALT' from dual union all
select 231,'CALT' from dual union all
select 232,'THKE' from dual union all
select 233,'IIAE' from dual union all
select 234,'THKU' from dual union all
select 235,'THKU' from dual union all
select 236,'THKU' from dual union all
select 237,'THKU' from dual union all
select 238,'LRS' from dual union all
select 239,'LRS' from dual union all
select 24,'AJP' from dual union all
select 240,'LRS' from dual union all
select 241,'LRS' from dual union all
select 242,'IIAE' from dual union all
select 243,'THKE' from dual union all
select 244,'RPEW' from dual union all
select 245,'DRG' from dual union all
select 246,'DRG' from dual union all
select 247,'LRBA' from dual union all
select 248,'AJGC' from dual union all
select 249,'JPL' from dual union all
select 25,'KLIMOV' from dual union all
select 250,'RPEW' from dual union all
select 251,'RPEW' from dual union all
select 252,'RPEW' from dual union all
select 253,'RPEW' from dual union all
select 254,'RLABU' from dual union all
select 255,'PARD' from dual union all
select 256,'LARC' from dual union all
select 257,'THKE' from dual union all
select 258,'THKE' from dual union all
select 259,'LARC' from dual union all
select 259,'THKE' from dual union all
select 26,'KLIMOV' from dual union all
select 260,'LARC' from dual union all
select 261,'HERC' from dual union all
select 261,'THKU' from dual union all
select 262,'HERC' from dual union all
select 263,'DORBIT' from dual union all
select 264,'DAC' from dual union all
select 265,'CALT' from dual union all
select 266,'SANJ' from dual union all
select 267,'ARMT' from dual union all
select 268,'PLA' from dual union all
select 269,'ARMT' from dual union all
select 27,'NII125' from dual union all
select 27,'YUZH' from dual union all
select 270,'ARMT' from dual union all
select 271,'ARMT' from dual union all
select 272,'ARMT' from dual union all
select 273,'ARMT' from dual union all
select 274,'ARMT' from dual union all
select 275,'ARMT' from dual union all
select 276,'ARMT' from dual union all
select 277,'DARE' from dual union all
select 278,'IIAE' from dual union all
select 279,'ABL' from dual union all
select 28,'YUZH' from dual union all
select 280,'KHMM' from dual union all
select 281,'THKE' from dual union all
select 282,'THKE' from dual union all
select 283,'MITT' from dual union all
select 284,'KHMM' from dual union all
select 285,'KHMM' from dual union all
select 286,'KHMM' from dual union all
select 287,'KHMM' from dual union all
select 288,'WRE' from dual union all
select 289,'STST' from dual union all
select 29,'YUZH' from dual union all
select 290,'WRE' from dual union all
select 291,'SPXTX' from dual union all
select 292,'NPOL' from dual union all
select 293,'ASTL' from dual union all
select 294,'EUROPK' from dual union all
select 295,'EUROPK' from dual union all
select 296,'JAXA' from dual union all
select 297,'CFTHB' from dual union all
select 298,'BAL' from dual union all
select 299,'RDYNE' from dual union all
select 3,'AJ' from dual union all
select 30,'YUZH' from dual union all
select 300,'CIT' from dual union all
select 300,'USN' from dual union all
select 301,'ERNO' from dual union all
select 302,'HEXI' from dual union all
select 303,'HEXI' from dual union all
select 304,'AVIB' from dual union all
select 305,'UTC' from dual union all
select 306,'UTC' from dual;
insert into ENGINE_MANUFACTURER
select 307,'SLREC' from dual union all
select 308,'THKX' from dual union all
select 309,'MSFC' from dual union all
select 31,'YUZH' from dual union all
select 310,'RDYNE' from dual union all
select 311,'ATKABL' from dual union all
select 312,'ATKH' from dual union all
select 313,'ATKH' from dual union all
select 314,'ATKH' from dual union all
select 315,'OATKH' from dual union all
select 316,'BWYIC' from dual union all
select 317,'BWYIC' from dual union all
select 318,'BSELC' from dual union all
select 319,'BSELC' from dual union all
select 32,'ARSL' from dual union all
select 320,'BSELC' from dual union all
select 321,'BSELC' from dual union all
select 322,'RPEW' from dual union all
select 323,'RPEW' from dual union all
select 324,'RPEW' from dual union all
select 325,'RORD' from dual union all
select 326,'RORD' from dual union all
select 327,'RORD' from dual union all
select 328,'CITED' from dual union all
select 329,'CITED' from dual union all
select 33,'ARSL' from dual union all
select 330,'RDYNE' from dual union all
select 331,'NISST' from dual union all
select 332,'NISST' from dual union all
select 333,'NISST' from dual union all
select 334,'NISST' from dual union all
select 335,'NISST' from dual union all
select 336,'AMROC' from dual union all
select 337,'UNK' from dual union all
select 338,'HAL5' from dual union all
select 339,'IHI' from dual union all
select 34,'ARSL' from dual union all
select 340,'HERC' from dual union all
select 341,'HERC' from dual union all
select 342,'SEP' from dual union all
select 343,'SEP' from dual union all
select 344,'NOTS' from dual union all
select 345,'NOTS' from dual union all
select 346,'KPA' from dual union all
select 347,'NAMMO' from dual union all
select 348,'CIT' from dual union all
select 348,'NOTS' from dual union all
select 349,'LMMI' from dual union all
select 35,'ARSL' from dual union all
select 350,'GEMTS' from dual union all
select 351,'GEMTS' from dual union all
select 352,'GEMTS' from dual union all
select 353,'RPI' from dual union all
select 354,'RPI' from dual union all
select 355,'RPI' from dual union all
select 356,'KPA' from dual union all
select 357,'LPC' from dual union all
select 358,'LPC' from dual union all
select 359,'EAC' from dual union all
select 36,'ARSL' from dual union all
select 360,'ADD' from dual union all
select 361,'ADD' from dual union all
select 362,'IAI' from dual union all
select 363,'IAI' from dual union all
select 364,'IAI' from dual union all
select 365,'IAI' from dual union all
select 366,'ISRO' from dual union all
select 367,'INTA' from dual union all
select 368,'FIATA' from dual union all
select 369,'TRW' from dual union all
select 37,'TSKB7' from dual union all
select 370,'VSSC' from dual union all
select 371,'ERAU' from dual union all
select 372,'ERAU' from dual union all
select 373,'THKU' from dual union all
select 374,'RDYNE' from dual union all
select 375,'PWRD' from dual union all
select 376,'NASDA' from dual union all
select 377,'NASDA' from dual union all
select 378,'JPL' from dual union all
select 379,'LPC' from dual union all
select 38,'YUZH' from dual union all
select 380,'LPC' from dual union all
select 381,'LPC' from dual union all
select 382,'LPC' from dual union all
select 383,'ONERA' from dual union all
select 384,'SDP' from dual union all
select 385,'RPI' from dual union all
select 386,'RPI' from dual union all
select 387,'DRDO' from dual union all
select 388,'DRDO' from dual union all
select 389,'DRDO' from dual union all
select 39,'YUZH' from dual union all
select 390,'DRDO' from dual union all
select 391,'HANWHA' from dual union all
select 392,'ISAS' from dual union all
select 393,'ISAS' from dual union all
select 394,'ISAS' from dual union all
select 395,'ISAS' from dual union all
select 396,'ISAS' from dual;
insert into ENGINE_MANUFACTURER
select 397,'ISAS' from dual union all
select 398,'ISAS' from dual union all
select 399,'ISAS' from dual union all
select 4,'THKE' from dual union all
select 40,'YUZH' from dual union all
select 400,'ISAS' from dual union all
select 401,'ISAS' from dual union all
select 402,'ISAEV' from dual union all
select 403,'ISAEV' from dual union all
select 404,'NISST' from dual union all
select 405,'NISST' from dual union all
select 406,'NISST' from dual union all
select 407,'NISST' from dual union all
select 408,'NISST' from dual union all
select 409,'NISST' from dual union all
select 41,'TSKB7' from dual union all
select 410,'NISST' from dual union all
select 411,'NISST' from dual union all
select 412,'CMIK' from dual union all
select 413,'ISAEV' from dual union all
select 414,'KP' from dual union all
select 415,'KP' from dual union all
select 416,'ISAEV' from dual union all
select 417,'ISAEV' from dual union all
select 418,'ISAEV' from dual union all
select 419,'KARI' from dual union all
select 42,'ARSL' from dual union all
select 420,'KARI' from dual union all
select 421,'KARI' from dual union all
select 422,'KARI' from dual union all
select 423,'ARMT' from dual union all
select 424,'ISAEV' from dual union all
select 425,'ISAEV' from dual union all
select 426,'ISAEV' from dual union all
select 427,'ISAEV' from dual union all
select 428,'ISAEV' from dual union all
select 429,'ISAEV' from dual union all
select 43,'BAL' from dual union all
select 430,'ISAEV' from dual union all
select 431,'ISAEV' from dual union all
select 432,'ISAEV' from dual union all
select 433,'ISAEV' from dual union all
select 434,'KHMM' from dual union all
select 435,'ARMT' from dual union all
select 436,'ARMT' from dual union all
select 437,'ARMT' from dual union all
select 438,'ISAS' from dual union all
select 439,'ISAS' from dual union all
select 44,'MITT' from dual union all
select 440,'ISAS' from dual union all
select 441,'IR' from dual union all
select 442,'SPXTX' from dual union all
select 443,'RPI' from dual union all
select 444,'RPI' from dual union all
select 445,'NISST' from dual union all
select 446,'NISST' from dual union all
select 447,'NISST' from dual union all
select 448,'NISST' from dual union all
select 449,'LAPAN' from dual union all
select 45,'MITT' from dual union all
select 450,'WRE' from dual union all
select 451,'STA' from dual union all
select 452,'STA' from dual union all
select 453,'MHIME' from dual union all
select 454,'STA' from dual union all
select 455,'MHIME' from dual union all
select 456,'MHIME' from dual union all
select 457,'MHIME' from dual union all
select 458,'MHIME' from dual union all
select 459,'MHIME' from dual union all
select 46,'MITT' from dual union all
select 460,'AJSP' from dual union all
select 461,'AJSP' from dual union all
select 462,'AJSP' from dual union all
select 463,'UNK' from dual union all
select 464,'LPC' from dual union all
select 465,'LPC' from dual union all
select 466,'LPC' from dual union all
select 467,'ISRO' from dual union all
select 468,'ISRO' from dual union all
select 469,'RDYNE' from dual union all
select 47,'VOTK' from dual union all
select 470,'RMI' from dual union all
select 471,'AJGC' from dual union all
select 472,'AJGC' from dual union all
select 473,'AJGC' from dual union all
select 474,'AJGC' from dual union all
select 475,'AJGC' from dual union all
select 476,'AJGC' from dual union all
select 477,'AJGC' from dual union all
select 478,'AJGC' from dual union all
select 479,'AJGC' from dual union all
select 48,'VOTK' from dual union all
select 480,'AJGC' from dual union all
select 481,'AJGC' from dual union all
select 482,'IR' from dual union all
select 483,'NASDA' from dual union all
select 484,'STA' from dual union all
select 485,'VSSC' from dual union all
select 486,'LARC' from dual;
insert into ENGINE_MANUFACTURER
select 487,'GCR' from dual union all
select 488,'BAJUK' from dual union all
select 489,'RPEW' from dual union all
select 49,'VOTK' from dual union all
select 490,'RPEW' from dual union all
select 491,'RORD' from dual union all
select 492,'GCR' from dual union all
select 492,'JPL' from dual union all
select 493,'GCR' from dual union all
select 494,'WRE' from dual union all
select 495,'WRE' from dual union all
select 496,'WRE' from dual union all
select 497,'WRE' from dual union all
select 498,'SEP' from dual union all
select 499,'NISST' from dual union all
select 5,'ISAEV' from dual union all
select 50,'VORO' from dual union all
select 500,'UNK' from dual union all
select 501,'UNK' from dual union all
select 502,'NISST' from dual union all
select 503,'NISST' from dual union all
select 504,'NISST' from dual union all
select 505,'NISST' from dual union all
select 506,'NISST' from dual union all
select 507,'NISST' from dual union all
select 508,'NISST' from dual union all
select 509,'NISST' from dual union all
select 51,'KHMM' from dual union all
select 510,'NISST' from dual union all
select 511,'NISST' from dual union all
select 512,'NISST' from dual union all
select 513,'NISST' from dual union all
select 514,'NISST' from dual union all
select 515,'NISST' from dual union all
select 516,'NISST' from dual union all
select 517,'ABL' from dual union all
select 518,'HERCR' from dual union all
select 519,'AJSP' from dual union all
select 52,'BAL' from dual union all
select 520,'AJSP' from dual union all
select 521,'PHILR' from dual union all
select 522,'AJ' from dual union all
select 523,'RDYNM' from dual union all
select 524,'UNK' from dual union all
select 525,'THKU' from dual union all
select 526,'AJSP' from dual union all
select 527,'AJSP' from dual union all
select 528,'HERC' from dual union all
select 529,'ABL' from dual union all
select 53,'ATLR' from dual union all
select 530,'THKXM' from dual union all
select 531,'RDYNE' from dual union all
select 532,'RDYNE' from dual union all
select 533,'RDYNE' from dual union all
select 534,'RDYNE' from dual union all
select 535,'RDYNE' from dual union all
select 536,'RDYNE' from dual union all
select 537,'RDYNE' from dual union all
select 538,'RDYNE' from dual union all
select 539,'RDYNE' from dual union all
select 54,'AJ' from dual union all
select 540,'RDYNE' from dual union all
select 541,'RDYNE' from dual union all
select 542,'ATLR' from dual union all
select 543,'ATLR' from dual union all
select 544,'ATLR' from dual union all
select 545,'ATLR' from dual union all
select 546,'ATLR' from dual union all
select 547,'ATLR' from dual union all
select 548,'ATLR' from dual union all
select 549,'ATLR' from dual union all
select 55,'AJ' from dual union all
select 550,'ATLR' from dual union all
select 551,'ATLR' from dual union all
select 552,'AJ' from dual union all
select 553,'RDYNE' from dual union all
select 554,'RDYNE' from dual union all
select 555,'RDYNE' from dual union all
select 556,'RDYNE' from dual union all
select 557,'VSSC' from dual union all
select 558,'KBPT' from dual union all
select 559,'LPC' from dual union all
select 56,'ABL' from dual union all
select 560,'VOUA' from dual union all
select 561,'GMS' from dual union all
select 562,'RRC' from dual union all
select 563,'RRC' from dual union all
select 564,'RRC' from dual union all
select 565,'RRC' from dual union all
select 566,'RRC' from dual union all
select 567,'RRC' from dual union all
select 568,'TAIFUN' from dual union all
select 569,'RRC' from dual union all
select 57,'AJ' from dual union all
select 570,'TAIFUN' from dual union all
select 571,'TAIFUN' from dual union all
select 572,'TAIFUN' from dual union all
select 573,'RRC' from dual union all
select 574,'G2P' from dual union all
select 574,'SNECPS' from dual;
insert into ENGINE_MANUFACTURER
select 575,'G2P' from dual union all
select 575,'SNECPS' from dual union all
select 576,'G2P' from dual union all
select 576,'SNECPS' from dual union all
select 577,'SNECMA' from dual union all
select 578,'ISAS' from dual union all
select 579,'RMI' from dual union all
select 58,'AJGC' from dual union all
select 580,'SEPB' from dual union all
select 581,'SEPB' from dual union all
select 582,'SEPB' from dual union all
select 583,'WRE' from dual union all
select 584,'ONERA' from dual union all
select 585,'ATEF' from dual union all
select 586,'SPXTX' from dual union all
select 587,'SPXTX' from dual union all
select 588,'SPXTX' from dual union all
select 589,'SPXTX' from dual union all
select 59,'NMC' from dual union all
select 590,'SPXTX' from dual union all
select 591,'SPXTX' from dual union all
select 592,'SPXTX' from dual union all
select 593,'SPXTX' from dual union all
select 594,'SPXTX' from dual union all
select 595,'PIHM' from dual union all
select 596,'PIHM' from dual union all
select 597,'AJCA' from dual union all
select 598,'UNK' from dual union all
select 599,'UNK' from dual union all
select 6,'KHMM' from dual union all
select 60,'AJ' from dual union all
select 600,'UNK' from dual union all
select 601,'UNK' from dual union all
select 602,'THKE' from dual union all
select 603,'NOTS' from dual union all
select 603,'THKE' from dual union all
select 604,'THKE' from dual union all
select 605,'UNK' from dual union all
select 606,'ATLR' from dual union all
select 607,'ATLR' from dual union all
select 607,'HERC' from dual union all
select 607,'THK' from dual union all
select 608,'RDYNE' from dual union all
select 609,'UNK' from dual union all
select 61,'BAL' from dual union all
select 610,'THKE' from dual union all
select 611,'AJSP' from dual union all
select 612,'ATLR' from dual union all
select 613,'UNK' from dual union all
select 614,'AJCA' from dual union all
select 615,'UTC' from dual union all
select 616,'UNK' from dual union all
select 617,'OSCC' from dual union all
select 618,'IST' from dual union all
select 619,'WRE' from dual union all
select 62,'BAL' from dual union all
select 620,'ABL' from dual union all
select 621,'ABL' from dual union all
select 622,'ABL' from dual union all
select 623,'SNPE' from dual union all
select 624,'SNPE' from dual union all
select 625,'SNPE' from dual union all
select 626,'SNPE' from dual union all
select 627,'SNPE' from dual union all
select 628,'NAL' from dual union all
select 629,'STA' from dual union all
select 63,'KAZOKB' from dual union all
select 630,'STA' from dual union all
select 631,'NISST' from dual union all
select 632,'NAA' from dual union all
select 633,'UNK' from dual union all
select 634,'NKE' from dual union all
select 635,'NKE' from dual union all
select 636,'NKE' from dual union all
select 637,'NKE' from dual union all
select 638,'NKE' from dual union all
select 639,'NKE' from dual union all
select 64,'KAZOKB' from dual union all
select 640,'NKE' from dual union all
select 641,'NKE' from dual union all
select 642,'NOTS' from dual union all
select 643,'NOTS' from dual union all
select 644,'NOTS' from dual union all
select 645,'NOTS' from dual union all
select 646,'NOTS' from dual union all
select 647,'NOTS' from dual union all
select 648,'NOTS' from dual union all
select 649,'IR' from dual union all
select 65,'KAZOKB' from dual union all
select 650,'RMI' from dual union all
select 651,'NRL' from dual union all
select 652,'NSPO' from dual union all
select 653,'NSPO' from dual union all
select 654,'SEPR' from dual union all
select 655,'KPA' from dual union all
select 656,'ALMAN' from dual union all
select 657,'ALMAN' from dual union all
select 658,'AJGC' from dual union all
select 659,'OTR' from dual union all
select 66,'DRDO' from dual;
insert into ENGINE_MANUFACTURER
select 660,'KBM' from dual union all
select 661,'UTC' from dual union all
select 662,'ATKE' from dual union all
select 663,'UTC' from dual union all
select 664,'UTC' from dual union all
select 665,'ATEF' from dual union all
select 666,'ATKH' from dual union all
select 667,'ATKH' from dual union all
select 668,'ATKH' from dual union all
select 669,'ATKH' from dual union all
select 67,'ABL' from dual union all
select 670,'ATKH' from dual union all
select 671,'ATKH' from dual union all
select 672,'ATKH' from dual union all
select 673,'ATKH' from dual union all
select 674,'ATKP' from dual union all
select 675,'IIAE' from dual union all
select 676,'IIAE' from dual union all
select 677,'ISAEV' from dual union all
select 678,'SEP' from dual union all
select 679,'SEP' from dual union all
select 68,'GCR' from dual union all
select 680,'AVIO' from dual union all
select 681,'SEP' from dual union all
select 682,'NAA' from dual union all
select 683,'NAA' from dual union all
select 684,'AVIO' from dual union all
select 685,'RDYNE' from dual union all
select 686,'RDYNE' from dual union all
select 687,'FIATA' from dual union all
select 688,'SEPVR' from dual union all
select 689,'ISKRM' from dual union all
select 69,'SDCO' from dual union all
select 690,'ISKRM' from dual union all
select 691,'VSSC' from dual union all
select 692,'VSSC' from dual union all
select 693,'VSSC' from dual union all
select 694,'AERO' from dual union all
select 695,'SOYUZ' from dual union all
select 696,'PWRD' from dual union all
select 697,'ADD' from dual union all
select 698,'MSFC' from dual union all
select 699,'HERC' from dual union all
select 7,'KHMM' from dual union all
select 70,'GCR' from dual union all
select 700,'HERC' from dual union all
select 701,'THKX' from dual union all
select 702,'ISAEV' from dual union all
select 703,'DRDO' from dual union all
select 704,'CITEF' from dual union all
select 705,'CITEF' from dual union all
select 706,'PERM' from dual union all
select 707,'ARSL' from dual union all
select 708,'YUZH' from dual union all
select 709,'MAK' from dual union all
select 71,'AJ' from dual union all
select 710,'PRIMX' from dual union all
select 711,'MARQ' from dual union all
select 712,'UNK' from dual union all
select 713,'UNK' from dual union all
select 714,'UNK' from dual union all
select 715,'UNK' from dual union all
select 716,'OKB1' from dual union all
select 717,'OKB1' from dual union all
select 718,'MITT' from dual union all
select 719,'MITT' from dual union all
select 72,'AJ' from dual union all
select 720,'MITT' from dual union all
select 721,'KBKHA' from dual union all
select 722,'KBKHA' from dual union all
select 723,'KBKHA' from dual union all
select 724,'KBKHA' from dual union all
select 725,'KBKHA' from dual union all
select 726,'KBKHA' from dual union all
select 727,'KBKHA' from dual union all
select 728,'KBKHA' from dual union all
select 729,'KBKHA' from dual union all
select 73,'GRC' from dual union all
select 730,'KBKHA' from dual union all
select 731,'KBKHA' from dual union all
select 732,'KBKHA' from dual union all
select 733,'KBKHA' from dual union all
select 734,'KBKHA' from dual union all
select 735,'KBKHA' from dual union all
select 735,'PERM' from dual union all
select 736,'KBKHA' from dual union all
select 736,'PERM' from dual union all
select 737,'KBKHA' from dual union all
select 738,'KBKHA' from dual union all
select 739,'KBKHA' from dual union all
select 74,'PERM' from dual union all
select 740,'KBKHA' from dual union all
select 741,'KBKHA' from dual union all
select 742,'KBKHA' from dual union all
select 742,'KOKT' from dual union all
select 743,'KBKHA' from dual union all
select 743,'KOKT' from dual union all
select 744,'KBKHA' from dual union all
select 744,'YKVZ' from dual union all
select 745,'KBKHA' from dual;
insert into ENGINE_MANUFACTURER
select 746,'KBKHA' from dual union all
select 746,'KRAS' from dual union all
select 747,'KBKHA' from dual union all
select 748,'KBKHA' from dual union all
select 749,'KBKHA' from dual union all
select 75,'YUZH' from dual union all
select 750,'EMASH' from dual union all
select 751,'EMASH' from dual union all
select 751,'YUZHM' from dual union all
select 752,'EMASH' from dual union all
select 752,'YUZHM' from dual union all
select 753,'EMASH' from dual union all
select 753,'YUZHM' from dual union all
select 754,'EMASH' from dual union all
select 754,'MOTOS' from dual union all
select 755,'EMASH' from dual union all
select 756,'EMASH' from dual union all
select 756,'MOTOS' from dual union all
select 757,'EMASH' from dual union all
select 758,'EMASH' from dual union all
select 759,'EMASH' from dual union all
select 76,'AJ' from dual union all
select 760,'EMASH' from dual union all
select 761,'EMASH' from dual union all
select 762,'EMASH' from dual union all
select 763,'EMASH' from dual union all
select 764,'EMASH' from dual union all
select 765,'EMASH' from dual union all
select 765,'KRAS' from dual union all
select 766,'EMASH' from dual union all
select 766,'YUZHM' from dual union all
select 767,'EMASH' from dual union all
select 768,'EMASH' from dual union all
select 769,'EMASH' from dual union all
select 77,'FAKEL' from dual union all
select 770,'EMASH' from dual union all
select 771,'EMASH' from dual union all
select 772,'EMASH' from dual union all
select 773,'EMASH' from dual union all
select 773,'PERM' from dual union all
select 774,'EMASH' from dual union all
select 775,'EMASH' from dual union all
select 776,'EMASH' from dual union all
select 776,'KRAS' from dual union all
select 777,'EMASH' from dual union all
select 778,'EMASH' from dual union all
select 779,'EMASH' from dual union all
select 78,'ISAEV' from dual union all
select 780,'EMASH' from dual union all
select 781,'EMASH' from dual union all
select 782,'EMASH' from dual union all
select 782,'YUZHM' from dual union all
select 783,'EMASH' from dual union all
select 784,'EMASH' from dual union all
select 784,'PERM' from dual union all
select 785,'EMASH' from dual union all
select 785,'PERM' from dual union all
select 786,'EMASH' from dual union all
select 786,'YUZHM' from dual union all
select 787,'EMASH' from dual union all
select 788,'EMASH' from dual union all
select 788,'YUZHM' from dual union all
select 789,'EMASH' from dual union all
select 79,'ABL' from dual union all
select 790,'EMASH' from dual union all
select 791,'EMASH' from dual union all
select 791,'PERM' from dual union all
select 792,'YUZH' from dual union all
select 793,'YUZH' from dual union all
select 794,'YUZH' from dual union all
select 795,'YUZH' from dual union all
select 796,'YUZH' from dual union all
select 797,'YUZH' from dual union all
select 798,'YUZH' from dual union all
select 799,'YUZH' from dual union all
select 8,'ISAEV' from dual union all
select 80,'ABL' from dual union all
select 800,'YUZH' from dual union all
select 801,'YUZH' from dual union all
select 802,'YUZH' from dual union all
select 802,'YUZHM' from dual union all
select 803,'YUZH' from dual union all
select 803,'YUZHM' from dual union all
select 804,'YUZH' from dual union all
select 805,'YUZH' from dual union all
select 806,'YUZH' from dual union all
select 807,'YUZH' from dual union all
select 808,'YUZH' from dual union all
select 809,'YUZH' from dual union all
select 81,'AJ' from dual union all
select 810,'YUZH' from dual union all
select 811,'NIIM' from dual union all
select 812,'NIIM' from dual union all
select 813,'NIIM' from dual union all
select 814,'NIIM' from dual union all
select 815,'HASTD' from dual union all
select 816,'VSSC' from dual union all
select 817,'VSSC' from dual union all
select 818,'VSSC' from dual union all
select 819,'VSSC' from dual;
insert into ENGINE_MANUFACTURER
select 82,'GCR' from dual union all
select 820,'VSSC' from dual union all
select 821,'VSSC' from dual union all
select 822,'VSSC' from dual union all
select 823,'VSSC' from dual union all
select 824,'VSSC' from dual union all
select 825,'VSSC' from dual union all
select 826,'PWFL' from dual union all
select 827,'PWFL' from dual union all
select 828,'PWFL' from dual union all
select 829,'PWFL' from dual union all
select 83,'IHI' from dual union all
select 830,'PWFL' from dual union all
select 831,'PWFL' from dual union all
select 832,'PWFL' from dual union all
select 833,'PWFL' from dual union all
select 834,'PWFL' from dual union all
select 835,'PWFL' from dual union all
select 836,'PWFL' from dual union all
select 837,'PWFL' from dual union all
select 838,'SCAL' from dual union all
select 838,'SPDEV' from dual union all
select 839,'COOP' from dual union all
select 839,'GCR' from dual union all
select 84,'KAZSOY' from dual union all
select 840,'COOP' from dual union all
select 841,'SCAL' from dual union all
select 841,'SPDEV' from dual union all
select 842,'COOP' from dual union all
select 842,'JPL' from dual union all
select 843,'KBKHA' from dual union all
select 844,'KBKHA' from dual union all
select 845,'RDYNE' from dual union all
select 846,'RDYNE' from dual union all
select 847,'RDYNE' from dual union all
select 848,'RDYNE' from dual union all
select 849,'RDYNE' from dual union all
select 85,'KLIMOV' from dual union all
select 850,'RDYNE' from dual union all
select 851,'RDYNE' from dual union all
select 852,'RDYNE' from dual union all
select 853,'RDYNE' from dual union all
select 854,'RDYNE' from dual union all
select 855,'RDYNE' from dual union all
select 856,'HOUW' from dual union all
select 857,'HOUW' from dual union all
select 858,'HOUW' from dual union all
select 859,'OKB1' from dual union all
select 86,'KAZSOY' from dual union all
select 86,'PERM' from dual union all
select 860,'JPL' from dual union all
select 861,'THKX' from dual union all
select 862,'LAPAN' from dual union all
select 863,'LAPAN' from dual union all
select 864,'LAPAN' from dual union all
select 865,'RROY' from dual union all
select 866,'RPEW' from dual union all
select 867,'RPEW' from dual union all
select 868,'RPEW' from dual union all
select 869,'RPEW' from dual union all
select 87,'KLIMOV' from dual union all
select 870,'RPEW' from dual union all
select 871,'RPEW' from dual union all
select 872,'RPEW' from dual union all
select 873,'THKE' from dual union all
select 874,'ABMA' from dual union all
select 875,'SEP' from dual union all
select 876,'RPEW' from dual union all
select 877,'RPEW' from dual union all
select 878,'RLABU' from dual union all
select 879,'RLABU' from dual union all
select 88,'THKX' from dual union all
select 880,'ISAS' from dual union all
select 881,'IAE' from dual union all
select 882,'ISAS' from dual union all
select 883,'IAE' from dual union all
select 884,'IAE' from dual union all
select 885,'ISAS' from dual union all
select 886,'IAE' from dual union all
select 887,'ISAS' from dual union all
select 888,'RDYNE' from dual union all
select 889,'IAE' from dual union all
select 89,'KAZSOY' from dual union all
select 890,'ASTL' from dual union all
select 891,'ASTL' from dual union all
select 892,'IAE' from dual union all
select 893,'IAE' from dual union all
select 894,'CSXT' from dual union all
select 895,'NISST' from dual union all
select 896,'STA' from dual union all
select 897,'STA' from dual union all
select 898,'STA' from dual union all
select 899,'ISAEV' from dual union all
select 9,'KHMM' from dual union all
select 90,'KAZSOY' from dual union all
select 900,'OKB1' from dual union all
select 901,'VSSC' from dual union all
select 902,'VSSC' from dual union all
select 903,'ISAEV' from dual union all
select 904,'ISAEV' from dual;
insert into ENGINE_MANUFACTURER
select 905,'ISAEV' from dual union all
select 906,'ISAEV' from dual union all
select 907,'ISAEV' from dual union all
select 908,'ISAEV' from dual union all
select 909,'ISAEV' from dual union all
select 91,'KAZSOY' from dual union all
select 91,'PERM' from dual union all
select 910,'ISAEV' from dual union all
select 911,'ISAEV' from dual union all
select 912,'ISAEV' from dual union all
select 913,'VSSC' from dual union all
select 914,'ISAS' from dual union all
select 915,'ISAEV' from dual union all
select 916,'ISAEV' from dual union all
select 917,'ISAEV' from dual union all
select 918,'ISAEV' from dual union all
select 919,'ISAEV' from dual union all
select 92,'AJP' from dual union all
select 920,'ISAEV' from dual union all
select 921,'ISAEV' from dual union all
select 922,'ISAEV' from dual union all
select 923,'ISAEV' from dual union all
select 924,'ISAEV' from dual union all
select 925,'ISAEV' from dual union all
select 926,'KHMM' from dual union all
select 927,'KHMM' from dual union all
select 928,'KHMM' from dual union all
select 929,'ISAEV' from dual union all
select 93,'EMASH' from dual union all
select 930,'VSSC' from dual union all
select 931,'STA' from dual union all
select 932,'VENT' from dual union all
select 933,'EXOSA' from dual union all
select 934,'NISST' from dual union all
select 935,'NISST' from dual union all
select 936,'RDYNE' from dual union all
select 937,'SEP' from dual union all
select 938,'SEP' from dual union all
select 939,'SEP' from dual union all
select 94,'EMASH' from dual union all
select 940,'SEP' from dual union all
select 941,'SEP' from dual union all
select 942,'SEP' from dual union all
select 943,'SEP' from dual union all
select 944,'SEPR' from dual union all
select 945,'SEPR' from dual union all
select 946,'ONERA' from dual union all
select 947,'SEPR' from dual union all
select 948,'ONERA' from dual union all
select 948,'SEPR' from dual union all
select 949,'SEPR' from dual union all
select 95,'EMASH' from dual union all
select 950,'SEPR' from dual union all
select 951,'SEPR' from dual union all
select 952,'SEP' from dual union all
select 953,'SEPR' from dual union all
select 954,'SEP' from dual union all
select 955,'SEPR' from dual union all
select 956,'THKU' from dual union all
select 957,'AJSP' from dual union all
select 958,'HERC' from dual union all
select 959,'RDYNE' from dual union all
select 96,'EMASH' from dual union all
select 960,'VSSC' from dual union all
select 961,'VSSC' from dual union all
select 962,'VSSC' from dual union all
select 963,'VSSC' from dual union all
select 964,'MHI' from dual union all
select 965,'MHI' from dual union all
select 966,'ISAS' from dual union all
select 967,'ONERA' from dual union all
select 968,'SANJ' from dual union all
select 969,'ABL' from dual union all
select 969,'AJSP' from dual union all
select 97,'EMASH' from dual union all
select 970,'AJSP' from dual union all
select 971,'HERC' from dual union all
select 972,'AJSP' from dual union all
select 973,'ATKP' from dual union all
select 974,'SDCO' from dual union all
select 975,'AJSP' from dual union all
select 976,'ATKP' from dual union all
select 977,'THKU' from dual union all
select 978,'ATKP' from dual union all
select 979,'AVIB' from dual union all
select 98,'EMASH' from dual union all
select 980,'IIAE' from dual union all
select 981,'ISAS' from dual union all
select 982,'ISAS' from dual union all
select 983,'RDYNE' from dual union all
select 984,'ISAS' from dual union all
select 985,'ISAS' from dual union all
select 986,'ARMAD' from dual union all
select 987,'ARMAD' from dual union all
select 988,'AJSP' from dual union all
select 989,'IR' from dual union all
select 99,'EMASH' from dual union all
select 990,'IMIS' from dual union all
select 991,'IMIS' from dual union all
select 992,'UNK' from dual;
insert into ENGINE_MANUFACTURER
select 993,'IR' from dual union all
select 994,'IR' from dual union all
select 995,'DRG' from dual union all
select 996,'DRG' from dual union all
select 997,'PKNDC' from dual union all
select 998,'DRDO' from dual union all
select 999,'DRDO' from dual;


--------------------------------------------------------------------------------
-- Compress tables and rebuild indexes.
--------------------------------------------------------------------------------

exec dbms_output.put_line('Compressing tables and rebuilding indexes...');

alter table ORGANIZATION move compress;
alter index ORGANIZATION_PK rebuild;
alter table PLATFORM move compress;
alter index PLATFORM_IDX1 rebuild;
alter index PLATFORM_IDX2 rebuild;
alter index PLATFORM_PK rebuild;
alter table ORGANIZATION_ORG_TYPE move compress;
alter index ORGANIZATION_ORG_TYPE_PK rebuild;
alter table SITE move compress;
alter index SITE_IDX rebuild;
alter index SITE_PK rebuild;
alter table SITE_ORG move compress;
alter index SITE_ORG_PK rebuild;
alter table LAUNCH_VEHICLE_FAMILY move compress;
alter index LAUNCH_VEHICLE_FAMILY_PK rebuild;
alter table LAUNCH_VEHICLE move compress;
alter index LAUNCH_VEHICLE_IDX1 rebuild;
alter index LAUNCH_VEHICLE_PK rebuild;
alter index LAUNCH_VEHICLE_UQ rebuild;
alter table LAUNCH_VEHICLE_MANUFACTURER move compress;
alter index LAUNCH_VEHICLE_MANUFACT_IDX1 rebuild;
alter index LAUNCH_VEHICLE_MAN_ORG_PK rebuild;
alter table LAUNCH move compress;
alter index LAUNCH_IDX1 rebuild;
alter index LAUNCH_IDX2 rebuild;
alter index LAUNCH_IDX3 rebuild;
alter index LAUNCH_IDX4 rebuild;
alter index LAUNCH_PK rebuild;
alter table LAUNCH_PAYLOAD_ORG move compress;
alter index LAUNCH_PAYLOAD_ORG_PK rebuild;
alter table LAUNCH_AGENCY move compress;
alter index LAUNCH_AGENCY_PK rebuild;
alter table SATELLITE move compress;
alter index SATELLITE_IDX1 rebuild;
alter index SATELLITE_PK rebuild;
alter table SATELLITE_ORG move compress;
alter index SATELLITE_ORG_PK rebuild;
alter table ENGINE move compress;
alter index ENGINE_PK rebuild;
alter table STAGE move compress;
alter index STAGE_IDX1 rebuild;
alter index STAGE_PK rebuild;
alter table LAUNCH_VEHICLE_STAGE move compress;
alter index LAUNCH_VEHICLE_STAGE_IDX1 rebuild;
alter index LAUNCH_VEHICLE_STAGE_IDX2 rebuild;
alter index LAUNCH_VEHICLE_STAGE_PK rebuild;
alter table STAGE_MANUFACTURER move compress;
alter index STAGE_MANUFACTURER_IDX1 rebuild;
alter index STAGE_MANUFACTURER_PK rebuild;
alter table PROPELLANT move compress;
alter index PROPELLANT_PK rebuild;
alter index PROPELLANT_UQ rebuild;
alter table ENGINE_PROPELLANT move compress;
alter index ENGINE_PROPELLANT_IDX1 rebuild;
alter index ENGINE_PROPELLANT_PK rebuild;
alter table ENGINE_MANUFACTURER move compress;
alter index ENGINE_MANUFACTURER_IDX1 rebuild;
alter index ENGINE_MANUFACTURER_PK rebuild;



-- Print intro message.
exec dbms_output.put_line('------------------------------------------------------------------------');
exec dbms_output.put_line('-- Done.  The space database was successfully installed.');
exec dbms_output.put_line('------------------------------------------------------------------------');

--------------------------------------------------------------------------------
-- DONE
--------------------------------------------------------------------------------
